/**
 * Copyright (C) 2012 51Degrees.mobi Limited
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Affero General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <stdarg.h>
#include "51Degrees.mobi.h"
#include "snprintf/snprintf.h"

#ifdef _MSC_VER
#define CDECL __cdecl
#else
#define CDECL
#endif

// A pointer to the memory containing the property strings.
char* _allProperties;

// A pointer to the start of each of the properties.
char** _properties;

// The number of properties the system is configured to return.
int _propertiesCount = 0;


const char S0[] = "False";
const char S1[] = "AnimationTiming";
const char S2[] = "True";
const char S3[] = "BlobBuilder";
const char S4[] = "Canvas";
const char S5[] = "CssBackground";
const char S6[] = "CssBorderImage";
const char S7[] = "CssCanvas";
const char S8[] = "CssColor";
const char S9[] = "CssColumn";
const char S10[] = "CssFlexbox";
const char S11[] = "CssFont";
const char S12[] = "CssImages";
const char S13[] = "CssMediaQueries";
const char S14[] = "CssMinMax";
const char S15[] = "CssOverflow";
const char S16[] = "CssPosition";
const char S17[] = "CssText";
const char S18[] = "CssTransforms";
const char S19[] = "CssTransitions";
const char S20[] = "CssUI";
const char S21[] = "DataSet";
const char S22[] = "DataUrl";
const char S23[] = "DeviceOrientation";
const char S24[] = "FileReader";
const char S25[] = "FileSaver";
const char S26[] = "FileWriter";
const char S27[] = "FormData";
const char S28[] = "Fullscreen";
const char S29[] = "GeoLocation";
const char S30[] = "History";
const char S31[] = "Html5";
const char S32[] = "Html-Media-Capture";
const char S33[] = "Iframe";
const char S34[] = "IndexedDB";
const char S35[] = "Json";
const char S36[] = "Webkit";
const char S37[] = "LayoutEngine";
const char S38[] = "Masking";
const char S39[] = "PostMessage";
const char S40[] = "Progress";
const char S41[] = "Prompts";
const char S42[] = "Selector";
const char S43[] = "Svg";
const char S44[] = "TouchEvents";
const char S45[] = "Track";
const char S46[] = "Video";
const char S47[] = "Viewport";
const char S48[] = "WebWorkers";
const char S49[] = "Xhr2";
const char S50[] = "IsMobile";
const char S51[] = "800";
const char S52[] = "ScreenPixelsHeight";
const char S53[] = "400";
const char S54[] = "ScreenPixelsWidth";
const char S55[] = "Unknown";
const char S56[] = "480";
const char S57[] = "320";
const char S58[] = "240";
const char S59[] = "Mango";
const char S60[] = "Trident";
const char S61[] = "160";
const char S62[] = "128";
const char S63[] = "640";
const char S64[] = "360";
const char S65[] = "Netfront";
const char S66[] = "Presto";
const char S67[] = "220";
const char S68[] = "176";
const char S69[] = "854";
const char S70[] = "U3";
const char S71[] = "120";
const char S72[] = "MotoWebkit";
const char S73[] = "432";
const char S74[] = "208";
const char S75[] = "260";
const char S76[] = "Gecko";
const char S77[] = "416";
const char S78[] = "352";
const char S79[] = "440";
const char S80[] = "200";
const char S81[] = "275";
const char S82[] = "960";
const char S83[] = "272";
const char S84[] = "600";
const char S85[] = "1024";
const char S86[] = "48";
const char S87[] = "84";
const char S88[] = "Lumi";
const char S89[] = "540";
const char S90[] = "345";
const char S91[] = "65";
const char S92[] = "96";
const char S93[] = "132";
const char S94[] = "1280";
const char S95[] = "300";
const char S96[] = "80";
const char S97[] = "101";
const char S98[] = "144";
const char S99[] = "192";
const char S100[] = "64";
const char S101[] = "U2";
const char S102[] = "768";
const char S103[] = "KHTML";
const char S104[] = "720";
const char S105[] = "427";
const char S106[] = "256";
const char S107[] = "266";
const char S108[] = "864";
const char S109[] = "276";
const char S110[] = "172";
const char S111[] = "420";
const char S112[] = "130";
const char S113[] = "67";
const char S114[] = "98";
const char S115[] = "90";
const char S116[] = "323";
const char S117[] = "232";
const char S118[] = "228";
const char S119[] = "280";
const char S120[] = "92";
const char S121[] = "177";
const char S122[] = "1200";
const char S123[] = "824";
const char S124[] = "37";
const char S125[] = "230";
const char S126[] = "544";
const char S127[] = "1920";
const char S128[] = "1080";
const char S129[] = "1366";
const char S130[] = "700";
const char S131[] = "60";
const char S132[] = "340";
const char S133[] = "758";
const char S134[] = "1600";
const char S135[] = "900";
const char S136[] = "533";
const char S137[] = "279";
const char S138[] = "728";
const char S139[] = "1440";
const char S140[] = "2560";
const char S141[] = "Blink";
const char S142[] = "1028";
const char S143[] = "40";
const char S144[] = "2048";
const char S145[] = "1536";
const char S146[] = "1240";
const char S147[] = "601";
const char S148[] = "853";
const char S149[] = "690";
const char S150[] = "752";
const char S151[] = "1136";
const char S152[] = "2160";
const char S153[] = "3840";
const char S154[] = "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; Media Center PC 4.0; SLCC1; .NET CLR 3.0.04320)";
const char S155[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S156[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0)";
const char S157[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1)";
const char S158[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2)";
const char S159[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S160[] = "Windows-RSS-Platform/2.0 (MSIE 8.0; Windows NT 5.1)";
const char S161[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S162[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S163[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S164[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S165[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S166[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) ; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) (none))";
const char S167[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S168[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S169[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S170[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1)";
const char S171[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; Media Center PC 2.8)";
const char S172[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S173[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 8.0; Win32; GMX); (gmx/1.1.0.21); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S174[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)";
const char S175[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)";
const char S176[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)";
const char S177[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)";
const char S178[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; 3M/MSIE 8.0)";
const char S179[] = "Windows-RSS-Platform/2.0 (MSIE 8.0; Windows NT 5.2)";
const char S180[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Tablet PC 1.7; Media Center PC 4.0; .NET4.0C; .NET4.0E; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322; InfoPath.3)";
const char S181[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; Media Center PC 2.8)";
const char S182[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)";
const char S183[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S184[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) )";
const char S185[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 8.0; Win32; GMX); (gmx/1.1.0.21); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S186[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0(Compatible Mozilla/4.0(Compatible-EmbeddedWB 14.59 http://bsalsa.com/ EmbeddedWB- 14.59  from: http://bsalsa.com/ ; .NET CLR 2.0.50727)";
const char S187[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; 3M/MSIE 8.0;; 3M/MSIE 8.0)";
const char S188[] = "Mozilla/4.0 (compatible; MSIE 7.0; MSIE 8.0; AD1; Windows NT 5.1; Trident/4.0; AD1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; AD1)";
const char S189[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S190[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)";
const char S191[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Win64; x64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S192[] = "Mozilla/6.0 (compatible; MSIE 8.0; Windows NT 5.1)";
const char S193[] = "Empty (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S194[] = "Mozilla/4.0 (compatible, MSIE 8.0; Windows NT 5.1; Trident/4.0;)";
const char S195[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; Media Center PC 3.0; Media Center PC 2.8)";
const char S196[] = "Mozilla/4.0 (compatible; MSIE 8.0.6001.18702; Windows NT 5.1)";
const char S197[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; SV1)";
const char S198[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0a; Windows NT; Getxoko Udala); InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S199[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; Verizon Windows Live; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S200[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; SRS MSIE 7.0; .NET4.0C; .NET4.0E; SRS MSIE 8.0)";
const char S201[] = "Mozilla/4.0 (compatible ; MSIE 8.0.6001.18702 ; Microsoft Windows XP Professional Service Pack 3 ; Placeware RPC 1.0)";
const char S202[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 8.0; Win32; GMX); Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S203[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)";
const char S204[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; Xerox_Win32PWS; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.2; MS-RTC LM 8; Xerox_Win32PWS)";
const char S205[] = "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.1; SLCC1; .NET CLR 1.1.4322)";
const char S206[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; Verizon Windows Live; Verizon Windows Live)";
const char S207[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.2; 3M/MSIE 8.0)";
const char S208[] = "mozilla/4.0 (compatible; msie 8.0; windows nt 5.1; trident/4.0; .net clr 1.0.3705; .net clr 1.1.4322; media center pc 4.0; .net clr 2.0.50727; .net clr 3.0.04506.30; .net clr 3.0.04506.648; .net clr 3.0.4506.2152; .net clr 3.5.30729; bri/2; officeliveconnector.1.5; officelivepatch.1.3)";
const char S209[] = "mozilla/4.0 (compatible; msie 8.0; windows nt 5.1; trident/4.0; gtb6.5; .net clr 2.0.50727; .net clr 3.0.04506.648; .net clr 3.5.21022; .net clr 3.0.4506.2152; .net clr 3.5.30729; .net clr 1.1.4322; infopath.3)";
const char S210[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; MSIE 6.0 ( MSIE 7.0; MSIE 8.0; MSIE 9.0 )  Mozilla/3.0; Mozilla/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727)";
const char S211[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; Microsoft Windows Media Center PC 6.0)";
const char S212[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Tablet PC 1.7; .NET CLR 1.0.3705; MS-RTC LM 8)";
const char S213[] = "mozilla/4.0 (compatible; msie 8.0; windows nt 5.1; trident/4.0; gtb7.3; .net clr 1.1.4322; .net clr 2.0.50727; .net clr 3.0.4506.2152; .net clr 3.5.30729; iph 1.1.21.4019; .net4.0c; .net4.0e)";
const char S214[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S215[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S216[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 8.0; Win32; GMX); (gmx/1.1.0.21); .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S217[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; MCI Windows Corporate Image; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S218[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8)";
const char S219[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 7.0; MyBee Browser); .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S220[] = "mozilla/4.0 (compatible; msie 8.0; windows nt 5.1; trident/4.0; searchtoolbar 1.2; gtb7.3; .net clr 1.1.4322; .net clr 2.0.50727; .net clr 3.0.04506.648; .net clr 3.5.21022; .net clr 3.0.4506.2152; .net clr 3.5.30729)";
const char S221[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; 3M/IE 6.0;; 3M/MSIE 6.0;; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; 3M/MSIE 8.0)";
const char S222[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (Compatible; MSIE 6.0;Windows NT;ASPI 1.0); .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Mozilla/4.0 (Compatible; MSIE 6.0;Windows NT;ASPI 1.0))";
const char S223[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; SRS MSIE 7.0; .NET4.0C; .NET4.0E; SRS MSIE 7.0)";
const char S224[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Windows Live Writer 1.0)";
const char S225[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Mozilla/4.0 (compatible; MSIE 7.0; MyBee Browser); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S226[] = "Mozilla/4.0 (Windows; MSIE 8.0; Windows NT 5.1; InetLoader)";
const char S227[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; Media Center PC 3.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)";
const char S228[] = "Mozilla/8.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/2.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1);)";
const char S229[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Mozilla/4.0 (compatible; MSIE 6.0; Win32))";
const char S230[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; 3M/MSIE 6.0;; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; 3M/MSIE 8.0)";
const char S231[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSIE 6.0)";
const char S232[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 7.0; Win32; WEB.DE); Mozilla/4.0 (compatible; MSIE 8.0; Win32; WEB.DE); (webde/1.1.0.21); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S233[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0)";
const char S234[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; .NET CLR 2.0.50727; Zune 4.7)";
const char S235[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; MSIE 6.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; InfoPath.1; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSIE 6.0; MSIE 6.0)";
const char S236[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; SRS MSIE 7.0)";
const char S237[] = "Mozilla/4.0 (compatible; MSIE 6.0; MSIE 7.0; MSIE 8.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S238[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S239[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; MyIE2)";
const char S240[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S241[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; SV1)";
const char S242[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; SV1)";
const char S243[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; AskTbPPC/5.15.4.23821)";
const char S244[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; MSN 9.0;MSN 9.1;MSN 9.6;MSN 10.0;MSN 10.2;MSN 10.5; MSNbMSNI; MSNmen-us; MSNcOTH)";
const char S245[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; MSN 9.0;MSN 9.1;MSN 9.6;MSN 10.0;MSN 10.2;MSN 10.5; MSNbMSNI; MSNmen-us; MSNcOTH)";
const char S246[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; MSIE 6.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; InfoPath.1; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSIE 6.0)";
const char S247[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Sicent1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Sicent1; Media Center PC 6.0; Microsoft Windows Media Center PC 6.0)";
const char S248[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; Media Center PC 6.0; Microsoft Windows Media Center PC 6.0)";
const char S249[] = "Mozilla/4.0 ( ; MSIE 8.0; Windows NT 5.1; Trident/4.0)";
const char S250[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; ARMXKD; .NET CLR 1.1.4322; .NET CLR 2.0.50727)";
const char S251[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; MS-RTC LM 8; MSIE 7.0 (Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Talent2 Hong Kong)))";
const char S252[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; MSIE 6.0)";
const char S253[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.2; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; Verizon Windows Live; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; Verizon Windows Live)";
const char S254[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; AskTbPPC/5.15.4.23821)";
const char S255[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Win32; Trident/5.0)";
const char S256[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0(Compatible Mozilla/4.0(Compatible-EmbeddedWB 14.59 http://bsalsa.com/ EmbeddedWB- 14.59  from: http://bsalsa.com/ ; Mozilla/4.0(Compatible Mozilla/4.0EmbeddedWB- 14.59  from: http://bsalsa.com/ )";
const char S257[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; FunWebProducts; GTB7.4; Mozilla/4.0 (compatible; MSN 9.0;MSN 9.1;MSN 9.6;MSN 10.0;MSN 10.2;MSN 10.5; MSNbMSNI; MSNmen-us; MSNcOTH)";
const char S258[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Internet Explorer (Windows 2k/XP, Child Support Enforcement); .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Internet Explorer (Windows 2k/XP, Child Support Enforcement))";
const char S259[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; (webde/1.1.0.21); (1und1/1.1.0.22); Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1); Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; EasyBits GO v1.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; OfficeLiveConnector.1.5; OfficeLivePatch.1.3)";
const char S260[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1); GTB7.4; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S261[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0; Media Center PC 6.0)";
const char S262[] = "SINA_WEIBO; Mozilla/5.0 (Windows; U; Windows NT 5.1; MSIE8.0; zh-CN; rv:1.9.1.8) Gecko/X Firef8 (.NET CLR 3.5.30729)";
const char S263[] = "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.1) (http://name911.com)";
const char S264[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows XP 5.1; MSIE 8.0.6001.18702)";
const char S265[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; MRA 6.0 (build 6005); User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.1; .NET4.0C; .NET4.0E; MRIE8PACK 2.0.1)";
const char S266[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1); Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1); Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1))";
const char S267[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C; .NET4.0E; Zune 4.7)";
const char S268[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Mozilla/4.0; compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S269[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; Media Center PC 3.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S270[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; QQBrowser/7.1.5753.400)";
const char S271[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; QQBrowser/7.1.5753.400)";
const char S272[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; mz1.9.1.4/xp (compatible;MSIE 6.0; Windows XP); .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; mz1.9.1.4/xp (compatible;MSIE 6.0; Windows XP))";
const char S273[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Windows 98/2000/XP; BWCH-BWA-de-3.1; GTB7.4; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S274[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0(Compatible Mozilla/4.0(Compatible-EmbeddedWB 14.58 http://bsalsa.com/ EmbeddedWB- 14.58  from: http://bsalsa.com/ ; Mozilla/4.0(Compatible RogueCleanerEmbeddedWB- 14.58  from: http://bsalsa.com/ )";
const char S275[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7)";
const char S276[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; QQBrowser/7.2.7006.400)";
const char S277[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Mozilla/4.0 (compatible; MSIE 6.0; Win32); Mozilla/4.0 (compatible; MSIE 6.0; Win32))";
const char S278[] = "Mozilla/4.0+(compatible;+MSIE+8.0;+Windows+NT+5.2)";
const char S279[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; QQBrowser/7.2.7006.400)";
const char S280[] = "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; BRI/2; InfoPath.2; .NET4.0C)";
const char S281[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S282[] = "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/5.0)";
const char S283[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 3.0.30618; Media Center PC 5.0; SLCC1; .NET4.0C; Tablet PC 2.0; .NET4.0E)";
const char S284[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; Trident/4.0; BTRSX; MRSPUTNIK 2, 4, 1, 44; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S285[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; SV1; SV1)";
const char S286[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; SV1; EmbeddedWB 14.52 from: http://www.bsalsa.com/ EmbeddedWB 14.52; Safari)";
const char S287[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 5.2; MSIE 8.0.6001.18702)";
const char S288[] = "MOZNSLYCTBSVNEJQDJ (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; BOIE8;ENUS)";
const char S289[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; EnableKerberos=\"Enable Integrated Windows Authentication\")";
const char S290[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; MSIE 6.0 ( MSIE 7.0; MSIE 8.0; MSIE 9.0 )  Mozilla/3.0; Mozilla/4.0)";
const char S291[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; INFARMED, I.P.; .NET CLR 2.0.50727; INFARMED, I.P.)";
const char S292[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; Trident/4.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0)";
const char S293[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; SV1; SV1)";
const char S294[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; SV1; SV1)";
const char S295[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; SV1; SV1)";
const char S296[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; BTRSX; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S297[] = "Mozilla/4.0 (user_patch_01; MSIE 8.0; Windows NT 5.1; Trident/4.0; pc_patch_01; pc_patch_04; pc_patch_02; pc_patch_03; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S298[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; IDS7 Agent; .NET4.0C; .NET4.0E; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT; pack2); InfoPath.3; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT; pack2))";
const char S299[] = "Mozilla/4.0 (Compatible; Windows NT 5.1; MSIE 8.0) (compatible; MSIE 8.0; Windows NT 5.1;)";
const char S300[] = "Mozilla/4.0 (MSIE 8.0; Windows NT 5.1)";
const char S301[] = "Mozilla/4.0 (compatible: MSIE 8.0; Windows NT 6.1; Trident/4.0; MooMooFroyo/2.2";
const char S302[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1)";
const char S303[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S304[] = "mozilla/4.0 (compatible; msie 8.0; windows nt 6.1; trident/4.0; gtb6.4; slcc2; .net clr 2.0.50727; .net clr 3.5.30729; .net clr 3.0.307)";
const char S305[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0)";
const char S306[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)";
const char S307[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S308[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0)";
const char S309[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S310[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Zune 3.0)";
const char S311[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S312[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 3.0.04506; Media Center PC 5.0; SLCC1; Tablet PC 2.0)";
const char S313[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)";
const char S314[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Media Center PC 5.0; SLCC1)";
const char S315[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)";
const char S316[] = "Windows-RSS-Platform/2.0 (MSIE 8.0; Windows NT 6.1)";
const char S317[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S318[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2; Tablet PC 2.0)";
const char S319[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; 3M/MSIE 8.0;; 3M/MSIE 8.0)";
const char S320[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S321[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 3.0.04506; Media Center PC 5.0; SLCC1)";
const char S322[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Tablet PC 2.0; 3M/MSIE 8.0;; 3M/MSIE 8.0)";
const char S323[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2)";
const char S324[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; Zune 4.7)";
const char S325[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; GTB6.6; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Media Center PC 5.0; SLCC1; Tablet PC 2.0; .NET4.0C)";
const char S326[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7)";
const char S327[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Zune 4.7)";
const char S328[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2; Tablet PC 2.0)";
const char S329[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C)";
const char S330[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Tablet PC 2.0)";
const char S331[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S332[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; .NET4.0C; .NET4.0E)";
const char S333[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S334[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S335[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C; .NET4.0E; 3M/MSIE 8.0)";
const char S336[] = "Mozilla/4.0(compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NETCLR3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S337[] = "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.1; Tridfent/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C)";
const char S338[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S339[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Media Center PC 5.0; SLCC1)";
const char S340[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; 3M/MSIE 8.0;; 3M/MSIE 8.0)";
const char S341[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C; .NET4.0E; 3M/MSIE 8.0)";
const char S342[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Zune 4.7)";
const char S343[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; eSobiSubscriber 2.0.4.16; MAAR; .NET4.0C; AskTbPPC/5.13.1.18107)";
const char S344[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Zune 4.7)";
const char S345[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Tablet PC 2.0; .NET4.0C; .NET4.0E; 3M/MSIE 8.0)";
const char S346[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; 3M/MSIE 8.0)";
const char S347[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2))";
const char S348[] = "Mozilla/4.0 (compatible; MSIE 8.0.7600.16385; Windows NT 6.1)";
const char S349[] = "Mozilla/4.0 (MSIE 8.0; Windows NT 6.1;)";
const char S350[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Zune 4.7)";
const char S351[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Tablet PC 2.0; .NET4.0C; .NET4.0E; 3M/MSIE 8.0)";
const char S352[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7)";
const char S353[] = "Mozilla/4.0 (compatible ; MSIE 8.0.7601.17514 ; Microsoft Windows 7 Professional Service Pack 1 ; Placeware RPC 1.0)";
const char S354[] = "Mozilla/4.0 (compatible ; MSIE 8.0.7600.16385 ; Microsoft Windows 7 Professional  ; Placeware RPC 1.0)";
const char S355[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; SV1; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S356[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; Tablet PC 2.0; .NET4.0E)";
const char S357[] = "Mozilla/4.0 (compatible; MSIE 8.0.7601.17514; Windows NT 6.1)";
const char S358[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Windows NT 6.1;)";
const char S359[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2; .NET4.0C; .NET4.0E; 3M/MSIE 8.0)";
const char S360[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Windows Live Writer 1.0)";
const char S361[] = "Mozilla/4.0 (compatible; MSIE 7.0; MSIE 8.0; PAG-IE8; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S362[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S363[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SV1; [eburo v4.0]; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.3)";
const char S364[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3; Xerox_Win32PWS)";
const char S365[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Puesto Oficina Windows 7 Banesto)";
const char S366[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Puesto Oficina Windows 7 Banesto)";
const char S367[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2))";
const char S368[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; 3M/MSIE 8.0)";
const char S369[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; compatible with MSIE 8.0)";
const char S370[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 3.0.04506; Media Center PC 5.0; SLCC1; .NET4.0C)";
const char S371[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; .NET4.0C; Tablet PC 2.0; MSIE 7.0)";
const char S372[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; 3M/MSIE 8.0;; 3M/MSIE 8.0)";
const char S373[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2; .NET4.0C; .NET4.0E; Zune 4.7; NSN)";
const char S374[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S375[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S376[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Tablet PC 2.0; 3M/MSIE 8.0;; 3M/MSIE 8.0)";
const char S377[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C; .NET4.0E; 3M/MSIE 8.0)";
const char S378[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 3.0.04506; Media Center PC 5.0; SLCC1; Tablet PC 2.0)";
const char S379[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Windows Internet Explorer provided by ABB)";
const char S380[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; GTB6.5; QQDownload 534; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729)";
const char S381[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2))";
const char S382[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Tablet PC 2.0; 3M/MSIE 8.0;; 3M/MSIE 8.0)";
const char S383[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Zune 4.7)";
const char S384[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; SV1; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3)";
const char S385[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Tablet PC 2.0; 3M/MSIE 8.0)";
const char S386[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; SV1; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0)";
const char S387[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) ; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) (none); SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S388[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; SV1)";
const char S389[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2))";
const char S390[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; SV1; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S391[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Xerox_Win32PWS)";
const char S392[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727)";
const char S393[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MSIE 7.0)";
const char S394[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1 SB.X.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MS-RTC LM 8; InfoPath.3; Active Content Browser)";
const char S395[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; GTB7.4; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; AskTbPPC/5.15.4.23821)";
const char S396[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; InfoPath.2; 3M/MSIE 8.0)";
const char S397[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0)";
const char S398[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; SV1; .NET CLR 2.0.50727; Microsoft Windows Media Center PC 6.0)";
const char S399[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; Zune 4.0; .NET CLR 1.";
const char S400[] = "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.1)";
const char S401[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Tablet PC 2.0; Zune 4.7)";
const char S402[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Tablet PC 2.0; Windows Internet Explorer provided by ABB)";
const char S403[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C; .NET4.0E; 3M/MSIE 8.0;; 3M/MSIE 8.0)";
const char S404[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; MSIE 8.0)";
const char S405[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; AskTbPPC/5.13.1.18107)";
const char S406[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E)";
const char S407[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2))";
const char S408[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) ; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) (none); SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)";
const char S409[] = "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; GTB7.4; InfoPath.2; SV1; .NET CLR 3.3.69573; WOW64; en-US)";
const char S410[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; GTB6.6; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Z/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; AskTbMPC2/5.12.2.16749)";
const char S411[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; GTB7.4; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 3.0.30618; Media Center PC 5.0; SLCC1; .NET4.0C; Zune 4.7)";
const char S412[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0(Compatible Mozilla/4.0(Compatible-EmbeddedWB 14.59 http://bsalsa.com/ EmbeddedWB- 14.59  from: http://bsalsa.com/ ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729)";
const char S413[] = "Mozilla/4.0 (Compatible; MSIE 8.0; Windows NT 6.1; WOW64; SLCC1; .NET CLR 3.0.04506; Windows NT 6.1; Trident/4.0; Mozilla/4.0; MSIE 6.0; Windows NT 5.1; SV1 ; SLCC2; .NET CLR 2.0.50727; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NE";
const char S414[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S415[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3; Xerox_Win32PWS)";
const char S416[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.1; MSIE 8.0.7601.17514)";
const char S417[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.1; MSIE 8.0.7600.16385)";
const char S418[] = "Mozilla:4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3)";
const char S419[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; GTB7.4; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; AskTbPPC/5.15.4.23821)";
const char S420[] = "Mozilla/4.0 (compatible: MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3)";
const char S421[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Media Center PC 3.1; SV1; WOW64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 4.0.20402; MS-RTC LM 8)";
const char S422[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) ; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) (none); SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)";
const char S423[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/7.1.5753.400)";
const char S424[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/7.1.5753.400)";
const char S425[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Mozilla/4.0 (compatiblt; MSIE 8.0; Windows NT; LCBSECURE))";
const char S426[] = "Mozilla/4.0 (compatible; MSIE 7.0; MSIE 8.0; PAG-IE8; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S427[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7)";
const char S428[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; 3M/MSIE 8.0)";
const char S429[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SV1; [eburo v4.0]; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.3; Zune 4.7)";
const char S430[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; QQBrowser/7.1.5753.400)";
const char S431[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/7.2.7006.400)";
const char S432[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/7.2.7006.400)";
const char S433[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; Mozilla/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)";
const char S434[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Zune 4.7; .NET4.0C; .NET4.0E; 3M/MSIE 8.0)";
const char S435[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MDDC; Media Center PC 5.0; SLCC1; Tablet PC 2.0; .NET4.0C; .NET4.0E; AskTbCLM/5.9.1.14019)";
const char S436[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) ; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) (none); SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S437[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; MathPlayer 2.10b; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 3.5.21022; InfoPath.2; Media Center PC 5.0; MS-RTC LM 8; SLCC1; Tablet PC 2.0; Zune 4.0; InfoPath.3; .NET4.0C; .NET4.0E)";
const char S438[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Mozilla/6.0 (compatible; MSIE 2.0; Windows Z; Shinigami))";
const char S439[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1 SB.X.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MS-RTC LM 8; InfoPath.3; Tablet PC 2.0; Active Content Browser)";
const char S440[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; SV1; SV1)";
const char S441[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3; Zune 4.7; Xerox_Win32PWS)";
const char S442[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Windows NT 6.1 Enterprise; .NET4.0C; .NET4.0E; InfoPath.2; MS-RTC LM 8)";
const char S443[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Tablet PC 2.0; .NET4.0C; MSIE 6.0)";
const char S444[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Zune 4.7)";
const char S445[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; web-jl-13-76-45-21; web-jl-13-76-45-21; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MS-RTC LM 8; InfoPath.3)";
const char S446[] = "Mozilla/4.0 (Compatible win32 69.124.0.167; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; AskTbTKR/5.13.2.19379)";
const char S447[] = "Mozilla/4.0 (Compatible; MSIE 8.0; Windows NT 6.1; FDM; .NET CLR 1.1.4322; Windows NT 6.1; 001|Mozilla/4.0; MSIE 6.0; Windows NT 5.1; SV1 ; SLCC2; .NET CLR 2.0.50727; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.1.";
const char S448[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; LG_UA; AD_LOGON=LGE.NET; SLCC2; Media Center PC 6.0; InfoPath.2; .NET4.0C; LG_UA; AD_LOGON=LGE.NET)";
const char S449[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; QQBrowser/7.1.5753.400)";
const char S450[] = "Mozilla/4.0+(compatible;+MSIE+8.0;+Windows+NT+6.1;+WOW64;+Trident/4.0;+SLCC2;+.NET+CLR+2.0.50727;+.NET+CLR+3.5.30729;+.NET+CLR+3.0.30729;+.NET4.0C;+.NET4.0E)";
const char S451[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SIMBAR={5634CEA2-A21A-4CE0-B555-C46AEACA80C0}; MVH; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 3.0.30618; .NET CLR 3.5.21022; .NET4.0C; MDDC; Media Center PC 5.0; OfficeLiveConnector.1.3; OfficeLivePatch.0.0; SLCC1; Tablet PC 2.0; Zune 4.7)";
const char S452[] = "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.0.3705; .NET CLR 1.1.4322)";
const char S453[] = "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.30729; Media Center PC 6.0; InfoPath.3)";
const char S454[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 3.5.21022; Media Center PC 5.0; SLCC1; .NET4.0C; .NET4.0E)";
const char S455[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; SV1; SV1)";
const char S456[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Mozilla/4.0 (compatiblt; MSIE 8.0; Windows NT; LCBSECURE))";
const char S457[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SV1; [eburo v4.0]; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.3)";
const char S458[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; SLCC1; Zune 4.7; InfoPath.3; .NET4.0C)";
const char S459[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Tablet PC 2.0; .NET4.0C; .NET4.0E; 3M/MSIE 8.0)";
const char S460[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; 3M/MSIE 8.0)";
const char S461[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET4.0C; .NET4.0E; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MS-RTC LM 8; InfoPath.3; Mozilla/5.0)";
const char S462[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Mozilla/4.0 (compatiblt; MSIE 8.0; Windows NT; LCBSECURE); Mozilla/4.0 (compatiblt; MSIE 9.0; Windows NT; LCBSECURE))";
const char S463[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; LG_UA; AD_LOGON=LGE.NET; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C; LG_UA; AD_LOGON=LGE.NET)";
const char S464[] = "Mozilla/5.0 (compatible; MSIE 8.0; MSBrowserIE8; Windows NT 6.1; WOW64; Trident/5.0; MSBrowserIE9)";
const char S465[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win32; Trident/5.0)";
const char S466[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; Windows NT 5.1; SV1)";
const char S467[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Win64; x64; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S468[] = "Mozilla/4.0 (compatible; MSIE 7.0; America Online Browser 1.1; rev1.5; Windows NT 5.1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; InfoPath.1; .NET CLR 2.0.50727; Media Center PC 3.0; InfoPath.2)";
const char S469[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1)";
const char S470[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2;  SLCC1;  .NET CLR 1.1.4322;  .NET CLR 2.0.50727;  .NET CLR 3.0.04506.648)";
const char S471[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0)";
const char S472[] = "Mozilla/6.0 (compatible; MSIE 7.0a1; Windows NT 5.2; SV1)";
const char S473[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; .NET CLR 2.0.50727)";
const char S474[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2;  SV1;  .NET CLR 1.1.4322;  .NET CLR 2.0.40607;  .NET CLR 3.0.04506.648)";
const char S475[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1;  SV1;  .NET CLR 1.1.4325;  .NET CLR 2.0.40607;  .NET CLR 3.0.04506.648)";
const char S476[] = "Mozilla/4.0 (Mozilla/4.0; MSIE 7.0; Windows NT 5.1; FDM; SV1; .NET CLR 3.0.04506.30)";
const char S477[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S478[] = "Mozilla/4.0 (Windows; MSIE 7.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727)";
const char S479[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S480[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S481[] = "Windows-RSS-Platform/1.0 (MSIE 7.0; Windows NT 5.1)";
const char S482[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; SV1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S483[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2)";
const char S484[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)";
const char S485[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S486[] = "Mozilla/4.0 (compatiable; MSIE 7.0; Windows NT 5.1) HiScan";
const char S487[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) ; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) (none))";
const char S488[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows XP SP2; ERMC HQ); Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2)";
const char S489[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S490[] = "Mozilla/4.0 (compatibile; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322)";
const char S491[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)";
const char S492[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)";
const char S493[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows XP SP2; ERMC HQ); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; .NET4.0C; .NET4.0E)";
const char S494[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; SRS MSIE 7.0; SRS MSIE 7.0)";
const char S495[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; MSIE 6.1 compatible; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.1; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S496[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows 5.1)";
const char S497[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; Verizon Windows Live; Verizon Windows Live)";
const char S498[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; .NET CLR 1.1.4322; MS-RTC LM 8)";
const char S499[] = "Mozilla/5.0 (Windows; U; MSIE 7.0; Windows NT 5.2)";
const char S500[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible;MSIE 5.5;Windows NT; Customised to New Zealand Police Standards); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S501[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727)";
const char S502[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727)";
const char S503[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1 )";
const char S504[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8; Xerox_Win32PWS; Xerox_Win32PWS)";
const char S505[] = "Mozilla/4.0 (compatible; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727); Windows NT 5.1; .NET CLR 2.0.50727)";
const char S506[] = "Mozilla/4.2 (compatible; MSIE 7.0; Windows NT 5.2; en-US)";
const char S507[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; MCI Windows Corporate Image; .NET CLR 1.1.4322; .NET CLR 2.0.50727; MS-RTC LM 8; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S508[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0(Compatible Mozilla/4.0(Compatible-EmbeddedWB 14.59 http://bsalsa.com/ EmbeddedWB- 14.59  from: http://bsalsa.com/ ; .NET CLR 2.0.50727)";
const char S509[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S510[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S511[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 5.1)";
const char S512[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.2; MSIE 6.0; .NET CLR 2.0.50727; MS-RTC LM 8; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; MSIE 6.0)";
const char S513[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1;.NET CLR 2.0.50727)";
const char S514[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 7.0; MyBee Browser); .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S515[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT.5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30))";
const char S516[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1))";
const char S517[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; .NET CLR 1.0.3705; MSIE 7.0;)";
const char S518[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; local_Mobile)";
const char S519[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows XP)";
const char S520[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; SRS MSIE 7.0)";
const char S521[] = "Mozilla/4.0 (compatible MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727)";
const char S522[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Win64; x64; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S523[] = "Mozilla/4.0 (compatible ; MSIE 7.0.5730.13 ; Microsoft Windows XP Professional Service Pack 3 ; Placeware RPC 1.0)";
const char S524[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Mozilla/4.0 (compatible; MSIE 6.0; Win32); Mozilla/4.0 (compatible; MSIE 6.0; Win32))";
const char S525[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)";
const char S526[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Windows Live Messenger 14.0.8117.0416)";
const char S527[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0)";
const char S528[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/5.0)";
const char S529[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; WOW64; SV1)";
const char S530[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1.4; SV1)";
const char S531[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1) ; .NET CLR 3.0.04506.648; MEGAUPLOAD 2.0)";
const char S532[] = "Windows-RSS-Platform/1.0 (MSIE 7.0; Windows NT 5.2)";
const char S533[] = "Mozilla/6.0 (compatible;MSIE 7.0;  Windows NT 5.1; QQDownload)";
const char S534[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; ARMXKD; .NET CLR 1.1.4322; .NET CLR 2.0.50727)";
const char S535[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; Media Center PC 2.8)";
const char S536[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Win64; x64; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S537[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 5.2; Media Center PC 4.0; .NET CLR 3.0.4506.2152; .NET CLR 3.0.04506.30)";
const char S538[] = "Mozilla/3.0 (compatible;MSIE 7.0;  Windows NT 5.1;  MyIE2)";
const char S539[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Crazy Browser 3.0.0 Beta2) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; Zune 2.5)";
const char S540[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Win32); .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Mozilla/4.0 (compatible; MSIE 6.0; Win32); Mozilla/4.0 (compatible; MSIE 6.0; Win32))";
const char S541[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; MS-RTC LM 8; SV1)";
const char S542[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S543[] = "Mozilla/4.0 (compatible; MSIE 7.0; MSBrowserIE7; Windows NT 5.1; msie6xpv1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8; .NET4.0C; .NET4.0E; MSIE 6.0; dmx:m; mty:p; slx:ik.hk.as; ssx:NonCitrix)";
const char S544[] = "Mozilla/5.0 (Windows; U; MSIE 7.0; Windows NT 5.1; en-US)";
const char S545[] = "Mozilla/4.0 (compatible; MSIE 7.5; Windows NT 5.1)";
const char S546[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ;  Embedded Web Browser from: http://bsalsa.com/; .NET CLR 1.1.4322; .NET4.0C; .NET CLR 2.0.50727; .NET4.0E; .NET CLR 3.5.30729; .NET CLR 3.0.4506.2152)";
const char S547[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; Mozilla/4.0 (compatible; MSIE 6.0; Win32))";
const char S548[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S549[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; Xerox_Win32PWS; Xerox_Win32PWS)";
const char S550[] = "Mozilla/7.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/1.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1);)";
const char S551[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Embedded Web Browser from: http://bsalsa.com/; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; Zune 2.5; .NET CLR";
const char S552[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; 360SE)";
const char S553[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; Microsoft Windows Media Center PC 6.0)";
const char S554[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S555[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Cobbetts LLP Citrix; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.1)";
const char S556[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 5.2; SV1)";
const char S557[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0)";
const char S558[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; INFARMED, I.P.; INFARMED, I.P.)";
const char S559[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SearchToolbar 1.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; AskTbNSC-N/5.12.2.16749)";
const char S560[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; MSN 9.0;MSN 9.1;MSN 9.6;MSN 10.0;MSN 10.2;MSN 10.5; MSNbMSNI; MSNmen-us; MSNcOTH)";
const char S561[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; FunWebProducts; GTB7.2; FunWebProducts; AskTbPPC/5.13.1.18107; .NET CLR 2.0.50727)";
const char S562[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Media Center PC 6.0; Microsoft Windows Media Center PC 6.0)";
const char S563[] = "MSIE 7.0; Windows 5.1; compatible";
const char S564[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; IEMB3; GTB7.2; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; MS-RTC EA 2; AskTbPPC/5.12.5.17640; BRI/2; .NET CLR 3.5.30729)";
const char S565[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0; Media Center PC 6.0)";
const char S566[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; SV1; .NET CLR 2.0.50727; Zune 4.7; SE 2.X MetaSr 1.0)";
const char S567[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; MSN 9.0;MSN 9.1;MSN 9.6;MSN 10.0;MSN 10.2;MSN 10.5; MSNbMSNI; MSNmen-us; MSNcOTH)";
const char S568[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows XP 5.1; MSIE 7.0.5730.13)";
const char S569[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; eSobiSubscriber 2.0.4.16; InfoPath.2; OfficeLiveConnector.1.5; OfficeLivePatch.1.3)";
const char S570[] = "('Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30)', 'Internet Explorer 7, Windows XP')";
const char S571[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; QQBrowser/7.1.5753.400)";
const char S572[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; SIMBAR={81E3F033-E6B1-4DC5-82DD-4B34FCB2BBE8}; Mozilla/4.0 (compatible; MSN 9.0;MSN 9.1;MSN 9.6;MSN 10.0; MSNbQ002; MSNmen-us; MSNcOTH)";
const char S573[] = "Mozilla/4.0 (compatible; MSIE 7.0.5730.13; Windows NT 5.1)";
const char S574[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; SV1; QQBrowser/7.1.5753.400)";
const char S575[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; QQBrowser/7.2.7006.400)";
const char S576[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; Windows NT 5.1; SV1)";
const char S577[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; Verizon Windows Live; Verizon Windows Live)";
const char S578[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Win64; x64; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S579[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; GoogleT5; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2; Zune 2.5)";
const char S580[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.0.3705; .NET CLR 2.0.50727; Media Center PC 4.0; .NET CLR 3.0.04506.30; .NET CLR 1.1.4322; Media Center PC 3.0)";
const char S581[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; Safari)";
const char S582[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7)";
const char S583[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1); GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S584[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1;  SV1; Zune 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; FDM)";
const char S585[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64;  SV1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; FDM)";
const char S586[] = "Mozilla/4.0 (compat?vel; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322)";
const char S587[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.21022; Media Center PC 6.0; InfoPath.2; OfficeLiveConnector.1.5; OfficeLivePatch.1.3)";
const char S588[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; WOW64; Trident/4.0; MRA 5.10 (build 5339; SE 2.x); Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET CLR 1.1.4322)";
const char S589[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Sicent1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Sicent1; Media Center PC 6.0; Microsoft Windows Media Center PC 6.0)";
const char S590[] = "Mozilla/4.0 (compatible; GoogleToolbar 4.0.1306.3130-big; Windows XP 5.1; MSIE 7.0.5730.11)";
const char S591[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2;  SV1; GTB6.5; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3)";
const char S592[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1)";
const char S593[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S594[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S595[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; XBLWP7; ZuneWP7)";
const char S596[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S597[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0)";
const char S598[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S599[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S600[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7)";
const char S601[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7)";
const char S602[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Media Center PC 5.0; SLCC1)";
const char S603[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7)";
const char S604[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Zune 4.7)";
const char S605[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S606[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Media Center PC 5.0; SLCC1; .NET4.0C)";
const char S607[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Media Center PC 5.0; SLCC1; Tablet PC 2.0; .NET4.0C)";
const char S608[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Windows Live Messenger 15.4.3508.1109)";
const char S609[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Zune 4.7)";
const char S610[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Media Center PC 5.0; SLCC1; Tablet PC 2.0)";
const char S611[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S612[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0)";
const char S613[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Windows Live Messenger 15.4.3538.0513)";
const char S614[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Windows Live Messenger 15.4.3538.0513)";
const char S615[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Windows Live Messenger 15.4.3538.0513)";
const char S616[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S617[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)";
const char S618[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0)";
const char S619[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET4.0C; .NET4.0E)";
const char S620[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3)";
const char S621[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/4.0; SV1; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; SLCC1)";
const char S622[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0)";
const char S623[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2; Tablet PC 2.0)";
const char S624[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Windows Live Messenger 15.4.3538.0513)";
const char S625[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Windows Live Messenger 14.0.8117.0416)";
const char S626[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Windows Live Messenger 15.4.3502.0922)";
const char S627[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2; Tablet PC 2.0)";
const char S628[] = "Mozilla/4.2 (compatible; MSIE 7.01; Windows NT 6.1)";
const char S629[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Windows Live Messenger 15.4.3508.1109)";
const char S630[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Windows Live Messenger 15.4.3508.1109)";
const char S631[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; BRI/2; Windows Live Messenger 15.4.3508.1109)";
const char S632[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Tablet PC 2.0; Media Center PC 6.0; .NET4.0C)";
const char S633[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S634[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Media Center PC 5.0; SLCC1; Tablet PC 2.0; .NET4.0C; Zune 4.7)";
const char S635[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MS-RTC LM 8; Tablet PC 2.0)";
const char S636[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Windows Live Messenger 15.4.3555.0308)";
const char S637[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Zune 4.7)";
const char S638[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ;  Embedded Web Browser from: http://bsalsa.com/; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2)";
const char S639[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Windows Live Messenger 15.4.3555.0308)";
const char S640[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Windows Live Messenger 15.4.3555.0308)";
const char S641[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; eSobiSubscriber 1.0.0.40; BRI/2; MAAR; .NET4.0C; Windows Live Messenger 15.4.3538.0513)";
const char S642[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.2; MS-RTC LM 8; MSOffice 12)";
const char S643[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Windows Live Messenger 15.4.3555.0308)";
const char S644[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S645[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; InfoPath.3; .NET4.0C; .NET4.0E)";
const char S646[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; Windows Live Messenger 15.4.3555.0308)";
const char S647[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Windows Live Messenger 15.4.3555.0308)";
const char S648[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MSIE 7.0)";
const char S649[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Windows Live Messenger 15.4.3502.0922)";
const char S650[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows XP SP2; ERMC HQ); SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.2)";
const char S651[] = "Mozilla/4.0(compatible;MSIE 7.0;Windows NT 6.1;SV1;.NET CLR 1.0.3705;.NET CLR 3.0.30618)";
const char S652[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; Windows Live Messenger 15.4.3502.0922)";
const char S653[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0); QQBrowser/5.0.6642.400 (trident)";
const char S654[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; MSIE 7.0)";
const char S655[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; MSIE 6.0)";
const char S656[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; GTB7.2; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; AskTbPPC/5.12.2.16749)";
const char S657[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Windows NT 6.1 Enterprise; .NET4.0C; .NET4.0E; InfoPath.2; MS-RTC LM 8)";
const char S658[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Windows Live Messenger 16.4.3503.0728)";
const char S659[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; AskTbPPC/5.13.1.18107)";
const char S660[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S661[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Tablet PC 2.0; MSIE 7.0)";
const char S662[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Tablet PC 2.0; Windows Live Messenger 15.4.3555.0308)";
const char S663[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 6.1; SV1)";
const char S664[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Windows Live Messenger 16.4.3503.0728)";
const char S665[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C)";
const char S666[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0)";
const char S667[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Windows Live Messenger 16.4.3505.0912)";
const char S668[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3; Xerox_Win32PWS)";
const char S669[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Windows Live Messenger 14.0.8117.0416)";
const char S670[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; BRI/2; NP07; .NET4.0C; NP07; Windows Live Messenger 15.4.3508.1109)";
const char S671[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows XP SP2; ERMC HQ); SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.2; Tablet PC 2.0)";
const char S672[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7; Windows Live Messenger 15.4.3555.0308)";
const char S673[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Windows Live Messenger 16.4.3505.0912)";
const char S674[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Zune 4.7; MS-RTC LM 8; Tablet PC 2.0)";
const char S675[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Windows Live Messenger 16.4.3505.0912)";
const char S676[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0)";
const char S677[] = "Mozilla/4.0 ( ; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S678[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/7.1.5753.400)";
const char S679[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/4.0; SV1; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S680[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; Windows Live Messenger 16.4.3505.0912)";
const char S681[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Windows Live Messenger 16.4.3505.0912)";
const char S682[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Zune 4.7)";
const char S683[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; QQBrowser/7.1.5753.400)";
const char S684[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; MathPlayer 2.20; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; Tablet PC 2.0)";
const char S685[] = "Mozilla/5.0  (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media";
const char S686[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.2); QQBrowser/5.1.7061.201 (trident)";
const char S687[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; QQDownload 675; Mozilla/4.0(Compatible Mozilla/4.0(Compatible-EmbeddedWB 14.59 http://bsalsa.com/ EmbeddedWB- 14.59  from: http://bsalsa.com/ ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET";
const char S688[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Windows Media Center PC 6.0)";
const char S689[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Media Center PC 5.0; SLCC1; Zune 4.7)";
const char S690[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; MCSC-CL-001)";
const char S691[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0)";
const char S692[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SV1; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 1.1.4322; .NET CLR 3.5.30729; .NET CLR 3.0.30618)";
const char S693[] = "Windows-RSS-Platform/2.0 (MSIE 8.0; Windows NT 6.0)";
const char S694[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0)";
const char S695[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S696[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)";
const char S697[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S698[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)";
const char S699[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729)";
const char S700[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Verizon Windows Live)";
const char S701[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 3.5.30729)";
const char S702[] = "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/5.0)";
const char S703[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C; Zune 4.7)";
const char S704[] = "Mozilla/4.0 (compatible; GoogleToolbar 5.0.2124.2070; Windows 6.0; MSIE 8.0.6001.18241)";
const char S705[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C; Zune 4.7)";
const char S706[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C; Zune 4.7)";
const char S707[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7)";
const char S708[] = "Mozilla/4.0 (compatible; MSIE8.0; Windows NT 6.0) .NET CLR 2.0.50727)";
const char S709[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; DI60SP1001; SV1; SLCC1; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 3.0.30729; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; .NET CLR 1.1.4322)";
const char S710[] = "Mozilla/4.0 (compatible; GoogleToolbar 6.4.1321.1732; Windows 6.0; MSIE 8.0.6001.19154)";
const char S711[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Win64; x64; Trident/4.0)";
const char S712[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; Mozilla/4.0 (compatible; MSIE 8.0; Win32; WEB.DE); SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)";
const char S713[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7)";
const char S714[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.1.2003.1856; Windows 6.0; MSIE 8.0.6001.19120)";
const char S715[] = "Mozilla/4.0 (compatible; GoogleToolbar 6.4.1321.1732; Windows 6.0; MSIE 8.0.6001.19170)";
const char S716[] = "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.0.3705; .NET CLR 1.1.4322)";
const char S717[] = "Mozilla/4.0 (Windows; MSIE 8.0; Windows NT 6.0)";
const char S718[] = "Mozilla/4.0 (compatible; GoogleToolbar 6.4.1321.1732; Windows 6.0; MSIE 8.0.6001.19190)";
const char S719[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)";
const char S720[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.30729; MDDC; .NET4.0C; Zune 4.7)";
const char S721[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7)";
const char S722[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC1; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)";
const char S723[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; InfoPath.2; .NET CLR 3.0.30729; Zune 4.0; MS-RTC LM 8; MSN Optimized;US)";
const char S724[] = "Mozilla/4.0 (compatible; GoogleToolbar 6.4.1321.1732; Windows 6.0; MSIE 8.0.6001.19222)";
const char S725[] = "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0)";
const char S726[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; .NET CLR 2.0.50727; InfoPath.2)Mozilla/5.0 (Windows; U; MSIE 7.0; Windows NT 6.0; en-US)";
const char S727[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; .NET CLR 3.5.21022; .NET CLR 3.5.30729)";
const char S728[] = "mozilla/4.0 (compatible; msie 8.0; windows nt 6.0; trident/4.0; gtb7.2; slcc1; .net clr 2.0.50727; media center pc 5.0; .net clr 3.5.30729; .net clr 3.0.30618; officeliveconnector.1.5; officelivepatch.1.3)";
const char S729[] = "Mozilla/5.0 (Windows; MSIE 8.0; Windows NT 6.0; en-US)";
const char S730[] = "compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0";
const char S731[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; MSIE 6.4; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; MS-RTC LM 8; .NET CLR 3.0.30729)";
const char S732[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.30729; InfoPath.2; 3M/MSIE 8.0)";
const char S733[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C; Zune 4.7)";
const char S734[] = "Mozilla/4.0 (compatible; GoogleToolbar 6.3.1106.427; Windows 6.0; MSIE 8.0.6001.19048)";
const char S735[] = "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; InfoPath.1; SV1; .NET CLR 3.8.36217; WOW64; en-US)";
const char S736[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.3.2710.138; Windows 6.0; MSIE 8.0.6001.19272)";
const char S737[] = "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; .NET CLR 2.7.58687; SLCC2; Media Center PC 5.0; Zune 3.4; Tablet PC 3.6; InfoPath.3)";
const char S738[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.4; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S739[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.4; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S740[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S741[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; SV1)";
const char S742[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Win64; x64; Trident/4.0; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)";
const char S743[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S744[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S745[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C; AskTbPPC/5.15.4.23821)";
const char S746[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Win64; x64; Trident/4.0; GTB7.4; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; Tablet PC 2.0; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S747[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3230.2052; Windows 6.0; MSIE 8.0.6001.19298)";
const char S748[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3230.2052; Windows 6.0; MSIE 8.0.6001.18904)";
const char S749[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3230.2052; Windows 6.0; MSIE 8.0.6001.19328)";
const char S750[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GIL 3.5; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.21022; .NET CLR 3.5.30729; InfoPath.2; .NET4.0C; .NET4.0E; Windows Server Design)";
const char S751[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB6.6; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; OfficeLiveConnect";
const char S752[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727)";
const char S753[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT6.0)";
const char S754[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.2; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.21022; .NET CLR 3.5.30729; MDDC; .NET4.0C; .NET CLR 3.0.30729; AskTbPPC/5.14.1.20007)";
const char S755[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; Mozilla/4.0 (compatible; MSIE 8.0; Win32; WEB.DE); SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S756[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C; AskTbPPC/5.15.4.23821)";
const char S757[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 8.0.6001.19393)";
const char S758[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 8.0.6001.19088)";
const char S759[] = "Mozilla/4.0 (compatible; MSIE 8.0.6001.19088; Windows NT 6.0)";
const char S760[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; GTB7.2; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)(Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Alexa Toobar)); Alexa Toolbar)";
const char S761[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3230.2052; Windows 6.0; MSIE 8.0.6001.18999)";
const char S762[] = "Mozilla/4.0 ( ; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB6.6; .NET CLR 3.5.30729)";
const char S763[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 8.0.6001.18882)";
const char S764[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Win64; x64; Trident/4.0; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; SLCC1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S765[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 8.0.6001.19400)";
const char S766[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 8.0.6001.19401)";
const char S767[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 8.0.6001.19412)";
const char S768[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; Mozilla/4.0 (compatible; MSIE 7.0; MyBee Browser); SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S769[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; MSN 9.0;MSN 9.1;MSN 9.6;MSN 10.0;MSN 10.2;MSN 10.5; MSNbMSNI; MSNmen-us; MSNcOTH)";
const char S770[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 8.0.6001.18928)";
const char S771[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 8.0.6001.18702)";
const char S772[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 8.0.6001.19019)";
const char S773[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 8.0.6001.18975)";
const char S774[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 8.0.6001.19418)";
const char S775[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C; Zune 4.7)";
const char S776[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; SV1; SV1)";
const char S777[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.5.4209.2358; Windows 6.0; MSIE 8.0.6001.19437)";
const char S778[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; Mozilla/4.0; compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 3.5.30729)";
const char S779[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.5.4209.2358; Windows 6.0; MSIE 8.0.6001.19443)";
const char S780[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; SV1; SV1)";
const char S781[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; InfoPath.1; SV1; .NET CLR 4.8.88265; WOW64; en-US)";
const char S782[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.5.4413.1752; Windows 6.0; MSIE 8.0.6001.19453)";
const char S783[] = "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C)";
const char S784[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.0.3705; .NET CLR 1.1.4322))";
const char S785[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.5.4413.1752; Windows 6.0; MSIE 8.0.6001.19458)";
const char S786[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1); Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1); Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0))";
const char S787[] = "Mozilla/4.0 (comptible; MSIE 8.0; Windows NT 6.0)";
const char S788[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/4.0)";
const char S789[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)";
const char S790[] = "Mozilla/5.0 (Windows; U; MSIE 7.0; Windows NT 6.0; en-US)";
const char S791[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.0)";
const char S792[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)";
const char S793[] = "Mozilla/4.0 (compatible; MSIE 7.0b; Windows NT 6.0 ; .NET CLR 2.0.50215; SL Commerce Client v1.0; Tablet PC 2.0";
const char S794[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.0; en-US)";
const char S795[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SV1; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)";
const char S796[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S797[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SV1; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 3.5.30729; InfoPath.2)";
const char S798[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; Tablet PC 2.0)";
const char S799[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; SLCC1; .NET CLR 3.0.04506)";
const char S800[] = "Windows-RSS-Platform/1.0 (MSIE 7.0; Windows NT 6.0)";
const char S801[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Media Center PC 5.0)";
const char S802[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)";
const char S803[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.30618; .NET CLR 3.5.30729; InfoPath.2; Mozilla/4.0 (compatible; MSIE 7.0b; Windows NT 6.0))";
const char S804[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; Tablet PC 2.0; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)";
const char S805[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7)";
const char S806[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; c .NET CLR 3.0.04506; .NET CLR 3.5.30707; InfoPath.1; el-GR)";
const char S807[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7)";
const char S808[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Windows Live Messenger 15.4.3508.1109)";
const char S809[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7)";
const char S810[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; InfoPath.2; Mozilla/4.0 (compatible; MSIE 7.0b; Windows NT 6.0); .NET CLR 3.0.30729; Mozilla/4.0 (compatible; MSIE 7.0b; Windows NT 6.0))";
const char S811[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7)";
const char S812[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; c .NET CLR 3.0.04506; .NET CLR 3.5.30707; InfoPath.1)";
const char S813[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Windows Live Messenger 15.4.3538.0513)";
const char S814[] = "Mozilla/4.0 (compatible MSIE 7.0 Windows NT 6.0)";
const char S815[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Windows Live Messenger 15.4.3538.0513)";
const char S816[] = "Mozilla/5.0 (Windows; U; MSIE 7.0; Windows NT 6.0)";
const char S817[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC1; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S818[] = "Mozilla/4.0 (Windows; MSIE 7.0; Windows NT 6.0)";
const char S819[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; Zune 4.7)";
const char S820[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.04506; .NET CLR 1.1.4322; InfoPath.2)";
const char S821[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Mozilla/4.0(Compatible Mozilla/4.0(Compatible-EmbeddedWB 14.59 http://bsalsa.com/ EmbeddedWB- 14.59  from: http://bsalsa.com/ ; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618)";
const char S822[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7)";
const char S823[] = "Mozilla/3.0 (compatible;MSIE 7.0;  Windows NT 6.0; 360safe)";
const char S824[] = "Mozilla /4.0 (compatible;MSIE 7.0;Windows NT6.0)";
const char S825[] = "Mozilla/4.0(compatible; MSIE 7.0b; Windows NT 6.0)";
const char S826[] = "Mozilla/6.0 (compatible;MSIE 7.0;  Windows NT 6.0;  MyIE2)";
const char S827[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SV1; SU 3.011; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; .NET CLR 1.1.4322; InfoPath.1)";
const char S828[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; Tablet PC 2.0)";
const char S829[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Windows Live Messenger 15.4.3555.0308)";
const char S830[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Windows Live Messenger 15.4.3555.0308)";
const char S831[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Comcast Install 1.0; FunWebProducts; GTB5; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; Zune 3.0)";
const char S832[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Windows Live Messenger 15.4.3508.1109)";
const char S833[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET CLR 3.0.30618)";
const char S834[] = "Empty (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)";
const char S835[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SU 3.011; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; PeoplePal 6.6; Verizon Windows Live)";
const char S836[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows Vista; Trident/5.0)";
const char S837[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Win64; x64; Trident/5.0; Tablet PC 2.0; .NET4.0C; .NET CLR 3.5.30729)";
const char S838[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Windows Live Messenger 15.4.3555.0308)";
const char S839[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 6.0; SV1)";
const char S840[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S841[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S842[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GIL 3; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.21022; .NET CLR 3.5.30729; InfoPath.2; .NET4.0C; .NET4.0E; Windows Server Design; Windows Server Design)";
const char S843[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S844[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S845[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3230.2052; Windows 6.0; MSIE 7.0.6000.17037)";
const char S846[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SV1; SLCC1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.5.21022; .NET CLR 3.5.30729; InfoPath.2; Tablet PC 2.0; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S847[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.2; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.21022; .NET CLR 3.0.30618; .NET CLR 3.5.30729; .NET4.0C; AskTbPPC/5.14.1.20007)";
const char S848[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Win64; x64; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S849[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; BRI/1; BRI/2; AskTbPPC/5.13.1.18107)";
const char S850[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C; AskTbPPC/5.14.1.20007)";
const char S851[] = "Mozilla/4.0 (Mozilla/5.0 (Windows; U; MSIE 7.0; Windows NT 6.0; en-US)";
const char S852[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; F-6.0SP2-X; SV1; SLCC1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.30729; .NET CLR 3.5.30729; InfoPath.3; InfoPath.1)";
const char S853[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Tablet PC 2.0; InfoPath.2; .NET CLR 1.1.4322; .NET CLR 3.5.30729; MS-RTC LM 8; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S854[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Win64; x64; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; SLCC1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S855[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 7.0.6001.18000)";
const char S856[] = "Mozilla/4.0 (compatible; 7.7; MSIE 7.7; Windows NT 6.0; Trident/5.0)";
const char S857[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 7.0.6000.16982)";
const char S858[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 7.0.6002.18005)";
const char S859[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; Tablet PC 2.0; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7)";
const char S860[] = "Internet Explorer 7 (Windows Vista): Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1)";
const char S861[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30618; .NET CLR 3.5.30729; Zune 4.7)";
const char S862[] = "('Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1)', 'Internet Explorer 7, Windows Vista')";
const char S863[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Vista; Trident/4.0; FunWebProducts; GTB7.4; MS-RTC LM 8; SLCC1; SLCC2; .NET CLR 2.0.50727; InfoPath.3; .NET CLR 3.5.10729; .NET CLR 3.0.33729; .NET CLR 1.1.4322)";
const char S864[] = "Mozilla/4.0 (compatible; MSIE 7.0.6001.18000; Windows NT 6.0)";
const char S865[] = "Mozilla/4.0 (compatible; MSIE 7.0.6000.16982; Windows NT 6.0)";
const char S866[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/4.0; GIL 3.5; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.21022; .NET CLR 3.5.30729; InfoPath.2; .NET4.0C; .NET4.0E; Windows Server Design; MSOffice 12)";
const char S867[] = "Mozilla/4.0 (compatible  MSIE 7.0  Windows NT 6.0  SLCC1  .NET CLR 2.0.50727  Media Center PC 5.0  Tablet PC 2.0  .NET CLR 3.5.30729  .NET CLR 3.0.30729  .NET4.0C)";
const char S868[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0)";
const char S869[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0)";
const char S870[] = "Windows-RSS-Platform/2.0 (MSIE 9.0; Windows NT 6.1)";
const char S871[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0)";
const char S872[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)";
const char S873[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1)";
const char S874[] = "Empty (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0)";
const char S875[] = "Mozilla/5.0 ( ; MSIE 9.0; Windows NT 6.1; Trident/5.0)";
const char S876[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0)";
const char S877[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Zune 4.0; Tablet PC 2.0; InfoPath.3; .NET4.0C; .NET4.0E)";
const char S878[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0)";
const char S879[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Windows Live Writer 1.0)";
const char S880[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; Zune 4.7)";
const char S881[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1)";
const char S882[] = "Mozilla/5.0 ( ; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0)";
const char S883[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; XBLWP7; ZuneWP7)";
const char S884[] = "Empty (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0)";
const char S885[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Verizon Windows Live)";
const char S886[] = "Mozilla/4.0 (compatible; MSIE 9.0.8112.16421; Windows NT 6.1)";
const char S887[] = "Mozilla/4.0 (compatible ; MSIE 9.0.8112.16421 ; Microsoft Windows 7 Professional Service Pack 1 ; Placeware RPC 1.0)";
const char S888[] = "Mozilla/5.0  (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0)";
const char S889[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows 7; Trident/5.0)";
const char S890[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Verizon Windows Live)";
const char S891[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Xerox_Win32PWS)";
const char S892[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; WindowsDownloads)";
const char S893[] = "Mozilla/4.0 (compatible ; MSIE 9.0.8112.16421 ; Microsoft Windows 7 Professional  ; Placeware RPC 1.0)";
const char S894[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S895[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; SV1)";
const char S896[] = "Mozilla/4.0 (compatible; GoogleToolbar 6.6.1409.1944; Windows 6.1; MSIE 9.0.8080.16413)";
const char S897[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET CLR 1.1.4322; .NET4.0C; Tablet PC 2.0)";
const char S898[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT6.1; WOW64; Trident/5.0)";
const char S899[] = "Mozilla/4.0 (compatible; msie 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; SE 2.X MetaSr 1.0)";
const char S900[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1.2; SV1)";
const char S901[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; SV1)";
const char S902[] = "Empty (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0)";
const char S903[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S904[] = "Mozilla/4.0 (compatible; msie 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; SE 2.X MetaSr 1.0)";
const char S905[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1))";
const char S906[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1))";
const char S907[] = "Mozilla/4.0 (compatible; msie 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; SE 2.X MetaSr 1.0)";
const char S908[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Mozilla/4.0 (compatible; MSIE 6.0; Menara))";
const char S909[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)";
const char S910[] = "mozilla/5.0 (compatible; msie 9.0; windows nt 6.1; win64; x64; trident/5.0; funwebproducts; yie9)";
const char S911[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; MSIE 7.0)";
const char S912[] = "Mozilla/5.0 (Windows; U; MSIE 9.0; Windows NT 6.1; de-DE) Trident/5.0";
const char S913[] = "Mozilla/5.0 (MSIE 9.0; Windows NT 6.1; Trident/5.0)";
const char S914[] = "Mozilla/4.0 (compatible; msie 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; SE 2.X MetaSr 1.0)";
const char S915[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1 SB.X.1; Trident/5.0; Active Content Browser)";
const char S916[] = "Mozilla/5.0 ( ; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0)";
const char S917[] = "Mozilla/5.0 (compatible; MSIE 9.0; WindowsNT 6.1; WOW64; Trident/5.0)";
const char S918[] = "Mozilla/4.0 (Windows; MSIE 9.0; Windows NT 6.1; InetLoader)";
const char S919[] = "MSIE 9.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0)";
const char S920[] = "Mozilla/5.0  (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0)";
const char S921[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)";
const char S922[] = "Mozilla/4.0 (compatible; msie 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; SE 2.X MetaSr 1.0)";
const char S923[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; MSIE 6.0)";
const char S924[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1))";
const char S925[] = "MSIE 9.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0)";
const char S926[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727)";
const char S927[] = "Mozilla/4.0 (compatible; msie 9.0; qdesk 2.3.1185.202; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; SE 2.X MetaSr 1.0)";
const char S928[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S929[] = "Internet Explorer (compatible; MSIE 9.0; Windows 7; Trident/5.0)";
const char S930[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Windows Live Mail 1.0)";
const char S931[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1))";
const char S932[] = "Mozilla/4.0 (compatible; msie 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MALN)";
const char S933[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; ARMXKD)";
const char S934[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Windows NT)";
const char S935[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; YDP))";
const char S936[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1); Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1))";
const char S937[] = "Mozilla/4.0 (compatible; msie 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MALN; .NET4.0C; SE 2.X MetaSr 1.0)";
const char S938[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) )";
const char S939[] = "Mozilla/4.0 (compatible; msie 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; SE 2.X MetaSr 1.0)";
const char S940[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; GTB7.4; InfoPath.1; SV1; .NET CLR 2.8.52393; WOW64; en-US)";
const char S941[] = "Mozilla/5.0 (compatible; MSIE 9.10; Windows NT 6.1; Trident/5.0)";
const char S942[] = "Mozilla/4.0 (compatible; msie 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; BRI/2)";
const char S943[] = "MSIE 9.0 ( ; MSIE 9.0; Windows 7; Trident/5.0)";
const char S944[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.1; MSIE 9.0.8112.16421)";
const char S945[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.1; MSIE 9.10.9200.16438)";
const char S946[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Verizon Windows Live)";
const char S947[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0)";
const char S948[] = "Mozilla/5.0  (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0)";
const char S949[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; SV1; QQDownload 713; .NET CLR 2.0.50727; .NET4.0C; WOW64; .NET CLR 3.8.25433; Media Center PC 3.7)";
const char S950[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; QQBrowser/7.1.5753.400)";
const char S951[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; QQBrowser/7.1.5753.400)";
const char S952[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; MSIE 9.0)";
const char S953[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.1; MSIE 9.0.8112.16443)";
const char S954[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.1; MSIE 9.10.9200.16521)";
const char S955[] = "Mozilla/5.0 (compatible MSIE 9.0 Windows NT 6.1 WOW64 Trident/5.0)";
const char S956[] = "Mozilla/4.0 (compatible; MSIE 9.10; Windows NT 6.1; Windows Live Writer 1.0)";
const char S957[] = "Mozilla/5.0 (compatible; msie 9.0; Windows NT 6.1; WOW64; Trident/6.0; MASP)";
const char S958[] = "Mozilla/5.0 (compatible; MSIE 9.10; Windows NT 6.1; WOW64; Trident/6.0; SiteKiosk 7.8 Build 332)";
const char S959[] = "Mozilla/4.0 (compatible ; MSIE 9.10.9200.16521 ; Microsoft Windows 7 Professional Service Pack 1 ; Placeware RPC 1.0)";
const char S960[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; QQBrowser/7.2.7006.400)";
const char S961[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; QQBrowser/7.2.7006.400)";
const char S962[] = "Mozilla/4.0 (compatible; MSIE 9.10; Windows NT 6.1)";
const char S963[] = "Mozilla/5.0 (compatible; msie 9.0; Windows NT 6.1; Trident/6.0; BOIE9;ZHCN)";
const char S964[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Mozilla/4.0)";
const char S965[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Tablet PC 2.0)";
const char S966[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; SV1; SV1)";
const char S967[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; GTB7.4; InfoPath.2; SV1; .NET CLR 4.4.58799; WOW64; en-US)";
const char S968[] = "Mozilla/4.0 (compatible; MSIE 9.10.9200.16521; Windows NT 6.1)";
const char S969[] = "\"Mozilla/5.0 compatible MSIE 9.0 Windows NT 6.1 WOW64 Trident/5.0 BOIE9 ENUS\"";
const char S970[] = "Mozilla/5.0 (compatible; MSIE 9.10; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; SiteKiosk 7.8 Build 332)";
const char S971[] = "Mozilla/4.0 (compatible ; MSIE 9.10.9200.16540 ; Microsoft Windows 7 Professional Service Pack 1 ; Placeware RPC 1.0)";
const char S972[] = "Mozilla/4.0 (compatible; MSIE 9.10.9200.16540; Windows NT 6.1)";
const char S973[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.1; MSIE 9.10.9200.16540)";
const char S974[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1))";
const char S975[] = "Mozilla/4.0 (compatible; MSIE 9.10; Windows NT 6.1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S976[] = "MSIE 9.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0)";
const char S977[] = "Mozilla/4.0 (compatible; MSIE 9.10.9200.16576; Windows NT 6.1)";
const char S978[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.1; MSIE 9.10.9200.16576)";
const char S979[] = "Mozilla/4.0 (compatible ; MSIE 9.10.9200.16576 ; Microsoft Windows 7 Professional Service Pack 1 ; Placeware RPC 1.0)";
const char S980[] = "'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center C 6.0; InfoPath.2; .NET CLR 1.1.4322; .NET4.0C; Tablet PC 2.0)";
const char S981[] = "'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; edia Center PC 6.0; Zune 4.0; Tablet PC 2.0; InfoPath.3; .NET4.0C; .NET4.0E)";
const char S982[] = "'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media enter PC 6.0; Zune 4.0; InfoPath.3; MS-RTC LM 8; .NET4.0C; .NET4.0E)";
const char S983[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.11; WOW64; Trident/5.0)";
const char S984[] = "Mozilla/4.0 (compatible ; MSIE 9.10.9200.16614 ; Microsoft Windows 7 Professional Service Pack 1 ; Placeware RPC 1.0)";
const char S985[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Mozilla/4.0)";
const char S986[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Mozilla/6.0 (compatible; MSIE 2.0; Windows Z; Shinigami))";
const char S987[] = "Mozilla/4.0 (compatible; msie 9.0; Windows NT 6.1; Trident/5.0; SE 2.X MetaSr 1.0)";
const char S988[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Media Center PC 5.0; SLCC1)";
const char S989[] = "Mozilla/4.0 (compatible; msie 9.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S990[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.5.4209.2358; Windows 6.1; MSIE 9.10.9200.16614)";
const char S991[] = "Mozilla/4.0 (compatible; MSIE 9.10.9200.16614; Windows NT 6.1)";
const char S992[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.5.4209.2358; Windows 6.1; MSIE 9.10.9200.16618)";
const char S993[] = "mozilla/4.0 (compatible; MSIE 9.0; windows 7)";
const char S994[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729))";
const char S995[] = "Mozilla/4.0 (compatible ; MSIE 9.10.9200.16635 ; Microsoft Windows 7 Professional Service Pack 1 ; Placeware RPC 1.0)";
const char S996[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.5.4209.2358; Windows 6.1; MSIE 9.10.9200.16635)";
const char S997[] = "Mozilla/4.0 (compatible; MSIE 9.10.9200.16635; Windows NT 6.1)";
const char S998[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; MSIE 9.0)";
const char S999[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; SV1; SV1)";
const char S1000[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1); Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1))";
const char S1001[] = "Mozilla/4.0 (compatible; MSIE 9.10; Windows NT 6.1; Windows Live Mail 1.0)";
const char S1002[] = "Mozilla/5.0 (compatible; MSIE 9.10; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C; MALC; SiteKiosk 7.8 Build 332)";
const char S1003[] = "Mozilla/5.0 (compatible; MSIE 9.10; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; SiteKiosk 7.8 Build 332)";
const char S1004[] = "Mozilla/5.0 (MSIE 9.0; Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko";
const char S1005[] = "Mozilla/5.0 (compatible; MSIE 9.11; Windows NT 6.1; Trident/5.0)";
const char S1006[] = "Mozilla/4.0 (compatible ; MSIE 9.10.9200.16660 ; Microsoft Windows 7 Professional Service Pack 1 ; Placeware RPC 1.0)";
const char S1007[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.5.4413.1752; Windows 6.1; MSIE 9.10.9200.16660)";
const char S1008[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7)";
const char S1009[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Windows Internet Explorer provided by ABB)";
const char S1010[] = "Mozilla/4.0 (compatible ; MSIE 9.10.9200.16618 ; Microsoft Windows 7 Professional Service Pack 1 ; Placeware RPC 1.0)";
const char S1011[] = "Mozilla/4.0 (compatible; MSIE 9.10.9200.16660; Windows NT 6.1)";
const char S1012[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; en-US)";
const char S1013[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1))";
const char S1014[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.5.4413.1752; Windows 6.1; MSIE 9.10.9200.16686)";
const char S1015[] = "Mozilla/4.0 (compatible ; MSIE 9.10.9200.16686 ; Microsoft Windows 7 Professional Service Pack 1 ; Placeware RPC 1.0)";
const char S1016[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.4506.2152; Safari; InfoPath.1)";
const char S1017[] = "Mozilla/4.0 (compatible; MSIE 9.10.9200.16686; Windows NT 6.1)";
const char S1018[] = "Mozilla/5.0 (MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/7.0; rv:11.0) like Gecko";
const char S1019[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.5.4413.1752; Windows 6.1; MSIE 9.11.9600.16384)";
const char S1020[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)";
const char S1021[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; XV6975)";
const char S1022[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; XV6875.1)";
const char S1023[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; XV6875)";
const char S1024[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 1.1.4322)";
const char S1025[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)";
const char S1026[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Mozilla/4.0(compatible; MSIE 6; WindowsNT; ISP26; SV1; PeoplePal 3.0)";
const char S1027[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Media Center PC";
const char S1028[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; Win64; AMD64)";
const char S1029[] = "Mozilla/4.0 (compatible ; MSIE 6.0; Windows NT 5.1)";
const char S1030[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows XP)";
const char S1031[] = "Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.1)";
const char S1032[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; YDP); Wanadoo 5.6)";
const char S1033[] = "Mozilla/4.0 (compatible- MSIE 6.0- Windows NT 5.1- SV1- .NET CLR 1.1.4322";
const char S1034[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) )";
const char S1035[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727)";
const char S1036[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; Win64; x64; SV1; .NET CLR 2.0.50727)";
const char S1037[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; .NET CLR 1.1.4322)";
const char S1038[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; WOW64; SV1; .NET CLR 2.0.50727)";
const char S1039[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727)";
const char S1040[] = "Mozilla/5.0 (Windows; U; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727)";
const char S1041[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Menara))";
const char S1042[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2)";
const char S1043[] = "Mozilla/4.0 (Compatible; Windows NT 5.1; MSIE 6.0) (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727)";
const char S1044[] = "Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 5.1)";
const char S1045[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) )";
const char S1046[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S1047[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322)";
const char S1048[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)";
const char S1049[] = "MSNPTC/1.0 (compatible; MSIE 6.0; Windows NT 5.2; MyIE2; .NET CLR 1.1.4322; .NET CLR 1.0.3705)";
const char S1050[] = "User-Agent: Mozilla/5.0 (compatible; MSIE 6.0;Windows XP)";
const char S1051[] = "Mozilla/6.0 (MSIE 6.0; Windows NT 5.1;)";
const char S1052[] = "Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/5.0)";
const char S1053[] = "Mozilla/4.0 (MSIE 6.0; Windows NT 5.1)";
const char S1054[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; Zune 4.7)";
const char S1055[] = "Mozilla/4.08 (compatible; MSIE 6.0; Windows NT 5.1)";
const char S1056[] = "Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727)";
const char S1057[] = "Mozilla/4.0 (Windows; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727)";
const char S1058[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QS 4.2.1.0; Windows 2000; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.1; .NET CLR 1.1.4322)";
const char S1059[] = "Mozila/4.0 (compatible; MSIE 6.0; Windows NT 5.1;)";
const char S1060[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Poco 0.31; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) ; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) ( EmbeddedWB";
const char S1061[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; 3M/IE 6.0;; 3M/MSIE 6.0;; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.1)";
const char S1062[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; Media Center PC 2.8)";
const char S1063[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0(Compatible Mozilla/4.0(Compatible-EmbeddedWB 14.59 http://bsalsa.com/ EmbeddedWB- 14.59  from: http://bsalsa.com/ )";
const char S1064[] = "Mozilla/45.0 (compatible; MSIE 6.0; Windows NT 5.1)";
const char S1065[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; Win 9x 4.90; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) )";
const char S1066[] = "Mozilla/4.0 (Windows; MSIE 6.0; Windows NT 5.2)";
const char S1067[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1.)";
const char S1068[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S1069[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; Win 9x 4.90; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) )";
const char S1070[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; SV1)";
const char S1071[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Windows Live Messenger 14.0.8117.0416)";
const char S1072[] = "Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) )";
const char S1073[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0 (Compatible; MSIE 6.0;Windows NT;ASPI 1.0); .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S1074[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QS 4.2.1.0; Windows 2000; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 2.0.50727; InfoPath.1)";
const char S1075[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)";
const char S1076[] = "Mozilla/4.0 (compatible; MSIE6.0; Windows XP; SV1)";
const char S1077[] = "Mozila/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; MyIE2;";
const char S1078[] = "Mozilla/4.0 (compatiable; MSIE 6.0; Windows NT 5.1) HiScan";
const char S1079[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0)";
const char S1080[] = "Mozilla/4.0 (compatible; MSIE 6.0.2800; Windows NT 5.1)";
const char S1081[] = "Mozilla/4.1 (compatible; MSIE 6.0; Windows NT 5.1; en)";
const char S1082[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322)";
const char S1083[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1.8; SV1)";
const char S1084[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1.2; SV1)";
const char S1085[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S1086[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Sicent1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Sicent1; Media Center PC 6.0; Microsoft Windows Media Center PC 6.0)";
const char S1087[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; GX755 XPUSV1.X; SVCVersion 7.30; SVCVersion 8.11; SVCVersion 8.14; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; InfoPath.1)";
const char S1088[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1.3; SV1)";
const char S1089[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows XP 5.1) Lobo/0.98.4";
const char S1090[] = "Mozilla/4.0 (Compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; InfoPath.2; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S1091[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) ; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) (none))";
const char S1092[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; AskTbPPC/5.12.2.16749)";
const char S1093[] = "Mozilla/4.0 (MSIE 6.0; Windows NT 5.1; Search)";
const char S1094[] = "Mozilla/5.0 (compatible; MSIE 5.0; Windows NT 5.1; Trident/1.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1);)";
const char S1095[] = "Mozilla/6.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/2.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1);)";
const char S1096[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0)";
const char S1097[] = "Mozilla/4.0 (compatible; U; MSIE 6.0; Windows NT 5.1) (Compatible; Trident/4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.0.3705; .NET CLR 1.1.4322)";
const char S1098[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; FunWebProducts; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; Zune 2.0; MS-RTC LM 8)";
const char S1099[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C)";
const char S1100[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0)";
const char S1101[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; Zune 4.7)";
const char S1102[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) )";
const char S1103[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Sicent; WoShiHoney.B; Mozilla/4.0(Compatible Mozilla/4.0(Compatible-EmbeddedWB 14.59 http://bsalsa.com/ EmbeddedWB- 14.59  from: http://bsalsa.com/ ; Media Center PC 6.0)";
const char S1104[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QS 4.2.1.0; Windows 2000 CIT CS; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.1; .NET CLR 1.1.4322)";
const char S1105[] = "Mozilla/4.0 (compatible- MSIE 6.0- Windows NT 5.1- SV1- .NET CLR1.1.4322";
const char S1106[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)";
const char S1107[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; 360SE) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S1108[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0(Compatible Mozilla/4.0(Compatible-EmbeddedWB 14.58 http://bsalsa.com/ EmbeddedWB- 14.58  from: http://bsalsa.com/ ; Mozilla/4.0(Compatible RogueCleanerEmbeddedWB- 14.58  from: http://bsalsa.com/ )";
const char S1109[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S1110[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Tablet PC 2.0)";
const char S1111[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0; Media Center PC 6.0)";
const char S1112[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 5.1)";
const char S1113[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0(Compatible Mozilla/4.0(Compatible-EmbeddedWB 14.59 http://bsalsa.com/ EmbeddedWB- 14.59  from: http://bsalsa.com/ )";
const char S1114[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)";
const char S1115[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; SV1; Mozilla/4.0 (X11; MSIE 6.0; i686; .NET CLR 1.1.4322; .NET CLR 2.0.50727; FDM))";
const char S1116[] = "('Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 1.1.4322)', 'Internet Explorer 6, Windows XP')";
const char S1117[] = "MSIE 6 (Win XP): Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)";
const char S1118[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0; Media Center PC 6.0)";
const char S1119[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; TencentTraveler 4.0; QQDownload 677); QQBrowser/5.0.6648.400 (trident)";
const char S1120[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; Foxy/1; BTRSX; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) ; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) (Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)); .NET CLR 2.0.50727; AskTbSPC2/5.13.1.18107)";
const char S1121[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows XP 5.1; MSIE 6.0.2900.5512)";
const char S1122[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7)";
const char S1123[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QQBrowser/7.1.5753.400)";
const char S1124[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S1125[] = "Mozilla/4.0 (compatible ; MSIE 6.0.2900.5512 ; Microsoft Windows XP Professional Service Pack 3 ; Placeware RPC 1.0)";
const char S1126[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7)";
const char S1127[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows XP 5.1; MSIE 6.0.2900.2180)";
const char S1128[] = "Mozilla/6.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727)";
const char S1129[] = "Mozilla/4.0 (compatible\\; MSIE 6.0\\; Windows NT 5.1\\; SV1\\; .NET CLR 1.1.4322)";
const char S1130[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 3.0.30618; Media Center PC 5.0; SLCC1; .NET4.0C; Tablet PC 2.0; .NET4.0E)";
const char S1131[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; Trident/4.0; BTRSX; MRSPUTNIK 2, 4, 1, 44; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S1132[] = "Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 5.2; Trident/1.0)";
const char S1133[] = "Mozilla/4.0 (iPassConnect/PU 8.6.0.4; MSIE 6.0; Windows NT 5.1)";
const char S1134[] = "Mozilla/4.0   (compatible; MSIE 6.0; America Online Browser 1.1; rev1.5; Windows NT 5.1;)";
const char S1135[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C; .NET4.0E; Zune 4.7)";
const char S1136[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; icafe8; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Microsoft Windows Media Center PC 6.0)";
const char S1137[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7)";
const char S1138[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; Trident/4.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; ASU2JS)";
const char S1139[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; BTRSX; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S1140[] = "Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 5.2)";
const char S1141[] = "CrawlerBoy Pinpoint.com";
const char S1142[] = "msnbot/1.1 (+http://search.msn.com/msnbot.htm)";
const char S1143[] = "msnbot/2.0b (+http://search.msn.com/msnbot.htm)";
const char S1144[] = "Baiduspider+(+http://www.baidu.com/search/spider.htm)";
const char S1145[] = "SimpleCrawler/0.1";
const char S1146[] = "bot";
const char S1147[] = "AdsBot-Google (+http://www.google.com/adsbot.html)";
const char S1148[] = "Mozilla/5.0 (compatible; sindice-fetcher/0.1.0 +http://sindice.com/developers/bot)";
const char S1149[] = "Baiduspider+(+http://www.baidu.jp/spider/)";
const char S1150[] = "bitlybot";
const char S1151[] = "TurnitinBot/2.1 (http://www.turnitin.com/robot/crawlerinfo.html)";
const char S1152[] = "Y!J-BRI/0.0.1 crawler ( http://help.yahoo.co.jp/help/jp/search/indexing/indexing-15.html )";
const char S1153[] = "ia_archiver (+http://www.alexa.com/site/help/webmasters; crawler@alexa.com)";
const char S1154[] = "Sogou web spider/4.0(+http://www.sogou.com/docs/help/webmasters.htm#07)";
const char S1155[] = "Mozilla/5.0 (compatible; pmoz.info ODP link checker; +http://pmoz.info/doc/botinfo.htm)";
const char S1156[] = "Magus Bot 1.0";
const char S1157[] = "msnbot-UDiscovery/2.0b (+http://search.msn.com/msnbot.htm)";
const char S1158[] = "RSSMicro.com RSS/Atom Feed Robot";
const char S1159[] = "AlbertBot";
const char S1160[] = "Mozilla/4.0 (compatible; NaverBot/1.0; http://help.naver.com/delete_main.asp)";
const char S1161[] = "Willow Internet Crawler by Twotrees V2.1";
const char S1162[] = "Checklinks/1.3 (pywikipedia robot; http://toolserver.org/~dispenser/view/Checklinks)";
const char S1163[] = "Baiduspider-image+(+http://www.baidu.com/search/spider.htm)";
const char S1164[] = "Mozilla/5.0 (compatible; RSSMicro.com RSS/Atom Feed Robot)";
const char S1165[] = "Mozilla/5.0 (compatible; Exabot/3.0; +http://www.exabot.com/go/robot)";
const char S1166[] = "MLBot (www.metadatalabs.com/mlbot)";
const char S1167[] = "Mozilla/5.0 (compatible; woriobot support [at] worio [dot] com +http://worio.com)";
const char S1168[] = "Mozilla/5.0 (compatible; MJ12bot/v1.2.1; http://www.majestic12.co.uk/bot.php?+)";
const char S1169[] = "Mozilla/5.0 (compatible; YoudaoBot/1.0; http://www.youdao.com/help/webmaster/spider/; )";
const char S1170[] = "NextGenSearchBot 1 (for information visit http://www.zoominfo.com/About/misc/NextGenSearchBot.aspx)";
const char S1171[] = "msnbot-media/1.1 (+http://search.msn.com/msnbot.htm)";
const char S1172[] = "R6_FeedFetcher(www.radian6.com/crawler)";
const char S1173[] = "Sosospider+(+http://help.soso.com/webspider.htm)";
const char S1174[] = "Mozilla/5.0 (compatible; FriendFeedBot/0.1; +Http://friendfeed.com/about/bot)";
const char S1175[] = "Mozilla/5.0 (compatible; woriobot +http://worio.com)";
const char S1176[] = "Baiduspider+(+http://help.baidu.jp/system/05.html)";
const char S1177[] = "Mozilla/5.0 (compatible; Purebot/1.1; +http://www.puritysearch.net/)";
const char S1178[] = "Mozilla/5.0 (compatible; aiHitBot/1.0; +http://www.aihit.com/)";
const char S1179[] = "ShopWiki/1.0 ( +http://www.shopwiki.com/wiki/Help:Bot)";
const char S1180[] = "Mozilla/5.0 (compatible; Dow Jones Searchbot)";
const char S1181[] = "Mozilla/5.0 (compatible; FatBot 2.0; http://www.thefind.com/crawler)";
const char S1182[] = "Xenu Link Sleuth/1.3.7";
const char S1183[] = "Mozilla/5.0 (compatible; Embedly/0.2; +http://support.embed.ly/)";
const char S1184[] = "SuperPagesUrlVerifyBot/1.0";
const char S1185[] = "Mozilla/5.0 (compatible; discobot/1.1; +http://discoveryengine.com/discobot.html)";
const char S1186[] = "spider";
const char S1187[] = "Mozilla/5.0 (compatible; seexie.com_bot/4.1; +http://www.seexie.com)";
const char S1188[] = "Mozilla/5.0 (compatible; MarketwireBot/1.0; +http://www.marketwire.com)";
const char S1189[] = "Mozilla/4.0 (compatible; Inspyder-Crawler; http://www.inspyder.com)";
const char S1190[] = "Mozilla/5.0 (compatible; YodaoBot/1.0; http://www.yodao.com/help/webmaster/spider/; )";
const char S1191[] = "TjoosBot/1.0 (http://www.tjoos.com/Page-Content/34/TjoosBot)";
const char S1192[] = "ICC-Crawler/2.0 (Mozilla-compatible; ; http://kc.nict.go.jp/project1/crawl.html)";
const char S1193[] = "psbot/0.1 (+http://www.picsearch.com/bot.html)";
const char S1194[] = "Mozilla/5.0 (compatible; Qualidator.com Bot 1.0;)";
const char S1195[] = "UCMore Crawler App";
const char S1196[] = "Mozilla/5.0 (compatible; Nigma.ru/3.0; crawler@nigma.ru)";
const char S1197[] = "Sosoimagespider+(+http://help.soso.com/soso-image-spider.htm)";
const char S1198[] = "Mozilla/5.0 (compatible; YandexBot/3.0; +http://yandex.com/bots)";
const char S1199[] = "Mozilla/5.0 (compatible; YandexImages/3.0; +http://yandex.com/bots)";
const char S1200[] = "crawler4j (http://code.google.com/p/crawler4j/)";
const char S1201[] = "Mozilla/5.0 (compatible; NetSeer crawler/2.0; +http://www.netseer.com/crawler.html; crawler@netseer.com)";
const char S1202[] = "Mozilla/5.0 (compatible;YodaoBot-Image/1.0;http://www.youdao.com/help/webmaster/spider/;)";
const char S1203[] = "Mozilla/5.0 (compatible; bnf.fr_bot; +http://www.bnf.fr/fr/outils/a.dl_web_capture_robot.html)";
const char S1204[] = "TwengaBot";
const char S1205[] = "Caliperbot/1.0 (+http://www.conductor.com/caliperbot)";
const char S1206[] = "zibber-v0.1(www.zibb.com/crawler/)";
const char S1207[] = "Flamingo_SearchEngine (+http://www.flamingosearch.com/bot)";
const char S1208[] = "oodlebot/1.0";
const char S1209[] = "Mozilla/5.0 (compatible; YesupBot/1.0; +http://www.yesup.net/bot.html)";
const char S1210[] = "Mozilla/5.0 (compatible; Feedage/2.0; +http://www.feedage.com/bot.php)";
const char S1211[] = "HLTBot/1.3 (+http://hlt.fbk.eu/hltbot.html)";
const char S1212[] = "Crawler";
const char S1213[] = "HubSpot Crawler 1.0 http://www.hubspot.com/";
const char S1214[] = "AddThis.com robot tech.support@clearspring.com";
const char S1215[] = "Mozilla/5.0 (compatible; Yahoo!-AdCrawler; http://help.yahoo.com/yahoo_adcrawler)";
const char S1216[] = "Gigabot/3.0 (http://www.gigablast.com/spider.html)";
const char S1217[] = "Who.is Bot";
const char S1218[] = "web18bot";
const char S1219[] = "Jyxobot/1";
const char S1220[] = "Mozilla/5.0 (compatible; Apercite; +http://www.apercite.fr/robot/index.html)";
const char S1221[] = "Mozilla/4.0 (compatible;  Vagabondo/4.0; webcrawler at wise-guys dot nl; http://webagent.wise-guys.nl/)";
const char S1222[] = "CatchBot/1.0; +http://www.catchbot.com";
const char S1223[] = "R6_CommentReader(www.radian6.com/crawler)";
const char S1224[] = "Mozilla/4.0 (compatible;  Vagabondo/4.0; webcrawler at wise-guys dot nl; http://webagent.wise-guys.nl/; http://www.wise-guys.nl/)";
const char S1225[] = "msnbot/2.0b (+http://search.msn.com/msnbot.htm)._";
const char S1226[] = "iisbot/1.0 (+http://www.iis.net/iisbot.html)";
const char S1227[] = "solfo-linkchecker/1.0 (http://solfo.com/linkbot.html)";
const char S1228[] = "Mozilla/5.0 (compatible; Search17Bot/1.1; http://www.search17.com/bot.php)";
const char S1229[] = "Twitterbot/0.1";
const char S1230[] = "LinkedInBot/1.0 (compatible; Mozilla/5.0; Jakarta Commons-HttpClient/3.1 +http://www.linkedin.com)";
const char S1231[] = "Mozilla/5.0 (compatible; YandexImageResizer/2.0; +http://yandex.com/bots)";
const char S1232[] = "msnbot-NewsBlogs/2.0b (+http://search.msn.com/msnbot.htm)";
const char S1233[] = "bitlybot/2.0";
const char S1234[] = "adidxbot/1.1 (+http://search.msn.com/msnbot.htm)";
const char S1235[] = "my-robot/0.1";
const char S1236[] = "Xenu Link Sleuth/1.3.8";
const char S1237[] = "FlaxCrawler/1.0";
const char S1238[] = "CatchBot/2.0; +http://www.catchbot.com";
const char S1239[] = "eventBot";
const char S1240[] = "Mozilla/5.0 (compatible; MJ12bot/v1.3.3; http://www.majestic12.co.uk/bot.php?+)";
const char S1241[] = "Mozilla/5.0 (compatible; archive.org_bot +http://www.archive.org/details/archive.org_bot)";
const char S1242[] = "123peoplebot/1.0";
const char S1243[] = "wikiwix-bot-3.0";
const char S1244[] = "Sosospider+(+http://help.soso.com/webspider.htm),gzip(gfe) (via docs.google.com/viewer)";
const char S1245[] = "Mozilla/5.0 (compatible; YandexAntivirus/2.0; +http://yandex.com/bots)";
const char S1246[] = "Synthesio Crawler release MonaLisa (contact at synthesio dot fr)";
const char S1247[] = "ssearch_bot (sSearch Crawler; http://www.semantissimo.de)";
const char S1248[] = "R6_Primer(www.radian6.com/crawler)";
const char S1249[] = "Zing-BottaBot/1.0";
const char S1250[] = "Toplistbot";
const char S1251[] = "WebZIP/x.x (http://www.spidersoft.com)";
const char S1252[] = "TVersity Media Robot";
const char S1253[] = "WordChampBot";
const char S1254[] = "Mozilla/5.0 (compatible; rogerBot/1.0; UrlCrawler; http://www.seomoz.org/dp/rogerbot)";
const char S1255[] = "BLP_bbot/0.1";
const char S1256[] = "Mozilla/5.0 (compatible; rogerBot/1.0; http://www.seomoz.org/dp/rogerbot)";
const char S1257[] = "Mozilla/5.0 AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7; NextpointBot/1.0; http://nextpoint.com/";
const char S1258[] = "GumGum Bot (http://gumgum.com; support@gumgum.com)";
const char S1259[] = "netEstate NE Crawler (+http://www.sengine.info/)";
const char S1260[] = "Twitmunin Crawler http://www.twitmunin.com";
const char S1261[] = "Altresium/4.3 (+http://www.altresium.com/bot.html)";
const char S1262[] = "eSyndiCat Bot";
const char S1263[] = "Xenu Link Sleuth/1.3.9 beta";
const char S1264[] = "Mozilla/4.0 (CMS Crawler: http://www.cmscrawler.com)";
const char S1265[] = "FAST Enterprise Crawler/6.7.0 (crawler@fast.no)";
const char S1266[] = "ScooperBot www.customscoop.com";
const char S1267[] = "Mozilla/5.0 (compatible; spbot/3.0; +http://www.seoprofiler.com/bot )";
const char S1268[] = "SinaWeiboBot";
const char S1269[] = "Mozilla/5.0 (compatible; sindice-site-manager/0.1.0 +http://sindice.com/developers/bot)";
const char S1270[] = "Mozilla/5.0 (compatible; trovitBot/1.0; +http://www.trovit.com/bot.html)";
const char S1271[] = "TwengaBot-2.0 (http://www.twenga.com/bot.html)";
const char S1272[] = "reach-local-bot/0.1";
const char S1273[] = "Mozilla/5.0 (compatible; Ezooms/1.0; ezooms.bot@gmail.com)";
const char S1274[] = "RSSIncludeBot/1.0 (http://www.rssinclude.com/spider)";
const char S1275[] = "YoonoCrawler/1.0 (crawler@yoono.com)";
const char S1276[] = "Mozilla/5.0 (compatible; suggybot v0.01a, http://blog.suggy.com/was-ist-suggy/suggy-webcrawler/)";
const char S1277[] = "Mozilla/5.0 (compatible; Baiduspider/2.0; +http://www.baidu.com/search/spider.html)";
const char S1278[] = "fastbot crawler beta 2.0 (+http://www.fastbot.de)";
const char S1279[] = "Mozilla/5.0 (compatible; InfuzApp/1.0; +http://www.infuz.com/bot.html)";
const char S1280[] = "Mozilla/5.0 (compatible; ZapBot/0.2n; +http://www.zapbot.net)";
const char S1281[] = "Mozilla/5.0 (compatible; TweetedTimes Bot/1.0; +http://tweetedtimes.com)";
const char S1282[] = "NL-Crawler";
const char S1283[] = "Mozilla/5.0 (compatible; NerdByNature.Bot; http://www.nerdbynature.net/bot)";
const char S1284[] = "Mozilla/5.0 (compatible; KaloogaBot; http://kalooga.com/crawler)";
const char S1285[] = "Screaming Frog SEO Spider/1.40";
const char S1286[] = "Mozilla/5.0 (compatible; musobot/1.0; info@muso.com; +http://www.muso.com)";
const char S1287[] = "WillyBot/1.1 (http://www.willyfogg.com/info/willybot)";
const char S1288[] = "vcbot";
const char S1289[] = "Mozilla/5.0 (compatible; YandexDirect/3.0; +http://yandex.com/bots)";
const char S1290[] = "Twitterbot/1.0";
const char S1291[] = "mysmutsearch-crawler +http://www.mysmutsearch.com/faq.shtml+";
const char S1292[] = "Mozilla/5.0 (compatible; ZapBot/0.2c; +http://www.zapbot.com)";
const char S1293[] = "Mozilla/5.0 (compatible; ZapBot/0.2o; +http://www.zapbot.org)";
const char S1294[] = "Mozilla/5.0 (compatible; YandexNews/3.0; +http://yandex.com/bots)";
const char S1295[] = "woobot/1.1";
const char S1296[] = "yolinkBot";
const char S1297[] = "Mozilla/5.0 (compatible; pub-crawler; +http://wiki.github.com/bixo/bixo/bixocrawler; bixo-dev@yahoogroups.com)";
const char S1298[] = "msnbot/0.01 (+http://search.msn.com/msnbot.htm)";
const char S1299[] = "Mozilla/5.0 (compatible; LikaholixCrawler/1.0; +http://mylikes.com/about/crawler)";
const char S1300[] = "Mozilla/5.0 (compatible; YandexBot/3.0; +http://yandex.com/bots),gzip(gfe) (via docs.google.com/viewer)";
const char S1301[] = "Oxyme.Search - Web crawler";
const char S1302[] = "Screaming Frog SEO Spider/1.60";
const char S1303[] = "Mozilla/5.0 (compatible; Bender; http://benderthewebrobot.tumblr.com)";
const char S1304[] = "Kyoto-Crawler/2.0 (Mozilla-compatible; kyoto-crawler-contact(at)nlp(dot)kuee(dot)kyoto-u(dot)ac(dot)jp; http://nlp.ist.i.kyoto-u.ac.jp/)";
const char S1305[] = "Newsle bot";
const char S1306[] = "SeznamBot/3.0 (+http://fulltext.sblog.cz/)";
const char S1307[] = "Mozilla/5.0 (compatible; SEOkicks-Robot +http://www.seokicks.de/robot.html)";
const char S1308[] = "BacklinkCrawler (http://www.backlinktest.com/crawler.html)";
const char S1309[] = "Y!J-BRW/1.0 crawler (http://help.yahoo.co.jp/help/jp/search/indexing/indexing-15.html)";
const char S1310[] = "MozillaTest/5.0 (compatible; YodaoBot/1.0; http://www.yodao.com/help/webmaster/spider/; )";
const char S1311[] = "Mozilla/5.0 (compatible; MJ12bot/v1.4.0; http://www.majestic12.co.uk/bot.php?+)";
const char S1312[] = "Mozilla/5.0 (compatible; MojeekBot/0.2; http://www.mojeek.com/bot.html)";
const char S1313[] = "UnisterBot (Mozilla/5.0 compatible; crawler@unister-gmbh.de)";
const char S1314[] = "Mozilla/5.0 (compatible; Jetslide; +http://jetsli.de/crawler)";
const char S1315[] = "Screaming Frog SEO Spider/1.70";
const char S1316[] = "gsa-crawler (Enterprise; GID-01422; jplastiras.com)";
const char S1317[] = "Mozilla/5.0 (compatible;Baiduspider/2.0;+http://www.baidu.com/search/spider.html)";
const char S1318[] = "Mozilla/5.0 (compatible; SISTRIX Crawler; http://crawler.sistrix.net/)";
const char S1319[] = "Screaming Frog SEO Spider/1,70";
const char S1320[] = "Mozilla/5.0+(compatible;+PiplBot;++http://www.pipl.com/bot/)";
const char S1321[] = "RediffNewsBot;betatest;contact:searchops@rediff.co.in";
const char S1322[] = "AdnormCrawler www.adnorm.com/crawler";
const char S1323[] = "Mozilla/5.0 (compatible; LucidWorks/; +; crawler at example dot com)";
const char S1324[] = "Yepi/1.0 (NHN Corp.; http://help.naver.com/robots/)";
const char S1325[] = "Mozilla/5.0 (compatible; oBot/2.3.1; +http://filterdb.iss.net/crawler/)";
const char S1326[] = "Content Crawler";
const char S1327[] = "SearQuBot/SearQuBot v1.0";
const char S1328[] = "Mozilla/5.0 (compatible; YandexNews/3.0; robot; +http://yandex.com/bots)";
const char S1329[] = "X3Watch Crawler (http://x3watch.com)";
const char S1330[] = "Mozilla/5.0+(compatible;+CukBot;+Not+a+spammer;+++http://www.companiesintheuk.co.uk/bot.html)";
const char S1331[] = "PostPost/1.0 (+http://postpost.com/crawlers)";
const char S1332[] = "ZumBot/1.0 (ZUM Search; http://help.zum.com/inquiry)";
const char S1333[] = "Screaming Frog SEO Spider/1.80";
const char S1334[] = "Mozilla/5.0 (compatible; Goodzer/2.0; crawler@goodzer.com)";
const char S1335[] = "Mozilla/5.0 (compatible; Moatbot/2.2; +http://www.moat.com/pages/moatbot)";
const char S1336[] = "Mozilla/5.0 (compatible; WBSearchBot/1.1; +http://www.warebay.com/bot.html)";
const char S1337[] = "BufferBot";
const char S1338[] = "Wepbot/3.0 (+http://www.nurago.com/en/wepbot/); Jakarta Commons-HttpClient/3.1";
const char S1339[] = "FAST-WebCrawler/3.8 (crawler at trd dot overture dot com; http://www.alltheweb.com/help/webmaster/crawler)";
const char S1340[] = "Mozilla/4.0 (compatible; crawlx, crawler@trd.overture.com)";
const char S1341[] = "CCBot/1.0 (+http://www.commoncrawl.org/bot.html)";
const char S1342[] = "Snapbot/1.0 (Snap Shots, +http://www.snap.com)";
const char S1343[] = "thumbshots-de-bot (+http://www.thumbshots.de/)";
const char S1344[] = "Mozilla/5.0 (compatible; YandexMedia/3.0; +http://yandex.com/bots)";
const char S1345[] = "Mozilla/5.0 (compatible; YandexFavicons/1.0; +http://yandex.com/bots)";
const char S1346[] = "librabot/2.0 (+http://academic.research.microsoft.com/)";
const char S1347[] = "Mozilla/5.0 (compatible; SiteBot/0.1; +http://www.sitebot.org/robot/)";
const char S1348[] = "ibot";
const char S1349[] = "GSiteCrawler/v1.23 rev. 286 (http://gsitecrawler.com/)";
const char S1350[] = "Mozilla/5.0 (compatible; ia_archiver/1.0; +http://www.alexa.com/help/webmasters; crawler@alexa.com)";
const char S1351[] = "IssueCrawler";
const char S1352[] = "LocalBot/1.1 (+http://www.localbot.co.uk/)";
const char S1353[] = "InnovantageBot/1.0 (http://www.innovantage.co.uk/technology/webmaster_information.htm)";
const char S1354[] = "Mozilla/5.0 (compatible; redditbot/1.0; +http://www.reddit.com/feedback)";
const char S1355[] = "Baiduspider";
const char S1356[] = "Mozilla/5.0 (compatible; YandexBlogs/0.99; robot; B; +http://yandex.com/bots) 1 readers";
const char S1357[] = "Mozilla/5.0 (compatible; FatBot 2.0; http://www.thefind.com/crawler; panda)";
const char S1358[] = "Yahoo-Newscrawler/3.9 (news-search-crawler at yahoo-inc dot com)";
const char S1359[] = "SMXCrawler/1.0 (www.socialmetrix.com)";
const char S1360[] = "Mozilla/5.0 (compatible; Zimbiobot/2.0; support@zimbio.com)";
const char S1361[] = "GigaBot/2.0";
const char S1362[] = "private test crawler curl-based";
const char S1363[] = "FlightDeckReportsBot/2.0 (http://www.flightdeckreports.com/pages/bot)";
const char S1364[] = "Mozilla/5.0 (compatible; EmailMarketingRobot/2.1; +http://www.emailmarketingrobot.com/webmasters/)";
const char S1365[] = "Screaming Frog SEO Spider/1,60";
const char S1366[] = "Mozilla/5.0 (compatible; SWbot/0.1; +http://app.saidwot.com/bot.html";
const char S1367[] = "Mozilla/5.0 (SpiffySearchBot http://www.spiffysearch.com/bot.php)";
const char S1368[] = "Mozilla/5.0 (compatible; Qualidator.com Bot 0.9;)";
const char S1369[] = "R6_FeedFetcher_(www.radian6.com/crawler)";
const char S1370[] = "FAST Enterprise Crawler 6 used by LexisNexis (lnbot@lexisnexis.com)";
const char S1371[] = "rogerbot/1.0";
const char S1372[] = "360spider(http://webscan.360.cn)";
const char S1373[] = "gsa-crawler (Enterprise; GID-01742;gsatesting@rediffmail.com)";
const char S1374[] = "NikoBot1.0";
const char S1375[] = "kulokobot www.kuloko.com kuloko@backweave.com";
const char S1376[] = "Updownerbot (+http://www.updowner.com/bot)";
const char S1377[] = "Mozilla/5.0 (compatible; archive.org_bot; Archive-It; +http://archive-it.org/files/site-owners.html)";
const char S1378[] = "FTRF: Friendly robot/1.3";
const char S1379[] = "minicrawler/1";
const char S1380[] = "SAE/fetchurl-kw31jl4xol compatible; SiBot; http://www.siteinfor.com/";
const char S1381[] = "Zookabot/2.4;++http://zookabot.com";
const char S1382[] = "InsiteRobot/2011.12.07";
const char S1383[] = "Mozilla/5.0 (compatible; GrapeshotCrawler/2.0; +http://www.grapeshot.co.uk/crawler.php)";
const char S1384[] = "microbot";
const char S1385[] = "LinksCrawler 0.1beta";
const char S1386[] = "Mozilla/5.0 (compatible; MJ12bot/v1.4.1; http://www.majestic12.co.uk/bot.php?+)";
const char S1387[] = "Sogou inst spider/4.0(+http://www.sogou.com/docs/help/webmasters.htm#07";
const char S1388[] = "Mozilla/5.0 (compatible; WASALive-Bot ; http://blog.wasalive.com/wasalive-bots/)";
const char S1389[] = "Mozilla/5.0 (compatible; Semager/1.4c; +http://www.semager.de/blog/semager-bots/)";
const char S1390[] = "Mozilla/5.0 (compatible; discobot/2.0; +http://discoveryengine.com/discobot.html)";
const char S1391[] = "WongBot // http://www.mister-wong.de/search/";
const char S1392[] = "NewsGator/2.0 Bot (http://www.newsgator.com)";
const char S1393[] = "Lijit Crawler (+http://www.lijit.com/robot/crawler)";
const char S1394[] = "Feedjit Favicon Crawler 1.0";
const char S1395[] = "DS/1.5 robots";
const char S1396[] = "Pressarmybot/beta (+http://beta.pressarmy.com/bot)";
const char S1397[] = "Castabot/0.1 (+http://topixtream.com/)";
const char S1398[] = "FAST Enterprise Crawler/5.3.4 (crawler@fast.no)";
const char S1399[] = "scrapybot/1.0";
const char S1400[] = "Baiduspider-cpro";
const char S1401[] = "percbotspider <ops@percolate.com>";
const char S1402[] = "Mozilla/5.0 (compatible; MJ12bot/v1.4.2; http://www.majestic12.co.uk/bot.php?+)";
const char S1403[] = "Mozilla/5.0 (compatible; YioopBot; +http://www.yioop.com/bot.php)";
const char S1404[] = "FAST Enterprise Crawler 6 / Scirus scirus-crawler@fast.no; http://www.scirus.com/srsapp/contactus/";
const char S1405[] = "SurveyBot/2.3 (Whois Source)";
const char S1406[] = "Sogou+web+robot+(+http://www.sogou.com/docs/help/webmasters.htm#07)";
const char S1407[] = "Mozilla/5.0 (compatible; VideoSurf_bot +http://www.videosurf.com/bot.html)";
const char S1408[] = "Mozilla/4.0 (compatible: FDSE robot)";
const char S1409[] = "Swooglebot/2.0. (+http://swoogle.umbc.edu/swooglebot.html)";
const char S1410[] = "Mozilla/5.0 (compatible; Web CEO Online robot)";
const char S1411[] = "SmartAndSimpleWebCrawler/1.3 (https://crawler.dev.java.net)";
const char S1412[] = "Mozilla/5.0 (Compatible; Supybot 0.83.4.1)";
const char S1413[] = "Ruky-Roboter (Version: 1.06, powered by www.ruky.de +http://www.ruky.de/zeige/bot.html)";
const char S1414[] = "Mozilla/4.0 (compatible; focuseekbot)";
const char S1415[] = "Liquida.it-Crawler/1.0 ( crawler@liquida.it +http://www.liquida.it )";
const char S1416[] = "JoobleBot";
const char S1417[] = "Mozilla/5.0 (compatible; Wootbot/1.0; +http://woot.com/)";
const char S1418[] = "TweetedTimes Bot/1.0 (Mozilla/5.0 Compatible, +http://tweetedtimes.com)";
const char S1419[] = "Sogou inst spider/4.0(+http://www.sogou.com/docs/help/webmasters.htm#07)";
const char S1420[] = "Mozilla/5.0 (compatible; UnisterBot; crawler@unister.de)";
const char S1421[] = "intelium_bot";
const char S1422[] = "PagePeeker.com (info: http://pagepeeker.com/robots)";
const char S1423[] = "Mozilla/bintellibot";
const char S1424[] = "DotNetShoutout-Bot(http://dotnetshout.com)";
const char S1425[] = "Mozilla/5.0+(compatible; UptimeRobot/1.0; http://www.uptimerobot.com/)";
const char S1426[] = "ShowyouBot (http://showyou.com/crawler)";
const char S1427[] = "Mozilla/5.0 (compatible; Scribdbot/1.0; +http://www.scribd.com)";
const char S1428[] = "BoardReader/1.0 (http://boardreader.com/info/robots.htm)-CommentCrawler5";
const char S1429[] = "Jbot";
const char S1430[] = "DealOzBot/7.2 (+http://www.dealoz.com/bot.pl)";
const char S1431[] = "OpenSearchServer_Bot";
const char S1432[] = "aranhabot";
const char S1433[] = "InfohubbleBot";
const char S1434[] = "Mozilla/5.0 (compatible; AMZNKAssocBot/4.0)";
const char S1435[] = "Mozilla/5.0 (compatible; lexxbot/1.0; +http://www.artiesoft.com/lexxbot.php)";
const char S1436[] = "Mozilla/5.0 (compatible; ApptusBot/2.0; +http://www.apptus.com/apptusbot)";
const char S1437[] = "Mozilla/5.0 (compatible; AskPeterBot; +http://www.askpeter.info/bot.html)";
const char S1438[] = "Screaming Frog SEO Spider/1.90";
const char S1439[] = "GG PeekBot 1.0 (http://gg.pl)";
const char S1440[] = "checkgzipcompression.com robot";
const char S1441[] = "spider-on-the-fly 20q v4.256 Discover Life www.discoverlife.org";
const char S1442[] = "FAST Enterprise Crawler 6 used by My Company (administrator@example.com)";
const char S1443[] = "Mozilla/5.0 (compatible; Widgetbox; http://www.widgetbox.com/bot.html)";
const char S1444[] = "Kliksafe Search Bot/2.0";
const char S1445[] = "Mozilla/5.0 (compatible; Ralocobot; +http://raloco.com/bot)";
const char S1446[] = "SNBot 0.93 on human users' behalf http://www.shoppingnotes.com/help/webmasterfaq";
const char S1447[] = "WikioFeedBot 1.0 (http://www.wikio.com)";
const char S1448[] = "Mozilla/5.0 (compatible; i4dwebbot)";
const char S1449[] = "emefgebot/BETA (+http://emefge.de/bot.html)";
const char S1450[] = "Mozilla/5.0 (compatible; AskPeterBot; +http://www.askpeter.de/bot.html)";
const char S1451[] = "Mozilla/5.0 (compatible; ltbot/1.3; bot@language-tools.com)";
const char S1452[] = "WeSEE:Search/0.1 (Alpha, http://www.wesee.com/en/support/bot/)";
const char S1453[] = "DomainCrawler/2.0 (info@domaincrawler.com; http://www.domaincrawler.com/bot";
const char S1454[] = "HolmesBot (http://holmes.ge)";
const char S1455[] = "ClickTale bot";
const char S1456[] = "Mozilla/5.0 (compatible; DCPbot/1.2; +http://domains.checkparams.com/)";
const char S1457[] = "Mozilla/5.0 (compatible; OpenfosBot/2.4; +http://www.openfos.com)";
const char S1458[] = "Xerka WebBot v1.0.0 [Transporte]";
const char S1459[] = "Mozilla/5.0 (Kwickbot 1.0; http://hilfe.kwick.de/KwickBot)";
const char S1460[] = "PGBot/0.1";
const char S1461[] = "LinksManager.com_bot";
const char S1462[] = "stc-crawler/0.84";
const char S1463[] = "suma spider";
const char S1464[] = "vspider";
const char S1465[] = "Mozilla/5.0 (compatible; LuminateBot/1.0; +http://www.luminate.com/bot/)";
const char S1466[] = "SEOENGWorldBot/1.0 (+http://www.seoengine.com/seoengbot.htm)";
const char S1467[] = "Mozilla/5.0 (compatible; Satrix Bot/1.0; +dima@tajseed.com)";
const char S1468[] = "Mozilla/5.0 (compatible; YandexSitelinks; Dyatel; +http://yandex.com/bots)";
const char S1469[] = "Sspider_X";
const char S1470[] = "BotLaju v2.0 (http://laju.my/)";
const char S1471[] = "Mozilla/5.0 (compatible; BeetleBot; )";
const char S1472[] = "Mozilla/5.0 (compatible; aiHitBot/1.1; +http://www.aihit.com/)";
const char S1473[] = "GG PeekBot 1.0 ( http://gg.pl/ http://info.gadu-gadu.pl/praca )";
const char S1474[] = "Mozilla/5.0 (compatible; aiHitBot-BP/1.1; +http://www.aihit.com/)";
const char S1475[] = "Mozilla/4.0 (compatible; Mavicanet robot; www.mavicanet.org)";
const char S1476[] = "TinEye/1.1 (http://tineye.com/crawler.html)";
const char S1477[] = "Mozilla/5.0 (MrCarlito-0.1 http://www.mrcarlito.com/spider.html)";
const char S1478[] = "LinkAider (http://linkaider.com/crawler/)";
const char S1479[] = "'citeseerxbot'";
const char S1480[] = "ConveraCrawler/0.9e (+http://ews.converasearch.com/crawl.htm)";
const char S1481[] = "aulinkbot/1.0";
const char S1482[] = "SciSpi v.1.beta - science spider (+http://www.abitofscience.com/)";
const char S1483[] = "findlinks/1.1.6-beta1 (+http://wortschatz.uni-leipzig.de/findlinks/)";
const char S1484[] = "Mozilla/5.0 (compatible; OpenfosBot/2.1; +http://www.openfos.com)";
const char S1485[] = "msnbot-Products/1.0 (+http://search.msn.com/msnbot.htm)";
const char S1486[] = "A .NET Web Crawler";
const char S1487[] = "ALeadSoftbot/2.0 (+http://www.aleadsoft.com/bot.htm)";
const char S1488[] = "ERACrawler/1.0";
const char S1489[] = "msnbot-media/1.0 (+http://search.msn.com/msnbot.htm)";
const char S1490[] = "QrobotW";
const char S1491[] = "BoardReader/1.0 (http://boardreader.com/info/robots.htm)";
const char S1492[] = "utmailspider";
const char S1493[] = "my-search-bot";
const char S1494[] = "OpenLink Virtuoso RDF crawler";
const char S1495[] = "Baiduspider-favo+(+http://www.baidu.com/search/spider.htm)";
const char S1496[] = "SWAT Crawler. AGH University project. In case of problem contact: opal@tempus.metal.agh.edu.pl. Thanks.";
const char S1497[] = "MetaTagRobot/2.1 (http://www.seocentro.com/tools/search-engines/metatag-analyzer.html)";
const char S1498[] = "Influencebot/0.9; (Automatic classification of websites; http://www.influencebox.com/; info@influencebox.com)";
const char S1499[] = "fastbot.de crawler 2.0 beta (http://www.fastbot.de)";
const char S1500[] = "SocSciBot";
const char S1501[] = "Mozilla/5.0 (compatible; YandexCatalog/3.0; +http://yandex.com/bots)";
const char S1502[] = "webLyzard/1.1 (+http://www.webLyzard.com/crawler)";
const char S1503[] = "shopstylebot/1.0";
const char S1504[] = "RSSingBot (http://www.rssing.com)";
const char S1505[] = "BejiBot Crawler (BNL Services; http://www.bejjan.net/crawler/)";
const char S1506[] = "Startsidenbot/1.0 (+http://www.startsiden.no)";
const char S1507[] = "ServiceUptime.robot";
const char S1508[] = "Baiduspider-sfkr";
const char S1509[] = "Mozilla/5.0 (compatible; ApptusBot/2.0; +http://www.eniro.se)";
const char S1510[] = "RyzeCrawler/1.1.1 (+http://www.domain2day.nl/crawler/)";
const char S1511[] = "businessdbbot/v1.2 (http://www.businessdb.com/bot.php)";
const char S1512[] = "Xerka WebBot v1.0.0 [Tecnotour]";
const char S1513[] = "rbot http util 0.9.15 (http://ruby-rbot.org)";
const char S1514[] = "Traackr.com Bot";
const char S1515[] = "Screaming Frog SEO Spider/1.10";
const char S1516[] = "DomainTaggingbot; +http://www.opendns.com/community/domaintagging";
const char S1517[] = "Doubanbot/1.0 (bot@douban.com http://www.douban.com)";
const char S1518[] = "Robots.txt finder";
const char S1519[] = "sedo_parking_robot";
const char S1520[] = "Mozilla/5.0 (compatible; sharegloo_bot/1.0; +http://www.sharegloo.com)";
const char S1521[] = "WikioFeedBot 1.0";
const char S1522[] = "Mozilla/5.0  (compatible; Wer-liefert-was Crawler 1.1; http://www.wlw.de/extern/crawler/Wer-liefert-was-Crawler.html)";
const char S1523[] = "SearchBot";
const char S1524[] = "SocialOomphLinkBot/1.0 (info@socialoomph.com http://socialoomph.com/)";
const char S1525[] = "cis455crawler";
const char S1526[] = "Scanmine newsspider. See http://www.scanmine.com for more information. NB: We need css files!";
const char S1527[] = "your-search-bot";
const char S1528[] = "Mozilla/5.0+(compatible;+OkidokiBot/0.1;++http://www.okidoki.ro/bot.html)";
const char S1529[] = "Mozilla/5.0 (compatible; YandexWebmaster/2.0; +http://yandex.com/bots)";
const char S1530[] = "sunwukong bot/1.0 (+http://www.swkong.com/addurl)";
const char S1531[] = "MetaTagRobot/1.9 (http://www.widexl.com/remote/search-engines/metatag-analyzer.html)";
const char S1532[] = "Mozilla/5.0 (compatible; YandexBlogs/0.99; robot; B; +http://yandex.com/bots) 2 readers";
const char S1533[] = "Mozilla/5.0 (compatible; BuzzRankingBot/1.0; +http://www.buzzrankingbot.com/)";
const char S1534[] = "My Crawler";
const char S1535[] = "FAST Enterprise Crawler/6.7.0 used by ThomasNet.com (admin@thomasnet.com)";
const char S1536[] = "Mozilla/4.75 (Nikto/2.1.4) (Evasions:None) (Test:robots)";
const char S1537[] = "OdklBot/1.0 (klass@odnoklassniki.ru)";
const char S1538[] = "SEOENGBot/1.2 (+http://www.seoengine.com/seoengbot.htm)";
const char S1539[] = "eduCrawler";
const char S1540[] = "KeepRightBot/0.2 (KeepRight OpenStreetMap Checker; http://keepright.ipax.at)";
const char S1541[] = "Update Profile Bot/1.0 (compatible; NZS.com mini-spider,  Local Search New Zealand; http://www.nzs.com/; bot at nzs.com)";
const char S1542[] = "Mozilla/4.0 (compatible; merlinkbot/1.0)";
const char S1543[] = "Aboundex/0.2 (http://www.aboundex.com/crawler/)";
const char S1544[] = "Pastec bot";
const char S1545[] = "Mozilla/5.0 (compatible; semvisuBot 2.0; +http://fr.semvisu.com/bot)";
const char S1546[] = "Mozilla/5.0 (compatible; jobs.de-Robot +http://www.jobs.de/robot.html)";
const char S1547[] = "BuiBui-Bot/1.0 (email: buibui[at]dadapro[dot]com)";
const char S1548[] = "AdBot 1.0 (www.adnetik.com)";
const char S1549[] = "FreeWebMonitoring SiteChecker/0.2 (+http://www.freewebmonitoring.com/bot.html)";
const char S1550[] = "Mozilla/5.0 (compatible; geourl/3.1 - http://geourl.org/bot)";
const char S1551[] = "Whoismindbot/1.0 (+http://www.whoismind.com/bot.html)";
const char S1552[] = "x28-job-bot; +http://x28.ch/bot.html";
const char S1553[] = "Mozilla/5.0 (compatible; special_archiver/3.1.1 +http://www.archive.org/details/archive.org_bot)";
const char S1554[] = "Mozilla/5.0 (compatible; metager2-verification-bot; +http://metager2.de/technology.php)";
const char S1555[] = "100k_bot";
const char S1556[] = "Statastico DK bot - Learn more at: statastico.dk/bot";
const char S1557[] = "SensikaBot/x.33 (+http://sensika.com)";
const char S1558[] = "xabaducrawler/0.9";
const char S1559[] = "Mozilla/5.0 (X11; U; Linux x86_64; c) AppleWebKit/531.2+ (KHTML, like Gecko) Version/5.0 WatchMouse/$Revision: 24072 $";
const char S1560[] = "Mozilla/5.0 (compatible; Mail.RU/2.0)";
const char S1561[] = "Mozilla/5.0 (compatible; Embedly/0.2; snap; +http://support.embed.ly/)";
const char S1562[] = "Mozilla/5.0 (compatible; Imperialbot/1.2; +http://http://www.imperialdirectory.com/ContactUs)";
const char S1563[] = "Mozilla/5.0 (PHP 5) PPBBCrawler 0.1";
const char S1564[] = "Mozilla/5.0 (compatible; news bot /2.1)";
const char S1565[] = "Mozilla/5.0 (compatible; CareerBot/1.1; +http://www.career-x.de/bot.html)";
const char S1566[] = "RSSInclude/1.0 (http://www.rssinclude.com/spyder)";
const char S1567[] = "adidxbot/2.0 (+http://search.msn.com/msnbot.htm)";
const char S1568[] = "Mozilla/5.0 (compatible; MJ12bot/v1.4.3; http://www.majestic12.co.uk/bot.php?+)";
const char S1569[] = "Explorer Bot";
const char S1570[] = "FocusedCrawler (http://www2.informatik.hu-berlin.de/~brettsch/focusedcrawler/)";
const char S1571[] = "Cutbot; 1.5; http://cutbot.net/";
const char S1572[] = "findlinks/2.6 (+http://wortschatz.uni-leipzig.de/findlinks/)";
const char S1573[] = "MoodleBot/1.0";
const char S1574[] = "Whitevector Crawler (+http://www.whitevector.com/crawler)";
const char S1575[] = "Phantom.js bot";
const char S1576[] = "taptubot; requests from this IP address are taptubot, see http://taptu.com/corp/taptubot";
const char S1577[] = "Mozilla/5.0 (compatible; archive.org_bot/3.1.1 +http://www.archive.org/details/archive.org_bot)";
const char S1578[] = "Curious George - www.analyticsseo.com/crawler";
const char S1579[] = "AdMedia bot";
const char S1580[] = "BuiBui-CheckBot/1.0 (email: buibui[at]dadapro[dot]com)";
const char S1581[] = "PopScreenBot";
const char S1582[] = "MeMoNewsBot/2.0 (http://www.memonews.com/en/crawler)";
const char S1583[] = "AC-BaiduBot/1.1(+http://www.ac-baidu.com)";
const char S1584[] = "Mozilla/5.0 (compatible; YandexBot/3.0; MirrorDetector; +http://yandex.com/bots)";
const char S1585[] = "Mozilla/5.0 (compatible; FriendFeedBot/0.1; +Http://friendfeed.com/about/bot; 3 subscribers; feed-id=X)";
const char S1586[] = "Healthbot/Health_and_Longevity_Project_(HealthHaven.com)";
const char S1587[] = "Mozilla/5.0+(compatible;+Baiduspider/2.0;++http://www.baidu.com/search/spider.html)";
const char S1588[] = "QippoBot/1.0 (+http://www.qippo.com/bot/)";
const char S1589[] = "Mozilla/5.0 (compatible; YandexNewslinks; +http://yandex.com/bots)";
const char S1590[] = "Mozilla/5.0 (compatible; YandexBlogs/0.99; robot; B; +http://yandex.com/bots) 0 readers";
const char S1591[] = "SiteCrawler";
const char S1592[] = "dirbot/1.0";
const char S1593[] = "Mozilla/5.0 (compatible; Sosoimagespider/2.0; +http://help.soso.com/soso-image-spider.htm)";
const char S1594[] = "i-market-bot";
const char S1595[] = "Abrave Spider v5.3 Robot 1 (http://robot.abrave.com)";
const char S1596[] = "DiBot Java/1.5.0_04";
const char S1597[] = "EasyDL/3.04  http://keywen.com/Encyclopedia/Bot";
const char S1598[] = "betaBot";
const char S1599[] = "Mozilla/5.0 (compatible; archive.bibalex.org_bot; +http://archive.bibalex.org/bot/)";
const char S1600[] = "TextTrust Crawler";
const char S1601[] = "Mozilla/5.0 (compatible; Nesotebot/2.1; +http://www.inoutscripts.com/bot.html)";
const char S1602[] = "Mozilla/5.0 (compatible;wi jobroboter spider 4 for ams;+http://www.webintegration.at)";
const char S1603[] = "Industrial Interface Web Crawler - http://www.industrycortex.com/crawler.php";
const char S1604[] = "GermCrawler";
const char S1605[] = "COIBot/2.0";
const char S1606[] = "Mozilla/4.0 (Toread-Crawler/1.1; +http://news.toread.cc/crawler.php)";
const char S1607[] = "Mozilla/5.0 QunarBot/1.0";
const char S1608[] = "Mozilla/5.0 (compatible; Saralbot/4.2; +http://www.saralweb.com/cache/public/pages/bot)";
const char S1609[] = "BaiduImagespider+(+http://www.baidu.jp/spider/)";
const char S1610[] = "Mozilla/5.0 (compatible; DotBot/1.1; http://www.dotnetdotcom.org/, crawler@dotnetdotcom.org)";
const char S1611[] = "LexxeBot/1.0 (lexxebot@lexxe.com)";
const char S1612[] = "Mozilla/5.0 (compatible; Exabot/3.0 (BiggerBetter); +http://www.exabot.com/go/robot)";
const char S1613[] = "SheenBot/SheenBot-1.0.4 (Sheen web crawler)";
const char S1614[] = "msnbot/2.0b (+http://search.msn.com/msnbot.htm) Test";
const char S1615[] = "Mozilla/5.0 (compatible; AboutUsBot Johnny5/2.0; +http://www.AboutUs.org/)";
const char S1616[] = "wf_crawler (http://www.websitefigures.com)";
const char S1617[] = "msnbot/1.0 (+http://search.msn.com/msnbot.htm)";
const char S1618[] = "Linguee Bot (http://www.linguee.com/bot; bot@linguee.com)";
const char S1619[] = "Mozilla/5.0 (compatible; SummizeBot +http://www.summize.com)";
const char S1620[] = "Mozilla/5.0 (compatible; Baiduspider/2.0; +http://www.baidu.com/search/spider.html) (via babelfish.yahoo.com)";
const char S1621[] = "Mozilla/5.0 (compatible; Baiduspider/2.0; +http://www.baidu.com/search/spider.html) AppEngine-Google; (+http://code.google.com/appengine; appid: surf6009)";
const char S1622[] = "webmeasurement-bot, http://rvs.informatik.uni-leipzig.de/bot.php";
const char S1623[] = "Mozilla/5.0 (compatible; Herold Crawler; +http://www.herold.at)";
const char S1624[] = "Mozilla/5.0 (compatible; http://www.UptimeAuditor.com/bot/ )";
const char S1625[] = "WebRobot";
const char S1626[] = "Mozilla/5.0 (compatible; SecretSerachEngineLabs.com-SBSearch/0.9; http://www.secretsearchenginelabs.com/secret-web-crawler.php)";
const char S1627[] = "JavaCrawler/1.1";
const char S1628[] = "PTech/1.0 (crawler@positiontech.com; http://www.positiontech.com/)";
const char S1629[] = "Jayde crawler";
const char S1630[] = "Mozilla/5.0 (compatible; dlcbot/0.1; +http://www.drlinkcheck.com/)";
const char S1631[] = "pd-crawler (pdsupport@purediscovery.com)";
const char S1632[] = "Screaming Frog SEO Spider/1,80";
const char S1633[] = "Screaming Frog SEO Spider/1,90";
const char S1634[] = "Crawler0.1";
const char S1635[] = "rapid-miner-crawler";
const char S1636[] = "Abrave Spider v5.3 Robot 2 (http://robot.abrave.com)";
const char S1637[] = "CodeGator Crawler v1.0";
const char S1638[] = "Xenu Link Sleuth/1.3.6";
const char S1639[] = "gsa-crawler (Enterprise; GID-01422; jplastiras@google.com)";
const char S1640[] = "naverbookmarkcrawler";
const char S1641[] = "ldspider (http://code.google.com/p/ldspider/wiki/Robots)";
const char S1642[] = "netEstate Trend Crawler (+http://www.trendmonitor.org/)";
const char S1643[] = "Mozilla/3.0 (INGRID/3.0 MT; webcrawler@NOSPAMexperimental.net; http://aanmelden.ilse.nl/?aanmeld_mode=webhints)";
const char S1644[] = "bot@1web.ro/1.1 (+http://www.1web.ro/bot.html)";
const char S1645[] = "BilgiBot/1.0(beta) (http://www.bilgi.com/; bilgi at bilgi dot com)";
const char S1646[] = "Screaming Frog SEO Spider/1,40";
const char S1647[] = "Mozilla/4.0 (compatible; ChristCrawler.com ChristCrawler@ChristCENTRAL.com)";
const char S1648[] = "Mozilla/5.0 (compatible; PWeBot/3.1; http://www.programacionweb.net/robot.php)";
const char S1649[] = "r2iBot/1.0 http://www.r2integrated.com";
const char S1650[] = "Mozilla/5.0 (compatible; HomeTags/1.0; +http://www.hometags.nl/bot)";
const char S1651[] = "Site-Perf.com performance testing bot";
const char S1652[] = "websphinx.Crawler";
const char S1653[] = "Mozilla/5.0 (compatible; CrawlBot/1.0.0)";
const char S1654[] = "WebZIP/7.0 (http://www.spidersoft.com)";
const char S1655[] = "Test Webbot";
const char S1656[] = "squirrobot/1.0 (http://intro.squirro.com/squirrobot/)";
const char S1657[] = "Mozilla/5.0 (compatible; 008/0.83; http://www.80legs.com/spider.html;) Gecko/X";
const char S1658[] = "Mozilla 4.0(compatible; BotSeer/1.0; +http://botseer.ist.psu.edu)";
const char S1659[] = "bot/1.0";
const char S1660[] = "Mozilla/5.0 (compatible; SinaaBot/1.0; +http://www.marketdefender.com/bot.html)";
const char S1661[] = "Mozilla/5.0 (compatible; spbot/2.0.4; +http://www.seoprofiler.com/bot )";
const char S1662[] = "SiteCrawler/1.1.3";
const char S1663[] = "Mozilla/5.0 (compatible; alexa verifybot/1.0; +http://www.alexa.com/help; help@alexa.com)";
const char S1664[] = "Mozilla/4.0 (Mozilla; http://www.mozilla.org/docs/en/bot.html; master@mozilla.com)";
const char S1665[] = "Mozilla/5.0 (compatible; BlogRefsBot/0.1; http://www.blogrefs.com/about/bloggers)";
const char S1666[] = "msnbot";
const char S1667[] = "Mozilla/5.0 (compatible; Peew/1.0; http://www.peew.de/crawler/)";
const char S1668[] = "SuperPagesBot/0.1";
const char S1669[] = "Mozilla/5.0 (compatible; bixolabs/1.0; +http://bixolabs.com/crawler/general; crawler@bixolabs.com)";
const char S1670[] = "rogerbot/1.0 (http://www.seomoz.org, rogerbot-crawler@seomoz.org)";
const char S1671[] = "MIA DEV/search:robot/0.0.1 (This is the MIA Bot - crawling for mia research project. If you feel unhappy and do not want to be visited by our crawler send an email to spider@neofonie.de; http://spider.neofonie.de; spider@neofonie.de)";
const char S1672[] = "Mozilla/5.0 (compatible; Finderbots finder bot; +http://wiki.github.com/bixo/bixo/bixocrawler; bixo-dev@yahoogroups.com)";
const char S1673[] = "QroboDomainBot";
const char S1674[] = "WocBot/Mozilla/5.0 (Wocodi Web Crawler 1.0; http://www.wocodi.com/crawler; crawler@wocodi.com)";
const char S1675[] = "Y!J-BSC/1.0 crawler (http://help.yahoo.co.jp/help/jp/blog-search/)";
const char S1676[] = "Mozilla/5.0 (compatible; imrbot/1.01.18 +http://www.internetmemory.org)";
const char S1677[] = "ZillaCrawler";
const char S1678[] = "Mozilla/5.0 (compatible; newslebot/1.0; +http://newsle.com)";
const char S1679[] = "QroBot";
const char S1680[] = "GrepNetstat.com Bot/1.0; +http://www.grepnetstat.com)";
const char S1681[] = "123pagesbot/1.0";
const char S1682[] = "Mozilla/5.0 (compatible; archive.org_bot/3.1.2 +http://www.bne.es/es/LaBNE/PreservacionDominioES/AvisoWebmasters/index.html)";
const char S1683[] = "Mozilla/5.0 (compatible; GrepNetstatBot/1.0; +http://www.grepnetstat.com/bot)";
const char S1684[] = "Screaming Frog SEO Spider/2.00";
const char S1685[] = "wsAnalyzer/1.0; ++http://www.wsanalyzer.com/bot.html";
const char S1686[] = "Wotbox/2.01 (+http://www.wotbox.com/bot/)";
const char S1687[] = "rbbot";
const char S1688[] = "Mozilla/5.0 (compatible; DomainVader/1.0; +http://domainvader.com/bot/info.php)";
const char S1689[] = "www.integromedb.org/Crawler";
const char S1690[] = "Mozilla/5.0 (compatible; ProCogBot/1.0; +http://www.procog.com/spider.html)";
const char S1691[] = "atwatch-bot";
const char S1692[] = "brainbot/1.0 (digitalbrain@brainstreammedia @digitalbrain)";
const char S1693[] = "Mozilla/5.0 (compatible; ClearSpike_bot/1.0; +http://www.clearspike.com/bot.html)";
const char S1694[] = "spider/1.0";
const char S1695[] = "woopingbot/1.1";
const char S1696[] = "Mozilla/5.0 (compatible; SearchmetricsBot; http://www.searchmetrics.com/en/searchmetrics-bot/)";
const char S1697[] = "Abrave Spider v5.3 Robot 1 (http://robot.abrave.co.uk)";
const char S1698[] = "Mozilla/5.0 (compatible; linkfluence; http://linkfluence.net/; bot@linkfluence.net)";
const char S1699[] = "Screaming Frog SEO Spider/2.01";
const char S1700[] = "Screaming Frog SEO Spider/2,01";
const char S1701[] = "Mozilla/5.0 (compatible; WebNL; +http://www.web.nl/webmasters/spider.html; helpdesk@web.nl)";
const char S1702[] = "riight.com/RIIGHT-1.1 (crawler; http://riight.com/spider; spider@riight.com)";
const char S1703[] = "Mozilla/5.0 (compatible; proximic; +http://www.proximic.com/info/spider.php)";
const char S1704[] = "Mozilla/5.0 (compatible; SIBot/1.0; +http://www.profound.net/sibot.html)";
const char S1705[] = "OSS-bot/0.02 (see http://michaelnielsen.org/blog/oss-bot/ or contact Michael Nielsen, mn@michaelnielsen.org)";
const char S1706[] = "MOMspider/1.00 libwww-perl/0.40";
const char S1707[] = "ilocal company search (+http://www.ilocal.nl/about/crawler.jsp)";
const char S1708[] = "Mozilla/5.0 (Yahoo-MMCrawler/4.0; mailto:vertical-crawl-support@yahoo-inc.com)";
const char S1709[] = "nb-bot";
const char S1710[] = "netEstate NE Crawler (+http://www.website-datenbank.de/)";
const char S1711[] = "Mozilla/5.0 (compatible; YandexBlogs/0.99; robot; B; +http://yandex.com/bots)";
const char S1712[] = "Screaming Frog SEO Spider/1.30";
const char S1713[] = "Mozilla/5.0 (compatible; Zenbot/1.3; +http://zen.co.za/webmasters/)";
const char S1714[] = "Open Web Analytics Bot 1.4.1";
const char S1715[] = "Uptime Robot";
const char S1716[] = "Mozilla/5.0 (compatible; BecomeBot/1.23; http://www.become.com/webmasters.html)";
const char S1717[] = "SAS Web Crawler";
const char S1718[] = "FAST Search Web Crawler 14.0.0325.0000";
const char S1719[] = "Mozilla/5.0 (compatible; Zenbot/2.6.1; +http://zen.co.za/webmasters/)";
const char S1720[] = "User-Agent: Lijit Crawler (+http://www.lijit.com/robot/crawler)";
const char S1721[] = "Mozilla/5.0 (compatible; Proximic crawler; +http://www.proximic.com/en/about-us/contact-us.html)";
const char S1722[] = "Mozilla/5.0 (compatible; ShunixBot/1.x; http://www.shunix.com/bot.htm)";
const char S1723[] = "Mozilla/5.0 [en] (compatible; Gulper Web Bot 0.2.4 www.ecsl.cs.sunysb.edu/~maxim/cgi-bin/Link/GulperBot)";
const char S1724[] = "Sosospider+(+help.soso.com/webspider.htm)";
const char S1725[] = "Mozilla/5.0 (compatible; Bot; +http://pressemitteilung.ws/spamfilter";
const char S1726[] = "Mozilla/5.0 (compatible; archive.org_bot/1.10.0 +http://www.loc.gov/minerva/crawl.html)";
const char S1727[] = "Mozilla/5.0 (compatible; archive.org_bot/1.5.0-X http://crawler.archive.org) Hurricane Katrina";
const char S1728[] = "Mozilla/4.0 (compatible; GPU p2p crawler http://gpu.sourceforge.net/search_engine.php)";
const char S1729[] = "Mozilla/5.0 (compatible; Synoobot/0.9; http://www.synoo.com/search/bot.html)";
const char S1730[] = "Mozilla/5.0 (compatible; Charlotte/1.0b; charlotte@betaspider.com)";
const char S1731[] = "Mozilla/4.7 (compatible; WhizBang; http://www.whizbang.com/crawler)";
const char S1732[] = "YakazBot/1.0";
const char S1733[] = "Mozilla/5.0 (Clustered-Search-Bot/1.0; support@clush.com; http://www.clush.com/)";
const char S1734[] = "Mozilla/5.0 (compatible; AnsearchBot/1.x; +http://www.ansearch.com.au/)";
const char S1735[] = "Mozilla/3.0 (compatible; ScollSpider; http://www.webwobot.com)";
const char S1736[] = "Mozilla/5.0 (compatible; robtexbot/1.0; http://www.robtex.com/ )";
const char S1737[] = "Mozilla/5.0 (compatible; Galbot/1.0; +http://www.galbot.com/bot.html)";
const char S1738[] = "Mozilla/5.0 (compatible; SkreemRBot +http://skreemr.com)";
const char S1739[] = "Mozilla/3.0 (Vagabondo/2.0 MT; webcrawler@NOSPAMwise-guys.nl; http://webagent.wise-guys.nl/)";
const char S1740[] = "Mozilla/4.0 compatible FurlBot/Furl Search 2.0 (FurlBot; http://www.furl.net; wn.furlbot@looksmart.net)";
const char S1741[] = "Mozilla/3.0 (compatible; Fluffy the spider; http://www.searchhippo.com/; info@searchhippo.com)";
const char S1742[] = "Mozilla/5.0 (compatible;archive.org_bot/1.7.1; collectionId=316; Archive-It; +http://www.archive-it.org)";
const char S1743[] = "Mozilla/5.0 (compatible; ShunixBot/1.x.x +http://www.shunix.com/robot.htm)";
const char S1744[] = "Mozilla/4.7 (compatible; http://eidetica.com/spider)";
const char S1745[] = "Mozilla/5.0 (compatible; Webduniabot/1.0; +http://search.webdunia.com/bot.aspx)";
const char S1746[] = "Mozilla/5.0 (compatible; archive.org_bot/1.13.1x http://crawler.archive.org)";
const char S1747[] = "Mozilla/5.0 (compatible;FindITAnswersbot/1.0;+http://search.it-influentials.com/bot.htm)";
const char S1748[] = "Mozilla/5.0 (compatible; Vagabondo/2.1; webcrawler at wise-guys dot nl; http://webagent.wise-guys.nl/)";
const char S1749[] = "Mozilla/5.0 (compatible; Diffbot/0.1; +http://www.diffbot.com)";
const char S1750[] = "Mozilla/4.0 (compatible; FastCrawler3 support-fastcrawler3@fast.no)";
const char S1751[] = "Mozilla/4.7 [en](BecomeBot@exava.com)";
const char S1752[] = "Mozilla/3.0 (Vagabondo/2.0 MT; webcrawler@NOSPAMexperimental.net; http://aanmelden.ilse.nl/?aanmeld_mode=webhints)";
const char S1753[] = "Mozilla/5.0 (compatible; SpurlBot/0.2)";
const char S1754[] = "Mozilla/5.0 (compatible; http://www.IsMySiteUp.Net/bot/ )";
const char S1755[] = "Mozilla/5.0 (compatible; MojeekBot/2.0; http://www.mojeek.com/bot.html)";
const char S1756[] = "Mozilla/4.04 (compatible; Dulance bot; +http://www.dulance.com/bot.jsp)";
const char S1757[] = "Mozilla/5.0 (compatible; BecomeBot/2.0beta; http://www.become.com/webmasters.html)";
const char S1758[] = "Mozilla/5.0 (compatible; XTbot/1.0v; +http://www.externaltest.com)";
const char S1759[] = "Mozilla/5.0+(compatible;+Baiduspider/2.0;++http://www.baidu.com/search/spider.html";
const char S1760[] = "TppRFbot/1.1 (compatible; http://www.ruschamber.net, http://www.geocci.com)";
const char S1761[] = "NAVER Blog Rssbot";
const char S1762[] = "Mozilla/5.0 (compatible; Quantcastbot/1.0; www.quantcast.com)";
const char S1763[] = "Wepbot/3.0 (+http://www.nurago.de/wepbot/); Jakarta Commons-HttpClient/3.1";
const char S1764[] = "PrestaShopBot/1.1";
const char S1765[] = "Mozilla/5.0 (compatible; 3W1bot/2.6; +http://3w1.eu/bot)";
const char S1766[] = "Zing-BottaBot/2.0";
const char S1767[] = "Mozilla/5.0 (compatible; Baiduspider-cpro; +http://www.baidu.com/search/spider.html)";
const char S1768[] = "WebCompanyCrawler";
const char S1769[] = "bl.uk_lddc_bot/3.1.1 (+http://www.bl.uk/aboutus/stratpolprog/digi/domresproj/index.html)";
const char S1770[] = "bot-pge.chlooe.com/1.0.0 (+http://www.chlooe.com/)";
const char S1771[] = "Zookabot/2.5;++http://zookabot.com";
const char S1772[] = "coccoc/1.0 (http://help.coccoc.vn/)";
const char S1773[] = "Scope PreviewBot/1.3 (+http://www.getscopeapp.com)";
const char S1774[] = "Mozilla/5.0 (compatible; Infohelfer/1.3.0; +http://www.infohelfer.de/crawler.php)";
const char S1775[] = "Mozilla/5.0(compatible; Sosospider/2.0; +http://help.soso.com/webspider.htm)";
const char S1776[] = "Sogou web spider/4.0";
const char S1777[] = "LSSRocketCrawler/1.0 LightspeedSystems";
const char S1778[] = "Mozilla/5.0 (compatible; SemrushBot/0.95; +http://www.semrush.com/bot.html)";
const char S1779[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler@seomoz.org)";
const char S1780[] = "QuickFinder Crawler";
const char S1781[] = "Baiduspider+(+http://www.baidu.com/search/spider_jp.html)";
const char S1782[] = "Linkbot";
const char S1783[] = "FAST-WebCrawler/3.6 (atw-crawler at fast dot no; http://fast.no/support/crawler.asp)";
const char S1784[] = "aipbot/1.0 (aipbot; http://www.aipbot.com; aipbot@aipbot.com)";
const char S1785[] = "WinistaBot/1.6.7-dev (Winista - Crwaler for HTTP content indexing; http://www.netomatix.com/Winista/bot.html; support@netomatix.com)";
const char S1786[] = "Mozilla/4.0 (compatible; Vagabondo/2.2; webcrawler at wise-guys dot nl; http://webagent.wise-guys.nl/)";
const char S1787[] = "Mozilla/5.0 (compatible; IDBot/1.0; +http://www.id-search.org/bot.html)";
const char S1788[] = "Mozilla/5.0 (compatible; PagestackerBot; http://www.pagestacker.com)";
const char S1789[] = "Mozilla/5.0 (compatible; WGToolsBot/1.0; http://wgtools.com/link-checker-bot/)";
const char S1790[] = "Mozilla/4.0 (compatible; Vagabondo/4.0Beta; webcrawler at wise-guys dot nl; http://webagent.wise-guys.nl/)";
const char S1791[] = "Mozilla/5.0 (compatible; Baiduspider/2.0; +http://www.baidu.com/search/spider.html) AppEngine-Google; (+http://code.google.com/appengine; appid: proxy6000)";
const char S1792[] = "ProductAdsBot/1.0";
const char S1793[] = "Mozilla/5.0 (compatible; BanBots/2.0b; Fetch; +http://www.banbots.com)";
const char S1794[] = "Mozilla/5.0 (compatible; Tbot/1.0;)";
const char S1795[] = "Mozilla/5.0 (compatible; DonkeyBot/0.1; +http://www.dasdonkey.com)";
const char S1796[] = "Junut Bot 1.0.2";
const char S1797[] = "Screaming Frog SEO Spider/2,00";
const char S1798[] = "Mozilla/5.0 (compatible; WebCeoOnlineBot/1.0;)";
const char S1799[] = "NextGenSearchBot 69 (for information visit http://www.zoominfo.com/business/nextgensearchbot.html)";
const char S1800[] = "gsa-crawler (Enterprise; GID-01422; pabloeinstaincordeiro.com)";
const char S1801[] = "CrawlerProcess (http://www.PowerMapper.com) /4.7.566.0";
const char S1802[] = "Open Web Analytics Bot 1.5.4";
const char S1803[] = "Mozilla/3.0 (Vagabondo/1.1 MT; webcrawler@NOSPAMwise-guys.nl; http://webagent.wise-guys.nl/)";
const char S1804[] = "Mozilla/5.0 (compatible; Peepowbot/1.0; +http://www.peepow.com/bot.php)";
const char S1805[] = "GarlikCrawler/1.2 (http://garlik.com/, crawler@garlik.com)";
const char S1806[] = "Mozilla/5.0 (compatible; NewsCatchBot\\/1.1; +http://mrnet.in)";
const char S1807[] = "item-db bot/1.0(http://item-db.net/)";
const char S1808[] = "Mozilla/5.0 (compatible; bixolabs; +crawler@scaleunlimited.com; crawler@scaleunlimited.com)";
const char S1809[] = "omgilibot/0.4 +http://chatterfinder.com +http://omgili.com";
const char S1810[] = "Mozilla/5.0 (compatible; DripfeedBot/2.0; +http://dripfeedbookmark.com/bot.html";
const char S1811[] = "bot.wsowner.com/1.0.0 (+http://wsowner.com/)";
const char S1812[] = "Mozilla/5.0 (compatible; CompSpyBot/1.0; +http://www.compspy.com/spider.html)";
const char S1813[] = "Mozilla/5.0 (compatible; TweetmemeBot/3.0; +http://tweetmeme.com/)";
const char S1814[] = "Marketing Grader Crawler 1.0 (http://marketing.grader.com/)";
const char S1815[] = "peerindex/0.1 (http://www.peerindex.com/; crawler AT peerindex DOT com)";
const char S1816[] = "SolomonoBot/1.03 (http://www.solomono.ru)";
const char S1817[] = "VX_Pingdom_Bot";
const char S1818[] = "First Search Engine Dedicated The Adult World - Bot version ( 12.09.13 )";
const char S1819[] = "Screaming Frog SEO Spider/2.03";
const char S1820[] = "Screaming Frog SEO Spider/2,03";
const char S1821[] = "Mozilla/5.0 (compatible; 200PleaseBot/1.0; +http://www.200please.com/bot)";
const char S1822[] = "PercolateCrawler/3.1.30 (ops@percolate.com)";
const char S1823[] = "FAST Search Web Crawler 14.0.0291.0000";
const char S1824[] = "A1 Website Download/3.0.3 (+http://www.microsystools.com/products/website-download/) miggibot";
const char S1825[] = "Arroba MailRobot/2.0 (compatible; Mozilla/5.0; By cristobal.alcayaga@gmail.com;)";
const char S1826[] = "Mozilla/5.0 (compatible; FatBot 2.0; http://www.thefind.com/main/CrawlerFAQs.fhtml)";
const char S1827[] = "Mozilla/5.0 (X11; U; Linux x86_64; c) AppleWebKit/531.2+ (KHTML, like Gecko) Version/5.0 WatchMouse/$Revision: 24640 $";
const char S1828[] = "WikioBot 1.0";
const char S1829[] = "LinkBackBot/1.0; +http://www.linkback.co/bot.html - Automated Reciprocal LinkBack directory";
const char S1830[] = "ZCrawler Alpha (+http://www.zazzle.com/)";
const char S1831[] = "QippoBot/1.4 (+http://www.qippo.com/bot/)";
const char S1832[] = "emyuu bot/1.0(http://emyuu.com/)";
const char S1833[] = "PagesInventory.com (robot http://www.pagesinvenotry.com)";
const char S1834[] = "Mozilla/5.0 (compatible; Cliqusbot/0.1)";
const char S1835[] = "Baiduspider (+http://www.baidu.com/search/spider.htm)";
const char S1836[] = "NoyaBot";
const char S1837[] = "MIA DEV/search:robot/0.0.1 (MIA Bot for research project MIA (www.MIA-marktplatz.de); http://spider.neofonie.de; spider@neofonie.de)";
const char S1838[] = "MIA DEV/search:robot/0.0.1 (MIA Bot for research project MIA (www.MIA-marktplatz.de); http://spider.neofonie.de; spider-l@neofonie.de)";
const char S1839[] = "UnlimitBot(ulmt.in)";
const char S1840[] = "PagesInventory (robot http://www.pagesinvenotry.com)";
const char S1841[] = "Mozilla/5.0 (compatible; spbot/3.1; +http://www.seoprofiler.com/bot )";
const char S1842[] = "Mozilla/5.0 (compatible; Mail.RU_Bot/2.0)";
const char S1843[] = "HubSpot Links Crawler 1.0 http://www.hubspot.com/";
const char S1844[] = "Mozilla/5.0 (compatible; IstellaBot/1.10.2 +http://www.tiscali.it/)";
const char S1845[] = "SomonesBot 1.0; +http://www.somones.com/";
const char S1846[] = "Mozilla/5.0 (compatible; archive.org_bot/3.1.2 +http://www.eotarchive.cdlib.org/)";
const char S1847[] = "Mozilla/5.0 (compatible; LinkBackBot/1.0; +http://www.linkback.co/bot.html - Automated Reciprocal LinkBack directory)";
const char S1848[] = "Mozilla/5.0 (compatible; MFGPagesBot/2.1; +http://www.mfgpages.com)";
const char S1849[] = "Mozilla/5.0 (compatible; Statsbot/2.1;)";
const char S1850[] = "Mozilla/5.0 (compatible; ratup bot 1.4)";
const char S1851[] = "Mozilla/5.0 (compatible; Tourlentabot/0.55; +http://tourlenta.com)";
const char S1852[] = "PagesInventory (robot http://www.pagesinventory.com)";
const char S1853[] = "Smarte Bot v1.0";
const char S1854[] = "Diabot/1.0";
const char S1855[] = "AlestiFeedBot/0.7 (http://www.alesti.org/; 1 subscriber)";
const char S1856[] = "Confirm Domain Bot/1.0 (compatible; NZS.com mini-spider,  Local Search New Zealand; http://www.nzs.com/; bot at nzs.com)";
const char S1857[] = "Baiduspider-video+(+http://www.baidu.com/search/spider.htm)";
const char S1858[] = "Mozilla/5.0 (compatible; YandexBlogs/0.99; robot; B; +http://yandex.com/bots)0 readers";
const char S1859[] = "TestCrawler";
const char S1860[] = "E-Crawler";
const char S1861[] = "Mozilla/5.0 (compatible; ExaBot/3.0; +http://www.exalead.com/go/robot)";
const char S1862[] = "iCorpusBot (http://www.n-grams.org/icorpusbot.html)";
const char S1863[] = "OVGbot";
const char S1864[] = "spiderman";
const char S1865[] = "scoopbot/0.1";
const char S1866[] = "Mozilla/5.0 (compatible; Baiduspider/2.0; +http://www.baidu.com/search/spider.html) AppEngine-Google; (+http://code.google.com/appengine; appid: cgiproxy6)";
const char S1867[] = "WebZIP/7.1 (http://www.spidersoft.com)";
const char S1868[] = "Justdialbot/1.0";
const char S1869[] = "nrron-crawler (pdsupport@purediscovery.com)";
const char S1870[] = "Mozilla/5.0 (compatible; Kraken/0.1; http://linkfluence.net/; bot@linkfluence.net)";
const char S1871[] = "Mozilla/5.0 (compatible; Esribot/1.0; http://www.esrihu.hu/)";
const char S1872[] = "FlightDeckReportsBot/2.0 (http://www.flightdeckreports.com/bot)";
const char S1873[] = "Mozilla/5.0 (compatible; TotientBot 1.0; birrell@microsoft.com)";
const char S1874[] = "Bobot";
const char S1875[] = "JCSearch NetHungry JCSearchbot/1.0 (http://jcsearch.com/nethungry.htm)";
const char S1876[] = "Mozilla/5.0 (compatible; EbozavrBot/1.0; +http://www.ebozavr.com/about_bot.php";
const char S1877[] = "Mozilla/5.0 (compatible; Infohelfer/1.3.3; +http://www.infohelfer.de/crawler.php)";
const char S1878[] = "Mozilla/5.0 (compatible; Alexabot/1.0; +http://www.alexa.com/help/certifyscan; certifyscan@alexa.com)";
const char S1879[] = "Online Chat Crawler";
const char S1880[] = "CMS Identify Crawler/2.0";
const char S1881[] = "FeedBot";
const char S1882[] = "BizwikiBot/0.3 (http://www.bizwiki.com/bizwikibot.htm)";
const char S1883[] = "AppEngine-Google; (+http://code.google.com/appengine; appid: s~stremor-crawler-dev)";
const char S1884[] = "Mozilla/5.0 (compatible; crawler; +; crawler.info.2@gmail.com)";
const char S1885[] = "MSN bot/1.0";
const char S1886[] = "SolomonoBot/1.04 (http://www.solomono.ru)";
const char S1887[] = "pjcrawler";
const char S1888[] = "Backlink Crawler v1.0";
const char S1889[] = "CollarBar logistics research bot v0.26 (collarbar.com)";
const char S1890[] = "Mozilla/5.0 (compatible; 008/0.85; http://www.80legs.com/webcrawler.html) Gecko/X";
const char S1891[] = "Mozzila/5.0 (compatible; YamanaLab-Robot/1.0; http://www.yama.info.waseda.ac.jp/~crawler/info.html)";
const char S1892[] = "Mozilla/5.0 (compatible; Steeler/3.5; http://www.tkl.iis.u-tokyo.ac.jp/~crawler/)";
const char S1893[] = "TasBot (http://www.httpool.com/)";
const char S1894[] = "Mozilla/5.0 (compatible; myAbstractCrawler +http://www.mysite.com)";
const char S1895[] = "Kentico CMS Smart Search Crawler";
const char S1896[] = "Mozilla/5.0 (compatible; MJ12bot/v1.4.3; http://www.majestic12.co.uk/bot.php?+) AppEngine-Google; (+http://code.google.com/appengine; appid: pr0xy--me)";
const char S1897[] = "Mozilla/5.0 (compatible; ProductAdsBot/1.0 +https://sellercentral.amazon.com/gp/help/help.html?ie=UTF8&itemID=X)";
const char S1898[] = "User-Agent:Mozilla/5.0 (SaidwotBot)";
const char S1899[] = "CrawlerProcess (http://www.PowerMapper.com) /5.0.592.0";
const char S1900[] = "Sogou web spider/4.0l";
const char S1901[] = "grokkit-crawler (pdsupport@purediscovery.com)";
const char S1902[] = "AppEngine-Google; (+http://code.google.com/appengine; appid: s~stremor-crawler)";
const char S1903[] = "AltiSEO.com bot v0.1";
const char S1904[] = "Uptimerobot";
const char S1905[] = "Y!J-BRJ/YATS crawler (http://help.yahoo.co.jp/help/jp/search/indexing/indexing-15.html)";
const char S1906[] = "oneQube/1 (http://www.chirpaloo.com/bot.html)";
const char S1907[] = "Mozilla/5.0 (compatible; ratup.com bot 1.8)";
const char S1908[] = "PopScreen Bot";
const char S1909[] = "stremorbot AppEngine-Google; (+http://code.google.com/appengine; appid: s~stremor-crawler)";
const char S1910[] = "Reportl crawler (http://reportl.com/crawler.txt)";
const char S1911[] = "Sogou web spider/4.0tml";
const char S1912[] = "Mozilla/5.0 (X11; compatible; semantic-visions.com crawler; HTTPClient 3.1)";
const char S1913[] = "stremorbot AppEngine-Google; (+http://code.google.com/appengine; appid: s~stremor-crawler-dev)";
const char S1914[] = "Mozilla/5.0 (compatible;Zalbot/1.0.4; +http://zal.ir/bot.html)";
const char S1915[] = "Mozilla/5.0 (compatible; YioopBot; +http://173.13.143.74/bot.php)";
const char S1916[] = "ExB Language Crawler 2.0-1 (+http://www.exb.de/crawler)";
const char S1917[] = "Mozilla/5.0 (compatible; Memorybot/1.10.51 +http://www.mignify.com/)";
const char S1918[] = "WikiDo/1.1 (http://wikido.com; crawler@wikido.com)";
const char S1919[] = "Mozilla/5.0 (compatible; Statsbot/2.1)";
const char S1920[] = "ExB Language Crawler 2.0 (+http://www.exb.de/)";
const char S1921[] = "Mozilla/5.0 (compatible; Mail.RU_Bot/2.0; +http://go.mail.ru/help/robots)";
const char S1922[] = "Pinterest-bot/1.0";
const char S1923[] = "Mozilla/5.0 (compatible; 4SeoHuntBot; +http://4seohunt.com/bot.html)";
const char S1924[] = "RankFlex.com Webspider";
const char S1925[] = "SightupBot/1.1 (+http://www.sightup.com/en/bot.html)";
const char S1926[] = "Exabot/2.0";
const char S1927[] = "envolk/1.7 (+http://www.envolk.com/envolkspiderinfo.php)";
const char S1928[] = "ConveraCrawler/0.9d (+http://www.authoritativeweb.com/crawl)";
const char S1929[] = "Attribot/1.1 (compatible; Attribot-Feeds; http://static.attribyte.com/robotreadme.txt)";
const char S1930[] = "spider-feed";
const char S1931[] = "Open Web Analytics Bot 1.5.2";
const char S1932[] = "Baiduspider (+http://www.baidu.com/search/spider.html)";
const char S1933[] = "coccoc/1.0 (http://help.coccoc.vn/) AppEngine-Google; (+http://code.google.com/appengine; appid: s~doingfly)";
const char S1934[] = "Sogou web spider/4.0ml";
const char S1935[] = "Mozilla/5.0 (compatible; special_archiver/3.1.2 bot@archive.org +http://www.archives.gov/crawl.html)";
const char S1936[] = "Mozilla/5.0 (compatible; HeeherBot/1.0,Search for High; http://www.heeher.com)";
const char S1937[] = "NettRapport Crawler";
const char S1938[] = "Mozilla/5.0 (compatible; emefgebot/beta; +http://emefge.de/bot.html)";
const char S1939[] = "Mozilla/4.0+(compatible;+Robot/1.2 CL)";
const char S1940[] = "ExB Language Crawler 2.0-2 (+http://www.exb.de/crawler)";
const char S1941[] = "Mozilla/5.0 (compatible; woriobot support [at] zite [dot] com +http://zite.com)";
const char S1942[] = "Hello Bot v.2.1.74";
const char S1943[] = "pr-cy.ru Screenshot Bot";
const char S1944[] = "ExB Language Crawler 2.1 (+http://www.exb.de/crawler)";
const char S1945[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+shiny@seomoz.org)";
const char S1946[] = "ExB Language Crawler 2.0-3 (+http://www.exb.de/crawler)";
const char S1947[] = "OpenLD.ORG Spider v0.1 (http://www.openwebspider.org/)";
const char S1948[] = "Mozilla/5.0+(compatible;+WebofantBot+v3.14;+http://www.webofant.ch)";
const char S1949[] = "TinEye-bot/0.02 (see http://www.tineye.com/crawler.html)";
const char S1950[] = "TiofoBot/1.1(+http://www.tiofo.com)";
const char S1951[] = "Screaming Frog SEO Spider/2.10";
const char S1952[] = "Mozilla/5.0 (compatible; Baiduspider/2.0;+http://www.baidu.com/search/spider.html)";
const char S1953[] = "ExB Language Crawler 2.1.2 (+http://www.exb.de/crawler)";
const char S1954[] = "ExB Language Crawler 2.1.4 (+http://www.exb.de/crawler)";
const char S1955[] = "ExB Language Crawler 2.1.3 (+http://www.exb.de/crawler)";
const char S1956[] = "Web front page analyser (norw.acd.inst@gmail.com or edit robots.txt)";
const char S1957[] = "ExB Language Crawler 2.1.1 (+http://www.exb.de/crawler)";
const char S1958[] = "UCI IR crawler X";
const char S1959[] = "Web front page analyser. robots.txt complaint (norw.acd.inst@gmail.com)";
const char S1960[] = "Screaming Frog SEO Spider/2,10";
const char S1961[] = "PiStato indexer (+http://pistato.ru/bot.html)";
const char S1962[] = "PyCrawler";
const char S1963[] = "Mozilla/5.0 (compatible; imrbot/1.10.8 +http://www.mignify.com)";
const char S1964[] = "Mozilla/5.0 (compatible; web-ceo-online-bot/1.0; +http://online.webceo.com)";
const char S1965[] = "Domain-Informer.com Searchbot";
const char S1966[] = "Mozilla/5.0 (compatible; Mozilla; +http://wiki.github.com/bixo/bixo/bixocrawler; bixo-dev@yahoogroups.com)";
const char S1967[] = "Mozilla/5.0 (compatible; Ejeniobot/1.0; support@ejenio.com)";
const char S1968[] = "MiaDev/0.0.1 (MIA Bot for research project MIA (www.MIA-marktplatz.de); http://www.mia-marktplatz.de/spider; spider@mia-marktplatz.de)";
const char S1969[] = "coccoc/1.0 (http://help.coccoc.com/)";
const char S1970[] = "Spyder (Mag Reiting: http://magic-top.com)";
const char S1971[] = "testCrawler/1.0";
const char S1972[] = "PagesInventory (robot +http://www.pagesinventory.com)";
const char S1973[] = "gimme60 (Gimme60 Store ID Bot; gimme60.com)";
const char S1974[] = "ExB Language Crawler 2.1.5 (+http://www.exb.de/crawler)";
const char S1975[] = "Mozilla/5.0 (compatible; AndStartBot/2.0; +http://www.andstart.ro/bot.html)";
const char S1976[] = "Mozilla/5.0 (compatible; Crawler/0.9; http://linkfluence.net/)";
const char S1977[] = "Mozilla/5.0 (compatible; Gravitybot/1.0; +http://www.gravity.com/)";
const char S1978[] = "Mozilla/5.0 (compatible; ZEERCHBOT; +http://la.zeerch.com/bot.php)";
const char S1979[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+factors@seomoz.org)";
const char S1980[] = "Mozilla/5.0 (compatible; ZumBot/1.0; http://help.zum.com/inquiry)";
const char S1981[] = "Mozilla/4.0+(compatible;+Robot/1.0)";
const char S1982[] = "PulseCrawler/1.1 (+http://www.pulse.me/) AppEngine-Google; (+http://code.google.com/appengine; appid: s~hr-pulsesubscriber)";
const char S1983[] = "FisigBot/1.1(+http://www.fisig.com)";
const char S1984[] = "MP463Crawler1.2";
const char S1985[] = "Screaming Frog SEO Spider/2.11";
const char S1986[] = "socialbm_bot http://spider.socialbm.net";
const char S1987[] = "RavenCrawler";
const char S1988[] = "CrawlerProcess (http://www.PowerMapper.com) /5.1.600.0";
const char S1989[] = "ip-web-crawler.com";
const char S1990[] = "gimme60/1.6 (Gimme60 Store ID Bot; gimme60.com)";
const char S1991[] = "bot(www.cuwhois.com)";
const char S1992[] = "zimmbot/1.0";
const char S1993[] = "JSpider (http://j-spider.sourceforge.net)";
const char S1994[] = "myurlcrawler/0.1";
const char S1995[] = "AppEngine-Google; (+http://code.google.com/appengine; appid: s~vvalertscrawler)";
const char S1996[] = "Mozilla/5.0 (compatible; SpellCheck Bot 0.9; +http://showword.info/bot.html)";
const char S1997[] = "SNK Screenshot Bot/0.20 ( abuse at savenkeep dot_com )";
const char S1998[] = "Mozilla/5.0 (compatible; scanbot/1.0; +https://dazzlepod.com/ip/)";
const char S1999[] = "Mozilla/5.0 (X11; U; Linux x86_64; c) AppleWebKit/531.2+ (KHTML, like Gecko) Version/5.0 WatchMouse/$Revision: 25590 $";
const char S2000[] = "Mozilla/4.0+(compatible;+Robot/1.1t)";
const char S2001[] = "Mozilla/5.0 (compatible; eCrawler +http://www.cloudda.com)";
const char S2002[] = "Screaming Frog SEO Spider/2,11";
const char S2003[] = "smart-crawler";
const char S2004[] = "QuerySeekerSpider ( http://queryseeker.com/bot.html )";
const char S2005[] = "Mozilla/5.0 (compatible; AMZNKAssocBot/4.0 +http://affiliate-program.amazon.com)";
const char S2006[] = "it2media-domain-crawler/1.0 on crawler-prod.it2media.de";
const char S2007[] = "SiteCrawler/1.1.2";
const char S2008[] = "WebarooBot (Webaroo Bot; http://64.124.122.252/feedback.html)";
const char S2009[] = "Mozilla/5.0 (compatible; VR FrontPage Fetcher; +http://www.visualrevenue.com/bot.html)";
const char S2010[] = "CorenSearchBot/1.7 en libwww-perl/6.04";
const char S2011[] = "nahb.org link verification bot";
const char S2012[] = "Mozilla/5.0 (compatible; Autobot/1.0";
const char S2013[] = "Crawler for Netopian (http://www.netopian.co.uk/)";
const char S2014[] = "Mozilla/5.0 (compatible; SearchBot)";
const char S2015[] = "InsieveBot";
const char S2016[] = "Mozilla/5.0 (compatible; YandexBlogs/0.99; robot; B; +http://yandex.com/bots)1 readers";
const char S2017[] = "Mozilla/5.0 (compatible; Spiderlytics/1.0; +spider@spiderlytics.com)";
const char S2018[] = "San Francisco Low Impact Robot";
const char S2019[] = "Mozilla/5.0 (compatible; socialbm_bot/1.0; +http://spider.socialbm.net)";
const char S2020[] = "Mozilla/5.0 (compatible; MerchantCentricBot/1.0)";
const char S2021[] = "bot (+http://www.local.com)";
const char S2022[] = "adbeat_bot adbeat.com/policy";
const char S2023[] = "Jabse.com Crawler v.1.0 www.jabse.com/crawler.php//imagecrawler";
const char S2024[] = "omgilibot/0.4 +http://omgili.com";
const char S2025[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-15@seomoz.org)";
const char S2026[] = "adbeat_bot";
const char S2027[] = "blogtop.us crawler - http://blogtop.us/";
const char S2028[] = "Greegle Bot";
const char S2029[] = "Mozilla/5.0 (compatible; jjBot/1.0; +http://www.fhnw.ch; crawler.info.2@gmail.com)";
const char S2030[] = "CCTestBot/2.0 (Experimental)";
const char S2031[] = "Mozilla/5.0 (compatible; WebMahBot/3.0; http://www.webmah.com/bot.html; bot@webmah.com)";
const char S2032[] = "Mozilla/5.0 (compatible; Snipebot/1.0; +http://snipebot.com/bot.html)";
const char S2033[] = "NodePing";
const char S2034[] = "MyCrawler";
const char S2035[] = "BacklinkCrawler V (http://www.backlinktest.com/crawler.html)";
const char S2036[] = "Swiftbot";
const char S2037[] = "Snapbot/1.0";
const char S2038[] = "Mozilla/5.0 StashMetricsCrawler/1.0";
const char S2039[] = "IXEbot";
const char S2040[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-23@seomoz.org)";
const char S2041[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-attercob-01s@seomoz.org)";
const char S2042[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-27@seomoz.org)";
const char S2043[] = "Mozilla/5.0 (compatible; archive.org_bot/3.1.2 +http://pandora.nla.gov.au/crawl.html)";
const char S2044[] = "Mozilla/5.0 (compatible; uMBot-FC/1.0; mailto: crawling@ubermetrics-technologies.com)";
const char S2045[] = "ScreenerBot Crawler Beta 2.0 (+http://www.ScreenerBot.com)";
const char S2046[] = "Grahambot/0.1 (+http://www.sunaga-lab.com/graham-bot)";
const char S2047[] = "Mozilla/5.0 LabsCrawler/1.0";
const char S2048[] = "YisouSpider";
const char S2049[] = "CatchBot/5.0; +http://www.catchbot.com";
const char S2050[] = "Mozilla/5.0 (compatible; SemrushBot/0.96.2; +http://www.semrush.com/bot.html)";
const char S2051[] = "bl.uk_lddc_bot/3.1.1 (+http://www.bl.uk/aboutus/legaldeposit/websites/websites/faqswebmaster/index.html)";
const char S2052[] = "Aboundex/0.3 (http://www.aboundex.com/crawler/)";
const char S2053[] = "Mozilla/5.0 (compatible; Sosospider/2.0; +http://help.soso.com/webspider.htm)";
const char S2054[] = "Mozilla/5.0 (compatible; BlogBeatbot/2.1; +http://www.blogbeat.nl/)";
const char S2055[] = "Syntryx ANT Scout Chassis Pheromone; Mozilla/4.0 compatible crawler";
const char S2056[] = "Mozilla/5.0 (compatible; YandexMetrika/2.0; +http://yandex.com/bots)";
const char S2057[] = "Gigabot/2.0; http://www.gigablast.com/spider.html";
const char S2058[] = "yoono/1.0 web-crawler/1.0";
const char S2059[] = "MJ12bot/v1.0.8 (http://majestic12.co.uk/bot.php?+)";
const char S2060[] = "ReadBag (http://readbag.com/bot.html)";
const char S2061[] = "Elmer, the Thinglink ImageBot (http://www.thinglink.com/help/ThinglinkImageBot)";
const char S2062[] = "Mozilla/5.0 (compatible; ProductAdsBot/1.0 https://sellercentral.amazon.com/gp/help/help.html?ie=UTF8&itemID=X)";
const char S2063[] = "Insieve Bot";
const char S2064[] = "FTRF: Friendly robot/1.4";
const char S2065[] = "Peeplo Screenshot Bot/0.20 ( abuse at peeplo dot_com )";
const char S2066[] = "Crawler/3.0";
const char S2067[] = "Mozilla/5.0+(compatible; UptimeRobot/2.0; http://www.uptimerobot.com/)";
const char S2068[] = "searchch UltraSpider3000/2.0 (+http://www.search.ch/about.html)";
const char S2069[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-18@seomoz.org)";
const char S2070[] = "Mozilla/5.0 (compatible; trovitBot 1.0; +http://www.trovit.com/bot.html)";
const char S2071[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-28@seomoz.org)";
const char S2072[] = "Filestubebot/1.0 (+http://www.filestube.com)";
const char S2073[] = "ArtavisBot 1.0";
const char S2074[] = "Mozilla/5.0 (compatible; bot)";
const char S2075[] = "Mozilla/5.0 (compatible; ProCogSEOBot/1.0; +http://www.procog.com/ )";
const char S2076[] = "Xenu Link Sleuth/1.3.9 beta,gzip(gfe) (via docs.google.com/viewer)";
const char S2077[] = "Mozilla/5.0 (compatible; JobdiggerSpider +http://www.jobdigger.nl/spider)";
const char S2078[] = "Mozilla/5.0 (compatible; linkdexbot/2.1; +http://www.linkdex.com/about/bots/)";
const char S2079[] = "Mozilla/5.0 (compatible; HDB2-Bot/2.3; +http://hdb.0xff.org/bot.html)";
const char S2080[] = "CCBot/2.0";
const char S2081[] = "Mozilla/5.0 (compatible; IEZhanSpider/1.0; +http://www.iezhan.com/)";
const char S2082[] = "PennCISCrawler";
const char S2083[] = "Skywalker Bot";
const char S2084[] = "Mozilla/5.0 (compatible; awcheckBot; +http://awcheck.com/en/about)";
const char S2085[] = "Mozilla/5.0+(compatible;+Sosospider/2.0;++http://help.soso.com/webspider.htm)";
const char S2086[] = "Mozilla/5.0 (compatible; coccoc/1.0; +http://help.coccoc.com/)";
const char S2087[] = "Mozilla/5.0 (compatible; Baiduspider/2.0; +http://www.baidu.com/search/spider.html),gzip(gfe) (via docs.google.com/viewer)";
const char S2088[] = "Mozilla/5.0 (compatible; World Wide Web Wanderer (Wandex Bot)/1.4; http://wandex.net)";
const char S2089[] = "Mozilla/5.0 (compatible; URLAppendBot/1.0; +http://www.profound.net/urlappendbot.html)";
const char S2090[] = "Mozilla/5.0 (compatible; BazQuxBot/2.4; +http://crawler.bazqux.com)";
const char S2091[] = "Mozilla/5.0 (compatible; YRSpider; +http://www.yunyun.com/SiteInfo.php?r=about)";
const char S2092[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-04@seomoz.org)";
const char S2093[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-13@seomoz.org)";
const char S2094[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-05@seomoz.org)";
const char S2095[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-08@seomoz.org)";
const char S2096[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-03@seomoz.org)";
const char S2097[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-24@seomoz.org)";
const char S2098[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-10@seomoz.org)";
const char S2099[] = "ElisaBot 1.0";
const char S2100[] = "CERT FigleafBot/0.1 (www.cert.org)";
const char S2101[] = "Mozilla/5.0 (X11; U; Linux x86_64; c) AppleWebKit/531.2+ (KHTML, like Gecko) Version/5.0 WatchMouse/$Revision: 26005 $";
const char S2102[] = "WMU-bot";
const char S2103[] = "Mozilla/5.0 (compatible; GeliyooBot/1.0; +http://www.geliyoo.com/)";
const char S2104[] = "NCBot (http://netcomber.com : tool for finding true domain owners)";
const char S2105[] = "*Sogou web spider/3.0(+http://www.sogou.com/docs/help/webmasters.htm#07)";
const char S2106[] = "mbot v.1.16";
const char S2107[] = "NCBot (http://netcomber.com : tool for finding true domain owners) Queries/complaints: bot@netcomber.com";
const char S2108[] = "Symfony Spider (http://symfony.com/spider)";
const char S2109[] = "visaduhoc.info Crawler";
const char S2110[] = "Mozilla/5.0 (compatible; YYSpider; +http://www.yunyun.com/spider.html)";
const char S2111[] = "Noramy 0.1b (knowledge search and index bot)";
const char S2112[] = "AportCatalogRobot/2.0";
const char S2113[] = "Mozilla/5.0 (compatible; METASpider; +http://meta.ua/spider)";
const char S2114[] = "Accoona-AI-Agent/1.1.1 (crawler at accoona dot com)";
const char S2115[] = "Dudu bot";
const char S2116[] = "DocolocBot";
const char S2117[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-20@seomoz.org)";
const char S2118[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-06@seomoz.org)";
const char S2119[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-17@seomoz.org)";
const char S2120[] = "Mozilla/5.0 (compatible; lufsbot/0.1; +http://www.lufs.org/bot.html)";
const char S2121[] = "Mozilla/5.0 (compatible; DCPbot/1.4; +http://domains.checkparams.com/)";
const char S2122[] = "Mozilla/5.0 (compatible; FeedWindBot/2.0;+http://feed.mikle.com/en/guide/crawler.html)";
const char S2123[] = "Mozilla/5.0 (compatible; firmilybot/0.2; +http://www.firmily.com/bot.php";
const char S2124[] = "boitho.com-robot/1.0";
const char S2125[] = "Mozilla/5.0+(compatible;+Robot/1.3)";
const char S2126[] = "Mozilla/4.0 (compatible; Struct.it crawler; +http://struct.it/contacts/)";
const char S2127[] = "Mozilla/5.0 (compatible; GeliyooBot/1.0beta; +http://www.geliyoo.com/)";
const char S2128[] = "Curator Crawler";
const char S2129[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-26@seomoz.org)";
const char S2130[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-12@seomoz.org)";
const char S2131[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-02@seomoz.org)";
const char S2132[] = "Mozilla/5.0 (compatible; whoisrequest.org Bot/0.1)";
const char S2133[] = "Mozilla/5.0 (compatible; HttpConduitDownloader/1.0; +http://hackage.haskell.org/package/http-conduit-downloader; Heins & Partner Impressumscrawler)";
const char S2134[] = "Mozilla/5.0 (compatible; FindLinksBot/0.1)";
const char S2135[] = "calculon spider/0.1.0";
const char S2136[] = "Mozilla/5.0+(compatible;+Robot/1.2)";
const char S2137[] = "Mozilla/5.0 (compatible; aiHitBot/2.7; +http://www.aihit.com/)";
const char S2138[] = "Mozilla/5.0 (compatible; meanpathbot/1.0; +http://www.meanpath.com/meanpathbot.html)";
const char S2139[] = "dorkobot/kutzooi (xxx; www.xxx.com; xxx at xxx dot xxx)";
const char S2140[] = "infobot/1.0";
const char S2141[] = "\"Mozilla/5.0 (compatible; Sosospider/2.0; +http://help.soso.com/webspider.htm)\"";
const char S2142[] = "BLEXBot";
const char S2143[] = "Mozilla/5.0 (compatible; firmilybot/0.3; +http://www.firmily.com/bot.php";
const char S2144[] = "Mozilla/5.0 (compatible; uMBot-LN/1.0; mailto: crawling@ubermetrics-technologies.com)";
const char S2145[] = "Mozilla/5.0 (compatible; ExactSeekTest; +http://yioop.exactseek.com/bot.php)";
const char S2146[] = "BrightEdge Crawler/1.0 (crawler@brightedge.com)";
const char S2147[] = "FAST Enterprise Crawler 6 used by Bygglo / Svensk Byggtjanst";
const char S2148[] = "Mippin feed crawler. www.mippin.com";
const char S2149[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-09@seomoz.org)";
const char S2150[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-01@seomoz.org)";
const char S2151[] = "XiblBot/1.0 (+http://www.xibl.com)";
const char S2152[] = "CyotekWebCopy/1.0 CyotekWebCrawler/1.0";
const char S2153[] = "AWS-EMR-Research-WebCrawler";
const char S2154[] = "CrawlerProcess (http://www.PowerMapper.com) /5.2.618.0";
const char S2155[] = "nrsbot/6";
const char S2156[] = "Mozilla/5.0 (compatible; GigaBot/1.0; +http://www.gigablast.com/ )";
const char S2157[] = "DuckDuckBot/1.1; (+http://duckduckgo.com/duckduckbot.html)";
const char S2158[] = "CPQ RSS Crawler";
const char S2159[] = "Mozilla/5.0 (compatible; Taboolabot/3.7; +http://www.taboola.com)";
const char S2160[] = "Website-Rating.com Searchbot";
const char S2161[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-25@seomoz.org)";
const char S2162[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-29@seomoz.org)";
const char S2163[] = "node-crawler/0.2.4";
const char S2164[] = "Mozilla/4.0 (squirrobot/1.0; http://squirro.com/squirrobot/)";
const char S2165[] = "\"NCBot.";
const char S2166[] = "SSFC 1.0 Focused Crawler, a research project which advances data mining research; Conatact: andy.xue@unimelb.edu.au";
const char S2167[] = "Mozilla/5.0 (compatible; SemrushBot/0.96.3; +http://www.semrush.com/bot.html)";
const char S2168[] = "Mozilla/5.0 (compatible; DonkeyBot/0.2; +http://www.dasdonkey.com)";
const char S2169[] = "Whatsnew Bot";
const char S2170[] = "Baiduspider ( http://www.baidu.com/search/spider.htm)";
const char S2171[] = "endeca webcrawler/WebCrawler";
const char S2172[] = "carbonbot/1.0";
const char S2173[] = "msnbot/0.9 (+http://search.msn.com/msnbot.htm)";
const char S2174[] = "Mozilla/5.0 (compatible; sitecrawler/1.1; +http://www.siteencyclopedia.com/)";
const char S2175[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-07@seomoz.org)";
const char S2176[] = "EbuzzingFeedBot 1.0";
const char S2177[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-21@seomoz.org)";
const char S2178[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-22@seomoz.org)";
const char S2179[] = "Mozilla/5.0 (compatible; ReplazBot/1.5; +http://www.replaz.com)";
const char S2180[] = "kraken-crawler/0.2.0; +http://www.kontera.com/kraken-crawler.html";
const char S2181[] = "gsa-crawler (Enterprise; T3-KY8WUCXTQWSQG; it@pattisonoutdoor.com,ievans@pattisonoutdoor.com,soliveira@pattisonoutdoor.com)";
const char S2182[] = "Math. Comp. PA Crawler";
const char S2183[] = "FRCrawler";
const char S2184[] = "Mozilla/5.0 (compatible; firmilybot/0.1; +http://www.firmily.com/bot.php";
const char S2185[] = "Mozilla/5.0 (BCS spider 2003-2013)";
const char S2186[] = "g2reader-bot/1.0 (+http://www.g2reader.com/)";
const char S2187[] = "User-Agent	Baiduspider";
const char S2188[] = "Mozilla/5.0 (compatible; BarrigasBot/2.0)";
const char S2189[] = "Mozilla/5.0 (compatible; CloudServerMarketSpider/1.0; +http://www.cloudservermarket.com/spider.html)";
const char S2190[] = "\"NCBot http://netcomber.com . Tool for finding their domain names\"";
const char S2191[] = "Mozilla/5.0 (compatible; SolomonoBot/1.05; +http://www.solomono.ru)";
const char S2192[] = "Mozilla/5.0 (compatible; ZEERCHBOT; +http://www.zeerch.com/bot.php)";
const char S2193[] = "EMR-Research-WebCrawler";
const char S2194[] = "Mozilla/5.0 (compatible; heritrix/3.1.2-SNAPSHOT-X.X +http://www.google.com/bot.html)";
const char S2195[] = "Mozilla/5.0 (compatible; BLEXBot/1.0; +http://webmeup.com/crawler.html)";
const char S2196[] = "Mozilla/5.0(compatible;Sosospider/2.0;+http://help.soso.com/webspider.htm)";
const char S2197[] = "Mozilla/5.0 (compatible; GoblioBot/0.3.7; +http://goblio.eu/bot.htm)";
const char S2198[] = "crawler/1.0";
const char S2199[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-14@seomoz.org)";
const char S2200[] = "User-Agent: Curious George - www.analyticsseo.com/crawler";
const char S2201[] = "Mozilla/5.0 (compatible; Crawler; +null; null)";
const char S2202[] = "InoReader Crawler (https://www.inoreader.com) - support@inoreader.com";
const char S2203[] = "gsa-crawler";
const char S2204[] = "TrustpilotCrawler/1.0 (compatible; http://www.trustpilot.com)";
const char S2205[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-19@seomoz.org)";
const char S2206[] = "CrawlerProcess (http://www.PowerMapper.com) /";
const char S2207[] = "Mozilla/5.0 (compatible; MetaJobBot; http://www.metajob.at/crawler)";
const char S2208[] = "WebCrawler v1.2";
const char S2209[] = "ICC-Crawler (Mozilla-compatible; icc-crawl-contact@ml.nict.go.jp; http://kc.nict.go.jp/project1/crawl-ja.html)";
const char S2210[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr2-crawler-02@seomoz.org)";
const char S2211[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-16@seomoz.org)";
const char S2212[] = "Mozilla/5.0 (compatible; Mozilla/5.0; +http://wiki.github.com/bixo/bixo/bixocrawler; bixo-dev@yahoogroups.com)";
const char S2213[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr2-crawler-05@seomoz.org)";
const char S2214[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr2-crawler-08@seomoz.org)";
const char S2215[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr2-crawler-11@seomoz.org)";
const char S2216[] = "FTBCrawler";
const char S2217[] = "Mozilla/5.0 (compatible; 008/0.83; http://www.80legs.com/webcrawler.html) Gecko/X;c2VvLWNyYXdsZXI=";
const char S2218[] = "Hello from Axtaris-Crawler! (http://axtaris.org)";
const char S2219[] = "Mozilla/5.0 (compatible; oBot/2.3.1; http://filterdb.iss.net/crawler/)";
const char S2220[] = "Mozilla/5.0 (compatible; MixrankBot; crawler@mixrank.com)";
const char S2221[] = "rogerbot/1.0 (http://www.moz.com/dp/rogerbot, rogerbot-crawler@moz.com)";
const char S2222[] = "rogerbot/1.0 (http://moz.com/help/pro/what-is-rogerbot-, rogerbot-crawler+shiny@moz.com)";
const char S2223[] = "Mozilla/5.0 (compatible; BIXOCRAWLER; +http://wiki.github.com/bixo/bixo/bixocrawler; bixo-dev@yahoogroups.com)";
const char S2224[] = "Mozilla/5.0 (compatible; MJ12bot/v1.4.4; http://www.majestic12.co.uk/bot.php?+)";
const char S2225[] = "Marfeel-crawler";
const char S2226[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-30@seomoz.org)";
const char S2227[] = "ownCloud Server Crawler";
const char S2228[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr2-crawler-06@seomoz.org)";
const char S2229[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr2-crawler-07@seomoz.org)";
const char S2230[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr2-crawler-10@seomoz.org)";
const char S2231[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr2-crawler-27@seomoz.org)";
const char S2232[] = "Mozilla/5.0 (compatible; socialrep/1.0; +http://bixolabs.com/crawler/general; crawler@bixolabs.com)";
const char S2233[] = "Mozilla/5.0 (compatible; MJ12bot/v1.2.4; http://www.majestic12.co.uk/bot.php?+);c2VvLWNyYXdsZXI=";
const char S2234[] = "Mozilla/5.0 (compatible; ayna-crawler +http://www.ayna.com);c2VvLWNyYXdsZXI=";
const char S2235[] = "Netchart Adv Crawler 1.0;c2VvLWNyYXdsZXI=";
const char S2236[] = "wf_crawler (http://www.websitefigures.com);c2VvLWNyYXdsZXI=";
const char S2237[] = "Mozilla/5.0 (compatible; BLEXBot/1.0; +http://webmeup-crawler.com/)";
const char S2238[] = "updated/0.1-alpha (updated crawler; http://www.updated.com; crawler@updated.com)";
const char S2239[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr2-crawler-01@seomoz.org)";
const char S2240[] = "rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr2-crawler-14@seomoz.org)";
const char S2241[] = "T3census-Crawler/1.0";
const char S2242[] = "Ronsmap crawler/0.1;c2VvLWNyYXdsZXI=";
const char S2243[] = "Mozilla/5.0 (compatible; MJ12bot/v1.2.1; http://www.majestic12.co.uk/bot.php?+);c2VvLWNyYXdsZXI=";
const char S2244[] = "Mozilla/5.0 (compatible; MJ12bot/v1.3.3; http://www.majestic12.co.uk/bot.php?+);c2VvLWNyYXdsZXI=";
const char S2245[] = "Mozilla/5.0 (compatible; MJ12bot/v1.3.1; http://www.majestic12.co.uk/bot.php?+);c2VvLWNyYXdsZXI=";
const char S2246[] = "Willow Internet Crawler by Twotrees V2.1;c2VvLWNyYXdsZXI=";
const char S2247[] = "Mozilla/5.0 (compatible; MJ12bot/v1.3.2; http://www.majestic12.co.uk/bot.php?+);c2VvLWNyYXdsZXI=";
const char S2248[] = "ReutersCrawler/1.0 (http://reuters.com/crawler.html);c2VvLWNyYXdsZXI=";
const char S2249[] = "radian6_linkcheck_(www.radian6.com/crawler);c2VvLWNyYXdsZXI=";
const char S2250[] = "PostPost/1.0 (+http://postpo.st/crawlers);c2VvLWNyYXdsZXI=";
const char S2251[] = "MultiPLX.com RSS Crawler/1.0";
const char S2252[] = "Mozilla/5.0 (compatible; MJ12bot/v1.4.3; http://catsynetwork.dyndns.org/bots.html";
const char S2253[] = "SSFC 1.0 Focused Crawler, a research project which advances data mining research; Conatact: andy(DOT)xue(AT)unimelb(DOT)edu(DOT)au";
const char S2254[] = "sproose/0.1 (sproose bot; http://www.sproose.com/bot.html; crawler@sproose.com)";
const char S2255[] = "HubSpot Crawler";
const char S2256[] = "Mozilla/5.0 (compatible; GABot/1.0; +http://www.groupangle.com/crawler.html)";
const char S2257[] = "Mozilla/5.0 (compatible; GABot/1.0; crawler@yoterra.com; +http://www.groupangle.com/crawler.html)";
const char S2258[] = "rogerbot/1.0 (http://moz.com/help/pro/what-is-rogerbot-, rogerbot-crawler+pr1-crawler-08@moz.com)";
const char S2259[] = "rogerbot/1.0 (http://moz.com/help/pro/what-is-rogerbot-, rogerbot-crawler+pr2-crawler-02@moz.com)";
const char S2260[] = "rogerbot/1.0 (http://moz.com/help/pro/what-is-rogerbot-, rogerbot-crawler+pr1-crawler-31@moz.com)";
const char S2261[] = "rogerbot/1.0 (http://moz.com/help/pro/what-is-rogerbot-, rogerbot-crawler+pr1-crawler-13@moz.com)";
const char S2262[] = "rogerbot/1.0 (http://moz.com/help/pro/what-is-rogerbot-, rogerbot-crawler+pr1-crawler-32@moz.com)";
const char S2263[] = "Mozilla/5.0 (compatible; SEO-crawler/1.0;)";
const char S2264[] = "gsa-crawler (Enterprise; S5-BCT4XJX3FUJAB; nobody@google.com)";
const char S2265[] = "Mozilla/5.0 (compatible; ElectricMonk/3.1.1 +https://www.duedil.com/our-crawler/)";
const char S2266[] = "Mozilla/5.0 (compatible; BlueEyeTestCrawler/1.0; +http://blueeyewebtest.com)";
const char S2267[] = "node-crawler/0.2.5";
const char S2268[] = "rogerbot/1.0 (http://moz.com/help/pro/what-is-rogerbot-, rogerbot-crawler+pr2-crawler-14@moz.com)";
const char S2269[] = "rogerbot/1.0 (http://moz.com/help/pro/what-is-rogerbot-, rogerbot-crawler+pr1-crawler-05@moz.com)";
const char S2270[] = "rogerbot/1.0 (http://moz.com/help/pro/what-is-rogerbot-, rogerbot-crawler+pr2-crawler-04@moz.com)";
const char S2271[] = "rogerbot/1.0 (http://moz.com/help/pro/what-is-rogerbot-, rogerbot-crawler+pr1-crawler-09@moz.com)";
const char S2272[] = "CrawlerProcess (http://www.PowerMapper.com) /5.3.640.0";
const char S2273[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B5105c";
const char S2274[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B5117b";
const char S2275[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B5126b";
const char S2276[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S2277[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141";
const char S2278[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143";
const char S2279[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144";
const char S2280[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142";
const char S2281[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone4,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/6.1;FBSS/2; FBCR/Verizon;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S2282[] = "Mozilla/5.0 (iPhone5,1; iPhone; U; CPU OS 6_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N41AP (gzip)";
const char S2283[] = "Pulse 3.0.4 (iPhone; iPhone OS 6.1; en_US)";
const char S2284[] = "Mozilla/5.0 (iPhone5,2; iPhone; U; CPU OS 6_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N42AP (gzip)";
const char S2285[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S2286[] = "Mozilla/5.0 (iPhone4,1; iPhone; U; CPU OS 6_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N94AP (gzip)";
const char S2287[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B311";
const char S2288[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 iPad2,1 BingWeb/3.03.1428.X";
const char S2289[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145";
const char S2290[] = "NAVER(inapp; cafe; 100; 2.9.6) Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName";
const char S2291[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone4,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/6.1.1;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S2292[] = "Mozilla/5.0 (iPhone3,1; iPhone; U; CPU OS 6_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N90AP (gzip)";
const char S2293[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146";
const char S2294[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S2295[] = "Mozilla/5.0 (iPhone4,1; iPhone; U; CPU OS 6_1_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N94AP (gzip)";
const char S2296[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B147";
const char S2297[] = "Miu Ptt 1.9.2 (iPhone; iPhone OS 6.1; zh_TW)";
const char S2298[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_2 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone4,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/6.1.2;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S2299[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B318";
const char S2300[] = "Mozilla/5.0 (iPhone4,1; iPhone; U; CPU OS 6_1_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N94AP (gzip)";
const char S2301[] = "Mozilla/5.0 (iPhone3,3; iPhone; U; CPU OS 6_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N92AP (gzip)";
const char S2302[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 iPad2,1 BingWeb/3.03.1428.X";
const char S2303[] = "Mozilla/5.0 (iPhone5,2; iPhone; U; CPU OS 6_1_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N42AP (gzip)";
const char S2304[] = "Mozilla/5.0 (iPhone3,1; iPhone; U; CPU OS 6_1_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N90AP (gzip)";
const char S2305[] = "Mozilla/5.0 (iPhone; CPU OS 6_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48 Instapaper/4.2.7 (+http://www.instapaper.com/iphone)";
const char S2306[] = "wp-iphone/3.5.1 (iPhone OS 6.1.2, iPhone) Mobile";
const char S2307[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/9Z999";
const char S2308[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/8536.26.7 DiigoBrowser";
const char S2309[] = "Mozilla/5.0 (iPhone5,1; iPhone; U; CPU OS 6_1_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N41AP (gzip)";
const char S2310[] = "NAVER(inapp; cafe; 100; 2.9.6) Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_2 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName";
const char S2311[] = "wp-iphone/3.5.1 (iPhone OS 6.1.2, iPad) Mobile";
const char S2312[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 bline/1.04 (iPhone OS 6.1.2, iPhone)";
const char S2313[] = "Mozilla/5.0 (iPhone3,3; iPhone; U; CPU OS 6_1_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N92AP (gzip)";
const char S2314[] = "Miu Ptt 1.9.2 (iPhone; iPhone OS 6.1.2; zh_TW)";
const char S2315[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329";
const char S2316[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S2317[] = "Mozilla/5.0 (iPhone; CPU iPhone OS6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146";
const char S2318[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone4,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/6.1.3;FBSS/2; FBCR/Verizon;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S2319[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/9Z999";
const char S2320[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 bline/1.06 (iPhone OS 6.1.3, iPhone)";
const char S2321[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 iPad2,1 BingWeb/3.03.1428.X";
const char S2322[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari IPhoneAppVersion/2.5.2 IPhoneWebkitWrapper BillingSupported";
const char S2323[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari IPhoneAppVersion/2.5.2 IPhoneWebkitWrapper BillingSupported";
const char S2324[] = "Mozilla/5.0 (iPhone; CPU iPhone OS6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329";
const char S2325[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/8536.26.7 DiigoBrowser";
const char S2326[] = "Miu Ptt 1.9.2 (iPhone; iPhone OS 6.1.3; zh_TW)";
const char S2327[] = "NAVER(inapp; cafe; 100; 2.9.7) Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName";
const char S2328[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329";
const char S2329[] = "Mozilla/5.0 (iPhone5,1; iPhone; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/16077 (KHTML, like Gecko) Mobile/N41AP (gzip)";
const char S2330[] = "Mozilla/5.0 (iPhone5,2; iPhone; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/16077 (KHTML, like Gecko) Mobile/N42AP (gzip)";
const char S2331[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S2332[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350";
const char S2333[] = "wp-iphone/3.5.3 (iPhone OS 6.1.3, iPhone) Mobile";
const char S2334[] = "Mozilla/5.0 (iPhone; U; CPU iPhone 6_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.8 (iPhone4,1/6.1.3)";
const char S2335[] = "Mozilla/5.0 (iPhone; U; CPU iPhone 6_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.8 (iPad2,1/6.1.3)";
const char S2336[] = "wp-iphone/3.5.3 (iPhone OS 6.1.4, iPhone) Mobile";
const char S2337[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_4 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone5,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/6.1.4;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S2338[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 bline/1.06 (iPhone OS 6.1.4, iPhone)";
const char S2339[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146";
const char S2340[] = "Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/8536.25";
const char S2341[] = "NAVER(inapp; cafe; 100; 3.0.0) Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_4 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName";
const char S2342[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Safari/8536.26.7 DiigoBrowser";
const char S2343[] = "Miu Ptt 1.9.2 (iPhone; iPhone OS 6.1.1; zh_TW)";
const char S2344[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 [FBAN/FBIOS;FBAV/5.5;FBBV/X;FBDV/iPad3,4;FBMD/iPad;FBSN/iPhone OS;FBSV/6.1.2;FBSS/2; FBCR/;FBID/tablet;FBLC/en_US;FBOP/0]";
const char S2345[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/8536.26.7 DiigoBrowser";
const char S2346[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 [FBAN/MessengerForiOS;FBAV/2.5;FBBV/X;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/6.1.3;FBSS/1; FBCR/;FBID/phone;FBLC/en_US;FBOP/1]";
const char S2347[] = "Mozilla/5.0 (iPhone4,1; iPhone; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/N94AP (gzip)";
const char S2348[] = "Mozilla/5.0 (iPhone3,1; iPhone; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/N90AP (gzip)";
const char S2349[] = "Mozilla/5.0 (iPhone5,2; iPhone; U; CPU OS 6_1_4 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/N42AP (gzip)";
const char S2350[] = "Mozilla/5.0 (iPhone5,1; iPhone; U; CPU OS 6_1_4 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/N41AP (gzip)";
const char S2351[] = "wp-iphone/3.6.3 (iPhone OS 6.1.3, iPad) Mobile";
const char S2352[] = "Mozilla/5.0 (iPhone3,3; iPhone; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/N92AP (gzip)";
const char S2353[] = "Mozilla/5.0 ( ; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/8536.25";
const char S2354[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari IPhoneAppVersion/2.6.0 IPhoneWebkitWrapper BillingSupported";
const char S2355[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 TwonkyBeamBrowser/3.4.3-37 (iPhone iOS Ver:6.1.3)";
const char S2356[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329,Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329";
const char S2357[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 webmdapp_wbmd/3.4";
const char S2358[] = "Mozilla/5.0 ( ; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/8536.25";
const char S2359[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 TwonkyBeamBrowser/3.4.3-37 (iPhone iOS Ver:6.1.4)";
const char S2360[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari IPhoneAppVersion/2.6.0 IPhoneWebkitWrapper BillingSupported";
const char S2361[] = "Mozilla/5.0 (iPhone2,1; iPhone; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/N88AP (gzip)";
const char S2362[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari IPhoneAppVersion/2.6.1 IPhoneWebkitWrapper BillingSupported";
const char S2363[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari IPhoneAppVersion/2.6.1 IPhoneWebkitWrapper BillingSupported";
const char S2364[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/8536.25";
const char S2365[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0)";
const char S2366[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0)";
const char S2367[] = "Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/5.0)";
const char S2368[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Win64; x64; Trident/6.0)";
const char S2369[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/4.0; InfoPath.2; SV1; .NET CLR 2.0.50727; WOW64)";
const char S2370[] = "Mozilla/5.0 (compatible; MSIE 10.6; Windows NT 6.1; Trident/5.0; InfoPath.2; SLCC1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 2.0.50727) 3gpp-gba";
const char S2371[] = "Windows-RSS-Platform/2.0 (MSIE 10.0; Windows NT 6.1)";
const char S2372[] = "Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.1)";
const char S2373[] = "Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.1; SV1)";
const char S2374[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)";
const char S2375[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)";
const char S2376[] = "Mozilla/5.0 ( ; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0)";
const char S2377[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; 3M/MSIE 8.0)";
const char S2378[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; QQBrowser/7.1.6004.400)";
const char S2379[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; QQBrowser/7.1.6004.400)";
const char S2380[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows 7; Trident/6.0)";
const char S2381[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; QQBrowser/7.2.7006.400)";
const char S2382[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; QQBrowser/7.2.7006.400)";
const char S2383[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1)";
const char S2384[] = "Mozilla/5.0 ( ; MSIE 10.0; Windows NT 6.1; Trident/6.0)";
const char S2385[] = "Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SlimBrowser)";
const char S2386[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; EIE10;ESARMSN)";
const char S2387[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0)";
const char S2388[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Verizon Windows Live)";
const char S2389[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; 3M/MSIE 8.0)";
const char S2390[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; EIE10;ESARMSN)";
const char S2391[] = "Empty (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0)";
const char S2392[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Xerox_Win32PWS)";
const char S2393[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Verizon Windows Live)";
const char S2394[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2))";
const char S2395[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S2396[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Mozilla/4.0 (compatiblt; MSIE 8.0; Windows NT; LCBSECURE))";
const char S2397[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Mozilla/4.0 (compatible; MSIE 6.0; Menara))";
const char S2398[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S2399[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7)";
const char S2400[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2))";
const char S2401[] = "MASBJS (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0)";
const char S2402[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; EmbeddedWB 14.52 from: http://www.bsalsa.com/ EmbeddedWB 14.52; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7; Tablet PC 2.0; .NET4.0E; 360SE)";
const char S2403[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; 3M/MSIE 8.0;; 3M/MSIE 8.0)";
const char S2404[] = "Mozilla/5.0  (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0)";
const char S2405[] = "Mozilla/5.0 (MSIE 10.0; Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko";
const char S2406[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chrome/24.0 (compatible; MSIE 6.0; Windows NT; Windows 7); Mozilla/18.0.1 (Compatible; MSIE 6.0;MSIE 7.0; MSIE 9.0; Windows NT; Windows 7))";
const char S2407[] = "Microsoft Office/14.0 (Windows NT 6.1; Microsoft Outlook 14.0.4760; Pro; ms-office; MSOffice 14)";
const char S2408[] = "Microsoft Office/14.0 (Windows NT 6.1; Microsoft Outlook 14.0.4734; Pro; ms-office; MSOffice 14)";
const char S2409[] = "Microsoft Office/14.0 (Windows NT 6.1; Microsoft Outlook 14.0.5128; Pro; ms-office; MSOffice 14)";
const char S2410[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2411[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2412[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2413[] = "Microsoft Office/14.0 (Windows NT 6.1; Microsoft Outlook 14.0.6023; Pro; ms-office; MSOffice 14)";
const char S2414[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2415[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2416[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; Tablet PC 2.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2417[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2418[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.5136; ms-office; MSOffice 14)";
const char S2419[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.5128; ms-office; MSOffice 14)";
const char S2420[] = "Microsoft Office/14.0 (Windows NT 6.1; Microsoft Outlook 14.0.6106; Pro; ms-office; MSOffice 14)";
const char S2421[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2422[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.3; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.5136; ms-office; MSOffice 14)";
const char S2423[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MS-RTC LM 8; InfoPath.3; Microsoft Outlook 14.0.5136; ms-office; MSOffice 14)";
const char S2424[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.5139; ms-office; MSOffice 14)";
const char S2425[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.5138; ms-office; MSOffice 14)";
const char S2426[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.5128; ms-office; MSOffice 14)";
const char S2427[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; SASstaged; Microsoft Outlook 14.0.5128; ms-office; MSOffice 14)";
const char S2428[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S2429[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S2430[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S2431[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2432[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2433[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2434[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2435[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2436[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2437[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2438[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2439[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2440[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2441[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S2442[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2443[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.30729; .NET4.0C; Media Center PC 6.0; SLCC2; Microsoft Outlook 14.0.6106; ms-office; MSOffice 14)";
const char S2444[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S2445[] = "Microsoft Office/14.0 (Windows NT 6.1; Microsoft Outlook 14.0.6112; Pro; ms-office; MSOffice 14)";
const char S2446[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6114; ms-office; MSOffice 14)";
const char S2447[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S2448[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Zune 4.7; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2449[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S2450[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.5139; ms-office; MSOffice 14)";
const char S2451[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; Zune 4.7; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S2452[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S2453[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S2454[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; Tablet PC 2.0; Creative AutoUpdate v1.40.01; Microsoft Outlook 14.0.61";
const char S2455[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; Zune 4.7; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S2456[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.5130; ms-office; MSOffice 14)";
const char S2457[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2458[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S2459[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Media Center PC 5.0; SLCC1; OfficeLiveConnector.1.5; OfficeLivePatch.1.3; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2460[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Zune 4.7; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2461[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6112; ms-office; MSOffice 14)";
const char S2462[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Tablet PC 2.0; Microsoft Outlook 14.0.5136; ms-office; MSOffice 14)";
const char S2463[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2464[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Media Center PC 5.0; SLCC1; InfoPath.3; .NET4.0C; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S2465[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S2466[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; IE8Mercury; Microsoft Outlook 14.0.6114; ms-office; MSOffice 14)";
const char S2467[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Media Center PC 5.0; SLCC1; Tablet PC 2.0; OfficeLiveConnector.1.5; OfficeLivePatch.1.3; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2468[] = "Mozilla/4.0 (compatible; MSIE 8.0; TS7; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.5136; ms-office; MSOffice 14)";
const char S2469[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2470[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Tablet PC 2.0; Zune 4.7; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2471[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Tablet PC 2.0; Zune 4.7; BRI/2; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2472[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6122; ms-office; MSOffice 14)";
const char S2473[] = "Microsoft Office/14.0 (Windows NT 6.1; Microsoft Outlook 14.0.6123; Pro)";
const char S2474[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.5136; ms-office; MSOffice 14)";
const char S2475[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2476[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6114; ms-office; MSOffice 14)";
const char S2477[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Avant Browser; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2478[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2479[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; Tablet PC 2.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.5128; ms-office; MSOffice 14)";
const char S2480[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6114; ms-office; MSOffice 14)";
const char S2481[] = "Microsoft Office/14.0 (Windows NT 6.1; Microsoft Outlook 14.0.4536; Pro; MSOffice 14)";
const char S2482[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Zune 4.7; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2483[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S2484[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Tablet PC 2.0; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S2485[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6114; ms-office; MSOffice 14)";
const char S2486[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2487[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2488[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2489[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2490[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2491[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2492[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2493[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2494[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2495[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2496[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2497[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2498[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2499[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2500[] = "Microsoft Office Outlook 2010 (14.0.6123) Windows NT 6.1";
const char S2501[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6123; ms-office; MSOffice 14)";
const char S2502[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6123; ms-office; MSOffice 14)";
const char S2503[] = "Microsoft Office/14.0 (Windows NT 6.1; Microsoft Outlook 14.0.6129; Pro)";
const char S2504[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; Zune 4.7; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2505[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6129; ms-office; MSOffice 14)";
const char S2506[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Zune 4.7; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2507[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2508[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Zune 4.7; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2509[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; Zune 4.7; InfoPath.3; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2510[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6120; ms-office; MSOffice 14)";
const char S2511[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6123; ms-office; MSOffice 14)";
const char S2512[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; MS-RTC LM 8; .NET4.0E; Tablet PC 2.0; S.N.O.W.4; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S2513[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6129; ms-office; MSOffice 14)";
const char S2514[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Media Center PC 5.0; SLCC1; .NET4.0C; InfoPath.3; Tablet PC 2.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2515[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Media Center PC 5.0; SLCC1; OfficeLiveConnector.1.5; OfficeLivePatch.1.3; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2516[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2517[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2518[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2519[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Tablet PC 2.0; Microsoft Outlook 14.0.6112; ms-office; MSOffice 14)";
const char S2520[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2521[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Zune 4.7; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2522[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2523[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2524[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2525[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2526[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2527[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2528[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2529[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2530[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2531[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; .NET4.0E; Microsoft Outlook 14.0.6102; ms-office; MSOffice 14)";
const char S2532[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2533[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2534[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2535[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2536[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S2537[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S2538[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6112; ms-office; MSOffice 14)";
const char S2539[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6129; ms-office; MSOffice 14)";
const char S2540[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2541[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2542[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2543[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2544[] = "Microsoft Office Outlook 2010 (14.0.6112) Windows NT 6.1";
const char S2545[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2546[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Win64; x64; Trident/6.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2547[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2548[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6114; ms-office; MSOffice 14)";
const char S2549[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; FBViewer-5.0.1.33; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MS-RTC LM 8; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6102; ms-office; MSOffice 14)";
const char S2550[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6120; ms-office; MSOffice 14)";
const char S2551[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2552[] = "Microsoft Office Outlook 2010 (14.0.6129) Windows NT 6.1";
const char S2553[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2554[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2555[] = "Mozilla/4.0 (compatible; MSIE 9.0; AD1; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2556[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; MS-RTC LM 8; SFIEAUTH1; Tablet PC 2.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2557[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Microsoft Outlook 14.0.5128; ms-office; MSOffice 14)";
const char S2558[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2559[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.5130; ms-office; MSOffice 14)";
const char S2560[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2561[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2562[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6133; ms-office; MSOffice 14)";
const char S2563[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2564[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.5136; ms-office; MSOffice 14)";
const char S2565[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6120; ms-office; MSOffice 14)";
const char S2566[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2567[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6133; ms-office; MSOffice 14)";
const char S2568[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2569[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Media Center PC 5.0; SLCC1; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2570[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Avant Browser; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2571[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2572[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Tablet PC 2.0; Microsoft Outlook 14.0.6133; ms-office; MSOffice 14)";
const char S2573[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6133; ms-office; MSOffice 14)";
const char S2574[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SV1; [eburo v4.0]; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2575[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; Zune 4.7; ATT; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2576[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2577[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Media Center PC 5.0; SLCC1; Tablet PC 2.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2578[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2579[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2580[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S2581[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; GIL 3.5; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6136; ms-office; MSOffice 14)";
const char S2582[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6136; ms-office; MSOffice 14)";
const char S2583[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2584[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2585[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6136; ms-office; MSOffice 14)";
const char S2586[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2587[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S2588[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S2589[] = "Microsoft Office/14.0 (Windows NT 6.1; Microsoft Outlook 14.0.7015; Pro)";
const char S2590[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S2591[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6112; ms-office; MSOffice 14)";
const char S2592[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2593[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8; Tablet PC 2.0; Microsoft Outlook 14.0.6136; ms-office; MSOffice 14)";
const char S2594[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2595[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.7101; ms-office; MSOffice 14)";
const char S2596[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S2597[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S2598[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S2599[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S2600[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S2601[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S2602[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S2603[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S2604[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S2605[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S2606[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Win64; x64; Trident/6.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S2607[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; GIL 3.5; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.7101; ms-office; MSOffice 14)";
const char S2608[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S2609[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S2610[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S2611[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S2612[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S2613[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S2614[] = "Mozilla/4.0 (compatible; MSIE 9.0; ADSSO; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S2615[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S2616[] = "Microsoft Office/14.0 (Windows NT 6.1; Microsoft Outlook 14.0.7106; Pro)";
const char S2617[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S2618[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S2619[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S2620[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S2621[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S2622[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S2623[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S2624[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S2625[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S2626[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S2627[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Win64; x64; Trident/6.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S2628[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S2629[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S2630[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S2631[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S2632[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S2633[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S2634[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S2635[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S2636[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S2637[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Media Center PC 6.0; Microsoft Outlook 14.0.7101; ms-office; MSOffice 14)";
const char S2638[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Media Center PC 5.0; SLCC1; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S2639[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S2640[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0)";
const char S2641[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0)";
const char S2642[] = "Windows-RSS-Platform/2.0 (MSIE 9.0; Windows NT 6.0)";
const char S2643[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Win64; x64; Trident/5.0)";
const char S2644[] = "Empty (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0)";
const char S2645[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1))";
const char S2646[] = "Empty (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0)";
const char S2647[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Verizon Windows Live)";
const char S2648[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.0)";
const char S2649[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; WindowsDownloads)";
const char S2650[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729))";
const char S2651[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Verizon Windows Live; Verizon Windows Live)";
const char S2652[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.0; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S2653[] = "Mozilla/4.0 (compatible; MSIE 9.0.8112.16421; Windows NT 6.0)";
const char S2654[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0)";
const char S2655[] = "Mozilla/5.0 ( ; MSIE 9.0; Windows NT 6.0; Trident/5.0)";
const char S2656[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC2; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; Zune 4.7)";
const char S2657[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; WindowsDownloads)";
const char S2658[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1); Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1))";
const char S2659[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; YDP))";
const char S2660[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Zune 4.0; Tablet PC 2.0; InfoPath.3; .NET4.0C; .NET4.0E)";
const char S2661[] = "Mozilla/5.0 (Windows; MSIE 9.0; WIndows NT 6.0; en-US)";
const char S2662[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S2663[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)";
const char S2664[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Vista; Trident/5.0)";
const char S2665[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; WindowsDownload)";
const char S2666[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Mozilla/4.0 (compatible; MSIE 6.0; Menara))";
const char S2667[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; SV1)";
const char S2668[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; SV1)";
const char S2669[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)";
const char S2670[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.0; Windows Live Writer 1.0)";
const char S2671[] = "Mozilla/5.0 ( ; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0)";
const char S2672[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/4.0; GTB7.4; InfoPath.3; SV1; .NET CLR 3.1.76908; WOW64; en-US)";
const char S2673[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; 360SE)";
const char S2674[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 9.0.8112.16421)";
const char S2675[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S2676[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; QQBrowser/7.1.5753.400)";
const char S2677[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)";
const char S2678[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)";
const char S2679[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/4.0; GTB7.4; InfoPath.3; SV1; .NET CLR 3.4.53360; WOW64; en-US)";
const char S2680[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; QQBrowser/7.2.7006.400)";
const char S2681[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7)";
const char S2682[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; SV1; SV1)";
const char S2683[] = "Mozilla/5.0 (Windows; U; MSIE 9.0; Windows NT 6.0; en-US) Trident/5.0";
const char S2684[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; SV1; SV1)";
const char S2685[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone4,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/6.0;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S2686[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.13 (KHTML, like Gecko) Mobile/10A5338d";
const char S2687[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A5355d";
const char S2688[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S2689[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A5376e";
const char S2690[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403";
const char S2691[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405";
const char S2692[] = "Mozilla/5.0 (iPhone; U; CPU iPhone 6_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPhone4,1/6.0)";
const char S2693[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 iPad2,1 BingWeb/3.03.1428.X";
const char S2694[] = "wp-iphone/3.1.4 (iPhone OS 6.0, iPhone) Mobile";
const char S2695[] = "Mozilla/5.0 (iPhone; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48 Instapaper/4.2.6 (+http://www.instapaper.com/iphone)";
const char S2696[] = "Mozilla/5.0 (iPhone4S; iPhone; U; CPU OS 6_0 like Mac OS X; en_US) com.google.GooglePlus/10352 (KHTML, like Gecko) Mobile/N94AP (gzip)";
const char S2697[] = "NAVER(inapp; cafe; 100; 2.7.1) Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName";
const char S2698[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A406";
const char S2699[] = "Mozilla/5.0 (iPhone5; iPhone; U; CPU OS 6_0 like Mac OS X; en_US) com.google.GooglePlus/10352 (KHTML, like Gecko) Mobile/N41AP (gzip)";
const char S2700[] = "Mozilla/5.0 (iPhone4; iPhone; U; CPU OS 6_0 like Mac OS X; en_US) com.google.GooglePlus/10352 (KHTML, like Gecko) Mobile/N90AP (gzip)";
const char S2701[] = "Mozilla/5.0 (iPhone; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10A405 Safari/7534.48 Instapaper/4.2.6 (+http://www.instapaper.com/iphone)";
const char S2702[] = "wp-iphone/3.1.4 (iPhone OS 6.0, iPad) Mobile";
const char S2703[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper";
const char S2704[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S2705[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523";
const char S2706[] = "Mozilla/5.0 (iPhone; CPU iPhone OS6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403";
const char S2707[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525";
const char S2708[] = "Mozilla/5.0 (iPhone4CDMA; iPhone; U; CPU OS 6_0 like Mac OS X; en_US) com.google.GooglePlus/10352 (KHTML, like Gecko) Mobile/N92AP (gzip)";
const char S2709[] = "Mozilla/5.0 (iPhone5CDMA; iPhone; U; CPU OS 6_0 like Mac OS X; en_US) com.google.GooglePlus/10352 (KHTML, like Gecko) Mobile/N42AP (gzip)";
const char S2710[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone4,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/6.0.1;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S2711[] = "Mozilla/5.0 (iPhone; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48 Instapaper/4.2.6 (+http://www.instapaper.com/iphone)";
const char S2712[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A407";
const char S2713[] = "Mozilla/5.0 (iPhone; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48 Instapaper/4.2.6 (+http://www.instapaper.com/iphone)";
const char S2714[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 iPad2,1 BingWeb/3.03.1428.X";
const char S2715[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8426";
const char S2716[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/8536.26.7 DiigoBrowser";
const char S2717[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper";
const char S2718[] = "Mozilla/5.0 (iPhone5; iPhone; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/11546 (KHTML, like Gecko) Mobile/N41AP (gzip)";
const char S2719[] = "Mozilla/5.0 (iPhone; CPU iPhone OS6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523";
const char S2720[] = "Mozilla/5.0 (iPhone5CDMA; iPhone; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/11546 (KHTML, like Gecko) Mobile/N42AP (gzip)";
const char S2721[] = "wp-iphone/3.3 (iPhone OS 6.0.1, iPhone) Mobile";
const char S2722[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3";
const char S2723[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 bline/1.03 (iPhone OS 6.0.1, iPhone)";
const char S2724[] = "Mozilla/5.0 (iPhone; U; CPU iPhone 6_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPad2,1/6.0.1)";
const char S2725[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550";
const char S2726[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPad3,4;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0.1;FBSS/2; FBCR/;FBID/tablet;FBLC/en_US]";
const char S2727[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8500";
const char S2728[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_2 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone5,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/6.0.2;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S2729[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551";
const char S2730[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S2731[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 bline/1.04 (iPhone OS 6.0.1, iPhone)";
const char S2732[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A505";
const char S2733[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser";
const char S2734[] = "Mozilla/5.0 (iPhone; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10A551 Safari/7534.48 Instapaper/4.2.7 (+http://www.instapaper.com/iphone)";
const char S2735[] = "NAVER(inapp; cafe; 100; 2.9.0) Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName";
const char S2736[] = "Mozilla/5.0 (iPhone4,1; iPhone; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N94AP (gzip)";
const char S2737[] = "Miu Ptt 1.9.2 (iPhone; iPhone OS 6.0.1; zh_TW)";
const char S2738[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari/8536.26.7 DiigoBrowser";
const char S2739[] = "Mozilla/5.0 (iPhone3,1; iPhone; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N90AP (gzip)";
const char S2740[] = "Mozilla/5.0 (iPhone3,3; iPhone; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N92AP (gzip)";
const char S2741[] = "Mozilla/5.0 (iPhone5,2; iPhone; U; CPU OS 6_0_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N42AP (gzip)";
const char S2742[] = "Mozilla/5.0 (iPhone5,1; iPhone; U; CPU OS 6_0_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N41AP (gzip)";
const char S2743[] = "NAVER(inapp; cafe; 100; 2.9.0) Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_2 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName";
const char S2744[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 bline/1.04 (iPhone OS 6.0.2, iPhone)";
const char S2745[] = "wp-iphone/3.3.1 (iPhone OS 6.0.1, iPad) Mobile";
const char S2746[] = "Miu Ptt 1.9.2 (iPhone; iPhone OS 6.0.2; zh_TW)";
const char S2747[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/8536.26.7 DiigoBrowser";
const char S2748[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/8536.26.7 DiigoBrowser";
const char S2749[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari IPhoneAppVersion/2.5.2 IPhoneWebkitWrapper BillingSupported";
const char S2750[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550 iPad2,5 BingWeb/3.03.1428.X";
const char S2751[] = "Miu Ptt 1.9.2 (iPhone; iPhone OS 6.0; zh_TW)";
const char S2752[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/9Z999";
const char S2753[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523(iPhone Vox)";
const char S2754[] = "wp-iphone/3.3 (iPhone OS 6.0.2, iPhone) Mobile";
const char S2755[] = "Mozilla/5.0 (iPhone2,1; iPhone; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N88AP (gzip)";
const char S2756[] = "Mozilla/5.0 (iPhone 4; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/8536.25";
const char S2757[] = "Vodafone/1.0/0Vodafone543/W08.40.SP1.V32.F3 Browser/Opera Mini/v4.2.0099-2010-04-14(0)";
const char S2758[] = "Z221; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.22250; U; en-US) Opera 9.50";
const char S2759[] = "Opera/9.60 (J2ME/MIDP; Opera Mini/4.2.13337/504; U; ru) Presto/2.2.0";
const char S2760[] = "Z331; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.22250; U; en-US) Opera 9.50";
const char S2761[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/4.2.13221/25.623; U; en) Presto/2.5.25 Version/10.54";
const char S2762[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/4.2.14320/19.916; U; pl) Presto/2.5.25";
const char S2763[] = "Opera/9.50 (J2ME/MIDP; Opera Mini/4.2.21071/28.3165; U; en)";
const char S2764[] = "0 Opera/9.80 (J2ME/MIDP; Opera Mini/4.2.23226/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S2765[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/4.2.23449/30.3214; U; en) Presto/2.8.119 Version/11.10";
const char S2766[] = "Opera/9.80 (MTK; Opera Mini/4.2.1255/31.1395; U; en) Presto/2.8.119 Version/11.10";
const char S2767[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0)";
const char S2768[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Win64; x64; Trident/6.0)";
const char S2769[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0)";
const char S2770[] = "Windows-RSS-Platform/2.0 (MSIE 10.0; Windows NT 6.2)";
const char S2771[] = "Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.2; SV1)";
const char S2772[] = "Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.2)";
const char S2773[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S2774[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; XBLWP7; ZuneWP7)";
const char S2775[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S2776[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; QQBrowser/7.1.5753.400)";
const char S2777[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; QQBrowser/7.1.5753.400)";
const char S2778[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; QQBrowser/7.2.7006.400)";
const char S2779[] = "Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.2; 360SE)";
const char S2780[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)";
const char S2781[] = "Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; SLCC1; .NET CLR 2.0.50727)";
const char S2782[] = "Mozilla/5.0 ( ; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0)";
const char S2783[] = "Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; SV1)";
const char S2784[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; ASU2JS)";
const char S2785[] = "Mozilla/4.0 (compatible; MSIE 10.0; Windows 8; UCWEB;)";
const char S2786[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2)";
const char S2787[] = "Mozilla/5.0 (compatible; WOW64; MSIE 10.0; Windows NT 6.2)";
const char S2788[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; HPNTDFJS; Zune 4.7)";
const char S2789[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B5105c";
const char S2790[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B5117b";
const char S2791[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B5126b";
const char S2792[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S2793[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141";
const char S2794[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/8536.26.7 DiigoBrowser";
const char S2795[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 [FBAN/FBIOS;FBAV/5.4.2;FBBV/X;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/6.1;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]";
const char S2796[] = "Mozilla/5.0 (iPad2,1; iPad; U; CPU OS 6_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/K93AP (gzip)";
const char S2797[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser";
const char S2798[] = "Mozilla/5.0 (iPad3,1; iPad; U; CPU OS 6_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/J1AP (gzip)";
const char S2799[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176";
const char S2800[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10B141 Safari/7534.48 Instapaper/4.2.7 (+http://www.instapaper.com/iphone)";
const char S2801[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 YaMobile/1.01";
const char S2802[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/7534.48.3";
const char S2803[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 6_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/6.1;FBSS/1; FBCR/unavailable;FBID/tablet;FBLC/en_US;FBSF/1.0]";
const char S2804[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 iPad2,1/K93AP Zite/2.1";
const char S2805[] = "Mozilla/5.0 (iPad; CPU OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B311";
const char S2806[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B147";
const char S2807[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S2808[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176";
const char S2809[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/8536.26.7 DiigoBrowser";
const char S2810[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 6_1_2 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/6.1.2;FBSS/1; FBCR/unavailable;FBID/tablet;FBLC/en_US;FBSF/1.0]";
const char S2811[] = "Mozilla/5.0 (iPad3,4; iPad; U; CPU OS 6_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/P101AP (gzip)";
const char S2812[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 YaMobile/1.10";
const char S2813[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146";
const char S2814[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 [FBAN/FBIOS;FBAV/5.5;FBBV/X;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/6.1.2;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBOP/0]";
const char S2815[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B147 [FBAN/FBIOS;FBAV/5.5;FBBV/X;FBDV/iPad3,5;FBMD/iPad;FBSN/iPhone OS;FBSV/6.1.2;FBSS/2; FBCR/AT&T;FBID/tablet;FBLC/en_US;FBOP/0]";
const char S2816[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/8536.25";
const char S2817[] = "Mozilla/5.0 (iPad2,1; iPad; U; CPU OS 6_1_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/K93AP (gzip)";
const char S2818[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser";
const char S2819[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48 Instapaper/4.2.7 (+http://www.instapaper.com/iphone)";
const char S2820[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/8536.25";
const char S2821[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B318a";
const char S2822[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 iPad2,1/K93AP Zite/2.2";
const char S2823[] = "Mozilla/5.0 (iPad3,1; iPad; U; CPU OS 6_1_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/J1AP (gzip)";
const char S2824[] = "Mozilla/5.0 (iPad3,4; iPad; U; CPU OS 6_1_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/P101AP (gzip)";
const char S2825[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3";
const char S2826[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S2827[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser";
const char S2828[] = "Mozilla/5.0 (iPad2,5; iPad; U; CPU OS 6_1_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/P105AP (gzip)";
const char S2829[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329";
const char S2830[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/8536.26.7 DiigoBrowser";
const char S2831[] = "Mozilla/5.0 (iPad2,2; iPad; U; CPU OS 6_1_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/K94AP (gzip)";
const char S2832[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48 Instapaper/4.2.7 (+http://www.instapaper.com/iphone)";
const char S2833[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser";
const char S2834[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B147 YaMobile/1.11";
const char S2835[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 iPad2,1/K93AP Zite/2.2";
const char S2836[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176";
const char S2837[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 YaMobile/1.11";
const char S2838[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 6_1_3 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/6.1.3;FBSS/1; FBCR/unavailable;FBID/tablet;FBLC/en_US;FBSF/1.0]";
const char S2839[] = "AppCake 3.8 (iPad; iPhone OS 6.1; en_US)";
const char S2840[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/9Z999";
const char S2841[] = "Miu Ptt 1.9.2 (iPad; iPhone OS 6.1.2; zh_TW)";
const char S2842[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B147 iPad3,6/P103AP Zite/2.2";
const char S2843[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 [FBAN/FBIOS;FBAV/6.1.1;FBBV/X;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/6.1.3;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBOP/1]";
const char S2844[] = "Miu Ptt 1.9.2 (iPad; iPhone OS 6.1; zh_TW)";
const char S2845[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/9Z999";
const char S2846[] = "Zeusmos 2.6.9 (iPad; iPhone OS 6.1.2; en_US)";
const char S2847[] = "Miu Ptt 1.9.2 (iPad; iPhone OS 6.1.3; zh_TW)";
const char S2848[] = "Mozilla/5.0 (iPad2,2; iPad; U; CPU OS 6_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/K94AP (gzip)";
const char S2849[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/528.16";
const char S2850[] = "Mozilla/5.0 (iPad2,5; iPad; U; CPU OS 6_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/P105AP (gzip)";
const char S2851[] = "Mozilla/5.0 (iPad2,5; iPad; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/P105AP (gzip)";
const char S2852[] = "Mozilla/5.0 (iPad3,1; iPad; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/J1AP (gzip)";
const char S2853[] = "QQ???HD 3.2 (iPad; iPhone OS 6.1.3; zh_CN)";
const char S2854[] = "Mozilla/5.0 (iPad2,1; iPad; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/K93AP (gzip)";
const char S2855[] = "Mozilla/5.0 (iPad3,4; iPad; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/P101AP (gzip)";
const char S2856[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329";
const char S2857[] = "Mozilla/5.0 (iPad2,2; iPad; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/K94AP (gzip)";
const char S2858[] = "Mozilla/5.0 (iPad3,3; iPad; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/J2aAP (gzip)";
const char S2859[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 bline/1.04 (iPhone OS 6.1.3, iPad)";
const char S2860[] = "Mozilla/5.0 (iPad3,6; iPad; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/P103AP (gzip)";
const char S2861[] = "Mozilla/5.0 (iPad3,2; iPad; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/J2AP (gzip)";
const char S2862[] = "Mozilla/5.0 (iPad2,4; iPad; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/K93aAP (gzip)";
const char S2863[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/525.20";
const char S2864[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/1.0 (KHTML, like Gecko) Mobile/10B329";
const char S2865[] = "Mozilla/5.0 (iPad2,3; iPad; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/K95AP (gzip)";
const char S2866[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/4.6 (KHTML, like Gecko) Mobile/10B329";
const char S2867[] = "Mozilla/5.0 (iPad3,5; iPad; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/P102AP (gzip)";
const char S2868[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser";
const char S2869[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/1.2 (KHTML, like Gecko) Mobile/10B329";
const char S2870[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) DLBrowser/1.0 Mobile/10B329 Safari/8536.25";
const char S2871[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B147 Safari/8536.26.7 DiigoBrowser";
const char S2872[] = "Mozilla/5.0 (iPad; CPU OS6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329";
const char S2873[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/4.3 (KHTML, like Gecko) Mobile/10B329";
const char S2874[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile Safari/10B329";
const char S2875[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729)";
const char S2876[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729)";
const char S2877[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7)";
const char S2878[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7)";
const char S2879[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727)";
const char S2880[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7)";
const char S2881[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3)";
const char S2882[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S2883[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2)";
const char S2884[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729)";
const char S2885[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; InfoPath.3)";
const char S2886[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Windows Live Messenger 15.4.3555.0308)";
const char S2887[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2)";
const char S2888[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Windows Live Messenger 15.4.3555.0308)";
const char S2889[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0)";
const char S2890[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Windows Live Messenger 16.4.3503.0728)";
const char S2891[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Windows Live Messenger 16.4.3505.0912)";
const char S2892[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0)";
const char S2893[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Zune 4.7)";
const char S2894[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Zune 4.7)";
const char S2895[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; ASU2JS)";
const char S2896[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3)";
const char S2897[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Tablet PC 2.0; ASU2JS)";
const char S2898[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)";
const char S2899[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Windows Live Messenger 16.4.3505.0912)";
const char S2900[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Windows Live Messenger 16.4.3505.0912)";
const char S2901[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 5.0; Media Center PC 6.0; SLCC1; SLCC2; Tablet PC 2.0)";
const char S2902[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 5.0; Media Center PC 6.0; SLCC1; SLCC2)";
const char S2903[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 6.0; SLCC1; SLCC2; Tablet PC 2.0)";
const char S2904[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Zune 4.7)";
const char S2905[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/4.0)";
const char S2906[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7)";
const char S2907[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7)";
const char S2908[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 6.0; SLCC1; SLCC2)";
const char S2909[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; SiteKiosk 8.7 Build 1435)";
const char S2910[] = "Opera/9.80 (Android; Opera Mini/7.5.33495/27.2338; U; en) Presto/2.8.119 480x800 Android";
const char S2911[] = "Opera/9.80 (Android; Opera Mini/7.5.33361/31.1475; U; en) Presto/2.8.119 Version/11.10";
const char S2912[] = "Mozilla/5.0(Windows; U; Windows NT 5.2; rv:1.9.2) Gecko/20100101 Firefox/3.6";
const char S2913[] = "Mozilla/5.0 Mozilla/5.0 (Windows; U; Windows NT 5.1; de; rv:1.9.2.13) Firefox/3.6.13";
const char S2914[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.12) Gecko/X Firefox/3.6.12\",\"Mozilla/5.0 (Windows; U; Windows NT 5.1; pl-PL; rv:1.8.1.24pre) Gecko/X K-Meleon/1.5.4\",\"Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/540.0 (KHTML,";
const char S2915[] = "Mozilla/6.0 (Windows; U; Windows NT 5.1; pl; rv:1.6.1.6) Gecko/X Firefox/3.6.6 (.NET CLR 3.1.36666)\"";
const char S2916[] = "Mozilla/10.0.2 (Windows; U; Windows NT 5.1;en-US;rv:1.9.2.12) Gecko/X Firefox/3.6.12";
const char S2917[] = "Mozilla/5.0+(Windows;+U;+Windows+NT+5.1;+de;+rv:1.9.2.16)+Gecko/X+Firefox/3.6.16";
const char S2918[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.15) Gecko/X Firefox/3.6.15 GTB7.1 WebMoney Advisor";
const char S2919[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; Win64; x64; de; rv:1.9.2.6) Gecko/X Firefox/3.6.6 (tete009 SSE2 PGO)";
const char S2920[] = "Mozilla/5.0(Windows NT5.1;rv2.0) Gecko/X Firefox/3.6.15";
const char S2921[] = "Mozilla/5.0 (Windows NT 5.1; es; rv:1.9.2.16) Gecko/X Firefox/3.6.16";
const char S2922[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; zh-CN; rv:1.9.2) Gecko/X Firefox/3.6 GTB6";
const char S2923[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)";
const char S2924[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179";
const char S2925[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone4,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/5.1;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S2926[] = "NaverCafe/1.0 Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName";
const char S2927[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone4,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/5.1.1;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S2928[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206";
const char S2929[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176";
const char S2930[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)";
const char S2931[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406";
const char S2932[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405";
const char S2933[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 iPad2,2 BingWeb/3.02.1641.X";
const char S2934[] = "Mozilla/5.0 (iPhone4; U; CPU iPhone OS 5_1 like Mac OS X; en_US) com.google.GooglePlus/5173 (KHTML, like Gecko) Mobile/N90AP (gzip)";
const char S2935[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper";
const char S2936[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 iPad1,1 BingWeb/3.03.1428.X";
const char S2937[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S2938[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208";
const char S2939[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9Z999";
const char S2940[] = "Mozilla/5.0 (; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206";
const char S2941[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper";
const char S2942[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405";
const char S2943[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/525.20";
const char S2944[] = "Mozilla/5.0 (iPhone4CDMA; iPhone; U; CPU OS 5_1_1 like Mac OS X; en_US) com.google.GooglePlus/8654 (KHTML, like Gecko) Mobile/N92AP (gzip)";
const char S2945[] = "Mozilla/5.0 (iPhone4S; iPhone; U; CPU OS 5_1_1 like Mac OS X; en_US) com.google.GooglePlus/10352 (KHTML, like Gecko) Mobile/N94AP (gzip)";
const char S2946[] = "Mozilla/5.0 (iPhone; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48 Instapaper/4.2.6 (+http://www.instapaper.com/iphone)";
const char S2947[] = "NAVER(inapp; cafe; 100; 2.7.1) Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName";
const char S2948[] = "Mozilla/5.0 (iPhone; U; CPU iPhone 5_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPhone4,1/5.1.1)";
const char S2949[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/534.46";
const char S2950[] = "Mozilla/5.0 (iPhone4; iPhone; U; CPU OS 5_1_1 like Mac OS X; en_US) com.google.GooglePlus/10352 (KHTML, like Gecko) Mobile/N90AP (gzip)";
const char S2951[] = "Mozilla/5.0 (iPhone; U; CPU iPhone 5_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPad1,1/5.1.1)";
const char S2952[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 bline/1.03 (iPhone OS 5.1.1, iPhone)";
const char S2953[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406";
const char S2954[] = "Miu Ptt 1.9.2 (iPhone; iPhone OS 5.1.1; zh_TW)";
const char S2955[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari IPhoneAppVersion/2.5.2 IPhoneWebkitWrapper BillingSupported";
const char S2956[] = "Miu Ptt 1.9.2 (iPhone; iPhone OS 5.1; zh_TW)";
const char S2957[] = "Facebook 4100.0 (iPhone; iPhone OS 5.1.1; en_AU)";
const char S2958[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206(iPhone Vox)";
const char S2959[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/8536.26.7 DiigoBrowser";
const char S2960[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 [FBAN/MessengerForiOS;FBAV/2.5;FBBV/X;FBDV/iPad1,1;FBMD/iPad;FBSN/iPhone OS;FBSV/5.1.1;FBSS/1; FBCR/;FBID/phone;FBLC/en_US;FBOP/1]";
const char S2961[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2962[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9300 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2963[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2964[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2965[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2966[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-I9300 Build/JRO03C)";
const char S2967[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9300 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2968[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2969[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2970[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2971[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2972[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2973[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-i9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2974[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I535 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2975[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; SAMSUNG GT-I9300/I9300XXDLIH Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2976[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2977[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9300 Build/JZO54K)";
const char S2978[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S2979[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9300 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S2980[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T999 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2981[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2982[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SCH-I535 Build/JRO03L)";
const char S2983[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I93000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2984[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2985[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SGH-T999 Build/JRO03L)";
const char S2986[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2987[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; SGH-T999 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2988[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; GT-I9300T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2989[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-I535 Build/JZO54K)";
const char S2990[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; GT-I9300-ORANGE/I9300XXDLJ4 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2991[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-T999V Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2992[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-I9300T Build/JRO03C)";
const char S2993[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; SCH-I939 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2994[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I9300/I9300XXELL5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2995[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-i9300 Build/IMM76D)";
const char S2996[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; i9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S2997[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9300 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S2998[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SGH-T999V Build/JRO03L)";
const char S2999[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-I9300-ORANGE/I9300XXELL1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3000[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; I9300i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3001[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; GT_I9300 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S3002[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3003[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I9300/I9300XXELLA Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3004[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I9300/I9300XXELL1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3005[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3006[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3007[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3008[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; SCH-I535 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3009[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; SCH-I939 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S3010[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I93000 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3011[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300-ORANGE/I9300XXELLA Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3012[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T999 Build/JZO54K)";
const char S3013[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T999 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3014[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-I9300T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3015[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9300T Build/JZO54K)";
const char S3016[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I9300/I9300XXEMR2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3017[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3018[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; SCH-I939D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3019[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S3020[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.9; GT-I9300 Build/IMM76D)";
const char S3021[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; SAMSUNG GT-I9300/I9300XXEMA2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3022[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30";
const char S3023[] = "Mozilla/5.0 (Linux; U; Android 4.1.5; en-us; GT-I9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S3024[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30";
const char S3025[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T999L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3026[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I939 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3027[] = "Mozilla/5.0 (Linux; U; Android 4.1.5; en-us; GT-I9300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3028[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; I9300i Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3029[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-l9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3030[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SCH-I535 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3031[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; i9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3032[] = "Mozilla/5.0 (Linux; U; Android 4.1.3; en-us; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3033[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-gb; GT_I9300 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S3034[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I939D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3035[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I939 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S3036[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-I9300-ORANGE/I9300XXELLC Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3037[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; SAMSUNG GT-I9300/I9300XXEMR7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3038[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-I9300-ORANGE/I9300XXEMR7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3039[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9300 Build/_miui.polimorph_ v1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3040[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; SGH-T999V Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3041[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; ar-eg; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3042[] = "Mozilla/5.0 (Linux; U; Android 4.1.4; en-us; GT-i9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S3043[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9300 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S3044[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-I939 Build/JZO54K)";
const char S3045[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-I93000 Build/JRO03C)";
const char S3046[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; es-es; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3047[] = "Mozilla/5.0 (Linux; U; Android 4.1.4; es-es; GT-i9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S3048[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; mini I9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S3049[] = "Mozilla/5.0 (Linux; U; Android 4.1.2_????_??????; he-il; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3050[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3051[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; I9300 Galaxy SIII Build/MocorDroid4.1.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S3052[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I9300/I9300XXEMRA Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3053[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I9300/I9300XXEMC2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3054[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SCH-I939 Build/JRO03C)";
const char S3055[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; I9300+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3056[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SGH-T999 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3057[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; fr-fr; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3058[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/By Rohit) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3059[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; SAMSUNG GT-I9300/I9300XXELLC Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3060[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; tr-tr; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3061[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; I9300 Build/IMM76D)";
const char S3062[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9300 Build/MocorDroid4.1.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S3063[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S3064[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T999L Build/JZO54K)";
const char S3065[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9300 Build/GT-I9300) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3066[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3067[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-ch; SAMSUNG GT-I9300/I9300XXELL4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3068[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SC-06D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3069[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SC-06D Build/JZO54K)";
const char S3070[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; it-it; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3071[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T999N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3072[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300-ORANGE/I9300XXEMF1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3073[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I9300/I9300XXEMF1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3074[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9300. Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S3075[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-T999V Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3076[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T999V Build/JZO54K)";
const char S3077[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I9300/I9300XXEMRD Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3078[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-ch; SAMSUNG GT-I9300/I9300XXEMF6 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3079[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; GT-I9300 Build/JRO03C-1.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3080[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T999N Build/JZO54K)";
const char S3081[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9300 Build/JZO54K) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 T5/2.0";
const char S3082[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=d2vzw;devm=SCH-I535;man=samsung;brw=Android;shwd=720x1280;shwa=720x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S3083[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I535 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S3084[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-I939D Build/JZO54K)";
const char S3085[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-I9300T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3086[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=d2tmo;devm=SGH-T999;man=samsung;brw=Android;shwd=720x1280;shwa=720x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S3087[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SGH-T999V Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3088[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9300 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S3089[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-; GT-I9300 Build/JZO54K) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 T5/2.0";
const char S3090[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/26.0.1410.12) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.12 Safari/537.31";
const char S3091[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/26.0.1410.19) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.19 Safari/537.31";
const char S3092[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/26.0.1410.28) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.28 Safari/537.31";
const char S3093[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/26.0.1410.33) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.33 Safari/537.31";
const char S3094[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/26.0.1386.0) AppleWebKit/537.27 (KHTML, like Gecko) Chrome/26.0.1386.0 Safari/537.27";
const char S3095[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/26.0.1410.40) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Safari/537.31";
const char S3096[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/26.0.1410.43) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/537.31";
const char S3097[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/26.0.1410.43) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/537.31";
const char S3098[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/26.0.1410.43) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/537.31";
const char S3099[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/26.0.1410.64; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S3100[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/26.0.1410.64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S3101[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/26.0.1410.64; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S3102[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/26.0.1410.64; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S3103[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/26.0.1410.64; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S3104[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/26.0.1410.64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S3105[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/26.0.1410.64; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S3106[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/26.0.1410.64; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S3107[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/26.0.1410.64)";
const char S3108[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/26.0.1410.64; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)";
const char S3109[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/26.0.1410.64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S3110[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/26.0.1410.64; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S3111[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/26.0.1410.64; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S3112[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/26.0.1410.64)";
const char S3113[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; GTB7.4; chromeframe/26.0.1410.64; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S3114[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; chromeframe/26.0.1410.64; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S3115[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/26.0.1410.64; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; .NET4.0C; .NET4.0E; Zune 4.7)";
const char S3116[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/26.0.1410.64; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S3117[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; chromeframe/26.0.1410.64; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S3118[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; SV1; chromeframe/26.0.1410.64; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S3119[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/26.0.1410.64; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S3120[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/26.0.1410.43; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322; .NET CLR 1.0.3705; Tablet PC 1.7; .NET4.0C; .NET4.0E; Zurich Financial Services; Zurich Financial Services)";
const char S3121[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; chromeframe/26.0.1410.64; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8; .NET4.0C; .NET4.0E)";
const char S3122[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/26.0.1410.67) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.67 Safari/537.31";
const char S3123[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; TmstmpExt)";
const char S3124[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; TmstmpExt)";
const char S3125[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; Mozilla/4.0 ; MSIE 6.0; Windows NT 5.1; SV1; ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; TmstmpExt)";
const char S3126[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; TmstmpExt)";
const char S3127[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; TmstmpExt)";
const char S3128[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7; TmstmpExt)";
const char S3129[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Media Center PC 5.0; SLCC1; TmstmpExt)";
const char S3130[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Media Center PC 5.0; SLCC1; Tablet PC 2.0; TmstmpExt)";
const char S3131[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; .NET4.0C; TmstmpExt)";
const char S3132[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7; TmstmpExt)";
const char S3133[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 ; MSIE 6.0; Windows NT 5.1; SV1; ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; TmstmpExt)";
const char S3134[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 ; MSIE 6.0; Windows NT 5.1; SV1; ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; TmstmpExt)";
const char S3135[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Tablet PC 2.0; TmstmpExt)";
const char S3136[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Avant Browser; TmstmpExt)";
const char S3137[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Zune 4.7; TmstmpExt)";
const char S3138[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7; TmstmpExt)";
const char S3139[] = "Outlook-Express/7.0 (MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; TmstmpExt)";
const char S3140[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7; TmstmpExt)";
const char S3141[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Avant Browser; TmstmpExt)";
const char S3142[] = "Outlook-Express/7.0 (MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; TmstmpExt)";
const char S3143[] = "Outlook-Express/7.0 (MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; TmstmpExt)";
const char S3144[] = "Outlook-Express/7.0 (MSIE 10.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; TmstmpExt)";
const char S3145[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/27.0.1453.15) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.15 Safari/537.36";
const char S3146[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/27.0.1453.56) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.56 Safari/537.36";
const char S3147[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/27.0.1453.47) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.47 Safari/537.36";
const char S3148[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/27.0.1453.73) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.73 Safari/537.36";
const char S3149[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/27.0.1453.81) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.81 Safari/537.36";
const char S3150[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/27.0.1453.65) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.65 Safari/537.36";
const char S3151[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/27.0.1453.93) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36";
const char S3152[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/27.0.1453.94) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36";
const char S3153[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/27.0.1453.94) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36";
const char S3154[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/27.0.1453.94) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36";
const char S3155[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/27.0.1453.94; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S3156[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/27.0.1453.110) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36";
const char S3157[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/27.0.1453.110) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36";
const char S3158[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/27.0.1453.110; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S3159[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/27.0.1453.110) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36";
const char S3160[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/27.0.1453.116; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; .NET4.0C; .NET4.0E; Zune 4.7)";
const char S3161[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/27.0.1453.116; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S3162[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/27.0.1453.116) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S3163[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/27.0.1453.116)";
const char S3164[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/27.0.1453.116; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S3165[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/27.0.1453.116; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S3166[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/27.0.1453.116) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S3167[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/27.0.1453.116; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S3168[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/27.0.1453.116; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S3169[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/27.0.1453.116) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S3170[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/27.0.1453.116; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)";
const char S3171[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/27.0.1453.116; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S3172[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/27.0.1453.116; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S3173[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/27.0.1453.116)";
const char S3174[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/27.0.1453.116; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 1.1.4322; InfoPath.1)";
const char S3175[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; chromeframe/27.0.1453.116; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S3176[] = "Mozilla/5.0(Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3177[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3;en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3178[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OSX 10_6_3; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3179[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; nl-nl) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3180[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; it-it) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3181[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; de-de) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3182[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3183[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; nb-no) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3184[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; it-it) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3185[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3186[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; zh-cn) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3187[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; sv-se) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3188[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; es-es) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3189[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; de-de) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3190[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-au) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3191[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3192[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; es-es) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3193[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3194[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ja-jp) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3195[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fr-fr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3196[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pt-br) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3197[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; tr-tr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3198[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ar) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3199[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ru-ru) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3200[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pl-pl) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3201[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3202[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pt-pt) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3203[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ru-ru) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3204[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; nl-nl) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3205[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3206[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fr-fr) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3207[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; zh-cn) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3208[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; it-it) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3209[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ja-jp) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3210[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-gb) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3211[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pl-pl) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3212[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; es-es) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3213[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; de-de) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3214[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-us) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3215[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; nb-no) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3216[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fi-fi) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3217[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3218[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pt-br) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3219[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ar) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3220[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fr-fr) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3221[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; th-th) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3222[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; sv-se) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3223[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; de-de) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3224[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fr-fr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3225[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; nl-nl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3226[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; fr-fr) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3227[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3228[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; sv-se) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3229[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ja-jp) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3230[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; cs-cz) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3231[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-gb) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3232[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ru-ru) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3233[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; it-it) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3234[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; es-es) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3235[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-ca) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3236[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pt-br) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3237[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; zh-tw) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3238[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pl-pl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3239[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nl-nl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3240[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ar) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3241[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fr-fr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3242[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3243[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pt-pt) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3244[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; nb-no) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3245[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ko-kr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3246[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; da-dk) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3247[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3248[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; zh-cn) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3249[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; es-es) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3250[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fi-fi) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3251[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-au) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3252[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; th-th) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3253[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; he-il) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3254[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; es-xl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3255[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3256[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; tr-tr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3257[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; fr-fr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3258[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; is-is) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3259[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; nl-nl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3260[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; de-de) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3261[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ru-ru) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3262[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; sv-se) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3263[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fa-ir) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3264[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; es-es) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3265[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; es-es) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3266[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-gb) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3267[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; zh-tw) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3268[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; de-de) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3269[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; nb-no) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3270[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; zh-cn) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3271[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-au) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3272[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ja-jp) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3273[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; da-dk) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3274[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; fi-fi) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3275[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; pt-br) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3276[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3277[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; tr-tr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3278[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; th-th) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3279[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; it-it) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3280[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; nl-nl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3281[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; it-it) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3282[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-gb) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3283[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ko-kr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3284[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3285[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ko-kr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3286[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; fr-fr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3287[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; pt-br) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3288[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ru-ru) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3289[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ja-jp) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3290[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; lt-lt) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3291[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; zh-tw) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3292[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko, Safari/6533.19.4) ADM/722";
const char S3293[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; es-es) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3294[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; pt-pt) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3295[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; tr-tr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3296[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; da-dk) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3297[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; sv-se) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3298[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; fi-fi) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3299[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-au) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3300[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; fa-ir) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3301[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; it-it) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3302[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; de-ch) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3303[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3304[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; cs-cz) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3305[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fr-fr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3306[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; nl-nl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3307[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; nb-no) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3308[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3309[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; de-de) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3310[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; es-es) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3311[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; vi-vn) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3312[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ru-ru) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3313[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; el-gr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3314[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; es-es) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3315[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; es-xl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3316[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-gb) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3317[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; sv-se) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3318[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pt-br) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3319[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; da-dk) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3320[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-au) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3321[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ru-ru) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3322[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; nl-nl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3323[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; nl-nl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3324[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-gb) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3325[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pl-pl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3326[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pt-pt) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3327[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; zh-cn) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3328[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; is-is) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3329[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; zh-tw) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3330[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; it-it) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3331[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; es-es) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3332[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; de-de) AppleWebKit/534.15+ (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3333[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ja-jp) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3334[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; it-it) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3335[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.19.4";
const char S3336[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pt-br) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3337[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fi-fi) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3338[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ko-kr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3339[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-ca) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3340[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; he-il) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3341[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; de-de) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3342[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3343[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; th-th) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3344[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ar) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3345[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; tr-tr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3346[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; de-ch) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3347[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; cs-cz) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3348[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; nb-no) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3349[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; nb-no) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3350[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-ca) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3351[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-fr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3352[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; de-de) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3353[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ja-jp) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3354[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fi-fi) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3355[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; sl-si) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3356[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; af-za) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3357[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-fr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3358[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; da-dk) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3359[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; sk-sk) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3360[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ko-kr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3361[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; sv-se) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3362[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; lt-lt) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3363[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; zh-cn) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3364[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pt-br) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3365[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-au) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3366[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; es-es) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3367[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3368[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3369[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; tr-tr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3370[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fr-fr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3371[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; de-de) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3372[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; nl-nl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3373[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; cs-cz) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3374[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; de-de) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3375[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3376[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; zh-cn) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3377[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ja-jp) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3378[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-gb) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3379[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; el-gr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3380[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-au) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3381[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; sk-sk) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3382[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-ca) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3383[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pt-br) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3384[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fr-fr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3385[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; pt-pt) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3386[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; da-dk) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3387[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; zh-tw) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3388[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; pl-pl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3389[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; es-es) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3390[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; sv-se) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3391[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; de-de) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3392[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ru-ru) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3393[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; is-is) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3394[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; nb-no) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3395[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; pt-br) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3396[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ko-kr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3397[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; es-xl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3398[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fi-fi) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3399[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; sv-se) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3400[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3401[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.20.27";
const char S3402[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; th-th) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3403[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3404[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; da-dk) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3405[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3406[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; el-gr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3407[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3408[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3409[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; is-is) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3410[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-gb) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3411[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-au) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3412[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ja-jp) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3413[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3414[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-ca) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3415[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; pt-br) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3416[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ko-kr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3417[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; zh-cn) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3418[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3419[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; zh-tw) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3420[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; sk-sk) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3421[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; lt-lt) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3422[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3423[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; pt-pt) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3424[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3425[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; cs-cz) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3426[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3427[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fi-fi) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3428[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3429[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; pl-pl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3430[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; nb-no) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3431[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.21.1";
const char S3432[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; tr-tr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3433[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fa-ir) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3434[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3435[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; sl-si) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3436[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3437[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; es-xl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3438[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; vi-vn) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3439[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; it-it) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3440[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ja-jp) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3441[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ar) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3442[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; he-il) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3443[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3444[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; nl-nl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3445[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; es-es) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3446[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; sv-se) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3447[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-fr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3448[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pl-pl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3449[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; sv-se) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3450[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fr-ca) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3451[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pt-br) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3452[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; it-it) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3453[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; zh-cn) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3454[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-gb) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3455[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3456[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; da-dk) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3457[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fi-fi) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3458[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; th-th) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3459[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; th-th) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3460[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; tr-tr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3461[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fr-ch) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3462[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; de-de) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3463[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; nb-no) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3464[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3465[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ru-ru) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3466[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ko-kr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3467[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; nn-no) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3468[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; hu-hu) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3469[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; zh-tw) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3470[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-ca) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3471[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3472[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ro-ro) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3473[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-au) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3474[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; tr-tr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3475[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; lt-lt) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3476[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; de-ch) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3477[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; nl-nl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3478[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/534.16+ (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3479[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3480[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; sv-se) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3481[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3482[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-gb) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3483[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3484[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3485[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3486[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nb-no) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3487[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; hu-hu) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3488[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fi-fi) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3489[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; is-is) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3490[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pl-pl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3491[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ja-jp) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3492[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pt-br) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3493[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sk-sk) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3494[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.21.1";
const char S3495[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ko-kr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3496[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; lt-lt) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3497[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; cs-cz) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3498[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh-tw) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3499[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; af-za) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3500[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; tr-tr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3501[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3502[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; es-es) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3503[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; id) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3504[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es-xl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3505[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3506[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr-ca) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3507[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; da-dk) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3508[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pt-pt) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3509[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3510[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-ca) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3511[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3512[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-au) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3513[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; el-gr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3514[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3515[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh-cn) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3516[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ru-ru) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3517[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; vi-vn) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3518[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3519[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-ch) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3520[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fa-ir) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3521[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-at) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3522[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; th-th) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3523[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sv-se) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3524[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; et-ee) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3525[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3526[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ar) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3527[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-gb) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3528[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; tl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3529[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; hr-hr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3530[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3531[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3532[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; la) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3533[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sl-si) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3534[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; he-il) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3535[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; uk-ua) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3536[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr-ch) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3537[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; USCCADT71US; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3538[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; bg-bg) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3539[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_US) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3540[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ja-jp) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3541[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pt-br) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3542[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fr-fr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3543[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; sv-se) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3544[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; zh-tw) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3545[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ru-ru) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3546[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; da-dk) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3547[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ko-kr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3548[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; fr-fr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3549[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pt-pt) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3550[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-ca) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3551[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-au) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3552[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fi-fi) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3553[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; da-dk) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3554[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; zh-tw) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3555[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ko-kr) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3556[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3557[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; tr-tr) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3558[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; de-ch) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3559[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; el-gr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3560[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; vi-vn) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3561[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; sk-sk) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3562[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.18.5";
const char S3563[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3564[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; nl-nl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3565[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; hu-hu) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3566[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; it-it) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3567[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3568[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; de-de) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3569[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ru-ru) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3570[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; fr-fr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3571[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; es-es) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3572[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; cs-cz) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3573[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; el-gr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3574[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-ca) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3575[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; pl-pl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3576[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; de-ch) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3577[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; zh-cn) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3578[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-ca) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3579[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; hu-hu) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3580[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; is-is) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3581[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; th-th) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3582[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; pl-pl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3583[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; pt-pt) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3584[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ar) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3585[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; vi-vn) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3586[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ro-ro) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3587[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ru-ru) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3588[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ar) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3589[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; nb-no) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3590[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pt-br) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3591[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; sv-se) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3592[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-au) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3593[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; hu-hu) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3594[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ca-es) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3595[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; hr-hr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3596[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; el-gr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3597[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3598[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; vi-vn) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3599[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pl-pl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3600[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3601[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fa-ir) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3602[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; bg-bg) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3603[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; es-xl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3604[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; cs-cz) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3605[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pt-pt) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3606[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; zh-tw) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3607[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; th-th) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3608[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fa-ir) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3609[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-au) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3610[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; cs-cz) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3611[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fr-ca) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3612[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; it-it) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3613[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pt-pt) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3614[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; el-gr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3615[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-ca) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3616[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.20.27";
const char S3617[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ca-es) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3618[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ru-ru) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3619[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ar) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3620[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; es-es) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3621[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; de-de) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3622[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; af-za) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3623[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; el-gr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3624[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ar) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3625[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; de-ch) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3626[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; sl-si) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3627[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; he-il) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3628[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ca-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3629[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-gb) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3630[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; af-za) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3631[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; de-at) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3632[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; vi-vn) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3633[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-gb) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3634[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; lv-lv) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3635[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3636[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ja-jp) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3637[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; uk-ua) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3638[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fa-ir) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3639[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ru-ru) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3640[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; la) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3641[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3642[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3643[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pt-br) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3644[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; tl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3645[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; nb-no) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3646[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3647[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ca-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3648[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nn-no) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3649[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3650[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ro-ro) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3651[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; hy-am) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3652[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3653[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3654[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; yi) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3655[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.17.8 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.17.8";
const char S3656[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fr-ch) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3657[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ca-es) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3658[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; es-xl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3659[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; lt-lt) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3660[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; af-za) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3661[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; sr-cs) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3662[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pcdadr6350; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3663[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; zh-cn) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3664[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nl-nl) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3665[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ca-es) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3666[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; de-de) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3667[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ja-jp) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3668[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; sk-sk) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3669[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; hr-hr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3670[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; uk-ua) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3671[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-ca) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3672[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; bg-bg) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3673[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; he-il) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3674[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; da-dk) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3675[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3676[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; mn-mong) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3677[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ur-pk) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3678[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; mk-mk) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3679[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fo-fo) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3680[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; mn-mong) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3681[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; de-de) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3682[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3683[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl-be) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3684[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ta) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3685[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; af-za) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3686[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3687[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ko-kr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3688[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-ca) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3689[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; sk-sk) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3690[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fr-ca) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3691[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; it-it) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3692[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; el-gr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3693[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; eu) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3694[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3695[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_GB) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3696[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; Flyer_P512; en-hk) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3697[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ro-ro) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3698[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3699[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; hu-hu) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3700[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; hy-am) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3701[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; hr-hr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3702[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ga-ie) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3703[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; zh-tw) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3704[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; hy-am) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3705[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; lv-lv) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3706[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; lt-lt) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3707[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; th-th) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3708[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; id) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3709[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ms) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3710[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; de-at) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3711[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ru-ru) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3712[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3713[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; el-gr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3714[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; nl-nl) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3715[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ca-es) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3716[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3717[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fa-ir) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3718[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; hu-hu) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3719[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; id) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3720[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; is-is) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3721[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; hr-hr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3722[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; id) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3723[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; lv-lv) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3724[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; lt-lt) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3725[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; sl-si) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3726[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; es-es) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3727[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; sv-se) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3728[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; de-ch) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3729[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; he-il) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3730[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; fr-ca) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3731[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; zh-cn) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3732[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; es-xl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3733[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ca-es) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3734[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3735[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ro-ro) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3736[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; yi) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3737[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; pt-br) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3738[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko, Safari/6533.18.5) ADM/722";
const char S3739[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; la) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3740[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; mk-mk) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3741[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; lv-lv) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3742[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-ch) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3743[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; sk-sk) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3744[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ja-jp) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3745[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-au) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3746[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ru-ru) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3747[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; he-il) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3748[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ga-ie) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3749[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ro-ro) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3750[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; lv-lv) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3751[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fr-ch) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3752[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ca-es) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3753[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3754[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pt-br) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3755[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; la) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3756[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ms) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3757[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; yi) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3758[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; cs-cz) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3759[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; pt-br) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3760[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pt-pt) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3761[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; it-it) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3762[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; id) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3763[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3764[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.21.1";
const char S3765[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ja-jp) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3766[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3767[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sw) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3768[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ms) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3769[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3770[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3771[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.21.1";
const char S3772[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sr-cs) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3773[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; so) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3774[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ps) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3775[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; iu-ca) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3776[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-au) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3777[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.21.1";
const char S3778[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; hi-in) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3779[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/5.0.6 Safari/533.22.3";
const char S3780[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; bo) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3781[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl_NL) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3782[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; Sprint APX515CKT; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3783[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-gb) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3784[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; he-il) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3785[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; hr-hr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3786[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; TB07FTA; pt-br) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3787[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; af-za) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3788[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; af-za) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3789[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3790[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_ca) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3791[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.16 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.16";
const char S3792[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; zh-tw) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3793[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; th-th) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3794[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; af-za) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3795[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; it-it) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3796[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3797[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; es-es) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3798[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3799[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3800[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; th-th) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3801[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; zh-cn) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3802[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; uk-ua) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3803[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; sv-se) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3804[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; et-ee) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3805[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3806[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3807[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; he-il) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3808[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; uk-ua) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3809[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ca-es) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3810[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de_DE) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3811[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; TB07STA; pt-br) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3812[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; sv-se) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3813[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_PH) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3814[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; sv-se) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3815[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; es-xl) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3816[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; is-is) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3817[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; it-it) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3818[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; pt-br) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3819[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pl_PL) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3820[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fr-fr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3821[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; hu-hu) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3822[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh_cn) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3823[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ps) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3824[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nb-no) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3825[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; af-za) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3826[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_AU) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3827[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; he_il) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3828[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3829[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fi_fi) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3830[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; bq DaVinci; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3831[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; hu-hu) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3832[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fr-ca) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3833[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sq) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3834[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ar) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3835[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es_es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3836[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3837[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ro-ro) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3838[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; bg-bg) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3839[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sv_se) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3840[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_SG) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3841[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_za) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3842[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ja-jp) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3843[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-au) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3844[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_NZ) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3845[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nb-no) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3846[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3847[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it_it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3848[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; sk-sk) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3849[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; hr-hr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3850[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.18.5";
const char S3851[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fi-fi) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3852[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pt_BR) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3853[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nl-nl) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3854[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_IE) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3855[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; sr-cs) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3856[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; fa-ir) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3857[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; th-th) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3858[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-gb) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3859[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3860[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; he-il) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3861[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; de-de) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3862[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.5+ (KHTML, like Gecko) Version/5.0.5 Safari/533.5+";
const char S3863[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; da_DK) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3864[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ka-ge) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3865[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ru_RU) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3866[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; th-th) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3867[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; et-ee) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3868[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_HK) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3869[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3870[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh_HK) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3871[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; bg-bg) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3872[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; fr-fr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3873[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; sv-se) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3874[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; cs-cz) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3875[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ga-ie) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3876[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; sl-si) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3877[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; et-ee) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3878[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_IN) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3879[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3880[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3881[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; bg-bg) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3882[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; et-ee) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3883[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; sl-si) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3884[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; de-ch) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3885[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ST070I00GB; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3886[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; da-dk) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3887[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; uk-ua) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3888[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr_FR) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3889[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; cy) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3890[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-gb) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3891[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3892[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; cs-cz) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3893[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; az-latn) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3894[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ar) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3895[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; zh-tw) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3896[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nb-no) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3897[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; uk-ua) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3898[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; fr-ca) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3899[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; it-it) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3900[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ru-ru) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3901[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; nl-nl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3902[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ru-ru) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3903[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; zh-cn) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3904[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; af-za) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3905[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ko_kr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3906[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; tr-tr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3907[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ug) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3908[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; th_TH) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3909[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1";
const char S3910[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ar) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3911[] = "Mozilla/5.0 (Macintosh; I; Intel Mac OS X 10_6_7; ru-ru) AppleWebKit/534.31+ (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3912[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_gb) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.5 Safari/6534.46";
const char S3913[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_ca) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.5 Safari/6534.46";
const char S3914[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_us) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.5 Safari/6534.46";
const char S3915[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl_nl) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.5 Safari/6534.46";
const char S3916[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_au) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.5 Safari/6534.46";
const char S3917[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de_de) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.5 Safari/6534.46";
const char S3918[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S3919[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_au) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S3920[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_gb) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S3921[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl_nl) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S3922[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_ca) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S3923[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; lt-lt) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3924[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; hr-hr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3925[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; vi-vn) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3926[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; el-gr) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3927[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; it-it) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3928[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fa-ir) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3929[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; et-ee) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3930[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-ca) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3931[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; cs-cz) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3932[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; de-at) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3933[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; tl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3934[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; yi) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3935[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; hu-hu) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3936[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fa-ir) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3937[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; nn-no) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3938[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3939[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; is-is) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3940[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; es-xl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3941[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ca-es) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3942[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-ca) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3943[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; nb-no) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3944[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; vi-vn) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3945[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; zh-cn) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3946[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; sl-si) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3947[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; sk-sk) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3948[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.5+ (KHTML, like Gecko) Version/5.0.4 Safari/533.5+";
const char S3949[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; id) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3950[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ar) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3951[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-gb) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3952[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; cy) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3953[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ms) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3954[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; nb-no) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3955[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.21.1";
const char S3956[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; km) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3957[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ne-np) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3958[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3959[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh-cn) AppleWebKit/533.21.1 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.21.1";
const char S3960[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; kk) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3961[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3962[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es_MX) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3963[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; tr_TR) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3964[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ja_JP) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3965[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_sg) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S3966[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nb_no) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S3967[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_us) AppleWebKit/6533.17.9 (KHTML, like Gecko) Version/5.0.5 Safari/6533.17.9";
const char S3968[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ru_ru) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S3969[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.5 Safari/533.17.9";
const char S3970[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh_cn) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S3971[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr_fr) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S3972[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ADR6400L 4G; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3973[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it_it) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S3974[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; YPY_07FTA; pt-br) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3975[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de_de) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S3976[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_za) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S3977[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sv_se) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S3978[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S3979[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; vi-vn) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3980[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; sl-si) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3981[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S3982[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3983[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; is-is) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S3984[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fi-fi) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3985[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; zh-cn) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3986[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ru-ru) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3987[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/6533.19.4 (KHTML, like Gecko) Version/5.0 Safari/6533.19.4 FOH:R032";
const char S3988[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ur-pk) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S3989[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pl-pl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S3990[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S3991[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; mn-mong) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S3992[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es_es) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S3993[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_ph) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S3994[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; he_il) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S3995[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; da_dk) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S3996[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_nz) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S3997[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; id_id) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S3998[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fi_fi) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S3999[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; tr_tr) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S4000[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_in) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S4001[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh_hk) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S4002[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; th_th) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S4003[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pl_pl) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S4004[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it_it) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4005[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_hk) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S4006[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_gb) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4007[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_ca) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4008[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de_ch) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S4009[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de_de) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4010[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_au) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4011[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; tl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S4012[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; et-ee) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S4013[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; hr-hr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S4014[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; tl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S4015[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S4016[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; cs-cz) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S4017[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ru-ru) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S4018[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ay) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S4019[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.21.1";
const char S4020[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ja_jp) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S4021[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pt_br) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S4022[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl_nl) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4023[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_in) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4024[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es_es) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4025[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; el_GR) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S4026[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sv_se) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4027[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; de-ch) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S4028[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-au) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S4029[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; lt-lt) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S4030[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; sq) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S4031[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ro-ro) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S4032[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; tl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S4033[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; zh-cn) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S4034[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; az-latn) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S4035[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ug) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S4036[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; iu-ca) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S4037[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-ca) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S4038[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sv-se) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S4039[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; gl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S4040[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pt-pt) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S4041[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; de-ch) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S4042[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de_CH) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S4043[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh_TW) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S4044[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; sl-si) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S4045[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-au) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S4046[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ar_SA) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S4047[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de_at) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S4048[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_ie) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S4049[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ar_sa) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S4050[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_nz) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4051[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ru_ru) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4052[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; he_il) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4053[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fi_fi) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4054[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pl_pl) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4055[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_sg) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4056[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; da_dk) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4057[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_za) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4058[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr_fr) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4059[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5 Safari/533.16";
const char S4060[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ms_my) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S4061[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh_cn) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4062[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_ph) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4063[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nb_no) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4064[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_hk) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4065[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pt_br) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4066[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de_ch) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4067[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh_tw) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S4068[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_gb) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.5 Safari/533.17.9";
const char S4069[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_ie) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4070[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; tr_tr) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4071[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; th_th) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4072[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh_hk) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4073[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; da-dk) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S4074[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10\\_6\\_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S4075[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de_at) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4076[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh_cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.5 Safari/533.17.9";
const char S4077[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ja_jp) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4078[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pt_pt) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4079[] = "Mozilla/5.0 (AMacintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S4080[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; YPY_07STA; pt-br) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S4081[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr_ca) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4082[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; vi_vn) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4083[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_us) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.5 Safari/6536.26";
const char S4084[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es_mx) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S4085[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; cs_cz) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4086[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S4087[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; vi_vn) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S4088[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; id_id) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4089[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es_mx) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4090[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1068; en-US) AppleWebKit/533.21.1 (KHTML like Gecko) Version/5.0.5 Safari/533.21.1";
const char S4091[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1065; en-US) AppleWebKit/533.18.1 (KHTML like Gecko) Version/5.0.2 Safari/533.18.5";
const char S4092[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1064; en-US) AppleWebKit/533.17.8 (KHTML like Gecko) Version/5.0.1 Safari/533.17.8";
const char S4093[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1067; en-US) AppleWebKit/533.20.25 (KHTML like Gecko) Version/5.0.4 Safari/533.20.27";
const char S4094[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1063; en-US) AppleWebKit/533.16 (KHTML like Gecko) Version/5.0 Safari/533.16";
const char S4095[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1066; haw-US) AppleWebKit/533.20.25 (KHTML like Gecko) Version/5.0.4 Safari/533.20.27";
const char S4096[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sv-se) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S4097[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1064; en-US) AppleWebKit/533.18.1 (KHTML like Gecko) Version/5.0.2 Safari/533.18.5";
const char S4098[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl_be) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4099[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ms_my) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4100[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46";
const char S4101[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; CA907AAC0G; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S4102[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ko_kr) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4103[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/531.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/531.18.5";
const char S4104[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ar_sa) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4105[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; az-latn) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S4106[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; br) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S4107[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fi-fi) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S4108[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; de-DE) AppleWebKit/534.16+ (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S4109[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; hu_hu) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4110[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh_tw) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4111[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ro_ro) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4112[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; el_gr) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4113[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es_us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4114[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/534.48 (KHTML, like Gecko) Version/5.0.6 Safari/533.22.3;u=test6";
const char S4115[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; hi-in) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S4116[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; hr_hr) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4117[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S4118[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fr-FR) AppleWebKit/534.16+ (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S4119[] = "Snapito! 1.0 (CutyCapt) Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; da-dk) AppleWebKit/533.211 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S4120[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sa) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S4121[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_us) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/5.0.5 Safari/537.51.1";
const char S4122[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_gb) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/5.0.5 Safari/537.51.1";
const char S4123[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it_it) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/5.0.5 Safari/537.51.1";
const char S4124[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_ca) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/5.0.5 Safari/537.51.1";
const char S4125[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_au) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/5.0.5 Safari/537.51.1";
const char S4126[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de_de) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/5.0.5 Safari/537.51.1";
const char S4127[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36";
const char S4128[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36";
const char S4129[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.1.0.0 Safari/537.36";
const char S4130[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.1.0.0 Safari/537.36";
const char S4131[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S4132[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S4133[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe)";
const char S4134[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe)";
const char S4135[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S4136[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S4137[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.2.0.0 Safari/537.36";
const char S4138[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.2.0.0 Safari/537.36";
const char S4139[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/534.7 ZemanaAID/FFFF0096";
const char S4140[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36 AppEngine-Google; (+http://code.google.com/appengine; appid: s~optimizelyedit-hrd)";
const char S4141[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36 AppEngine-Google; (+http://code.google.com/appengine; appid: s~alexX)";
const char S4142[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36 AppEngine-Google; (+http://code.google.com/appengine; appid: s~jiangpeili)";
const char S4143[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.11.1453.94 Safari/537.36 QIPSurf";
const char S4144[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.11.1453.94 Safari/537.36 QIPSurf";
const char S4145[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/28.0.1500.20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.20 Safari/537.36";
const char S4146[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/28.0.1500.29) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.29 Safari/537.36";
const char S4147[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/28.0.1500.37) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.37 Safari/537.36";
const char S4148[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/28.0.1500.44) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.44 Safari/537.36";
const char S4149[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/28.0.1500.52) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36";
const char S4150[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/28.0.1500.63) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36";
const char S4151[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/28.0.1500.68) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.68 Safari/537.36";
const char S4152[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/28.0.1500.71) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36";
const char S4153[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/28.0.1500.70) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.70 Safari/537.36";
const char S4154[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/28.0.1500.71) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36";
const char S4155[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/28.0.1500.72) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36";
const char S4156[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/28.0.1500.71) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36";
const char S4157[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/28.0.1500.71; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 1.1.4322; InfoPath.1)";
const char S4158[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/28.0.1500.72) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36";
const char S4159[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/28.0.1500.72) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36";
const char S4160[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/28.0.1500.95) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S4161[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/28.0.1500.95; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4162[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/28.0.1500.95; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4163[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/28.0.1500.95; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S4164[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/28.0.1500.95) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S4165[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/28.0.1500.95; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; .NET4.0C; .NET4.0E; Zune 4.7)";
const char S4166[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/28.0.1500.95; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4167[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/28.0.1500.95; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4168[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; GTB7.5; chromeframe/28.0.1500.95; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4169[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/28.0.1500.95; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4170[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/28.0.1500.95)";
const char S4171[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/28.0.1500.95)";
const char S4172[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/28.0.1500.95; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4173[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.5; chromeframe/28.0.1500.95; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4174[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/28.0.1500.95) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S4175[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/28.0.1500.95; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)";
const char S4176[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; chromeframe/28.0.1500.71; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8; InfoPath.3; .NET4.0C)";
const char S4177[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; de; rv:1.9.2.8) Gecko/X Firefox 3.6.8";
const char S4178[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; ru; rv:1.9.2.28) Gecko/X Firefox/3.6.28 sputnik 2.5.2.8";
const char S4179[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.13) Gecko/X Firefox/3.6.13 WinNT-A8I 05.01.2011";
const char S4180[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; ru; rv:1.9.2) Gecko/X Firefox/3.6	Firefox 3.6";
const char S4181[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2) Gecko/X Firefox/3.6 MSIE 7.0";
const char S4182[] = "Mozilla/5.0 (Windows NT 6.1; rv:1.9.2.8) Gecko/X Firefox/3.6.8";
const char S4183[] = "Mozilla/4.0 (compatible; U; Windows NT 6.1; it; rv:1.9.2.8; Gecko/X AskTbADAP/3.9.1.14019 Firefox/3.6.8)";
const char S4184[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/23.0.1271.17) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.17 Safari/537.11";
const char S4185[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/23.0.1271.10) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.10 Safari/537.11";
const char S4186[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/23.0.1271.40) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.40 Safari/537.11";
const char S4187[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/23.0.1271.52) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.52 Safari/537.11";
const char S4188[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/23.0.1271.26) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.26 Safari/537.11";
const char S4189[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/23.0.1271.60) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.60 Safari/537.11";
const char S4190[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/23.0.1271.64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S4191[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/23.0.1271.64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S4192[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/23.0.1271.64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S4193[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/23.0.1271.91) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.91 Safari/537.11";
const char S4194[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/23.0.1271.91) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.91 Safari/537.11";
const char S4195[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/23.0.1271.95) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11";
const char S4196[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/23.0.1271.95) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11";
const char S4197[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/23.0.1271.97) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S4198[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/23.0.1271.97; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4199[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/23.0.1271.97; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4200[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/23.0.1271.97)";
const char S4201[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/23.0.1271.97; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4202[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/23.0.1271.97; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4203[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/23.0.1271.97; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4204[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/23.0.1271.97; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4205[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/23.0.1271.97; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4206[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/23.0.1271.97; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)";
const char S4207[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/23.0.1271.97; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S4208[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/23.0.1271.97) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S4209[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; chromeframe/23.0.1271.97; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8; .NET4.0C; .NET4.0E)";
const char S4210[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/23.0.1271.97) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S4211[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/23.0.1271.97; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4212[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/23.0.1271.97)";
const char S4213[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/23.0.1271.95) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11";
const char S4214[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/23.0.1271.97; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4215[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/23.0.1271.97; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)";
const char S4216[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/23.0.1271.97; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4217[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; SV1; chromeframe/23.0.1271.95; .NET CLR 1.1.4322; .NET CLR 2.0.50727)";
const char S4218[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML like Gecko) Chrome/28.0.1469.0 Safari/537.36";
const char S4219[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36 AppEngine-Google; (+http://code.google.com/appengine; appid: s~optimizelyedit-hrd)";
const char S4220[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S4221[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S4222[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe)";
const char S4223[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S4224[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe)";
const char S4225[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36; campus-TU";
const char S4226[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.1.0.0 Safari/537.36";
const char S4227[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.1.0.0 Safari/537.36";
const char S4228[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/534.7 ZemanaAID/FFFF0096";
const char S4229[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S4230[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe),gzip(gfe)";
const char S4231[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe),gzip(gfe)";
const char S4232[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; en-US;) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S4233[] = "Mozilla/5.0+(Windows+NT+6.1;+WOW64)+AppleWebKit/537.36+(KHTML,+like+Gecko)+Chrome/28.0.1500.95+Safari/537.36";
const char S4234[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36 AppEngine-Google; (+http://code.google.com/appengine; appid: s~zyfqwe123)";
const char S4235[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.2.0.0 Safari/537.22";
const char S4236[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.2.0.0 Safari/537.22";
const char S4237[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S4238[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S4239[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)";
const char S4240[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)";
const char S4241[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe) (via docs.google.com/viewer)";
const char S4242[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe) (via docs.google.com/viewer)";
const char S4243[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.1.0.0 Safari/537.22";
const char S4244[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.1.0.0 Safari/537.22";
const char S4245[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S4246[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/534.7 ZemanaAID/FFFF0096";
const char S4247[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S4248[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML like Gecko) Chrome/25.0.1364.97 Safari/537.22";
const char S4249[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22 AppEngine-Google; (+http://code.google.com/appengine; appid: s~stalkthisapp)";
const char S4250[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.7.1364.18260 (gnctrkcll) Safari/537.22";
const char S4251[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.7.1364.17262 Safari/537.22,gzip(gfe),gzip(gfe)";
const char S4252[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22 BitdefenderSafepay/17.14.0.0 (OBK; 25.0.1364.172; r1194.2013)";
const char S4253[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22 BitdefenderSafepay/17.14.0.0 (OBK; 25.0.1364.172; r1194.2013)";
const char S4254[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.7.1364.21027 (gnctrkcll) Safari/537.22";
const char S4255[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 6_0 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0;FBSS/1; FBCR/unavailable;FBID/tablet;FBLC/en_US;FBSF/1.0]";
const char S4256[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.13 (KHTML, like Gecko) Mobile/10A5338d";
const char S4257[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A5355d";
const char S4258[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S4259[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A5376e";
const char S4260[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403";
const char S4261[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]";
const char S4262[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 iPad2,1/K93AP Zite/1.3.3";
const char S4263[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/8536.25";
const char S4264[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48 Instapaper/4.2.6 (+http://www.instapaper.com/iphone)";
const char S4265[] = "Mozilla/5.0 (iPad3G; iPad; U; CPU OS 6_0 like Mac OS X; en_US) com.google.GooglePlus/10352 (KHTML, like Gecko) Mobile/J1AP (gzip)";
const char S4266[] = "Mozilla/5.0 (iPad2; iPad; U; CPU OS 6_0 like Mac OS X; en_US) com.google.GooglePlus/10352 (KHTML, like Gecko) Mobile/K93AP (gzip)";
const char S4267[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A406";
const char S4268[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S4269[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/8536.26.7 DiigoBrowser";
const char S4270[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523";
const char S4271[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A407";
const char S4272[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser";
const char S4273[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 6_0_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0.1;FBSS/1; FBCR/unavailable;FBID/tablet;FBLC/en_US;FBSF/1.0]";
const char S4274[] = "MQQBrowser/10 Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403";
const char S4275[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/8536.26.7 DiigoBrowser";
const char S4276[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser";
const char S4277[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8426";
const char S4278[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 YaMobile/1.00";
const char S4279[] = "Educreations 1.2.3 rv:345 (iPad; iPhone OS 6.0; en_US)";
const char S4280[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176";
const char S4281[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/8536.25";
const char S4282[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/528.16";
const char S4283[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser";
const char S4284[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A407 iPad3,4__weibo__3.1.1__ipad__os6.0";
const char S4285[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48 Instapaper/4.2.7 (+http://www.instapaper.com/iphone)";
const char S4286[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 iPad2,1/K93AP Zite/2.0";
const char S4287[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176";
const char S4288[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3";
const char S4289[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3";
const char S4290[] = "360 Browser 1.0.0 rv:1.0 (iPad; iPhone OS 6.0.1; en_US)";
const char S4291[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8426 iPad3,5/P102AP Zite/2.0";
const char S4292[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S4293[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550 iPad2,5/P105AP Zite/2.0";
const char S4294[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0.1;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]";
const char S4295[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A407 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPad3,4;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0;FBSS/2; FBCR/;FBID/tablet;FBLC/en_US]";
const char S4296[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A406 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPad2,5;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]";
const char S4297[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8426 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPad3,5;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0.1;FBSS/2; FBCR/AT&T;FBID/tablet;FBLC/en_US]";
const char S4298[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8500 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPad2,6;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0.2;FBSS/1; FBCR/AT&T;FBID/tablet;FBLC/en_US]";
const char S4299[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550";
const char S4300[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPad2,5;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0.2;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]";
const char S4301[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8500";
const char S4302[] = "MQQBrowser/10 Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523";
const char S4303[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8500 iPad2,7/P107AP Zite/2.0";
const char S4304[] = "Mozilla/5.0 (iPad2,1; iPad; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/K93AP (gzip)";
const char S4305[] = "Mozilla/5.0 (iPad3,1; iPad; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/J1AP (gzip)";
const char S4306[] = "Mozilla/5.0 (iPad2,5; iPad; U; CPU OS 6_0_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/P105AP (gzip)";
const char S4307[] = "Mozilla/5.0 (iPad3,4; iPad; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/P101AP (gzip)";
const char S4308[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10A550 Safari/7534.48 Instapaper/4.2.7 (+http://www.instapaper.com/iphone)";
const char S4309[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 6_0_2 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,5;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0.2;FBSS/1; FBCR/unavailable;FBID/tablet;FBLC/en_US;FBSF/1.0]";
const char S4310[] = "Miu Ptt 1.9.2 (iPad; iPhone OS 6.0.1; zh_TW)";
const char S4311[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176";
const char S4312[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26.17 (KHTML, like Gecko) Mobile/10A403";
const char S4313[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8426 Safari/8536.26.7 DiigoBrowser";
const char S4314[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8426 YaMobile/1.11";
const char S4315[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8425";
const char S4316[] = "Mozilla/5.0 (iPad2,2; iPad; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/K94AP (gzip)";
const char S4317[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8424";
const char S4318[] = "Mozilla/5.0 (iPad2,4; iPad; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/K93aAP (gzip)";
const char S4319[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8500 YaMobile/1.01";
const char S4320[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/528.16";
const char S4321[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A406 iPad2,5__weibo__3.1.1__ipad__os6.0";
const char S4322[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser";
const char S4323[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 YaMobile/1.11";
const char S4324[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/525.20";
const char S4325[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550 Safari/8536.26.7 DiigoBrowser";
const char S4326[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-N7100 Build/JRO03C)";
const char S4327[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4328[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L900 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4329[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T889 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4330[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; SHV-E250S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4331[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I317 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4332[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; SAMSUNG GT-N7100/N7100XXALIJ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4333[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; SHV-E250L Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4334[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7105 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4335[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; SHV-E250K Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4336[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-I317M Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4337[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-R950 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 USCC-R950";
const char S4338[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GT-N7100-ORANGE/N7100XXALIJ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4339[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; SAMSUNG GT-N7100/N7100XXALIE Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4340[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-T889V Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4341[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7100 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S4342[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GT-N7100-ORANGE/N7100XXALID Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4343[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-N7105 Build/JRO03C)";
const char S4344[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SPH-L900 Build/JRO03C)";
const char S4345[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SGH-T889 Build/JRO03C)";
const char S4346[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; SAMSUNG GT-N7105/N7105XXALIJ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4347[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SAMSUNG-SGH-I317 Build/JRO03C)";
const char S4348[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4349[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; SC-02E Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4350[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SHV-E250S Build/JRO03C)";
const char S4351[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4352[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; GT-N7105T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4353[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SCH-R950 Build/JRO03C)";
const char S4354[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SGH-I317M Build/JRO03C)";
const char S4355[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N7100 Build/JZO54K)";
const char S4356[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4357[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E250L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4358[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L900 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4359[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; SCH-N719 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4360[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7105 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4361[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T889 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4362[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4363[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K)";
const char S4364[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I317 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4365[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T889 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4366[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-N7105/N7105XXDLL4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4367[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-N7100/N7100XXDLL4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4368[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E250K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4369[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SHV-E250S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4370[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SHV-E250K Build/JRO03C)";
const char S4371[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SHV-E250L Build/JRO03C)";
const char S4372[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-ch; SAMSUNG GT-N7105/N7105XXALJ5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4373[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N7105 Build/JZO54K)";
const char S4374[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-nz; GT-N7100T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4375[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; SCH-N719 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S4376[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4377[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I317 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4378[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E250S Build/JZO54K)";
const char S4379[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7100 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S4380[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I317M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4381[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-N7105T Build/JRO03C)";
const char S4382[] = "Mozilla/5.0 (Linux; U; Android 4.1.1 ALK8; en-us; SGH-T889 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4383[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I317M Build/JZO54K)";
const char S4384[] = "Mozilla/5.0 (Linux; U; Android 4.1.1 V2.1 ALK8; en-us; SGH-T889 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4385[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; SAMSUNG GT-N7105/N7105XXDMA5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4386[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I317M Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4387[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SGH-T889V Build/JRO03C)";
const char S4388[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-gb; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4389[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SPH-L900 Build/JZO54K)";
const char S4390[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-T889V Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4391[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-N7105T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4392[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-N7100/N7100XXDMA5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4393[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-N7105/N7105XXDMB2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4394[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; zh-tw; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4395[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7105 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S4396[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SC-02E Build/JRO03C)";
const char S4397[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SCH-N719 Build/JRO03C)";
const char S4398[] = "Mozilla/5.0 (Linux; U; Android 4.1.2 DLL1; en-us; SGH-T889 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4399[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E250K Build/JZO54K)";
const char S4400[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R950 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 USCC-R950";
const char S4401[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4402[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-N7105/N7105XXDLL7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4403[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-N7105-ORANGE/N7105XXDMA5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4404[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-N7100/N7100XXDMB2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4405[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-N7100-ORANGE/N7100XXDMB2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4406[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T889 Build/JZO54K)";
const char S4407[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-R950 Build/JZO54K)";
const char S4408[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-N719 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4409[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I317 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4410[] = "Mozilla 5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7100 Build JRO03C) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S4411[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E250L Build/JZO54K)";
const char S4412[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4413[] = "Mozilla/5.0 (Linux; U; Android 4.1.3; en-us; GT-N7100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4414[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4415[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; es-es; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4416[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-nz; GT-N7100T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4417[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; vi-vn; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4418[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; SGH-T889 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4419[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N7100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4420[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4421[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG GT-N7105/N7105XXDLL1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4422[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N7105T Build/JZO54K)";
const char S4423[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E250K/KKVMC8 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4424[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-N7105/N7105XXDMC3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4425[] = "Mozilla/5.0 (Linux; U; Android 4.1.2 Jelly Bean; vi-vn; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4426[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; de-de; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4427[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-N7105-ORANGE/N7105XXDMB2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4428[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-N719 Build/JZO54K)";
const char S4429[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T889V Build/JZO54K)";
const char S4430[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-US; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4431[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; N7100+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4432[] = "Mozilla/5.0 (Linux; U; Android 4.1.3; es-es; GT-N7100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4433[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E250K/KKVME2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4434[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I317 Build/JZO54K)";
const char S4435[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SPH-L900 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4436[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-N7100/N7100XXDMC3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4437[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4438[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; it-it; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4439[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.9; GT-N7100 Build/IMM76D)";
const char S4440[] = "Mozilla/5.0 (Linux; U; Android 4.1.5; en-us; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4441[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-N7105 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4442[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; th-th; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4443[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SGH-T889 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4444[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E250K/KKVME9 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4445[] = "Mozilla/5.0 (Linux; U; Android 4.1.1 Jelly Bean; vi-vn; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4446[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/TELL//X/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4447[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; N7100DXALJ1 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4448[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ie; SAMSUNG GT-N7100/N7100XXDMF3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4449[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E250K/KKVMF6 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4450[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; ms-my; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4451[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; fr-fr; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4452[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-N719 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S4453[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S4454[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; SAMSUNG GT-N7105/N7105XXDMG1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4455[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-N7100-ORANGE/N7100XXDMF1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4456[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-N7105-ORANGE/N7105XXDMF2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4457[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; id-id; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4458[] = "Mozilla/5.0 (Linux; U; Android 4.1.2 AmnoSperum ROM  Z.7.0.1; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4459[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N7100 Build/JZO54K) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 T5/2.0";
const char S4460[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E250K/KKVMH1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4461[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=t0lteatt;devm=SAMSUNG-SGH-I317;man=samsung;brw=Android;shwd=720x1280;shwa=720x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S4462[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N7100T Build/JZO54K)";
const char S4463[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7100 Build/JRO03C) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 T5/2.0 baidubrowser/3.1.6.4 (Baidu; P1 4.1.1)";
const char S4464[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7100 Build/JRO03C KOUNGY.COM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S4465[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.6 (KHTML like Gecko) Chrome/23.0.1239.0 Safari/537.6";
const char S4466[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.10 (KHTML, like Gecko) Chrome/23.0.1262.0 Safari/537.10 AppEngine-Google; (+http://code.google.com/appengine; appid: s~my-gate32)";
const char S4467[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.1.0.0 Safari/537.11";
const char S4468[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.1.0.0 Safari/537.11";
const char S4469[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/534.7 ZemanaAID/FFFF0061";
const char S4470[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S4471[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S4472[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe)";
const char S4473[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe)";
const char S4474[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.2.0.0 Safari/537.11";
const char S4475[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S4476[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.4.0.0 Safari/537.11";
const char S4477[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.4.0.0 Safari/537.11";
const char S4478[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe) (via docs.google.com/viewer)";
const char S4479[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe) (via docs.google.com/viewer)";
const char S4480[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.2.0.0 Safari/537.11";
const char S4481[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.3.0.0 Safari/537.11";
const char S4482[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.3.0.0 Safari/537.11";
const char S4483[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11 AppEngine-Google; (+http://code.google.com/appengine; appid: s~vivillrocku)";
const char S4484[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11 AppEngine-Google; (+http://code.google.com/appengine; appid: optimizelyedit)";
const char S4485[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; ; CJPMS_AAP715ACX) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11";
const char S4486[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; ; NCLIENT50_AAP715AF4E5CBC) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S4487[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) Chrome/23.0.1271.64 Safari/537.11";
const char S4488[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko GomezAgent 3.0) Chrome/23.0.1271.64 Safari/537.11";
const char S4489[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1284.2) AppleWebKit/537.13 (KHTML, like Gecko) Chrome/24.0.1284.2 Safari/537.13";
const char S4490[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.5) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.5 Safari/537.17";
const char S4491[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.14) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.14 Safari/537.17";
const char S4492[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.27) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.27 Safari/537.17";
const char S4493[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.25) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.25 Safari/537.17";
const char S4494[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.35) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.35 Safari/537.17";
const char S4495[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.32) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.32 Safari/537.17";
const char S4496[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.45) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.45 Safari/537.17";
const char S4497[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.40) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.40 Safari/537.17";
const char S4498[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.52) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17";
const char S4499[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.49) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.49 Safari/537.17";
const char S4500[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/24.0.1312.52) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17";
const char S4501[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/24.0.1312.57)";
const char S4502[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/24.0.1312.57; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4503[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.56) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17";
const char S4504[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/24.0.1312.56) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17";
const char S4505[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/24.0.1312.57; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4506[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/24.0.1312.57; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4507[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.57) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S4508[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/24.0.1312.57; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4509[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/24.0.1312.57; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S4510[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/24.0.1312.57) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S4511[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; GTB7.4; chromeframe/24.0.1312.57; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4512[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/24.0.1312.57; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4513[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; chromeframe/24.0.1312.57; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8; .NET4.0C; .NET4.0E)";
const char S4514[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/24.0.1312.57; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4515[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/24.0.1312.57; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)";
const char S4516[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/24.0.1312.57)";
const char S4517[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/24.0.1312.57) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S4518[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/24.0.1312.57; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)";
const char S4519[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/24.0.1312.57; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4520[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.71) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.71 Safari/537.17";
const char S4521[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; chromeframe/24.0.1312.27; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S4522[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/24.0.1312.56) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17";
const char S4523[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; SV1; chromeframe/24.0.1312.57; .NET CLR 1.1.4322; .NET CLR 2.0.50727)";
const char S4524[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/24.0.1312.52;  Embedded Web Browser from: http://bsalsa.com/; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.1; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S4525[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; SV1; chromeframe/24.0.1312.71; .NET CLR 2.0.50727; InfoPath.1; .NET4.0C; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4526[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.13 (KHTML, like Gecko) Chrome/24.0.1290.1 Safari/537.13 AppEngine-Google; (+http://code.google.com/appengine; appid: s~flybear008)";
const char S4527[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.1.0.0 Safari/537.17";
const char S4528[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.1.0.0 Safari/537.17";
const char S4529[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S4530[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe) (via docs.google.com/viewer)";
const char S4531[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.2.0.0 Safari/537.17";
const char S4532[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S4533[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.2.0.0 Safari/537.17";
const char S4534[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe) (via docs.google.com/viewer)";
const char S4535[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)";
const char S4536[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)";
const char S4537[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/534.7 ZemanaAID/FFFF0096";
const char S4538[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML like Gecko) Chrome/24.0.1312.52 Safari/537.17";
const char S4539[] = "Mozilla/5.0 (Windows NT 6.1; ; NCLIENT50_AAP715B4218F57) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S4540[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; ; CJPMS_AAP715ACX) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17";
const char S4541[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.17 (KHTML like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S4542[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17 AppEngine-Google; (+http://code.google.com/appengine; appid: s~dnnwave)";
const char S4543[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.10.1312.57 Safari/537.17 QIPSurf";
const char S4544[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.10.1312.57 Safari/537.17 QIPSurf";
const char S4545[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S4546[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S4547[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36";
const char S4548[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36";
const char S4549[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe)";
const char S4550[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe)";
const char S4551[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe),gzip(gfe)";
const char S4552[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe),gzip(gfe)";
const char S4553[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/29.0.1600.1 Chrome/29.0.1600.1 Safari/537.36";
const char S4554[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Iron/29.0.1600.1 Chrome/29.0.1600.1 Safari/537.36";
const char S4555[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/534.7 ZemanaAID/FFFF0096";
const char S4556[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36Accept-Encoding: gzip, deflate";
const char S4557[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36 AppEngine-Google; (+http://code.google.com/appengine; appid: s~optimizelyedit-hrd)";
const char S4558[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4559[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-I9100M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4560[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; GT-I9100T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4561[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4562[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4563[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; SAMSUNG GT-I9100/I9100XWKG1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4564[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ar-ae; GT-I9100 Build/DR.AKRIM BLACKDROID) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4565[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SAMSUNG GT-I9100/I9100BUKG2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4566[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-at; GT-I9100-ORANGE/I9100BVKG8 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4567[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; GT-I9100-ORANGE/I9100BVKG2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4568[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; GT-I9100-ORANGE/I9100BVKH1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4569[] = "Mozilla/5.0 (Linux; U; Android2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4570[] = "Mozilla/5.0 (Linux; U;Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4571[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4572[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100-ORANGE/I9100BVKG3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4573[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4574[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SAMSUNG GT-I9100/I9100BUKE5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4575[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; SAMSUNG GT-I9100/I9100BUKE4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4576[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SAMSUNG GT-I9100/I9100BUKE1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4577[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100-ORANGE/I9100BVKE2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4578[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; SAMSUNG GT-I9100/I9100BUKE2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4579[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-I9100M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4580[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4581[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; he-il; GT-I9100T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4582[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100-ORANGE/I9100BVKH2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4583[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4584[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; pl-pl; GT-I9100-ORANGE/I9100BVKF2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4585[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; SAMSUNG GT-I9100/I9100BUKI3F Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4586[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; GT-I9100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4587[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; GT-I9100 Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4588[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-I9100 Build/GINGERBREAD)";
const char S4589[] = "Mozilla/5.0 (Linux; U; de-de; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S4590[] = "Mozilla/5.0 (Linux; U; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S4591[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-I9100 Build/GINGERBREAD)";
const char S4592[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-I9100T Build/GINGERBREAD)";
const char S4593[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-I9100 Build/GINGERBREAD)";
const char S4594[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-I9100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4595[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; SAMSUNG GT-I9100/I9100BUKI3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4596[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; GT-I9100T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4597[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4598[] = "Mozilla/5.0 (Linux; U; Android ANDROIDHOUSE 2.3.6; es-es; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4599[] = "Logos/0.9.4 (Linux; U; Android 2.3.4; en_GB; GT-I9100 Build/GINGERBREAD.XWKI4)";
const char S4600[] = "Android/1.0 (GT-I9100M GINGERBREAD)";
const char S4601[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; GT-I9100T Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S4602[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9100 Build/GINGERBREAD; 480*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S4603[] = "Mozilla/5.0 (Linux; U; nl-nl; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S4604[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-I9100M Build/GINGERBREAD)";
const char S4605[] = "Mozilla/5.0 (Linux; U; fr-fr; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S4606[] = "Mozilla/5.0 (Linux; U; de-ch; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S4607[] = "Mozilla/5.0 (Linux; U; en-ie; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S4608[] = "Mozilla/5.0 (Linux; U; es-es; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S4609[] = "Mozilla/5.0 (Linux; U; en-us; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S4610[] = "Mozilla/5.0 (Linux; U; Android 2.4; en-gb; GT-I9100X Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4611[] = "SAMSUNG-GT-I9108_TD/1.0 Android/2.3.3 Release/03.15.2011 Browser/AppleWebKit533.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S4612[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100-ORANGE/I9100BVKJ4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4613[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4614[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; SAMSUNG GT-I9100/I9100BUKJ3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4615[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9100 Build/GINGERBREAD)";
const char S4616[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-I9100T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4617[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9100G Build/GINGERBREAD)";
const char S4618[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-I9100-ORANGE/I9100BVKL1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4619[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100G Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S4620[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-I9100G Build/GINGERBREAD)";
const char S4621[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9100 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S4622[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; GT-I9100 Build/GRJ22)";
const char S4623[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; GT-I9100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4624[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-I9100 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4625[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9108 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4626[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S4627[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9108 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S4628[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9108 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S4629[] = "Mozilla/5.0 (Linux; U; cs-cz; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S4630[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-I9100T Build/GINGERBREAD)";
const char S4631[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9108 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4632[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S4633[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S4634[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9108 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S4635[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; I9100/V100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4636[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9108 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4637[] = "Mozilla/5.0 (Linux; U; pl-pl; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S4638[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S4639[] = "Mozilla/5.0 (Linux; U; pt-pt; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S4640[] = "Mozilla/5.0 (Linux; U; de-at; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S4641[] = "Mozilla/5.0 (Linux; U; sl-si; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S4642[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.12011-10-16 20:22:55";
const char S4643[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S4644[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S4645[] = "Mozilla/5.0 (Linux; U; fr-ch; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S4646[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4647[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; iw-il; GT-I9100 Build/HebMIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4648[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en_gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)";
const char S4649[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-I9100 Build/WIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4650[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-I9100T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4651[] = "Mozilla/5.0 (Linux; U; Android Android 2.3.6 Gingerbread XWLA4; es-es; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4652[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9100 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S4653[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; GT-I9100 Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4654[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S4655[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9108 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S4656[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9100 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S4657[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100G Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S4658[] = "Mozilla/5.0 (Linux; U; Android ANDROIDHOUSE 2.3.5; es-es; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4659[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; SAMSUNG GT-I9100/I9100BUKG2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4660[] = "Mozilla/5.0 (Linux; U; Android Android 2.3.6 XWKK5; es-es; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4661[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9100G Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S4662[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4663[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100 Build/GINGERBREAD.AX918) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S4664[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4665[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4666[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-I9100M Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S4667[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-I9100G Build/GINGERBREAD)";
const char S4668[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9100 Build/GINGERBREAD.AX918) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4669[] = "Mozilla/5.0 (Linux; U; Android AndroidHouse Team 2.3.6; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4670[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9100T Build/GINGERBREAD)";
const char S4671[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9108 Build/GINGERBREAD)";
const char S4672[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-I9100M Build/GINGERBREAD) AppleWebKit/533.1+ (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4673[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9100G Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S4674[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9108 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S4675[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100 Build/GINGERBREAD.AX918) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S4676[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT-I9100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4677[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; GT-I9100T Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S4678[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S4679[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9108 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S4680[] = "Mozilla/5.0 (Linux; U; hr-hr; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S4681[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-S959G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4682[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4683[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4684[] = "Mozilla/5.0 (Linux; U; ro-ro; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S4685[] = "Mozilla/5.0 (Linux; U; tr-tr; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S4686[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-S959G Build/GINGERBREAD)";
const char S4687[] = "Mozilla/5.0 (Linux; U; en-au; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S4688[] = "Mozilla/5.0 (Linux; U; it-it; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S4689[] = "Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S4690[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4691[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-I9100T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4692[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; GT-i9100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4693[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S4694[] = "Mozilla/5.0 (Linux; U; Android 2.3.8; zh-tw; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4695[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S4696[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; I9100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4697[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4698[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SGH-S959G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S4699[] = "Opera/9.50 (J2ME/MIDP; Opera Mini/4.1.11355/574; U; en)";
const char S4700[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/4.1.13572/20.2485; U; en) Presto/2.5.25";
const char S4701[] = "Opera/9.60 (J2ME/MIDP; Opera Mini/4.1.11320/608; U; en) Presto/2.2.0";
const char S4702[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/4.1.13907/25.729; U; fr) Presto/2.5.25 Version/10.54";
const char S4703[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/4.1.15082/30.3214; U; en) Presto/2.8.119 Version/11.10";
const char S4704[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Win64; x64; Trident/6.0; Touch)";
const char S4705[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; Touch)";
const char S4706[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Touch)";
const char S4707[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; HPDTDFJS)";
const char S4708[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Win64; x64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Tablet PC 2.0; ASU2JS)";
const char S4709[] = "Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S4710[] = "i-mobilehitz218/2.0 (03.15) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S4711[] = "NGM_Vanity /ObigoInternetBrowser/QO3C Profile/MIDP2.0 Configuration/CLDC-1.1";
const char S4712[] = "Jbed-FastBCC Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S4713[] = "GIONEE-M902/SW1.0.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S4714[] = "Duplii/1.0 TMSS-Browser/1.0.0 Profile/MIDP/2.0-compatible Configuration/CLDC-1.1";
const char S4715[] = "Beetel-TD590/S108 LMP/LML Release/2010.08.16 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S4716[] = "OGO-O78/Obigo WapBrowser 2.0/MIDP2.0Configuration/CLDC1.1";
const char S4717[] = "TMN/1.0/G7002/SW1.0.0/WAP2.0/MIDP-2.0 Configuration/CLDC-1.1";
const char S4718[] = "IN2010 Nano2/Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S4719[] = "VertuConstellationF/2.0 (08w11500.01) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S4720[] = "G-Fone777/Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S4721[] = "Sansui_S44/22.11.2007 Browser/wap2.0 Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S4722[] = "W3C-mobileOK/DDC-1.0 (see http://www.w3.org/2006/07/mobileok-ddc)";
const char S4723[] = "RT240X320/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S4724[] = "OT-S680/1.0 MTK/W07.32 Release/01.01.2007 Browser/Teleca-1.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S4725[] = "OT-S686/1.0 BRCM/2124 Release/CC84-V1.0.0 Browser/Teleca-1.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S4726[] = "Mozilla/5.0 (PUMA Phone M1/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1; 240x320) AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S4727[] = "Fly_E146/WapBrowser2.0/MIDP-2.0Configuration/CLDC-1.1";
const char S4728[] = "FeedDemon/3.1 (http://www.feeddemon.com/; Microsoft Windows)";
const char S4729[] = "Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S4730[] = "M760/SW1.0.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S4731[] = "RT176X220/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S4732[] = "RT128X160/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S4733[] = "Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S4734[] = "SJT_E861A/wap2.0Profile/MIDP-1.0/2.0Configuration/CLDC-1.1";
const char S4735[] = "Mozilla/4.0 (jig browser web; 1.0.4; 941SH)";
const char S4736[] = "Mozilla/4.0 (jig browser web; 1.0.4; 831SH)";
const char S4737[] = "Mozilla/4.0 (jig browser web; 1.0.4; SH01B)";
const char S4738[] = "Mozilla/4.0 (jig browser web; 1.0.4; F01C)";
const char S4739[] = "Mozilla/4.0 (jig browser web; 1.0.4; F06B)";
const char S4740[] = "python-requests/0.8.1";
const char S4741[] = "FeedDemon/2.7 (http://www.newsgator.com/; Microsoft Windows)";
const char S4742[] = "python-requests/0.10.6";
const char S4743[] = "Mozilla/4.0 (jig browser web; 1.0.4; SH38)";
const char S4744[] = "curl/7.15.5";
const char S4745[] = "Mozilla/4.0 (jig browser web; 1.0.4; 923SH)";
const char S4746[] = "Mozilla/4.0 (jig browser web; 1.0.4; 004SH)";
const char S4747[] = "python-requests/0.10.8";
const char S4748[] = "Java/1.5.0_10";
const char S4749[] = "Amazon CloudFront";
const char S4750[] = "Mozilla/4.0 (jig browser web; 1.0.4; 940SH)";
const char S4751[] = "Java/1.4.2_04";
const char S4752[] = "Mozilla/4.0 (jig browser web; 1.0.4; 920SH)";
const char S4753[] = "Mozilla/4.0 (jig browser web; 1.0.4; 931SH)";
const char S4754[] = "Mozilla/4.0 (jig browser web; 1.0.4; 001SH)";
const char S4755[] = "Mozilla/4.0 (jig browser web; 1.0.4; P906i)";
const char S4756[] = "Mozilla/4.0 (jig browser web; 1.0.4; P06B)";
const char S4757[] = "Mozilla/4.0 (jig browser web; 1.0.4; N07B)";
const char S4758[] = "Mozilla/4.0 (jig browser web; 1.0.4; 840P)";
const char S4759[] = "Mozilla/4.0 (jig browser web; 1.0.4; P04B)";
const char S4760[] = "Mozilla/4.0 (jig browser web; 1.0.4; SH04A)";
const char S4761[] = "Mozilla/4.0 (jig browser web; 1.0.4; P02B)";
const char S4762[] = "Mozilla/4.0 (jig browser web; 1.0.4; 932SH)";
const char S4763[] = "Mozilla/4.0 (jig browser web; 1.0.4; F02B)";
const char S4764[] = "Mozilla/4.0 (jig browser web; 1.0.4; SH02A)";
const char S4765[] = "Mozilla/4.0 (jig browser web; 1.0.4; 933SH)";
const char S4766[] = "Mozilla/4.0 (jig browser web; 1.0.4; N04B)";
const char S4767[] = "Mozilla/4.0 (jig browser web; 1.0.4; 943SH)";
const char S4768[] = "Mozilla/4.0 (jig browser web; 1.0.4; N02C)";
const char S4769[] = "Mozilla/4.0 (jig browser web; 1.0.4; N08A3)";
const char S4770[] = "Mozilla/4.0 (jig browser web; 1.0.4; SH01C)";
const char S4771[] = "Mozilla/4.0 (jig browser web; 1.0.4; SH01A)";
const char S4772[] = "Mozilla/4.0 (jig browser web; 1.0.4; 824SH)";
const char S4773[] = "Mozilla/4.0 (jig browser web; 1.0.4; 841SH)";
const char S4774[] = "Mozilla/4.0 (jig browser web; 1.0.4; 002SH)";
const char S4775[] = "Mozilla/4.0 (jig browser web; 1.0.4; 941P)";
const char S4776[] = "python-requests/0.8.6";
const char S4777[] = "Java/1.8.0-ea";
const char S4778[] = "python-requests/0.11.1";
const char S4779[] = "siclab (cboc-test@lab.ntt.co.jp)";
const char S4780[] = "python-requests/0.11.2";
const char S4781[] = "python-requests/0.12.1";
const char S4782[] = "PHP/5.2.17p1";
const char S4783[] = "curl/7.19.7";
const char S4784[] = "Mozilla/4.0 (jig browser web; 1.0.4; SH03B)";
const char S4785[] = "Mozilla/4.0 (jig browser web; 1.0.4; 944SH)";
const char S4786[] = "python-requests/0.10.1";
const char S4787[] = "Mozilla/4.0 (jig browser web; 1.0.4; P02A)";
const char S4788[] = "Mozilla/4.0 (jig browser web; 1.0.4; SH07B)";
const char S4789[] = "Mozilla/4.0 (jig browser web; 1.0.4; N906imyu)";
const char S4790[] = "Mozilla/4.0 (jig browser web; 1.0.4; N01B)";
const char S4791[] = "curl/7.18.0 (i586-pc-mingw32msvc) libcurl/7.18.0 zlib/1.2.3";
const char S4792[] = "Mozilla/4.0 (jig browser web; 1.0.4; F03B)";
const char S4793[] = "Mozilla/4.0 (jig browser web; 1.0.4; F01B)";
const char S4794[] = "Mozilla/4.0 (jig browser web; 1.0.4; N06B)";
const char S4795[] = "Mozilla/4.0 (jig browser web; 1.0.4; 820SH)";
const char S4796[] = "Mozilla/4.0 (jig browser web; 1.0.4; SH906i)";
const char S4797[] = "Mozilla/4.0 (jig browser web; 1.0.4; 840SH)";
const char S4798[] = "Mozilla/4.0 (jig browser web; 1.0.4; 830CA)";
const char S4799[] = "Mozilla/4.0 (jig browser web; 1.0.4; P01A)";
const char S4800[] = "Mozilla/4.0 (jig browser web; 1.0.4; P10A)";
const char S4801[] = "Mozilla/4.0 (jig browser web; 1.0.4; P03C)";
const char S4802[] = "Mozilla/4.0 (jig browser web; 1.0.4; F01A)";
const char S4803[] = "Mozilla/4.0 (jig browser web; 1.0.4; 940SC)";
const char S4804[] = "Mozilla/4.0 (jig browser web; 1.0.4; F09A3)";
const char S4805[] = "Mozilla/4.0 (jig browser web; 1.0.4; 930SH)";
const char S4806[] = "Mozilla/4.0 (jig browser web; 1.0.4; SH08B)";
const char S4807[] = "Mozilla/4.0 (jig browser web; 1.0.4; SH02B)";
const char S4808[] = "Mozilla/4.0 (jig browser web; 1.0.4; 832SH)";
const char S4809[] = "Mozilla/4.0 (jig browser web; 1.0.4; N02A)";
const char S4810[] = "Mozilla/4.0 (jig browser web; 1.0.4; 830SH)";
const char S4811[] = "Mozilla/4.0 (jig browser web; 1.0.4; F08B)";
const char S4812[] = "Mozilla/4.0 (jig browser web; 1.0.4; P905i)";
const char S4813[] = "Mozilla/5.0 (X11; U; IRIX64 IP35; en-US; rv:1.4.3) Gecko/X";
const char S4814[] = "Mozilla/4.0 (jig browser web; 1.0.4; P03A)";
const char S4815[] = "python-requests/0.12.0";
const char S4816[] = "python-requests/0.10.2";
const char S4817[] = "curl/7.24.0 (i686-pc-cygwin) libcurl/7.24.0 OpenSSL/1.0.1b zlib/1.2.5 libidn/1.22 libssh2/1.4.0";
const char S4818[] = "python-requests/0.13.0";
const char S4819[] = "curl/7.26.0";
const char S4820[] = "python-requests/0.13.1";
const char S4821[] = "Wada.vn Vietnamese Search";
const char S4822[] = "Podcasts/1.0";
const char S4823[] = "python-requests/0.13.2";
const char S4824[] = "Mozilla/5.0 (compatible; Add Catalog/2.1;)";
const char S4825[] = "Apache-HttpClient/4.2.1 (java 1.5)";
const char S4826[] = "Mozilla/4.0 (jig browser web; 1.0.4; P01B)";
const char S4827[] = "Mozilla/4.0 (jig browser web; 1.0.4; SH03A)";
const char S4828[] = "Mozilla/4.0 (jig browser web; 1.0.4; 936SH)";
const char S4829[] = "Mozilla/4.0 (jig browser web; 1.0.4; N905i)";
const char S4830[] = "Mozilla/4.0 (jig browser web; 1.0.4; P08A3)";
const char S4831[] = "Mozilla/4.0 (jig browser web; 1.0.4; F02A)";
const char S4832[] = "Mozilla/4.0 (jig browser web; 1.0.4; SH05A3)";
const char S4833[] = "Mozilla/4.0 (jig browser web; 1.0.4; 831P)";
const char S4834[] = "Mozilla/4.0 (jig browser web; 1.0.4; SH06B)";
const char S4835[] = "Mozilla/4.0 (jig browser web; 1.0.4; N09A3)";
const char S4836[] = "Mozilla/4.0 (jig browser web; 1.0.4; SH905i)";
const char S4837[] = "Mozilla/4.0 (jig browser web; 1.0.4; 942P)";
const char S4838[] = "Mozilla/4.0 (jig browser web; 1.0.4; F05C)";
const char S4839[] = "Mozilla/4.0 (jig browser web; 1.0.4; 001N)";
const char S4840[] = "Mozilla/4.0 (jig browser web; 1.0.4; SH10C)";
const char S4841[] = "python-requests/0.8.2";
const char S4842[] = "python-requests/0.9.3";
const char S4843[] = "Podcasts/1.0.1";
const char S4844[] = "Podcasts/1.0.2";
const char S4845[] = "Python-urllib/2.7 AppEngine-Google; (+http://code.google.com/appengine; appid: s~pulsecontentserver)";
const char S4846[] = "Java/1.6.0_35";
const char S4847[] = "Podcasts/1.1.1";
const char S4848[] = "Podcasts/1.1";
const char S4849[] = "Podcasts/1.1.2";
const char S4850[] = "Mozilla/5.0 (compatible; evc/1.0)";
const char S4851[] = "Apache-HttpClient/4.2.2 (java 1.5)";
const char S4852[] = "Mozilla/5.0 (Python-urllib2)";
const char S4853[] = "Apache-HttpClient/4.2.3 (java 1.5)";
const char S4854[] = "Podcasts/1.2";
const char S4855[] = "Mozilla/5.0 (compatible; human_curl.async; 0.1.1; +http://h.wrttn.me/human_curl)";
const char S4856[] = "Go 1.1 package http";
const char S4857[] = "Podcasts/1.2.1";
const char S4858[] = "Apache-HttpClient/4.2.5 (java 1.5)";
const char S4859[] = "Google_Analytics_Snippet_Validator";
const char S4860[] = "Mozilla/5.0 (compatible; Prlog/1.0; +http://prlog.ru/)";
const char S4861[] = "HTTPClient/1.0 (2.3.3, ruby 1.9.3 (2013-02-22))";
const char S4862[] = "Java/1.7.0_25";
const char S4863[] = "Podcasts/1.2.2";
const char S4864[] = "ADmantX Platform Semantic Analyzer - ADmantX Inc. - www.admantx.com - support@admantx.com";
const char S4865[] = "Podcasts/1.2.3";
const char S4866[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.26916/26.1424; U; ru) Presto/2.8.119 Version/10.54";
const char S4867[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.31891/29.3417; U; en) Presto/2.8.119 Version/11.10";
const char S4868[] = "Opera/9.50 (J2ME/MIDP; Opera Mini/4.4.28962/29.3757; U; en)";
const char S4869[] = "GT-E3300 Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.29644/30.3697; U; en) Presto/2.8.119 Version/11.10";
const char S4870[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5";
const char S4871[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F192 Safari/6533.18.5";
const char S4872[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F191 Safari/6533.18.5";
const char S4873[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8G4 Safari/6533.18.5";
const char S4874[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F190 Safari/6533.18.5";
const char S4875[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H8 Safari/6533.18.5";
const char S4876[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5";
const char S4877[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J3 Safari/6533.18.5";
const char S4878[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S4879[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9Z999 Safari/6533.18.5";
const char S4880[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8K2 Safari/6533.18.5";
const char S4881[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5";
const char S4882[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S4883[] = "Mozilla/5.0 (iPad; CPU OS 4_3_5 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0 Mobile/8L1 Safari/7533.17.9";
const char S4884[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S4885[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F183 Safari/6533.18.5";
const char S4886[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S4887[] = "Mozilla/5.0 (iPad; CPU OS 4_3_3 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0 Mobile/8J2 Safari/7533.17.9";
const char S4888[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 Version/5.0.2 Safari/6533.18.5 Kikin/1.9.0";
const char S4889[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9Z999 Safari/6533.18.5";
const char S4890[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; zh-cn Model:iPad2,1) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S4891[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en_us) AppleWebKit/6533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5";
const char S4892[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F190 Safari/6533.18.5";
const char S4893[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en_us) AppleWebKit/6533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5";
const char S4894[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S4895[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; 6533.17.9) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5";
const char S4896[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; 6533.17.9) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5";
const char S4897[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en_us) AppleWebKit/6533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5";
const char S4898[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en_us) AppleWebKit/6533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5";
const char S4899[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F5166b Safari/6533.18.5";
const char S4900[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5";
const char S4901[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5";
const char S4902[] = "Mozilla/5.0 (iPad; U; CPU OS 432 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8H8 Safari/6533.18.5";
const char S4903[] = "Mozilla/5.0 (iPad; U; CPU OS 434 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8K2 Safari/6533.18.5";
const char S4904[] = "Mozilla/5.0 (iPad; U; CPU OS 431 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8G4 Safari/6533.18.5";
const char S4905[] = "Mozilla/5.0 (iPad; U; CPU OS 435 like Mac OS X; en-CA) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5";
const char S4906[] = "Mozilla/5.0 (iPad; U; CPU OS 433 like Mac OS X; es-US) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5";
const char S4907[] = "Mozilla/5.0 (iPad; U; CPU OS 433 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8J3 Safari/6533.18.5gzip(gfe)";
const char S4908[] = "Mozilla/5.0 (iPad; U; CPU iPad OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5";
const char S4909[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.02 Guidebook/43 Mobile/8L1 Safari/6533.18.5";
const char S4910[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)";
const char S4911[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/534.7 ZemanaAID/FFFF0089";
const char S4912[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)";
const char S4913[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.1.3.0 Safari/537.31";
const char S4914[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.1.3.0 Safari/537.31";
const char S4915[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S4916[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S4917[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe)";
const char S4918[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe)";
const char S4919[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.2.2.0 Safari/537.31";
const char S4920[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.2.2.0 Safari/537.31";
const char S4921[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 AppEngine-Google; (+http://code.google.com/appengine; appid: optimizelyedit)";
const char S4922[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 AlexaToolbar/alxg-3.1 AppEngine-Google; (+http://code.google.com/appengine; appid: s~znyX)";
const char S4923[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S4924[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 AppEngine-Google; (+http://code.google.com/appengine; appid: s~tinaX)";
const char S4925[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.27 (KHTML, like Gecko) Chrome/26.0.1384.0 (Dart) Safari/537.27";
const char S4926[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/537.31 AppEngine-Google; (+http://code.google.com/appengine; appid: s~easy-carson)";
const char S4927[] = "Mozilla\\5.0 (Windows NT 6.1) AppleWebKit\\537.31 (KHTML, like Gecko) Chrome\\26.0.1410.64 Safari\\537.31; campus-TU";
const char S4928[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/0.0.0.0 Chrome/26.0.1410.43 Safari/537.1";
const char S4929[] = "Opera/9.80 (Series 60; Opera Mini/7.0.31380/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S4930[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1323.1) AppleWebKit/537.19 (KHTML, like Gecko) Chrome/25.0.1323.1 Safari/537.19";
const char S4931[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.5 Safari/537.22";
const char S4932[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.36) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.36 Safari/537.22";
const char S4933[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.29) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.29 Safari/537.22";
const char S4934[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.45) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.45 Safari/537.22";
const char S4935[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.58) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.58 Safari/537.22";
const char S4936[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.68) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.68 Safari/537.22";
const char S4937[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.84) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.84 Safari/537.22";
const char S4938[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/25.0.1364.97) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22";
const char S4939[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.97) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22";
const char S4940[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.172) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S4941[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/25.0.1364.172; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4942[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/25.0.1364.172; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4943[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/25.0.1364.172)";
const char S4944[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/25.0.1364.172) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S4945[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/25.0.1364.172; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S4946[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.152) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22";
const char S4947[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/25.0.1364.172; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4948[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.155) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.155 Safari/537.22";
const char S4949[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/25.0.1364.152) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22";
const char S4950[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.160) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.160 Safari/537.22";
const char S4951[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/25.0.1364.172; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)";
const char S4952[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/25.0.1364.172; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4953[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/25.0.1364.172; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4954[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/25.0.1364.172; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4955[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/25.0.1364.172)";
const char S4956[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/25.0.1364.172) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S4957[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/25.0.1364.172; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4958[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/25.0.1364.172; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S4959[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/25.0.1364.160; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; .NET4.0C; .NET4.0E; Zune 4.7)";
const char S4960[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/25.0.1364.97) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22";
const char S4961[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S4962[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S4963[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via translate.google.com)";
const char S4964[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via translate.google.com)";
const char S4965[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via docs.google.com/viewer)";
const char S4966[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/534.7 ZemanaAID/FFFF0061";
const char S4967[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)";
const char S4968[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)";
const char S4969[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via docs.google.com/viewer)";
const char S4970[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 AppEngine-Google; (+http://code.google.com/appengine; appid: optimizelyedit)";
const char S4971[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S4972[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S4973[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (gtsr) Safari/537.4";
const char S4974[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (gtsr) Safari/537.4";
const char S4975[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (fnrb) Safari/537.4";
const char S4976[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (fnrb) Safari/537.4";
const char S4977[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.520 YaBrowser/1.5.1105.520 (bskt) Safari/537.4";
const char S4978[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.520 YaBrowser/1.5.1105.520 (bskt) Safari/537.4";
const char S4979[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.1.9999.433 Safari/537.4";
const char S4980[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.1.9999.433 Safari/537.4";
const char S4981[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/29.0.1547.22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.22 Safari/537.36";
const char S4982[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/29.0.1547.22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.22 Safari/537.36";
const char S4983[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/29.0.1547.32) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.32 Safari/537.36";
const char S4984[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/29.0.1547.32) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.32 Safari/537.36";
const char S4985[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/29.0.1547.41) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.41 Safari/537.36";
const char S4986[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/29.0.1547.41) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.41 Safari/537.36";
const char S4987[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/29.0.1547.57) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36";
const char S4988[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/29.0.1547.49) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.49 Safari/537.36";
const char S4989[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/29.0.1547.57) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36";
const char S4990[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/29.0.1547.49) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.49 Safari/537.36";
const char S4991[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/29.0.1547.62) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.62 Safari/537.36";
const char S4992[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/29.0.1547.62) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.62 Safari/537.36";
const char S4993[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S4994[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S4995[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)";
const char S4996[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/29.0.1547.66)";
const char S4997[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/29.0.1547.66) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36";
const char S4998[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/29.0.1547.66)";
const char S4999[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/29.0.1547.66) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36";
const char S5000[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/29.0.1547.66)";
const char S5001[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/29.0.1547.66)";
const char S5002[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S5003[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S5004[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S5005[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)";
const char S5006[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S5007[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0)";
const char S5008[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Tablet PC 2.0)";
const char S5009[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S5010[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S5011[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S5012[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/29.0.1547.76) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36";
const char S5013[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/29.0.1547.76) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36";
const char S5014[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/29.0.1547.76; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S5015[] = "Mozilla/5.0 (compatible; MSIE 8.0; MSBrowserIE8; Windows NT 6.1; WOW64; Trident/5.0; MSBrowserIE9; chromeframe/29.0.1547.66)";
const char S5016[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0; chromeframe/29.0.1547.76) like Gecko";
const char S5017[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/29.0.1547.76; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S5018[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/29.0.1547.76; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C; .NET4.0E; 3M/MSIE 8.0)";
const char S5019[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I747 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5020[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I747 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5021[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5022[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5023[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5024[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9305 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5025[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/tMIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5026[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; GT-I9305T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5027[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5028[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5029[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-I747M Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5030[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I747 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5031[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; SAMSUNG GT-I9305/I9305XXALI5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5032[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I747 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5033[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5034[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SPH-L710 Build/JRO03L)";
const char S5035[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5036[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-I9305 Build/JRO03C)";
const char S5037[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; SAMSUNG GT-I9305/I9305XXALI7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5038[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I747 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5039[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; SHV-E210K Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5040[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5041[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; SHV-E210L Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5042[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; SHV-E210S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5043[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; GT-I9305N Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5044[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-I9305T Build/JRO03C)";
const char S5045[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S5046[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; SPH-L710 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5047[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K)";
const char S5048[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SGH-I747M Build/JRO03L)";
const char S5049[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L)";
const char S5050[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; SAMSUNG GT-I9305/I9305XXALJ4 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5051[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; SAMSUNG GT-I9305/I9305XXALK5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5052[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-R530U Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5053[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9305 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5054[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-I747M-parrot Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5055[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; SGH-I747M Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5056[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-R530U Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 USCC-R530U";
const char S5057[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E210K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5058[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E210S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5059[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E210L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5060[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SCH-R530U Build/JRO03L)";
const char S5061[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5062[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SPH-L710 Build/JZO54K)";
const char S5063[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I9305/I9305XXBLL5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5064[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I9305/I9305XXBLL3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5065[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-ch; SAMSUNG GT-I9305/I9305XXALK3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5066[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9305 Build/JZO54K)";
const char S5067[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E210S Build/JZO54K)";
const char S5068[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I747 Build/JZO54K)";
const char S5069[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-I9305T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5070[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I9305/I9305XXBMA3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5071[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E210K Build/JZO54K)";
const char S5072[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-I9305-ORANGE/I9305XXBMA1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5073[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-R530U Build/JZO54K)";
const char S5074[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5075[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530U Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 USCC-R530U";
const char S5076[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9305T Build/JZO54K)";
const char S5077[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; SAMSUNG GT-I9305/I9305XXBMB4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5078[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; SAMSUNG GT-I9305/I9305XXBMA4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5079[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SPH-L710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5080[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SHV-E210S Build/JRO03C)";
const char S5081[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-I9305N Build/JRO03C)";
const char S5082[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E210L Build/JZO54K)";
const char S5083[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S5084[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530C Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5085[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; GT-I9305N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5086[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5087[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; SAMSUNG GT-I9305/I9305XXBMC1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5088[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-R530C Build/JZO54K)";
const char S5089[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I9305/I9305XXBMD1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5090[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-I747M Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S5091[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E210K/KKJMD2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5092[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9305N Build/JZO54K)";
const char S5093[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I747 Build/JZO51K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5094[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30";
const char S5095[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I9305/I9305XXBMD5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5096[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SGH-I747 Build/JRO03L)";
const char S5097[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SHV-E210K Build/JRO03C)";
const char S5098[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SHV-E210L Build/JRO03C)";
const char S5099[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I747M Build/JZO54K)";
const char S5100[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I747M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5101[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I747M-parrot Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5102[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SGH-I747M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5103[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I9305/I9305XXBME3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5104[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-I9305-ORANGE/I9305XXBMF1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5105[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E210K/KKJMF6 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5106[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; el-gr; SAMSUNG GT-I9305/I9305XXBLL1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5107[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-I9305 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5108[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SCH-R530U Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5109[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; SAMSUNG GT-I9305/I9305XXBMG3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5110[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=d2spr;devm=SPH-L710;man=samsung;brw=Android;shwd=720x1280;shwa=720x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S5111[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=d2att;devm=SAMSUNG-SGH-I747;man=samsung;brw=Android;shwd=720x1280;shwa=720x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S5112[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E210K/KKJMH1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5113[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GT-I9305-ORANGE/I9305XXALIA Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5114[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5115[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SCH-R530C Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5116[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5117[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5118[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-i9100 Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5119[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; es-es; GT-I9100 Build/ICS) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S5120[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5121[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; SAMSUNG GT-I9100/I9100BOLPD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5122[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100-ORANGE/I9100BVLPB Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5123[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SAMSUNG GT-I9100/I9100BULPC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5124[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SAMSUNG GT-I9100/I9100BVLPB Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5125[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76I-RR) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5126[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; GT-I9100M Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5127[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; GT-I9100T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5128[] = "BBC iPlayer (AndroidApp; 1.3; 13014) GT-I9100 (Android 4.0.3, SDK 15)";
const char S5129[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; GT-I9100 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30";
const char S5130[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.1 Mobile Safari/534.30";
const char S5131[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KH";
const char S5132[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-I9100 Build/IML74K)";
const char S5133[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5134[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-I9100M Build/IML74K)";
const char S5135[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9100 Build/IMM76D)";
const char S5136[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9100 Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S5137[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-I9100T Build/IML74K)";
const char S5138[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5139[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5140[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; SAMSUNG GT-I9100/I9100BOLPE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5141[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-I9100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S5142[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100 Build/IMM76I_RR) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5143[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5144[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-I9100G Build/IML74K)";
const char S5145[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy SII T-mobile Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5146[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9100 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S5147[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SAMSUNG GT-I9100/I9100BULPD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5148[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SAMSUNG GT-I9100/I9100BVLPD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5149[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100-ORANGE/I9100BVLPD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5150[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; GT-I9100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S5151[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100G Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S5152[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/WIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5153[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5154[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/cMIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5155[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5156[] = "Mozilla/5.0 (Linux; U; Android 4.0.2; es-es; GT-I9100 Build/4.0.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S5157[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100 Build/AdyScorpiusMIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5158[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S5159[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5160[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5161[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-I9100T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5162[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5163[] = "Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9100 Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S5164[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9100G Build/IMM76D)";
const char S5165[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9100/I9100BVLPH Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5166[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; SAMSUNG GT-I9100/I9100BOLPB Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5167[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S5168[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/Neutrino_system) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5169[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100-ORANGE/I9100BVLPH Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5170[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-I9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5171[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; GT-I9100T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5172[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SAMSUNG GT-I9100/I9100BOLPM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5173[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; I9100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S5174[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-I9100G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5175[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9100/I9100BULPG Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5176[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S5177[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S5178[] = "Mozilla/5.0 (Linux; U; Android 4.0UI; en-us; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S5179[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-I9100 Build/XWLPW IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5180[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-I9100G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5181[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; SAMSUNG GT-I9100/I9100BULPD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5182[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9108 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S5183[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG GT-I9100/I9100BULPG Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5184[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; GT-I9100T Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S5185[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9108 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5186[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9100 Build/ICS.g12refM1005.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5187[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9100T Build/IMM76D)";
const char S5188[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5189[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9100G Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S5190[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; GT-I9100 Build/IMM76D.JPLPH) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5191[] = "Samsung GT-I9100 stagefright/1.2 (Linux;Android 4.0.4)";
const char S5192[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-I9100M Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5193[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9100/I9100BVLPJ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5194[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100-ORANGE/I9100BVLPJ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5195[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100 Build/IMM76L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S5196[] = "sprd-I9100/1.0 Linux/2.6.35.7 Android 4.0.1 Release/12.29.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S5197[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-US; GT-I9100 Build/ITL41D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5198[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; GT-I9100 Build/IMM76D.JHLPC) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5199[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-us; GT-I9100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S5200[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-US; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S5201[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100M Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S5202[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5203[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-I9100 Build/7comp_1.3.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5204[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG GT-I9100/I9100BVLPH Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5205[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; GT-I9100 Build/vauss miui.es) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5206[] = "GT-I9100 Linux/3.0.13 Android/4.1.9 Release/05.07.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S5207[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-I9100 Build/7comp_1.3.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5208[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; GT-I9100G Build/unknown) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S5209[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-us; GT-I9100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S5210[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-I9100 Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5211[] = "Dalvik/1.4.0 (Linux; U; Android 4.0.9; GT-I9100 Build/GRK39F)";
const char S5212[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/7.1.32052/30.3214; U; en) Presto/2.8.119 Version/11.10";
const char S5213[] = "Opera/9.50 (J2ME/MIDP; Opera Mini/7.1.32052/29.3757; U; en)";
const char S5214[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A4372q";
const char S5215[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko)";
const char S5216[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A401";
const char S5217[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A396";
const char S5218[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A4400f";
const char S5219[] = "Mozilla/5.0 (iPhone; U; CPU iPhone 7_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.8 (iPhone5,2/7.0)";
const char S5220[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko)";
const char S5221[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Mobile/11A4414e";
const char S5222[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Mobile/11A421";
const char S5223[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Mobile/11A4414c";
const char S5224[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko)";
const char S5225[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4435d";
const char S5226[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A458";
const char S5227[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449a";
const char S5228[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A462";
const char S5229[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449b";
const char S5230[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d";
const char S5231[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A450";
const char S5232[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A449";
const char S5233[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465";
const char S5234[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A464";
const char S5235[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A466";
const char S5236[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A470a";
const char S5237[] = "Mozilla/5.0 (iPhone4,1; iPhone; U; CPU OS 7_0 like Mac OS X; en_US) com.google.GooglePlus/23341 (KHTML, like Gecko) Mobile/N94AP (gzip)";
const char S5238[] = "Mozilla/5.0 (iPhone5,1; iPhone; U; CPU OS 7_0 like Mac OS X; en_US) com.google.GooglePlus/23341 (KHTML, like Gecko) Mobile/N41AP (gzip)";
const char S5239[] = "Mozilla/5.0 (iPhone5,2; iPhone; U; CPU OS 7_0 like Mac OS X; en_US) com.google.GooglePlus/23341 (KHTML, like Gecko) Mobile/N42AP (gzip)";
const char S5240[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko)";
const char S5241[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko)";
const char S5242[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 7_0 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone4,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/7.0;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S5243[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 bline/1.09 (iPhone OS 7.0, iPhone)";
const char S5244[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari/7534.48.3";
const char S5245[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501";
const char S5246[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11B470";
const char S5247[] = "Mozilla/5.0 (iPhone4,1; iPhone; U; CPU OS 7_0_2 like Mac OS X; en_US) com.google.GooglePlus/23341 (KHTML, like Gecko) Mobile/N94AP (gzip)";
const char S5248[] = "Mozilla/5.0 (iPhone5,2; iPhone; U; CPU OS 7_0_2 like Mac OS X; en_US) com.google.GooglePlus/23341 (KHTML, like Gecko) Mobile/N42AP (gzip)";
const char S5249[] = "Mozilla/5.0 (iPhone5,1; iPhone; U; CPU OS 7_0_2 like Mac OS X; en_US) com.google.GooglePlus/23341 (KHTML, like Gecko) Mobile/N41AP (gzip)";
const char S5250[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/28.0.1500.20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.20 Safari/537.36";
const char S5251[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/28.0.1500.20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.20 Safari/537.36";
const char S5252[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/28.0.1500.29) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.29 Safari/537.36";
const char S5253[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/28.0.1500.29) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.29 Safari/537.36";
const char S5254[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/28.0.1500.37) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.37 Safari/537.36";
const char S5255[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/28.0.1500.37) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.37 Safari/537.36";
const char S5256[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/28.0.1500.44) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.44 Safari/537.36";
const char S5257[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/28.0.1500.44) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.44 Safari/537.36";
const char S5258[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/28.0.1500.52) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36";
const char S5259[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/28.0.1500.52) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36";
const char S5260[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/28.0.1500.63) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36";
const char S5261[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/28.0.1500.63) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36";
const char S5262[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/28.0.1500.71) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36";
const char S5263[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/28.0.1500.71) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36";
const char S5264[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/28.0.1500.71; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)";
const char S5265[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/28.0.1500.72) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36";
const char S5266[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/28.0.1500.72) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36";
const char S5267[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/28.0.1500.95)";
const char S5268[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)";
const char S5269[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S5270[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0; chromeframe/28.0.1500.95) like Gecko";
const char S5271[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/28.0.1500.95)";
const char S5272[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S5273[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/28.0.1500.95)";
const char S5274[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S5275[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S5276[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/28.0.1500.95)";
const char S5277[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S5278[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S5279[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/28.0.1500.95) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S5280[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/28.0.1500.95; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S5281[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/28.0.1500.95) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S5282[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S5283[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S5284[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; MS-RTC LM 8; InfoPath.2)";
const char S5285[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C; .NET4.0E; 3M/MSIE 8.0)";
const char S5286[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/28.0.1500.95; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S5287[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S5288[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S5289[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Avant Browser; chromeframe/28.0.1500.95)";
const char S5290[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S5291[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/28.0.1500.95; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S5292[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S5293[] = "Mozilla/5.0 (compatible; MSIE 8.0; MSBrowserIE8; Windows NT 6.1; WOW64; Trident/5.0; MSBrowserIE9; chromeframe/28.0.1500.95)";
const char S5294[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.12 (KHTML, like Gecko) Version/6.0 Safari/536.12";
const char S5295[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.14 (KHTML, like Gecko) Version/6.0 Safari/536.14";
const char S5296[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.15 (KHTML, like Gecko) Version/6.0 Safari/536.16";
const char S5297[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.17 (KHTML, like Gecko) Version/6.0 Safari/536.17";
const char S5298[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.19 (KHTML, like Gecko) Version/6.0 Safari/536.19";
const char S5299[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.22 (KHTML, like Gecko) Version/6.0 Safari/536.22";
const char S5300[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.24 (KHTML, like Gecko) Version/6.0 Safari/536.24";
const char S5301[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.25 (KHTML, like Gecko) Version/6.0 Safari/536.25";
const char S5302[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.25 (KHTML, like Gecko) WebClip/8536.24 Safari/8536.25";
const char S5303[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.17 (KHTML, like Gecko) Version/6.0.2 Safari/536.26.17";
const char S5304[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) WebClip/8536.26.7 Safari/8536.26.17";
const char S5305[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_1; en) AppleWebKit/536.25 (KHTML, like Gecko) Version/6.0 Safari/536.25";
const char S5306[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8.2) AppleWebKit/8536.26.14 (KHTML, like Gecko) Version/6.0.2 Safari/8536.26.17 Raven for Mac/0.7.14665";
const char S5307[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1081) AppleWebKit/536.25 (KHTML like Gecko) Version/6.0 Safari/536.25";
const char S5308[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) Version/6.0.3 Safari/536.28.10";
const char S5309[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8) AppleWebKit/8536.25 (KHTML, like Gecko) Version/6.0 Safari/8536.25 Raven for Mac/0.7.14665";
const char S5310[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) WebClip/8536.28.3 Safari/8536.28.10";
const char S5311[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 108) AppleWebKit/536.22 (KHTML like Gecko) Version/6.0 Safari/536.22";
const char S5312[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 108) AppleWebKit/536.15 (KHTML like Gecko) Version/6.0 Safari/536.16";
const char S5313[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1082) AppleWebKit/536.26.14 (KHTML like Gecko) Version/6.0.1 Safari/536.26.14";
const char S5314[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) WebClip/8536.29.3 Safari/8536.29.13";
const char S5315[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.29.13 (KHTML, like Gecko) Version/6.0.4 Safari/536.29.13";
const char S5316[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_2; en) AppleWebKit/536.26.14 (KHTML, like Gecko) Version/6.0.1 Safari/536.26.14";
const char S5317[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) Version/6.0.5 Safari/536.30.1";
const char S5318[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) WebClip/8536.30 Safari/8536.30.1";
const char S5319[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1084) AppleWebKit/536.30.1 (KHTML like Gecko) Version/6.0.5 Safari/536.30.1";
const char S5320[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8.4) AppleWebKit/8536.30.1 (KHTML, like Gecko) Version/6.0.5 Safari/8536.30.1 Raven for Mac/0.7.14665";
const char S5321[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/538.1+ (KHTML, like Gecko) Version/6.0.5 Safari/538.1+";
const char S5322[] = "Opera/9.80 (BlackBerry; Opera Mini/7.1.33553/31.1475; U; en) Presto/2.8.119 Version/11.10";
const char S5323[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B5141a";
const char S5324[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 5_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/5.1;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]";
const char S5325[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 iPad2,1/K93AP Zite/1.3.3";
const char S5326[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 5_1_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/5.1.1;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]";
const char S5327[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176";
const char S5328[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206";
const char S5329[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B17";
const char S5330[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)";
const char S5331[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)";
const char S5332[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 iPad1,1/K48AP Zite/1.3.3";
const char S5333[] = "MQQBrowser/11 Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S5334[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405";
const char S5335[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/525.20";
const char S5336[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/525.20";
const char S5337[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S5338[] = "MQQBrowser/10 Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206";
const char S5339[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48 Instapaper/4.2.4 (+http://www.instapaper.com/iphone)";
const char S5340[] = "Mozilla/5.0 (iPad2; iPad; U; CPU OS 5_1_1 like Mac OS X; en_US) com.google.GooglePlus/8654 (KHTML, like Gecko) Mobile/K93AP (gzip)";
const char S5341[] = "MQQBrowser/1.9.1 Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3";
const char S5342[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9Z999";
const char S5343[] = "Mozilla/5.0 (iPad3G; iPad; U; CPU OS 5_1_1 like Mac OS X; en_US) com.google.GooglePlus/8654 (KHTML, like Gecko) Mobile/J1AP (gzip)";
const char S5344[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/5.1;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]";
const char S5345[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48 Instapaper/4.2.6 (+http://www.instapaper.com/iphone)";
const char S5346[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/8536.26.7 DiigoBrowser";
const char S5347[] = "MQQBrowser/1.9.1 Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S5348[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405";
const char S5349[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176";
const char S5350[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser";
const char S5351[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 YaMobile/1.00";
const char S5352[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/8536.26.7 DiigoBrowser";
const char S5353[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPad1,1;FBMD/iPad;FBSN/iPhone OS;FBSV/5.1.1;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]";
const char S5354[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/534.46";
const char S5355[] = "Miu Ptt 1.9.2 (iPad; iPhone OS 5.1.1; zh_TW)";
const char S5356[] = "QQ???HD 2.8.1 (iPad; iPhone OS 5.1.1; zh_CN)";
const char S5357[] = "Mozilla/5.0 (iPad1,1; iPad; U; CPU OS 5_1_1 like Mac OS X; en_US) com.google.GooglePlus/16077 (KHTML, like Gecko) Mobile/K48AP (gzip)";
const char S5358[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/534.46";
const char S5359[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/528.16";
const char S5360[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/534.42 (KHTML, like Gecko) Version/5.1 Safari/534.42";
const char S5361[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50";
const char S5362[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.50 (KHTML, like Gecko) WebClip/6534.49 Safari/6534.50";
const char S5363[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22";
const char S5364[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50";
const char S5365[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.52.7 (KHTML, like Gecko) WebClip/6534.51.13 Safari/6534.52.7";
const char S5366[] = "iChromy Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1 Safari/534.51.22";
const char S5367[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1 Safari/534.52.7";
const char S5368[] = "DiigoBrowser Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1 Safari/534.51.22";
const char S5369[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16";
const char S5370[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.54.16 (KHTML, like Gecko) WebClip/6534.51.13 Safari/6534.54.16";
const char S5371[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.48 (KHTML, like Gecko) Version/5.1 Safari/534.48";
const char S5372[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3";
const char S5373[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6.8) AppleWebKit/6534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/6534.52.7 Raven for Mac/0.7.14612";
const char S5374[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2";
const char S5375[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7";
const char S5376[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) WebClip/6534.51.13 Safari/6534.57.2";
const char S5377[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.55.3 (KHTML, like Gecko, Safari/6534.55.3) ADM/722";
const char S5378[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1 Safari/534.51.22 DiigoBrowser";
const char S5379[] = "Mozilla/5.0 (IMDb iOS; Intel Mac OS X 10_6_8) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50";
const char S5380[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3";
const char S5381[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) WebClip/6534.51.13 Safari/6534.51.13";
const char S5382[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.6 Safari/534.57.2";
const char S5383[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.5+ (KHTML, like Gecko) Version/5.1 Safari/533.5+";
const char S5384[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6.8) AppleWebKit/6534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/6534.57.2 Raven for Mac/0.7.14665";
const char S5385[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16";
const char S5386[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.42 (KHTML, like Gecko) Version/5.1 Safari/534.42";
const char S5387[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/534.34.4 (KHTML, like Gecko) Version/5.1 Safari/534.34.4";
const char S5388[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Cruz/0.4 Safari/534.57.2";
const char S5389[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10\\_6\\_8) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22";
const char S5390[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.58.2 (KHTML, like Gecko) Version/5.1.8 Safari/534.58.2";
const char S5391[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.58.2 (KHTML, like Gecko) WebClip/6534.51.13 Safari/6534.58.2";
const char S5392[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.59.8 (KHTML, like Gecko) Version/5.1.9 Safari/534.59.8";
const char S5393[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.59.8 (KHTML, like Gecko) WebClip/6534.51.13 Safari/6534.59.8";
const char S5394[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1068) AppleWebKit/534.52.7 (KHTML like Gecko) Version/5.1.2 Safari/534.52.7";
const char S5395[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.59.8 (KHTML, like Gecko) Version/5.1.8 Safari/534.59.8";
const char S5396[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10\\_6\\_8) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50";
const char S5397[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.59.8 (KHTML, like Gecko) Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en) AppleWebKit/534.59.8 (KHTML, like Gecko) Version/5.1.9 Safari/534.59.8";
const char S5398[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B5105c Safari/8536.25";
const char S5399[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B114 Safari/8536.25";
const char S5400[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B5117b Safari/8536.25";
const char S5401[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B5126b Safari/8536.25";
const char S5402[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B135 Safari/8536.25";
const char S5403[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B130 Safari/8536.25";
const char S5404[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B133 Safari/8536.25";
const char S5405[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B137 Safari/8536.25";
const char S5406[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S5407[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B143 Safari/8536.25";
const char S5408[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B136 Safari/8536.25";
const char S5409[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B142 Safari/8536.25";
const char S5410[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B144 Safari/8536.25";
const char S5411[] = "Mozilla/5.0 (iPhone; CPU iPhone OS6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B142 Safari/8536.25";
const char S5412[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B142Safari/8536.25";
const char S5413[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141Safari/8536.25";
const char S5414[] = "Mozilla/5.0 (iPhone; CPU iPhone OS6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S5415[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B311 Safari/8536.25";
const char S5416[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/9Z999 Safari/8536.25";
const char S5417[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B145 Safari/8536.25";
const char S5418[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S5419[] = "Mozilla/5.0 (iPhone; CPU iPhone OS6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B145 Safari/8536.25";
const char S5420[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S5421[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146Safari/8536.25";
const char S5422[] = "Mozilla/5.0 (iPhone; CPU iPhone OS6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146 Safari/8536.25";
const char S5423[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146 Safari/8536.25";
const char S5424[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B145Safari/8536.25";
const char S5425[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/6.0.5 Mobile/8B117 Safari/6531.22.7";
const char S5426[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B318 Safari/8536.25";
const char S5427[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S5428[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/9Z999 Safari/8536.25";
const char S5429[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329 Safari/8536.25";
const char S5430[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S5431[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B134 Safari/8536.25";
const char S5432[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329 Safari/8536.25";
const char S5433[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329Safari/8536.25";
const char S5434[] = "Mozilla/5.0 (iPhone; CPU iPhone OS6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329 Safari/8536.25";
const char S5435[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146 Safari/8536.25";
const char S5436[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S5437[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B350 Safari/8536.25";
const char S5438[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B350Safari/8536.25";
const char S5439[] = "Mozilla/5.0 (iPhone; CPU iPhone OS6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B350 Safari/8536.25";
const char S5440[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B131 Safari/8536.25";
const char S5441[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/9Z999 Safari/8536.25";
const char S5442[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B143 Safari/8536.25";
const char S5443[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S5444[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S5445[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S5446[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146 Safari/8536.25";
const char S5447[] = "Mozilla/5.0 (Linux; en-us;)";
const char S5448[] = "Mozilla/4.61 [en] (X11; U; ) - BrowseX (2.0.0 Windows)";
const char S5449[] = "curl/7.15.5 (x86_64-redhat-linux-gnu) libcurl/7.15.5 OpenSSL/0.9.8b zlib/1.2.3 libidn/0.6.5";
const char S5450[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-us) AppleWebKit/525.1+ (KHTML, like Gecko, Safari/525.1+) pythumbnail.py";
const char S5451[] = "curl/7.15.5 (i686-redhat-linux-gnu) libcurl/7.15.5 OpenSSL/0.9.8b zlib/1.2.3 libidn/0.6.5";
const char S5452[] = "curl/7.19.7 (x86_64-pc-linux-gnu) libcurl/7.19.7 OpenSSL/0.9.8k zlib/1.2.3.3 libidn/1.15";
const char S5453[] = "Mozilla/3.01Gold (X11; I; Linux 2.0.32 i486)";
const char S5454[] = "curl/7.19.7 (i486-pc-linux-gnu) libcurl/7.19.7 OpenSSL/0.9.8k zlib/1.2.3.3 libidn/1.15";
const char S5455[] = "Mozilla/5.0 (X11; U; Linux; en-US) AppleWebKit/531.2 (KHTML, like Gecko) Safari/531.2";
const char S5456[] = "Mozilla/4.5b1 [en] (X11; I; Linux 2.0.35 i586)";
const char S5457[] = "Mozilla/3.01 (compatible; Netbox/3.5 R92; Linux 2.2)";
const char S5458[] = "Mozilla/4.72C-CCK-MCD Caldera Systems OpenLinux [en] (X11; U; Linux 2.2.14 i686)";
const char S5459[] = "Links (2.xpre7; Linux 2.4.18 i586; x)";
const char S5460[] = "Links (0.9xpre12; Linux 2.2.14-5.0 i686; 80x24)";
const char S5461[] = "Mozilla/5.0 (X11; U; Linux i686; de-DE) AppleWebKit/534.3 (KHTML, like Gecko) WeTab-Browser Safari/534.3";
const char S5462[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Arora/0.10.2 Safari/533.3";
const char S5463[] = "Mozilla/5.0 (X11; U; Linux; ja-JP) AppleWebKit/527+ (KHTML, like Gecko, Safari/419.3)";
const char S5464[] = "Mozilla/5.0 (compatible; Konqueror/3.2; Linux 2.6.2)";
const char S5465[] = "Mozilla/5.0 (X11; U; Linux i686; en-US)";
const char S5466[] = "Mozilla/5.0 (compatible; Konqueror/3.1; Linux; en)";
const char S5467[] = "Mozilla/5.0 (DirectFB; U; Linux 35230; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+";
const char S5468[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Boxee/1.0.1 bxapi/7.1";
const char S5469[] = "Mozilla/5.0 (X11; U; Linux i686; undefined) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.7";
const char S5470[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Epiphany/2.30.6 Safari/534.7";
const char S5471[] = "curl/7.21.2 (x86_64-unknown-linux-gnu) libcurl/7.21.2 OpenSSL/1.0.0c zlib/1.2.5 libidn/1.15 libssh2/1.2.7";
const char S5472[] = "Infoseek SideWinder/2.0B (Linux 2.4 i686)";
const char S5473[] = "curl/7.21.0 (x86_64-pc-linux-gnu) libcurl/7.21.0 OpenSSL/0.9.8o zlib/1.2.3.4 libidn/1.15 libssh2/1.2.6";
const char S5474[] = "Mozilla/5.0 (Linux; webOS/2.2.0; U; en-GB) AppleWebKit/534.6 (KHTML, like Gecko) webOSBrowser/221.54 Safari/534.6 Pre/3.0";
const char S5475[] = "Mozilla/5.0 (Linux; webOS/2.2.3; U; en-US) AppleWebKit/534.6 (KHTML, like Gecko) webOSBrowser/221.54 Safari/534.6 Pre/3.0";
const char S5476[] = "Mozilla/5.0 (Unknown; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) FbxQmlTV Safari/534.34";
const char S5477[] = "Mozilla/5.0 (compatible; Konqueror/3.3; Linux) (KHTML, like Gecko)";
const char S5478[] = "Mozilla/5.0 (X11; U; Linux i686; c) AppleWebKit/528.5+ (KHTML, like Gecko, Safari/528.5+) <unknown>";
const char S5479[] = "Mozilla/4.73 [en] (X11; U; Linux 2.2.15 i686)";
const char S5480[] = "Mozilla/5.0 (U; Linux) AppleWebKit/533.3 (KHTML, like Gecko) konqueror/4.6.0 Safari/533.3";
const char S5481[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.6";
const char S5482[] = "Mozilla/5.0 (Linux; webOS/2.2.0; U; en-US) AppleWebKit/534.6 (KHTML, like Gecko) webOSBrowser/221.53 Safari/534.6 Pre/3.0";
const char S5483[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ Debian/sid/experimental (3.0.4-1) Epiphany/3.0.4";
const char S5484[] = "Mozilla/5.0 (X11; Linux i686) PrintFriendly.com";
const char S5485[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.4+ (KHTML, like Gecko) Version/5.0 Safari/535.4+ Debian/sid/experimental (3.2.1-2) Epiphany/3.2.1";
const char S5486[] = "Mozilla/5.0 (Linux; webOS/2.2.4; U; en-GB) AppleWebKit/534.6 (KHTML, like Gecko) webOSBrowser/221.56 Safari/534.6 Pre/1.2";
const char S5487[] = "Mozilla/5.0 (Linux; webOS/2.2.4; U; en-CA) AppleWebKit/534.6 (KHTML, like Gecko) webOSBrowser/221.56 Safari/534.6 Pre/1.2";
const char S5488[] = "Mozilla/5.0 (Linux; webOS/2.2.4; U; en-GB) AppleWebKit/534.6 (KHTML, like Gecko) webOSBrowser/221.56 Safari/534.6 Pre/3.0";
const char S5489[] = "curl/7.21.0 (i486-pc-linux-gnu) libcurl/7.21.0 OpenSSL/0.9.8o zlib/1.2.3.4 libidn/1.15 libssh2/1.2.6";
const char S5490[] = "curl/7.15.1 (x86_64-suse-linux) libcurl/7.15.1 OpenSSL/0.9.8a zlib/1.2.3 libidn/0.6.0";
const char S5491[] = "curl/7.19.7 (x86_64-unknown-linux-gnu) libcurl/7.19.7 NSS/3.12.6.2 zlib/1.2.3 libidn/1.18 libssh2/1.2.2";
const char S5492[] = "curl/7.19.7 (x86_64-redhat-linux-gnu) libcurl/7.19.7 NSS/3.13.1.0 zlib/1.2.3 libidn/1.18 libssh2/1.2.2";
const char S5493[] = "Mozilla/5.0 Galeon/1.2.0 (X11; Linux i686; U;) Gecko/X";
const char S5494[] = "Mozilla/5.0 (X11; U; Linux i686; ) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.1";
const char S5495[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/533.3 (KHTML, like Gecko) kded/4.7.2 Safari/533.3";
const char S5496[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) imgsizer Safari/534.34";
const char S5497[] = "curl/7.21.6 (x86_64-pc-linux-gnu) libcurl/7.21.6 OpenSSL/1.0.0e zlib/1.2.3.4 libidn/1.22 librtmp/2.3";
const char S5498[] = "curl/7.22.0 (x86_64-pc-linux-gnu) libcurl/7.22.0 OpenSSL/1.0.1 zlib/1.2.3.4 libidn/1.23 librtmp/2.3";
const char S5499[] = "Mozilla/5.0 (X11; U; Linux i686; undefined) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.5";
const char S5500[] = "Mozilla/5.0 (X11; U; Linux i686; en-US;)";
const char S5501[] = "curl/7.22.0 (i686-pc-linux-gnu) libcurl/7.22.0 OpenSSL/1.0.1 zlib/1.2.3.4 libidn/1.23 librtmp/2.3";
const char S5502[] = "Mozilla/5.0 (X11; Linux x86_64) Superfish/1.1 (platform image loader)";
const char S5503[] = "curl/7.19.4 (i386-redhat-linux-gnu) libcurl/7.19.4 NSS/3.12.2.0 zlib/1.2.3 libidn/0.6.14 libssh2/0.18";
const char S5504[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/533.3 (KHTML, like Gecko) kded/4.8.5 Safari/533.3";
const char S5505[] = "python-requests/0.14.2 CPython/2.7.3 Linux/3.2.0-33-generic";
const char S5506[] = "python-requests/0.14.2 CPython/2.7.3 Linux/3.2.0-34-generic";
const char S5507[] = "Mozilla/5.0 (X11; Linux x86_64; rv:7.0.1) Gecko/ /7.0.1,gzip(gfe) (via docs.google.com/viewer)";
const char S5508[] = "Mozilla/5.0+(X11;+U;+Linux+i686;+en-US;+rv:1.7.3)+Gecko/X";
const char S5509[] = "python-requests/1.1.0 CPython/2.7.3 Linux/3.2.0-36-virtual";
const char S5510[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) kded/4.8.4 Safari/534.34";
const char S5511[] = "Mozilla/5.0 (X11; U; Linux; en-US) AppleWebKit/532.4 (KHTML, like Gecko)";
const char S5512[] = "Mozilla/4_77 [en] (X11; I; IRIX;64 6_5 IP30";
const char S5513[] = "python-requests/1.1.0 CPython/2.7.3 Linux/3.2.0-34-generic";
const char S5514[] = "python-requests/0.14.2 CPython/2.7.2+ Linux/3.2.13-grsec-xxxx-grs-ipv6-64";
const char S5515[] = "python-requests/1.1.0 CPython/2.6.5 Linux/2.6.32-346-ec2";
const char S5516[] = "python-requests/1.2.0 CPython/2.7.3 Linux/3.2.0-29-virtual";
const char S5517[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) kded/4.9.4 Safari/534.34";
const char S5518[] = "curl/7.19.7 (x86_64-redhat-linux-gnu) libcurl/7.19.7 NSS/3.14.0.0 zlib/1.2.3 libidn/1.18 libssh2/1.4.2";
const char S5519[] = "python-requests/1.0.4 CPython/2.7.3 Linux/3.2.37.1.amd64-smp";
const char S5520[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.11 (KHTML, like Gecko) DumpRenderTree/0.0.0.0 Safari/536.11";
const char S5521[] = "python-requests/1.2.0 CPython/2.7.4 Linux/3.8.0-21-generic";
const char S5522[] = "python-requests/1.2.3 CPython/2.7.3 Linux/3.9.3-x86_64-linode33";
const char S5523[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.21 (KHTML, like Gecko) imgsizer Safari/537.21";
const char S5524[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) kded/4.10.4 Safari/534.34";
const char S5525[] = "Mozilla/5.0 (X11; U; Linux i686; zh-CN; rv:1.9.1.2) Gecko/X";
const char S5526[] = "python-requests/1.2.3 CPython/2.6.6 Linux/3.2.0-0.bpo.4-amd64";
const char S5527[] = "python-requests/1.1.0 CPython/2.7.4 Linux/3.8.0-19-generic";
const char S5528[] = "python-requests/1.2.3 CPython/2.7.3 Linux/3.2.0-4-amd64";
const char S5529[] = "python-requests/1.2.3 CPython/2.7.3 Linux/3.3.8-gcg-X";
const char S5530[] = "python-requests/1.2.3 CPython/2.6.8 Linux/3.4.43-43.43.amzn1.x86_64";
const char S5531[] = "python-requests/1.2.3 CPython/2.7.3 Linux/3.5.0-23-generic";
const char S5532[] = "Opera/9.80 (Series 60; Opera Mini/7.1.32444/31.1475; U; en) Presto/2.8.119 Version/11.10";
const char S5533[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/416.11 (KHTML, like Gecko) Safari/416.12";
const char S5534[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/412.7 (KHTML, like Gecko) Safari/412.5";
const char S5535[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/312.8 (KHTML, like Gecko) Safari/312.5";
const char S5536[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/418 (KHTML, like Gecko) Safari/417.9.2";
const char S5537[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/312.9 (KHTML, like Gecko) Safari/312.6";
const char S5538[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/125.4 (KHTML, like Gecko) Safari/125.9";
const char S5539[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/124 (KHTML, like Gecko) Safari/125.1";
const char S5540[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/85.7 (KHTML, like Gecko) Safari/85.5";
const char S5541[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/417.9 (KHTML, like Gecko) Safari/417.8";
const char S5542[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/125.2 (KHTML, like Gecko) Safari/125.8";
const char S5543[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/12.0.0";
const char S5544[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/521.25 (KHTML, like Gecko) Safari/521.24";
const char S5545[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/125.5.5 (KHTML, like Gecko) Safari/125.11";
const char S5546[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) Excel/14.0.0";
const char S5547[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/106.2 (KHTML, like Gecko) Safari/100.1";
const char S5548[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/14.0.0";
const char S5549[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/124 (KHTML, like Gecko) Safari/125";
const char S5550[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/522.11 (KHTML, like Gecko) Safari/3.0.2";
const char S5551[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/523.10.3 (KHTML, like Gecko)";
const char S5552[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/14.10.0";
const char S5553[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.7";
const char S5554[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/419 (KHTML, like Gecko) NetNewsWire/3.0";
const char S5555[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.7.1";
const char S5556[] = "Pulp/2.1.2 (Mac OS X; http://www.acrylicapps.com/pulp/)";
const char S5557[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.0";
const char S5558[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; fr-fr) AppleWebKit/312.5 (KHTML, like Gecko) Safari/312.3";
const char S5559[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/412 (KHTML, like Gecko) Pandora/2.0";
const char S5560[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/412.6 (KHTML, like Gecko) Safari/412.2";
const char S5561[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/417.3 (KHTML, like Gecko) Safari/417.2";
const char S5562[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/416.12 (KHTML, like Gecko) Safari/416.13";
const char S5563[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/417.9 (KHTML, like Gecko) HistoryHound/1.9";
const char S5564[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/125.5.7 (KHTML, like Gecko) Safari/125.12";
const char S5565[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.5.1";
const char S5566[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.6";
const char S5567[] = "Pulp/2.0.1 (Mac OS X; http://www.acrylicapps.com/pulp/)";
const char S5568[] = "Pulp/2.0 (Mac OS X; http://www.acrylicapps.com/pulp/)";
const char S5569[] = "Pulp/2.1 (Mac OS X; http://www.acrylicapps.com/pulp/)";
const char S5570[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; de-de) AppleWebKit/125.2 (KHTML, like Gecko) Safari/125.7";
const char S5571[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/523.12 (KHTML, like Gecko, Safari/523.12) ADM/722";
const char S5572[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/526.9+ (KHTML, like Gecko) AdobeAIR/1.5.3";
const char S5573[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/85.7 (KHTML, like Gecko) Safari/85.7";
const char S5574[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.0.2";
const char S5575[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X;en) AppleWebKit/419.2.1 (KHTML, like Gecko) Safari/419.4";
const char S5576[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.2";
const char S5577[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/14.20.0";
const char S5578[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.1";
const char S5579[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/125.2 (KHTML, like Gecko) Safari/85.8";
const char S5580[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/412 (KHTML, like Gecko) Pandora/2.3";
const char S5581[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) VIDEODOWNLOAD Safari/533.3";
const char S5582[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; pl-PL; rv:1.0.1) Gecko/X Chimera/0.6";
const char S5583[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.5) Gecko/X Firebird/0.7";
const char S5584[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/521.32.1 (KHTML, like Gecko) Safari/521.32.1";
const char S5585[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/85.7 (KHTML, like Gecko) Safari/85.6";
const char S5586[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/412 (KHTML, like Gecko) Skyscraper/2.0";
const char S5587[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/125.5.5 (KHTML, like Gecko) Safari/125.5.5";
const char S5588[] = "Pulp/2.5.1 (Mac; http://www.acrylicapps.com/pulp/)";
const char S5589[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.3";
const char S5590[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/14.23.0";
const char S5591[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/412.7 (KHTML, like Gecko) Safari/412.6";
const char S5592[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/125.5.7 (KHTML, like Gecko) SunriseBrowser/0.853";
const char S5593[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.4";
const char S5594[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/14.24.0";
const char S5595[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/523.15 (KHTML, like Gecko) Safari/419.3 Qt";
const char S5596[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-US) AppleWebKit/85 (KHTML, like Gecko) OmniWeb/v496";
const char S5597[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-US) AppleWebKit/85 (KHTML, like Gecko) OmniWeb/v558.48";
const char S5598[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.6.1 Safari/534.34";
const char S5599[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) AppleWebKit/534.34 (KHTML, like Gecko) mention/1.1.24 Safari/534.34";
const char S5600[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/417.2 (KHTML, like Gecko) Safari/417.1";
const char S5601[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/418.9 (KHTML, like Gecko) AppleWebKit/418.9 Cheshire/1.0.ALPHA";
const char S5602[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.5";
const char S5603[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/14.25.0";
const char S5604[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/12.35.0";
const char S5605[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.7.0 Safari/534.34";
const char S5606[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/521.9 (KHTML, like Gecko) Safari/521.9";
const char S5607[] = "Pulp/2.5.3 (Mac; http://www.acrylicapps.com/pulp/)";
const char S5608[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/14.31.0";
const char S5609[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.6";
const char S5610[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/14.32.0";
const char S5611[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/12.36.0";
const char S5612[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) AppleWebKit/535.14 (KHTML, like Gecko) JavaFX/2.2 Safari/535.14";
const char S5613[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.7";
const char S5614[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/14.34.0";
const char S5615[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/14.35.0";
const char S5616[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/728.18 (KHTML, like Gecko) Safari/728.18";
const char S5617[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/14.36.0";
const char S5618[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.13) Gecko/20101203 Firefox/3.6.13 (.NET CLR 3.5.30729)";
const char S5619[] = "Mozilla/5.0 (X11; U; Windows NT 6.0; en-US; rv:1.9.2) Gecko/X Firefox/3.6";
const char S5620[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)(Mozilla/5.0 (Windows; U; Windows NT 6.0; nl; rv:1.9.2.6) Gecko/X Firefox/3.6.6); SLCC2; .NET CLR 2.0.50727; .N";
const char S5621[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-GB; rv:1.9.2.3) Gecko/X Firefox/3.6.3 ( .NET CLR 3.5.30729; .NET4.0C) WinTSI 25.04.2010";
const char S5622[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; fr; rv:1.9.2.20) Gecko/X Firefox/3.6.20 (Windows NT 5.1; rv:5.0) Gecko/X Firefox/5.0";
const char S5623[] = "Mozilla/5.0(Windows;U;Windows NT 6.0;en-US;rv:1.9.2.8)Gecko/X Firefox/3.6.8";
const char S5624[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/27.0.1453.15) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.15 Safari/537.36";
const char S5625[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/27.0.1453.47) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.47 Safari/537.36";
const char S5626[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/27.0.1453.47) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.47 Safari/537.36";
const char S5627[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/27.0.1453.56) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.56 Safari/537.36";
const char S5628[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/27.0.1453.56) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.56 Safari/537.36";
const char S5629[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/27.0.1453.65) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.65 Safari/537.36";
const char S5630[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/27.0.1453.73) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.73 Safari/537.36";
const char S5631[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/27.0.1453.73) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.73 Safari/537.36";
const char S5632[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/27.0.1453.65) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.65 Safari/537.36";
const char S5633[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/27.0.1453.81) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.81 Safari/537.36";
const char S5634[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/27.0.1453.81) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.81 Safari/537.36";
const char S5635[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/27.0.1453.93) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36";
const char S5636[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/27.0.1453.93) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36";
const char S5637[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/27.0.1453.94; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.2)";
const char S5638[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/27.0.1453.94) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36";
const char S5639[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/27.0.1453.94; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S5640[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/27.0.1453.94) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36";
const char S5641[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/27.0.1453.94; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S5642[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/27.0.1453.110) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36";
const char S5643[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/27.0.1453.110) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36";
const char S5644[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/27.0.1453.15) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.15 Safari/537.36";
const char S5645[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/27.0.1453.116)";
const char S5646[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/27.0.1453.116)";
const char S5647[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/27.0.1453.116)";
const char S5648[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/27.0.1453.116) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S5649[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/27.0.1453.116)";
const char S5650[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/27.0.1453.116) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S5651[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/27.0.1453.116; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S5652[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/27.0.1453.116; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S5653[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/27.0.1453.116; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S5654[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/27.0.1453.116; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S5655[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/27.0.1453.116; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S5656[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/27.0.1453.116; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S5657[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/27.0.1453.116; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S5658[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/27.0.1453.116; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S5659[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/27.0.1453.116; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S5660[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/27.0.1453.116; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S5661[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/27.0.1453.116; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S5662[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/27.0.1453.94; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S5663[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F190 Safari/6533.18.5";
const char S5664[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8G4 Safari/6533.18.5";
const char S5665[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9Z999 Safari/6533.18.5";
const char S5666[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5";
const char S5667[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5";
const char S5668[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C99 Safari/6533.18.5";
const char S5669[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C99 Safari/6533.18.5";
const char S5670[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C99 Safari/6533.18.5";
const char S5671[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C99 Safari/6533.18.5";
const char S5672[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8K2 Safari/6533.18.5";
const char S5673[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5";
const char S5674[] = "Mozilla/5.0 (iPhone Simulator; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F192 Safari/6533.18.5";
const char S5675[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J3 Safari/6533.18.5";
const char S5676[] = "Mozilla/5.0 (device; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5";
const char S5677[] = "Mozilla/5.0 (; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5";
const char S5678[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9Z999 Safari/6533.18.5";
const char S5679[] = "Mozilla/5.0 (; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8G4 Safari/6533.18.5";
const char S5680[] = "Mozilla/5.0 (; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5";
const char S5681[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9Z999 Safari/6533.18.5";
const char S5682[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5";
const char S5683[] = "Mozilla/5.0 (device; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8G4 Safari/6533.18.5";
const char S5684[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5";
const char S5685[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en_us) AppleWebKit/6533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5";
const char S5686[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; 6533.17.9) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5";
const char S5687[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F5153d Safari/6533.18.5";
const char S5688[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en_us) AppleWebKit/6533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5";
const char S5689[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8K2 Safari/6533.18.5";
const char S5690[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; 6533.17.9) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5";
const char S5691[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S5692[] = "Mozilla/5.0 (iPhone; CPU OS 4_3_3 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0 Mobile/8J2 Safari/7533.17.9";
const char S5693[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S5694[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; es-es) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S5695[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5";
const char S5696[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 43 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8F190 Safari/6533.18.5";
const char S5697[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 433 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5";
const char S5698[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 435 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5";
const char S5699[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 432 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5";
const char S5700[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 434 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8K2 Safari/6533.18.5";
const char S5701[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 431 like Mac OS X; es-US) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8G4 Safari/6533.18.5";
const char S5702[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S5703[] = "Mozilla/5.0 (iPhone; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.02 Guidebook/39 Mobile/8L1 Safari/6533.18.5";
const char S5704[] = "Mozilla/5.0 (iphone; U; CPU iPhone OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5";
const char S5705[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5";
const char S5706[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5";
const char S5707[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; es-es) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5";
const char S5708[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7; en-us) AppleWebKit/534.20.8 (KHTML, like Gecko) Version/5.1 Safari/534.20.8";
const char S5709[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7) AppleWebKit/534.31.2 (KHTML, like Gecko) Version/5.1 Safari/534.31.2";
const char S5710[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7) AppleWebKit/534.42 (KHTML, like Gecko) Version/5.1 Safari/534.42";
const char S5711[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7) AppleWebKit/534.34.4 (KHTML, like Gecko) Version/5.1 Safari/534.34.4";
const char S5712[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Safari/534.45";
const char S5713[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3";
const char S5714[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/534.49.1 (KHTML, like Gecko) Version/5.1 Safari/534.49";
const char S5715[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22";
const char S5716[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/534.51.22 (KHTML, like Gecko) WebClip/7534.51.13 Safari/7534.51.22";
const char S5717[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7";
const char S5718[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7) AppleWebKit/534.26.4 (KHTML, like Gecko) Version/5.1 Safari/534.26.4";
const char S5719[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7) AppleWebKit/534.28.5 (KHTML, like Gecko) Version/5.1 Safari/534.28.5";
const char S5720[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7) AppleWebKit/534.48.3 (KHTML, like Gecko) WebClip/7534.48 Safari/7534.48.3";
const char S5721[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1 Safari/534.50";
const char S5722[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.53.11 (KHTML, like Gecko) Version/5.1.3 Safari/534.53.10";
const char S5723[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.53.11 (KHTML, like Gecko) WebClip/7534.53.1 Safari/7534.53.10";
const char S5724[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.53.11 (KHTML, like Gecko) WebClip/7534.54.15 Safari/7534.54.16";
const char S5725[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3";
const char S5726[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) Version/5.1.6 Safari/534.56.5";
const char S5727[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16";
const char S5728[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.53.11 (KHTML, like Gecko) WebClip/7534.55.2 Safari/7534.55.3";
const char S5729[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/534.52.7 (KHTML, like Gecko) WebClip/7534.51.13 Safari/7534.52.7";
const char S5730[] = "Mozilla/5.0 Macintosh Intel Mac OS X 10_7_2 AppleWebKit/534.52.7 KHTML,like Gecko Version/5.1.2 Safari/534.52.7";
const char S5731[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7) AppleWebKit/534.47 (KHTML, like Gecko) Version/5.1 Safari/534.47";
const char S5732[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) WebClip/7534.56.5 Safari/7534.56.5";
const char S5733[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7.3) AppleWebKit/7534.53.11 (KHTML, like Gecko) Version/5.1.3 Safari/7534.53.10 Raven for Mac/0.7.14612";
const char S5734[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7.3) AppleWebKit/7534.53.11 (KHTML, like Gecko) Version/5.1.5 Safari/7534.55.3 Raven for Mac/0.7.14665";
const char S5735[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1.5 Safari/534.46";
const char S5736[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7.4) AppleWebKit/7534.56.5 (KHTML, like Gecko) Version/5.1.7 Safari/7534.57.2 Raven for Mac/0.7.14665";
const char S5737[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.7";
const char S5738[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) WebClip/7534.57.2 Safari/7534.57.7";
const char S5739[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7; en-us) AppleWebKit/534.15.15 (KHTML, like Gecko) Version/5.1 Safari/534.15.16";
const char S5740[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7) AppleWebKit/534.43 (KHTML, like Gecko) Version/5.1 Safari/534.43";
const char S5741[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10\\_7\\_2) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22";
const char S5742[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 107) AppleWebKit/534.48.3 (KHTML like Gecko) Version/5.1 Safari/534.48.3gzip(gfe)";
const char S5743[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10\\_7\\_2) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7";
const char S5744[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.1.8 Safari/534.58.2";
const char S5745[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.1.9 Safari/534.59.8";
const char S5746[] = "Digg Feed Fetcher 1.0 (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)";
const char S5747[] = "Digg Fetcher 1.0 (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)";
const char S5748[] = "Betaworks Data Science 1.0 (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1)AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)";
const char S5749[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en) AppleWebKit/534.56.5 (KHTML, like Gecko) Version/5.1.6 Safari/534.56.5";
const char S5750[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/26.0.1386.0) AppleWebKit/537.27 (KHTML, like Gecko) Chrome/26.0.1386.0 Safari/537.27";
const char S5751[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/26.0.1410.19) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.19 Safari/537.31";
const char S5752[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/26.0.1410.28) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.28 Safari/537.31";
const char S5753[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/26.0.1410.33) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.33 Safari/537.31";
const char S5754[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/26.0.1410.40) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Safari/537.31";
const char S5755[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/26.0.1410.43) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/537.31";
const char S5756[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/26.0.1410.43) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/537.31";
const char S5757[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/26.0.1410.43; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S5758[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S5759[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S5760[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S5761[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)";
const char S5762[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S5763[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/26.0.1410.64)";
const char S5764[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/26.0.1410.64)";
const char S5765[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/26.0.1410.64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S5766[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S5767[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/26.0.1410.64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S5768[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/26.0.1410.64)";
const char S5769[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/26.0.1410.64)";
const char S5770[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Tablet PC 2.0)";
const char S5771[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Tablet PC 2.0)";
const char S5772[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)";
const char S5773[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/26.0.1410.64; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S5774[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)";
const char S5775[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Avant Browser; chromeframe/26.0.1410.64)";
const char S5776[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/26.0.1410.64; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S5777[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S5778[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Zune 4.7)";
const char S5779[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/26.0.1410.64; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S5780[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/26.0.1410.12) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.12 Safari/537.31";
const char S5781[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S5782[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S5783[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S5784[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S5785[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6136; ms-office; MSOffice 14)";
const char S5786[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S5787[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/7.0.31438/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S5788[] = "Opera/9.50 (J2ME/MIDP; Opera Mini/7.0.32351/29.3757; U; en)";
const char S5789[] = "Opera/9.80 (MTK; Opera Mini/7.0.31243/30.3214; U; en) Presto/2.8.119 Version/11.10";
const char S5790[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0)";
const char S5791[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; SV1; .NET CLR 1.1.4322; InfoPath.2; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022)";
const char S5792[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322)";
const char S5793[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; QS 4.2.1.0; Windows 2000; .NET CLR 1.1.4322; InfoPath.1)";
const char S5794[] = "Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 5.0)";
const char S5795[] = "Mozilla/4.0 (MSIE 6.0; Windows NT 5.0)";
const char S5796[] = "Mozilla/4.0 (Windows; MSIE 6.0; Windows NT 5.0)";
const char S5797[] = "Mozilla/4.0 (compatible, MSIE 6.0, Windows NT 5.0)";
const char S5798[] = "Mozilla/4.0 (compatible; MSIE 6.01; Windows NT 5.0)";
const char S5799[] = "ChangeDetect/1.0 (compatible; MSIE 6.0; Windows NT 5.0)";
const char S5800[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; WOW64; Trident/4.0; SLCC1)";
const char S5801[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S5802[] = "Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 5.0; Trident/5.0)";
const char S5803[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S5804[] = "BlackBerry8520_CMCC/5.0.0.1056 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/204";
const char S5805[] = "BlackBerry8520/5.0.0.592 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/168";
const char S5806[] = "MQQBrowser/for Blackberry2.0.0 (BlackBerry8520/5.0.0.592)";
const char S5807[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Full Android on Wingray Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5808[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; I9000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5809[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Ventana Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5810[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; QW TB-1207 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5811[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; XELIO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5812[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; MID1200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5813[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Full AOSP on Maguro Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5814[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PTM805HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5815[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; PC677 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5816[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; CRYM752HCF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5817[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; msm8960 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5818[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; asus_laptop Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5819[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Q8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5820[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Notion Ink ADAM Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5821[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CMP745d Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5822[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4 Build/IMM76D)";
const char S5823[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; mfld_pr3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5824[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; imito am801 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5825[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M760W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5826[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; 97FC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5827[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; DFP7005 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5828[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; ALODM712HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5829[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; RubiX ICS Infused Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S5830[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PTM723HR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5831[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; generic_vortex Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5832[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; Gpad Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5833[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; NAM1006HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5834[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; mfld_dv10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5835[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Newsmy Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5836[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A10-MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5837[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexusone Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5838[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; V700B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5839[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VegaCream_BETA-2.1 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5840[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; PC657 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5841[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M1001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5842[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 3C97 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5843[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN8BG3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5844[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; EXLP2W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5845[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Full Android on x86 VirtualBox Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5846[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; QW TB-1217 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5847[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; M-MP711i Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5848[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VegaCream Beta 3 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5849[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4 Build/IMM76I)";
const char S5850[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A10_MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5851[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; RICATECH-DR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5852[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; EVOTAB 2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5853[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; EXLP2P Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5854[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; MVPM727HCF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5855[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; VISION Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5856[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; i-Joy-Andromeda Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5857[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; C906 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5858[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; RK2918 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5859[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4 Build/IMM76L)";
const char S5860[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; AllviewSpeed2+ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5861[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Informer 701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5862[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M9701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5863[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M701C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5864[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; TAB-PROTAB2.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5865[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; JOYPAD C71 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5866[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; Tablet724W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5867[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M711 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5868[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; TechTAB Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5869[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EPAD Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5870[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; QPad-C-X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5871[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-7200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5872[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Notion Ink ADAM Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5873[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TB782B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5874[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; hu-hu; WayteQ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5875[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M1001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5876[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; Gpad Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5877[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; hu-hu; HFM752HCF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5878[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; Newpad-K97 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5879[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; 1040C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5880[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; H5500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5881[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TRIO 7C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5882[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; RK701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5883[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A10MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5884[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; SP-120 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5885[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M760 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5886[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; X7GS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5887[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; msm8960 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5888[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DFP7005B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5889[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; NEXT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5890[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SXZ-PDX0-01 Build/SXZ-PDX0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5891[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; B920 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S5892[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; DOMO_Slate_N9SE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5893[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Newsmy P72-C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5894[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; WP7001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5895[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; amd_brazos Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5896[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; TAB200PR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5897[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; TOTEMPO-sun Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S5898[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; 970 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5899[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; CRYM752HC_C930 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5900[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; PC1008R Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5901[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AK-MINI-PC-001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5902[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; QW TB-1210 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5903[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TAB364 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5904[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; G-PAD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5905[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; ActiveTab Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5906[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; B001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5907[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Tablet PC T-702 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5908[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Full AOSP on Maguro Build/IML74K)";
const char S5909[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M1005 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5910[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; T-90 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5911[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A721 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5912[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; WT-M768 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5913[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; D70W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5914[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; wetab Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5915[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Broncho N802 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5916[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Allwinner A10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5917[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID-A10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5918[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Newpad Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5919[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; Z5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5920[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A1001T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5921[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; NVSBL DRAGON Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5922[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; BiPad 9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5923[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; Nexus EvoHd2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5924[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; vm Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5925[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; vivow Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5926[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; AT10H-A10WP Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5927[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; M70 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5928[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VegaCream_BETA-2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5929[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SANEI Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5930[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LR97A01 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5931[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; el-gr; \"Turbo-X BREEZE\" Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5932[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; hyundai S900 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5933[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DFP1008R Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5934[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; PI E002 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5935[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; PC7315 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5936[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-pt; TB700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5937[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; Multilaser Diamond Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5938[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; Pentagram TAB80 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5939[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MYIMAN-D07 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5940[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; 3C97 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5941[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; i9701ips Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5942[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; EB-A71GJ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5943[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; i-Joy-Andromeda-3G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5944[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; OR-TB17 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5945[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M805 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5946[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LUBM816HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5947[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; Artes i701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5948[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Tablet7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5949[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ENERGY I828HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5950[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; X2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5951[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; PC728 Build/PC728) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5952[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PITM727HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5953[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-7205S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5954[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VegaCream RC2 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5955[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AR10e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5956[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; asus_laptop Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5957[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TB1082B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5958[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; AN8BG3-LZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5959[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; SP501 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5960[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; 1000TPC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5961[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; TAB200BR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5962[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Tab 820 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5963[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Junior 4.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5964[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TVPAD Slim K3409 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5965[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A710 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5966[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; TB-207 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5967[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; CT0712 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5968[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VegaCream Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5969[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GL-7TA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5970[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; MID06S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5971[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Yan9.7G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5972[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; GASEM712HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5973[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; TAB200R Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5974[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T6 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5975[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; T-131 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5976[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; hu-hu; M-TECH aTAB 7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5977[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Motorola Milestone MAXX Build/6.7.1-68_DHD-15_NA-12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5978[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; hebe_inco Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5979[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; RK701 Build/2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5980[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Vi40 Elite Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5981[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ET-9018 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5982[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Teclast_P85(A9D3) Build/IMM76D; 768*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S5983[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; UM-760 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5984[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; M7012 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5985[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; COMEM712HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5986[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; My Audio Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5987[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; M-701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5988[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GN868 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S5989[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; T03A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5990[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GN868 Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S5991[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; HM0712C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5992[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; YL-Coolpad_9900/4.0.036.P2.X.9900; 480*960; CTC/2.0) CoolpadWebkit/534.30";
const char S5993[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Yan9.7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5994[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; RK2906 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5995[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; M-MP917I Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5996[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; COMEM816HC_A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5997[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID970 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5998[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; QPad C-X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S5999[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; QW TB-1010 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6000[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Wxt Tab PC 65CXi Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6001[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; 9701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6002[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; M753 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6003[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SC-73MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6004[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID971 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6005[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CT802 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6006[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I-TAB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6007[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CHARM712HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6008[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; TC-973D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6009[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; N70 DUAL CORE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6010[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M19 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6011[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; Android B1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6012[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 97S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6013[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TABULET Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6014[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; DFP7005 Build/IML74K)";
const char S6015[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; GoPad Surf Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6016[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MID104C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6017[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; D70GT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6018[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; N80 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6019[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Dezire Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6020[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; PC8001A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6021[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; el-gr; N6W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6022[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; SpeedUp S3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6023[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Ideapad S10-3T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6024[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; STAB970 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6025[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S13 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6026[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; NAM752HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6027[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-pt; GROWING GTR718 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6028[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; COMEM816HC_B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6029[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; MVPM805HCF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6030[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A8003T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6031[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; ALLDRO2SPEED Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6032[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; RICATECH-DR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6033[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BC1077 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6034[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; Mustek-DR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6035[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; disgo9104 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6036[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; M971w Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6037[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; G21 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6038[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T1002 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6039[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GOCLM712HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6040[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; SOLO831 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6041[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; da-dk; TAC-97042 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6042[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; HLV-T702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6043[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-uk; cyclone Titan Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6044[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC408PR002 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6045[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ACCM700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6046[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-uk; TAB10212 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6047[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GY-M1004 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6048[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; 756TPC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6049[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Pentagram TAB10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6050[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GIONEE S101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6051[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; T8830 Build/HuaweiT8830) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S6052[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; ALEXIS-RX5BTBT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6053[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 101T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6054[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ACTM816HCX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6055[] = "HUAWEI_T8830_TD/1.0 Android/4.0.3 (Linux; U; Android 4.0.3; zh-cn) Release/05.31.2012 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30";
const char S6056[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NM7RKTC8-C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6057[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PTM712MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6058[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PTM805MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6059[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SP-120 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6060[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Z7H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6061[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; PENTAGRAM TAB 8.4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6062[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; FJMID-700C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6063[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MID-WCDMA Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6064[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; Maui Surf tab TAB-10IPS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6065[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Z5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6066[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; K-BOOK Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6067[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; KN_MOBILE_QT_A06 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6068[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; V12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6069[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; UTAB430 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6070[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; M-300TC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6071[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; ALLVIEWSPEED Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6072[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Yan7.1V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6073[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PTM723MR Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6074[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M805 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6075[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M1061 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6076[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; aTab7s Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6077[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; platinum Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6078[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TRIO 7C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6079[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; it-it; EFOX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6080[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; AT-707 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6081[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; Q3M752HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6082[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; V930 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6083[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; S800 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6084[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; MID700SC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6085[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; OVO 1.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6086[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; F721 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6087[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; 8008 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6088[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; aussiePAD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6089[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; RMD-830 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6090[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MP847 Build/IML74K) AppleWebKit/534.30(KHTML, like Gecko) Version/4.0Safari/534.30";
const char S6091[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; AirTab M81g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6092[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MID-GSM Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6093[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; TC-971 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6094[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; T5500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6095[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TSM_U4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6096[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Newsmy T7II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6097[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M701C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6098[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; B928A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6099[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Pro 10 dual core Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6100[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; My Audio Build/RK2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6101[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N101 DUAL CORE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6102[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; M7512 Build/M7512) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6103[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Acqua Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6104[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; STS AR SERIES Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6105[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Lemon_LT9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6106[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; S5-N Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6107[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; INTELLIPAD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6108[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Rikomagic MK802 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6109[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; monga-M6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6110[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; M7511 Build/M7511) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6111[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6112[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TB782B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6113[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; INM803HCZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6114[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; uni-phone_MING_SII Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6115[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; WT-M768 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6116[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GPLUS GN700W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6117[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; MVM908HWZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6118[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; E900 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6119[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M1060W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6120[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; TREQ A10C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6121[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; H8000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6122[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; AGFAPHOTO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6123[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; C905 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6124[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MFC155EN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6125[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; TX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6126[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; f7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6127[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; AirTab M971g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6128[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; PC9701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6129[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MID1106 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6130[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; M91 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6131[] = "MOT-MT887_TD/1.0 Android/4.0 (Linux; U; Android 4.0.3; zh-cn) Release/05.16.2012 Browser/WAP 2.0 Mozilla/5.0 AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6132[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TABZ Build/TABZ) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6133[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; Kruger Matz Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6134[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; jsa_MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6135[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; MORAL_N01 Build/HuaqinMORAL_N01) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S6136[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 101D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6137[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; YL-Coolpad_5870/4.0.005.X.5870; 480*800; CTC/2.0) CoolpadWebkit/534.30";
const char S6138[] = "Mozilla/5.0 (Android 4.0.1; en-us) AppleWebKit/1.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.1";
const char S6139[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; N80HH Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6140[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; 712F Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6141[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TM-MID 750 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6142[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; PTM712MCF Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6143[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TZ300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6144[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TB1082 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6145[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; YL-Coolpad_5910/4.0.012.X.5910; 480*800; CTC/2.0) CoolpadWebkit/534.30";
const char S6146[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; ALEXIS-RX5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6147[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LA-I Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S6148[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID-A10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6149[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; TC-7120 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6150[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DFP7004 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6151[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; Platinum Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6152[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; X12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6153[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; 4S Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6154[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; G11 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6155[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SANEI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6156[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TM-MID600 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6157[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; NAM752MC_A Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6158[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; 56372 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6159[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; CW-V9-DUAL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6160[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ICS Build/ICS.f08refem31.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6161[] = "sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/Android4.0.3 Release/09.17.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S6162[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A10-4core Build/R13.4-rc2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6163[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; IPRO i5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6164[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; CT-1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6165[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; mi-book Mi3 Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6166[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6167[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MFC180EN Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6168[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6169[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; JY131C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6170[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VB80a Pro Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6171[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; START901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6172[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A9DualCore Build/ICS.d5h10_8188.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6173[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-be; Axxion ATAB-912 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6174[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; P101g Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6175[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Full Android on x86 VirtualBox Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6176[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMID800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6177[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JSA-MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6178[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; OCA7TA Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6179[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MFC270EN_09 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6180[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; PTM712MCFC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6181[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MFC180FR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6182[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 975TPC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6183[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; BASE_Lutea_3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6184[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; CJ ThL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6185[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q88 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6186[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; J-Q88D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6187[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N77 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6188[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; M81g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6189[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Penta WS702C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6190[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; J-Q8D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6191[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MD-0696B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6192[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; AQUILA 097-1006 Build/ICS.g12refM1006CMX.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6193[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; PT001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6194[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GASEM805MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6195[] = "Mozilla/5.0(Linux;U; Android 4.0.4; en-us;NEC-0912 Build/AX)AppleWebKit/534.30(KHTML, Like Gecko)Version/4.0 Mobile Safari/534.30";
const char S6196[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V930M10E_Burma Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6197[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 704 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6198[] = "AMOI N820 Linux/3.0.13 Android/4.0.4 Release/09.24.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S6199[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC408 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6200[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N83 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6201[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; J-Q8B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6202[] = "N1 Linux/3.0.13 Android/4.0.4 Release/10.15.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S6203[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; CAPTIVA PAD 8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6204[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Expad Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6205[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M9701THW Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6206[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID-001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6207[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KOCASO 970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6208[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; E-Boda Supreme X80 Dual Core Build/ICS.g12refM806A1YBD.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6209[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ms-my; i7S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6210[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; PICOpad_GEA Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6211[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; POP 7.1C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6212[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 4SPA0910C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6213[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MHU001D Build/MHU001D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6214[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Clear Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6215[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TAB700_4GB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6216[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; AQUILA 080-1008 Build/ICS.g12refM805CMX.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6217[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; sp0735 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6218[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; 7E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6219[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T-200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6220[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; MANHATTAN Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6221[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ATO7P Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6222[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; VOLXM712MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6223[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; MVPM1005HNF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6224[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; AllviewSpeedEco Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6225[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID_X5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6226[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TCC8925_HDMI_DONGLE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6227[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HanZpad Build/master-ics-android-4.0.4_r1.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6228[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; hu-hu; xTAB-70 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6229[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V700B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6230[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; T900 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6231[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; BC1077 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6232[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M811 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6233[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; GASEM723HRF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6234[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-be; Axxion ATAB-703 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6235[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MIDC907PR001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6236[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CW V8 Dual Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6237[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; M-MP927I Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6238[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; STM912HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6239[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i7c Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6240[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; IMPRESSPE3D E200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6241[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A2000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6242[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; W990 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6243[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; E61 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6244[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; AK8456 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6245[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; V800 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6246[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M-970 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6247[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; WOHTC G36 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6248[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; N8Zero Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6249[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SS707 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6250[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; 5910 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6251[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TZ200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6252[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Polaroid 40140 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6253[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; imatch i21 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6254[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SC-74MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6255[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; A21plus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6256[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; NAM805ND Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6257[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; CW V70 Build/ICS.g08ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6258[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Cat Tablet CSG 7J0 Build/ICS.g08ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6259[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N9977 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6260[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MW0821+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6261[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-uk; softwinners Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6262[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; smartTV Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6263[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; Android Multimedia  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6264[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q75 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6265[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CRANE EVB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6266[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; UM-A13 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6267[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; INM752HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6268[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; GNET Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6269[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; MFC155ES Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6270[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; RK2906 Build/rk29sdk_Android 4.0.4_X fwd.test-keys) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6271[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GOTAB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6272[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Obreey SURFpad Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6273[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VL-109 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6274[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A9DualCore Build/ICS.d5h1016_8188.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6275[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST9 Marvel Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6276[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; H12 Build/ICS.f08refem31.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6277[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMID10C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6278[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; FJMID-700 C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6279[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; 7D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6280[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MFC170FR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6281[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PTM805ND Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6282[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N10-4core Build/R13.4-rc2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6283[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; x806 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6284[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMID1000B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6285[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MFC155EN_09 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6286[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ITAB Build/ICS.g12refM806A1SPISY.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6287[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Playtab Build/ICS.g12refM806A1SPISY.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6288[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDCD10 Build/ICS.d5h1083_8188.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6289[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ATO712L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6290[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ZYNC Z990 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6291[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; MID05V2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6292[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; FS-M787 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6293[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; Z520m-parrot Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6294[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6295[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GTEL S101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6296[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; 930i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6297[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; T101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6298[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; 5870 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6299[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ViewPad E100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6300[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EP750 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6301[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; EXLW2R Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6302[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; PX8837 Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6303[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M750 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6304[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; SMX-T701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 MYI/1.10";
const char S6305[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TB892B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6306[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; MFC155DE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6307[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MFC175FR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6308[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; M972w Build/M972w-eng 4.0.4 IMM76D test-keys_OTA:MID_2.1.1.1_X.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6309[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; KidsPad1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6310[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fa-ir; G1 Snowy Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6311[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Tablet X2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6312[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; DIFRNCE DIT7050 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6313[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; T9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6314[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; CT710 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6315[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; SAMURAI Build/SAMURAI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6316[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; COROM752MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6317[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; KBOOK Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6318[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TPC0752 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6319[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 900TPCII Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6320[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; MFC155IT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6321[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SP500 Build/1.0.749_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6322[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M5000 Build/1.0.528_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6323[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MP880 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6324[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DA988 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6325[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; TAB10-07 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6326[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; My Play Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6327[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN13FP Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6328[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; HTC VAE_WX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6329[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MA70 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6330[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; MVM900HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6331[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; MOVIMAX P3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6332[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; MVM900HWZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6333[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D90T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6334[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-cl; G970C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6335[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6336[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; F900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6337[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; classic101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6338[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; PENTAGRAM TAB 8.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6339[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; disgo8104 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6340[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; P80w Build/SuningP80w) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6341[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID-1088B Dual Core Build/ICS.g12refM1005XK.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6342[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; T-702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6343[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; EVOAT7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6344[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ST0911 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6345[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; CASIA TAB 10'' Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6346[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A088 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6347[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; tablet-c Build/tablet-c) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6348[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ST0719 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6349[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C3_77_5647 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6350[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 704 Build/ICS.g08ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6351[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-uk; PA0108C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6352[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 19220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6353[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Mi-A400 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6354[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Penta WS703C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6355[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; BUSTAB7R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6356[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; QW TB-1217 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6357[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Zigo Nebula Tab 7.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6358[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 795TPC Build/795TPC) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6359[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Spark Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6360[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; RK2906 Build/rk29sdk_Android 4.0.4_X.X fwd.test-keys) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6361[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Brilliant Pad TPC7026C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6362[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M5000 Build/1.0.741_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6363[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; OVO 2.0 Build/FR-M768) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6364[] = "Mozilla/5.0 (Linux;U;Android 4.0.3;en-us;Digitac Ultratab;C1029 Build/ICS) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S6365[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ALLVIEW P4i Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6366[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; NXM727HCFCN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6367[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-eg; Q88B:7A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6368[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JY139 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6369[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; POM805HCF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6370[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NEO-G4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6371[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IMAGINE 10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6372[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-gb; N9977 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6373[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GP501 Build/ICS_IMM76D_B162) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6374[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; GR-TB7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6375[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MFC270EN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6376[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT9700DC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6377[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PWS700FJ Build/eng.glwx.X.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6378[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; A8826 Build/ICS.A8826.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6379[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Tagus Tablet Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6380[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; Motion Build/Motion) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6381[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; T-M719A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6382[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PTM712MCFC1 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6383[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MD-0651 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6384[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GY-M501 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6385[] = "aTAB5 Linux/3.0.13 Android/4.0.4 Release/10.22.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S6386[] = "Symphony Linux/3.0.13 Android/4.0.4 Release/12.06.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S6387[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; IFQM712MCF Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6388[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KY-901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6389[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; APPTB101S Build/ICS.g12refM1005APL.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6390[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MIDR47B Build/eng.glwx.X.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6391[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AIRIS S211 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6392[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TRIO-97C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6393[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AIRIS GN135 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6394[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Vexia Tablet 780 GPS Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6395[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; 740RS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6396[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6397[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; Pentagram TAB70 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6398[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; AirTab M81w Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6399[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; rkSDK Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6400[] = "Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; B2000+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6401[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; A750 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6402[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; T03A_mma7660 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6403[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Parkly RK2906 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6404[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; P10MAN R2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6405[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; RYOMA M1S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6406[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; T710P Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6407[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; D703B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6408[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; STM908HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6409[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; NAM1006HDW Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6410[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ETAB 0208A Build/ICS.f08refem86.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6411[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; T8830 Build/HuaweiT8830) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6412[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; VITA-V5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6413[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; YL-Coolpad_9120/4.0.020.X.9120; 480*800; CTC/2.0) CoolpadWebkit/534.30";
const char S6414[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; ICE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6415[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TabTop 7.16 - MGPT 07 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6416[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; 10BPM3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6417[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; YL-Coolpad_7266/4.0.006.X.7266; 480*800; CUCC/2.0) CoolpadWebkit/534.30";
const char S6418[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; shooter Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6419[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; 9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6420[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Tab970 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6421[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; TH-1129M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6422[] = "GPLUS N930 Linux/3.0.13 Android/4.0.4 Release/09.11.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S6423[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; FP710 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6424[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; STX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6425[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ar-eg; E10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6426[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MW0922 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6427[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Vexia Tablet 780 Build/tablet-b) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6428[] = "i10 Linux/3.0.13 Android/4.0.4 Release/09.13.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S6429[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PTM727MCF Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6430[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MFC154FR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6431[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; Primux_Omega Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6432[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MGPT09 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6433[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; FTABU Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6434[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; RC9716B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6435[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; TabTop 10.4-MGPT 05 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6436[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MiTraveler Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6437[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Informer 921 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6438[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; AT97I-RK66BP Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6439[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy S3 EX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6440[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PICOpad_GFI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6441[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-gb; SX5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6442[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMID1000D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6443[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; XL4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6444[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-be; CHARM805NC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6445[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; 1040RS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6446[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Quicki-728 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6447[] = "Mozilla/5.0 (Linux;U; Android 4.0.4; en-us; MP777 Build/IMM76D) AppleWebKit/534.30 (KHTML,like Gecko) Version/4.0 Safari/534.30";
const char S6448[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S5A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6449[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; disgo9000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6450[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; TAB-8DC Build/ICS.g12refM805ITO.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6451[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6452[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; signal75_ics2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6453[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; NTT 611 Build/ICS.g08refem611.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6454[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T1A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6455[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TPC0952 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6456[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; DC-0163 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6457[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; CTAB Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6458[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M701F Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6459[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; TPAD M1004A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6460[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GN868 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S6461[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; I10a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6462[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ro-ro; Neura WIFI Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6463[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; STAR N8000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6464[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Q7C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6465[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Ipad3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6466[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Touch Kem430 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6467[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A088 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6468[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; RK701 Build/RK2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6469[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fa-ir; Dimo 900 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6470[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZENDEX 8 Advanced Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6471[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; A1001C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6472[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; IMO Z6 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6473[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-RU; Informer 703 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6474[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; I04S Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6475[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; HYUNDAI Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6476[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Cat Tablet Galactica 9,7CA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6477[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; 5910 Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S6478[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; NTT-758 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6479[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; H6000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6480[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC408PR001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6481[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; M1005D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6482[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; S4001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6483[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-; CTB-701H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6484[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; AT7E-RK06WA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6485[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Orbit_5.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6486[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6487[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Next7Q12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6488[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; HLV-T9701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6489[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; VIG07 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6490[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; m720vw_vortex Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6491[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Tab9.7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6492[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; FPT Tablet HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6493[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; MC603CH Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6494[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ro-ro; AllviewSpeed2+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6495[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; RC9711B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6496[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; RC9724C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6497[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EON3I Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6498[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; IMO X5 Build/IMO.X5_MARS.Vxx.(date of built/release software)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6499[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Colortab-MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6500[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; M83w Build/M83w-eng 4.0.4 IMM76D test-keys_OTA:MID_2.1.1.1_X.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6501[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; Clempad_CPT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 ClemBrowser/1.0 Clempad_CPT";
const char S6502[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; Clempad Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 ClemBrowser/1.0 Clempad X";
const char S6503[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T-M703A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6504[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMID1000/PMID10C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6505[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AW101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6506[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MFC270FR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6507[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MW0710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6508[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; - Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6509[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HomeChoice-V12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6510[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TY-Q8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6511[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PEDI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6512[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; B902E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6513[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; 8BPE0 Build/ICS.g08refem618.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6514[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; T271 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6515[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; AT97I-RK66WA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6516[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ACGM816HCX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6517[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VM5610 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6518[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Matrix III Build/X.tm) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6519[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CABALT Tab S1010 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6520[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; aTAB7t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6521[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; hr-hr; TPC-7120 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6522[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; A721 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6523[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Phablet PIII Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6524[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MFC155EN_10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6525[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Scroll Tablet PC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6526[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; Fatih Build/AR.ETIS.PSI-R6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6527[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D7015 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6528[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 4U Sapphire Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6529[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ektATB-1108 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6530[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IdolPad 7+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6531[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fa-ir; Marshal ME-700A Build/Marshal_ME-700A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6532[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; full Android on Microsoft Windows, pad, pc, n*books Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6533[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Symphony Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6534[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT704 Build/GT704-A13005-G02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6535[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; J-Q88D? Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6536[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SPX-12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6537[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 1688-XHB-2357 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6538[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC POO_U Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6539[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; m1099RA Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6540[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ZM971 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6541[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; AMPE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6542[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; Kiwie_SEVEN-PRO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6543[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; desktop_generic Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6544[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; T9200 Build/HuaweiT9200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6545[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; TAB5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6546[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; DATAM805HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6547[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; AMOI N820 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6548[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N101 DUAL CORE2 V11 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6549[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; PEM727HCFR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6550[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; HD7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6551[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; HLV-T701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6552[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; PTS AR SERIES Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6553[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; ConCorde tab 9.7IPS PLUS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6554[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; T291 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6555[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; Exagerate XZPAD970H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6556[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; QS0716D Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6557[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; Beyond B78 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6558[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PC1058 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6559[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; CASIA TAB 7\" Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6560[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; Primux_Alpha Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6561[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; CL500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6562[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Infinity Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6563[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A707_Infinity Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6564[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; EBEST V5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6565[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Rikomagic MK802II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6566[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; T1003 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6567[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M71G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6568[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; AP-80104 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6569[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; E8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6570[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SXZ-PD80-X05 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6571[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; PAPYRE pad 715 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6572[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TB782 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6573[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TB4-4/5580 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6574[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Phone5 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6575[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PA810G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6576[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; ANGB-CH Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6577[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; NAM752NC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6578[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; PMID70C Build/unknown) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6579[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GS3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6580[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; OV-SteelCore Build/ICS.g08refem611.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6581[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Digma iDn7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6582[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IMAGINE 9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6583[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i7a Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6584[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DTA-07HDAH Build/ICS.g08ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6585[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MFC250FR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6586[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; JUPITER Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6587[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Quicki-1041 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6588[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; ELEMENT7V2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6589[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M9702TH Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6590[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-eg; C72d Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6591[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; hu-hu; xTAB-70dc Build/xTAB-70dc.8GB.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6592[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DA788 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6593[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; hebe_plus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6594[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; T1300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6595[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; YF723B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6596[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; F1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6597[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; H7 Build/KO.Hanlym.H7.V2.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6598[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M77 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6599[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; OYSTERS T7M 3G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6600[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; 1707 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6601[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; EverGlory Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6602[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; XL4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6603[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PICOpad GJT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6604[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Z3Z6Z7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6605[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; TB1000 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6606[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C1029 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6607[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 5I Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6608[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; tablet Fnac7 Little Build/1.0.2.X-19:55) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6609[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PICOpad GEW Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6610[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB 170 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6611[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; E1020HD Build/ICS.E1020HD.Dualcore.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6612[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PAD1001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6613[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; _3QM819HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6614[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EBM1005ND Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6615[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; K-TOUCH W806 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6616[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TABLET Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6617[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; hexing77_ics2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6618[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Z500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6619[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; baoxue Build/F7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6620[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IMPULSE7120 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6621[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MID82C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6622[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Qumo Dotcom Build/ICS.b02ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6623[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; PD-970CH PLUS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6624[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MX970 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6625[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; USAGC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6626[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Helios Build/ICS.f08ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6627[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Cat Tablet GalacticaX 9.7JA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6628[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; TechTAB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6629[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Explay Informer 801 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0.4 Safari/534.30";
const char S6630[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; it-it; B63 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6631[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; AS-05 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6632[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; M72 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6633[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; N900 Build/IMM76D)";
const char S6634[] = "HUAWEI_T9510E_TD/1.0 Android/4.0.4 (Linux; U; Android 4.0.4; zh-cn) Release/05.31.2012 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30";
const char S6635[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; _3QM819HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6636[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; el-gr; Turbo-X BREEZE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6637[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; QT-A07 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6638[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; T71hk Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6639[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; POP8.1G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6640[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; SP-A20i Build/MF_ICS_02.24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6641[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q8-A13 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6642[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; R71 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6643[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; a106c_ics Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6644[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ING100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6645[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CABALT Tab S700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6646[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TP710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6647[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AT197F Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6648[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T-FREE 5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6649[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V9-1688-CJ-0308-31S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6650[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Sky Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6651[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; T9002 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6652[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T753 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6653[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; E704J Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6654[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PICOpad_GEA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6655[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; LUXURY Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6656[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; I5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6657[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; Multilaser Oxy Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6658[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; A77 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6659[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M713 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6660[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ET-971 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6661[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; MID08S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6662[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; BT210 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6663[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SXZ-PDX0-01 Build/SXZ-PDX0-01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6664[] = "X50 Linux/3.0.13 Android/4.0.4 Release/01.10.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S6665[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; 3310-9506 Build/ICS.Zoostorm_160_NEW.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6666[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; STXS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6667[] = "Mozilla/5.0 (Linux; U; Android 4.0.4;pt-pt; tmn smart a18 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6668[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; VM1017A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6669[] = "Sansui Cosmic SP 1001 Linux/3.0.13 Android/4.0.4 Release/01.06.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S6670[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; A10-MID Build/IML74K)";
const char S6671[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; SZENIO IPS 9700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6672[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M-301T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6673[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; P90 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6674[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LGL96G/V100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6675[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Q8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6676[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; i708 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6677[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; RM12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6678[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; SIROCO 2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6679[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; IHD908 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6680[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; J1000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6681[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; Airpad-DS820M Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6682[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Penta WS704C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6683[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AMPE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6684[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Gemini Apside Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6685[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Apollo Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6686[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q39 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6687[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; T5500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6688[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; N70DC-T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6689[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; 5677 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6690[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; RTAB-070 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6691[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; RMD-73G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6692[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A930 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6693[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A13 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6694[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SCH-W2013 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6695[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; TP9.7-1200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6696[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-by; MTC Viva Build/HuaweiU8816) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6697[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; P101w Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6698[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DFPQ88 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6699[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; AP-102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6700[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M78 Build/M78) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6701[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; TB-50 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6702[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; TP8-1200-metal Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6703[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; EPC-705 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6704[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MICROLAB TABLET Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6705[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; G901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6706[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; CMP746E Build/ITL41D) AppleWebKit/434.50 (KHTML, like Gecko) Version/4.0 Mobile";
const char S6707[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMID1000B_V2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6708[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; SPX_8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6709[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; crane-evb Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6710[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; DL HD-72 X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6711[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; ONEPAD940 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6712[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M010SQ8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6713[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Proton Lite Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6714[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; myTab10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6715[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fi-fi; 28126 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6716[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Lafone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6717[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; P10C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6718[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; YPY_10FTB Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6719[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M752 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6720[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A704J Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6721[] = "AMOI M8228 Linux/3.0.13 Android/4.0.4 Release/11.06.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S6722[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ICS AOSP on imx6q Build/R13.4-beta) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6723[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EXLB3B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6724[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TB782B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6725[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; Mio Pad A1001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6726[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; MFC270ES Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6727[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q88B-5S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6728[] = "sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/Android4.0.3 Release/11.17.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S6729[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; UM77-V1.11-073-01-01-02-01-00-01 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6730[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q88B-12S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6731[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NuclearSX-SP5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6732[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; EGS-5S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6733[] = "Yummy-Duo Linux/3.0.13 Android/4.0.4 Release/12.03.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S6734[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M762B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6735[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; W92210A1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6736[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; BPA9730 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6737[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; 10BPE0 Build/MystiQ970-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6738[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Inco Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6739[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; L9000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6740[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; SMARTLET Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6741[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Xtab745 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6742[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; WTG2-07G-S1 Build/ICS.g08refem667.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6743[] = "JXD-P300 Linux/3.0.13 Android/4.0.4 Release/01.23.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S6744[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; W70 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6745[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Alpha GT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6746[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT704 Build/GT704-A13005-F02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6747[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; START901.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6748[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; T101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6749[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ar-eg; i7005 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6750[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; LG-L160L Build/IML74K) AppleWebkit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6751[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID_Robin Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6752[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Tablet PC 2000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6753[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; 9900 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6754[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; AndroCore Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6755[] = "Athens15_TD/V2 Linux/3.0.13 Android/4.0 Release/02.15.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2 System/Android 4.0.1;";
const char S6756[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; MP810C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6757[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; RMD-720 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6758[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; H9701C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6759[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; VB97 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6760[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; M1005HN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6761[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Pentagram TAB70 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6762[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MID970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6763[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; EVOPAD_R7000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6764[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC_A320e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6765[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; KPT A11PLUS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6766[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; MID9107CM Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6767[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; AUX V920P Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6768[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PC721 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6769[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; DOOV_D7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6770[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; GSmart G1342 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6771[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Smartab MT12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6772[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; QWTB-1280BL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6773[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; M78 Build/M78) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6774[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ACGM727HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6775[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GY-M2906 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6776[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; EVOPad R9700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6777[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; DOOV_D900 Build/DoovDOOV_D900) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6778[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; A76 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6779[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Coolpad 7290 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6780[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I700C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6781[] = "X9 NOTE Linux/3.0.13 Android/4.0.4 Release/09.26.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S6782[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; ANDROID_HDMI_DONGLE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6783[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; L11 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6784[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN180 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6785[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TAB 7i 3G Build/WEXLER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6786[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SC-90MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6787[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; G1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6788[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; U23GT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6789[] = "sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/Android4.0.3 Release/10.27.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S6790[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; gs701b Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6791[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; RK2906-8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6792[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ALFAR 780 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6793[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; RC0721B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6794[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; J-88 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6795[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; NB0026 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6796[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; 300TC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6797[] = "Mozilla/5.0 (Linux; U; Android 4.0; vi-vn; 6515 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6798[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AG CHROME Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6799[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fa-ir; G1 SKY Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6800[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Starway Andromeda S910 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6801[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; GLX Mobile Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6802[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MP444 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6803[] = "A2 Linux/3.0.13 Android/4.0.4 Release/10.29.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S6804[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; YL-Coolpad_9960/4.0.013.P2.X.9960; 1280*720; CTC/2.0) CoolpadWebkit/534.30";
const char S6805[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JXD-S80 Build/ICS.g08refem618.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6806[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; J-Q8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6807[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; 8005 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6808[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; VEDIA X6 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6809[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DLR_MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6810[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; PT7-ETOOS Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6811[] = "MOT-MT788_TD/1.0 Android/4.0 (Linux; U; Android 4.0.4; zh-cn) Release/05.16.2012 Browser/WAP 2.0 Mozilla/5.0 AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6812[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; TKSP8M404BK Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6813[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Android 904 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6814[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; PMID70DC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6815[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SXZ-PDX0-05M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6816[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; H433 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6817[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; MID7500 Build/ICS.g08ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6818[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; 8190 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6819[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; F9S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6820[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X704 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6821[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; A13-DLR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6822[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SP-100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6823[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; G-PAD1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6824[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAB-NAVI7-3G-M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6825[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; E7010HD Build/ICS.E7010HD.Dualcore.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6826[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; YPY_10STB Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6827[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6828[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; E-Boda Impresspeed E300 Build/ICS.g12refM703A1YBD.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6829[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; HKPHONE REVO MAX Build/Haiduongbk) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6830[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; POP_smart_PS1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6831[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; POM727MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6832[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Mytel Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6833[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SmartPadv10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6834[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; TT-2500 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6835[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Sygnus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6836[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M702VM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6837[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; Dimo900 Plus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6838[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SP-700-A8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6839[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; optima vtab Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6840[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TB-712 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6841[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; LEOTEC LETAB704 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6842[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X714 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6843[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AMI-TT2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6844[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; S4501 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6845[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SC3077 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6846[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; D1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6847[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q75S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6848[] = "I10S Linux/3.0.13 Android/4.0.4 Release/12.10.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S6849[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; MMD-X100   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6850[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CDR KING FP-009-XON Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6851[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; H8000+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6852[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; 3Q_ER71B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6853[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; S3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6854[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MFC175FR_02 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6855[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; A99 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6856[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ekt KA-5926 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6857[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; A10GXM75 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6858[] = "sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/Android4.0.3 Release/12.28.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S6859[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q74 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6860[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AKL_M501 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6861[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; ASM722HCN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6862[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE N881F Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6863[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Pentagram Monster Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6864[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; PA0702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6865[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; 3D life+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6866[] = "Mozilla/5.0 (Linux; U; Android 4.0.4(6577); es-es; i-Call 701  Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6867[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MASHAIL EXTREME_I Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6868[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; mi-book Mi8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6869[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TELETALK_X100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6870[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; X40i Build/BCM21654AndroidTK_SystemRel_1.3.2P.05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6871[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MID-9108 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6872[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Tokio_Smartphone_1_0 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6873[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6874[] = "F51 Linux/3.0.13 Android/4.0.4 Release/01.23.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S6875[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; START101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6876[] = "V102 Linux/3.0.13 Android/4.0.4 Release/02.01.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S6877[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6878[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; W80 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6879[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID9762 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6880[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A13-Q88D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6881[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A751 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6882[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GP501S Build/ICS_IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6883[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; X-5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6884[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; G1 snowy+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6885[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V700NA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6886[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; M83g Build/M83g-eng 4.0.4 IMM76D test-keys_OTA:MID_2.1.1.1_X.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6887[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Z3Z6Z7W Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6888[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; YPY_07STB Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6889[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SP-360 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6890[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JOYPAD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6891[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; G-M7904 Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6892[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; A26 Build/CROSSCROSS_A26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6893[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6894[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; EverShine Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6895[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TPC0754 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6896[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; TM-9720 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6897[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; T736B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6898[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CRYM1005HN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6899[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; VertiCool AndyPad3 Build/IML74K)  AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6900[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; SPC 5064N Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6901[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Infinity Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6902[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; 2GO Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6903[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; W686 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6904[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT8000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6905[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; MB1003 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6906[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TB 712 Build/RK2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6907[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; TC-971 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6908[] = "Mozilla/5.0 (Linux; Android 4.0.3) AppleWebKit/535.70 (KHTML, like Gecko) Version/4.0 Mobile Safari/535.700";
const char S6909[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; 9120 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6910[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; KaiCloud 744 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6911[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; I97D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6912[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; TAB-9701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6913[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-ca; 9701BT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6914[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LNV-Lenovo_A710e Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6915[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT685 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6916[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; TCL S600 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6917[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; M-MP707i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6918[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; E900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6919[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MID7109 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6920[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Haier HW-W718 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6921[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; 7266 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6922[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; 9220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6923[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; A120 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6924[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; BTPC-1010 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6925[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ID1018WTA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6926[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Coolpad 7290 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S6927[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Telus HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6928[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; I 16 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6929[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M762B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6930[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; EGS001G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6931[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; AMI PAD 10 IPS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6932[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; VB73G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6933[] = "Mozilla/5.0 (Linux; U; Android 4.0; zh-cn; K-Touch U6 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6934[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; TAB Matrix Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6935[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; EVOPAD R7000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6936[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; M807 Build/ICS.g08refem618.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6937[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Holiday_X710a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6938[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Tablet PC 2000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6939[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; el-gr; me12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6940[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fa-ir; Dimo 700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6941[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; M736 Build/ICS.g08ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6942[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; Exagerate XZPAD970H3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6943[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; T29 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S6944[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MA901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6945[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MFC190BBFR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6946[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SKYTEX-SP438 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6947[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; I3000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6948[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch E688 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6949[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6950[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; HITOMI Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6951[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TB712 Build/RK2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6952[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; KidsPad2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6953[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Axxion ATAB-902 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6954[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TWZ HD8 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6955[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Note_mini Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6956[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; CW V80 Build/ICS.g08refem618.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6957[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Android_53 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6958[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB73GP Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6959[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; aigoPad T710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6960[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; IMO X5 Build/IMO.X5_MARS.V09.XAB) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6961[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V700D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6962[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN700T Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S6963[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; ibho II  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S6964[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; SuzukiRx7 Build/SuzukiRx7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6965[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; Exagerate XZPAD700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6966[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GY-M1006 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6967[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Vexia Tablet 780 GPS Build/tablet-b) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6968[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; SAMURAI Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6969[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; TECHNAXX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6970[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SC-91MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6971[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XW-I8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6972[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; MiPad 4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6973[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CDRKING FP010XON Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6974[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; TAB9 V4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6975[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; MTK5000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6976[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; IMO S68 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6977[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S5_77_9P017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6978[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V800H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6979[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TB1082B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6980[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q27B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6981[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-STYLE Q6A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6982[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; 8190 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S6983[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SRT_AN4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6984[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Tokio Tablet 2.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6985[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PAD 1001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6986[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; M92 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6987[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q88 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6988[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TS9708B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6989[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; EUTB-708 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6990[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MFC140FR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6991[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KT968 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6992[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Xtab781HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6993[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; M-MP705I Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6994[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; android ics 4.04 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6995[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; CityTab Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6996[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Proton Zero Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S6997[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; KDI-W360   Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6998[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TPC-EA900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S6999[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Netbook PC 12700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7000[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; TREQ 3G Turbo Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7001[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PMID-971C Build/ICS.g12refM1005SPIBL.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7002[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; TVSmart Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7003[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Z99 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7004[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; mTAB StreaQ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7005[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; E703J Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7006[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Strike Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7007[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Xtab1081HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7008[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; epade_i5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7009[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T98 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7010[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOTE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7011[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AP2-16-1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7012[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAB-73 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7013[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; E8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7014[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VIA101TAB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7015[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; Venera Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7016[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; AT9N-A13WP Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7017[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A704_INSPIRE_X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7018[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; s4502 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7019[] = "CETRIX-CS1280 Linux/3.0.13 Android/4.0.4 Release/12.11.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7020[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; Q73 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7021[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; P88(H3D7) Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7022[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; BT210 Build/IMM76D)";
const char S7023[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D26 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7024[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SMP53-210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7025[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; SKY Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7026[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; AQ FIVE Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7027[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; L29 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7028[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; CTLITE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7029[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Vexia Tablet 780 GPS Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7030[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Brand Infinityll Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7031[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; MHL3 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7032[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; SM70 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7033[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT7330 Build/ICS_IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7034[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; YPY_07FTB Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7035[] = "Mozilla/5.0 (Linux;U; Android 4.0.4; en-us; MID103C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7036[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7037[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-en; ETAB 0307 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7038[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; M84g Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7039[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; CROSS A28 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7040[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; RX3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7041[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; PULSO 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7042[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; PA0910CA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7043[] = "Mozilla/5.0 (Linux; U; Android 4.0; haw-US; U7 Build/GRK39F) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7044[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EVOTAB Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7045[] = "E1-B Linux/3.0.13 Android/4.0.4 Release/01.31.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7046[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; N007 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7047[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; MP810C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7048[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ALUMIUM10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7049[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; Zyrex_ZA987_OneScribe Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7050[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SCROLL_BASIC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7051[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; X825a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7052[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; I93 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7053[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CW-V17 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7054[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; MVM908HCZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7055[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Woxter Tablet PC 90BL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7056[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; T820 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7057[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; ELIYA S2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7058[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; T9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7059[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; Kesa Tablet Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7060[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; P4D Sirius Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7061[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7062[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; aigopad Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7063[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; v128 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7064[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; S70 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7065[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; SPC neo4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7066[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru;  DIGMA iDs10 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7067[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ENERGY s10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7068[] = "Mozilla/5.0 (Linux; U; Android 4.0; zh-cn; K-Touch W760 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7069[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Informer 704 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7070[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SIROCO 3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7071[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TM-MID760 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7072[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Tabplus 7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7073[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; M08W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7074[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; T910 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7075[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB-N8BS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7076[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; da-dk; TAC-7028 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7077[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; LG OPTIMUS HD LTE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7078[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT10K-BT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7079[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Q88 Build/IMM76D)";
const char S7080[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GPLUS_GN878 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7081[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TAB7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7082[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; ENOT J101 Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7083[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; Victory_S3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7084[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; GTR848 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7085[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DEVANTE SMART CALLING Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7086[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DEW H1406 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7087[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; S9220 Build/[S812T]S812T_HPD_H812_W12.09.10_V01B11_Release) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7088[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Cat Tablet Galactica X 9.7CA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7089[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SW-ARMM8S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7090[] = "Mozilla/5.0 (Linux; U; Android 4.0.8; en-us; 9300I Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7091[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GV7777 Build/GV7777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7092[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M747 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7093[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; NTT 617 Build/ICS.g08ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7094[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; TAB0004 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7095[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SABRESD-MX6DQ Build/R13.4-beta) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7096[] = "SXZ-PDX0-01 Linux/3.0.13 Android/4.0.4 Release/11.15.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7097[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C92d Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7098[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TPC1050 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7099[] = "AMOI N816 Linux/3.0.13 Android/4.0.4 Release/12.19.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7100[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; AT7D-TE25DA Build/AT7D-TE25DA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7101[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB 170N Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7102[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S708 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7103[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7104[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Lafone SIII Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7105[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; PTS CA SERIES Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7106[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I5-77 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7107[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HD II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7108[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; DNS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7109[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; PICOpad GGD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7110[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Explosion Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7111[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; s4503 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7112[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GTEL_A706_INFINITY_S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7113[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; es-es; A822 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7114[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; S5301 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7115[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; TH-1130M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7116[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X716S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7117[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sk-sk; V8200-Memphis-orange-Slovakia Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7118[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; M93 Build/ICS.g08refema9.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7119[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; M800 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7120[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GASEM712MC_WP Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7121[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MD-0697 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7122[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; F8GT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7123[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB 280 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7124[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; GO Live Dual Core Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7125[] = "E1A Linux/3.0.13 Android/4.0.4 Release/01.26.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7126[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AW710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7127[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; DL HD EV 01 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7128[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7129[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; GLX Jet Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7130[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; F307 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7131[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; i-Call 300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7132[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 7878_c Build/EDU) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7133[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; MD-828 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7134[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; M758A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7135[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID72C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7136[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; S4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7137[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Q8.3G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7138[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 9702T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7139[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; EVOLVE_FX400 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7140[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ar-eg; GX8404 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7141[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; RC0712B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7142[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; deovo V5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7143[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A106 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7144[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; zh-tw; PAD1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7145[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ratech77_ics2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7146[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; Andronote Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7147[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BIRDA_MODEL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7148[] = "Mozilla/5.0 (Linux; U; Android 4.0.6; es-es; MID7102SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7149[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; UTAB71 Build/IMM76D)";
const char S7150[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; AOLE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7151[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; HWA660 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7152[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Explay Informer 705 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7153[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AC-77 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7154[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; DK mobile Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7155[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; hu-hu; voyager GT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7156[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; SHIRU SHOGUN(QuadCore) Build/SHIRU_SHOGUN) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7157[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GFIVE Glory(A86) Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7158[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; NT-1710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7159[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; xgame II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7160[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I90 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7161[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch E621 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7162[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; M009S 16GB RTB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7163[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M006-Q Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7164[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; NITRO8B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7165[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Blast Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7166[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N79 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7167[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Magnum Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7168[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; EZPAD 810C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7169[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CRUISE_TAB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7170[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TAB9008GB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7171[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Walton Primo Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7172[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Artel-Adi5 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7173[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 7D-1A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7174[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Condor Tab SMB-B9705 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7175[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; 2627 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7176[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S7t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7177[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MTK5000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7178[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; O1A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7179[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; SPC P1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7180[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Maxpro Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7181[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; CR9A13-DC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7182[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AMAZE TAB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7183[] = "Mozilla/5.0 (Linux; U; Android 4.0.5; en-us; MTK6515 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7184[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; M009S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7185[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A102C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7186[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; OBSIDIAN1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7187[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M901-W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7188[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; M71G Build/IML74K)";
const char S7189[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; TCL S900 Build/S900Q-2SLCCN1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7190[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; California2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7191[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Newpad Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7192[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; DOOV_D7 Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S7193[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; ASK 110SP Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7194[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M803HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7195[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A710e Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7196[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 9710D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7197[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GGC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7198[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CW V80 Build/ICS.g08refem618.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7199[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; TCL S800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7200[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; 9220 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S7201[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IT-M744AL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7202[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U960s3 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S7203[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Vexia Navlet 2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7204[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; Xfab Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7205[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; SD02 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7206[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; 9700DN Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7207[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A530/V3.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7208[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; WizTab WT 7.03 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7209[] = "Mozilla/5.0 (Linux; U; Android 4.0.6; es-es; MPM_AVA758 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7210[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; RAPAX 070-0508 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7211[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; H5000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7212[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; TP7-1000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7213[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PAD 9715D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7214[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch E806 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7215[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; DOOV_D9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7216[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-eg; 07/083 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7217[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TW-900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7218[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; MTC_968 Build/ICECREAM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7219[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; QUER KOM0330 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7220[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; StarPAD 7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7221[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CVD601 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7222[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; E9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7223[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-au; Z15A Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7224[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; epade_S90 Build/epadeepade_S90) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7225[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LXT1001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7226[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; H1W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7227[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; NTT 618 Build/ICS.g08refem618.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7228[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; QS0730C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7229[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Connspeed AS9 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7230[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; NetTAB SKY 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7231[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MFC415FR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7232[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; ar-eg; BERAQ NB898 Build/VENUS_00.03.168.H1121) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7233[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; L36 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7234[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; DL 3G MOBILE TG-M71 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7235[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; AP-803 Build/ICS.AP-803.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7236[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; Avvio_790 Build/AvvioAvvio_790) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7237[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ViewPad E72 Build/ICS.VSA.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7238[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7239[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A703J Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7240[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SPT-800 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7241[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; HISENSE F0 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7242[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TA-734 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7243[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Vision Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7244[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; COMPUMAX Build/ICS.compumax_30_300_8188.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7245[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MASHAIL EXTREME_II Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7246[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; DL X10Note X1-N11 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7247[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; aTAB503D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7248[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Informer 705 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7249[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; sprd-M2/1.0 Android/4.0.4 Release/04.02.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7250[] = "Mozilla/5.0(Linux;U;Android 4.0.3;en-us;Digitac Ultratab;C714 Build/ICS) AppleWebKit/531.21.10(KTHML,like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S7251[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HX-M02 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7252[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; L29i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7253[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; U910 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7254[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; GPad Explorer Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7255[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC S720t Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S7256[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ro-ro; Neura 3G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7257[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; PULID F10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7258[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; TC-7110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7259[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; xDevice Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7260[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; V7S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7261[] = "Mozilla/5.0 (Linux; U; Android 4.0; vi-vn; VipPhone V4S Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7262[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; DOOV_D900 Build/DoovDOOV_D900) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S7263[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; SpeedUp S5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7264[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; my-Mate 3D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7265[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; JW101A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7266[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Informer 706 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7267[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; 5910 Build/IMM76I)";
const char S7268[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D101B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7269[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; C8715M Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7270[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; 4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7271[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; mi-book Mi5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7272[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; INFINITY_PASSION Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7273[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 4SPA752C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7274[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M750B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7275[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; OS458 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7276[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; M81w Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7277[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; UM-A90 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7278[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE N983 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7279[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; MD803S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7280[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch E780 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7281[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A99 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7282[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U802 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7283[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 4U Coral Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7284[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; 556 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7285[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Informer 705 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7286[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7287[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cyrus 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7288[] = "ratech77_ics2 Linux/3.0.13 Android/4.0.4 Release/11.21.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7289[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; IHA-C0901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7290[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; MOBO Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7291[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; 712F(0703) Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7292[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; JC-898-MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7293[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; Bpad2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7294[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TPC-7HG Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7295[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; WFJWAE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7296[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB 0170N Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7297[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GSM30 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7298[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; TY006 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7299[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; YL-Coolpad_5876/4.0.021.X.5876; 540*960; CTC/2.0) CoolpadWebkit/534.30";
const char S7300[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Mi406 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7301[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; M704 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7302[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MOBICEL HERO Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7303[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ViewPad 7D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7304[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A704H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7305[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; T98 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7306[] = "Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; Phonei5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7307[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A78-M02 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7308[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; Nova Pad 70 C124FC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7309[] = "Mozilla/5.0 (Linux; U; Android 4.0.8; vi-vn; S510+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7310[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; Airpad-DS101M Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7311[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Q3M907HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7312[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; WT5.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7313[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; TPAD-100A3GECA Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7314[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TR976 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7315[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; I-MEMO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7316[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; V960 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7317[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; 7266 Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S7318[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; ANDROpad TA-12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7319[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; INW908HCZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7320[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; epade N01 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7321[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; gs701b Build/IML74K)";
const char S7322[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MPCA410VD001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7323[] = "Mozilla/5.0 (Linux; U; Android 4.0; vi-vn; Smartphones Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7324[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; N80DC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7325[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Kaosa K3G1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7326[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M9702T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7327[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MID1107 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7328[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U1203 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7329[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; Ex'S1   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7330[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; WT7.0g Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7331[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T201 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7332[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB 0170 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7333[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MFC250EN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7334[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; a760sSamSungGS3 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7335[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; MID9711 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7336[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; VM0711A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7337[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; NB726A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7338[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; B9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7339[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; IMO Y-One Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7340[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; TS 501 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7341[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Livepad 7.5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7342[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; PB-W20 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7343[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CY26 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7344[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; SP-702-A5 Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7345[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; Q86 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7346[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q88B-11S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7347[] = "MALATA I10 Linux/3.0.13 Android/4.0.4 Release/01.10.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7348[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AX8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7349[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i800 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7350[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; Everest Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7351[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; E780 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7352[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Sky 3G DUO Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7353[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ETAB-1031 Build/ICS.g08refmid1061.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7354[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; TRACER OXYGEN GS1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7355[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TPC0702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7356[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; 8720 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S7357[] = "NIU D8 Linux/3.0.13 Android/4.0.4 Release/03.07.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7358[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HBD-761 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7359[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; S-534 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7360[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; N8010_APT Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7361[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; Tab745 Build/GT704-A13005-H01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7362[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-gb; N005 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7363[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 10901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7364[] = "TWZ AA8 Linux/3.0.13 Android/4.0.4 Release/11.27.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7365[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMID900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7366[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; DL 3G MOBILE TG-M71 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7367[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; 5858 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7368[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MI360 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7369[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; E704H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7370[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Infinix X400 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7371[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EX7201 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7372[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M700G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7373[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; wetab Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7374[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; QW TB-G100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7375[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I748 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7376[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CRYM805HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7377[] = "Mozilla/5.0 (Linux; U; Android 4.0; vi-vn; V8 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7378[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; HTC VAE_WX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7379[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Behold BeTAB Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7380[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; LANDA Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7381[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; G.Pad EXplorer Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7382[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; EPAD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7383[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; prevail Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7384[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; TAB971 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7385[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; Quicki 730 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7386[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; TM-7027W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7387[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC A320e Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S7388[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; PRO-7.0 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7389[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Ideapad S10-3T Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7390[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; SONY Q7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7391[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; E900 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S7392[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; M700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7393[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; E170BS Build/C006) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7394[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 101S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7395[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nb-no; ARGM816HCX_A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7396[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; N83 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7397[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; T1001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7398[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; thinkpad Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7399[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID700CVD102 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7400[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; N7T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7401[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; AUX V920 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7402[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; OnePAD 1100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7403[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; 7\" DELTA Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7404[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; BenQ R71 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7405[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; msm7627a Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7406[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S5/C2_77_5647 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7407[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; UBT081X Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7408[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT685 Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S7409[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; KPT A11PLUS Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S7410[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-ja; CTB-701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7411[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Swipe Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7412[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; IRUX M8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7413[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GPLUS GN700W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7414[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; OnePAD 735 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7415[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; aigopad Build/IMM76D)";
const char S7416[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; lt-lt; Estar Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7417[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GN868 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7418[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; PocketPad Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7419[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; PC7342 Build/RK2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7420[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sk-sk; TA8200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7421[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; HWA600 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7422[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Acer AK330s Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7423[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7424[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; CAPTIVA PAD 8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7425[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; M78D Build/M78D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7426[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; OR-TB7010 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7427[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; treq A10G Duo Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7428[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; M758A Build/IMM76D)";
const char S7429[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ET8001C-G12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7430[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; Consul 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7431[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; UPAD 9A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7432[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Quicki-732 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7433[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; ru-ru; U920+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7434[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID-7089D Dual Core Build/ICS.g12refM703A1XK.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7435[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; MAG101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7436[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Informer 703 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7437[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; virtual Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7438[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MIDR409 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7439[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; XPRESS PRO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7440[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D07 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7441[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch W780 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7442[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T-907 Build/T-907) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7443[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MOT-XT889/6.7.2_GC-200-YTZWE-3; 540*888; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7444[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; BAUSSEN BEN758N Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7445[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-co; SI09A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7446[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; CNC-TAB7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7447[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; P80w Build/SuningP80w) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S7448[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MIDR477 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7449[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; BKA-C1004 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7450[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; A85 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7451[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Mi501 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7452[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; S7916E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7453[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 900TPC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7454[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; SP-A20i Build/MF_ICS_02.19) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7455[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AM708-JH Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7456[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Advanced II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7457[] = "sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/Android4.0.3 Release/09.25.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S7458[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MIDC711PR001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7459[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; MID13 Build/MID13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7460[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KB706 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7461[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ETON T850 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7462[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; NetTAB RUNE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7463[] = "DOUBLE IV Linux/3.0.13 Android/4.0.4 Release/10.25.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7464[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SK2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7465[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; E-Tab 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7466[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M906G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7467[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; PMID80C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7468[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; TRIO7C Build/IMM76D)";
const char S7469[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; L26 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7470[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; F8A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7471[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A90-2core Build/R13.4-rc2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7472[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; OMSP4048MWT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7473[] = "I9220 Linux/3.0.13 Android/4.1.1 Release/10.16.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7474[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; G7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7475[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; OPPOR601 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7476[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GN180 Build/IMM76D)";
const char S7477[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; WT-PAD012 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7478[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U910 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7479[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; FS-M787S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7480[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; SmartPad1010i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7481[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; REVO MAX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7482[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N91 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7483[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; F8G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7484[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN135 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7485[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; monga M8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7486[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; FS-S4300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7487[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AND5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7488[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; SHOGUN Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7489[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; zh-cn; LA-I Dual core Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7490[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; tcc8800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7491[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; FLY_IQ310 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7492[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; M702VM Build/IMM76D)";
const char S7493[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SKYTEX-SP705 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7494[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Q88c_a13 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7495[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; A13-DLR Build/IMM76D)";
const char S7496[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NEO X5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7497[] = "Coolpad 7290 Linux/3.0.13 Android/4.0.4 Release/11.27.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7498[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; INU001D Build/INU001D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7499[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 975PTC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7500[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S580 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7501[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; PAN 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7502[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MP547 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7503[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MFC140FR1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7504[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; MFC270DE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7505[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; RC9713B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7506[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; U-TAB720 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7507[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID-1089D Build/ICS.g12refM1006XK.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7508[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN700T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7509[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Digma iDj7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7510[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iTab M7i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7511[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; GUNIUS713 Build/GUNIUS713) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7512[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sl-si; sc3068n Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7513[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; 118 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7514[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; MTC 1065 Build/MTC_RU) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7515[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TB-PH03 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7516[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SmartPadv10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S7517[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; PENTAGRAM TAB 8.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7518[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; zh-cn; LA-I Dual core Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7519[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Z1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7520[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HY731 Build/ICS.f08refem31.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7521[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; E830 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7522[] = "Mozilla/5.0 (Linux; U; Android 4.0; zh-cn; A811 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7523[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7524[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; FW8932 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7525[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TM475 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7526[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; T9108 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7527[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; K-BOOK PLUS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7528[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; S88 Build/S750) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7529[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; JT_SmartPC02 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7530[] = "Coolpad 7235 Linux/3.0.13 Android/4.0.4 Release/01.12.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7531[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; G500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7532[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; B93 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7533[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; A801 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7534[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; konka W970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7535[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; OPS-50 Build/S710) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7536[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; SN7005 Build) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7537[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; RMD-1070 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7538[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID-HC719U Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7539[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; ELIYA_S868 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7540[] = "Victory_A18 Linux/3.0.13 Android/4.0.4 Release/02.23.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7541[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sl-si; C450 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7542[] = "A117_MULB Linux/3.0.13 Android/4.0.4 Release/01.24.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7543[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M4304 Build/IML74K-V1.2-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7544[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy S3 EX V2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7545[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 741C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7546[] = "Lenovo A356:Mozilla/5.0 (Linux; U;  Android 4.0.4; zh-cn; Lenovo A356/S030) AppleWebKit534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7547[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; M73 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7548[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; MIDR237 Build/MIDR237) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7549[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; ZA985 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7550[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; SN7004 Build) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7551[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; G40C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7552[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; S3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7553[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Sen Pro One Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7554[] = "M1077 Linux/3.0.13 Android/4.1.1 Release/01.02.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7555[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ar-eg; i7008 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7556[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB 180 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7557[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; AirTab P970g Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7558[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; NB 1016A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7559[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; A-PAD 114+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7560[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; T8 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7561[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; Pandora 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7562[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; NXM900MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7563[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; SNOWY Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7564[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT7220S2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7565[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; WAISONIC TABLET Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7566[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; 712F(0901) Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7567[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; V9-1688-CJ-0308-31S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7568[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-tw; HTC_PROMIN_U Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7569[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Z7 Build/Android4.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7570[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; equiso.smart.tv.pro Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7571[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TPC0708 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7572[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; VIT-T1310 Build/2.7AG) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7573[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; YPY_AB7DC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7574[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; J-Q88S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7575[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JD-716SG-1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7576[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; zh-cn; sprd-GT8160/1.0 Android/4.0.1 Release/04.11.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7577[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID9760 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7578[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAB803 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7579[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Y7-3L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7580[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KT918+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7581[] = "sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/Android4.0.3 Release/04.01.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S7582[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; M76 Build/ICS.g08refem667.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7583[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; PRO4611 Build/PRO4611-V1.0-2013.03.22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7584[] = "Mi-A401 Linux/3.0.13 Android/4.0.4 Release/03.29.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7585[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AW880 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7586[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-gb; PI N005 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7587[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; f761B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7588[] = "G515 Linux/3.0.13 Android/4.0.4 Release/12.11.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7589[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; SAMURAI10(QuadCore) Build/SAMURAI10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7590[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AIRIS TM475 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7591[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Y-B07N Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7592[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Spark Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7593[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM204SH Build/S0004) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7594[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TM450 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7595[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; zh-tw; imatch i15 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7596[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TAB 8200C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7597[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Q88B-12S Build/IMM76D)";
const char S7598[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U817 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S7599[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; ACTIVE 10-A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7600[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SX Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7601[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IT-708 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7602[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 3G7271 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7603[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S3PRO Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7604[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N8000_USA_RS Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7605[] = "Mozilla/5.0 (Linux; U; Android 4.0.2; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.0.2 Release/04.24.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7606[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; A088 Build/IML74K)";
const char S7607[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; TCL S600 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S7608[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; EBEST V7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7609[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SFAS  T1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7610[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CS9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7611[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID09710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7612[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GIO-GiONEE_C610/SV1.0; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7613[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GLORY Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7614[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U880F1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7615[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; 6577 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7616[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JXD-S6600 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7617[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; kiwi k5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7618[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Zyrex_ZA989 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7619[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; MyPad 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7620[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; YPY_S400 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7621[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; s4504 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7622[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MT29 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7623[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; GV7780 Build/GV7780) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7624[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A630t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7625[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; 5876 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7626[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LXK901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7627[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TM-MID788 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7628[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; SHOGUNI010(QualCore) Build/SHOGUNI010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7629[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; IMO Y5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7630[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; ELIYA S858 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7631[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-en; bluedigit Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7632[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Pixcom PGM398 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7633[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; 8720 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7634[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; SMP45-210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7635[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; K-TOUCH W780 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7636[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Vexia Tablet 780 Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7637[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GPLUS-P7002 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7638[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; F817 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7639[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; F326 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7640[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HBD-761I Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7641[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; 7558L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7642[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; Next7D12 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7643[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; PC7134 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7644[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; @TAB-NINE-X1-B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7645[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; sprd-M2/1.0 Android/4.0.4 Release/04.24.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7646[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; POV_P517-8GB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7647[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; X45 Build/BCM21654AndroidTK_SystemRel_1.3.2P.05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7648[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xplorer T8i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7649[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; V8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7650[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; P76v(A3V5) Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S7651[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ph; Starmobile ENGAGE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7652[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7653[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; X2E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7654[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GIO-GiONEE_C700/SV1.0; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7655[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; technika8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7656[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; M730 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7657[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; OS969 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7658[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U960s3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7659[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; E738 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7660[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JXD-S9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7661[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; HWA680 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7662[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A9B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7663[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; MID9035C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7664[] = "Mozilla/5.0 (Linux; U; Android 4.0.2; zh-cn; senyujie Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7665[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; Navon Platinum Explorer Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7666[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Freelander I20 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7667[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; AVA747 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7668[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; KliTON-I51SH Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7669[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; OPSSON iMO1000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7670[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; T9108 Build/MocorDroid2.3.5) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S7671[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; ar-eg; BERAQ NB9305 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7672[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KONKA W990 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7673[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; VOVO_V5 Build/SunVOVO_V5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7674[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A630t Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S7675[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; 8150D Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7676[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MID-702SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7677[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; T735 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7678[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; START702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7679[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; NIU TEK 3G 3.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7680[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Coolpad 5213 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7681[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SAMSUNG-GT-B9388_TD/1.0 Android/4.0.4 Release/09.15.2012 Browser/AppleWebKit534.30 Build/IMM76D) ApplelWebkit/534.30 (KHTML,like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7682[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; M713 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7683[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Wb7-L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7684[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U817 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7685[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; G158 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7686[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; JS401 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7687[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; f761I Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7688[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MB9500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7689[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 3G7334 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7690[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HS-U909 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7691[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; Tablet Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7692[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; p507 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7693[] = "Mozilla/5.0 (Linux; U; AliyunOS 2.0; Android 4.0 Compatible; en-us; G9 Build/AliyunOs-2012) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7694[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; S85 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7695[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Atom Action Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7696[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; TAX Build/ICS.g08refmid1061.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7697[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PTBL9C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7698[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TJLMV70Q8-CPT-PHV0.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7699[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N78-M02 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7700[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cloudpad 970d Build/FSV01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7701[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I9082 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7702[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; TB-52 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7703[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Sky Plus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7704[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; innos i5plus Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7705[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; A851 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7706[] = "Infinix BUZZ Linux/3.0.13 Android/4.0.4 Release/04.13.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7707[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BOGO BO-LFSP5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7708[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; A22 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7709[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; PA0750 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7710[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; PRO4611 Build/PRO4611-V1.0-2013.04.29) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7711[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; iJoy Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7712[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; CASPER Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7713[] = "Mozilla/5.0 (Linux; U; Android 4.0.6; zh-tw; PONY_Speed Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7714[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ZYNC Z919 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7715[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; S600 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7716[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LA-I Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7717[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; W20 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7718[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; TM801 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7719[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ki60 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7720[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Galaxy Note DK9500 B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7721[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; V926B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7722[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; A76 Build/IMM76D)";
const char S7723[] = "GPLUS N809 Linux/3.0.13 Android/4.0.4 Release/10.26.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7724[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB-N7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7725[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; KIRA N10050 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7726[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; TP8-1000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7727[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; QN-7000HX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7728[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; eZee'Tab708 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7729[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; pad T3-1211 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7730[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; TPC7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7731[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; F8AS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7732[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; PRO 9.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7733[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; AK45 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7734[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST0916 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7735[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; BV4000 Build/ICS.f08refem32.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7736[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; TP7-1000-metal Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7737[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; eZee'Tab703 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7738[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Joypad C102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7739[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; DOOV_D910 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7740[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Ifa303 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7741[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; TA-M72CBT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7742[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q13T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7743[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; zh-tw; PB-S6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7744[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EXLP3P Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7745[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; T-990A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7746[] = "DOMO SLATE X3G SE Linux/3.0.13 Android/4.0.4 Release/11.24.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7747[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; z1000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7748[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; M4302 Build/IMM76D)";
const char S7749[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Android Smart TV Box Build/ICS.MBX.GIGATV.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7750[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Onliner1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7751[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HS-E956 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7752[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Chas100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7753[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-T327d/1.11.1401.9) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7754[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8826D/C8826DV100R001C92B620; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S7755[] = "KliTON I806LS Linux/3.0.13 Android/4.0.4 Release/01.09.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7756[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; S06 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7757[] = "Mozilla/5.0 (Linux; U; Android 4.0.8; es-us; MID7102SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7758[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MT-708G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7759[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; DOUBLE V Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7760[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru;Infinityll Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7761[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Quicki-729 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7762[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; TabLife Build/GT704-A13004-B02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7763[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PGM 398 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7764[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Solution8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7765[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; LC0723B Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7766[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7566 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7767[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Themis II Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7768[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-pt; TX200 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7769[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; SGC-S3 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7770[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Surf Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7771[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M1052S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7772[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Avvio_790s Build/AvvioAvvio_790s) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7773[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; softwinners Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7774[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; Lanix Ilium_S400 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7775[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB973G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7776[] = "Pro-S Linux/3.0.13 Android/4.0.4 Release/03.06.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7777[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MIDR477 Build/eng.glwx.X.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7778[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AXPAD 9E01 Build/ICS.g08refema9.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7779[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; YF730B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7780[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; NE-202 Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7781[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; C15 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7782[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TF--W700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7783[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Androtab7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7784[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; AW133 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7785[] = "K-TouchT789_TD/1.0 Android 4.0.3 Release/10.01.2012 Browser/WAP2.0 appleWebkit/534.30";
const char S7786[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; DC   Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7787[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; NE-103T Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7788[] = "Mozilla/5.0 (Linux; U; Android 4.0.4;en-us;Wynncom G41 build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7789[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; POMP_P8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7790[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ektATB-1108 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7791[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nam-Phone G40C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7792[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; 588 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7793[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T18+ Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7794[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 9930 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7795[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; T701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7796[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DT744B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7797[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; sprd-GT-N9300/1.0 Android/4.0.3 Release/04.02.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7798[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KINGTOP977 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7799[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; keytak15_ics Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7800[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LC0725B Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7801[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M707-W Build/1.0.883_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7802[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; IRON Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7803[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; sprd-9300/1.0 Android/4.0.3 Release/04.11.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7804[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; W33 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7805[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TabletPlus Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7806[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SKYPAD EXPRESS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7807[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; CA-201L Build/CA201LU377_00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7808[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NX700G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7809[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; E-tel i7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7810[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A7-11 Build/ARES_00.01.61.27.18.H1111F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7811[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; Q93S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7812[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; PC7058ME Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7813[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; FMT-NT7A40-02 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7814[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xplorer T7i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7815[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; A7# Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7816[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; 700P11A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7817[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M980 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7818[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Infinix X351 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7819[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NEC-NE-201A1A Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7820[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; Lanix Ilium_S210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7821[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; zh-cn; 9000H Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7822[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID-7088B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7823[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; EBEST V8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7824[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; Exagerate XZPAD970H Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7825[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; X5 Build/LOVMEX5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7826[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; DOUBLE I Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7827[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fa-ir; LD-453 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7828[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; B2000+ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7829[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Easy Home Tablet 8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7830[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; iB750C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7831[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; M4302 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7832[] = "Mozilla/5.0 (Linux; U; Android 4.0.6; vi-vn; CHI_Speed Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7833[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; S9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7834[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; YPY_10STB Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7835[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN868H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7836[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A96 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7837[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ar-eg; Mercury_magiQ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7838[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X808 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7839[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LA-I Build/IMM76I)";
const char S7840[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; YPY_TQ7 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7841[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; Polypad C504 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7842[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID-8000B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7843[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; 5870 Build/IMM76D)";
const char S7844[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TB-7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7845[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; EJTAB97A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7846[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Slicepad L7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7847[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; QW TB-1507 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7848[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; N710 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7849[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; 97M Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7850[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HOSIN U9 Build/HOSINA75P) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7851[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ULTRATAB F7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7852[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; FTABC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7853[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; CASPER Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7854[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; NB 2012 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 ?????? ???????Safari/534.30";
const char S7855[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GTC7.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7856[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; 1FC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7857[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7858[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ETON T770 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7859[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Swipe Halo Edge Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7860[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; ITAB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7861[] = "My503 Linux/3.0.13 Android/4.0.4 Release/11.16.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7862[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HX-GT1001 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7863[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; Wmate Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7864[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch U7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7865[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; DOOV_D50 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7866[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; PC8008 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7867[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; myTab 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7868[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; i90 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7869[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U980 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7870[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo S696 Build/IMM76I) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.2 Version/4.0 Mobile Safari/530.17";
const char S7871[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; TP73G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7872[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Nautica 3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7873[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; G_33_HLI Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7874[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; P860 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7875[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A530 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7876[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Coolgen E69 Note II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7877[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; BF9200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7878[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; E703H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7879[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-one Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7880[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TB-7HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7881[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; TURBOX STRIKE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7882[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; T6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7883[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; Moii E991 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7884[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A3-7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7885[] = "Mozilla/5.0 (Linux; U; Android 4.0; zh-cn; NOAIN A900S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7886[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; MID7040T-3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7887[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 9703 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7888[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; 8190 Build/IML74K)";
const char S7889[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SHV-E140K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7890[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MID800SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7891[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; CR7A10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7892[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 86VMC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7893[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; L100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7894[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; E960 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7895[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; I790 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7896[] = "Mozilla/5.0 (Linux; U; Android 4.0.0; en-us; Android_6515M Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7897[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GPLUS_GN858 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7898[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; SIM-Note Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7899[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; PICOpad GHM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7900[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; MD7072 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7901[] = "PC-828 Linux/3.0.13 Android/4.0.4 Release/11.09.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7902[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; TY-K Touch W656/K-Touch_W656_V1.1; 480*800;  CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7903[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; DOOV_i1314 Build/DOOVi1314) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7904[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; F918 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7905[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PAD1016 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7906[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Galileo Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7907[] = "P80w Linux/3.0.13 Android/4.0.4 Release/12.11.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7908[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M9703T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7909[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JIL 1101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7910[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; RT-MOMO07 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7911[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; TCL-D662/EG27_YMYM_V0.6; 480*800; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7912[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; HIKe 716 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7913[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; U360 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7914[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Omega S30 Build/ICS_IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7915[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; NV90 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7916[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; BF9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7917[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Coolpad 7235 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S7918[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; w500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7919[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LNV-Lenovo_S870e/Lenovo_S870e_S022_X; 540*960; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7920[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; _ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7921[] = "ZA987 Linux/3.0.13 Android/4.0.4 Release/11.22.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7922[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; Q79 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7923[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; advance Build/ICS.advance_30_0_8188.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7924[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; METAPAD-C2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7925[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; F761 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7926[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; UBPH1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7927[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Atom Action Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7928[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; K-TOUCH W610D Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7929[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; AK8461 Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7930[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; TB-90103G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7931[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; lt901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7932[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; 3310-9506 Build/ICS.Zoostorm_130_NEW.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7933[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDZ Race Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7934[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; Starmobile ENGAGE 9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7935[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MP10 Build/ICS.g08refem611.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7936[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SMP50-200 Build/1.0.316_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7937[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; UM77-V1.1-073-03-01-04-01-00-01 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7938[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Sansui Cosmic SP 1001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7939[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; zh-cn; L107 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7940[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fa-ir; G1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7941[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Coolpad 7235 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7942[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; T211 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7943[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; USmart CL600 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7944[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; RC9717B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7945[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; hebe_plus_gosco Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7946[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MID109 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7947[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; AK926 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7948[] = "A107_MULA Linux/3.0.13 Android/4.0.4 Release/01.08.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7949[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CN97V Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7950[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X406 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7951[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TURBOPAD 1000 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7952[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TwinTAB-T7283G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7953[] = "GT-I8190 Linux/3.0.13 Android/4.1.9 Release/03.01.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S7954[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; ONEPAD 940 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7955[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; MMB-525.SAT(Meteorit) Build/ICS.MBX.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7956[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; N90-2core Build/R13.4-rc2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7957[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; C007 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7958[] = "K-TouchT6_TD/1.0 Android 4.0.3 Release/10.01.2012 Browser/WAP2.0 appleWebkit/534.30";
const char S7959[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; P900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7960[] = "Mozilla/5.0 (Linux; U; Android 4.0; ar-eg; NNB-100 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7961[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; MHL5 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7962[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DG_70 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7963[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; EDUPLE7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7964[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; P102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7965[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Mswipe Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7966[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-be; TAB-9702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7967[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; DualDMax Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7968[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; IMO_Y-ONE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7969[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EF09-2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7970[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MOMO9P Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7971[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; VN10-V8850/4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7972[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; V71B Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7973[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Swift-Tab II 70FB5G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7974[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LPO-380W Build/FR-M868) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7975[] = "Mozilla/5.0 (Linux; U; Android 4.0.6; vi-vn; V800 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7976[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; POV_TAB-P527 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7977[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TF-MID702G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7978[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JD-720SG Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7979[] = "K-Touch_U81t_TD/V2 Linux/3.0.13 Android/4.0 Release/02.15.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2 System/Android 4.0.4";
const char S7980[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; T-704S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7981[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; AirTab P970w Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7982[] = "Mozilla/5.0 (Linux; U; Android 4.0.2; zh-cn; sprd-AS133/1.0 Android/4.0.2 Release/04.17.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7983[] = "Mozilla/5.0 (Linux; U; Android 4.0.4;en-us;Digitac Ultratab;C825 Build/ICS) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7984[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; i9080 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7985[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; RMD-855 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7986[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; X802 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7987[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; TAXB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7988[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; G18??(E3G5) Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7989[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SANSUI ETAB 902VP Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7990[] = "Mozilla/5.0 (Linux; U; Android 4.0; vi-vn; C29 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7991[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V100M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7992[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB719 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7993[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Magicon Q7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7994[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ar-eg; R760 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7995[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; T935 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S7996[] = "LenovoS868t_TD/1.0 Android 4.0.3 Release/10.01.2012 Browser/WAP2.0 appleWebkit/534.30";
const char S7997[] = "Mozilla/5.0 (Linux; U; Android 4.0; th-th; U7T Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S7998[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; GALAXY_S_II_plus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S7999[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Vexia tablet 7903G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8000[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Y7-3LB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8001[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; ELIYA S868 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8002[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A60T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8003[] = "Mozilla/5.0 (Linux; U; Android 4.0; vi-; Masstel M12  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8004[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TAX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8005[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PHICOMM i813 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8006[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Ace Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8007[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; E1-B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8008[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Proton One Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8009[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PAP5430 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8010[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KENWOOD DNN 2013 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8011[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A350 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8012[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; baoxue Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8013[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; RMD-1075 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8014[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; GV7777 Build/GV7777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8015[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; 8085 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8016[] = "sprd-T101/1.0 Linux/2.6.35.7 Android/Android4.0.3 Release/03.19.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S8017[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C761 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8018[] = "Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; KT918+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8019[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; es-es; cenon15_6626_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8020[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; POV_TAB-P506 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8021[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; OVO Lite GPS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8022[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; zh-cn; sprd-skk-A2/1.0 Android/4.0.1 Release/04.24.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8023[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; S59 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8024[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; PNDPP41DG7SLV^ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8025[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SIMM-X722 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8026[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; KOMU K1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8027[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; 8085 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S8028[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Philips W737 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8029[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID711 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8030[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; DNS S3502 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8031[] = "HUAWEI Y500-T00_TD/1.0 Android 4.0.3 Release/10.01.2012 Browser/WAP2.0 appleWebkit/534.30";
const char S8032[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; inovacel S23 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8033[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X712 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8034[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S4 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8035[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-ar; RAIM7000ND_AR Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8036[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB 270 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8037[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; zh-cn; sprd-F7/1.0 Android/4.0.1 Release/05.09.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8038[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; TB-120 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8039[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PD10-DYD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8040[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-be; Lenco CARTAB-920 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8041[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; ST70104-2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8042[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DT9138B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8043[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8044[] = "Austin Linux/3.0.13 Android/4.0.4 Release/05.07.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S8045[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; S25 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8046[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KINGTOP915 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8047[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; GG-2000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8048[] = "Byond B55 Linux/3.0.13 Android/4.0.4 Release/05.02.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S8049[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hr-hr; TPC-7121 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8050[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; V6888 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8051[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; G86 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8052[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ET-GSM-A78 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8053[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; You40 Build/RheaTK) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8054[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iBall_Andi5.9L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8055[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; AS128 Build/Android4.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8056[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PLT4311 Build/PLT4311-V1.2-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8057[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; BO-LFSP4 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8058[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; G-M7904 Build/G-M7904) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8059[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 435TPC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8060[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GP353 Build/RheaTK) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8061[] = "DTC GT9 Linux/3.0.13 Android/4.0.4 Release/05.14.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S8062[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; PHM727MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8063[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TM-3204R Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8064[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HBD-761K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8065[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AT007F Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8066[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BLU AMOUR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8067[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE N798 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8068[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; DOOV D360 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8069[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XELIO_P900A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8070[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; AS125 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8071[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; F6S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8072[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; OCEATAB7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8073[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; CA-201L Build/CA201LU410_00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8074[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; Smart 3 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8075[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; D2-430 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8076[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X700 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8077[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; F307-B Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8078[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8079[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; sprd-GT-N9300/1.0 Android/4.0.3 Release/06.06.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8080[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; AIKUN_AT73C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8081[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X-Treme Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8082[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB507G Build/ICS_IMM76I_B511) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8083[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Oysters T7D 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8084[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; PC704 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8085[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Z10 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8086[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; i-Call 501 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8087[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ONIDA i555 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8088[] = "Celkon A105 Linux/3.0.13 Android/4.0.4 Release/07.05.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S8089[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; L29K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8090[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ec2cloud_v2 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8091[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ec2cloud_v2 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8092[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; X15 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8093[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; M805HCX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8094[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch S757/K-Touch_S757_V1.1; 480*854;  CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8095[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; SMX-T9701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8096[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; BIRD A05 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8097[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; OS805 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8098[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Q8E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8099[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8100[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; generic_x86 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8101[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; D5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8102[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch S757/K-TOUCH_S757_V2.0; 480*854;  CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8103[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; U910 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8104[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; N70DC-S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8105[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TM-MID725 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8106[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; DOUBLE II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8107[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; S9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8108[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch W710 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8109[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; CT7013 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8110[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; T900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8111[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; 7266 Build/IMM76I)";
const char S8112[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; TB-107 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8113[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDZ Flash Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8114[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; SGW920 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8115[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; OUKI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8116[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; T-22B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8117[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; ar-eg; G1   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8118[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Q88B-11S Build/IMM76D)";
const char S8119[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; DAKELE MC001 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8120[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; F3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8121[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; Lanix Ilium_S500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8122[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch W689 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8123[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T-711D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8124[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; L8202 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8125[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; NOVO NV7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8126[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; NTT 730 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8127[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LNV-Lenovo S870e Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8128[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; ELEMENT 7 V2 BASIC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8129[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TB1082 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8130[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X50 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8131[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; lt_MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8132[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A356 Build/LenovoLePhone) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8133[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SMARTTVBOX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8134[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MOFUT-F8SH Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8135[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M711 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8136[] = "Mozilla/5.0 (Linux; U; Android 4.0.8; en-us; B1-EP2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8137[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Yummy-Duo Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8138[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; PGM 399 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8139[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GiONEE GN600 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8140[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; STS CA SERIES Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8141[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; SENCOR ELEMENT 9.7V3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8142[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; BF_A12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8143[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; P617S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8144[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AK401 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8145[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; B-TAB1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8146[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; NYX_FLY Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8147[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; UM77-V1.1-073-06-01-02-02-02-01 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8148[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; DL 3D MaxView TD-M71 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8149[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; E566 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8150[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID1054 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8151[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; TY-001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8152[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MBO Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8153[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-uk; mini7100 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8154[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Z6 Build/Android4.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8155[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NID-7006 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8156[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; Diamond_Lite Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8157[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; S758 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8158[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A55 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8159[] = "HTC Linux/3.0.13 Android/4.1.9 Release/02.26.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S8160[] = "X8 NOTE Linux/3.0.13 Android/4.0.4 Release/04.07.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S8161[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; E721L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8162[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TW900A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8163[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GEM7032G-rev2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8164[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; Treq 3G Basic 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8165[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LNV-Lenovo_S870e/Lenovo_S870e_S028_X; 540*960; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8166[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Aspire A1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8167[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PA938D Build/PA938D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8168[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TPC0902 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8169[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; ongo S5020 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8170[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-us; EX S20i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8171[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; hu-hu; PHA-3850 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8172[] = "Mozilla/5.0 (Linux; U; Android 4.0.5; en-us; 7080 Build/MocorDroid4.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8173[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; T7040 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8174[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S280 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8175[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; 820 Build/820-V1.0-2013.03.04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8176[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; sprd-Citycall-I930+/1.0 Android/4.0.3 Release/04.23.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8177[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; i88S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8178[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; F&U ETB7404 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8179[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TABLET LO+DEMODA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8180[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; Tab1031 Build/ICS.g08refmid1061.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8181[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; 9 inch Economy Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8182[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; Q74A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8183[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB 171 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8184[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CY7809 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8185[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; VIVAS_LOTUS_S1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8186[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; F761K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8187[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GXPAD300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8188[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; N5002 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8189[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CRCB_2093 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8190[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; VIT-T4000 Build/0.29VSG) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8191[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Mi-A351 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8192[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; TR71 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8193[] = "Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; GL Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8194[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 43BMID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8195[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; Dslide 971 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8196[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM204SH Build/S0016) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8197[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T5-B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8198[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDZ Drive+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8199[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TP12A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8200[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; MD1020 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8201[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; zh-cn; BIRDA_MODEL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8202[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; PC8005 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8203[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; P9000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8204[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; B12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8205[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; vollo Vx93 Build/volloVx93) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8206[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; MID950 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8207[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; PROG150 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8208[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; V700H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8209[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; MID7200 Build/MID7200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8210[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; PMID70C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8211[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; F-ONE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8212[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Woxter Tablet PC 76CXi Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8213[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; PMID70DC Build/unknown) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8214[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Chungha Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8215[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; g08ref Build/ICS.g08ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8216[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M9701TW Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8217[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A703H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8218[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; J-Q5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8219[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TAB-970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8220[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; MID4502 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8221[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; K-Touch T780/TBTX_8660_V0101) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Linux/3.0.8 Safari/534.30";
const char S8222[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; S4702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8223[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; DC   Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8224[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 86VE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8225[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; OKU1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8226[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; F8Cs Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8227[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Swift-Tab II 10WBNG Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8228[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Aole Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8229[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Z6_FWVGA_XD Build/Android4.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8230[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo A630t Build/IMM76D)";
const char S8231[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; WTAB 704 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8232[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDZ Ultimate Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8233[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JD-716SG Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8234[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MMD-X200 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8235[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; T272 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8236[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; AT7E-A13BP Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8237[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; ET9-1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8238[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB975G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8239[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q27H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8240[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M756 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8241[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; PICOpad GGD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8242[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Calipso4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8243[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; Smart 4 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8244[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; CORE DIGITAL CD-713WD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8245[] = "MP200 Linux/3.0.13 Android/4.0.4 Release/05.16.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S8246[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; E9000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8247[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A704L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8248[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; P700+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8249[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X65 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8250[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; AS131 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8251[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; A70A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8252[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; TB-721 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8253[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AX5_Duo Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8254[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; 29548 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8255[] = "Mozilla/5.0 (Linux; U; Android 4.0.357; en-us; UMO T99 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8256[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; QM734-2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8257[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KB-Karbonn A9*/Karbonn_A9*_V1.3; 480*800;  CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8258[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KB-Karbonn A9*/Karbonn_A9*_V1.2; 480*800;  CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8259[] = "Mozilla/5.0 (Linux;U; Android 4.0.4;en-us;Digitac Ultratab;C1029 Build/ICS) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S8260[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.0.3;dev=holiday;devm=HTC PH39100;man=HTC;brw=Android;shwd=540x960;shwa=540x923;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S8261[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MID-A10 Build/IMM76D)";
const char S8262[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; NEOBOX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8263[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A710e Build/ICS) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S8264[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; EBEST S6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8265[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch E6 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8266[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; N970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8267[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; NB021/22 Build/1.0.560_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8268[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; ar-eg; BlackBird_I7000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8269[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; PAD705 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8270[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; M901D1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8271[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; A9724 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8272[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; W970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8273[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; AigoPad Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8274[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; ASK SP558 HD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8275[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LNV-Lenovo S870e Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S8276[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; Ezcool Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8277[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N9500/N7100 PAD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8278[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; V10A Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8279[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; 7I01 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8280[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Spring W-100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8281[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CTA Build/ICS.casper.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8282[] = "MOFUT T3MT Linux/3.0.13 Android/4.0.4 Release/01.12.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S8283[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D36 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8284[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; T275 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8285[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; FR-M768 Build/FR-M768) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8286[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; QBA757 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8287[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; TA1033G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8288[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; 989 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8289[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch W700+ Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8290[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; Mobia J Build/RheaTK) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8291[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; S4702M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8292[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; FLY F50s Build/ICS_IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8293[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Z7_WVGA_TL Build/Android4.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8294[] = "sprd-G910/1.0 Linux/2.6.35.7 Android 4.0.4 Release/04.10.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S8295[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; T77 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8296[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; M74B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8297[] = "Extreme 4 Linux/3.0.13 Android/4.0.4 Release/05.13.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S8298[] = "HUAWEI Y320-T00_TD/1.0 Android 4.0.3 Release/10.01.2012 Browser/WAP2.0 appleWebkit/534.30";
const char S8299[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI M881 Build/HuaweiM881) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8300[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; P900i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8301[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; PI T005 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8302[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A630/S001) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8303[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Teta Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8304[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ONIDA i010 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8305[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.0.4;dev=SGH-I717;devm=SAMSUNG-SGH-I717;man=samsung;brw=Android;shwd=800x1280;shwa=800x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S8306[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S2-38AH0 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8307[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; G5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8308[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; PAD511 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8309[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; swipe Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8310[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Z520e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8311[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID-HC719 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8312[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Q8 Build/IMM76D)";
const char S8313[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ING800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8314[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; T8+ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8315[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; PI E005 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8316[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; Exagerate XZPAD750 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8317[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; UM77-V1.1-070-01-00-01-01-00-00 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8318[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; I5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8319[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ro-ro; EVOTAB3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8320[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; V929B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8321[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; MIRA-2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8322[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; INTRACON FX2 PAD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8323[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; K-Touch U6 Build/IMM76I)";
const char S8324[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; HYP008 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8325[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; E800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8326[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; Moii E990 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8327[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; 86V2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8328[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; BOUBLE VI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8329[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Clear Build/IMM76D)";
const char S8330[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BM mobile A6000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8331[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TB-7 HD 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8332[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MFC170FR_01 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8333[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Hphone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8334[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; UTime_T1+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8335[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; miTab SPACE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8336[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; FWS710EU Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8337[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; Full AOSP on godbox Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8338[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8339[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ChanghongW6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8340[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; NB021/22 Build/1.0.546_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8341[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; powerpadMAXX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8342[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; EvoPAD T7016 Build/EvoPAD_T7016) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8343[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Mercury X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8344[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TM-5377 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8345[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; D628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8346[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; A11Note Build/IMM76D)";
const char S8347[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; NB036 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8348[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; fa-ir; T9988 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8349[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GN168T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8350[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EZPAD 745C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8351[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; M703 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8352[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; FW8937 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8353[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; PTM805MCL Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8354[] = "WaterProof Linux/3.0.13 Android/4.0.4 Release/02.23.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S8355[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; JT Hot Q/JT_Hot Q_V1.0; 480x800; APT/2.0) AppleWebkit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S8356[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB972 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8357[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; IHA-C0801 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8358[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ar-eg; A330 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8359[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-8220S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8360[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; TOPTIL T900 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8361[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; vivo S7t Build/IMM76D)";
const char S8362[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PL7117 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8363[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AX8RACE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8364[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ANDY 3G 3.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8365[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Pentagram Ego Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8366[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A721J Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8367[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-; DC   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8368[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; S4 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8369[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; TR101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8370[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; DAYA A300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8371[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; DL 3G MOBILE TG-M73_2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8372[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Icon 2G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8373[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; zh-cn; sprd-GT8160/1.0 Android/4.0.1 Release/05.14.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8374[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; DualDMax Build/DualDrive) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8375[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IT-708C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8376[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; QM734 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8377[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; UPAD Go Smart G312 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8378[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; P91 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8379[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V703 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8380[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; sprd-I8730/1.0 Android/4.0.4 Release/05.23.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8381[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Axxion ATAB-701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8382[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; PICOpadGCE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8383[] = "Mozilla/5.0 (Linux; U; Android 4.0; vi-vn; smart five Build/Android4.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8384[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; OPAL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8385[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; RC0709B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8386[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; i-Call 300v2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8387[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N900 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8388[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8389[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; E4001 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8390[] = "N7500 Linux/3.0.13 Android/4.0.4 Release/07.12.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S8391[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AX3Duo Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8392[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; PICOpad GGG Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8393[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; msm8625 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8394[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; Cloud Y5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8395[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DT9138 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8396[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; V8200plus Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8397[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GFIVE A86 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8398[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GN360 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8399[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Coolpad 7290 Build/IMM76D)";
const char S8400[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M5000 Build/1.0.697_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8401[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; BOWAY I7 Build/BOWAYA75P) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8402[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZP111 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8403[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAB466 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8404[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; K-Touch E688 Build/IMM76I)";
const char S8405[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; gs701b Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8406[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; F-074 Build/V02R11A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8407[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; TAB9 3G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8408[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MID7117CM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8409[] = "A1G Linux/3.0.13 Android/4.0.4 Release/11.22.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S8410[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; TP860A Build/ICS.g08refem618.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8411[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; BF_G11 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8412[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; android ics Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8413[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TZX-cpt Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8414[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AM736 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8415[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; CTAB700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8416[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; PMP3570C Build/PMP3570C_X_v1.0.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8417[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; Tablet Build/GT704-A13005-E01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8418[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; MT0724B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8419[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Z6_WVGA_LC Build/Android4.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8420[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Connspeed T50 Build/ICS_IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8421[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Tablet Rebel Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8422[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; 8720 Build/IML74K)";
const char S8423[] = "Mozilla/5.0 (Linux; U; Android 4.0; vi-vn; Masstel M12  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8424[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; VP11 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8425[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Aero A1-110 Build/ICS.bangho.gt8105.130+.all.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8426[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; TB-100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8427[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; E721J Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8428[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; TAB7002 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8429[] = "Mozilla/5.0 (Linux; U; Android 4.0; es-es; DC   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8430[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; N7800 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8431[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Beta Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8432[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MMD-P706 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8433[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; reeder A7S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8434[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; AX540 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8435[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; f761K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8436[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; ChildPad Lazer Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8437[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ZTE U808 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S8438[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Android Style one Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8439[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S72-A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8440[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-us; AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8441[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NOTE_CREATIVE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8442[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; S4001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8443[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Oysters T37 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8444[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Masstel M500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8445[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; Radiomarelli SevenQ Build/ICS_IMM76I_B511) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8446[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; CT0712 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8447[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TSM-TP3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8448[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; MT7015 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8449[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.0.4;dev=edison;devm=MB865;man=motorola;brw=Android;shwd=540x960;shwa=540x923;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S8450[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-us; A51 Build/MocorDroid4.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8451[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8452[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I535 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8453[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8454[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T999 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8455[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9300 Build/IMM76D)";
const char S8456[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9300/I9300BUALE6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8457[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SC-06D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8458[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9300 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8459[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; GT-I9300T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8460[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SAMSUNG GT-I9300/I9300XXALE8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8461[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-T999V Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8462[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300-ORANGE/I9300BVALE4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8463[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-I535 Build/IMM76D)";
const char S8464[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9300/I9300BUALF1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8465[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9300/I9300XXALF2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8466[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9300/I9300BVALE4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8467[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I535 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8468[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9300/I9300XXALF6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8469[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-T999 Build/IMM76D)";
const char S8470[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300-ORANGE/I9300BVALF4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8471[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9300/I9300BVALF4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8472[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SAMSUNG GT-I9300/I9300XXBLFB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8473[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SCH-I939 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8474[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8475[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S8476[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9300T Build/IMM76D)";
const char S8477[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300-ORANGE/I9300BVBLG2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8478[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9300/I9300BUBLG3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8479[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9300/I9300BVBLG2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8480[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; SAMSUNG GT-I9300/I9300XXALF5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8481[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S8482[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SCH-I939 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S8483[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SC-06D Build/IMM76D)";
const char S8484[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; SAMSUNG GT-I9300/I9300XXBLH1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8485[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-T999V Build/IMM76D)";
const char S8486[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SAMSUNG GT-I9300/I9300XXBLH3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8487[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300-ORANGE/I9300BVBLH2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8488[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8489[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SGH-T999 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8490[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S8491[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9300/I9300BVBLH2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8492[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-I9300T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8493[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8494[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8495[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG GT-I9300/I9300BUBLG3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8496[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T999 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S8497[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I535 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S8498[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SAMSUNG GT-I9300/I9300XXALEF Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8499[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S8500[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I93000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8501[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T999 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S8502[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SCH-I535 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8503[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SCH-I939 Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S8504[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SGH-T999V Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8505[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; GT-l9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8506[] = "Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S8507[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG GT-I9300/I9300BVALF4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8508[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; it-it; I9300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8509[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9300 Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8510[] = "GT-I9300 Linux/3.0.13 Android/4.1.9 Release/01.04.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S8511[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-l9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8512[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SCH-I535 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S8513[] = "Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; SCH-I939 Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S8514[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-I9300-ORANGE/I9300BVBLH2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8515[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8516[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/RUISI.ORG) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8517[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-I939 Build/IMM76D)";
const char S8518[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; es-es; I9300+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8519[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-I9300 Build/IML74K)";
const char S8520[] = "Mozilla/5.0 (Linux; U; Android 4.0.8; en-us; I9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8521[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9300 Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8522[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-I9300-ORANGE/I9300BVBLG2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8523[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-US; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8524[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/ZSALG5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8525[] = "GT-9300 Linux/3.0.13 Android/4.1.9 Release/12.05.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S8526[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG GT-I9300/I9300BVALE4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8527[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG GT-I9300/I9300BVBLG2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8528[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; I9300 Build/IMM76D)";
const char S8529[] = "I9300 Linux/3.0.13 Android/4.1.9 Release/05.21.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S8530[] = "Samsung_I9300 Linux/3.0.13 Android/4.1.2 Release/03.19.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S8531[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-I9300 Build/IceCreamSandwhich) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8532[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; I9300i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8533[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; GT-I9300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S8534[] = "Mozilla/5.0 (Linux; U; Android 4.0.6; en-us; I9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S8535[] = "Samung_I9300 Linux/3.0.13 Android/4.1.1 Release/03.19.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S8536[] = "Domnutch-Bot/Nutch-1.0 (Domnutch; http://www.Nutch.de/)";
const char S8537[] = "KSCrawler/Nutch-1.0 (http://www.kindsight.net/en/kscrawler; crawler@kindsight.net)";
const char S8538[] = "IS Alpha/Nutch-1.1";
const char S8539[] = "Attentio/Nutch-0.9-dev (Attentio's beta blog crawler; www.attentio.com; info@attentio.com)";
const char S8540[] = "Aghaven/Nutch-1.2 (www.aghaven.com)";
const char S8541[] = "My Nutch Spider/Nutch-1.3";
const char S8542[] = "Mozilla/5.0 (compatible; OpenindexDeepSpider/Nutch-1.5-dev; +http://openindex.io/spider.html; systemsATopenindexDOTio)";
const char S8543[] = "Nutraspace/Nutch-1.2 (www.nutraspace.com)";
const char S8544[] = "Googlebot/Nutch-0.9-dev";
const char S8545[] = "My Nutch Spider/Nutch-1.4";
const char S8546[] = "COMODOSpider/Nutch-1.2";
const char S8547[] = "MaxPointCrawler/Nutch-1.1 (maxpoint.crawler at maxpointinteractive dot com)";
const char S8548[] = "nutch-1.4/Nutch-1.4";
const char S8549[] = "HAZY.SPIDER/Nutch-1.4";
const char S8550[] = "CloudACL/Nutch-1.4";
const char S8551[] = "SemrushBot/Nutch-1.5-SNAPSHOT";
const char S8552[] = "Netluchs/Nutch-1.0 ( ; http://www.netluchs.de/bot.html; _do_not_spam_me___humans_please_use_info_at_netluchs.de_without_the_dash)";
const char S8553[] = "spotinfluence/Nutch-1.4 (Spot Influence crawler; http://spotinfluence.com; hello at spotinfluence dot com)";
const char S8554[] = "NutchCVS/0.7.2 (Nutch; http://lucene.apache.org/nutch/bot.html; nutch-agent@lucene.apache.org)";
const char S8555[] = "RADaR-Bot/Nutch-1.4 (http://radar-bot.com/)";
const char S8556[] = "googlebot/Nutch-1.4";
const char S8557[] = "wttsearch/Nutch-1.0 (Crawler for SIFE (WTT); http://wtt.beecom.ch/wttsearch.html; wtt@beecom.ch)";
const char S8558[] = "lmspider/Nutch-0.9-dev (For research purposes.; www.nuance.com; lmspider@nuance.com)";
const char S8559[] = "emBot-GalaBuzz/Nutch-1.0 (http://emining.jp/; em@galabuzz.jp)";
const char S8560[] = "lmspider/Nutch-0.9 (For research purposes.; www.nuance.com; lmspider@nuance.com)";
const char S8561[] = "My Nutch Spider/Nutch-1.2";
const char S8562[] = "Nutch/Nutch-1.4 (A semantic crawler for a PhD thesis; http://issel.ee.auth.gr/doku.php/research; kvavliak at issel dot ee dot auth dot gr)";
const char S8563[] = "ExactSeek Crawler (nutch 1.4)/Nutch-1.4";
const char S8564[] = "Kiodia Spider/Nutch-1.4";
const char S8565[] = "ScholarUniverse/0.8 (Nutch; http://scholaruniverse.com/bot.jsp; fetch-agent@scholaruniverse.com)";
const char S8566[] = "merocrawl/Nutch-1.4 (merobase crawler; www.merobase.com; info@merobase.com)";
const char S8567[] = "wminer/Nutch-1.4";
const char S8568[] = "My Spider/Nutch-1.4";
const char S8569[] = "My Nutch Spider/Nutch-1.5";
const char S8570[] = "WikiDo/Nutch-1.4 (http://wikido.com; crawler@wikido.com)";
const char S8571[] = "crawler/Nutch-1.4";
const char S8572[] = "TosCrawler/Nutch-1.4 (http://www.toshiba.co.jp/rdc/about/crawl_info.htm; 'Rdc-crawler at ml dot toshiba dot co dot jp')";
const char S8573[] = "OpenWebIndex/Nutch-1.5";
const char S8574[] = "KinglandSystemsCorp/Nutch-1.2 (Web crawler run by Kingland Systems Corp; http://www.kingland.com; kyle dot hansen at Kingland dot com)";
const char S8575[] = "HD nutch agent/Nutch-1.1 (Think)";
const char S8576[] = "My Nutch Spider/Nutch-1.5.1";
const char S8577[] = "spider/Nutch-1.5.1 (spider; http://www.xxx.com)";
const char S8578[] = "NexiSpider/Nutch-1.5.1";
const char S8579[] = "TosCrawler/Nutch-1.5.1 (http://www.toshiba.co.jp/rdc/about/crawl_info.htm; 'Rdc-crawler at ml dot toshiba dot co dot jp')";
const char S8580[] = "Nutch/Nutch-2.0 (Nutch Crawler)";
const char S8581[] = "MSNBot/Nutch-1.5.1";
const char S8582[] = "NSIDC Libre Crawler/Nutch-1.5.1";
const char S8583[] = "NutchCVS (Nutch; http://lucene.apache.org/nutch/bot.html; nutch-agent@lucene.apache.org)";
const char S8584[] = "Spiderman/Nutch-1.5";
const char S8585[] = "vik-robot/Nutch-1.0 (vikspider; http://vik.com; vikspider@163.com )";
const char S8586[] = "Mysite/Nutch-2.0";
const char S8587[] = "Parveen Nutch Spider/Nutch-1.5";
const char S8588[] = "WebDataCommonsCrawler/Nutch-1.5.1";
const char S8589[] = "Mu Nutch Spider/Nutch-1.5.1";
const char S8590[] = "AstuteSocialCrawler/Nutch-1.4";
const char S8591[] = "hiva/Nutch-2.0-dev";
const char S8592[] = "MetricAlerts/Nutch-2.0 (Media monitoring; http://socialartisan.co.uk; crawler@socialartisan.co.uk)";
const char S8593[] = "spider/Nutch-2.0";
const char S8594[] = "Matt_Crawler/Nutch-1.5.1 (Academic test crawl; research_crawler at yahoo)";
const char S8595[] = "Affectv Robot v1.0/Nutch-2.1";
const char S8596[] = "Ruilin's Nutch Spider/Nutch-1.5";
const char S8597[] = "TosCrawler/Nutch-1.6 (http://www.toshiba.co.jp/rdc/about/crawl_info.htm; 'Rdc-crawler at ml dot toshiba dot co dot jp')";
const char S8598[] = "Nutch-Crawler-Manu/Nutch-1.5.1";
const char S8599[] = "ExactSeek Crawler (nutch 1.4)/Nutch-1.4 (ExactSeek Crawler; http://www.exactseek.com)";
const char S8600[] = "InboundScore/Nutch-1.4 (http://InboundScore.com/)";
const char S8601[] = "KSCrawler/Nutch-1.5.1 (http://www.kindsight.net/en/kscrawler; crawler@kindsight.net)";
const char S8602[] = "My Nutch Spider/Nutch-1.6";
const char S8603[] = "TosCrawler/Nutch-1.4 (http://www.toshiba.co.jp/rdc/about/crawl_info_en.htm; 'Rdc-crawler at ml dot toshiba dot co dot jp')";
const char S8604[] = "ReventIt Spider/Nutch-1.6";
const char S8605[] = "WhelanLabs Search/Nutch-1.4 (WhelanLabs Web Crawler Agent; searchengine@whelanlabs.com; http://www.whelanlabs.com)";
const char S8606[] = "LWNutch/Nutch-1.4 (another scientific bot - we accept your robots.txt! )";
const char S8607[] = "ReventIt/Nutch-1.6";
const char S8608[] = "BigBozz Bot/Nutch-1.4";
const char S8609[] = "Mozilla/5.0 (compatible; BigBozzBot/1.1; +http://www.bigbozz.com/)/Nutch-1.4";
const char S8610[] = "OpenWebIndex/Nutch-1.6";
const char S8611[] = "NutchCVS/0.8-dev (Nutch running at UW; http://www.nutch.org/docs/en/bot.html; sycrawl@cs.washington.edu)";
const char S8612[] = "Nutch Spider/Nutch-1.4";
const char S8613[] = "Test Spider/Nutch-2.1";
const char S8614[] = "Nutch Spider/Nutch-1.6";
const char S8615[] = "TosCrawler/Nutch-1.6 (http://www.toshiba.co.jp/rdc/about/crawl_info_en.htm; 'Rdc-crawler at ml dot toshiba dot co dot jp')";
const char S8616[] = "INLNutchSpider/Nutch-2.1 (Knowledge Indexer; http://www.inl.gov/)";
const char S8617[] = "Affectv Robot v1.0/Nutch-1.6";
const char S8618[] = "spider/Nutch-2.1";
const char S8619[] = "MZone Spider/Nutch-1.6";
const char S8620[] = "Super Nutch Spider/Nutch-1.6";
const char S8621[] = "Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)/Nutch-2.1";
const char S8622[] = "Crawler/Nutch-1.1";
const char S8623[] = "NutchCrawler/Nutch-1.1";
const char S8624[] = "Your Nutch Spider/Nutch-2.1";
const char S8625[] = "spyder/Nutch-2.1 (just another internet crawler; http://www.paloaltonetworks.com/products/features/url-filtering.html; ghalevy@paloaltonetworks.com)";
const char S8626[] = "My Nutch Spider/Nutch-1.6-SNAPSHOT";
const char S8627[] = "Googlebot/Nutch-1.4 (Prototype; http://en.wikipedia.org/wiki/Web_crawler; donotreply at prototype dot com)";
const char S8628[] = "NutchCrawler/Nutch-1.6";
const char S8629[] = "my crawler/Nutch-1.6";
const char S8630[] = "Affectv Robot v1.0/Nutch-1.6 (http://www.affectv.co.uk; austin at affectv dot co dot uk)";
const char S8631[] = "KzjnetBot/Nutch-1.6";
const char S8632[] = "NutchCVS/0.7.1 (Nutch; http://lucene.apache.org/nutch/bot.html; nutch-agent@lucene.apache.org)";
const char S8633[] = "HPI-BI-Crawler/0.1(+http://www.hpi.uni-potsdam.de/meinel/forschung/web_30/blog_intelligence.html)/Nutch-2.0-dev";
const char S8634[] = "MyNutchSpider/Nutch-2.1";
const char S8635[] = "Dorkobot/Nutch-1.6";
const char S8636[] = "\"Mozilla/5.0\"/Nutch-2.1 (just another internet crawler; http://www.paloaltonetworks.com/products/features/url-filtering.html; ghalevy@paloaltonetworks.com)";
const char S8637[] = "betasearch/Nutch-2.1 (Academic Beta Search; http://www.aramabeta.com; info@aramabeta.com)";
const char S8638[] = "Ziba/Nutch-1.4 (Ziba Links Web Spider; http://www.zibalinks.com)";
const char S8639[] = "My Nutch Spider/Nutch-1.7-SNAPSHOT";
const char S8640[] = "MyNutchTest/Nutch-1.6";
const char S8641[] = "Global-Crawler/Nutch-1.4 (Search platform crawler)";
const char S8642[] = "LinexBot/Nutch-1.0 (www.linexsystems.com; support at linexsystems dot com)";
const char S8643[] = "Permaculture Super Coop Nutch Spider/Nutch-1.6";
const char S8644[] = "TsolCrawler/Nutch-1.4";
const char S8645[] = "ManziaBot/Nutch-2.1 (Web crawler from Manzia Corp; http://www.manzia.com/crawler; info at manzia dot com)";
const char S8646[] = "Nimit is a web indexing system designed to focus crawl the web for educational content. the crawler is built on top of open-source nutch and is intended to to pocketize the web into domains eg content for engineering, medicine,law, humanities and business/Nutch-1.6";
const char S8647[] = "Nimit is a web indexing system designed to focus crawl the web for educational content. the crawler is built on top of open-source nutch and is intended to to pocketize the web into domains eg content for engineering, medicine,law, humanities and business./Nutch-1.6";
const char S8648[] = "Acc Crawler/Nutch-1.7";
const char S8649[] = "NutchCrawler/Nutch-2.2.1";
const char S8650[] = "Opera/9.80 (Windows NT 6.1; U; Edition Next; en) Presto/2.9.186 Version/12.00";
const char S8651[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.289 Version/12.00";
const char S8652[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.289 Version/12.00,gzip(gfe) (via translate.google.com)";
const char S8653[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5328); ru) Presto/2.10.289 Version/12.00";
const char S8654[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.289 Version/12.00";
const char S8655[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.289 Version/12.00";
const char S8656[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5310); ru) Presto/2.10.289 Version/12.00";
const char S8657[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5667); ru) Presto/2.10.289 Version/12.00";
const char S8658[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4876); ru) Presto/2.10.289 Version/12.00";
const char S8659[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.289 Version/12.00";
const char S8660[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.289 Version/12.00";
const char S8661[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.289 Version/12.00";
const char S8662[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5660); ru) Presto/2.10.289 Version/12.00";
const char S8663[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.289 Version/12.00";
const char S8664[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.289 Version/12.00";
const char S8665[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4947); ru) Presto/2.10.289 Version/12.00";
const char S8666[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.289 Version/12.00";
const char S8667[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.289 Version/12.00";
const char S8668[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.289 Version/12.00";
const char S8669[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.289 Version/12.00";
const char S8670[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5231); ru) Presto/2.10.289 Version/12.00";
const char S8671[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.289 Version/12.00";
const char S8672[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5218); ru) Presto/2.10.289 Version/12.00";
const char S8673[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.289 Version/12.00";
const char S8674[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.289 Version/12.00";
const char S8675[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.289 Version/12.00";
const char S8676[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.289 Version/12.00";
const char S8677[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5650); ru) Presto/2.10.289 Version/12.00";
const char S8678[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.289 Version/12.00";
const char S8679[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.289 Version/12.00";
const char S8680[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.10.289 Version/12.00";
const char S8681[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.289 Version/12.00";
const char S8682[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.289 Version/12.00";
const char S8683[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.289 Version/12.00";
const char S8684[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5278); ru) Presto/2.10.289 Version/12.00";
const char S8685[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.10.289 Version/12.00";
const char S8686[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5676); ru) Presto/2.10.289 Version/12.00";
const char S8687[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.289 Version/12.00";
const char S8688[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; de; rv:2.0) Gecko/X Firefox/4.0 Opera 12.00";
const char S8689[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4930); ru) Presto/2.10.289 Version/12.00";
const char S8690[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.289 Version/12.00";
const char S8691[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5288); ru) Presto/2.10.289 Version/12.00";
const char S8692[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.289 Version/12.00";
const char S8693[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5299); ru) Presto/2.10.289 Version/12.00";
const char S8694[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5230); ru) Presto/2.10.289 Version/12.00";
const char S8695[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5336); ru) Presto/2.10.289 Version/12.00";
const char S8696[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.289 Version/12.00";
const char S8697[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.10.289 Version/12.00";
const char S8698[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; ru) Opera 12.00";
const char S8699[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4110); ru) Presto/2.10.289 Version/12.00";
const char S8700[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.289 Version/12.00";
const char S8701[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; ru) Presto/2.10.289 Version/12.01";
const char S8702[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.289 Version/12.00";
const char S8703[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5300); ru) Presto/2.10.289 Version/12.00";
const char S8704[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5217); ru) Presto/2.10.289 Version/12.00";
const char S8705[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.289 Version/12.00";
const char S8706[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5288); ru) Presto/2.10.289 Version/12.01";
const char S8707[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; ru) Presto/2.10.289 Version/12.01,gzip(gfe) (via translate.google.com)";
const char S8708[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5831); ru) Presto/2.10.289 Version/12.01";
const char S8709[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5667); ru) Presto/2.10.289 Version/12.01";
const char S8710[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5704); ru) Presto/2.10.289 Version/12.01";
const char S8711[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4664); ru) Presto/2.10.289 Version/12.01";
const char S8712[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.289 Version/12.01";
const char S8713[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5328); ru) Presto/2.10.289 Version/12.01";
const char S8714[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.289 Version/12.01";
const char S8715[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.289 Version/12.01";
const char S8716[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.7 (build 03797); ru) Presto/2.10.289 Version/12.01";
const char S8717[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5680); ru) Presto/2.10.289 Version/12.01";
const char S8718[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.289 Version/12.01";
const char S8719[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.289 Version/12.01";
const char S8720[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5966); ru) Presto/2.10.289 Version/12.01";
const char S8721[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.289 Version/12.01";
const char S8722[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5660); ru) Presto/2.10.289 Version/12.01";
const char S8723[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.289 Version/12.01";
const char S8724[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.289 Version/12.01";
const char S8725[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.289 Version/12.01";
const char S8726[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5265); ru) Presto/2.10.289 Version/12.01";
const char S8727[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5676); ru) Presto/2.10.289 Version/12.01";
const char S8728[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.289 Version/12.01";
const char S8729[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5218); ru) Presto/2.10.289 Version/12.01";
const char S8730[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.289 Version/12.01";
const char S8731[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.289 Version/12.01";
const char S8732[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.289 Version/12.01";
const char S8733[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.289 Version/12.01";
const char S8734[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.289 Version/12.01";
const char S8735[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.289 Version/12.01";
const char S8736[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4156); ru) Presto/2.10.289 Version/12.01";
const char S8737[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5754); ru) Presto/2.10.289 Version/12.01";
const char S8738[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.289 Version/12.01";
const char S8739[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.289 Version/12.01";
const char S8740[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.289 Version/12.01";
const char S8741[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4157); ru) Presto/2.10.289 Version/12.01";
const char S8742[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.289 Version/12.01";
const char S8743[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.289 Version/12.01";
const char S8744[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.289 Version/12.01";
const char S8745[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5970); ru) Presto/2.10.289 Version/12.01";
const char S8746[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.289 Version/12.00";
const char S8747[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.289 Version/12.00";
const char S8748[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.289 Version/12.00";
const char S8749[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4832); ru) Presto/2.10.289 Version/12.00";
const char S8750[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.289 Version/12.01";
const char S8751[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.289 Version/12.01";
const char S8752[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4930); ru) Presto/2.10.289 Version/12.01";
const char S8753[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5230); ru) Presto/2.10.289 Version/12.01";
const char S8754[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.7 (build 03773); ru) Presto/2.10.289 Version/12.01";
const char S8755[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03649); ru) Presto/2.10.289 Version/12.01";
const char S8756[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5650); ru) Presto/2.10.289 Version/12.01";
const char S8757[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.289 Version/12.01";
const char S8758[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5970); ru) Presto/2.10.289 Version/12.00";
const char S8759[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5966); ru) Presto/2.10.289 Version/12.00";
const char S8760[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5976); ru) Presto/2.10.289 Version/12.00";
const char S8761[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5976); ru) Presto/2.10.289 Version/12.01";
const char S8762[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03639); ru) Presto/2.10.289 Version/12.00";
const char S8763[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.289 Version/12.01,gzip(gfe)";
const char S8764[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.289 Version/12.01";
const char S8765[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4661); ru) Presto/2.10.289 Version/12.01";
const char S8766[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03402); ru) Presto/2.10.289 Version/12.00";
const char S8767[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5161); ru) Presto/2.10.289 Version/12.00";
const char S8768[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.6 (build 03278); ru) Presto/2.10.289 Version/12.00";
const char S8769[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03780); ru) Presto/2.10.289 Version/12.00";
const char S8770[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5299); ru) Presto/2.10.289 Version/12.01";
const char S8771[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5336); ru) Presto/2.10.289 Version/12.01";
const char S8772[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5325); ru) Presto/2.10.289 Version/12.01";
const char S8773[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.289 Version/12.01";
const char S8774[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.289 Version/12.01";
const char S8775[] = "Opera/12.0 (Windows NT 6.1; U; en) Presto/2.9.181 Version/12.00";
const char S8776[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.6 (build 03278); ru) Presto/2.10.289 Version/12.01";
const char S8777[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.289 Version/12.00,gzip(gfe)";
const char S8778[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.289 Version/12.00";
const char S8779[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 6001); ru) Presto/2.10.289 Version/12.00";
const char S8780[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5278); ru) Presto/2.10.289 Version/12.01";
const char S8781[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5997); ru) Presto/2.10.289 Version/12.01";
const char S8782[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5784); ru) Presto/2.10.289 Version/12.00";
const char S8783[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.289 Version/12.01";
const char S8784[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.289 Version/12.00";
const char S8785[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 6005); ru) Presto/2.10.289 Version/12.01";
const char S8786[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4110); ru) Presto/2.10.289 Version/12.01";
const char S8787[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5745); ru) Presto/2.10.289 Version/12.00";
const char S8788[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.289 Version/12.01";
const char S8789[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5784); ru) Presto/2.10.289 Version/12.01";
const char S8790[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.289 Version/12.00";
const char S8791[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 6015); ru) Presto/2.10.289 Version/12.01";
const char S8792[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5993); ru) Presto/2.10.289 Version/12.01";
const char S8793[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5880); ru) Presto/2.10.289 Version/12.00";
const char S8794[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5745); ru) Presto/2.10.289 Version/12.01";
const char S8795[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5997); ru) Presto/2.10.289 Version/12.00";
const char S8796[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5981); ru) Presto/2.10.289 Version/12.01";
const char S8797[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5993); ru) Presto/2.10.289 Version/12.00";
const char S8798[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5880); ru) Presto/2.10.289 Version/12.01";
const char S8799[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.289 Version/12.01";
const char S8800[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5325); ru) Presto/2.10.289 Version/12.00";
const char S8801[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.289 Version/12.00";
const char S8802[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.289 Version/12.01";
const char S8803[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5977); ru) Presto/2.10.289 Version/12.01";
const char S8804[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 6037); ru) Presto/2.10.289 Version/12.00";
const char S8805[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.289 Version/12.00";
const char S8806[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 6007); ru) Presto/2.10.289 Version/12.01";
const char S8807[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.289 Version/12.01";
const char S8808[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6081); ru) Presto/2.10.289 Version/12.01";
const char S8809[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 6003); ru) Presto/2.10.289 Version/12.01";
const char S8810[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6081); ru) Presto/2.10.289 Version/12.00";
const char S8811[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6089); ru) Presto/2.10.289 Version/12.00";
const char S8812[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.289 Version/12.01";
const char S8813[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5217); ru) Presto/2.10.289 Version/12.01";
const char S8814[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6011); ru) Presto/2.10.289 Version/12.01";
const char S8815[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6089); ru) Presto/2.10.289 Version/12.01";
const char S8816[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4661); ru) Presto/2.10.289 Version/12.00";
const char S8817[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.7 (build 03639); ru) Presto/2.10.289 Version/12.01";
const char S8818[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5300); ru) Presto/2.10.289 Version/12.01";
const char S8819[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5161); ru) Presto/2.10.289 Version/12.01";
const char S8820[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5981); ru) Presto/2.10.289 Version/12.00";
const char S8821[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5989); ru) Presto/2.10.289 Version/12.01";
const char S8822[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6048); ru) Presto/2.10.289 Version/12.01";
const char S8823[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 6054); ru) Presto/2.10.289 Version/12.00";
const char S8824[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 6008); ru) Presto/2.10.289 Version/12.00";
const char S8825[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 6008); ru) Presto/2.10.289 Version/12.01";
const char S8826[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6083); ru) Presto/2.10.289 Version/12.00";
const char S8827[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.289 Version/12.00";
const char S8828[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.289 Version/12.01";
const char S8829[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 6003); ru) Presto/2.10.289 Version/12.00";
const char S8830[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 6083); ru) Presto/2.10.289 Version/12.01";
const char S8831[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 6007); ru) Presto/2.10.289 Version/12.00";
const char S8832[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6011); ru) Presto/2.10.289 Version/12.00";
const char S8833[] = "Opera/9.80 (Android; Opera Mini/7.0.29952/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S8834[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/18.6.872.0 Safari/535.8";
const char S8835[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19,gzip(gfe) (via docs.google.com/viewer)";
const char S8836[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19,gzip(gfe) (via docs.google.com/viewer)";
const char S8837[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/1854; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8838[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19,gzip(gfe) (via translate.google.com)";
const char S8839[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19,gzip(gfe) (via translate.google.com)";
const char S8840[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/534.7 ZemanaAID/FFFF0061";
const char S8841[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19";
const char S8842[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/18.6.872.0 Safari/535.2  3gpp-gba";
const char S8843[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/180.1.1025.40 Safari/535.19";
const char S8844[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/1895; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8845[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/1911; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8846[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Client/1911; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8847[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Client/0; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8848[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.1634 Safari/535.19 YE";
const char S8849[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19 AppEngine-Google; (+http://code.google.com/appengine; appid: s~cxX)";
const char S8850[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Client/1928; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8851[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/1950; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8852[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/1949; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8853[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/1957; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8854[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/1960; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8855[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19 AppEngine-Google; (+http://code.google.com/appengine; appid: s~eachshadow)";
const char S8856[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2008; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8857[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Client/1960; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8858[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/1999; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8859[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2001; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8860[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2004; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8861[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2005; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8862[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1;) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.45 Safari/535.19";
const char S8863[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; de) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/18.0 Safari/525.13";
const char S8864[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19 AppEngine-Google; (+http://code.google.com/appengine; appid: s~momomich)";
const char S8865[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2026; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8866[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2025; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8867[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2009; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8868[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2037; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8869[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2035; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8870[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2038; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8871[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2040; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8872[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Client/1950; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8873[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Client/2026; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8874[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2046; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8875[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2048; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8876[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2057; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8877[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2082; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8878[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2094; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8879[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2092; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8880[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2093; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8881[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Tenfoot/2095; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8882[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2095; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8883[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2100; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8884[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2102; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8885[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/18.0 BIDUPlayerBrowser/2.x Safari/534.7";
const char S8886[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2109; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8887[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Tenfoot/2109; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8888[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Client/2109; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8889[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Tenfoot/2102; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8890[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2106; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8891[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19,gzip(gfe)";
const char S8892[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2119; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8893[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2122; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8894[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2125; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8895[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Tenfoot/2125; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8896[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Client/2125; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8897[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Tenfoot/X; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8898[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19,gzip(gfe)";
const char S8899[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Client/X; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8900[] = "Mozilla/5.0 (Windows NT 6.1; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/18.0 BIDUPlayerBrowser/2.x Safari/534.7";
const char S8901[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19; Java/1.7.0_03";
const char S8902[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/180.1.1025.40 Safari/535.19";
const char S8903[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/X; ) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Safari/535.19";
const char S8904[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; DOTA 2 Client/0; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8905[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Half-Life/0; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S8906[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S8907[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S8908[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)";
const char S8909[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via docs.google.com/viewer)";
const char S8910[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via docs.google.com/viewer)";
const char S8911[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)";
const char S8912[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1 AppEngine-Google; (+http://code.google.com/appengine; appid: s~yunantongxue)";
const char S8913[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe)";
const char S8914[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe)";
const char S8915[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1 AppEngine-Google; (+http://code.google.com/appengine; appid: s~mintchipchoco)";
const char S8916[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1";
const char S8917[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1 AppEngine-Google; (+http://code.google.com/appengine; appid: s~text2solve)";
const char S8918[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S8919[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US)  AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1";
const char S8920[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.12 Safari/535.11";
const char S8921[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S8922[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11,gzip(gfe) (via docs.google.com/viewer)";
const char S8923[] = "User-Agent:Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S8924[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/534.7 ZemanaAID/FFFF0061";
const char S8925[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11,gzip(gfe) (via docs.google.com/viewer)";
const char S8926[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.84 Safari/535.11 SE 2.X MetaSr 1.0,gzip(gfe) (via translate.google.com)";
const char S8927[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.84 Safari/535.11 SE 2.X MetaSr 1.0,gzip(gfe) (via translate.google.com)";
const char S8928[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.84 Safari/535.11 SE 2.X MetaSr 1.0,gzip(gfe)";
const char S8929[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.84 Safari/535.11 SE 2.X MetaSr 1.0,gzip(gfe)";
const char S8930[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.33 Safari/535.11 AppEngine-Google; (+http://code.google.com/appengine; appid: optimizelyedit)";
const char S8931[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML	 like Gecko) Chrome/17.0.963.38 Safari/535.11";
const char S8932[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/17.0.944.0 Safari/535.8 BitdefenderSafepay/0.2.0.0 (OBK; 17.0.944.0; r396.2011)";
const char S8933[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/17.0.944.0 Safari/535.8 BitdefenderSafepay/0.2.0.0 (OBK; 17.0.944.0; r396.2011)";
const char S8934[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML like Gecko) Chrome/17.0.963.46 Safari/535.11";
const char S8935[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.1.963.51 Safari/535.11 MRCHROME SOC";
const char S8936[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.1.963.51 Safari/535.11 MRCHROME SOC";
const char S8937[] = "Opera/10.61 (J2ME/MIDP; Opera Mini/5.1.21219/19.999; en-US; rv:1.9.3a5) WebKit/534.5 Presto/2.6.30";
const char S8938[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/5.18487/26.1670; U; es) Presto/2.8.119 Version/10.54";
const char S8939[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/5.1.21219/19.999; en-US; rv:1.9.3a5) WebKit/534.5 Presto/2.6.30";
const char S8940[] = "Opera/9.50 (J2ME/MIDP; Opera Mini/5.1.2169/28.3165; U; en)";
const char S8941[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/5.1.24009/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S8942[] = "Opera/9.80 (MTK; Opera Mini/5.1.3119/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S8943[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36";
const char S8944[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.1.0.0 Safari/537.36";
const char S8945[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.1.0.0 Safari/537.36";
const char S8946[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S8947[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S8948[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe)";
const char S8949[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S8950[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S8951[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S8952[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.2.0.0 Safari/537.36";
const char S8953[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36";
const char S8954[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.1.0.0 Safari/537.36";
const char S8955[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36";
const char S8956[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S8957[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.2.0.0 Safari/537.36";
const char S8958[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36 AppEngine-Google; (+http://code.google.com/appengine; appid: s~nikajiyang)";
const char S8959[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.11.1453.94 Safari/537.36 QIPSurf";
const char S8960[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190";
const char S8961[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4";
const char S8962[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S8963[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F191";
const char S8964[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 Safari/533.17.9";
const char S8965[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7";
const char S8966[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2";
const char S8967[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S8968[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/533.17.9";
const char S8969[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S8970[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3";
const char S8971[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S8972[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1";
const char S8973[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S8974[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2";
const char S8975[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S8976[] = "Mozilla/5.0(iPhone;U;CPUiPhoneOS4_3_5likeMacOSX;ru-ru)AppleWebKit/533.17.9(KHTML,likeGecko)Mobile/8L1";
const char S8977[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en_us) AppleWebKit/525.18.1 (KHTML, like Gecko)";
const char S8978[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en_us) AppleWebKit/525.18.1 (KHTML, like Gecko)";
const char S8979[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en_us) AppleWebKit/525.18.1 (KHTML, like Gecko)";
const char S8980[] = "Mozilla/5.0 (iPhone Simulator; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F192";
const char S8981[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H8";
const char S8982[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190 Safari/533.17.9";
const char S8983[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 Safari/533.17.9";
const char S8984[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/9Z999";
const char S8985[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2 Safari/533.17.9";
const char S8986[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/533.17.9";
const char S8987[] = "Miu Ptt 1.5.2 (iPhone; iPhone OS 4.3.5; zh_TW)";
const char S8988[] = "Miu Ptt 1.5.3 (iPhone; iPhone OS 4.3.3; zh_TW)";
const char S8989[] = "NaverCafe/1.0 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName";
const char S8990[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 Safari/533.17.9";
const char S8991[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.3.4;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S8992[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.3;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S8993[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone2,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.3.1;FBSS/1; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/1.0]";
const char S8994[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/9Z999";
const char S8995[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/9Z999";
const char S8996[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; zh-tw) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2";
const char S8997[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.3.2;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S8998[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.3.3;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S8999[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone2,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.3.5;FBSS/1; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/1.0]";
const char S9000[] = "NaverCafe/1.0 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName";
const char S9001[] = "Mozilla/5.0 (device; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2";
const char S9002[] = "Mozilla/5.0 (; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2";
const char S9003[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper";
const char S9004[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 iPad2,1 BingWeb/3.03.1428.X";
const char S9005[] = "Mozilla/5.0 (iPhone; U; CPU iPhone 4_3_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPhone3,1/4.3.3)";
const char S9006[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 iPad1,1 BingWeb/3.03.1428.X";
const char S9007[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8K2";
const char S9008[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; vi-vn) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8F190";
const char S9009[] = "Mozilla/5.0 (iPhone; U; CPU iPhone 4_3_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPad2,2/4.3.3)";
const char S9010[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; es-es) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148";
const char S9011[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8J2";
const char S9012[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko)";
const char S9013[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S9014[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S9015[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S9016[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S9017[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S9018[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36,gzip(gfe)";
const char S9019[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S9020[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36,gzip(gfe)";
const char S9021[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36";
const char S9022[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.36 (KHTML, like Gecko) Iron/29.0.1600.1 Chrome/29.0.1600.1 Safari/537.36";
const char S9023[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36,gzip(gfe)";
const char S9024[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S9025[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; en) Presto/2.10.289 Version/12.02";
const char S9026[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5328); ru) Presto/2.10.289 Version/12.02";
const char S9027[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; en; rv:2.0) Gecko/X Firefox/4.0 Opera 12.02";
const char S9028[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.289 Version/12.02";
const char S9029[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.289 Version/12.02";
const char S9030[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4876); ru) Presto/2.10.289 Version/12.02";
const char S9031[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.7 (build 03797); ru) Presto/2.10.289 Version/12.02";
const char S9032[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.289 Version/12.02";
const char S9033[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.289 Version/12.02";
const char S9034[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5972); ru) Presto/2.10.289 Version/12.02";
const char S9035[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.289 Version/12.02,gzip(gfe) (via translate.google.com)";
const char S9036[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.289 Version/12.02";
const char S9037[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.289 Version/12.02";
const char S9038[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.289 Version/12.02";
const char S9039[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5711); ru) Presto/2.10.289 Version/12.02";
const char S9040[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.289 Version/12.02";
const char S9041[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5970); ru) Presto/2.10.289 Version/12.02";
const char S9042[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5680); ru) Presto/2.10.289 Version/12.02";
const char S9043[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5310); ru) Presto/2.10.289 Version/12.02";
const char S9044[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.10.289 Version/12.02";
const char S9045[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.289 Version/12.02";
const char S9046[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5831); ru) Presto/2.10.289 Version/12.02";
const char S9047[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5676); ru) Presto/2.10.289 Version/12.02";
const char S9048[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.289 Version/12.02";
const char S9049[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5218); ru) Presto/2.10.289 Version/12.02";
const char S9050[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.289 Version/12.02";
const char S9051[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.289 Version/12.02";
const char S9052[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; en) Opera 12.02";
const char S9053[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5308); ru) Presto/2.10.289 Version/12.02";
const char S9054[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.289 Version/12.02";
const char S9055[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.289 Version/12.02";
const char S9056[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5966); ru) Presto/2.10.289 Version/12.02";
const char S9057[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.289 Version/12.02";
const char S9058[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4933); ru) Presto/2.10.289 Version/12.02";
const char S9059[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.289 Version/12.02";
const char S9060[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5230); ru) Presto/2.10.289 Version/12.02";
const char S9061[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5300); ru) Presto/2.10.289 Version/12.02";
const char S9062[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5282); ru) Presto/2.10.289 Version/12.02";
const char S9063[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.289 Version/12.02";
const char S9064[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.289 Version/12.02";
const char S9065[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.289 Version/12.02";
const char S9066[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5667); ru) Presto/2.10.289 Version/12.02";
const char S9067[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4930); ru) Presto/2.10.289 Version/12.02";
const char S9068[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4848); ru) Presto/2.10.289 Version/12.02";
const char S9069[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5278); ru) Presto/2.10.289 Version/12.02";
const char S9070[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5660); ru) Presto/2.10.289 Version/12.02";
const char S9071[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5299); ru) Presto/2.10.289 Version/12.02";
const char S9072[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.289 Version/12.02";
const char S9073[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5650); ru) Presto/2.10.289 Version/12.02";
const char S9074[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.289 Version/12.02";
const char S9075[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.289 Version/12.02";
const char S9076[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.289 Version/12.02";
const char S9077[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03649); ru) Presto/2.10.289 Version/12.02";
const char S9078[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5288); ru) Presto/2.10.289 Version/12.02";
const char S9079[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.7 (build 03653); ru) Presto/2.10.289 Version/12.02";
const char S9080[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5976); ru) Presto/2.10.289 Version/12.02";
const char S9081[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.289 Version/12.02";
const char S9082[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4110); ru) Presto/2.10.289 Version/12.02";
const char S9083[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.7 (build 03639); ru) Presto/2.10.289 Version/12.02";
const char S9084[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5217); ru) Presto/2.10.289 Version/12.02";
const char S9085[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03278); ru) Presto/2.10.289 Version/12.02";
const char S9086[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.289 Version/12.02";
const char S9087[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4832); ru) Presto/2.10.289 Version/12.02";
const char S9088[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.5 (build 02842); ru) Presto/2.10.289 Version/12.02";
const char S9089[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.6 (build 03402); ru) Presto/2.10.289 Version/12.02";
const char S9090[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5734); ru) Presto/2.10.289 Version/12.02";
const char S9091[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.289 Version/12.02,gzip(gfe)";
const char S9092[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5745); ru) Presto/2.10.289 Version/12.02";
const char S9093[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03672); ru) Presto/2.10.289 Version/12.02";
const char S9094[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5325); ru) Presto/2.10.289 Version/12.02";
const char S9095[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5336); ru) Presto/2.10.289 Version/12.02";
const char S9096[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5993); ru) Presto/2.10.289 Version/12.02";
const char S9097[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.289 Version/12.02";
const char S9098[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5997); ru) Presto/2.10.289 Version/12.02";
const char S9099[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5784); ru) Presto/2.10.289 Version/12.02";
const char S9100[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5998); ru) Presto/2.10.289 Version/12.02";
const char S9101[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4661); ru) Presto/2.10.289 Version/12.02";
const char S9102[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03755); ru) Presto/2.10.289 Version/12.02";
const char S9103[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.289 Version/12.02";
const char S9104[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; de; rv:2.0) Gecko/X Firefox/4.0 Opera 12.02";
const char S9105[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5880); ru) Presto/2.10.289 Version/12.02";
const char S9106[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5861); ru) Presto/2.10.289 Version/12.02";
const char S9107[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6011); ru) Presto/2.10.289 Version/12.02";
const char S9108[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.289 Version/12.02";
const char S9109[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5977); ru) Presto/2.10.289 Version/12.02";
const char S9110[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.289 Version/12.02";
const char S9111[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5161); ru) Presto/2.10.289 Version/12.02";
const char S9112[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5730); ru) Presto/2.10.289 Version/12.02";
const char S9113[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5981); ru) Presto/2.10.289 Version/12.02";
const char S9114[] = "Opera/12.80 (Windows NT 6.1; U; en) Presto/22.12.289 Version/12.02";
const char S9115[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5309); ru) Presto/2.10.289 Version/12.02";
const char S9116[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6037); ru) Presto/2.10.289 Version/12.02";
const char S9117[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.289 Version/12.02";
const char S9118[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.289 Version/12.02";
const char S9119[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5989); ru) Presto/2.10.289 Version/12.02";
const char S9120[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 6007); ru) Presto/2.10.289 Version/12.02";
const char S9121[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 6054); ru) Presto/2.10.289 Version/12.02";
const char S9122[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 6003); ru) Presto/2.10.289 Version/12.02";
const char S9123[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6048); ru) Presto/2.10.289 Version/12.02";
const char S9124[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6081); ru) Presto/2.10.289 Version/12.02";
const char S9125[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6083); ru) Presto/2.10.289 Version/12.02";
const char S9126[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6042); ru) Presto/2.10.289 Version/12.02";
const char S9127[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 6008); ru) Presto/2.10.289 Version/12.02";
const char S9128[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6089); ru) Presto/2.10.289 Version/12.02";
const char S9129[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5999); ru) Presto/2.10.289 Version/12.02";
const char S9130[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03780); ru) Presto/2.10.289 Version/12.02";
const char S9131[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6080); ru) Presto/2.10.289 Version/12.02";
const char S9132[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.1 (build 6578); ru) Presto/2.10.289 Version/12.02";
const char S9133[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03638); ru) Presto/2.10.289 Version/12.02";
const char S9134[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5990); ru) Presto/2.10.289 Version/12.02";
const char S9135[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 6044); ru) Presto/2.10.289 Version/12.02";
const char S9136[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6046); ru) Presto/2.10.289 Version/12.02";
const char S9137[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 6017); ru) Presto/2.10.289 Version/12.02";
const char S9138[] = "Opera/9.80 (Windows NT 6.1; U; en) Presto/2.10.289 Version/12.02,gzip(gfe) (via docs.google.com/viewer)";
const char S9139[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 6019); ru) Presto/2.10.289 Version/12.02";
const char S9140[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.1 (build 6283); ru) Presto/2.10.289 Version/12.02";
const char S9141[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.1 (build 6638); ru) Presto/2.10.289 Version/12.02";
const char S9142[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6639); ru) Presto/2.10.289 Version/12.02";
const char S9143[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.1 (build 6341); ru) Presto/2.10.289 Version/12.02";
const char S9144[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6683); ru) Presto/2.10.289 Version/12.02";
const char S9145[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6673); ru) Presto/2.10.289 Version/12.02";
const char S9146[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/23.0.1271.17) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.17 Safari/537.11";
const char S9147[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/23.0.1271.10) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.10 Safari/537.11";
const char S9148[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/23.0.1271.26) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.26 Safari/537.11";
const char S9149[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/23.0.1271.40) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.40 Safari/537.11";
const char S9150[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/23.0.1271.26) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.26 Safari/537.11";
const char S9151[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/23.0.1271.60) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.60 Safari/537.11";
const char S9152[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/23.0.1271.52) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.52 Safari/537.11";
const char S9153[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/23.0.1271.64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S9154[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/23.0.1271.64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S9155[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; GTB7.4; chromeframe/23.0.1271.64; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)";
const char S9156[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/23.0.1271.91) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.91 Safari/537.11";
const char S9157[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/23.0.1271.91) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.91 Safari/537.11";
const char S9158[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/23.0.1271.91; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)";
const char S9159[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/23.0.1271.95) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11";
const char S9160[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/23.0.1271.95) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11";
const char S9161[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/23.0.1271.97)";
const char S9162[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/23.0.1271.97)";
const char S9163[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/23.0.1271.97; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Tablet PC 2.0)";
const char S9164[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/23.0.1271.97)";
const char S9165[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/23.0.1271.97) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S9166[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/23.0.1271.97) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S9167[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/23.0.1271.97; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S9168[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; chromeframe/23.0.1271.97)";
const char S9169[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/23.0.1271.97; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S9170[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; chromeframe/23.0.1271.97)";
const char S9171[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/23.0.1271.97; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S9172[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; chromeframe/23.0.1271.97)";
const char S9173[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Avant Browser; chromeframe/23.0.1271.97)";
const char S9174[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/23.0.1271.97; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S9175[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/23.0.1271.97; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S9176[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/23.0.1271.97)";
const char S9177[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/4.0; GTB7.4; chromeframe/23.0.1271.97; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)";
const char S9178[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Avant Browser; chromeframe/23.0.1271.97)";
const char S9179[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; .NET4.0C; chromeframe/23.0.1271.97)";
const char S9180[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/23.0.1271.91; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S9181[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/23.0.1271.97; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Zune 4.7)";
const char S9182[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/23.0.1271.91; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6136; ms-office; MSOffice 14)";
const char S9183[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.1) Gecko/2008070208 Firefox/3.0.1";
const char S9184[] = "Mozilla/5.01 (Windows; U; Windows NT 5.1; ru; rv:1.9.0.1) Gecko/X Firefox/3.0.4";
const char S9185[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9) Gecko/X IceWeasel/3.0";
const char S9186[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; de; rv:1.9.0.1) Gecko/X Firefox/3.0.0	Firefox 3.0";
const char S9187[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 (.NET CLR 3.5.30729) WinNT-PAI 31.08.2009";
const char S9188[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2) Gecko/X Firefox/3.0.1";
const char S9189[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 (.NET CLR 3.5.30729) WinNT-A8I 02.01.2011";
const char S9190[] = "Mozilla/6.0 (Windows; U; Windows NT 5.1; en-US; rv:2.0.0.0) Gecko/X Firefox/3.0";
const char S9191[] = "mozilla/5.0 (windows nt 5.2; u; ru; rv:1.9.1.6) gecko/X firefox/3.5.6 opera 10.70";
const char S9192[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9a1) Gecko/X GranParadiso/3.0a1";
const char S9193[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.13) Gecko/X firefox/3.6.18";
const char S9194[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 (.NET CLR 3.5.30729) WinNT-EVI 26.06.2010";
const char S9195[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.12) Gecko/X Firefox/3";
const char S9196[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9) Gecko/X IceWeasel/3.0 ( .NET CLR 3.5.30729)";
const char S9197[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9) Gecko/X IceWeasel/3.0 (.NET CLR 3.5.30729)";
const char S9198[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 (.NET CLR 3.5.30729) WinNT-PAI 30.08.2009";
const char S9199[] = "Mozilla/4.0 (Windows; U; Windows NT 5.1; lt; rv:1.9.0.14) Gecko/X Firefox/3.0.14 (.NET CLR 3.5.30729)";
const char S9200[] = "Windows; U; Windows NT 5.1; lt; rv:1.9.0.14) Gecko/X Firefox/3.0.14 (.NET CLR 3.5.30729)";
const char S9201[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1 en-US; rv: 1.9.0.10)Gecko/X FireFox/3.0.16";
const char S9202[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; EN; rv:1.9.0.6) Gecko/X Firefox/3.0.6";
const char S9203[] = "Mozilla/5.0 (Windows; U; Windows XP 5.2; en-US; rv:1.9) Gecko/Firefox/3.0";
const char S9204[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; de; rv:1.9.0.8; Gecko/X Firefox/3.07)";
const char S9205[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.14) Gecko/X Firefox/3.0.14 (.NET CLR 3.5.30729) WinNT-PAU 15.07.2009 FBSMTWB";
const char S9206[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9) Gecko/X IceWeasel/3.0 ( .NET CLR 3.5.30729; .NET4.0E)";
const char S9207[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36";
const char S9208[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36";
const char S9209[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.1.0.0 Safari/537.36";
const char S9210[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.1.0.0 Safari/537.36";
const char S9211[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S9212[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S9213[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe)";
const char S9214[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S9215[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.2.0.0 Safari/537.36";
const char S9216[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.2.0.0 Safari/537.36";
const char S9217[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe)";
const char S9218[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S9219[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9220[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9221[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9222[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-br) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9223[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9224[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; it-it) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9225[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sv-se) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9226[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pl-pl) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9227[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; da-dk) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9228[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9229[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9230[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fi-fi) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9231[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nb-no) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9232[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-cn) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9233[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ru-ru) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9234[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-tw) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9235[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9236[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9237[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9238[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9239[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9240[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sv-se) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9241[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ja-jp) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9242[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; it-it) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9243[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pl-pl) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9244[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; da-dk) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9245[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-br) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9246[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fi-fi) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9247[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ru-ru) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9248[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-cn) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9249[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9250[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9251[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9252[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-tw) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9253[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ru-ru) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9254[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-gb) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9255[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; it-it) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9256[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nb-no) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9257[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; da-dk) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9258[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9259[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ja-jp) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9260[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nb-no) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9261[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; el-gr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9262[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-au) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9263[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pl-pl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9264[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tr-tr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9265[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9266[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-cn) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9267[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-br) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9268[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fi-fi) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9269[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sv-se) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9270[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ca) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9271[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ar) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9272[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-xl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9273[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; af-za) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9274[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9275[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9276[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ja-jp) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9277[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9278[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9279[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9280[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ru-ru) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9281[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-tw) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9282[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; da-dk) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9283[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-gb) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9284[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; it-it) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9285[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nb-no) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9286[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; he-il) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9287[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-cn) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9288[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko, Safari/5533.19.4) ADM/722";
const char S9289[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pl-pl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9290[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-ca) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9291[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-au) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9292[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sv-se) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9293[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ko-kr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9294[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; is-is) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9295[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; uk-ua) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9296[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fi-fi) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9297[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-br) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9298[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tr-tr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9299[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-pt) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9300[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; el-gr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9301[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; cs-cz) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9302[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-ch) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9303[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; af-za) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9304[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sk-sk) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9305[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9306[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-xl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9307[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-pt) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9308[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9309[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sv-se) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9310[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; cs-cz) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9311[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9312[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9313[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ja-jp) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9314[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fi-fi) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9315[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9316[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-cn) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9317[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-tw) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9318[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-pt) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9319[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; is-is) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9320[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lt-lt) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9321[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-gb) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9322[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9323[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9324[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nb-no) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9325[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-ca) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9326[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pl-pl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9327[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-au) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9328[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ko-kr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9329[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9330[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-xl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9331[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-gb) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9332[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; th-th) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9333[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ca) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9334[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9335[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9336[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-au) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9337[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9338[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; is-is) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9339[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-tw) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9340[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-ch) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9341[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ru-ru) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9342[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-br) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9343[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9344[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sk-sk) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9345[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko, Safari/5533.21.1) ADM/722";
const char S9346[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; cs-cz) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9347[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; it-it) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9348[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ru-ru) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9349[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; da-dk) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9350[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-ca) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9351[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-br) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9352[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; da-dk) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9353[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fi-fi) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9354[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lv-lv) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9355[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sv-se) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9356[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nb-no) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9357[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ro-ro) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9358[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pl-pl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9359[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tr-tr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9360[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; el-gr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9361[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; vi-vn) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9362[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ar) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9363[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; el-gr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9364[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-xl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9365[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tr-tr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9366[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9367[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hu-hu) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9368[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; he-il) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9369[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; yi) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9370[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) WebClip/5534.49 Safari/5533.22.3";
const char S9371[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9372[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/5.0.6 Safari/533.22.3";
const char S9373[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9374[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9375[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9376[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9377[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9378[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ja-jp) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9379[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-ca) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9380[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-pt) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9381[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; el-gr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9382[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; th-th) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9383[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tr-tr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9384[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-tw) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9385[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tr-tr) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9386[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-gb) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9387[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-ca) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9388[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-pt) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9389[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; el-gr) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9390[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; cs-cz) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9391[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) WebClip/5527.2 Safari/5533.18.5";
const char S9392[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ko-kr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9393[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-ca) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9394[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-pt) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9395[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; th-th) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9396[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; he-il) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9397[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.17.8 (KHTML, like Gecko, Safari/5533.17.8) ADM/722";
const char S9398[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ca) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9399[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sk-sk) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9400[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hr-hr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9401[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ca-es) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9402[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; vi-vn) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9403[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; th-th) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9404[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-at) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9405[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fa-ir) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9406[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sl-si) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9407[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lt-lt) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9408[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ar) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9409[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; bg-bg) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9410[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9411[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ca) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9412[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko, Safari/5533.20.27) ADM/722";
const char S9413[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; he-il) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9414[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lt-lt) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9415[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; th-th) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9416[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fa-ir) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9417[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_8; ja-jp) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9418[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ko-kr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9419[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; et-ee) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9420[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; af-za) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9421[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_8; zh-cn) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9422[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ar) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9423[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; af-za) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9424[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ch) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9425[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ca-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9426[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hy-am) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9427[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; bg-bg) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9428[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sl-si) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9429[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fa-ir) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9430[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hr-hr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9431[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; uk-ua) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9432[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-at) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9433[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lv-lv) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9434[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; et-ee) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9435[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lt-lt) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9436[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; uk-ua) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9437[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-xl) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9438[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; he-il) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9439[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ko-kr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9440[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-au) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9441[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9442[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; is-is) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9443[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ro-ro) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9444[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lv-lv) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9445[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; th-th) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9446[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; is-is) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9447[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; cs-cz) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9448[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.16 (KHTML, like Gecko) WebClip/5527.2 Safari/5533.16";
const char S9449[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-xl) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9450[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9451[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sa) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9452[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fa-ir) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9453[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ga-ie) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9454[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hu-hu) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9455[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fa-ir) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9456[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9457[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; he-il) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9458[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hu-hu) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9459[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; et-ee) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9460[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ca) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9461[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ch) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9462[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hr-hr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9463[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ka-ge) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9464[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hu-hu) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9465[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ch) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9466[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ca-es) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9467[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; vi-vn) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9468[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-ch) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9469[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sk-sk) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9470[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sl-si) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9471[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.5+ (KHTML, like Gecko) Version/5.0.6 Safari/533.5+";
const char S9472[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9473[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ar) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9474[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sr-cs) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9475[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; mt-mt) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9476[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-au) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9477[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ka-ge) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9478[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; la) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9479[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ko-kr) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9480[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; af-za) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9481[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-ch) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9482[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-be) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9483[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; yi) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9484[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; id) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9485[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; af-za) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9486[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ca-es) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9487[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lt-lt) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9488[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hu-hu) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9489[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9490[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; uk-ua) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9491[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sl-si) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9492[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; vi-vn) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9493[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ro-ro) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9494[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ro-ro) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9495[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; et-ee) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9496[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ca) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9497[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fa-ir) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9498[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; id) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9499[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; et-ee) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9500[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-ch) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9501[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hy-am) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9502[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; mk-mk) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S9503[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9504[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; is-is) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9505[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ca-es) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9506[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ka-ge) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9507[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ar) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9508[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ro-ro) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9509[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.6 Safari/533.22.3";
const char S9510[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hr-hr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S9511[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; vi-vn) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9512[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; bg-bg) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9513[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; cs-cz) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S9514[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lt-lt) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9515[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-ch) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S9516[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; id) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9517[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; my) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S9518[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ru) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S9519[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10\\_5\\_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/5.0.6 Safari/533.22.3";
const char S9520[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9521[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; SAMSUNG GT-S5830/S5830BUKP6 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9522[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; SAMSUNG GT-S5830/S5830BUKPD Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9523[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SAMSUNG GT-S5830/S5830BUKPE Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9524[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; GT-S5830-ORANGE/S5830BVKQ7 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9525[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; GT-S5830B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9526[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; GT-S5830L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9527[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-nz; SAMSUNG GT-S5830/S5830BUKP5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9528[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9529[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5830-ORANGE/S5830BVKQ8 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9530[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; GT-S5830D-parrot Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9531[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9532[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; SAMSUNG GT-S5830/S5830BUKPDT Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9533[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5830-ORANGE/S5830BVKQA Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9534[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-S5830 Build/GINGERBREAD)";
const char S9535[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5830 Build/GINGERBREAD)";
const char S9536[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; GT-S5830D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9537[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; SAMSUNG GT-S5830/S5830BUKS1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9538[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; SAMSUNG GT-S5830/S5830BUKS2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9539[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9540[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; GT-S5830L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9541[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5830 Build/GRWK74) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S9542[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; GT-S5830-ORANGE/S5830BVKQ9 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9543[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830-ORANGE/S5830BVKT4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9544[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-S5830 Build/GINGERBREAD)";
const char S9545[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; GT-S5830T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9546[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-S5830-ORANGE/S5830BVKT3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9547[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-S5830M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9548[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-S5830C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9549[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5830L Build/GINGERBREAD)";
const char S9550[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5830 Build/GINGERBREAD)";
const char S9551[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5830D Build/GINGERBREAD)";
const char S9552[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; GT-S5830B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9553[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-S5830L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9554[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT-S5830 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9555[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S9556[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-S5830 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S9557[] = "Mozilla/5.0 (Linux; U; es-es; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S9558[] = "Mozilla/5.0 (Linux; U; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S9559[] = "Mozilla/5.0 (Linux; U; it-it; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S9560[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; GT-S5830G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9561[] = "Mozilla/5.0 (Linux; U; de-de; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S9562[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S9563[] = "Mozilla/5.0 (Linux; U; pt-pt; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S9564[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; GT-S5830 Build/GRWK74)";
const char S9565[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5830B Build/GINGERBREAD)";
const char S9566[] = "Mozilla/5.0 (Linux; U; fr-fr; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S9567[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5830T Build/GINGERBREAD)";
const char S9568[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5830M Build/GINGERBREAD)";
const char S9569[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; GT-S5830 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9570[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en_gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)";
const char S9571[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S9572[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S9573[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT-S5830 Build/GRWK74; Hidric by Fernando F.) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9574[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; he-il; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9575[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-S5830L Build/GINGERBREAD)";
const char S9576[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830 Build/OMEGA ROM by TeamCooper) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9577[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5830L Build/GINGERBREAD)";
const char S9578[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5830C Build/GINGERBREAD)";
const char S9579[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-S5830B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9580[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830 Build/Beats With Smooth Sense) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9581[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-S5830V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9582[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9583[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9584[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9585[] = "Mozilla/5.0 (Linux; U; nl-nl; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S9586[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-S5830/S5830BUKT2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9587[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9588[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-S5830T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9589[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-S5830-ORANGE/S5830BVKT5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9590[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-S5830 Build/GRWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S9591[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-S5830 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9592[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9593[] = "Mozilla 5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S9594[] = "Mozilla/5.0 (Linux; U; de-at; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S9595[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5830M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9596[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT-S5830 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9597[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5830V Build/GINGERBREAD)";
const char S9598[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; GT-S5830T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9599[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830 Build/CCGH1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9600[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-S5830-ORANGE/S5830BVKT6 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9601[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5830 Build/The Ultimatum; The Ultimatum) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9602[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5830 Build/luoshaohao; IPHONE5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9603[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5830T Build/GINGERBREAD)";
const char S9604[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; GT-S5830L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9605[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; GT-S5830 Build/S5830F1V102) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9606[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT-S5830 Build/Think Android Final by galaxy team; Think Android Final by galaxy team) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9607[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; GT-S5830 Build/S5830F1V100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9608[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5830 Build/love cry-A4; love cry-A4(?r?cH)) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9609[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5830G Build/GINGERBREAD)";
const char S9610[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.15 (KHTML, like Gecko) Version/6.0 Safari/536.16";
const char S9611[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.19 (KHTML, like Gecko) Version/6.0 Safari/536.19";
const char S9612[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.25 (KHTML, like Gecko) Version/6.0 Safari/536.25";
const char S9613[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) WebClip/7536.24 Safari/7536.25";
const char S9614[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) Version/6.0 Safari/534.56.5";
const char S9615[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) WebClip/7536.26.7 Safari/7536.26.17";
const char S9616[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/536.26.17 (KHTML, like Gecko) Version/6.0.2 Safari/536.26.17";
const char S9617[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/6.0.2 Safari/534.57.7";
const char S9618[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7.5) AppleWebKit/7534.57.7 (KHTML, like Gecko) Version/6.0.2 Safari/7536.26.17 Raven for Mac/0.7.14665";
const char S9619[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/536.28.10 (KHTML, like Gecko) Version/6.0.3 Safari/536.28.10";
const char S9620[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) WebClip/7536.28.3 Safari/7536.28.10";
const char S9621[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/536.29.13 (KHTML, like Gecko) Version/6.0.4 Safari/536.29.13";
const char S9622[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) WebClip/7536.29.3 Safari/7536.29.13";
const char S9623[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/536.30.1 (KHTML, like Gecko) Version/6.0.5 Safari/536.30.1";
const char S9624[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) WebClip/7536.30 Safari/7536.30.1";
const char S9625[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML,like Gecko) Chrome/29.0.1547.2 Safari/537.36";
const char S9626[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe)";
const char S9627[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S9628[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36";
const char S9629[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36";
const char S9630[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe)";
const char S9631[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/29.0.1600.1 Chrome/29.0.1600.1 Safari/537.36";
const char S9632[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe),gzip(gfe)";
const char S9633[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S9634[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Iron/29.0.1600.1 Chrome/29.0.1600.1 Safari/537.36";
const char S9635[] = "BlackBerry9800/6.0.0.600 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102";
const char S9636[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9800; en-US) AppleWebKit/534.8+ (KHTML, like Gecko) Version/6.0.0.600 Mobile Safari/534.8+";
const char S9637[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML like Gecko) Chrome/28.0.1469.0 Safari/537.36";
const char S9638[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe),gzip(gfe)";
const char S9639[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S9640[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S9641[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S9642[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe)";
const char S9643[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.1.0.0 Safari/537.36";
const char S9644[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.1.0.0 Safari/537.36";
const char S9645[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S9646[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe)";
const char S9647[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36";
const char S9648[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S9649[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36,gzip(gfe)";
const char S9650[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36,gzip(gfe)";
const char S9651[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36";
const char S9652[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S9653[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S9654[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S9655[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S9656[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe)";
const char S9657[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S9658[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S9659[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36";
const char S9660[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36";
const char S9661[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36";
const char S9662[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36 AppEngine-Google; (+http://code.google.com/appengine; appid: s~romantic-kimi)";
const char S9663[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S9664[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S9665[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S9666[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S9667[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe)";
const char S9668[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe),gzip(gfe)";
const char S9669[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.1.0.0 Safari/537.36";
const char S9670[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S9671[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.1.0.0 Safari/537.36";
const char S9672[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.1.0.0 Safari/537.36";
const char S9673[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.1.0.0 Safari/537.11";
const char S9674[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S9675[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S9676[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S9677[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe)";
const char S9678[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S9679[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.4.0.0 Safari/537.11";
const char S9680[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S9681[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe) (via docs.google.com/viewer)";
const char S9682[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.2.0.0 Safari/537.11";
const char S9683[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.3.0.0 Safari/537.11";
const char S9684[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.4.0.0 Safari/537.11";
const char S9685[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11 AppEngine-Google; (+http://code.google.com/appengine; appid: s~youniyimo1) AppEngine-Google; (+http://code.google.com/appengine; appid: s~youniyimo0)";
const char S9686[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11 AppEngine-Google; (+http://code.google.com/appengine; appid: s~youniyimo1) AppEngine-Google; (+http://code.google.com/appengine; appid: s~youniyimo2)";
const char S9687[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11 AppEngine-Google; (+http://code.google.com/appengine; appid: s~youniyimo1) AppEngine-Google; (+http://code.google.com/appengine; appid: s~youniyimo1)";
const char S9688[] = "Mozilla/5.0 (Windows NT 5.1; ; NCLIENT50_AAP7150B86519C) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11";
const char S9689[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11 AppEngine-Google; (+http://code.google.com/appengine; appid: s~youniyimo1) AppEngine-Google; (+http://code.google.com/appengine; appid: s~youniyimo3)";
const char S9690[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.4.0.0 Safari/537.11";
const char S9691[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko GomezAgent 3.0) Chrome/23.0.1271.64 Safari/537.11";
const char S9692[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11 AppEngine-Google; (+http://code.google.com/appengine; appid: s~youniyimo1) AppEngine-Google; (+http://code.google.com/appengine; appid: s~youniyimo4)";
const char S9693[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1207.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/22.0.1207.1 Safari/537.1";
const char S9694[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1221.0) AppleWebKit/537.3 (KHTML, like Gecko) Chrome/22.0.1221.0 Safari/537.3";
const char S9695[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1229.14) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.14 Safari/537.4";
const char S9696[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1229.26) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.26 Safari/537.4";
const char S9697[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1229.56) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.56 Safari/537.4";
const char S9698[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1229.79) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4";
const char S9699[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/22.0.1229.79) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4";
const char S9700[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1229.91) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.91 Safari/537.4";
const char S9701[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1229.64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.64 Safari/537.4";
const char S9702[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1229.92) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.92 Safari/537.4";
const char S9703[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/22.0.1229.94; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S9704[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1229.94) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S9705[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/22.0.1229.94; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S9706[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/22.0.1229.94)";
const char S9707[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/22.0.1229.94)";
const char S9708[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1229.96) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.96 Safari/537.4";
const char S9709[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/22.0.1229.94) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S9710[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/22.0.1229.94) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S9711[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/22.0.1229.94; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S9712[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; chromeframe/22.0.1229.94; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8; .NET4.0C; .NET4.0E)";
const char S9713[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/22.0.1229.94; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S9714[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/22.0.1229.94; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S9715[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; GTB7.4; chromeframe/22.0.1229.94; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)";
const char S9716[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/22.0.1229.94; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)";
const char S9717[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/22.0.1229.94; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)";
const char S9718[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/22.0.1229.94; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S9719[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/22.0.1229.79) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4";
const char S9720[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/22.0.1229.94; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S9721[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/22.0.1229.94; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.1; .NET4.0C; .NET4.0E)";
const char S9722[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1229.95) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.95 Safari/537.4";
const char S9723[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/22.0.1229.96; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S9724[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/22.0.1229.96) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.96 Safari/537.4";
const char S9725[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; chromeframe/22.0.1229.94; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S9726[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9900; en) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.746 Mobile Safari/534.11+";
const char S9727[] = "BlackBerry9900/7.1.0.694 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102";
const char S9728[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A5355d Safari/8536.25";
const char S9729[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A5376e Safari/8536.25";
const char S9730[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A403 Safari/8536.25";
const char S9731[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A402 Safari/8536.25";
const char S9732[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A403Safari/8536.25";
const char S9733[] = "Mozilla/5.0 (iPad; CPU OS6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A403 Safari/8536.25";
const char S9734[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A407 Safari/8536.25";
const char S9735[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A406 Safari/8536.25";
const char S9736[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S9737[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A8424 Safari/8536.25";
const char S9738[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A396 Safari/8536.25";
const char S9739[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A8426 Safari/8536.25";
const char S9740[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523Safari/8536.25";
const char S9741[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A8425 Safari/8536.25";
const char S9742[] = "Mozilla/5.0 (iPad; CPU OS6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S9743[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A8421 Safari/8536.25";
const char S9744[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A8500 Safari/8536.25";
const char S9745[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A550 Safari/8536.25";
const char S9746[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/6.0 Mobile/9B176 Safari/7534.48.3";
const char S9747[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A8418 Safari/8536.25";
const char S9748[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A391 Safari/8536.25";
const char S9749[] = "Mozilla/5.0 (iPad; CPU OS 6_0_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146 Safari/8536.25";
const char S9750[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A8416 Safari/8536.25";
const char S9751[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A8406 Safari/8536.25";
const char S9752[] = "Mozilla/5.0(iPad; U; CPU iPhone OS 60 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML like Gecko) Version/6.0 Mobile/10A5355d Safari/8536.25";
const char S9753[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S9754[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S9755[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A520 Safari/8536.25";
const char S9756[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/9Z999 Safari/8536.25";
const char S9757[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/9Z999 Safari/8536.25";
const char S9758[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A406 Safari/8536.25";
const char S9759[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A8423 Safari/8536.25";
const char S9760[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A406 Safari/8536.25";
const char S9761[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/24.0.1297.0) AppleWebKit/537.16 (KHTML, like Gecko) Chrome/24.0.1297.0 Safari/537.16";
const char S9762[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/24.0.1290.1) AppleWebKit/537.13 (KHTML, like Gecko) Chrome/24.0.1290.1 Safari/537.13";
const char S9763[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/24.0.1312.5) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.5 Safari/537.17";
const char S9764[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/24.0.1312.5) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.5 Safari/537.17";
const char S9765[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/24.0.1312.14) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.14 Safari/537.17";
const char S9766[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/24.0.1312.14) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.14 Safari/537.17";
const char S9767[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/24.0.1312.27) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.27 Safari/537.17";
const char S9768[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/24.0.1312.35) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.35 Safari/537.17";
const char S9769[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/24.0.1312.40) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.40 Safari/537.17";
const char S9770[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/24.0.1312.45) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.45 Safari/537.17";
const char S9771[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/24.0.1312.45) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.45 Safari/537.17";
const char S9772[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/24.0.1312.40) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.40 Safari/537.17";
const char S9773[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/24.0.1312.52) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17";
const char S9774[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/24.0.1312.52) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17";
const char S9775[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; chromeframe/24.0.1312.57)";
const char S9776[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/24.0.1312.56) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17";
const char S9777[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/24.0.1312.56) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17";
const char S9778[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/24.0.1312.57)";
const char S9779[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/24.0.1312.57)";
const char S9780[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/24.0.1312.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S9781[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/24.0.1312.57)";
const char S9782[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/24.0.1312.57)";
const char S9783[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/24.0.1312.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Tablet PC 2.0)";
const char S9784[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; chromeframe/24.0.1312.57)";
const char S9785[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/24.0.1312.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E)";
const char S9786[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/24.0.1312.57) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S9787[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/24.0.1312.57) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S9788[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/24.0.1312.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S9789[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/24.0.1312.57; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S9790[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/24.0.1312.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Tablet PC 2.0)";
const char S9791[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/24.0.1312.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729)";
const char S9792[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/24.0.1312.56; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S9793[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/24.0.1312.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Tablet PC 2.0)";
const char S9794[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/24.0.1312.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729)";
const char S9795[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/24.0.1312.57;  SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S9796[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/24.0.1312.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Tablet PC 2.0)";
const char S9797[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/24.0.1312.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S9798[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/24.0.1312.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Tablet PC 2.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S9799[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.2.0.0 Safari/537.22";
const char S9800[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S9801[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe) (via docs.google.com/viewer)";
const char S9802[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S9803[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S9804[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)";
const char S9805[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.1.0.0 Safari/537.22";
const char S9806[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S9807[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.2.0.0 Safari/537.22";
const char S9808[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S9809[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.7.1364.21027 Safari/537.22,gzip(gfe),gzip(gfe)";
const char S9810[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.7.1364.21027 (gnctrkcll) Safari/537.22";
const char S9811[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22 AppEngine-Google; (+http://code.google.com/appengine; appid: s~atmshowing01)";
const char S9812[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22 BitdefenderSafepay/17.14.0.0 (OBK; 25.0.1364.172; r1194.2013)";
const char S9813[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A5313e";
const char S9814[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A5274d";
const char S9815[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A5288d";
const char S9816[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)";
const char S9817[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334";
const char S9818[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)";
const char S9819[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405";
const char S9820[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A404";
const char S9821[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406";
const char S9822[] = "Miu Ptt 1.5.2 (iPhone; iPhone OS 5.0; zh_TW)";
const char S9823[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper";
const char S9824[] = "NaverCafe/1.0 Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName";
const char S9825[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A40";
const char S9826[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,3;FBMD/iPhone;FBSN/iPhone OS;FBSV/5.0.1;FBSS/2; FBCR/Verizon;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S9827[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 iPad2,1 BingWeb/3.02.1641.X";
const char S9828[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper";
const char S9829[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/5.0;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S9830[] = "NaverCafe/1.0 Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName";
const char S9831[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9Z999";
const char S9832[] = "Mozilla/5.0 (; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405";
const char S9833[] = "Mozilla/5.0 (iPhone; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A5302b";
const char S9834[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176";
const char S9835[] = "Mozilla/5.0 (iPhone; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Safari/7534.48 Instapaper/4.0.1 (+http://www.instapaper.com/iphone)";
const char S9836[] = "Mozilla/5.0 (iPhone; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48 Instapaper/4.2.1 (+http://www.instapaper.com/iphone)";
const char S9837[] = "Mozilla/5.0 (iPhone; U; CPU iPhone 5_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPhone3,1/5.0.1)";
const char S9838[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523";
const char S9839[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Safari/7534.48.3";
const char S9840[] = "Miu Ptt 1.9.2 (iPhone; iPhone OS 5.0.1; zh_TW)";
const char S9841[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 501 like Mac OS X) AppleWebKit/534.46 (KHTML like Gecko) Mobile/9A405 FBAN/FBIOS;FBAV/5.0;FBBV/40121;FBDV/iPhone31;FBMD/iPhone;FBSN/iPhone OS;FBSV/5.0.1;FBSS/2; FBCR/Sunrise;FBID/phone;FBLC/enUS";
const char S9842[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403";
const char S9843[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_WildfireS_A510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9844[] = "HTC_WildfireS_A510e/Mozilla/5.0(Linux; U; Android 2.3.3;it-it; Build/GRH78C) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9845[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-ch; HTC/WildfireS/1.33.166.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9846[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_Wildfire_S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9847[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; el-gr; HTC/WildfireS/1.33.168.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9848[] = "Mozilla/5.0 (Linux; U; Android2.3.3; en-us; HTC_Wildfire_S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9849[] = "Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; HTC_Wildfire_S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9850[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Wildfire S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9851[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;HTC_Wildfire_S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9852[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_Wildfire_S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S9853[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_Wildfire_SBuild/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9854[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Wildfire S A510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9855[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC/WildfireS/1.33.161.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9856[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_Marvel Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9857[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-at; HTC/WildfireS/1.33.169.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9858[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_WildfireS-orange-LS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9859[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; HTC/WildfireS/1.33.163.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9860[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTCA510e/1.0 Android/2.4 release/02.25.2011 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9861[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_WildfireS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9862[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; HTC/WildfireS/1.58.165.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9863[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; el-gr; HTC_WildfireS_A510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S9864[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; HTC/WildfireS/1.64.192.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9865[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Wildfire S A510e Build/GRI40)";
const char S9866[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; USCCADR6230US Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9867[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC-PG762 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9868[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_WildfireS_A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9869[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9870[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/WildfireS/2.14.161.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9871[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Wildfire_S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9872[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Wildfire_S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S9873[] = "Logos/0.9.4 (Linux; U; Android 2.3.3; en_GB; HTC Wildfire S A510e Build/GRI40)";
const char S9874[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-us; HTC Wildfire S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S9875[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Wildfire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9876[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Wildfire S A510e Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S9877[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; HTC/WildfireS/2.14.165.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9878[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_WildfireS-orange-LS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9879[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Wildfire S Build/GRI40)";
const char S9880[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-ch; HTC/WildfireS/2.14.166.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9881[] = "Mozilla/5.0 (Linux; U; Android2.3.5; en-us; HTC_Wildfire_S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9882[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Wildfire S A510e Build/GRJ90)";
const char S9883[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_WildfireS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9884[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Wildfire S Build/GRJ90)";
const char S9885[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; HTC_WildfireS_A515c Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9886[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC/WildfireS/2.14.163.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9887[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC A510e Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S9888[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC-PG762 Build/GRJ22)";
const char S9889[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Wildfire S A510e Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S9890[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-at; HTC/WildfireS/2.14.169.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9891[] = "HTC_WildfireS_A510e/Mozilla/5.0(Linux; U; Android 2.3.5;it-it;Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9892[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC A510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9893[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us;HTC_Wildfire_S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9894[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; el-gr; HTC/WildfireS/2.14.168.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9895[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/WildfireS/1.33.161.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9896[] = "Mozilla/5.0 (Linux; U;Android 2.3.5; en-us; HTC_Wildfire_S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9897[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Wildfire_SBuild/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9898[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC/WildfireS/2.14.192.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9899[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTCA510e/1.0 Android/2.4 release/02.25.2011 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9900[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC A510e Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S9901[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/WildfireS/2.26.161.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9902[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-at; HTC/WildfireS/2.26.169.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9903[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC/WildfireS/2.26.163.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9904[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; HTC/WildfireS/2.26.165.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9905[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/WildfireS/2.26.161.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9906[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; HTC Wildfire S A515c Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9907[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; USCCADR6230US Build/GRJ22)";
const char S9908[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-ch; HTC/WildfireS/2.26.166.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9909[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC/WildfireS/2.26.192.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9910[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Wildfire S A510e Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S9911[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC A510e Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S9912[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; el-gr; HTC/WildfireS/2.26.168.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9913[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9914[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC A510e Build/GRI40)";
const char S9915[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Wildfire S Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S9916[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Wildfire SBuild/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9917[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC WildfireS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9918[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Wildfire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S9919[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Wildfire S A510e Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S9920[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC A510e Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S9921[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us;HTC Wildfire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9922[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTCWildfire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9923[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Wildfire S Build/GRI40)";
const char S9924[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; nl-nl; HTC/WildfireS/2.26.161.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9925[] = "Mozilla/5.0 (Linux; U; Android2.3.5; en-us; HTC Wildfire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9926[] = "Mozilla 5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Wildfire S A510e Build GRI40) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S9927[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Wildfire S A510e Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S9928[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Maxthon/4.2.0.800 Chrome/30.0.1551.0 Safari/537.36";
const char S9929[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S9930[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S9931[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S9932[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Maxthon/4.2.0.800 Chrome/30.0.1551.0 Safari/537.36";
const char S9933[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36,gzip(gfe)";
const char S9934[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36,gzip(gfe)";
const char S9935[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.1.6) Gecko/20091201 Firefox/3.5.6";
const char S9936[] = "Mozilla/5.0 (Windows NT 5.1; U; ru; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.50";
const char S9937[] = "Mozilla/5.0 (Windows NT 5.1; U; Firefox/4.5; en; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.53";
const char S9938[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.10) Gecko/X Firefox/3.5.10 WinTSI 21.04.2010";
const char S9939[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.8) Gecko/X Firefox/3.5.8 WinNT-EVI 23.05.2010";
const char S9940[] = "Mozilla/5.0 (Windows NT 5.1; U; en-US; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.70";
const char S9941[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.5) Gecko/X Firefox/3.5.5 (.NET CLR 3.5.30729) WinTSI 30.01.2010";
const char S9942[] = "Mozilla/5.0 (Windows NT 5.1; U; Firefox/3.5; en; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.53";
const char S9943[] = "Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.63";
const char S9944[] = "Mozilla/5.0 (Windows NT 5.1; U; Firefox/5.0; en; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.53";
const char S9945[] = "Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 11.10";
const char S9946[] = "Mozilla/5.0 (Windows NT 5.1; U; zh-cn; rv:1.8.1) Gecko/X Firefox/3.5.5";
const char S9947[] = "Mozilla/5.0 (Windows NT 5.1; U; zh-cn; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.53";
const char S9948[] = "Mozilla/5.0 (Windows NT 5.1; U; ru; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.60";
const char S9949[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN; rv:1.9.1.2) Firefox/3.5.2";
const char S9950[] = "Mozilla/5.0 (Windows NT 5.1; U; ru; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.52";
const char S9951[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.8) Gecko/X Firefox/3.5.8 WinNT-EVU 13.01.2010";
const char S9952[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.1.0.0 Safari/537.17";
const char S9953[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe) (via docs.google.com/viewer)";
const char S9954[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S9955[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S9956[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.2.0.0 Safari/537.17";
const char S9957[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S9958[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)";
const char S9959[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.2.0.0 Safari/537.17";
const char S9960[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S9961[] = "Mozilla/5.0 (Windows NT 5.1; ; NCLIENT50_AAP7150B86519C) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17";
const char S9962[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML like Gecko) Chrome/24.0.1312.52 Safari/537.17";
const char S9963[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.10.1312.57 Safari/537.17 QIPSurf";
const char S9964[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.2.0.0 Safari/537.17,gzip(gfe)";
const char S9965[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)";
const char S9966[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A5376e Safari/8536.25";
const char S9967[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A383 Safari/8536.25";
const char S9968[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A402 Safari/8536.25";
const char S9969[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A403 Safari/8536.25";
const char S9970[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A405 Safari/8536.25";
const char S9971[] = "Mozilla/5.0 (iPhone; CPU iPhone OS6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A403 Safari/8536.25";
const char S9972[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10A403 Version/6.0 Safari/8536.25 Safari/6533.18.5";
const char S9973[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A403Safari/8536.25";
const char S9974[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10A405 Version/6.0 Safari/8536.25 Safari/6533.18.5";
const char S9975[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A505 Safari/8536.25";
const char S9976[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S9977[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A525 Safari/8536.25";
const char S9978[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A400 Safari/8536.25";
const char S9979[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A396 Safari/8536.25";
const char S9980[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A399 Safari/8536.25";
const char S9981[] = "Mozilla/5.0 (iPhone; CPU iPhone OS6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S9982[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523Safari/8536.25";
const char S9983[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10A523 Version/6.0 Safari/8536.25 Safari/6533.18.5";
const char S9984[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10A525 Version/6.0 Safari/8536.25 Safari/6533.18.5";
const char S9985[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A551 Safari/8536.25";
const char S9986[] = "Mozilla/5.0 (iPhone; CPU iPhone OS6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A525 Safari/8536.25";
const char S9987[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A525Safari/8536.25";
const char S9988[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10A551 Version/6.0 Safari/8536.25 Safari/6533.18.5";
const char S9989[] = "Mozilla/5.0 (iPhone; CPU iPhone OS6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A551 Safari/8536.25";
const char S9990[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A551Safari/8536.25";
const char S9991[] = "Mozilla/5.0 (iPhone; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A5355d Safari/8536.25";
const char S9992[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S9993[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S9994[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/9Z999 Safari/8536.25";
const char S9995[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A524 Safari/8536.25";
const char S9996[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/6.0 Mobile/9B176 Safari/7534.48.3";
const char S9997[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B142 Safari/8536.25";
const char S9998[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A551 Safari/8536.25";
const char S9999[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/9Z999 Safari/8536.25";
const char S10000[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/9Z999 Safari/8536.25";
const char S10001[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/537.4 (KHTML, like Gecko; Google Page Speed Insights) Version/6.0 Mobile/10A525 Safari/8536.25";
const char S10002[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S10003[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B143 Safari/8536.25";
const char S10004[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S10005[] = "Mozilla/5.0 (iPhone; CPU iPhone OS6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A405 Safari/8536.25";
const char S10006[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329 Safari/8536.25";
const char S10007[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe)";
const char S10008[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36";
const char S10009[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S10010[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36";
const char S10011[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36";
const char S10012[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36";
const char S10013[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe),gzip(gfe)";
const char S10014[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Iron/29.0.1600.1 Chrome/29.0.1600.1 Safari/537.36";
const char S10015[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1323.1) AppleWebKit/537.19 (KHTML, like Gecko) Chrome/25.0.1323.1 Safari/537.19";
const char S10016[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1364.5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.5 Safari/537.22";
const char S10017[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1364.36) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.36 Safari/537.22";
const char S10018[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1364.29) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.29 Safari/537.22";
const char S10019[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/25.0.1364.36) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.36 Safari/537.22";
const char S10020[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1364.45) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.45 Safari/537.22";
const char S10021[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1364.58) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.58 Safari/537.22";
const char S10022[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/25.0.1364.58) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.58 Safari/537.22";
const char S10023[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1364.68) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.68 Safari/537.22";
const char S10024[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/25.0.1364.84) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.84 Safari/537.22";
const char S10025[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1364.84) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.84 Safari/537.22";
const char S10026[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/25.0.1364.97; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S10027[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1364.97) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22";
const char S10028[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; chromeframe/25.0.1364.97)";
const char S10029[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/25.0.1364.97) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22";
const char S10030[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/25.0.1364.172)";
const char S10031[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/25.0.1364.172; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Tablet PC 2.0)";
const char S10032[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/25.0.1364.172)";
const char S10033[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/25.0.1364.172)";
const char S10034[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1364.172) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S10035[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/25.0.1364.172) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S10036[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/25.0.1364.172; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S10037[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/25.0.1364.172)";
const char S10038[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/25.0.1364.172; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S10039[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; chromeframe/25.0.1364.172)";
const char S10040[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1364.152) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22";
const char S10041[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/25.0.1364.152) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22";
const char S10042[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1364.160) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.160 Safari/537.22";
const char S10043[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/25.0.1364.160) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.160 Safari/537.22";
const char S10044[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/25.0.1364.172; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322)";
const char S10045[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/25.0.1364.172; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S10046[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/25.0.1364.172; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S10047[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/25.0.1364.172; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Tablet PC 2.0)";
const char S10048[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/25.0.1364.172; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3)";
const char S10049[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/25.0.1364.97; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Zune 4.7)";
const char S10050[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/25.0.1364.172;  SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S10051[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; chromeframe/25.0.1364.172; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322)";
const char S10052[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10053[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sv-se) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10054[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; it-it) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10055[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10056[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18";
const char S10057[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10058[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sv-se) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10059[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ja-jp) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10060[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10061[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10062[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10063[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ja-jp) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10064[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10065[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10066[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-br) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10067[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10068[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-cn) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10069[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ru-ru) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10070[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; it-it) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10071[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nb-no) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10072[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10073[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/528.16 (KHTML, like Gecko) Version/4.0 Safari/528.16";
const char S10074[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; cs-cz) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10075[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ja-jp) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10076[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nb-no) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10077[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ja-jp) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10078[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ru-ru) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10079[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-br) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10080[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fi-fi) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10081[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10082[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-gb) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10083[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10084[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-cn) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10085[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; fr-fr) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18";
const char S10086[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; fr-fr) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10087[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-xl) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10088[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-cn) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10089[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10090[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10091[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10092[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ru-ru) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10093[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10094[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19.2";
const char S10095[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sv-se) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10096[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; nl-nl) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18";
const char S10097[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nb-no) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10098[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10099[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10100[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-ca) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10101[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ru-ru) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10102[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/528.16 (KHTML, like Gecko) Version/4 Public Beta Safari/528.16";
const char S10103[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10104[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-ca) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10105[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10106[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; it-it) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10107[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-br) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10108[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-ca) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10109[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S10110[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-ca) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10111[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10112[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sv-se) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10113[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-pt) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10114[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; it-it) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10115[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-gb) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10116[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; de-de) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18";
const char S10117[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; de-de) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10118[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-tw) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10119[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tr-tr) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10120[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fi-fi) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10121[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-au) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10122[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; da-dk) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10123[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; it-it) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10124[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fi-fi) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10125[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; de-de) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10126[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-au) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10127[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/531.5 (KHTML, like Gecko) Version/4.0.3 Safari/531.7";
const char S10128[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10129[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pl-pl) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10130[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; es-es) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10131[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fi-fi) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10132[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; sv-se) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10133[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nb-no) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10134[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0 Safari/528.17";
const char S10135[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; th-th) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10136[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; de-de) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S10137[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-br) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10138[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; it-) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S10139[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-gb) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S10140[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10141[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10142[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10143[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10144[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S10145[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/4.0.1 Safari/530.18";
const char S10146[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-pt) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10147[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; el-gr) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10148[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; da-dk) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10149[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; da-dk) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10150[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-gb) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10151[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; da-dk) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10152[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-gb) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10153[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-au) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10154[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-cn) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10155[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pl-pl) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10156[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; nl-nl) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10157[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; fr-fr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10158[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; nl-nl) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10159[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-pt) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10160[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pl-pl) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10161[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-pt) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10162[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-au) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10163[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; nl-nl) AppleWebKit/528.16 (KHTML, like Gecko) Version/4.0 Safari/528.16";
const char S10164[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; es-es) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10165[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ar) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10166[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ko-kr) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10167[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-tw) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10168[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/4.0 Safari/528.16";
const char S10169[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; it-it) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S10170[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; el-gr) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10171[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; it-it) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18";
const char S10172[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; es-es) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S10173[] = "Mozilla/5.0(Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S10174[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.3 (KHTML, like Gecko) Version/4.0.3 Safari/533.3";
const char S10175[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/4.0 Safari/528.16";
const char S10176[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; he-il) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10177[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tr-tr) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10178[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) WebClip/5527.2 Safari/5531.22.7";
const char S10179[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lt-lt) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10180[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ko-kr) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10181[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ca) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10182[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; fr-fr) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S10183[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/531.21.8 (KHTML, like Gecko) WebClip/5527.2 Safari/5531.21.10";
const char S10184[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-gb) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10185[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ar) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10186[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.1 Safari/530.18";
const char S10187[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; it-it) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10188[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hr-hr) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10189[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-ch) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10190[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-tw) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10191[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ca) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10192[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; he-il) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10193[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hu-hu) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10194[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; el-gr) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10195[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ca-es) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10196[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ja-jp) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10197[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; cs-cz) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10198[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7;en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S10199[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-tw) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10200[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10201[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ca) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10202[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tr-tr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10203[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; nl-nl) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S10204[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; af-za) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10205[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ja-jp) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10206[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; et-ee) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10207[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; cs-cz) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10208[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tr-tr) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10209[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; af-za) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10210[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; af-za) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10211[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-us) AppleWebKit/528.4+ (KHTML, like Gecko) Version/4.0dp1 Safari/526.11.2";
const char S10212[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ar) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10213[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; uk-ua) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10214[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; is-is) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10215[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ru-ru) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S10216[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; uk-ua) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10217[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; cs-cz) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10218[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sl-si) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10219[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10220[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ch) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10221[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fa-ir) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10222[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lt-lt) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10223[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; da-dk) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18";
const char S10224[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-xl) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10225[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tl) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10226[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; he-il) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10227[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/528.18.1 (KHTML, like Gecko) Version/4 Public Beta Safari/528.17";
const char S10228[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ca-es) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10229[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; pt-br) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10230[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; el-gr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10231[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; is-is) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10232[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; is-is) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10233[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_6; it-it) AppleWebKit/528.16 (KHTML, like Gecko) Version/4.0 Safari/528.16";
const char S10234[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ko-kr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10235[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/4.0 Safari/528.17";
const char S10236[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-gb) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18";
const char S10237[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; he-il) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10238[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; vi-vn) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10239[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-gb) AppleWebKit/528.10+ (KHTML, like Gecko) Version/4.0dp1 Safari/526.11.2";
const char S10240[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ru-ru) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10241[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/5";
const char S10242[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; zh-cn) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S10243[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; fr-) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S10244[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; pt-br) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10245[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ru-ru) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18";
const char S10246[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10247[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ca) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10248[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; cs-cz) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10249[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sk-sk) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10250[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; th-th) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10251[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ko-kr) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10252[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lv-lv) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10253[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_7; en-us) AppleWebKit/528.16 (KHTML, like Gecko) Version/4.0 Safari/528.16";
const char S10254[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; sv-se) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S10255[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lt-lt) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10256[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/531.9 (KHTML, like Gecko) WebClip/5527.2 Safari/5531.9";
const char S10257[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; th-th) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10258[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ja-jp) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18";
const char S10259[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ch) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10260[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-gb) AppleWebKit/528.4+ (KHTML, like Gecko) Version/4.0dp1 Safari/526.11.2";
const char S10261[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ro-ro) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10262[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; th-th) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10263[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-xl) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10264[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ja-jp) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S10265[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ar) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10266[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; zh-tw) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18";
const char S10267[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10268[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S10269[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; vi-vn) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10270[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; af-za) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10271[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S10272[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10273[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-xl) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10274[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/528.16 (KHTML, like Gecko) Version/4 Public Beta Safari/528.16";
const char S10275[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sk-sk) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10276[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-au) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18";
const char S10277[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; zh-tw) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10278[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18";
const char S10279[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sq) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10280[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_7; es-es) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18";
const char S10281[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ro-ro) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10282[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/526.11 (KHTML, like Gecko) Version/4.0dp1 Safari/526.11.2";
const char S10283[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; nb-no) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18";
const char S10284[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; sv-se) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18";
const char S10285[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-au) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10286[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/528.14 (KHTML, like Gecko) Version/4.0 Safari/528.14";
const char S10287[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.11+ (KHTML, like Gecko) Version/4.0 Safari/530.11";
const char S10288[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/528.8+ (KHTML, like Gecko) Version/4.0 Safari/528.1";
const char S10289[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.6 (KHTML, like Gecko) Version/4.0 Safari/530.5";
const char S10290[] = "Mozilla/5.0 (Linux; U; Intel Mac OS X 10_5_7; en-us; Quatius-TV/SONIQ_51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S10291[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ca-es) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10292[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10293[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; pl-pl) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S10294[] = "Mozilla/5.0 (Linux; U; Intel Mac OS X 10_5_7; en-us; Quatius-TV/SONIQ_50) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S10295[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/528.12 (KHTML, like Gecko) Version/4.0 Safari/528.12";
const char S10296[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/526.11 (KHTML, like Gecko) Version/4.0dp1 Safari/526.11.2";
const char S10297[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ru-ru) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10298[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-gb) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10299[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.6+ (KHTML, like Gecko) Version/4.0 Safari/530.1";
const char S10300[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-ch) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10301[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-ch) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10302[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pl-pl) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9 FBSMTWB";
const char S10303[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; nb-no) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10304[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; fi-fi) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10305[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lt-lt) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10306[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; de-de) AppleWebKit/526.11 (KHTML, like Gecko) Version/4.0dp1 Safari/526.11.2";
const char S10307[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; da-dk) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10308[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; pl-pl) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10309[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hr-hr) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10310[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sw) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10311[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.6 (KHTML, like Gecko) Version/4.0 Safari/530.6";
const char S10312[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_4; en-us) AppleWebKit/528.1 (KHTML, like Gecko) Version/4.0 Safari/528.1";
const char S10313[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; us-us) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10314[] = "Mozilla/5.0 (Linux; U; Intel Mac OS X 10_5_7; en-us; Quatius-TV/SONIQ_52) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S10315[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_8; hu-hu) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10316[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; pt-pt) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S10317[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_5; en-us) AppleWebKit/528.1 (KHTML, like Gecko) Version/4.0 Safari/528.1";
const char S10318[] = "Mozilla/5.0 (Linux; U; Intel Mac OS X 10_5_7; en-us; Quatius-TV/SONIQ_53) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S10319[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sl-si) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10320[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9780; en) AppleWebKit/534.8+ (KHTML, like Gecko) Version/6.0.0.666 Mobile Safari/534.8+";
const char S10321[] = "BlackBerry9780/6.0.0.448 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102";
const char S10322[] = "BlackBerry9780/6.6.0.207 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S10323[] = "BlackBerry9780/6.5.0.54 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S10324[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.1)";
const char S10325[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/5.0)";
const char S10326[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/5.0)";
const char S10327[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1)";
const char S10328[] = "Mozilla/5.0 (compatible; MSIE 9.0; AOL 9.7; AOLBuild 4343.19; Windows NT 5.1; WOW64; Trident/5.0)";
const char S10329[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S10330[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.1; WOW64; Trident/5.0)";
const char S10331[] = "Mozilla/9.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1);)";
const char S10332[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S10333[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; SV1)";
const char S10334[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; SV1)";
const char S10335[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; SV1)";
const char S10336[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; MS-RTC LM 8)";
const char S10337[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322)";
const char S10338[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; SV1)";
const char S10339[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C)";
const char S10340[] = "Mozilla/5.0 (Windows; U; MSIE 9.0; Windows NT 5.1; de-DE) Trident/5.0";
const char S10341[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)";
const char S10342[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0)";
const char S10343[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3230.2052; Windows XP 5.1; MSIE 9.0.8112.16421)";
const char S10344[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/4.0; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S10345[] = "Mozilla/5.0 (Windows NT 5.1; MSIE 9.0; Edition IBIS; Trident/5.0)";
const char S10346[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.2; .NET CLR 1.0.3705;)";
const char S10347[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)";
const char S10348[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C)";
const char S10349[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; SV1; SV1)";
const char S10350[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; SV1; SV1)";
const char S10351[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; SV1; SV1)";
const char S10352[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; SV1; SV1)";
const char S10353[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/5.0; InfoPath.2; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S10354[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTCA9191/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10355[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; Desire HD Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10356[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_DesireHD_A9191 Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10357[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC_DesireHD_A9191 Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10358[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_DesireHD-orange-LS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10359[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10360[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_DesireHD_A9191 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10361[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire HD 2.36.161.12 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10362[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire HD 2.50.161.3 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10363[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_DesireHD_A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10364[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire HD Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10365[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-; 001HT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10366[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_DesireHD_A9191 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10367[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_DesireHD_A9191 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10368[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; HTC Desire HD 1.32.163.6 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10369[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10370[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; HTC Desire HD 2.36.163.13 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10371[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_DesireHD_A9191/1.0 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10372[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; HTC Desire HD 1.75.161.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10373[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_DesireHD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10374[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-jp; 001HT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10375[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; HTC_DesireHD_A9192 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10376[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire HD 2.50.161.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10377[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10378[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Desire HD Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10379[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Desire HD Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10380[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; Desire HD Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10381[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; HTC Desire HD 2.50.163.3 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10382[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; iw-; HTC_DesireHD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10383[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; iw-; HTC_DesireHD_A9191 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10384[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Desire HD Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10385[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; Desire HD Build/GRI40)";
const char S10386[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10387[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10388[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10389[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire HD 3.13.161.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10390[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Desire HD Build/unknown) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10391[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; HTC Desire HD 2.50.161.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10392[] = "Logos/0.9.4 (Linux; U; Android 2.3.3; en_US; Desire HD Build/GRI40)";
const char S10393[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; HTC_DesireHD_A9191 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10394[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Desire HD Build/GRJ90)";
const char S10395[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10396[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; 001HT Build/GRI40)";
const char S10397[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Desire HD Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S10398[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; Desire HD Build/GRI40)";
const char S10399[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Desire HD Build/GRJ90)";
const char S10400[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Desire HD A9191 Build/GRJ90)";
const char S10401[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; HTC_DesireHD_A9192 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10402[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_DesireHD-orange-LS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10403[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_DesireHD Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10404[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC Desire HD 3.13.163.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10405[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire HD A9191 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S10406[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Desire HD Build/GRI40) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S10407[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire HD with Beats Audio Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10408[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Desire HD Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S10409[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Desire HD Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S10410[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire HD A9191 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S10411[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-au; Desire HD Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10412[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; Desire HD Build/GRJ22)";
const char S10413[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; HTC Desire HD A9191 Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10414[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Desire HD Build/Eva.MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10415[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Desire HD Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10416[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-; HTC_DesireHD_A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10417[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Desire HD Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S10418[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire HD Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S10419[] = "Mozilla/5.0 (Linux; U; Android 2.3.7 base 280; en-gb; Desire HD Build/GRI40; LexBoosT DHD+) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10420[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC Desire HD 2.50.163.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10421[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Desire HD Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S10422[] = "Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire HD A9191 Build GRJ90) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S10423[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Desire HD Build/GRJ90) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S10424[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire HD 3.13.161.5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10425[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30";
const char S10426[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10427[] = "Mozilla 5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Desire HD Build MIUI) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S10428[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; HTC-DESIRE HD Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10429[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-hk; HTC_Desire_HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10430[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Desire HD Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10431[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 2.3.5;dev=ace;devm=Desire HD;man=HTC;brw=Android;shwd=480x800;shwa=480x763;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S10432[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; HTC Desire HD Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10433[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.2.0.0 Safari/537.22";
const char S10434[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.2.0.0 Safari/537.22";
const char S10435[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S10436[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S10437[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)";
const char S10438[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)";
const char S10439[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.1.0.0 Safari/537.22";
const char S10440[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.1.0.0 Safari/537.22";
const char S10441[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe) (via docs.google.com/viewer)";
const char S10442[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22 BitdefenderSafepay/17.14.0.0 (OBK; 25.0.1364.172; r1194.2013)";
const char S10443[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.22 (KHTML like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S10444[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1083) AppleWebKit/537.36 (KHTML like Gecko) Chrome/28.0.1469.0 Safari/537.36";
const char S10445[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36";
const char S10446[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36";
const char S10447[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36,gzip(gfe)";
const char S10448[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S10449[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S10450[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S10451[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe)";
const char S10452[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S10453[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S10454[] = "BlackBerry9300/6.0.0.526 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102";
const char S10455[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9300; en) AppleWebKit/534.8+ (KHTML, like Gecko) Version/6.0.0.668 Mobile Safari/534.8+";
const char S10456[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10457[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-I9100 Build/JRO03H)";
const char S10458[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9100 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10459[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; GT-I9100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10460[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9100 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10461[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10462[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10463[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9100 Build/JZO54K)";
const char S10464[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10465[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/cMIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10466[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100G Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10467[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; GT-I9100 Build/1.9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10468[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100 Build/cMIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10469[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S10470[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9100G Build/JZO54K)";
const char S10471[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9100 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S10472[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; GT-I9100M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10473[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; GT-I9100G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10474[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/M1UI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10475[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9100G Build/cMIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10476[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I9100/I9100XWLSD Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10477[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; GT-I9100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10478[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9100M Build/JZO54K)";
const char S10479[] = "Mozilla/5.0 (Linux; U; Android 4.11; en-us; GT-I9100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10480[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S10481[] = "Mozilla/5.0 (Linux; U; Android 4.1.2_????_??????; he-il; GT-I9100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10482[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-I9100T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10483[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9100T Build/JZO54K)";
const char S10484[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; GT-I9100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10485[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-I9100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10486[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9100g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10487[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100-ORANGE/I9100XWLSM Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10488[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I9100/I9100XWLSM Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10489[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9100G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10490[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-I9100-ORANGE/I9100XWLSU Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10491[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; SAMSUNG GT-I9100/I9100XWLSS Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10492[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100G Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10493[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT_I9100 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10494[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; I9100 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10495[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ar-ae; GT-I9100 Build/JZO54K_v12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10496[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9100G Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S10497[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9108 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10498[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100-ORANGE/I9100XWLSY Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10499[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I9100/I9100XWLSY Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10500[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-I9100G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10501[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; GT-I9100G Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10502[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; vi-vn; GT-I9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10503[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9100 Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S10504[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9100 Build/JZO54K.ASNET) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S10505[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pl-pl) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10506[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; zh-cn) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10507[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; da-dk) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10508[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10509[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; de-de) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10510[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ja-jp) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10511[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-us) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10512[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; it-it) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10513[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; es-es) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10514[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ja-jp) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10515[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-us) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10516[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; it-it) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10517[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fr-fr) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10518[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; sv-se) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10519[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ja-jp) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10520[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; sv-se) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10521[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-gb) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10522[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-us) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10523[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pt-br) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10524[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; de-de) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10525[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/530.13 (KHTML, like Gecko) Version/4.0 Safari/530.15";
const char S10526[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; it-it) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10527[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; de-de) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10528[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; pt-br) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10529[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ko-kr) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10530[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; es-es) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10531[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nb-no) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10532[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-gb) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10533[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; de-de) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10534[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-us) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10535[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ru-ru) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10536[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nb-no) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10537[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10538[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; nl-nl) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10539[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; nb-no) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10540[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ja-jp) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10541[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pl-pl) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10542[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; nb-no) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10543[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; fr-fr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10544[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nl-nl) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10545[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; es-es) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10546[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; zh-cn) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10547[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; fr-fr) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10548[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; sv-se) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10549[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; fr-fr) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10550[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; de-de) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10551[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; es-es) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10552[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pt-pt) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10553[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; de-at) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10554[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; nl-nl) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10555[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; zh-tw) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10556[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; sv-se) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10557[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; nl-nl) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10558[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-gb) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10559[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; es-es) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10560[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; th-th) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10561[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; fr-fr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10562[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fi-fi) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10563[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; zh-tw) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10564[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10565[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-au) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10566[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10567[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; sv-se) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10568[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pt-pt) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10569[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10570[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; ja-jp) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10571[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ar) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10572[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10573[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-ca) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10574[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; th-th) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10575[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; de-de) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10576[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; da-dk) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10577[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fr-fr) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10578[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pt-br) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10579[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ru-ru) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10580[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; zh-cn) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10581[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; nb-no) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10582[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-au) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10583[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; sv-se) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10584[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fa-ir) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10585[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; zh-tw) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10586[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10587[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; it-it) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10588[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; fi-fi) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10589[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nl-nl) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10590[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; nl-nl) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10591[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; pt-br) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10592[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; pt-br) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10593[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; pt-br) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10594[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; pt-pt) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10595[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; nb-no) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10596[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10597[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/531.4 (KHTML, like Gecko) Version/4.0.3 Safari/531.4";
const char S10598[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10599[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10600[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10601[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10602[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; es-xl) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10603[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10604[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10605[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10606[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ja-jp) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10607[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fi-fi) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10608[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; it-it) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10609[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-au) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10610[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ru-ru) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10611[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; zh-tw) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10612[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-gb) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10613[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ar) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10614[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; da-dk) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10615[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10616[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; pt-pt) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10617[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; es-es) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10618[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-gb) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10619[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; it-it) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10620[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; af-za) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10621[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ru-ru) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10622[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; zh-cn) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10623[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; fr-fr) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10624[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; pl-pl) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10625[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; th-th) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10626[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ko-kr) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10627[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/531.5 (KHTML, like Gecko) Version/4.0.3 Safari/531.7";
const char S10628[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; ru-ru) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10629[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; tr-tr) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10630[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; cs-cz) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10631[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-gb) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10632[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; el-gr) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10633[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; tr-tr) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10634[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; fi-fi) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10635[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; pl-pl) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10636[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; da-dk) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10637[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10638[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/530.8 (KHTML, like Gecko) Version/4.0 Safari/530.8";
const char S10639[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; ar) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10640[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; zh-tw) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10641[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko, Safari/6531.22.7) ADM/722";
const char S10642[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10643[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-au) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10644[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10645[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ru-ru) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10646[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; da-dk) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10647[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-ca) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10648[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/531.0 (KHTML, like Gecko) Version/4.0 Safari/531.0";
const char S10649[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; fi-fi) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10650[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; tr-tr) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10651[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10652[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; tr-tr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10653[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10654[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_6_1; en_GB, en_US) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10655[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; he-il) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10656[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10657[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; it-it) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10658[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; lt-lt) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10659[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sv-se) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10660[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ko-kr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10661[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; vi-vn) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10662[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10663[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; lt-lt) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10664[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; vi-vn) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10665[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10666[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-au) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10667[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; nl-nl) AppleWebKit/533.16 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10668[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; es-es) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10669[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; fr-fr) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10670[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; af-za) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10671[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; zh-cn) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10672[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; hu-hu) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10673[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ro-ro) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10674[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; cs-cz) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10675[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; hr-hr) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10676[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; is-is) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10677[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10678[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-us) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531";
const char S10679[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/531.9 (KHTML, like Gecko, Safari/6531.9) ADM/722";
const char S10680[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; fi-fi) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10681[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10682[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; el-gr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10683[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ca-es) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10684[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; de-de) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10685[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; et-ee) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10686[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-us) AppleWebKit/531.21.8 (KHTML, like Gecko, Safari/6531.21.10) ADM/722";
const char S10687[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; sk-sk) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10688[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; sv-se) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10689[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; pt-pt) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10690[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10691[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10692[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10693[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10694[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10695[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-fr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10696[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; zh-CN) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S10697[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10698[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; lt-lt) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10699[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; he-il) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10700[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; id) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10701[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; th-th) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10702[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; vi-vn) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10703[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-us) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.";
const char S10704[] = "Mozilla/5.0(Macintosh;U;Intel Mac OS X 10_6_3;en-us;V55 Build/MASTER)AppleWebKit/534.13(KHTML,like Gecko)Version4.0 Safari/534.13";
const char S10705[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; ca-es) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10706[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fr-ca) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10707[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10708[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10709[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/531.21.11 (KHTML, like Gecko) WebClip/6530.10 Safari/6531.21.10";
const char S10710[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10711[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; ko-kr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10712[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; de-ch) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10713[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; pt-pt) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10714[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fa-ir) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10715[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; he-il) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10716[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ca-es) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10717[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; de-de) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10718[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; tr-tr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10719[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-ca) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10720[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10721[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; fa-ir) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10722[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ca-es) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10723[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; fr-ca) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10724[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; cs-cz) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10725[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; el-gr) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10726[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10727[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; he-il) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10728[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ar) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10729[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; es-xl) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10730[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; el-gr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10731[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10732[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ko-kr) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10733[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; cs-cz) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10734[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; es-xl) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10735[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; pl-pl) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10736[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; es-es) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10737[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; de-de) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10738[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10739[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/533.21.1";
const char S10740[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; th-th) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10741[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-au) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10742[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; zh-cn) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10743[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; hu-hu) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10744[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ar) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10745[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10746[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-us) AppleWebKit/531.9 (KHTML, like Gecko, Safari/6531.9) ADM/722";
const char S10747[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; vi-vn) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10748[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; nl-nl) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10749[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; nl-nl) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10750[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/530.6+ (KHTML, like Gecko) Version/4.0 Safari/530.6";
const char S10751[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-us) AppleWebKit/531.9 (KHTML, like Gecko, Safari/6531.9) ADM/722";
const char S10752[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; cs-cz) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10753[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; is-is) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10754[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; es-xl) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10755[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; bg-bg) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10756[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; us-us) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10757[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10758[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; he-il) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10759[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ro-ro) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10760[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; zh-tw) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10761[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; sk-sk) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10762[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10763[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10764[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10765[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; af-za) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10766[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10767[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/531.1 (KHTML, like Gecko) Version/4.0 Safari/531.1";
const char S10768[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/530.10 (KHTML, like Gecko) Version/4.0 Safari/530.10";
const char S10769[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nb-no) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10770[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; lt-lt) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10771[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-ca) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10772[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fr-fr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10773[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; da-dk) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10774[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/526.9.1 (KHTML, like Gecko) Version/4.0dp1 Safari/526.10.1";
const char S10775[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; fa-ir) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10776[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; it-it) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10777[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; it-it) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10778[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10779[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; fr-ca) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10780[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10781[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10782[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6) AppleWebKit/531.4 (KHTML, like Gecko) Version/4.0.3 Safari/531.4";
const char S10783[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nb-no) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10784[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; de-ch) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10785[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pt-br) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10786[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.5 Safari/533.19.4";
const char S10787[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.6+ (KHTML, like Gecko) Version/4.0 Safari/528.16 Titanium/1.1.0";
const char S10788[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/530.4 (KHTML, like Gecko) Version/4.0 Safari/530.4";
const char S10789[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; us-us) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10790[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; us-us) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10 Opera/9.50 (Windows NT 6.0; U; en)";
const char S10791[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; de-ch) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10792[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.17.8 (KHTML, like Gecko) Mozilla/5.0 - AppleWebKit/533.17.8 (KHTML, like Gecko) Version/4.0.0 Safari/533.17.8";
const char S10793[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; es-es) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10794[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; hi-in) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10795[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1061; en-us) AppleWebKit/531.9 (KHTML like Gecko) Version/4.0.3 Safari/531.9";
const char S10796[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 106; en-us) AppleWebKit/531.22.7 (KHTML like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10797[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 106; en-us) AppleWebKit/531.4 (KHTML like Gecko) Version/4.0.3 Safari/531.4";
const char S10798[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 106; ar) AppleWebKit/531.9 (KHTML like Gecko) Version/4.0.3 Safari/531.9";
const char S10799[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 106; en-us) AppleWebKit/531.5 (KHTML like Gecko) Version/4.0.3 Safari/531.7";
const char S10800[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 106; en-us) AppleWebKit/530.13 (KHTML like Gecko) Version/4.0 Safari/530.15";
const char S10801[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sv-se) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S10802[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; hr-hr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10803[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pl-pl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10804[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/(null) (KHTML, like Gecko) Version/4.0 Safari/528.16 Titanium/1.1.0";
const char S10805[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-ca) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10 FBSMTWB";
const char S10806[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-fr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S10807[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; cy) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S10808[] = "Mozilla/5.0 (Macintosh; U; U; PPC Mac OS X 10_6_7 rv:6.0; en-US) AppleWebKit/532.23.3 (KHTML, like Gecko) Version/4.0.2 Safari/532.23.3";
const char S10809[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S10810[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S10811[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S10812[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S10813[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)";
const char S10814[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)";
const char S10815[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe) (via docs.google.com/viewer)";
const char S10816[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe) (via docs.google.com/viewer)";
const char S10817[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S10818[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S10819[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10820[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S10821[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-S5360L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10822[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-nz; GT-S5360T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10823[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-S5360 Build/GINGERBREAD)";
const char S10824[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; GT-S5360B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10825[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10826[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-S5369/S5369BUKL3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10827[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-S5369/S5369BULA1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10828[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-S5360B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10829[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5360 Build/GINGERBREAD)";
const char S10830[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; SAMSUNG GT-S5360T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10831[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-S5360 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S10832[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-S5369-ORANGE/S5369BVLC3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10833[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; GT-S5360L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10834[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SCH-i509 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10835[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SCH-i509 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10836[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10837[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-S5360L Build/GINGERBREAD)";
const char S10838[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5360L Build/GINGERBREAD)";
const char S10839[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ie; SAMSUNG GT-S5369/S5369BULD4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10840[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; SAMSUNG SAMSUNG GT-S5360T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10841[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; GT-S5369 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10842[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i509 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S10843[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-S5369-ORANGE/S5369BVLE1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10844[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ie; SAMSUNG GT-S5369/S5369BULF2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10845[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-i509 Build/GINGERBREAD)";
const char S10846[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SCH-i509 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S10847[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10848[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; GT-S5360B Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S10849[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5360B Build/GINGERBREAD)";
const char S10850[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-S5360B Build/GINGERBREAD)";
const char S10851[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5360T Build/GINGERBREAD)";
const char S10852[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5369 Build/GINGERBREAD)";
const char S10853[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i509 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S10854[] = "Mozilla/5.0(Linux;U;Android2.3.6;Zh_cn;GT-S5360;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S10855[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-S5369-ORANGE/S5369BVLF1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10856[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5360 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S10857[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; SCH-I509 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10858[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ie; SAMSUNG GT-S5369/S5369BULH1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10859[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.0 Version/4.0 Mobile Safari/530.17";
const char S10860[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ie; SAMSUNG GT-S5369/S5369BULL1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10861[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-S5369/S5369BUMB1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10862[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SAMSUNG GT-S5360T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10863[] = "ZingBrowser/3.1/Adr (Linux; U; 2.3.6; en; GT-S5360 Build/GINGERBREAD.DXLA1;240*320)";
const char S10864[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5360L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10865[] = "ZingBrowser/3.1/Adr (Linux; U; 2.3.6; en; GT-S5360 Build/GINGERBREAD.DXLE1;240*320)";
const char S10866[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-S5369-ORANGE/S5369BVLL1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10867[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SCH-i509 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10868[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; SAMSUNG SAMSUNG SAMSUNG GT-S5360T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S10869[] = "ZingBrowser/3.1/Adr (Linux; U; 2.3.6; en; GT-S5360 Build/GINGERBREAD.DXKJ3;240*320)";
const char S10870[] = "ZingBrowser/3.1/Adr (Linux; U; 2.3.6; en; GT-S5360 Build/GINGERBREAD.DXKK4;240*320)";
const char S10871[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5";
const char S10872[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5,gzip(gfe) (via translate.google.com)";
const char S10873[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S10874[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5,gzip(gfe) (via translate.google.com)";
const char S10875[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5,gzip(gfe) (via docs.google.com/viewer)";
const char S10876[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5,gzip(gfe) (via docs.google.com/viewer)";
const char S10877[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/534.7 ZemanaAID/FFFF0059";
const char S10878[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5,gzip(gfe)";
const char S10879[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5,gzip(gfe)";
const char S10880[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5";
const char S10881[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5";
const char S10882[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5,gzip(gfe)";
const char S10883[] = "Mozilla/5.0 (Windows; Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5";
const char S10884[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5410 Chrome/19.1.1084.5410 Safari/536.5,gzip(gfe)";
const char S10885[] = "User-Agent: Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S10886[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/19.77.34.5 Safari/537.1";
const char S10887[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S10888[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S10889[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S10890[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11,gzip(gfe)";
const char S10891[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11";
const char S10892[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/23.0.912.77 Safari/535.7 AppEngine-Google; (+http://code.google.com/appengine; appid: s~feedly-nikon)";
const char S10893[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11";
const char S10894[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11";
const char S10895[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11,gzip(gfe)";
const char S10896[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11,gzip(gfe) (via docs.google.com/viewer)";
const char S10897[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11 SEMvisu/BOT";
const char S10898[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/23.0.912.77 Safari/535.7 AppEngine-Google; (+http://code.google.com/appengine; appid: s~feedly-hercule)";
const char S10899[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S10900[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S10901[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S10902[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via translate.google.com)";
const char S10903[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S10904[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.<rndr(1000:1210";
const char S10905[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)";
const char S10906[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via docs.google.com/viewer)";
const char S10907[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 AppEngine-Google; (+http://code.google.com/appengine; appid: s~blassegg01)";
const char S10908[] = "Mozilla/5.0 (Windows NT 5.1; en-US) AppleWebKit/535.12 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/535.12";
const char S10909[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S10910[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (gtsr) Safari/537.4";
const char S10911[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (fnrb) Safari/537.4";
const char S10912[] = "Mozilla/5.0 (Windows NT 5.1; ; NCLIENT50_AAP7150B86519C) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S10913[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.520 YaBrowser/1.5.1105.520 (bskt) Safari/537.4";
const char S10914[] = "Mozilla/5.0 (Windows NT 5.1; zh-CN) AppleWebKit/535.12 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/535.12";
const char S10915[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.1.9999.433 Safari/537.4";
const char S10916[] = "BlackBerry9700/5.0.0.714 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/100";
const char S10917[] = "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.15";
const char S10918[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.15";
const char S10919[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.15";
const char S10920[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.15";
const char S10921[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.15";
const char S10922[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.15";
const char S10923[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.15";
const char S10924[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6080)) Presto/2.12.388 Version/12.15";
const char S10925[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64) Opera 12.15";
const char S10926[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.15";
const char S10927[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.15";
const char S10928[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.15";
const char S10929[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.15";
const char S10930[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.15";
const char S10931[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.15";
const char S10932[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.15";
const char S10933[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.15";
const char S10934[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.15";
const char S10935[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.15";
const char S10936[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.15";
const char S10937[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.15";
const char S10938[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.15";
const char S10939[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.15";
const char S10940[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.15";
const char S10941[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.15";
const char S10942[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.15";
const char S10943[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.15";
const char S10944[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.15";
const char S10945[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.15";
const char S10946[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.15";
const char S10947[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.15";
const char S10948[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.15";
const char S10949[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.15";
const char S10950[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.15";
const char S10951[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.15";
const char S10952[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.15";
const char S10953[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.15,gzip(gfe)";
const char S10954[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.15";
const char S10955[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.15";
const char S10956[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.15 Yandex.Translate";
const char S10957[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.15";
const char S10958[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.15";
const char S10959[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.15";
const char S10960[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.15";
const char S10961[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.15";
const char S10962[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.15";
const char S10963[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.15";
const char S10964[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.15";
const char S10965[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.15";
const char S10966[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.15";
const char S10967[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.15";
const char S10968[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.15";
const char S10969[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5997)) Presto/2.12.388 Version/12.15";
const char S10970[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.15";
const char S10971[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.15";
const char S10972[] = "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.15,gzip(gfe)";
const char S10973[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.15";
const char S10974[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.15";
const char S10975[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.15";
const char S10976[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.15";
const char S10977[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4933)) Presto/2.12.388 Version/12.15";
const char S10978[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.15";
const char S10979[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.15";
const char S10980[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.15";
const char S10981[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.15";
const char S10982[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.15";
const char S10983[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.15";
const char S10984[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.15";
const char S10985[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.15";
const char S10986[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.15";
const char S10987[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.15";
const char S10988[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.15";
const char S10989[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.15";
const char S10990[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5217)) Presto/2.12.388 Version/12.15";
const char S10991[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5299)) Presto/2.12.388 Version/12.15";
const char S10992[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5288)) Presto/2.12.388 Version/12.15";
const char S10993[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5278)) Presto/2.12.388 Version/12.15";
const char S10994[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03658)) Presto/2.12.388 Version/12.15";
const char S10995[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.15";
const char S10996[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.15";
const char S10997[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6035)) Presto/2.12.388 Version/12.15";
const char S10998[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03789)) Presto/2.12.388 Version/12.15";
const char S10999[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.15";
const char S11000[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.15";
const char S11001[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.15";
const char S11002[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6046)) Presto/2.12.388 Version/12.15";
const char S11003[] = "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.15,gzip(gfe) (via docs.google.com/viewer)";
const char S11004[] = "Opera/9.80 (Windows NT 6.1; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.15";
const char S11005[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.15";
const char S11006[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5996)) Presto/2.12.388 Version/12.15";
const char S11007[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5660)) Presto/2.12.388 Version/12.15";
const char S11008[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.15";
const char S11009[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.15";
const char S11010[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.15";
const char S11011[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6032)) Presto/2.12.388 Version/12.15";
const char S11012[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6044)) Presto/2.12.388 Version/12.15";
const char S11013[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5650)) Presto/2.12.388 Version/12.15";
const char S11014[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6014)) Presto/2.12.388 Version/12.15";
const char S11015[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.15,gzip(gfe) (via docs.google.com/viewer)";
const char S11016[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5230)) Presto/2.12.388 Version/12.15";
const char S11017[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5734)) Presto/2.12.388 Version/12.15";
const char S11018[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5988)) Presto/2.12.388 Version/12.15";
const char S11019[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4661)) Presto/2.12.388 Version/12.15";
const char S11020[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4832)) Presto/2.12.388 Version/12.15";
const char S11021[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6561)) Presto/2.12.388 Version/12.15";
const char S11022[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4110)) Presto/2.12.388 Version/12.15";
const char S11023[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5300)) Presto/2.12.388 Version/12.15";
const char S11024[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03639)) Presto/2.12.388 Version/12.15";
const char S11025[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6573)) Presto/2.12.388 Version/12.15";
const char S11026[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6261)) Presto/2.12.388 Version/12.15";
const char S11027[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6277)) Presto/2.12.388 Version/12.15";
const char S11028[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.15";
const char S11029[] = "Opera/9.80 (Windows NT 6.1; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.15";
const char S11030[] = "Opera/9.80 (Windows NT 6.1; MRA 5.6 (build 03402)) Presto/2.12.388 Version/12.15";
const char S11031[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03755)) Presto/2.12.388 Version/12.15";
const char S11032[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6283)) Presto/2.12.388 Version/12.15";
const char S11033[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03791)) Presto/2.12.388 Version/12.15";
const char S11034[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03672)) Presto/2.12.388 Version/12.15";
const char S11035[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6628)) Presto/2.12.388 Version/12.15";
const char S11036[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5336)) Presto/2.12.388 Version/12.15";
const char S11037[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03686); MRSPUTNIK OW 2, 3, 0, 244) Presto/2.12.388 Version/12.15";
const char S11038[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6638)) Presto/2.12.388 Version/12.15";
const char S11039[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6620)) Presto/2.12.388 Version/12.15";
const char S11040[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5161)) Presto/2.12.388 Version/12.15";
const char S11041[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03780)) Presto/2.12.388 Version/12.15";
const char S11042[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5325)) Presto/2.12.388 Version/12.15";
const char S11043[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.15";
const char S11044[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6323)) Presto/2.12.388 Version/12.15";
const char S11045[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.15";
const char S11046[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6322)) Presto/2.12.388 Version/12.15";
const char S11047[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6648)) Presto/2.12.388 Version/12.15";
const char S11048[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6325)) Presto/2.12.388 Version/12.15";
const char S11049[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6657)) Presto/2.12.388 Version/12.15";
const char S11050[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6320)) Presto/2.12.388 Version/12.15";
const char S11051[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6645)) Presto/2.12.388 Version/12.15";
const char S11052[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6327)) Presto/2.12.388 Version/12.15";
const char S11053[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6324)) Presto/2.12.388 Version/12.15";
const char S11054[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6665)) Presto/2.12.388 Version/12.15";
const char S11055[] = "Opera/9.80 (Windows NT 6.1; MRA 5.6 (build 03402); MRSPUTNIK OW 2, 3, 0, 104) Presto/2.12.388 Version/12.15";
const char S11056[] = "Opera/9.80 (Windows NT 6.1; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.15";
const char S11057[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6336)) Presto/2.12.388 Version/12.15";
const char S11058[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6666)) Presto/2.12.388 Version/12.15";
const char S11059[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6337)) Presto/2.12.388 Version/12.15";
const char S11060[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6669)) Presto/2.12.388 Version/12.15";
const char S11061[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6673)) Presto/2.12.388 Version/12.15";
const char S11062[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5730)) Presto/2.12.388 Version/12.15";
const char S11063[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6661)) Presto/2.12.388 Version/12.15";
const char S11064[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6667)) Presto/2.12.388 Version/12.15";
const char S11065[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6341)) Presto/2.12.388 Version/12.15";
const char S11066[] = "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.15,gzip(gfe),gzip(gfe)";
const char S11067[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.15,gzip(gfe),gzip(gfe)";
const char S11068[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6675)) Presto/2.12.388 Version/12.15";
const char S11069[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.15";
const char S11070[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6681)) Presto/2.12.388 Version/12.15";
const char S11071[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6676)) Presto/2.12.388 Version/12.15";
const char S11072[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6347)) Presto/2.12.388 Version/12.15";
const char S11073[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6668)) Presto/2.12.388 Version/12.15";
const char S11074[] = "Opera/9.80 (Windows NT 6.1; MRA 6.2 (build 7188)) Presto/2.12.388 Version/12.15";
const char S11075[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6682)) Presto/2.12.388 Version/12.15";
const char S11076[] = "Opera/9.80 (Windows NT 6.1; MRA 6.2 (build 7194)) Presto/2.12.388 Version/12.15";
const char S11077[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3 like Mac OS X; en_us) AppleWebKit/525.18.1 (KHTML, like Gecko )";
const char S11078[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3_1 like Mac OS X; fr-fr) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/7D11";
const char S11079[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S11080[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7";
const char S11081[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S11082[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H8";
const char S11083[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190";
const char S11084[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S11085[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F191";
const char S11086[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4";
const char S11087[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2";
const char S11088[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S11089[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3";
const char S11090[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/533.17.9";
const char S11091[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/528.16";
const char S11092[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S11093[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2";
const char S11094[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1";
const char S11095[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S11096[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/533.17.9";
const char S11097[] = "MQQBrowser/18 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3";
const char S11098[] = "MQQBrowser/18 Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3";
const char S11099[] = "MQQBrowser/1.9.1 Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3";
const char S11100[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190 Safari/528.16";
const char S11101[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F191 Safari/533.17.9";
const char S11102[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F192";
const char S11103[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 Safari/533.17.9";
const char S11104[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 Safari/533.17.9";
const char S11105[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 Safari/533.17.9";
const char S11106[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/528.16";
const char S11107[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 Safari/528.16";
const char S11108[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2 Safari/533.17.9";
const char S11109[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3_5 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.5;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]";
const char S11110[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3_2 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.2;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]";
const char S11111[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.1;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]";
const char S11112[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]";
const char S11113[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3_4 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.4;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]";
const char S11114[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H";
const char S11115[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K";
const char S11116[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L";
const char S11117[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H8 Safari/533.17.9";
const char S11118[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3_3 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.3;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]";
const char S11119[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190 Safari/533.17.9";
const char S11120[] = "MQQBrowser/1.9.1 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3";
const char S11121[] = "MQQBrowser/19 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3";
const char S11122[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2 Safari/528.16";
const char S11123[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 iPad2,1/K93AP Zite/1.3.3";
const char S11124[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 iPad2,1/K93AP Zite/1.3.3";
const char S11125[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 Safari/528.16";
const char S11126[] = "MQQBrowser/19 Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3";
const char S11127[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 iPad1,1/K48AP Zite/1.3.3";
const char S11128[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/9Z999";
const char S11129[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F191 iPad2,1/K93AP Zite/1.3.3";
const char S11130[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.2;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]";
const char S11131[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPad1,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]";
const char S11132[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.4;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]";
const char S11133[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 Safari/528.16";
const char S11134[] = "Mozilla/5.0 (iPad; CPU OS 4_3 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48 Instapaper/4.0.1 (+http://www.instapaper.com/iphone)";
const char S11135[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPad1,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.3;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]";
const char S11136[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.1;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]";
const char S11137[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 [FBAN/FBIOS;FBAV/5.4.2;FBBV/X;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.5;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]";
const char S11138[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 [FBAN/FBIOS;FBAV/5.4.2;FBBV/X;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.3;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]";
const char S11139[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F191 [FBAN/FBIOS;FBAV/5.4.2;FBBV/X;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]";
const char S11140[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H8 [FBAN/FBIOS;FBAV/5.4.2;FBBV/X;FBDV/iPad2,3;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.2;FBSS/1; FBCR/Verizon;FBID/tablet;FBLC/en_US]";
const char S11141[] = "Mozilla/5.0 (iPad; CPU OS 4_3_5 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176";
const char S11142[] = "Mozilla/5.0 (iPad; CPU OS 4_3_3 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176";
const char S11143[] = "Mozilla/5.0 (iPad; U; CPU OS 433 like Mac OS X; en-US) AppleWebKit/533.17.9 (KHTML like Gecko) Mobile/8J2";
const char S11144[] = "Mozilla/5.0 (iPad; U; CPU OS 435 like Mac OS X; en-US) AppleWebKit/533.17.9 (KHTML like Gecko) Mobile/8L1";
const char S11145[] = "AppEngine-Google; (+http://code.google.com/appengine; appid: s~getfavicon27)";
const char S11146[] = "AppEngine-Google; (+http://code.google.com/appengine; appid: tmobile-internet)";
const char S11147[] = "Mozilla/5.0 (compatible; GoogleDocs; script; +http://docs.google.com)";
const char S11148[] = "AppEngine-Google; (+http://code.google.com/appengine; appid: tmobile-w4z)";
const char S11149[] = "Public Hub (+http://pubsubhubbub.appspot.com; 1 subscribers) AppEngine-Google; (+http://code.google.com/appengine; appid: pubsubhubbub)";
const char S11150[] = "AppEngine-Google; (+http://code.google.com/appengine; appid: vi-mobile)";
const char S11151[] = "AppEngine-Google; (+http://code.google.com/appengine; appid: s~google.com:mobile-checker)";
const char S11152[] = "GoogleProducer; (+http://goo.gl/7y4SX)";
const char S11153[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S11154[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S11155[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S11156[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S11157[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S11158[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S11159[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S11160[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S11161[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.26.9 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S11162[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S11163[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S11164[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S11165[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S11166[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S11167[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S11168[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S11169[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S11170[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S11171[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S11172[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S11173[] = "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.11";
const char S11174[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.11";
const char S11175[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.11";
const char S11176[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.11";
const char S11177[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.11";
const char S11178[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.11";
const char S11179[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.11";
const char S11180[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.11";
const char S11181[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.11";
const char S11182[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64) Opera 12.11";
const char S11183[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.11";
const char S11184[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.11";
const char S11185[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.11";
const char S11186[] = "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.11,gzip(gfe)";
const char S11187[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.11";
const char S11188[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.11";
const char S11189[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.11";
const char S11190[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.11";
const char S11191[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.11";
const char S11192[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.11,gzip(gfe)";
const char S11193[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.11";
const char S11194[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.11";
const char S11195[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.11";
const char S11196[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.11";
const char S11197[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.11";
const char S11198[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.11";
const char S11199[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.11";
const char S11200[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.11";
const char S11201[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.11";
const char S11202[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.11";
const char S11203[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.11";
const char S11204[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.11";
const char S11205[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.11";
const char S11206[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.11";
const char S11207[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.11";
const char S11208[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.11";
const char S11209[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.11";
const char S11210[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.11";
const char S11211[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.11";
const char S11212[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.11";
const char S11213[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.11";
const char S11214[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.11";
const char S11215[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.11";
const char S11216[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.11";
const char S11217[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.11";
const char S11218[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.11";
const char S11219[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.11";
const char S11220[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.11";
const char S11221[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.11";
const char S11222[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.11";
const char S11223[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.11";
const char S11224[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4933)) Presto/2.12.388 Version/12.11";
const char S11225[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.11";
const char S11226[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.11";
const char S11227[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.11";
const char S11228[] = "Opera/9.80 (Windows NT 6.1; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.11";
const char S11229[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.11";
const char S11230[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03639)) Presto/2.12.388 Version/12.11";
const char S11231[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.11";
const char S11232[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5217)) Presto/2.12.388 Version/12.11";
const char S11233[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.11 Yandex.Translate";
const char S11234[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03658)) Presto/2.12.388 Version/12.11";
const char S11235[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5288)) Presto/2.12.388 Version/12.11";
const char S11236[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5650)) Presto/2.12.388 Version/12.11";
const char S11237[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5299)) Presto/2.12.388 Version/12.11";
const char S11238[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5300)) Presto/2.12.388 Version/12.11";
const char S11239[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5734)) Presto/2.12.388 Version/12.11";
const char S11240[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03789)) Presto/2.12.388 Version/12.11";
const char S11241[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.11";
const char S11242[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.11";
const char S11243[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5230)) Presto/2.12.388 Version/12.11";
const char S11244[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.11";
const char S11245[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.11";
const char S11246[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.11";
const char S11247[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4110)) Presto/2.12.388 Version/12.11";
const char S11248[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4832)) Presto/2.12.388 Version/12.11";
const char S11249[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4661)) Presto/2.12.388 Version/12.11";
const char S11250[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.11";
const char S11251[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5336)) Presto/2.12.388 Version/12.11";
const char S11252[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.11";
const char S11253[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.11";
const char S11254[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03780)) Presto/2.12.388 Version/12.11";
const char S11255[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03791)) Presto/2.12.388 Version/12.11";
const char S11256[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.11";
const char S11257[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.11";
const char S11258[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.11";
const char S11259[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.11";
const char S11260[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.11";
const char S11261[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.11";
const char S11262[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5660)) Presto/2.12.388 Version/12.11";
const char S11263[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.11";
const char S11264[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5278)) Presto/2.12.388 Version/12.11";
const char S11265[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.11";
const char S11266[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.11";
const char S11267[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.11";
const char S11268[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.11";
const char S11269[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.11";
const char S11270[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.11";
const char S11271[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03686); MRSPUTNIK OW 2, 3, 0, 244) Presto/2.12.388 Version/12.11";
const char S11272[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.11";
const char S11273[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.11";
const char S11274[] = "Opera/9.80 (Windows NT 6.1; MRA 5.6 (build 03402)) Presto/2.12.388 Version/12.11";
const char S11275[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03755)) Presto/2.12.388 Version/12.11";
const char S11276[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03672)) Presto/2.12.388 Version/12.11";
const char S11277[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 198)) Presto/2.12.388 Version/12.11";
const char S11278[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6080)) Presto/2.12.388 Version/12.11";
const char S11279[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.11";
const char S11280[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.11";
const char S11281[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6638)) Presto/2.12.388 Version/12.11";
const char S11282[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6657)) Presto/2.12.388 Version/12.11";
const char S11283[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.11";
const char S11284[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6673)) Presto/2.12.388 Version/12.11";
const char S11285[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.11";
const char S11286[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6337)) Presto/2.12.388 Version/12.11";
const char S11287[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6046)) Presto/2.12.388 Version/12.11";
const char S11288[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6573)) Presto/2.12.388 Version/12.11";
const char S11289[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6628)) Presto/2.12.388 Version/12.11";
const char S11290[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-N7000 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30";
const char S11291[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; GT-N7000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.1 Mobile Safari/534.30";
const char S11292[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-N7000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11293[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-N7000 Build/IML74K)";
const char S11294[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T879 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11295[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11296[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; SHV-E160S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11297[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; SHV-E160K Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11298[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; SHV-E160L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11299[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T879 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11300[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; SAMSUNG GT-N7000/N7000XXLQ3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11301[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; GT-N7000B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11302[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-N7000 Build/IMM76D)";
const char S11303[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; N7000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11304[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-N7000 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11305[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-N7000 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S11306[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SHV-E160K Build/IML74K)";
const char S11307[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SHV-E160L Build/IML74K)";
const char S11308[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SHV-E160S Build/IML74K)";
const char S11309[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHW-M480K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S11310[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; SAMSUNG GT-N7000/N7000BULP3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11311[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SAMSUNG GT-N7000/N7000XXLRI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11312[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; GT-N7000B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11313[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E160S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11314[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-N7000 Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S11315[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-N7000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11316[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-N7000 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S11317[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHW-M480W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S11318[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SCH-i889 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11319[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; N7000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11320[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-T879 Build/IMM76D)";
const char S11321[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E160K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11322[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SC-05D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11323[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHW-M480S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S11324[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11325[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHV-E160S Build/IMM76D)";
const char S11326[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E160L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11327[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30";
const char S11328[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N7000+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11329[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-N7000B Build/IMM76D)";
const char S11330[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHV-E160K Build/IMM76D)";
const char S11331[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-N7000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11332[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHV-E160L Build/IMM76D)";
const char S11333[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SAMSUNG GT-N7000/N7000XXLRU Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11334[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-ch; SAMSUNG GT-N7000/N7000BULR1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11335[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; N7000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11336[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-N7000B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11337[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SC-05D Build/IMM76D)";
const char S11338[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SCH-i889 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S11339[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHW-M480W Build/IMM76D)";
const char S11340[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; vi-vn; N7000 Build/DragonFly_V006EnX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11341[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; vi-vn; N7000 Build/DragonFly_V005EnX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11342[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-N7000 Build/RocketMIUI v8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S11343[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; N7000 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11344[] = "Opera/9.80 (Android; Opera Mini/6.5.27452/26.1305; U; en) Presto/2.8.119 Version/10.54";
const char S11345[] = "Opera/9.80 (Android; Opera Mini/6.5.27452/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S11346[] = "Opera/9.80 (Android; Opera Mini/6.5.29441/27.2314; U; en) Presto/2.8.119 480x800 Android";
const char S11347[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)";
const char S11348[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S11349[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.1.3.0 Safari/537.31";
const char S11350[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S11351[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe)";
const char S11352[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S11353[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S11354[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.2.2.0 Safari/537.31";
const char S11355[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.2.2.0 Safari/537.31";
const char S11356[] = "Mozilla/5.0 (Windows NT 5.1; en-US) AppleWebKit/535.12 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/535.12";
const char S11357[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe)";
const char S11358[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.2.2.0 Safari/537.31";
const char S11359[] = "Mozilla/5.0 (Windows NT 5.1; zh-CN) AppleWebKit/535.12 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/535.12";
const char S11360[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)";
const char S11361[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S11362[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S11363[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S11364[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S11365[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S11366[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31,gzip(gfe)";
const char S11367[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31,gzip(gfe)";
const char S11368[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)";
const char S11369[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.31 (KHTML like Gecko) Chrome/26.0.1410.63 Safari/537.31";
const char S11370[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S11371[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50";
const char S11372[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50";
const char S11373[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22";
const char S11374[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22";
const char S11375[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16";
const char S11376[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16";
const char S11377[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3";
const char S11378[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3";
const char S11379[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2";
const char S11380[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2";
const char S11381[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7";
const char S11382[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7";
const char S11383[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-us) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1 Safari/534.52.7";
const char S11384[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-us) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.6 Safari/534.57.2";
const char S11385[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-us) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50";
const char S11386[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-us) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1 Safari/534.51.22";
const char S11387[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-us) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3";
const char S11388[] = "Mozilla/5.0 (Windows; Windows NT 6.1) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2";
const char S11389[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.3 Safari/534.53.10";
const char S11390[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.3 Safari/534.53.10";
const char S11391[] = "Superfeedr: Superparser bot/1.1 http://superfeedr.com - Please read this http://blog.superfeedr.com/publishers.html or get in touch if we are polling too hard";
const char S11392[] = "Mozilla/4.0 (UNIX; I; amd64) DynoBot/0.8.2528";
const char S11393[] = "Mozilla/5.0 (compatible; Dataprovider Site Explorer; http://www.dataprovider.com/)";
const char S11394[] = "UniversalFeedParser/5.1 +https://code.google.com/p/feedparser/";
const char S11395[] = "UniversalFeedParser/5.1.2 +https://code.google.com/p/feedparser/";
const char S11396[] = "Xenu Link Sleuth/1.3.8 AppEngine-Google; (+http://code.google.com/appengine; appid: s~runningcheese01)";
const char S11397[] = "Mozilla/5.0 (compatible; DynamicTemplates/2.0; http://www.sli-systems.com/) Gecko";
const char S11398[] = "Superfeedr bot/2.0 http://superfeedr.com - Make your feeds realtime: get in touch!";
const char S11399[] = "GG PeekBot 2.0 ( http://gg.pl/ http://info.gadu-gadu.pl/praca )";
const char S11400[] = "Ara.com.tr / Bot 1.0";
const char S11401[] = "PollBot v1.2 (http://besome1.info/robots.html)";
const char S11402[] = "Mozilla/5.0 (compatible; linkdexbot/2.0; +http://www.linkdex.com/about/bots/)";
const char S11403[] = "Mozilla/xintellibot";
const char S11404[] = "Mozilla/5.0 (compatible; scanbot/1.0; +http://dazzlepod.com/ip/)";
const char S11405[] = "MaxPoint Bot (+http://www.maxpoint.com)";
const char S11406[] = "Sogou web spider/4.0?iVFA";
const char S11407[] = "Mozilla/5.0 (compatible; SafeLinks Bot; http://safelinkshq.com)";
const char S11408[] = "Nitidum Tarsier/1.0 (+http://www.nitidum.com/bot)";
const char S11409[] = "DistSearchBot/dev (http://www.tkolb.de/distsearch.txt)";
const char S11410[] = "Screaming Frog SEO Spider/2.20";
const char S11411[] = "Mozilla/5.0 (compatible; WebDataCentreBot/1.0; +http://WebDataCentre.com/;c2VvLWNyYXdsZXI=";
const char S11412[] = "Screaming Frog SEO Spider/2,20";
const char S11413[] = "Mozilla/5.0 (compatible; DonkeyBot/0.3a; +http://www.dasdonkey.com)";
const char S11414[] = "Mozilla/5.0 (compatible; SeznamBot/3.1-test1; +http://fulltext.sblog.cz/)";
const char S11415[] = "EhName Domain Registry Protection Bot";
const char S11416[] = "NCBot (http://netcomber.com?st=ba : tool for finding true domain owners) Queries/complaints: bot@netcomber.com";
const char S11417[] = "SocialOomphBot/1.0 (info@socialoomph.com http://socialoomph.com/)";
const char S11418[] = "Trackybot/0.1 (+https://tracky.com/bot.html)";
const char S11419[] = "Mozilla/5.0 (compatible; Exabot/3.0;  http://www.exabot.com/go/robot)";
const char S11420[] = "OpenWebSpider/0.7 (http://www.openwebspider.org)";
const char S11421[] = "Mozilla/5.0 (compatible; Siteliner/1.0; +http://www.siteliner.com/bot)";
const char S11422[] = "AppEngine-Google; (+http://code.google.com/appengine; appid: s~socialmediacrawler)";
const char S11423[] = "Mozilla/5.0 (compatible; YandexMetrika/2.0; +http://yandex.com/bots mon2)";
const char S11424[] = "Mozilla/5.0 (compatible; YandexMetrika/2.0; +http://yandex.com/bots mon1)";
const char S11425[] = "Clickagy Ad Intelligence Bot";
const char S11426[] = "datagnionbot (+http://www.datagnion.com/bot.html)";
const char S11427[] = "Mozilla/5.0 (compatible; Ephorusbot/undefined; +http://www.ephorus.com/)";
const char S11428[] = "Mozilla/5.0 (compatible; PerfectMarketKWTbot/1.0 +http://www.perfectmarket.com)";
const char S11429[] = "Mozilla/5.0 (compatible; equellaurlbot/1.0; +http://support.equella.com/)";
const char S11430[] = "A1 Website Download/4.2.4 (+http://www.microsystools.com/products/website-download/) miggibot";
const char S11431[] = "CatchBot/2.0; +http://www.catchbot.com;c2VvLWNyYXdsZXI=";
const char S11432[] = "Mozilla/5.0 (compatible; seexie.com_bot/4.1; +http://www.seexie.com);c2VvLWNyYXdsZXI=";
const char S11433[] = "CatchBot/1.0; +http://www.catchbot.com;c2VvLWNyYXdsZXI=";
const char S11434[] = "Tokenizer/1.1.19 (Shopping Search Engine; http://www.tokenizer.org/bot.html; agent@tokenizer.org);c2VvLWNyYXdsZXI=";
const char S11435[] = "Snapbot/1.0 (Snap Shots, +http://www.snap.com);c2VvLWNyYXdsZXI=";
const char S11436[] = "PageBitesHyperBot/600 (http://www.pagebites.com/);c2VvLWNyYXdsZXI=";
const char S11437[] = "finbot";
const char S11438[] = "Mozilla/5.0 (compatible; lookingforitbot/v1.0; http://www.lookingforit.net/bot.php?+)";
const char S11439[] = "Abyssbot";
const char S11440[] = "Mozilla/5.0 (compatible; DCPbot/1.5; +http://domains.checkparams.com/)";
const char S11441[] = "Kzjnetbot/Kzjnetbot-0.1 (http://search.kzjnet.com; search@kzjnet.com)";
const char S11442[] = "SupremeSearch.net Spider-Bot/5.4 (http://www.supremesearch.net)";
const char S11443[] = "NCBot http://netcomber.com?st=ba2Tool for finding all their domain names.";
const char S11444[] = "ht2p-bot/v1";
const char S11445[] = "EhName Domain Portfolio Bot http://ehname.com/DOMAIN?st=ba robot@ehname.com";
const char S11446[] = "Mozilla/5.0 (compatible; SemrushBot/0.96.4; +http://www.semrush.com/bot.html)";
const char S11447[] = "NCBot (http://netcomber.com/DOMAIN?st=ba : tool for finding true domain owners) Queries/complaints: bot@netcomber.com";
const char S11448[] = "ManziaBot";
const char S11449[] = "TitleBot-1.0";
const char S11450[] = "NCBot http://netcomber.com?st=ba2 Tool for finding all their domain names.";
const char S11451[] = "ContextAd Bot 1.0";
const char S11452[] = "Mozilla/5.0 (compatible; MojeekBot/0.5; http://www.mojeek.com/bot.html)";
const char S11453[] = "SupremeSearch.net Spider-Bot/5.5 (http://www.supremesearch.net)";
const char S11454[] = "Test spider";
const char S11455[] = "Mozilla/5.0 (compatible; openstat.ru/Bot)";
const char S11456[] = "Spider/Bot";
const char S11457[] = "Sogou web spider/4.02525A";
const char S11458[] = "Sogou web spider/4.05252A";
const char S11459[] = "semetrical (http://www.semetrical.com/bot.html)";
const char S11460[] = "Mozilla/5.0 (compatible; seexie.com_bot/4.1; +http://seexie.com)";
const char S11461[] = "GozaikBot (www.gozaik.com;webmaster@gozaik.com;www.gozaik.com/gozaikbot.html)";
const char S11462[] = "Mozilla/5.0 (compatible; YodaoBot/1.0; http://www.yodao.com/help/webmaster/spider/;);c2VvLWNyYXdsZXI=";
const char S11463[] = "Baiduspider+(+http://www.baidu.jp/spider/);c2VvLWNyYXdsZXI=";
const char S11464[] = "Open Web Analytics Bot 1.2.4;c2VvLWNyYXdsZXI=";
const char S11465[] = "Mozilla/5.0 (compatible; discobot/1.1; +http://discoveryengine.com/discobot.html;c2VvLWNyYXdsZXI=";
const char S11466[] = "Mozilla/5.0 (compatible; 008/0.83; http://www.80legs.com/spider.html;) Gecko/X;c2VvLWNyYXdsZXI=";
const char S11467[] = "lobot/0.3 (http://www.lofoso.com/robot.html);c2VvLWNyYXdsZXI=";
const char S11468[] = "CCBot/1.0 (+http://www.commoncrawl.org/bot.html);c2VvLWNyYXdsZXI=";
const char S11469[] = "MoFuseBot (Feed Parser; http://bot.mofuse.com/; Allow like Gecko);c2VvLWNyYXdsZXI=";
const char S11470[] = "CatchBot/1.0; http://www.catchbot.com;c2VvLWNyYXdsZXI=";
const char S11471[] = "TrueKnowledgeBot (http://www.trueknowledge.com/tkbot/; tkbot -AT- trueknowledge _dot_ com);c2VvLWNyYXdsZXI=";
const char S11472[] = "Mozilla/5.0 (compatible; Baiduspider/2.0; +http://www.baidu.com/search/spider.html);c2VvLWNyYXdsZXI=";
const char S11473[] = "[('User-agent', 'Mozilla/5.0 (compatible; ZumBot/1.0; http://help.zum.com/inquiry)')]";
const char S11474[] = "ElisaBot 1.0 [u-f-g-table]";
const char S11475[] = "Mozilla/5.0 (compatible; SemrushBot/0.97; +http://www.semrush.com/bot.html)";
const char S11476[] = "Mozilla/5.0 (compatible; YandexMetrika/3.0; +http://yandex.com/bots)";
const char S11477[] = "Contensive-Bot/3.0.150";
const char S11478[] = "Mozilla/5.0 (compatible; archive-nz.com/1.1; +http://archive-nz.com/bot)";
const char S11479[] = "Mozilla/5.0 (compatible; ReplazBot/2.5; +http://www.replaz.com)";
const char S11480[] = "Wepbot/3.0 (+http://www.nurago.com/index.php?id=210); Jakarta Commons-HttpClient/3.1;c2VvLWNyYXdsZXI=";
const char S11481[] = "Nekstbot - http://www.ipipan.waw.pl/nekst/nekstbot/";
const char S11482[] = "SolomonoBot/1.01 (http://www.solomono.ru);c2VvLWNyYXdsZXI=";
const char S11483[] = "msnbot/1.1 (+http://search.msn.com/msnbot.htm);c2VvLWNyYXdsZXI=";
const char S11484[] = "EvaalSE - bot@evaal.com;c2VvLWNyYXdsZXI=";
const char S11485[] = "Mozilla/5.0 (compatible; Purebot/1.1; +http://www.puritysearch.net/);c2VvLWNyYXdsZXI=";
const char S11486[] = "Checkbot/1.79 LWP/5.813;c2VvLWNyYXdsZXI=";
const char S11487[] = "msnbot/2.0b (+http://search.msn.com/msnbot.htm);c2VvLWNyYXdsZXI=";
const char S11488[] = "Mozilla/5.0 (compatible; BuiltWith/0.3; +http://builtwith.com/bot.html);c2VvLWNyYXdsZXI=";
const char S11489[] = "searchdotsh/Search.sh-2.2.1 (search.sh spider ver.0.5; http://search.sh/; webmaster at search dot sh)";
const char S11490[] = "Mozilla/5.0 (compatible; YaDirectFetcher/1.0; +http://yandex.com/bots)";
const char S11491[] = "Mozilla/4.0 (compatible; WMHelper/1.0; +http://www.wmhelper.com/bot.html)";
const char S11492[] = "searchdotsh/0.5.0 (search.sh spider ver.0.5; http://search.sh/; webmaster at search dot sh)";
const char S11493[] = "Mozilla/5.0 (compatible; YandexBot/3.0)";
const char S11494[] = "Mozilla/5.0 (compatible; seospyder/1.0; +http://www.mobiliodevelopment.com/seospyder/)";
const char S11495[] = "Cliqz Bot (+http://www.cliqz.com)";
const char S11496[] = "Mozilla/5.0 (compatible; spbot/4.0a; +http://www.seoprofiler.com/bot )";
const char S11497[] = "Mozilla/5.0 (compatible; uMBot-IC/1.0; mailto: crawling@ubermetrics-technologies.com)";
const char S11498[] = "mediawords bot (http://cyber.law.harvard.edu)";
const char S11499[] = "gigabot";
const char S11500[] = "Gaisbot/3.0 (robot@gais.cs.ccu.edu.tw; http://gais.cs.ccu.edu.tw/robot.php)";
const char S11501[] = "ElisaBot 1.0 unforgettable";
const char S11502[] = "Josh Robot/2.0 (http://www.joshtsp.com; tim@timgill.com)";
const char S11503[] = "Cliqzbot/0.1 (+http://cliqz.com +cliqzbot@cliqz.com)";
const char S11504[] = "Mozilla/5.0 (compatible; Please Name Your robot; +http://broadcart.com/websearch/bot.php)";
const char S11505[] = "HTTP/1.0 Mozilla/5.0+(compatible;+GetInfo.us+Bot;+http://getinfo.us/bot)";
const char S11506[] = "Gulper Web Bot 0.2.4 (www.ecsl.cs.sunysb.edu/~maxim/cgi-bin/Link/GulperBot)";
const char S11507[] = "Halebot (Mozilla/5.0 compatible; Halebot/2.1; http://www.tacitknowledge.com/halebot.shtml)";
const char S11508[] = "WebZIP/3.5 (http://www.spidersoft.com)";
const char S11509[] = "Mozilla/5.0 (compatible; ReplazBot/2.6; +http://www.replaz.com)";
const char S11510[] = "Mozilla/5.0 (compatible; spbot/4.0b; +http://www.seoprofiler.com/bot )";
const char S11511[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5";
const char S11512[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S11513[] = "Mozilla/5.0 (; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S11514[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9Z999 Safari/6533.18.5";
const char S11515[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C99 Safari/6533.18.5";
const char S11516[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5";
const char S11517[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2 like Mac OS X; nl-nl) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 Safari/6533.18.5";
const char S11518[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8E200 Safari/6533.18.5";
const char S11519[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8E128 Safari/6533.18.5";
const char S11520[] = "Mozilla/5.0 (iPhone; U; CPU OS 4_2_1 like Mac OS X) AppleWebKit/532.9 (KHTML, like Gecko) Version/5.0.3 Mobile/8B5097d Safari/6531.22.7";
const char S11521[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_7 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8E303 Safari/6533.18.5";
const char S11522[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9Z999 Safari/6533.18.5";
const char S11523[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8E401 Safari/6533.18.5";
const char S11524[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_9 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8E501 Safari/6533.18.5";
const char S11525[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_10 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8E600 Safari/6533.18.5";
const char S11526[] = "Mozilla/5.0 (device; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S11527[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; 6533.17.9) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5";
const char S11528[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en_US) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5";
const char S11529[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9Z999 Safari/6533.18.5";
const char S11530[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en_us) AppleWebKit/6533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5";
const char S11531[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; vi-vn) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/PV001 Safari/6533.18.5";
const char S11532[] = "Mozilla/5.0 (; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8E200 Safari/6533.18.5";
const char S11533[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en_us) AppleWebKit/6533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5";
const char S11534[] = "Mozilla/5.0 (iPhone; CPU OS 4_2_1 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0 Mobile/8C148 Safari/7533.17.9";
const char S11535[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en_us) AppleWebKit/6533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5";
const char S11536[] = "Mozilla/5.0 (iPhone Simulator; U; CPU iPhone OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C134 Safari/6533.18.5";
const char S11537[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 421 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S11538[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 426 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8E200 Safari/6533.18.5";
const char S11539[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 421 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5";
const char S11540[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 427 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8E303 Safari/6533.18.5";
const char S11541[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.163 Safari/535.1";
const char S11542[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S11543[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1,gzip(gfe) (via docs.google.com/viewer)";
const char S11544[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1,gzip(gfe) (via docs.google.com/viewer)";
const char S11545[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.802.30 Safari/535.1 SE 2.X MetaSr 1.0,gzip(gfe) (via translate.google.com)";
const char S11546[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.802.30 Safari/535.1 SE 2.X MetaSr 1.0,gzip(gfe) (via translate.google.com)";
const char S11547[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.94 Safari/535.1";
const char S11548[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535 (KHTML, like Gecko) Chrome/14 Safari/535";
const char S11549[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.802.30 Safari/535.1 SE 2.X MetaSr 1.0,gzip(gfe)";
const char S11550[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.802.30 Safari/535.1 SE 2.X MetaSr 1.0,gzip(gfe)";
const char S11551[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1";
const char S11552[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML	 like Gecko) Chrome/14.0.802.30 Safari/535.1 SE 2.X MetaSr 1.0";
const char S11553[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML like Gecko) Chrome/14.0.811.0 Safari/535.1";
const char S11554[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML like Gecko) Chrome/14.0.812.0 Safari/535.1";
const char S11555[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Desire_A8181 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11556[] = "Mozilla/5.0 (Linux; U; Android Android 2.2; en-gb; Desire_A8181 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11557[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC_Desire_A8181 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11558[] = "Mozilla/5.0 (Linux; U; Android 2.2; de-at; HTC Desire 1.34.166.5 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11559[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Desire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11560[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11561[] = "Mozilla/5.0 (Linux; U; Android 2.2; nl-nl; Desire_A8181 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11562[] = "Mozilla/5.0 (Linux; U; Android 2.2; Desire_A8181 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11563[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC_Desire-orange-LS Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11564[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTCA8180/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11565[] = "Mozilla/5.0 (Linux; U; Android 2.2; de-at; HTC Desire 2.33.163.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11566[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Desire 1.19.161.5 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11567[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Desire 2.10.161.2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11568[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Bravo Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11569[] = "Mozilla/5.0 (Linux; U; Android 2.2; de-at; HTC Desire 2.10.163.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11570[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC_Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11571[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-jp; X06HT Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11572[] = "Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; HTCX06HT Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11573[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-au; HTC_Desire_A8183 V1.16.841.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11574[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-au; HTC_Desire_A8183 V1.25.841.5 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11575[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; HTC_Desire_A8183 V2.26.841.2 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11576[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-au; HTC_Desire_A8183 V2.26.841.2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11577[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; Desire_A8181 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11578[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; HTC Desire Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11579[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; HTC Bravo_C Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11580[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC_Desire_A8181 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11581[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Desire 2.33.161.2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11582[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-hk; HTC_Desire_A8181 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11583[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Desire Build/FRF91) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-20101109)";
const char S11584[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; HTC Desire Build/FRF91)";
const char S11585[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11586[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Desire CDMA Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11587[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Desire Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11588[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC bravo Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11589[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; de-at; HTC Desire 2.33.163.1 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11590[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; HTC Desire 2.33.161.2 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11591[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; HTC Desire Build/FRG83G)";
const char S11592[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; HTC_Desire_A8181 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11593[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11594[] = "Mozilla/5.0 (Linux; U; Android Android 2.2; en-gb; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11595[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Desire Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S11596[] = "Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; Desire_A8181 Build/FRF91) App3leWebKit/53.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11597[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; HTC Desire 2.33.161.6 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11598[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; HTC Bravo Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11599[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; HTC_Desire Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11600[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; de-at; HTC Desire 2.33.163.5 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11601[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; HTC Desire Build/FRG83D)";
const char S11602[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Desire Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S11603[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; X06HT Build/FRF91)";
const char S11604[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; HTC Desire Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11605[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; HTC Desire CDMA Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11606[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-; HTC Desire Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11607[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HTC Desire Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S11608[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; nl-nl; HTC Desire 2.33.161.8 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S11609[] = "Mozilla/5.0 (Android 2.2; zh-cn; HTC Desire)/GoBrowser";
const char S11610[] = "Mozilla/5.0 (compatible; Googlebot/2.1; http://www.google.com/bot.html)";
const char S11611[] = "Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)";
const char S11612[] = "googlebot";
const char S11613[] = "Mozilla/5.0 (compatible; Googlebot/2.1;  http://www.google.com/bot.html)";
const char S11614[] = "Googlebot/2.1 (+http://www.google.com/bot.html)";
const char S11615[] = "Googlebot-Image/1.0";
const char S11616[] = "Mozilla/5.0+(compatible;+Googlebot/2.1;++http://www.google.com/bot.html)";
const char S11617[] = "Googlebot/2.1 (+http://www.googlebot.com/bot.html)";
const char S11618[] = "Googlebot/2.1 (http://www.googlebot.com/bot.html)";
const char S11619[] = "Googlebot-Image/1.0,gzip(gfe) (via docs.google.com/viewer)";
const char S11620[] = "Mozilla/4.0 (compatible; gallery_203.nasl; Googlebot)";
const char S11621[] = "Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html),gzip(gfe) (via docs.google.com/viewer)";
const char S11622[] = "Googlebot-Video/1.0";
const char S11623[] = "Googlebot-richsnippets";
const char S11624[] = "Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html) ASProxy/5.5b5";
const char S11625[] = "Googlebot-Image/1.0 AppEngine CirruxCache CoralWebPrxGooglebot-Image/1.0,gzip(gfe) (via docs.google.com/viewer)";
const char S11626[] = "Mozilla/5.0 (compatible; Googlebot/2.1; compatible; Googlebot/2.0; +http://www.google.com/bot.html)";
const char S11627[] = "Googlebot/2.1";
const char S11628[] = "Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html),gzip(gfe) (via translate.google.com)";
const char S11629[] = "Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html) AppEngine CirruxCache CoralWebPrxMozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html),gzip(gfe) (via docs.google.com/viewer)";
const char S11630[] = "Googlebot-News";
const char S11631[] = "?Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)";
const char S11632[] = "Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html) ASProxy/5.5b4";
const char S11633[] = "Googlebot/Googlebot 1.0 (GoogleBot; www.google.com; admin@google.com)";
const char S11634[] = "Googlebot/2.1 ( http://www.googlebot.com/bot.html)";
const char S11635[] = "Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html) [UsableNet Lift Mobile]";
const char S11636[] = "Googlebot-Image/1.0 ( http://www.googlebot.com/bot.html)";
const char S11637[] = "Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html";
const char S11638[] = "Googlebot/2.1+(+http://www.google.com/bot.html)";
const char S11639[] = "Googlebot/2.1 ( http://www.google.com/bot.html)";
const char S11640[] = "http://www.googlebot.com/bot.html";
const char S11641[] = "Mozilla (compatible; Mediapartners-Google/2.1; +http://www.google.com/bot.html)";
const char S11642[] = "Mozilla/4.0 (MobilePhone MM-8300/US/1.0) NetFront/3.1 MMP/2.0 (compatible; Mediapartners-Google/2.1; +http://www.google.com/bot.html)";
const char S11643[] = "Mozilla/5.0 (compatible; Googlebot/2.0; +http://www.google.com/bot.html)";
const char S11644[] = "Mozilla/5.0 (compatible; googlebot/2.1;";
const char S11645[] = "Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html) ASProxy/5.5b3";
const char S11646[] = "Mozilla/4.0 (compatible; ScanAlert; Googlebot)";
const char S11647[] = "Googlebot/1.0 (googlebot@googlebot.com http://googlebot.com/)";
const char S11648[] = "Googlebot 1.0";
const char S11649[] = "Googlebot-Image/1.0 ASProxy/5.5b5";
const char S11650[] = "Locubot (compatible; Googlebot; msnbot)";
const char S11651[] = "Googlebot 2.1";
const char S11652[] = "Googlebot/2.1 (+http://www.google.com/bot.html";
const char S11653[] = "Mozilla/5.0 (compatible; Googlebot/2.1;http://www.google.com/bot.html)";
const char S11654[] = "Mozilla (compatible; Mediapartners-Googlebot/2.1;(+http://www.google.com/bot.html)";
const char S11655[] = "Mozilla/4.0 (compatible; +Baiduspider/2.0;++http://www.baidu.com/search/spider.html +Googlebot/2.1;++http://www.google.com/bot.html)";
const char S11656[] = "Mozilla/5.0 (compatible; Googlebot/2.1;+http://www.google.com/bot.html)";
const char S11657[] = "Mozila/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)";
const char S11658[] = "Googlebot/2.1(+http://www.googlebot.com/bot.html)";
const char S11659[] = "Googlebot/2.1; +http://www.google.com/bot.html)";
const char S11660[] = "googlebot/bot.htm";
const char S11661[] = "Googlebot/2.1 (http://www.google.com/bot.html)";
const char S11662[] = "GoogleBot (http://www.google.com)";
const char S11663[] = "Mozilla/5.0 (compatible; Googlebot/2.1; http://www.deepcrawl.co.uk/bot.html)";
const char S11664[] = "* Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)";
const char S11665[] = "Mozilla/5.0 (compatible; GoogleDocs GoogleApps; script; +http://script.google.com/bot.html)";
const char S11666[] = "Mozilla/5.0 (compatible; Googlebot/2.1)";
const char S11667[] = "Googlebot/2.1(+http://www.google.com/bot.html)";
const char S11668[] = "Mozilla/5.0 (compatible; GoogleDocs GoogleApps script; +http://script.google.com/bot.html)";
const char S11669[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B5105c Safari/8536.25";
const char S11670[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B114 Safari/8536.25";
const char S11671[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B5117b Safari/8536.25";
const char S11672[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B5126b Safari/8536.25";
const char S11673[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S11674[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B137 Safari/8536.25";
const char S11675[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B135 Safari/8536.25";
const char S11676[] = "Mozilla/5.0 (iPad; CPU OS6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S11677[] = "Mozilla/5.0 (iPad; CPU OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B311 Safari/8536.25";
const char S11678[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B147 Safari/8536.25";
const char S11679[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141Safari/8536.25";
const char S11680[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146 Safari/8536.25";
const char S11681[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B318a Safari/8536.25";
const char S11682[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/9Z999 Safari/8536.25";
const char S11683[] = "Mozilla/5.0 (iPad; CPU OS6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146 Safari/8536.25";
const char S11684[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146Safari/8536.25";
const char S11685[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/9Z999 Safari/8536.25";
const char S11686[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329 Safari/8536.25";
const char S11687[] = "Mozilla/5.0 (iPad; CPU OS6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329 Safari/8536.25";
const char S11688[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329Safari/8536.25";
const char S11689[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329 Safari/8536.25";
const char S11690[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146 Safari/8536.25";
const char S11691[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B136 Safari/8536.25";
const char S11692[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B318 Safari/8536.25";
const char S11693[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S11694[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S11695[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A406 Safari/8536.25";
const char S11696[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A406 Safari/8536.25";
const char S11697[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A406 Safari/8536.25";
const char S11698[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S11699[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S11700[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11";
const char S11701[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11";
const char S11702[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11,gzip(gfe)";
const char S11703[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11,gzip(gfe) (via docs.google.com/viewer)";
const char S11704[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1082) AppleWebKit/537.11 (KHTML like Gecko) Chrome/23.0.1271.17 Safari/537.11";
const char S11705[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11";
const char S11706[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11";
const char S11707[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11,gzip(gfe) (via docs.google.com/viewer)";
const char S11708[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S11709[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S11710[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe) (via translate.google.com)";
const char S11711[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe) (via translate.google.com)";
const char S11712[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe) (via docs.google.com/viewer)";
const char S11713[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/534.7 ZemanaAID/FFFF0061";
const char S11714[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe)";
const char S11715[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe)";
const char S11716[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; Zune 4.7) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11  Firefox/16.0";
const char S11717[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML like Gecko) Chrome/20.0.1132.47 Safari/536.11";
const char S11718[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11 AppEngine-Google; (+http://code.google.com/appengine; appid: s~zyfqwe5766)";
const char S11719[] = "Opera/9.80+(iPhone;+Opera+Mini/7.0.1/28.2051;+U;+en)+Presto/2.8.119+Version/11.10";
const char S11720[] = "Opera/9.80 (iPhone; Opera Mini/7.0.5/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S11721[] = "Mercury3Free/6.0.1 CFNetwork/609 Darwin/13.0.0";
const char S11722[] = "Feeds/2.2.1 CFNetwork/609 Darwin/13.0.0";
const char S11723[] = "Terra/2.0.1 CFNetwork/609 Darwin/13.0.0";
const char S11724[] = "HOT%20HD%20WPs/9.19.1 CFNetwork/609 Darwin/13.0.0";
const char S11725[] = "MyPad/2.6.2 CFNetwork/609 Darwin/13.0.0";
const char S11726[] = "WOW%20WPs/9.26.1 CFNetwork/609 Darwin/13.0.0";
const char S11727[] = "Animations/9.30.1 CFNetwork/609 Darwin/13.0.0";
const char S11728[] = "vBrowse%202.0/2.03 CFNetwork/609 Darwin/13.0.0";
const char S11729[] = "Images/1.4 CFNetwork/609 Darwin/13.0.0";
const char S11730[] = "Mercury3/6.0.1 CFNetwork/609 Darwin/13.0.0";
const char S11731[] = "GQMagazine/1.0 CFNetwork/609 Darwin/13.0.0";
const char S11732[] = "Web%20Feeds/1.2 CFNetwork/609 Darwin/13.0.0";
const char S11733[] = "WEB%20to%20PDF/2.3 CFNetwork/609 Darwin/13.0.0";
const char S11734[] = "Prismatic/1.0.4 CFNetwork/609 Darwin/13.0.0";
const char S11735[] = "Qriket%20Game/3.1.5 CFNetwork/609 Darwin/13.0.0";
const char S11736[] = "Digg/X.3.1.1 CFNetwork/609 Darwin/13.0.0";
const char S11737[] = "CEBrowser/1.6.1 CFNetwork/609 Darwin/13.0.0";
const char S11738[] = "OWA/X CFNetwork/609 Darwin/13.0.0";
const char S11739[] = "JDate/101 CFNetwork/609 Darwin/13.0.0";
const char S11740[] = "KaywaReader/1.21 CFNetwork/609 Darwin/13.0.0";
const char S11741[] = "PhotoDiscover/2.1.1 CFNetwork/609 Darwin/13.0.0";
const char S11742[] = "iLoder/1.0 CFNetwork/609 Darwin/13.0.0";
const char S11743[] = "ePrint/4.8 CFNetwork/609 Darwin/13.0.0";
const char S11744[] = "NewsFree/1.1.1 CFNetwork/609 Darwin/13.0.0";
const char S11745[] = "TEE2/2.3.4 CFNetwork/609 Darwin/13.0.0";
const char S11746[] = "Reddit%20Pics/1.6 CFNetwork/609 Darwin/13.0.0";
const char S11747[] = "Scribbler/1.0 CFNetwork/609 Darwin/13.0.0";
const char S11748[] = "NewsMe/1.2.1 CFNetwork/609 Darwin/13.0.0";
const char S11749[] = "TwitterFon/628 CFNetwork/609 Darwin/13.0.0";
const char S11750[] = "MyPad%20for%20iPhone/2.6.2 CFNetwork/609 Darwin/13.0.0";
const char S11751[] = "NewsRack/254 CFNetwork/609 Darwin/13.0.0";
const char S11752[] = "QihooBrowser/1.0 CFNetwork/609 Darwin/13.0.0";
const char S11753[] = "Apsley/2.3 CFNetwork/609 Darwin/13.0.0";
const char S11754[] = "kikin/1.9.5 CFNetwork/609 Darwin/13.0.0";
const char S11755[] = "PhotoViewLite/2.4 CFNetwork/609 Darwin/13.0.0";
const char S11756[] = "Readability/1.2.4 CFNetwork/609 Darwin/13.0.0";
const char S11757[] = "TwitterFonPro/628 CFNetwork/609 Darwin/13.0.0";
const char S11758[] = "NewsifyFree/73 CFNetwork/609 Darwin/13.0.0";
const char S11759[] = "iSocializeFree/36 CFNetwork/609 Darwin/13.0.0";
const char S11760[] = "Grokr/1508 CFNetwork/609 Darwin/13.0.0";
const char S11761[] = "TheFeed/1.0.2 CFNetwork/609 Darwin/13.0.0";
const char S11762[] = "ChromeSync%20Lite/1.2.1 CFNetwork/609 Darwin/13.0.0";
const char S11763[] = "Celebrity/1.0 CFNetwork/609 Darwin/13.0.0";
const char S11764[] = "Demotivation/2.5.1 CFNetwork/609 Darwin/13.0.0";
const char S11765[] = "Web/1.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11766[] = "AtomicBrowser/7.0.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11767[] = "Google/2.5.1.13455 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11768[] = "AtomicLite/7.0.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11769[] = "Clipish/9.29.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11770[] = "browser/34122 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11771[] = "Searchx/2.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11772[] = "TweetSpeaker/1.0.1 CFNetwork/609 Darwin/13.0.0";
const char S11773[] = "Kik/6.1.0.20 CFNetwork/609 Darwin/13.0.0";
const char S11774[] = "DaumApps/3.3.2 CFNetwork/609 Darwin/13.0.0";
const char S11775[] = "Instapaper/4.2.7 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11776[] = "Celeb%20Wallpapers/9.13.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11777[] = "GeoServices/84 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11778[] = "ImageSearcherFreeS/1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11779[] = "SpringBoard/50 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11780[] = "Settings/1.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11781[] = "Extras4iMovie/10.01 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11782[] = "BrowserFree/1.2.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11783[] = "AirWatch/4.2.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11784[] = "123people/3.1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11785[] = "Hot%20Wallpapers/10.02 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11786[] = "ImageSearcherS/3.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11787[] = "Apolloplus/2.7.10 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11788[] = "FeeddlerPro/1.12.5 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11789[] = "ImageSearcherFree/1.0.5 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11790[] = "Million%20HD%20WPs/8.80.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11791[] = "Free%20Video%20Downloader/1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11792[] = "iAnnotate%20PDF/2.4.5 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11793[] = "ImageDownloader/1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11794[] = "ImageSearcherProS/2.1.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11795[] = "AssistantServices/1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11796[] = "ImageSearcher/3.1.3 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11797[] = "Y_Messenger/5467 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11798[] = "PhotoFind/5.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11799[] = "Mobile%20Learn/3.1.4 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11800[] = "securityd (unknown version) CFNetwork/609.1.4 Darwin/13.0.0";
const char S11801[] = "HideMyPhotos/9.29.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11802[] = "LastPass%20Tab/2.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11803[] = "HD%20&%20Retina%20Wallpapers/1.6 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11804[] = "iDownloaderLite/1.0.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11805[] = "SecureBrowser/3.5.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11806[] = "Holidays/10.01 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11807[] = "ImageSearcherPro/2.1.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11808[] = "ImagePro/2.6 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11809[] = "ImageHunter/2.5.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11810[] = "EpicWallPaper2011/1.6 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11811[] = "iDownloaderPro/2.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11812[] = "ApolloVideoDownloaderFree/1.2.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11813[] = "Pulse/3.0.4 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11814[] = "Love%20Pictures/3.7 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11815[] = "syncdefaultsd (unknown version) CFNetwork/609.1.4 Darwin/13.0.0";
const char S11816[] = "ubd/243.12.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11817[] = "PagesJaunes/5.3 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11818[] = "Weather/1.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11819[] = "ZBar/2.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11820[] = "Wallpapers/1.6 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11821[] = "Funny%20Pics/2.93 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11822[] = "Love%20Messenger/9.16.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11823[] = "AdSheet/1.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11824[] = "Snipbase/2535 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11825[] = "mstreamd/2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11826[] = "iPick/200 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11827[] = "Download%20Free/2.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11828[] = "SSC%20Forums/X CFNetwork/609.1.4 Darwin/13.0.0";
const char S11829[] = "tfctv/1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11830[] = "Newsify/87 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11831[] = "GoodReaderIPad/3.19.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11832[] = "DiigoBrowser/4.7 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11833[] = "Azul/3.9 CFNetwork/609 Darwin/13.0.0";
const char S11834[] = "Cool%20Retina%20Wallpapers/1.5 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11835[] = "Downloader%20copy/1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11836[] = "BarCodeScan/1.1.3 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11837[] = "2%20Million%20WPs/10.09 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11838[] = "IMRemoteURLConnectionAgent/900 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11839[] = "dataaccessd (unknown version) CFNetwork/609.1.4 Darwin/13.0.0";
const char S11840[] = "itunesstored (unknown version) CFNetwork/609.1.4 Darwin/13.0.0";
const char S11841[] = "VideoDLFree/1.1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11842[] = "locationd/1491.92 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11843[] = "%20iSearch/1.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11844[] = "Cydia/0.9 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11845[] = "mobileassetd (unknown version) CFNetwork/609.1.4 Darwin/13.0.0";
const char S11846[] = "FDDM/1.3 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11847[] = "Redd%20Alien/1630 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11848[] = "360%20Browser/5.3.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11849[] = "CLIPish%20Pro2/10.11 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11850[] = "iSocialize2/39 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11851[] = "TubeDownloader/1.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11852[] = "Job%20Hopper/1.3 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11853[] = "Perfect%20Browser/9.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11854[] = "NortonSafeBrowser/168.2.5.14 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11855[] = "360%20Lite/5.3.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11856[] = "VideoDLPlusPlusFree/1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11857[] = "DreamLeague/1.19 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11858[] = "apsd (unknown version) CFNetwork/609.1.4 Darwin/13.0.0";
const char S11859[] = "VideoDLPro/1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11860[] = "Cool%20Wallpapers/1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11861[] = "Symbaloo/1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11862[] = "IMTransferAgent/900 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11863[] = "ImageProHDFree/1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11864[] = "MusicDownloaderSSAFree/1.0.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11865[] = "ForumRunner/X CFNetwork/609.1.4 Darwin/13.0.0";
const char S11866[] = "Instagram/X CFNetwork/609.1.4 Darwin/13.0.0";
const char S11867[] = "Download/2.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11868[] = "ShoppingToday/2.04 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11869[] = "iCandyWP1/9.15.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11870[] = "Downloader/1.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11871[] = "MusicDownloaderBSOFree/1.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11872[] = "DL-Picture/1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11873[] = "ScoopIt/111 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11874[] = "Cooliris/71315 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11875[] = "Reader/2.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11876[] = "vBrowse/3.7 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11877[] = "ysearch2/3.0.4 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11878[] = "Keeper/6.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11879[] = "SliceMobile/2.3.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11880[] = "Maps/1.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11881[] = "ImageProHD/1.9 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11882[] = "Snapchat/4.0.3 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11883[] = "aosnotifyd/$%28CURRENT_PROJECT_VERSION%29 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11884[] = "mediaserverd (unknown version) CFNetwork/609.1.4 Darwin/13.0.0";
const char S11885[] = "configd (unknown version) CFNetwork/609.1.4 Darwin/13.0.0";
const char S11886[] = "iCarousel/1.2.7 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11887[] = "gamed (unknown version) CFNetwork/609.1.4 Darwin/13.0.0";
const char S11888[] = "AppStore/1.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11889[] = "xFeed/667 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11890[] = "VideoDownloader/1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11891[] = "Lync%X/2.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11892[] = "iReader/5.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11893[] = "GoodReader/3.19.3 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11894[] = "Epic%20Fail/2.8 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11895[] = "CommCenterMobileHelper/1464.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11896[] = "UnlimitedSMS/4.4.23 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11897[] = "UrlWebViewForIpad/1.8 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11898[] = "MyPad_Premium/3.0.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11899[] = "YService/3.0.6 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11900[] = "ImageSearch/1.5 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11901[] = "wewsiphone/11710 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11902[] = "playable/3.6 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11903[] = "Sigalert/1.0.1 CFNetwork/609 Darwin/13.0.0";
const char S11904[] = "Pulse%20News/2.9.5 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11905[] = "FreeCell/2.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11906[] = "AWBrowser/2.0.460 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11907[] = "MusicDownloaderBSO/1.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11908[] = "ApolloGeneralDownloader/1.1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11909[] = "Silly%20Pics/1.3 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11910[] = "BaDoink%20Lite/1.5 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11911[] = "Yellow%20Pages%20Local%20Search%20by%20Superpages/X CFNetwork/609.1.4 Darwin/13.0.0";
const char S11912[] = "ImageMobile/1.9 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11913[] = "Accounts/113 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11914[] = "Dual%20Browser/1.4.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11915[] = "iBuilder/1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11916[] = "iScanner/3.5 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11917[] = "ScrollingCredits/10.01 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11918[] = "OTACrashCopier (unknown version) CFNetwork/609.1.4 Darwin/13.0.0";
const char S11919[] = "TubeDownloaderLite/1.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11920[] = "iDownloadsAppPlus/1.5.8 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11921[] = "Yahoo!/9150 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11922[] = "WebSheet/1.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11923[] = "Codes%20Reader/2.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11924[] = "Christmas/10.01 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11925[] = "ImageHunterPro/2.5.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11926[] = "LexPrint/3.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11927[] = "NESN/1.03 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11928[] = "Flow/1.5 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11929[] = "PDF%20Printer/2.0.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11930[] = "%EB%8B%A4%EC%9D%8C/4.0.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11931[] = "Twitter/5.7 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11932[] = "ApolloVideoDownloader/1.2.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11933[] = "Messenger/X CFNetwork/609.1.4 Darwin/13.0.0";
const char S11934[] = "MxBrowser-iPhone/13 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11935[] = "Homes/1.5 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11936[] = "VideoDownloaderLite/1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11937[] = "HDWallpaper/2.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11938[] = "MyMedia/3.01 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11939[] = "Filebox/1.4 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11940[] = "Qrafter/7.6 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11941[] = "iFunny/22 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11942[] = "Guidebook/2.2.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11943[] = "Summly/2.0.2 CFNetwork/609 Darwin/13.0.0";
const char S11944[] = "Shazam/581 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11945[] = "Emoji%20PLUS/10.01 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11946[] = "Million%20Photos/8.60.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11947[] = "AppCake3/3.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11948[] = "PimpYourPic/9.14.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11949[] = "VideoDLPlusPlus/1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11950[] = "Pandora/4.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11951[] = "subwaysurfers/1.9.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11952[] = "WordsWithFriendsFree/5.90 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11953[] = "FlashLight/3.6 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11954[] = "Talking%20Ginger/1.4 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11955[] = "MessageMe/11 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11956[] = "Pic%20Collage/3.10.5 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11957[] = "MusicDownloaderLite/1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11958[] = "RoboForm/151 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11959[] = "MobileSearch/5774 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11960[] = "MusicDownloader/1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11961[] = "Accountable2You%20Mobile/1.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11962[] = "Ruzzle/1.4.10 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11963[] = "iPhone%20Free/X CFNetwork/609.1.4 Darwin/13.0.0";
const char S11964[] = "whisper/2.9.8 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11965[] = "MxBrowser-iPhone/14 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11966[] = "backupd (unknown version) CFNetwork/609.1.4 Darwin/13.0.0";
const char S11967[] = "LinkedIn/6.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11968[] = "catalogTattoos/1.71 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11969[] = "%E5%BF%AB%E6%92%AD%E5%A2%9E%E5%BC%BA%E7%89%88/1734 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11970[] = "PDF%20PROvider/2.0.4 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11971[] = "ksmobilebrowser/1.2.4342.200 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11972[] = "Qrafter%20Pro/7.6 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11973[] = "%E5%A4%9A%E5%B1%8F%E4%BA%92%E5%8A%A8%E6%B5%8F%E8%A7%88/1976 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11974[] = "SKOUT_SDK4/3.6.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11975[] = "BaiduHD/2.2.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11976[] = "NaverSearch/4.7.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11977[] = "Tweetlogix/2.1.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11978[] = "AppleIDAuthAgent/552 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11979[] = "WallPaper/1.3 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11980[] = "Florida/94 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11981[] = "Viber/3.0.0.4074 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11982[] = "Thumb/3.8.8 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11983[] = "Educreations/1243 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11984[] = "ColorLEDFlashlightPro/1.5.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11985[] = "YandexSearch/1480 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11986[] = "SkyGrid/2.6 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11987[] = "vBrowsePRO/3.7 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11988[] = "DownMan/4.0.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11989[] = "NorthCarolina/291 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11990[] = "Buffer/2.4 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11991[] = "logosQuiz/1.5.6 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11992[] = "NewJerseyFH/734 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11993[] = "SolitaireFree/3.8.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11994[] = "Ourpad/1.6.6 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11995[] = "Washington/208 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11996[] = "Voto/119 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11997[] = "WEB%20to%20PDF%20Plus/2.3.3 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11998[] = "QRReader/3.6 CFNetwork/609.1.4 Darwin/13.0.0";
const char S11999[] = "VZ%20Messages/1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12000[] = "FastBrowser/10.3.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12001[] = "GeorgiaFH/473 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12002[] = "NewJersey/518 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12003[] = "Louisiana/1291 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12004[] = "AlienBlueHD/2.2.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12005[] = "FreeDownload/2.35 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12006[] = "Clock/1.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12007[] = "PerfectDownloaderLite/2.5.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12008[] = "ChickOrDude-Lite/1.5 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12009[] = "Fun%20Animations/10.07 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12010[] = "WallpaperGeneratorHD%20copy/4.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12011[] = "RSSReader/1.04 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12012[] = "SoundBooth/9.13.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12013[] = "MobileStore/1.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12014[] = "Photon/4.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12015[] = "Print%20Utility/2.9.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12016[] = "iBooks/1523 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12017[] = "DownloadAll/1.0.3 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12018[] = "MicroMessenger/4.5.0.54 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12019[] = "Pennsylvania/1140 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12020[] = "Delaware/253 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12021[] = "Accessbox/1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12022[] = "Ginger%20%F0%9F%8E%82/1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12023[] = "Sudoku/1.4 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12024[] = "BabyBumpFree/7.1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12025[] = "Connecticut/718 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12026[] = "IconPopQuiz/1.41 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12027[] = "WMV%20player/2.10 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12028[] = "To%20PDF/1.0.3 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12029[] = "Speedtest/3.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12030[] = "Georgia/382 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12031[] = "SouthDakota/235 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12032[] = "Missouri/100 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12033[] = "Maine/47 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12034[] = "Songza/1433 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12035[] = "HP%20ePrint/5.2.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12036[] = "Kentucky/121 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12037[] = "eBay/3.0.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12038[] = "CamFind/98 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12039[] = "Springpad/4.0.6 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12040[] = "vine/1.3.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12041[] = "Montana/258 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12042[] = "Tennessee/243 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12043[] = "MyLifeRelease/312 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12044[] = "MusicDownloaderSSA/1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12045[] = "Incredimail/5649 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12046[] = "MxBrowser-iPhone/17 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12047[] = "Free%20Video%20Downloader%20Lite/1.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12048[] = "Pinterest/2.6.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12049[] = "passd/1.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12050[] = "Translate/1.3.1.2978 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12051[] = "MobileSlideShow/43 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12052[] = "Glassboard/2041 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12053[] = "iReddit/2.1.5 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12054[] = "Calculator%20Pro/2.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12055[] = "VideoDownloaderFree/2.8 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12056[] = "Virginia/169 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12057[] = "Kansas/105 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12058[] = "TextMe%202/2.2.5 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12059[] = "MilkTheCowFREE/2.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12060[] = "OfficeJerk/1.9.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12061[] = "Scan/369 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12062[] = "Utah/74 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12063[] = "NebraskaFH/152 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12064[] = "DespicableMe/1.1.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12065[] = "MusicDownloaderSokolovLite/1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12066[] = "123peopleUS/3.4 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12067[] = "etracksapp/969 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12068[] = "GroupMe/4.2.5 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12069[] = "Edmodo/12 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12070[] = "GWallPaper/1.7 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12071[] = "PhotoAge-Lite/1.5 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12072[] = "Xmarks/1.5.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12073[] = "Tap%20Forms/93 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12074[] = "Free%20Music%20Download/1.4.3 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12075[] = "TotalDownloader%20Free/29 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12076[] = "Solitaire/2.4.4 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12077[] = "food/2.3.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12078[] = "Dashlane/45475 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12079[] = "Coast/1.0.2.62956 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12080[] = "PanelLED/1.3.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12081[] = "PZPlayer/1.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12082[] = "Nebraska/398 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12083[] = "NewYork/184 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12084[] = "MAKE19/1.8 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12085[] = "classifieds_free/2232 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12086[] = "LINE/3.8.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12087[] = "AlienBlue/2.8.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12088[] = "TalkTablet/6.9 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12089[] = "Barracuda_Safe_Browser/2.0.2.12 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12090[] = "RhodeIsland/46 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12091[] = "mp3musicdownloaderfree/5.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12092[] = "BejBlitz/608 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12093[] = "Iowa/50 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12094[] = "MusicDownloaderSokolov/1.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12095[] = "Snip/1.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12096[] = "Dentist/1.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12097[] = "JazzBird/7.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12098[] = "Daum/4.2.2 CFNetwork/609.1.4 Darwin/13.0.0";
const char S12099[] = "Microsoft Office/14.0 (Windows NT 5.1; Microsoft Outlook 14.0.4760; Pro; ms-office; MSOffice 14)";
const char S12100[] = "Microsoft Office/14.0 (Windows NT 5.1; Microsoft Outlook 14.0.5128; Pro; ms-office; MSOffice 14)";
const char S12101[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S12102[] = "Microsoft Office/14.0 (Windows NT 5.1; Microsoft Outlook 14.0.6106; Pro; ms-office; MSOffice 14)";
const char S12103[] = "Microsoft Office/14.0 (Windows NT 5.1; Microsoft Outlook 14.0.4734; Pro; ms-office; MSOffice 14)";
const char S12104[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.3; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S12105[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; Schroders; Microsoft Outlook 14.0.6106; ms-office; MSOffice 14)";
const char S12106[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Outlook 14.0.5136; ms-office; MSOffice 14)";
const char S12107[] = "Microsoft Office/14.0 (Windows NT 5.1; Microsoft Outlook 14.0.6023; Pro)";
const char S12108[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.3; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S12109[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.3; Microsoft Outlook 14.0.5138; ms-office; MSOffice 14)";
const char S12110[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S12111[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S12112[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S12113[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S12114[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S12115[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.3; Microsoft Outlook 14.0.5128; ms-office; MSOffice 14)";
const char S12116[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.3; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S12117[] = "Microsoft Office/14.0 (Windows NT 5.1; Microsoft Outlook 14.0.6112; Pro; ms-office; MSOffice 14)";
const char S12118[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S12119[] = "Microsoft Office/14.0 (Windows NT 5.1; Microsoft Outlook 14.0.6123; Pro)";
const char S12120[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; GTB7.4; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.3; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S12121[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Outlook 14.0.6114; ms-office; MSOffice 14)";
const char S12122[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S12123[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S12124[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S12125[] = "Microsoft Office/14.0 (Windows NT 5.1; Microsoft Outlook 14.0.6129; Pro)";
const char S12126[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.3; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S12127[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S12128[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S12129[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; Microsoft Outlook 14.0.6123; ms-office; MSOffice 14)";
const char S12130[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S12131[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S12132[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S12133[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S12134[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.3; Microsoft Outlook 14.0.5138; ms-office; MSOffice 14)";
const char S12135[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; ms-office; MSOffice 14) ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Outlook 14.0.6131)";
const char S12136[] = "Microsoft Office/14.0 (Windows NT 5.1; Microsoft Outlook 14.0.7015; Pro)";
const char S12137[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S12138[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.3; MS-RTC LM 8; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S12139[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S12140[] = "Microsoft Office/14.0 (Windows NT 5.1; Microsoft Outlook 14.0.7106; Pro)";
const char S12141[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S12142[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S12143[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8; .NET4.0C; .NET4.0E; InfoPath.2; Microsoft Outlook 14.0.6133; ms-office; MSOffice 14)";
const char S12144[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S12145[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S12146[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S12147[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S12148[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S12149[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S12150[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)";
const char S12151[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe) (via docs.google.com/viewer)";
const char S12152[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30";
const char S12153[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30";
const char S12154[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30,gzip(gfe) (via translate.google.com)";
const char S12155[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.91 Safari/534.30,gzip(gfe) (via translate.google.com)";
const char S12156[] = "Mozilla/5.0 (Windows NT 6.1; en-US) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.750.0 Safari/534.30";
const char S12157[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/12.0.702.0 Safari/533.3 Awesomium/1.6.5";
const char S12158[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; AppleWebKit/534.24; KHTML; like Gecko; Chrome/12.0.702.0 Safari/534.24)";
const char S12159[] = "Mozilla/4.0 (compatible; Windows NT 6.1; AppleWebKit/534.24; KHTML, like Gecko; Chrome/12.0.702.0 Safari/534.24)";
const char S12160[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30,gzip(gfe)";
const char S12161[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.30 (KHTML	 like Gecko) Chrome/12.0.742.122 Safari/534.30 ChromePlus/1.6.3.1";
const char S12162[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30,gzip(gfe)";
const char S12163[] = "Mozilla/5.0 (Windows NT 6.1; en-CA) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.750.0 Safari/534.30";
const char S12164[] = "Mozilla/5.0 (Windows NT 6.1; es-US) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.750.0 Safari/534.30";
const char S12165[] = "Mozilla/5.0 (Windows NT 6.1; zh-CN) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.750.0 Safari/534.30";
const char S12166[] = "Mozilla/5.0 (Windows NT 6.1; haw-US) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.750.0 Safari/534.30";
const char S12167[] = "Mozilla/5.0 (Windows NT 6.1; fr-CA) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.750.0 Safari/534.30";
const char S12168[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30,gzip(gfe) (via docs.google.com/viewer)";
const char S12169[] = "Mozilla/5.0 (Windows NT 6.1; tr-DE) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.750.0 Safari/534.30";
const char S12170[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.24 (KHTML like Gecko) Chrome/12.0.702.0 Safari/534.24";
const char S12171[] = "Mozilla/5.0 (Windows NT 6.1; en-US) AppleWebKit/534.30 (KHTML like Gecko) Chrome/12.0.750.0 Safari/534.30";
const char S12172[] = "Mozilla/4.0 (compatible; Windows NT 6.1; WOW64; AppleWebKit/534.24; KHTML, like Gecko; Chrome/12.0.702.0 Safari/534.24)";
const char S12173[] = "Mozilla/4.0 (compatible; Windows NT 6.1; AppleWebKit/534.24; Trident/4.0; KHTML, like Gecko; Chrome/12.0.702.0 Safari/534.24)";
const char S12174[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36";
const char S12175[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.1.0.0 Safari/537.36";
const char S12176[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.1.0.0 Safari/537.36";
const char S12177[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36";
const char S12178[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S12179[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S12180[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe)";
const char S12181[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S12182[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.2.0.0 Safari/537.36";
const char S12183[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.2.0.0 Safari/537.36";
const char S12184[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe)";
const char S12185[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S12186[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.1.0.0 Safari/537.17";
const char S12187[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.1.0.0 Safari/537.17";
const char S12188[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)";
const char S12189[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)";
const char S12190[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S12191[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S12192[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.2.0.0 Safari/537.17";
const char S12193[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.2.0.0 Safari/537.17";
const char S12194[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; en-US) AppleWebKit/537.15 (KHTML, like Gecko) Chrome/24.0.1295.0 Safari/537.15";
const char S12195[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe) (via docs.google.com/viewer)";
const char S12196[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe) (via docs.google.com/viewer)";
const char S12197[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.10.1312.57 Safari/537.17 QIPSurf";
const char S12198[] = "Mozilla/4.0 (compatible; MSIE 6.01; Windows 98; Alexa Toolbar)";
const char S12199[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; Win 9x4.90)";
const char S12200[] = "Mozilla/5.0 (compatible; MSIE 6.0; Windows 98)";
const char S12201[] = "Mozilla/4.5 (compatible; MSIE 6.0; Windows 98)";
const char S12202[] = "Mozilla/9.0 (compatible; MSIE 6.0; Windows 98)";
const char S12203[] = "Mozilla/4.0 (compatible; MSIE 6.02; Windows 98)";
const char S12204[] = "Mozilla/5.0 (compatible; MSIE 6.0; Win 9x 4.90; Trident/4.0)";
const char S12205[] = "Mozilla/5.0 (compatible; MSIE 6.0; Win98; Trident/3.0)";
const char S12206[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/5.0.17443/24.746; U; en) Presto/2.5.25 Version/10.54";
const char S12207[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/5.0.3521/886; U; en) Presto/2.4.15";
const char S12208[] = "Opera/9.50 (J2ME/MIDP; Opera Mini/5.0.21463/28.3165; U; en)";
const char S12209[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/5.0/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S12210[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/5/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S12211[] = "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.10";
const char S12212[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.10";
const char S12213[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.10";
const char S12214[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.10";
const char S12215[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.10";
const char S12216[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.10";
const char S12217[] = "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.10,gzip(gfe)";
const char S12218[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.10";
const char S12219[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.10,gzip(gfe)";
const char S12220[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.10";
const char S12221[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.10";
const char S12222[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.10";
const char S12223[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.10";
const char S12224[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.10";
const char S12225[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.10";
const char S12226[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.10";
const char S12227[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.10";
const char S12228[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.10";
const char S12229[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.10";
const char S12230[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.10";
const char S12231[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.10";
const char S12232[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64) Opera 12.10";
const char S12233[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.10";
const char S12234[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.10";
const char S12235[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.10";
const char S12236[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.10";
const char S12237[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.10";
const char S12238[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.10";
const char S12239[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.10";
const char S12240[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.10";
const char S12241[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.10";
const char S12242[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.10";
const char S12243[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.10";
const char S12244[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.10";
const char S12245[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.10";
const char S12246[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.10";
const char S12247[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.10";
const char S12248[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.10";
const char S12249[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.10";
const char S12250[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.10";
const char S12251[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.10";
const char S12252[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.10";
const char S12253[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.10";
const char S12254[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.10";
const char S12255[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.10";
const char S12256[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.10";
const char S12257[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.10";
const char S12258[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.10";
const char S12259[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5288)) Presto/2.12.388 Version/12.10";
const char S12260[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.10 Yandex.Translate";
const char S12261[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.10";
const char S12262[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.10";
const char S12263[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.10";
const char S12264[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4933)) Presto/2.12.388 Version/12.10";
const char S12265[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.10";
const char S12266[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.10";
const char S12267[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.10";
const char S12268[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.10";
const char S12269[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5230)) Presto/2.12.388 Version/12.10";
const char S12270[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03789)) Presto/2.12.388 Version/12.10";
const char S12271[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.10";
const char S12272[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.10";
const char S12273[] = "Opera/9.80 (Windows NT 6.1; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.10";
const char S12274[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5660)) Presto/2.12.388 Version/12.10";
const char S12275[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.10";
const char S12276[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.10";
const char S12277[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.10";
const char S12278[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.10";
const char S12279[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.10";
const char S12280[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4110)) Presto/2.12.388 Version/12.10";
const char S12281[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.10";
const char S12282[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.10";
const char S12283[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.10";
const char S12284[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.10";
const char S12285[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.10";
const char S12286[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.10";
const char S12287[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5278)) Presto/2.12.388 Version/12.10";
const char S12288[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.10";
const char S12289[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03658)) Presto/2.12.388 Version/12.10";
const char S12290[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5299)) Presto/2.12.388 Version/12.10";
const char S12291[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4832)) Presto/2.12.388 Version/12.10";
const char S12292[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4661)) Presto/2.12.388 Version/12.10";
const char S12293[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5300)) Presto/2.12.388 Version/12.10";
const char S12294[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.10";
const char S12295[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.10";
const char S12296[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.10";
const char S12297[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5730)) Presto/2.12.388 Version/12.10";
const char S12298[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5650)) Presto/2.12.388 Version/12.10";
const char S12299[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.10";
const char S12300[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.10";
const char S12301[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.10";
const char S12302[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC One X Build/IML74K)";
const char S12303[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC One X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12304[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_X/1.26.161.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12305[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_X/1.26.110.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12306[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PJ83100/1.73.502.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12307[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC_One_X/1.26.163.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12308[] = "BBCBeacon/0.73 (HTC One X; Android 4.0.3) iplayer/1.3";
const char S12309[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12310[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_X-orange-LS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12311[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_X/1.28.161.9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12312[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC_One_X/1.26.166.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12313[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PJ83100/1.61.502.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12314[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC S720e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12315[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_X/1.28.110.7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12316[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC One X Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S12317[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_One_X/1.26.169.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12318[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_One_X/1.26.111.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12319[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_One_X/1.28.111.10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12320[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_One_X/1.26.112.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12321[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PJ83100/1.82.502.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12322[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC EVA_UL Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12323[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC_One_X/1.28.163.9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12324[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PJ83100/1.85.502.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12325[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_X/1.29.110.11 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12326[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12327[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_X/1.29.161.13 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12328[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_One_X/1.29.169.13 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12329[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC_One_X/1.29.166.13 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12330[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_One_X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12331[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC_One_X/1.29.163.13 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12332[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; HTC_One_X/1.28.118.11 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12333[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC_One_X/1.28.166.9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12334[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC One X Build/IMM76D)";
const char S12335[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PJ83100/2.20.502.7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12336[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PJ83100/1.82.502.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12337[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC S720e Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S12338[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC One X Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S12339[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PJ83100/1.68.502.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12340[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; One X Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12341[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC S720e Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S12342[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PJ83100/1.85.502.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12343[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_One_X/1.29.111.11 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12344[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; HTC One X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12345[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC S720e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12346[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PJ83100/2.20.502.5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12347[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_One_X/1.28.169.9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12348[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_One_X/2.17.161.4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12349[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_One_X-orange-LS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12350[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HTC_One_X/2.17.111.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12351[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; HTC_One_X/2.17.163.4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12352[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; HTC_One_X/2.17.169.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12353[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-ch; HTC_One_X/2.17.166.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12354[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; HTC EVA_UL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12355[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_One_X/2.17.110.5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12356[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; HTC_One_X/2.17.112.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12357[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S720e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12358[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; HTC_One_X/2.17.118.4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12359[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S12360[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC S720e Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S12361[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12362[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_One_X/1.29.161.13 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12363[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-; HTC_One_X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12364[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; HTC_One_X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12365[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PJ83100/1.61.502.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12366[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneX/1.26.111.4 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12367[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneX/1.29.166.13 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12368[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneX/1.26.110.5 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12369[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC One X Build/IML74K) UC AppleWebKit/534.31 (KHTML like Gecko) Mobile Safari/534.31";
const char S12370[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneX/1.28.110.7 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12371[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCOneX Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12372[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneX/1.28.111.10 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12373[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneX/1.26.161.2 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12374[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCOneX/2.17.161.4 Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12375[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCOneX-orange-LS Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12376[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneX/1.29.163.13 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12377[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCOneX/2.17.110.5 Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12378[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCOneX/1.29.161.13 Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12379[] = "HTC One X Linux/3.0.13 Android/4.1.9 Release/11.28.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S12380[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC One X Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12381[] = "Mozilla/5.0 (Linux; U; Android 4.0.6; en-us; HTC One X Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12382[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneX/1.29.161.13 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12383[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneX Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12384[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneX-orange-LS Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12385[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.0.4;dev=evita;devm=HTC One X;man=HTC;brw=Android;shwd=720x1280;shwa=720x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S12386[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe)";
const char S12387[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)";
const char S12388[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.1.3.0 Safari/537.31";
const char S12389[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S12390[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S12391[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.1.3.0 Safari/537.31";
const char S12392[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe)";
const char S12393[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.2.2.0 Safari/537.31";
const char S12394[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.2.2.0 Safari/537.31";
const char S12395[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)";
const char S12396[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/26.9.9.9 Safari/537.36";
const char S12397[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S12398[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S12399[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S12400[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S12401[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe),gzip(gfe)";
const char S12402[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe)";
const char S12403[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.1.0.0 Safari/537.36";
const char S12404[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.1.0.0 Safari/537.36";
const char S12405[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe)";
const char S12406[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5830i Build/GINGERBREAD)";
const char S12407[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12408[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S12409[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-S5839i-ORANGE/S5839iBVLC4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12410[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-S5839i-ORANGE/S5839iBVLC3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12411[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5839i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12412[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5839i/S5839iBULC1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12413[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; SAMSUNG GT-S5839i/S5839iBOLC5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12414[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; SAMSUNG GT-S5830i/S5830iXXLB2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12415[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5839i/S5839iBULE2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12416[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5839i Build/GINGERBREAD)";
const char S12417[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-S5839i-ORANGE/S5839iBVLD3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12418[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-S5839i/S5839iBOLE1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12419[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S12420[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-S5839i/S5839iBOLG1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12421[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12422[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-S5839i-ORANGE/S5839iBVLH1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12423[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5839i/S5839iBULI1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12424[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/RazoDroiD v1.0 by (rajrocks)rishee) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12425[] = "Mozilla/5.0 (Linux; U; Android 2.3.6(RazoDroiD); en-gb; GT-S5830i Build/RazoDroiD v2.0 by (rajrocks)rishee) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12426[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5839i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12427[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-S5830i Build/AceCream) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12428[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/MindCr Rom V1.0.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12429[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; SAMSUNG GT-S5830i/S5830iXXLI2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12430[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; SAMSUNG GT-S5839i/S5839iBOLI1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12431[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; sr-rs; SAMSUNG GT-S5830i/S5830iXXLA4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12432[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5839i/S5839iBULK1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12433[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; SAMSUNG GT-S5839i/S5839iBOLK2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12434[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/Retribution_beta#1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12435[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/Retribution Beta 3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12436[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/Retribution Beta 4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12437[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-S5839i-ORANGE/S5839iBVLK2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12438[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5839i/S5839iBULL1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12439[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT-S5830i Build/GingerCream-Berk Semsioglu) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12440[] = "Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S12441[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-S5830i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12442[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; SAMSUNG GT-S5830i/S5830iXXLK3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12443[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-S5830i/S5830iXXLH3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12444[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-S5830i Build/830iv1.5FX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12445[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; SAMSUNG GT-S5839i/S5839iBOMD1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12446[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; GT-S5830i Build/830iv1.4FX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12447[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-S5830i Build/830iv1.76513FX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12448[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; SAMSUNG GT-S5830i/S5830iXXMD1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12449[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/BoudZDroid v.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12450[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-S5830i Build/830iv1.96513FX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12451[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; SAMSUNG GT-S5839i/S5839iBOMG1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12452[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-S5830i Build/BoudZDroid V1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12453[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5839i/S5839iBUMG1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12454[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17";
const char S12455[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S12456[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S12457[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S12458[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe) (via docs.google.com/viewer)";
const char S12459[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)";
const char S12460[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2; en-US) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1309.0 Safari/537.17";
const char S12461[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S12462[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S12463[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.14";
const char S12464[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.14";
const char S12465[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.14";
const char S12466[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.14";
const char S12467[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.14";
const char S12468[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.14";
const char S12469[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.14";
const char S12470[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.14";
const char S12471[] = "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.14,gzip(gfe)";
const char S12472[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.14";
const char S12473[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.14";
const char S12474[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.14,gzip(gfe)";
const char S12475[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.14";
const char S12476[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.14";
const char S12477[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.14";
const char S12478[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.14";
const char S12479[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.14";
const char S12480[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.14";
const char S12481[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.14";
const char S12482[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.14";
const char S12483[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.14";
const char S12484[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.14";
const char S12485[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.14";
const char S12486[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.14";
const char S12487[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.14";
const char S12488[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.14";
const char S12489[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.14";
const char S12490[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.14";
const char S12491[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.14";
const char S12492[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.14";
const char S12493[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.14";
const char S12494[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.14";
const char S12495[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.14";
const char S12496[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64) Opera 12.14";
const char S12497[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.14";
const char S12498[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.14";
const char S12499[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.14";
const char S12500[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6046)) Presto/2.12.388 Version/12.14";
const char S12501[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.14";
const char S12502[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.14";
const char S12503[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.14";
const char S12504[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.14";
const char S12505[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.14";
const char S12506[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5997)) Presto/2.12.388 Version/12.14";
const char S12507[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.14";
const char S12508[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.14";
const char S12509[] = "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.14";
const char S12510[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.14";
const char S12511[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.14";
const char S12512[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.14";
const char S12513[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.14";
const char S12514[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4933)) Presto/2.12.388 Version/12.14";
const char S12515[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.14";
const char S12516[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.14";
const char S12517[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6044)) Presto/2.12.388 Version/12.14";
const char S12518[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.14";
const char S12519[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5660)) Presto/2.12.388 Version/12.14";
const char S12520[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.14";
const char S12521[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.14";
const char S12522[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.14";
const char S12523[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.14";
const char S12524[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.14";
const char S12525[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.14";
const char S12526[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.14 Yandex.Translate";
const char S12527[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.14";
const char S12528[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.14";
const char S12529[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.14";
const char S12530[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6035)) Presto/2.12.388 Version/12.14";
const char S12531[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.14";
const char S12532[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.14";
const char S12533[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.14";
const char S12534[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.14";
const char S12535[] = "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.14,gzip(gfe) (via docs.google.com/viewer)";
const char S12536[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.14";
const char S12537[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03658)) Presto/2.12.388 Version/12.14";
const char S12538[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03789)) Presto/2.12.388 Version/12.14";
const char S12539[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5299)) Presto/2.12.388 Version/12.14";
const char S12540[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.14";
const char S12541[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5217)) Presto/2.12.388 Version/12.14";
const char S12542[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4832)) Presto/2.12.388 Version/12.14";
const char S12543[] = "Opera/9.80 (Windows NT 6.1; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.14";
const char S12544[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5996)) Presto/2.12.388 Version/12.14";
const char S12545[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5288)) Presto/2.12.388 Version/12.14";
const char S12546[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.14";
const char S12547[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5650)) Presto/2.12.388 Version/12.14";
const char S12548[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.14,gzip(gfe) (via docs.google.com/viewer)";
const char S12549[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.14";
const char S12550[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.14";
const char S12551[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.14";
const char S12552[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6032)) Presto/2.12.388 Version/12.14";
const char S12553[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5988)) Presto/2.12.388 Version/12.14";
const char S12554[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.14";
const char S12555[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5230)) Presto/2.12.388 Version/12.14";
const char S12556[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4661)) Presto/2.12.388 Version/12.14";
const char S12557[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5300)) Presto/2.12.388 Version/12.14";
const char S12558[] = "Opera/9.80 (Windows NT 6.1; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.14";
const char S12559[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6080)) Presto/2.12.388 Version/12.14";
const char S12560[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.14";
const char S12561[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4110)) Presto/2.12.388 Version/12.14";
const char S12562[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5336)) Presto/2.12.388 Version/12.14";
const char S12563[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5734)) Presto/2.12.388 Version/12.14";
const char S12564[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03791)) Presto/2.12.388 Version/12.14";
const char S12565[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03639)) Presto/2.12.388 Version/12.14";
const char S12566[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5325)) Presto/2.12.388 Version/12.14";
const char S12567[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.14";
const char S12568[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5278)) Presto/2.12.388 Version/12.14";
const char S12569[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.14";
const char S12570[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03672)) Presto/2.12.388 Version/12.14";
const char S12571[] = "Opera/9.80 (Windows NT 6.1; MRA 5.6 (build 03402)) Presto/2.12.388 Version/12.14";
const char S12572[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03686); MRSPUTNIK OW 2, 3, 0, 244) Presto/2.12.388 Version/12.14";
const char S12573[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.14";
const char S12574[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03780)) Presto/2.12.388 Version/12.14";
const char S12575[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03638)) Presto/2.12.388 Version/12.14";
const char S12576[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03755)) Presto/2.12.388 Version/12.14";
const char S12577[] = "Opera/9.80 (Windows NT 6.1; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.14";
const char S12578[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.14";
const char S12579[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.14";
const char S12580[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.14,gzip(gfe)";
const char S12581[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.14";
const char S12582[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6561)) Presto/2.12.388 Version/12.14";
const char S12583[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.14";
const char S12584[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5161)) Presto/2.12.388 Version/12.14";
const char S12585[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 6014)) Presto/2.12.388 Version/12.14";
const char S12586[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6573)) Presto/2.12.388 Version/12.14";
const char S12587[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6283)) Presto/2.12.388 Version/12.14";
const char S12588[] = "Opera/9.80 (Windows NT 6.1; MRA 8.1 (build 6261)) Presto/2.12.388 Version/12.14";
const char S12589[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6638)) Presto/2.12.388 Version/12.14";
const char S12590[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6628)) Presto/2.12.388 Version/12.14";
const char S12591[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.14";
const char S12592[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6657)) Presto/2.12.388 Version/12.14";
const char S12593[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6665)) Presto/2.12.388 Version/12.14";
const char S12594[] = "Opera/9.80 (Windows NT 6.1; MRA 8.1 (build 6322)) Presto/2.12.388 Version/12.14";
const char S12595[] = "Opera/9.80 (Windows NT 6.1; MRA 8.1 (build 6323)) Presto/2.12.388 Version/12.14";
const char S12596[] = "Opera/9.80 (Windows NT 6.1; MRA 8.1 (build 6341)) Presto/2.12.388 Version/12.14";
const char S12597[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6648)) Presto/2.12.388 Version/12.14";
const char S12598[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6669)) Presto/2.12.388 Version/12.14";
const char S12599[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6673)) Presto/2.12.388 Version/12.14";
const char S12600[] = "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.14,gzip(gfe),gzip(gfe)";
const char S12601[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.14";
const char S12602[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6675)) Presto/2.12.388 Version/12.14";
const char S12603[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6337)) Presto/2.12.388 Version/12.14";
const char S12604[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.14,gzip(gfe),gzip(gfe)";
const char S12605[] = "Opera/9.80 (Windows NT 6.1; MRA 8.1 (build 6327)) Presto/2.12.388 Version/12.14";
const char S12606[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6676)) Presto/2.12.388 Version/12.14";
const char S12607[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6681)) Presto/2.12.388 Version/12.14";
const char S12608[] = "Opera/9.80 (Windows NT 6.1; MRA 8.1 (build 6347)) Presto/2.12.388 Version/12.14";
const char S12609[] = "Opera/9.80 (Windows NT 6.1; MRA 6.2 (build 7194)) Presto/2.12.388 Version/12.14";
const char S12610[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S12611[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S12612[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.1.0.0 Safari/537.11";
const char S12613[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.1.0.0 Safari/537.11";
const char S12614[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe)";
const char S12615[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe) (via docs.google.com/viewer)";
const char S12616[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.2.0.0 Safari/537.11";
const char S12617[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.4.0.0 Safari/537.11";
const char S12618[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.4.0.0 Safari/537.11";
const char S12619[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe)";
const char S12620[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe) (via docs.google.com/viewer)";
const char S12621[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.2.0.0 Safari/537.11";
const char S12622[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.3.0.0 Safari/537.11";
const char S12623[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.3.0.0 Safari/537.11";
const char S12624[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S12625[] = "Mozilla/5.0 (Windows NT 6.0) yi; AppleWebKit/X.12221 (KHTML, like Gecko) Chrome/23.0.1271.26 Safari/X.1221";
const char S12626[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S12627[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S12628[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S12629[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S12630[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S12631[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S12632[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe)";
const char S12633[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36";
const char S12634[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S12635[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S12636[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.1.0.0 Safari/537.11";
const char S12637[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.1.0.0 Safari/537.11";
const char S12638[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S12639[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.4.0.0 Safari/537.11";
const char S12640[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.2.0.0 Safari/537.11";
const char S12641[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.4.0.0 Safari/537.11";
const char S12642[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe) (via docs.google.com/viewer)";
const char S12643[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe)";
const char S12644[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe)";
const char S12645[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.2.0.0 Safari/537.11";
const char S12646[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.3.0.0 Safari/537.11";
const char S12647[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.3.0.0 Safari/537.11";
const char S12648[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe) (via docs.google.com/viewer)";
const char S12649[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.11 (KHTML like Gecko) Chrome/23.0.1271.91 Safari/537.11";
const char S12650[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/533.16 (KHTML, like Gecko) Version/4.1 Safari/533.16";
const char S12651[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/4.1.1 Safari/533.17.8";
const char S12652[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/4.1.2 Safari/533.18.5";
const char S12653[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/4.1.3 Safari/533.19.4";
const char S12654[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko, Safari/4533.19.4) ADM/722";
const char S12655[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; de) AppleWebKit/528.4+ (KHTML, like Gecko) Version/4.1 Safari/526.11.2";
const char S12656[] = "Mozilla/5.0(Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/4.1.3 Safari/533.19.4";
const char S12657[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10\\_4\\_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/4.1.3 Safari/533.19.4";
const char S12658[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12659[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12660[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-P5100 Build/IML74K)";
const char S12661[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; SAMSUNG GT-P5100/P5100BUALD5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12662[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12663[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SAMSUNG GT-P5100/P5100XWBLE4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12664[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P5100 Build/IMM76D)";
const char S12665[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P5100-ORANGE/P5100BVBLF3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12666[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12667[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P5100-ORANGE/P5100BVBLF5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12668[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12669[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; SAMSUNG GT-P5100/P5100BUBLH2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12670[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P5100-ORANGE/P5100BVBLH1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12671[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T779 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12672[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; sl-si; SAMSUNG GT-P5100/P5100BUALD4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12673[] = "Logos/1.1.3 (Linux; U; Android 4.0.3; en-GB; GT-P5100 Build/IML74K.P5100ZSALD5)";
const char S12674[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5100 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S12675[] = "com.google.android.youtube/3.5.5(Linux; U; Android 4.0.3; en_GB; GT-P5100 Build/IML74K) gzip";
const char S12676[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100-ORANGE/P5100BVBLE1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12677[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5100 Build/IML74K; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S12678[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hr-hr; SAMSUNG GT-P5100/P5100XWBLE2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12679[] = "Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5100 Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S12680[] = "Mozilla/5.0 (Linux; U; Android 4.0; xx-xx; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12681[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5100 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S12682[] = "Logos/2.1.2 (Linux; U; Android 4.0.3; en-GB; GT-P5100 Build/IML74K.P5100XWALD7)";
const char S12683[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; SAMSUNG GT-P5100/P5100XXBLE1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12684[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5100 Build/IMM76D; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S12685[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SAMSUNG GT-P5100/P5100XWBLE2.DM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12686[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG GT-P5100/P5100XWBLE4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12687[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; SAMSUNG GT-P5100/P5100BUALD5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12688[] = "FaithlifeStudyBible/2.0.11 (Linux; U; Android 4.0.3; ru-RU; GT-P5100 Build/IML74K.P5100XXALD7)";
const char S12689[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; GT-P5100-ORANGE/P5100BVALD1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12690[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; SAMSUNG GT-P5100/P5100BUALC1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12691[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-za; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.2.1-65 (Android 4.0.4; samsung GT-P5100 Build/IMM76D.P5100XWBLE4)";
const char S12692[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; SAMSUNG GT-P5100/P5100BUBLF1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12693[] = "Logos/2.1.2 (Linux; U; Android 4.0.4; zh-HK; GT-P5100 Build/IMM76D.P5100ZSBLH3)";
const char S12694[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 VoiceGO!(cybd08dd26fee94a5b4e5c27aa93e80fe56-0;Android,4.0.4,GT-P5100;3.6.0;1280x752,1.0;X;VoiceGo-TC;zh-TW)";
const char S12695[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S12696[] = "FaithlifeStudyBible/2.1.2 (Linux; U; Android 4.0.3; en-US; GT-P5100 Build/IML74K.P5100XWALD2)";
const char S12697[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; SAMSUNG GT-P5100/P5100XXBLI4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12698[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5100 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12699[] = "Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 4.0.3; SAMSUNG GT-P5100 Build/ICE_CREAM_SANDWICH_MR1);";
const char S12700[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ms-my; GT-P5100 Build/IML74K) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S12701[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; SAMSUNG GT-P5100/P5100XWBLE4.DM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12702[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5100 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12703[] = "Biblia/2.2.3 (Linux; U; Android 4.0.4; es-US; GT-P5100 Build/IMM76D.P5100XXBLH4)";
const char S12704[] = "Vyrso/2.2.3 (Linux; U; Android 4.0.3; en-GB; GT-P5100 Build/IML74K.P5100XWALD4)";
const char S12705[] = "MediaPlayer/2002/4.0.12 (Android 4.0.3; espresso10rf; GT-P5100; samsung; ; ; zh_CN)";
const char S12706[] = "Biblia/2.2.3 (Linux; U; Android 4.0.3; es-ES; GT-P5100 Build/IML74K.P5100UBALD4)";
const char S12707[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/1.9.10;FBDM/{density=1.0,width=1280,height=752};FBLC/en_GB;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P5100;FBSV/4.0.3;]";
const char S12708[] = "BBC News (AndroidApp; 1.0; 12) GT-P5100 (Android 4.0.4, SDK 15)";
const char S12709[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12710[] = "Biblia/2.1.2 (Linux; U; Android 4.0.3; es-US; GT-P5100 Build/IML74K.P5100UBALE1)";
const char S12711[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12712[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; SAMSUNG GT-P5100/P5100BUBLG4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12713[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12714[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/1.6.4;FBPN/com.facebook.katana;FBDV/GT-P5100;FBSV/4.0.3;FBDM/{density=1.0,width=1280,height=752};FBCR/MY MAXIS;FBLC/en_GB;FB_FW/1;]";
const char S12715[] = "Logos/2.2.8.1 (Linux; U; Android 4.0.3; en-GB; GT-P5100 Build/IML74K.P5100XWALE1)";
const char S12716[] = "Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-GB; GT-P5100 Build/IMM76D.P5100XWBLF1)";
const char S12717[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; SAMSUNG GT-P5100/P5100XWBLH2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12718[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Android_Gmarket Qoo10 MY_2.0.8_25(f6ce56dee1cbe648____X;GT-P5100;4.0.3)";
const char S12719[] = "Biblia/2.2.8.1 (Linux; U; Android 4.0.4; es-US; GT-P5100 Build/IMM76D.P5100UBBLJ1)";
const char S12720[] = "Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-GB; GT-P5100 Build/IMM76D.P5100XWBLE4)";
const char S12721[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; GT-P5100 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S12722[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K like Xoom) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12723[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG GT-P5100/P5100BUBLH2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12724[] = "nApps (Android 4.0.3; GT-P5100; Android.Line; 2.5.4)";
const char S12725[] = "Logos/2.2.8.1 (Linux; U; Android 4.0.3; en-GB; GT-P5100 Build/IML74K.P5100XWALE2)";
const char S12726[] = "Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-GB; GT-P5100 Build/IMM76D.P5100XXBLH5)";
const char S12727[] = "Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-GB; GT-P5100 Build/IMM76D.P5100XWBLF3)";
const char S12728[] = "VKAndroidApp/2.7.5-62 (Android 4.0.4; SDK 15; armeabi-v7a; samsung GT-P5100; ru)";
const char S12729[] = "Logos/2.2.8.1 (Linux; U; Android 4.0.4; es-ES; GT-P5100 Build/IMM76D.P5100UBBLH2)";
const char S12730[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; gbr-gbr; GT-P5100 Build/IML74K) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S12731[] = "Groupon/2.0.2414 (Android 4.0.3; Samsung Espresso10rf / Samsung GT-P5100; )[preload=false;locale=en_GB;clientidbase=android-samsung]";
const char S12732[] = "Vyrso/2.2.8.1 (Linux; U; Android 4.0.3; en-AU; GT-P5100 Build/IML74K.P5100XWALE3)";
const char S12733[] = "Showtimezz@MY/1.4.8 (Android 4.0.3; samsung-GT-P5100%404.0.3 Build/15";
const char S12734[] = "Mrd/1.1.0 (Linux; U; Android 4.0.3; samsung GT-P5100) jp.co.paperboy.picto/37";
const char S12735[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-GB; GT-P5100)";
const char S12736[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 APPDIY/2.1.10 (zh-CN;800*1232*1.0;X/X/X) zhoujielun/2.2.0.4491 (Id:298,Date:2013-01-08 20:03:27,Channel:anzhuoshichang)";
const char S12737[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S12738[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-za; GT-P5100 Build/IMM76D) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 Whisk/3.8.1204.57";
const char S12739[] = "Logos/2.2.10 (Linux; U; Android 4.0.3; es-US; GT-P5100 Build/IML74K.P5100UBALD5)";
const char S12740[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; SAMSUNG GT-P5100/P5100XWALD7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12741[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ph; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12742[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P5100 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S12743[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IceCreamSandwich_mr1) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S12744[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ph; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Vet-Encoding: gzip,deflate";
const char S12745[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.2.1-65 (Android 4.0.3; samsung GT-P5100 Build/IML74K.P5100XWALD7)";
const char S12746[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/IMM76D/534.30 (KHTML, like Gecko) Verse Gecko) Version/4.0 Safari/534.30";
const char S12747[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K; samsung; mediatap) 752X1280 samsung GT-P5100 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12748[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; SAMSUNG GT-P5100/P5100XWALE2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12749[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S12750[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5100 Build/IMM76D) AppleWebKit/534ment;      this.validation = val4.0 Safari/534.30";
const char S12751[] = "MediaPlayer/2002/4.5.0 (Android 4.0.3; espresso10rf; GT-P5100; samsung; ; ; en_GB)";
const char S12752[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-za; GT-P5100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S12753[] = "com.google.android.youtube/3.5.5(Linux; U; Android 4.0.4; en_GB; GT-P5100 Build/IMM76D) gzip";
const char S12754[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.2 Version/4.0 Mobile Safari/530.17";
const char S12755[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P5100-ORANGE/P5100BVBLL5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12756[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-T779 Build/IMM76D)";
const char S12757[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SAMSUNG GT-P5100/P5100XXBLK2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S12758[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36";
const char S12759[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36";
const char S12760[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe)";
const char S12761[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe)";
const char S12762[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe),gzip(gfe)";
const char S12763[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Iron/29.0.1600.1 Chrome/29.0.1600.1 Safari/537.36";
const char S12764[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/29.0.1600.1 Chrome/29.0.1600.1 Safari/537.36";
const char S12765[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S12766[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S12767[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144";
const char S12768[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141";
const char S12769[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 iPod4,1 BingWeb/3.03.1428.X";
const char S12770[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 6_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.1;FBSS/2; FBCR/unavailable;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S12771[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 [FBAN/FBIOS;FBAV/5.4.2;FBBV/X;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]";
const char S12772[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 [FBAN/FBIOS;FBAV/5.4.2;FBBV/X;FBDV/iPod5,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]";
const char S12773[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S12774[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 iPod5,1 BingWeb/3.03.1428.X";
const char S12775[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146";
const char S12776[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 iPod4,1 BingWeb/3.03.1428.X";
const char S12777[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 [FBAN/FBIOS;FBAV/5.5;FBBV/X;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.1.2;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBOP/0]";
const char S12778[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 iPod5,1 BingWeb/3.03.1428.X";
const char S12779[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S12780[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 6_1_2 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.1.2;FBSS/2; FBCR/unavailable;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S12781[] = "Mozilla/5.0 (iPod4,1; iPhone; U; CPU OS 6_1_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N81AP (gzip)";
const char S12782[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329";
const char S12783[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 iPod4,1 BingWeb/3.03.1428.X";
const char S12784[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 iPod5,1 BingWeb/3.03.1428.X";
const char S12785[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 6_1_3 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.1.3;FBSS/2; FBCR/unavailable;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S12786[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S12787[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 bline/1.06 (iPhone OS 6.1.3, iPod touch)";
const char S12788[] = "Mozilla/5.0 (iPhone; U; CPU iPhone 6_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.8 (iPod4,1/6.1.3)";
const char S12789[] = "AppCake 3.8 (iPod touch; iPhone OS 6.1.2; en_US)";
const char S12790[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B5126b";
const char S12791[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Safari IPhoneAppVersion/2.5.2 IPhoneWebkitWrapper BillingSupported";
const char S12792[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/8536.25";
const char S12793[] = "Mozilla/5.0 (iPod4,1; iPhone; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/N81AP (gzip)";
const char S12794[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 [FBAN/FBIOS;FBAV/6.3;FBBV/X;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.1.3;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBOP/1]";
const char S12795[] = "AppCake 3.8 (iPod touch; iPhone OS 6.1; en_US)";
const char S12796[] = "Mozilla/5.0 (iPod5,1; iPhone; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/N78AP (gzip)";
const char S12797[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari IPhoneAppVersion/2.5.2 IPhoneWebkitWrapper BillingSupported";
const char S12798[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari IPhoneAppVersion/2.6.1 IPhoneWebkitWrapper BillingSupported";
const char S12799[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S12800[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S12801[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S12802[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S12803[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S12804[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S12805[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36,gzip(gfe)";
const char S12806[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S12807[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Iron/29.0.1600.1 Chrome/29.0.1600.1 Safari/537.36";
const char S12808[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.289 Version/12.00";
const char S12809[] = "Mozilla/5.0 (Windows NT 5.1; en; rv:2.0) Gecko/X Firefox/4.0 Opera 12.00";
const char S12810[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.289 Version/12.00";
const char S12811[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.10.289 Version/12.00";
const char S12812[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.289 Version/12.00";
const char S12813[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; en) Opera 12.00";
const char S12814[] = "Opera/9.80 (Windows NT 5.2; U; en) Presto/2.10.289 Version/12.00";
const char S12815[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5328); ru) Presto/2.10.289 Version/12.00";
const char S12816[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.289 Version/12.00";
const char S12817[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.289 Version/12.00";
const char S12818[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.10.289 Version/12.00";
const char S12819[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.10.289 Version/12.00";
const char S12820[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.10.289 Version/12.00";
const char S12821[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.289 Version/12.00";
const char S12822[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.289 Version/12.00";
const char S12823[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.289 Version/12.00";
const char S12824[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.289 Version/12.00";
const char S12825[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.289 Version/12.00";
const char S12826[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.289 Version/12.00,gzip(gfe) (via translate.google.com)";
const char S12827[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.289 Version/12.00";
const char S12828[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.289 Version/12.00";
const char S12829[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.289 Version/12.00";
const char S12830[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.289 Version/12.00";
const char S12831[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.289 Version/12.00";
const char S12832[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.289 Version/12.00";
const char S12833[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4930); ru) Presto/2.10.289 Version/12.00";
const char S12834[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.289 Version/12.00";
const char S12835[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.289 Version/12.00";
const char S12836[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.289 Version/12.00";
const char S12837[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5667); ru) Presto/2.10.289 Version/12.00";
const char S12838[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.289 Version/12.00";
const char S12839[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.289 Version/12.00";
const char S12840[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.289 Version/12.00";
const char S12841[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5161); ru) Presto/2.10.289 Version/12.00";
const char S12842[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.289 Version/12.00";
const char S12843[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.289 Version/12.00";
const char S12844[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5676); ru) Presto/2.10.289 Version/12.00";
const char S12845[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.289 Version/12.00";
const char S12846[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.289 Version/12.00";
const char S12847[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.289 Version/12.00";
const char S12848[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.289 Version/12.00";
const char S12849[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.289 Version/12.00";
const char S12850[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.289 Version/12.00";
const char S12851[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5299); ru) Presto/2.10.289 Version/12.00";
const char S12852[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5218); ru) Presto/2.10.289 Version/12.00";
const char S12853[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5660); ru) Presto/2.10.289 Version/12.00";
const char S12854[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02746); ru) Presto/2.10.289 Version/12.00";
const char S12855[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5230); ru) Presto/2.10.289 Version/12.00";
const char S12856[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.289 Version/12.00";
const char S12857[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5650); ru) Presto/2.10.289 Version/12.00";
const char S12858[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5217); ru) Presto/2.10.289 Version/12.00";
const char S12859[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.289 Version/12.00";
const char S12860[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.289 Version/12.00";
const char S12861[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5336); ru) Presto/2.10.289 Version/12.00";
const char S12862[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5288); ru) Presto/2.10.289 Version/12.00";
const char S12863[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.289 Version/12.00";
const char S12864[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.289 Version/12.00";
const char S12865[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.10.289 Version/12.00";
const char S12866[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5300); ru) Presto/2.10.289 Version/12.00";
const char S12867[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.10.289 Version/12.00";
const char S12868[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.289 Version/12.00";
const char S12869[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.10.289 Version/12.00";
const char S12870[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4661); ru) Presto/2.10.289 Version/12.00";
const char S12871[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.10.289 Version/12.00";
const char S12872[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.10.289 Version/12.00";
const char S12873[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.289 Version/12.00,gzip(gfe) (via translate.google.com)";
const char S12874[] = "Opera/9.80 (Windows NT 5.1; U; en) Presto/2.10.289 Version/12.01";
const char S12875[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5278); ru) Presto/2.10.289 Version/12.00";
const char S12876[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.289 Version/12.01";
const char S12877[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.289 Version/12.01";
const char S12878[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.10.289 Version/12.01";
const char S12879[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4930); ru) Presto/2.10.289 Version/12.01";
const char S12880[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.289 Version/12.01";
const char S12881[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.289 Version/12.01";
const char S12882[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5650); ru) Presto/2.10.289 Version/12.01";
const char S12883[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.289 Version/12.01";
const char S12884[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.10.289 Version/12.01";
const char S12885[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.289 Version/12.01";
const char S12886[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.289 Version/12.01";
const char S12887[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.10.289 Version/12.01";
const char S12888[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.289 Version/12.01";
const char S12889[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.289 Version/12.01";
const char S12890[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.289 Version/12.01";
const char S12891[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.289 Version/12.01";
const char S12892[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.10.289 Version/12.01";
const char S12893[] = "Opera/9.80 (Windows NT 5.2; U; en) Presto/2.10.289 Version/12.01";
const char S12894[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.289 Version/12.01";
const char S12895[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5667); ru) Presto/2.10.289 Version/12.01";
const char S12896[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.289 Version/12.01";
const char S12897[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.289 Version/12.01";
const char S12898[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.10.289 Version/12.01";
const char S12899[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.289 Version/12.01";
const char S12900[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.289 Version/12.01,gzip(gfe) (via translate.google.com)";
const char S12901[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.289 Version/12.01";
const char S12902[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.10.289 Version/12.00";
const char S12903[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.289 Version/12.01";
const char S12904[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.289 Version/12.01";
const char S12905[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.289 Version/12.01";
const char S12906[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.289 Version/12.01";
const char S12907[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.289 Version/12.01";
const char S12908[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.289 Version/12.01";
const char S12909[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.289 Version/12.01";
const char S12910[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.289 Version/12.01";
const char S12911[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5218); ru) Presto/2.10.289 Version/12.01";
const char S12912[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.289 Version/12.01";
const char S12913[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5328); ru) Presto/2.10.289 Version/12.01";
const char S12914[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.289 Version/12.01";
const char S12915[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.289 Version/12.01";
const char S12916[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5676); ru) Presto/2.10.289 Version/12.01";
const char S12917[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.289 Version/12.01";
const char S12918[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.289 Version/12.01";
const char S12919[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.289 Version/12.01";
const char S12920[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5660); ru) Presto/2.10.289 Version/12.01";
const char S12921[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.10.289 Version/12.01";
const char S12922[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.10.289 Version/12.01";
const char S12923[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5288); ru) Presto/2.10.289 Version/12.01";
const char S12924[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.289 Version/12.01";
const char S12925[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5966); ru) Presto/2.10.289 Version/12.01";
const char S12926[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.289 Version/12.01";
const char S12927[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.289 Version/12.00";
const char S12928[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.289 Version/12.00";
const char S12929[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03672); ru) Presto/2.10.289 Version/12.00";
const char S12930[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.289 Version/12.01";
const char S12931[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5217); ru) Presto/2.10.289 Version/12.01";
const char S12932[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5278); ru) Presto/2.10.289 Version/12.01";
const char S12933[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.10.289 Version/12.01";
const char S12934[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.289 Version/12.01";
const char S12935[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5336); ru) Presto/2.10.289 Version/12.01";
const char S12936[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03672); ru) Presto/2.10.289 Version/12.01";
const char S12937[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.289 Version/12.01";
const char S12938[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5299); ru) Presto/2.10.289 Version/12.01";
const char S12939[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5966); ru) Presto/2.10.289 Version/12.00";
const char S12940[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.10.289 Version/12.01";
const char S12941[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.10.289 Version/12.00";
const char S12942[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5325); ru) Presto/2.10.289 Version/12.00";
const char S12943[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5300); ru) Presto/2.10.289 Version/12.01";
const char S12944[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.289 Version/12.01";
const char S12945[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5325); ru) Presto/2.10.289 Version/12.01";
const char S12946[] = "Opera/9.80 (Windows NT 5.1; U; en) Presto/2.10.289 Version/12.01 AppEngine-Google; (+http://code.google.com/appengine; appid: schmittify)";
const char S12947[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.289 Version/12.01,gzip(gfe)";
const char S12948[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02552); ru) Presto/2.10.289 Version/12.00";
const char S12949[] = "Opera/9.80 (Windows NT 5.1; U; compatible; MSIE 7.0; Windows NT 6.0; bg) Presto/2.10.289 Version/12.00";
const char S12950[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.289 Version/12.00,gzip(gfe) (via translate.google.com)";
const char S12951[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5230); ru) Presto/2.10.289 Version/12.01";
const char S12952[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.10.289 Version/12.01";
const char S12953[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.289 Version/12.00,gzip(gfe)";
const char S12954[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.289 Version/12.01";
const char S12955[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.289 Version/12.00";
const char S12956[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.289 Version/12.01";
const char S12957[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5997); ru) Presto/2.10.289 Version/12.01";
const char S12958[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.289 Version/12.00";
const char S12959[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.289 Version/12.00";
const char S12960[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.289 Version/12.01";
const char S12961[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.289 Version/12.01";
const char S12962[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4661); ru) Presto/2.10.289 Version/12.01";
const char S12963[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5745); ru) Presto/2.10.289 Version/12.00";
const char S12964[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5997); ru) Presto/2.10.289 Version/12.00";
const char S12965[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.289 Version/12.01";
const char S12966[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.289 Version/12.00";
const char S12967[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5784); ru) Presto/2.10.289 Version/12.00";
const char S12968[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5880); ru) Presto/2.10.289 Version/12.00";
const char S12969[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5880); ru) Presto/2.10.289 Version/12.01";
const char S12970[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5784); ru) Presto/2.10.289 Version/12.01";
const char S12971[] = "Opera/12.0(Windows NT 5.1;U;en)Presto/22.9.168 Version/12.00";
const char S12972[] = "Opera/12.0(Windows NT 5.2;U;en)Presto/22.9.168 Version/12.00";
const char S12973[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6011); ru) Presto/2.10.289 Version/12.00";
const char S12974[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5993); ru) Presto/2.10.289 Version/12.00";
const char S12975[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.289 Version/12.01";
const char S12976[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.289 Version/12.00";
const char S12977[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02746); ru) Presto/2.10.289 Version/12.01";
const char S12978[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5981); ru) Presto/2.10.289 Version/12.00";
const char S12979[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03780); ru) Presto/2.10.289 Version/12.00";
const char S12980[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02552); ru) Presto/2.10.289 Version/12.01";
const char S12981[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5993); ru) Presto/2.10.289 Version/12.01";
const char S12982[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5977); ru) Presto/2.10.289 Version/12.01";
const char S12983[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.289 Version/12.01";
const char S12984[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6083); ru) Presto/2.10.289 Version/12.00";
const char S12985[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5745); ru) Presto/2.10.289 Version/12.01";
const char S12986[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5981); ru) Presto/2.10.289 Version/12.01";
const char S12987[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.289 Version/12.00";
const char S12988[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6089); ru) Presto/2.10.289 Version/12.00";
const char S12989[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6089); ru) Presto/2.10.289 Version/12.01";
const char S12990[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6081); ru) Presto/2.10.289 Version/12.00";
const char S12991[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02652); ru) Presto/2.10.289 Version/12.00";
const char S12992[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.10.289 Version/12.01";
const char S12993[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02652); ru) Presto/2.10.289 Version/12.01";
const char S12994[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6011); ru) Presto/2.10.289 Version/12.01";
const char S12995[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5977); ru) Presto/2.10.289 Version/12.00";
const char S12996[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6037); ru) Presto/2.10.289 Version/12.01";
const char S12997[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 6003); ru) Presto/2.10.289 Version/12.00";
const char S12998[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6081); ru) Presto/2.10.289 Version/12.01";
const char S12999[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.289 Version/12.00";
const char S13000[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.289 Version/12.01";
const char S13001[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02560); ru) Presto/2.10.289 Version/12.00";
const char S13002[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 6007); ru) Presto/2.10.289 Version/12.00";
const char S13003[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 6007); ru) Presto/2.10.289 Version/12.01";
const char S13004[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 6008); ru) Presto/2.10.289 Version/12.01";
const char S13005[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6054); ru) Presto/2.10.289 Version/12.00";
const char S13006[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5989); ru) Presto/2.10.289 Version/12.01";
const char S13007[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13008[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13009[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; GT-S5570L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13010[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; GT-S5570L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13011[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13012[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-S5570-ORANGE/S5570BVKT1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13013[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570I Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S13014[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5570 Build/GINGERBREAD)";
const char S13015[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; GT-S5570B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13016[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5570 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13017[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; he-il; GT-S5570B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13018[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5570-ORANGE/S5570BVKQ4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13019[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13020[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; SAMSUNG GT-S5570/S5570BUKS2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13021[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-S5570 Build/GINGERBREAD)";
const char S13022[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-S5570L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13023[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5570 Build/GINGERBREAD)";
const char S13024[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-nz; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13025[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5570I Build/GINGERBREAD)";
const char S13026[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-S5570B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13027[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S13028[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5570 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S13029[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13030[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; SAMSUNG GT-S5570/S5570BUKS1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13031[] = "Mozilla/5.0 (Linux; U; en-gb; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S13032[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; GT-S5570 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13033[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5570L Build/GINGERBREAD)";
const char S13034[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-S5570 Build/GINGERBREAD)";
const char S13035[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-S5570L Build/GINGERBREAD)";
const char S13036[] = "Mozilla/5.0 (Linux; U; Android 2.3.7.1; vi-vn; GT-S5570 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13037[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; GT-S5570 Build/GRI40)";
const char S13038[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT-S5570 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13039[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5570B Build/GINGERBREAD)";
const char S13040[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5570B Build/GINGERBREAD)";
const char S13041[] = "Mozilla/5.0 (Linux; U; Android 2.3.8; en-us; GT-S5570 Build/GRI40; CyanMobile-eXperience) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13042[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5570 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S13043[] = "Mozilla/5.0 (Linux; U; es-es; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S13044[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13045[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570-ORANGE/S5570BVKT4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13046[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13047[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13048[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5570/S5570BUKT2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13049[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570-ORANGE/S5570BVKT5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13050[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13051[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-S5570/S5570MJKT3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13052[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5570/S5570BUKT4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13053[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570-ORANGE/S5570BVKT6 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13054[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5570 Build/GWK74; New Experience v6.0 FINAL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13055[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT-S5570 Build/GINGERBREAD; JellyBread v2.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13056[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Full AOSP on Toro-VZW Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13057[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Dark Jelly Belly Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13058[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; msm8960 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13059[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Dark JellyBellyG1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13060[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Toasted Marshmallow Build/Toasted Marshmallow) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13061[] = "Mozilla/5.0 (Linux; U; Android 4.1.5; id-id; TREQ_A10_Pocket2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13062[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; SKY Limo Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13063[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC_ENRC2B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13064[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N101 DUAL CORE2 V11 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13065[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; 4S Build/F88) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13066[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; 9300 Build/F9AD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13067[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; N101 DUAL CORE 2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13068[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; X720D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13069[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; S1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13070[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; RAPID5 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13071[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; POP9.7G2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13072[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HC801 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13073[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; msm8960 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13074[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; Pro 10 dual core Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13075[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; DPS Dream 9 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13076[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Q50 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13077[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nb-no; 7024EP Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13078[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nb-no; 9748EP Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13079[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SP0718 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13080[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; ALEXIS RX5 DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13081[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; X2-IPS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13082[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Crystal Build/JRO03C.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13083[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Rikomagic MK802III Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13084[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; RAPID5H Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13085[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; SPICE IV Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13086[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; U1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13087[] = "Mozilla/5.0 (Linux; U; Android 4.1; es-es; 9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13088[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; POP 7.1HD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13089[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ENERGY i10 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13090[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Full Android on PrimoC Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13091[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-P600 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13092[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Tab9.7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13093[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nb-no; 8028EP Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13094[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; M1001 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13095[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; STUDIO Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13096[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; AllviewCity Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13097[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; RAPID5-H Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13098[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; I9277 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13099[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; imito am802 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13100[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Q50R Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13101[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; CAPTIVA PAD 10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13102[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; i907c Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13103[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Unique7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13104[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ROCK X Build/ROCK X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13105[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Fast Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13106[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-uk; cyclone Titan Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13107[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; S2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13108[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; M758A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13109[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; F8 SUP2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13110[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; platinum Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13111[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; Slick973V2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13112[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; E-Boda Supreme Dual Core X190 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13113[] = "Mozilla/5.0 (Linux; U; Android 4.1.4; en-us; Note3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13114[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IMAGINE 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13115[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; EPAD Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13116[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Hold X Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13117[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; vangogh Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13118[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Tablet Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13119[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TP7-1500DC-metal Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13120[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; TA10100 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13121[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; E74 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13122[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TABLOID Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13123[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; NBLT10-A4 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13124[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; AQUILA 080-1008 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13125[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; N10-2CPU Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13126[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; U101 Build/FR-MX68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13127[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; AQUILA 097-1006 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13128[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; PC728 Build/PC728) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13129[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Papyre Pad 972 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13130[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TM-MID940 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13131[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; P10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13132[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N70HD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13133[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; I9420 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13134[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; M973W Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13135[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; Tablet7525B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13136[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TAGIMID 140 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13137[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; A10-2CPU Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13138[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PUBLIC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13139[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; P0V_TV-HDMI-KB-01 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13140[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; ConCorde tab PLAY Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13141[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Unique10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13142[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TORNADO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13143[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; Plug 10.1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13144[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PULID F13 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13145[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; TX85 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13146[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; CTAB2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13147[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; rk30 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13148[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1 Build/JRO03C)";
const char S13149[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; ATP7713 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13150[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; G1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13151[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Royqueen 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13152[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; Spirit S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13153[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; U1 Pro Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13154[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-at; TABLO Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13155[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; TA8300 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13156[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 9706 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13157[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 9300mini Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13158[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; T-1006 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13159[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Primux_Omega Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13160[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Tablet-PC-4 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13161[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D725 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13162[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A10-2cpu Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13163[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; KTab7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13164[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; DROIDZ Portal Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13165[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; M972g Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13166[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; MyAudio Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13167[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Mobiltab Sleek 5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13168[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; KTab9.7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13169[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; B63 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13170[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; PAD 9718DR Build/PAD 9718DR) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13171[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 97R Build/97R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13172[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; S8+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13173[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; mini STUDIO Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13174[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; androVM for VirtualBox ('Tablet' version) Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13175[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; Ultimate 10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13176[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; mfld_pr2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13177[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EX'3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13178[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Thrive Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13179[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Cute  X Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13180[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; RC9724C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13181[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; U30GT mini Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13182[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; D9702 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13183[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Primux_Alpha Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13184[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EX'Note ? Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13185[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; Classic103 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13186[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; DIT7070 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13187[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; B94M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13188[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; RAPAX 070LE Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13189[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SC-72 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13190[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Android_53 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13191[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; prestige7L_4.1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13192[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; Gpad Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13193[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; TR736 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13194[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; LETAB903 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13195[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NINE T7800 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13196[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; M100 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13197[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; BRVTX11D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13198[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Pandora 9.10 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13199[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; T10 3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13200[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; GPLUS N930+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13201[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; iconBIT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13202[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; VB100a Pro Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13203[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; A07I-D15A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13204[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S701-R2A-1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13205[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NC-eLite Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13206[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MID08S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13207[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; Obreey SURFpad Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13208[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Mobiltab Sleek5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13209[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; I699 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13210[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Tab8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13211[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13212[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; MID-801 Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13213[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nb-no; 1038EP Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13214[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13215[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-uk; 777TPC Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13216[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; Shogun 10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13217[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; Ultimix99 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13218[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Visture V4 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13219[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; OV-EDUTAB Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13220[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; CTP980 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13221[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CW V8 Dual Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13222[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Decontaminate V3.0 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13223[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; TB7020+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13224[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MID1002 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13225[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; PC677 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13226[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; Quicki-811 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13227[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Teraware K977 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13228[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; SAMURAI Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13229[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; NEXO Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13230[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N10-2cpu Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13231[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Cat Tablet CSG2 7.1J0 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13232[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Tablet X2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13233[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 9930 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13234[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; B9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13235[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; DIN09.97 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13236[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13237[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HD470 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13238[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; PC1002 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13239[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; M74 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13240[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Androtab 7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13241[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; L9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13242[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Android Note II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13243[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; Ak8462 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13244[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; 10DCIX Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13245[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; 9458DCB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13246[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Phablet PII Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13247[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; FCB tablet Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13248[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; MMS-874.dualcore Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13249[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Android_53DC Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13250[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Phablet PIII Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13251[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; M758C1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13252[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Neon9 Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13253[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; LUXURY Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13254[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ie; JB7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13255[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; SELECT 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13256[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; CAPTIVA PAD 10.1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13257[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; VS TOUCHTAB 9.7RT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13258[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TAB700 NV Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13259[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TAB7DUAL 8GB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13260[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N9977 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13261[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fa-ir; G4 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13262[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; A9724 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13263[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S71 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13264[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13265[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; EVO LTE Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13266[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Full Android on Grouper Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13267[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HTC DLX_WLJ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13268[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; ConCorde tab NOVUM Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13269[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; X817 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13270[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-uk; SWIPE Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13271[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; EVOLVE_FX420 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13272[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; Q7pro-H10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13273[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; bg-bg; M976 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13274[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; California Access TAB MM 1001 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13275[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; sl-si; XP791 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13276[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; EVOLVE_FX520 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13277[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; FT-2008 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13278[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MID9701 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13279[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; TX12D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13280[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; F1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13281[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_Y300C/Y300CV100R001C92B166; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S13282[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-co; aigopad Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13283[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; - Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13284[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; AT400 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13285[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; PAN 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13286[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A85-2cpu Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13287[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; skyfire S+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13288[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NID-7006A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13289[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; RMD-1026 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13290[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; MaxiIQ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13291[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; PENTAGRAM TAB 8.3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13292[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; TAB7B_3G Build/WEXLER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13293[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Newsmy P72 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13294[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; WB8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13295[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; A8826 Build/ICS.A8826.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13296[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13297[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; M15GC702P Build/PD715Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13298[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ENERGY s9 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13299[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; GTD866 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13300[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; Maxx II Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13301[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; Diamond_Lite Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13302[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; mid Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13303[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; GiONEE_Infinity_GN708 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13304[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SC-75JB Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13305[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 975TPCII Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13306[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ACGM816KC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13307[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; M7R4C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13308[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; Android Note II Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13309[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13310[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TBDB763 Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13311[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; NTT 611 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13312[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ph; T-746 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13313[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TPC-EA901 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13314[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; minipad Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13315[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; DPS Dual 7 Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13316[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; maTab LION Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13317[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M1400 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13318[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Xtab1081HD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13319[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; TELETALK_X110Q Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13320[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MOMO20-AU Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13321[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fa-ir; PC7558 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13322[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; B943 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13323[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M736 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13324[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; da-dk; Ennovia TAB E309(V2) Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13325[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MIDCD97 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13326[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D7018 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13327[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC DLX_WL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13328[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; YL-Coolpad 9960 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13329[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; iM970 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13330[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; msm8625 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13331[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; da-dk; M972 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13332[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; es209ra Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13333[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; crane-evb Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13334[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; AMOI M8228 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13335[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Speed_MB-R2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13336[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PIRANHA BUSINESS III TAB 10.1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13337[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ADM712KC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13338[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; MP973G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13339[] = "Mozilla/5.0 (Linux; U; Android 4.1.0; zh-tw; HWA680 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13340[] = "Mozilla/5.0 (Linux; U; Android 4.1.4; en-us; sanstar6577_ics2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13341[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; Tablet Mint Max10 DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13342[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; Kruger Matz 1010 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13343[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; IPRO i5S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13344[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Tab70s Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13345[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; Max_X2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13346[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Mercury X Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13347[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M10+ Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13348[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; SPC P2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13349[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ENERGY s7 Dual Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13350[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; RMD-877G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13351[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Boost Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13352[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; 1028DCS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13353[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; JPAD730 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13354[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; androVM for VirtualBox ('Tablet' version with phone caps) Build/JRO03S) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13355[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; TB7900 Build/TB7900) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13356[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; M78A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13357[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CT1006 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13358[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; MITO t300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13359[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; MT-801 Build/MT-801) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13360[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; klu1506 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13361[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; A9600 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13362[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Primux_SonoraDual Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13363[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 9706-T Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13364[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Troy Duos 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13365[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; YL-Coolpad_5930/4.1.016.X.5930;540*960;CTC/2.0) CoolpadWebkit/534.30";
const char S13366[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MOMO20 Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13367[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; TAB709D-3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13368[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MT6517-MT6517-H369X/1.0 Android/4.1.1 Release/04.02.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13369[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; NPC001 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13370[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TBDB863 Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13371[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; PC906T Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13372[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; FUSION Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13373[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; iTab971 Dual Core Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13374[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; Shogun 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13375[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; R10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13376[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; ASK 780SP HD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13377[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; m6 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13378[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NEO-X5 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13379[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; Padphone 3G-61 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13380[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; V811 Core4 Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13381[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MT8377 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13382[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; CTAB101 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13383[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; id-id; PICOpad_GFI Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13384[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; RMP-600 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13385[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; NetTAB SPACE II Plus Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13386[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; N9zero Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13387[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; D1002 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13388[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MOBILTAB sleek 10.1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13389[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; g08refem618 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13390[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Pascal 2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13391[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EZPAD 830DC Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13392[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; PC98IPS Dual Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13393[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; A8849 Build/ICS.A8849.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13394[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; T-1005 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13395[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; PRO9-A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13396[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; TOUCHTAB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13397[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TBD1083B Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13398[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; Tab Play 002 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13399[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; H2-9506CM Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13400[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MT-GT-T7100/1.0 Android/4.1.1 Release/04.02.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13401[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MT6517 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13402[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Woxter Tablet PC98IPS DUAL Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13403[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; CTP978 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13404[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMID706 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13405[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; TM903 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13406[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; i708 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13407[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; S1 Build/JRO03H)";
const char S13408[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AMOI N820 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13409[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; H-SP 9702 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13410[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; WICKED-H Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13411[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; SINA-1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13412[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; avio_sen_S5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13413[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; its2520 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13414[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB-R3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13415[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Ex_Note Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13416[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; PENTAGRAM TAB 8.5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13417[] = "Mozilla/5.0 (Linux; U; Android 4.1.4; es-es; Note3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13418[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; 10BPE0 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13419[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; MID72C Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13420[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; TY-Q8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13421[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; HLV-T1001W Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13422[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; OS988 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13423[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N83-2cpu Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13424[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; KN_Mobile_QT_A08 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13425[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; PT10.1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13426[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; AEON Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13427[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; ZP958 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13428[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_Y300C/Y300CV100R001C92B168; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S13429[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; IMO S99 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13430[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; VP70 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13431[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; MR18 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13432[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; PAD701 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13433[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; GS3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13434[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; Pro dual 10 core Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13435[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; Tab1030 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13436[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fa-ir; TAX Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13437[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; CTAB743 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13438[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; i-mobile i-style 8 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13439[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; Luna TAB10-150 Build/Luna TAB10-150) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13440[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Mitraveler 7D8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13441[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMID720 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13442[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; N70DC-S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13443[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SP0718 Build/JRO03H)";
const char S13444[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; REVO MAX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13445[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; B92M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13446[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IT-M804RC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13447[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; L36h Build/10.1.A.1.350) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 BaiduBoxApp/3.6_130g";
const char S13448[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; L003 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13449[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; Aveo NOTE II Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13450[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; jpad802 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13451[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; g08refem617 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13452[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; MW0710 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13453[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Tablet Planet II-v3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13454[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; AML-MX REF Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13455[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; S33D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13456[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; M73 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13457[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; TB1010 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13458[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; Connspeed IP71 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13459[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; RX3-DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13460[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Wb7-L Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13461[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M772 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13462[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; PANDORA 9.20 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13463[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 9D181 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13464[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; EVO LTE Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13465[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; ALEXIS-RX5DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13466[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; QRD-8625DSDS Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13467[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; x5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13468[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; MIDM-N81S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13469[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; N7Zero Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13470[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; iTab1011 Dual Core Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13471[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; G1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13472[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ACGM727KC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13473[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; LA-Q1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13474[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-uk; MP1041 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13475[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IT-TITAN3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13476[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 5930 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13477[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; BTPC-1011-DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13478[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; Nexo 10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13479[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAB R76.2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13480[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PICOpad GEW Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13481[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; F918 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13482[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; TAB9 3G V8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13483[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; LA-M1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13484[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; F600 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13485[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MP7007 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13486[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; QBA757 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13487[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; HD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13488[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; H9500+ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13489[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Sweet M720 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13490[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ACGM727KC_N Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13491[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Energyi8Dual Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13492[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; SP-120 DC Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13493[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; aussiePAD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13494[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MiSS Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13495[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; 97S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13496[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; mfld_pr3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13497[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; G701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13498[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; F220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13499[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; OBEE_OS969 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13500[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 97ST Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13501[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; id-id; TREQ Turbo Build/FR-MX68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13502[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 4u + Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13503[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; DFP8007 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13504[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CHUWI V99 Build/CHUWI V99) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13505[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; zh-tw; GT-N7077 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13506[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID-X4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13507[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TLS-V4 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13508[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; i90 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13509[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; M704S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13510[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AM818 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13511[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 4U+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13512[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-ca; U30GT MINI Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13513[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; SSW-T7A41HD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13514[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; N710 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13515[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; PAD1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13516[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Wanden.co.uk_N90 CZ Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13517[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SX9722 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13518[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MD-706S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13519[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NEO-G4 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13520[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; C3_77_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13521[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GF-2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13522[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; Core 10.1 3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13523[] = "Mozilla/5.0 (Linux; U; Android 4.1.1_JELLYCUBE_V1; fr-ca; GT-N700 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13524[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MID1003 Build/FR-MX68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13525[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ADM8000KP_A Build/JRO03H)";
const char S13526[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; Ultimix94 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13527[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; VB80a Pro Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13528[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TABLET 8.5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13529[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; P102 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13530[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; v1801mt77 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13531[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Woxter Tablet PC 85IPS Dual Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13532[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; RC9730C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13533[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID-709S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13534[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; DINO7.17 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13535[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Intellibook Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13536[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XPAD-2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13537[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; KTab10.1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13538[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; QPAD9700 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13539[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; CE15012-16 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13540[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; DP-101 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13541[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fa-ir; G4+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13542[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HUAWEI D2-0082 Build/HuaweiD2-0082) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13543[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; MW0731 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13544[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Greenstone Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13545[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; E8050HG Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13546[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; M84g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13547[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; PER5274B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13548[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MT12 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13549[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Neon7 Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13550[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; PREMIER Build/PREMIER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13551[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AXPAD 7E01 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13552[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MID1028 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13553[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MID1001 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13554[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ViewPad 100N Pro Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13555[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Bryte Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13556[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; Surfer 8_02 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13557[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-de; M-12 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13558[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; SIROCO 4 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13559[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; KIRK3-A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13560[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; ConCorde tab Speed Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13561[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; FreeTAB1003 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13562[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; da-dk; EN-TAB327 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13563[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; T-763 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13564[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; K47 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13565[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; TM703 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13566[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MP717 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13567[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N900 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13568[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_D2-2010/D2-2010V100R001C92B132) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13569[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S750L Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13570[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nitro 400D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13571[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ID1018WTA Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13572[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-; S2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13573[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MT6517-MT6517-H369X/1.0 Android/4.1.1 Release/05.09.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13574[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; TAB700 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13575[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; DITX Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13576[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-I710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13577[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; he-il; u_force_U7I Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13578[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; NGM_Forward_Infinity Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13579[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Q88 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13580[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I759 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13581[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MID703 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13582[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MC7411B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13583[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Touchlet X10.dual Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13584[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; BRVQ750D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13585[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; ETL-T970 Build/ETL-T970) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13586[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID82C Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13587[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; HFM7002KD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13588[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MD700 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13589[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TechniPad 10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13590[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; 10BPEOH Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13591[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Grand DUOS  Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13592[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; Space Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13593[] = "MT6589_TD/V1 Linux/3.4.0 Android/4.1.2  Release/12.24.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2 System/Android 4.1.2;";
const char S13594[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Fast1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13595[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_Y300C/Y300CV100R001C92B165; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S13596[] = "Tablet-PC-4.1-Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ADM8000KP_A Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30,gzip(gfe)";
const char S13597[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Ex_Note6.0 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13598[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-530M Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13599[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; RMD_600 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13600[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; th-th; jFone JS501 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13601[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MA715 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13602[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; ZTE N5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S13603[] = "Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; K-Touch U86 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13604[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; Z520m-parrot Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13605[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; ZP808 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13606[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; 820 Build/820-V1.0-2013.01.30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13607[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; Gpad Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13608[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-R530M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13609[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; H3-8106L Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13610[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; H100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13611[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; TAB1052 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13612[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A1020 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13613[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; P9070 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13614[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; Trevi_PHABLET_5_S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13615[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GIONEE_INFINITY+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13616[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; FP-020-VE Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13617[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; Q049 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13618[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HCAT-01ST Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13619[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SwipeHaloSpeed Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13620[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ro-ro;Nivo Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S13621[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Tokio_Smartphone_1.0 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13622[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAB10-212 Build/Xenta TAB10-212) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13623[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; GT10K-DC Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13624[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; L36h Build/10.1.1.A.1.253) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13625[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; GT-N7I00 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13626[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Walton Primo H1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13627[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; WT71 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13628[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MT-X5292/1.0 Android/4.1.1 Release/04.03.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13629[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; T-SC2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13630[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; NABIJR-NV5B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13631[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; MIDCD10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13632[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 9920 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13633[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Q8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13634[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; E-Boda Supreme X80 Dual Core Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13635[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; TB750 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13636[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; sunfire Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13637[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IFA10.1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13638[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; OV-TB-O9C Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13639[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MID9702 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13640[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; TY-Q8 Build/JRO03C)";
const char S13641[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI D2-2010 Build/HuaweiD2-2010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13642[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Ex'Note S  Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13643[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TAB700 4GB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13644[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Cat Tablet PHOENIX 8.1J0 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13645[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; 8506IPS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13646[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SV7612-RK2926 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13647[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Tablet Ner?n Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13648[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; RC0722C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13649[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; LearnPad Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13650[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; M75GC702G Build/PD700) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13651[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; TAB852 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13652[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; ZP908 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13653[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; xTAB-700dc Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13654[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Orin S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13655[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; xTAB-70dci Build/w28ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13656[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; M1003 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13657[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; klu8506 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13658[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CVD601 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13659[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Alcatel 7030L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S13660[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; DTP-07HDAH Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13661[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Kepler 2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13662[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; FPT F12 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13663[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-uk; MAGPAD_MP1041 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13664[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; cj-31s-jrrj Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13665[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; Infinity POP Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S13666[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; S9 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13667[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; baoxue Build/F10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13668[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; SKYTEX-SP723 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13669[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; TELETALK_X200Q Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13670[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; A10-QM Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13671[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; QUANTUM65220 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13672[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; i-Call 503 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13673[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; AQUILA 097-1016 BT + 3G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13674[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; TEACTAB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13675[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; T277 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13676[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Surfer 7.32 3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13677[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; i-mobile i-STYLE 7.2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13678[] = "Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1 Release/04.26.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13679[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LXK901 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13680[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 1018 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13681[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; F7100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13682[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Zeepad 9XN Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13683[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; G926 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13684[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; fnac 8 v2 Build/1.0.0_X-19:36) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13685[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; M-pro Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13686[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; CRT97 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13687[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB-R2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13688[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; Speedstar S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13689[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; CTP988 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13690[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC S720t Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13691[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Soaiy Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13692[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; MonsterPad Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13693[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; i926 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13694[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; A690 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13695[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; HK-i7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13696[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; i6577 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13697[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Z770M Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13698[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S7300B Build/S7300B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13699[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; Ultimix95 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13700[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; TCL S820 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S13701[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Aprix D97+ Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13702[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; StarPAD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13703[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; U30GT2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13704[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; Victorys_V8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13705[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ICOU10 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13706[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Bambook S1 Build/NS4.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13707[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; ARIA_Mini_wifi Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13708[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hr-hr; MeanIT MG940A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13709[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fa-ir; LD-459 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13710[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; Sigma-X10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13711[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SpeedStar S Pro Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13712[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; V-PAD10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13713[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; LETAB1006 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13714[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ar-eg; G-Tab G100 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13715[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; T750 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13716[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; P700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13717[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Primus 94HD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13718[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; CTP818 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13719[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AM818RC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13720[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ATP7717 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13721[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; moii E896 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13722[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; DTR-07HDAH Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13723[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; W700 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13724[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; TAB9701 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13725[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; D125 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13726[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; MW0712 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13727[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; NM890 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13728[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HUAWEI Y300C Build/HuaweiY300C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13729[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAC-80031 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13730[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; R830 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13731[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-nz; CORE DIGITAL CD-7066 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13732[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; FreeTAB 9704 IPS2 X4 Build/unknown) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13733[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; NB032 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13734[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Surfer 7.31 3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13735[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CrystalQuadCore Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13736[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; 3G7100i-PANELi Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13737[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; KN_Mobile_QT_A06 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13738[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MT6517-MT6517-H369X/1.0 Android/4.1.1 Release/04.08.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13739[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CW-V88-QUAD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13740[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; YL-Coolpad_5930/4.1.025.X.5930;540*960;CTC/2.0) CoolpadWebkit/534.30";
const char S13741[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PRBT761 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13742[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; I-Life wtab 702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13743[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; G-TiDE E6 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13744[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; T1400 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13745[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; PAD 9716DR Build/PAD 9716DR) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13746[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; TM-9748 3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13747[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; QUEEN TAB 100 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13748[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Cloudpad_800d Build/FSV03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13749[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; XZPAD270G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13750[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ar-eg; Q75S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13751[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SDT-737 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13752[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; EHT7HD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13753[] = "Mozilla/5.0 (Linux; Android 4.1.2; i-mobile IQ1-1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13754[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EGS004 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13755[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; sl-si; XPADK90 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13756[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fa-ir; TB1000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13757[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; MIDC470 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13758[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13759[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI MT1-T00 Build/HuaweiMT1-T00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13760[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.1 Release/04.26.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13761[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SKYTEX-SP705 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13762[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; CTA Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13763[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MT6517-MT6517-H369X/1.0 Android/4.1.1 Release/05.20.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13764[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TP8-1500DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13765[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A600 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13766[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; CRXT 1125 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13767[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.1 Release/05.31.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13768[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D7018 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13769[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; Tab810 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13770[] = "Tablet-PC-4.1-Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; INM8002KP Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13771[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Cherry Fusion Wind Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13772[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; Tab760 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13773[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; da-dk; TAB-E309(V3) Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13774[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; da-dk; TAB-E419(V2) Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13775[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; MTC 970 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S13776[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 11_v77_gq2000_a41_6628_v20 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13777[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; TA-FONE A39 HD Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13778[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; E58 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13779[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; PL801 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13780[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; N80HH Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13781[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; N70DC-T Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13782[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; NewPad Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13783[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 7010S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13784[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; 9706IPS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13785[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nb-no; Tablet 9748DCB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13786[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; S2 Build/JRO03H)";
const char S13787[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Mini Plus 4G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13788[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; T5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13789[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Cayon X3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13790[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; TAB 177 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13791[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; StarPAD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13792[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 9709 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13793[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Intki E96?? Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13794[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; F097 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13795[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; 8730 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S13796[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAB364 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13797[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HD_3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13798[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; NetTAB SPACE II Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13799[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CP101 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13800[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; L106 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13801[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; U23GT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13802[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 5930 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S13803[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; OUKI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13804[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; NEO-X5 Build/JRO03H)";
const char S13805[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; V350 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13806[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; u12 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13807[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; TAB R105BK Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13808[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; HKPSG Max Pro Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13809[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Motorola RaZR3Ds Build/9.8.2O-72_VZW-8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13810[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A86-2cpu Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13811[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MEEG_101 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13812[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; I8000S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13813[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; F23 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13814[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI Y300C Build/HuaweiY300C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S13815[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; Evopad R9730 Build/Evopad R9730) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13816[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; Tablet Mint 8x8i DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13817[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Touchlet X10.dual+ Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13818[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 070R9 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13819[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; NX501 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13820[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; U8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13821[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; MT-901 Build/MT-901) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13822[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Elite Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13823[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N86-2cpu Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13824[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; Kruger Matz 0793 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13825[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID8089D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13826[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; OMEGA-MID9711 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13827[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MODECOM FreeTAB 8001 IPS X23G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13828[] = "Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; Lenovo-A706/S100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S13829[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 7014 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13830[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolgen E69 NOTE2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13831[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; ZTE V956 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13832[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Superb Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13833[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PC738 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13834[] = "Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; 7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13835[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Phablet P1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13836[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; dual7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13837[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; AML-MX REF Build/JRO03C)";
const char S13838[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; AP-803 Build/AP-803) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13839[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; K-Touch U86 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13840[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; POLYTRON_W7450 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13841[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; M83w Build/M83w-eng 4.1.1 JRO03H test-keys_OTA:MID_2.1.1.1_X.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13842[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; ZTE U956 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13843[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; NTT 617 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13844[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; M101w Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13845[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Mi-A531 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13846[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; H100 Build/JRO03C)";
const char S13847[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; NetTAB_Space_3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13848[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; MW0931 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13849[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Acme TB04 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13850[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TZ100 plus Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13851[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HR-Q10R Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13852[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; V511M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13853[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; GDC PegasusII Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13854[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; CE15711-54 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13855[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; ZTE N818 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13856[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; Bouygues Telecom Bs 351 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13857[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Quickfone N1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13858[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-gb; C3_77_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13859[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; iB755C Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13860[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; MID8128 Build/GreenDemon_v1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13861[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; XC11-01A Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13862[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; C1041IPS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13863[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; V801 Core4 Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13864[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; CyrusApel Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13865[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; H9600+ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13866[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; PC738 Build/PC738) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13867[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; DTA-07FSBH Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13868[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; OPT-220 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13869[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; NBX-7012 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13870[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; S4701 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13871[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; TB-308 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13872[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; YOUPU U6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13873[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Tablet PC szenio 2500 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13874[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; LETAB907 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13875[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZTE-Z998 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13876[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; RT3066BT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13877[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; U803D Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13878[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; id-id; T753 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13879[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; S98 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13880[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; T9100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13881[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; T3188 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13882[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; HS-8DTB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13883[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Y88 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13884[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; androVM for VirtualBox ('Phone' version) Build/JRO03S) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13885[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ET7-20C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13886[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; TR-1010 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13887[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; H38 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13888[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; XTAB-781+ Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13889[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A9DualCore Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13890[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; S508 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13891[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; H7000+ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13892[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N80RK Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13893[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; BEN7116N Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13894[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AX9Z_Race Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13895[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fa-ir; NX7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13896[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HS-E956Q Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13897[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AMAZE-731 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13898[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; NoVa Pad 80 C168D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13899[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TY-Q88 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13900[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ph; MyPhone A919 3D Duo Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13901[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ANS-01/8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13902[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-pt; Boston 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13903[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; hu-hu; AKAIPHA_5800 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13904[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; KN_Mobile_QT-A08 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13905[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A13 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13906[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; S2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13907[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; EMPEROR Build/EMPEROR) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13908[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; uphone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13909[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI MT1-T00 Build/HuaweiMT1-T00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13910[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; EUTB-730 ARGOS NAV Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13911[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZTE-V72A Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Safari/534.30";
const char S13912[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; ETB8448 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13913[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SV7612-RK2926 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13914[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; E-Boda Supreme XL200IPS Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13915[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; M805 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13916[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; i-mobile i-STYLE 7A Build/JZO54K)";
const char S13917[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; C-501 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13918[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; SZENIO 2500 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13919[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ST9001 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13920[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PLT7045K Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13921[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; G402 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13922[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; J9500 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13923[] = "Mozilla/5.0 (Linux; U; Android 4.1.0; id-id; Venera Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13924[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID8080 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13925[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; momo20-w2 Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13926[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; M15GC702P Build/PD717) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13927[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TBDC1093 Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 GSA/2.0.0.X";
const char S13928[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IN700 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13929[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; S32 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13930[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; PENTAGRAM TAB 7.5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13931[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; G19 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13932[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Fontab SFT071 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13933[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; CAPHG10-01 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13934[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ET7183G-V13 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13935[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; WTAB 705 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13936[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; GPLUS_GN810 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13937[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; NVR-TAB8 S1 4GB Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13938[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID-7089D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13939[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; GT-N7I02 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13940[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ar-eg; Q75 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13941[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; M709 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13942[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; KINGO TAB 7 PLUS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13943[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; AS127 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13944[] = "AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; TBDC1093 Build/X)";
const char S13945[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; DSlide973 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13946[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hr-hr; TPC-97150 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13947[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A721 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13948[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; BS101 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13949[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; RMD-855 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13950[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; i908 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13951[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; Xenta TAB10-212 Build/Xenta TAB10-212) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13952[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; EverClassic Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13953[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A105 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13954[] = "Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1 Release/04.22.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13955[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; N10QM Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13956[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; es-es; QBA769 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13957[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; S4501M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13958[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; S5301M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13959[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; RoverPad 3W9.4 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13960[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; Mio Pad A1001 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13961[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; FONE 450 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13962[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; i-mobile i-style 8 Build/JZO54K)";
const char S13963[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; J920 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13964[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HTC_606w Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13965[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; VIVO X2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13966[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M71 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13967[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; TY-K-Touch_C966e Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13968[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HS-E912S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13969[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nitro 500D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13970[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; Radiance Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13971[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Sweet M1018 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13972[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MIDCD10PR Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13973[] = "Mozilla/5.0 (Linux; U; Android 4.1.4; en-us; Luna TAB07-101 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13974[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Y82 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13975[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SAMSUNG-GT-S7898_TD/1.0 Android/4.1.2 Release/11.15.2012 Browser/AppleWebKit534.30 Build/JRO03L) ApplelWebkit/534.30 (KHTML,like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13976[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; G7 Build/X06I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13977[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; PAD 9719QR Build/PAD 9719QR) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13978[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CRCB2094 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13979[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; BEE_9500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13980[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E455g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13981[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; A8839 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13982[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; IFQM712HCNF Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13983[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IT-M747-RK Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13984[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.1 Release/05.28.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S13985[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMP3970B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13986[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; M6 Build/M6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13987[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; M045 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13988[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Flytouch XR Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13989[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; CTA-E07-11Z Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13990[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; YL-Coolpad_5216S/4.1.022.X.5216S;480*800;CTC/2.0) CoolpadWebkit/534.30";
const char S13991[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GOCLM7000KD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13992[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; N79 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13993[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Tab701D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13994[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Troy_Duos_S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13995[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; SENSUELLE Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S13996[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; E1050HG Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13997[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TechniPad 8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13998[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; SL1066 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S13999[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; mini S4 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14000[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fa-ir; G4_NFC Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14001[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; TPC-840M Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14002[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; CTA-E07-10Z Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14003[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EX020D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14004[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14005[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PD200 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14006[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; Thunderbird Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14007[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Primux_Alpha_2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14008[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; F816 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14009[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Starmobile JUMP Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14010[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; HIPSTREET Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14011[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; XZPAD210G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14012[] = "Mozilla/5.0 (Linux; U; Android 4.1 ;zh-TW; A4 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14013[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xenta-TAB07-210 Build/Xenta-TAB07-210) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14014[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Fusion Flare v1.0 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14015[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; baoxue Build/F8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14016[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; TID-97062 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14017[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; VENERA 907 Build/X06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14018[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Primux_Omega_2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14019[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ENERGY i10 DUAL Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14020[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; 5100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14021[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; EverMagic Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14022[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; A701D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14023[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ST70216-1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14024[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; PocketBook SURFpad2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14025[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; P717A Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14026[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SBM205SH Build/S0002) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14027[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; SP401 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14028[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; Orion7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14029[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; busbi 7 dual v1.0 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14030[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; fnac 7 plus v2 Build/1.0.0_X-15:03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14031[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Xtab785 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14032[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; CTA-E07-13A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14033[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XM721RK Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14034[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; G526-L22 Build/HuaweiG526-L22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14035[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MacTab Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14036[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; R7X Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14037[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SBM205SH Build/S0004) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14038[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; I110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14039[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A77 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14040[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PTAB1050 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14041[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; PAD1001 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14042[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; BTL-703 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14043[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TAB9DUALC8GB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14044[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; i-mobile i-note 3 Build/JB_JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14045[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; CORE 10.1 DUAL 3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14046[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Symphony Xplorer W71 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14047[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; fnac 10 3G v2 Build/1.0.1_X-09:03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14048[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; Lanix Ilium_S115 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14049[] = "Mozilla/5.0 (Linux; Android 4.1.2; i-mobile IQ1-1A Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14050[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TB1000 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14051[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; G905 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14052[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; VA75 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14053[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E451g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14054[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; 3G9728 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14055[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IMPERIUS TAB 10 MT7011 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14056[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_D2-2010/D2-2010V100R001C92B133) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14057[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 10D8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14058[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NB4-3/5788 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14059[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Woxter Tablet PC Nimbus 70 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14060[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Tablet Biox Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14061[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AM1006 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14062[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MTV SLASH Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14063[] = "Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1 Release/05.27.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14064[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.2 Release/06.18.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14065[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XOLO Play Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14066[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; fnac 10 3G v2 Build/1.0.0_X-17:05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14067[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TBQC1063 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14068[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Mi-491 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14069[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; VX70 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14070[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; t520 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14071[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; CT0712 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14072[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EGM002 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14073[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A60 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14074[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TechniPad_10-3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14075[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; IS-T71 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14076[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; M4TEL SS990 Build/JRO03C) AppleWebKit/534.70 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.70";
const char S14077[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LR102 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14078[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; CnM-TP-7DC-16-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14079[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Trio Stealth_8 Build/JR003C.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14080[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; CnM-7DC-16-3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14081[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; F818 Build/F818) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14082[] = "Mozilla/5.0(Linux;U; Android 4.1.1;en-us;Digitac Ultratab;C755 Build/ICS) AppleWebKit/531.21.10(KTHML,like Gecko) Version/4.1.1 Mobile/7B367 Safari/531.21.10";
const char S14083[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; AS-AT-SEVEN-X2-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14084[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MID1014 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14085[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; INGO_TAB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14086[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; N10one Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14087[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A86 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14088[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; D712 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14089[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; OBLIO Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14090[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; M6 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14091[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; MID-972R-3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14092[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ctab1012c Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14093[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; OS818 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14094[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PRBT742 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14095[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M3000 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14096[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; U23GT-S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14097[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; I5P Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14098[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Woxter Tablet PC 85IPS Dual Build/JRO03H)";
const char S14099[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TA2508-7BK Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14100[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; INCA-Astro Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14101[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; GF-1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14102[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; i9900 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14103[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; Ezcool Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14104[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; N1010 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14105[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; EUTB-708 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14106[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; KYOTO SM16 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14107[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HMIA5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14108[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; 1006-IPS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14109[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; U21GT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14110[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14111[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; T9510E Build/HuaweiT9510E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14112[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; KATBL10A08E Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14113[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; TLINK475 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14114[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HUAWEI Y300C Build/HuaweiY300C)";
const char S14115[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; Polypad 528HD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14116[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; TB04 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14117[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; M7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14118[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; sl-si; RC0817C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14119[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; Padphone 71 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14120[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; H700S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14121[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; ilium Pad E10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14122[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Disgo_Tablet_8400G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14123[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Teraware K707G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14124[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; RC9712C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14125[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; X718 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14126[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 97RQ Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14127[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; SP0978 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14128[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; MiniGpad Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14129[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; X7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14130[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ET754-B13 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14131[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SonyM35h Build/12.0.A.1.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14132[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; E-boda essential smile Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14133[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; Xenta TAB09-211 Build/Xenta_TAB09-211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14134[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A430 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14135[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A530HD Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14136[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; RM-19 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14137[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; MT-101 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14138[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; AN7IG3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14139[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; U3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14140[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; 8BPE0H Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14141[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; ETB7447 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14142[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; UM-770 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14143[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; M1032 Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14144[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; HW-03E Build/HuaweiD2-6114) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14145[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MMDI9900 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14146[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; Elektra XL Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14147[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; DENARI 1029 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14148[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; Explosion Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14149[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; KLYAN2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14150[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; CTAB1013BT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14151[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; YL-Coolpad_7268/4.1.005.X.7268; 540*960; CUCC/2.0) CoolpadWebkit/534.30";
const char S14152[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ET-102 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14153[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; Jet core doble-7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14154[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.1 Release/05.18.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14155[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Maxwell 2 Plus Build/1.0.0_X-15:51) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14156[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TABLET 10-2X Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14157[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; KID PLAY Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14158[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; T6188 Build/MocorDroid4.1.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14159[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; CT600 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14160[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; 720M 3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14161[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HTC 606w Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14162[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; R452 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14163[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; X10.Quad Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14164[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HS-EG970 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14165[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; fnac 7 v2 Build/1.0.0_X-20:13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14166[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; U705W Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14167[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; COROM8000KP Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14168[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SKYTEX-SP910 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14169[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; N451 F1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14170[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 7038 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14171[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; s4003 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14172[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; VOLT Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14173[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; M709 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14174[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; Tab3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14175[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; Andy A4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14176[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; MD-1008 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14177[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Canvas 2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14178[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; IMO_Z9 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14179[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_G610-C00/G610-C00V100R001C92B120; 540*960; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S14180[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; s7503 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14181[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; MT7A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14182[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; Joyz Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14183[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.1 Release/06.17.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14184[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Polo Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14185[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; el-gr; iQTab 3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14186[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.2 Release/06.26.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14187[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ADM8000KP Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14188[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; UNUSUAL U7X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14189[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Infinix X530 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14190[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; R7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14191[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; V7011 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14192[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; fnac 8 3G v2 Build/1.0.0_X-14:45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14193[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NX010HI8G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14194[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Digital2 Platinum Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14195[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; CROSS AT1G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14196[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; NOTE SMART Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14197[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Trio Stealth_9 Build/JR003C.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14198[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; id-; mbk77_twn_cu_jb Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14199[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; MTC 970H Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30 SVN/040HPG4";
const char S14200[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_C8815/C8815V100R001C92B122; 540*960; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S14201[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Camelio Family tablet Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14202[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Voxson Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14203[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Andi4a Radium Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14204[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; A7S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14205[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SOLT JB Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14206[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Atrix HD Build/HoloBlur-X v0.9 PurePerformanceT Edition) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14207[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; DTR10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14208[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Z99 Dual Core Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14209[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PRO7D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14210[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; HN-M740 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14211[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; AT-AS43D4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14212[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Aoson M11 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14213[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; X5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14214[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MT887 Build/6.7.2_GC-384) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14215[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 097R1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14216[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; P76t??(M7H6) Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14217[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; ovo 2.0 Build/FR-MX68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14218[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; Koobe S7 Easy tablet Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14219[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; id-id; Realpad P700 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14220[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; POV_TAB-PROTAB30IPS10-3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14221[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; AT97C RK66WA Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14222[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HOSIN V60 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14223[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 820 Build/820-V1.0-2013.01.18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14224[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; E-Boda Essential A160 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14225[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; JXD-S6600-M6 Build/S6600-M6-01-XCN) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14226[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; QW TB-1517 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14227[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; KYOTO SM23 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14228[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 5216S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14229[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; GALAXY S2 HD LTE E120K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14230[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TransBook TRA01 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14231[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; ZEST D740 Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14232[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; NeoTab H105 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14233[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; TC55 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14234[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ET-250 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14235[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; Mini one Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14236[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; vivo S7w Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14237[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MT-719 SII Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14238[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; MID-9108 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14239[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; N810 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14240[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Omega Q Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14241[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Z2_WVGA_QXH Build/Android4.1.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14242[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; N10-QM Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14243[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fi-fi; RD805 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14244[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Full AOSP on Leopard SOC Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14245[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; H58 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14246[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; jpad 723a Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14247[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xenta TAB08-201-3G Build/Xenta TAB08-201-3G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14248[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-; Connspeed IP71 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14249[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; el-gr; iQTab Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14250[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A5ENZE Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14251[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; QMobile_A950 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14252[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; aTAB7DCI Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14253[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Platium Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14254[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TU-6818E Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14255[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.1 Release/06.25.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14256[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; CloudPad 700TV Build/FSV01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14257[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A29 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14258[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; ilium Pad E8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14259[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; F815 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14260[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ph; Livall_TELPAD_D7_tablet Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14261[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; 3G7271 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14262[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MID1047 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14263[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LG-E410B/V10c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14264[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; V0714 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14265[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XO Learning tablet Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14266[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MPQC974 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14267[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; YL-Coolpad_5950 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14268[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=m7;devm=HTC One;man=HTC;brw=Android;shwd=1080x1920;shwa=1080x1845;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S14269[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; E9000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14270[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D9018 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14271[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; @TAB-EIGHT-X2-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14272[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; CL500 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14273[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; PB-S8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14274[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PIRANHA Pro II Tab 9.0\" Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14275[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; TAB7B Build/WEXLER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14276[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 740T Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14277[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HR-738R Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14278[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; BOHP-P5A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14279[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GIPO-T100F Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14280[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; NeoTab B90 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14281[] = "AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; klu1506 Build/JRO03H)";
const char S14282[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; HTC_PN074 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14283[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID-7000B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14284[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HTC_801e Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14285[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; da-dk; DSRV-9708C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14286[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; S9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14287[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MID-970DCHI Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14288[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; samsung Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14289[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; ND-973 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14290[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SC-97JB Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14291[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; CTVISION Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14292[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HELLOTAB2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14293[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TPC1051 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14294[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; M35h Build/12.0.A.1.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 BaiduBoxApp/3.6_Xc";
const char S14295[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xenta TAB08-200 Build/Xenta TAB08-200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14296[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; SL966 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14297[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; GiONEE_GN810 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14298[] = "Mozilla/5.0 (Linux; U; Android 4.1.X; de-de; Xelio10Pro Build/ODYS_Xelio) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14299[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Commodore  IS-7107C Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14300[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-S7898 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14301[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI C8815 Build/HuaweiC8815) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14302[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SBM205SH Build/S0004)";
const char S14303[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; A6S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14304[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PLT8223G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14305[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; G9 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14306[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; Trevi PHABLET 6 S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14307[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; NaviPad TM-7045 3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14308[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; smart_a17 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S14309[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.2 Release/06.17.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14310[] = "Mozilla/5.0 (Linux; Android 4.1.2; ICON Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14311[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; JAY-TECH Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14312[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; P886A Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14313[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; TIGER 3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14314[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LT7035-F Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14315[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; Quadra 7 UltraSlim Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14316[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; FUSION2IN1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14317[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; TAB10IS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14318[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 101B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14319[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; AF9922 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14320[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; V100MD Build/V100MD.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14321[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; KB706 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14322[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.2 Release/07.02.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14323[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; LIVE Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14324[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-nz; innos i3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14325[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; CTC07SO Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14326[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GSSMID-7002S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14327[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; FT970 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14328[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; CW-V19 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14329[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; C7/K7_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14330[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC_603e Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14331[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 7268 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14332[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; Impact 2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14333[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; 8730 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14334[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; PAD-2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14335[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Visture V5 HD Build/Visture V5 HD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14336[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; U1203 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14337[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; O+ 9.76 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14338[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; Nettab Thor X Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14339[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC 603e Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14340[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; TOKS7G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14341[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; K-Touch U90 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14342[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I759 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S14343[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; N90FHDRK Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14344[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MT6517-H369X Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14345[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; N800 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14346[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Titan6575-SA Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14347[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; B95+ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14348[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; RC0718C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14349[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; ID-K97 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14350[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Xtab1081HD+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14351[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; M716D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14352[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; M-PAD Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14353[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; TY-K-Touch_E88 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14354[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; ZTE Kis Pro Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14355[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Polaroid 40490 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14356[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GM80 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14357[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; iTAB704j Build/PD715T) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14358[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; I9082 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14359[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; ES802 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14360[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Device_Navigator Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14361[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; TouchTab 13.3 DC Build/TouchTab 13.3 DC) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14362[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; N9510 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14363[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 101P11C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14364[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; APPTB102S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14365[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; T06 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14366[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; NTT 528 Build/NTTX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14367[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MT6517-Rave-S89/1.0 Android/4.1.1 Release/05.06.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14368[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; c410 Build/FSV04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14369[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; LEOTEC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14370[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; TA55 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14371[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PROG400 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14372[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HTC 608t Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14373[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; TA-734 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14374[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZTE_N9511 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14375[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; SN97T41W Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14376[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; TGA22 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14377[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; M83g Build/M83g-eng 4.1.1 JRO03H test-keys_OTA:MID_2.1.1.1_X.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14378[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PRODG16 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14379[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ro-ro; ALLVIEW_P5_Qmax Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14380[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Engage 8 Build/Engage 8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14381[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; YL-Coolpad_5879/4.1.002.X.5879; 480*854; CTC/2.0) CoolpadWebkit/534.30";
const char S14382[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZTE-Z995 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14383[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Huawei Y301A2 Build/HuaweiY301A2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14384[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; TwinTAB-T7283GD1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14385[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; DL HD EV 01 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14386[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Zigo Eon6i Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14387[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; u18 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14388[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MIDC970 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14389[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A820 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14390[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; PMP7380D3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14391[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; AIRIS_TM530 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14392[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; Micromax A86 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14393[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Table PC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14394[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; 5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14395[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-P712 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14396[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; P1001A Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14397[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; P5100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14398[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; el-gr; Turbox-X G410 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14399[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; el-gr; TURBOX X-2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14400[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; LG-E415f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14401[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MPQC804HD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14402[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; YL-Coolpad_5891 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14403[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; Mi731 Build/Mi731) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14404[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI C8815 Build/HuaweiC8815) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S14405[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HALO_VALUE+ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14406[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; A53 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14407[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; E812 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14408[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAB10-400Build/NobleTAB10-400) AppleWebKit/534.30 (KHTML, likeGecko) Version/4.0 Safari/534.30";
const char S14409[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=SGH-I717;devm=SAMSUNG-SGH-I717;man=samsung;brw=Android;shwd=800x1280;shwa=800x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S14410[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xtreme-V30 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14411[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Nimbus 100Q Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14412[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NuclearSX-SP5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14413[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; i-mobile IQ5.1 Pro Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14414[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GEM8902 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14415[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; E821 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14416[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; Pixus Play Two Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14417[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; EBEST L5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14418[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; V8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14419[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; T500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14420[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolgen E70 NOTES Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14421[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; T29 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14422[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; Ultimix77 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14423[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; S71 Build/JRO03H)";
const char S14424[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; ODEON TVTab-701 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14425[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; SUPREME IPS Dual Core X200 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14426[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LINSAY COSMOS F-7HD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14427[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SC-1007JB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14428[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MyPhone A888 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14429[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; RTB7.4DFUN Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14430[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; INP97XP Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14431[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; MTO_MK318 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14432[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ARCHOS 97G10 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14433[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; i1017 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14434[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; XO Learning tablet Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14435[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; SZENIO 2000 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14436[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; ND-R726B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14437[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; EVERPAD SC-710 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14438[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; PENTAGRAM TAB Quadra 9.7 Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14439[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; SELECT7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14440[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.1 Release/05.03.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14441[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SNM-M35c Build/12.0.B.2.42) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14442[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; X920 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14443[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; TY-K-Touch_E616 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14444[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; I5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14445[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; HTC-one Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14446[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; ADT2.6 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14447[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; F53s Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14448[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; H38 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14449[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; N70 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14450[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; M720 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14451[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; K-Touch Q4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14452[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; C22 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14453[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; Space Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14454[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; M-MP980S4 Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14455[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.1 Release/05.15.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14456[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; LC1016C Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14457[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; TB-7 v2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14458[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; M5-3G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14459[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.1 Release/05.07.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14460[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EX_NOTE_? Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14461[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; MID800 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14462[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; HRA_M710 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14463[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; PAN7 HS Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14464[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CLP 282 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14465[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hr-hr; 01_v77_yp1_y7800ntv Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14466[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; A7S Build/JZO54K)";
const char S14467[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; PrimuxZonda3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14468[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; T8 3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14469[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TBDG773 Build/Zeki.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14470[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; SPC P5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14471[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; a5mid Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14472[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; RX4DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14473[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AX8 NOTE II Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14474[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xolo QC800 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14475[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GPad 7.0 Excellent 4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14476[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; NBX-8012 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14477[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LGMS500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14478[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ETAB123MKII Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14479[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; DC-I9508 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14480[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HX-M02 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14481[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; REVOMAX_2012 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14482[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Maxwell 2 Plus Build/1.0.1_X-09:33) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14483[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; KDI-I406 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14484[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; KB708 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14485[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; KBOOKPLUS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14486[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; Combo Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14487[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 5950 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14488[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; fnac 10 3G v2 Build/1.0.2_X-09:39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14489[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; SM-T211-ORANGE/T211XXAMGC Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14490[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; id-id; CROSS-A20G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14491[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GV701S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14492[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; BioniQ 800 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14493[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; U680C Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14494[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; EHT9 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14495[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PLT9045K Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S14496[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; NABIJR-NV5B Build/JZO54L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S14497[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S14498[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11";
const char S14499[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11";
const char S14500[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11";
const char S14501[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11";
const char S14502[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11";
const char S14503[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11";
const char S14504[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11";
const char S14505[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11";
const char S14506[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11,gzip(gfe)";
const char S14507[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11,gzip(gfe) (via docs.google.com/viewer)";
const char S14508[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1060) AppleWebKit/537.11 (KHTML like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S14509[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/23.0.648.205 Safari/534.16";
const char S14510[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/23.0.648.205 Safari/534.16";
const char S14511[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/23.0.648.205 Safari/534.16";
const char S14512[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A523 Safari/8536.25";
const char S14513[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A8426 Safari/8536.25";
const char S14514[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A403 Safari/8536.25";
const char S14515[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A550 Safari/8536.25";
const char S14516[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A8500 Safari/8536.25";
const char S14517[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A406 Safari/8536.25";
const char S14518[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A407 Safari/8536.25";
const char S14519[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S14520[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S14521[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S14522[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S14523[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S14524[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S14525[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)";
const char S14526[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe) (via docs.google.com/viewer)";
const char S14527[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)";
const char S14528[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4; en-US) AppleWebKit/537.13 (KHTML, like Gecko) Chrome/24.0.1290.1 Safari/537.13";
const char S14529[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SAMSUNG GT-I9001/I9001BUKG1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14530[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14531[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; GT-I9001-ORANGE/I9001BVKP7 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14532[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14533[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; GT-I9001-ORANGE/I9001BVKP8 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14534[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; GT-I9001-ORANGE/I9001BVKPA Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14535[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; nl-nl; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14536[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; SAMSUNG GT-I9001/I9001BUKE5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14537[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-I9001 Build/GINGERBREAD)";
const char S14538[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-I9001 Build/GINGERBREAD)";
const char S14539[] = "Mozilla/5.0 (Linux; U; de-de; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S14540[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-I9001/I9001BUKP7 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14541[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; cs-cz; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30";
const char S14542[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-I9001 Build/GINGERBREAD)";
const char S14543[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14544[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; SAMSUNG GT-I9001/I9001BUKP2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14545[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9001 Build/GINGERBREAD)";
const char S14546[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9001 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S14547[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9001 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S14548[] = "Mozilla/5.0 (Linux; U; en-gb; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S14549[] = "Mozilla/5.0 (Linux; U; es-es; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S14550[] = "Mozilla/5.0 (Linux; U; en-ie; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S14551[] = "Mozilla/5.0 (Linux; U; it-it; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S14552[] = "Mozilla/5.0 (Linux; U; ro-ro; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S14553[] = "Mozilla/5.0 (Linux; U; en-us; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S14554[] = "Mozilla/5.0 (Linux; U; de-at; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S14555[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-I9001-ORANGE/I9001BVKPC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14556[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-I9001/I9001BUKP9 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14557[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9001 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S14558[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT-I9001 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S14559[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; GT-I9001 Build/NitroX GingerBread) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14560[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-I9001-ORANGE/I9001BVKPG Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14561[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-I9001 Build/GB. BlazR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14562[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9001 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S14563[] = "Mozilla/5.0 (Linux; U; de-ch; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S14564[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14565[] = "Mozilla/5.0 (Linux; U; pt-pt; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S14566[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-I9001/I9001BOKQ4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14567[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14568[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14569[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14570[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-I9001/I9001BUKPC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14571[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fa-fa; GT-I9001 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S14572[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-I9001-ORANGE/I9001BVKPK Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14573[] = "Mozilla/5.0 (Linux; U; fr-ch; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S14574[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-I9001-ORANGE/I9001BVKPN Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14575[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.2.0.0 Safari/537.22";
const char S14576[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.2.0.0 Safari/537.22";
const char S14577[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S14578[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S14579[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)";
const char S14580[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.1.0.0 Safari/537.22";
const char S14581[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.1.0.0 Safari/537.22";
const char S14582[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)";
const char S14583[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe) (via docs.google.com/viewer)";
const char S14584[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.22 (KHTML like Gecko) Chrome/25.0.1364.97 Safari/537.22";
const char S14585[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22 BitdefenderSafepay/17.14.0.0 (OBK; 25.0.1364.172; r1194.2013)";
const char S14586[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonST25i Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14587[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; ST25i Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14588[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-ca; SonyEricssonST25a Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14589[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; SonyEricssonST25i-o Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14590[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-at; SonyEricssonST25iv Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14591[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonST25i-o Build/6.0.B.1.576) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14592[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-ie; SonyEricssonST25i Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14593[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; SonyEricssonST25a Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14594[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14595[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14596[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; ST25a Build/6.0.B.1.564)";
const char S14597[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ST25i Build/6.0.B.1.564) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S14598[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-mx; ST25a Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14599[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ST25i Build/6.0.B.3.184) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S14600[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; ST25i Build/6.0.B.3.184)";
const char S14601[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; ST25i Build/6.0.B.1.576) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14602[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonST25i-o Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14603[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonST25i Build/6.0.B.1.576) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14604[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-nz; SonyEricssonST25iv Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14605[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; SonyEricssonST25i Build/6.0.B.3.189) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14606[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-nz; SonyEricssonST25iv Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14607[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SonyEricssonST25i Build/6.0.B.3.188) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14608[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; ST25a Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14609[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-mx; SonyEricssonST25a Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14610[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SonyEricssonST25a Build/6.0.B.3.188) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14611[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ST25i Build/6.0.B.3.188) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14612[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; ST25i Build/6.0.B.3.189) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14613[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-mx; ST25a Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14614[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ST25a Build/6.0.B.3.188) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14615[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-ie; ST25i Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14616[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; iw-; SonyEricssonST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14617[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonST25i Build/KA02 Xperia SSpeed) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14618[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; iw-; SonyEricssonST25a Build/6.0.B.3.188) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14619[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonST25i Build/GB-ROCKS V3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14620[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.16";
const char S14621[] = "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.16";
const char S14622[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.16";
const char S14623[] = "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.16,gzip(gfe)";
const char S14624[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.16";
const char S14625[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.16";
const char S14626[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.16";
const char S14627[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.16";
const char S14628[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.16";
const char S14629[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6327)) Presto/2.12.388 Version/12.16";
const char S14630[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.16,gzip(gfe)";
const char S14631[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6561)) Presto/2.12.388 Version/12.16";
const char S14632[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6648)) Presto/2.12.388 Version/12.16";
const char S14633[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64) Opera 12.16";
const char S14634[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.16";
const char S14635[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.16";
const char S14636[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.16";
const char S14637[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.16";
const char S14638[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.16";
const char S14639[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6657)) Presto/2.12.388 Version/12.16";
const char S14640[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.16";
const char S14641[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.16";
const char S14642[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.16";
const char S14643[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6628)) Presto/2.12.388 Version/12.16";
const char S14644[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.16";
const char S14645[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.16";
const char S14646[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.16";
const char S14647[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.16";
const char S14648[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.16";
const char S14649[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6322)) Presto/2.12.388 Version/12.16";
const char S14650[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.16";
const char S14651[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.16";
const char S14652[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.16";
const char S14653[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.16";
const char S14654[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.16";
const char S14655[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.16";
const char S14656[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6323)) Presto/2.12.388 Version/12.16";
const char S14657[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.16";
const char S14658[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.16";
const char S14659[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.16";
const char S14660[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.16";
const char S14661[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6325)) Presto/2.12.388 Version/12.16";
const char S14662[] = "Opera/9.80 (Windows NT 6.1; MRA 8.1 (build 6283)) Presto/2.12.388 Version/12.16";
const char S14663[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.16";
const char S14664[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6638)) Presto/2.12.388 Version/12.16";
const char S14665[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.16";
const char S14666[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.16";
const char S14667[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.16";
const char S14668[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.16";
const char S14669[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.16";
const char S14670[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.16";
const char S14671[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.16";
const char S14672[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.16";
const char S14673[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6336)) Presto/2.12.388 Version/12.16";
const char S14674[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6665)) Presto/2.12.388 Version/12.16";
const char S14675[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.16";
const char S14676[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.16";
const char S14677[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.16";
const char S14678[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.16";
const char S14679[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.16";
const char S14680[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.16";
const char S14681[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6573)) Presto/2.12.388 Version/12.16";
const char S14682[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.16";
const char S14683[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6261)) Presto/2.12.388 Version/12.16";
const char S14684[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.16";
const char S14685[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6666)) Presto/2.12.388 Version/12.16";
const char S14686[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.16";
const char S14687[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.16";
const char S14688[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5997)) Presto/2.12.388 Version/12.16";
const char S14689[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6277)) Presto/2.12.388 Version/12.16";
const char S14690[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.16";
const char S14691[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.16";
const char S14692[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.16";
const char S14693[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6645)) Presto/2.12.388 Version/12.16";
const char S14694[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6080)) Presto/2.12.388 Version/12.16";
const char S14695[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.16";
const char S14696[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.16";
const char S14697[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.16";
const char S14698[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.16";
const char S14699[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.16";
const char S14700[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.16";
const char S14701[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.16";
const char S14702[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6668)) Presto/2.12.388 Version/12.16";
const char S14703[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.16";
const char S14704[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.16";
const char S14705[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.16";
const char S14706[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.16";
const char S14707[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.16";
const char S14708[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6337)) Presto/2.12.388 Version/12.16";
const char S14709[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.16";
const char S14710[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6667)) Presto/2.12.388 Version/12.16";
const char S14711[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6341)) Presto/2.12.388 Version/12.16";
const char S14712[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6669)) Presto/2.12.388 Version/12.16";
const char S14713[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6673)) Presto/2.12.388 Version/12.16";
const char S14714[] = "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.16,gzip(gfe),gzip(gfe)";
const char S14715[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.16";
const char S14716[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6014)) Presto/2.12.388 Version/12.16";
const char S14717[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.16,gzip(gfe) (via docs.google.com/viewer)";
const char S14718[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.16";
const char S14719[] = "Opera/9.80 (Windows NT 6.1; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.16";
const char S14720[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.16";
const char S14721[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03658)) Presto/2.12.388 Version/12.16";
const char S14722[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5217)) Presto/2.12.388 Version/12.16";
const char S14723[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.16";
const char S14724[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6661)) Presto/2.12.388 Version/12.16";
const char S14725[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4933)) Presto/2.12.388 Version/12.16";
const char S14726[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.16";
const char S14727[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5996)) Presto/2.12.388 Version/12.16";
const char S14728[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5300)) Presto/2.12.388 Version/12.16";
const char S14729[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.16";
const char S14730[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5660)) Presto/2.12.388 Version/12.16";
const char S14731[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4661)) Presto/2.12.388 Version/12.16";
const char S14732[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6620)) Presto/2.12.388 Version/12.16";
const char S14733[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03789)) Presto/2.12.388 Version/12.16";
const char S14734[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.16";
const char S14735[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.16";
const char S14736[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6324)) Presto/2.12.388 Version/12.16";
const char S14737[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5299)) Presto/2.12.388 Version/12.16";
const char S14738[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03639)) Presto/2.12.388 Version/12.16";
const char S14739[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.16 Yandex.Translate";
const char S14740[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.16";
const char S14741[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.6 (build 03402)) Presto/2.12.388 Version/12.16";
const char S14742[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5278)) Presto/2.12.388 Version/12.16";
const char S14743[] = "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.16,gzip(gfe) (via docs.google.com/viewer)";
const char S14744[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.16";
const char S14745[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5230)) Presto/2.12.388 Version/12.16";
const char S14746[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.16";
const char S14747[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6035)) Presto/2.12.388 Version/12.16";
const char S14748[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6044)) Presto/2.12.388 Version/12.16";
const char S14749[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.16";
const char S14750[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.16,gzip(gfe),gzip(gfe)";
const char S14751[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6675)) Presto/2.12.388 Version/12.16";
const char S14752[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6676)) Presto/2.12.388 Version/12.16";
const char S14753[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6681)) Presto/2.12.388 Version/12.16";
const char S14754[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.16";
const char S14755[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6346)) Presto/2.12.388 Version/12.16";
const char S14756[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6682)) Presto/2.12.388 Version/12.16";
const char S14757[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6347)) Presto/2.12.388 Version/12.16";
const char S14758[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6320)) Presto/2.12.388 Version/12.16";
const char S14759[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6046)) Presto/2.12.388 Version/12.16";
const char S14760[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4110)) Presto/2.12.388 Version/12.16";
const char S14761[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5336)) Presto/2.12.388 Version/12.16";
const char S14762[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6345)) Presto/2.12.388 Version/12.16";
const char S14763[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5988)) Presto/2.12.388 Version/12.16";
const char S14764[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5288)) Presto/2.12.388 Version/12.16";
const char S14765[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6032)) Presto/2.12.388 Version/12.16";
const char S14766[] = "Opera/9.80 (Windows NT 6.1; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.16";
const char S14767[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5734)) Presto/2.12.388 Version/12.16";
const char S14768[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.2 (build 7166)) Presto/2.12.388 Version/12.16";
const char S14769[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5650)) Presto/2.12.388 Version/12.16";
const char S14770[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.2 (build 7178)) Presto/2.12.388 Version/12.16";
const char S14771[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.2 (build 7188)) Presto/2.12.388 Version/12.16";
const char S14772[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03755)) Presto/2.12.388 Version/12.16";
const char S14773[] = "Opera/9.80 (Windows NT 6.1; MRA 8.1 (build 6340)) Presto/2.12.388 Version/12.16";
const char S14774[] = "Opera/9.80 (Windows NT 6.1; MRA 6.2 (build 7194)) Presto/2.12.388 Version/12.16";
const char S14775[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4832)) Presto/2.12.388 Version/12.16";
const char S14776[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6338)) Presto/2.12.388 Version/12.16";
const char S14777[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.289 Version/12.02";
const char S14778[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.10.289 Version/12.02";
const char S14779[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.289 Version/12.02";
const char S14780[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.289 Version/12.02";
const char S14781[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.289 Version/12.02";
const char S14782[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.289 Version/12.02";
const char S14783[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5328); ru) Presto/2.10.289 Version/12.02";
const char S14784[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.10.289 Version/12.02";
const char S14785[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.10.289 Version/12.02";
const char S14786[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.10.289 Version/12.02";
const char S14787[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.289 Version/12.02";
const char S14788[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.289 Version/12.02,gzip(gfe) (via translate.google.com)";
const char S14789[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.289 Version/12.02";
const char S14790[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.289 Version/12.02";
const char S14791[] = "Opera/9.80 (Windows NT 5.2; WOW64; U; en) Presto/2.10.289 Version/12.02";
const char S14792[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.289 Version/12.02";
const char S14793[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.289 Version/12.02";
const char S14794[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.289 Version/12.02";
const char S14795[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.289 Version/12.02";
const char S14796[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.289 Version/12.02";
const char S14797[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.289 Version/12.02";
const char S14798[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5966); ru) Presto/2.10.289 Version/12.02";
const char S14799[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.10.289 Version/12.02";
const char S14800[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.289 Version/12.02";
const char S14801[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5218); ru) Presto/2.10.289 Version/12.02";
const char S14802[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.289 Version/12.02";
const char S14803[] = "Mozilla/5.0 (Windows NT 5.1; en; rv:2.0) Gecko/X Firefox/4.0 Opera 12.02";
const char S14804[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.289 Version/12.02";
const char S14805[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.10.289 Version/12.02";
const char S14806[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.289 Version/12.02";
const char S14807[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.289 Version/12.02";
const char S14808[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5667); ru) Presto/2.10.289 Version/12.02";
const char S14809[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.289 Version/12.02";
const char S14810[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.289 Version/12.02";
const char S14811[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.289 Version/12.02";
const char S14812[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.289 Version/12.02";
const char S14813[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5676); ru) Presto/2.10.289 Version/12.02";
const char S14814[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.289 Version/12.02";
const char S14815[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.289 Version/12.02";
const char S14816[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; en) Opera 12.02";
const char S14817[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.289 Version/12.02";
const char S14818[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.289 Version/12.02";
const char S14819[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.289 Version/12.02";
const char S14820[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.289 Version/12.02";
const char S14821[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5230); ru) Presto/2.10.289 Version/12.02";
const char S14822[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.289 Version/12.02";
const char S14823[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.289 Version/12.02";
const char S14824[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.289 Version/12.02";
const char S14825[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.289 Version/12.02";
const char S14826[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4930); ru) Presto/2.10.289 Version/12.02";
const char S14827[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.10.289 Version/12.02";
const char S14828[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.289 Version/12.02";
const char S14829[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5217); ru) Presto/2.10.289 Version/12.02";
const char S14830[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5278); ru) Presto/2.10.289 Version/12.02";
const char S14831[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5660); ru) Presto/2.10.289 Version/12.02";
const char S14832[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5288); ru) Presto/2.10.289 Version/12.02";
const char S14833[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.289 Version/12.02";
const char S14834[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.289 Version/12.02";
const char S14835[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.10.289 Version/12.02";
const char S14836[] = "Opera/9.80 (Windows NT 5.1; U; compatible; MSIE 7.0; Windows NT 6.0; bg) Presto/2.10.289 Version/12.02";
const char S14837[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.289 Version/12.02";
const char S14838[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02746); ru) Presto/2.10.289 Version/12.02";
const char S14839[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.10.289 Version/12.02";
const char S14840[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5650); ru) Presto/2.10.289 Version/12.02";
const char S14841[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5299); ru) Presto/2.10.289 Version/12.02";
const char S14842[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5325); ru) Presto/2.10.289 Version/12.02";
const char S14843[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4661); ru) Presto/2.10.289 Version/12.02";
const char S14844[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5300); ru) Presto/2.10.289 Version/12.02";
const char S14845[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.10.289 Version/12.02";
const char S14846[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02552); ru) Presto/2.10.289 Version/12.02";
const char S14847[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03672); ru) Presto/2.10.289 Version/12.02";
const char S14848[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5336); ru) Presto/2.10.289 Version/12.02";
const char S14849[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.10.289 Version/12.02";
const char S14850[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.289 Version/12.02";
const char S14851[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.289 Version/12.02,gzip(gfe)";
const char S14852[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5745); ru) Presto/2.10.289 Version/12.02";
const char S14853[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02560); ru) Presto/2.10.289 Version/12.02";
const char S14854[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03638); ru) Presto/2.10.289 Version/12.02";
const char S14855[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.289 Version/12.02";
const char S14856[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.289 Version/12.02";
const char S14857[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02564); ru) Presto/2.10.289 Version/12.02";
const char S14858[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5734); ru) Presto/2.10.289 Version/12.02";
const char S14859[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5784); ru) Presto/2.10.289 Version/12.02";
const char S14860[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5997); ru) Presto/2.10.289 Version/12.02";
const char S14861[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5993); ru) Presto/2.10.289 Version/12.02";
const char S14862[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02652); ru) Presto/2.10.289 Version/12.02";
const char S14863[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.289 Version/12.02";
const char S14864[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5161); ru) Presto/2.10.289 Version/12.02";
const char S14865[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5861); ru) Presto/2.10.289 Version/12.02";
const char S14866[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5880); ru) Presto/2.10.289 Version/12.02";
const char S14867[] = "Opera/12.80 (Windows NT 5.1; U; en) Presto/2.10.289 Version/12.02";
const char S14868[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6011); ru) Presto/2.10.289 Version/12.02";
const char S14869[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.289 Version/12.02";
const char S14870[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03399); MRSPUTNIK OW 2, 3, 0, 104; ru) Presto/2.10.289 Version/12.02";
const char S14871[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5309); ru) Presto/2.10.289 Version/12.02";
const char S14872[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03649); ru) Presto/2.10.289 Version/12.02";
const char S14873[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5981); ru) Presto/2.10.289 Version/12.02";
const char S14874[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5977); ru) Presto/2.10.289 Version/12.02";
const char S14875[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6037); ru) Presto/2.10.289 Version/12.02";
const char S14876[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5989); ru) Presto/2.10.289 Version/12.02";
const char S14877[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6048); ru) Presto/2.10.289 Version/12.02";
const char S14878[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6054); ru) Presto/2.10.289 Version/12.02";
const char S14879[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.289 Version/12.02";
const char S14880[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.289 Version/12.02";
const char S14881[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5999); ru) Presto/2.10.289 Version/12.02";
const char S14882[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03385); MRSPUTNIK OW 2, 3, 0, 104; ru) Presto/2.10.289 Version/12.02";
const char S14883[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 6007); ru) Presto/2.10.289 Version/12.02";
const char S14884[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6046); ru) Presto/2.10.289 Version/12.02";
const char S14885[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 6003); ru) Presto/2.10.289 Version/12.02";
const char S14886[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6081); ru) Presto/2.10.289 Version/12.02";
const char S14887[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5730); ru) Presto/2.10.289 Version/12.02";
const char S14888[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 6008); ru) Presto/2.10.289 Version/12.02";
const char S14889[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6083); ru) Presto/2.10.289 Version/12.02";
const char S14890[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6089); ru) Presto/2.10.289 Version/12.02";
const char S14891[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6042); ru) Presto/2.10.289 Version/12.02";
const char S14892[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.289 Version/12.02";
const char S14893[] = "Opera/9.80 (Windows NT 5.1; U; en) Presto/2.10.289 Version/12.02,gzip(gfe) (via docs.google.com/viewer)";
const char S14894[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02830); ru) Presto/2.10.289 Version/12.02";
const char S14895[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02620); ru) Presto/2.10.289 Version/12.02";
const char S14896[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 3925); ru) Presto/2.10.289 Version/12.02";
const char S14897[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 6017); ru) Presto/2.10.289 Version/12.02";
const char S14898[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 6019); ru) Presto/2.10.289 Version/12.02";
const char S14899[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6639); ru) Presto/2.10.289 Version/12.02";
const char S14900[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6683); ru) Presto/2.10.289 Version/12.02";
const char S14901[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6673); ru) Presto/2.10.289 Version/12.02";
const char S14902[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.289 Version/12.02,gzip(gfe),gzip(gfe)";
const char S14903[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.1 (build 6341); ru) Presto/2.10.289 Version/12.02";
const char S14904[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14905[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14906[] = "Mozilla/5.0 (Linux; U; Android >>>>>CRISKELO-ROM Ver 54<<<<< 2.3.3; es-es; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14907[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-I9000M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14908[] = "Mozilla/5.0 (Linux; U; Android XXJVO 2.3.3 Powered; it-it; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14909[] = "Mozilla/5.0 (Linux; U; Android GINGERBREAD 2.3.3; vi-vn; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14910[] = "Mozilla/5.0 (Linux; U; Android >>>>>CRISKELO-ROM Ver 56<<<<< 2.3.3; fr-fr; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14911[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9000-ORANGE/I9000BVJV7 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14912[] = "Mozilla/5.0 (Linux; U; Android >>>>>CRISKELO-ROM Ver 59<<<<< 2.3.3; es-es; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14913[] = "Mozilla/5.0 (Linux; U; Android >>>>>CRISKELO-ROM Ver 51<<<<< 2.3.3; es-es; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14914[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; GT-I9000-ORANGE/I9000BVJV8 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14915[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; GT-I9000B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14916[] = "Mozilla/5.0 (Linux; U; Android Speed N Looks 2.3.3; en-us; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14917[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9000 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S14918[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14919[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-I9000 Build/GINGERBREAD)";
const char S14920[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14921[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-I9000 Build/GINGERBREAD)";
const char S14922[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9000 Build/GalnetMIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14923[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14924[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-I9000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14925[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14926[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-I9000 Build/GINGERBREAD)";
const char S14927[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14928[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9000 Build/GINGERBREAD)";
const char S14929[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-I9000M Build/GINGERBREAD)";
const char S14930[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9000-ORANGE/I9000BVJVC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14931[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; iw-il; GT-I9000 Build/HebMIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14932[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14933[] = "Mozilla/5.0 (Linux; U; Android 2.3.6JW1; nl-nl; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14934[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-I9000B Build/GINGERBREAD)";
const char S14935[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; GT-I9000 Build/GINGERBREAD)";
const char S14936[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9000 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S14937[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9000 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S14938[] = "Mozilla/5.0 (Linux; U; Android 2.3.6JW4; nl-nl; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14939[] = "Mozilla/5.0 (Linux; U; Android 2.3.3;en-US;GT-I9000 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 AppEngine-Google; (+http://code.google.com/appengine; appid: s~google.com:mobile-checker)";
const char S14940[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14941[] = "Mozilla/5.0 (Linux; U; Android >>>>>CRISKELO-ROM Ver 69<<<<< 2.3.4; es-es; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14942[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; GT-I9000 Build/MYUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14943[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; fa-fa; GT-I9000 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S14944[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9000 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S14945[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; iw-il; GT-I9000 Build/HEBMIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14946[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT-I9000 Build/Conspicuous) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14947[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9000 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S14948[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9000 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S14949[] = "Mozilla/5.0 (Linux; U; Android 2.3.6 XXJVU; it-it; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14950[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9000 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S14951[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9000 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S14952[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-I9000B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14953[] = "Mozilla/5.0 (Linux; U; Android 2.3.6 XXJW4; it-it; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14954[] = "Mozilla/5.0 (Linux; U; Android 2.3.6JVU; nl-nl; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14955[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14956[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-I9000/I9000BOJW6 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14957[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14958[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9000 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S14959[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9000 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S14960[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-I9000-ORANGE/I9000BVJVD Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14961[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-I9000M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14962[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14963[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-cn; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14964[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S14965[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3)";
const char S14966[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Tablet PC 2.0; Media Center PC 6.0; SLCC2)";
const char S14967[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Media Center PC 6.0)";
const char S14968[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; ASU2JS)";
const char S14969[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Tablet PC 2.0; Zune 4.7; InfoPath.3)";
const char S14970[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; ASU2JS)";
const char S14971[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Tablet PC 2.0; ASU2JS)";
const char S14972[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; MASMJS)";
const char S14973[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; ASU2JS)";
const char S14974[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Tablet PC 2.0; Media Center PC 6.0; SLCC2; Zune 4.7)";
const char S14975[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; Touch; .NET4.0E; .NET4.0C; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7)";
const char S14976[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Media Center PC 6.0; SLCC2)";
const char S14977[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; Tablet PC 2.0; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7)";
const char S14978[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17";
const char S14979[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.1.0.0 Safari/537.17";
const char S14980[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.1.0.0 Safari/537.17";
const char S14981[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S14982[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.2.0.0 Safari/537.17";
const char S14983[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.2.0.0 Safari/537.17";
const char S14984[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)";
const char S14985[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe) (via docs.google.com/viewer)";
const char S14986[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)";
const char S14987[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe) (via docs.google.com/viewer)";
const char S14988[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.17 (KHTML like Gecko) Chrome/24.0.1312.52 Safari/537.17";
const char S14989[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S14990[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S14991[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.106 Safari/535.2,gzip(gfe) (via docs.google.com/viewer)";
const char S14992[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.106 Safari/535.2,gzip(gfe) (via translate.google.com)";
const char S14993[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2,gzip(gfe) (via translate.google.com)";
const char S14994[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/534.7 ZemanaAID/FFFF0061";
const char S14995[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2,gzip(gfe) (via docs.google.com/viewer)";
const char S14996[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2,gzip(gfe)";
const char S14997[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2,gzip(gfe)";
const char S14998[] = "Opera/9.80 (Series 60; Opera Mini/6.5.27309/26.1305; U; id) Presto/2.8.119 Version/10.54";
const char S14999[] = "Opera/9.80 (Series 60; Opera Mini/6.5.27354/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S15000[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC/DesireS/1.32.161.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15001[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC/DesireS/1.32.161.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15002[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_DesireS-orange-LS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15003[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15004[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_DesireS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15005[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_DesireS_S510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15006[] = "HTC_DesireS_S510e_Mozilla/5.0(Linux; U; Android 2.3.3;it-it; Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15007[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-ch; HTC/DesireS/1.47.166.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15008[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_DesireS_S510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15009[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sl-; HTC_DesireS_S510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15010[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC_DesireS_S510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15011[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; HTC/DesireS/1.47.163.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15012[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC/DesireS/1.47.161.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15013[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; HTC/DesireS/1.32.163.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15014[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-ch; HTC/DesireS/1.32.166.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15015[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; th-th; HTC_DesireS_S510b Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15016[] = "Mozilla/5.0 (Linux; U; Android 2.3; xx-xx; HTC/DesireS/1.07.163.1 Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15017[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15018[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Desire S Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15019[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC_Desire_S_S510e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15020[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_Desire_S_S510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15021[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_DesireS_S510e/1.0 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15022[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC_DesireS_S510e Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15023[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; pt-pt; HTC/DesireS/1.32.168.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15024[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Desire S Build/GRI40)";
const char S15025[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/DesireS/2.10.161.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15026[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_DesireS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15027[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh_tw; HTC Desire S Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)";
const char S15028[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-ch; HTC/DesireS/2.10.166.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15029[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Desire S Build/GRJ90)";
const char S15030[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_DesireS-orange-LS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15031[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-at; HTC/DesireS/2.10.163.5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15032[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15033[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-th; HTC_DesireS_S510b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15034[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire S Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S15035[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/DesireS/1.47.161.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15036[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Desire S Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S15037[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_Desire_S_S510e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15038[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC_DesireS_S510e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15039[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_Desire_SMS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15040[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTCS510e/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15041[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Desire S Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S15042[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-pt; HTC/DesireS/2.10.168.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15043[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire S Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S15044[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-; HTC_DesireS_S510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15045[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Desire S Build/GRJ90)";
const char S15046[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15047[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-hk; HTCS510e/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15048[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_DesireS_S510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile";
const char S15049[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/DesireS/2.10.161.5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15050[] = "Mozilla 5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Desire S Build GRI40) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S15051[] = "Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire S Build GRJ90) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S15052[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15053[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC_DesireS_S510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15054[] = "stagefright/1.2 (Linux;Android 4.0.3)";
const char S15055[] = "Mozilla/5.0 (Linux; U; Android 4.0.3.0.2.0.1.0; en-us; Build/OPENMASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15056[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/IML75) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15057[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15058[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15059[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ITL41F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15060[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15061[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/ITL41F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15062[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15063[] = "Mozilla/5.0 (Linux; U; Android 4.0.4.0.4.0.4; en-us; Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15064[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15065[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; CHM805HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15066[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; STM712HCZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15067[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/4.03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15068[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/OPENMASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15069[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15070[] = "stagefright/1.2 (Linux;Android 4.0.4)";
const char S15071[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ICS.f04ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15072[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/IMM76K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30";
const char S15073[] = "Mozilla/5.0 (Linux; U; Android 4.0.3.0.2.0.1.0; en-us; Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15074[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; MID Build/IML74K)";
const char S15075[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/IMM76K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15076[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15077[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/IMM30B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15078[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; STM805HCZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15079[] = "Mozilla/5.0 (Linux; U; Android 4.0.3.0.2.0.1.0; en-us; Build/OPENMASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15080[] = "stagefright/1.2 (Linux;Android 4.0.2)";
const char S15081[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; 97D Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15082[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; NAM805HCX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15083[] = "stagefright/1.2 (Linux;Android 4.0.1)";
const char S15084[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; CRYM1005HN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15085[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/OPENMASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15086[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; e1108_v73_jbaol1g18flash8p4 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15087[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; TEACM1005HN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15088[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T-90 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15089[] = "Mozilla/5.0 (Linux; U; Android 4.0.3.0.2.0.1.0; en-us; Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15090[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; eeepc Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15091[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15092[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15093[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; pad Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15094[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15095[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; ST8001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15096[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_jbl1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15097[] = "Mozilla/5.0 (Linux; U; Android 4.0.4.0.4.0.4; en-us; Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15098[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Android SDK built for x86 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15099[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_hjy1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15100[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ICS.f06ref848L.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15101[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/Xv2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15102[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15103[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Cat Tablet Android 4.0.4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15104[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; NEXT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15105[] = "Mozilla/5.0 (Linux; U; Android 4.0.4.0.4.0.4; en-us; Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15106[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3 Build/IML74K)";
const char S15107[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; PEM727HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15108[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; CHARM712HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15109[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PTM712HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15110[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; DATAM805HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15111[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; ARGM816HCX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15112[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15113[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ITL41F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15114[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; ADM816HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15115[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; ARGM727HCX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15116[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; ARGM816HCX_A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15117[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; CRYM805HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15118[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ICS.g12refM1005.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15119[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; e1108_v73_jbaol1g18 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15120[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; TR720F Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15121[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GPLUS S101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15122[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GOCLM712HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15123[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VegaCream RC2 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15124[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; eeepc Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15125[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; e1808_v75_hjy1a6x Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15126[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-us; generic Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S15127[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16;";
const char S15128[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3 Build/ICS.f04ref.X)";
const char S15129[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15130[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15131[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; e1808_v75_jbl1 Build/IML74K)";
const char S15132[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/XGS2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15133[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_jbl2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15134[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; 720F Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15135[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_jbl1_5640 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15136[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC_GOF_U Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15137[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/OPENMASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15138[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-1230 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15139[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; %ModelName% Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15140[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LUBM816HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15141[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; PAD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15142[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3 Build/ITL41F)";
const char S15143[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_jbaol1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15144[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ICS.f11ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15145[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; EFM727HCF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15146[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15147[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; DLHD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15148[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_hjy1_5647 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15149[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1808_v75_jbl2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15150[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1808_v75_jbl1_5640 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15151[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; PC936 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15152[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.2 Version/4.0 Mobile Safari/530.17";
const char S15153[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; V1013 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15154[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; NEOBOX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15155[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; e1808_v75_hjy1a6x_5647 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15156[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TWD_MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15157[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.2 Version/4.0 Mobile Safari/530.17";
const char S15158[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; HTC_POO_U Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15159[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15160[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MID Build/IMM76D)";
const char S15161[] = "Mozilla/5.0 (Linux; U; Android 4.0.0; it-it; Android Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15162[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_jbl1a535 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15163[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_hjy1a6x_in Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15164[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; P079R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15165[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S15166[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn Build/S-2-20) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S15167[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1911_v77_hjy1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15168[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Android T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15169[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/GB.AMLOGIC.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15170[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SPX_8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15171[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Tender Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15172[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EBM727KC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15173[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Telechips TCC8800 Evaluation Board Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15174[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Wxt Tab PC 65CXi Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15175[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Oysters T8 A4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15176[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VB100a Pro Build/ICS.g08refem611.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15177[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ICS.g04ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15178[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MID72C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15179[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; PTM723MR Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15180[] = "N1 Linux/3.0.13 Android/4.0.4 Release/08.15.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S15181[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-gb; e1909c_v77_gq1008_9p017_6228 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15182[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; mi-book Mi1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15183[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID970T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15184[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15185[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Junior 4.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15186[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15187[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; CW V80 Build/ICS.g08refem618.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15188[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JY131C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15189[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Alpha GT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15190[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TRIO7C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15191[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Mercury_magiQ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15192[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; G1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15193[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HanZpad Build/master-ics-android-4.0.4_r1.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15194[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i7S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15195[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TM-MID732 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15196[] = "Mozilla/5.0 (Linux; U; Android 4.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15197[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PLT1044 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15198[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; TB4-4/5580 Build/ITL41D) AppleWebKit/434.50 (KHTML, like Gecko) Version/4.0 Mobile";
const char S15199[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; PC678 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15200[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3 Build/X)";
const char S15201[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; 97FC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15202[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; MIDC907PR001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15203[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; IGUANAPAD M7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15204[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SANEI Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15205[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; M971B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15206[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TZ100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15207[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; M19 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15208[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; P90 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15209[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; samsung galaxy s2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15210[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; HFM752HCF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15211[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M746 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15212[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; TR976 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15213[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15214[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; SXZ-PDX0-01 Build/SXZ-PDX0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15215[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ICS.f06ref1024.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15216[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; MID08 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15217[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D701A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15218[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ATOM PAD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15219[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; zh-cn; 9151 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15220[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; RC0710B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15221[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; M701F Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15222[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; INFINITY_PASSION Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15223[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; Europe_4G-4s Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15224[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LA-I Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15225[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E140K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15226[] = "MOT-XT685/1.0 Android/4.0.4 Release/9.15.2011 Browser/AppleWebKit534.30  Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT685 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15227[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Explay Informer 701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15228[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; PC8005 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15229[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT7220S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15230[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; 18622 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15231[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ar-eg; ACCM700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15232[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; el-gr; IFQM712HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15233[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; BOULM805ND Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15234[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M758A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15235[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ASTRO+9.7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15236[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GEM9212 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15237[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; EVOPAD R7010 Build/2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15238[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; YAN 7.1C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15239[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; EXLP2R Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15240[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DOPO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15241[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PTM712MCF Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15242[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GX8404 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15243[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MW0922 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15244[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; California Access TAB MM 1001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15245[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PICOpad_GEA Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15246[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ACGM727HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15247[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; VORTEX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15248[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; KidsPad1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15249[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; FAR7 Build/V02R10A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15250[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Mi501 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15251[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; e1911_v77_gq2000_ov5647 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15252[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JW101A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15253[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; ALEXIS-RX5DC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15254[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; POM805HCF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15255[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GASEM712MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15256[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; WALMM752MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15257[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; INFINITY_POWER Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15258[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; HTC_H1000C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15259[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M4302 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15260[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; UTAB71 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15261[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Tablet-PC-4 Build/ICS.g08refem618.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15262[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15263[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MW0710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15264[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HD7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15265[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; CTC07SO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15266[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; unknown Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15267[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; E71 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15268[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; TAB7012 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15269[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T03A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15270[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; M805 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15271[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; P-08D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15272[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; T9200 Build/HuaweiT9200) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S15273[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; G15 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15274[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; amd_brazos Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15275[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; GASEM805HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15276[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; TAB9 3G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15277[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; tablet_pc Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15278[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; PNDPP4MT9G2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15279[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; MID900SC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15280[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N101 DUAL CORE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15281[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; PC728 Build/PC728) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15282[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; MID14 Build/MID14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15283[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; 7018 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15284[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Zyrex_ZA987_OneScribe Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15285[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; AMOI N820 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S15286[] = "sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/Android4.0.3 Release/08.22.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S15287[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; PA912B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15288[] = "HUAWEI_T9200_TD/1.0 Android/4.0.3 (Linux; U; Android 4.0.3; zh-cn) Release/05.31.2012 Browser/WAP2.0 (AppleWebKit/534.30)";
const char S15289[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CJ ThL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15290[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; AN8BG3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15291[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; XPRESS PRO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15292[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; EVOPad R8000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15293[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; gs701b Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15294[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; XELIO Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15295[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; 101T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15296[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; sl-si; XP717 Build/ICS.f08refem32.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15297[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TAB7004GBBK Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15298[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I97D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15299[] = "HUAWEI_T9200_TD/1.0 Android/4.0.3 (Linux; U; Android 4.0.3; zh-cn) Release/05.31.2012 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30";
const char S15300[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-gb; e1909c_v77_gq1008_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15301[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID-001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15302[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; TB512-16G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15303[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15304[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; kiwi k5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15305[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; NB017 / NB018 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15306[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M1005D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15307[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PX1345 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15308[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Build/ICS.Zoostorm_130.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15309[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q29 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15310[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N9977 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15311[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MHU001D Build/MHU001D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15312[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; CTA Build/ICS.f32ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15313[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ro-ro; SUPREME IPS Dual Core X200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15314[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Swipe AllinOne Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15315[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Build/ICS.MID-701SC.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15316[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N10-4core Build/R13.4-rc2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15317[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Eclipse-X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15318[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; J-Q8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15319[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; SKYTEX-SP438 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15320[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Pad Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15321[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; Velocity-V1 Build/T860) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15322[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MANHATTAN Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15323[] = "Android 4.0.4; Application: com.e8tracks";
const char S15324[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Android Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15325[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; MID ro.product.brand=Android Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15326[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; unknown Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15327[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1908_v77_jbl1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15328[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; Build/ICS.w17ref.Dualcore.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15329[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; e1911_v77_jbaol_g20 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15330[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-gb; e1909c_v77_gq1008_5647 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15331[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; Build/ICS.Lazer.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15332[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-nz; Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15333[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; CTM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15334[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; yuanda75_ics Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15335[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; CT710FR Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15336[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; 116DN Build/ICS.g08refem611.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15337[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; POM727MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15338[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1808_v75_jbl2_6628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15339[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NUCLEAR Build/GT704-A13005-A04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15340[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-; Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15341[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1908_v77_jbl1a535 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15342[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; My Play Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15343[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1911_v77_mt6628_hjy1_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15344[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_hjy1_5640_maxwest Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15345[] = "stagefright/1.1 (Linux;Android 4.0.3)";
const char S15346[] = "stagefright/1.1 (Linux;Android 4.0.4)";
const char S15347[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9999_custom Build/RMU) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15348[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IDOIPAD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15349[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn Build/S-2-20) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S15350[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ICS.f03ref_NX.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15351[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; GLORY Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15352[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; hexing77_ics2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15353[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S150 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15354[] = "Mozilla/5.0 (Linux; U; Android 4.0.0; en-us; Android Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15355[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/ICS.f06ref848L.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15356[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; S3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15357[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; TWD_MID Build/IMM76D)";
const char S15358[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Build/ICS.VSA.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15359[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_hjy1_5647_maxwest Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15360[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1808_v75_jbl2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15361[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MSM7227A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15362[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_jbaol1_5647 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15363[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1908_v77_hjy1a6x Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15364[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iPhone5 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15365[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; HD II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15366[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1808_v75_jbl1a535mt9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15367[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Pioneer E80w Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15368[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mini-2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15369[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; e1909c_v77_gq1008_5647_6228 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15370[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; WP7002 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15371[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; ST8002 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15372[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; MID Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15373[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Swipe Walk n Talk Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15374[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1809l_v75_gq1008_ov5647_v20 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15375[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 97D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15376[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; DOUBLE II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15377[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1808_v75_jbl1_5647 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15378[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-us; Smartphones Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15379[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; iPhone 4S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15380[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Build/Coolpad7728_V02.02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15381[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Build/ICS.f03ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15382[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NEOBOX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15383[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; S815_HZT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15384[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1808_v75_hjy1_5640_maxwest Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15385[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fi-fi; Build/ICS.pad10ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15386[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Build/ICS.g12refM805.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15387[] = "Mozilla/5.0 (Linux; U; Android 4.0.8; en-us; Android Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15388[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; e1808_v75_jbl1_5640 Build/IML74K)";
const char S15389[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Android Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15390[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; msm8x25 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15391[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; unknown Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15392[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1911_v77_jyuany1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15393[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1911_v77_hjy1_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15394[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15395[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1911_v77_mt6628_hjy1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15396[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; Build/ICS.ATABLET7306.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15397[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID Serials Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15398[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ICS.g05_n1asref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15399[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T730 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15400[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; HTC_PROMIN_U Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15401[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; CTA Build/ICS.casper.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15402[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; hexing75_hspa   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15403[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Build/06.03.001.X.7728) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15404[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; unknown Build/IMM76D)";
const char S15405[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Build/ICS.g05_n3aenref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15406[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1911_v77_jbaol_g20mt9p017fwvga_6628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15407[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Build/ICS.g05_n1benref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15408[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; S88 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15409[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1809l_v75_jbl_253_v20 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15410[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_nd1antv Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15411[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us Build/SmartDroid Berserk_V 2.3_DummyPlug_) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15412[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; ru-ru; iPhone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15413[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1808_v75_hjy1_9p017_maxwest Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15414[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; e1908_v77_nd3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15415[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1908_v77_jbl1ntv Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15416[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; HIGH QUALITY MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15417[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; S129 Build/Android4.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15418[] = "Dalvik/1.4.0 (Linux; U; Android 4.0.9; e1108_v73_jbaol1g18flash8p4 Build/GRK39F)";
const char S15419[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iphone 5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15420[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; TWD_MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15421[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_jbl1_5640a522 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15422[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Build/HuaweiT9510E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15423[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1908_v77_mt6628_hjy1a6x_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15424[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; STM712HCZ Build/IML74K)";
const char S15425[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3 Build/Xv2)";
const char S15426[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3 Build/ICS.f06ref848L.X)";
const char S15427[] = "AndroidCvpPlayer(Linux; U; Android 4.0.4; en-us)";
const char S15428[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; e1808_v75_jbl2 Build/IML74K)";
const char S15429[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Build/ICS.MBX.ENERGY.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15430[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Eclipse 4.3 MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15431[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Build/ICS.g05_n3aref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15432[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Build/ICS.f06ref1024.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15433[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; J-Q88D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15434[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/G6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15435[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-nz; Build/ICS.OCTAVO.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15436[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; msm8x25 Build/JIEVIS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15437[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; hexing75_ics Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15438[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PHM727MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15439[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; T730 Build/IMM76D)";
const char S15440[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; e1808_v75_hjy1a6x_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15441[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; e1808_v75_hjy1_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15442[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1911_v77_jbaol_g20mt9p017_6628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15443[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; hexing75_hspa   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15444[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; PAD Build/IMM76D)";
const char S15445[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; 720F Build/IML74K)";
const char S15446[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 4G LTE  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15447[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; Build/ICS.g04ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15448[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; TR720F Build/IML74K)";
const char S15449[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3 Build/S-2-20)";
const char S15450[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; %ModelName% Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15451[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15452[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; e1808_v75_jbl1_5640_cony Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15453[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; MID 940 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15454[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; e1908_v77_jbl1mt9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15455[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15456[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1808_v75_jbaol1_6628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15457[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; iPhone Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15458[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; E60w Build/PioneerE60w) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15459[] = "Mozilla/5.0 (Linux; U; Android 4.0.8; en-us; Android Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15460[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1911_v77_jbaol_g20mt9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15461[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Build/ICS.by Djzemych mod 21.09.12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15462[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; e101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15463[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Build/IRPMTD_6_02.47.00RPS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15464[] = "Mozilla/5.0 (Linux; U; Android 4.0.0; vi-vn; Android Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15465[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15466[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ICS.BEAT.DUOS.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15467[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Tender Build/IMM76D)";
const char S15468[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; e1911_v77_zlh1_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15469[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MID 008 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15470[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; e1808_v75_hjy1a6x Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15471[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; in-id; e61 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15472[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; e1909c_v77_jbl_9p017_6628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15473[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1808_v75_mt6628_hjy1a6x_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15474[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Unknown Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S15475[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/4.0.13574/20.2485; U; en) Presto/2.5.25";
const char S15476[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/4.0.10031/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S15477[] = "Opera/9.50 (J2ME/MIDP; Opera Mini/4.0.10031/298; U; en)";
const char S15478[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/4/30.3558; U; en) Presto/2.8.119 Version/11.10";
const char S15479[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.36 Safari/535.7 AppEngine-Google; (+http://code.google.com/appengine; appid: s~quexdoo)";
const char S15480[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7,gzip(gfe) (via translate.google.com)";
const char S15481[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7,gzip(gfe) (via translate.google.com)";
const char S15482[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7,gzip(gfe) (via docs.google.com/viewer)";
const char S15483[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S15484[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S15485[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/534.7 ZemanaAID/FFFF0061";
const char S15486[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7,gzip(gfe)";
const char S15487[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7,gzip(gfe)";
const char S15488[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.910.20417 KChrome/3.2.5.20417  Safari/535.7";
const char S15489[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.910.20417 KChrome/3.2.5.20417  Safari/535.7";
const char S15490[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S15491[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; JRCapture) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S15492[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.904.0 Safari/535.7,gzip(gfe) (via docs.google.com/viewer)";
const char S15493[] = "Mozilla/5.0 (Windows 7) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S15494[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S15495[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S15496[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S15497[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)";
const char S15498[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via docs.google.com/viewer)";
const char S15499[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe)";
const char S15500[] = "Mozilla/5.0 (Windows NT 5.1; ; NCLIENT50_AAPXA934) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.79 Safari/537.1";
const char S15501[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.0 AOL/9.7 AOLBuild/4343.2039.US Safari/537.1";
const char S15502[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/4.3.25172/25.858; U; en) Presto/2.5.25 Version/10.54";
const char S15503[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/4.3.25985/30.3697; U; en) Presto/2.8.119 Version/11.10";
const char S15504[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/4.3.13057/19.836; U; ru) Presto/2.5.25";
const char S15505[] = "Opera/9.80 (BlackBerry; Opera Mini/7.0.29990/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S15506[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/6.5.26955/26.1283; U; en) Presto/2.8.119 Version/10.54";
const char S15507[] = "Opera/9.50 (J2ME/MIDP; Opera Mini/6.5.28416/28.3165; U; en)";
const char S15508[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/6.5.26955/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S15509[] = "0 Opera/9.80 (J2ME/MIDP; Opera Mini/6.5.27510/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S15510[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.46 Safari/535.11";
const char S15511[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S15512[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S15513[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11,gzip(gfe) (via docs.google.com/viewer)";
const char S15514[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/534.7 ZemanaAID/FFFF0061";
const char S15515[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.31 (KHTML, like Gecko) Chrome/17.0.558.0 Safari/534.31";
const char S15516[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11 AppEngine-Google; (+http://code.google.com/appengine; appid: s~huozhaiyuren)";
const char S15517[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.84 Safari/535.11 SE 2.X MetaSr 1.0,gzip(gfe) (via translate.google.com)";
const char S15518[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.84 Safari/535.11 SE 2.X MetaSr 1.0,gzip(gfe)";
const char S15519[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/17.0.944.0 Safari/535.8 BitdefenderSafepay/0.2.0.0 (OBK; 17.0.944.0; r396.2011)";
const char S15520[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.84 Safari/535.11 SE 2.X MetaSr 1.0";
const char S15521[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML like Gecko) Chrome/17.0.963.46 Safari/535.11";
const char S15522[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.1.963.51 Safari/535.11 MRCHROME SOC";
const char S15523[] = "Mozilla/5.0 (Windows NT 5.1; Alibrowser 1.8) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S15524[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31,gzip(gfe)";
const char S15525[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S15526[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S15527[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S15528[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S15529[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S15530[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S15531[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)";
const char S15532[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31,gzip(gfe)";
const char S15533[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S15534[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S15535[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S15536[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S15537[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S15538[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S15539[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S15540[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S15541[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S15542[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)";
const char S15543[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe) (via docs.google.com/viewer)";
const char S15544[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/25.0.648.205 Safari/534.16";
const char S15545[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/25.0.648.205 Safari/534.16";
const char S15546[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/25.0.648.205 Safari/534.16";
const char S15547[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1";
const char S15548[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1";
const char S15549[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/534.7 ZemanaAID/FFFF0061";
const char S15550[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.218 Safari/535.1,gzip(gfe) (via translate.google.com)";
const char S15551[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1,gzip(gfe) (via translate.google.com)";
const char S15552[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; ru-RU) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/13.0.782.218 Safari/533.3";
const char S15553[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1 AppEngine-Google; (+http://code.google.com/appengine; appid: s~pickieco10)";
const char S15554[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1,gzip(gfe)";
const char S15555[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1,gzip(gfe)";
const char S15556[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML like Gecko) Chrome/13.0.782.24 Safari/535.1";
const char S15557[] = "Mozilla/5.0 (Windows NT 6.1; compatible; AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1";
const char S15558[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT18i Build/4.0.1.A.0.119) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15559[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT18i Build/4.0.1.A.1.17) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15560[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; SonyEricssonLT18a Build/4.0.1.A.0.227) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15561[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT18i Build/4.0.1.A.0.254) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15562[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT18i Build/4.0.1.A.0.266) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15563[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT18i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15564[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; LT18i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15565[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT18i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15566[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT18iv Build/4.0.1.A.0.266) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15567[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT18i Build/4.0.1.A.0.284) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15568[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LT18i Build/4.0.1.A.0.266) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15569[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SonyEricssonLT18i-o Build/4.0.1.A.0.266) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15570[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; SonyEricssonLT18a Build/4.0.1.A.0.165) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15571[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SonyEricssonLT18iv Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15572[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SonyEricssonLT18i-o Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15573[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LT18i Build/4.0.2.A.0.62)";
const char S15574[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonLT18a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15575[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; SonyEricssonLT18a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15576[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SonyEricssonLT18i-o Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15577[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15578[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LT18i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15579[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT18i Build/4.0.2.C.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15580[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonLT18i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15581[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonLT18iv Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15582[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15583[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.42) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S15584[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.58) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S15585[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; SonyEricssonLT18a Build/4.0.2.D.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15586[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.62) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S15587[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; LT18a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15588[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LT18i Build/4.0.2.C.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15589[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-hk; SonyEricssonLT18i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15590[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; SonyEricssonLT18a Build/4.0.2.D.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15591[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; SonyEricssonLT18a Build/4.0.1.A.0.266) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15592[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LT18a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15593[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; LT18i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15594[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.1.A.0.266) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S15595[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.62) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S15596[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15597[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LT18a Build/4.0.2.A.0.62)";
const char S15598[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; LT18a Build/4.0.1.A.0.266) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15599[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-; SonyEricssonLT18a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15600[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-; SonyEricssonLT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15601[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; LT18a Build/4.0.2.D.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15602[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.58) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S15603[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.69) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S15604[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.42) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S15605[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonLT18iv Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15606[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.1.A.0.266) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S15607[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonLT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15608[] = "Mozilla 5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build 4.0.2.A.0.62) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S15609[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15610[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC One X Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15611[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15612[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC One X Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15613[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15614[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; One X Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15615[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC One X Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15616[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; One X Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15617[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC One X Build/JRO03C)";
const char S15618[] = "Mozilla/5.0 (Linux; U; Android 4.1.5; en-us; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15619[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15620[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; One X Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15621[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; One X Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15622[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; zh-cn; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15623[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; EndeavorU Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15624[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC One X Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S15625[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; HTC EVA_UL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15626[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_X-orange-LS Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15627[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_X/3.14.161.27 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15628[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; EndeavorU Build/JRO03C)";
const char S15629[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; HTC_One_X/3.14.163.27 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15630[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-ch; HTC_One_X/3.14.166.27 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15631[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_X/3.16.110.4 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15632[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_X/3.14.111.27 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15633[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC S720e Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15634[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_X/2.17.161.4 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15635[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC One X Build/JZO54K)";
const char S15636[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-at; HTC_One_X/3.14.169.27 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15637[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-at; HTC_One_X/3.14.112.27 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15638[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_X/2.17.111.3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15639[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; HTC_One_X/3.14.118.27 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15640[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC S720e Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S15641[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_X/2.17.110.5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15642[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; HTC_One_X/3.14.114.27 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15643[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PJ83100/3.18.502.6 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15644[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; HTC One X Build/IMM76D) UC AppleWebKit/534.31 (KHTML like Gecko) Mobile Safari/534.31";
const char S15645[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; HTC_One_X/2.17.163.4 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15646[] = "Mozilla/5.0 (Linux; U; Android 4.1.5; en-us; HTC One X Build/IML74K) UC AppleWebKit/534.31 (KHTML like Gecko) Mobile Safari/534.31";
const char S15647[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15648[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; One X Build/JZO54K)";
const char S15649[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-; HTC_One_X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15650[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; HTC One X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15651[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PJ83100/2.20.502.7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15652[] = "Mozilla/5.0 (Linux; U; Android 4.1.5; zh-cn; HTC One X Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S15653[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC One X Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15654[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15655[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_X/3.14.161.28 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15656[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; HTC One X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15657[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-ch; HTC_One_X/2.17.166.3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15658[] = "Mozilla/5.0 (Linux; U; Android 4.1.5; zh-cn; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15659[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; zh-cn; HTC One X Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S15660[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC S720e Build/JRO03C)";
const char S15661[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTCOneX Build/JRO03C) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15662[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S720e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15663[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PJ83100/3.18.502.6 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15664[] = "Mozilla/5.0 (Linux; U; Android 4.1.5; en-us; HTC One X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15665[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_X/3.14.111.28 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15666[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PJ83100/3.18.502.6 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15667[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; vi-vn; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15668[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; vi-vn; HTC One X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15669[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_X Build/JB REMIX 3.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15670[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.1;dev=evita;devm=HTC One X;man=HTC;brw=Android;shwd=720x1280;shwa=720x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S15671[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.25 (KHTML, like Gecko) Sparrow/1081.53";
const char S15672[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) QuickLook/4.0";
const char S15673[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.25 (KHTML, like Gecko) Sparrow/1081.52";
const char S15674[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en-US; rv:1.9.2.10) Gecko Expeditor 6.2.2.X-1115 Gecko/X-1115";
const char S15675[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/1178";
const char S15676[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/1164";
const char S15677[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/1173";
const char S15678[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/1172";
const char S15679[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) NetNewsWire/3.3";
const char S15680[] = "QuickTime/7.7.1 (qtver=7.7.1;cpu=IA32;os=Mac 10.8.2)";
const char S15681[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/1176";
const char S15682[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) NetNewsWire/3.3.2";
const char S15683[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Vienna/2.6.0.2601";
const char S15684[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Gruml/1.2";
const char S15685[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) NetNewsWire/3.2.15";
const char S15686[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Safari/522.0";
const char S15687[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Vienna/3.0b.2820";
const char S15688[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Gruml/1.3.0";
const char S15689[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Shrook/2.85l";
const char S15690[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sunrise";
const char S15691[] = "Mozilla/4.0 (Mac OS X 10.8.2) Java/1.7.0_09";
const char S15692[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/1143.6";
const char S15693[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) NetNewsWire/3.3.1";
const char S15694[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) RealPlayer/12.0.1.1750";
const char S15695[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Vienna/3.0b.2821 Safari/536.26.17";
const char S15696[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) iWeb/304";
const char S15697[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) NetNewsWire/3.1.7";
const char S15698[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) TubeTV";
const char S15699[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/1043.1";
const char S15700[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) NetNewsWire/3.2.7";
const char S15701[] = "QuickTime/7.7.1 (qtver=7.7.1;cpu=IA32;os=Mac 10.8.3)";
const char S15702[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/904";
const char S15703[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/1134";
const char S15704[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/1081.32";
const char S15705[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/1144";
const char S15706[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko)";
const char S15707[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) Vienna/3.0.0 Beta 11 :9d8d6f6: Safari/536.29.13";
const char S15708[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Propane/1.2.5";
const char S15709[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) Safari/536.28.10 webkit2png/dev";
const char S15710[] = "Airmail 1.0.1 rv:172 (Macintosh; Mac OS X 10.8.3; en_US)";
const char S15711[] = "Airmail 1.0.2 rv:175 (Macintosh; Mac OS X 10.8.3; en_US)";
const char S15712[] = "Airmail 1.0.3 rv:176 (Macintosh; Mac OS X 10.8.3; en_US)";
const char S15713[] = "Airmail 1.0.3 rv:176 (Macintosh; Mac OS X 10.8.4; en_US)";
const char S15714[] = "QuickTime/7.7.1 (qtver=7.7.1;cpu=IA32;os=Mac 10.8.4)";
const char S15715[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Safari/417.8";
const char S15716[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/536.11 (KHTML, like Gecko) Safari/536.11";
const char S15717[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/1043.2";
const char S15718[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) Safari/531.9";
const char S15719[] = "Airmail 1.0.4 rv:183 (Macintosh; Mac OS X 10.8.4; en_US)";
const char S15720[] = "Airmail 1.0.4 rv:183 (Macintosh; Mac OS X 10.8.4; en_AU)";
const char S15721[] = "Airmail 1.0.4 rv:183 (Macintosh; Mac OS X 10.8.4; en_GB)";
const char S15722[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) NetNewsWire/4.0.0";
const char S15723[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) Paparazzi!/0.6.7 (like Safari)";
const char S15724[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) Safari/536.30.1 webkit2png/dev";
const char S15725[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/959.4";
const char S15726[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko),gzip(gfe)";
const char S15727[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) NetNewsWire";
const char S15728[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) Shrook/2.88l";
const char S15729[] = "Airmail 1.0.5 rv:192 (Macintosh; Mac OS X 10.8.4; en_AU)";
const char S15730[] = "Airmail 1.0.6 rv:194 (Macintosh; Mac OS X 10.8.4; en_US)";
const char S15731[] = "Airmail 1.0.5 rv:192 (Macintosh; Mac OS X 10.8.4; en_US)";
const char S15732[] = "Airmail 1.0.5 rv:192 (Macintosh; Mac OS X 10.8.4; en_GB)";
const char S15733[] = "Airmail 1.0.5 rv:192 (Macintosh; Mac OS X 10.8.5; en_US)";
const char S15734[] = "Blackberry9700/6.0.0.526 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102";
const char S15735[] = "BlackBerry9700";
const char S15736[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9700; en-US) AppleWebKit/534.8+ (KHTML, like Gecko) Version/6.0.0.448 Mobile Safari/534.8+";
const char S15737[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/22.0.1229.79) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4";
const char S15738[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/22.0.1229.79) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4";
const char S15739[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/22.0.1229.91) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.91 Safari/537.4";
const char S15740[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/22.0.1229.91) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.91 Safari/537.4";
const char S15741[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/22.0.1229.92) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.92 Safari/537.4";
const char S15742[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/22.0.1229.92) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.92 Safari/537.4";
const char S15743[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/22.0.1229.94; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E)";
const char S15744[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/22.0.1229.94)";
const char S15745[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/22.0.1229.94)";
const char S15746[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/22.0.1229.94) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S15747[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/22.0.1229.94) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S15748[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/22.0.1229.94; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Tablet PC 2.0)";
const char S15749[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/22.0.1229.94; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S15750[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; chromeframe/22.0.1229.94)";
const char S15751[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/22.0.1229.94; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S15752[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/22.0.1229.56) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.56 Safari/537.4";
const char S15753[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/22.0.1229.94; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S15754[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/22.0.1229.96) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.96 Safari/537.4";
const char S15755[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/22.0.1229.94; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S15756[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; chromeframe/22.0.1229.94)";
const char S15757[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; GTB7.4; chromeframe/22.0.1229.94; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)";
const char S15758[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; chromeframe/22.0.1229.94)";
const char S15759[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; chromeframe/22.0.1229.94)";
const char S15760[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/22.0.1229.96) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.96 Safari/537.4";
const char S15761[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Avant Browser; chromeframe/22.0.1229.94)";
const char S15762[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/22.0.1229.26) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.26 Safari/537.4";
const char S15763[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/22.0.1229.56) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.56 Safari/537.4";
const char S15764[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; GTB7.4; chromeframe/22.0.1229.94; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C)";
const char S15765[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/22.0.1229.94)";
const char S15766[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/4.0; GTB7.4; chromeframe/22.0.1229.94; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)";
const char S15767[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/22.0.1229.94; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Zune 4.7)";
const char S15768[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/22.0.1229.94)";
const char S15769[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/22.0.1229.94)";
const char S15770[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/22.0.1229.95) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.95 Safari/537.4";
const char S15771[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S15772[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S15773[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S15774[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S15775[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S15776[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S15777[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S15778[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S15779[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe)";
const char S15780[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S15781[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S15782[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT26i Build/6.1.A.0.452) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15783[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT26i Build/6.1.A.0.453) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15784[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26i Build/6.1.A.0.452) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S15785[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LT26i Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15786[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT26i Build/6.1.A.0.454) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15787[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonLT26i Build/6.1.A.2.45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15788[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT26i Build/6.1.A.2.45)";
const char S15789[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT26i Build/6.1.A.2.45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15790[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26i Build/6.1.A.2.45) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S15791[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT26i Build/6.1.A.2.50) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15792[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; LT26i Build/6.1.A.1.58) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15793[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26i Build/6.1.A.0.453) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S15794[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT26i Build/6.1.A.2.55) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15795[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonLT26i-o Build/6.1.A.2.55) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15796[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26i Build/6.1.A.2.50) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S15797[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26i Build/6.1.A.2.55) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S15798[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; SonyEricssonLT26iv Build/6.1.A.2.55) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15799[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT26i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15800[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; LT26i Build/6.1.A.2.45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S15801[] = "Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; LT26i Build 6.1.A.2.45) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S15802[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; LT26i Build/IML74K)";
const char S15803[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S15804[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9505 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S15805[] = "Mozilla/5.0 (Linux; Android 4.2.2; fr-fr; SAMSUNG GT-I9505/I9505XXUAMDF Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S15806[] = "Mozilla/5.0 (Linux; Android 4.2.2; fr-fr; SAMSUNG GT-I9505-ORANGE Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S15807[] = "Mozilla/5.0 (Linux; Android 4.2.2; de-de; SAMSUNG GT-I9505/I9505XXUAMDC Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S15808[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9505X Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S15809[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9505/I9505XXUAMDD Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S15810[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505X Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S15811[] = "Mozilla/5.0 (Linux; Android 4.2.2; de-de; SAMSUNG GT-I9505/I9505XXUAME2 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S15812[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9505/I9505XXUBMEA Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S15813[] = "Mozilla/5.0 (Linux; Android 4.2.2; nl-nl; SAMSUNG GT-I9505/I9505XXUBMF4 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S15814[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; GT-I9505 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S15815[] = "Mozilla/5.0 (Linux; Android 4.2.2; de-de; SAMSUNG GT-I9505/I9505XXUBMGC Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S15816[] = "Mozilla/5.0 (Linux; Android 4.2.2; nl-nl; SAMSUNG GT-I9505/I9505XXUBMGA Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S15817[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9505/I9505XXUDMGG Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S15818[] = "Mozilla/5.0 (Linux; Android 4.2.2; nl-nl; SAMSUNG GT-I9505/I9505XXUDMH7 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S15819[] = "Mozilla/5.0 (Linux; Android 4.2.2; de-de; SAMSUNG GT-I9505/I9505XXUDMH9 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S15820[] = "Mozilla/5.0 (Linux; Android 4.2.2; de-de; SAMSUNG GT-I9505/I9505XXUDMI1 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S15821[] = "Mozilla/5.0 (Linux; Android 4.2.2; nl-nl; SAMSUNG GT-I9505/I9505XXUDMH6 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S15822[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S15823[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S15824[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S15825[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S15826[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S15827[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S15828[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S15829[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S15830[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S15831[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36,gzip(gfe)";
const char S15832[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S15833[] = "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.12";
const char S15834[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.12";
const char S15835[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.12,gzip(gfe)";
const char S15836[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.12";
const char S15837[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.12";
const char S15838[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.12";
const char S15839[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.12";
const char S15840[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.12";
const char S15841[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.12";
const char S15842[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64) Opera 12.12";
const char S15843[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.12";
const char S15844[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.12";
const char S15845[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.12";
const char S15846[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.12";
const char S15847[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.12";
const char S15848[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.12";
const char S15849[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.12";
const char S15850[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.12";
const char S15851[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.12";
const char S15852[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.12";
const char S15853[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.12";
const char S15854[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.12";
const char S15855[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.12";
const char S15856[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.12";
const char S15857[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.12";
const char S15858[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.12";
const char S15859[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.12";
const char S15860[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.12";
const char S15861[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.12";
const char S15862[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.12";
const char S15863[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.12";
const char S15864[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.12";
const char S15865[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.12";
const char S15866[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.12";
const char S15867[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.12";
const char S15868[] = "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.12,gzip(gfe)";
const char S15869[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.12";
const char S15870[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.12";
const char S15871[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.12";
const char S15872[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.12";
const char S15873[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.12";
const char S15874[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.12";
const char S15875[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.12";
const char S15876[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.12";
const char S15877[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.12";
const char S15878[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.12";
const char S15879[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.12";
const char S15880[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.12";
const char S15881[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.12";
const char S15882[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.12";
const char S15883[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.12";
const char S15884[] = "Opera/9.80 (Windows NT 6.1; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.12";
const char S15885[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.12";
const char S15886[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.12";
const char S15887[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.12";
const char S15888[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.12";
const char S15889[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.12 Yandex.Translate";
const char S15890[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.12";
const char S15891[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.12";
const char S15892[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5660)) Presto/2.12.388 Version/12.12";
const char S15893[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03639)) Presto/2.12.388 Version/12.12";
const char S15894[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.12";
const char S15895[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.12";
const char S15896[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4933)) Presto/2.12.388 Version/12.12";
const char S15897[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.12";
const char S15898[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4661)) Presto/2.12.388 Version/12.12";
const char S15899[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03789)) Presto/2.12.388 Version/12.12";
const char S15900[] = "Opera/9.80 (Windows NT 6.1; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.12";
const char S15901[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.12";
const char S15902[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4832)) Presto/2.12.388 Version/12.12";
const char S15903[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.12";
const char S15904[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5278)) Presto/2.12.388 Version/12.12";
const char S15905[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5230)) Presto/2.12.388 Version/12.12";
const char S15906[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03658)) Presto/2.12.388 Version/12.12";
const char S15907[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5217)) Presto/2.12.388 Version/12.12";
const char S15908[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5650)) Presto/2.12.388 Version/12.12";
const char S15909[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4110)) Presto/2.12.388 Version/12.12";
const char S15910[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5288)) Presto/2.12.388 Version/12.12";
const char S15911[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5336)) Presto/2.12.388 Version/12.12";
const char S15912[] = "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.12,gzip(gfe) (via docs.google.com/viewer)";
const char S15913[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5325)) Presto/2.12.388 Version/12.12";
const char S15914[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5299)) Presto/2.12.388 Version/12.12";
const char S15915[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.12";
const char S15916[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5300)) Presto/2.12.388 Version/12.12";
const char S15917[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.12";
const char S15918[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03791)) Presto/2.12.388 Version/12.12";
const char S15919[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.12";
const char S15920[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.12";
const char S15921[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.12";
const char S15922[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.12";
const char S15923[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5734)) Presto/2.12.388 Version/12.12";
const char S15924[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5988)) Presto/2.12.388 Version/12.12";
const char S15925[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.12";
const char S15926[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.12";
const char S15927[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6035)) Presto/2.12.388 Version/12.12";
const char S15928[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.12";
const char S15929[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6046)) Presto/2.12.388 Version/12.12";
const char S15930[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.12";
const char S15931[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.12";
const char S15932[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6032)) Presto/2.12.388 Version/12.12";
const char S15933[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.12";
const char S15934[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.12";
const char S15935[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.12";
const char S15936[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6044)) Presto/2.12.388 Version/12.12";
const char S15937[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.12";
const char S15938[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03780)) Presto/2.12.388 Version/12.12";
const char S15939[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.12";
const char S15940[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.12";
const char S15941[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.12,gzip(gfe) (via docs.google.com/viewer)";
const char S15942[] = "Opera/9.80 (Windows NT 6.1; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.12";
const char S15943[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.12";
const char S15944[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03638)) Presto/2.12.388 Version/12.12";
const char S15945[] = "Opera/9.80 (Windows NT 6.1; MRA 5.6 (build 03402)) Presto/2.12.388 Version/12.12";
const char S15946[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6638)) Presto/2.12.388 Version/12.12";
const char S15947[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.12";
const char S15948[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6261)) Presto/2.12.388 Version/12.12";
const char S15949[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6628)) Presto/2.12.388 Version/12.12";
const char S15950[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.12";
const char S15951[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6573)) Presto/2.12.388 Version/12.12";
const char S15952[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6673)) Presto/2.12.388 Version/12.12";
const char S15953[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6337)) Presto/2.12.388 Version/12.12";
const char S15954[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A551 Safari/8536.25";
const char S15955[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A525 Safari/8536.25";
const char S15956[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A523 Safari/8536.25";
const char S15957[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A405 Safari/8536.25";
const char S15958[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A403 Safari/8536.25";
const char S15959[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; Synergy Rom Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15960[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Broodcomb Build/4.5.1_57_DX5-26) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15961[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; FIH-FB0 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15962[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Inspire4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15963[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; zh-cn; m9 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15964[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Gummy Charged GBE 2.0 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15965[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HUMBLE by Danalo1979 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15966[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Generation X Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15967[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; CSL-MI410 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15968[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SX-SP700A Build/SKYPAD_Alpha) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15969[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SHIFTAO5P Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15970[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; VEGAn-TAB Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15971[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; BLACKSOURC3 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15972[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; AIRPAD Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15973[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; X7G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15974[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; sv-se; ARGM900HCA Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15975[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HS-ET919 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15976[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; HTC-Z510d/1.15.1401.1) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15977[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; SO1 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S15978[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; B5 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15979[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; KPT A9 Build/GRJ22)";
const char S15980[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 7005 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15981[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.1; C905 Build/GINGERBREAD)";
const char S15982[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SANSUI ETAB 700 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15983[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; ARGM900HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15984[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; GIONEE GN380 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15985[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; SK-S150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15986[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; AIR-910 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15987[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; PlayTabPro-GZ Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15988[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; Fly_IQ260; Build/ALPS.GB.FDD2.MP.V2.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15989[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; iw-il; ALODM722HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15990[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; hd-us; BlueStacks Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15991[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Gummy Charged GBE 2.1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15992[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; md-us; VEGAn-TAB Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15993[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; EM73_SIS_GC2M_V2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15994[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; MID1106 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15995[] = "Mozilla/5.0 (Linux; U; Android 2.3; ru-ru; MID7025 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15996[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; de-de; Cat StarGate Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15997[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; th-th; WellcoM-A99 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15998[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; Cat StarGate Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S15999[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; pt-br; SEBM912HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16000[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; I10-50 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16001[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; Wxt Tab PC Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16002[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; LUBM912HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16003[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; ko-kr; YRHG803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16004[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; I10-80 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16005[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Blade2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16006[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; STM908HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16007[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; USCCADT71US Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16008[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; UTV210 Build/GR144) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16009[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; iw-il; CRYM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16010[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; DGM912HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16011[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; BlackBerry Runtime for Android Apps Build/2.0.0.56)";
const char S16012[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; fr-fr; ST8000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16013[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SHIFTAO5P Build/FRG83D; CM-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16014[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; KATBL10AND Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16015[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Vision Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16016[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; 97FC Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16017[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; ZTE Blade 2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16018[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Cloud9/1.0.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16019[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; SLIDEPAD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16020[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Sensation 4g Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16021[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; tr-tr; NAM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16022[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Bionix 2 - A TeamWhiskey Production Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16023[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Cydle M7 (v0004.06.03.02.en) Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16024[] = "Mozilla/5.0 (Linux; U; Android 2.3; pt-br; SWT_GT-7220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16025[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; LUBM908HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16026[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; PC677 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16027[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; CHARM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16028[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; md-us; m799ca Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16029[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; t9199p Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16030[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Gruesomes Rom Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16031[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; in-id; PTM722HR Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16032[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-us; GASEM722HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16033[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Novo7Advanced Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16034[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; de-de; ODYS-Xpress Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16035[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; KESAM726HN Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16036[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; M70014 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16037[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; CAT NOVA Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16038[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; STM912HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16039[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; One Pad Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16040[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; imito am801 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16041[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; iw-il; CRYM908HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16042[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; M778 Tablet PC Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16043[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Precedent Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16044[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; QW TB-1010 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16045[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; LUBM900HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16046[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; CreativESense Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16047[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; B63M Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16048[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; de-de; INM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16049[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; MAPLM912HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16050[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; I10 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16051[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-L95G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S16052[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; KESAM726HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16053[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SP-80 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16054[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; sr-rs; ZTE roamer Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16055[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; A20 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16056[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; T-112 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16057[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Evo Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16058[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Incredible 2 HD Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16059[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; ZTE-Tureis Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16060[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; eTouch 702et Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16061[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; M865 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16062[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HERO Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16063[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; el-gr; PLAIM900HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16064[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo P70/S100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16065[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HUMBLE 4.5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16066[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; in-id; TV pad K3410 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16067[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; it-it; DATAM722HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16068[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; hu-hu; xTAB-70 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16069[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; 7461-CI1100 Build/GRJ22)";
const char S16070[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; el-gr; IQM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16071[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; X1 Build/HCL ME Tablet X1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16072[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; generic_vortex Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16073[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; CIUI Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16074[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fa-ir; G1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16075[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; ZTE V960 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16076[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; 7260 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S16077[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-tw; eye1828 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16078[] = "ZTE-TU960s_TD/1.0 Linux/2.6.35 Android/2.3 Release/9.25.2011 Browser/AppleWebKit533.1";
const char S16079[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; B63M Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16080[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; I700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16081[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; M865 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16082[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; MS-N7Y1 Build/MS-N7Y1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16083[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Mi701 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16084[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; MID73_SIS_GC2M Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16085[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Full Android on A91 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16086[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; COWON D3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16087[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; pl-pl; Android for YG A700 Board Build/A700) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16088[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; iw-il; CRYM912HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16089[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; S42HW Build/HuaweiS42HW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16090[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; taurus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16091[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; iw-il; MOM726HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16092[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; MS-N0Y1 Build/MS-N0Y1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16093[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; B7 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16094[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; zh-cn; Newpad P9 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S16095[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; EIS01PT Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16096[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; COBY KYROS 7024 Build/GR144) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16097[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6330VWBuild/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16098[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PGX Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16099[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; mfld_pr3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16100[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Bulletproof_Doubleshot Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16101[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Gingerbread on Dream Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16102[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; blitzrs8 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16103[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; N10020 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16104[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; A710 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16105[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Precedent Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16106[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; Mega Taicho Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16107[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Hannspadd Build/GRK39F; chrisd1a1 gingerpadd) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16108[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; GASEM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16109[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; KUTBL07ANDC Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16110[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; I-TAB Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S16111[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-in; MMI-CP11 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16112[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; A9100D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16113[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; MTS-SP150 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16114[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; id-id; SpeedUp S1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16115[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; STM908HDZ Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16116[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Hello Tab Build/0.0.1747.0094) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16117[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; in-id; NX-A899 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16118[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; MITO 898 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16119[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; BLUM732HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16120[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15 Build/GRH55; kuyaDROID) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16121[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; I04S-3G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16122[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LDK-ICK v1.4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16123[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MSM Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16124[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; edifi (v0005.07.03.02.en) Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16125[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; A20 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16126[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Acquire Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16127[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; COWON Z2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16128[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; Fly_IQ285 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16129[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; HIGHSCREEN Jet Duo Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16130[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; SWTGT-7250 Build/gingerbread_house_r62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16131[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MikTouched 4G Slide Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16132[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; HTC Inspire 3D Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16133[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; X10S Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16134[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; ja-jp; MVPM726HN Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16135[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; android Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16136[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; GASEM900HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16137[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; HW-N80W Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16138[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; el-gr; MLS IQTALK Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16139[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; MC701A Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16140[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ADR555 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16141[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.2 Build/GINGERBREAD)";
const char S16142[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; UTV210 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16143[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; Zpad Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16144[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; EM73_SIS_HY30_V2 Build/GRI40)";
const char S16145[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; pl-pl; 701 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16146[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; PC687 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16147[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; in-id; Axioo-VIGO410 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16148[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Vision Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16149[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Shakira Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16150[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn;Lenovo A1-32AB0 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S16151[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; a7272_sphone Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16152[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HUMBLE by Danalo1979 4G Build/2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16153[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; TX72 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16154[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; MP727 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16155[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; UP-8786 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16156[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MTAB701 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16157[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Gin-Tazz Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16158[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Samsung EPIC SPH-700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16159[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; CLIQ XT Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16160[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; M703C Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16161[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME811 Build/SHDWR_X7_2.67.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16162[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; A01HW Build/HuaweiS7-901w) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16163[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; M778 Tablet PC Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16164[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; Novo7Advanced Build/GRJ22)";
const char S16165[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; 7D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16166[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Gingerbread on Sapphire Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16167[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; G Harmony v2.7.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16168[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; 97U Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16169[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; I04 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16170[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; tr-tr; Polypad C7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16171[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; android Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16172[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn;Lenovo S2-38AH0 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S16173[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Htcclay's SuperLite 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16174[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; X2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16175[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Cruz L47 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16176[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; H5000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16177[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; msm7627_surf Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16178[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; md-us; disgo7000 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16179[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; nl-nl; 97CU Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16180[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; BB EH8 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S16181[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; th-th; Gpad Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16182[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; SH8288U Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16183[] = "Mozilla/5.0 (Linux; U; Android 2.3; pl-pl; TAB S73 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16184[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; el-gr; Passepartout 70 C124 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16185[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; ru-ru; DNSM912HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16186[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; Geeksphone ZERO Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16187[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Z510d) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16188[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; tr-tr; EFTAM722HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16189[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Octa Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16190[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; OnePAD 720 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16191[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; T1301 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S16192[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; ETOOS 700C Build/R001(V0.17-JT-V961-113.GINGERBREAD.eng.chenxiang.X)) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16193[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; CA017AAC0G/ABC0G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16194[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; zh-cn; Newpad Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S16195[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME811 Build/SHDWR_X7_2.67.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S16196[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; pl-pl; GOCLM908HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16197[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; iw-il; CRYM732HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16198[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; I100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16199[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; openAOS Build of Android 2.3 (cyanogenmod edition) Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16200[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; zh-tw; POM726HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16201[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; m799ca Build/GRI40)";
const char S16202[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; SH8298U Build/S8041) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16203[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; msm7627a_sku3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16204[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; H7-3G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16205[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; CAT NOVA Build/GRI40)";
const char S16206[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; zh-cn; Newpad Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S16207[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; KATBL07AND Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16208[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; A10-MID Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16209[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; FT701 Build/FT701-005) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16210[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; EBEST W50L Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16211[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; cs-cz; GOCLM722HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16212[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; V8 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16213[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; A710 Build/GRJ22)";
const char S16214[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.1; Full Android on SMDKV210 Build/GINGERBREAD)";
const char S16215[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; QB7211 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16216[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; FX2-PAD10 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16217[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; GIONEE GN106 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16218[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; SWT_GT_7205 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16219[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; nuoyaxin Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16220[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; NOVO7 Advanced Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16221[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GOOAPPLE V5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16222[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; GM800TW Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16223[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; A301C Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16224[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; 702et Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16225[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ChangHong V7 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16226[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; imx51_bbg Build/TouchSquid) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16227[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; minitab Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16228[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; world Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16229[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HOSIN_U3 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16230[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; 898 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16231[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; GIONEE GN205 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16232[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; IQ240 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16233[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; TCL_A966 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16234[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Bifer A10 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16235[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; Infinity Build/V0.19-QM.CPT-V961.114-GB-X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16236[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; ELIYA i5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16237[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; Woxter Tablet PC 75 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16238[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; FIH-FB0 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16239[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; EBEST W50L Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S16240[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; 7461-CI1100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16241[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; id-id; NX A7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16242[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; A10t(5DM3) Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16243[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; edifi (v0004.07.03.02.en) Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16244[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; VITA_V3S Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16245[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; A66 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16246[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; 97FC Build/GRJ22)";
const char S16247[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; V13 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16248[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; tr-tr; ATIM908HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16249[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HUAWEI T8828 Build/HuaweiT8828) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16250[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo Lenovo A1-32AB0 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S16251[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; zh-cn; PAD Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16252[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; MP705C Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16253[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; a7272_sphone Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16254[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC-Z510d/1.16.1401.3) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16255[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; A106 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16256[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; KPT A9 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16257[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; zh-cn; Newpad Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16258[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; Changjiang P5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16259[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; A05 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16260[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Q9 Build/Q9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16261[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; NEC-101T Build/X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16262[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; GIONEE GN109 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16263[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; Q07CL01 Build/v00.12.04_Q07_CRA_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16264[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; SWTGT-7250 Build/gingerbread_house_r58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16265[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; tetc Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16266[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC-Z510d/1.13.1401.0 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16267[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; KUTBL08ANDC Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16268[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; H701 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S16269[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; M019 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16270[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; IS701R Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16271[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; de-de; INM803HCZ Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16272[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LD-ICS v1.2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16273[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; ru-ru; MVM900HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16274[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; in-id; NX-A895 Build/ALPS.GB.FDD2.MP.V2.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16275[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ThL V7 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16276[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; mobiletek13_gb Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16277[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; H701 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16278[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Z3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16279[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; SF68 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16280[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; tr-tr; EFTAM722HR Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16281[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; DOOV_D3 Build/DoovDOOV_D3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16282[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; ru-ru; MVM908HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16283[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; LT8029 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16284[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; Kyivstar Spark Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16285[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; B63 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16286[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DoubleShot Lite Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16287[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; CM008 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16288[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; H7-3G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16289[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A610 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16290[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; in-id; PTM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16291[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ar-eg; IZOOM-I8002 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16292[] = "Mozilla/5.0 (Linux; U; Android 2.3; zh-cn; ARMM3V Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16293[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; DOOV_D3 Build/DoovDOOV_D3) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S16294[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; Multilaser Diamond Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16295[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; zh-cn; M16 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16296[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT9326 Build/W326) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16297[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; DOOV_D770 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S16298[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; I9350 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16299[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-AS840 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16300[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GN205 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16301[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; H700 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16302[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; A006 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16303[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Mi400 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16304[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; U9 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16305[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; XFactor Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16306[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; FWS610_EU Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16307[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; tr-tr; M778 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16308[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ko-kr; ITP-S110WD Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16309[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; W1350 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16310[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; G7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16311[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; H700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S16312[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; msm7627_sku5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16313[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Full Android on x86 Emulator Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16314[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; th-th; 697 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16315[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; I04S Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16316[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; Mtag 352 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16317[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; C215 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16318[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ETON T800 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16319[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Philips W635 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16320[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Z3Z6Z7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16321[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; SWTGT-7250 Build/gingerbread_house_r75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16322[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ro-ro; P3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16323[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; iXperiA 8s Build/iXperiA 8s v1.7; iXperiA 8s v1.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16324[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; T-111 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16325[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; l97D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16326[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; A301C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16327[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; Wxt Tab PC Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16328[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; FJJB091 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16329[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Z3A Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16330[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; ja-jp; MVPM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16331[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; H16 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16332[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; MS-N7Y1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16333[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; tr-tr; E-TAB Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16334[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; SUNVEK Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16335[] = "Mozilla/5.0 (Linux; U; Android 2.3; es-es; CasiaTAB8 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16336[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-tw; APD-B705CC Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16337[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; 18FR Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16338[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; PAPYRE pad 712 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16339[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; M88 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16340[] = "Mozilla/5.0 (Linux; U; Android 2.3; zh-tw; moii E598 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16341[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Full Android on AD1000A Build/00.06) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16342[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; S820 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16343[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; FT701 Build/FT701-004) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16344[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE-N960/N960V1.0.0B03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16345[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; Axioo-VIGO410 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16346[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo A1-32AB0 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S16347[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; D23 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16348[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; M701 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16349[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; 4GS Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16350[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; TAB10-06 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16351[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; AS3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16352[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; OBEE-OS805 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16353[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; de-at; SLPad PA810G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S16354[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PAD7 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16355[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; ru-ru; BLADM722HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16356[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MORAL T16 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16357[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A621 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16358[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; A70HT3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16359[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; G20 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16360[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Philips_PI5000 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16361[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; CushZero Build/FRG83G; zero.sholes.X.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16362[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; T1301 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16363[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ChangHongW3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16364[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; W808 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16365[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; TAB-7C-4GW-1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16366[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ar-eg; izoom-i8001 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16367[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Phone 4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16368[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; M730W Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16369[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; ru-ru; MVM900HWZ Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16370[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; 7260 Build/GRJ90)";
const char S16371[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; wimo wf Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16372[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TCL A986 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16373[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; MW0821_V2.0 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16374[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LT-722/M72T2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16375[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; TAB420 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16376[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; HB-100 DASAN Networks, Inc. Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16377[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-ch; Orange Dublin Build/GB_P752V_OCHV1.0.13B04) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16378[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; MT791 Build/GINGERBREAD.b04MT1ref.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16379[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hd-us; Andi4d Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16380[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; C110 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16381[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; GIONEE GN100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16382[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; A8T Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16383[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8022 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16384[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Precedent Build/AreaRom Test) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16385[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; IMO X2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16386[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; W5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16387[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; ATLAS W Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16388[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; 56021 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16389[] = "Discovery Linux/2.6.35 Android/2.3.6 Release/07.30.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1";
const char S16390[] = "HUAWEI_T8828_TD/1.0 OPhone/2.6 (Linux; Android 2.3.7) Release/03.31.2012 Browser/WAP2.0 (AppleWebKit/533.1)Mobile Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S16391[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; R5830i Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16392[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; el-gr; IQTALK Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16393[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HOSIN_U2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16394[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Skate4.3 H Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16395[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; AQ_4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16396[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; EverSmart Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16397[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; de-de; 970 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16398[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; USMART SL55 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16399[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LT170 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16400[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ChangHong-Z-ME Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16401[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Sc0r3d Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16402[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DRD-I Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16403[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Zigo Eon 53 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16404[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MSI_Enjoy_10_Plus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16405[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; G Harmony v2.8 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16406[] = "Pandora/1.5.18 Android/2.3.4 inc";
const char S16407[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; i50 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16408[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; H500 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16409[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; m7800h Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16410[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Changhong H5018 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16411[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; DOOV_D2 Build/DOOV) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16412[] = "MOT-MT680_TD/1.0 Android/2.3 (Linux; Android 2.3) Release/04.13.2012 Browser/WAP 2.0 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16413[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; gaophone13_gb Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16414[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; tr-tr; Parrot Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16415[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; hu-hu; Atlas W Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16416[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; V881 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16417[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; magcomm73_gb Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16418[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; nine i7400 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16419[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-pt; GP-351 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16420[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; MW0821_V2.0 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16421[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; IMO S78 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16422[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; ATB-1108 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16423[] = "Pandora/1.5.17 Android/2.3.4 inc";
const char S16424[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Spice Mi 415 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16425[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; MS-N0Y1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16426[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; de-de; techTAB Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16427[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; EBEST W58 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16428[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; N60 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16429[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; TAB360 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16430[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GN106 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16431[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Z510d Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16432[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; M910A Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16433[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; W2400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16434[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; NB 2012 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16435[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; Athena X2 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16436[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A621W Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16437[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Android_904 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16438[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; S1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16439[] = "Mozilla/5.0 (Linux; U; Android 2.3.9; en-us; HAPPLE Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16440[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GS1 Build/MocorDroid2.3.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16441[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A651 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16442[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; U880 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16443[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Walton_Primo Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16444[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; TCL_A919 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16445[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; TCL_W939 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16446[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; YL-Coolpad_9900/2.3.018.P1.X.9900; 480*960; CTC/2.0) CoolpadWebkit/533.1";
const char S16447[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MT680 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16448[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; A109 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16449[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Changjiang Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16450[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; V880 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16451[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; N505 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16452[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; 510_v2.3.8 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16453[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; GPLUS-Z500 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16454[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ru-ru; MTC-962 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 SVN/01004";
const char S16455[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; B700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16456[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HEDY_H701 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16457[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; iXperiA 8s Build/iXperiA 8s; iXperiA 8S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16458[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Xfactor_mini Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16459[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; in-id; A10GBM73 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16460[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; JT Smart Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16461[] = "HUAWEI-HUAWEI-Y-220T/1.0 Linux/2.6.35.7 Android/2.3.5 Release/11.13.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S16462[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GOOAPPLE V5-3G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16463[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; ru-ru; Q3M732HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16464[] = "Mozilla/5.0 (Linux; U; Android 2.3; zh-cn; HTCA6390_TD/1.0 Android/2.3 release/4.1.2011 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16465[] = "Mozilla/5.0 (Linux; U; Android 2.3.9; it-it; B63M Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16466[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ALCATEL_one_touch_979 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16467[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; BTPC-2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16468[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; dkb Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16469[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; DOOV_D3 Build/DoovDOOV_D3) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16470[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; USmart_SL88 Build/USmart_SL88USmart_SL88) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16471[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; signal75_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16472[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SS706 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16473[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A66   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16474[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; KS306 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16475[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; B50 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16476[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; B601 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16477[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-za; MOBICEL TITAN Build/MocorDroid2.3.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16478[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; X40 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16479[] = "WINGTECH-INCO/1.0 Linux/2.6.35.7 Android/2.3.5 Release/11.03.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S16480[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; U670C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16481[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; Qumo DotCom Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16482[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; GIONEE GN210 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16483[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo Lenovo S2-38AH0 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S16484[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; BA014AAS0G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16485[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I339 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16486[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; Uno Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16487[] = "ZTEU788_TD/1.0 Linux/2.6.35 Android/2.3 Release/12.25.2011 Browser/AppleWebKit533.1";
const char S16488[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; COWON Q7 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16489[] = "V6500_ChinaUnicom/1.0 Linux/2.6.38.6(Android 2.3.6;) Release/08.06.2012 Browser/AppleWebKit533.1 Mobile Safari/533.1";
const char S16490[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; I9400NM Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16491[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; I9400 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16492[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; AL-6 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16493[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ZEN U1 Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16494[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; benrui15_6626_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16495[] = "Mozilla/5.0 (Linux; U; Android 2.3.9; en-us; G-TiDE Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16496[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; BASE_Varia Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16497[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; X35 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16498[] = "sprd-yuanpeng/1.0 Linux/2.6.35.7 Android/2.3.5 Release/11.09.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S16499[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; Polytron W1320 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16500[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; ADR21 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16501[] = "Mozilla/5.0 (Linux; U; Android 2.3; in-id; froyo_f698u202a Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16502[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; ru-ru; MVM908HWZ Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16503[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Z1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16504[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8180 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16505[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; imx53_smd Build/R10.3.3-r1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16506[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; V850 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16507[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Z3_Z6_Z7 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16508[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; KPT A88 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16509[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MORAL_N01 Build/HuaqinMORAL_N01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16510[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; YL-Coolpad_5880/2.3.011.X.5880; 800*480; CTC/2.0) CoolpadWebkit/533.1";
const char S16511[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; Lanix IliumS50Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LNX-Android-MMS-V1.1/1.2";
const char S16512[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; XD-S6000 Build/msm7627a_sku1-eng2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16513[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; MegaFon SP-A5 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16514[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; AX-340 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16515[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ICE MOBILE Galaxy Prime Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16516[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; Mtag 281 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16517[] = "KONKA_V926_TD/1.0 Android/2.3.5 MocorDroid/SpreadTrum Release/3.22.2012 Browser/AppleWebKit5333.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S16518[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; sk-sk; Orange Dublin Build/GB_P752V_OSKV1.0.13B05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16519[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; AND1  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16520[] = "CoolPad8060_CMCC_TD/1.0 Linux/2.6.35 Android/2.3 Release/8.30.2012 Browser/AppleWebkit533.1";
const char S16521[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; i5830 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16522[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; MOBO BT Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16523[] = "Mozilla/5.0(Linux;U;Android 2.3.6;zh-tw;E650 Build/GRK39F)AppleWebkit/533.1(KHTML,like Gecko)Version/4.0 Mobile Safari/533.1";
const char S16524[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GN205 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16525[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; FireBird Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16526[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; imx51_bbg Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16527[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; A66 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16528[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; mx1002 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16529[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MAC B200/V400) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16530[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GN106 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16531[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT536 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16532[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; id-id; Q7079W Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16533[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LT S800 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16534[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; PULID F7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16535[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 5010 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16536[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HUAWEI T8828 Build/HuaweiT8828) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16537[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; pl-pl; MID02 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16538[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U8667 Build/HuaweiU8667) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16539[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; U6 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16540[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8022 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16541[] = "Mozilla/5.0 (Linux; U; Android Android 2.3 GingerBread; it-it; Android for AVANTIS WEBBY5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16542[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; MID8128 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16543[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; pt-br; TAB 800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16544[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Android Smart Phone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16545[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; G_7C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16546[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-mx; Andy 3G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16547[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE U885 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16548[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; DOOV_D600 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16549[] = "ZTEU885_TD/1.0 Linux/2.6.35 Android/2.3 Release/2.25.2012 Browser/AppleWebKit533.1";
const char S16550[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; verykool S757 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16551[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; epade A360S Build/alpsepade) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16552[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; tr-tr; U8816 Build/HuaweiU8816) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16553[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; Movistar Express Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16554[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; ALCATEL ONE TOUCH 928D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16555[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ms-my; MI360 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16556[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; iBall 3G 7325 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16557[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; C208 Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16558[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; GT7250S Build/gingerbread_qrd_r209) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16559[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; sk-sk; Orange Dublin Build/GB_P752V_OSKV1.0.13B06) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16560[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; n903e Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16561[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A702_Andy Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16562[] = "HUAWEI-HUAWEI-Y-220T/1.0 Linux/2.6.35.7 Android/2.3.5 Release/11.01.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S16563[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; P1 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16564[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; X501_USA_RS Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16565[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; AND1   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16566[] = "HUAWEI-HUAWEI-Y-220T/1.0 Linux/2.6.35.7 Android/2.3.5 Release/12.15.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S16567[] = "HUAWEI-HUAWEI-Y-220T/1.0 Linux/2.6.35.7 Android/2.3.5 Release/11.28.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S16568[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; AQ 4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16569[] = "null Linux/2.6.35 Android/4.1.2 Release/null Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1 Android 2.3.5;";
const char S16570[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; C3100 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16571[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; G3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16572[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; msm7627a_i300 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16573[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16574[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Zedd Clan Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16575[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; Bliss A70 Build/LCAQF20.1.0_W1230) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16576[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; VINCI Tab II VS-3001 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16577[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; DOOV_D770 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16578[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; GIONEE GN777 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16579[] = "CoolPad8026_CMCC_TD/1.0 Linux/2.6.32 Android/2.3 Release/03.15.2012 Browser/WAP2.0 (AppleWebKit/533.1) Mobile Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S16580[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A7013 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16581[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_5110/2.3.007.X.5110; 320*480; CTC/2.0) CoolpadWebkit/533.1";
const char S16582[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HT-I617 Build/MocorDroid2.3.5) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16583[] = "Lephone_TD8208_TD/1.0 Android/2.3.5 MocorDroid/SpreadTrum Release/3.22.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S16584[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; RPSPG3501 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16585[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PLUS Pro Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16586[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; A20 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16587[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; SPC Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16588[] = "sprd-B51/1.0 Linux/2.6.35.7 Android/2.3.6 Release/12.29.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S16589[] = "HUAWEI-HUAWEI-Y-220T/1.0 Linux/2.6.35.7 Android/2.3.5 Release/12.25.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S16590[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; LINGWIN U820S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16591[] = "MOT-XT536/1.0 Android/2.3.7 Release/2.28.2012 Browser/AppleWebKit533.1 Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MOT-XT536 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16592[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; W808 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16593[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Mi-A303 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16594[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; ru-ru; AP-801 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S16595[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HC-HE-N620E/HE-N620E-S003-CT; 320*480; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16596[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GN777 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16597[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; IQ236 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16598[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Bambook S1 Build/EA1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16599[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; cs-cz; TA7200 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S16600[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Oplus A16 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16601[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; E5680 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16602[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Epade S3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16603[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; A7500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16604[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE N790 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16605[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-RU; PAP4000 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16606[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A326/S100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16607[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Z9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16608[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; B710 Build/2.0.1083_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16609[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DTC Build/UNKNOWN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16610[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; Vega x Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16611[] = "sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/2.3.5 Release/11.09.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S16612[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; YPY_S350 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16613[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; EverStar Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16614[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; TCL-D662/EG27_FLID_V0.8; 480*800; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16615[] = "sprd-Byond-B54/1.0 Linux/2.6.35.7 Android/2.3.6 Release/01.07.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S16616[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; Slim Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16617[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; 31TL04 Build/01.03.000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16618[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPARK Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16619[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-US;) AppleWebKit/999+ (KHTML, like Gecko) Safari/999.9";
const char S16620[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; W6800 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16621[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SunPhone-G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16622[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-gb; S16 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16623[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; KIRA N10021 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16624[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo S2-38AH0 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16625[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DB50 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16626[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DIGICEL Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16627[] = "T-smart D08_TD/1.0 Android/2.3 Release/3.20.2011 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S16628[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE-T U812 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16629[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; TCL A919 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16630[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SunTab-G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16631[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; MBX DVBT reference board (c03ref) Build/3010B.X.V28) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16632[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; M900HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16633[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; G5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16634[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo A1-32AB0 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16635[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; KSP8000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16636[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; W58SX Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16637[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; EBEST V6 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16638[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Mytel Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16639[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; RSD-V1 Build/alpsRSD-V1) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16640[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; V880 Build/GWK74; MoKeeOS-v0.3.Xalpha) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16641[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ar-eg; imden A9106 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16642[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; TCLW989 Build/REL_2.3.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16643[] = "KakaoTalkAndroid/3.3.3 Android/2.3.4";
const char S16644[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; th-th; GT0001ALTH Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16645[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; CHL-S3000/Hualu_S3000_V1.0; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16646[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; JC-S9220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16647[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; T9987 Build/MocorDroid2.3.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16648[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8020 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16649[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; Etab 0208 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16650[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; ZTE V881 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16651[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; epade A360S Build/alpsepade) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16652[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HT-I617 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16653[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MID-Q5 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16654[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ar-eg; UR-A38 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16655[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; i66 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16656[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; gb92236a Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16657[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; C7500 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16658[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; imx51_bbg Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16659[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPSSON iMO1000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16660[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ru-ru; SP-A20i Build/MF_GB_01.02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16661[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; YT521 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16662[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; Helsinki Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16663[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; LUX Build/LUX01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16664[] = "MASTONE G3_TD/1.0 Android/2.3 Release/4.20.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0";
const char S16665[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; RPSPB4001 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16666[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; W381 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16667[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad 5216 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16668[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; nl-nl; VALENCIA Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16669[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-tw; eye1828 Build/R10.3.2_3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16670[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE N790S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16671[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; W05 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16672[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; S7054 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16673[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Ofone Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16674[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; PROA680 Build/V1.0.09) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16675[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; AirTab M70G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16676[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SP100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16677[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; Star TV Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16678[] = "Telego_G902_TD/G902_V2.00 Linux/2.6.35 Android/2.3.5 Release/07.14.2012 Browser/AppleWebKit533.1";
const char S16679[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; X350 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16680[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; KDI-i360   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16681[] = "CoolPad8070_CMCC_TD/1.08 Linux/2.6.35 Android/2.3.5 Release/03.29.2012 Mozilla/5.0 AppleWebKit/533.1 Version/4.0 Mobile Safari/533.1";
const char S16682[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A69 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16683[] = "sprd-B51+/1.0 Linux/2.6.35.7 Android/2.3.6 Release/01.25.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S16684[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; el-gr; iQTalk Crystal Build/V1.0.27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16685[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; SVP Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16686[] = "sprd-yuanpeng/1.0 Linux/2.6.35.7 Android/2.3.5 Release/01.21.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S16687[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SL-S1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16688[] = "HUAWEI-HUAWEI-Y-220T/1.0 Linux/2.6.35.7 Android/2.3.5 Release/01.16.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S16689[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; W105 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16690[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; TR719 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16691[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; sdkDemo Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16692[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; TAB-PROTAB2XL Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16693[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; acsip13_gb Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16694[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; EBEST N4 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16695[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; E one Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16696[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; Ypy 7 - TB07STA Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16697[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; G.fone G1345 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16698[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; I9000 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16699[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ALCATEL_one_touch_979_HelloKitty Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16700[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 (Mobile; afma-sdk-a-v4.1.1)";
const char S16701[] = "Mozilla/5.0 (Linux; U; Android Android 2.3 GingerBread; it-it; Android for AVANTIS WEBBY5 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16702[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; XT760 Build/02.3C.08RPS) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16703[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; E5838 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16704[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; 9000H Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16705[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-EG870 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16706[] = "Mozilla/5.0 (Linux;U;Android 2.3.5;zh-cn;GO M2 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16707[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BA014AAM0G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16708[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; E909B+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16709[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HOSIN_U2 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16710[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Bambook S1 Build/EA1.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16711[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; leepoo i500 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16712[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; K-Touch E619 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16713[] = "ZTEU960s2_TD/1.0 Linux/2.6.35 Android/2.3 Release/3.21.2012 Browser/AppleWebKit533.1";
const char S16714[] = "Mozilla 5.0 (Linux; U; Android 2.3.7; zh-cn; 8022 Build GWK74) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S16715[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; G'HONGT628 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16716[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; moii-E796 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16717[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-XT553 Build/AGSPIN_B_02.01.35RPD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16718[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-XT553 Build/AGSPIN_B_02.01.38RPD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16719[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; AMOI_N89 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16720[] = "ZTE-TU805_TD/1.0 Linux/2.6.35 Android/2.3 Release/10.08.2011 Browser/AppleWebKit533.1";
const char S16721[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; JC-A107 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16722[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; D118 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16723[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BRS-BROR W60/BROR_W60_V1.0; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16724[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SIENDON P1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16725[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; sv-se; Doro PhoneEasy 740 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16726[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; M.T.T. Tablet Build/R10.3.2_3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16727[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Seals TS3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16728[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XLTEL X800 Build/MocorDroid2.3.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16729[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Cydle M7 (v0005.07.03.01.en) Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16730[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Android Multimedia  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16731[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; 4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16732[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; MITO T100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16733[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; NODIS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16734[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; EF68 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16735[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; AX525 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16736[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; U670C Build/GRK39F)";
const char S16737[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SunTab-C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16738[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Lephone_C03 Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16739[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; BERAQ NB-1000 Build/VENUS_00.03.165.H1096) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16740[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LNV-Lenovo_A365e Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16741[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB626 Build/5.5.1Q-172-DEFY2-CCI-6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16742[] = "Haier_HT-I600_TD/I600_MocorDroid2.2_W11.41_V1.00 Release/06.26.2012 Mozilla/5.0 (Linux; U; Android 2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16743[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; 31TL04 Build/01.02.000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16744[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MAGNUS FANTASY G3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16745[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; Dublin Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16746[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; S900 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16747[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; V8110 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16748[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; tr-tr; NEOI 588-734 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16749[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; Gvon 990 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16750[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME811 Build/SHDWR_X7_2.74.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16751[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; X1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16752[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; TCL A968 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16753[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; HTC_IncredibleE_S715e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16754[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-za; ZTE V10 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16755[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SHP-SH330T/V03.03; 320*480; CTC/2.0) eZBrowser/1.00";
const char S16756[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPSSON iMO850 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16757[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; 610 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16758[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Haier HW-N86W Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16759[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ANTAI Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16760[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; N620 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16761[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4)";
const char S16762[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; CS40G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16763[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BW-BenWee 5100S/BenWee_5100S_V1.1; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16764[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Binatone iHomePhone 3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16765[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GIO-GiONEE_C600/CBD75B04_01_T9161 ; 320*480; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16766[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; maxwest Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16767[] = "Mozilla/5.0 (Linux; U; Android 5.5.5(Android 2.3.6); en-us; MC919LL Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16768[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; H943 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16769[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; Athena Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16770[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; tr-tr; MC603CH Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16771[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; N855D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16772[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; YL-Coolpad 5216 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16773[] = "KONKA_V913_TD/1.0 Release/7.13.2012 Mozilla/5.0(Linux;U;Android 2.3.5) AppleWebKit/533.1(KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16774[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; WING W35 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16775[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; M1507 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16776[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; xpndr_imh3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16777[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BW-BenWee 5100S/BenWee_5100S_V1.5; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16778[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; PROA680 Build/V1.0.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16779[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Malata I5 Build/MalataI5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16780[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; S30 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16781[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; SM55 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Versao/4.0 Mobile Safari/533.1";
const char S16782[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 5211 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16783[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; L1-Forst Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16784[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; P8 Build/MocorDroid2.3.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16785[] = "sprd-S7100/1.0 Linux/2.6.35.7 Android/2.3.5 Release/04.01.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S16786[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; I60M Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16787[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; gaoxinqi Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16788[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; B920 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16789[] = "Lenovo_S2-38AT0_TD/3.0 LeOS/3.0 (Linux; Android 2.3) Release/10.01.2011 Browser/WAP 2.0 (AppleWebKit/530.17)";
const char S16790[] = "PhilipsW635/1.0 Linux/2.6 Android/2.3 Browser/AppleWebKit533.1 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S16791[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MT680 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16792[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; C600 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16793[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; TCL_A988 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16794[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; DH U890 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16795[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ar-eg; N76WIraq Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16796[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; BF_A11 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16797[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; GC500SF Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16798[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Callbar A1 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16799[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ar-eg; Lawzi Group SGC101 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16800[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TY-K-Touch_E619+ Build/TBEX_8405_V0101) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16801[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; Hi-Class Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16802[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; OPS-60 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16803[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; P10 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16804[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; el-gr; IQM722HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16805[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; S800_ares4 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16806[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; N62 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16807[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; hexing15_6626_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16808[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ET1 Build/01-X-2272-0100-00-D1-X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16809[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; EBEST N4 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16810[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; AOLE Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16811[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; ZTE N960 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16812[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BLF-lephone 1800/lephone_1800_V1.3; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16813[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; AT-5915 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16814[] = "HUAWEI-HUAWEI-Y-220T/1.0 Linux/2.6.35.7 Android/2.3.5 Release/10.20.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S16815[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; U710 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16816[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HD6 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16817[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; el-gr; iQTalk Crystal Build/V1.0.25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16818[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; F32 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16819[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; W337 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16820[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ZJT-H1 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16821[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; M.T.T. SmartFun Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16822[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; AW 550D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16823[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; 31TL04 Build/01.05.000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16824[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; 91001E Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16825[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; IncredibleS Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16826[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Q7 Build/Q7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16827[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; I800B+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16828[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; HTC-EVO-3G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16829[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BLF-lephone 1800/lephone_1800_V1.1; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16830[] = "Mozilla/5.0 (Linux; U; Android 5.5.3(Android 2.3.6); en-us; MC919LL Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16831[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; BF-A10 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16832[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; Telenor_Smart_Pro Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16833[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; OS323 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16834[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_5230/2.3.004.X.5230; 320*480; CTC/2.0) CoolpadWebkit/533.1";
const char S16835[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; BERAQ NB-1000 Build/VENUS_00.03.151.H1096) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16836[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad 5211/2.3.007.P2.X.5211; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16837[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ICE_MOBILE_APOLLO_TOUCH Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16838[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; E619 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16839[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; Mi230 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16840[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fa-ir; GLX Luster Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16841[] = "ZTEU793_TD/1.0 Linux/2.6.35 Android/2.3.5 Release/01.04.2013 Browser/AppleWebKit533.1";
const char S16842[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; WX06K Build/114.1.0029) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16843[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HUAWEI G7500 Build/HUAWEIG7500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16844[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MALATA Z100A Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16845[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; GIONEE GN170 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16846[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; H5500 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16847[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; A51 Build/A51Gizoom-i8004) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16848[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TW-TONEWIN_S8310 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16849[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE-T U960 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16850[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; MegaFon SIM+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16851[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ru-ru; G23 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16852[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; W9001A Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16853[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; moii-E501 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16854[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; EBEST W70 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16855[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; D500 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16856[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; th-th; NEC-101S Build/X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16857[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ZTE-U V760 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16858[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; NOAIN A901 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16859[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Saihon I91 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16860[] = "Mozilla/5.0 (Linux; U; Android 2.3.9; en-us; FDT G505 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16861[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Snexian Mi320 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16862[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; sr-rs; Telenor Smart Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16863[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Z3M Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16864[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; V880 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16865[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; TCA200 Build/R10.3.3-r1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16866[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ALCATEL one touch D662 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16867[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; i508d Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16868[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; N790 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16869[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; fa-ir; GLX Mobile Build/GLX-2013.03.18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16870[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; Amin Rom Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16871[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; T60 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16872[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; pt-br; MYSM722HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16873[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ru-ru; S700 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16874[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; KIRA N1002x Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16875[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; S710D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16876[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; YUSUN - W 800 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16877[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; 5300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16878[] = "Mozilla/5.0 (Linux; U; Android 2.3;es-co; ZTE-U V760 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16879[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; EngelDroid EN1001 Build/X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16880[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; AndroTabGIN Build/0.0.1758.0257) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16881[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; minitab7 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16882[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; Z909 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16883[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ru-ru; ImPAD1311 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16884[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MORAL_N01 Build/HuaqinMORAL_N01) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S16885[] = "Mozilla/5.0 (Linux; U; Android 2.3 YunOs 1.0.0.3; zh-cn; K-Touch W658 Build/AliyunOs-2012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16886[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ET1 Build/02-X-1371-0601-00-M1-X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16887[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; P4D ORION Build/GB.b12ref.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16888[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; GM800 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16889[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Solar Pad Build/gingerbread_house_r59) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16890[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Hosin_U7 Build/HosinU7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16891[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Z180 Build/MALATAZ180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16892[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HOSIN_U2 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S16893[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Z710e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16894[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; S710D Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16895[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo S2-38AT0 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16896[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; EBEST W66A Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16897[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; HTC-EVO-3G Build/chinhko) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16898[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-us; G75 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S16899[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ZXY-ZTE-C_X500/ZTE-C X500MMB_31_Z4_C_TS9CF313B_04) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16900[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; E810 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16901[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; TCL A968 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S16902[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; G888 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16903[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; innos a35 Build/A35_UMTS_V3_2.3_6190_V005_11b_AFRICA) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16904[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HC-HE-E617 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16905[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MTW20 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16906[] = "MOT-XT536/1.0 Android/2.3.7 Release/2.28.2012 Browser/AppleWebKit533.1 Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT536 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16907[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GIO-GiONEE_C600/CBD75B04_01_T9133 ; 320*480; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16908[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; G9C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16909[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Changhong H5018 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16910[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Android Smart Phone Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S16911[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ThL V7 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16912[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8180 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16913[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME811 Build/SHDWR_X7_2.71.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16914[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; TCL-C995/AT02_CTCN_V2.0; 480*800; CTC/2.0) TAPAS BROWSER/2.0";
const char S16915[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GIO-GiONEE_C500/CBE73A04_02_T9096 ; 320*480; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16916[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; HD7+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16917[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; E5670 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16918[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; W708&W706 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16919[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; T710 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16920[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Haier HW-N86W Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S16921[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; G910+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16922[] = "Mozilla/5.0 (Linux; U; Android 2.3.9; ar-ye; TCL C990 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16923[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; U910 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16924[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TCL W969 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16925[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; M35 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16926[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LeWa_OS Build/v1.3.0; LeWaOS-HappyFrog) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16927[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; K-Touch_W680 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16928[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; CHER A91 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16929[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; DOOV_D770 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16930[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; RSD-V1 Build/alpsRSD-V1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16931[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; AM968 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16932[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; G9C_BSX Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16933[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; MITO 677 DEVICE Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16934[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; K-Touch E619 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16935[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; 5110 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16936[] = "Mozilla/5.0 (Linux; U;AliyunOS 1.5.1.16-RT-X.X; zh-cn; K-Touch E619 Build/AliyunOs-2012;Android 2.3 Compatible) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16937[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Optimus Net Build/Sony HD 2.0; Xperia by AD Bros.) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16938[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; A209 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16939[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; X3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16940[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE U960s2 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16941[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; 7230 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S16942[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; V.LAND EX350 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16943[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GN160T Build/MocorDroid2.3.5) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S16944[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HEDY_S803 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16945[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; MCT666 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16946[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; WPF-W9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16947[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SC160 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16948[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; MPM-i616 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16949[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; iBall Andi4.3J Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16950[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Galaxy Precedent Build/AreaRom Test) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16951[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-by; MTC Viva Build/HuaweiU8816) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16952[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; nb-no; AKS-0085SM Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16953[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; BF_A15 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16954[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; M910A Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S16955[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Android Multimedia Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16956[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; KN69 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16957[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; F999 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16958[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; MOX-PAD752 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16959[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; D888 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16960[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; GSTAR P12 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16961[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Epade S3 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S16962[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OKA13 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16963[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; imx53_smd Build/R10.3.2_3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16964[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; pt-br; TAB 810 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16965[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; 596 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16966[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N960 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16967[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8060 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16968[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; U8667 Build/HuaweiU8667)";
const char S16969[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-ph; ALCATEL_one_touch_978 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16970[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; FLY IQ256 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16971[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SmartWorld Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16972[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Oyster500 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16973[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; CR888 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16974[] = "sprd-B.S-mobile/1.0 Linux/Apps_2.6.32.9-perf Android/2.3.9 Release/11.19.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S16975[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; BF_A8 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16976[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; C918 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16977[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; C9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16978[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ug-; LT S800 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16979[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; C01 Build/MocorDroid2.3.5) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S16980[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; gaophone15_6628_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16981[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; SH-Saihon_I97 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16982[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; id-id; Andriod Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16983[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; X20 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16984[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ICE_MOBILE_APOLLO Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16985[] = "CoolPadCoolpad-8056_TD/1.0 Android/2.3.5(Linux; Android 2.3.5) Release/10.30.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S16986[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; XUNHU S900 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16987[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Colors X17 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16988[] = "Mozilla/5.0 (Linux; U; Android 2.3.9; zh-cn; MTK6515M-MTK6515M-H003/1.0 Android/2.3.9 Release/04.15.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16989[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; BLU Tigo Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16990[] = "Konka E5660S/1.0 Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Konka E5660S Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16991[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; XD-S6000 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16992[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; YEPEN P666 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16993[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GN106 Build/GRJ22)";
const char S16994[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; A811 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16995[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Mi_A200 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16996[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A370/S101) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16997[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; OT-990C Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16998[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ET1 Build/01-X-2602-0100-00-D1-X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S16999[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; WHAM WS35 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17000[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE U793 Build/MocorDroid2.3.5) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S17001[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Leopard MF800 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17002[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; AS123 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17003[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ?1??O-I727 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17004[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; epade Q3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17005[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; S306 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17006[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; MD Smart Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17007[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; CHL-S3500/Hualu_S3500_V1.0; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17008[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; GPLUS-W628 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17009[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT_P5100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17010[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PLUS Stil Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17011[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; E908 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17012[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; TD505 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17013[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; SK-W15 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17014[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; Dmobile A8 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17015[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A105 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17016[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; Ilium_AVVIO_760 Build/a200Ilium_AVVIO_760)";
const char S17017[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A2_Classic Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17018[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; PROA680 Build/V1.0.11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17019[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; GSM25 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17020[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; FIREFLY_MOBILE-FIREFLY_S60/1.0 Android/2.3.7 Release/04.03.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17021[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; sprd-B51+/1.0 Android/2.3.6 Release/04.08.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17022[] = "FLY-FLY-F351/1.0 Linux/2.6.35.7 Android/2.3.5 Release 05.22.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S17023[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; VOTO W5300 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17024[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; yusun W702 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17025[] = "ZTE-Tu960_TD/1.0 Linux/2.6.35 Android/2.3 Release/8.25.2011 Browser/AppleWebKit533.1";
const char S17026[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TETC-G19 Build/alpsTETC_G19) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17027[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8020 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17028[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lovme_A_18 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17029[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; Action Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17030[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; A330 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17031[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; DROID3 Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17032[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Solar Pad Build/gingerbread_house_r66) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17033[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; CHL-S3000/Hualu_S3000_V1.5; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17034[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; GO_Live_S1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17035[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; BERAQ NB-929 Build/VENUS_00.03.165.H1100A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17036[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 7011 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17037[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; POLYTRON S2350 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17038[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; EF68A Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17039[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; ZTE V8110 Build/GRK39F) AppleWebKit/533.1(KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17040[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-nz; innos a35 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17041[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-; Mi240   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17042[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Kricket S1 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17043[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; Ilium_AVVIO_760 Build/a200Ilium_AVVIO_760) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17044[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; LAVORUM Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17045[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; BP306 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17046[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; G-TiDE E60 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17047[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-E912 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17048[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; X15 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17049[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; MeuAN200 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17050[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Mtech DEZIRE Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17051[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; Shadow Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17052[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; B305A Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17053[] = "Konka E5660S/1.0 Mozilla/5.0 (Linux; U; Android 2.3.5; en-nz; T640S Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17054[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; AX520 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17055[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; LUBM908HW Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17056[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pl-pl; NavRoad MOBIO Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17057[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-cn; HTC_S715e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17058[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; M203 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17059[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HYUNDAI H11 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17060[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ouboxin Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17061[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; TCLW989 Build/REL_2.3.7) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S17062[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; LINGWIN U880 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17063[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; fulinpengda Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17064[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; TCL A860 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17065[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; BESTA UNO BSP01 Release/2.24.2011) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17066[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; innos i5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17067[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; N98 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17068[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-XPERIA S Build/MiniXperiaMod; MiniXperiaMod-V5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17069[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; AXF-01B Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17070[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GN160T Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17071[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; BF9200 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17072[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; G_7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17073[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SWTGT-7250 Build/gingerbread_house_r79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17074[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; HD7-PRO Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17075[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; Z2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17076[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; VB808 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17077[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; N619 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17078[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; C9220 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17079[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; RSD_V2 Build/HUAQINRSD_V2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17080[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Ouki_A18 Build/MocorDroid2.3.5) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S17081[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; W6100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17082[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; GiONEE GN108 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17083[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SOP-G928 Build/alpsSOP_G928) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17084[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZJ105 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17085[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; TP703 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17086[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HEDY_H712 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17087[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; CETRIX CS1216 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17088[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; 7230-B Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S17089[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPSSON iMO888 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17090[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; STM753HD Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17091[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; NOAIN A902 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17092[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; IM-A730K Build/Bcola) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17093[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; G5C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17094[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lovme X3 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17095[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; BT Home Smart Phone Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17096[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; V3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17097[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; Niutek-3G-4.0 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17098[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; TCL A909 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17099[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; X402 Build/X402-Android-2.3.6-TecSync-2012/10/10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17100[] = "Mozilla/5.0 (Linux; U; nToBeBox 2.3.4; tr-tr; ITS10BR Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S17101[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; sl-si; B705A Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17102[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BW-BenWee 5100S/BenWee_5100S_V1.2; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17103[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Apollo one Build/MocorDroid2.3.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17104[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; QY-S10 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17105[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BLW-VE600/VE600_D11_V1.00_V1.4_X; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17106[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; Ilium S50 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17107[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZY-FLY_N9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17108[] = "Lephone_TD505_TD/1.0 Android/2.3.5 (Linux; Android 2.3.5) Release/8.9.2012 Mozilla/5.0 (Linux; U; Android 2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17109[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; B51+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17110[] = "Konka E5660S/1.0 Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; ZA-966 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17111[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; el-gr; iQTalk Crystal Build/V1.0.31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17112[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; Newman A17  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17113[] = "HS-T912_TD/1.0 Android/2.3 Release/1.31.2013 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S17114[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; el-gr; iQTalk Crystal Build/V1.0.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17115[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DTC GT7 Build/GT822) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17116[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; M2 Elegante Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17117[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; Avvio 701 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17118[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; th-th; A301 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17119[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; M39 Sawy Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17120[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-AS-45/57/1.0 Android/2.3.5 Release/05.08.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17121[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; Z10 Build/10.1.0.103)";
const char S17122[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; FIREFLY_MOBILE-FIREFLY_A530/1.0 Android/2.3.7 Release/04.26.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17123[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-E820 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17124[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A703_Andy Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17125[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-nz; innos a35plus Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17126[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-bo; Bmobile_AX525 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17127[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; BY2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17128[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; 12018 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17129[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; Essentielb-Pixis Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17130[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-Android-351/1.0 Android/2.3.5 Release/04.24.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17131[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-Android-401/1.0 Android/2.3.5 Release/04.18.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17132[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ar-eg; msm7627a_sku1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17133[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SL-S10 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17134[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; FIREFLY_MOBILE-FIREFLY_S70/1.0 Android/2.3.7 Release/05.06.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17135[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Masstel M05 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17136[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; AS126 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17137[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; nl-nl; SMP35-100 Build/3.0.1398_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17138[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; C-1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17139[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; Lanix Ilium_S105 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LNX-Android-MMS-V1.1/1.2";
const char S17140[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; V1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17141[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LIYANG Z9 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17142[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Q1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17143[] = "HUAWEI-HUAWEI-Y-220T/1.0 Linux/2.6.35.7 Android/2.3.5 Release/03.20.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S17144[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-pt; M8402 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17145[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; N8303 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17146[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Magic-35 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17147[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-pt; M8403 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17148[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; iNote mini Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17149[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A3* Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17150[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 5880 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17151[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ar-eg; izoom-i8001 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17152[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; BRS-BROR_S9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17153[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ChangHongV7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17154[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; YOORD D599 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17155[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; SOP-G998 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17156[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; U10 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17157[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; DK P170 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17158[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; DK P120 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17159[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; NEC909e Build/X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17160[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; C868 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17161[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; Bliss Pad Q7011 Build/1.0.540_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17162[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; E9005 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17163[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; KPT A88 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S17164[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; COOCO_A6 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17165[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SHT-H539 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17166[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TETC G16 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17167[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TAIXING Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17168[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; I3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17169[] = "KONKA V920_TD/1.0 Android/2.3 Release/2.2.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S17170[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; T9108 Build/MocorDroid2.3.5) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S17171[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ET1 Build/03-X-1852-0602-00-M1-X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17172[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; zh-ch; zh-tw; Aigo D1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17173[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; cs-cz; Kis Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17174[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Karbonn_A5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17175[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; ZTE V960 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17176[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad8050 Build/MocorDroid2.3.5) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S17177[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; C3668 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17178[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BW-BenWee 5100S/BenWee_5100S_V1.3; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17179[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; KPT A55 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17180[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; U16 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17181[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; JDD-F99 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17182[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YCT-TE690/ctyon_TE690-1_V1.6; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17183[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Opal 800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17184[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZY-FLY_S1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17185[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; FORTUNE Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17186[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ginwave15_6626_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17187[] = "ZTEU887_TD/1.0 Linux/2.6.35.7 Android/2.3.5 Release/01.01.2012 Browser/AppleWebKit533.1 Mozilla/5.0 Mobile";
const char S17188[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; FDT E18 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17189[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MT6517-WINDS-NOTE-W/1.0 Android/2.3.5 Release/04.16.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17190[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; XUNHU S990 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17191[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; AS135 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17192[] = "Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE U793 Build MocorDroid2.3.5) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S17193[] = "Mozilla/5.0 (Linux; U; Android 2.3.9; zh-cn; MTK6515M-MTK6515M-H003/1.0 Android/2.3.9 Release/05.16.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17194[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; sprd-B51+/1.0 Android/2.3.6 Release/05.29.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17195[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-yuanpeng/1.0 Android/2.3.5 Release/06.13.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17196[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; cs-cz; ALIGATOR S4000Duo Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17197[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Trinity-T3 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17198[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-WINDS-NOTE/1.0 Android/2.3.5 Release/07.13.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17199[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; AMAZE S250 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17200[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; SPC S1 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17201[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-UltraPhone-40/1.0 Android/2.3.5 Release/07.31.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17202[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; TLINK351 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17203[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ru-ru; HTC_Incredible_A6300 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17204[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; A208 Build/UNKNOWN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17205[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TCL A919 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17206[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; T868 Build/MocorDroid2.3.5) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S17207[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; EBEST X1 Build/BC40_NIECHE_X1W_HW1.1_V001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17208[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BRS-BROR W60+/BROR_W60+_V1.0; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17209[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8060 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S17210[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; AE808 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17211[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; i5300 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17212[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; 7230-B Build/GRJ90)";
const char S17213[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Nexus_One Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17214[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; INCO Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17215[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; BF_A12P Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17216[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad8050 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17217[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; AX3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17218[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; HYPERION 8 Mod by echcon04 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17219[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; HANEL E21 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17220[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 8056 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17221[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; TCL J210C Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17222[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; G-TiDE E9i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17223[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE U887 Build/MocorDroid2.3.5) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S17224[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; OPS-51   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17225[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; telego Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17226[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; S712 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17227[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; MOBO BOSSY Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17228[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-S7100/1.0 Android/2.3.5 Release/05.04.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17229[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; GN107 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17230[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad-8076_TD/1.0 Android/2.3.5 Release/03.11.2012 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17231[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hr-hr; RACER III mini Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17232[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; U35X Build/V1.0.09) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17233[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; iNote beyond Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17234[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; EB-WX1GJ Build/2.01d) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17235[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; de-de; 1370 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17236[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; VITA_V4W Build/HuaqinA51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17237[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; S8022 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17238[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; CHL-S3000/Hualu_S3000_V1.2; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17239[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; YL-Coolpad 5216 Build/GRJ90)";
const char S17240[] = "DESAY_TS908_CMCC_TD/1.0 Android/2.3.7 (LinuxOS 2.6.35.7) Release/04.13.2012 Browser/WAP2.0";
const char S17241[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; th-th; UPAD U30 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17242[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/2.3.5 Release/04.15.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17243[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; V8405 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17244[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; m190s Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17245[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; sprd-GaLaX-X5570/1.0 Android/2.3.5 Release/03.01.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17246[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LNV-Lenovo_A370e/Lenovo A370e_S110_X; 480*800; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17247[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MT-NOTE-LIFE/1.0 Android/2.3.5 Release/05.24.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17248[] = "Mozilla/5.0 (Linux; U; Android 2.3.8; en-us; V-S3500 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17249[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; T780+/TBTX_8990_V0101 Android/2.3.5 Release/05.10.2013 Browser/AppleWebKit533.1 Build/IMM76D)ApplelWebkit/533.1 (KHTML,like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17250[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; VENERA Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17251[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; 2GO Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17252[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME811 Build/SHDWR_X7_2.71.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17253[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; TCLW989 Build/REL_2.3.3.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17254[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; kailitong Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17255[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; C1000 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17256[] = "Mozilla 5.0 (Linux; U; Android 2.3.7; zh-cn; T92 Build GWK74) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S17257[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BLW-VE600/VLAND_VE600_V1.3; 480*800;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17258[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BLF-lephone 1800/lephone_1800_V1.0; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17259[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; shiyu Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17260[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; KliTON I608 Build/KliTONKliTON) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17261[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; tr-tr; E-TAB_GOV Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17262[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; DESAY TS908 Build/MocorDroid2.3.5) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S17263[] = "Mozilla/5.0 (Linux; U; Android 5.3.4(Android 2.3.6); zh-cn; MC919LL Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17264[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; T232 Build/MocorDroid2.3.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17265[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; k9p Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17266[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A370 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17267[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; BOWAY_U3 Build/alpsBOWAY-) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17268[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A326 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17269[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; S9 Build/GRK39F)";
const char S17270[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; Nuqleo-Fusion Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17271[] = "Mozilla/5.0 (Linux; U; Android 2.3.8; in-id; PGM 205 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17272[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; epade A360S Build/alpsepade)";
const char S17273[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Z990 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17274[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-cr; AX520 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17275[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; Mi240   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17276[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-EG876 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17277[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; IHS-C250 Build/IHS-C250-V001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17278[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; Sansui SA41 Build/MocorDroid2.3.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17279[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; BERAQ NB-939 Build/VENUS_00.03.170.H1101A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17280[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; F307 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17281[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lephone_TD9229_TD/1.0 Android/2.3.5 Release/8.9.2012 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5 ) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17282[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ro-ro; Evotech Cyano  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17283[] = "Mozilla/5.0 (Linux; U; Android 2.3; %s; OlivePad) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S17284[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-A1-INFINITY/1.0 Android/2.3.5 Release/04.22.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17285[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-EG901 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17286[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LNV-Lenovo A370e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17287[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; sprd-B54/1.0 Android/2.3.6 Release/05.10.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17288[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; pt-br; Multilaser Orion Build/Multilaser-Orion-V00.00.01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17289[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; ZA966 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17290[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; FIREFLY_S80 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17291[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; starTIM1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17292[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; S350 Build/S350_VK_MX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17293[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; AX353 Build/MocorDroid2.3.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17294[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; ONE TOUCH 4007D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17295[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; T1+ Build/MocorDroid2.3.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17296[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; FWS610 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17297[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-us; QBEN QB700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17298[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; mx1001 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17299[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ouqi_key Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17300[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Z1i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17301[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; kliton I30 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17302[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; M910A Build/GRK39F)";
const char S17303[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZIPO-W919 F Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17304[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; T9555 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17305[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; USMART SL33 Build/USMARTSL33) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17306[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ET1 Build/04-X-2399-0601-00-M1-X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17307[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; epade V05 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17308[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ar-eg; K99+ Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17309[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; sprd-B51/1.0 Android/2.3.6 Release/04.08.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17310[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; sr-rs; Telenor Smart 2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17311[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; itel iNote mini Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17312[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/2.3.5 Release/05.09.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17313[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; MOBO Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17314[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; FLY F40+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17315[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; ONE TOUCH 4007A Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17316[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; OPS-41 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17317[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-S5831i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17318[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-Greentel-G11/1.0 Android/2.3.5 Release/06.19.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17319[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 2.3.6;dev=SGH-I717;devm=SAMSUNG-SGH-I717;man=samsung;brw=Android;shwd=800x1280;shwa=800x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S17320[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; AT-AS35D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S17321[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.813.0 Safari/535.1";
const char S17322[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.187 Safari/535.1";
const char S17323[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S17324[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/534.13";
const char S17325[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.802.30 Safari/535.1 SE 2.X MetaSr 1.0,gzip(gfe) (via translate.google.com)";
const char S17326[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1,gzip(gfe) (via docs.google.com/viewer)";
const char S17327[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.802.30 Safari/535.1 SE 2.X MetaSr 1.0,gzip(gfe)";
const char S17328[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1";
const char S17329[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.1 (KHTML like Gecko) Chrome/14.0.794.0 Safari/535.1";
const char S17330[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML like Gecko) Chrome/14.0.809.0 Safari/535.1";
const char S17331[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.94 Safari/535.1";
const char S17332[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A4400f";
const char S17333[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A4400f [FBAN/FBIOS;FBAV/6.2;FBBV/X;FBDV/iPad2,5;FBMD/iPad;FBSN/iPhone OS;FBSV/7.0;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBOP/1]";
const char S17334[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Mobile/11A4414e";
const char S17335[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Mobile/11A4414e [FBAN/FBIOS;FBAV/6.3;FBBV/X;FBDV/iPad2,5;FBMD/iPad;FBSN/iPhone OS;FBSV/7.0;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBOP/1]";
const char S17336[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko)";
const char S17337[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4435d";
const char S17338[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko)";
const char S17339[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4435d [FBAN/FBIOS;FBAV/6.3;FBBV/X;FBDV/iPad2,5;FBMD/iPad;FBSN/iPhone OS;FBSV/7.0;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBOP/1]";
const char S17340[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449a";
const char S17341[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d";
const char S17342[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d [FBAN/FBIOS;FBAV/6.4;FBBV/X;FBDV/iPad2,5;FBMD/iPad;FBSN/iPhone OS;FBSV/7.0;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBOP/1]";
const char S17343[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449a [FBAN/FBIOS;FBAV/6.4;FBBV/X;FBDV/iPad2,5;FBMD/iPad;FBSN/iPhone OS;FBSV/7.0;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBOP/1]";
const char S17344[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449b";
const char S17345[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465";
const char S17346[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 [FBAN/FBIOS;FBAV/6.5;FBBV/X;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/7.0;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBOP/1]";
const char S17347[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari/8536.25 (AirWatch Browser v2.7.1427)";
const char S17348[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501";
const char S17349[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 iPad2,1/K93AP Zite/2.4.1";
const char S17350[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari/8536.26.7 DiigoBrowser";
const char S17351[] = "Mozilla/5.0 (iPad3,1; iPad; U; CPU OS 7_0 like Mac OS X; en_US) com.google.GooglePlus/23341 (KHTML, like Gecko) Mobile/J1AP (gzip)";
const char S17352[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 7_0 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/7.0;FBSS/1; FBCR/unavailable;FBID/tablet;FBLC/en_US;FBSF/1.0]";
const char S17353[] = "Mozilla/5.0 (iPad2,1; iPad; U; CPU OS 7_0 like Mac OS X; en_US) com.google.GooglePlus/23341 (KHTML, like Gecko) Mobile/K93AP (gzip)";
const char S17354[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko)";
const char S17355[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 [FBAN/FBIOS;FBAV/6.5.1;FBBV/X;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/7.0.2;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBOP/1]";
const char S17356[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) BaiduHD 2.3.4 AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176";
const char S17357[] = "Mozilla/5.0 (iPad3,4; iPad; U; CPU OS 7_0 like Mac OS X; en_US) com.google.GooglePlus/23341 (KHTML, like Gecko) Mobile/P101AP (gzip)";
const char S17358[] = "QQ???HD 3.3 (iPad; iPhone OS 7.0; zh_CN)";
const char S17359[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 iPad2,1/K93AP Zite/2.4.1";
const char S17360[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Safari/8536.26.7 DiigoBrowser";
const char S17361[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) BaiduHD 2.3.4 AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176";
const char S17362[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S17363[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S17364[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S17365[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S17366[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S17367[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S17368[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.26.9 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S17369[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.26.9 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S17370[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S17371[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S17372[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S17373[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S17374[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S17375[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S17376[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S17377[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S17378[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S17379[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S17380[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S17381[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.127 Safari/534.16,gzip(gfe) (via translate.google.com)";
const char S17382[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; ru-RU) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.11 Safari/534.16";
const char S17383[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; de-DE) AppleWebKit/534.17 (KHTML, like Gecko) Chrome/10.0.649.0 Safari/534.17";
const char S17384[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16,gzip(gfe) (via docs.google.com/viewer)";
const char S17385[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/10.0 baidubrowser/1.x Safari/534.7";
const char S17386[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; ru-RU; AppleWebKit/534.16; KHTML; like Gecko; Chrome/10.0.648.11;Safari/534.16)";
const char S17387[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.127 Safari/534.16,gzip(gfe)";
const char S17388[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/10.0.782.112 Safari/535.1";
const char S17389[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.64 Safari/537.31";
const char S17390[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; de-DE) AppleWebKit/534.17 (KHTML like Gecko) Chrome/10.0.649.0 Safari/534.17";
const char S17391[] = "Mozilla/5.0+(Windows;+U;+Windows+NT+6.1;+en-US)+AppleWebKit/534.16+(KHTML,+like+Gecko)+Chrome/10.0.648.204+Safari/534.16";
const char S17392[] = "=Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S17393[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9900; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.576 Mobile Safari/534.11+";
const char S17394[] = "BlackBerry9900/7.0.0.576 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102";
const char S17395[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/18.6.872.0 Safari/535.8";
const char S17396[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19,gzip(gfe) (via docs.google.com/viewer)";
const char S17397[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/535.12 (KHTML, like Gecko) Chrome/18.0.966.0 Safari/535.12,gzip(gfe) (via translate.google.com)";
const char S17398[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19";
const char S17399[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19,gzip(gfe) (via translate.google.com)";
const char S17400[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S17401[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S17402[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/535.12 (KHTML, like Gecko) Chrome/18.0.966.0 Safari/535.12";
const char S17403[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/18.6.872.0 Safari/535.8";
const char S17404[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/18.6.872.0 Safari/535.8";
const char S17405[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay/1911; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S17406[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.1634 Safari/535.19 YI";
const char S17407[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Client/0; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S17408[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/535.12 (KHTML, like Gecko) Chrome/18.0.966.0 Safari/535.12";
const char S17409[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay/1928; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S17410[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay/1950; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S17411[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay/1960; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S17412[] = "Mozilla/5.0 (Windows NT 5.1; en-US) AppleWebKit/535.12 (KHTML, like Gecko) Chrome/18.0.966.0 Safari/535.12";
const char S17413[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay/2005; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S17414[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay/2026; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S17415[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19,gzip(gfe)";
const char S17416[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/535.12 (KHTML, like Gecko) Chrome/18.0.966.0 Safari/535.12,gzip(gfe)";
const char S17417[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2;) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.45 Safari/535.19";
const char S17418[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1;) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.45 Safari/535.19";
const char S17419[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay/2095; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S17420[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; de) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/18.0 Safari/525.13";
const char S17421[] = "Mozilla/5.0 (Windows NT 5.1; zh-CN) AppleWebKit/535.12 (KHTML, like Gecko) Chrome/18.0.966.0 Safari/535.12";
const char S17422[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/18.0 BIDUPlayerBrowser/2.x Safari/534.7";
const char S17423[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay/2109; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S17424[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay/2102; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S17425[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay/2125; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S17426[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay/X; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S17427[] = "Mozilla/5.0 (Windows NT 5.1; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/18.0 BIDUPlayerBrowser/2.x Safari/534.7";
const char S17428[] = "Mozilla/5.0 (Windows NT 5.1; en-US; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025 Safari/535.19";
const char S17429[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B5141a Safari/7534.48.3";
const char S17430[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B148 Safari/7534.48.3";
const char S17431[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3";
const char S17432[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7534.48.3";
const char S17433[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B179 Safari/7534.48.3";
const char S17434[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A406 Safari/7534.48.3";
const char S17435[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B150 Safari/7534.48.3";
const char S17436[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B157 Safari/7534.48.3";
const char S17437[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B155 Safari/7534.48.3";
const char S17438[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B172 Safari/7534.48.3";
const char S17439[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B174 Safari/7534.48.3";
const char S17440[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S17441[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3";
const char S17442[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9Z999 Safari/7534.48.3";
const char S17443[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3";
const char S17444[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A406 Safari/7534.48.3";
const char S17445[] = "Mozilla/5.0 (iPhone; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.46";
const char S17446[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B208 Safari/7534.48.3";
const char S17447[] = "Mozilla/5.0 (; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S17448[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari Version/5.1";
const char S17449[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari Version/5.1";
const char S17450[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari Version/5.1";
const char S17451[] = "Mozilla/5.0 (iPhone; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7534.46";
const char S17452[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9Z999 Safari/7534.48.3";
const char S17453[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7534.48.3";
const char S17454[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari Version/5.1";
const char S17455[] = "Mozilla/5.0 (device; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S17456[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206Safari/7534.48.3";
const char S17457[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3";
const char S17458[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9B206 Version/5.1 Safari/8536.25 Safari/6533.18.5";
const char S17459[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/10A403 Safari/7534.48.3";
const char S17460[] = "Mozilla/5.0 (iPhone; CPU iPhone OS5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S17461[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/10A523 Safari/7534.48.3";
const char S17462[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3";
const char S17463[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/10B329 Safari/7534.48.3";
const char S17464[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; Trident/6.0)";
const char S17465[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; Win64; x64; Trident/6.0)";
const char S17466[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2)";
const char S17467[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0)";
const char S17468[] = "Mozilla/5.0 (compatible; MSIE 9.10; Windows NT 6.2; Trident/5.0)";
const char S17469[] = "Mozilla/4.0 (compatible; MSIE 9.10; Windows NT 6.2; Windows Live Writer 1.0)";
const char S17470[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3230.2052; Windows 6.2; MSIE 9.10.8400.0)";
const char S17471[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3230.2052; Windows 6.2; MSIE 9.10.9200.16420)";
const char S17472[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3230.2052; Windows 6.2; MSIE 9.10.9200.16433)";
const char S17473[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; Trident/5.0; XBLWP7; ZuneWP7)";
const char S17474[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S17475[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.2; MSIE 9.10.9200.16453)";
const char S17476[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.2; MSIE 9.10.9200.16384)";
const char S17477[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0)";
const char S17478[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2)";
const char S17479[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.2; MSIE 9.10.9200.16484)";
const char S17480[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; ASU2JS)";
const char S17481[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.2; MSIE 9.10.9200.16519)";
const char S17482[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.2; MSIE 9.10.9200.16540)";
const char S17483[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; SE 2.X MetaSr 1.0)";
const char S17484[] = "Mozilla/4.0 (compatible; MSIE 9.10; Windows NT 6.2)";
const char S17485[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.2; MSIE 9.10.9200.16580)";
const char S17486[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729)";
const char S17487[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Tablet PC 2.0; ASU2JS)";
const char S17488[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.5.4209.2358; Windows 6.2; MSIE 9.10.9200.16599)";
const char S17489[] = "Mozilla/5.0 (compatible; MSIE 9.11; Windows NT 6.2; Trident/5.0)";
const char S17490[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.5.4209.2358; Windows 6.2; MSIE 9.10.9200.16635)";
const char S17491[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.5.4413.1752; Windows 6.2; MSIE 9.10.9200.16660)";
const char S17492[] = "Mozilla/5.0 (compatible; MSIE 9.10; Windows NT 6.2; WOW64; Trident/6.0; SiteKiosk 7.8 Build 332)";
const char S17493[] = "Mozilla/4.0 (compatible; GoogleToolbar 7.5.4413.1752; Windows 6.2; MSIE 9.10.9200.16688)";
const char S17494[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4128; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S17495[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4128; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S17496[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S17497[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S17498[] = "Microsoft Office/15.0 (Windows NT 6.2; Microsoft Outlook 15.0.4128; Pro)";
const char S17499[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4128; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S17500[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 15.0.4128; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S17501[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4128; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S17502[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S17503[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4128; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S17504[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S17505[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17506[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17507[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17508[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S17509[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S17510[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17511[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17512[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17513[] = "Microsoft Office/15.0 (Windows NT 6.2; Microsoft Outlook 15.0.4420; Pro)";
const char S17514[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17515[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S17516[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17517[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17518[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S17519[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17520[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17521[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17522[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17523[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17524[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17525[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17526[] = "Microsoft Office/15.0 (Windows NT 6.2; Microsoft Outlook 15.0.4433; Pro)";
const char S17527[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Media Center PC 6.0; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17528[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4433; Microsoft Outlook 15.0.4433; ms-office; MSOffice 15)";
const char S17529[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17530[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17531[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S17532[] = "Microsoft Office/15.0 (Windows NT 6.2; Microsoft Outlook 15.0.4454; Pro)";
const char S17533[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4454; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S17534[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4433; Microsoft Outlook 15.0.4433; ms-office; MSOffice 15)";
const char S17535[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Media Center PC 6.0; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17536[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S17537[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S17538[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4454; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S17539[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S17540[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17541[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17542[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 15.0.4128; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S17543[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17544[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S17545[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4454; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S17546[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S17547[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S17548[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17549[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17550[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17551[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17552[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4433; ms-office; MSOffice 15)";
const char S17553[] = "Microsoft Office/15.0 (Windows NT 6.2; Microsoft Outlook 15.0.4481; Pro)";
const char S17554[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4454; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S17555[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 15.0.4454; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S17556[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17557[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17558[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17559[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17560[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17561[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4433; Microsoft Outlook 15.0.4433; ms-office; MSOffice 15)";
const char S17562[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17563[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17564[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17565[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17566[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17567[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17568[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4454; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S17569[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17570[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17571[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17572[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17573[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17574[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17575[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17576[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17577[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17578[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; HPNTDFJS; Tablet PC 2.0; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17579[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Media Center PC 6.0; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17580[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17581[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17582[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17583[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17584[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17585[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 15.0.4433; ms-office; MSOffice 15)";
const char S17586[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17587[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17588[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Media Center PC 6.0; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17589[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17590[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17591[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17592[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17593[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17594[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17595[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17596[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17597[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17598[] = "Microsoft Office/15.0 (Windows NT 6.2; Microsoft Outlook 15.0.4505; Pro)";
const char S17599[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17600[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17601[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17602[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17603[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17604[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17605[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17606[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Tablet PC 2.0; InfoPath.3; MALCJS; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17607[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17608[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17609[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17610[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4454; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S17611[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17612[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17613[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17614[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17615[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17616[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S17617[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S17618[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17619[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17620[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S17621[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S17622[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S17623[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4454; Microsoft Outlook 15.0.4454; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S17624[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Media Center PC 6.0; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17625[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17626[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17627[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17628[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17629[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17630[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; MASMJS; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17631[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17632[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17633[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17634[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17635[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17636[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17637[] = "Microsoft Office/15.0 (Windows NT 6.2; Microsoft Outlook 15.0.4517; Pro)";
const char S17638[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17639[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17640[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17641[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17642[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17643[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17644[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17645[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17646[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17647[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17648[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17649[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17650[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17651[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17652[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17653[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17654[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; MASMJS; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17655[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17656[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17657[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17658[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17659[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17660[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17661[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17662[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17663[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17664[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S17665[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17666[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17667[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17668[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17669[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17670[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S17671[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17672[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S17673[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S17674[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S17675[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; ASU2JS; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S17676[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S17677[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S17678[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17679[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Win64; x64; Trident/6.0; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17680[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S17681[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/7.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4535; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S17682[] = "Microsoft Office/15.0 (Windows NT 6.2; Microsoft Outlook 15.0.4535; Pro)";
const char S17683[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4535; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S17684[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4535; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S17685[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S17686[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S17687[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S17688[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S17689[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S17690[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Tablet PC 2.0; InfoPath.3; MALCJS; Microsoft Outlook 15.0.4535; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S17691[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.20 (KHTML, like Gecko) Chrome/11.0.672.2 Safari/534.20";
const char S17692[] = "Mozilla/5.0 (Windows; Windows NT 6.1) AppleWebKit/534.23 (KHTML, like Gecko) Chrome/11.0.686.3 Safari/534.23";
const char S17693[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S17694[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24";
const char S17695[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.60 Safari/534.24,gzip(gfe) (via translate.google.com)";
const char S17696[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24,gzip(gfe) (via translate.google.com)";
const char S17697[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.20 (KHTML, like Gecko) Chrome/11.0.672.2 Safari/534.20,gzip(gfe) (via translate.google.com)";
const char S17698[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1) AppleWebKit/534.21 (KHTML, like Gecko) Chrome/11.0.682.0 Safari/534.21";
const char S17699[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.697.0 Safari/534.24Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S17700[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; de-DE) AppleWebKit/534.19 (KHTML, like Gecko) Chrome/11.0.661.0 Safari/534.19";
const char S17701[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.60 Safari/534.24,gzip(gfe)";
const char S17702[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24,gzip(gfe)";
const char S17703[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML like Gecko) Chrome/11.0.696.57 Safari/534.24";
const char S17704[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N8000 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S17705[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-N8000 Build/JRO03C)";
const char S17706[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; GT-N8020 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S17707[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; SAMSUNG GT-N8000/N8000XXBLJ9 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S17708[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N8000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S17709[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; SAMSUNG GT-N8020/N8020XXALJD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S17710[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N8000 Build/JZO54K)";
const char S17711[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; SAMSUNG GT-N8020/N8020XDALK1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S17712[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-N8020/N8020XXBMA3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S17713[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N8020 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S17714[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; tr-tr; GT-N8005 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S17715[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; SAMSUNG GT-N8000/N8000XXCMA1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S17716[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GT-N8020-ORANGE/N8020XDALK1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S17717[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-N8020 Build/JRO03C)";
const char S17718[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N8020 Build/JZO54K)";
const char S17719[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-N8020-ORANGE/N8020XDBMA2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S17720[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; SAMSUNG GT-N8020/N8020XDBMA4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S17721[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-at; SAMSUNG GT-N8020/N8020XDALK5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S17722[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; SAMSUNG GT-N8020/N8020XXALK3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S17723[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-at; SAMSUNG GT-N8020/N8020XDBMA2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S17724[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N8005 Build/JZO54K)";
const char S17725[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-N8020/N8020XXBMF2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S17726[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N8000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S17727[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-N8000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S17728[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-pt; SAMSUNG GT-N8000/N8000XXCMD2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S17729[] = "Mozilla/5.0 (X11; U; Linux i686; es-ES) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M";
const char S17730[] = "Mozilla/5.0 (X11; U; Linux i686; fr-fr) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M";
const char S17731[] = "Mozilla/5.0 (X11; U; Linux i686; en-gb) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M";
const char S17732[] = "Mozilla/5.0 (X11; U; Linux i686; fa-fa) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M";
const char S17733[] = "Mozilla/5.0 (X11; U; Linux i686; fa-ir) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M";
const char S17734[] = "Mozilla/5.0 (X11; U; Linux i686; zh-HK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M";
const char S17735[] = "Mozilla/5.0 (X11; U; Linux i686; en-SG) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M";
const char S17736[] = "Mozilla/5.0 (X11; U; Linux i686; en-za) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M";
const char S17737[] = "Mozilla/5.0 (X11; U; Linux i686; en-ae) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M";
const char S17738[] = "Mozilla/5.0 (X11; U; Linux i686; ar-EG) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M";
const char S17739[] = "Mozilla/5.0 (X11; U; Linux i686; fr-ca) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M";
const char S17740[] = "Mozilla/5.0 (X11; U; Linux i686; ar-KW) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M";
const char S17741[] = "Mozilla/5.0 (X11; U; Linux i686; de-AT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M";
const char S17742[] = "Mozilla/5.0 (X11; U; Linux i686; en-IN) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M";
const char S17743[] = "Mozilla/5.0 (X11; U; Linux i686; en-PK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M";
const char S17744[] = "Mozilla/5.0 (X11; U; Linux i686; ar-BH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M";
const char S17745[] = "Mozilla/5.0 (X11; U; Linux i686; en-AS) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M";
const char S17746[] = "Mozilla/5.0 (X11; U; Linux i686; nl-nl) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6838M";
const char S17747[] = "Mozilla/5.0 (X11; U; Linux i686; en-us) AppleWebKit/534.35 (KHTML	 like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.4250M";
const char S17748[] = "Mozilla/5.0 (X11; U; Linux i686; ar-OM) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M";
const char S17749[] = "Mozilla/5.0 (X11; U; Linux i686; en-au) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6885M";
const char S17750[] = "Mozilla/5.0 (X11; U; Linux i686; fa-IR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6885M";
const char S17751[] = "Mozilla/5.0 (X11; U; Linux i686; en-ie) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6885M";
const char S17752[] = "Mozilla/5.0 (X11; U; Linux i686; ru-ru) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6885M";
const char S17753[] = "Mozilla/5.0 (X11; U; Linux i686; zh-hk) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6885M";
const char S17754[] = "Mozilla/5.0 (X11; U; Linux i686; de-de) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6953M";
const char S17755[] = "Mozilla/5.0 (X11; U; Linux i686; it-it) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7055";
const char S17756[] = "Mozilla/5.0 (X11; U; Linux i686; en-sg) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6993";
const char S17757[] = "Mozilla/5.0 (X11; U; Linux i686; sv-se) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6953M";
const char S17758[] = "Mozilla/5.0 (X11; U; Linux i686; ar-AE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6993";
const char S17759[] = "Mozilla/5.0 (X11; U; Linux i686; en-ca) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7310M";
const char S17760[] = "Mozilla/5.0 (X11; U; Linux i686; ko-KR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7310M";
const char S17761[] = "Mozilla/5.0 (X11; U; Linux i686; th-TH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7310M";
const char S17762[] = "Mozilla/5.0 (X11; U; Linux i686; ro-RO) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7310M";
const char S17763[] = "Mozilla/5.0 (X11; U; Linux i686; ar-SA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17764[] = "Mozilla/5.0 (X11; U; Linux i686; en-AU) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17765[] = "Mozilla/5.0 (X11; U; Linux i686; fr-FR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17766[] = "Mozilla/5.0 (X11; U; Linux i686; es-MX) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17767[] = "Mozilla/5.0 (X11; U; Linux i686; ja-JP) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17768[] = "Mozilla/5.0 (X11; U; Linux i686; zh-TW) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17769[] = "Mozilla/5.0 (X11; U; Linux i686; ar-ae) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17770[] = "Mozilla/5.0 (X11; U; Linux i686; nl-NL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17771[] = "Mozilla/5.0 (X11; U; Linux i686; de-DE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17772[] = "Mozilla/5.0 (X11; U; Linux i686; ko-kr) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17773[] = "Mozilla/5.0 (X11; U; Linux i686; pt-br) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7310M";
const char S17774[] = "Mozilla/5.0 (X11; U; Linux i686; vi-vn) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7310M";
const char S17775[] = "Mozilla/5.0 (X11; U; Linux i686; es-CL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7310M";
const char S17776[] = "Mozilla/5.0 (X11; U; Linux i686; ru-RU) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17777[] = "Mozilla/5.0 (X11; U; Linux i686; pt-PT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17778[] = "Mozilla/5.0 (X11; U; Linux i686; pt-BR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17779[] = "Mozilla/5.0 (X11; U; Linux i686; es-es) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17780[] = "Mozilla/5.0 (X11; U; Linux i686; en-GB) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17781[] = "Mozilla/5.0 (X11; U; Linux i686; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17782[] = "Mozilla/5.0 (X11; U; Linux i686; it-IT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17783[] = "Mozilla/5.0 (X11; U; Linux i686; tr-TR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17784[] = "Mozilla/5.0 (X11; U; Linux i686; en-CA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17785[] = "Mozilla/5.0 (X11; U; Linux i686; es-mx) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7193";
const char S17786[] = "Mozilla/5.0 (X11; U; Linux i686; en-HK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17787[] = "Mozilla/5.0 (X11; U; Linux i686; th-th) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17788[] = "Mozilla/5.0 (X11; U; Linux i686; en-IE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17789[] = "Mozilla/5.0 (X11; U; Linux i686; nb-NO) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17790[] = "Mozilla/5.0 (X11; U; Linux i686; fr-CA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17791[] = "Mozilla/5.0 (X11; U; Linux i686; fr-CH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7310M";
const char S17792[] = "Mozilla/5.0 (X11; U; Linux i686; he-IL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17793[] = "Mozilla/5.0 (X11; U; Linux i686; es-CO) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17794[] = "Mozilla/5.0 (X11; U; Linux i686; es-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17795[] = "Mozilla/5.0 (X11; U; Linux i686; zh-CN) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17796[] = "Mozilla/5.0 (X11; U; Linux i686; es-AR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17797[] = "Mozilla/5.0 (X11; U; Linux i686; de-CH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17798[] = "Mozilla/5.0 (X11; U; Linux i686; en-PH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17799[] = "Mozilla/5.0 (X11; U; Linux i686; en-NZ) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17800[] = "Mozilla/5.0 (X11; U; Linux i686; nl-BE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17801[] = "Mozilla/5.0 (X11; U; Linux i686; pl-PL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17802[] = "Mozilla/5.0 (X11; U; Linux i686; vi-VN) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17803[] = "Mozilla/5.0 (X11; U; Linux i686; da-DK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17804[] = "Mozilla/5.0 (X11; U; Linux i686; sv-SE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17805[] = "Mozilla/5.0 (X11; U; Linux i686; el-GR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17806[] = "Mozilla/5.0 (X11; U; Linux i686; es-VE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17807[] = "Mozilla/5.0 (X11; U; Linux i686; en-ZA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17808[] = "Mozilla/5.0 (X11; U; Linux i686; fr-BE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17809[] = "Mozilla/5.0 (X11; U; Linux i686; hr-HR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17810[] = "Mozilla/5.0 (X11; U; Linux i686; ro-ro) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17811[] = "Mozilla/5.0 (X11; U; Linux i686; de-at) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17812[] = "Mozilla/5.0 (X11; U; Linux i686; es-PE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17813[] = "Mozilla/5.0 (X11; U; Linux i686; fi-FI) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17814[] = "Mozilla/5.0 (X11; U; Linux i686; ar-eg) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17815[] = "Mozilla/5.0 (X11; U; Linux i686; ru-UA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17816[] = "Mozilla/5.0 (X11; U; Linux i686; hu-hu) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17817[] = "Mozilla/5.0 (X11; U; Linux i686; es-CR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17818[] = "Mozilla/5.0 (X11; U; Linux i686; id-ID) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17819[] = "Mozilla/5.0 (X11; U; Linux i686; ms-MY) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M";
const char S17820[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7812";
const char S17821[] = "Mozilla/5.0 (X11; U; Linux i686; tr-tr) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7812";
const char S17822[] = "Mozilla/5.0 (X11; U; Linux i686; en-in) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7812";
const char S17823[] = "Mozilla/5.0 (X11; U; Linux i686; es-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7812";
const char S17824[] = "Mozilla/5.0 (X11; U; Linux i686; zh-cn) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7812";
const char S17825[] = "Mozilla/5.0 (X11; U; Linux i686; en-nz) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7812";
const char S17826[] = "Mozilla/5.0 (X11; U; Linux i686; iw-il) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7812";
const char S17827[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-nz) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7910";
const char S17828[] = "Mozilla/5.0 (X11; U; Linux x86_64; zh-hk) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7910";
const char S17829[] = "Mozilla/5.0 (X11; U; Linux x86_64; tr-tr) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7910";
const char S17830[] = "Mozilla/5.0 (X11; U; Linux x86_64; vi-VN) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7910";
const char S17831[] = "Mozilla/5.0 (X11; U; Linux x86_64; nb-no) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7910";
const char S17832[] = "Mozilla/5.0 (X11; U; Linux x86_64; el-GR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7910";
const char S17833[] = "Mozilla/5.0 (X11; U; Linux x86_64; nl-be) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7910";
const char S17834[] = "Mozilla/5.0 (X11; U; Linux x86_64; ar-eg) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7910";
const char S17835[] = "Mozilla/5.0 (X11; U; Linux x86_64; de-at) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7910";
const char S17836[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-GB) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17837[] = "Mozilla/5.0 (X11; U; Linux x86_64; fa-IR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17838[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17839[] = "Mozilla/5.0 (X11; U; Linux x86_64; zh-TW) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17840[] = "Mozilla/5.0 (X11; U; Linux x86_64; fr-CA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17841[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-AU) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17842[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-MX) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17843[] = "Mozilla/5.0 (X11; U; Linux x86_64; sv-SE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17844[] = "Mozilla/5.0 (X11; U; Linux x86_64; ar-ae) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17845[] = "Mozilla/5.0 (X11; U; Linux x86_64; he-IL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17846[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-CA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17847[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-CO) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17848[] = "Mozilla/5.0 (X11; U; Linux x86_64; it-IT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17849[] = "Mozilla/5.0 (X11; U; Linux x86_64; de-DE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17850[] = "Mozilla/5.0 (X11; U; Linux x86_64; th-TH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17851[] = "Mozilla/5.0 (X11; U; Linux x86_64; fr-FR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17852[] = "Mozilla/5.0 (X11; U; Linux x86_64; ar-SA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17853[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-ES) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17854[] = "Mozilla/5.0 (X11; U; Linux x86_64; ko-KR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17855[] = "Mozilla/5.0 (X11; U; Linux x86_64; ja-JP) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17856[] = "Mozilla/5.0 (X11; U; Linux x86_64; tr-TR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17857[] = "Mozilla/5.0 (X11; U; Linux x86_64; zh-HK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17858[] = "Mozilla/5.0 (X11; U; Linux x86_64; vi-vn) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17859[] = "Mozilla/5.0 (X11; U; Linux x86_64; nl-NL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17860[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-NZ) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17861[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-PH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17862[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17863[] = "Mozilla/5.0 (X11; U; Linux x86_64; nl-BE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17864[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-SG) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17865[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-ZA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17866[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-IN) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17867[] = "Mozilla/5.0 (X11; U; Linux x86_64; ru-RU) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17868[] = "Mozilla/5.0 (X11; U; Linux x86_64; pl-pl) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17869[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-PK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17870[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-HK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17871[] = "Mozilla/5.0 (X11; U; Linux x86_64; da-DK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17872[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-IE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17873[] = "Mozilla/5.0 (X11; U; Linux x86_64; nb-NO) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17874[] = "Mozilla/5.0 (X11; U; Linux x86_64; pt-PT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17875[] = "Mozilla/5.0 (X11; U; Linux x86_64; zh-CN) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17876[] = "Mozilla/5.0 (X11; U; Linux x86_64; ar-EG) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17877[] = "Mozilla/5.0 (X11; U; Linux x86_64; pt-BR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17878[] = "Mozilla/5.0 (X11; U; Linux x86_64; el-gr) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17879[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-CL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17880[] = "Mozilla/5.0 (X11; U; Linux x86_64; fr-CH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17881[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-AR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17882[] = "Mozilla/5.0 (X11; U; Linux x86_64; de-CH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17883[] = "Mozilla/5.0 (X11; U; Linux x86_64; ms-MY) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17884[] = "Mozilla/5.0 (X11; U; Linux x86_64; de-AT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17885[] = "Mozilla/5.0 (X11; U; Linux x86_64; ro-RO) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17886[] = "Mozilla/5.0 (X11; U; Linux x86_64; fr-BE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17887[] = "Mozilla/5.0 (X11; U; Linux x86_64; en) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17888[] = "Mozilla/5.0 (X11; U; Linux x86_64; ar-KW) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17889[] = "Mozilla/5.0 (X11; U; Linux x86_64; cs-CZ) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17890[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-PE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17891[] = "Mozilla/5.0 (X11; U; Linux x86_64; bg-BG) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17892[] = "Mozilla/5.0 (X11; U; Linux x86_64; ru-UA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17893[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-VE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17894[] = "Mozilla/5.0 (X11; U; Linux x86_64; fi-FI) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17895[] = "Mozilla/5.0 (X11; U; Linux x86_64; hu-HU) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17896[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-EC) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17897[] = "Mozilla/5.0 (X11; U; Linux x86_64; iw-il) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17898[] = "Mozilla/5.0 (X11; U; Linux x86_64; id-ID) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17899[] = "Mozilla/5.0 (X11; U; Linux x86_64; ca-ES) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17900[] = "Mozilla/5.0 (X11; U; Linux x86_64; sl-SI) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17901[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-PR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17902[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-CR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17903[] = "Mozilla/5.0 (X11; U; Linux x86_64; hr-HR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17904[] = "Mozilla/5.0 (X11; U; Linux x86_64; sk-SK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17905[] = "Mozilla/5.0 (X11; U; Linux x86_64; in-id) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17906[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-BE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17907[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-GT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17908[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-DO) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17909[] = "Mozilla/5.0 (X11; U; Linux x86_64; it-CH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17910[] = "Mozilla/5.0 (X11; U; Linux x86_64; zh-MO) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17911[] = "Mozilla/5.0 (X11; U; Linux x86_64; ar-BH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17912[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-AS) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17913[] = "Mozilla/5.0 (X11; U; Linux x86_64; ru-KZ) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17914[] = "Mozilla/5.0 (X11; U; Linux x86_64; it-it) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8506";
const char S17915[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-au) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8506";
const char S17916[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-es) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8506";
const char S17917[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-PA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17918[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-VI) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17919[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-MT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17920[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17921[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-HN) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17922[] = "Mozilla/5.0 (X11; U; Linux x86_64; ar-MA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17923[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-SV) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17924[] = "Mozilla/5.0 (X11; U; Linux x86_64; ar-OM) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17925[] = "Mozilla/5.0 (X11; U; Linux x86_64; ar-QA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17926[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-TT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17927[] = "Mozilla/5.0 (X11; U; Linux x86_64; fr-fr) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8777";
const char S17928[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-ca) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8777";
const char S17929[] = "Mozilla/5.0 (X11; U; Linux x86_64; pl-PL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8777";
const char S17930[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-gb) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8909";
const char S17931[] = "Mozilla/5.0 (X11; U; Linux x86_64; uk-UA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17932[] = "Mozilla/5.0 (X11; U; Linux x86_64; ar-IQ) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17933[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-BB) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17934[] = "Mozilla/5.0 (X11; U; Linux x86_64; ur-PK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17935[] = "Mozilla/5.0 (X11; U; Linux x86_64; zh-tw) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8909";
const char S17936[] = "Mozilla/5.0 (X11; U; Linux x86_64; et-EE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17937[] = "Mozilla/5.0 (X11; U; Linux x86_64; ru-ru) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.9039";
const char S17938[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-dz) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17939[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.9133";
const char S17940[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US-POSIX) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17941[] = "Mozilla/5.0 (X11; U; Linux x86_64; ja-JP@calendar=japanese) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17942[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US@calendar=buddhist) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17943[] = "Mozilla/5.0 (X11; U; Linux x86_64; bn-BD) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17944[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-UM) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17945[] = "Mozilla/5.0 (X11; U; Linux x86_64; lt-LT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17946[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-NG) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17947[] = "Mozilla/5.0 (X11; U; Linux x86_64; is-IS) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17948[] = "Mozilla/5.0 (X11; U; Linux x86_64; ko-kr) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181";
const char S17949[] = "Mozilla/5.0 (X11; U; Linux x86_64; de-de) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181";
const char S17950[] = "Mozilla/5.0 (X11; U; Linux x86_64; fa-IR@calendar=persian) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17951[] = "Mozilla/5.0 (X11; U; Linux x86_64; fa-fa) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181";
const char S17952[] = "Mozilla/5.0 (X11; U; Linux x86_64; fa-ir) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181";
const char S17953[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-tw) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961";
const char S17954[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181";
const char S17955[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-ae) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181";
const char S17956[] = "Mozilla/5.0 (X11; U; Linux x86_64; pt-br) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181";
const char S17957[] = "Mozilla/5.0 (X11; U; Linux x86_64; pt-pt) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181";
const char S17958[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-in) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181";
const char S17959[] = "Mozilla/5.0 (X11; U; Linux x86_64; zh-cn) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181";
const char S17960[] = "Mozilla/5.0 (X11; U; Linux x86_64; fa-) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181";
const char S17961[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181,gzip(gfe)";
const char S17962[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-bh) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181";
const char S17963[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-za) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181";
const char S17964[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-jo) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181";
const char S17965[] = "Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17";
const char S17966[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17";
const char S17967[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17,gzip(gfe)";
const char S17968[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.70 Safari/537.17";
const char S17969[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17,gzip(gfe)";
const char S17970[] = "Mozilla/5.0 (X11; U; Unix; en-US) AppleWebKit/537.15 (KHTML, like Gecko) Chrome/24.0.1295.0 Safari/537.15 Surf/0.6";
const char S17971[] = "Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/24.0.1410.63 Safari/537.31";
const char S17972[] = "Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/24.0.1410.63 Safari/537.31";
const char S17973[] = "Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/24.0.1410.63 Safari/537.31";
const char S17974[] = "Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/24.0.1410.63 Safari/537.31";
const char S17975[] = "Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/24.0.1410.63 Safari/537.31";
const char S17976[] = "Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/24.0.1410.63 Safari/537.31";
const char S17977[] = "Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/24.0.1410.63 Safari/537.31";
const char S17978[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC One S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S17979[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_One_S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S17980[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC VLE_U Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S17981[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_S/1.53.161.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S17982[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_S/1.53.110.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S17983[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC One S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.1 Mobile Safari/534.30";
const char S17984[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_S-orange-LS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S17985[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC One S Build/IML74K)";
const char S17986[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_One_S/1.77.111.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S17987[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC_One_S/1.53.166.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S17988[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC VLE_U Build/IML74K)";
const char S17989[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_One_S/1.53.169.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S17990[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_One_S/1.53.111.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S17991[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; Z520m Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S17992[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_S/1.78.161.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S17993[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; HTC_One_S/1.53.114.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S17994[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_One_S/1.77.111.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S17995[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC_VLE_U Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S17996[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_One_S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S17997[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC_One_S/1.53.163.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S17998[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_One_S/1.53.112.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S17999[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC_One_S/1.78.166.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18000[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_One_S/1.78.169.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18001[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_S/1.77.110.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18002[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC One S Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S18003[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC One S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18004[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; HTC One xs Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18005[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC One S Build/IMM76D)";
const char S18006[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC VLE_U Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18007[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; Z520m Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18008[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; HTC_One_S/2.38.169.6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18009[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-ch; HTC_One_S/2.38.166.6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18010[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC VLE_U Build/IMM76D)";
const char S18011[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_One_S-orange-LS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18012[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_One_S/2.38.110.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18013[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_One_S/2.38.161.6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18014[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; HTC_One_S/1.77.114.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18015[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_One_S_pj40200/1.11.169.132 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18016[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_One_S_pj40200/1.11.161.133 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18017[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HTC_One_S/2.38.111.10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18018[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; HTC_One_S/2.38.114.10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18019[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; HTC_One_S/2.38.112.10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18020[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC One S Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S18021[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; HTC_One_S/1.11.118.122 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18022[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; HTC_One_SC_T528d Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18023[] = "Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; HTC One S Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S18024[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneS/1.53.112.4 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18025[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCOneS Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18026[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC ONE S Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18027[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneS/1.78.161.2 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18028[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCOneS/2.38.111.10 Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18029[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneS/1.77.111.4 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18030[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneS/1.77.110.5 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18031[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneS/1.53.110.3 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18032[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCOneS/2.38.161.6 Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18033[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCOneS-orange-LS Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18034[] = "Mozilla/5.0 (Linux; U; Android 4.0; es-US; HTC_One_S/1.53.161.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18035[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-cz; HTC_One_S/1.11.113.123 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18036[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; HTC_One_S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18037[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; HTC_One_S/2.38.163.7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18038[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCOneS/2.38.166.6 Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18039[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneS/1.53.161.3 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18040[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneS Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18041[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; HTC_One_SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18042[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_One_S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18043[] = "Mozilla/5.0 Macintosh Intel Mac OS X 10_6_8 AppleWebKit/537.17 KHTML, like Gecko Chrome/24.0.1312.5 Safari/537.17";
const char S18044[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S18045[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S18046[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S18047[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S18048[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S18049[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S18050[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S18051[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S18052[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S18053[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe) (via docs.google.com/viewer)";
const char S18054[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)";
const char S18055[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/24.0.648.205 Safari/534.16";
const char S18056[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/24.0.648.205 Safari/534.16";
const char S18057[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/24.0.648.205 Safari/534.16";
const char S18058[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S18059[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S18060[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S18061[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S18062[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S18063[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S18064[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S18065[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe)";
const char S18066[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B5141a Safari/7534.48.3";
const char S18067[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7534.48.3";
const char S18068[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3";
const char S18069[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B157 Safari/7534.48.3";
const char S18070[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B169 Safari/7534.48.3";
const char S18071[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B174 Safari/7534.48.3";
const char S18072[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7534.46";
const char S18073[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S18074[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.46";
const char S18075[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3";
const char S18076[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9Z999 Safari/7534.48.3";
const char S18077[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari Version/5.1";
const char S18078[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari Version/5.1";
const char S18079[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B155 Safari/7534.48.3";
const char S18080[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S18081[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B172 Safari/7534.48.3";
const char S18082[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B179 Safari/7534.48.3";
const char S18083[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Version/5.1 Safari/8536.25";
const char S18084[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B202 Safari/7534.48.3";
const char S18085[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Version/5.1 Safari/7534.48.3 Kikin/1.9.5";
const char S18086[] = "Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X; zh-cn Model:iPad3,1) UC AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7543.48.3";
const char S18087[] = "Mozilla/5.0 (iPad; CPU OS 5_1_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3";
const char S18088[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/10A523 Safari/7534.48.3";
const char S18089[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Version/5.1 Safari/8536.25";
const char S18090[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/10A403 Safari/7534.48.3";
const char S18091[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B208 Safari/7534.48.3";
const char S18092[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7534.48.3";
const char S18093[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/10B329 Safari/7534.48.3";
const char S18094[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5";
const char S18095[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5";
const char S18096[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5,gzip(gfe) (via docs.google.com/viewer)";
const char S18097[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S18098[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5,gzip(gfe) (via translate.google.com)";
const char S18099[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S18100[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5,gzip(gfe)";
const char S18101[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5";
const char S18102[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5";
const char S18103[] = "Mozilla/5.0 (compatible; Windows NT 5.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/19.0.1084.36 Safari/535.19";
const char S18104[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5";
const char S18105[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5,gzip(gfe)";
const char S18106[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.10";
const char S18107[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S18108[] = "Mozilla/5.0(iPad; U; CPU iPhone OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B314 Safari/531.21.10";
const char S18109[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2_1 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B405 Safari/531.21.10";
const char S18110[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7W367a Safari/531.21.10";
const char S18111[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B500 Safari/531.21.10";
const char S18112[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/7B367 Safari/6531.22.7";
const char S18113[] = "UserAgent: Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B405 Safari/531.21.10";
const char S18114[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176,Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S18115[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B500 Safari/53";
const char S18116[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/9Z999 Safari/531.21.10";
const char S18117[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.10 Safari/525.20";
const char S18118[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206,Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S18119[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; es-es) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B360 Safari/531.21.10";
const char S18120[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.X-10-16 20:23:50";
const char S18121[] = "Mozilla/5.0(iPad; U; CPU iPhone OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B314 Safari/123";
const char S18122[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.10";
const char S18123[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21 (KHTML, like Gecko) Version/4.0.4 Mobile/7B314 Safari/531.21";
const char S18124[] = "Mozilla/5.0 (iPad; U; CPU OS 322 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML like Gecko) Version/4.0.4 Mobile/7B500 Safari/531.21.10";
const char S18125[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.102.3.3)";
const char S18126[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.1021.10gin_lib.cc";
const char S18127[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B314";
const char S18128[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18129[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-N7000-ORANGE/N7000XXKKA Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18130[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHV-E160K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18131[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHV-E160K Build/GINGERBREAD)";
const char S18132[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-N7000/N7000XXKL7 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18133[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18134[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ja-jp; SC-05D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18135[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-N7000B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18136[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHV-E160S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18137[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-N7000 Build/GINGERBREAD)";
const char S18138[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; SAMSUNG GT-N7000/N7000XXLC1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18139[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ie; SAMSUNG GT-N7000/N7000XXLB1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18140[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-N7000/N7000XXKKA Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18141[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHV-E160L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18142[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-N7000 Build/GINGERBREAD)";
const char S18143[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHV-E160L Build/GINGERBREAD)";
const char S18144[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHV-E160S Build/GINGERBREAD)";
const char S18145[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-N7000 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S18146[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i889 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18147[] = "Mozilla/5.0 (Linux; U; Android 2.3.6-mobile-star; iw-il; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18148[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-N7000/N7000XXLA6 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18149[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-N7000 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S18150[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18151[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SC-05D Build/GINGERBREAD)";
const char S18152[] = "Mozilla/5.0 (Linux; U; fr-fr; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S18153[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-N7000-ORANGE/N7000XXLA6 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18154[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-N7000 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S18155[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-N7000 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S18156[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-N7000B Build/GINGERBREAD)";
const char S18157[] = "Mozilla/5.0 (Linux; U; en-ie; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S18158[] = "Mozilla/5.0 (Linux; U; de-de; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S18159[] = "Mozilla/5.0 (Linux; U; it-it; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S18160[] = "Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; GT-N7000 Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S18161[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-i889 Build/GINGERBREAD)";
const char S18162[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-N7000-ORANGE/N7000XXLC1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18163[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i889 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S18164[] = "Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; GT-N7000 Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S18165[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18166[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-N7000 Build/GRJ22)";
const char S18167[] = "Mozilla/5.0 (Linux; U; en-gb; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S18168[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT-N7000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18169[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18170[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SHV-E160L Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S18171[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe)";
const char S18172[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)";
const char S18173[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe)";
const char S18174[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.1.3.0 Safari/537.31";
const char S18175[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.1.3.0 Safari/537.31";
const char S18176[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S18177[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S18178[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)";
const char S18179[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.2.2.0 Safari/537.31";
const char S18180[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.2.2.0 Safari/537.31";
const char S18181[] = "Mozilla/5.0 (X11; U; Linux x86_64; C) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.8.0 Safari/533.3";
const char S18182[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.1 Safari/534.34";
const char S18183[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.2";
const char S18184[] = "Mozilla/5.0 (QtEmbedded; Linux) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.0 Safari/534.34";
const char S18185[] = "Mozilla/5.0 (QtEmbedded; Linux; en) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.1 Safari/534.34";
const char S18186[] = "Mozilla/5.0 (QtEmbedded; Linux; po) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.1 Safari/534.34";
const char S18187[] = "Mozilla/5.0 (QtEmbedded; Linux; de) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.1 Safari/534.34";
const char S18188[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.21 (KHTML, like Gecko) Qt/4.8.4 Safari/537.21";
const char S18189[] = "Mozilla/5.0 (QtEmbedded; Linux; en-US) AppleWebKit/537.4 (KHTML, like Gecko) Qt/4.8.3 Safari/534.34";
const char S18190[] = "Mozilla/5.0 (QtEmbedded; Linux; en-GB) AppleWebKit/537.4 (KHTML, like Gecko) Qt/4.8.3 Safari/534.34";
const char S18191[] = "Mozilla/5.0 (QtEmbedded; Linux; fr) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.1 Safari/534.34";
const char S18192[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36";
const char S18193[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36";
const char S18194[] = "Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36";
const char S18195[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36,gzip(gfe)";
const char S18196[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36,gzip(gfe)";
const char S18197[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S18198[] = "Mozilla/4.0 (compatible; Blog Search;)";
const char S18199[] = "NetworkedBlogs (+http://www.networkedblogs.com;) AppEngine-Google; (+http://code.google.com/appengine; appid: s~networkedblogshr)";
const char S18200[] = "Mozilla/5.0 (compatible; LinkChecker/8.2; +http://linkchecker.sourceforge.net/)";
const char S18201[] = "OppO 1.0 ( http://www.inboundscore.com/ )";
const char S18202[] = "Open Web Indexer (+http://www.icedteapowered.com/openweb/)";
const char S18203[] = "BellPagesCA/1.0; +http://www.BellPages.ca";
const char S18204[] = "Mozilla/5.0 (compatible; Allrati/1.2; +http://www.allrati.com/)";
const char S18205[] = "Mozilla/5.0 (compatible; Allrati/1.21; +http://www.allrati.com/)";
const char S18206[] = "PulseParser/0.1 AppEngine-Google; (+http://code.google.com/appengine; appid: s~hr-pulsesubscriber)";
const char S18207[] = "SiteExplorer/1.0 http://siteexplorer.info/";
const char S18208[] = "PulseParser/0.3 AppEngine-Google; (+http://code.google.com/appengine; appid: s~hr-pulsesubscriber)";
const char S18209[] = "Mozilla/5.0 (compatible; SiteExplorer/1.0b; +http://siteexplorer.info/)";
const char S18210[] = "RockmeltEmbedder (http://rockmelt.com; feedback@rockmelt.com)";
const char S18211[] = "BUbiNG (+http://law.di.unimi.it/BUbiNG.html)";
const char S18212[] = "A6-Indexer/1.0 (http://www.a6corp.com/a6-web-scraping-policy/)";
const char S18213[] = "Feed-Fetcher AppEngine-Google; (+http://code.google.com/appengine; appid: s~ziin-me)";
const char S18214[] = "Mozilla/5.0 (compatible; Web-Monitoring/1.0; +http://monoid.nic.ru/)";
const char S18215[] = "VSE/1.0 (John.Macartney@cengage.com)";
const char S18216[] = "Scrapy/0.16.5 (+http://scrapy.org)";
const char S18217[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/26.0.648.205 Safari/534.16";
const char S18218[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/26.0.648.205 Safari/534.16";
const char S18219[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31,gzip(gfe)";
const char S18220[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)";
const char S18221[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S18222[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S18223[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S18224[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S18225[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S18226[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S18227[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S18228[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S18229[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S18230[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/26.0.648.205 Safari/534.16";
const char S18231[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.91 Safari/534.30";
const char S18232[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30,gzip(gfe) (via translate.google.com)";
const char S18233[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30";
const char S18234[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S18235[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30,gzip(gfe) (via docs.google.com/viewer)";
const char S18236[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.724.100 Safari/534.30";
const char S18237[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.27 (KHTML, like Gecko) Chrome/12.0.712.0 (Chromium Portable 9.2)";
const char S18238[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/12.0.702.0 Safari/533.3 Awesomium/1.6.5";
const char S18239[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30, SurfCop 1.5";
const char S18240[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.30 (KHTML	 like Gecko) Chrome/12.0.742.91 Safari/534.30";
const char S18241[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30,gzip(gfe)";
const char S18242[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; de) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/12.0 Safari/525.19";
const char S18243[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML like Gecko) Chrome/12.0.742.53 Safari/534.30";
const char S18244[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-CA) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.724.100 Safari/534.30";
const char S18245[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; es-US) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.724.100 Safari/534.30";
const char S18246[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; fr-CA) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.724.100 Safari/534.30";
const char S18247[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; haw-US) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.724.100 Safari/534.30";
const char S18248[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; br-FR) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.724.100 Safari/534.30";
const char S18249[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.724.100 Safari/534.30";
const char S18250[] = "Mozilla/5.0 (Windows NT 5.1; Alibrowser 1.0) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.9 Safari/534.30";
const char S18251[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.27 (KHTML, like Gecko) Chrome/12.0.712.0 (Chromium Portable 9.2)";
const char S18252[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.30 (KHTML like Gecko) Chrome/12.0.724.100 Safari/534.30";
const char S18253[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S18254[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S18255[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko; Google Web Preview) Chrome/22.0.1229 Safari/537.4";
const char S18256[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229 Safari/537.4";
const char S18257[] = "Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S18258[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko; Google Page Speed Insights) Chrome/22.0.1229 Safari/537.4";
const char S18259[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko; Google Web Preview) Chrome/22.0.1229 Safari/537.4,gzip(gfe) (via docs.google.com/viewer)";
const char S18260[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)";
const char S18261[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4,gzip(gfe) (via translate.google.com)";
const char S18262[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)";
const char S18263[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML like Gecko) Chrome/22.0.1229.56 Safari/537.4";
const char S18264[] = "Mozilla/5.0 (X11; Linux armv6l) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S18265[] = "Mozilla/5.0 (Linux; U) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4";
const char S18266[] = "Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/22.0.1410.63 Safari/537.31";
const char S18267[] = "Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/22.0.1410.63 Safari/537.31";
const char S18268[] = "Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/22.0.1410.63 Safari/537.31";
const char S18269[] = "Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/22.0.1410.63 Safari/537.31";
const char S18270[] = "Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/22.0.1410.63 Safari/537.31";
const char S18271[] = "Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/22.0.1410.63 Safari/537.31";
const char S18272[] = "Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/22.0.1410.63 Safari/537.31";
const char S18273[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko; Google Web Preview) Chrome/22.0.1229 Safari/537.4,gzip(gfe)";
const char S18274[] = "Mozilla/5.0 (X11; U; Linux i586) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S18275[] = "Mozilla/5.0 (Linux; U) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4,gzip(gfe)";
const char S18276[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I8190 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18277[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-I8190 Build/JRO03H)";
const char S18278[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; SAMSUNG GT-I8190/I8190XXALK6 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18279[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; SAMSUNG GT-I8190/I8190XXALJL Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18280[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; SAMSUNG GT-I8190/I8190XXALL2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18281[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8190 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18282[] = "Mozilla/5.0 (Linux; U; Android 4.1.1_????-??????; he-il; GT-I8190 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18283[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-I8190 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18284[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8190 Build/JZO54K)";
const char S18285[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; SAMSUNG GT-I8190/I8190XXALK1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18286[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I8190/I8190XXAMB3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18287[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-I8190 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18288[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I8190 Build/JRO03H) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18289[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I8190 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18290[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I8190/I8190XXAME1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18291[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I8190 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18292[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I8190/I8190XXAMD3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18293[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I8190 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18294[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-I8190 Build/JRO03H) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18295[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-pt; SAMSUNG GT-I8190/I8190XXAMG1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18296[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; I8190 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18297[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; el-gr; SAMSUNG GT-I8190/I8190XXAMB2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18298[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S18299[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S18300[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P5110 Build/IMM76D)";
const char S18301[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-P5110 Build/IML74K)";
const char S18302[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S18303[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S18304[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18305[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5110 Build/IML74K; 1280*752) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S18306[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5110 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S18307[] = "BBC iPlayer (AndroidApp; 1.3; 13014) GT-P5110 (Android 4.0.4, SDK 15)";
const char S18308[] = "Biblia/2.1.2 (Linux; U; Android 4.0.3; es-US; GT-P5110 Build/IML74K.P5110XXALD6)";
const char S18309[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18310[] = "FaithlifeStudyBible/2.1.2 (Linux; U; Android 4.0.4; en-GB; GT-P5110 Build/IMM76D.P5110XXBLH3)";
const char S18311[] = "com.google.android.youtube/4.1.23(Linux; U; Android 4.0.3; en_AU; GT-P5110 Build/IML74K) gzip";
const char S18312[] = "Biblia/2.1.2 (Linux; U; Android 4.0.3; es-ES; GT-P5110 Build/IML74K.P5110XWALD4)";
const char S18313[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5110 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S18314[] = "BBC News (AndroidApp; 1.0; 12) GT-P5110 (Android 4.0.4, SDK 15)";
const char S18315[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.2.1-65 (Android 4.0.3; samsung GT-P5110 Build/IML74K.P5110XWALD2)";
const char S18316[] = "Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 4.0.4; SAMSUNG GT-P5110 Build/ICE_CREAM_SANDWICH_MR1);";
const char S18317[] = "Logos/2.1.2 (Linux; U; Android 4.0.4; nl-BE; GT-P5110 Build/IMM76D.P5110XXBLI1)";
const char S18318[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S18319[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5110 Build/IMM76D; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S18320[] = "Vyrso/2.2.3 (Linux; U; Android 4.0.3; en-AU; GT-P5110 Build/IML74K.P5110XWALE2)";
const char S18321[] = "Logos/2.2.3 (Linux; U; Android 4.0.3; de-DE; GT-P5110 Build/IML74K.P5110XWALD2)";
const char S18322[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5110 Build/IML74K) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S18323[] = "Logos/2.2.3 (Linux; U; Android 4.0.3; en-NZ; GT-P5110 Build/IML74K.P5110XWALE1)";
const char S18324[] = "BBC News (AndroidApp; 1.1; 17) GT-P5110 (Android 4.0.4, SDK 15)";
const char S18325[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hr-hr; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30";
const char S18326[] = "Groupon/2.1.2483 (Android 4.0.3; Samsung Espresso10wifi / Samsung GT-P5110; )[preload=false;locale=es_US;clientidbase=android-samsung]";
const char S18327[] = "com.google.android.apps.plus/X (Linux; U; Android 4.0.3; zh_TW; GT-P5110; Build/IML74K) (gzip)";
const char S18328[] = "com.google.android.apps.plus/X (Linux; U; Android 4.0.4; en_US; GT-P5110; Build/IMM76D) (gzip)";
const char S18329[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/1.9.6;FBDM/{density=1.0,width=1280,height=752};FBLC/es_US;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P5110;FBSV/4.0.3;]";
const char S18330[] = "Logos/2.2.3 (Linux; U; Android 4.0.4; de-DE; GT-P5110 Build/cm_p5110-userdebug 4.0.4 IMM76L eng..X.X test-keys)";
const char S18331[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 sessionId/e4eb0725-5a8d-403c-b7ef-2f571f6a5400 tiantian(securitypay) tiantian(tenpay)";
const char S18332[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5110 Build/IML74K) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 baidubrowser/061_1.41.3.2_diordna_2321_008/gnusmas_51_3.0.4_0115P-TG/1200a/C88EF6E4BA2CB8DA86308DB47B04845C%7C0/1";
const char S18333[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5110 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18334[] = "VKAndroidApp/2.7.6-64 (Android 4.0.3; SDK 15; armeabi-v7a; samsung GT-P5110; ru)";
const char S18335[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/5.0.2 Version/4.0 Safari/534.30 Safari/6533.18.5";
const char S18336[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S18337[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko)6.X.1.1.utmcsr=google|utmccn=(organic)|utmcmd=organic|utmctr=sofas; _msuuid_5620s53616=BDA191D0-2F2A-43DC-B131-0BAX";
const char S18338[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5110 Buil=X; __utma=X.X.X.X.X.1; __utmb=X.12.10.X; __utmc=X; __utmz=X.X.1.1.utmcsr=google|utmccn=(organic)|utmcmd=organic|utmctr=sofas";
const char S18339[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5110 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18340[] = "com.google.android.youtube/4.0.23(Linux; U; Android 4.0.4; en_GB; GT-P5110 Build/IMM76D) gzip";
const char S18341[] = "FaithlifeStudyBible/2.2.10 (Linux; U; Android 4.0.4; nb-NO; GT-P5110 Build/IMM76D.P5110XXBLH4)";
const char S18342[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5110 Build/IMM76L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S18343[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.3.5-95 (Android 4.0.4; samsung GT-P5110 Build/IMM76D.P5110XXBLH4)";
const char S18344[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5110 Build/IMM76D) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 baidubrowser/061_1.41.3.2_diordna_257_0821/gnusmas_51_4.0.4_0115P-TG/Xf/40C92E4CB5D423DXF0E2C882FDC%7C0/1";
const char S18345[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; GT-P5110 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S18346[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; GT-P5110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MantanoReader";
const char S18347[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 NAVER(inapp; cafe; 100; 2.9.10) com.nhn.android.navercafe/2.9.10 (Linux; U; Android 4.0.3; ko-kr; GT-P5110 Build/IML74K; samsung) 800X1232 samsung GT-P5110";
const char S18348[] = "BBC News (AndroidApp; 1.1; 9) GT-P5110 (Android 4.0.3, SDK 15)";
const char S18349[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5110urniture.co.uk%2pple26ei%3DaNlEUajNGsSJ7Ab3r4HYDQ%26usg%3DAFQjCNEKOKin3_wddMYo8ZfbgmAjmxb4jg%26bvm%3Dbv.X%2Cd.ZGU%22%2C%22landing%22%3A%22http%3A//www.harveysfurniture.co.uk/mobile/home.aspx%22%2C%22time%22%3AX%7D%5D%2C%22sessionLandingPage%22%3A%22http%3A//www.harveysfurniture.co.uk/mobile/home.aspx%22%2C%22__v%22%3A%7B%7D%7D; _qubitTracker=X.X; _qst=%5B2%2C0%5D; qb_etc_status=%7B%22c%22%3A%7B%7D%7D; qb_ss_";
const char S18350[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5110 Build/IMM76D) AppleWebKit/534.UajNGsSJ7Ab3r4HYGeck6usg%3DAFQjCNEKOKin3_wddMYo8ZfbgmAjmxb4jg%26bvm%3Dbv.X%2Cd.ZGU%22%2C%22landing%22%3A%22http%3A//www.harveysfurniture.furniture.co.uk%e.as%26ei%3DaNlEUajNGsSJ7Ab3r4HYDQ%26usg%3DAFQjCNEKOKin3_wddMYo8ZfbgmAjmxb4jg%26bvm%3Dbv.X%2Cd.ZGU%22%2C%22landing%22%3A%22http%3A//www.harveysfurniture.co.uk/mobile/home.aspx%22%2C%22time%22%3AX%7D%5D%2C%22sessionLandingPage%22%3A%22htt";
const char S18351[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S18352[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S18353[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S18354[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S18355[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S18356[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S18357[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36,gzip(gfe)";
const char S18358[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.00";
const char S18359[] = "Opera/8.00 (Windows NT 5.1; U; en)";
const char S18360[] = "Opera/8.01 (Windows NT 5.1)";
const char S18361[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; ru) Opera 8.01";
const char S18362[] = "Mozilla/5.0 (Windows NT 5.1; U; en) Opera 8.01";
const char S18363[] = "Mozilla/5.0 (Windows NT 5.1; U; en) Opera 8.00";
const char S18364[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.0";
const char S18365[] = "Opera/8.50 (Windows NT 5.1; U; en)";
const char S18366[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.02";
const char S18367[] = "Opera/8.xx (Windows NT 5.1; U; en)";
const char S18368[] = "Opera/8.53 (Windows NT 5.1; U; de)";
const char S18369[] = "Opera/8.52 (Windows NT 5.1; U; en)";
const char S18370[] = "Opera/8.51 (Windows NT 5.1; U; en)";
const char S18371[] = "Opera/8.54 (Windows NT 5.1; U; ru)";
const char S18372[] = "Opera/8.10 (Windows NT 5.1; U; en)";
const char S18373[] = "Opera/8.01 (Windows NT 5.1; U; en)";
const char S18374[] = "Opera/8.02 (Windows NT 5.1; U; en)";
const char S18375[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.0	Opera 8.0";
const char S18376[] = "Opera/8.0 (Windows NT 5.1; U; en)";
const char S18377[] = "Mozilla/5.0 (Windows NT 5.1; U; en) Opera 8.02";
const char S18378[] = "Opera/8.53 (Windows NT 5.2; U; en)";
const char S18379[] = "Mozilla/5.0 (Windows NT 5.1; U; en) Opera 8.0";
const char S18380[] = "Opera/8.40 (Windows NT 5.1; U; pl) Version/10.8";
const char S18381[] = "Opera/8.20 (Windows NT 5.1; U; pl) Version/10.8";
const char S18382[] = "Opera/8.20 (Windows NT 5.1; U; pl) Presto/2.2.15 Version/10.10";
const char S18383[] = "Opera/8.40 (Windows NT 5.1; U; pl) Presto/2.4.15 Version/10.10";
const char S18384[] = "Opera/8.10 (Windows NT 5.1; U; pl) Version/10.8";
const char S18385[] = "Opera/8.30 (Windows NT 5.1; U; pl) Version/10.8";
const char S18386[] = "Opera/8.10 (Windows NT 5.1; U; pl) Presto/2.1.15 Version/10.10";
const char S18387[] = "Opera/8.30 (Windows NT 5.1; U; pl) Presto/2.3.15 Version/10.10";
const char S18388[] = "('Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.0', 'Opera 8.0, Windows XP')";
const char S18389[] = "Opera/8.50 (Windows NT 5.2; U; ru)";
const char S18390[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Maxthon)";
const char S18391[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Maxthon; .NET CLR 1.1.4322)";
const char S18392[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Maxthon)";
const char S18393[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Maxthon; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.1)";
const char S18394[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322) Maxthon";
const char S18395[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Maxthon; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Embedded Web Browser from: http://bsalsa.com/; .NET CLR 1.1.4322; FDM; .NET CLR 2.0.50727)";
const char S18396[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; CNCDialer; Maxthon; .NET CLR 1.1.4322; .NET CLR 2.0.50727)";
const char S18397[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; MAXTHON)";
const char S18398[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; SV1; Maxthon; .NET CLR 1.1.4322; .NET CLR 2.0.50727)";
const char S18399[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MAXTHON)";
const char S18400[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Maxthon; .NET CLR 1.1.4322)";
const char S18401[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; SV1; MAXTHON)";
const char S18402[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; MAXTHON)";
const char S18403[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Maxthon; .NET CLR 2.0.50727)";
const char S18404[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; MAXTHON)";
const char S18405[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; MAXTHON)";
const char S18406[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Maxthon; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30)";
const char S18407[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Maxthon; Maxthon)";
const char S18408[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Maxthon; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S18409[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.1; SV1; MAXTHON)";
const char S18410[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/4.0; MAXTHON)";
const char S18411[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)";
const char S18412[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAXTHON)";
const char S18413[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAXTHON)";
const char S18414[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 5.1; Maxthon)";
const char S18415[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Maxthon)";
const char S18416[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; Maxthon)";
const char S18417[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Maxthon; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S18418[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)";
const char S18419[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; Maxthon)";
const char S18420[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; Maxthon)";
const char S18421[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; Maxthon)";
const char S18422[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Maxthon)";
const char S18423[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; Maxthon)";
const char S18424[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 5.2; Maxthon)";
const char S18425[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAXTHON)";
const char S18426[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; MAXTHON)";
const char S18427[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MAXTHON)";
const char S18428[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MAXTHON)";
const char S18429[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; SV1; MAXTHON)";
const char S18430[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; Maxthon)";
const char S18431[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Maxthon; Maxthon; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S18432[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)";
const char S18433[] = "Mozilla/4.0(compatible; MSIE 8.0; Windows NT 5.2; Maxthon; .NET CLR 1.1.4322; .NET CLR 2.0.50727)";
const char S18434[] = "Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 5.1; Maxthon)";
const char S18435[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; Maxthon; Avant Browser)";
const char S18436[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; SV1; Maxthon)";
const char S18437[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Avant Browser; Avant Browser; Maxthon; .NET CLR 2.0.50727; .NET CLR 1.1.4322)";
const char S18438[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; SV1; MAXTHON)";
const char S18439[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Maxthon; SV1)";
const char S18440[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; EmbeddedWB 14.52 from: http://www.bsalsa.com/ EmbeddedWB 14.52; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C; .NET4.0E; MAXTHON)";
const char S18441[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; Maxthon; Maxthon)";
const char S18442[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)";
const char S18443[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)";
const char S18444[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Maxthon; SV1)";
const char S18445[] = "Mozilla/4.0   (compatible;   MSIE   7.0;   Windows   NT   5.1;   SV1;   Maxthon;   .NET   CLR   2.0.50727;   .NET   CLR   3.0.04506.30)";
const char S18446[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MAXTHON)";
const char S18447[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; WOW64; SV1; Maxthon)";
const char S18448[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; Maxthon; Maxthon)";
const char S18449[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; Avant Browser; Maxthon)";
const char S18450[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; WebSaver; Maxthon; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022)";
const char S18451[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; WebSaver; Maxthon; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022)";
const char S18452[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; Maxthon; Avant Browser)";
const char S18453[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; Avant Browser; Maxthon)";
const char S18454[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; Maxthon; SV1)";
const char S18455[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; SV1; Maxthon)";
const char S18456[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; Maxthon; Avant Browser)";
const char S18457[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Maxthon; Maxthon)";
const char S18458[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Maxthon; Avant Browser)";
const char S18459[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; Maxthon; Avant Browser)";
const char S18460[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; SV1; Maxthon)";
const char S18461[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; Maxthon; Maxthon)";
const char S18462[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Maxthon; Maxthon)";
const char S18463[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; Maxthon; Maxthon)";
const char S18464[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MAXTHON)";
const char S18465[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Avant Browser; Maxthon)";
const char S18466[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; Maxthon; Maxthon)";
const char S18467[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; SV1; Maxthon)";
const char S18468[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Maxthon; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; Avant Browser)";
const char S18469[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; Maxthon; Maxthon)";
const char S18470[] = "W3C_Validator/1.1";
const char S18471[] = "W3C_Validator/1.432.2.22";
const char S18472[] = "W3C_Validator/1.2";
const char S18473[] = "W3C_Validator/1.3";
const char S18474[] = "Python-urllib/2.7 AppEngine-Google; (+http://code.google.com/appengine; appid: s~hr-pulsesubscriber)";
const char S18475[] = "Python-urllib/2.5 AppEngine-Google; (+http://code.google.com/appengine; appid: ggghelper)";
const char S18476[] = "Readability/e86230 - http://readability.com/about/";
const char S18477[] = "Readability/b27ecf - http://readability.com/about/";
const char S18478[] = "Readability/348c87 - http://readability.com/about/";
const char S18479[] = "Readability/b32b87 - http://readability.com/about/";
const char S18480[] = "Python-urllib/1.17 AppEngine-Google; (+http://code.google.com/appengine; appid: s~hr-pulsesubscriber)";
const char S18481[] = "heimoicom/1.0 (+http://heimoicom.appspot.com/) AppEngine-Google; (+http://code.google.com/appengine; appid: s~heimoisrv)";
const char S18482[] = "W3C_Validator/1.3 http://validator.w3.org/services";
const char S18483[] = "Readability/4824a0 - http://readability.com/about/";
const char S18484[] = "Mozilla/5.0 AppEngine-Google; (+http://code.google.com/appengine; appid: s~ziin-me)";
const char S18485[] = "Readability/bdc18f - http://readability.com/about/";
const char S18486[] = "Readability/4aa255 - http://readability.com/about/";
const char S18487[] = "Readability/36a234 - http://readability.com/about/";
const char S18488[] = "Readability/dbf201 - http://readability.com/about/";
const char S18489[] = "Readability/744cab - http://readability.com/about/";
const char S18490[] = "Readability/95e8c2 - http://readability.com/about/";
const char S18491[] = "Tiny Tiny RSS/1.9 (http://tt-rss.org/)";
const char S18492[] = "Readability/38b3ab - http://readability.com/about/";
const char S18493[] = "Readability/b5bf8e - http://readability.com/about/";
const char S18494[] = "Readability/73c8c6 - http://readability.com/about/";
const char S18495[] = "Tiny Tiny RSS/1.10 (http://tt-rss.org/)";
const char S18496[] = "Readability/7a48f3 - http://readability.com/about/";
const char S18497[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_PN071 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18498[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC PN071 Build/JZO54K)";
const char S18499[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC One Build/JZO54K)";
const char S18500[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18501[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTCONE Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18502[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; HTC_One/1.28.111.6 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18503[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC_One/1.28.161.7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18504[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTCONE Build/JZO54K)";
const char S18505[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC_PN071 Build/JZO54K)";
const char S18506[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; HTC One 801e Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18507[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC One 801e Build/JZO54K)";
const char S18508[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18509[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_PN07120/1.26.502.6 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18510[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_PN07120/1.26.502.10 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18511[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; HTC_One/1.29.163.11 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18512[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; HTC_One/1.28.163.3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18513[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC_One/1.29.161.11 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18514[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; HTC_PN071-orange-LS/1.28.73.7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18515[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; HTC_One/1.29.111.11 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18516[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; HTC_PN071-orange-LS/1.28.75.8 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18517[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_PN07120/1.26.502.12 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18518[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; HTC_One/1.29.111.16 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18519[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; HTC One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18520[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC_One/1.29.161.16 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18521[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; HTC_One/1.29.161.17 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18522[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; HTC_PN071-orange-LS/1.28.73.10 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18523[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; HTC_PN071-orange-LS/1.28.75.10 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18524[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_PN07120/1.21.502.1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18525[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; HTC_PN071-orange-LS/1.28.75.11 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18526[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; HTC_PN071-orange-LS/1.28.69.8 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18527[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; HTC_One/1.29.163.16 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18528[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; HTC_PN071-orange-LS/1.28.69.10 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18529[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_PN07120/1.26.502.15 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18530[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; HTCONE Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18531[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; One Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18532[] = "BlackBerry9300/5.0.0.955 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102";
const char S18533[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S18534[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S18535[] = "Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S18536[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36,gzip(gfe)";
const char S18537[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36,gzip(gfe)";
const char S18538[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/29.0.1600.1 Chrome/29.0.1600.1 Safari/537.36";
const char S18539[] = "Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S18540[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Iron/29.0.1600.1 Chrome/29.0.1600.1 Safari/537.36";
const char S18541[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5313e Safari/7534.48.3";
const char S18542[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5274d Safari/7534.48.3";
const char S18543[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5288d Safari/7534.48.3";
const char S18544[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5302b Safari/7534.48.3";
const char S18545[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3";
const char S18546[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A404 Safari/7534.48.3";
const char S18547[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A402 Safari/7534.48.3";
const char S18548[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5220p Safari/7534.48.3";
const char S18549[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A406 Safari/7534.48.3";
const char S18550[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5248d Safari/7534.48.3";
const char S18551[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A332 Safari/7534.48.3";
const char S18552[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3";
const char S18553[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9Z999 Safari/7534.48.3";
const char S18554[] = "Mozilla/5.0 (iPhone; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.46";
const char S18555[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7534.48.3";
const char S18556[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3";
const char S18557[] = "Mozilla/5.0 (; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3";
const char S18558[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9Z999 Safari/7534.48.3";
const char S18559[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S18560[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari Version/5.1";
const char S18561[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.32";
const char S18562[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Safari Version/5.1";
const char S18563[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3";
const char S18564[] = "Mozilla/5.0 (iPhone; iPhone OS 5_0) AppleWebKit/535.7 (KHTML, like Gecko) Version/5.1 Mobile Safari/7354.48";
const char S18565[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X; en) AppleWebKit/528.18 (KHTML, like Gecko) Version/5.1 Mobile/7A341 Safari/528.16";
const char S18566[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/by alpden Safari/7534.48.3";
const char S18567[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B208 Safari/7534.48.3";
const char S18568[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safar";
const char S18569[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S18570[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S18571[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S18572[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S18573[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S18574[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S18575[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S18576[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S18577[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S18578[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe)";
const char S18579[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S18580[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S18581[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S18582[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S18583[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S18584[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S18585[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S18586[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S18587[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; BOLT/2.800) AppleWebKit/534.6 (KHTML, like Gecko) Version/5.0 Safari/534.6.3";
const char S18588[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S18589[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S18590[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S18591[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; BOLT/2.802) AppleWebKit/534.6 (KHTML, like Gecko) Version/5.0 Safari/534.6.3";
const char S18592[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S18593[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/535.3 (KHTML, like Gecko) Version/5.0.1 Safari/535.3";
const char S18594[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S18595[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S18596[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.12 (KHTML, like Gecko) Version/5.0.1 Safari/534.12";
const char S18597[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S18598[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.34 (KHTML, like Gecko) Version/5.0 Dooble/1.31 Safari/534.34";
const char S18599[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S18600[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S18601[] = "Mozilla/5.0 (Windows NT 5.1; en-US) AppleWebKit/535.12 (KHTML, like Gecko) Version/5.0.1 Safari/535.12";
const char S18602[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S18603[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S18604[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S18605[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S18606[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S18607[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S18608[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S18609[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S18610[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/531.1 (KHTML, like Gecko) FlyFlow/2.3 Version/5.0 Safari/531.1";
const char S18611[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; LG-E400-Orange/V10d Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18612[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; LG-E400/V10g Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18613[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; LG-E400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18614[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; LG-E400-Orange/V10g Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18615[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; LG-E400/V10e Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18616[] = "LG-E400 Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; E400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18617[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-E400 Build/GRK39F)";
const char S18618[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; LG-E400f Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18619[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; LG-E400/V10d Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18620[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-sa; LG-E400g Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18621[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; LG-E400f V10b Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18622[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; LG-E400-Orange/V10k Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18623[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; sk-sk; LG-E400/V10c Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18624[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; LG-E400/V10h Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18625[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; LG-E400/V10k Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18626[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-E400f Build/GRK39F)";
const char S18627[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hr-hr; LG-E400/V10b Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18628[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; LG-E400f V10d Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18629[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; LG-E400R Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18630[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; LG-E400-Orange/V10m Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18631[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; LG-E400/V10i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18632[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; el-gr; LG-E400/V10f Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18633[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; LG-E400/V10m Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18634[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; LG-E400/V10n Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18635[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; LG-E400/V10j Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18636[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; LG-E400b-parrot Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18637[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-E400g Build/GRK39F)";
const char S18638[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; LG-E400/V10p Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18639[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; LG-E400-Orange/V10p Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18640[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; LG-E400g Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18641[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; LG-E400b Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18642[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; LG-E400-Orange/V10i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18643[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en; LG-E400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S18644[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-E400R Build/GRK39F)";
const char S18645[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S18646[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S18647[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S18648[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/535.3 (KHTML, like Gecko) Chrome/16.0.883.0 Safari/535.3";
const char S18649[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7,gzip(gfe) (via translate.google.com)";
const char S18650[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7";
const char S18651[] = "Mozilla/5.0 Windows NT 5.1 AppleWebKit/535.7 KHTML, like Gecko Chrome/16.0.912.0 Safari/535.7";
const char S18652[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/535.3 (KHTML, like Gecko) Chrome/16.0.883.0 Safari/535.3,gzip(gfe) (via translate.google.com)";
const char S18653[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.904.0 Safari/535.7 AppEngine-Google; (+http://code.google.com/appengine; appid: s~daisyw110)";
const char S18654[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/535.3 (KHTML, like Gecko) Chrome/16.0.883.0 Safari/535.3";
const char S18655[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.910.20417 KChrome/3.2.5.20417  Safari/535.7";
const char S18656[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7,gzip(gfe)";
const char S18657[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.6 (KHTML	 like Gecko) Chrome/16.0.899.0 Safari/535.6";
const char S18658[] = "Mozilla/5.0+(Windows+NT+5.1)+AppleWebKit/535.7+(KHTML,+like+Gecko)+Chrome/16.0.912.77+Safari/535.7";
const char S18659[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML like Gecko) Chrome/16.0.912.75 Safari/535.7 CoolNovo/2.0.0.9";
const char S18660[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.904.0 Safari/535.7,gzip(gfe) (via docs.google.com/viewer)";
const char S18661[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/535.3 (KHTML, like Gecko) Chrome/16.0.883.0 Safari/535.3,gzip(gfe)";
const char S18662[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Galaxy Nexus Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18663[] = "Dalvik/1.6.0 (Linux; U; Android 4.2; Galaxy Nexus Build/JOP40C)";
const char S18664[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18665[] = "Mozilla/5.0 (Linux; U; Android 4.2/4.1.2; en-us; Galaxy Nexus Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18666[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; Galaxy Nexus Build/JOP40D)";
const char S18667[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18668[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Galaxy Nexus Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18669[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18670[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Galaxy Nexus Build/JOP40D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S18671[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JOQ10A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18672[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18673[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JB_MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18674[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18675[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18676[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; Galaxy Nexus Build/JDQ39)";
const char S18677[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18678[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; Galaxy Nexus Build/JDQ39) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S18679[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18680[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-; Galaxy Nexus Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18681[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-; Galaxy Nexus Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18682[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18683[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JB_MR1.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18684[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JER09) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18685[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JWR66N) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18686[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.19 (KHTML, like Gecko) Chrome/11.0.661.0 Safari/534.19";
const char S18687[] = "Mozilla/5.0 (Windows; Windows NT 5.1) AppleWebKit/534.23 (KHTML, like Gecko) Chrome/11.0.686.3 Safari/534.23";
const char S18688[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.60 Safari/534.24,gzip(gfe) (via translate.google.com)";
const char S18689[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.60 Safari/534.24";
const char S18690[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S18691[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S18692[] = "Mozilla/5.0 (Windows; Windows NT 5.2) AppleWebKit/534.23 (KHTML, like Gecko) Chrome/11.0.686.3 Safari/534.23";
const char S18693[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko; PTST 2.284) Chrome/11.0.696.68 Safari/534.24";
const char S18694[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.17 (KHTML, like Gecko) Chrome/11.0.652.0 Safari/534.17";
const char S18695[] = "Mozilla/5.0 (Windows; Windows NT 5.1) AppleWebKit/534.23 (KHTML, like Gecko) Chrome/11.0.686.3 Safari/534.23,gzip(gfe) (via translate.google.com)";
const char S18696[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.60 Safari/534.24,gzip(gfe) (via docs.google.com/viewer)";
const char S18697[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.60 Safari/534.24, SurfCop 1.5";
const char S18698[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.14 (Chromium Portable 9.2)";
const char S18699[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML	 like Gecko) Chrome/11.0.696.77 Safari/534.24";
const char S18700[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.60 Safari/534.24,gzip(gfe)";
const char S18701[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; de-DE) AppleWebKit/534.19 (KHTML, like Gecko) Chrome/11.0.661.0 Safari/534.19";
const char S18702[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML like Gecko) Chrome/11.0.696.71 Safari/534.24";
const char S18703[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko; PTST 2.280) Chrome/11.0.696.65 Safari/534.24";
const char S18704[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.17 (KHTML like Gecko) Chrome/11.0.652.0 Safari/534.17";
const char S18705[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S18706[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S18707[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S18708[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe) (via translate.google.com)";
const char S18709[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe) (via docs.google.com/viewer)";
const char S18710[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S18711[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe)";
const char S18712[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.11 (KHTML like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S18713[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8G4 Safari/6533.18.5";
const char S18714[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F190 Safari/6533.18.5";
const char S18715[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5";
const char S18716[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8K2 Safari/6533.18.5";
const char S18717[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5";
const char S18718[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5";
const char S18719[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9Z999 Safari/6533.18.5";
const char S18720[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S18721[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S18722[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 433 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5";
const char S18723[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 432 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5";
const char S18724[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 43 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8F190 Safari/6533.18.5";
const char S18725[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 435 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5";
const char S18726[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5";
const char S18727[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 431 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8G4 Safari/6533.18.5";
const char S18728[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 434 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8K2 Safari/6533.18.5";
const char S18729[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_1 like Mac OS X; es-es) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S18730[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S18731[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S18732[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S18733[] = "Microsoft Office/15.0 (Windows NT 6.1; Microsoft Outlook 15.0.4128; Pro)";
const char S18734[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S18735[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4128; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S18736[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S18737[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S18738[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S18739[] = "Microsoft Office/15.0 (Windows NT 6.1; Microsoft Outlook 15.0.4420; Pro)";
const char S18740[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S18741[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S18742[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S18743[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S18744[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S18745[] = "Microsoft Office/15.0 (Windows NT 6.1; Microsoft Outlook 15.0.4454; Pro)";
const char S18746[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S18747[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S18748[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S18749[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S18750[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S18751[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S18752[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MDDRJS; Microsoft Outlook 15.0.4454; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S18753[] = "Microsoft Office/15.0 (Windows NT 6.1; Microsoft Outlook 15.0.4481; Pro)";
const char S18754[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S18755[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S18756[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S18757[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S18758[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S18759[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S18760[] = "Microsoft Office/15.0 (Windows NT 6.1; Microsoft Outlook 15.0.4433; Pro)";
const char S18761[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S18762[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)";
const char S18763[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S18764[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S18765[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S18766[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S18767[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S18768[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S18769[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S18770[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S18771[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S18772[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S18773[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S18774[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S18775[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S18776[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S18777[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S18778[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S18779[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S18780[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S18781[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S18782[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)";
const char S18783[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S18784[] = "Microsoft Office/15.0 (Windows NT 6.1; Microsoft Outlook 15.0.4505; Pro)";
const char S18785[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S18786[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S18787[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S18788[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S18789[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S18790[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4128; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)";
const char S18791[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S18792[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4433; ms-office; MSOffice 15)";
const char S18793[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S18794[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S18795[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S18796[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S18797[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S18798[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S18799[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18800[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18801[] = "Microsoft Office/15.0 (Windows NT 6.1; Microsoft Outlook 15.0.4517; Pro)";
const char S18802[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18803[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18804[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18805[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S18806[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S18807[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18808[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18809[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18810[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18811[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18812[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S18813[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)";
const char S18814[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S18815[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4433; Microsoft Outlook 15.0.4433; ms-office; MSOffice 15)";
const char S18816[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18817[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18818[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18819[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18820[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18821[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18822[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18823[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18824[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18825[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Win64; x64; Trident/6.0; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18826[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18827[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18828[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Win64; x64; Trident/6.0; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18829[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S18830[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18831[] = "Microsoft Office/15.0 (Windows NT 6.1; Microsoft Outlook 15.0.4535; Pro)";
const char S18832[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S18833[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S18834[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4535; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S18835[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18836[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18837[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18838[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S18839[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S18840[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4535; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S18841[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S18842[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)";
const char S18843[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S18844[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4535; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S18845[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4535; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)";
const char S18846[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C5101c";
const char S18847[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C134";
const char S18848[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/X.X (KHTML, like Gecko) Version/X.X Mobile/XXXX Safari/X.X";
const char S18849[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148";
const char S18850[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S18851[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148a";
const char S18852[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2 like Mac OS X; en_us) AppleWebKit/525.18.1 (KHTML, like Gecko)";
const char S18853[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_7 like Mac OS X; en_us) AppleWebKit/525.18.1 (KHTML, like Gecko)";
const char S18854[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S18855[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/533.17.9";
const char S18856[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148a Safari/533.17.9";
const char S18857[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S18858[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E200";
const char S18859[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S18860[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E128";
const char S18861[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E200 Safari/533.17.9";
const char S18862[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_7 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E303";
const char S18863[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_7 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S18864[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S18865[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E401";
const char S18866[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E401 Safari/533.17.9";
const char S18867[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_10 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E600 Safari/533.17.9";
const char S18868[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_9 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S18869[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_9 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E501";
const char S18870[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_10 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E600";
const char S18871[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_10 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S18872[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/9Z999";
const char S18873[] = "Mozilla/5.0 (; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148";
const char S18874[] = "Miu Ptt 1.5.3 (iPhone; iPhone OS 4.2.1; zh_TW)";
const char S18875[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,3;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.2.6;FBSS/2; FBCR/Verizon;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S18876[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8C148";
const char S18877[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_5 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,3;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.2.5;FBSS/2; FBCR/Verizon;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S18878[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_10 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,3;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.2.10;FBSS/2; FBCR/Verizon;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S18879[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,3;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.2.8;FBSS/2; FBCR/Verizon;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S18880[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone2,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.2.1;FBSS/1; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/1.0]";
const char S18881[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_7 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,3;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.2.7;FBSS/2; FBCR/Verizon;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S18882[] = "NaverCafe/1.0 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName";
const char S18883[] = "Mozilla/5.0 (device; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148";
const char S18884[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_9 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,3;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.2.9;FBSS/2; FBCR/Verizon;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S18885[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 iPad1,1 BingWeb/3.03.1428.X";
const char S18886[] = "Mozilla/5.0 (iPhone; CPU OS 4_2 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/8C148 Safari/7534.48 Instapaper/4.0.4 (+http://www.instapaper.com/iphone)";
const char S18887[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C134b";
const char S18888[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2 like Mac OS X; nl_NL) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.2;FBSS/2; FBCR/vfnl;FBID/phone;FBLC/nl_NL;FBSF/2.0]";
const char S18889[] = "Mozilla/5.0 (iPhone; U; CPU iPhone 4_2_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPhone1,2/4.2.1)";
const char S18890[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper";
const char S18891[] = "Mozilla/5.0 (iPhone; U; CPU iPhone 4_2_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPad1,1/4.2.1)";
const char S18892[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18893[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9070 Build/GINGERBREAD)";
const char S18894[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-I9070/I9070BULD1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18895[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; GT-I9070P Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18896[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070P-ORANGE/I9070PBVLD3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18897[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-I9070/I9070BULE2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18898[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070P-TMUK/I9070PBVLD3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18899[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-I9070P/I9070PBOLE2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18900[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9070 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S18901[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070P-ORANGE/I9070PBVLF1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18902[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; SAMSUNG GT-I9070/I9070BOLE1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18903[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9070P Build/GINGERBREAD)";
const char S18904[] = "Mozilla/5.0 (Linux; U; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S18905[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; SAMSUNG GT-I9070/I9070XXLF4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18906[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18907[] = "Mozilla/5.0 (Linux; U; Android 2.3.6_mobile-star; iw-il; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18908[] = "Mozilla/5.0 (Linux; U; es-es; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S18909[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-I9070P-ORANGE/I9070PBVLG1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18910[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-I9070P/I9070PBOLK1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18911[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; SAMSUNG GT-I9070/I9070BOLK1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18912[] = "Mozilla/5.0 (Linux; U; it-it; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S18913[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-I9070/I9070BULK1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18914[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; SAMSUNG GT-I9070/I9070XXLK1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18915[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; SAMSUNG GT-I9070/I9070XXLK2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18916[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070P-TMUK/I9070PBVLF1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18917[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-I9070P-ORANGE/I9070PBVLK2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18918[] = "Mozilla/5.0 (Linux; U; de-de; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S18919[] = "Mozilla/5.0 (Linux; U; de-ch; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S18920[] = "Mozilla/5.0 (Linux; U; fr-ch; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S18921[] = "Mozilla/5.0 (Linux; U; Android 2.3.6_mobile-star; iw-; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S18922[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.91 Mobile/10B5105c Safari/8536.25";
const char S18923[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.91 Mobile/10B5117b Safari/8536.25";
const char S18924[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B5126b Safari/8536.25";
const char S18925[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B144 Safari/8536.25";
const char S18926[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B141 Safari/8536.25";
const char S18927[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B142 Safari/8536.25";
const char S18928[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B143 Safari/8536.25";
const char S18929[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B145 Safari/8536.25";
const char S18930[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B146 Safari/8536.25";
const char S18931[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B329 Safari/8536.25";
const char S18932[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B350 Safari/8536.25";
const char S18933[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18934[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-L710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18935[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E210K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18936[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I747M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18937[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E210S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18938[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E210L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18939[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SPH-L710 Build/IMM76D)";
const char S18940[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-R530U Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18941[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D)";
const char S18942[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I747 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18943[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-R530U Build/IMM76D)";
const char S18944[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I747M Build/IMM76D)";
const char S18945[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHV-E210L Build/IMM76D)";
const char S18946[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I747M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S18947[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHV-E210S Build/IMM76D)";
const char S18948[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SPH-L710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18949[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-L710 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18950[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SGH-I747M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18951[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHV-E210K Build/IMM76D)";
const char S18952[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I747 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S18953[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-L710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S18954[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18955[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I747M Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S18956[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S18957[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Galaxy S III Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18958[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-R530C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18959[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-R530C Build/IMM76D)";
const char S18960[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SHV-E210L Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S18961[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SHV-E210S Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S18962[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.0.4;dev=d2att;devm=SAMSUNG-SGH-I747;man=samsung;brw=Android;shwd=720x1280;shwa=720x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S18963[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I747 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18964[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC One V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18965[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC One V Build/IML74K)";
const char S18966[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; T320a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18967[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_V/1.65.161.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18968[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC One V Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S18969[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC-ADR6290US Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18970[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; T320a-parrot Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18971[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_V/2.09.161.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18972[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; HTC_One_V-orange-LS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18973[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_T120C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18974[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_One_V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18975[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; HTC_One_V/1.61.113.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18976[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_One_V/1.61.111.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18977[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC_One_V/2.09.166.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18978[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC One V Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S18979[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; HTC_One_V/1.61.114.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18980[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; HTC_One_V/2.09.114.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18981[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; HTC_One_V/2.09.113.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18982[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_One_V/2.09.111.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18983[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC_One_V/1.65.166.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18984[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; HTC_One_V/2.22.114.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18985[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC_One_V/2.22.166.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18986[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_One_V/2.22.111.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18987[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; HTC_One_V/2.22.113.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18988[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_One_V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18989[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_V/2.22.161.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18990[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_V/2.22.161.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18991[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; HTC_One_V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18992[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-mx; HTC ONE V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18993[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC One V Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18994[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; HTC One V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18995[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_One_V Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18996[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneV Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18997[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; HTC_One_V/2.22.161.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18998[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_One_V/2.22.111.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S18999[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC One V Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19000[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; LT15i Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19001[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; SonyEricssonLT15iv Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19002[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; sv-se; SonyEricssonLT15i Build/3.0.A.2.52) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19003[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; SonyEricssonLT15i Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19004[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; SonyEricssonLT15i Build/3.0.A.0.326) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19005[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; SonyEricssonLT15i-o Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19006[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; sv-se; SonyEricssonLT15i Build/3.0.A.2.182) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19007[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; sv-se; SonyEricssonLT15i Build/3.0.A.2.179) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19008[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; sv-se; SonyEricssonLT15i Build/3.0.A.2.91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19009[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; LT15i Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19010[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; SonyEricssonLT15i Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19011[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonLT15i Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19012[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; LT15i Build/3.0.1.A.0.150) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19013[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonLT15i-o Build/3.0.1.A.0.150) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19014[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; SonyEricssonLT15i Build/3.0.1.A.0.151) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19015[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; LT15i Build/3.0.1.A.0.151) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19016[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT15i Build/4.0.1.A.0.239) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19017[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; LT15i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19018[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonLT15i Build/4.0.1.A.0.227) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19019[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonLT15i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19020[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT15i Build/4.0.1.A.0.203) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19021[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ie; SonyEricssonLT15iv Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19022[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonLT15i Build/4.0.A.2.377) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19023[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ar-; SonyEricssonLT15i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19024[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT15i Build/4.0.1.A.0.150) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19025[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-za; SonyEricssonLT15 Build/3.0.A.2.181_R2A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19026[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT15i Build/4.0.1.A.0.266) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19027[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT15i Build/4.0.1.A.0.254) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19028[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT15i Build/4.0.1.A.0.277) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19029[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT15i Build/4.0.2.A.0.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19030[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonLT15iv Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19031[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonLT15i Build/3.0.1.A.0.100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19032[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; pl-pl; SonyEricssonLT15i-o Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19033[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; LT15i Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19034[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; LT15i Build/3.0.1.A.0.146) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19035[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; sv-se; SonyEricssonLT15i Build/3.0.J.2.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19036[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonLT15i Build/4.0.A.0.263) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19037[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonLT15i-o Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19038[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; SonyEricssonLT15iv Build/3.0.1.A.0.146) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19039[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; SonyEricssonLT15i Build/3.0.1.A.0.146) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19040[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonLT15i Build/4.0.A.2.100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19041[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; sv-se; SonyEricssonLT15i Build/3.0.A.2.119) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19042[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonLT15i Build/4.0.A.2.357) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19043[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonLT15i Build/4.0.A.2.362) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19044[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonLT15i Build/4.0.A.2.305) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19045[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT15i Build/private) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19046[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; SonyEricssonLT15 Build/3.0.A.2.181_R1A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19047[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT15i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19048[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT15i Build/4.0.1.A.0.217) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19049[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT15i Build/4.0.1.A.0.284) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19050[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; SonyEricssonLT15iv Build/3.0.A.0.326) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19051[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LT15i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19052[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; SonyEricssonLT15i Build/4.0.A.2.257) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19053[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT15i Build/4.0.2.A.0.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19054[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; es-es; SonyEricssonLT15i Build/3.0.A.2.118) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19055[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT15i Build/4.0.2.A.0.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19056[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT15i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19057[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonLT15i Build/4.0.A.2.371) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19058[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ro-ro; SonyEricssonLT15i Build/3.0.1.A.0.150) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19059[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonLT15i Build/4.0.A.0.55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19060[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.2; LT15i Build/3.0.A.2.181)";
const char S19061[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; LT15i Build/3.0.1.A.0.145)";
const char S19062[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LT15i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19063[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT15i-o Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19064[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT15iv Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19065[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19066[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; SonyEricssonLT15 Build/3.0.A.2.181_R3B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19067[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonLT15i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19068[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT15i Build/4.0.2.A.0.62) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S19069[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; SonyEricssonLT15iv Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19070[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT15i-o Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19071[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LT15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19072[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LT15i Build/4.0.2.A.0.62)";
const char S19073[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LT15i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19074[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; LT15i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19075[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; LT15i Build/3.0.1.A.0.145) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S19076[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT15i Build/4.0.2.A.0.42) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S19077[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT15i Build/4.0.2.A.0.62) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S19078[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LT15i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19079[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; LT15i Build/4.0.A.2.368) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S19080[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT15i Build/4.0.2.A.0.58) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S19081[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT15i Build/4.0.2.A.0.42) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S19082[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-; SonyEricssonLT15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19083[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; LT15i Build/3.0.1.A.0.145) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S19084[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; nl-nl; LT15i Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19085[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Xperia Arc Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19086[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ko-kr; SonyEricssonLT15i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19087[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Sony Xperia Arc Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19088[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; SonyEricssonLT15i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19089[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonLT15iv Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19090[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S19091[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.106 Safari/535.2,gzip(gfe) (via translate.google.com)";
const char S19092[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2,gzip(gfe) (via docs.google.com/viewer)";
const char S19093[] = "Mozilla/5.0 (Windows NT 5.1) Chrome/15.0.874.121 Safari/535.2";
const char S19094[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S19095[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S19096[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S19097[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S19098[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2 AppEngine-Google; (+http://code.google.com/appengine; appid: s~qiquwanlu1)";
const char S19099[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2,gzip(gfe)";
const char S19100[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/150.1.2.3 Safari/535.1";
const char S19101[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.2 (KHTML like Gecko) Chrome/15.0.872.0 Safari/535.2";
const char S19102[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S19103[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S19104[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16,gzip(gfe) (via translate.google.com)";
const char S19105[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/10.0 Safari/534.7";
const char S19106[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16,gzip(gfe) (via docs.google.com/viewer)";
const char S19107[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16,gzip(gfe)";
const char S19108[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S19109[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.64 Safari/537.31";
const char S19110[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; haw-US) AppleWebKit/534.16 (KHTML like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S19111[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-CA) AppleWebKit/534.16 (KHTML like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S19112[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; es-US) AppleWebKit/534.16 (KHTML like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S19113[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; fr-CA) AppleWebKit/534.16 (KHTML like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S19114[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.64 Safari/537.31";
const char S19115[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; oc-FR) AppleWebKit/534.16 (KHTML like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S19116[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/534.16 (KHTML like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S19117[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.133 Safari/534.16 (via babelfish.yahoo.com)";
const char S19118[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22";
const char S19119[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S19120[] = "Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S19121[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.45 Safari/537.22 AppEngine-Google; (+http://code.google.com/appengine; appid: s~google.com:pssics)";
const char S19122[] = "Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/25.0.1410.63 Safari/537.31";
const char S19123[] = "Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/25.0.1410.63 Safari/537.31";
const char S19124[] = "Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/25.0.1410.63 Safari/537.31";
const char S19125[] = "Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/25.0.1410.63 Safari/537.31";
const char S19126[] = "Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/25.0.1410.63 Safari/537.31";
const char S19127[] = "Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/25.0.1410.63 Safari/537.31";
const char S19128[] = "Mozilla/5.0 (Linux; en-US) AppleWebKit/534.6 (KHTML, like Gecko) Chrome/25.0.500.0 Safari/534.6";
const char S19129[] = "Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/25.0.1410.63 Safari/537.31";
const char S19130[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.32 (KHTML, like Gecko) Chromium/25.0.1349.2 Chrome/25.0.1349.2 Safari/537.32 Debian/7.0 (3.7.91-1) Epiphany/3.7.91";
const char S19131[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)";
const char S19132[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22,gzip(gfe)";
const char S19133[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.32 (KHTML, like Gecko) Chromium/25.0.1349.2 Chrome/25.0.1349.2 Safari/537.32 Debian/unstable (3.8.2-1) Epiphany/3.8.2";
const char S19134[] = "Mozilla/5.0 (X11; Jolicloud Linux i686) AppleWebKit/537.22 (KHTML, like Gecko) Chromium/25.0.1364.160 Chrome/25.0.1364.160 Safari/537.22";
const char S19135[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.32 (KHTML, like Gecko) Chromium/25.0.1349.2 Chrome/25.0.1349.2 Safari/537.32 Debian/7.0 (3.4.2-2.1) Epiphany/3.4.2";
const char S19136[] = "Mozilla/5.0 (X11; Jolicloud Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Chromium/25.0.1364.160 Chrome/25.0.1364.160 Safari/537.22";
const char S19137[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.32 (KHTML, like Gecko) Chromium/25.0.1349.2 Chrome/25.0.1349.2 Safari/537.32 Debian/unstable (3.8.2-3) Epiphany/3.8.2";
const char S19138[] = "Mozilla/4.0 (compatible; MSIE 6.1; Windows XP)";
const char S19139[] = "Mozilla/3.0 (x86 [en] Windows NT 5.1; Sun)";
const char S19140[] = "Mozilla/4.0 (compatible; MSIE 6.5; Windows NT 5.1)";
const char S19141[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1)";
const char S19142[] = "Mozilla/4.0 (compatible; Windows XP 5.1; MSIE 6)";
const char S19143[] = "FeedDemon/3.1 (http://www.feeddemon.com/; Microsoft Windows XP)";
const char S19144[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.14)";
const char S19145[] = "Mozilla/4.0+(compatible;+MSIE+6.0;+Windows+NT+5.1;+SV1;+.NET+CLR+1.1.4322)";
const char S19146[] = "Mozilla 5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.0.11)";
const char S19147[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN; rv:1.9.1.2)";
const char S19148[] = "FeedDemon/2.7 (http://www.newsgator.com/; Microsoft Windows XP)";
const char S19149[] = "Internet Explorer 6 (MSIE 6; Windows XP)";
const char S19150[] = "Mozilla/4.0 (compatible; ArgClrInt/1.0; Windows NT 5.1; Trident/4.0; GTB6.4; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S19151[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US)";
const char S19152[] = "FeedDemon/4.0 (http://www.feeddemon.com/; Microsoft Windows XP)";
const char S19153[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.17pre) Gecko K-Meleon/1.6.0";
const char S19154[] = "Mozilla/4.0 (Windows XP 5.1) Java/1.6.0_24";
const char S19155[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-gb; SiteCon/8.8.14)";
const char S19156[] = "Mozilla/4.0 (Windows; U; Windows NT 5.1; zh-TW; rv:1.9.0.11)";
const char S19157[] = "Mozilla/4.0+(compatible;+MSIE+7.0;+Windows+NT+5.2;+.NET+CLR+1.1.4322;+.NET+CLR+2.0.50727)";
const char S19158[] = "Sunrise/0.42g (Windows XP)";
const char S19159[] = "FeedDemon/3.0 (http://www.feeddemon.com/; Microsoft Windows XP)";
const char S19160[] = "Mozilla/4.0 (compatible; MyApp 1.0; Windows NT 5.1)";
const char S19161[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533+ (KHTML, like Gecko)";
const char S19162[] = "Mozilla/5.0 (Windows NT 5.1)";
const char S19163[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.57.2 (KHTML, like Gecko)";
const char S19164[] = "FeedDemon/4.1 (http://www.feeddemon.com/; Microsoft Windows XP)";
const char S19165[] = "Mozilla/5.0 (Windows NT 5.1; AppleWebKit/535.19 (KHTML, like Gecko; Windows NT 5.1)";
const char S19166[] = "Mozilla/5.0 (compatible; Windows; U; Windows NT 5.1; en-US; rv:1.8.1.2) Gecko/X";
const char S19167[] = "IBM-LC-Base-Notes_8.5.3.X_0921-Windows XP Java/1.6.0";
const char S19168[] = "Mozilla/5.0 (Windows NT 5.1; en-US; rv:1.7.6) Gecko/X";
const char S19169[] = "Mozilla/4.0 (Windows XP 5.1) Java/1.7.0_09";
const char S19170[] = "Mozilla/4.0 (Windows; U; Windows NT 5.1; en-GB; rv:1.9.2.20)";
const char S19171[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; EN; rv:1.9.0.6) Gecko/2";
const char S19172[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.13)";
const char S19173[] = "FeedDemon/4.5 (http://www.feeddemon.com/; Microsoft Windows XP)";
const char S19174[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S19175[] = "Mozilla/5.0 (X11; Jolicloud Linux i686) AppleWebKit/537.6 (KHTML, like Gecko) Joli OS/1.2 Chromium/23.0.1240.0 Chrome/23.0.1240.0 Safari/537.6";
const char S19176[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S19177[] = "Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S19178[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe)";
const char S19179[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe)";
const char S19180[] = "Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/23.0.1410.63 Safari/537.31";
const char S19181[] = "Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/23.0.1410.63 Safari/537.31";
const char S19182[] = "Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/23.0.1410.63 Safari/537.31";
const char S19183[] = "Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/23.0.1410.63 Safari/537.31";
const char S19184[] = "Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/23.0.1410.63 Safari/537.31";
const char S19185[] = "Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/23.0.1410.63 Safari/537.31";
const char S19186[] = "Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/23.0.1410.63 Safari/537.31";
const char S19187[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe) (via docs.google.com/viewer)";
const char S19188[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/23.0.1271.95 Chrome/23.0.1271.95 Safari/537.6+ SUSE/12.3 (3.6.1) Epiphany/3.6.1";
const char S19189[] = "Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/23.0.1271.95 Chrome/23.0.1271.95 Safari/537.6+ SugarLabs/0.98";
const char S19190[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50";
const char S19191[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50";
const char S19192[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22";
const char S19193[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22";
const char S19194[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7";
const char S19195[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16";
const char S19196[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16";
const char S19197[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3";
const char S19198[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3";
const char S19199[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2";
const char S19200[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2";
const char S19201[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7";
const char S19202[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.3 Safari/534.53.10";
const char S19203[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.3 Safari/534.53.10";
const char S19204[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19205[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19206[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; SAMSUNG GT-P3100/P3100BUALD5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19207[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19208[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; GT-P3100B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19209[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P3100 Build/IMM76D)";
const char S19210[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-P3100 Build/IML74K)";
const char S19211[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19212[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; SAMSUNG GT-P3100/P3100BUBLH3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19213[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19214[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; GT-P3100B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19215[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S19216[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P3100-ORANGE/P3100BVBLH1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19217[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19218[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3100 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S19219[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-P3100B Build/IML74K)";
const char S19220[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P3100 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19221[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3100 Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S19222[] = "com.google.android.youtube/3.5.5(Linux; U; Android 4.0.3; en_US; GT-P3100 Build/IML74K) gzip";
const char S19223[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3100 Build/IML74K; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S19224[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S19225[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; hr-hr; SAMSUNG GT-P3100/P3100BUALD1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19226[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19227[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.7 Mobile Safari/534.30";
const char S19228[] = "Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3100 Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S19229[] = "Vyrso/2.1.2 (Linux; U; Android 4.0.4; en-PH; GT-P3100 Build/IMM76D.P3100XXBLE3)";
const char S19230[] = "com.google.android.youtube/4.0.23(Linux; U; Android 4.0.4; en_GB; GT-P3100 Build/IMM76D) gzip";
const char S19231[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build/IMM76D; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S19232[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; GT-P3100 Build/IMM76D) An-ph; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19233[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S19234[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.2.1-65 (Android 4.0.4; samsung GT-P3100 Build/IMM76D.P3100DDBLF3)";
const char S19235[] = "Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S19236[] = "Logos/2.1.2 (Linux; U; Android 4.0.4; en-GB; GT-P3100 Build/IMM76D.P3100DDBLF3)";
const char S19237[] = "BBC iPlayer (AndroidApp; 1.3; 13014) GT-P3100 (Android 4.0.3, SDK 15)";
const char S19238[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/XXLQ1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19239[] = "Vyrso/2.1.2 (Linux; U; Android 4.0.4; en-GB; GT-P3100 Build/IMM76D.P3100XWBLF1)";
const char S19240[] = "Logos/2.2.3 (Linux; U; Android 4.0.4; pt-BR; GT-P3100 Build/IMM76D.P3100UBBLJ1)";
const char S19241[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 VoiceGO!(cyb42f2f9766a5b045ced0d29bX-0;Android,4.0.4,GT-P3100;3.6.0;1024x552,1.0;X;VoiceGo-TC;zh-TW)";
const char S19242[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3100 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19243[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P3100-ORANGE/P3100BVBLF6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19244[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-P3100/P3100BUBLF1.DM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19245[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-P3100/P3100BUBLH3.DM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19246[] = "Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 4.0.4; SAMSUNG GT-P3100 Build/ICE_CREAM_SANDWICH_MR1);";
const char S19247[] = "Logos/2.2.3 (Linux; U; Android 4.0.3; en-US; GT-P3100 Build/IML74K.P3100UBALD3)";
const char S19248[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S19249[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P3100B Build/IMM76D)";
const char S19250[] = "Logos/2.2.3 (Linux; U; Android 4.0.4; en-GB; GT-P3100 Build/IMM76D.P3100DDBLH3)";
const char S19251[] = "Logos/2.2.3 (Linux; U; Android 4.0.4; en-PH; GT-P3100 Build/IMM76D.P3100XXBLH2)";
const char S19252[] = "Logos/2.2.3 (Linux; U; Android 4.0.3; en-US; GT-P3100 Build/IML74K.P3100XXALD4)";
const char S19253[] = "Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 4.0.3; SAMSUNG GT-P3100 Build/ICE_CREAM_SANDWICH_MR1);";
const char S19254[] = "BBC News (AndroidApp; 1.3; 4) GT-P3100 (Android 4.0.3, SDK 15)";
const char S19255[] = "Showtimezz@MY/1.4.8 (Android 4.0.4; samsung-GT-P3100%404.0.4 Build/15";
const char S19256[] = "Scanner Radio/3.8.5 (Android 4.0.4, GT-P3100)";
const char S19257[] = "Logos/2.2.3 (Linux; U; Android 4.0.3; en-GB; GT-P3100 Build/IML74K.P3100XWALD2)";
const char S19258[] = "Biblia/2.2.7 (Linux; U; Android 4.0.4; es-ES; GT-P3100 Build/IMM76D.P3100XWBLF3)";
const char S19259[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ms-my; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/1.7.1;FBPN/com.facebook.katana;FBDV/GT-P3100;FBSV/4.0.3;FBDM/{density=1.0,width=600,height=976};FBCR/MY MAXIS;FBLC/ms_MY;FB_FW/1;]";
const char S19260[] = "MediaPlayer/2002/4.0.12 (Android 4.0.4; espressorf; GT-P3100; samsung; ; ; en_GB)";
const char S19261[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100B Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S19262[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3100B Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19263[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/1.6.4;FBPN/com.facebook.katana;FBDV/GT-P3100;FBSV/4.0.4;FBDM/{density=1.0,width=1024,height=552};FBCR/MY MAXIS;FBLC/en_GB;FB_FW/1;]";
const char S19264[] = "AndroidNative/2.0.30 (Linux; U; Android 4.0.4; en-us; SAMSUNG GT-P3100; Build/IMM76D) tablet; 1024x552; FilmOn";
const char S19265[] = "MediaPlayer/2004/4.0.12 (Android 4.0.4; espressorf; GT-P3100; samsung; ; ; en_US)";
const char S19266[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sr-rs; SAMSUNG GT-P3100/P3100BUBLF2.DM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19267[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S19268[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; GT-P3100 Buioid 4.0.4; en-ph; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19269[] = "BEETmobile Hotspot/1.0.2 (Android 4.0.4, GT-P3100)";
const char S19270[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG GT-P3100/Sungsonic??? RC-01 Built By Adi??? Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19271[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19272[] = "Logos/2.2.8.1 (Linux; U; Android 4.0.4; zh-HK; GT-P3100 Build/IMM76D.P3100ZSBLH4)";
const char S19273[] = "VKAndroidApp/2.7.6-64 (Android 4.0.4; SDK 15; armeabi-v7a; samsung GT-P3100; ru)";
const char S19274[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19275[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P3100-ORANGE/P3100BVBLF4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19276[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG GT-P3100/Sungsonic\" RC-01 Built By Adi\" Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19277[] = "Biblia/2.2.8.1 (Linux; U; Android 4.0.3; es-US; GT-P3100 Build/IML74K.P3100UBALD4)";
const char S19278[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG GT-P3100/SungsonicT RC-01 Built By AdiT Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19279[] = "Logos/2.2.8.1 (Linux; U; Android 4.0.3; zh-HK; GT-P3100 Build/IML74K.P3100ZSALD6)";
const char S19280[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3100 Build/IMM76D) Appleko) Version/4.0 Safari/534.30";
const char S19281[] = "Logos/2.2.10 (Linux; U; Android 4.0.4; en-GB; GT-P3100 Build/IMM76D.P3100XWBLJ1)";
const char S19282[] = "BBC News (AndroidApp; 1.3; 4) GT-P3100 (Android 4.0.4, SDK 15)";
const char S19283[] = "Groupon/2.2.2561 (Android 4.0.4; Samsung Espressorf / Samsung GT-P3100; )[preload=false;locale=it_IT;clientidbase=android-samsung]";
const char S19284[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SAMSUNG GT-P3100/P3100BUALD5T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19285[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; GT-P3100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19286[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KSafari/534.30";
const char S19287[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19288[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-; GT-P3100B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19289[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; GT-P3100 Build/ 4.0.4; en-ph; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19290[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; GT-P3100 Build/IML74K   http://stubes.tk) AppleWebKit/534.30 (KHTML, like Gecko   http://stubes.tk) Version/4.0 Safari/534.30";
const char S19291[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19292[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19293[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-P3100B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19294[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; SAMSUNG GT-P3100/P3100BUALD5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19295[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; in-id; GT-P3100 Build/IML74K) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.1 Version/4.0 Mobile Safari/530.17";
const char S19296[] = "nApps (Android 4.0.4; GT-P3100; androidapp.lineplay; 1.2.1)";
const char S19297[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3100 Build/Smartfren.andromaxtab7.0.v.37.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19298[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-P3100/P3100BUBLF1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19299[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; se-se; GT-P3100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19300[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P3100 Build/IMM76D) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.2 Version/4.0 Mobile Safari/530.17";
const char S19301[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P3100 Build/IMM76D) [FBAN/Orca-Android;FBAV/2.3.2-release;FBLC/ms_MY;FBBV/X;FBCR/MY_MAXIS;FBMF/samsung;FBBD/samsung;FBDV/GT-P3100;FBSV/4.0.4]";
const char S19302[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P3100-ORANGE/P3100BVBLL4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19303[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; in-id; GT-P3100 Build/IMM76D) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.1 Version/4.0 Mobile Safari/530.17";
const char S19304[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S19305[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S19306[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S19307[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S19308[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S19309[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S19310[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)";
const char S19311[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via translate.google.com)";
const char S19312[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via translate.google.com)";
const char S19313[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)";
const char S19314[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via docs.google.com/viewer)";
const char S19315[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via docs.google.com/viewer)";
const char S19316[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 AppEngine-Google; (+http://code.google.com/appengine; appid: s~inqarticles)";
const char S19317[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 AppEngine-Google; (+http://code.google.com/appengine; appid: s~inqimg)";
const char S19318[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 AppEngine-Google; (+http://code.google.com/appengine; appid: s~inqimx)";
const char S19319[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1070) AppleWebKit/537.4 (KHTML like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S19320[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.510 YaBrowser/1.5.1105.510 (gtsr) Safari/537.4";
const char S19321[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.510 YaBrowser/1.5.1105.510 (fnrb) Safari/537.4";
const char S19322[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.519 YaBrowser/1.5.1105.519 (bskt) Safari/537.4";
const char S19323[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S19324[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S19325[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via translate.google.com)";
const char S19326[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via translate.google.com)";
const char S19327[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)";
const char S19328[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)";
const char S19329[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via docs.google.com/viewer)";
const char S19330[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S19331[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (fnrb) Safari/537.4";
const char S19332[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (gtsr) Safari/537.4";
const char S19333[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.520 YaBrowser/1.5.1105.520 (bskt) Safari/537.4";
const char S19334[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.1.9999.433 Safari/537.4";
const char S19335[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (gtsr) Safari/537.4";
const char S19336[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9360; en-GB) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.336 Mobile Safari/534.11+";
const char S19337[] = "BlackBerry9360/7.1.0.694 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102";
const char S19338[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/21.0.1180.15) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.15 Safari/537.1";
const char S19339[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/21.0.1180.41) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.41 Safari/537.1";
const char S19340[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/21.0.1180.49) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.49 Safari/537.1";
const char S19341[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/21.0.1180.60) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.60 Safari/537.1";
const char S19342[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/21.0.1180.57) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.57 Safari/537.1";
const char S19343[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/21.0.1180.60) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.60 Safari/537.1";
const char S19344[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/21.0.1180.60; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S19345[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/21.0.1180.83) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1";
const char S19346[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/21.0.1180.83; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S19347[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/21.0.1180.75) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1";
const char S19348[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/21.0.1180.79) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.79 Safari/537.1";
const char S19349[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/21.0.1180.75) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1";
const char S19350[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/21.0.1180.83; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S19351[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/21.0.1180.77) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.77 Safari/537.1";
const char S19352[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/21.0.1180.79) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.79 Safari/537.1";
const char S19353[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/21.0.1180.83) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1";
const char S19354[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/21.0.1180.89) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S19355[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/21.0.1180.89; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S19356[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/21.0.1180.89)";
const char S19357[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/21.0.1180.89) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S19358[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/21.0.1180.89; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)";
const char S19359[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/21.0.1180.89; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S19360[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/21.0.1180.89) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S19361[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; chromeframe/21.0.1180.89; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E)";
const char S19362[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/21.0.1180.89; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S19363[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; MS-RTC LM 8; InfoPath.1; chromeframe/21.0.1180.83)";
const char S19364[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.2; chromeframe/21.0.1180.89)";
const char S19365[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/21.0.1180.79) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.79 Safari/537.1";
const char S19366[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/21.0.1180.89; InfoPath.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S19367[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/21.0.1180.89; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Tablet PC 1.7; .NET CLR 2.0.50727; InfoPath.1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S19368[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/21.0.1180.89; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)";
const char S19369[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:17.0) Gecko/17.0 Firefox/17.0";
const char S19370[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:17.0) Gecko/17.0 Firefox/17.0";
const char S19371[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2) Gecko/X MRA 5.6 (build 03278) Firefox/17.0 (.NET CLR 3.5.30729)";
const char S19372[] = "Mozilla/5.0 (Windows; Windows NT 6.1; Win64; x64; rv:17.0) Gecko/17.0 Firefox/17.0";
const char S19373[] = "Opera/9.00 (Windows NT 5.1; U; en)";
const char S19374[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 9.0";
const char S19375[] = "Opera/9.0 (Windows NT 5.1; U; en)";
const char S19376[] = "Opera/9.01 (Windows NT 5.1; U; en)";
const char S19377[] = "Opera/9.02 (Windows NT 5.1; U; en)";
const char S19378[] = "Opera/9.02 (Windows NT 5.2; U; en)";
const char S19379[] = "Opera/9.00 (Windows NT 5.2; U; en)";
const char S19380[] = "Opera/9.02 (Windows XP; U; ru)";
const char S19381[] = "Opera/9.01 (Windows NT 5.2; U; en)";
const char S19382[] = "Opera/9.01 (Windows NT 5.1)";
const char S19383[] = "Opera - Opera/9.00 (Windows NT 5.1; U; en)";
const char S19384[] = "('Opera/9.00 (Windows NT 5.1; U; en)', 'Opera 9.0, Windows XP')";
const char S19385[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9810; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.694 Mobile Safari/534.11+";
const char S19386[] = "BlackBerry9810/7.1.0.694 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102";
const char S19387[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Zune 4.7; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S19388[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S19389[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S19390[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Zune 4.7; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S19391[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S19392[] = "Microsoft Office/14.0 (Windows NT 6.2; Microsoft Outlook 14.0.4760; Pro; ms-office; MSOffice 14)";
const char S19393[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S19394[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Tablet PC 2.0; Zune 4.7; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S19395[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; SLCC2; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S19396[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S19397[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 14.0.6122; ms-office; MSOffice 14)";
const char S19398[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S19399[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S19400[] = "Microsoft Office/14.0 (Windows NT 6.2; Microsoft Outlook 14.0.6123; Pro)";
const char S19401[] = "Microsoft Office/14.0 (Windows NT 6.2; Microsoft Outlook 14.0.6112; Pro)";
const char S19402[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S19403[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Media Center PC 6.0; SLCC2; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S19404[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Zune 4.7; Microsoft Outlook 14.0.6122; ms-office; MSOffice 14)";
const char S19405[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S19406[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S19407[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S19408[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S19409[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S19410[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S19411[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S19412[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; SLCC2; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S19413[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S19414[] = "Microsoft Office/14.0 (Windows NT 6.2; Microsoft Outlook 14.0.6023; Pro)";
const char S19415[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S19416[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S19417[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S19418[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Media Center PC 6.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S19419[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Zune 4.7; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S19420[] = "Microsoft Office/14.0 (Windows NT 6.2; Microsoft Outlook 14.0.6129; Pro)";
const char S19421[] = "Microsoft Office/14.0 (Windows NT 6.2; Microsoft Outlook 14.0.4734; Pro; ms-office; MSOffice 14)";
const char S19422[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S19423[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; HPNTDFJS; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S19424[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S19425[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S19426[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S19427[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Tablet PC 2.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S19428[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S19429[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S19430[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S19431[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S19432[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S19433[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; SLCC2; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S19434[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S19435[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Media Center PC 6.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S19436[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S19437[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S19438[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Media Center PC 6.0; SLCC2; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S19439[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S19440[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S19441[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S19442[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S19443[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; MALCJS; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S19444[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Win64; x64; Trident/6.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S19445[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; SLCC2; Zune 4.7; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S19446[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S19447[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; MASMJS; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S19448[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S19449[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S19450[] = "Microsoft Office/14.0 (Windows NT 6.2; Microsoft Outlook 14.0.5128; Pro; ms-office; MSOffice 14)";
const char S19451[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S19452[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Zune 4.7; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S19453[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S19454[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S19455[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S19456[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S19457[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S19458[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S19459[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S19460[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S19461[] = "Microsoft Office/14.0 (Windows NT 6.2; Microsoft Outlook 14.0.7015; Pro)";
const char S19462[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S19463[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S19464[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S19465[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S19466[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S19467[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S19468[] = "Microsoft Office/14.0 (Windows NT 6.2; Microsoft Outlook 14.0.7106; Pro)";
const char S19469[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; MALCJS; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S19470[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S19471[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S19472[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S19473[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S19474[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S19475[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S19476[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S19477[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Media Center PC 6.0; SLCC2; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S19478[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S19479[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9320; en) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.398 Mobile Safari/534.11+";
const char S19480[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST21i Build/11.0.A.0.16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19481[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST21i Build/11.0.A.0.16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19482[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; SonyST21i-o Build/11.0.A.0.16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19483[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; SonyST21i Build/11.0.A.0.25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19484[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyST21i Build/2.1.J.3.8.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19485[] = "SonyST21i Build/11.0.A.0.16 stagefright/1.2 (Linux;Android 4.0.4)";
const char S19486[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-mx; SonyST21a Build/11.0.A.0.16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19487[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; SonyST21i2 Build/11.0.A.1.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19488[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SonyST21i Build/11.0.A.0.26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19489[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; SonyST21a2 Build/11.0.A.1.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19490[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; ST21i2 Build/11.0.A.1.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19491[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SonyST21i-o Build/11.0.A.0.26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19492[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hr-hr; SonyST21iv Build/11.0.A.0.16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19493[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; ST21a2 Build/11.0.A.1.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19494[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SonyST21iv Build/11.0.A.0.26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19495[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-mx; ST21a Build/11.0.A.0.16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19496[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST21i Build/11.0.A.4.22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19497[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ST21i Build/11.0.A.0.26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19498[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyST21a Build/11.0.A.0.30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19499[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST21i2 Build/11.0.A.1.8)";
const char S19500[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyST21a2 Build/11.0.A.1.12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19501[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST21i Build/11.0.A.4.22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19502[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST21i Build/11.0.A.4.22)";
const char S19503[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; SonyST21i-o Build/11.0.A.4.22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19504[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyST21i2 Build/11.0.A.6.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19505[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; SonyST21i Build/11.0.A.0.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19506[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyST21a Build/11.0.A.4.22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19507[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SonyST21i Build/11.0.A.4.25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19508[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST21a Build/11.0.A.0.16)";
const char S19509[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hr-hr; SonyST21iv Build/11.0.A.4.22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19510[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST21i2 Build/11.0.A.6.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19511[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SonyST21iv Build/11.0.A.4.25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19512[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST21a2 Build/11.0.A.1.12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19513[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; SonyST21i Build/11.0.A.4.27) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19514[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST21a Build/11.0.A.4.22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19515[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ST21i Build/11.0.A.4.25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19516[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; SonyST21i2 Build/11.0.A.6.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19517[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; ST21i Build/11.0.A.4.27) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19518[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; ST21i2 Build/11.0.A.6.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19519[] = "SonyST21i Build/11.0.A.4.22 stagefright/1.2 (Linux;Android 4.0.4)";
const char S19520[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; iw-; SonyST21i Build/11.0.A.4.22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19521[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyST21a2 Build/11.0.A.6.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19522[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST21a Build/11.0.A.0.30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19523[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fa-; SonyST21i2 Build/11.0.A.6.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19524[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST21a2 Build/11.0.A.6.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19525[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST21a2 Build/11.0.A.1.8)";
const char S19526[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/15.0.874.106) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.106 Safari/535.2";
const char S19527[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/15.0.874.106)";
const char S19528[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/15.0.874.120) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.120 Safari/535.2";
const char S19529[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/15.0.874.121; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S19530[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/15.0.874.121) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S19531[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/15.0.874.121; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; Zurich Financial Services; Zurich Financial Services)";
const char S19532[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; chromeframe/15.0.874.121; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8)";
const char S19533[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/15.0.874.121) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S19534[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/15.0.874.121; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; InfoPath.1)";
const char S19535[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/15.0.874.121; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; .NET4.0C; .NET4.0E)";
const char S19536[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/15.0.874.121; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322; .NET CLR 1.0.3705; Tablet PC 1.7; .NET4.0C; .NET4.0E; Zurich Financial Services; Zurich Financial Services)";
const char S19537[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/15.0.874.121; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.2)";
const char S19538[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/15.0.854.0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.854.0 Safari/535.2";
const char S19539[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/15.0.874.102) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2";
const char S19540[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/15.0.874.121; GTB7.2; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S19541[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/15.0.849.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/15.0.849.0 Safari/535.1";
const char S19542[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; SV1; chromeframe/15.0.874.121; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.1; MS-RTC LM 8)";
const char S19543[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/15.0.874.121; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.2; MS-RTC LM 8)";
const char S19544[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/15.0.874.121; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.2)";
const char S19545[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; chromeframe/15.0.874.106; .NET CLR 2.0.50727; InfoPath.2)";
const char S19546[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.229 Version/11.64";
const char S19547[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.64";
const char S19548[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.229 Version/11.64";
const char S19549[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.229 Version/11.64";
const char S19550[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.229 Version/11.64";
const char S19551[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5328); ru) Presto/2.10.229 Version/11.64";
const char S19552[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.229 Version/11.64";
const char S19553[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5278); ru) Presto/2.10.229 Version/11.64";
const char S19554[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.64";
const char S19555[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5218); ru) Presto/2.10.229 Version/11.64";
const char S19556[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.64";
const char S19557[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.229 Version/11.64";
const char S19558[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.229 Version/11.64";
const char S19559[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.229 Version/11.64";
const char S19560[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.229 Version/11.64";
const char S19561[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.229 Version/11.64";
const char S19562[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5267); ru) Presto/2.10.229 Version/11.64";
const char S19563[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5300); ru) Presto/2.10.229 Version/11.64";
const char S19564[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4664); ru) Presto/2.10.229 Version/11.64";
const char S19565[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.229 Version/11.64";
const char S19566[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.229 Version/11.64";
const char S19567[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.229 Version/11.64";
const char S19568[] = "Opera/9.80 (Windows NT 6.1; U; en) Presto/2.10.229 Version/11.64,gzip(gfe) (via translate.google.com)";
const char S19569[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.229 Version/11.64";
const char S19570[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.229 Version/11.64";
const char S19571[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5299); ru) Presto/2.10.229 Version/11.64";
const char S19572[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5217); ru) Presto/2.10.229 Version/11.64";
const char S19573[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4947); ru) Presto/2.10.229 Version/11.64";
const char S19574[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.229 Version/11.64";
const char S19575[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.229 Version/11.64";
const char S19576[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5230); ru) Presto/2.10.229 Version/11.64";
const char S19577[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4848); ru) Presto/2.10.229 Version/11.64";
const char S19578[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5288); ru) Presto/2.10.229 Version/11.64";
const char S19579[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; ru; rv:2.0) Gecko/X Firefox/4.0 Opera 11.64";
const char S19580[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5336); ru) Presto/2.10.229 Version/11.64";
const char S19581[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03780); ru) Presto/2.10.229 Version/11.64";
const char S19582[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.64";
const char S19583[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4110); ru) Presto/2.10.229 Version/11.64";
const char S19584[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.229 Version/11.64";
const char S19585[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.229 Version/11.64";
const char S19586[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4157); ru) Presto/2.10.229 Version/11.64";
const char S19587[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03278); ru) Presto/2.10.229 Version/11.64";
const char S19588[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.64";
const char S19589[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.229 Version/11.64";
const char S19590[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.229 Version/11.64";
const char S19591[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4930); ru) Presto/2.10.229 Version/11.64";
const char S19592[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4871); ru) Presto/2.10.229 Version/11.64";
const char S19593[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5667); ru) Presto/2.10.229 Version/11.64";
const char S19594[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5650); ru) Presto/2.10.229 Version/11.64";
const char S19595[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5660); ru) Presto/2.10.229 Version/11.64";
const char S19596[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5676); ru) Presto/2.10.229 Version/11.64";
const char S19597[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.229 Version/11.64";
const char S19598[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.229 Version/11.64";
const char S19599[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.229 Version/11.64";
const char S19600[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.229 Version/11.64";
const char S19601[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.10.229 Version/11.64";
const char S19602[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.229 Version/11.64";
const char S19603[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.229 Version/11.64";
const char S19604[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; ru) Opera 11.64";
const char S19605[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5970); ru) Presto/2.10.229 Version/11.64";
const char S19606[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.229 Version/11.64";
const char S19607[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.229 Version/11.64";
const char S19608[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5161); ru) Presto/2.10.229 Version/11.64";
const char S19609[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.10.229 Version/11.64";
const char S19610[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5325); ru) Presto/2.10.229 Version/11.64";
const char S19611[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.229 Version/11.64,gzip(gfe)";
const char S19612[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.229 Version/11.64";
const char S19613[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.229 Version/11.64";
const char S19614[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5966); ru) Presto/2.10.229 Version/11.64";
const char S19615[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.229 Version/11.64";
const char S19616[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.229 Version/11.64";
const char S19617[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5784); ru) Presto/2.10.229 Version/11.64";
const char S19618[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5880); ru) Presto/2.10.229 Version/11.64";
const char S19619[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 6068); ru) Presto/2.10.229 Version/11.64";
const char S19620[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4661); ru) Presto/2.10.229 Version/11.64";
const char S19621[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.229 Version/11.64";
const char S19622[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5745); ru) Presto/2.10.229 Version/11.64";
const char S19623[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5997); ru) Presto/2.10.229 Version/11.64";
const char S19624[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.229 Version/11.64";
const char S19625[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5977); ru) Presto/2.10.229 Version/11.64";
const char S19626[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6083); ru) Presto/2.10.229 Version/11.64";
const char S19627[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B329 Safari/8536.25";
const char S19628[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B146 Safari/8536.25";
const char S19629[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B144 Safari/8536.25";
const char S19630[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B145 Safari/8536.25";
const char S19631[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B142 Safari/8536.25";
const char S19632[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B143 Safari/8536.25";
const char S19633[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B141 Safari/8536.25";
const char S19634[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B350 Safari/8536.25";
const char S19635[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Sensation 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19636[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19637[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; zh-tw; HTC Sensation Z710e Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19638[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Sensation 4G with Beats Audio Build/IML74K)";
const char S19639[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sensation_4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19640[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19641[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Sensation_Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19642[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Sensation 4G with Beats Audio Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19643[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_Sensation_4G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19644[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_Sensation_4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19645[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-sg; HTC_Sensation_Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19646[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Sensation Z710e Build/IML74K)";
const char S19647[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation Z710e Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S19648[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Sensation_Z710e Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19649[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Sensation 4G Build/IML74K)";
const char S19650[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Sensation 4G XE with Beats Audio Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19651[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sensation 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19652[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation Z710e Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S19653[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Sensation Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19654[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Z710e Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S19655[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; HTC_Sensation_Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19656[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Sensation 4G Build/IML74K)";
const char S19657[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Z710e Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S19658[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_Sensation_4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19659[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Sensation 4G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19660[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Sensation_Z710e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19661[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_Sensation_4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19662[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; HTC Sensation Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19663[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Sensation_Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19664[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Sensation Z710e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19665[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Sensation 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19666[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Sensation Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19667[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Sensation Z710E Build/IMM76D)";
const char S19668[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Sensation 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19669[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Sensation Z710e Build/IML74K)";
const char S19670[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; HTC Sensation 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19671[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; Sensation_Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19672[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Sensation Z710e Build/IML74K)";
const char S19673[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19674[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-I8160L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19675[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; SAMSUNG GT-I8160/I8160XXLD8 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19676[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I8160 Build/GINGERBREAD)";
const char S19677[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-at; SAMSUNG GT-I8160/I8160BOLE1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19678[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pl-pl; SAMSUNG GT-I8160/I8160BOLF2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19679[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-I8160P Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19680[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19681[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8160 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S19682[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I8160L Build/GINGERBREAD)";
const char S19683[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; SAMSUNG GT-I8160/I8160BOLK2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19684[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-I8160P-ORANGE/I8160PBVLK3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19685[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I8160P Build/GINGERBREAD)";
const char S19686[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; SAMSUNG GT-I8160/I8160XXLL2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19687[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; SAMSUNG GT-I8160/I8160XXLK6 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19688[] = "samsung-I8160/1.0 Linux/2.6.35.7 Android/2.3.5 Release/01.05.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S19689[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; Build/JRN84D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19690[] = "stagefright/1.2 (Linux;Android 4.1.1)";
const char S19691[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; VegaBean Beta 1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19692[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19693[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; VegaBean Beta 3 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19694[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Android SDK built for x86 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19695[] = "stagefright/1.2 (Linux;Android 4.1.2)";
const char S19696[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; ATP7717 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19697[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; PC936 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19698[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; U1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19699[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; 9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19700[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; N101 DUAL CORE2 V11 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19701[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HC801 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19702[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; VegaBean Beta 6 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19703[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ACM3066-8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19704[] = "Mozilla/5.0 (Linux; U; Android 4.1.5; en-us; TREQ_A10_Pocket2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19705[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Express Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19706[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; SHOGUN Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19707[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; Tablet 9748DCB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19708[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; X720D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19709[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; zh-tw; N7077 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19710[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; SP0718 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19711[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; Slick973 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19712[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19713[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TABLO Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19714[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.2 Version/4.0 Mobile Safari/530.17";
const char S19715[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; VegaBean Beta 5 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19716[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; EVOPad R9720 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19717[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M1001 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19718[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; X2-IPS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19719[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Android SDK built for x86 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19720[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19721[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; cm_tenderloin Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19722[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19723[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19724[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1911_v77_gq2000_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19725[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; e1909c_v77_gq1008_9p017_sui_a41_6628 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19726[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Android Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19727[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1911_v77_gq2000_9p017_6628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19728[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; VegaBean Beta 2.2 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19729[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_jbla668_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19730[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ADM816KC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19731[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_gq2002_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19732[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; unknown Build/Mobiltab Sleek 10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19733[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; e1901_v77_jdt1_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19734[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_jbla668_9p017_fwvga Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19735[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; cm_tenderloin Build/JZO54K)";
const char S19736[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_gq2002_5647_notv Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19737[] = "Mozilla/5.0 (Linux; U; Android 4.1.5; en-us; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19738[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1908_v77_jbl1_9p017_6628 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19739[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TAGI MID 950 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19740[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_gq2002_9p017_fg Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19741[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; PUBLIC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19742[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_jbl1_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19743[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1911_v77_gq2000_a41_6628_v20 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19744[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_jbl1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19745[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1911_v77_gq2000_ov5647_6628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19746[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Motorola Galaxy s3 Build/9.8.2O-72_VZW-8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19747[] = "Tablet-PC-4.1-Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ADM8000KP_A Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19748[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; da-dk; EN-TAB309 Build/IML74K..X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19749[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; V2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19750[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19751[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HD3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19752[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_jbla668_9p017_nozhrtw Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19753[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1908_v77_jbl1_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19754[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1920_v77_zlh_9p017_asx Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19755[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; V1013 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19756[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_jbla668_fwvga Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19757[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1909l_v77_gq1008_a41_v20 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19758[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_cxq_asx Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19759[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; unknown Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19760[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1921_v77_cxq1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19761[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; CTM Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19762[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TY-Q8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19763[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; e1901_v77_jdt1_9p017_fwvga Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19764[] = "Tablet-PC-4.1-Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ADM8000KP_B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19765[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1909c_v77_gq1008_a41_6628 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19766[] = "Mozilla/5.0 (Linux; U; Android 4.1.5; vi-vn; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19767[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ss_tenderloin Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19768[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AML-MX REF Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19769[] = "AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; Build/JRO03C)";
const char S19770[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_gq2002 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19771[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1911_v77_mt6628_hjy1_9p017_v2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19772[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_gq2008_ov8825 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19773[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MR18 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19774[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; vi-vn; iphone Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19775[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; MID Build/JRO03H)";
const char S19776[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; RK736H Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19777[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_cxq_asx Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19778[] = "Mozilla/5.0 (Linux; U; Android 4.1.5; en-us; iPhone Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19779[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1911_v77_mt6628_hjy1_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19780[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_gq2002tc Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19781[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_cxq Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19782[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1920_v77_zlh_fwvga Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19783[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; MID-709S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19784[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; H100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19785[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; iPhone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19786[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MID Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19787[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; MAX-SGS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19788[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; GF-2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19789[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; e2001_v89_gq2008s Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19790[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1920_v77_zlh_9p017_fwvga Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19791[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_gq2008 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19792[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1901_v77_jbla668_fwvga Build/JRO03C)";
const char S19793[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1911_v77_mt6628_hjy1_9p017_v20 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19794[] = "Mozilla/5.0 (Linux; U; Android 4.1.5; tr-tr; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19795[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_gq2002_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19796[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1901_v77_jbla668_9p017_fwvga Build/JRO03C)";
const char S19797[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; e1901_v77_gq2002_fg Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19798[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1911_v77_gq2000_a41_6628_v20 Build/JRO03C)";
const char S19799[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; e1909c_v77_jbl_9p017_a41_6628 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19800[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; unknown Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19801[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_jbl_hd_rtw Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19802[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19803[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; S98 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19804[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1908_v77_jbl1a535_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19805[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1908_v77_jbl1_9p017_6628 Build/JRO03C)";
const char S19806[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; iphone5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19807[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; VegaBean Final Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19808[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1909l_v77_gq1008_a41_v20 Build/JRO03C)";
const char S19809[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; iPhone5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19810[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; e2001_v89_jbl1a698 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19811[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MID Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19812[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; CTP838BT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19813[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1920_v77_gq2006_9p017_32p8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19814[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19815[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; HFM8000KP Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19816[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; HTC_POO_U Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19817[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; HRA_M801 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19818[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v15_hy_mt9t113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19819[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; unknown Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19820[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; unknown Build/JZO54K)";
const char S19821[] = "Mozilla/5.0 (Linux; U; Android 4.1.4; en-us; e1908_v77_nd6_6628_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19822[] = "Mozilla/5.0 (Linux; U; Android 4.1.3; en-us; unknown Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19823[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1921_v77_mt6628_hjy Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19824[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; samsung Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19825[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_jbl_hd Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19826[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; eidolon15_nand_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19827[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; GASEM7000KD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19828[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; e1901_v77_gq2002 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19829[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; e1920_v77_zlh_ov5647_fwvga Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19830[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1901_v77_jdt1_9p017 Build/JRO03C)";
const char S19831[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1901_v77_jbl1 Build/JRO03C)";
const char S19832[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; e1901_v77_gq2002gn Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19833[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19834[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us;LC0720C Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 4.1.1; cs-cz; Build/JRO03CSafari/534.30";
const char S19835[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 09l_v77_gq1008_a41_v20 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19836[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; e1908_v77_nd3_6628_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19837[] = "stagefright/1.1 (Linux;Android 4.1.2)";
const char S19838[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1920_v77_zlh_9p017_asx Build/JRO03C)";
const char S19839[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1901_v77_jdt1_9p017_fwvga Build/JRO03C)";
const char S19840[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; e1901_v77_jdt1_9p017_fwvga Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19841[] = "stagefright/1.2 (Linux;Android stagefright/1.2 (Linux;Android 4.1.2)";
const char S19842[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MID Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19843[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; CNM-TP-10DC-16-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19844[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; e1901_v77_gq3002gn Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19845[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; CnM-TP-8DC-16-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19846[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; CnM-TP-7DC-8-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19847[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1901_v77_jbla668_9p017 Build/JRO03C)";
const char S19848[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; iPhone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S19849[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1920_v77_zlh_fwvga Build/JRO03C)";
const char S19850[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1902_v77_jblw005 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19851[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19852[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Build/Zeki.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S19853[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1901_v77_jbl1_9p017 Build/JRO03C)";
const char S19854[] = "Dalvik/1.4.0 (Linux; U; Android 4.1.1; eidolon15_nand_gb2 Build/GRK39F)";
const char S19855[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Android Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19856[] = "AndroidCvpPlayer(Linux; U; Android 4.1.2; en-us)";
const char S19857[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; e1908_v77_mt6628_hjy1a6x_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19858[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1910_v77_yz Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19859[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S19860[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1901_v77_gq2002tc Build/JRO03C)";
const char S19861[] = "Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31";
const char S19862[] = "Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31";
const char S19863[] = "Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31";
const char S19864[] = "Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31";
const char S19865[] = "Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31";
const char S19866[] = "Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31";
const char S19867[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31";
const char S19868[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31";
const char S19869[] = "Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31";
const char S19870[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31,gzip(gfe)";
const char S19871[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31,gzip(gfe)";
const char S19872[] = "Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31";
const char S19873[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)";
const char S19874[] = "Mozilla/5.0 (X11; Linux x86; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31";
const char S19875[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7; en-us) AppleWebKit/534.20.8 (KHTML, like Gecko)";
const char S19876[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/535.1 (KHTML, like Gecko) Safari/535.1";
const char S19877[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3)";
const char S19878[] = "QuickTime/7.7.1 (qtver=7.7.1;cpu=IA32;os=Mac 10.7.3)";
const char S19879[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.2.10) Gecko Expeditor 6.2.2.X-1115 Gecko/X-1115";
const char S19880[] = "QuickTime/7.7.1 (qtver=7.7.1;cpu=IA32;os=Mac 10.7.4)";
const char S19881[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.53.11 (KHTML, like Gecko) Sparrow/1043";
const char S19882[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.53.11 (KHTML, like Gecko) LittleSnapper/12900 Safari/525.13";
const char S19883[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) Sparrow/1081.38";
const char S19884[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) Sparrow/1081.32";
const char S19885[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3; AppleWebKit/534.55.3 (KHTML, like Gecko)";
const char S19886[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) Sparrow/1081.51";
const char S19887[] = "Mozilla/4.0 (Mac OS X 10.7.4) Java/1.6.0_33";
const char S19888[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) Sparrow/1144";
const char S19889[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) NetNewsWire/3.2.7";
const char S19890[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) NetNewsWire/3.3.1";
const char S19891[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko)";
const char S19892[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) NetNewsWire/3.3.2";
const char S19893[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/1176";
const char S19894[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/1172";
const char S19895[] = "QuickTime/7.7.1 (qtver=7.7.1;cpu=IA32;os=Mac 10.7.5)";
const char S19896[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Safari/531.9";
const char S19897[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/1178";
const char S19898[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) NetNewsWire/3.3";
const char S19899[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/1173";
const char S19900[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) NetNewsWire/3.2.11";
const char S19901[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/534.52.7 (KHTML	 like Gecko) Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_2; en) AppleWebKit/534.52.7 (KHTML	 like Gecko) &quot;wKiosk for Mac&quot;";
const char S19902[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Vienna/2.6.0.2601";
const char S19903[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Safari/522.0";
const char S19904[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/1143.6";
const char S19905[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/1043.1";
const char S19906[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Vienna/3.0b.2820";
const char S19907[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) NetNewsWire/3.2.15";
const char S19908[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebune 4.7; .NET4.0C)";
const char S19909[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Paparazzi!/0.6.5 (like Safari)";
const char S19910[] = "CalendarStore/5.0.3 (1204.2); iCal/5.0.3 (1605.4); Mac OS X/10.7.5 (11G63)";
const char S19911[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/1081.52";
const char S19912[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) RealPlayer/12.0.1.1750";
const char S19913[] = "Mac OS X/10.7.5 (11G63); ExchangeWebServices/2.2 (127); iCal/5.0.3 (1605.4)";
const char S19914[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sunrise";
const char S19915[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Safari/417.8";
const char S19916[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Gruml/1.3.0";
const char S19917[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) iWeb/304";
const char S19918[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/1164";
const char S19919[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/904";
const char S19920[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) RealPlayer/12.0.0.1725";
const char S19921[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/1081.53";
const char S19922[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/536.26.17 (KHTML, like Gecko),gzip(gfe)";
const char S19923[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Vienna/3.0b.2821 Safari/536.26.17";
const char S19924[] = "Mac OS X/10.7.5 (11G63); ExchangeWebServices/2.2 (127); AddressBookSourceSync/1.4 (1091)";
const char S19925[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/959.4";
const char S19926[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) RealPlayer/11.1.0.1116";
const char S19927[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/1043.2";
const char S19928[] = "Airmail 1.0.2 rv:175 (Macintosh; Mac OS X 10.7.5; en_US)";
const char S19929[] = "Airmail 1.0.3 rv:176 (Macintosh; Mac OS X 10.7.5; en_US)";
const char S19930[] = "Mozilla/4.0 (Mac OS X 10.7.5) Java/1.7.0_09";
const char S19931[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) TubeTV";
const char S19932[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Vienna/3.0.0 Beta 11 :9d8d6f6: Safari/536.28.10";
const char S19933[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Vienna/3.0.0 Beta 11 :9d8d6f6: Safari/536.29.13";
const char S19934[] = "Airmail 1.0.4 rv:183 (Macintosh; Mac OS X 10.7.5; en_US)";
const char S19935[] = "Airmail 1.0.4 rv:183 (Macintosh; Mac OS X 10.7.5; en_GB)";
const char S19936[] = "Airmail 1.0.4 rv:183 (Macintosh; Mac OS X 10.7.5; de_DE)";
const char S19937[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/1081.27";
const char S19938[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) NetNewsWire/3.1.7";
const char S19939[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) NetNewsWire/4.0.0";
const char S19940[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Vienna/3.0.0 Safari/536.30.1";
const char S19941[] = "Airmail 1.0.5 rv:192 (Macintosh; Mac OS X 10.7.5; en_US)";
const char S19942[] = "Airmail 1.0.5 rv:192 (Macintosh; Mac OS X 10.7.5; en_GB)";
const char S19943[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.1.9) Gecko/X Mozilla/2.0";
const char S19944[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Safari/412.0";
const char S19945[] = "Mozilla/5.0 (X11; U; Linux x86_64; C) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.2 Safari/533.3";
const char S19946[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.7.4 Safari/534.34";
const char S19947[] = "Mozilla/5.0 (Unknown; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.7.4 Safari/534.34";
const char S19948[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Qt/4.7.4 Safari/534.3";
const char S19949[] = "Mozilla/5.0 (QtEmbedded; U; Linux; C) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.1 Safari/533.3";
const char S19950[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Qt/4.7.1 Safari/534.10";
const char S19951[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.30 (KHTML, like Gecko) Qt/4.7.1 Safari/534.30";
const char S19952[] = "Mozilla/5.0 (QtEmbedded; U; Linux; fr-FR) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.2 Safari/533.3";
const char S19953[] = "Mozilla/5.0 (QtEmbedded; U; Linux; ar-SA) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.2 Safari/533.3";
const char S19954[] = "Mozilla/5.0 (QtEmbedded; Linux) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.7.1 Safari/534.34";
const char S19955[] = "Mozilla/5.0 (QtEmbedded; U; Linux; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.2 Safari/533.3";
const char S19956[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.107 Safari/534.13 BuiltWith/1.2";
const char S19957[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.84 Safari/534.13,gzip(gfe) (via translate.google.com)";
const char S19958[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13 SunChrome/9.0";
const char S19959[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13, SurfCop 1.5";
const char S19960[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597 Safari/534.13";
const char S19961[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.84 Safari/534.13,gzip(gfe)";
const char S19962[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-us) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13";
const char S19963[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-CA) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13";
const char S19964[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; es-US) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13";
const char S19965[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; de-DE) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13";
const char S19966[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; fr-FR) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13";
const char S19967[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; fr-CA) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13";
const char S19968[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; haw-US) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13";
const char S19969[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; zh-CN) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13";
const char S19970[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.15";
const char S19971[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.15";
const char S19972[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.15";
const char S19973[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.15,gzip(gfe)";
const char S19974[] = "Opera/9.80 (Windows NT 5.2) Presto/2.12.388 Version/12.15";
const char S19975[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1) Opera 12.15";
const char S19976[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.15";
const char S19977[] = "Opera/9.80 (Windows NT 5.1; Edition Campaign 21) Presto/2.12.388 Version/12.15";
const char S19978[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.15";
const char S19979[] = "Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.15";
const char S19980[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.15";
const char S19981[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.15";
const char S19982[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.15";
const char S19983[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.15";
const char S19984[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.15";
const char S19985[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.15";
const char S19986[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.15";
const char S19987[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.15";
const char S19988[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.15";
const char S19989[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.15";
const char S19990[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.15";
const char S19991[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.15";
const char S19992[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.15";
const char S19993[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.15";
const char S19994[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.15";
const char S19995[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.15";
const char S19996[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.15";
const char S19997[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.15";
const char S19998[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.15";
const char S19999[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.15";
const char S20000[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.15 Yandex.Translate";
const char S20001[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.15";
const char S20002[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.15";
const char S20003[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.15";
const char S20004[] = "Mozilla/5.0 (Windows NT 5.1; rv:2.0) Gecko/X Firefox/4.0 Opera 12.15";
const char S20005[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.15";
const char S20006[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.15";
const char S20007[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.15";
const char S20008[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.15";
const char S20009[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.15";
const char S20010[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.15";
const char S20011[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.15";
const char S20012[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.15";
const char S20013[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.15";
const char S20014[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.15";
const char S20015[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.15";
const char S20016[] = "Opera/9.80 (Windows NT 5.1; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.15";
const char S20017[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.15";
const char S20018[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6080)) Presto/2.12.388 Version/12.15";
const char S20019[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.15";
const char S20020[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.15";
const char S20021[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4933)) Presto/2.12.388 Version/12.15";
const char S20022[] = "Opera/9.80 (Windows NT 5.2; WOW64) Presto/2.12.388 Version/12.15";
const char S20023[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.15";
const char S20024[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.15";
const char S20025[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.15";
const char S20026[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.15";
const char S20027[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.15";
const char S20028[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.15";
const char S20029[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.15";
const char S20030[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.15";
const char S20031[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.15";
const char S20032[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.15";
const char S20033[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.15";
const char S20034[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.15";
const char S20035[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.15";
const char S20036[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.15";
const char S20037[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.15";
const char S20038[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.15";
const char S20039[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.15";
const char S20040[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.15";
const char S20041[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.15";
const char S20042[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.15";
const char S20043[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.15";
const char S20044[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.15";
const char S20045[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5650)) Presto/2.12.388 Version/12.15";
const char S20046[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03686); MRSPUTNIK OW 2, 3, 0, 244) Presto/2.12.388 Version/12.15";
const char S20047[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03639)) Presto/2.12.388 Version/12.15";
const char S20048[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6035)) Presto/2.12.388 Version/12.15";
const char S20049[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.15";
const char S20050[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03402)) Presto/2.12.388 Version/12.15";
const char S20051[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6044)) Presto/2.12.388 Version/12.15";
const char S20052[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5230)) Presto/2.12.388 Version/12.15";
const char S20053[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.15";
const char S20054[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.15";
const char S20055[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.15";
const char S20056[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.15";
const char S20057[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.15";
const char S20058[] = "Opera/9.80 (Windows NT 5.1; Edition Yx) Presto/2.12.388 Version/12.15,gzip(gfe)";
const char S20059[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6046)) Presto/2.12.388 Version/12.15";
const char S20060[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.15";
const char S20061[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03658)) Presto/2.12.388 Version/12.15";
const char S20062[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.15";
const char S20063[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.15";
const char S20064[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5278)) Presto/2.12.388 Version/12.15";
const char S20065[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03755)) Presto/2.12.388 Version/12.15";
const char S20066[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03789)) Presto/2.12.388 Version/12.15";
const char S20067[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.15";
const char S20068[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6014)) Presto/2.12.388 Version/12.15";
const char S20069[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.15";
const char S20070[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03402); MRSPUTNIK OW 2, 3, 0, 104) Presto/2.12.388 Version/12.15";
const char S20071[] = "Opera/9.80 (Windows NT 5.1; MRA 5.3 (build 02552)) Presto/2.12.388 Version/12.15";
const char S20072[] = "Opera/9.80 (Windows NT 5.1; MRA 5.3 (build 02560)) Presto/2.12.388 Version/12.15";
const char S20073[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4110)) Presto/2.12.388 Version/12.15";
const char S20074[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03672)) Presto/2.12.388 Version/12.15";
const char S20075[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5660)) Presto/2.12.388 Version/12.15";
const char S20076[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03757); MRSPUTNIK OW 2, 3, 0, 288) Presto/2.12.388 Version/12.15";
const char S20077[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5336)) Presto/2.12.388 Version/12.15";
const char S20078[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5217)) Presto/2.12.388 Version/12.15";
const char S20079[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5288)) Presto/2.12.388 Version/12.15";
const char S20080[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6561)) Presto/2.12.388 Version/12.15";
const char S20081[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.15,gzip(gfe) (via docs.google.com/viewer)";
const char S20082[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6032)) Presto/2.12.388 Version/12.15";
const char S20083[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4832)) Presto/2.12.388 Version/12.15";
const char S20084[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6573)) Presto/2.12.388 Version/12.15";
const char S20085[] = "Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6261)) Presto/2.12.388 Version/12.15";
const char S20086[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.15";
const char S20087[] = "Opera/9.80 (Windows NT 5.1; MRA 5.3 (build 02564)) Presto/2.12.388 Version/12.15";
const char S20088[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03791)) Presto/2.12.388 Version/12.15";
const char S20089[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5300)) Presto/2.12.388 Version/12.15";
const char S20090[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4661)) Presto/2.12.388 Version/12.15";
const char S20091[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5299)) Presto/2.12.388 Version/12.15";
const char S20092[] = "Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6277)) Presto/2.12.388 Version/12.15";
const char S20093[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5996)) Presto/2.12.388 Version/12.15";
const char S20094[] = "Opera/9.80 (Windows NT 5.1; MRA 5.4 (build 02652)) Presto/2.12.388 Version/12.15";
const char S20095[] = "Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6283)) Presto/2.12.388 Version/12.15";
const char S20096[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.15 WebWasher 3.4";
const char S20097[] = "Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02746)) Presto/2.12.388 Version/12.15";
const char S20098[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5325)) Presto/2.12.388 Version/12.15";
const char S20099[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6628)) Presto/2.12.388 Version/12.15";
const char S20100[] = "Opera/9.80 (Windows NT 5.1; MRA 5.4 (build 02620)) Presto/2.12.388 Version/12.15";
const char S20101[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5734)) Presto/2.12.388 Version/12.15";
const char S20102[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6638)) Presto/2.12.388 Version/12.15";
const char S20103[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6620)) Presto/2.12.388 Version/12.15";
const char S20104[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.15";
const char S20105[] = "Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6323)) Presto/2.12.388 Version/12.15";
const char S20106[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6657)) Presto/2.12.388 Version/12.15";
const char S20107[] = "Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6327)) Presto/2.12.388 Version/12.15";
const char S20108[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6648)) Presto/2.12.388 Version/12.15";
const char S20109[] = "Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6322)) Presto/2.12.388 Version/12.15";
const char S20110[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5161)) Presto/2.12.388 Version/12.15";
const char S20111[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03399)) Presto/2.12.388 Version/12.15";
const char S20112[] = "Opera/9.80 (Windows NT 5.1; MRA 5.4 (build 02625)) Presto/2.12.388 Version/12.15";
const char S20113[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6665)) Presto/2.12.388 Version/12.15";
const char S20114[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6666)) Presto/2.12.388 Version/12.15";
const char S20115[] = "Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6341)) Presto/2.12.388 Version/12.15";
const char S20116[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6669)) Presto/2.12.388 Version/12.15";
const char S20117[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6673)) Presto/2.12.388 Version/12.15";
const char S20118[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03780)) Presto/2.12.388 Version/12.15";
const char S20119[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03658); MRSPUTNIK OW 2, 3, 0, 216) Presto/2.12.388 Version/12.15";
const char S20120[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03638)) Presto/2.12.388 Version/12.15";
const char S20121[] = "Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6325)) Presto/2.12.388 Version/12.15";
const char S20122[] = "Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6337)) Presto/2.12.388 Version/12.15";
const char S20123[] = "Opera/10.00 (Windows NT 5.1; U; Edition Englend Local; en) Presto/2.9.168 Version/12.15";
const char S20124[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6675)) Presto/2.12.388 Version/12.15";
const char S20125[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.15,gzip(gfe),gzip(gfe)";
const char S20126[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6681)) Presto/2.12.388 Version/12.15";
const char S20127[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6676)) Presto/2.12.388 Version/12.15";
const char S20128[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.15";
const char S20129[] = "Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6336)) Presto/2.12.388 Version/12.15";
const char S20130[] = "Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6347)) Presto/2.12.388 Version/12.15";
const char S20131[] = "Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02842); MRSPUTNIK OW 1, 0, 0, 25) Presto/2.12.388 Version/12.15";
const char S20132[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6667)) Presto/2.12.388 Version/12.15";
const char S20133[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6682)) Presto/2.12.388 Version/12.15";
const char S20134[] = "Opera/9.80 (Windows NT 5.1; MRA 6.2 (build 7188)) Presto/2.12.388 Version/12.15";
const char S20135[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B329 Safari/8536.25";
const char S20136[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B142 Safari/8536.25";
const char S20137[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B350 Safari/8536.25";
const char S20138[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B145 Safari/8536.25";
const char S20139[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B146 Safari/8536.25";
const char S20140[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B143 Safari/8536.25";
const char S20141[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B144 Safari/8536.25";
const char S20142[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B141 Safari/8536.25";
const char S20143[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9Z999 Safari/8536.25";
const char S20144[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.4; en-GB; rv:1.9.2.28) Gecko/X Firefox/3.6.28";
const char S20145[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20146[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20147[] = "Mozilla/5.0 (Linux; Android 4.1.1; SPH-L900 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20148[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20149[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I317 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20150[] = "Mozilla/5.0 (Linux; Android 4.1.1; SHV-E250S Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20151[] = "Mozilla/5.0 (Linux; Android 4.1.1; SHV-E250K Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20152[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I317M Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20153[] = "Mozilla/5.0 (Linux; Android 4.1.1; SHV-E250L Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20154[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-R950 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20155[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I317 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20156[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889V Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20157[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20158[] = "Mozilla/5.0 (Linux; Android 4.1.1; SC-02E Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20159[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105T Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20160[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20161[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20162[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20163[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100T Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20164[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20165[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20166[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S20167[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20168[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20169[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20170[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20171[] = "Mozilla/5.0 (Linux; Android 4.1.2 DLL1; SGH-T889 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20172[] = "Mozilla/5.0 (Linux; Android 4.1.9; GT-N7100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20173[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-N719 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20174[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R950 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20175[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889V Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20176[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20177[] = "Mozilla/5.0 (Linux; Android 4.1.1 V2.1 ALK8; SGH-T889 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20178[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S20179[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S20180[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S20181[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9360; en) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.319 Mobile Safari/534.11+";
const char S20182[] = "BlackBerry9360/7.0.0.586 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102";
const char S20183[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/6.0.24093/24.746; U; en) Presto/2.5.25 Version/10.54";
const char S20184[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/6.0.24093/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S20185[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S20186[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S20187[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S20188[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via translate.google.com)";
const char S20189[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via docs.google.com/viewer)";
const char S20190[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)";
const char S20191[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.4 (KHTML like Gecko) Chrome/22.0.1229.79 Safari/537.4";
const char S20192[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S20193[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1082) AppleWebKit/537.4 (KHTML like Gecko) Chrome/22.0.1229.92 Safari/537.4";
const char S20194[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1081) AppleWebKit/537.4 (KHTML like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S20195[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.510 YaBrowser/1.5.1105.510 (gtsr) Safari/537.4";
const char S20196[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.510 YaBrowser/1.5.1105.510 (gtsr) Safari/537.4";
const char S20197[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.519 YaBrowser/1.5.1105.519 (bskt) Safari/537.4";
const char S20198[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S20199[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S20200[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/1742; ) AppleWebKit/534.1 (KHTML, like Gecko) Chrome/6.0.444.0 Safari/534.1";
const char S20201[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.33 Safari/534.3 SE 2.X MetaSr 1.0";
const char S20202[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.33 Safari/534.3 SE 2.X MetaSr 1.0,gzip(gfe) (via translate.google.com)";
const char S20203[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/1738; ) AppleWebKit/534.1 (KHTML, like Gecko) Chrome/6.0.444.0 Safari/534.1";
const char S20204[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Source Client/0; ) AppleWebKit/534.1 (KHTML, like Gecko) Chrome/6.0.444.0 Safari/534.1";
const char S20205[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay; ) AppleWebKit/534.1 (KHTML, like Gecko) Chrome/6.0.444.0 Safari/534.1";
const char S20206[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-GB) AppleWebKit/534.1 (KHTML, like Gecko) Chrome/6.0.428.0 Safari/534.1";
const char S20207[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.3 (KHTML	 like Gecko) Chrome/6.0.458.1 Safari/534.3";
const char S20208[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.464.0 Safari/534.3,gzip(gfe)";
const char S20209[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonST18a Build/4.0.A.2.299) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20210[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonST18i Build/4.0.1.A.0.215) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20211[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonST18i Build/4.0.1.A.0.165) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20212[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonST18i Build/4.0.A.2.357) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20213[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonST18i Build/4.0.A.2.335) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20214[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonST18i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20215[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; ST18i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20216[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonSO-03C Build/4.0.D.2.61) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20217[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonST18i Build/4.0.1.A.0.277) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20218[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonST18a Build/4.0.1.A.0.284) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20219[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonST18i Build/4.0.A.2.136) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20220[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST18i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20221[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; ST18i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20222[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; ST18i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20223[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ie; SonyEricssonST18iv Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20224[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonST18i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20225[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; SonyEricssonST18av Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20226[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; SonyEricssonST18i Build/4.0.1.A.2.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20227[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-mx; SonyEricssonST18a Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20228[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonST18i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20229[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-03C Build/4.0.1.C.1.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20230[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; SonyEricssonST18a Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20231[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonST18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20232[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; ST18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20233[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonST18iv Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20234[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; ST18i Build/4.0.2.A.0.62)";
const char S20235[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; SonyEricssonST18av Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20236[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-03C Build/4.0.1.C.1.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20237[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-hk; SonyEricssonST18i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20238[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; ST18a Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20239[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonST18iv Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20240[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; ST18i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20241[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; ST18i Build/4.0.A.2.368)";
const char S20242[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ST18i Build/4.0.2.A.0.62) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S20243[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonST18a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20244[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; ST18a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20245[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-mx; ST18a Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20246[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; ST18a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20247[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; SonyEricssonST18i Build/4.0.A.2.362) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20248[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ST18i Build/4.0.2.A.0.62) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S20249[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ST18i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20250[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-hk; ST18i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20251[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; ST18a Build/4.0.A.2.368)";
const char S20252[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ST18i Build/4.0.2.A.0.58) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S20253[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ST18i Build/4.0.2.A.0.42) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S20254[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; ST18i Build/4.0.A.2.368) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S20255[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-nz; SonyEricssonST18a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20256[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-03C Build/4.0.1.C.1.31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20257[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ST18i Build/4.0.2.A.0.42) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S20258[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-; SonyEricssonST18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20259[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonST18iv Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20260[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; ST18a Build/4.0.2.A.0.62)";
const char S20261[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.7.62 Version/11.00";
const char S20262[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.7.62 Version/11.00";
const char S20263[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.7.62 Version/11.01";
const char S20264[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.7.62 Version/11.01";
const char S20265[] = "Opera/9.80 (Windows NT 6.1; U; en-US) Presto/2.7.62 Version/11.01";
const char S20266[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.7.62 Version/11.00,gzip(gfe) (via translate.google.com)";
const char S20267[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.7.62 Version/11.00";
const char S20268[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.7.62 Version/11.00";
const char S20269[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.7.62 Version/11.01";
const char S20270[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.7.62 Version/11.01";
const char S20271[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.7.62 Version/11.01,gzip(gfe) (via translate.google.com)";
const char S20272[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.7.62 Version/11.01";
const char S20273[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.7.62 Version/11.01";
const char S20274[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.7.62 Version/11.01";
const char S20275[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.7.62 Version/11.01";
const char S20276[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.7.39 Version/11.00";
const char S20277[] = "Mozilla/5.0 (Windows NT 6.1; U; de; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 11.01";
const char S20278[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.7.62 Version/11.01";
const char S20279[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.7.62 Version/11.01";
const char S20280[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.7.62 Version/11.00";
const char S20281[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03639); ru) Presto/2.7.62 Version/11.01";
const char S20282[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03278); ru) Presto/2.7.62 Version/11.01";
const char S20283[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.7.62 Version/11.01";
const char S20284[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; de) Opera 11.01";
const char S20285[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.13) Gecko/X Opera/9.80 (Windows NT 6.1; U; zh-tw) Presto/2.7.62 Version/11.01";
const char S20286[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.7.62 Version/11.00";
const char S20287[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.7.62 Version/11.00";
const char S20288[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.7.62 Version/11.00";
const char S20289[] = "Opera/9.80 (Windows NT 6.1 x64; U; en) Presto/2.7.62 Version/11.00";
const char S20290[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.7.62 Version/11.00";
const char S20291[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.7.62 Version/11.00";
const char S20292[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.7.62 Version/11.01";
const char S20293[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.7.62 Version/11.01";
const char S20294[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.7.62 Version/11.00";
const char S20295[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; fr) Opera 11.00";
const char S20296[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.7.62 Version/11.01";
const char S20297[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.7.62 Version/11.00";
const char S20298[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.7.62 Version/11.00";
const char S20299[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.7.62 Version/11.00";
const char S20300[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.7.62 Version/11.01";
const char S20301[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.7.62 Version/11.00";
const char S20302[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.7.62 Version/11.01";
const char S20303[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.7.62 Version/11.01";
const char S20304[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.7.62 Version/11.00";
const char S20305[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.7.62 Version/11.00";
const char S20306[] = "Mozilla/5.0 (Windows NT 6.1; U; ru; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 11.00";
const char S20307[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4110); ru) Presto/2.7.62 Version/11.01";
const char S20308[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.7.62 Version/11.01";
const char S20309[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4848); ru) Presto/2.7.62 Version/11.00";
const char S20310[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5218); ru) Presto/2.7.62 Version/11.00";
const char S20311[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.7.62 Version/11.00";
const char S20312[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.7.62 Version/11.01";
const char S20313[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.7.62 Version/11.01";
const char S20314[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.7.62 Version/11.01";
const char S20315[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4848); ru) Presto/2.7.62 Version/11.01";
const char S20316[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.7.62 Version/11.01";
const char S20317[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.7.62 Version/11.00";
const char S20318[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.7.62 Version/11.01";
const char S20319[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5265); ru) Presto/2.7.62 Version/11.00";
const char S20320[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.7.62 Version/11.00";
const char S20321[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.7.62 Version/11.00";
const char S20322[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.7.62 Version/11.00";
const char S20323[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.7.62 Version/11.00";
const char S20324[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.7.62 Version/11.01";
const char S20325[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.7.62 Version/11.00";
const char S20326[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.7.62 Version/11.00";
const char S20327[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5196); ru) Presto/2.7.62 Version/11.00";
const char S20328[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.7.62 Version/11.00";
const char S20329[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.7.62 Version/11.00";
const char S20330[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.7.62 Version/11.01";
const char S20331[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.7.62 Version/11.01";
const char S20332[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.7.62 Version/11.01";
const char S20333[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.7.62 Version/11.01,gzip(gfe)";
const char S20334[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5267); ru) Presto/2.7.62 Version/11.00";
const char S20335[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.7.62 Version/11.00,gzip(gfe)";
const char S20336[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.7.62 Version/11.01";
const char S20337[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.7.62 Version/11.00";
const char S20338[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5650); ru) Presto/2.7.62 Version/11.00";
const char S20339[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.7.62 Version/11.00";
const char S20340[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.7.62 Version/11.01";
const char S20341[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.7.62 Version/11.01";
const char S20342[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.7.62 Version/11.00";
const char S20343[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.7.62 Version/11.01";
const char S20344[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.7.62 Version/11.00";
const char S20345[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.7.62 Version/11.00";
const char S20346[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6060); ru) Presto/2.7.62 Version/11.00";
const char S20347[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.7.62 Version/11.01";
const char S20348[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5966); ru) Presto/2.7.62 Version/11.00";
const char S20349[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6089); ru) Presto/2.7.62 Version/11.00";
const char S20350[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5981); ru) Presto/2.7.62 Version/11.01";
const char S20351[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6578); ru) Presto/2.7.62 Version/11.01";
const char S20352[] = "Mozilla/5.0 (Linux; U; Android Bonsai/Kernel_v5.0.0-B13; en-us Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20353[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky/1.6.1 Safari/533.1";
const char S20354[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv_se) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky/1.5.0 Safari/533.1";
const char S20355[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20356[] = "stagefright/1.1 (Linux;Android 2.3.3)";
const char S20357[] = "stagefright/1.1 (Linux;Android 2.3.4)";
const char S20358[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; generic) AppleWebKit/534.51 (KHTML, like Gecko; Google Web Preview) Version/4.0 Mobile Safari/534.51";
const char S20359[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Android for Telechips TCC8900 Evaluation Board Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20360[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1101_v73_klx1ctp_noalsps Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20361[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; generic Build/4.4.1_214) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20362[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20363[] = "stagefright/1.1 (Linux;Android 2.3.5)";
const char S20364[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us) AppleWebKit/999+ (KHTML, like Gecko) Safari/999.9";
const char S20365[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; de-de Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20366[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20367[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20368[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; Android for Telechips M801 Evaluation Board Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20369[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; generic) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S20370[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20371[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20372[] = "stagefright/1.1 (Linux;Android 2.3.6)";
const char S20373[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; generic) AppleWebKit/535.1 (KHTML, like Gecko; Google Web Preview) Version/4.0 Mobile Safari/535.1";
const char S20374[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; generic) AppleWebKit/534.24 (KHTML, like Gecko; Google Web Preview) Version/4.0 Mobile Safari/534.24";
const char S20375[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-tw; eye 2828 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20376[] = "stagefright/1.1 (Linux;Android 2.3.1)";
const char S20377[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_hjy2a7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20378[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; generic) AppleWebKit/536.8 (KHTML, like Gecko; Google Web Preview) Version/4.0 Mobile Safari/536.8";
const char S20379[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; MID Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20380[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Android Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20381[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; iPhone 4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20382[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1102_v73_gq1001c Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20383[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_hjy1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20384[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; in-id; e1107_v73_jhgg1_imo Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20385[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; Android for Telechips TCC8900 Evaluation Board Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20386[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; huaqin73_gb Build/ALPS.GB.FDD2.MP.V2.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20387[] = "stagefright/1.1 (Linux;Android 2.3.7)";
const char S20388[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; 3G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20389[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; youwave_custom Build/YWV) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20390[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1109_v73_gq1002_ctp Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20391[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Android Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20392[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Android Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20393[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1109_v73_gq1002_ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20394[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; nl-nl Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20395[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; es-es Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20396[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Full Android Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20397[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MID Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20398[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; e1107_v73_jhgg1_imo Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20399[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_hjy2a6 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20400[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; hd-us; eeepc Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20401[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20402[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Android Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20403[] = "Windowshop/1.0.42 (Android/2.3.4/Kindle Fire)";
const char S20404[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1101_v73_jbl3ctp_1208 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20405[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1103_v13_jhgg1ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20406[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1101_v73_jbl3ctp_1208 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20407[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1109_v73_jxlc1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20408[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1109_v73_gq1003_rtp Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20409[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1109_v73_gq1003_rtp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20410[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1107_v73_ddb1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20411[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1101_v73_klx1ctp_capella Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20412[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1107_v73_jhgg1_ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20413[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1103_v73_jhgg1ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20414[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_jbl1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20415[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1109_v73_jxlc1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20416[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; generic Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20417[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1103_v73_hjy1ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20418[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_jbl1flash8p4 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20419[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; e1101_v73_jbl3ctp_1208_inco Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20420[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-au Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20421[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1109_v13_gq1002_ctp Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20422[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; S150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20423[] = "Dalvik/1.4.0 (Linux; U; Android 2.3; MID Build/GRH55)";
const char S20424[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_hjy1a6 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20425[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1101_v73_jbl2ctp_a519 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20426[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; SWTGT-7250 Build/gingerbread_house_r79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20427[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Parrot Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20428[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; M5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20429[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; ADR555 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20430[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_hjy1maxwest Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20431[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; i9988_custom Build/RMO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20432[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20433[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; fr-fr Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20434[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Android for Telechips M801 Evaluation Board Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20435[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; Android for Telechips M801 Evaluation Board Build/GRI40)";
const char S20436[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.1; Android for Telechips TCC8900 Evaluation Board Build/GRH78)";
const char S20437[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; Android Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20438[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; MID Build/MID) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20439[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Android Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S20440[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; Android Build/GRJ22)";
const char S20441[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; huaqin73_gb Build/GRJ22)";
const char S20442[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; MID reference device (b01ref) Build/GB.b01ref.X)";
const char S20443[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1109_v13_gq1003_rtp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20444[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1101_v73_jbl2ctp_a519 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20445[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; MID Build/GRJ22)";
const char S20446[] = "Android/2.3";
const char S20447[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1102_v73_gq1001c Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20448[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_jbaol1g18 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20449[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1109c_v73_gq1008 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20450[] = "CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 2.3.6)";
const char S20451[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_hjy2a5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20452[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1107_v73_ht_ntv Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20453[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1101_v73_nd1ctp Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20454[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; zh-cn; PAD Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S20455[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1107_v73_hjy_ctp Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20456[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1105_v73_gq1006_rtp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20457[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; T7303 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20458[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; VITA_V4W Build/HuaqinA51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20459[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1101_v73_jbl1ctp_a515 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20460[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; 7260 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S20461[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HKV MID 2S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20462[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1107_v73_hjy_ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20463[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; unknown Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20464[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1103_v73_hjy1ctp Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20465[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1107_v73_ddb1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20466[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; generic Build/unknown) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20467[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Android Build/GRJ90)";
const char S20468[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; e1107_v73_jhgg1_mox Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20469[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1107_v73_ht Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20470[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1102_v73_jbl1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20471[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1109_v73_jxlc1nps Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20472[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1107_v73_ht_84g Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20473[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1101_v73_jbl3ctp_1208_newboot Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20474[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1103_v73_jxlc1btctp Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20475[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1107_v73_jhgg1_ctp_84g Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20476[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1109_v13_gq1002_ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20477[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1103_v13_jhgg1ctpflb Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20478[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; iDroid Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20479[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1102_v73_jbl1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20480[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Android Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20481[] = "MSM7627A/1.0 Android/2.3 gingerbread/2.3.5 Release/11.16.2011 Browser/WebKit533.1 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S20482[] = "null Linux/2.6.35 Android/2.3.6 Release/null Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1 Android 2.3.5;";
const char S20483[] = "Pandora/1.6 Android/2.3.3 cdma_shadow";
const char S20484[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.2 Version/4.0 Mobile Safari/530.17";
const char S20485[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; acsip13_gb Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20486[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; O+_8.7_Android Build/OplusOplus_A100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20487[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; generic) AppleWebKit/537.4 (KHTML, like Gecko; Google Web Preview) Version/4.0 Mobile Safari/537.4";
const char S20488[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Pioneer Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20489[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.2 Version/4.0 Mobile Safari/530.17";
const char S20490[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1101_v73_jhgg1ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20491[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; POCKET Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20492[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; generic Build/GRJ22)";
const char S20493[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; iPhone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20494[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.2 Version/4.0 Mobile Safari/530.17";
const char S20495[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1109l_jyuany2_6622_nd Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20496[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; iDroid S Build/iDroidiDroid) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20497[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; MID Build/MID) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20498[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.2 Version/4.0 Mobile Safari/530.17";
const char S20499[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; generic Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20500[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; U8667 Build/HuaweiU8667) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20501[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; FLY IQ256 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20502[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_5010/5010_V1.10; 240*320; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20503[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; W1350 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20504[] = "Mozilla/5.0 (Linux; U; Android 2.3;en-us; ZTE-U V760 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20505[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.2 Version/4.0 Mobile Safari/530.17";
const char S20506[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; BlackBerry Runtime for Android Apps Build/2.1.0.131)";
const char S20507[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-pt; tmn smart a8 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20508[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Changjiang P5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20509[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; SAGETEL G3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20510[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ThL V8 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20511[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; LG-su760 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S20512[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; da-dk; CHARM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20513[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_nd1gti9220flash8p4 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20514[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U670C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20515[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-mx; M4TEL SS550 Build/GRJ90) AppleWebKit/533.1(KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20516[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; UR-A38 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20517[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; TWZ TA2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20518[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; W2400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20519[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BW-BenWee 5100S/BenWee_5100S_V1.0; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20520[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; P9100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20521[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; MTAG 351 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20522[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; H500 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20523[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; 4U Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20524[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; GP-351 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20525[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; Telenor_One_Touch_C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20526[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; WX06K Build/113.0.0027) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20527[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; MTC 960 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20528[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; e1109c_v73_gq1008a Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20529[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; x603 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20530[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; COOLPIX S800c Build/CP01_WW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20531[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; izoom i7001 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20532[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; MegaFon_SP-A10 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20533[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A99 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20534[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; sr-rs; Telenor_Smart_Pro Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20535[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; iBall-Andi4.3J Build/ALPS.GB2.MP.V1.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20536[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; Ilium_S100 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20537[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; GT7250S Build/gingerbread_qrd_r209) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20538[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; IMO S900 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20539[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; X501_USA_RS Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20540[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GIO-GiONEE_C600/CBD75B04_01_T9151 ; 320*480; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20541[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Q7079W Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20542[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; M019 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20543[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Mystique Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20544[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; A51 Build/A51Gizoom-i8004) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20545[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; yuanda73_gb Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20546[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-B9120 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20547[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HENGXUN Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20548[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; Snexian Mi320 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20549[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; X3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20550[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; B4000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20551[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; pt-br; GP-431 Build/REL_2.3.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20552[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MI360 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20553[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; maxwest Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20554[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; S9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20555[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; X40 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20556[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1101_v13_hjy1ctp Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20557[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; e1108_v73_jbl1flash_extreme Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20558[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; M20 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20559[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_jbl1a535 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20560[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; E9771 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20561[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; SmartTouch Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20562[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; e1109_v73_jy1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20563[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1103_v73_nd2ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20564[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_jbl1mt9p017 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20565[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SICT Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20566[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1109l_jyuany2_6622 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20567[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_hjy1a5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20568[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1815_v20_hy1qixian_6622 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20569[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1117_v13_jhgg Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20570[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; e1109c_v73_i9220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20571[] = "AndroidCvpPlayer(Linux; U; Android 2.3.5; en-us)";
const char S20572[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; SmartWorld Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20573[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1117_v13_jhggtv_flb Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20574[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; e1117_v73_jhgg_mox Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20575[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; e1117_v73_zxd2ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20576[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; Ilium_AVVIO_760 Build/a200Ilium_AVVIO_760) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20577[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Android Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20578[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1109c_v73_gq1008ov Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20579[] = "sprd-F200/1.0 Linux/2.6.35.7 Android/2.3.6 Release/01.21.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S20580[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1103_v73_nd1ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20581[] = "Mozilla/5.0 (Linux; U; Android 5.1.1; th-th; iPhone Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20582[] = "Mozilla/5.0 (Linux; U; Android 5.1.1; en-us; iPhone Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20583[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; O+8.1Android Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20584[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; Pioneer Build/GRK39F)";
const char S20585[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; T730 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20586[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; e1102_v73_geg1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20587[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_hjy1a6w Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20588[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; e1108_v73_jbl1a535mt9p017 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20589[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; S150 Build/GINGERBREAD)";
const char S20590[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; e1109_v73_gq1002_ctp Build/GRK39F)";
const char S20591[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PhydeuxROM Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20592[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; gb92236a Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20593[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; POCKET Build/GRK39F)";
const char S20594[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; GK-A1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20595[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de;  Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 TOLINO_BROWSER_1.1.1";
const char S20596[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; e1109c_v73_gxqg3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20597[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Build/Android_V01.01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20598[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; e1109_v73_gq1002_ctp Build/GRJ22)";
const char S20599[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; e1117_v73_jhgg Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20600[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Parrot Build/GRJ90)";
const char S20601[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; e1108_v73_jbl1custom8p4 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20602[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; MID Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20603[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Android Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S20604[] = "Android 2.3.6; Application: com.e8tracks";
const char S20605[] = "TCL-E906_TD/1.0 Android/2.3 (Linux; U; Android 2.3.7; zh-cn) Release/12.09.2011 Browser/WAP 2.0 Mozilla/5.0 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20606[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; e1815_v20_lbx2_6622 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20607[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Phone 5s Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20608[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1109l_jyuany2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20609[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pl-pl; LUX Build/LUX01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20610[] = "Mozilla/5.0 (Linux; U; Android 2.3.9; es-es; HAPPLE Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20611[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; e1117_v73_zxd3ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20612[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1107_v13_jhgg1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20613[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; tr-tr Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20614[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1102_v73_jbl1_lt18i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20615[] = "Android-Cloud-X2/1.0 Linux/2.6.35.7 Android/2.3 Release/06.17.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S20616[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; iPhone 4S (MC605CH) Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20617[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; e1109c_v73_jhgg1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20618[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; e1101_v73_jhgg1ctp_vi Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20619[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-; hexing75_hspa   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20620[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; el-gr; e1117_v73_zxd3ctp_turbox Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20621[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn;Iphone 4S Build/Iphone 4S V1.0.0) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20622[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; hexing75_hspa   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20623[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; e1108_v73_jbl1_top5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20624[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1815_hy1qixian_i667 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S20625[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S20626[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P3110 Build/JRO03C)";
const char S20627[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S20628[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S20629[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P3110 Build/JZO54K)";
const char S20630[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GT-P3110 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S20631[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S20632[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; GT-P3110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S20633[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S20634[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S20635[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/5.0.2 Version/4.0 Safari/534.30 Safari/6533.18.5";
const char S20636[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.3.4-94 (Android 4.1.1; samsung GT-P3110 Build/JRO03C.P3110XXCLK7)";
const char S20637[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S20638[] = "Logos/2.2.8.1 (Linux; U; Android 4.1.1; en-GB; GT-P3110 Build/JRO03C.P3110XXCLK7)";
const char S20639[] = "BBC iPlayer (AndroidApp; 1.5.0.666; X) GT-P3110 (Android 4.1.1, SDK 16)";
const char S20640[] = "Biblia/2.2.8.1 (Linux; U; Android 4.1.1; es-MX; GT-P3110 Build/JRO03C.P3110XXCLK5)";
const char S20641[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3110 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S20642[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3110 Build/JRO03C) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.4 Version/4.0 Mobile Safari/530.17 baidubrowser/061_1.8.4.2_diordna_679_006/gnusmas_61_1.1.4_0113P-TG/Xy/92E1087FBD8BB71D23F46C0EFX%7C0/1";
const char S20643[] = "BBC News (AndroidApp; 2.1; 30) GT-P3110 (Android 4.1.1, SDK 16)";
const char S20644[] = "BBC iPlayer (AndroidApp; 1.5.2.170; X) GT-P3110 (Android 4.1.1, SDK 16)";
const char S20645[] = "com.google.android.youtube/4.3.9(Linux; U; Android 4.1.1; en_GB; GT-P3110 Build/JRO03C) gzip";
const char S20646[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P3110 Build/JRO03C) [FBAN/Orca-Android;FBAV/2.3.2-release;FBLC/en_GB;FBBV/X;FBCR/null;FBMF/samsung;FBBD/samsung;FBDV/GT-P3110;FBSV/4.1.1]";
const char S20647[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; GT-P3110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S20648[] = "Mozilla/5.0 (Linux; U;Android 4.1.1; fr-fr; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S20649[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30";
const char S20650[] = "GoogleAnalytics/2.0 (Linux; U; Android 4.1.2; en-gb; GT-P3110 Build/JZO54K)";
const char S20651[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 GoogleGoggles-Android/1.9; gzip";
const char S20652[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P3110 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S20653[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Bui Android 4.1.1; en-gb; GT-P3110 HTML, like Gecko) Version/4.0 Safari/534.30";
const char S20654[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3110 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 GoogleGoggles-Android/1.9; gzip";
const char S20655[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWn-gb; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S20656[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like 6236.8.10.X; __utmc4.30";
const char S20657[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JR5D; _qsst=X; qb_etc_ likus=%7B%22c%22%3A%7B%7D%7D; __utma=X.X.X.X.X.1; __utmb=X.14.10.X; __utmc=X; __utmz=X.X.1.1.utmgclid=cjsardbeprycfunmtaodplkaxa|utmccn=brand%20pure%20(pc)%20-%20exact|utmcmd=ppc|utmctr=harveys; qb_ss_status=%7B%22c%22%3A%7B%7D%2C%22cg%22%3A0.X%7D";
const char S20658[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRedetails%3Dital%2520white%22%2C%22time%22%3AX%7D%5D%2C%22sessionLandingPage%22%3A%22http%3A//www.harveysfurniture.co.uk/mobile/products/dining-room/categories/display-cabinets.aspx%3Frangedetails%3Dital%2520white%22%2C%22__v%22%3A%7B%7D%7D; _qubitTracker=X.X; _qPageNum_harveysfurniture=3; _qst=%5B1%2C0%5D; _qsst=X; qb_etc_status=%7B%22c%22%3A%7B%7D%7D; qb_ss_status=%7B%22c%22%3A%7B%7D%2C%22cg%2";
const char S20659[] = "Mozilla/5.0 (Ultimate Edition; X11; Linux x86; rv:8.0) Gecko/X Ultimate Edition / 3.2 Firefox/10.0";
const char S20660[] = "Mozilla/5.0 (X11; Linux x86_64; rv:10.0.11) Gecko/X Firefox/10.0.11";
const char S20661[] = "Mozilla/5.0 (X11; Linux x86_64; rv:10.0.12) Gecko/X Firefox/10.0.12 Iceweasel/10.0.12 Snapper/1.0";
const char S20662[] = "Mozilla/5.0 (X11; Linux x86_64; rv:10.0.12) Gecko/X Iceweasel/10.0.12";
const char S20663[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A403 Safari/7534.48.3";
const char S20664[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A406 Safari/7534.48.3";
const char S20665[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A407 Safari/7534.48.3";
const char S20666[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A523 Safari/7534.48.3";
const char S20667[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A8426 Safari/7534.48.3";
const char S20668[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A550 Safari/7534.48.3";
const char S20669[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A8500 Safari/7534.48.3";
const char S20670[] = "Web/1.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20671[] = "Million%20HD%20WPs/8.80.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20672[] = "2%20Million%20WPs/9.16.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20673[] = "Feeds/2.2.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20674[] = "WOW%20WPs/9.26.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20675[] = "ABCNews/1.4 CFNetwork/485.13.9 Darwin/11.0.0";
const char S20676[] = "locationd (unknown version) CFNetwork/485.13.9 Darwin/11.0.0";
const char S20677[] = "ChromeSync%20Lite/1.2 CFNetwork/485.13.9 Darwin/11.0.0";
const char S20678[] = "SexyNFunWPs/8.50.1 CFNetwork/485.13.9 Darwin/11.0.0";
const char S20679[] = "NaverCafe/1.1.9 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20680[] = "Clipish/9.29.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20681[] = "HideMyPhotos/9.29.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20682[] = "ZBar/2.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20683[] = "GQMagazine/1.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20684[] = "EpicWallPaper2011/1.6 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20685[] = "HOT%20HD%20WPs/9.19.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20686[] = "Animations/9.30.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20687[] = "BrowserFree/1.2.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20688[] = "MobileFilter/3.01 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20689[] = "Web%20Feeds/1.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20690[] = "SpringBoard/50 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20691[] = "TweetSpeaker/1.0.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20692[] = "Celeb%20Wallpapers/9.13.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20693[] = "vBrowse%202.0/2.03 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20694[] = "iScanner/1.4 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20695[] = "Settings/1.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20696[] = "VanillaSurf/2.9.6 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20697[] = "ImageSearcherS/2.1.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20698[] = "DL-Picture/1.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20699[] = "Diigo%20Browser/2.6 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20700[] = "kikin/1.9.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20701[] = "Love%20Messenger/9.16.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20702[] = "Codes%20Reader/2.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20703[] = "Pinterest/1.5.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20704[] = "Dot%20Lock/2.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20705[] = "Fera/5.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20706[] = "Dual%20Browser/1.4.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20707[] = "TheFeed/1.0.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20708[] = "ForumRunner/41 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20709[] = "iBuilder/1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20710[] = "SexyShelves/9.02.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20711[] = "Love%20Pictures/3.6 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20712[] = "ImageSearch/1.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20713[] = "iCandyWP1/9.15.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20714[] = "BarCodeScan/1.1.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20715[] = "Epic%20Fail/2.7.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20716[] = "PimpYourPic/9.14.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20717[] = "Downloader%20copy/1.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20718[] = "ImageSearcherProS/2.1.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20719[] = "ImageSearcherFree/1.0.5 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20720[] = "ImageSearcherPro/2.1.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20721[] = "Images/1.4 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20722[] = "ImageSearcher/3.1.3 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20723[] = "Mercury3/6.0.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20724[] = "ImageSearcherFreeS/1.0.5 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20725[] = "Mercury3Free/6.0.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20726[] = "360%20Browser/5.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20727[] = "CFKnottsBerryFarm/61 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20728[] = "CFCanadaWonderland/48 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20729[] = "CFWorldsOfFun/56 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20730[] = "browser/31759 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20731[] = "InstapaperPro/4.0.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20732[] = "RedLaser/3.3.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20733[] = "NetNewsWire/1.0.5 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20734[] = "Reader/2.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20735[] = "iSocialize2/31 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20736[] = "FastBrowser/1.7 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20737[] = "NewsMe/1.2.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20738[] = "xFeed/2.0.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20739[] = "VideoDownloaderLite/2.0.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20740[] = "LexPrint/3.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20741[] = "QihooBrowser/2.5.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20742[] = "Tweetlogix/1.9.5 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20743[] = "MyMedia/2.41 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20744[] = "Readability/1.1.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20745[] = "360%20Lite/5.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20746[] = "LastPass%20Tab/2.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20747[] = "JDate/101 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20748[] = "Corus/3.0.3832 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20749[] = "Browser/1.2.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20750[] = "Dolphin%20HD/5.3 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20751[] = "PhotoFind/5.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20752[] = "Pulse%20News/2.9.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20753[] = "Terra/2.0.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20754[] = "Evanov-CidcRadio/2.0.5369 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20755[] = "MyPad/2.6.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20756[] = "MobileLearn/3.0.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20757[] = "$%7BPRODUCT_NAME%7D/1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20758[] = "FML/2.3.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20759[] = "Reddit%20Pics/1.6 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20760[] = "Demotivation/2.5.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20761[] = "MyPad%20for%20iPhone/2.6.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20762[] = "FindMyiPhone/1.4 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20763[] = "ImageProHD/1.9 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20764[] = "ImageProHDFree/1.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20765[] = "Silly%20Pics/1.1.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20766[] = "Cydia/0.9 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20767[] = "SpeakText%20FREE/12.4 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20768[] = "Scribbler/1.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20769[] = "Scan/38 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20770[] = "Azul/3.6 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20771[] = "iAnnotate%20PDF/2.2.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20772[] = "myPDFReader_old_design/1.0.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20773[] = "ePrint/4.8 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20774[] = "Photo%20Gallery%20Pro/2.6 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20775[] = "KaywaReader/1.21 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20776[] = "Download%20Free/2.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20777[] = "Extras4iMovie/9.30.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20778[] = "Digg/X.3.0.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20779[] = "AtomicBrowser/7.0.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20780[] = "MyPad_Premium/2.6.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20781[] = "123people/3.1.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20782[] = "Qrafter/5.6 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20783[] = "match.com/2.1.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20784[] = "FanballNews/1.1.2316 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20785[] = "BejBlitz/588 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20786[] = "Download/2.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20787[] = "Slotomania/1.12.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20788[] = "iDownloaderPro/2.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20789[] = "NFL_US_Premium/247 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20790[] = "ImageHunter/2.4.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20791[] = "DiigoBrowser/4.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20792[] = "Pic%20Collage/2.21.14 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20793[] = "Qriket%20Game/3.1.5 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20794[] = "AngryBirdsFree/1.2.1 CFNetwork/485.13.9 Darwin/11.0.0";
const char S20795[] = "Maps/1.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20796[] = "Netflix/X CFNetwork/548.1.4 Darwin/11.0.0";
const char S20797[] = "MusicDownloader/2.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20798[] = "WordsWithFriendsFree/4.13 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20799[] = "Perfect%20Browser/8.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20800[] = "ScoopIt/55 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20801[] = "SmartSync/4.5.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20802[] = "WEB%20to%20PDF/2.3 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20803[] = "SuperProber/3.0 CFNetwork/485.13.9 Darwin/11.0.0";
const char S20804[] = "Job%20Hopper/1.3 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20805[] = "iCarousel/1.0.5 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20806[] = "FreeCell/2.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20807[] = "Fun%20Animations/9.29.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20808[] = "GoodReader/3.14.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20809[] = "ESCPR01/2.2.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20810[] = "Wallpapers/1.6 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20811[] = "Million%20Photos/8.60.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20812[] = "Weather/1.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20813[] = "CommCenterMobileHelper/1285.21 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20814[] = "The_Boys/4.1.6 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20815[] = "Downloader/1.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20816[] = "Amazon/2.0.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20817[] = "AppCake3/3.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20818[] = "PDF%20PROvider/2.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20819[] = "Y_Messenger/5467 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20820[] = "SliceMobile/2.0.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20821[] = "MusicDownloaderLite/2.2.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20822[] = "FeeddlerPro/1.12.5 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20823[] = "Google/2.5.1.13455 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20824[] = "AtBat.Full/1497 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20825[] = "Action%20Movie/1.2.7 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20826[] = "Cooliris/68643 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20827[] = "CEBrowser/1.6.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20828[] = "adlibd (unknown version) CFNetwork/485.13.9 Darwin/11.0.0";
const char S20829[] = "GroupMe/3.3.4 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20830[] = "Apsley/2.3 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20831[] = "%20iSearch/1.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20832[] = "PDF%20Printer/2.0.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20833[] = "vBrowse/2.04 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20834[] = "AtomicLite/7.0.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20835[] = "GoodReaderIPad/3.18.6 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20836[] = "Keeper/5.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20837[] = "Textfree/5.0.5 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20838[] = "FindMyFriends/1.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20839[] = "ScoreCenter/322 CFNetwork/548.0.4 Darwin/11.0.0";
const char S20840[] = "syncdefaultsd (unknown version) CFNetwork/548.1.4 Darwin/11.0.0";
const char S20841[] = "iReddit/2.1.5 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20842[] = "Free%20Video%20Downloader/1.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20843[] = "Instapaper/4.2.7 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20844[] = "TEE2/2.3.4 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20845[] = "textPlus/6698 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20846[] = "ZillowMap/5.3.0.86 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20847[] = "FindMyiPhone/1.4 CFNetwork/548.0.4 Darwin/11.0.0";
const char S20848[] = "Apolloplus/2.7.10 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20849[] = "MSN/1.1.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20850[] = "BarcodeFree/1.0.7 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20851[] = "securityd (unknown version) CFNetwork/548.1.4 Darwin/11.0.0";
const char S20852[] = "Crackle/2.4.8 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20853[] = "itunesstored (unknown version) CFNetwork/548.1.4 Darwin/11.0.0";
const char S20854[] = "iDownloader/2.5 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20855[] = "Chess/2.4.5 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20856[] = "Mega%20Run/1.2.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20857[] = "iPhone/X CFNetwork/548.0.4 Darwin/11.0.0";
const char S20858[] = "Installous/1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20859[] = "ChasingYello/1.0.4 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20860[] = "SportsTap/305 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20861[] = "Funny%20Pics/2.93 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20862[] = "ysearch2/3.0.3 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20863[] = "Searchx/2.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20864[] = "Hot%20Wallpapers/10.02 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20865[] = "FastFood/4.0.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20866[] = "BubbleBreakerPro%20Free/1.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20867[] = "ScrambleWithFriendsFree/4.15 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20868[] = "TwitterFon/583 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20869[] = "SolitaireFree/3.6 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20870[] = "Solitaire%20Free/1.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20871[] = "midomi-free/5.0.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20872[] = "Prismatic/1.0.4 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20873[] = "ApolloVideoDownloaderFree/1.2.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20874[] = "urbanspin/2.1.0.17 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20875[] = "iPick/200 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20876[] = "ImageMobile/1.8 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20877[] = "tfctv/1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20878[] = "Homes/1.5 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20879[] = "HD%20&%20Retina%20Wallpapers/1.6 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20880[] = "MONOPOLY%20Hotels/1.0.28 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20881[] = "PicFrame_ver_1.0.0/1.3 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20882[] = "NYTimes/9886.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20883[] = "LineRunner/1.9 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20884[] = "eBay/2.1.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20885[] = "ScrambleWithFriendsPaid/4.17 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20886[] = "UberSocial/1.0.0.251 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20887[] = "VideoDownloader/2.2.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20888[] = "playable/3.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20889[] = "ImagePro/2.6 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20890[] = "Newsify/87 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20891[] = "BaiduHD/2.0.3 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20892[] = "CLIPish%20Pro2/10.11 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20893[] = "AlienBlue/2.6.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20894[] = "Pandora/3.1.17 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20895[] = "Us/1.3 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20896[] = "HootSuite/2.4.7 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20897[] = "EditorsChoice/10451.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20898[] = "MatchingWithFriendsFree/4.12 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20899[] = "iGmail/9.6.3 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20900[] = "Qrafter%20Pro/5.6 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20901[] = "Holidays/10.01 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20902[] = "ooVoo/1.1.2.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20903[] = "SecureBrowser/3.6.0%20Beta CFNetwork/548.1.4 Darwin/11.0.0";
const char S20904[] = "iDownloaderLite/1.0.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20905[] = "Groupon/953 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20906[] = "TubeDownloaderLite/2.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20907[] = "AdSheet/1.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20908[] = "OregonSettler/1.0.7 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20909[] = "ApolloGeneralDownloader/1.1.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20910[] = "BarcodeQRFree/1.0.5 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20911[] = "TubeDownloader/2.2.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20912[] = "Downloader-Pro/2.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20913[] = "DreamLeague/1.19 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20914[] = "Cool%20Retina%20Wallpapers/1.5 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20915[] = "AirWatch/4.2.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20916[] = "ImageDownloader/1.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20917[] = "Mobile%20Learn/3.1.4 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20918[] = "MusicDownloaderBSOFree/1.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20919[] = "VideoDLPlusPlusFree/1.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20920[] = "NortonSafeBrowser/168.2.5.14 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20921[] = "MSNBC/2.6.3 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20922[] = "HuffPost/2.9.2.7 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20923[] = "KakaoTalk/3.0.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20924[] = "wewsiphone/11710 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20925[] = "iFunny/22 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20926[] = "Flow/1.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20927[] = "mallstreet/627 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20928[] = "VideoDLFree/1.1.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20929[] = "Talking%20Ginger/1.4 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20930[] = "MxBrowser-iPhone/14 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20931[] = "%E5%BF%AB%E6%92%AD%E5%A2%9E%E5%BC%BA%E7%89%88/1977 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20932[] = "Twitter/4.1.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20933[] = "HuffingtonPost/2.8.5.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20934[] = "MusicDownloaderSSAFree/1.0.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20935[] = "UnlimitedSMS/4.4.23 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20936[] = "AlienBlueHD/2.2.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20937[] = "LinkedIn/6.1.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20938[] = "RageComics/3.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20939[] = "Smurfs-IPad/1.2.5 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20940[] = "SSC%20Forums/X CFNetwork/548.1.4 Darwin/11.0.0";
const char S20941[] = "Florida/110 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20942[] = "MxBrowser-iPhone/17 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20943[] = "%E5%A4%9A%E5%B1%8F%E4%BA%92%E5%8A%A8%E6%B5%8F%E8%A7%88/1976 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20944[] = "logosQuiz/1.5.6 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20945[] = "Washington/208 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20946[] = "Pennsylvania/1153 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20947[] = "Instagram/2.4.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20948[] = "subwaysurfers/1.2.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20949[] = "Georgia/376 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20950[] = "LetsGolf3/1.1.0 CFNetwork/485.13.9 Darwin/11.0.0";
const char S20951[] = "NorthCarolina/296 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20952[] = "Differences/1.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20953[] = "MxBrowser-iPhone/13 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20954[] = "Coast/1.0.1.62563 CFNetwork/548.1.4 Darwin/11.0.0";
const char S20955[] = "Mozilla/5.0 (Windows NT 6.1; U; ru; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.60";
const char S20956[] = "Mozilla/5.0 (Windows NT 6.1; U; ru; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.50";
const char S20957[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; pl; rv:1.9.1) Gecko/X Firefox/3.5 (.NET CLR 3.5.30729)	Firefox 3.5";
const char S20958[] = "Mozilla/5.0 (Windows NT 6.1; U; YB/3.5.1; ru; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.63";
const char S20959[] = "Mozilla/5.0 (Windows NT 6.1; U; en-GB; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.51";
const char S20960[] = "Mozilla/5.0 (Windows NT 6.1; U; en; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 11.10";
const char S20961[] = "Mozilla/5.0 (Windows NT 6.1; U; ru; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.70";
const char S20962[] = "Mozilla/5.0 (Windows NT 6.1; U; ru; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.52";
const char S20963[] = "Mozilla/5.0 (Windows NT 6.1; U; ru; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.53";
const char S20964[] = "Mozilla/5.0 /Windows; U; Windows NT 6.1; de; rv:1.9.1.5) Gecko/X Firefox/3.5.5";
const char S20965[] = "Mozilla/5.0 (Windows NT 6.1; U; zh-cn; rv:1.9.1.6) Gecko/X Firefox/3.5.6";
const char S20966[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en_US; rv:1.9.1.9) Gecko/X Firefox/3.5.9";
const char S20967[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Maxthon/4.2.0.800 Chrome/30.0.1551.0 Safari/537.36";
const char S20968[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S20969[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S20970[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36,gzip(gfe)";
const char S20971[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Maxthon/4.2.0.800 Chrome/30.0.1551.0 Safari/537.36";
const char S20972[] = "Mozilla/5.0 (Windows; U;MSIE9.0; Windows NT 9.0; it-IT) AppleWebKit/525 (KHTML, like Gecko) BrowserNG/7.2.6.9 3gpp-gba";
const char S20973[] = "Mozilla/3.0 (compatible; WebCapture 2.0; Auto; Windows)";
const char S20974[] = "Mozilla/0.91 Beta (Windows)";
const char S20975[] = "Mozilla/4.0 (compatible; Lotus-Notes/6.0; Windows-NT)";
const char S20976[] = "Windows-Media-Player/9.00.00.4503";
const char S20977[] = "Windows-Media-Player/11.0.5721.5268";
const char S20978[] = "Windows-Media-Player/10.00.00.xxxx";
const char S20979[] = "Mozilla/4.0 (compatible; Lotus-Notes/5.0; Windows-NT)";
const char S20980[] = "LinkExaminer/1.01 (Windows)";
const char S20981[] = "Mozilla/4.0 (compatible; WebCapture 3.0; Windows)";
const char S20982[] = "Windows-Media-Player/11.0.5721.5280";
const char S20983[] = "Windows-Media-Player/10.00.00.4081";
const char S20984[] = "Windows-Media-Player/12.0.7600.16667";
const char S20985[] = "Windows-Media-Player/11.0.6002.18311";
const char S20986[] = "Windows-Media-Player/9.00.00.4510";
const char S20987[] = "FeedDemon/4.0 (http://www.feeddemon.com/; Microsoft Windows)";
const char S20988[] = "Mozilla/4.0 (compatible; MSIE5.0; Windows NT; DigExt)";
const char S20989[] = "Mozilla/4.0 (compatible; Check&Get 3.4; Windows NT)";
const char S20990[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.6";
const char S20991[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.7";
const char S20992[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.7.1";
const char S20993[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.0";
const char S20994[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/526.9+ (KHTML, like Gecko) AdobeAIR/1.5.3";
const char S20995[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.5.1";
const char S20996[] = "Mozilla/4.0 (compatible; MSIE 6.1; Windows NT)";
const char S20997[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.0.2";
const char S20998[] = "curl/7.19.2 (i386-pc-win32) libcurl/7.19.2 OpenSSL/0.9.8i zlib/1.2.3";
const char S20999[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.0.3";
const char S21000[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.5";
const char S21001[] = "curl/7.21.0 (amd64-pc-win32) libcurl/7.21.0 OpenSSL/0.9.8o zlib/1.2.3";
const char S21002[] = "FeedDemon/3.0 (http://www.feeddemon.com/; Microsoft Windows)";
const char S21003[] = "curl/7.21.6 (amd64-pc-win32) libcurl/7.21.6 OpenSSL/0.9.8r zlib/1.2.5";
const char S21004[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/523+ (KHTML, like Gecko) AdobeAIR/1.5 Community Help Client 1.0";
const char S21005[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.1";
const char S21006[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.2";
const char S21007[] = "curl/7.10.4 (win32) libcurl/7.10.4 zlib/1.1.4";
const char S21008[] = "curl/7.22.0 (i386-pc-win32) libcurl/7.22.0 zlib/1.2.5";
const char S21009[] = "Mozilla/5.0(Windows; U) Gecko/X";
const char S21010[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/526.9+ (KHTML, like Gecko) AdobeAIR/1.5";
const char S21011[] = "curl/7.21.1 (i386-pc-win32) libcurl/7.21.1 zlib/1.2.5";
const char S21012[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.2 X/Yammer";
const char S21013[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.0.4";
const char S21014[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.0";
const char S21015[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.3";
const char S21016[] = "FeedDemon/4.1 (http://www.feeddemon.com/; Microsoft Windows)";
const char S21017[] = "curl/7.10.8 (win32) libcurl/7.10.8 OpenSSL/0.9.7c";
const char S21018[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/523+ (KHTML, like Gecko) AdobeAIR/1.1";
const char S21019[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/526.9+ (KHTML, like Gecko) AdobeAIR/1.5.2";
const char S21020[] = "curl/7.19.7 (i386-pc-win32) libcurl/7.19.7 zlib/1.2.3";
const char S21021[] = "curl/7.23.1 (x86_64-pc-win32) libcurl/7.23.1 OpenSSL/0.9.8r zlib/1.2.5";
const char S21022[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.4";
const char S21023[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/526.9+ (KHTML, like Gecko) AdobeAIR/1.5.1";
const char S21024[] = "curl/7.23.1 (x86_64-pc-win32) libcurl/7.23.1";
const char S21025[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.5";
const char S21026[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.4 Zoodles/3.0.5 Safari/525.27.1";
const char S21027[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.6";
const char S21028[] = "curl/7.12.1 (i386-pc-win32) libcurl/7.12.1 OpenSSL/0.9.7d";
const char S21029[] = "Mozilla/5.0 (Windows; U;) AppleWebKit/532.0 (KHTML, like Gecko) Iron";
const char S21030[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.7";
const char S21031[] = "FeedDemon/4.5 (http://www.feeddemon.com/; Microsoft Windows)";
const char S21032[] = "Mozilla/5.0 (Windows; U; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.8";
const char S21033[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7000 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21034[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7000 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21035[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21036[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7000 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21037[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7000 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21038[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N7000 Build/JZO54K)";
const char S21039[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-N7000 Build/JRO03C)";
const char S21040[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHW-M480W Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S21041[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHW-M480S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S21042[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHW-M480K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S21043[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E160L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21044[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E160K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21045[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E160S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21046[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7000 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S21047[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-N7000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21048[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E160L Build/JZO54K)";
const char S21049[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E160S Build/JZO54K)";
const char S21050[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E160K/KKJMC3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21051[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-N7000/N7000XXLSZ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21052[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E160K Build/JZO54K)";
const char S21053[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-N7000B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21054[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N7000B Build/JZO54K)";
const char S21055[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; SAMSUNG GT-N7000/N7000XXLT5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21056[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-i889 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21057[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-N7000/N7000XXLT9 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21058[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21059[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SC-05D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21060[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000-ORANGE/N7000XXLT9 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21061[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHW-M480W Build/JZO54K)";
const char S21062[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000-ORANGE/N7000XXLSZ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21063[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E160K/KKJMH1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21064[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N7000 Build/VeTW3.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21065[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-N7000/N7000XXLTA Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21066[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SC-05D Build/JZO54K)";
const char S21067[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-S6500T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21068[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-S6500/S6500BULC5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21069[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; GT-S6500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21070[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6500D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21071[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S6500D Build/GINGERBREAD)";
const char S21072[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S6500/S6500BVLD2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21073[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6500-ORANGE/S6500BVLD2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21074[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-S6500L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21075[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hr-hr; SAMSUNG GT-S6500D/S6500DXXLD3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21076[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; GT-S6500-ORANGE/S6500BVLC5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21077[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6500-ORANGE/S6500BVLE3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21078[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S6500/S6500BVLE3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21079[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pl-pl; SAMSUNG GT-S6500/S6500XXLD1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21080[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S6500 Build/GINGERBREAD)";
const char S21081[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-S6500D/S6500DXXLE2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21082[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-at; SAMSUNG GT-S6500/S6500XXLE1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21083[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; SAMSUNG GT-S6500D/S6500DXXLC4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21084[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S6500/S6500BVLG2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21085[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6500-ORANGE/S6500BVLG2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21086[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-S6500/S6500BULE3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21087[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hr-hr; SAMSUNG GT-S6500D/S6500DXXLH2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21088[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S6500D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21089[] = "Mozilla/5.0 (Linux; U; it-it; GT-S6500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S21090[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-at; SAMSUNG GT-S6500/S6500XXLH2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21091[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-S6500/S6500BULK2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21092[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hr-hr; SAMSUNG GT-S6500D/S6500DXXLK2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21093[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S6500L Build/GINGERBREAD)";
const char S21094[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-S6500-ORANGE/S6500BVLK2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21095[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; cs-cz; SAMSUNG GT-S6500/S6500XXLK2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21096[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S6500T Build/GINGERBREAD)";
const char S21097[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S6500/S6500BVLK2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21098[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6500D Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S21099[] = "Mozilla/5.0 (Linux; U; es-es; GT-S6500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S21100[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-S6500/S6500BUMD1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21101[] = "Mozilla/5.0 (Linux; U; el-gr; GT-S6500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S21102[] = "Mozilla/5.0 (Linux; U; hu-hu; GT-S6500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S21103[] = "Mozilla/5.0 (Linux; U; cs-cz; GT-S6500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S21104[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S6500T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21105[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hr-hr; SAMSUNG GT-S6500D/S6500DXXMF1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21106[] = "Opera/7.11 (Windows NT 5.1; U) [en]";
const char S21107[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.54 [en]";
const char S21108[] = "Opera/7.54 (Windows NT 5.1; U)  [pl]";
const char S21109[] = "Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.1) Opera 7.01 [en]";
const char S21110[] = "Opera/7.60 (Windows NT 5.2; U)  [en] (IBM EVV/3.0/EAK01AG9/LE)";
const char S21111[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 7.60";
const char S21112[] = "Opera/7.x (Windows NT 5.1; U) [en]";
const char S21113[] = "Opera/7.50 (Windows XP; U)";
const char S21114[] = "Opera/7.50 (Windows NT 5.1; U) [en]";
const char S21115[] = "Opera/7.51 (Windows NT 5.1; U) [en]";
const char S21116[] = "Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.1) Opera 7.02 [en]";
const char S21117[] = "Opera/7.22 (Windows NT 5.1; U) [en]";
const char S21118[] = "Opera/7.10 (Windows NT 5.1; U) [en]";
const char S21119[] = "Opera/7.20 (Windows NT 5.1; U) [en]";
const char S21120[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.20 [de]";
const char S21121[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.10 [en]";
const char S21122[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.51 [en]";
const char S21123[] = "Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows XP) Opera 7.0 [en]";
const char S21124[] = "Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.1) Opera 7.04 [de]";
const char S21125[] = "Opera/7.02 (Windows NT 5.1; U) [fr]";
const char S21126[] = "Opera/7.20 (Windows NT 5.1; U; pl) Version/10.7";
const char S21127[] = "Opera/7.40 (Windows NT 5.1; U; pl) Version/10.7";
const char S21128[] = "Opera/7.20 (Windows NT 5.1; U; pl) Presto/2.2.15 Version/10.10";
const char S21129[] = "Opera/7.30 (Windows NT 5.1; U; pl) Version/10.7";
const char S21130[] = "Opera/7.10 (Windows NT 5.1; U; pl) Version/10.7";
const char S21131[] = "Opera/7.30 (Windows NT 5.1; U; pl) Presto/2.3.15 Version/10.10";
const char S21132[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.23 [ru]";
const char S21133[] = "Opera/7.40 (Windows NT 5.1; U; pl) Presto/2.4.15 Version/10.10";
const char S21134[] = "Opera/7.10 (Windows NT 5.1; U; pl) Presto/2.1.15 Version/10.10";
const char S21135[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.52 [en]";
const char S21136[] = "Opera/7.03 (Windows NT 5.1; U) [en]";
const char S21137[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.11 [en]";
const char S21138[] = "Opera/7.23 (Windows NT 5.1; U)  [en]";
const char S21139[] = "Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.1) Opera 7.03 [ru]";
const char S21140[] = "Opera 7.5 (Win XP): Opera/7.50 (Windows XP; U)";
const char S21141[] = "('Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.1) Opera 7.02 [en]', 'Opera 7.02, Windows XP')";
const char S21142[] = "Opera 7.51 (Win XP): Opera/7.51 (Windows NT 5.1; U)";
const char S21143[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.60 [en] (IBM EVV/3.0/EAK01AG9/LE)";
const char S21144[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.53 [ru]";
const char S21145[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.21 [ru]";
const char S21146[] = "Opera/7.52 (Windows NT 5.1; U) [ru]";
const char S21147[] = "Mozilla/3.0 (Windows NT 5.1; U) Opera 7.23 [ru]";
const char S21148[] = "Opera/7.0 (Windows NT 5.1; U; [en])";
const char S21149[] = "Mozilla/5.0 (Windows NT 5.1; U) Opera 7.54 [ru]";
const char S21150[] = "Opera/7.01 (Windows NT 5.1; U) [ru]";
const char S21151[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1";
const char S21152[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1";
const char S21153[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1,gzip(gfe)";
const char S21154[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1,gzip(gfe)";
const char S21155[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.4.1000  Safari/537.1";
const char S21156[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1";
const char S21157[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1";
const char S21158[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1,gzip(gfe)";
const char S21159[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1,gzip(gfe)";
const char S21160[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1";
const char S21161[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21162[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.21 (KHTML, like Gecko) Chrome/11.0.678.0 Safari/534.21";
const char S21163[] = "Mozilla/5.0 (X11; Jolicloud Linux i686) AppleWebKit/534.24 (KHTML, like Gecko) Joli OS/1.2 Chromium/11.0.696.14 Chrome/11.0.696.14 Safari/534.24";
const char S21164[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.23 (KHTML, like Gecko) Chrome/11.0.686.3 Safari/534.23";
const char S21165[] = "Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24";
const char S21166[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko; Google Web Preview) Chrome/11.0.696 Safari/534.24";
const char S21167[] = "Mozilla/5.0 (X11; U; Linux; en-us) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21168[] = "Mozilla/5.0 (X11; Linux x86_64; en_us) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21169[] = "Mozilla/5.0 Slackware/13.37 (X11; U; Linux x86_64; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/11.0.696.50";
const char S21170[] = "Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US) AppleWebKit/534.18 (KHTML, like Gecko) Chrome/11.0.660.0 Safari/534.18";
const char S21171[] = "Mozilla/5.0 (Linux;U;Linux x86_64;en-us;I10a Build/IML74K) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21172[] = "Mozilla/5.0 (X11; Linux x86_64; en-us; TB782B Build/Ice Cream) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21173[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24,gzip(gfe) (via translate.google.com)";
const char S21174[] = "Mozilla/5.0 (X11; Linux x86_64; Z520m; en-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21175[] = "Mozilla/5.0 (X11; Linux x86_64; T320a; en-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21176[] = "Mozilla/5.0 (X11; Linux x86_64; T320a; fr-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21177[] = "Mozilla/5.0 (X11; Linux x86_64; Z520m; fr-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21178[] = "Mozilla/5.0 (X11; Linux x86_64; A320a; en-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21179[] = "Mozilla/5.0 (X11; Linux x86_64; A320a; fr-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21180[] = "Mozilla/5.0 (X11; Linux x86_64; en_gb) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21181[] = "Mozilla/5.0 (Linux;U;Linux x86_64;en-us;GA20 Build/IML74K) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21182[] = "Mozilla/5.0 (Linux;U;Linux x86_64;en-us;I10-12 Build/IML74K) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21183[] = "Mozilla/5.0 (Linux;U;Linux x86_64;en-us;I7a Build/IML74K) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21184[] = "Mozilla/5.0 (X11; Linux x86_64; en-us; TB1082 Build/Ice Cream) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21185[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24,gzip(gfe)";
const char S21186[] = "Mozilla/5.0 (Linux;U;Linux x86_64;en-us;i7c Build/IML74K) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21187[] = "Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Safari/534.24";
const char S21188[] = "Mozilla/5.0 (X11; Linux x86_64; en-us; CMP741E) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21189[] = "Mozilla/5.0 (X11; U; Linux i686) AppleWebKit/534.22 (KHTML, like Gecko) Chrome/11.0.683.0 Safari/534.22";
const char S21190[] = "Mozilla/5.0 (i7a; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21191[] = "Mozilla/5.0 (X11; Linux x86_64; LT26i-o) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21192[] = "Mozilla/5.0 (X11; ) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21193[] = "Mozilla/5.0 (X11; Linux x86_64; A320a; zh-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21194[] = "Mozilla/5.0 (X11; Linux x86_64; Z520m; es-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21195[] = "Mozilla/5.0 (X11; Linux x86_64; A320a; es-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21196[] = "Tablet-PC-4 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21197[] = "Mozilla/5.0 (X11; Linux x86_64; X525a; en-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21198[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24; 360browser(securitypay,securityinstalled)";
const char S21199[] = "VB100a Pro Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21200[] = "ELEMENT10 1 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21201[] = "Mozilla/5.0 (X11; Linux x86_64; Z520m; zh-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21202[] = "OV-SteelCore(B) Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21203[] = "ST80208-1 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21204[] = "Mozilla/5.0 (X11; Linux x86_64; de_de) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21205[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24,gzip(gfe) (via docs.google.com/viewer)";
const char S21206[] = "Mozilla/5.0 (X11;Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21207[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_DesireHD_Beats_X315e; en-gb) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21208[] = "Mozilla/5.0 (110A; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21209[] = "Cat Tablet PHOENIX 8.1J0 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21210[] = "TBD1083B Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21211[] = "TBDB763 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21212[] = "ST80216-1 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21213[] = "Mozilla/5.0 (X11; Linux x86_64; vi_vn) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21214[] = "Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/11.0.1410.63 Safari/537.31";
const char S21215[] = "Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/11.0.1410.63 Safari/537.31";
const char S21216[] = "Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/11.0.1410.63 Safari/537.31";
const char S21217[] = "Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/11.0.1410.63 Safari/537.31";
const char S21218[] = "Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/11.0.1410.63 Safari/537.31";
const char S21219[] = "Mozilla/5.0 (X11; Linux x86_64; it_it) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21220[] = "ICOU10 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21221[] = "Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/11.0.1410.63 Safari/537.31";
const char S21222[] = "TBDB863 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21223[] = "Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/11.0.1410.63 Safari/537.31";
const char S21224[] = "MID7500 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21225[] = "Mozilla/5.0 (X11; Linux x86_64; es_us) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21226[] = "Mozilla/5.0 (X11; Linux x86_64; X525a; fr-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21227[] = "Mozilla/5.0 (X11; Linux x86_64; nl_nl) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21228[] = "Mozilla/5.0 (Linux; U; Linux x86_64; en-us; I7a Build/IML74K) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21229[] = "Mozilla/5.0 (X11; Linux x86_64; fr_fr) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21230[] = "Mozilla/5.0 (X11; ) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24,gzip(gfe)";
const char S21231[] = "TBD753 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21232[] = "TBD1083 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21233[] = "Mozilla/5.0 (X11; Linux x86_64; X525a; en-us) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21234[] = "Tabloid_SW70-01A Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21235[] = "TBDG773 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21236[] = "g08refem618 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21237[] = "8JC1 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21238[] = "DINO762 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21239[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24,gzip(gfe),gzip(gfe)";
const char S21240[] = "TABX Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21241[] = "TM903 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21242[] = "Mozilla/5.0 (X11; Linux x86_64; ja-jp;) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24 TT/4.1.2-X.X";
const char S21243[] = "AXPAD 7I05 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21244[] = "NTT 611 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21245[] = "Mozilla/5.0 (X11; Linux x86_64; en-us) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S21246[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; LT18i Build/4.1.A.0.562) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21247[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SonyEricssonLT18i Build/4.1.A.0.562) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21248[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; LT18i Build/4.1.A.0.562)";
const char S21249[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; LT18i Build/4.1.K.0.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21250[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT18i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21251[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT18i Build/4.1.L.0.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21252[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; LT18a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21253[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT18i Build/4.1.B.0.431) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S21254[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT18i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21255[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; LT18i Build/4.1.A.0.562) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S21256[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LT18a Build/4.1.A.0.562) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21257[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; LT18i Build/4.1.C.0.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21258[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyEricssonLT18a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21259[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SonyEricssonLT18iv Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21260[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT18i Build/4.1.B.0.587)";
const char S21261[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT18a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21262[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT18i Build/4.1.B.0.587) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S21263[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-ch; SonyEricssonLT18i-o Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21264[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; LT18i Build/4.1.B.1.13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21265[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT18a Build/4.1.B.0.587)";
const char S21266[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; LT18i Build/ThilinaC@xda_pureness_13.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21267[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; LT18i Build/ThilinaC@xda_pureness_14.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21268[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT18i Build/Xperia Ultimate HDT 3.0.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21269[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-; LT18i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21270[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.1b4) Gecko/X Firefox/3.5b4 GTB5 (.NET CLR 3.5.30729)";
const char S21271[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.1.2) Gecko/X Firefox/3.5.2 (.NET CLR 3.5.30729) WinNT-PAU 24.08.2009";
const char S21272[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.1.1) Gecko/X Firefox/3.5.1 (.NET CLR 3.5.30729) WinTSI 09.02.2010";
const char S21273[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.1.10) Gecko/X Firefox/3.5.10 (.NET CLR 3.5.30729) WinNT-EVI 25.04.2010";
const char S21274[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; rv:1.8.1) Gecko/X Firefox/3.5.0Mozilla/5.0 (Windows; U; Windows NT 6.1; rv:1.9.2) Gecko/X Firefox/3.5.0";
const char S21275[] = "Mozilla/5.0 (Windows NT 6.0; U; ja; rv:1.9.1.6) Gecko/X Firefox/3.5.6";
const char S21276[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-GB; rv:1.9.1.3) Gecko/X Firefox/3.5.3 ( .NET CLR 3.5.30729) WinTSI 30.10.2009";
const char S21277[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36,gzip(gfe)";
const char S21278[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36";
const char S21279[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36";
const char S21280[] = "Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36";
const char S21281[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36";
const char S21282[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36";
const char S21283[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36,gzip(gfe)";
const char S21284[] = "BlackBerry8530/5.0.0.973 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/105";
const char S21285[] = "BlackBerry9810/7.0.0.261 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102";
const char S21286[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9810; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.261 Mobile Safari/534.11+";
const char S21287[] = "TwitterForBlackBerry/2.1.0.28 (BlackBerry; U; BlackBerry 9810; en-US) Version/7.0.0.261";
const char S21288[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.229 Version/11.64";
const char S21289[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.229 Version/11.64";
const char S21290[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.229 Version/11.64";
const char S21291[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.229 Version/11.64";
const char S21292[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.229 Version/11.64";
const char S21293[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.64";
const char S21294[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.229 Version/11.64";
const char S21295[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.229 Version/11.64";
const char S21296[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5230); ru) Presto/2.10.229 Version/11.64";
const char S21297[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.229 Version/11.64";
const char S21298[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.229 Version/11.64";
const char S21299[] = "Mozilla/5.0 (Windows NT 5.1; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.64";
const char S21300[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.229 Version/11.64";
const char S21301[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.64";
const char S21302[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.229 Version/11.64";
const char S21303[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.229 Version/11.64";
const char S21304[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.10.229 Version/11.64";
const char S21305[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5328); ru) Presto/2.10.229 Version/11.64";
const char S21306[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5218); ru) Presto/2.10.229 Version/11.64";
const char S21307[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.229 Version/11.64";
const char S21308[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.229 Version/11.64";
const char S21309[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5299); ru) Presto/2.10.229 Version/11.64";
const char S21310[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5309); ru) Presto/2.10.229 Version/11.64";
const char S21311[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.229 Version/11.64";
const char S21312[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.229 Version/11.64";
const char S21313[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.64";
const char S21314[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.10.229 Version/11.64";
const char S21315[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.10.229 Version/11.64";
const char S21316[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.229 Version/11.64";
const char S21317[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.229 Version/11.64";
const char S21318[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.229 Version/11.64,gzip(gfe) (via translate.google.com)";
const char S21319[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5300); ru) Presto/2.10.229 Version/11.64";
const char S21320[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.229 Version/11.64";
const char S21321[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.229 Version/11.64";
const char S21322[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5161); ru) Presto/2.10.229 Version/11.64";
const char S21323[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.229 Version/11.64";
const char S21324[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.64";
const char S21325[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.229 Version/11.64";
const char S21326[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.64";
const char S21327[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.229 Version/11.64,gzip(gfe) (via translate.google.com)";
const char S21328[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.10.229 Version/11.64";
const char S21329[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5288); ru) Presto/2.10.229 Version/11.64";
const char S21330[] = "Opera/9.80 (Windows NT 5.2; U; en) Presto/2.10.229 Version/11.64";
const char S21331[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5217); ru) Presto/2.10.229 Version/11.64";
const char S21332[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.229 Version/11.64";
const char S21333[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.229 Version/11.64";
const char S21334[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.229 Version/11.64";
const char S21335[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02652); ru) Presto/2.10.229 Version/11.64";
const char S21336[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5278); ru) Presto/2.10.229 Version/11.64";
const char S21337[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; ru) Opera 11.64";
const char S21338[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.229 Version/11.64";
const char S21339[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.229 Version/11.64";
const char S21340[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5336); ru) Presto/2.10.229 Version/11.64";
const char S21341[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.10.229 Version/11.64";
const char S21342[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4930); ru) Presto/2.10.229 Version/11.64";
const char S21343[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.229 Version/11.64";
const char S21344[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.10.229 Version/11.64";
const char S21345[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03780); ru) Presto/2.10.229 Version/11.64";
const char S21346[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.10.229 Version/11.64";
const char S21347[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.229 Version/11.64";
const char S21348[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5650); ru) Presto/2.10.229 Version/11.64";
const char S21349[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.229 Version/11.64";
const char S21350[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5667); ru) Presto/2.10.229 Version/11.64";
const char S21351[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5660); ru) Presto/2.10.229 Version/11.64";
const char S21352[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.229 Version/11.64";
const char S21353[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.229 Version/11.64";
const char S21354[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.229 Version/11.64";
const char S21355[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5325); ru) Presto/2.10.229 Version/11.64";
const char S21356[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.10.229 Version/11.64";
const char S21357[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03672); ru) Presto/2.10.229 Version/11.64";
const char S21358[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5676); ru) Presto/2.10.229 Version/11.64";
const char S21359[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4661); ru) Presto/2.10.229 Version/11.64";
const char S21360[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.10.229 Version/11.64";
const char S21361[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.229 Version/11.64";
const char S21362[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.10.229 Version/11.64";
const char S21363[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02560); ru) Presto/2.10.229 Version/11.64";
const char S21364[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.229 Version/11.64,gzip(gfe)";
const char S21365[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5966); ru) Presto/2.10.229 Version/11.64";
const char S21366[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.229 Version/11.64";
const char S21367[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.229 Version/11.64";
const char S21368[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.229 Version/11.64";
const char S21369[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.229 Version/11.64";
const char S21370[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5784); ru) Presto/2.10.229 Version/11.64";
const char S21371[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5997); ru) Presto/2.10.229 Version/11.64";
const char S21372[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.229 Version/11.64";
const char S21373[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5745); ru) Presto/2.10.229 Version/11.64";
const char S21374[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5981); ru) Presto/2.10.229 Version/11.64";
const char S21375[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.229 Version/11.64";
const char S21376[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5880); ru) Presto/2.10.229 Version/11.64";
const char S21377[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6089); ru) Presto/2.10.229 Version/11.64";
const char S21378[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 6003); ru) Presto/2.10.229 Version/11.64";
const char S21379[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.229 Version/11.64";
const char S21380[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5989); ru) Presto/2.10.229 Version/11.64";
const char S21381[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21382[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-S7500L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21383[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S7500/S7500BULB1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21384[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S7500 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S21385[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S7500 Build/GINGERBREAD)";
const char S21386[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-S7500/S7500BULC1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21387[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S7500/S7500BULE2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21388[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S7500W Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21389[] = "Mozilla/5.0 (Linux; U; en-gb; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S21390[] = "Mozilla/5.0 (Linux; U; Android 2.3.6_mobile-star; iw-il; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21391[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S7500 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S21392[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21393[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-S7500T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21394[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S7500L Build/GINGERBREAD)";
const char S21395[] = "Mozilla/5.0 (Linux; U; Android 2.3.6_mobile-star; iw-il; SAMSUNG GT-S7500/S7500BULE2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21396[] = "Mozilla/5.0 (Linux; U; it-it; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S21397[] = "Mozilla/5.0 (Linux; U; de-de; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S21398[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-S7500/S7500BULK1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21399[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S7500T Build/GINGERBREAD)";
const char S21400[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-S7500/S7500BUMB1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21401[] = "Mozilla/5.0 (Linux; U; ro-ro; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S21402[] = "Mozilla/5.0 (Linux; U; pt-pt; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S21403[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.82 Safari/537.1";
const char S21404[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S21405[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S21406[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)";
const char S21407[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)";
const char S21408[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)";
const char S21409[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S21410[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1080) AppleWebKit/537.1 (KHTML like Gecko) Chrome/21.0.1180.57 Safari/537.1";
const char S21411[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1082) AppleWebKit/537.1 (KHTML like Gecko) Chrome/21.0.1180.82 Safari/537.1";
const char S21412[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1081) AppleWebKit/537.1 (KHTML like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S21413[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S21414[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S21415[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A442 Safari/9537.53";
const char S21416[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A441 Safari/9537.53";
const char S21417[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A446a Safari/9537.53";
const char S21418[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A451 Safari/9537.53";
const char S21419[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A445 Safari/9537.53";
const char S21420[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A4449d Safari/9537.53";
const char S21421[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A4449a Safari/9537.53";
const char S21422[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A449 Safari/9537.53";
const char S21423[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A452 Safari/9537.53";
const char S21424[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A462 Safari/9537.53";
const char S21425[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A450 Safari/9537.53";
const char S21426[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A457 Safari/9537.53";
const char S21427[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A464 Safari/9537.53";
const char S21428[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A448 Safari/9537.53";
const char S21429[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A463 Safari/9537.53";
const char S21430[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A459 Safari/9537.53";
const char S21431[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A456 Safari/9537.53";
const char S21432[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A453 Safari/9537.53";
const char S21433[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A458 Safari/9537.53";
const char S21434[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A465 Safari/9537.53";
const char S21435[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B472 Safari/9537.53";
const char S21436[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B473 Safari/9537.53";
const char S21437[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A466 Safari/9537.53";
const char S21438[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B470 Safari/9537.53";
const char S21439[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B475 Safari/9537.53";
const char S21440[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B481 Safari/9537.53";
const char S21441[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B478 Safari/9537.53";
const char S21442[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A4449b Safari/9537.53";
const char S21443[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A470a Safari/9537.53";
const char S21444[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A444 Safari/9537.53";
const char S21445[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B479 Safari/9537.53";
const char S21446[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A468 Safari/9537.53";
const char S21447[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B488 Safari/9537.53";
const char S21448[] = "Mozilla/5.0 (iPhone; CPU iPhone OS7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A465 Safari/9537.53";
const char S21449[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A465Safari/9537.53";
const char S21450[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A501 Safari/9537.53";
const char S21451[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B480 Safari/9537.53";
const char S21452[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_3 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B491 Safari/9537.53";
const char S21453[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B487 Safari/9537.53";
const char S21454[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A470 Safari/9537.53";
const char S21455[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_3 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B494 Safari/9537.53";
const char S21456[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B489 Safari/9537.53";
const char S21457[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A501Safari/9537.53";
const char S21458[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_3 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B498a Safari/9537.53";
const char S21459[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_3 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B497 Safari/9537.53";
const char S21460[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_3 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B495 Safari/9537.53";
const char S21461[] = "Mozilla/5.0 (iPhone; CPU iPhone OS7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A501 Safari/9537.53";
const char S21462[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; Galaxy Nexus Build/JRN84D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21463[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; Galaxy Nexus Build/JRN84D) AppleWebKit/535.7 (KHTML, like Gecko) Mobile Safari/535.7";
const char S21464[] = "Dalvik/1.6.0 (Linux; U; Android 4.1; Galaxy Nexus Build/JRN84D)";
const char S21465[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21466[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Galaxy Nexus Build/JRO03C)";
const char S21467[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRN84D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21468[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21469[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21470[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21471[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Galaxy Nexus Build/JRO03C) AppleWebKit/535.7 (KHTML, like Gecko) Mobile Safari/535.7";
const char S21472[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; Galaxy Nexus Build/IMM76K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21473[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21474[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21475[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JR003H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21476[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21477[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03O) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21478[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Galaxy Nexus Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S21479[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21480[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S21481[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03U) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21482[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21483[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Galaxy Nexus Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21484[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Galaxy Nexus Build/JZ054K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21485[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Galaxy Nexus Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21486[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Galaxy Nexus Build/JZO54K)";
const char S21487[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Galaxy Nexus Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21488[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Galaxy Nexus Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S21489[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03O) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30";
const char S21490[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Galaxy Nexus Build/JZO54K/4.2 Hybrid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21491[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; Galaxy X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21492[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03O) AppleWebKit/535.7 (KHTML, like Gecko) Mobile Safari/535.7";
const char S21493[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; Galaxy Nexus Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21494[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Galaxy Nexus Build/JR003H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21495[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; Galaxy Nexus Build/WWW.ROMUP.COM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21496[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Galaxy Nexus Build/Wind-Galaxy Nexus-V1.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21497[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Galaxy X Build/JRO03C)";
const char S21498[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/531.21.11 (KHTML, like Gecko)";
const char S21499[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/531.9 (KHTML, like Gecko)";
const char S21500[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-us) AppleWebKit/531.9 (KHTML, like Gecko)";
const char S21501[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-us) AppleWebKit/531.9 (KHTML, like Gecko)";
const char S21502[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko)";
const char S21503[] = "User-Agent:Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.18.1";
const char S21504[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko)";
const char S21505[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko)";
const char S21506[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko)";
const char S21507[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko)";
const char S21508[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/534.24 (KHTML, like Gecko)";
const char S21509[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.52.7 (KHTML, like Gecko) Sparrow/1043";
const char S21510[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.17.8 (KHTML, like Gecko) Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en) AppleWebKit/533.17.8 (KHTML, like Gecko) \"wKiosk for Mac\"";
const char S21511[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/531.21.8+(KHTML, like Gecko, Safari/528.16) (null)";
const char S21512[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; it-it) AppleWebKit/533.20.25 (KHTML, like Gecko) Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en) AppleWebKit/533.20.25 (KHTML, like Gecko) \"wKiosk for Mac\"";
const char S21513[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Paparazzi!/0.5b5 (like Safari)";
const char S21514[] = "Mac OS X/10.6.8 (10K549); ExchangeWebServices/1.3 (61); iCal/4.0.4 (1395.7)";
const char S21515[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en)";
const char S21516[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) WebClip/6530.10 Safari/6530.10";
const char S21517[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en) AppleWebKit/531.22.7 (KHTML, like Gecko) \"wKiosk for Mac\"";
const char S21518[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.51.22 (KHTML, like Gecko) Sparrow/959.1";
const char S21519[] = "MacOutlook/14.12.0.X (Intel Mac OS X 10.6.8)";
const char S21520[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) NetNewsWire/3.2.15";
const char S21521[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1043.1";
const char S21522[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) NetNewsWire/3.2.7";
const char S21523[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en) AppleWebKit/533.21.1 (KHTML, like Gecko) \"wKiosk for Mac\"";
const char S21524[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) NetNewsWire/3.2.15";
const char S21525[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) NetNewsWire/3.3";
const char S21526[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.2.10) Gecko Expeditor 6.2.2.X-1115 Gecko/X-1115";
const char S21527[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Vienna/2.6.0.2601";
const char S21528[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) NetNewsWire/3.2.15";
const char S21529[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Safari/531.9";
const char S21530[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Safari/531.9";
const char S21531[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1081.27";
const char S21532[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1081.51";
const char S21533[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1081.52";
const char S21534[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1144";
const char S21535[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) NetNewsWire/3.2.7";
const char S21536[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Safari/522.0";
const char S21537[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Safari/531.9";
const char S21538[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Safari/522.0";
const char S21539[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1043.2";
const char S21540[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) NetNewsWire/3.2.7";
const char S21541[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sunrise";
const char S21542[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1143.6";
const char S21543[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1081.53";
const char S21544[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Safari/522.0";
const char S21545[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1164";
const char S21546[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) NetNewsWire/3.3.2";
const char S21547[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1172";
const char S21548[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) LittleSnapper/12474 Safari/525.13";
const char S21549[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Sparrow/1043.1";
const char S21550[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1081.28";
const char S21551[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Safari/417.8";
const char S21552[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) RealPlayer/12.0.0.1725";
const char S21553[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) NetNewsWire/3.3.1";
const char S21554[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1173";
const char S21555[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1176";
const char S21556[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/904";
const char S21557[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) NetNewsWire/3.1.7";
const char S21558[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1178";
const char S21559[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) FluidApp/1.1 Safari/533.21.1";
const char S21560[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) (null) FBSMTWB";
const char S21561[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) LittleSnapper/12474 Safari/525.13";
const char S21562[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.51.22 (KHTML, like Gecko) Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en) AppleWebKit/533.21.1 (KHTML, like Gecko) \"wKiosk for Mac\"";
const char S21563[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) NetNewsWire/3.2.11";
const char S21564[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) iWeb/304";
const char S21565[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.1.7) Zimbra/7.2.0";
const char S21566[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Paparazzi!/0.6.5 (like Safari)";
const char S21567[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) RealPlayer/11.1.0.1116";
const char S21568[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Vienna/2.5.0.2501";
const char S21569[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) TubeTV";
const char S21570[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) LittleSnapper/12267 Safari/531.21.10";
const char S21571[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/(null) Safari/533.21.1";
const char S21572[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Safari/531.2+ Miro/5.0.4 (http://www.getmiro.com/)";
const char S21573[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) KidZui K2";
const char S21574[] = "Mozilla/4.0 (compatible; Mozilla/4.0; Mozilla/5.0; Mozilla/6.0; Safari/431.7; Macintosh; U; PPC Mac OS X 10.6 Leopard; AppleWebKit/421.9 (KHTML, like Gecko) )";
const char S21575[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) RealPlayer/12.0.1.1750";
const char S21576[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) RealPlayer/12.0.0.1569";
const char S21577[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) (null) FBSMTWB";
const char S21578[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/(null) Safari/533.21.1";
const char S21579[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/534.24 (KHTML, like Gecko) Contact: feedback@getprismatic.com";
const char S21580[] = "MacAppStore/1.0.2 (Macintosh; U; Intel Mac OS X 10.6.8; en) AppleWebKit/534.57.2";
const char S21581[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Vienna/2.5.1.2502";
const char S21582[] = "iPhoto/8.1.2 (Macintosh; U; Intel Mac OS X 10.6.8; English)";
const char S21583[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Sparrow/1176";
const char S21584[] = "DAVKit/4.0.3 (732.2); CalendarStore/4.0.4 (997.7); iCal/4.0.4 (1395.7); Mac OS X/10.6.8 (10K549)";
const char S21585[] = "MacAppStore/1.0.2 (Macintosh; U; Intel Mac OS X 10.6.8; en) AppleWebKit/533.21.1";
const char S21586[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/959.4";
const char S21587[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.55.3 (KHTML, like Gecko) Safari/534.55.3";
const char S21588[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us";
const char S21589[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/533.16 (KHTML, like Gecko) OceanBrowser/2.0 Safari/533.16";
const char S21590[] = "Mozilla/4.0 (Mac OS X 10.6.8) Java/1.6.0_51";
const char S21591[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.59.8 (KHTML, like Gecko) Vienna/3.0.0 Safari/534.59.8";
const char S21592[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.59.8 (KHTML, like Gecko) Elluminate Live! Web Tour";
const char S21593[] = "MacAppStore/1.0.2 (Macintosh; U; Intel Mac OS X 10.6.8; en) AppleWebKit/534.59.8";
const char S21594[] = "Entourage/12.36.0 (Intel Mac OS X 10.6.8)";
const char S21595[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One S Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21596[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21597[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21598[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC One S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21599[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC One S Build/JRO03C)";
const char S21600[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_S/3.16.110.3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21601[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_S/3.16.161.9 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21602[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC_One_S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21603[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-ch; HTC_One_S/3.16.166.8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21604[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC_One_S_Special_Edition Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21605[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; Z520m Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21606[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC One S Build/JZO54K)";
const char S21607[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_S-orange-LS Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21608[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-at; HTC_One_S/3.16.169.8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21609[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_S/3.16.111.5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21610[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; HTC_One_S/3.16.114.5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21611[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_S/3.16.111.7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21612[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; HTC_One_S/2.38.161.6 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21613[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; HTC_One_S/3.17.113.102 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21614[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-at; HTC_One_S/3.16.112.5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21615[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_S_pj40200/3.19.161.101 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21616[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC One S Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S21617[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; HTC_One_S/3.16.163.8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21618[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-at; HTC_One_S_pj40200/3.19.169.101 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21619[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC VLE_U Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21620[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_S_pj40200/3.19.161.102 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21621[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; HTC_VLE_U Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21622[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_S/3.16.161.10 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21623[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC VLE_U Build/JRO03C)";
const char S21624[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC One S Special Edition Build/JRO03C)";
const char S21625[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One S Special Edition Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21626[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; HTC One S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21627[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-ch; HTC_One_S/2.38.166.6 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21628[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; HTC_One_S/3.17.118.102 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21629[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_S/2.38.111.10 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21630[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_S/3.16.111.10 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21631[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-; HTC_One_S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21632[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_One_S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21633[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_S/2.38.110.2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S21634[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B5126b Safari/8536.25";
const char S21635[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B141 Safari/8536.25";
const char S21636[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B147 Safari/8536.25";
const char S21637[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B146 Safari/8536.25";
const char S21638[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B329 Safari/8536.25";
const char S21639[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/534.7 ZemanaAID/FFFF0061";
const char S21640[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1";
const char S21641[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1,gzip(gfe) (via translate.google.com)";
const char S21642[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1";
const char S21643[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1";
const char S21644[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1";
const char S21645[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/13 Safari/525.13";
const char S21646[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1, SurfCop 1.5";
const char S21647[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1,gzip(gfe) (via docs.google.com/viewer)";
const char S21648[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1,gzip(gfe)";
const char S21649[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.1 (KHTML like Gecko) Chrome/13.0.782.41 Safari/535.1";
const char S21650[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5288d Safari/7534.48.3";
const char S21651[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5313e Safari/7534.48.3";
const char S21652[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5302b Safari/7534.48.3";
const char S21653[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3";
const char S21654[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.46";
const char S21655[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A402 Safari/7534.48.3";
const char S21656[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.46";
const char S21657[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334";
const char S21658[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3";
const char S21659[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5274d Safari/7534.48.3";
const char S21660[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9Z999 Safari/7534.48.3";
const char S21661[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7534.48.3";
const char S21662[] = "Mozilla/5.0 (iPad; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A404 Safari/7534.48.3";
const char S21663[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5220p Safari/7534.48.3";
const char S21664[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari Version/5.1";
const char S21665[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S21666[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3";
const char S21667[] = "Mozilla/5.0 (iPad; CPU iPhone OS 501 like Mac OS X) AppleWebKit/534.46 (KHTML like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3";
const char S21668[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/537.4 (KHTML, like Gecko; Google Web Preview) Version/5.1 Mobile/9A334 Safari/7534.48.3";
const char S21669[] = "Mozilla/5.0 (iPad; U; CPU OS 5 like Mac OS X; zh-cn Model:iPad2,1) UC AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7543.48.3";
const char S21670[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S21671[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC_Explorer_A310b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21672[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC_Explorer_A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21673[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC Explorer A310b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21674[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21675[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_Explorer Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21676[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Explorer Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21677[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/Explorer/1.17.161.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21678[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_Explorer-orange-LS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21679[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; HTC/Explorer/1.29.161.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21680[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Explorer A310e Build/GRJ90)";
const char S21681[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-ch; HTC/Explorer/1.29.166.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21682[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Explorer A310e Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S21683[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC/Explorer/1.29.163.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21684[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC/Explorer/1.17.163.5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21685[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/Explorer/1.41.161.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21686[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-ch; HTC/Explorer/1.17.166.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21687[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Explorer Build/GRJ90)";
const char S21688[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-ch; HTC/Explorer/1.41.166.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21689[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC/Explorer/1.41.163.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21690[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC A310e Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S21691[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Explorer A310b Build/GRJ90)";
const char S21692[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-cn; HTC_A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21693[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC A310e Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S21694[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21695[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Explorer A310e Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S21696[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; HTC/Explorer/1.41.161.6 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21697[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC/Explorer/1.41.192.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21698[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A405 Safari/7534.48.3";
const char S21699[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A403 Safari/7534.48.3";
const char S21700[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A523 Safari/7534.48.3";
const char S21701[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A525 Safari/7534.48.3";
const char S21702[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A551 Safari/7534.48.3";
const char S21703[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4";
const char S21704[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S21705[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S21706[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S21707[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S21708[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S21709[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S21710[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S21711[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S21712[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via translate.google.com)";
const char S21713[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)";
const char S21714[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via docs.google.com/viewer)";
const char S21715[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1060) AppleWebKit/537.4 (KHTML like Gecko) Chrome/22.0.1229.79 Safari/537.4";
const char S21716[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/22.0.648.205 Safari/534.16";
const char S21717[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/22.0.648.205 Safari/534.16";
const char S21718[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/22.0.648.205 Safari/534.16";
const char S21719[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.510 YaBrowser/1.5.1105.510 (gtsr) Safari/537.4";
const char S21720[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.510 YaBrowser/1.5.1105.510 (fnrb) Safari/537.4";
const char S21721[] = "Mozilla/5.0 (GTmetrix http://gtmetrix.com/; X11; U; Linux i686 (x86_64); en-US; rv:1.9.2.13) Gecko/X Firefox/3.6.13";
const char S21722[] = "Mozilla/5.0 (X11;Arch Linux i686; en-US; rv:1.9.2.8) Gecko/X Firefox/3.6.8";
const char S21723[] = "Mozilla/5.0 (X11; U; Linux i686; de; rv:1.9.2.12) Gecko/X Iceweasel/3.6.12 (like Firefox/3.6.12)";
const char S21724[] = "Mozilla/5.0 (X11; U; Linux i686; en-AU; rv:1.9.2.26) Gecko/X Firefox/2.0.0.3 (like Firefox/3.6.26)";
const char S21725[] = "OMozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.9) Gecko/X Red Hat/3.6.9-2.el6 Firefox/3.6.9";
const char S21726[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.17) Gecko/X Firefox/8.04 (hardy) Firefox/3.6.17";
const char S21727[] = "Mozilla/5.0 (X11; Linux i686 on x86_64; rv:5.0) Gecko/X Firefox/3.6.17 Firefox/3.6.17";
const char S21728[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50";
const char S21729[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50";
const char S21730[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50";
const char S21731[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50";
const char S21732[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22";
const char S21733[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22";
const char S21734[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22";
const char S21735[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22";
const char S21736[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7";
const char S21737[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7";
const char S21738[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16";
const char S21739[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16";
const char S21740[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3";
const char S21741[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2";
const char S21742[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7";
const char S21743[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3";
const char S21744[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3";
const char S21745[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2";
const char S21746[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16";
const char S21747[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7";
const char S21748[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3";
const char S21749[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2";
const char S21750[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2";
const char S21751[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16";
const char S21752[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.34 (KHTML, like Gecko) SlimBoat/1.0.12 Version/5.1 Safari/534.34";
const char S21753[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.3 Safari/534.53.10";
const char S21754[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.3 Safari/534.53.10";
const char S21755[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 6_0 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.0;FBSS/2; FBCR/unavailable;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S21756[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A5376e";
const char S21757[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403";
const char S21758[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S21759[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 iPod4,1 BingWeb/3.03.1428.X";
const char S21760[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.0;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]";
const char S21761[] = "Mozilla/5.0 (iPodTouch4G; iPhone; U; CPU OS 6_0 like Mac OS X; en_US) com.google.GooglePlus/10352 (KHTML, like Gecko) Mobile/N81AP (gzip)";
const char S21762[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A406";
const char S21763[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 6_0_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.0.1;FBSS/2; FBCR/unavailable;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S21764[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523";
const char S21765[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 iPod4,1 BingWeb/3.03.1428.X";
const char S21766[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S21767[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 iPod5,1 BingWeb/3.03.1428.X";
const char S21768[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 bline/1.03 (iPhone OS 6.0.1, iPod touch)";
const char S21769[] = "Mozilla/5.0 (iPhone; U; CPU iPhone 6_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPod4,1/6.0.1)";
const char S21770[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.0.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]";
const char S21771[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A406 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPod5,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.0;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]";
const char S21772[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper";
const char S21773[] = "Mozilla/5.0 (iPod4,1; iPhone; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N81AP (gzip)";
const char S21774[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPad3,4;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0.1;FBSS/2; FBCR/;FBID/tablet;FBLC/en_US]";
const char S21775[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3";
const char S21776[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper";
const char S21777[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S21778[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en_US) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5";
const char S21779[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 Safari/6533.18.5";
const char S21780[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134b Safari/6533.18.5";
const char S21781[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9Z999 Safari/6533.18.5";
const char S21782[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en_us) AppleWebKit/6533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5";
const char S21783[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; zh-cn Model:iPad1,1) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S21784[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C5115c Safari/6533.18.5";
const char S21785[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.3";
const char S21786[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C5101c Safari/6533.18.5";
const char S21787[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C134 Safari/6533.18.5";
const char S21788[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.1";
const char S21789[] = "Mozilla/5.0 (iPad; U; CPU OS 421 like Mac OS X; haw-US) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S21790[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.5 (KHTML, likeGecko) Chrome/4.1.249.1045 Safari/532.5";
const char S21791[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Chrome/4.0.221.7 Safari/532.2";
const char S21792[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.1.249.1045 Safari/532.5";
const char S21793[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; AppleWebKit/540.0 (KHTML, like Gecko) ChromePlus/4.0.222.3 Chrome/4.0.222.3 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S21794[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US Alexa Toolbar) AppleWebKit/532.5 (KHTML	 like Gecko Alexa Toolbar) Chrome/4.0.249.0 Safari/532.5";
const char S21795[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; AppleWebKit/532.2; KHTML?? like Gecko; Chrome/4.0.222.12 Safari/532.2)";
const char S21796[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; it-IT) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.25 Safari/532.5";
const char S21797[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.14";
const char S21798[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.14";
const char S21799[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.14";
const char S21800[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.14";
const char S21801[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.14";
const char S21802[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.14";
const char S21803[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.14";
const char S21804[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.14";
const char S21805[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.14";
const char S21806[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.14";
const char S21807[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.14";
const char S21808[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.14";
const char S21809[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.14";
const char S21810[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.14";
const char S21811[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.14";
const char S21812[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.14";
const char S21813[] = "Opera/9.80 (Windows NT 5.2; WOW64) Presto/2.12.388 Version/12.14";
const char S21814[] = "Opera/9.80 (Windows NT 5.2) Presto/2.12.388 Version/12.14";
const char S21815[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.14";
const char S21816[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.14";
const char S21817[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.14";
const char S21818[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.14";
const char S21819[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.14";
const char S21820[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.14";
const char S21821[] = "Opera/9.80 (Windows NT 5.1; Edition Campaign 21) Presto/2.12.388 Version/12.14";
const char S21822[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.14";
const char S21823[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.14";
const char S21824[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.14";
const char S21825[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.14";
const char S21826[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.14";
const char S21827[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.14";
const char S21828[] = "Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.14";
const char S21829[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.14";
const char S21830[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.14,gzip(gfe)";
const char S21831[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.14";
const char S21832[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.14";
const char S21833[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1) Opera 12.14";
const char S21834[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.14";
const char S21835[] = "Mozilla/5.0 (Windows NT 5.1; rv:2.0) Gecko/X Firefox/4.0 Opera 12.14";
const char S21836[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.14";
const char S21837[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.14";
const char S21838[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6046)) Presto/2.12.388 Version/12.14";
const char S21839[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.14";
const char S21840[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.14";
const char S21841[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.14";
const char S21842[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.14";
const char S21843[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03658)) Presto/2.12.388 Version/12.14";
const char S21844[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.14";
const char S21845[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.14";
const char S21846[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.14";
const char S21847[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.14 Yandex.Translate";
const char S21848[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.14";
const char S21849[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.14";
const char S21850[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03789)) Presto/2.12.388 Version/12.14";
const char S21851[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6044)) Presto/2.12.388 Version/12.14";
const char S21852[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.14";
const char S21853[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.14";
const char S21854[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.14";
const char S21855[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.14";
const char S21856[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.14";
const char S21857[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.14";
const char S21858[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.14";
const char S21859[] = "Opera/9.80 (Windows NT 5.1; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.14";
const char S21860[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.14";
const char S21861[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.14";
const char S21862[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.14";
const char S21863[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.14";
const char S21864[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.14";
const char S21865[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.14";
const char S21866[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4933)) Presto/2.12.388 Version/12.14";
const char S21867[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.14";
const char S21868[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.14";
const char S21869[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.14";
const char S21870[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.14";
const char S21871[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03402); MRSPUTNIK OW 2, 3, 0, 104) Presto/2.12.388 Version/12.14";
const char S21872[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6035)) Presto/2.12.388 Version/12.14";
const char S21873[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.14";
const char S21874[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5278)) Presto/2.12.388 Version/12.14";
const char S21875[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03686); MRSPUTNIK OW 2, 3, 0, 244) Presto/2.12.388 Version/12.14";
const char S21876[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5230)) Presto/2.12.388 Version/12.14";
const char S21877[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.14";
const char S21878[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.14";
const char S21879[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.14";
const char S21880[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.14";
const char S21881[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5288)) Presto/2.12.388 Version/12.14";
const char S21882[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03402)) Presto/2.12.388 Version/12.14";
const char S21883[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.14";
const char S21884[] = "Opera/9.80 (Windows NT 5.1; Edition Yx) Presto/2.12.388 Version/12.14,gzip(gfe)";
const char S21885[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.14";
const char S21886[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.14";
const char S21887[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6032)) Presto/2.12.388 Version/12.14";
const char S21888[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5650)) Presto/2.12.388 Version/12.14";
const char S21889[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.14";
const char S21890[] = "Opera/9.80 (Windows NT 5.1; MRA 5.3 (build 02552)) Presto/2.12.388 Version/12.14";
const char S21891[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03639)) Presto/2.12.388 Version/12.14";
const char S21892[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5299)) Presto/2.12.388 Version/12.14";
const char S21893[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4110)) Presto/2.12.388 Version/12.14";
const char S21894[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5336)) Presto/2.12.388 Version/12.14";
const char S21895[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5996)) Presto/2.12.388 Version/12.14";
const char S21896[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6080)) Presto/2.12.388 Version/12.14";
const char S21897[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.14";
const char S21898[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4661)) Presto/2.12.388 Version/12.14";
const char S21899[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5734)) Presto/2.12.388 Version/12.14";
const char S21900[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4832)) Presto/2.12.388 Version/12.14";
const char S21901[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03672)) Presto/2.12.388 Version/12.14";
const char S21902[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03757); MRSPUTNIK OW 2, 3, 0, 288) Presto/2.12.388 Version/12.14";
const char S21903[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.14,gzip(gfe) (via docs.google.com/viewer)";
const char S21904[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5660)) Presto/2.12.388 Version/12.14";
const char S21905[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5217)) Presto/2.12.388 Version/12.14";
const char S21906[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03755)) Presto/2.12.388 Version/12.14";
const char S21907[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03791)) Presto/2.12.388 Version/12.14";
const char S21908[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.14";
const char S21909[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5325)) Presto/2.12.388 Version/12.14";
const char S21910[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5300)) Presto/2.12.388 Version/12.14";
const char S21911[] = "Opera/9.80 (Windows NT 5.1; MRA 5.3 (build 02560)) Presto/2.12.388 Version/12.14";
const char S21912[] = "Opera/9.80 (Windows NT 5.1; MRA 5.3 (build 02564)) Presto/2.12.388 Version/12.14";
const char S21913[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.14";
const char S21914[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.14";
const char S21915[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5161)) Presto/2.12.388 Version/12.14";
const char S21916[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.14";
const char S21917[] = "Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02842); MRSPUTNIK OW 2, 3, 0, 104) Presto/2.12.388 Version/12.14";
const char S21918[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.14";
const char S21919[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03649)) Presto/2.12.388 Version/12.14";
const char S21920[] = "Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02830)) Presto/2.12.388 Version/12.14";
const char S21921[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.14";
const char S21922[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6561)) Presto/2.12.388 Version/12.14";
const char S21923[] = "Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02743)) Presto/2.12.388 Version/12.14";
const char S21924[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03638)) Presto/2.12.388 Version/12.14";
const char S21925[] = "Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02746)) Presto/2.12.388 Version/12.14";
const char S21926[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03399); MRSPUTNIK OW 2, 3, 0, 104) Presto/2.12.388 Version/12.14";
const char S21927[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03653)) Presto/2.12.388 Version/12.14";
const char S21928[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6573)) Presto/2.12.388 Version/12.14";
const char S21929[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.14";
const char S21930[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5988)) Presto/2.12.388 Version/12.14";
const char S21931[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6657)) Presto/2.12.388 Version/12.14";
const char S21932[] = "Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6283)) Presto/2.12.388 Version/12.14";
const char S21933[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6638)) Presto/2.12.388 Version/12.14";
const char S21934[] = "Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6341)) Presto/2.12.388 Version/12.14";
const char S21935[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6673)) Presto/2.12.388 Version/12.14";
const char S21936[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.14,gzip(gfe),gzip(gfe)";
const char S21937[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6628)) Presto/2.12.388 Version/12.14";
const char S21938[] = "Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6327)) Presto/2.12.388 Version/12.14";
const char S21939[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.14";
const char S21940[] = "Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6337)) Presto/2.12.388 Version/12.14";
const char S21941[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03796); MRSPUTNIK OW 2, 3, 0, 104) Presto/2.12.388 Version/12.14";
const char S21942[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6648)) Presto/2.12.388 Version/12.14";
const char S21943[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6675)) Presto/2.12.388 Version/12.14";
const char S21944[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6681)) Presto/2.12.388 Version/12.14";
const char S21945[] = "Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6347)) Presto/2.12.388 Version/12.14";
const char S21946[] = "Opera/9.80 (Windows NT 5.1; MRA 6.2 (build 7166)) Presto/2.12.388 Version/12.14";
const char S21947[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.5) Gecko/X iceweasel/3.0.5";
const char S21948[] = "Mozilla/5.0 (X11; U; Linux i586; en-US; rv:1.9.0.1) Gecko/X OLPC/Firefox-6 (XO) Firefox/3.0.1";
const char S21949[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9) Gecko/X Iceweasel/3.0 (Debian-3.0~rc2-2)";
const char S21950[] = "Mozilla/5.0 (X11; U; Linux i686; pl-PL; rv:1.9.1.3) Gecko/X Iceweasel/3.5.3 (Debian-3.5.3-1)";
const char S21951[] = "Mozilla/5.0 (X11; U; Linux i686; tr-TR; rv:1.9b5) Gecko/X SUSE/2.9.95-25.1 Firefox/3.0b5";
const char S21952[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.16) Gecko/X firefox/3.0.6 (Debian-3.0.6-3)";
const char S21953[] = "Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.8.1.1) Gecko/X Firefox/3.0 (Debian-2.0.0.1+dfsg-2)";
const char S21954[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.2) Gecko/X Iceweasel/3.5.2 (Zenwalk GNU Linux)";
const char S21955[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.14) Gecko/X Firefox/3.0.14 (Debian-3.0.14-1)";
const char S21956[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.0.16) Gecko/X Firefox/3.0.6 (Windows NT 5.1)";
const char S21957[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.11) Gecko GranParadiso/3.0.11";
const char S21958[] = "mozilla/5.0 (x11; u; linux i686; sv-se; rv:1.9.0.3) gecko/X suse/3.0.3-1.1 firefox/3.0.3";
const char S21959[] = "Mozilla/5.0 (X11; U; Linux ppc; en-US; rv:1.9a8) Gecko/X GranParadiso/3.1";
const char S21960[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.0.8) Gecko/X GranParadiso/3.0.8";
const char S21961[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.0.5) Gecko/X Gentoo Iceweasel/3.0.5";
const char S21962[] = "Opera/9.50 (J2ME/MIDP; Opera Mini/4.5.33867/29.3757; U; en)";
const char S21963[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/4.5.33867/31.1475; U; en) Presto/2.8.119 Version/11.10";
const char S21964[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonSK17i Build/4.0.A.0.263) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21965[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonSK17i Build/4.0.A.2.335) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21966[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonSK17i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21967[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonSK17iv Build/4.0.A.2.335) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21968[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SK17i Build/4.0.A.2.335) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21969[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonSK17i Build/4.0.A.2.364) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21970[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonSK17i-o Build/4.0.A.2.335) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21971[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SK17i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21972[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonSK17i Build/4.0.A.2.204) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21973[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonSK17i-o Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21974[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonSK17i Build/4.0.A.2.377) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21975[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SK17i Build/4.0.A.2.377) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21976[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SK17i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21977[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SK17i Build/4.0.A.2.335)";
const char S21978[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonSK17i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21979[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonSK17i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21980[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; SonyEricssonSK17i-o Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21981[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; SonyEricssonSK17a Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21982[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SK17i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21983[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonSK17i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21984[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SK17i Build/4.0.2.A.0.62)";
const char S21985[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonSK17i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21986[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonSK17a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21987[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-ar; SonyEricssonSK17a Build/4.0.A.2.335) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21988[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonSK17i-o Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21989[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonSK17iv Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21990[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SK17i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21991[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonSK17i-o Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21992[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; SonyEricssonSK17iv Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21993[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SK17a Build/4.0.A.2.335)";
const char S21994[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SK17i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21995[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SK17a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21996[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SK17a Build/4.0.2.A.0.58)";
const char S21997[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; SK17a Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21998[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-ar; SK17a Build/4.0.A.2.335) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S21999[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; SK17i Build/4.0.2.A.0.62) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S22000[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-pt; SonyEricssonSK17iv Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22001[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; SK17i Build/4.0.2.A.0.58) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S22002[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.6.30 Version/10.60";
const char S22003[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.6.30 Version/10.61";
const char S22004[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.6.30 Version/10.62";
const char S22005[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.6.30 Version/10.61";
const char S22006[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.6.30 Version/10.63";
const char S22007[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.6.30 Version/10.63";
const char S22008[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.6.30 Version/10.63";
const char S22009[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.6.30 Version/10.63";
const char S22010[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.6.30 Version/10.63";
const char S22011[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.6.30 Version/10.63";
const char S22012[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.6.30 Version/10.63";
const char S22013[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.6.30 Version/10.63";
const char S22014[] = "Opera/9.80 (Windows NT 5.2; U; en) Presto/2.6.30 Version/10.60";
const char S22015[] = "Opera/9.80 (Windows NT 5.2; U; en) Presto/2.6.30 Version/10.62";
const char S22016[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.6.30 Version/10.62";
const char S22017[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.6.30 Version/10.62";
const char S22018[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.6.30 Version/10.61";
const char S22019[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.6.30 Version/10.63";
const char S22020[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.6.30 Version/10.63";
const char S22021[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.6.30 Version/10.63";
const char S22022[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.6.30 Version/10.63";
const char S22023[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); MRSPUTNIK OW 2, 3, 0, 104; ru) Presto/2.6.30 Version/10.63";
const char S22024[] = "Opera/9.80 (Windows NT 5.2; U; ru) Presto/2.6.30 Version/10.61";
const char S22025[] = "Opera/9.80 (Windows NT 5.2; U; en) Presto/2.6.30 Version/10.63";
const char S22026[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.6.30 Version/10.63";
const char S22027[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.6.30 Version/10.62";
const char S22028[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.6.30 Version/10.62,gzip(gfe) (via translate.google.com)";
const char S22029[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03638); ru) Presto/2.6.30 Version/10.63";
const char S22030[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.6.30 Version/10.61";
const char S22031[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.6.30 Version/10.60";
const char S22032[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.6.30 Version/10.61";
const char S22033[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.6.30 Version/10.63";
const char S22034[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.6.30 Version/10.60";
const char S22035[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03672); ru) Presto/2.6.30 Version/10.63";
const char S22036[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.6.30 Version/10.61";
const char S22037[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.6.30 Version/10.60";
const char S22038[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.6.30 Version/10.63";
const char S22039[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.6.30 Version/10.63";
const char S22040[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.6.30 Version/10.63";
const char S22041[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.6.30 Version/10.63";
const char S22042[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.6.30 Version/10.61";
const char S22043[] = "Opera/10.60 (Windows NT 5.1; U; en-US) Presto/2.6.30 Version/10.60";
const char S22044[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.6.30 Version/10.62";
const char S22045[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.6.30 Version/10.63";
const char S22046[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.6.30 Version/10.62";
const char S22047[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.6.30 Version/10.60";
const char S22048[] = "Opera/9.80 (Windows NT 5.1; U; en) Presto/2.6.30 Version/10.63,gzip(gfe) (via translate.google.com)";
const char S22049[] = "Opera/6.30 (Windows NT 5.1; U; pl) Version/10.6";
const char S22050[] = "Opera/9.80 (Windows NT 5.1; U; en) Presto/2.6.30 Version/10.60,gzip(gfe) (via translate.google.com)";
const char S22051[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.6.30 Version/10.63";
const char S22052[] = "Opera/6.40 (Windows NT 5.1; U; pl) Version/10.6";
const char S22053[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.6.30 Version/10.60";
const char S22054[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.6.30 Version/10.61";
const char S22055[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.6.30 Version/10.61";
const char S22056[] = "Opera/6.10 (Windows NT 5.1; U; pl) Version/10.6";
const char S22057[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.6.30 Version/10.62";
const char S22058[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.6.30 Version/10.61";
const char S22059[] = "Opera/6.20 (Windows NT 5.1; U; pl) Version/10.6";
const char S22060[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.6.30 Version/10.63";
const char S22061[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.6.30 Version/10.63";
const char S22062[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.6.30 Version/10.63";
const char S22063[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.6.30 Version/10.62";
const char S22064[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.6.30 Version/10.63";
const char S22065[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.6.30 Version/10.63,gzip(gfe)";
const char S22066[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.6.30 Version/10.62";
const char S22067[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.6.30 Version/10.61";
const char S22068[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.6.30 Version/10.63";
const char S22069[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.6.30 Version/10.63";
const char S22070[] = "Opera/9.80 (Windows NT 5.1; U; en) Presto/2.6.30 Version/10.60,gzip(gfe)";
const char S22071[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.6.30 Version/10.63";
const char S22072[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1568.2 Safari/537.36 AppEngine-Google; (+http://code.google.com/appengine; appid: s~unalin001)";
const char S22073[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S22074[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36,gzip(gfe)";
const char S22075[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Maxthon/4.2.0.800 Chrome/30.0.1551.0 Safari/537.36";
const char S22076[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S22077[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S22078[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S22079[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.11";
const char S22080[] = "Opera/9.80 (Windows NT 5.2; WOW64) Presto/2.12.388 Version/12.11";
const char S22081[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.11";
const char S22082[] = "Opera/9.80 (Windows NT 5.1; Edition Yx) Presto/2.12.388 Version/12.11";
const char S22083[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.11";
const char S22084[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.11";
const char S22085[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.11";
const char S22086[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.11";
const char S22087[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.11";
const char S22088[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.11";
const char S22089[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1) Opera 12.11";
const char S22090[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.11";
const char S22091[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.11";
const char S22092[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.11";
const char S22093[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.11";
const char S22094[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.11";
const char S22095[] = "Opera/9.80 (Windows NT 5.2) Presto/2.12.388 Version/12.11";
const char S22096[] = "Mozilla/5.0 (Windows NT 5.1; rv:2.0) Gecko/X Firefox/4.0 Opera 12.11";
const char S22097[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.11,gzip(gfe)";
const char S22098[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.11";
const char S22099[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.11";
const char S22100[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.11";
const char S22101[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.11";
const char S22102[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.11";
const char S22103[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.11";
const char S22104[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.11";
const char S22105[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.11";
const char S22106[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.11";
const char S22107[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4933)) Presto/2.12.388 Version/12.11";
const char S22108[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.11";
const char S22109[] = "Opera/9.80 (Windows NT 5.1; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.11";
const char S22110[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.11";
const char S22111[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.11";
const char S22112[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.11";
const char S22113[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.11";
const char S22114[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.11";
const char S22115[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.11";
const char S22116[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.11";
const char S22117[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.11";
const char S22118[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.11";
const char S22119[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.11";
const char S22120[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.11";
const char S22121[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.11";
const char S22122[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.11";
const char S22123[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.11";
const char S22124[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.11";
const char S22125[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.11";
const char S22126[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.11 Yandex.Translate";
const char S22127[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.11";
const char S22128[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.11";
const char S22129[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.11 WebWasher 3.4";
const char S22130[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.11";
const char S22131[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.11";
const char S22132[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03658)) Presto/2.12.388 Version/12.11";
const char S22133[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.11";
const char S22134[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.11";
const char S22135[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.11";
const char S22136[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.11";
const char S22137[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5288)) Presto/2.12.388 Version/12.11";
const char S22138[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.11";
const char S22139[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.11";
const char S22140[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.11";
const char S22141[] = "Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.11";
const char S22142[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.11";
const char S22143[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.11";
const char S22144[] = "Opera/9.80 (Windows NT 5.1; Edition Yx) Presto/2.12.388 Version/12.11,gzip(gfe)";
const char S22145[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03789)) Presto/2.12.388 Version/12.11";
const char S22146[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03402)) Presto/2.12.388 Version/12.11";
const char S22147[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5299)) Presto/2.12.388 Version/12.11";
const char S22148[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03639)) Presto/2.12.388 Version/12.11";
const char S22149[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.11";
const char S22150[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5230)) Presto/2.12.388 Version/12.11";
const char S22151[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.11";
const char S22152[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03686); MRSPUTNIK OW 2, 3, 0, 244) Presto/2.12.388 Version/12.11";
const char S22153[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5660)) Presto/2.12.388 Version/12.11";
const char S22154[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03402); MRSPUTNIK OW 2, 3, 0, 104) Presto/2.12.388 Version/12.11";
const char S22155[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.11";
const char S22156[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.11";
const char S22157[] = "Opera/9.80 (Windows NT 5.1; MRA 5.3 (build 02552)) Presto/2.12.388 Version/12.11";
const char S22158[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.11";
const char S22159[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5336)) Presto/2.12.388 Version/12.11";
const char S22160[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5650)) Presto/2.12.388 Version/12.11";
const char S22161[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4832)) Presto/2.12.388 Version/12.11";
const char S22162[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5278)) Presto/2.12.388 Version/12.11";
const char S22163[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5217)) Presto/2.12.388 Version/12.11";
const char S22164[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.11";
const char S22165[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5734)) Presto/2.12.388 Version/12.11";
const char S22166[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4110)) Presto/2.12.388 Version/12.11";
const char S22167[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5300)) Presto/2.12.388 Version/12.11";
const char S22168[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03672)) Presto/2.12.388 Version/12.11";
const char S22169[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.11";
const char S22170[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.11";
const char S22171[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.11";
const char S22172[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.11";
const char S22173[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.11";
const char S22174[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5325)) Presto/2.12.388 Version/12.11";
const char S22175[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4661)) Presto/2.12.388 Version/12.11";
const char S22176[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.11";
const char S22177[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.11";
const char S22178[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03755)) Presto/2.12.388 Version/12.11";
const char S22179[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03791)) Presto/2.12.388 Version/12.11";
const char S22180[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.11,gzip(gfe) (via docs.google.com/viewer)";
const char S22181[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03638)) Presto/2.12.388 Version/12.11";
const char S22182[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.11";
const char S22183[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.11";
const char S22184[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.11";
const char S22185[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.11";
const char S22186[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.11";
const char S22187[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.11";
const char S22188[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.11";
const char S22189[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.11";
const char S22190[] = "Opera/9.80 (Windows NT 5.1; MRA 5.3 (build 02564)) Presto/2.12.388 Version/12.11";
const char S22191[] = "Opera/9.80 (Windows NT 5.1; MRA 5.4 (build 02625)) Presto/2.12.388 Version/12.11";
const char S22192[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03780)) Presto/2.12.388 Version/12.11";
const char S22193[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6035)) Presto/2.12.388 Version/12.11";
const char S22194[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.11";
const char S22195[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6080)) Presto/2.12.388 Version/12.11";
const char S22196[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.11";
const char S22197[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.11";
const char S22198[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6673)) Presto/2.12.388 Version/12.11";
const char S22199[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.11";
const char S22200[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.11";
const char S22201[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6638)) Presto/2.12.388 Version/12.11";
const char S22202[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6573)) Presto/2.12.388 Version/12.11";
const char S22203[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.6.30 Version/10.60";
const char S22204[] = "Opera/9.80 (Windows NT 6.1; U; en) Presto/2.6.30 Version/10.61";
const char S22205[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.6.30 Version/10.62";
const char S22206[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.6.30 Version/10.62";
const char S22207[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.6.30 Version/10.63";
const char S22208[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.6.30 Version/10.63";
const char S22209[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.6.30 Version/10.63";
const char S22210[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03755); ru) Presto/2.6.30 Version/10.63";
const char S22211[] = "Opera/9.80 (Windows NT 6.1; U; YB/3.5.1; ru) Presto/2.6.30 Version/10.63";
const char S22212[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.6.30 Version/10.63";
const char S22213[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.6.30 Version/10.63";
const char S22214[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.6.30 Version/10.62";
const char S22215[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.6.30 Version/10.60";
const char S22216[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.6.30 Version/10.62";
const char S22217[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.6.30 Version/10.60";
const char S22218[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.6.30 Version/10.62,gzip(gfe) (via translate.google.com)";
const char S22219[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.6.30 Version/10.61,gzip(gfe) (via translate.google.com)";
const char S22220[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03639); ru) Presto/2.6.30 Version/10.63";
const char S22221[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.6.30 Version/10.63";
const char S22222[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.6.30 Version/10.63";
const char S22223[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.6.30 Version/10.60";
const char S22224[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.6.30 Version/10.61";
const char S22225[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.6.30 Version/10.63";
const char S22226[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.6.30 Version/10.63";
const char S22227[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.6.30 Version/10.63,gzip(gfe) (via translate.google.com)";
const char S22228[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.6.30 Version/10.61";
const char S22229[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.6.30 Version/10.63";
const char S22230[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.6.30 Version/10.63";
const char S22231[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.6.30 Version/10.61";
const char S22232[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.6.30 Version/10.63";
const char S22233[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.6.30 Version/10.63";
const char S22234[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.6.30 Version/10.63";
const char S22235[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.6.30 Version/10.60";
const char S22236[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.6.30 Version/10.62";
const char S22237[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.6.30 Version/10.61";
const char S22238[] = "Opera/10.60 (Windows NT 6.1; U; ru) Presto/2.6.30 Version/10.60";
const char S22239[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.6.30 Version/10.63";
const char S22240[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.6.30 Version/10.63";
const char S22241[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.6.30 Version/10.60,gzip(gfe) (via translate.google.com)";
const char S22242[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.6.30 Version/10.63";
const char S22243[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.6.30 Version/10.61";
const char S22244[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.6.30 Version/10.63";
const char S22245[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.6.30 Version/10.61";
const char S22246[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.6.30 Version/10.62";
const char S22247[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.6.30 Version/10.60";
const char S22248[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.6.30 Version/10.61";
const char S22249[] = "Opera/9.80 (Windows NT 6.1; U; en) Presto/2.6.30 Version/10.60 CSVT/1V8DCX7NLW-4LW48EQOX(ENU)";
const char S22250[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.6.30 Version/10.63";
const char S22251[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.6.30 Version/10.63";
const char S22252[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.6.30 Version/10.63";
const char S22253[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.6.30 Version/10.63,gzip(gfe)";
const char S22254[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.6.30 Version/10.62";
const char S22255[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.6.30 Version/10.62";
const char S22256[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.6.30 Version/10.63";
const char S22257[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.6.30 Version/10.63";
const char S22258[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.6.30 Version/10.63";
const char S22259[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.6.30 Version/10.61";
const char S22260[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.6.30 Version/10.63";
const char S22261[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.6.30 Version/10.62";
const char S22262[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S22263[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S22264[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via translate.google.com)";
const char S22265[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)";
const char S22266[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)";
const char S22267[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (gtsr) Safari/537.4";
const char S22268[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (gtsr) Safari/537.4";
const char S22269[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (fnrb) Safari/537.4";
const char S22270[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (fnrb) Safari/537.4";
const char S22271[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.520 YaBrowser/1.5.1105.520 (bskt) Safari/537.4";
const char S22272[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.520 YaBrowser/1.5.1105.520 (bskt) Safari/537.4";
const char S22273[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.1.9999.433 Safari/537.4";
const char S22274[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.1.9999.433 Safari/537.4";
const char S22275[] = "FyberSpider/1.3 (http://www.fybersearch.com/fyberspider.php)";
const char S22276[] = "SpokeSpider/1.0 (http://support.spoke.com/webspider/) Mozilla/5.0 (not really)";
const char S22277[] = "OpenWebSpider v0.1.2.B (http://www.openwebspider.org/)";
const char S22278[] = "Mozilla/5.0 (compatible; WangIDSpider/1.0; +http://www.wangid.com/spider.html)";
const char S22279[] = "Mozilla/5.0 (compatible; JikeSpider; +http://shoulu.jike.com/spider.html)";
const char S22280[] = "Mozilla/4.0 (compatible; BOTW Spider; +http://botw.org)";
const char S22281[] = "Mozilla/5.0 (compatible;WI Job Roboter Spider Version 3;+http://www.webintegration.at)";
const char S22282[] = "DRKSpider - Website link validator - http://www.drk.com.ar/spider/";
const char S22283[] = "Mozilla/5.0 (compatible; YRSpider; +http://www.yunrang.com/yrspider.html)";
const char S22284[] = "OpenWebSpider v0.1.4 (http://www.openwebspider.org/)";
const char S22285[] = "http://www.activesearchresults.com/addwebsite.php ASR Ranking Technology/Spider/Crawler";
const char S22286[] = "SupremeSearch.net Spider-Bot/2.0 (http://www.supremesearch.net)";
const char S22287[] = "OrbitSpiderBot/1";
const char S22288[] = "Mozilla/5.0 (compatible; OpenindexSpider; +http://www.openindex.io/en/webmasters/spider.html)";
const char S22289[] = "Mozilla/5.0 (compatible; Googlebot/2.1; Sycara +http://www.sycara.com/SpiderInfo.php)";
const char S22290[] = "Wspider-1.0;http://www.w3456.com/ PrimarySpider";
const char S22291[] = "Mozilla/5.0 (compatible; EasouSpider; +http://www.easou.com/spider.html)";
const char S22292[] = "Content Crawler Spider";
const char S22293[] = "Mozilla/5.0 (compatible; EasouSpider; +http://www.easou.com/search/spider.html)";
const char S22294[] = "Mozilla/5.0 (compatible; SecMindSpider; +http://securemymind.com/spider.html)";
const char S22295[] = "Mozilla/5.0 (compatible; SecMindSpider; +http://www.securemymind.com/spider.html)";
const char S22296[] = "Mozilla/5.0 (compatible; SouthWestSpider; +http://dianlake.com/spider.html)";
const char S22297[] = "Mozilla/5.0 (compatible; GreatChinaSpider; +http://www.learnatchina.com/spider.html)";
const char S22298[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; rv:2.2) Gecko/20110201";
const char S22299[] = "QuickTime/7.6.9 (qtver=7.6.9;os=Windows NT 6.1)";
const char S22300[] = "Mozilla/4.0 (compatible; SuperCleaner 2.96; Windows NT 6.1)";
const char S22301[] = "QuickTime/7.7 (qtver=7.7;os=Windows NT 6.1Service Pack 1)";
const char S22302[] = "Mozilla/5.0 (compatible; MSIE/9.0; Windows NT 6.1; Trident/5.0; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.1.4322)";
const char S22303[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US)";
const char S22304[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv";
const char S22305[] = "Mozilla/4.0 (compatible; ArgClrInt/1.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S22306[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534+ (KHTML, like Gecko) BingPreview/1.0b";
const char S22307[] = "Mozilla/4.0 (compatible; ArgClrInt/1.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET4.0C; .NET4.0E; .NET CLR 3.5.30729; .NET CLR 3.0.30729)";
const char S22308[] = "Mozilla/4.0 (Windows 7 6.1) Java/1.6.0_31";
const char S22309[] = "Mozilla/5.0 (Java 1.6.0_17; Windows 7 6.1 amd64; en_US) ICEbrowser/v6_0_2 Java/1.6.0_17";
const char S22310[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; fr; rv:1.8.1.20)";
const char S22311[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; fr; rv:1.9.0.6)";
const char S22312[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; zh-CN) AppleWebKit/533+ (KHTML, like Gecko)";
const char S22313[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1)";
const char S22314[] = "InetURL/1.0 (Windows NT 6.1)";
const char S22315[] = "Mozilla/5.0 (compatible; Custo 3 (Netwu.com); Windows NT 6.1)";
const char S22316[] = "IBM-LC-Base-Notes_8.5.3.X_0921-Windows 7 Java/1.6.0";
const char S22317[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko)";
const char S22318[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.57.2 (KHTML, like Gecko)";
const char S22319[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534+ (KHTML, like Gecko) BingPreview/1.0b,gzip(gfe) (via docs.google.com/viewer)";
const char S22320[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.0) Gecko/ /12.0";
const char S22321[] = "Mozilla/4.0 (Windows 7 6.1) Java/1.7.0_07";
const char S22322[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.9 (KHTML, like Gecko) Boxee/1.0.1 bxapi/7.1";
const char S22323[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1) Gecko/X";
const char S22324[] = "Mozilla/5.0 (Windows NT 6.1)";
const char S22325[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534+ (KHTML, like Gecko) OpenWebKitSharp";
const char S22326[] = "WinM RSS Reader/1.2 (Windows; U; Windows NT 6.1; en-GB; rv:1.9.2.13)";
const char S22327[] = "User-Agent:Mozilla/5.0 (Windows NT 6.1; WOW64)";
const char S22328[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-my) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22329[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-in) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22330[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; pt-br) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22331[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; el-gr) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22332[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; da-dk) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22333[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-no) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22334[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; fr-fr) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22335[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; cs-cz) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22336[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-gb) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22337[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; nl-nl) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22338[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; de-de) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22339[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-sg) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22340[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; sv-se) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22341[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-tw) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22342[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; zh-tw) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22343[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-za) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22344[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-ca) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22345[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-ph) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22346[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-se) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22347[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-cz) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22348[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; es-ve) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22349[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-pl) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22350[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-us) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13";
const char S22351[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22352[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22353[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22354[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; de-de) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22355[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20";
const char S22356[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18";
const char S22357[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; nl-nl) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22358[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; nl-nl) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18";
const char S22359[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; nl-nl) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22360[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; es-es) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22361[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; it-it) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22362[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; es-es) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22363[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; fr-fr) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22364[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; fr-fr) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20";
const char S22365[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; es-es) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22366[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_4; fr-fr) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22367[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22368[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; de-de) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22369[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-au) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22370[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; it-it) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22371[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; fr-fr) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18";
const char S22372[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; de-de) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18";
const char S22373[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3;) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22374[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; ru-ru) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22375[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3;) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.22";
const char S22376[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; pt-br) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22377[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; it-it) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22378[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; ru-ru) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22379[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; sv-se) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22380[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; de-de) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20";
const char S22381[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; nb-no) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22382[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; hu-hu) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20";
const char S22383[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; nb-no) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22384[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/3.1.1 Safari/525.18";
const char S22385[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22386[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/3.1.1 Safari/525.18";
const char S22387[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-gb) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22388[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3;) AppleWebKit/525.19 (KHTML, like Gecko) Version/3.1.2 Safari/525.21";
const char S22389[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; ru-ru) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22390[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; de-de) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13";
const char S22391[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; sv-se) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22392[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; nb-no) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22393[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; ja-jp) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22394[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-gb) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22395[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/3.1 Safari/525.13";
const char S22396[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.1 Safari/525.13";
const char S22397[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; ja-jp) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22398[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; pt-pt) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22399[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; cs-cz) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13";
const char S22400[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; fi-fi) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22401[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; fi-fi) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22402[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; zh-tw) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22403[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; it-it) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13";
const char S22404[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; ja-jp) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22405[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; pt-br) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22406[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; pl-pl) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22407[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22408[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; ja-jp) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18";
const char S22409[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; es-es) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13";
const char S22410[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; ru-ru) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18";
const char S22411[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; ru-ru) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20";
const char S22412[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-au) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22413[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; ko-kr) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22414[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; pl-pl) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22415[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; ja-jp) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20";
const char S22416[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; cs-cz) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22417[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-au) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22418[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; es-es) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20";
const char S22419[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; el-gr) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22420[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; it-it) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18";
const char S22421[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-us) AppleWebKit/525.12 (KHTML, like Gecko) Version/3.1 Safari/525.12";
const char S22422[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; it-it) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.1";
const char S22423[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en) AppleWebKit/525.9 (KHTML, like Gecko) Version/3.1 Safari/525.9";
const char S22424[] = "Mozilla/5.0 (WebTubeHD; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22425[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; fi-fi) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22426[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; nl-nl) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20";
const char S22427[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; tr-tr) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22428[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; zh-cn) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22429[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; it-it) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20";
const char S22430[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; da-dk) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22431[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; zh-cn) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22432[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; es-es) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18";
const char S22433[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; nb-no) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20";
const char S22434[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; en-gb) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20";
const char S22435[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; ar) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22436[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22437[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; pl-pl) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22438[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; es-xl) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22439[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_2; en-gb) AppleWebKit/526+ (KHTML, like Gecko) Version/3.1 Safari/525.9";
const char S22440[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; fr-fr) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13";
const char S22441[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_2; en) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18";
const char S22442[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; en-ca) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20";
const char S22443[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_4; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1 Safari/525.13";
const char S22444[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_3; en) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20";
const char S22445[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; nl-nl) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13";
const char S22446[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22447[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; zh-cn) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22448[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; ar) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22449[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; zh-tw) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22450[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; pt-pt) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22451[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; pt-br) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18";
const char S22452[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; fr-fr) AppleWebKit/525.9 (KHTML, like Gecko) Version/3.1 Safari/525.9";
const char S22453[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22454[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; da-dk) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22455[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; tr-tr) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22456[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sv-se) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22457[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-gb) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18";
const char S22458[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22459[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; ru-ru) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13";
const char S22460[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; th-th) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22461[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; ja-jp) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13";
const char S22462[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-us) AppleWebKit/525.9 (KHTML, like Gecko) Version/3.1 Safari/525.9";
const char S22463[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; he-il) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22464[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13";
const char S22465[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22466[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-us) AppleWebKit/525.7 (KHTML, like Gecko) Version/3.1 Safari/525.7";
const char S22467[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; de-de) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20";
const char S22468[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; pl-pl) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20";
const char S22469[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_3; sv-se) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20";
const char S22470[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; es-xl) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S22471[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; pt-br) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13";
const char S22472[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-gb) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13";
const char S22473[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22474[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22475[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-I535 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22476[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-i9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22477[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22478[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22479[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22480[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I93000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22481[] = "Mozilla/5.0 (Linux; Android 4.1.1;  GT-I9300 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S22482[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300T Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22483[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S22484[] = "Mozilla/5.0 (Linux; Android 4.1.9; GT-I9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22485[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999V Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22486[] = "Mozilla/5.0 (Linux; Android 4.1.2; SC-06D Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22487[] = "Mozilla/5.0 (Linux; Android 4.1.1; I9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22488[] = "Mozilla/5.0 (Linux; Android 4.1.1; I9300i Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22489[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22490[] = "Mozilla/5.0 (Linux; Android 4.1; i9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22491[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22492[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S22493[] = "Mozilla/5.0 (Linux; Android 4.1.2_????_??????; GT-I9300 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22494[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999N Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22495[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999V Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22496[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I939 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22497[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-I939 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22498[] = "Opera/9.64(Windows NT 5.1; U; en) Presto/2.1.1";
const char S22499[] = "Opera/9.64 (Windows NT 5.2; U; en) Presto/2.1.1";
const char S22500[] = "Opera/9.63 (Windows NT 5.1; U; ru) Presto/2.1.1";
const char S22501[] = "Opera/9.60 (Windows NT 5.1; U; ru) Presto/2.1.1";
const char S22502[] = "Opera/9.64 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.1.1";
const char S22503[] = "Opera/9.61 (Windows NT 5.1; U; en) Presto/2.1.1";
const char S22504[] = "Opera/9.62 (Windows NT 5.1; U; ru) Presto/2.1.1";
const char S22505[] = "Opera/9.63 (Windows NT 5.2; U; en) Presto/2.1.1";
const char S22506[] = "Opera/9.64 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.1.1";
const char S22507[] = "Opera/9.63 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.1.1";
const char S22508[] = "Opera/9.62 (Windows NT 5.2; U; en) Presto/2.1.1";
const char S22509[] = "Opera/9.64 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.1.1";
const char S22510[] = "Opera/9.60 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.1.1";
const char S22511[] = "Opera/9.63 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.1.1";
const char S22512[] = "Opera/9.60 (Windows NT 5.2; U; en) Presto/2.1.1";
const char S22513[] = "Opera/9.64 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.1.1";
const char S22514[] = "Opera/9.62 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.1.1";
const char S22515[] = "Opera/9.61 (Windows NT 5.2; U; en) Presto/2.1.1";
const char S22516[] = "Opera/9.64 (Windows NT 5.1; U; MRA 5.7 (build 03686); MRSPUTNIK OW 2, 3, 0, 244; ru) Presto/2.1.1";
const char S22517[] = "Opera/9.60 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.1.1";
const char S22518[] = "Opera/9.60 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.1.1";
const char S22519[] = "Opera/9.64 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.1.1";
const char S22520[] = "Opera/9.63 (Windows NT 5.1; U; MRA 5.7 (build 03686); MRSPUTNIK OW 2, 3, 0, 244; ru) Presto/2.1.1";
const char S22521[] = "Opera/9.64 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.1.1";
const char S22522[] = "Opera/9.63 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.1.1";
const char S22523[] = "Opera/9.64 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.1.1";
const char S22524[] = "Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 9.63";
const char S22525[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 9.64";
const char S22526[] = "Opera/9.64 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.1.1";
const char S22527[] = "Opera/9.63 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.1.1";
const char S22528[] = "Mozilla/5.0 (Windows NT 5.1; U; en-GB; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 9.61";
const char S22529[] = "Opera/9.62 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.1.1";
const char S22530[] = "Opera/9.63 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.1.1";
const char S22531[] = "Opera/9.64 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.1.1";
const char S22532[] = "Opera/9.64 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.1.1";
const char S22533[] = "Opera/9.63 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.1.1";
const char S22534[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 9.63";
const char S22535[] = "Opera/9.64 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.1.1";
const char S22536[] = "Opera/9.62 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.1.1";
const char S22537[] = "Opera/9.64 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.1.1";
const char S22538[] = "Opera/9.60 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.1.1";
const char S22539[] = "Opera/9.64 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.1.1";
const char S22540[] = "Opera/9.63 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.1.1";
const char S22541[] = "Mozilla/5.0 (Windows NT 5.1; U; ru; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 9.64";
const char S22542[] = "Opera/9.62 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.1.1";
const char S22543[] = "Opera/9.63 (Windows NT 5.1; U; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94; ru) Presto/2.1.1";
const char S22544[] = "Opera/9.62 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.1.1";
const char S22545[] = "Opera/9.64 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.1.1";
const char S22546[] = "Opera/9.63 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.1.1";
const char S22547[] = "Opera/9.64 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.1.1";
const char S22548[] = "Opera/9.64 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.1.1";
const char S22549[] = "Opera/9.64 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.1.1";
const char S22550[] = "Opera/9.63 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.1.1";
const char S22551[] = "Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 9.62";
const char S22552[] = "Opera/9.62 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.1.1";
const char S22553[] = "Opera/9.64 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.1.1";
const char S22554[] = "Opera/9.63 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.1.1";
const char S22555[] = "Opera/9.64 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.1.1";
const char S22556[] = "Opera/9.63 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.1.1";
const char S22557[] = "Opera/9.63 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.1.1";
const char S22558[] = "Opera/9.64 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.1.1";
const char S22559[] = "Opera/9.64 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.1.1";
const char S22560[] = "Opera/9.63 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.1.1";
const char S22561[] = "Opera/9.63 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.1.1";
const char S22562[] = "Opera/9.64 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.1.1";
const char S22563[] = "Opera/9.64 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.1.1";
const char S22564[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10";
const char S22565[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10,gzip(gfe) (via translate.google.com)";
const char S22566[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; de-DE) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10";
const char S22567[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.10 (KHTML	 like Gecko) Chrome/8.0.552.237 Safari/534.10 control/6.02164";
const char S22568[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10,gzip(gfe)";
const char S22569[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.84 Safari/534.13";
const char S22570[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13";
const char S22571[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.12 (KHTML, like Gecko) Chrome/9.0.587.0 Safari/534.12,gzip(gfe) (via translate.google.com)";
const char S22572[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13 SunChrome/9.0";
const char S22573[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.13 (KHTML	 like Gecko) Chrome/9.0.597.19 Safari/534.13";
const char S22574[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13,gzip(gfe)";
const char S22575[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-CA) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.0 Safari/534.13";
const char S22576[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.12 (KHTML like Gecko) Chrome/9.0.576.0 Safari/534.12";
const char S22577[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; fr-CA) AppleWebKit/534.12 (KHTML like Gecko) Chrome/9.0.576.0 Safari/534.12";
const char S22578[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; es-US) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13";
const char S22579[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; haw-US) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.0 Safari/534.13";
const char S22580[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; fr-FR) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.0 Safari/534.13";
const char S22581[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; da-DE) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.0 Safari/534.13";
const char S22582[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13";
const char S22583[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; ku-DE) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13";
const char S22584[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; ru-DE) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13";
const char S22585[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/534.12 (KHTML, like Gecko) Chrome/9.0.576.0 Safari/534.12";
const char S22586[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; de-DE) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.0 Safari/534.13";
const char S22587[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; br-FR) AppleWebKit/534.12 (KHTML like Gecko) Chrome/9.0.576.0 Safari/534.12";
const char S22588[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; fy-DE) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.0 Safari/534.13";
const char S22589[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; oc-FR) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.0 Safari/534.13";
const char S22590[] = "Mozilla/5.0 (Windows U Windows NT 5.1 en-US) AppleWebKit/534.12 (KHTML, like Gecko) Chrome/9.0.583.0 Safari/534.12";
const char S22591[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22592[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22593[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22594[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5660V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22595[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; GT-S5660L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22596[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-S5660 Build/GINGERBREAD)";
const char S22597[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5660 Build/GINGERBREAD)";
const char S22598[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22599[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; SAMSUNG GT-S5660V/S5660VXXKQE Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22600[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; SAMSUNG GT-S5660V/S5660VXXKQK Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22601[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5660 Build/GINGERBREAD)";
const char S22602[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5660 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S22603[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5660V Build/GINGERBREAD)";
const char S22604[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-S5660L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22605[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-S5660V/S5660VXXKT4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22606[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-S5660 Build/GINGERBREAD)";
const char S22607[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5660 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S22608[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5660L Build/GINGERBREAD)";
const char S22609[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; fa-fa; GT-S5660 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S22610[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5660 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S22611[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22612[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22613[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5660 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22614[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; GT-S5660 Build/GRWK74)";
const char S22615[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fa-fa; GT-S5660 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S22616[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-S5660V/S5660VXXKT8 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22617[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/23.0.1271.64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S22618[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/23.0.1271.64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S22619[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/23.0.1271.91) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.91 Safari/537.11";
const char S22620[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/23.0.1271.91) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.91 Safari/537.11";
const char S22621[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/23.0.1271.95) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11";
const char S22622[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/23.0.1271.95) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11";
const char S22623[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/23.0.1271.97; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S22624[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/23.0.1271.97)";
const char S22625[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/23.0.1271.97)";
const char S22626[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/23.0.1271.97) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S22627[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; chromeframe/23.0.1271.97; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S22628[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/23.0.1271.97) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S22629[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; chromeframe/23.0.1271.97; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S22630[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; chromeframe/23.0.1271.97; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S22631[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/23.0.1271.97; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S22632[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/23.0.1271.97; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C)";
const char S22633[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; chromeframe/23.0.1271.97; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S22634[] = "Mozilla/5.0 (compatible; AhrefsBot/1.0; +http://ahrefs.com/robot/)";
const char S22635[] = "Mozilla/5.0 (compatible; AhrefsBot/2.0; +http://ahrefs.com/robot/)";
const char S22636[] = "Mozilla/5.0 (compatible; AhrefsBot/2.0; +http://ahrefs.com/robot/),gzip(gfe) (via docs.google.com/viewer)";
const char S22637[] = "Mozilla/5.0 (compatible; AhrefsBot/3.1; +http://ahrefs.com/robot/)";
const char S22638[] = "Mozilla/5.0 (compatible; AhrefsBot/4.0; +http://ahrefs.com/robot/)";
const char S22639[] = "AhrefsBot.Feeds v0.1; http://ahrefs.com/";
const char S22640[] = "Mozilla/5.0 (compatible; AhrefsBot.FreshPages/0.1; +http://ahrefs.com/robot/)";
const char S22641[] = "Mozilla/5.0 (compatible; AhrefsBot/5.0; +http://ahrefs.com/robot/)";
const char S22642[] = "Mozilla/5.2 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.2.3) Gecko/X Firefox/3.6.3 Java/1.6.0_31";
const char S22643[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6;en-US; rv:1.9.2.9) Gecko/X Firefox/3.6.9";
const char S22644[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US) Gecko/X Firefox/3.6.11";
const char S22645[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.2.28) Gecko/X (CK-Firefox) Firefox/3.6.28";
const char S22646[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.2) Gecko/X Firefox 3.6";
const char S22647[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1";
const char S22648[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S22649[] = "Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S22650[] = "Mozilla/5.0 (X11; Jolicloud Linux i686) AppleWebKit/537.1 (KHTML, like Gecko) Joli OS/1.2 Chromium/21.0.1180.79 Chrome/21.0.1180.79 Safari/537.1";
const char S22651[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)";
const char S22652[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)";
const char S22653[] = "Mozilla/5.0 \\(X11; Linux x8664\\) AppleWebKit/537.1 \\(KHTML like Gecko\\) Chrome/21.0.1180.89 Safari/537.1";
const char S22654[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe)";
const char S22655[] = "Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/21.0.1410.63 Safari/537.31";
const char S22656[] = "Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/21.0.1410.63 Safari/537.31";
const char S22657[] = "Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/21.0.1410.63 Safari/537.31";
const char S22658[] = "Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/21.0.1410.63 Safari/537.31";
const char S22659[] = "Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/21.0.1410.63 Safari/537.31";
const char S22660[] = "Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/21.0.1410.63 Safari/537.31";
const char S22661[] = "Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/21.0.1410.63 Safari/537.31";
const char S22662[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.12 Safari/535.11";
const char S22663[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S22664[] = "Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.83 Safari/535.11";
const char S22665[] = "Mozilla/5.0 (X11; Jolicloud Linux i686) AppleWebKit/535.11 (KHTML, like Gecko) Joli OS/1.2 Chromium/17.0.963.46 Chrome/17.0.963.46 Safari/535.11";
const char S22666[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/unstable (3.4.1-1) Epiphany/3.4.1";
const char S22667[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/unstable (3.4.2-1) Epiphany/3.4.2";
const char S22668[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/unstable (3.4.2-1) Epiphany/3.4.2";
const char S22669[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11,gzip(gfe) (via translate.google.com)";
const char S22670[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/sid/experimental (3.2.1-2) Epiphany/3.2.1";
const char S22671[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/unstable (3.4.2-1+build1) Epiphany/3.4.2";
const char S22672[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/unstable (3.4.2-1+build1) Epiphany/3.4.2";
const char S22673[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+; (+http://immediatenet.com/thumbnail_api.html; free thumbnails)";
const char S22674[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/unstable (3.4.2-2) Epiphany/3.4.2";
const char S22675[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/unstable (3.4.2-2) Epiphany/3.4.2";
const char S22676[] = "Mozilla/5.0 (X11; Linux) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+";
const char S22677[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/unstable (3.4.1-1) Epiphany/3.4.1";
const char S22678[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/sid/experimental (3.2.1-2) Epiphany/3.2.1";
const char S22679[] = "Mozilla/5.0 (X11; Linux armv6l) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+";
const char S22680[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/7.0 (3.4.2-2.1) Epiphany/3.4.2";
const char S22681[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/7.0 (3.4.2-2.1) Epiphany/3.4.2";
const char S22682[] = "Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/17.0.1410.63 Safari/537.31";
const char S22683[] = "Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/17.0.1410.63 Safari/537.31";
const char S22684[] = "Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/17.0.1410.63 Safari/537.31";
const char S22685[] = "Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/17.0.1410.63 Safari/537.31";
const char S22686[] = "Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/17.0.1410.63 Safari/537.31";
const char S22687[] = "Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/17.0.1410.63 Safari/537.31";
const char S22688[] = "Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/17.0.1410.63 Safari/537.31";
const char S22689[] = "Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/537.6+ SugarLabs/0.98";
const char S22690[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ SUSE/12.2 (3.4.2) Epiphany/3.4.2";
const char S22691[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; E15a Build/2.0.2.A.0.24) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22692[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; SonyEricssonE15av Build/2.0.2.A.0.24) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22693[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-ar; SonyEricssonE15a Build/2.0.1.A.0.47) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22694[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; SonyEricssonE15i-o Build/2.1.1.A.0.16) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22695[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; SonyEricssonE15i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22696[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1 - JIT Enabled; vi-vn; E15i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22697[] = "Mozilla/5.0(Linux;U;Android2.1-update1;Zh_cn;E15i;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S22698[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; E15i Build/2.0.1.A.0.47) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22699[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; SonyEricssonE15iv Build/2.0.1.A.0.47) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22700[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; E15i Build/2.0.2.A.0.24) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22701[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; E15i Build/2.1.1.A.0.16) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22702[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; E15i Build/2.1.1.C.0.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22703[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-mx; E15a Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22704[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; E15i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22705[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; iw-il; E15a Build/2.1.1.A.0.16) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22706[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ph; SonyEricssonE15iBuild/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22707[] = "Android (Linux; U; Android 2.1-update1; zh-cn; E15i) AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S22708[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-mx; E15a Build/2.0.1.A.0.47) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22709[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; E15i Build/2.1.1.A.0.6)";
const char S22710[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; E15i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22711[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fa-ir; E15i Build/2.1.1.A.0.6) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S22712[] = "PVPlayer E15i CORE/7.507.1.1 OpenCORE/2.05 (Linux;Android2.1-update1)";
const char S22713[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; E15a Build/2.1.1.A.0.6)";
const char S22714[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; E15i Build/2.1.1.A.0.16) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S22715[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; E15i Build/2.0.1.A.0.47) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S22716[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fa-ir; E15i Build/2.1.1.C.0.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S22717[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; E15i Build/2.1.1.A.0.6) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S22718[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; E15i Build/2.1.1.A.0.16) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S22719[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; E15i Build/2.1.1.B.0.5) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22720[] = "Mozilla/5.0 (X11; CrOS i686 4008.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1480.0 Safari/537.36";
const char S22721[] = "Mozilla/5.0 (X11; CrOS x86_64 4008.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1480.0 Safari/537.36";
const char S22722[] = "Mozilla/5.0 (X11; CrOS i686 4068.1.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1494.8 Safari/537.36";
const char S22723[] = "Mozilla/5.0 (X11; CrOS armv7l 4068.1.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1494.8 Safari/537.36";
const char S22724[] = "Mozilla/5.0 (X11; CrOS i686 4068.2.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1494.8 Safari/537.36";
const char S22725[] = "Mozilla/5.0 (X11; CrOS x86_64 4050.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1489.4 Safari/537.36";
const char S22726[] = "Mozilla/5.0 (X11; CrOS i686 4028.0.2013) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1484.1 Safari/537.36";
const char S22727[] = "Mozilla/5.0 (X11; CrOS x86_64 4068.1.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1494.8 Safari/537.36";
const char S22728[] = "Mozilla/5.0 (X11; CrOS i686 4035.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1485.1 Safari/537.36";
const char S22729[] = "Mozilla/5.0 (X11; CrOS armv7l 4035.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1485.1 Safari/537.36";
const char S22730[] = "Mozilla/5.0 (X11; CrOS x86_64 4035.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1485.1 Safari/537.36";
const char S22731[] = "Mozilla/5.0 (X11; CrOS x86_64 4100.1.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.4 Safari/537.36";
const char S22732[] = "Mozilla/5.0 (X11; CrOS armv7l 4100.1.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.4 Safari/537.36";
const char S22733[] = "Mozilla/5.0 (X11; CrOS i686 4100.2.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.4 Safari/537.36";
const char S22734[] = "Mozilla/5.0 (X11; CrOS armv7l 4100.7.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.11 Safari/537.36";
const char S22735[] = "Mozilla/5.0 (X11; CrOS x86_64 4100.7.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.11 Safari/537.36";
const char S22736[] = "Mozilla/5.0 (X11; CrOS i686 4100.7.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.11 Safari/537.36";
const char S22737[] = "Mozilla/5.0 (X11; CrOS i686 4100.1.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.4 Safari/537.36";
const char S22738[] = "Mozilla/5.0 (X11; CrOS x86_64 4100.17.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.20 Safari/537.36";
const char S22739[] = "Mozilla/5.0 (X11; CrOS i686 4100.17.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.20 Safari/537.36";
const char S22740[] = "Mozilla/5.0 (X11; CrOS armv7l 4100.17.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.20 Safari/537.36";
const char S22741[] = "Mozilla/5.0 (X11; CrOS armv7l 4100.38.5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.35 Safari/537.36";
const char S22742[] = "Mozilla/5.0 (X11; CrOS i686 4100.38.4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.35 Safari/537.36";
const char S22743[] = "Mozilla/5.0 (X11; CrOS x86_64 4100.38.3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.35 Safari/537.36";
const char S22744[] = "Mozilla/5.0 (X11; CrOS i686 4100.38.3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.35 Safari/537.36";
const char S22745[] = "Mozilla/5.0 (X11; CrOS i686 4100.44.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.47 Safari/537.36";
const char S22746[] = "Mozilla/5.0 (X11; CrOS x86_64 4100.44.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.47 Safari/537.36";
const char S22747[] = "Mozilla/5.0 (X11; CrOS armv7l 4100.44.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.47 Safari/537.36";
const char S22748[] = "Mozilla/5.0 (X11; CrOS i686 4100.48.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.53 Safari/537.36";
const char S22749[] = "Mozilla/5.0 (X11; CrOS x86_64 4100.48.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.53 Safari/537.36";
const char S22750[] = "Mozilla/5.0 (X11; CrOS armv7l 4100.48.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.53 Safari/537.36";
const char S22751[] = "Mozilla/5.0 (X11; CrOS i686 4100.53.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.61 Safari/537.36";
const char S22752[] = "Mozilla/5.0 (X11; CrOS x86_64 4100.53.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.61 Safari/537.36";
const char S22753[] = "Mozilla/5.0 (X11; CrOS armv7l 4100.53.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.61 Safari/537.36";
const char S22754[] = "Mozilla/5.0 (X11; CrOS i686 4100.53.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.66 Safari/537.36";
const char S22755[] = "Mozilla/5.0 (X11; CrOS x86_64 4100.53.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.66 Safari/537.36";
const char S22756[] = "Mozilla/5.0 (X11; CrOS armv7l 4100.53.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.66 Safari/537.36";
const char S22757[] = "Mozilla/5.0 (X11; CrOS i686 4100.53.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.66 Safari/537.36";
const char S22758[] = "Mozilla/5.0 (X11; CrOS x86_64 4100.68.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.68 Safari/537.36";
const char S22759[] = "Mozilla/5.0 (X11; CrOS armv7l 4100.68.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.68 Safari/537.36";
const char S22760[] = "Mozilla/5.0 (X11; CrOS i686 4100.78.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.68 Safari/537.36";
const char S22761[] = "Mozilla/5.0 (X11; CrOS i686 4100.68.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.68 Safari/537.36";
const char S22762[] = "Mozilla/5.0 (X11; CrOS i686 3927.0.2013) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1456.1 Safari/537.36";
const char S22763[] = "Mozilla/5.0 (X11; CrOS i686 4100.86.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S22764[] = "Mozilla/5.0 (X11; CrOS x86_64 4100.86.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S22765[] = "Mozilla/5.0 (X11; CrOS armv7l 4100.86.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S22766[] = "Mozilla/5.0 (X11; CrOS x86_64 4100.89.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S22767[] = "Mozilla/5.0 (X11; CrOS armv7l 4100.89.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S22768[] = "Mozilla/5.0 (X11; CrOS i686 4100.89.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S22769[] = "Mozilla/5.0 (X11; CrOS armv7l 4100.53.15) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S22770[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S22771[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S22772[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S22773[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S22774[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S22775[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)";
const char S22776[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S22777[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via docs.google.com/viewer)";
const char S22778[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N8010 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S22779[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-N8010 Build/JRO03C)";
const char S22780[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N8010 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S22781[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N8010 Build/JZO54K)";
const char S22782[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-N8010 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S22783[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N8010 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S22784[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I9000/I9000ZSJPD Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22785[] = "Mozilla/5.0 (Linux; U; Android 2.2; hr-hr; GT-I9000 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22786[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-I9000 Build/XWJS5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22787[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22788[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22789[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; en-gb; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22790[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-I9000 Build/Vibrace V.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22791[] = "Mozilla/5.0 (Linux; U; Android 2.2; pt-br; GT-I9000B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22792[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; hr-hr; GT-I9000 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22793[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22794[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22795[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22796[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I9000 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22797[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I9000 Build/FROYO)";
const char S22798[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-I9000M Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22799[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-I9000 Build/FROYO)";
const char S22800[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; GT-I9000B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22801[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I9000 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22802[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I9000M Build/FROYO)";
const char S22803[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-I9000 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S22804[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-I9000 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S22805[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I9000B Build/FROYO)";
const char S22806[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-I9000 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S22807[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-I9000 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S22808[] = "Mozilla/5.0 (Linux; Android 4.2; Nexus 7 Build/JOP40C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S22809[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 7 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S22810[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 7 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S22811[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S22812[] = "Mozilla/5.0 (Linux; Android 4.2; en-US; Nexus 7 Build/JOP40C) AppleWebKit/535.19 (KHTML like Gecko) Chrome/18.0.1025.166 Safari/535.19";
const char S22813[] = "Mozilla/5.0 (Linux; Android 4.2.1; haw-US; Nexus 7 Build/JOP40D) AppleWebKit/535.19 (KHTML like Gecko) Chrome/18.0.1025.166 Safari/535.19";
const char S22814[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 7 3G Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S22815[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:16.0) Gecko/X Firefox/16.0 AlexaToolbar/alxf-2.17";
const char S22816[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:16.0) Gecko/X Firefox/16.0";
const char S22817[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:16.0) Gecko/X Firefox/16.00";
const char S22818[] = "Mozilla/5.0 (Windows; Windows NT 6.1; Win64; x64; rv:16.0) Gecko/16.0 Firefox/16.0";
const char S22819[] = "Mozilla/6.0 (Windows NT 6.1; WOW64; rv:16.0.1) Gecko/X Firefox/16.0.1";
const char S22820[] = "Mozilla/5.0 Windows NT 6.1 WOW64 rv 16.0 Gecko/X Firefox/16.0";
const char S22821[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; rv:16.0.1) Gecko/X Firefox/16.0.1";
const char S22822[] = "Mozilla/5.0 (compatible; Windows; U; Windows NT 6.0; en-US; rv:1.9.0.7; .NET CLR 1.1.4322; .NET CLR 1.0.3705) Gecko/X Firefox/3.0.7";
const char S22823[] = "Mozilla /5.0 (Windows NT 6.0; U; en) Gecko/X Firefox/3.0.1";
const char S22824[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; rv:1.8.1) Gecko/X Firefox/3.0Mozilla/5.0 (Windows; U; Windows NT 6.1; rv:1.9.2) Gecko/X Firefox/3.0";
const char S22825[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.14) Gecko/X YFF3 Firefox/3.0.14 ( .NET CLR 3.5.30729) WinNT-PAI 08.08.2009 SearchToolbar/1.2";
const char S22826[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 (.NET CLR 3.5.30729) WinNT-EVI 01.05.2010";
const char S22827[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 (.NET CLR 3.5.30729) WinNT-EVI 30.04.2010 FBSMTWB";
const char S22828[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.14) Gecko/X Firefox/3.0.14 (.NET CLR 3.5.30729) WinNT-PAI 17.08.2009 FBSMTWB";
const char S22829[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; de; rv:1.9b5) Gecko/X Firefox/3.0b5";
const char S22830[] = "Mozilla/5.0 (compatible; U; Windows NT 6.0) Gecko/X Firefox/3.0.3";
const char S22831[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.11) Gecko/X YFF3 Firefox/3.0.11 (.NET CLR 3.5.30729) WinNT-PAI 18.08.2009 SearchToolbar FBSMTWB";
const char S22832[] = "Mozilla/4.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.3) Gecko/X Firefox/3.0.3";
const char S22833[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 WinNT-PAI 30.06.2009";
const char S22834[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.16) Gecko/X Firefox/3.0.16 (.NET CLR 3.5.30729) WinNT-PAI 13.09.2009 FBSMTWB";
const char S22835[] = "Mozi    lla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.3) Gecko/X Firefox/3.0.3";
const char S22836[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.17) Gecko/X Firefox/3.0.17 GTB6 (.NET CLR 3.5.30729) WinTSI 24.12.2009 FBSMTWB";
const char S22837[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; LG-E610-Orange Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S22838[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; LG-E612 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S22839[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; LG-E610 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S22840[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-mx; LG-E612f Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S22841[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-E610/V10d Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S22842[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; LG-E610v/V10c Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S22843[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; LG-E610v/V10d Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S22844[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; LG-E610/V10a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S22845[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-sa; LG-E612g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S22846[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-E610 Build/IML74K)";
const char S22847[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; et; LG-E610 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S22848[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-E610/V10e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S22849[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-E612 Build/IML74K)";
const char S22850[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; LG-E610v Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S22851[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; LG-E610/V10f Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S22852[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-E612f Build/IML74K)";
const char S22853[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-E610/V10l Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S22854[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; LG-E610v/V10f Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S22855[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-E610v Build/IML74K)";
const char S22856[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-E612g Build/IML74K)";
const char S22857[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; LG-E610/V10c Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S22858[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; LG-E610/V10b Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S22859[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; LG-E610v/V10g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S22860[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-E610/V10j Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S22861[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-E610/V10g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S22862[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22863[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22864[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; GT-P1000L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22865[] = "Mozilla/5.0(Linux;U;Android2.2;Zh_cn;GT-P1000;600*1024;)AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S22866[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-us; GT-P1000N Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22867[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-P1000R Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22868[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-au; GT-P1000T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22869[] = "Mozilla/5.0 (Linux; U; Android 2.2; pt-br; GT-P1000L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22870[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22871[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22872[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-P1000M Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22873[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Tablet Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22874[] = "Mozilla/5.0(Linux;U;Android2.2;en-ie;GT-P1000Build/FROYO)AppleWebKit/533.1(KHTML,likeGecko)Version/4.0MobileSafari/533.1";
const char S22875[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; GT-P1000 Build/FROYO)";
const char S22876[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; GT-P1000T Build/FROYO)";
const char S22877[] = "Mozilla/5.0 (Linux; U;Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22878[] = "Logos/0.9.4 (Linux; U; Android 2.2; en_GB; GT-P1000 Build/FROYO.XWJJ7)";
const char S22879[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-P1000 Build/FROYO)";
const char S22880[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; GT-P1000L Build/FROYO)";
const char S22881[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; GT-P1000N Build/FROYO)";
const char S22882[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; GT-P1000M Build/FROYO)";
const char S22883[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S22884[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22885[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; GT-P1000R Build/FROYO)";
const char S22886[] = "Mozilla/5.0 (Linux; U; Android 2.2; fa-fa; GT-P1000 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S22887[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-P1000N Build/FROYO)";
const char S22888[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S22889[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S22890[] = "Mozilla/5.0 (Linux; U; Android 2.2; en2; SAMSUNG GT-P1000 Tablet Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22891[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22892[] = "Mozilla/5.0 (Linux; U; Android 2.21.0; en-gb; GT-P1000) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S22893[] = "Mozilla/5.0 (Linux U Android 2.2 es-es GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22894[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-P1000L Build/FROYO)";
const char S22895[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-; GT-P1000L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22896[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; SAMSUNG GT-P1000 Tablet Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22897[] = "Mozilla/5.0 (Linux; U; Android2.2; en-au; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22898[] = "Logos/0.9.1 (Linux; U; Android 2.2; en_GB; GT-P1000 Build/FROYO.JVJM5)";
const char S22899[] = "BBC News (AndroidApp; 1.0; 1) GT-P1000 (Android 2.2, SDK 8)";
const char S22900[] = "Mozilla/5.0 (Linux; U; Android 2.2; en_gb; GT-P1000 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)";
const char S22901[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-; GT-P1000T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22902[] = "Logos/0.9.3 (Linux; U; Android 2.2; en_GB; GT-P1000 Build/FROYO.DXJK2)";
const char S22903[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO; 600*1024) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S22904[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-P1000 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22905[] = "Mozilla/5.0 (Linux; U; Android 2.2; nl-nl; GT-P1000 Build/Android.Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22906[] = "BBC News (AndroidApp; 1.0; 1) GT-P1000M (Android 2.2, SDK 8)";
const char S22907[] = "Mozilla 5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build FROYO) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S22908[] = "Mozilla 5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build FROYO) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S22909[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-us; GT-P1000L Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S22910[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; SAMSUNG GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22911[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-P1000) AntivirusAM/2.5 (KHTML, like Gecko) Version/Version 2.5/Antivirus";
const char S22912[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO; 1024*600) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S22913[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 [FBAN/FB4A;FBAV/1.9.3;FBDM/{density=1.5,width=600,height=1024};FBLC/en_GB;FB_FW/2;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P1000;FBSV/2.2;]";
const char S22914[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/53uaprof/GT-P1000.xml, Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/53YS00NmI1LWE5ZjctZTQwY2EzYmYzOTQ00; ASP.NET_SessionId=aky0bl55ihlfv0aiiwp5ul45; __atuvc=3|20; language=en-GB; pay=9,-1,54,0,0; __utma=X.X.X.X.X.1; __utmz=X.X.1.1.utmcsr=google|utmccn=(organic)|utmcmd=organic|utmctr=r59%20truck%20spares; _em_hl=1; _em_vt=533b1d8000";
const char S22915[] = "BookShelf/XX-XXXXXXXXXXXX/Android 2.2/X-60:A1:0A:94:41:04/samsung GT-P1000";
const char S22916[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mob";
const char S22917[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-P1000N Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S22918[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-au; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile bKit/533.1 (KHTML, like Gecko) VMOUS=RgJeycPJzQEkAAAANjc4NDk5MmEtMzYwNi00YmM5LTkwM2ItZjBhOGRkYjBiZTM40; language=en-US; WT_FPC=id=1.42.30.87-X.X:lv=X:ss=X; __utma=X.X.X.X.X.2; __utmb=X.5.10.X; __utmc=X; __utmz=X.X.2.2.utmcsr=careersearch.colesgroup.com.au|u";
const char S22919[] = "Vyrso/2.1.2 (Linux; U; Android 2.2; en-GB; GT-P1000 Build/FROYO.JPJM5)";
const char S22920[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-au; GT-P1000T Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S22921[] = "Biblia/2.1.2 (Linux; U; Android 2.2; es-US; GT-P1000L Build/FROYO.UTJM3)";
const char S22922[] = "Vyrso/2.1.2 (Linux; U; Android 2.2; en-GB; GT-P1000 Build/FROYO.ZHJK1)";
const char S22923[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22924[] = "Biblia/2.1.2 (Linux; U; Android 2.2; es-US; GT-P1000L Build/FROYO.UMJK1)";
const char S22925[] = "Biblia/2.1.2 (Linux; U; Android 2.2; es-US; GT-P1000L Build/FROYO.UTJM2)";
const char S22926[] = "Biblia/2.1.2 (Linux; U; Android 2.2; es-US; GT-P1000L Build/FROYO.UMJJ3)";
const char S22927[] = "FaithlifeStudyBible/2.1.2 (Linux; U; Android 2.2; en-AU; GT-P1000 Build/FROYO.XWJJ4)";
const char S22928[] = "Logos/2.1.2 (Linux; U; Android 2.2; cs-CZ; GT-P1000 Build/FROYO.XWJMI)";
const char S22929[] = "Mozilla/5.0 (Linux; U; Android 2.2; pt-br; GT-P1000 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22930[] = "Biblia/2.2.3 (Linux; U; Android 2.2.1; es-US; GT-P1000N Build/FROYO.UBJM7)";
const char S22931[] = "Biblia/1.0.3 (Linux; U; Android 2.2; es-US; GT-P1000L Build/FROYO.UTJJ5)";
const char S22932[] = "Biblia/2.2.3 (Linux; U; Android 2.2; es-US; GT-P1000N Build/FROYO.VFJM2)";
const char S22933[] = "Biblia/2.2.3 (Linux; U; Android 2.2; es-US; GT-P1000N Build/FROYO.UBJJ5)";
const char S22934[] = "Mozilla/5.0 (Linux; U; Android 2.2; de-; GT-P1000R Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22935[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-P1000L Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22936[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22937[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22938[] = "Logos/2.2.3 (Linux; U; Android 2.2; en-CA; GT-P1000R Build/FROYO.UXJJ6)";
const char S22939[] = "GoogleAnalytics/2.0 (Linux; U; Android 2.2; en-ca; GT-P1000R Build/FROYO)";
const char S22940[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-P1000N Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22941[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-P1000N) antivirusHttpClient/2.0 (KHTML, like Gecko) Version/Version 2.3/Antivirus pro";
const char S22942[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-P1000M Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S22943[] = "com.google.android.youtube/2.4.4(Linux; U; Android 2.2; en_CA; GT-P1000M Build/FROYO) Experiment/1 gzip";
const char S22944[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) Hotels.com/3.0";
const char S22945[] = "Vyrso/2.2.3 (Linux; U; Android 2.2; en-GB; GT-P1000 Build/FROYO.JVJJ1)";
const char S22946[] = "MediaPlayer/2002/4.0.12 (Android 2.2; GT-P1000; GT-P1000; samsung; ; ; en_GB)";
const char S22947[] = "Rdio.Net; Android/2.2.8; Android/2.2; DM=samsung; DN=GT-P1000M; DC=samsung; DF=samsung/GT-P1000M/GT-P1000M/GT-P1000M:2.2/FROYO/UGJJ7:user/release-keys; DH=GT-P1000";
const char S22948[] = "com.google.android.apps.plus/X (Linux; U; Android 2.2; en_CA; GT-P1000M; Build/FROYO) (gzip)";
const char S22949[] = "Groupon/2.1.2483 (Android 2.2; Samsung GT-P1000R / Samsung GT-P1000R; )[preload=false;locale=en_CA;clientidbase=android-samsung]";
const char S22950[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SAMSUNG GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22951[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; SAMSUNG GT-P1000 Tablet Build/MASTER)";
const char S22952[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-ca; SAMSUNG GT-P1000M Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22953[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 [FBAN/FB4A;FBAV/1.6.4;FBPN/com.facebook.katana;FBDV/GT-P1000;FBSV/2.2;FBDM/{density=1.5,width=600,height=1024};FBCR/MY MAXIS;FBLC/en_GB;FB_FW/1;]";
const char S22954[] = "BBC News (AndroidApp; 1.3; 4) GT-P1000 (Android 2.2, SDK 8)";
const char S22955[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 [FBAN/FB4A;FBAV/1.9.7;FBDM/{density=1.5,width=600,height=1024};FBLC/en_GB;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P1000;FBSV/2.2;]";
const char S22956[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (innerActive-ANDROID-2061)";
const char S22957[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-ca; SAMSUNG GT-P1000R Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22958[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-P1000 Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S22959[] = "Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; GT-P1000Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22960[] = "Logos/2.2.8.1 (Linux; U; Android 2.2; en-GB; GT-P1000 Build/FROYO.XWJJ3)";
const char S22961[] = "FaithlifeStudyBible/2.2.8.1 (Linux; U; Android 2.2; en-GB; GT-P1000 Build/FROYO.DXKM2)";
const char S22962[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-P1000R) BBC News/1.3";
const char S22963[] = "BBC News (AndroidApp; 1.3; 4) GT-P1000R (Android 2.2, SDK 8)";
const char S22964[] = "Mozilla/5.0 (Linux; U; Android 2.2; ar-ae; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22965[] = "Logos/2.2.8.1 (Linux; U; Android 2.2.1; en-GB; GT-P1000 Build/FROYO.XWJMP)";
const char S22966[] = "Scanner Radio/3.9.1 (Android 2.2, GT-P1000M)";
const char S22967[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-au; SAMSUNG GT-P1000T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22968[] = "Dalvik 2.05.12114; (Linux; U; Android 2.2; GT-P1000R; ON) 9774d56d682e549c.KfLAGZ2j; 400x682x1.5";
const char S22969[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 baidubrowser/042_1.41.3.2_diordna_006_4201/gnusmas_8_1.2.2_0001P-TG/Xb/A4EXF7CAB4F1C11C4FF0%7CX/1";
const char S22970[] = "Mozilla/5.0 (Linux; U; Android 2.2; pt_br; GT-P1000L Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)";
const char S22971[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-P1000M) CBC News/1.0.5";
const char S22972[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.Xc0b415a654c29b6e84";
const char S22973[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.16255c1d48804bXf166";
const char S22974[] = "GoogleAnalytics/2.0 (Linux; U; Android 2.2; es-us; GT-P1000N Build/FROYO)";
const char S22975[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Versionie: ASP.NET_SessionId=1ogopqx44t0mmbuajquc3iha; __utmmobile=0x6BD61FDEXB5; __utma=X.X.X.X.X.2; __utmc=X; __utmz=X.X.1.1.utmcsr=(direct)|utmccn=(direct)|utmcmd=(none)";
const char S22976[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.177b4abXd4e8f030";
const char S22977[] = "Android 2.2 samsung GT-P1000 FROYO.DXJK2;600X1024";
const char S22978[] = "BBC News (AndroidApp; 1.01; 2) GT-P1000M (Android 2.2, SDK 8) flash:11.1.102.59";
const char S22979[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-au; GT-P1000T Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S22980[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-P1000T Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22981[] = "Scanner Radio/3.9.3 (Android 2.2, GT-P1000M)";
const char S22982[] = "OneBrowser/4.0.2/Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22983[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-P1000R Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S22984[] = "Mozilla/5.0 (Linux; U; Android 2.2; fr-fr;GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22985[] = "nApps (Android 2.2; GT-P1000; androidapp.lineplay; 1.2.1)";
const char S22986[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWt/533.1 (KHTML, like Gecko) VersVersion/4.0 Mobile Safari/533.1";
const char S22987[] = "Mozilla/5.0(Linux;U;Android2.2.1;Zh_cn;GT-P1000N;600*1024;)AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S22988[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-; GT-P1000M Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S22989[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-US; SAMSUNG GT-P1000 Tablet Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S22990[] = "APKXDL (Linux; U; Android 2.2;en_CA; GT-P1000M/FROYO)com.capcom.smurfsandroid";
const char S22991[] = "Mozilla/5.0 (compatible; bingbot/2.0 +http://www.bing.com/bingbot.htm)";
const char S22992[] = "Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm)";
const char S22993[] = "Mozilla/5.0 (compatible;bingbot/2.0;+http://www.bing.com/bingbot.htm)";
const char S22994[] = "Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: flyproxy0)";
const char S22995[] = "Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm),gzip(gfe) (via docs.google.com/viewer)";
const char S22996[] = "Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine CirruxCache CoralWebPrxMozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm),gzip(gfe) (via docs.google.com/viewer)";
const char S22997[] = "Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: proxydisk9)";
const char S22998[] = "Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: buttonses)";
const char S22999[] = "Mozilla/5.0 (seoanalyzer; compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm)";
const char S23000[] = "Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: proxyfile2)";
const char S23001[] = "Mozilla/5.0 (compatible; Bingbot/2.0;+http://www.bing.com/bingbot.htm)";
const char S23002[] = "Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) SitemapProbe";
const char S23003[] = "Mozilla/5.0 (compatible; bingbot/2.0  http://www.bing.com/bingbot.htm)";
const char S23004[] = "Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: scroll-down)";
const char S23005[] = "Mozilla/5.0 (Windows NT 6.1; rv:14.0) Gecko/X Firefox/14.0";
const char S23006[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:14.0) Gecko/X Firefox/14.0.1";
const char S23007[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:14.0) Gecko/X Firefox/14.0.1 Firefox/14.0.1";
const char S23008[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; WOW64; en-US; rv:2.0.4) Gecko/X AskTbAVR-IDW/3.12.5.17700 Firefox/14.0.1";
const char S23009[] = "Mozilla/5.0 (Windows; Windows NT 6.1; Win64; x64; rv:14.0) Gecko/X Firefox/14.0.1";
const char S23010[] = "magpie-crawler/1.1 (U; Linux amd64; en-GB; +http://www.brandwatch.net)";
const char S23011[] = "magpie-crawler/1.1 (U; Linux i386; en-GB; +http://www.brandwatch.net)";
const char S23012[] = "Mozilla/5.0 (compatible; Linux; Socialradarbot/2.0; en-US; bot@infegy.com)";
const char S23013[] = "yacybot (freeworld/global; amd64 Linux 2.6.32-5-amd64; java 1.6.0_18; Europe/de) http://yacy.net/bot.html";
const char S23014[] = "yacybot (freeworld/global; amd64 Linux 2.6.32-5-amd64; java 1.6.0_18; America/en) http://yacy.net/bot.html";
const char S23015[] = "yacybot (freeworld/global; i386 Linux 2.6.32-5-686; java 1.6.0_18; America/en) http://yacy.net/bot.html";
const char S23016[] = "yacybot (freeworld/global; amd64 Linux 2.6.32-5-amd64; java 1.6.0_18; Europe/en) http://yacy.net/bot.html";
const char S23017[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) eyeVerifyBot (Linux x86_64) Webkit/Apple Safari/1.5 Safari/534.34";
const char S23018[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.8) nxsCrawler-09 http://null32.de";
const char S23019[] = "yacybot (freeworld/global; amd64 Linux 2.6.32-5-amd64; java 1.6.0_26; Europe/en) http://yacy.net/bot.html";
const char S23020[] = "yacybot (webportal/global; amd64 Linux 2.6.32-5-amd64; java 1.6.0_18; Europe/en) http://yacy.net/bot.html";
const char S23021[] = "yacybot (freeworld/global; amd64 Linux 2.6.32-custom; java 1.6.0_26; Europe/en) http://yacy.net/bot.html";
const char S23022[] = "yacybot (freeworld/global; amd64 Linux 2.6.26-2-amd64; java 1.6.0_18; Europe/de) http://yacy.net/bot.html";
const char S23023[] = "yacybot (freeworld/global; i386 Linux 2.6.32-5-686; java 1.6.0_18; Europe/en) http://yacy.net/bot.html";
const char S23024[] = "yacybot (freeworld/global; i386 Linux 2.6.32-5-686; java 1.6.0_18; Europe/de) http://yacy.net/bot.html";
const char S23025[] = "yacybot (freeworld/global; i386 Linux 2.6.32-5-686; java 1.6.0_18; Europe/fr) http://yacy.net/bot.html";
const char S23026[] = "yacybot (freeworld-global; amd64 Linux 2.6.32-5-amd64; java 1.6.0_18; Europe/en) http://yacy.net/bot.html";
const char S23027[] = "yacybot (freeworld/global; amd64 Linux 2.6.32-5-openvz-amd64; java 1.6.0_18; Europe/en) http://yacy.net/bot.html";
const char S23028[] = "yacybot (webportal/global; amd64 Linux 2.6.26-2-amd64; java 1.6.0_18; Europe/en) http://yacy.net/bot.html";
const char S23029[] = "yacybot (freeworld/global; amd64 Linux 3.2.0-23-generic; java 1.6.0_24; Europe/en) http://yacy.net/bot.html";
const char S23030[] = "yacybot (freeworld/global; amd64 Linux 3.1.10-1.16-desktop; java 1.6.0_24; Europe/de) http://yacy.net/bot.html";
const char S23031[] = "yacybot (freeworld/global; amd64 Linux 2.6.26-2-amd64; java 1.6.0_18; Europe/en) http://yacy.net/bot.html";
const char S23032[] = "yacybot (freeworld/global; amd64 Linux 3.2.0-31-generic; java 1.6.0_24; Europe/en) http://yacy.net/bot.html";
const char S23033[] = "yacybot (freeworld/global; amd64 Linux 3.2.0-23-generic; java 1.6.0_24; Europe/de) http://yacy.net/bot.html";
const char S23034[] = "yacybot (freeworld/global; amd64 Linux 3.2.0-32-generic; java 1.6.0_24; Europe/en) http://yacy.net/bot.html";
const char S23035[] = "yacybot (freeworld/global; amd64 Linux 3.5.7-gentoo; java 1.7.0_09; Europe/de) http://yacy.net/bot.html";
const char S23036[] = "yacybot (freeworld/global; amd64 Linux 3.1.10-1.16-default; java 1.6.0_24; Europe/de) http://yacy.net/bot.html";
const char S23037[] = "yacybot (freeworld/global; amd64 Linux 3.4.11-2.16-desktop; java 1.7.0_09; Europe/nl) http://yacy.net/bot.html";
const char S23038[] = "yacybot (freeworld/global; i386 Linux 3.1.10-slacko_4gA; java 1.7.0; Europe/de) http://yacy.net/bot.html";
const char S23039[] = "yacybot (freeworld-global; amd64 Linux 3.2.0-4-amd64; java 1.6.0_24; Europe/en) http://yacy.net/bot.html";
const char S23040[] = "yacybot (torworld/any; amd64 Linux 3.4.0-5-generic; java 1.7.0_09; Europe/en) http://yacy.net/bot.html";
const char S23041[] = "yacybot (freeworld/global; amd64 Linux 3.6.11-gentoo; java 1.7.0_09; Europe/de) http://yacy.net/bot.html";
const char S23042[] = "yacybot (freeworld/global; amd64 Linux 3.2.35-minimal; java 1.7.0_10; Europe/en) http://yacy.net/bot.html";
const char S23043[] = "yacybot (freeworld-global; amd64 Linux 3.2.0-35-generic; java 1.7.0_09; Europe/en) http://yacy.net/bot.html";
const char S23044[] = "yacybot (freeworld/global; amd64 Linux 3.2.0-4-amd64; java 1.6.0_24; Europe/en) http://yacy.net/bot.html";
const char S23045[] = "yacybot (freeworld/global; amd64 Linux 3.4.11-2.16-desktop; java 1.7.0_09; Europe/de) http://yacy.net/bot.html";
const char S23046[] = "Mozilla/5.0 (Linux 3.0.0-13-virtual x86_64) NettRapport Crawler (post@nettrapport.no)";
const char S23047[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) RMSS_WEBKIT_CRAWLER Safari/534.34";
const char S23048[] = "yacybot (freeworld/global; amd64 Linux 3.2.0-4-amd64; java 1.7.0_03; Europe/en) http://yacy.net/bot.html";
const char S23049[] = "yacybot (freeworld/global; amd64 Linux 3.1.10-1.16-default; java 1.6.0_27; Europe/de) http://yacy.net/bot.html";
const char S23050[] = "yacybot (freeworld/global; amd64 Linux 3.7.10-gentoo; java 1.7.0_17; Europe/de) http://yacy.net/bot.html";
const char S23051[] = "Mozilla/5.0 (compatible; Linux i686; Yandex.Gazeta Bot/1.0; +http://gazeta.yandex.ru)";
const char S23052[] = "yacybot (webportal-global; amd64 Linux 2.6.32-5-amd64; java 1.6.0_45; Europe/en) http://yacy.net/bot.html";
const char S23053[] = "yacybot (freeworld/global; amd64 Linux 3.7.10-gentoo-r1; java 1.7.0_17; Europe/de) http://yacy.net/bot.html";
const char S23054[] = "yacybot (freeworld/global; amd64 Linux 3.2.0-4-amd64; java 1.6.0_27; Europe/en) http://yacy.net/bot.html";
const char S23055[] = "Mozilla/5.0 (compatible; Linux x86_64; Mail.RU_Bot/2.0; +http://go.mail.ru/help/robots)";
const char S23056[] = "yacybot (freeworld/global; amd64 Linux 3.1.10-1.19-desktop; java 1.6.0_27; Europe/de) http://yacy.net/bot.html";
const char S23057[] = "yacybot (freeworld/global; amd64 Linux 3.8.0-22-generic; java 1.6.0_27; Pacific/en) http://yacy.net/bot.html";
const char S23058[] = "yacybot (freeworld/global; amd64 Linux 3.8.13-gentoo; java 1.7.0_17; Europe/de) http://yacy.net/bot.html";
const char S23059[] = "yacybot (freeworld/global; amd64 Linux 3.8.13-gentoo; java 1.7.0_21; Europe/de) http://yacy.net/bot.html";
const char S23060[] = "yacybot (freeworld/global; amd64 Linux 3.5.0-23-generic; java 1.6.0_27; America/en) http://yacy.net/bot.html";
const char S23061[] = "yacybot (freeworld/global; amd64 Linux 3.8.0-23-generic; java 1.6.0_27; Pacific/en) http://yacy.net/bot.html";
const char S23062[] = "yacybot (freeworld/global; amd64 Linux 3.7.9-gentoo; java 1.7.0_21; Europe/pl) http://yacy.net/bot.html";
const char S23063[] = "yacybot (freeworld/global; amd64 Linux 2.6.32-47-server; java 1.6.0_26; Europe/de) http://yacy.net/bot.html";
const char S23064[] = "yacybot (freeworld/global; amd64 Linux 3.2.0-4-amd64; java 1.6.0_27; Europe/de) http://yacy.net/bot.html";
const char S23065[] = "yacybot (freeworld/global; amd64 Linux 3.1.10-1.29-default; java 1.6.0_27; Europe/de) http://yacy.net/bot.html";
const char S23066[] = "yacybot (freeworld/global; amd64 Linux 3.7.10-gentoo-r1; java 1.7.0_21; Europe/pl) http://yacy.net/bot.html";
const char S23067[] = "yacybot (freeworld/global; amd64 Linux 3.8.0-25-generic; java 1.7.0_21; Europe/de) http://yacy.net/bot.html";
const char S23068[] = "Mozilla/5.0 (compatible; Linux i686; en-US; URLfilterDB-crawler/1.0) ufdb/1.0";
const char S23069[] = "WIRE/0.21 (Linux; x86_64; Bot,Robot,Spider,Crawler)";
const char S23070[] = "yacybot (freeworld/global; amd64 Linux 3.2.0-4-amd64; java 1.7.0_03; US/en) http://yacy.net/bot.html";
const char S23071[] = "yacybot (freeworld/global; amd64 Linux 2.6.32-49-server; java 1.6.0_27; Europe/en) http://yacy.net/bot.html";
const char S23072[] = "yacybot (freeworld/global; amd64 Linux 3.2.0-4-amd64; java 1.7.0_03; Etc/de) http://yacy.net/bot.html";
const char S23073[] = "yacybot (freeworld/global; amd64 Linux 3.9.7; java 1.7.0_25; Europe/en) http://yacy.net/bot.html";
const char S23074[] = "yacybot (freeworld/global; amd64 Linux 3.2.0-4-amd64; java 1.7.0_25; US/en) http://yacy.net/bot.html";
const char S23075[] = "yacybot (freeworld/global; amd64 Linux 2.6.32-5-amd64; java 1.7.0_21; Europe/en) http://yacy.net/bot.html";
const char S23076[] = "yacybot (freeworld-global; amd64 Linux 3.2.0-4-amd64; java 1.6.0_27; Europe/de) http://yacy.net/bot.html";
const char S23077[] = "yacybot (freeworld/global; i386 Linux 3.2.0-4-686-pae; java 1.6.0_27; Europe/de) http://yacy.net/bot.html";
const char S23078[] = "yacybot (freeworld/global; amd64 Linux 3.5.0-23-generic; java 1.6.0_27; US/en) http://yacy.net/bot.html";
const char S23079[] = "yacybot (freeworld/global; amd64 Linux 3.8.13-gentoo; java 1.7.0_40; Europe/de) http://yacy.net/bot.html";
const char S23080[] = "yacybot (freeworld/global; amd64 Linux 3.8.0-27-generic; java 1.7.0_25; Europe/en) http://yacy.net/bot.html";
const char S23081[] = "yacybot (freeworld/global; amd64 Linux 3.10.1-pf; java 1.8.0-ea; America/en) http://yacy.net/bot.html";
const char S23082[] = "yacybot (freeworld/global; amd64 Linux 3.5.0-27-generic; java 1.7.0_25; Europe/en) http://yacy.net/bot.html";
const char S23083[] = "yacybot (freeworld/global; i386 Linux 3.2.0-4-686-pae; java 1.6.0_27; Europe/en) http://yacy.net/bot.html";
const char S23084[] = "yacybot (freeworld/global; amd64 Linux 3.2.0-4-amd64; java 1.7.0_25; Europe/en) http://yacy.net/bot.html";
const char S23085[] = "yacybot (amd64 Linux 2.6.31-19-server; java 1.6.0_0; Europe/en) http://yacy.net/bot.html;c2VvLWNyYXdsZXI=";
const char S23086[] = "yacybot (amd64 Linux 2.6.26-2-amd64; java 1.6.0_12; Zulu/en) http://yacy.net/bot.html;c2VvLWNyYXdsZXI=";
const char S23087[] = "yacybot (freeworld/global; amd64 Linux 2.6.32-50-server; java 1.6.0_26; Europe/de) http://yacy.net/bot.html";
const char S23088[] = "yacybot (freeworld/global; amd64 Linux 3.10.7-gentoo; java 1.7.0_40; Europe/de) http://yacy.net/bot.html";
const char S23089[] = "yacybot (superfi/global; amd64 Linux 2.6.32-042stab079.5; java 1.6.0_27; Etc/en) http://yacy.net/bot.html";
const char S23090[] = "yacybot (freeworld/global; i386 Linux 3.2.41-042stab078.27; java 1.6.0_27; Europe/de) http://yacy.net/bot.html";
const char S23091[] = "yacybot (freeworld/global; amd64 Linux 2.6.32-028stab092.1; java 1.6.0_27; Europe/en) http://yacy.net/bot.html";
const char S23092[] = "yacybot (amd64 Linux 2.6.28-18-generic; java 1.6.0_19; GMT/en) http://yacy.net/bot.html;c2VvLWNyYXdsZXI=";
const char S23093[] = "yacybot (webportal-global; amd64 Linux 3.2.0-4-amd64; java 1.7.0_03; Europe/en) http://yacy.net/bot.html";
const char S23094[] = "yacybot (freeworld/global; amd64 Linux 3.7.10-1.16-desktop; java 1.7.0_40; Europe/de) http://yacy.net/bot.html";
const char S23095[] = "yacybot (superfi/global; amd64 Linux 2.6.32-042stab079.6; java 1.6.0_27; Etc/en) http://yacy.net/bot.html";
const char S23096[] = "yacybot (webportal-global; amd64 Linux 3.2.0-4-amd64; java 1.7.0_25; Europe/en) http://yacy.net/bot.html";
const char S23097[] = "yacybot (webportal-global; amd64 Linux 3.2.0-4-amd64; java 1.6.0_27; Europe/en) http://yacy.net/bot.html";
const char S23098[] = "yacybot (freeworld/global; amd64 Linux 3.2.0-4-amd64; java 1.7.0_25; Etc/de) http://yacy.net/bot.html";
const char S23099[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:15.0) Gecko/15.0 Firefox/15.0a1";
const char S23100[] = "Mozilla/5.0 (Windows NT 6.1; rv:15.0) Gecko/X Firefox/15.0a2";
const char S23101[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X Firefox/15.0.1 Firefox/15.0.1";
const char S23102[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:15.0) Gecko/X Firefox/15.0 Firefox/15.0";
const char S23103[] = "Mozilla/5.0 (Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X Firefox/15.0.1";
const char S23104[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; rv:15.0) Gecko/X Firefox/15.0.1";
const char S23105[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23106[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/NOCTURNAL VENUM Rom) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23107[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SPH-D710 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23108[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23109[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SPH-D710 Build/IML74K)";
const char S23110[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SPH-D710 Build/IMM76I)";
const char S23111[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23112[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SPH-D710 Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23113[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/SPH.D710.SNROM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23114[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/HyDrA_Rom_v0.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23115[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23116[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/Slim-n-Trim) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23117[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23118[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710BST Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23119[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SPH-D710BST Build/IMM76I)";
const char S23120[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S23121[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710VMUB Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23122[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SPH-D710 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23123[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 androidAppVersion/2.5.16 AndroidWebkitWrapper BillingSupported";
const char S23124[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SPH-D710VMUB Build/IMM76I)";
const char S23125[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SPH-D710 Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S23126[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SPH-D710 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 ??Safari/534.30";
const char S23127[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.0.4;dev=SPH-D710;devm=SPH-D710;man=samsung;brw=Android;shwd=480x800;shwa=480x763;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S23128[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176";
const char S23129[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 5_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/5.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S23130[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)";
const char S23131[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 5_1_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/5.1.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S23132[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206";
const char S23133[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 iPod4,1 BingWeb/3.03.1428.X";
const char S23134[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)";
const char S23135[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 iPod3,1 BingWeb/3.03.1428.X";
const char S23136[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper";
const char S23137[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/5.1.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]";
const char S23138[] = "Mozilla/5.0 (iPodTouch4G; iPhone; U; CPU OS 5_1_1 like Mac OS X; en_US) com.google.GooglePlus/8654 (KHTML, like Gecko) Mobile/N81AP (gzip)";
const char S23139[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/5.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]";
const char S23140[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405";
const char S23141[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper";
const char S23142[] = "Mozilla/5.0 (iPhone; U; CPU iPhone 5_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPod3,1/5.1.1)";
const char S23143[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S23144[] = "Zeusmos 2.6.9 (iPod touch; iPhone OS 5.1.1; en_US)";
const char S23145[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Sprint APA9292KT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23146[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PC36100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23147[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Sprint APA9292KT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23148[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; -us; Sprint APA9292KT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23149[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; PC36100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23150[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Sprint APA9292KT Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23151[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; PC36100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23152[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PC36100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23153[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; PC36100 Build/GRI40)";
const char S23154[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PC36100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23155[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23156[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PC36100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23157[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; PC36100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23158[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PC36100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23159[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; PC36100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23160[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/GRJ90) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S23161[] = "Logos/0.9.4 (Linux; U; Android 2.3.5; en_US; PC36100 Build/4.67.651.3)";
const char S23162[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Sprint APA9292KT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23163[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Sprint APA9292KT Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23164[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23165[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; PC36100 Build/GRJ90)";
const char S23166[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; PC36100 Build/GRI40)";
const char S23167[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23168[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; PC36100 Build/GRJ22)";
const char S23169[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; Sprint APA9292KT Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23170[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; -us; Sprint APA9292KT Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23171[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PC36100 Build/GRI40) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S23172[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; PC36100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23173[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; EVO 4G Plus Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23174[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; EVO Build/GRI40; CIUI-HTC) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23175[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; PC36100 Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S23176[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en_us; PC36100 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)";
const char S23177[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; PC36100 Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S23178[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; PC36100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23179[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/GRJ90) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30";
const char S23180[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; PC36100 Build/thundershock; ThunderShock v.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23181[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PC36100-HTC EVO Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23182[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PC36100 Build/Sense and AOSP; SensuAOSP 2.55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23183[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ar-eg; HTC EVO 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23184[] = "Mozilla/5.0 (Linux; U; Android Gingerbread 2.3.7; en-us; PC36100 Build/GRI40; Unity 0.0.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23185[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ar-eg; EVO 4G Build/GRI40; Dream Rise_EVO4G_2.3.7_X_ANZHI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23186[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; EVO 4G Build/unknown) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S23187[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.10";
const char S23188[] = "Mozilla/5.0 (Windows NT 5.1; rv:2.0) Gecko/X Firefox/4.0 Opera 12.10";
const char S23189[] = "Opera/9.80 (Windows NT 5.2) Presto/2.12.388 Version/12.10";
const char S23190[] = "Opera/9.80 (Windows NT 5.1; Edition Yx) Presto/2.12.388 Version/12.10";
const char S23191[] = "Opera/9.80 (Windows NT 5.2; WOW64) Presto/2.12.388 Version/12.10";
const char S23192[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1) Opera 12.10";
const char S23193[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.10";
const char S23194[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.10";
const char S23195[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.10";
const char S23196[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.10";
const char S23197[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.10";
const char S23198[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.10";
const char S23199[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.10";
const char S23200[] = "Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.10";
const char S23201[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.10";
const char S23202[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.10";
const char S23203[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.10";
const char S23204[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.10";
const char S23205[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.10";
const char S23206[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.10";
const char S23207[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.10,gzip(gfe)";
const char S23208[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.10";
const char S23209[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.10";
const char S23210[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.10";
const char S23211[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.10";
const char S23212[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.10";
const char S23213[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.10";
const char S23214[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.10";
const char S23215[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.10";
const char S23216[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.10";
const char S23217[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.10";
const char S23218[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.10";
const char S23219[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.10";
const char S23220[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.10";
const char S23221[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.10";
const char S23222[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.10";
const char S23223[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03789)) Presto/2.12.388 Version/12.10";
const char S23224[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.10";
const char S23225[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.10 Yandex.Translate";
const char S23226[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.10";
const char S23227[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.10";
const char S23228[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.10";
const char S23229[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.10";
const char S23230[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.10";
const char S23231[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.10";
const char S23232[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.10";
const char S23233[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.10";
const char S23234[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.10";
const char S23235[] = "Opera/9.80 (Windows NT 5.1; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.10";
const char S23236[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.10";
const char S23237[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.10";
const char S23238[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.10";
const char S23239[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.10";
const char S23240[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.10";
const char S23241[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.10";
const char S23242[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.10";
const char S23243[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.10";
const char S23244[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4933)) Presto/2.12.388 Version/12.10";
const char S23245[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.10";
const char S23246[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.10";
const char S23247[] = "Opera/9.80 (Windows NT 5.1; Edition Campaign 21) Presto/2.12.388 Version/12.10,gzip(gfe)";
const char S23248[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5230)) Presto/2.12.388 Version/12.10";
const char S23249[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.10";
const char S23250[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.10";
const char S23251[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03658)) Presto/2.12.388 Version/12.10";
const char S23252[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5288)) Presto/2.12.388 Version/12.10";
const char S23253[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.10";
const char S23254[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.10";
const char S23255[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5650)) Presto/2.12.388 Version/12.10";
const char S23256[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.10";
const char S23257[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.10";
const char S23258[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.10";
const char S23259[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.10";
const char S23260[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.10";
const char S23261[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.10";
const char S23262[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4110)) Presto/2.12.388 Version/12.10";
const char S23263[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.10";
const char S23264[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5278)) Presto/2.12.388 Version/12.10";
const char S23265[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.10";
const char S23266[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.10";
const char S23267[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03402)) Presto/2.12.388 Version/12.10";
const char S23268[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.10";
const char S23269[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03639)) Presto/2.12.388 Version/12.10";
const char S23270[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.10";
const char S23271[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.10";
const char S23272[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5660)) Presto/2.12.388 Version/12.10";
const char S23273[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4832)) Presto/2.12.388 Version/12.10";
const char S23274[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03402); MRSPUTNIK OW 2, 3, 0, 104) Presto/2.12.388 Version/12.10";
const char S23275[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.10";
const char S23276[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.10";
const char S23277[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.10";
const char S23278[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.10";
const char S23279[] = "Mozilla/5.0 (Windows; U; MSIE 9.0; WIndows NT 9.0; en-US))";
const char S23280[] = "Mozilla/5.0 (Windows; U; MSIE 9.0; Windows NT 9.0; en-US)";
const char S23281[] = "Mozilla/5.0 (compatible; MSIE 9.0; Win32; Trident/5.0)";
const char S23282[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 7.1; Trident/5.0)";
const char S23283[] = "Mozilla/5.0 (compatible; MSIE 9.0; Win64; Trident/5.0)";
const char S23284[] = "Mozilla/5.0 (Windows; U; MSIE 9.0; Windows NT 9.0)";
const char S23285[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 9.0; en-US)";
const char S23286[] = "Mozilla/5.0 (Windows; U; MSIE 9.0; Windows NT 9.0; en-US)Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 7.1; Trident/5.0)";
const char S23287[] = "Mozilla/4.0 (compatible; U; MSIE 9.0; WIndows NT 9.0; en-US)";
const char S23288[] = "Mozilla/5.0 (Windows; U; MSIE 9.0; WIndows NT 9.0; de-DE))";
const char S23289[] = "Mozilla/5.0 (compatible; MSIE 9.0; 68K; Trident/3.0)";
const char S23290[] = "Mozilla/5.0 (Windows; U; MSIE 9.0; WIndows NT 9.0; en-GB)";
const char S23291[] = "Mozilla/5.0 (Windows; U; MSIE 9.0; WIndows NT 9.0; de-DE)";
const char S23292[] = "Mozilla/5.0 (Windows; U; MSIE 9.0; WIndows NT 9.0; en-CA)";
const char S23293[] = "Mozilla/5.0 (compatible; MSIE 9.0; 68K; WOW64; Trident/3.0)";
const char S23294[] = "Mozilla/5.0 (MSIE 9.0; Windows NT 9.0; en-US)";
const char S23295[] = "Mozilla/5.0 (Windows; U; MSIE 9.0; WIndows NT 9.0; zh-cn)";
const char S23296[] = "Mozilla/5.0 (compatible; MSIE 9.0; PPC; Win64; x64; Trident/1.0)";
const char S23297[] = "Mozilla/5.0 (compatible; MSIE 9.0; 68K; SV1; Trident/5.0)";
const char S23298[] = "Mozilla/5.0 (compatible; MSIE 9.0; PPC; WOW64; Trident/3.0)";
const char S23299[] = "Mozilla/5.0 (compatible; MSIE 9.0; PPC; .NET CLR; Trident/5.0)";
const char S23300[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A400 Safari/6531.22.7";
const char S23301[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A306 Safari/6531.22.7";
const char S23302[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.4 Mobile/8A260b Safari/531.21.10";
const char S23303[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293 Safari/6531.22.7";
const char S23304[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A400a Safari/6531.22.7";
const char S23305[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293";
const char S23306[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/9Z999 Safari/6531.22.7";
const char S23307[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/9Z999 Safari/6531.22.7";
const char S23308[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/8A293 Safari/528.16";
const char S23309[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293 Safari/531.22.7";
const char S23310[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293 Safari/6531.22.7";
const char S23311[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.05 Mobile/8A293 Safari/6531.22.7";
const char S23312[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/9Z999 Safari/6531.22.7";
const char S23313[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S23314[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S23315[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S23316[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/525.13 (KHTML, like Gecko; Google Web Preview) Version/4.0.5 Mobile Safari/525.13";
const char S23317[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X: en-us) AppleWebKit/532.9  (KHTML, like Gecko) Version/4.0 Mobile/8A293 Safari/6531.22.7";
const char S23318[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; en-us) AppleWebKit/536.8 (KHTML, like Gecko; Google Page Speed Insights) Version/4.0.5 Mobile/8A306 Safari/6531.22.7";
const char S23319[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; en-us) AppleWebKit/537.4 (KHTML, like Gecko; Google Page Speed Insights) Version/4.0.5 Mobile/8A306 Safari/6531.22.7";
const char S23320[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 40 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML like Gecko) Version/4.0.5 Mobile/8A293 Safari/6531.22.7";
const char S23321[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 401 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML like Gecko) Version/4.0.5 Mobile/8A306 Safari/6531.22.7";
const char S23322[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sony Tablet S Build/TISU0077) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23323[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Sony Tablet S Build/TISU0084) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23324[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Sony Tablet S Build/TISU0093) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23325[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Sony Tablet S Build/TISU0R0110) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23326[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sony Tablet S Build/TISU0077) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23327[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Sony Tablet S Build/TISU0085) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23328[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sony Tablet S Build/TISU0124) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23329[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; Turbo-X Tablet Spice III Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23330[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sony Tablet S Build/TISU0143) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23331[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Sony Tablet S Build/TISU0143)";
const char S23332[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Tablet Scooby Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23333[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sony Tablet S Build/TISU0143) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23334[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; Sony Tablet S Build/TISU0143) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23335[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B146 Safari/8536.25";
const char S23336[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B144 Safari/8536.25";
const char S23337[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B142 Safari/8536.25";
const char S23338[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B143 Safari/8536.25";
const char S23339[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B145 Safari/8536.25";
const char S23340[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B141 Safari/8536.25";
const char S23341[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B329 Safari/8536.25";
const char S23342[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B318 Safari/8536.25";
const char S23343[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B350 Safari/8536.25";
const char S23344[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B5126b Safari/8536.25";
const char S23345[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B5105c Safari/8536.25";
const char S23346[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S23347[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B144 Safari/8536.25";
const char S23348[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146 Safari/8536.25";
const char S23349[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329 Safari/8536.25";
const char S23350[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329 Safari/8536.25";
const char S23351[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.125 Safari/533.4";
const char S23352[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.125 Safari/533.4,gzip(gfe) (via translate.google.com)";
const char S23353[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-us) AppleWebKit/533.2 (KHTML, like Gecko) Chrome/5.0.342.7 Safari/533.2";
const char S23354[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/533 (KHTML, like Gecko) Chrome/5.0.375 Safari/533";
const char S23355[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.86 Safari/533.4,gzip(gfe)";
const char S23356[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; AppleWebKit/533.3; KHTML?? like Gecko; Chrome/5.0.354.0 Safari/533.3)";
const char S23357[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T989 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23358[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SAMSUNG-SGH-T989 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23359[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T989 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23360[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T989 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23361[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SGH-T989 Build/IML74K)";
const char S23362[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T989 Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23363[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T989 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23364[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T989 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23365[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-T989 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23366[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; SGH-T989D Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23367[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SGH-T989D Build/IML74K)";
const char S23368[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-T989D-parrot Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23369[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-T989D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23370[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T989 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23371[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SAMSUNG-SGH-T989 Build/IMM76L)";
const char S23372[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T989 Build/5.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23373[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T989 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S23374[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-T989 Build/IMM76D)";
const char S23375[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-T989D Build/IMM76D)";
const char S23376[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SGH-T989 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23377[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; SGH-T989 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23378[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SGH-T989D-parrot Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23379[] = "Mozilla/5.0 (X11; CrOS x86_64 4191.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1520.2 Safari/537.36";
const char S23380[] = "Mozilla/5.0 (X11; CrOS i686 4191.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1520.2 Safari/537.36";
const char S23381[] = "Mozilla/5.0 (X11; CrOS armv7l 4191.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1520.2 Safari/537.36";
const char S23382[] = "Mozilla/5.0 (X11; CrOS i686 4287.1.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1541.3 Safari/537.36";
const char S23383[] = "Mozilla/5.0 (X11; CrOS x86_64 4287.1.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1541.3 Safari/537.36";
const char S23384[] = "Mozilla/5.0 (X11; CrOS i686 4287.2.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1541.3 Safari/537.36";
const char S23385[] = "Mozilla/5.0 (X11; CrOS armv7l 4287.1.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1541.3 Safari/537.36";
const char S23386[] = "Mozilla/5.0 (X11; CrOS i686 4319.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.2 Safari/537.36";
const char S23387[] = "Mozilla/5.0 (X11; CrOS x86_64 4319.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.2 Safari/537.36";
const char S23388[] = "Mozilla/5.0 (X11; CrOS armv7l 4319.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.2 Safari/537.36";
const char S23389[] = "Mozilla/5.0 (X11; CrOS x86_64 4319.3.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.2 Safari/537.36";
const char S23390[] = "Mozilla/5.0 (X11; CrOS i686 4319.3.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.2 Safari/537.36";
const char S23391[] = "Mozilla/5.0 (X11; CrOS i686 4319.16.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.16 Safari/537.36";
const char S23392[] = "Mozilla/5.0 (X11; CrOS x86_64 4319.16.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.16 Safari/537.36";
const char S23393[] = "Mozilla/5.0 (X11; CrOS armv7l 4319.16.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.16 Safari/537.36";
const char S23394[] = "Mozilla/5.0 (X11; CrOS armv7l 4319.28.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.22 Safari/537.36";
const char S23395[] = "Mozilla/5.0 (X11; CrOS i686 4319.28.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.22 Safari/537.36";
const char S23396[] = "Mozilla/5.0 (X11; CrOS x86_64 4319.28.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.22 Safari/537.36";
const char S23397[] = "Mozilla/5.0 (X11; CrOS i686 4319.32.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.23 Safari/537.36";
const char S23398[] = "Mozilla/5.0 (X11; CrOS x86_64 4319.32.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.23 Safari/537.36";
const char S23399[] = "Mozilla/5.0 (X11; CrOS armv7l 4319.32.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.23 Safari/537.36";
const char S23400[] = "Mozilla/5.0 (X11; CrOS x86_64 4319.39.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.32 Safari/537.36";
const char S23401[] = "Mozilla/5.0 (X11; CrOS armv7l 4319.39.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.32 Safari/537.36";
const char S23402[] = "Mozilla/5.0 (X11; CrOS i686 4319.39.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.32 Safari/537.36";
const char S23403[] = "Mozilla/5.0 (X11; CrOS armv7l 4319.52.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.41 Safari/537.36";
const char S23404[] = "Mozilla/5.0 (X11; CrOS i686 4319.52.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.41 Safari/537.36";
const char S23405[] = "Mozilla/5.0 (X11; CrOS x86_64 4319.52.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.41 Safari/537.36";
const char S23406[] = "Mozilla/5.0 (X11; CrOS armv7l 4319.74.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36";
const char S23407[] = "Mozilla/5.0 (X11; CrOS x86_64 4319.74.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36";
const char S23408[] = "Mozilla/5.0 (X11; CrOS i686 4319.74.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36";
const char S23409[] = "Mozilla/5.0 (X11; CrOS armv7l 4319.59.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.49 Safari/537.36";
const char S23410[] = "Mozilla/5.0 (X11; CrOS x86_64 4319.59.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.49 Safari/537.36";
const char S23411[] = "Mozilla/5.0 (X11; CrOS i686 4319.59.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.49 Safari/537.36";
const char S23412[] = "Mozilla/5.0 (X11; CrOS armv7l 4319.65.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.56 Safari/537.36";
const char S23413[] = "Mozilla/5.0 (X11; CrOS x86_64 4319.65.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.56 Safari/537.36";
const char S23414[] = "Mozilla/5.0 (X11; CrOS i686 4319.65.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.56 Safari/537.36";
const char S23415[] = "Mozilla/5.0 (X11; CrOS armv7l 4319.79.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.70 Safari/537.36";
const char S23416[] = "Mozilla/5.0 (X11; CrOS i686 4319.79.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.70 Safari/537.36";
const char S23417[] = "Mozilla/5.0 (X11; CrOS x86_64 4319.79.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.70 Safari/537.36";
const char S23418[] = "Mozilla/5.0 (X11; CrOS armv7l 4319.96.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.74 Safari/537.36";
const char S23419[] = "Mozilla/5.0 (X11; CrOS i686 4319.96.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.74 Safari/537.36";
const char S23420[] = "Mozilla/5.0 (X11; CrOS x86_64 4319.96.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.74 Safari/537.36";
const char S23421[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500R Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S23422[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500D Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S23423[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; SAMSUNG-GT-P7500/P7500BUKG7 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S23424[] = "Mozilla/5.0 (Linux; U; Android 3.1; de-de; SAMSUNG-GT-P7500/P7500BUKG5 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S23425[] = "Mozilla/5.0 (Linux; U; Android 3.1; de-de; SAMSUNG-GT-P7500/P7500BUKH1 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S23426[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S23427[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500M Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S23428[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; SAMSUNG-GT-P7500/P7500BUKI1 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S23429[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; GT-P7500 Build/HMJ37)";
const char S23430[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S23431[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; GT-P7500R Build/HMJ37)";
const char S23432[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; GT-P7500M Build/HMJ37)";
const char S23433[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37; 1280*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S23434[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S23435[] = "Mozilla/5.0 (Linux; U; Android 3.1; cs-cz; SAMSUNG-GT-P7500/P7500BUKEE Build/HONEYCOMB) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S23436[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-; GT-P7500R Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S23437[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; GT-P7500D Build/HMJ37)";
const char S23438[] = "Mozilla/5.0 (Linux; U; Android 3.1; el-gr; SAMSUNG-GT-P7500/P7500BUKH4 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S23439[] = "Mozilla/5.0 (Linux; U; Android 3.1; se-se; GT-P7500 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23440[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S23441[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S23442[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S23443[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500) antivirusAM/2.4.5 (KHTML, like Gecko) Version/Version 2.4.5/antivirus";
const char S23444[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S23445[] = "Android/3.9.1 (Linux; U; Android 3.1; en_CA; GT-P7500R)";
const char S23446[] = "Logos/2.0.11 (Linux; U; Android 3.1; en-GB; GT-P7500 Build/HMJ37)";
const char S23447[] = "BBC News (AndroidApp; 1.0; 1) GT-P7500 (Android 3.1, SDK 12)";
const char S23448[] = "Logos/2.1.2 (Linux; U; Android 3.1; es-ES; GT-P7500 Build/HMJ37.UBKG9)";
const char S23449[] = "Mozilla/5.0 (Linux; U; Android 3.1; nl-be; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 TwonkyBeamBrowser/3.2.0-57 (Android 3.1; samsung GT-P7500 Build/HMJ37)";
const char S23450[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; SAMSUNG-GT-P7500/P7500BUKEG Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S23451[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23452[] = "Mozilla/5.0 (Linux; U; Android 3.1; sv-se; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version /4.0 Safari/534.13 AndroidApp";
const char S23453[] = "Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 3.1; SAMSUNG GT-P7500 Build/HONEYCOMB_MR1);";
const char S23454[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S23455[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500 Build/HMJ37 like Xoom) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S23456[] = "GoogleAnalytics/2.0 (Linux; U; Android 3.1; en-ca; GT-P7500R Build/HMJ37)";
const char S23457[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500M) BBC News/1.0";
const char S23458[] = "BBC News (AndroidApp; 1.0; 5) GT-P7500M (Android 3.1, SDK 12)";
const char S23459[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.6.3;FBPN/com.facebook.katana;FBDV/GT-P7500;FBSV/3.1;FBDM/{density=1.0,width=1280,height=800};FBCR/MY MAXIS;FBLC/en_US;FB_FW/1;]";
const char S23460[] = "MediaPlayer/2002/4.0.12 (Android 3.1; GT-P7500; GT-P7500; samsung; ; ; en_GB)";
const char S23461[] = "Logos/2.1.2 (Linux; U; Android 3.1; en-US; GT-P7500 Build/HMJ37.ZSKH2)";
const char S23462[] = "MediaPlayer/2002/4.0.12 (Android 3.1; GT-P7500; GT-P7500; samsung; ; ; ms_MY)";
const char S23463[] = "Logos/2.0.11 (Linux; U; Android 3.1; pt-BR; GT-P7500 Build/HMJ37.UBKH1)";
const char S23464[] = "Logos/2.2.3 (Linux; U; Android 3.1; en-US; GT-P7500 Build/HMJ37.UBKG6)";
const char S23465[] = "Groupon/2.1.2483 (Android 3.1; Samsung GT-P7500 / Samsung GT-P7500; )[preload=false;locale=en_US;clientidbase=android-samsung]";
const char S23466[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23467[] = "Logos/2.2.3 (Linux; U; Android 3.1; en-CA; GT-P7500R Build/HMJ37.UXKH4)";
const char S23468[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S23469[] = "com.google.android.apps.plus/X (Linux; U; Android 3.1; en_CA; GT-P7500R; Build/HMJ37) (gzip)";
const char S23470[] = "VKAndroidApp/2.7.5-60 (Android 3.1; SDK 12; armeabi-v7a; samsung GT-P7500; uk)";
const char S23471[] = "Mozilla/5.0 (Linux; U; Android 3.1; hr-hr; SAMSUNG-GT-P7500/P7500BUKG2 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S23472[] = "com.google.android.apps.plus/X (Linux; U; Android 3.1; en_CA; GT-P7500M; Build/HMJ37) (gzip)";
const char S23473[] = "Mozilla/5.0 (Linux; U; Android 3.1; ms-my; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 ZyngaBundleIdentifier/com.zynga.zjayakashi ZyngaBundleVersion/1.8.0";
const char S23474[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500R Build/HMJ37) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S23475[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.12;FBDM/{density=1.0,width=1280,height=800};FBLC/en_US;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P7500;FBSV/3.1;]";
const char S23476[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/2.0;FBDM/{density=1.0,width=1280,height=800};FBLC/en_US;FB_FW/2;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P7500;FBSV/3.1;]";
const char S23477[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500M Build/HMJ37) Globe News/1.10";
const char S23478[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500R Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/2.0;FBDM/{density=1.0,width=1280,height=800};FBLC/en_CA;FB_FW/2;FBCR/ROGERS;FBPN/com.facebook.katana;FBDV/GT-P7500R;FBSV/3.1;]";
const char S23479[] = "Mozilla/5.0 (Android 3.1; en-us; GT-P7500 Build/HMJ37) FreeWheelAdManager/5.2.2-r9326-X;com.cnn.mobile.android.phone/109";
const char S23480[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Android_Gmarket Qoo10 MY_2.0.9_26(4a90ee6f1a06a162____X;GT-P7500;3.1)";
const char S23481[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500R Build/HMJ37 like Xoom) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S23482[] = "Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 3.1; SAMSUNG GT-P7500R Build/HONEYCOMB_MR1);";
const char S23483[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500 Build/HMJ37) AppleWo) Version/4.0 Safari/534.13";
const char S23484[] = "MediaPlayer/2002/4.1.2 (Android 3.1; GT-P7500; GT-P7500; samsung; ; ; en_US)";
const char S23485[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500R Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 androidAppVersion/2.5.17 AndroidWebkitWrapper BillingSupported";
const char S23486[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500D Build/HMJ37) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S23487[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500M Build/HMJ37) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S23488[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; SAMSUNG-GT-P7500/P7500BUKF6 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S23489[] = "DinnerSpinnerFree/2.1.2 (Android 3.1; en-us; GT-P7500R/HMJ37.UXKH4)";
const char S23490[] = "BBC iPlayer (AndroidApp; 1.3; 13014) GT-P7500 (Android 3.1, SDK 12)";
const char S23491[] = "VKAndroidApp/2.7.6-64 (Android 3.1; SDK 12; armeabi-v7a; samsung GT-P7500; ru)";
const char S23492[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_Desire_C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23493[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_Desire_C_N/1.45.161.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23494[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Desire C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23495[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_Desire_C_N/1.45.161.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23496[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; A320a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23497[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Desire C Build/IML74K)";
const char S23498[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; HTC_Desire_C/1.45.162.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23499[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC_Desire_C/1.45.163.6 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23500[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC_Desire_C/1.45.163.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23501[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_Desire_C/1.45.162.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23502[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_Desire_C/1.45.169.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23503[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; HTC_Desire_C/1.46.113.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23504[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_Desire_C/1.46.112.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23505[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_Desire_C/1.45.169.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23506[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_Desire_C/1.46.111.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23507[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; HTC_Desire_C/1.46.114.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23508[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_Desire_C/1.46.123.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23509[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC_Desire_C-orange-LS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23510[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Desire C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23511[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; HTC_Desire_C/1.45.162.7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23512[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; el-gr; HTC_Desire_C/1.46.124.7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23513[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_Desire_C_N/2.00.161.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23514[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; HTC_Desire_C/2.00.114.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23515[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_Desire_C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23516[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_Desire_C/2.00.162.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23517[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_Desire_C/2.00.111.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23518[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC_Desire_C_N/2.00.163.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23519[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_Desire_C_N/2.00.169.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23520[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_Desire_C/2.00.123.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23521[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; HTC_Desire_C/2.00.113.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23522[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; el-gr; HTC_Desire_C/2.00.124.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23523[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Desire C Build/IMM76D)";
const char S23524[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_Desire_C/2.00.112.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23525[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; HTC_Desire_C/2.00.162.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23526[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; HTC Desire C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23527[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_Desire_C/2.00.111.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23528[] = "Opera/9.80 (Android; Opera Mini/5.1.25169/26.1670; U; es) Presto/2.8.119 Version/10.54";
const char S23529[] = "Opera/9.80 (Android; Opera Mini/5.1.24639/30.3214; U; en) Presto/2.8.119 Version/11.10";
const char S23530[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MB860 Build/4.5.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23531[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; MB860 Build/4.5.2A-51_OLL-17) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23532[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; MB860 Build/4.5.2A-51_OLL-37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23533[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; MB860 Build/4.5.2A-SKT-75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23534[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; MB860 Build/4.5.2A-51_OLL-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23535[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/4.5.1-174) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23536[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/4.5.70) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23537[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/CherryBlur v0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23538[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/4.5.1-178) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23539[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MB860 Build/4.5.47) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23540[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB860 Build/4.5.2A-74_OLE-26) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23541[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB860 Build/4.5.2A-74_OLE-19) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23542[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB860 Build/4.5.2A-74_OLE-21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23543[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB860 Build/4.5.2A-11_OLE-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23544[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/4.5.2A-74_OLE-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23545[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/Navalynt Test v1 Alpha) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23546[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/CherryBlur v1.4a) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23547[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MB860 Build/4.5.31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23548[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/4.5.78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23549[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/4.5.91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23550[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/4.5.2A-5_OLH-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23551[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/4.5.1_134) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23552[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/4.5.88) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23553[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MB860 Build/4.5.1_74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23554[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; MB860 Build/4.5.2A-74_OLE-20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23555[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB860 Build/4.5.130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23556[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB860 Build/4.5.2A-74_OLE-29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23557[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/4.5.9153) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23558[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/Home Base Orange) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23559[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB860 Build/4.5.131) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23560[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB860 Build/4.5.132) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23561[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB860 Build/4.5.91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23562[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB860 Build/4.5.91; RedPill Hybrid V1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23563[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; MB860 Build/4.5.2A-51_OLL-17.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23564[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; MB860 Build/4.5.2A-51_OLL-50) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23565[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; MB860 Build/4.5.2A-51_OLL-23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23566[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/4.5.91) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.1 Mobile Safari/534.30";
const char S23567[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB860 Build/4.5.141) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23568[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB860 Build/4.5.2A-74_OLE-31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23569[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB860 Build/4.5.2A-74_OLE-31.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23570[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; MB860 Build/4.5.2A-51_OLL-52) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23571[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MB860 Build/4.5.2A-51_OLL-48) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23572[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; MB860 Build/4.5.141)";
const char S23573[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; he-il; MB860 Build/4.5.2A-74_OLE-33) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23574[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MB860 Build/4.5.2A-51_OLL-49) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23575[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; MB860 Build/4.5.2A-SKT-167) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23576[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB860 Build/4.5.133) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23577[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB860 Build/4.5.140) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23578[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB860 Build/4.5.2A-74_OLE-31.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23579[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; MB860 Build/4.5.2A-74_OLE-31.20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23580[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; MB860 Build/4.5.2A-74_OLE-31.11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23581[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; MB860 Build/4.5.2A-51_OLL-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23582[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; MB860 Build/4.5.2A-74_OLE-31)";
const char S23583[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; MB860 Build/4.5.2A-74_OLE-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23584[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MB860 Build/4.5.2A-51_OLL-51.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23585[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB860 Build/4.5.2A-74_OLE-27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23586[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; MB860 Build/4.5.2A-SKT-167)";
const char S23587[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; MB860 Build/4.5.2A-51_OLL-35.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23588[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-; MB860 Build/4.5.141) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23589[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; MB860 Build/4.5.141)";
const char S23590[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB860 Build/4.5.141; MROM-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23591[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB860 Build/4.5.141) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S23592[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB860 Build/4.5.1A-129_OLY-145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23593[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-; MB860 Build/4.5.2A-74_OLE-31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23594[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB860 Build/4.5.141) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Sa";
const char S23595[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.89 Safari/532.5";
const char S23596[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.1.249.1064 Safari/532.5";
const char S23597[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.1.249.1064 Safari/532.5";
const char S23598[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Chrome/4.0.223.2 Safari/532.2";
const char S23599[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; de-DE) Chrome/4.0.223.3 Safari/532.2";
const char S23600[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1) AppleWebKit/532.6 (KHTML, like Gecko) Chrome/4.0.573.150 Safari/532.6";
const char S23601[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/26.0.1410.43) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/537.31";
const char S23602[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/26.0.1410.43) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/537.31";
const char S23603[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/26.0.1410.64; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S23604[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; chromeframe/26.0.1410.64; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S23605[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; chromeframe/26.0.1410.64; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S23606[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.4; chromeframe/26.0.1410.64; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S23607[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/26.0.1410.64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S23608[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/26.0.1410.64)";
const char S23609[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/26.0.1410.64)";
const char S23610[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/26.0.1410.64; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C)";
const char S23611[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/26.0.1410.64; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S23612[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/26.0.1410.64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S23613[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/26.0.1410.64; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S23614[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; chromeframe/26.0.1410.64; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S23615[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/26.0.1410.64; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S23616[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; chromeframe/26.0.1410.64; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S23617[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/532.9 (KHTML, like Gecko) Chrome/5.0.307.7 Safari/532.9";
const char S23618[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/532.9 (KHTML, like Gecko) Chrome/5.0.307.11 Safari/532.9";
const char S23619[] = "Mozilla/5.0 (X11; U; Jolicloud Linux i686; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.125 Safari/533.4";
const char S23620[] = "Mozilla/5.0 (X11; U; Linux i586; en-US) AppleWebKit/533.2 (KHTML, like Gecko) Chrome/5.0.342.1 Safari/533.2";
const char S23621[] = "Mozilla/5.0 (X11; U; x86_64 Linux; en_GB, en_US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/5.0.358.0 Safari/533.3";
const char S23622[] = "Mozilla/5.0 (X11; U; x86_64 Linux; en_US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/5.0.358.0 Safari/533.3";
const char S23623[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/6.1.25378/26.1424; U; ru) Presto/2.8.119 Version/10.54";
const char S23624[] = "Opera/9.50 (J2ME/MIDP; Opera Mini/6.1.27786/28.3165; U; en)";
const char S23625[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/6.1.25378/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S23626[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMT11i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23627[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMT11i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23628[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SonyEricssonMT11i-o Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23629[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SonyEricssonMT11iv Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23630[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-hk; SonyEricssonMT11i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23631[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SonyEricssonMT11iv Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23632[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MT11i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23633[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; MT11a Build/4.0.2.D.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23634[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-za; SonyEricssonMT11iv Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23635[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; MT11i Build/4.0.2.A.0.62)";
const char S23636[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SonyEricssonMT11i-o Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23637[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23638[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonMT11i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23639[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SonyEricssonMT11i-o Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23640[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; SonyEricssonMT11a Build/4.0.2.D.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23641[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; MT11i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23642[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; MT11i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23643[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; SonyEricssonMT11a Build/4.0.2.D.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23644[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23645[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; MT11i Build/4.0.2.A.0.62) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S23646[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; MT11a Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23647[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; SonyEricssonMT11a Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23648[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonMT11a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23649[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; MT11i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23650[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MT11a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23651[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-ar; SonyEricssonMT11a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23652[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MT11i Build/4.0.2.A.0.42) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S23653[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-hk; MT11i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23654[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; MT11a Build/4.0.2.D.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23655[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-ar; MT11a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23656[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MT11i Build/4.0.2.A.0.62) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S23657[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; MT11a Build/4.0.2.A.0.62)";
const char S23658[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ar-; SonyEricssonMT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23659[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/13";
const char S23660[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1";
const char S23661[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1";
const char S23662[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1";
const char S23663[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML	 like Gecko) Chrome/13.0.782.220 Safari/535.1";
const char S23664[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1,gzip(gfe)";
const char S23665[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-CA) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1";
const char S23666[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; es-US) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1";
const char S23667[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; fr-FR) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1";
const char S23668[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; haw-US) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1";
const char S23669[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; zh-CN) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1";
const char S23670[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; fr-CA) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1";
const char S23671[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; ku-DE) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1";
const char S23672[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML like Gecko) Chrome/13.0.782.41 Safari/535.1";
const char S23673[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; de-DE) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1";
const char S23674[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9930; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.755 Mobile Safari/534.11+";
const char S23675[] = "BlackBerry9930/7.1.0.163 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/105";
const char S23676[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Transformer TF101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23677[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Transformer TF101 Build/IML74K)";
const char S23678[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Transformer TF101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23679[] = "Mozilla/5.0 (Linux; U; Android 4.0.3 FROZN by jcarrz1; en-us; Transformer TF101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23680[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Transformer TF101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23681[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; TF101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23682[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Transformer TF101 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23683[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; Transformer TF101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23684[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Transformer_TF101 Build/IML74K; 1280*752) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S23685[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Transformer TF101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S23686[] = "facebookexternalhit/1.0 (+http://www.facebook.com/externalhit_uatext.php)";
const char S23687[] = "facebookplatform/1.0 (+http://developers.facebook.com)";
const char S23688[] = "facebookexternalhit/1.1 (+http://www.facebook.com/externalhit_uatext.php)";
const char S23689[] = "facebookexternalhit/1.0 (+http://www.facebook.com/externalhit_uatext.php) AppEngine-Google; (+http://code.google.com/appengine; appid: rtweetme)";
const char S23690[] = "facebookexternalhit/1.1 (+http://www.facebook.com/externalhit_uatext.php) Yandex.Translate";
const char S23691[] = "facebookexternalhit/1.1";
const char S23692[] = "facebookexternalhit/1.1 (+http://www.facebook.com/externalhit_uatext.php),gzip(gfe) (via docs.google.com/viewer)";
const char S23693[] = "facebookexternalhit";
const char S23694[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.12";
const char S23695[] = "Opera/9.80 (Windows NT 5.2; WOW64) Presto/2.12.388 Version/12.12";
const char S23696[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.12";
const char S23697[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.12";
const char S23698[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.12";
const char S23699[] = "Opera/9.80 (Windows NT 5.1; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.12";
const char S23700[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.12";
const char S23701[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.12";
const char S23702[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.12";
const char S23703[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.12";
const char S23704[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.12";
const char S23705[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.12";
const char S23706[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.12";
const char S23707[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.12";
const char S23708[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.12";
const char S23709[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.12";
const char S23710[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.12";
const char S23711[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.12";
const char S23712[] = "Mozilla/5.0 (Windows NT 5.1; rv:2.0) Gecko/X Firefox/4.0 Opera 12.12";
const char S23713[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1) Opera 12.12";
const char S23714[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.12";
const char S23715[] = "Opera/9.80 (Windows NT 5.1; Edition Yx) Presto/2.12.388 Version/12.12";
const char S23716[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.12";
const char S23717[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.12";
const char S23718[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.12";
const char S23719[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.12";
const char S23720[] = "Opera/9.80 (Windows NT 5.2) Presto/2.12.388 Version/12.12";
const char S23721[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.12";
const char S23722[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.12";
const char S23723[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.12,gzip(gfe)";
const char S23724[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.12";
const char S23725[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.12";
const char S23726[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.12";
const char S23727[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.12";
const char S23728[] = "Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.12";
const char S23729[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.12";
const char S23730[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.12";
const char S23731[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.12";
const char S23732[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.12";
const char S23733[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.12";
const char S23734[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.12";
const char S23735[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.12";
const char S23736[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.12";
const char S23737[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.12";
const char S23738[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.12";
const char S23739[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.12";
const char S23740[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.12";
const char S23741[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.12";
const char S23742[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.12";
const char S23743[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.12";
const char S23744[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.12";
const char S23745[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.12";
const char S23746[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.12";
const char S23747[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.12";
const char S23748[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.12";
const char S23749[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.12";
const char S23750[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.12";
const char S23751[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.12";
const char S23752[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.12";
const char S23753[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.12";
const char S23754[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.12";
const char S23755[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4933)) Presto/2.12.388 Version/12.12";
const char S23756[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.12";
const char S23757[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.12";
const char S23758[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03658)) Presto/2.12.388 Version/12.12";
const char S23759[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.12 Yandex.Translate";
const char S23760[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5278)) Presto/2.12.388 Version/12.12";
const char S23761[] = "Opera/9.80 (Windows NT 5.1; Edition Campaign 21) Presto/2.12.388 Version/12.12,gzip(gfe)";
const char S23762[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03402)) Presto/2.12.388 Version/12.12";
const char S23763[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.12";
const char S23764[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5650)) Presto/2.12.388 Version/12.12";
const char S23765[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.12";
const char S23766[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5660)) Presto/2.12.388 Version/12.12";
const char S23767[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03789)) Presto/2.12.388 Version/12.12";
const char S23768[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03639)) Presto/2.12.388 Version/12.12";
const char S23769[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.12";
const char S23770[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4110)) Presto/2.12.388 Version/12.12";
const char S23771[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4661)) Presto/2.12.388 Version/12.12";
const char S23772[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5336)) Presto/2.12.388 Version/12.12";
const char S23773[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5230)) Presto/2.12.388 Version/12.12";
const char S23774[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4832)) Presto/2.12.388 Version/12.12";
const char S23775[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5734)) Presto/2.12.388 Version/12.12";
const char S23776[] = "Opera/9.80 (Windows NT 5.1; MRA 5.3 (build 02564)) Presto/2.12.388 Version/12.12";
const char S23777[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5288)) Presto/2.12.388 Version/12.12";
const char S23778[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03686); MRSPUTNIK OW 2, 3, 0, 244) Presto/2.12.388 Version/12.12";
const char S23779[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.12";
const char S23780[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03755)) Presto/2.12.388 Version/12.12";
const char S23781[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5325)) Presto/2.12.388 Version/12.12";
const char S23782[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5299)) Presto/2.12.388 Version/12.12";
const char S23783[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.12,gzip(gfe) (via docs.google.com/viewer)";
const char S23784[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5217)) Presto/2.12.388 Version/12.12";
const char S23785[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.12";
const char S23786[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.12";
const char S23787[] = "Opera/9.80 (Windows NT 5.1; MRA 5.3 (build 02552)) Presto/2.12.388 Version/12.12";
const char S23788[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03402); MRSPUTNIK OW 2, 3, 0, 104) Presto/2.12.388 Version/12.12";
const char S23789[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.12";
const char S23790[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.12";
const char S23791[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6044)) Presto/2.12.388 Version/12.12";
const char S23792[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.12";
const char S23793[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.12";
const char S23794[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.12";
const char S23795[] = "Opera/9.80 (Windows NT 5.1; MRA 5.4 (build 02652)) Presto/2.12.388 Version/12.12";
const char S23796[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6035)) Presto/2.12.388 Version/12.12";
const char S23797[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6046)) Presto/2.12.388 Version/12.12";
const char S23798[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.12";
const char S23799[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.12";
const char S23800[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.12 WebWasher 3.4";
const char S23801[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.12";
const char S23802[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5161)) Presto/2.12.388 Version/12.12";
const char S23803[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5300)) Presto/2.12.388 Version/12.12";
const char S23804[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.12";
const char S23805[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.12";
const char S23806[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5988)) Presto/2.12.388 Version/12.12";
const char S23807[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.12";
const char S23808[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.12";
const char S23809[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03672)) Presto/2.12.388 Version/12.12";
const char S23810[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6080)) Presto/2.12.388 Version/12.12";
const char S23811[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03396); MRSPUTNIK OW 2, 3, 0, 104) Presto/2.12.388 Version/12.12";
const char S23812[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.12";
const char S23813[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6032)) Presto/2.12.388 Version/12.12";
const char S23814[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03791)) Presto/2.12.388 Version/12.12";
const char S23815[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03780)) Presto/2.12.388 Version/12.12";
const char S23816[] = "Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02746)) Presto/2.12.388 Version/12.12";
const char S23817[] = "Opera/9.80 (Windows NT 5.1; MRA 5.3 (build 02560)) Presto/2.12.388 Version/12.12";
const char S23818[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.12";
const char S23819[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.12";
const char S23820[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.12";
const char S23821[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03638)) Presto/2.12.388 Version/12.12";
const char S23822[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6673)) Presto/2.12.388 Version/12.12";
const char S23823[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.12,gzip(gfe),gzip(gfe)";
const char S23824[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.12";
const char S23825[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6573)) Presto/2.12.388 Version/12.12";
const char S23826[] = "Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6341)) Presto/2.12.388 Version/12.12";
const char S23827[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_ChaCha Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23828[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; el-gr; HTC/ChaCha/1.19.168.3 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23829[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; HTC/ChaCha/1.19.163.4 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23830[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; HTC_ChaChaCha_A810e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23831[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_ChaCha_A810e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23832[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC ChaCha A810e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23833[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_ChaCha-orange-LS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23834[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC_ChaCha_A810b Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23835[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-at; HTC/ChaCha/1.19.169.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23836[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC/ChaCha/1.19.161.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23837[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-; HTC_ChaCha_A810e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23838[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_ChaCha_A810e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23839[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-nz; HTC/ChaCha/1.27.179.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23840[] = "HTC_ChaCha_A810e/Mozilla/5.0(Linux; U; Android 2.3.3;it-it; Build/GRH78C) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23841[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC ChaCha A810b Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23842[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; HTC ChaChaCha A810e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23843[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; HTC_ChaChaCha-orange-LS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23844[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC ChaCha A810e Build/GRI40)";
const char S23845[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC ChaCha A810e Build/GRJ90)";
const char S23846[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC ChaCha A810b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23847[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC_ChaCha_A810b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23848[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; HTC_ChaChaCha_A810e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23849[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC ChaCha A810e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23850[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/ChaCha/1.57.161.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23851[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_ChaCha Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23852[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC/ChaCha/1.57.163.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23853[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_ChaCha-orange-LS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23854[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-at; HTC/ChaCha/1.57.169.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23855[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; HTC ChaChaCha A810e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23856[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-nz; HTC/ChaCha/1.57.179.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23857[] = "HTC_ChaCha_A810e/Mozilla/5.0(Linux; U; Android 2.3.5;it-it; Build/GRH78C) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23858[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC A810e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23859[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC ChaCha A810e Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S23860[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/ChaCha/1.19.161.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23861[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC A810e Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S23862[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/ChaCha/1.57.161.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23863[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-cn; HTCA810e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23864[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC/ChaCha/1.57.163.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23865[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC ChaCha A810e Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S23866[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC/ChaCha/1.53.192.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23867[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC A810e Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S23868[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC ChaCha A810e Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S23869[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-pt; HTC/ChaCha/1.57.168.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23870[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC ChaCha A810e Build/GRJ90; SuperOSR-chacha) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23871[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC ChaCha A810e Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S23872[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; nl-nl; HTC/ChaCha/1.57.161.4 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23873[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-at; HTC/ChaCha/1.57.169.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23874[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-nz; HTC/ChaCha/1.57.179.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23875[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC EVO 3D X515m Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23876[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_EVO_3D_GSM_X515m Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23877[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC EVO 3D X515m Build/IML74K)";
const char S23878[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_X515C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23879[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; EVO3D_X515m Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23880[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; HTC_EVO3D_X515a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23881[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; HTC EVO 3D X515a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23882[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-sg; HTC_EVO3D_X515m Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23883[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC EVO 3D X515m Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S23884[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-kr; HTC_X515E Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23885[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC X515d Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S23886[] = "Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; HTC EVO 3D X515m Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S23887[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC_X515E Build/IML74K)";
const char S23888[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC EVO 3D X515a Build/IML74K)";
const char S23889[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Kingdom Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23890[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC_X515a V3.29.841.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23891[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-X515d/2.21.1401.3) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23892[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC EVO 3D X515m Build/IML74K) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 baidubrowser/042_1.41.3.2_diordna_069_045/CTH_51_3.0.4_m515X-D3-OVE-CTH/Xa/0A4794C9CE42717CF87A1F2472B68FFA%7CX/1";
const char S23893[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC X515d) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23894[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; HTC X515e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23895[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; EVO3D_X515m Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23896[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC_X515a V1.24.841.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23897[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-; EVO3D_X515m Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23898[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC X515m Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23899[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; ARM; Trident/6.0)";
const char S23900[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; ARM; Trident/6.0; Touch)";
const char S23901[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; ARM; Trident/6.0; Touch; ARMBJS)";
const char S23902[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; ARM; Trident/6.0; ARMBJS)";
const char S23903[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; ARM; Trident/6.0; Touch; .NET4.0E; .NET4.0C; Tablet PC 2.0; WebView/1.0)";
const char S23904[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23905[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; SAMSUNG GT-P3100/P3100BUCLK1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23906[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23907[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P3100 Build/JRO03C)";
const char S23908[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23909[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; GT-P3100B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23910[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P3100 Build/JZO54K)";
const char S23911[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23912[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3100 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23913[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3100 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23914[] = "FaithlifeStudyBible/2.1.2 (Linux; U; Android 4.1.1; en-GB; GT-P3100 Build/JRO03C.P3100DDBLI1)";
const char S23915[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S23916[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3100 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S23917[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/TELL// X/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23918[] = "Logos/2.2.8.1 (Linux; U; Android 4.1.1; en-US; GT-P3100 Build/JRO03C.P3100DDCLL1)";
const char S23919[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; SAMSUNG GT-P3100/P3100BUCLJ1.DM Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23920[] = "FaithlifeStudyBible/2.2.10 (Linux; U; Android 4.1.1; en-US; GT-P3100 Build/JRO03C.P3100BUCLK1)";
const char S23921[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3100 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23922[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23923[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 VoiceGO!(cyb471cfd6517cc855a94a1c1c91fc15bc7-0;Android,4.1.1,GT-P3100;3.8.0;600x895,1.0;X;VoiceGo-TC;zh-TW;X;X)";
const char S23924[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23925[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; SAMSUNG GT-P3100/P3100BUCLK1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23926[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3100 Build/TELL// X/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23927[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) Applcko) Version/4.0 Safari/534.30";
const char S23928[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P3100B Build/JRO03C)";
const char S23929[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3100 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S23930[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.1 Version/4.0 Mobile Safari/530.17";
const char S23931[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30";
const char S23932[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23933[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.4.0-99 (Android 4.1.1; samsung GT-P3100 Build/JRO03C.P3100DDCLL1)";
const char S23934[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C.P3100DDCLL1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23935[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-; GT-P3100B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23936[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23937[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; GT-P3100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S23938[] = "Mozilla/5.0 (Linux; U;Android 4.1.2; fr-fr; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23939[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-gb; GT-P3100 Build/JellyBean) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S23940[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P3100 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S23941[] = "Mozilla 5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3100 Build JRO03C) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S23942[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3100B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S23943[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3100 Build/JZO54K) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.2 Version/4.0 Mobile Safari/530.17";
const char S23944[] = "com.google.android.youtube/4.2.16(Linux; U; Android 4.1.2; en_GB; GT-P3100 Build/JZO54K) gzip";
const char S23945[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; SAMSUNG GT-P3100/P3100BUDMB2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23946[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 GoogleGoggles-Android/1.9; gzip";
const char S23947[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; GT-P3100 Build/JRO03C) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S23948[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-P3100-ORANGE/P3100BVDMD1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23949[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-P3100-ORANGE/P3100BVDMF1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S23950[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/523.12 (KHTML, like Gecko) Version/3.0.4 Safari/523.12";
const char S23951[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-us) AppleWebKit/523.15.1 (KHTML, like Gecko) Version/3.0.4 Safari/523.15";
const char S23952[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-us) AppleWebKit/523.10.3 (KHTML, like Gecko) Version/3.0.4 Safari/523.10";
const char S23953[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/522.11.1 (KHTML, like Gecko) Version/3.0.3 Safari/522.12.1";
const char S23954[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/522.10.1 (KHTML, like Gecko) Version/3.0 Safari/522.11";
const char S23955[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-us) AppleWebKit/523.10.5 (KHTML, like Gecko) Version/3.0.4 Safari/523.13.2";
const char S23956[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/523.6 (KHTML, like Gecko) Version/3.0.3 Safari/523.6";
const char S23957[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/523.1 (KHTML, like Gecko) Version/3.0.3 Safari/523.1";
const char S23958[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en_US) AppleWebKit/522.7 (KHTML, like Gecko) Dreamweaver/10.0.0.4117 Version/3.0 Safari/522.7";
const char S23959[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-us) AppleWebKit/523.15.1 (KHTML, like Gecko) Version/3.0.4 Safari/99999 MultiSafari/5523.15";
const char S23960[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.125 Safari/533.4";
const char S23961[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.99 Safari/533.4";
const char S23962[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.125 Safari/533.4,gzip(gfe) (via translate.google.com)";
const char S23963[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/531.0 (KHTML, like Gecko) Chrome/5.0.195.0 Safari/531.0";
const char S23964[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.86 Safari/533.4,gzip(gfe)";
const char S23965[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.70 Safari/533.4 AppEngine-Google; (+http://code.google.com/appengine; appid: maipurokushi)";
const char S23966[] = "Mozilla/5.0 (Windows NT 5.1; rv:17.0) Gecko/17.0 Firefox/17.0";
const char S23967[] = "Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:17.0) Gecko/17.0 Firefox/17.0";
const char S23968[] = "Mozilla/5.0 (Windows NT 5.2; rv:17.0) Gecko/17.0 Firefox/17.0";
const char S23969[] = "Mozilla/5.0(Windows; U; Windows NT 5.2; rv:1.9.2) Gecko/X Firefox/17.0";
const char S23970[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.1.7) Gecko/X Firefox/17.0 (.NET CLR 3.0.04506.648)";
const char S23971[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:17.0) Gecko/X Firefox/17.0ms1";
const char S23972[] = "BlackBerry8520/4.6.1.314 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/600";
const char S23973[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B146 Safari/8536.25";
const char S23974[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B141 Safari/8536.25";
const char S23975[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B147 Safari/8536.25";
const char S23976[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B329 Safari/8536.25";
const char S23977[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100 (Edition Next)";
const char S23978[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100 (Edition Next)";
const char S23979[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.130 (Edition Rambler)";
const char S23980[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.130 (Edition Campaign 06)";
const char S23981[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.130 (Edition Campaign 06)";
const char S23982[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.130 (Edition Seznam)";
const char S23983[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Yx)";
const char S23984[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153";
const char S23985[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153";
const char S23986[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153,gzip(gfe)";
const char S23987[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Seznam)";
const char S23988[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Campaign 21)";
const char S23989[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Rambler)";
const char S23990[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153,gzip(gfe)";
const char S23991[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Yx)";
const char S23992[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Campaign 21)";
const char S23993[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153,gzip(gfe),gzip(gfe)";
const char S23994[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S23995[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.28 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.28";
const char S23996[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.28 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.28";
const char S23997[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.50 Safari/534.7 SunChrome/7.0";
const char S23998[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.16; QQBrowser/5.1.6829.201 (webkit)";
const char S23999[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; de-DE) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/7.0.540.0 Safari/534.10";
const char S24000[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.16; QQBrowser/5.1.7061.201 (webkit)";
const char S24001[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0 baidubrowser/1.x Safari/534.7";
const char S24002[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/7.0.544.0 Safari/534.10,gzip(gfe) (via translate.google.com)";
const char S24003[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.16; QQBrowser/5.1.7257.201 (webkit)";
const char S24004[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.41 Safari/534.7,gzip(gfe)";
const char S24005[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus 7 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S24006[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9B206 Safari/7534.48.3";
const char S24007[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9B176 Safari/7534.48.3";
const char S24008[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9A405 Safari/7534.48.3";
const char S24009[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9Z999 Safari/7534.48.3";
const char S24010[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; DigExt; DTS Agent";
const char S24011[] = "Mozilla/4.0 (compatible; MSIE 5.01; Windows NT)";
const char S24012[] = "Mozilla/4.0 (compatible; MSIE 5.02; Windows NT)";
const char S24013[] = "Mozilla/4.0 (compatible; MSIE 5.0; Win32)";
const char S24014[] = "Mozilla/4.0 (compatible;MSIE 5.0;  Windows NT 7.0; LOVEYOU)";
const char S24015[] = "Mozilla/6.0 (compatible;MSIE 5.0;  Windows NT 7.0; QQLOADOW)";
const char S24016[] = "Mozilla/5.0 (compatible; MSIE 5.0; 68K; Trident/1.0)";
const char S24017[] = "Mozilla/5.0 (compatible; MSIE 5.0; 68K; WOW64; Trident/4.0)";
const char S24018[] = "Mozilla/5.0 (compatible; MSIE 5.0; PPC; Win64; x64; Trident/5.0)";
const char S24019[] = "Mozilla/5.0 (compatible; MSIE 5.0; 68K; SV1; Trident/4.0)";
const char S24020[] = "Mozilla/5.0 (compatible; MSIE 5.0; PPC; .NET CLR; Trident/5.0)";
const char S24021[] = "Mozilla/5.0 (compatible; MSIE 5.0; PPC; WOW64; Trident/2.0)";
const char S24022[] = "Mozilla/5.0 (compatible; MSIE 5.0; 68K; Win64; x64; Trident/1.0)";
const char S24023[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Sensation Z710e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24024[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Sensation Z710e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24025[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC_Sensation_Z710e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24026[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTCZ710e/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24027[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; HTC Sensation Z710e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24028[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Sensation_Z710e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24029[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; HTC Sensation 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24030[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;HTC_Sensation_4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24031[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_Sensation_4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S24032[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; HTC_Sensation_4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24033[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; HTC_Sensation_4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24034[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_Sensation_4GBuild/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24035[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Sensation 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24036[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_Sensation_4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S24037[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_Sensation_4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24038[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Sensation Z710e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24039[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_Sensation_Z710e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24040[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Sensation4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24041[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Sensation 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24042[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_Sensation_4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24043[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;HTC Sensation 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24044[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Sensation 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S24045[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Sensation 4GBuild/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24046[] = "Mozilla/5.0 (Linux; U; Android2.3.3; en-us; HTC_Sensation_4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24047[] = "Mozilla/5.0 (Linux; U; Android2.3.3; en-us; HTC Sensation 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24048[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;HTC_Sensation_4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24049[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Sensation 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S24050[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Sensation 4GBuild/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24051[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;HTC Sensation 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24052[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_Sensation_4GBuild/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24053[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Sensation_Z710e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24054[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Sensation Z710e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24055[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Htc Sensation 4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24056[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Sensation_4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24057[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; vi-; HTC_Sensation_4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24058[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Sensation_4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S24059[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Sensation Z710e Build/GRI40)";
const char S24060[] = "Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; Sensation_4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S24061[] = "Mozilla/5.0 (Linux; U; Android2.3.3; en-us; Sensation_4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S24062[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Sensation_4GBuild/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S24063[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;Sensation_4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S24064[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Sensation Z710e Build/GRJ22)";
const char S24065[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Sensation 4G Build/GRJ22)";
const char S24066[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Sensation 4G Build/GRI40)";
const char S24067[] = "Logos/0.9.3 (Linux; U; Android 2.3.4; en_US; HTC Sensation 4G Build/GRJ22)";
const char S24068[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; HTC Sensation 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24069[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTCSensation 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24070[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Sensation4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24071[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Sensation 4G XE with Beats Audio Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24072[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Sensation Z710e Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S24073[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Sensation Z710e Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S24074[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Sensation Z710e Build/GRJ90)";
const char S24075[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Z710e Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S24076[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Sensation 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24077[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Z710e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24078[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Sensation Z710e Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S24079[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Sensation Z710e Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24080[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Sensation Z710e Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24081[] = "Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; HTC Sensation 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24082[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Sensation 4G Build/GRJ22) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S24083[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-; HTC Sensation Z710e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24084[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Htcclay's SuperSensation 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24085[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Sensation Z710e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24086[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Sensation 4G Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S24087[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; HTC Sensation 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24088[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Sensation Z710e with Beats Audio Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24089[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Sensation Z710e Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24090[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Z710e Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24091[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Sensation Z710e with Beats Audio Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24092[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/27.0.1453.47) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.47 Safari/537.36";
const char S24093[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/27.0.1453.81) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.81 Safari/537.36";
const char S24094[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/27.0.1453.93) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36";
const char S24095[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/27.0.1453.94; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S24096[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/27.0.1453.94) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36";
const char S24097[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/27.0.1453.93) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36";
const char S24098[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/27.0.1453.94) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36";
const char S24099[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/27.0.1453.110) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36";
const char S24100[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.4; chromeframe/27.0.1453.110; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S24101[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/27.0.1453.110) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36";
const char S24102[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/27.0.1453.110; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S24103[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; chromeframe/27.0.1453.110; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S24104[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; chromeframe/27.0.1453.110; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S24105[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/27.0.1453.110; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S24106[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/27.0.1453.116)";
const char S24107[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/27.0.1453.116)";
const char S24108[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/27.0.1453.116) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S24109[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/27.0.1453.116) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S24110[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/27.0.1453.116; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S24111[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; chromeframe/27.0.1453.116; GTB7.5; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S24112[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S24113[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293 Safari/6531.22.7";
const char S24114[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B5097d Safari/6531.22.7";
const char S24115[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/9Z999 Safari/6531.22.7";
const char S24116[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293";
const char S24117[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S24118[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; vi-vn) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/PV004 Safari/6531.22.7";
const char S24119[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; vi-vn) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/PV001 Safari/6531.22.7";
const char S24120[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/ 8B117  Safari/6531.22.7";
const char S24121[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 41 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S24122[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S24123[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7";
const char S24124[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2";
const char S24125[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en_us) AppleWebKit/525.18.1 (KHTML, like Gecko)";
const char S24126[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en_us) AppleWebKit/525.18.1 (KHTML, like Gecko)";
const char S24127[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S24128[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S24129[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190";
const char S24130[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4";
const char S24131[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S24132[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2";
const char S24133[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S24134[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1";
const char S24135[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S24136[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/533.17.9";
const char S24137[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_3_2 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3.2;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S24138[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_3 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S24139[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_3_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S24140[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_3_5 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3.5;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S24141[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_3_3 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3.3;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S24142[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 iPod4,1 BingWeb/3.03.1428.X";
const char S24143[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/533.17.9";
const char S24144[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_3_4 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3.4;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S24145[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 iPod4,1 BingWeb/3.03.1428.X";
const char S24146[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 iPod4,1 BingWeb/3.03.1428.X";
const char S24147[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 iPod4,1 BingWeb/3.03.1428.X";
const char S24148[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3.5;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]";
const char S24149[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]";
const char S24150[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3.2;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]";
const char S24151[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]";
const char S24152[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3.4;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]";
const char S24153[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190 iPod4,1 BingWeb/3.03.1428.X";
const char S24154[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2 iPod4,1 BingWeb/3.03.1428.X";
const char S24155[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3.3;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]";
const char S24156[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 43 like Mac OS X; en-US) AppleWebKit/533.17.9 (KHTML like Gecko) Mobile/8F190";
const char S24157[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 435 like Mac OS X; en-US) AppleWebKit/533.17.9 (KHTML like Gecko) Mobile/8L1";
const char S24158[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 433 like Mac OS X; es-US) AppleWebKit/533.17.9 (KHTML like Gecko) Mobile/8J2";
const char S24159[] = "Mozilla/5.0 (iPhone; U; CPU iPhone 4_3_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPod4,1/4.3.3)";
const char S24160[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24161[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24162[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24163[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24164[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I317 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24165[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24166[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24167[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24168[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24169[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S24170[] = "Mozilla/5.0 (Linux; Android 4.1.1; SC-02E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24171[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24172[] = "Mozilla/5.0 (Linux; Android 4.1.1; SPH-L900 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24173[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24174[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24175[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24176[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I317M Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24177[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24178[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24179[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24180[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24181[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24182[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24183[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-N719 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24184[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S24185[] = "Mozilla/5.0 (Linux; Android 4.1.1; SHV-E250S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24186[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889V Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24187[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-R950 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24188[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S24189[] = "Mozilla/5.0 (Linux; Android 4.1.9; GT-N7100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24190[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-N719 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24191[] = "Mozilla/5.0 (Linux; Android 4.1.1; SHV-E250K Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S24192[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; de) Opera 9.21";
const char S24193[] = "Opera/9.23 (Windows NT 5.1; U; ru)";
const char S24194[] = "Opera/9.27 (Windows NT 5.1; U; ru)";
const char S24195[] = "Opera/9.24 (Windows NT 5.1; U; ru)";
const char S24196[] = "Opera/9.20 (Windows NT 5.1; U; de)";
const char S24197[] = "Opera/9.21 (Windows NT 5.1; U; en)";
const char S24198[] = "Opera/9.22 (Windows NT 5.1; U; en)";
const char S24199[] = "Opera/9.26 (Windows NT 5.1; U; ru)";
const char S24200[] = "Opera/9.25 (Windows NT 5.1; U; ru)";
const char S24201[] = "Opera/9.21 (Windows NT 5.2; U; ru)";
const char S24202[] = "opera/9.26 (windows nt 5.2; u; de)";
const char S24203[] = "Opera/9.27 (Windows NT 5.2; U; ru)";
const char S24204[] = "Opera/9.23 (Windows NT 5.2; U; en)";
const char S24205[] = "Opera/9.20 (Windows NT 5.2; U; en)";
const char S24206[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 9.24";
const char S24207[] = "Opera/9.27 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru)";
const char S24208[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; en) Opera 9.27";
const char S24209[] = "Mozilla/5.0 (Windows NT 5.1; U; es-la; rv:1.8.0) Gecko/X Firefox/1.5.0 Opera 9.27";
const char S24210[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 9.26";
const char S24211[] = "Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.8.0) Gecko/X Firefox/1.5.0 Opera 9.24";
const char S24212[] = "Opera/9.25 (Windows NT 5.2; U; en)";
const char S24213[] = "Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.8.0) Gecko/X Firefox/1.5.0 Opera 9.22";
const char S24214[] = "Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.8.0) Gecko/X Firefox/1.5.0 Opera 9.26";
const char S24215[] = "Mozilla/5.0 (Windows NT 5.2; U; en; rv:1.8.0) Gecko/X Firefox/1.5.0 Opera 9.27";
const char S24216[] = "Opera/9.20 (Windows NT 5.1; U; pl) Version/10.9";
const char S24217[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 9.22";
const char S24218[] = "Opera/9.20 (Windows NT 5.1; U; pl) Presto/2.2.15 Version/10.10";
const char S24219[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; MEGAUPLOAD 1.0; ru) Opera 9.23";
const char S24220[] = "Opera/9.27 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru)";
const char S24221[] = "Opera/9.27 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru)";
const char S24222[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT22i Build/6.1.B.0.544) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24223[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonLT22i Build/6.1.B.0.544) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24224[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; LT22i Build/6.1.B.0.550) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24225[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT22i Build/6.1.B.0.544) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24226[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT22i Build/6.1.1.B.1.54)";
const char S24227[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SonyEricssonLT22i-o Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24228[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT22i Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24229[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; LT22i Build/6.1.B.0.559) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24230[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT22i Build/6.1.1.B.1.54) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24231[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; LT22i Build/6.1.1.B.1.75) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24232[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:19.0) Gecko/19.0 Firefox/19.0";
const char S24233[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:19.0) Gecko/X Firefox/19.0 AlexaToolbar/alxf-2.17";
const char S24234[] = "Mozilla/5.0 (Windows; Windows NT 6.1; Win64; x64; rv:19.0) Gecko/19.0 Firefox/19.0";
const char S24235[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; WOW64; en-US; rv:19.0) Gecko/X AskTbAVR-IDW/3.12.5.17700 Firefox/19.0";
const char S24236[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.9 Safari/536.5";
const char S24237[] = "Mozilla/5.0 (X11; Linux x86_64) Chrome/19.0.1042.0 Safari/535.21";
const char S24238[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.8 (KHTML, like Gecko; Google Web Preview) Chrome/19.0.1084.36 Safari/536.8";
const char S24239[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5,gzip(gfe) (via translate.google.com)";
const char S24240[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/536.8 (KHTML, like Gecko) Chrome/19.0.1084.36 Safari/536.8";
const char S24241[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S24242[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.8 (KHTML, like Gecko; Google Web Preview) Chrome/19.0.1084.36 Safari/536.8,gzip(gfe) (via docs.google.com/viewer)";
const char S24243[] = "Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S24244[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.8 (KHTML, like Gecko; Google Page Speed Insights) Chrome/19.0.1084.36 Safari/536.8";
const char S24245[] = "\"Mozilla/5.0 (X11; gzOS Linux i686) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.9 Safari/536.5\"";
const char S24246[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.20 (KHTML	 like Gecko) Chrome/19.0.1036.7 Safari/535.20";
const char S24247[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.20 (KHTML	 like Gecko) Chrome/19.0.1036.7 Safari/535.20";
const char S24248[] = "Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.9 Safari/536.5";
const char S24249[] = "Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/19.0.1410.63 Safari/537.31";
const char S24250[] = "Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/19.0.1410.63 Safari/537.31";
const char S24251[] = "Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/19.0.1410.63 Safari/537.31";
const char S24252[] = "Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/19.0.1410.63 Safari/537.31";
const char S24253[] = "Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/19.0.1410.63 Safari/537.31";
const char S24254[] = "Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/19.0.1410.63 Safari/537.31";
const char S24255[] = "Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/19.0.1410.63 Safari/537.31";
const char S24256[] = "Mozilla/5.0 (Linux) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.9 Safari/536.5";
const char S24257[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/19.77.34.5 Safari/537.1";
const char S24258[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/19.77.34.5 Safari/537.1";
const char S24259[] = "Mozilla/5.0 (X11; U; Linux i586) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/19.77.34.5 Safari/537.1";
const char S24260[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:10.0.1) Gecko/X Firefox/10.0.1 Firefox/10.0.1";
const char S24261[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:10.0.2) Gecko/X Firefox/10.0.2";
const char S24262[] = "Mozilla/5.0 (Windows; Windows NT 6.1; rv:10.0) Gecko/X Firefox/10.0";
const char S24263[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:10.0.12) Gecko/X Firefox/10.0.12";
const char S24264[] = "Mozilla/10 (Windows; U; Windows NT 6.1; en-US;     rv:1.9.2.13) Gecko/X Firefox/10 (.NET CLR 3.5.30729)";
const char S24265[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24266[] = "Mozilla/5.0 (Linux; Android 4.1.1; SPH-L710 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24267[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24268[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-R530U Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24269[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747M Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24270[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24271[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305T Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24272[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24273[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24274[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24275[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24276[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305N Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24277[] = "Mozilla/5.0 (Linux; Android 4.1.1; SHV-E210K Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24278[] = "Mozilla/5.0 (Linux; Android 4.1.1; SHV-E210S Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24279[] = "Mozilla/5.0 (Linux; Android 4.1.1; SHV-E210L Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24280[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24281[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24282[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24283[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24284[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24285[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530C Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24286[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305N Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24287[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747m Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S24288[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.5; en-GB; rv:1.9.2.28) Gecko/X Firefox/3.6.28";
const char S24289[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24290[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24291[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24292[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24293[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24294[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24295[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24296[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24297[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24298[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24299[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24300[] = "Mozilla/5.0 (Linux; Android 4.1.1; SC-02E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24301[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24302[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24303[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I317 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24304[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24305[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24306[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24307[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S24308[] = "Mozilla/5.0 (Linux; Android 4.1.1; SPH-L900 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24309[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24310[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I317M Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24311[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24312[] = "Mozilla/5.0 (Linux; Android 4.1.1; SHV-E250S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24313[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-R950 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24314[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889V Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24315[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-N719 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S24316[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S24317[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S24318[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3,gzip(gfe) (via translate.google.com)";
const char S24319[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.4 (KHTML, like Gecko) Chrome/6.0.481.0 Safari/534.4";
const char S24320[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Source Client/0; ) AppleWebKit/534.1 (KHTML, like Gecko) Chrome/6.0.444.0 Safari/534.1";
const char S24321[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay; ) AppleWebKit/534.1 (KHTML, like Gecko) Chrome/6.0.444.0 Safari/534.1";
const char S24322[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.3 (KHTML	 like Gecko) Chrome/6.0.472.53 Safari/534.3";
const char S24323[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3,gzip(gfe)";
const char S24324[] = "Sapere/Nutch-2.2";
const char S24325[] = "My Nutch Spider/Nutch-1.7";
const char S24326[] = "guoming/Nutch-1.6";
const char S24327[] = "NIS Nutch Spider/Nutch-1.7";
const char S24328[] = "MyNutchTest/Nutch-1.7";
const char S24329[] = "ccsnutchspider/Nutch-2.2";
const char S24330[] = "Focus Spider/Nutch-1.6";
const char S24331[] = "crawl/Nutch-2.0";
const char S24332[] = "linguatools-bot/Nutch-1.6 (searching for translated pages; http://www.linguatools.de/linguatoolsbot.html; peter dot kolb at linguatools dot org)";
const char S24333[] = "NutchSpider/Nutch-1.6";
const char S24334[] = "grank/Nutch-1.6";
const char S24335[] = "shiva/Nutch-1.4";
const char S24336[] = "NutchCVS/0.7.1 (Nutch; http://lucene.apache.org/nutch/bot.html; nutch-agent@lucene.apache.org);c2VvLWNyYXdsZXI=";
const char S24337[] = "My Nutch Spider/Nutch-1.7 (http://wiki.creativecommons.org/DiscoverEd)";
const char S24338[] = "tomit spider/Nutch-1.7";
const char S24339[] = "zctech agent/Nutch-2.1";
const char S24340[] = "CCResearchBot/1.0 commoncrawl.org/research//Nutch-1.7-SNAPSHOT";
const char S24341[] = "Mysite/Nutch-2.2.1";
const char S24342[] = "YourNutchSpider/Nutch-2.2.1";
const char S24343[] = "PulsePoint bot/Nutch-1.6 (www.pulsepoint.com; info)";
const char S24344[] = "foresealife/Nutch-1.6";
const char S24345[] = "Domnutch-Bot/Nutch-1.0 (Domnutch; http://www.Nutch.de/);c2VvLWNyYXdsZXI=";
const char S24346[] = "CB/Nutch-1.7";
const char S24347[] = "Oculus360/Nutch-1.7";
const char S24348[] = "My Nutch Spider/Nutch-2.2.1";
const char S24349[] = "nutch-solr-integration/Nutch-1.7";
const char S24350[] = "gorgor/Nutch-2.0-dev";
const char S24351[] = "Nutch12/Nutch-1.2";
const char S24352[] = "LWNutch/Nutch-1.4 (another scientific bot - we check your robots.txt! ; contact by mail: abuse _at- languageweaver.com)";
const char S24353[] = "Telecom/Nutch-1.6 (warning@gmail.com)";
const char S24354[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-ca; GT-P7500R Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S24355[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7500 Build/HTJ85B)";
const char S24356[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; SAMSUNG-GT-P7500/P7500BUKL1 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S24357[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S24358[] = "Mozilla/5.0 (Linux; U; Android 3.2; fr-ca; GT-P7500V Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S24359[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7500R Build/HTJ85B)";
const char S24360[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7500/P7500BOKK1 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S24361[] = "Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7500-ORANGE/P7500BVKK2 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S24362[] = "Mozilla/5.0 (Linux; U; Android 3.2; es-es; GT-P7500-ORANGE/P7500BVLA1 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S24363[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S24364[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S24365[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7500/P7500BOLA1 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S24366[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; SAMSUNG-GT-P7500/P7500BUKJ1 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S24367[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-ca; GT-P7500R Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S24368[] = "Logos/0.9.4 (Linux; U; Android 3.2; en_CA; GT-P7500R Build/HTJ85B.UXKK2)";
const char S24369[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7500-ORANGE/P7500BVKK1 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S24370[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7500V Build/HTJ85B)";
const char S24371[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7500 Build/HTJ85B) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S24372[] = "Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7500-ORANGE/P7500BVKJ5 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S24373[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7500 Build/HTJ85B) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24374[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7500) AntivirusAM/2.5 (KHTML, like Gecko) Version/Version 2.5/Antivirus";
const char S24375[] = "Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7500-ORANGE/P7500BVLA1T Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S24376[] = "Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 3.2; SAMSUNG GT-P7500 Build/HONEYCOMB_MR2);";
const char S24377[] = "Mozilla/5.0 (Linux; U; Android 3.2; es-; GT-P7500R Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S24378[] = "Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7500 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24379[] = "Mozilla/5.0 (Android 3.2; en-gb; GT-P7500 Build/HTJ85B) FreeWheelAdManager/5.0.3-r9054-X;com.gotv.crackle.handset/X";
const char S24380[] = "com.google.android.youtube/4.1.23(Linux; U; Android 3.2; en_GB; GT-P7500 Build/HTJ85B) gzip";
const char S24381[] = "Mozilla 5.0 (Linux; U; Android 3.2; zh-cn; GT-P7500 Build HTJ85B) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S24382[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P7500V Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S24383[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-; SAMSUNG-GT-P7500/P7500BUKL1 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S24384[] = "Mozilla/5.0 (Linux; U; Android 3.2; en_ph-; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S24385[] = "Logos/2.1.2 (Linux; U; Android 3.2; pt-BR; GT-P7500 Build/HTJ85B.UBKL3)";
const char S24386[] = "BBC News (AndroidApp; 1.0; 1) GT-P7500 (Android 3.2, SDK 13)";
const char S24387[] = "Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24388[] = "Logos/2.1.2 (Linux; U; Android 3.2; es-US; GT-P7500 Build/HTJ85B.UBKJ1)";
const char S24389[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.11;FBDM/{density=1.0,width=1280,height=752};FBLC/en_US;FBCR/Smart;FBPN/com.facebook.katana;FBDV/GT-P7500;FBSV/3.2;]";
const char S24390[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7500 Build/HTJ85B) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S24391[] = "Biblia/2.2.3 (Linux; U; Android 3.2; es-ES; GT-P7500 Build/HTJ85B.UBKL1)";
const char S24392[] = "Logos/2.2.3 (Linux; U; Android 3.2; pt-BR; GT-P7500 Build/HTJ85B.UBLB2)";
const char S24393[] = "Mozilla/5.0 (Linux; U; Android 3.2; ms-my; GT-P7500 Build/HTJ85B) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S24394[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7500 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24395[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-ca; GT-P7500R Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.11;FBDM/{density=1.0,width=1280,height=752};FBLC/en_CA;FB_FW/2;FBCR/ROGERS;FBPN/com.facebook.katana;FBDV/GT-P7500R;FBSV/3.2;]";
const char S24396[] = "Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 3.2; SAMSUNG GT-P7500R Build/HONEYCOMB_MR2);";
const char S24397[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.11;FBDM/{density=1.0,width=1280,height=752};FBLC/en_GB;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P7500;FBSV/3.2;]";
const char S24398[] = "MediaPlayer/2002/4.0.12 (Android 3.2; GT-P7500; GT-P7500; samsung; ; ; en_GB)";
const char S24399[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.6.0;FBPN/com.facebook.katana;FBDV/GT-P7500;FBSV/3.2;FBDM/{density=1.0,width=1280,height=752};FBCR/MY MAXIS;FBLC/en_GB;FB_FW/1;]";
const char S24400[] = "GoogleAnalytics/2.0 (Linux; U; Android 3.2; en-ca; GT-P7500R Build/HTJ85B)";
const char S24401[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7500 Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S24402[] = "GroupMe-Android/3.3.5 (Android 3.2; en_US; samsung GT-P7500 Build/HTJ85B)";
const char S24403[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/2.0;FBDM/{density=1.0,width=1280,height=752};FBLC/en_US;FB_FW/2;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P7500;FBSV/3.2;]";
const char S24404[] = "MediaPlayer/2002/4.1.0 (Android 3.2; GT-P7500; GT-P7500; samsung; ; ; en_US)";
const char S24405[] = "Android (Linux; U; Android 3.2; fa-fa; GT-P7500) AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S24406[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; GT-P7500 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S24407[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7500 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24408[] = "MediaPlayer/2002/4.2.0 (Android 3.2; GT-P7500; GT-P7500; samsung; ; ; ms_MY)";
const char S24409[] = "Dalvik 2.05.12114; (Linux; U; Android 3.2; GT-P7500R; ON) 372b581e949b1424; 1280x752x1.0";
const char S24410[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7500/P7500BUKL1T Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S24411[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-ca; GT-P7500R Build/HTJ85B) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S24412[] = "VKAndroidApp/2.7.6-64 (Android 3.2; SDK 13; armeabi-v7a; samsung GT-P7500; en)";
const char S24413[] = "BBC News (AndroidApp; 1.0; 5) GT-P7500R (Android 3.2, SDK 13)";
const char S24414[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-ca; GT-P7500R) CBC News/1.0.5";
const char S24415[] = "Groupon/2.0.2414 (Android 3.2; Samsung GT-P7500R / Samsung GT-P7500R; ROGERS)[preload=false;locale=en_CA;clientidbase=android-samsung]";
const char S24416[] = "Mozilla/5.0 (Linux; U; Android 3.2; fr-ca; GT-P7500V Build/HTJ85B) GameCenter/3.0206";
const char S24417[] = "Mozilla/5.0 (Linux; U; Android 3.2; HTJ85B; samsung; GT-P7500R; en-us; dream) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S24418[] = "BBC News (AndroidApp; 1.1; 9) GT-P7500R (Android 3.2, SDK 13)";
const char S24419[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7500R Build/HTJ85B) [FBAN/Orca-Android;FBAV/2.3.2-release;FBLC/en_CA;FBBV/X;FBCR/null;FBMF/samsung;FBBD/samsung;FBDV/GT-P7500R;FBSV/3.2]";
const char S24420[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24421[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B329 Safari/8536.25";
const char S24422[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B146 Safari/8536.25";
const char S24423[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B141 Safari/8536.25";
const char S24424[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B147 Safari/8536.25";
const char S24425[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-P1000M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24426[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24427[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000 (Seetah) Build/Seetah) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24428[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-P1000R Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24429[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000N Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24430[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; GT-P1000T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24431[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; GT-P1000L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24432[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-P1000 Build/GINGERBREAD)";
const char S24433[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-P1000L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24434[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-P1000N Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24435[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-P1000 Build/GINGERBREAD)";
const char S24436[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24437[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000 Build/GR144) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24438[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24439[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24440[] = "Mozilla/5.0 (Linux; U; es-es; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S24441[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-P1000N Build/GINGERBREAD)";
const char S24442[] = "Mozilla/5.0 (Linux; U; it-it; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S24443[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24444[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S24445[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-P1000T Build/GINGERBREAD)";
const char S24446[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-P1000M Build/GINGERBREAD)";
const char S24447[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-P1000R Build/GINGERBREAD)";
const char S24448[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24449[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-P1000 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S24450[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24451[] = "Mozilla/5.0 (Linux; U; de-de; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S24452[] = "Mozilla/5.0 (Linux; U; pt-pt; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S24453[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-P1000/P1000BOJU3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24454[] = "Mozilla/5.0 (Linux; U; fr-fr; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S24455[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24456[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-P1000T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24457[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-P1000 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24458[] = "Mozilla/5.0 (Linux; U; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S24459[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.1; GT-P1000 Build/GINGERBREAD)";
const char S24460[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-P1000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24461[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-P1000L Build/GINGERBREAD)";
const char S24462[] = "Mozilla/5.0(Linux;U;Android2.3.3;Zh_cn;GT-P1000;600*1024;)AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S24463[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24464[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-ca; GT-P1000 Build/BOCA) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24465[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en_gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)";
const char S24466[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT-P1000 Build/GR144) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24467[] = "Mozilla/5.0 (Linux; U; Android2.3.3; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24468[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-P1000T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24469[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; GT-P1000 Build/GINGERBREAD)";
const char S24470[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-P1000 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24471[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24472[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-P1000 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S24473[] = "Logos/0.9.4 (Linux; U; Android 2.3.3; en_GB; GT-P1000 Build/Genesis 3.0 update 2.1.1)";
const char S24474[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; GT-P1000 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24475[] = "Android/1.0 (GT-P1000 GINGERBREAD)";
const char S24476[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-P1000 Build/GINGERBREAD)";
const char S24477[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24478[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-P1000 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S24479[] = "Mozilla 5.0 (Linux; U; Android 2.3.3; zh-cn; GT-P1000 Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S24480[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24481[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-P1000N Build/GINGERBREAD)";
const char S24482[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-P1000L Build/GINGERBREAD)";
const char S24483[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-P1000 Build/GINGERBREAD)";
const char S24484[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24485[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24486[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SAMSUNG GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24487[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-P1000 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24488[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 [FBAN/FB4A;FBAV/1.9.3;FBDM/{density=1.5,width=600,height=1024};FBLC/en_GB;FB_FW/2;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P1000;FBSV/2.3.3;]";
const char S24489[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-P1000 Build/GINGERBREAD; 600*1024) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S24490[] = "BBC News (AndroidApp; 1.0; 1) GT-P1000 (Android 2.3.3, SDK 10)";
const char S24491[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-P1000 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24492[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000T Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S24493[] = "Logos/2.1.2 (Linux; U; Android 2.3.3; en-GB; GT-P1000 Build/GINGERBREAD.DXJPJ)";
const char S24494[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; GT-P1000 Build/BOCA) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24495[] = "Logos/2.1.2 (Linux; U; Android 2.3.3; en-ZA; GT-P1000 Build/GINGERBREAD.XXJQ1)";
const char S24496[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24497[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; GT-P1000T Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24498[] = "Biblia/2.1.2 (Linux; U; Android 2.3.3; es-US; GT-P1000N Build/GINGERBREAD.UBJP7)";
const char S24499[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-P1000 Build/GalnetMIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24500[] = "Logos/2.1.2 (Linux; U; Android 2.3.3; es-ES; GT-P1000 Build/GINGERBREAD.XWJPA)";
const char S24501[] = "Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24502[] = "Logos/2.1.2 (Linux; U; Android 2.3.3; en-ZA; GT-P1000 Build/GINGERBREAD.XWJPE)";
const char S24503[] = "Vyrso/2.1.2 (Linux; U; Android 2.3.3; en-US; GT-P1000 Build/GINGERBREAD.XWJP9)";
const char S24504[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en_us; GT-P1000 Build/NFP91) AppleWebKit/534.15.12 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.15.12";
const char S24505[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S24506[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000) antivirusAM/2.4.5 (KHTML, like Gecko) Version/Version 2.4.5/antivirus";
const char S24507[] = "Logos/2.1.2 (Linux; U; Android 2.3.3; en-GB; GT-P1000 Build/GINGERBREAD.JPJPD)";
const char S24508[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S24509[] = "Logos/2.1.2 (Linux; U; Android 2.3.3; en-GB; GT-P1000 Build/GINGERBREAD.DXJPE)";
const char S24510[] = "Logos/2.1.2 (Linux; U; Android 2.3.3; zh-HK; GT-P1000 Build/GINGERBREAD.ZHJP7)";
const char S24511[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-P1000L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24512[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-P1000 Build/7G21C0)";
const char S24513[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24514[] = "Vyrso/2.1.2 (Linux; U; Android 2.3.3; en-AU; GT-P1000 Build/GINGERBREAD.XWJPJ)";
const char S24515[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-us; GT-P1000L Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S24516[] = "MediaPlayer/2002/4.0.12 (Android 2.3.3; GT-P1000; GT-P1000; samsung; ; ; en_US)";
const char S24517[] = "Mozilla/5.0 (Linux; U; Android BadBoyBunny; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24518[] = "BBC News (AndroidApp; 1.3.3; 15) GT-P1000 (Android 2.3.3, SDK 10)";
const char S24519[] = "Vyrso/2.2.3 (Linux; U; Android 2.3.6; en-GB; GT-P1000 Build/Overcome 7 Series v4.1.0)";
const char S24520[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24521[] = "Logos/2.0.11 (Linux; U; Android 2.3.6; en-AU; GT-P1000 Build/GINGERBREAD.XWJU5)";
const char S24522[] = "GoogleAnalytics/2.0 (Linux; U; Android 2.3.3; en-us; GT-P1000N Build/GINGERBREAD)";
const char S24523[] = "com.google.android.youtube/2.3.4(Linux; U; Android 2.3.3; en_US; GT-P1000 Build/GINGERBREAD) gzip";
const char S24524[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ms-my; GT-P1000 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S24525[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 [FBAN/FB4A;FBAV/1.6.3;FBPN/com.facebook.katana;FBDV/GT-P1000;FBSV/2.3.3;FBDM/{density=1.5,width=600,height=1024};FBCR/MY MAXIS;FBLC/en_GB;FB_FW/1;]";
const char S24526[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; Galaxy-Tab-GT-P1000 Build/RP1VR6)";
const char S24527[] = "Vyrso/0.9.4 (Linux; U; Android 2.3.3; en_PH; GT-P1000 Build/GINGERBREAD.DXJPI)";
const char S24528[] = "BBC News (AndroidApp; 1.3; 4) GT-P1000T (Android 2.3.3, SDK 10)";
const char S24529[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-P1000) antivirusHttpClient/2.0 (KHTML, like Gecko) Version/Version 2.1/antivirus";
const char S24530[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24531[] = "Rdio.Net; Android/2.4.2; Android/4.1.2; DM=samsung; DN=GT-P1000; DC=samsung; DF=samsung/GT-P1000/GT-P1000:2.3.5/GINGERBREAD/XXJVT:user/release-keys; DH=p1";
const char S24532[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-P1000 Build/GFAN_ROM_X) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24533[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24534[] = "Logos/2.2.7 (Linux; U; Android 2.3.3; zh-TW; GT-P1000 Build/GINGERBREAD.ZUJP8)";
const char S24535[] = "BBC News (AndroidApp; 1.01; 2) GT-P1000 (Android 2.3.3, SDK 10) flash:11.1.102.59";
const char S24536[] = "Biblia/2.2.3 (Linux; U; Android 2.3.6; es-US; GT-P1000N Build/GINGERBREAD.UBJR4)";
const char S24537[] = "Logos/2.2.7 (Linux; U; Android 2.3.6; zh-HK; GT-P1000 Build/GINGERBREAD.ZHJQ5)";
const char S24538[] = "Biblia/2.2.7 (Linux; U; Android 2.3.3; es-US; GT-P1000L Build/HumberOS)";
const char S24539[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-P1000 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S24540[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; GT-P1000 Build/GINGERBREAD.XWJQ8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24541[] = "GoogleAnalytics/2.0 (Linux; U; Android 2.3.6; es-us; GT-P1000N Build/GINGERBREAD)";
const char S24542[] = "Biblia/2.2.8.1 (Linux; U; Android 2.3.3; es-US; GT-P1000N Build/GINGERBREAD.VFJP6)";
const char S24543[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-P1000R) BBC News/1.3";
const char S24544[] = "BBC News (AndroidApp; 1.3; 4) GT-P1000R (Android 2.3.3, SDK 10)";
const char S24545[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-P1000T Build/GINGERBREAD)";
const char S24546[] = "MediaPlayer/0000/3.0 (Android 2.3.3; GT-P1000; GT-P1000; samsung; ; ; en_GB)";
const char S24547[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; GT-P1000 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24548[] = "Logos/2.2.8.1 (Linux; U; Android 2.3.6; de-DE; GT-P1000 Build/GINGERBREAD.XWJQ8)";
const char S24549[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGke Gecko) Version/4.0 Mobile Safari/533.1";
const char S24550[] = "VKAndroidApp/2.7.6-64 (Android 2.3.6; SDK 10; armeabi-v7a; samsung GT-P1000; ru)";
const char S24551[] = "BBC News (AndroidApp; 2.0; 19) GT-P1000 (Android 2.3.3, SDK 10)";
const char S24552[] = "FaithlifeStudyBible/2.2.8.1 (Linux; U; Android 2.3.3; en-CA; GT-P1000R Build/GINGERBREAD.UXKG1)";
const char S24553[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-P1000M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24554[] = "Groupon/2.2.2561 (Android 2.3.3; Samsung GT-P1000 / Samsung GT-P1000; )[preload=false;locale=en_GB;clientidbase=android-samsung]";
const char S24555[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-P1000 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24556[] = "Logos/2.2.8.1 (Linux; U; Android 2.3.3; en-US; GT-P1000 Build/Overcome v3.1.0-KRATOS)";
const char S24557[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000T Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24558[] = "Android/2.3.3; samsung GT-P1000; 2.3.2";
const char S24559[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-P1000L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24560[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-P1000 Build/GINGERBREAD) Daily Mail Online/1.8.3";
const char S24561[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en_us; GT-P1000N Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)";
const char S24562[] = "BBC News (AndroidApp; 1.0; 1) GT-P1000R (Android 2.3.3, SDK 10)";
const char S24563[] = "com.google.android.youtube/4.2.16(Linux; U; Android 2.3.3; en_CA; GT-P1000R Build/GINGERBREAD) gzip";
const char S24564[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.0 Version/4.0 Mobile Safari/530.17";
const char S24565[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S24566[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000N) AntivirusAM/2.5 (KHTML, like Gecko) Version/Version 2.5/Antivirus";
const char S24567[] = "VKAndroidApp/2.7.6-64 (Android 2.3.6; SDK 10; armeabi-v7a; samsung GT-P1000; en)";
const char S24568[] = "Logos/2.2.10 (Linux; U; Android 2.3.3; en-US; GT-P1000 Build/GINGERBREAD.ZTJP8)";
const char S24569[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000T Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.0 Version/4.0 Mobile Safari/530.17";
const char S24570[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; cs-cz; GT-P1000L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24571[] = "nApps (Android 2.3.3; GT-P1000R; %EB%84%A4%EC%9D%B4%EB%B2%84+%EC%B9%B4%EB%A9%94%EB%9D%BC; 1.8.2)";
const char S24572[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hr-hr; SAMSUNG GT-P1000/P1000BOJU2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24573[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 baidubrowser/042_1.41.3.2_diordna_4201_006/gnusmas_01_4.3.2_0001P-TG/Xb/59E3D8A74D6B206FBB45E5A5C4C356D6%7CX/1";
const char S24574[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-P1000M) CBC News/1.0.5";
const char S24575[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-P1000R Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S24576[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.7 Mobile Safari/534.30";
const char S24577[] = "Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; GT-P1000 Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S24578[] = "Dalvik 2.11.12449; (Linux; U; Android 2.3.3; GT-P1000; ON) c1b9a046dccce90a; 400x682x1.5";
const char S24579[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-; GT-P1000N Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24580[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; SAMSUNG GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24581[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-P1000L Build/GINGERBREAD) [FBAN/Orca-Android;FBAV/2.3.1-release;FBLC/en_US;FBBV/X;FBCR/chatr;FBMF/samsung;FBBD/samsung;FBDV/GT-P1000L;FBSV/2.3.3]";
const char S24582[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 INTSafari/534.30";
const char S24583[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; GT-P1000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24584[] = "Mozilla/5.0 (Linux; U; Android 2.3.6 E-3F 'A1H4G; en-us; GT-P1000  by mohsen afrousheh Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24585[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-; GT-P1000N Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24586[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-P1000 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S24587[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; HTC Sensation XE with Beats Audio Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24588[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-gb; HTC Sensation XE with Beats Audio Z715e Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24589[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SensationXE_Beats_Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24590[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Sensation XE with Beats Audio Build/IML74K)";
const char S24591[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24592[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24593[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-sg; HTC_SensationXE_Beats_Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24594[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Sensation XE with Beats Audio Z715e Build/IML74K)";
const char S24595[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_SensationXE_Beats Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24596[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XE with Beats Audio Z715e Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S24597[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24598[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC_SensationXE_Beats_Z715a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24599[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC Sensation XE with Beats Audio Z715a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24600[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XE with Beats Audio Z715e Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24601[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation XE Build/SmartDroid Berserk Line) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24602[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24603[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; HTC_SensationXE_Beats_Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24604[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Sensation XE with Beats Audio Z715a Build/IML74K)";
const char S24605[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC_Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24606[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_SensationXE_Beats/1.0 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24607[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Z715e Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24608[] = "Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XE with Beats Audio Z715e Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S24609[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_SensationXE_Beats_Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24610[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24611[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Sensation_XE_with_Beats_Audio_Z715e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24612[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Sensation XE with Beats Audio Z715e Build/IML74K)";
const char S24613[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_SensationXE_Beats_Z715e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24614[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24615[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:18.0) Gecko/18.0 Firefox/18.0";
const char S24616[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:18.0) Gecko/X Firefox/18.0";
const char S24617[] = "Mozilla/5.0 (Windows; Windows NT 6.1; Win64; x64; rv:18.0) Gecko/18.0 Firefox/18.0";
const char S24618[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; rv:11.0) Gecko/X Firefox/18.0";
const char S24619[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24620[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24621[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24622[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24623[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24624[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24625[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24626[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24627[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105T Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24628[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24629[] = "Mozilla/5.0 (Linux; Android 4.1.1; SPH-L900 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24630[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24631[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24632[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I317M Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24633[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889V Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24634[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105T Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24635[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24636[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-R950 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24637[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24638[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24639[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I317 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24640[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24641[] = "Mozilla/5.0 (Linux; Android 4.1.1; SC-02E Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24642[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24643[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24644[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R950 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24645[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24646[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24647[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24648[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24649[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24650[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24651[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100T Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24652[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24653[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R950 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24654[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24655[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-N719 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24656[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889V Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24657[] = "Mozilla/5.0 (Linux; Android 4.1.1; SPH-L900 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24658[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I317 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24659[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889V Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24660[] = "Mozilla/5.0 (Linux; Android 4.1.1; SHV-E250S Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24661[] = "Mozilla/5.0 (Linux; Android 4.1.1; SHV-E250K Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24662[] = "Mozilla/5.0 (Linux; Android 4.1.9; GT-N7100 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24663[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I317M Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24664[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A5313e";
const char S24665[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A5288d";
const char S24666[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A5302b";
const char S24667[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334";
const char S24668[] = "Mozilla/5.0 (iPad; U; CPU iPad OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334";
const char S24669[] = "MQQBrowser/18 Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Safari/7534.48.3";
const char S24670[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)";
const char S24671[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405";
const char S24672[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)";
const char S24673[] = "MQQBrowser/1.9.1 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S24674[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/525.20";
const char S24675[] = "MQQBrowser/19 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S24676[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 5_0_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1;FBBV/4100.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/5.0.1;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]";
const char S24677[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 5_0 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1;FBBV/4100.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/5.0;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]";
const char S24678[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A40";
const char S24679[] = "MQQBrowser/18 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S24680[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48 Instapaper/4.1.1 (+http://www.instapaper.com/iphone)";
const char S24681[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9Z999";
const char S24682[] = "MQQBrowser/1.9.1 Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Safari/7534.48.3";
const char S24683[] = "MQQBrowser/10 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405";
const char S24684[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/5.0.1;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]";
const char S24685[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/5.0;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]";
const char S24686[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 iPad2,1__weibo__3.1.1__ipad__os5.0.1";
const char S24687[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/8536.26.7 DiigoBrowser";
const char S24688[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176";
const char S24689[] = "Lynx/2.8.6rel.4 libwww-FM/2.14 SSL-MM/1.4.1 GNUTLS/2.0.4";
const char S24690[] = "Lynx/2.8.7dev.9 libwww-FM/2.14 SSL-MM/1.4.1";
const char S24691[] = "Lynx/2.8.8dev.2 libwww-FM/2.14 SSL-MM/1.4.1";
const char S24692[] = "Lynx/2.8.5rel.1 libwww-FM/2.14 SSL-MM/1.4.1 OpenSSL/0.9.8e-fips-rhel5";
const char S24693[] = "Lynx/2.8.3dev.9 libwww-FM/2.14 SSL-MM/1.4.1 OpenSSL/0.9.6";
const char S24694[] = "Lynx/2.8.6rel.4 libwww-FM/2.14 SSL-MM/1.4.1 OpenSSL/0.9.8g";
const char S24695[] = "Lynx/2.8.8dev.5 libwww-FM/2.14 SSL-MM/1.4.1 GNUTLS/2.8.6";
const char S24696[] = "Lynx/2.8.8dev.3 libwww-FM/2.14 SSL-MM/1.4.1";
const char S24697[] = "Lynx/2.8.8dev.9 libwww-FM/2.14 SSL-MM/1.4.1 GNUTLS/2.10.5";
const char S24698[] = "Lynx/2.8.5rel.1 libwww-FM/2.14FM SSL-MM/1.4.1 OpenSSL/0.9.7d-dev";
const char S24699[] = "Lynx/2.8.6rel.5 libwww-FM/2.14 SSL-MM/1.4.1 OpenSSL/1.0.0-fips";
const char S24700[] = "Lynx/2.8.7rel.1 libwww-FM/2.14 SSL-MM/1.4.1 OpenSSL/1.0.1c";
const char S24701[] = "Lynx/2.8.5rel.1 libwww-FM/2.15FC SSL-MM/1.4.1c OpenSSL/0.9.7e-dev";
const char S24702[] = "Lynx/2.8.8dev.9 libwww-FM/2.14 SSL-MM/1.4.1 GNUTLS/2.12.14";
const char S24703[] = "Lynx/2.8.6rel.5 libwww-FM/2.14 SSL-MM/1.4.1 OpenSSL/0.9.8h";
const char S24704[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24705[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24706[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-I535 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24707[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-I535 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24708[] = "Mozilla/5.0 (Linux; Android 4.1.9; GT-I9300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S24709[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24710[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24711[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24712[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24713[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999V Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24714[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24715[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24716[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300T Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24717[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24718[] = "Mozilla/5.0 (Linux; Android 4.1.9; GT-I9300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24719[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24720[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24721[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I93000 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24722[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999V Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24723[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-i9300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24724[] = "Mozilla/5.0 (Linux; Android 4.1.1; I9300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24725[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S24726[] = "Mozilla/5.0 (Linux; Android 4.1; i9300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24727[] = "Mozilla/5.0 (Linux; Android 4.1.2; SC-06D Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24728[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999V Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S24729[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S24730[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S24731[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S24732[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S24733[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S24734[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S24735[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S24736[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S24737[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)";
const char S24738[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.90 Safari/537.1";
const char S24739[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via docs.google.com/viewer)";
const char S24740[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1060) AppleWebKit/537.1 (KHTML like Gecko) Chrome/21.0.1180.79 Safari/537.1";
const char S24741[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/21.0.648.205 Safari/534.16";
const char S24742[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/21.0.648.205 Safari/534.16";
const char S24743[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/21.0.648.205 Safari/534.16";
const char S24744[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16";
const char S24745[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16";
const char S24746[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S24747[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.127 Safari/534.16";
const char S24748[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.133 Safari/534.16";
const char S24749[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16";
const char S24750[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.133 Safari/534.16";
const char S24751[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.639.0 Safari/534.16";
const char S24752[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.11 Safari/534.16";
const char S24753[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X  10_6_8;  fr_US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.3 idtz";
const char S24754[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X  10_6_4;  fr_US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/10.0.648.134 Safari/534.3 idtz";
const char S24755[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; de-DE) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S24756[] = "Opera/9.80 (X11; Linux i686; U; en) Presto/2.7.62 Version/11.00Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16";
const char S24757[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; LT26i Build/6.0.A.3.67) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24758[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonLT26i-o Build/6.0.A.3.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24759[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonLT26i-o Build/6.0.A.3.73) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24760[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonLT26i Build/6.0.A.3.67) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24761[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; LT26i Build/6.0.A.3.73) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24762[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; SonyEricssonLT26i-o Build/6.0.A.3.67) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24763[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; LT26i Build/6.0.A.3.62)";
const char S24764[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; SonyEricssonLT26i Build/6.0.A.3.73) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24765[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT26i Build/6.0.A.3.62) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S24766[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-mx; SonyEricssonLT26i Build/6.0.A.3.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24767[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-mx; LT26i Build/6.0.A.3.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24768[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; SonyEricssonLT26i Build/6.0.A.3.76) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24769[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; LT26i Build/6.0.A.3.76) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24770[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonLT26i Build/6.0.A.3.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24771[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; LT26i Build/6.0.A.3.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24772[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; SonyEricssonLT26i-o Build/6.0.A.3.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24773[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT26i Build/6.0.A.3.75) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24774[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT26i Build/6.0.A.3.73) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24775[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; SonyEricssonLT26i Build/6.0.A.1.249) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24776[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT26i Build/6.0.A.3.67) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24777[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; fr-be; SonyEricssonLT26iv Build/6.0.A.3.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24778[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT26i Build/6.0.A.3.62) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24779[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT26i Build/6.0.A.3.67) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S24780[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT26i Build/6.0.A.3.73) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S24781[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT26i Build/6.0.A.3.76) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24782[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; LT26i Build/6.0.A.1.249) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24783[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9790; en-GB) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.714 Mobile Safari/534.11+";
const char S24784[] = "Mozilla/5.0 (X11; U; Linux x86_64; fr-BE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9737IT";
const char S24785[] = "Mozilla/5.0 (X11; U; Linux x86_64; zh-CN) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9737IT";
const char S24786[] = "Mozilla/5.0 (X11; U; Linux x86_64; zh-HK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9737IT";
const char S24787[] = "Mozilla/5.0 (X11; U; Linux x86_64; fr-CH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9737IT";
const char S24788[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-ZA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9737IT";
const char S24789[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-TT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9737IT";
const char S24790[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-BE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9737IT";
const char S24791[] = "Mozilla/5.0 (X11; U; Linux x86_64; id-ID) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9737IT";
const char S24792[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-NG) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9737IT";
const char S24793[] = "Mozilla/5.0 (X11; U; Linux x86_64; de-AT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24794[] = "Mozilla/5.0 (X11; U; Linux x86_64; zh-TW) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24795[] = "Mozilla/5.0 (X11; U; Linux x86_64; de-DE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24796[] = "Mozilla/5.0 (X11; U; Linux x86_64; it-IT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24797[] = "Mozilla/5.0 (X11; U; Linux x86_64; ru-RU) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24798[] = "Mozilla/5.0 (X11; U; Linux x86_64; pl-PL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24799[] = "Mozilla/5.0 (X11; U; Linux x86_64; vi-VN) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IP";
const char S24800[] = "Mozilla/5.0 (X11; U; Linux x86_64; tr-TR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24801[] = "Mozilla/5.0 (X11; U; Linux x86_64; nl-NL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24802[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-ES) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24803[] = "Mozilla/5.0 (X11; U; Linux x86_64; ja-JP) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24804[] = "Mozilla/5.0 (X11; U; Linux x86_64; pt-BR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24805[] = "Mozilla/5.0 (X11; U; Linux x86_64; fr-FR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24806[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-IE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24807[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-CA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24808[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-MX) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24809[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-NZ) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24810[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24811[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-AU) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24812[] = "Mozilla/5.0 (X11; U; Linux x86_64; he-IL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24813[] = "Mozilla/5.0 (X11; U; Linux x86_64; sv-SE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24814[] = "Mozilla/5.0 (X11; U; Linux x86_64; fr-CA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24815[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-SG) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24816[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-HK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24817[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-GB) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24818[] = "Mozilla/5.0 (X11; U; Linux x86_64; nl-BE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24819[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-IN) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24820[] = "Mozilla/5.0 (X11; U; Linux x86_64; th-TH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24821[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-CO) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24822[] = "Mozilla/5.0 (X11; U; Linux x86_64; el-GR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24823[] = "Mozilla/5.0 (X11; U; Linux x86_64; pt-PT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24824[] = "Mozilla/5.0 (X11; U; Linux x86_64; ko-KR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24825[] = "Mozilla/5.0 (X11; U; Linux x86_64; da-DK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24826[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-PH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24827[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24828[] = "Mozilla/5.0 (X11; U; Linux x86_64; de-CH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24829[] = "Mozilla/5.0 (X11; U; Linux x86_64; fa-IR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24830[] = "Mozilla/5.0 (X11; U; Linux x86_64; nb-NO) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24831[] = "Mozilla/5.0 (X11; U; Linux x86_64; ar-SA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24832[] = "Mozilla/5.0 (X11; U; Linux x86_64; fi-FI) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24833[] = "Mozilla/5.0 (X11; U; Linux x86_64; ru-UA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24834[] = "Mozilla/5.0 (X11; U; Linux x86_64; ar-KW) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IP";
const char S24835[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-EC) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24836[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-PK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IP";
const char S24837[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-VE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24838[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-AR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24839[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-PR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24840[] = "Mozilla/5.0 (X11; U; Linux x86_64; ms-MY) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24841[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-CL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24842[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-PE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24843[] = "Mozilla/5.0 (X11; U; Linux x86_64; ca-ES) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24844[] = "Mozilla/5.0 (X11; U; Linux x86_64; ar-AE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24845[] = "Mozilla/5.0 (X11; U; Linux x86_64; en) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24846[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-DO) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT";
const char S24847[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-AS) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IP";
const char S24848[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; et-ee; LG-P500 Build/FRG83) AppleWebKit/528.5+ (KHTML, nagu Gecko) Versioon/3.1.2 Mobile Safari/525.20,1";
const char S24849[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; pt-; LG-P500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S24850[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P500/v10q Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S24851[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; LG-P500/v10p Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S24852[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; it-it; LG-P500/V10b Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S24853[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; LG-P500h Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S24854[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ko-; LG-P500h Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S24855[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; LG-P500h-parrot Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S24856[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S24857[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; LG-P500/v10a Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S24858[] = "Mozilla/5.0 (Linux; U; Android 2.2; ru-ru; LG-P500 Build/FRF91) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S24859[] = "Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; LG-P500-Bouygues/Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S24860[] = "Mozilla/5.0 (Linux; U; Android 2.2; it-it; LG-P500/V10a Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S24861[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; LG-P500 Build/FRG83) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S24862[] = "Mozilla/5.0(Linux;U;Android2.2;Zh_cn;LG-P500;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S24863[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pt-; LG-P500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S24864[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; LG-P500-Bouygues/ Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S24865[] = "Mozilla/5.0(Linux;U;Android2.2.2;Zh_cn;LG-P500;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S24866[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; LG-P500 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S24867[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P500h Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S24868[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pt-; LG-P500h-parrot Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S24869[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-ca; LG-P500h-parrot Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S24870[] = "Mozilla/5.0 (Linux; U; Android 2.2; ko-; LG-P500h Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S24871[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-sa; LG-P500h Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S24872[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; LG Optimus One P500 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24873[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; LG-P500h Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24874[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; LG-P500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S24875[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; LG-P500 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S24876[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; LG-P500h Build/FRG83) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S24877[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; LG-P500 Build/FRF91)";
const char S24878[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; LG-P500 Build/FRG83)";
const char S24879[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P500 Build/FRG83)";
const char S24880[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; LG-P500h Build/FRF91)";
const char S24881[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; LG-P500h Build/FRG83)";
const char S24882[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; LG-P500 Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S24883[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-in; LG-P500 Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S24884[] = "Mozilla/5.0(Linux; U; Android 2.2; en-gb; LG-P500 Build/FRF91) AppleWebKit/533.0 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.";
const char S24885[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N8000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S24886[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-N8000 Build/IMM76D)";
const char S24887[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-N8000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24888[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-N8000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S24889[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; GT-N8005 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S24890[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SAMSUNG GT-N8000/N8000XWALG9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S24891[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-N8000 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24892[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16";
const char S24893[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7C144 Safari/528.16";
const char S24894[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7D11 Safari/528.16";
const char S24895[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7C116a Safari/528.16";
const char S24896[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7C145 Safari/528.16";
const char S24897[] = "Mozilla/5.0 (device; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16";
const char S24898[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/9Z999 Safari/528.16";
const char S24899[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/PV02 Safari/528.16";
const char S24900[] = "Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16";
const char S24901[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S24902[] = "Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16";
const char S24903[] = "Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; de-de) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16";
const char S24904[] = "Mozilla/5.0 (device; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7D11 Safari/528.16";
const char S24905[] = "Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; it-it) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16";
const char S24906[] = "Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; es-es) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16";
const char S24907[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7C97d Safari/528.16";
const char S24908[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/9Z999 Safari/528.16";
const char S24909[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/PV01 Safari/528.16";
const char S24910[] = "Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; fr-fr) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16";
const char S24911[] = "Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; nl-nl) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16";
const char S24912[] = "Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; ru-ru) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16";
const char S24913[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7D11 Safari/528.16";
const char S24914[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 312 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML like Gecko) Version/4.0 Mobile/7D11 Safari/528.16";
const char S24915[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 313 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML like Gecko) Version/4.0 Mobile/7E18 Safari/528.16";
const char S24916[] = "Mozilla/5.0 (heaveniphone; U; CPU iPhone OS 3_1_3 like Mac OS X; en) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16";
const char S24917[] = "Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; ar) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16";
const char S24918[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-tw; HTC_SensationXL_Beats_X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24919[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Sensation XL with Beats Audio X315e Build/IML74K)";
const char S24920[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SensationXL_Beats_X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24921[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24922[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC_SensationXL_Beats_X315b Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24923[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC Sensation XL with Beats Audio X315b Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24924[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC/SensationXL_Beats/2.32.161.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24925[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_SensationXL_Beats/2.31.111.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24926[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC_SensationXL_Beats-orange-LS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24927[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_SensationXL_Beats/2.36.111.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24928[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; Sensation XL with Beats Audio Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24929[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XL with Beats Audio X315e Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24930[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Sensation XL Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24931[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC Sensation XL with Beats Audio X315b Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24932[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Sensation XL with Beats Audio X315b Build/IML74K)";
const char S24933[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XL with Beats Audio X315e Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S24934[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_DesireHD_X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24935[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24936[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation XL with Beats Audio X315e Build/Vertigo's Total Senseless v4.1.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24937[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_TelusHD_Beats_X315e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24938[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC_X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24939[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; HTC_SensationXL_Beats_X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24940[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; SensationXL_Beats_X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24941[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SensationXL_Beats_X315e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24942[] = "Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XL with Beats Audio X315e Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S24943[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24944[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24945[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S24946[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S24947[] = "Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S24948[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B350 Safari/8536.25";
const char S24949[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B145 Safari/8536.25";
const char S24950[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B329 Safari/8536.25";
const char S24951[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B146 Safari/8536.25";
const char S24952[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B143 Safari/8536.25";
const char S24953[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B142 Safari/8536.25";
const char S24954[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B144 Safari/8536.25";
const char S24955[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B141 Safari/8536.25";
const char S24956[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9Z999 Safari/8536.25";
const char S24957[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24958[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-I8150 Build/GINGERBREAD)";
const char S24959[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24960[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-I8150T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24961[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I8150 Build/GINGERBREAD)";
const char S24962[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; GT-I8150B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24963[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S24964[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-I8150B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24965[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24966[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I8150T Build/GINGERBREAD)";
const char S24967[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I8150 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24968[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24969[] = "Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S24970[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S24971[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P5100 Build/JRO03C)";
const char S24972[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; SAMSUNG GT-P5100/P5100BUCLL1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S24973[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P5100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S24974[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T779 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S24975[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5100 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S24976[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5100 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S24977[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P5100 Build/JZO54K)";
const char S24978[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; GT-P5100 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24979[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; SAMSUNG GT-P5100/P5100BUCLJ1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S24980[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; GT-P5100 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S24981[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P5100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S24982[] = "FaithlifeStudyBible/2.2.10 (Linux; U; Android 4.1.1; en-ZA; GT-P5100 Build/JRO03C.P5100ZSCLL1)";
const char S24983[] = "com.google.android.youtube/4.1.47(Linux; U; Android 4.1.1; en_GB; GT-P5100 Build/JRO03C) gzip";
const char S24984[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-P5100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S24985[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S24986[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5100 Build/JRO03C) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S24987[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; sl-si; SAMSUNG GT-P5100/P5100BUCLJ2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S24988[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P5100 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S24989[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5100 Build/JRO03C) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.2 Version/4.0 Mobile Safari/530.17";
const char S24990[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; GT-P5100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30";
const char S24991[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-P5100/P5100BUDMA2.DM Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S24992[] = "nApps (Android 4.1.1; GT-P5100; androidapp.lineplay; 1.2.2)";
const char S24993[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5100 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S24994[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; SAMSUNG GT-P5100/P5100BUDMD1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S24995[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-P5100-ORANGE/P5100BVDMD1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S24996[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T779 Build/JZO54K)";
const char S24997[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-P5100-ORANGE/P5100BVDMF1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S24998[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P7500 Build/HTJ85B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S24999[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25000[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P7500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25001[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25002[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25003[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7500 Build/IMM76D)";
const char S25004[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25005[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25006[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7500R Build/IMM76D)";
const char S25007[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25008[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; SAMSUNG GT-P7500/P7500BULP5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25009[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SAMSUNG GT-P7500/P7500XWLPB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25010[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-P7500 Build/HTJ85B)";
const char S25011[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; GT-P7500R Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25012[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; GT-P7500D Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25013[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SAMSUNG GT-P7500/P7500BULP1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25014[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7500 Build/IMM76D; 1280*752) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S25015[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7500M Build/IMM76D)";
const char S25016[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25017[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7500D Build/IMM76D)";
const char S25018[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25019[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-P7500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.5 Mobile Safari/534.30";
const char S25020[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25021[] = "Logos/2.1.2 (Linux; U; Android 4.0.4; en-CA; GT-P7500R Build/IMM76D.RUXLH2)";
const char S25022[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/IMM76) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25023[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S25024[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25025[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S25026[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P7500 Build/TISU0093) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25027[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-; GT-P7500R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25028[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S25029[] = "Vyrso/2.1.2 (Linux; U; Android 4.0.4; zh-CN; GT-P7500 Build/IMM76D.ZSLP8)";
const char S25030[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.2.1-65 (Android 4.0.4; samsung GT-P7500 Build/IMM76D.XXLQ8)";
const char S25031[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sr-rs; SAMSUNG GT-P7500/P7500XWLP5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25032[] = "Logos/2.1.2 (Linux; U; Android 4.0.4; en-US; GT-P7500 Build/IMM76D.XWLP5)";
const char S25033[] = "com.google.android.youtube/4.0.23(Linux; U; Android 4.0.4; en_GB; GT-P7500 Build/IMM76D) gzip";
const char S25034[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500) antivirusHttpClient/2.0 (KHTML, like Gecko) Version/Version 2.3/Antivirus pro";
const char S25035[] = "nApps (Android 4.0.4; GT-P7500; Android.Line; 3.1.3)";
const char S25036[] = "Logos/2.1.2 (Linux; U; Android 4.0.4; nl-NL; GT-P7500 Build/IMM76D.XWLP6)";
const char S25037[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SAMSUNG GT-P7500/P7500BULP3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25038[] = "Logos/2.2.3 (Linux; U; Android 4.0.4; en-GB; GT-P7500 Build/IMM76D.XXLQ8)";
const char S25039[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25040[] = "Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 4.0.4; SAMSUNG GT-P7500 Build/ICE_CREAM_SANDWICH_MR1);";
const char S25041[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P7500-ORANGE/P7500BVLP6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25042[] = "GoogleAnalytics/2.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500R Build/IMM76D)";
const char S25043[] = "GoogleAnalytics/2.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/IML74K)";
const char S25044[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500R Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MantanoReader";
const char S25045[] = "GoogleAnalytics/2.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500D Build/IMM76D)";
const char S25046[] = "Logos/2.1.2 (Linux; U; Android 4.0.4; en-US; GT-P7500 Build/IMM76D.JPLPC)";
const char S25047[] = "com.google.android.apps.plus/X (Linux; U; Android 4.0.4; en_CA; GT-P7500R; Build/IMM76D) (gzip)";
const char S25048[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25049[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P7500 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S25050[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; SAMSUNG GT-P7500/P7500XWLP6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25051[] = "Logos/2.2.3 (Linux; U; Android 4.0.4; en-CA; GT-P7500D Build/IMM76D.DTLLH1)";
const char S25052[] = "Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 4.0.4; SAMSUNG GT-P7500R Build/ICE_CREAM_SANDWICH_MR1);";
const char S25053[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG GT-P7500/P7500BULP5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25054[] = "com.google.android.apps.plus/X (Linux; U; Android 4.0.4; en_CA; GT-P7500D; Build/IMM76D) (gzip)";
const char S25055[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25056[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; GT-P7500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25057[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S25058[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 4G Build/6.7.2-223_DBN_M4-23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25059[] = "MediaPlayer/2002/4.1.0 (Android 4.0.4; GT-P7500; GT-P7500; samsung; ; ; en_GB)";
const char S25060[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; GT-P7500 Build/HTJ85B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25061[] = "Scanner Radio Pro/3.9.0.2 (Android 4.0.4, GT-P7500D)";
const char S25062[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; GT-P7500 Build/TISU0124) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25063[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25064[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500R Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S25065[] = "Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-GB; GT-P7500 Build/IMM76D.XWLP9)";
const char S25066[] = "Hotmail/7.8.2.8 (Android 4.0.4; samsungGT-P7500R)";
const char S25067[] = "Dalvik 2.02.11978; (Linux; U; Android 4.0.4; GT-P7500) 3efd9aa46fac3720; 1280x752x1.0";
const char S25068[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P7500R Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S25069[] = "Scanner Radio Pro/3.9.1 (Android 4.0.4, GT-P7500D)";
const char S25070[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P7500-ORANGE/P7500BVLP8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25071[] = "FaithlifeStudyBible/2.2.8.1 (Linux; U; Android 4.0.4; en-US; GT-P7500 Build/IMM76D.DXLP9)";
const char S25072[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500-ORANGE/P7500BVLP3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25073[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P7500 Build/R13.4-rc2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25074[] = "Logos/2.2.10 (Linux; U; Android 4.0.4; en-US; GT-P7500M Build/IMM76D.MUGLH4)";
const char S25075[] = "Android/3.7.3 (Linux; U; Android 4.0.4; en_CA; GT-P7500R)";
const char S25076[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P7500 wI-fI & 3g Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25077[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P7500 Build/Xqm) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25078[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S25079[] = "Android/4.7 (Linux; U; Android 4.0.4; en_CA; GT-P7500M)";
const char S25080[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-P7500 Build/IMM76D) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.0 Version/4.0 Mobile Safari/530.17";
const char S25081[] = "com.google.android.youtube/4.1.47(Linux; U; Android 4.0.4; en_CA; GT-P7500M Build/IMM76D) gzip";
const char S25082[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7500 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S25083[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) Appleko) Version/4.0 Safari/534.30";
const char S25084[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P7500 WIFI_3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25085[] = "Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7500 Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S25086[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ms-my; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/2.2;FBBV/X;FBDM/{density=1.0,width=1280,height=752};FBLC/ms_MY;FB_FW/2;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P7500;FBSV/4.0.4;]";
const char S25087[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P7500 Build/TISU0143) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25088[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SAMSUNG GT-P7500/P7500BULP1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25089[] = "Mozilla/5.0 (Android 4.0.4; en-ca; GT-P7500R Build/IMM76D) FreeWheelAdManager/5.3.0-r9362-X;com.vevo/442";
const char S25090[] = "BBC News (AndroidApp; 1.1; 9) GT-P7500M (Android 4.0.4, SDK 15)";
const char S25091[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG GT-P7500/P7500BULP5.DM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25092[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500M Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MantanoReader";
const char S25093[] = "VKAndroidApp/2.7.6-64 (Android 4.0.4; SDK 15; armeabi-v7a; samsung GT-P7500; ru)";
const char S25094[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500D Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S25095[] = "APKXDL (Linux; U; Android 4.0.4;en_CA; GT-P7500R/IMM76D)com.sygic.aura";
const char S25096[] = "Groupon/2.3.2787 (Android 4.0.4; Samsung GT-P7500 / Samsung GT-P7500; )[preload=false;locale=en_US;clientidbase=android-samsung]";
const char S25097[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25098[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-P3110 Build/IML74K)";
const char S25099[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25100[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P3110 Build/IMM76D)";
const char S25101[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25102[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25103[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3110 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S25104[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3110 Build/IML74K; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S25105[] = "BBC iPlayer (AndroidApp; 1.3; 13014) GT-P3110 (Android 4.0.3, SDK 15)";
const char S25106[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S25107[] = "BBC News (AndroidApp; 1.3.3; 15) GT-P3110 (Android 4.0.4, SDK 15)";
const char S25108[] = "BBC iPlayer (AndroidApp; 1.3; 13014) GT-P3110 (Android 4.0.4, SDK 15)";
const char S25109[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25110[] = "BBC News (AndroidApp; 1.3.3; 15) GT-P3110 (Android 4.0.3, SDK 15)";
const char S25111[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30";
const char S25112[] = "FaithlifeStudyBible/2.1.2 (Linux; U; Android 4.0.3; en-PH; GT-P3110 Build/IML74K.P3110XXALD4)";
const char S25113[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.2.1-65 (Android 4.0.4; samsung GT-P3110 Build/IMM76D.P3110XXBLH2)";
const char S25114[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3110C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; 360 Aphone Browser (Version 2.6.3); 360browser(securitypay,securityinstalled)";
const char S25115[] = "Logos/2.2.3 (Linux; U; Android 4.0.3; en-AU; GT-P3110 Build/IML74K.P3110ZSALD5)";
const char S25116[] = "Logos/2.2.3 (Linux; U; Android 4.0.3; en-AU; GT-P3110 Build/IML74K.P3110XWALD2)";
const char S25117[] = "FaithlifeStudyBible/2.2.3 (Linux; U; Android 4.0.4; en-GB; GT-P3110 Build/IMM76D.P3110XXBLH2)";
const char S25118[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ph; GT-P3110 Build/IML74K) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S25119[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3110 Build/IMM76D; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S25120[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM7ategoryId\":\"Search\",\"Click\":0,\"Carted\":0,\"Purchased\":0,\"ProfileC4.30";
const char S25121[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3110 Build/IMM76D) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 baidubrowser/061_1.6.3.2_diordna_679_006/gnusmas_51_4.0.4_0113P-TG/Xa/7C3794EBF80E2CB90F3BA3BA4B6BD2CD%7C0/1";
const char S25122[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3110 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S25123[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3110 Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S25124[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25125[] = "com.google.android.youtube/3.5.5(Linux; U; Android 4.0.3; en_GB; GT-P3110 Build/IML74K) gzip";
const char S25126[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.3.4-94 (Android 4.0.3; samsung GT-P3110 Build/IML74K.P3110XXALD4)";
const char S25127[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 parameters/{scr=552_1024,cm=1,ql=l,imei=94:63:D1:FD:C1:AA,bm=1,eng=adrmini15,mdm=1,mdl=GT-P3110,ps=1} tiantian(securitypay) tiantian(tenpay)";
const char S25128[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, likX.X.X.X.1; __utmb=X.4.10.X; __utmc=X; __utmz=X.X.1.1.utmcsr=google|utmccn=(organic)|utmcmd=organic|utmctr=harveys";
const char S25129[] = "com.google.android.youtube/4.0.23(Linux; U; Android 4.0.4; en_GB; GT-P3110 Build/IMM76D) gzip";
const char S25130[] = "VKAndroidApp/2.7.6-64 (Android 4.0.4; SDK 15; armeabi-v7a; samsung GT-P3110; ru)";
const char S25131[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/5.0.2 Version/4.0 Safari/534.30 Safari/6533.18.5";
const char S25132[] = "BBC iPlayer (AndroidApp; 1.5.0.666; X) GT-P3110 (Android 4.0.4, SDK 15)";
const char S25133[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25134[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; GT-P3110 Build/IMM76D) Appecko) Version/4.0 Safari/534.30";
const char S25135[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fi-fi; GT-P3110 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S25136[] = "BBC iPlayer (AndroidApp; 1.5.0.666; X) GT-P3110 (Android 4.0.3, SDK 15)";
const char S25137[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IML76D) appleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25138[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25139[] = "Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:14.0) Gecko/X Firefox/14.0.1 Firefox/14.0.1";
const char S25140[] = "Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:14.0; ips-agent) Gecko/X Firefox/14.0.1";
const char S25141[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S25142[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:20.0) Gecko/X Firefox/20.0 AlexaToolbar/alxf-2.18";
const char S25143[] = "Mozilla/5.0 (Windows; Windows NT 6.1; Win64; x64; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S25144[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.1";
const char S25145[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.1-x64";
const char S25146[] = "Mozilla/5.0 (Windows; Windows NT 6.1; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S25147[] = "Mozilla/5.0 (Windows NT 6.1) Gecko/X Firefox/20.0";
const char S25148[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; MAXTHON 2.0)";
const char S25149[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; MAXTHON 2.0)";
const char S25150[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Maxthon 2.0)";
const char S25151[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; MAXTHON 2.0)";
const char S25152[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; Maxthon 2.0)";
const char S25153[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; Maxthon 2.0)";
const char S25154[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; MAXTHON 2.0)";
const char S25155[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; Maxthon 2.0)";
const char S25156[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; MAXTHON 2.0)";
const char S25157[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Maxthon 2.0)";
const char S25158[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; MAXTHON 2.0)";
const char S25159[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; MAXTHON 2.0)";
const char S25160[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Maxthon; .NET CLR 2.0.50727; Maxthon 2.0)";
const char S25161[] = "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; Trident/4.0; Media Center PC 4.0; SLCC1; .NET CLR 3.0.04320; Maxthon 2.0)";
const char S25162[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Avant Browser; .NET CLR 2.0.50727; MAXTHON 2.0)";
const char S25163[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Maxthon 2.0)";
const char S25164[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; MAXTHON 2.0)";
const char S25165[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MAXTHON 2.0)";
const char S25166[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Maxthon; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MAXTHON 2.0)";
const char S25167[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; MAXTHON 2.0)";
const char S25168[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Maxthon 2; MAXTHON 2.0)";
const char S25169[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Maxthon 2.0)";
const char S25170[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Maxthon; MAXTHON 2.0)";
const char S25171[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Maxthon; .NET CLR 2.0.50727; Maxthon 2.0)";
const char S25172[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; Maxthon 2.0)";
const char S25173[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MAXTHON 2.0)";
const char S25174[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; Maxthon 2.0)";
const char S25175[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MAXTHON 2.0)";
const char S25176[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; IEMB3; IEMB3; MAXTHON 2.0)";
const char S25177[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; SV1; Maxthon 2.0)";
const char S25178[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/28.0.1500.29) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.29 Safari/537.36";
const char S25179[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/28.0.1500.52) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36";
const char S25180[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/28.0.1500.63) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36";
const char S25181[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/28.0.1500.71) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36";
const char S25182[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/28.0.1500.71) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36";
const char S25183[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/28.0.1500.72) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36";
const char S25184[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/28.0.1500.72) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36";
const char S25185[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/28.0.1500.95)";
const char S25186[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.5; chromeframe/28.0.1500.95; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S25187[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/28.0.1500.95) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S25188[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/28.0.1500.95)";
const char S25189[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/28.0.1500.95) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S25190[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/28.0.1500.95; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)";
const char S25191[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/28.0.1500.95; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C)";
const char S25192[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.5; chromeframe/28.0.1500.95; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S25193[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.5; chromeframe/28.0.1500.95; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S25194[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.5; chromeframe/28.0.1500.95; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S25195[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.5; chromeframe/28.0.1500.95; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S25196[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.5; chromeframe/28.0.1500.95; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S25197[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25198[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P5110 Build/JRO03C)";
const char S25199[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25200[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P5110 Build/JZO54K)";
const char S25201[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25202[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5110 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25203[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-P5110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25204[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-P5110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25205[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5110 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25206[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5110 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25207[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S25208[] = "com.google.android.youtube/4.1.23(Linux; U; Android 4.1.1; en_GB; GT-P5110 Build/JRO03C) gzip";
const char S25209[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P5110 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S25210[] = "Logos/2.2.10 (Linux; U; Android 4.1.1; en-GB; GT-P5110 Build/JRO03C.P5110XXCLK7)";
const char S25211[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P5110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 sessionId/c36b5d49-0d14-4b82-84d9-6b0a5f965fb8 tiantian(securitypay) tiantian(tenpay)";
const char S25212[] = "com.google.android.apps.plus/X (Linux; U; Android 4.1.1; fr_FR; GT-P5110; Build/JRO03C) (gzip)";
const char S25213[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P5110 Build/JRO03C) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 baidubrowser/061_1.41.3.2_diordna_2321_008/gnusmas_61_1.1.4_0115P-TG/Xk/B0AAF0DDXED091EBB86749AA4CD%7C0/1";
const char S25214[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/5%2C%22landing%22%3A%22http%3A//www.harveysfurniture.co.uk/mobile/home.aspx%22%2C%22time%22%3AX%7D%2C%7B%22url%22%3A%22direct%22%2C%22landing%22%3A%22http%3A//www.harveysfurniture.co.uk/mobile/error.aspx%3Faspxerrorpath%3D%252fmobile%252fcategory.aspx%22%2C%22time%22%3AX%7D%5D%2C%22sessionLandingPage%22%3A%22http%3A//www.harveysfurniture.co.uk/mobile/error.aspx%3Faspxerrorpath%3D%252fmobile%25";
const char S25215[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Ve2C%22sessionLandingPage%22%3A%22http%3A//www.harveysfurniturept-Charset: utf-8, iso-8859-1, utf-16, *;q=0.7";
const char S25216[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P511GT-P5110 Build/JRO03C) AppleWebK30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25217[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S25218[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 BuiPC=id=Xa7d1b1aa785dX:lv=X:ss=X; _qubitTracker=X.X; _qPageNum_harveysfurniture=7; _qst=%5B1%2C0%5D; _qsst=X; qb_etc_status=%7B%22c%22%3A%7B%7D%7D; __utma=X.X.X.X.X.1; __utmb=X.8.10.X; __utmc=X; __utmz=X.X.1.1.utmgclid=cog4sav0l7ucffhjtaoddnqaog|utmccn=brand%20pure%20(pc)%20-%20exact|utmcmd=pp";
const char S25219[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 VoiceGO!(cyb9bXa78aXf3d91aea03735e-0;Android,4.1.1,GT-P5110;3.8.0;1280x696,1.0;X;VoiceGo-TC;zh-TW;X;X)";
const char S25220[] = "Android/4.8 (Linux; U; Android 4.1.1; en_GB; GT-P5110)";
const char S25221[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; GT-P5110 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25222[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P5110 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S25223[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P5110 Build/JZO54K) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.4 Version/4.0 Mobile Safari/530.17 baidubrowser/061_1.8.4.2_diordna_2321_008/gnusmas_61_2.1.4_0115P-TG/Xz/BBXDB6CC5BXE8%7C0/1";
const char S25224[] = "BBC News (AndroidApp; 2.1; 33) GT-P5110 (Android 4.1.1, SDK 16)";
const char S25225[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P5110 Build/JZO54K; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S25226[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-P5110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25227[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWe52fcategory.aspx%22%2C%22timko) Version/4.0 4566%7D%2C%7B%22url%22%3A%22direct%22%2C%22landing%22%3A%22http%3A//www.harveysfurniture.co.uk/mobile/error.aspx%3Faspxerrorpath%3D%252fmobile%252fcategory.aspx%22%2C%22time%22%3AX%7D%5D%2C%22sessionLandingPage%22%3A%22http%3A//www.harveysfurniture.co.uk/mobile/error.aspx%3Faspxerrorpath%3D%252fmobile%252fcategory.aspx%22%2C%22__v%22%3A%7B%7D%7D; _qubitTracker=X";
const char S25228[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO0 en-gb; GT-P5110 Build/JRO03C) Aike Gecko) Version/4.0 Safari/534.30";
const char S25229[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Bu33655; __utmc=X; __utmz=7KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25230[] = "BBC News (AndroidApp; 1.0; 5) GT-P5110 (Android 4.1.2, SDK 16)";
const char S25231[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P5110 Build/JRO03C; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S25232[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25233[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25234[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25235[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3113 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25236[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P3113 Build/JRO03C)";
const char S25237[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25238[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25239[] = "Logos/2.0.8 (Linux; U; Android 4.1.1; en-US; GT-P3113 Build/cm_p3113-userdebug 4.1.1 JRO03L eng.codeworkx.X.X test-keys)";
const char S25240[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25241[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S25242[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P3113 Build/JZO54K)";
const char S25243[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25244[] = "Logos/2.2.10 (Linux; U; Android 4.1.1; en-US; GT-P3113 Build/JRO03C.P3113UECLK7)";
const char S25245[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.3.4-94 (Android 4.1.1; samsung GT-P3113 Build/JRO03C.P3113UECLK7)";
const char S25246[] = "BBC News (AndroidApp; 1.3; 4) GT-P3113 (Android 4.1.1, SDK 16)";
const char S25247[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/5.0.2 Version/4.0 Safari/534.30 Safari/6533.18.5";
const char S25248[] = "nApps (Android 4.1.1; GT-P3113; line-antivirus-android; 1.0.6)";
const char S25249[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.0 Version/4.0 Mobile Safari/530.17";
const char S25250[] = "com.google.android.youtube/4.1.23(Linux; U; Android 4.1.1; en_US; GT-P3113 Build/JRO03C) gzip";
const char S25251[] = "Mozilla/5.0(YAdSDK; 2.6); (Linux; U; Android 4.1.1; SAMSUNG GT-P3113 Build/JELLY_BEAN);";
const char S25252[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; GT-P3113 Build/JRO03C like Xoom) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25253[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S25254[] = "Scanner Radio Pro/3.9.2 (Android 4.1.1, GT-P3113)";
const char S25255[] = "com.google.android.apps.plus/X (Linux; U; Android 4.1.1; en_US; GT-P3113; Build/JRO03C) (gzip)";
const char S25256[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S25257[] = "Mozilla/5.0 (Android 4.1.1; en-ca; GT-P3113 Build/JRO03C) FreeWheelAdManager/5.3.0-r9362-X;com.vevo/442";
const char S25258[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 androidAppVersion/2.5.17 AndroidWebkitWrapper BillingSupported";
const char S25259[] = "Dalvik 2.06.12230; (Linux; U; Android 4.1.1; GT-P3113; ON) 659c8688ccdde2dd; 1024x552x1.0";
const char S25260[] = "Android/4.7 (Linux; U; Android 4.1.1; en_CA; GT-P3113)";
const char S25261[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25262[] = "TwonkyBeamBrowser/3.4.0-99 (Android 4.1.1; samsung GT-P3113 Build/JRO03C.P3113UECLK7)";
const char S25263[] = "Groupon/2.2.2561 (Android 4.1.1; Samsung Espressowifi / Samsung GT-P3113; )[preload=true;locale=en_US;clientidbase=android-samsung]";
const char S25264[] = "MediaPlayer/2002/4.2.3 (Android 4.1.1; espressowifi; GT-P3113; samsung; ; ; en_CA)";
const char S25265[] = "Scanner Radio Pro/3.8.3 (Android 4.1.1, GT-P3113)";
const char S25266[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; GT-P3113 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MantanoReader";
const char S25267[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en_CA; GT-P3113 BUILD/JRO03C) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S25268[] = "nApps (Android 4.1.1; GT-P3113; androidapp.lineplay; 1.2.1)";
const char S25269[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; GT-P3113 Build/JRO03C) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 GoogleGoggles-Android/1.9; gzip";
const char S25270[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P3113 Build/JRO03C) [FBAN/Orca-Android;FBAV/2.3.2-release;FBLC/en_US;FBBV/X;FBCR/null;FBMF/samsung;FBBD/samsung;FBDV/GT-P3113;FBSV/4.1.1]";
const char S25271[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25272[] = "BBC News (AndroidApp; 2.2; 26) GT-P3113 (Android 4.1.1, SDK 16)";
const char S25273[] = "nApps (Android 4.1.1; GT-P3113; androidapp.lineplay; 1.2.3)";
const char S25274[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25275[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; MSN Optimized)";
const char S25276[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; SV1)";
const char S25277[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; SV1)";
const char S25278[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; MS-RTC LM 8)";
const char S25279[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; SV1; SV1)";
const char S25280[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows 98; Win 9x 4.90; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)";
const char S25281[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; SV1; SV1)";
const char S25282[] = "Mozilla/5.0 (compatible; MSIE 8.0; Win98; Trident/5.0)";
const char S25283[] = "Mozilla/5.0 (compatible; MSIE 8.0; Win 9x 4.90; Trident/5.0)";
const char S25284[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/24.0.1312.14) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.14 Safari/537.17";
const char S25285[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/24.0.1312.45) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.45 Safari/537.17";
const char S25286[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/24.0.1312.52) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17";
const char S25287[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/24.0.1312.52) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17";
const char S25288[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/24.0.1312.57) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S25289[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/24.0.1312.57; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S25290[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/24.0.1312.56) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17";
const char S25291[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/24.0.1312.56) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17";
const char S25292[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; chromeframe/24.0.1312.57; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S25293[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/24.0.1312.57)";
const char S25294[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/24.0.1312.57; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S25295[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/24.0.1312.57) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S25296[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/24.0.1312.57)";
const char S25297[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; chromeframe/24.0.1312.57; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S25298[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.4; chromeframe/24.0.1312.57; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S25299[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; chromeframe/24.0.1312.57; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S25300[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; chromeframe/24.0.1312.57; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S25301[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/24.0.1312.57; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S25302[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25303[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25304[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P5113 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25305[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P5113 Build/JRO03C)";
const char S25306[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25307[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25308[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25309[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25310[] = "FaithlifeStudyBible/2.1.2 (Linux; U; Android 4.1.1; en-US; GT-P5113 Build/cm_p5113-userdebug 4.1.1 JRO03R eng..X.X test-keys)";
const char S25311[] = "Logos/2.2.3 (Linux; U; Android 4.1.2; en-US; GT-P5113 Build/cm_p5113-userdebug 4.1.2 JZO54K eng..X.X test-keys)";
const char S25312[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P5113 Build/JZO54K)";
const char S25313[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25314[] = "Logos/2.2.8.1 (Linux; U; Android 4.1.1; en-US; GT-P5113 Build/JRO03C.P5113UECLK7)";
const char S25315[] = "Groupon/2.1.2486 (Android 4.1.1; Samsung Espresso10wifi / Samsung GT-P5113; )[preload=false;locale=en_CA;clientidbase=android-samsung]";
const char S25316[] = "com.google.android.apps.plus/X (Linux; U; Android 4.1.1; en_CA; GT-P5113; Build/JRO03C) (gzip)";
const char S25317[] = "BBC News (AndroidApp; 1.1; 9) GT-P5113 (Android 4.1.1, SDK 16)";
const char S25318[] = "Dalvik 2.06.12230; (Linux; U; Android 4.1.1; GT-P5113; ON) 6df6e536c10c2ec6; 1280x752x1.0";
const char S25319[] = "Mozilla/5.0 (Android 4.1.1; en-za; GT-P5113 Build/JRO03C) FreeWheelAdManager/5.2.3-r9361-X;com.gotv.crackle.handset/X";
const char S25320[] = "Android/4.7 (Linux; U; Android 4.1.1; en_CA; GT-P5113)";
const char S25321[] = "rad.io/1.1.12 (Android 4.1.1) samsung/GT-P5113";
const char S25322[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25323[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; GT-P5113 Build/JRO03C) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25324[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; GT-P5113 Build/JRO03C) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S25325[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/5.0.2 Version/4.0 Safari/534.30 Safari/6533.18.5";
const char S25326[] = "com.google.android.youtube/4.1.23(Linux; U; Android 4.1.1; en_US; GT-P5113 Build/JRO03C) gzip";
const char S25327[] = "Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 4.1.1; SAMSUNG GT-P5113 Build/Unknown);";
const char S25328[] = "Gospel_Library 2.3.1.3 / Android 4.1.1 P5113UECLK7 / samsung GT-P5113";
const char S25329[] = "Android 4.1.1; en_US; GT-P5113/espresso10wifi; com.getsetgames.megajump 35 1.6; Kiip/Java 11 1.0.14-2 2012-05-24";
const char S25330[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; JRO03C; samsung; GT-P5113; en-us; dream) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S25331[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P5113 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S25332[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 androidAppVersion/2.5.17 AndroidWebkitWrapper BillingSupported";
const char S25333[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S25334[] = "nApps (Android 4.1.1; GT-P5113; androidapp.lineplay; 1.2.1)";
const char S25335[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-ca; GT-P5113 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MantanoReader";
const char S25336[] = "BBC News (AndroidApp; 1.0; 5) GT-P5113 (Android 4.1.1, SDK 16)";
const char S25337[] = "VKAndroidApp/2.7.6-64 (Android 4.1.1; SDK 16; armeabi-v7a; samsung GT-P5113; en)";
const char S25338[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/2.1;FBDM/{density=1.0,width=1280,height=752};FBLC/es_US;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P5113;FBSV/4.1.1;]";
const char S25339[] = "ZillowMap/5.2.194 (Android 4.1.1, samsung GT-P5113)";
const char S25340[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.4.1-100 (Android 4.1.1; samsung GT-P5113 Build/JRO03C.P5113UECLK7)";
const char S25341[] = "nApps:(Android 4.1.1; GT-P5113; %EB%84%A4%EC%9D%B4%EB%B2%84+%EC%BA%98%EB%A6%B0%EB%8D%94; 1.9.6)";
const char S25342[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-ca; GT-P5113 Build/JRO03C like Xoom) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25343[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr_TR; GT-P5113 BUILD/JRO03C) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S25344[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 GoogleGoggles-Android/1.9; gzip";
const char S25345[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:15.0) Gecko/X Firefox/15.0.1";
const char S25346[] = "Mozilla/5.0 (Windows NT 5.1; rv:15.0) Gecko/X Firefox/15.0.1 Firefox/15.0.1";
const char S25347[] = "Mozilla/5.0 (Windows NT 5.1; WOW64; rv:15.0) Gecko/X Firefox/15.0.1";
const char S25348[] = "Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:15.0) Gecko/X Firefox/15.0";
const char S25349[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.3) Gecko/X Firefox/15.0.0.1";
const char S25350[] = "Mozilla/5.0(Windows; U; Windows NT 5.2; rv:1.9.2) Gecko/X Firefox/15.0";
const char S25351[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.1.7) Gecko/X Firefox/15.0 (.NET CLR 3.0.04506.648)";
const char S25352[] = "BlackBerry8900/5.0.0.900 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/100";
const char S25353[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Alexa Toolbar)";
const char S25354[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; SV1)";
const char S25355[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; MyIE2)";
const char S25356[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; SV1)";
const char S25357[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; SV1; SV1)";
const char S25358[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; SV1; SV1)";
const char S25359[] = "Mozilla/5.0 (compatible; MSIE 9.0; Win98; Trident/4.0)";
const char S25360[] = "Mozilla/5.0 (compatible; MSIE 9.0; Win 9x 4.90; Trident/2.0)";
const char S25361[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:12.0) Gecko/X Firefox/12.0";
const char S25362[] = "Mozilla/5.0 (Windows NT 6.1; rv:12.0; StumbleUpon; noc@stumbleupon.com) Gecko/X Firefox/12.0";
const char S25363[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.0) Gecko/X Firefox/12.0 Firefox/12.0";
const char S25364[] = "Mozilla/5.0 (Windows; Windows NT 6.1; rv:12.0) Gecko/X Firefox/12.0";
const char S25365[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B146 Safari/8536.25";
const char S25366[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B147 Safari/8536.25";
const char S25367[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B141 Safari/8536.25";
const char S25368[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B329 Safari/8536.25";
const char S25369[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows 98)";
const char S25370[] = "Mozilla/4.0 (compatible; MSIE 5.50; Windows 98; SiteKiosk 4.8)";
const char S25371[] = "MSIE 5.5 (Win ME): Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; Win 9x 4.90)";
const char S25372[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:21.0) Gecko/X Firefox/21.0 AlexaToolbar/alxf-2.18";
const char S25373[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:16.0.1) Gecko/X Firefox/21.0.1";
const char S25374[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:21.0) Gecko/X Firefox/21.0 IceDragon/21.0.1.15";
const char S25375[] = "Mozilla/5.0 (Windows; Windows NT 6.1; rv:21.0) Gecko/21.0 Firefox/21.0";
const char S25376[] = "Mozilla/5.0 (Windows; Windows NT 6.1; Win64; x64; rv:21.0) Gecko/21.0 Firefox/21.0";
const char S25377[] = "Mozilla/5.0 (X11; CrOS i686 2876.0.0) AppleWebKit/537.10 (KHTML, like Gecko) Chrome/23.0.1262.2 Safari/537.10";
const char S25378[] = "Mozilla/5.0 (X11; CrOS x86_64 2876.0.0) AppleWebKit/537.10 (KHTML, like Gecko) Chrome/23.0.1262.2 Safari/537.10";
const char S25379[] = "Mozilla/5.0 (X11; CrOS x86_64 2913.49.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.19 Safari/537.11";
const char S25380[] = "Mozilla/5.0 (X11; CrOS i686 2913.49.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.19 Safari/537.11";
const char S25381[] = "Mozilla/5.0 (X11; CrOS i686 2913.50.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.19 Safari/537.11";
const char S25382[] = "Mozilla/5.0 (X11; CrOS i686 2913.21.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.8 Safari/537.11";
const char S25383[] = "Mozilla/5.0 (X11; CrOS i686 2913.27.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.11 Safari/537.11";
const char S25384[] = "Mozilla/5.0 (X11; CrOS x86_64 2913.27.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.11 Safari/537.11";
const char S25385[] = "Mozilla/5.0 (X11; CrOS x86_64 2913.3.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.2 Safari/537.11";
const char S25386[] = "Mozilla/5.0 (X11; CrOS i686 2913.3.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.2 Safari/537.11";
const char S25387[] = "Mozilla/5.0 (X11; CrOS i686 2913.41.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.16 Safari/537.11";
const char S25388[] = "Mozilla/5.0 (X11; CrOS i686 2913.73.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.30 Safari/537.11";
const char S25389[] = "Mozilla/5.0 (X11; CrOS x86_64 2913.73.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.30 Safari/537.11";
const char S25390[] = "Mozilla/5.0 (X11; CrOS x86_64 2913.106.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.41 Safari/537.11";
const char S25391[] = "Mozilla/5.0 (X11; CrOS i686 2913.106.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.41 Safari/537.11";
const char S25392[] = "Mozilla/5.0 (X11; CrOS armv7l 2913.84.10) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.49 Safari/537.11";
const char S25393[] = "Mozilla/5.0 (X11; CrOS x86_64 2913.135.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.53 Safari/537.11";
const char S25394[] = "Mozilla/5.0 (X11; CrOS i686 2913.135.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.53 Safari/537.11";
const char S25395[] = "Mozilla/5.0 (X11; CrOS armv7l 2913.157.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.83 Safari/537.11";
const char S25396[] = "Mozilla/5.0 (X11; CrOS i686 2913.156.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.83 Safari/537.11";
const char S25397[] = "Mozilla/5.0 (X11; CrOS x86_64 2913.156.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.83 Safari/537.11";
const char S25398[] = "Mozilla/5.0 (X11; CrOS x86_64 2913.41.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.16 Safari/537.11";
const char S25399[] = "Mozilla/5.0 (X11; CrOS armv7l 2913.135.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.53 Safari/537.11";
const char S25400[] = "Mozilla/5.0 (X11; CrOS x86_64 2913.166.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.84 Safari/537.11";
const char S25401[] = "Mozilla/5.0 (X11; CrOS i686 2913.166.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.84 Safari/537.11";
const char S25402[] = "Mozilla/5.0 (X11; CrOS armv7l 2913.168.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.84 Safari/537.11";
const char S25403[] = "Mozilla/5.0 (X11; CrOS x86_64 2913.187.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.89 Safari/537.11";
const char S25404[] = "Mozilla/5.0 (X11; CrOS i686 2913.187.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.89 Safari/537.11";
const char S25405[] = "Mozilla/5.0 (X11; CrOS i686 2913.156.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.83 Safari/537.11";
const char S25406[] = "Mozilla/5.0 (X11; CrOS armv7l 2913.187.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.89 Safari/537.11";
const char S25407[] = "Mozilla/5.0 (X11; CrOS x86_64 2913.181.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.87 Safari/537.11";
const char S25408[] = "Mozilla/5.0 (X11; CrOS armv7l 2913.53.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.20 Safari/537.11";
const char S25409[] = "Mozilla/5.0 (X11; CrOS armv7l 2913.224.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.94 Safari/537.11";
const char S25410[] = "Mozilla/5.0 (X11; CrOS x86_64 2913.224.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.94 Safari/537.11";
const char S25411[] = "Mozilla/5.0 (X11; CrOS i686 2913.224.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.94 Safari/537.11";
const char S25412[] = "Mozilla/5.0 (X11; CrOS i686 2913.120.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.49 Safari/537.11";
const char S25413[] = "Mozilla/5.0 (X11; CrOS i686 2913.200.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.92 Safari/537.11";
const char S25414[] = "Mozilla/5.0 (X11; CrOS armv7l 2913.251.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S25415[] = "Mozilla/5.0 (X11; CrOS x86_64 2913.260.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.99 Safari/537.11";
const char S25416[] = "Mozilla/5.0 (X11; CrOS i686 2913.260.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.99 Safari/537.11";
const char S25417[] = "Mozilla/5.0 (X11; CrOS i686 2913.276.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.110 Safari/537.11";
const char S25418[] = "Mozilla/5.0 (X11; CrOS armv7l 2913.260.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.99 Safari/537.11";
const char S25419[] = "Mozilla/5.0 (X11; CrOS x86_64 2913.270.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.110 Safari/537.11";
const char S25420[] = "Mozilla/5.0 (X11; CrOS armv7l 2913.212.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.93 Safari/537.11";
const char S25421[] = "Mozilla/5.0 (X11; CrOS x86_64 2913.278.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.110 Safari/537.11";
const char S25422[] = "Mozilla/5.0 (X11; CrOS x86_64 2913.225.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.94 Safari/537.11";
const char S25423[] = "Mozilla/5.0 (X11; CrOS x86_64 2913.331.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.111 Safari/537.11";
const char S25424[] = "Mozilla/5.0 (X11; CrOS i686 2913.331.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.111 Safari/537.11";
const char S25425[] = "Mozilla/5.0 (X11; CrOS armv7l 2913.331.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.111 Safari/537.11";
const char S25426[] = "Mozilla/5.0 (X11; CrOS i686 2913.345.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.113 Safari/537.11";
const char S25427[] = "Mozilla/5.0 (X11; CrOS armv7l 2913.345.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.113 Safari/537.11";
const char S25428[] = "Mozilla/5.0 (X11; CrOS x86_64 2913.187.6) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.89 Safari/537.11";
const char S25429[] = "Mozilla/5.0 (X11; CrOS x86_64 2913.187.3) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.89 Safari/537.11";
const char S25430[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25431[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25432[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P7510 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25433[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P7510 Build/HTJ85B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25434[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T859 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25435[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7510 Build/IMM76D)";
const char S25436[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25437[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P7510-ORANGE/P7510BVLP5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25438[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-T859 Build/IMM76D)";
const char S25439[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P7510-ORANGE/P7510BVLP7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25440[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-P7510 Build/IML74K)";
const char S25441[] = "Logos/1.1.2 (Linux; U; Android 4.0.4; en-CA; GT-P7510 Build/cm_p4wifi-userdebug 4.0.4 IMM76L eng.jenkins.X.X test-keys)";
const char S25442[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25443[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S25444[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7510 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S25445[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7510 Build/IMM76D; 1280*752) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S25446[] = "Mozilla/5.0 (Android 4.0.4; en-us; GT-P7510 Build/IMM76D) FreeWheelAdManager/4.9.2-r8793-X;com.vevo/435";
const char S25447[] = "Logos/2.1.2 (Linux; U; Android 4.0.4; en-US; GT-P7510 Build/IMM76D.UELPL)";
const char S25448[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25449[] = "BBC iPlayer (AndroidApp; 1.3; 13014) GT-P7510 (Android 4.0.4, SDK 15)";
const char S25450[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7510 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 parameters/{scr=752_1280,cm=1,ql=l,imei=BC:47:60:F7:DE:1A,bm=1,eng=adrmini15,mdm=1,mdl=GT-P7510,ps=1} tiantian(securitypay,securityinstalled) tiantian(tenpay)";
const char S25451[] = "Biblia/2.1.2 (Linux; U; Android 4.0.4; es-US; GT-P7510 Build/Task650's ICS)";
const char S25452[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S25453[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25454[] = "Logos/2.1.2 (Linux; U; Android 4.0.4; en-GB; GT-P7510 Build/IMM76D.XWLP6)";
const char S25455[] = "Logos/2.2.3 (Linux; U; Android 4.0.4; en-US; GT-P7510 Build/IMM76D.ZSLP8)";
const char S25456[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.2.1-65 (Android 4.0.4; samsung GT-P7510 Build/IMM76D.UELPL)";
const char S25457[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 androidAppVersion/2.2.2.1 AndroidWebkitWrapper BillingSupported";
const char S25458[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P7510-ORANGE/P7510BVLP1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25459[] = "FaithlifeStudyBible/2.2.3 (Linux; U; Android 4.0.4; en-GB; GT-P7510 Build/IMM76D.XWLP5)";
const char S25460[] = "com.google.android.apps.plus/X (Linux; U; Android 4.0.4; en_US; GT-P7510; Build/IMM76D) (gzip)";
const char S25461[] = "Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 4.0.4; SAMSUNG GT-P7510 Build/ICE_CREAM_SANDWICH_MR1);";
const char S25462[] = "Groupon/2.1.2483 (Android 4.0.4; Samsung GT-P7510 / Samsung GT-P7510; )[preload=false;locale=en_US;clientidbase=android-samsung]";
const char S25463[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25464[] = "MediaPlayer/2002/4.1.0 (Android 4.0.4; GT-P7510; GT-P7510; samsung; ; ; en_US)";
const char S25465[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S25466[] = "Scanner Radio/3.9.0.2 (Android 4.0.4, GT-P7510)";
const char S25467[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; GT-P7510 Build/HTJ85B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25468[] = "Mozilla/5.0 (Linux; U; Android 4.0.4, en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25469[] = "Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-GB; GT-P7510 Build/IMM76D.XXLPH)";
const char S25470[] = "com.google.android.youtube/4.2.16(Linux; U; Android 4.0.4; en_US; GT-P7510 Build/IMM76D) gzip";
const char S25471[] = "Scanner Radio/3.9.1 (Android 4.0.4, GT-P7510)";
const char S25472[] = "Scanner Radio/3.9.1.1 (Android 4.0.4, GT-P7510)";
const char S25473[] = "Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-GB; GT-P7510 Build/IMM76D.VLLH3)";
const char S25474[] = "FaithlifeStudyBible/2.2.10 (Linux; U; Android 4.0.4; en-US; GT-P7510 Build/IMM76D.XWLPC)";
const char S25475[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; it-it; GT-P7510 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25476[] = "Dalvik 2.02.11978; (Linux; U; Android 4.0.4; GT-P7510) 5a5d79363c64d681; 1280x752x1.0";
const char S25477[] = "Mozilla/5.0 (Linux; U; Android 4.04; en-us; GT-P7510 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25478[] = "ZillowMap/5.2.194 (Android 4.0.4, samsung GT-P7510)";
const char S25479[] = "BBC News (AndroidApp; 1.0; 5) GT-P7510 (Android 4.0.4, SDK 15)";
const char S25480[] = "BBC News (AndroidApp; 2.1; 33) GT-P7510 (Android 4.0.4, SDK 15)";
const char S25481[] = "Scanner Radio/3.9.2 (Android 4.0.4, GT-P7510)";
const char S25482[] = "BBC News (AndroidApp; 1.1; 9) GT-P7510 (Android 4.0.4, SDK 15)";
const char S25483[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-; GT-P7510-ORANGE/P7510BVLP5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S25484[] = "????/2.12.24 (Linux; U; Android 4.0.4; zh-cn; GT-P7510 Build/IMM76D) AppleWebKit/528.5+";
const char S25485[] = "Scanner Radio/3.9.3 (Android 4.0.4, GT-P7510)";
const char S25486[] = "VKAndroidApp/2.7.6-64 (Android 4.0.4; SDK 15; armeabi-v7a; samsung GT-P7510; en)";
const char S25487[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/2.1;FBDM/{density=1.0,width=1280,height=752};FBLC/en_US;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P7510;FBSV/4.0.4;]";
const char S25488[] = "Android/4.8.1 (Linux; U; Android 4.0.4; en_US; GT-P7510)";
const char S25489[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 androidAppVersion/2.5.17 AndroidWebkitWrapper BillingSupported";
const char S25490[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25491[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 ZyngaBundleIdentifier/com.zynga.zjayakashi ZyngaBundleVersion/2.0.0";
const char S25492[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7510 Build/IMM76L) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S25493[] = "Scanner Radio/3.9.4 (Android 4.0.4, GT-P7510)";
const char S25494[] = "rad.io/1.1.19 (Android 4.0.4) samsung/GT-P7510";
const char S25495[] = "BBC News (AndroidApp; 2.2; 29) GT-P7510 (Android 4.0.4, SDK 15)";
const char S25496[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; ) AppleWebKit/533.9 (KHTML, like Gecko) Maxthon/3.0 Safari/533.9";
const char S25497[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; ) AppleWebKit/534.12 (KHTML, like Gecko) Maxthon/3.0 Safari/534.12";
const char S25498[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; ) AppleWebKit/534.12 (KHTML, like Gecko) Maxthon/3.0 Safari/534.12Mozilla/5.0 (Windows; U; Windows NT 6.1; ) AppleWebKit/534.12 (KHTML, like Gecko) Maxthon/3.0 Safari/534.12";
const char S25499[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Maxthon/3.0.6.27 Safari/532.4";
const char S25500[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.1 (KHTML, like Gecko) Maxthon/3.0.11.8 Safari/533.1";
const char S25501[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12";
const char S25502[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12";
const char S25503[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/530.6 (KHTML, like Gecko) Maxthon/3.0 Safari/530.6";
const char S25504[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12,gzip(gfe) (via translate.google.com)";
const char S25505[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1";
const char S25506[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1";
const char S25507[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12,gzip(gfe) (via translate.google.com)";
const char S25508[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12";
const char S25509[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12,gzip(gfe)";
const char S25510[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12,gzip(gfe)";
const char S25511[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1,gzip(gfe)";
const char S25512[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1,gzip(gfe)";
const char S25513[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12,gzip(gfe) (via docs.google.com/viewer)";
const char S25514[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12,gzip(gfe) (via docs.google.com/viewer)";
const char S25515[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.558.0 Safari/534.10";
const char S25516[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10";
const char S25517[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10,gzip(gfe) (via translate.google.com)";
const char S25518[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10,gzip(gfe) (via docs.google.com/viewer)";
const char S25519[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; vi) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.558.0 Safari/534.10";
const char S25520[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10,gzip(gfe)";
const char S25521[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; id) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.558.0 Safari/534.10";
const char S25522[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; Pt-br) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.558.0 Safari/534.10";
const char S25523[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; es-la) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.558.0 Safari/534.10";
const char S25524[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; fr-fr) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.558.0 Safari/534.10";
const char S25525[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; haw-US) AppleWebKit/534.10 (KHTML like Gecko) Chrome/8.0.552.224 Safari/534.10";
const char S25526[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.10 (KHTML like Gecko) Chrome/8.0.552.224 Safari/534.10";
const char S25527[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-CA) AppleWebKit/534.10 (KHTML like Gecko) Chrome/8.0.552.224 Safari/534.10";
const char S25528[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; fr-CA) AppleWebKit/534.10 (KHTML like Gecko) Chrome/8.0.552.224 Safari/534.10";
const char S25529[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/534.10 (KHTML like Gecko) Chrome/8.0.552.224 Safari/534.10";
const char S25530[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; es-US) AppleWebKit/534.10 (KHTML like Gecko) Chrome/8.0.552.224 Safari/534.10";
const char S25531[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; br-FR) AppleWebKit/534.10 (KHTML like Gecko) Chrome/8.0.552.224 Safari/534.10";
const char S25532[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; pl-DE) AppleWebKit/534.10 (KHTML like Gecko) Chrome/8.0.552.224 Safari/534.10";
const char S25533[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; de-DE) AppleWebKit/534.10 (KHTML like Gecko) Chrome/8.0.552.224 Safari/534.10";
const char S25534[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; ru-DE) AppleWebKit/534.10 (KHTML like Gecko) Chrome/8.0.552.224 Safari/534.10";
const char S25535[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9790; en) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.528 Mobile Safari/534.11+";
const char S25536[] = "Mozilla/4.0 (compatible; MSIE 5.00; Windows 98)";
const char S25537[] = "Mozilla/4.0(compatible; MSIE 5.0; Windows 98; DigExt)";
const char S25538[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows 98; DigExt)";
const char S25539[] = "Mozilla/4.0 (compatible; MSIE 5.01; Windows 98)";
const char S25540[] = "Mozilla/4.0(compatible;MSIE 5.00;Windows 98)";
const char S25541[] = "Mozilla/4.0 (compatible; MSIE 5.02; Windows 98)";
const char S25542[] = "Mozilla/4.0 (compatible; MSIE 5.01; Windows 98; Linux 2.4.2-mvista_01) [Netgem; 4.6.23c; i-Player; netbox; btdigitaltv]";
const char S25543[] = "Mozilla/4.0 (compatible; MSIE 5.05; Windows 98; .NET CLR 1.1.4322)";
const char S25544[] = "Mozilla/5.0 (compatible; MSIE 5.0; Win 9x 4.90; Trident/4.0)";
const char S25545[] = "Mozilla/5.0 (compatible; MSIE 5.0; Win98; Trident/5.0)";
const char S25546[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows 98; Avant Browser; Avant Browser)";
const char S25547[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25548[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-S5300B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25549[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5300 Build/GINGERBREAD)";
const char S25550[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S25551[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-S5300L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25552[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25553[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5300B Build/GINGERBREAD)";
const char S25554[] = "Opera/9.80 (Android; Opera Mini/6.1.27762/26.1670; U; es) Presto/2.8.119 Version/10.54";
const char S25555[] = "Opera/9.80 (Android; Opera Mini/6.1.25577/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S25556[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.57 Safari/537.1";
const char S25557[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.90 Safari/537.1";
const char S25558[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.90 Safari/537.1";
const char S25559[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.90 Safari/537.1";
const char S25560[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_5) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.90 Safari/537.1";
const char S25561[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.90 Safari/537.1";
const char S25562[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.90 Safari/537.1,gzip(gfe) (via translate.google.com)";
const char S25563[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S25564[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.90 Safari/537.1,gzip(gfe)";
const char S25565[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.90 Safari/537.1,gzip(gfe) (via docs.google.com/viewer)";
const char S25566[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.79 Safari/537.1";
const char S25567[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/21.0.648.205 Safari/534.16";
const char S25568[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/21.0.648.205 Safari/534.16";
const char S25569[] = "Mozilla/5.0 (Linux; Android 4.1.9; GT-I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36";
const char S25570[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S25571[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S25572[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S25573[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S25574[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S25575[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S25576[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S25577[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S25578[] = "Mozilla/5.0 (Linux; Android 4.1.2; SC-06D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S25579[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-I535 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S25580[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999V Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S25581[] = "Mozilla/5.0 (Linux; Android 4.1.9; GT-I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S25582[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S25583[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-i9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S25584[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S25585[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I939 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S25586[] = "Mozilla/5.0 (Linux; Android 4.1.9; I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S25587[] = "Mozilla/5.0 (Linux; Android 4.1.1; I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S25588[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I93000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S25589[] = "Mozilla/5.0 (Windows; I; Windows NT 5.1; en; rv:1.9.2.13) Gecko/X Firefox/4.0";
const char S25590[] = "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.0.6) Gecko/X Firefox/4.0.1";
const char S25591[] = "Mozilla/5.0 (Windows NT 5.2; rv:2.0b13pre) Gecko/X Firefox/4.0b13pre";
const char S25592[] = "Mozilla/5.0 (Windows NT 5.1) Gecko/X Firefox/4.0.1";
const char S25593[] = "Mozilla/5.0(Windows NT 5.1;rv:2.0) Gecko/X Firefox/4.0";
const char S25594[] = "Mozilla/5.0 (Windows NT 5.1; rv:2.0.1) Gecko/X Firefox/4.0.1,gzip(gfe)";
const char S25595[] = "Mozilla/5.0(WindowsNT5.1;rv:2.0)Gecko/XFirefox/4.0";
const char S25596[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.1) Gecko/X Firefox/4";
const char S25597[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-137) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25598[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-163) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25599[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-154_MR-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25600[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-154) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25601[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-154_4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25602[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25603[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-154_MR-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25604[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-154_5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25605[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-143) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25606[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Motorola Electrify Build/4.5.1A_SUN_USC_15.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25607[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Motorola Electrify Build/4.5.1A_SUN_USC_16.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25608[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-122) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25609[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-133) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25610[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MB855 Build/4.5.1A-1_SUN-101) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25611[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-154_6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25612[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-198_5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25613[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-198_6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25614[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-198_GSM-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25615[] = "Logos/0.9.4 (Linux; U; Android 2.3.4; en_US; MB855 Build/4.5.1A-1_SUN-198_6)";
const char S25616[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; MB855 Build/4.5.1A-1_SUN-198_6)";
const char S25617[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-198_7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25618[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Motorola Electrify Build/4.5.1A_SUN_USC_19.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25619[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Motorola Electrify Build/4.5.1A_SUN_USC_19.0)";
const char S25620[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; Motorola Electrify Build/4.5.1A_SUN_USC_16.7)";
const char S25621[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.1A-1_SUN-254_12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25622[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25623[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; MB855 Build/4.5.1A-1_SUN-254_13)";
const char S25624[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.1A-1_SUN-254_4_GSM-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25625[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.1A-1_SUN-254_6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25626[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.1A-1_SUN-254_4_GSM-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25627[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Motorola Electrify Build/4.5.1A_SUN_USC_19.0) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S25628[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Motorola Electrify Build/4.5.1A_SUN_USC_19.0) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S25629[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Slim Photon Build/4.5.1A-1_SUN-254_13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25630[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30";
const char S25631[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25632[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.0B-1_May-265_13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25633[] = "Mozilla/5.0 (Linux; U; Android 2.3.8; en-us; MB855 Build/4.5.1A-1_SUN-198_6; CyanMobile-eXperience) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25634[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25635[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT-S7562L Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25636[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-S7562 Build/IMM76I)";
const char S25637[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-S7562 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25638[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562 Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S25639[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562i Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25640[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562i Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S25641[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-S7562L Build/IMM76I)";
const char S25642[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-S7562i Build/IMM76I)";
const char S25643[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S7562 Build/862_F1_V2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25644[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/IMM76I) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S25645[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25646[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S7562 Build/862_F1_V2.4_S501UL) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25647[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S7562 Build/862_F1_V2.3_S501U) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25648[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S7562 Build/IMM761.S7562ZCALK2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25649[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; GT-S7562 Build/862_F1_V2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25650[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/320) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25651[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/340) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25652[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S25653[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.28 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.28";
const char S25654[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.41 Safari/534.7";
const char S25655[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7,gzip(gfe) (via translate.google.com)";
const char S25656[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.50 Safari/534.7 SunChrome/7.0";
const char S25657[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0 baidubrowser/1.x Safari/534.7";
const char S25658[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.16; QQBrowser/5.1.7061.201 (webkit)";
const char S25659[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.16; QQBrowser/5.1.6829.201 (webkit)";
const char S25660[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.41 Safari/534.7,gzip(gfe)";
const char S25661[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.6 (KHTML like Gecko) Chrome/7.0.503.0 Safari/534.6";
const char S25662[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-CA) AppleWebKit/534.6 (KHTML like Gecko) Chrome/7.0.503.0 Safari/534.6";
const char S25663[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; fr-CA) AppleWebKit/534.6 (KHTML like Gecko) Chrome/7.0.503.0 Safari/534.6";
const char S25664[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; fr-FR) AppleWebKit/534.6 (KHTML like Gecko) Chrome/7.0.503.0 Safari/534.6";
const char S25665[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; es-US) AppleWebKit/534.6 (KHTML like Gecko) Chrome/7.0.503.0 Safari/534.6";
const char S25666[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; haw-US) AppleWebKit/534.6 (KHTML like Gecko) Chrome/7.0.503.0 Safari/534.6";
const char S25667[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/534.6 (KHTML like Gecko) Chrome/7.0.503.0 Safari/534.6";
const char S25668[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; ku-DE) AppleWebKit/534.6 (KHTML like Gecko) Chrome/7.0.503.0 Safari/534.6";
const char S25669[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; AppleWebKit/534.7; KHTML?? like Gecko; Chrome/7.0.517.41 Safari/534.7 ChromePlus/1.5.0.0alpha1)";
const char S25670[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S25671[] = "Mozilla/5.0 (Linux; Android 4.1.1; SPH-L710 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S25672[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747M Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S25673[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-R530U Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S25674[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S25675[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S25676[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S25677[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S25678[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S25679[] = "Mozilla/5.0 (Linux; Android 4.1.1; SPH-L710 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S25680[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S25681[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S25682[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S25683[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-R530U Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S25684[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305T Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S25685[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747M Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S25686[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305N Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S25687[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S25688[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S25689[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S25690[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S25691[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S25692[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S25693[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S25694[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S25695[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S25696[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S25697[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305N Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S25698[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305T Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S25699[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530C Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S25700[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530C Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S25701[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747M Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S25702[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747M Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S25703[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; en) Presto/2.10.229 Version/11.62";
const char S25704[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.62";
const char S25705[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5230); ru) Presto/2.10.229 Version/11.62";
const char S25706[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.229 Version/11.62";
const char S25707[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.229 Version/11.62";
const char S25708[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.229 Version/11.62";
const char S25709[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5288); ru) Presto/2.10.229 Version/11.62";
const char S25710[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.229 Version/11.62";
const char S25711[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5218); ru) Presto/2.10.229 Version/11.62";
const char S25712[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.62";
const char S25713[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.62";
const char S25714[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5299); ru) Presto/2.10.229 Version/11.62";
const char S25715[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.229 Version/11.62";
const char S25716[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; ru) Opera 11.62";
const char S25717[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.229 Version/11.62";
const char S25718[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4871); ru) Presto/2.10.229 Version/11.62";
const char S25719[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5300); ru) Presto/2.10.229 Version/11.62";
const char S25720[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5161); ru) Presto/2.10.229 Version/11.62";
const char S25721[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4661); ru) Presto/2.10.229 Version/11.62";
const char S25722[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5309); ru) Presto/2.10.229 Version/11.62";
const char S25723[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5308); ru) Presto/2.10.229 Version/11.62";
const char S25724[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.229 Version/11.62";
const char S25725[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.229 Version/11.62";
const char S25726[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.62";
const char S25727[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.229 Version/11.62";
const char S25728[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.62";
const char S25729[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.229 Version/11.62";
const char S25730[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.229 Version/11.62";
const char S25731[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.62";
const char S25732[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.229 Version/11.62";
const char S25733[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.229 Version/11.62";
const char S25734[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.229 Version/11.62";
const char S25735[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.229 Version/11.62";
const char S25736[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4664); ru) Presto/2.10.229 Version/11.62";
const char S25737[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.229 Version/11.62";
const char S25738[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.229 Version/11.62";
const char S25739[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5217); ru) Presto/2.10.229 Version/11.62";
const char S25740[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5328); ru) Presto/2.10.229 Version/11.62";
const char S25741[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; en) Presto/2.10.229 Version/11.62,gzip(gfe) (via translate.google.com)";
const char S25742[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.7 (build 03789); ru) Presto/2.10.229 Version/11.62";
const char S25743[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.229 Version/11.62";
const char S25744[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.229 Version/11.62";
const char S25745[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5278); ru) Presto/2.10.229 Version/11.62";
const char S25746[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.229 Version/11.62";
const char S25747[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.229 Version/11.62";
const char S25748[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4930); ru) Presto/2.10.229 Version/11.62";
const char S25749[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.229 Version/11.62";
const char S25750[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.229 Version/11.62";
const char S25751[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.229 Version/11.62";
const char S25752[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.229 Version/11.62";
const char S25753[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.229 Version/11.62";
const char S25754[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4110); ru) Presto/2.10.229 Version/11.62";
const char S25755[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.229 Version/11.62";
const char S25756[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5754); ru) Presto/2.10.229 Version/11.62";
const char S25757[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03278); ru) Presto/2.10.229 Version/11.62";
const char S25758[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.7 (build 03658); ru) Presto/2.10.229 Version/11.62";
const char S25759[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5667); ru) Presto/2.10.229 Version/11.62";
const char S25760[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03780); ru) Presto/2.10.229 Version/11.62";
const char S25761[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5831); ru) Presto/2.10.229 Version/11.62";
const char S25762[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5970); ru) Presto/2.10.229 Version/11.62";
const char S25763[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5972); ru) Presto/2.10.229 Version/11.62";
const char S25764[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.7 (build 03639); ru) Presto/2.10.229 Version/11.62";
const char S25765[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5325); ru) Presto/2.10.229 Version/11.62";
const char S25766[] = "Opera/11.62 (Windows NT 6.1; WOW64; U; en) Presto/2.10.229 Version/11.62";
const char S25767[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.10.229 Version/11.62";
const char S25768[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.229 Version/11.62,gzip(gfe)";
const char S25769[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 6005); ru) Presto/2.10.229 Version/11.62";
const char S25770[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.229 Version/11.62";
const char S25771[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.229 Version/11.62";
const char S25772[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.229 Version/11.62";
const char S25773[] = "Opera/9.80 (Windows NT 6.1) Presto/2.10.229 Version/11.62";
const char S25774[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5336); ru) Presto/2.10.229 Version/11.62";
const char S25775[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5784); ru) Presto/2.10.229 Version/11.62";
const char S25776[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.229 Version/11.62";
const char S25777[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5966); ru) Presto/2.10.229 Version/11.62";
const char S25778[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 6081); ru) Presto/2.10.229 Version/11.62";
const char S25779[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.229 Version/11.62";
const char S25780[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.91 Safari/534.30";
const char S25781[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30";
const char S25782[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30";
const char S25783[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30";
const char S25784[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S25785[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S25786[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30";
const char S25787[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S25788[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30";
const char S25789[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30,gzip(gfe) (via translate.google.com)";
const char S25790[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.16";
const char S25791[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/12.0.648.205 Safari/534.16";
const char S25792[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/12.0.648.205 Safari/534.16";
const char S25793[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/12.0.648.205 Safari/534.16";
const char S25794[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7,gzip(gfe) (via translate.google.com)";
const char S25795[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7,gzip(gfe) (via translate.google.com)";
const char S25796[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7";
const char S25797[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7";
const char S25798[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/534.7 ZemanaAID/FFFF0061";
const char S25799[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.7 (KHTML	 like Gecko) Chrome/16.0.912.36 Safari/535.7";
const char S25800[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.8 (KHTML	 like Gecko) Chrome/16.0.912.63 Safari/535.8";
const char S25801[] = "Mozilla/5.0 (Linux; Android 4.1.9; GT-I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S25802[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S25803[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S25804[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S25805[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S25806[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S25807[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S25808[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S25809[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S25810[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-I535 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S25811[] = "Mozilla/5.0 (Linux; Android 4.1.2; SC-06D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S25812[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S25813[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999V Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S25814[] = "Mozilla/5.0 (Linux; Android 4.1.9; GT-I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S25815[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-i9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S25816[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S25817[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I939 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S25818[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I93000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S25819[] = "Mozilla/5.0 (Mozilla/5.0; Windows NT 6.1 WOW64; 6.0 ) AppleWebKit/533.4 (KHTML, like Gecko) Firefox/6.0/7.4.1.8 Mobile Safari/533.4 3gpp-gba";
const char S25820[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.3a3pre) Gecko/X Firefox6.0 (.NET CLR 3.5.30729)";
const char S25821[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:6.0.2) Gecko/X Firefox/6.0.2 Firefox/6.0.2";
const char S25822[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:6.0.2) Gecko/X Firefox/6.0.2";
const char S25823[] = "Mozilla/5.0 (Windows NT 6.1; rv:6.0) Gecko/X Firefox/6.00";
const char S25824[] = "Mozilla/5.0 (Windows NT 6.1; rv:6.0) Gecko/X Firefox/6.0 Google (+https://developers.google.com/+/web/snippet/)";
const char S25825[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; rv:6.0a2) Gecko/X Firefox/6.0a2";
const char S25826[] = "Mozilla/5.0 (Windows NT 6.1; rv:6.0) Gecko/X Firefox/6.0 Safari/5.05 Pico/3.2";
const char S25827[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ph; GT-I8190N Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25828[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8190N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25829[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I8190N/I8190NXXALL4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25830[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8190N Build/JZO54K)";
const char S25831[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-I8190N-ORANGE/I8190NXXALL5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25832[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-I8190N-ORANGE/I8190NXXAMB1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25833[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I8190N/I8190NXXAMB2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25834[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I8190N/I8190NXXAMD3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25835[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-I8190N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25836[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; SAMSUNG GT-I8190N/I8190NXXAMG6 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25837[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I8190N/I8190NXXAMG3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S25838[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A4449a Safari/9537.53";
const char S25839[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A4449d Safari/9537.53";
const char S25840[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A458 Safari/9537.53";
const char S25841[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A465 Safari/9537.53";
const char S25842[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A462 Safari/9537.53";
const char S25843[] = "Mozilla/5.0 (iPad; CPU OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B475 Safari/9537.53";
const char S25844[] = "Mozilla/5.0 (iPad; CPU OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B478 Safari/9537.53";
const char S25845[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A4449b Safari/9537.53";
const char S25846[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A465 Safari/";
const char S25847[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Version/7.0 Safari/8536.25";
const char S25848[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A501 Safari/9537.53";
const char S25849[] = "Mozilla/5.0 (iPad; CPU OS7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A465 Safari/9537.53";
const char S25850[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Version/7.0 Safari/8536.25";
const char S25851[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A501 Safari/";
const char S25852[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonMT15i Build/3.0.1.A.0.108) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25853[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; zh-cn; MT15i Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25854[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/3.0.1.A.0.135) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25855[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/3.0.1.A.0.64) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25856[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/3.0.1.A.0.142) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25857[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/3.0.1.A.0.143) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25858[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; SonyEricssonMT15i Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25859[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/4.0.A.2.286) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25860[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; es-es; MT15i Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25861[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; MT15i Build/3.0.1.A.0.146) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25862[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; SonyEricssonMT15iv Build/3.0.1.A.0.146) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25863[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonMT15i-o Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25864[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; SonyEricssonMT15i Build/3.0.A.0.326) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25865[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; MT15i Build/3.0.A.0.326) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25866[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ie; SonyEricssonMT15iv Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25867[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonMT15i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25868[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonMT15i Build/4.0.2.A.0.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25869[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonMT15i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25870[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonMT15i Build/4.0.1.A.2.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25871[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonMT15i Build/4.0.1.A.0.277) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25872[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; sv-se; SonyEricssonMT15i Build/3.0.A.2.15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25873[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; sv-se; SonyEricssonMT15i Build/3.0.A.2.79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25874[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonMT15iv Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25875[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; SonyEricssonMT15i Build/3.0.1.A.0.146) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25876[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; SonyEricssonMT15i Build/3.0.A.2.52) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25877[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/3.0.1.A.0.149) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25878[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/3.0.1.A.0.150) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25879[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; pt-pt; SonyEricssonMT15i Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25880[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; MT15i Build/3.0.1.A.0.135) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25881[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; es-es; SonyEricssonMT15i-o Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25882[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; sv-se; SonyEricssonMT15i Build/3.0.A.2.182) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25883[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; MT15i Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25884[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonMT15i Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25885[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/4.0.A.2.262) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25886[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/4.0.A.2.204) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25887[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; MT15i Build/4.0.A.2.240) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25888[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/4.0.A.2.330) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25889[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/4.0.A.2.305) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25890[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/4.0.A.2.364) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25891[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonMT15i Build/4.0.1.A.0.119) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25892[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/4.0.A.2.362) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25893[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; SonyEricssonMT15i-o Build/3.0.1.A.0.150) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25894[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonMT15i Build/4.0.1.A.0.284) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25895[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MT15i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25896[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMT15i Build/4.0.1.A.0.227) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25897[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; MT15i Build/3.0.1.A.0.150) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25898[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; MT15i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25899[] = "Android (Linux; U; Android 2.3.3; zh-cn; MT15i) AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S25900[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; es-es; SonyEricssonMT15iv Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25901[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; MT15i Build/3.0.1.A.0.145)";
const char S25902[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMT15i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25903[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MT15i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25904[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMT15iv Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25905[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMT15i-o Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25906[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMT15i-o Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25907[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMT15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25908[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MT15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25909[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; MT15i Build/4.0.2.A.0.62)";
const char S25910[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonMT15i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25911[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-za; SonyEricssonMT15iv Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25912[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MT15i Build/4.0.2.A.0.42) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S25913[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; MT15i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25914[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MT15i Build/4.0.2.A.0.62) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S25915[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonMT15i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S25916[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MT15i Build/4.0.2.A.0.62) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S25917[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; MT15i Build/3.0.1.A.0.145) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S25918[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; MT15i Build/4.0.A.2.368) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S25919[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; MT15i Build/3.0.1.A.0.145) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S25920[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MT15i Build/4.0.2.A.0.42) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S25921[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.229 Version/11.61";
const char S25922[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.61";
const char S25923[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.229 Version/11.61";
const char S25924[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.229 Version/11.61";
const char S25925[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.229 Version/11.61";
const char S25926[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.229 Version/11.61";
const char S25927[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.229 Version/11.61";
const char S25928[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5217); ru) Presto/2.10.229 Version/11.61";
const char S25929[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5218); ru) Presto/2.10.229 Version/11.61";
const char S25930[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5161); ru) Presto/2.10.229 Version/11.61";
const char S25931[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.229 Version/11.61";
const char S25932[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5230); ru) Presto/2.10.229 Version/11.61";
const char S25933[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.229 Version/11.61,gzip(gfe) (via translate.google.com)";
const char S25934[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4661); ru) Presto/2.10.229 Version/11.61";
const char S25935[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; ru) Opera 11.61";
const char S25936[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.229 Version/11.61";
const char S25937[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.229 Version/11.61";
const char S25938[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.229 Version/11.61";
const char S25939[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.61";
const char S25940[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.229 Version/11.61";
const char S25941[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03278); ru) Presto/2.10.229 Version/11.61";
const char S25942[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.229 Version/11.61";
const char S25943[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4110); ru) Presto/2.10.229 Version/11.61";
const char S25944[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.229 Version/11.61";
const char S25945[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.229 Version/11.61";
const char S25946[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.229 Version/11.61";
const char S25947[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.61";
const char S25948[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4930); ru) Presto/2.10.229 Version/11.61";
const char S25949[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.229 Version/11.61";
const char S25950[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.229 Version/11.61";
const char S25951[] = "Mozilla/5.0 (Windows NT 6.1; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.61";
const char S25952[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.229 Version/11.61";
const char S25953[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.229 Version/11.61";
const char S25954[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.229 Version/11.61";
const char S25955[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5328); ru) Presto/2.10.229 Version/11.61";
const char S25956[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.229 Version/11.61";
const char S25957[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.61";
const char S25958[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.229 Version/11.61";
const char S25959[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.10.229 Version/11.61";
const char S25960[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.229 Version/11.61";
const char S25961[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.229 Version/11.61";
const char S25962[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.61";
const char S25963[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.229 Version/11.61";
const char S25964[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.229 Version/11.61";
const char S25965[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03780); ru) Presto/2.10.229 Version/11.61";
const char S25966[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.229 Version/11.61";
const char S25967[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5278); ru) Presto/2.10.229 Version/11.61";
const char S25968[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.229 Version/11.61";
const char S25969[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.229 Version/11.61";
const char S25970[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.229 Version/11.61";
const char S25971[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.229 Version/11.61";
const char S25972[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03755); ru) Presto/2.10.229 Version/11.61";
const char S25973[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03639); ru) Presto/2.10.229 Version/11.61";
const char S25974[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5288); ru) Presto/2.10.229 Version/11.61";
const char S25975[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.229 Version/11.61";
const char S25976[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5299); ru) Presto/2.10.229 Version/11.61";
const char S25977[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.10.229 Version/11.61";
const char S25978[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5667); ru) Presto/2.10.229 Version/11.61";
const char S25979[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.229 Version/11.61";
const char S25980[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.10.229 Version/11.61";
const char S25981[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.10.229 Version/11.61";
const char S25982[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.229 Version/11.61";
const char S25983[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03649); ru) Presto/2.10.229 Version/11.61";
const char S25984[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.229 Version/11.61";
const char S25985[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.229 Version/11.61,gzip(gfe)";
const char S25986[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.229 Version/11.61";
const char S25987[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.229 Version/11.61";
const char S25988[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03402); ru) Presto/2.10.229 Version/11.61";
const char S25989[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5993); ru) Presto/2.10.229 Version/11.61";
const char S25990[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.229 Version/11.61";
const char S25991[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5966); ru) Presto/2.10.229 Version/11.61";
const char S25992[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6011); ru) Presto/2.10.229 Version/11.61";
const char S25993[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.229 Version/11.61";
const char S25994[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.229 Version/11.61";
const char S25995[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6081); ru) Presto/2.10.229 Version/11.61";
const char S25996[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5676); ru) Presto/2.10.229 Version/11.61";
const char S25997[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5880); ru) Presto/2.10.229 Version/11.61";
const char S25998[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6083); ru) Presto/2.10.229 Version/11.61";
const char S25999[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5784); ru) Presto/2.10.229 Version/11.61";
const char S26000[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6089); ru) Presto/2.10.229 Version/11.61";
const char S26001[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5977); ru) Presto/2.10.229 Version/11.61";
const char S26002[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5989); ru) Presto/2.10.229 Version/11.61";
const char S26003[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6037); ru) Presto/2.10.229 Version/11.61";
const char S26004[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6054); ru) Presto/2.10.229 Version/11.61";
const char S26005[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.229 Version/11.61";
const char S26006[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5981); ru) Presto/2.10.229 Version/11.61";
const char S26007[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6048); ru) Presto/2.10.229 Version/11.61";
const char S26008[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 6008); ru) Presto/2.10.229 Version/11.61";
const char S26009[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5990); ru) Presto/2.10.229 Version/11.61";
const char S26010[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6683); ru) Presto/2.10.229 Version/11.61";
const char S26011[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6638); ru) Presto/2.10.229 Version/11.61";
const char S26012[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6639); ru) Presto/2.10.229 Version/11.61";
const char S26013[] = "BlackBerry9000/4.6.0.167 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102 ips-agent";
const char S26014[] = "Opera/9.80 (iPad; Opera Mini/7.0.5/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S26015[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.45 Safari/535.19";
const char S26016[] = "Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19";
const char S26017[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19";
const char S26018[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19,gzip(gfe) (via translate.google.com)";
const char S26019[] = "Mozilla/5.0 (X11; Jolicloud Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Joli OS/1.2 Chromium/18.0.1025.162 Chrome/18.0.1025.162 Safari/535.19";
const char S26020[] = "User-Agent: Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19";
const char S26021[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.45 Safari/535.19,gzip(gfe) (via translate.google.com)";
const char S26022[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.45 Safari/535.19,gzip(gfe)";
const char S26023[] = "Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/18.0.1410.63 Safari/537.31";
const char S26024[] = "Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/18.0.1410.63 Safari/537.31";
const char S26025[] = "Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/18.0.1410.63 Safari/537.31";
const char S26026[] = "Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/18.0.1410.63 Safari/537.31";
const char S26027[] = "Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/18.0.1410.63 Safari/537.31";
const char S26028[] = "Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/18.0.1410.63 Safari/537.31";
const char S26029[] = "Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/18.0.1410.63 Safari/537.31";
const char S26030[] = "Mozilla/5.0 (X11; Linux x86_64; en-us) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.45 Safari/535.19";
const char S26031[] = "Mozilla/5.0 (Linux; U; X11; en-US; Valve Client/0; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S26032[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; M865 Build/HuaweiM865) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26033[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; M865 Build/HuaweiM865) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26034[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; M865 Build/HuaweiM865) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26035[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; USCCADR3305 Build/HuaweiM865) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26036[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; USCCADR3305 Build/HuaweiM865) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26037[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; USCCADR3305 Build/HuaweiM865) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S26038[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; M865 Build/HuaweiM865) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 androidAppVersion/2.3.1 AndroidWebkitWrapper BillingSupported";
const char S26039[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; M865 Build/HuaweiM865)";
const char S26040[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; M865 Build/HuaweiM865) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26041[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; M865 Build/HuaweiM865)";
const char S26042[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; M865 Build/HuaweiM865)";
const char S26043[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; M865C Build/HuaweiM865C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26044[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; M865C Build/HuaweiM865C)";
const char S26045[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; USCCADR3305 Build/HuaweiM865)";
const char S26046[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; USCCADR3305 Build/HuaweiM865)";
const char S26047[] = "HuaweiM865/M865C153B826 CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 2.3.3)";
const char S26048[] = "HuaweiM865C/C378B876 CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 2.3.6)";
const char S26049[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ASUS Transformer Pad TF300T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S26050[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ASUS Transformer Pad TF300T Build/JRO03C)";
const char S26051[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ASUS Transformer Pad TF300TG Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S26052[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ASUS Transformer Pad TF300TG Build/JRO03C)";
const char S26053[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; ASUS Pad TF300T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S26054[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ASUS Transformer Pad TF300TL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S26055[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ASUS Transformer Pad TF300T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26056[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Transformer Pad TF300T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S26057[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ASUS Transformer Pad TF300TL Build/JRO03C)";
const char S26058[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ASUS Transformer Pad TF300T Build/BlueAndroid@V3.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S26059[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ASUS Transformer Pad TF300T Build/JB@OC_V03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S26060[] = "com.google.android.apps.plus/X (Linux; U; Android 4.2; en_US; Nexus 7; Build/JOP23E) (gzip)";
const char S26061[] = "com.google.android.videos/2.2.2(Linux; U; Android 4.2; en_US; Nexus 7 Build/JOP23E) gzip";
const char S26062[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 7 Build/JOP03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 GSA/2.0.6.X";
const char S26063[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 7 Build/JOP08B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S26064[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 7 Build/JOP11B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S26065[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 7 Build/JOP09D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S26066[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 7 Build/JVP15S) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S26067[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 7 Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S26068[] = "Dalvik/1.6.0 (Linux; U; Android 4.2; Nexus 7 Build/JOP40C)";
const char S26069[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; Nexus 7 Build/JOP40D)";
const char S26070[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 7 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S26071[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-; Nexus 7 Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S26072[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-; Nexus 7 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S26073[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 7 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26074[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Nexus 7 Build/JOP40D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S26075[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 7 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S26076[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S26077[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; Nexus 7 Build/JDQ39)";
const char S26078[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26079[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-; Nexus 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S26080[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S26081[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S26082[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 7 Build/JRN84D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S26083[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 3G Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S26084[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S26085[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11,gzip(gfe) (via translate.google.com)";
const char S26086[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11,gzip(gfe) (via docs.google.com/viewer)";
const char S26087[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.70 Safari/535.11";
const char S26088[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.8 (KHTML	 like Gecko) Chrome/17.0.932.0 Safari/535.8";
const char S26089[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/17.0.944.0 Safari/535.8 BitdefenderSafepay/0.2.0.0 (OBK; 17.0.944.0; r396.2011)";
const char S26090[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.12 Safari/535.11";
const char S26091[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/17.0.944.0 Safari/535.8 BitdefenderSafepay/0.2.0.0 (OBK; 17.0.944.0; r396.2011)";
const char S26092[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.84 Safari/535.11 SE 2.X MetaSr 1.0,gzip(gfe)";
const char S26093[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.45 Safari/535.19";
const char S26094[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.54 Safari/535.19";
const char S26095[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/180.1.1025.40 Safari/535.19";
const char S26096[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S26097[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S26098[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S26099[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/180.1.1025.40 Safari/535.19";
const char S26100[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S26101[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/180.1.1025.40 Safari/535.19";
const char S26102[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.20 (KHTML, like Gecko) Chrome/19.0.1036.7 Safari/535.20";
const char S26103[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.24 (KHTML, like Gecko) Chrome/19.0.1055.1 Safari/535.24";
const char S26104[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5";
const char S26105[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S26106[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S26107[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5,gzip(gfe) (via docs.google.com/viewer)";
const char S26108[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S26109[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5";
const char S26110[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5";
const char S26111[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5";
const char S26112[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; GT-S5570L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26113[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; GT-S5570-ORANGE/S5570BVKE1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26114[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5570-ORANGE/S5570BVKE2 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26115[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pl-pl; GT-S5570-ORANGE/S5570BVKC4 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26116[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; GT-S5570B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26117[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ro-ro; GT-S5570-ORANGE/S5570BVKC3 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26118[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; GT-S5570B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26119[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26120[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-S5570 Build/FROYO)";
const char S26121[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5570 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S26122[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-S5570L Build/FROYO)";
const char S26123[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-S5570B Build/FROYO)";
const char S26124[] = "Mozilla/5.0(Linux;U;Android2.2.1;Zh_cn;GT-S5570;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S26125[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26126[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; GT-S5570L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26127[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-S5570 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S26128[] = "Opera/9.80 (Series 60; Opera Mini/6.1.26266/26.1069; U; en) Presto/2.8.119 Version/10.54";
const char S26129[] = "Opera/9.80 (Series 60; Opera Mini/6.1.25570/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S26130[] = "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:17.0) Gecko/X Firefox/17.0";
const char S26131[] = "Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.10.289 Version/12.02";
const char S26132[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.10.289 Version/12.02";
const char S26133[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.10.289 Version/12.02";
const char S26134[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0); en) Presto/2.10.289 Version/12.02";
const char S26135[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5972); ru) Presto/2.10.289 Version/12.02";
const char S26136[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5970); ru) Presto/2.10.289 Version/12.02";
const char S26137[] = "Opera/9.80 (Windows NT 5.1; U; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0); en) Presto/2.10.289 Version/12.02";
const char S26138[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4953); ru) Presto/2.10.289 Version/12.02";
const char S26139[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5310); ru) Presto/2.10.289 Version/12.02";
const char S26140[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.10.289 Version/12.02";
const char S26141[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5339); ru) Presto/2.10.289 Version/12.02";
const char S26142[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5831); ru) Presto/2.10.289 Version/12.02";
const char S26143[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5754); ru) Presto/2.10.289 Version/12.02";
const char S26144[] = "Opera/9.80 (Windows NT 6.0; U; pl) Presto/2.10.289 Version/12.02,gzip(gfe) (via translate.google.com)";
const char S26145[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.4 (build 02647); ru) Presto/2.10.289 Version/12.02";
const char S26146[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5976); ru) Presto/2.10.289 Version/12.02";
const char S26147[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5711); ru) Presto/2.10.289 Version/12.02";
const char S26148[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; en) Opera 12.02";
const char S26149[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5680); ru) Presto/2.10.289 Version/12.02";
const char S26150[] = "Mozilla/5.0 (Windows NT 6.0; en; rv:2.0) Gecko/X Firefox/4.0 Opera 12.02";
const char S26151[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.6 (build 03278); ru) Presto/2.10.289 Version/12.02";
const char S26152[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4157); ru) Presto/2.10.289 Version/12.02";
const char S26153[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03686); ru) Presto/2.10.289 Version/12.02";
const char S26154[] = "Opera/9.80 (Windows NT 6.1; U; compatible; MSIE 7.0; Windows NT 6.0; bg) Presto/2.10.289 Version/12.02";
const char S26155[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03790); ru) Presto/2.10.289 Version/12.02";
const char S26156[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4133); ru) Presto/2.10.289 Version/12.02";
const char S26157[] = "Opera/9.80 (Windows NT 6.0; U; pl) Presto/2.10.289 Version/12.02,gzip(gfe)";
const char S26158[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 6001); ru) Presto/2.10.289 Version/12.02";
const char S26159[] = "Opera/12.02 (Windows NT 6.0; U; en) Presto/2.10.289 Version/12.02";
const char S26160[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4876); ru) Presto/2.10.289 Version/12.02";
const char S26161[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5998); ru) Presto/2.10.289 Version/12.02";
const char S26162[] = "Opera/9.80 (Windows NT 6.0; U; MRA 8.0 (build 5745); ru) Presto/2.10.289 Version/12.02";
const char S26163[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 6005); ru) Presto/2.10.289 Version/12.02";
const char S26164[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5244); ru) Presto/2.10.289 Version/12.02";
const char S26165[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4598); ru) Presto/2.10.289 Version/12.02";
const char S26166[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 6015); ru) Presto/2.10.289 Version/12.02";
const char S26167[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4664); ru) Presto/2.10.289 Version/12.02";
const char S26168[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 6068); ru) Presto/2.10.289 Version/12.02";
const char S26169[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5328); ru) Presto/2.10.289 Version/12.02";
const char S26170[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.3 (build 02564); ru) Presto/2.10.289 Version/12.02";
const char S26171[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5231); ru) Presto/2.10.289 Version/12.02";
const char S26172[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.6 (build 03402); ru) Presto/2.10.289 Version/12.02";
const char S26173[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.4 (build 02652); ru) Presto/2.10.289 Version/12.02";
const char S26174[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5993); ru) Presto/2.10.289 Version/12.02";
const char S26175[] = "Mozilla/5.0 (Windows NT 6.2; rv:16.0) Gecko/X Firefox/16.0";
const char S26176[] = "Mozilla/6.0 (Windows NT 6.2; WOW64; rv:16.0.1) Gecko/X Firefox/16.0.1";
const char S26177[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:16.0.1) Gecko/X Firefox/16.0.1";
const char S26178[] = "Mozilla/5.0 (Windows NT 6.1; rv:2.0) Gecko/20100101 Firefox/4.0";
const char S26179[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:2.0b10) Gecko/X Firefox/4.0b10";
const char S26180[] = "Mozilla/4.0 (Windows NT 6.1); Gecko/X Firefox/4.0.1; 10; Windows NT 6.1)";
const char S26181[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.0) Gecko/X Firefox/4.0";
const char S26182[] = "Mozilla/4.0 (Windows NT 6.1); Gecko/X Firefox/4.0.1; 10; Windows NT 6.1; Trident/4.0; InfoPath.2)";
const char S26183[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0.1) Gecko/X Firefox/4.0.1 Firefox/4.0.1";
const char S26184[] = "Mozilla/5.0 (Windows NT 6.1); Gecko/X Firefox/4.0.1; 10; Windows NT 6.1; Trident/6.0)";
const char S26185[] = "Mozilla/5.0 (Windows NT 6.1) Gecko/X Firefox/4.0";
const char S26186[] = "Mozilla/4.0 (compatible; Windows NT 6.1; Win64; x64; rv:2.0b8pre; Gecko/X Firefox/4.0b8pre)";
const char S26187[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S26188[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S26189[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)";
const char S26190[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via docs.google.com/viewer)";
const char S26191[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via docs.google.com/viewer)";
const char S26192[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe)";
const char S26193[] = "Mozilla/5.0 (Windows NT 6.0; Neustar WPM) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1";
const char S26194[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/21.0.597.0 Safari/534.13";
const char S26195[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:22.0) Gecko/X Firefox/22.0 AlexaToolbar/alxf-2.18";
const char S26196[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:22.0) Gecko/X Firefox/22.0 IceDragon/22.0.0.1";
const char S26197[] = "<![CDATA[Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:22.0) Gecko/X Firefox/22.0";
const char S26198[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S6102 Build/GINGERBREAD)";
const char S26199[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S26200[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26201[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-S6102B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26202[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6102E Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26203[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S6102B Build/GINGERBREAD)";
const char S26204[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26205[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6102E Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S26206[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S26207[] = "BlackBerry9000/5.0.0.1067 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102";
const char S26208[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:23.0) Gecko/X Firefox/23.0 AlexaToolbar/alxf-2.18";
const char S26209[] = "Mozilla/5.0 (Windows; Windows NT 6.1; Win64; x64; rv:23.0) Gecko/23.0 Firefox/23.0";
const char S26210[] = "<![CDATA[Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:23.0) Gecko/X Firefox/23.0";
const char S26211[] = "Mozilla/5.0 (Linux; U; Android2.3.3; en-us; C771 Build/C771M070) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26212[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M070) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S26213[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M070) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26214[] = "Logos/0.9.4 (Linux; U; Android 2.3.3; en_US; C771 Build/GINGERBREAD)";
const char S26215[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26216[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26217[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M080) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26218[] = "Mozilla/5.0 (Linux; U; Android2.3.3; en-us; C771 Build/C771M100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26219[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M110) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S26220[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M120) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26221[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;C771 Build/C771M070) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26222[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;C771 Build/C771M100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26223[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771Build/C771M100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26224[] = "Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; C771 Build/C771M100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26225[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771Build/C771M070) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26226[] = "Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; C771 Build/C771M080) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26227[] = "Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; C771 Build/C771M070) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26228[] = "Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; C771 Build/C771M110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26229[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771Build/C771M110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26230[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S26231[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M120) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S26232[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;C771 Build/C771M120) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26233[] = "Mozilla/5.0 (Linux; U; Android2.3.3; en-us; C771 Build/C771M120) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26234[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S26235[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;C771 Build/C771M110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26236[] = "Mozilla/5.0 (Linux; U; Android2.3.3; en-us; C771 Build/C771M110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26237[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26238[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; C771 Build/C771M130)";
const char S26239[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;C771 Build/C771M130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26240[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771Build/C771M130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26241[] = "Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; C771 Build/C771M130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26242[] = "Mozilla/5.0 (Linux; U; Android2.3.3; en-us; C771 Build/C771M130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26243[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S26244[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M140) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26245[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; XT910 Build/6.7.1_SPU-ICS-DUMMY) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26246[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MOT-XT910 Build/6.7.3-94_SPI-324) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26247[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MOT-XT910 Build/6.7.2-180_SPU-19-TA-11.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26248[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MOT-XT910 Build/6.7.2-180_SLC-29_R01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26249[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT910 Build/6.7.3-94_SPI-324) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26250[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MOT-XT910 Build/6.7.2-180_SLC-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26251[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MOT-XT910 Build/6.7.3-94_SPI-328) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26252[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT910 Build/6.7.3-94_SPI-328) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26253[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26254[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; MOT-XT910 Build/6.7.2-180_SLC-35_R02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26255[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOT-XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26256[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT910 Build/6.7.2_GC-145-SPDU-10) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S26257[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MOT-XT910 Build/6.7.2_GC-145-SPDU-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26258[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT910 Build/6.7.2-180_SLC-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26259[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MOT-XT910 Build/6.7.2-180_SLC-34) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26260[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOT-XT910 Build/6.7.6-226_SPU-DE-TA-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26261[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XT910 Build/6.7.6-226_SPU-DE-TA-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26262[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; MOT-XT910K Build/6.7.4-27_SA-17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26263[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOT-XT910 Build/6.7.2-180_SPU-19-TA-5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26264[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; XT910 Build/6.7.2-180_SLC-35_R02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26265[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MOT-XT910 Build/6.7.2-180_SLC-29) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26266[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT910 Build/6.7.2-180_SPU-19-TA-11.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26267[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; MOT-XT910S Build/6.7.4-27_SA-17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26268[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT910 Build/6.7.2-180_SLC-35) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26269[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; XT910 Build/6.7.2-180_SPU-19-TA-11.6)";
const char S26270[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MOT-XT910 Build/6.7.2-180_SLC-35) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26271[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XT910 Build/6.7.2-180_SPU-19-TA-5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26272[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT910 Build/6.7.2-180_SLC-34) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26273[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MOT-XT910 Build/ULTIMOBILE-RAZRME-4.0.4-MTP-X-2.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26274[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; MOT-XT910 Build/6.7.2-180_SPU-19TA11.9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26275[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MOT-XT910 Build/6.7.2-180_SLC-34_R01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26276[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MOT-XT910 Build/6.7.2-180_SLC-35_R01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26277[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MOT-XT910 Build/6.7.2_GC-213-SPDU-6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26278[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MOT-XT910 Build/6.7.2-180_SLC-29_R03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26279[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MOT-XT910 Build/6.7.2-180_SLC-31_R01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26280[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; MOT-XT910 Build/6.7.3-94_SPI-329) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26281[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; XT910 Build/6.7.2-180_SLC-29_R01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26282[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT910 Build/6.7.3-94_SPI-328) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S26283[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; XT910 Build/6.7.2-180_SPU-19TA11.9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26284[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MOT-XT910 Build/6.7.2_GC-145-SPDUF-910) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26285[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT910 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26286[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MOT-XT910 Build/ULTIMOBILE-RAZR-4.0.4-VIA-X-1.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26287[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; MOT-XT910K Build/6.7.4-27_SA-46) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26288[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT910 Build/6.7.2_GC-145-SPDUF-910) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S26289[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; XT910 Build/6.7.3-94_SPI-329) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26290[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT910 Build/6.7.2_GC-213-SPDU-6) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S26291[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MOT-XT910 Build/6.7.2_GC-213-SPDUF-906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26292[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; MOT-XT910S Build/6.7.4-27_SA-51) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26293[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT910 Build/6.7.2-180_SLC-35_R01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26294[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT910 Build/6.7.2-180_SLC-29_R03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26295[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT910 Build/6.7.2-180_SPU-19-TA-5) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S26296[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOT-XT910 Build/6.7.2-152_SPU-17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26297[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; XT910 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26298[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT910 Build/ULTIMOBILE-RAZRME-4.0.4-MTP-X-2.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26299[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; XT910K Build/6.7.4-27_SA-46) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26300[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT910 Build/6.7.2-180_SLC-34_R01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26301[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; XT910 Build/6.7.2-180_SLC-31_R01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26302[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT910 Build/6.7.2_GC-213-SPDU-6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26303[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT910 Build/ULTIMOBILE-RAZR-4.0.4-VIA-X-1.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26304[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT910 Build/6.7.2-180_SLC-29) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26305[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOT-XT910 Build/6.7.3-94_SPI-32) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26306[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT910 Build/6.7.2_GC-213-SPDUF-906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26307[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT910 Build/6.7.2_GC-213-SPDUF-906) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S26308[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MOT-XT910 Build/6.7.2-180_SPU-19-TA-11.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26309[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-; MOT-XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26310[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT910 Build/6.7.2_GC-145-SPDUF-910) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26311[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MOT-XT910 Build/ULTIMOBILE-RAZR-4.0.4-VIA-X-1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26312[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; MOT-XT910 Build/6.7.3-94_SPI-326) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26313[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MOT-XT910 Build/ULTIMOBILE-RAZR-4.0.4-MTP-X-1.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26314[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.7.62 Version/11.00";
const char S26315[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.7.62 Version/11.00,gzip(gfe) (via translate.google.com)";
const char S26316[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.7.62 Version/11.00";
const char S26317[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.7.62 Version/11.00";
const char S26318[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.7.62 Version/11.00";
const char S26319[] = "Opera/9.80 (Windows NT 5.1; U; en) Presto/2.7.62 Version/11.01";
const char S26320[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.7.62 Version/11.01";
const char S26321[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.7.62 Version/11.01,gzip(gfe) (via translate.google.com)";
const char S26322[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.7.62 Version/11.01";
const char S26323[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.7.62 Version/11.00";
const char S26324[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.7.62 Version/11.00";
const char S26325[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.7.62 Version/11.00";
const char S26326[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.7.62 Version/11.01";
const char S26327[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.7.62 Version/11.01";
const char S26328[] = "Opera/9.80 (Windows NT 5.2; U; ru) Presto/2.7.62 Version/11.01";
const char S26329[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.7.62 Version/11.01";
const char S26330[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.7.62 Version/11.01";
const char S26331[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.7.62 Version/11.01";
const char S26332[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.7.62 Version/11.01";
const char S26333[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.7.62 Version/11.00";
const char S26334[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.7.62 Version/11.01";
const char S26335[] = "Opera/9.80 (Windows NT 5.2; U; ru) Presto/2.7.62 Version/11.00";
const char S26336[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.7.62 Version/11.00";
const char S26337[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; en) Opera 11.01";
const char S26338[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.7.62 Version/11.01";
const char S26339[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.7.62 Version/11.00";
const char S26340[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.7.62 Version/11.00";
const char S26341[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.7.62 Version/11.01";
const char S26342[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.7.62 Version/11.00";
const char S26343[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.7.62 Version/11.00";
const char S26344[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.7.62 Version/11.00";
const char S26345[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.7.62 Version/11.00";
const char S26346[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.7.62 Version/11.01";
const char S26347[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94; ru) Presto/2.7.62 Version/11.01";
const char S26348[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 3797); ru) Presto/2.7.62 Version/11.01";
const char S26349[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.7.62 Version/11.01";
const char S26350[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.7.62 Version/11.00";
const char S26351[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.7.62 Version/11.00";
const char S26352[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.7.62 Version/11.00";
const char S26353[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.7.62 Version/11.01";
const char S26354[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.7.62 Version/11.00";
const char S26355[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.7.62 Version/11.01";
const char S26356[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.7.62 Version/11.01";
const char S26357[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.7.62 Version/11.00";
const char S26358[] = "Mozilla/5.0 (Windows NT 5.1; U; de; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 11.00";
const char S26359[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.7.62 Version/11.01";
const char S26360[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; pl) Opera 11.00";
const char S26361[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.7.62 Version/11.01";
const char S26362[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.7.62 Version/11.01";
const char S26363[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.7.62 Version/11.01";
const char S26364[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.7.62 Version/11.00";
const char S26365[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.7.62 Version/11.01";
const char S26366[] = "Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 11.01";
const char S26367[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.7.62 Version/11.01";
const char S26368[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.7.62 Version/11.01";
const char S26369[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.7.62 Version/11.00";
const char S26370[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.7.62 Version/11.00";
const char S26371[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.7.62 Version/11.01";
const char S26372[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.7.62 Version/11.00";
const char S26373[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.7.62 Version/11.00";
const char S26374[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.7.62 Version/11.01";
const char S26375[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.7.62 Version/11.01";
const char S26376[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5218); ru) Presto/2.7.62 Version/11.01";
const char S26377[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.7.62 Version/11.00";
const char S26378[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.7.62 Version/11.00";
const char S26379[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.7.62 Version/11.01";
const char S26380[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.7.62 Version/11.00";
const char S26381[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4841); ru) Presto/2.7.62 Version/11.00";
const char S26382[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.7.62 Version/11.01";
const char S26383[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.7.62 Version/11.00";
const char S26384[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.7.62 Version/11.00";
const char S26385[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.7.62 Version/11.00";
const char S26386[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.7.62 Version/11.00";
const char S26387[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4871); ru) Presto/2.7.62 Version/11.00";
const char S26388[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5328); ru) Presto/2.7.62 Version/11.00";
const char S26389[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.7.62 Version/11.01";
const char S26390[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.7.62 Version/11.01";
const char S26391[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.7.62 Version/11.00";
const char S26392[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.7.62 Version/11.00";
const char S26393[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03791); ru) Presto/2.7.62 Version/11.00";
const char S26394[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5267); ru) Presto/2.7.62 Version/11.00";
const char S26395[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5328); ru) Presto/2.7.62 Version/11.01";
const char S26396[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.7.62 Version/11.01";
const char S26397[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.7.62 Version/11.00,gzip(gfe)";
const char S26398[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.7.62 Version/11.01";
const char S26399[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.7.62 Version/11.01";
const char S26400[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.7.62 Version/11.01";
const char S26401[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.7.62 Version/11.01";
const char S26402[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.7.62 Version/11.00";
const char S26403[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.7.62 Version/11.01,gzip(gfe)";
const char S26404[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5267); ru) Presto/2.7.62 Version/11.01";
const char S26405[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.7.62 Version/11.00";
const char S26406[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.7.62 Version/11.00";
const char S26407[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.7.62 Version/11.00";
const char S26408[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5282); ru) Presto/2.7.62 Version/11.00";
const char S26409[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.7.62 Version/11.01";
const char S26410[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.7.62 Version/11.01";
const char S26411[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.7.62 Version/11.00";
const char S26412[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5667); ru) Presto/2.7.62 Version/11.00";
const char S26413[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.7.62 Version/11.01";
const char S26414[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.7.62 Version/11.01";
const char S26415[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6089); ru) Presto/2.7.62 Version/11.00";
const char S26416[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6578); ru) Presto/2.7.62 Version/11.00";
const char S26417[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-GB; rv:1.8.1.6) Gecko/X Firefox/2.0	Firefox 2.0.0.6";
const char S26418[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.8.1a2) Gecko/X BonEcho/2.0a2";
const char S26419[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.20) Gecko/X Firefox/2.0.0.20 GTB7.0 (.NET CLR 3.5.30729) WinNT-PAI 21.07.2009";
const char S26420[] = "Mozilla/5.0 (Windows; U; WindowsNT 5.1; en-US; rv1.8.1.6) Gecko/X Firefox/2.0.0.6";
const char S26421[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1) Gecko/X Firefox/2.0.0.3";
const char S26422[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.20) Gecko/X Firefox/2.0.0.20 (.NET CLR 3.5.30729) WinNT-EVI 18.05.2010";
const char S26423[] = "Mozilla/5.0 (Windows; U;XMPP Tiscali Communicator v.10.0.1; Windows NT 5.1; it; rv:1.8.1.3) Gecko/X Firefox/2.0.0.3";
const char S26424[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-GB; rv:1.8.1.13) Gecko/X Firefox/2.0.0.13";
const char S26425[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1b2) Gecko/X BonEcho/2.0b2";
const char S26426[] = "Mozilla/5.0 (http://vx.netlux.org/ Windows; U; Windows NT 5.1; en-US; rv:1.8.1.9) Gecko/X Firefox/2.0.0.9";
const char S26427[] = "Mozilla/5.0+(Windows;+U;+Windows+NT+5.1;en-US;+rv:1.8.1.6)+Gecko/X+Firefox/2.0.0.6";
const char S26428[] = "Mozilla/5.0 (Windows NT 5.1; rv:9.0.1) Gecko/X Firefox/2.0";
const char S26429[] = "('Mozilla/5.0 (Windows; U; Windows NT 5.1; en-GB; rv:1.8.1.6) Gecko/X Firefox/2.0.0.6', 'Firefox 2.0, Windows XP')";
const char S26430[] = "Mozilla/6.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.1) Gecko/X Firefox/2.0.0.1";
const char S26431[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC One Build/JDQ39)";
const char S26432[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC_One Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26433[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC One Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26434[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC_PN07120 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26435[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26436[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; HTC_One Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26437[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-tw; HTC One 801e Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26438[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC One 801e Build/JDQ39)";
const char S26439[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; HTC_PN071 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26440[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC_PN07120/1.26.502.10 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26441[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; One Build/JZO54K)";
const char S26442[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC_One/2.24.161.1 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26443[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; HTC_PN071-orange-LS/2.24.73.1 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26444[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC_PN071 Build/JDQ39)";
const char S26445[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC_PN07120/1.26.502.12 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26446[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26447[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; HTC_PN071-orange-LS/2.24.75.1 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26448[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; pl-pl; HTC_PN071-orange-LS/2.24.69.1 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26449[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; HTC_One/2.24.163.2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26450[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; HTC_PN071-orange-LS/2.24.73.3 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26451[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; de-de; HTC_One/2.24.111.3 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26452[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-; HTC One Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26453[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26454[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; HTC_PN071-orange-LS/2.24.73.8 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26455[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ru-ru; HTC One dual sim Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26456[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; HTC One Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26457[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT28at Build/6.1.C.1.105) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26458[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyEricssonLT28at Build/6.1.C.1.105) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26459[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SonyEricssonLT28i Build/6.1.E.1.19) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26460[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT28h Build/6.1.E.0.233) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26461[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyEricssonLT28h Build/6.1.E.1.20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26462[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LT28i Build/6.1.E.1.19) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26463[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT28at Build/6.1.C.1.105)";
const char S26464[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; LT28at Build/6.1.C.1.105) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26465[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT28h Build/6.1.E.1.20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26466[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT28h Build/6.1.E.3.7)";
const char S26467[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; LT28h Build/6.1.E.2.68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26468[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT28h Build/6.1.E.3.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26469[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT28h Build/6.1.E.0.233) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S26470[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LT28i Build/6.1.E.2.68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26471[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT28h Build/6.1.E.1.20) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S26472[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT28i Build/6.1.E.3.7)";
const char S26473[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LT28i Build/6.1.E.3.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26474[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT28h Build/6.1.E.3.7) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S26475[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT28i Build/6.1.E.3.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26476[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT28at Build/6.1.C.1.111) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26477[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.0.4;dev=LT28at;devm=LT28at;man=Sony Ericsson;brw=Android;shwd=720x1280;shwa=720x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S26478[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/12";
const char S26479[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.91 Safari/534.30";
const char S26480[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S26481[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30,gzip(gfe) (via translate.google.com)";
const char S26482[] = "MMozilla/5.0 (Windows NT 6.0) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30 ChromePlus/1.6.3.0alpha4";
const char S26483[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.27 (KHTML, like Gecko) Chrome/12.0.712.0 (Chromium Portable)";
const char S26484[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/12.0.702.0 Safari/533.3 Awesomium/1.6.5";
const char S26485[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.30 (KHTML	 like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S26486[] = "Mozilla/5.0 (Windows NT 6.0; AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30; Windows NT 6.0; Trident/5.0)";
const char S26487[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; de) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/12.0 Safari/525.19";
const char S26488[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.91 Safari/534.30,gzip(gfe) (via docs.google.com/viewer)";
const char S26489[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.30 (KHTML like Gecko) Chrome/12.0.742.100 Safari/534.30";
const char S26490[] = "Mozilla/5.0 (Windows NT 6.0; AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30; Windows NT 6.0; Trident/6.0)";
const char S26491[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-gb; Nexus S Build/IML63B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26492[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S Build/JRO03E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26493[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26494[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; Nexus S Build/JRN83C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26495[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Nexus S Build/JRO03E)";
const char S26496[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26497[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Nexus S Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26498[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Nexus S Build/JRO03E) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S26499[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Full Android on Crespo Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26500[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26501[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Nexus S Build/JZO54K)";
const char S26502[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26503[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Nexus S Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S26504[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; Nexus S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26505[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A5355d Safari/8536.25";
const char S26506[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A5376e Safari/8536.25";
const char S26507[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A403 Safari/8536.25";
const char S26508[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A406 Safari/8536.25";
const char S26509[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S26510[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10A523 Version/6.0 Safari/8536.25 Safari/6533.18.5";
const char S26511[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10A403 Version/6.0 Safari/8536.25 Safari/6533.18.5";
const char S26512[] = "Mozilla/5.0 (iPod; CPU iPhone OS 60 like Mac OS X) AppleWebKit/536.26 (KHTML like Gecko) Version/6.0 Mobile/10A403 Safari/8536.25";
const char S26513[] = "Mozilla/5.0 (iPod; CPU iPhone OS 60 like Mac OS X) AppleWebKit/536.26 (KHTML like Gecko) Version/6.0 Mobile/10A406 Safari/8536.25";
const char S26514[] = "Mozilla/5.0 (iPod; CPU iPhone OS 601 like Mac OS X) AppleWebKit/536.26 (KHTML like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S26515[] = "Mozilla/5.0 (iPod; CPU iPhone OS 60 like Mac OS X) AppleWebKit/536.26 (KHTML like Gecko) Version/6.0 Mobile/10A5376e Safari/8536.25";
const char S26516[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S26517[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S26518[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S26519[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S26520[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S26521[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S26522[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S26523[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530C Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S26524[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S26525[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S26526[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747M Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S26527[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S26528[] = "Mozilla/5.0 (Linux; Android 4.1.1; SPH-L710 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S26529[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S26530[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S26531[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S26532[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-R530U Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S26533[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S26534[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S26535[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305N Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S26536[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/25.0.1364.97) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22";
const char S26537[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/25.0.1364.97) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22";
const char S26538[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/25.0.1364.160) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.160 Safari/537.22";
const char S26539[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/25.0.1364.172)";
const char S26540[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/25.0.1364.172)";
const char S26541[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; chromeframe/25.0.1364.172; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S26542[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/25.0.1364.172) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S26543[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/25.0.1364.172; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S26544[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/25.0.1364.172; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S26545[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/25.0.1364.172) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S26546[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/25.0.1364.152) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22";
const char S26547[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/25.0.1364.152) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22";
const char S26548[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/25.0.1364.160) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.160 Safari/537.22";
const char S26549[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.4; chromeframe/25.0.1364.172; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)";
const char S26550[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/25.0.1364.172; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C)";
const char S26551[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; chromeframe/25.0.1364.172; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S26552[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; chromeframe/25.0.1364.172; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S26553[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/25.0.1364.172; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S26554[] = "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:20.0) Gecko/X Firefox/20.0";
const char S26555[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7 (compatible; Googlebot-Mobile/2.1; +http://www.google.com/bot.html)";
const char S26556[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7 (compatible; Mediapartners-Google/2.1; +http://www.google.com/bot.html)";
const char S26557[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/29.0.1547.32) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.32 Safari/537.36";
const char S26558[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/29.0.1547.49) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.49 Safari/537.36";
const char S26559[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/29.0.1547.57) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36";
const char S26560[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/29.0.1547.57; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 1.1.4322; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S26561[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/29.0.1547.57) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36";
const char S26562[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/29.0.1547.62) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.62 Safari/537.36";
const char S26563[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/29.0.1547.66) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36";
const char S26564[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.5; chromeframe/29.0.1547.66; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S26565[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/29.0.1547.66) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36";
const char S26566[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/29.0.1547.62) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.62 Safari/537.36";
const char S26567[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.5; chromeframe/29.0.1547.66; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S26568[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.5; chromeframe/29.0.1547.66; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S26569[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/29.0.1547.66)";
const char S26570[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/29.0.1547.66)";
const char S26571[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.5; chromeframe/29.0.1547.66; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S26572[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.5; chromeframe/29.0.1547.66; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S26573[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/29.0.1547.76) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36";
const char S26574[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/29.0.1547.76) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36";
const char S26575[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9B206 Safari/7534.48.3";
const char S26576[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9B208 Safari/7534.48.3";
const char S26577[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9B176 Safari/7534.48.3";
const char S26578[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9B179 Safari/7534.48.3";
const char S26579[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9Z999 Safari/7534.48.3";
const char S26580[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9A405 Safari/7534.48.3";
const char S26581[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S26582[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S26583[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S26584[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S26585[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S26586[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S26587[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.777 Mobile Safari/537.10+";
const char S26588[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.837 Mobile Safari/537.10+";
const char S26589[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1002 Mobile Safari/537.10+";
const char S26590[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1020 Mobile Safari/537.10+";
const char S26591[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1212 Mobile Safari/537.10+";
const char S26592[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.936 Mobile Safari/537.10+";
const char S26593[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1110 Mobile Safari/537.10+";
const char S26594[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1248 Mobile Safari/537.10+";
const char S26595[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1284 Mobile Safari/537.10+";
const char S26596[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1392 Mobile Safari/537.10+";
const char S26597[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1465 Mobile Safari/537.10+";
const char S26598[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1320 Mobile Safari/537.10+";
const char S26599[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1471 Mobile Safari/537.10+";
const char S26600[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1411 Mobile Safari/537.10+";
const char S26601[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1507 Mobile Safari/537.10+";
const char S26602[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1447 Mobile Safari/537.10+";
const char S26603[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1429 Mobile Safari/537.10+";
const char S26604[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1720 Mobile Safari/537.10+";
const char S26605[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1552 Mobile Safari/537.10+";
const char S26606[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1483 Mobile Safari/537.10+";
const char S26607[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1696 Mobile Safari/537.10+";
const char S26608[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1756 Mobile Safari/537.10+";
const char S26609[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1597 Mobile Safari/537.10+";
const char S26610[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1666 Mobile Safari/537.10+";
const char S26611[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1609 Mobile Safari/537.10+";
const char S26612[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1627 Mobile Safari/537.10+";
const char S26613[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1485 Mobile Safari/537.10+";
const char S26614[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1895 Mobile Safari/537.10+";
const char S26615[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1762 Mobile Safari/537.10+";
const char S26616[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1832 Mobile Safari/537.10+";
const char S26617[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1916 Mobile Safari/537.10+";
const char S26618[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1880 Mobile Safari/537.10+";
const char S26619[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2342 Mobile Safari/537.10+";
const char S26620[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2336 Mobile Safari/537.10+";
const char S26621[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2324 Mobile Safari/537.10+";
const char S26622[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2009 Mobile Safari/537.10+";
const char S26623[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2354 Mobile Safari/537.10+";
const char S26624[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4297 Mobile Safari/537.10+";
const char S26625[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2390 Mobile Safari/537.10+";
const char S26626[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2420 Mobile Safari/537.10+";
const char S26627[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2050 Mobile Safari/537.10+";
const char S26628[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4333 Mobile Safari/537.10+";
const char S26629[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2025 Mobile Safari/537.10+";
const char S26630[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4369 Mobile Safari/537.10+";
const char S26631[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.987 Mobile Safari/537.10+";
const char S26632[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4429 Mobile Safari/537.10+";
const char S26633[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2414 Mobile Safari/537.10+";
const char S26634[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4393 Mobile Safari/537.10+";
const char S26635[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4453 Mobile Safari/537.10+";
const char S26636[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2062 Mobile Safari/537.10+";
const char S26637[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4537 Mobile Safari/537.10+";
const char S26638[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4543 Mobile Safari/537.10+";
const char S26639[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2074 Mobile Safari/537.10+";
const char S26640[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4477 Mobile Safari/537.10+";
const char S26641[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2151 Mobile Safari/537.10+";
const char S26642[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4597 Mobile Safari/537.10+";
const char S26643[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4633 Mobile Safari/537.10+";
const char S26644[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4651 Mobile Safari/537.10+";
const char S26645[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4687 Mobile Safari/537.10+";
const char S26646[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1768 Mobile Safari/537.10+";
const char S26647[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4261 Mobile Safari/537.10+";
const char S26648[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4492 Mobile Safari/537.10+";
const char S26649[] = "LinkedInBlackBerry/10.1.4 (1) Z10 STL100-2/10.1.0.4633 (Research In Motion;RIM BlackBerry Device;Z10;STL100-2;1)";
const char S26650[] = "LinkedInBlackBerry/10.1.4 (1) Z10 STL100-3/10.1.0.4633 (Research In Motion;RIM BlackBerry Device;Z10;STL100-3;1)";
const char S26651[] = "LinkedInBlackBerry/10.1.4 (1) Z10 STL100-4/10.1.0.2151 (Research In Motion;RIM BlackBerry Device;Z10;STL100-4;1)";
const char S26652[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4780 Mobile Safari/537.10+";
const char S26653[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4828 Mobile Safari/537.10+";
const char S26654[] = "LinkedInBlackBerry/10.1.4 (1) Z10 STL100-1/10.1.0.4633 (Research In Motion;RIM BlackBerry Device;Z10;STL100-1;1)";
const char S26655[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4246 Mobile Safari/537.10+";
const char S26656[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4657 Mobile Safari/537.10+";
const char S26657[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4339 Mobile Safari/537.10+";
const char S26658[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0)";
const char S26659[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; .NET CLR 1.1.4322)";
const char S26660[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0 )";
const char S26661[] = "Mozilla/4.0 (Compatible; MSIE 5.5; Windows NT5.0; QX; SV1; .NET CLR 1.1.4322; InfoPath.2)";
const char S26662[] = "Mozilla/5.0 (compatible; MSIE 5.5; Windows NT 5.0)";
const char S26663[] = "MSIE 5.5 (Win 2000): Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0 )";
const char S26664[] = "Opera/9.80 (Windows NT 6.2; Win64; x64) Presto/2.12.388 Version/12.15";
const char S26665[] = "Opera/9.80 (Windows NT 6.2) Presto/2.12.388 Version/12.15";
const char S26666[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.15";
const char S26667[] = "Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.15,gzip(gfe)";
const char S26668[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.15";
const char S26669[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.15";
const char S26670[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.15";
const char S26671[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.15";
const char S26672[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.15";
const char S26673[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.15";
const char S26674[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.15";
const char S26675[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.15";
const char S26676[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.15";
const char S26677[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.15";
const char S26678[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.15";
const char S26679[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.15";
const char S26680[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.15";
const char S26681[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.15";
const char S26682[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.15";
const char S26683[] = "Opera/9.80 (Windows NT 6.2) Presto/2.12.388 Version/12.15,gzip(gfe)";
const char S26684[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.15";
const char S26685[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64) Opera 12.15";
const char S26686[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.15";
const char S26687[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.15";
const char S26688[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.15";
const char S26689[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.15";
const char S26690[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.15";
const char S26691[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.15";
const char S26692[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.15";
const char S26693[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6080)) Presto/2.12.388 Version/12.15";
const char S26694[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.15";
const char S26695[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.15";
const char S26696[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.15";
const char S26697[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.15";
const char S26698[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.15";
const char S26699[] = "Opera/9.80 (Windows NT 6.2; Win64; x64) Presto/2.12.388 Version/12.15 AppEngine-Google; (+http://code.google.com/appengine; appid: schmittify)";
const char S26700[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.15";
const char S26701[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.15";
const char S26702[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6561)) Presto/2.12.388 Version/12.15";
const char S26703[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6573)) Presto/2.12.388 Version/12.15";
const char S26704[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.15";
const char S26705[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6283)) Presto/2.12.388 Version/12.15";
const char S26706[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6261)) Presto/2.12.388 Version/12.15";
const char S26707[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.15";
const char S26708[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.15";
const char S26709[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.15";
const char S26710[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.15";
const char S26711[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6638)) Presto/2.12.388 Version/12.15";
const char S26712[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6628)) Presto/2.12.388 Version/12.15";
const char S26713[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 5.10 (build 5300)) Presto/2.12.388 Version/12.15";
const char S26714[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6322)) Presto/2.12.388 Version/12.15";
const char S26715[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.15";
const char S26716[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.15";
const char S26717[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6648)) Presto/2.12.388 Version/12.15";
const char S26718[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6657)) Presto/2.12.388 Version/12.15";
const char S26719[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6323)) Presto/2.12.388 Version/12.15";
const char S26720[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6327)) Presto/2.12.388 Version/12.15";
const char S26721[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6620)) Presto/2.12.388 Version/12.15";
const char S26722[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6277)) Presto/2.12.388 Version/12.15";
const char S26723[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6325)) Presto/2.12.388 Version/12.15";
const char S26724[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6337)) Presto/2.12.388 Version/12.15";
const char S26725[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6673)) Presto/2.12.388 Version/12.15";
const char S26726[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6669)) Presto/2.12.388 Version/12.15";
const char S26727[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.15";
const char S26728[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.15";
const char S26729[] = "Mozilla/5.0 (Windows NT 5.1; rv:6.0) Gecko/X Firefox/10.0.2";
const char S26730[] = "Mozilla/5.0 (Windows NT 5.1; rv:10.0) Gecko/X Firefox/10.0 Firefox/10.0";
const char S26731[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:10.0.10) Gecko/X Firefox/10.0.10";
const char S26732[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:10.0.11) Gecko/X Firefox/10.0.11ms1";
const char S26733[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; sv-se; SonyEricssonR800i Build/3.0.A.2.179) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26734[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; R800i Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26735[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonR800i Build/3.0.1.A.0.64) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26736[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; R800i Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26737[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; SonyEricssonR800i Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26738[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; SonyEricssonR800i Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26739[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ar-; SonyEricssonR800i Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26740[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonR800i Build/3.0.1.A.0.150) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26741[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonR800i Build/3.0.1.A.0.146) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26742[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; R800i Build/3.0.1.A.0.150) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26743[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-za; SonyEricssonR800i Build/4.0.1.A.0.165) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26744[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonR800i Build/4.0.A.2.257) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26745[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonR800i Build/4.0.1.A.0.215) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26746[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonR800i Build/4.0.1.A.0.247) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26747[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; R800i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26748[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonR800i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26749[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonR800i Build/4.0.1.A.0.277) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26750[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonR800i Build/4.0.2.A.0.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26751[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonR800i Build/4.0.1.A.2.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26752[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonR800i Build/3.0.1.A.0.79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26753[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; R800i Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26754[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonR800i Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26755[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; sv-se; SonyEricssonR800i Build/3.0.A.2.119) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26756[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; R800i Build/3.0.1.A.0.146) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26757[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonR800i Build/3.0.1.A.0.151) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26758[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonR800i Build/3.0.1.A.0.95) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26759[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonR800i Build/4.0.A.2.286) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26760[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonR800i Build/4.0.A.2.364) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26761[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonR800i Build/GSX (Beta 2)) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26762[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonR800i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26763[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonR800i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26764[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; R800i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26765[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonR800i Build/4.0.2.A.0.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26766[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; R800i Build/3.0.1.A.0.145)";
const char S26767[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonR800i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26768[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonR800i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26769[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.2; R800i Build/3.0.A.2.184)";
const char S26770[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; R800i Build/4.0.2.A.0.42)";
const char S26771[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonR800i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26772[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ar-; SonyEricssonR800i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26773[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; R800i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26774[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; R800i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26775[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; R800i Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26776[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonR800i Build/6.0.A.3.67) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26777[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; R800i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26778[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonR800i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26779[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; R800i Build/4.0.2.A.0.42) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S26780[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; R800i Build/4.0.2.A.0.58) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S26781[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; R800i Build/4.0.2.A.0.42) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S26782[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-mx; SonyEricssonR800i Build/6.0.A.3.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26783[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; R800i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26784[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; R800i Build/4.0.2.A.0.79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26785[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-mx; R800i Build/6.0.A.3.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26786[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; R800i Build/4.0.2.A.0.79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26787[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonR800i Build/4.0.2.A.0.79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26788[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; R800i Build/4.0.2.A.0.69)";
const char S26789[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S26790[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S26791[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36,gzip(gfe)";
const char S26792[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; LG-P970h Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S26793[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P970 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S26794[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P970/V10c Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S26795[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-mx; LG-P970h Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S26796[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; LG-P970g Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S26797[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P970 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S26798[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P970-Orange Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S26799[] = "LG-P970/ Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S26800[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P970-Orange Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S26801[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; de; LG-P970/V10b Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S26802[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-au; LG-P970/V10a Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S26803[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P970/V10b Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S26804[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; LG-P970/V11b Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S26805[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; nl-nl; LG-P970/V10e Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S26806[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ru; LG-P970h Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S26807[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; de; LG-P970/V10c Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S26808[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P970/V10d Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S26809[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; LG-P970/V11c Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S26810[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P970 Build/FRG83G)";
const char S26811[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P970h Build/FRG83G)";
const char S26812[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; LG-P970 Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S26813[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; sr; LG-P970 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S26814[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; LG-P970 Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S26815[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P970g Build/FRG83G)";
const char S26816[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; de; LG-P970/V10d Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S26817[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; LG-P970 Build/Black Pearl) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S26818[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-co; LG-P970h Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Versi?n/4.0 Mobile Safari/533.1";
const char S26819[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S26820[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S26821[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S26822[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S26823[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530C Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S26824[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S26825[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S26826[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S26827[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S26828[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S26829[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S26830[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747M Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S26831[] = "Mozilla/5.0 (Linux; Android 4.1.1; SPH-L710 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S26832[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S26833[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S26834[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S26835[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S26836[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S26837[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-R530U Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S26838[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305N Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S26839[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE5-00u/043.115; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.7.4 3gpp-gba";
const char S26840[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE5-00.2/071.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.26 Mobile Safari/533.4 3gpp-gba";
const char S26841[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE5-00.2/046.001; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.2 3gpp-gba";
const char S26842[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE5-00/042.007; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.2 3gpp-gba";
const char S26843[] = "NOKIAE5-00";
const char S26844[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE5-00u/072.001; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.26 Mobile Safari/533.4 3gpp-gba";
const char S26845[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2NokiaE5-00/071.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.26 Mobile Safari/533.4 3gpp-gba";
const char S26846[] = "NOKIAE5-00.2";
const char S26847[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE5-00/101.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.35 Mobile Safari/533.4 3gpp-gba";
const char S26848[] = "(C)NokiaE5-00/SymbianOS/9.1 Series60/3.0 3gpp-gba";
const char S26849[] = "3gpp-gba Mozilla/4.0 (compatible; Nokia Podcasting; SymbianOS; NokiaE5-00/101.003)";
const char S26850[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NOKIAE5-00";
const char S26851[] = "Mozilla/4.8 [en] (Windows NT 5.0; U)";
const char S26852[] = "Mozilla/4.76 [en] (Windows NT 5.0; U)";
const char S26853[] = "Mozilla/4.79 [en] (Windows NT 5.0; U)";
const char S26854[] = "Mozilla/4.73 [en] (Windows NT 5.0)";
const char S26855[] = "Mozilla/4.79 [en] (Windows NT 5.0)";
const char S26856[] = "Mozilla/4.77 [en] (Windows NT 5.0)";
const char S26857[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.5b) Gecko/X";
const char S26858[] = "Mozilla/4.0 (Windows NT 5.0)";
const char S26859[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S26860[] = "Mozilla/5.0 (X11; U; Jolicloud Linux i686; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Joli OS/1.2 Chromium/10.0.648.82 Chrome/10.0.648.82 Safari/534.16";
const char S26861[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.16 (KHTML, like Gecko, Google Keyword Suggestion) Chrome/10.0.648.127 Safari/534.16";
const char S26862[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.16 SUSE/10.0.626.0 (KHTML, like Gecko) Chrome/10.0.626.0 Safari/534.16";
const char S26863[] = "Mozilla/5.0 (X11; U; Linux armv7l; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S26864[] = "Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.634.0 Safari/534.16";
const char S26865[] = "Mozilla/5.0 (U; compatible; Linux; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S26866[] = "Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.63 Safari/537.31";
const char S26867[] = "Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.63 Safari/537.31";
const char S26868[] = "Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.63 Safari/537.31";
const char S26869[] = "Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.63 Safari/537.31";
const char S26870[] = "Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.63 Safari/537.31";
const char S26871[] = "Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.63 Safari/537.31";
const char S26872[] = "Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.63 Safari/537.31";
const char S26873[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDXBuild/4.5.1_57_DX5-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26874[] = "Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26875[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26876[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26877[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-33) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26878[] = "Mozilla/5.0 (Linux; U; Android2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26879[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;DROIDX Build/4.5.1_57_DX5-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26880[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-26) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26881[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26882[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDXBuild/4.5.1_57_DX5-26) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26883[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S26884[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;DROIDX Build/4.5.1_57_DX5-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26885[] = "Mozilla/5.0 (Linux; U; Android2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26886[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDXBuild/4.5.1_57_DX5-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26887[] = "Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26888[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26889[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26890[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26891[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROIDX Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26892[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROIDX Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26893[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; DROIDX Build/4.5.1_57_DX5-35)";
const char S26894[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROIDX Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26895[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26896[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26897[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-35) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S26898[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26899[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26900[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; DROIDX Build/4.5.1_57_DX8-51)";
const char S26901[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26902[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S26903[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26904[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26905[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDXBuild/4.5.1_57_DX8-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26906[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26907[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROIDX Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26908[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROIDX Build/GRH78C; 4000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26909[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; DROIDX Build/VZW)";
const char S26910[] = "Mozilla/5.0 (Linux; U; Android Gingercomb; en-us; DroidX Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26911[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S26912[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; DROIDX Build/VZW)";
const char S26913[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROIDX Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26914[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/4.5.1_57_DX8-50) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26915[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26916[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30";
const char S26917[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/4.5.1_57_DX9-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26918[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; LG-P760/V20a Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26919[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P760/P76020b Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26920[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; LG-P760 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26921[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-P768 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26922[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P769 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26923[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P760 Build/JZO54K)";
const char S26924[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P760/P76020c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26925[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; sk-sk; LG-P760 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26926[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; LG-P760/V20b Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26927[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P768 Build/JZO54K)";
const char S26928[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-P760-Orange Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26929[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; LG-P760/P76020b Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26930[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LGMS769 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26931[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P769 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26932[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P769 Build/JZO54K)";
const char S26933[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; et; LG-P760 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26934[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LGMS769 Build/JZO54K)";
const char S26935[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; LG-P760/V20c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26936[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P760/P76020i Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26937[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; LG-P760/P76020a Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26938[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P760/P76020h Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S26939[] = "Mozilla/5.0 (IE 11.0; Windows NT 6.3; Win64; x64; Trident/7.0; rv:11.0) like Gecko";
const char S26940[] = "Mozilla/5.0 (IE 11.0; Windows NT 6.3; Trident/7.0; rv:11.0) like Gecko";
const char S26941[] = "Mozilla/5.0 (IE 11.0; Windows NT 6.3; WOW64; Trident/7.0; rv:11.0) like Gecko";
const char S26942[] = "Mozilla/5.0 (Windows NT 6.3; Win64; x64; Trident/7.0; rv:11.0) like Gecko";
const char S26943[] = "Mozilla/5.0 (Windows NT 6.3; WOW64; Trident/7.0; rv:11.0) like Gecko";
const char S26944[] = "Mozilla/5.0 (IE 11.0; Windows NT 6.3; Trident/7.0; .NET4.0E; .NET4.0C; rv:11.0) like Gecko";
const char S26945[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S26946[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9Z999 Safari/6533.18.5";
const char S26947[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 421 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S26948[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonWT19i Build/4.0.1.A.0.55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26949[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonWT19i Build/4.0.1.A.0.271) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26950[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonWT19i Build/4.0.1.A.0.258) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26951[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; WT19i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26952[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonWT19i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26953[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonWT19i Build/4.0.2.C.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26954[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; WT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26955[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; WT19i Build/4.0.2.A.0.62) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S26956[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-; SonyEricssonWT19i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26957[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SonyEricssonWT19i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26958[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pl-pl; SonyEricssonWT19i-o Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26959[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; WT19i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26960[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; WT19i Build/4.0.2.A.0.62)";
const char S26961[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; SonyEricssonWT19i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26962[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; WT19i Build/4.0.2.C.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26963[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonWT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26964[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SonyEricssonWT19iv Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26965[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-za; SonyEricssonWT19iv Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26966[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; WT19i Build/4.0.2.A.0.58) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S26967[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonWT19i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26968[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-nz; SonyEricssonWT19iv Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26969[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; SonyEricssonWT19i-o Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26970[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; WT19i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26971[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-be; SonyEricssonWT19iv Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S26972[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; WT19i Build/4.0.2.A.0.62) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S26973[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; WT19i Build/4.0.1.A.0.283) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S26974[] = "Mozilla/5.0 (X11; CrOS i686 3196.1.0) AppleWebKit/537.19 (KHTML, like Gecko) Chrome/25.0.1324.1 Safari/537.19";
const char S26975[] = "Mozilla/5.0 (X11; CrOS x86_64 3196.1.0) AppleWebKit/537.19 (KHTML, like Gecko) Chrome/25.0.1324.1 Safari/537.19";
const char S26976[] = "Mozilla/5.0 (X11; CrOS x86_64 3196.2.0) AppleWebKit/537.19 (KHTML, like Gecko) Chrome/25.0.1324.1 Safari/537.19";
const char S26977[] = "Mozilla/5.0 (X11; CrOS armv7l 3196.4.0) AppleWebKit/537.19 (KHTML, like Gecko) Chrome/25.0.1324.1 Safari/537.19";
const char S26978[] = "Mozilla/5.0 (X11; CrOS armv7l 3196.3.0) AppleWebKit/537.19 (KHTML, like Gecko) Chrome/25.0.1324.1 Safari/537.19";
const char S26979[] = "Mozilla/5.0 (X11; CrOS i686 3337.1.0) AppleWebKit/537.21 (KHTML, like Gecko) Chrome/25.0.1347.5 Safari/537.21";
const char S26980[] = "Mozilla/5.0 (X11; CrOS armv7l 3337.1.0) AppleWebKit/537.21 (KHTML, like Gecko) Chrome/25.0.1347.5 Safari/537.21";
const char S26981[] = "Mozilla/5.0 (X11; CrOS x86_64 3337.1.0) AppleWebKit/537.21 (KHTML, like Gecko) Chrome/25.0.1347.5 Safari/537.21";
const char S26982[] = "Mozilla/5.0 (X11; CrOS x86_64 3379.2.0) AppleWebKit/537.21 (KHTML, like Gecko) Chrome/25.0.1354.2 Safari/537.21";
const char S26983[] = "Mozilla/5.0 (X11; CrOS armv7l 3379.2.0) AppleWebKit/537.21 (KHTML, like Gecko) Chrome/25.0.1354.2 Safari/537.21";
const char S26984[] = "Mozilla/5.0 (X11; CrOS i686 3379.2.0) AppleWebKit/537.21 (KHTML, like Gecko) Chrome/25.0.1354.2 Safari/537.21";
const char S26985[] = "Mozilla/5.0 (X11; CrOS x86_64 3428.0.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.2 Safari/537.22";
const char S26986[] = "Mozilla/5.0 (X11; CrOS armv7l 3428.0.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.2 Safari/537.22";
const char S26987[] = "Mozilla/5.0 (X11; CrOS i686 3428.0.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.2 Safari/537.22";
const char S26988[] = "Mozilla/5.0 (X11; CrOS i686 3417.0.2012) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1363.0 Safari/537.22";
const char S26989[] = "Mozilla/5.0 (X11; CrOS x86_64 3428.49.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.26 Safari/537.22";
const char S26990[] = "Mozilla/5.0 (X11; CrOS i686 3428.49.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.26 Safari/537.22";
const char S26991[] = "Mozilla/5.0 (X11; CrOS armv7l 3428.49.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.26 Safari/537.22";
const char S26992[] = "Mozilla/5.0 (X11; CrOS i686 3428.65.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.33 Safari/537.22";
const char S26993[] = "Mozilla/5.0 (X11; CrOS x86_64 3428.65.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.33 Safari/537.22";
const char S26994[] = "Mozilla/5.0 (X11; CrOS armv7l 3428.65.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.33 Safari/537.22";
const char S26995[] = "Mozilla/5.0 (X11; CrOS i686 3428.92.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.46 Safari/537.22";
const char S26996[] = "Mozilla/5.0 (X11; CrOS x86_64 3428.92.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.46 Safari/537.22";
const char S26997[] = "Mozilla/5.0 (X11; CrOS x86_64 3428.94.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.49 Safari/537.22";
const char S26998[] = "Mozilla/5.0 (X11; CrOS x86_64 3428.77.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.40 Safari/537.22";
const char S26999[] = "Mozilla/5.0 (X11; CrOS armv7l 3428.114.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.63 Safari/537.22";
const char S27000[] = "Mozilla/5.0 (X11; CrOS x86_64 3428.110.2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.57 Safari/537.22";
const char S27001[] = "Mozilla/5.0 (X11; CrOS x86_64 3428.127.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.68 Safari/537.22";
const char S27002[] = "Mozilla/5.0 (X11; CrOS i686 3428.127.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.68 Safari/537.22";
const char S27003[] = "Mozilla/5.0 (X11; CrOS armv7l 3428.127.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.68 Safari/537.22";
const char S27004[] = "Mozilla/5.0 (X11; CrOS x86_64 3428.149.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.87 Safari/537.22";
const char S27005[] = "Mozilla/5.0 (X11; CrOS x86_64 3428.131.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.74 Safari/537.22";
const char S27006[] = "Mozilla/5.0 (X11; CrOS i686 3428.149.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.87 Safari/537.22";
const char S27007[] = "Mozilla/5.0 (X11; CrOS armv7l 3428.149.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.87 Safari/537.22";
const char S27008[] = "Mozilla/5.0 (X11; CrOS x86_64 3428.167.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.91 Safari/537.22";
const char S27009[] = "Mozilla/5.0 (X11; CrOS armv7l 3428.149.2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.87 Safari/537.22";
const char S27010[] = "Mozilla/5.0 (X11; CrOS i686 3428.193.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.126 Safari/537.22";
const char S27011[] = "Mozilla/5.0 (X11; CrOS x86_64 3428.193.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.126 Safari/537.22";
const char S27012[] = "Mozilla/5.0 (X11; CrOS armv7l 3428.193.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.126 Safari/537.22";
const char S27013[] = "Mozilla/5.0 (X11; CrOS x86_64 3428.210.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.173 Safari/537.22";
const char S27014[] = "Mozilla/5.0 (X11; CrOS i686 3428.210.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.173 Safari/537.22";
const char S27015[] = "Mozilla/5.0 (X11; CrOS armv7l 3428.210.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.173 Safari/537.22";
const char S27016[] = "Mozilla/5.0 (X11; CrOS x86_64 3428.183.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22";
const char S27017[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonST27i Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27018[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; ST27i Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27019[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonST27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27020[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; ST27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27021[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; ST27i Build/6.0.B.3.184)";
const char S27022[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-mx; SonyEricssonST27a Build/6.0.B.3.191) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27023[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SonyEricssonST27a Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27024[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-mx; ST27a Build/6.0.B.3.191) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27025[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-za; SonyEricssonST27iv Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27026[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ST27I Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27027[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; ST27a Build/6.0.B.3.191)";
const char S27028[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ST27i Build/6.0.B.3.184) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S27029[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; iw-; SonyEricssonST27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27030[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; th-th; ST27a Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27031[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9B206 Safari/7534.48.3";
const char S27032[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9B176 Safari/7534.48.3";
const char S27033[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9A405 Safari/7534.48.3";
const char S27034[] = "Mozilla/5.0 (X11; CrOS i686 3635.0.0) AppleWebKit/537.28 (KHTML, like Gecko) Chrome/26.0.1398.0 Safari/537.28";
const char S27035[] = "Mozilla/5.0 (X11; CrOS x86_64 3635.0.0) AppleWebKit/537.28 (KHTML, like Gecko) Chrome/26.0.1398.0 Safari/537.28";
const char S27036[] = "Mozilla/5.0 (X11; CrOS armv7l 3635.0.0) AppleWebKit/537.28 (KHTML, like Gecko) Chrome/26.0.1398.0 Safari/537.28";
const char S27037[] = "Mozilla/5.0 (X11; CrOS armv7l 3658.0.0) AppleWebKit/537.30 (KHTML, like Gecko) Chrome/26.0.1403.0 Safari/537.30";
const char S27038[] = "Mozilla/5.0 (X11; CrOS i686 3658.0.0) AppleWebKit/537.30 (KHTML, like Gecko) Chrome/26.0.1403.0 Safari/537.30";
const char S27039[] = "Mozilla/5.0 (X11; CrOS x86_64 3658.0.0) AppleWebKit/537.30 (KHTML, like Gecko) Chrome/26.0.1403.0 Safari/537.30";
const char S27040[] = "Mozilla/5.0 (X11; CrOS i686 3701.0.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.4 Safari/537.31";
const char S27041[] = "Mozilla/5.0 (X11; CrOS armv7l 3701.0.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.4 Safari/537.31";
const char S27042[] = "Mozilla/5.0 (X11; CrOS x86_64 3701.0.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.4 Safari/537.31";
const char S27043[] = "Mozilla/5.0 (X11; CrOS i686 3701.30.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.15 Safari/537.31";
const char S27044[] = "Mozilla/5.0 (X11; CrOS x86_64 3701.30.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.15 Safari/537.31";
const char S27045[] = "Mozilla/5.0 (X11; CrOS armv7l 3701.30.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.15 Safari/537.31";
const char S27046[] = "Mozilla/5.0 (X11; CrOS x86_64 3701.23.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.12 Safari/537.31";
const char S27047[] = "Mozilla/5.0 (X11; CrOS armv7l 3701.23.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.12 Safari/537.31";
const char S27048[] = "Mozilla/5.0 (X11; CrOS i686 3701.23.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.12 Safari/537.31";
const char S27049[] = "Mozilla/5.0 (X11; CrOS i686 3701.42.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.28 Safari/537.31";
const char S27050[] = "Mozilla/5.0 (X11; CrOS x86_64 3701.42.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.28 Safari/537.31";
const char S27051[] = "Mozilla/5.0 (X11; CrOS armv7l 3701.42.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.28 Safari/537.31";
const char S27052[] = "Mozilla/5.0 (X11; CrOS armv7l 3701.30.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.15 Safari/537.31";
const char S27053[] = "Mozilla/5.0 (X11; CrOS i686 3701.46.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.30 Safari/537.31";
const char S27054[] = "Mozilla/5.0 (X11; CrOS armv7l 3701.46.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.30 Safari/537.31";
const char S27055[] = "Mozilla/5.0 (X11; CrOS x86_64 3701.46.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.30 Safari/537.31";
const char S27056[] = "Mozilla/5.0 (X11; CrOS i686 3701.62.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Safari/537.31";
const char S27057[] = "Mozilla/5.0 (X11; CrOS x86_64 3701.62.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Safari/537.31";
const char S27058[] = "Mozilla/5.0 (X11; CrOS armv7l 3701.62.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Safari/537.31";
const char S27059[] = "Mozilla/5.0 (X11; CrOS i686 3701.76.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.51 Safari/537.31";
const char S27060[] = "Mozilla/5.0 (X11; CrOS x86_64 3701.76.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.51 Safari/537.31";
const char S27061[] = "Mozilla/5.0 (X11; CrOS armv7l 3701.76.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.51 Safari/537.31";
const char S27062[] = "Mozilla/5.0 (X11; CrOS x86_64 3701.81.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.57 Safari/537.31";
const char S27063[] = "Mozilla/5.0 (X11; CrOS i686 3701.81.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.57 Safari/537.31";
const char S27064[] = "Mozilla/5.0 (X11; CrOS i686 3701.81.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.57 Safari/537.31";
const char S27065[] = "Mozilla/5.0 (X11; CrOS x86_64 3701.81.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.57 Safari/537.31";
const char S27066[] = "Mozilla/5.0 (X11; CrOS armv7l 3701.81.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.57 Safari/537.31";
const char S27067[] = "Mozilla/5.0 (X11; CrOS armv7l 3701.81.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.57 Safari/537.31";
const char S27068[] = "Mozilla/5.0 (X11; CrOS x86_64 3701.90.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31";
const char S27069[] = "Mozilla/5.0 (X11; CrOS x86_64 3701.111.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.66 Safari/537.31";
const char S27070[] = "Mozilla/5.0 (X11; CrOS x86_64 3701.109.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S27071[] = "Mozilla/5.0 (X11; CrOS x86_64 3701.112.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.67 Safari/537.31";
const char S27072[] = "Mozilla/5.0 (X11; CrOS i686 3701.81.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.57 Safari/537.31,gzip(gfe)";
const char S27073[] = "Mozilla/5.0 (X11; CrOS armv7l 3701.81.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.57 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)";
const char S27074[] = "Mozilla/5.0 (X11; CrOS armv7l 3701.81.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.57 Safari/537.31,gzip(gfe)";
const char S27075[] = "Mozilla/5.0 (X11; CrOS i686 3545.0.2013) AppleWebKit/537.26 (KHTML, like Gecko) Chrome/26.0.1382.0 Safari/537.26";
const char S27076[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.1+ (KHTML, like Gecko) Version/10.0.0.1337 Mobile Safari/537.1+";
const char S27077[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.3+ (KHTML, like Gecko) Version/10.0.0.1337 Mobile Safari/537.3+";
const char S27078[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.3+ (KHTML, like Gecko) Version/10.0.9.388 Mobile Safari/537.3+";
const char S27079[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.785 Mobile Safari/537.10+";
const char S27080[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.902 Mobile Safari/537.10+";
const char S27081[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.0.1337 Mobile Safari/537.10+";
const char S27082[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.945 Mobile Safari/537.10+";
const char S27083[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1077 Mobile Safari/537.10+";
const char S27084[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1287 Mobile Safari/537.10+";
const char S27085[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1179 Mobile Safari/537.10+";
const char S27086[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1407 Mobile Safari/537.10+";
const char S27087[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1473 Mobile Safari/537.10+";
const char S27088[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1425 Mobile Safari/537.10+";
const char S27089[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1103 Mobile Safari/537.10+";
const char S27090[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1595 Mobile Safari/537.10+";
const char S27091[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1673 Mobile Safari/537.10+";
const char S27092[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1655 Mobile Safari/537.10+";
const char S27093[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1994 Mobile Safari/537.10+";
const char S27094[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1757 Mobile Safari/537.10+";
const char S27095[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1675 Mobile Safari/537.10+";
const char S27096[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2108 Mobile Safari/537.10+";
const char S27097[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1904 Mobile Safari/537.10+";
const char S27098[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2342 Mobile Safari/537.10+";
const char S27099[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2264 Mobile Safari/537.10+";
const char S27100[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2222 Mobile Safari/537.10+";
const char S27101[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2318 Mobile Safari/537.10+";
const char S27102[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2360 Mobile Safari/537.10+";
const char S27103[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2372 Mobile Safari/537.10+";
const char S27104[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.75 Mobile Safari/537.10+";
const char S27105[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2320 Mobile Safari/537.10+";
const char S27106[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.261 Mobile Safari/537.10+";
const char S27107[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.252 Mobile Safari/537.10+";
const char S27108[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.225 Mobile Safari/537.10+";
const char S27109[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.263 Mobile Safari/537.10+";
const char S27110[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2466 Mobile Safari/537.10+";
const char S27111[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.357 Mobile Safari/537.10+";
const char S27112[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.243 Mobile Safari/537.10+";
const char S27113[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.462 Mobile Safari/537.10+";
const char S27114[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.552 Mobile Safari/537.10+";
const char S27115[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.576 Mobile Safari/537.10+";
const char S27116[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.672 Mobile Safari/537.10+";
const char S27117[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.588 Mobile Safari/537.10+";
const char S27118[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.648 Mobile Safari/537.10+";
const char S27119[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2709 Mobile Safari/537.10+";
const char S27120[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.690 Mobile Safari/537.10+";
const char S27121[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.684 Mobile Safari/537.10+";
const char S27122[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2598 Mobile Safari/537.10+";
const char S27123[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.738 Mobile Safari/537.10+";
const char S27124[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.702 Mobile Safari/537.10+";
const char S27125[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2743 Mobile Safari/537.10+";
const char S27126[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1754 Mobile Safari/537.10+";
const char S27127[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.768 Mobile Safari/537.10+";
const char S27128[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.822 Mobile Safari/537.10+";
const char S27129[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.3+ (KHTML, like Gecko) Version/10.0.9.386 Mobile Safari/537.3+";
const char S27130[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.696 Mobile Safari/537.10+";
const char S27131[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.3+ (KHTML, like Gecko) Version/10.0.9.443 Mobile Safari/537.3+";
const char S27132[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.606 Mobile Safari/537.10+";
const char S27133[] = "LinkedInBlackBerry/10.0.3 (4) Z10 STL100-3/10.0.10.672 (Research In Motion;RIM BlackBerry Device;Z10;STL100-3;1)";
const char S27134[] = "LinkedInBlackBerry/10.0.3 (4) Z10 STL100-4/10.0.9.2743 (Research In Motion;RIM BlackBerry Device;Z10;STL100-4;1)";
const char S27135[] = "LinkedInBlackBerry/10.0.3 (4) Z10 STL100-1/10.0.10.648 (Research In Motion;RIM BlackBerry Device;Z10;STL100-1;1)";
const char S27136[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2442 Mobile Safari/537.10+";
const char S27137[] = "LinkedInBlackBerry/10.0.3 (4) Z10 STL100-2/10.0.10.690 (Research In Motion;RIM BlackBerry Device;Z10;STL100-2;1)";
const char S27138[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.291 Mobile Safari/537.10+";
const char S27139[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27140[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27141[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SCH-I500 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27142[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27143[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-I500 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27144[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I500 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27145[] = "Mozilla/5.0 (Linux; U; Android2.3.5; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27146[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I500Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27147[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ko-; SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27148[] = "Logos/0.9.4 (Linux; U; Android 2.3.5; en_US; SCH-I500 Build/GINGERBREAD.EI20)";
const char S27149[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SCH-I500 Build/GINGERBREAD)";
const char S27150[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27151[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SCH-I500 Build/GINGERBREAD)";
const char S27152[] = "Mozilla/5.0 (Linux; U;Android 2.3.5; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27153[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us;SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27154[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S27155[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 CSOUTH-i500";
const char S27156[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S27157[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-I500 Build/GINGERBREAD)";
const char S27158[] = "Mozilla/5.0 (Linux; U; Android BeigePhone Fascinate v3.0 - 2.3.5; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27159[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9860; en-GB) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.342 Mobile Safari/534.11+";
const char S27160[] = "BlackBerry9860/7.1.0.694 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102";
const char S27161[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S27162[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/528.16 (KHTML, like Gecko) Version/4.0 Safari/528.16";
const char S27163[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S27164[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S27165[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19";
const char S27166[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S27167[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/528.18.1 (KHTML, like Gecko) Version/4.0 Safari/528.17";
const char S27168[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/528.16 (KHTML, like Gecko) Version/4 Public Beta Safari/528.16";
const char S27169[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/528.4+ (KHTML, like Gecko) Version/4.0dp1 Safari/526.11.2";
const char S27170[] = "Mozilla/5.0 (X11; CrOS x86_64 3797.0.0) AppleWebKit/537.33 (KHTML, like Gecko) Chrome/27.0.1430.0 Safari/537.33";
const char S27171[] = "Mozilla/5.0 (X11; CrOS i686 3797.0.0) AppleWebKit/537.33 (KHTML, like Gecko) Chrome/27.0.1430.0 Safari/537.33";
const char S27172[] = "Mozilla/5.0 (X11; CrOS armv7l 3797.0.0) AppleWebKit/537.33 (KHTML, like Gecko) Chrome/27.0.1430.0 Safari/537.33";
const char S27173[] = "Mozilla/5.0 (X11; CrOS x86_64 3833.0.0) AppleWebKit/537.33 (KHTML, like Gecko) Chrome/27.0.1438.8 Safari/537.33";
const char S27174[] = "Mozilla/5.0 (X11; CrOS armv7l 3833.0.0) AppleWebKit/537.33 (KHTML, like Gecko) Chrome/27.0.1438.8 Safari/537.33";
const char S27175[] = "Mozilla/5.0 (X11; CrOS i686 3833.0.0) AppleWebKit/537.33 (KHTML, like Gecko) Chrome/27.0.1438.8 Safari/537.33";
const char S27176[] = "Mozilla/5.0 (X11; CrOS armv7l 3881.4.0) AppleWebKit/537.35 (KHTML, like Gecko) Chrome/27.0.1446.5 Safari/537.35";
const char S27177[] = "Mozilla/5.0 (X11; CrOS i686 3881.4.0) AppleWebKit/537.35 (KHTML, like Gecko) Chrome/27.0.1446.5 Safari/537.35";
const char S27178[] = "Mozilla/5.0 (X11; CrOS x86_64 3912.4.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.6 Safari/537.36";
const char S27179[] = "Mozilla/5.0 (X11; CrOS i686 3912.4.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.6 Safari/537.36";
const char S27180[] = "Mozilla/5.0 (X11; CrOS armv7l 3912.4.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.6 Safari/537.36";
const char S27181[] = "Mozilla/5.0 (X11; CrOS x86_64 3881.4.0) AppleWebKit/537.35 (KHTML, like Gecko) Chrome/27.0.1446.5 Safari/537.35";
const char S27182[] = "Mozilla/5.0 (X11; CrOS i686 3887.0.2013) AppleWebKit/537.35 (KHTML, like Gecko) Chrome/27.0.1448.0 Safari/537.35";
const char S27183[] = "Mozilla/5.0 (X11; CrOS x86_64 3912.10.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.36 Safari/537.36";
const char S27184[] = "Mozilla/5.0 (X11; CrOS armv7l 3912.10.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.36 Safari/537.36";
const char S27185[] = "Mozilla/5.0 (X11; CrOS i686 3912.10.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.36 Safari/537.36";
const char S27186[] = "Mozilla/5.0 (X11; CrOS x86_64 3912.23.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.46 Safari/537.36";
const char S27187[] = "Mozilla/5.0 (X11; CrOS i686 3912.23.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.46 Safari/537.36";
const char S27188[] = "Mozilla/5.0 (X11; CrOS x86_64 3912.33.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.56 Safari/537.36";
const char S27189[] = "Mozilla/5.0 (X11; CrOS armv7l 3912.33.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.56 Safari/537.36";
const char S27190[] = "Mozilla/5.0 (X11; CrOS i686 3912.33.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.56 Safari/537.36";
const char S27191[] = "Mozilla/5.0 (X11; CrOS armv7l 3912.23.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.46 Safari/537.36";
const char S27192[] = "Mozilla/5.0 (X11; CrOS x86_64 3912.52.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.76 Safari/537.36";
const char S27193[] = "Mozilla/5.0 (X11; CrOS armv7l 3912.52.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.76 Safari/537.36";
const char S27194[] = "Mozilla/5.0 (X11; CrOS i686 3912.52.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.76 Safari/537.36";
const char S27195[] = "Mozilla/5.0 (X11; CrOS armv7l 3912.43.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.69 Safari/537.36";
const char S27196[] = "Mozilla/5.0 (X11; CrOS i686 3912.43.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.69 Safari/537.36";
const char S27197[] = "Mozilla/5.0 (X11; CrOS x86_64 3912.43.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.69 Safari/537.36";
const char S27198[] = "Mozilla/5.0 (X11; CrOS x86_64 3912.63.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.83 Safari/537.36";
const char S27199[] = "Mozilla/5.0 (X11; CrOS i686 3912.63.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.83 Safari/537.36";
const char S27200[] = "Mozilla/5.0 (X11; CrOS armv7l 3912.63.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.83 Safari/537.36";
const char S27201[] = "Mozilla/5.0 (X11; CrOS i686 3912.79.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.103 Safari/537.36";
const char S27202[] = "Mozilla/5.0 (X11; CrOS armv7l 3912.79.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.103 Safari/537.36";
const char S27203[] = "Mozilla/5.0 (X11; CrOS x86_64 3912.79.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.103 Safari/537.36";
const char S27204[] = "Mozilla/5.0 (X11; CrOS i686 3733.0.2013) AppleWebKit/537.32 (KHTML, like Gecko) Chrome/27.0.1417.2 Safari/537.32";
const char S27205[] = "Mozilla/5.0 (X11; CrOS x86_64 3912.93.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.112 Safari/537.36";
const char S27206[] = "Mozilla/5.0 (X11; CrOS armv7l 3912.101.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S27207[] = "Mozilla/5.0 (X11; CrOS i686 3912.101.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S27208[] = "Mozilla/5.0 (X11; CrOS x86_64 3912.101.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S27209[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27210[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27211[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; SGH-T989D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27212[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us;SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27213[] = "Mozilla/5.0 (Linux; U; Android2.3.5; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27214[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T989Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27215[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-T989 Build/GINGERBREAD)";
const char S27216[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us;SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27217[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S27218[] = "Android/2.3.6; samsung SGH-T989; 2.2.0";
const char S27219[] = "com.google.android.youtube/2.3.4(Linux; U; Android 2.3.6; en_US; SGH-T989 Build/GINGERBREAD) gzip";
const char S27220[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27221[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S27222[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SGH-T989D Build/GINGERBREAD)";
const char S27223[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27224[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SGH-T989 Build/GINGERBREAD)";
const char S27225[] = "Mozilla/5.0 (Linux; U; Android2.3.6; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27226[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD 2.3.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27227[] = "Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27228[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27229[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/Hercules Latest Greatest) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27230[] = "Mozilla/5.0 (Linux; U;Android 2.3.5; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27231[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S27232[] = "Android/1.0 (SGH-T989 GINGERBREAD)";
const char S27233[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30";
const char S27234[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S27235[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; SGH-T989D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27236[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.65 Safari/535.11";
const char S27237[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11";
const char S27238[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11";
const char S27239[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11";
const char S27240[] = "User-Agent:Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.83 Safari/535.11";
const char S27241[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S27242[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.10 (KHTML, like Gecko) Chrome/17.0.925.0 QQBrowser/1.0.0.1096 Safari/535.10";
const char S27243[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.3.0.1804 Safari/535.11";
const char S27244[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.4.0.2117 Safari/535.11";
const char S27245[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.10 (KHTML, like Gecko) Chrome/17.0.925.0 QQBrowser/1.0.0.1096 Safari/535.10";
const char S27246[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.10 (KHTML, like Gecko) Chrome/17.0.925.0 QQBrowser/1.1.0.1475 Safari/535.10";
const char S27247[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S27248[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.4.0.2117 Safari/535.11";
const char S27249[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.3.0.1804 Safari/535.11";
const char S27250[] = "Opera/9.80 (Windows Mobile; Opera Mini/5.1.21595/25.657; U; en) Presto/2.5.25 Version/10.54";
const char S27251[] = "Opera/9.80 (Windows Mobile; Opera Mini/5.1.21594/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S27252[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; SHW-M250S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27253[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; SHW-M250L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27254[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; SHW-M250K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27255[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SHW-M250S Build/GINGERBREAD)";
const char S27256[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; SHW-M250L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27257[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; SHW-M250S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27258[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; SHW-M250K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27259[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; SHW-M250S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27260[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; SHW-M250L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27261[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; SHW-M250K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27262[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHW-M250K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27263[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHW-M250S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27264[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHW-M250S Build/GINGERBREAD)";
const char S27265[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; vi-; SHW-M250L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27266[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M250S Build/GINGERBREAD)";
const char S27267[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SHW-M250S Build/GINGERBREAD)";
const char S27268[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SHW-M250K Build/GINGERBREAD)";
const char S27269[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHW-M250L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27270[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHW-M250K Build/GINGERBREAD)";
const char S27271[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SHW-M250L Build/GINGERBREAD)";
const char S27272[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SHW-M250K Build/GINGERBREAD)";
const char S27273[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SHW-M240S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27274[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M250L Build/GINGERBREAD)";
const char S27275[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M250K Build/GINGERBREAD)";
const char S27276[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M240S Build/GINGERBREAD)";
const char S27277[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SHW-M250L Build/GINGERBREAD)";
const char S27278[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHW-M250L Build/GINGERBREAD)";
const char S27279[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M240S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27280[] = "DaumMobileApp (SHW-M250K; U; Android 2.3.6|10; ko-kr) DaumCafe/6";
const char S27281[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-us) AppleWebKit/531.2+ (KHTML, like Gecko) Version/5.0 Safari/531.2+";
const char S27282[] = "Mozilla/5.0 (Linux; U; Intel X86; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S27283[] = "Mozilla/5.0 (DirectFB; Linux; ko-KR) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+";
const char S27284[] = "Mozilla/5.0 (Linux; U; Grid OS 1.0; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Version/5.0 Safari/533.1";
const char S27285[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.7 (KHTML, like Gecko) Version/5.0 Safari/534.7";
const char S27286[] = "Mozilla/5.0 (Linux; U; Linux; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S27287[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.4+ (KHTML, like Gecko) Version/5.0 Safari/535.4+";
const char S27288[] = "Mozilla/5.0 (X11; Linux; ko-KR) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+";
const char S27289[] = "Mozilla/5.0 (X11; U; Linux i686; en-us) AppleWebKit/534.16+ (KHTML, like Gecko) Version/5.0 Safari/534.16+ Epiphany/2.30.6";
const char S27290[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.16+ (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S27291[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.20+ (KHTML, like Gecko) Version/5.0 Safari/535.20+";
const char S27292[] = "Mozilla/5.0 (X11; U; Linux; en-us) AppleWebKit/535.22 (KHTML, like Gecko) Version/5.0 Safari/535.22 LinutopKiosk/5.0.2";
const char S27293[] = "Mozilla/5.0 (Linux) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S27294[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/21.0.1180.60) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.60 Safari/537.1";
const char S27295[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/21.0.1180.60) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.60 Safari/537.1";
const char S27296[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/21.0.1180.49) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.49 Safari/537.1";
const char S27297[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/21.0.1180.77) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.77 Safari/537.1";
const char S27298[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/21.0.1180.77) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.77 Safari/537.1";
const char S27299[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/21.0.1180.75) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1";
const char S27300[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/21.0.1180.75) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1";
const char S27301[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/21.0.1180.83) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1";
const char S27302[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/21.0.1180.79) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.79 Safari/537.1";
const char S27303[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/21.0.1180.79) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.79 Safari/537.1";
const char S27304[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/21.0.1180.83) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1";
const char S27305[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/21.0.1180.79; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S27306[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/21.0.1180.79; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729)";
const char S27307[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/21.0.1180.79; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S27308[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/21.0.1180.89)";
const char S27309[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/21.0.1180.89)";
const char S27310[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/21.0.1180.89) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S27311[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/21.0.1180.89) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S27312[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/21.0.1180.89; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S27313[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; chromeframe/21.0.1180.89)";
const char S27314[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/21.0.1180.89; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S27315[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/21.0.1180.79; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S27316[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/21.0.1180.89; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.2; MS-RTC EA 2; MS-RTC LM 8)";
const char S27317[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/21.0.1180.83; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; MRA 5.7 (build 03796); SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.2)";
const char S27318[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; chromeframe/21.0.1180.83; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; .NET4.0C; .NET4.0E)";
const char S27319[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; chromeframe/21.0.1180.83; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S27320[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/21.0.1180.89; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S27321[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/21.0.1180.89)";
const char S27322[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/21.0.1180.83; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8; Tablet PC 2.0)";
const char S27323[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/21.0.1180.89)";
const char S27324[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/21.0.1180.75; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3)";
const char S27325[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SV1; chromeframe/21.0.1180.83; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; .NET4.0C; .NET4.0E)";
const char S27326[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB525 Build/3.4.2-107_JDN-4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27327[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; ME525 Build/JDGC_1.11.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27328[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB525 Build/3.4.2-107_JDN-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27329[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; MB525 Build/3.4.2-74_JST-84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27330[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME525 Build/JDGC_2.6.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27331[] = "Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; MB525 Build/3.4.2-107_JDN-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27332[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB525 Build/3.4.2-90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27333[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; ME525 Build/JDGC_0.26.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27334[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MB525 Build/3.4.2-117) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27335[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; MB525 Build/3.4.3-11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27336[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ru-; MB525 Build/3.4.2-155) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27337[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MB525 Build/3.4.2-145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27338[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB525 Build/3.4.2-131) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27339[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; MB525 Build/3.4.2-164) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27340[] = "MOT-MB525/Blur_Version.34.4.706.MB525.TELUS.en.CA Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; MB525 Build/3.4.2-107_JDNL-6_R02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27341[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB525 Build/3.4.2-177) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27342[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-au; MB525 Build/3.4.2-145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27343[] = "MOT-MB525/Blur_Version.34.4.709.MB525.AmericaMovil.en.01 Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; MB525 Build/3.4.2-107_JDNL-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27344[] = "MOT-MB525/Blur_Version.34.4.707.MB525.Personal.en.AR Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; MB525 Build/3.4.2-107_JDNL-7_R02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27345[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB525 Build/3.4.2-107_JDN-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27346[] = "MOT-MB525/Blur_Version.34.4.706.MB525.TELUS.en.CA Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; MB525 Build/3.4.2-107_JDNL-6_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27347[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; he-il; MB525 Build/JEM_3.4.3-36-1.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27348[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; cs-; MB525 Build/3.4.2-179) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27349[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB525Build/3.4.2-107_JDN-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27350[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME525 Build/JDGC_2.5.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27351[] = "Mozilla/5.0 (Linux; U; Android2.2.1; en-us; MB525 Build/3.4.2-107_JDN-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27352[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME525 Build/JDGC_2.10.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27353[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB525 Build/3.4.2-107_JDN-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S27354[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;MB525 Build/3.4.2-107_JDN-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27355[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB525 Build/JEM_3.4.3-36-1.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27356[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; MotoMB525 Build/3.4.2-179) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27357[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB525 Build/JEM_3.4.3-21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27358[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB525 Build/JEM_3.4.3-36-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27359[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB525 Build/goapk_cg_3.4.3-11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27360[] = "MOT-MB525/Blur_Version.34.4.729.MB525.Latam.en.01 Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; MB525 Build/3.4.2-107_JDNL-29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27361[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MB525 Build/3.4.3-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27362[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB525 Build/3.4.2-179) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27363[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB525 Build/JEM_3.4.3-33-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27364[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; MB525 Build/3.4.2-155) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27365[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB525 Build/JEM_3.4.3-27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27366[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB525 Build/JEM_3.4.3-36-1.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27367[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB525 Build/JEM_3.4.3-20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27368[] = "MOT-MB525/Blur_Version.34.4.802.MB525.AmericaMovil.en.01 Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; MB525 Build/3.4.2-108_JDNL-2_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27369[] = "MOT-MB525/Blur_Version.34.4.802.MB525.Movistar.en.01 Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; MB525 Build/3.4.2-108_JDNL-2_R02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27370[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; MB525 Build/3.4.2-107_JDN-9)";
const char S27371[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; MB525 Build/3.4.2-177)";
const char S27372[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB525 Build/JEM_3.4.3-33-1) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S27373[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MB525 Build/3.4.2-117) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S27374[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME525 Build/JDGC_2.10.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S27375[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME525 Build/JDGC_2.13.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S27376[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME525 Build/JDGC_2.13.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27377[] = "MOT-MB525/Blur_Version.34.4.806.MB525.Latam.en.01 Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB525 Build/3.4.2-108_JDNL-6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27378[] = "MOT-MB525/Blur_Version.34.4.805.MB525.AmericaMovil.en.01 Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; MB525 Build/3.4.2-108_JDNL-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27379[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB525 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27380[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB525 Build/JEM_3.4.3-36-1.7) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S27381[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; ME525 Build/JDGC_1.11.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S27382[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME525 Build/JDGC_2.10.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S27383[] = "MOT-MB525/Blur_Version.34.4.807.MB525.AmericaMovil.en.BR Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; MB525 Build/3.4.2-108_JDNL-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27384[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB525 Build/3.4.2-177) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S27385[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME525 Build/JDGC_2.6.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S27386[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; ME525 Build/JDGC_0.26.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S27387[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MB525 Build/3.4.3-3) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S27388[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME525 Build/JDGC_2.13.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S27389[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB525 Build/JEM_3.4.3-36-1.7) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S27390[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MB525 Build/3.4.2-117) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S27391[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; ME525 Build/JDGC_1.11.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S27392[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB525 Build/3.4.2-177) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S27393[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB525 Build/JEM_3.4.3-33-1) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S27394[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME525 Build/JDGC_2.12.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27395[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME525 Build/JDGC_2.6.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S27396[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; SGH-T859 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S27397[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; SGH-T859 Build/HTJ85B)";
const char S27398[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S27399[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7510 Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S27400[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7510 Build/HTJ85B)";
const char S27401[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S27402[] = "Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7510-ORANGE/P7510BVKL2 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S27403[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S27404[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7510 Build/HTJ85B; 1280*752) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S27405[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7510 Build/HTJ85B) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S27406[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7510 Build/HTJ85B) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S27407[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7510 Build/HTJ85B) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S27408[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7510 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27409[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S27410[] = "BBC News (AndroidApp; 1.0; 5) GT-P7510 (Android 3.2, SDK 13)";
const char S27411[] = "Logos/2.0.11 (Linux; U; Android 3.2; en-US; GT-P7510 Build/HTJ85B.UEKMP)";
const char S27412[] = "Mozilla/5.0 (Android 3.2; en-us; GT-P7510 Build/HTJ85B) FreeWheelAdManager/5.0.3-r9054-X;com.gotv.crackle.handset/X";
const char S27413[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.7.2;FBPN/com.facebook.katana;FBDV/GT-P7510;FBSV/3.2;FBDM/{density=1.0,width=1280,height=752};FBCR/;FBLC/en_US;FB_FW/1;]";
const char S27414[] = "Vyrso/2.1.2 (Linux; U; Android 3.2; en-US; GT-P7510 Build/HTJ85B.VLKL2)";
const char S27415[] = "Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 3.2; SAMSUNG GT-P7510 Build/HONEYCOMB_MR2);";
const char S27416[] = "Biblia/2.1.2 (Linux; U; Android 3.2; es-ES; GT-P7510 Build/HTJ85B.UBKL2)";
const char S27417[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.11;FBDM/{density=1.0,width=1280,height=752};FBLC/en_US;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P7510;FBSV/3.2;]";
const char S27418[] = "Groupon/2.1.2483 (Android 3.2; Samsung GT-P7510 / Samsung GT-P7510; )[preload=false;locale=en_US;clientidbase=android-samsung]";
const char S27419[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7510 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 parameters/{scr=752_1280,cm=1,ql=l,imei=94:63:D1:F7:7E:E5,bm=1,eng=adrmini13,mdm=1,mdl=GT-P7510,ps=1} tiantian(securitypay) tiantian(tenpay)";
const char S27420[] = "Logos/2.2.8.1 (Linux; U; Android 3.2; en-US; GT-P7510 Build/HTJ85B.UEKMM)";
const char S27421[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, likeb_etc_status=%7B%22c%22%3A%7B%7D%7D; _qubitTracker=X.X; _qPageNum_harveysfu2o5t2hw55r0pl5155; X-Mapping-ihjijfgc=6021E6A5F9C100D0B84AE856E8CBBCFE; HomestyleBasket=X; HViewID=c4e40b3a-65b1-4217-a490-af7710fdf0a3; _qubitTracker_s=X.X; WT_FPC=id=216e37d2f1d84b71cX:lv=X:ss=X; __utma=X.X.X.X.135";
const char S27422[] = "com.google.android.youtube/4.2.16(Linux; U; Android 3.2; en_US; GT-P7510 Build/HTJ85B) gzip";
const char S27423[] = "Android/4.7 (Linux; U; Android 3.2; en_US; GT-P7510)";
const char S27424[] = "BBC News (AndroidApp; 1.1; 9) GT-P7510 (Android 3.2, SDK 13)";
const char S27425[] = "VKAndroidApp/2.7.6-64 (Android 3.2; SDK 13; armeabi-v7a; samsung GT-P7510; ru)";
const char S27426[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; SGH-T859 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S27427[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.0; DigExt; .NET CLR 1.1.4322)";
const char S27428[] = "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.0; Trident/4.0; InfoPath.1; SV1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 3.0.04506.30)";
const char S27429[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.0)";
const char S27430[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; InfoPath.2; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET CLR 3.0.30618)";
const char S27431[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0;GB)";
const char S27432[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; SV1)";
const char S27433[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)";
const char S27434[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S27435[] = "Mozilla/4.0 (compatible; MSIE 8.00; Windows NT 5.0; MyIE 3.01)";
const char S27436[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; SV1; SV1)";
const char S27437[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; SV1; SV1)";
const char S27438[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5830-ORANGE Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27439[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; GT-S5830T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27440[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27441[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; GT-S5830L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27442[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; GT-S5830L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27443[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; GT-S5830B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27444[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; GT-S5830B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27445[] = "Mozilla/5.0 (Linux U Android 2.2.1 en-gb GT-S5830 Build/FROYO) AppleWebKit/533.1(KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27446[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-S5830 Build/FROYO)";
const char S27447[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-S5830T Build/FROYO)";
const char S27448[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; GT-S5830L Build/FROYO)";
const char S27449[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-S5830 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S27450[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-S5830L Build/FROYO)";
const char S27451[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-S5830 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S27452[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27453[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-S5830B Build/FROYO)";
const char S27454[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-S5830 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27455[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27456[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27457[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27458[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; es-es) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27459[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; ja-jp) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27460[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; nb-no) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27461[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; fr-fr) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27462[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27463[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.12";
const char S27464[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-gb) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27465[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; nl-nl) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27466[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; nl-nl) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27467[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; de-de) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27468[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; fr-fr) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27469[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; de-de) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27470[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; es-es) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27471[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; en-us) AppleWebKit/525.19 (KHTML, like Gecko) Version/3.2.1 Safari/525.19";
const char S27472[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; it-it) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27473[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27474[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; it-it) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27475[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; nb-no) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27476[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; sv-se) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27477[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; pt-br) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27478[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; pt-br) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27479[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.12";
const char S27480[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27481[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27482[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ja-jp) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27483[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; sv-se) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27484[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27485[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; zh-cn) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27486[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; nl-nl) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27487[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; de-de) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27488[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; da-dk) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27489[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ru-ru) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27490[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; ru-ru) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27491[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; fi-fi) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27492[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; pt-br) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27493[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; cs-cz) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27494[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; ko-kr) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27495[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; zh-cn) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27496[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; pl-pl) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27497[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-au) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27498[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; de-de) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.12";
const char S27499[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; fr-fr) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27500[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; fr-fr) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.12";
const char S27501[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; zh-tw) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27502[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; ar) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27503[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; he-il) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27504[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27505[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/3.2 Safari/525.26.12";
const char S27506[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; ja-jp) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27507[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; sv-se) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.12";
const char S27508[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_8; ja-jp) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27509[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.25 (KHTML, like Gecko) Version/3.2 Safari/525.25";
const char S27510[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; af-za) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27511[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; fi-fi) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27512[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27513[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; pl-pl) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27514[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; pt-pt) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27515[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; da-dk) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27516[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_8; ja-jp) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27517[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; zh-tw) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27518[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-au) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27519[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-gb) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27520[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; el-gr) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27521[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; hr-hr) AppleWebKit/530.1+ (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27522[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; ja-jp) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.12";
const char S27523[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; th-th) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27524[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; es-es) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27525[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ar) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27526[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27527[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_5; fi-fi) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.12";
const char S27528[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; tr-tr) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27529[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/525.19 (KHTML, like Gecko) Version/3.2.1 Safari/525/19";
const char S27530[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27531[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; el-gr) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27532[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; af-za) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27533[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-ca) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27534[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27535[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; pt-pt) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S27536[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; it-it) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27537[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; it-it) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.12";
const char S27538[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27539[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S27540[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.0.6) Gecko/X Version/3.2.1 Safari/525.27.1";
const char S27541[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30";
const char S27542[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.124 Safari/534.30";
const char S27543[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.51 (KHTML, like Gecko; Google Web Preview) Chrome/12.0.742 Safari/534.51";
const char S27544[] = "Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.124 Safari/534.30";
const char S27545[] = "Mozilla/5.0 ArchLinux (X11; U; Linux x86_64; en-US) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30";
const char S27546[] = "'Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.91 Safari/534.30'";
const char S27547[] = "Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/12.0.1410.63 Safari/537.31";
const char S27548[] = "Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/12.0.1410.63 Safari/537.31";
const char S27549[] = "Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/12.0.1410.63 Safari/537.31";
const char S27550[] = "Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/12.0.1410.63 Safari/537.31";
const char S27551[] = "Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/12.0.1410.63 Safari/537.31";
const char S27552[] = "Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/12.0.1410.63 Safari/537.31";
const char S27553[] = "Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/12.0.1410.63 Safari/537.31";
const char S27554[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30 AppEngine-Google; (+http://code.google.com/appengine; appid: s~croossa)";
const char S27555[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ASUS Transformer Pad TF300T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S27556[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; ASUS Transformer Pad TF300T Build/JOP40D)";
const char S27557[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; ASUS Transformer Pad TF300TG Build/JOP40D)";
const char S27558[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ASUS Transformer Pad TF300TG Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S27559[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; ASUS Transformer Pad TF300TL Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S27560[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; ja-jp; ASUS Pad TF300T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S27561[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Transformer Pad TF300T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S27562[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Transformer Pad TF300T Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S27563[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ASUS Transformer Pad TF300T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27564[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; ASUS Transformer Pad TF300TL Build/JOP40D)";
const char S27565[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S27566[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S27567[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36,gzip(gfe)";
const char S27568[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S27569[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S27570[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S27571[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S27572[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S27573[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S27574[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S27575[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.85 Safari/537.36";
const char S27576[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27577[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27578[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27579[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27580[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27581[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I317M Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27582[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27583[] = "Mozilla/5.0 (Linux; Android 4.1.1; SC-02E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27584[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27585[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27586[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I317 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27587[] = "Mozilla/5.0 (Linux; Android 4.1.1; SPH-L900 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27588[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27589[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27590[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27591[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27592[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27593[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27594[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27595[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27596[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27597[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27598[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S27599[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-R950 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27600[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889V Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27601[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S27602[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-N719 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27603[] = "Mozilla/5.0 (Linux; Android 4.1.1; SHV-E250S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27604[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S27605[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-N719 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27606[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S27607[] = "Mozilla/5.0 (Linux; Android 4.1.1; SHV-E250K Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27608[] = "Mozilla/5.0 (Linux; Android 4.1.9; GT-N7100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27609[] = "Mozilla/5.0 (Linux; Android 4.1.1; N7100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S27610[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11,gzip(gfe) (via translate.google.com)";
const char S27611[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S27612[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S27613[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S27614[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S27615[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S27616[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S27617[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe) (via docs.google.com/viewer)";
const char S27618[] = "Mozilla/5.0 Macintosh Intel Mac OS X 10_7_4 AppleWebKit/536.11 KHTML, like Gecko Chrome/20.0.1132.57 Safari/536.11";
const char S27619[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1070) AppleWebKit/536.11 (KHTML like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S27620[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Sensation Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27621[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC/Sensation/3.32.161.11 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27622[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC/Sensation/3.32.162.11 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27623[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Sensation Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27624[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC/Sensation/3.32.166.11 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27625[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC/Sensation/3.32.163.12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27626[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_Sensation-orange-LS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27627[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; HTC/Sensation/3.32.165.11 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27628[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Sensation Build/IML74K)";
const char S27629[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Sensation Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27630[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Sensation Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27631[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; HTC-Z710a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27632[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC/Sensation/1.45.161.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27633[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Sensation Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27634[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC_Sensation_Z710a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27635[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC/Sensation/1.50.161.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27636[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC Sensation Z710a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27637[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_Sensation Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27638[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC_Z710a V3.32.841.15 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27639[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-gb; HTC Sensation Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27640[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC_Z710a V1.45.841.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27641[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Sensation Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27642[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; HTC/Sensation/3.32.187.11 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27643[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-pt; HTC/Sensation/3.32.168.11 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27644[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Sensation Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27645[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Sensation Z710a Build/IML74K)";
const char S27646[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC/Sensation/1.45.161.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27647[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Sensation Build/HyperNonSense 2.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27648[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Sensation Build/IMM76I)";
const char S27649[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC-Z710a Build/IML74K)";
const char S27650[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation Z710 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27651[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC/Sensation/1.45.162.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27652[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Sensation Build/IMM76L)";
const char S27653[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC/Sensation/1.50.162.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27654[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC/Sensation/1.45.163.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27655[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC/Sensation/3.32.161.52 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27656[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC/Sensation/3.32.162.52 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27657[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC/Sensation/3.32.166.52 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27658[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; HTC/Sensation/3.32.165.52 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27659[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC/Sensation/3.32.163.53 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27660[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation Z710a Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S27661[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-pt; HTC/Sensation/3.32.168.52 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27662[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; HTC/Sensation/3.32.187.52 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27663[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1";
const char S27664[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1";
const char S27665[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1,gzip(gfe)";
const char S27666[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1,gzip(gfe)";
const char S27667[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyLT26i-o Build/6.2.B.0.197) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27668[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LT26i Build/6.2.B.0.200)";
const char S27669[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyLT26i Build/6.2.B.0.200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27670[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LT26i Build/6.2.B.0.200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27671[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyLT26i-o Build/6.2.B.0.200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27672[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT26i Build/6.2.B.0.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27673[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT26i Build/6.2.B.0.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27674[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LT26i Build/6.2.B.0.197) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27675[] = "SonyLT26i Build/6.2.B.0.200 stagefright/1.2 (Linux;Android 4.1.2)";
const char S27676[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-be; SonyLT26iv Build/6.2.B.0.200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27677[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyLT26i Build/6.2.B.1.96) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27678[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT26i Build/6.2.B.1.96) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27679[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT26i Build/Trinity Vengeance RaptorXL) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27680[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24";
const char S27681[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S27682[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S27683[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S27684[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.698.0 Safari/534.24";
const char S27685[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S27686[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/534.18 (KHTML, like Gecko) Chrome/11.0.660.0 Safari/534.18";
const char S27687[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S27688[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S27689[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.17 (KHTML, like Gecko) Chrome/11.0.655.0 Safari/534.17";
const char S27690[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24";
const char S27691[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/534.20 (KHTML, like Gecko) Chrome/11.0.672.2 Safari/534.20";
const char S27692[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/534.20 (KHTML, like Gecko) Chrome/11.0.672.2 Safari/534.20";
const char S27693[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/534.18 (KHTML, like Gecko) Chrome/11.0.660.0 Safari/534.18";
const char S27694[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/534.24 (KHTML	 like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S27695[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/11.0.648.205 Safari/534.16";
const char S27696[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/11.0.648.205 Safari/534.16";
const char S27697[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/11.0.648.205 Safari/534.16";
const char S27698[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One X+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27699[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC One X+ Build/JRO03C)";
const char S27700[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_OneXplus Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27701[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PM63100/1.15.502.9 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27702[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_OneXplus/1.14.161.13 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27703[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_OneXplus/1.14.161.18 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27704[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; X525a Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27705[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; HTC EVARE_UL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27706[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_OneXplus/1.17.161.7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27707[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PM63100/1.08.502.1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27708[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTCOneXplus Build/JRO03C) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27709[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PM63100/1.19.502.1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27710[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC EVARE_UL Build/JRO03C)";
const char S27711[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_OneXplus/1.17.161.8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27712[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; HTC One X+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27713[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HTC_EVARE_UL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27714[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One X+ Build/QX Senseless v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27715[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PM63100/1.15.502.9 Build/QX Senseless v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S27716[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Desire Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27717[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Desire_A8181 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27718[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_Desire_A8181 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27719[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; HTC_Desire-orange-LS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27720[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; HTC Desire 2.33.161.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27721[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC_Desire_A8183 V2.26.841.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27722[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; Desire Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27723[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27724[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_Desire_A8181 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27725[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; iw-; HTC_Desire_A8181 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27726[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27727[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Desire CDMA Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27728[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Desire Build/GRI40)";
const char S27729[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Desire Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27730[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27731[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ERIS-ADR6200 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27732[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27733[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire Build/GINGERVillain-2.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27734[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Desire CDMA Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27735[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27736[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27737[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Desire_A8181 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27738[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27739[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Eris ADR6200 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27740[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Desire CDMA Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27741[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Desire CDMA Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27742[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6200 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27743[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Desire CDMA Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27744[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire Build/GINGERVillain-3.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27745[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Desire CDMA Build/GRI40)";
const char S27746[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; HTC Desire Build/MvMiui) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27747[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; HTC Desire Build/Villain_AuDT 3.2U1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27748[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; HTC Desire Build/Villain_AuPC 3.2U1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27749[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Desire Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S27750[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Desire Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27751[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Desire Build/GRI40)";
const char S27752[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire Build/GRI54) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27753[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Desire Build/GINGERVillain-2.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27754[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Desire Build/GINGERVillain-2.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27755[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; HTC Desire Build/Villain_AuPC 2.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27756[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Desire CDMA Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30";
const char S27757[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Desire Build/GRJ22)";
const char S27758[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Desire CDMA Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27759[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Desire Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S27760[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; HTC Desire Build/Villain_Au 3.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27761[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Desire Build/GRJ90)";
const char S27762[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27763[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27764[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Desire CDMA Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27765[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire Build/GINGERVillain-3.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27766[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Desire CDMA Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27767[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; HTC Desire Build/Villain_AuPC 3.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27768[] = "Mozilla/5.0 (Linux;u;Android 2.3.7;zh-cn;HTC Desire Build) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27769[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27770[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6200 ERIS Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27771[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; HTC Desire Build/GINGERVillain-0.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27772[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC Desire CDMA Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27773[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire Build/GINGERVillain-1.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27774[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire Build/GINGERVillain-1.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27775[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Desire Build/GINGERVillain-2.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27776[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; HTC Desire Build/GINGERVillain-2.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27777[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Desire CDMA Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27778[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC Desire Build/Villain_AuDT 2.8b) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27779[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC Desire Build/Villain_AuPC 2.8b) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27780[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Desire Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S27781[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fa-ir; HTC Desire Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27782[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; HTC Desire Build/Villain_AuDT 3.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27783[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Desire Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S27784[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; Desire Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27785[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; HTC_Desire_A8181 Build/Mod Lonas Sense 0.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27786[] = "Mozilla/5.0 (Linux; U; Android Gingerbread 2.3.7; en-gb; HTC Desire Build/Port and Mod by RMG2; MokeeOS v0.5 RC1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27787[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire Build/Port and Mod by RMG2; MokeeOS v0.6 RC2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27788[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Desire Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S27789[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; HTC Desire Build/SoulMIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27790[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; HTC Desire Build/xMIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27791[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Desire Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S27792[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27793[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; X06HT Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27794[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ru-ru; HTC Desire Build/FCR MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27795[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; HTC Desire Build/Siberia) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27796[] = "Android-Desire/1.0 Linux/2.6.35.7 Android/2.3.6 Release/10.29.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S27797[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; HTC_Desire_A8181 Build/Mod Lonas Sense 0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27798[] = "Android-Desire/1.0 Linux/2.6.35.7 Android/2.3.6 Release/10.30.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S27799[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ru-ru; HTC Desire Build/NMR 2.2.30RC4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27800[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; HTC Desire Build/Miui_Pecha2k) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27801[] = "BlackBerry9650/6.0.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/105";
const char S27802[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9650; en-US) AppleWebKit/534.8+ (KHTML, like Gecko) Version/6.0.0.719 Mobile Safari/534.8+";
const char S27803[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9500 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S27804[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9500 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S27805[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S27806[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:17.0) Gecko/17.0 Firefox/17.0";
const char S27807[] = "Mozilla/5.0 (Windows NT 6.0; rv:17.0) Gecko/17.0 Firefox/17.0";
const char S27808[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.3) Gecko/X Firefox/17.0 GTB7.0 ( .NET CLR 3.5.30729; .NET4.0E)";
const char S27809[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.13";
const char S27810[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.13";
const char S27811[] = "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.13";
const char S27812[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.13";
const char S27813[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.13";
const char S27814[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.13";
const char S27815[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.13";
const char S27816[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.13";
const char S27817[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.13";
const char S27818[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.13";
const char S27819[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.13";
const char S27820[] = "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.13,gzip(gfe)";
const char S27821[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.13";
const char S27822[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.13";
const char S27823[] = "Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.13,gzip(gfe)";
const char S27824[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.13";
const char S27825[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.13";
const char S27826[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.13";
const char S27827[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.13";
const char S27828[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.13";
const char S27829[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.13";
const char S27830[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1) Opera 12.13";
const char S27831[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.13";
const char S27832[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.13";
const char S27833[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6035)) Presto/2.12.388 Version/12.13";
const char S27834[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.13";
const char S27835[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.13";
const char S27836[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.13";
const char S27837[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.13";
const char S27838[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.13";
const char S27839[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.13";
const char S27840[] = "Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.13";
const char S27841[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.13";
const char S27842[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.13";
const char S27843[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.13";
const char S27844[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.13";
const char S27845[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.13";
const char S27846[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.13";
const char S27847[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.13";
const char S27848[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.13";
const char S27849[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.13";
const char S27850[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.13";
const char S27851[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.13";
const char S27852[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.13";
const char S27853[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.13";
const char S27854[] = "Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.13";
const char S27855[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.13";
const char S27856[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.13";
const char S27857[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.13";
const char S27858[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.13";
const char S27859[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.13";
const char S27860[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.13";
const char S27861[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.13";
const char S27862[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.13";
const char S27863[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.13";
const char S27864[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.13";
const char S27865[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.13";
const char S27866[] = "Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.13";
const char S27867[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.13";
const char S27868[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.13";
const char S27869[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.13";
const char S27870[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6032)) Presto/2.12.388 Version/12.13";
const char S27871[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.13";
const char S27872[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.13";
const char S27873[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.13";
const char S27874[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.13";
const char S27875[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6046)) Presto/2.12.388 Version/12.13";
const char S27876[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.13";
const char S27877[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.13";
const char S27878[] = "Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.13";
const char S27879[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.13";
const char S27880[] = "Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.13";
const char S27881[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.13";
const char S27882[] = "Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.13";
const char S27883[] = "Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.13";
const char S27884[] = "Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.13";
const char S27885[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.813.0 Safari/535.1";
const char S27886[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S27887[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S27888[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S27889[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S27890[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S27891[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S27892[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S27893[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10_6_7) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.790.0 Safari/535.1";
const char S27894[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1";
const char S27895[] = "Ruby, Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S27896[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/535.1 (KHTML	 like Gecko) Chrome/14.0.835.186 Safari/535.1";
const char S27897[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/535.1 (KHTML	 like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S27898[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.1 (KHTML	 like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S27899[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/14.0.648.205 Safari/534.16";
const char S27900[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/14.0.648.205 Safari/534.16";
const char S27901[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/14.0.648.205 Safari/534.16";
const char S27902[] = "Mozilla/5.0 (X11; Jolicloud Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Joli OS/1.2 Chromium/13.0.782.107 Chrome/13.0.782.107 Safari/535.1";
const char S27903[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1";
const char S27904[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1";
const char S27905[] = "Mozilla/5.0 Slackware/13.37 (X11; U; Linux x86_64; en-US) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41";
const char S27906[] = "Mozilla/5.0 (X11; Linux amd64) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/13.0.766.0 Safari/534.36";
const char S27907[] = "Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/13.0.765.0 Safari/534.36";
const char S27908[] = "Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/13.0.1410.63 Safari/537.31";
const char S27909[] = "Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/13.0.1410.63 Safari/537.31";
const char S27910[] = "Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/13.0.1410.63 Safari/537.31";
const char S27911[] = "Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/13.0.1410.63 Safari/537.31";
const char S27912[] = "Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/13.0.1410.63 Safari/537.31";
const char S27913[] = "Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/13.0.1410.63 Safari/537.31";
const char S27914[] = "Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/13.0.1410.63 Safari/537.31";
const char S27915[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonX10i Build/private) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27916[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonX10i Build/3.0.1.G.0.78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27917[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonX10i Build/3.0.1.G.0.75 Ultima) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27918[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonX10iv Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27919[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonX10i Build/3.0.1.G.0.76) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27920[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonX10i-o Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27921[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; vi-vn; SonyEricssonX10i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27922[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27923[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonX10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27924[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonX10i Build/3.0.1.G.0.68) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27925[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonX10i Build/3.0.1.G.0.35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27926[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonX10i Build/3.0.1.G.0.70) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27927[] = "Android (Linux; U; Android 2.3.3; fa-ir; X10i) AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S27928[] = "Mozilla/5.0(Linux;U;Android2.3.3;Zh_cn;X10i;480*854;)AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S27929[] = "Mozilla/5.0 (Linux; U; Android 2.3.5 RRQ ilove BMG; es-es; SonyEricssonX10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27930[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; X10i Build/3.0.1.G.0.75)";
const char S27931[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; X10i Build/3.0.1.G.0.75) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S27932[] = "Mozilla/5.0 (Linux; U; Android 2.3.7 pRiMe X; en-gb; X10i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27933[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonX10i Build/6.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27934[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27935[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; X10i Build/6.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27936[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; X10i Build/3.0.1.G.0.75) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S27937[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonX10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27938[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; X10i Build/3.0.1.G.0.75)";
const char S27939[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ar-; SonyEricssonX10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27940[] = "Mozilla/5.0 (Linux; U; Android 2.3.7 pRiMe 7; en-gb; X10i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27941[] = "Mozilla/5.0 (Linux; U; Android 2.3.9; en-gb; Xperia X10 Build/6.0.A.3.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27942[] = "Mozilla/5.0 (Linux; U; Android 2.3.7 V6.0 MM TEAM; en-us; X10i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27943[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonX10S Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27944[] = "Mozilla/5.0 (Linux; U; Android 2.3.7 V7 MM TEAM; en-us; X10i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27945[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27946[] = "Mozilla/5.0 (Linux; U; Android 2.3.9; en-us; Xperia X10 Build/6.0.A.3.73) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27947[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; SonyEricssonX10 Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27948[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; Xperia X10 Build/6.0.B.0.743) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27949[] = "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:16.0) Gecko/X Firefox/16.0";
const char S27950[] = "Mozilla/5.0 (X11: Ubuntu; Linux x86_64; rv:16.0; Rover) Gecko/X Firefox/16.0";
const char S27951[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID X2 Build/4.5.1A-DTN-117-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S27952[] = "Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; DROID X2 Build/4.5.1A-DTN-117-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27953[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID X2 Build/4.5.1A-DTN-150-26) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27954[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;DROID X2 Build/4.5.1A-DTN-117-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27955[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX2 Build/4.5.1A-DTN-117-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27956[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID X2 Build/4.5.1A-DTN-150-30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27957[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID X2 Build/4.5.1A-DTN-150-14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27958[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID X2 Build/4.5.1A-DTN-117-14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27959[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID X2 Build/4.5.1_93) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27960[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID X2 Build/4.5.1A-DTN-117-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27961[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID X2 Build/4.5.1A-DTN-150-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27962[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID X2 Build/4.5.1A-DTN-117-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27963[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID X2 Build/4.5.1A-DTN-150-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27964[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID X2 Build/4.5.1A-DTN-117-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27965[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID X2 Build/4.5.1A-DTN-117-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27966[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID X2 Build/4.5.91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27967[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID X2 Build/4.5.1A-DTN-150-27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27968[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; DROID X2 Build/4.5.1A-DTN-150-30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27969[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID X2 Build/4.5.1A-DTN-117-8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27970[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;DROID X2 Build/4.5.1A-DTN-150-30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27971[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX2 Build/4.5.1A-DTN-150-30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27972[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; DROID X2 Build/4.5.1A-DTN-150-30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27973[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID X2 Build/4.5.1A-DTN-150-30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S27974[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID X2Build/4.5.1A-DTN-150-30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27975[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; DROID X2 Build/4.5.1A-DTN-117-15)";
const char S27976[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; DROID X2 Build/4.5.1A-DTN-150-30)";
const char S27977[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; Milestone X2 Build/45.0.43)";
const char S27978[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Milestone X2 Build/45.0.43) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27979[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27980[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Milestone X2 Build/45.0.25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27981[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27982[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; DROID X2 Build/4.5.1A-DTN-200-18)";
const char S27983[] = "Mozilla/5.0 (Linux; U; Android2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27984[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27985[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROIDX2 Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27986[] = "Mozilla/5.0 (Linux; U;Android 2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27987[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID X2Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27988[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S27989[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us;DROID X2 Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27990[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Milestone X2 Build/45.0.18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27991[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Milestone X2 Build/45.0.25)";
const char S27992[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; MB870 Build/4.5.1A-DTN-150-30)";
const char S27993[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB870 Build/4.5.1A-DTN-150-30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27994[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; DROID X2 Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S27995[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; DROID X2 Build/4.5.1A-DTN-150-30) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S27996[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-18) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S27997[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; ) AppleWebKit/533.9 (KHTML, like Gecko) Maxthon/3.0 Safari/533.9";
const char S27998[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; ) AppleWebKit/534.12 (KHTML, like Gecko) Maxthon/3.0 Safari/534.12";
const char S27999[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/530.6 (KHTML, like Gecko) Maxthon/3.0 Safari/530.6";
const char S28000[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/535.3 (KHTML, like Gecko) Maxthon/3.0 Safari/535.3";
const char S28001[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12";
const char S28002[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; ) AppleWebKit/533.9 (KHTML, like Gecko) Maxthon/3.0 Safari/533.9";
const char S28003[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12";
const char S28004[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.1 (KHTML, like Gecko) Maxthon/3.0.9.10 Safari/533.1";
const char S28005[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; ) AppleWebKit/534.12 (KHTML,like Gecko) Maxthon/3.0 Safari/534.12";
const char S28006[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12";
const char S28007[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/19.0.1084.52 Safari/535.12";
const char S28008[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12";
const char S28009[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12";
const char S28010[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1";
const char S28011[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12";
const char S28012[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12,gzip(gfe) (via translate.google.com)";
const char S28013[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12,gzip(gfe)";
const char S28014[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1,gzip(gfe)";
const char S28015[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1";
const char S28016[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1";
const char S28017[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1";
const char S28018[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/26.0.1410.43 Safari/535.12";
const char S28019[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/26.0.1410.43 Safari/535.12,gzip(gfe)";
const char S28020[] = "Opera/9.80 (MAUI Runtime; Opera Mini/4.4.31762/30.3697; U; en) Presto/2.8.119 Version/11.10";
const char S28021[] = "NokiaE71-2;Mozilla/5.0(symbianos/9.2;u;nokiae71-2/100.07.76 profile/midp-2.0 Configuration/cldc-1.1)applewebkit/413";
const char S28022[] = "NokiaE71x/ATT.03.28 Mozilla/5.0 SymbianOS/9.3; U; [en]; Series60/3.2; Profile/MIDP-2.1 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S28023[] = "Mozilla/5.0 (SymbianOS/9.3 U; Series60/3.2 NokiaE71x Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S28024[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE71-1/510.21.009; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S28025[] = "NokiaE71/1.0 (510.21.009) SymbianOS/9.2 Series60/3.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S28026[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE71-1/200.21.118; 60/3.1 NokiaE71-1/200.21.118; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S28027[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE71-2/500.21.009; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S28028[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE71-3/300.21.012; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S28029[] = "NokiaE71";
const char S28030[] = "Mozilla/4.0 (compatible; Nokia Podcasting; SymbianOS; NokiaE71-1/500.21.009;)";
const char S28031[] = "nokiae71-1";
const char S28032[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NOKIAE71/1.0; Profile/MIDP-2.0 Configuration/CLDC-1.1) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S28033[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NOKIAE71/1.0; Profile/MIDP-2.0 Configuration/CLDC-1.1) AppleWebKit/413 (KHTML, like Gecko) Safari/413 Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NOKIAE71/1.0; Profile/MIDP-2.0 Configuration/CLDC-1.1) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S28034[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11";
const char S28035[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S28036[] = "Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S28037[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11,gzip(gfe) (via translate.google.com)";
const char S28038[] = "Mozilla/5.0 (X11; Linux armv6l) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S28039[] = "Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/20.0.1410.63 Safari/537.31";
const char S28040[] = "Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/20.0.1410.63 Safari/537.31";
const char S28041[] = "Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/20.0.1410.63 Safari/537.31";
const char S28042[] = "Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/20.0.1410.63 Safari/537.31";
const char S28043[] = "Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/20.0.1410.63 Safari/537.31";
const char S28044[] = "Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/20.0.1410.63 Safari/537.31";
const char S28045[] = "Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/20.0.1410.63 Safari/537.31";
const char S28046[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11 (larbin2.6.3@unspecified.mail)";
const char S28047[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe)";
const char S28048[] = "Mozilla/5.0 (X11; U; Linux i586) AppleWebKit/536.6 (KHTML, like Gecko) Chrome/20.0.1092.0 Safari/536.6";
const char S28049[] = "Mozilla/5.0 (Linux; U; Android2.3.3; en-us; pcdadr6350 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28050[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; pcdadr6350 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S28051[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;pcdadr6350 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28052[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; pcdadr6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28053[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-; pcdadr6350 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28054[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; pcdadr6350 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28055[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; pcdadr6350Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28056[] = "Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; pcdadr6350 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28057[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; pcdadr6350 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28058[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ADR6350 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28059[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28060[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; pcdadr6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S28061[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; ADR6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28062[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S28063[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;pcdadr6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28064[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; pcdadr6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28065[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; pcdadr6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28066[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; pcdadr6350Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28067[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-; pcdadr6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28068[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; ADR6350 Build/GRI40)";
const char S28069[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; ADR6350 Build/GRJ22)";
const char S28070[] = "Logos/0.9.4 (Linux; U; Android 2.3.4; en_US; ADR6350 Build/GRJ22)";
const char S28071[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6350Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28072[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ADR6350 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28073[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; pcdadr6350 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28074[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; pcdadr6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28075[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; ADR6350 Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S28076[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;ADR6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28077[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6350 Build/GRJ22) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S28078[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; ADR6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28079[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; ADR6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28080[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; ADR6350 Build/GRJ90)";
const char S28081[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; ADR6350 Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S28082[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6350 Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30";
const char S28083[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/546.10 (KHTML, like Gecko) Version/6.0 Mobile/7E18WD Safari/8536.25";
const char S28084[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A381 Safari/8536.25";
const char S28085[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S28086[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A394 Safari/8536.25";
const char S28087[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A4372q Safari/8536.25";
const char S28088[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A386 Safari/8536.25";
const char S28089[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A396 Safari/8536.25";
const char S28090[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A392 Safari/8536.25";
const char S28091[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A383 Safari/8536.25";
const char S28092[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A400 Safari/8536.25";
const char S28093[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.44 (KHTML, like Gecko) Version/6.0 Mobile/11A404 Safari/8536.25";
const char S28094[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A4372r Safari/8536.25";
const char S28095[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A401 Safari/8536.25";
const char S28096[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.44 (KHTML, like Gecko) Version/6.0 Mobile/11A403 Safari/8536.25";
const char S28097[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A4400f Safari/8536.25";
const char S28098[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.44 (KHTML, like Gecko) Version/6.0 Mobile/11A407 Safari/8536.25";
const char S28099[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.44 (KHTML, like Gecko) Version/6.0 Mobile/11A409 Safari/8536.25";
const char S28100[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.44 (KHTML, like Gecko) Version/6.0 Mobile/11A408 Safari/8536.25";
const char S28101[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.44 (KHTML, like Gecko) Version/6.0 Mobile/11A410 Safari/8536.25";
const char S28102[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11A416 Safari/8536.25";
const char S28103[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11A415 Safari/8536.25";
const char S28104[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11A414 Safari/8536.25";
const char S28105[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S28106[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11A4414e Safari/8536.25";
const char S28107[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11A421 Safari/8536.25";
const char S28108[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11A4414c Safari/8536.25";
const char S28109[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11A428 Safari/8536.25";
const char S28110[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.49.1 (KHTML, like Gecko) Version/6.0 Mobile/11A432 Safari/8536.25";
const char S28111[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11A430 Safari/8536.25";
const char S28112[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/11A435 Safari/8536.25";
const char S28113[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/11A436 Safari/8536.25";
const char S28114[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/11A438 Safari/8536.25";
const char S28115[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/11A4435d Safari/8536.25";
const char S28116[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/11A439 Safari/8536.25";
const char S28117[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S28118[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S28119[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S28120[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S28121[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S28122[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Avant Browser)";
const char S28123[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Avant Browser)";
const char S28124[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Avant Browser)";
const char S28125[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; Avant Browser)";
const char S28126[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; AvantBrowser)";
const char S28127[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; SV1; Avant Browser)";
const char S28128[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; Avant Browser; Avant Browser)";
const char S28129[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; Avant Browser; Avant Browser)";
const char S28130[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Avant Browser)";
const char S28131[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; SV1; Avant Browser)";
const char S28132[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Avant Browser)";
const char S28133[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.8.131 Version/11.11";
const char S28134[] = "Opera/9.80 (Windows NT 6.1; U; en) Presto/2.8.131 Version/11.10";
const char S28135[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.8.131 Version/11.10";
const char S28136[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.8.131 Version/11.11";
const char S28137[] = "Opera/9.80 (Windows NT 6.1; U; en) Presto/2.8.131 Version/11.11";
const char S28138[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.8.131 Version/11.11";
const char S28139[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.8.131 Version/11.10";
const char S28140[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.8.131 Version/11.10";
const char S28141[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.8.131 Version/11.10";
const char S28142[] = "Opera/9.80 (Windows NT 6.1; U; en) Presto/2.8.131 Version/11.10,gzip(gfe) (via translate.google.com)";
const char S28143[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.8.131 Version/11.10";
const char S28144[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.8.131 Version/11.10";
const char S28145[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.8.131 Version/11.10";
const char S28146[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.8.131 Version/11.10";
const char S28147[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4110); ru) Presto/2.8.131 Version/11.11";
const char S28148[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4110); ru) Presto/2.8.131 Version/11.10";
const char S28149[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.8.131 Version/11.11";
const char S28150[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.8.131 Version/11.11";
const char S28151[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.8.131 Version/11.11";
const char S28152[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.8.131 Version/11.11";
const char S28153[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.8.131 Version/11.11";
const char S28154[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.8.131 Version/11.11";
const char S28155[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.8.131 Version/11.11";
const char S28156[] = "Opera/11.1 (Windows NT 6.1: U: en)";
const char S28157[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.8.131 Version/11.10";
const char S28158[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.8.131 Version/11.10";
const char S28159[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.8.131 Version/11.10";
const char S28160[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.8.131 Version/11.10";
const char S28161[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.8.131 Version/11.11";
const char S28162[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.8.131 Version/11.10";
const char S28163[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.8.131 Version/11.10";
const char S28164[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.8.131 Version/11.10";
const char S28165[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.8.131 Version/11.10";
const char S28166[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.8.131 Version/11.11";
const char S28167[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.8.131 Version/11.11";
const char S28168[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.8.131 Version/11.11,gzip(gfe) (via translate.google.com)";
const char S28169[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.8.131 Version/11.11";
const char S28170[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.8.131 Version/11.10";
const char S28171[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.8.131 Version/11.11";
const char S28172[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.8.131 Version/11.11";
const char S28173[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.8.131 Version/11.11";
const char S28174[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.8.131 Version/11.10";
const char S28175[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.8.131 Version/11.11";
const char S28176[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.8.131 Version/11.10";
const char S28177[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.8.131 Version/11.11";
const char S28178[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.8.131 Version/11.11";
const char S28179[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.8.131 Version/11.10";
const char S28180[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.8.131 Version/11.10";
const char S28181[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.8.131 Version/11.11";
const char S28182[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5196); ru) Presto/2.8.131 Version/11.11";
const char S28183[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.8.131 Version/11.11";
const char S28184[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5265); ru) Presto/2.8.131 Version/11.10";
const char S28185[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.8.131 Version/11.11";
const char S28186[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.8.131 Version/11.11";
const char S28187[] = "Opera/11.1 (Windows NT 6.1; U; en)";
const char S28188[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.8.131 Version/11.11";
const char S28189[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.8.131 Version/11.11";
const char S28190[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.8.131 Version/11.10";
const char S28191[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.8.131 Version/11.10";
const char S28192[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.8.131 Version/11.11";
const char S28193[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.8.131 Version/11.10";
const char S28194[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.8.131 Version/11.11";
const char S28195[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.8.131 Version/11.11";
const char S28196[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.8.131 Version/11.11";
const char S28197[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.8.131 Version/11.11,gzip(gfe)";
const char S28198[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.8.131 Version/11.10,gzip(gfe)";
const char S28199[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.8.131 Version/11.11";
const char S28200[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.8.131 Version/11.10";
const char S28201[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5308); ru) Presto/2.8.131 Version/11.11";
const char S28202[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5328); ru) Presto/2.8.131 Version/11.11";
const char S28203[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.8.131 Version/11.11";
const char S28204[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.8.131 Version/11.10";
const char S28205[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.8.131 Version/11.10";
const char S28206[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.8.131 Version/11.10";
const char S28207[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.8.131 Version/11.11";
const char S28208[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.8.131 Version/11.10";
const char S28209[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28210[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T679 Build/BPEARromICB.RC.3.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28211[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T679 Build/Peach.Sunrise.1.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28212[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28213[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; SGH-T679M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28214[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SGH-T679 Build/GINGERBREAD)";
const char S28215[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us;SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28216[] = "Mozilla/5.0 (Linux; U; Android2.3.5; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28217[] = "Mozilla/5.0 (Linux; U;Android 2.3.5; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28218[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T679Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28219[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S28220[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T679 Build/Peach.Sunrise.1.2a) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28221[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-T679 Build/GINGERBREAD)";
const char S28222[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SGH-T679M Build/GINGERBREAD)";
const char S28223[] = "Mozilla/5.0 (Linux; U; Android2.3.6; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28224[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us;SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28225[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S28226[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T679Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28227[] = "Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28228[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S28229[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S28230[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T679 Build/ExHibition) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28231[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T679 Build/Galaxy ExHibition) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28232[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28233[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; SGH-T679M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28234[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SGH-T679 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S28235[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S28236[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7";
const char S28237[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7";
const char S28238[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7";
const char S28239[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7 AppEngine-Google; (+http://code.google.com/appengine; appid: s~feedly-social)";
const char S28240[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7";
const char S28241[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7 AppEngine-Google; (+http://code.google.com/appengine; appid: s~feedly-twitter)";
const char S28242[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S28243[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7 AppEngine-Google; (+http://code.google.com/appengine; appid: s~feedly-images)";
const char S28244[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10\\_7\\_2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.59 Safari/535.7";
const char S28245[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.458 Mobile Safari/537.10+";
const char S28246[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.413 Mobile Safari/537.10+";
const char S28247[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.479 Mobile Safari/537.10+";
const char S28248[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.693 Mobile Safari/537.10+";
const char S28249[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.278 Mobile Safari/537.10+";
const char S28250[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.212 Mobile Safari/537.10+";
const char S28251[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.416 Mobile Safari/537.10+";
const char S28252[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.422 Mobile Safari/537.10+";
const char S28253[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.642 Mobile Safari/537.10+";
const char S28254[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.542 Mobile Safari/537.10+";
const char S28255[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.512 Mobile Safari/537.10+";
const char S28256[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.440 Mobile Safari/537.10+";
const char S28257[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.206 Mobile Safari/537.10+";
const char S28258[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.311 Mobile Safari/537.10+";
const char S28259[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.362 Mobile Safari/537.10+";
const char S28260[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.582 Mobile Safari/537.10+";
const char S28261[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.401 Mobile Safari/537.10+";
const char S28262[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.395 Mobile Safari/537.10+";
const char S28263[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.155 Mobile Safari/537.10+";
const char S28264[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.297 Mobile Safari/537.10+";
const char S28265[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.461 Mobile Safari/537.10+";
const char S28266[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.455 Mobile Safari/537.10+";
const char S28267[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.786 Mobile Safari/537.10+";
const char S28268[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.777 Mobile Safari/537.10+";
const char S28269[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.269 Mobile Safari/537.10+";
const char S28270[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.527 Mobile Safari/537.10+";
const char S28271[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.254 Mobile Safari/537.10+";
const char S28272[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.392 Mobile Safari/537.10+";
const char S28273[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.368 Mobile Safari/537.10+";
const char S28274[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.383 Mobile Safari/537.10+";
const char S28275[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.305 Mobile Safari/537.10+";
const char S28276[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.699 Mobile Safari/537.10+";
const char S28277[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.152 Mobile Safari/537.10+";
const char S28278[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.230 Mobile Safari/537.10+";
const char S28279[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.491 Mobile Safari/537.10+";
const char S28280[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.594 Mobile Safari/537.10+";
const char S28281[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.789 Mobile Safari/537.10+";
const char S28282[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.684 Mobile Safari/537.10+";
const char S28283[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.476 Mobile Safari/537.10+";
const char S28284[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.795 Mobile Safari/537.10+";
const char S28285[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.792 Mobile Safari/537.10+";
const char S28286[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.224 Mobile Safari/537.10+";
const char S28287[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.816 Mobile Safari/537.10+";
const char S28288[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.42 Mobile Safari/537.10+";
const char S28289[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.837 Mobile Safari/537.10+";
const char S28290[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1002 Mobile Safari/537.10+";
const char S28291[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1110 Mobile Safari/537.10+";
const char S28292[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1020 Mobile Safari/537.10+";
const char S28293[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1471 Mobile Safari/537.10+";
const char S28294[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1320 Mobile Safari/537.10+";
const char S28295[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1483 Mobile Safari/537.10+";
const char S28296[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1507 Mobile Safari/537.10+";
const char S28297[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1720 Mobile Safari/537.10+";
const char S28298[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1429 Mobile Safari/537.10+";
const char S28299[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1447 Mobile Safari/537.10+";
const char S28300[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1756 Mobile Safari/537.10+";
const char S28301[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1597 Mobile Safari/537.10+";
const char S28302[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1609 Mobile Safari/537.10+";
const char S28303[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1627 Mobile Safari/537.10+";
const char S28304[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1666 Mobile Safari/537.10+";
const char S28305[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1832 Mobile Safari/537.10+";
const char S28306[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1916 Mobile Safari/537.10+";
const char S28307[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1895 Mobile Safari/537.10+";
const char S28308[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2336 Mobile Safari/537.10+";
const char S28309[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2342 Mobile Safari/537.10+";
const char S28310[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1485 Mobile Safari/537.10+";
const char S28311[] = "LinkedInBlackBerry/10.1.2 (5) Q10 SQN100-3/10.1.0.1483 (Research In Motion;RIM BlackBerry Device;Q10;SQN100-3;1)";
const char S28312[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2390 Mobile Safari/537.10+";
const char S28313[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2038 Mobile Safari/537.10+";
const char S28314[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2420 Mobile Safari/537.10+";
const char S28315[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4285 Mobile Safari/537.10+";
const char S28316[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4333 Mobile Safari/537.10+";
const char S28317[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2062 Mobile Safari/537.10+";
const char S28318[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4369 Mobile Safari/537.10+";
const char S28319[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1552 Mobile Safari/537.10+";
const char S28320[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1411 Mobile Safari/537.10+";
const char S28321[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1910 Mobile Safari/537.10+";
const char S28322[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2354 Mobile Safari/537.10+";
const char S28323[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2025 Mobile Safari/537.10+";
const char S28324[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4429 Mobile Safari/537.10+";
const char S28325[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2009 Mobile Safari/537.10+";
const char S28326[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4453 Mobile Safari/537.10+";
const char S28327[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2121 Mobile Safari/537.10+";
const char S28328[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4537 Mobile Safari/537.10+";
const char S28329[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4477 Mobile Safari/537.10+";
const char S28330[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4543 Mobile Safari/537.10+";
const char S28331[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1200 Mobile Safari/537.10+";
const char S28332[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1880 Mobile Safari/537.10+";
const char S28333[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4393 Mobile Safari/537.10+";
const char S28334[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4597 Mobile Safari/537.10+";
const char S28335[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4633 Mobile Safari/537.10+";
const char S28336[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4561 Mobile Safari/537.10+";
const char S28337[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4651 Mobile Safari/537.10+";
const char S28338[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4687 Mobile Safari/537.10+";
const char S28339[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2324 Mobile Safari/537.10+";
const char S28340[] = "LinkedInBlackBerry/10.1.2 (5) Q10 SQN100-2/10.1.0.2038 (Research In Motion;RIM BlackBerry Device;Q10;SQN100-2;1)";
const char S28341[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4699 Mobile Safari/537.10+";
const char S28342[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4780 Mobile Safari/537.10+";
const char S28343[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4828 Mobile Safari/537.10+";
const char S28344[] = "LinkedInBlackBerry/10.1.4 (1) Q10 SQN100-1/10.1.0.4633 (Research In Motion;RIM BlackBerry Device;Q10;SQN100-1;1)";
const char S28345[] = "LinkedInBlackBerry/10.1.4 (1) Q10 SQN100-5/10.1.0.1720 (Research In Motion;RIM BlackBerry Device;Q10;SQN100-5;1)";
const char S28346[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9860; en-GB) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.296 Mobile Safari/534.11+";
const char S28347[] = "BlackBerry9860/7.0.0.576 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102";
const char S28348[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:16.0) Gecko/X Firefox/16.0";
const char S28349[] = "Mozilla/5.0 (Windows NT 5.1; rv:16.0) Gecko/X Firefox/16.0,gzip(gfe)";
const char S28350[] = "Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:16.0) Gecko/X Firefox/16.0";
const char S28351[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; rv:16.0.1) Gecko/X Firefox/16.0.1";
const char S28352[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Incredible S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28353[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTCS710e/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28354[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; IncredibleS_S710e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28355[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Incredible S Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S28356[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_IncredibleS_S710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28357[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Incredible S Build/IMM76D)";
const char S28358[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; HTC Incredible S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28359[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; HTC_IncredibleS_S710e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28360[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Incredible S Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S28361[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Incredible S Build/IML74K)";
const char S28362[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Incredible S Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S28363[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCS710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28364[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-; IncredibleS_S710e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28365[] = "Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Incredible S Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S28366[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Incredible S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28367[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Incredible_S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28368[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-; HTC Incredible S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28369[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_IncredibleS_S710e Build/2.15.401.100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28370[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_IncredibleS_S710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28371[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I605 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28372[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SCH-I605 Build/JRO03C)";
const char S28373[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I605 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28374[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I605 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28375[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I605 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28376[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; SCH-I605 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28377[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-I605 Build/JZO54K)";
const char S28378[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SCH-I605 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28379[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=t0ltevzw;devm=SCH-I605;man=samsung;brw=Android;shwd=720x1280;shwa=720x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S28380[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S28381[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-P5113 Build/IML74K)";
const char S28382[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S28383[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P5113 Build/IMM76D)";
const char S28384[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S28385[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28386[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28387[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-P5113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S28388[] = "Logos/1.1.3 (Linux; U; Android 4.0.4; en-CA; GT-P5113 Build/IMM76D.P5113UEBLE4)";
const char S28389[] = "com.google.android.youtube/3.5.5(Linux; U; Android 4.0.3; en_US; GT-P5113 Build/IML74K) gzip";
const char S28390[] = "com.google.android.youtube/4.0.23(Linux; U; Android 4.0.4; en_US; GT-P5113 Build/IMM76D) gzip";
const char S28391[] = "Logos/2.1.2 (Linux; U; Android 4.0.4; en-US; GT-P5113 Build/IMM76D.P5113UEBLG5)";
const char S28392[] = "Mozilla/5.0 (Android 4.0.3; en-us; GT-P5113 Build/IML74K) FreeWheelAdManager/4.9.2-r8793-X;com.gotv.crackle.handset/X";
const char S28393[] = "Mozilla/5.0 (Android 4.0.4; en-us; GT-P5113 Build/IMM76D) FreeWheelAdManager/5.0.0-r8951-X;com.gotv.crackle.handset/X";
const char S28394[] = "Logos/2.1.2 (Linux; U; Android 4.0.4; en-US; GT-P5113 Build/IMM76D.P5113UEBLG2)";
const char S28395[] = "Android/4.4.1 (Linux; U; Android 4.0.4; en_US; GT-P5113)";
const char S28396[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S28397[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/5.0.2 Version/4.0 Safari/534.30 Safari/6533.18.5";
const char S28398[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S28399[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5113 Build/IMM76D; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S28400[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S28401[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5113 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S28402[] = "Logos/2.2.3 (Linux; U; Android 4.0.3; en-US; GT-P5113 Build/IML74K.P5113UEALD7)";
const char S28403[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/1.9.11;FBDM/{density=1.0,width=800,height=1232};FBLC/zh_TW;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P5113;FBSV/4.0.4;]";
const char S28404[] = "Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 4.0.4; SAMSUNG GT-P5113 Build/ICE_CREAM_SANDWICH_MR1);";
const char S28405[] = "BBC News (AndroidApp; 1.1; 9) GT-P5113 (Android 4.0.4, SDK 15)";
const char S28406[] = "Android/4.4.1 (Linux; U; Android 4.0.3; en_US; GT-P5113)";
const char S28407[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28408[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P5113 Build/IMM76D like Xoom) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S28409[] = "com.google.android.apps.plus/X (Linux; U; Android 4.0.4; en_US; GT-P5113; Build/IMM76D) (gzip)";
const char S28410[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28411[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 androidAppVersion/2.5.16 AndroidWebkitWrapper BillingSupported";
const char S28412[] = "Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-CA; GT-P5113 Build/IMM76D.P5113UEBLH3)";
const char S28413[] = "Groupon/2.1.2486 (Android 4.0.4; Samsung Espresso10wifi / Samsung GT-P5113; )[preload=false;locale=en_CA;clientidbase=android-samsung]";
const char S28414[] = "Dalvik 2.04.12078; (Linux; U; Android 4.0.4; GT-P5113; ON) de4a3718b1a6296; 1280x752x1.0";
const char S28415[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; GT-P5113 Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S28416[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5113 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S28417[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MantanoReader";
const char S28418[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D   http://stubes.tk) AppleWebKit/534.30 (KHTML, like Gecko   http://stubes.tk) Version/4.0 Safari/534.30";
const char S28419[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5113 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28420[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28421[] = "VKAndroidApp/2.7.6-64 (Android 4.0.4; SDK 15; armeabi-v7a; samsung GT-P5113; ru)";
const char S28422[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5113 Build/IML74K) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S28423[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5690/S5690BVLC5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28424[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5690 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28425[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; GT-S5690M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28426[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-S5690L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28427[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5690 Build/GINGERBREAD)";
const char S28428[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; GT-S5690R Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28429[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5690/S5690BVLD1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28430[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5690-ORANGE/S5690BVLD1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28431[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-S5690/S5690BOLE2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28432[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5690/S5690BVLF2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28433[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5690-ORANGE/S5690BVLF2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28434[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-S5690-ORANGE/S5690BVLC5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28435[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-S5690/S5690BOLL1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28436[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5690 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S28437[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5690L Build/GINGERBREAD)";
const char S28438[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5690 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28439[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5690R Build/GINGERBREAD)";
const char S28440[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5690/S5690BVLL2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28441[] = "Mozilla/5.0 (Linux; U; Android 2.3.6_UOU?-O??O??; iw-il; GT-S5690 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28442[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ro-ro; GT-S5690-ORANGE/S5690BVLL2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28443[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/28.0.1500.71 Chrome/28.0.1500.71 Safari/537.36";
const char S28444[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/28.0.1500.71 Chrome/28.0.1500.71 Safari/537.36";
const char S28445[] = "Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/28.0.1500.71 Chrome/28.0.1500.71 Safari/537.36";
const char S28446[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/28.0.1500.71 Chrome/28.0.1500.71 Safari/537.36,gzip(gfe)";
const char S28447[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/28.0.1500.71 Chrome/28.0.1500.71 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S28448[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/28.0.1500.71 Chrome/28.0.1500.71 Safari/537.36,gzip(gfe)";
const char S28449[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/28.0.1500.71 Chrome/28.0.1500.71 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)";
const char S28450[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:12.0) Gecko/X Firefox/12.0";
const char S28451[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:12.0) Gecko/X Firefox/12.0";
const char S28452[] = "Mozilla/5.0 (compatible; Windows; U; Windows NT 6.2; WOW64; en-US; rv:12.0) Gecko/X Firefox/12.0";
const char S28453[] = "Mozilla/5.0 (compatible; Windows; U; Windows NT 6.2; en-US; rv:12.0) Gecko/X Firefox/12.0";
const char S28454[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100P-ORANGE/I9100PBVLP6 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28455[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100P Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28456[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100P-ORANGE/I9100PBVLP8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28457[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100P-ORANGE/I9100PBVLPA Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28458[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; SAMSUNG GT-I9100P/I9100PXXLPS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28459[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-I9100P Build/IML74K)";
const char S28460[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; SAMSUNG GT-I9100P/I9100PXXLPP Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28461[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; GT-I9100P Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28462[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100P-ORANGE/I9100PBVLPC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28463[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9100P Build/IMM76D)";
const char S28464[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SAMSUNG GT-I9100P/I9100PXXLQ8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28465[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100P-ORANGE/I9100PBVLPE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28466[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-I9100P Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28467[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9220; en) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.337 Mobile Safari/534.11+";
const char S28468[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9.1";
const char S28469[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Safari/528.17";
const char S28470[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S28471[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19.1";
const char S28472[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S28473[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S28474[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; ru-RU) AppleWebKit/528.16 (KHTML, like Gecko) Version/4.0 Safari/528.16";
const char S28475[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.26.9 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S28476[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S28477[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S28478[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S28479[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28480[] = "Mozilla/5.0 (Linux; Android 4.1.1; SC-02E Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28481[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S28482[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I317 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28483[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28484[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S28485[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28486[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28487[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S28488[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28489[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28490[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28491[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28492[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28493[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28494[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28495[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28496[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105T Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28497[] = "Mozilla/5.0 (Linux; Android 4.1.1; SPH-L900 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28498[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S28499[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105T Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S28500[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889V Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28501[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S28502[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I317M Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28503[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S28504[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R950 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28505[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28506[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28507[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S28508[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S28509[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S28510[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S28511[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889V Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S28512[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-R950 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28513[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R950 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S28514[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28515[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-N719 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28516[] = "Mozilla/5.0 (Linux; Android 4.1.1; SPH-L900 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S28517[] = "Mozilla/5.0 (Linux; Android 4.1.9; GT-N7100 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28518[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I317 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S28519[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889V Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28520[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-N719 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28521[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S28522[] = "Mozilla/5.0 (Linux; Android 4.1.1; SHV-E250S Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S28523[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S28524[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S28525[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC-A9192/1.0 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28526[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Inspire 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28527[] = "Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; HTC-A9192/1.0 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28528[] = "Mozilla/5.0 (Linux; U; Android2.3.3; en-us; HTC-A9192/1.0 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28529[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;HTC-A9192/1.0 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28530[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Inspire\" 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28531[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC-A9192 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28532[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC-A9192 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28533[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_Inspire_4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28534[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Inspire 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28535[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC-A9192/1.0 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28536[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Inspire 4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28537[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC-A9192/1.0 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28538[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Inspire 4G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28539[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Inspire 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28540[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Inspire 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28541[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Inspire 4G Build/GRJ90)";
const char S28542[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Inspire 4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28543[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC-A9192/1.0 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28544[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; Inspire 4G Build/GRI40)";
const char S28545[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Inspire 4G Build/GRI40; Absolution v3.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28546[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Inspire 4G Build/Eva.MIUI-2.1.13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28547[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC-A9192 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28548[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Inspire 4G Build/Eva.MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28549[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC-A9192/1.0 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S28550[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC-A9192/1.0Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28551[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Inspire 4G Build/Eva.MIUI.1.12.16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28552[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC-A9192/1.0 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28553[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Inspire 4G Build/GRJ90)";
const char S28554[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC-A9192/1.0 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28555[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC-A9192/1.0 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mob";
const char S28556[] = "Opera/9.5  (Windows NT 5.1; U; ko) Presto/2.0.0";
const char S28557[] = "Opera/9.50 (Windows NT 5.1; MDA_Compact_V/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1; U; de)";
const char S28558[] = "Opera/9.50 (Windows NT 5.1; U; en)";
const char S28559[] = "Opera/9.51 (Windows NT 5.1; U; ru)";
const char S28560[] = "Opera/9.52 (Windows NT 5.1; U; ru)";
const char S28561[] = "Opera/9.51 (Windows NT 5.2; U; en)";
const char S28562[] = "Opera/9.5 (Windows NT 5.1; U; en)";
const char S28563[] = "Opera/9.50 (Windows NT 5.2; U; zh-cn)";
const char S28564[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; ru) Opera 9.52";
const char S28565[] = "Opera/9.52 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru)";
const char S28566[] = "Opera/9.52 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru)";
const char S28567[] = "Opera/9.51 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru)";
const char S28568[] = "Opera/9.53 (Windows NT 5.1; U; ru)";
const char S28569[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; en) Opera 9.50";
const char S28570[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; ru) Opera 9.51";
const char S28571[] = "Opera/9.52 (Windows NT 5.2; U; ru)";
const char S28572[] = "Opera/9.50 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru)";
const char S28573[] = "Mozilla/5.0 (X11; Linux x86_64; en; rv:2.0) Gecko/X Firefox/4.0 Opera 12.00";
const char S28574[] = "Opera/9.80 (Linux; U; en) Presto/2.10.254 Version/12.00";
const char S28575[] = "Opera/9.80 (X11; Linux i686; U; en) Presto/2.10.289 Version/12.01";
const char S28576[] = "Opera/9.80 (X11; Linux x86_64) Presto/2.12.423 Version/12.00";
const char S28577[] = "Opera/9.80 (Linux i686; U; fr) Presto/2.10.287 Version/12.00 ; SC/IHD92 STB";
const char S28578[] = "Opera/9.80 (Linux armv7l; U;  CE-HTML/1.0 NETTV/4.1.1; en) Presto/2.10.287 Version/12.00";
const char S28579[] = "NokiaC3-00/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S28580[] = "NokiaC3-00.1/5.0(ATT.03.61) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S28581[] = "NokiaC3-00/5.0 (04.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac3-00";
const char S28582[] = "NokiaC3-00/5.0 (08.71) Profile/MIDP-2.1 Configuration/CLDC-1.1 NokiaC3-00/5.0 (08.71) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S28583[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; MOT-XT910 Build/6.5.1-73_SPU-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28584[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; MOT-XT910 Build/6.5.1_73_SLC-25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28585[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; MOT-XT910 Build/6.5.1-73_SLC-24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28586[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; XT910 Build/6.5.1-73_SPU-11-M1-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28587[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; XT910 Build/6.5.1-167_SPU-14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28588[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; XT910 Build/6.5.1-167-SPD-IRD-40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28589[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; MOT-XT910 Build/6.5.1-73_SPU-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28590[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; XT910 Build/6.5.1_73-spd-ird-12) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S28591[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; MOT-XT910 Build/6.5.1-167_SLC-M33.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28592[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; XT910 Build/6.5.1-167_SLC-M33.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28593[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; XT910 Build/6.5.1-167_SLC-M33.1) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30";
const char S28594[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; XT910 Build/6.5.1_73_SLC-24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28595[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MOT-XT910 Build/6.5.1-167_SPU-15-M2-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28596[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MOT-XT910 Build/6.5.1-167_SPU-15-M2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28597[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; MOT-XT910 Build/6.5.1_73-SPD-IRD-21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28598[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; MOT-XT910 Build/6.5.1_73_SLC-24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28599[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; XT910 Build/6.5.1_73-SPD-IRD-21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28600[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; MOT-XT910 Build/6.5.1-73_SPU-11-M1-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28601[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MOT-XT910 Build/6.5.1-167-SPD-IRD-40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28602[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; MOT-XT910 Build/6.5.1-73_SPU-9-TA-6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28603[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; MOT-XT910 Build/6.5.1-167_SPU-15-M2-1-YO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28604[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MOT-XT910 Build/6.5.1-167_SPU-15-M2-1-DE) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28605[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; XT910 Build/6.5.1-73_SLC-24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28606[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MOT-XT910 Build/6.5.1-167_SLC-M37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28607[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; XT910 Build/6.5.1-167_SPU-15-M2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28608[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; XT910 Build/6.5.1-167_SPU-15-M2-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28609[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MOT-XT910 Build/6.5.1_73-spd-ird-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28610[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; MOT-XT910 Build/6.5.1_73_SLC-8.3M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28611[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; XT910 Build/6.5.1-73_SPU-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28612[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; XT910 Build/6.5.1_73-spd-ird-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28613[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; MOT-XT910 Build/6.5.1-73_SLC-ME6.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28614[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; MOT-XT910 Build/6.5.1-73_SPU-9-TA-4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28615[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; XT910K Build/6.5.1-8_SDK-31_LE-106) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28616[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; XT910 Build/6.5.1-167-SPD-IRD-40)";
const char S28617[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; XT910 Build/6.5.1_73_SLC-25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28618[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; XT910 Build/6.5.1-167_SPU-15-M2-1-DE) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28619[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; MOT-XT910 Build/6.5.1_73_SLC-8.2M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28620[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MOT-XT910 Build/6.5.1_73-spd-ird-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28621[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; MOT-XT910 Build/6.5.1-73_SPU-9-TA-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28622[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; XT910K Build/6.5.1-8_SDK-31_LE-58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28623[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; MOT-XT910 Build/6.5.1-73_SPU-11-M1-2-DE) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28624[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MOT-XT910 Build/6.5.1-73_SPU-11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28625[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MOT-XT910 Build/6.5.1-167_SLC-M36_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28626[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; XT910 Build/6.5.1-73_SLC-ME6.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28627[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; he-il; XT910 Build/6.5.1_73-spd-ird-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28628[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; MOT-XT910 Build/6.5.1-73_SPU-11-M1-2-SFR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28629[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; MOT-XT910 Build/6.5.1_73_SLC-8.1M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28630[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT910 Build/6.5.1-167_SLC-M37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28631[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT910 Build/6.5.1-167_SLC-M36_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28632[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT910 Build/6.5.1_GC-136-SPDU-13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28633[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; XT910 Build/6.5.1-167_SPU-15-M2-1-YO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28634[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; XT910S Build/6.5.1-8_SDS-29_LE-106) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28635[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MOT-XT910 Build/6.5.1-167_SPU-15-M2-1-FR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28636[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; MOT-XT910 Build/6.5.1-167_SPU-15-M2-4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28637[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT910 Build/6.5.1_GC-88-SPDU-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28638[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; XT910S Build/6.5.1-8_SDS-29_LE-58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28639[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MOT-XT910 Build/6.5.1-150_SPU-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28640[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; MOT-XT910 Build/6.5.1-73_SPU-9-TA-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28641[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; XT910 Build/6.5.1-73_SPU-9-TA-6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28642[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; XT910 Build/6.5.1-73_SPU-11-M1-2-SFR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28643[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; XT910 Build/6.5.1_73_SLC-8.3M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28644[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MOT-XT910 Build/6.5.1-167_SPU-15-M2-1-CEE) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28645[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MOT-XT910 Build/6.5.1-167_SLC-M36_R02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28646[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT910 Build/6.5.1-167-SPD-IRD-40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S28647[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; XT910 Build/6.5.1_73-SPD-IRD-21)";
const char S28648[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MOT-XT910 Build/6.5.1-167_SLC-M34_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28649[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; he-il; MOT-XT910 Build/6.5.1-167-SPD-IRD-39) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28650[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; XT910 Build/6.5.1-150_SPU-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28651[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MOT-XT910 Build/6.5.1-167_SPU-13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28652[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MOT-XT910 Build/6.5.1-144_SPU-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28653[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; XT910 Build/6.5.1-73_SPU-11-M1-2-DE) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28654[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; el-gr; MOT-XT910 Build/6.5.1-73_SPU-9-TA-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28655[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; da-dk; XT910 Build/6.5.1-167_SPU-15-M2-1-CEE) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28656[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MOT-XT910 Build/6.5.1-167_SPU-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28657[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT910 Build/6.5.1_GC-136-SPDU-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28658[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; XT910 Build/6.5.1_73_SLC-8.1M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28659[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; XT910 Build/6.5.1_73_SLC-8.2M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28660[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT910 Build/6.5.1-167_SLC-M34_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28661[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; XT910 Build/6.5.1-167_SPU-15-M2-1-FR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28662[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT910 Build/6.5.1-167-SPD-IRD-40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S28663[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; MOT-XT910 Build/6.5.1-73_SPU-11-M1-2-RT) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28664[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; MOT-XT910 Build/6.5.1-167-SPD-IRD-37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28665[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; MOT-XT910 Build/6.5.1-167_SLC-M33) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28666[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT910 Build/6.5.1_GC-136-SPDU-13) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S28667[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; XT910 Build/6.5.1_73-SPD-IRD-21) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S28668[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; XT910 Build/6.5.1_73-spd-ird-12) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S28669[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT910 Build/6.5.1-167_SLC-M36_R02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28670[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MOT-XT910 Build/6.5.1-167_SLC-M36_R03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28671[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MOT-XT910 Build/6.5.1-167_SPU-16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28672[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-I405 4G Build/EGB) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28673[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I405 4G Build/EH2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28674[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-I405 4G Build/EH2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28675[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I405 4G Build/EI2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28676[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I405 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28677[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SCH-I405 Build/GINGERBREAD)";
const char S28678[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I405 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S28679[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I405 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28680[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I405 4G Build/FC1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28681[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I405U Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 USCC-I405U";
const char S28682[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I405 4G Build/FF1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28683[] = "Android/1.0 (SCH-I405 GINGERBREAD)";
const char S28684[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-I405 Build/GINGERBREAD)";
const char S28685[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-I405U Build/GINGERBREAD)";
const char S28686[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I405 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S28687[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I405 4G Build/GC1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28688[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SCH-I405 4G Build/GC1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28689[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; SonyST23 Build/2.3.J.3.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28690[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyST23 Build/2.3.J.2.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28691[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; SonyST23i Build/11.0.A.0.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28692[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyST23i Build/11.0.A.1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28693[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-mx; SonyST23a Build/11.0.A.2.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28694[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyST23i Build/2.3.X.2.8010.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28695[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyST23i Build/2.3.J.2.10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28696[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST23i Build/11.0.A.2.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28697[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST23i Build/11.0.A.2.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28698[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST23i Build/11.0.A.2.16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28699[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST23i Build/11.0.A.2.19) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28700[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST23i Build/11.0.A.5.5)";
const char S28701[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST23i Build/11.0.A.5.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28702[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST23i Build/11.0.A.2.16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28703[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-mx; SonyST23a Build/11.0.A.2.16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28704[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-be; SonyST23iv Build/11.0.A.2.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28705[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST23i Build/11.0.A.2.19) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28706[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST23i Build/11.0.A.5.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28707[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyST23a Build/11.0.A.5.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28708[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST23i Build/11.0.A.5.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28709[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST23i Build/11.0.A.5.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28710[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-mx; ST23a Build/11.0.A.2.16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28711[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-be; SonyST23iv Build/11.0.A.5.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28712[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-mx; ST23a Build/11.0.A.2.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28713[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST23a Build/11.0.A.5.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28714[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST23a Build/11.0.A.2.16)";
const char S28715[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-mx; SonyST23a Build/11.0.A.5.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28716[] = "SonyST23i Build/11.0.A.2.10 stagefright/1.2 (Linux;Android 4.0.4)";
const char S28717[] = "SonyST23i Build/11.0.A.5.5 stagefright/1.2 (Linux;Android 4.0.4)";
const char S28718[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9.1";
const char S28719[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S28720[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Safari/528.17";
const char S28721[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S28722[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S28723[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/528.16 (KHTML, like Gecko) Version/4.0 Safari/528.16";
const char S28724[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en) AppleWebKit/526.9 (KHTML, like Gecko) Version/4.0dp1 Safari/526.8";
const char S28725[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19.1";
const char S28726[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S28727[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S28728[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S28729[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; zh-CN) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19.1";
const char S28730[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9.1";
const char S28731[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S28732[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/526.11 (KHTML, like Gecko) Version/4.0dp1 Safari/526.12.2";
const char S28733[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; BOLT/2.340) AppleWebKit/530+ (KHTML, like Gecko) Version/4.0 Safari/530.17  3gpp-gba";
const char S28734[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/531.21.8 (KHTML like Gecko) Version/4.0.4 Safari/531.21.10";
const char S28735[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S28736[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S28737[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/19.77.34.5 Safari/537.1";
const char S28738[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5";
const char S28739[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5";
const char S28740[] = "Mozilla/5.0 (Windows NT 5.2; rv:23.0) Gecko/X Firefox/23.0";
const char S28741[] = "Mozilla/5.0 (Windows NT 5.1; WOW64; rv:23.0) Gecko/X Firefox/23.0";
const char S28742[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 800)";
const char S28743[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 800; Orange)";
const char S28744[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 800; 800 BOUYGUES)";
const char S28745[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 800; Vodafone)";
const char S28746[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 800; SFR)";
const char S28747[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 800; T-Mobile)";
const char S28748[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Nokia 800C)";
const char S28749[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 800; Meteor)";
const char S28750[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Nokia 800)";
const char S28751[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 800; Optimus)";
const char S28752[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 610; 800 BOUYGUES)";
const char S28753[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Nokia; 800C)";
const char S28754[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 800),gzip(gfe)";
const char S28755[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA_blocked; Lumia 800)";
const char S28756[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 800),gzip(gfe) (via docs.google.com/viewer)";
const char S28757[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/11";
const char S28758[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.20 (KHTML, like Gecko) Chrome/11.0.672.2 Safari/534.20";
const char S28759[] = "Mozilla/5.0 (Windows; Windows NT 6.0) AppleWebKit/534.23 (KHTML, like Gecko) Chrome/11.0.686.3 Safari/534.23";
const char S28760[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24";
const char S28761[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.16 Safari/534.24";
const char S28762[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.65 Safari/534.24,gzip(gfe) (via translate.google.com)";
const char S28763[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.24 (KHTML	 like Gecko) Chrome/11.0.696.65 Safari/534.24";
const char S28764[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.24 (KHTML	 like Gecko) Chrome/11.0.696.71 Safari/534.24";
const char S28765[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.20 (KHTML like Gecko) Chrome/11.0.672.2 Safari/534.20";
const char S28766[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.24 (KHTML like Gecko) Chrome/11.0.696.3 Safari/534.24";
const char S28767[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.24 (KHTML like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S28768[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SAMSUNG-SGH-I727 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28769[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SAMSUNG-SGH-I727 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28770[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SAMSUNG-SGH-I727 Build/GINGERBREAD)";
const char S28771[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I727 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28772[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SAMSUNG-SGH-I727 Build/GINGERBREAD)";
const char S28773[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; SAMSUNG-SGH-I727 Build/Sky Rocket Ice Cream Sandwich) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28774[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SAMSUNG-SGH-I727 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28775[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; SAMSUNG-SGH-I727 Build/DJ ICS Plus) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28776[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; SAMSUNG-SGH-I727 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28777[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; SGH-I727 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28778[] = "Android/1.0 (SGH-I727 GINGERBREAD)";
const char S28779[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 2.3.6;dev=SGH-I727;devm=SAMSUNG-SGH-I727;man=samsung;brw=Android;shwd=480x800;shwa=480x763;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S28780[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; Nook Tablet Build/GRK39F)";
const char S28781[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; Nook Tablet Build/GINGERBREAD)";
const char S28782[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK BNTV250 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28783[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; BNTV250 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S28784[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK BNTV250A Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28785[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; BNTV250A Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S28786[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; BNTV250 Build/GINGERBREAD)";
const char S28787[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; BNTV250A Build/GINGERBREAD)";
const char S28788[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK BNTV250A Build/GINGERBREAD 1.4.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S28789[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK BNTV250 Build/GINGERBREAD 1.4.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S28790[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK BNTV250A Build/GINGERBREAD 1.4.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28791[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK BNTV250 Build/GINGERBREAD 1.4.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28792[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.163 Safari/535.19";
const char S28793[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S28794[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S28795[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S28796[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S28797[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S28798[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S28799[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S28800[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S28801[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/180.1.1025.40 Safari/535.19";
const char S28802[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1060) AppleWebKit/535.19 (KHTML like Gecko) Chrome/18.0.1025.142 Safari/535.19";
const char S28803[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/18.0.648.205 Safari/534.16";
const char S28804[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/18.0.648.205 Safari/534.16";
const char S28805[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/18.0.648.205 Safari/534.16";
const char S28806[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/534.1 (KHTML, like Gecko) Chrome/18.0.427.0 Safari/534.1";
const char S28807[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S28808[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; SGH-T859 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S28809[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7510 Build/HONEYCOMB) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S28810[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; GT-P7510 Build/HMJ37)";
const char S28811[] = "Logos/0.9.4 (Linux; U; Android 3.1; en_US; GT-P7510 Build/HMJ37.UEKME)";
const char S28812[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S28813[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7510 Build/HMJ37; 1280*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S28814[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S28815[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7510 Build/HMJ37) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S28816[] = "Logos/0.9.3 (Linux; U; Android 3.1; en_US; GT-P7510 Build/HMJ37.UEKMB)";
const char S28817[] = "Mozilla/5.0 (Linux; U; Android 3.1; en_US; GT-P7510)";
const char S28818[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7510 Build/HMJ37) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S28819[] = "Logos/0.9.4 (Linux; U; Android 3.1; en_US; GT-P7510 Build/HMJ37.ZSKI1)";
const char S28820[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7510 Build/HMJ29) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S28821[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7510 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 sessionId/X-e3a9-4022-aca6-7c84cc7392f7 tiantian(securitypay,securityinstalled) tiantian(tenpay)";
const char S28822[] = "Logos/1.1.3 (Linux; U; Android 3.1; es-US; GT-P7510 Build/HMJ37.UBKH1)";
const char S28823[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.10;FBDM/{density=1.0,width=1280,height=800};FBLC/en_US;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P7510;FBSV/3.1;]";
const char S28824[] = "Mozilla/5.0 (Android 3.1; en-us; GT-P7510 Build/HMJ37) FreeWheelAdManager/5.0.3-r9054-X;com.gotv.crackle.handset/X";
const char S28825[] = "BBC iPlayer (AndroidApp; 1.3; 13014) GT-P7510 (Android 3.1, SDK 12)";
const char S28826[] = "Mozilla/5.0 (Linux; U; Android 3.1; es-es; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.8.4;FBDM/{density=1.0,width=1280,height=800};FBLC/es_ES;FB_FW/1;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P7510;FBSV/3.1;]";
const char S28827[] = "Logos/2.1.2 (Linux; U; Android 3.1; en-GB; GT-P7510 Build/HMJ37.XXKH3)";
const char S28828[] = "Mozilla/5.0(YAdSDK; 2.2.1); (Linux; U; Android 3.1; SAMSUNG GT-P7510 Build/HONEYCOMB_MR1);";
const char S28829[] = "com.google.android.youtube/4.1.23(Linux; U; Android 3.1; en_US; GT-P7510 Build/HMJ37) gzip";
const char S28830[] = "Biblia/2.1.2 (Linux; U; Android 3.1; es-ES; GT-P7510 Build/HMJ37.UBKG6)";
const char S28831[] = "Logos/2.2.3 (Linux; U; Android 3.1; en-US; GT-P7510 Build/HMJ37.UEKF3)";
const char S28832[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 fb4a androidAppVersion/2.5.16 AndroidWebkitWrapper BillingSupported";
const char S28833[] = "BBC News (AndroidApp; 1.1; 9) GT-P7510 (Android 3.1, SDK 12)";
const char S28834[] = "Groupon/2.1.2486 (Android 3.1; Samsung GT-P7510 / Samsung GT-P7510; )[preload=false;locale=en_US;clientidbase=android-samsung]";
const char S28835[] = "MediaPlayer/2002/4.2.0 (Android 3.1; GT-P7510; GT-P7510; samsung; ; ; fr_FR)";
const char S28836[] = "Mozilla/5.0 (Linux; U;Android 3.1; fr-fr; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S28837[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-; GT-P7510 Build/HMJ37) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S28838[] = "BBC News (AndroidApp; 1.0; 5) GT-P7510 (Android 3.1, SDK 12)";
const char S28839[] = "Gospel_Library 2.4.2.7 / Android 3.1 XXKG6 / samsung GT-P7510";
const char S28840[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16";
const char S28841[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7C145 Safari/528.16";
const char S28842[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7D11 Safari/528.16";
const char S28843[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7C144 Safari/528.16";
const char S28844[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7C146 Safari/528.16";
const char S28845[] = "Mozilla/5.0 (iPod; U; CPU OS 3_1_3 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S28846[] = "Mozilla/5.0 (whited00r iPod; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16";
const char S28847[] = "Mozilla/5.0 (whited00r iPod; U; CPU iPhone OS 3_1_3 like Mac OS X; es-es) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16";
const char S28848[] = "Mozilla/5.0 (whited00r iPod; U; CPU iPhone OS 3_1_3 like Mac OS X; en-gb) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16";
const char S28849[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 312 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML like Gecko) Version/4.0 Mobile/7D11 Safari/528.16";
const char S28850[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3 1 2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML like Gecko) Version/4.0 Mobile/7D11 Safari/528.16";
const char S28851[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 311 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML like Gecko) Version/4.0 Mobile/7C145 Safari/528.16";
const char S28852[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 313 like Mac OS X; haw-US) AppleWebKit/528.18 (KHTML like Gecko) Version/4.0 Mobile/7E18 Safari/528.16";
const char S28853[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P3113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S28854[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-P3113 Build/IML74K)";
const char S28855[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S28856[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28857[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P3113 Build/IMM76D)";
const char S28858[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S28859[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P3113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28860[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-P3113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S28861[] = "Logos/1.1.2 (Linux; U; Android 4.0.3; en-US; GT-P3113 Build/IML74K.P3113UEALD3)";
const char S28862[] = "Logos/1.1.2 (Linux; U; Android 4.0.4; en-US; GT-P3113 Build/cm_p3113-userdebug 4.0.4 IMM76L eng.codeworkx.X.X test-keys)";
const char S28863[] = "Logos/1.1.2 (Linux; U; Android 4.0.4; en-US; GT-P3113 Build/cm_espressowifi-userdebug 4.0.4 IMM76I eng.xoomdev.X.X test-keys)";
const char S28864[] = "com.google.android.youtube/3.3.0(Linux; U; Android 4.0.3; en_US; GT-P3113 Build/IML74K) gzip";
const char S28865[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S28866[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28867[] = "Mozilla/5.0 (Android 4.0.3; en-us; GT-P3113 Build/IML74K) FreeWheelAdManager/4.7.0-r8234-X;com.gotv.crackle.handset/X";
const char S28868[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3113 Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S28869[] = "com.google.android.youtube/4.0.23(Linux; U; Android 4.0.4; en_US; GT-P3113 Build/IMM76D) gzip";
const char S28870[] = "Mozilla/5.0 (Android 4.0.4; en-us; GT-P3113 Build/IMM76D) FreeWheelAdManager/5.0.3-r9054-X;com.gotv.crackle.handset/X";
const char S28871[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S28872[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/5.0.2 Version/4.0 Safari/534.30 Safari/6533.18.5";
const char S28873[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3113 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S28874[] = "Logos/2.2.3 (Linux; U; Android 4.0.4; en-US; GT-P3113 Build/IMM76D.P3113UEBLG2)";
const char S28875[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P3113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/1.9.6;FBDM/{density=1.0,width=600,height=976};FBLC/en_US;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P3113;FBSV/4.0.3;]";
const char S28876[] = "Logos/2.2.3 (Linux; U; Android 4.0.4; en-US; GT-P3113 Build/cm_p3113-userdebug 4.0.4 IMM76L eng..X.X test-keys)";
const char S28877[] = "Groupon/2.1.2483 (Android 4.0.4; Samsung Espressowifi / Samsung GT-P3113; )[preload=false;locale=en_US;clientidbase=android-samsung]";
const char S28878[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; GT-P3113 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.7 Mobile Safari/534.30";
const char S28879[] = "BBC News (AndroidApp; 1.3; 4) GT-P3113 (Android 4.0.4, SDK 15)";
const char S28880[] = "Logos/2.2.3 (Linux; U; Android 4.0.3; en-US; GT-P3113 Build/IML74K.P3113UEALCB)";
const char S28881[] = "com.google.android.apps.plus/X (Linux; U; Android 4.0.4; en_US; GT-P3113; Build/IMM76D) (gzip)";
const char S28882[] = "rad.io/1.1.9 (Android 4.0.4) samsung/GT-P3113";
const char S28883[] = "Mozilla/4.0 (Compatible, Aggregation, Twonky, 7.1-beta-X-Android ) TwonkyMedia-NMC DLNADOC/1.50 mobileview TwonkyBeamBrowser/3.3.1-89 (Android 4.0.4; samsung GT-P3113 Build/IMM76D.P3113UEBLH2)";
const char S28884[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76D) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28885[] = "Mozilla/5.0(YAdSDK; 2.2.1); (Linux; U; Android 4.0.4; SAMSUNG GT-P3113 Build/ICE_CREAM_SANDWICH_MR1);";
const char S28886[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S28887[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28888[] = "Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-US; GT-P3113 Build/cm_p3113-userdebug 4.0.4 IMM76L eng.jenkins.X.X test-keys)";
const char S28889[] = "Android/4.6.1 (Linux; U; Android 4.0.4; en_US; GT-P3113)";
const char S28890[] = "nApps (Android 4.0.4; GT-P3113; %EB%84%A4%EC%9D%B4%EB%B2%84%EC%9B%B9%ED%88%B0; 1.2.7)";
const char S28891[] = "Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-US; GT-P3113 Build/IMM76D.P3113UEBLH2)";
const char S28892[] = "Dalvik 2.02.11978; (Linux; U; Android 4.0.4; GT-P3113) f3ba87a5eae70a74; 600x976x1.0";
const char S28893[] = "Groupon/2.2.2561 (Android 4.0.3; Samsung Espressowifi / Samsung GT-P3113; )[preload=false;locale=en_US;clientidbase=android-samsung]";
const char S28894[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; az-az; GT-P3113 Build/IML74K) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S28895[] = "GroupMe-Android/3.3.9 (Android 4.0.3; en_US; samsung GT-P3113 Build/IML74K)";
const char S28896[] = "FaithlifeStudyBible/2.2.10 (Linux; U; Android 4.0.4; en-US; GT-P3113 Build/IMM76D.P3113UEBLG5)";
const char S28897[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P3113Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S28898[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 androidAppVersion/2.5.17 AndroidWebkitWrapper BillingSupported";
const char S28899[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S28900[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en_CA; GT-P3113 BUILD/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S28901[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-ca; GT-P3113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S28902[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P3113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S28903[] = "Archie Comics/1.04 (Android 4.0.4/GT-P3113 [samsung espressowifiue])";
const char S28904[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P3113) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28905[] = "Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0)";
const char S28906[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 5.0)";
const char S28907[] = "Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; NetCaptor 6.5.0RC1)";
const char S28908[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 5.0; Alexa Toolbar)";
const char S28909[] = "Mozilla/5.0 (compatible; MSIE 5.01; Windows NT 5.0)";
const char S28910[] = "Mozilla/3.0 (compatible;MSIE 5.0;  Windows NT 5.0; 360safe)";
const char S28911[] = "Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; SV1)";
const char S28912[] = "Mozilla/4.0 (compatible; MSIE 5.05; Windows NT 5.0)";
const char S28913[] = "Mozilla/6.0 (compatible;MSIE 5.0;  Windows NT 5.0; 360safe)";
const char S28914[] = "EmailsGet (compatible; MSIE 5.01; Windows NT 5.0)";
const char S28915[] = "Mozilla/5.0 (compatible; MSIE 5.0; Windows NT 5.0; Trident/4.0)";
const char S28916[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28917[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28918[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-I9003L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28919[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-I9003 Build/GINGERBREAD)";
const char S28920[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9003 Build/GINGERBREAD)";
const char S28921[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28922[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-I9003 Build/GINGERBREAD)";
const char S28923[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9003 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S28924[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9003 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S28925[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9003 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S28926[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-I9003 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28927[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9003 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S28928[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28929[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28930[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; GT-I9003L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S28931[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S28932[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Acer Iconia A500 Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S28933[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; A500 Build/IML74K)";
const char S28934[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Iconia A500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S28935[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28936[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S28937[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; A500 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S28938[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A500 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S28939[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; A500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S28940[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A500 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28941[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A500 Build/ICS_IMM76I_B230) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S28942[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; A500 Build/IMM76I)";
const char S28943[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.9.168 Version/11.52";
const char S28944[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.9.168 Version/11.52";
const char S28945[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.9.168 Version/11.52";
const char S28946[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.9.168 Version/11.52";
const char S28947[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.9.168 Version/11.52";
const char S28948[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.9.168 Version/11.52";
const char S28949[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.9.168 Version/11.52";
const char S28950[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.9.168 Version/11.52";
const char S28951[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.9.168 Version/11.52";
const char S28952[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.9.168 Version/11.52";
const char S28953[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4871); ru) Presto/2.9.168 Version/11.52";
const char S28954[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4832); ru) Presto/2.9.168 Version/11.52";
const char S28955[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.9.168 Version/11.52";
const char S28956[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4848); ru) Presto/2.9.168 Version/11.52";
const char S28957[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.9.168 Version/11.52";
const char S28958[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.9.168 Version/11.52";
const char S28959[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.9.168 Version/11.52";
const char S28960[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4841); ru) Presto/2.9.168 Version/11.52";
const char S28961[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4110); ru) Presto/2.9.168 Version/11.52";
const char S28962[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.9.168 Version/11.52";
const char S28963[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.9.168 Version/11.52";
const char S28964[] = "Opera/9.80 (Windows NT 6.1; U; Edition Ukraine Local; ru) Presto/2.9.168 Version/11.52,gzip(gfe) (via translate.google.com)";
const char S28965[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03278); ru) Presto/2.9.168 Version/11.52";
const char S28966[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.9.168 Version/11.52";
const char S28967[] = "Opera/9.80 (Windows NT 6.1; U; en) Presto/2.8.131 Version/11.52";
const char S28968[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.9.168 Version/11.52";
const char S28969[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4930); ru) Presto/2.9.168 Version/11.52";
const char S28970[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.9.168 Version/11.52";
const char S28971[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4933); ru) Presto/2.9.168 Version/11.52";
const char S28972[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.9.168 Version/11.52";
const char S28973[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03639); ru) Presto/2.9.168 Version/11.52";
const char S28974[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5265); ru) Presto/2.9.168 Version/11.52";
const char S28975[] = "Mozilla/5.0 (Windows NT 6.1; ru; rv:2.0) Gecko/X Firefox/4.0 Opera 11.52";
const char S28976[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03755); ru) Presto/2.9.168 Version/11.52";
const char S28977[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.9.168 Version/11.52";
const char S28978[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4661); ru) Presto/2.9.168 Version/11.52";
const char S28979[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03402); ru) Presto/2.9.168 Version/11.52";
const char S28980[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; ru) Opera 11.52";
const char S28981[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5218); ru) Presto/2.9.168 Version/11.52";
const char S28982[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5196); ru) Presto/2.9.168 Version/11.52";
const char S28983[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.9.168 Version/11.52";
const char S28984[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.9.168 Version/11.52";
const char S28985[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.9.168 Version/11.52";
const char S28986[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.9.168 Version/11.52";
const char S28987[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.9.168 Version/11.52";
const char S28988[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.9.168 Version/11.52";
const char S28989[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03780); ru) Presto/2.9.168 Version/11.52";
const char S28990[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4947); ru) Presto/2.9.168 Version/11.52";
const char S28991[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5328); ru) Presto/2.9.168 Version/11.52";
const char S28992[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5267); ru) Presto/2.9.168 Version/11.52";
const char S28993[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.9.168 Version/11.52";
const char S28994[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.9.168 Version/11.52";
const char S28995[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.9.168 Version/11.52,gzip(gfe)";
const char S28996[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.9.168 Version/11.52";
const char S28997[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.9.168 Version/11.52";
const char S28998[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.9.168 Version/11.52";
const char S28999[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.9.168 Version/11.52";
const char S29000[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.9.168 Version/11.52";
const char S29001[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6060); ru) Presto/2.9.168 Version/11.52";
const char S29002[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5308); ru) Presto/2.9.168 Version/11.52";
const char S29003[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5966); ru) Presto/2.9.168 Version/11.52";
const char S29004[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6578); ru) Presto/2.9.168 Version/11.52";
const char S29005[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6089); ru) Presto/2.9.168 Version/11.52";
const char S29006[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:13.0) Gecko/X Firefox/13.0";
const char S29007[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:13.0) Gecko/X Firefox/13.0.1";
const char S29008[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:13.0) Gecko/X Firefox/13.0.1 Firefox/13.0.1";
const char S29009[] = "Mozilla/5.0 (Windows; Windows NT 6.1; rv:13.0) Gecko/X Firefox/13.0.1";
const char S29010[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:13.0) Gecko/X Firefox/13.0 Firefox/13.0";
const char S29011[] = "Mozilla/5.0 (Windows; Windows NT 6.1; Win64; x64; rv:13.0) Gecko/X Firefox/13.0.1";
const char S29012[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B5105c Twitter for iPhone";
const char S29013[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B5117b Twitter for iPhone";
const char S29014[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B5126b Twitter for iPhone";
const char S29015[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Twitter for iPhone";
const char S29016[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Twitter for iPhone";
const char S29017[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Twitter for iPhone";
const char S29018[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 Twitter for iPhone";
const char S29019[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145 Twitter for iPhone";
const char S29020[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Twitter for iPhone";
const char S29021[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Twitter for iPhone";
const char S29022[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B318 Twitter for iPhone";
const char S29023[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Twitter for iPhone";
const char S29024[] = "Opera/9.80 (Windows NT 6.1; U; Edition Next; ru) Presto/2.11.310 Version/12.50";
const char S29025[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N8013 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S29026[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N8013 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S29027[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N8013 Build/JZO54K)";
const char S29028[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N8013 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29029[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-N8013 Build/JRO03C)";
const char S29030[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-N8013 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S29031[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; T- Mobile G2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29032[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; ZTE-C_N606/N606_QQV1.0.0B01; 240*320; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29033[] = "Mozilla/5.0 (Linux; U; Android 2.2; pt-pt; V9c Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29034[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB611Build/BGN_1.1.30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29035[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; asadullah Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29036[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-T839Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29037[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; ZTE-C_N606/N606QQ_OV1.0.0B01; 240*320; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29038[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; th-th; WellcoM-A89 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29039[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A32 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29040[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; FIH-F0X Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29041[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; UPC300-2.2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29042[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A28 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29043[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; MoDaCo Custom ROM for the Advent Vega r8 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29044[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MSM Build/FRX07) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29045[] = "Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; TOSHIBA_FOLIO_AND_A Build/TOSHIBA_FOLIO_AND_A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29046[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; nonsensikal froyo Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29047[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Vanilla Tazz Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29048[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; meizu_m9 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29049[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; VEGAn-TAB-v1.0.0b5.1.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29050[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; A7-040 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29051[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; wwe10 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29052[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; CSL-MI410 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29053[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; TC970 (Wi-Fi) Build/s3394) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29054[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAWEE10 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29055[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; P041 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29056[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; cius Build/9.2.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29057[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; cius Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29058[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; DroidBuild/FRK76) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29059[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; NABI-A Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29060[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-es; ADR-2100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29061[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; EBEST W16 Build/BQ3_W16_HW1.1_V004) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29062[] = "ua:Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; America Build/W800_V01.05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29063[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; FrankenTAB Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29064[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad_W706+/Coolpad_W706+_V4.60; 320*480) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S29065[] = "Mozilla/5.0 (Linux; U; Android 2.2; nl-nl; sec_smdkv210 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29066[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; IDEOS Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29067[] = "Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; Hikari-iFrame/WDPF-701ME Build/2.0.0E) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29068[] = "Mozilla/5.0 (Linux; U; Android 2.2; pt-br; NB002 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29069[] = "Mozilla/5.0 (Linux; U; Android 2.2; nl-nl; DMEDIA_ATM3470 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29070[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; CometBuild/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29071[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; CRICKET DROID-X Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29072[] = "Mozilla/5.0 (Linux; U; Edge ERMINE-0.9; Android 2.2.1; en-us; edge Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29073[] = "Mozilla/5.0 (Linux; U; Android 2.2; de-de; ODYS Xtreme Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29074[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DroniX Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29075[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; TPC800W Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29076[] = "QuickPlayer2.0 CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 2.2.2)";
const char S29077[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; MID7020 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29078[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; MID8020 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29079[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MID7008 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29080[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; UPC300-2.2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29081[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; X2 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29082[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; hd-us; generic_x86 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29083[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; JellyBean Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29084[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; vi-vn; simcom16_bu1_a10y Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29085[] = "Mozilla/5.0 (Linux; U; Android 2.2; tr-tr; EASYPAD Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29086[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; apanda_A60 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29087[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MD70 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29088[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; maple Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29089[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Mi320 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29090[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; CSL-MI410 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29091[] = "Mozilla/5.0 (Linux; U; Android 2.2-update1; en-us; Next CEWS10 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S29092[] = "Mozilla/5.0 (Linux; U; Android 2.2; pt-br; FS-M74TG Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S29093[] = "Mozilla/5.0 (Linux; U; Android 2.2; pl-pl; NaviTablet Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S29094[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; it-it; Andy Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29095[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; lepad_001n Build/PQXU100.4.0073.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29096[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; MID-71 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29097[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MID-2 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29098[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; Android-for-Rockchip-2818 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29099[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; NEXT NXTTAB10BL Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29100[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; BEBOOK070I00 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29101[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ru-ru; Beeline M2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29102[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; id-id; NX-A891 Build/FRG83G)         AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S29103[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; md-us; H900 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29104[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; NX-A891 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29105[] = "Mozilla/5.0 (Linux; U; Android 2.2; nl-nl; DMEDIA ATM3470 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29106[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; lynx Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29107[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SP01 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29108[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-; SF4Y6 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29109[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; P0106RG Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29110[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; M799 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29111[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; Lenovo 3GC101/S-5-02;320*533;CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S29112[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ASP320Q_ANDi Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29113[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; tablet_p2 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29114[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; in-id; NX-A712 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29115[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; one touch 906 Build/FSR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29116[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; VEGAn-TAB-v1.0.0B5.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29117[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; i7 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29118[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; C8500S Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S29119[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; FMT-NT7A22-01 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29120[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; A7-004 Build/QAJ3000.3.0018) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29121[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; vm Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29122[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; xam5001 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29123[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; CIUS-7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29124[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; TEL7J, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29125[] = "Mozilla/5.0 (Linux; U; Android 2.2; nl-nl; QWTB9010 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29126[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; in-id; CROSS-A1 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29127[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; P4D SIRIUS Build/MID) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29128[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; NBPC724 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29129[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; YL-Coolpad_E239/1.01.019.X.E239; 320*480; CTC/2.0) CoolpadWebkit/533.1";
const char S29130[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Livall Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29131[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; nl-nl; Los Angeles Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29132[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; YENO100, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29133[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; OP070 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29134[] = "Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; venus Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29135[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Mi-A300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29136[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; AND01 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29137[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; WM8650 Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29138[] = "Mozilla/5.0 (Linux; U; Android 2.2; ko-kr; ILT-MX100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29139[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; TOSHIBA_AC_AND_AZ Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29140[] = "Mozilla/5.0 (Linux; U; Android 2.2; pt-pt; Slidepad Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29141[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; MFC150ES, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29142[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; TO101 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29143[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; P801W Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29144[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; GTablet-TnT-Lite Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29145[] = "Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; Hikari-iFrame/WDPF-703TI Build/1.04.00) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29146[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-us; EF101 Build/Kaissen_I7003G_BOE_OV2_TV7.2_X_WCDMA) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29147[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MFC150GB_02, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29148[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; SLIDEPAD Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29149[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-C_N880S/N880SV1.0.0B03; 480*800; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29150[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; MID, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29151[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; Lenovo 3GC101/S-5-03;320*533;CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S29152[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; X2G Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29153[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; V9e Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29154[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; in-id; AD350 Build/ATHENA_00.03.68.10.45.H1007) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29155[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; ZTE V880 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29156[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; tr-tr; A2000 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29157[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; IVIO_DE38 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29158[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Z999 Build/I700T_P7_T13_TSZY_FT_S42_R_0_01_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29159[] = "Mozilla/5.0 (Linux; U; Android 2.2; de-de; NB_10_HD Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29160[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ZiiLABS ZiiO Shenbi  Build/v2.01.01-FROYO) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile Safari/528.16";
const char S29161[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ZiiLABS ViewBook 730  Build/v1.00.08-FROYO) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile Safari/528.16";
const char S29162[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; ST1001 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29163[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; SP-60 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29164[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; Movistar Prime Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29165[] = "Mozilla/5.0 (Linux; U;Android 2.2.1; zh-cn; YL-Coolpad_5855/1.02.001.X.5855; 480*320; CTC/2.0) CoolpadWebkit/533.1";
const char S29166[] = "Mozilla/5.0 (Linux; U; Android 2.2; de-de; U0101HA Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29167[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; lepad_001n Build/PQXU100.4.0091.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29168[] = "Mozilla/5.0 (Linux; U; Android 2.2; nl-nl; TAB-TEGRA-10-1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29169[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; id-id; SpeedUp S1 Build/SpeedUp_S1_Ver01_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29170[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; Lenovo Lenovo S1-37AH0 Build/FRG83) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S29171[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; CIUS-7-AT Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29172[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; KM7J2, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29173[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; md-us; avlite Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29174[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; WellcoM-A99 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29175[] = "Mozilla/5.0 (Linux; U; Android 2.2; ld-us; AD1000 Build/H6.1-S1.6.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29176[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; YL-Coolpad_N930/1.01.012.X.N930; 480*800; CTC/2.0) CoolpadWebkit/533.1";
const char S29177[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; FIH-FM6 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29178[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; tr-tr; V9-MUL_F680 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29179[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; A602 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29180[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SH72x8U Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29181[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A55 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29182[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-; sec_smdkv210 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29183[] = "Mozilla/5.0 (Linux; U; Android 2.2; pt-br; FS-M73T Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S29184[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-es; AIRIS Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29185[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-jp; S31HW Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29186[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; SP-40 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29187[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; COSMO DUO Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29188[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; i7 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29189[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; SF4Y6 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29190[] = "Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; MoFing_701 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29191[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; DroniX Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29192[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; G3 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29193[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; AM728 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29194[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; el-gr; Carl Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29195[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; M718G Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29196[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; m87vg Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29197[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; A602 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29198[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Dk1031 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29199[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; L601 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29200[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; I-L1 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29201[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; s710i Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29202[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; M778R Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29203[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; A611 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29204[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-id; EVO_Tab Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29205[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; A607 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29206[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; nl-nl; TR718D, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29207[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; D000-X-S01 Build/00.20.32-02.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29208[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; V-T200 Build/I700T_P6_T08_FLOL_FT_S42_R_0_05_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29209[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; lepad_001n Build/PQXU100.4.0097.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29210[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; SWTGT-7250 Build/froyo_almond_r245) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29211[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; md-us; DW-UBT7+.A Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29212[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MTAG7 Build/V.MTS.MTAG7.B09) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29213[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; A612 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29214[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad_W706+/Coolpad_W706+_V4.70; 320*480) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S29215[] = "Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; Hikari-iFrame/WDPF-703TI Build/2.02.00) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29216[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-es; Tairu Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29217[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; A615 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29218[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ro-ro; Vancouver_Orange Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29219[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; 4.3 Internet Tablet Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29220[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Chromatic Dream Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29221[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; GW620 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29222[] = "Mozilla/5.0 (Linux; U; Android 2.2; de-de; smartpad Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29223[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; firetrap TwoNights2.2 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29224[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; ME811 Build/SHDWR_X6_2.200.25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29225[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; TnT G-Harmony Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29226[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; imx51_bbg Build/FRF85B)";
const char S29227[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; tr-tr; I7500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29228[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; TR718D Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29229[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; Geeksphone ZERO Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29230[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Prince-I688 Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29231[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SH8158U Build/S3181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29232[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; Corvus5 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29233[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; SAWEE10 Build/FRF85B)";
const char S29234[] = "Mozilla/5.0 (Linux; U; Android 2.2; it-it; 4Geek Horus Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29235[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Corvus5 SP1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29236[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Android TV Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29237[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-es; MID-10 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29238[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; earlgrey Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29239[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Onida_i9 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29240[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME811 Build/SHDWR_X6_2.200.33) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29241[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pt-pt; u8220 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29242[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; VX780A Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29243[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; TCL A906 Build/FSR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29244[] = "Mozilla/5.0 (Linux; U; Android 2.2; ms-my; Mi320 Build/FRF91) AppleWebKit/533.1 (KHTML, sperti Gecko) Versi/4.0 Mobil Safari/533.1";
const char S29245[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Zoompad Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29246[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ro-ro; ALLVIEW_P1 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29247[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; T720 SE, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29248[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; th-th; G8 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29249[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; FIH-F0X Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29250[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; tr-tr; ST070I00 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29251[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; i7 Build/FRF85B)";
const char S29252[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ru-ru; MTC 950 Build/HuaweiMTC_950) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29253[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; M721 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29254[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; MTC 950 Build/HuaweiMTC_950) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29255[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; YL-Coolpad_9930/1.02.117.X.9930; 960*480; CTC/2.0) CoolpadWebkit/533.1";
const char S29256[] = "Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; LT-NA7F Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29257[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; tr-tr; Polypad C7000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29258[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; 2GO Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29259[] = "CoolPad8809_CMCC_TD/1.0 OPhone/2.5 (Linux; Android 2.2) Release/04.28.2011 Browser/WAP2.0 (AppleWebKit/533.1) Mobile Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S29260[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pt-pt; TB-002 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29261[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ru-by; MTC Mini Build/HuaweiMTC_Mini) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29262[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; id-id; MOVI M1 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29263[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; i808 Build/ATHENA_00.03.66.03.07.H1001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29264[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; ME811 Build/SHDWR_X6_2.200.25) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S29265[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; saturn Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29266[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; E239 Build/MASTER) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S29267[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-C N606 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S29268[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-us; agave Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29269[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; in-id; AD350 Build/ATHENA_00.03.68.10.40.H1007) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29270[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; M701, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29271[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; CAP6_US Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29272[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; SN10T2 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29273[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; M31 Build/1.0.4506.0251) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29274[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; TY-K-Touch_E800/X_7950_V0713; 480*800; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29275[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; BL-R78242-EW Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29276[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; 5855 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29277[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; R81P4 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29278[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; A35 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29279[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; VX580A Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29280[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad W706+ Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S29281[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ZeusPAD ZP-712 Build/I700T_P7_T11_SPWE_CY_R_0_04_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29282[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; CFITD, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29283[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; TCL A990 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29284[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; A7-004 Build/QAJ3000.3.0017) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29285[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; DNP-725Lucky Build/1.0.5478.0317) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29286[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; meizu_m9 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S29287[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; U8S_MID, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29288[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; in-id; F698-DG30M Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29289[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; Entel-N700 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29290[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; in-id; Twilight_7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29291[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; in-id; Mito 9800 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29292[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MF8604 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29293[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; G.fone G1315 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29294[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; TR718DC, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29295[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Ritroid T1 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29296[] = "Mozilla/5.0 (Linux; U; Android 2.2; nl-nl; TAB410 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29297[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; id-id; AndroTab Build/1.0.5478.0317)AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29298[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-es; PC7000 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29299[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; Lenovo S1-37AH0 Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S29300[] = "Mozilla/5.0 (Linux; U; Android 2.2_v1.5.3; hd-gb; WM8650 Build/FRG83D_v1.5.3 Android Froyo 2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29301[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-es; Qpad Q760 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29302[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; HWA505 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29303[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Galaxy Pad Build/froyo_almond) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29304[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; public Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29305[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; A603 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29306[] = "ua:Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; AX_W_RTP Build/AX_W_VER13_P3_RTP_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29307[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; 3GC101 Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S29308[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; MID-71, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29309[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; rk2818 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29310[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME811 Build/SHDWR_X6_2.200.33) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S29311[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SH8268U Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29312[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; LEXIBOOK, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29313[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; MUSN-M500 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29314[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; CTG701 Build/I700T_P7_T04_TSCL_FT_S42_R_0_01_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29315[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MBI i7 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29316[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GN200 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29317[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad_W706/Coolpad_W706_V2.41; 320*480;CTC/2.0) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S29318[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; OIDZ Glee Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29319[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Precision Tablet ERP-A1-38 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29320[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ZiiLABS ZiiO WaWaYaYa  Build/v2.00.12-Froyo) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile Safari/528.16";
const char S29321[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; MI5 Build/JOYUI-3G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29322[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; M721, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29323[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad W706+ Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29324[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad_W706/Coolpad_W706_V3.10; 320*480;CTC/2.0) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S29325[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; C8500S Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S29326[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; D000-X-S25 Build/00.21.29-01.23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29327[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MI355 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29328[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; YL-Coolpad_9100/1.02.009.X.9100; 800*480; CTC/2.0) CoolpadWebkit/533.1";
const char S29329[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-ve; C8512 Build/HuaweiC8512) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29330[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; T92 Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S29331[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; QB57 Build/QB57L25.YC1.00.B25.54.03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29332[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; tikx2 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29333[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; M302 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29334[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; 3GC101 Build/FRG83) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S29335[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-es; Vexia 7600 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29336[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Galaxy Pad Build/froyo_almond_r232) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29337[] = "Mozilla/5.0 (Linux; U; Android 2.2; ko-kr; ESP_E201V7 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29338[] = "Mozilla/5.0 (Linux; U; Android 2.2; pt-br; AMOSTRA Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29339[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; A7-004 Build/QAJ3000.3.0022) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29340[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad W706+ Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S29341[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; Wizlet Rise T-308 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29342[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; W606 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29343[] = "Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; Hikari-iFrame/WDPF-703TI Build/2.03.00) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29344[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; Fly_IQ250; Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29345[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; NBPC824 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29346[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MT620 Build/THTTD_N_01.01.36I) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S29347[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; YL-Coolpad_5899/2.2.007.X.5899; 480*320; CTC/2.0) CoolpadWebkit/533.2";
const char S29348[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; T92 Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S29349[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad_W706/Coolpad_W706+_V2.00; 320*480) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S29350[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; pnx6715_e150b Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S29351[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; AX 8.6 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29352[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-us; EF101 Build/Kaissen_I7003G_BOE_OV2_TV7.6_X_WCDMA) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29353[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; NBPC724A Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29354[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-es; m77vg Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29355[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; SM07 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29356[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; TEL7JB, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29357[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HW-HUAWEI_S8520/S8520V100R001C92B011; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S29358[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Fusion Tab Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29359[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-co; ADR1105 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29360[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ru-ru; Explay A320 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29361[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; WM8650-mid Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29362[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; M4301 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29363[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; in-id; A850 Build/SDB-P2_1.3.20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29364[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; NBPC824A Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29365[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; A35DM Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29366[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; FFS070 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29367[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MT620 Build/THTTD_N_01.01.36I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S29368[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; in-id; MG-251 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29369[] = "Mozilla/5.0 (Linux; U; Android 2.2; tr-tr; WM8650-pid Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29370[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; MFC150DEX, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29371[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; A2000 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29372[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Mi280 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29373[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Z100 Build/SDB-P2_1.3.20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29374[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Mypad Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29375[] = "Coolpad8050_CMCC_TD/1.0 Linux/2.6.35 Android/2.3.5 Release/7.16.2012 Mozilla/5.0 (Linux; U; Android 2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29376[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; CTG701 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29377[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; GFIVE A57 Build/S21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29378[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; IB-Q6 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29379[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A70H2 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29380[] = "Mozilla/5.0 (Linux; U; Android 2.2-X; ru-ru; Inch OS Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29381[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; in-id; ZTE-U V859 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29382[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; A60 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29383[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; V871 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29384[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; NIU TEK N109 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29385[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-; maple Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29386[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Lynx Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29387[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; A605 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29388[] = "Mozilla/5.0 (Linux; U; Android 2.2; vi-vn; LY-F528 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29389[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Lemon P1 Build/1.0.6008.0254) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29390[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; Lenovo S1-37AH0 Build/FRG83) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S29391[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; yap01 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29392[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; BizSmartPhone Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29393[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; Energy 63 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29394[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; fa-ir; GLX Mobile Build/A2-V1.0-2012.04.26) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29395[] = "Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; ZP02 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29396[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; vi-vn; ALCATEL_one_touch_927 Build/FSR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29397[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; EX 3mini Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29398[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; EVOTab Build/FRG83G) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29399[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; EVO Tab Build/FRG83G) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29400[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Aedupac Xtab Light 100 Build/froyoalmondr238) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29401[] = "QuickPlayer2.0 CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 2.2)";
const char S29402[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-in; Sansui_A11 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29403[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; V8502 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29404[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; T7033 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29405[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; yezz_Andy Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29406[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad_W706+/Coolpad_W706+_V5.50; 320*480) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S29407[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; S700 Build/B1326) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29408[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HelloTab Build/1.0.4992.0138) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29409[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; AndroTab Build/1.0.4383.0267)AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29410[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HW-HUAWEI_C8550/C8550V100R001C92B665; 240*320; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S29411[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-C_N880S/N880SV1.0.0B04; 480*800; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29412[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Magnum 2X Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29413[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-id; EVO_7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29414[] = "Mozilla/5.0 (Linux; U; Android 2.2; ru-ru; Inch_os Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29415[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; th-th; G.net A6 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29416[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; TC970 (Wi-Fi) Build/4749)";
const char S29417[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; epade A300 Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29418[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad W706 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29419[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGT-SMA2 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29420[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; TIK Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29421[] = "ua:Mozilla/5.0 (Linux; U; Android 2.2.2; tr-tr; Neoi-585 Build/Neoi-585_V01.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29422[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-ph; ICE+ Build/CloudfoneICE+) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29423[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; one touch 906 Build/FSR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29424[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; real3d Build/FRG22D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29425[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; in-id; A6T Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29426[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; ahong16_a10y Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29427[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; Full Android on Sapphire Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29428[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; in-id; ZTE-U V859 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29429[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; AM728 3G Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29430[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; xpndr_imh Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29431[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME811 Build/SHDWR_X6_2.200.33) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S29432[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; zh-cn; HW-HUAWEI_S8520/S8520V100R001C92B011; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S29433[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SYTAB10ST-DS Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29434[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; aura 7.0 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29435[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; X2 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/";
const char S29436[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; Coolpad W706+ Build/FRG83G)";
const char S29437[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; CROSS-A1 Build/MASTER)";
const char S29438[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; GS30 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29439[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; D000-X-S01 Build/00.21.29-02.20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29440[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; rlg L8 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29441[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Z100 Build/SDB-P1_1.3.28) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29442[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HomeChoiceE360 Alpha Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29443[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; msm7227_ffa Build/I700T_P6_T12_FLMO_FT_S42_R_0_02_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29444[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; it-it; MID701, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29445[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; Vodafone 858 Build/DeBranded_r0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29446[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; P1-S Build/1.0.6008.0254) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29447[] = "Mozilla/5.0 (Windows NT 5.1; rv:14.0) Gecko/X Firefox/14.0";
const char S29448[] = "Mozilla/5.0 (Windows NT 5.2; rv:14.0) Gecko/X Firefox/14.0";
const char S29449[] = "Mozilla/5.0 (Windows NT 5.1; rv:14.0) Gecko/X Firefox/14.0.1 Firefox/14.0.1";
const char S29450[] = "Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:14.0) Gecko/X Firefox/14.0.1";
const char S29451[] = "Mozilla/5.0 (Windows; Windows NT 5.1; rv:14.0) Gecko/X Firefox/14.0.1";
const char S29452[] = "Mozilla/5.0 (Windows NT 5.1) Gecko/X Firefox/14.0 Opera/12.0";
const char S29453[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.43.7 (KHTML, like Gecko) Version/7.0 Safari/537.43.7";
const char S29454[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.44 (KHTML, like Gecko) Version/7.0 Safari/537.44";
const char S29455[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.45 (KHTML, like Gecko) Version/7.0 Safari/537.45";
const char S29456[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.46.5 (KHTML, like Gecko) Version/7.0 Safari/537.46.5";
const char S29457[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.47 (KHTML, like Gecko) Version/7.0 Safari/537.47";
const char S29458[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.48.3 (KHTML, like Gecko) Version/7.0 Safari/537.48.3";
const char S29459[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.50 (KHTML, like Gecko) Version/7.0 Safari/537.50";
const char S29460[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.51.3 (KHTML, like Gecko) Version/7.0 Safari/537.51.3";
const char S29461[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.52 (KHTML, like Gecko) Version/7.0 Safari/537.52";
const char S29462[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.53 (KHTML, like Gecko) Version/7.0 Safari/537.53";
const char S29463[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.58 (KHTML, like Gecko) Version/7.0 Safari/537.58";
const char S29464[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.56 (KHTML, like Gecko) Version/7.0 Safari/537.56";
const char S29465[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.59 (KHTML, like Gecko) Version/7.0 Safari/537.59";
const char S29466[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.54.1 (KHTML, like Gecko) Version/7.0 Safari/537.54.1";
const char S29467[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.55 (KHTML, like Gecko) Version/7.0 Safari/537.55";
const char S29468[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/538.1 (KHTML, like Gecko) Version/7.0 Safari/538.1";
const char S29469[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.63 (KHTML, like Gecko) Version/7.0 Safari/537.63";
const char S29470[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.64 (KHTML, like Gecko) Version/7.0 Safari/537.64";
const char S29471[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.65 (KHTML, like Gecko) Version/7.0 Safari/537.65";
const char S29472[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.61 (KHTML, like Gecko) Version/7.0 Safari/537.61";
const char S29473[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.66 (KHTML, like Gecko) Version/7.0 Safari/537.66";
const char S29474[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.68 (KHTML, like Gecko) Version/7.0 Safari/537.68";
const char S29475[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.71 (KHTML, like Gecko) Version/7.0 Safari/537.71";
const char S29476[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/18.6.872.0 Safari/535.8";
const char S29477[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/18.6.872.0 Safari/535.8";
const char S29478[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.45 Safari/535.19";
const char S29479[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19,gzip(gfe) (via translate.google.com)";
const char S29480[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19";
const char S29481[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19,gzip(gfe) (via translate.google.com)";
const char S29482[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Client/0; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S29483[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.19 (KHTML,             like Gecko) Chrome/18.0.1025.45 Safari/535.19";
const char S29484[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam GameOverlay/1960; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S29485[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam GameOverlay/2005; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S29486[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam GameOverlay/2009; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S29487[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam GameOverlay/2026; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S29488[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; de) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/18.0 Safari/525.13";
const char S29489[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam GameOverlay/2095; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S29490[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam GameOverlay/2109; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S29491[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam GameOverlay/2102; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S29492[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam GameOverlay/2125; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S29493[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam GameOverlay/X; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S29494[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/18.0 BIDUPlayerBrowser/2.x Safari/534.7";
const char S29495[] = "Mozilla/5.0 (Windows NT 6.0; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/18.0 BIDUPlayerBrowser/2.x Safari/534.7";
const char S29496[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.19 (KHTML,         like Gecko) Chrome/18.0.1025.45 Safari/535.19";
const char S29497[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam Client/X; ) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Safari/535.19";
const char S29498[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:15.0) Gecko/X Firefox/15.0";
const char S29499[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:15.0) Gecko/X Firefox/15.0.1";
const char S29500[] = "Mozilla/5.0 (Windows NT 6.0; rv:15.0) Gecko/X Firefox/15.0.1 Firefox/15.0.1";
const char S29501[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; rv:15.0) Gecko/X Firefox/15.0.2";
const char S29502[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.0) LinkCheck by Siteimprove.com";
const char S29503[] = "Mozilla/4.79 [en] (compatible; MSIE 7.0; Windows NT 5.0; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 1.1.4322; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648)";
const char S29504[] = "Mozilla/4.0 (compatible; MSIE 7.01; Windows NT 5.0)";
const char S29505[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 5.0; Trident/4.0; FBSMTWB; .NET CLR 2.0.34861; .NET CLR 3.0.3746.3218; .NET CLR 3.5.33652; msn OptimizedIE8;ENUS)";
const char S29506[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.0; SV1)";
const char S29507[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 5.0)";
const char S29508[] = "Mozilla/6.0 (compatible;MSIE 7.0;  Windows NT 5.0; QQDownload)";
const char S29509[] = "Mozilla/3.0 (compatible;MSIE 7.0;  Windows NT 5.0; LOVEYOU)";
const char S29510[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 5.0; SV1)";
const char S29511[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1))";
const char S29512[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)";
const char S29513[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.0; SEOlyzer 1.0 by SOFTPAE.com)";
const char S29514[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; FDM)";
const char S29515[] = "Mozilla/5.0 (compatible; MSIE 7.01; Windows NT 5.0)";
const char S29516[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S29517[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S29518[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P6200 Build/HTJ85B)";
const char S29519[] = "Mozilla/5.0 (Linux; U; Android 3.2; pt-br; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S29520[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P6200L Build/HTJ85B)";
const char S29521[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B; 1024*552) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S29522[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S29523[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S29524[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S29525[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200L Build/HTJ85B; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S29526[] = "Mozilla 5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build HTJ85B) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S29527[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6200) antivirusAM/2.4.5 (KHTML, like Gecko) Version/Version 2.4.5/antivirus";
const char S29528[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6200 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29529[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.3;FBDM/{density=1.0,width=600,height=976};FBLC/en_GB;FB_FW/2;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P6200;FBSV/3.2;]";
const char S29530[] = "BBC News (AndroidApp; 1.3; 4) GT-P6200 (Android 3.2, SDK 13)";
const char S29531[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S29532[] = "Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 3.2; SAMSUNG GT-P6200 Build/HONEYCOMB_MR2);";
const char S29533[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200L Build/HTJ85B) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S29534[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S29535[] = "Mozilla/5.0 (Linux; U; Android 3.2; pt-br; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.10;FBDM/{density=1.0,width=600,height=976};FBLC/pt_BR;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P6200L;FBSV/3.2;]";
const char S29536[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S29537[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29538[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29539[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-ph; GT-P6200 B3 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S29540[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6200L Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29541[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.11;FBDM/{density=1.0,width=600,height=976};FBLC/en_GB;FBCR/Smart;FBPN/com.facebook.katana;FBDV/GT-P6200;FBSV/3.2;]";
const char S29542[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 ZyngaBundleIdentifier/com.zynga.zjayakashi ZyngaBundleVersion/1.7.0";
const char S29543[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B like Xoom) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S29544[] = "MediaPlayer/2002/4.0.12 (Android 3.2; GT-P6200; GT-P6200; samsung; ; ; en_US)";
const char S29545[] = "Logos/2.2.3 (Linux; U; Android 3.2; en-US; GT-P6200 Build/HTJ85B.XXKL7)";
const char S29546[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S29547[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) Expedia/2.0.1";
const char S29548[] = "Mozilla/5.0 (Linux; U; Android 3.2; ms-my; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.10;FBDM/{density=1.0,width=1024,height=552};FBLC/ms_MY;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P6200;FBSV/3.2;]";
const char S29549[] = "Mozilla/5.0 (Linux; U; Android 3.2; ms-my; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.11;FBDM/{density=1.0,width=1024,height=552};FBLC/ms_MY;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P6200;FBSV/3.2;]";
const char S29550[] = "MediaPlayer/2002/4.0.12 (Android 3.2; GT-P6200; GT-P6200; samsung; ; ; en_GB)";
const char S29551[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-ph; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.12;FBDM/{density=1.0,width=600,height=976};FBLC/en_PH;FBCR/SMART;FBPN/com.facebook.katana;FBDV/GT-P6200;FBSV/3.2;]";
const char S29552[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.8.3;FBDM/{density=1.0,width=1024,height=552};FBLC/en_US;FB_FW/1;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P6200;FBSV/3.2;]";
const char S29553[] = "Logos/2.2.8.1 (Linux; U; Android 3.2; en-GB; GT-P6200 Build/HTJ85B.XXKK6)";
const char S29554[] = "Groupon/2.1.2486 (Android 3.2; Samsung GT-P6200 / Samsung GT-P6200; MAXIS)[preload=false;locale=en_US;clientidbase=android-samsung]";
const char S29555[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-ph; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 androidAppVersion/2.5.16 AndroidWebkitWrapper BillingSupported";
const char S29556[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 ZyngaBundleIdentifier/com.zynga.zjayakashi ZyngaBundleVersion/1.8.0";
const char S29557[] = "Logos/2.2.10 (Linux; U; Android 3.2; es-ES; GT-P6200L Build/HTJ85B.UBLA1)";
const char S29558[] = "com.google.android.youtube/4.0.23(Linux; U; Android 3.2; en_US; GT-P6200L Build/HTJ85B) gzip";
const char S29559[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like GeckAccept-Encoding: gzip,deflate";
const char S29560[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 ZyngaBundleIdentifier/com.zynga.zjayakashi ZyngaBundleVersion/1.9.0";
const char S29561[] = "Scanner Radio/3.9.1.1 (Android 3.2, GT-P6200)";
const char S29562[] = "nApps (Android 3.2; GT-P6200; androidapp.lineplay; 1.2.1)";
const char S29563[] = "Mozilla/5.0 (Linux; U; Android 3.2; se-se; GT-P6200L Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29564[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.85 Safari/537.36";
const char S29565[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S29566[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S29567[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S29568[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S29569[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-I535 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S29570[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999V Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S29571[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S29572[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S29573[] = "Mozilla/5.0 (Linux; Android 4.1.9; GT-I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S29574[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S29575[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S29576[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-i9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S29577[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I93000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S29578[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I939 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S29579[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.74 Safari/537.36";
const char S29580[] = "Mozilla/5.0 (Linux; Android 4.1.2; SC-06D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S29581[] = "Mozilla/5.0 (Linux; Android 4.1.1; i9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S29582[] = "Mozilla/5.0 (Linux; Android 4.1.9; I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S29583[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S29584[] = "Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:15.0) Gecko/X Firefox/15.0.1";
const char S29585[] = "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:15.0) Gecko/X Firefox/15.0.1 Firefox/15.0.1";
const char S29586[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.229 Version/11.61";
const char S29587[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.61";
const char S29588[] = "Opera/9.80 (Windows NT 5.2; U; en) Presto/2.10.229 Version/11.61";
const char S29589[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.229 Version/11.61,gzip(gfe) (via translate.google.com)";
const char S29590[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.229 Version/11.61";
const char S29591[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.229 Version/11.61";
const char S29592[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.229 Version/11.61";
const char S29593[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.229 Version/11.61";
const char S29594[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.229 Version/11.61";
const char S29595[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.10.229 Version/11.61";
const char S29596[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.229 Version/11.61";
const char S29597[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; en) Opera 11.61";
const char S29598[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.229 Version/11.61";
const char S29599[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.229 Version/11.61";
const char S29600[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.61";
const char S29601[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03780); ru) Presto/2.10.229 Version/11.61";
const char S29602[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.229 Version/11.61";
const char S29603[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.10.229 Version/11.61";
const char S29604[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5161); ru) Presto/2.10.229 Version/11.61";
const char S29605[] = "Mozilla/5.0 (Windows NT 5.1; ru; rv:2.0) Gecko/X Firefox/4.0 Opera 11.61";
const char S29606[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5218); ru) Presto/2.10.229 Version/11.61";
const char S29607[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5230); ru) Presto/2.10.229 Version/11.61";
const char S29608[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.229 Version/11.61";
const char S29609[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.229 Version/11.61";
const char S29610[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.61";
const char S29611[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.229 Version/11.61";
const char S29612[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.229 Version/11.61";
const char S29613[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5217); ru) Presto/2.10.229 Version/11.61";
const char S29614[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.229 Version/11.61";
const char S29615[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.61";
const char S29616[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.229 Version/11.61";
const char S29617[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.229 Version/11.61";
const char S29618[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.229 Version/11.61";
const char S29619[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.61";
const char S29620[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.10.229 Version/11.61";
const char S29621[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.229 Version/11.61";
const char S29622[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.229 Version/11.61";
const char S29623[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.229 Version/11.61";
const char S29624[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4930); ru) Presto/2.10.229 Version/11.61";
const char S29625[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.229 Version/11.61";
const char S29626[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.229 Version/11.61";
const char S29627[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.10.229 Version/11.61";
const char S29628[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.229 Version/11.61";
const char S29629[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.229 Version/11.61";
const char S29630[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.10.229 Version/11.61";
const char S29631[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4661); ru) Presto/2.10.229 Version/11.61";
const char S29632[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02652); ru) Presto/2.10.229 Version/11.61";
const char S29633[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.229 Version/11.61";
const char S29634[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.10.229 Version/11.61";
const char S29635[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.229 Version/11.61";
const char S29636[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5278); ru) Presto/2.10.229 Version/11.61";
const char S29637[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.229 Version/11.61";
const char S29638[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.229 Version/11.61";
const char S29639[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.229 Version/11.61";
const char S29640[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03672); ru) Presto/2.10.229 Version/11.61";
const char S29641[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03638); ru) Presto/2.10.229 Version/11.61";
const char S29642[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); MRSPUTNIK OW 2, 3, 0, 104; ru) Presto/2.10.229 Version/11.61";
const char S29643[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02560); ru) Presto/2.10.229 Version/11.61";
const char S29644[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5288); ru) Presto/2.10.229 Version/11.61";
const char S29645[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.229 Version/11.61";
const char S29646[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.229 Version/11.61";
const char S29647[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02552); ru) Presto/2.10.229 Version/11.61";
const char S29648[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5328); ru) Presto/2.10.229 Version/11.61";
const char S29649[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.10.229 Version/11.61";
const char S29650[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5667); ru) Presto/2.10.229 Version/11.61";
const char S29651[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.10.229 Version/11.61";
const char S29652[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.229 Version/11.61";
const char S29653[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.10.229 Version/11.61";
const char S29654[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02564); ru) Presto/2.10.229 Version/11.61";
const char S29655[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5299); ru) Presto/2.10.229 Version/11.61";
const char S29656[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.229 Version/11.61";
const char S29657[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.229 Version/11.61,gzip(gfe)";
const char S29658[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5966); ru) Presto/2.10.229 Version/11.61";
const char S29659[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.229 Version/11.61";
const char S29660[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.229 Version/11.61";
const char S29661[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.229 Version/11.61";
const char S29662[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.229 Version/11.61";
const char S29663[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.229 Version/11.61";
const char S29664[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6089); ru) Presto/2.10.229 Version/11.61";
const char S29665[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5993); ru) Presto/2.10.229 Version/11.61";
const char S29666[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5880); ru) Presto/2.10.229 Version/11.61";
const char S29667[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6011); ru) Presto/2.10.229 Version/11.61";
const char S29668[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.229 Version/11.61";
const char S29669[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6054); ru) Presto/2.10.229 Version/11.61";
const char S29670[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6083); ru) Presto/2.10.229 Version/11.61";
const char S29671[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.229 Version/11.61";
const char S29672[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6081); ru) Presto/2.10.229 Version/11.61";
const char S29673[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE72-2/031.023; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.3.1";
const char S29674[] = "NokiaE72/021.024 (SymbianOS/9.3; U; Series60/3.2 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.1.19424";
const char S29675[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE72-2/071.004; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.26 Mobile Safari/533.4 3gpp-gba";
const char S29676[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE72-1/031.023; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.3.1";
const char S29677[] = "NokiaE72-1/SymbianOS/9.1 Series60/3.0";
const char S29678[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE72-1/091.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.34 Mobile Safari/533.4 3gpp-gba";
const char S29679[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7534.48.3";
const char S29680[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S29681[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3";
const char S29682[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9Z999 Safari/7534.48.3";
const char S29683[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9B206 Version/5.1 Safari/8536.25 Safari/6533.18.5";
const char S29684[] = "Mozilla/5.0 (iPod; CPU iPhone OS 511 like Mac OS X) AppleWebKit/534.46 (KHTML like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S29685[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3";
const char S29686[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-I510 4G Build/EP1F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29687[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-I510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29688[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I510 4G Build/EP3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29689[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29690[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 4G Build/0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29691[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-I510 4G Build/EP1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29692[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-I510 Build/GINGERBREAD)";
const char S29693[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-I510 4G Build/EP1H) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29694[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-I510 4G Build/B 4.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29695[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 4G Build/EP4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29696[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29697[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 4G Build/FP1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29698[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SCH-I510 4G Build/EG2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29699[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 4G Build/FP5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29700[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SCH-I510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29701[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 4G Build/FP3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29702[] = "Android/1.0 (SCH-I510 GINGERBREAD)";
const char S29703[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S29704[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S29705[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 4G Build/FP8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29706[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 4G Build/FP6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29707[] = "com.google.android.youtube/2.4.4(Linux; U; Android 2.3.6; en_US; SCH-I510 Build/GINGERBREAD) gzip";
const char S29708[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SCH-I510 4G Build/FP8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29709[] = "Mozilla/5.0 (Windows NT 5.1; rv:13.0) Gecko/X Firefox/13.0.1";
const char S29710[] = "Mozilla/5.0 (Windows NT 5.2; rv:13.0) Gecko/X Firefox/13.0.1";
const char S29711[] = "Mozilla/5.0 (Windows NT 5.1; rv:13.0) Gecko/X Firefox/13.0.1 Firefox/13.0.1";
const char S29712[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MOT-XT615 Build/V1.52A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29713[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; MOT-XT615 Build/V1.540) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29714[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; XT615 Build/V1.37H) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29715[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; XT615 Build/V1.540) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29716[] = "MOT-XT615/Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT615 Build/V1.62D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29717[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MOT-XT615 Build/V1.48C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29718[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; XT615 Build/V1.52A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29719[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT615 Build/V1.590) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29720[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; XT615 Build/V1.48C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29721[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT615 Build/V1.600) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29722[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MOT-XT389 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29723[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT615 Build/V1.59A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29724[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT615 Build/V1.60B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29725[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT615 Build/V1.54B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29726[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; MOT-XT615 Build/V1.67C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29727[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MOT-XT389 Build/06.13.03R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29728[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MOT-XT615 Build/V1.670) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29729[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MOT-XT389 Build/0B.10.00R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29730[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; XT615 Build/V1.54C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29731[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT615 Build/V1.540) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S29732[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT615 Build/V1.37I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29733[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT615 Build/V1.540) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S29734[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; MOT-XT389 Build/0A.17.02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29735[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; XT615 Build/V1.67C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29736[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; XT615 Build/V1.37L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29737[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; XT615 Build/V1.540)";
const char S29738[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; XT389 Build/06.13.03R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29739[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MOT-XT389 Build/06.1A.00R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29740[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MOT-XT389 Build/0B.13.00R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29741[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; XT389 Build/0B.10.00R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29742[] = "MOT-XT615/Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT615 Build/V1.74D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29743[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT615 Build/V1.54B) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S29744[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; MOT-XT389 Build/0A.26.00R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29745[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; XT389 Build/0C.03.05R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29746[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; XT615 Build/V1.670) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29747[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT389 Build/07.1F.04R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29748[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; XT389 Build/0B.13.00R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29749[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT615 Build/V1.60E) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29750[] = "MOT-XT615/Mozilla/5.0 (Linux; U; Android 2.3.7; fr-ca; XT615 Build/V1.74F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29751[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ro-ro; MOT-XT615 Build/V1.67A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29752[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT615 Build/V1.37I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S29753[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT615 Build/V1.60G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29754[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MOT-XT389 Build/0A.1A.06R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29755[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT615 Build/V1.37H) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S29756[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; XT389 Build/0A.17.02R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29757[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; XT389 Build/06.1A.00R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29758[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT389 Build/07.1F.1CR_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29759[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT615 Build/V1.60C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29760[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5";
const char S29761[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5,gzip(gfe) (via translate.google.com)";
const char S29762[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S29763[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5";
const char S29764[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5";
const char S29765[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; OfficeLiveConnector.1.4; OfficeLivePatch.1.3)";
const char S29766[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Avant Browser)";
const char S29767[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Avant Browser)";
const char S29768[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Avant Browser)";
const char S29769[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Avant Browser)";
const char S29770[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Avant Browser)";
const char S29771[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 6.1; Avant Browser)";
const char S29772[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; Avant Browser)";
const char S29773[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Avant Browser)";
const char S29774[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Avant Browser)";
const char S29775[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7; Avant Browser)";
const char S29776[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Avant Browser)";
const char S29777[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Avant Browser)";
const char S29778[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; MAAU; NP08; Avant Browser)";
const char S29779[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_DesireZ_A7272 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29780[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-ch; HTC Desire Z 1.82.166.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29781[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-ch; HTC Desire Z  2.42.166.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29782[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire Z 2.42.161.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29783[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire Z 1.82.161.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29784[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-ch; HTC Desire Z 2.42.166.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29785[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC Desire Z 1.85.861.3 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29786[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; HTC-A7275 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29787[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_DesireZ_A7272 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29788[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC Desire Z 2.42.861.6 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29789[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire Z 2.42.161.20 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29790[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC Desire Z 2.42.861.8 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29791[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-at; HTC Desire Z 2.42.166.4 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29792[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-at; HTC Desire Z  2.42.166.4 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29793[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; nl-nl; HTC Desire Z 2.42.161.22 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29794[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus 4 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S29795[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S29796[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S29797[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)";
const char S29798[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe)";
const char S29799[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe)";
const char S29800[] = "Mozilla/5.0 (Linux; Android 4.1; Galaxy Nexus Build/JRN84D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S29801[] = "Mozilla/5.0 (Linux; Android 4.1.1; Galaxy Nexus Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S29802[] = "Mozilla/5.0 (Linux; Android 4.1;  Galaxy Nexus Build/JRN84D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S29803[] = "Mozilla/5.0 (Linux; Android 4.1.1; Galaxy Nexus Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S29804[] = "Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S29805[] = "Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S29806[] = "Mozilla/5.0 (Linux; Android 4.1.1;  Galaxy Nexus Build/JRO03O) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S29807[] = "Mozilla/5.0 (Linux; Android 4.1.1; Galaxy X Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S29808[] = "Mozilla/5.0 (Linux; Android 4.1.2;  Galaxy Nexus Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S29809[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S29810[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S29811[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S29812[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S29813[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S29814[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.65 Safari/535.11";
const char S29815[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11";
const char S29816[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11";
const char S29817[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.83 Safari/535.11";
const char S29818[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.10 (KHTML, like Gecko) Chrome/17.0.925.0 QQBrowser/1.0.0.1096 Safari/535.10";
const char S29819[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) UC AppleWebKit/534.31 (KHTML, like Gecko) Chrome/17.0.558.0 Safari/534.31";
const char S29820[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/535.11 (KHTML	 like Gecko) Chrome/17.0.963.46 Safari/535.11";
const char S29821[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.4.0.2117 Safari/535.11";
const char S29822[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1061) AppleWebKit/535.11 (KHTML like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S29823[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1060) AppleWebKit/535.11 (KHTML like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S29824[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/17.0.648.205 Safari/534.16";
const char S29825[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/17.0.648.205 Safari/534.16";
const char S29826[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/17.0.648.205 Safari/534.16";
const char S29827[] = "Mozilla/5.0 (Windows; I; Windows NT 5.1; en; rv:1.9.2.13) Gecko/X Firefox/5.0.1";
const char S29828[] = "Mozilla/5.0 (Windows NT 5.1; rv:5.0.1) Gecko/X Firefox/5.0.1 Firefox/5.0.1";
const char S29829[] = "Mozilla/5.0 (Windows NT 5.1) Gecko/X Firefox/5.0";
const char S29830[] = "Mozilla/5.0 (Windows NT 5.2; rv:5.0) Gecko/X Firefox/5.0,gzip(gfe) (via translate.google.com)";
const char S29831[] = "Mozilla/5.0 (Windows NT 5.1; rv:5.0) Gecko/X Firefox/5.0,gzip(gfe)";
const char S29832[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:5.0) Gecko/X Firefox/5.0";
const char S29833[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Avant Browser)";
const char S29834[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Avant Browser; Avant Browser)";
const char S29835[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Avant Browser)";
const char S29836[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; Avant Browser)";
const char S29837[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; Avant Browser)";
const char S29838[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; Avant Browser)";
const char S29839[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Avant Browser; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Avant Browser)";
const char S29840[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Avant Browser; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Avant Browser)";
const char S29841[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Avant Browser; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; MSN Optimized;FR; .NET CLR 2.0.50727; FDM; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSN Optimized;FR)";
const char S29842[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; Avant Browser; Avant Browser)";
const char S29843[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; Avant Browser; Avant Browser)";
const char S29844[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; Avant Browser; SV1)";
const char S29845[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Avant Browser)";
const char S29846[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; Avant Browser; Avant Browser)";
const char S29847[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; SV1; Avant Browser)";
const char S29848[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; SV1; Avant Browser)";
const char S29849[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; SV1; Avant Browser)";
const char S29850[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Avant Browser)";
const char S29851[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Avant Browser; Avant Browser; Avant Browser)";
const char S29852[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29853[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9082 Build/JZO54K)";
const char S29854[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9082L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29855[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29856[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.3 Version/4.0 Mobile Safari/530.17";
const char S29857[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9082L Build/JZO54K)";
const char S29858[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9082i Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29859[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9082 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S29860[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Samsung-GT-I9082/1.0 Android/4.1.1 Release/05.24.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29861[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ST18i Build/4.1.A.0.562) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29862[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; Xperia Ray Build/ITL41F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29863[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Xperia ray Build/4.1.A.0.562) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29864[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; ST18i Build/4.1.A.0.562)";
const char S29865[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST18i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29866[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SonyEricssonST18a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29867[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; ST18a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29868[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonST18i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29869[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST18i Build/4.1.B.0.587)";
const char S29870[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST18i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29871[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ST18i Build/4.1.B.0.431) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S29872[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonST18iv Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29873[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; SonyEricssonST18av Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29874[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ST18i Build/4.1.B.0.587) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S29875[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST18a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29876[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; ST18i Build/4.1.B.1.13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29877[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST18a Build/4.1.B.0.587)";
const char S29878[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST15i Build/4.0.1.A.0.169) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29879[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; ST15i Build/4.0.A.2.364) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29880[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonST15i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29881[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST15i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29882[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST15i Build/4.0.1.A.2.24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29883[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonST15i Build/4.0.A.2.330) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29884[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonST15i Build/4.0.A.2.319) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29885[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonST15i Build/4.0.A.2.364) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29886[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonST15i Build/4.0.A.2.329) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29887[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonST15i Build/4.0.A.2.254) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29888[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonST15i Build/4.0.A.2.240) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29889[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ST15i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29890[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ie; SonyEricssonST15i Build/4.0.A.2.377) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29891[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; ST15i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29892[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST15i Build/4.0.2.A.0.45) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29893[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; ST15i Build/4.0.A.2.377)";
const char S29894[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; ST15i Build/4.0.2.A.0.62) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S29895[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; ST15i Build/4.0.2.A.0.62)";
const char S29896[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonST15i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29897[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonST15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29898[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; ST15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29899[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ie; SonyEricssonST15i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29900[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-mx; SonyEricssonST15a Build/4.0.A.2.377) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29901[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; ST15i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29902[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-mx; ST15a Build/4.0.A.2.377) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29903[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; ST15i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29904[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-cr; SonyEricssonST15a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29905[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; hr-hr; SonyEricssonST15iv Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29906[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; SonyEricssonST15a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29907[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; ST15a Build/4.0.A.2.377)";
const char S29908[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; SonyEricssonST15a Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29909[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonST15i Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29910[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; hr-hr; SonyEricssonST15iv Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29911[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; ST15i Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29912[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-cr; ST15a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29913[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; ST15a Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29914[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; hr-hr; SonyEricssonST15iv Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29915[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; ST15a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29916[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; ST15i Build/4.0.A.2.377) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29917[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fa-; SonyEricssonST15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29918[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XT912 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29919[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT912 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29920[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29921[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/Fat-Free Bean) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29922[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29923[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16)";
const char S29924[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S29925[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/BATAKANG) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29926[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29927[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29928[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30";
const char S29929[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/Breakneck Blur 98.72.16 (r7)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29930[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/Blurry JB) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S29931[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B329 Safari/8536.25";
const char S29932[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B146 Safari/8536.25";
const char S29933[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B141 Safari/8536.25";
const char S29934[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B147 Safari/8536.25";
const char S29935[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300T Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S29936[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-i9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S29937[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S29938[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I93000 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S29939[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-I535 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S29940[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S29941[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S29942[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S29943[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S29944[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S29945[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S29946[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S29947[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S29948[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S29949[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-I535 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S29950[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999V Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S29951[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S29952[] = "Mozilla/5.0 (Linux; Android 4.1.9; GT-I9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S29953[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999V Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S29954[] = "Mozilla/5.0 (Linux; Android 4.1.1; I9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S29955[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S29956[] = "Mozilla/5.0 (Linux; Android 4.1.9; I9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S29957[] = "Mozilla/5.0 (Linux; Android 4.1; i9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S29958[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999V Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S29959[] = "Mozilla/5.0 (Linux; Android 4.1.2; SC-06D Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S29960[] = "Opera/9.80 (Android; Opera Mini/6.0.24556/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S29961[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-R720 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29962[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SCH-R720 Build/GINGERBREAD)";
const char S29963[] = "Logos/0.9.4 (Linux; U; Android 2.3.4; en_US; SCH-R720 Build/GINGERBREAD.EH02)";
const char S29964[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-R720 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S29965[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-R720 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29966[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SCH-R720 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29967[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; -us; SCH-R720 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29968[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-R720 Build/GINGERBREAD)";
const char S29969[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en_us; SCH-R720 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)";
const char S29970[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-R720 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S29971[] = "Android/1.0 (SCH-R720 GINGERBREAD)";
const char S29972[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-R720 Build/V6_1.2.60412.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29973[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-R720 Build/Viperdink) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29974[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; -us; SCH-R720 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29975[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-R720 Build/Chicken-CTMod) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S29976[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-R720 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Android MetroPCS androidAppVersion/2.5.10 AndroidWebkitWrapper BillingSupported";
const char S29977[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9B206 Safari/7534.48.3";
const char S29978[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9B176 Safari/7534.48.3";
const char S29979[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9B208 Safari/7534.48.3";
const char S29980[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9B179 Safari/7534.48.3";
const char S29981[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9A405 Safari/7534.48.3";
const char S29982[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.2.149.29 Safari/525.13";
const char S29983[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.3.154.0 Safari/525.19";
const char S29984[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.A.B.C Safari/525.13";
const char S29985[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.4.154.29 Safari/525.19";
const char S29986[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.2.149.27 Safari/525.13";
const char S29987[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.2.149.27 Safari/525.13";
const char S29988[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.3.154.6 Safari/525.19";
const char S29989[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.4.154.23 Safari/525.19";
const char S29990[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/0.0.0.0 Safari/534.24";
const char S29991[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; ru-RU) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.A.B.C Safari/525.13";
const char S29992[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; de) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.4.154.29 Safari/525.19";
const char S29993[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; de) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.2.149.27 Safari/525.13";
const char S29994[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/0.0.0.0 Safari/534.16";
const char S29995[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; es-ES) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.A.B.C Safari/525.13";
const char S29996[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; fr-FR) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.A.B.C Safari/525.13";
const char S29997[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.A.B.C Safari/525.13";
const char S29998[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.A.B.C Safari/525.13 AppEngine-Google; (+http://code.google.com/appengine; appid: s~cookn-cloud)";
const char S29999[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; AppleWebKit/525.13; KHTML?? like Gecko; Chrome/0.2.149.27 Safari/525.13)";
const char S30000[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1;) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.A.B.C Safari/525.13";
const char S30001[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LG-P999 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S30002[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;LG-P999 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S30003[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LG-P999 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S30004[] = "Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; LG-P999 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S30005[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LG-P999Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S30006[] = "Mozilla/5.0 (Linux; U; Android2.3.3; en-us; LG-P999 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S30007[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P999 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30008[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P999 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30009[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; LG-P999 Build/GRI40)";
const char S30010[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P999 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30011[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P999 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30012[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P999 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30013[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P999 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S30014[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P999 Build/FRG83G; LG-P990 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30015[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LG-P999 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30016[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LG-P999 Build/GRI40) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S30017[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P999 Build/GRJ22)";
const char S30018[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P999 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S30019[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; LG-P999 Build/GRI40)";
const char S30020[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P999 Build/SINGH-UI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30021[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P999 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S30022[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;LG-P999 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S30023[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; LG-P999 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S30024[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; LG-P999 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S30025[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P999Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S30026[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P999 Build/GRJB54C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S30027[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; LG-P999 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30028[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P999 Build/RG2x) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30029[] = "com.google.android.youtube/2.4.4(Linux; U; Android 2.3.4; en_US; LG-P999 Build/GRJ22) gzip";
const char S30030[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1";
const char S30031[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1";
const char S30032[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1";
const char S30033[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1";
const char S30034[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1";
const char S30035[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1";
const char S30036[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1";
const char S30037[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1";
const char S30038[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1";
const char S30039[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1";
const char S30040[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.1 (KHTML	 like Gecko) Chrome/13.0.782.107 Safari/535.1";
const char S30041[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/13.0.648.205 Safari/534.16";
const char S30042[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/13.0.648.205 Safari/534.16";
const char S30043[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/13.0.648.205 Safari/534.16";
const char S30044[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID RAZR Build/6.7.1-68_DHD-15_M4-7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30045[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDRAZR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30046[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID RAZR 4G Build/6.7.1-68_DHD-15_M4-11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30047[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID RAZR 4G Build/6.7.1-68_DHD-15_M4-7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30048[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID RAZR 4G Build/6.7.1-68_DHD-15_M4-17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30049[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID RAZR Build/6.7.1-68_DHD-15_M4-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30050[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Droid Razr Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30051[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID RAZR 4G Build/6.7.1-68_DHD-15_M4-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30052[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT912 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30053[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID RAZR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30054[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID RAZR Build/6.7.1-68_DHD-15_M4-11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30055[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT912 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30056[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR 4G Build/6.7.2-180_DHD-16_M4-26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30057[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR 4G Build/6.7.2-180_DHD-16_M4-29) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30058[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR 4G Build/6.7.2-180_DHD-16_M4-30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30059[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30060[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR 4G Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30061[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; DROID RAZR Build/6.7.2-180_DHD-16_M4-31)";
const char S30062[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30063[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR 4G Build/6.7.2-180_DHD-16_M4-34) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30064[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-35) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30065[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR 4G Build/6.7.2-180_DHD-16_M4-35) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30066[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30067[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/Arctic Beta) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30068[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-34) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30069[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; DROID RAZR Build/6.7.1-68_DHD-15_M4-7)";
const char S30070[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-29) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30071[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30072[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR 4G Build/6.7.2-180_DHD-16_M4-28) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30073[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30";
const char S30074[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR 4G Build/6.7.2-180_DHD-16_M4-36) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30075[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR 4G Build/6.7.2-180_DHD-16_M4-24) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30076[] = "Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 4.0.4; MOTOROLA DROID RAZR Build/ICE_CREAM_SANDWICH_MR1);";
const char S30077[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; DROID RAZR 4G Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30078[] = "MOT-DROID RAZR/ Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) 1617 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30079[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT912 Build/B4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30080[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.215 Safari/535.1";
const char S30081[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.215 Safari/535.1";
const char S30082[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1";
const char S30083[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1";
const char S30084[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1";
const char S30085[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1";
const char S30086[] = "ArchitextSpider";
const char S30087[] = "Spider.TerraNautic.net";
const char S30088[] = "ZoomSpider - wrensoft.com [ZSEBOT]";
const char S30089[] = "Comodo-Certificates-Spider";
const char S30090[] = "Xaldon WebSpider";
const char S30091[] = "JOC Web Spider";
const char S30092[] = "Sogou Pic Spider/3.0(+http://www.sogou.com/docs/help/webmasters.htm#07)";
const char S30093[] = "Liquida Spider 1.0 +http://liquida.com/";
const char S30094[] = "Test Spider 0.2";
const char S30095[] = "QBikSpider/2.0";
const char S30096[] = "WCH Web Spider";
const char S30097[] = "DaylifeFeedFetcher/1.0 (http://www.daylife.com)";
const char S30098[] = "NodejsSpider/1.0";
const char S30099[] = "DLE_Spider.exe";
const char S30100[] = "XYZ Spider";
const char S30101[] = "Mozilla/5.0 (compatible; SpiderLing (a SPIDER for LINGustic research); http://nlp.fi.muni.cz/projects/biwec/)";
const char S30102[] = "Blogshares Spiders (Renewed V1.65)";
const char S30103[] = "Sogou News Spider/4.0(+http://www.sogou.com/docs/help/webmasters.htm#07)";
const char S30104[] = "Sogou Mobile Spider1.0 (http://wap.sogou.com)";
const char S30105[] = "Sogou Web Spider";
const char S30106[] = "EasouSpider";
const char S30107[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B5105c Safari/7534.48.3";
const char S30108[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B141 Safari/7534.48.3";
const char S30109[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B146 Safari/7534.48.3";
const char S30110[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B147 Safari/7534.48.3";
const char S30111[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B329 Safari/7534.48.3";
const char S30112[] = "Opera/9.80 (Windows NT 6.1; U; en) Presto/2.9.168 Version/11.51";
const char S30113[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.9.168 Version/11.51";
const char S30114[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.9.168 Version/11.51";
const char S30115[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.9.168 Version/11.51";
const char S30116[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.9.168 Version/11.51";
const char S30117[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.9.168 Version/11.51";
const char S30118[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.9.168 Version/11.51";
const char S30119[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.9.168 Version/11.51";
const char S30120[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.9.168 Version/11.51";
const char S30121[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.9.168 Version/11.51,gzip(gfe) (via translate.google.com)";
const char S30122[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.9.168 Version/11.51";
const char S30123[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.9.168 Version/11.51";
const char S30124[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; en) Opera 11.51";
const char S30125[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4841); ru) Presto/2.9.168 Version/11.51";
const char S30126[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4848); ru) Presto/2.9.168 Version/11.51";
const char S30127[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.9.168 Version/11.51";
const char S30128[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.9.168 Version/11.51";
const char S30129[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.9.168 Version/11.51";
const char S30130[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.9.168 Version/11.51";
const char S30131[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4661); ru) Presto/2.9.168 Version/11.51";
const char S30132[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03278); ru) Presto/2.9.168 Version/11.51";
const char S30133[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4110); ru) Presto/2.9.168 Version/11.51";
const char S30134[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.9.168 Version/11.51";
const char S30135[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4832); ru) Presto/2.9.168 Version/11.51";
const char S30136[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4871); ru) Presto/2.9.168 Version/11.51";
const char S30137[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.9.168 Version/11.51";
const char S30138[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03638); ru) Presto/2.9.168 Version/11.51";
const char S30139[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.9.168 Version/11.51";
const char S30140[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.9.168 Version/11.51";
const char S30141[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03780); ru) Presto/2.9.168 Version/11.51";
const char S30142[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.9.168 Version/11.51";
const char S30143[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.9.168 Version/11.51";
const char S30144[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.9.168 Version/11.51";
const char S30145[] = "Mozilla/5.0 (Windows NT 6.1; ru; rv:2.0) Gecko/X Firefox/4.0 Opera 11.51";
const char S30146[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.9.168 Version/11.51";
const char S30147[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.9.168 Version/11.51";
const char S30148[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.9.168 Version/11.51";
const char S30149[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.9.168 Version/11.51";
const char S30150[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.9.168 Version/11.51";
const char S30151[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.9.168 Version/11.51";
const char S30152[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.9.168 Version/11.51";
const char S30153[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5267); ru) Presto/2.9.168 Version/11.51";
const char S30154[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5265); ru) Presto/2.9.168 Version/11.51";
const char S30155[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.9.168 Version/11.51";
const char S30156[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.9.168 Version/11.51";
const char S30157[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.9.168 Version/11.51,gzip(gfe)";
const char S30158[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.9.168 Version/11.51";
const char S30159[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.9.168 Version/11.51";
const char S30160[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.9.168 Version/11.51";
const char S30161[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5966); ru) Presto/2.9.168 Version/11.51";
const char S30162[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6578); ru) Presto/2.9.168 Version/11.51";
const char S30163[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6089); ru) Presto/2.9.168 Version/11.51";
const char S30164[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE63-3/200.21.012; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S30165[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE63-1/510.21.010; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S30166[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE63-1/500U; Series60/3.1 NokiaE63-1/500.21.009; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S30167[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokianOS/9.2; U; Series60/3.1 NokiaE63-1/500.21.009; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S30168[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE63-2/200.21.012; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S30169[] = "NokiaE63/200.21.005 (SymbianOS/9.2; U; Series60/3.1 Mozilla/5.0; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S30170[] = "NokiaE63/UCBrowser7.7.0.81/28/800";
const char S30171[] = "NokiaE63";
const char S30172[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/22.0.1229.79) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4";
const char S30173[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/22.0.1229.79) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4";
const char S30174[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/22.0.1229.92) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.92 Safari/537.4";
const char S30175[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/22.0.1229.92) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.92 Safari/537.4";
const char S30176[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/22.0.1229.94) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S30177[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/22.0.1229.94)";
const char S30178[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/22.0.1229.94)";
const char S30179[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/22.0.1229.94; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S30180[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/22.0.1229.94; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S30181[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/22.0.1229.94) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S30182[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.4; chromeframe/22.0.1229.94; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S30183[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; chromeframe/22.0.1229.94; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S30184[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; chromeframe/22.0.1229.94; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S30185[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; chromeframe/22.0.1229.94; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S30186[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/22.0.1229.94; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S30187[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/22.0.1229.96) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.96 Safari/537.4";
const char S30188[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:17.0) Gecko/X Thunderbird/17.0.8";
const char S30189[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.38 Safari/532.0";
const char S30190[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.27 Safari/532.0";
const char S30191[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.8 (KHTML, like Gecko) Chrome/3.1.0.0 Safari/532.8";
const char S30192[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.0 (KHTML,like Gecko) Chrome/3.0.195.27";
const char S30193[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Source Client; ) AppleWebKit/532.1 (KHTML, like Gecko) Chrome/3.0.195.24 Safari/532.1";
const char S30194[] = "LongURL API";
const char S30195[] = "UniversalFeedParser/4.2-pre-316-svn +http://feedparser.org/";
const char S30196[] = "UniversalFeedParser/5.1.1 +https://code.google.com/p/feedparser/";
const char S30197[] = "Mozilla/4.0 (compatible; grub-client-0.3.x; Crawl your own stuff with http://grub.org)";
const char S30198[] = "Mozilla/2.0 (compatible; Ask Jeeves/Teoma; http://sp.ask.com/docs/about/tech_crawling.html)";
const char S30199[] = "Mozilla/5.0 (compatible; Genieo/1.0 http://www.genieo.com/webfilter.html";
const char S30200[] = "Mozilla/5.0 (compatible; Genieo/1.0 http://www.genieo.com/webfilter.html)";
const char S30201[] = "Mozilla/5.0 (compatible; Crawling jpeg; http://www.yama.info.waseda.ac.jp)";
const char S30202[] = "Mozilla/5.0 (compatible; zermelo +http://www.powerset.com) [email:paul@page-store.com,crawl@powerset.com]";
const char S30203[] = "Mozilla/4.0 (compatible; grub-client-0.2.x; Crawl your stuff with http://grub.org)";
const char S30204[] = "RockmeltEmbedService (http://rockmelt.com)";
const char S30205[] = "Mozilla/5.0 (X11; Linux i686; rv:2.0b12pre) Gecko/X Firefox/4.0b12pre";
const char S30206[] = "Mozilla/5.0 (X11; Linux i686; rv:2.0.1) Gecko/X Firefox/4.0.1 Iceweasel/4.0.1 WebMoney Advisor";
const char S30207[] = "a/5.0 (X11; Linux x86_64; rv:2.0b11) Gecko/X Firefox/4.0b11";
const char S30208[] = "User-Agent: Mozilla/5.0 (X11; U; Linux x86_64; en-GB; rv:1.9.2.3) Gecko/X Gentoo Firefox/4";
const char S30209[] = "Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:18.0) Gecko/X Firefox/18.0";
const char S30210[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:11.0) Gecko Firefox/11.0";
const char S30211[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:11.0) Gecko/X Firefox/11.0 Firefox/11.0";
const char S30212[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; rv:11.0) Gecko/X Firefox/11.0";
const char S30213[] = "Mozilla/5.0 (Windows; Windows NT 6.1; Win64; x64; rv:11.0) Gecko/X Firefox/11.0";
const char S30214[] = "Mozilla/5.0 Panguso Transcoder (Windows NT 6.1; rv:11.0) Gecko/X Firefox/11.0";
const char S30215[] = "SAMSUNG-SGH-E250/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0 (compatible; Googlebot-Mobile/2.1; +http://www.google.com/bot.html)";
const char S30216[] = "SAMSUNG-SGH-E250/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0 (compatible; Googlebot-Mobile/2.1; +http://www.g;c2VvLWNyYXdsZXI=";
const char S30217[] = "SAMSUNG-SGH-E250/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0 (compatible; Googlebot-Mobile/2.1;  http://www.google.com/bot.html)";
const char S30218[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/10";
const char S30219[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.151 Safari/534.16";
const char S30220[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16,gzip(gfe) (via translate.google.com)";
const char S30221[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.64 Safari/537.31";
const char S30222[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.64 Safari/537.31";
const char S30223[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.85 Safari/537.36";
const char S30224[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S30225[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S30226[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747M Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S30227[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S30228[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.74 Safari/537.36";
const char S30229[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S30230[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S30231[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S30232[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S30233[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530C Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S30234[] = "Mozilla/5.0 (Linux; Android 4.1.1; SPH-L710 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S30235[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S30236[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S30237[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-R530U Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S30238[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S30239[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S30240[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S30241[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S30242[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305N Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S30243[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S30244[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S30245[] = "Mozilla/5.0 (compatible; Yahoo! Slurp/3.0; http://help.yahoo.com/help/us/ysearch/slurp)";
const char S30246[] = "Mozilla/5.0 (compatible; Yahoo! Slurp; http://help.yahoo.com/help/us/ysearch/slurp)";
const char S30247[] = "Mozilla/5.0 (compatible; Yahoo! Slurp;http://help.yahoo.com/help/us/ysearch/slurp)";
const char S30248[] = "Mozilla/5.0 (compatible; Yahoo! Slurp/3.0 ; http://help.yahoo.com/help/us/ysearch/slurp)";
const char S30249[] = "Mozilla/5.0 (compatible; Yahoo! Slurp/3.0 Asia; http://help.yahoo.com/help/us/ysearch/slurp)";
const char S30250[] = "Mozilla/5.0 (compatible; Yahoo! DE Slurp; http://help.yahoo.com/help/us/ysearch/slurp)";
const char S30251[] = "Mozilla/5.0 (compatible; Yahoo! Slurp Asia; http://help.yahoo.com/help/us/ysearch/slurp)";
const char S30252[] = "Mozilla/5.0 (Slurp/si; slurp@inktomi.com; http://www.inktomi.com/slurp.html)";
const char S30253[] = "Mozilla/5.0 (Slurp/cat; slurp@inktomi.com; http://www.inktomi.com/slurp.html)";
const char S30254[] = "Mozilla/5.0 (compatible; Yahoo! Nano; http://help.yahoo.com/help/us/ysearch/slurp)";
const char S30255[] = "Mozilla/3.0 (Slurp.so/Goo; slurp@inktomi.com; http://www.inktomi.com/slurp.html)";
const char S30256[] = "Mozilla/3.0 (Slurp/si; slurp@inktomi.com; http://www.inktomi.com/slurp.html)";
const char S30257[] = "Mozilla/3.0 (Slurp/cat; slurp@inktomi.com; http://www.inktomi.com/slurp.html)";
const char S30258[] = "Mozilla/5.0 (compatible; Yahoo! Slurp/3.0; http://help.yahoo.com/help/us/ysearch/slurp),gzip(gfe) (via docs.google.com/viewer)";
const char S30259[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xperia Arc Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30260[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; Xperia Arc Build/ITL41F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30261[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Xperia Arc Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30262[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; SonyEricssonXperia Arc Build/4.1.A.0.562) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30263[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT15i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30264[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyEricssonLT15i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30265[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT15i Build/4.1.B.0.587)";
const char S30266[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonLT15iv Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30267[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT15i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30268[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; LT15i Build/4.1.B.1.13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30269[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A551 Safari/8536.25";
const char S30270[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A405 Safari/8536.25";
const char S30271[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A403 Safari/8536.25";
const char S30272[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A523 Safari/8536.25";
const char S30273[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A525 Safari/8536.25";
const char S30274[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A406 Safari/8536.25";
const char S30275[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A403 Safari/8536.25";
const char S30276[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A523 Safari/8536.25";
const char S30277[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A407 Safari/8536.25";
const char S30278[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A8426 Safari/8536.25";
const char S30279[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A550 Safari/8536.25";
const char S30280[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A8500 Safari/8536.25";
const char S30281[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer Prime TF201 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30282[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer Prime TF201 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30283[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Transformer Prime TF201 Build/JRO03C)";
const char S30284[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Transformer Prime TF201 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30285[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; TF201 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30286[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; Transformer Prime TF201 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30287[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Galaxy Nexus Build/ITL41F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30288[] = "Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; Galaxy Nexus Build/ICL53C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30289[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Galaxy Nexus Build/I9250XWKK5_ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30290[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.1; Galaxy Nexus Build/ITL41F)";
const char S30291[] = "Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; Galaxy Nexus Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30292[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.2; Galaxy Nexus Build/ICL53F)";
const char S30293[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Galaxy Nexus Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30294[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Galaxy Nexus Build/IML74K)";
const char S30295[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Galaxy Nexus Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30296[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Galaxy Nexus Build/IML75) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30297[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Galaxy Nexus Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) Mobile Safari/535.7";
const char S30298[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Galaxy Nexus Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30299[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GalaxyNexus Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30300[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30301[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM30B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30302[] = "com.google.android.youtube/3.3.0(Linux; U; Android 4.0.2; en_US; Galaxy Nexus Build/ICL53F) gzip";
const char S30303[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GalaxyNexus Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30304[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM30D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30305[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM30C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30306[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM62B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30307[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76B) AppleWebKit/535.7 (KHTML, like Gecko) Mobile Safari/535.7";
const char S30308[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30309[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76F) AppleWebKit/535.7 (KHTML, like Gecko) Mobile Safari/535.7";
const char S30310[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30311[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76I) AppleWebKit/535.7 (KHTML, like Gecko) Mobile Safari/535.7";
const char S30312[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-us; Galaxy Nexus Build/IFL10B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30313[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30314[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30315[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30316[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Galaxy Nexus Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.1 Mobile Safari/534.30";
const char S30317[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Galaxy Nexus Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30318[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Galaxy Nexus Build/ITL41F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30319[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30320[] = "Mozilla/5.0 (Linux; U; Android 4.0.2; pt-br; Galaxy X Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30321[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Galaxy Nexus Build/IMM76K)";
const char S30322[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Galaxy Nexus Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30323[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Galaxy Nexus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30324[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Sprint Galaxy Nexus CDMA Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30325[] = "Mozilla/5.0 (Linux; U; Android 4.0.4 IMM76D; en-us; Galaxy Nexus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30326[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/ITL41F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30327[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Galaxy Nexus Build/IMM30B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30328[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM62) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30329[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM72B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30330[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76K) AppleWebKit/535.7 (KHTML, like Gecko) Mobile Safari/535.7";
const char S30331[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30332[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30333[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; GT-I9250_EUR_XX Build/ICL23D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30334[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; zh-cn; Galaxy Nexus Build/ITL41F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S30335[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30336[] = "Mozilla/5.0 (Linux; U; Android 4.0.2; zh-cn; Galaxy Nexus Build/ICL53F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S30337[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Galaxy Nexus Build/IMM76K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S30338[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; Galaxy X Build/IMM76K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30339[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-us; Galaxy Nexus Build/IFL10C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30340[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Galaxy Nexus Build/ICL25B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30341[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Galaxy Nexus Build/ITL39B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30342[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Galaxy Nexus Build/IML77B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30343[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Galaxy Nexus Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S30344[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Galaxy Nexus Build/IMM76I) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S30345[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S30346[] = "Mozilla/5.0 (Linux; U; Android 4.0.2; zh-cn; Galaxy Nexus Build/ICL53F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S30347[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30348[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Samsung Galaxy Nexus Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30349[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; Galaxy Nexus Build/M7511) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30350[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Galaxy Nexus Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S30351[] = "Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; Galaxy Nexus Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30352[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30353[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; Galaxy Nexus Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30354[] = "Mozilla/5.0 (Linux; U; Android 4.0.2; ja-jp; Galaxy Nexus Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30355[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30356[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; el-gr; Galaxy Nexus Build/M7513) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30357[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; Galaxy Nexus Build/IMM48) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30358[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Galaxy Nexus Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30359[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; Galaxy Nexus Build/TB27TGPS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30360[] = "BlackBerry9630/5.0.0.975 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/105";
const char S30361[] = "Opera/9.80 (SpreadTrum; Opera Mini/4.4.31492/31.1475; U; en) Presto/2.8.119 Version/11.10";
const char S30362[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.0; U; en-CA) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/233.70 Safari/534.6 TouchPad/1.0";
const char S30363[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.0; U; en-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/233.70 Safari/534.6 TouchPad/1.0";
const char S30364[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.0; U; de-DE) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/233.70 Safari/534.6 TouchPad/1.0";
const char S30365[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.0; U; en-GB) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/233.70 Safari/534.6 TouchPad/1.0";
const char S30366[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; en-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0";
const char S30367[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; en-GB) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0";
const char S30368[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; en-CA) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0";
const char S30369[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; de-DE) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0";
const char S30370[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.3; U; en-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.3 Safari/534.6 TouchPad/1.0";
const char S30371[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; fr-FR) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0";
const char S30372[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; en-IE) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0";
const char S30373[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; en-AU) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0";
const char S30374[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; en-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0";
const char S30375[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; fr-FR) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0";
const char S30376[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; de-DE) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0";
const char S30377[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; en-GB) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0";
const char S30378[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; en-IE) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0";
const char S30379[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; en-CA) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0";
const char S30380[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; en-NZ) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0";
const char S30381[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; es-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0";
const char S30382[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; fr-CA) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0";
const char S30383[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; en-AU) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0";
const char S30384[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; en-SG) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0";
const char S30385[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.1; U; en-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/233.72 Safari/534.6 TouchPad/1.0";
const char S30386[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; es-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0";
const char S30387[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; en-SG) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0";
const char S30388[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; en-NZ) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0";
const char S30389[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; fr-CA) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0";
const char S30390[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; en-GB) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0";
const char S30391[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; de-DE) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0";
const char S30392[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; en-CA) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0";
const char S30393[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; fr-CA) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0";
const char S30394[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; es-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0";
const char S30395[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; en-IE) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0";
const char S30396[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; en-SG) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0";
const char S30397[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; es-MX) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0";
const char S30398[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; en-HK) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0";
const char S30399[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; en-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0";
const char S30400[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; fr-FR) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0";
const char S30401[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.0; U; es-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/233.70 Safari/534.6 TouchPad/1.0";
const char S30402[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; en-AU) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0";
const char S30403[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; es-ES) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0";
const char S30404[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; es-ES) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0";
const char S30405[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; es-MX) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0";
const char S30406[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; en-NZ) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0";
const char S30407[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; en-MX) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0";
const char S30408[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; zh-CN) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0";
const char S30409[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.0; U; fr-FR) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/233.70 Safari/534.6 TouchPad/1.0";
const char S30410[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; es-ES) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0";
const char S30411[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; es-MX) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0";
const char S30412[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; it-IT) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0";
const char S30413[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.6; U; en-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0";
const char S30414[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.0; U; en-IE) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/233.70 Safari/534.6 TouchPad/1.0";
const char S30415[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; en-HK) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0";
const char S30416[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; zh-CN) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0";
const char S30417[] = "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; zh-CN) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0";
const char S30418[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_C715c Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30419[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCEVODesign4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30420[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTCEVODesign4G Build/IML74K)";
const char S30421[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sprint APC715CKT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30422[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PH44100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30423[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; PH44100 Build/IML74K)";
const char S30424[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N8010 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30425[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-N8010 Build/IMM76D)";
const char S30426[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-N8010 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30427[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LT30p Build/9.1.A.0.489) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30428[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyLT30p Build/9.1.A.0.489) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30429[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-ca; SonyLT30p Build/9.1.A.0.492) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30430[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LT30p Build/9.1.A.1.140) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30431[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyLT30p Build/9.1.A.1.140) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30432[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; LT30p Build/9.1.A.0.492) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30433[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyLT30p-o Build/9.1.A.1.141) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30434[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LT30p Build/9.1.A.1.141)";
const char S30435[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyLT30p Build/9.1.A.1.141) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30436[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LT30p Build/9.1.A.1.141) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30437[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; SonyLT30p Build/9.1.A.1.142) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30438[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyLT30p-o Build/9.1.A.1.145) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30439[] = "SonyLT30p Build/9.1.A.1.140 stagefright/1.2 (Linux;Android 4.1.2)";
const char S30440[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LT30p Build/9.1.A.1.145) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30441[] = "SonyLT30p Build/9.1.A.1.141 stagefright/1.2 (Linux;Android 4.1.2)";
const char S30442[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fa-; SonyLT30p Build/9.1.A.1.141) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30443[] = "Mozilla/5.0 (Linux; Android 4.1.1; DROID RAZR HD Build/9.8.1Q_39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S30444[] = "Mozilla/5.0 (Linux; Android 4.1.1; XT925 Build/9.8.0Q-97-XT925_VQU-19) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S30445[] = "Mozilla/5.0 (Linux; Android 4.1.2; RAZR HD Build/9.8.2Q-8-XT925_VQL-6) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S30446[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT925 Build/9.8.2Q-8-XT925_VQL-12.2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S30447[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S30448[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SCH-I110 Build/GINGERBREAD)";
const char S30449[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I110 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30450[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I110 Build/FE2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30451[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I110 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30452[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-I110 Build/GINGERBREAD)";
const char S30453[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I110 Build/FH1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30454[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I110 Build/FJ1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30455[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I110 Build/GA1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30456[] = "Android/1.0 (SCH-I110 GINGERBREAD)";
const char S30457[] = "DoCoMo/2.0 N905i(c100;TB;W24H16) (compatible; Googlebot-Mobile/2.1; +http://www.google.com/bot.html)";
const char S30458[] = "DoCoMo/2.0 N905i(c100;TB;W24H16) (compatible; Googlebot-Mobile/2.1;  http://www.google.com/bot.html)";
const char S30459[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19 AvantBrowser/Tri-Core";
const char S30460[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19 AvantBrowser/Tri-Core";
const char S30461[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 AvantBrowser/Tri-Core";
const char S30462[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 AvantBrowser/Tri-Core";
const char S30463[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11 AvantBrowser/Tri-Core";
const char S30464[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11 AvantBrowser/Tri-Core";
const char S30465[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 AvantBrowser/Tri-Core";
const char S30466[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 AvantBrowser/Tri-Core";
const char S30467[] = "Mozilla/5.0 (Windows NT 6.1; Avant TriCore) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S30468[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; Avant TriCore) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S30469[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; Avant TriCore) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.96 Safari/537.4";
const char S30470[] = "Mozilla/5.0 (Windows NT 6.1; Avant TriCore) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.96 Safari/537.4";
const char S30471[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; Avant TriCore) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S30472[] = "Mozilla/5.0 (Windows NT 6.1; Avant TriCore) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S30473[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; Avant TriCore) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S30474[] = "Mozilla/5.0 (Windows NT 6.1; Avant TriCore) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S30475[] = "Mozilla/5.0 (Windows NT 6.1; Avant TriCore) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S30476[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; Avant TriCore) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S30477[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; Avant TriCore) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S30478[] = "Mozilla/5.0 (Windows NT 6.1; Avant TriCore) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S30479[] = "Mozilla/5.0 (Windows NT 6.1; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36";
const char S30480[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36";
const char S30481[] = "Mozilla/5.0 (Windows NT 6.1; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36";
const char S30482[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36";
const char S30483[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36";
const char S30484[] = "Mozilla/5.0 (Windows NT 6.1; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36";
const char S30485[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30486[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; A200 Build/IML74K)";
const char S30487[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30488[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; A200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30489[] = "Opera/9.80 (Windows NT 5.2; U; en) Presto/2.10.229 Version/11.62";
const char S30490[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.229 Version/11.62";
const char S30491[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.229 Version/11.62";
const char S30492[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.229 Version/11.62";
const char S30493[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.229 Version/11.62";
const char S30494[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.10.229 Version/11.62";
const char S30495[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5288); ru) Presto/2.10.229 Version/11.62";
const char S30496[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.229 Version/11.62";
const char S30497[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4661); ru) Presto/2.10.229 Version/11.62";
const char S30498[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.229 Version/11.62";
const char S30499[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.229 Version/11.62";
const char S30500[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.229 Version/11.62";
const char S30501[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4930); ru) Presto/2.10.229 Version/11.62";
const char S30502[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.229 Version/11.62";
const char S30503[] = "Mozilla/5.0 (Windows NT 5.1; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.62";
const char S30504[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5217); ru) Presto/2.10.229 Version/11.62";
const char S30505[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5299); ru) Presto/2.10.229 Version/11.62";
const char S30506[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.229 Version/11.62";
const char S30507[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.229 Version/11.62";
const char S30508[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.229 Version/11.62";
const char S30509[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5161); ru) Presto/2.10.229 Version/11.62";
const char S30510[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5309); ru) Presto/2.10.229 Version/11.62";
const char S30511[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.229 Version/11.62";
const char S30512[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5328); ru) Presto/2.10.229 Version/11.62";
const char S30513[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.229 Version/11.62";
const char S30514[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.229 Version/11.62";
const char S30515[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.229 Version/11.62";
const char S30516[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.62";
const char S30517[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.229 Version/11.62";
const char S30518[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.62";
const char S30519[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.62";
const char S30520[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.62";
const char S30521[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5278); ru) Presto/2.10.229 Version/11.62";
const char S30522[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.229 Version/11.62";
const char S30523[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.229 Version/11.62";
const char S30524[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.229 Version/11.62";
const char S30525[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.229 Version/11.62";
const char S30526[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.62";
const char S30527[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.229 Version/11.62";
const char S30528[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5218); ru) Presto/2.10.229 Version/11.62";
const char S30529[] = "Opera/9.80 (Windows NT 5.1; U; en) Presto/2.10.229 Version/11.62,gzip(gfe) (via translate.google.com)";
const char S30530[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.229 Version/11.62";
const char S30531[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.229 Version/11.62";
const char S30532[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.229 Version/11.62";
const char S30533[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.10.229 Version/11.62";
const char S30534[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.229 Version/11.62";
const char S30535[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.229 Version/11.62";
const char S30536[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; en) Opera 11.62";
const char S30537[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.10.229 Version/11.62";
const char S30538[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5230); ru) Presto/2.10.229 Version/11.62";
const char S30539[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5300); ru) Presto/2.10.229 Version/11.62";
const char S30540[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.229 Version/11.62";
const char S30541[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.10.229 Version/11.62";
const char S30542[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.229 Version/11.62";
const char S30543[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.229 Version/11.62";
const char S30544[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.229 Version/11.62";
const char S30545[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.229 Version/11.62";
const char S30546[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5667); ru) Presto/2.10.229 Version/11.62";
const char S30547[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.10.229 Version/11.62";
const char S30548[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.10.229 Version/11.62";
const char S30549[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03672); ru) Presto/2.10.229 Version/11.62";
const char S30550[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.10.229 Version/11.62";
const char S30551[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.10.229 Version/11.62";
const char S30552[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.229 Version/11.62";
const char S30553[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.10.229 Version/11.62";
const char S30554[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.10.229 Version/11.62";
const char S30555[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02746); ru) Presto/2.10.229 Version/11.62";
const char S30556[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03399); ru) Presto/2.10.229 Version/11.62";
const char S30557[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.229 Version/11.62,gzip(gfe)";
const char S30558[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02552); ru) Presto/2.10.229 Version/11.62";
const char S30559[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02652); ru) Presto/2.10.229 Version/11.62";
const char S30560[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.229 Version/11.62";
const char S30561[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6011); ru) Presto/2.10.229 Version/11.62";
const char S30562[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.229 Version/11.62";
const char S30563[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.229 Version/11.62";
const char S30564[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.229 Version/11.62";
const char S30565[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5880); ru) Presto/2.10.229 Version/11.62";
const char S30566[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.229 Version/11.62";
const char S30567[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.229 Version/11.62";
const char S30568[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5784); ru) Presto/2.10.229 Version/11.62";
const char S30569[] = "Opera/9.80 (Windows NT 5.1; U; Edition IBIS; Trident/4.0; zh-cn) Presto/2.10.229 Version/11.62; zh-cn) Presto/2.10.229 Version/11.62)";
const char S30570[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A4449d Safari/8536.25";
const char S30571[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A465 Safari/8536.25";
const char S30572[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A4449a Safari/8536.25";
const char S30573[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A466 Safari/8536.25";
const char S30574[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A501 Safari/8536.25";
const char S30575[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A470a Safari/8536.25";
const char S30576[] = "Mozilla/5.0 (Windows NT 5.1; WOW64; rv:21.0) Gecko/X Firefox/21.0";
const char S30577[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:21.0) Gecko/X Firefox/21.0";
const char S30578[] = "Mozilla/5.0 (Windows NT 5.1; rv:21.0) Gecko/X Firefox/21.0 IceDragon/21.0.1.15";
const char S30579[] = "Mozilla/5.0 (Windows; Windows NT 5.1; rv:21.0) Gecko/21.0 Firefox/21.0";
const char S30580[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1";
const char S30581[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.3.6000 Chrome/22.0.1229.79 Safari/537.1";
const char S30582[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1";
const char S30583[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1";
const char S30584[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1,gzip(gfe)";
const char S30585[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1,gzip(gfe)";
const char S30586[] = "Opera/9.80 (X11; Linux i686; U; en) Presto/2.8.131 Version/11.11";
const char S30587[] = "Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10";
const char S30588[] = "0 Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10";
const char S30589[] = "Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:19.0) Gecko/19.0 Firefox/19.0";
const char S30590[] = "Mozilla/5.0 (Windows NT 5.2; rv:19.0) Gecko/19.0 Firefox/19.0";
const char S30591[] = "Mozilla/5.0 (Windows NT 5.1; rv:19.0) Gecko/X Firefox/19.0,gzip(gfe)";
const char S30592[] = "Mozilla/5.0(Windows; U; Windows NT 5.2; rv:1.9.2) Gecko/X Firefox/19.0";
const char S30593[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:19.0) Gecko/X MRA 5.5 (build 02842) Firefox/19.0 (.NET CLR 3.5.30729)";
const char S30594[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:19.0) Gecko/X Firefox/19.0";
const char S30595[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:13.0) Gecko/X Firefox/13.0";
const char S30596[] = "Mozilla/5.0 (Windows NT 6.0; rv:13.0) Gecko/X Firefox/13.0.1";
const char S30597[] = "Mozilla/5.0 (Windows NT 6.0; rv:13.0) Gecko/X Firefox/13.0.1 Firefox/13.0.1";
const char S30598[] = "Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:13.0) Gecko/X Firefox/13.0";
const char S30599[] = "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:13.0) Gecko/X Firefox/13.0.1 Firefox/13.0.1";
const char S30600[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Nexus One Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30601[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Nexus One Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30602[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; Nexus One Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30603[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; Nexus One Build/GRJ22)";
const char S30604[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Nexus One Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30605[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Nexus One Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30606[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Nexus One Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30607[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus One Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30608[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Nexus One Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30609[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Nexus One Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30610[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; Nexus One Build/GRI40)";
const char S30611[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; Nexus One Build/GRK39F)";
const char S30612[] = "Mozilla/5.0 (Linux; U; Android 2.4; en-gb; Nexus One Build/GRI05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30613[] = "Mozilla/5.0 (Linux; U; Android 2.4; en-us; Nexus One Build/GRH84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30614[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; Nexus One Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30615[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; Nexus One Build/MOG) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30616[] = "Mozilla/5.0 (Macintosh; Android 2.3; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30617[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en_gb; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30618[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus One Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30619[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; Nexus One Build/GRK39F)";
const char S30620[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Nexus One Build/GRH70B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30621[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Nexus One Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30622[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Nexus One Build/GRI54) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30623[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Nexus One Build/GRJ90)";
const char S30624[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; nl-; Nexus One Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30625[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en_us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30626[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Nexus One Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S30627[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Nexus One Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S30628[] = "Mozilla/5.0 (Linux; U; Android 2.3.7p0; en-us; Nexus One Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30629[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; Nexus One Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30630[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1(compatible; goggles-proxy; +http://www.google.com/mobile/goggles/)";
const char S30631[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; Nexus One Build/GRH87C) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30632[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30633[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30634[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus One Build/GRJ22) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30635[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30636[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Nexus One Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30637[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Galaxy ACE Build/JELLYBEAN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30638[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; i777 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30639[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; msm8960 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30640[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; sanstar6577_ics2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30641[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Full Android on Grouper Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30642[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-l900 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30643[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; mk808b Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30644[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; iTouch infinity-HD89+ Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30645[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; v89_gq2008s Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30646[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; v89_zlh_hd Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30647[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; redhookbay Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30648[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Q508 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30649[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; 2330 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30650[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; MAX III HD Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30651[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; Z5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30652[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; 7102 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30653[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; Aevo NOTE II+ Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30654[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; msm8974 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30655[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; p4wifi Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30656[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; Android Ativ S Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30657[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; GF-3 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30658[] = "Mozilla/5.0 (Linux; U; Android 4.2; zh-tw; ASK SP581 HD Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30659[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; HD-7 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30660[] = "OPPO_R809T/1.0 Linux/3.4.0 Android/4.2.1  Release/12.24.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2 System/Android 4.2.1;";
const char S30661[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; A19S Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30662[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; zh-cn; MT-GT-T9500/1.0 Android/4.2.0 Release/05.03.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30663[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; NOTE II PRO Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30664[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2 Build/JDQ39E)";
const char S30665[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; X508 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30666[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; zh-cn; MT-GT-T9500/1.0 Android/4.2.0 Release/05.15.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30667[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; nl-nl; MIDC490 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30668[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; B943 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30669[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; mk808 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30670[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; Colossus II Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30671[] = "Mozilla/5.0 (Linux; U; Android 4.2.1;zh-cn; ZTE V967S Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30;";
const char S30672[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; MTO MK319 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30673[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Phone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30674[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; GF-4 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30675[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; Z2 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30676[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; v89_gq3008s Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30677[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; K5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30678[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; v89_jdt1qhd Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30679[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; v89_jbl1a668 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30680[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; JB7+ Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30681[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-M919G Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30682[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GFIVE President(G9) Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30683[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; BLAZE 2.0 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30684[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; H3088 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30685[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; HX-9299A Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30686[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; MC002 Build/Retail_WGMC002) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30687[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; v89_jbl1a668_fwvga Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30688[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; P7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30689[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; v89_gq3002sc Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30690[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; Orion IV Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30691[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Z2_WVGA_ADM Build/Android4.2.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30692[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; RKM MK802IV Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30693[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; zh-cn; MT-H9082/1.0 Android/4.2.0 Release/04.26.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30694[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SM-T217A Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30695[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; Aveo NOTE PRO Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30696[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; v89_jbla768_asx Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30697[] = "HUAWEI_P6-T00_TD/1.0 Android/4.2.2 (Linux; U; Android 4.2.2; zh-cn) Release/03.20.2013 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30";
const char S30698[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; 06v20_v89_jbla768_asx Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30699[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; M6 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30700[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; IMO S89 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30701[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; W150 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30702[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; V973 Core4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30703[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; n7100 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30704[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; YOUPU U6 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30705[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; VirtualBox Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30706[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; th-th; i-mobile IQ 9 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30707[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Transformer Pad Infinity Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30708[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; HOSIN V70 Build/HOSINA728) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30709[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; R809T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30710[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; GPLUS N939 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30711[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; H920 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30712[] = "Mozilla/5.0 (Linux; U; Android 4.2; zh-tw; ASK SP553 HD Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30713[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; HUAWEI G700-U00 Build/HuaweiG700-U00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30714[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; G1 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30715[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Z6_WVGA_LC Build/Android4.2.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30716[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; itouch_HD5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30717[] = "Mozilla/5.0 (Linux; U; Android 4.2.1;zh-cn; Lenovo_K860i/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2.1 Mobile Safari/534.30";
const char S30718[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-P5220 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30719[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; A670 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30720[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; TS4 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30721[] = "Mozilla/5.0 (Linux; U; Android 4.2.1;en-us; Lenovo_S920_ROW/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2.1 Mobile Safari/534.30";
const char S30722[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; X509 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30723[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; EX_NOTE_? Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30724[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; GN708T Build/JOP40D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S30725[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; HD-89 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30726[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; ru-ru; HD Quad Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30727[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Andi4.7G COBALT Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30728[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; EX'Note IV Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30729[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; v89_gq2002s Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30730[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; N-06E Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30731[] = "Mozilla/5.0 (Linux; U; Android 4.2.1;en-us; Lenovo_S820_ROW/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2.1 Mobile Safari/534.30";
const char S30732[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-ph; MyPhone Agua Iceberg Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30733[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; A55HD Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30734[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; xt875 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30735[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; th-th; SP0728 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30736[] = "Mozilla/5.0 (Linux; U; Android 4.2.1;zh-cn; Lenovo_S820/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2.1 Mobile Safari/534.30";
const char S30737[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; DOOV S1 Build/DOOVS1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30738[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; W89 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30739[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; VOW V5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30740[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; vi-vn; VDM-HD4 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30741[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Tablet Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30742[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; Gmate G2000 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30743[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; 06_v89_jbla768_asx Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30744[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; v89_jdt1fwvga Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30745[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; AT10LE-A Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30746[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MK908 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30747[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; zh-cn; MT-GT-T9500/1.0 Android/4.2.0 Release/05.16.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30748[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Primo-G3 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30749[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; I30 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30750[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; HY5001 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30751[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; zh-cn; MT-GT-T9500/1.0 Android/4.2.0 Release/05.24.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30752[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; Forward_Prime Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30753[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; S4 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30754[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; i4000 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30755[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; X130_HD Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30756[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HUAWEI P6-T00 Build/HuaweiP6-T00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30757[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; 06_v89_jbla768_asx_s4lockscreen Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30758[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; T600 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30759[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; STB9097 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30760[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Cherry thunder2.0 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30761[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; 202K Build/100.2.2840) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30762[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; HUAWEI G610-U00 Build/HuaweiG610-U00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30763[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Atrix HD Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30764[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; S39h Build/16.0.A.0.36) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30765[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HW-HUAWEI_P6-C00/P6-C00V100R001C92B112SP01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30766[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; I9389 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30767[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HUAWEI P6-C00 Build/HuaweiP6-C00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30768[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; EveryPad Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30769[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; LESPHONE01 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30770[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; en-us; MINI-S4 Build/MINI-S4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30771[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HW-HUAWEI_P6-C00/P6-C00V100R001C92B115SP01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30772[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; Glory 2 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30773[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; SMARTLET 2 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30774[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; vivo X510t Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30775[] = "OPPO_R821T/V1 Linux/3.4.5 Android/4.2.2 Release/03.26.2013 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2 System/Android 4.2.2;";
const char S30776[] = "Mozilla/5.0 (Linux; U; Android 4.2.3; en-us; v89_jdt1fwvga Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30777[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; nl-nl; 1023Q Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30778[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; M-PP2G530 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30779[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; M470BSE Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30780[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Symphony W65 Build/UNKNOWN) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30781[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; A24 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30782[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; AUXUS Nuclea N1 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30783[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; ZTE N986 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30784[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Transformer Prime Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30785[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Full JellyBean on Mako Build/LGEP19) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30786[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; OS988 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30787[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; HD5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30788[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; i9480 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30789[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; AMOI M8448 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30790[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; ELIYA S898 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30791[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; H9700+ Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30792[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-LS980 Build/JDQ39B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2 Mobile Safari/534.30";
const char S30793[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; H9500+ Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30794[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; de-de; Q880_XK Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30795[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; UMI_S1 Build/UMI_S1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30796[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; WA520 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30797[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; G6 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30798[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; 01_v89_jbla728_asx Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30799[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; H7189 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30800[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Q5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30801[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; itouch HD5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30802[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ThL Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30803[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGT-P470 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30804[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; CW-V88-QUAD Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30805[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; es-es; MINI-S4 Build/MINI-S4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30806[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; S9 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30807[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; 11_gq2000s_fwvga Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30808[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Z1 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30809[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; HUAWEI P6-C00 Build/HuaweiP6-C00)";
const char S30810[] = "Mozilla/5.0 (Linux; U; Android 4.2.1;zh-cn; Lenovo_K860/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2.1 Mobile Safari/534.30";
const char S30811[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; A70X Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30812[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Zeepad 9XN Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30813[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; _ Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30814[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; S1 pro Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30815[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Primo_NX Build/WALTONWALTON) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30816[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; NX40X Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30817[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; Mlais MX58 Air Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30818[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Cherry Razor Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30819[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Meitu Kiss Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30820[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Symphony ZII Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30821[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC_PN072 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30822[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; i6000 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30823[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ThL W9 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30824[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; it-it; CENTURION Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30825[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; de-de; TPC-9000 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30826[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; W200 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30827[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; S5-F Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30828[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Symphony Xplorer W32 Build/UNKNOWN) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30829[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; W8s Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30830[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Vandroid Build/S5-V1.0-2013.07.18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30831[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; BLU Life View Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30832[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; CRXT 1131 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30833[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; PX2 Build/PX2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30834[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; M9 pro Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30835[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; en-gb; MID-756 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30836[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; AP-7S118 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30837[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; it-it; X50D Build/X50D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30838[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; C1059 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30839[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.2.2;dev=jflteatt;devm=SAMSUNG-SGH-I337;man=samsung;brw=Android;shwd=1080x1920;shwa=1080x1845;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S30840[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; GN708T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30841[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; V976 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30842[] = "Mozilla/5.0 (Linux; U; Android 4.2.9; zh-tw; ASK SP554 HD Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30843[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; APOLLO ONE K6589 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30844[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; ITOUCH HD89S Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30845[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; HTC_PO582 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30846[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; en-us; GT-T9500 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30847[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; vi-vn; V813 Core4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30848[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; X120Q Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30849[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; vi-vn; V811 Core4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30850[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SM-T215 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30851[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; x909 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30852[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; 01v21_v89_gq2008s Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30853[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; TCL S950 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2 Mobile Safari/534.30";
const char S30854[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; ND500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30855[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HUAWEI P6-C00 Build/HuaweiP6-C00) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S30856[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; zh-cn; MT-H9082/1.0 Android/4.2.0 Release/05.24.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30857[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; K1001L1 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30858[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; rk30mtk Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30859[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; C3 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30860[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; CX-919 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30861[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ru-ru; X-basic Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30862[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; U55GT Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30863[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-au; M470BSS Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30864[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ViewPad 100Q Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30865[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; Stylo Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30866[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ko-kr; IM-A880S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30867[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; th-th; i-mobile i-STYLE 7.3A Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30868[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Etel i4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30869[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Symphony W68 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30870[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; POMP W89 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30871[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HW-HUAWEI_P6-C00/P6-C00V100R001C92B116SP01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30872[] = "HUAWEI_G610-T00_TD/1.0 Android/4.2.1 (Linux; U; Android 4.2.1; zh-cn) Release/05.15.2013 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30";
const char S30873[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; MID9724 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30874[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Q1000S Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30875[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ASP-5000H Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30876[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-in; Micromax A240 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30877[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Symphony W82 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30878[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Full AOSP on Toroplus Build/PHONELAB_JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30879[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Maximus V.1.2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30880[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; AT10PE-A Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30881[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Lenovo S820 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30882[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ME302KL Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30883[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; ALWAY-A8 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30884[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; DROIDZ Race X Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30885[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Silver Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30886[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-tw; S510 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30887[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; RM-25 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30888[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; B1-711 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Safari/534.30";
const char S30889[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; S1S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30890[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; 9960 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30891[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-tw; Mini one Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30892[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; 9400 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30893[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; fr-fr; STAIRWAY Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30894[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; ro-ro; V1_Viper Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30895[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SM-T217S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30896[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; U8 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30897[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; COSMOS X2 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30898[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-uk; CYCLONE VOYAGER-BT Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30899[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; 01v21_v89_jbl1a698_2g Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30900[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; PMID906/PMID910 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30901[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; de-de; PAD 9719QR Build/PAD 9719QR) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30902[] = "Mozilla/5.0 (Linux; U; Android 4.2.3; es-es; v89_jdt1fwvga Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30903[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; M-PP2S500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30904[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; HS-U970 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30905[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; DT mini Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30906[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Symphony W92 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30907[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; it-it; M-MP720GO Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30908[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; X81 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30909[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; L36h Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30910[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nocturnal Google Edition CDMA 5.0 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30911[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; ARNOVA 90 G4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30912[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; SOWA S8800 Pro Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30913[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; G9589/N7102 PAD Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30914[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; K47/C9_89 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30915[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Matricom G-Box Midnight MX2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30916[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; neken N5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30917[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; imatch i26 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30918[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nitro 450QD Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30919[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; i9501 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30920[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; mini pad Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30921[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; VOVO_VII Build/MocorDroid4.2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30922[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; EX_S4 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30923[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; K1001L Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30924[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Wammy Passion Z Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30925[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; it-it; MID9035C Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30926[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; Pandora 9.21 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30927[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Akai A8 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30928[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; th-th; i-mobile IQ 9A Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30929[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; HUAWEI G610-U00 Build/HuaweiG610-U00)";
const char S30930[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; V97 HD Build/V97 HD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30931[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; sv-se; 1048QCS Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30932[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ifiveX2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30933[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; HUAWEI P6-T00 Build/HuaweiP6-T00)";
const char S30934[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; 01v21_v89_gq3008s Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30935[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; zh-cn; MT-GT-A9500/1.0 Android/4.2.0 Release/07.25.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30936[] = "Kinder-Tablet-1.0-Weltbild-Mozilla/5.0 (Linux; U; Android 4.2.2; de-DE; PEDI_PLUS_W Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30937[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Cosmos X Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30938[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; V70 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30939[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; ru-ru; Five Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30940[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; de-de; AVUS A-24 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30941[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; UM-A90 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30942[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Andi5h quadro Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30943[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; NEO-X7-216A Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30944[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; Tablet PC 2008 DC Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30945[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I7105 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30946[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ANDROIDGMAILAPP Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30947[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; nl-nl; POV_TAB-P1025(v1.0) Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30948[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; N900D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30949[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; DLX Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30950[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; OZZY Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30951[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; GT-P5200-ORANGE/P5200XXUAMFC Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30952[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; it-it; LUXURY 3 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30953[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; fr-fr; DARKSIDE Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30954[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; P2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30955[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Symphony W85 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30956[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; vi-vn; F13 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30957[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; vi-vn; Nova E Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30958[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Karbonn A8 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30959[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; th-th; i-mobile i-STYLE 7.3 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30960[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn;GiONEE-E3/ E3 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30961[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; K-one Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30962[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Newman K1 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30963[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-p6210 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30964[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; ASK SP460 HD Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30965[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; v89_gq2002sc Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30966[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; ZP958 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30967[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; ru-ru; ZTE V880H Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30968[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; A13 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30969[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Full AOSP on Maguro Build/4.2.2.632) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30970[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; TCL J620 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Versio/4.0 Mobile Safari/534.30";
const char S30971[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; PL7117 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30972[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; pt-pt; ALEXISRX5DC Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30973[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; Funker X504 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30974[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; vi-vn; F19 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30975[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; 06v20_v89_gq3009hd Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30976[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Luna TAB09-100 Build/Luna TAB09-100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30977[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Z10 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S30978[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; 03_v89_jysj_f668_sx Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30979[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; 06_v89_gq3009fwvga Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30980[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GoTab-GTD9 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30981[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; it-it; F9192 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30982[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9158 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30983[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; MOMO20-W2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30984[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GCFONE 570Q Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30985[] = "Mozilla/5.0 (Linux; U; Android 4.2.2;zh-cn; ZTE V818 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30;";
const char S30986[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Kurio4S Build/C13200US102a) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30987[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Lenovo S820_ROW Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30988[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MID0928 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30989[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; th-th; i-mobile i-STYLE 8.1 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30990[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; en-au; MID-756 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30991[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; de-de; NO1_S6 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30992[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; vivo X3t Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30993[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; S6 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30994[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; Revo Lead Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30995[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Y9190   Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30996[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Mi-510 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30997[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; vivo S7i(t) Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S30998[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R960 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S30999[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; id-id; CROSS A88 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31000[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; Unusual_U10Z Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31001[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; X40 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31002[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; fnac 3.5 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31003[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-ca; LG-D803/D803B10a Build/JDQ39B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2 Mobile Safari/534.30";
const char S31004[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Numy 3G AX1 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31005[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; TECNO P5 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31006[] = "Mozilla/5.0 (Linux; Android 4.2; Nexus 4 Build/JOP40C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S31007[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 4 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S31008[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 4 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S31009[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S31010[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S31011[] = "Mozilla/5.0 (Linux; Android 4.2.1; es-US; Nexus 4 Build/JOP40D) AppleWebKit/535.19 (KHTML like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S31012[] = "Opera/9.80 (Windows NT 6.0; WOW64) Presto/2.12.388 Version/12.14";
const char S31013[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.14";
const char S31014[] = "Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.14";
const char S31015[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.14";
const char S31016[] = "Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.14,gzip(gfe)";
const char S31017[] = "Opera/9.80 (Windows NT 6.0; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.14";
const char S31018[] = "Mozilla/5.0 (Windows NT 6.0; rv:2.0) Gecko/X Firefox/4.0 Opera 12.14";
const char S31019[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.14";
const char S31020[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.14";
const char S31021[] = "Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.14";
const char S31022[] = "Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.14";
const char S31023[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0) Opera 12.14";
const char S31024[] = "Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.14";
const char S31025[] = "Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.14";
const char S31026[] = "Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.14";
const char S31027[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.14";
const char S31028[] = "Opera/9.80 (Windows NT 6.0; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.14";
const char S31029[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.14";
const char S31030[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.14";
const char S31031[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.14";
const char S31032[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.14";
const char S31033[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.14";
const char S31034[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.14";
const char S31035[] = "Opera/9.80 (Windows NT 6.1; WOW64; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)) Presto/2.12.388 Version/12.14";
const char S31036[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.14";
const char S31037[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.14";
const char S31038[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.14";
const char S31039[] = "Opera/9.80 (Windows NT 6.0; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.14";
const char S31040[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.14";
const char S31041[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.14";
const char S31042[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.14";
const char S31043[] = "Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.14";
const char S31044[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.14";
const char S31045[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.14";
const char S31046[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.14";
const char S31047[] = "Opera/9.80 (Windows NT 6.0; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.14";
const char S31048[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.14";
const char S31049[] = "Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.14";
const char S31050[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.14";
const char S31051[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.14";
const char S31052[] = "Opera/9.80 (Windows NT 6.0; Edition Campaign 21) Presto/2.12.388 Version/12.14,gzip(gfe)";
const char S31053[] = "Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.14";
const char S31054[] = "Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.14";
const char S31055[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.14";
const char S31056[] = "Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.14";
const char S31057[] = "Opera/9.80 (Windows NT 6.1; compatible; MSIE 7.0; Windows NT 6.0) Presto/2.12.388 Version/12.14";
const char S31058[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.14";
const char S31059[] = "Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.14";
const char S31060[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.14";
const char S31061[] = "Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.14";
const char S31062[] = "Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.14";
const char S31063[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.14";
const char S31064[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.14";
const char S31065[] = "Opera/9.80 (Windows NT 6.0; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.14";
const char S31066[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 4.0)";
const char S31067[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 4.0; .NET CLR 1.0.2914)";
const char S31068[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 4.0; Mozilla/4.0 (compatable; MSIE 4.0; Windows NT; Windows 95); TX)";
const char S31069[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; TmstmpExt)";
const char S31070[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; TmstmpExt)";
const char S31071[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; TmstmpExt)";
const char S31072[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; TmstmpExt)";
const char S31073[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; TmstmpExt)";
const char S31074[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Media Center PC 6.0; TmstmpExt)";
const char S31075[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; TmstmpExt)";
const char S31076[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; TmstmpExt)";
const char S31077[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; Tablet PC 2.0; TmstmpExt)";
const char S31078[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Zune 4.7; TmstmpExt)";
const char S31079[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 5.0; Media Center PC 6.0; SLCC1; SLCC2; TmstmpExt)";
const char S31080[] = "Outlook-Express/7.0 (MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; TmstmpExt)";
const char S31081[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Avant Browser; TmstmpExt)";
const char S31082[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 5.0; Media Center PC 6.0; SLCC1; SLCC2; Tablet PC 2.0; TmstmpExt)";
const char S31083[] = "Outlook-Express/7.0 (MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; TmstmpExt)";
const char S31084[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; TmstmpExt)";
const char S31085[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; MASMJS; TmstmpExt)";
const char S31086[] = "Outlook-Express/7.0 (MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; HPDTDFJS; TmstmpExt)";
const char S31087[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; TmstmpExt)";
const char S31088[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SlimBrowser/6.01.091)";
const char S31089[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SlimBrowser/6.01.091)";
const char S31090[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Avant Browser; SlimBrowser/6.01.091)";
const char S31091[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Avant Browser; SlimBrowser/6.01.091)";
const char S31092[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SlimBrowser/6.01.091)";
const char S31093[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SlimBrowser/6.01.101)";
const char S31094[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; SlimBrowser/6.01.101)";
const char S31095[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Avant Browser; SlimBrowser/6.01.101)";
const char S31096[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Avant Browser; SlimBrowser/6.01.101)";
const char S31097[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; SlimBrowser/6.01.101)";
const char S31098[] = "Mozilla/5.0 (X11; Linux x86_64; rv:13.0) Gecko/X Firefox/13.0";
const char S31099[] = "Mozilla/5.0 (X11; Linux x86_64; rv:13.0) Gecko/X Firefox/13.0.1 Iceweasel/13.0.1";
const char S31100[] = "Mozilla/5.0 (X11; Linux i686; rv:13.0) Gecko/X Iceweasel/13.0";
const char S31101[] = "Mozilla/5.0 (X11; Linux i686; rv:13.0) Gecko/X Firefox/13.0.1 Firefox/13.0.1";
const char S31102[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B5095f Safari/7534.48.3";
const char S31103[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B5105c Safari/7534.48.3";
const char S31104[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B144 Safari/7534.48.3";
const char S31105[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B143 Safari/7534.48.3";
const char S31106[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B142 Safari/7534.48.3";
const char S31107[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B141 Safari/7534.48.3";
const char S31108[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B145 Safari/7534.48.3";
const char S31109[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B146 Safari/7534.48.3";
const char S31110[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B329 Safari/7534.48.3";
const char S31111[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B350 Safari/7534.48.3";
const char S31112[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S31113[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S31114[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-R530U Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S31115[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S31116[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S31117[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S31118[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305T Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S31119[] = "Mozilla/5.0 (Linux; Android 4.1.1; SPH-L710 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S31120[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530C Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S31121[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S31122[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S31123[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747M Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S31124[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S31125[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S31126[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S31127[] = "Mozilla/5.0 (Linux; Android 4.1.1; SPH-L710 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S31128[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305N Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S31129[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747M Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S31130[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S31131[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S31132[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S31133[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S31134[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S31135[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S31136[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S31137[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305N Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S31138[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S31139[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530C Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S31140[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-R530U Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S31141[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S31142[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305N Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S31143[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S31144[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S31145[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305T Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S31146[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; Xoom Build/IML34B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31147[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; Xoom Build/IML41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31148[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; Xoom Build/IML41B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31149[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; Xoom Build/IML52C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31150[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; Xoom Build/IML48B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31151[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; Xoom Build/IML46) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31152[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; Xoom Build/IML62B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31153[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xoom Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31154[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xoom Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31155[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Xoom Build/JRO03H)";
const char S31156[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xoom Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31157[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xoom Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31158[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xoom Build/JRO03O) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31159[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xoom Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31160[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Xoom Build/JZO54K)";
const char S31161[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; Xoom Build/JRN82) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31162[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xoom Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31163[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xoom Build/JRO03T) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31164[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xoom Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31165[] = "Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; Xoom Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.1 Safari/534.13";
const char S31166[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; Xoom Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31167[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xoom Build/JZO54L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31168[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xoom Build/JZO54M) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31169[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9330; en-US) AppleWebKit/534.8+ (KHTML, like Gecko) Version/6.0.0.719 Mobile Safari/534.8+";
const char S31170[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 920)";
const char S31171[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 920; ANZ821)";
const char S31172[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 920; Vodafone)";
const char S31173[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 920; Orange)";
const char S31174[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia 920)";
const char S31175[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; Nokia; Nokia 920T)";
const char S31176[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 920; 1326-2)";
const char S31177[] = "Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Nokia5800d-1b /11.2.008; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S31178[] = "Nokia5800/30.0.011.1 (SymbianOS/9.4; U; Series60/5.0 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S31179[] = "Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Nokia5800d-1; U; Series60/5.0 Nokia5800d-1/11.0.008; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S31180[] = "Nokia5800i/10.3.007 (SymbianOS/9.4; U; Series60/5.0; Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/413 (KHTML,like Gecko) Safari/413";
const char S31181[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5800d-1b/51.2.007; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.5.2 3gpp-gba";
const char S31182[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5800d-1/52.0.007; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.9 3gpp-gba";
const char S31183[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5800d-1/60.0.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 Mobile Safari/533.4 3gpp-gba";
const char S31184[] = "Nokia5800w/52.0.003 (SymbianOS/9.4; U; Series60/5.0; Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/413 (KHTML,like Gecko) Safari/413 3gpp-gba";
const char S31185[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5800/30.0.004; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 3gpp-gba";
const char S31186[] = "Mozilla/5.0 (X11; U; Linux i686; es-AR; rv:1.9.1.6) Gecko/X Firefox/3.6 Iceweasel/3.5.6 (like Firefox/3.5.6; Debian-3.5.6-1~bpo50+1)";
const char S31187[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-ES; rv:1.9.1.8) Gecko/X Firefox/3.0.6 (like Firefox/3.5.8)";
const char S31188[] = "Mozilla/5.0 (Linux i686; U; en; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.51";
const char S31189[] = "Mozilla/5.0 (X11; U; Linux i686; pt-BR; rv:1.9.1.6) Gecko/X Iceweasel/3.5.6 (like Firefox/3.5.6; Debian-3.5.6-1)";
const char S31190[] = "Mozilla/5.0 (X11; U; Linux x86_64; it; rv:1.9.1.16) Gecko/X Iceweasel/3.5.16 (like Firefox/3.5.16)";
const char S31191[] = "Mozilla/5.0 (X11; U; Linux i686; fr; rv:1.9.1.1) Gecko/X Firefox/3.5.1";
const char S31192[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.6) Gecko/X Firefox/3.5.6 (like Firefox/3.5.6; Debian-3.5.6-1~bpo50+1)";
const char S31193[] = "Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.1.12) Gecko/X Firefox/2.0.0.3 (like Firefox/3.5.12)";
const char S31194[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.3) Gecko/X Firefox/2.0 (like Firefox/3.5.5; Debian-3.5.5-1)";
const char S31195[] = "Mozilla/5.0 (Linux; U; pt-BR; rv:1.9.1.1) Gecko/X Firefox/3.5.1";
const char S31196[] = "Mozilla/5.0 (Linux; U;Linux X86_64; en-US; rv:1.9.1) Gecko/X Firefox/3.5";
const char S31197[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.16) Gecko/X Firefox/3.5.16 (like Firefox/3.5.16)";
const char S31198[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1.16) Gecko/X firefox/3.0.6 (like Firefox/3.5.16)";
const char S31199[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-C800 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31200[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; LG-C800G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31201[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-C800 Build/GRJ22) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S31202[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-C800 Build/GRJ22)";
const char S31203[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; LG-C800 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31204[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-C800Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31205[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;LG-C800 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31206[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-C800 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31207[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; LG-C800 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31208[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-C800 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31209[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-C800 Build/GRK39F)";
const char S31210[] = "Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; LG-C800 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31211[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us;LG-C800 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31212[] = "Mozilla/5.0 (Linux; U; Android2.3.6; en-us; LG-C800 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31213[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-C800 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31214[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-C800Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31215[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-C800G Build/GRJ22)";
const char S31216[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; LG-C800G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31217[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-C800G Build/GRK39F)";
const char S31218[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1";
const char S31219[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1,gzip(gfe)";
const char S31220[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1";
const char S31221[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.3.6000 Chrome/22.0.1229.79 Safari/537.1";
const char S31222[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.3.6000 Chrome/22.0.1229.79 Safari/537.1";
const char S31223[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1";
const char S31224[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1,gzip(gfe)";
const char S31225[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1";
const char S31226[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1";
const char S31227[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1";
const char S31228[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Maxthon/4.0 Chrome/30.0.1551.0 Safari/537.36";
const char S31229[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB526 Build/4.5.1_128) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31230[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB526 Build/4.5.1-134_DFP-67) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31231[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB526 Build/4.5.1-134_DFP-82) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31232[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; MB526 Build/4.5.1-134_DFP-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31233[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; MB526 Build/4.5.1-134_DFP-74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31234[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MB526 Build/4.5.1-134_DFP-132) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31235[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MotoMB526 Build/4.5.1-134_DFP-188) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31236[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MB526 Build/4.5.3-134_DFP-132-anzhi.com) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31237[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; he-il; MB526 Build/4.5.1-134_DFP-161) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31238[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; sv-se; MB526 Build/4.5.1-134_DFP-137) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31239[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; MB526 Build/4.5.1-134_DFP-891) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31240[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MB526 Build/4.5.1-134_DFP-1881) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31241[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; MB526 Build/4.5.1-134_DFP-1321) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31242[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; MB526 Build/4.5.1-134_DFP-125) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31243[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MB526 Build/4.5.1-134_DFP-231) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31244[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; MB526 Build/4.5.1-134_DFP-231)";
const char S31245[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; MB526 Build/4.5.2-51_DFL-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31246[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; MB526 Build/4.5.1-134_DFP-89) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31247[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; MB526 Build/4.5.2-51_DFL-50) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31248[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; MB526 Build/4.5.2-51_DFL-39) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31249[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; MotoMB526 Build/4.5.1-134_DFP-1251) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31250[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; MB526 Build/4.5.1-134_DFP-823) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31251[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MB526 Build/4.5.1-134_DFP-1321) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S31252[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; MB526 Build/4.5.1-134_DFP-821) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31253[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MotoMB526 Build/4.5.1-134_DFP-231) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31254[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MotoMB526 Build/4.5.1-134_DFP-2312) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31255[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; MB526 Build/4.5.1-134_DFP-822) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31256[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MB526 Build/4.5.1-134_DFP-2311) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31257[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MB526 Build/4.5.1-134_DFP-139) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31258[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MB526 Build/4.5.1-134_DFP-188) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31259[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; MB526 Build/4.5.1-134_DFP-1252) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31260[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB526 Build/4.5.1-134_DFP-24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31261[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MB526 Build/4.5.1-134_DFP-137) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S31262[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; MB526 Build/4.5.1-134_DFP-891)";
const char S31263[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB526 Build/4.5.1-134_DFP-31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31264[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MB526 Build/4.5.1-134_DFP-1321) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S31265[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MB526 Build/4.5.1-134_DFP-231) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S31266[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MB526 Build/4.5.1-134_DFP-231) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S31267[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; MB526 Build/4.5.1-134_DFP-146) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31268[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MB526 Build/4.5.1-134_DFP-2312) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31269[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB526 Build/4.5.2-109_DHT-25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31270[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MB526 Build/GWK74) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S31271[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; MB526 Build/4.5.2-51_DFL-50)";
const char S31272[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MB526 Build/WajkIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31273[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MB526 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S31274[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; MB526 Build/4.5.1-134_DFP-1251) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31275[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; MB526 Build/4.5.1-134_DFP-120) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31276[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; MB526 Build/GWK74)";
const char S31277[] = "Opera/9.50 (J2ME/MIDP; Opera Mini/2.1197/28.3165; U; en)";
const char S31278[] = "Opera/9.80 (MTK; Opera Mini/2.1198/29.3417; U; en) Presto/2.8.119 Version/11.10";
const char S31279[] = "Opera/9.80 (Windows NT 5.1; U; en) Presto/2.9.168 Version/11.51";
const char S31280[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.9.168 Version/11.51";
const char S31281[] = "Opera/9.80 (Windows NT 5.2; U; en) Presto/2.9.168 Version/11.51";
const char S31282[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.9.168 Version/11.51";
const char S31283[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.9.168 Version/11.51";
const char S31284[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.9.168 Version/11.51";
const char S31285[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.9.168 Version/11.51,gzip(gfe) (via translate.google.com)";
const char S31286[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.9.168 Version/11.51";
const char S31287[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.9.168 Version/11.51";
const char S31288[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.9.168 Version/11.51";
const char S31289[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.9.168 Version/11.51";
const char S31290[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.9.168 Version/11.51";
const char S31291[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.9.168 Version/11.51";
const char S31292[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.9.168 Version/11.51";
const char S31293[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.9.168 Version/11.51";
const char S31294[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94; ru) Presto/2.9.168 Version/11.51";
const char S31295[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4841); ru) Presto/2.9.168 Version/11.51";
const char S31296[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.9.168 Version/11.51";
const char S31297[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.9.168 Version/11.51";
const char S31298[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.9.168 Version/11.51";
const char S31299[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.9.168 Version/11.51";
const char S31300[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.9.168 Version/11.51";
const char S31301[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.9.168 Version/11.51";
const char S31302[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.9.168 Version/11.51";
const char S31303[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4832); ru) Presto/2.9.168 Version/11.51";
const char S31304[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4871); ru) Presto/2.9.168 Version/11.51";
const char S31305[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.9.168 Version/11.51";
const char S31306[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.9.168 Version/11.51";
const char S31307[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.9.168 Version/11.51";
const char S31308[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.9.168 Version/11.51";
const char S31309[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; ru) Opera 11.51";
const char S31310[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4661); ru) Presto/2.9.168 Version/11.51";
const char S31311[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.9.168 Version/11.51";
const char S31312[] = "Mozilla/5.0 (Windows NT 5.1; ru; rv:2.0) Gecko/X Firefox/4.0 Opera 11.51";
const char S31313[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03791); ru) Presto/2.9.168 Version/11.51";
const char S31314[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.9.168 Version/11.51";
const char S31315[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.9.168 Version/11.51";
const char S31316[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.9.168 Version/11.51";
const char S31317[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.9.168 Version/11.51";
const char S31318[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.9.168 Version/11.51";
const char S31319[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.9.168 Version/11.51";
const char S31320[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.9.168 Version/11.51";
const char S31321[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.9.168 Version/11.51";
const char S31322[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.9.168 Version/11.51";
const char S31323[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.9.168 Version/11.51";
const char S31324[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.9.168 Version/11.51,gzip(gfe)";
const char S31325[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.9.168 Version/11.51";
const char S31326[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.9.168 Version/11.51";
const char S31327[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.9.168 Version/11.51";
const char S31328[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.9.168 Version/11.51";
const char S31329[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.9.168 Version/11.51";
const char S31330[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.9.168 Version/11.51";
const char S31331[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.9.168 Version/11.51";
const char S31332[] = "Mozilla/5.0 (X11; Linux x86_64; rv:15.0) Gecko/X Firefox/15.0.1 Iceweasel/15.0.1";
const char S31333[] = "Mozilla/5.0 (X11; Linux x86_64; rv:15.0) Gecko/X Firefox/15.0.1";
const char S31334[] = "Mozilla/5.0 (Ultimate Edition; X11; Linux x86; rv:8.0) Gecko/X Ultimate Edition / 3.5 Firefox/15.0.1";
const char S31335[] = "Mozilla/5.0 (X11; Linux i686; rv:15.0) Gecko/X Firefox/15.0.1 Firefox/15.0.1";
const char S31336[] = "Mozilla/1.2 (Mozilla/5.0 (X11; Linux i686; rv:15.0) Gecko/X Firefox/15.0)";
const char S31337[] = "Mozilla/5.0 (X11; Linux x86_64; rv:15.0) Gecko/X Debian Iceweasel/15.02";
const char S31338[] = "Mozilla/5.0 (Windows NT 5.1; rv:18.0) Gecko/18.0 Firefox/18.0";
const char S31339[] = "Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:18.0) Gecko/18.0 Firefox/18.0";
const char S31340[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:18.0) Gecko/X Firefox/18.0";
const char S31341[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.1) Gecko/X Firefox/18.0.0.1[en]C-CCK-MCD DOJ3jx7bf";
const char S31342[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.60";
const char S31343[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4930); ru) Presto/2.10.229 Version/11.60";
const char S31344[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.10.229 Version/11.60";
const char S31345[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.229 Version/11.60";
const char S31346[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.229 Version/11.60";
const char S31347[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.229 Version/11.60";
const char S31348[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.229 Version/11.60";
const char S31349[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.229 Version/11.60";
const char S31350[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.229 Version/11.60";
const char S31351[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.229 Version/11.60";
const char S31352[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.229 Version/11.60";
const char S31353[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.229 Version/11.60";
const char S31354[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.229 Version/11.60";
const char S31355[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.60";
const char S31356[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.229 Version/11.60";
const char S31357[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4110); ru) Presto/2.10.229 Version/11.60";
const char S31358[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.229 Version/11.60";
const char S31359[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.229 Version/11.60";
const char S31360[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4661); ru) Presto/2.10.229 Version/11.60";
const char S31361[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.60";
const char S31362[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03649); ru) Presto/2.10.229 Version/11.60";
const char S31363[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03638); ru) Presto/2.10.229 Version/11.60";
const char S31364[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.229 Version/11.60";
const char S31365[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.60";
const char S31366[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.229 Version/11.60";
const char S31367[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.229 Version/11.60";
const char S31368[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.229 Version/11.60";
const char S31369[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.229 Version/11.60";
const char S31370[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.229 Version/11.60,gzip(gfe) (via translate.google.com)";
const char S31371[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.60";
const char S31372[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.229 Version/11.60";
const char S31373[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.229 Version/11.60";
const char S31374[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.229 Version/11.60";
const char S31375[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03755); ru) Presto/2.10.229 Version/11.60";
const char S31376[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.229 Version/11.60";
const char S31377[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.229 Version/11.60";
const char S31378[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; ru) Opera 11.60";
const char S31379[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03278); ru) Presto/2.10.229 Version/11.60";
const char S31380[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03639); ru) Presto/2.10.229 Version/11.60";
const char S31381[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.229 Version/11.60";
const char S31382[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.229 Version/11.60";
const char S31383[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03672); ru) Presto/2.10.229 Version/11.60";
const char S31384[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03402); ru) Presto/2.10.229 Version/11.60";
const char S31385[] = "Mozilla/5.0 (Windows NT 6.1; ru; rv:2.0) Gecko/X Firefox/4.0 Opera 11.60";
const char S31386[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.229 Version/11.60";
const char S31387[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.229 Version/11.60";
const char S31388[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.229 Version/11.60";
const char S31389[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5218); ru) Presto/2.10.229 Version/11.60";
const char S31390[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5328); ru) Presto/2.10.229 Version/11.60";
const char S31391[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.229 Version/11.60";
const char S31392[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.229 Version/11.60";
const char S31393[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.10.229 Version/11.60";
const char S31394[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.10.229 Version/11.60";
const char S31395[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.229 Version/11.60";
const char S31396[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.10.229 Version/11.60";
const char S31397[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03780); ru) Presto/2.10.229 Version/11.60";
const char S31398[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5667); ru) Presto/2.10.229 Version/11.60";
const char S31399[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5230); ru) Presto/2.10.229 Version/11.60";
const char S31400[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.229 Version/11.60";
const char S31401[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.229 Version/11.60,gzip(gfe)";
const char S31402[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5966); ru) Presto/2.10.229 Version/11.60";
const char S31403[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.229 Version/11.60";
const char S31404[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.229 Version/11.60";
const char S31405[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.229 Version/11.60";
const char S31406[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.229 Version/11.60";
const char S31407[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.229 Version/11.60";
const char S31408[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.229 Version/11.60";
const char S31409[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5745); ru) Presto/2.10.229 Version/11.60";
const char S31410[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5880); ru) Presto/2.10.229 Version/11.60";
const char S31411[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.229 Version/11.60";
const char S31412[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5784); ru) Presto/2.10.229 Version/11.60";
const char S31413[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.229 Version/11.60";
const char S31414[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5676); ru) Presto/2.10.229 Version/11.60";
const char S31415[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6089); ru) Presto/2.10.229 Version/11.60";
const char S31416[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6011); ru) Presto/2.10.229 Version/11.60";
const char S31417[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6054); ru) Presto/2.10.229 Version/11.60";
const char S31418[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5977); ru) Presto/2.10.229 Version/11.60";
const char S31419[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6083); ru) Presto/2.10.229 Version/11.60";
const char S31420[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.229 Version/11.60";
const char S31421[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6081); ru) Presto/2.10.229 Version/11.60";
const char S31422[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5981); ru) Presto/2.10.229 Version/11.60";
const char S31423[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 6008); ru) Presto/2.10.229 Version/11.60";
const char S31424[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/15";
const char S31425[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S31426[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S31427[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.106 Safari/535.2,gzip(gfe) (via translate.google.com)";
const char S31428[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2,gzip(gfe) (via translate.google.com)";
const char S31429[] = "Chrome/15.0.860.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/15.0.860.0";
const char S31430[] = "Chrome/15.0.860.0 (Windows; U; Windows NT 6.0; es-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/15.0.860.0";
const char S31431[] = "Chrome/15.0.860.0 (Windows; U; Windows NT 6.0; fr-FR) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/15.0.860.0";
const char S31432[] = "Chrome/15.0.860.0 (Windows; U; Windows NT 6.0; en-CA) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/15.0.860.0";
const char S31433[] = "Chrome/15.0.860.0 (Windows; U; Windows NT 6.0; haw-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/15.0.860.0";
const char S31434[] = "Chrome/15.0.860.0 (Windows; U; Windows NT 6.0; fr-CA) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/15.0.860.0";
const char S31435[] = "Chrome/15.0.860.0 (Windows; U; Windows NT 6.0; zh-CN) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/15.0.860.0";
const char S31436[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.2 (KHTML like Gecko) Chrome/15.0.860.0 Safari/535.2Chrome/15.0.860.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.20.25 (KHTML like Gecko) Version/15.0.860.0";
const char S31437[] = "Chrome/15.0.860.0 (Windows; U; Windows NT 6.0; pl-DE) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/15.0.860.0";
const char S31438[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en-US; rv:1.9.2.28) Gecko/X Firefox/3.6.28";
const char S31439[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; LG-P500/V20d Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31440[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; uk-ua; LG-P500 Build/GRI40) AppleWebKit/528.5+ (KHTML, O: Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S31441[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; LG-P500 Build/AGA-ROM) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31442[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; LG-P500 Build/openOptimus build 2.202) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31443[] = "Mozilla/5.0 (Linux; U; Android 2.3.3 by Snegovik; ru-ru; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31444[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-sa; LG-P500h/v20a Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31445[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; LG-P500/v20g Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31446[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; LG-P500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31447[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31448[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31449[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; LG-P500/v20h Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31450[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31451[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31452[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; LG-P500 Build/GRI40)";
const char S31453[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P500 Build/GRK39F; Oxygen-2.3.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31454[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; LG-P500h Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31455[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-mx; LG-P500h/v10a Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31456[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; LG-P500h Build/GRI40)";
const char S31457[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; LG-P500h-parrot Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31458[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; LG-P500h-parrot Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31459[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; LG-P500 Build/GRI40)";
const char S31460[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-in; LG-P500 Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S31461[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; LG-P500/V20i Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31462[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; LG-P500h Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31463[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P500 Build/GINGERBREAD)";
const char S31464[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; LG Optimus One P500 Build/openOptimus build 2.202) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31465[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; LG-P500-Bouygues/ Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31466[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; LG-P500 Build/GRI40)";
const char S31467[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P500 Build/GINGERBREAD; Oxygen-2.3.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31468[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-mx; LG-P500h Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Versi?n/4.0 Mobile Safari/533.1";
const char S31469[] = "Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.15";
const char S31470[] = "Opera/9.80 (Windows NT 6.0; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.15";
const char S31471[] = "Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.15";
const char S31472[] = "Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.15,gzip(gfe)";
const char S31473[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.15";
const char S31474[] = "Opera/9.80 (Windows NT 6.0; WOW64) Presto/2.12.388 Version/12.15";
const char S31475[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.15";
const char S31476[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.15";
const char S31477[] = "Mozilla/5.0 (Windows NT 6.0; rv:2.0) Gecko/X Firefox/4.0 Opera 12.15";
const char S31478[] = "Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.15";
const char S31479[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.15";
const char S31480[] = "Opera/9.80 (Windows NT 6.1; WOW64; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)) Presto/2.12.388 Version/12.15";
const char S31481[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.15";
const char S31482[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.15";
const char S31483[] = "Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.15";
const char S31484[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.15";
const char S31485[] = "Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.15";
const char S31486[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.15";
const char S31487[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.15";
const char S31488[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.15";
const char S31489[] = "Opera/9.80 (Windows NT 6.0; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.15";
const char S31490[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.15";
const char S31491[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.15";
const char S31492[] = "Opera/9.80 (Windows NT 6.0; Edition Campaign 21) Presto/2.12.388 Version/12.15,gzip(gfe)";
const char S31493[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.15";
const char S31494[] = "Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.15";
const char S31495[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.15";
const char S31496[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.15";
const char S31497[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.15";
const char S31498[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.15";
const char S31499[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0) Opera 12.15";
const char S31500[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.15";
const char S31501[] = "Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.15";
const char S31502[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.15";
const char S31503[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.15";
const char S31504[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.15";
const char S31505[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.15";
const char S31506[] = "Opera/9.80 (Windows NT 6.0; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.15";
const char S31507[] = "Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.15";
const char S31508[] = "Opera/9.80 (Windows NT 6.0; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.15";
const char S31509[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5997)) Presto/2.12.388 Version/12.15";
const char S31510[] = "Opera/9.80 (Windows NT 6.1; compatible; MSIE 7.0; Windows NT 6.0) Presto/2.12.388 Version/12.15";
const char S31511[] = "Opera/9.80 (Windows NT 6.0; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.15";
const char S31512[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.15";
const char S31513[] = "Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.15";
const char S31514[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.15";
const char S31515[] = "Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.15";
const char S31516[] = "Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.15";
const char S31517[] = "Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.15";
const char S31518[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.15";
const char S31519[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.15";
const char S31520[] = "Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.15";
const char S31521[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.15";
const char S31522[] = "Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.15";
const char S31523[] = "Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.15";
const char S31524[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.15";
const char S31525[] = "Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.15";
const char S31526[] = "Opera/9.80 (Windows NT 6.0; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.15";
const char S31527[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.15";
const char S31528[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; TmstmpExt)";
const char S31529[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; TmstmpExt)";
const char S31530[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; TmstmpExt)";
const char S31531[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.0; Trident/4.0; Mozilla/4.0 ; MSIE 6.0; Windows NT 5.1; SV1; ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; TmstmpExt)";
const char S31532[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; TmstmpExt)";
const char S31533[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7; TmstmpExt)";
const char S31534[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7; TmstmpExt)";
const char S31535[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; .NET4.0C; .NET CLR 3.5.30729; TmstmpExt)";
const char S31536[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Tablet PC 2.0; TmstmpExt)";
const char S31537[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST25i Build/6.1.1.B.1.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31538[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyEricssonST25a Build/6.1.1.B.1.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31539[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST25i Build/6.1.1.B.1.10)";
const char S31540[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonST25i Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31541[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; SonyEricssonST25iv Build/6.1.1.B.1.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31542[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SonyEricssonST25i-o Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31543[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST25i Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31544[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST25a Build/6.1.1.B.1.10)";
const char S31545[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST25a Build/6.1.1.B.1.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31546[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; ST25a Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31547[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST25i Build/6.1.1.C.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31548[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; ST25i Build/6.1.1.B.1.89) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31549[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ST25i Build/6.1.1.B.1.100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31550[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST26i Build/11.0.A.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31551[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST26i Build/11.0.A.3.18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31552[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST26i Build/11.0.A.3.18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31553[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST26i Build/11.0.A.3.28) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31554[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; SonyST26i-o Build/11.0.A.3.18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31555[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SonyST26i Build/11.0.A.3.22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31556[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; SonyST26a Build/11.0.A.3.18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31557[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; SonyST26iv Build/11.0.A.3.28) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31558[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST26i Build/11.0.A.3.28) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31559[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST26i Build/11.0.A.7.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31560[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; ST26i Build/11.0.A.3.22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31561[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST26i Build/11.0.A.7.5)";
const char S31562[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST26i Build/11.0.A.7.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31563[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; SonyST26i-o Build/11.0.A.7.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31564[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; SonyST26a Build/11.0.A.7.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31565[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST26i Build/11.0.A.7.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31566[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST26i Build/11.0.A.7.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31567[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; SonyST26iv Build/11.0.A.7.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31568[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; SonyST26iv Build/11.0.A.3.18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31569[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; iw-il; SonyST26a Build/11.0.A.3.28) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31570[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-mx; SonyST26a Build/11.0.A.7.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31571[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; ST26a Build/11.0.A.7.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31572[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST26a Build/11.0.A.7.5)";
const char S31573[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-mx; ST26a Build/11.0.A.7.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31574[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; ST26a Build/11.0.A.3.28) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31575[] = "SonyST26i Build/11.0.A.7.5 stagefright/1.2 (Linux;Android 4.0.4)";
const char S31576[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8500M/S8500MUGJF5; U; Bada/1.0; en-ca) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WVGA SMM-MMS/1.2.0 OPN-B";
const char S31577[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8500B/1.0; U; Bada/1.0; pt-br) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WVGA SMM-MMS/1.2.0 OPN-B";
const char S31578[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8500/S8500XXJEE; U; Bada/1.0; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WVGA SMM-MMS/1.2.0 OPN-B";
const char S31579[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.14";
const char S31580[] = "Opera/9.80 (Windows NT 6.2) Presto/2.12.388 Version/12.14";
const char S31581[] = "Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.14";
const char S31582[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.14";
const char S31583[] = "Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.14,gzip(gfe)";
const char S31584[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.14";
const char S31585[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.14";
const char S31586[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.14";
const char S31587[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.14";
const char S31588[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.14";
const char S31589[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.14";
const char S31590[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.14";
const char S31591[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.14";
const char S31592[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.14";
const char S31593[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.14";
const char S31594[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.14";
const char S31595[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.14";
const char S31596[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.14";
const char S31597[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.14";
const char S31598[] = "Opera/9.80 (Windows NT 6.2) Presto/2.12.388 Version/12.14,gzip(gfe)";
const char S31599[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.14";
const char S31600[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.14";
const char S31601[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.14";
const char S31602[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64) Opera 12.14";
const char S31603[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.14";
const char S31604[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.14";
const char S31605[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.14";
const char S31606[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.14";
const char S31607[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.14";
const char S31608[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.14";
const char S31609[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.14";
const char S31610[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.14";
const char S31611[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.14";
const char S31612[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.14";
const char S31613[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.14";
const char S31614[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.14";
const char S31615[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5997)) Presto/2.12.388 Version/12.14";
const char S31616[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6080)) Presto/2.12.388 Version/12.14";
const char S31617[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.14";
const char S31618[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.14";
const char S31619[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.14";
const char S31620[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6035)) Presto/2.12.388 Version/12.14";
const char S31621[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.14";
const char S31622[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.14";
const char S31623[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.14";
const char S31624[] = "Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.16";
const char S31625[] = "Opera/9.80 (Windows NT 6.2) Presto/2.12.388 Version/12.16";
const char S31626[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.16";
const char S31627[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.16";
const char S31628[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.16";
const char S31629[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.16";
const char S31630[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.16";
const char S31631[] = "Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.16,gzip(gfe)";
const char S31632[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6657)) Presto/2.12.388 Version/12.16";
const char S31633[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.16";
const char S31634[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.16";
const char S31635[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6283)) Presto/2.12.388 Version/12.16";
const char S31636[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.16";
const char S31637[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6322)) Presto/2.12.388 Version/12.16";
const char S31638[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.16";
const char S31639[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.16";
const char S31640[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.16";
const char S31641[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6665)) Presto/2.12.388 Version/12.16";
const char S31642[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.16";
const char S31643[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6638)) Presto/2.12.388 Version/12.16";
const char S31644[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.16";
const char S31645[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.16";
const char S31646[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.16";
const char S31647[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6573)) Presto/2.12.388 Version/12.16";
const char S31648[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6327)) Presto/2.12.388 Version/12.16";
const char S31649[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.16";
const char S31650[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6648)) Presto/2.12.388 Version/12.16";
const char S31651[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.16";
const char S31652[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.16";
const char S31653[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.16";
const char S31654[] = "Opera/9.80 (Windows NT 6.2) Presto/2.12.388 Version/12.16,gzip(gfe)";
const char S31655[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6337)) Presto/2.12.388 Version/12.16";
const char S31656[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6628)) Presto/2.12.388 Version/12.16";
const char S31657[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.16";
const char S31658[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6323)) Presto/2.12.388 Version/12.16";
const char S31659[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.16";
const char S31660[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6341)) Presto/2.12.388 Version/12.16";
const char S31661[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64) Opera 12.16";
const char S31662[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6673)) Presto/2.12.388 Version/12.16";
const char S31663[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6675)) Presto/2.12.388 Version/12.16";
const char S31664[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.16";
const char S31665[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.16";
const char S31666[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6325)) Presto/2.12.388 Version/12.16";
const char S31667[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.16";
const char S31668[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.16";
const char S31669[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.16";
const char S31670[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.16";
const char S31671[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6620)) Presto/2.12.388 Version/12.16";
const char S31672[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6669)) Presto/2.12.388 Version/12.16";
const char S31673[] = "Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.16,gzip(gfe),gzip(gfe)";
const char S31674[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.16";
const char S31675[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6261)) Presto/2.12.388 Version/12.16";
const char S31676[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6676)) Presto/2.12.388 Version/12.16";
const char S31677[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6681)) Presto/2.12.388 Version/12.16";
const char S31678[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6347)) Presto/2.12.388 Version/12.16";
const char S31679[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.16";
const char S31680[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.16";
const char S31681[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.16";
const char S31682[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.16";
const char S31683[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.16";
const char S31684[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6336)) Presto/2.12.388 Version/12.16";
const char S31685[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.16";
const char S31686[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6561)) Presto/2.12.388 Version/12.16";
const char S31687[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.16";
const char S31688[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.2 (build 7188)) Presto/2.12.388 Version/12.16";
const char S31689[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.2 (build 7194)) Presto/2.12.388 Version/12.16";
const char S31690[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9100P Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31691[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9100P-ORANGE/I9100PBVKI3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31692[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100P Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31693[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9100P-ORANGE/I9100PBVKL2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31694[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-I9100P Build/GINGERBREAD)";
const char S31695[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-I9100P Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31696[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9100P Build/GINGERBREAD)";
const char S31697[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-I9100P Build/GINGERBREAD)";
const char S31698[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-I9100P Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31699[] = "Mozilla/5.0 (Unknown; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.9.0 Safari/534.34";
const char S31700[] = "Mozilla/5.0 (Unknown; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.9.0 Safari/534.34";
const char S31701[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:15.0) Gecko/15.0 Firefox/15.0a1";
const char S31702[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:15.0) Gecko/X Firefox/15.0.1";
const char S31703[] = "Mozilla/5.0 (Linux; U; Android 2.2.1 Community 2; es-es; HTC_Wildfire_A3333 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31704[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pt-; HTC_Wildfire_A3333 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31705[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC_Wildfire_A3333 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31706[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows 95)";
const char S31707[] = "Mozilla/4.0 (compatible; MSIE 5.50; Windows 95; SiteKiosk 4.8)";
const char S31708[] = "Mozilla/5.0 (Linux; Android 4.1.1; XT907 Build/9.8.1Q_27-2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S31709[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT905 Build/9.8.2Q_SMUIRD-7) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S31710[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-66) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S31711[] = "Opera/9.80 (MAUI Runtime; Opera Mini/6.1.27412/31.1475; U; en) Presto/2.8.119 Version/11.10";
const char S31712[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U8815 Build/HuaweiU8815) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31713[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; U8815 Build/HuaweiU8815C02B882) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31714[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; U8815 Build/HuaweiU8815C02B885) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31715[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U8818 Build/HuaweiU8818) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31716[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-nz; U8815 Build/HuaweiU8815C02B884) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31717[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ie; U8815 Build/HuaweiU8815C02B886) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31718[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; U8815N Build/HuaweiU8815NC02B891) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31719[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; U8815 Build/HuaweiU8815C02B892) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31720[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; U8815 Build/HuaweiU8815C346B893) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31721[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; HUAWEI_Ascend G 300 Build/HuaweiU8815) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31722[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; U8815 Build/r0_de-branded_u8815) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31723[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ie; U8815 Build/HuaweiU8815C02B893) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31724[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; U8818 Build/HuaweiU8818) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S31725[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; U8815 Build/HuaweiU8815)";
const char S31726[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; U8815 Build/AtomicMod BX (By Tillaz)) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31727[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; U8815N Build/HuaweiU8815NC02B894) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31728[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; U8815 Build/HuaweiU8815C02B895) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31729[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; U8815 Build/HuaweiU8815C346B900) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31730[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; U8815 Build/HuaweiU8815C346B801) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31731[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; U8818 Build/HuaweiU8818)";
const char S31732[] = "Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:19.0) Gecko/X Firefox/19.0";
const char S31733[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31734[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5670 Build/GINGERBREAD)";
const char S31735[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT-S5670L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31736[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; hr-hr; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31737[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31738[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5670 Build/GINGERBREAD)";
const char S31739[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; GT-S5670L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31740[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5670 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S31741[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-S5670L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31742[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-S5670 Build/GINGERBREAD)";
const char S31743[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5670 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S31744[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-S5670B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31745[] = "Mozilla/5.0 (Linux; U; Android 2.3.6(RazoDroiD); en-gb; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31746[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5670L Build/GINGERBREAD)";
const char S31747[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5670 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S31748[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31749[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31750[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31751[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5670L Build/GINGERBREAD)";
const char S31752[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Amaze_4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31753[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_Amaze_4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31754[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC_Amaze_4G Build/IML74K)";
const char S31755[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; HTC Ruby Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31756[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; HTC_Ruby Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31757[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; HTC Amaze 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31758[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Ruby Build/IML74K)";
const char S31759[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Amaze 4G Build/IML74K)";
const char S31760[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Ruby Build/Speed Rom 7.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31761[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Ruby Build/Speed Rom 7.8 By EclipzeRemix) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31762[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_Amaze_4G Build/JellybeanFusion) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31763[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Ruby Build/Speed Rom 8.0 By EclipzeRemix) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31764[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Amaze 4G Build/Fourth Bar: Infusion) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31765[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Ruby Build/Speed Rom 7.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31766[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Amaze Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31767[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_Amaze_4g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31768[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Ruby Build/Speed Rom 9.0 Beta1 By EclipzeRemix) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31769[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Amaze 4g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31770[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; es-es; Amaze_4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31771[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Amaze S400 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31772[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_Amaze_4G Build/Kaioken Rom by SuperAfnan) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31773[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P769 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31774[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; LG-P760/V10a Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31775[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; LG-P760 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31776[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-P768 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31777[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; LG-P760/V10c Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31778[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P769 Build/IMM76L)";
const char S31779[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; LG-P760/P76010e Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31780[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P760 Build/IMM76L)";
const char S31781[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; LG-P760/V10f Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31782[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; LG-P760/P76010a Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31783[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P768 Build/IMM76L)";
const char S31784[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; LG-P760/V10b Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31785[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; LG-P760/P76010h Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31786[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sr; LG-P760 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31787[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it; LG-P769 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31788[] = "Mozilla/5.0 (Linux; Android 4.1.1; DROID RAZR HD Build/9.8.1Q_39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36";
const char S31789[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-79) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S31790[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT925 Build/9.8.2Q-8-XT925_VQL-12.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S31791[] = "Mozilla/5.0 (Linux; Android 4.1.1; XT925 Build/9.8.0Q-97-XT925_VQU-19) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S31792[] = "FeedFetcher-Google; (+http://www.google.com/feedfetcher.html)";
const char S31793[] = "Mozilla/5.0 (compatible) Feedfetcher-Google; (+http://www.google.com/feedfetcher.html)";
const char S31794[] = "Mozilla/5.0 (compatible) Feedfetcher-Google;(+http://www.google.com/feedfetcher.html)";
const char S31795[] = "Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 2 subscribers; feed-id=X)";
const char S31796[] = "Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 16 subscribers; feed-id=X)";
const char S31797[] = "Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 11 subscribers; feed-id=X)";
const char S31798[] = "Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 61 subscribers; feed-id=X)";
const char S31799[] = "Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 12 subscribers; feed-id=X)";
const char S31800[] = "Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 150 subscribers; feed-id=X)";
const char S31801[] = "Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 39 subscribers; feed-id=X)";
const char S31802[] = "Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 203 subscribers; feed-id=X)";
const char S31803[] = "Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 70 subscribers; feed-id=X)";
const char S31804[] = "Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 14 subscribers; feed-id=X)";
const char S31805[] = "Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 59 subscribers; feed-id=X)";
const char S31806[] = "Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 2 subscribers; feed-id=X) AppEngine-Google; (+http://code.google.com/appengine; appid: voxelshader)";
const char S31807[] = "Feedly/1.0 (+http://www.feedly.com/fetcher.html; like FeedFetcher-Google)";
const char S31808[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9105P Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31809[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I9105P/I9105PXXALK4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31810[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9105 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31811[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9105P Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31812[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9105P Build/JZO54K)";
const char S31813[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9105 Build/JZO54K)";
const char S31814[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-at; SAMSUNG GT-I9105P/I9105PXXAMC2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31815[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I9105P/I9105PXXAMB6 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31816[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; GT-I9105P-ORANGE/I9105PXXAMBA Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31817[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; sl-si; SAMSUNG GT-I9105P/I9105PXXAMA2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31818[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ro-ro; GT-I9105P-ORANGE/I9105PXXAMD1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31819[] = "Mozilla/5.0 (Linux; U; Android 4.1.2_M.S; he-il; GT-I9105 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31820[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P970 Build/GRJ22)";
const char S31821[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; is; LG-P970 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31822[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; LG-P970g-parrot Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31823[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; LG-P970-Orange Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31824[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LG-P970 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31825[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P970h Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31826[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; LG-P970/V20o Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31827[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; LG-P970g Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31828[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; LG-P970-Orange/V20g Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31829[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LG-P970 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S31830[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LG-P970/V20b Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31831[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; LG-P970/V20c Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31832[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; LG-P970-Bouygues Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31833[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LG-P970 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S31834[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; LG-P970/$(Version) Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31835[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-pt; LG-P970/V20a Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31836[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P970g Build/GRJ22)";
const char S31837[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P970h Build/GRJ22)";
const char S31838[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; LG-P970-Orange/V20h Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31839[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; LG-P970/V20e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31840[] = "LG-P970/ Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S31841[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P970 Build/GWK74; CiCi-v3.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31842[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; LG-P970h Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Versi?n/4.0 Mobile Safari/533.1";
const char S31843[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.2.15 Version/10.10";
const char S31844[] = "Opera/9.80 (Windows NT 5.2; U; en) Presto/2.2.15 Version/10.10";
const char S31845[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.2.15 Version/10.10";
const char S31846[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.2.15 Version/10.10";
const char S31847[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.2.15 Version/10.10";
const char S31848[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.2.15 Version/10.10";
const char S31849[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.2.15 Version/10.10,gzip(gfe) (via translate.google.com)";
const char S31850[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.2.15 Version/10.10";
const char S31851[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.2.15 Version/10.10";
const char S31852[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03638); ru) Presto/2.2.15 Version/10.10";
const char S31853[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.2.15 Version/10.10";
const char S31854[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.2.15 Version/10.10";
const char S31855[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.2.15 Version/10.10";
const char S31856[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.2.15 Version/10.10";
const char S31857[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.2.15 Version/10.10";
const char S31858[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.2.15 Version/10.10";
const char S31859[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.2.15 Version/10.10";
const char S31860[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.2.15 Version/10.10";
const char S31861[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; ru) Opera 10.10";
const char S31862[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.2.15 Version/10.10";
const char S31863[] = "Mozilla/5.0 (Windows NT 5.1; U; YB/3.5.1; ru; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 10.10";
const char S31864[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.2.15 Version/10.10";
const char S31865[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.2.15 Version/10.10";
const char S31866[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.2.15 Version/10.10";
const char S31867[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.2.15 Version/10.10";
const char S31868[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5267); ru) Presto/2.2.15 Version/10.10";
const char S31869[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.2.15 Version/10.10";
const char S31870[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5328); ru) Presto/2.2.15 Version/10.10";
const char S31871[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5308); ru) Presto/2.2.15 Version/10.10";
const char S31872[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5288); ru) Presto/2.2.15 Version/10.10";
const char S31873[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.2.15 Version/10.10";
const char S31874[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.2.15 Version/10.10";
const char S31875[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.2.15 Version/10.10";
const char S31876[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.2.15 Version/10.10";
const char S31877[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.2.15 Version/10.10";
const char S31878[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.2.15 Version/10.10";
const char S31879[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.2.15 Version/10.10";
const char S31880[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.2.15 Version/10.10";
const char S31881[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.2.15 Version/10.10";
const char S31882[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.2.15 Version/10.10";
const char S31883[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4832); ru) Presto/2.2.15 Version/10.10";
const char S31884[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.2.15 Version/10.10";
const char S31885[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4930); ru) Presto/2.2.15 Version/10.10";
const char S31886[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.2.15 Version/10.10";
const char S31887[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.2.15 Version/10.10,gzip(gfe)";
const char S31888[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.2.15 Version/10.10";
const char S31889[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.2.15 Version/10.10";
const char S31890[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.2.15 Version/10.10";
const char S31891[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5966); ru) Presto/2.2.15 Version/10.10";
const char S31892[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.2.15 Version/10.10";
const char S31893[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.2.15 Version/10.10";
const char S31894[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6089); ru) Presto/2.2.15 Version/10.10";
const char S31895[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6578); ru) Presto/2.2.15 Version/10.10";
const char S31896[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:16.0) Gecko/X Firefox/16.0";
const char S31897[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:16.0) Gecko/X Firefox/16.0";
const char S31898[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; rv:16.0.1) Gecko/X Firefox/16.0.1";
const char S31899[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S31900[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S31901[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S31902[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S31903[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5,gzip(gfe) (via translate.google.com)";
const char S31904[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S31905[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S31906[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S31907[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S31908[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S31909[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5";
const char S31910[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1060) AppleWebKit/536.5 (KHTML like Gecko) Chrome/19.0.1084.53 Safari/536.5";
const char S31911[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/19.0.648.205 Safari/534.16";
const char S31912[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/19.0.648.205 Safari/534.16";
const char S31913[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/19.0.648.205 Safari/534.16";
const char S31914[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyST26i Build/11.2.A.0.21) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31915[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; ST26i Build/11.2.A.0.21) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31916[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-at; SonyST26iv Build/11.2.A.0.21) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31917[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyST26a Build/11.2.A.0.21) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31918[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; ST26a Build/11.2.A.0.21) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31919[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; ST26i Build/11.2.A.0.31)";
const char S31920[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyST26i Build/11.2.A.0.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31921[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; SonyST26i-o Build/11.2.A.0.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31922[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-at; SonyST26iv Build/11.2.A.0.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31923[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; ST26i Build/11.2.A.0.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31924[] = "SonyST26i Build/11.2.A.0.21 stagefright/1.2 (Linux;Android 4.1.2)";
const char S31925[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyST26a Build/11.2.A.0.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31926[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; ST26a Build/11.2.A.0.21)";
const char S31927[] = "SonyST26i Build/11.2.A.0.31 stagefright/1.2 (Linux;Android 4.1.2)";
const char S31928[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ST26a Build/11.2.A.0.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31929[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyST26i Build/11.2.A.0.33) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31930[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyST26a Build/11.2.A.0.33) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31931[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; ST26a Build/11.2.A.0.33) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31932[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ST26i Build/11.2.A.0.33) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31933[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; cm_tenderloin Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31934[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; cm_tenderloin Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31935[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; cm_tenderloin Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31936[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; cm_tenderloin Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31937[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; cm_tenderloin Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31938[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; cm_tenderloin Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31939[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; cm_tenderloin Build/GWK74)";
const char S31940[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; cm_tenderloin Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S31941[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; cm_tenderloin Build/IMM76L)";
const char S31942[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/IMM76I; CyanogenMod-Dark Unicorn Resurrected) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31943[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; cm_tenderloin Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31944[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; cm_tenderloin Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31945[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SonyEricssonMT27i Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31946[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; MT27i Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31947[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SonyEricssonMT27i Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31948[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; SonyEricssonMT27i Build/6.0.B.1.576) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31949[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SonyEricssonMT27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31950[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MT27i Build/6.0.B.1.564) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S31951[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MT27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31952[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; MT27i Build/6.0.B.3.184)";
const char S31953[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; MT27i Build/6.0.B.1.576) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31954[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-sg; SonyEricssonMT27i Build/6.0.B.3.188) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31955[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MT27i Build/6.0.B.3.184) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S31956[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; iw-; SonyEricssonMT27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31957[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; in-id; MT27i Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31958[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-sg; MT27i Build/6.0.B.3.188) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S31959[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A523 Safari/8536.25";
const char S31960[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A403 Safari/8536.25";
const char S31961[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A8426 Safari/8536.25";
const char S31962[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A550 Safari/8536.25";
const char S31963[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A406 Safari/8536.25";
const char S31964[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A8500 Safari/8536.25";
const char S31965[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A407 Safari/8536.25";
const char S31966[] = "Mozilla/5.0 (IE 11.0; Windows NT 6.3; WOW64; Trident/7.0; Touch; rv:11.0) like Gecko";
const char S31967[] = "Mozilla/5.0 (IE 11.0; Windows NT 6.3; Win64; x64; Trident/7.0; Touch; rv:11.0) like Gecko";
const char S31968[] = "Mozilla/5.0 (IE 11.0; Windows NT 6.3; Trident/7.0; Touch; rv:11.0) like Gecko";
const char S31969[] = "Mozilla/5.0 (Windows NT 6.3; WOW64; Trident/7.0; Touch; rv:11.0) like Gecko";
const char S31970[] = "Mozilla/5.0 (Windows NT 6.3; Win64; x64; Trident/7.0; Touch; rv:11.0) like Gecko";
const char S31971[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S31972[] = "Mozilla/5.0 (Linux; Android 4.2; Nexus 7 Build/JOP40C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S31973[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 7 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S31974[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Desire X Build/IMM76D)";
const char S31975[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Desire_X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31976[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; HTC_Desire_X/1.14.169.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31977[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; HTC_Desire_X-orange-LS/1.14.75.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31978[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Desire_X/1.14.161.11 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31979[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Desire X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31980[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HTC_Desire_X/1.14.111.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31981[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; HTC_Desire_X/1.14.114.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31982[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; HTC_Desire_X/1.14.113.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31983[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; HTC_Desire_X/1.14.118.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31984[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Desire_X/1.18.161.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31985[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; HTC_Desire_X/1.18.169.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31986[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; HTC_Desire_X/1.18.114.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31987[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Desire_X/1.26.161.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31988[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; HTC_Desire_X/1.18.118.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31989[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; HTC_Desire_X/1.18.114.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31990[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HTC_Desire_X/1.18.111.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31991[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; HTC_Desire_X/1.26.169.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31992[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; HTC_Desire_X/1.18.118.4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31993[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; HTC_Desire_X/1.18.113.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31994[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; HTC_Desire_X/1.25.123.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31995[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ch; HTC_Desire_X-orange-LS/1.14.65.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31996[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_Desire_X Build/SVHD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31997[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; HTC_Desire_X/1.25.112.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31998[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Desire_XS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S31999[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9380; en) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.336 Mobile Safari/534.11+";
const char S32000[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; U20i Build/2.0.2.A.0.24) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S32001[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; U20i Build/2.0.A.0.504) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S32002[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; SonyEricssonU20iv Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S32003[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; U20i Build/2.1.1.B.0.5) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S32004[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; U20i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S32005[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; pl-pl; U20i Build/2.1.1.A.0.16) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S32006[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; U20i Build/2.1.1.C.0.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S32007[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; U20i Build/2.1.1.A.0.19) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S32008[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; SonyEricssonU20i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S32009[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-kr; U20i Build/2.1.1.A.0.28) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S32010[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; U20i Build/2.1.1.A.0.6)";
const char S32011[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; U20i Build/2.1.1.A.0.6) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S32012[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; U20i Build/2.0.2.A.0.24) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S32013[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/530.5 (KHTML, like Gecko) Chrome/2.0.172.43 Safari/530.5";
const char S32014[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/530.5 (KHTML, like Gecko) Chrome/2.0.172.43 Safari/530.5";
const char S32015[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1;) AppleWebKit/530.1 (KHTML, like Gecko) Chrome/2.0.169.1 Safari/530.1";
const char S32016[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; eu) AppleWebKit/530.4 (KHTML, like Gecko) Chrome/2.0.172.0 Safari/530.4";
const char S32017[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/530.5 (KHTML, like Gecko) Chrome/2.0.172.33 Safari/530.5 AppEngine-Google; (+http://code.google.com/appengine; appid: metafabric)";
const char S32018[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.10 (KHTML, like Gecko) Chrome/2.0.1.3 Safari/536.10";
const char S32019[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/2.4.2.86 Safari/537.22 BIDUPlayerBrowser/2.x";
const char S32020[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S32021[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en_us) AppleWebKit/525.18.1 (KHTML, like Gecko)";
const char S32022[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148";
const char S32023[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/533.17.9";
const char S32024[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper";
const char S32025[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_2_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod2,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.2.1;FBSS/1; FBCR/;FBID/phone;FBLC/en_US;FBSF/1.0]";
const char S32026[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 iPod2,1 BingWeb/3.03.1428.X";
const char S32027[] = "Mozilla/5.0 (iPhone; U; CPU iPhone 4_2_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPod2,1/4.2.1)";
const char S32028[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 421 like Mac OS X; es-US) AppleWebKit/533.17.9 (KHTML like Gecko) Mobile/8C148 iPod21 BingWeb/3.02.1641.X";
const char S32029[] = "Instagram 2.5.0 (iPod touch; iPhone OS 4.2.1; en_US)";
const char S32030[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.27 Safari/532.0";
const char S32031[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.8 (KHTML, like Gecko) Chrome/3.1.0.0 Safari/532.8";
const char S32032[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay; ) AppleWebKit/532.1 (KHTML, like Gecko) Chrome/3.0.195.24 Safari/532.1";
const char S32033[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Source Client; ) AppleWebKit/532.1 (KHTML, like Gecko) Chrome/3.0.195.24 Safari/532.1";
const char S32034[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/531.0 (KHTML	 like Gecko) Chrome/3.0.190.1 Safari/531.0";
const char S32035[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; LG-P705g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32036[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; LG-P705 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32037[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-mx; LG-P708g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32038[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; LG-P705f Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32039[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-P705 Build/IML74K)";
const char S32040[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; LG-P705 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S32041[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-P708g Build/IML74K)";
const char S32042[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-P705g Build/IML74K)";
const char S32043[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-nz; LG-P705/V10g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32044[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-nz; LG-P705/V10c Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32045[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-nz; LG-P705/V10e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32046[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-P705f Build/IML74K)";
const char S32047[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; LG-P705 Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S32048[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST27i Build/6.1.1.B.1.10)";
const char S32049[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST27i Build/6.1.1.B.1.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32050[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SonyEricssonST27a Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32051[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonST27i Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32052[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST27i Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32053[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; ST27a Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32054[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-za; SonyEricssonST27iv Build/6.1.1.B.1.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32055[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST27a Build/6.1.1.B.1.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32056[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST27a Build/6.1.1.B.1.10)";
const char S32057[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PG86100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32058[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sprint APX515CKT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32059[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; HTC/EVO_3D/3.28.161.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32060[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Evo 3D Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32061[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; PG86100 Build/IML74K)";
const char S32062[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC/EVO_3D/3.28.166.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32063[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; HTC/EVO_3D/3.28.165.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32064[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PG86100 Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32065[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC/EVO_3D/3.28.163.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32066[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ISW12HT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32067[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PG86100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32068[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PG86100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30";
const char S32069[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PG86100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32070[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Sprint APX515CKT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32071[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; ISW12HT Build/IML74K)";
const char S32072[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Sprint APX515CKT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32073[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; PG86100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32074[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; HTC/EVO_3D/1.21.161.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32075[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; -us; Sprint APX515CKT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32076[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sprint APX515CKT Build/MyUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32077[] = "Mozilla/5.0 (Linux; Android 4.1; Nexus 7 Build/JRN84D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S32078[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus 7 Build/JRO03D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S32079[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus 7 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S32080[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S32081[] = "Mozilla/5.0 (Linux; Android 4.1.1; Google Nexus 7 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S32082[] = "Mozilla/5.0 (Linux; Android 4.1; KRAKD Nexus 7 Build/JRN84D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S32083[] = "Mozilla/5.0 (Linux; Android 4.1.1; Ainovo Flame (Nexus 7) Build/JRO03D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S32084[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; Nexus 7) AppleWebKit/537.7 (KHTML, like Gecko) Chrome/18.0.1025.308 Safari/537.1";
const char S32085[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus 7 16G Build/JRO03D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S32086[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S32087[] = "Mozilla/5.0 (Linux; Android 4.1; en-US; Nexus 7 Build/JRN84D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Safari/535.19";
const char S32088[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; LG-P700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32089[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; LG-P700/V10c Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32090[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-P700 Build/IML74K)";
const char S32091[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; LG-P700-Orange Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32092[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-P700/V10g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32093[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; LG-P700/V10f Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32094[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; LG-P700/V10a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32095[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; bs; LG-P700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32096[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; LG-P700/V10h Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32097[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; LG-P700/V10d Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32098[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; LG-P700/V10i Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32099[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-P700/V10k Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32100[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; LG-P700/V10e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32101[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Sony Xperia Neo V Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32102[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MT11i Build/4.1.A.0.562) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32103[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; MT11i Build/4.1.A.0.562)";
const char S32104[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MT11i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32105[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MT11a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32106[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyEricssonMT11a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32107[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; MT11i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32108[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; SonyEricssonMT11i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32109[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SonyEricssonMT11iv Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32110[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MT11i Build/4.1.B.0.587)";
const char S32111[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MT11i Build/ThilinaC@xda_pureness_13.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32112[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MT11a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32113[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MT11i Build/ThilinaC@xda_pureness_14.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32114[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MT11a Build/4.1.B.0.587)";
const char S32115[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-sa; LG-P350 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32116[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-P350-Orange Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32117[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P350-Orange Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32118[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P350/V10a/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32119[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P350 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32120[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P350/V10d Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32121[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-mx; LG-P350g Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32122[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; LG-P350/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32123[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ro-ro; LG-P350/V10b/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32124[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; LG-P350 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32125[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-mx; LG-P350f Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32126[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; hu-hu; LG-P350/V10f Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32127[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; iw; LG-P350 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32128[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P350 Build/FRG83)";
const char S32129[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; hr-hr; LG-P350/V10g Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32130[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; LG-P350 Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S32131[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P350f Build/FRG83G)";
const char S32132[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P350g Build/FRG83G)";
const char S32133[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; hr-hr; LG-P350/V10i Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32134[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; LG-P350 Build/FRG83)";
const char S32135[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; LG-P350 Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S32136[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; nl-nl; LG-P350/V10e Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32137[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0Nokia5230-c/20.2.098; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) BrowserNG/7.2.3.1";
const char S32138[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5230-1b/20.2.005; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.3";
const char S32139[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5230-1d/20.4.005; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.3";
const char S32140[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0Nokia5230/50.0.001; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.25 Mobile Safari/533.4 3gpp-gba";
const char S32141[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5230-1d/50.4.001; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.25 Mobile Safari/533.4 3gpp-gba";
const char S32142[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5230-c/20.2.098; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) BrowserNG/7.2.3.1";
const char S32143[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5230/20.0.005; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.3";
const char S32144[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5230-1c/20.8.007; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.3";
const char S32145[] = "Nokia5230/UCBrowser7.7.0.81/50/999";
const char S32146[] = "Nokia5230";
const char S32147[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5230/51.0.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 Mobile Safari/533.4 3gpp-gba";
const char S32148[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5230-1b/51.2.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 Mobile Safari/533.4 3gpp-gba";
const char S32149[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.2.15 Version/10.00";
const char S32150[] = "Opera/9.80 (Windows NT 5.2; U; en) Presto/2.2.15 Version/10.00";
const char S32151[] = "Opera/9.80 (Windows NT 5.1; U; en) Presto/2.2.15 Version/10.01";
const char S32152[] = "Opera/10.00 (Windows NT 5.1; U; en) Presto/2.2.0";
const char S32153[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.2.15 Version/10.00";
const char S32154[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02603); ru) Presto/2.2.15 Version/10.00";
const char S32155[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.2.15 Version/10.00";
const char S32156[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.2.15 Version/10.00";
const char S32157[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.2.15 Version/10.01";
const char S32158[] = "Opera/9.80 (Windows NT 5.2; U; en) Presto/2.2.15 Version/10.01";
const char S32159[] = "Mozilla/5.0 (Windows NT 5.1; U; ru; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 10.00";
const char S32160[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.2.15 Version/10.00";
const char S32161[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.2.15 Version/10.00";
const char S32162[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.2.15 Version/10.00";
const char S32163[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.2.15 Version/10.00";
const char S32164[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.2.15 Version/10.00";
const char S32165[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.2.15 Version/10.00";
const char S32166[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.2.15 Version/10.00";
const char S32167[] = "Opera/9.80 (Windows NT 5.1; U; en) Presto/2.2.15 Version/10.00,gzip(gfe) (via translate.google.com)";
const char S32168[] = "Mozilla/5.0 (Windows NT 5.1; U; ru; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 10.01";
const char S32169[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; ru) Opera 10.00";
const char S32170[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; ru) Opera 10.01";
const char S32171[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.2.15 Version/10.01";
const char S32172[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.2.15 Version/10.00";
const char S32173[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.2.15 Version/10.00";
const char S32174[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.2.15 Version/10.00";
const char S32175[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.2.15 Version/10.00";
const char S32176[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.2.15 Version/10.00";
const char S32177[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.2.15 Version/10.00";
const char S32178[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.2.15 Version/10.00";
const char S32179[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.2.15 Version/10.00";
const char S32180[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.2.15 Version/10.00";
const char S32181[] = "Opera/10.02 (Windows NT 5.1; U; en) Presto/22.10.289 Version/10.02";
const char S32182[] = "Opera/10.0 (Windows NT 5.2; WOW64; U; en)Presto/22.10.168 Version/10.00";
const char S32183[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.2.15 Version/10.00";
const char S32184[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.2.15 Version/10.00";
const char S32185[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.2.15 Version/10.00,gzip(gfe)";
const char S32186[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.2.15 Version/10.00";
const char S32187[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.2.15 Version/10.00";
const char S32188[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.2.15 Version/10.00";
const char S32189[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.2.15 Version/10.00";
const char S32190[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.2.15 Version/10.00";
const char S32191[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.2.15 Version/10.00";
const char S32192[] = "Opera/10.0 (Windows NT 5.1; U)";
const char S32193[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.2.15 Version/10.00";
const char S32194[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:17.0) Gecko/17.0 Firefox/17.0";
const char S32195[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:17.0) Gecko/17.0 Firefox/17.0";
const char S32196[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-MS770 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32197[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-MS770 Build/IMM76I)";
const char S32198[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-MS770 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Android MetroPCS androidAppVersion/2.5.10 AndroidWebkitWrapper BillingSupported";
const char S32199[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; LG-MS770 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32200[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-MS770 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 androidAppVersion/2.6.8 AndroidWebkitWrapper";
const char S32201[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.0.4;dev=l0;devm=LG-MS770;man=lge;brw=Android;shwd=320x480;shwa=320x455;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S32202[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; 710)";
const char S32203[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 710)";
const char S32204[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 710; T-Mobile)";
const char S32205[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 710; Vodafone)";
const char S32206[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 710; Orange)";
const char S32207[] = "Mozilla/5.0 (compatible; MSIE 9.0;Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 710)";
const char S32208[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 710; Meteor)";
const char S32209[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Nokia 710)";
const char S32210[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 710; Optimus)";
const char S32211[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Nokia_blocked; Lumia 710)";
const char S32212[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 710),gzip(gfe)";
const char S32213[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 710),gzip(gfe) (via docs.google.com/viewer)";
const char S32214[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A523 Safari/8536.25";
const char S32215[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A403 Safari/8536.25";
const char S32216[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A525 Safari/8536.25";
const char S32217[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A551 Safari/8536.25";
const char S32218[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A405 Safari/8536.25";
const char S32219[] = "DoCoMo/2.0 P900i(c100;TB;W24H11) (compatible; ichiro/mobile goo;+http://search.goo.ne.jp/option/use/sub4/sub4-1/)";
const char S32220[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:15.0) Gecko/X Firefox/15.0.1";
const char S32221[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:15.0) Gecko/X Firefox/15.0.1 Firefox/15.0.1";
const char S32222[] = "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:21.0) Gecko/X Firefox/21.0";
const char S32223[] = "Mozilla/5.0 (X11; U; Linux; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Qt/4.6.3 Safari/532.4";
const char S32224[] = "Mozilla/5.0 (X11; U; Linux; en-US) AppleWebKit/527+ (KHTML, like Gecko, Safari/419.3) Qt/4.5.2";
const char S32225[] = "Mozilla/5.0 (X11; U; Linux; en-US) AppleWebKit/523.15 (KHTML, like Gecko, Safari/419.3) Qt/4.4.3";
const char S32226[] = "Mozilla/5.0 (QtEmbedded; U; Linux; en-US) AppleWebKit/523.15 (KHTML, like Gecko, Safari/419.3) Qt/4.4.0";
const char S32227[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Qt/4.6.2 Safari/534.3";
const char S32228[] = "Mozilla/5.0 (QtEmbedded; U; Linux; C -) AppleWebKit/532.4 (KHTML, like Gecko) Qt/4.6.2 Safari/532.4";
const char S32229[] = "Mozilla/5.0 (X11; U; Linux x86_64; C) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.6.2 Safari/533.3";
const char S32230[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.18 (KHTML, like Gecko)";
const char S32231[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/525.28.3 (KHTML, like Gecko)";
const char S32232[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-us) AppleWebKit/525.18 (KHTML, like Gecko)";
const char S32233[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-us) AppleWebKit/525.18 (KHTML, like Gecko)";
const char S32234[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/525.27.1 (KHTML, like Gecko)";
const char S32235[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US)";
const char S32236[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko)";
const char S32237[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko)";
const char S32238[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) NetNewsWire/3.2.15";
const char S32239[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) WebClip/5527.2 Safari/5530.19";
const char S32240[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.18 (KHTML, like Gecko) WebClip/5525.7 Safari/5525.20.1";
const char S32241[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) WebClip/5527.2 Safari/5527.2";
const char S32242[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Sunrise/1.7.1 like Safari/5525.18";
const char S32243[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) (null) FBSMTWB";
const char S32244[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko, Safari/5525.20.1) ADM/559";
const char S32245[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) NetNewsWire/3.2.11";
const char S32246[] = "Mozilla/4.0 (Mac OS X 10.5.8) Java/1.5.0_30";
const char S32247[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) NetNewsWire/3.1.7";
const char S32248[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; fr; rv:1.9.0.3)";
const char S32249[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; en-us) AppleWebKit/525.18 (KHTML, like Gecko)";
const char S32250[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) (null) FBSMTWB";
const char S32251[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.0.6) Gecko/X Safari/525.27.1";
const char S32252[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/531.21.8+(KHTML, like Gecko, Safari/528.16) (null)";
const char S32253[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) RealPlayer/10.1.0.412";
const char S32254[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Safari/531.9";
const char S32255[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Safari/417.8";
const char S32256[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Safari/522.0";
const char S32257[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Shiira Safari/125";
const char S32258[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Sunrise";
const char S32259[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) RealPlayer/11.0.1.949";
const char S32260[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/525.28.3 (KHTML, like Gecko) WebClip/5525.7.1 Safari/5525.28.3";
const char S32261[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/525.27.1 (KHTML, like Gecko) WebClip/5525.7.1 Safari/5525.27.1";
const char S32262[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) NetNewsWire/3.2.7";
const char S32263[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) TheDancingQueen";
const char S32264[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; ja-jp) AppleWebKit/525.18 (KHTML, like Gecko) Sunrise/1.7.5 like Safari/5525.20.1";
const char S32265[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.18 (KHTML, like Gecko) TubeTV";
const char S32266[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_5; en-us) AppleWebKit/525.27.1 (KHTML, like Gecko) Shiira Safari/125";
const char S32267[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Vienna/2.5.0.2501";
const char S32268[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) KidZui K2";
const char S32269[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) NetNewsWire/3.2";
const char S32270[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Sunrise/2.1.5 Safari/533.22.3";
const char S32271[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.2.10) Gecko Expeditor 6.2.2.X-1115 Gecko/X-1115";
const char S32272[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) RealPlayer/12.0.0.1725";
const char S32273[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.18 (KHTML, like Gecko) LittleSnapper/9020 Safari/525.13";
const char S32274[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) RealPlayer/11.1.0.1116";
const char S32275[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) RealPlayer/10.1.0.503";
const char S32276[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko, Safari/534.50.2) ADM/737";
const char S32277[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) TubeTV";
const char S32278[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) RealPlayer/12.0.1.1750";
const char S32279[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) RealPlayer/12.0.0.1569";
const char S32280[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Safari/531.9";
const char S32281[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Safari/417.8";
const char S32282[] = "Galaxy/1.0 [en] (Mac OS X 10.5.6)";
const char S32283[] = "Galaxy/1.0 [en] (Mac OS X 10.5.6; U; en)";
const char S32284[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-B5510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32285[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-B5510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32286[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; SAMSUNG GT-B5510/B5510ATLA1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32287[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; GT-B5510L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32288[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-B5510-ORANGE/B5510BVKJ2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32289[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-B5510L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32290[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; GT-B5510B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32291[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; SAMSUNG GT-B5510/B5510ATKK2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32292[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-B5510-ORANGE/B5510BVLB1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32293[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-B5510 Build/GINGERBREAD)";
const char S32294[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-B5510-ORANGE/B5510BVLD1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32295[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-B5510/B5510XXLE1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32296[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-B5510B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32297[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-B5510 Build/GINGERBREAD)";
const char S32298[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-B5510L Build/GINGERBREAD)";
const char S32299[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-B5510/B5510XXLH4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32300[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-B5510-ORANGE/B5510BVLF1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32301[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-B5510-ORANGE/B5510BVLH1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32302[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-B5510/B5510XXLL1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32303[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-B5510L Build/GINGERBREAD)";
const char S32304[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-B5510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32305[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-B5510-ORANGE/B5510BVLK1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32306[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11";
const char S32307[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.70 Safari/535.11";
const char S32308[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/17.0.944.0 Safari/535.8 BitdefenderSafepay/0.2.0.0 (OBK; 17.0.944.0; r396.2011)";
const char S32309[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/17.0.944.0 Safari/535.8 BitdefenderSafepay/0.2.0.0 (OBK; 17.0.944.0; r396.2011)";
const char S32310[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.84 Safari/535.11 SE 2.X MetaSr 1.0,gzip(gfe)";
const char S32311[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.1.963.51 Safari/535.11 MRCHROME SOC";
const char S32312[] = "Mozilla/5.0 (Windows NT 5.2; rv:20.0) Gecko/X Firefox/20.0";
const char S32313[] = "Mozilla/5.0 (Windows NT 5.1; WOW64; rv:20.0) Gecko/X Firefox/20.0";
const char S32314[] = "Mozilla/5.0 (Windows; Windows NT 5.1; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S32315[] = "Mozilla/5.0 (Windows NT 5.1; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.1";
const char S32316[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.1";
const char S32317[] = "CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 2.2.2)";
const char S32318[] = "Mozilla/5.0 (Linux; U; Android 2.2; pt-pt Build/FINAL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32319[] = "CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 2.2)";
const char S32320[] = "stagefright/1.0 (Linux;Android 2.2.1)";
const char S32321[] = "CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 2.2.1)";
const char S32322[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; imx51_bbg Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32323[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S32324[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Android Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32325[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; generic Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32326[] = "Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S32327[] = "Mozilla/5.0 (Linux; U; Android 2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32328[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Full Android Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32329[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; ST070I00 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32330[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; mercury Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32331[] = "Mozilla/5.0 (Linux; U; Android 2.2; nl-nl; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32332[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S32333[] = "Mozilla/5.0 (Linux; U; Android 2.2; ld-us; MID Build/JIEVIS) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32334[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; TSB_CLOUD_COMPANION;TOSHIBA_AC_AND_AZ) AppleWebkit/533.1(KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S32335[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; MID_Serails Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32336[] = "stagefright/1.0 (Linux;Android 2.2.2)";
const char S32337[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; generic Build/Uberoid.Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32338[] = "stagefright/1.0 (Linux;Android 2.2)";
const char S32339[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; Android Build/MASTER)";
const char S32340[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; FIH-F0X Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32341[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; msm7227_ffa Build/I700T_P7_T07_NEXX_FT_R_0_01_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32342[] = "Mozilla/5.0 (Linux; U; Android 2.2; md-us; sec_smdkv210 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32343[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; generic Build/Froyo)";
const char S32344[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; id-id; msm7227_ffa Build/I700T_P4_10_AC73_RE_R_0_02_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32345[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; FUSIONideos Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32346[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; ckt16_a10y Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32347[] = "Mozilla/5.0 (Linux; U; Android 2.2; it; TSB_CLOUD_COMPANION;TOSHIBA_AC_AND_AZ) AppleWebkit/533.1(KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S32348[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MID Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32349[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; D000-X-S01 Build/00.21.29-02.20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32350[] = "Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; generic/sourceidnull) AppleWebKit/533.1(KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32351[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; msm7227_ffa Build/I700T_P6_T06_AC86A_FT_H44_R_0_01_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32352[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; MID Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32353[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; hd-us; eeepc Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32354[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; msm7227_ffa Build/I700T_P6_T06_AC79_FT_H44_R_0_01_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32355[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; msm7227_ffa Build/I700T_P7_T07_NEXX_FT_H44_R_0_02_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32356[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; msm7227_ffa Build/I700T_P7_T01_TSMT_FT_S42_R_0_01_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32357[] = "Mozilla/5.0 (Android 2.2; Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S32358[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; eeepc Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32359[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; msm7227_ffa Build/I700T_P6_T02_NEXX_FT_R_0_03_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32360[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; mercury Build/FRF85B)";
const char S32361[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; msm7227_ffa Build/I700T_P6_T02_NEXX_FT_R_0_04_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32362[] = "stagefright/1.0 (Linux;Android 2.2.3)";
const char S32363[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pl-pl; MID 01 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32364[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; hr-hr; ARIZONA Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32365[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; generic Build/Froyo) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S32366[] = "Mozilla/5.0 (Linux; U; Android 2.2-update1; en-us; MID Build/froyo) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S32367[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; MID_Serails Build/FRF91)";
const char S32368[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-mx; FUSIONideos Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32369[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; Full Android Build/MASTER)";
const char S32370[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; msm7227_ffa Build/I700T_P4_10_AC73_RE_R_0_05_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32371[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32372[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; msm7227_ffa Build/I700T_P4_10_AC74_RE_R_0_01_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32373[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; vi-vn; IPHONE4 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32374[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Android for Telechips TCC8900 Evaluation Board (US) Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32375[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; msm7227_ffa Build/I700T_P4_11A_TSLSB_FT_R_0_01_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32376[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; msm7227_ffa Build/I700T_P6_T02_AC01_FT4_H44_R_0_02_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32377[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MID Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S32378[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; th-th; msm7227_ffa Build/InoteLite_P4_17_FLSE_FT_R_0_07_1010_TH009_X) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S32379[] = "CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 2.2) SW:V10e";
const char S32380[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; hr-hr; ARIZONA Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32381[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S32382[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; vi-vn; Iphone A4S Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32383[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; msm7227_ffa Build/I700T_P6_T06_AC80D_FT_H44_R_0_01_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32384[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; msm7227_ffa Build/I700T_P6_T10_GSXX_FT_R_0_01_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32385[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-; ANDROID MID Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32386[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; msm7227_ffa Build/I700T_P7_T04_TSCDC_FT_S42_R_0_02_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32387[] = "Mozilla/5.0 (Linux; U; Android 2.2; ko-kr; Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32388[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; St-A77 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32389[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; EVO_Tab Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32390[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Aedupac Xtab Light 100 Build/froyo_almond_r238) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32391[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-us; EF101 Build/Kaissen_neutral_I7003G_BOE_OV2_TV9.1_X_WCDMA) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32392[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; A35DE Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32393[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; ILT-MX100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32394[] = "Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; WDPF-703SE-E Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32395[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; A505 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32396[] = "HuaweiT8500_TD/1.0 (Linux; U; Android 2.2.2; zh-cn) Release/7.27.2011 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 AppleWebKit/533.1";
const char S32397[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; unknown Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32398[] = "Android 2.2";
const char S32399[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; device Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S32400[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.2 Version/4.0 Mobile Safari/530.17";
const char S32401[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; ANDROID MID Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32402[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; msm7227_ffa Build/I700T_P6_T12_FLMO_FT_S42_R_0_02_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32403[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; unknown Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32404[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:17.0) Gecko/X Firefox/17.0";
const char S32405[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32406[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/ITL41F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32407[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32408[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32409[] = "Logos/0.9.4 (Linux; U; Android 4.0.3; en_US; Xoom Build/IML77)";
const char S32410[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM30B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32411[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM30C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32412[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-us; Xoom Build/IRK77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32413[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Xoom Build/IRL09B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32414[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Xoom Build/IRL20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32415[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/IML75) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32416[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32417[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/IML74B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32418[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/IML81) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32419[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32420[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/IML77) AppleWebKit/535.7 (KHTML, like Gecko)  Safari/535.7";
const char S32421[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/IML70) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32422[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32423[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/HTK75D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32424[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM19) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32425[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Xoom Build/IML77)";
const char S32426[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32427[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32428[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/HLK75F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32429[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32430[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM48) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32431[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM76) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32432[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32433[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32434[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32435[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Xoom Build/IMM76L)";
const char S32436[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32437[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM76) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32438[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MZ505 Build/Moto_MZ505) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32439[] = "Mozilla/5.0 (Linux; U; Android 4.0; cn-zh; Xoom Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S32440[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MZ505 Build/Moto_MZ505)";
const char S32441[] = "Mozilla/5.0 (X11; U; Linux x86; %lang_code%) adbeat.com/policy Gecko/X Ubuntu/10.04 (lucid) Firefox/3.6.3 AppleWebKit/532.4 Safari/532.4";
const char S32442[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-T999 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32443[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-I535 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32444[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-I9300 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32445[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-T999 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32446[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-I535 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32447[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-I9300 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32448[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32449[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T999 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32450[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I535 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32451[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9300 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32452[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I535 Build/MROM; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32453[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/4.60; CyanogenMod-4.60) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32454[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/4.70; CyanogenMod-4.70) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32455[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T999 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/d2tmo";
const char S32456[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/4.80) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/4.80/i9300";
const char S32457[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I535 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/d2vzw";
const char S32458[] = "Mozilla/4.0 (compatible; MSIE 6.0;";
const char S32459[] = "MSIE 6.0";
const char S32460[] = "Mozilla/5.0 (compatible; MSIE 6.0; ) Gecko";
const char S32461[] = "Mozilla/4.0i(compatible;MSIE 6.0)";
const char S32462[] = "Mozilla/4.0 (X11; MSIE 6.0; i686; .NET CLR 1.1.4322; .NET CLR 2.0.50727; FDM)";
const char S32463[] = "Mozilla/5.0 (compatible;MSIE6.0)";
const char S32464[] = "Mozilla/4.0 (MSIE 6.0); dtaglim:128";
const char S32465[] = "Mozilla/4.0 (compatible; MSIE 6.00;)";
const char S32466[] = "Mozilla/5.0 (compatible: MSIE 6.0)";
const char S32467[] = "Mozilla/4.0 (MSIE 6.0)";
const char S32468[] = "Mozilla/5.0 (Linux; U; Android2.2; en-us; LG-P509 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32469[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us;LG-P509 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32470[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-P509Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32471[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-P509 Build/FRF91) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S32472[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-P509 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32473[] = "Mozilla/5.0 (Linux; U;Android 2.2; en-us; LG-P509 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32474[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P509Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32475[] = "Mozilla/5.0 (Linux; U; Android2.2.2; en-us; LG-P509 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32476[] = "Mozilla/5.0 (Linux; U; Android 2.2; ru-; LG-P509 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32477[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P509 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32478[] = "Mozilla/5.0 (Linux; U;Android 2.2.2; en-us; LG-P509 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32479[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;LG-P509 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32480[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P509 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32481[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-P509 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32482[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; LG-P509 Build/FRF91)";
const char S32483[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P509 Build/FRG83G)";
const char S32484[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P509 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S32485[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A523 Safari/8536.25";
const char S32486[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A403 Safari/8536.25";
const char S32487[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A407 Safari/8536.25";
const char S32488[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A550 Safari/8536.25";
const char S32489[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A8426 Safari/8536.25";
const char S32490[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A406 Safari/8536.25";
const char S32491[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A8500 Safari/8536.25";
const char S32492[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT875 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32493[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; XT875 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32494[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID BIONIC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32495[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; XT875 Build/5.5.1_84_DBN-71) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32496[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2_223) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32497[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2-223_DBN_M4-3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32498[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2_223) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32499[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2-223_DBN_M4-3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32500[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2-223_DBN_M4-7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32501[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2-223_DBN_M4-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32502[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2-223_DBN_M4-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32503[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; XT875 Build/5.5.1_84_D3G-55) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32504[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2-223_DBN_M4-6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32505[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2-223_DBN_M4-7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32506[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2-223_DBN_M4-6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32507[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2-223_DBN_M4-12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32508[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2-223_DBN_M4-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32509[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2-223_DBN_M4-15) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32510[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/LIQUIDBLUR-V1.0-JMOD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32511[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2-223_DBN_M4-23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32512[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2-223_DBN_M4-24) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32513[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; DROID BIONIC Build/6.7.2-223_DBN_M4-23)";
const char S32514[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32515[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2-223_DBN_M4-24) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32516[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S32517[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S32518[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2-223_DBN_M4-12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32519[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2-223_DBN_M4-19) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32520[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32521[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32522[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2-223_DBN_M4-21) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32523[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32524[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32525[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/Icarus HD 1.7.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32526[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/Icarus HD 1.7.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32527[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32528[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/Icarus HD 1.8.3 ReMiX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32529[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/Icarus HD 1.8.3 ReMiX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32530[] = "Opera/9.80 (BlackBerry; Opera Mini/6.5.27548/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S32531[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-78) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S32532[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT905 Build/9.8.2Q_SMUIRD-7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S32533[] = "Mozilla/5.0 (Linux; Android 4.1.1; XT907 Build/9.8.1Q_27-2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S32534[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC)";
const char S32535[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Accord)";
const char S32536[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; PM23300)";
const char S32537[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 1.00.183.02)";
const char S32538[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; Orange)";
const char S32539[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 1.00.166.02)";
const char S32540[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.166.01)";
const char S32541[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.161.01)";
const char S32542[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.166.02)";
const char S32543[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.163.02)";
const char S32544[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 1.00.162.02)";
const char S32545[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 1.00.163.03)";
const char S32546[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.172.02)";
const char S32547[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 1.00.161.02)";
const char S32548[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.163.01)";
const char S32549[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.161.02)";
const char S32550[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.163.04)";
const char S32551[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.161.04)";
const char S32552[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.172.04)";
const char S32553[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.166.04)";
const char S32554[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.162.04)";
const char S32555[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.164.02)";
const char S32556[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; C620e)";
const char S32557[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 1.00.172.02)";
const char S32558[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.162.01)";
const char S32559[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; C620t; CMCC)";
const char S32560[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.183.04)";
const char S32561[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.01.163.00)";
const char S32562[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.01.161.00)";
const char S32563[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.01.166.00)";
const char S32564[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.172.01)";
const char S32565[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.01.172.00)";
const char S32566[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.01.162.00)";
const char S32567[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 1.00.164.02)";
const char S32568[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.183.01)";
const char S32569[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.01.164.00)";
const char S32570[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.01.183.00)";
const char S32571[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; C620d; ChinaTelecom)";
const char S32572[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.164.04)";
const char S32573[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 3.41.161.02)";
const char S32574[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 3.41.162.02)";
const char S32575[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 3.41.166.02)";
const char S32576[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 3.41.163.02)";
const char S32577[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 3.41.164.02)";
const char S32578[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 3.41.183.02)";
const char S32579[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-52) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32580[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32581[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-52) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32582[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-50) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32583[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32584[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-91-LaunchKit) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32585[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-44) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32586[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-47) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32587[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-46) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32588[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32589[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-56) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32590[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-91-LaunchKit) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32591[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-57) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32592[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-59) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32593[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32594[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-61) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32595[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-61) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32596[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32597[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-60) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32598[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32599[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-62_MR-11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32600[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-70) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32601[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-70) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32602[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-73) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32603[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-73) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32604[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-71) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32605[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-71) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32606[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-62_MR-11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32607[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-62_MR-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32608[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-62_MR-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32609[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32610[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32611[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; DROID BIONIC Build/5.5.1_84_DBN-74)";
const char S32612[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-62_MR-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32613[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-68) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32614[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S32615[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-74) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S32616[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1590.0 Mobile/12E55 Safari/8536.25";
const char S32617[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A465 Safari/8536.25";
const char S32618[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A4449d Safari/8536.25";
const char S32619[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A4449a Safari/8536.25";
const char S32620[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A501 Safari/8536.25";
const char S32621[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S32622[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2";
const char S32623[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.861.0 Safari/535.2";
const char S32624[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2";
const char S32625[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2";
const char S32626[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2";
const char S32627[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SPH-M930 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32628[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SPH-M930BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32629[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M930BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32630[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SPH-M930 Build/GINGERBREAD)";
const char S32631[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SPH-M930BST Build/GINGERBREAD)";
const char S32632[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SPH-M930BST Build/GINGERBREAD)";
const char S32633[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M930 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32634[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SPH-M930 Build/GINGERBREAD)";
const char S32635[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SPH-M930BST Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S32636[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M930BST Build/ROM by Bloodawn) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32637[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M930 Build/ROM by Bloodawn) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32638[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SPH-M930BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32639[] = "Mozilla/5.0 (Windows NT 5.2; rv:22.0) Gecko/X Firefox/22.0";
const char S32640[] = "Mozilla/5.0 (Windows NT 5.1; WOW64; rv:22.0) Gecko/X Firefox/22.0";
const char S32641[] = "Mozilla/5.0 (Windows NT 5.1; rv:22.0) Gecko/X Firefox/22.0 IceDragon/22.0.0.1";
const char S32642[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:22.0) Gecko/X Firefox/22.0 IceDragon/22.0.0.1";
const char S32643[] = "Mozilla/5.0 (Windows; Windows NT 5.1; rv:22.0) Gecko/22.0 Firefox/22.0";
const char S32644[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu Chromium/25.0.1364.160 Chrome/25.0.1364.160 Safari/537.22";
const char S32645[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu Chromium/25.0.1364.160 Chrome/25.0.1364.160 Safari/537.22";
const char S32646[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu Chromium/25.0.1364.160 Chrome/25.0.1364.160 Safari/537.22,gzip(gfe)";
const char S32647[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu Chromium/25.0.1364.160 Chrome/25.0.1364.160 Safari/537.22,gzip(gfe)";
const char S32648[] = "Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu Chromium/25.0.1364.160 Chrome/25.0.1364.160 Safari/537.22";
const char S32649[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu Chromium/25.0.1364.160 Chrome/25.0.1364.160 Safari/537.22,gzip(gfe) (via docs.google.com/viewer)";
const char S32650[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu Chromium/25.0.1364.160 Chrome/25.0.1364.160 Safari/537.22,gzip(gfe) (via docs.google.com/viewer)";
const char S32651[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.22 (KHTML like Gecko) Ubuntu Chromium/25.0.1364.160 Chrome/25.0.1364.160 Safari/537.22";
const char S32652[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B118 Safari/6531.22.7";
const char S32653[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S32654[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/9Z999 Safari/6531.22.7";
const char S32655[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S32656[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 41 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S32657[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 41 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML like Gecko) Version/4.0.5 Mobile/8B118 Safari/6531.22.7";
const char S32658[] = "Opera/9.80 (Windows NT 6.2) Presto/2.12.388 Version/12.11";
const char S32659[] = "Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.11";
const char S32660[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.11";
const char S32661[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.11";
const char S32662[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.11";
const char S32663[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.11";
const char S32664[] = "Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.11,gzip(gfe)";
const char S32665[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.11";
const char S32666[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.11";
const char S32667[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.11";
const char S32668[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.11";
const char S32669[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.11";
const char S32670[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.11";
const char S32671[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:17.0) Gecko/X Firefox/17.0";
const char S32672[] = "Opera/9.80 (VRE; Opera Mini/4.2/30.3214; U; en) Presto/2.8.119 Version/11.10";
const char S32673[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SAMSUNG-SGH-I777 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32674[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SAMSUNG-SGH-I777 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32675[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-I777 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32676[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-I777 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32677[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SAMSUNG-SGH-I777 Build/GINGERBREAD)";
const char S32678[] = "Logos/0.9.4 (Linux; U; Android 2.3.6; en_US; SAMSUNG-SGH-I777 Build/GINGERBREAD.UCKK6)";
const char S32679[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SAMSUNG-SGH-I777 Build/GINGERBREAD)";
const char S32680[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I777 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32681[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SAMSUNG-SGH-I777 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32682[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; SGH-I777 Build/GWK74)";
const char S32683[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SGH-I777 Build/DoomZday) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32684[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.60";
const char S32685[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.229 Version/11.60";
const char S32686[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.60";
const char S32687[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.229 Version/11.60";
const char S32688[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.10.229 Version/11.60";
const char S32689[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.229 Version/11.60,gzip(gfe) (via translate.google.com)";
const char S32690[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.60";
const char S32691[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.229 Version/11.60";
const char S32692[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.229 Version/11.60";
const char S32693[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; en) Opera 11.60";
const char S32694[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.229 Version/11.60";
const char S32695[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.60";
const char S32696[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.229 Version/11.60";
const char S32697[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4930); ru) Presto/2.10.229 Version/11.60";
const char S32698[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.229 Version/11.60";
const char S32699[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.229 Version/11.60";
const char S32700[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.229 Version/11.60";
const char S32701[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.229 Version/11.60";
const char S32702[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.10.229 Version/11.60";
const char S32703[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.229 Version/11.60";
const char S32704[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.10.229 Version/11.60";
const char S32705[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94; ru) Presto/2.10.229 Version/11.60";
const char S32706[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.229 Version/11.60";
const char S32707[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4661); ru) Presto/2.10.229 Version/11.60";
const char S32708[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03672); ru) Presto/2.10.229 Version/11.60";
const char S32709[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.60";
const char S32710[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.229 Version/11.60";
const char S32711[] = "Opera/9.80 (Windows NT 5.1; U; en) Presto/2.10.229 Version/11.60";
const char S32712[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.229 Version/11.60";
const char S32713[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.229 Version/11.60";
const char S32714[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.229 Version/11.60";
const char S32715[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.229 Version/11.60";
const char S32716[] = "Opera/9.80 (Windows NT 5.2; U; en) Presto/2.10.229 Version/11.60";
const char S32717[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.229 Version/11.60";
const char S32718[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.229 Version/11.60";
const char S32719[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03780); ru) Presto/2.10.229 Version/11.60";
const char S32720[] = "Mozilla/5.0 (Windows NT 5.1; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.60";
const char S32721[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.229 Version/11.60";
const char S32722[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.229 Version/11.60";
const char S32723[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.229 Version/11.60";
const char S32724[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.229 Version/11.60";
const char S32725[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.10.229 Version/11.60";
const char S32726[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.10.229 Version/11.60";
const char S32727[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.229 Version/11.60";
const char S32728[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.10.229 Version/11.60";
const char S32729[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02564); ru) Presto/2.10.229 Version/11.60";
const char S32730[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.229 Version/11.60";
const char S32731[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5218); ru) Presto/2.10.229 Version/11.60";
const char S32732[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.229 Version/11.60";
const char S32733[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.229 Version/11.60";
const char S32734[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.229 Version/11.60";
const char S32735[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.229 Version/11.60";
const char S32736[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5328); ru) Presto/2.10.229 Version/11.60";
const char S32737[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.10.229 Version/11.60";
const char S32738[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.229 Version/11.60";
const char S32739[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.10.229 Version/11.60";
const char S32740[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.10.229 Version/11.60";
const char S32741[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02560); ru) Presto/2.10.229 Version/11.60";
const char S32742[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.229 Version/11.60";
const char S32743[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5230); ru) Presto/2.10.229 Version/11.60";
const char S32744[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5667); ru) Presto/2.10.229 Version/11.60";
const char S32745[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.229 Version/11.60";
const char S32746[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.229 Version/11.60,gzip(gfe)";
const char S32747[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.229 Version/11.60";
const char S32748[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.229 Version/11.60";
const char S32749[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.229 Version/11.60";
const char S32750[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5784); ru) Presto/2.10.229 Version/11.60";
const char S32751[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.229 Version/11.60";
const char S32752[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5966); ru) Presto/2.10.229 Version/11.60";
const char S32753[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.229 Version/11.60";
const char S32754[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6083); ru) Presto/2.10.229 Version/11.60";
const char S32755[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6037); ru) Presto/2.10.229 Version/11.60";
const char S32756[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6089); ru) Presto/2.10.229 Version/11.60";
const char S32757[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6081); ru) Presto/2.10.229 Version/11.60";
const char S32758[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.229 Version/11.60";
const char S32759[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.229 Version/11.60";
const char S32760[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C5170 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32761[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; C5170 Build/IML77)";
const char S32762[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Hydro Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32763[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Hydro Build/IMM76I)";
const char S32764[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; C5170 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32765[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C5170 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 androidAppVersion/2.5.22 AndroidWebkitWrapper BillingSupported";
const char S32766[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C5170 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 androidAppVersion/2.6.8 AndroidWebkitWrapper";
const char S32767[] = "Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.10.289 Version/12.00";
const char S32768[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4953); ru) Presto/2.10.289 Version/12.00";
const char S32769[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5310); ru) Presto/2.10.289 Version/12.00";
const char S32770[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5339); ru) Presto/2.10.289 Version/12.00";
const char S32771[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.10.289 Version/12.00";
const char S32772[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.10.289 Version/12.00";
const char S32773[] = "Opera/9.80 (Windows NT 6.1; WOW64; U; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0); en) Presto/2.10.289 Version/12.00";
const char S32774[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5711); ru) Presto/2.10.289 Version/12.00";
const char S32775[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.4 (build 02647); ru) Presto/2.10.289 Version/12.00";
const char S32776[] = "Opera/9.80 (Windows NT 5.1; U; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0); en) Presto/2.10.289 Version/12.00";
const char S32777[] = "Opera/9.80 (Windows NT 6.1; U; compatible; MSIE 7.0; Windows NT 6.0; bg) Presto/2.10.289 Version/12.00";
const char S32778[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5680); ru) Presto/2.10.289 Version/12.00";
const char S32779[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5754); ru) Presto/2.10.289 Version/12.00";
const char S32780[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.10.289 Version/12.00";
const char S32781[] = "Opera/9.80 (Windows NT 6.0; U; de) Presto/2.10.289 Version/12.00,gzip(gfe) (via translate.google.com)";
const char S32782[] = "Opera/9.80 (Windows NT 6.0; U; en) Presto/2.10.289 Version/12.01";
const char S32783[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.10.289 Version/12.01";
const char S32784[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.10.289 Version/12.01";
const char S32785[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5339); ru) Presto/2.10.289 Version/12.01";
const char S32786[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5310); ru) Presto/2.10.289 Version/12.01";
const char S32787[] = "Opera/9.80 (Windows NT 6.1; U; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0); en) Presto/2.10.289 Version/12.01";
const char S32788[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5831); ru) Presto/2.10.289 Version/12.01";
const char S32789[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4953); ru) Presto/2.10.289 Version/12.01";
const char S32790[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03686); ru) Presto/2.10.289 Version/12.00";
const char S32791[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5680); ru) Presto/2.10.289 Version/12.01";
const char S32792[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.10.289 Version/12.01";
const char S32793[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03686); ru) Presto/2.10.289 Version/12.01";
const char S32794[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5972); ru) Presto/2.10.289 Version/12.01";
const char S32795[] = "Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.10.289 Version/12.01,gzip(gfe) (via translate.google.com)";
const char S32796[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5831); ru) Presto/2.10.289 Version/12.00";
const char S32797[] = "Mozilla/5.0 (Windows NT 6.0; en; rv:2.0) Gecko/X Firefox/4.0 Opera 12.00";
const char S32798[] = "Opera/12.0 (Windows NT 6.0; U; en)Presto/22.9.168 Version/12.00";
const char S32799[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5754); ru) Presto/2.10.289 Version/12.01";
const char S32800[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4598); ru) Presto/2.10.289 Version/12.00";
const char S32801[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5970); ru) Presto/2.10.289 Version/12.01";
const char S32802[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4133); ru) Presto/2.10.289 Version/12.00";
const char S32803[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94; ru) Presto/2.10.289 Version/12.00";
const char S32804[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4139); ru) Presto/2.10.289 Version/12.00";
const char S32805[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-nz; IDEOS X5 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32806[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-za; IDEOS X5 Build/HuaweiU8800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32807[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-ve; U8800 Build/HuaweiU8800-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32808[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Huawei U8800-51 Build/HWU8800B635) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32809[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Huawei U8800-51 Build/HWU8800B630) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32810[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; u8800 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32811[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; u8800 Build/HuaweiU8800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32812[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; tr-tr; IDEOS X5 Build/HuaweiU8800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32813[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-; IDEOS X5 Build/HuaweiU8800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32814[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; u8800 Build/FRG83)";
const char S32815[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; IDEOS X5 Build/FRG83)";
const char S32816[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; Huawei U8800-51 Build/HWU8800B635)";
const char S32817[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; nb-no; u8800 Build/HuaweiU8800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32818[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; U8800-51 Build/HuaweiU8800-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32819[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Huawei U8800-51 Build/HuaweiU8800-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32820[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ru-uz; Ucell Build/HuaweiU8800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32821[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ru-by; MTC Neo Build/HuaweiU8800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32822[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Huawei U8800-51 Build/HWU8800B661) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32823[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Avant Browser)";
const char S32824[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Avant Browser)";
const char S32825[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Win64; x64; Trident/6.0; Avant Browser)";
const char S32826[] = "Mozilla/5.0 (Linux; Android 4.0.3;  GT-I9100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S32827[] = "Mozilla/5.0 (Linux; Android 4.0.4;  GT-I9100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S32828[] = "Mozilla/5.0 (Linux; Android 4.0.3;  GT-I9100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S32829[] = "Mozilla/5.0 (Linux; Android 4.0.3;  GT-I9100T Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S32830[] = "Mozilla/5.0 (Linux; Android 4.0.3;  GT-I9100M Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S32831[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S32832[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S32833[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100M Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S32834[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100T Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S32835[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S32836[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S32837[] = "Mozilla/5.0 (Linux; Android 4.0.3;  GT-I9100G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S32838[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S32839[] = "Mozilla/5.0 (Linux; Android 4.0.4; Galaxy SII T-mobile Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S32840[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S32841[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100T Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S32842[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8530/S8530JPKA1; U; Bada/1.2; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.2 Mobile WVGA SMM-MMS/1.2.0 OPN-B";
const char S32843[] = "SAMSUNG-GT-S8530/1.0 SHP/VPP/R5 Dolfin/2.2 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S32844[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-D710 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32845[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-D710 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32846[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-D710 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32847[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-D710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32848[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SPH-D710 Build/JRO03L)";
const char S32849[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SPH-D710 Build/JZO54K)";
const char S32850[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-D710VMUB Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32851[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-D710BST Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32852[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SPH-D710VMUB Build/JZO54K)";
const char S32853[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SPH-D710BST Build/JZO54K)";
const char S32854[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SPH-D710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32855[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ar-ye; SPH-D710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 ?????? Safari/534.30";
const char S32856[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A5355d Twitter for iPhone";
const char S32857[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Twitter for iPhone";
const char S32858[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Twitter for iPhone";
const char S32859[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Twitter for iPhone";
const char S32860[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Twitter for iPhone";
const char S32861[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Twitter for iPhone";
const char S32862[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/14";
const char S32863[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S32864[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S32865[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1,gzip(gfe) (via translate.google.com)";
const char S32866[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100 (Edition Next)";
const char S32867[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100 (Edition Next)";
const char S32868[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153";
const char S32869[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153";
const char S32870[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Campaign 21)";
const char S32871[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Rambler)";
const char S32872[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153,gzip(gfe)";
const char S32873[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Yx)";
const char S32874[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Campaign 21)";
const char S32875[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Campaign 06)";
const char S32876[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT22i Build/6.2.A.0.400) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32877[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; LT22i Build/6.2.A.0.402) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32878[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-at; LT22i Build/6.2.A.0.404) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32879[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyLT22i Build/6.2.A.1.100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32880[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LT22i Build/6.2.A.1.100)";
const char S32881[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LT22i Build/6.2.A.1.100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32882[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; SonyLT22i-o Build/6.2.A.1.100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32883[] = "SonyLT22i Build/6.2.A.0.400 stagefright/1.2 (Linux;Android 4.1.2)";
const char S32884[] = "SonyLT22i Build/6.2.A.1.100 stagefright/1.2 (Linux;Android 4.1.2)";
const char S32885[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; sv-se; SonyEricssonLT22i Build/6.0.B.0.639) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32886[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; SonyEricssonLT22i-o Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32887[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonLT22i Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32888[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT22i Build/6.0.B.1.564) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S32889[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; LT22i Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32890[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; LT22i Build/6.0.B.1.564)";
const char S32891[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; SonyEricssonLT22i Build/6.0.B.1.576) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32892[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; SonyEricssonLT22i-o Build/6.0.B.1.576) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32893[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-mx; SonyEricssonLT22i Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32894[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LT22i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32895[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; nl-nl; SonyEricssonLT22i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32896[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; SonyEricssonLT22i-o Build/6.0.B.3.187) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32897[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-sg; SonyEricssonLT22i Build/6.0.B.3.188) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32898[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT22i Build/6.0.B.1.564) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S32899[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; sv-se; SonyEricssonLT22i Build/6.0.B.3.115) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32900[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-at; SonyEricssonLT22i Build/6.0.B.3.187) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32901[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; SonyEricssonLT22i-o Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32902[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-mx; LT22i Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32903[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; LT22i Build/6.0.B.3.187) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32904[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT22i Build/6.0.B.3.184) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S32905[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; LT22i Build/6.0.B.1.576) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32906[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:15.0) Gecko/X Firefox/15.0.1";
const char S32907[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:15.0) Gecko/X Firefox/15.0.1 Firefox/15.0.1";
const char S32908[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-P505/V10b Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32909[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-P505/V10e Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32910[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-P505/V09c Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32911[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-P505/V11e Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32912[] = "Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; LG-P505/V11e Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32913[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P505/V11h Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32914[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P505 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32915[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P505/V10i Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32916[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-P505 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32917[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; LG-P505R Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32918[] = "Logos/0.9.1 (Linux; U; Android 2.2.1; en_US; LG-P505 Build/FRG83)";
const char S32919[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; LG-P505CH Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32920[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-P505/V11e Build/FRG85) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32921[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P505 Build/FRG83G)";
const char S32922[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; LG-P505 Build/FRG83)";
const char S32923[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P505R Build/FRG83G)";
const char S32924[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; LG-P505R Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32925[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P505CH Build/FRG83G)";
const char S32926[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11,gzip(gfe) (via docs.google.com/viewer)";
const char S32927[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S32928[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S32929[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe) (via translate.google.com)";
const char S32930[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT905 Build/9.8.2Q_SMUIRD-7) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S32931[] = "Mozilla/5.0 (Linux; Android 4.1.1; XT907 Build/9.8.1Q_27-2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S32932[] = "Mozilla/5.0 (Linux; Android 4.1.1; XT907 Build/9.8.1Q_27-2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S32933[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT905 Build/9.8.2Q_SMUIRD-7) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S32934[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-78) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S32935[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-78) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S32936[] = "Opera/9.80 (Windows NT 6.0; U; en) Presto/2.7.62 Version/11.00";
const char S32937[] = "Opera/9.80 (Windows NT 6.0; U; pl) Presto/2.7.62 Version/11.01";
const char S32938[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.7.62 Version/11.01";
const char S32939[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.7.62 Version/11.01";
const char S32940[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; en) Opera 11.00";
const char S32941[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.7.62 Version/11.01";
const char S32942[] = "Mozilla/5.0 (Windows NT 6.0; U; ja; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 11.00";
const char S32943[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.7.62 Version/11.00";
const char S32944[] = "Mozilla/5.0 (Linux; Android 4.2; Galaxy Nexus Build/JOP40C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S32945[] = "Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S32946[] = "Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S32947[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S32948[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S32949[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG Galaxy Nexus Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S32950[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us; Silk/1.0.13.328_X) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16 Silk-Accelerated=true";
const char S32951[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us; Silk/1.1.0-84) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16 Silk-Accelerated=true";
const char S32952[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A400 Safari/528.16";
const char S32953[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S32954[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293 Safari/6531.22.7";
const char S32955[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 30 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S32956[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SAMSUNG-SGH-I727 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32957[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; SAMSUNG-SGH-I727 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32958[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SAMSUNG-SGH-I727 Build/IML74K)";
const char S32959[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32960[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SAMSUNG-SGH-I727 Build/IMM76D)";
const char S32961[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32962[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I727 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32963[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SAMSUNG-SGH-I727 Build/Stock Revamped) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32964[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S32965[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I727 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S32966[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.";
const char S32967[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.0.4;dev=SGH-I727;devm=SAMSUNG-SGH-I727;man=samsung;brw=Android;shwd=480x800;shwa=480x763;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S32968[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; HTC Wildfire 1.29.163.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32969[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; HTC Wildfire 1.14.162.3 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32970[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; de-at; HTC Wildfire 2.31.163.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32971[] = "HTC_Wildfire_Mozilla/5.0 (Linux; U; Android 2.2; it-it;HTC Buzz Built/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S32972[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32973[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; HTC Wildfire 2.24.164.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32974[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Wildfire 2.24.161.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32975[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; nl-nl; HTC Wildfire 2.24.172.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32976[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Wildfire 1.14.161.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32977[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; HTC Wildfire 2.24.163.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32978[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; HTC Wildfire 1.36.164.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32979[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; HTC Wildfire 2.24.162.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32980[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; HTC Wildfire 1.25.162.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32981[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Wildfire 1.26.161.3 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32982[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; HTC Wildfire 1.13.163.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32983[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Wildfire 2.24.161.2 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32984[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; HTC_Wildfire-orange-LS Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32985[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; nl-nl; HTC Wildfire 1.14.172.2 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32986[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; hr-; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32987[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; HTC Wildfire 1.17.164.3 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32988[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Wildfire Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32989[] = "Mozilla/5.0 (Linux; U; Android 2.2.1 Orange Froyo; es-es; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32990[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; HTC Wildfire Build/FRG83D)";
const char S32991[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; HTC Wildfire Build/FRG83D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S32992[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Wildfire 2.24.161.4 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32993[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Wildfire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32994[] = "Mozilla/5.0 (Linux; U; Android 2.2.1 Community 2; es-es; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32995[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Wildfire Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S32996[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; HTC Wildfire Build/FRG83D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S32997[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; HTC Wildfire 2.24.163.3 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32998[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; HTC Wildfire 2.24.164.3 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S32999[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; HTC Wildfire 2.24.162.3 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33000[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Wildfire Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S33001[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; nl-nl; HTC Wildfire 2.24.172.5 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33002[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33003[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-S6802B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33004[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S6802 Build/GINGERBREAD)";
const char S33005[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33006[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6352 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33007[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6802 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S33008[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S6802B Build/GINGERBREAD)";
const char S33009[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6352 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S33010[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S6352 Build/GINGERBREAD)";
const char S33011[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.9.168 Version/11.50";
const char S33012[] = "Opera/9.80 (Windows NT 5.2; U; en) Presto/2.9.168 Version/11.50";
const char S33013[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.9.168 Version/11.50";
const char S33014[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.9.168 Version/11.50";
const char S33015[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.9.168 Version/11.50";
const char S33016[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.9.168 Version/11.50";
const char S33017[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.9.168 Version/11.50";
const char S33018[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.9.168 Version/11.50";
const char S33019[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.9.168 Version/11.50";
const char S33020[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.9.168 Version/11.50";
const char S33021[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.9.168 Version/11.50";
const char S33022[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.9.168 Version/11.50";
const char S33023[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; en) Opera 11.50";
const char S33024[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.9.168 Version/11.50";
const char S33025[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.9.168 Version/11.50";
const char S33026[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.9.168 Version/11.50";
const char S33027[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.9.168 Version/11.50";
const char S33028[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4661); ru) Presto/2.9.168 Version/11.50";
const char S33029[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.9.168 Version/11.50";
const char S33030[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.9.168 Version/11.50";
const char S33031[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); en) Presto/2.9.168 Version/11.50";
const char S33032[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02746); ru) Presto/2.9.168 Version/11.50";
const char S33033[] = "Mozilla/5.0 (Windows NT 5.1; ru; rv:2.0) Gecko/X Firefox/4.0 Opera 11.50";
const char S33034[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.9.168 Version/11.50";
const char S33035[] = "Opera/9.80 (Windows NT 5.1; U; es-ES) Presto/2.9.168 Version/11.50,gzip(gfe) (via translate.google.com)";
const char S33036[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.9.168 Version/11.50";
const char S33037[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.9.168 Version/11.50";
const char S33038[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.9.168 Version/11.50";
const char S33039[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.9.168 Version/11.50";
const char S33040[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.9.168 Version/11.50";
const char S33041[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.9.168 Version/11.50";
const char S33042[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.9.168 Version/11.50";
const char S33043[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.9.168 Version/11.50";
const char S33044[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.9.168 Version/11.50";
const char S33045[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.9.168 Version/11.50";
const char S33046[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.9.168 Version/11.50";
const char S33047[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.9.168 Version/11.50";
const char S33048[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03791); ru) Presto/2.9.168 Version/11.50";
const char S33049[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.9.168 Version/11.50";
const char S33050[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03780); ru) Presto/2.9.168 Version/11.50";
const char S33051[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4930); ru) Presto/2.9.168 Version/11.50";
const char S33052[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4841); ru) Presto/2.9.168 Version/11.50";
const char S33053[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.9.168 Version/11.50";
const char S33054[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.9.168 Version/11.50";
const char S33055[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.9.168 Version/11.50";
const char S33056[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.9.168 Version/11.50";
const char S33057[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.9.168 Version/11.50";
const char S33058[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.9.168 Version/11.50";
const char S33059[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.9.168 Version/11.50";
const char S33060[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.9.168 Version/11.50";
const char S33061[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.9.168 Version/11.50,gzip(gfe)";
const char S33062[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.9.168 Version/11.50";
const char S33063[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.9.168 Version/11.50";
const char S33064[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.9.168 Version/11.50";
const char S33065[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.9.168 Version/11.50";
const char S33066[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.9.168 Version/11.50";
const char S33067[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6578); ru) Presto/2.9.168 Version/11.50";
const char S33068[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5745); ru) Presto/2.9.168 Version/11.50";
const char S33069[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.5.24 Version/10.52";
const char S33070[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.5.22 Version/10.50";
const char S33071[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.5.24 Version/10.54";
const char S33072[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.5.22 Version/10.51";
const char S33073[] = "Opera/9.80 (Windows NT 5.1; U; de) Presto/2.5.24 Version/10.53";
const char S33074[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.5.24 Version/10.53";
const char S33075[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.5.24 Version/10.53";
const char S33076[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.5.24 Version/10.54,gzip(gfe) (via translate.google.com)";
const char S33077[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.5.24 Version/10.54";
const char S33078[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.5.22 Version/10.50";
const char S33079[] = "Opera/9.80 (Windows NT 5.2; U; en) Presto/2.5.24 Version/10.53";
const char S33080[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.5.24 Version/10.53";
const char S33081[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.5.24 Version/10.53";
const char S33082[] = "Opera/9.80 (Windows NT 5.2; U; en) Presto/2.5.22 Version/10.50";
const char S33083[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.5.22 Version/10.51";
const char S33084[] = "Opera/9.80 (Windows NT 5.2; U; ru) Presto/2.5.22 Version/10.51";
const char S33085[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.5.24 Version/10.53";
const char S33086[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.5.22 Version/10.51";
const char S33087[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.5.24 Version/10.53";
const char S33088[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.5.24 Version/10.53";
const char S33089[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.5.24 Version/10.53";
const char S33090[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.5.24 Version/10.53";
const char S33091[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.5.24 Version/10.53";
const char S33092[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.5.22 Version/10.51";
const char S33093[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.5.22 Version/10.51";
const char S33094[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.5.24 Version/10.53";
const char S33095[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.5.22 Version/10.51";
const char S33096[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.5.24 Version/10.54,gzip(gfe)";
const char S33097[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.5.24 Version/10.53";
const char S33098[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.5.24 Version/10.53";
const char S33099[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.5.24 Version/10.53";
const char S33100[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.5.24 Version/10.53";
const char S33101[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SAMSUNG-SGH-I717 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33102[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I717 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33103[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SAMSUNG-SGH-I717 Build/GINGERBREAD)";
const char S33104[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I717 Build/CollectiveCustom) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33105[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SAMSUNG-SGH-I717 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S33106[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3Build/5.5.1_84_D3G-20_TA-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33107[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-52) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33108[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-20_TA-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S33109[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-19) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33110[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-54_MR-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33111[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-54) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33112[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-20_TA-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33113[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33114[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID3 Build/5.5.1_53_D3G-11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33115[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_80) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33116[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-20_TA-99) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33117[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-20_TA-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33118[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33119[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S33120[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;DROID3 Build/5.5.1_84_D3G-55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33121[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3Build/5.5.1_84_D3G-55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33122[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-55_MR-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33123[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33124[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID3 Build/5.5.1_84_D3G-71) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33125[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID3 Build/5.5.1_84_D3G-74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33126[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 4G Build/5.5.1_84_D3G-55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33127[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID3 Build/5.5.1_84_D3G-65) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33128[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-66_M2-6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33129[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID3 Build/5.5.1_84_D3G-65) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30";
const char S33130[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33131[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-64) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33132[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; DROID3 Build/5.5.1_84_D3G-66_M2-10)";
const char S33133[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-66_M2-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33134[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S33135[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33136[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;DROID3 Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33137[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33138[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 4G Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33139[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33140[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S33141[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK DROID3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33142[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; XT862 Build/SLNCT-57.1.60) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33143[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S33144[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S33145[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S33146[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2";
const char S33147[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2";
const char S33148[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S33149[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2";
const char S33150[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2";
const char S33151[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S33152[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10\\_6\\_8) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S33153[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/15.0.648.205 Safari/534.16";
const char S33154[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/15.0.648.205 Safari/534.16";
const char S33155[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/15.0.648.205 Safari/534.16";
const char S33156[] = "BlackBerry8900/4.6.1.231 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100";
const char S33157[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; A500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S33158[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; A500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S33159[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; A500 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S33160[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; A500 Build/HTJ85B)";
const char S33161[] = "Logos/0.9.4 (Linux; U; Android 3.2.1; en_US; A500 Build/Acer_A500_7.014.01_COM_GEN2)";
const char S33162[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; cs-; A500 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S33163[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; A500 Build/HTK55D)";
const char S33164[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; A500 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S33165[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; A500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S33166[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; A500 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S33167[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6410LVW 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33168[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6410LVW Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33169[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; ADR6410LVW Build/IML74K)";
const char S33170[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6410LVW 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33171[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6410LRA Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33172[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6410LVW Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33173[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ADR6410LVW Build/IMM76D)";
const char S33174[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-pr; ADR6410OM Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33175[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6410LRA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33176[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.13) Gecko/X Firefox/2.0 (Debian-3.0.12-1~backports50+1)";
const char S33177[] = "Mozilla/5.0 (X11;U;Linux i686;en-US;rv:1.8.1) Gecko/X Firefox/2.0";
const char S33178[] = "Mozilla/5.0 (X11; U; Linux i686 (x86_64); pl-PL; rv:1.8.1) Gecko/X Firefox/2.0-1.fc4.remi Firefox/2.0";
const char S33179[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.11) Firefox/2.0.0.11";
const char S33180[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.15) Gecko/X Iceweasel/2.0.0.15 (Debian-2.0.0.15-0etch1)";
const char S33181[] = "Mozilla/5.0 (X11; U; Linux i686; ja; rv:1.8.1.11) Gecko/X Firefox/2.0.0.11 (Debian-2.0.0.11-1)";
const char S33182[] = "Mozilla/5.0 (X11; U; Windows NT i686; fr; rv:1.9.0.1) Gecko/X Firefox/2.0.0.8";
const char S33183[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1b2) Gecko/X BonEcho/2.0b2";
const char S33184[] = "Mozilla/5.0 (X11; U; Linux i686; nl; rv:1.8.1b2) Gecko/X BonEcho/2.0b2 (Debian-1.99+2.0b2+dfsg-1)";
const char S33185[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; pl; rv:1.8.1.1) Gecko/X Mozilla/5.0 (X11; U; Linux i686; fr; rv:1.8.1) Gecko/X Firefox/2.0b2";
const char S33186[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:17.0) Gecko/X Firefox/17.0";
const char S33187[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus 7 Build/JWR66V) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S33188[] = "Mozilla/5.0 (Linux; Android 4.3; Google Nexus 7 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S33189[] = "index.boo.ly/1.0 (windows; u; windows nt 6.1; en-gb; rv:1.9.2.15) gecko/X firefox/3.6.15 gtb7.1 (.net clr 3.5.30729)";
const char S33190[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9a8) Gecko/X GranParadiso/3.0a8";
const char S33191[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-GB; rv:1.9.0.10) Gecko/X Firefox/3.0.10 (.NET CLR 3.5.30729)";
const char S33192[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.1.18) Gecko/X FireFox/3.6.13";
const char S33193[] = "Mozilla/5.0 (Symbian/3; Series60/5.2NokiaN8-00/011.012; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.7.2 3gpp-gba";
const char S33194[] = "Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaN8-00/014.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.8.10 3gpp-gba";
const char S33195[] = "Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaN8-00/022.014; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.37 Mobile Safari/533.4 3gpp-gba";
const char S33196[] = "Mozilla/5.0 (Symbian/3; Series60/5.2 Nokia N8-00/010.020; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6 3gpp-gba";
const char S33197[] = "NokiaN8-00";
const char S33198[] = "Mozilla/4.5 [en] (Win98; I)";
const char S33199[] = "Mozilla/4.5 (compatible; HTTrack 3.0x; Windows 98)";
const char S33200[] = "Mozilla/4.72 [en] (Win98)";
const char S33201[] = "Mozilla/4.73 [en] (Win98; U)";
const char S33202[] = "Mozilla/4.76 [en] (Win98)";
const char S33203[] = "Mozilla/4.76 [en] (Win98; U)";
const char S33204[] = "Mozilla/4.75 [en] (Win98; U)";
const char S33205[] = "Mozilla/3.0 [en] (compatible; Win98; U)";
const char S33206[] = "Mozilla/4.61 [en] (Win98; I)";
const char S33207[] = "Mozilla/4.7 (compatible; OffByOne; Windows 98) Webster Pro V3.2";
const char S33208[] = "Mozilla/4.5 [en]C-CCK-MCD {RuralNet} (Win98; I)";
const char S33209[] = "Mozilla/4.7 [en] (Win98; I)";
const char S33210[] = "Mozilla/5.0(Windows;N;Win98;m18)Gecko/X";
const char S33211[] = "Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.5) Gecko/X K-Meleon/0.8.2";
const char S33212[] = "Mozilla/4.0 (Windows; U; Win98; en-US; rv:1.7.2) Gecko/X";
const char S33213[] = "Mozilla/5.0 (Windows; U; Win 9x 4.90; en-US; rv:1.7.5) Gecko/X K-Meleon/0.9";
const char S33214[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; X10i Build/2.0.A.0.504) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S33215[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; X10i Build/2.0.2.A.0.24) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S33216[] = "Mozilla/5.0 (Linux; U; Android 2.1-Eternity|build003 Sempiternal@aioboards.com; en-gb; X10i Build/2.0.2.A.0.24) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S33217[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; SonyEricssonX10i Build/2.0.A.0.504) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S33218[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; SonyEricssonX10iv Build/2.1.A.0.435) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S33219[] = "Android (Linux; U; Android 2.1-update1; fa-ir; X10i) AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S33220[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; X10i Build/2.0.A.0.504) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S33221[] = "Mozilla/5.0 (Linux; U; Android 2.1-rooted-zdz; en-us; X10i Build/2.0.2.A.0.24) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S33222[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; X10i Build/2.1.1.A.0.28) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S33223[] = "Mozilla/5.0(Linux;U;Android2.1-update1;Zh_cn;X10i;480*854;)AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S33224[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; X10i Build/2.1.B.0.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S33225[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; X10i Build/2.1.A.0.435) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S33226[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; X10i Build/2.1.A.0.435)";
const char S33227[] = "PVPlayer X10i CORE/7.507.1.1 OpenCORE/2.05 (Linux;Android2.1-update1)";
const char S33228[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fa-ir; X10i Build/2.1.A.0.435) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S33229[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; X10i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S33230[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; X10i Build/2.0.A.0.504) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S33231[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; X10i Build/2.1.1.A.0.16) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S33232[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; X10i Build/2.1.A.0.435) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S33233[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LG-P990 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S33234[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Optimus 2X Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33235[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en; LG-P990 Build/GRJ23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S33236[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P990 Build/GRJ23)";
const char S33237[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; LG-P990hN Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S33238[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; Optimus 2X Build/GRJ23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33239[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Optimus 2X Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33240[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; LG-P990/V20e Build/GRJ23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S33241[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; LG-P990 Build/GRJ23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S33242[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; Optimus 2X Build/GRJ22)";
const char S33243[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; LG-P990/V20d Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S33244[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; Optimus 2X Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33245[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Optimus 2X Build/FRG83G; LG-P990 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33246[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; LG-P990h Build/GRJ23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S33247[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Optimus 2X Build/WIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33248[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-sa; LG-P990h Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S33249[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-ch; LG-P990/V20c Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S33250[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LG-P990 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S33251[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Optimus 2X Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S33252[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P990hN Build/GRJ22)";
const char S33253[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LG-P990 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S33254[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Optimus 2X Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S33255[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en; LG-P990hN Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S33256[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-ch; LG-P990/V20f Build/GRJ23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S33257[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es; LG-P990/V20e Build/GRJ23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S33258[] = "Mozilla/5.0 (X11; Linux x86_64) Gecko Firefox/5.0";
const char S33259[] = "Mozilla/5.0 (X11; Linux i686 on x86_64; rv:5.0) Gecko/X Firefox/5.0 Iceweasel/5.0";
const char S33260[] = "Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaC7-00/014.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.8.10 3gpp-gba";
const char S33261[] = "Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaC7-00s/022.014; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.37 Mobile Safari/533.4 3gpp-gba";
const char S33262[] = "Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaC7-00/022.014; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.37 Mobile Safari/533.4 3gpp-gba";
const char S33263[] = "Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaC7-00.1/024.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.37 Mobile Safari/533.4 3gpp-gba";
const char S33264[] = "Mozilla/5.0 (Symbian/3; Series60/5.2NokiaC7-00.1/024.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.37 Mobile Safari/533.4 3gpp-gba";
const char S33265[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC_Bliss_S510b Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33266[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC_S510b V1.36.841.3 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33267[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_Rhyme_S510b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33268[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC_S510b V1.36.841.5 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33269[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_Bliss_S510b Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33270[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_Bliss_S510b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33271[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC_S510b V2.14.841.4 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33272[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Rhyme S510b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33273[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Bliss_S510b Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33274[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Rhyme S510b Build/GRJ90)";
const char S33275[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Rhyme S510b Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S33276[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC_S510b V1.36.841.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33277[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC_S510b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33278[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC S510b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33279[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC_S510b V1.36.841.5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33280[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC S510b Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S33281[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Rhyme S510b Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S33282[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC S510b Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S33283[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-; HTC_Rhyme_S510b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33284[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:18.0) Gecko/X Firefox/18.0";
const char S33285[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:18.0) Gecko/X Firefox/18.0";
const char S33286[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaC5-03.2/20.1.024; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.25 Mobile Safari/533.4 3gpp-gba";
const char S33287[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaC5-03.2/12.1.023; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.9 3gpp-gba";
const char S33288[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaC5-03/11.0.024; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.9 3gpp-gba";
const char S33289[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaC5-03/22.0.007; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 Mobile Safari/533.4 3gpp-gba";
const char S33290[] = "Mozilla/4.0 (compatible; MSIE 8.0; Win32)";
const char S33291[] = "Mozilla/4.0 (compatible; MSIE 8.0; Win64; Trident/4.0)";
const char S33292[] = "Mozilla/5.0 (Windows; U; MSIE 8.0; WIndows NT 9.0; en-US))";
const char S33293[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows)";
const char S33294[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT)";
const char S33295[] = "Mozilla/5.0 (compatible; MSIE 8.0; 68K; Trident/4.0)";
const char S33296[] = "Mozilla/5.0 (compatible; MSIE 8.0; 68K; Win64; x64; Trident/5.0)";
const char S33297[] = "Mozilla/5.0 (compatible; MSIE 8.0; 68K; SV1; Trident/5.0)";
const char S33298[] = "Mozilla/5.0 (compatible; MSIE 8.0; PPC; WOW64; Trident/1.0)";
const char S33299[] = "Mozilla/5.0 (compatible; MSIE 8.0; PPC; .NET CLR; Trident/2.0)";
const char S33300[] = "Mozilla/5.0 (compatible; MSIE 8.0; 68K; WOW64; Trident/5.0)";
const char S33301[] = "Mozilla/5.0 (compatible; MSIE 8.0; PPC; Win64; IA64; Trident/2.0)";
const char S33302[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaC5-00/071.005; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.26 Mobile Safari/533.4 3gpp-gba";
const char S33303[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaC5-00.2/071.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.26 Mobile Safari/533.4 3gpp-gba";
const char S33304[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaC5-00.1/071.005; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.26 Mobile Safari/533.4 3gpp-gba";
const char S33305[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaC5-00.1/032.010; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.5.2";
const char S33306[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaC5-00/032.010; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.5.2";
const char S33307[] = "NokiaC5-00i/071.003 (SymbianOS/9.3; U; Series60/3.2 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/533.4 (KHTML, like Gecko) Version/3.0 Safari/533.4 3gpp-gba";
const char S33308[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaC5-00.3/071.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.26 Mobile Safari/533.4 3gpp-gba";
const char S33309[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S33310[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:20.0) Gecko/X Firefox/20.0";
const char S33311[] = "Mozilla/5.0 (Windows NT 6.0; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.1";
const char S33312[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.1-x64";
const char S33313[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JWR66N) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33314[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33315[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33316[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JWR66X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33317[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33318[] = "Dalvik/1.6.0 (Linux; U; Android 4.3; Galaxy Nexus Build/JWR66Y)";
const char S33319[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33320[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-; Galaxy Nexus Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33321[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33322[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33323[] = "Opera/9.80 (X11; Linux x86_64; U; en) Presto/2.9.168 Version/11.50";
const char S33324[] = "Opera/9.80 (Linux armv7l; U; en) Presto/2.9.201 Version/11.50";
const char S33325[] = "Mozilla/5.0 (X11; Linux i686; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.50";
const char S33326[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT26w Build/6.1.A.2.45)";
const char S33327[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; SonyEricssonLT26w Build/6.1.A.2.45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33328[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT26w Build/6.1.A.2.45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33329[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT26w Build/6.1.A.2.50) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33330[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26w Build/6.1.A.2.45) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S33331[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT26w Build/6.1.A.1.58) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33332[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT26w Build/6.1.A.2.55) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33333[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; IdeaTab A2107A-H Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S33334[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; IdeaTab A2107A-H Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33335[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; IdeaTab A2107A-F Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33336[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb, SmartTabII7 Build/A2107A_A404_010_002_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33337[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ro-ro, SmartTabII7 Build/A2107A_A404_004_043_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33338[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; IdeaTab A2107A-H Build/IML74K)";
const char S33339[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ie, SmartTabII7 Build/A2107A_A404_004_043_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33340[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A2107A-H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33341[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; IdeaTab A2107A-F Build/IML74K)";
const char S33342[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr, SmartTabII7 Build/A2107A_A404_004_043_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33343[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt, SmartTabII7 Build/A2107A_A404_010_002_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33344[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es, SmartTabII7 Build/A2107A_A404_010_002_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33345[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SmartTabII7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33346[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it, SmartTabII7 Build/A2107A_A404_107_055_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33347[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de, SmartTabII7 Build/A2107A_A404_107_055_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33348[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-be, SmartTabII7 Build/A2107A_A404_004_043_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33349[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr, SmartTabII7 Build/A2107A_A404_004_043_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33350[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu, SmartTabII7 Build/A2107A_A404_004_043_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33351[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz, SmartTabII7 Build/A2107A_A404_004_043_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33352[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl, SmartTabII7 Build/A2107A_A404_108_057_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33353[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us, SmartTabII7 Build/A2107A_A404_107_055_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33354[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SmartTabII7 Build/IMM76D)";
const char S33355[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; A2107A-H Build/IMM76D)";
const char S33356[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; GT-I9000 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S33357[] = "Mozilla/5.0 (Linux; U; Android 2.1; fr-fr; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33358[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; GT-I9000B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S33359[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; GT-I9000M Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S33360[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I9000 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S33361[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; GT-I9000M Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.1;CBCAndroidNewsAppUserAgent";
const char S33362[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I9000 Build/ECLAIR)";
const char S33363[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I9000M Build/ECLAIR)";
const char S33364[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I9000 Build/ECLAIR) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S33365[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn;GT-I9000Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S33366[] = "2.1 GT-I9000---Mozilla/5.0 (Linux; U; Android 2.1; xx-xx; GT-I9000 Build/ECLAIR) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S33367[] = "Mozilla/5.0 (Linux; U; Android 2.1.1; pl-pl; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 WykopDroid";
const char S33368[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9850; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.755 Mobile Safari/534.11+";
const char S33369[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-79) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S33370[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT925 Build/9.8.2Q-8-XT925_VQL-12.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S33371[] = "Mozilla/5.0 (Linux; Android 4.1.1; DROID RAZR HD Build/9.8.1Q_39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S33372[] = "Mozilla/5.0 (Linux; Android 4.1.1; XT925 Build/9.8.0Q-97-XT925_VQU-19) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S33373[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A525 Safari/8536.25";
const char S33374[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A405 Safari/8536.25";
const char S33375[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A403 Safari/8536.25";
const char S33376[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A523 Safari/8536.25";
const char S33377[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A551 Safari/8536.25";
const char S33378[] = "Mozilla/5.0 (iPad; U; CPU OS 6_1 like Mac OS X; zh-cn Model:iPad3,4) UC AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7543.48.3";
const char S33379[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari Version/5.1";
const char S33380[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B141 Safari/7536.26";
const char S33381[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Version/5.1 Safari/7534.48.3 Kikin/1.9.5";
const char S33382[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Version/5.1 Safari/7534.48.3 NAVER(inapp; search; 103; 4.3.0)";
const char S33383[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S33384[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Version/5.1 Safari/7534.48.3 NAVER(inapp; search; 103; 4.3.0)";
const char S33385[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B146 Safari/7536.26";
const char S33386[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari Version/5.1";
const char S33387[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S33388[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B147 Safari Version/5.1";
const char S33389[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S33390[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari Version/5.1";
const char S33391[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B329 Safari/7536.26";
const char S33392[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B147 Version/5.1 Safari/7534.48.3 NAVER(inapp; search; 103; 4.3.0)";
const char S33393[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B147 Safari/7536.26";
const char S33394[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Version/5.1 Safari/7534.48.3 NAVER(inapp; search; 310; 4.7.1)";
const char S33395[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyC6603 Build/10.1.A.1.350) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33396[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C6603 Build/10.1.A.1.350) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33397[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; C6603 Build/10.1.A.1.434)";
const char S33398[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C6603 Build/10.1.A.1.434) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33399[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyC6603 Build/10.1.A.1.434) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33400[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyC6603 Build/10.1.1.A.1.253) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33401[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; C6603 Build/10.1.1.A.1.253) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33402[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyC6603 Build/10.1.1.A.1.307) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33403[] = "SonyC6603 Build/10.1.A.1.434 stagefright/1.2 (Linux;Android 4.1.2)";
const char S33404[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C6603 Build/10.1.1.A.1.307) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33405[] = "SonyC6603 Build/10.1.1.A.1.253 stagefright/1.2 (Linux;Android 4.1.2)";
const char S33406[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; C6603 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33407[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JWR66N) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33408[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33409[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33410[] = "Dalvik/1.6.0 (Linux; U; Android 4.3; Nexus 7 Build/JWR66Y)";
const char S33411[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33412[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33413[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-; Nexus 7 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33414[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33415[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JSS15R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33416[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33417[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S33418[] = "Mozilla/5.0 (Linux; Android 4.1.1; DROID RAZR HD Build/9.8.1Q_39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S33419[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT925 Build/9.8.2Q-8-XT925_VQL-12.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S33420[] = "Mozilla/5.0 (Linux; Android 4.1.2; RAZR HD Build/9.8.2Q-8-XT925_VQL-6) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S33421[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT925 Build/9.8.2Q-8-XT925_VQL-12.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S33422[] = "Mozilla/5.0 (Linux; Android 4.1.1; DROID RAZR HD Build/9.8.1Q_39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S33423[] = "Mozilla/5.0 (Linux; Android 4.1.1; XT925 Build/9.8.0Q-97-XT925_VQU-19) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S33424[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-79) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S33425[] = "Opera/9.80 (Windows NT 6.1; U; en) Presto/2.9.168 Version/11.50";
const char S33426[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.9.168 Version/11.50";
const char S33427[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.9.168 Version/11.50";
const char S33428[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.9.168 Version/11.50";
const char S33429[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.9.168 Version/11.50";
const char S33430[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.9.168 Version/11.50";
const char S33431[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.9.168 Version/11.50";
const char S33432[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.9.168 Version/11.50,gzip(gfe) (via translate.google.com)";
const char S33433[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.9.168 Version/11.50";
const char S33434[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.9.168 Version/11.50";
const char S33435[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.9.168 Version/11.50";
const char S33436[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4110); ru) Presto/2.9.168 Version/11.50";
const char S33437[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.9.168 Version/11.50";
const char S33438[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.9.168 Version/11.50";
const char S33439[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.9.168 Version/11.50";
const char S33440[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.9.168 Version/11.50";
const char S33441[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.9.168 Version/11.50";
const char S33442[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.9.168 Version/11.50";
const char S33443[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.9.168 Version/11.50";
const char S33444[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4661); ru) Presto/2.9.168 Version/11.50";
const char S33445[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4848); ru) Presto/2.9.168 Version/11.50";
const char S33446[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.9.168 Version/11.50";
const char S33447[] = "Mozilla/5.0 (Windows NT 6.1; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.50";
const char S33448[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.9.168 Version/11.50";
const char S33449[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.9.168 Version/11.50";
const char S33450[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.9.168 Version/11.50";
const char S33451[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.9.168 Version/11.50";
const char S33452[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03639); ru) Presto/2.9.168 Version/11.50";
const char S33453[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.9.168 Version/11.50";
const char S33454[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03672); ru) Presto/2.9.168 Version/11.50";
const char S33455[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03791); ru) Presto/2.9.168 Version/11.50";
const char S33456[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.9.168 Version/11.50";
const char S33457[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03638); ru) Presto/2.9.168 Version/11.50";
const char S33458[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4871); ru) Presto/2.9.168 Version/11.50";
const char S33459[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.9.168 Version/11.50";
const char S33460[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5265); ru) Presto/2.9.168 Version/11.50";
const char S33461[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.9.168 Version/11.50";
const char S33462[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.9.168 Version/11.50";
const char S33463[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.9.168 Version/11.50";
const char S33464[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.9.168 Version/11.50";
const char S33465[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.9.168 Version/11.50";
const char S33466[] = "Opera/9.80 (Windows 7; U; en) Presto/2.9.168 Version/11.50";
const char S33467[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03755); ru) Presto/2.9.168 Version/11.50";
const char S33468[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; ru) Opera 11.50";
const char S33469[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5196); ru) Presto/2.9.168 Version/11.50";
const char S33470[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5267); ru) Presto/2.9.168 Version/11.50";
const char S33471[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5328); ru) Presto/2.9.168 Version/11.50";
const char S33472[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.9.168 Version/11.50,gzip(gfe)";
const char S33473[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5308); ru) Presto/2.9.168 Version/11.50";
const char S33474[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.9.168 Version/11.50";
const char S33475[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.9.168 Version/11.50";
const char S33476[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.9.168 Version/11.50";
const char S33477[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.9.168 Version/11.50";
const char S33478[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.9.168 Version/11.50";
const char S33479[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5966); ru) Presto/2.9.168 Version/11.50";
const char S33480[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6060); ru) Presto/2.9.168 Version/11.50";
const char S33481[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6578); ru) Presto/2.9.168 Version/11.50";
const char S33482[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6089); ru) Presto/2.9.168 Version/11.50";
const char S33483[] = "Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5784); ru) Presto/2.9.168 Version/11.50";
const char S33484[] = "Mozilla/5.0 (Series40; Nokia200/11.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S33485[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonR800x Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33486[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; SonyEricssonR800x Build/3.0.E.2.89) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33487[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; R800x Build/3.0.E.2.89) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33488[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonR800x Build/3.0.1.E.0.88) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33489[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; R800x Build/3.0.1.E.0.88) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33490[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; SonyEricssonR800x Build/3.0.E.2.52) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33491[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.2; R800x Build/3.0.E.2.89)";
const char S33492[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonR800x Build/4.0.2.E.0.57) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33493[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonR800xhttp://www.sonyericsson.com/UAprof/R800xR301.xml Build/3.0.1.E.1.44) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33494[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; R800x Build/4.0.2.E.0.57) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33495[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; R800x Build/3.0.1.E.0.88)";
const char S33496[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; R800x Build/3.0.1.E.1.44) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33497[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; R800x Build/4.0.2.E.0.57)";
const char S33498[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonR800x Build/3.0.1.E.0.82) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33499[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; R800x Build/3.0.1.E.0.88) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S33500[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonR800xhttp://www.sonyericsson.com/UAprof/R800xR301.xml Build/4.0.2.E.0.63) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33501[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonR800x Build/4.0.2.E.0.57) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S33502[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonR800xBuild/4.0.2.E.0.57) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33503[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; SonyEricssonR800x Build/4.0.2.E.0.57) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33504[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;SonyEricssonR800x Build/4.0.2.E.0.57) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33505[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; R800x Build/4.0.2.E.0.63) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33506[] = "Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.20 (KHTML, like Gecko) Chrome/25.0.1338.0 Mobile Safari/537.20";
const char S33507[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S33508[] = "Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S33509[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S33510[] = "Mozilla/5.0 (Linux; Android 4.2; Galaxy Nexus Build/JOP40C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S33511[] = "Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S33512[] = "Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko/X Firefox/12.0";
const char S33513[] = "Mozilla/5.0 (Windows NT 5.1; rv:12.0) Gecko/X Firefox/12.0 Firefox/12.0";
const char S33514[] = "Mozilla/5.0WindowsNT5.1rv12.0Gecko/XFirefox/12.0";
const char S33515[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; es-ES; rv:1.9.2) Gecko/X Firefox/12.0";
const char S33516[] = "Mozilla/5.0 (compatible; Firefox/12.0; Windows NT 5.1;) vAutoBrowser/1304";
const char S33517[] = "Mozilla/5.0 (Windows NT 5.2; rv:12.0) Gecko/12.0 Firefox/12.0";
const char S33518[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:21.0) Gecko/X Firefox/21.0";
const char S33519[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:21.0) Gecko/X Firefox/21.0";
const char S33520[] = "Mozilla/5.0 (Windows NT 6.0; rv:21.0) Gecko/X Firefox/21.0 IceDragon/21.0.1.15";
const char S33521[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; fr; rv:1.9.0.1) Gecko/X Firefox/21.0";
const char S33522[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.8.131 Version/11.10";
const char S33523[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.8.131 Version/11.10,gzip(gfe) (via translate.google.com)";
const char S33524[] = "Opera/9.80 (Windows NT 5.2; U; en) Presto/2.8.131 Version/11.10";
const char S33525[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.8.131 Version/11.10";
const char S33526[] = "Opera/9.80 (Windows NT 5.1; U; en) Presto/2.8.131 Version/11.10";
const char S33527[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.8.131 Version/11.10";
const char S33528[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.8.131 Version/11.10";
const char S33529[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.8.131 Version/11.11";
const char S33530[] = "Opera/9.80 (Windows NT 5.1; U; en) Presto/2.8.131 Version/11.11";
const char S33531[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.8.131 Version/11.11";
const char S33532[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.8.131 Version/11.11";
const char S33533[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.8.131 Version/11.11";
const char S33534[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.8.131 Version/11.11";
const char S33535[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.8.131 Version/11.11";
const char S33536[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.8.131 Version/11.11";
const char S33537[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.8.131 Version/11.10";
const char S33538[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.8.131 Version/11.10";
const char S33539[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.8.131 Version/11.10";
const char S33540[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.8.131 Version/11.10";
const char S33541[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.8.131 Version/11.10";
const char S33542[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.8.131 Version/11.10";
const char S33543[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.8.131 Version/11.11";
const char S33544[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.8.131 Version/11.11";
const char S33545[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.8.131 Version/11.10";
const char S33546[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.8.131 Version/11.11";
const char S33547[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.8.131 Version/11.11";
const char S33548[] = "Opera/9.80 (Windows NT 5.2; U; en) Presto/2.8.131 Version/11.11";
const char S33549[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.8.131 Version/11.11";
const char S33550[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.8.131 Version/11.11,gzip(gfe) (via translate.google.com)";
const char S33551[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.8.131 Version/11.11";
const char S33552[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.8.131 Version/11.10";
const char S33553[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.8.131 Version/11.11";
const char S33554[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.8.131 Version/11.11";
const char S33555[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.8.131 Version/11.11";
const char S33556[] = "Opera/11.1 (Windows NT 5.1: U: en)";
const char S33557[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.8.131 Version/11.10";
const char S33558[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.8.131 Version/11.10";
const char S33559[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.8.131 Version/11.11";
const char S33560[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.8.131 Version/11.11";
const char S33561[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.8.131 Version/11.10";
const char S33562[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.8.131 Version/11.11";
const char S33563[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.8.131 Version/11.10";
const char S33564[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.8.131 Version/11.11";
const char S33565[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.8.131 Version/11.11";
const char S33566[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.8.131 Version/11.10";
const char S33567[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.8.131 Version/11.11";
const char S33568[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.8.131 Version/11.11";
const char S33569[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.8.131 Version/11.11";
const char S33570[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.8.131 Version/11.10";
const char S33571[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.8.131 Version/11.10";
const char S33572[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.8.131 Version/11.10";
const char S33573[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.8.131 Version/11.10";
const char S33574[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.8.131 Version/11.10";
const char S33575[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.8.131 Version/11.10";
const char S33576[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.8.131 Version/11.10";
const char S33577[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.8.131 Version/11.11";
const char S33578[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.8.131 Version/11.10";
const char S33579[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.8.131 Version/11.11";
const char S33580[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.8.131 Version/11.11";
const char S33581[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.8.131 Version/11.11";
const char S33582[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.8.131 Version/11.11";
const char S33583[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.8.131 Version/11.11";
const char S33584[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.8.131 Version/11.11";
const char S33585[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.8.131 Version/11.10";
const char S33586[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.8.131 Version/11.11";
const char S33587[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.8.131 Version/11.11";
const char S33588[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.8.131 Version/11.10";
const char S33589[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.8.131 Version/11.11";
const char S33590[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.8.131 Version/11.10";
const char S33591[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4871); ru) Presto/2.8.131 Version/11.11";
const char S33592[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.8.131 Version/11.11";
const char S33593[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.8.131 Version/11.10";
const char S33594[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.8.131 Version/11.10";
const char S33595[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.8.131 Version/11.11";
const char S33596[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.8.131 Version/11.10";
const char S33597[] = "Opera/11.1 (Windows NT 5.1; U; en)";
const char S33598[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.8.131 Version/11.10";
const char S33599[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.8.131 Version/11.11";
const char S33600[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.8.131 Version/11.10";
const char S33601[] = "Opera/11.1 (Windows NT 5.2: U; en)";
const char S33602[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.8.131 Version/11.10,gzip(gfe)";
const char S33603[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.8.131 Version/11.11";
const char S33604[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.8.131 Version/11.11";
const char S33605[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.8.131 Version/11.11,gzip(gfe)";
const char S33606[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.8.131 Version/11.11";
const char S33607[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.8.131 Version/11.10";
const char S33608[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.8.131 Version/11.11";
const char S33609[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.8.131 Version/11.10";
const char S33610[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.8.131 Version/11.10";
const char S33611[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.8.131 Version/11.10";
const char S33612[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.8.131 Version/11.10";
const char S33613[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.8.131 Version/11.11";
const char S33614[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6060); ru) Presto/2.8.131 Version/11.11";
const char S33615[] = "Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5880); ru) Presto/2.8.131 Version/11.11";
const char S33616[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.8.131 Version/11.10";
const char S33617[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.8.131 Version/11.10";
const char S33618[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MZ604 Build/I.7.1-14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33619[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; MZ604 Build/I.7.1-34) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33620[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; MZ604 Build/I.7.1-34_KDDI-6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33621[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; MZ604 Build/I.7.1-42) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33622[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; MZ604 Build/I.7.1-34)";
const char S33623[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MZ604 Build/I.7.1-39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33624[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MZ604 Build/I.7.1-45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S33625[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MZ604 Build/I.7.1-45)";
const char S33626[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; HTC_PH39100/3.15.0.0 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33627[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC PH39100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33628[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_PH39100/3.26.502.56 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33629[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC PH39100 Build/IML74K)";
const char S33630[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_PH39100/3.26.502.52 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33631[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_PH39100/3.26.502.54 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33632[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_PH39100/1.63.502.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33633[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; HTC_PH39100/3.26.502.56 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33634[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; HTC PH39100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33635[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTCPH39100/3.26.502.56 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33636[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC PH39100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33637[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11";
const char S33638[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11";
const char S33639[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S33640[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S33641[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S33642[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S33643[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S33644[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S33645[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S33646[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe) (via docs.google.com/viewer)";
const char S33647[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1060) AppleWebKit/536.11 (KHTML like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S33648[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/20.0.648.205 Safari/534.16";
const char S33649[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/20.0.648.205 Safari/534.16";
const char S33650[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/20.0.648.205 Safari/534.16";
const char S33651[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:16.0) Gecko/X Firefox/16.0";
const char S33652[] = "Mozilla/6.0 (Macintosh; Intel Mac OS X 10.6; rv:16.0) Gecko/X Firefox/16.0";
const char S33653[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22";
const char S33654[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22";
const char S33655[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7";
const char S33656[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7";
const char S33657[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50";
const char S33658[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2";
const char S33659[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3";
const char S33660[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16";
const char S33661[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3";
const char S33662[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2";
const char S33663[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16";
const char S33664[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50";
const char S33665[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.3 Safari/534.53.10";
const char S33666[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.3 Safari/534.53.10";
const char S33667[] = "PycURL/7.19.5";
const char S33668[] = "Mozilla/5.0 (compatible; pycurl)";
const char S33669[] = "PycURL/7.19.7";
const char S33670[] = "PycURL/7.21.3";
const char S33671[] = "PycURL/7.21.2";
const char S33672[] = "PycURL/7.15.5";
const char S33673[] = "PycURL/7.25.0";
const char S33674[] = "PycURL/7.23.1";
const char S33675[] = "PycURL/7.18.2";
const char S33676[] = "PycURL/7.19.4";
const char S33677[] = "PycURL/7.21.6";
const char S33678[] = "PycURL/7.21.0";
const char S33679[] = "PycURL/7.22.0";
const char S33680[] = "PycURL/7.21.4";
const char S33681[] = "PycURL/7.24.0";
const char S33682[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; VM670 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33683[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; VM670 Build/Aphrodite Extra Pure 1.x) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33684[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; VM670 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33685[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-; VM670 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33686[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-; VM670 Build/FRG83) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S33687[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; VM670 Build/FRG83)";
const char S33688[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; VM670 Build/FRG83G)";
const char S33689[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-VM670 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33690[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; VM670 Build/FRG83) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S33691[] = "Mozilla/5.0 (Linux; U; Android 2.2; en_us; VM670 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)";
const char S33692[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; VM670 Build/FRG83G) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S33693[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-; VM670 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33694[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; VM670 Build/OverDrive) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33695[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; VM670 Build/StockOV) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33696[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9380; en-GB) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.513 Mobile Safari/534.11+";
const char S33697[] = "Mozilla/5.0 (Linux; Android 4.2; Nexus 7 Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S33698[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S33699[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 7 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S33700[] = "Mozilla/5.0 (Linux; U; Android 2.2.1 Gingerlike Joneidy; en-us; LG-MS690 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33701[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-MS690 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33702[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-MS690 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33703[] = "Mozilla/5.0 (Linux; U; Android 2.2.1 Sense Rom Joneidy 7/18/2011; en-us; LG-MS690 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33704[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-MS690 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33705[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-; LG-MS690 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33706[] = "Mozilla/5.0 (Linux; U; Android 2.2.1 Sense Rom Joneidy; en-us; LG-MS690 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33707[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-MS690 Build/FRG83) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S33708[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; LG-MS690 Build/FRG83)";
const char S33709[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; LG-MS690 Build/FRF91)";
const char S33710[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-MS690 Build/FRG83G)";
const char S33711[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-; LG-MS690 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33712[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-; LG-MS690 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33713[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; Touch)";
const char S33714[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; Win64; x64; Trident/6.0; Touch)";
const char S33715[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; Trident/6.0; Touch)";
const char S33716[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0)";
const char S33717[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; Win64; x64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; MDDCJS)";
const char S33718[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E) chromeframe/8.0.552.224";
const char S33719[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 3.0.04506.30; .NET4.0C; .NET4.0E) chromeframe/8.0.552.224";
const char S33720[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10 chromeframe/8.0.552.224";
const char S33721[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 3.0.04506.30; .NET4.0C; .NET4.0E) chromeframe/8.0.552.224";
const char S33722[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10 chromeframe/8.0.552.224";
const char S33723[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1) chromeframe/8.0.552.224";
const char S33724[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; GX755 XPUSV1.X; SVCVersion 7.30; SVCVersion 8.11; SVCVersion 8.14; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; InfoPath.1) chromeframe/8.0.552.224";
const char S33725[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0) chromeframe/8.0.552.237";
const char S33726[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; Zune 4.7; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729) chromeframe/8.0.552.237";
const char S33727[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9505 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33728[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9505 Build/JDQ39)";
const char S33729[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9505 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33730[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-; GT-I9505 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33731[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9505G Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33732[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9505G Build/JDQ39)";
const char S33733[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9505 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33734[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC_IncredibleS_S710e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33735[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTCS710e/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33736[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; HTC Incredible S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33737[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Incredible S Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33738[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ru-; HTC_IncredibleS_S710e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33739[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Incredible S Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33740[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Incredible S Build/GRI40)";
const char S33741[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Incredible S Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S33742[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC_IncredibleS_S710e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33743[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTCS710e/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33744[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC Incredible S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33745[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Incredible S Build/GRJ90)";
const char S33746[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Incredible S Build/Nik's TriNity v4.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33747[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Incredible S Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S33748[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; Incredible S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33749[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_IncredibleS_S710e Build/Nik's TriNity v4.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33750[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-; HTC_IncredibleS_S710e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33751[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Incredible S Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S33752[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-kr; HTC_S710E Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33753[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-kr; HTC_S710E Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33754[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Incredible S Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S33755[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; Incredible S Build/GRI40)";
const char S33756[] = "Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Incredible S Build GRJ90) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S33757[] = "Mozilla 5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Incredible S Build GRI40) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S33758[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; Incredible S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33759[] = "Mozilla/5.0 (X11; U; Linux x86_64; iw-il) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9174AT";
const char S33760[] = "Mozilla/5.0 (X11; U; Linux x86_64; de-de) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9737AP";
const char S33761[] = "Mozilla/5.0 (X11; U; Linux x86_64; fr-fr) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9737AT";
const char S33762[] = "Mozilla/5.0 (X11; U; Linux x86_64; in-id) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9737AP";
const char S33763[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-gb) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP";
const char S33764[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AT";
const char S33765[] = "Mozilla/5.0 (X11; U; Linux x86_64; ko-kr) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP";
const char S33766[] = "Mozilla/5.0 (X11; U; Linux x86_64; vi-vn) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP";
const char S33767[] = "Mozilla/5.0 (X11; U; Linux x86_64; tr-tr) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP";
const char S33768[] = "Mozilla/5.0 (X11; U; Linux x86_64; fa-ir) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP";
const char S33769[] = "Mozilla/5.0 (X11; U; Linux x86_64; pl-pl) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AT";
const char S33770[] = "Mozilla/5.0 (X11; U; Linux x86_64; zh-tw) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP";
const char S33771[] = "Mozilla/5.0 (X11; U; Linux x86_64; ar-ae) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP";
const char S33772[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-ca) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP";
const char S33773[] = "Mozilla/5.0 (X11; U; Linux x86_64; fa-fa) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP";
const char S33774[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP";
const char S33775[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-in) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP";
const char S33776[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-au) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AT";
const char S33777[] = "Mozilla/5.0 (X11; U; Linux x86_64; it-it) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AT";
const char S33778[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-es) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AT";
const char S33779[] = "Mozilla/5.0 (X11; U; Linux x86_64; pt-br) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP";
const char S33780[] = "Mozilla/5.0 (X11; U; Linux x86_64; ru-ru) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AT";
const char S33781[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-dz) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP";
const char S33782[] = "Mozilla/5.0 (X11; U; Linux x86_64; nl-nl) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AT";
const char S33783[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-sg) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP";
const char S33784[] = "Mozilla/5.0 (X11; U; Linux x86_64; es-mx) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP";
const char S33785[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-ph) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP";
const char S33786[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-ae) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP";
const char S33787[] = "Mozilla/5.0 (X11; U; Linux x86_64; fa-) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP";
const char S33788[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-nz) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AT";
const char S33789[] = "Mozilla/5.0 (X11; U; Linux x86_64; pt-pt) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AT";
const char S33790[] = "Mozilla/5.0 (X11; U; Linux x86_64; zh-cn) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP";
const char S33791[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-ie) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP";
const char S33792[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 Build/4.5.1_57_DR2-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33793[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2Build/4.5.1_57_DR2-31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33794[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 Build/4.5.1_57_DR2-31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S33795[] = "Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; DROID2 Build/4.5.1_57_DR2-31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33796[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 Build/4.5.1_57_DR2-31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33797[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;DROID2 Build/4.5.1_57_DR2-31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33798[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 Build/4.5.1_57_DR2-29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33799[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en_us; DROID2 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.2 Mobile (Dragonfruit)";
const char S33800[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 Build/STAB) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33801[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; DROID2 Build/4.5.1_57_DR2-31)";
const char S33802[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID2 Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33803[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID2 Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33804[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID2 Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33805[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID2 Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33806[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Droid 2 Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33807[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 Build/4.5.1_57_DR2-31) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S33808[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; DROID2 Build/4.5.1_57_DR4-51)";
const char S33809[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; DROID2 Build/4.5.1_57_DR4-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33810[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID2Build/4.5.1_57_DR4-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33811[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID2 Build/4.5.1_57_DR4-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33812[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID2 Build/4.5.1_57_DR4-52) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33813[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;DROID2 Build/4.5.1_57_DR4-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33814[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID2 Build/4.5.1_57_DR4-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S33815[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; DROID2 Build/4.5.1_57_DR4-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33816[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; DROID2 Build/VZW)";
const char S33817[] = "Mozilla/4.0 (compatible; MSIE 4.01; Digital AlphaServer 1000A 4/233; Windows NT; Powered By 64-Bit Alpha Processor)";
const char S33818[] = "Mozilla/3.0 (compatible; MSIE 4.0; Win32)";
const char S33819[] = "Mozilla/4.0 (compatible; MSIE 4.0; Windows NT)";
const char S33820[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows; U; 32bit)";
const char S33821[] = "Mozilla/4.0 (compatible; MSIE 4.0; Win32)";
const char S33822[] = "Mozilla/4.5 (compatible; MSIE 4.01; Windows NT)";
const char S33823[] = "Mozilla/5.0 (compatible; MSIE 4.0; 68K; Trident/5.0)";
const char S33824[] = "Mozilla/5.0 (compatible; MSIE 4.0; 68K; WOW64; Trident/5.0)";
const char S33825[] = "Mozilla/5.0 (compatible; MSIE 4.0; 68K; Win64; IA64; Trident/4.0)";
const char S33826[] = "Mozilla/5.0 (compatible; MSIE 4.0; 68K; SV1; Trident/2.0)";
const char S33827[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; SAMSUNG-SGH-I717 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33828[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SAMSUNG-SGH-I717 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33829[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33830[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D)";
const char S33831[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33832[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I717 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33833[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SAMSUNG-SGH-I717 Build/IML74K)";
const char S33834[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S33835[] = "MAUI WAP Browser";
const char S33836[] = "MAUI_WAP_Browser";
const char S33837[] = "MAUI WAP Browser, MAUI WAP Browser";
const char S33838[] = "Maui Browser";
const char S33839[] = "MAUI WAP Browser/WAP2.0 Profile/MT6253/QCIF";
const char S33840[] = "MAUI WAP Browser/WAP2.0 Profile/MT6253/QVGA";
const char S33841[] = "M70/Q03C MAUI-browser/MIDP2.0/CLDC1.1/Screen-240X400";
const char S33842[] = "Maui Browser, Maui Browser";
const char S33843[] = "D66/Q03C MAUI-browser/Screen-240X320";
const char S33844[] = "TELSDA55_11B_HW (MRE\\2.3.00(2048) resolution\\X chipset\\MT6255 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) G68_LS_F2_EN_X Release/2012.08.30 WAP Browser/MAUI Profile/  Q03C1-2.40 en-US";
const char S33845[] = "TELSDA55_11B_HW (MRE\\2.3.00(2048) resolution\\X chipset\\MT6255 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) G68_LS_WIFI_TV_EN_X Release/2012.08.16 WAP Browser/MAUI Profile/  Q03C1-2.40 en-US";
const char S33846[] = "TELSDA55_11B_HW (MRE\\2.3.00(2048) resolution\\X chipset\\MT6255 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) G68_LS_EN_X Release/2012.08.06 WAP Browser/MAUI Profile/  Q03C1-2.40 en-US";
const char S33847[] = "maui TRUSTED/1.0";
const char S33848[] = "SYM_FT04i_MAUI WAP Browser";
const char S33849[] = "CKT50_COSMOS_11B_HW (MRE\\2.5.00(2500) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) SLT-S04A_UTL_L2EN_202_X Release/2013.01.06 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US";
const char S33850[] = "TINNO50_6464_W1224_11B_HW (MRE\\2.5.00(800) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\1 gsensor\\0 keyboard\\normal) F7013B_PP_F2_PK_V8 Release/2012.11.29 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US";
const char S33851[] = "DEWAV50_COSMOS_SLIM_11B_HW (MRE/3.0.00(2500);MAUI/_DL32571_INTEX-PLAYER_L101;BDATE/2013/03/19 17:43;LCD/X;CHIP/MT6250;KEY/Reduced;TOUCH/1;CAMERA/0;SENSOR/0;DEV/DEWAV50_COSMOS_SLIM_11B_HW;WAP Browser/MAUI (HTTP PGDL;HTTPS);GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/005;) L3257_DL32571_INTEX-PLAYER_L101 Release/2013.03.19 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US";
const char S33852[] = "CKT50_COSMOS_11B_HW (MRE\\2.5.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) SLT-S04A_UTL_L2EN_206_X Release/2013.04.10 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US";
const char S33853[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e)";
const char S33854[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar 4G)";
const char S33855[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar; Orange)";
const char S33856[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e; 2.01.161.03)";
const char S33857[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar)";
const char S33858[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e; 2.05.161.01)";
const char S33859[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e; 1.08.164.02)";
const char S33860[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e; 2.01.164.03)";
const char S33861[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e; 1.08.161.01)";
const char S33862[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e; 1.08.172.01)";
const char S33863[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e; 4.06.164.01)";
const char S33864[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e; 4.06.166.01)";
const char S33865[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e; 4.06.172.01)";
const char S33866[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e; 4.06.161.02)";
const char S33867[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e; 4.06.168.01)";
const char S33868[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18";
const char S33869[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13";
const char S33870[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.22";
const char S33871[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_7; en-us) AppleWebKit/526.1+ (KHTML, like Gecko) Version/3.1 Safari/525.13";
const char S33872[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaN97-1/22.0.110; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) BrowserNG/7.1.4";
const char S33873[] = "NokiaN97/22.1.112 (SymbianOS/9.4; Series60/5.0 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebkit/525 (KHTML, like Gecko) BrowserNG/7.1.4";
const char S33874[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaN97-4/30.0.004; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.28 3gpp-gba";
const char S33875[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaN97-5/30.2.004; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.28 3gpp-gba";
const char S33876[] = "NokiaN97i/22.0.110 (SymbianOS/9.4; Series60/5.0 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebkit/525 (KHTML, like Gecko) BrowserNG/7.1.4";
const char S33877[] = "Mozilla/4.0 (compatible; Nokia Podcasting; SymbianOS; NokiaN97-1/22.0.110)";
const char S33878[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaN97-1/22.0.110; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) NokiaBrowser/7.3.1.25 3gpp-gba";
const char S33879[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaN97-5/11.2.102; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) BrowserNG/7.1.4";
const char S33880[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaN97-3/22.2.110; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) BrowserNG/7.1.4";
const char S33881[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaN97-4/12.0.110; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) BrowserNG/7.1.4";
const char S33882[] = "NokiaN97";
const char S33883[] = "Mozilla/5.0 (compatible; Yahoo! Slurp/3.0; http://help.yahoo.com/help/us/ysearch/slurp) NOT Firefox/3.5";
const char S33884[] = "Mozilla/5.0 (compatible; Yahoo! Slurp/3.0; http://help.yahoo.com/help/us/ysearch/slurp) NOT Firefox/3.5,gzip(gfe) (via docs.google.com/viewer)";
const char S33885[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT)";
const char S33886[] = "Mozilla/4.0 (compatible; MSIE 5.5; Win32)";
const char S33887[] = "LeapTag (compatible; Mozilla 4.0; MSIE 5.5; http://beta.leaptag.com/?p=win32&v=0.8.4.trunk.r5774)";
const char S33888[] = "Mozilla/4.0 (compatible; MSIE 5.50; Windows NT; SiteKiosk 4.8; SiteCoach 1.0)";
const char S33889[] = "Mozilla/5.0 (compatible; MSIE 5.5; Windows NT)";
const char S33890[] = "Mozilla/5.0(compatible;MSIE 5.5;Windows NT)";
const char S33891[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5380K/S5380KDDKK6; U; Bada/2.0; en-us) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile HVGA SMM-MMS/1.2.0 OPN-B";
const char S33892[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5380B/1.0; U; Bada/2.0; pt-br) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile HVGA SMM-MMS/1.2.0 OPN-B";
const char S33893[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5380/S5380AELB3; U; Bada/2.0; fr-fr) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile HVGA SMM-MMS/1.2.0 OPN-B";
const char S33894[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5380D/S5380DGTLE1; U; Bada/2.0; en-us) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile HVGA SMM-MMS/1.2.0 OPN-B";
const char S33895[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5380L/1.0; U; Bada/2.0; es) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile HVGA SMM-MMS/1.2.0 OPN-B";
const char S33896[] = "Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.16";
const char S33897[] = "Opera/9.80 (Windows NT 6.0; WOW64) Presto/2.12.388 Version/12.16";
const char S33898[] = "Opera/9.80 (Windows NT 6.0; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.16";
const char S33899[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.16";
const char S33900[] = "Opera/9.80 (Windows NT 6.0; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.16";
const char S33901[] = "Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.16,gzip(gfe)";
const char S33902[] = "Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.16";
const char S33903[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.16";
const char S33904[] = "Opera/9.80 (Windows NT 6.1; WOW64; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)) Presto/2.12.388 Version/12.16";
const char S33905[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.16";
const char S33906[] = "Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.16";
const char S33907[] = "Mozilla/5.0 (Windows NT 6.0; rv:2.0) Gecko/X Firefox/4.0 Opera 12.16";
const char S33908[] = "Opera/9.80 (Windows NT 6.0; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.16";
const char S33909[] = "Opera/9.80 (Windows NT 6.0; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.16";
const char S33910[] = "Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.16";
const char S33911[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.16";
const char S33912[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.16";
const char S33913[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.16";
const char S33914[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.16";
const char S33915[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.16";
const char S33916[] = "Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.16";
const char S33917[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.16";
const char S33918[] = "Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.16";
const char S33919[] = "Opera/9.80 (Windows NT 6.0; Edition Campaign 21) Presto/2.12.388 Version/12.16,gzip(gfe)";
const char S33920[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.16";
const char S33921[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.16";
const char S33922[] = "Opera/9.80 (Windows NT 6.0; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.16";
const char S33923[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.16";
const char S33924[] = "Opera/9.80 (Windows NT 6.0; MRA 8.1 (build 6337)) Presto/2.12.388 Version/12.16";
const char S33925[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.16";
const char S33926[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.16";
const char S33927[] = "Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.16,gzip(gfe),gzip(gfe)";
const char S33928[] = "Opera/9.80 (Windows NT 6.0; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.16";
const char S33929[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.16";
const char S33930[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.16";
const char S33931[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.16";
const char S33932[] = "Opera/9.80 (Windows NT 6.0; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.16";
const char S33933[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0) Opera 12.16";
const char S33934[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.16";
const char S33935[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.16";
const char S33936[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.16";
const char S33937[] = "Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.16";
const char S33938[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.16";
const char S33939[] = "Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.16";
const char S33940[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.16";
const char S33941[] = "Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.16";
const char S33942[] = "Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.16";
const char S33943[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.16";
const char S33944[] = "Mozilla/5.0 (Windows NT 6.2; rv:18.0) Gecko/X Firefox/18.0";
const char S33945[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:18.0) Gecko/X Firefox/18.0";
const char S33946[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SPH-M820-BST Build/GINGERBREAD)";
const char S33947[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SPH-M820-BST Build/GINGERBREAD)";
const char S33948[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33949[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SPH-M820-BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33950[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SPH-M820-BST Build/Chicken Tits) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33951[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; -us; SPH-M820-BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33952[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/Chicken Tits by Bloodawn) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33953[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; -us; SPH-M820-BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33954[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/Chicken-Tits by Bloodawn) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33955[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/Chicken Tits) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33956[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SPH-M820-BST Build/HONEYBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33957[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S33958[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/ROM by Bloodawn) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33959[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/Beats by Dr.Dre) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33960[] = "Mozilla/5.0 (X11; Linux i686; rv:17.0) Gecko/17.0 Firefox/17.0";
const char S33961[] = "Mozilla/5.0 (X11; Linux x86_64; rv:17.0) Gecko/X Firefox/17.0 Iceweasel/17.0.9";
const char S33962[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; HTC Mecha Build/TeamVicious-MIUI-ICECREAM) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S33963[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6425LVW 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33964[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33965[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33966[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6425LVW Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33967[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33968[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33969[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; ADR6425LVW Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33970[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; ADR6425LVW Build/IML74K)";
const char S33971[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ADR6425LVW Build/IMM76D)";
const char S33972[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6425LVW 4G Build/IMM30B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33973[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW 4G Build/Rezound) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33974[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30";
const char S33975[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW Build/Rezound) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33976[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6425LVW Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33977[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW Build/EclipticRez-Revamped) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33978[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW 4G Build/EclipticRez-Revamped) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33979[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; ADR6425LVW Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33980[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; -us; ADR6425LVW 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S33981[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:11.0)";
const char S33982[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; rv:11.0) Gecko/X";
const char S33983[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko";
const char S33984[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; Trident/7.0; rv:11.0) like Gecko";
const char S33985[] = "Windows-RSS-Platform/2.0 (IE 11.0; Windows NT 6.1)";
const char S33986[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; Touch; Tablet PC 2.0; rv:11.0) like Gecko";
const char S33987[] = "Mozilla/5.0 (AOL 9.7; AOLBuild 4343.1028; Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko";
const char S33988[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-E739 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/V1.2";
const char S33989[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-E739 Build/GRK39F)";
const char S33990[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-E739 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/V1.2";
const char S33991[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-E739 Build/GRJ22)";
const char S33992[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-E739Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/V1.2";
const char S33993[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;LG-E739 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/V1.2";
const char S33994[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-E739Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/V1.2";
const char S33995[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; LG-E739 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/V1.2";
const char S33996[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us;LG-E739 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/V1.2";
const char S33997[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; LG-E739 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/V1.2";
const char S33998[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-E739 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 MMS/LG-Android-MMS-V1.0/V1.2";
const char S33999[] = "Mozilla/5.0 (Linux; U; Android2.3.6; en-us; LG-E739 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/V1.2";
const char S34000[] = "Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; LG-E739 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/V1.2";
const char S34001[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-E739 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 MMS/LG-Android-MMS-V1.0/V1.2";
const char S34002[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; LG-E739 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Versi?n/4.0 Mobile Safari/533.1";
const char S34003[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; LG-E739 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Versi?n/4.0 Mobile Safari/533.1";
const char S34004[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-E739 Build/GRJ22; JellyBred-v1.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34005[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36";
const char S34006[] = "Mozilla/5.0 (Linux; Android 4.2; Nexus 4 Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S34007[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 4 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S34008[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S34009[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SlimBrowser/7.00.024)";
const char S34010[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SlimBrowser/7.00.024)";
const char S34011[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SlimBrowser/7.00.035)";
const char S34012[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; SlimBrowser/7.00.035)";
const char S34013[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Avant Browser; SlimBrowser/7.00.024)";
const char S34014[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Avant Browser; SlimBrowser/7.00.035)";
const char S34015[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Avant Browser; SlimBrowser/7.00.035)";
const char S34016[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Avant Browser; SlimBrowser/7.00.035)";
const char S34017[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SlimBrowser/7.00.035)";
const char S34018[] = "Opera/9.80 (BREW; Opera Mini/5.1/27.2338; U; en) Presto/2.8.119 400X240 LG VN271";
const char S34019[] = "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:22.0) Gecko/X Firefox/22.0";
const char S34020[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; SHW-M250K Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34021[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; SHW-M250S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34022[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SHW-M250S Build/IML74K)";
const char S34023[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; SHW-M250L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34024[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SHW-M250L Build/IML74K)";
const char S34025[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SHW-M250K Build/IML74K)";
const char S34026[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHW-M250S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34027[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHW-M250K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34028[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHW-M250S Build/IMM76D)";
const char S34029[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHW-M250L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34030[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHW-M250K Build/IMM76D)";
const char S34031[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHW-M250L Build/IMM76D)";
const char S34032[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; SHW-M250S Build/LPNAN_1.3.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34033[] = "Mozilla/5.0 (Linux; Android 4.2; Nexus 10 Build/JVP15P) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34034[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 10 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34035[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 10 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34036[] = "Mozilla/5.0 (Linux; Android 4.2.1; en-US; Nexus 10 Build/JOP40D) AppleWebKit/535.19 (KHTML like Gecko) Chrome/18.0.1025.166 Safari/535.19";
const char S34037[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S34038[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S34039[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 4 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S34040[] = "Mozilla/5.0 (Linux; Android 4.2; Nexus 4 Build/JOP40C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S34041[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 4 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S34042[] = "MobileSafari/8536.25 CFNetwork/609.1.4 Darwin/13.0.0";
const char S34043[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:18.0) Gecko/X Firefox/18.0";
const char S34044[] = "Mozilla/5.0 (Linux; Android 4.0.3; M1001 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34045[] = "Mozilla/5.0 (Linux; Android 4.0.3; QW TB-1207 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34046[] = "Mozilla/5.0 (Linux; Android 4.0.3; SXZ-PDX0-01 Build/SXZ-PDX0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34047[] = "Mozilla/5.0 (Linux; Android 4.0.3; DFP7005 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34048[] = "Mozilla/5.0 (Linux; Android 4.0.4; VegaCream RC2 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34049[] = "Mozilla/5.0 (Linux; Android 4.0.4; VegaCream Beta 3 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34050[] = "Mozilla/5.0 (Linux; Android 4.0.3; Wxt Tab PC 65CXi Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34051[] = "Mozilla/5.0 (Linux; Android 4.0.3; A721 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34052[] = "Mozilla/5.0 (Linux; Android 4.0.3; M805 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34053[] = "Mozilla/5.0 (Linux; Android 4.0.3; CT802 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34054[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-7200 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34055[] = "Mozilla/5.0 (Linux; Android 4.0.4; M11 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34056[] = "Mozilla/5.0 (Linux; Android 4.0.3; 97FC Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34057[] = "Mozilla/5.0 (Linux; Android 4.0.3; Android B1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34058[] = "Mozilla/5.0 (Linux; Android 4.0.4; M1001 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34059[] = "Mozilla/5.0 (Linux; Android 4.0.3; A710 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34060[] = "Mozilla/5.0 (Linux; Android 4.0.3; XELIO Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34061[] = "Mozilla/5.0 (Linux; Android 4.0.4; Play Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34062[] = "Mozilla/5.0 (Linux; Android 4.0.4; M1005 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34063[] = "Mozilla/5.0 (Linux; Android 4.0.4; MP907C Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34064[] = "Mozilla/5.0 (Linux; Android 4.0.3; TC-971 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34065[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID970 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34066[] = "Mozilla/5.0 (Linux; Android 4.0.3; QW TB-1217 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34067[] = "Mozilla/5.0 (Linux; Android 4.0.3; DFP7005B Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34068[] = "Mozilla/5.0 (Linux; Android 4.0.3; vivow Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34069[] = "Mozilla/5.0 (Linux; Android 4.0.4; EPAD Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34070[] = "Mozilla/5.0 (Linux; Android 4.0.4; BC1077 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34071[] = "Mozilla/5.0 (Linux; Android 4.0.4; 101T Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34072[] = "Mozilla/5.0 (Linux; Android 4.0.3; TC-7120 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34073[] = "Mozilla/5.0 (Linux; Android 4.0.4; cyclone Titan Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34074[] = "Mozilla/5.0 (Linux; Android 4.0.3; ALLVIEWSPEED Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34075[] = "Mozilla/5.0 (Linux; Android 4.0.4; JW101A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34076[] = "Mozilla/5.0 (Linux; Android 4.0.4; VIG07 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34077[] = "Mozilla/5.0 (Linux; Android 4.0.4; f7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34078[] = "Mozilla/5.0 (Linux; Android 4.0.4; F721 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34079[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID-WCDMA Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34080[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID970 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34081[] = "Mozilla/5.0 (Linux; Android 4.0.3; M91 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34082[] = "Mozilla/5.0 (Linux; Android 4.0.4; M805 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34083[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID9107CM Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34084[] = "Mozilla/5.0 (Linux; Android 4.0.3; A088 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34085[] = "Mozilla/5.0 (Linux; Android 4.0.4; Ray Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34086[] = "Mozilla/5.0 (Linux; Android 4.0.4; 930i Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34087[] = "Mozilla/5.0 (Linux; Android 4.0.4; PC936 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34088[] = "Mozilla/5.0 (Linux; Android 4.0.4; TC-973D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34089[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID900SC Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34090[] = "Mozilla/5.0 (Linux; Android 4.0.4; N900 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34091[] = "Mozilla/5.0 (Linux; Android 4.0.3; QW TB-1210 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34092[] = "Mozilla/5.0 (Linux; Android 4.0.3; TAB-9701 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34093[] = "Mozilla/5.0 (Linux; Android 4.0.3; SX-SP435A Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34094[] = "Mozilla/5.0 (Linux; Android 4.0.4; TC-971 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34095[] = "Mozilla/5.0 (Linux; Android 4.0.4; MiSS Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34096[] = "Mozilla/5.0 (Linux; Android 4.0.4; CW V8 Dual Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34097[] = "Mozilla/5.0 (Linux; Android 4.0.4; N1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34098[] = "Mozilla/5.0 (Linux; Android 4.0.4; M1005D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34099[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID-001 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34100[] = "Mozilla/5.0 (Linux; Android 4.0.4; Q8-A13 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34101[] = "Mozilla/5.0 (Linux; Android 4.0.4; BASE_Lutea_3 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34102[] = "Mozilla/5.0 (Linux; Android 4.0.3; 070V2 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34103[] = "Mozilla/5.0 (Linux; Android 4.0.4; GOTAB Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34104[] = "Mozilla/5.0 (Linux; Android 4.0.4; TOUCH90B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34105[] = "Mozilla/5.0 (Linux; Android 4.0.4; M9701THW Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34106[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST9 Marvel Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34107[] = "Mozilla/5.0 (Linux; Android 4.0.4; maxtouuch Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34108[] = "Mozilla/5.0 (Linux; Android 4.0.4; UM-A91 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34109[] = "Mozilla/5.0 (Linux; Android 4.0.3; 3C97 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34110[] = "Mozilla/5.0 (Linux; Android 4.0.3; TAB971 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34111[] = "Mozilla/5.0 (Linux; Android 4.0.4; 3C97 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34112[] = "Mozilla/5.0 (Linux; Android 4.0.3; Alpha GT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34113[] = "Mozilla/5.0 (Linux; Android 4.0.4; Smartab MT12 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34114[] = "Mozilla/5.0 (Linux; Android 4.0.4; Milagrow Tab Top 7.4 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34115[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID970T Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34116[] = "Mozilla/5.0 (Linux; Android 4.0.4; Exagerate XZPAD970H Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34117[] = "Mozilla/5.0 (Linux; Android 4.0.4; CRANE EVB Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34118[] = "Mozilla/5.0 (Linux; Android 4.0.4; Q29 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34119[] = "Mozilla/5.0 (Linux; Android 4.0.3; KN_MOBILE_QT_A06 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34120[] = "Mozilla/5.0 (Linux; Android 4.0.4; 9700DN Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34121[] = "Mozilla/5.0 (Linux; Android 4.0.4; INFINITY_PASSION Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34122[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMURAI Build/SAMURAI) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34123[] = "Mozilla/5.0 (Linux; Android 4.0.4; ELEMENT7V2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34124[] = "Mozilla/5.0 (Linux; Android 4.0.4; M9702TH Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34125[] = "Mozilla/5.0 (Linux; Android 4.0.4; CT710 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34126[] = "Mozilla/5.0 (Linux; Android 4.0.4; M901 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34127[] = "Mozilla/5.0 (Linux; Android 4.0.4; Q8 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34128[] = "Mozilla/5.0 (Linux; Android 4.0.4; ASTRO+9.7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34129[] = "Mozilla/5.0 (Linux; Android 4.0.4; QWTB-1280BL Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34130[] = "Mozilla/5.0 (Linux; Android 4.0.4; N9977 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34131[] = "Mozilla/5.0 (Linux; Android 4.0.4; CT710FR Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34132[] = "Mozilla/5.0 (Linux; Android 4.0.4; AW101 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34133[] = "Mozilla/5.0 (Linux; Android 4.0.4; disgo9000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34134[] = "Mozilla/5.0 (Linux; Android 4.0.4; G1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34135[] = "Mozilla/5.0 (Linux; Android 4.0.4; UM-A90 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34136[] = "Mozilla/5.0 (Linux; Android 4.0.3; Broncho N802 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34137[] = "Mozilla/5.0 (Linux; Android 4.0.4; MIRA-2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34138[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID-GSM Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34139[] = "Mozilla/5.0 (Linux; Android 4.0.4; 9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34140[] = "Mozilla/5.0 (Linux; Android 4.0.3; A750 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34141[] = "Mozilla/5.0 (Linux; Android 4.0.4; Acqua Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34142[] = "Mozilla/5.0 (Linux; Android 4.0.4; Pacha Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34143[] = "Mozilla/5.0 (Linux; Android 4.0.3; J1000 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34144[] = "Mozilla/5.0 (Linux; Android 4.0.4; JSA-MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34145[] = "Mozilla/5.0 (Linux; Android 4.0.4; Polaroid 40140 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34146[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID1107 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34147[] = "Mozilla/5.0 (Linux; Android 4.0.3; PICOpad_GEA Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34148[] = "Mozilla/5.0 (Linux; Android 4.0.3; MTC Viva Build/HuaweiU8816) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34149[] = "Mozilla/5.0 (Linux; Android 4.0.3; E-Boda Supreme X80 Dual Core Build/ICS.g12refM806A1YBD.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34150[] = "Mozilla/5.0 (Linux; Android 4.0.4; Q75 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34151[] = "Mozilla/5.0 (Linux; Android 4.0.4; Swipe AllinOne Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34152[] = "Mozilla/5.0 (Linux; Android 4.0.4; yuanda75_ics Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34153[] = "Mozilla/5.0 (Linux; Android 4.0.4; Q88 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34154[] = "Mozilla/5.0 (Linux; Android 4.0.4; Exagerate XZPAD700 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34155[] = "Mozilla/5.0 (Linux; Android 4.0.3; 740RS Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34156[] = "Mozilla/5.0 (Linux; Android 4.0.4; TPC0752 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34157[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAD1001 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34158[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB 170 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34159[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID7040T-3G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34160[] = "Mozilla/5.0 (Linux; Android 4.0.3; AN13FP Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34161[] = "Mozilla/5.0 (Linux; Android 4.0.3; M971B Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34162[] = "Mozilla/5.0 (Linux; Android 4.0.3; treq A10G Duo Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34163[] = "Mozilla/5.0 (Linux; Android 4.0.4; Z7H Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34164[] = "Mozilla/5.0 (Linux; Android 4.0.4; PC728 Build/PC728) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34165[] = "Mozilla/5.0 (Linux; Android 4.0.4; Tablet PC 2000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34166[] = "Mozilla/5.0 (Linux; Android 4.0.4; Touch Kem430 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34167[] = "Mozilla/5.0 (Linux; Android 4.0.4; RYOMA M1S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34168[] = "Mozilla/5.0 (Linux; Android 4.0.9; X12 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34169[] = "Mozilla/5.0 (Linux; Android 4.0.4; WOHTC G36 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34170[] = "Mozilla/5.0 (Linux; Android 4.0.4; MING_SII Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34171[] = "Mozilla/5.0 (Linux; Android 4.0.4; V12 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34172[] = "Mozilla/5.0 (Linux; Android 4.0.4; P10MAN R2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34173[] = "Mozilla/5.0 (Linux; Android 4.0.4; i10 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34174[] = "Mozilla/5.0 (Linux; Android 4.0.4; A21plus Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34175[] = "Mozilla/5.0 (Linux; Android 4.0.4; 101D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34176[] = "Mozilla/5.0 (Linux; Android 4.0.4; Gadmei ICS Build/E8HD-T364-1280x768) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34177[] = "Mozilla/5.0 (Linux; Android 4.0.4; SP-A20i Build/MF_ICS_02.24) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34178[] = "Mozilla/5.0 (Linux; Android 4.0.4; JXD-S6600 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34179[] = "Mozilla/5.0 (Linux; Android 4.0.4; M972w Build/M972w-eng 4.0.4 IMM76D test-keys_OTA:MID_2.1.1.1_X.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34180[] = "Mozilla/5.0 (Linux; Android 4.0.4; M9702T Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34181[] = "Mozilla/5.0 (Linux; Android 4.0.4; Tablet-PC-4 Build/ICS.g08refem618.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34182[] = "Mozilla/5.0 (Linux; Android 4.0.4; MD-0696B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34183[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAD 1001 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34184[] = "Mozilla/5.0 (Linux; Android 4.0.4; F1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34185[] = "Mozilla/5.0 (Linux; Android 4.0.4; V800H Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34186[] = "Mozilla/5.0 (Linux; Android 4.0.4; iliumPAD Build/GS.NN2R3N.421SGL.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34187[] = "Mozilla/5.0 (Linux; Android 4.0.4; myTab 7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34188[] = "Mozilla/5.0 (Linux; Android 4.0.4; Q88B-5S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34189[] = "Mozilla/5.0 (Linux; Android 4.0.4; V9-1688-CJ-0308-31S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34190[] = "Mozilla/5.0 (Linux; Android 4.0.3; Netbook PC 12700 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34191[] = "Mozilla/5.0 (Linux; Android 4.0.4; P6 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34192[] = "Mozilla/5.0 (Linux; Android 4.0.4; TOUCH78 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34193[] = "Mozilla/5.0 (Linux; Android 4.0.4; KY-901 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34194[] = "Mozilla/5.0 (Linux; Android 4.0.3; GN868 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34195[] = "Mozilla/5.0 (Linux; Android 4.0.3; P077A Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34196[] = "Mozilla/5.0 (Linux; Android 4.0.4; My Audio Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34197[] = "Mozilla/5.0 (Linux; Android 4.0.4; AMOI N820 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34198[] = "Mozilla/5.0 (Linux; Android 4.0.3; MAG101 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34199[] = "Mozilla/5.0 (Linux; Android 4.0.3; V12 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34200[] = "Mozilla/5.0 (Linux; Android 4.0.4; B928A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34201[] = "Mozilla/5.0 (Linux; Android 4.0.3; Z5 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34202[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPX_8 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34203[] = "Mozilla/5.0 (Linux; Android 4.0.3;  Artes D708 Build/ICS.g12refM708A1AST.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S34204[] = "Mozilla/5.0 (Linux; Android 4.0.3; DL HD-72 X Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34205[] = "Mozilla/5.0 (Linux; Android 4.0.4; NEO-G4 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34206[] = "Mozilla/5.0 (Linux; Android 4.0.4; Q38 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34207[] = "Mozilla/5.0 (Linux; Android 4.0.3; NT-2750 Build/MID) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34208[] = "Mozilla/5.0 (Linux; Android 4.0.4; MW0821+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34209[] = "Mozilla/5.0 (Linux; Android 4.0.4; JY139 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34210[] = "Mozilla/5.0 (Linux; Android 4.0.9; I5 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34211[] = "Mozilla/5.0 (Linux; Android 4.0.4; J-Q8B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34212[] = "Mozilla/5.0 (Linux; Android 4.0.4; OVO 2.0 Build/FR-M768) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34213[] = "Mozilla/5.0 (Linux; Android 4.0.3; W27(Dualcore) Build/ICS.w27ref.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34214[] = "Mozilla/5.0 (Linux; Android 4.0.3; POM805HCF Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34215[] = "Mozilla/5.0 (Linux; Android 4.0.4; T271 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34216[] = "Mozilla/5.0 (Linux; Android 4.0.4; V8200-Memphis-orange-Slovakia Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34217[] = "Mozilla/5.0 (Linux; Android 4.0.4; TLF TPC701 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34218[] = "Mozilla/5.0 (Linux; Android 4.0.4; V100D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34219[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHIRU SHOGUN(QuadCore) Build/SHIRU_SHOGUN) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34220[] = "Mozilla/5.0 (Linux; Android 4.0.4; tmn smart a18 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34221[] = "Mozilla/5.0 (Linux; Android 4.0.4; BT210 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34222[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID-A10 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34223[] = "Mozilla/5.0 (Linux; Android 4.0.3; 9701 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34224[] = "Mozilla/5.0 (Linux; Android 4.0.4; aigopad Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34225[] = "Mozilla/5.0 (Linux; Android 4.0.4; mfld_dv10 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34226[] = "Mozilla/5.0 (Linux; Android 4.0.4; iB750C Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34227[] = "Mozilla/5.0 (Linux; Android 4.0.4; SIROCO 3 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34228[] = "Mozilla/5.0 (Linux; Android 4.0.4; ratech77_ics2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34229[] = "Mozilla/5.0 (Linux; Android 4.0.4; SXZ-PDX0-05M Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34230[] = "Mozilla/5.0 (Linux; Android 4.0.4; LXT1001 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34231[] = "Mozilla/5.0 (Linux; Android 4.0.4; myTab10 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34232[] = "Mozilla/5.0 (Linux; Android 4.0.4; Planet II v2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34233[] = "Mozilla/5.0 (Linux; Android 4.0.4; Royqueen 702 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34234[] = "Mozilla/5.0 (Linux; Android 4.0.4; Ezcool Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34235[] = "Mozilla/5.0 (Linux; Android 4.0.4; 1688-CJ-C30Q8-31S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34236[] = "Mozilla/5.0 (Linux; Android 4.0.3; Explosion Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34237[] = "Mozilla/5.0 (Linux; Android 4.0.4; N70 DUAL CORE Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34238[] = "Mozilla/5.0 (Linux; Android 4.0.4; JT-Smart PC01 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34239[] = "Mozilla/5.0 (Linux; Android 4.0.4; J-88 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34240[] = "Mozilla/5.0 (Linux; Android 4.0.3; G1 SKY Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34241[] = "Mozilla/5.0 (Linux; Android 4.0.4; M9100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34242[] = "Mozilla/5.0 (Linux; Android 4.0.3; SZENIO IPS 9700 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34243[] = "Mozilla/5.0 (Linux; Android 4.0.4; Mini Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34244[] = "Mozilla/5.0 (Linux; Android 4.0.4; RMD-973G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34245[] = "Mozilla/5.0 (Linux; Android 4.0.3; Z500 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34246[] = "Mozilla/5.0 (Linux; Android 4.0.3; DFP7006 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34247[] = "Mozilla/5.0 (Linux; Android 4.0.4; N10-4core Build/R13.4-rc2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34248[] = "Mozilla/5.0 (Linux; Android 4.0.4; TZ100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34249[] = "Mozilla/5.0 (Linux; Android 4.0.4; MTC_968 Build/ICECREAM) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34250[] = "Mozilla/5.0 (Linux; Android 4.0.4; GNET Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34251[] = "Mozilla/5.0 (Linux; Android 4.0.4; Eliya S6 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34252[] = "Mozilla/5.0 (Linux; Android 4.0.4; S5A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34253[] = "Mozilla/5.0 (Linux; Android 4.0.4; hexing77_ics2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34254[] = "Mozilla/5.0 (Linux; Android 4.0.4; V800A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34255[] = "Mozilla/5.0 (Linux; Android 4.0.4; PICOpad GJT Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34256[] = "Mozilla/5.0 (Linux; Android 4.0.3; IS801C Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34257[] = "Mozilla/5.0 (Linux; Android 4.0.3; X2 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34258[] = "Mozilla/5.0 (Linux; Android 4.0.4; Telus HD Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34259[] = "Mozilla/5.0 (Linux; Android 4.0.4; WT-M768 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34260[] = "Mozilla/5.0 (Linux; Android 4.0.4; CW-V9-DUAL Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34261[] = "Mozilla/5.0 (Linux; Android 4.0.4; GIONEE-GN700 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34262[] = "Mozilla/5.0 (Linux; Android 4.0.4; A106 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34263[] = "Mozilla/5.0 (Linux; Android 4.0.4; A088 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34264[] = "Mozilla/5.0 (Linux; Android 4.0.4; A721 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34265[] = "Mozilla/5.0 (Linux; Android 4.0.4; J-Q88D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34266[] = "Mozilla/5.0 (Linux; Android 4.0.4; MegaFon_SP-A10 Build/IceCreamSandwich) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34267[] = "Mozilla/5.0 (Linux; Android 4.0.4; Alpha GT Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34268[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC PROMIN_U Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34269[] = "Mozilla/5.0 (Linux; Android 4.0.4; reedPAD4N Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34270[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB 170N Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34271[] = "Mozilla/5.0 (Linux; Android 4.0.4; EG950 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34272[] = "Mozilla/5.0 (Linux; Android 4.0.4; GEM9212 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34273[] = "Mozilla/5.0 (Linux; Android 4.0.4; T291 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34274[] = "Mozilla/5.0 (Linux; Android 4.0.3; NB017 / NB018 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34275[] = "Mozilla/5.0 (Linux; Android 4.0.4; Informer 921 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34276[] = "Mozilla/5.0 (Linux; Android 4.0.4; V700N Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34277[] = "Mozilla/5.0 (Linux; Android 4.0.4; M702VM Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34278[] = "Mozilla/5.0 (Linux; Android 4.0.4; JXD-P300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34279[] = "Mozilla/5.0 (Linux; Android 4.0.4; M5000 Build/1.0.528_X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34280[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB 0170N Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34281[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB 180 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34282[] = "Mozilla/5.0 (Linux; Android 4.0.3; AllviewSpeed2+ Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34283[] = "Mozilla/5.0 (Linux; Android 4.0.4; monga M8 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34284[] = "Mozilla/5.0 (Linux; Android 4.0.4; My play Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34285[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB 0170 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34286[] = "Mozilla/5.0 (Linux; Android 4.0.4; Z1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34287[] = "Mozilla/5.0 (Linux; Android 4.0.4; NOVO NV7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34288[] = "Mozilla/5.0 (Linux; Android 4.0.4; I110 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34289[] = "Mozilla/5.0 (Linux; Android 4.0.4; P102G Build/GS.NN2R3N.421S3D.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34290[] = "Mozilla/5.0 (Linux; Android 4.0.4; reedPAD4+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34291[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB 280 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34292[] = "Mozilla/5.0 (Linux; Android 4.0.4; lt1002b Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34293[] = "Mozilla/5.0 (Linux; Android 4.0.4; Quadro Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34294[] = "Mozilla/5.0 (Linux; Android 4.0.4; Condor Tab SMB-B9705 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34295[] = "Mozilla/5.0 (Linux; Android 4.0.3; T736B Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34296[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC POO_U Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34297[] = "Mozilla/5.0 (Linux; Android 4.0.3; XPRESS PRO Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34298[] = "Mozilla/5.0 (Linux; Android 4.0.4; A10-4core Build/R13.4-rc2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34299[] = "Mozilla/5.0 (Linux; Android 4.0.4; NetTAB SPACE II Plus Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34300[] = "Mozilla/5.0 (Linux; Android 4.0.4; P4i Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34301[] = "Mozilla/5.0 (Linux; Android 4.0.4; A930 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34302[] = "Mozilla/5.0 (Linux; Android 4.0.4; Mio Pad A1001 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34303[] = "Mozilla/5.0 (Linux; Android 4.0.4; 1688-XHB-2357 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34304[] = "Mozilla/5.0 (Linux; Android 4.0.4; C800 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34305[] = "Mozilla/5.0 (Linux; Android 4.0.4; Neura 3G Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34306[] = "Mozilla/5.0 (Linux; Android 4.0.4; EverGlory Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34307[] = "Mozilla/5.0 (Linux; Android 4.0.4; V71B Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34308[] = "Mozilla/5.0 (Linux; Android 4.0.3; StarPAD 7 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34309[] = "Mozilla/5.0 (Linux; Android 4.0.4; HISENSE F0 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34310[] = "Mozilla/5.0 (Linux; Android 4.0.4; UM-A13B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34311[] = "Mozilla/5.0 (Linux; Android 4.0.3; TB782B Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34312[] = "Mozilla/5.0 (Linux; Android 4.0.3; PC728 Build/PC728) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34313[] = "Mozilla/5.0 (Linux; Android 4.0.3; Herpa Pad c5900 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34314[] = "Mozilla/5.0 (Linux; Android 4.0.3; A10-MID Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34315[] = "Mozilla/5.0 (Linux; Android 4.0.3; EB-A71GJ Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34316[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID971 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34317[] = "Mozilla/5.0 (Linux; Android 4.0.3; f7 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34318[] = "Mozilla/5.0 (Linux; Android 4.0.3; NVSBL DRAGON Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34319[] = "Mozilla/5.0 (Linux; Android 4.0.3; JXD-S9100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34320[] = "Mozilla/5.0 (Linux; Android 4.0.4; OCOCCI_MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34321[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID08 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34322[] = "Mozilla/5.0 (Linux; Android 4.0.3; MB1003 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34323[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID06N Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34324[] = "Mozilla/5.0 (Linux; Android 4.0.4; I97D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34325[] = "Mozilla/5.0 (Linux; Android 4.0.3; P4 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34326[] = "Mozilla/5.0 (Linux; Android 4.0.4; D101 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34327[] = "Mozilla/5.0 (Linux; Android 4.0.4; HLV-T702 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34328[] = "Mozilla/5.0 (Linux; Android 4.0.4; S5/C2_77_5647 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34329[] = "Mozilla/5.0 (Linux; Android 4.0.4; GSmart G1342 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34330[] = "Mozilla/5.0 (Linux; Android 4.0.3; 097Q2 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34331[] = "Mozilla/5.0 (Linux; Android 4.0.3; TAB468 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34332[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT685 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34333[] = "Mozilla/5.0 (Linux; Android 4.0.4; M750 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34334[] = "Mozilla/5.0 (Linux; Android 4.0.3; Pentagram TAB80 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34335[] = "Mozilla/5.0 (Linux; Android 4.0.4; X18 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34336[] = "Mozilla/5.0 (Linux; Android 4.0.3; H8000 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34337[] = "Mozilla/5.0 (Linux; Android 4.0.4; MW0922 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34338[] = "Mozilla/5.0 (Linux; Android 4.0.4; HLV-T9701 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34339[] = "Mozilla/5.0 (Linux; Android 4.0.3; AirTab P71g Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34340[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALEXIS-RX5DC Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34341[] = "Mozilla/5.0 (Linux; Android 4.0.4; platinum Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34342[] = "Mozilla/5.0 (Linux; Android 4.0.3; Pad 7208HD Build/ICS.polypad.7208HD.Dualcore.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34343[] = "Mozilla/5.0 (Linux; Android 4.0.4; novo7 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34344[] = "Mozilla/5.0 (Linux; Android 4.0.4; DLR_MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34345[] = "Mozilla/5.0 (Linux; Android 4.0.4; C3_77_5647 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34346[] = "Mozilla/5.0 (Linux; Android 4.0.4; TPC0952 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34347[] = "Mozilla/5.0 (Linux; Android 4.0.3; SmartPadv10 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34348[] = "Mozilla/5.0 (Linux; Android 4.0.4; TRIO7C Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34349[] = "Mozilla/5.0 (Linux; Android 4.0.4; Tablet Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34350[] = "Mozilla/5.0 (Linux; Android 4.0.4; S3 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34351[] = "Mozilla/5.0 (Linux; Android 4.0.4; Q75S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34352[] = "Mozilla/5.0 (Linux; Android 4.0.4; NetTAB Pocket Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34353[] = "Mozilla/5.0 (Linux; Android 4.0.4; MD-0651 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34354[] = "Mozilla/5.0 (Linux; Android 4.0.3; mw07 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34355[] = "Mozilla/5.0 (Linux; Android 4.0.4; N8010_APT Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34356[] = "Mozilla/5.0 (Linux; Android 4.0.4; HS-U909 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34357[] = "Mozilla/5.0 (Linux; Android 4.0.4; N8000_USA_RS Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34358[] = "Mozilla/5.0 (Linux; Android 4.0.4; PHM727MC Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34359[] = "Mozilla/5.0 (Linux; Android 4.0.3; ARIA Build/ICS.w22ref.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34360[] = "Mozilla/5.0 (Linux; Android 4.0.4; XM908 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34361[] = "Mozilla/5.0 (Linux; Android 4.0.4; Swipe 9X Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34362[] = "Mozilla/5.0 (Linux; Android 4.0.4; PICOpad_GFI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34363[] = "Mozilla/5.0 (Linux; Android 4.0.4; NE-103T Build/X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34364[] = "Mozilla/5.0 (Linux; Android 4.0.4; GEM7032G-rev2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34365[] = "Mozilla/5.0 (Linux; Android 4.0.4; Quince Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34366[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB 270 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34367[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB 171 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34368[] = "Mozilla/5.0 (Linux; Android 4.0.4; Penta IS907C Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34369[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E140K Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34370[] = "Mozilla/5.0 (Linux; Android 4.0.4; i7S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34371[] = "Mozilla/5.0 (Linux; Android 4.0.3; 1040RS Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34372[] = "Mozilla/5.0 (Linux; Android 4.0.4; YPY_S400 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34373[] = "Mozilla/5.0 (Linux; Android 4.0.3; NT-2755 Build/MID) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34374[] = "Mozilla/5.0 (Linux; Android 4.0.4; amity 8GB-V2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34375[] = "Mozilla/5.0 (Linux; Android 4.0.4; F8G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34376[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB 143 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34377[] = "Mozilla/5.0 (Linux; Android 4.0.3; W13PRO Build/ICS.g04wXref.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34378[] = "Mozilla/5.0 (Linux; Android 4.0.4; CA-201L Build/CA201LU325_00) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34379[] = "Mozilla/5.0 (Linux; Android 4.0.4; BT230 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34380[] = "Mozilla/5.0 (Linux; Android 4.0.4; NT-3902S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34381[] = "Mozilla/5.0 (Linux; Android 4.0.4; FLY Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34382[] = "Mozilla/5.0 (Linux; Android 4.0.4; T7040 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34383[] = "Mozilla/5.0 (Linux; Android 4.0.4; GR-TB10S Build/ICS.GR-TB10S.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34384[] = "Mozilla/5.0 (Linux; Android 4.0.4; KINGTOP915 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34385[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenco CARTAB-920 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34386[] = "Mozilla/5.0 (Linux; Android 4.0.4; KINGTOP977 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34387[] = "Mozilla/5.0 (Linux; Android 4.0.4; Calipso4 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34388[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAP5430 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34389[] = "Mozilla/5.0 (Linux; Android 4.0.4; T5500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34390[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT97I-RK66WA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34391[] = "Mozilla/5.0 (Linux; Android 4.0.3; W76 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34392[] = "Mozilla/5.0 (Linux; Android 4.0.4; HD II Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34393[] = "Mozilla/5.0 (Linux; Android 4.0.4; Philips W737 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34394[] = "Mozilla/5.0 (Linux; Android 4.0.3; DL DroidTV DR-T71 Build/tablet-c) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34395[] = "Mozilla/5.0 (Linux; Android 4.0.4; Axxion ATAB-701 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34396[] = "Mozilla/5.0 (Linux; Android 4.0.4; monga M5 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34397[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB 0180 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S34398[] = "Mozilla/5.0 (Linux; Android 4.0.4; DNS S3502 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34399[] = "Mozilla/5.0 (Linux; Android 4.0.4; W5510 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34400[] = "Mozilla/5.0 (Linux; Android 4.0.3; LGL96G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34401[] = "Mozilla/5.0 (Linux; Android 4.0.4; Sygnus Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34402[] = "Mozilla/5.0 (Linux; Android 4.0.4; tmn smart a60 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34403[] = "Mozilla/5.0 (Linux; Android 4.0.4; IHA-C0707 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34404[] = "Mozilla/5.0 (Linux; Android 4.0.4; W2500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34405[] = "Mozilla/5.0 (Linux; Android 4.0.4; X35E Build/RheaTK) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34406[] = "Mozilla/5.0 (Linux; Android 4.0.4; Sky Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34407[] = "Mozilla/5.0 (Linux; Android 4.0.4; NEC-NE-201A1A Build/X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34408[] = "Mozilla/5.0 (Linux; Android 4.0.4; C-4 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S34409[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; X500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34410[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ZTE-X500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34411[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; X500 Build/GINGERBREAD)";
const char S34412[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; ZTE-X500 Build/FRG83)";
const char S34413[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; X500_USA_General Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34414[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; X500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34415[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; X500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, como Gecko) versi?n/4.0 Mobile Safari/533.1,gzip(gfe)";
const char S34416[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S34417[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S34418[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S34419[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S34420[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1080) AppleWebKit/536.11 (KHTML like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S34421[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S34422[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S34423[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/530.5 (KHTML, like Gecko) Chrome/2.0.172.43 Safari/530.5";
const char S34424[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.10 (KHTML, like Gecko) Chrome/2.0.1.3 Safari/536.10";
const char S34425[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.10 (KHTML, like Gecko) Chrome/2.0.1.3 Safari/536.10";
const char S34426[] = "Mozilla/5.0 (Windows NT 6.1; en-US) AppleWebKit/528.8 (KHTML, like Gecko) Chrome/2.0.156.0 Safari/528.8";
const char S34427[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9100 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34428[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9100G Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34429[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-I9100 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34430[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9100 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34431[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9100G Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34432[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9100 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34433[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9100G Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34434[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9100 Build/4.75; CyanogenMod-4.75) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34435[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9100 Build/4.60; CyanogenMod-4.60) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34436[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9100 Build/4.80) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/4.80/i9100";
const char S34437[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9100G Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/i9100g";
const char S34438[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9100 Build/4.50; CyanogenMod-4.50) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34439[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S34440[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7300 Build/HONEYCOMB) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S34441[] = "Mozilla/5.0 (Linux; U; Android 3.1; th-th; GT-P7300B Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S34442[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; GT-P7300 Build/HMJ37)";
const char S34443[] = "Mozilla/5.0 (Linux; U; Android 3.1; cs-cz; SAMSUNG-GT-P7300/P7300BOKH5 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S34444[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S34445[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S34446[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7300 Build/HMJ37) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S34447[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; GT-P7300B Build/HMJ37)";
const char S34448[] = "Mozilla/5.0 (Linux; U; Android 3.1; fa-fa; GT-P7300 Build/HMJ37) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S34449[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7300 Build/HMJ37; 1280*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S34450[] = "Mozilla/5.0 (Linux; U; Android 3.1; se-se; GT-P7300 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34451[] = "Mozilla/5.0 (Linux; U; Android 3.1; hr-hr; SAMSUNG-GT-P7300/P7300BOKH4 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S34452[] = "Vyrso/2.1.2 (Linux; U; Android 3.1; en-GB; GT-P7300 Build/HMJ37)";
const char S34453[] = "Mozilla/5.0 (Linux; U; Android 3.1; pt-br; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.10;FBDM/{density=1.0,width=800,height=1280};FBLC/pt_BR;FB_FW/2;FBCR/TIM;FBPN/com.facebook.katana;FBDV/GT-P7300;FBSV/3.1;]";
const char S34454[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-tw; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 VoiceGO!(cybb6c19fd1eXfcdbce5cdddb0245-0;Android,3.1,GT-P7300;3.6.0;1280x752,1.0;X;VoiceGo-TC;zh-TW)";
const char S34455[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7300) AntivirusAM/2.5 (KHTML, like Gecko) Version/Version 2.5/Antivirus";
const char S34456[] = "Biblia/2.2.3 (Linux; U; Android 3.1; es-US; GT-P7300 Build/HMJ37.UTKH6)";
const char S34457[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.11;FBDM/{density=1.0,width=1280,height=800};FBLC/en_US;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P7300;FBSV/3.1;]";
const char S34458[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7300 Build/HMJ37) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (innerActive-ANDROID-2061)";
const char S34459[] = "Biblia/2.2.3 (Linux; U; Android 3.1; es-ES; GT-P7300 Build/HMJ37.UTKI2)";
const char S34460[] = "Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 3.1; SAMSUNG GT-P7300 Build/HONEYCOMB_MR1);";
const char S34461[] = "MediaPlayer/2002/4.0.12 (Android 3.1; GT-P7300; GT-P7300; samsung; ; ; en_GB)";
const char S34462[] = "VKAndroidApp/2.7.6-64 (Android 3.1; SDK 12; armeabi-v7a; samsung GT-P7300; ru)";
const char S34463[] = "Logos/2.2.8.1 (Linux; U; Android 3.1; en-US; GT-P7300B Build/HMJ37)";
const char S34464[] = "Mozilla/5.0 (Linux; U; Android 3.1; fr-fr;GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S34465[] = "Mozilla/5.0 (Linux; U;Android 3.1; fr-fr; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S34466[] = "Mozilla/5.0 (Linux; U; Android 3.1; th-th; GT-P7300B Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/2.2;FBBV/X;FBDM/{density=1.0,width=800,height=1280};FBLC/th_TH;FB_FW/2;FBCR/DTAC;FBPN/com.facebook.katana;FBDV/GT-P7300B;FBSV/3.1;]";
const char S34467[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:24.0) Gecko/X Firefox/24.0";
const char S34468[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:24.0) Gecko/X Firefox/24.0 AlexaToolbar/alxf-2.19";
const char S34469[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:24.0) Gecko/X Firefox/24.0 PaleMoon/24.0.1";
const char S34470[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:24.0) Gecko/X Firefox/24.0 PaleMoon/24.0.1";
const char S34471[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 4 Build/JOP14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34472[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 4 Build/JVP15D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34473[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 4 Build/JVP15B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34474[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 4 Build/JOP19B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34475[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 4 Build/JOP22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34476[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 4 Build/JOP24B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34477[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 4 Build/JOP24G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34478[] = "Currents/2.0.0 (Linux; U; Android 4.2; en-us; phone/Nexus 4 Build/JOP14; Density/320; gzip) com.google.android.apps.currents/234";
const char S34479[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 4 Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34480[] = "Dalvik/1.6.0 (Linux; U; Android 4.2; Nexus 4 Build/JOP40C)";
const char S34481[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 4 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34482[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; Nexus 4 Build/JOP40D)";
const char S34483[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34484[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 4 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34485[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34486[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; Nexus 4 Build/JDQ39)";
const char S34487[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-; Nexus 4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34488[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34489[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-I9300 Build/JOP40D)";
const char S34490[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-I9300 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34491[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-I535 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34492[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-T999 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34493[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-I9300 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34494[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I535 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34495[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9300 Build/JDQ39)";
const char S34496[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-I535 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34497[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34498[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T999 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34499[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; GT-i9300 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34500[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I535 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34501[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34502[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SCH-I535 Build/JDQ39E)";
const char S34503[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34504[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34505[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGH-T999 Build/JDQ39E)";
const char S34506[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; SCH-I535 Build/JOP40D)";
const char S34507[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; it-it; GT-i9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34508[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9300 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S34509[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; SGH-T999 Build/JOP40D)";
const char S34510[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:24.0) Gecko/X Firefox/24.0";
const char S34511[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:24.0) Gecko/X Firefox/24.0";
const char S34512[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:24.0) Gecko/X Firefox/24.0 PaleMoon/24.0.1";
const char S34513[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:24.0) Gecko/X Firefox/24.0 PaleMoon/24.0.1";
const char S34514[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 610)";
const char S34515[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Nokia 610)";
const char S34516[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 610; Vodafone)";
const char S34517[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 610; 610 BOUYGUES)";
const char S34518[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 610; Orange)";
const char S34519[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 610 NFC)";
const char S34520[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 610; Meteor)";
const char S34521[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 610; NRJMobile)";
const char S34522[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 610; SFR)";
const char S34523[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 610 NFC; Orange)";
const char S34524[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Nokia 610C)";
const char S34525[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Nokia; Cliff)";
const char S34526[] = "Mozilla/3.01Gold (Win95; I)";
const char S34527[] = "Microsoft Internet Explorer/4.0b1 (Windows 95)";
const char S34528[] = "Mozilla/2.02Gold (Win95; I)";
const char S34529[] = "Mozilla/4.5 [en] (Win95; I)(compatible; Powermarks/3.0; Windows 95/NT4)";
const char S34530[] = "Mozilla/4.0 (compatible; Powermarks/3.5; Windows 95/98/2000/NT)";
const char S34531[] = "Mozilla/3.0 (Win95; I)";
const char S34532[] = "Mozilla/4.05 [en] (Win95; I)";
const char S34533[] = "Mozilla/4.5 [en] (Win95; I)";
const char S34534[] = "NCSA Mosaic/3.0 (Windows 95)";
const char S34535[] = "Mozilla/4.7 [en] (Win95; U)";
const char S34536[] = "Mozilla/2.02E (Win95; U)";
const char S34537[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.12 Mobile/11A4372q Safari/8536.25";
const char S34538[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.12 Mobile/11A4400f Safari/8536.25";
const char S34539[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.12 Mobile/11A4414e Safari/8536.25";
const char S34540[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A4435d Safari/8536.25";
const char S34541[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A4449a Safari/8536.25";
const char S34542[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A4449d Safari/8536.25";
const char S34543[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A465 Safari/8536.25";
const char S34544[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A501 Safari/8536.25";
const char S34545[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A466 Safari/8536.25";
const char S34546[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A470a Safari/8536.25";
const char S34547[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Vodafone Smart II Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 SVN/01016";
const char S34548[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; Vodafone Smart II Build/GINGERBREAD)";
const char S34549[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; tr-tr; Vodafone 861 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 SVN/010 06";
const char S34550[] = "Mozilla/5.0 (Linux; U; Android AG3N 4.0; es-es; Vodafone Smart II Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 SVN/0104.0";
const char S34551[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ASUS Transformer Pad TF300T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S34552[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; ASUS Transformer Pad TF300T Build/IML74K)";
const char S34553[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ASUS Transformer Pad TF300TG Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S34554[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ASUS Pad TF300T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S34555[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; ASUS Transformer Pad TF300TG Build/IML74K)";
const char S34556[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Transformer Pad TF300T Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S34557[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ASUS Transformer Pad TF300TL Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S34558[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ASUS Transformer Pad TF300T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34559[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-78) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S34560[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT905 Build/9.8.2Q_SMUIRD-7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S34561[] = "Mozilla/5.0 (Linux; Android 4.1.1; XT907 Build/9.8.1Q_27-2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S34562[] = "WordPress/2.7";
const char S34563[] = "WordPress/2.7.1";
const char S34564[] = "WordPress.com mShots; http://support.wordpress.com/contact/";
const char S34565[] = "WordPress/2.6.2";
const char S34566[] = "ImgPress - WordPress.com";
const char S34567[] = "WordPress/3.1.3; http://politics.zonkeyblog.co.uk";
const char S34568[] = "WordPress/3.1; http://isaveholidays.co.uk";
const char S34569[] = "WordPress/2.9.2; http://polinlawoffice.shikshik.org";
const char S34570[] = "WordPress/2.9.2; http://molicyber.shikshik.org";
const char S34571[] = "WordPress/3.0.4; http://fixhealthcarepolicy.com";
const char S34572[] = "WordPress/3.2.1; http://abcnews.go.com/blogs/politics";
const char S34573[] = "WordPress/x.x.x.x PHP/4.x.xx";
const char S34574[] = "Automattic Analytics Crawler/0.1; http://wordpress.com/crawler/";
const char S34575[] = "WordPress/2.9.2; http://aceflyer.shikshik.org";
const char S34576[] = "WordPress/3.3.1; http://x.X.com";
const char S34577[] = "WordPress/3.0.4; http://testwp.co.uk/canvas";
const char S34578[] = "WordPress/MU; http://chartroose.wordpress.com";
const char S34579[] = "WordPress/3.2.1; http://thefirsttimebuyerclub.com";
const char S34580[] = "WordPress/3.3; http://gpcnews.com";
const char S34581[] = "WordPress/3.3.1; http://screwcable.com";
const char S34582[] = "WordPress/3.3.1; http://horsesdolphins.x10.mx";
const char S34583[] = "WordPress/3.0.4; http://ismysiteindexed.com";
const char S34584[] = "WordPress/3.2; http://xindxr.info";
const char S34585[] = "WordPress/3.3.1; http://celebritynews.fansite.biz";
const char S34586[] = "WordPress/3.1; http://yellowfield.net";
const char S34587[] = "WordPress/3.5-alpha-21535; http://thefashionloverblogging.wordpress.com";
const char S34588[] = "WordPress/3.5-alpha-21535; http://myfavoritelooks.wordpress.com";
const char S34589[] = "WordPress/3.5-alpha-21989; http://apps2.wordpress.com";
const char S34590[] = "WordPress/3.5-alpha-21989; http://p2wp.wordpress.com";
const char S34591[] = "WordPress/3.4.2; http://www.isitwp.com";
const char S34592[] = "WordPress/3.5-alpha-21989;";
const char S34593[] = "WordPress/3.5-alpha-21989; http://thelightnc.com";
const char S34594[] = "WordPress/3.5-alpha-21989; http://ipowerrichmond.com";
const char S34595[] = "WordPress/3.5-alpha-21989; http://hiphopnc.com";
const char S34596[] = "WordPress/3.5-alpha-21989; http://news.stylecaster.com";
const char S34597[] = "WordPress/3.5-alpha-21989; http://praiserichmond.com";
const char S34598[] = "WordPress/3.5-alpha-21535; http://lastnightidrank.wordpress.com";
const char S34599[] = "WordPress/3.5-alpha-21535; http://kymx.cbslocal.com";
const char S34600[] = "WordPress/3.5-alpha-21989; http://annaleonie.wordpress.com";
const char S34601[] = "WordPress/3.4.2; http://infoegypt.com";
const char S34602[] = "WordPress/3.6-alpha-23334;";
const char S34603[] = "WordPress.com Reader";
const char S34604[] = "WordPress/3.6-alpha-23334; http://goldblum.wordpress.com";
const char S34605[] = "WordPress/3.6-alpha-23334; http://wordpress.com";
const char S34606[] = "WordPressPollBot v1.2 (http://besome1.info/robots.html)";
const char S34607[] = "WordPress/3.5.1; http://ppt.4istudent.com,gzip(gfe) (via docs.google.com/viewer)";
const char S34608[] = "WordPress/3.4.2; http://patrikaa.in";
const char S34609[] = "WordPress/3.5.1; http://localhost/wordpress";
const char S34610[] = "WordPress/3.4.2; http://newindianews.in";
const char S34611[] = "WordPress/3.4.2; http://morenews.in";
const char S34612[] = "WordPress/3.5; http://indianreport.in";
const char S34613[] = "WordPress/3.5; http://indiadailyreport.in";
const char S34614[] = "WordPress/3.5; http://theindiannews.in";
const char S34615[] = "WordPress/3.4.2; http://indiabusinessreport.in";
const char S34616[] = "WordPress/3.4.2; http://indiabusinessandfinance.in";
const char S34617[] = "WordPress/3.5; http://channelindia.in";
const char S34618[] = "WordPress/3.5; http://goodmorningindia.asia";
const char S34619[] = "WordPress/3.6-alpha-23334; http://elcidharth.com";
const char S34620[] = "WordPress/3.5; http://bharatvaartaa.in";
const char S34621[] = "WordPress/3.6-beta1-24023;";
const char S34622[] = "WordPress/3.5; http://indiamedianews.in";
const char S34623[] = "WordPress/3.4.2; http://indianbusinessnews.in";
const char S34624[] = "WordPress/3.5; http://indianewsfeed.in";
const char S34625[] = "WordPress/3.4.2; http://mumbaicommercialnews.in";
const char S34626[] = "WordPress/3.5; http://indianmarketnews.in";
const char S34627[] = "WordPress/3.5; http://indiatoday.asia";
const char S34628[] = "WordPress/3.4.2; http://indiatechnews.in";
const char S34629[] = "WordPress/3.4.2; http://indiamarketreport.in";
const char S34630[] = "WordPress/3.4.2; http://indiainsider.in";
const char S34631[] = "WordPress/3.5; http://theindianbusinessline.in";
const char S34632[] = "WordPress/3.4.2; http://indiaglobalnews.in";
const char S34633[] = "WordPress/3.4.2; http://theindianreporter.in";
const char S34634[] = "WordPress/3.5; http://thepagesofindia.in";
const char S34635[] = "WordPress/3.5; http://hindustandaily.in";
const char S34636[] = "WordPress/3.5; http://vaartaapatram.in";
const char S34637[] = "WordPress/3.5; http://theindiancommunicator.in";
const char S34638[] = "WordPress/3.5; http://theindiandaily.in";
const char S34639[] = "WordPress/3.4.2; http://indiabroadcast.in";
const char S34640[] = "WordPress/3.5.1; http://e-books.4istudent.com,gzip(gfe) (via docs.google.com/viewer)";
const char S34641[] = "WordPress/3.6-beta3;";
const char S34642[] = "WordPress/3.6-beta3-24330;";
const char S34643[] = "WordPress/3.6-beta3-24375;";
const char S34644[] = "WordPress/3.5; http://www.colombotecnologia.co";
const char S34645[] = "WordPress/3.5; http://noticiasdeempresas.us";
const char S34646[] = "WordPress/3.4.2; http://couponalert.blog.mindspark.com";
const char S34647[] = "WordPress/3.5; http://venezuelanews.ws";
const char S34648[] = "WordPress/3.6-beta4;";
const char S34649[] = "WordPress/3.5.1; http://free-pdf-download.4istudent.com,gzip(gfe) (via docs.google.com/viewer)";
const char S34650[] = "WordPress/3.6-beta4-24523;";
const char S34651[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; myTouch4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34652[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; myTouch4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34653[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; myTouch4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S34654[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;myTouch4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34655[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; myTouch4GBuild/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34656[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; myTouch4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34657[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; myTouch4G Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34658[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; myTouch4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34659[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_myTouch4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34660[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; myTouch4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34661[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S34662[] = "Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S34663[] = "Mozilla/5.0 (Linux; Android 4.2; Galaxy Nexus Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S34664[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S34665[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC_A510b V1.52.841.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34666[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC Wildfire S A510b Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34667[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC_A510b V1.52.841.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34668[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-nz; HTC_WildfireS_A510b Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34669[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; HTC-A510a Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34670[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_A510c Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34671[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-nz; HTC_WildfireS_A510b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34672[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Wildfire S A510b Build/GRI40)";
const char S34673[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC_A510c Build/GRJ22)";
const char S34674[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC-A510a Build/GRI40)";
const char S34675[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC_A510b V2.26.841.4 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34676[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC Wildfire S A510b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34677[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC-A510c/1.06.1401.4) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34678[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC A510c) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34679[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Wildfire S A510b Build/GRJ90)";
const char S34680[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC A510c Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S34681[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_A510c Build/TeamMarvelUSC2.3.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34682[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC_A510b V2.26.841.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34683[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC_A510b V1.52.841.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34684[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC A510c Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S34685[] = "Mozilla/5.0 (en-us) AppleWebKit/534.14 (KHTML, like Gecko; Google Wireless Transcoder) Chrome/9.0.597 Safari/534.14";
const char S34686[] = "Chrome/9";
const char S34687[] = "Mozilla/5.0 (en-us) AppleWebKit/534.14 (KHTML, like Gecko; Google Wireless Transcoder) Chrome/9.0.597 Safari/534.14,gzip(gfe) (via translate.google.com)";
const char S34688[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; ru) Opera 8.50";
const char S34689[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.51";
const char S34690[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.53";
const char S34691[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.54";
const char S34692[] = "Mozilla/5.0 (Windows NT 5.1; U; en) Opera 8.50";
const char S34693[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.50	Opera 8.5";
const char S34694[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.52";
const char S34695[] = "Mozilla/5.0 (Windows NT 5.1; U; en) Opera 8.51";
const char S34696[] = "Mozilla/5.0 (Windows NT 5.1; U; pl) Opera 8.54";
const char S34697[] = "Mozilla/5.0 (Windows NT 5.1; U; en) Opera 8.53";
const char S34698[] = "Mozilla/5.0 (Windows NT 5.1; U; en) Opera 8.52";
const char S34699[] = "('Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.50', 'Opera 8.5, Windows XP')";
const char S34700[] = "NokiaX2-01.1/5.0 (07.22) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S34701[] = "NokiaX2-01/5.0 (08.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S34702[] = "NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-01";
const char S34703[] = "NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S34704[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:22.0) Gecko/X Firefox/22.0";
const char S34705[] = "Mozilla/5.0 (Windows NT 6.2; rv:22.0) Gecko/X Firefox/22.0";
const char S34706[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:22.0) Gecko/X Firefox/22.0 IceDragon/22.0.0.1";
const char S34707[] = "Microsoft Office/14.0 (Windows NT 6.0; Microsoft Outlook 14.0.4760; Pro; ms-office; MSOffice 14)";
const char S34708[] = "Microsoft Office/14.0 (Windows NT 6.0; Microsoft Outlook 14.0.5128; Pro; ms-office; MSOffice 14)";
const char S34709[] = "Microsoft Office/14.0 (Windows NT 6.0; Microsoft Outlook 14.0.4734; Pro; ms-office; MSOffice 14)";
const char S34710[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S34711[] = "Microsoft Office/14.0 (Windows NT 6.0; Microsoft Outlook 14.0.6106; Pro; ms-office; MSOffice 14)";
const char S34712[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S34713[] = "Microsoft Office/14.0 (Windows NT 6.0; Microsoft Outlook 14.0.6112; Pro)";
const char S34714[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S34715[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S34716[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S34717[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S34718[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S34719[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S34720[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S34721[] = "Microsoft Office/14.0 (Windows NT 6.0; Microsoft Outlook 14.0.6123; Pro)";
const char S34722[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S34723[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Zune 4.7; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S34724[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)";
const char S34725[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S34726[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S34727[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S34728[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S34729[] = "Microsoft Office/14.0 (Windows NT 6.0; Microsoft Outlook 14.0.6129; Pro)";
const char S34730[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S34731[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S34732[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S34733[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S34734[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S34735[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S34736[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S34737[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S34738[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S34739[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S34740[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC1; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S34741[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S34742[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S34743[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)";
const char S34744[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S34745[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S34746[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S34747[] = "Microsoft Office/14.0 (Windows NT 6.0; Microsoft Outlook 14.0.7106; Pro)";
const char S34748[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)";
const char S34749[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Avant Browser)";
const char S34750[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Avant Browser; Avant Browser; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30)";
const char S34751[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; CHWIE_AT71; SV1; Avant Browser; MEGAUPLOAD 1.0)";
const char S34752[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Avant Browser; Avant Browser)";
const char S34753[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Avant Browser; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Alexa Toolbar; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648)";
const char S34754[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 5.1; Avant Browser)";
const char S34755[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 5.2; Avant Browser)";
const char S34756[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Avant Browser; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Avant Browser)";
const char S34757[] = "Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 5.2; Avant Browser)";
const char S34758[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Avant Browser; Avant Browser; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Avant Browser)";
const char S34759[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2;  SV1; Avant Browser; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; Media Center PC 6.0; FDM)";
const char S34760[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; Avant Browser; .NET CLR 1.1.4322; Avant Browser)";
const char S34761[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1;  SV1; Avant Browser; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; FDM)";
const char S34762[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Avant Browser; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; Media Center PC 6.0; FDM)";
const char S34763[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S34764[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S34765[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S34766[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S34767[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S34768[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7";
const char S34769[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7";
const char S34770[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S34771[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S34772[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7,gzip(gfe) (via translate.google.com)";
const char S34773[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.7 (KHTML	 like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S34774[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/535.7 (KHTML	 like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S34775[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/535.7 (KHTML	 like Gecko) Chrome/16.0.912.75 Safari/535.7";
const char S34776[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/16.0.648.205 Safari/534.16";
const char S34777[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/16.0.648.205 Safari/534.16";
const char S34778[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/16.0.648.205 Safari/534.16";
const char S34779[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34780[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34781[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34782[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T959V Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S34783[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34784[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34785[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-T959V Build/GINGERBREAD)";
const char S34786[] = "Mozilla/5.0 (Linux; U; Android2.3.6; en-us; SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34787[] = "Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34788[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S34789[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T959VBuild/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34790[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us;SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34791[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SGH-T959V Build/GINGERBREAD)";
const char S34792[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SGH-T959V Build/GINGERBREAD)";
const char S34793[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SGH-T959V Build/GINGERBREAD)";
const char S34794[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3";
const char S34795[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5274d Safari/7534.48.3";
const char S34796[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3";
const char S34797[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9Z999 Safari/7534.48.3";
const char S34798[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5220p Safari/7534.48.3";
const char S34799[] = "Mozilla/5.0 (iPod; CPU iPhone OS 50 like Mac OS X) AppleWebKit/534.46 (KHTML like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3";
const char S34800[] = "Mozilla/5.0 (iPod; CPU iPhone OS 501 like Mac OS X) AppleWebKit/534.46 (KHTML like Gecko) Version/5.1 Mobile/9B176 Safari/7534.48.3";
const char S34801[] = "Mozilla/5.0 (iPod; CPU iPhone OS 501 like Mac OS X) AppleWebKit/534.46 (KHTML like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3";
const char S34802[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:22.0) Gecko/X Firefox/22.0";
const char S34803[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:13.0) Gecko/X Firefox/13.0.1";
const char S34804[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; USCCADR3310 Build/HuaweiM866) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34805[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; H866C Build/HuaweiH866C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34806[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; H866C Build/HuaweiH866C)";
const char S34807[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HUAWEI M866 Build/HuaweiM866) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34808[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; H866CBuild/HuaweiH866C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34809[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; M866 Build/HuaweiM866) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34810[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; USCCADR3310 Build/HuaweiM866)";
const char S34811[] = "Mozilla/5.0 (Linux; U; Android2.3.6; en-us; H866C Build/HuaweiH866C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34812[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; H866C Build/HuaweiH866C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S34813[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; -us; H866C Build/HuaweiH866C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34814[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:18.0) Gecko/X Firefox/18.0";
const char S34815[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; LG-P880 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34816[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-P880 Build/IML74K)";
const char S34817[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-P880/V10b Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34818[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-P880/V10a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34819[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-P880/V10d Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34820[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-P880/V10f Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34821[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; LG-P880g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34822[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; LG-P880 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S34823[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-P880/V10h Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34824[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; bs; LG-P880 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34825[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-P880g Build/IML74K)";
const char S34826[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S34827[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S34828[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7,gzip(gfe)";
const char S34829[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7,gzip(gfe)";
const char S34830[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC6435LVW 4G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34831[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC6435LVW Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34832[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC6435LVW Build/JRO03C)";
const char S34833[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC6435LVW 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34834[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC6435LVW Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34835[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC6435LVW Build/JZO54K)";
const char S34836[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; HTC6435LVW Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34837[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; LG-P920 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S34838[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P920/V10f/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S34839[] = "LG-P920/ Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S34840[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; LG-P920/V10d  Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S34841[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P920/V10d/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S34842[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; LG-P920/V10c  Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S34843[] = "LG-P920 Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P920; Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S34844[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P920 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S34845[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P920-Orange Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S34846[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; LG-P920/V10f  Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S34847[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P920 Build/FRG83G)";
const char S34848[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-mx; LG-P920h Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S34849[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P920/V10k/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S34850[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; LG-P920/V10l  Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S34851[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P920h Build/FRG83G)";
const char S34852[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; hr-hr; LG-P920/V10b  Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S34853[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; LG-P920 Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S34854[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; el-gr; LG-P920/V10c/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S34855[] = "Mozilla/1.22 (compatible; MSIE 2.0d; Windows NT)";
const char S34856[] = "Mozilla/1.22 (compatible; MSIE 1.5; Windows NT)";
const char S34857[] = "Mozilla/4.0 (compatible; MSIE; Win32)";
const char S34858[] = "lla/1.22 (compatible; MSIE 2.0d; Windows NT)";
const char S34859[] = "Mozilla/5.0 (compatible; MSIE 1.0; 68K; Trident/3.0)";
const char S34860[] = "Mozilla/5.0 (compatible; MSIE 2.0; 68K; Trident/2.0)";
const char S34861[] = "Mozilla/5.0 (compatible; MSIE 1.0; PPC; Win64; x64; Trident/1.0)";
const char S34862[] = "Mozilla/5.0 (compatible; MSIE 1.0; PPC; .NET CLR; Trident/2.0)";
const char S34863[] = "Mozilla/5.0 (compatible; MSIE 2.0; 68K; Win64; x64; Trident/3.0)";
const char S34864[] = "Mozilla/5.0 (compatible; MSIE 2.0; PPC; WOW64; Trident/1.0)";
const char S34865[] = "Mozilla/5.0 (compatible; MSIE 1.0; 68K; Win64; x64; Trident/5.0)";
const char S34866[] = "Mozilla/5.0 (compatible; MSIE 1.0; PPC; WOW64; Trident/4.0)";
const char S34867[] = "Mozilla/5.0 (compatible; MSIE 1.0; 68K; SV1; Trident/1.0)";
const char S34868[] = "Mozilla/5.0 (compatible; MSIE 2.0; PPC; .NET CLR; Trident/3.0)";
const char S34869[] = "Mozilla/5.0 (compatible; MSIE 2.0; 68K; WOW64; Trident/2.0)";
const char S34870[] = "Mozilla/5.0 (compatible; MSIE 2.0; 68K; SV1; Trident/4.0)";
const char S34871[] = "Mozilla/5.0 (compatible; MSIE 1.0; 68K; WOW64; Trident/1.0)";
const char S34872[] = "Mozilla/5.0 (compatible; MSIE 2.0; PPC; Win64; x64; Trident/3.0)";
const char S34873[] = "Mozilla/4.0 (compatible; MSIE; Windows)";
const char S34874[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-I9500 Build/JOP40D)";
const char S34875[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34876[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9500 Build/JDQ39)";
const char S34877[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34878[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9500 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34879[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34880[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34881[] = "Mozilla/5.0 (Linux; U; Android 4.2.3; en-us; I9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34882[] = "Mozilla/5.0 (Linux; U; Android 4.2.9; en-gb; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34883[] = "Mozilla/5.0 (Linux; U; Android 4.2.3; es-es; I9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34884[] = "Mozilla/5.0 (Linux; U; Android 4.2.9; en-us; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34885[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34886[] = "Mozilla/5.0 (Linux; U; Android 4.2.9; es-es; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34887[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34888[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; GT-i9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34889[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; vi-vn; I9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34890[] = "Mozilla/5.0 (Linux; U; Android 4.2.9; vi-vn; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34891[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; vi-vn; GT-i9500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34892[] = "Mozilla/5.0 (Linux; U; Android 4.2.3; it-it; I9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34893[] = "Mozilla/5.0 (Linux; U; Android 4.2.9; it-it; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34894[] = "Mozilla/5.0 (Linux; U; Android 4.2.9; nl-nl; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34895[] = "Mozilla/5.0 (Linux; U; Android 4.2.3; en-gb; I9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34896[] = "Mozilla/5.0 (Linux; U; Android 4.2.3; nl-nl; I9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34897[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; I9500 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34898[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Samsung Galaxy S 4 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34899[] = "Mozilla/5.0 (Linux; U; Android 4.2.9; en-us; GT-I9500 Build/JDQ39.I9500ZSUAMDF) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34900[] = "Mozilla/5.0 (Linux; U; Android 4.2.3; en-us; GT-I9500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34901[] = "Mozilla/5.0 (Linux; U; Android 4.2.9; ar-eg; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34902[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9500 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34903[] = "Mozilla/5.0 (Linux; U; Android 4.2.9; fr-fr; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34904[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9500 Build/JDQ39) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 T5/1.0 baiduboxapp/4.5.1 (Baidu; P1 4.2.2)";
const char S34905[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/4";
const char S34906[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.78 Safari/532.5";
const char S34907[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.1.249.1025 Safari/532.5";
const char S34908[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0 (x86_64); de-DE) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/4.0.202.2 Safari/532.0";
const char S34909[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1063.0 Safari/536.3";
const char S34910[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S34911[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S34912[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1063.0 Safari/536.3 (2.3.0.1, ruby 1.9.2 (2012-05-01) [java])";
const char S34913[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5";
const char S34914[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S34915[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S34916[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1063.0 Safari/536.3 (2.3.3, ruby 1.9.3 (2013-02-22))";
const char S34917[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S34918[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1063.0 Safari/536.3 (2.3.4.1, ruby 2.0.0 (2013-06-27))";
const char S34919[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XT907 Build/9.8.1Q_27-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34920[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; XT907 Build/9.8.1Q_27-2)";
const char S34921[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XT907 Build/9.8.1Q_25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34922[] = "stagefright/1.2 (Linux;Android 4.1.1;motorola XT907 Build/9.8.1Q_27-2)";
const char S34923[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; XT905 Build/9.8.2Q_SMUIRD-7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34924[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-66) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34925[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT905 Build/9.8.2Q_SMUIRD-7)";
const char S34926[] = "MOT-XT907/ Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XT907 Build/9.8.1Q_27-2) 1617 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34927[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-66-1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34928[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; XT907 Build/9.8.1Q_27-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34929[] = "MOT-XT907/ Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-66) 1617 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34930[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-78) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34931[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-78-1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34932[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; XT907 Build/9.8.1Q-66) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34933[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT907 Build/9.8.1Q-78)";
const char S34934[] = "stagefright/1.2 (Linux;Android 4.1.2;motorola XT907 Build/9.8.1Q-78)";
const char S34935[] = "MOT-XT907/ Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-78) 1617 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34936[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=scorpion_mini;devm=XT907;man=motorola;brw=Android;shwd=540x888;shwa=540x851;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S34937[] = "MOT-XT907/ Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-78-1) 1617 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34938[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S34939[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6300Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34940[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; ADR6300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34941[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; ADR6300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34942[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;ADR6300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34943[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34944[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ADR6300 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34945[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ADR6300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34946[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34947[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34948[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34949[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; ADR6300 Build/GRJ22)";
const char S34950[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34951[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34952[] = "Logos/0.9.4 (Linux; U; Android 2.3.4; en_US; ADR6300 Build/GRJ22)";
const char S34953[] = "Android/2.3.4; HTC ADR6300; 2.2.0";
const char S34954[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; ADR6300 Build/FRF91)";
const char S34955[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6300 Build/GRJ22) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S34956[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; ADR6300 Build/GRJ22)";
const char S34957[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34958[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; -us; ADR6300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S34959[] = "SAMSUNG-GT-S5222 Opera/9.80 (J2ME/MIDP; Opera Mini/6.5.29806/30.3214; U; en) Presto/2.8.119 Version/11.10";
const char S34960[] = "Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.785.0 Safari/535.1";
const char S34961[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S34962[] = "Mozilla/5.0 (X11; Jolicloud Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Joli OS/1.2 Chromium/14.0.835.126 Chrome/14.0.835.126 Safari/535.1";
const char S34963[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.824.0 Safari/535.1";
const char S34964[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1 (ru-RU,ru;q=0.8,en-US;q=0.6,en;q=0.4)";
const char S34965[] = "Mozilla/5.0 (X11; U; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.94 Safari/535.1";
const char S34966[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1,gzip(gfe) (via translate.google.com)";
const char S34967[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML	 like Gecko) Chrome/14.0.803.0 Safari/535.1";
const char S34968[] = "Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/14.0.1410.63 Safari/537.31";
const char S34969[] = "Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/14.0.1410.63 Safari/537.31";
const char S34970[] = "Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/14.0.1410.63 Safari/537.31";
const char S34971[] = "Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/14.0.1410.63 Safari/537.31";
const char S34972[] = "Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/14.0.1410.63 Safari/537.31";
const char S34973[] = "Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/14.0.1410.63 Safari/537.31";
const char S34974[] = "Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/14.0.1410.63 Safari/537.31";
const char S34975[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyEricssonSK17a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34976[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SK17i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34977[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonSK17i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34978[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SK17i Build/4.1.B.0.587)";
const char S34979[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SK17i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34980[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SonyEricssonSK17iv Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34981[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SK17a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34982[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SK17a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S34983[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SK17a Build/4.1.B.0.431)";
const char S34984[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/3";
const char S34985[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.25 Safari/532.0";
const char S34986[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:15.0) Gecko/X Firefox/15.0.1";
const char S34987[] = "Mosssszilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:15.0) Gecko/X Firefox/15.0";
const char S34988[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/17.0.942.0) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/17.0.942.0 Safari/535.8";
const char S34989[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/17.0.963.26) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.26 Safari/535.11";
const char S34990[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/17.0.963.46) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.46 Safari/535.11";
const char S34991[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/17.0.963.56) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S34992[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/17.0.963.56; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S34993[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/17.0.963.56; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S34994[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/17.0.963.65) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.65 Safari/535.11";
const char S34995[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/17.0.963.66) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11";
const char S34996[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/17.0.963.78) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.78 Safari/535.11";
const char S34997[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/17.0.963.79) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11";
const char S34998[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/17.0.963.79; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S34999[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/17.0.963.83) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.83 Safari/535.11";
const char S35000[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/17.0.963.83; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S35001[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/17.0.963.33) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.33 Safari/535.11";
const char S35002[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/17.0.963.38) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.38 Safari/535.11";
const char S35003[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/17.0.963.56; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S35004[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/17.0.963.56) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S35005[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/17.0.963.56; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S35006[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; chromeframe/17.0.963.46; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MuW; InfoPath.2; .NET4.0C; .NET4.0E; MuW 64; MuW 64; QG Managed;)";
const char S35007[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/17.0.963.78; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.3)";
const char S35008[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/17.0.963.79; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S35009[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/17.0.963.79)";
const char S35010[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/17.0.963.46; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; AskTbFWV5/5.14.0.19709)";
const char S35011[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.2; chromeframe/17.0.963.46; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; Windows-Media-Player/10.00.00.3990; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729;";
const char S35012[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:15.0) Gecko/X Firefox/15.0.1";
const char S35013[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; SonyEricssonE10i-o Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S35014[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; SonyEricssonE10iv Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S35015[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; E10i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S35016[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-kr; E10i Build/2.0.A.0.497) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S35017[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; E10i Build/2.0.A.0.504) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S35018[] = "Mozilla/5.0(Linux;U;Android2.1-update1;Zh_cn;E10i;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S35019[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1 - JIT Enabled; sv-se; E10i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S35020[] = "Android (Linux; U; Android 2.1-update1; fa-ir; E10i) AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S35021[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; E10i Build/2.1.1.C.0.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S35022[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; pl-pl; E10i Build/2.1.1.A.0.16) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S35023[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; SonyEricssonE10i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S35024[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; E10i Build/2.1.1.A.0.19) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S35025[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; E10i Build/2.1.1.A.0.6)";
const char S35026[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; E10i Build/2.0.2.A.0.24) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S35027[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fa-ir; E10i Build/2.1.1.A.0.6) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S35028[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fa-ir; E10i Build/2.1.1.C.0.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S35029[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I777 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35030[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-I777 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35031[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SAMSUNG-SGH-I777 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35032[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I777 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35033[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SAMSUNG-SGH-I777 Build/IML74K)";
const char S35034[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; Galaxy S II Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35035[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SGH-I777 Build/IML74K)";
const char S35036[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I777 Build/cMIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35037[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I777 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35038[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I777 Build/IMM76L)";
const char S35039[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I777 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35040[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SAMSUNG-SGH-I777 Build/IMM76D)";
const char S35041[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.6 (KHTML, like Gecko) Chrome/20.0.1090.0 Safari/536.6";
const char S35042[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S35043[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe) (via translate.google.com)";
const char S35044[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/20 Safari/537.17";
const char S35045[] = "Mozilla/5.0 (Series40; NokiaC2-03/07.48; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S35046[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VS840 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35047[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; VS840 4G Build/IMM76D)";
const char S35048[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VS840PP Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35049[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; VS840PP Build/IMM76D)";
const char S35050[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; VS840 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35051[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Nexus S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35052[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; Nexus S Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35053[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; Nexus S Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35054[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Nexus S Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35055[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; Nexus S Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35056[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Nexus S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35057[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Nexus S Build/GRI54) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35058[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Nexus S Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35059[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; Nexus S Build/GRJ22)";
const char S35060[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Nexus S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35061[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Nexus S Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35062[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Nexus S Build/GTJ61) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35063[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Nexus S Build/GRK39C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35064[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; Nexus S Build/GRK39F)";
const char S35065[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Nexus S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35066[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus S Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35067[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus S Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35068[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35069[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; Nexus S Build/GRI54)";
const char S35070[] = "Mozilla/5.0 (Linux; U; Android 2.4; en-us; Nexus S Build/GRI16B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35071[] = "Mozilla/5.0 (Linux; U; Android 2.4; en-gb; Nexus S Build/GRH84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35072[] = "Mozilla/5.0 (Linux; U; Android 2.4; en-us; Nexus S Build/GRI06B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35073[] = "Mozilla/5.0 (Linux; U; Android 2.4; en-us; Nexus S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35074[] = "Mozilla/5.0 (Linux; U; Android 2.4; en-us; Full Android on Crespo Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35075[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Nexus S Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35076[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; Nexus S Build/GWK74)";
const char S35077[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Nexus S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35078[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Nexus S Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S35079[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.1; Nexus S Build/GRH78)";
const char S35080[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Samsung Nexus S Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35081[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; iw-; Nexus S Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35082[] = "Mozilla/5.0 (Linux; U; Android 2.3; xx-xx; Nexus S Build/GRH41B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35083[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; Nexus S Build/GKH77) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35084[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Nexus S Build/WS0.5.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35085[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35086[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK Nexus S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35087[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Nexus S Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S35088[] = "Mozilla/5.0 (Linux; U; Android 2.3.7p0; en-us; Nexus S Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35089[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Nexus S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S35090[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Nexus S Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S35091[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; Nexus S Build/GRI20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35092[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; Nexus S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35093[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ADR6400L Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35094[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ADR6400L 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35095[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ADR6400L 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35096[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ADR6400L 4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35097[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6400L Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35098[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6400L 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35099[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ADR6400L 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35100[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; ADR6400L Build/GRJ22)";
const char S35101[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ThunderBolt Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35102[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Thunderbolt Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35103[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Thunderbolt Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35104[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ADR6400L Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35105[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6400L Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35106[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC ThunderBolt Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35107[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC ThunderBolt Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35108[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; ADR6400L Build/GRK39F)";
const char S35109[] = "Logos/0.9.4 (Linux; U; Android 2.3.4; en_US; ADR6400L Build/GRJ22)";
const char S35110[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC ThunderBolt Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35111[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC ThunderBolt Build/GRJ22; liquid.mecha.X.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35112[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6400L 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S35113[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ADR6400L 4G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35114[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6400L4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35115[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6400L 4GBuild/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35116[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; ADR6400L 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35117[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; ADR6400L 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35118[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;ADR6400L 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35119[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-; ADR6400L 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35120[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; A05PTH3ORYB0LT Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35121[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ADR6400L Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35122[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6400L Build/GRJ22) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S35123[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC ThunderBolt Build/GRJ22)";
const char S35124[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6400L Build/GRJ22; JELLYBLUR-V1.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35125[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC ThunderBolt Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35126[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; ADR6400L Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35127[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ADR6400L Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S35128[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ADR6400L Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S35129[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6400L Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S35130[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; ADR6400L Build/GRJ22)";
const char S35131[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; nl-nl; LG-P990/V10a Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S35132[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; nl-nl; LG-P990/V10b Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S35133[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; LG-P990/V10d Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S35134[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; LG-P990/V10c Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S35135[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P990 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S35136[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en; LG-P990 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S35137[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; LG-P990/V10f Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S35138[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-sa; LG-P990H Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S35139[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; LG-P990hN Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S35140[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-mx; LG-P990h Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S35141[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; LG-P990 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S35142[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; LG-P990/V10e Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S35143[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P990 Build/FRG83G)";
const char S35144[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P990 Build/HannsTitan-0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35145[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; LG-P990 Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S35146[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P990hN Build/FRG83G)";
const char S35147[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P990h Build/FRG83G)";
const char S35148[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; LG-P990 Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S35149[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT30p Build/7.0.A.3.64) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35150[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT30p Build/7.0.A.3.108) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35151[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyLT30p Build/TESTFEATURE-SUPERSTAMINA-X-0705) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35152[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyLT30p Build/7.0.A.1.303) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35153[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyLT30p Build/7.0.A.3.116) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35154[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT30p Build/TESTFEATURE-SUPERSTAMINA-X-0541) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35155[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyLT30p Build/TESTFEATURE-SUPERSTAMINA-X-0856) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35156[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT30p Build/7.0.A.1.300) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35157[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT30p Build/BLUE1.0-DECOUPLED-X-1600) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35158[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyLT30p Build/7.0.X.0.2303) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35159[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT30p Build/7.0.A.1.129) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35160[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT30p Build/7.0.A.3.35) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35161[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT30p Build/7.0.A.1.269) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35162[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; SonyLT30p Build/7.0.A.2.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35163[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT30p Build/private) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35164[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT30p Build/7.0.A.1.303) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35165[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyLT30p-o Build/7.0.A.1.303) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35166[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyLT30p Build/7.0.A.3.195) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35167[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT30p Build/7.0.A.3.195) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35168[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT30p Build/7.0.A.3.195)";
const char S35169[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyLT30p-o Build/7.0.A.3.195) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35170[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; SonyLT30p Build/7.0.A.3.197) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35171[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyLT30p Build/7.0.A.3.223) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35172[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT30p Build/7.0.A.3.223) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35173[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT30p Build/7.0.A.3.193) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35174[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyLT30p Build/7.0.A.3.226) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35175[] = "SonyLT30p Build/7.0.A.3.195 stagefright/1.2 (Linux;Android 4.0.4)";
const char S35176[] = "Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.8.0.7) Gecko/X Firefox/1.5.0.7 (Debian-1.5.dfsg+1.5.0.7-1~bpo.1)";
const char S35177[] = "Mozilla/5.0 (X11; U; Linux i686; pl; rv:1.8.0.7) Gecko/X Firefox/1.5.0.7 (Swiftfox)";
const char S35178[] = "Opera/8.01 (J2ME/MIDP; Opera Mini/3.1.10423/1724; en; U; ssr)";
const char S35179[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S35180[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P6800 Build/HTJ85B)";
const char S35181[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S35182[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S35183[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S35184[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S35185[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S35186[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6800) antivirusHttpClient/2.0 (KHTML, like Gecko) Version/Version 2.3/Antivirus pro";
const char S35187[] = "Mozilla 5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build HTJ85B) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S35188[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 parameters/{scr=1232_800,cm=1,ql=l,imei=X,bm=1,eng=adrmini13,mdm=1,mdl=GT-P6800,ps=1} tiantian(securitypay) tiantian(tenpay)";
const char S35189[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-tw; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 MobileSafari/534.13 VoiceGO!(cyb61ee75964e97b054ae89afa2366b5238-0;Android,3.2,GT-P6800;3.6.0;1280x752,1.0;X;VoiceGo-TC;zh-TW)";
const char S35190[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6800 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35191[] = "Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 3.2; SAMSUNG GT-P6800 Build/HONEYCOMB_MR2);";
const char S35192[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6800 Build/HTJ85B) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S35193[] = "Logos/2.2.3 (Linux; U; Android 3.2; cs-CZ; GT-P6800 Build/HTJ85B)";
const char S35194[] = "MediaPlayer/2002/4.0.12 (Android 3.2; GT-P6800; GT-P6800; samsung; ; ; en_GB)";
const char S35195[] = "Logos/2.2.3 (Linux; U; Android 3.2; en-US; GT-P6800 Build/HTJ85B.ZSLA2)";
const char S35196[] = "com.google.android.apps.plus/X (Linux; U; Android 3.2; en_GB; GT-P6800; Build/HTJ85B) (gzip)";
const char S35197[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13 [FBAN/FB4A;FBAV/1.9.3;FBDM/{density=1.0,width=545,height=320};FBLC/en_GB;FB_FW/2;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P6800;FBSV/3.2;]";
const char S35198[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/2.0;FBDM/{density=1.0,width=800,height=1232};FBLC/en_GB;FB_FW/2;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P6800;FBSV/3.2;]";
const char S35199[] = "Mrd/1.1.0 (Linux; U; Android 3.2; samsung GT-P6800) jp.co.paperboy.picto/37";
const char S35200[] = "Logos/2.2.8.1 (Linux; U; Android 3.2; pt-BR; GT-P6800 Build/HTJ85B.UBLA2)";
const char S35201[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6800) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S35202[] = "VKAndroidApp/2.7.6-64 (Android 3.2; SDK 13; armeabi-v7a; samsung GT-P6800; ru)";
const char S35203[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.10;FBDM/{density=1.0,width=1280,height=752};FBLC/en_US;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P6800;FBSV/3.2;]";
const char S35204[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb;GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S35205[] = "Groupon/2.2.2572 (Android 3.2; Samsung GT-P6800 / Samsung GT-P6800; )[preload=true;locale=en_US;clientidbase=android-samsung]";
const char S35206[] = "Android 3.2;AppleWebKit/533.1;Build/HTJ85B;GT-P6800 Build/HTJ85B";
const char S35207[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/18.0.1025.54) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.54 Safari/535.19";
const char S35208[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/18.0.1025.142) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19";
const char S35209[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/18.0.1025.142) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19";
const char S35210[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.3; chromeframe/18.0.1025.142; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C)";
const char S35211[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/18.0.1025.151) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19";
const char S35212[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/18.0.1025.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.1 Safari/535.19";
const char S35213[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/18.0.1025.152) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.152 Safari/535.19";
const char S35214[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/18.0.1025.162) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19";
const char S35215[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/18.0.1025.162) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19";
const char S35216[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/18.0.1025.162; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S35217[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/18.0.1025.162)";
const char S35218[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/18.0.1025.162; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S35219[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/18.0.1025.162) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19";
const char S35220[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/18.0.1025.162; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S35221[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/18.0.1025.168) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S35222[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/18.0.1025.168) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S35223[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/18.0.1025.168; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S35224[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/18.0.1025.168) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S35225[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/18.0.1025.168; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S35226[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Master Desktop XP-FR-AF V02.00.00.00.00  R3; SVCVersion 8.14; chromeframe/18.0.1025.152; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S35227[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/18.0.1025.162; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S35228[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/18.0.1025.168; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.1)";
const char S35229[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; chromeframe/18.0.1025.162; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)";
const char S35230[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; MathPlayer 2.20; chromeframe/18.0.1025.168; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.3; IPH 1.1.21.4019)";
const char S35231[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/19.0.1084.15) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.15 Safari/536.5";
const char S35232[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/19.0.1084.24) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.24 Safari/536.5";
const char S35233[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/19.0.1084.41) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.41 Safari/536.5";
const char S35234[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/19.0.1084.46) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5";
const char S35235[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/19.0.1084.46) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5";
const char S35236[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/19.0.1084.52) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5";
const char S35237[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/19.0.1084.52; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S35238[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/19.0.1084.52; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S35239[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/19.0.1084.52; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)";
const char S35240[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/19.0.1084.52) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5";
const char S35241[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/19.0.1084.52; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S35242[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/19.0.1084.52)";
const char S35243[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/19.0.1084.52; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; InfoPath.2)";
const char S35244[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/19.0.1084.52; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S35245[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/19.0.1084.56) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S35246[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/19.0.1084.56) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S35247[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/19.0.1084.56) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S35248[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; chromeframe/19.0.1084.52; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.3)";
const char S35249[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/19.0.1084.52; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S35250[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/19.0.1084.52; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.1)";
const char S35251[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; chromeframe/19.0.1084.56; .NET4.0C)";
const char S35252[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; DROID RAZR HD Build/9.8.1Q_37) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35253[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; DROID RAZR HD Build/9.8.1Q_39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35254[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; DROID RAZR HD Build/9.8.1Q_39)";
const char S35255[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; XT925 Build/9.8.2Q-8-XT925_VQU-13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35256[] = "stagefright/1.2 (Linux;Android 4.1.1;motorola DROID RAZR HD Build/9.8.1Q_39)";
const char S35257[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; XT925 Build/9.8.2Q-8-XT925_VQL-12.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35258[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35259[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2)";
const char S35260[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; XT925 Build/9.8.2Q-8-XT925_VQUL-1601) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35261[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT925 Build/9.8.2Q-8-XT925_VQL-12.2)";
const char S35262[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; DROID RAZR HD Build/BATAKANG) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35263[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; XT925 Build/9.8.2Q-8-XT925_VQU-25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35264[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; XT925 Build/9.8.2Q-8-XT925_VQU-27) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35265[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT925 Build/9.8.2Q-50-XT925_VQLM-21) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35266[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35267[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35268[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; XT925 Build/9.8.2Q-8-XT925_VQUL-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35269[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT925 Build/9.8.2Q-50-XT925_VQLM-20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35270[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; XT925 Build/9.8.0Q-97-XT925_VQU-19) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35271[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/RAGE HD v1.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35272[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-79) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35273[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-79-1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35274[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT925 Build/9.8.2Q-50-XT925_VQLM-24) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35275[] = "stagefright/1.2 (Linux;Android 4.1.2;motorola DROID RAZR HD Build/9.8.1Q-79)";
const char S35276[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=vanquish;devm=DROID RAZR HD;man=motorola;brw=Android;shwd=720x1184;shwa=720x1134;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S35277[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/20.0.1132.11) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.11 Safari/536.11";
const char S35278[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/20.0.1132.17) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.17 Safari/536.11";
const char S35279[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/20.0.1132.27) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.27 Safari/536.11";
const char S35280[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/20.0.1132.43) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.43 Safari/536.11";
const char S35281[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/20.0.1132.47) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11";
const char S35282[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/20.0.1132.47) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11";
const char S35283[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.3; chromeframe/20.0.1132.47; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S35284[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/20.0.1132.47; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S35285[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/20.0.1132.57) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S35286[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/20.0.1132.57; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)";
const char S35287[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/20.0.1132.57) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S35288[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/20.0.1132.57; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S35289[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/20.0.1132.57)";
const char S35290[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/20.0.1132.57) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S35291[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/20.0.1132.57; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) )";
const char S35292[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/20.0.1132.47; InfoPath.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S35293[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/20.0.1132.57; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S35294[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/20.0.1132.57; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S35295[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/20.0.1132.47; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; MS-RTC LM 8)";
const char S35296[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/20.0.1132.57; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S35297[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; FBViewer-6.0.2.0; chromeframe/20.0.1132.57; FBViewer-6.0.5.0; FBViewer-5.0.1.23; FBViewer-6.0.2.0; FBViewer-6.0.5.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; FBViewer-6.0.2.0; FBViewer-6.0.5.0; FBViewer-6.0.2.0; FBViewer-6.0.5.0)";
const char S35298[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1_73_DHD-4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35299[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1-73_DHD-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35300[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR Build/6.5.1-73_DHD-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35301[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1-73_DHD-10_PS-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35302[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; DROID RAZR Build/6.5.1-73_DHD-11_M1-2)";
const char S35303[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1-73_DHD-11_M1-29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35304[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M3-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35305[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M2-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35306[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR Build/6.5.1-167_DHD-14_M2-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35307[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1-73_DHD-11_TA-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35308[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1-73_DHD-11_TA-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35309[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M2-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35310[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR Build/6.5.1-167_DHD-14_M2-5) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30";
const char S35311[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1-73_DHD-11_M1-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35312[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M3-8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35313[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR Build/6.5.1-167_DHD-14_M3-8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35314[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR Build/6.5.1-73_DHD-11_M1-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35315[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR Build/6.5.1-73_DHD-11_TA-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35316[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; DROID RAZR Build/6.5.1-167_DHD-14_M3-8)";
const char S35317[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1-73_DHD-90_TA-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35318[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1-73_DHD-11_M1-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35319[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1-73_DHD-91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35320[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1-73_DHD-11_TA-33) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35321[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M2-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35322[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M2-4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35323[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35324[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR Build/6.5.1-73_DHD-11_M1-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35325[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35326[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR Build/6.5.1-73_DHD-90_TA-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35327[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1-73_DHD-11_TA-4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35328[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR Build/6.5.1-167_DHD-14_M3-8) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S35329[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M3-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35330[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us;DROID RAZR 4G Build/6.5.1-167_DHD-14_M3-8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35331[] = "Mozilla/5.0 (Linux; U; Android2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M3-8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35332[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M3-8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S35333[] = "Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M3-8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35334[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M3-4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35335[] = "Mozilla/5.0 (Linux; U; Android2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M2-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35336[] = "Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M2-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35337[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROIDRAZR 4G Build/6.5.1-167_DHD-14_M3-8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35338[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M2-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S35339[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4GBuild/6.5.1-167_DHD-14_M2-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35340[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M3-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35341[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR Build/6.5.1-167_DHD-14_M3-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35342[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR Build/6.5.1-73_DHD-91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35343[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_Desire_X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35344[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC Desire X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35345[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC Desire X Build/JRO03C)";
const char S35346[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; HTC_Desire_X-orange-LS/2.22.75.4 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35347[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; HTC_Desire_X/2.23.114.1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35348[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; HTC_Desire_X/2.23.113.2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35349[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_Desire_X/2.20.161.11 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35350[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-at; HTC_Desire_X/2.20.169.7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35351[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; HTC_Desire_X/2.23.118.2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35352[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-at; HTC_Desire_X/2.23.123.2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35353[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_Desire_X/2.23.111.4 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35354[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; HTC_Desire_X_dual_sim Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35355[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; HTC_Desire_X/2.23.114.2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35356[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; HTC_Desire_XC_dual_sim Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35357[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; HTC Desire X dual sim Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35358[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-at; HTC_Desire_X/2.23.112.2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35359[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; HTC_Desire_X-orange-LS/1.14.75.1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35360[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; HTC Desire XC dual sim Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35361[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; HTC_Desire_X-orange-LS/2.22.75.8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35362[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:18.0) Gecko/X Firefox/18.0";
const char S35363[] = "RSS Scout 0.11.2";
const char S35364[] = "OppO 1.0 ( http://www.inboundscore.com )";
const char S35365[] = "news.me extractor";
const char S35366[] = "LinkChecker/7.9 (+http://linkchecker.sourceforge.net/)";
const char S35367[] = "StatusNet/1.1.0-release (Fight for Your Right)";
const char S35368[] = "SSL Labs (https://www.ssllabs.com/about/assessment.html)";
const char S35369[] = "Mozilla/5.0 (compatible; LinkJumper/0.1.2; +Someday ya skazhu)";
const char S35370[] = "Mozilla/5.0 (compatible; IODC-Internet Engineering Odysseus 001; +http://iodc.co.uk)";
const char S35371[] = "http://www.americanexpress.com [amex:prodSearch]";
const char S35372[] = "check_http/v1.4.16 (nagios-plugins 1.4.16)";
const char S35373[] = "Tumblr/14.0.835.186";
const char S35374[] = "Mechanize/2.6.0 Ruby/1.9.2p290 (http://github.com/sparklemotion/mechanize/)";
const char S35375[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S35376[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7,gzip(gfe) (via translate.google.com)";
const char S35377[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7";
const char S35378[] = "Mozilla/5.0 (X11; Jolicloud Linux i686) AppleWebKit/535.7 (KHTML, like Gecko) Joli OS/1.2 Chromium/16.0.912.63 Chrome/16.0.912.63 Safari/535.7";
const char S35379[] = "Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S35380[] = "Mozilla/5.0 (X11; U; Linux; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7 Safari/532.4";
const char S35381[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.7 (KHTML	 like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S35382[] = "Mozilla/5.0 (X11; Jolicloud Linux i686) AppleWebKit/535.7 (KHTML	 like Gecko) Joli OS/1.2 Chromium/16.0.912.63 Chrome/16.0.912.63 Safari/535.7";
const char S35383[] = "\"Mozilla/5.0(X11;Linuxx86_64)AppleWebKit/535.7(KHTML,likeGecko)Chrome/16.0.912.77Safari/535.7\"";
const char S35384[] = "Mozilla/5.0 (Linux) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S35385[] = "Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/16.0.1410.63 Safari/537.31";
const char S35386[] = "Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/16.0.1410.63 Safari/537.31";
const char S35387[] = "Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/16.0.1410.63 Safari/537.31";
const char S35388[] = "Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/16.0.1410.63 Safari/537.31";
const char S35389[] = "Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/16.0.1410.63 Safari/537.31";
const char S35390[] = "Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/16.0.1410.63 Safari/537.31";
const char S35391[] = "Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/16.0.1410.63 Safari/537.31";
const char S35392[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405";
const char S35393[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334";
const char S35394[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)";
const char S35395[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)";
const char S35396[] = "Miu Ptt 1.5.2 (iPod touch; iPhone OS 5.0.1; zh_TW)";
const char S35397[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 5_0 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/5.0;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S35398[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 5_0_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/5.0.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S35399[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 iPod4,1 BingWeb/3.03.1428.X";
const char S35400[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 iPod4,1 BingWeb/3.02.1641.X";
const char S35401[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/5.0.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]";
const char S35402[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/5.0;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]";
const char S35403[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper";
const char S35404[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:19.0) Gecko/X Firefox/19.0";
const char S35405[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8600/S8600DDKK4; U; Bada/2.0; en-us) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile WVGA SMM-MMS/1.2.0 OPN-B";
const char S35406[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T769 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35407[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-T769 Build/GINGERBREAD)";
const char S35408[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T769Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35409[] = "Mozilla/5.0 (Linux; U; Android2.3.6; en-us; SGH-T769 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35410[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us;SGH-T769 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35411[] = "Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; SGH-T769 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35412[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T769 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S35413[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T769 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S35414[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SGH-T769 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35415[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-SGH-I897/I897UCJK1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35416[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-I897 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35417[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-SGH-I897/I897UCKB1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35418[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-SGH-I897/I897UCJL2 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35419[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SAMSUNG-SGH-I897 Build/FROYO XWJS5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35420[] = "Mozilla/5.0 (Linux; U; Android 2.2.1 JS3; en-us; SGH-I897 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35421[] = "Mozilla/5.0 (Linux; U; Android 2.2.1 JS5; en-us; SGH-I897 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35422[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-SGH-I897 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35423[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SAMSUNG-SGH-I897 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35424[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-I897 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35425[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-SGH-I897/I897UCJL1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35426[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-SGH-I897/I897UCJI6 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35427[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-SGH-I897/I897UCJJ4 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35428[] = "Mozilla/5.0 (Linux; U;Android 2.2; en-us; SAMSUNG-SGH-I897/I897UCKB1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35429[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-SGH-I897/I897UCKB1Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35430[] = "Mozilla/5.0 (Linux; U; Android2.2; en-us; SAMSUNG-SGH-I897/I897UCKB1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35431[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us;SAMSUNG-SGH-I897/I897UCKB1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35432[] = "Mozilla/5.0 (Linux; U; Android 2.2; ko-; SAMSUNG-SGH-I897/I897UCKB1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35433[] = "Mozilla/5.0 (Linux; U; Android 2.2.1 JS8; en-gb; SAMSUNG-SGH-I897 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35434[] = "Logos/0.9.1 (Linux; U; Android 2.2; en_US; SAMSUNG-SGH-I897 Build/FROYO.UCKB2)";
const char S35435[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-SGH-I897/I897UCJJ3 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35436[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-SGH-I897Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35437[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; SAMSUNG-SGH-I897 Build/FROYO)";
const char S35438[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-I897 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35439[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-SGH-I897 Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S35440[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SAMSUNG-SGH-I897 Build/FROYO)";
const char S35441[] = "Mozilla/5.0 (Linux; U; Android 2.2.1 JS7; en-us; SAMSUNG-SGH-I897 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35442[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SGH-I897 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S35443[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-; SAMSUNG-SGH-I897 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35444[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N5110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S35445[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N5110 Build/JZO54K)";
const char S35446[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/5";
const char S35447[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.55 Safari/533.4";
const char S35448[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; YP-G70 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35449[] = "Logos/0.9.4 (Linux; U; Android 2.3.5; en_US; YP-G70 Build/GINGERBREAD.UEKI8)";
const char S35450[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; YP-G70 Build/GINGERBREAD)";
const char S35451[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; YP-G70 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35452[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; YP-G70 Build/GINGERBREAD)";
const char S35453[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; YP-G70 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35454[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB865 Build/6.7.2_169) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35455[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB865 Build/6.7.2-226-EDICS-8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35456[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MB865 Build/6.7.2_EDEM-18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35457[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35458[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ME865 Build/6.7.2_GC-220-EDS-3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35459[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10)";
const char S35460[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ME865 Build/6.7.2_GC-180-EDS-20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35461[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S35462[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB865 Build/67.21.125 by timmytim) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35463[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MB865 Build/6.7.2_EDEM-20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35464[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ME865 Build/6.7.2_GC-180-EDS-20) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S35465[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ME865 Build/6.7.2_GC-220-EDS-3)";
const char S35466[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35467[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; MB865 Build/6.7.2_EDEM-17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35468[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MB865 Build/6.7.2-226-EDICS-8) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S35469[] = "MOT-MB865/ Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB865 Build/6.7.2-226-EDICS-10) 1617 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35470[] = "BlackBerry9650/5.0.0.1006 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/105";
const char S35471[] = "Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.12";
const char S35472[] = "Opera/9.80 (Windows NT 6.0; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.12";
const char S35473[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.12";
const char S35474[] = "Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.12";
const char S35475[] = "Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.12";
const char S35476[] = "Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.12,gzip(gfe)";
const char S35477[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.12";
const char S35478[] = "Opera/9.80 (Windows NT 6.0; WOW64) Presto/2.12.388 Version/12.12";
const char S35479[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.12";
const char S35480[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.12";
const char S35481[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.12";
const char S35482[] = "Opera/9.80 (Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)) Presto/2.12.388 Version/12.12";
const char S35483[] = "Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.12";
const char S35484[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.12";
const char S35485[] = "Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.12";
const char S35486[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0) Opera 12.12";
const char S35487[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.12";
const char S35488[] = "Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.12";
const char S35489[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.12";
const char S35490[] = "Opera/9.80 (Windows NT 6.1; WOW64; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)) Presto/2.12.388 Version/12.12";
const char S35491[] = "Opera/9.80 (Windows NT 6.1; WOW64; compatible; MSIE 7.0; Windows NT 6.0) Presto/2.12.388 Version/12.12";
const char S35492[] = "Opera/9.80 (Windows NT 6.0; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.12";
const char S35493[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.12";
const char S35494[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.12";
const char S35495[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.12";
const char S35496[] = "Mozilla/5.0 (Windows NT 6.0; rv:2.0) Gecko/X Firefox/4.0 Opera 12.12";
const char S35497[] = "Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.12";
const char S35498[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.12";
const char S35499[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.12";
const char S35500[] = "Opera/9.80 (Windows NT 5.1; compatible; MSIE 7.0; Windows NT 6.0) Presto/2.12.388 Version/12.12";
const char S35501[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.12";
const char S35502[] = "Opera/9.80 (Windows NT 6.0; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.12";
const char S35503[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.12";
const char S35504[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.12";
const char S35505[] = "Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.12";
const char S35506[] = "Opera/9.80 (Windows NT 6.0; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.12";
const char S35507[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.12";
const char S35508[] = "Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.12";
const char S35509[] = "Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.12";
const char S35510[] = "Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.12";
const char S35511[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.12";
const char S35512[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Silk/1.0.13.328_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Silk-Accelerated=true";
const char S35513[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Silk/1.0.22.79_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Silk-Accelerated=true";
const char S35514[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Silk/1.1.0-84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Silk-Accelerated=true";
const char S35515[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Silk/1.1.0-80) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Silk-Accelerated=true";
const char S35516[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Silk/1.0.13.81_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Silk-Accelerated=true,gzip(gfe) (via translate.google.com)";
const char S35517[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Silk/1.0.22.153_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Silk-Accelerated=true";
const char S35518[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:16.0) Gecko/X Firefox/16.0";
const char S35519[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB865 Build/5.5.1-175_EDIFW2-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35520[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB865 Build/5.5.1-134) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35521[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB865 Build/5.5.1-149) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35522[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB865 Build/5.5.1_100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35523[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB865 Build/5.5.1-175_EDFFW1-13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35524[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB865 Build/5.5.1-175_EDFFW1-16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35525[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB865 Build/5.5.1-175_EDFFW1-17) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35526[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB865 Build/5.5.1-175_EDFFW1-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35527[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB865 Build/5.5.1-175_EDFFW1-11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35528[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB865 Build/5.5.1-175_EDMR1.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35529[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; ME865 Build/5.5.1-1_GC-89) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35530[] = "Logos/0.9.4 (Linux; U; Android 2.3.6; en_US; MB865 Build/5.5.1-175_EDMR1.25)";
const char S35531[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; MB865 Build/5.5.1-175_EDMR1.25)";
const char S35532[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB865 Build/5.5.1-175_EDMR1.25) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.1 Mobile Safari/534.30";
const char S35533[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB865 Build/5.5.1-175_EDMR1.25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35534[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; MB865 Build/5.5.1-175_EDFFW1-16)";
const char S35535[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB865 Build/5.5.1-175_EDMR1.22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35536[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MB865 Build/5.5.1-EDEM-28) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35537[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MB865 Build/5.5.1-EDEM-27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35538[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MB865 Build/5.5.1-EDEM-28-37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35539[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MB865 Build/5.5.1-256_EDL-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35540[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB865 Build/5.5.1-175_EDMR1.15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35541[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME865 Build/5.5.1-1_GC-90_EDISON-6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35542[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME865 Build/5.5.1-1_GC-90_EDISON-11) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S35543[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ME865 Build/5.5.1-1_GC-109) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35544[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB865 Build/5.5.1-256_EDL-27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35545[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MB865 Build/5.5.1-256_EDL-34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35546[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB865 Build/5.5.1-175_EDMR1.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35547[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ME865 Build/5.5.1-1_GC-86_EDSCHN-6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35548[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ME865 Build/5.5.1-1_GC-90_EDISON-11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35549[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ME865 Build/5.5.1-1_GC-90_EDSHKTW-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35550[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME865 Build/5.5.1-1_GC-109) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S35551[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME865 Build/5.5.1-1_GC-109) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S35552[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME865 Build/5.5.1-1_GC-90_EDSHKTW-2) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S35553[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME865 Build/5.5.1-1_GC-90_EDISON-11) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S35554[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MB865 Build/5.5.1-256_EDL-34.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35555[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M580 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35556[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M580BST Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S35557[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M580BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35558[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SPH-M580BST Build/GINGERBREAD)";
const char S35559[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SPH-M580 Build/GINGERBREAD)";
const char S35560[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; -us; SPH-M580BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35561[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M580 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S35562[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; -us; SPH-M580 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35563[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B117 Safari/532.9";
const char S35564[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko)";
const char S35565[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B117";
const char S35566[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/X.X (KHTML, like Gecko) Version/X.X Mobile/XXXX Safari/X.X";
const char S35567[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/9Z999";
const char S35568[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.1;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S35569[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8J2";
const char S35570[] = "Mozilla/5.0 (iPhone; U; CPU iPhone 4_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPhone2,1/4.1)";
const char S35571[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35572[] = "Mozilla/5.0 (Linux; U; Android Bonsai/Kernel_v5.0.0-EF02; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35573[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35574[] = "Mozilla/5.0 (Linux; U; Android RandomROM 2.3.4; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35575[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SPH-D700 Build/GINGERBREAD)";
const char S35576[] = "Mozilla/5.0 (Linux; U; Android Samurai-2.6.35_gen1.0.2; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35577[] = "Mozilla/5.0 (Linux; U; Android Gingerbread 2.3.5; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35578[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S35579[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SPH-D700 Build/GINGERBREAD)";
const char S35580[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SPH-D700 Build/GINGERBREAD)";
const char S35581[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35582[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35583[] = "Mozilla/5.0 (Linux; U; Android nubernel-2.6.35_v0.0.1; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35584[] = "Mozilla/5.0 (Linux; U; Android EI22_IAP_Samurai-2.6.35_gen1.0.4; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35585[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SPH-D700 Build/GINGERBREAD)";
const char S35586[] = "Mozilla/5.0 (Linux; U; Android nubernel-2.6.35_v0.0.0; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35587[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en_us; SPH-D700 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)";
const char S35588[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; SPH-D700 Build/GINGERBREAD)";
const char S35589[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35590[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-D700 Build/Mortify ROM v2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35591[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PC36100 Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35592[] = "Mozilla/5.0 (Linux; U; Android 4.0.3 slow churned; en-us; PC36100 Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35593[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Evo 4G Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35594[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PC36100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35595[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PC36100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35596[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; EVO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35597[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PC36100 Build/IBZ40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35598[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PC36100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35599[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Evo 4G Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35600[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; EVO Build/IML74K)";
const char S35601[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PC36100 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35602[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; PC36100 Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35603[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EVO Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35604[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PC36100 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35605[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sprint APA9292KT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35606[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; EVO Build/IMM76D)";
const char S35607[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; PC36100 Build/IMM76I)";
const char S35608[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EVO Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S35609[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; DEVO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S35610[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; ODYS-EVO Build/ODYS-EVO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S35611[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; EVO Build/ElegenciaRom) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35612[] = "Mozilla/5.0 (Linux; U; Android 4.0.2; zh-cn; HTC EVO 4G+ For Sprint Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S35613[] = "Opera/9.80 (Series 60; Opera Mini/6.0.24095/24.760; U; en) Presto/2.5.25 Version/10.54";
const char S35614[] = "Opera/9.80 (Series 60; Opera Mini/6.0.24455/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S35615[] = "Mozilla/5.0 (compatible; cdlwas_bot; heritrix/1.14.1 +http://webarchives.cdlib.org/p/webmasters)";
const char S35616[] = "webcrawler (compatible; heritrix/1.14.4 ++http://www.onb.ac.at/about/webarchivierung.htm)";
const char S35617[] = "Arquivo-web-crawler  (compatible; heritrix/1.14.3 +http://arquivo.pt/faq-crawling)";
const char S35618[] = "Mozilla/5.0 (compatible; heritrix/1.14.3 +http://www.accelobot.com)";
const char S35619[] = "Mozilla/5.0 (compatible; heritrix/3.1.0-RC1 +http://vefsafn.is/crawler)";
const char S35620[] = "Mozilla/5.0 (compatible; heritrix/1.14.4 +http://netarkivet.dk/webcrawler/)";
const char S35621[] = "Mozilla/5.0 (compatible;archive.org_bot/heritrix-1.9.0-X +http://pandora.nla.gov.au/crawl.html)";
const char S35622[] = "Mozilla/5.0 (compatible; heritrix/1.x.x +http://www.accelobot.com)";
const char S35623[] = "Mozilla/5.0 (compatible; worio bot heritrix/1.10.0 +http://worio.com)";
const char S35624[] = "Mozilla/5.0 (compatible; heritrix/1.3.0 http://archive.crawler.org)";
const char S35625[] = "Mozilla/5.0 (compatible; heritrix/3.1.2-SNAPSHOT-X.X +http://archive.org/details/archive.org_bot)";
const char S35626[] = "HopperBot/1.1 (compatible; heritrix/3.1.0 +http://www.hopper.com)";
const char S35627[] = "Mozilla/5.0 (compatible; heritrix/3.1.2-SNAPSHOT-X.X +http://www.archive.org/details/archive.org_bot)";
const char S35628[] = "Mozilla/5.0 (compatible; heritrix/3.1.1 +http://www.baidu.com)";
const char S35629[] = "citeseerxbot (compatible; heritrix/1.14.4 +http://citeseerx.ist.psu.edu)";
const char S35630[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.13";
const char S35631[] = "Mozilla/5.0 (Windows NT 5.1; rv:2.0) Gecko/X Firefox/4.0 Opera 12.13";
const char S35632[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.13";
const char S35633[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.13";
const char S35634[] = "Opera/9.80 (Windows NT 5.2; WOW64) Presto/2.12.388 Version/12.13";
const char S35635[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.13";
const char S35636[] = "Opera/9.80 (Windows NT 5.2) Presto/2.12.388 Version/12.13";
const char S35637[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.13";
const char S35638[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.13";
const char S35639[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.13";
const char S35640[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.13,gzip(gfe)";
const char S35641[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.13";
const char S35642[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.13";
const char S35643[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.13";
const char S35644[] = "Opera/9.80 (Windows NT 5.1; Edition Campaign 21) Presto/2.12.388 Version/12.13";
const char S35645[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.13";
const char S35646[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.13";
const char S35647[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.13";
const char S35648[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.13";
const char S35649[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.13";
const char S35650[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.13";
const char S35651[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.13";
const char S35652[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.13";
const char S35653[] = "Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.13";
const char S35654[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.13";
const char S35655[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.13";
const char S35656[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.13";
const char S35657[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.13";
const char S35658[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.13";
const char S35659[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.13";
const char S35660[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1) Opera 12.13";
const char S35661[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.13";
const char S35662[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.13";
const char S35663[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.13";
const char S35664[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.13";
const char S35665[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.13";
const char S35666[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.13";
const char S35667[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.13";
const char S35668[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.13";
const char S35669[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.13";
const char S35670[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.13";
const char S35671[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.13";
const char S35672[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.13";
const char S35673[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.13";
const char S35674[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.13";
const char S35675[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.13";
const char S35676[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.13";
const char S35677[] = "Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.13 Yandex.Translate";
const char S35678[] = "Opera/9.80 (Windows NT 5.1; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.13";
const char S35679[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.13";
const char S35680[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.13";
const char S35681[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.13";
const char S35682[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.13";
const char S35683[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6035)) Presto/2.12.388 Version/12.13";
const char S35684[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.13";
const char S35685[] = "Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.13";
const char S35686[] = "Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.13";
const char S35687[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.13";
const char S35688[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.13";
const char S35689[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.13";
const char S35690[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.13";
const char S35691[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.13";
const char S35692[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.13";
const char S35693[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.13";
const char S35694[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.13";
const char S35695[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6032)) Presto/2.12.388 Version/12.13";
const char S35696[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.13";
const char S35697[] = "Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.13";
const char S35698[] = "Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.13";
const char S35699[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.13";
const char S35700[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.13";
const char S35701[] = "Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.13";
const char S35702[] = "Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.13";
const char S35703[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.13";
const char S35704[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.13";
const char S35705[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.13";
const char S35706[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.13";
const char S35707[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.13";
const char S35708[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.13";
const char S35709[] = "Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6044)) Presto/2.12.388 Version/12.13";
const char S35710[] = "Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.13";
const char S35711[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; U8650 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35712[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-tw; HUAWEI_IDEOS_U8650 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35713[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HUAWEI SONIC Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35714[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GGR Sonic Next Gen Build/HuaweiU8650)";
const char S35715[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-hr; GGR Sonic Next Gen Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35716[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; U8650 Build/HuaweiU8650B836) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35717[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; U8650 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35718[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SONIC Build/HuaweiU8660) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S35719[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; U8650 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35720[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; U8650 Build/HuaweiU8650)";
const char S35721[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; U8650 Build/HuaweiU8650B835SP01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35722[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; el-gr; U8650 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35723[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-pt; U8650-1 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35724[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HuaweiU8650 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35725[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-cn; SONIC Build/HuaweiU8660) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35726[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-cn; SONIC Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35727[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; HUAWEI_IDEOS_U8650 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35728[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; Kyivstar Aqua Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35729[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; MTC 955 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35730[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; U8650 Build/HuaweiU8650)";
const char S35731[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; U8650 Build/HuaweiU8650B882) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35732[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; MTC 955 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35733[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; U8650 Build/HuaweiU8650B836)";
const char S35734[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; tr-tr; U8650-1 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35735[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HUAWEI SONIC Build/HuaweiU8650)";
const char S35736[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SONIC Build/HuaweiU8650) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S35737[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; u8650 Build/benru89) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35738[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SONIC Build/HuaweiU8660) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S35739[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SONIC Build/HuaweiU8660)";
const char S35740[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; U8650-1 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35741[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; U8650 Build/HuaweiU8650B836) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35742[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-in; HuaweiU8650 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35743[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SONIC Build/HuaweiU8650) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S35744[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; Huawei u8650 Build/Lunar Prime; Lunar Prime) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35745[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; U8650 Build/HuaweiU8650)";
const char S35746[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; U8650 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35747[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; U8650 Build/HuaweiU8650B888) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35748[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; Lucky Ultra Sonic U8650 Build/HuaweiU8650B836) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35749[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.55 Safari/533.4";
const char S35750[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.127 Safari/533.4";
const char S35751[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/533.2 (KHTML, like Gecko) Chrome/5.0.342.9 Safari/533.2";
const char S35752[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.127 Safari/533.4";
const char S35753[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/533.2 (KHTML, like Gecko) Chrome/5.0.342.9 Safari/533.2";
const char S35754[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.86 Safari/533.4";
const char S35755[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fr-FR) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.126 Safari/533.4";
const char S35756[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.127 Safari/533.4";
const char S35757[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.86 Safari/533.4";
const char S35758[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/5.0.354.0 Safari/533.3";
const char S35759[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; DL DesireHD v2.0 internal Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35760[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; HTCA9191/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35761[] = "Mozilla/5.0 (Linux; U; Android 2.2; iw-; HTC_DesireHD_A9191 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35762[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC_DesireHD-orange-LS Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35763[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-tw; HTC_DesireHD_A9191 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35764[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Desire HD 1.39.161.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35765[] = "Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; HTC Desire HD 1.32.163.6 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35766[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; HTC Desire HD 1.75.163.2 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35767[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Desire HD Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35768[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Desire HD 1.75.161.2 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35769[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; HTC Desire HD 1.32.163.6 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35770[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; HTC Desire HD 1.32.163.5 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35771[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Desire HD 1.39.161.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35772[] = "Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; HTC Desire HD 1.32.163.5 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35773[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC_DesireHD-orange-LS Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35774[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; HTCA9191/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35775[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-mx; HTC_DesireHD Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35776[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;Desire HD Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35777[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC_DesireHD Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35778[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; Desire HD Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35779[] = "Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; 001HT Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35780[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Desire HD Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35781[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; HTC_DesireHD_A9192 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35782[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC_DesireHD_A9191 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35783[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC_DesireHD_A9191 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35784[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; Desire HD Build/FRF91)";
const char S35785[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; Desire HD Build/FRG83D)";
const char S35786[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; Desire HD Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S35787[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; Desire HD Build/FRG83D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S35788[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; 001HT Build/FRF91)";
const char S35789[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Desire HD Build/FRG83D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S35790[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; Desire HD Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S35791[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; Desire HD Build/FRG83D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S35792[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S35793[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S35794[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/528.16 (KHTML, like Gecko) Version/4.0 Safari/528.16";
const char S35795[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S35796[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9.1";
const char S35797[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S35798[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532+ (KHTML, like Gecko) Version/4.0.2 Safari/530.19.1";
const char S35799[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Safari/528.17";
const char S35800[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/526.11 (KHTML, like Gecko) Version/4.0dp1 Safari/526.12.2";
const char S35801[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S35802[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S35803[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S35804[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Sprint APX515CKT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35805[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PG86100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35806[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; HTC/EVO_3D/1.21.161.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35807[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-ch; HTC/EVO_3D/1.21.166.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35808[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; HTC/EVO_3D/1.21.163.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35809[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; Sprint APX515CKT Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35810[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PG86100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35811[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Sprint APX515CKT Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35812[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; PG86100 Build/GRI40)";
const char S35813[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; PG86100 Build/GRJ22)";
const char S35814[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PG86100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35815[] = "Logos/0.9.4 (Linux; U; Android 2.3.4; en_US; PG86100 Build/GRJ22)";
const char S35816[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-ch; HTC/EVO_3D/1.21.166.5 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35817[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; ISW12HT Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35818[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; ISW12HT Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30";
const char S35819[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Sprint APX515CKT Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35820[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; HTC/EVO_3D/1.21.161.5 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35821[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PG86100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35822[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; HTC/EVO_3D/1.24.165.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35823[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; HTC/EVO_3D/1.21.163.4 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35824[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PG86100 Build/GRJ22) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S35825[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; PG86100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35826[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; ISW12HT Build/GRJ22)";
const char S35827[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; HTC/EVO_3D/1.24.165.2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35828[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; APX515CKT Build/GRJ22) AppleWebKit/533.1 (KHTML	 like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35829[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; PG86100 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S35830[] = "Mozilla/5.0 (Linux; U; Android 2.2; pt-br; GT-I5500B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35831[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; GT-I5500L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35832[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-I5500L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35833[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I5500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35834[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; GT-I5500L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35835[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I5500 Build/FROYO)";
const char S35836[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I5500L Build/FROYO)";
const char S35837[] = "Mozilla/5.0 (Linux; U; Android 2.2; hr-hr; GT-I5500 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S35838[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-I5500L Build/FROYO)";
const char S35839[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I5500B Build/FROYO)";
const char S35840[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; GT-I5500B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35841[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-I5500M Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35842[] = "Mozilla/5.0 (Linux; U; Android 2.2;es-es;GT-I5500 Build/ FROYO) AppleWebKit/533.1 (KHTML, like Geccko) Version/4.0 Mobile Safari/533.1";
const char S35843[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; GT-I5500L Build/FROYO)";
const char S35844[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-I5500 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35845[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; GT-I5500B Build/FROYO)";
const char S35846[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; GT-I5500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35847[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-; GT-I5500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35848[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGPT12 Build/TISU0105) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S35849[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGPT12 Build/TISU0122) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S35850[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGPT12 Build/TID0129) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S35851[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGPT12 Build/TID0136) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S35852[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGPT13 Build/TID0129) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S35853[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGPT12 Build/TID0142) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S35854[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGPT13 Build/TID0142) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S35855[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; SGPT12 Build/TISU0118) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S35856[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SGPT12 Build/TID0142)";
const char S35857[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SGPT13 Build/TID0142)";
const char S35858[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; SGPT13 Build/TID0136) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S35859[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N860 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S35860[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N860 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35861[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; N860 Build/GINGERBREAD)";
const char S35862[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N860 Build/WP7 ROM 2.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35863[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N860 Build/Shinjitsu Rom) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35864[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; N860 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35865[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N860 Build/n00b Rom) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35866[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N860 Build/ROM by Bloodawn) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35867[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; GT-I9003L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35868[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35869[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-I9003 Build/FROYO)";
const char S35870[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-I9003 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S35871[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-I9003L Build/FROYO)";
const char S35872[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-I9003 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S35873[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35874[] = "Mozilla/5.0 (Linux; Android 4.0.4;  GT-I9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S35875[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S35876[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300T Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S35877[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9300 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S35878[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S35879[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I535 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S35880[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999V Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S35881[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-06D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S35882[] = "Mozilla/5.0 (Linux; Android 4.0.4;  GT-I9300T Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S35883[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S35884[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S35885[] = "Mozilla/5.0 (Linux; Android 4.0.4; i9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S35886[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I939 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S35887[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I93000 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S35888[] = "Mozilla/5.0 (Linux; Android 4.0.9; GT-l9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S35889[] = "Mozilla/5.0 (Linux; Android 4.0.9; I9300+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S35890[] = "Mozilla/5.0 (Linux; Android 4.0.9; I9300 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S35891[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-06D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari 535.19";
const char S35892[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-l9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S35893[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16 chromeframe/10.0.648.204";
const char S35894[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/10.0.648.204; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322; InfoPath.2; .NET4.0C; .NET4.0E; MS-RTC LM 8)";
const char S35895[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; SV1; chromeframe/10.0.648.205; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.1; MS-RTC LM 8)";
const char S35896[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/10.0.648.151; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S35897[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/10.0.648.151; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C)";
const char S35898[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16 chromeframe/10.0.648.205";
const char S35899[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/10.0.648.204; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322; InfoPath.2; .NET4.0C; .NET4.0E; MS-RTC LM 8)";
const char S35900[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/10.0.648.205; SV1; MS-RTC LM 8; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S35901[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B143 Safari/8536.25";
const char S35902[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B350 Safari/8536.25";
const char S35903[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B329 Safari/8536.25";
const char S35904[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B146 Safari/8536.25";
const char S35905[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B144 Safari/8536.25";
const char S35906[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B145 Safari/8536.25";
const char S35907[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B142 Safari/8536.25";
const char S35908[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B141 Safari/8536.25";
const char S35909[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U9200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35910[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U9200 Build/HuaweiU9200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35911[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; GS03 Build/HuaweiU9200-92) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35912[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; U9200 Build/HuaweiU9200) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S35913[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; U9200 Build/HuaweiU9200)";
const char S35914[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; U9200 Build/HuaweiU9200C02B013) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35915[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; U9200 Build/HuaweiU9200C02B021) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35916[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; U9202L-1 Build/HuaweiU9202L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35917[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; U9200E Build/HuaweiU9200E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35918[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; U9200 Build/HuaweiU9200C02B023) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35919[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; U9200 Build/HuaweiU9200C02B009) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35920[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; U9200 Build/HuaweiU9200C02B013SP3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35921[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; U9202L-3 Build/HuaweiU9202L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35922[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; U9200E Build/HuaweiU9200E)";
const char S35923[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; U9200E Build/HuaweiU9200E) AppleWebKit/534.30 (KHTML, like Gecko) Phi?n b?n/4.0 Mobile Safari/534.30";
const char S35924[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 820)";
const char S35925[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 820; Vodafone)";
const char S35926[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 820; ANZ825)";
const char S35927[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 820; Orange)";
const char S35928[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia 820)";
const char S35929[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 820; 1328-3)";
const char S35930[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S35931[] = "Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7300-ORANGE/P7300BVKJ5 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S35932[] = "Mozilla/5.0 (Linux; U; Android 3.2; th-th; GT-P7300B Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S35933[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7300 Build/HTJ85B)";
const char S35934[] = "Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7300-ORANGE/P7300BVLC2 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S35935[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S35936[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S35937[] = "Mozilla/5.0 (Linux; U; Android 3.2; cs-cz; SAMSUNG-GT-P7300/P7300BOLC1 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S35938[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7300B Build/HTJ85B)";
const char S35939[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7300 Build/HTJ85B; 1280*752) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S35940[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7300) AntivirusAM/2.5 (KHTML, like Gecko) Version/Version 2.5/Antivirus";
const char S35941[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S35942[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7300 Build/HTJ85B) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S35943[] = "Mozilla/5.0 (Linux; U; Android 3.2; nl-nl; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 TwonkyBeamBrowser/3.2.1-65 (Android 3.2; samsung GT-P7300 Build/HTJ85B)";
const char S35944[] = "Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 3.2; SAMSUNG GT-P7300 Build/HONEYCOMB_MR2);";
const char S35945[] = "Logos/2.2.3 (Linux; U; Android 3.2; en-GB; GT-P7300 Build/HTJ85B)";
const char S35946[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7300 Build/HTJ85B) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S35947[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7300 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S35948[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7300 Build/HTJ85B like Xoom) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S35949[] = "Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7300-ORANGE/P7300BVKT5 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S35950[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/2.0;FBDM/{density=1.0,width=1280,height=752};FBLC/en_US;FB_FW/2;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P7300;FBSV/3.2;]";
const char S35951[] = "Groupon/2.2.2572 (Android 3.2; Samsung GT-P7300 / Samsung GT-P7300; )[preload=false;locale=en_US;clientidbase=android-samsung]";
const char S35952[] = "Android 3.2; 13; samsung; GT-P7300";
const char S35953[] = "Mozilla/5.0 (Linux; U; Android 3.2; fr-fr;GT-P7300-ORANGE/P7300BVLC2 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S35954[] = "Mozilla/5.0 (Linux; U; Android 3.2; fa-fa; GT-P7300 Build/HTJ85B) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S35955[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S35956[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2)";
const char S35957[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2; WOW64; Trident/4.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729)";
const char S35958[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2; WOW64; Trident/4.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Zune 4.7)";
const char S35959[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2; WOW64; Trident/4.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S35960[] = "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.2; Trident/4.0)";
const char S35961[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2; WOW64; Trident/4.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0)";
const char S35962[] = "'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center C 6.0)";
const char S35963[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727)";
const char S35964[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; LG-E510f Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S35965[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-E510 Build/GRJ22)";
const char S35966[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-sa; LG-E510g Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S35967[] = "LG-E510/V10c Mozilla/5.0 (Linux; U; Android 2.3.6; el-gr; LG-E510/V10c Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S35968[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-E510f Build/GRJ22)";
const char S35969[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-E510 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S35970[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; LG-E510 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S35971[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; LG-E510f Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S35972[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; LG-E510/${Version} Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S35973[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-E510 Build/GRJ22)";
const char S35974[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-E510g Build/GRJ22)";
const char S35975[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; LG-E510/${Version} Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S35976[] = "LG-E510/V10d Mozilla/5.0 (Linux; U; Android 2.3.6; el-gr; LG-E510/V10d Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S35977[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; LG-E510 Build/GRJ22)";
const char S35978[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaC6-00.1/20.2.042; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) BrowserNG/7.2.6.9 3gpp-gba";
const char S35979[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaC6-00/20.0.042; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) BrowserNG/7.2.6.9 3gpp-gba";
const char S35980[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaC6-00/42.0.004; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 3gpp-gba";
const char S35981[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaC6-00.1/42.2.004; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 3gpp-gba";
const char S35982[] = "Mozilla/5.0 (Linux; Android 4.1.1; DROID RAZR HD Build/9.8.1Q_39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S35983[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT925 Build/9.8.2Q-8-XT925_VQL-12.2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S35984[] = "Mozilla/5.0 (Linux; Android 4.1.1; DROID RAZR HD Build/9.8.1Q_39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S35985[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S35986[] = "Mozilla/5.0 (Linux; Android 4.1.2; RAZR HD Build/9.8.2Q-8-XT925_VQL-6) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S35987[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT925 Build/9.8.2Q-8-XT925_VQL-12.2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S35988[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S35989[] = "Mozilla/5.0 (Linux; Android 4.1.1; XT925 Build/9.8.0Q-97-XT925_VQU-19) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S35990[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; MT25i Build/4.1.B.0.479) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35991[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; MT25i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35992[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; MT25i Build/4.1.B.0.626) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35993[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; SonyEricssonMT25i Build/4.1.B.0.631) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35994[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; MT25i Build/4.1.B.0.631) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35995[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MT25i Build/4.1.B.0.631)";
const char S35996[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fa-; MT25i Build/4.1.B.0.631) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S35997[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B329 Safari/8536.25";
const char S35998[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B350 Safari/8536.25";
const char S35999[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B146 Safari/8536.25";
const char S36000[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B144 Safari/8536.25";
const char S36001[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B142 Safari/8536.25";
const char S36002[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B145 Safari/8536.25";
const char S36003[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B143 Safari/8536.25";
const char S36004[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B141 Safari/8536.25";
const char S36005[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ALCATEL_one_touch_918M Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36006[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; ALCATEL_one_touch_918D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36007[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; ALCATEL ONE TOUCH 918D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36008[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; ALCATEL ONE TOUCH 918 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36009[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ALCATEL ONE TOUCH 918S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36010[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ALCATEL ONE TOUCH 918A Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36011[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; ALCATEL ONE TOUCH 918D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36012[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; iw-il; ALCATEL_ONE_TOUCH_918M Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36013[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; nl-nl; ALCATEL_OT_918D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36014[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ro-ro; Alcatel_one_touch_918_Orange Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36015[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; ALCATEL_one_touch_918 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36016[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; ALCATEL ONE TOUCH 918D Build/GRJ90)";
const char S36017[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-; ALCATEL ONE TOUCH 918A Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36018[] = "Opera/9.80 (Android; Opera Mini/14.0.1074/30.3214; U; en) Presto/2.8.119 Version/11.10";
const char S36019[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 900)";
const char S36020[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 900; Orange)";
const char S36021[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Nokia 900)";
const char S36022[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 900; Vodafone)";
const char S36023[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 900; None)";
const char S36024[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 900),gzip(gfe) (via docs.google.com/viewer)";
const char S36025[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6400L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36026[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6400L 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36027[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6400L 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36028[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ThunderBolt Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36029[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6400L 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36030[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6400L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36031[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6400L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36032[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6400L 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36033[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ADR6400L Build/IMM76D)";
const char S36034[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ThunderBolt Build/IML74K)";
const char S36035[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; ADR6400L Build/IML74K)";
const char S36036[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; ADR6400L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36037[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6400L Build/MaceksMod Mecha Rebirth) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36038[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6400L 4G Build/ICE-ICE Baby) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36039[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S36040[] = "Mozilla/5.0 (Linux; Android 4.2; Nexus 7 Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S36041[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 7 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S36042[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S36043[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows 98; Maxthon; Maxthon)";
const char S36044[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; Maxthon)";
const char S36045[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows 98; Maxthon; Avant Browser)";
const char S36046[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Maxthon)";
const char S36047[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; Maxthon)";
const char S36048[] = "Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; Maxthon)";
const char S36049[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Maxthon)";
const char S36050[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Maxthon)";
const char S36051[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Maxthon)";
const char S36052[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; SV1; Maxthon)";
const char S36053[] = "Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; SV1; Maxthon)";
const char S36054[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Maxthon; Maxthon)";
const char S36055[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; Maxthon; Avant Browser)";
const char S36056[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Maxthon; SV1)";
const char S36057[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Maxthon; Maxthon)";
const char S36058[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Maxthon; Maxthon)";
const char S36059[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Avant Browser; Maxthon)";
const char S36060[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Maxthon; SV1)";
const char S36061[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Avant Browser; Maxthon)";
const char S36062[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Avant Browser; Maxthon)";
const char S36063[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Maxthon; Avant Browser)";
const char S36064[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; SV1; Maxthon)";
const char S36065[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Maxthon; SV1)";
const char S36066[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Maxthon; Maxthon)";
const char S36067[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; SV1; Avant Browser; Maxthon)";
const char S36068[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; QQBrowser/7.3.8126.400)";
const char S36069[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; QQBrowser/7.3.8126.400)";
const char S36070[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; QQBrowser/7.3.11251.400)";
const char S36071[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; QQBrowser/7.3.11251.400)";
const char S36072[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/7.3.11251.400)";
const char S36073[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/7.3.11251.400)";
const char S36074[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; QQBrowser/7.3.11251.400)";
const char S36075[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; QQBrowser/7.3.11251.400)";
const char S36076[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; QQBrowser/7.3.11251.400)";
const char S36077[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; QQBrowser/7.3.11251.400)";
const char S36078[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; QQBrowser/7.3.11251.400)";
const char S36079[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; QQBrowser/7.3.11251.400)";
const char S36080[] = "Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.0) Opera 7.02 Bork-edition [en]";
const char S36081[] = "Opera/7.54 (Windows NT 5.0; U) [de]";
const char S36082[] = "Mozilla/5.0 (Windows NT 5.0; U) Opera 7.01 [en]";
const char S36083[] = "Opera/7.01 (Windows NT 5.0; U) [en]";
const char S36084[] = "Mozilla/5.0 (Windows NT 5.0; U) Opera 7.11 [en]";
const char S36085[] = "Opera/7.03 (Windows NT 5.0; U) [en]";
const char S36086[] = "Mozilla/5.0 (Windows NT 5.0; U) Opera 7.21 [en]";
const char S36087[] = "Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.0) Opera 7.01 [en]";
const char S36088[] = "Mozilla/3.0 (Windows NT 5.0; U) Opera 7.10 [de]";
const char S36089[] = "Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.0) Opera 7.03 [en]";
const char S36090[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) Opera 7.54u1";
const char S36091[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) Opera 7.11 [en]";
const char S36092[] = "Opera/7.10 (Windows NT 5.0; U) [en]";
const char S36093[] = "Opera/7.23 (Windows NT 5.0; U)  [ja]";
const char S36094[] = "Opera/7.60 (Windows NT 5.0; U)  [en] (IBM EVV/3.0/EAK01AG9/LE)";
const char S36095[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) Opera 7.22 [en]";
const char S36096[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) Opera 7.20 [en]";
const char S36097[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) Opera 7.21 [ru]";
const char S36098[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) Opera 7.53 [ru]";
const char S36099[] = "Opera/7.53 (Windows NT 5.0; U) [ru]";
const char S36100[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) Opera 7.23 [ru]";
const char S36101[] = "Opera/7.52 (Windows NT 5.0; U)  [en]";
const char S36102[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1";
const char S36103[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1";
const char S36104[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.803.0 Safari/535.1";
const char S36105[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S36106[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S36107[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S36108[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PC36100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36109[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PC36100 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36110[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EVO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36111[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PC36100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36112[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EVO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36113[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; EVO Build/JRO03C)";
const char S36114[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; EVO Build/IMM76D)";
const char S36115[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EVO 4G LTE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36116[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PC36100 Build/Androxide 3.1.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36117[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; EVO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36118[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; EVO Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36119[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PC36100 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36120[] = "SAMSUNG-GT-E2252 Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.29595/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S36121[] = "SAMSUNG GT-E2252 Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.29060/31.1475; U; en) Presto/2.8.119 Version/11.10";
const char S36122[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-L710 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S36123[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S36124[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I747M Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S36125[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530U Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S36126[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S36127[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210K Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S36128[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S36129[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530C Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S36130[] = "Mozilla/5.0 (Linux; Android 4.0.4; Galaxy S III Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S36131[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.11 (KHTML, like Gecko) Ubuntu/12.04 Chromium/20.0.1132.47 Chrome/20.0.1132.47 Safari/536.11";
const char S36132[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.11 (KHTML, like Gecko) Ubuntu/12.04 Chromium/20.0.1132.47 Chrome/20.0.1132.47 Safari/536.11";
const char S36133[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.11 (KHTML, like Gecko) Ubuntu/12.04 Chromium/20.0.1132.47 Chrome/20.0.1132.47 Safari/536.11,gzip(gfe)";
const char S36134[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.11 (KHTML, like Gecko) Ubuntu/12.04 Chromium/20.0.1132.47 Chrome/20.0.1132.47 Safari/536.11,gzip(gfe) (via translate.google.com)";
const char S36135[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.11 (KHTML, like Gecko) Ubuntu/12.04 Chromium/20.0.1132.47 Chrome/20.0.1132.47 Safari/536.11,gzip(gfe)";
const char S36136[] = "Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/536.11 (KHTML, like Gecko) Ubuntu/12.04 Chromium/20.0.1132.47 Chrome/20.0.1132.47 Safari/536.11";
const char S36137[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.11 (KHTML, like Gecko) Ubuntu/12.04 Chromium/20.0.1132.47 Chrome/20.0.1132.47 Safari/536.11,gzip(gfe) (via docs.google.com/viewer)";
const char S36138[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Safari/535.19 Silk-Accelerated=true";
const char S36139[] = "Mozilla/5.0 (Symbian/3; Series60/5.3 NokiaN8-00; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.4.2.6 Mobile Safari/533.4 3gpp-gba";
const char S36140[] = "Mozilla/5.0 (Symbian/3; Series60/5.3 NokiaN8-00/111.040.1511; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.1.4 Mobile Safari/535.1 3gpp-gba";
const char S36141[] = "Opera/9.80 (Windows NT 6.2; U; en) Presto/2.10.289 Version/12.02";
const char S36142[] = "Opera/9.80 (Windows NT 6.2; WOW64; U; MRA 6.0 (build 5970); ru) Presto/2.10.289 Version/12.02";
const char S36143[] = "Opera/9.80 (Windows NT 6.2; WOW64; U; MRA 6.0 (build 5831); ru) Presto/2.10.289 Version/12.02";
const char S36144[] = "Opera/9.80 (Windows NT 6.2; WOW64; U; MRA 6.0 (build 5976); ru) Presto/2.10.289 Version/12.02";
const char S36145[] = "Opera/9.80 (Windows NT 6.2; WOW64; U; MRA 6.0 (build 5998); ru) Presto/2.10.289 Version/12.02";
const char S36146[] = "Opera/9.80 (Windows NT 6.2; WOW64; U; MRA 6.0 (build 6015); ru) Presto/2.10.289 Version/12.02";
const char S36147[] = "Opera/9.80 (Windows NT 6.2; WOW64; U; MRA 6.0 (build 6005); ru) Presto/2.10.289 Version/12.02";
const char S36148[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AT100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S36149[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; AT100 Build/IMM76D)";
const char S36150[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AT100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36151[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; AT100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S36152[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AT100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S36153[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SCH-I500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36154[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SCH-I500 Build/FROYO)";
const char S36155[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; SCH-I500 Build/FROYO)";
const char S36156[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SCH-I500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36157[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SCH-I500 Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S36158[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;SCH-I500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36159[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ko-; SCH-I500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36160[] = "Mozilla/5.0 (Linux; U; Android2.2.1; en-us; SCH-I500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36161[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;SCH-I500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36162[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SCH-I500Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36163[] = "Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; SCH-I500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36164[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SCH-I500Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36165[] = "Mozilla/5.0 (Linux; U;Android 2.2.2; en-us; SCH-I500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36166[] = "Mozilla/5.0 (Linux; U; Android2.2.2; en-us; SCH-I500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36167[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ru-; SCH-I500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36168[] = "ichiro/3.0 (http://help.goo.ne.jp/help/article/1142)";
const char S36169[] = "ichiro/3.0 (http://help.goo.ne.jp/door/crawler.html)";
const char S36170[] = "ichiro/3.0 (http://search.goo.ne.jp/option/use/sub4/sub4-1/)";
const char S36171[] = "Mozilla/5.0 (Nintendo 3DS; U; ; en) Version/1.7455.US";
const char S36172[] = "Mozilla/5.0 (Nintendo 3DS; U; ; en) Version/1.7455.EU";
const char S36173[] = "Mozilla/5.0 (Nintendo 3DS; U; ; en) Version/1.7498.EU";
const char S36174[] = "Mozilla/5.0 (Nintendo 3DS; U; ; en) Version/1.7498.US";
const char S36175[] = "Mozilla/5.0 (Nintendo 3DS; U; ; es) Version/1.7498.US";
const char S36176[] = "Mozilla/5.0 (Nintendo 3DS; U; ; nl) Version/1.7498.EU";
const char S36177[] = "Mozilla/5.0 (Nintendo 3DS; U; ; de) Version/1.7498.EU";
const char S36178[] = "Mozilla/5.0 (Nintendo 3DS; U; ; fr) Version/1.7498.EU";
const char S36179[] = "Mozilla/5.0 (Nintendo 3DS; U; ; en) Version/1.7552.EU";
const char S36180[] = "Mozilla/5.0 (Nintendo 3DS; U; ; en) Version/1.7552.US";
const char S36181[] = "Mozilla/5.0 (Nintendo 3DS; U; ; ja) Version/1.7552.JP";
const char S36182[] = "Mozilla/5.0 (Nintendo 3DS; U; ; nl) Version/1.7552.EU";
const char S36183[] = "Mozilla/5.0 (Nintendo 3DS; U; ; fr) Version/1.7552.EU";
const char S36184[] = "Mozilla/5.0 (Nintendo 3DS; U; ; de) Version/1.7552.EU";
const char S36185[] = "Mozilla/5.0 (Nintendo 3DS; U; ; es) Version/1.7552.US";
const char S36186[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T989 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36187[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T989 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36188[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T989 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36189[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T989 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36190[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T989 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36191[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-T989 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36192[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T989 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S36193[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T989 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36194[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-T989 Build/JZO54K)";
const char S36195[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-T989 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36196[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-T989 Build/TweakWiz V2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36197[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T989 Build/JZO54K)";
const char S36198[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-T989D-parrot Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36199[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-T989D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36200[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-T989 Build/Slim Bean All-Star V5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36201[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T989D Build/JZO54K)";
const char S36202[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SGH-T989 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36203[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:13.0) Gecko/X Firefox/13.0.1";
const char S36204[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:13.0) Gecko/X Firefox/13.0.1 Firefox/13.0.1";
const char S36205[] = "Mozilla/5.0 (Windows NT 6.1; rv:5.0) Gecko/X Firefox/5.02";
const char S36206[] = "Mozilla/5.0 (Windows NT 6.1; rv:5.0) Gecko/X Firefox/5.02011-10-16 20:21:42";
const char S36207[] = ",ozilla/3.0 (Windows NT 6.1; rv:2.0.1) Gecko/X Firefox/5.0.1";
const char S36208[] = "=Mozilla/5.0 (Windows NT 6.1; WOW64; rv:5.0) Gecko/X Firefox/5.0";
const char S36209[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2) Gecko/X Firefox/5.0";
const char S36210[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; HTC-Z710a Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36211[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_Sensation Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36212[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Sensation Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36213[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; pt-pt; HTC/Sensation/1.35.168.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36214[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-ch; HTC/Sensation/1.45.166.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36215[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; HTC/Sensation/1.45.162.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36216[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC/Sensation/1.45.161.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36217[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-ch; HTC/Sensation/1.35.166.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36218[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; HTC_Sensation_Z710a Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36219[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; HTC-Z710a Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36220[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; HTC_Z710a V1.34.841.6 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36221[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-ch; HTC/Sensation/1.35.166.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36222[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC_Sensation_Z710a Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36223[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC/Sensation/1.35.161.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36224[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; HTC/Sensation/1.35.163.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36225[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; HTC/Sensation/1.35.162.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36226[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC/Sensation/1.24.161.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36227[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; HTC/Sensation/1.24.162.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36228[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_Sensation-orange-LS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36229[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC/Sensation/1.27.161.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36230[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; HTC/Sensation/1.27.163.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36231[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; HTC/Sensation/1.27.162.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36232[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ru-ru; HTC/Sensation/1.27.187.5 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36233[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-ch; HTC/Sensation/1.27.166.5 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36234[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC Sensation Z710a Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36235[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC_Z710a V1.34.841.6 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36236[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC/Sensation/1.35.161.2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36237[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; HTC/Sensation/1.35.163.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36238[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC_Sensation-orange-LS Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36239[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; HTC/Sensation/1.17.162.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36240[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; HTC Sensation Z710a Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36241[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Sensation Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36242[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Sensation Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36243[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC_Sensation Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36244[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Sensation Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36245[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; HTC/Sensation/1.45.163.4 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36246[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; HTC_Z710a V1.45.841.5 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36247[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Sensation Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36248[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC/Sensation/1.50.161.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36249[] = "Mozilla/5.0 (Linux; U; Android 2.3; xx-xx; HTC/Sensation/1.17.162.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36250[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC/Sensation/1.45.161.2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36251[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; el-gr; HTC/Sensation/1.45.168.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36252[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; HTC/Sensation/1.45.165.2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36253[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; HTC/Sensation/1.45.162.2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36254[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; HTC/Sensation/1.50.162.2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36255[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Sensation Build/GRJ22)";
const char S36256[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-ch; HTC/Sensation/1.50.166.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36257[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC-Z710a Build/GRI40)";
const char S36258[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; HTC/Sensation/1.45.165.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36259[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; HTC/Sensation/1.50.163.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36260[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-ch; HTC/Sensation/1.45.166.3 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36261[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; HTC/Sensation/1.34.165.6 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36262[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Sensation Z710a Build/GRI40)";
const char S36263[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Sensation Build/GRI40)";
const char S36264[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; el-gr; HTC/Sensation/1.27.168.5 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36265[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC-Z710a Build/GRJ22)";
const char S36266[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; HTC/Sensation/1.45.187.3 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36267[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; Sensation Build/GRJ22)";
const char S36268[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; HTC Sensation kingdroid v11 sense 3.5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36269[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; HTC/Sensation/1.50.165.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36270[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Sensation Z710a Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S36271[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Sensation Z710a Build/GRJ22)";
const char S36272[] = "Mozilla/5.0 (Linux; Android 4.1.1; XT907 Build/9.8.1Q_27-2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S36273[] = "Mozilla/5.0 (Linux; Android 4.1.1; XT907 Build/9.8.1Q_27-2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S36274[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT905 Build/9.8.2Q_SMUIRD-7) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S36275[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT905 Build/9.8.2Q_SMUIRD-7) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S36276[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-66) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S36277[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-66) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S36278[] = "BlackBerry8330/4.5.0.77 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105";
const char S36279[] = "BlackBerry8330m/4.5.0.186 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/104";
const char S36280[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 GLOBAL Build/4.5.1_57_D2G-37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36281[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 GLOBAL Build/4.5.1_57_D2G-36) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36282[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 GLOBAL Build/4.5.1_57_D2G-30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36283[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 GLOBAL Build/4.5.1_57_D2G-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36284[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 GLOBAL Build/4.5.1_57_D2G-38) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36285[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2GLOBAL Build/4.5.1_57_D2G-38) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36286[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID2 GLOBAL Build/S273) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36287[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; DROID2 GLOBAL Build/4.5.1_57_D2G-38)";
const char S36288[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID2 GLOBAL Build/S273) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36289[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID2 GLOBAL Build/4.5.1_57_D2GA-59) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36290[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID2 GLOBALBuild/4.5.1_57_D2GA-59) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36291[] = "Mozilla/5.0 (Linux; U; Android2.3.3; en-us; DROID2 GLOBAL Build/4.5.1_57_D2G-38) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36292[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 GLOBALBuild/4.5.1_57_D2G-38) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36293[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;DROID2 GLOBAL Build/4.5.1_57_D2G-38) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36294[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; DROID2 GLOBAL Build/4.5.1_57_D2GA-59) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36295[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; DROID2 GLOBAL Build/4.5.1_57_D2GA-59)";
const char S36296[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID2GLOBAL Build/4.5.1_57_D2GA-59) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36297[] = "Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; DROID2 GLOBAL Build/4.5.1_57_D2G-38) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36298[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 GLOBAL Build/4.5.1_57_D2G-38) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S36299[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID2 GLOBAL Build/4.5.1_57_D2GA-59) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S36300[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;DROID2 GLOBAL Build/4.5.1_57_D2GA-59) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36301[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; DROID2 GLOBAL Build/4.5.1_57_D2GA-59) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36302[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID2 GLOBAL Build/VZW; liquid.droid2we.X.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36303[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID2 GLOBAL Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36304[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; DROID2 GLOBAL Build/4.5.1_57_D2G-38) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S36305[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; DROID2 GLOBAL Build/4.5.1_57_D2G-38) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S36306[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Moto Droid Build/FRG01B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36307[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Droid Build/FRF57) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36308[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Droid Build/FRFxx) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36309[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Droid Build/FRF84B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36310[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Droid Build/FRG01B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36311[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; droid Build/FRF83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36312[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Droid Build/FRG22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36313[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Droid Build/FRG22D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36314[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Droid Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36315[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Droid Build/FRG22D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36316[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Droid Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36317[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Droid Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36318[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; Droid Build/FRG83G)";
const char S36319[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Droid Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36320[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; Droid Build/FRG83D)";
const char S36321[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Droid Build/FRG83S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36322[] = "Logos/0.9.4 (Linux; U; Android 2.2.3; en_US; Droid Build/FRK76)";
const char S36323[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; Droid Build/FRK76) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36324[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.3; Droid Build/FRK76)";
const char S36325[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Droid Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36326[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; en-us;Droid Build/FRK76) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36327[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; Droid Build/FRK76) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S36328[] = "Mozilla/5.0 (Linux; U; Android2.2.3; en-us; Droid Build/FRK76) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36329[] = "Mozilla/5.0 (Linux; U;Android 2.2.3; en-us; Droid Build/FRK76) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36330[] = "Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; sholes/sourceidnull) AppleWebKit/533.1(KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36331[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; Droid Build/FRG22D)";
const char S36332[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Droid Build/FRG01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36333[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Droid Build/FRG22D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0";
const char S36334[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; ko-; Droid Build/FRK76) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36335[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; Droid Build/FRK76) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S36336[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S36337[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148";
const char S36338[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)";
const char S36339[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C134b";
const char S36340[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/533.17.9";
const char S36341[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 4_2_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad1,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.2.1;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]";
const char S36342[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 iPad1,1/K48AP Zite/1.3.3";
const char S36343[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/528.16";
const char S36344[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; ru-ru) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C134";
const char S36345[] = "MQQBrowser/1.9.1 Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/7534.48.3";
const char S36346[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/7B367";
const char S36347[] = "MQQBrowser/18 Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/7534.48.3";
const char S36348[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 4_2 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad1,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.2;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]";
const char S36349[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36350[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; DROID4 Build/9.8.2O-72_VZW-18)";
const char S36351[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S36352[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID4 Build/9.8.2O-72_VZW-18-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36353[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36354[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Prism Build/HuaweiU8651) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36355[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U8651T Build/HuaweiU8651) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36356[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; Prism Build/HuaweiU8651)";
const char S36357[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; PrismBuild/HuaweiU8651) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36358[] = "Mozilla/5.0 (Linux; U; Android2.3.6; en-us; Prism Build/HuaweiU8651) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36359[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Prism Build/HuaweiU8651) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S36360[] = "Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; Prism Build/HuaweiU8651) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36361[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us;Prism Build/HuaweiU8651) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36362[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; U8651T Build/HuaweiU8651)";
const char S36363[] = "Google-Site-Verification/1.0";
const char S36364[] = "Mozilla 5.0 (compatible; Google-Site-Verification/1.0; +http://www.google.com/webmasters/verification/)";
const char S36365[] = "Mozilla/5.0 (compatible; Google-Site-Verification/1.0; +http://www.google.com/webmasters/verification/)";
const char S36366[] = "Mozilla/5.0 (compatible; Google-Site-Verification/1.0)";
const char S36367[] = "Yeti/1.0 (NHN Corp.; http://help.naver.com/robots/)";
const char S36368[] = "Yeti/1.0 (NHN Corp.; http://help.naver.com/robots/),gzip(gfe) (via translate.google.com)";
const char S36369[] = "Yeti/1.1 (NHN Corp.; http://help.naver.com/robots/)";
const char S36370[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.74 Safari/537.36";
const char S36371[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S36372[] = "Mozilla/5.0 (Linux; Android 4.2; Nexus 4 Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S36373[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 4 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S36374[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Maxthon/3.0)";
const char S36375[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Maxthon/3.0)";
const char S36376[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Maxthon/3.0)";
const char S36377[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Maxthon/3.0)";
const char S36378[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Maxthon/3.0)";
const char S36379[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Maxthon/3.0)";
const char S36380[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Maxthon/3.0)";
const char S36381[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Maxthon/3.0)";
const char S36382[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Avant Browser; Maxthon/3.0)";
const char S36383[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Avant Browser; Maxthon/3.0)";
const char S36384[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Maxthon/3.0)";
const char S36385[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Maxthon/3.0)";
const char S36386[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; Maxthon/3.0)";
const char S36387[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Maxthon/3.0)";
const char S36388[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Avant Browser; Maxthon/3.0)";
const char S36389[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.1; Tablet PC 2.0; .NET4.0C; .NET4.0E; Maxthon/3.0)";
const char S36390[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; MB860 Build/OLYLA_U4_0.37.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36391[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.4.1.16.MB860.ATT.en.US Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36392[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; MB860 Build/OLSKT_U4_1.14.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36393[] = "MOT-MB860/ Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; MB860 Build/OLYLA_U4_0.50.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36394[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB860 Build/Blur_Version.4.1.31.MB860.Orange.en.GB Flex/P025) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36395[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/Blur_Version.0.0.1.MB860.ATT.en.US Flex/P013) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36396[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; MB860 Build/Blur_Version.4.0.240.MB860.Telstra.en.AU Flex/P003) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36397[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; MB860 Build/Blur_Version.0.37.4.MB860.BellMobility.en.CA Flex/P025) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36398[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/OLYFR_U4_1.5.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36399[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/OLYFR_U4_1.8.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36400[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/Blur_Version.4.1.83.MB860.ATT.en.US Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36401[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.4.1.52.MB860.ATT.en.US Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36402[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.4.1.18.MB860.ATT.en.US Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36403[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.4.1.57.MB860.ATT.en.US Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36404[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/OLYFR_U4_1.5.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36405[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.4.0.1960.MB860.ATT.en.US Flex/P015) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36406[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.4.1.23.MB860.ATT.en.US Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36407[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.0.0.1.MB860.ATT.en.US Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36408[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/OLYFR_U4_1.2.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36409[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.4.0.19915.MB860.ATT.en.US Flex/P009) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36410[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/Blur_Version.4.1.75.MB860.ATT.en.US Flex/P013) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36411[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/Blur_Version.4.1.83.MB860.ATT.en.US Flex/P013) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36412[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.0.0.1.MB860.ATT.en.US) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36413[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.4.1.26.MB860.ATT.en.US Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36414[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; MB860 Build/Blur_Version.0.37.4.MB860.BellMobility.en.CA Flex/P024) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36415[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-au; MB860 Build/OLYEM_U4_1.4.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36416[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/Blur_Version.4.0.530.MB860.Retail.en.GB Flex/P022) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36417[] = "MOT-MB860/ Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; MB860 Build/OLYLA_U4_0.54.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36418[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-sg; MB860 Build/OLYEM_U4_3.1.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36419[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/Blur_Version.4.3.10.MB860.MERetail.en.06 Flex/P008) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36420[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; MB860 Build/OLYEM_U4_2.1.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36421[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB860 Build/OLYEM_U4_4.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36422[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-sg; MB860 Build/Blur_Version.4.3.13.MB860.AsiaRetail.en.03 Flex/P021) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36423[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; cs-; MB860 Build/Blur_Version.4.4.20.MB860.Retail.en.DE Flex/P023) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36424[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; he-il; MB860 Build/Blur_Version.4.4.80.MB860.Partner.en.IL Flex/P013) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36425[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB860 Build/Blur_Version.4.4.35.MB860.Retail.en.GB Flex/P026) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36426[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; MB860 Build/Blur_Version.4.2.13.MB860.Retail.en.FR Flex/P016) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36427[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; MB860 Build/Blur_Version.4.4.20.MB860.Retail.en.DE Flex/P023) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36428[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-sg; MB860 Build/Blur_Version.4.3.12.MB860.Starhub.en.SG Flex/P007) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36429[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; MB860 Build/Blur_Version.4.4.23.MB860.Vodafone.en.EU Flex/P016) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36430[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/OLYEM_U4_3.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36431[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; MB860 Build/OLYLA_U4_0.37.23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36432[] = "MOT-MB860/ Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; MB860 Build/OLYLA_U4_0.50.0_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36433[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; MB860 Build/Blur_Version.0.37.23.MB860.BellMobility.en.CA Flex/P026) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36434[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-au; MB860 Build/Blur_Version.4.1.42.MB860.Telstra.en.AU Flex/P029) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36435[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; MB860 Build/Blur_Version.4.1.200.MB860.SKT.en.KR Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36436[] = "MOT-MB860/ Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; MB860 Build/OLYLA_U4_0.50.0_R03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36437[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.4.1.120.ME860.ChinaRetail.en.CN Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36438[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; MB860 Build/OLSKT_U4_1.20.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36439[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-au; MB860 Build/OLYEM_U4_1.4.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36440[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; MB860 Build/OLYEM_U4_1.3.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36441[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-au; MB860 Build/Blur_Version.4.1.46.MB860.Telstra.en.AU Flex/P032) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36442[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/Blur_Version.4.1.140.MB860.SKT.en.KR Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36443[] = "MOT-MB860/ Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; MB860 Build/OLYLA_U4_0.43.0_R03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36444[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; MB860 Build/Blur_Version.4.2.11.MB860.Retail - Blurred. Flex/P014) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36445[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/Blur_Version.4.1.84.MB860.ATT.en.US Flex/P014) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36446[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB860 Build/OLYEM_U4_1.3.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36447[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; MB860 Build/Blur_Version.4.1.33.MB860.Orange.en.FR Flex/P034) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36448[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/Blur_Version.4.1.170.MB860.SKT.en.KR Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36449[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.4.1.57.MB860.ATT.en.US Flex/P013) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36450[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-au; MB860 Build/Blur_Version.4.1.47.MB860.Telstra.en.AU Flex/P033) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36451[] = "MOT-MB860/ Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; MB860 Build/OLYLA_U4_0.54.0_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36452[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-sg; MB860 Build/Blur_Version.4.3.10.MB860.AsiaRetail.en.03 Flex/P020) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36453[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB860 Build/Blur_Version.4.2.23.MB860.T-Mobile.en.GB Flex/P021) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36454[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB860 Build/OLYEM_U4_2.2.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36455[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB860 Build/OLYEM_U4_4.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36456[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB860 Build/Blur_Version.4.4.20.MB860.Orange.en.GB Flex/P035) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36457[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; MB860 Build/Blur_Version.4.4.20.MB860.Orange.en.FR Flex/P038) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36458[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-sg; MB860 Build/OLYEM_U4_3.1.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36459[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; MB860 Build/Blur_Version.4.4.20.MB860.O2.en.DE Flex/P020) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36460[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/Blur_Version.4.2.11.MB860.Retail.en.FR Flex/P014) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36461[] = "MOT-MB860/ Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; MB860 Build/OLYLA_U4_0.54.0_R05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36462[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB860 Build/Blur_Version.4.1.120.ME860.ChinaRetail.en.CN Flex/P014) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36463[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; MB860 Build/Blur_Version.4.4.20.MB860.TIM.en.IT Flex/P017) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36464[] = "MOT-MB860/ Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/OLYLA_U4_0.56.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36465[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB860 Build/Blur_Version.4.4.80.MB860.MERetail.en.06 Flex/P013) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36466[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; MB860 Build/Blur_Version.4.0.300.MB860.Orange.en.GB Flex/P009) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36467[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; MB860 Build/OLYFR_U4_1.8.3)";
const char S36468[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB860 Build/Blur_Version.4.4.35.MB860.Retail.en.GB Flex/P023) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36469[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; MB860 Build/OLYFR_U4_1.2.6)";
const char S36470[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB860 Build/OLYEM_U4_2.1.1) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S36471[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; he-il; MB860 Build/OLYEM_U4_4.8.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36472[] = "MOT-MB860/ Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; MB860 Build/OLYLA_U4_0.54.0_R06) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36473[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB860 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36474[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB860 Build/OLYEM_U4_2.1.1) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S36475[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB860 Build/OLYEM_U4_2.1.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36476[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB860 Build/Blur_Version.4.4.33.MB860.Retail.en.ES Flex/P029) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36477[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB860 Build/OLYEM_U4_3.1.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S36478[] = "SAMSUNG-GT-C3312 Opera/9.80 (J2ME/MIDP; Opera Mini/6.5.29390/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S36479[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S36480[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTCONE Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S36481[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC_PN071 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S36482[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One 801e Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S36483[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One dual sim Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S36484[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Sony Tablet S Build/THMAS11000) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S36485[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; Sony Tablet S Build/THMASS11004) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S36486[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; Sony Tablet S Build/THMAS0042)";
const char S36487[] = "Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; Sony Tablet S Build/THMAS11002) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S36488[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; Sony Tablet S Build/THMAS11000)";
const char S36489[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Sony Tablet S Build/THMASU0035) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S36490[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; es-es; Sony Tablet S Build/THMAS0050) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S36491[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Sony Tablet S Build/THMAS0042) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S36492[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; Sony Tablet S Build/THMAS11001) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S36493[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; Sony Tablet S Build/THMA3G11005) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S36494[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; de-de; Sony Tablet S Build/THMAST0066) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S36495[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; Sony Tablet S Build/THMD01900) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S36496[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; zh-cn; Sony Tablet S Build/THMAS0030) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S36497[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; tr-tr; Sony Tablet S Build/THMASU0032) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S36498[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/537.9 (KHTML, like Gecko) Version/6.1 Safari/537.9";
const char S36499[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/537.12 (KHTML, like Gecko) Version/6.1 Safari/537.12.2";
const char S36500[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/537.15.1 (KHTML, like Gecko) Version/6.1 Safari/537.15.1";
const char S36501[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.11 (KHTML, like Gecko) Version/6.1 Safari/537.11";
const char S36502[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/537.17 (KHTML, like Gecko) Version/6.1 Safari/537.17";
const char S36503[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/537.18.1 (KHTML, like Gecko) Version/6.1 Safari/537.18.1";
const char S36504[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.7 (KHTML, like Gecko) Version/6.1 Safari/537.7";
const char S36505[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/537.20.3 (KHTML, like Gecko) Version/6.1 Safari/537.20.3";
const char S36506[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/537.22 (KHTML, like Gecko) Version/6.1 Safari/537.22";
const char S36507[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/537.22 (KHTML, like Gecko) Version/6.1 Safari/537.23";
const char S36508[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.28.1 (KHTML, like Gecko) Version/6.1 Safari/537.28.1";
const char S36509[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.41 (KHTML, like Gecko) Version/6.1 Safari/537.41";
const char S36510[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.43.58 (KHTML, like Gecko) Version/6.1 Safari/537.43.58";
const char S36511[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.31.2 (KHTML, like Gecko) Version/6.1 Safari/537.31.2";
const char S36512[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.46.3 (KHTML, like Gecko) Version/6.1 Safari/537.46.3";
const char S36513[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.48.2 (KHTML, like Gecko) Version/6.1 Safari/537.48.2";
const char S36514[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.51.3 (KHTML, like Gecko) Version/6.1 Safari/537.51.3";
const char S36515[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.59 (KHTML, like Gecko) Version/6.1 Safari/537.59";
const char S36516[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.54.1 (KHTML, like Gecko) Version/6.1 Safari/537.54.1";
const char S36517[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.66 (KHTML, like Gecko) Version/6.1 Safari/537.66";
const char S36518[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.71 (KHTML, like Gecko) Version/6.1 Safari/537.71";
const char S36519[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8G4 Safari/7534.48.3";
const char S36520[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8J2 Safari/7534.48.3";
const char S36521[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8L1 Safari/7534.48.3";
const char S36522[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8J3 Safari/7534.48.3";
const char S36523[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8H7 Safari/7534.48.3";
const char S36524[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8F190 Safari/7534.48.3";
const char S36525[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8F191 Safari/7534.48.3";
const char S36526[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8K2 Safari/7534.48.3";
const char S36527[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8H8 Safari/7534.48.3";
const char S36528[] = "NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax3-02";
const char S36529[] = "NokiaX3-02/5.0 (05.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 NokiaX3-02/05.60";
const char S36530[] = "NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S36531[] = "NokiaX3-02.5/5.0 (06.75) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S36532[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/19.0.1077.3) AppleWebKit/536.4 (KHTML, like Gecko) Chrome/19.0.1077.3 Safari/536.4";
const char S36533[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/19.0.1084.30) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.30 Safari/536.5";
const char S36534[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/19.0.1084.46) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5";
const char S36535[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/19.0.1084.46) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5";
const char S36536[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/19.0.1084.52) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5";
const char S36537[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/19.0.1084.52) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5";
const char S36538[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/19.0.1084.52; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S36539[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/19.0.1084.52; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S36540[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/19.0.1084.52; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S36541[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/19.0.1084.52; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S36542[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/19.0.1084.56)";
const char S36543[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/19.0.1084.56; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8)";
const char S36544[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/19.0.1084.56) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S36545[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/19.0.1084.56) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S36546[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/19.0.1084.56)";
const char S36547[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/19.0.1084.52; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)";
const char S36548[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/19.0.1084.52; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S36549[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/19.0.1084.56; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8)";
const char S36550[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/19.0.1084.52; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MS-RTC LM 8)";
const char S36551[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/19.0.1084.56; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S36552[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/19.0.1084.56)";
const char S36553[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A523 Safari/8536.25";
const char S36554[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A405 Safari/8536.25";
const char S36555[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A551 Safari/8536.25";
const char S36556[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A403 Safari/8536.25";
const char S36557[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A525 Safari/8536.25";
const char S36558[] = "Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S36559[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S36560[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S36561[] = "Mozilla/5.0 (Linux; Android 4.2; Galaxy Nexus Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S36562[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1";
const char S36563[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1";
const char S36564[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1,gzip(gfe)";
const char S36565[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17a Build/4.0.1.A.0.32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36566[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.1.A.0.233) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36567[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.1.A.0.215) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36568[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.1.A.0.204) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36569[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.1.A.0.247) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36570[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.1.A.0.263) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36571[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.1.A.0.271) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36572[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.1.A.0.284) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36573[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonST17i Build/4.0.1.A.2.24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36574[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.1.A.0.57) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36575[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.2.A.0.19) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36576[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.2.A.0.35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36577[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.2.A.0.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36578[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-nz; SonyEricssonST17a Build/4.0.1.A.0.284) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36579[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; ST17i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36580[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-nz; ST17a Build/4.0.1.A.0.284) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36581[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36582[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; nb-no; SonyEricssonST17i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36583[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonST17i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36584[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; ST17i Build/4.0.2.A.0.62)";
const char S36585[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-za; SonyEricssonST17iv Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36586[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-cl; SonyEricssonST17a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36587[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-nz; SonyEricssonST17a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36588[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; ST17i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36589[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36590[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ST17i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36591[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ST17i Build/4.0.1.A.0.284) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36592[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ST17a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36593[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonST17i Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36594[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonST17a Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36595[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; ST17i Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36596[] = "Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko/X Firefox/11.0 Firefox/11.0";
const char S36597[] = "Mozilla/5.0 (Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko/X Firefox/11.0";
const char S36598[] = "Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko Firefox/11.0";
const char S36599[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; rv:11.0) Gecko/X Firefox/11.0";
const char S36600[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2) Gecko/X Firefox/11.0";
const char S36601[] = "Mozilla/5.0 (Windows NT 5.2; rv:11.0) Gecko/11.0 Firefox/11.0";
const char S36602[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A403 Safari/8536.25";
const char S36603[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A523 Safari/8536.25";
const char S36604[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A550 Safari/8536.25";
const char S36605[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A8426 Safari/8536.25";
const char S36606[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A407 Safari/8536.25";
const char S36607[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A8500 Safari/8536.25";
const char S36608[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A406 Safari/8536.25";
const char S36609[] = "Mozilla/5.0 (X11; Linux i686; rv:11.0) Gecko/X Firefox/11.0 Iceweasel/11.0";
const char S36610[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.16) Gecko/X Gecko Firefox/11.0";
const char S36611[] = "Mozilla/5.0 (X11; Linux i686; rv:11.0) Gecko/X Firefox/11.0 Firefox/11.0";
const char S36612[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Safari/535.19 Silk-Accelerated=true";
const char S36613[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; 5860E Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36614[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; 5860e Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36615[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; 5860E Build/GWK74)";
const char S36616[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; 5860E Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Android MetroPCS androidAppVersion/2.5.10 AndroidWebkitWrapper BillingSupported";
const char S36617[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-; 5860e Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36618[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; 5860E Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 androidAppVersion/2.6.8 AndroidWebkitWrapper";
const char S36619[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36620[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S36621[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SAMSUNG-SGH-I997Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36622[] = "Logos/0.9.0 (Linux; U; Android 2.2.1; en_US; SAMSUNG-SGH-I997 Build/FROYO.UCKE3)";
const char S36623[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36624[] = "Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36625[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SAMSUNG-SGH-I997 Build/FROYO)";
const char S36626[] = "Logos/0.9.4 (Linux; U; Android 2.2.1; en_US; SAMSUNG-SGH-I997 Build/FROYO.UCKH1)";
const char S36627[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SAMSUNG-SGH-I997 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S36628[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36629[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36630[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S36631[] = "Mozilla/5.0 (Linux; U; Android2.2.1; en-us; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36632[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC)";
const char S36633[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; A620b)";
const char S36634[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; A620e)";
const char S36635[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; Orange)";
const char S36636[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; A620m)";
const char S36637[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 1.04.163.03)";
const char S36638[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 1.04.161.01)";
const char S36639[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 1.04.166.01)";
const char S36640[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 1.04.165.01)";
const char S36641[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 1.04.169.01)";
const char S36642[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 1.04.162.01)";
const char S36643[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 1.04.172.01)";
const char S36644[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 2.00.161.01)";
const char S36645[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 1.04.183.01)";
const char S36646[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 2.00.165.01)";
const char S36647[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 2.00.166.01)";
const char S36648[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 2.00.163.02)";
const char S36649[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 2.00.172.01)";
const char S36650[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 2.00.169.01)";
const char S36651[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 2.00.162.01)";
const char S36652[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; A620t; CMCC)";
const char S36653[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 2.00.162.02)";
const char S36654[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 2.00.172.02)";
const char S36655[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 2.00.183.01)";
const char S36656[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; A620d; ChinaTelecom)";
const char S36657[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 3.29.163.01)";
const char S36658[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 3.29.161.01)";
const char S36659[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 3.29.166.01)";
const char S36660[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 3.29.162.01)";
const char S36661[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 3.29.172.01)";
const char S36662[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 3.29.169.01)";
const char S36663[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 3.29.165.01)";
const char S36664[] = "Mozilla/5.0 (Series40; NokiaX2-02/11.79; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S36665[] = "Mozilla/5.0 (X11; Linux i686 on x86_64; rv:16.0) Gecko/X Firefox/16.0";
const char S36666[] = "Mozilla/5.0 (X11; Linux x86_64; rv:16.0) Gecko/X Firefox/16.0 Iceweasel/16.0.2";
const char S36667[] = "Mozilla/5.0 (X11; Linux i686; rv:16.0) Gecko/X Abrowser/16.0";
const char S36668[] = "Mozilla/5.0 (X11; Linux x86_64; rv:16.0) Gecko/X Abrowser/16.0";
const char S36669[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-ca; SonyEricssonLT15a Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36670[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonLT15a Build/4.0.A.2.286) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36671[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; SonyEricssonLT15a Build/3.0.A.2.118) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36672[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; es-cl; SonyEricssonLT15a Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36673[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; SonyEricssonLT15a Build/3.0.A.0.326) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36674[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; LT15a Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36675[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; SonyEricssonLT15a Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36676[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-mx; SonyEricssonLT15a Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36677[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; SonyEricssonLT15av Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36678[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; LT15a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36679[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonLT15a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36680[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; LT15a Build/3.0.1.A.0.145)";
const char S36681[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; SonyEricssonLT15a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36682[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; SonyEricssonLT15a Build/4.0.2.D.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36683[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; SonyEricssonLT15a Build/4.0.2.D.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36684[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; SonyEricssonLT15av Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36685[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LT15a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36686[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LT15a Build/4.0.2.A.0.42)";
const char S36687[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; SonyEricssonLT15av Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36688[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; iw-il; LT15a Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36689[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; LT15a Build/4.0.2.D.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36690[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-cl; SonyEricssonLT15a Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36691[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; es-cl; LT15a Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36692[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; LT15a Build/4.0.2.D.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36693[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; es-co; LT15a Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36694[] = "Mozilla/5.0 (Windows NT 5.1; rv:9.0.1) Gecko/X Firefox/9.0.1 Firefox/9.0.1";
const char S36695[] = "Mozilla/5.0 (Windows NT 5.2; rv:9.0) Gecko/X Firefox/9.0";
const char S36696[] = "Mozilla/5.0 (Windows; Windows NT 5.1; rv:9.0.1) Gecko/X Firefox/9.0.1";
const char S36697[] = "Mozilla/5.0 (Windows NT 5.1) Gecko/X Firefox/9.0.1";
const char S36698[] = "Mozilla/5.0 (Windows NT 5.1; rv:9.0.1) Gecko/X Firefox/9.0.1,gzip(gfe)";
const char S36699[] = "Opera/9.80 (X11; Linux zbov) Presto/2.11.355 Version/12.10";
const char S36700[] = "Opera/9.80 (X11; Linux zbov) Presto/2.11.355 Version/12.10,gzip(gfe)";
const char S36701[] = "Mozilla/5.0 (compatible; MSIE 9.0; X11; Linux i686) Opera 12.10";
const char S36702[] = "Mozilla/5.0 (X11; Linux x86_64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.10";
const char S36703[] = "Opera/9.80 (Windows NT 6.2) Presto/2.12.388 Version/12.12";
const char S36704[] = "Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.12";
const char S36705[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.12";
const char S36706[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.12";
const char S36707[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.12";
const char S36708[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.12";
const char S36709[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.12";
const char S36710[] = "Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.12,gzip(gfe)";
const char S36711[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.12";
const char S36712[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.12";
const char S36713[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.12";
const char S36714[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64) Opera 12.12";
const char S36715[] = "Opera/9.80 (Windows NT 6.2) Presto/2.12.388 Version/12.12,gzip(gfe)";
const char S36716[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.12";
const char S36717[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.12";
const char S36718[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.12";
const char S36719[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.12";
const char S36720[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.12";
const char S36721[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.12";
const char S36722[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.12";
const char S36723[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.12";
const char S36724[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.12";
const char S36725[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; C6603 Build/10.3.A.0.423) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36726[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SonyC6603 Build/10.3.A.0.423) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36727[] = "SonyC6603 Build/10.3.A.0.423 stagefright/1.2 (Linux;Android 4.2.2)";
const char S36728[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; C6603 Build/10.3.1.A.0.244)";
const char S36729[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SonyC6603 Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36730[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; C6603 Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36731[] = "SonyC6603 Build/10.3.1.A.0.244 stagefright/1.2 (Linux;Android 4.2.2)";
const char S36732[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; C6603 Build/10.3.1.A.1.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36733[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-; C6603 Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36734[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; C6603 Build/10.3.1.A.2.67) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36735[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SonyC6603 Build/10.3.1.A.2.67) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36736[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MZ601 Build/I.7.1-23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S36737[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; MZ601 Build/I.7.1-42) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S36738[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; MZ601 Build/ITL41F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S36739[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MZ601 Build/I.7.1-45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S36740[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MZ601 Build/I.7.1-45)";
const char S36741[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MZ601 Build/I.7.1-21) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S36742[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MZ601 Build/I.7.1-17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S36743[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MZ601 Build/I.7.1-34) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S36744[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S36745[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S36746[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S36747[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S36748[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S36749[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S36750[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7; en-us) AppleWebKit/534.20.8 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S36751[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_2; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S36752[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_3; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S36753[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5 Safari/533.16";
const char S36754[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S36755[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S36756[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko)";
const char S36757[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A293";
const char S36758[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A400";
const char S36759[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A306";
const char S36760[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko)";
const char S36761[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko)";
const char S36762[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/7D11";
const char S36763[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A400 Safari/532.9";
const char S36764[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A306 Safari/532.9";
const char S36765[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_2 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.0.2;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S36766[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.0.1;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S36767[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A293 Safari/532.9";
const char S36768[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone2,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.0;FBSS/1; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/1.0]";
const char S36769[] = "OSSProxy 1.3.333.307 (Build 333.307 Win32 en-us)(Feb 22 2012 18:20:37)";
const char S36770[] = "OSSProxy 1.3.326.329 (Build 326.329 Win32 en-us)(Apr 15 2010 15:40:21)";
const char S36771[] = "OSSProxy 1.3.333.310 (Build 333.310 Win32 en-us)(May 24 2012 17:45:09)";
const char S36772[] = "OSSProxy 1.3.334.331 (Build 334.331 Win32 en-us)(Aug 31 2012 15:09:13)";
const char S36773[] = "OSSProxy 1.3.335.312 (Build 335.312 Win32 en-us)(Oct 25 2012 23:16:24)";
const char S36774[] = "OSSProxy 1.3.335.325 (Build 335.325 Win32 en-us)(Dec 20 2012 17:03:03)";
const char S36775[] = "OSSProxy 1.3.335.327 (Build 335.327 Win32 en-us)(Mar  6 2013 12:10:47)";
const char S36776[] = "OSSProxy 1.3.335.329 (Build 335.329 Win32 en-us)(Mar 28 2013 11:13:41)";
const char S36777[] = "OSSProxy 1.3.335.337 (Build 335.337 Win32 en-us)(Apr  4 2013 16:51:42)";
const char S36778[] = "OSSProxy 1.3.336.313 (Build 336.313 Win32 en-us)(Jun 12 2013 18:51:39)";
const char S36779[] = "OSSProxy 1.3.336.314 (Build 336.314 Win32 en-us)(Jul 11 2013 11:28:10)";
const char S36780[] = "OSSProxy 1.3.335.337 (Build 335.337 Win32 en-us)(Apr 4 2013 16:51:42)";
const char S36781[] = "OSSProxy 1.3.336.317 (Build 336.317 Win32 en-us)(Jul 26 2013 13:44:52)";
const char S36782[] = "OSSProxy 1.3.336.318 (Build 336.318 Win32 en-us)(Aug  6 2013 18:31:22)";
const char S36783[] = "OSSProxy 1.3.336.320 (Build 336.320 Win32 en-us)(Aug 16 2013 17:38:43)";
const char S36784[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGPT12 Build/TJS0166) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S36785[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGPT13 Build/TJS0166) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S36786[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SGPT13 Build/TJS0166)";
const char S36787[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SGPT12 Build/TJS0166)";
const char S36788[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGPT12 Build/TJDS0170) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S36789[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGPT13 Build/TJDS0170) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S36790[] = "Java/1.7.0_21";
const char S36791[] = "Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.10";
const char S36792[] = "Opera/9.80 (Windows NT 6.0; WOW64) Presto/2.12.388 Version/12.10";
const char S36793[] = "Opera/9.80 (Windows NT 6.0; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.10";
const char S36794[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.10";
const char S36795[] = "Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.10";
const char S36796[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.10";
const char S36797[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.10";
const char S36798[] = "Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.10,gzip(gfe)";
const char S36799[] = "Opera/9.80 (Windows NT 6.1; WOW64; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)) Presto/2.12.388 Version/12.10";
const char S36800[] = "Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.10";
const char S36801[] = "Mozilla/5.0 (Windows NT 6.0; rv:2.0) Gecko/X Firefox/4.0 Opera 12.10";
const char S36802[] = "Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.10";
const char S36803[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.10";
const char S36804[] = "Opera/9.80 (Windows NT 6.0; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.10";
const char S36805[] = "Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.10";
const char S36806[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0) Opera 12.10";
const char S36807[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.10";
const char S36808[] = "Opera/9.80 (Windows NT 6.0; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.10";
const char S36809[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.10";
const char S36810[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT925 Build/7.7.1Q-144_VQL_S3-49.2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S36811[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR HD Build/7.7.1Q-144_VQW_S33425-2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S36812[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; SHV-E110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36813[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; SGH-I727R Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36814[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SGH-I727R Build/GINGERBREAD)";
const char S36815[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; SGH-I727R Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30";
const char S36816[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHV-E110S Build/GINGERBREAD)";
const char S36817[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHV-E110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36818[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9210 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36819[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SHV-E110S Build/GINGERBREAD)";
const char S36820[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; SGH-I727R Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36821[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9210 Build/GINGERBREAD)";
const char S36822[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-I9210/I9210XXLC2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36823[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-I9210/I9210XXLD1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36824[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHV-E110S Build/GINGERBREAD)";
const char S36825[] = "Opera/9.80 (Android; Opera Mini/15.0.1162/30.3697; U; en) Presto/2.8.119 Version/11.10";
const char S36826[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ADR8995 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36827[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ADR8995 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36828[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; ADR8995 Build/GRI40)";
const char S36829[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ADR8995 4G Build/Broken Out) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36830[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ADR8995 Build/Broken Out) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36831[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-; ADR8995 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36832[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525Build/JORDN_U3_6.36.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36833[] = "Mozilla/5.0 (Linux; U;Android 2.1-update1; en-us; MB525 Build/JORDN_U3_6.36.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36834[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JORDN_U3_6.36.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 MobileSafari/530.17";
const char S36835[] = "Mozilla/5.0 (Linux; U; Android2.1-update1; en-us; MB525 Build/JORDN_U3_6.36.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36836[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JDSKT_U3_02.18.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36837[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JORDN_U3_6.35.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36838[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JORDN_U3_6.17.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36839[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us;MB525 Build/JORDN_U3_6.36.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36840[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-kr; MB525 Build/JDSKT_U3_02.20.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36841[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; MB525 Build/JRDNEM_U3_2.51.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36842[] = "MOT-MB525/Blur_Version.6.36.1.MB525.TIM.en.BR Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; MB525 Build/JORLA_U3_6.36.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36843[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-; MB525 Build/JRDNEM_U3_2.51.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36844[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JRDNEM_U3_2.34.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36845[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-sg; MB525 Build/JRDNEM_U3_2.49.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36846[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JRDNEM_U3_2.54.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36847[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JORDN_U3_6.34.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36848[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB525 Build/JRDNEM_U3_2.21.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36849[] = "MOT-MB525/Blur_Version.6.14.5.MB525.AmericaMovil.en.BR Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; MB525 Build/JORLA_U3_6.14.5_R01) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36850[] = "MOT-MB525/Blur_Version.6.14.7.MB525.Personal.en.AR Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; MB525 Build/JORLA_U3_6.14.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36851[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; MB525 Build/JRDNEM_U3_2.24.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36852[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB525 Build/JRDNEM_U3_2.22.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36853[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JRDNEM_U3_2.59.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36854[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JORDN_U3_6.36.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36855[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JORDN_U3_6.19.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36856[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; MB525 Build/JRDNEM_U3_2.26.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36857[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JORDN_U3_6.29.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36858[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JORDN_U3_6.28.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36859[] = "MOT-MB525/Blur_Version.6.36.1.MB525.AmericaMovil.en.MX Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; MB525 Build/JORLA_U3_6.36.1_R01) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36860[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-za; MB525 Build/JRDNEM_U3_2.51.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36861[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB525 Build/JRDNEM_U3_2.51.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36862[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB525 Build/JRDNEM_U3_2.51.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36863[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB525 Build/JRDNEM_U3_2.21.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36864[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JORDN_U3_6.11.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36865[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; MB525 Build/JRDNEM_U3_2.59.0)";
const char S36866[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; MB525 Build/JRDNEM_U3_2.51.1) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S36867[] = "Mozilla/5.0 (Linux; U; Android2.2; en-us; SGH-T959 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36868[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T959 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S36869[] = "Mozilla/5.0 (Linux; U; Android 2.2.Awesome.ButNot.1; en-us; SGH-T959 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36870[] = "Mozilla/5.0 (Linux; U; Android 2.2; ru-; SGH-T959 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36871[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; SGH-T959 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36872[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T959 Build/SKRILLER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36873[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; SGH-T959P Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36874[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; GT-T959 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36875[] = "Mozilla/5.0 (Linux; U; Android 2.2; en_us; SGH-T959 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.2 Mobile (Dragonfruit)";
const char S36876[] = "Mozilla/5.0 (Linux; U; Android 2.2 - UVKB5; en-us; SGH-T959 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36877[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us;SGH-T959 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36878[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T959 Build/TOXICFROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36879[] = "Mozilla/5.0 (Linux; U;Android 2.2; en-us; SGH-T959 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36880[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T959Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36881[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-T959 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36882[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T959 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36883[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-ca; SGH-T959D Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36884[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; SGH-T959 Build/FROYO)";
const char S36885[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T959 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36886[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; SGH-T959D Build/FROYO)";
const char S36887[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SGH-T959P Build/FROYO)";
const char S36888[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SGH-T959 Build/FROYO)";
const char S36889[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; SGH-T959 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S36890[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SGH-T959 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S36891[] = "MSIE 7.0";
const char S36892[] = "Mozilla/4.0 (compatible; MSIE 7.0)";
const char S36893[] = "mAgent MSIE 7";
const char S36894[] = "Mozilla/6.0 (compatible; MSIE 7.01; Mac_PowerPC)";
const char S36895[] = "Fitel5Browser(compatible; MSIE 7.0;)";
const char S36896[] = "Mozilla/5.0 (compatible; MSIE 7.0 +http://internetmemory.org/en/); crawling on behalf of The National Archives UK Government Web Archive: http://www.nationalarchives.gov.uk/webarchive/; email: webarchive@nationalarchives.gsi.gov.uk";
const char S36897[] = "CitiStreet.com/1.0+(compatible;Mozilla/5.0;MSIE 7.0;++http://citistreet.com/)";
const char S36898[] = "Mozilla/5.0 (compatible; MSIE 7.0; PPC; Trident/5.0)";
const char S36899[] = "Mozilla/5.0 (compatible; MSIE 7.0; PPC; SV1; Trident/1.0)";
const char S36900[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Vortex Build/FRF91) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S36901[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Vortex Build/FRG83) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S36902[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Vortex Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36903[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Vortex Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36904[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Vortex Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36905[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; Vortex Build/FRG83G)";
const char S36906[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; Vortex Build/FRF91)";
const char S36907[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; Vortex Build/FRG83)";
const char S36908[] = "Logos/0.9.4 (Linux; U; Android 2.2.2; en_US; Vortex Build/FRG83G)";
const char S36909[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;Vortex Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36910[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Vortex Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S36911[] = "Mozilla/5.0 (Linux; U;Android 2.2.2; en-us; Vortex Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36912[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; VortexBuild/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36913[] = "Mozilla/5.0 (Linux; U; Android2.2.2; en-us; Vortex Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36914[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Vortex Build/FRG83G) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S36915[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG Vortex Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36916[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; -us; Vortex Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36917[] = "Opera/9.20 (Windows NT 6.0; U; en)";
const char S36918[] = "Opera/9.25 (Windows NT 6.0; U; en)";
const char S36919[] = "Opera/9.26 (Windows NT 6.0; U; en)";
const char S36920[] = "Opera/9.24 (Windows NT 6.0; U; en)";
const char S36921[] = "Opera/9.23 (Windows NT 6.0; U; en)";
const char S36922[] = "Opera/9.21 (Windows NT 6.0; U; en)";
const char S36923[] = "Opera/9.27 (Windows NT 6.0; U; ru)";
const char S36924[] = "Opera/9.22 (Windows NT 6.0; U; ru)";
const char S36925[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.0; en) Opera 9.26";
const char S36926[] = "('Opera/9.20 (Windows NT 6.0; U; en)', 'Opera 9.2, Windows Vista')";
const char S36927[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; AT100 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S36928[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; AT100 Build/HMJ37)";
const char S36929[] = "Logos/0.9.3 (Linux; U; Android 3.1; en_US; AT100 Build/HMJ37.31.5.0011)";
const char S36930[] = "Logos/0.9.4 (Linux; U; Android 3.1; en_US; AT100 Build/HMJ37.01.5.0032)";
const char S36931[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus 4 Build/JWR66V) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S36932[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S36933[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_5) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S36934[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S36935[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11";
const char S36936[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.65 Safari/535.11";
const char S36937[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_2) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.65 Safari/535.11";
const char S36938[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.65 Safari/535.11";
const char S36939[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.65 Safari/535.11";
const char S36940[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/17.0.648.205 Safari/534.16";
const char S36941[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/17.0.648.205 Safari/534.16";
const char S36942[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:24.0) Gecko/X Firefox/24.0";
const char S36943[] = "Mozilla/5.0 (Windows NT 6.0; rv:24.0) Gecko/X Firefox/24.0";
const char S36944[] = "Mozilla/5.0 (Windows NT 6.0; rv:24.0) Gecko/X Firefox/24.0 PaleMoon/24.0.1";
const char S36945[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:24.0) Gecko/X Firefox/24.0 PaleMoon/24.0.1";
const char S36946[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; TmstmpExt)";
const char S36947[] = "Outlook-Express/7.0 (MSIE 6.0; Windows NT 5.1; SV1; TmstmpExt)";
const char S36948[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; TmstmpExt)";
const char S36949[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 ; MSIE 6.0; Windows NT 5.1; SV1; ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; TmstmpExt)";
const char S36950[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 5.1; Trident/4.0; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; TmstmpExt)";
const char S36951[] = "Outlook-Express/7.0 (MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7; TmstmpExt)";
const char S36952[] = "Outlook-Express/7.0 (MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; TmstmpExt)";
const char S36953[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; LG-P940 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S36954[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ko-kr; LG-KU5400 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36955[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; LG-P940/V10h Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S36956[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; LG-P940-Orange Build/V10h) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S36957[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; LG-P940 Build/GWK74)";
const char S36958[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ko-kr; LG-SU540 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36959[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; LG-P940h Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S36960[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; LG-P940-Orange Build/V10n) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S36961[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; LG-P940/V10n Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S36962[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; LG-P940 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S36963[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; LG-SU540 Build/GWK74)";
const char S36964[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; LG-KU5400 Build/GWK74)";
const char S36965[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; sl; LG-P940 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S36966[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; LG-P940/V10o Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S36967[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P940h/ Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S36968[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LG-P940 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S36969[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; LG-P940/V10k Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S36970[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; LG-P940/V10f Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S36971[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S36972[] = "Mozilla/5.0 (Linux; Android 4.1.1; DROID RAZR HD Build/9.8.1Q_39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S36973[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT925 Build/9.8.2Q-8-XT925_VQL-12.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S36974[] = "Mozilla/5.0 (Linux; Android 4.1.1; XT925 Build/9.8.0Q-97-XT925_VQU-19) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S36975[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100 (Edition Next)";
const char S36976[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153";
const char S36977[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Rambler)";
const char S36978[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153";
const char S36979[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Campaign 21)";
const char S36980[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153";
const char S36981[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153,gzip(gfe)";
const char S36982[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Yx)";
const char S36983[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Seznam)";
const char S36984[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Campaign 06)";
const char S36985[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S36986[] = "Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S36987[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S36988[] = "Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S36989[] = "Mozilla/5.0 (Linux; Android 4.2; Galaxy Nexus Build/JOP40C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S36990[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT556 Build/V1.58R) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36991[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT557 Build/V1.65G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36992[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT556 Build/V1.70B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36993[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT556 Build/V1.58S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36994[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT557 Build/V1.65H) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36995[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; XT556 Build/V1.58R)";
const char S36996[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT557 Build/V1.65K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S36997[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; XT557 Build/V1.65K)";
const char S36998[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S36999[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20)";
const char S37000[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; MB886 Build/9.8.2Q-8_MB886_FFW-11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37001[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB886 Build/9.8.0Q-97_MB886_LE-7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37002[] = "stagefright/1.2 (Linux;Android 4.1.1;motorola MB886 Build/9.8.0Q-97_MB886_FFW-20)";
const char S37003[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37004[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; MB886 Build/9.8.2Q-8_MB886_FFW-11)";
const char S37005[] = "MOT-MB886/ Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB886 Build/9.8.0Q-97_MB886_FFW-20) 1617 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37006[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; MB886 Build/9.8.2Q-50_MB886_NII_TA-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37007[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.1;dev=qinara;devm=MB886;man=motorola;brw=Android;shwd=720x1184;shwa=720x1134;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S37008[] = "Mozilla/5.0 (Linux; U; Android2.2.1; en-us; SGH-T959V Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37009[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-T959V Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S37010[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;SGH-T959V Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37011[] = "Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; SGH-T959V Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37012[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-T959VBuild/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37013[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T959V Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37014[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-; SGH-T959V Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37015[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-T959V Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37016[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-T959V Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 androidAppVersion/2.0.2 AndroidWebkitWrapper";
const char S37017[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SGH-T959V Build/FROYO)";
const char S37018[] = "Logos/0.9.4 (Linux; U; Android 2.2.1; en_US; SGH-T959V Build/FROYO.VUVKD1)";
const char S37019[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-T959V Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S37020[] = "Logos/0.9.4 (Linux; U; Android 3.2.1; en_US; AT100 Build/HTK55D.31.5.0002)";
const char S37021[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; AT100 Build/HTK55D)";
const char S37022[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; AT100 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S37023[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; AT100 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S37024[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; at100 Build/v_1.2) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S37025[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; AT100 Build/v_1.3) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S37026[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; AT100 Build/v_1.1) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S37027[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-; AT100 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S37028[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB886 Build/7.7.1Q-115_MB886_FFW-14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37029[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; MB886 Build/7.7.1Q-115_MB886_BELL_FFW-12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37030[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB886 Build/7.7.1Q-144_MB886_MR1-11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37031[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB886 Build/7.7.1Q-144_MB886_MR1-22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37032[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MB886 Build/7.7.1Q-144_MB886_MR1-22)";
const char S37033[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; MB886 Build/7.7.1Q-115_MB886_BELL_FFW-11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37034[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB886 Build/7.7.1Q-115_MB886_FFW-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37035[] = "stagefright/1.2 (Linux;Android 4.0.4;motorola MB886 Build/7.7.1Q-144_MB886_MR1-22)";
const char S37036[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.3) Gecko/X Fedora/3.0.15-1.fc10 Firefox/3.0.15";
const char S37037[] = "SAMSUNG-GT-S5620B/1.0 SHP/VPP/R5 Dolfin/1.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S37038[] = "SAMSUNG-GT-S5620L/1.0 SHP/VPP/R5 Dolfin/1.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S37039[] = "SAMSUNG-GT-S5620/1.0 SHP/VPP/R5 Dolfin/1.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S37040[] = "SAMSUNG-GT-C3262 Opera/9.80 (J2ME/MIDP; Opera Mini/7.0.30281/30.3214; U; en) Presto/2.8.119 Version/11.10";
const char S37041[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9310; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.477 Mobile Safari/534.11+";
const char S37042[] = "iTunes/10.6 (Windows; Microsoft Windows 7 Home Premium Edition (Build 7600)) AppleWebKit/534.54.16";
const char S37043[] = "iTunes/10.6.3 (Windows; Microsoft Windows 7 x64 Home Premium Edition Service Pack 1 (Build 7601)) AppleWebKit/534.57.2";
const char S37044[] = "iTunes/10.6.3 (Windows; Microsoft Windows 7 Starter Edition (Build 7600)) AppleWebKit/534.57.2";
const char S37045[] = "iTunes/10.7 (Windows; Microsoft Windows 7 x64 Business Edition Service Pack 1 (Build 7601)) AppleWebKit/536.26.9";
const char S37046[] = "iTunes/10.7 (Windows; Microsoft Windows 7 Business Edition Service Pack 1 (Build 7601)) AppleWebKit/536.26.9";
const char S37047[] = "iTunes/10.7 (Windows; Microsoft Windows 7 x64 Ultimate Edition Service Pack 1 (Build 7601)) AppleWebKit/536.26.9";
const char S37048[] = "iTunes/10.7 (Windows; Microsoft Windows 7 Home Premium Edition Service Pack 1 (Build 7601)) AppleWebKit/536.26.9";
const char S37049[] = "iTunes/10.7 (Windows; Microsoft Windows 7 Ultimate Edition Service Pack 1 (Build 7601)) AppleWebKit/536.26.9";
const char S37050[] = "iTunes/10.7 (Windows; Microsoft Windows 7 x64 Home Premium Edition (Build 7600)) AppleWebKit/536.26.9";
const char S37051[] = "iTunes/10.7 (Windows; Microsoft Windows 7 x64 Ultimate Edition (Build 7600)) AppleWebKit/536.26.9";
const char S37052[] = "iTunes/10.7 (Windows; Microsoft Windows 7 Starter Edition Service Pack 1 (Build 7601)) AppleWebKit/536.26.9";
const char S37053[] = "iTunes/10.7 (Windows; Microsoft Windows 7 x64 Business Edition (Build 7600)) AppleWebKit/536.26.9";
const char S37054[] = "iTunes/10.7 (Windows; Microsoft Windows 7 Enterprise Edition Service Pack 1 (Build 7601)) AppleWebKit/536.26.9";
const char S37055[] = "iTunes/10.7 (Windows; Microsoft Windows 7 x64 Enterprise Edition Service Pack 1 (Build 7601)) AppleWebKit/536.26.9";
const char S37056[] = "iTunes/10.7 (Windows; Microsoft Windows 7 Business Edition (Build 7600)) AppleWebKit/536.26.9";
const char S37057[] = "iTunes/10.7 (Windows; Microsoft Windows 7 Ultimate Edition (Build 7600)) AppleWebKit/536.26.9";
const char S37058[] = "iTunes/10.7 (Windows; Microsoft Windows 7 x64 Enterprise Edition (Build 7600)) AppleWebKit/536.26.9";
const char S37059[] = "iTunes/10.7 (Windows; Microsoft Windows 7 x64 Home Basic Edition Service Pack 1 (Build 7601)) AppleWebKit/536.26.9";
const char S37060[] = "iTunes/10.7 (Windows; Microsoft Windows 7 x64 Service Pack 1 (Build 7601)) AppleWebKit/536.26.9";
const char S37061[] = "iTunes/10.7 (Windows; Microsoft Windows 7 Service Pack 1 (Build 7601)) AppleWebKit/536.26.9";
const char S37062[] = "iTunes/10.7 (Windows; Microsoft Windows 7 Enterprise Edition (Build 7600)) AppleWebKit/536.26.9";
const char S37063[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37064[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; A100 Build/IML74K)";
const char S37065[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; A100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37066[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ro-ro; IMPRESSPE3D Essential A100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37067[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37068[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; A100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37069[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; A100 Build/IMM76D)";
const char S37070[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; A-BELL A100 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37071[] = "Mozilla/5.0 (Series40; Nokia200/11.56; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S37072[] = "Mozilla/5.0 (Series40; Nokia200/10.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5";
const char S37073[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I577 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37074[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SAMSUNG-SGH-I577 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37075[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SAMSUNG-SGH-I577 Build/GINGERBREAD)";
const char S37076[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 2.3.6;dev=SGH-I577;devm=SAMSUNG-SGH-I577;man=samsung;brw=Android;shwd=480x800;shwa=480x763;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S37077[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S37078[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S37079[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S37080[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SGH-I337 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S37081[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SGH-I337 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S37082[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5233/40.1.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.7.4 3gpp-gba";
const char S37083[] = "Nokia5233/UCBrowser7.7.0.81/50/800";
const char S37084[] = "Nokia5233";
const char S37085[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5233/51.1.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 Mobile Safari/533.4 3gpp-gba";
const char S37086[] = "Nokia 5233";
const char S37087[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; VS910 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37088[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; VS910 4G Build/GRK39F)";
const char S37089[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; VS910 4G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37090[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; VS910 4G Build/GRJ22)";
const char S37091[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; VS910 4G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37092[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Maxthon 2.0)";
const char S37093[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAXTHON 2.0)";
const char S37094[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Maxthon 2.0)";
const char S37095[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; Maxthon 2.0)";
const char S37096[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MAXTHON 2.0)";
const char S37097[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Maxthon 2.0)";
const char S37098[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Maxthon 2.0)";
const char S37099[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Maxthon 2.0)";
const char S37100[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Tablet PC 2.0; MAXTHON 2.0)";
const char S37101[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; MAXTHON 2.0)";
const char S37102[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Maxthon 2.0)";
const char S37103[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; MAXTHON 2.0)";
const char S37104[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Tablet PC 2.0; Maxthon 2.0)";
const char S37105[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; Maxthon 2.0)";
const char S37106[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Tablet PC 2.0; Maxthon 2.0)";
const char S37107[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; MAXTHON 2.0)";
const char S37108[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Maxthon 2.0)";
const char S37109[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAXTHON 2.0)";
const char S37110[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; MAXTHON 2.0)";
const char S37111[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; WOW64; Trident/5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; Maxthon 2.0)";
const char S37112[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Maxthon 2.0)";
const char S37113[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2; .NET4.0C; MAXTHON 2.0)";
const char S37114[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Zune 4.0; InfoPath.3; MS-RTC LM 8; .NET4.0C; .NET4.0E; Maxthon 2.0)";
const char S37115[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; MAXTHON 2.0)";
const char S37116[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Maxthon 2.0)";
const char S37117[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; MAXTHON 2.0)";
const char S37118[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; SV1; .NET CLR 1.1.4322; MAXTHON 2.0)";
const char S37119[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Maxthon 2.0)";
const char S37120[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 3.0.04506; Media Center PC 5.0; SLCC1; .NET4.0C; Maxthon 2.0)";
const char S37121[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; MAXTHON 2.0)";
const char S37122[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; MAXTHON 2.0)";
const char S37123[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.3 (KHTML, like Gecko) Maxthon/3.3.1.600 Safari/535.3";
const char S37124[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.3 (KHTML, like Gecko) Maxthon/3.3.1.600 Safari/535.3";
const char S37125[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.3 (KHTML, like Gecko) Maxthon/3.3.3.1000 Chrome/16.0.883.0 Safari/535.3";
const char S37126[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.3 (KHTML, like Gecko) Maxthon/3.3.3.1000 Chrome/16.0.883.0 Safari/535.3";
const char S37127[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.3.9.1000 Chrome/18.0.966.0 Safari/535.12";
const char S37128[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.3.9.1000 Chrome/18.0.966.0 Safari/535.12";
const char S37129[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.12 (KHTML like Gecko) Maxthon/3.3.4.4000 Chrome/18.0.966.0 Safari/535.12";
const char S37130[] = "Mozilla/5.0 (Series40; Nokia311/03.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S37131[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18";
const char S37132[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko)";
const char S37133[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7D11";
const char S37134[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko)";
const char S37135[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7C144";
const char S37136[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko)";
const char S37137[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7D11 Safari/528.16";
const char S37138[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko)";
const char S37139[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Safari/528.18";
const char S37140[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7D11 Safari/528.18";
const char S37141[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7C144 Safari/528.18";
const char S37142[] = "Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko)";
const char S37143[] = "Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18";
const char S37144[] = "Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; es-es) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18";
const char S37145[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Safari/528.16";
const char S37146[] = "Mozilla/5.0 (device; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18";
const char S37147[] = "Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18";
const char S37148[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3 1 2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7D11";
const char S37149[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HUAWEI MediaPad Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37150[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SpringBoard Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37151[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HUAWEI MediaPad Build/HuaweiMediaPad)";
const char S37152[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MediaPad 7 Lite Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37153[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; MediaPad 7 Lite Build/HuaweiMediaPad)";
const char S37154[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; GT01 Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37155[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HUAWEI OsiMood MediaPad Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37156[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MediaPad 10 LINK Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37157[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SpringBoard Build/HuaweiMediaPad)";
const char S37158[] = "Nokia6120c/4.21 (SymbianOS/9.2; U; Series60/3.1 Nokia6120c/4.21; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) Mozilla/5.0 AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S37159[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Nokia6120ci/7.02; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S37160[] = "Nokia6120ci/7.10 (SymbianOS/9.2; U; Series60/3.1 Nokia6120ci/7.10; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) Mozilla/5.0 AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S37161[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Nokia6120c/7.10; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S37162[] = "Nokia6120ciB11.04/SymbianOS/9.1 Series60/3.0";
const char S37163[] = "Nokia6120c/3.83; Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S37164[] = "Nokia6120c(B20.03)/SymbianOS/9.1 Series60/3.0";
const char S37165[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Nokia6120guration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S37166[] = "Nokia6120ci";
const char S37167[] = "Mozilla/5.0 (PlayBook; U; RIM Tablet OS 2.1.0; en-US) AppleWebKit/536.2+ (KHTML, like Gecko) Version/7.2.1.0 Safari/536.2+";
const char S37168[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9670; en-US) AppleWebKit/534.8+ (KHTML, like Gecko) Version/6.0.0.706 Mobile Safari/534.8+";
const char S37169[] = "Nokia300/5.0 (07.03) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S37170[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:14.0) Gecko/X Firefox/14.0.1";
const char S37171[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:14.0) Gecko/X Firefox/14.0.1 Firefox/14.0.1";
const char S37172[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:10.0.1) Gecko/X Firefox/10.0.1";
const char S37173[] = "Mozilla/5.0 (Windows NT 6.0; rv:10.0.2) Gecko/X Firefox/10.0.2 Firefox/10.0.2";
const char S37174[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:10.0.2) Gecko/X Firefox/10.0.2";
const char S37175[] = "Mozilla/5.0 (Windows NT 6.0; rv:10.0.12) Gecko/X Firefox/10.0.12";
const char S37176[] = "Mozilla/6.0 (Windows NT 6.0; rv:10.0.3) Gecko/X Firefox/10.0.2";
const char S37177[] = "Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.11";
const char S37178[] = "Opera/9.80 (Windows NT 6.0; WOW64) Presto/2.12.388 Version/12.11";
const char S37179[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.11";
const char S37180[] = "Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.11";
const char S37181[] = "Opera/9.80 (Windows NT 6.0; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.11";
const char S37182[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.11";
const char S37183[] = "Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.11,gzip(gfe)";
const char S37184[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0) Opera 12.11";
const char S37185[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.11";
const char S37186[] = "Opera/9.80 (Windows NT 6.1; WOW64; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)) Presto/2.12.388 Version/12.11";
const char S37187[] = "Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.11";
const char S37188[] = "Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.11";
const char S37189[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.11";
const char S37190[] = "Opera/9.80 (Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)) Presto/2.12.388 Version/12.11";
const char S37191[] = "Opera/9.80 (Windows NT 5.1; compatible; MSIE 7.0; Windows NT 6.0) Presto/2.12.388 Version/12.11";
const char S37192[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.11";
const char S37193[] = "Mozilla/5.0 (Windows NT 6.0; rv:2.0) Gecko/X Firefox/4.0 Opera 12.11";
const char S37194[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.11";
const char S37195[] = "Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.11";
const char S37196[] = "Opera/9.80 (Windows NT 6.1; WOW64; compatible; MSIE 7.0; Windows NT 6.0) Presto/2.12.388 Version/12.11";
const char S37197[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.11";
const char S37198[] = "Opera/9.80 (Windows NT 6.0; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.11";
const char S37199[] = "Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.11";
const char S37200[] = "Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.11";
const char S37201[] = "Opera/9.80 (Windows NT 6.0; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.11";
const char S37202[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.11";
const char S37203[] = "Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.11";
const char S37204[] = "Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.11";
const char S37205[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.11";
const char S37206[] = "Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.11";
const char S37207[] = "Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.11";
const char S37208[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.11";
const char S37209[] = "Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.11";
const char S37210[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.11";
const char S37211[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LS670 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37212[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LS670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37213[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-LS670 Build/0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37214[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LS670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37215[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; LS670 Build/GRI40)";
const char S37216[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-LS670 Build/106) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37217[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-LS670 Build/107) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37218[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-LS670 Build/109) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37219[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-LS670 Build/STABLE-001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37220[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LS670 Build/GRI40) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S37221[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; LS670 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37222[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-LS670 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37223[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-LS670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37224[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C5155 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37225[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; C5155 Build/IML77)";
const char S37226[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; Rise Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37227[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Rise Build/IMM76I)";
const char S37228[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SAMSUNG-SGH-I747 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37229[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-L710 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37230[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-R530U Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37231[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SAMSUNG-SGH-I747 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37232[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-L710 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37233[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L710 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37234[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I747 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37235[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-R530U Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37236[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530U Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37237[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; GT-I9305 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37238[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530C Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37239[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; GT-I9305 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37240[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I747 Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37241[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L710 Build/MROM; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37242[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I747 Build/MROM; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37243[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L710 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/d2spr";
const char S37244[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530U Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/d2usc";
const char S37245[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530C Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/d2cri";
const char S37246[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; GT-I9305 Build/4.80) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/4.80/i9305";
const char S37247[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ASUS Transformer Pad TF700T Build/JRO03C)";
const char S37248[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ASUS Transformer Pad TF700T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37249[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; ASUS Pad TF700T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37250[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; da-dk; ASUS Transformer Pad TF700KL Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37251[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ASUS Transformer Pad TF700T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37252[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JWR66N) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37253[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37254[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37255[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JWR66X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37256[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37257[] = "Dalvik/1.6.0 (Linux; U; Android 4.3; Nexus 4 Build/JWR66Y)";
const char S37258[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-; Nexus 4 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37259[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37260[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37261[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37262[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:10.0.2) Gecko/X Firefox/10.0.2 Firefox/10.0.2";
const char S37263[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:10.0.4) Gecko/X Firefox/10.0.4";
const char S37264[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2; rv:10.0.1) Gecko/X Firefox/10.0.1";
const char S37265[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Legend 2.05.161.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37266[] = "Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; HTC Legend 1.32.163.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37267[] = "Mozilla/5.0 (Linux; U; Android 2.2; ko-; HTC Legend Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37268[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-au; HTC Legend Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37269[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Legend 1.32.161.4 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37270[] = "Mozilla/5.0 (Linux; U; Android 2.2; de-at; HTC Legend 3.14.177.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37271[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Legend 1.23.161.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37272[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Legend 3.14.161.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37273[] = "Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; HTC Legend 2.05.163.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37274[] = "Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; HTC Legend 3.14.163.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37275[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; HTC Legend Build/FRF91)";
const char S37276[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Legend Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S37277[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Legend 3.14.161.2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37278[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; Legend Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37279[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; Legend Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37280[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Legend Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S37281[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.231 Safari/534.10";
const char S37282[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.231 Safari/534.10";
const char S37283[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.231 Safari/534.10";
const char S37284[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.231 Safari/534.10";
const char S37285[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/525.13 (KHTML, like Gecko; Google Page Speed) Chrome/8.0.552.237 Safari/525.13";
const char S37286[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.231 Safari/534.10";
const char S37287[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.17.8 (KHTML, like Gecko) Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en) AppleWebKit/533.17.8 (KHTML, like Gecko) Chrome/8.1.0.0 Safari/533.17.8";
const char S37288[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.210 Safari/534.10";
const char S37289[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10";
const char S37290[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.215 Safari/534.10";
const char S37291[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.231 Safari/534.10";
const char S37292[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1064; en-CA) AppleWebKit/525.13 (KHTML like Gecko; Google Page Speed) Chrome/8.0.552.237 Safari/525.13";
const char S37293[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1064; es-US) AppleWebKit/525.13 (KHTML like Gecko; Google Page Speed) Chrome/8.0.552.237 Safari/525.13";
const char S37294[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1064; en-US) AppleWebKit/525.13 (KHTML like Gecko; Google Page Speed) Chrome/8.0.552.237 Safari/525.13";
const char S37295[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1064; haw-US) AppleWebKit/525.13 (KHTML like Gecko; Google Page Speed) Chrome/8.0.552.237 Safari/525.13";
const char S37296[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1064; fr-CA) AppleWebKit/525.13 (KHTML like Gecko; Google Page Speed) Chrome/8.0.552.237 Safari/525.13";
const char S37297[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1064; zh-CN) AppleWebKit/525.13 (KHTML like Gecko; Google Page Speed) Chrome/8.0.552.237 Safari/525.13";
const char S37298[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1064; fr-FR) AppleWebKit/525.13 (KHTML like Gecko; Google Page Speed) Chrome/8.0.552.237 Safari/525.13";
const char S37299[] = "Opera/9.80 (X11; Linux x86_64; U; ru) Presto/2.10.289 Version/12.02";
const char S37300[] = "Mozilla/5.0 (X11; Linux i686; en; rv:2.0) Gecko/X Firefox/4.0 Opera 12.02";
const char S37301[] = "Mozilla/5.0 (compatible; MSIE 9.0; X11; Linux i686; en) Opera 12.02";
const char S37302[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; rv:1.7.3) Gecko/20041001 Firefox/0.10.1";
const char S37303[] = "Mozilla/5.0 (Windows; U; Windows NT5.1; en-US; rv:1.7) Gecko/X Firefox/0.8.0+";
const char S37304[] = "Mozilla/5.0 (Windows; ?; Windows NT 5.1; *rv:*) Gecko/* Firefox/0.9*";
const char S37305[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.6) Gecko/X Firefox/0.8 Mnenhy/0.6.0.103";
const char S37306[] = "SAMSUNG-GT-C3303/C3303DDJG4 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1,SAMSUNG-GT-C3303/C3303DDJG4 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S37307[] = "SAMSUNG-GT-C3303i/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S37308[] = "SAMSUNG-GT-C3303/C3303DDJG4 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S37309[] = "SAMSUNG-GT-C3303i/C3303iDDKB2 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1,SAMSUNG-GT-C3303i/C3303iDDKB2 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S37310[] = "SAMSUNG-GT-C3303K/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S37311[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37312[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-S720C Build/GINGERBREAD)";
const char S37313[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us;SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37314[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-S720CBuild/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37315[] = "Mozilla/5.0 (Linux; U; Android2.3.6; en-us; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37316[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S37317[] = "Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37318[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37319[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37320[] = "Android/1.0 (SCH-S720C GINGERBREAD)";
const char S37321[] = "Mozilla/5.0 (Windows NT 5.1; WOW64; rv:24.0) Gecko/X Firefox/24.0";
const char S37322[] = "Mozilla/5.0 (Windows NT 5.2; rv:24.0) Gecko/X Firefox/24.0";
const char S37323[] = "Mozilla/5.0 (Windows NT 5.1; rv:24.0) Gecko/X Firefox/24.0 PaleMoon/24.0.1";
const char S37324[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; Nexus S 4G Build/JRN84D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37325[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S 4G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37326[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S 4G Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37327[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Full Android on Crespo4G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37328[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S 4G Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37329[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Nexus S 4G Build/JRO03R)";
const char S37330[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Full Android on Crespo4G Build/PHONELAB_JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37331[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus S 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37332[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:19.0) Gecko/19.0 Firefox/19.0";
const char S37333[] = "Mozilla/5.0 (Windows NT 6.0; rv:19.0) Gecko/X Firefox/19.0 AlexaToolbar/alxf-2.17";
const char S37334[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:19.0) Gecko/X Firefox/19.0 GTB6";
const char S37335[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; Orange Monte Carlo Build/OUK_B05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37336[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; Orange Monte Carlo Build/OUK_B03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37337[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-at; Orange Monte Carlo Build/OAT_B05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37338[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; ZTE-Skate Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37339[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; BASE Lutea 2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37340[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sk-sk; Orange Monte Carlo Build/OSK_B04) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37341[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Orange Monte Carlo Build/OUK_B05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37342[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-at; Orange Monte Carlo Build/OAT_B07) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37343[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Orange Monte Carlo Build/OUK_B08) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37344[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; ZTE Skate Build/V1.0.0B03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37345[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-at; Orange Monte Carlo Build/OAT_B06) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37346[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; Orange Monte Carlo Build/OSP_B02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37347[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; Orange Monte Carlo Build/OSP_B03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37348[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; ZTE-SKATE Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37349[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; ZTE Skate Build/OUK_B05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37350[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; ZTE-SKATE Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37351[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; sk-sk; Orange Monte Carlo Build/OSK_B05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37352[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; ZTE Skate Build/V1.0.0B05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37353[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; ZTE Skate Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37354[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; Orange Monte Carlo Build/OUK_B05)";
const char S37355[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; Skate Build/R0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37356[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; ZTE-SKATE Build/GRJ22)";
const char S37357[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; Skate Build/GRJ22)";
const char S37358[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; ZTE-SKATE Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37359[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-at; Orange Monte Carlo Build/OAT_B08) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37360[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; sk-sk; Orange Monte Carlo Build/OSK_B06) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37361[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Orange Monte Carlo Build/OSP_B03)";
const char S37362[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Orange Monte Carlo Build/OUK_B07) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37363[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; BASE Lutea 2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37364[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; ZTE Skate Build/OUK_B08) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37365[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; ZTE-Skate Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37366[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-mx; ZTE-U V960 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37367[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; Skate Build/GRJ22; The Dark ICS By Faia) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37368[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; Orange Monte Carlo Build/OSP_B04) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37369[] = "Mozilla/5.0 (Linux; Android 4.2; Nexus 4 Build/JOP40C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S37370[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S37371[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S37372[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 4 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S37373[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 4 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S37374[] = "Mozilla/5.0 (Linux; Android 4.2; Nexus 4 Build/JOP40C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S37375[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Sprint APA7373KT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37376[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Sprint APA7373KT Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37377[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Sprint APA7373KT Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37378[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; HD7)";
const char S37379[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; T9296)";
const char S37380[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; HD7 T9292)";
const char S37381[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.2; Trident/5.0; IEMobile/9.0; HTC; HD7 T9292)";
const char S37382[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Gold)";
const char S37383[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Touch-IT HD7)";
const char S37384[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Mersad HD7 T9292)";
const char S37385[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; HD7 Infinity)";
const char S37386[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; GOLD-7PRO)";
const char S37387[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; WpsLondonTest; HTC; HD7 T9292)";
const char S37388[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:10.0.5) Gecko/X Firefox/10.0.5";
const char S37389[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:10.0.2) Gecko/X Firefox/10.0.2 Firefox/10.0.2";
const char S37390[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2; rv:10.0.1) Gecko/X Firefox/10.0.1";
const char S37391[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I8530 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37392[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I8530 Build/GINGERBREAD)";
const char S37393[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; SAMSUNG GT-I8530/I8530BULE3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37394[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; SAMSUNG GT-I8530/I8530BULH1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37395[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-I8530 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37396[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8530 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S37397[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7310 Build/HTJ85B)";
const char S37398[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7310 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S37399[] = "Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHW-M305W Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S37400[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7310 Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S37401[] = "Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHW-M305W Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S37402[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7310 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S37403[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7310 Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S37404[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7310 Build/HTJ85B) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S37405[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P7310 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S37406[] = "Biblia/2.1.2 (Linux; U; Android 3.2; es-ES; GT-P7310 Build/HTJ85B)";
const char S37407[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7310 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S37408[] = "Logos/2.2.3 (Linux; U; Android 3.2; en-US; GT-P7310 Build/HTJ85B.UELA3)";
const char S37409[] = "Logos/2.2.3 (Linux; U; Android 3.2; en-US; GT-P7310 Build/HTJ85B.UEKJ5)";
const char S37410[] = "Logos/2.2.8.1 (Linux; U; Android 3.2; en-US; GT-P7310 Build/Overcome 8.9 Series v2.2.0)";
const char S37411[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7310 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.12;FBDM/{density=1.0,width=800,height=1232};FBLC/en_US;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P7310;FBSV/3.2;]";
const char S37412[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7310 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37413[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7310 Build/HTJ85B) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S37414[] = "Mozilla/5.0 (Series40; Nokia302/14.26; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S37415[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/12.04 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19";
const char S37416[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/12.04 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19";
const char S37417[] = "Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/12.04 Chromium/18.0.1025.151 Chrome/18.0.1025.151 Safari/535.19";
const char S37418[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/12.04 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19,gzip(gfe) (via translate.google.com)";
const char S37419[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/12.04 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19,gzip(gfe) (via translate.google.com)";
const char S37420[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/12.04 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19,gzip(gfe)";
const char S37421[] = "WordPress 3.5-alpha-21535 Feed Client Mozilla/5.0 Compatible";
const char S37422[] = "WordPress 3.5-alpha-21989 Feed Client Mozilla/5.0 Compatible";
const char S37423[] = "WordPress 3.5-RC6-23166 Feed Client Mozilla/5.0 Compatible";
const char S37424[] = "WordPress 3.6-alpha-23334 Feed Client Mozilla/5.0 Compatible";
const char S37425[] = "WordPress 3.6-beta3-24375 Feed Client Mozilla/5.0 Compatible";
const char S37426[] = "WordPress 3.6-beta4 Feed Client Mozilla/5.0 Compatible";
const char S37427[] = "WordPress 3.6-beta4-24523 Feed Client Mozilla/5.0 Compatible";
const char S37428[] = "WordPress 3.6-RC1-24757 Feed Client Mozilla/5.0 Compatible";
const char S37429[] = "WordPress 3.6-RC2 Feed Client Mozilla/5.0 Compatible";
const char S37430[] = "WordPress/3.5.1; http://news.money.ca";
const char S37431[] = "WordPress/3.5.2; http://www.isitwp.com";
const char S37432[] = "WordPress/3.5.2; http://boocu.com,gzip(gfe) (via docs.google.com/viewer)";
const char S37433[] = "WordPress/3.5.2; http://localhost/wordpress";
const char S37434[] = "WordPress 3.6 Feed Client Mozilla/5.0 Compatible";
const char S37435[] = "WordPress/3.6-RC2-24920; http://ioneglobalgrind.wordpress.com";
const char S37436[] = "WordPress 3.7-alpha-24954 Feed Client Mozilla/5.0 Compatible";
const char S37437[] = "WordPress 3.7-alpha-25000 Feed Client Mozilla/5.0 Compatible";
const char S37438[] = "WordPress/3.7-alpha-25000;";
const char S37439[] = "WordPress 3.7-alpha-25157 Feed Client Mozilla/5.0 Compatible";
const char S37440[] = "WordPress/3.7-alpha-25157;";
const char S37441[] = "WordPress/3.5.1; http://tommyscostaricadreams.com";
const char S37442[] = "Mozilla/5.0 (Linux; Android 4.1.1; C6603 Build/10.1.A.0.287) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S37443[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6603 Build/10.1.A.1.434) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S37444[] = "Mozilla/5.0 (Series40; Nokia201/11.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S37445[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MT15i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37446[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonMT15i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37447[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonMT15iv Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37448[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MT15i Build/4.1.B.0.587)";
const char S37449[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MT15i Build/4.1.B.0.431) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S37450[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; MT15i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37451[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; MT15i Build/4.1.B.1.13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37452[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8J2 Safari/6533.18.5";
const char S37453[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8L1 Safari/6533.18.5";
const char S37454[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8G4 Safari/6533.18.5";
const char S37455[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8J3 Safari/6533.18.5";
const char S37456[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8H7 Safari/6533.18.5";
const char S37457[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8F190 Safari/6533.18.5";
const char S37458[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8K2 Safari/6533.18.5";
const char S37459[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8F191 Safari/6533.18.5";
const char S37460[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8H8 Safari/6533.18.5";
const char S37461[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SGH-T589 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37462[] = "Mozilla/5.0 (Linux; U; Android2.2.2; en-us; SGH-T589 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37463[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;SGH-T589 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37464[] = "Mozilla/5.0 (Linux; U;Android 2.2.2; en-us; SGH-T589 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37465[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; SGH-T589R Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37466[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SGH-T589Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37467[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SGH-T589 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S37468[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; SGH-T589 Build/FROYO)";
const char S37469[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; SGH-T589R Build/FROYO)";
const char S37470[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; T-Mobile G2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37471[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; T-Mobile G2Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37472[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; T-Mobile G2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37473[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; T-Mobile G2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S37474[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; T-Mobile G2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S37475[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; T-MobileG2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37476[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;T-Mobile G2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37477[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; T-Mobile G2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37478[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;T-Mobile G2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37479[] = "Mozilla/5.0 (Linux; U; Android2.3.3; en-us; T-Mobile G2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37480[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; T-MobileG2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37481[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; T-Mobile G2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37482[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; T-Mobile G2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37483[] = "Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; T-Mobile G2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37484[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; T-Mobile G2 Build/GRJ22)";
const char S37485[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; T-Mobile G2 Build/GRI40)";
const char S37486[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; T-Mobile G2 Build/GRJ22) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S37487[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; T-Mobile G2 Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30";
const char S37488[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; T-Mobile G2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37489[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/ZV4.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37490[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; LG-VM696 Build/ZV5.GWK74)";
const char S37491[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/ZV5.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37492[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/Sleipnir 2.0.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37493[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/ICS Elite) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37494[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-; LG-VM696 Build/ZV5.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37495[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/TwIsT3D v1.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37496[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/Tagged 1.0.5 (Full)) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37497[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/Tagged 1.0.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37498[] = "NokiaC3-01/5.0 (06.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S37499[] = "NokiaC3-01/5.0 (05.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac3-01";
const char S37500[] = "NokiaC3-01.5/5.0 (06.75) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S37501[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; HTC_K2_UL Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37502[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; HTC_OneSV Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37503[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC One SV Build/JZO54K)";
const char S37504[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC_OneSV/2.14.161.7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37505[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC_C525u Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37506[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-ch; HTC_OneSV/2.14.166.7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37507[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-at; HTC_OneSV/2.14.169.7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37508[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; HTC_OneSV/2.14.164.107 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37509[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; HTC_OneSV/2.14.162.8 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37510[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One SV Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37511[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; HTC_OneSV/2.15.118.2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37512[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; HTC_OneSV/2.16.111.3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37513[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; HTC_C525u-orange-LS/2.16.73.3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37514[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; cs-cz; HTC_OneSV/2.15.113.2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37515[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S37516[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505G Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S37517[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505X Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S37518[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; SGH-T959 Build/ECLAIR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37519[] = "Mozilla/5.0 (Linux; U;Android 2.1-update1; en-us; SGH-T959 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S37520[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SGH-T959 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 MobileSafari/530.17";
const char S37521[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SGH-T959Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S37522[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us;SGH-T959 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S37523[] = "Mozilla/5.0 (Linux; U; Android2.1-update1; en-us; SGH-T959 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S37524[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SGH-T959 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S37525[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; SGH-T959 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S37526[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SGH-T959 Build/ECLAIR) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S37527[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SGH-T959 Build/ECLAIR)";
const char S37528[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; SGH-T959D Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S37529[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SGH-T959D Build/ECLAIR)";
const char S37530[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en) Presto/2.9.168 Version/11.52";
const char S37531[] = "Opera 11.52(Intel Mac OS X 10.6.8): Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en-US) Presto/2.9.168 Version/11.52";
const char S37532[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A523 Safari/8536.25";
const char S37533[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A403 Safari/8536.25";
const char S37534[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A406 Safari/8536.25";
const char S37535[] = "Mozilla/5.0 (iPod; CPU iPhone OS 601 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML like Gecko) CriOS/23.0.1271.96 Mobile/10A523 Safari/8536.25";
const char S37536[] = "Mozilla/5.0 (Linux; Android 4.2; Galaxy Nexus Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S37537[] = "Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S37538[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S37539[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S37540[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTCONE Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S37541[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC_PN071 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S37542[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One 801e Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S37543[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One dual sim Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S37544[] = "BlackBerry8310_CMCC/4.5.0.142 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/241";
const char S37545[] = "BlackBerry8310/4.5.0.182 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102";
const char S37546[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; B1-A71 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Safari/534.30";
const char S37547[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; B1-A71 Build/JZO54K)";
const char S37548[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; B1-A71 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37549[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9A406 Safari/7534.48.3";
const char S37550[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9A405 Safari/7534.48.3";
const char S37551[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9A334 Safari/7534.48.3";
const char S37552[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; A501 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S37553[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; A501 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S37554[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; A501 Build/HTJ85B)";
const char S37555[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; A501 Build/HTK55D)";
const char S37556[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37557[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-P1010 Build/FROYO)";
const char S37558[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37559[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37560[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1010 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S37561[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; GT-P1010 Build/FROYO)";
const char S37562[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; GT-P1010 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S37563[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1010 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S37564[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; GT-P1010 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S37565[] = "Biblia/2.1.2 (Linux; U; Android 2.2.1; es-US; GT-P1010 Build/FROYO.UJKB4)";
const char S37566[] = "Logos/2.1.2 (Linux; U; Android 2.2.1; en-GB; GT-P1010 Build/FROYO.XWKB8)";
const char S37567[] = "Android/4.5 (Linux; U; Android 2.2.1; en_US; GT-P1010)";
const char S37568[] = "Logos/2.1.2 (Linux; U; Android 2.2.1; de-DE; GT-P1010 Build/FROYO.XWKM2)";
const char S37569[] = "Biblia/2.2.3 (Linux; U; Android 2.2.1; es-ES; GT-P1010 Build/FROYO.UJKM1)";
const char S37570[] = "Logos/2.2.3 (Linux; U; Android 2.2.1; en-US; GT-P1010 Build/FROYO.JPKC4)";
const char S37571[] = "Vyrso/2.2.8.1 (Linux; U; Android 2.2.2; pt-BR; GT-P1010 Build/FROYO.UJKM5)";
const char S37572[] = "Logos/2.2.8.1 (Linux; U; Android 2.2.1; en-US; GT-P1010 Build/FROYO.UEKC6)";
const char S37573[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-P1010 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37574[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1010 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 baidubrowser/042_1.41.3.2_diordna_4201_006/gnusmas_8_1.2.2_0101P-TG/Xy/6EC45499CXCBDFFE6D53AE8C2B0%7C0/1";
const char S37575[] = "FaithlifeStudyBible/2.2.10 (Linux; U; Android 2.2.1; en-GB; GT-P1010 Build/FROYO.XWKC1)";
const char S37576[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37577[] = "Android 2.2.1; 8; samsung; GT-P1010";
const char S37578[] = "Mozilla/5.0 (Unknown; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.8.1 Safari/534.34";
const char S37579[] = "Mozilla/5.0 (Unknown; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.8.2 Safari/534.34";
const char S37580[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Twitter for iPhone";
const char S37581[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Twitter for iPhone";
const char S37582[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B147 Twitter for iPhone";
const char S37583[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Twitter for iPhone";
const char S37584[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P870/P87009l Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37585[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P870 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37586[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P870/P87010c Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37587[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P870/P87010f Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37588[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P870 Build/IMM76I)";
const char S37589[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P870/P87010i Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37590[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SAMSUNG-SGH-I927 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37591[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SAMSUNG-SGH-I927 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37592[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SAMSUNG-SGH-I927 Build/GINGERBREAD)";
const char S37593[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I927 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37594[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SAMSUNG-SGH-I927 Build/GINGERBREAD)";
const char S37595[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SAMSUNG-SGH-I927 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37596[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.30 (KHTML, like Gecko) Ubuntu/11.04 Chromium/12.0.742.112 Chrome/12.0.742.112 Safari/534.30";
const char S37597[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.30 (KHTML, like Gecko) Ubuntu/11.04 Chromium/12.0.742.112 Chrome/12.0.742.112 Safari/534.30";
const char S37598[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SGH-T499Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37599[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SGH-T499 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S37600[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; fr-ca; SGH-T499V Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37601[] = "Mozilla/5.0 (Linux; U;Android 2.2.2; en-us; SGH-T499 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37602[] = "Mozilla/5.0 (Linux; U; Android2.2.2; en-us; SGH-T499 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37603[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SGH-T499 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37604[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;SGH-T499 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37605[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; SGH-T499Y Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37606[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SGH-T499 Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S37607[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; SGH-T499 Build/FROYO)";
const char S37608[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SGH-T499 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S37609[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; SGH-T499Y Build/FROYO)";
const char S37610[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37611[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; KFTT Build/IML74K)";
const char S37612[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; KFTT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37613[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; KFTT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37614[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SM-T210 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37615[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SM-T210R Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37616[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SM-T210R Build/JZO54K)";
const char S37617[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SM-T210 Build/JZO54K)";
const char S37618[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; SonyEricssonWT19i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37619[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; WT19i Build/4.1.L.0.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37620[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; WT19i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37621[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; WT19i Build/4.1.B.0.587)";
const char S37622[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; WT19i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37623[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SonyEricssonWT19iv Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37624[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; M886 Build/HuaweiM886)";
const char S37625[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; M886 Build/HuaweiM886) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37626[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; M886 Build/HuaweiM886) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37627[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; M886 Build/HuaweiM886)";
const char S37628[] = "Mozilla/5.0 (Linux; U; Android 2.3.6-Oriax123; en-us; M886 Build/HuaweiM886) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37629[] = "Mozilla/5.0 (Linux; U; Android 2.3.6 - iCe - xXNubCakeXx; en-us; M886 Build/HuaweiM886) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37630[] = "Mozilla/5.0 (Linux; U; Android 2.3.6_TheTrooperRom_v2.0; en-us; M886 Build/HuaweiM886) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37631[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; XT603 Build/5.5.1Q-117_PAX-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37632[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; XT603 Build/5.5.1Q-117_PAX-33) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37633[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; XT603 Build/5.5.1Q-117_PAX-63) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37634[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; XT603 Build/5.5.1Q-117_PAX-58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37635[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; XT603 Build/5.5.1Q-117_PAX-69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37636[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; XT603 Build/5.5.1Q-117_PAX-79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37637[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; XT603 Build/5.5.1Q-117_PAX-79)";
const char S37638[] = "Mozilla/5.0 (iPad; U; CPU OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9B206 Safari/7534.48.3";
const char S37639[] = "Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9B176 Safari/7534.48.3";
const char S37640[] = "Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9A405 Safari/7534.48.3";
const char S37641[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N5100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37642[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N5100 Build/JZO54K)";
const char S37643[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-N5100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37644[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID Pro Build/4.5.1-110-VNS-11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37645[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID Pro Build/4.5.1-110-VNS-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37646[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;DROID Pro Build/4.5.1-110-VNS-22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37647[] = "Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; DROID Pro Build/4.5.1-110-VNS-22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37648[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID Pro Build/4.5.1-110-VNS-22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37649[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID Pro Build/4.5.1-110-VNS-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37650[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; DROID Pro Build/4.5.1-110-VNS-22)";
const char S37651[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID Pro Build/4.5.1-110-VNS-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S37652[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT610 Build/4.5.2-51_V2L-28_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37653[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID ProBuild/4.5.1-110-VNS-22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37654[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDPro Build/4.5.1-110-VNS-22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37655[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID Pro Build/4.5.1-110-VNS-22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S37656[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDPro Build/4.5.1-110-VNS-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37657[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;DROID Pro Build/4.5.1-110-VNS-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37658[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID ProBuild/4.5.1-110-VNS-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37659[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; DROID Pro Build/4.5.1-110-VNS-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37660[] = "Mozilla/5.0 (Linux; U; Android2.3.3; en-us; DROID Pro Build/4.5.1-110-VNS-22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37661[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; DROID Pro Build/4.5.1-110-VNS-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37662[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID Pro Build/S273) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37663[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; DROID Pro Build/4.5.1-110-VNS-35) com.google.android.apps.books/1.1.5 (1105) Safari/";
const char S37664[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-ca; SonyEricssonR800a Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37665[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonR800a Build/3.0.1.A.0.126) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37666[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonR800a Build/3.0.1.A.0.135) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37667[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-mx; R800a Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37668[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; iw-il; SonyEricssonR800a Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37669[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-ca; R800a Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37670[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; es-mx; SonyEricssonR800a Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37671[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-mx; SonyEricssonR800a Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37672[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; es-mx; R800a Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37673[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; SonyEricssonR800a Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37674[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; R800a Build/4.0.2.A.0.42)";
const char S37675[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; SonyEricssonR800a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37676[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; R800a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37677[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-cr; SonyEricssonR800a Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37678[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; R800a Build/3.0.1.A.0.145)";
const char S37679[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.2; R800a Build/3.0.A.2.181)";
const char S37680[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonR800a Build/3.0.1.A.0.148) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37681[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; iw-il; R800a Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37682[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es;LG-P920/V20a Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S37683[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; LG-P920 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S37684[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es;LG-P920/V20c Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S37685[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; LG-P920 Build/GRJ90)";
const char S37686[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; LG-P920/V20a Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S37687[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; nl-nl; LG-P920/V21b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S37688[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-sa; LG-P920h Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S37689[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-sa; LG-P920h Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S37690[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es;LG-P920/V20e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S37691[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-nz;LG-P920/V20b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S37692[] = "Mozilla/5.0 (Linux; U; Android 2.3.6 www.htcspain.com; es-es; LG-P920 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S37693[] = "Mozilla/5.0 (Linux; U; Android 2.3.6/AcuraTeam; es-sa; LG-P920h Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S37694[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; LG-P920/V21a Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S37695[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; LG-P920/V20b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S37696[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ar-ar; LG-P920 Build/GRJ90) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S37697[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P920 Build/PromethiusProject v1.5 21E) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S37698[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; LG-P920-Orange Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S37699[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LG-P920 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S37700[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-ch;LG-P920/V20f Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S37701[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; LG-P920 Build/PromethiusProject v1.5 20F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S37702[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; nl-nl; LG-P920/V21e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S37703[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AT300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37704[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; AT300 Build/IML74K)";
const char S37705[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AT300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37706[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Safari/535.19 Silk-Accelerated=true";
const char S37707[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S37708[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 7 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S37709[] = "Mozilla/5.0 (Linux; Android 4.2; Nexus 7 Build/JOP40C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S37710[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S37711[] = "Mozilla/5.0 (Series40; Nokia200/11.56; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6";
const char S37712[] = "Mozilla/5.0 (Series40; Nokia200/11.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S37713[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:2.0b11pre) Gecko/X Firefox/4.0b11pre";
const char S37714[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6.8; en; rv:2.0) Gecko/X Firefox/4.0";
const char S37715[] = "Mozilla/4.0 (compatible; Intel Mac OS X 10.6; rv:2.0b8) Gecko/X Firefox/4.0b8)";
const char S37716[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300T Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S37717[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300TG Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S37718[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300TL Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S37719[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Pad TF300T Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S37720[] = "Mozilla/5.0 (Linux; Android 4.1.2; Transformer Pad TF300T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S37721[] = "Mozilla/5.0 (Linux; Android 4.1.1; Transformer Pad TF300T Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S37722[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; ru; rv:1.8.1.12) Gecko/20080201 Firefox";
const char S37723[] = "Mozilla/5.0 (compatible; Firefox Addon; Windows XP 5.1)";
const char S37724[] = "Mozilla/4.0 (Firefox; Windows NT 5.1)";
const char S37725[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.25) Gecko/X AskTbFWV5/3.14.0.19709  Firefox[xSP_2:f862ceb31b6660fXb111b35a7e_208]  X  (.NET CLR 3.5.30729) FBSMTWB";
const char S37726[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.13) Gecko/X  Firefox  GTB7.1";
const char S37727[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.25) Gecko/X  Firefoxcontrol/5.00294  X  SearchToolbar/1.2";
const char S37728[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.25) Gecko/X  Firefox  (.NET CLR 3.5.30729)";
const char S37729[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.12) Gecko/X AskTbX-SD/3.14.1.20007  FirefoxBuild/13.00287  X";
const char S37730[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.13) Gecko/X  Firefox  ( .NET CLR 3.5.30729)";
const char S37731[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-GB; rv:1.8.1.6) Gecko/X Firefox /2.0.0.6";
const char S37732[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.7) Gecko/X Firefox_123/0.9.3";
const char S37733[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.452) Gecko/X Mnenhy/0.6.0.104";
const char S37734[] = "Mozilla/5.0 (Windows NT 5.1; rv:2.0) Gecko/X Firefox,##ProjectID10##2071####";
const char S37735[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; de; rv:1.9.2.6) Gecko/X Firefox/ Anonymisiert durch AlMiSoft Browser-Anonymisierer X";
const char S37736[] = "Mozilla/3.0 (Windows; U; Windows NT 5.1; en-US; rv:1.452) Gecko/X Mnenhy/0.6.0.104";
const char S37737[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; ASUS Transformer Pad TF700T Build/JOP40D)";
const char S37738[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ASUS Transformer Pad TF700T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37739[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Transformer Pad TF700T Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37740[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; ja-jp; ASUS Pad TF700T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37741[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/16.0.912.63) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S37742[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/16.0.912.63; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S37743[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/16.0.912.63; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S37744[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/16.0.912.75) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7";
const char S37745[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/16.0.912.75; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S37746[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/16.0.912.77) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S37747[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/16.0.912.75; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S37748[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/16.0.912.75)";
const char S37749[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/16.0.912.75; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S37750[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/16.0.912.75) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7";
const char S37751[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/16.0.912.75; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S37752[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/16.0.912.75; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S37753[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/16.0.912.63) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S37754[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Desire Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37755[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Desire Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37756[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; HTC Desire Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.1 Mobile Safari/534.30";
const char S37757[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Desire Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37758[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Desire Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37759[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC Desire V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37760[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC_Desire_V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37761[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; HTC_Desire_VC_T328d Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37762[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Desire Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37763[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Desire Build/IMM76L)";
const char S37764[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-id; HTC Desire VC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37765[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC_Desire_VC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37766[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Desire Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37767[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Desire V Build/IML74K)";
const char S37768[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; HTC Desire VC T328d Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37769[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ar-; HTC_Desire_V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37770[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Desire VC Build/IML74K)";
const char S37771[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Desire Build/mariusa v2.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37772[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; HTC_Desire_P Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37773[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Desire Build/mariusa v3.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37774[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; HTC_Desire_Q Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37775[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; HTC_Desire_200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37776[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Desire VC T328d Build/IML74K)";
const char S37777[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/12.0.742.91; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S37778[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/12.0.742.100) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30";
const char S37779[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/12.0.742.91)";
const char S37780[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/12.0.742.112)";
const char S37781[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 6.1; chromeframe/12.0.742.100; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)";
const char S37782[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/12.0.742.122) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S37783[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/12.0.742.122) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S37784[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/12.0.742.91; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729)";
const char S37785[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; chromeframe/12.0.742.100)";
const char S37786[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/12.0.742.112) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30";
const char S37787[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/12.0.742.91) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.91 Safari/534.30";
const char S37788[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/12.0.742.112) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30";
const char S37789[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/12.0.742.91; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S37790[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/12.0.742.100) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30";
const char S37791[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/12.0.742.122; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S37792[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; chromeframe/12.0.742.100; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)";
const char S37793[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B329 Safari/8536.25";
const char S37794[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B146 Safari/8536.25";
const char S37795[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B141 Safari/8536.25";
const char S37796[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B147 Safari/8536.25";
const char S37797[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I547 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37798[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SAMSUNG-SGH-I547 Build/JRO03L)";
const char S37799[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I547C Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37800[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I547C Build/JZO54K)";
const char S37801[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I547 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37802[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-I547 Build/JZO54K)";
const char S37803[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=comancheatt;devm=SAMSUNG-SGH-I547;man=samsung;brw=Android;shwd=480x800;shwa=480x763;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S37804[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SCH-I545 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S37805[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SCH-I545 4G Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S37806[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S37807[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SCH-I545L Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S37808[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; sdk Build/ICS_MR0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37809[] = "Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; sdk Build/ICS_MR0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37810[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; sdk Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37811[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; sdk Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37812[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Full AOSP on Rk29sdk Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37813[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Full AOSP on Rk29sdk Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37814[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; rk30sdk Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37815[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Full AOSP on Rk29sdk Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37816[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; sdk Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37817[] = "Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; google_sdk Build/ICS_MR0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37818[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; google_sdk Build/MR1)";
const char S37819[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; sdk Build/MR1)";
const char S37820[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; sdk Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37821[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; google_sdk Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37822[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Full AOSP on Rk29sdk Build/IML74K)";
const char S37823[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; google_sdk Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37824[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 BaDoinkDownloader_Android_2.0";
const char S37825[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 BaDoinkDownloader_AndroidTab_2.0";
const char S37826[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.2; sdk Build/ICS_MR0)";
const char S37827[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Full AOSP on Rk29sdk Build/IMM76D)";
const char S37828[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; mmi_apps_sdk Build/6.7.1_22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37829[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; rk29sdk Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37830[] = "Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; sdk Build/ICS_MR0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37831[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; rk30sdk Build/IMM76D)";
const char S37832[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SGH-M919 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S37833[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SGH-M919 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S37834[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A210 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37835[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; A210 Build/JRO03H)";
const char S37836[] = "stagefright/1.2 (Linux;Android 4.1.1 acer_A210)";
const char S37837[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A210 Build/JRO4HDb1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37838[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Safari/535.19 Silk-Accelerated=true";
const char S37839[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/1.0.154.36 Safari/525.19";
const char S37840[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/1, 6, 0, 4 Safari/533.3";
const char S37841[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/1, 6, 2, 0 Safari/533.3";
const char S37842[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; cs-CZ) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/1.0.154.48 Safari/525.19";
const char S37843[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/1.0.154.48 Safari/525.19";
const char S37844[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/1, 6, 0, 6 Safari/533.3";
const char S37845[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.19 (KHTML	 like Gecko) Chrome/1.0.154.59 Safari/525.19";
const char S37846[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.3 (KHTML, like Gecko) Chrome/1, 6, 0, 4 Safari/535.3";
const char S37847[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/1.0.862.0 Safari/535.2";
const char S37848[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9000 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37849[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9000 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37850[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9000 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37851[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9000 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/galaxysmtd";
const char S37852[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 4G Build/6.7.2-180_DR4-16_M2-33) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37853[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 4G Build/6.7.2-180_DR4-16_M2-20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37854[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37855[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 4G Build/6.7.2-180_DR4-16_M2-35) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37856[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 Build/6.7.2-180_DR4-16_M2-37) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37857[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 4G Build/6.7.2-180_DR4-16_M2-37) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37858[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; DROID4 Build/6.7.2-180_DR4-16_M2-37)";
const char S37859[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 Build/6.7.2-180_DR4-16_M2-33) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37860[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 4G Build/6.7.2-180_DR4-16_M2-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37861[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 4G Build/6.7.2-180_DR4-16_M2-26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37862[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 4G Build/6.7.2-180_DR4-16_M2-32) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37863[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 4G Build/6.7.2-180_DR4-16_M2-28) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37864[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 Build/6.7.2-180_DR4-16_M2-37) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S37865[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 Build/6.7.2-180_DR4-16_M2-35) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37866[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 Build/6.7.2-180_DR4-16_M2-37) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30";
const char S37867[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9195 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S37868[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9195/I9195XXUAMF6 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S37869[] = "Mozilla/5.0 (Linux; Android 4.2.2; es-es; SAMSUNG GT-I9195/I9195XXUAMG1 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S37870[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9195X Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S37871[] = "Mozilla/5.0 (Linux; Android 4.2.2; fr-fr; SAMSUNG GT-I9195/I9195XXUAMG6 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S37872[] = "Mozilla/5.0 (Linux; Android 4.2.2; es-us; SAMSUNG GT-I9195L Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S37873[] = "Mozilla/5.0 (Linux; Android 4.2.2; fr-fr; SAMSUNG GT-I9195/I9195XXUAMG5 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S37874[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-au; SAMSUNG GT-I9195T Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S37875[] = "Mozilla/5.0 (Linux; Android 4.2.2; es-es; SAMSUNG GT-I9195-ORANGE Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S37876[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:14.0) Gecko/X Firefox/14.0.1";
const char S37877[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:14.0) Gecko/X Firefox/14.0.1 Firefox/14.0.1";
const char S37878[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaX6-00.1/30.2.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.6 3gpp-gba";
const char S37879[] = "NokiaX6-00m/21.1.004 (SymbianOS/9.4; U; Series60/5.0 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Safari/525 3gpp-gba";
const char S37880[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaX6-00/21.0.004; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.5.2 3gpp-gba";
const char S37881[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaX6-00/40.0.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 Mobile Safari/533.4 3gpp-gba";
const char S37882[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaX6-00.1/40.2.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 Mobile Safari/533.4 3gpp-gba";
const char S37883[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1809c_v75_gq1008_9p017 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37884[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1809c_v75_jbl_253 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37885[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1809c_v75_gq1008_ov5647 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37886[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1809c_v75_gq1008_ov5647_exnote Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37887[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-gb; e1809c_v77_gq1008_5647 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37888[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; e1809c_v75_gq1008_9p017 Build/IML74K)";
const char S37889[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1809c_v75_jbl_5647 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37890[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1809c_v75_jbl_5647 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37891[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1809c_v75_jbl_253 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37892[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1809c_v75_gq1008_5640 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37893[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; e1809c_v75_gq1008_ov5647_uta Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37894[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; e1809c_v75_gq1008_ov5647 Build/IML74K)";
const char S37895[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1809c_v75_jbl_253_6628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37896[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1809c_v75_jdt1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37897[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1809c_v75_jbl_9p017 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37898[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1809c_v75_gq1008_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37899[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1809c_v75_gq1008_9p017_6628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37900[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1809c_v15_zlh_l328 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37901[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1809c_v75_gq1008_5647_6628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37902[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; el-gr; e1809c_v75_gq1008_ov5647_turbo_x Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37903[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1809c_v75_jbl_9p017_6628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37904[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; e1809c_v77_gq1008_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37905[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1809c_v75_jbl_253_tronix Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37906[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1809c_v75_jhgg_9p017 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37907[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; e1809c_v77_jbl_5647 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37908[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; e1809c_v75_jbl_5640 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37909[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; e1809c_v75_hrtm1quest Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37910[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; e1809c_v75_hlx1_5647 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37911[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1809c_v75_hlx1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37912[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; e1809c_v75_gq1008_5640 Build/IML74K)";
const char S37913[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1809c_v75_jbs1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S37914[] = "Opera/9.80 (Windows NT 5.1; U; en) Presto/2.9.168 Version/11.52";
const char S37915[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.9.168 Version/11.52";
const char S37916[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4661); ru) Presto/2.9.168 Version/11.52";
const char S37917[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.9.168 Version/11.52";
const char S37918[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.9.168 Version/11.52";
const char S37919[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.9.168 Version/11.52";
const char S37920[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.9.168 Version/11.52";
const char S37921[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.9.168 Version/11.52";
const char S37922[] = "Opera/9.80 (Windows NT 5.2; U; ru) Presto/2.9.168 Version/11.52";
const char S37923[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.9.168 Version/11.52";
const char S37924[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.9.168 Version/11.52";
const char S37925[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.9.168 Version/11.52";
const char S37926[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.9.168 Version/11.52";
const char S37927[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.9.168 Version/11.52";
const char S37928[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.9.168 Version/11.52";
const char S37929[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4832); ru) Presto/2.9.168 Version/11.52";
const char S37930[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.9.168 Version/11.52,gzip(gfe) (via translate.google.com)";
const char S37931[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.9.168 Version/11.52";
const char S37932[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.9.168 Version/11.52";
const char S37933[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.9.168 Version/11.52";
const char S37934[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4841); ru) Presto/2.9.168 Version/11.52";
const char S37935[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.9.168 Version/11.52";
const char S37936[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4871); ru) Presto/2.9.168 Version/11.52";
const char S37937[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.9.168 Version/11.52";
const char S37938[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; en) Opera 11.52";
const char S37939[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03791); ru) Presto/2.9.168 Version/11.52";
const char S37940[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.9.168 Version/11.52";
const char S37941[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.9.168 Version/11.52";
const char S37942[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.9.168 Version/11.52";
const char S37943[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4930); ru) Presto/2.9.168 Version/11.52";
const char S37944[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.9.168 Version/11.52";
const char S37945[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.9.168 Version/11.52";
const char S37946[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.9.168 Version/11.52";
const char S37947[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.9.168 Version/11.52";
const char S37948[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.9.168 Version/11.52";
const char S37949[] = "Mozilla/5.0 (Windows NT 5.1; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.52";
const char S37950[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.9.168 Version/11.52";
const char S37951[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03672); ru) Presto/2.9.168 Version/11.52";
const char S37952[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03649); ru) Presto/2.9.168 Version/11.52";
const char S37953[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03638); ru) Presto/2.9.168 Version/11.52";
const char S37954[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.9.168 Version/11.52";
const char S37955[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.9.168 Version/11.52";
const char S37956[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.9.168 Version/11.52";
const char S37957[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.9.168 Version/11.52";
const char S37958[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.9.168 Version/11.52";
const char S37959[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.9.168 Version/11.52";
const char S37960[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.9.168 Version/11.52";
const char S37961[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); MRSPUTNIK OW 2, 3, 0, 213; ru) Presto/2.9.168 Version/11.52";
const char S37962[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.9.168 Version/11.52";
const char S37963[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.9.168 Version/11.52";
const char S37964[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02552); ru) Presto/2.9.168 Version/11.52";
const char S37965[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.9.168 Version/11.52";
const char S37966[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5282); ru) Presto/2.9.168 Version/11.52";
const char S37967[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.9.168 Version/11.52,gzip(gfe)";
const char S37968[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5308); ru) Presto/2.9.168 Version/11.52";
const char S37969[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.9.168 Version/11.52";
const char S37970[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.9.168 Version/11.52";
const char S37971[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.9.168 Version/11.52";
const char S37972[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.9.168 Version/11.52";
const char S37973[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.9.168 Version/11.52";
const char S37974[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.9.168 Version/11.52";
const char S37975[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.9.168 Version/11.52";
const char S37976[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5328); ru) Presto/2.9.168 Version/11.52";
const char S37977[] = "Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6578); ru) Presto/2.9.168 Version/11.52";
const char S37978[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonMK16i Build/4.0.A.0.261) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37979[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonMK16i Build/4.0.1.A.0.169) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37980[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonMK16i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37981[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonMK16i Build/4.0.1.A.0.277) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37982[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonMK16i Build/4.0.A.2.335) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37983[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMK16i Build/4.0.A.2.364) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37984[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMK16i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37985[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MK16i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37986[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; MK16i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37987[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMK16i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37988[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; MK16i Build/4.0.2.A.0.62)";
const char S37989[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonMK16i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37990[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonMK16i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37991[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MK16i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37992[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MK16i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37993[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-za; SonyEricssonMK16iv Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37994[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMK16i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S37995[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MK16i Build/4.0.2.A.0.62) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S37996[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/11.10 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19";
const char S37997[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/11.10 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19";
const char S37998[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/11.10 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19,gzip(gfe) (via translate.google.com)";
const char S37999[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/11.10 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19,gzip(gfe)";
const char S38000[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7310 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S38001[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7310 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S38002[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7310 Build/IMM76D)";
const char S38003[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHW-M305W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S38004[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P7310 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S38005[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; GT-P7310 Build/HTJ85B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S38006[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P7310 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S38007[] = "Logos/0.9.4 (Linux; U; Android 4.0.3.0.2.0.1.0; de_DE; GT-P7310 Build/p5wifi-userdebug 4.0.3.0.2.0.1.0 OPENMASTER eng.cainm.X.X test-keys)";
const char S38008[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; GT-P7310 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/1.9.11;FBDM/{density=1.0,width=1280,height=752};FBLC/de_AT;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P7310;FBSV/4.0.4;]";
const char S38009[] = "com.google.android.youtube/4.1.23(Linux; U; Android 4.0.4; en_US; GT-P7310 Build/IMM76D) gzip";
const char S38010[] = "Logos/2.2.3 (Linux; U; Android 4.0.4; en-US; GT-P7310 Build/IMM76D.UELPB)";
const char S38011[] = "com.google.android.youtube/3.5.5(Linux; U; Android 4.0.4.0.4.0.4; nl_NL; GT-P7310 Build/IMM76D) gzip";
const char S38012[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7310 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38013[] = "Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-GB; GT-P7310 Build/IMM76D.XXLPO)";
const char S38014[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7310 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S38015[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7310 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S38016[] = "Groupon/2.2.2561 (Android 4.0.4; Samsung GT-P7310 / Samsung GT-P7310; )[preload=false;locale=en_US;clientidbase=android-samsung]";
const char S38017[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-P7310 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38018[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7310 Build/IMM76D; 1280*752) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S38019[] = "MediaPlayer/2002/4.2.3 (Android 4.0.4; GT-P7310; GT-P7310; samsung; ; ; en_GB)";
const char S38020[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7310 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S38021[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9A334 Safari/7534.48.3";
const char S38022[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9A405 Safari/7534.48.3";
const char S38023[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S38024[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Avant Browser)";
const char S38025[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Avant Browser)";
const char S38026[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Avant Browser)";
const char S38027[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; Avant Browser)";
const char S38028[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; Avant Browser)";
const char S38029[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; Avant Browser)";
const char S38030[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; Avant Browser; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Avant Browser)";
const char S38031[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; Avant Browser; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Avant Browser)";
const char S38032[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Avant Browser)";
const char S38033[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Avant Browser)";
const char S38034[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Avant Browser)";
const char S38035[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; Avant Browser; SV1)";
const char S38036[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; SV1; Avant Browser)";
const char S38037[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; Avant Browser; Avant Browser)";
const char S38038[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C; .NET4.0E; Avant Browser)";
const char S38039[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MAEM; .NET4.0C; Zune 4.7; Media Center PC 6.0; BOIE9;ENUS; Avant Browser)";
const char S38040[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-I9100 Build/JOP40D)";
const char S38041[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9100 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38042[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9100G Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38043[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-I9100 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38044[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9100 Build/JDQ39)";
const char S38045[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9100 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38046[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9100G Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38047[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9100 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38048[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9100G Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38049[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9100 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S38050[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9100G Build/JDQ39E)";
const char S38051[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9100 Build/ThunderBolt ROM V8 By ALHussein Mohammed) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38052[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:13.0) Gecko/X Firefox/13.0";
const char S38053[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:13.0) Gecko/X Firefox/13.0.1";
const char S38054[] = "Opera/9.80 (Windows NT 6.2) Presto/2.12.388 Version/12.10";
const char S38055[] = "Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.10";
const char S38056[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.10";
const char S38057[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.10";
const char S38058[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.10";
const char S38059[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.10";
const char S38060[] = "Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.10,gzip(gfe)";
const char S38061[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.10; SV1)";
const char S38062[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.10; Trident/4.0)";
const char S38063[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Alexa Toolbar)";
const char S38064[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; SV1)";
const char S38065[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; IEMB3)";
const char S38066[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; SV1)";
const char S38067[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.10; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)";
const char S38068[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.10; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S38069[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; SV1; SV1)";
const char S38070[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.10; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S38071[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; SV1; SV1)";
const char S38072[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; ALCATEL ONE TOUCH 991 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38073[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; ALCATEL ONE TOUCH 991D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38074[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; ALCATEL_ONE_TOUCH_991_Orange Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38075[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ALCATEL ONE TOUCH 991S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38076[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; ALCATEL_one_touch_991 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38077[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; ALCATEL ONE TOUCH 991T Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38078[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; ALCATEL_ONE_TOUCH_991A Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38079[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; ALCATEL ONE TOUCH 991A Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38080[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; ALCATEL_ONE_TOUCH_991 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38081[] = "SAMSUNG-GT-S5230/1.0 SHP/VPP/R5 Jasmine/0.8 NexPlayer/2.12 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S38082[] = "SAMSUNG-GT-S5230N-ORANGE/S5230NBVKD1 SHP/VPP/R5 Jasmine/0.8 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S38083[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Ubuntu/10.10 Chromium/8.0.552.224 Chrome/8.0.552.224 Safari/534.10";
const char S38084[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/540.0 (KHTML, like Gecko) Ubuntu/10.10 Chrome/8.1.0.0 Safari/540.0";
const char S38085[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Ubuntu/10.10 Chromium/8.0.552.237 Chrome/8.0.552.237 Safari/534.10";
const char S38086[] = "Mozilla/5.0 (Windows NT 6.0; rv:8.0.1) Gecko/X Firefox/8.0.1";
const char S38087[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.3) Gecko/X Firefox/8.0 (.NET CLR 3.5.30729)";
const char S38088[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:8.0.1) Gecko/X Firefox/8.0.1";
const char S38089[] = "Mozilla/5.0 (Windows NT 6.0; rv:8.0) Gecko/X Firefox/8.0 Firefox/8.0";
const char S38090[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:17.0) Gecko/17.0 Thunderbird/17.0";
const char S38091[] = "Mozilla/5.0 (Windows NT 5.1; rv:17.0) Gecko/X Thunderbird/17.0.8";
const char S38092[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Maxthon; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)";
const char S38093[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Maxthon; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729)";
const char S38094[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Maxthon; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)";
const char S38095[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; SV1; .NET CLR 1.1.4322; MAXTHON)";
const char S38096[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Maxthon; Win64; x64; Trident/4.0)";
const char S38097[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; SV1; Maxthon)";
const char S38098[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Maxthon; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S38099[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; SV1; MAXTHON)";
const char S38100[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; Maxthon; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)";
const char S38101[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; SV1; .NET CLR 1.1.4322; MAXTHON)";
const char S38102[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; MAXTHON)";
const char S38103[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; SV1; MAXTHON)";
const char S38104[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; Maxthon)";
const char S38105[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; Maxthon)";
const char S38106[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAXTHON)";
const char S38107[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; .NET CLR 2.0.50727; MAXTHON)";
const char S38108[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 6.1; Maxthon)";
const char S38109[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; Maxthon)";
const char S38110[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; Maxthon)";
const char S38111[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)";
const char S38112[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAXTHON)";
const char S38113[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; Trident/4.0; Maxthon)";
const char S38114[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MAXTHON)";
const char S38115[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; .NET CLR 1.1.4322; MAXTHON)";
const char S38116[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAXTHON)";
const char S38117[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)";
const char S38118[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)";
const char S38119[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; MAXTHON)";
const char S38120[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; Avant Browser; Maxthon)";
const char S38121[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; SV1; Maxthon)";
const char S38122[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/en-us  Safari/537.1";
const char S38123[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; Avant Browser; Maxthon)";
const char S38124[] = "Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.1; SV1; MAXTHON)";
const char S38125[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; SV1; TencentTraveler 4.0; Maxthon)";
const char S38126[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; Maxthon; Maxthon)";
const char S38127[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAXTHON)";
const char S38128[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Maxthon; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; EmbeddedWB 14.52 from: http://www.bsalsa.com/ EmbeddedWB 14.52; .NET CLR 2.0.50727; .NET4.0C; .NET4.0E; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; KBX)";
const char S38129[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; Maxthon; SV1)";
const char S38130[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; MAXTHON)";
const char S38131[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MAXTHON)";
const char S38132[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; Maxthon; Maxthon)";
const char S38133[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; Maxthon; Maxthon)";
const char S38134[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; GTB6.6; Maxthon; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C; MASN; AskTB5.6)";
const char S38135[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; Avant Browser; Maxthon)";
const char S38136[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; SV1; Maxthon)";
const char S38137[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; Avant Browser; Maxthon)";
const char S38138[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; Maxthon; Maxthon)";
const char S38139[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; Maxthon; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3)";
const char S38140[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MALN; MAXTHON)";
const char S38141[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MAXTHON)";
const char S38142[] = "Mozilla/5.0 (Linux;u;Android 2.3.7;zh-cn;) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Mobile Safari/533.1 (compatible; +http://www.baidu.com/search/spider.html)";
const char S38143[] = "Opera/9.64 (Windows NT 6.0; U; en) Presto/2.1.1";
const char S38144[] = "Opera/9.63 (Windows NT 6.0; U; en) Presto/2.1.1";
const char S38145[] = "Opera/9.61 (Windows NT 6.0; U; en) Presto/2.1.1";
const char S38146[] = "Opera/9.62 (Windows NT 6.0; U; en) Presto/2.1.1";
const char S38147[] = "Opera/9.64 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.1.1";
const char S38148[] = "Opera/9.60 (Windows NT 6.0; U; bg) Presto/2.1.1";
const char S38149[] = "Opera/9.63 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.1.1";
const char S38150[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Runnymede Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38151[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Runnymede Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38152[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Sensation XL with Beats Audio X315e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38153[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; tr-; HTC Sensation XL with Beats Audio X315e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38154[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Runnymede Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38155[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Runnymede Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38156[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC_SensationXL_Beats_X315b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38157[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC Sensation XL with Beats Audio X315b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38158[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_SensationXL_Beats_X315e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38159[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/SensationXL_Beats/1.05.161.4 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38160[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Sensation XL with Beats Audio X315e Build/GRJ90)";
const char S38161[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; HTC_SensationXL_Beats Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38162[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC_SensationXL_Beats-orange-LS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38163[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Sensation XL with Beats Audio X315e Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S38164[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-kr; HTC-X315E Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38165[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; Sensation XL with Beats Audio Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38166[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Sensation XL with Beats Audio X315b Build/GRJ90)";
const char S38167[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Sensation XL with Beats Audio X315e Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S38168[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Runnymede Build/GRJ90)";
const char S38169[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC Sensation XL with Beats Audio X315b Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S38170[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC_X315e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38171[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Sensation XL with Beats Audio X315b Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S38172[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC-X315E Build/GRJ90)";
const char S38173[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC X315e Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S38174[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC X315e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38175[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus 10 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S38176[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-66) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S38177[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT905 Build/9.8.2Q_SMUIRD-7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S38178[] = "Mozilla/5.0 (Linux; Android 4.1.1; XT907 Build/9.8.1Q_27-2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S38179[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows ME) Opera 7.11  [en]";
const char S38180[] = "Opera/7.50 (Windows ME; U) [en]";
const char S38181[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows ME) Opera 7.54 [en]";
const char S38182[] = "Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows ME) Opera 7.03 [de]";
const char S38183[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows ME) Opera 7.53 [en]";
const char S38184[] = "Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows ME) Opera 7.02 [en]";
const char S38185[] = "Opera 7.5 (Win ME): Opera/7.50 (Windows ME; U)";
const char S38186[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows ME) Opera 7.23 [ru]";
const char S38187[] = "Opera/7.22 (Windows ME; U)  [en]";
const char S38188[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30";
const char S38189[] = "Mozilla/5.0 (Windows 8) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30";
const char S38190[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.27 (KHTML, like Gecko) Chrome/12.0.712.0 Safari/534.27";
const char S38191[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/12.0.702.0 Safari/533.3 Awesomium/1.6.5";
const char S38192[] = "Mozilla/5.0 (Windows 8) AppleWebKit/534.30 (KHTML like Gecko) Chrome/12.0.742.112 Safari/534.30";
const char S38193[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38194[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M110S Build/GINGERBREAD)";
const char S38195[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38196[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38197[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHW-M110S Build/GINGERBREAD)";
const char S38198[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SHW-M110S Build/GINGERBREAD)";
const char S38199[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38200[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SHW-M110S Build/GINGERBREAD)";
const char S38201[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SHW-M110S Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38202[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; SHW-M110S Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38203[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; SHW-M110S Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38204[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SHW-M110S Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38205[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; SHW-M110S Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S38206[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; SHW-M110S Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S38207[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38208[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; SHW-M110S Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S38209[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38210[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SHW-M110S Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S38211[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M110S Build/??????????) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38212[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38213[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XT875 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38214[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT875 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38215[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID BIONIC Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38216[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38217[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22)";
const char S38218[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S38219[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38220[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID BIONIC Build/Blurry JB) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38221[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-R680 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 USCC-R680";
const char S38222[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-R680 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38223[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SCH-R680 Build/GINGERBREAD)";
const char S38224[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-R680 Build/GINGERBREAD)";
const char S38225[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:12.0) Gecko/X Firefox/12.0";
const char S38226[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:12.0) Gecko/X Firefox/12.0 Firefox/12.0";
const char S38227[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P925/V08g Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38228[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P925/V10a Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38229[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P925 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38230[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P925/V10d Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38231[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P925/V10e Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38232[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P925 Build/FRG83G)";
const char S38233[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P925/V09c Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38234[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-; LG-P925/V10e Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38235[] = "CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 2.2) lge LG-P925 SW:V20p";
const char S38236[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B329 Safari/8536.25";
const char S38237[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B146 Safari/8536.25";
const char S38238[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B144 Safari/8536.25";
const char S38239[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B141 Safari/8536.25";
const char S38240[] = "Mozilla/1.22 (compatible; MSIE 2.0; Windows 95)";
const char S38241[] = "Mozilla/5.0 (compatible; MSIE 2.0; Win95; Trident/2.0)";
const char S38242[] = "Mozilla/5.0 (compatible; MSIE 1.0; Win95; Trident/5.0)";
const char S38243[] = "Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:11.0) Gecko/X Firefox/11.0";
const char S38244[] = "Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:11.0) Gecko/X Firefox/11.0 Firefox/11.0";
const char S38245[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; SonyLT25i Build/9.1.A.0.490) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38246[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT25i Build/9.1.A.0.490) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38247[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; SonyLT25i Build/9.1.A.0.492) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38248[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LT25i Build/9.1.A.1.140)";
const char S38249[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; SonyLT25i Build/9.1.A.1.140) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38250[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; LT25i Build/9.1.A.1.140) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38251[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; SonyLT25i Build/9.1.A.1.142) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38252[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; LT25i Build/9.1.A.0.492) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38253[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-sg; SonyLT25i Build/9.1.A.1.145) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38254[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; LT25i Build/9.1.A.1.142) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38255[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-sg; LT25i Build/9.1.A.1.145) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38256[] = "SonyLT25i Build/9.1.A.1.140 stagefright/1.2 (Linux;Android 4.1.2)";
const char S38257[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S38258[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.163 Safari/535.19";
const char S38259[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S38260[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/180.1.1025.40 Safari/535.19";
const char S38261[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S38262[] = "Mozilla/5.0 (Macintosh; AMD Mac OS X 10_8_2) AppleWebKit/535.22 (KHTML, like Gecko) Chrome/18.6.872";
const char S38263[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/180.1.1025.40 Safari/535.19";
const char S38264[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S38265[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S38266[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9B206 Safari/7534.48.3";
const char S38267[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9B176 Safari/7534.48.3";
const char S38268[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9A405 Safari/7534.48.3";
const char S38269[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38270[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38271[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-I9103 Build/GINGERBREAD)";
const char S38272[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-I9103 Build/GINGERBREAD)";
const char S38273[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38274[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9103 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S38275[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9103 Build/GINGERBREAD)";
const char S38276[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9103 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S38277[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9103 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S38278[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9103 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S38279[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9103 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S38280[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38281[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9350; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.580 Mobile Safari/534.11+";
const char S38282[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-N8013 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S38283[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-N8013 Build/IMM76D)";
const char S38284[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-N8013 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S38285[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S38286[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P6800 Build/IMM76D)";
const char S38287[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S38288[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S38289[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; GT-P6800 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S38290[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S38291[] = "Logos/2.1.2 (Linux; U; Android 4.0.4; cs-CZ; GT-P6800 Build/cm_p6800-userdebug 4.0.4 IMM76L eng.locerra.X.X test-keys)";
const char S38292[] = "Logos/2.1.2 (Linux; U; Android 4.0.4; en-US; GT-P6800 Build/IMM76D.DXLP7)";
const char S38293[] = "Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S38294[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38295[] = "com.google.android.apps.plus/X (Linux; U; Android 4.0.4; en_US; GT-P6800; Build/IMM76D) (gzip)";
const char S38296[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S38297[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D, compatible iPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S38298[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6800) antivirusHttpClient/2.0 (KHTML, like Gecko) Version/Version 2.3/Antivirus pro";
const char S38299[] = "Logos/2.2.3 (Linux; U; Android 4.0.4; en-GB; GT-P6800 Build/IMM76D.XXLQ1)";
const char S38300[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S38301[] = "Mozilla/5.0 (Linux; U; Android 4.0.41.0; en-gb; GT-P6800) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S38302[] = "Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 4.0.4; SAMSUNG GT-P6800 Build/ICE_CREAM_SANDWICH_MR1);";
const char S38303[] = "Logos/2.2.3 (Linux; U; Android 4.0.4; pt-BR; GT-P6800 Build/IMM76D.UBLP5)";
const char S38304[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6800 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S38305[] = "Logos/2.2.3 (Linux; U; Android 4.0.4; zh-HK; GT-P6800 Build/IMM76D.ZSLP7)";
const char S38306[] = "TapSonic 1.4.0 (Android 4.0.4 , GT-P6800, samsung)";
const char S38307[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6800 Build/IMM76D) Daily Mail Online/1.8.3";
const char S38308[] = "Vyrso/2.2.7 (Linux; U; Android 4.0.4; en-GB; GT-P6800 Build/SkyHigh ROM P6800DXLP8 ICS v3.5)";
const char S38309[] = "Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-US; GT-P6800 Build/IMM76D.DDLP5)";
const char S38310[] = "Groupon/2.1.2486 (Android 4.0.4; Samsung GT-P6800 / Samsung GT-P6800; )[preload=false;locale=en_GB;clientidbase=android-samsung]";
const char S38311[] = "Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-GB; GT-P6800 Build/IMM76D.DXLP8)";
const char S38312[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 parameters/{scr=1232_800,cm=1,ql=l,imei=X,bm=1,eng=adrmini15,mdm=1,mdl=GT-P6800,ps=1} tiantian(securitypay) tiantian(tenpay)";
const char S38313[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GT-P6800 Build/IMM76D) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 baidubrowser/061_0.41.3.2_diordna_2321_008/gnusmas_51_4.0.4_0086P-TG/1092a/E46AA9836BAFXF4290E099E6A%7CX/1";
const char S38314[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P6800 Build/Xqm) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S38315[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 NAVER(inapp; cafe; 100; 2.9.10) com.nhn.android.navercafe/2.9.10 (Linux; U; Android 4.0.4; ko-kr; GT-P6800 Build/IMM76D; samsung) 800X1232 samsung GT-P6800";
const char S38316[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6800 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38317[] = "nApps (Android 4.0.4; GT-P6800; androidapp.lineplay; 1.2.0)";
const char S38318[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; myTouch_4G_Slide Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38319[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; myTouch_4G_Slide Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S38320[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; myTouch_4G_Slide Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38321[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;myTouch_4G_Slide Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38322[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; myTouch_4G_Slide Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38323[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; myTouch_4G_SlideBuild/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38324[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; myTouch_4G_Slide Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38325[] = "Logos/0.9.4 (Linux; U; Android 2.3.4; en_US; myTouch_4G_Slide Build/GRJ22)";
const char S38326[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; myTouch_4G_Slide Build/GRJ22)";
const char S38327[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; myTouch_4G_Slide Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38328[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; myTouch_4G_Slide Build/GRJ22) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S38329[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; myTouch_4G_Slide Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38330[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A403 Safari/7534.48.3";
const char S38331[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A406 Safari/7534.48.3";
const char S38332[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A523 Safari/7534.48.3";
const char S38333[] = "Mozilla/5.0 (iPod; CPU iPhone OS 601 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/21.0.1180.82 Mobile/10A523 Safari/7534.48.3";
const char S38334[] = "Mozilla/5.0 (iPod; CPU iPhone OS 60 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/21.0.1180.82 Mobile/10A403 Safari/7534.48.3";
const char S38335[] = "Mozilla/5.0 (Windows NT 5.0; rv:12.0) Gecko/X Firefox/12.0";
const char S38336[] = "Mozilla/5.0 (Windows NT 5.0; rv:12.0) Gecko/X Firefox/12.0 Firefox/12.0";
const char S38337[] = "NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S38338[] = "NokiaC2-01/5.0 (11.20) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-01";
const char S38339[] = "NokiaC2-01/5.0 (11.20) Profile/MIDP-2.1 Configuration/CLDC-1.1 NokiaC2-01/5.0 (11.20) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S38340[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HUAWEI-M860 Build/ERE27) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S38341[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HUAWEI-M860 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38342[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; M860 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 androidAppVersion/2.0.3 AndroidWebkitWrapper";
const char S38343[] = "Huawei-M860/M860V100R001C153B251SP03 Player/OpenCORE-Player/2.02 (Linux;Android 2.1-update1)";
const char S38344[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; M860 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38345[] = "Huawei-HUAWEI-M860/M860V100R001C177B261 Player/OpenCORE-Player/2.02 (Linux;Android 2.1-update1)";
const char S38346[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; M860 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38347[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; M860 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 androidAppVersion/2.1 AndroidWebkitWrapper";
const char S38348[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; M860 Build/ERE27)";
const char S38349[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; HUAWEI-M860 Build/ERE27)";
const char S38350[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; M860 Build/ERE27) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S38351[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; ADR6200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 MobileSafari/530.17";
const char S38352[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; ADR6200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38353[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; ADR6200 Build/Flash2Talk V3) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38354[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Desire_A8181 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38355[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-tw; HTC_Desire_A8181 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38356[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; desire_A8181 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38357[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us;ADR6200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38358[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Desire 1.24.161.3 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38359[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Desire 1.15.161.4 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38360[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC_Desire Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38361[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; HTC Desire 1.19.161.5 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38362[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Bravo Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38363[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; de-at; HTC Desire 1.21.163.2 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38364[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Desire Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38365[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; HTCX06HT Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38366[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; HTC_Desire_A8183 V1.16.841.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38367[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; HTC_Desire_A8183 V1.15.841.14 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38368[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; HTC_Desire_A8183 V1.25.841.5 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38369[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; HTC_Desire_A8183 V1.15.841.13 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38370[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Desire 1.15.161.3 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38371[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-gb; HTC Bravo Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38372[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HTC Bravo_C Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38373[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC_Desire-orange-LS Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38374[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; ADR6200Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38375[] = "Mozilla/5.0 (Linux; U;Android 2.1; en-us; ADR6200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38376[] = "Mozilla/5.0 (Linux; U; Android 2.1; -us; ADR6200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38377[] = "Mozilla/5.0 (Linux; U; Android2.1; en-us; ADR6200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38378[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; HTC Desire Build/ERE27)";
const char S38379[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; HTC Desire Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S38380[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; HTC Desire 1.19.161.5 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.";
const char S38381[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:12.0) Gecko/X Firefox/12.0";
const char S38382[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AT300SE Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S38383[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AT300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S38384[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; AT300SE Build/JRO03C)";
const char S38385[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; AT300 Build/JRO03C)";
const char S38386[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LS840 Build/ZVI.IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38387[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LS840 Build/ZVK.IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38388[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-LS840 Build/ZVK.IMM76D)";
const char S38389[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LS840 Build/ZVJ.IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38390[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LS840 Build/ZVM.IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38391[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38392[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38393[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38394[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38395[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ZTE V768 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38396[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; ZTE V768 Build/GINGERBREAD)";
const char S38397[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ZTE V768 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S38398[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us;ZTE V768 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38399[] = "Mozilla/5.0 (Linux; U; Android2.3.7; en-us; ZTE V768 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38400[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ZTEV768 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38401[] = "Mozilla/5.0 (Linux; U;Android 2.3.7; en-us; ZTE V768 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38402[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ZTE V768Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38403[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S38404[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505G Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S38405[] = "Opera/9.80 (Windows NT 6.1; U; en) Presto/2.6.31 Version/10.70";
const char S38406[] = "Acoon v4.9.5 (www.acoon.de)";
const char S38407[] = "Acoon v4.10.1 (www.acoon.de)";
const char S38408[] = "Mozilla/5.0 (compatible; AcoonBot/4.11.0; +http://www.acoon.de/robot.asp)";
const char S38409[] = "Mozilla/5.0 (compatible; AcoonBot/4.11.1; +http://www.acoon.de/robot.asp)";
const char S38410[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Ubuntu/10.10 Chromium/10.0.648.133 Chrome/10.0.648.133 Safari/534.16";
const char S38411[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Ubuntu/10.10 Chromium/10.0.648.127 Chrome/10.0.648.127 Safari/534.16";
const char S38412[] = "Mozilla/5.0 (X11; U; Linux armv7l; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Ubuntu/10.10 Chromium/10.0.648.205 Chrome/10.0.648.205 Safari/534.16";
const char S38413[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2";
const char S38414[] = "Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.106 Safari/535.2";
const char S38415[] = "Mozilla/5.0 (X11; Jolicloud Linux i686) AppleWebKit/535.2 (KHTML, like Gecko) Joli OS/1.2 Chromium/15.0.874.106 Chrome/15.0.874.106 Safari/535.2";
const char S38416[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S38417[] = "Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/15.0.1410.63 Safari/537.31";
const char S38418[] = "Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/15.0.1410.63 Safari/537.31";
const char S38419[] = "Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/15.0.1410.63 Safari/537.31";
const char S38420[] = "Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/15.0.1410.63 Safari/537.31";
const char S38421[] = "Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/15.0.1410.63 Safari/537.31";
const char S38422[] = "Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/15.0.1410.63 Safari/537.31";
const char S38423[] = "Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/15.0.1410.63 Safari/537.31";
const char S38424[] = "Mozilla/4.7 (compatible; OffByOne; Windows 2000) Webster Pro V3.4";
const char S38425[] = "ICE Browser/5.05 (Java 1.4.0; Windows 2000 5.0 x86)";
const char S38426[] = "Mozilla/4.7 (compatible; OffByOne; Windows 2000)";
const char S38427[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PantechP9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38428[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PantechP9070 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38429[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; PantechP9070 Build/GRJ90)";
const char S38430[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; PantechP9070 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38431[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 2.3.5;dev=presto;devm=PantechP9070;man=PANTECH;brw=Android;shwd=480x800;shwa=480x763;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S38432[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST15i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38433[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyEricssonST15i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38434[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST15i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38435[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST15i Build/4.1.B.0.587)";
const char S38436[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyEricssonST15a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38437[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hr-hr; SonyEricssonST15iv Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38438[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST15a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38439[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; ST15a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38440[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9.2.28) Gecko/X Firefox/3.6.28";
const char S38441[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Kindle Fire Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38442[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Kindle_Fire Build/GINGERBREAD; 1024*600) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S38443[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Kindle Fire Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30";
const char S38444[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; Kindle Fire Build/GINGERBREAD)";
const char S38445[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; Kindle Fire Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38446[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; Kindle Fire Build/GRK39F)";
const char S38447[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; md-us; Kindle Fire Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38448[] = "Mozilla/4.0 (compatible; MSIE 6.0b; Windows NT 4.0)";
const char S38449[] = "Mozilla/4.0 (compatible; MSIE 6.0b; Windows NT 4.0; .NET CLR 1.0.2914)";
const char S38450[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.0; SV1)";
const char S38451[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)";
const char S38452[] = "Mozilla/5.0 (compatible; MSIE 6.0; WinNT4.0; Trident/3.0)";
const char S38453[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.0; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S38454[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S38455[] = "Safari/6534.59.8 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro7%2C1)";
const char S38456[] = "Safari/6534.59.8 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (iMac10%2C1)";
const char S38457[] = "Safari/6534.59.8 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBook4%2C1)";
const char S38458[] = "Safari/6534.59.8 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (Macmini3%2C1)";
const char S38459[] = "Safari/6534.59.8 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacPro1%2C1)";
const char S38460[] = "Safari/6534.59.8 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookAir3%2C2)";
const char S38461[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5";
const char S38462[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.53 Safari/536.5";
const char S38463[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.53 Safari/536.5";
const char S38464[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.53 Safari/536.5";
const char S38465[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_2) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.53 Safari/536.5";
const char S38466[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S38467[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S38468[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_5) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S38469[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/19.0.648.205 Safari/534.16";
const char S38470[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/19.0.648.205 Safari/534.16";
const char S38471[] = "Mozilla/5.0 (Series40; Nokia305/03.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S38472[] = "Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Series40; Nokia305/03.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S38473[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38474[] = "Mozilla/5.0 (Linux; Android 4.1.1; One X Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38475[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One X Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38476[] = "Mozilla/5.0 (Linux; Android 4.1.2; One X Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38477[] = "Mozilla/5.0 (Linux; Android 4.1.1; EndeavorU Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38478[] = "Mozilla/5.0 (Linux; Android 4.1.9; HTC One X Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38479[] = "Mozilla/5.0 (Linux; Android 4.1.2; EndeavorU Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38480[] = "Mozilla/5.0 (Linux; Android 4.1.5; HTC One X Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38481[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One X Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S38482[] = "Mozilla/5.0 (Linux; Android 4.1.1;  HTC One X Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S38483[] = "Mozilla/5.0 (Linux; Android 4.1.2; One X Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S38484[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1";
const char S38485[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1";
const char S38486[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1,gzip(gfe)";
const char S38487[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1";
const char S38488[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1";
const char S38489[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.803.0 Safari/535.1";
const char S38490[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.814.0 Safari/535.1";
const char S38491[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1;  http://www.changedetection.com/bot.html )";
const char S38492[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; CollapsarWEB qihoobot@qihoo.net)";
const char S38493[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; CrystalSemanticsBot http://www.crystalsemantics.com/service-navigation/imprint/useragent/)";
const char S38494[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; QihooBot 1.0 qihoobot@qihoo.net)";
const char S38495[] = "Mozilla/6.0 (MSIE 6.0; Windows NT 5.1; Searcharoo.NET Robot)";
const char S38496[] = "Mozilla/6.0 (MSIE 6.0; Windows NT 5.1; Searcharoo.NET; robot)";
const char S38497[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; maxamine.com-robot)";
const char S38498[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; .NET CLR 1.1.4322; AdSonar Bot)";
const char S38499[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727) yolinkBot";
const char S38500[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://www.changedetection.com/bot.html )";
const char S38501[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; QihooBot 1.0  qihoobot@qihoo.net)";
const char S38502[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; CrystalSemanticsBot http://www.crystalsemantics.com/user-agent/)";
const char S38503[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P930/V10f Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38504[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P930 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38505[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P930/V10j Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38506[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; LG-P930/V11a Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38507[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; LG-P930 Build/GRJ90)";
const char S38508[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LogicPD Zoom2 Build/ERD79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38509[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Nook Color (zoom2) Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38510[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; NookColor Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38511[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; NOOK BNRV200 Build/ERD79) Apple WebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38512[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; BNRV200 Build/MASTER)";
const char S38513[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; NOOK BNRV200 Build/ERD79 1.4.3) Apple WebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38514[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; NOOKcolor Build/MASTER)";
const char S38515[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; NOOK BNRV200Build/ERD79 1.4.3) Apple WebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38516[] = "Mozilla/5.0 (Linux; U; Android2.2.1; en-us; NOOK BNRV200 Build/ERD79 1.4.3) Apple WebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38517[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; BNRV200 Build/MASTER) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S38518[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; NOOK BNRV200 Build/ERD79 1.4.3) Apple WebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S38519[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; NOOK BNRV200 Build/ERD79) Apple WebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S38520[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; BNRV200 Build/MASTER) WWReader/1.5.0 build 800";
const char S38521[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; NOOK BNRV200 Build/ERD791.4.3) Apple WebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38522[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; NOOK BNRV200 Build/ERD79 1.4.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38523[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Huawei-U8652 Build/HuaweiU8652B840) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38524[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Huawei-U8652 Build/HuaweiU8652B855) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38525[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-nz; U8652-51 Build/HuaweiU8652-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38526[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-mx; U8652-51 Build/HuaweiU8652-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38527[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Huawei-U8652 Build/HuaweiU8652B855)";
const char S38528[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; U8652 Build/HuaweiU8652) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38529[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; Huawei-U8652 Build/HuaweiU8652B840)";
const char S38530[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Huawei-U8652 Build/HuaweiU8652) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38531[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U8652-51 Build/HuaweiU8652-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38532[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; Huawei-U8652 Build/HuaweiU8652)";
const char S38533[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT28at Build/6.1.C.1.105) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38534[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT28i Build/6.1.E.2.68) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38535[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT28h Build/6.1.E.3.7) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38536[] = "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.1) KomodiaBot/1.0";
const char S38537[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Banca Caboto s.p.a.; InfoPath.1; Banca Caboto s.p.a.)";
const char S38538[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.1; Banca Caboto s.p.a.)";
const char S38539[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; InfoPath.1; Banca Caboto s.p.a.)";
const char S38540[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Banca Caboto s.p.a.; InfoPath.1)";
const char S38541[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; botw; .NET CLR 2.0.50727; .NET CLR 1.1.4322; InfoPath.3; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; botw)";
const char S38542[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-T759 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38543[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;SGH-T759 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38544[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-T759 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S38545[] = "Mozilla/5.0 (Linux; U; Android2.3.3; en-us; SGH-T759 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38546[] = "Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; SGH-T759 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38547[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-T759Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38548[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; SGH-T759 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38549[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-T759 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 androidAppVersion/2.0.3 AndroidWebkitWrapper";
const char S38550[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SGH-T759 Build/GINGERBREAD)";
const char S38551[] = "Logos/0.9.4 (Linux; U; Android 2.3.3; en_US; SGH-T759 Build/GINGERBREAD.UVKE8)";
const char S38552[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SGH-T759 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38553[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-T759 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S38554[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; Avant Browser [avantbrowser.com]; Hotbar 4.4.5.0)";
const char S38555[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; Avant Browser; Avant Browser)";
const char S38556[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7D11";
const char S38557[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B367";
const char S38558[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko)";
const char S38559[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2_1 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B405";
const char S38560[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2_1 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko)";
const char S38561[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko)";
const char S38562[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B500";
const char S38563[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B367 Safari/528.16 NightBrowser/1.0";
const char S38564[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B367 Safari/531.21.10";
const char S38565[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2_1 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B405 Safari/531.21.10";
const char S38566[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B500 Safari/531.21.10";
const char S38567[] = "Mozilla/5.0(iPad; U; iPhone OS 3_2; en-us) AppleWebKit/531.21.10 Mobile/7B314";
const char S38568[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 4.0)";
const char S38569[] = "Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 4.0)";
const char S38570[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 4.0; Alexa Toolbar)";
const char S38571[] = "Mozilla/5.0 (compatible;MSIE 5.0;  Windows NT 4.0; 360safe)";
const char S38572[] = "Mozilla/4.0 (compatible; MSIE 5.05; Windows NT 4.0)";
const char S38573[] = "Mozilla/3.0 (compatible;MSIE 5.0;  Windows NT 4.0; 360safe)";
const char S38574[] = "Mozilla/5.0 (compatible; MSIE 5.01; Windows NT 4.0)";
const char S38575[] = "Mozilla/5.0 (compatible; MSIE 5.0; WinNT4.0; Trident/3.0)";
const char S38576[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; T-Mobile myTouch Q Build/HuaweiU8730) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38577[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; T-Mobile myTouch Q Build/HuaweiU8730)";
const char S38578[] = "Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; T-Mobile myTouch Q Build/HuaweiU8730) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38579[] = "Mozilla/5.0 (Linux; U; Android2.3.6; en-us; T-Mobile myTouch Q Build/HuaweiU8730) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38580[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; T-Mobile myTouch Q Build/HuaweiU8730) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S38581[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; T-MobilemyTouch Q Build/HuaweiU8730) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38582[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us;T-Mobile myTouch Q Build/HuaweiU8730) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38583[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; T-Mobile myTouch QBuild/HuaweiU8730) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38584[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; T-Mobile myTouchQ Build/HuaweiU8730) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38585[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-I897 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38586[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SAMSUNG-SGH-I897 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38587[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SAMSUNG-SGH-I897 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38588[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SAMSUNG-SGH-I897 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38589[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SGH-I897 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38590[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SGH-I897 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38591[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-I897 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38592[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-I897 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38593[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-I897 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38594[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SAMSUNG-SGH-I897 Build/GINGERBREAD)";
const char S38595[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-I897 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38596[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SAMSUNG-SGH-I897 Build/GINGERBREAD)";
const char S38597[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-I897 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38598[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SGH-I897 Build/GRJ22)";
const char S38599[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SAMSUNG-SGH-I897 Build/GINGERBREAD)";
const char S38600[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; SGH-I897 Build/GINGERBREAD)";
const char S38601[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SGH-I897 Build/GINGERBREAD)";
const char S38602[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SGH-I897 Build/GRJ22)";
const char S38603[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-I897 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38604[] = "Mozilla/5.0 (Windows NT 5.2; rv:7.0.1) Gecko/X Firefox/7.0.1,gzip(gfe) (via translate.google.com)";
const char S38605[] = "Mozilla/5.0 (Windows NT 5.1) Gecko/X Firefox/7.0.1";
const char S38606[] = "Mozilla/5.0 (Windows NT 5.1; rv:7.0.1) Gecko/X Firefox/7.0.1,gzip(gfe)";
const char S38607[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:7.0a1) Gecko/X Firefox/7.0a1";
const char S38608[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:7.0a1) Gecko/X Firefox/7.0a1";
const char S38609[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LT28at Build/6.0.C.1.257) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38610[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SonyEricssonLT28at Build/6.0.C.1.257) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38611[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-; SonyEricssonLT28at Build/6.0.C.1.257) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38612[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-; LT28at Build/6.0.C.1.257) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38613[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; LT28at Build/6.0.C.1.257)";
const char S38614[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; SonyEricssonLT28i Build/6.0.F.0.72) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38615[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-ca; SonyEricssonLT28i Build/6.0.F.0.111) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38616[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; LT28i Build/6.0.F.0.111)";
const char S38617[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-ca; LT28i Build/6.0.F.0.111) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38618[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; LT28i Build/6.0.F.0.72) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38619[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT28i Build/6.0.F.0.111) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S38620[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HUAWEI U8818 Build/HuaweiU8818) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38621[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HUAWEI U8815 Build/HuaweiU8815) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38622[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HUAWEI U8815 Build/HuaweiU8815C02B936) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38623[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; HUAWEI U8815N Build/HuaweiU8815NC02B936) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38624[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; HUAWEI U8815 Build/HuaweiU8815C02B937) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38625[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HUAWEI U8815 Build/HuaweiU8815C02B952) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38626[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HUAWEI U8818 Build/HuaweiU8818)";
const char S38627[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; HUAWEI U8815N Build/HuaweiU8815NC02B942) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38628[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HUAWEI U8815 Build/HuaweiU8815)";
const char S38629[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; HUAWEI Ascend G 300 Build/HuaweiU8815) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38630[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI U8818 Build/HuaweiU8818) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S38631[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HUAWEI U8815 Build/HuaweiU8815C02B960) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38632[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; HUAWEI U8815N Build/HuaweiU8815NC02B943) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38633[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Ascend G300 Build/HuaweiU8815) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38634[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Ascend G300 Build/HuaweiU8815C02B952) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38635[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HUAWEI U8815N Build/HuaweiU8815NC02B942)";
const char S38636[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; U8815-51 Build/HuaweiU8815) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38637[] = "BlackBerry9105/5.0.0.884 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/603";
const char S38638[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19";
const char S38639[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19";
const char S38640[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; Valve Steam GameOverlay/2026; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S38641[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; Valve Steam GameOverlay/2005; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S38642[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; Valve Client/0; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S38643[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; Valve Steam GameOverlay/2095; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S38644[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; Valve Steam GameOverlay/2109; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S38645[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; Valve Steam GameOverlay/2102; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S38646[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; Valve Steam GameOverlay/2125; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S38647[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/18.0 BIDUPlayerBrowser/2.x Safari/534.7";
const char S38648[] = "Mozilla/5.0 (Windows NT 6.2; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/18.0 BIDUPlayerBrowser/2.x Safari/534.7";
const char S38649[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; Valve Steam Tenfoot/X; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S38650[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; Valve Steam GameOverlay/X; ) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Safari/535.19";
const char S38651[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; Valve Steam Client/X; ) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Safari/535.19";
const char S38652[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38653[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38654[] = "Mozilla/5.0 (Linux; Android 4.1.1_????-??????; GT-I8190 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38655[] = "Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 4.0) Opera 7.0 [en]";
const char S38656[] = "Opera/7.0 (Windows NT 4.0; U) [de]";
const char S38657[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.0) Opera 7.23 [de]";
const char S38658[] = "Opera/7.10 (Windows NT 4.0; U) [de]";
const char S38659[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.0) Opera 7.11 [de]";
const char S38660[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-LG855 Build/GRJ22)";
const char S38661[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LG855 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38662[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LG855 Build/CTMod 3.6.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38663[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LG855 Build/CTMod 3.68) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38664[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LG855 Build/CTMod 3.75 Final) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38665[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LG855 Build/CTMod 3.80) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38666[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11";
const char S38667[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11";
const char S38668[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_5) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11";
const char S38669[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11";
const char S38670[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11";
const char S38671[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11";
const char S38672[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11,gzip(gfe) (via translate.google.com)";
const char S38673[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_2) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11";
const char S38674[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.43 Safari/536.11";
const char S38675[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/20.0.648.205 Safari/534.16";
const char S38676[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/20.0.648.205 Safari/534.16";
const char S38677[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B329 Safari/8536.25";
const char S38678[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B146 Safari/8536.25";
const char S38679[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B141 Safari/8536.25";
const char S38680[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B147 Safari/8536.25";
const char S38681[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Ally Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38682[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; Ally Build/FRG83D)";
const char S38683[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; Ally Build/FRG83G)";
const char S38684[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Ally Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38685[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Ally Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38686[] = "Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; Ally Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38687[] = "Mozilla/5.0 (Linux; U;Android 2.2.2; en-us; Ally Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38688[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;Ally Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38689[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Ally Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S38690[] = "Mozilla/5.0 (Linux; U; Android2.2.2; en-us; Ally Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38691[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; AllyBuild/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38692[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Ally Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S38693[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;Ally Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38694[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; AllyBuild/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38695[] = "Mozilla/5.0 (Linux; U; Android2.2.1; en-us; Ally Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38696[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Ally Build/FRG83G) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S38697[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; vi-vn; GT540 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38698[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT540 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38699[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT540 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38700[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; GT540 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38701[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; GT540 Build/GRI40)";
const char S38702[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LGL35G/V100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38703[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LGL35G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38704[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LGL35G Build/GRK39F)";
const char S38705[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en; LGL35G/V100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38706[] = "Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.6.30 Version/10.60";
const char S38707[] = "Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.6.30 Version/10.61";
const char S38708[] = "Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.6.30 Version/10.62";
const char S38709[] = "Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.6.30 Version/10.63";
const char S38710[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03686); ru) Presto/2.6.30 Version/10.63";
const char S38711[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.6.30 Version/10.63";
const char S38712[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.6.30 Version/10.63";
const char S38713[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.6.30 Version/10.63";
const char S38714[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.6.30 Version/10.62";
const char S38715[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-I800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38716[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38717[] = "Logos/0.9.3 (Linux; U; Android 2.3.5; en_US; SCH-I800 Build/GINGERBREAD.EI04)";
const char S38718[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ko-; SCH-I800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38719[] = "com.google.android.youtube/2.3.4(Linux; U; Android 2.3.5; en_US; SCH-I800 Build/GINGERBREAD) gzip";
const char S38720[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SCH-I800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38721[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I800 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30";
const char S38722[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SCH-I800 Build/GINGERBREAD)";
const char S38723[] = "Mozilla/5.0 (Linux; U;Android 2.3.5; en-us; SCH-I800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38724[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S38725[] = "Mozilla/5.0 (Linux; U; Android2.3.5; en-us; SCH-I800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38726[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I800Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38727[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us;SCH-I800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38728[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows 2000) Opera 6.0 [en]";
const char S38729[] = "Opera/6.05 (Windows 2000; U)";
const char S38730[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows 2000) Opera 6.03 [en]";
const char S38731[] = "Mozilla/5.0 (Windows 2000; U) Opera 6.03 [en]";
const char S38732[] = "Opera/6.0 (Windows 2000; U) [fr]";
const char S38733[] = "Mozilla/4.78 (Windows 2000; U) Opera 6.01 [en]";
const char S38734[] = "Opera/6.01 (Windows 2000; U) [de]";
const char S38735[] = "Opera/6.04 (Windows 2000; U)  [en]";
const char S38736[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SAMSUNG-SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38737[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SAMSUNG-SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38738[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SAMSUNG-SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38739[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38740[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-I997 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38741[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38742[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38743[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SAMSUNG-SGH-I997 Build/GINGERBREAD)";
const char S38744[] = "Mozilla/5.0 (Linux; U; Android 2.3.6 UCLB3; en-us; SAMSUNG-SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38745[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38746[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38747[] = "Mozilla/5.0 (Linux; U; Android XXJVT 2.3.5; en-us; SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38748[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SAMSUNG-SGH-I997 Build/GINGERBREAD)";
const char S38749[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SAMSUNG-SGH-I997 Build/GINGERBREAD)";
const char S38750[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; SGH-I997 Build/GINGERBREAD)";
const char S38751[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SAMSUNG-SGH-I997 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S38752[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SAMSUNG-SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38753[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Huawei-U8665 Build/HuaweiU8665B037) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38754[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; Huawei-U8665 Build/HuaweiU8665B037)";
const char S38755[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; Huawei-U8665 Build/HuaweiU8665B037) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38756[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 2.3.6;dev=hwu8665;devm=Huawei-U8665;man=HUAWEI;brw=Android;shwd=320x480;shwa=320x455;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S38757[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; fr; rv:1.8.1) VoilaBot BETA 1.2 (support.voilabot@orange-ftgroup.com)";
const char S38758[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1;fr;rv:1.8.1) VoilaBotCollector BETA 0.1  (http://www.voila.com/)";
const char S38759[] = "Opera/9.80 (BREW; Opera Mini/6.0.3/27.2338; U; en) Presto/2.8.119 320X240 Samsung SCH-U485";
const char S38760[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P5210 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S38761[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P5210 Build/JDQ39)";
const char S38762[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-P5210XD1 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S38763[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:11.0) Gecko/X Firefox/11.0 CometBird/11.0";
const char S38764[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:11.0) Gecko/X Firefox/11.0 CometBrowser/11.0";
const char S38765[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S38766[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC_PN071 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S38767[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One 801e Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S38768[] = "Mozilla/5.0 (Linux; Android 4.2.2; One Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S38769[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One dual sim Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S38770[] = "Mozilla/5.0 (Linux; Android 4.2.1; HTC_One Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S38771[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTCONE Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S38772[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S38773[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S38774[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100T Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S38775[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S38776[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100G Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S38777[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100M Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S38778[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100G Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S38779[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100T Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S38780[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S38781[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; HTC One X Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38782[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC One X Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38783[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One X Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38784[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC One X Build/JDQ39)";
const char S38785[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; One X Build/JDQ39)";
const char S38786[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; HTC One X Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38787[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC One X Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38788[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One X Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38789[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC_One_X Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38790[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; HTC EVA_UL Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38791[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; de-ch; HTC_One_X/4.17.166.2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38792[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC_One_X/4.17.161.2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38793[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; de-at; HTC_One_X/4.17.169.2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38794[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HTC S720e Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38795[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; HTC_One_X/4.17.163.3 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38796[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZTE N9120 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38797[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ZTE N9120 Build/IMM76I)";
const char S38798[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZTE N9120 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Android MetroPCS androidAppVersion/2.5.10 AndroidWebkitWrapper BillingSupported";
const char S38799[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZTE N9120 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 androidAppVersion/2.6.8 AndroidWebkitWrapper";
const char S38800[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100 Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38801[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38802[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy SII T-mobile Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38803[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy SII T-mobile Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38804[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100G Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38805[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S38806[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy SII T-mobile Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38807[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38808[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100G Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38809[] = "Mozilla/4.0 (Windows; MSIE 6.0; Windows NT 6.0)";
const char S38810[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows; U; Windows NT 6.0;)";
const char S38811[] = "Mozilla/4.0 (compatible; MSIE 6.01; Windows NT 6.0)";
const char S38812[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.0)";
const char S38813[] = "Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 6.0; Trident/5.0)";
const char S38814[] = "Mozilla/5.0 (Windows; U; MSIE 6.0; Windows NT 6.0; en-US)";
const char S38815[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.0; SV1; MRA 5.8 (build 4157); .NET CLR 2.0.50727; AskTbPTV/5.11.3.15590)";
const char S38816[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.0; WOW64; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618)";
const char S38817[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.24) Gecko/X Ubuntu/10.10 (maverick) Firefox/3.6.24";
const char S38818[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.12 Mobile/11A4400f Safari/8536.25";
const char S38819[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.12 Mobile/11A4414e Safari/8536.25";
const char S38820[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A4435d Safari/8536.25";
const char S38821[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A4449a Safari/8536.25";
const char S38822[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A4449d Safari/8536.25";
const char S38823[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A465 Safari/8536.25";
const char S38824[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A501 Safari/8536.25";
const char S38825[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; C5120 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38826[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; C5121 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38827[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; C5120 Build/GINGERBREAD)";
const char S38828[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; C5121 Build/GINGERBREAD)";
const char S38829[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; C5120 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38830[] = "Mozilla/5.0 (Linux; Android 4.0.3;  SAMSUNG-SGH-T989 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S38831[] = "Mozilla/5.0 (Linux; Android 4.0.3;  SGH-T989 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S38832[] = "Mozilla/5.0 (Linux; Android 4.0.4;  SGH-T989 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S38833[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38834[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38835[] = "Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-T989 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38836[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-T989 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38837[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989D Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38838[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S38839[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38840[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S38841[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S5301 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38842[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; SAMSUNG GT-S5301/S5301XXAMA3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38843[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-S5301 Build/IMM76D)";
const char S38844[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; GT-S5301L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38845[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT-S5301B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38846[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-S5301 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38847[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-fj; Vodafone 858 Build/Vodafone858C02B623) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38848[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-al; Vodafone 858 Build/Vodafone858C02B622) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38849[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ro-ro; Vodafone 858 Build/Vodafone858C02B617SP01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38850[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-mt; Vodafone 858 Build/Vodafone858C02B621) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38851[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; Vodafone 858 Build/Vodafone858C02B617) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38852[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-za; Vodafone 858 Build/Vodafone858C02B620) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38853[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; lt-lt; Vodafone 858 Build/Vodafone858C02B619) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38854[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gh; Vodafone 858 Build/Vodafone858C02B625) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38855[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; Vodafone 858 Build/Vodafone858C02B650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38856[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-au; Vodafone 858 Build/Vodafone858C02B638SP01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38857[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; Vodafone 858 Build/Vodafone858C02B650)";
const char S38858[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; Vodafone 858 Build/Vodafone858C02B617)";
const char S38859[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; Vodafone 858 Build/Vodafone858C02B652) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38860[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; Vodafone 858 Build/Vodafone858C02B626) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38861[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-qa; Vodafone 858 Build/Vodafone858C02B637) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38862[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; tr-tr; Vodafone 858 Build/Vodafone858C02B663) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38863[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-za; Vodafone 858 Build/Vodafone858C02B627) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38864[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; hr-hr; Vodafone 858 Build/Vodafone858C02B636) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38865[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; el-gr; Vodafone 858 Build/Vodafone858C02B652SP01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38866[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; SmartLucky Build/Vodafone858C02B625) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38867[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; Vodafone 858 Build/Vodafone858C02B636SP01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38868[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; tr-tr; Vodafone 858 Build/Vodafone858C02B678) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38869[] = "Mozilla/5.0 (compatible; Blekkobot; ScoutJet; +http://blekko.com/about/blekkobot)";
const char S38870[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-L38C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S38871[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-L38C Build/GRK39F)";
const char S38872[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-S7560M Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38873[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-S7560M-parrot Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38874[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-S7560M Build/IMM76I)";
const char S38875[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-S7560M Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38876[] = "Mozilla/5.0 (Android; Mobile; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S38877[] = "Mozilla/5.0 (Android; Mobile; rv:20.0a2) Gecko/20.0a2 Firefox/20.0a2";
const char S38878[] = "Mozilla/5.0 (Android; Mobile; rv:20.0.1) Gecko/20.0.1 Firefox/20.0.1";
const char S38879[] = "Mozilla/5.0 (Android; Mobile; rv:20.0) Gecko/20.0 OAFirefox/20.0";
const char S38880[] = "Mozilla/5.0 (Android; Mobile: SAMSUNG-SGH-I337; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S38881[] = "Mozilla/5.0 (X11; U; Jolicloud Linux i686; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Jolicloud/1.0 Chromium/6.0.472.36 Chrome/6.0.472.36 Safari/534.3";
const char S38882[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S38883[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S38884[] = "Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S38885[] = "Mozilla/5.0 (Linux; U; Linux 2.6.29; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.62 Safari/534.3";
const char S38886[] = "Mozilla/5.0 (X11; U; Linux armv6l; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S38887[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9B206 Safari/7534.48.3";
const char S38888[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9B176 Safari/7534.48.3";
const char S38889[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9A405 Safari/7534.48.3";
const char S38890[] = "Opera/9.80 (X11; Linux i686; U; en) Presto/2.9.168 Version/11.52";
const char S38891[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S38892[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/15.0.849.0 Safari/535.1";
const char S38893[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_5) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19";
const char S38894[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19";
const char S38895[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19";
const char S38896[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19";
const char S38897[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S38898[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S38899[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S38900[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S38901[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/18.0.648.205 Safari/534.16";
const char S38902[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/18.0.648.205 Safari/534.16";
const char S38903[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-ca; SGH-I337M Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S38904[] = "Mozilla/5.0 (Linux; Android 4.2.2; es-us; SAMSUNG SGH-I337M Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S38905[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-ca; SGH-I337M-parrot Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S38906[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38907[] = "Mozilla/5.0 (Linux; Android 4.0.3;  GT-N7000 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S38908[] = "Mozilla/5.0 (Linux; Android 4.0.4;  GT-N7000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S38909[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-N7000 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38910[] = "Mozilla/5.0 (Linux; Android 4.0.3; SHV-E160L Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38911[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38912[] = "Mozilla/5.0 (Linux; Android 4.0.3; SHV-E160K Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38913[] = "Mozilla/5.0 (Linux; Android 4.0.3; SHV-E160S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38914[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S38915[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-N7000B Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38916[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T879 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38917[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38918[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38919[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160K Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38920[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-05D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38921[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M480W Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S38922[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38923[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M480S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S38924[] = "Mozilla/5.0 (Linux; Android 4.0.4; N7000+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S38925[] = "NokiaE52/033.003 (SymbianOS/9.3; U; Series60/3.2 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525";
const char S38926[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE52-1/054.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.2 3gpp-gba";
const char S38927[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE52-2/034.001; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.1.5";
const char S38928[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE52-1/091.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.34 Mobile Safari/533.4 3gpp-gba";
const char S38929[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE52-2/091.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.34 Mobile Safari/533.4 3gpp-gba";
const char S38930[] = "BlackBerry9520/5.0.0.713 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/125";
const char S38931[] = "BlackBerry9550/5.0.0.1015 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/105";
const char S38932[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:12.0) Gecko/X Firefox/12.0";
const char S38933[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:12.0) Gecko/X Firefox/12.0 Firefox/12.0";
const char S38934[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Vision Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38935[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; HTC Vision Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38936[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Vision Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38937[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Vision Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38938[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Vision Build/GRI40)";
const char S38939[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Vision Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38940[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Vision Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38941[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Vision Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38942[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC Vision Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38943[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Vision Build/FRF91)";
const char S38944[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Vision Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38945[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Vision Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38946[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Vision Build/GRI40)";
const char S38947[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Vision Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38948[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Vision Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S38949[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Vision Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S38950[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Ideos Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38951[] = "Mozilla/5.0 (Linux; U; Android 2.2; it-it; Ideos Build/B827SP01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38952[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-ke; Ideos Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38953[] = "Mozilla/5.0 (Linux; U; Android 2.2; it-it; Ideos Build/B827SP03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38954[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; Ideos Build/FRF91)";
const char S38955[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; Ideos Build/FRG83)";
const char S38956[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-nz; Ideos Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38957[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-ke; Ideos Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S38958[] = "Mozilla/5.0 (compatible; discoverybot/2.0; +http://discoveryengine.com/discoverybot.html)";
const char S38959[] = "Mozilla/5.0 (compatible; discoverybot/2.0; +http://discoveryengine.com/discoverybot.html),gzip(gfe) (via docs.google.com/viewer)";
const char S38960[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.0)";
const char S38961[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0;GB)";
const char S38962[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Alexa Toolbar)";
const char S38963[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; SV1)";
const char S38964[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; SV1)";
const char S38965[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; SV1; SV1)";
const char S38966[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; SV1; SV1)";
const char S38967[] = "Puffin%20Free/9090 CFNetwork/609.1.4 Darwin/13.0.0";
const char S38968[] = "Puffin/9073 CFNetwork/609.1.4 Darwin/13.0.0";
const char S38969[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT)";
const char S38970[] = "Mozilla/4.0 (compatible; MSIE 6.0; MS Web Services Client Protocol 2.0.50727.3053)";
const char S38971[] = "Mozilla/4.0 (compatible; MSIE 6.0; Win32)";
const char S38972[] = "Mozilla/5.0 (compatible; MSIE 6.0; 68K; Trident/5.0)";
const char S38973[] = "Mozilla/5.0 (compatible; MSIE 6.0; 68K; Win64; IA64; Trident/2.0)";
const char S38974[] = "Mozilla/5.0 (compatible; MSIE 6.0; 68K; WOW64; Trident/2.0)";
const char S38975[] = "Mozilla/5.0 (compatible; MSIE 6.0; 68K; SV1; Trident/4.0)";
const char S38976[] = "Mozilla/5.0 (compatible; MSIE 6.0; Windows NT)";
const char S38977[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38978[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9070 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38979[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9070 Build/JZO54K)";
const char S38980[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; GT-I9070P Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38981[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; SAMSUNG GT-I9070/I9070XXLQE Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38982[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9070P Build/JZO54K)";
const char S38983[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-I9070 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S38984[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB612 Build/4.5.1-KNS-29-BST-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38985[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB612 Build/4.5.1-110-KNS-46) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S38986[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; MB612 Build/4.5.1-110-KNS-46)";
const char S38987[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:13.0) Gecko/X Firefox/13.0.1";
const char S38988[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.6)";
const char S38989[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.7)";
const char S38990[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.8)";
const char S38991[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.8)";
const char S38992[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.7)";
const char S38993[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.7)";
const char S38994[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.9)";
const char S38995[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.6)";
const char S38996[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C; Sleipnir/2.9.6)";
const char S38997[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.3)";
const char S38998[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; YTB730; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.7)";
const char S38999[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.8)";
const char S39000[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.10)";
const char S39001[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.10)";
const char S39002[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.9)";
const char S39003[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.10)";
const char S39004[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.3)";
const char S39005[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.9)";
const char S39006[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.4)";
const char S39007[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Sleipnir/2.9.9)";
const char S39008[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.11)";
const char S39009[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.11)";
const char S39010[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.4)";
const char S39011[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.6)";
const char S39012[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.11)";
const char S39013[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.12)";
const char S39014[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.12)";
const char S39015[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.1)";
const char S39016[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.14)";
const char S39017[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.14)";
const char S39018[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.12)";
const char S39019[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.14)";
const char S39020[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Sleipnir/2.9.10)";
const char S39021[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.11)";
const char S39022[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; BRI/2; Sleipnir/2.9.8)";
const char S39023[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Sleipnir/2.9.14)";
const char S39024[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.15)";
const char S39025[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.15)";
const char S39026[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.15)";
const char S39027[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Sleipnir/2.9.6)";
const char S39028[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.15)";
const char S39029[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/2.9.15)";
const char S39030[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Sleipnir/2.9.15)";
const char S39031[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/2.9.14)";
const char S39032[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.16)";
const char S39033[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.16)";
const char S39034[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/2.9.16)";
const char S39035[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Sleipnir/2.9.16)";
const char S39036[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Sleipnir/2.9.16)";
const char S39037[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/2.9.17)";
const char S39038[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.17)";
const char S39039[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Sleipnir/2.9.17)";
const char S39040[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.14)";
const char S39041[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Sleipnir/2.9.14)";
const char S39042[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.17)";
const char S39043[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/2.9.10)";
const char S39044[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/2.9.7)";
const char S39045[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/2.9.9)";
const char S39046[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/2.9.12)";
const char S39047[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Sleipnir/2.9.18)";
const char S39048[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/2.9.18)";
const char S39049[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.18)";
const char S39050[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.18)";
const char S39051[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Sleipnir/2.9.9)";
const char S39052[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/2.9.3)";
const char S39053[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Sleipnir/2.9.10)";
const char S39054[] = "Mozilla/5.0 (Linux; Android 4.0.4;  Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S39055[] = "Mozilla/5.0 (Linux; Android 4.0.3;  Build/ITL41F) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S39056[] = "Mozilla/5.0 (Linux; Android 4.0.4;  Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S39057[] = "Mozilla/5.0 (Linux; Android 4.0.4; Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39058[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39059[] = "Mozilla/5.0 (Linux; Android 4.0.3; Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39060[] = "Mozilla/5.0 (Linux; Android 4.0.4.0.4.0.4; Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39061[] = "Mozilla/5.0 (Linux; Android 4.0.4; Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39062[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39063[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39064[] = "Mozilla/5.0 (Linux; Android 4.0.3.0.2.0.1.0; Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39065[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbl1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39066[] = "Mozilla/5.0 (Linux; Android 4.0.3; STM712HCZ Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39067[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_hjy1a6x Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39068[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_hjy1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39069[] = "Mozilla/5.0 (Linux; Android 4.0.3; STM805HCZ Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39070[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbaol1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39071[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbl1_5640 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39072[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1808_v75_jbl1_5640 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39073[] = "Mozilla/5.0 (Linux; Android 4.0.4; TWD_MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39074[] = "Mozilla/5.0 (Linux; Android 4.0.3; V1013 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39075[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1808_v75_jbl2 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39076[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbl2 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39077[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39078[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbl1a535 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39079[] = "Mozilla/5.0 (Linux; Android 4.0.3; Build/ICS.g04ref.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39080[] = "Mozilla/5.0 (Linux; Android 4.0.3; disgo9104 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39081[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_hjy1_5647 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39082[] = "Mozilla/5.0 (Linux; Android 4.0.3; Penta WS703C Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39083[] = "Mozilla/5.0 (Linux; Android 4.0.3; M711 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39084[] = "Mozilla/5.0 (Linux; Android 4.0.3; T03A Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39085[] = "Mozilla/5.0 (Linux; Android 4.0.3; MAGPAD7.1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39086[] = "Mozilla/5.0 (Linux; Android 4.0.4; jsa_MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39087[] = "Mozilla/5.0 (Linux; Android 4.0.3; Infinity Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39088[] = "Mozilla/5.0 (Linux; Android 4.0.3; M71G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39089[] = "Mozilla/5.0 (Linux; Android 4.0.3; DFP7004 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39090[] = "Mozilla/5.0 (Linux; Android 4.0.4; M758A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39091[] = "Mozilla/5.0 (Linux; Android 4.0.3; 756TPC Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39092[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1911_v77_jbaol_g20 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39093[] = "Mozilla/5.0 (Linux; Android 4.0.3; X9015 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39094[] = "Mozilla/5.0 (Linux; Android 4.0.3; classic101 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39095[] = "Mozilla/5.0 (Linux; Android 4.0.3; Penta WS702C Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39096[] = "Mozilla/5.0 (Linux; Android 4.0.9; e1909c_v77_gq1008_9p017_6228 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39097[] = "Mozilla/5.0 (Linux; Android 4.0.4; A10??(8DM1) Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39098[] = "Mozilla/5.0 (Linux; Android 4.0.3; Tablet PC 2000 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39099[] = "Mozilla/5.0 (Linux; Android 4.0.3; S5-N Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39100[] = "Mozilla/5.0 (Linux; Android 4.0.4; N101 DUAL CORE Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39101[] = "Mozilla/5.0 (Linux; Android 4.0.4; CJ ThL Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39102[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1911_v77_hjy1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39103[] = "Mozilla/5.0 (Linux; Android 4.0.4; T272 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39104[] = "Mozilla/5.0 (Linux; Android 4.0.3; Clear Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39105[] = "Mozilla/5.0 (Linux; Android 4.0.4; UM-A13 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39106[] = "Mozilla/5.0 (Linux; Android 4.0.4; unknown Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39107[] = "Mozilla/5.0 (Linux; Android 4.0.3; ARGM727HCX Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39108[] = "Mozilla/5.0 (Linux; Android 4.0.4; unknown Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39109[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_hjy1a6x_5647 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39110[] = "Mozilla/5.0 (Linux; Android 4.0.9; e1909c_v77_gq1008_5647 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39111[] = "Mozilla/5.0 (Linux; Android 4.0.4; Tender Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39112[] = "Mozilla/5.0 (Linux; Android 4.0.9; e1911_v77_gq2000_ov5647 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39113[] = "Mozilla/5.0 (Linux; Android 4.0.4; INFINITY_POWER Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39114[] = "Mozilla/5.0 (Linux; Android 4.0.4; POM727MC Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39115[] = "Mozilla/5.0 (Linux; Android 4.0.3; 720F Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39116[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1908_v77_jbl1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39117[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID 008 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39118[] = "Mozilla/5.0 (Linux; Android 4.0.9; e1909c_v77_gq1008_9p017 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39119[] = "Mozilla/5.0 (Linux; Android 4.0.4; Pad Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39120[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1808_v75_jbl2_6628 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39121[] = "Mozilla/5.0 (Linux; Android 4.0.4; EBM727KC Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39122[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1911_v77_mt6628_hjy1_9p017 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39123[] = "Mozilla/5.0 (Linux; Android 4.0.4; S150 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39124[] = "Mozilla/5.0 (Linux; Android 4.0.3; NEOBOX Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39125[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1908_v77_jbl1a535 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39126[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbl1_5640a522 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39127[] = "Mozilla/5.0 (Linux; Android 4.0.9; e1909c_v77_gq1008_5647_6228 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39128[] = "Mozilla/5.0 (Linux; Android 4.0.3; FAR7 Build/V02R10A) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39129[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19";
const char S39130[] = "Mozilla/5.0 (Linux; Android 4.0.3; TR720F Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39131[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_hjy1a6x_in Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39132[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1911_v77_jbaol_g20mt9p017 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39133[] = "Mozilla/5.0 (Linux; Android 4.0.4; NEOBOX Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39134[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbaol1_5647 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39135[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1908_v77_jbl1a535ntv Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39136[] = "Mozilla/5.0 (Linux; Android 4.0.3; X Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39137[] = "Mozilla/5.0 (Linux; Android 4.1.1; XT912 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39138[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT912 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39139[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39140[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S39141[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5253/S5253DDKJ2; U; Bada/1.0; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 OPN-B";
const char S39142[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XT890 Build/8.7.1I-110_IFW-29) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39143[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XT890 Build/8.7.1I-110_IFW-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39144[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; XT890 Build/8.7.1I-110_IFW-DE-32) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39145[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; XT890 Build/8.7.1I-110_SML-12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39146[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT890 Build/8.7.1I-110_SML-13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39147[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; XT890 Build/8.7.1I-110_IFW-DE-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39148[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XT890 Build/8.7.1I-110_IFW-39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39149[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; XT890 Build/8.7.1I-110_IFW-DE-39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39150[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; XT890 Build/8.7.1I-110_IFW-DE-40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39151[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT890 Build/8.7.1I-110_SML-16.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39152[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT890 Build/8.7.1I-110_SML-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39153[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT890 Build/8.7.1I-110_SML-16.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39154[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; XT890 Build/8.7.1I-110_IFW-39)";
const char S39155[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; XT890 Build/8.7.1I-110_SML-8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39156[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT890 Build/8.7.1I-110_SML-20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39157[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XT890 Build/unknown) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39158[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I200 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39159[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-I200 Build/JZO54K)";
const char S39160[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SCH-I200 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39161[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I200PP Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39162[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; GT-I8190L Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39163[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-I8190L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39164[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-I8190L Build/JRO03H)";
const char S39165[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8190L Build/JZO54K)";
const char S39166[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.9) Gecko/X Mandriva/2.0.4-69.1mib2010.0 SeaMonkey/2.0.4";
const char S39167[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1.17) Gecko/X SeaMonkey/2.0.12";
const char S39168[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; sv-se; HTC Desire Build/GRI40; Elelinux-CM7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39169[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MildWild for Desire Build/GWK 74 MildWild for Desire) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39170[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire Build/GWK74-MildWild for Desire) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39171[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Desire CDMA Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39172[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire Build/GRI40; MildWild CM-5.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39173[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire Build/GRI40; CyanogenMod-7.2.0.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39174[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MildWild for Desire Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39175[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Desire Build/GWK74-MildWild for Desire)";
const char S39176[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-tw; HTC_Glacier Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39177[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39178[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTCGlacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39179[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39180[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Glacier Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39181[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39182[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S39183[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39184[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Glacier Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39185[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC Glacier Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39186[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Glacier Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39187[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Glacier Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39188[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39189[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Glacier Build/GRJ22)";
const char S39190[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39191[] = "Logos/0.9.4 (Linux; U; Android 2.3.4; en_US; HTC Glacier Build/GRJ22)";
const char S39192[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC GlacierBuild/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39193[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Glacier Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39194[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; HTC Glacier Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39195[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Glacier Build/GRI40)";
const char S39196[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Glacier Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39197[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Glacier Build/GRJ22) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S39198[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Glacier Build/GRJ22)";
const char S39199[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39200[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Glacier Build/GRJ90)";
const char S39201[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Glacier Build/GRI40; Absolution v3.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39202[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39203[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Glacier Build/DarKsyS 3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39204[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Glacier Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S39205[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Glacier Build/GRJ90; CyanUX V3.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39206[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Glacier Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S39207[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; ) AppleWebKit/534.12 (KHTML, like Gecko) Maxthon/3.0 Safari/534.12";
const char S39208[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12";
const char S39209[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12";
const char S39210[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1";
const char S39211[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1";
const char S39212[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12,gzip(gfe)";
const char S39213[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; ) AppleWebKit/533.9 (KHTML, like Gecko) Maxthon/3.0 Safari/533.9";
const char S39214[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12,gzip(gfe)";
const char S39215[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1,gzip(gfe)";
const char S39216[] = "Mozilla/5.0 (Series40; Nokia303/13.47; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S39217[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; YP-GS1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39218[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; YP-GS1 Build/GINGERBREAD)";
const char S39219[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; YP-GS1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39220[] = "Mozilla/5.0 (Series40; Nokia202/20.28; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S39221[] = "Mozilla/5.0 (Series40; Nokia203/20.36; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S39222[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.5.24 Version/10.53";
const char S39223[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.5.22 Version/10.50";
const char S39224[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.5.24 Version/10.54";
const char S39225[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.5.22 Version/10.51";
const char S39226[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.5.24 Version/10.52";
const char S39227[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.5.24 Version/10.53,gzip(gfe) (via translate.google.com)";
const char S39228[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.5.22 Version/10.50";
const char S39229[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.5.24 Version/10.53";
const char S39230[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.5.24 Version/10.53";
const char S39231[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.5.24 Version/10.53";
const char S39232[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.5.22 Version/10.51";
const char S39233[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.5.24 Version/10.53";
const char S39234[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.5.24 Version/10.53";
const char S39235[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.5.24 Version/10.52";
const char S39236[] = "Opera/7.03 (Windows 95; U)  [en]";
const char S39237[] = "Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows 95) Opera 7.03 [de]";
const char S39238[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; GT-S5660L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39239[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5660 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39240[] = "Mozilla/5.0(Linux;U;Android2.2.1;Zh_cn;GT-S5660;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S39241[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-S5660 Build/FROYO)";
const char S39242[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-S5660 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S39243[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-S5660 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S39244[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-; GT-S5660 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39245[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; Nexus 7 Build/JFN53N) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39246[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39247[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39248[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Nexus 7 Build/JRO03D)";
const char S39249[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; Nexus 7 Build/JRN84D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39250[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39251[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39252[] = "Dalvik/1.6.0 (Linux; U; Android 4.1; Nexus 7 Build/JRN84D)";
const char S39253[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Nexus 7 Build/JRO03D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S39254[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03S) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39255[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39256[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S39257[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Nexus 7 Build/JZO54K)";
const char S39258[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39259[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Ainovo Flame (Nexus 7) Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39260[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; Nexus 7 Build/JRO03S) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39261[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Google Nexus 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39262[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Google Nexus 7 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39263[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03S) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39264[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Nexus 7 Build/JRO03L by Team Shogun v1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39265[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S39266[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/CodenameSammy-1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39267[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03S) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S39268[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39269[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S39270[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Nexus 7 Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.2.1-65 (Android 4.1.1; asus Nexus 7 Build/JRO03D)";
const char S39271[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39272[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S39273[] = "Logos/2.0.11 (Linux; U; Android 4.1.1; en-US; Nexus 7 Build/cm_elf2-userdebug 4.1.1 JRO03L eng.alan.X.X test-keys)";
const char S39274[] = "Logos/2.1.2 (Linux; U; Android 4.1.1; en-US; Nexus 7 Build/pa_grouper-userdebug 4.1.1 JRO03R eng.agascoig.X.X test-keys)";
const char S39275[] = "Logos/2.1.2 (Linux; U; Android 4.1.1; en-US; Nexus 7 Build/Glazed ICS V1.1.1)";
const char S39276[] = "Logos/2.1.2 (Linux; U; Android 4.1.1; en-US; Nexus 7 Build/JRO03R)";
const char S39277[] = "Logos/2.1.2 (Linux; U; Android 4.1.1; en-US; Nexus 7 Build/cm_grouper-userdebug 4.1.1 JRO03L eng.winner00.X.X test-keys)";
const char S39278[] = "Logos/2.1.2 (Linux; U; Android 4.1.2; en-US; Nexus 7 Build/cm_elf2-userdebug 4.1.2 JZO54K eng.alan.X.X test-keys)";
const char S39279[] = "Logos/2.1.2 (Linux; U; Android 4.1.2; en-US; Nexus 7 Build/pa_grouper-userdebug 4.1.2 JZO54K eng.agascoig.X.X test-keys)";
const char S39280[] = "FaithlifeStudyBible/2.0.11 (Linux; U; Android 4.1.1; en-US; Nexus 7 Build/pa_grouper-userdebug 4.1.1 JRO03L eng.root.X.X test-keys)";
const char S39281[] = "GoogleAnalytics/2.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03D)";
const char S39282[] = "com.google.android.apps.plus/X (Linux; U; Android 4.1.2; en_US; Nexus 7; Build/JZO54K) (gzip)";
const char S39283[] = "com.google.android.apps.plus/X (Linux; U; Android 4.1.1; en_US; Nexus 7; Build/JRO03D) (gzip)";
const char S39284[] = "com.google.android.videos/2.0.10(Linux; U; Android 4.1.2; en_US; Nexus 7 Build/JZO54K) gzip";
const char S39285[] = "com.google.android.youtube/4.0.8(Linux; U; Android 4.1.1; en_US; Nexus 7 Build/JRO03S) gzip";
const char S39286[] = "Bible 4.0.9 (asus Nexus 7 nakasi;Android 4.1.1;en_US)";
const char S39287[] = "BBC iPlayer (AndroidApp; 1.3; 13014) Nexus 7 (Android 4.1.2, SDK 16)";
const char S39288[] = "BBCBeacon/0.73 (Nexus 7; Android 4.1.2) iplayer/1.4.2.3";
const char S39289[] = "BBC News (AndroidApp; 1.0; 8) Nexus 7 (Android 4.1.2, SDK 16)";
const char S39290[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Google Nexus 7 Build/JRO03H)";
const char S39291[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Ainovo Flame (Nexus 7) Build/JRO03D)";
const char S39292[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Ainovo Flame (Nexus 7) Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S39293[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S39294[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; Nexus 7 Build/JRN87) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39295[] = "Bible 4.0.7 (asus Nexus 7 nakasi;Android 4.1.1;en_US)";
const char S39296[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Nexus 7 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 parameters/{scr=553_962,cm=1,ql=l,imei=10:bf:48:cb:dd:cb,bm=1,eng=adrmini16,mdm=1,mdl=Nexus 7,ps=1} tiantian(securitypay,securityinstalled) tiantian(tenpay)";
const char S39297[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 GSA/2.0.0.X";
const char S39298[] = "Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; Nexus 7 Build/JRN84D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S39299[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Nexus 7 Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S39300[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; Nexus 7 16G Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 GSA/2.0.5.X";
const char S39301[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39302[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZ054K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39303[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.2.1-65 (Android 4.1.2; asus Nexus 7 Build/JZO54K)";
const char S39304[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Nexus 7 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S39305[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Nexus 7 Build/JRO03H) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S39306[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39307[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; Nexus 7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39308[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/1.9.10;FBDM/{density=1.33125,width=800,height=1205};FBLC/en_US;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/Nexus 7;FBSV/4.1.2;]";
const char S39309[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S39310[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; Nexus 7 Build/ADEPT01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39311[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Nexus 7 Build/JZO54K by Team Shogun v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39312[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-gb; Nexus 7 Build/JRN84D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39313[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO54K) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S39314[] = "Mozilla/5.0 (Android 4.1.1; en-us; Nexus 7 Build/JRO03D) FreeWheelAdManager/5.0.2-r9017-X;com.gotv.crackle.handset/X";
const char S39315[] = "Mozilla/5.0(YAdSDK; 2.2.1); (Linux; U; Android 4.1.2; ASUS Nexus 7 Build/Unknown);";
const char S39316[] = "Scanner Radio/3.8.3 (Android 4.1.1, Nexus 7)";
const char S39317[] = "Mozilla/5.0 (Linux; U; Android 4.1.1_Generick_V6_by_darkrickpr; ru-ru; Nexus 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39318[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/V5 HD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39319[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows 3.1)";
const char S39320[] = "Mozilla/4.0 (compatible; MSIE 5.0; Win3.1; ATHMWWW1.1;)";
const char S39321[] = "Mozilla/5.0 (compatible; MSIE 5.0; Win3.11; Trident/4.0)";
const char S39322[] = "GoogleEarth/7.0.1.8244(Windows;Microsoft Windows (6.1.7601.1);en;kml:2.2;client:Free;type:default)";
const char S39323[] = "GoogleEarth/7.0.1.8244(Windows;Microsoft Windows (6.1.7601.1);de;kml:2.2;client:Free;type:default)";
const char S39324[] = "GoogleEarth/7.0.2.8415(Windows;Microsoft Windows (6.1.7600.0);en;kml:2.2;client:Free;type:default)";
const char S39325[] = "GoogleEarth/7.0.2.8415(Windows;Microsoft Windows (6.1.7601.1);de;kml:2.2;client:Free;type:default)";
const char S39326[] = "GoogleEarth/7.0.2.8415(Windows;Microsoft Windows (6.1.7601.1);en;kml:2.2;client:Free;type:default)";
const char S39327[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.1.7601.1);ru;kml:2.2;client:Free;type:default)";
const char S39328[] = "Mozilla/5.0 (Windows; N; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/7.0.3.8542 Safari/532.4";
const char S39329[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.1.7601.1);en;kml:2.2;client:Free;type:default)";
const char S39330[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.1.7600.0);en;kml:2.2;client:Free;type:default)";
const char S39331[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.1.7601.1);de;kml:2.2;client:Free;type:default)";
const char S39332[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.1.7601.1);es-419;kml:2.2;client:Free;type:default)";
const char S39333[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.1.7601.1);nl;kml:2.2;client:Free;type:default)";
const char S39334[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.1.7601.1);pt;kml:2.2;client:Free;type:default)";
const char S39335[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.1.7601.1);fr;kml:2.2;client:Free;type:default)";
const char S39336[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.1.7601.1);it;kml:2.2;client:Free;type:default)";
const char S39337[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.1.7601.1);es;kml:2.2;client:Free;type:default)";
const char S39338[] = "Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320)";
const char S39339[] = "Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; Smartphone; 176x220)";
const char S39340[] = "Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; 240x320)";
const char S39341[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322; MS-RTC LM 8; InfoPath.2)";
const char S39342[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; chromeframe; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S39343[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; OfficeLiveConnector.1.4; OfficeLivePatch.1.3)";
const char S39344[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; chromeframe)";
const char S39345[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; chromeframe; .NET CLR 2.0.50727)";
const char S39346[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; XT320 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39347[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; XT321 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39348[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; MOT-XT320 Build/0A.1F.3E) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39349[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT320 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S39350[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; XT320 Build/GRK39F)";
const char S39351[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hr-hr; MOT-XT320 Build/0A.1F.3A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39352[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MOT-XT320 Build/0A.21.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39353[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MOT-XT320 Build/0A.1F.37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39354[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; MOT-XT320 Build/0A.1F.49) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39355[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; XT320 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Versao/4.0 Mobile Safari/533.1";
const char S39356[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; XT321 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Versao/4.0 Mobile Safari/533.1";
const char S39357[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; MOT-XT320 Build/0A.1D.36) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39358[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; MOT-XT320 Build/0A.21.15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39359[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; MOT-XT320 Build/0A.1D.38) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39360[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; MOT-XT320 Build/0A.21.2C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39361[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pl-pl; MOT-XT320 Build/0A.21.2D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39362[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-pt; MOT-XT320 Build/0A.21.21) AppleWebKit/533.1 (KHTML, like Gecko) Versao/4.0 Mobile Safari/533.1";
const char S39363[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; XT321 Build/GRK39F)";
const char S39364[] = "Mozilla/4.0 (compatible; MSIE 7.0; Win32)";
const char S39365[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT)";
const char S39366[] = "Mozilla/6.0 (compatible; MSIE 7.01; Windows NT)";
const char S39367[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.5)";
const char S39368[] = "Mozilla/5.0 (compatible; MSIE 7.0 Win32 +http://www.internetmemory.org, +http://www.parliament.uk)";
const char S39369[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 7.1; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C)";
const char S39370[] = "Mozilla/6.0 (compatible;MSIE 7.0;  Windows NT 7.0; 360safe)";
const char S39371[] = "Mozilla/3.0 (compatible;MSIE 7.0;  Windows NT 7.0;  MyIE2)";
const char S39372[] = "Mozilla/4.0 (compatible;MSIE 7.0;  Windows NT 7.0; 360safe)";
const char S39373[] = "Mozilla/5.0 (compatible;MSIE 7.0;  Windows NT 7.0; 360safe)";
const char S39374[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows)";
const char S39375[] = "Mozilla/5.0 (compatible; MSIE 7.0; 68K; Trident/4.0)";
const char S39376[] = "Mozilla/5.0 (compatible; MSIE 7.0; 68K; SV1; Trident/2.0)";
const char S39377[] = "Mozilla/5.0 (compatible; MSIE 7.0; 68K; Win64; x64; Trident/4.0)";
const char S39378[] = "Mozilla/5.0 (compatible; MSIE 7.0; PPC; WOW64; Trident/3.0)";
const char S39379[] = "Mozilla/5.0 (compatible; MSIE 7.0; 68K; WOW64; Trident/1.0)";
const char S39380[] = "Mozilla/5.0 (compatible; MSIE 7.0; PPC; .NET CLR; Trident/5.0)";
const char S39381[] = "Mozilla/5.0 (compatible; MSIE 7.0; PPC; Win64; x64; Trident/3.0)";
const char S39382[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39383[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Orange Daytona Build/ESPC224B164) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39384[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI G510-0010 Build/HuaweiG510-0010) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S39385[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Orange Daytona Build/HuaweiG510-0200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39386[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HUAWEI G510-0200 Build/HuaweiG510-0200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39387[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HUAWEI G510-0100 Build/C02B171) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39388[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HUAWEI G510-0010 Build/HuaweiG510-0010)";
const char S39389[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; HUAWEI G510-0100 Build/C02B170) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39390[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HUAWEI G510-0100 Build/HuaweiG510-0100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39391[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Orange Daytona Build/HuaweiG510-0200)";
const char S39392[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Orange Daytona Build/C224B179) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39393[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HUAWEI G510-0200 Build/HuaweiG510-0200)";
const char S39394[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; HUAWEI G510-0251 Build/HuaweiG510-0251) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39395[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HUAWEI G510-0100 Build/HuaweiG510-0100)";
const char S39396[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HUAWEI Ascend G510 Build/HuaweiG510-0200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39397[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HUAWEI G510-0251 Build/HuaweiG510-0251)";
const char S39398[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.4.1.1000 Chrome/18.0.966.0 Safari/535.12";
const char S39399[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.4.1.1000 Chrome/18.0.966.0 Safari/535.12";
const char S39400[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.4.5.2000 Chrome/22.0.1229.79 Safari/537.1";
const char S39401[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.4.5.2000 Chrome/22.0.1229.79 Safari/537.1";
const char S39402[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/16.0.912.63) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S39403[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/16.0.912.63; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S39404[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/16.0.912.63; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; Tablet PC 2.0)";
const char S39405[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/16.0.912.75) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7";
const char S39406[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/16.0.912.75) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7";
const char S39407[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/16.0.912.75)";
const char S39408[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/16.0.912.77) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S39409[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/16.0.912.77) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S39410[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/16.0.912.77)";
const char S39411[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/16.0.912.75; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.2)";
const char S39412[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/16.0.912.75; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729)";
const char S39413[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/16.0.912.75; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3)";
const char S39414[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; chromeframe/16.0.912.77)";
const char S39415[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/16.0.912.63) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S39416[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/16.0.912.75; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Tablet PC 2.0)";
const char S39417[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/16.0.912.77;  SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E)";
const char S39418[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/16.0.912.75; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729)";
const char S39419[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/16.0.912.75; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S39420[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/16.0.912.75; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Tablet PC 2.0)";
const char S39421[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/16.0.912.77;  SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S39422[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/16.0.912.77)";
const char S39423[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/16.0.912.77;  SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)";
const char S39424[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/16.0.912.75)";
const char S39425[] = "desktop";
const char S39426[] = "Linux";
const char S39427[] = "D?xO?";
const char S39428[] = "-1";
const char S39429[] = "S728";
const char S39430[] = "wp.exe";
const char S39431[] = "-1 or 107=107";
const char S39432[] = "-1\" or \"70\"=\"70";
const char S39433[] = "-1\" or \"39\"=\"0";
const char S39434[] = "-1\" or \"77\"=\"0";
const char S39435[] = "-1 or 58=0";
const char S39436[] = "1.11.4";
const char S39437[] = "UMT";
const char S39438[] = "Toshiba";
const char S39439[] = "Flash";
const char S39440[] = "IGX";
const char S39441[] = "-1\" or \"27\"=\"0";
const char S39442[] = "jack";
const char S39443[] = "-1\" or \"16\"=\"0";
const char S39444[] = "-1' or '30'='0";
const char S39445[] = "S732";
const char S39446[] = "testing";
const char S39447[] = "fliptop";
const char S39448[] = "SN55TV";
const char S39449[] = "Newsify";
const char S39450[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-R820 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39451[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-R820 Build/GINGERBREAD)";
const char S39452[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SCH-R820 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39453[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:9.0a2) Gecko/X Firefox/9.0a2";
const char S39454[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:9.0.1) Gecko/X Firefox/9.0.1 Firefox/9.0.1";
const char S39455[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:9.0a2) Gecko/X Firefox/9.0a2 Firefox 9.0.1";
const char S39456[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; SGH-i917)";
const char S39457[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; SGH-i917.)";
const char S39458[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; CETUS)";
const char S39459[] = "Mozilla/5.0 (compatible: MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; SGH-i917)";
const char S39460[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; WpsLondonTest; SAMSUNG; SGH-i917)";
const char S39461[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1";
const char S39462[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1";
const char S39463[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1";
const char S39464[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1";
const char S39465[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-MS910 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39466[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-MS910 Build/GINGERBREAD)";
const char S39467[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-MS910 Build/GINGERBREAD+) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39468[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; LG-MS910 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39469[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-MS910 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S39470[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-MS910 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39471[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-MS695 Build/GRK39F)";
const char S39472[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-MS695 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39473[] = "Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:23.0) Gecko/X Firefox/23.0";
const char S39474[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; U8655-1 Build/HuaweiU8655) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39475[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; STARTRAIL II Build/HuaweiU8655) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39476[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; U8655-51 Build/HuaweiU8655) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39477[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; U8655-1 Build/HuaweiU8655)";
const char S39478[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; U8655 Build/HuaweiU8655) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39479[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_C8655/C8655V100R001C92B818; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S39480[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; HUAWEI IDEOS Y 200 Build/HuaweiU8655) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39481[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HUAWEI C8655 Build/HuaweiC8655) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39482[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_C8655/C8655V100R001C92B816SP01; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S39483[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_C8655/C8655V100R001C92B816; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S39484[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; Etisalat Build/HuaweiU8655) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39485[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; MTC 965 Build/HuaweiU8655) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39486[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; U8655-51 Build/HuaweiU8655)";
const char S39487[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows 95) Opera 6.01  [en]";
const char S39488[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows 95) Opera 6.02 [de]";
const char S39489[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en) Presto/2.8.131 Version/11.11";
const char S39490[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.7; U; en) Presto/2.8.131 Version/11.10";
const char S39491[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VS920 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39492[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; VS920 4G Build/IMM76D)";
const char S39493[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/13.0.782.220; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S39494[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/13.0.782.218; chromeframe; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S39495[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/13.0.782.107; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S39496[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/13.0.782.107) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1";
const char S39497[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/13.0.782.215) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.215 Safari/535.1";
const char S39498[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/13.0.782.218) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.218 Safari/535.1";
const char S39499[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/13.0.782.220) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1";
const char S39500[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/13.0.782.112; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8; .NET4.0C; .NET4.0E)";
const char S39501[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/13.0.782.220; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S39502[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; chromeframe/13.0.782.112; .NET CLR 1.1.4322; .NET CLR 2.0.50727)";
const char S39503[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; chromeframe/13.0.782.215; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8)";
const char S39504[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/13.0.782.112) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1";
const char S39505[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/13.0.782.220; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S39506[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/13.0.782.220)";
const char S39507[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/13.0.767.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/13.0.767.1 Safari/534.36";
const char S39508[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/13.0.782.55) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.55 Safari/535.1";
const char S39509[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; SearchToolbar 1.2; chromeframe/13.0.782.220; GTB7.2; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; AV3; .NET CLR 1.1";
const char S39510[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/13.0.782.215; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Tablet PC 1.7; .NET CLR 2.0.50727; InfoPath.1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S39511[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/13.0.782.112) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1";
const char S39512[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/13.0.782.32) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.32 Safari/535.1";
const char S39513[] = "Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Large Screen Safari/534.24 GoogleTV/S67254";
const char S39514[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.20 (KHTML, like Gecko) Chrome/11.0.672.2 Large Screen Safari/534.20 GoogleTV/X";
const char S39515[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Large Screen Safari/534.24 GoogleTV/b70199";
const char S39516[] = "Mozilla/5.0 (Linux; GoogleTV 3.2; LG Google TV Build/MASTER) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Safari/534.24";
const char S39517[] = "Mozilla/5.0 (Linux; GoogleTV 3.2; VAP430 Build/MASTER) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Safari/534.24";
const char S39518[] = "Mozilla/5.0 (Linux; GoogleTV 3.2; NSZ-GS7/GX70 Build/MASTER) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Safari/534.24";
const char S39519[] = "Mozilla/5.0 (Linux; GoogleTV 3.2; LG Google TV G3 Build/MASTER) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Safari/534.24";
const char S39520[] = "Mozilla/5.0 (Linux; GoogleTV 3.2; GTV100 Build/MASTER) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Safari/534.24";
const char S39521[] = "Mozilla/5.0 (Linux; GoogleTV 3.2; hisense_gx1200v Build/MASTER) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Safari/534.24";
const char S39522[] = "Mozilla/5.0 (Linux; GoogleTV 3.2; asus_google_cube Build/MASTER) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Safari/534.24";
const char S39523[] = "Mozilla/5.0 (Linux; GoogleTV 3.2; TI320-DU Build/MASTER) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Safari/534.24";
const char S39524[] = "Mozilla/5.0 (compatible; Plukkie/1.5; http://www.botje.com/plukkie.htm)";
const char S39525[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N861 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39526[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; N861 Build/IMM76D)";
const char S39527[] = "Mozilla/5.0 (compatible; Konqueror/4.3; Linux) KHTML/4.3.5 (like Gecko)";
const char S39528[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; X501_USA_Cricket Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39529[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; X501_USA_Cricket Build/GRK39F)";
const char S39530[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; X501_USA_Cricket Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 androidAppVersion/2.5.22 AndroidWebkitWrapper BillingSupported";
const char S39531[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; X501_USA_Cricket Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 androidAppVersion/2.6.8 AndroidWebkitWrapper";
const char S39532[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9930; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.254 Mobile Safari/534.11+";
const char S39533[] = "BlackBerry9930/7.0.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/105";
const char S39534[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One X Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39535[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One X Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39536[] = "Mozilla/5.0 (Linux; Android 4.0.4; One X Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39537[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTCOneX Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S39538[] = "Mozilla/5.0 (Linux; Android 4.0.4; S720e Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39539[] = "Mozilla/5.0 (Linux; Android 4.0.4;  HTC One X Build/IMM76D) AppleWebKit/535.19 (KHTML like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S39540[] = "Mozilla/5.0 (Linux; Android 4.0.3;  HTC One X Build/IML74K) AppleWebKit/535.19 (KHTML like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S39541[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:14.0) Gecko/X Firefox/14.0.1";
const char S39542[] = "AdsBot-Google-Mobile (+http://www.google.com/mobile/adsbot.html) Mozilla (iPhone; U; CPU iPhone OS 3 0 like Mac OS X) AppleWebKit (KHTML, like Gecko) Mobile Safari";
const char S39543[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us; EasouSpider; +http://www.easou.com/search/spider.html)";
const char S39544[] = "iCoreService (unknown version) CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro9%2C2)";
const char S39545[] = "ThumbnailService/39600.038 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro10%2C1)";
const char S39546[] = "iCoreService (unknown version) CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookAir5%2C2)";
const char S39547[] = "iCoreService (unknown version) CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (iMac13%2C1)";
const char S39548[] = "WebProcess/8536.30.1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro9%2C2)";
const char S39549[] = "iCoreService (unknown version) CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (Macmini6%2C2)";
const char S39550[] = "ThumbnailService/39600.038 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookAir5%2C2)";
const char S39551[] = "WebProcess/8536.30.1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookAir5%2C2)";
const char S39552[] = "Norton%20Identity%20Safe/1.03 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro10%2C1)";
const char S39553[] = "ThumbnailService/39600.038 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (iMac12%2C2)";
const char S39554[] = "Flint/165 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro10%2C1)";
const char S39555[] = "WebProcess/8536.30.1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (iMac13%2C2)";
const char S39556[] = "Miro/5.0.4 (http://www.getmiro.com/; Darwin 12.4.0 x86_64)";
const char S39557[] = "iCoreService (unknown version) CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBook7%2C1)";
const char S39558[] = "CoverScout%203/3.5.7 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (iMac12%2C2)";
const char S39559[] = "XProtectUpdater (unknown version) CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro9%2C2)";
const char S39560[] = "Socialite/7848 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro10%2C1)";
const char S39561[] = "iCoreService (unknown version) CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacPro5%2C1)";
const char S39562[] = "ThumbnailService/39600.038 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (Macmini6%2C2)";
const char S39563[] = "ThumbnailService/39600.038 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBook5%2C1)";
const char S39564[] = "CoverScout%203/3.5.7 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro9%2C2)";
const char S39565[] = "POP/0.24 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro10%2C1)";
const char S39566[] = "RSS%20Notifier/7 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro10%2C1)";
const char S39567[] = "POP/0.24 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookAir5%2C2)";
const char S39568[] = "WebProcess/8536.30.1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBook5%2C1)";
const char S39569[] = "POP/0.24 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (iMac12%2C2)";
const char S39570[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e)";
const char S39571[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e; 2.01.161.04)";
const char S39572[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e; 1.08.161.02)";
const char S39573[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; PI39100)";
const char S39574[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e; 2.05.161.01)";
const char S39575[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e; 1.08.165.02)";
const char S39576[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; X310e)";
const char S39577[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e; 2.05.162.01)";
const char S39578[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e; 1.08.162.02)";
const char S39579[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Titan)";
const char S39580[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e; 4.06.162.01)";
const char S39581[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e; 4.06.163.03)";
const char S39582[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e; 4.06.165.02)";
const char S39583[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e; 4.06.161.01)";
const char S39584[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e; 4.07.162.03)";
const char S39585[] = "Opera/9.00 (Windows NT 4.0; U; en)";
const char S39586[] = "Mozilla/5.0 (Series40; NokiaC2-03/07.48; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S39587[] = "Mozilla/5.0 (Series40; NokiaC2-03/06.97; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5";
const char S39588[] = "Mozilla/5.0 (Android; Mobile; rv:23.0) Gecko/23.0 Firefox/23.0";
const char S39589[] = "Mozilla/5.0 (Android; Mobile; rv:23.0a1) Gecko/23.0a1 Firefox/23.0a1";
const char S39590[] = "Mozilla/5.0 (Android; Mobile; rv:23.0a2) Gecko/23.0a2 Firefox/23.0a2";
const char S39591[] = "Mozilla/5.0 (Android; Mobile; rv:23.0) Gecko/23.0 Firefox/23.0,gzip(gfe) (via docs.google.com/viewer)";
const char S39592[] = "Mozilla/5.0 (Android; Mobile; rv:23.0) Gecko/23.0 Firefox/23.0,gzip(gfe)";
const char S39593[] = "RealPlayer%20Downloader/1569 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (iMac11%2C2)";
const char S39594[] = "Spector%20Pro/3950 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (iMac10%2C1)";
const char S39595[] = "CoverScout%203/3.4.3 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro7%2C1)";
const char S39596[] = "RealPlayer%20Downloader/1444 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro7%2C1)";
const char S39597[] = "RealPlayer%20Downloader/1569 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBook7%2C1)";
const char S39598[] = "RealPlayer%20Downloader/1569 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookAir3%2C2)";
const char S39599[] = "NewsFire/84 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro5%2C3)";
const char S39600[] = "WebProcess/6534.57.2 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro7%2C1)";
const char S39601[] = "WebProcess/6534.57.2 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBook7%2C1)";
const char S39602[] = "Spector%20Pro/3731 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro5%2C5)";
const char S39603[] = "WebProcess/6534.57.2 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (iMac10%2C1)";
const char S39604[] = "Google%20Quick%20Search%20Box/2.0.0.1447 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro5%2C5)";
const char S39605[] = "WebProcess/6534.57.2 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (Macmini3%2C1)";
const char S39606[] = "XProtectUpdater (unknown version) CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro7%2C1)";
const char S39607[] = "Google%20Quick%20Search%20Box/2.0.0.1447 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (iMac11%2C2)";
const char S39608[] = "NewsFire/85 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (Macmini1%2C1)";
const char S39609[] = "WebProcess/6534.57.2 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookAir3%2C2)";
const char S39610[] = "URL%20Extractor/3.4.1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro5%2C2)";
const char S39611[] = "iCoreService (unknown version) CFNetwork/454.12.4 Darwin/10.8.0 (x86_64) (MacBookPro8%2C1)";
const char S39612[] = "iCoreService (unknown version) CFNetwork/454.12.4 Darwin/10.8.0 (i386) (iMac11%2C2)";
const char S39613[] = "WebProcess/6534.57.2 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacPro1%2C1)";
const char S39614[] = "ScreenSaverEngine/3.0.3 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro6%2C2)";
const char S39615[] = "iCoreService (unknown version) CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBook7%2C1)";
const char S39616[] = "Software%20Update (unknown version) CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro7%2C1)";
const char S39617[] = "iCoreService (unknown version) CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookAir3%2C2)";
const char S39618[] = "XProtectUpdater (unknown version) CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBook7%2C1)";
const char S39619[] = "XProtectUpdater (unknown version) CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookAir3%2C2)";
const char S39620[] = "sysagent/1.0 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro7%2C1)";
const char S39621[] = "Software%20Update/270 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro7%2C1)";
const char S39622[] = "ocspd (unknown version) CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro6%2C2)";
const char S39623[] = "sysagent/1.0 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (iMac11%2C3)";
const char S39624[] = "Software%20Update (unknown version) CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBook7%2C1)";
const char S39625[] = "iCoreService (unknown version) CFNetwork/454.12.4 Darwin/10.8.0 (i386) (Macmini4%2C1)";
const char S39626[] = "Mozilla/5.0 (Android; Mobile; rv:19.0) Gecko/19.0 Firefox/19.0";
const char S39627[] = "Mozilla/5.0 (Android; Mobile; rv:19.0a2) Gecko/19.0a2 Firefox/19.0a2";
const char S39628[] = "Mozilla/5.0 (Android; Mobile; rv:19.0.2) Gecko/19.0.2 Firefox/19.0.2";
const char S39629[] = "Mozilla/5.0 (Android; Mobile; rv:19.0.0) Gecko/19.0.0 Firefox/19.0.0";
const char S39630[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-B5330 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39631[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-B5330 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39632[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-B5330 Build/IMM76D)";
const char S39633[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; GT-B5330L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39634[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; SAMSUNG GT-B5330/B5330XWALI4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39635[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT-B5330B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39636[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; SAMSUNG GT-B5330/B5330XWALI2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39637[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; SAMSUNG GT-B5330/B5330XWALH6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39638[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-B5330L Build/IMM76D)";
const char S39639[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.1 (KHTML, like Gecko) Maxthon/3.0.8.2 Safari/533.1";
const char S39640[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; ) AppleWebKit/533.9 (KHTML, like Gecko) Maxthon/3.0 Safari/533.9";
const char S39641[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; ) AppleWebKit/534.12 (KHTML, like Gecko) Maxthon/3.0 Safari/534.12";
const char S39642[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12";
const char S39643[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12";
const char S39644[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1";
const char S39645[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1";
const char S39646[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12,gzip(gfe)";
const char S39647[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST23i Build/11.0.A.5.5) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39648[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST23a Build/11.0.A.2.16) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39649[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko)";
const char S39650[] = "Apple-iPhone3C1/801.293";
const char S39651[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko)";
const char S39652[] = "Apple-iPhone4C1/1001.403";
const char S39653[] = "Apple-iPhone4C1/1001.523";
const char S39654[] = "Apple-iPhone5C2/1001.525";
const char S39655[] = "Apple-iPhone3C1/1001.523";
const char S39656[] = "Apple-iPhone5C1/1001.525";
const char S39657[] = "Apple-iPhone5C2/1001.551";
const char S39658[] = "Apple-iPhone5C2/1002.146";
const char S39659[] = "Apple-iPhone4C1/1002.146";
const char S39660[] = "Apple-iPhone5C2/1002.329";
const char S39661[] = "Apple-iPhone3C3/1002.329";
const char S39662[] = "Apple-iPhone3C1/1002.329";
const char S39663[] = "Apple-iPhone4C1/1002.329";
const char S39664[] = "YellowPages-Intl/3.8.0-3349 (iPhone; iPhone OS; 6.1.3; en_CA)";
const char S39665[] = "Apple-iPhone5C1/1002.350";
const char S39666[] = "Apple-iPhone5C2/1002.350";
const char S39667[] = "Mozilla/5.0 (Linux; Android 4.0.3;  HTC One S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S39668[] = "Mozilla/5.0 (Linux; Android 4.0.3;  HTC VLE_U Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S39669[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC VLE_U Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39670[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39671[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39672[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC VLE_U Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39673[] = "Mozilla/5.0 (Linux; Android 4.0.4;  HTC One S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S39674[] = "Mozilla/5.0 (Linux; Android 4.0.4;  HTC VLE_U Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S39675[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One xs Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39676[] = "Opera/9.80 (X11; Linux x86_64; U; Ubuntu/10.10 (maverick); en) Presto/2.7.62 Version/11.01";
const char S39677[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.1 Safari/535.1";
const char S39678[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.4 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.4";
const char S39679[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S39680[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 7 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S39681[] = "Mozilla/5.0 (Linux; Android 4.2; Nexus 7 Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S39682[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T769 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39683[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-T769 Build/IMM76D)";
const char S39684[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us; Silk/1.1.0-84) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16 Silk-Accelerated=false";
const char S39685[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us; Silk/1.0.22.153_X) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16 Silk-Accelerated=false";
const char S39686[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/10.04 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19";
const char S39687[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/10.04 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19";
const char S39688[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/10.04 Chromium/18.0.1025.151 Chrome/18.0.1025.151 Safari/535.19,gzip(gfe) (via translate.google.com)";
const char S39689[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/10.04 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19,gzip(gfe)";
const char S39690[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.3) Gecko/X Firefox/15.3.2 PaleMoon/15.3.2";
const char S39691[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:15.3) Gecko/X Firefox/15.3.2-x64 PaleMoon/15.3.2-x64";
const char S39692[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:14.0) Gecko/X Firefox/14.0.1";
const char S39693[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:14.0) Gecko/X Firefox/14.0.1 Firefox/14.0.1";
const char S39694[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B329 Safari/8536.25";
const char S39695[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B146 Safari/8536.25";
const char S39696[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B141 Safari/8536.25";
const char S39697[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B144 Safari/8536.25";
const char S39698[] = "Mozilla/6.0 (Macintosh; I; Intel Mac OS X 11_7_9; de-LI; rv:1.9b4) Gecko/X Firefox/10.0a4";
const char S39699[] = "Mozilla/5.0 (Macintosh; I; Intel Mac OS X 11_7_9; de-LI; rv:1.9b4) Gecko/X Firefox/10.0a4";
const char S39700[] = "Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:12.0) Gecko/X Firefox/12.0";
const char S39701[] = "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:12.0) Gecko/X Firefox/12.0 Firefox/12.0";
const char S39702[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT897 Build/7.7.1Q-6_SPR-89_ASA-36) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39703[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT897 Build/7.7.1Q-6_SPR-125_CSP-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39704[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT897 Build/7.7.1Q-6_SPR-125_ASA-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39705[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT897 Build/7.7.1Q-6_SPR-125_CSP-14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39706[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT897 Build/7.7.1Q-6_SPR-125_ASA-14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39707[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; XT897 Build/7.7.1Q-6_SPR-125_ASA-14)";
const char S39708[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyEricssonLT26ii Build/6.1.A.2.45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39709[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT26ii Build/6.1.A.2.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39710[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26ii Build/6.1.A.2.45) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S39711[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT26ii Build/6.1.A.2.45)";
const char S39712[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT26ii Build/6.1.A.2.45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39713[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT26ii Build/6.1.A.2.50) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39714[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT26ii Build/6.1.A.2.55) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39715[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-; SAMSUNG-SGH-I897/I897UCJH7 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S39716[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SAMSUNG-SGH-I897/I897UCJF1 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S39717[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SAMSUNG-SGH-I897/I897UCJE5 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S39718[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-; SAMSUNG-SGH-I897/I897UCJF6 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S39719[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SAMSUNG-SGH-I897/I897UCJE1 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S39720[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SAMSUNG-SGH-I897 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S39721[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SAMSUNG-SGH-I897/I897UCJF6 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S39722[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; SAMSUNG-SGH-I897/I897UCJF6 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S39723[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SAMSUNG-SGH-I897/I897UCJH7 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S39724[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SAMSUNG-SGH-I897/I897UCJH3 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S39725[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SAMSUNG-SGH-I897/I897UCJH2 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S39726[] = "Mozilla/5.0 (Linux; U;Android 2.1-update1; en-us; SAMSUNG-SGH-I897/I897UCJH7 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S39727[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SAMSUNG-SGH-I897/I897UCJH7 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 MobileSafari/530.17";
const char S39728[] = "Mozilla/5.0 (Linux; U; Android2.1-update1; en-us; SAMSUNG-SGH-I897/I897UCJH7 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S39729[] = "Logos/0.9.1 (Linux; U; Android 2.1-update1; en_US; SAMSUNG-SGH-I897 Build/ECLAIR)";
const char S39730[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SAMSUNG-SGH-I897 Build/ECLAIR) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S39731[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SAMSUNG-SGH-I897 Build/ECLAIR)";
const char S39732[] = "HTTP_Request2/2.1.1 (http://pear.php.net/package/http_request2) PHP/5.3.2-1ubuntu4.15";
const char S39733[] = "HTTP_Request2/2.1.1 (http://pear.php.net/package/http_request2) PHP/5.3.2-1ubuntu4.17";
const char S39734[] = "HTTP_Request2/2.1.1 (http://pear.php.net/package/http_request2) PHP/5.3.2-1ubuntu4.18";
const char S39735[] = "HTTP_Request2/2.1.1 (http://pear.php.net/package/http_request2) PHP/5.3.2-1ubuntu4.19";
const char S39736[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.12";
const char S39737[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S39738[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3";
const char S39739[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10\\_4\\_11; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S39740[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I437 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39741[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I437 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39742[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-I437 Build/JZO54K)";
const char S39743[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I437Z Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39744[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I437P Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39745[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-I437P Build/JZO54K)";
const char S39746[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=expressatt;devm=SAMSUNG-SGH-I437;man=samsung;brw=Android;shwd=480x800;shwa=480x763;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S39747[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; XT890 Build/9.8.2I-50_SMI-26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39748[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT890 Build/9.8.2I-50_SML-20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39749[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT890 Build/9.8.2I-50_SMI-26)";
const char S39750[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; XT890 Build/9.8.2I-50_SMI-29) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39751[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT890 Build/9.8.2I-50_SML-25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39752[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT890 Build/9.8.2I-50_SML-23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39753[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; XT890 Build/9.8.2I-50_SMI-23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39754[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT890 Build/9.8.2I-50_SML-28) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39755[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT890 Build/9.8.2I-50_SML-29) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39756[] = "Mozilla/5.0 (Android; Mobile; rv:22.0) Gecko/22.0 Firefox/22.0";
const char S39757[] = "Mozilla/5.0 (Android; Mobile; rv:22.0a2) Gecko/22.0a2 Firefox/22.0a2";
const char S39758[] = "Mozilla/5.0 (Android; Mobile; rv:22.0a1) Gecko/22.0a1 Firefox/22.0a1";
const char S39759[] = "Mozilla/5.0 (Android; Mobile; rv:22.0) Gecko/22.0 Firefox/22.0,gzip(gfe) (via docs.google.com/viewer)";
const char S39760[] = "Mozilla/5.0 (Android; Mobile; rv:22.0) Gecko/22.0 Firefox/22.0,gzip(gfe)";
const char S39761[] = "Mozilla/5.0 (Android; Mobile; rv:21.0) Gecko/21.0 Firefox/21.0";
const char S39762[] = "Mozilla/5.0 (Android; Mobile; rv:21.0a2) Gecko/21.0a2 Firefox/21.0a2";
const char S39763[] = "Mozilla/5.0 (Android; Mobile; rv:21.0) Gecko/21.0 Firefox/21.0,gzip(gfe) (via docs.google.com/viewer)";
const char S39764[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 620)";
const char S39765[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 620; Vodafone)";
const char S39766[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia 620)";
const char S39767[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 620; ANZ846)";
const char S39768[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 620; Meteor)";
const char S39769[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6330VW Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39770[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; ADR6330VW Build/GRJ22)";
const char S39771[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;ADR6330VW Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39772[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; ADR6330VW Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39773[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6330VW Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S39774[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; ADR6330VW Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39775[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; MB525 Build/4.5.1_85) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39776[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME525+ Build/4.5.3-66) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39777[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB525 Build/4.5.1-134_DFP-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39778[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; MB525 Build/4.5.1_112) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39779[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME525+ Build/4.5.3-109_DPP-6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39780[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; ME525+ Build/4.5.2-109_DHT-17) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39781[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB525 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39782[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB525 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39783[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; MB525 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39784[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; ME525+ Build/4.5.2-109_DHT-25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39785[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; MB525 Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30";
const char S39786[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; MB525 Build/GWK74)";
const char S39787[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MB525 Build/GWK74) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S39788[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MB525 Build/WajkIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39789[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME525+ Build/4.5.3-109_DPP-14) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S39790[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME525+ Build/4.5.3-109_DPP-11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39791[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME525+ Build/4.5.3-109_DPP-11) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S39792[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME525+ Build/4.5.3-109_DPP-14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39793[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME525+ Build/4.5.3-109_DPP-6) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S39794[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MB525 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S39795[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; ME525+ Build/4.5.2-109_DHT-22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39796[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME525+ Build/4.5.3-109_DPP-6) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S39797[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MB525 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S39798[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME525+ Build/4.5.3-109_DPP-11) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S39799[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME525+ Build/4.5.3-109_DPP-17) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39800[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME525+ Build/4.5.3-109_DPP-14) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S39801[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; MB525 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39802[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; MB525 Build/GRJ22)";
const char S39803[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MB525 Build/VIPFXY) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39804[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; MB525 Build/GRJ90)";
const char S39805[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MB525 Build/VIPFXY) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39806[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MB525 Build/VIPFXY) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S39807[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB525 Build/4.5.2-109_DHT-17) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39808[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; iw-il; MB525 Build/HebMIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39809[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME525+ Build/4.5.2-109_DHT-17) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S39810[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; ME525+ Build/4.5.2-109_DHT-25)";
const char S39811[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME525+ Build/4.5.3-109_DPP-17) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S39812[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MB525 Build/4.5.2-109_DHT-25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39813[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME525+ Build/4.5.3-109_DPP-17) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S39814[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT555C Build/V1.67D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39815[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME525+ Build/4.5.2-109_DHT-25) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S39816[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; XT555C Build/V1.67D)";
const char S39817[] = "Mozilla 5.0 (Linux; U; Android 2.3.7; zh-cn; MB525 Build MIUI) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S39818[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; ME525+ Build/4.5.3-109_DPP-6)";
const char S39819[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; MB525 Build/4.5.1-134_DFP-231) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39820[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T999 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S39821[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S39822[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I535 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S39823[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9300 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S39824[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S39825[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-T999 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S39826[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1A543 Safari/419.3";
const char S39827[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1A538a Safari/419.3";
const char S39828[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C25 Safari/419.3";
const char S39829[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1A543a Safari/419.3";
const char S39830[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C28 Safari/419.3";
const char S39831[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/3B48b Safari/419.3";
const char S39832[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/3B48a Safari/419.3";
const char S39833[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/3A109a Safari/419.3";
const char S39834[] = "Mozilla/5.0 (iPhone; U; XXXXX like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/241 Safari/419.3";
const char S39835[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/4A102 Safari/419.3";
const char S39836[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/4A93 Safari/419.3";
const char S39837[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1A537a Safari/419.3";
const char S39838[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/3A110a Safari/419.3";
const char S39839[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/8B117";
const char S39840[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1A498b Safari/419.3";
const char S39841[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/4A102 Safari/419 (United States)";
const char S39842[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C25 Safari/419.3 Safari/525.20";
const char S39843[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1A535b Safari/419.3";
const char S39844[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/u15 Safari/5";
const char S39845[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/A2DUs2 Safari/5";
const char S39846[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1A498b Safari/4.03";
const char S39847[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; XT860 Build/5.5.1-112_SLU-61.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39848[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; XT860 Build/5.5.1-112_SLU-54.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39849[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; XT860 Build/5.5.1-112_SLU-60) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39850[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT860 Build/5.5.1-112_SLU-57M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39851[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; XT860 Build/5.5.1-112_SLU-47.6M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39852[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT860 Build/5.5.1-112_SLU-56M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39853[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; ME863 Build/SLNCU-60.2.200) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39854[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; XT860 Build/5.5.1-112_SLU-89) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39855[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT860 Build/5.5.1-112_SLU-79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39856[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT860 Build/5.5.1-112_SLU-88) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39857[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; ME863 Build/SLNCU-60.1.103) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39858[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; XT860 Build/5.5.1-112_SLU-44M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39859[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; XT860 Build/5.5.1-112_SLU-39M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39860[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; XT860 Build/5.5.1-112_SLU-42M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39861[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; XT860 Build/5.5.1-112_SLU-54.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39862[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; XT860 Build/5.5.1-112_SLU-47.6M)";
const char S39863[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ME863 Build/SLNCU-60.1.101) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39864[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ME863 Build/SLNCU-60.1.102) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39865[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT860 Build/5.5.1-112_SLU-87) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39866[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MILESTONE3 Build/5.5.1.1-84_D3G-25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39867[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; MILESTONE3 Build/5.5.1.1-84_D3G-25)";
const char S39868[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; ME863 Build/SLNCU-60.2.200)";
const char S39869[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; XT860 Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39870[] = "Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; ME863 Build SLNCU-60.1.102) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S39871[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5363 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39872[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-S5363 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39873[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5363 Build/GINGERBREAD)";
const char S39874[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-S5363 Build/GINGERBREAD)";
const char S39875[] = "Mozilla/5.0 (Linux; U; Android 2.3.6_mobile-star; iw-il; GT-S5363 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39876[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5363 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39877[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.4.2 Mobile/10A523 Safari/8536.25";
const char S39878[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.4.2 Mobile/10A403 Safari/8536.25";
const char S39879[] = "Opera/9.80 (BREW; Opera Mini/5.0/27.2338; U; en) Presto/2.8.119 320X240 Samsung SCH-U380";
const char S39880[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Twitter for iPhone";
const char S39881[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Twitter for iPhone";
const char S39882[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A407 Twitter for iPhone";
const char S39883[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A406 Twitter for iPhone";
const char S39884[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8426 Twitter for iPhone";
const char S39885[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8500 Twitter for iPhone";
const char S39886[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550 Twitter for iPhone";
const char S39887[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; SV1)";
const char S39888[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; MSN Optimized)";
const char S39889[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; SV1)";
const char S39890[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; MS-RTC LM 8)";
const char S39891[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; SV1; SV1)";
const char S39892[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; SV1; SV1)";
const char S39893[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V8000_USA_Cricket Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39894[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; V8000_USA_Cricket Build/IMM76D)";
const char S39895[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N8000_USA_Cricket Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39896[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; N8000_USA_Cricket Build/IMM76I)";
const char S39897[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.4.2 Mobile/10A523 Safari/8536.25";
const char S39898[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.4.2 Mobile/10A403 Safari/8536.25";
const char S39899[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39900[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14)";
const char S39901[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; XT910 Build/9.8.2O-124_SPUL-18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39902[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MOTOROLA RAZR Build/9.8.2O-72_VZW-8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39903[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT910 Build/9.8.2O-124_SPUL-17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39904[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MOT-XT910 Build/6.7.2_GC-384) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39905[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MOT-XT910 Build/6.7.2_GC-384-SPDU-801) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39906[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT910 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39907[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; XT910 Build/6.7.2_GC-384) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39908[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; XT910 Build/6.7.2_GC-384-SPDU-801) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39909[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT910 Build/9.8.2O-72_VZW-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39910[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9B176 Safari/7534.48.3";
const char S39911[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9B208 Safari/7534.48.3";
const char S39912[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9B179 Safari/7534.48.3";
const char S39913[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9B206 Safari/7534.48.3";
const char S39914[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9Z999 Safari/7534.48.3";
const char S39915[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Slider SL101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39916[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Slider SL101 Build/IML74K)";
const char S39917[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3;) Gecko/X Firefox/3.0";
const char S39918[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6 ; nl; rv:1.9) Gecko/X Firefox/3.0";
const char S39919[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.2a1pre) Gecko/X Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.5; en-US; rv:1.9.0.7) Gecko/X Firefox/3.0.7";
const char S39920[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.1b1) Gecko/X Firefox/3.0";
const char S39921[] = "Mozilla/4.0 (compatible; MSIE 5.01; Windows 95; MSIECrawler)";
const char S39922[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows 95) VoilaBot; 1.6";
const char S39923[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows 95) VoilaBot BETA 1.2 (http://www.voila.com/)";
const char S39924[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Twitter for iPhone";
const char S39925[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Twitter for iPhone";
const char S39926[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Twitter for iPhone";
const char S39927[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Twitter for iPhone";
const char S39928[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Twitter for iPhone";
const char S39929[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Twitter for iPhone";
const char S39930[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SPH-L720 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S39931[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-R760 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39932[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-R760 Build/GINGERBREAD)";
const char S39933[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Avant Browser)";
const char S39934[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; Avant Browser)";
const char S39935[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Win64; x64; Trident/6.0; Avant Browser)";
const char S39936[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39937[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P768 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39938[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P769 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39939[] = "Mozilla/5.0 (Linux; Android 4.1.2; LGMS769 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S39940[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S39941[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S39942[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S39943[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S39944[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9100 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S39945[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S39946[] = "Mozilla/5.0 (compatible; Firefox compatible; MS IE compatible; +http://search.daum.net/) Daumoa-feedfetcher/2.0";
const char S39947[] = "Mozilla/5.0 (compatible; MSIE or Firefox mutant; not on Windows server; + http://tab.search.daum.net/aboutWebSearch.html) Daumoa/3.0";
const char S39948[] = "NokiaN95_8GB-3;Mozilla/5.0 SymbianOS/9.2;U; Series60/3.1 NokiaN95_8GB-3/11.2.011 Profile/MIDP-2.0 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML, like Gecko)";
const char S39949[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaN95_8GB-3/1.2.011 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S39950[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaN95_8GB/20.0.016; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S39951[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Next8P12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39952[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Next8P12 Build/IML74K)";
const char S39953[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; NEXT8P Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39954[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Next8P12 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39955[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Next8P12 Build/IMM76I)";
const char S39956[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NEXT8P12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S39957[] = "Mozilla/5.0 (X11; Linux i686 on x86_64; rv:18.0) Gecko/X Firefox/18.0";
const char S39958[] = "Mozilla/5.0 (X11; Linux i686; rv:18.0) Gecko/X Firefox/18.0 Iceweasel/18.0.1";
const char S39959[] = "Mozilla/5.0 (Ultimate Edition; X11; Linux x64; rv:8.0) Gecko/X Ultimate Edition / 3.4 Firefox/18.0";
const char S39960[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaN95-5/22.1.002; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S39961[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaN95-3/20.2.011 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S39962[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaN95/35.0.002; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S39963[] = "NokiaN95";
const char S39964[] = "NOKIAN95 8GB";
const char S39965[] = "NokiaN95/1.0(15.1.002)SymbianOS/9.2 Series60/3.01 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S39966[] = "Mozilla/5.0 (Series40; NokiaC3-00/so6.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15";
const char S39967[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A4449a Safari/9537.53";
const char S39968[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A4449d Safari/9537.53";
const char S39969[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A465 Safari/9537.53";
const char S39970[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A501 Safari/9537.53";
const char S39971[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Nexus One Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39972[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Nexus One Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39973[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39974[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; NexusOne Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39975[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; NexusOne Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39976[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; Nexus One Build/FRF50) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39977[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus One Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S39978[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus One Build/FRG33) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39979[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Nexus One Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39980[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; Nexus One Build/FRG83D)";
const char S39981[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Nexus One Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39982[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus One Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39983[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus One Build/FRF83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39984[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Nexus One Build/RoDrIgUeZsTyLe MIUI-MOD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39985[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en_us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39986[] = "Mozilla/5.0 (Linux; U; PocketEdge FROYO; Android 2.2; en-us; Nexus One Build/FRG83) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S39987[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en; Nexus One Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39988[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; Nexus One Build/FRF91)";
const char S39989[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus One Build/FRF51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39990[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus One Build/FRF58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39991[] = "Mozilla/5.0(Linux;U;Android 2.2.1;en-us;Nexus One Build.FRG83) AppleWebKit/553.1(KHTML,like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39992[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S39993[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39994[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus One Build/ FRF91) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39995[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; A1_07 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S39996[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; A1_07 Build/GRJ22)";
const char S39997[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; A1_07 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S39998[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; A1_07 Build/GRJ22; 600*1024) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S39999[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; A1_07 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S40000[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; T-Mobile myTouch Build/HuaweiU8680) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40001[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; T-Mobile myTouch Build/HuaweiU8680)";
const char S40002[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us;T-Mobile myTouch Build/HuaweiU8680) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40003[] = "Mozilla/5.0 (Linux; U; Android2.3.6; en-us; T-Mobile myTouch Build/HuaweiU8680) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40004[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; T-Mobile myTouch Build/HuaweiU8680) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S40005[] = "Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; T-Mobile myTouch Build/HuaweiU8680) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40006[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; T-MobilemyTouch Build/HuaweiU8680) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40007[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; T-Mobile myTouchBuild/HuaweiU8680) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40008[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; ST27i Build/6.2.A.0.400) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40009[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-nz; ST27a Build/6.2.A.0.400) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40010[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyST27i Build/6.2.A.1.100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40011[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; ST27i Build/6.2.A.1.100)";
const char S40012[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; ST27i Build/6.2.A.1.100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40013[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyST27a Build/6.2.A.1.100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40014[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; ST27a Build/6.2.A.1.100)";
const char S40015[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-za; SonyST27iv Build/6.2.A.1.100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40016[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; ST27a Build/6.2.A.1.100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40017[] = "SAMSUNG-GT-E1282T Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.31524/30.3697; U; en) Presto/2.8.119 Version/11.10";
const char S40018[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Safari/535.19 Silk-Accelerated=true";
const char S40019[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC EVO 3D X515m Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40020[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC_EVO3D_X515m Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40021[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; HTC_EVO3D_X515a Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40022[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_Kingdom Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40023[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; vi-vn; HTC_Kingdom Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40024[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; HTC_X515E Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40025[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; HTC_X515a V1.24.841.2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40026[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; HTC EVO 3D X515a Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40027[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; HTC_X515E Build/GRK39F)";
const char S40028[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-kr; HTC_X515E Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40029[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HTC X515d) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40030[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC_X515E Build/GRJ22)";
const char S40031[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; HTC EVO 3D X515a Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30";
const char S40032[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC EVO 3D X515m Build/GRJ22)";
const char S40033[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC EVO 3D X515a Build/GRJ22)";
const char S40034[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC EVO 3D X515m Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S40035[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; HTC_X515a V1.24.841.3 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40036[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC EVO 3D X515m Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S40037[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC X515m Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S40038[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC X515m Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40039[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC EVO 3D X515m Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S40040[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HTC-X515d/1.09.1401.13) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40041[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC X515m Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S40042[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC EVO 3D X515m Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S40043[] = "Mozilla 5.0 (Linux; U; Android 2.3.4; zh-cn; HTC EVO 3D X515m Build GRJ22) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S40044[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC EVO 3D X515m Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40045[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HTC X515d Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S40046[] = "Nokia5530/32.0.007 (SymbianOS/9.4; U; Series60/5.0 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Safari/525 3gpp-gba";
const char S40047[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5530c-2/32.0.007; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.9 3gpp-gba";
const char S40048[] = "Nokia5530";
const char S40049[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5530c-2/40.0.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 Mobile Safari/533.4 3gpp-gba";
const char S40050[] = "Mozilla/5.0 (Android; Mobile; rv:14.0) Gecko/14.0 Firefox/14.0a2";
const char S40051[] = "Mozilla/5.0 (Android; Mobile; rv:14.0) Gecko/14.0 Firefox/14.0a1";
const char S40052[] = "Mozilla/5.0 (Android; Mobile; rv:14.0) Gecko/14.0 Firefox/14.0";
const char S40053[] = "Mozilla/5.0 (Android; Mobile; rv:14.0) Gecko/14.0 Fennec/14.0";
const char S40054[] = "Mozilla/5.0 (Android; Mobile; rv:14.0) Gecko/14.0 Firefox/14.0.1";
const char S40055[] = "Mozilla/5.0 (Android; Mobile; rv:14.0) Gecko/14.0 Firefox/14.0.2";
const char S40056[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A400 Safari/6531.22.7";
const char S40057[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293 Safari/6531.22.7";
const char S40058[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 40 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML like Gecko) Version/4.0.5 Mobile/8A293 Safari/6531.22.7";
const char S40059[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 402 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML like Gecko) Version/4.0.5 Mobile/8A400 Safari/6531.22.7";
const char S40060[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36";
const char S40061[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S40062[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S40063[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S40064[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S40065[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9100 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S40066[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B144 Safari/8536.25";
const char S40067[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B141 Safari/8536.25";
const char S40068[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B146 Safari/8536.25";
const char S40069[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B329 Safari/8536.25";
const char S40070[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9100 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S40071[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9100G Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S40072[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S40073[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S40074[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S40075[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9100 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S40076[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100M Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S40077[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S40078[] = "Mozilla/5.0 (Linux; Android 4.1.2;  GT-I9100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S40079[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E980/E98008e Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40080[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E980 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40081[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E980/E98010g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40082[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E980 Build/JZO54K)";
const char S40083[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E980/E98008g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40084[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E980/E98010k Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40085[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E980/E98010i Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40086[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=geefhd;devm=LG-E980;man=LGE;brw=Android;shwd=1080x1920;shwa=1080x1845;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S40087[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ZTE-Z990G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40088[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; ZTE-Z990G Build/GRJ22)";
const char S40089[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; -us; ZTE-Z990G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40090[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:10.0) Gecko/X Firefox/10.0";
const char S40091[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S40092[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:10.0.7) Gecko/X Firefox/10.0.7";
const char S40093[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A4372q";
const char S40094[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko)";
const char S40095[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A4372q [FBAN/FBIOS;FBAV/6.2;FBBV/X;FBDV/iPod5,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/7.0;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBOP/1]";
const char S40096[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A4400f";
const char S40097[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A4400f [FBAN/FBIOS;FBAV/6.2;FBBV/X;FBDV/iPod5,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/7.0;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBOP/1]";
const char S40098[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Mobile/11A4414e";
const char S40099[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Mobile/11A4414e [FBAN/FBIOS;FBAV/6.3;FBBV/X;FBDV/iPod5,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/7.0;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBOP/1]";
const char S40100[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4435d [FBAN/FBIOS;FBAV/6.3;FBBV/X;FBDV/iPod5,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/7.0;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBOP/1]";
const char S40101[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4435d";
const char S40102[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449a [FBAN/FBIOS;FBAV/6.3;FBBV/X;FBDV/iPod5,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/7.0;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBOP/1]";
const char S40103[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449a";
const char S40104[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d";
const char S40105[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d [FBAN/FBIOS;FBAV/6.4;FBBV/X;FBDV/iPod5,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/7.0;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBOP/1]";
const char S40106[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465";
const char S40107[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 [FBAN/FBIOS;FBAV/6.5;FBBV/X;FBDV/iPod5,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/7.0;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBOP/5]";
const char S40108[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko)";
const char S40109[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 iPod5,1 BingWeb/4.1.1502.X";
const char S40110[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501";
const char S40111[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 [FBAN/FBIOS;FBAV/6.5.1;FBBV/X;FBDV/iPod5,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/7.0.2;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBOP/5]";
const char S40112[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-T839 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40113[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;SGH-T839 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40114[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-T839 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S40115[] = "Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; SGH-T839 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40116[] = "Mozilla/5.0 (Linux; U; Android2.2.1; en-us; SGH-T839 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40117[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-T839 Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S40118[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SGH-T839 Build/FROYO)";
const char S40119[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SGH-T839 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40120[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SGH-T839 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40121[] = "Mozilla/5.0 (X11; CrOS i686 2430.0.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1172.0 Safari/537.1";
const char S40122[] = "Mozilla/5.0 (X11; CrOS i686 2465.30.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.18 Safari/537.1";
const char S40123[] = "Mozilla/5.0 (X11; CrOS i686 2465.62.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.41 Safari/537.1";
const char S40124[] = "Mozilla/5.0 (X11; CrOS x86_64 2465.82.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.50 Safari/537.1";
const char S40125[] = "Mozilla/5.0 (X11; CrOS i686 2465.82.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.50 Safari/537.1";
const char S40126[] = "Mozilla/5.0 (X11; CrOS i686 2465.97.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.55 Safari/537.1";
const char S40127[] = "Mozilla/5.0 (X11; CrOS i686 2465.105.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.68 Safari/537.1";
const char S40128[] = "Mozilla/5.0 (X11; CrOS x86_64 2465.116.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.78 Safari/537.1";
const char S40129[] = "Mozilla/5.0 (X11; CrOS x86_64 2465.105.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.68 Safari/537.1";
const char S40130[] = "Mozilla/5.0 (X11; CrOS x86_64 2465.127.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1";
const char S40131[] = "Mozilla/5.0 (X11; CrOS i686 2465.127.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1";
const char S40132[] = "Mozilla/5.0 (X11; CrOS i686 2465.121.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.79 Safari/537.1";
const char S40133[] = "Mozilla/5.0 (X11; CrOS x86_64 2465.121.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.79 Safari/537.1";
const char S40134[] = "Mozilla/5.0 (X11; CrOS i686 2465.116.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.78 Safari/537.1";
const char S40135[] = "Mozilla/5.0 (X11; CrOS i686 2465.163.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.91 Safari/537.1";
const char S40136[] = "Mozilla/5.0 (X11; CrOS x86_64 2465.142.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S40137[] = "Mozilla/5.0 (X11; CrOS x86_64 2465.148.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S40138[] = "Mozilla/5.0 (X11; CrOS i686 2465.142.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S40139[] = "Mozilla/5.0 (X11; CrOS x86_64 2465.163.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.91 Safari/537.1";
const char S40140[] = "Mozilla/5.0 (X11; CrOS x86_64 2465.211.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.92 Safari/537.1";
const char S40141[] = "Mozilla/5.0 (X11; CrOS i686 2465.211.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.92 Safari/537.1";
const char S40142[] = "Mozilla/5.0 (X11; CrOS x86_64 2465.173.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.91 Safari/537.1";
const char S40143[] = "Mozilla/5.0 (X11; CrOS i686 2465.227.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.92 Safari/537.1";
const char S40144[] = "Mozilla/5.0 (X11; CrOS x86_64 2465.228.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.92 Safari/537.1";
const char S40145[] = "Mozilla/5.0 (X11; CrOS x86_64 2465.227.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.92 Safari/537.1";
const char S40146[] = "Mozilla/5.0 (X11; CrOS x86_64 2465.238.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.92 Safari/537.1";
const char S40147[] = "Mozilla/5.0 (Android; Mobile; rv:18.0) Gecko/18.0 Firefox/18.0";
const char S40148[] = "Mozilla/5.0 (Android; Mobile; rv:18.0a2) Gecko/18.0a2 Firefox/18.0a2";
const char S40149[] = "Mozilla/5.0 (Android; Mobile; rv:18.0a1) Gecko/18.0a1 Firefox/18.0a1";
const char S40150[] = "Mozilla/5.0 (Android; Mobile; rv:18.0.2) Gecko/18.0.2 Firefox/18.0.2";
const char S40151[] = "Mozilla/5.0 (Android; Mobile; rv:18.0) Gecko/18.0 Firefox/18.0,gzip(gfe) (via docs.google.com/viewer)";
const char S40152[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6603 Build/10.3.A.0.423) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S40153[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6603 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S40154[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; GT-S5670L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40155[] = "Mozilla/5.0(Linux;U;Android2.2.1;Zh_cn;GT-S5670;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S40156[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; GT-S5670B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40157[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5670 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40158[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ar-ae; GT-S5670 Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S40159[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-S5670 Build/FROYO)";
const char S40160[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5670 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S40161[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-S5670L Build/FROYO)";
const char S40162[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-; GT-S5670 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40163[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-S5670 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S40164[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19 AvantBrowser/Tri-Core";
const char S40165[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5 AvantBrowser/Tri-Core";
const char S40166[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11 AvantBrowser/Tri-Core";
const char S40167[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 AvantBrowser/Tri-Core";
const char S40168[] = "Mozilla/5.0 (Windows NT 5.1; Avant TriCore) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S40169[] = "Mozilla/5.0 (Windows NT 5.1; Avant TriCore) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.96 Safari/537.4";
const char S40170[] = "Mozilla/5.0 (Windows NT 5.1; Avant TriCore) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S40171[] = "Mozilla/5.0 (Windows NT 5.1; Avant TriCore) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S40172[] = "Mozilla/5.0 (Windows NT 5.1; Avant TriCore) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S40173[] = "Mozilla/5.0 (Windows NT 5.1; Avant TriCore) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S40174[] = "Mozilla/5.0 (Windows NT 5.1; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36";
const char S40175[] = "Mozilla/5.0 (Windows NT 5.1; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36";
const char S40176[] = "Mozilla/5.0 (Windows NT 5.1; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36";
const char S40177[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.2.28) Gecko/X Firefox/3.6.28";
const char S40178[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6) Gecko/X Firefox/5.0.1";
const char S40179[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:5.0.1) Gecko/X Firefox/5.0.1 SmartLinksAddon";
const char S40180[] = "Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; 001DL Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40181[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Dell Streak Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40182[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; Dell Streak Build/FRF91)";
const char S40183[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-; Dell Streak Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40184[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Dell Streak Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40185[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Dell Streak Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40186[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-; Dell Streak Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40187[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; 001DL Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40188[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; Dell Streak Build/FRG83G)";
const char S40189[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; 001DL Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40190[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; Dell Streak Build/FRG83)";
const char S40191[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; 001DL Build/FRG83G)";
const char S40192[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Dell Streak Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S40193[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; Dell Streak Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S40194[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Dell Streak Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S40195[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40196[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40197[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40198[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40199[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; SPH-M910 Build/FROYO)";
const char S40200[] = "Logos/0.9.4 (Linux; U; Android 2.2.2; en_US; SPH-M910 Build/FROYO.EC07)";
const char S40201[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SPH-M910 Build/FROYO)";
const char S40202[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40203[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.3; SPH-M910 Build/FROYO)";
const char S40204[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; en-; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40205[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST26i Build/11.0.A.7.5) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S40206[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST26a Build/11.0.A.7.5) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S40207[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ThinkPad Tablet Build/ThinkPadTablet_A400_03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S40208[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; ThinkPad Tablet Build/ThinkPadTablet_A400_03)";
const char S40209[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; ThinkPad Tablet Build/ThinkPadTabletA40003) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Safari/534.30";
const char S40210[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ThinkPad Tablet Build/waynetss_A400_03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S40211[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P769 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S40212[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P760 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S40213[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P768 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S40214[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S40215[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S40216[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S40217[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S40218[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S40219[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S40220[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S40221[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_5) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.35 Safari/535.1";
const char S40222[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.1 (KHTML	 like Gecko) Chrome/14.0.803.0 Safari/535.1";
const char S40223[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/14.0.648.205 Safari/534.16";
const char S40224[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/14.0.648.205 Safari/534.16";
const char S40225[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Avant Browser)";
const char S40226[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Avant Browser)";
const char S40227[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Win64; x64; Trident/5.0; Avant Browser)";
const char S40228[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; Avant Browser)";
const char S40229[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; Avant Browser; Avant Browser)";
const char S40230[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; Avant Browser; SV1)";
const char S40231[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; SV1; Avant Browser)";
const char S40232[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; Avant Browser; Avant Browser)";
const char S40233[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I5500 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S40234[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; GT-I5500B Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S40235[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; GT-I5500L Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S40236[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; GT-I5500M Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S40237[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; GT-I5500 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S40238[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I5500 Build/ERE27)";
const char S40239[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I5500B Build/ERE27)";
const char S40240[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I5500L Build/ERE27)";
const char S40241[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I5500M Build/ECLAIR)";
const char S40242[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-R920 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40243[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-R920 Build/GINGERBREAD)";
const char S40244[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/12.10 Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/537.4";
const char S40245[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/12.10 Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/537.4";
const char S40246[] = "Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/12.10 Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/537.4";
const char S40247[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/12.10 Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)";
const char S40248[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; en) Opera 10.62";
const char S40249[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B146 Safari/8536.25";
const char S40250[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B144 Safari/8536.25";
const char S40251[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B141 Safari/8536.25";
const char S40252[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B329 Safari/8536.25";
const char S40253[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S40254[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One 801e Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S40255[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC_PN071 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S40256[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTCONE Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S40257[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1";
const char S40258[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1";
const char S40259[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1";
const char S40260[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1";
const char S40261[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.31 (KHTML, like Gecko) Chrome/13.0.748.0 Safari/534.31";
const char S40262[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1";
const char S40263[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.215 Safari/535.1";
const char S40264[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_5) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.772.0 Safari/535.1";
const char S40265[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/13.0.648.205 Safari/534.16";
const char S40266[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/13.0.648.205 Safari/534.16";
const char S40267[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9B206 Safari/7534.48.3";
const char S40268[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9B176 Safari/7534.48.3";
const char S40269[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8500/S8500XXLA1; U; Bada/2.0; en-us) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile WVGA SMM-MMS/1.2.0 OPN-B";
const char S40270[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Flyer_P510e; ru-ru) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S40271[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Flyer_P512; nl-nl) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S40272[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Flyer_P512_NA; en-ca) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S40273[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTCP510e/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1; zh-cn) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S40274[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HUAWEI-M920 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40275[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; HUAWEI-M920 Build/GINGERBREAD)";
const char S40276[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; -us; HUAWEI-M920 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40277[] = "Mozilla/5.0 (Windows NT 5.1; rv:16.0) Gecko/X SeaMonkey/2.13.1";
const char S40278[] = "Mozilla/5.0 (Windows NT 5.1; rv:16.0) Gecko/X Firefox/16.0 SeaMonkey/2.13.2";
const char S40279[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:16.0) Gecko/X Firefox/16.0 SeaMonkey/2.13.2";
const char S40280[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Safari/535.19 Silk-Accelerated=true";
const char S40281[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.0.3) Gecko/X Ubuntu/8.04 (hardy) Firefox/3.0.3 (Linux Mint)";
const char S40282[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.2) Gecko/X Ubuntu/1.4.0 (hardy) Firefox/3.0.2";
const char S40283[] = "Mozilla/5.0 (X11; U; Linux i686; fr; rv:1.9.0.3) Gecko/X Ubuntu/8.04 (hardy) Firefox/3.03";
const char S40284[] = "Mozilla/5.0 (X11; U; Linux i686; de; rv:1.9.0.13) Gecko/X Ubuntu/8.04 (hardy) Firefox/3.0.13";
const char S40285[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.1) Gecko/X Ubuntu/8.04 (hardy) Firefox/3.0.3 (Linux Mint)";
const char S40286[] = "Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.10.229 Version/11.64";
const char S40287[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.64";
const char S40288[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5310); ru) Presto/2.10.229 Version/11.64";
const char S40289[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.4 (build 02647); ru) Presto/2.10.229 Version/11.64";
const char S40290[] = "Opera/9.80 (Windows NT 6.1; U; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0); en) Presto/2.10.229 Version/11.64";
const char S40291[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.64";
const char S40292[] = "Opera/9.80 (Windows NT 5.1; U; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0); en) Presto/2.10.229 Version/11.64";
const char S40293[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.64";
const char S40294[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.64";
const char S40295[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5339); ru) Presto/2.10.229 Version/11.64";
const char S40296[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; en) Opera 11.64";
const char S40297[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.64";
const char S40298[] = "Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5680); ru) Presto/2.10.229 Version/11.64";
const char S40299[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5265); ru) Presto/2.10.229 Version/11.64";
const char S40300[] = "Opera/9.80 (BlackBerry; Opera Mini/4.4.31892/30.3214; U; en) Presto/2.8.119 Version/11.10";
const char S40301[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9) Gecko Minefield/3.0";
const char S40302[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.6) Gecko/X Minefield/3.0.6";
const char S40303[] = "Mozilla/5.0(X11;U;Linuxi686;en-US;rv:1.9a1)Gecko/XMinefield/3.0a1";
const char S40304[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:13.0) Gecko/X Firefox/13.0.1";
const char S40305[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MT27i Build/6.1.1.B.1.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40306[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MT27i Build/6.1.1.B.1.54)";
const char S40307[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; SonyEricssonMT27i Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40308[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MT27i Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40309[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S40310[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P768 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S40311[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S40312[] = "Mozilla/5.0 (Linux; Android 4.1.2; LGMS769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S40313[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LGL55C Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40314[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; LGL55C Build/GRJ22) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S40315[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LGL55C Build/GRJ22)";
const char S40316[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LGL55C Build/AreaRomQ v2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40317[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LGL55C Build/AreaRomQ v1.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40318[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LGL55c Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40319[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LGL55C Build/AreaRomQ v1.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40320[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LGL55C Build/AreaRomQ v2.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40321[] = "Mozilla/5.0 (Linux; Android 4.3; Galaxy Nexus Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S40322[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S40323[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545L Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S40324[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Zio Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40325[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Zio Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40326[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; Zio Build/FRG83)";
const char S40327[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; Zio Build/FRF91)";
const char S40328[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Zio Build/ERD79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40329[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/12.0.742.91; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S40330[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/12.0.742.91) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.91 Safari/534.30";
const char S40331[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/12.0.742.100) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30";
const char S40332[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/12.0.742.122; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S40333[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/12.0.742.122) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S40334[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/12.0.742.122)";
const char S40335[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/12.0.742.112) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30";
const char S40336[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/12.0.742.122; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C)";
const char S40337[] = "Mozilla/5.0 (Windows NT 5.2; chromeframe/12.0.742.122) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S40338[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/12.0.742.122; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)";
const char S40339[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/12.0.742.30) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.30 Safari/534.30";
const char S40340[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ME301T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S40341[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; ME301T Build/JOP40D)";
const char S40342[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:19.0) Gecko/X Firefox/19.0 PaleMoon/19.0.2-x64";
const char S40343[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:19.0) Gecko/X Firefox/19.0 PaleMoon/19.0.2";
const char S40344[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; U8180 Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40345[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; Cataclysm Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40346[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; Starshine Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40347[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; DroniX-0.5 Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40348[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-ph; Netphone 501 Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40349[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; cs-cz; IDEOS X1 Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40350[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; nl-be; Stockholm Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40351[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; U8180 Build/HuaweiU8180) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S40352[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; U8180 Build/HuaweiU8180)";
const char S40353[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; Huawei Ideos X1 Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40354[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ru-ru; MegaFon U8180 Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40355[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ALCATEL_one_touch_995 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40356[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ALCATEL_one_touch_995S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40357[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ALCATEL_one_touch_995A Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40358[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A4372q Twitter for iPhone";
const char S40359[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A4400f Twitter for iPhone";
const char S40360[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Mobile/11A4414e Twitter for iPhone";
const char S40361[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4435d Twitter for iPhone";
const char S40362[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449a Twitter for iPhone";
const char S40363[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d Twitter for iPhone";
const char S40364[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Twitter for iPhone";
const char S40365[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A466 Twitter for iPhone";
const char S40366[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A470a Twitter for iPhone";
const char S40367[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Twitter for iPhone";
const char S40368[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Next7P12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40369[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Next7P12 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S40370[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Next7P12-8G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S40371[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Next7P12 Build/IMM76I)";
const char S40372[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Next7P12-8G Build/IMM76I)";
const char S40373[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Transformer TF101G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S40374[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Transformer TF101G Build/IML74K)";
const char S40375[] = "Mozilla/5.0 (X11; Linux x86_64; rv:12.0) Gecko/X Firefox/12.0";
const char S40376[] = "Mozilla/5.0 (X11; Linux i686; rv:12.0) Gecko/X Firefox/12.0 Iceweasel/12.0";
const char S40377[] = "Mozilla/5.0 (X11; Linux i686; rv:12.0) Gecko/X Firefox/12.0 Firefox/12.0";
const char S40378[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-LS696 Build/ZV9.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40379[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-LS696 Build/ZVF.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40380[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; LG-LS696 Build/ZVF.GWK74)";
const char S40381[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-LS696 Build/ZVG.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40382[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40383[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-S5302B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40384[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40385[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5302 Build/GINGERBREAD)";
const char S40386[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5302 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S40387[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5302B Build/GINGERBREAD)";
const char S40388[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 2_2 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5G77a Safari/525.20";
const char S40389[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 2_2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5H11a Safari/525.20";
const char S40390[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 2_2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5H11 Safari/525.20";
const char S40391[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 2_2 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5G77 Safari/525.20";
const char S40392[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 221 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML like Gecko) Version/3.1.1 Mobile/5H11 Safari/525.20";
const char S40393[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 221 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML like Gecko) Version/3.1.1 Mobile/5H11a Safari/525.20";
const char S40394[] = "MSIE9";
const char S40395[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonR800at Build/3.0.1.B.0.131) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40396[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; R800at Build/3.0.1.B.0.270) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40397[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonR800at Build/3.0.1.B.0.270) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40398[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonR800at Build/3.0.1.B.0.148) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40399[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonR800at Build/3.0.1.B.0.273) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40400[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; R800at Build/3.0.1.B.0.285) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40401[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonR800at Build/3.0.1.B.0.285) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40402[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; R800at Build/3.0.1.B.0.285)";
const char S40403[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.0;.NET CLR 1.0.3705; ContextAd Bot 1.0)";
const char S40404[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6210 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S40405[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P6210 Build/HTJ85B)";
const char S40406[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6210 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S40407[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6210 Build/HTJ85B) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S40408[] = "BBC News (AndroidApp; 1.3; 4) GT-P6210 (Android 3.2, SDK 13)";
const char S40409[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P6210 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S40410[] = "Logos/0.9.4 (Linux; U; Android 3.2; en_US; GT-P6210 Build/HTJ85B.UELA4)";
const char S40411[] = "Logos/2.1.2 (Linux; U; Android 3.2; en-US; GT-P6210 Build/HTJ85B)";
const char S40412[] = "Logos/2.1.2 (Linux; U; Android 3.2; pt-BR; GT-P6210 Build/HTJ85B.UBLA1)";
const char S40413[] = "Vyrso/2.1.2 (Linux; U; Android 3.2; en-US; GT-P6210 Build/HTJ85B.UEKL1)";
const char S40414[] = "Logos/2.0.11 (Linux; U; Android 3.2; es-US; GT-P6210 Build/HTJ85B.UBKL2)";
const char S40415[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6210 Build/HTJ85B; 1024*552) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S40416[] = "Logos/2.2.3 (Linux; U; Android 3.2; en-US; GT-P6210 Build/HTJ85B.UELA3)";
const char S40417[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6210 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MantanoReader";
const char S40418[] = "Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 3.2; SAMSUNG GT-P6210 Build/HONEYCOMB_MR2);";
const char S40419[] = "com.google.android.apps.plus/X (Linux; U; Android 3.2; en_US; GT-P6210; Build/HTJ85B) (gzip)";
const char S40420[] = "Groupon/2.1.2486 (Android 3.2; Samsung GT-P6210 / Samsung GT-P6210; )[preload=true;locale=en_US;clientidbase=android-samsung]";
const char S40421[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6210 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 ZyngaBundleIdentifier/com.zynga.zjmontopia ZyngaBundleVersion/1.8.1";
const char S40422[] = "BBC iPlayer (AndroidApp; 1.3; 13014) GT-P6210 (Android 3.2, SDK 13)";
const char S40423[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6210) BBC News/1.3";
const char S40424[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6210 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.12;FBDM/{density=1.0,width=600,height=976};FBLC/en_US;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P6210;FBSV/3.2;]";
const char S40425[] = "BBC News (AndroidApp; 2.1; 30) GT-P6210 (Android 3.2, SDK 13)";
const char S40426[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6210 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 ZyngaBundleIdentifier/com.zynga.zjayakashi ZyngaBundleVersion/2.0.0";
const char S40427[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S40428[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC_PN071 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S40429[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTCONE Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S40430[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One dual sim Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S40431[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One 801e Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S40432[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHV-E120S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40433[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHV-E120L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40434[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHV-E120L Build/GINGERBREAD)";
const char S40435[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHV-E120S Build/GINGERBREAD)";
const char S40436[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHV-E120K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40437[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SHV-E120L Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S40438[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHV-E120K Build/GINGERBREAD)";
const char S40439[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SHV-E120L Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S40440[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SHV-E120S Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S40441[] = "Read%20Later/133 CFNetwork/520.4.3 Darwin/11.4.0 (x86_64) (MacBookAir4%2C2)";
const char S40442[] = "NewsFire/84 CFNetwork/520.4.3 Darwin/11.4.0 (x86_64) (MacBook5%2C1)";
const char S40443[] = "iCoreService (unknown version) CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)";
const char S40444[] = "Google%20Quick%20Search%20Box/2.0.0.1447 CFNetwork/520.4.3 Darwin/11.4.0 (x86_64) (MacBookPro8%2C2)";
const char S40445[] = "Miro/5.0.3 (http://www.getmiro.com/; Darwin 11.4.0 x86_64)";
const char S40446[] = "iCoreService (unknown version) CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookAir4%2C1)";
const char S40447[] = "iCoreService (unknown version) CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (iMac12%2C1)";
const char S40448[] = "RealPlayer%20Downloader/1725 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)";
const char S40449[] = "WebProcess/7536.26.17 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)";
const char S40450[] = "WebProcess/7536.26.17 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (iMac12%2C1)";
const char S40451[] = "WebProcess/7536.26.17 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookAir4%2C2)";
const char S40452[] = "ThumbnailService/39600.038 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)";
const char S40453[] = "ThumbnailService/39600.038 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookAir4%2C2)";
const char S40454[] = "ocspd (unknown version) CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)";
const char S40455[] = "XProtectUpdater (unknown version) CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)";
const char S40456[] = "ocspd (unknown version) CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookAir4%2C2)";
const char S40457[] = "iCoreService (unknown version) CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBook5%2C1)";
const char S40458[] = "XProtectUpdater (unknown version) CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookAir4%2C2)";
const char S40459[] = "RealPlayer%20Downloader/1725 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (iMac12%2C1)";
const char S40460[] = "RealPlayer%20Downloader/1725 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookAir4%2C2)";
const char S40461[] = "ThumbnailService/39600.038 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (iMac12%2C1)";
const char S40462[] = "WebProcess/7536.26.17 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBook5%2C1)";
const char S40463[] = "Spector%20Pro/4821 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)";
const char S40464[] = "Spector%20Pro/4821 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (iMac12%2C1)";
const char S40465[] = "Software%20Update (unknown version) CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)";
const char S40466[] = "iCoreService (unknown version) CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (Macmini5%2C1)";
const char S40467[] = "ThumbnailService/39600.038 CFNetwork/520.5.1 Darwin/11.4.2 (i386) (MacPro1%2C1)";
const char S40468[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Safari/535.19 Silk-Accelerated=true";
const char S40469[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; id102)";
const char S40470[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 822)";
const char S40471[] = "Mozilla/5.0 (Series40; NokiaX2-02/11.57; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6";
const char S40472[] = "Mozilla/5.0 (Series40; NokiaX2-02/10.91; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S40473[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; SonyC1505 Build/11.3.A.0.26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40474[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; C1505 Build/11.3.A.0.26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40475[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SonyC1505 Build/11.3.A.0.12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40476[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; SonyC1505 Build/11.3.A.0.47) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40477[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; C1505 Build/11.3.A.0.47) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40478[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; C1505 Build/11.3.A.0.47)";
const char S40479[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; SonyC1505-o Build/11.3.A.0.47) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40480[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; SonyC1505 Build/11.3.A.0.56) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40481[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; SonyC1505 Build/11.3.A.0.59) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40482[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-be; SonyC1505v Build/11.3.A.0.47) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40483[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-be; SonyC1505v Build/11.3.A.0.56) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40484[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; SonyC1505 Build/11.3.A.2.13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40485[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; C1505 Build/11.3.A.0.59) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40486[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; C1505 Build/11.3.A.2.13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40487[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; SonyC1505-o Build/11.3.A.2.13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40488[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; C1505 Build/11.3.A.0.56) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40489[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; SonyC1505 Build/11.3.A.2.23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40490[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; C1505 Build/11.3.A.2.23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40491[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9A405 Safari/7534.48.3";
const char S40492[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9A406 Safari/7534.48.3";
const char S40493[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9A334 Safari/7534.48.3";
const char S40494[] = "SAMSUNG-GT-S5230G/S523GBKJG1 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S40495[] = "SAMSUNG-GT-S5230/S5230XPKB1 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S40496[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Desire S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40497[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_DesireS_S510e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40498[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Desire S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40499[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Desire S Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S40500[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTCS510e/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40501[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; DesireS_S510e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40502[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Desire S Build/IMM76D)";
const char S40503[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_DesireS_S510e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40504[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Desire S Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S40505[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Desire S Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S40506[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Desire S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40507[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; HTC_DesireSV Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40508[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Desire_S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40509[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; HTC Desire SV Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40510[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Desire S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40511[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Desire SV Build/IMM76D)";
const char S40512[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Desire_S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40513[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Desire S Build/IML74K)";
const char S40514[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; HTC Desire S Build/miui.es) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40515[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-; HTC_DesireSV Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40516[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; HTC_DesireS_S510e Build/qsense) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40517[] = "Mozilla/4.8 [en] (Windows NT 6.0; U)";
const char S40518[] = "Sean's Agent/1.0 (compatible; SA 1.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.04506; .NET CLR 1.1.4322;";
const char S40519[] = "Mozilla/4.0 (compatible; ArgClrInt/1.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S40520[] = "QuickTime/7.6.9 (qtver=7.6.9;os=Windows NT 6.0Service Pack 1)";
const char S40521[] = "QuickTime/7.6.9 (qtver=7.6.9;os=Windows NT 6.0Service Pack 2)";
const char S40522[] = "Mozilla/4.0 (compatible; <a href=http://www.reget.com>ReGet Deluxe 5.1</a>; Windows NT 6.0)";
const char S40523[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko)";
const char S40524[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; fr; rv:1.9.0.1)";
const char S40525[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.57.2 (KHTML, like Gecko)";
const char S40526[] = "Mozilla/4.0 (compatible; SuperCleaner 2.96; Windows NT 6.0)";
const char S40527[] = "Mozilla/5.0 (Windows NT 6.0; rv:12.0) Gecko/ /12.0";
const char S40528[] = "Mozilla/4.0 (Windows Vista 6.0) Java/1.7.0_09";
const char S40529[] = "QuickTime/7.7.3 (qtver=7.7.3;os=Windows NT 6.0Service Pack 2)";
const char S40530[] = "Mozilla/4.0 (Windows Vista 6.0) Java/1.6.0_37";
const char S40531[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.11) Gecko/X";
const char S40532[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.6)";
const char S40533[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.11)";
const char S40534[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.4; en; rv:1.9.2.28) Gecko/X Camino/2.1.2 (like Firefox/3.6.28)";
const char S40535[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.4; en; rv:1.9.2.28) Gecko/X Camino/2.1.2 (like Firefox/13.0)";
const char S40536[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7310 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S40537[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7310 Build/HONEYCOMB) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S40538[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7310 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S40539[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; GT-P7310 Build/HMJ37)";
const char S40540[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7310 Build/HMJ37) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S40541[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7310 Build/HMJ37; 1280*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S40542[] = "Logos/1.1.3 (Linux; U; Android 3.1; en-US; GT-P7310 Build/HMJ37.UEKHA)";
const char S40543[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7310 Build/HMJ37) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S40544[] = "Mozilla/5.0 (Android 3.1; en-us; GT-P7310 Build/HMJ37) FreeWheelAdManager/5.0.3-r9054-X;com.gotv.crackle.handset/X";
const char S40545[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7310 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.10;FBDM/{density=1.0,width=1280,height=800};FBLC/en_GB;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P7310;FBSV/3.1;]";
const char S40546[] = "Logos/2.2.8.1 (Linux; U; Android 3.1; en-GB; GT-P7310 Build/HMJ37)";
const char S40547[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-; GT-P7310 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S40548[] = "Dalvik 2.06.12230; (Linux; U; Android 3.1; GT-P7310; ON) b0a76adfbda439e1; 1280x800x1.0";
const char S40549[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ARCHOS 80G9 Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S40550[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; ARCHOS 80G9 Build/MR1)";
const char S40551[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ARCHOS 80G9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S40552[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ARCHOS 80G9 Build/IMM76D)";
const char S40553[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC One VX Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40554[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PM36100/1.17.502.5 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40555[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC One VX Build/IMM76I)";
const char S40556[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PM36100/1.17.502.4 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40557[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PM36100/1.17.502.9 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40558[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR910L 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40559[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR910L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40560[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; ADR910L Build/IML74K)";
const char S40561[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR910L 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40562[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR910L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40563[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ADR910L Build/IMM76D)";
const char S40564[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S7230E/S723EDTKB1; U; Bada/1.0; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 NexPlayer/3.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B";
const char S40565[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9B206 Safari/7534.48.3";
const char S40566[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9B176 Safari/7534.48.3";
const char S40567[] = "Mozilla/5.0 (iPod; CPU iPhone OS 511 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/21.0.1180.82 Mobile/9B206 Safari/7534.48.3";
const char S40568[] = "Mozilla/5.0 (iPod; CPU iPhone OS 51 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/21.0.1180.82 Mobile/9B176 Safari/7534.48.3";
const char S40569[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/11.04 Chromium/18.0.1025.151 Chrome/18.0.1025.151 Safari/535.19";
const char S40570[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/11.04 Chromium/18.0.1025.151 Chrome/18.0.1025.151 Safari/535.19";
const char S40571[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/11.04 Chromium/18.0.1025.151 Chrome/18.0.1025.151 Safari/535.19,gzip(gfe) (via translate.google.com)";
const char S40572[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/11.04 Chromium/18.0.1025.151 Chrome/18.0.1025.151 Safari/535.19,gzip(gfe)";
const char S40573[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-VS700 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40574[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-VS700 Build/GRJ22)";
const char S40575[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;LG-VS700 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40576[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-VS700 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S40577[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; LG-VS700 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40578[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; LG-VS700 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40579[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-VS700Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40580[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-VS700 Build/GRJ22) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S40581[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; LG-VS700 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40582[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; X10a Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S40583[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-mx; X10a Build/2.0.A.0.504) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S40584[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SonyEricssonX10a Build/2.1.A.0.492) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S40585[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; X10a Build/2.1.1.A.0.16) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S40586[] = "Android (Linux; U; Android 2.1-update1; fa-ir; X10a) AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S40587[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-co; X10a Build/2.0.2.A.0.24) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S40588[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; X10a Build/2.1.A.0.435) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S40589[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; X10a Build/2.1.A.0.492) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S40590[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; X10a Build/2.0.A.0.504)";
const char S40591[] = "PVPlayer X10a CORE/7.507.1.1 OpenCORE/2.05 (Linux;Android2.1-update1)";
const char S40592[] = "PantechP7040/JFUS03152010F; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.16479; U; en-US) Opera 9.50";
const char S40593[] = "PantechP7040/JFUS03152010F; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.16479; U; es) Opera 9.50";
const char S40594[] = "PantechP7040/JLUS05242010; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.16479; U; en-US) Opera 9.50";
const char S40595[] = "PantechP7040/JLUS04042011; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.16479; U; en-US) Opera 9.50";
const char S40596[] = "PantechP7040/JLUSX; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.16479; U; fr) Opera 9.50";
const char S40597[] = "PantechP7040/JFUSXF; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.16479; U; fr) Opera 9.50";
const char S40598[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.8.3) Presto/2.12.388 Version/12.15";
const char S40599[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.1 (KHTML, like Gecko) CEF/0.0.0.0 Safari/534.1";
const char S40600[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) VIDEODOWNLOAD Safari/533.3";
const char S40601[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.3 (KHTML, like Gecko) Lunascape/6.8.0.25815 Safari/535.3";
const char S40602[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.3 (KHTML, like Gecko) Lunascape/6.8.0.25815 Safari/535.3";
const char S40603[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) iLivid/1.0 Safari/534.34";
const char S40604[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.34 (KHTML, like Gecko) iLivid/1.0 Safari/534.34";
const char S40605[] = "Mozilla/5.0 (N; Windows NT 6.1; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) Raptr Safari/534.34";
const char S40606[] = "Mozilla/5.0 (N; Windows NT 6.1) AppleWebKit/534.34 (KHTML, like Gecko) Raptr Safari/534.34";
const char S40607[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.11 (KHTML, like Gecko) SuperBird/23.0 Safari/537.11";
const char S40608[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) SuperBird/23.0 Safari/537.11";
const char S40609[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Superbird/24.0.1312.57 Safari/537.17";
const char S40610[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.17 (KHTML, like Gecko) Superbird/24.0.1312.57 Safari/537.17";
const char S40611[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.14 (KHTML, like Gecko) JavaFX/2.2 Safari/535.14";
const char S40612[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML, like Gecko) Superbird/25.0.1364.97 Safari/537.22";
const char S40613[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, Like Gecko) 20.0.1132.47 Safari/536.11";
const char S40614[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Superbird/27.0.1450.0 Safari/537.36";
const char S40615[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.11 (KHTML, Like Gecko) 20.0.1132.47 Safari/536.11";
const char S40616[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.15 (KHTML, like Gecko) Steam Safari/535.15";
const char S40617[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-GB) AppleWebKit/534.3 (KHTML, like Gecko) MBMANDR/2.2 Safari/534.3";
const char S40618[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-GB) AppleWebKit/537.11 (KHTML, like Gecko) MBMANDR/2.2 Safari/537.11";
const char S40619[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Superbird/27.0.1454.94 Safari/537.22";
const char S40620[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/ Safari/";
const char S40621[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Superbird/28.0.1500.71 Safari/537.36";
const char S40622[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Steam Safari/535.19";
const char S40623[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.21 (KHTML, like Gecko) Banana Safari/537.21";
const char S40624[] = "Passbook/1.0 CFNetwork/646.4 Darwin/14.0.0";
const char S40625[] = "SiriViewService/1.0 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40626[] = "Web/1.0 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40627[] = "AtomicBrowser/7.0.1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40628[] = "Coast/1.0.2.62956 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40629[] = "Instapaper/5.0 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40630[] = "Wallpapers/1.6 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40631[] = "Newsify/115 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40632[] = "Google/2.5.1.13455 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40633[] = "ImageMobile/2.0 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40634[] = "AlienBlueHD/2.3.0 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40635[] = "Guidebook/2.2.6 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40636[] = "itunesstored (unknown version) CFNetwork/672.0.2 Darwin/14.0.0";
const char S40637[] = "locationd/1613 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40638[] = "MusicDownloaderLite/1.1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40639[] = "Edmodo/14 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40640[] = "Apolloplus/2.8.1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40641[] = "IMRemoteURLConnectionAgent/1000 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40642[] = "GroupMe/4.2.5 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40643[] = "Perfect%20Browser/9.1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40644[] = "NortonSafeBrowser/168.2.5.14 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40645[] = "itunescloudd (unknown version) CFNetwork/672.0.2 Darwin/14.0.0";
const char S40646[] = "Keeper/7.0.4 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40647[] = "AtomicLite/7.0.1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40648[] = "assetsd (unknown version) CFNetwork/672.0.2 Darwin/14.0.0";
const char S40649[] = "Hot%20Wallpapers/10.02 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40650[] = "GeoServices/702.14 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40651[] = "Dashlane/45475 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40652[] = "browser/40887 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40653[] = "ImagePro/2.6 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40654[] = "AlienBlue/2.8.2 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40655[] = "ImageSearcherFreeS/1.1.1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40656[] = "NaverSearch/4.7.6 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40657[] = "2%20Million%20WPs/10.09 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40658[] = "syncdefaultsd (unknown version) CFNetwork/672.0.2 Darwin/14.0.0";
const char S40659[] = "dataaccessd (unknown version) CFNetwork/672.0.2 Darwin/14.0.0";
const char S40660[] = "SpringBoard/50 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40661[] = "securityd (unknown version) CFNetwork/672.0.2 Darwin/14.0.0";
const char S40662[] = "AssistantServices/1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40663[] = "Weather/1.0 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40664[] = "Lync%X/2.2 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40665[] = "IMTransferAgent/1000 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40666[] = "CLIPish%20Pro2/10.11 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40667[] = "Clipish/9.29.1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40668[] = "YandexSearch/1480 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40669[] = "Incredimail/6361 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40670[] = "MusicDownloader/1.1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40671[] = "%E5%A4%9A%E5%B1%8F%E4%BA%92%E5%8A%A8%E6%B5%8F%E8%A7%88/1976 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40672[] = "Qrafter/7.7.1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40673[] = "TubeDownloader/1.3 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40674[] = "Prismatic/1.1.4 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40675[] = "Pinterest/2.6.2 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40676[] = "mstreamd/2 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40677[] = "LinkedIn/6.2 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40678[] = "Pic%20Collage/3.15.7 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40679[] = "Readability/1.3.0 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40680[] = "GoodReaderIPad/3.19.4 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40681[] = "ScoopIt/111 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40682[] = "BrowserFree/1.2.1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40683[] = "ubd/289.1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40684[] = "MusicDownloaderSSA/1.1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40685[] = "ysearch2/3.0.4 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40686[] = "Million%20HD%20WPs/8.80.1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40687[] = "123people/3.4 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40688[] = "SKOUT_SDK4/3.8.0 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40689[] = "VZ%20Messages/1.1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40690[] = "ImageSearcherFree/1.0.6 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40691[] = "BaiduHD/2.3.1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40692[] = "Searchx/2.0 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40693[] = "Pulse/3.2.1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40694[] = "MusicDownloaderSSAFree/1.1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40695[] = "VideoDLPlusPlusFree/1.1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40696[] = "AdSheet/1.0 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40697[] = "iDownloaderLite/1.2 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40698[] = "FreeRSSReader/1.9.5 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40699[] = "FastBrowser/10.3.4 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40700[] = "VideoDLFree/1.1.1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40701[] = "Free%20Video%20Downloader/1.5 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40702[] = "ImageHunter/2.5.1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40703[] = "whisper/2.9.94 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40704[] = "MyPad/3.1.1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S40705[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0)";
const char S40706[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-LS970 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40707[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-LS970 Build/JZO54K)";
const char S40708[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; 7 Mozart; Orange)";
const char S40709[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; 7 Mozart T8698)";
const char S40710[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; 7 Mozart)";
const char S40711[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; T8697)";
const char S40712[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Mozart)";
const char S40713[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; PD67100)";
const char S40714[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Mersad 7 Mozart T8698)";
const char S40715[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Mozart T8698)";
const char S40716[] = "Mozilla/5.0 (Android; Mobile; rv:17.0) Gecko/17.0 Firefox/17.0";
const char S40717[] = "Mozilla/5.0 (Android; Mobile; rv:17.0a1) Gecko/17.0a1 Firefox/17.0a1";
const char S40718[] = "Mozilla/5.0 (Android; Mobile; rv:17.0a2) Gecko/17.0a2 Firefox/17.0a2";
const char S40719[] = "Mozilla/5.0 (Android; Mobile; rv:17.0) Gecko/17.0 OAFirefox/17.0";
const char S40720[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; LG-E405f Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S40721[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; LG-E405 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S40722[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-E405 Build/GRK39F)";
const char S40723[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-E405f Build/GRK39F)";
const char S40724[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P999 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S40725[] = "Mozilla/5.0 (Linux; U;Android 2.2.2; en-us; LG-P999 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S40726[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en; LG-P999 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S40727[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-P999; Build/FRE83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S40728[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P999Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S40729[] = "Mozilla/5.0 (Linux; U; Android2.2.2; en-us; LG-P999 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S40730[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;LG-P999 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S40731[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P999 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S40732[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P999 Build/FRG83G)";
const char S40733[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-P999 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40734[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; SonyEricssonX10i Build/R2BA026) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S40735[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; SonyEricssonX10iv Build/R2BA024) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S40736[] = "Mozilla/5.0 (Windows; Windows NT 6.1; rv:10.0) Gecko/X Firefox/9.0";
const char S40737[] = "Mozilla/5.0 (Windows NT 6.1; rv:9.0.1) Gecko/X Firefox/9.0.1,gzip(gfe)";
const char S40738[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:6.0.1) Gecko/X myibrow/6.0.1 Firefox/9.0";
const char S40739[] = "Mozilla/5.0 (Windows:Windows NT 6.1;rv:10.0) Gecko/X Firefox/9.0";
const char S40740[] = "Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaC6-01/014.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.8.10 3gpp-gba";
const char S40741[] = "Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaC6-01.3/014.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.8.10 3gpp-gba";
const char S40742[] = "Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaC6-01.3/022.014; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.37 Mobile Safari/533.4 3gpp-gba";
const char S40743[] = "Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaC6-01/022.014; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.37 Mobile Safari/533.4 3gpp-gba";
const char S40744[] = "NokiaC6-01";
const char S40745[] = "Yahoo! Slurp China";
const char S40746[] = "Mozilla/5.0 (Unknown; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.6.1 Safari/534.34";
const char S40747[] = "Mozilla/5.0 (Unknown; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.6.0 Safari/534.34";
const char S40748[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; YP-G70 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40749[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; YP-G70 Build/FROYO)";
const char S40750[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-; YP-G70 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40751[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SCH-R970 USCC Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S40752[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SCH-R970C Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S40753[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SCH-R970X Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S40754[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SCH-R970 USCC-R970 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0.X Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S40755[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nb-no) AppleWebKit/531.21.8 (KHTML, like Gecko) Gecko/X Firefox/3.5.5";
const char S40756[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.5; de; rv:1.9.1.7) Gecko/X Firefox/3.5.7";
const char S40757[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S40758[] = "Mozilla/5.0 (Linux; Android 4.1; Galaxy Nexus Build/IMM96K) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/25.0.1207.1 Mobile Safari/8536.25";
const char S40759[] = "Mozilla/5.0 (Linux; Android 4.1.1; Galaxy Nexus Build/JRO03O) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S40760[] = "Mozilla/5.0 (Linux; Android 4.1.1; Galaxy Nexus Build/JRO03O) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S40761[] = "Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S40762[] = "Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S40763[] = "Mozilla/5.0 (Linux; Android 4.1; Galaxy Nexus Build/JRN84D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S40764[] = "Mozilla/5.0 (Linux; Android 4.1.1; Galaxy X Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S40765[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; SHV-E120K Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40766[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SHV-E120L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40767[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SHV-E120S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40768[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E120S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40769[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E120K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40770[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHV-E120S Build/IMM76D)";
const char S40771[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E120L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40772[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SHV-E120L Build/IML74K)";
const char S40773[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; SHV-E120L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?i Safari/534.30";
const char S40774[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHV-E120K Build/IMM76D)";
const char S40775[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; vi-vn; E120L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40776[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHV-E120L Build/IMM76D)";
const char S40777[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; zh-cn; SHV-E120L Build/Mander) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40778[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SHV-E120L Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S40779[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; SHV-E120K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?i Safari/534.30";
const char S40780[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.12.12934.201";
const char S40781[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.7 (KHTML, like Gecko) Chrome/20.0.1099.0 Safari/536.7 QQBrowser/6.12.13631.201";
const char S40782[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0) QQBrowser/6.12.12934.201";
const char S40783[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; QQBrowser/6.12)";
const char S40784[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QQBrowser/6.12)";
const char S40785[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; SV1; QQBrowser/6.12)";
const char S40786[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; QQBrowser/6.12)";
const char S40787[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/6.12)";
const char S40788[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/6.12)";
const char S40789[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; QQBrowser/6.12)";
const char S40790[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)";
const char S40791[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)";
const char S40792[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; QQBrowser/6.12)";
const char S40793[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.12.12934.201";
const char S40794[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; QQBrowser/6.12)";
const char S40795[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)";
const char S40796[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C; QQBrowser/6.12)";
const char S40797[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/6.12)";
const char S40798[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)";
const char S40799[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)";
const char S40800[] = "NokiaC1-01/2.0 (04.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac1-01";
const char S40801[] = "NokiaC1-01/2.0 (04.40) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S40802[] = "NokiaC1-01_CMCC/2.0 (05.40) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S40803[] = "NokiaC1-01.1/2.0 (04.40a) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S40804[] = "NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S40805[] = "Opera/9.62 (Windows NT 6.1; U; en) Presto/2.1.1";
const char S40806[] = "Opera/9.60 (Windows NT 6.1; U; en) Presto/2.1.1";
const char S40807[] = "Opera/9.64 (Windows NT 6.1; U; en) Presto/2.1.1";
const char S40808[] = "Opera/9.63 (Windows NT 6.1; U; en) Presto/2.1.1";
const char S40809[] = "Opera/9.61 (Windows NT 6.1; U; en) Presto/2.1.1";
const char S40810[] = "Mozilla/5.0 (Windows NT 6.1; U; YB/3.5.1; ru; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 9.64";
const char S40811[] = "Opera/9.64 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.1.1";
const char S40812[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; YB/3.5.1; ru) Opera 9.64";
const char S40813[] = "Mozilla/5.0 (Windows NT 6.1; U; en; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 9.63";
const char S40814[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-MS840 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40815[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-MS840 Build/GRK39F)";
const char S40816[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-MS840 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40817[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; LG-MS840 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40818[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S40819[] = "Mozilla/5.0 (Linux; Android 4.1.2; LGMS769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S40820[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P768 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S40821[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S40822[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40823[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-R530M Build/JZO54K)";
const char S40824[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SCH-R530M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40825[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-R530M Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40826[] = "SAMSUNG-GT-S3850/1.0 SHP/VPP/R5 Dolfin/2.0 NexPlayer/3.0 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B";
const char S40827[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P930 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30";
const char S40828[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LG-P930/V18f Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40829[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LG-P930 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40830[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P930 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40831[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P930/V20c Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40832[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-P930/V20e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40833[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P930 Build/IMM76D)";
const char S40834[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-P930/V20f Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40835[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.7 (KHTML, like Gecko) Chrome/20.0.1099.0 Safari/536.7 QQBrowser/6.14.15493.201";
const char S40836[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C) QQBrowser/6.14.14517.201";
const char S40837[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.14.15493.201";
const char S40838[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.14.15493.201";
const char S40839[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C) QQBrowser/6.14.15493.201";
const char S40840[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0) QQBrowser/6.14.15493.201";
const char S40841[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729) QQBrowser/6.14.15493.201";
const char S40842[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.14.15493.201";
const char S40843[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.7 (KHTML, like Gecko) Chrome/20.0.1099.0 Safari/536.7 QQBrowser/6.14.15493.201,gzip(gfe)";
const char S40844[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.14.14517.201";
const char S40845[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0) QQBrowser/6.14.15493.201";
const char S40846[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Alexa Toolbar) QQBrowser/6.14.15493.201";
const char S40847[] = "Mozilla/4.0 (compatible; MSIE 9.0; qdesk 2.5.1270.201; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; BRI/2) QQBrowser/6.14.15493.201";
const char S40848[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/540.0 (KHTML,like Gecko) Chrome/9.1.0.0 Safari/540.0";
const char S40849[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13";
const char S40850[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.14 (KHTML, like Gecko; Google Web Preview) Chrome/9.0.597 Safari/534.14";
const char S40851[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.0 Safari/534.13";
const char S40852[] = "Mozilla/5.0 (X11; U; Windows NT 6; en-US) AppleWebKit/534.12 (KHTML, like Gecko) Chrome/9.0.587.0 Safari/534.12";
const char S40853[] = "Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US) AppleWebKit/534.12 (KHTML, like Gecko) Chrome/9.0.576.0 Safari/534.12";
const char S40854[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:11.0) Gecko/X Firefox/11.0";
const char S40855[] = "Mozilla/5.0 (Windows NT 6.0; rv:11.0) Gecko/X Firefox/11.0 Firefox/11.0";
const char S40856[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:11.0) Gecko/X Firefox/11.0";
const char S40857[] = "Opera/9.80 (Windows NT 6.1; U; en) Presto/2.2.15 Version/10.00";
const char S40858[] = "Opera/9.80 (Windows NT 6.1; U; en) Presto/2.2.15 Version/10.01";
const char S40859[] = "Opera/10.00 (Windows NT 6.1; U; en) Presto/2.2.0";
const char S40860[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.2.15 Version/10.00";
const char S40861[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.2.15 Version/10.00";
const char S40862[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.2.15 Version/10.00";
const char S40863[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.2.15 Version/10.00,gzip(gfe) (via translate.google.com)";
const char S40864[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.2.15 Version/10.01";
const char S40865[] = "Mozilla/5.0 (Windows NT 6.1; U; ru; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 10.00";
const char S40866[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.2.15 Version/10.00";
const char S40867[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.2.15 Version/10.00";
const char S40868[] = "Mozilla/5.0 (Windows NT 6.1; U; ru; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 10.01";
const char S40869[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.2.15 Version/10.01";
const char S40870[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.2.15 Version/10.00";
const char S40871[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.2.15 Version/10.00";
const char S40872[] = "Opera/10.02 (Windows NT 6.1; U; en) Presto/22.10.289 Version/10.02";
const char S40873[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.2.15 Version/10.00";
const char S40874[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.2.15 Version/10.00";
const char S40875[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.2.15 Version/10.00";
const char S40876[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.2.15 Version/10.00";
const char S40877[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SCH-I500 Build/ECLAIR) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S40878[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SCH-I500 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S40879[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SCH-I500 Build/ECLAIR)";
const char S40880[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-; SCH-I500 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S40881[] = "Mozilla/5.0 (Linux; U;Android 2.1-update1; en-us; SCH-I500 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S40882[] = "Mozilla/5.0 (Linux; U; Android2.1-update1; en-us; SCH-I500 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S40883[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SCH-I500Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S40884[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us;SCH-I500 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S40885[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SCH-I500 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 MobileSafari/530.17";
const char S40886[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN; rv:1.8.0.11)  Firefox/1.5.0.11; 360Spider";
const char S40887[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN; rv:1.8.0.11) Firefox/1.5.0.11 360Spider;";
const char S40888[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.1.7) Gecko/20091221 Firefox/3.5.7";
const char S40889[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6.8; U; en; rv:1.9.1.6) Gecko/X Firefox/3.5.6";
const char S40890[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.30 (KHTML, like Gecko) Ubuntu/10.10 Chromium/12.0.742.112 Chrome/12.0.742.112 Safari/534.30";
const char S40891[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.30 (KHTML, like Gecko) Ubuntu/10.10 Chromium/12.0.742.112 Chrome/12.0.742.112 Safari/534.30";
const char S40892[] = "Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/534.30 (KHTML, like Gecko) Ubuntu/10.10 Chromium/12.0.742.91 Chrome/12.0.742.91 Safari/534.30";
const char S40893[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SPH-D600 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40894[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SPH-D600 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40895[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-D600 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40896[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SPH-D600 Build/GINGERBREAD)";
const char S40897[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SPH-D600 Build/GINGERBREAD)";
const char S40898[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SPH-D600 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40899[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8000 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S40900[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8020 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S40901[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8000 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S40902[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8020 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S40903[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8005 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S40904[] = "Mozilla/5.0 (Linux; Android 4.1.2;  GT-N8000 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S40905[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; SonyEricssonWT19a Build/4.0.2.A.0.61) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40906[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; SonyEricssonWT19a Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40907[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-cl; SonyEricssonWT19a Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40908[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; WT19a Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40909[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-cl; WT19a Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40910[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-ar; SonyEricssonWT19a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40911[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-gt; SonyEricssonWT19a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40912[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-ar; WT19a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40913[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; WT19a Build/4.0.1.A.0.283)";
const char S40914[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; SonyEricssonWT19a Build/4.0.2.D.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40915[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-gt; WT19a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40916[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; WT19a Build/4.0.2.D.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40917[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; WT19a Build/4.0.2.A.0.61) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40918[] = "mozilla/5.0 (windows; u; windows nt 5.1; es-ar; rv:1.8.0.7) gecko/X firefox/1.5.0.7";
const char S40919[] = "mozilla/5.0 (windows; u; windows nt 5.1; it; rv:1.8.0.7) gecko/X firefox/1.5.0.7";
const char S40920[] = "Mozilla/5.0 (Windows NT 5.1; U; tr; rv:1.8.0) Gecko/X Firefox/1.5.0";
const char S40921[] = "mozilla/5.0 (windows; u; windows nt 5.1; pt-br; rv:1.8.0.7) gecko/X firefox/1.5.0.7";
const char S40922[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1) Gecko/X Firefox/1.5.0.12";
const char S40923[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.8.0.8) Gecko/X Firefox/1.5.0.8";
const char S40924[] = "mozilla/5.0 (windows; u; windows nt 5.1; ja; rv:1.8.0.7) gecko/X firefox/1.5.0.7";
const char S40925[] = "mozilla/5.0 (windows; u; windows nt 5.1; sl; rv:1.8.0.7) gecko/X firefox/1.5.0.7";
const char S40926[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN; rv:1.8.0.11) Firefox/1.5.0.11;";
const char S40927[] = "Mozilla/5.0+(Windows;+U;+Windows+NT+5.1;+de;+rv:1.8.0.1)+Gecko/X+Firefox/1.5.0.1";
const char S40928[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HUAWEI Y210-0010 Build/HuaweiY210-0010) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40929[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_Y210-2010/Y210-2010V100R001C92B823; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S40930[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HUAWEI Y210-0100 Build/HuaweiY210-0100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40931[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_Y210-2010/Y210-2010V100R001C92B817; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S40932[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HUAWEI Y210-0200 Build/HuaweiY210-0200) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40933[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_Y210-2010/Y210-2010V100R001C92B821; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S40934[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_Y210S/Y210SV100R001C92B823; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S40935[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; HUAWEI Y210-0151 Build/HuaweiY210-0151) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40936[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_Y210-2010/Y210-2010V100R001C92B825; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S40937[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_Y210-2010/Y210-2010V100R001C92B826; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S40938[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_Y210-2010/Y210-2010V100R001C92B824; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S40939[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; HUAWEI Y210-0251 Build/HuaweiY210-0251) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40940[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-ca; NATCOM N8302 Build/HuaweiY210-0200) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40941[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_Y210-2010/Y210-2010V100R001C92B828; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S40942[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; HUAWEI Ascend Y 210 Build/HuaweiY210-0100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40943[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; HUAWEI Y210-0100 Build/HuaweiY210-0100)";
const char S40944[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; HUAWEI Ascend Y 210D Build/HuaweiY210-0200) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40945[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; HUAWEI Y210-2010 Build/HuaweiY210-2010) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40946[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus 7 Build/JWR66V) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.111 Safari/537.36";
const char S40947[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:20.0) Gecko/X Firefox/20.0 SeaMonkey/2.17.1";
const char S40948[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:20.0) Gecko/X SeaMonkey/2.17.1";
const char S40949[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:20.0) Gecko/X Firefox/20.0 SeaMonkey/2.17.1";
const char S40950[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190 Twitter for iPhone";
const char S40951[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 Twitter for iPhone";
const char S40952[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 Twitter for iPhone";
const char S40953[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Twitter for iPhone";
const char S40954[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Twitter for iPhone";
const char S40955[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2 Twitter for iPhone";
const char S40956[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/9Z999 Twitter for iPhone";
const char S40957[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-LW690 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40958[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-LW690 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40959[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; LG-LW690 Build/FRG83)";
const char S40960[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-LW690 Build/FRG83G)";
const char S40961[] = "Mozilla/5.0 (Series40; Nokia201/11.56; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6";
const char S40962[] = "Mozilla/5.0 (Series40; Nokia201/11.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S40963[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; tr-tr; GM Ultimate Slim Build/HuaweiU8510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40964[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; S41HW Build/HuaweiU8510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40965[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; U8510 Build/HuaweiU8510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40966[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; U8510 Build/HuaweiU8510)";
const char S40967[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; U8510 Build/HuaweiU8510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40968[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; U8510 Build/HuaweiU8510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40969[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; U8510 Build/HuaweiU8510)";
const char S40970[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; U8510 Build/HuaweiU8510)";
const char S40971[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HuaweiU8510 Build/HuaweiU8510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40972[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-by; MTC Bravo Build/HuaweiU8510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40973[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U8510 Build/HuaweiU8510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S40974[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SonyEricssonLT15a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40975[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; SonyEricssonLT15av Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40976[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LT15a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40977[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Arc Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S40978[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT15a Build/4.1.B.0.587)";
const char S40979[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-; Arc Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S40980[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Arc Build/IMM76L)";
const char S40981[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; LT15a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40982[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-M830 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40983[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SPH-M830 Build/IMM76I)";
const char S40984[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SPH-M830 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40985[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ARCHOS 101G9 Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S40986[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; ARCHOS 101G9 Build/MR1)";
const char S40987[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ARCHOS 101G9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S40988[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ARCHOS 101G9 Build/IMM76D)";
const char S40989[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40990[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40991[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100G Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40992[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40993[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9100 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40994[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9100G Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40995[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100G Build/SlimTW5-jelly-Bean-v3.0#_; CyanogenMod-SlimTW5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40996[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100 Build/1.5; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S40997[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9100 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S40998[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.904.0 Safari/535.7";
const char S40999[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.904.0 Safari/535.7";
const char S41000[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.910.20417 KChrome/3.2.5.20417  Safari/535.7";
const char S41001[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S41002[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S41003[] = "Mozilla/5.0 (Android; Tablet; rv:23.0) Gecko/23.0 Firefox/23.0";
const char S41004[] = "Mozilla/5.0 (Android; Tablet; rv:23.0a2) Gecko/23.0a2 Firefox/23.0a2";
const char S41005[] = "Mozilla/5.0 (Android; Tablet; rv:23.0) Gecko/23.0 Firefox/23.0,gzip(gfe)";
const char S41006[] = "Mozilla/5.0 (Android; Tablet; rv:23.0) Gecko/23.0 Firefox/23.0,gzip(gfe) (via docs.google.com/viewer)";
const char S41007[] = "Mozilla/5.0 (Android; Tablet; rv:23.0) Gecko/23.0 Firefox/23.0,gzip(gfe),gzip(gfe)";
const char S41008[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; SAMSUNG-SGH-I957 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41009[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-ca; SGH-I957M Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41010[] = "Logos/0.9.4 (Linux; U; Android 3.2; en_US; SAMSUNG-SGH-I957 Build/MASTER.CKJ7)";
const char S41011[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; SAMSUNG-SGH-I957 Build/MASTER)";
const char S41012[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-ca; SGH-I957R Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41013[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-ca; SGH-I957D Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41014[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; SGH-I957R Build/MASTER)";
const char S41015[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; SGH-I957M Build/MASTER)";
const char S41016[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; SAMSUNG-SGH-I957 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S41017[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HUAWEI-M835 Build/HuaweiM835) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41018[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-; HUAWEI-M835 Build/HuaweiM835) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41019[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HUAWEI-M835 Build/HuaweiM835) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Android MetroPCS androidAppVersion/2.1 AndroidWebkitWrapper";
const char S41020[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; HUAWEI-M835 Build/HuaweiM835)";
const char S41021[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HUAWEI-M835 Build/HuaweiM835) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S41022[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HUAWEI-M835 Build/HuaweiM835) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Android MetroPCS androidAppVersion/2.5.10 AndroidWebkitWrapper BillingSupported";
const char S41023[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; MZ604 Build/H.6.6-23) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41024[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ604 Build/H.6.6-27) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41025[] = "Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; MZ604 Build/H.6.6-29) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41026[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; MZ604 Build/H.6.6-23)";
const char S41027[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-ca; MZ604 Build/H.6.6-30) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41028[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Safari/535.19 Silk-Accelerated=true";
const char S41029[] = "Mozilla/5.0 (Symbian/3; Series60/5.3 Nokia700/111.020.0308; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.4.1.14 Mobile Safari/533.4 3gpp-gba";
const char S41030[] = "Mozilla/5.0 (Symbian/3; Series60/5.4 Nokia700/112.010.1404; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.2.1.20 Mobile Safari/535.1 3gpp-gba";
const char S41031[] = "Mozilla/5.0 (Symbian/3; Series60/5.3 Nokia700; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.4.1.14 Mobile Safari/533.4 3gpp-gba";
const char S41032[] = "Mozilla/5.0 (Symbian/3; Series60/5.5 Nokia700/113.010.1506; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.2.21 Mobile Safari/535.1 3gpp-gba";
const char S41033[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S41034[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S41035[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT912 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S41036[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-E970/E97010l Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41037[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-E970/E97010o Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41038[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-E970 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41039[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-E970/E97011c Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41040[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-E970 Build/IMM76L)";
const char S41041[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S41042[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 4 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S41043[] = "Mozilla/5.0 (Linux; Android 4.2; Nexus 4 Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S41044[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S41045[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; SGH-T959 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41046[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; SGH-T959D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41047[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SGH-T959 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41048[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T959 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41049[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; SGH-T959P Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41050[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-T959 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41051[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SGH-T959 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41052[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; SGH-T959W Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41053[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; T959 Build/ReVaMpEd) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41054[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T959 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41055[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T959 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41056[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SGH-T959P Build/GINGERBREAD)";
const char S41057[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; SGH-T959 Build/GRJ22)";
const char S41058[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SGH-T959D Build/GINGERBREAD)";
const char S41059[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-T959 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41060[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-T959W Build/GINGERBREAD)";
const char S41061[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; T959 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41062[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-M830 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41063[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SPH-M830 Build/JZO54K)";
const char S41064[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SPH-M830 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41065[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.5) Gecko/X Ubuntu/9.04 (intrepid) Firefox/3.0.5";
const char S41066[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5250/1.0; U; bada/1.0; es) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 OPN-B";
const char S41067[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5250/S5250XEKJ3; U; Bada/1.0; ru-ru) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 NexPlayer/3.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B";
const char S41068[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; FDM; MSIECrawler; Media Center PC 5.0)";
const char S41069[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; MSIECrawler)";
const char S41070[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; CrawlDaddy v0.1.0 abot v1.1.0.0 http://code.google.com/p/abot)";
const char S41071[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; abot v1.1.1.0 http://code.google.com/p/abot)";
const char S41072[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.2.25) Gecko/X Firefox/3.6.25";
const char S41073[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S41074[] = "Mozilla/5.0 (Linux; Android 4.1.1; EndeavorU Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S41075[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S41076[] = "Mozilla/5.0 (Linux; Android 4.1.2; One X Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S41077[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One X Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S41078[] = "Mozilla/5.0 (Linux; Android 4.1.2; One X Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S41079[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SCH-R910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41080[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SCH-R910 Build/FROYO)";
const char S41081[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SCH-R910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41082[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; SCH-R910 Build/FROYO)";
const char S41083[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SCH-R910 Build/FROYO)";
const char S41084[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:20.0) Gecko/X Firefox/20.0 AlexaToolbar/alxf-2.18";
const char S41085[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64;) Gecko/X Firefox/20.0";
const char S41086[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.1-x64";
const char S41087[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.1";
const char S41088[] = "Instagram 3.4.2 (iPhone4,1; iPhone OS 6.1; en_US; en) AppleWebKit/420+";
const char S41089[] = "Instagram 3.4.2 (iPhone4,1; iPhone OS 6.1.1; en_US; en) AppleWebKit/420+";
const char S41090[] = "Instagram 3.4.2 (iPhone4,1; iPhone OS 6.1.2; en_US; en) AppleWebKit/420+";
const char S41091[] = "Instagram 3.4.2 (iPhone4,1; iPhone OS 6.1.3; en_US; en) AppleWebKit/420+";
const char S41092[] = "Instagram 3.4.2 (iPhone5,2; iPhone OS 6.1.4; en_US; en) AppleWebKit/420+";
const char S41093[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari Version/5.1";
const char S41094[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Version/5.1 Safari/7534.48.3 NAVER(inapp; search; 103; 3.8.2)";
const char S41095[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10A403 Safari/7536.26";
const char S41096[] = "Mozilla/5.0 (iPad; CPU OS 6 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Version/5.1 Safari/7534.48.3 Kikin/1.9.5";
const char S41097[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10A523 Safari/7536.26";
const char S41098[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari Version/5.1";
const char S41099[] = "Mozilla/5.0 (iPad; U; CPU OS 6 like Mac OS X; zh-cn Model:iPad3,4) UC AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7543.48.3";
const char S41100[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Version/5.1 Safari/7534.48.3 NAVER(inapp; search; 103; 4.2.6)";
const char S41101[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550 Version/5.1 Safari/7534.48.3 NAVER(inapp; search; 103; 4.2.6)";
const char S41102[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8426 Safari Version/5.1";
const char S41103[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S41104[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S41105[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S41106[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8500 Safari Version/5.1";
const char S41107[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550 Safari Version/5.1";
const char S41108[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3";
const char S41109[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/10A403 Safari/7534.48.3";
const char S41110[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S41111[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S41112[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A407 Safari Version/5.1";
const char S41113[] = "Mozilla/5.0 (Windows NT x.y; Win64; x64; rv:10.0.1) Gecko/X Firefox/10.0.1";
const char S41114[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SHV-E110S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41115[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I727R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41116[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E110S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41117[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I727R Build/IMM76D)";
const char S41118[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; GT-I9210 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41119[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHV-E110S Build/IMM76D)";
const char S41120[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SGH-I727R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41121[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-hk; GT-I9210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41122[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SHV-E110S-parrot Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41123[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SHV-E110S Build/IML74K)";
const char S41124[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9210 Build/IML74K)";
const char S41125[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_3; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S41126[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S41127[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S41128[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/4.0.3 Safari/531.9";
const char S41129[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S41130[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S41131[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_8 en-us) Version/4.0";
const char S41132[] = "Mozilla/5.0 (compatible; MSIE 2.0; Linux i686; Trident/3.0)";
const char S41133[] = "Mozilla/5.0 (compatible; MSIE 1.0; Linux x86_64; Trident/5.0)";
const char S41134[] = "Mozilla/5.0 (compatible; MSIE 1.0; Linux i686; Win64; x64; Trident/2.0)";
const char S41135[] = "Mozilla/5.0 (compatible; MSIE 2.0; Linux i686; Tablet PC; Trident/2.0)";
const char S41136[] = "Mozilla/5.0 (compatible; MSIE 1.0; Linux i686; SV1; Trident/1.0)";
const char S41137[] = "Mozilla/5.0 (compatible; MSIE 1.0; Linux i686; WOW64; Trident/5.0)";
const char S41138[] = "Mozilla/5.0 (compatible; MSIE 1.0; Linux x86_64; Tablet PC; Trident/5.0)";
const char S41139[] = "Mozilla/5.0 (compatible; MSIE 2.0; Linux x86_64; Win64; IA64; Trident/1.0)";
const char S41140[] = "Mozilla/5.0 (compatible; MSIE 2.0; Linux x86_64; SV1; Trident/5.0)";
const char S41141[] = "Mozilla/5.0 (compatible; MSIE 2.0; Linux x86_64; WOW64; Trident/3.0)";
const char S41142[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VS950 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41143[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; VS950 4G Build/IMM76D)";
const char S41144[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/9";
const char S41145[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13";
const char S41146[] = "Mozilla/5.0 (Windows; U;                             Windows NT 6.0; en-US)                             AppleWebKit/534.13 (KHTML, like Gecko)                             Chrome/9.0.597.98 Safari/534.13";
const char S41147[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 920)";
const char S41148[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Juggernaut; Alpha)";
const char S41149[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 920; ANZ821)";
const char S41150[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 920; Vodafone)";
const char S41151[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 920; Orange)";
const char S41152[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Nokia; Nokia 920)";
const char S41153[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Nokia 920T)";
const char S41154[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 920; 1326-2)";
const char S41155[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; HTC Flyer P512 Build/HTK75C)";
const char S41156[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us;HTC_Flyer_P512 Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41157[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; HTC_Flyer_P512_NA Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41158[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; HTC Flyer P510e Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41159[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; HTC_Flyer Build/HTK75C) AppleWebkit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41160[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-; HTC Flyer P510e Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41161[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; de-de; HTC Flyer Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41162[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; de-de;HTC_Flyer Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41163[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; HTC_Flyer_P512_NA Build/HTK75C)";
const char S41164[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-gb; HTC Flyer P510e Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41165[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-gb; HTC Flyer P512 Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41166[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-gb;HTC_Flyer_P510e Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41167[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; HTC Flyer P510e Build/HTK75C)";
const char S41168[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-ca;HTC_Flyer_P512_NA Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41169[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Sprint APA9292KT Build/OMJ_EVO_2.2_Froyo_v3.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41170[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Cricket APA9292KT Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41171[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Sprint APA9292KT Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41172[] = "Mozilla/5.0 (Linux; U; Android 2.2; -us; Sprint APA9292KT Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41173[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Sprint APA9292KT Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41174[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; PC36100 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41175[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; PC36100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41176[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Sprint APA9292KT Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41177[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; PC36100 Build/FRF91)";
const char S41178[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; PC36100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41179[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; PC36100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41180[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; PC36100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41181[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; PC36100 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S41182[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; PC36100 Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S41183[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.17 (KHTML, like Gecko) Ubuntu Chromium/24.0.1312.56 Chrome/24.0.1312.56 Safari/537.17";
const char S41184[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.17 (KHTML, like Gecko) Ubuntu Chromium/24.0.1312.56 Chrome/24.0.1312.56 Safari/537.17";
const char S41185[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.17 (KHTML, like Gecko) Ubuntu Chromium/24.0.1312.56 Chrome/24.0.1312.56 Safari/537.17,gzip(gfe)";
const char S41186[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.17 (KHTML, like Gecko) Ubuntu Chromium/24.0.1312.56 Chrome/24.0.1312.56 Safari/537.17,gzip(gfe)";
const char S41187[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3113 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41188[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3113 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41189[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3113 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41190[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P3113 Build/JDQ39)";
const char S41191[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/7";
const char S41192[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S41193[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.28 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.28";
const char S41194[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.28 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.28";
const char S41195[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; K1 Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41196[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; K1 Build/HTK75C)";
const char S41197[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Milestone X Build/4.5.1_57_MX2-27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41198[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Milestone X Build/4.5.1_57_MX2-20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41199[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Milestone X Build/4.5.1_57_MX2-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41200[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Milestone X Build/4.5.1_57_MX2-34)";
const char S41201[] = "Logos/0.9.4 (Linux; U; Android 2.3.5; en_US; Milestone X Build/4.5.1_57_MX2-32)";
const char S41202[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Milestone X Build/4.5.1_57_MX2-34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41203[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S41204[] = "Mozilla/5.0 (Linux; Android 4.0.3; SPH-D710 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S41205[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710BST Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S41206[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710VMUB Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S41207[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A4400f Safari/8536.25";
const char S41208[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11A4414e Safari/8536.25";
const char S41209[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11A4414c Safari/8536.25";
const char S41210[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/11A4435d Safari/8536.25";
const char S41211[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/11A465 Safari/8536.25 YaBrowser/2.1.1364.172";
const char S41212[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S41213[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A406 Safari/8536.25";
const char S41214[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/11A501 Safari/8536.25 YaBrowser/2.1.1364.172";
const char S41215[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S41216[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:20.2) Gecko/X Firefox/20.2 PaleMoon/20.2.1-x64";
const char S41217[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:20.2) Gecko/X Firefox/20.2 PaleMoon/20.2.1";
const char S41218[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P5100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S41219[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P5100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S41220[] = "Mozilla/5.0 (Linux; Android 4.0.3;  GT-P5100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S41221[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T779 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S41222[] = "Mozilla/5.0 (Linux; Android 4.0.4;  GT-P5100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S41223[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; NABI2-NV7A Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41224[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NABI2-NV7A Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41225[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; NABI2-NV7A-UK Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41226[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ie; NABI2-NV7A-IE Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41227[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NABI2-NV7A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41228[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; NABI2-NV7A Build/IMM76L)";
const char S41229[] = "Mozilla/5.0 (Android; Mobile; rv:16.0) Gecko/16.0 Firefox/16.0a1";
const char S41230[] = "Mozilla/5.0 (Android; Mobile; rv:16.0) Gecko/16.0 Firefox/16.0";
const char S41231[] = "Mozilla/5.0 (Android; Mobile; rv:16.0a2) Gecko/16.0a2 Firefox/16.0a2";
const char S41232[] = "Mozilla/5.0 (Android; Mobile; rv:16.0.1) Gecko/16.0.1 Firefox/16.0.1";
const char S41233[] = "Mozilla/5.0 (Android; Mobile; rv:16.0.2) Gecko/16.0.2 Firefox/16.0.2";
const char S41234[] = "Mozilla/5.0 (Android; Mobile; rv:16.0) Gecko/16.0 OAFirefox/16.0";
const char S41235[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; sdk Build/JRN83C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41236[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; sdk Build/JRO03E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41237[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; google_sdk Build/JRO03E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41238[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; rk30sdk Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41239[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; sdk Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41240[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; rk30sdk Build/JRO03H)";
const char S41241[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; sdk Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41242[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 BaDoinkDownloader_Android_2.0";
const char S41243[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; google_sdk Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41244[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 BaDoinkDownloader_Android_2.0";
const char S41245[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; rk2928sdk Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41246[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; Full AOSP on Rk29sdk Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41247[] = "Mozilla/5.0 (Linux; U; Android 4.1.0; en-us; Full AOSP on Rk29sdk Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41248[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; rk2928sdk Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41249[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; rk2926sdk Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41250[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; sdk Build/JRO03E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41251[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; rk2928sdk Build/JRO03H)";
const char S41252[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; rk292Xsdk Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41253[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; rk30sdk Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41254[] = "Mozilla/5.0 (compatible; MSIE 10.0; AOL 9.7; AOLBuild 4343.55; Windows NT 5.1; Trident/6.0)";
const char S41255[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 5.2; Trident/2.0)";
const char S41256[] = "Mozilla/5.0 (compatible; MSIE 10.0; AOL 9.7; AOLBuild 4343.1028; Windows NT 5.1; WOW64; Trident/6.0)";
const char S41257[] = "Mozilla/5.0 (X11; Linux x86_64; rv:19.0) Gecko/X Firefox/19.0 Iceweasel/19.0";
const char S41258[] = "Mozilla/5.0 (X11; Linux i686 on x86_64; rv:19.0) Gecko/X Firefox/19.0";
const char S41259[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Safari/537.36";
const char S41260[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S41261[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I535 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S41262[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T999 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S41263[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9300 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S41264[] = "MobileZone v0.1 (Android)";
const char S41265[] = "AndroidDownloadManager";
const char S41266[] = "Mozilla/5.0 (Linux; U; Android";
const char S41267[] = "Android-EAS/0.1";
const char S41268[] = "Android/0.3";
const char S41269[] = "SAMSUNG-Android";
const char S41270[] = "Mozilla/5.0 (Android; U; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.1";
const char S41271[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Safari/535.19 Silk-Accelerated=true";
const char S41272[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; YP-GI1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41273[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; YP-GI1 Build/GINGERBREAD)";
const char S41274[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; YP-GI1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41275[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F200S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41276[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F200K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41277[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LG-E973 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41278[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LG-E973/E97320h Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41279[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F200L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41280[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E973 Build/JZO54K)";
const char S41281[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F200S Build/JZO54K)";
const char S41282[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F200K Build/JZO54K)";
const char S41283[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F200L Build/JZO54K)";
const char S41284[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F200K/20d Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41285[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LG-E973-parrot Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41286[] = "Mozilla/5.0 (Series40; NokiaC2-02/06.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S41287[] = "Mozilla/5.0 (Series40; NokiaC2-02.1/07.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S41288[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8530/S8530DDLC2; U; Bada/2.0; en-us) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile WVGA SMM-MMS/1.2.0 OPN-B";
const char S41289[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 10 Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41290[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 10 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41291[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; Nexus 10 Build/JOP40D)";
const char S41292[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 10 Build/JVP15P) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41293[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 10 Build/JOP40F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41294[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 10 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41295[] = "Dalvik/1.6.0 (Linux; U; Android 4.2; Nexus 10 Build/JVP15P)";
const char S41296[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; Nexus 10 Build/JDQ39)";
const char S41297[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 10 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41298[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 10 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41299[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 10 Build/JVP15I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41300[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; ALCATEL_one_touch_990 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41301[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; ALCATEL one touch 990 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41302[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ALCATEL_one_touch_990A Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41303[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Alcatel one touch 990S Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41304[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; Alcatel one touch 990 Orange Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41305[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ALCATEL_one_touch_990S Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41306[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; ALCATEL_one_touch_990 Build/FRG83G)";
const char S41307[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; ALCATEL one touch 990 Build/FRG83G)";
const char S41308[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; Alcatel one touch 990S Build/FRG83G)";
const char S41309[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; ALCATEL_one_touch_990A Build/FRG83G)";
const char S41310[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 Safari/7534.48.3 TeamStream (iPhone4,1 306 2.3.1)";
const char S41311[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Safari/7534.48.3 TeamStream (iPhone5,1 306 2.3.1)";
const char S41312[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/7534.48.3 TeamStream (iPhone3,3 306 2.3.1)";
const char S41313[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145 Safari/7534.48.3 TeamStream (iPhone4,1 313 2.4)";
const char S41314[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 TeamStream (iPhone4,1 313 2.4)";
const char S41315[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Safari/7534.48.3 TeamStream (iPhone3,1 313 2.4)";
const char S41316[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 TeamStream (iPhone4,1 336 2.5.1)";
const char S41317[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3 TeamStream (iPhone5,2 379 2.7.2)";
const char S41318[] = "Safari/8536.30.1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro10%2C1)";
const char S41319[] = "Safari/8536.30.1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (iMac12%2C2)";
const char S41320[] = "Safari/8536.30.1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookAir5%2C2)";
const char S41321[] = "Safari/8536.30.1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacPro3%2C1)";
const char S41322[] = "Safari/8536.30.1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBook5%2C1)";
const char S41323[] = "Safari/8536.30.1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (Macmini6%2C2)";
const char S41324[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A510 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41325[] = "AndroidDownloadManager/4.1.2 (Linux; U; Android 4.1.2; A510 Build/JZO54K)";
const char S41326[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; A510 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41327[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.7 (KHTML, like Gecko) Chrome/20.0.1099.0 Safari/536.7 QQBrowser/6.14.15493.201";
const char S41328[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0) QQBrowser/6.14.15493.201";
const char S41329[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) QQBrowser/6.14.15493.201";
const char S41330[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/536.7 (KHTML, like Gecko) Chrome/20.0.1099.0 Safari/536.7 QQBrowser/6.14.15493.201";
const char S41331[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1) QQBrowser/6.14.15493.201";
const char S41332[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.7 (KHTML, like Gecko) Chrome/20.0.1099.0 Safari/536.7 QQBrowser/6.14.15493.201,gzip(gfe)";
const char S41333[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QQDownload 718; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ) QQBrowser/6.14.15493.201";
const char S41334[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0) QQBrowser/6.14.15493.201";
const char S41335[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; SV1; QQDownload 718; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; BRI/2; u9dnfsh) QQBrowser/6.14.14517.201";
const char S41336[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7C106c";
const char S41337[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko)";
const char S41338[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko)";
const char S41339[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko)";
const char S41340[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7D11";
const char S41341[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7C146";
const char S41342[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7C144";
const char S41343[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7C145";
const char S41344[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18";
const char S41345[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko)";
const char S41346[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Safari/528.18";
const char S41347[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7D11 Safari/528.18";
const char S41348[] = "Mozilla/5.0 (whited00r iPod; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18";
const char S41349[] = "Mozilla/5.0 (whited00r iPod; U; CPU iPhone OS 3_1_3 like Mac OS X; es-es) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18";
const char S41350[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NOOK BNTV600 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41351[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BNTV600 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41352[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; BNTV600 Build/IMM76L)";
const char S41353[] = "Nokia302/5.0 (14.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S41354[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 4 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41355[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 4 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41356[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41357[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/mako";
const char S41358[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/4.75; CyanogenMod-4.75) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41359[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LGL75C Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41360[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LGL75C Build/GRJ22)";
const char S41361[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S41362[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S41363[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I200 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41364[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41365[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-I200 Build/IMM76D)";
const char S41366[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E975 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41367[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E975 Build/JZO54K)";
const char S41368[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; LG-E975/V10b Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41369[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; LG-E975/V10c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41370[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; LG-E975/V10d Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41371[] = "LG-E975K-AB/E975K10d Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; LG-E975K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41372[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-E975-Orange Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41373[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E975K Build/JZO54K)";
const char S41374[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-E975/V10f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41375[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-E975/E97510d Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41376[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-E975/V10a Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41377[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-E975/V10g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41378[] = "Mozilla/5.0 (windows; U; Windows NT 5.1; zh-CN; rv:1.7.12) Gecko/20050919 Firefox/1.0.7";
const char S41379[] = "Mozilla/5.0 (windows; U; Windows NT 5.1; zh-CN; rv:1.7.12) Gecko/X Firefox/1.0.7,gzip(gfe) (via translate.google.com)";
const char S41380[] = "Mozilla/6.0 (Windows; U; Windows NT 5.1; en-US; rv:1.7.7) Gecko/X Firefox/1.0.3";
const char S41381[] = "Mozilla/5.0   (Windows; U; Windows NT 5.1; nl-NL; rv:1.7.5) Gecko/X Firefox/1.0";
const char S41382[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-TW; rv:1.7.5) Gecko/X Firefox/1.0";
const char S41383[] = "Mozilla/5.0 (Windows; U; Windows NT5.1; en; rv:1.7.10) Gecko/X Firefox/1.0.5";
const char S41384[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; zh-TW; rv:1.7) Gecko/X Firefox/1.0";
const char S41385[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:17.0) Gecko/X Thunderbird/17.0.8";
const char S41386[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41387[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39)";
const char S41388[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I337 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41389[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I337 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41390[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41391[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGH-I337 Build/JDQ39E)";
const char S41392[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Safari/535.19 Silk-Accelerated=true";
const char S41393[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/523.15 (KHTML, like Gecko) Version/3.0 Safari/523.15";
const char S41394[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en) AppleWebKit/522.13.1 (KHTML, like Gecko) Version/3.0.2 Safari/522.13.1";
const char S41395[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en) AppleWebKit/522.11.3 (KHTML, like Gecko) Version/3.0 Safari/522.11.3";
const char S41396[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/523.12.9 (KHTML, like Gecko) Version/3.0 Safari/523.12.9";
const char S41397[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en) AppleWebKit/522.15.5 (KHTML, like Gecko) Version/3.0.3 Safari/522.15.5";
const char S41398[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; nl) AppleWebKit/522.11.3 (KHTML, like Gecko) Version/3.0 Safari/522.11.3";
const char S41399[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en) AppleWebKit/522.13.1 (KHTML, like Gecko) Version/3.0.2 Safari/522.13.1";
const char S41400[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en) AppleWebKit/522.12.1 (KHTML, like Gecko) Version/3.0.1 Safari/522.12.2";
const char S41401[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Safari/419.3";
const char S41402[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Safari/312.6";
const char S41403[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Safari/417.8";
const char S41404[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Safari/412.5";
const char S41405[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/525.18 (KHTML, like Gecko) Sunrise/1.7.4 like Safari/4525.22";
const char S41406[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; nl-nl) AppleWebKit/533.19.4 (KHTML, like Gecko)";
const char S41407[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Safari/412";
const char S41408[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Safari/416.12";
const char S41409[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Safari/412.2";
const char S41410[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Safari/417.9.2";
const char S41411[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/(null) Safari/533.19.4";
const char S41412[] = "Mozilla/4.0 (Mac OS X 10.4.11) Java/1.5.0_19";
const char S41413[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) RealPlayer/12.0.0.1725";
const char S41414[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) RealPlayer/10.1.0.412";
const char S41415[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) RealPlayer/10.1.0.503";
const char S41416[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) RealPlayer/10.0.0.331";
const char S41417[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9000 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41418[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9000 Build/JRO03H; CyanogenMod-TAJB//MKalter\\\\V7.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41419[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9000 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41420[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9000 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41421[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; GT-I9000B Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41422[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F160K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41423[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F160L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41424[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F160S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41425[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F160LV Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41426[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F160L Build/JZO54K)";
const char S41427[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F160K Build/JZO54K)";
const char S41428[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F160S Build/JZO54K)";
const char S41429[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F160K/F160K20f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41430[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; LG-F160L/LV Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41431[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; LG-F160S/K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41432[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F160K/F160K20g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41433[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F160LV Build/JZO54K)";
const char S41434[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; LG-F160 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41435[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F160K/F160K20h Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41436[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9B176 Safari/7534.48.3";
const char S41437[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9B206 Safari/7534.48.3";
const char S41438[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9B208 Safari/7534.48.3";
const char S41439[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9B179 Safari/7534.48.3";
const char S41440[] = "Opera/9.70 (Linux ppc64 ; U; en) Presto/2.2.1";
const char S41441[] = "Mozilla/4.0 (compatible; MSIE 6.0; Linux i686 ; en) Opera 9.70";
const char S41442[] = "Opera/9.70 (Linux i686 ; U; zh-cn) Presto/2.2.0";
const char S41443[] = "Opera/9.70 (Linux armv6l ; U; CE-HTML/1.0 (<profilelist><ui_profile name=\"PHILIPS_OLS_2010\"/></profilelist>); en) Presto/2.2.1";
const char S41444[] = "Opera/9.70 (Linux armv7l ; U; HomeManagerV1.5/TSV3.1/1.09M; en) Presto/2.2.1";
const char S41445[] = "Mozilla/5.0 (Linux i686 ; U; en; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 9.70";
const char S41446[] = "Opera/9.70 (Linux armv6l ; U; HomeManager/HSV5.0/1.15C; en) Presto/2.2.1";
const char S41447[] = "Opera/9.70 (Linux 7405d0-smp ; U; en) Presto/2.2.0";
const char S41448[] = "Opera/9.70 (Linux x86_64 ; U; CE-HTML/1.0 (<profilelist><ui_profile name=\"PHILIPS_OLS_2008\"/></profilelist>); en) Presto/2.2.1";
const char S41449[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S41450[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8020 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S41451[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8020 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S41452[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S41453[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8005 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S41454[] = "Mozilla/5.0 (X11; Linux i686 on x86_64; rv:14.0) Gecko/X Firefox/14.0.1";
const char S41455[] = "Mozilla/5.0 (X11; Linux i686; rv:14.0) Gecko/X Firefox/14.0.1 Iceweasel/14.0.1";
const char S41456[] = "Mozilla/5.0 (X11; Linux i686; rv:14.0) Gecko/X Firefox/14.0.1 Firefox/14.0.1";
const char S41457[] = "Mozilla/5.0 (Ultimate Edition; X11; Linux x86; rv:8.0) Gecko/X Ultimate Edition / 3.5 Firefox/14.0.1";
const char S41458[] = "SAMSUNG-GT-C3222W/C3222WJVKC2 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S41459[] = "SAMSUNG-GT-C3222/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S41460[] = "Mozilla/5.0 (Symbian/3; Series60/5.3 NokiaC7-00/111.040.1511; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.1.4 Mobile Safari/535.1 3gpp-gba";
const char S41461[] = "Mozilla/5.0 (Symbian/3; Series60/5.3 NokiaC7-00s/111.040.1511; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.1.4 Mobile Safari/535.1 3gpp-gba";
const char S41462[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9850; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.261 Mobile Safari/534.11+";
const char S41463[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S41464[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S41465[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Special Edition Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S41466[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC VLE_U Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S41467[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Silk/1.1.0-84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Silk-Accelerated=false";
const char S41468[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Silk/1.0.22.79_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Silk-Accelerated=false";
const char S41469[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Silk/1.1.0-80) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Silk-Accelerated=false";
const char S41470[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Silk/1.0.13.328_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Silk-Accelerated=false";
const char S41471[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Silk/1.0.22.153_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Silk-Accelerated=false";
const char S41472[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-I605 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S41473[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S41474[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-I605 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S41475[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S41476[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S41477[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S41478[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S41479[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100M Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S41480[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S41481[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9100 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S41482[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100M Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S41483[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S41484[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S41485[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NOOK BNTV400 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41486[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BNTV400 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41487[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; BNTV400 Build/IMM76L)";
const char S41488[] = "Mozilla/5.0 (Series40; NokiaC2-03/06.51; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.62.10";
const char S41489[] = "Mozilla/5.0 (Series40; NokiaC2-03/07.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6";
const char S41490[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26i Build/6.1.A.2.45) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S41491[] = "Mozilla/5.0 (Linux; Android 4.0.3; LT26i Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S41492[] = "Mozilla/5.0 (Linux; Android 4.0.4;  LT26i Build/6.1.A.2.45) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S41493[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SCH-I400 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41494[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SCH-I400 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41495[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; SCH-I400 Build/FROYO)";
const char S41496[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SCH-I400Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41497[] = "Mozilla/5.0 (Linux; U; Android2.2.2; en-us; SCH-I400 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41498[] = "Mozilla/5.0 (Linux; U;Android 2.2.2; en-us; SCH-I400 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41499[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;SCH-I400 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41500[] = "Mozilla/5.0 (Android; Tablet; rv:22.0) Gecko/22.0 Firefox/22.0";
const char S41501[] = "Mozilla/5.0 (Android; Tablet; rv:22.0a2) Gecko/22.0a2 Firefox/22.0a2";
const char S41502[] = "Mozilla/5.0 (Android; Tablet; rv:22.0) Gecko/22.0 Firefox/22.0,gzip(gfe)";
const char S41503[] = "Mozilla/5.0 (Android; Tablet; rv:22.0) Gecko/22.0 Firefox/22.0,gzip(gfe) (via docs.google.com/viewer)";
const char S41504[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus 7 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S41505[] = "Mozilla/5.0 (Linux; Android 4.3.1; Nexus 7 Build/JLS36I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S41506[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xperia T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S41507[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.0.489) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S41508[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S41509[] = "Mozilla/5.0 (Linux; Android 4.1.2; One X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S41510[] = "Mozilla/5.0 (Linux; Android 4.1.1; EndeavorU Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S41511[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S41512[] = "Mozilla/5.0 (Linux; Android 4.1.9; HTC One X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S41513[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S41514[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41515[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41516[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; Transformer TF101 Build/HTK75)";
const char S41517[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; Transformer TF101 Build/HTJ85B)";
const char S41518[] = "Mozilla/5.0 (Linux; U; Android 3.2.1 KRAKD by jcarrz1; en-us; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41519[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; ja-jp; TF101-WiMAX Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41520[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S41521[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S41522[] = "Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S41523[] = "Mozilla/5.0 (Linux; Android 4.1.2; MB886 Build/9.8.2Q-8_MB886_FFW-11) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S41524[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.215 Safari/534.10";
const char S41525[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10";
const char S41526[] = "Mozilla/5.0 (Linux; U; Linux Ventana; en-US) AppleWebKit/533.1 (KHTML, like Gecko) Chrome/8.0 Safari/533.1";
const char S41527[] = "Mozilla/5.0 (X11; U; Jolicloud Linux i686; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Jolicloud/1.1 Chromium/8.0.552.237 Chrome/8.0.552.237 Safari/534.10";
const char S41528[] = "Mozilla/5.0 (Linux; U; Linux Ventana; en-us; EPAD Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S41529[] = "Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10";
const char S41530[] = "Mozilla/5.0 (Linux; U; Linux Ventana; en-gb; EPAD Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S41531[] = "Mozilla/5.0 (X11; U; Linux i686; ru-RU) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10";
const char S41532[] = "Mozilla/5.0 (Linux; U; Linux Ventana; en-us; A500 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S41533[] = "Mozilla/5.0 (Linux; U; Linux Ventana; en-us; ETBW11AA Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S41534[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ie; SAMSUNG GT-I8730/I8730XWAMB8 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41535[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-I8730-ORANGE/I8730XWAMC2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41536[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-I8730 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41537[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8730 Build/JZO54K)";
const char S41538[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-I8730T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41539[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8730T Build/JZO54K)";
const char S41540[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-I8730-ORANGE/I8730XWAMF1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41541[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I8730/I8730XWAMG1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41542[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I8730/I8730XWAMD7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41543[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; SAMSUNG GT-I8730/I8730XWAMD2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41544[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ie; SAMSUNG GT-I8730/I8730XWAMG6 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41545[] = "HTCS720t_TD/1.0 Android/4.0 release/2012 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S41546[] = "Mozilla/5.0 MSM7627A/1.0 Linux/3.0.21 Android/4.0.4 Release/11.16.2011 Browser/AppleWebKit533.1 Profile/MIDP-1.0 Configuration/CLDC-1.0 Mobile/4.0.4";
const char S41547[] = "9220 Linux/2.6.35 Android/4.0.3 Release/08.24.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1";
const char S41548[] = "CoolPad8190_CMCC_TD/1.0 Linux/3.0.8 Android/4.0 Release/10.15.2012 Browser/AppleWebkit534.3";
const char S41549[] = "Z15A-Z15A/1.0 Linux/2.6.35.7 Android/4.0 Release/10.31.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S41550[] = "sprd-C3100/1.0 Linux/2.6.35.7 Android/android 4.0.3 Release/12.25.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S41551[] = "sprd-S560/1.0 Linux/2.6.35.7 Android/4.0.3 Release/11.16.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S41552[] = "sprd-C3100/1.0 Linux/2.6.35.7 Android/android 4.0.3 Release/12.27.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S41553[] = "sprd-M2/1.0 Linux/2.6.35.7 Android/4.0.4 Release/01.28.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S41554[] = "Z15A-Z15A/1.0 Linux/2.6.35.7 Android/4.0 Release/12.26.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S41555[] = "sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/4.0.4 Release/12.18.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S41556[] = "sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/4.0.2 Release/02.21.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S41557[] = "sprd-C6200/1.0 Linux/2.6.35.7 Android/android 4.0.3 Release/01.09.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S41558[] = "sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/4.0.4 Release/02.22.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S41559[] = "sprd-M6A/1.0 Linux/2.6.35.7 Android/4.0.3 Release/03.21.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S41560[] = "sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/4.0.2 Release/03.23.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S41561[] = "Lenovo-A630t_TD/S100 Linux/3.0.13 Android/4.0 Release/10.08.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30";
const char S41562[] = "ZTEU880F1_TD/1.0 Linux/3.0.13 Android/4.0 Release/7.10.2012 Browser/AppleWebKit534.30";
const char S41563[] = "ZTEU817_TD/1.0 Linux/3.0.13 Android/4.0 Release/11.20.2012 Browser/AppleWebKit534.30";
const char S41564[] = "CoolPad8720_CMCC_TD/1.0 Linux/3.0.8 Android/4.0 Release/03.31.2013 Browser/AppleWebkit534.3";
const char S41565[] = "MTK-SKING/1.0 Linux/2.6.35.7 Android/4.0.3 Release/03.27.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S41566[] = "ZTEU960s3_TD/1.0 Linux/2.6.39 Android/4.0 Release/6.10.2012 Browser/AppleWebKit534.30";
const char S41567[] = "HuaweiT8808D_TD/1.0 Android/4.0 Release/02.01.2012 Browser/WAP2.0 appleWebkit/534.30";
const char S41568[] = "HS-T950_TD/1.0 Android/4.0.3 Release/28.10.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S41569[] = "sprd-GT-N9300/1.0 Linux/2.6.35.7 Android/4.0.3 Release/03.26.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S41570[] = "ZTE U880F_TD/1.0 Linux/2.6.39 Android/4.0 Release/5.24.2012 Browser/AppleWebKit534.30";
const char S41571[] = "HS-T909_TD/1.0 Android/4.0.3 Release/28.10.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S41572[] = "HUAWEI_G606-T00_TD/1.0 Linux/3.0.8 Android/4.0.3 Release/01.06.2013 Browser/Webkit54731 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S41573[] = "ZTE U808_TD/1.0 Linux/3.0.8 Android/4.0 Release/3.6.2013 Browser/AppleWebKit533.1(KHTML,like Gecko) Mozilla/5.0 Mobile";
const char S41574[] = "Mozilla/5.0 (Android; Tablet; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S41575[] = "Mozilla/5.0 (Android; Tablet; rv:20.0a2) Gecko/20.0a2 Firefox/20.0a2";
const char S41576[] = "Mozilla/5.0 (Android; Tablet; rv:20.0) Gecko/20.0 Firefox/20.0,gzip(gfe)";
const char S41577[] = "Mozilla/5.0 (Android; Tablet; rv:20.0.1) Gecko/20.0.1 Firefox/20.0.1";
const char S41578[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.43.7 (KHTML, like Gecko)";
const char S41579[] = "Airmail 1.0.4 rv:183 (Macintosh; Mac OS X 10.9.0; en_US)";
const char S41580[] = "Airmail 1.0.3 rv:176 (Macintosh; Mac OS X 10.9.0; en_US)";
const char S41581[] = "Airmail 1.0.4 rv:184 (Macintosh; Mac OS X 10.9.0; en_US)";
const char S41582[] = "Airmail 1.0.4 rv:183 (Macintosh; Mac OS X 10.9.0; en_GB)";
const char S41583[] = "Airmail 1.0.5 rv:189 (Macintosh; Mac OS X 10.9.0; en_US)";
const char S41584[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.51.3 (KHTML, like Gecko) Version/1.0 Safari/1";
const char S41585[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.51.3 (KHTML, like Gecko) QuickLook/5.0";
const char S41586[] = "Airmail 1.0.5 rv:192 (Macintosh; Mac OS X 10.9.0; en_GB)";
const char S41587[] = "Airmail 1.0.5 rv:192 (Macintosh; Mac OS X 10.9.0; en_US)";
const char S41588[] = "Airmail 1.0.6 rv:194 (Macintosh; Mac OS X 10.9.0; en_US)";
const char S41589[] = "Airmail 1.0.6 rv:193 (Macintosh; Mac OS X 10.9.0; en_US)";
const char S41590[] = "Airmail 1.0.5 rv:192 (Macintosh; Mac OS X 10.9.0; en_AU)";
const char S41591[] = "Airmail 1.1 rv:201 (Macintosh; Mac OS X 10.9.0; en_US)";
const char S41592[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.71 (KHTML, like Gecko) Sparrow/1178";
const char S41593[] = "Airmail 1.0.5 rv:192 (Macintosh; Mac OS X 10.9.0; de_DE)";
const char S41594[] = "ThumbnailService/39600.038 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookAir4%2C2)";
const char S41595[] = "ThumbnailService/39600.038 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (iMac12%2C2)";
const char S41596[] = "NewsFire/84 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookPro5%2C5)";
const char S41597[] = "Socialite/7825 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookPro8%2C2)";
const char S41598[] = "iCoreService (unknown version) CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (MacBookPro9%2C2)";
const char S41599[] = "CoverScout%203/3.4.3 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookPro8%2C1)";
const char S41600[] = "ThumbnailService/39600.038 CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (MacBookPro10%2C1)";
const char S41601[] = "NewsFire/84 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookAir4%2C2)";
const char S41602[] = "CoverScout%203/3.5.6 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (iMac11%2C3)";
const char S41603[] = "WebProcess/8536.26.17 CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (MacBookPro9%2C2)";
const char S41604[] = "iCoreService (unknown version) CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (MacBookAir5%2C2)";
const char S41605[] = "WebProcess/8536.26.17 CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (MacBookAir5%2C2)";
const char S41606[] = "WebProcess/8536.26.17 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (iMac12%2C1)";
const char S41607[] = "WebProcess/8536.26.17 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBook5%2C1)";
const char S41608[] = "RealPlayer%20Downloader/1725 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookPro8%2C1)";
const char S41609[] = "ThumbnailService/39600.038 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBook5%2C1)";
const char S41610[] = "Email%20Marketing/4.0 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (iMac12%2C1)";
const char S41611[] = "iCoreService (unknown version) CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (iMac13%2C1)";
const char S41612[] = "ThumbnailService/39600.038 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacPro3%2C1)";
const char S41613[] = "Caffeinated/1.3.2 CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (MacBookPro10%2C1)";
const char S41614[] = "ThumbnailService/39600.038 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (Macmini5%2C1)";
const char S41615[] = "Cookie%20Stumbler/1631 CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (MacBookPro10%2C1)";
const char S41616[] = "XProtectUpdater (unknown version) CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookPro8%2C1)";
const char S41617[] = "iCoreService (unknown version) CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBook5%2C1)";
const char S41618[] = "Caffeinated/1.3.2 CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (MacBookAir5%2C2)";
const char S41619[] = "iCoreService (unknown version) CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (Macmini6%2C1)";
const char S41620[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S41621[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S41622[] = "Mozilla/5.0 (Windows NT 6.0; rv:17.0) Gecko/X Thunderbird/17.0.8";
const char S41623[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9B206 Safari/7534.48.3";
const char S41624[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9B208 Safari/7534.48.3";
const char S41625[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9B179 Safari/7534.48.3";
const char S41626[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9B176 Safari/7534.48.3";
const char S41627[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A700 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41628[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; A700 Build/JRO03H)";
const char S41629[] = "Mozilla/5.0 (X11; Linux x86_64; rv:20.0) Gecko/X Firefox/20.0 Iceweasel/20.0";
const char S41630[] = "Mozilla/5.0 (X11; Linux i686 on x86_64; rv:20.0) Gecko/X Firefox/20.0";
const char S41631[] = "Mozilla/5.0 (Ultimate Edition; X11; Linux x64; rv:8.0) Gecko/X Ultimate Edition / 3.4.1 Firefox/20.0";
const char S41632[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Transformer Prime TF201 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41633[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Transformer Prime TF201 Build/IML74K)";
const char S41634[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; TF201 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41635[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Transformer Prime TF201 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41636[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Transformer Prime TF201 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41637[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; Transformer Prime TF201 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41638[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/11.04 Chromium/14.0.825.0 Chrome/14.0.825.0 Safari/535.1";
const char S41639[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/11.04 Chromium/14.0.835.202 Chrome/14.0.835.202 Safari/535.1";
const char S41640[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/11.04 Chromium/14.0.835.202 Chrome/14.0.835.202 Safari/535.1,gzip(gfe) (via translate.google.com)";
const char S41641[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML	 like Gecko) Ubuntu/11.04 Chromium/14.0.803.0 Chrome/14.0.803.0 Safari/535.1";
const char S41642[] = "SAMSUNG-GT-C3312/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S41643[] = "SAMSUNG-GT-C3312/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10";
const char S41644[] = "SAMSUNG-GT-C3312/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (J2ME/MIDP; Opera Mini/6.5.27510/30.3214; U; en) Presto/2.8.119 Version/11.10";
const char S41645[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.2.15 Version/10.10";
const char S41646[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.2.15 Version/10.10";
const char S41647[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.2.15 Version/10.10";
const char S41648[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.2.15 Version/10.10";
const char S41649[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4848); ru) Presto/2.2.15 Version/10.10";
const char S41650[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.2.15 Version/10.10";
const char S41651[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; ru) Opera 10.10";
const char S41652[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.2.15 Version/10.10";
const char S41653[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.2.15 Version/10.10";
const char S41654[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.2.15 Version/10.10";
const char S41655[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.2.15 Version/10.10";
const char S41656[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.2.15 Version/10.10";
const char S41657[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.2.15 Version/10.10";
const char S41658[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.2.15 Version/10.10";
const char S41659[] = "Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.2.15 Version/10.10";
const char S41660[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.2.15 Version/10.10";
const char S41661[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.2.15 Version/10.10";
const char S41662[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.2.15 Version/10.10";
const char S41663[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.2.15 Version/10.10";
const char S41664[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.2.15 Version/10.10";
const char S41665[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.2.15 Version/10.10";
const char S41666[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.2.15 Version/10.10";
const char S41667[] = "Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.2.15 Version/10.10";
const char S41668[] = "Slurp";
const char S41669[] = "*Mozilla/5.0 (compatible; Yahoo! SlurpChina; http://misc.yahoo.com.cn/help.html)";
const char S41670[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:18.0) Gecko/X SeaMonkey/2.15";
const char S41671[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:18.0) Gecko/X Firefox/18.0 SeaMonkey/2.15.2";
const char S41672[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6603 Build/10.1.1.A.1.253) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S41673[] = "SAMSUNG-SGH-F480/1.0 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S41674[] = "SAMSUNG-SGH-F480T/1.0 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S41675[] = "SAMSUNG-SGH-F480L/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S41676[] = "SAMSUNG-SGH-F480-ORANGE/F480ABIB1 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S41677[] = "SAMSUNG-SGH-F480-Vodafone/F480BUHK1 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S41678[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S41679[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S41680[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LGL45C Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41681[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LGL45C Build/GRJ22)";
const char S41682[] = "Mozilla/5.0 (webOS/1.3.1; U; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Pixi/1.0";
const char S41683[] = "Mozilla/5.0 (webOS/1.3.5.1; U; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Pixi/1.1";
const char S41684[] = "Mozilla/5.0 (webOS/1.4.0; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1";
const char S41685[] = "Mozilla/5.0 (webOS/1.4.5; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.0";
const char S41686[] = "Mozilla/5.0 (webOS/1.4.5; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1";
const char S41687[] = "Mozilla/5.0 (webOS/1.4.5; U; de-DE) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1";
const char S41688[] = "Mozilla/5.0 (webOS/1.4.5; U; en-GB) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1";
const char S41689[] = "Mozilla/5.0 (webOS/1.4.5.1; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1";
const char S41690[] = "Mozilla/5.0 (webOS/1.4.5.1; U; es-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1";
const char S41691[] = "Mozilla/5.0 (webOS/1.4.1.1; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1";
const char S41692[] = "Mozilla/5.0 (webOS/1.4.5; U; es-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.0";
const char S41693[] = "Mozilla/5.0 (webOS/1.4.5; U; fr-FR) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1";
const char S41694[] = "Mozilla/5.0 (webOS/1.4.5; U; es-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1";
const char S41695[] = "Mozilla/5.0 (webOS/1.4.5; U; es-ES) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1";
const char S41696[] = "Mozilla/5.0 (webOS/1.4.3; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1";
const char S41697[] = "Mozilla/5.0 (webOS/1.2.9.1; U; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Pixi/1.0";
const char S41698[] = "Mozilla/5.0 (webOS/1.4.0; U; es-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1";
const char S41699[] = "Mozilla/5.0(webOS/1.4.5.1; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1";
const char S41700[] = "Dolphin http client/10.0.0(222) (Android)";
const char S41701[] = "Dolphin http client/10.0.0(223) (Android)";
const char S41702[] = "Dolphin http client/10.0.0(224) (Android)";
const char S41703[] = "Dolphin http client/10.0.0(225) (Android)";
const char S41704[] = "Dolphin http client/10.0.1 INT(228) (Android)";
const char S41705[] = "Dolphin http client/10.0.0(226) (Android)";
const char S41706[] = "Dolphin http client/10.0.1 Beta2(229) (Android)";
const char S41707[] = "Dolphin http client/10.0.1(230) (Android)";
const char S41708[] = "Dolphin http client/10.0.1(231) (Android)";
const char S41709[] = "Dolphin http client/10.0.2 beta1(232) (Android)";
const char S41710[] = "Dolphin http client/10.0.2 beta2(233) (Android)";
const char S41711[] = "Dolphin http client/10.0.3 Beta(235) (Android)";
const char S41712[] = "Dolphin http client/10.0.3 Beta2(236) (Android)";
const char S41713[] = "Dolphin http client/10.0.3(238) (Android)";
const char S41714[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Safari/535.19 Silk-Accelerated=true";
const char S41715[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41716[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; MZ505 Build/Moto_MZ505) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41717[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMI87C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41718[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ01) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41719[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ13) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41720[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ19) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41721[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ27) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41722[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMI80B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41723[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41724[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ29) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41725[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ28B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41726[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ04B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41727[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMI84D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41728[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ07B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41729[] = "Mozilla/5.0 (Linux; U; Android 3.1; ru-; Xoom Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41730[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ33) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41731[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMI73) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41732[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMI90) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S41733[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; MZ505 Build/Moto_MZ505)";
const char S41734[] = "NokiaE66/1.0 (410.21.010) SymbianOS/9.2 Series60/3.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S41735[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE66-1/500.21.009; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S41736[] = "NokiaE66";
const char S41737[] = "Mozilla/5.0 (Android; Tablet; rv:19.0) Gecko/19.0 Firefox/19.0";
const char S41738[] = "Mozilla/5.0 (Android; Tablet; rv:19.0a2) Gecko/19.0a2 Firefox/19.0a2";
const char S41739[] = "Mozilla/5.0 (Android; Tablet; rv:19.0) Gecko/19.0 Firefox/19.0,gzip(gfe)";
const char S41740[] = "Mozilla/5.0 (Android; Tablet; rv:19.0.2) Gecko/19.0.2 Firefox/19.0.2";
const char S41741[] = "Mozilla/5.0 (Android; Tablet; rv:19.0.0) Gecko/19.0.0 Firefox/19.0.0";
const char S41742[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41743[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41744[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-I9000 Build/IMM76K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30";
const char S41745[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41746[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-I9000 Build/IMM76K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41747[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41748[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-I9000 Build/IML74K)";
const char S41749[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9000 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41750[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9000 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41751[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9000 Build/IMM76L)";
const char S41752[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; es-es; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41753[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9000 Build/MyICS Ultimate) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41754[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9000 Build/MyICS Ultimate Fusion) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41755[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-I9000B Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41756[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT-I9000B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41757[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9000 Build/IMM30B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41758[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-I9000 Build/IMM26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41759[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-I9000 Build/I9000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41760[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-I9000B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41761[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.2 Mobile/10A523 Safari/8536.25";
const char S41762[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.2 Mobile/10A403 Safari/8536.25";
const char S41763[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S41764[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11";
const char S41765[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S41766[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S41767[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S41768[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S41769[] = "Mozilla/5.0 (Linux; U; Android 1.6; es-es; E10i Build/1.1.A.0.12) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S41770[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; SonyEricssonE10iv Build/1.2.A.1.174) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S41771[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; SonyEricssonE10i-o Build/1.2.A.1.174) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S41772[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-za; E10i Build/1.0.A.1.37) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S41773[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; E10i Build/1.0.A.1.36) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S41774[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; E10i Build/1.2.A.1.174) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S41775[] = "Mozilla/5.0 (Linux; U; Android 1.6; fr-fr; E10i Build/1.1.A.0.8) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S41776[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; E10i Build/1.0.A.0.13) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S41777[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; SonyEricssonE10i Build/1.0.A.1.36) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S41778[] = "SonyEricssonE10i CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 1.6)";
const char S41779[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; E10i Build/1.0.A.1.38) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S41780[] = "Mozilla/5.0 (Linux; Android 4.0.3;  Transformer TF101 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S41781[] = "Mozilla/5.0 (Linux; Android 4.0.4;  Transformer TF101 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S41782[] = "Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S41783[] = "Mozilla/5.0 (Linux; Android 4.0.4; Transformer TF101 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S41784[] = "Mozilla/5.0 (Linux; Android 4.0.4; Transformer Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S41785[] = "Mozilla/5.0 (Linux; Android 4.0.3; TF101 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S41786[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I905 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41787[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I905 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41788[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-I905 Build/IMM76D)";
const char S41789[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S41790[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I535 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S41791[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T999 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S41792[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9300 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S41793[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S41794[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-T999 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S41795[] = "Mozilla/5.0 (Linux; Android 4.2.1; SCH-I535 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S41796[] = "Mozilla/5.0 (Linux; Android 4.2.2; SC-06D Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S41797[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_Salsa_C510e Build/LibertySense_GINGERBREAD_V0.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41798[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Salsa C510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41799[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_Salsa_C510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41800[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC_Salsa_C510b Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41801[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC Salsa C510b Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41802[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Salsa C510b Build/GRI40)";
const char S41803[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Salsa C510e Build/GRI40)";
const char S41804[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC C510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41805[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC C510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41806[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTCC510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41807[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Salsa C510e Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S41808[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC C510e Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S41809[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-cn; HTCC510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41810[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC C510e Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S41811[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC C510e Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S41812[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Salsa C510e Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S41813[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC C510e Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S41814[] = "Opera/9.80 (Linux; en) Presto/2.7.81 Version/11.00";
const char S41815[] = "Opera/9.80 (X11; Linux x86_64; U; pl) Presto/2.7.62 Version/11.00";
const char S41816[] = "Opera/9.80 (X11; Linux i686; U; ja) Presto/2.7.62 Version/11.01";
const char S41817[] = "Mozilla/4.0 (compatible; MSIE 8.0; X11; Linux x86_64; pl) Opera 11.00";
const char S41818[] = "Mozilla/5.0 (Windows NT 6.0; rv:14.0) Gecko/X Firefox/14.0";
const char S41819[] = "Mozilla/5.0 (Windows NT 6.0; rv:14.0) Gecko/X Firefox/14.0.1 Firefox/14.0.1";
const char S41820[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:14.0) Gecko/X Firefox/14.0.1";
const char S41821[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0 AppleWebKit (KHTML, like Gecko Chrome/6";
const char S41822[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S41823[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam GameOverlay/1742; ) AppleWebKit/534.1 (KHTML, like Gecko) Chrome/6.0.444.0 Safari/534.1";
const char S41824[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Source Client/0; ) AppleWebKit/534.1 (KHTML, like Gecko) Chrome/6.0.444.0 Safari/534.1";
const char S41825[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3,gzip(gfe) (via translate.google.com)";
const char S41826[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam GameOverlay; ) AppleWebKit/534.1 (KHTML, like Gecko) Chrome/6.0.444.0 Safari/534.1";
const char S41827[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I577 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41828[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG-SGH-I577 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41829[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SAMSUNG-SGH-I577 Build/IMM76D)";
const char S41830[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13";
const char S41831[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13";
const char S41832[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13";
const char S41833[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13";
const char S41834[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.15 Safari/534.13";
const char S41835[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13";
const char S41836[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.0 Safari/534.13";
const char S41837[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13";
const char S41838[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.84 Safari/534.13";
const char S41839[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9A405 Safari/7534.48.3";
const char S41840[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9A334 Safari/7534.48.3";
const char S41841[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:23.0) Gecko/X Firefox/23.0";
const char S41842[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:23.0) Gecko/X Firefox/23.0 AlexaToolbar/alxf-2.18";
const char S41843[] = "Mozilla/5.0 (Android; Tablet; rv:21.0) Gecko/21.0 Firefox/21.0";
const char S41844[] = "Mozilla/5.0 (Android; Tablet; rv:21.0) Gecko/21.0 Firefox/21.0,gzip(gfe)";
const char S41845[] = "Mozilla/5.0 (Android; Tablet; rv:21.0a2) Gecko/21.0a2 Firefox/21.0a2";
const char S41846[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I777 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41847[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I777 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41848[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I777 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41849[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I777 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41850[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I777 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41851[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I777 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41852[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I777 Build/JZO54K)";
const char S41853[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Galaxy S II Epic 4G Touch Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41854[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SGH-I777 Build/JRO03H)";
const char S41855[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I777 Build/cMIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41856[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I777 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41857[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-I777 Build/JZO54K)";
const char S41858[] = "Opera/9.30 (Nintendo Wii; U; ; 3642; en)";
const char S41859[] = "Mozilla/5.0 (Linux; Android 4.2.1; Transformer Pad TF300T Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S41860[] = "Mozilla/5.0 (Linux; Android 4.2.1; TF300T Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S41861[] = "Mozilla/5.0 (Linux; Android 4.2.2; TF300T Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S41862[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300T Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S41863[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300TG Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S41864[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300TL Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S41865[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Pad TF300T Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S41866[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41867[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HUAWEI Y300-0100 Build/HuaweiY300-0100)";
const char S41868[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; HTC_K2_UL Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41869[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_C525u Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41870[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_OneSV Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41871[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC One SV Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41872[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC One SV Build/IMM76I)";
const char S41873[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-ch; HTC_OneSV/1.17.166.7 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41874[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; HTC_OneSV/1.17.169.7 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41875[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_OneSV/1.18.161.2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41876[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HTC_OneSV/1.17.162.6 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41877[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; HTC_C525u-orange-LS/1.17.73.5 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41878[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; HTC_OneSV/1.16.164.5 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41879[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HTC_OneSV/1.17.111.13 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41880[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HTC_K2_UL/1.17.111.8 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41881[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; HTC_C525u-orange-LS/1.17.73.10 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41882[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; HTC_OneSV/1.17.118.5 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41883[] = "Mozilla/5.0 (SMART-TV; X11; Linux i686) AppleWebKit/534.7 (KHTML, like Gecko) Version/5.0 Safari/534.7";
const char S41884[] = "Mozilla/5.0 (SMART-TV; X11; Linux i686) AppleWebKit/535.20+ (KHTML, like Gecko) Version/5.0 Safari/535.20+";
const char S41885[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; SGH-i677)";
const char S41886[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.7.10) Gecko/X Firefox/1.0.4 (Debian package 1.0.4-2sarge5)";
const char S41887[] = "Mozilla/5.0/Nutch-1.1 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.13)";
const char S41888[] = "Mozilla/5.0 (Series40; Nokia306/03.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S41889[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I497 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41890[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SAMSUNG-SGH-I497 Build/IMM76D)";
const char S41891[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I497 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S41892[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T599 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41893[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T599N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41894[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T599 Build/JZO54K)";
const char S41895[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-T599V Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41896[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T599N Build/JZO54K)";
const char S41897[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T599V Build/JZO54K)";
const char S41898[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROIDX Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola DROIDX";
const char S41899[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROIDX Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola DROIDX";
const char S41900[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROIDXBuild/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola DROIDX";
const char S41901[] = "Mozilla/5.0 (Linux; U; Android2.2.1; en-us; DROIDX Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola DROIDX";
const char S41902[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;DROIDX Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola DROIDX";
const char S41903[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROIDX Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1480X854 motorola DROIDX";
const char S41904[] = "Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; DROIDX Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola DROIDX";
const char S41905[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROIDX Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 480X854 motorola DROIDX";
const char S41906[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROIDX Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41907[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; DROIDX Build/VZW)";
const char S41908[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; DROIDX Build/VZW)";
const char S41909[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5303 Build/12.0.A.1.257) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S41910[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus 4 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S41911[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-LG730 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41912[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-LG730 Build/JZO54K)";
const char S41913[] = "Mozilla/5.0 (Android; Mobile; rv:15.0) Gecko/15.0 Firefox/15.0a1";
const char S41914[] = "Mozilla/5.0 (Android; Mobile; rv:15.0) Gecko/15.0 Firefox/15.0a2";
const char S41915[] = "Mozilla/5.0 (Android; Mobile; rv:15.0) Gecko/15.0 Firefox/15.0";
const char S41916[] = "Mozilla/5.0 (Android; Mobile; rv:15.0) Gecko/15.0 Firefox/15.0.1";
const char S41917[] = "Mozilla/5.0 (Android; Mobile; rv:15.0.1) Gecko/15.0.1 Firefox/15.0.1";
const char S41918[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4372q Safari/8536.25";
const char S41919[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4400f Safari/8536.25";
const char S41920[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4414e Safari/8536.25";
const char S41921[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4435d Safari/8536.25";
const char S41922[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4449a Safari/8536.25";
const char S41923[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4449d Safari/8536.25";
const char S41924[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A465 Safari/8536.25";
const char S41925[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A501 Safari/8536.25";
const char S41926[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41927[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI-M931 Build/HuaweiM931)";
const char S41928[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 androidAppVersion/2.5.22 AndroidWebkitWrapper BillingSupported";
const char S41929[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MOTWX435KT Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41930[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; MOTWX435KT Build/FROYO)";
const char S41931[] = "Mozilla/5.0 (Linux; U; Android 2.2.2 (PR5); en-us; MOTWX435KT Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41932[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-; MOTWX435KT Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41933[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Triumph Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41934[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/25.0.648.205 Safari/534.16";
const char S41935[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/25.0.782.112 Safari/535.1";
const char S41936[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/25.0.742.122 Safari/534.30";
const char S41937[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.25 (KHTML, like Gecko) Chrome/25.0.703.0 Safari/534.25";
const char S41938[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/25.0.648.205 Safari/534.16";
const char S41939[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Xbox)";
const char S41940[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:21.0) Gecko/X Firefox/21.0 AlexaToolbar/alxf-2.18";
const char S41941[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:16.0.1) Gecko/X Firefox/21.0.1";
const char S41942[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:21.0) Gecko/X Firefox/21.0 IceDragon/21.0.1.15";
const char S41943[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; ADR6325 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41944[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; USCCADR6325US Carrier ID 45 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41945[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; ADR6325VW Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41946[] = "Mozilla/5.0 (Linux; U;Android 2.2; en-us; ADR6325VW Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41947[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; ADR6325 Build/FRF91)";
const char S41948[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; ADR6325 Build/FRF91) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S41949[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; ADR6325VW Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S41950[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S41951[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505X Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S41952[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S41953[] = "SAMSUNG-GT-S5260/S5260XXKG2 SHP/VPP/R5 Dolfin/2.0 NexPlayer/3.0 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B";
const char S41954[] = "SAMSUNG-GT-S5260P-ORANGE/S5260PBVKJ1 SHP/VPP/R5 Dolfin/2.0 NexPlayer/3.0 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B";
const char S41955[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S41956[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S41957[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_2) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2";
const char S41958[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S41959[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_5) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S41960[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2";
const char S41961[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2";
const char S41962[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2";
const char S41963[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/15.0.648.205 Safari/534.16";
const char S41964[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/15.0.648.205 Safari/534.16";
const char S41965[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:5.0) Gecko/20100101 Firefox/5.0";
const char S41966[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PG06100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41967[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PG06100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41968[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; PG06100 Build/GRI40)";
const char S41969[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PG06100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41970[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PG06100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41971[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PG06100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41972[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PG06100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41973[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; PG06100 Build/GRJ22)";
const char S41974[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PG06100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41975[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PG06100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41976[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LG-LS855 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41977[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LS855 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41978[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-LS855 Build/GRJ22)";
const char S41979[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LS855 Build/CTMod 3.68) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41980[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LS855 Build/CTMod 3.75 Final) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41981[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LS855 Build/CTMod 3.80) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S41982[] = "Mozilla/5.0 (Android; Tablet; rv:18.0) Gecko/18.0 Firefox/18.0";
const char S41983[] = "Mozilla/5.0 (Android; Tablet; rv:18.0a2) Gecko/18.0a2 Firefox/18.0a2";
const char S41984[] = "Mozilla/5.0 (Android; Tablet; rv:18.0) Gecko/18.0 Firefox/18.0,gzip(gfe)";
const char S41985[] = "Mozilla/5.0 (Android; Tablet; rv:18.0.2) Gecko/18.0.2 Firefox/18.0.2";
const char S41986[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S41987[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710VMUB Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S41988[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710BST Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S41989[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S41990[] = "Mozilla/5.0 (Linux; Android 4.0.3; SPH-D710 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S41991[] = "SAMSUNG-SGH-A817/A817UCLD2; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.21648; U; en-US) Opera 9.50";
const char S41992[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R830 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 USCC-R830";
const char S41993[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-R830 Build/JZO54K)";
const char S41994[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R830C Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S41995[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-R830C Build/JZO54K)";
const char S41996[] = "Nokia5233/UCWEB8.7.0.187/50/999";
const char S41997[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5233) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S41998[] = "Nokia5233UCBrowser/8.7.1.234/50/444/UCWEB";
const char S41999[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia 5233) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/352/UCWEB Mobile";
const char S42000[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5233) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba";
const char S42001[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Avant Browser)";
const char S42002[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Avant Browser; SV1; Avant Browser; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; yplus 4.1.00b)";
const char S42003[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Avant Browser; Avant Browser; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.1)";
const char S42004[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Avant Browser [avantbrowser.com]; iOpus-I-M; QXW03416; .NET CLR 1.1.4322)";
const char S42005[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Avant Browser)";
const char S42006[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Avant Browser; Avant Browser)";
const char S42007[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Avant Browser; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 2.8; .NET CLR 2.0.50727)";
const char S42008[] = "Sogou Web Sprider(compatible; Mozilla 4.0; MSIE 6.0; Windows NT 5.1; SV1; Avant Browser; InfoPath.1; .NET CLR 2.0.50727; .NET CLR1.1.4322)";
const char S42009[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Avant Browser)";
const char S42010[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Avant Browser; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; Alexa Toolbar; .NET CLR 2.0.50727)";
const char S42011[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Avant Browser)";
const char S42012[] = "Opera/9.80 (Android; Opera Mini/5.0.18302/30.3214; U; en) Presto/2.8.119 Version/11.10";
const char S42013[] = "Opera/9.80 (Android; Opera Mini/5.0.23388/27.2338; U; en) Presto/2.8.119 320x480 Android";
const char S42014[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100 (Edition Next)";
const char S42015[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100 (Edition Next)";
const char S42016[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153";
const char S42017[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153";
const char S42018[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Campaign 21)";
const char S42019[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Rambler)";
const char S42020[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Campaign 21)";
const char S42021[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-R930 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 USCC-R930";
const char S42022[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-R930 Build/GINGERBREAD)";
const char S42023[] = "Mozilla/5.0 (Linux; U; Android 2.1-goapk.com; zh-cn; HTC Hero Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42024[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-gb; Cronos Hero Build/ERD72) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42025[] = "Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; HTC HERO Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42026[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1 (7hero-astar9.3); ru-ru; HTC Hero Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42027[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; HTC Hero-Orange Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42028[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HTC Hero Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42029[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; HTC Hero Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42030[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-in; HTC Hero Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42031[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC_HERO Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42032[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Hero Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42033[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-v1.0; en-gb; HTC Hero Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42034[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; HTC Hero Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S42035[] = "Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; HTC HERO Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17HTC HERO";
const char S42036[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One 801e Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42037[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42038[] = "Mozilla/5.0 (Linux; Android 4.2.2; One Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42039[] = "Mozilla/5.0 (Linux; Android 4.2.1; HTC_One Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42040[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One dual sim Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42041[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One with MoDaCo.SWITCH Beta 8 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42042[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC_PN071 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42043[] = "Mozilla/5.0 (Series40; NokiaX2-02/11.57; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S42044[] = "Mozilla/5.0 (Series40; NokiaX2-02/10.90; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5";
const char S42045[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/8";
const char S42046[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10";
const char S42047[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10,gzip(gfe) (via translate.google.com)";
const char S42048[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S42049[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S42050[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_5) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S42051[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S42052[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7";
const char S42053[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S42054[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S42055[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S42056[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/16.0.648.205 Safari/534.16";
const char S42057[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/16.0.648.205 Safari/534.16";
const char S42058[] = "Mozilla/5.0 (Linux; Android 4.1; Build/JRN84D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42059[] = "Mozilla/5.0 (Linux; Android 4.1.1; cm_tenderloin Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S42060[] = "Mozilla/5.0 (Linux; Android 4.1.1; Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42061[] = "Mozilla/5.0 (Linux; Android 4.1.2; cm_tenderloin Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S42062[] = "Mozilla/5.0 (Linux; Android 4.1.2; EPAD Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S42063[] = "Mozilla/5.0 (Linux; Android 4.1.1; Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S42064[] = "Mozilla/5.0 (Linux; Android 4.1.2; Transformer Pad Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S42065[] = "Mozilla/5.0 (Linux; Android 4.1.1; MiSS Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S42066[] = "Mozilla/5.0 (Linux; Android 4.1.1; Dark JellyBellyG1 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42067[] = "Mozilla/5.0 (Linux; Android 4.1.1; X720D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42068[] = "Mozilla/5.0 (Linux; Android 4.1.2; VegaBean Beta 6 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S42069[] = "Mozilla/5.0 (Linux; Android 4.1.2; PG8600 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42070[] = "Mozilla/5.0 (Linux; Android 4.1.1;  Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S42071[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_gq2002_5647_notv Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42072[] = "Mozilla/5.0 (Linux; Android 4.1.2; unknown Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42073[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbla668_9p017 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42074[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbla668_9p017_fwvga Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42075[] = "Mozilla/5.0 (Linux; Android 4.1.2; skz_tenderloin Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S42076[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbl1_9p017 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42077[] = "Mozilla/5.0 (Linux; Android 4.1.2; Motorola Galaxy s3 Build/9.8.2O-72_VZW-8) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42078[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1911_v77_gq2000_9p017 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42079[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1911_v77_gq2000_9p017_6628 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42080[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1909c_v77_gq1008_9p017_sui_a41_6628 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42081[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbla668_9p017_nozhrtw Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42082[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1908_v77_jbl1_9p017_6628 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42083[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_gq2002_9p017 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42084[] = "Mozilla/5.0 (Linux; Android 4.1.5; iPhone5 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42085[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbl1 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42086[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jdt1_9p017 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42087[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_cxq_asx Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42088[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1911_v77_gq2000_a41_6628_v20 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42089[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbla668_fwvga Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42090[] = "Mozilla/5.0 (Linux; Android 4.1.1; AML-MX REF Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S42091[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1920_v77_zlh_9p017_asx Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42092[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1909l_v77_gq1008_a41_v20 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42093[] = "Mozilla/5.0 (Linux; Android 4.1.1; Android Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S42094[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_gq2002_9p017_fg Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42095[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-C729 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S42096[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-C729Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S42097[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; LG-C729 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S42098[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-C729 Build/GRJ22)";
const char S42099[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-C729 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S42100[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;LG-C729 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S42101[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; LG-C729 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S42102[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IdeaTabA2109A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42103[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; IdeaTabA2109A Build/IMM76D)";
const char S42104[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IdeaTabA2109A Build/IMM76D) UC AppleWebKit/534.31 (KHTML like Gecko) Mobile Safari/534.31";
const char S42105[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I545 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42106[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SCH-I545 Build/JDQ39)";
const char S42107[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-; SCH-I545 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42108[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I545 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42109[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I545L Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42110[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.2.2;dev=jfltevzw;devm=SCH-I545;man=samsung;brw=Android;shwd=1080x1920;shwa=1080x1845;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S42111[] = "Mozilla/5.0(Windows; U; Windows NT 5.2; rv:1.9.2) Gecko/X Firefox/8.0";
const char S42112[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.2.3) Gecko/X Firefox/8.0";
const char S42113[] = "User-Agent: Mozilla/5.0 (Windows NT 5.2; rv:8.0) Gecko/X Firefox/8.0";
const char S42114[] = "Mozilla/5.0 (Windows NT 5.1; rv:8.0) Gecko/X Firefox/8.0 Firefox/8.0";
const char S42115[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.3) Gecko/X Firefox/8.0";
const char S42116[] = "Mozilla/5.0 (Windows NT 5.1; rv:8.0; en_us) Gecko/X Firefox/8.0";
const char S42117[] = "Mozilla/5.0 (Series40; Nokia302/14.53; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6";
const char S42118[] = "Mozilla/5.0 (Series40; Nokia302/14.78; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.9";
const char S42119[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I727R Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42120[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E110S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42121[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I727R Build/JZO54K)";
const char S42122[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E110S Build/JZO54K)";
const char S42123[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; GT-I9210 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42124[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SGH-I727R Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42125[] = "Mozilla/5.0 (Linux; Android 4.2.3; I9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S42126[] = "Mozilla/5.0 (Linux; Android 4.2.9; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S42127[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S42128[] = "Mozilla/5.0 (Linux; Android 4.2.3; I9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S42129[] = "Mozilla/5.0 (Linux; Android 4.2.9; GT-I9500 Build/JDQ39.I9500ZSUAMDF) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S42130[] = "Mozilla/5.0 (Linux; Android 4.2.9; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S42131[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S42132[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42133[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S42134[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHW-M250K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42135[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHW-M250S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42136[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHW-M250L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42137[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHW-M250S Build/JZO54K)";
const char S42138[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHW-M250K Build/JZO54K)";
const char S42139[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHW-M250K/KKJME3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42140[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHW-M250L Build/JZO54K)";
const char S42141[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SHW-M250S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42142[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; SHW-M250S Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42143[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; SHW-M250K Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42144[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHW-M250K/KKJMG1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42145[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I847 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42146[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SAMSUNG-SGH-I847 Build/GINGERBREAD)";
const char S42147[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SAMSUNG-SGH-I847 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42148[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9B206 Safari/7534.48.3";
const char S42149[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9B179 Safari/7534.48.3";
const char S42150[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9B176 Safari/7534.48.3";
const char S42151[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9B208 Safari/7534.48.3";
const char S42152[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9A405 Safari/7534.48.3";
const char S42153[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MDDR; .NET4.0C; .NET4.0E; .NET CLR 1.1.4322; Tablet PC 2.0); 360Spider";
const char S42154[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P5110 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S42155[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P5110 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S42156[] = "Mozilla/5.0 (Linux; Android 4.0.3;  GT-P5110 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S42157[] = "Mozilla/5.0 (Linux; Android 4.0.4;  GT-P5110 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S42158[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-MS870 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42159[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-MS870 Build/IMM76L)";
const char S42160[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 10 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S42161[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 10 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S42162[] = "Mozilla/5.0 (Linux; Android 4.2; Nexus 10 Build/JVP15P) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S42163[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; HTC6990LVW)";
const char S42164[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6603 Build/10.1.A.1.434) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42165[] = "Mozilla/5.0 (Symbian/3; Series60/5.2 Nokia500/011.033; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.37 Mobile Safari/533.4 3gpp-gba";
const char S42166[] = "Nokia500";
const char S42167[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; NABI2-NV7A Build/IMM76L)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42168[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NABI2-NV7A Build/IMM76L)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42169[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; NABI2-NV7A-UK Build/IMM76L)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42170[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ie; NABI2-NV7A-IE Build/IMM76L)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42171[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NABI2-NV7A Build/IMM76D)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42172[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NABI2-NV7A Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon";
const char S42173[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; PadFone 2 Build/JRO03L)";
const char S42174[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; PadFone 2 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42175[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PadFone 2 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42176[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T699 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42177[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-T699 Build/IMM76D)";
const char S42178[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Iron/5.0.381.0 Chrome/5.0.381 Safari/533.4";
const char S42179[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_Amaze_4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S42180[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_Amaze_4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42181[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; HTC Amaze 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42182[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC_Amaze_4G Build/GRJ22)";
const char S42183[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;HTC_Amaze_4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42184[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; HTC_Ruby Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42185[] = "Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; HTC_Amaze_4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42186[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Amaze 4G Build/GRJ22)";
const char S42187[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_Amaze_4GBuild/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42188[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;HTC_Amaze_4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S42189[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_Amaze_4GBuild/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S42190[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Ruby Build/GRJ22)";
const char S42191[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Amaze 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42192[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Ruby Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S42193[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-pk; HTC_Amaze Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42194[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-pk; HTC Ruby Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42195[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HITECH-Amaze-S500 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42196[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SM-T310 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42197[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SM-T310 Build/JDQ39)";
const char S42198[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SM-T310X Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42199[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8H7 Safari/7534.48.3";
const char S42200[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8J2 Safari/7534.48.3";
const char S42201[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8G4 Safari/7534.48.3";
const char S42202[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8F190 Safari/7534.48.3";
const char S42203[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8L1 Safari/7534.48.3";
const char S42204[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8K2 Safari/7534.48.3";
const char S42205[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 435 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/21.0.1180.77 Mobile/8L1 Safari/7534.48.3";
const char S42206[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:19.0) Gecko/X SeaMonkey/2.16";
const char S42207[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:19.0) Gecko/X Firefox/19.0 SeaMonkey/2.16.2";
const char S42208[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus 4 Build/JWR66V) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.111 Mobile Safari/537.36";
const char S42209[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.3) Gecko/X Firefox/8.0 (.NET CLR 3.5.30729)";
const char S42210[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:8.0) Gecko/X Firefox/8.0 Firefox/8.0";
const char S42211[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:8.0) Gecko/X Firefox/8.0";
const char S42212[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:8.0) Gecko/X Firefox/8.02011-10-05 17:28:18";
const char S42213[] = "Mozilla/5.0 (Windows; Windows NT 6.1; Win64; x64; rv:8.0.1) Gecko/X Firefox/8.0.1";
const char S42214[] = "Mozilla/5.0 (Windows NT 6.1; rv:8.0.1) Gecko/X Firefox/8.0.1,gzip(gfe)";
const char S42215[] = "Mozilla/5.0 (compatible; Firefox/8.0.1; Windows NT 6.1; WOW64; Trident/5.0)";
const char S42216[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.1.2) Gecko/X Firefox/8 GTB5";
const char S42217[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3G Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42218[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3G Build/FRG83B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42219[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42220[] = "Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; T-Mobile myTouch 3G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42221[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-; T-Mobile myTouch 3G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42222[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-MobilemyTouch 3G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42223[] = "Mozilla/5.0 (Linux; U; Android2.2.1; en-us; T-Mobile myTouch 3G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42224[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;T-Mobile myTouch 3G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42225[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3GBuild/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42226[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S42227[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch3G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42228[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3G Build/FRG83D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S42229[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; T-Mobile myTouch 3G Build/FRG83D)";
const char S42230[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; T-Mobile myTouch 3G Build/FRK76C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42231[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.3; T-Mobile myTouch 3G Build/FRK76C)";
const char S42232[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3G Build/FRD91_INT) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42233[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; T-MobilemyTouch 3G Build/FRK76C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42234[] = "Mozilla/5.0 (Linux; U; Android2.2.3; en-us; T-Mobile myTouch 3G Build/FRK76C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42235[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; en-us;T-Mobile myTouch 3G Build/FRK76C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42236[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; es-; T-Mobile myTouch 3G Build/FRK76C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42237[] = "Mozilla/5.0 (Linux; U;Android 2.2.3; en-us; T-Mobile myTouch 3G Build/FRK76C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42238[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Kindle Fire Build/GRK39F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42239[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Amazon Kindle Fire Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42240[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Kindle Fire Build/IML74k) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42241[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Amazon Kindle Fire Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42242[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Amazon Kindle Fire Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42243[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Amazon Kindle Fire Build/MR1)";
const char S42244[] = "Mozilla/5.0 (Linux; U; Android 4.0.4-2.4.20; en-us; Amazon Kindle Fire Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42245[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Amazon Kindle Fire Build/6.2.1_GX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42246[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Amazon Kindle Fire Build/IML74k)";
const char S42247[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Amazon Kindle Fire Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42248[] = "Biblia/1.1.3 (Linux; U; Android 4.0.4; es-US; Amazon Kindle Fire Build/cm_otter-userdebug 4.0.4 IMM76I eng.x0xhellx0x.X.X test-keys)";
const char S42249[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Kindle Fire Build/IML74k)";
const char S42250[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42251[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; KFOT Build/IML74K)";
const char S42252[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; KFOT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42253[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; KF012 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42254[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; KFOT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42255[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SPH-D710 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42256[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SPH-D710 Build/GINGERBREAD)";
const char S42257[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SPH-D710 Build/GINGERBREAD)";
const char S42258[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-D710 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42259[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-; SPH-D710 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42260[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SPH-D710 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42261[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-D710 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S42262[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; MID7107SC Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42263[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; DOOV_D710 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42264[] = "Mozilla/5.0 (Linux; U; Android GINGERBREAD 2.3.6; en-us; SPH-D710 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42265[] = "BlackBerry8320/4.5.0.81 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100";
const char S42266[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S42267[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300TL Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S42268[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300TG Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S42269[] = "Mozilla/5.0 (Linux; Android 4.2.2; TF300T Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S42270[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Pad TF300T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S42271[] = "Mozilla/5.0 (Linux; Android 4.2.2; Transformer Pad TF300T Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S42272[] = "nutch-solr-integration/1.1";
const char S42273[] = "Tiny Tiny RSS/1.5.5 (http://tt-rss.org/)";
const char S42274[] = "Tiny Tiny RSS/1.5.10 (http://tt-rss.org/)";
const char S42275[] = "Tiny Tiny RSS/1.5.11 (http://tt-rss.org/)";
const char S42276[] = "Tiny Tiny RSS/1.5.9 (http://tt-rss.org/)";
const char S42277[] = "Tiny Tiny RSS/1.6.0 (http://tt-rss.org/)";
const char S42278[] = "Tiny Tiny RSS/1.5.8.1 (http://tt-rss.org/)";
const char S42279[] = "R6_VoteReader(www.radian6.com/votes)";
const char S42280[] = "Tiny Tiny RSS/1.6.1 (http://tt-rss.org/)";
const char S42281[] = "Tiny Tiny RSS/1.6.2 (http://tt-rss.org/)";
const char S42282[] = "Mozilla/5.0 (compatible; LinkChecker/8.4; +http://wummel.github.com/linkchecker/)";
const char S42283[] = "Tiny Tiny RSS/1.7.2 (http://tt-rss.org/)";
const char S42284[] = "Tiny Tiny RSS/1.7.4 (http://tt-rss.org/)";
const char S42285[] = "Tiny Tiny RSS/1.7.5 (http://tt-rss.org/)";
const char S42286[] = "Tiny Tiny RSS/1.7.8 (http://tt-rss.org/)";
const char S42287[] = "Tiny Tiny RSS/1.7.6 (http://tt-rss.org/)";
const char S42288[] = "Tiny Tiny RSS/1.7.9 (http://tt-rss.org/)";
const char S42289[] = "Python-urllib/2.7 AppEngine-Google; (+http://code.google.com/appengine; appid: s~servicescdt)";
const char S42290[] = "Tiny Tiny RSS/1.8 (http://tt-rss.org/)";
const char S42291[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; htc_h2000c Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42292[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; HTC_H2000C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42293[] = "Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Safari/535.19 Silk-Accelerated=true";
const char S42294[] = "Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaE7-00/014.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.8.10 3gpp-gba";
const char S42295[] = "Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaE7-00/022.014; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.37 Mobile Safari/533.4 3gpp-gba";
const char S42296[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B144 Safari/8536.25";
const char S42297[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B329 Safari/8536.25";
const char S42298[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B146 Safari/8536.25";
const char S42299[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B141 Safari/8536.25";
const char S42300[] = "Opera/8.01 (J2ME/MIDP; Opera Mini; en; U; ssr)";
const char S42301[] = "Opera/9.50 (J2ME/MIDP; Opera Mini; U; en)";
const char S42302[] = "Opera/9.80 (J2ME/MIDP; Opera Mini; U; en) Presto/2.8.119 Version/10.54";
const char S42303[] = "Opera/9.80 (J2ME/MIDP; Opera Mini; U; en) Presto/2.8.119 Version/11.10";
const char S42304[] = "Opera/9.80 (J2ME/MIDP; Opera Mini; U; en) Presto/2.8.119  ? ?";
const char S42305[] = "Opera/9.80 (J2ME/MIDP; Opera Mini//28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S42306[] = "0 Opera/9.80 (J2ME/MIDP; Opera Mini; U; en) Presto/2.8.119 Version/11.10";
const char S42307[] = "Opera/9.80 (J2ME/MIDP; Opera Mini//29.3417; U; en) Presto/2.8.119 Version/11.10";
const char S42308[] = "Opera/9.80 (J2ME/MIDP; Opera Mini//30.3214; U; en) Presto/2.8.119 Version/11.10";
const char S42309[] = "Opera/9.80 (J2ME/MIDP; Opera Mini//31.1395; U; en) Presto/2.8.119 Version/11.10";
const char S42310[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PantechP8010 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42311[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; PantechP8010 Build/IMM76D)";
const char S42312[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42313[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P6200 Build/IMM76D)";
const char S42314[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT-P6200L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42315[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42316[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-P6200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42317[] = "Vyrso/2.0.7 (Linux; U; Android 4.0.3; en-US; GT-P6200 Build/X)";
const char S42318[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6200 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S42319[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6200 Build/IMM76D; 1024*552) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S42320[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6200 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 parameters/{scr=552_1024,cm=1,ql=h,imei=X,bm=1,eng=adrmini15,mdm=1,mdl=GT-P6200,ps=1} tiantian(securitypay,securityinstalled) tiantian(tenpay)";
const char S42321[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6200 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42322[] = "Logos/2.1.2 (Linux; U; Android 4.0.4; en-US; GT-P6200 Build/IMM76D.DXLPB)";
const char S42323[] = "Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6200 Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S42324[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P6200L Build/IMM76D)";
const char S42325[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S42326[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6200) antivirusHttpClient/2.0 (KHTML, like Gecko) Version/Version 2.3/Antivirus pro";
const char S42327[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6200 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42328[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42329[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6200 Build/IMM76D) Daily Mail Online/1.8.3";
const char S42330[] = "Logos/2.2.7 (Linux; U; Android 4.0.4; en-GB; GT-P6200 Build/IMM76D.DDLP5)";
const char S42331[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GT-P6200 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42332[] = "Logos/2.2.8.1 (Linux; U; Android 4.0.4; ru-RU; GT-P6200 Build/IMM76D.XXLQ1)";
const char S42333[] = "Logos/2.2.12 (Linux; U; Android 4.0.4; en-US; GT-P6200 Build/IMM76D.XXLQ3)";
const char S42334[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6200L Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 parameters/{scr=552_1024,cm=1,ql=l,imei=X,bm=1,eng=adrmini15,mdm=1,mdl=GT-P6200L,ps=1} tiantian(securitypay) tiantian(tenpay)";
const char S42335[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6200 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42336[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-; GT-P6200L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42337[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6200 Build/IMM76D) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.2 Version/4.0 Mobile Safari/530.17";
const char S42338[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42339[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42340[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42341[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100T Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42342[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100M Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42343[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42344[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100T Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42345[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; SonyEricssonMK16a Build/4.0.A.2.335) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42346[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; MK16a Build/4.0.A.2.335) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42347[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonMK16a Build/4.0.1.A.0.24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42348[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; SonyEricssonMK16a Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42349[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; MK16a Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42350[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; SonyEricssonMK16a Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42351[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; MK16a Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42352[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MK16a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42353[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonMK16a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42354[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; MK16a Build/4.0.2.A.0.58)";
const char S42355[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; MK16a Build/4.0.A.2.335)";
const char S42356[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E970 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42357[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E970/E970 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42358[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E970 Build/JZO54K)";
const char S42359[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E970/E97020j Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42360[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; generic Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S42361[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; xpndr_ihome Build/DRD35) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S42362[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; eeepc Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S42363[] = "CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 1.6)";
const char S42364[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S42365[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_2) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S42366[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S42367[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S42368[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_5) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S42369[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S42370[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S42371[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30";
const char S42372[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/12.0.648.205 Safari/534.16";
const char S42373[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/12.0.648.205 Safari/534.16";
const char S42374[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; ME371MG Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42375[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; ME371MG Build/JZO54K)";
const char S42376[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ME371MG Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42377[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.246 Mobile Safari/537.35+";
const char S42378[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.402 Mobile Safari/537.35+";
const char S42379[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1337 Mobile Safari/537.35+";
const char S42380[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.228 Mobile Safari/537.35+";
const char S42381[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.444 Mobile Safari/537.35+";
const char S42382[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.483 Mobile Safari/537.35+";
const char S42383[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.93 Mobile Safari/537.35+";
const char S42384[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.341 Mobile Safari/537.35+";
const char S42385[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.468 Mobile Safari/537.35+";
const char S42386[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.519 Mobile Safari/537.35+";
const char S42387[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.552 Mobile Safari/537.35+";
const char S42388[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.663 Mobile Safari/537.35+";
const char S42389[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.720 Mobile Safari/537.35+";
const char S42390[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.783 Mobile Safari/537.35+";
const char S42391[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.810 Mobile Safari/537.35+";
const char S42392[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.852 Mobile Safari/537.35+";
const char S42393[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.936 Mobile Safari/537.35+";
const char S42394[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.981 Mobile Safari/537.35+";
const char S42395[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1023 Mobile Safari/537.35+";
const char S42396[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1077 Mobile Safari/537.35+";
const char S42397[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1047 Mobile Safari/537.35+";
const char S42398[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1197 Mobile Safari/537.35+";
const char S42399[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1113 Mobile Safari/537.35+";
const char S42400[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1155 Mobile Safari/537.35+";
const char S42401[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1425 Mobile Safari/537.35+";
const char S42402[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1323 Mobile Safari/537.35+";
const char S42403[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1371 Mobile Safari/537.35+";
const char S42404[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1443 Mobile Safari/537.35+";
const char S42405[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.42 Mobile Safari/537.35+";
const char S42406[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1157 Mobile Safari/537.35+";
const char S42407[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1521 Mobile Safari/537.35+";
const char S42408[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1671 Mobile Safari/537.35+";
const char S42409[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1629 Mobile Safari/537.35+";
const char S42410[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1581 Mobile Safari/537.35+";
const char S42411[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1725 Mobile Safari/537.35+";
const char S42412[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.1.200 Mobile Safari/537.35+";
const char S42413[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1719 Mobile Safari/537.35+";
const char S42414[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1707 Mobile Safari/537.35+";
const char S42415[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1065 Mobile Safari/537.35+";
const char S42416[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1743 Mobile Safari/537.35+";
const char S42417[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1737 Mobile Safari/537.35+";
const char S42418[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1731 Mobile Safari/537.35+";
const char S42419[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1445 Mobile Safari/537.35+";
const char S42420[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1761 Mobile Safari/537.35+";
const char S42421[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1767 Mobile Safari/537.35+";
const char S42422[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1233 Mobile Safari/537.35+";
const char S42423[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.1.536 Mobile Safari/537.35+";
const char S42424[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1779 Mobile Safari/537.35+";
const char S42425[] = "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.1.641 Mobile Safari/537.35+";
const char S42426[] = "Mozilla/5.0 (Android; Tablet; rv:17.0) Gecko/17.0 Firefox/17.0";
const char S42427[] = "Mozilla/5.0 (Android; Tablet; rv:17.0a1) Gecko/17.0a1 Firefox/17.0a1";
const char S42428[] = "Mozilla/5.0 (Android; Tablet; rv:17.0a2) Gecko/17.0a2 Firefox/17.0a2";
const char S42429[] = "Mozilla/5.0 (Android; Tablet; rv:17.0) Gecko/17.0 Firefox/17.0,gzip(gfe)";
const char S42430[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9000 Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42431[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9000 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42432[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9000 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42433[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9000 Build/GINGERBREAD; CyanogenMod-RemICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42434[] = "MobileSafari/7534.48.3 CFNetwork/548.1.4 Darwin/11.0.0";
const char S42435[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; USCCADR6325US Carrier ID 45 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42436[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6325 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42437[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; ADR6325 Build/GRJ22)";
const char S42438[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8020 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S42439[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8005 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S42440[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S42441[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8020 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S42442[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S42443[] = "Nokia7230-1c/5.0 (10.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S42444[] = "Nokia7230/5.0 (10.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7230";
const char S42445[] = "Nokia7230/5.0 (06.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S42446[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; Inspire 4G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42447[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC-A9192/1.0Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42448[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;HTC-A9192/1.0 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42449[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC-A9192/1.0 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42450[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC-A9192/1.0 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42451[] = "Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; HTC-A9192/1.0 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42452[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC-A9192/1.0 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S42453[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-mx; Inspire HD Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42454[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; Inspire HD Build/FRG83D)";
const char S42455[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I5800 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42456[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I5800 Build/ECLAIR)";
const char S42457[] = "2.1 GT-I5800---Mozilla/5.0 (Linux; U; Android 2.1; xx-xx; GT-I5800 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42458[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.4; zh-CN; rv:1.9.0.17) Gecko/X Firefox/3.0.17";
const char S42459[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Acoo Browser; GTB5; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Maxthon; InfoPath.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618)";
const char S42460[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Maxthon; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.04506; Media Center PC 5.0; InfoPath.1)";
const char S42461[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.0; SV1; .NET CLR 1.1.4322; MAXTHON)";
const char S42462[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Maxthon; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; .NET CLR 1.1.4322; InfoPath.2)";
const char S42463[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Maxthon; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET CLR 3.0.30618)";
const char S42464[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; Maxthon)";
const char S42465[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 6.0; Maxthon)";
const char S42466[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Maxthon)";
const char S42467[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; Maxthon)";
const char S42468[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Maxthon)";
const char S42469[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; SV1; .NET CLR 1.1.4322; MAXTHON)";
const char S42470[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Maxthon; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.04506; Media Center PC 5.0; InfoPath.1)";
const char S42471[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Maxthon; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; .NET CLR 1.1.4322)";
const char S42472[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAXTHON)";
const char S42473[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; MAXTHON)";
const char S42474[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)";
const char S42475[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; Avant Browser; Maxthon)";
const char S42476[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; SV1; Maxthon)";
const char S42477[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Maxthon; Maxthon)";
const char S42478[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)";
const char S42479[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Maxthon; SV1)";
const char S42480[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Maxthon; Maxthon)";
const char S42481[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; SV1; Maxthon)";
const char S42482[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Avant Browser; Maxthon)";
const char S42483[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; MAXTHON)";
const char S42484[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; Maxthon; Avant Browser)";
const char S42485[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; Maxthon; Maxthon)";
const char S42486[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; SV1; Maxthon)";
const char S42487[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Avant Browser; Maxthon)";
const char S42488[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; Maxthon; Maxthon)";
const char S42489[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.0; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)";
const char S42490[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-R530M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42491[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-R530M Build/IMM76D)";
const char S42492[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SCH-R530M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42493[] = "NokiaX3-00/5.0 (11.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax3-00";
const char S42494[] = "NokiaX3-00/5.0 (04.11) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S42495[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/17.0.963.12) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.12 Safari/535.11";
const char S42496[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/17.0.963.46) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.46 Safari/535.11";
const char S42497[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/17.0.963.56) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S42498[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/17.0.963.56)";
const char S42499[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/17.0.963.56)";
const char S42500[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/17.0.963.56) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S42501[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/17.0.963.65) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.65 Safari/535.11";
const char S42502[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/17.0.963.66) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11";
const char S42503[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/17.0.963.66) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11";
const char S42504[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/17.0.963.78) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.78 Safari/535.11";
const char S42505[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/17.0.963.78) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.78 Safari/535.11";
const char S42506[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/17.0.963.79) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11";
const char S42507[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/17.0.963.79) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11";
const char S42508[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/17.0.963.79; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S42509[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/17.0.963.83) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.83 Safari/535.11";
const char S42510[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/17.0.963.46; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; MS-RTC LM 8; AskTbORJ/5.13.1.18107)";
const char S42511[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/17.0.963.56; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)";
const char S42512[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/17.0.963.83) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.83 Safari/535.11";
const char S42513[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/17.0.963.78; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)";
const char S42514[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/17.0.963.46) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.46 Safari/535.11";
const char S42515[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/17.0.963.46; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; Tablet PC 2.0; MS-RTC LM 8; QG Managed)";
const char S42516[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/17.0.963.65; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S42517[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MDDR; MS-RTC LM 8; .NET4.0C; BRI/2; chromeframe/17.0.963.46)";
const char S42518[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/17.0.963.65; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S42519[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/17.0.963.78; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; MS-RTC LM 8; InfoPath.3)";
const char S42520[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/17.0.963.78; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)";
const char S42521[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/17.0.963.78)";
const char S42522[] = "Opera/9.80 (Android; Opera Mini/7.28879/27.1662; U; en-us) Presto/2.8.119 Version/11.10 UCBrowser/8.5.3.246/145/355/";
const char S42523[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ME172V Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42524[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ME172V Build/JRO03H)";
const char S42525[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S42526[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S42527[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S42528[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S42529[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9100 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S42530[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24";
const char S42531[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S42532[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S42533[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_2) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24";
const char S42534[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S42535[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24";
const char S42536[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24";
const char S42537[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_5) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.57 Safari/534.24";
const char S42538[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.20 (KHTML, like Gecko) Chrome/11.0.672.2 Safari/534.20";
const char S42539[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.24 (KHTML	 like Gecko) Chrome/11.0.696.65 Safari/534.24";
const char S42540[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/11.0.648.205 Safari/534.16";
const char S42541[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/11.0.648.205 Safari/534.16";
const char S42542[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 Twitter for iPad";
const char S42543[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190 Twitter for iPad";
const char S42544[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 Twitter for iPad";
const char S42545[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 Twitter for iPad";
const char S42546[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Twitter for iPad";
const char S42547[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2 Twitter for iPad";
const char S42548[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Twitter for iPad";
const char S42549[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F191 Twitter for iPad";
const char S42550[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H8 Twitter for iPad";
const char S42551[] = "Mozilla/5.0 (Linux; U; Android 1.6; sv-se; E15i Build/1.3.A.0.62) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S42552[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; E15i Build/1.3.A.0.50) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S42553[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; SonyEricssonE15i Build/1.3.A.0.58) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S42554[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-ie; SonyEricssonE15iv Build/1.3.A.0.50) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S42555[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; E15i Build/1.3.A.0.58) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S42556[] = "Mozilla/5.0 (Linux; U; Android 1.6; fr-fr; SonyEricssonE15i-o Build/1.3.A.0.50) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S42557[] = "Mozilla/5.0 (Linux; U; Android 1.6; es-mx; SonyEricssonE15a Build/1.3.A.0.50) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S42558[] = "Mozilla/5.0 (Linux; U; Android 1.6; es-mx; E15a Build/1.3.A.0.50) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S42559[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; E15a Build/1.3.A.0.68) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S42560[] = "PVPlayer SonyEricssonE15i CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 1.6)";
const char S42561[] = "Mozilla/5.0 (Series40; Nokia201/11.56; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S42562[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42563[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42564[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42565[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42566[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42567[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42568[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42569[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42570[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42571[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I535 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42572[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42573[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T999 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42574[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-t999 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42575[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; SC-06D Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42576[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L710 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42577[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9305 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42578[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I747 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42579[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I747 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42580[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530U Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42581[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-I747 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42582[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530C Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42583[] = "Mozilla/5.0 (Linux; Android 4.2.1; SAMSUNG-SGH-I747 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42584[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/15.0.874.121)";
const char S42585[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/15.0.874.106; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; MS-RTC LM 8)";
const char S42586[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/15.0.874.106) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.106 Safari/535.2";
const char S42587[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/15.0.874.106) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.106 Safari/535.2";
const char S42588[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/15.0.874.106)";
const char S42589[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/15.0.874.121; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.2)";
const char S42590[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/15.0.874.121) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S42591[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/15.0.874.121) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S42592[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/15.0.874.121; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)";
const char S42593[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/15.0.874.120) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.120 Safari/535.2";
const char S42594[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/15.0.874.120) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.120 Safari/535.2";
const char S42595[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/15.0.874.121; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2)";
const char S42596[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/15.0.874.121; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)";
const char S42597[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/15.0.874.121; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C; .NET4.0E)";
const char S42598[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/15.0.849.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/15.0.849.0 Safari/535.1";
const char S42599[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/15.0.854.0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.854.0 Safari/535.2";
const char S42600[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/15.0.874.120; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.2; Zune 4.7)";
const char S42601[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/15.0.874.121; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.2; Tablet PC 2.0)";
const char S42602[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11";
const char S42603[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11";
const char S42604[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.60 Safari/534.24";
const char S42605[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.60 Safari/534.24";
const char S42606[] = "Mozilla/5.0 (Windows; Windows NT 6.2) AppleWebKit/534.23 (KHTML, like Gecko) Chrome/11.0.686.3 Safari/534.23";
const char S42607[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/534.20 (KHTML, like Gecko) Chrome/11.0.672.2 Safari/534.20";
const char S42608[] = "Opera/9.80 (Windows NT 5.1; U; Edition IBIS; Trident/5.0)";
const char S42609[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ke; U8185 Build/HuaweiU8185) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42610[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; U8185 Build/HuaweiU8185)";
const char S42611[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U8185 Build/r0_dr-brand_hwu8185) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42612[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I827 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42613[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; SGH-I827D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42614[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SAMSUNG-SGH-I827 Build/GINGERBREAD)";
const char S42615[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-I827D Build/GINGERBREAD)";
const char S42616[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S42617[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S42618[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S42619[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S42620[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S42621[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S42622[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S42623[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L720 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S42624[] = "Opera/9.80 (BREW; Opera Mini/6.0.3/27.2338; U; en) Presto/2.8.119 320X240 Samsung SCH-U380";
const char S42625[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; USCCADR6285US Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42626[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Hero S Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42627[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Hero S Build/GRJ22)";
const char S42628[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Hero S Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S42629[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9B176 Safari/7534.48.3";
const char S42630[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9B206 Safari/7534.48.3";
const char S42631[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9A405 Safari/7534.48.3";
const char S42632[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; A1-810 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42633[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; A1-810 Build/JDQ39)";
const char S42634[] = "stagefright/1.2 (Linux;Android 4.2.2) acer_A1-810";
const char S42635[] = "NokiaX2-00/5.0 (04.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-00";
const char S42636[] = "NokiaX2-00/5.0 (08.25) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S42637[] = "NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S42638[] = "Opera/9.80 (X11; Linux x86_64) Presto/2.12.388 Version/12.15";
const char S42639[] = "Mozilla/5.0 (X11; Linux x86_64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.15";
const char S42640[] = "Mozilla/5.0 (compatible; MSIE 9.0; X11; Linux i686) Opera 12.15";
const char S42641[] = "Opera/9.80 (X11; Linux i686) Presto/2.12.388 Version/12.15,gzip(gfe)";
const char S42642[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9350; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.261 Mobile Safari/534.11+";
const char S42643[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42644[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42645[] = "Opera/9.80 (Android; Opera Mini/7.28879/27.1662; U; en-us) Presto/2.8.119 Version/11.10 UCBrowser/8.6.1.262/145/355/";
const char S42646[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; MyIE2; Maxthon)";
const char S42647[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Maxthon)";
const char S42648[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Maxthon)";
const char S42649[] = "Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 5.0; Maxthon)";
const char S42650[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Maxthon)";
const char S42651[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 5.0; Maxthon)";
const char S42652[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Maxthon)";
const char S42653[] = "Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; Maxthon)";
const char S42654[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Avant Browser; Maxthon)";
const char S42655[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Maxthon; Maxthon)";
const char S42656[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Maxthon; SV1)";
const char S42657[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Avant Browser; Maxthon)";
const char S42658[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)";
const char S42659[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Avant Browser; Maxthon)";
const char S42660[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; SV1; Maxthon)";
const char S42661[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; SV1; Maxthon)";
const char S42662[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Maxthon; Avant Browser)";
const char S42663[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Maxthon; Maxthon)";
const char S42664[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Maxthon; SV1)";
const char S42665[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Maxthon; Maxthon)";
const char S42666[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)";
const char S42667[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Maxthon; Maxthon)";
const char S42668[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; QQBrowser/7.3.11251.400)";
const char S42669[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; QQBrowser/7.3.11251.400)";
const char S42670[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QQBrowser/7.3.11251.400)";
const char S42671[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; QQBrowser/7.3.11251.400)";
const char S42672[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST26i Build/11.2.A.0.21) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42673[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST26a Build/11.2.A.0.21) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42674[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko)";
const char S42675[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en_us) AppleWebKit/525.18.1 (KHTML, like Gecko)";
const char S42676[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B117";
const char S42677[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B118";
const char S42678[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B118 Safari/532.9";
const char S42679[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBSF/2.0]";
const char S42680[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B118 iPod4,1 BingWeb/3.03.1428.X";
const char S42681[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B117 Safari/532.9";
const char S42682[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B117 iPod3,1 BingWeb/3.03.1428.X";
const char S42683[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 41 like Mac OS X; en-US) AppleWebKit/532.9 (KHTML like Gecko) Mobile/8B118";
const char S42684[] = "Mozilla/5.0 (Linux; U; Android v1.02.09-2010.11.30_English; en-us; MID Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42685[] = "Mozilla/5.0 (Linux; U; Android Eclair; en-us; MID 1 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42686[] = "Mozilla/5.0 (Linux; U; Android 2.0.5; ru-ru Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42687[] = "Mozilla/5.0 (Linux; U; Android v1.02_15.14-MC_US@20110514; en-us; MID Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42688[] = "Mozilla/5.0 (Linux; U; Android v1.02_15.14-MC_US@20110503; en-us; MID Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42689[] = "Mozilla/5.0 (Linux; U; Android Eclair; en- Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42690[] = "Mozilla/5.0 (Linux; U; Android Eclair; md-us Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42691[] = "Mozilla/5.0 (Linux; U; Android Eclair; en-us Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42692[] = "Mozilla/5.0 (Linux; U; Android S20_09_30_B_PD_INX7E_ENG_6410POP; en-us Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42693[] = "Mozilla/5.0 (Linux; U; Android S20_09_30_B_PD_INX7E_ENG_6410POP; md-us Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42694[] = "Mozilla/5.0 (Linux; U; Android Eclair; en-ca Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42695[] = "Mozilla/5.0 (Linux; U; Android v1.02_14.13-MC_EN-2011.01.13; en-us; MID Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42696[] = "Mozilla/5.0 (Linux; U; Android Eclair; en-us;  Build/v1.2.1-17197fd) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42697[] = "Mozilla/5.0 (Linux; U; Android Eclair; nl-nl Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42698[] = "Mozilla/5.0 (Linux; U; Android SL20_X_B_PD_INX7E_ENG_6410POP; nl-nl Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42699[] = "Mozilla/5.0 (Linux; U; Android Eclair; es-us Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42700[] = "Mozilla/5.0 (Linux; U; Android Eclair; es-es Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42701[] = "Mozilla/5.0 (Linux; U; Android Eclair; en-us;  Build/RC18-e904b9c) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42702[] = "Mozilla/5.0 (Linux; U; Android Eclair; ar-eg Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42703[] = "Mozilla/5.0 (Linux; U; Android Eclair; fr-fr Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42704[] = "Mozilla/5.0 (Linux; U; Android Eclair; ru-ru Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42705[] = "Mozilla/5.0 (Linux; U; Android Eclair; en-gb Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42706[] = "Mozilla/5.0 (Linux; U; Android Eclair; en-au Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42707[] = "CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android Eclair)";
const char S42708[] = "Mozilla/5.0 (Linux; U; Android 2.0.6_b1; en-us Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42709[] = "Mozilla/5.0 (Linux; U; Android 2.0.6_b1; ru-ru Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42710[] = "Mozilla/5.0 (compatible; Konqueror/3.5; Linux) KHTML/3.5.5 (like Gecko) (Exabot-Thumbnails)";
const char S42711[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MS-RTC LM 8; InfoPath.3; .NET4.0C; .NET4.0E) chromeframe/8.0.552.224";
const char S42712[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10 chromeframe/8.0.552.224";
const char S42713[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; Tablet PC 2.0) chromeframe/8.0.552.224";
const char S42714[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0) chromeframe/8.0.552.224";
const char S42715[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0) chromeframe/8.0.552.224";
const char S42716[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) chromeframe/8.0.552.237";
const char S42717[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; .NET4.0E) chromeframe/8.0.552.237";
const char S42718[] = "'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media enter PC 6.0; MS-RTC LM 8; InfoPath.3; .NET4.0C; .NET4.0E) chromeframe/8.0.552.224";
const char S42719[] = "stagefright/1.2 (Linux;Android 4.2)";
const char S42720[] = "stagefright/1.2 (Linux;Android 4.2.1)";
const char S42721[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; toroplus Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42722[] = "stagefright/1.2 (Linux;Android 4.2.2)";
const char S42723[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Android SDK built for x86 Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42724[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; iphone 5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42725[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; en-us; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42726[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42727[] = "Mozilla/5.0 (Linux; U; Android 4.2.2.2.2.2.2.2.2.2; en-us; Build/OPENMASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42728[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; Aveo NOTE II+ Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42729[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42730[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42731[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; PAD-2 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42732[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; e2001_v89_gq2008s Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42733[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; e2001_v89_jdt1qhd Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42734[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; e2001_v89_jbl1a698 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42735[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; v89_jbl1a698 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42736[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; en-us; iPhone Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42737[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; vi-vn; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42738[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; en-us; iPhone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42739[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; e2001_v89_gq2002qhd Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42740[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; en-us; iPhone5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42741[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Android SDK built for x86 Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42742[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; e2001_v89_jbl1a668_fwvga Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42743[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; PIXELDROID HD2 Build/PIXEL_4_2_2_JELLY_BEAN) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42744[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; e2001_v89_gq2002sc Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42745[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; vi-vn; VDM HD4 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42746[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; es-es; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42747[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; e2001v21_v89_jbl1a698_2g Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42748[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; iPhone5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42749[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; tr-tr; iPhone Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42750[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; v89_jbl1a698 Build/JOP40D)";
const char S42751[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; unknown Build/JDQ39)";
const char S42752[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; ru-ru; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42753[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42754[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42755[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; e2001v21_v89_zlh_hd Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42756[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; unknown Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42757[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42758[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42759[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Android Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42760[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MID Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42761[] = "Pioneer E71t/V100 Linux/3.4.5 Android/4.2.1  Release/12.24.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2 System/Android 4.2.1;";
const char S42762[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; th-th; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42763[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; e2001_v89_zlh_hd Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42764[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; TWD_MID Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42765[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; iphone Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42766[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; en-us; TWD_MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42767[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; X Build/HMX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42768[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; Android Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42769[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; tr-tr; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42770[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Pioneer S90w Build/PioneerS90w) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42771[] = "Mozilla/5.0 (Linux; U; Android 4.2.0; vi-vn; iPhone Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42772[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; MotoMB511 Build/RUTEM_U3_01.19.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42773[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; MotoMB511 Build/RUTEM_U3_01.14.20) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42774[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB511 Build/RUTEM_U3_01.14.16) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42775[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; MotoMB511 Build/RUTEM_U3_01.17.12) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42776[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; MotoMB511 Build/RUTEM_U3_01.17.4) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42777[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; MB511 Build/RUTEM_U3_01.19.4) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42778[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB511 Build/RUTEM_U3_01.10.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42779[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB511 Build/RUTEM_U3_01.14.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42780[] = "MOT-MB511/Blur_Version.0.60.101.MB511.Brasil.en.BR Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; MB511 Build/RUTLA_U3_00.60.101) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42781[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; MotoMB511 Build/RUTEM_U3_01.19.8) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42782[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB511 Build/Blur_Version.1.27.12.MB511.ATT.en.US Flex/P044) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42783[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB511 Build/RUTEM_U3_01.14.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42784[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MotoMB511 Build/RUTEM_U3_01.17.10) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42785[] = "MOT-MB511/Blur_Version.0.53.0.MB511.Movistar.en.01 Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; MB511 Build/RUTLA_U3_00.53.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42786[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-za; MB511 Build/RUTEM_U3_01.08.4) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42787[] = "MOT-MB511/Blur_Version.0.49.102.MB511.AmericaMovil.en.MX Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; MB511 Build/RUTLA_U3_00.49.101_R02) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42788[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; MB511 Build/RURGS_U3_1.81.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42789[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; MB511 Build/RUTEM_U3_01.12.4) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42790[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB511 Build/RUTEM_U3_01.13.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42791[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MotoMB511 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42792[] = "MOT-MB511/Blur_Version.0.45.204.MB511.Vivo.en.BR Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; MB511 Build/RUTLA_U3_00.45.204) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42793[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-nz; MB511 Build/RUTEM_U3_01.12.8) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42794[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MotoMB511 Build/RUTEM_U3_01.08.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42795[] = "MOT-MB511/Blur_Version.0.45.203.MB511.TIM.en.BR Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; MB511 Build/RUTLA_U3_00.45.203) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42796[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; MB511 Build/Blur_Version.3.1.810.MB511.Rogers.en.CA Flex/P014) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42797[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB511 Build/Blur_Version.1.28.4.MB511.ATT.en.US Flex/P044) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42798[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB511 Build/RUTH_U3_00.28.4) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42799[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; MB511 Build/RUTEM_U3_01.14.20) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42800[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB511 Build/Blur_Version.0.19.41.MB511.ATT.en.US Flex/P025) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42801[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB511 Build/RUTEM_U3_01.14.10) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42802[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB511 Build/ERE27) AppleWebKit/525.10 +(KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S42803[] = "MOT-MB511/Blur_Version.0.60.0.MB511.Latam.en.01 Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB511 Build/RUTLA_U3_00.60.0_R01) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42804[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P6210 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42805[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42806[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-P6210 Build/IML74K)";
const char S42807[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P6210 Build/IMM76D)";
const char S42808[] = "Mozilla/5.0 (Linux; U; Android 4.0.3 LeoROM; zh-tw; GT-P6210 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42809[] = "Logos/1.0.3 (Linux; U; Android 4.0.3; en-US; GT-P6210 Build/X.X)";
const char S42810[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; GT-P6210 Build/RKS7S01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42811[] = "Logos/1.1.3 (Linux; U; Android 4.0.3; es-US; GT-P6210 Build/Novo 7 Essential v0.6 X [D80])";
const char S42812[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6210 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S42813[] = "Mozilla/5.0 (Android 4.0.4; en-us; GT-P6210 Build/IMM76D) FreeWheelAdManager/5.0.0-r8951-X;com.gotv.crackle.handset/X";
const char S42814[] = "Logos/2.1.2 (Linux; U; Android 4.0.4; en-US; GT-P6210 Build/IMM76D.UELPC)";
const char S42815[] = "Logos/2.1.2 (Linux; U; Android 4.0.4; ru-RU; GT-P6210 Build/IMM76D.XXLPL)";
const char S42816[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S42817[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P6210 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S42818[] = "Logos/2.2.3 (Linux; U; Android 4.0.4; en-GB; GT-P6210 Build/IMM76D.XXLPN)";
const char S42819[] = "Android/4.6.1 (Linux; U; Android 4.0.4; en_US; GT-P6210)";
const char S42820[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GT-P6210 Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42821[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6210 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S42822[] = "com.google.android.apps.plus/X (Linux; U; Android 4.0.4; en_US; GT-P6210; Build/IMM76D) (gzip)";
const char S42823[] = "com.google.android.youtube/3.5.5(Linux; U; Android 4.0.4; en_US; GT-P6210 Build/IMM76D) gzip";
const char S42824[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P6210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42825[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; U20i Build/1.1.A.0.8) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S42826[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; U20i Build/1.1.A.0.12) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S42827[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; SonyEricssonU20iv Build/1.2.A.1.174) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S42828[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; U20i Build/1.2.A.1.174) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S42829[] = "Mozilla/5.0 (Linux; U; Android 1.6; es-es; U20i Build/1.1.A.0.13) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S42830[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; SonyEricssonU20i Build/1.2.A.1.174) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S42831[] = "Mozilla/5.0 (compatible; Konqueror/3.5; Linux) KHTML/3.5.5 (like Gecko) (Debian)";
const char S42832[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT30p Build/7.0.A.3.195) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42833[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42834[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L300 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42835[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SPH-L300 Build/JZO54K)";
const char S42836[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8J2 Safari/6533.18.5";
const char S42837[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8G4 Safari/6533.18.5";
const char S42838[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8L1 Safari/6533.18.5";
const char S42839[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8F190 Safari/6533.18.5";
const char S42840[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8H7 Safari/6533.18.5";
const char S42841[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8K2 Safari/6533.18.5";
const char S42842[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 431 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) CriOS/23.0.1271.96 Mobile/8G4 Safari/6533.18.5";
const char S42843[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-; LG-GT540 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S42844[] = "Mozilla/5.0 (Linux; U; Android 2.1 Black Edition by Quarx; ru-ru; GT540 Build/ERE27) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S42845[] = "LG-GT540f/v20a Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; GT540f Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 MMS/LG-Android-MMS-V1.0/1.2";
const char S42846[] = "Mozilla/5.0 (Linux; U; Android 2.1 Black Edition by Quarx; en-us; GT540 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42847[] = "Mozilla/5.0 (Linux; U; Android 2.1 FY.FA668; vi-vn; GT540 Build/ERE27) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S42848[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-ca; LG-GT540f Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S42849[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT540 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42850[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; pl-pl; GT540 Build/ERE27) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S42851[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; LG-GT540/V20a Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 MMS/LG-Android-MMS-V1.0/1.2";
const char S42852[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; et-ee; GT540 Build/ERE27) AppleWebKit/528.5+ (KHTML, nagu Gecko) Versioon/3.1.2 Mobile Safari/525.20,1";
const char S42853[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; LG-GT540 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S42854[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; GT540 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42855[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT540 Build/ERE27)";
const char S42856[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT540 Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S42857[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-mx; GT540 Build/ERE27) AppleWebKit/528.5+ (KHTML, like Gecko) Versi?n/3.1.2 Mobile Safari/525.20.1";
const char S42858[] = "Mozilla/5.0 (Series40; Nokia302/14.26; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S42859[] = "Mozilla/5.0 (Series40; Nokia302/14.26; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5";
const char S42860[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S42861[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S42862[] = "Mozilla/5.0 (X11; U; Linux x86_64; fr-FR) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.514.0 Safari/534.7";
const char S42863[] = "Mozilla/5.0 (Linux; en-US) AppleWebKit/534.6 (KHTML, like Gecko) Chrome/7.0.500.0 Safari/534.6";
const char S42864[] = "Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US) AppleWebKit/534.8 (KHTML, like Gecko) Chrome/7.0.521.0 Safari/534.8";
const char S42865[] = "Mozilla/5.0 (X11; U; Linux i686; ru-RU) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S42866[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1.15) Gecko/X Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/7.0.540.0 Safari/534.10";
const char S42867[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-VM701 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42868[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-VM701 Build/GRJ22)";
const char S42869[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-VM701 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42870[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en; rv:1.9.2.28) Gecko/X Camino/2.1.2 (like Firefox/3.6.28)";
const char S42871[] = "SAMSUNG-GT-S3650C/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S42872[] = "SAMSUNG-GT-S3650W/S3650WXEJA2 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S42873[] = "SAMSUNG-GT-S3650/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S42874[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.53) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42875[] = "Mozilla/5.0(Linux;U;Android2.1-update1;Zh_cn;E16i;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S42876[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; SonyEricssonE16i Build/2.1.1.A.0.53) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42877[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.46) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S42878[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.53) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S42879[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; E16i Build/2.1.1.A.0.53)";
const char S42880[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.53) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S42881[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.46) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S42882[] = "Mozilla/5.0 (X11; U; Linux MIPS32 1074Kf CPS QuadCore; en-US; rv:1.9.2.13) Gecko/X Fedora/3.6.13-1.fc14 Firefox/3.6.13";
const char S42883[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PantechP9070 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42884[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; PantechP9070 Build/IMM76I)";
const char S42885[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:8.0.1) Gecko/X Firefox/8.0.1";
const char S42886[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-H Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S42887[] = "Mozilla/5.0 (Linux; Android 4.0.4; SmartTabII7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S42888[] = "Mozilla/5.0 (Linux; Android 4.0.4; A2107A-H Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S42889[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-F Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S42890[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.4) Gecko/X Firefox/15.4.1 PaleMoon/15.4.1";
const char S42891[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:15.4) Gecko/X Firefox/15.4.1-x64 PaleMoon/15.4.1-x64";
const char S42892[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I727 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42893[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I727 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42894[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I727 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42895[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I727 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42896[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I727 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42897[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42898[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K)";
const char S42899[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I727 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42900[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42901[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SoftwinerEvb Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42902[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SoftwinerEvb Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42903[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SoftwinerEvbV13 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42904[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SoftwinerEvb Build/IML74K)";
const char S42905[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SoftwinerEvb Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42906[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SoftwinerEvb Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42907[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SoftwinerEvb Build/IMM76D)";
const char S42908[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Softwinerf721 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42909[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Softwinerf900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42910[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Softwinerf761 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42911[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SoftwinerfX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42912[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Softwinerf761I Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42913[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SoftwinerEvbV13 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42914[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Softwinerkf026 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42915[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Softwinerf761 Build/IMM76D)";
const char S42916[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; Softwinerf900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S42917[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Softwinerf761K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42918[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Softwinerf900 Build/IMM76D)";
const char S42919[] = "Opera/9.80 (BREW; Opera Mini/6.0.0/28.3392; U; id) Presto/2.8.119 Version/11.10";
const char S42920[] = "Opera/9.80 (BREW; Opera Mini/6.0.0/27.2338; U; en) Presto/2.8.119 240X320 Casio C781";
const char S42921[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; VS840 4G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42922[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; VS840 4G Build/GRK39F)";
const char S42923[] = "Mozilla/5.0 (Linux; Android 4.0.3;  HTC Sensation Z710e Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S42924[] = "Mozilla/5.0 (Linux; Android 4.0.3;  HTC Sensation 4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S42925[] = "Mozilla/5.0 (Linux; Android 4.0.3;  HTC Sensation 4G with Beats Audio Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S42926[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710e Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42927[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42928[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G with Beats Audio Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42929[] = "Mozilla/5.0 (Linux; Android 4.0.3; Sensation Z710e Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42930[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G XE with Beats Audio Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42931[] = "Mozilla/5.0 (Linux; Android 4.0.3; Sensation 4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42932[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Z710e Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42933[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation 4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42934[] = "Mozilla/5.0 (Linux; Android 4.0.4; Sensation 4G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42935[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I415 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42936[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-I415 Build/JZO54K)";
const char S42937[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Chaser Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S42938[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Chaser Build/GRJ90)";
const char S42939[] = "Safari/8536.29.13 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro10%2C1)";
const char S42940[] = "Safari/8536.29.13 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (iMac12%2C2)";
const char S42941[] = "Safari/8536.29.13 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBook5%2C1)";
const char S42942[] = "Safari/8536.29.13 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (Macmini3%2C1)";
const char S42943[] = "Safari/8536.29.13 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookAir5%2C2)";
const char S42944[] = "Safari/8536.29.13 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacPro3%2C1)";
const char S42945[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Avant Browser)";
const char S42946[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Avant Browser)";
const char S42947[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Avant Browser; SV1)";
const char S42948[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Avant Browser; Avant Browser)";
const char S42949[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Avant Browser; Avant Browser)";
const char S42950[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; SV1; Avant Browser)";
const char S42951[] = "Mozilla/5.0 (Symbian/3; Series60/5.3 NokiaE7-00; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.4.2.6 Mobile Safari/533.4 3gpp-gba";
const char S42952[] = "Mozilla/5.0 (Symbian/3; Series60/5.3 NokiaE7-00/111.040.1511; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.1.4 Mobile Safari/535.1 3gpp-gba";
const char S42953[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S42954[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100T Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S42955[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S42956[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S42957[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100T Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S42958[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S42959[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S42960[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S42961[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100M Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S42962[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:22.0) Gecko/X Firefox/22.0";
const char S42963[] = "Mozilla/5.0 (Windows NT 6.0; rv:22.0) Gecko/X Firefox/22.0 AlexaToolbar/alxf-2.18";
const char S42964[] = "Mozilla/5.0 (Windows NT 6.0; rv:22.0) Gecko/X Firefox/22.0 IceDragon/22.0.0.1";
const char S42965[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S6810P Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42966[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-S6810P/S6810PXXAMC3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42967[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S6810P Build/JZO54K)";
const char S42968[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-S6810P/S6810PXXAMC4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42969[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; SAMSUNG GT-S6810P/S6810PXXAMD3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42970[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; SAMSUNG GT-S6810P/S6810PXXAME3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42971[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; sk-sk; GT-S6810P-ORANGE/S6810PXXAMC5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S42972[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/534.15 (KHTML, like Gecko) Chrome/10.0.612.1 Safari/534.15";
const char S42973[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.64 Safari/537.31";
const char S42974[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0; Avant TriCore) Gecko/X Firefox/15.0.1";
const char S42975[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:16.0; Avant TriCore) Gecko/X Firefox/16.0";
const char S42976[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:17.0; Avant TriCore) Gecko/X Firefox/17.0";
const char S42977[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:18.0; Avant TriCore) Gecko/X Firefox/18.0";
const char S42978[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:19.0; Avant TriCore) Gecko/X Firefox/19.0";
const char S42979[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:20.0; Avant TriCore) Gecko/X Firefox/20.0";
const char S42980[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:21.0; Avant TriCore) Gecko/X Firefox/21.0";
const char S42981[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:22.0; Avant TriCore) Gecko/X Firefox/22.0";
const char S42982[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:23.0; Avant TriCore) Gecko/X Firefox/23.0";
const char S42983[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:6.0.2) Gecko/X Firefox/6.0.2";
const char S42984[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42985[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC VLE_U Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42986[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42987[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Special Edition Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S42988[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.1.141) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S42989[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.1.96) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S42990[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.2 Mobile/10A523 Safari/8536.25";
const char S42991[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.2 Mobile/10A403 Safari/8536.25";
const char S42992[] = "Mozilla/5.0 (Linux; Android 4.1.1; Full AOSP on Toro-VZW Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42993[] = "Mozilla/5.0 (Linux; Android 4.1.1; Dark Jelly Belly Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42994[] = "Mozilla/5.0 (Linux; Android 4.1.1; LT18 Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42995[] = "Mozilla/5.0 (Linux; Android 4.1.1; Toasted Marshmallow Build/Toasted Marshmallow) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42996[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC ENRC2B Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42997[] = "Mozilla/5.0 (Linux; Android 4.1.2; ThinkPadTablet Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S42998[] = "Mozilla/5.0 (Linux; Android 4.1; 9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S42999[] = "Mozilla/5.0 (Linux; Android 4.1.1; Tablet-PC-4 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43000[] = "Mozilla/5.0 (Linux; Android 4.1.2; Full AOSP on Maguro Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43001[] = "Mozilla/5.0 (Linux; Android 4.1.2; tervigon Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43002[] = "Mozilla/5.0 (Linux; Android 4.1.1; imito am802 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43003[] = "Mozilla/5.0 (Linux; Android 4.1.2; Notion Ink ADAM Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43004[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-P600 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43005[] = "Mozilla/5.0 (Linux; Android 4.1.1; CTP980 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43006[] = "Mozilla/5.0 (Linux; Android 4.1.1; AllviewCity Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43007[] = "Mozilla/5.0 (Linux; Android 4.1.1; iM970 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43008[] = "Mozilla/5.0 (Linux; Android 4.1.2; mfld_pr2 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43009[] = "Mozilla/5.0 (Linux; Android 4.1.1; AMOI N820 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43010[] = "Mozilla/5.0 (Linux; Android 4.1.5; TREQ_A10_Pocket2 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43011[] = "Mozilla/5.0 (Linux; Android 4.1.1; TAB R105BK Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43012[] = "Mozilla/5.0 (Linux; Android 4.1.1; ACGM727KC Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43013[] = "Mozilla/5.0 (Linux; Android 4.1.4; Note3 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43014[] = "Mozilla/5.0 (Linux; Android 4.1.1; ST80208-1 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43015[] = "Mozilla/5.0 (Linux; Android 4.1.1; G1 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43016[] = "Mozilla/5.0 (Linux; Android 4.1.1; CRXT1075 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43017[] = "Mozilla/5.0 (Linux; Android 4.1.1; ST80216-1 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43018[] = "Mozilla/5.0 (Linux; Android 4.1.1; W27(Dualcore) Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43019[] = "Mozilla/5.0 (Linux; Android 4.1.1; MaxiIQ Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43020[] = "Mozilla/5.0 (Linux; Android 4.1.1; DIN09.97 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43021[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC DLX_WLJ Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43022[] = "Mozilla/5.0 (Linux; Android 4.1.1; Cat Tablet PHOENIX 8.1J0 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43023[] = "Mozilla/5.0 (Linux; Android 4.1.1; TBD1083B Build/X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43024[] = "Mozilla/5.0 (Linux; Android 4.1.1; TBDB763 Build/X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43025[] = "Mozilla/5.0 (Linux; Android 4.1.2; Thrive Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43026[] = "Mozilla/5.0 (Linux; Android 4.1.1; N80HH Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43027[] = "Mozilla/5.0 (Linux; Android 4.1.1; CTP988 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43028[] = "Mozilla/5.0 (Linux; Android 4.1.1; Pad 7208HD Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43029[] = "Mozilla/5.0 (Linux; Android 4.1.1; ICOU10 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43030[] = "Mozilla/5.0 (Linux; Android 4.1.1; ACGM816KC Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43031[] = "Mozilla/5.0 (Linux; Android 4.1.1; TBDB863 Build/X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43032[] = "Mozilla/5.0 (Linux; Android 4.1.1; YOUPU U6 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43033[] = "Mozilla/5.0 (Linux; Android 4.1.1; EPAD Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43034[] = "Mozilla/5.0 (Linux; Android 4.1.1; GEM10312G Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43035[] = "Mozilla/5.0 (Linux; Android 4.1.1; N101 DUAL CORE2 V11 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43036[] = "Mozilla/5.0 (Linux; Android 4.1.1; I9420 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43037[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nivo Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43038[] = "Mozilla/5.0 (Linux; Android 4.1.1; CMP748 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43039[] = "Mozilla/5.0 (Linux; Android 4.1.1; ACGM727KC_N Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43040[] = "Mozilla/5.0 (Linux; Android 4.1.1; Infinity POP Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43041[] = "Mozilla/5.0 (Linux; Android 4.1.1; Allwinner A10 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43042[] = "Mozilla/5.0 (Linux; Android 4.1.2; OT-995 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43043[] = "Mozilla/5.0 (Linux; Android 4.1.2; X7 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43044[] = "Mozilla/5.0 (Linux; Android 4.1.1; I9277 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43045[] = "Mozilla/5.0 (Linux; Android 4.1.2; 3dp71g Build/10.2.C.0.83) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43046[] = "Mozilla/5.0 (Linux; Android 4.1.2; B92M Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43047[] = "Mozilla/5.0 (Linux; Android 4.1.2; B94M Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43048[] = "Mozilla/5.0 (Linux; Android 4.1.1; Z930 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43049[] = "Mozilla/5.0 (Linux; Android 4.1.1; EN-TAB309 Build/IML74K..X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43050[] = "Mozilla/5.0 (Linux; Android 4.1.1; DINO7.17 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43051[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAD701 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43052[] = "Mozilla/5.0 (Linux; Android 4.1.2; S4701 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43053[] = "Mozilla/5.0 (Linux; Android 4.1.2; ARIA Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43054[] = "Mozilla/5.0 (Linux; Android 4.1.1; Bouygues Telecom Bs 351 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43055[] = "Mozilla/5.0 (Linux; Android 4.1.2; I110 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43056[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT400 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43057[] = "Mozilla/5.0 (Linux; Android 4.1.2; S9 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43058[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E455g Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43059[] = "Mozilla/5.0 (Linux; Android 4.1.1; Amazing A4 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43060[] = "Mozilla/5.0 (Linux; Android 4.1.2; A600 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43061[] = "Mozilla/5.0 (Linux; Android 4.1.1; Quickfone N2 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43062[] = "Mozilla/5.0 (Linux; Android 4.1.2; Z1 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43063[] = "Mozilla/5.0 (Linux; Android 4.1.2; CyrusApel Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43064[] = "Mozilla/5.0 (Linux; Android 4.1.1; xTAB-70dci Build/w28ref.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43065[] = "Mozilla/5.0 (Linux; Android 4.1.1; ITQ701 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43066[] = "Mozilla/5.0 (Linux; Android 4.1.2; AQ710A Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43067[] = "Mozilla/5.0 (Linux; Android 4.1.1; KLYAN2 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43068[] = "Mozilla/5.0 (Linux; Android 4.1.1; SP401 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43069[] = "Mozilla/5.0 (Linux; Android 4.1.1; TP7-1000DC Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43070[] = "Mozilla/5.0 (Linux; Android 4.1.1; Z993 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43071[] = "Mozilla/5.0 (Linux; Android 4.1.1; ZTE Kis Pro Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43072[] = "Mozilla/5.0 (Linux; Android 4.1.1; Positivo Ypy L700 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43073[] = "Mozilla/5.0 (Linux; Android 4.1.1; TPC-840M Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43074[] = "Mozilla/5.0 (Linux; Android 4.1.1; TBD1083 Build/X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43075[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGP351 Build/10.1.1.A.1.307) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43076[] = "Mozilla/5.0 (Linux; Android 4.1.2; CRXT 1125 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43077[] = "Mozilla/5.0 (Linux; Android 4.1.2; NaviPad TM-7045 3G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43078[] = "Mozilla/5.0 (Linux; Android 4.1.1; TBDG773 Build/Zeki.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43079[] = "Mozilla/5.0 (Linux; Android 4.1.2; MBX reference board (g02ref) Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43080[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A850 S/K Build/VEGAVIET.COM JELLYBEAN V1.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43081[] = "Mozilla/5.0 (Linux; Android 4.1.2; i-mobile i-STYLE 7A Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43082[] = "Mozilla/5.0 (Linux; Android 4.1.1; MID-7089D Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43083[] = "Mozilla/5.0 (Linux; Android 4.1.1; CRCB2094 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43084[] = "Mozilla/5.0 (Linux; Android 4.1; UM-A13B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43085[] = "Mozilla/5.0 (Linux; Android 4.1.2; one touch D920 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43086[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P712 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43087[] = "Mozilla/5.0 (Linux; Android 4.1.1; smart_a17 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43088[] = "Mozilla/5.0 (Linux; Android 4.1.2; Casper VIA A3216 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43089[] = "Mozilla/5.0 (Linux; Android 4.1.2; M35c Build/12.0.B.3.26) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43090[] = "Mozilla/5.0 (Linux; Android 4.1.1; HALOVALUE Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43091[] = "Mozilla/5.0 (Linux; Android 4.1.2; Perfeo7777HD Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43092[] = "Mozilla/5.0 (Linux; Android 4.1.1; ACGM8001KD Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43093[] = "Mozilla/5.0 (Linux; Android 4.1.1; Positivo Ypy L1050E Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43094[] = "Mozilla/5.0 (Linux; Android 4.1.1;  Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S43095[] = "Mozilla/5.0 (Linux; Android 4.1.1; Quickfone X2 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43096[] = "Mozilla/5.0 (Linux; Android 4.1.1; TP-800 Build/ICS.TP-800.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43097[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZTE V956 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43098[] = "Mozilla/5.0 (Linux; Android 4.1.2; INHON PAPILIO G2 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43099[] = "Mozilla/5.0 (Linux; Android 4.1.1; TAQ-10042 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43100[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E975w Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43101[] = "Mozilla/5.0 (Linux; Android 4.1.1; DINO762 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43102[] = "SAMSUNG-GT-C3322/C3322DDKK1 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S43103[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; C6602 Build/TEST.YUGA.XSSM.93)";
const char S43104[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C6602 Build/10.1.A.1.290) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43105[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SonyC6602 Build/10.1.A.1.113) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43106[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C6602 Build/10.1.A.1.182) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43107[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-02E Build/10.1.D.0.317) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43108[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-02E Build/10.1.D.0.317) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43109[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-02E Build/10.1.D.0.322) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43110[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-02E Build/10.1.D.0.322) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43111[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-02E Build/10.1.D.0.333) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43112[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C6602 Build/10.1.A.1.350) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43113[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; C6602 Build/10.1.A.1.434)";
const char S43114[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-02E Build/10.1.D.0.333) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43115[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C6602 Build/10.1.A.1.434) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43116[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SonyL36h Build/10.1.A.1.350) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43117[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-02E Build/10.1.D.0.343) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43118[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-02E Build/10.1.D.0.343) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43119[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SO-02E Build/10.1.D.0.343)";
const char S43120[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C6602 Build/10.1.1.A.1.253) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43121[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SonyL36h Build/10.1.1.A.1.253) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43122[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; C6602 Build/10.1.1.A.1.307) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43123[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100P Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43124[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9100P Build/JZO54K)";
const char S43125[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100P-ORANGE/I9100PXXLSQ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43126[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100P-ORANGE/I9100PXXLSS Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43127[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; SAMSUNG GT-I9100P/I9100PXXLSQ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43128[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; SAMSUNG GT-I9100P/I9100PXXLSU Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43129[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; ALCATEL ONE TOUCH 903 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43130[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-nz; ALCATEL_ONE_TOUCH_903 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43131[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ALCATEL ONE TOUCH 903D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43132[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ALCATEL ONE TOUCH 903A Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43133[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0) chromeframe/10.0.648.205";
const char S43134[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0) chromeframe/10.0.648.204";
const char S43135[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/10.0.648.204; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8)";
const char S43136[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16 chromeframe/10.0.648.204";
const char S43137[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/10.0.648.205; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)";
const char S43138[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/10.0.648.151; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2; .NET4.0C)";
const char S43139[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/10.0.648.204; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)";
const char S43140[] = "Opera/9.80 (Bada; Opera Mini/6.5/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S43141[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2";
const char S43142[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2";
const char S43143[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2";
const char S43144[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2";
const char S43145[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2";
const char S43146[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2";
const char S43147[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; LGC660 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S43148[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LG-C660 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43149[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LGC660 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S43150[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-sa; LG-C660h Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S43151[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; LGC660r/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S43152[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; LGC660h Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S43153[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; LG-C660R Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43154[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; LGC660h/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S43155[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; nl-; LGC660 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S43156[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-C660h Build/GRJ22)";
const char S43157[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-C660 Build/GRJ22)";
const char S43158[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-C660R Build/GRI40)";
const char S43159[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; nl-; LG-C660 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43160[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; LG-C660h Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Versao/4.0 Mobile Safari/533.1";
const char S43161[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; GS02 Build/HuaweiU8860) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43162[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U8860 Build/HuaweiU8860) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43163[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; U8860 Build/HuaweiU8860)";
const char S43164[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ja-jp; GS02 Build/HuaweiU8860) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43165[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; U8860 Build/HuaweiU8860) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S43166[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; HUAWEI_U8860 Build/HuaweiU8860) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43167[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; U8860-51 Build/HuaweiU8860) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43168[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; HUAWEI_U8860 Build/HuaweiU8860) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43169[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; U8860 Build/HuaweiU8860) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43170[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; U8860 Build/HuaweiU8860) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S43171[] = "SonyEricssonU5a/R2EA; Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525";
const char S43172[] = "SonyEricssonU5at/R2FA; Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525";
const char S43173[] = "SonyEricssonU5iv/R2EA; Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525";
const char S43174[] = "SonyEricssonU5i/R2EA; Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525";
const char S43175[] = "SuperBot/4.7.0.74 (Windows 7)";
const char S43176[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.6.0_25; Europe/de) http://yacy.net/bot.html";
const char S43177[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.6.0_29; Europe/en) http://yacy.net/bot.html";
const char S43178[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0; America/en) http://yacy.net/bot.html";
const char S43179[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_02; Europe/de) http://yacy.net/bot.html";
const char S43180[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0; Europe/de) http://yacy.net/bot.html";
const char S43181[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.6.0_31; America/en) http://yacy.net/bot.html";
const char S43182[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko; WEBTRENDSPAGEINFOBOT) Version/5.0.3 Safari/533.19.4";
const char S43183[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.6.0_26; Europe/de) http://yacy.net/bot.html";
const char S43184[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.6.0_29; Europe/de) http://yacy.net/bot.html";
const char S43185[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.6.0_31; Europe/de) http://yacy.net/bot.html";
const char S43186[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.6.0_23; Europe/de) http://yacy.net/bot.html";
const char S43187[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_04; Europe/de) http://yacy.net/bot.html";
const char S43188[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.6.0_29; Europe/fr) http://yacy.net/bot.html";
const char S43189[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_01; Europe/de) http://yacy.net/bot.html";
const char S43190[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_07; Europe/de) http://yacy.net/bot.html";
const char S43191[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_07; Europe/fr) http://yacy.net/bot.html";
const char S43192[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_05; Europe/de) http://yacy.net/bot.html";
const char S43193[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_09; Europe/de) http://yacy.net/bot.html";
const char S43194[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_04; Europe/en) http://yacy.net/bot.html";
const char S43195[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_13; Europe/de) http://yacy.net/bot.html";
const char S43196[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_04; America/en) http://yacy.net/bot.html";
const char S43197[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; aggregator VocusBot 0.4; +http://www.vocus.com/vnhs.html)";
const char S43198[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_21; Europe/de) http://yacy.net/bot.html";
const char S43199[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_10; Europe/fr) http://yacy.net/bot.html";
const char S43200[] = "yacybot (freeworld-global; amd64 Windows 7 6.1; java 1.7.0_04; America/en) http://yacy.net/bot.html";
const char S43201[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_25; Europe/de) http://yacy.net/bot.html";
const char S43202[] = "yacybot (webportal-global; amd64 Windows 7 6.1; java 1.7.0_25; Europe/en) http://yacy.net/bot.html";
const char S43203[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.6.0_38; Europe/de) http://yacy.net/bot.html";
const char S43204[] = "yacybot (webportal-global; amd64 Windows 7 6.1; java 1.8.0-ea; Europe/en) http://yacy.net/bot.html";
const char S43205[] = "yacybot (webportal-global; x86 Windows 7 6.1; java 1.7.0_25; Europe/en) http://yacy.net/bot.html";
const char S43206[] = "yacybot (webportal-global; amd64 Windows 7 6.1; java 1.7.0_04; Europe/de) http://yacy.net/bot.html";
const char S43207[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_25; America/en) http://yacy.net/bot.html";
const char S43208[] = "yacybot (amd64 Windows 7 6.1; java 1.6.0_20; Europe/de) http://yacy.net/bot.html;c2VvLWNyYXdsZXI=";
const char S43209[] = "yacybot (x86 Windows 7 6.1; java 1.6.0_18; Europe/sv) http://yacy.net/bot.html;c2VvLWNyYXdsZXI=";
const char S43210[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_25; Europe/en) http://yacy.net/bot.html";
const char S43211[] = "yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_25; Europe/fr) http://yacy.net/bot.html";
const char S43212[] = "yacybot (amd64 Windows 7 6.1; java 1.6.0_17; Europe/de) http://yacy.net/bot.html";
const char S43213[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Nexus S Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43214[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Nexus S Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43215[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus S Build/IML69B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43216[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus S Build/IML70C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43217[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus S Build/IML74B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43218[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43219[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Nexus S Build/IML74K)";
const char S43220[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Full Android on Crespo Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43221[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43222[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Nexus S Build/IMM76B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43223[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30";
const char S43224[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; Nexus S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30";
const char S43225[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43226[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Nexus S Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S43227[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Nexus S Build/IMM76D)";
const char S43228[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Crespo Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43229[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43230[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM30D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43231[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43232[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM76M) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43233[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Nexus S Build/ICS_MR0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43234[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Crespo Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43235[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43236[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM20C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43237[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus S Build/v15) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43238[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Nexus S Build/IMM76L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S43239[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Nexus S Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S43240[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; nexus s Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S43241[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LGL86C/V100 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43242[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LGL86C Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43243[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LGL86C Build/IMM76L)";
const char S43244[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S43245[] = "Mozilla/5.0 (Linux; Android 4.1.1; EndeavorU Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S43246[] = "Mozilla/5.0 (Linux; Android 4.1.2; One X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S43247[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S43248[] = "Mozilla/5.0 (Linux; Android 4.0.4;  GT-P7510 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S43249[] = "Mozilla/5.0 (Linux; Android 4.0.3;  GT-P7510 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S43250[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P7510 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43251[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43252[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T859 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43253[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43254[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB612 Build/3.4.2-120) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43255[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB612 Build/KRNS-X4-1.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43256[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB612 Build/KRNS-X4-0.13.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43257[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB612 Build/KRNS-X4-1.1.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43258[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB612 Build/KRNS-X4-1.1.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43259[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB612 Build/KRNS-X4-1.1.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43260[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; MB612 Build/KRNS-X4-1.1.10)";
const char S43261[] = "Mozilla/5.0 (Android; Mobile; rv:10.0.4) Gecko/10.0.4 Firefox/10.0.4 Fennec/10.0.4";
const char S43262[] = "Mozilla/5.0 (Android; Mobile; rv:10.0.5) Gecko/10.0.5 Firefox/10.0.5 Fennec/10.0.5";
const char S43263[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; VS920 4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43264[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; VS920 4G Build/GRJ90)";
const char S43265[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; VS920 4G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43266[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; VS920 4G Build/GRK39F)";
const char S43267[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT901 Build/7.7.1Q-144_SLS_S240-58) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43268[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 5.1; .NET CLR 1.1.4322)";
const char S43269[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 5.1)";
const char S43270[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.3; Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; Paragon Labels))";
const char S43271[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows XP)";
const char S43272[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 5.2; .NET CLR 1.1.4322)";
const char S43273[] = "Mozilla/3.0 (compatible;MSIE 5.0;  Windows NT 5.1; QQDownload)";
const char S43274[] = "Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.1)";
const char S43275[] = "Mozilla/6.0 (compatible;MSIE 5.0;  Windows NT 5.1; QQDownload)";
const char S43276[] = "('Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 5.1; .NET CLR 1.1.4322)', 'Internet Explorer 5, Windows XP')";
const char S43277[] = "Mozilla/5.0 (compatible; MSIE 5.0; Windows NT 5.1; Trident/3.0)";
const char S43278[] = "Mozilla/5.0 (compatible; MSIE 5.0; Windows NT 5.2; Trident/1.0)";
const char S43279[] = "Mozilla/4.0 (Compatible; MSIE 5.0; Windows NT 5.1;)";
const char S43280[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT912 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43281[] = "Mozilla/5.0 (Linux; Android 4.0.3; DROID RAZR Build/6.7.1-68_DHD-15_M4-7) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43282[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43283[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43284[] = "Mozilla/5.0 (Linux; Android 4.0.4; Droid Razr Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43285[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LG730 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43286[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-LG730 Build/IMM76L)";
const char S43287[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C5303 Build/12.0.A.1.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43288[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; C5303 Build/12.0.A.1.257)";
const char S43289[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyC5303 Build/12.0.A.1.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43290[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C5303 Build/12.0.A.1.257) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43291[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyC5303 Build/12.0.A.1.257) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43292[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyC5303 Build/12.0.A.1.237) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43293[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C5303 Build/12.0.A.2.245) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43294[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyC5303 Build/12.0.A.2.245) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43295[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyC5303 Build/12.0.A.2.254) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43296[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C5303 Build/12.0.A.2.254) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43297[] = "SonyC5303 Build/12.0.A.1.257 stagefright/1.2 (Linux;Android 4.1.2)";
const char S43298[] = "Mozilla/5.0 (Symbian/3; Series60/5.3 Nokia500/111.021.0028; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.1.4 Mobile Safari/535.1 3gpp-gba";
const char S43299[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S43300[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S43301[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S43302[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100M Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S43303[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100T Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S43304[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S43305[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100T Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S43306[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9105; en-GB) AppleWebKit/534.8+ (KHTML, like Gecko) Version/6.0.0.668 Mobile Safari/534.8+";
const char S43307[] = "BlackBerry9105/6.0.0.570 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/123";
const char S43308[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; GT-B7510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43309[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; GT-B7510L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43310[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; GT-B7510B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43311[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; GT-B7510 Build/FROYO)";
const char S43312[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; GT-B7510 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S43313[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; GT-B7510L Build/FROYO)";
const char S43314[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-; GT-B7510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43315[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A850K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43316[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A850S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43317[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A850K Build/JZO54K)";
const char S43318[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A850L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43319[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A850S Build/JZO54K)";
const char S43320[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A850K/2.12 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43321[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A850L Build/JZO54K)";
const char S43322[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A850K/2.14 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43323[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A850K/2.16 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43324[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A850L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30";
const char S43325[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A850K/2.22 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43326[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A850K/A850K2.10 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30";
const char S43327[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; IM-A850K/A850K2.10 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43328[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A850S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30";
const char S43329[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A850S/L/K/2.17 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30";
const char S43330[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; IM-A850L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43331[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SPH-P100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43332[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-; SPH-P100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43333[] = "Logos/0.9.1 (Linux; U; Android 2.3.4; en_US; SPH-P100 Build/GINGERBREAD.EF17)";
const char S43334[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SPH-P100 Build/GINGERBREAD)";
const char S43335[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; XT530 Build/V4.35C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43336[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; XT530 Build/V4.480) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43337[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; XT530 Build/V4.530) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43338[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT531 Build/V4.410) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43339[] = "MOT-XT530/Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; XT530 Build/V4.460) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43340[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; XT531 Build/V4.31B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43341[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; XT531 Build/V4.54A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43342[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; XT530 Build/V4.35G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43343[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT531 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43344[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; XT531 Build/V4.510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43345[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; XT531 Build/V4.530) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43346[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; XT531 Build/V4.480) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43347[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; XT531 Build/V4.540) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43348[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; XT530 Build/V4.54C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43349[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; XT531 Build/V4.480) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S43350[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT531 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S43351[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; XT531 Build/V4.48D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43352[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT531 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S43353[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; XT530 Build/V4.530)";
const char S43354[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.62";
const char S43355[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.62";
const char S43356[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4133); ru) Presto/2.10.229 Version/11.62";
const char S43357[] = "Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.10.229 Version/11.62,gzip(gfe) (via translate.google.com)";
const char S43358[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.6 (build 03278); ru) Presto/2.10.229 Version/11.62";
const char S43359[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.62";
const char S43360[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5310); ru) Presto/2.10.229 Version/11.62";
const char S43361[] = "Opera/9.80 (Windows NT 6.0; U; pl) Presto/2.10.229 Version/11.62";
const char S43362[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.62";
const char S43363[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.62";
const char S43364[] = "Mozilla/5.0 (Windows NT 6.0; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.62";
const char S43365[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.4 (build 02647); ru) Presto/2.10.229 Version/11.62";
const char S43366[] = "Opera/9.80 (Windows NT 6.1; U; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0); en) Presto/2.10.229 Version/11.62";
const char S43367[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4876); ru) Presto/2.10.229 Version/11.62";
const char S43368[] = "Opera/11.62 (Windows NT 6.0; U; en) Presto/2.10.229 Version/11.62";
const char S43369[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S43370[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-I605 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S43371[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S43372[] = "Mozilla/5.0 (Linux; Android 4.0.3;  cm_tenderloin Build/GWK74) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S43373[] = "Mozilla/5.0 (Linux; Android 4.0.3; cm_tenderloin Build/GWK74) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43374[] = "Mozilla/5.0 (Linux; Android 4.0.4; cm_tenderloin Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43375[] = "Mozilla/5.0 (Linux; Android 4.0.4;  cm_tenderloin Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S43376[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-au; GT-I5510T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43377[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-us; GT-I5510L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43378[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I5510 Build/Arkas) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43379[] = "Mozilla/5.0 (Linux; U; Android 2.2; -au; GT-I5510T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43380[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; GT-I5510L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43381[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; GT-I5510T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43382[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I5510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43383[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I5510 Build/FROYO)";
const char S43384[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I5510 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S43385[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-I5510T Build/FROYO)";
const char S43386[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-; GT-I5510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43387[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Avant Browser)";
const char S43388[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Avant Browser)";
const char S43389[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Avant Browser; SV1)";
const char S43390[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Avant Browser; Avant Browser)";
const char S43391[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Avant Browser; Avant Browser)";
const char S43392[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; SV1; Avant Browser)";
const char S43393[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S43394[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S43395[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S43396[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7,gzip(gfe)";
const char S43397[] = "Safari/7536.26.17 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (Macmini5%2C1)";
const char S43398[] = "Safari/7536.30.1 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookAir4%2C2)";
const char S43399[] = "Safari/7536.30.1 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)";
const char S43400[] = "Safari/7536.30.1 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (iMac12%2C1)";
const char S43401[] = "Safari/7536.30.1 CFNetwork/520.5.1 Darwin/11.4.2 (i386) (MacPro1%2C1)";
const char S43402[] = "Safari/7536.30.1 CFNetwork/520.5.1 Darwin/11.4.2 (i386) (MacBook4%2C1)";
const char S43403[] = "Nokia6300/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S43404[] = "Nokia6300/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6300";
const char S43405[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A501 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S43406[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; A501 Build/IML74K)";
const char S43407[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; GT-S7710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43408[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S7710 Build/JZO54K)";
const char S43409[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-S7710L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43410[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-S7710/S7710XXAMF1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43411[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ASUS Transformer Pad TF700T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S43412[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; ASUS Transformer Pad TF700T Build/IML74K)";
const char S43413[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; da-dk; ASUS Transformer Pad TF700KL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S43414[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ASUS Pad TF700T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S43415[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/10.10 Chromium/18.0.1025.151 Chrome/18.0.1025.151 Safari/535.19";
const char S43416[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/10.10 Chromium/18.0.1025.151 Chrome/18.0.1025.151 Safari/535.19";
const char S43417[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/10.10 Chromium/18.0.1025.151 Chrome/18.0.1025.151 Safari/535.19,gzip(gfe)";
const char S43418[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.7.0 Safari/534.34";
const char S43419[] = "Mozilla/5.0 (Series40; Nokia200/11.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S43420[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:15.0) Gecko/X Firefox/15.2-x64 PaleMoon/15.2-x64";
const char S43421[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X Firefox/15.2 PaleMoon/15.2";
const char S43422[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC PH39100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43423[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_PH39100/1.59.502.2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43424[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_PH39100/1.63.502.2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43425[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_PH39100/1.07.502.0 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43426[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_PH39100/1.63.502.3 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43427[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC_PH39100/1.63.502.4 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43428[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_PH39100/1.63.502.4 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43429[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC PH39100 Build/GRJ22)";
const char S43430[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC_PH39100 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43431[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC PH39100 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43432[] = "Mozilla/5.0 (Linux; Android 4.0.4; BNTV600 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S43433[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; MZ601 Build/H.6.5-10) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43434[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; MZ601 Build/H.6.5-17) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43435[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; MZ601 Build/H.6.5-17-3) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43436[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; MZ601 Build/H.6.5-17-1)";
const char S43437[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-au; MZ601 Build/H.6.5-17-1) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43438[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ601 Build/H.6.1-17_CN-54) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43439[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S43440[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S43441[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S43442[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC VLE_U Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S43443[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S43444[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Special Edition Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S43445[] = "Mozilla/5.0 (Series40; Nokia110/03.04; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S43446[] = "Safari/8536.26.17 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (iMac12%2C2)";
const char S43447[] = "Safari/8536.26.17 CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (MacBookPro10%2C1)";
const char S43448[] = "Safari/8536.26.17 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacPro3%2C1)";
const char S43449[] = "Safari/8536.26.17 CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (MacBookAir5%2C2)";
const char S43450[] = "Safari/8536.26.17 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBook5%2C1)";
const char S43451[] = "Safari/8536.26.17 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (Macmini3%2C1)";
const char S43452[] = "Opera/9.80 (Windows NT 6.0; U; en) Presto/2.8.99 Version/11.10";
const char S43453[] = "Opera/9.80 (Windows NT 6.0; U; en) Presto/2.8.131 Version/11.11";
const char S43454[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.8.131 Version/11.10";
const char S43455[] = "Opera/11.1 (Windows NT 6.0: U: en)";
const char S43456[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.8.131 Version/11.11";
const char S43457[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.8.131 Version/11.11";
const char S43458[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.8.131 Version/11.10";
const char S43459[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.8.131 Version/11.10";
const char S43460[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.8.131 Version/11.11";
const char S43461[] = "Opera/11.1 (Windows NT 6.0; WOW64; U; en)";
const char S43462[] = "Opera/9.80 (Android; Opera Mini/7.28879/27.1662; U; en-us) Presto/2.8.119 Version/11.10 UCBrowser/8.4.1.204/145/444/";
const char S43463[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I747 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43464[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-L710 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43465[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SAMSUNG-SGH-I747 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43466[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-R530U Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43467[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SAMSUNG-SGH-I747 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43468[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-L710 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43469[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I747 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43470[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I747 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43471[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L710 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43472[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I747 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43473[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530U Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43474[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGH-I747 Build/JDQ39)";
const char S43475[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I747 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43476[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L710 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43477[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SAMSUNG-SGH-I747 Build/JDQ39E)";
const char S43478[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SPH-L710 Build/JDQ39E)";
const char S43479[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9305 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43480[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; SAMSUNG-SGH-I747 Build/JOP40D)";
const char S43481[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-CA; SPH-L710Android 4.2.2; fr-CA; SCH-R530U Build Build/JRO03L) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43482[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I747 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43483[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530U Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43484[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9305 Build/JDQ39E)";
const char S43485[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; SGH-I747 Build/JOP40D)";
const char S43486[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S43487[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100T Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S43488[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100T Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S43489[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S43490[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100M Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S43491[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S43492[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S43493[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S43494[] = "Mozilla/5.0 (Linux; Android 4.1.1; Transformer Prime TF201 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43495[] = "Mozilla/5.0 (Linux; Android 4.1.2; Transformer Prime TF201 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43496[] = "Mozilla/5.0 (Linux; Android 4.1.1;  Transformer Prime TF201 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S43497[] = "Mozilla/5.0 (Linux; Android 4.1.1; TF201 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43498[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1";
const char S43499[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1";
const char S43500[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 10 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Safari/537.36";
const char S43501[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 10 Build/JOP40F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S43502[] = "Mozilla/5.0 (Linux; Android 4.2; Nexus 10 Build/JVP15P) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S43503[] = "Opera/9.63 (X11; Linux i686; U; en) Presto/2.1.1";
const char S43504[] = "Opera/9.60 (X11; Linux i686; U; en) Presto/2.1.1";
const char S43505[] = "Opera/9.61 (X11; Linux i686; U; en) Presto/2.1.1";
const char S43506[] = "Opera/9.62 (X11; Linux i686; U; pl) Presto/2.1.1";
const char S43507[] = "Opera/9.64 (X11; Linux x86_64; U; en) Presto/2.1.1";
const char S43508[] = "Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux x86_64; en) Opera 9.60";
const char S43509[] = "Mozilla/5.0 (X11; Linux x86_64; U; en; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 9.60";
const char S43510[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Sensation XE with Beats Audio Z715e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43511[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Sensation XE with Beats Audio Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43512[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC_SensationXE_Beats_Z715e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43513[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC_SensationXE_Beats/1.0 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43514[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; enhanced HTC Sensation XE Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43515[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Sensation XE with Beats Audio Z715e Build/GRJ22)";
const char S43516[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; HTC_SensationXE_Beats_Z715a Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43517[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; HTC Sensation XE with Beats Audio Z715a Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43518[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Sensation XE with Beats Audio Z715e Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S43519[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Sensation XE with Beats Audio Build/GRJ22)";
const char S43520[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; HTC_SensationXE_Beats Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43521[] = "HTC_SensationXE_Beats_Z715e_Mozilla/5.0 (Linux; U; Android 2.3.4;it-it;Build/GRH78C) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S43522[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Sensation XE with Beats Audio Z715e Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S43523[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Z715e Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S43524[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Sensation with Beats Audio Z715e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43525[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Z715e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43526[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Z715e Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S43527[] = "Mozilla/5.0 (Series40; Nokia203/20.36; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6";
const char S43528[] = "Mozilla/5.0 (Series40; Nokia202/20.36; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6";
const char S43529[] = "Mozilla/5.0 (Series40; Nokia202/20.52; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S43530[] = "Mozilla/5.0 (Series40; Nokia203/20.52; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S43531[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; VS930 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43532[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; VS930 4G Build/JZO54K)";
const char S43533[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L720 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S43534[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-S6310 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43535[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S6310N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43536[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-S6310T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43537[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-S6310L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43538[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; el-gr; SAMSUNG GT-S6310/S6310XXAMC4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43539[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S6310 Build/JZO54K)";
const char S43540[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; SAMSUNG GT-S6310N/S6310NXXAMD1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43541[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ro-ro; SAMSUNG GT-S6310/S6310XXAMB6 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43542[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S6310N Build/JZO54K)";
const char S43543[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; GT-S6310B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43544[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-pt; SAMSUNG GT-S6310N/S6310NXXAMG1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43545[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-S6310N-ORANGE/S6310NXXAMG1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43546[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S6310L Build/JZO54K)";
const char S43547[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-S6310N-ORANGE/S6310NXXAME1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43548[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S6310T Build/JZO54K)";
const char S43549[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-S6310N-ORANGE/S6310NXXAMG5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43550[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_C525c Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43551[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C525c Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43552[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; C525c Build/IMM76I)";
const char S43553[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:19.0) Gecko/19.0 Firefox/19.0";
const char S43554[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:19.0) Gecko/19.0 Firefox/19.0";
const char S43555[] = "Mozilla/5.0 (Linux; Android 4.1.1; Xoom Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43556[] = "Mozilla/5.0 (Linux; Android 4.1.1;  Xoom Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S43557[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xoom Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43558[] = "Mozilla/5.0 (Linux; Android 4.1.2;  Xoom Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S43559[] = "Mozilla/5.0 (Linux; Android 4.1; Xoom Build/JRN82) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43560[] = "Mozilla/5.0 (Series40; Nokia303/13.47; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S43561[] = "Mozilla/5.0 (Series40; Nokia303/13.44; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5";
const char S43562[] = "MobileSafari/9537.53 CFNetwork/672.0.2 Darwin/14.0.0";
const char S43563[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGH-M919 Build/JDQ39)";
const char S43564[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-M919 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43565[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-; SGH-M919 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43566[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-M919 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43567[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:8.0.1) Gecko/X Firefox/8.0.1";
const char S43568[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:25.0) Gecko/X Firefox/25.0";
const char S43569[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:25.0) Gecko/X Firefox/25.0";
const char S43570[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ZTE-N910 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43571[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; ZTE-N910 Build/GRJ90)";
const char S43572[] = "Mozilla/5.0 (Linux; Android 4.0.4; BNTV600 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S43573[] = "BlackBerry9530/5.0.0.328 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/105";
const char S43574[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-S738C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43575[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-S738C Build/IMM76D)";
const char S43576[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S43577[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P768 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S43578[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S43579[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P769 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S43580[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P769 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S43581[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT897 Build/9.8.2Q-122_XT897_FFW-5)";
const char S43582[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43583[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT897 Build/9.8.2Q-122_XT897_FFW-7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43584[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT897 Build/9.8.2Q-122_XT897_FFW-8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43585[] = "stagefright/1.2 (Linux;Android 4.1.2;motorola XT897 Build/9.8.2Q-122_XT897_FFW-5)";
const char S43586[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HUAWEI-M860 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43587[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HUAWEI-M860 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43588[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HUAWEI-M860 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 androidAppVersion/2.1 AndroidWebkitWrapper";
const char S43589[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; M860 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43590[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; HUAWEI-M860 Build/FRG83G)";
const char S43591[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HUAWEI-M860 Build/HuaweiM860) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43592[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; HUAWEI-M860 Build/FRG83)";
const char S43593[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Huawei Ascend Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43594[] = "Mozilla/5.0 (X11; U; Linux ppc; en-GB; rv:1.9.0.12) Gecko/X Ubuntu/8.10 (intrepid) Firefox/3.0.12";
const char S43595[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S43596[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S43597[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S43598[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S43599[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S43600[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S43601[] = "SAMSUNG-GT-S3653/S365MDDJJ2 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S43602[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6425LVW Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43603[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6425LVW 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43604[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HTC_Mecha Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43605[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; ADR6425LVW Build/GRJ22)";
const char S43606[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HTC Mecha Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43607[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;ADR6425LVW 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43608[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; HTC Mecha Build/GRK39F)";
const char S43609[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Mecha Build/TEAMVICIOUS-MIUI X BETA-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43610[] = "Mozilla/5.0 (Linux; U; Android2.3.4; en-us; ADR6425LVW 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43611[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Mecha Build/TeamVicious-MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43612[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:10.0.2) Gecko/X Firefox/10.0.2";
const char S43613[] = "Mozilla/5.0 (Windows NT 6.2; rv = 10.0.1) Gecko/X Firefox/10.0.1 companydatatrees";
const char S43614[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-au; GT-P7320T Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43615[] = "Mozilla/5.0 (Linux; U; Android 3.2; pl-pl; GT-P7320 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43616[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7320 Build/MASTER)";
const char S43617[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7320T Build/MASTER)";
const char S43618[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7320/P7320XXLC4 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43619[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7320/P7320BOLD1 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43620[] = "Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7320-ORANGE/P7320BVLE3 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43621[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7320/P7320BOLC1 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43622[] = "Mozilla/5.0 (Linux; U; Android 3.2; it-it; SAMSUNG-GT-P7320/P7320XXLI2 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43623[] = "Mozilla/5.0 (Linux; U; Android 3.2; -au; GT-P7320T Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43624[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; SAMSUNG-GT-P7320/P7320BOLB2 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43625[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7320/P7320BOKK2 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43626[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-au; GT-P7320T Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S43627[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7320/P7320XXLC3 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43628[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-au; GT-P7320T Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.12;FBDM/{density=1.0,width=1280,height=752};FBLC/en_AU;FB_FW/2;FBCR/Telstra;FBPN/com.facebook.katana;FBDV/GT-P7320T;FBSV/3.2;]";
const char S43629[] = "Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7320-ORANGE/P7320BVLJ1 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43630[] = "Mozilla/5.0 (Linux; U; Android 3.2; it-it; SAMSUNG-GT-P7320/P7320XXLJ1 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43631[] = "Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7320-ORANGE/P7320BVLD2 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43632[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; SAMSUNG-GT-P7320/P7320XXLB1 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43633[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7320/P7320XXLG1 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43634[] = "Mozilla/5.0 (Linux; U; Android 3.2; da-dk; GT-P7320 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13,gzip(gfe)";
const char S43635[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; VTAB1008 Build/HTK55) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43636[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; VTAB1008 Build/HTK55)";
const char S43637[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; VTAB1008 Build/HTK55) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S43638[] = "Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaE6-00/021.014; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.28 Mobile Safari/533.4 3gpp-gba";
const char S43639[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MZ617 4G Build/7.7.1-85_MZ617-26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S43640[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MZ617 4G Build/7.7.1-85_MZ617-27) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S43641[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MZ617 Build/7.7.1-85_MZ617-27) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S43642[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MZ617 Build/7.7.1-85_MZ617-27)";
const char S43643[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MZ617 4G Build/7.7.1-85_MZ617-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S43644[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43645[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43646[] = "Mozilla/5.0 (Symbian/3; Series60/5.3 Nokia701/111.020.0307; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.4.1.14 Mobile Safari/533.4 3gpp-gba";
const char S43647[] = "Mozilla/5.0 (Symbian/3; Series60/5.4 Nokia701/112.010.1404; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.2.1.20 Mobile Safari/535.1 3gpp-gba";
const char S43648[] = "Mozilla/5.0 (Symbian/3; Series60/5.3 Nokia701; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.4.1.14 Mobile Safari/533.4 3gpp-gba";
const char S43649[] = "Mozilla/5.0 (Symbian/3; Series60/5.5 Nokia701/113.010.1506; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.2.21 Mobile Safari/535.1 3gpp-gba";
const char S43650[] = "Mozilla/5.0 (compatible; AcoonBot/4.12.1; +http://www.acoon.de/robot.asp)";
const char S43651[] = "Mozilla/5.0(iPad; U;CPU OS 6_1 like Mac OS X; zh-CN; iPad3,4) AppleWebKit/534.46 (KHTML, like Gecko) UCBrowser/2.0.1.280 U3/0.8.0 Safari/7543.48.3";
const char S43652[] = "Mozilla/5.0 (Windows NT 6.2; rv:14.0) Gecko/X Firefox/14.0.1";
const char S43653[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:14.0) Gecko/X Firefox/14.0.1";
const char S43654[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; YP-G1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43655[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; YP-G1 Build/GINGERBREAD)";
const char S43656[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; YP-G1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43657[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; YP-GB1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43658[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; YP-G1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43659[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.3 (KHTML, like Gecko) Maxthon/3.3.1.600 Safari/535.3";
const char S43660[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.3 (KHTML, like Gecko) Maxthon/3.3.3.1000 Chrome/16.0.883.0 Safari/535.3";
const char S43661[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.3.6.2000 Chrome/18.0.966.0 Safari/535.12";
const char S43662[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.3.9.1000 Chrome/18.0.966.0 Safari/535.12";
const char S43663[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.3.7.2000 Chrome/18.0.966.0 Safari/535.12";
const char S43664[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.3.9.2000 Chrome/18.0.966.0 Safari/535.12";
const char S43665[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ME301T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S43666[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ME301T Build/JRO03C)";
const char S43667[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S43668[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S43669[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8X by HTC)";
const char S43670[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; PM23300)";
const char S43671[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8X by HTC; Orange)";
const char S43672[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8X by HTC; 2.00.161.04)";
const char S43673[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8X by HTC; 2.00.161.01)";
const char S43674[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8X by HTC; 2.01.161.00)";
const char S43675[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8X by HTC; 2.01.166.00)";
const char S43676[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8X by HTC; 2.01.163.00)";
const char S43677[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8X by HTC; 2.01.172.00)";
const char S43678[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8X by HTC; 3.41.161.02)";
const char S43679[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8X by HTC; 1.00.161.02)";
const char S43680[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8X by HTC; 2.00.172.04)";
const char S43681[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; C620e)";
const char S43682[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P3100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43683[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43684[] = "Mozilla/5.0 (Linux; Android 4.0.3;  GT-P3100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S43685[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P3100B Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43686[] = "Mozilla/5.0 (Linux; Android 4.0.4;  GT-P3100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S43687[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3100B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43688[] = "Mozilla/5.0 (Linux; Android 4.0.3;  GT-P3100B Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S43689[] = "Mozilla/5.0 (Linux; Android 4.0.4;  GT-P3100B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S43690[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; streak7 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43691[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Dell Streak 7 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S43692[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; Dell Streak 7 Build/HTJ85B)";
const char S43693[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT897 Build/7.7.1Q-6_SPR-125_ASA-14) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43694[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.11 (KHTML, like Gecko) Ubuntu/11.10 Chromium/17.0.963.65 Chrome/17.0.963.65 Safari/535.11";
const char S43695[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.11 (KHTML, like Gecko) Ubuntu/11.10 Chromium/17.0.963.65 Chrome/17.0.963.65 Safari/535.11";
const char S43696[] = "Mozilla/Firefox";
const char S43697[] = "Mozilla/5.0 (Gecko) Firefox/64";
const char S43698[] = "Mozilla/5.0 NewsFox/1.0.8.4.2";
const char S43699[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S43700[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S43701[] = "Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_5) AppleWebKit (KHTML, like Gecko) Chrome/5";
const char S43702[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/532.9 (KHTML, like Gecko) Chrome/5.0.307.7 Safari/532.9";
const char S43703[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_0; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.99 Safari/533.4";
const char S43704[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.127 Safari/533.4";
const char S43705[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.125 Safari/533.4";
const char S43706[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.99 Safari/533.4";
const char S43707[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.127 Safari/533.4";
const char S43708[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.125 Safari/533.4";
const char S43709[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.371.0 Safari/533.4";
const char S43710[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/532.9 (KHTML	 like Gecko) Chrome/5.0.307.9 Safari/532.9";
const char S43711[] = "iCoreService (unknown version) CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro9%2C2)";
const char S43712[] = "WebProcess/8536.28.10 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro9%2C2)";
const char S43713[] = "iCoreService (unknown version) CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookAir5%2C2)";
const char S43714[] = "ThumbnailService/39600.038 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro10%2C1)";
const char S43715[] = "iCoreService (unknown version) CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (iMac13%2C1)";
const char S43716[] = "ThumbnailService/39600.038 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookAir4%2C2)";
const char S43717[] = "ThumbnailService/39600.038 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (iMac12%2C2)";
const char S43718[] = "iCoreService (unknown version) CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (Macmini6%2C2)";
const char S43719[] = "Socialite/7845 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro10%2C1)";
const char S43720[] = "Miro/5.0.4 (http://www.getmiro.com/; Darwin 12.3.0 x86_64)";
const char S43721[] = "WebProcess/8536.28.10 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (iMac13%2C1)";
const char S43722[] = "NewsBar/162 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro10%2C1)";
const char S43723[] = "XProtectUpdater (unknown version) CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro9%2C2)";
const char S43724[] = "Norton%20Identity%20Safe/1.03 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro10%2C1)";
const char S43725[] = "WebProcess/8536.29.13 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookAir5%2C2)";
const char S43726[] = "Reader%20Notifier/4 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro10%2C1)";
const char S43727[] = "CoverScout%203/3.5.7 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro10%2C1)";
const char S43728[] = "CoverScout%203/3.5.7 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (iMac12%2C2)";
const char S43729[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID4 4G Build/6.5.1_73_DR4-224_TA-6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43730[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID4 4G Build/6.5.1_167_DR4-1_M1-207) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43731[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID4 4G Build/6.5.1_167_DR4-1_M1-215) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43732[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID4 4G Build/6.5.1_167_DR4-1_M1-215-LK) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43733[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; DROID4 Build/6.5.1_167_DR4-1_M1-219)";
const char S43734[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID4 4G Build/6.5.1_167_DR4-1_M1-219) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43735[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID4 Build/6.5.1_167_DR4-1_M1-219) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43736[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID4 Build/6.5.1_167_DR4-1_M1-215) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43737[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID4 4G Build/6.5.1_73_DR4-220) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43738[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID4 4G Build/6.5.1_73_DR4-223) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43739[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID4 4G Build/6.5.1_167_DR4-1_M1-219) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S43740[] = "Mozilla/5.0 (Linux; U; Android2.3.6; en-us; DROID4 4G Build/6.5.1_167_DR4-1_M1-219) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43741[] = "Mozilla/5.0 (iPad; U; CPU OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A5338d Safari/7534.48.3";
const char S43742[] = "Mozilla/5.0 (iPad; U; CPU OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A403 Safari/7534.48.3";
const char S43743[] = "Mozilla/5.0 (iPad; U; CPU OS 6_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A523 Safari/7534.48.3";
const char S43744[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/19.0.1084.60 Version/6.0 Mobile/10A403 Safari/8536.25 HaoWangZhiDaQuanHD 1.4";
const char S43745[] = "Mozilla/5.0 (iPad; U; CPU OS 6_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A8426 Safari/7534.48.3";
const char S43746[] = "Mozilla/5.0 (iPad; U; CPU OS 6_0_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A550 Safari/7534.48.3";
const char S43747[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-I605 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43748[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43749[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Runnymede; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S43750[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_SensationXL_Beats_X315e; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S43751[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC/SensationXL_Beats/1.05.161.4; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S43752[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_SensationXL_Beats_X315b; en-th) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S43753[] = "Nokia6303iclassic/5.0 (07.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S43754[] = "Nokia6303iclassic/5.0 (07.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303iclassic";
const char S43755[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-us) AppleWebKit/535+ (KHTML, like Gecko) Version/5.0 Safari/535.4+ Midori/0.4";
const char S43756[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-us) AppleWebKit/534+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ Midori/0.4";
const char S43757[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-us) AppleWebKit/535+ (KHTML, like Gecko) Version/5.0 Sabayon Safari/535.7+ Midori/0.4";
const char S43758[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-us) AppleWebKit/535+ (KHTML, like Gecko) Version/5.0 Safari/535.20+ Midori/0.4";
const char S43759[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-gb) AppleWebKit/535+ (KHTML, like Gecko) Version/5.0 Safari/535.22+ Midori/0.4";
const char S43760[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; pl-pl) AppleWebKit/531+ (KHTML, like Gecko) Version/5.0 Safari/531.2+ Midori/0.4";
const char S43761[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-us) AppleWebKit/537+ (KHTML, like Gecko) Version/5.0 Safari/537.6+ Midori/0.4";
const char S43762[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S43763[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB502 Build/BASEM_U3_01.12.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43764[] = "Mozilla/5.0 (Linux; U;Android 2.1-update1; en-us; MB502 Build/BASIL_U3_02.60.5) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43765[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB502Build/BASIL_U3_03.90.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43766[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us;MB502 Build/BASIL_U3_03.90.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43767[] = "Mozilla/5.0 (Linux; U;Android 2.1-update1; en-us; MB502 Build/BASIL_U3_03.90.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43768[] = "Mozilla/5.0 (Linux; U; Android2.1-update1; en-us; MB502 Build/BASIL_U3_03.90.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43769[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB502 Build/BASIL2174) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43770[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB502 Build/BASIL_U3_03.90.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 MobileSafari/530.17";
const char S43771[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; MB502 Build/BASTE_U3_02.39.5) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43772[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB502 Build/BASIL2177) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43773[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB502 Build/BASEM_U3_01.12.4) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43774[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; MB502 Build/BASTE_U3_02.39.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43775[] = "Mozilla/5.0 (Linux; U; Android2.1-update1; en-us; MB502 Build/BASIL_U3_02.60.5) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43776[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB502 Build/BASIL_U3_03.90.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43777[] = "MOT-MB502/Blur_Version.0.40.0.MB502.Brasil.en.BR Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; MB502 Build/BASLA_U3_00.40.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43778[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB502 Build/BASIL_U3_02.60.5) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43779[] = "Mozilla/5.0 (Linux; U; Android 2.1-NEW_T%5P_MB502; zh-cn; MB502 Build/BASIL_U3_03.90.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43780[] = "MOT-MB502/Blur_Version.0.32.209.MB502.AmericaMovil.en.01 Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB502 Build/BASLA_U3_00.32.203_R04) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43781[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB502Build/BASIL_U3_02.60.5) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43782[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us;MB502 Build/BASIL_U3_02.60.5) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43783[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; MB502 Build/BASIL_U3_03.90.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43784[] = "MOT-MB502/Blur_Version.0.32.202.MB502.Movistar.en.01 Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; MB502 Build/BASLA_U3_00.32.202) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43785[] = "MOT-MB502/Blur_Version.0.32.101.MB502.Vivo.en.BR Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; MB502 Build/BASLA_U3_00.32.101_R02) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43786[] = "MOT-MB502/Blur_Version.0.30.1.MB502.AmericaMovil.en.MX Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; MB502 Build/BASLA_U3_00.30.0_R01) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43787[] = "MOT-MB502/Blur_Version.0.32.101.MB502.Brasil.en.BR Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; MB502 Build/BASLA_U3_00.32.101) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43788[] = "MOT-MB502/Blur_Version.0.32.205.MB502.Movilnet.en.VE Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; MB502 Build/BASLA_U3_00.32.203_R02) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43789[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; he-il; MB502 Build/BASEM_U3_01.19.4) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S43790[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; MB502 Build/BASIL_U3_03.90.7)";
const char S43791[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB502 Build/BASEM_U3_01.12.4) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S43792[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT907 Build/7.7.1Q-144_SMQ_S136-10) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43793[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT905 Build/7.7.1Q_GCIRD-16) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S43794[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR MAXX 4G Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43795[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Motorola RAZR MAXX Build/6.7.2-180_DHD-16_NA-27) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43796[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Motorola RAZR MAXX Build/6.7.2-180_DHD-16_NA-30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43797[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Motorola RAZR MAXX Build/6.7.2-180_DHD-16_NA-30)";
const char S43798[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43799[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43800[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43801[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Galaxy Nexus Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43802[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus 7 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43803[] = "Mozilla/5.0 (Linux; Android 4.2.3; I9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36";
const char S43804[] = "Mozilla/5.0 (Linux; Android 4.2.9; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36";
const char S43805[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S43806[] = "Mozilla/5.0 (Linux; Android 4.2.3; I9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S43807[] = "Mozilla/5.0 (Linux; Android 4.2.2; I9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S43808[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S43809[] = "Mozilla/5.0 (Linux; Android 4.2.9; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S43810[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy GT-I9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S43811[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; PantechP8000 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43812[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; PantechP8000/JPUS01242011 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43813[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; PantechP8000 Build/FRG83)";
const char S43814[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/18.0.1025.142) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19";
const char S43815[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/18.0.1025.142) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19";
const char S43816[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/18.0.1025.142; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; MS-RTC EA 2; MS-RTC LM 8; .NET4.0C)";
const char S43817[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/18.0.1025.142; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3)";
const char S43818[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/18.0.1025.151) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19";
const char S43819[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/18.0.1025.151) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19";
const char S43820[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/18.0.1025.152) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.152 Safari/535.19";
const char S43821[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/18.0.1025.152) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.152 Safari/535.19";
const char S43822[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/18.0.1025.162)";
const char S43823[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/18.0.1025.162) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19";
const char S43824[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/18.0.1025.162)";
const char S43825[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/18.0.1025.162) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19";
const char S43826[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/18.0.1025.162; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S43827[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/18.0.1025.162; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.3; .NET4.0C; .NET4.0E)";
const char S43828[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/18.0.1025.168) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S43829[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/18.0.1025.168) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S43830[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/18.0.1025.168; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)";
const char S43831[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/18.0.1025.54; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S43832[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; GTB7.2; chromeframe/18.0.1025.11; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; eSobiSubscriber 2.0.4.16; .NET4.0C)";
const char S43833[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/18.0.1025.152; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)";
const char S43834[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/18.0.1025.162)";
const char S43835[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/18.0.1025.162)";
const char S43836[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG-C900)";
const char S43837[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG-C900k)";
const char S43838[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG-C900B)";
const char S43839[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Samsung Galaxy Note I717 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43840[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I717 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43841[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I717 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43842[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I717 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43843[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43844[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I717 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43845[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I717 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43846[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I717 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43847[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-I717 Build/JZO54K)";
const char S43848[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43849[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; Samsung-SGH-I717 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43850[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4";
const char S43851[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4";
const char S43852[] = "Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Safari/535.19 Silk-Accelerated=true";
const char S43853[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-hk; HTC_One_XL Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43854[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC One XL Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43855[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC One XL Build/IML74K)";
const char S43856[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC/One_XL/1.82.162.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43857[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_One_XL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43858[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; HTC One XL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43859[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC One XL Build/IMM76D)";
const char S43860[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HTC/One_XL/2.33.162.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43861[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC/One_XL/1.82.162.6 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43862[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HTC_One_XL/2.27.111.4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43863[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; HTC/One_XL/2.34.163.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43864[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-ch; HTC/One_XL/2.41.166.51 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43865[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; HTC/One_XL/2.41.163.53 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43866[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HTC_One_XL/2.27.111.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43867[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCOneXL Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43868[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; HTC/One_XL/2.41.163.51 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43869[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43870[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U8825D Build/HuaweiU8825D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S43871[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C92B933SP05; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S43872[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C92B936; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S43873[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C92B931SP06; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S43874[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C92B938; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S43875[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8825D Build/HuaweiC8825D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43876[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8825D Build/HuaweiC8825D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S43877[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C92B943SP01; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S43878[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI U8825D Build/HuaweiU8825D)";
const char S43879[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8825D Build/HuaweiC8825D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S43880[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HUAWEI U8825-1 Build/HuaweiU8825-1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43881[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C92B945; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S43882[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C92B941; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S43883[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI U8825-1 Build/HuaweiU8825-1)";
const char S43884[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI C8825D Build/HuaweiC8825D)";
const char S43885[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Bs 401 Build/HuaweiU8825-1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43886[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C92B947; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S43887[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C92B939SP01; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S43888[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C117B948; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S43889[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C117B943; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S43890[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C92B968; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S43891[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; HUAWEI Ascend G 330 Build/HuaweiU8825-1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43892[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AN10BG3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S43893[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; AN10BG3-LZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S43894[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; AN10BG3DT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S43895[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; AN10BG3 Build/IML74K)";
const char S43896[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MK16i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43897[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonMK16i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43898[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MK16i Build/4.1.B.0.587)";
const char S43899[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MK16i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43900[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700T Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43901[] = "Mozilla/5.0 (Linux; Android 4.1.2; ASUS Transformer Pad TF700T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43902[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700KL Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43903[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Pad TF700T Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S43904[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S43905[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-T999 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S43906[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I535 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S43907[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I535 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S43908[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S43909[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T999 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S43910[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9300 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S43911[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T999 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S43912[] = "Mozilla/5.0 (Linux; Android 4.2.1; SCH-I535 Build/JOP40G) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S43913[] = "Opera/9.80 (X11; Linux x86_64) Presto/2.12.388 Version/12.16";
const char S43914[] = "Mozilla/5.0 (X11; Linux x86_64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.16";
const char S43915[] = "Mozilla/5.0 (compatible; MSIE 9.0; X11; Linux x86_64) Opera 12.16";
const char S43916[] = "Opera/9.80 (X11; Linux i686) Presto/2.12.388 Version/12.16,gzip(gfe)";
const char S43917[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IdeaTabA2109A Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S43918[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; IdeaTabA2109A Build/JRO03R)";
const char S43919[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.10.289 Version/12.02";
const char S43920[] = "Opera/9.80 (Windows NT 5.0; U; MRA 6.0 (build 5972); ru) Presto/2.10.289 Version/12.02";
const char S43921[] = "Mozilla/5.0 (Android; Mobile; rv:24.0) Gecko/24.0 Firefox/24.0";
const char S43922[] = "Mozilla/5.0 (Android; Mobile; rv:24.0a2) Gecko/24.0a2 Firefox/24.0a2";
const char S43923[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_PH06130/1.23.502.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43924[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_PH06130/2.14.502.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43925[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_PH06130/1.23.502.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43926[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG-E900; Orange)";
const char S43927[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG-E900h)";
const char S43928[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG-E900)";
const char S43929[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG E-900)";
const char S43930[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG-E900; TESCO)";
const char S43931[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG-E900; O2UK)";
const char S43932[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; MWP6985)";
const char S43933[] = "Mozilla/5.0 (Windows; N; Windows NT 6.0; en-US) AppleWebKit/527+ (KHTML, like Gecko, Safari/419.3) Qt/4.5.3";
const char S43934[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:17.0) Gecko/X Firefox/17.0 SeaMonkey/2.14.1";
const char S43935[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:17.0) Gecko/X SeaMonkey/2.14.1";
const char S43936[] = "Mozilla/6.0 (compatible; MSIE 7.01; AOL 9.0; Windows 98)";
const char S43937[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows 98; SpamBlockerUtility 6.3.91; SpamBlockerUtility 6.2.91; .NET CLR 4.1.89;GB)";
const char S43938[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows 98; SV1; SV1)";
const char S43939[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; SV1)";
const char S43940[] = "Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; MSN Optimized)";
const char S43941[] = "Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; SV1)";
const char S43942[] = "Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; SV1; SV1)";
const char S43943[] = "Mozilla/5.0 (compatible; MSIE 7.0; Win98; Trident/3.0)";
const char S43944[] = "Mozilla/5.0 (compatible; MSIE 7.0; Win 9x 4.90; Trident/2.0)";
const char S43945[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.67; en-us; 8520/5.2.0.67) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S43946[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.64; en-us; 8520/5.2.0.64) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S43947[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.104; en-us; 8520/5.2.0.104) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S43948[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.113; en-us; 8520/5.2.0.113) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S43949[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.135; en-us; 8520/4.2.0.135) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S43950[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.96; en-us; 8520/5.2.0.96) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S43951[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.47; en-us; 8520/5.2.0.47) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S43952[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.118; en-us; 8520/4.2.0.118) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S43953[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.76; en-us; 8520/5.2.0.76) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S43954[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.110; en-us; 8520/5.2.0.110) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S43955[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.111; en-us; 8520/5.2.0.111) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S43956[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.58; en-us; 8520/5.2.0.58) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S43957[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.23; en-us; 8520/5.2.0.23) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S43958[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.122; en-us; 8520/4.2.0.122) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S43959[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.94; en-us; 8520/5.2.0.94) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S43960[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.31; en-us; 8520/5.2.0.31) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S43961[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.102; en-us; 8520/5.2.0.102) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S43962[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.116; en-us; 8520/4.2.0.116) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S43963[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.124; en-us; 8520/4.2.0.124) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S43964[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Arc Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S43965[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-; Arc Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S43966[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Arc Build/JRO03L)";
const char S43967[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L720 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43968[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SPH-L720 Build/JDQ39)";
const char S43969[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-; SPH-L720 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43970[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.2.2;dev=jfltespr;devm=SPH-L720;man=samsung;brw=Android;shwd=1080x1920;shwa=1080x1845;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S43971[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S43972[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-I605 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S43973[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S43974[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT26w Build/6.2.B.0.200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43975[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LT26w Build/6.2.B.0.200)";
const char S43976[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT26w Build/6.2.B.0.200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43977[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT26w Build/6.2.B.0.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43978[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT26w Build/6.2.B.0.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43979[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT26w Build/6.2.B.1.96) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43980[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT26w Build/6.2.B.1.96) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43981[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6603 Build/10.1.A.1.434) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S43982[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6603 Build/10.1.A.1.434) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S43983[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ADR3010 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S43984[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; ADR3010 Build/GRK39F)";
const char S43985[] = "BlackBerry9100/5.0.0.921 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102";
const char S43986[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S43987[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S43988[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S43989[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S43990[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT_I8190 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S43991[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30";
const char S43992[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S43993[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S43994[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S43995[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S43996[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S43997[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E610 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S43998[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E610 Build/JZO54K)";
const char S43999[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E612f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44000[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E612 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44001[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E612 Build/JZO54K)";
const char S44002[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-E610/E61020d-EUR-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44003[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; LG-E610v/V20d Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44004[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-E610v/V20e Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44005[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-E610/V20b-Orange Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44006[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; LG-E610/V20a Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44007[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E612f Build/JZO54K)";
const char S44008[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; bs; LG-E610 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44009[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E612g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44010[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; WX445 Build/VZW) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 240X320 motorola WX445";
const char S44011[] = "Dalvik/1.2.0 (Linux; U; Android 2.1-update1; WX445 Build/VZW)";
const char S44012[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; WX445 Build/OPM) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 240X320 motorola WX445";
const char S44013[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; WX445 Build/VZW) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17240X320 motorola WX445";
const char S44014[] = "Mozilla/5.0 (Linux; Android 4.0.2; Galaxy Nexus Build/ICL53F) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S44015[] = "Mozilla/5.0 (Linux; Android 4.0.1;  Galaxy Nexus Build/ITL41F) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S44016[] = "Mozilla/5.0 (Linux; Android 4.0.3;  Galaxy Nexus Build/ITL41F) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S44017[] = "Mozilla/5.0 (Linux; Android 4.0.2;  Galaxy Nexus Build/ICL53F) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S44018[] = "Mozilla/5.0 (Linux; Android 4.0.4;  Galaxy Nexus Build/IMM76K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S44019[] = "Mozilla/5.0 (Linux; Android 4.0.4;  Galaxy Nexus Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S44020[] = "Mozilla/5.0 (Linux; Android 4.0.1; Galaxy Nexus Build/ITL41F) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S44021[] = "Mozilla/5.0 (Linux; Android 4.0.2; Galaxy X Build/ICL53F) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S44022[] = "Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S44023[] = "Mozilla/5.0 (Linux; Android 4.0.3; Galaxy Nexus Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S44024[] = "Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S44025[] = "Mozilla/5.0 (Linux; Android 4.0.4; Galaxy X Build/IMM76K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S44026[] = "Mozilla/5.0 (Linux; Android 4.0.4; Sprint Galaxy Nexus CDMA Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S44027[] = "Mozilla/5.0 (Linux; Android 4.0.4 IMM76D; Galaxy Nexus Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S44028[] = "Mozilla/5.0 (Linux; Android 4.0.1; GT-I9250_EUR_XX Build/ICL23D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S44029[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L720 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S44030[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L720 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S44031[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows 95)";
const char S44032[] = "Mozilla/5.0 (compatible; MSIE 5.0; Win95; Trident/1.0)";
const char S44033[] = "Mozilla/4.0 (compatible; MSIE 5.01 Windows 95)";
const char S44034[] = "Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_5) AppleWebKit (KHTML, like Gecko) Chrome/4";
const char S44035[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5";
const char S44036[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5";
const char S44037[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_0; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5";
const char S44038[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5";
const char S44039[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5";
const char S44040[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.0 Safari/532.5";
const char S44041[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/4.0.202.0 Safari/532.0";
const char S44042[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/532.5 (KHTML	 like Gecko) Chrome/4.0.249.30 Safari/532.5";
const char S44043[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/4.0.202.0 Safari/532.0)";
const char S44044[] = "Mozilla/5.0 (Linux; Android 4.0.4; V8000_USA_Cricket Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S44045[] = "Mozilla/5.0 (Linux; Android 4.0.4; N8000_USA_Cricket Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S44046[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Ubuntu/10.10 Chromium/9.0.597.94 Chrome/9.0.597.94 Safari/534.13";
const char S44047[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.14 (KHTML, like Gecko) Ubuntu/10.10 Chromium/9.0.600.0 Chrome/9.0.600.0 Safari/534.14";
const char S44048[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/540.0 (KHTML, like Gecko) Ubuntu/10.10 Chrome/9.1.0.0 Safari/540.0";
const char S44049[] = "Mozilla/5.0 (X11; U; Linux armv7l; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Ubuntu/10.10 Chromium/9.0.597.94 Chrome/9.0.597.94 Safari/534.13";
const char S44050[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC One XL Build/JRO03C)";
const char S44051[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; HTC One XL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44052[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_XL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44053[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_XL/3.17.111.1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44054[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-ch; HTC/One_XL/3.17.166.2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44055[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; HTC/One_XL/3.17.163.2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44056[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC/One_XL/3.17.162.7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44057[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC/One_XL/3.17.162.8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44058[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_XL/3.17.111.2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44059[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One XL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44060[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC EVA_UTL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44061[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One XL Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44062[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HP Slate 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S44063[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HP Slate 7 Build/JRO03H)";
const char S44064[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S44065[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710BST Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S44066[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710VMUB Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S44067[] = "Mozilla/5.0 (Linux; Android 4.0.3; SPH-D710 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S44068[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S44069[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S44070[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S44071[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S44072[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S44073[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S44074[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB520 Build/Blur_Version.37.4.0.MB520.ATT.en.US Flex/P015) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44075[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB520 Build/Blur_Version.1.0.939.MB520-continuous.ATT.en.US Flex/P008) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44076[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB520 Build/Blur_Version.36.10.0.MB520.ATT.en.US Flex/P014) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44077[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB520 Build/3.4.2-125_KOB_FFW-4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44078[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB520 Build/Blur_Version.6.16.939.MB520.ATT.en.US Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44079[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB520 Build/3.4.2-125_KOB_CCI-6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44080[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB520 Build/STAB2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44081[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB520 Build/Blur_Version.35.9.94.MB520.ATT.en.US Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44082[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB520 Build/3.4.2_8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44083[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB520 Build/Blur_Version.36.15.0.MB520.ATT.en.US Flex/P015) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44084[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB520 Build/Blur_Version.38.6.0.MB520.Generic.en.US Flex/P002) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44085[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB520 Build/Blur_Version.37.5.0.MB520.ATT.en.US Flex/P015) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44086[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; MB520 Build/3.4.2-125_KOB_FFW-4)";
const char S44087[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; MB520 Build/3.4.2-125_KOB_CCI-6)";
const char S44088[] = "Opera/9.80 (Windows Mobile; WCE; Opera Mobi/WMD-50433; U; en) Presto/2.4.13 Version/10.00";
const char S44089[] = "Mozilla/4.0 (compatible; Windows Mobile; WCE; Opera Mobi/WMD-50433; U; de) Presto/2.4.13 Version/10.00";
const char S44090[] = "Mozilla/5.0 (Series40; Nokia311/03.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.13.1";
const char S44091[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S44092[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S44093[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S44094[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9100 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S44095[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S44096[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S44097[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S44098[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S44099[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S44100[] = "MAUI WAP Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S44101[] = "NX-W702/V0.1.4 Release/24.8.2010 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S44102[] = "GHT PIANO/MTK Release/Oct-23-2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S44103[] = "GHT CHAT/MTK Release/Oct-23-2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S44104[] = "MTK_MAUI_01/07 Release/22.11.2007 Browser/wap2.0 Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S44105[] = "T23 WIFI Duo/MTKRelease/2011/07/01Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0";
const char S44106[] = "SAGETEL55_FTE_11B_HW (MRE\\3.0.00(20480) resolution\\X chipset\\MT6255 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) V09-MX-KL01-T605-MP Release/2012.11.14 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44107[] = "KP26_11B_HW (MRE\\3.0.00(20480) resolution\\X chipset\\MT6255 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) V06_MX_KP26_T731_MP Release/2012.11.15 WAP Browser/MAUI (HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44108[] = "SAGETEL55_FTE_11B_HW (MRE\\3.0.00(20480) resolution\\X chipset\\MT6255 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) V07-MX-KL01-T605-MP Release/2012.11.03 WAP Browser/MAUI (HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44109[] = "GIONEE50_12864_11B_HW (MRE\\2.5.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) 1307SY_0201_V0802 Release/2012.10.17 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44110[] = "TINNO50_6464_W1224_11B_HW (MRE\\2.5.00(1536) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\1 gsensor\\0 keyboard\\reduced) F5100B_V14_X_2130 Release/2012.11.21 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 vi-VN";
const char S44111[] = "GIONEE50_12864_11B_HW (MRE\\3.0.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) 1303QM_0201_V0933 Release/2012.11.20 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44112[] = "SAGETEL55_FTE_11B_HW (MRE\\3.0.00(20480) resolution\\X chipset\\MT6255 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) V13-MX-KL01-T605-MP Release/2012.12.29 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44113[] = "TINNO50_12864_W1224_11B_HW (MRE\\2.5.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\1 gsensor\\0 keyboard\\reduced) F7100B_V15_X_1548 Release/2012.11.22 WAP Browser/MAUI (HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 vi-VN";
const char S44114[] = "GIONEE50_12864_COSMOS_11B_HW (MRE\\2.5.00(2500) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) 1302SY_0201_V1934 Release/2012.11.15 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44115[] = "TINNO50_12864_W1224_11B_HW (MRE\\2.5.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\1 gsensor\\0 keyboard\\reduced) F7100B_V13_X_1850 Release/2012.11.12 WAP Browser/MAUI (HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 vi-VN";
const char S44116[] = "TINNO50_6464_W1224_11B_HW (MRE/3.0.00(1536);MAUI/B_PP_F1_PK_F2_08_V9;BDATE/2012/11/06 15:53;LCD/X;CHIP/MT6250;KEY/Normal;TOUCH/0;CAMERA/1;SENSOR/0;DEV/TINNO50_6464_W1224_11B_HW;WAP Browser/MAUI (HTTP PGDL;HTTPS);GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/005;GAMELOFT/) B2040B_PP_F1_PK_F2_08_V9 Release/2012.11.06 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44117[] = "RATECH50_12864_11B_HW (MRE\\2.5.00(3072) resolution\\X chipset\\MT6250 touch\\0 tpannel\\0 camera\\1 gsensor\\0 keyboard\\qwerty) E532_TV40_A1_11B_V4_13_1117 Release/2012.11.17 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44118[] = "RATECH55_X_11B_HW (MRE\\2.5.00(2560) resolution\\X chipset\\MT6255 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\qwerty) E630_T736_A1_11B_V5_8_1109 Release/2012.11.10 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44119[] = "SYM.S103.GIONEE50_12864_11B_HW (MRE\\3.0.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) 1303SY_0201_V0972 Release/2012.12.15 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44120[] = "SAGETEL55_FTE_11B_HW (MRE\\3.0.00(20480) resolution\\X chipset\\MT6255 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) V07-MX-KL02-T611-MP Release/2012.12.14 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44121[] = "RATECH55_X_11B_HW (MRE\\2.5.00(2560) resolution\\X chipset\\MT6255 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normalWAP Browser/MAUI (HTTP PGDL;HTTPS)) A638_T511_B1_MLA_11B_V2_6_1218 Release/2012.12.19 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44122[] = "TINNO50_12864_W1224_11B_HW (MRE\\2.5.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\1 gsensor\\0 keyboard\\reduced) F7100B_PP_F4_PK_F2_02_V9 Release/2012.12.11 WAP Browser/MAUI (HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44123[] = "RATECH55_1G256_11B_HW (MRE\\2.3.00(9560) resolution\\X chipset\\MT6255 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) X610_T721_A3_11B_V4_15_1213 Release/2012.12.14 WAP Browser/MAUI Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44124[] = "RATECH55_1G256_11B_HW (MRE\\2.3.00(1536) resolution\\X chipset\\MT6255 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) X615_6255_CY_TV70_B2_27S1116 Release/2012.11.16 WAP Browser/MAUI Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44125[] = "SAGETEL55_FTE_11B_HW (MRE\\3.0.00(20480) resolution\\X chipset\\MT6255 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) V11-MX-KL02-T611-MP Release/2013.01.24 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44126[] = "GIONEE50_12864_11B_HW (MRE\\3.0.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) 1303QM_0201_V0986 Release/2013.01.05 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44127[] = "SAGETEL55_FTE_11B_HW (MRE\\3.0.00(20480) resolution\\X chipset\\MT6255 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) V14-MX-KL01-T605-MP Release/2013.02.19 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44128[] = "KP26_11B_HW (MRE\\3.0.00(20480) resolution\\X chipset\\MT6255 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) V10_MX_KP26_T731_MP Release/2013.01.05 WAP Browser/MAUI (HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44129[] = "RATECH55_1G256_11B_HW (MRE\\2.3.00(1536) resolution\\X chipset\\MT6255 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) X615_6255_CY_TV70_B2_21S0716 Release/2012.07.16 WAP Browser/MAUI Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44130[] = "RATECH55_X_11B_HW (MRE\\2.5.00(2560) resolution\\X chipset\\MT6255 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\qwerty) E690_T638_A1_MLA_11B_V4_5_1108 Release/2012.11.09 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44131[] = "RATECH55_X_11B_HW (MRE\\2.5.00(2560) resolution\\X chipset\\MT6255 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) A638_T551_B1_MLA_11B_V5_3_1026 Release/2012.10.30 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44132[] = "RATECH50_12864_11B_HW (MRE\\2.5.00(3072) resolution\\X chipset\\MT6250 touch\\0 tpannel\\0 camera\\1 gsensor\\0 keyboard\\qwerty) E532_T531_B2_11B_V3_6_1119 Release/2012.11.19 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44133[] = "GIONEE50_12864_11B_HW (MRE\\3.0.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) 1300QM_0201_V0525 Release/2013.01.11 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44134[] = "TINNO50_6464_W1224_11B_HW (MRE/3.0.00(1536);MAUI/B_MP_F4_PK_F20_V8_07;BDATE/2012/12/10 21:07;LCD/X;CHIP/MT6250;KEY/Reduced;TOUCH/1;CAMERA/1;SENSOR/0;DEV/TINNO50_6464_W1224_11B_HW;WAP Browser/MAUI (HTTP PGDL;HTTPS);GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/005;GAMELOF) F5100B_MP_F4_PK_F20_V8_07 Release/2012.12.10 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44135[] = "RATECH55_X_11B_HW (MRE\\2.5.00(2560) resolution\\X chipset\\MT6255 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normalWAP Browser/MAUI (HTTP PGDL;HTTPS)) A638_T551_B1_MLA_11B_V5_8_0129 Release/2013.01.31 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44136[] = "HONGYU50_COSMOS_11B_HW (MRE\\3.0.00(2500) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) K60_karbonn_P108A_V1.4_X Release/2013.03.18 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44137[] = "SAGETEL55_FTE_11B_HW (MRE\\3.0.00(20480) resolution\\X chipset\\MT6255 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) V12-MX-KL02-T611-MP Release/2013.02.28 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44138[] = "TINNO50_12864_W1224_11B_HW (MRE\\2.5.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\1 gsensor\\0 keyboard\\reduced) F7100B_PP_F4_PK_F2_02_V11 Release/2013.03.07 WAP Browser/MAUI (HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44139[] = "GIONEE50_12864_COSMOS_11B_HW (MRE\\2.5.00(2500) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) 1302SY_0201_V1915 Release/2012.10.20 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44140[] = "GIONEE50_12864_11B_HW (MRE\\3.0.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) 1305QM_0201_V1958 Release/2013.01.05 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44141[] = "GIONEE50_12864_11B_HW (MRE\\2.5.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) 1305GA_0201_V1711 Release/2012.10.24 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44142[] = "RATECH55_X_11B_HW (MRE\\2.5.00(2560) resolution\\X chipset\\MT6255 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normalWAP Browser/MAUI (HTTP PGDL;HTTPS)) A638_T511_B1_MLA_11B_V2_9_0304 Release/2013.03.05 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44143[] = "SAGETEL55_FTE_11B_HW (MRE\\3.0.00(20480) resolution\\X chipset\\MT6255 touch\\0 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) V09_MX_KL16_TV30_MP Release/2013.02.07 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44144[] = "TINNO50_6464_W1224_11B_HW (MRE\\2.5.00(800) resolution\\X chipset\\MT6250 touch\\0 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) B2040A_B18TV_X_V1.05 Release/2012.10.01 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44145[] = "GIONEE50_12864_COSMOS_11B_HW (MRE\\2.5.00(2500) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) 1302GI_0201_V1957 Release/2012.12.25 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44146[] = "SAGETEL55_FTE_11B_HW (MRE\\3.0.00(20480) resolution\\X chipset\\MT6255 touch\\0 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) V02_MX_KL16_T608_MP Release/2012.12.29 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44147[] = "RATECH55_X_11B_HW (MRE\\2.5.00(2560) resolution\\X chipset\\MT6255 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normalWAP Browser/MAUI (HTTP PGDL;HTTPS)) A638_T551_B1_MLA_11B_V5_7_0108 Release/2013.01.09 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44148[] = "RATECH55_1G256_11B_HW (MRE\\2.3.00(9560) resolution\\X chipset\\MT6255 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) X601_T751_B3_11B_V5_6_1121 Release/2012.11.24 WAP Browser/MAUI Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44149[] = "RATECH55_X_11B_HW (MRE\\2.5.00(3072) resolution\\X chipset\\MT6255 touch\\0 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normalWAP Browser/MAUI (HTTP PGDL;HTTPS)) A685_T21_A1_MLA_11B_V4_5_0411 Release/2013.04.11 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44150[] = "KP515_T756_HW (MRE\\3.0.00(20480) resolution\\X chipset\\MT6255 touch\\1 tpannel\\1 camera\\1 gsensor\\0 keyboard\\qwerty) V05-MX-KP515-T756-MP Release/2013.02.26 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44151[] = "RATECH55_X_11B_HW (MRE\\2.5.00(3072) resolution\\X chipset\\MT6255 touch\\0 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normalWAP Browser/MAUI (HTTP PGDL;HTTPS)) A685_T21_A1_MLA_11B_V4_4_0330 Release/2013.03.30 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44152[] = "RATECH55_1G256_11B_HW (MRE\\2.3.00(1536) resolution\\X chipset\\MT6255 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) X615_6255_CY_M50_B2_07S1031 Release/2012.10.31 WAP Browser/MAUI Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44153[] = "GIONEE50_12864_11B_HW (MRE\\3.0.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) 1303GA_0201_V0997 Release/2013.01.18 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44154[] = "RATECH55_X_11B_HW (MRE\\2.5.00(2560) resolution\\X chipset\\MT6255 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normalWAP Browser/MAUI (HTTP PGDL;HTTPS)) A638_T511_B1_MLA_11B_V2_7_0109 Release/2013.01.10 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44155[] = "SAGETEL55_FTE_11B_HW (MRE\\3.0.00(20480) resolution\\X chipset\\MT6255 touch\\0 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) V04_MX_KL16_TV65_MP Release/2013.02.20 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44156[] = "SAGETEL55_FTE_11B_HW (MRE\\3.0.00(20480) resolution\\X chipset\\MT6255 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) V04-MX-KL01-T605-ETH-MP Release/2013.01.23 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44157[] = "RATECH55_X_11B_HW (MRE\\2.5.00(2560) resolution\\X chipset\\MT6255 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\qwerty) E690_T638_A1_MLA_11B_V5_2_1221 Release/2012.12.22 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44158[] = "GIONEE50_12864_11B_HW (MRE\\3.0.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) 1305QM_0201_V1944 Release/2012.12.15 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44159[] = "GIONEE50_12864_11B_HW (MRE\\3.0.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) 1303GI_0202_V0981 Release/2012.12.26 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44160[] = "RATECH55_X_11B_HW (MRE\\3.0.00(2560) resolution\\X chipset\\MT6255 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\qwerty) E630_T736_A1_11B_V5_9_1123_S Release/2012.11.28 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44161[] = "GIONEE50_12864_11B_HW (MRE\\3.0.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) 1303GA_0201_V1056 Release/2013.04.02 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44162[] = "SAGETEL55_FTE_11B_HW (MRE\\3.0.00(20480) resolution\\X chipset\\MT6255 touch\\0 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) V02_MX_KL16_T608_ET_MP Release/2013.03.13 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44163[] = "GIONEE50_12864_11B_HW (MRE\\2.5.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) 1307SY_0201_V1019 Release/2013.01.30 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44164[] = "GIONEE50_12864_COSMOS_11B_HW (MRE\\2.5.00(2500) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) 1302SY_0201_V1960 Release/2013.01.23 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44165[] = "RATECH55_1G256_11B_HW (MRE\\2.3.00(9560) resolution\\X chipset\\MT6255 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) X601_T751_B3_11B_V6_0_0201 Release/2013.02.01 WAP Browser/MAUI Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44166[] = "SAGETEL55_FTE_11B_HW (MRE\\3.0.00(20480) resolution\\X chipset\\MT6255 touch\\0 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) V04_MX_KL16_T608_MP Release/2013.03.15 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44167[] = "RATECH55_X_11B_HW (MRE\\2.5.00(2560) resolution\\X chipset\\MT6255 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\qwerty) E690_T638_A1_MLA_11B_V5_4_0204 Release/2013.02.04 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44168[] = "TINNO50_6464_W1224_11B_HW (MRE/3.0.00(1536);MAUI/D_MP_F4_PK_F20_V8_09;BDATE/2013/03/05 10:08;LCD/X;CHIP/MT6250;KEY/Reduced;TOUCH/1;CAMERA/1;SENSOR/0;DEV/TINNO50_6464_W1224_11B_HW;WAP Browser/MAUI (HTTP PGDL;HTTPS);GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/005;GAMELOF) F5100D_MP_F4_PK_F20_V8_09 Release/2013.03.05 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44169[] = "SYM.D67.GIONEE50_12864_11B_HW (MRE\\3.0.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) 1300SY_0201_V0543 Release/2013.03.21 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44170[] = "SYM.S103.GIONEE50_12864_11B_HW (MRE\\3.0.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) 1303SY_0201_V1060 Release/2013.04.18 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44171[] = "SYM.D67.GIONEE50_12864_11B_HW (MRE\\3.0.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) 1300SY_0201_V0557 Release/2013.04.18 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44172[] = "HONGYU50_COSMOS_11B_HW (MRE/3.0.00(2500);MAUI/_GLT01_P108B_KON_X;BDATE/2013/05/13 15:41;LCD/X;CHIP/MT6250;KEY/Reduced;TOUCH/1;CAMERA/0;SENSOR/0;DEV/HONGYU50_COSMOS_11B_HW;WAP Browser/MAUI (HTTP PGDL;HTTPS);GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/005;GAMEL) C737D_GLT01_P108B_KON_X Release/2013.05.13 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44173[] = "GIONEE50_12864_11B_HW (MRE\\3.0.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) 1305GA_0201_V1952 Release/2012.12.25 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44174[] = "RATECH55_X_11B_HW (MRE\\3.0.00(2560) resolution\\X chipset\\MT6255 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\qwerty) E630_T736_A1_11B_V6_0_0222_S Release/2013.02.22 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44175[] = "GIONEE50_12864_11B_HW (MRE\\3.0.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) 1303GI_0202_V1039 Release/2013.03.13 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44176[] = "TINNO50_6464_W1224_11B_HW (MRE/3.0.00(1536);MAUI/B_MP_F3F8_PK_QMB_02_10;BDATE/2013/02/06 11:36;LCD/X;CHIP/MT6250;KEY/Normal;TOUCH/0;CAMERA/1;SENSOR/0;DEV/TINNO50_6464_W1224_11B_HW;WAP Browser/MAUI (HTTP PGDL;HTTPS);GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/005;GAMELO) B2060B_MP_F3F8_PK_QMB_02_10 Release/2013.02.06 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44177[] = "TINNO50_6464_W1224_11B_HW (MRE/3.0.00(1536);MAUI/B_MP_F3F8_PK_QMB_05_11;BDATE/2013/03/28 15:17;LCD/X;CHIP/MT6250;KEY/Normal;TOUCH/0;CAMERA/1;SENSOR/0;DEV/TINNO50_6464_W1224_11B_HW;WAP Browser/MAUI (HTTP PGDL;HTTPS);GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/005;GAMELO) B2070B_MP_F3F8_PK_QMB_05_11 Release/2013.03.28 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44178[] = "SYM.FT33.HONGYU50_COSMOS_11B_HW (MRE/3.0.00(2500);MAUI/ONY_FT33_C737D1_M01_S04B_X;BDATE/2013/05/24 10:52;LCD/X;CHIP/MT6250;KEY/Reduced;TOUCH/1;CAMERA/0;SENSOR/0;DEV/HONGYU50_COSMOS_11B_HW;WAP Browser/MAUI (HTTP PGDL;HTTPS);GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/0) SYMPHONY_FT33_C737D1_M01_S04B_X Release/2013.05.24 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44179[] = "TINNO50_12864_W1224_11B_HW (MRE\\2.5.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\1 gsensor\\0 keyboard\\reduced) F7101A_TW30TV_DUO_X_V16_0 Release/2013.06.09 WAP Browser/MAUI (HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44180[] = "GIONEE50_12864_11B_HW (MRE\\3.0.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) 1303GA_0201_V1058 Release/2013.04.17 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44181[] = "TINNO50_12864_W1224_11B_HW (MRE\\2.5.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\1 gsensor\\0 keyboard\\reduced) F7100B_PP_F4_PK_F2_02_V14 Release/2013.07.01 WAP Browser/MAUI (HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44182[] = "GIONEE50_12864_11B_HW (MRE\\3.0.00(3072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) 1305GA_0201_V1968 Release/2013.03.23 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44183[] = "KP515_T756_HW (MRE\\3.0.00(20480) resolution\\X chipset\\MT6255 touch\\1 tpannel\\1 camera\\1 gsensor\\0 keyboard\\qwerty) V08-MX-KP515-T756-MP Release/2013.04.02 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S44184[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11";
const char S44185[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11";
const char S44186[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Ubuntu Chromium/23.0.1271.97 Chrome/23.0.1271.97 Safari/537.11";
const char S44187[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.11 (KHTML, like Gecko) Ubuntu Chromium/23.0.1271.97 Chrome/23.0.1271.97 Safari/537.11";
const char S44188[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.11 (KHTML, like Gecko) Ubuntu Chromium/23.0.1271.97 Chrome/23.0.1271.97 Safari/537.11,gzip(gfe)";
const char S44189[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 925)";
const char S44190[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 925; Vodafone)";
const char S44191[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 925; ANZ892)";
const char S44192[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 925; Orange)";
const char S44193[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 925; BOUYGUES TELECOM)";
const char S44194[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 925; 1326-2)";
const char S44195[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315";
const char S44196[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S44197[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S44198[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S44199[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S44200[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S44201[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S44202[] = "Mozilla/5.0 (Linux; Android 4.1.9; GT-I9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S44203[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999V Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S44204[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-I535 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S44205[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S44206[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S44207[] = "Opera/8.01 (J2ME/MIDP; Opera Mini/2.0.4509/1724; en; U; ssr)";
const char S44208[] = "Opera/9.50 (J2ME/MIDP; Opera Mini/2/28.3165; U; en)";
const char S44209[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE51-2/411.34.001; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S44210[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE51-1/100.34.20; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S44211[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows NT; MS Search 5.0 Robot)";
const char S44212[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows NT; MS Search 6.0 Robot)";
const char S44213[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyEricssonST17i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44214[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; SonyEricssonST17a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44215[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST17i Build/4.1.B.0.587)";
const char S44216[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST17i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44217[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; ST17i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44218[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; ST17a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44219[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 810)";
const char S44220[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; LG-F160S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44221[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; LG-F160L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44222[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; LG-F160K Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44223[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-F160K Build/IML74K)";
const char S44224[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F160L Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44225[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-F160L Build/IML74K)";
const char S44226[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; LG Optimus LTE2 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S44227[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F160LV Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44228[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-F160L Build/IMM76I)";
const char S44229[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-F160S Build/IML74K)";
const char S44230[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; LG-F160 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44231[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-F160LV Build/IMM76I)";
const char S44232[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; LG-F160L Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?i Safari/534.30";
const char S44233[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; LG-F160 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S44234[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Version/3.1.2 Safari/525.21";
const char S44235[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.17";
const char S44236[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13";
const char S44237[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Version/3.1 Safari/525.19";
const char S44238[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Version/3.1 Safari/525.1";
const char S44239[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; ru-RU) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13.3";
const char S44240[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; ca-es) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20";
const char S44241[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13";
const char S44242[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en) AppleWebKit/525.9 (KHTML, like Gecko) Version/3.1 Safari/525.9";
const char S44243[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; pt-BR) AppleWebKit/525.19 (KHTML, like Gecko) Version/3.1.2 Safari/525.21";
const char S44244[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.26 (KHTML, like Gecko) Version/3.1 Safari/525.26";
const char S44245[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 10 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S44246[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 10 Build/JOP40F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S44247[] = "Mozilla/5.0 (Linux; Android 4.2; Nexus 10 Build/JVP15P) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S44248[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S44249[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S44250[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S44251[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989D Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S44252[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-T989 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S44253[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S44254[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S44255[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/21.0.1180.75 Safari/537.1";
const char S44256[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.3.6000 Chrome/21.0.1180.75 Safari/537.1";
const char S44257[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.22 (KHTML, like Gecko) Maxthon/4.0.4.3000 Chrome/25.0.1364.99 Safari/537.22";
const char S44258[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.22 (KHTML, like Gecko) Maxthon/4.0.5.2000 Chrome/25.0.1364.99 Safari/537.22";
const char S44259[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.22 (KHTML, like Gecko) Maxthon/4.0.5.2000 Chrome/25.0.1364.99 Safari/537.22";
const char S44260[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.22 (KHTML, like Gecko) Maxthon/4.0.6.3000 Chrome/25.0.1364.99 Safari/537.22";
const char S44261[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.22 (KHTML, like Gecko) Maxthon/4.0.6.3000 Chrome/25.0.1364.99 Safari/537.22";
const char S44262[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; HUAWEI MediaPad Build/HuaweiMediaPad) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44263[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; MediaPad Build/HuaweiMediaPad) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44264[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; SpringBoard Build/HuaweiMediaPad) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44265[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; HUAWEI MediaPad Build/HuaweiMediaPad)";
const char S44266[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; SpringBoard Build/HuaweiMediaPad)";
const char S44267[] = "ZTE-Z431/1.4.0 NetFront/4.2 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S44268[] = "ZTE-Z431/1.7.0 NetFront/4.2 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S44269[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9B206 Safari/7534.48.3";
const char S44270[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9B208 Safari/7534.48.3";
const char S44271[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9B179 Safari/7534.48.3";
const char S44272[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9B176 Safari/7534.48.3";
const char S44273[] = "Mozilla/5.0 (Android; Tablet; rv:16.0) Gecko/16.0 Firefox/16.0";
const char S44274[] = "Mozilla/5.0 (Android; Tablet; rv:16.0a2) Gecko/16.0a2 Firefox/16.0a2";
const char S44275[] = "Mozilla/5.0 (Android; Tablet; rv:16.0.1) Gecko/16.0.1 Firefox/16.0.1";
const char S44276[] = "Mozilla/5.0 (Android; Tablet; rv:16.0) Gecko/16.0 Firefox/16.0,gzip(gfe) (via translate.google.com)";
const char S44277[] = "Mozilla/5.0 (Android; Tablet; rv:16.0.2) Gecko/16.0.2 Firefox/16.0.2";
const char S44278[] = "Mozilla/5.0 (Android; Tablet; rv:16.0) Gecko/16.0 Firefox/16.0,gzip(gfe)";
const char S44279[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S44280[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xperia P Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S44281[] = "Mozilla/5.0 (Linux; Android 4.0.3; LT22i Build/6.1.1.B.1.75) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S44282[] = "Mozilla/5.0 (Linux; Android 4.0.4;  LT22i Build/6.1.B.0.544) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S44283[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S44284[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S44285[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; A110 Build/JZO54K)";
const char S44286[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; Micromax A110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44287[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Micromax A110 (Canvas 2) Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44288[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; A110 Build/JRO03C)";
const char S44289[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Micromax A110 Build/JRO03C)";
const char S44290[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Micromax A110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44291[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; Raven A110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44292[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A110 Canvas 2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44293[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; Micromax A110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44294[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44295[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; Micromax A110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44296[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/13.0.782.107; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)";
const char S44297[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/13.0.782.215)";
const char S44298[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/13.0.782.215)";
const char S44299[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/13.0.782.215) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.215 Safari/535.1";
const char S44300[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/13.0.782.215) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.215 Safari/535.1";
const char S44301[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/13.0.782.112) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1";
const char S44302[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/13.0.782.107) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1";
const char S44303[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/13.0.782.220) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1";
const char S44304[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/13.0.782.112) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1";
const char S44305[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/13.0.782.220) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1";
const char S44306[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/13.0.782.215; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; InfoPath.3; .NET4.0C; .NET4.0E; BOIE9;FRFR)";
const char S44307[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/13.0.782.107) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1";
const char S44308[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/13.0.782.220; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0)";
const char S44309[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/13.0.782.107; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3)";
const char S44310[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/13.0.782.107; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S44311[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/13.0.782.107; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3)";
const char S44312[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0 ; Claritybot)";
const char S44313[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S44314[] = "Mozilla/5.0 (Linux; Android 4.1.2; One X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S44315[] = "Mozilla/5.0 (Linux; Android 4.1.1; EndeavorU Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S44316[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S44317[] = "Mozilla/5.0 (Linux; Android 4.1.9; HTC One X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S44318[] = "Mozilla/4.02 [en] (X11; I; SunOS 5.6 sun4u)";
const char S44319[] = "Mozilla/3.01Gold (X11; I; SunOS 5.5.1 sun4m)";
const char S44320[] = "Mozilla/4.76 [ru] (X11; U; SunOS 5.7 sun4u)";
const char S44321[] = "Mozilla/4.76 [en] (X11; U; SunOS 5.7 sun4u)";
const char S44322[] = "Mozilla/4.05 [en] (X11; I; SunOS 4.1.4 sun4m)";
const char S44323[] = "Mozilla/3.0 (X11; I; SunOS 5.4 sun4m)";
const char S44324[] = "NCSA Mosaic/1.0 (X11;SunOS 4.1.4 sun4m)";
const char S44325[] = "Mozilla/4.61 [en] (X11; U; SunOS 5.6 sun4u)";
const char S44326[] = "Mozilla/5.0 (X11; U; SunOS sun4u; en-US; rv:1.2.1) Gecko/X";
const char S44327[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; WOW64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S44328[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; SV1)";
const char S44329[] = "Mozilla/4.0 (compatible;MSIE 6.0;Windows NT 6.1)";
const char S44330[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; Trident/5.0)";
const char S44331[] = "Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 6.1; Trident/5.0)";
const char S44332[] = "Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 6.1; WOW64; Trident/5.0)";
const char S44333[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S44334[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)";
const char S44335[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322)";
const char S44336[] = "Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 6.1; Win64; x64; Trident/5.0)";
const char S44337[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; MS-RTC LM 8)";
const char S44338[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; WOW64; SLCC2; .NET CLR 2.0.50727; .NET4.0C)";
const char S44339[] = "Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows NT 6.1; Trident/5.0)";
const char S44340[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:16.0) Gecko/X Firefox/16.0 SeaMonkey/2.13.2";
const char S44341[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:16.0) Gecko/X SeaMonkey/2.13.2";
const char S44342[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P5113 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S44343[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P5113 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S44344[] = "Opera/9.80 (Windows NT 6.0; U; en) Presto/2.2.15 Version/10.01";
const char S44345[] = "Opera/9.80 (Windows NT 6.0; U; de) Presto/2.2.15 Version/10.00";
const char S44346[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.3 (build 02564); ru) Presto/2.2.15 Version/10.00";
const char S44347[] = "Opera/10.00 (Windows NT 6.0; U; Edition Turbo; en) Presto/2.2.0";
const char S44348[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.0; ja) Opera 10.00";
const char S44349[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-R830 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 USCC-R830";
const char S44350[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-R830 Build/IMM76D)";
const char S44351[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.6) Gecko/X Firefox/2.0.0.6 Ara.com.tr AraBot 1.0";
const char S44352[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; ADR6300Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44353[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us;ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44354[] = "Mozilla/5.0 (Linux; U;Android 2.2; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44355[] = "Mozilla/5.0 (Linux; U; Android2.2; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44356[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S44357[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44358[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; ADR6300 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44359[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44360[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; ADR6300 Build/FRF91) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S44361[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; ADR6300 Build/FRF91)";
const char S44362[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; ADR6300 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S44363[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P925/V20p Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44364[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P925 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44365[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; LG-P925 Build/GRJ90)";
const char S44366[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P925/V20p Build/PromethiusProject v1.5.1 20P) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44367[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-mx; LG-P925 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Versi?n/4.0 Mobile Safari/533.1";
const char S44368[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S44369[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S44370[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S44371[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S44372[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S44373[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S44374[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S44375[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Twitter for iPhone";
const char S44376[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Twitter for iPhone";
const char S44377[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Twitter for iPhone";
const char S44378[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Twitter for iPhone";
const char S44379[] = "Mozilla/5.0 (Series40; Nokia308/05.85; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S44380[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; HTC HD2 Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44381[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC HD2 Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44382[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC HD2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44383[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; nl-nl; HTC HD2 Build/GRI54) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44384[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC HD2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44385[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HD2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44386[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC HD2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44387[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC HD2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44388[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC HD2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44389[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HD2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44390[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC HD2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44391[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HD2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44392[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC HD2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44393[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC HD2 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44394[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC HD2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44395[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ie; HTC HD2 Build/1.4.1-MIUIVANILLA) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44396[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HD2 Gingerbread Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44397[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; HTC HD2 Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44398[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC HD2 Build/FRF91)";
const char S44399[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC HD2 Build/GRI40)";
const char S44400[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC HD2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44401[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; HD2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44402[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HD2 Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44403[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; HTC HD2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44404[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC HD2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44405[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC HD2 Gingerbread Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44406[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC HD2 Build/FRF91)";
const char S44407[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HD2 Build/HSTR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44408[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC HD2 Build/JaguarMIUI-V21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44409[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.1; HTC HD2 Build/GRH78)";
const char S44410[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; HD2 Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44411[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC HD2 Build/JaguarMIUI-V14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44412[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC HD2 Build/JaguaraMIUI V-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44413[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; HD2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44414[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC HD2 Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S44415[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC HD2 Build/GRJ90)";
const char S44416[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; HD2 Build/GRK39F)";
const char S44417[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC HD2 Build/JaguarMIUI-V20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44418[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; T8585 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44419[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC HD2 Build/GWK74; MokeeOS) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44420[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; NexusHD2 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44421[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC HD2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44422[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:23.0) Gecko/X Firefox/23.0";
const char S44423[] = "Mozilla/5.0 (Windows NT 6.0; rv:23.0) Gecko/X Firefox/23.0,gzip(gfe)";
const char S44424[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; SAMSUNG; GT-I8750)";
const char S44425[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; SAMSUNG; SGH-T899M)";
const char S44426[] = "Opera/9.80 (S60; SymbOS; Opera Mobi/498; U; en-GB) Presto/2.4.18 Version/10.00";
const char S44427[] = "LG-GT550/V11d; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.18377; U; en-US) Opera 9.50";
const char S44428[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/6.5.27431/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S44429[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/6.5.27431/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S44430[] = "Mozilla/5.0 (Unknown; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) CasperJS/1.0.2+Phantomjs/1.9.0 Safari/534.34";
const char S44431[] = "Mozilla/5.0 (Unknown; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) CasperJS/1.0.2+Phantomjs/1.9.0 Safari/534.34";
const char S44432[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.3 Mobile/11A4414e Safari/8536.25";
const char S44433[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.5 Mobile/11A4435d Safari/8536.25";
const char S44434[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A465 Safari/8536.25";
const char S44435[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A4449d Safari/8536.25";
const char S44436[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A4449a Safari/8536.25";
const char S44437[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A466 Safari/8536.25";
const char S44438[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A501 Safari/8536.25";
const char S44439[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A470a Safari/8536.25";
const char S44440[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:11.0) Gecko/X Firefox/11.0";
const char S44441[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PantechP9060 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44442[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PantechP9060 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44443[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; PantechP9060 Build/GINGERBREAD)";
const char S44444[] = "Mozilla/5.0 (X11; U; OpenBSD i386; en-US; rv:1.9.2.8) Gecko/X Firefox/3.6.8";
const char S44445[] = "BlackBerry9330/5.0.0.913 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/104";
const char S44446[] = "SAMSUNG-C3300/C3300DDDJG2 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S44447[] = "SAMSUNG-GT-C3300K/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S44448[] = "Vodafone/1.0/SAMSUNG-GT-C3300i/BUKA1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S44449[] = "Vodafone/1.0/SAMSUNG-GT-C3300K/BUJG4 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S44450[] = "Vodafone/1.0/SAMSUNG-GT-C3300K/BUJH1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S44451[] = "Vodafone/1.0/SAMSUNG-GT-C3300K/BUJG6 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S44452[] = "Vodafone/1.0/SAMSUNG-GT-C3300K/BUJK1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S44453[] = "Vodafone/1.0/SAMSUNG-GT-C3300K/BUJI1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S44454[] = "Vodafone/1.0/SAMSUNG-GT-C3300K/PIJI1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S44455[] = "SAMSUNG-GT-C3300i/C3300iXXKJ2 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S44456[] = "Vodafone/1.0/SAMSUNG-GT-C3300K/AEJG4 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S44457[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S44458[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S44459[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S44460[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S44461[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S44462[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S44463[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S44464[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S44465[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S44466[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 1060) AppleWebKit/535.7 (KHTML like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77";
const char S44467[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7102 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44468[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7102 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S44469[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-N7102 Build/JRO03C)";
const char S44470[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; zh-tw; GT-N7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44471[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; GT-N7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44472[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-gb; GT-N7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44473[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; es-es; GT-N7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44474[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44475[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; vi-vn; GT-N7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44476[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; it-it; GT-N7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44477[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7102 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44478[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; fr-fr; GT-N7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44479[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N7102 Build/JZO54K)";
const char S44480[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; de-de; GT-N7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44481[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.9; GT-N7102 Build/IMM76D)";
const char S44482[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; zh-cn; GT-N7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44483[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7102 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S44484[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.4.154.25 Safari/525.19";
const char S44485[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.2.149.27 Safari/525.13";
const char S44486[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.3.154.0 Safari/525.19";
const char S44487[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/0.0.0.0 Safari/534.24";
const char S44488[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/0.0.0.0 Safari/534.24";
const char S44489[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; de) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.4.154.29 Safari/525.19";
const char S44490[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.3 (KHTML, like Gecko) Chrome/0.0.0.0 Safari/532.3";
const char S44491[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; de) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.2.149.27 Safari/525.13";
const char S44492[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/0.1.1.7 Safari/535.7";
const char S44493[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/28.0.1500.71) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36";
const char S44494[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/28.0.1500.72) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36";
const char S44495[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; chromeframe/28.0.1500.95)";
const char S44496[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/28.0.1500.95)";
const char S44497[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/28.0.1500.95)";
const char S44498[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/28.0.1500.95) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S44499[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; HPNTDFJS; chromeframe/28.0.1500.95)";
const char S44500[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-B5512 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44501[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-B5512B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44502[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-B5512 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S44503[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-B5512 Build/GINGERBREAD)";
const char S44504[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-B5512 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44505[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S44506[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S44507[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Sprint APC715CKT Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44508[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PH44100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44509[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; PH44100 Build/GRJ22)";
const char S44510[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-tw; HTC_EVO_Design_C715e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44511[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-tw; HTC EVO Design C715e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44512[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/20.0.1132.11) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.11 Safari/536.11";
const char S44513[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/20.0.1132.43) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.43 Safari/536.11";
const char S44514[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/20.0.1132.43) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.43 Safari/536.11";
const char S44515[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/20.0.1132.47) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11";
const char S44516[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/20.0.1132.47) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11";
const char S44517[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; chromeframe/20.0.1132.47)";
const char S44518[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/20.0.1132.57)";
const char S44519[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/20.0.1132.57)";
const char S44520[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/20.0.1132.57) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S44521[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/20.0.1132.57) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S44522[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/20.0.1132.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3)";
const char S44523[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/20.0.1132.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S44524[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/20.0.1132.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; Tablet PC 2.0)";
const char S44525[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/20.0.1132.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; MS-RTC LM 8; InfoPath.3)";
const char S44526[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/20.0.1132.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3)";
const char S44527[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; chromeframe/20.0.1132.57)";
const char S44528[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/20.0.1132.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0)";
const char S44529[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/20.0.1132.57; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)";
const char S44530[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/20.0.1132.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C)";
const char S44531[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/20.0.1132.57)";
const char S44532[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/20.0.1132.47)";
const char S44533[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-us; ZTE-BLADE Build/P729USV1.0.0B01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44534[] = "Mozilla/5.0 (Linux; U; Android 2.2; lt-lt; ZTE-BLADE Build/OPEN-MARKET-V1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44535[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; ZTE-BLADE Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44536[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; ZTE-BLADE Build/FRF91)";
const char S44537[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; ZTE-BLADE Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44538[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; ZTE-BLADE Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S44539[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; ZTE-BLADE Build/FRF91)";
const char S44540[] = "New-Sogou-Spider/1.0 (compatible; MSIE 5.5; Windows 98)";
const char S44541[] = "Sogou-Test-Spider/4.0 (compatible; MSIE 5.5; Windows 98)";
const char S44542[] = "Mozilla/5.0 (Windows NT 5.1; rv:20.0) Gecko/X SeaMonkey/2.17.1";
const char S44543[] = "Mozilla/5.0 (Windows NT 5.1; rv:20.0) Gecko/X Firefox/20.0 SeaMonkey/2.17.1";
const char S44544[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:20.0) Gecko/X Firefox/20.0 SeaMonkey/2.17.1";
const char S44545[] = "GoogleEarth/6.2.1.6014(Windows;Microsoft Windows (6.1.7601.1);en;kml:2.2;client:Free;type:default)";
const char S44546[] = "Mozilla/5.0 (Windows; N; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.2.2.6613 Safari/532.4";
const char S44547[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);en;kml:2.2;client:Free;type:default)";
const char S44548[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7600.0);en;kml:2.2;client:Free;type:default)";
const char S44549[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);de;kml:2.2;client:Free;type:default)";
const char S44550[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);nl;kml:2.2;client:Free;type:default)";
const char S44551[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);fr;kml:2.2;client:Free;type:default)";
const char S44552[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);es-419;kml:2.2;client:Free;type:default)";
const char S44553[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);it;kml:2.2;client:Free;type:default)";
const char S44554[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);ru;kml:2.2;client:Free;type:default)";
const char S44555[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);pt;kml:2.2;client:Free;type:default)";
const char S44556[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);es;kml:2.2;client:Free;type:default)";
const char S44557[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);tr;kml:2.2;client:Free;type:default)";
const char S44558[] = "GoogleEarth/6.2.0.5905(Windows;Microsoft Windows (6.1.7601.1);de;kml:2.2;client:Free;type:default)";
const char S44559[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);ko;kml:2.2;client:Free;type:default)";
const char S44560[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);en;kml:2.2;client:Pro;type:default)";
const char S44561[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-US670 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44562[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-US670 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44563[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-us; LG-US670 Build/FRF91) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S44564[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; LG-US670 Build/FRF91)";
const char S44565[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-US670 Build/FRG83G)";
const char S44566[] = "Mozilla/5.0 (PlayBook; U; RIM Tablet OS 2.0.0; en-US) AppleWebKit/535.1+ (KHTML, like Gecko) Version/7.2.0.0 Safari/535.1+";
const char S44567[] = "Mozilla/5.0 (PlayBook; U; RIM Tablet OS 2.0.0; en-US) AppleWebKit/535.8+ (KHTML, like Gecko) Version/7.2.0.0 Safari/535.8+";
const char S44568[] = "Mozilla/5.0 (PlayBook; U; RIM Tablet OS 2.0.1; en-US) AppleWebKit/535.8+ (KHTML, like Gecko) Version/7.2.0.1 Safari/535.8+";
const char S44569[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Safari/535.19 Silk-Accelerated=true";
const char S44570[] = "Opera/9.80 (X11; Linux i686) Presto/2.12.388 Version/12.14";
const char S44571[] = "Mozilla/5.0 (X11; Linux i686; rv:2.0) Gecko/X Firefox/4.0 Opera 12.14";
const char S44572[] = "Mozilla/5.0 (compatible; MSIE 9.0; X11; Linux i686) Opera 12.14";
const char S44573[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-E973 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44574[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F200L Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44575[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F200K Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44576[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F200S Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44577[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-E973/E97310f Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44578[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-E973/E97310h Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44579[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-E973 Build/IMM76L)";
const char S44580[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-F200S Build/IMM76L)";
const char S44581[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-F200L Build/IMM76L)";
const char S44582[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-E973/E97310c Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44583[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:2.0.1) Gecko/X Firefox/4.0.1";
const char S44584[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i917)";
const char S44585[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i917.)";
const char S44586[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0) SAMSUNG;SGH-i917";
const char S44587[] = "Mozilla/4.0 (compatible: MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i917)";
const char S44588[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; CETUS)";
const char S44589[] = "Mozilla/4.0 (compatible: MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; WpsLondonTest; SAMSUNG; SGH-i917)";
const char S44590[] = "Mozilla/5.0 (Android; Tablet; rv:15.0) Gecko/15.0 Firefox/15.0a2";
const char S44591[] = "Mozilla/5.0 (Android; Tablet; rv:15.0) Gecko/15.0 Firefox/15.0";
const char S44592[] = "Mozilla/5.0 (Android; Tablet; rv:15.0) Gecko/15.0 Firefox/15.0a1";
const char S44593[] = "Mozilla/5.0 (Android; Tablet; rv:15.0) Gecko/15.0 Firefox/15.0.1";
const char S44594[] = "Mozilla/5.0 (Android; Tablet; rv:15.0.1) Gecko/15.0.1 Firefox/15.0.1";
const char S44595[] = "Mozilla/5.0 (Android; Tablet; rv:15.0) Gecko/15.0 Firefox/15.0.1,gzip(gfe) (via translate.google.com)";
const char S44596[] = "Mozilla/5.0 (Android; Tablet; rv:15.0) Gecko/15.0 Firefox/15.0.1,gzip(gfe)";
const char S44597[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-i705 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S44598[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-i705 Build/JZO54K)";
const char S44599[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.127 Safari/534.16";
const char S44600[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16";
const char S44601[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16";
const char S44602[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S44603[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S44604[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S44605[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16";
const char S44606[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.82 Safari/534.16";
const char S44607[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I5800 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44608[] = "Mozilla/5.0 (Linux; U; Android 2.2; hr-hr; GT-I5800 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S44609[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I5800 Build/FROYO)";
const char S44610[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-I5800D Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44611[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I5800D Build/FROYO)";
const char S44612[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; LIFETAB_P9514 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S44613[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; LIFETAB_P9514 Build/IML74K)";
const char S44614[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I437 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44615[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SAMSUNG-SGH-I437 Build/IMM76D)";
const char S44616[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94; ru) Presto/2.10.229 Version/11.61";
const char S44617[] = "Opera/9.80 (Windows NT 6.0; U; en) Presto/2.10.229 Version/11.61,gzip(gfe) (via translate.google.com)";
const char S44618[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; en) Opera 11.61";
const char S44619[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.4 (build 02647); ru) Presto/2.10.229 Version/11.61";
const char S44620[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4876); ru) Presto/2.10.229 Version/11.61";
const char S44621[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5231); ru) Presto/2.10.229 Version/11.61";
const char S44622[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5244); ru) Presto/2.10.229 Version/11.61";
const char S44623[] = "Opera/9.80 (Windows NT 6.0; U; en) Presto/2.10.229 Version/11.61";
const char S44624[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.61";
const char S44625[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.61";
const char S44626[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.61";
const char S44627[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.61";
const char S44628[] = "Mozilla/5.0 (Windows NT 6.0; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.61";
const char S44629[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4664); ru) Presto/2.10.229 Version/11.61";
const char S44630[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03790); ru) Presto/2.10.229 Version/11.61";
const char S44631[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.61";
const char S44632[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5196); ru) Presto/2.10.229 Version/11.61";
const char S44633[] = "Opera/9.80 (Windows NT 6.1; U; &quot;Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0); en) Presto/2.10.229 Version/11.61";
const char S44634[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4157); ru) Presto/2.10.229 Version/11.61";
const char S44635[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4598); ru) Presto/2.10.229 Version/11.61";
const char S44636[] = "FeeddlerRSS 1.12.5 (iPhone; iPhone OS 6.1; en_US)";
const char S44637[] = "FeeddlerRSS 1.12.5 (iPhone; iPhone OS 6.1.2; en_US)";
const char S44638[] = "Appcelerator Titanium/1.7.2 (iPhone/6.1.3; iPhone OS; en_US;)";
const char S44639[] = "FeeddlerRSS 1.12.5 (iPhone; iPhone OS 6.1.3; en_US)";
const char S44640[] = "FeeddlerRSS 1.12.5 (iPhone; iPhone OS 6.1.4; en_US)";
const char S44641[] = "Miu Ptt 1.9.2 (iPhone; iPhone OS 6.1.3; en_US)";
const char S44642[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36";
const char S44643[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36";
const char S44644[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.0.1-x64";
const char S44645[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.0.1";
const char S44646[] = "Opera/9.80 (iPhone; Opera Mini/6.5.1.23995/26.1283; U; fr) Presto/2.8.119 Version/10.54";
const char S44647[] = "Opera/9.80 (iPhone; Opera Mini/6.5.2/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S44648[] = "Opera/9.80 (BlackBerry; Opera Mini/6.1.25376/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S44649[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F180K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44650[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F180L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44651[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F180S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44652[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F180L Build/JZO54K)";
const char S44653[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F180S Build/JZO54K)";
const char S44654[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F180K/F180K20k Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44655[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F180K Build/JZO54K)";
const char S44656[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; LG-F180 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44657[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F180K/F180K20n Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44658[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/535.9 (KHTML, like Gecko) Version/5.2 Safari/535.9";
const char S44659[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/535.18.5 (KHTML, like Gecko) Version/5.2 Safari/535.18.5";
const char S44660[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.2.3 (KHTML, like Gecko) Version/5.2 Safari/536.2.3";
const char S44661[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.5 (KHTML, like Gecko) Version/5.2 Safari/536.5";
const char S44662[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.6.1 (KHTML, like Gecko) Version/5.2 Safari/536.6.1";
const char S44663[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.8.5 (KHTML, like Gecko) Version/5.2 Safari/536.8.5";
const char S44664[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.9 (KHTML, like Gecko) Version/5.2 Safari/536.9";
const char S44665[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.10.3 (KHTML, like Gecko) Version/5.2 Safari/536.10.3";
const char S44666[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.11 (KHTML, like Gecko) Version/5.2 Safari/536.11";
const char S44667[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 108) AppleWebKit/535.18.5 (KHTML like Gecko) Version/5.2 Safari/535.18.5";
const char S44668[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 108) AppleWebKit/536.10.3 (KHTML like Gecko) Version/5.2 Safari/536.10.3";
const char S44669[] = "Mozilla/5.0 (Symbian/3; Series60/5.4 Nokia603/112.010.1404; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.2.1.20 Mobile Safari/535.1 3gpp-gba";
const char S44670[] = "Mozilla/5.0 (Symbian/3; Series60/5.3 Nokia603/111.020.0310; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.4.1.14 Mobile Safari/533.4 3gpp-gba";
const char S44671[] = "Mozilla/5.0 (Symbian/3; Series60/5.5 Nokia603/113.010.1506; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.2.21 Mobile Safari/535.1 3gpp-gba";
const char S44672[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Safari/535.19 Silk-Accelerated=true";
const char S44673[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-au; MotoA953 Build/MILS2_U6_2.2.19) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44674[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-sg; MotoA953 Build/MILS2_U6_2.4.18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44675[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MotoA953 Build/MILS2_U6_2.4.16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44676[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MotoA953 Build/MILS2_U6_2.4.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44677[] = "MOT-A953/Blur_Version.3.18.3.A953.AmericaMovil.en.MX Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; A953 Build/MILA2_U6_3.18.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44678[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; A953Blur Build/S274) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44679[] = "MOT-A953/Blur_Version.3.9.0.A953.Brasil.en.BR Mozilla/5.0 (Linux; U; Android 2.2; en-us; A953 Build/MILA2_U6_3.9.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44680[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; MotoA953 Build/MILS2_U6_2.2.16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44681[] = "Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; MotoA953 Build/MILS2_U6_2.3.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44682[] = "Mozilla/5.0 (Linux; U; Android 2.2; de-de; MotoA953 Build/MS2_2.2.13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44683[] = "MOT-A953/Blur_Version.3.12.0.A953.Brasil.en.BR Mozilla/5.0 (Linux; U; Android 2.2; pt-br; A953 Build/MILA2_U6_3.12.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44684[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MotoA953 Build/S274) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44685[] = "MOT-A953/Blur_Version.3.25.0.A953.Latam.en.01 Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; A953 Build/MILA2_U6_3.25.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44686[] = "MOT-A953/Blur_Version.3.21.0.A953.TIM.en.BR Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A953 Build/MILA2_U6_3.21.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44687[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; MotoA953 Build/MILS2_U6_2.4.19) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44688[] = "MOT-A953/Blur_Version.3.25.1.A953.Latam.en.01 Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; A953 Build/MILA2_U6_3.25.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44689[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MotoA953 Build/STAB2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44690[] = "MOT-A953/Blur_Version.3.18.4.A953.Latam.en.01 Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; A953 Build/MILA2_U6_3.18.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44691[] = "MOT-A953/Blur_Version.3.18.100.A953.Latam.en.01 Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; A953 Build/MILA2_U6_3.18.100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44692[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MotoA953 Build/MILS2_U6_2.4.24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44693[] = "MOT-A953/Blur_Version.3.18.1.A953.AmericaMovil.en.01 Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; A953 Build/MILA2_U6_3.18.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44694[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MotoA953 Build/MILS2_U6_2.5.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44695[] = "MOT-A953/Blur_Version.3.15.3.A953.Movistar.en.01 Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; A953 Build/MILA2_U6_3.15.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44696[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; he-il; MotoA953 Build/MILS2_U6_2.5.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44697[] = "MOT-A953/Blur_Version.3.24.0.A953.TIM.en.BR Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; A953 Build/MILA2_U6_3.24.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44698[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MotoA953 Build/MILS2_U6_2.2.20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44699[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; A953Blur Build/S260) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44700[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; A953 Build/MILA2_U6_3.12.0)";
const char S44701[] = "MOT-A953/Blur_Version.3.23.0.A953.Latam.en.01 Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; A953 Build/MILA2_U6_3.23.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44702[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; MotoA953 Build/MILS2_U6_2.4.18)";
const char S44703[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; MotoA953 Build/MILS2_U6_2.4.24)";
const char S44704[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; MotoA953 Build/MILS2_U6_2.2.16) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S44705[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; MotoA953 Build/MILS2_U6_2.2.16) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S44706[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG-SGH-I537 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S44707[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SHW-M180S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44708[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; SHW-M180L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44709[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SHW-M180S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44710[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; SHW-M180L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44711[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; SHW-M180W Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44712[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; SHW-M180K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44713[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHW-M180K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44714[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; SHW-M180K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44715[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M180S Build/GINGERBREAD)";
const char S44716[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M180S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44717[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHW-M180L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44718[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHW-M180S Build/GINGERBREAD)";
const char S44719[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHW-M180W Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44720[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SHW-M180S Build/GINGERBREAD)";
const char S44721[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M180L Build/GINGERBREAD)";
const char S44722[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; OMNIA7)";
const char S44723[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; OMNIA7; Orange)";
const char S44724[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; GT-I8700)";
const char S44725[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; OMNIA7)";
const char S44726[] = "Mozilla/5.0 (Windows NT 6.0; rv:12.0) Gecko/X Firefox/12.0,gzip(gfe) (via translate.google.com)";
const char S44727[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:12.0) Gecko/X Firefox/12.0";
const char S44728[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:12.0) Gecko/X Firefox/12.0 Firefox/12.0";
const char S44729[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows XP) Opera 6.0  [en]";
const char S44730[] = "Mozilla/5.0 (Windows XP; U) Opera 6.01 [en]";
const char S44731[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows XP) Opera 6.05 [en]";
const char S44732[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows XP) Opera 6.06 [fr]";
const char S44733[] = "Mozilla/5.0 (Windows XP; U) Opera 6.06 [en]";
const char S44734[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows XP) Opera 6.01  [ru]";
const char S44735[] = "Opera/6.04 (Windows XP; U) [en]";
const char S44736[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows XP) Opera 6.04 [fr]";
const char S44737[] = "Opera/6.05 (Windows XP; U) [de]";
const char S44738[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows XP) Opera 6.03 [en]";
const char S44739[] = "Opera/6.01 (Windows XP; U) [ru]";
const char S44740[] = "Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.1) Opera 6.02 [en]";
const char S44741[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VS930 4G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44742[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; VS930 4G Build/IMM76I)";
const char S44743[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.53 Safari/534.3";
const char S44744[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S44745[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-i9505 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S44746[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505G Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S44747[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; htc_h1000c Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44748[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S44749[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S44750[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S44751[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.0 Safari/534.24";
const char S44752[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_0; en-US) AppleWebKit/534.21 (KHTML, like Gecko) Chrome/11.0.678.0 Safari/534.21";
const char S44753[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S44754[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S44755[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S44756[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7; en-us) AppleWebKit/534.20.8 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.13";
const char S44757[] = "Mozilla/5.0 (Series40; Nokia110/03.04; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.5.16.6";
const char S44758[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I747 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S44759[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I747 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S44760[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L710 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S44761[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9305 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S44762[] = "Mozilla/5.0 (Linux; Android 4.2.1; SAMSUNG-SGH-I747 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S44763[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530U Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S44764[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-I747 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S44765[] = "Mozilla/5.0 (Linux; Android 4.2.1; SPH-L710 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S44766[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530C Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S44767[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.1.141) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S44768[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-tw; HTC Aria A6380 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44769[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HTC Aria Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44770[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-au; HTC_Aria_A6380 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44771[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC_Gratia_A6380 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44772[] = "Mozilla/5.0 (Linux; U; Android 2.2; sv-se; HTC Gratia A6380 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44773[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; S31HT Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44774[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HTC_Gratia_A6380 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44775[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; de-at; HTC Aria Build/FRG83G) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S44776[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; HTC Aria Build/FRG83G)";
const char S44777[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; HTC Aria A6380 Build/FRF91)";
const char S44778[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; HTC Gratia A6380 Build/FRF91)";
const char S44779[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Aria A6380 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S44780[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Aria A6380 Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S44781[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2) Presto/2.12.388 Version/12.14";
const char S44782[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P870 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44783[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P870/P87020d Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44784[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P870 Build/JZO54K)";
const char S44785[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Event Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44786[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Event Build/IML77)";
const char S44787[] = "GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (6.1.7601.1);en;kml:2.2;client:Free;type:default)";
const char S44788[] = "GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (6.1.7601.1);de;kml:2.2;client:Free;type:default)";
const char S44789[] = "GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (6.1.7600.0);en;kml:2.2;client:Free;type:default)";
const char S44790[] = "GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (6.1.7601.1);es-419;kml:2.2;client:Free;type:default)";
const char S44791[] = "GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (6.1.7601.1);fr;kml:2.2;client:Free;type:default)";
const char S44792[] = "GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (6.1.7601.1);es;kml:2.2;client:Free;type:default)";
const char S44793[] = "GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (6.1.7601.1);zh-Hans;kml:2.2;client:Free;type:default)";
const char S44794[] = "GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (6.1.7601.1);en;kml:2.2;client:Pro;type:default)";
const char S44795[] = "GoogleEarth/7.1.1.1871(Windows;Microsoft Windows (6.1.7601.1);en;kml:2.2;client:Free;type:default)";
const char S44796[] = "Mozilla/5.0 (Windows; N; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/7.1.1.1888 Safari/532.4";
const char S44797[] = "GoogleEarth/7.1.1.1888(Windows;Microsoft Windows (6.1.7601.1);en;kml:2.2;client:Free;type:default)";
const char S44798[] = "GoogleEarth/7.1.1.1888(Windows;Microsoft Windows (6.1.7601.1);de;kml:2.2;client:Free;type:default)";
const char S44799[] = "GoogleEarth/7.1.1.1888(Windows;Microsoft Windows (6.1.7600.0);en;kml:2.2;client:Free;type:default)";
const char S44800[] = "GoogleEarth/7.1.1.1888(Windows;Microsoft Windows (6.1.7601.1);es-419;kml:2.2;client:Free;type:default)";
const char S44801[] = "GoogleEarth/7.1.1.1888(Windows;Microsoft Windows (6.1.7601.1);es;kml:2.2;client:Free;type:default)";
const char S44802[] = "GoogleEarth/7.1.1.1888(Windows;Microsoft Windows (6.1.7601.1);fr;kml:2.2;client:Free;type:default)";
const char S44803[] = "Mozilla/5.0 (Linux; U;Android 1.5; en-us; MB501 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S44804[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB501 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 MobileSafari/525.20.1";
const char S44805[] = "Mozilla/5.0 (Linux; U; Android1.5; en-us; MB501 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S44806[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB501Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S44807[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us;MB501 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S44808[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-; MB501 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S44809[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB501 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S44810[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB501 Build/CUPCAKE) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S44811[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; SonyEricssonX10a Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44812[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; X10a Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44813[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ph; SonyEricssonX10a Build/3.0.1.G.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44814[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; X10 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44815[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; X10a Build/3.0.1.G.0.75)";
const char S44816[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; X10 TripNMiUI-1.8.26 Build/TripNMiUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44817[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; X10 Mini Pro Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44818[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; X10 Build/COB017) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44819[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; X10 Build/COB020) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44820[] = "Mozilla/5.0 (Linux; U; Android 2.3.9; en-us; X10 Build/6.0.B.0.743) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44821[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SMART X10 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44822[] = "Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko/X Firefox/11.0 CometBird/11.0";
const char S44823[] = "Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko/X Firefox/11.0 CometBrowser/11.0";
const char S44824[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:11.0) Gecko/X Firefox/11.0 CometBird/11.0";
const char S44825[] = "Mozilla/5.0 (Linux; Android 4.0.3; Sony Tablet S Build/TISU0143) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S44826[] = "Mozilla/5.0 (Linux; Android 4.0.3;  Sony Tablet S Build/TISU0143) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S44827[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8000 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S44828[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8020 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S44829[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8020 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S44830[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8000 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S44831[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8005 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S44832[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-P7501 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44833[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7501/P7501BOKL2 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44834[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7501/P7501BOLA1 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44835[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7501 Build/HTJ85B)";
const char S44836[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7501/P7501BUKL3 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44837[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P7501 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44838[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-P7501 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S44839[] = "Logos/0.9.4 (Linux; U; Android 3.2; en_US; GT-P7501 Build/HTJ85B.XXKJ7)";
const char S44840[] = "Logos/2.2.3 (Linux; U; Android 3.2; en-GB; GT-P7501 Build/HTJ85B.XXLB1)";
const char S44841[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300T Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S44842[] = "Mozilla/5.0 (Linux; Android 4.2.2; TF300T Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S44843[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300TG Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S44844[] = "Mozilla/5.0 (Linux; Android 4.2.1; es-US; ASUS Transformer Pad TF300T Build/JOP40D) AppleWebKit/537.22 (KHTML like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S44845[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300TL Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S44846[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/27.0.1453.94) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36";
const char S44847[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/27.0.1453.110) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36";
const char S44848[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/27.0.1453.116)";
const char S44849[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/27.0.1453.116) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S44850[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; chromeframe/27.0.1453.116)";
const char S44851[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; chromeframe/27.0.1453.116)";
const char S44852[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LG-P990 Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44853[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; LG-P990 Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44854[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-P990 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44855[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P990 Build/IMM76L)";
const char S44856[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; LG-P990/P99030a Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44857[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-P990hN Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44858[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en; LG-P990 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44859[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; LG-P990/P99030b Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44860[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/2";
const char S44861[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/530.5 (KHTML, like Gecko) Chrome/2.0.172.37 Safari/530.5";
const char S44862[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/530.6 (KHTML like Gecko) Chrome/2.0.174.0 Safari/530.6";
const char S44863[] = "Mozilla/6.0 (Windows; U; Windows NT 6.0; en-US) Gecko/X Chrome/2.0.172.6 Safari/530.7";
const char S44864[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.10 (KHTML, like Gecko) Chrome/2.0.1.3 Safari/536.10";
const char S44865[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; AppleWebKit/530.6; KHTML?? like Gecko; Chrome/2.0.174.0 Safari/530.6)";
const char S44866[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LG-E730/V09a Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S44867[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; LG-E730/V10b Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S44868[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-E730 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S44869[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; LG-E730f Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S44870[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; LG-E730/V10a Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S44871[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-E730 Build/GRJ22)";
const char S44872[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; LG-E730/V10j Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S44873[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ro-ro; LG-E730/V10f Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S44874[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; LG-E730/V10i Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S44875[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; LG-E730f Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Versi?n/4.0 Mobile Safari/533.1";
const char S44876[] = "NetFront/4.2 (BMP 1.0.4; U; en-us; LG; NetFront/4.2/AMB) Sprint LN272 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S44877[] = "NetFront/4.2 (BMP 1.0.4; U; en-us; LG; NetFront/4.2/AMB) Boost LG272 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S44878[] = "NetFront/4.2 (BMP 1.0.4; U; en-us; LG; NetFront/4.2/AMB) Sprint LN272S MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S44879[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S44880[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S44881[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S44882[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S44883[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S44884[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S44885[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.2 (KHTML, like Gecko) Chrome/6.0.451.0 Safari/534.2";
const char S44886[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.2 (KHTML, like Geckto) Chrome/6.0.451.0 Safari/534.2";
const char S44887[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S44888[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.456.0 Safari/534.3";
const char S44889[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/534.3 (KHTML	 like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S44890[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44891[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Xoom Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44892[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Xoom Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44893[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; Xoom Build/HLK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44894[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; Xoom Build/HTJ85B)";
const char S44895[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; Xoom Build/HLK42B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44896[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; Xoom Build/HLK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44897[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; Xoom Build/HTK75D)";
const char S44898[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.4; Xoom Build/HLK75F)";
const char S44899[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLJ76) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44900[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLJ86) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44901[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HTJ85) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44902[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLK08) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44903[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLK11) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44904[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HTJ77) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44905[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLK36B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44906[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLJ77B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44907[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLK42) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44908[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Xoom Build/HTK55B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44909[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLK33C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44910[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HTJ78) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44911[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLK01B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44912[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLK25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44913[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLJ83) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44914[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; Xoom Build/HLK42C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44915[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.2; Xoom Build/HLK42B)";
const char S44916[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; Xoom Build/HLK42E) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44917[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; Xoom Build/HLK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44918[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Xoom Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44919[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Xoom Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S44920[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HTJ74) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44921[] = "Mozilla/5.0 (Linux; U; Android 3.2.4; en-us; Xoom Build/HLK75F) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S44922[] = "Mozilla/5.0 (Linux; U; Android 3.2.4; en-us; Xoom Build/HLK75E) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44923[] = "Mozilla/5.0 (Linux; U; Android 3.2.4; en-us; Xoom Build/HLK75F) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44924[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLK28B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44925[] = "Mozilla/5.0 (Linux; U; Android 3.2.6; en-us; Xoom Build/HLK75H) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44926[] = "Mozilla/5.0 (Linux; U; Android 3.2.6; en-us; Xoom Build/HLK75G) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44927[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; ru-; Xoom Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44928[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Xoom Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44929[] = "Mozilla/5.0 (Linux; U; Android 3.2.6; en-us; Xoom Build/HLK75H) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S44930[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.6; Xoom Build/HLK75H)";
const char S44931[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; MZ505 Build/Moto_MZ505) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S44932[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; IDEOS S7 Slim Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44933[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; IDEOS S7 Slim Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44934[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; IDEOS S7 Slim Build/FRG83G)";
const char S44935[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; IDEOS S7 Slim Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S44936[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; IDEOS S7 Slim Build/pirate) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44937[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN; rv:1.8.0.11) Gecko/X Firefox/1.5.0.11; 360spider";
const char S44938[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S44939[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2C Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S44940[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2S Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S44941[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2SC Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S44942[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2A Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S44943[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; MI 2 Build/miui.es) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S44944[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; MI 2 Build/miui.es JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S44945[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; MI 2S Build/miui.es JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S44946[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; MI 2S Build/JRO03L) AppleWebKit/534.30 (KHTML, zoals Gecko) Versie/4.0 MobielSafari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S44947[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; MI 2A Build/miui.es JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S44948[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; MI 2 Build/JRO03L) AppleWebKit/534.30 (KHTML, zoals Gecko) Versie/4.0 MobielSafari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S44949[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MI 2 Build/WiUi) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S44950[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 520; Vodafone)";
const char S44951[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 520; Orange)";
const char S44952[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia 520)";
const char S44953[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 520; ANZ914)";
const char S44954[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 520; Meteor)";
const char S44955[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia 520T; CMCC)";
const char S44956[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 520; ANZ915)";
const char S44957[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 520),gzip(gfe) (via docs.google.com/viewer)";
const char S44958[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.8.1b2) Gecko/X Firefox/2.0b2";
const char S44959[] = "Mozilla/5.0 (Windows NT 6.0; U; tr; rv:1.8.1) Gecko/X Firefox/2.0.0";
const char S44960[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S44961[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I535 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S44962[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9300 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S44963[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T999 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S44964[] = "Mozilla/5.0 (Linux; Android 4.2.1; SCH-I535 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S44965[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-T999 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S44966[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I535 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S44967[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S44968[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T999 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S44969[] = "Opera/9.80 (Windows NT 6.0; WOW64) Presto/2.12.388 Version/12.13";
const char S44970[] = "Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.13";
const char S44971[] = "Opera/9.80 (Windows NT 6.0; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.13";
const char S44972[] = "Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.13,gzip(gfe)";
const char S44973[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.13";
const char S44974[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.13";
const char S44975[] = "Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.13";
const char S44976[] = "Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.13";
const char S44977[] = "Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.13";
const char S44978[] = "Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.13";
const char S44979[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; K080 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44980[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; K080 Build/GRI40)";
const char S44981[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Vox Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S44982[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; Vox Build/GRJ22)";
const char S44983[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19";
const char S44984[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19";
const char S44985[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.1.249.1064 Safari/532.5";
const char S44986[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.78 Safari/532.5";
const char S44987[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT30at Build/9.1.F.1.120) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44988[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT30at Build/9.1.F.1.120) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44989[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LT30at Build/9.1.F.1.120)";
const char S44990[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT30at Build/9.1.F.1.119) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44991[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SonyLT30at Build/9.1.F.1.120) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S44992[] = "SonyLT30at Build/9.1.F.1.120 stagefright/1.2 (Linux;Android 4.1.2)";
const char S44993[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S44994[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One 801e Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S44995[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTCONE Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S44996[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S44997[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC_PN071 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S44998[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTCONE Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S44999[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC_PN071 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S45000[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I927 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45001[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I927 Build/IMM76D)";
const char S45002[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ZTE-Z990 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45003[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ZTE-Z990 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45004[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; ZTE-Z990 Build/GRJ22)";
const char S45005[] = "Mozilla/5.0 (Linux; U; Android Gingerbread; en-us; ZTE-Z990 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45006[] = "Mozilla/5.0 (MeeGo; NokiaN9) AppleWebKit/534.13 (KHTML, like Gecko) NokiaBrowser/8.5.0 Mobile Safari/534.13";
const char S45007[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; de-at; HTC Desire Z  1.82.166.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45008[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; HTC Desire Z 1.85.861.4 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45009[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; de-ch; HTC Desire Z 1.82.166.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45010[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Desire Z 1.82.161.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45011[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DL DesireZ v3.4 final Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45012[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; HTC Desire Z 1.85.861.3 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45013[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; HTC-A7275 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45014[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC_DesireZ_A7272 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45015[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Desire Z 1.34.161.6 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45016[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Desire Z 1.34.161.6 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45017[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Desire Z 1.72.161.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45018[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; de-ch; HTC Desire Z 1.72.166.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45019[] = "Mozilla/5.0 (Linux; U; Android 2.2; de-at; HTC Desire Z 1.34.166.5 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45020[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC_DesireZ_A7272 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45021[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; HTC_DesireZ-orange-LS Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45022[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-ca; HTC-A7275 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45023[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC_DesireZ_A7272 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45024[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; de-ch; HTC Desire Z 1.34.166.5 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45025[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I757M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45026[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I757M Build/IMM76D)";
const char S45027[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SGH-I757M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45028[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 7 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45029[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 7 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45030[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45031[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45032[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 7 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45033[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45034[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 7 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45035[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45036[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/4.60; CyanogenMod-4.60) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45037[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U8860 Build/HuaweiU8860) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45038[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; U8860 Build/HuaweiU8860) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S45039[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; U8860 Build/HuaweiU8860)";
const char S45040[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; U8860 Build/HuaweiU8860) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S45041[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; U8860 Build/HuaweiU8860) AppleWebKit/534.30 (KHTML, like Gecko) Phi?n b?n/4.0 Mobile Safari/534.30";
const char S45042[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/11.0.696.57)";
const char S45043[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/11.0.696.65)";
const char S45044[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/11.0.696.68; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S45045[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/11.0.696.71) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24";
const char S45046[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/11.0.696.57) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.57 Safari/534.24";
const char S45047[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/11.0.696.57) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.57 Safari/534.24";
const char S45048[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/11.0.696.65) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.65 Safari/534.24";
const char S45049[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/11.0.696.68) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S45050[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/11.0.696.68) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S45051[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/11.0.696.71) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24";
const char S45052[] = "Mozilla/5.0 (Linux; Android 4.0.4; BNTV400 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S45053[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-I605 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S45054[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S45055[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-I605 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S45056[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S45057[] = "Mozilla/5.0 (Windows; N; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.1.0.5001 Safari/532.4";
const char S45058[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);en-CA;kml:2.2;client:Free;type:default)";
const char S45059[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);de-DE;kml:2.2;client:Free;type:default)";
const char S45060[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);fr-FR;kml:2.2;client:Free;type:default)";
const char S45061[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);it-IT;kml:2.2;client:Free;type:default)";
const char S45062[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7600.0);es-419;kml:2.2;client:Free;type:default)";
const char S45063[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);en-US;kml:2.2;client:Free;type:default)";
const char S45064[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);en;kml:2.2;client:Free;type:default)";
const char S45065[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);en-GB;kml:2.2;client:Free;type:default)";
const char S45066[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7600.0);en-GB;kml:2.2;client:Free;type:default)";
const char S45067[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);de-CH;kml:2.2;client:Free;type:default)";
const char S45068[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);en-AU;kml:2.2;client:Free;type:default)";
const char S45069[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7600.0);en-US;kml:2.2;client:Free;type:default)";
const char S45070[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);ru-RU;kml:2.2;client:Free;type:default)";
const char S45071[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);es-419;kml:2.2;client:Free;type:default)";
const char S45072[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);nl-NL;kml:2.2;client:Free;type:default)";
const char S45073[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17";
const char S45074[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17";
const char S45075[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Safari/535.19 Silk-Accelerated=true";
const char S45076[] = "Nokia311/5.0 (03.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S45077[] = "Nokia311/5.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 Nokia311/05.92 CLDC-1.1";
const char S45078[] = "Mozilla/5.0 (X11; Linux x86_64; rv:17.0) Gecko/17.0 Thunderbird/17.0";
const char S45079[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.1 (KHTML, like Gecko) CEF/0.0.0.0 Safari/534.1";
const char S45080[] = "Mozilla/5.0 (Windows NT 5.1.1; U; en) AppleWebKit/535.2 (KHTML, like Gecko) xp/234.76 Safari/535.2 01/07";
const char S45081[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/528+ (KHTML, like Gecko, Safari/528.0) Lunascape/5.0.2.0";
const char S45082[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; fa-IR) AppleWebKit/532.4 (KHTML, like Gecko) Usejump/0.10.2 Safari/532.4";
const char S45083[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Lunascape/6.5.8.24780 Safari/535.1";
const char S45084[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.3 (KHTML, like Gecko) Lunascape/6.8.0.25815 Safari/535.3";
const char S45085[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/523.15 (KHTML, like Gecko, Safari/419.3) Arora/0.3 (Change: 287 c9dfb30)";
const char S45086[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.34 (KHTML, like Gecko) iLivid/1.0 Safari/534.34";
const char S45087[] = "Mozilla/5.0(Windows NT 5.1) AppleWebKit/534.50 (KHTML, like Gecko) Safari/535.1";
const char S45088[] = "Mozilla/5.0 (Windows; N; Windows NT 5.1; en-US) AppleWebKit/529 (KHTML, like Gecko, Safari/529.0) Lunascape/4.9.9.94";
const char S45089[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.11 (KHTML, Like Gecko) 20.0.1132.47 Safari/536.11";
const char S45090[] = "Mozilla/5.0 (N; Windows NT 5.1) AppleWebKit/534.34 (KHTML, like Gecko) Raptr Safari/534.34";
const char S45091[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) SuperBird/23.0 Safari/537.11";
const char S45092[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML, like Gecko) Superbird/24.0.1312.57 Safari/537.17";
const char S45093[] = "Mozilla/5.0 (Windows; N; Windows NT 5.2; ru-RU) AppleWebKit/529 (KHTML, like Gecko, Safari/529.0) Lunascape/4.9.9.94";
const char S45094[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; cs-CZ) AppleWebKit/533.3 (KHTML, like Gecko) Lunascape/6.1.0.20995 Safari/533.3";
const char S45095[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.21 (KHTML, like Gecko) legacy Safari/537.21";
const char S45096[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Superbird/27.0.1450.0 Safari/537.36";
const char S45097[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.21 (KHTML, like Gecko) Capter Safari/537.21";
const char S45098[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Superbird/27.0.1454.94 Safari/537.22";
const char S45099[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; pt-BR) AppleWebKit/534.12 (KHTML, like Gecko) Navscape/Pre-0.1 Safari/534.12";
const char S45100[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.14 (KHTML, like Gecko) JavaFX/2.2 Safari/535.14";
const char S45101[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.4; fr; rv:1.9.1.7) Gecko/X Firefox/3.5.7";
const char S45102[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; GT-P3110 Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45103[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-P3110 Build/JOP40D)";
const char S45104[] = "Dalvik/1.6.0 (Linux; U; Android 4.2; GT-P3110 Build/JOP40C)";
const char S45105[] = "Logos/2.2.8.1 (Linux; U; Android 4.2.1; en-US; GT-P3110 Build/cm_p3110-userdebug 4.2.1 JOP40D eng..X.X test-keys)";
const char S45106[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P3110 Build/JDQ39)";
const char S45107[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3110 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45108[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-P3110 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45109[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3110 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45110[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-P3110 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45111[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.29 Safari/537.36 OPR/15.0.1147.24 (Edition Next)";
const char S45112[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100 (Edition Next)";
const char S45113[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100 (Edition Next)";
const char S45114[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153";
const char S45115[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153";
const char S45116[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153";
const char S45117[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153";
const char S45118[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153";
const char S45119[] = "Mozilla/4.0 (Windows NT 6.1; Win64; x64; rv:4.0a1) Gecko/X Firefox/7.0a1";
const char S45120[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:10.0a1) Gecko/X Firefox/7.0.1";
const char S45121[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:7.0.1) Gecko/X Firefox/7.0.12011-10-16 20:23:00";
const char S45122[] = "Mozilla/5.0(WindowsNT6.1;rv:7.0.1)Gecko/XFirefox/7.0.1";
const char S45123[] = "Mozilla/5.0 (Windows NT 6.1;  rv:7.0.1) Chrome/12.0.702.0 Gecko/X AppleWebKit/533.3 (KHTML, like Gecko) Version/5.0.5 Safari/533.3  Awesomium/1.6.5 Firefox/7.0.1";
const char S45124[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.1.2) Gecko/X Firefox/7 GTB5";
const char S45125[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:7.0) Gecko/X Firefox/7.0 Firefox/7.0";
const char S45126[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:7.0a1) Gecko/X Firefox/7.0a1 (.NET CLR 3.5.30729)";
const char S45127[] = "Mozilla/5.0 (Linux; U; Android 3.1; de-de; MZ601 Build/H.6.3-25-9) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45128[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; MZ601 Build/H.6.1-17_CN-40) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45129[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; MZ601 Build/H.6.3-14) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45130[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; MZ601 Build/H.6.3-16) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45131[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; MZ601 Build/H.6.3-25-5) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45132[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-au; MZ601 Build/H.6.3-25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45133[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; MZ601 Build/H.6.3-24) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45134[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; MZ601 Build/H.6.3-20) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45135[] = "Mozilla/5.0 (Linux; U; Android 3.1; ko-kr; MZ601 Build/H.6.3-25.2) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45136[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; MZ601 Build/H.6.3-25-5)";
const char S45137[] = "Mozilla/5.0 (Linux; U; Android 3.1; es-es; MZ601 Build/H.6.3-28-3) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45138[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-S5830 Build/GRWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45139[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5830 Build/GRWK74; CyanogenMod7-APOCALYPSE) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45140[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5830 Build/GRWK74; JOYOS-CyanogenMod) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45141[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT-S5830 Build/GRWK74; CM7.2-RC3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45142[] = "Mozilla/5.0 (Unknown; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.7.0 Safari/534.34";
const char S45143[] = "Mozilla/5.0 (Unknown; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.7.0 Safari/534.34";
const char S45144[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST25i Build/6.1.1.B.1.10) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45145[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST25a Build/6.1.1.B.1.10) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45146[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xperia U Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45147[] = "Mozilla/5.0 (Linux; Android 4.0.4; BNTV400 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S45148[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S45149[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S45150[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; PI86100)";
const char S45151[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radiant)";
const char S45152[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9B206 Safari/7534.48.3";
const char S45153[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9B176 Safari/7534.48.3";
const char S45154[] = "Mozilla/5.0 (iPod; CPU iPhone OS 511 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML like Gecko) CriOS/23.0.1271.91 Mobile/9B206 Safari/7534.48.3";
const char S45155[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45156[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45157[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; 7 Trophy T8686)";
const char S45158[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; 7 Trophy)";
const char S45159[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Spark)";
const char S45160[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; PC40100)";
const char S45161[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Touch-IT Trophy)";
const char S45162[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Trophy)";
const char S45163[] = "Mozilla/5.0 (Linux; Android 4.1.1; EndeavorU Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S45164[] = "Mozilla/5.0 (Linux; Android 4.1.2; One X Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S45165[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S45166[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S45167[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One X Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S45168[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5H11 Safari/525.20";
const char S45169[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_2 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5G77 Safari/525.20";
const char S45170[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_2 like Mac OS X; en-us) AppleWebKit/525.181 (KHTML, like Gecko) Version/3.1.1 Mobile/5H11 Safari/525.20";
const char S45171[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; SAMSUNG-SGH-I997R Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45172[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SAMSUNG-SGH-I997R Build/GINGERBREAD)";
const char S45173[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; SAMSUNG-SGH-I997R Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45174[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; SAMSUNG-SGH-I997R Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S45175[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; SAMSUNG-SGH-I997R Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45176[] = "Mozilla/5.0 (X11; Linux x86_64; rv:23.0) Gecko/X Firefox/23.0 Iceweasel/23.0";
const char S45177[] = "Mozilla/5.0 (X11; Linux i686 on x86_64; rv:23.0) Gecko/X Firefox/23.0";
const char S45178[] = "Mozilla/5.0 (X11; Linux x86_64; rv:23.0) Gecko/X Iceweasel/23.0.1";
const char S45179[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; ARM; Trident/6.0; Touch; .NET4.0E; .NET4.0C; Tablet PC 2.0)";
const char S45180[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; ARM; Trident/6.0; Touch; .NET4.0E; .NET4.0C; Tablet PC 2.0; ARMBJS)";
const char S45181[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; ARM; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; ms-office)";
const char S45182[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; ARM; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; ARMBJS; ms-office; MSOffice 15)";
const char S45183[] = "Mozilla/5.0 (X11; Linux i686 on x86_64; rv:21.0) Gecko/X Firefox/21.0";
const char S45184[] = "Mozilla/5.0 (X11; Linux x86_64; rv:21.0) Gecko/X Firefox/21.0 Iceweasel/21.0";
const char S45185[] = "Mozilla/5.0 (X11; U; Linux i686; en; rv:1.9.0.19) Gecko/X Mozilla/5.0 (X11; Linux i686; rv:21.0) Gecko/X Firefox/21.0";
const char S45186[] = "Mozilla/5.0 (X11; Linux x86_64; rv:21.0) Gecko/X Iceweasel/21.0";
const char S45187[] = "Mozilla/5.0 (Ultimate Edition; X11; Linux x64; rv:8.0) Gecko/X Ultimate Edition / 3.5 Firefox/21.0.1";
const char S45188[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-I9000T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45189[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-I9000T Build/GINGERBREAD)";
const char S45190[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-I9000T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45191[] = "Mozilla/5.0 (Linux; Android 4.0.4; BNTV600 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S45192[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; ME511 Build/RuthGC_0.18.4) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45193[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-ru; Beeline M2 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45194[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gh; l9 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45195[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; TSP21 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45196[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; th-th; Smile Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45197[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-ru; Beeline_E300 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45198[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-by; MTC Evo Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45199[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-bd; Grameenphone Crystal Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45200[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; de-de; ADM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45201[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; AIRIS Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45202[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A28 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45203[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; X6D Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45204[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Android for Livall TCC8902 GENBook (multi-language) Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45205[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-gb; jv02 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45206[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MID-7AS Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45207[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; wwe10 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45208[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; es-es; MW0811 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45209[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; M718 Build/ECLAIR)";
const char S45210[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; M718 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45211[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.4; en-us; SHAGM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45212[] = "Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; E239 Build/ERD79) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S45213[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A32 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45214[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; md-us; deb Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45215[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-za; NUM7UGW Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45216[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; ViewQuest SLATE2 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45217[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.2; es-es; STM7UW Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45218[] = "Mozilla/5.0 (Linux; U; Android 2.1-2010.11.4; de-de; XST2 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45219[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; es-es; ARGM7U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45220[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; ZEN Touch 2 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45221[] = "PVPlayer E10i CORE/7.507.1.1 OpenCORE/2.05 (Linux;Android2.1-update1)";
const char S45222[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; PALROID PR70HD Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45223[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; ZTE-LINK Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45224[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.2; es-es; STM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45225[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.8; es-es; STM7UW Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45226[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-us; MID-710YH-M4 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45227[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.2; es-es; STM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45228[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.9; es-es; STM703U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45229[] = "Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; YL-Coolpad_D539/1.01.008.X.D539; 320*480; CTC/2.0) CoolpadWebkit/530.18";
const char S45230[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-us; EBW204 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45231[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; V7500 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45232[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-uk; WR762 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45233[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; OMAP_SS Build/ECLAIR) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S45234[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; WellcoM-A88 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45235[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-uk; WR761 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45236[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; M82VG Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45237[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-us; M702 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45238[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-tw; Taiwan Mobile T2 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45239[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.9; en-us; NXM707 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45240[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; L8 Build/L8_R05_X) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45241[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; Maestro Build/Maestro_R26_X) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45242[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.9; es-es; STM703U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45243[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; OMAP_SS Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45244[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.1.0.1; es-es; ARGM7U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45245[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; es-es; STM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45246[] = "Mozilla/5.0(Linux;U;Android 2.1-update1;zh-tw)AppleWebKit/530.17(KHTML,like Gecko)Version/4.0 Mobile Safari/530.17";
const char S45247[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.1.21; en-us; PMP3084B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45248[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.1.0; es-es; STM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45249[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-us; WR780 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45250[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.1.15; en-us; PMP3084B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45251[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.7; es-es; STM703U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45252[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; de-de; Genesis Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45253[] = "Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; HS-E87/E434.6.01.02.00; 240*320; CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/3.1.2 Mobile Safari/530.17";
const char S45254[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; sv-se; SIM710 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45255[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-kr; ESP_E201V Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45256[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Scroll Tablet Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45257[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; tr-tr; hkdkc100 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45258[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.3; es-es; STM703U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45259[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.9; es-es; STM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45260[] = "Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; YL-Coolpad_E239/1.01.015.X.E239; 320*480; CTC/2.0) CoolpadWebkit/530.18";
const char S45261[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; F9191 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45262[] = "Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; D539 Build/ERD79) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S45263[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.1.5; es-es; ARGM7U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45264[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.6; es-es; GASEM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45265[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.4; es-es; DAHSM710UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45266[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; es-es; PAPYRE_pad_7.1 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45267[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.1.3; es-es; GASEM710C Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45268[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.8; es-es; GASEM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45269[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-gb; TOSHIBA_AC_AND_AZ) AppleWebkit/530.17(KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45270[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; w9 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45271[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; es-es; SIGMM7U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45272[] = "Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; YL-Coolpad_N930/1.01.012.X.N930; 480*800; CTC/2.0) CoolpadWebkit/530.17";
const char S45273[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; Nvsbl Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45274[] = "Mozilla/5.0 (Linux; U; Android 2.1-2010.12.7; de-de; XST2 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45275[] = "PVPlayer U20i CORE/7.507.1.1 OpenCORE/2.05 (Linux;Android2.1-update1)";
const char S45276[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.2; nl-nl; LUM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45277[] = "Mozilla/5.0 (Linux; U; Android 2.1; ja-jp; NTT-ME ME-DPF1.0 Build/0001) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S45278[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; es-es; HackDROID Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45279[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-TC21-HCV13.0-B20; en-us; ATZ 7015/7015A HONEYCOMB V13.0-B20 ACADEV Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45280[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; WellcoM-A86 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45281[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.5; es-es; GASEM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45282[] = "Mozilla/5.0 (Linux;U;Android 2.1-update1; zh-cn; ZXY-ZTE-C X920/X920-C-TS9CD038; 240*320;CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45283[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; W711 Build/W711_VER10.21) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45284[] = "Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; TL-Twin_T90/E370.6.01.02.00; 320*480; CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/3.1.2 Mobile Safari/530.17";
const char S45285[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; D000-X-C01 Build/01.10.12-01.20) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45286[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.1.2; es-es; STM7UW Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45287[] = "Mozilla/5.0 (Linux;U;Android 2.1;zh-cn;HS-E89/E329.6.02.00.00;240*320;CTC/2.0) AppleWebKit/530.17(KHTML, like Gecko) Version/3.1.2 Mobile Safari/530.17";
const char S45288[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.4; es-es; STM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45289[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.1.1; es-es; GASEM710C Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45290[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.5; es-es; GASEM710C Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45291[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.8; es-es; STM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45292[] = "Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; HX-HE-N100/E331.6.01.01.00; 240*320; CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/3.1.2 Mobile Safari/530.17";
const char S45293[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; pt-br; BAK Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45294[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-v1.02_15.14-M_ES-2011.03.23; es-es; Pandora Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45295[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.6; es-es; STM703U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45296[] = "Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; YFZ-EPHONE_A8/E381.6.01.02.00; 240*320; CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/3.1.2 Mobile Safari/530.17";
const char S45297[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; es-es; STM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45298[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; ME511 Build/RuthGC_0.18.4) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S45299[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.3.12; es-es; GASEIM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45300[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.1; es-es; ARGM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45301[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.2.1; de-de; BRM7U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45302[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.6; en-za; NUM7UGW Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45303[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; pt-br; VOXM710UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45304[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; IBAK-GPS701 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45305[] = "Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; HS-E89/E329.6.02.00.00; 240*320; CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/3.1.2 Mobile Safari/530.17";
const char S45306[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; IBAK-GPS702DTV Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45307[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.13; pt-pt; ZIPM707 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45308[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; pl-pl; K30 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45309[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; OMS1_6 Build/ECLAIR) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S45310[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Rstream-A1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45311[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.2; es-es; ARGM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45312[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.1; en-za; NUM7UGW Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45313[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.7; es-es; STM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45314[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; es-es; TL-7A Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45315[] = "Mozilla/5.0 (Android 2.1; en-us) AppleWebKit/1.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45316[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; nl-nl; EB0700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45317[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; ru-ru; RBK-490 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45318[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.2.0; es-es; ARGM7U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45319[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; PM1152 Tabula (V9) Build/ERE27) AppleWebKit/530.17 (KHTML like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45320[] = "Mozilla/5.0 (Linux; U; Android 2.1; ru-ru; DV Build/Donut) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45321[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; VW1 Build/VW1_R14_X) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45322[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZTE BLADE III Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45323[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hr-hr; Vip Racer III Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45324[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hr-hr; ZTE Blade III Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45325[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; BLADE III Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45326[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Blade III Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45327[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; BLADE III_IL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45328[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ZTE BLADE III Build/IMM76D)";
const char S45329[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Blade III Build/IMM76D)";
const char S45330[] = "Opera/9.23 (X11; Linux i686; U; en)";
const char S45331[] = "Opera/9.20 (X11; Linux i686; U; en)";
const char S45332[] = "Opera/9.24 (X11; Linux i686; U; en)";
const char S45333[] = "Opera/9.25 (X11; Linux i686; U; en)";
const char S45334[] = "Opera/9.21 (X11; Linux i686; U; en)";
const char S45335[] = "Opera/9.27 (X11; Linux i686; U; ru)";
const char S45336[] = "Opera/9.26 (X11; Linux i686; U; ru)";
const char S45337[] = "Opera/9.22 (X11; Linux i686; U; pt-br)";
const char S45338[] = "Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux i686; en) Opera 9.27";
const char S45339[] = "Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux i686; en) Opera 9.22";
const char S45340[] = "Mozilla/5.0 (X11; Linux i686; U; en; rv:1.8.0) Gecko/X Firefox/1.5.0 Opera 9.23";
const char S45341[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; M660 Build/HuaweiM660) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45342[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; M660 Build/HuaweiM660)";
const char S45343[] = "KWC-S1310-A/1004 UP.Browser/7.2.6.1.840 (GUI) MMP/2.0";
const char S45344[] = "KWC-S1310-A/1003 UP.Browser/7.2.6.1.840 (GUI) MMP/2.0";
const char S45345[] = "KWC-S1310-A/1002 UP.Browser/7.2.6.1.840 (GUI) MMP/2.0";
const char S45346[] = "KWC-S1310-A/1103 UP.Browser/7.2.6.1.840 (GUI) MMP/2.0";
const char S45347[] = "KWC-S1310-A/1105 UP.Browser/7.2.6.1.840 (GUI) MMP/2.0";
const char S45348[] = "KWC-S1310-A/1110 UP.Browser/7.2.6.1.840 (GUI) MMP/2.0";
const char S45349[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; SGH-i667)";
const char S45350[] = "Mozilla/5.0 (Series40; NokiaC2-05/08.69; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S45351[] = "Mozilla/5.0 (Series40; NokiaC2-05.1/08.69; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S45352[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; ARCHOS 80G9 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45353[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; ARCHOS 80G9 Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45354[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; ARCHOS 80G9 Build/HTK75D)";
const char S45355[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; ARCHOS 80G9 Build/HTJ85B)";
const char S45356[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AT200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45357[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; AT200 Build/IML74K)";
const char S45358[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45359[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45360[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7300 Build/IMM76D)";
const char S45361[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P7300-ORANGE/P7300BVLP7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45362[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P7300 Build/HTJ85B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45363[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P7300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45364[] = "Logos/1.1.2 (Linux; U; Android 4.0.4; en-US; GT-P7300 Build/cm_p5-userdebug 4.0.4 IMM76L eng.kalltkaffe.X.X test-keys)";
const char S45365[] = "Vyrso/2.1.2 (Linux; U; Android 4.0.4; en-US; GT-P7300 Build/IMM76D.ZSLP7)";
const char S45366[] = "Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7300 Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S45367[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S45368[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45369[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P7300-ORANGE/P7300BVLP4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45370[] = "com.google.android.youtube/4.0.23(Linux; U; Android 4.0.4; en_US; GT-P7300 Build/IMM76D) gzip";
const char S45371[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7300 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S45372[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P7300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45373[] = "Vyrso/2.2.8.1 (Linux; U; Android 4.0.4; in-ID; GT-P7300 Build/IMM76D.UBLP4)";
const char S45374[] = "Logos/2.2.10 (Linux; U; Android 4.0.4; en-PH; GT-P7300 Build/IMM76D.XXLQ6)";
const char S45375[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; GT-P7300B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45376[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; in-id; GT-P7300 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S45377[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7300B Build/IMM76D)";
const char S45378[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; SAMSUNG GT-P7300/P7300XXLQD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45379[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus 10 Build/JWR66V) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S45380[] = "Xenu Link Sleuth 1.3";
const char S45381[] = "Xenu Link Sleuth 1.2i";
const char S45382[] = "Xenu Link Sleuth 1.3.5";
const char S45383[] = "Xenu Link Sleuth 1.3c";
const char S45384[] = "Xenu Link Sleuth 1.3.4";
const char S45385[] = "Xenu Link Sleuth 1.2j";
const char S45386[] = "Xenu Link Sleuth 1.2e";
const char S45387[] = "Xenu Link Sleuth 1.2g";
const char S45388[] = "Lynx/2.6 libwww-FM/2.14";
const char S45389[] = "Xenu Link Sleuth 1.2d";
const char S45390[] = "Xenu Link Sleuth 1.2f";
const char S45391[] = "BinGet/1.00.A (http://www.bin-co.com/php/scripts/load/)";
const char S45392[] = "Mechanize/2.5.1 Ruby/1.9.2p290 (http://github.com/tenderlove/mechanize/)";
const char S45393[] = "Facebook Update/1.2.205.0;winhttp;cup";
const char S45394[] = "Mechanize/2.6.0 Ruby/1.9.3p392 (http://github.com/sparklemotion/mechanize/)";
const char S45395[] = "Mozilla/5.0 (PLAYSTATION 3 4.46) AppleWebKit/531.22.8 (KHTML, like Gecko)";
const char S45396[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8010 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S45397[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8010 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S45398[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:18.0) Gecko/X Firefox/18.0 SeaMonkey/2.15.2";
const char S45399[] = "Mozilla/5.0 (Windows NT 5.1; rv:18.0) Gecko/X Firefox/18.0 SeaMonkey/2.15.2";
const char S45400[] = "Mozilla/5.0 (Windows NT 5.1; rv:18.0) Gecko/X SeaMonkey/2.15.2";
const char S45401[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S45402[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S45403[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3113 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S45404[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3113 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S45405[] = "SAMSUNG-GT-E2222/E2222JVKJ3 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S45406[] = "SAMSUNG-GT-E2222L/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S45407[] = "AppleCoreMedia/1.0.0.10B142 (iPhone; U; CPU OS 6_1 like Mac OS X; en_us)";
const char S45408[] = "AppleCoreMedia/1.0.0.10B145 (iPhone; U; CPU OS 6_1_1 like Mac OS X; en_us)";
const char S45409[] = "AppleCoreMedia/1.0.0.10B146 (iPhone; U; CPU OS 6_1_2 like Mac OS X; en_us)";
const char S45410[] = "AppleCoreMedia/1.0.0.10B329 (iPhone; U; CPU OS 6_1_3 like Mac OS X; en_us)";
const char S45411[] = "AppleCoreMedia/1.0.0.10B350 (iPhone; U; CPU OS 6_1_4 like Mac OS X; en_us)";
const char S45412[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6603 Build/10.1.A.1.350) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S45413[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6603 Build/10.1.A.1.350) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S45414[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; fr; rv:1.8.1.16) Gecko/X Firefox/2.0.0.16";
const char S45415[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.8.1a2) Gecko/X BonEcho/2.0a2";
const char S45416[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.8) Gecko/X Firefox/2.0a1";
const char S45417[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:5.0.1) Gecko/X Firefox/5.0.1";
const char S45418[] = "Mozilla/5.0 (Series40; Nokia202/20.28; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S45419[] = "Mozilla/5.0 (Series40; Nokia203/20.36; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S45420[] = "Mozilla/5.0 (Series40; Nokia202/20.28; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5";
const char S45421[] = "Nokia6700c/2.0 (13.21) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S45422[] = "Nokia6700c-1/2.0 (10.50) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S45423[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/11.0.696.57)";
const char S45424[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/11.0.696.68)";
const char S45425[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/11.0.696.57) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.57 Safari/534.24";
const char S45426[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/11.0.696.57; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S45427[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB6.6; chromeframe/11.0.696.65; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)";
const char S45428[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/11.0.696.71) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24";
const char S45429[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; FDM; .NET4.0C; .NET4.0E; chromeframe/11.0.696.57)";
const char S45430[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; FDM; .NET4.0C; .NET4.0E; chromeframe/11.0.696.57)";
const char S45431[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45432[] = "Mozilla/5.0 (Linux; Android 4.0.3; XT910 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45433[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S45434[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT910K Build/6.7.4-27_SA-46) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45435[] = "SAMSUNG-SGH-A927/A927UCJF5; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.19039; U; en-KR) Opera 9.50";
const char S45436[] = "NokiaC2-03/2.0 (07.48) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S45437[] = "NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-03";
const char S45438[] = "NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S45439[] = "Mozilla/5.0 (Series40; Nokia201/11.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S45440[] = "Mozilla/5.0 (Linux; Android 4.0.4; BNTV400 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S45441[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; T-Mobile_Espresso Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45442[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile_Espresso Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45443[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;T-Mobile_Espresso Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45444[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile_Espresso Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S45445[] = "Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; T-Mobile_Espresso Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45446[] = "Mozilla/5.0 (Linux; U; Android2.2.1; en-us; T-Mobile_Espresso Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45447[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile_EspressoBuild/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45448[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-I605 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.74 Safari/537.36";
const char S45449[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S45450[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-I605 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S45451[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S45452[] = "Mozilla/5.0 (Series40; NokiaX2-02/11.79; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S45453[] = "Mozilla/5.0 (IE 11.0; Windows NT 6.3; ARM; Trident/7.0; Touch; rv:11.0) like Gecko";
const char S45454[] = "Mozilla/5.0 (Windows NT 6.3; ARM; Trident/7.0; Touch; rv:11.0) like Gecko";
const char S45455[] = "Mozilla/5.0 (Windows NT 6.3; ARM; Trident/7.0; rv:11.0) like Gecko";
const char S45456[] = "Mozilla/5.0 (Linux; Android 4.1.1; XT875 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45457[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT875 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45458[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45459[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S45460[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I747 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S45461[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L710 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S45462[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I747 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S45463[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9305 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S45464[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530U Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S45465[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-I747 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S45466[] = "Mozilla/5.0 (Linux; Android 4.2.1; SAMSUNG-SGH-I747 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S45467[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; SGH-i937)";
const char S45468[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; Haden)";
const char S45469[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.29";
const char S45470[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S45471[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.28 (KHTML, like Gecko) Version/3.2.2 Safari/525.28.1";
const char S45472[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/531.0 (KHTML, like Gecko) Version/3.2.1 Safari/531.0";
const char S45473[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.13";
const char S45474[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/525.28 (KHTML, like Gecko) Version/3.2.2 Safari/525.28.1";
const char S45475[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; de-DE) AppleWebKit/528+ (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S45476[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 Nokia6700s/071.004; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.26 Mobile Safari/533.4 3gpp-gba";
const char S45477[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 Nokia6700s-1c/071.004; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.26 Mobile Safari/533.4 3gpp-gba";
const char S45478[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 Nokia6700s-1c/033.014; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6";
const char S45479[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 Nokia6700s/032.005; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.1.6";
const char S45480[] = "(C)Nokia6700s/SymbianOS/9.1 Series60/3.0";
const char S45481[] = "Opera/9.80 (Windows NT 6.2) Presto/2.12.388 Version/12.13";
const char S45482[] = "Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.13";
const char S45483[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.13";
const char S45484[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.13";
const char S45485[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.13";
const char S45486[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.13";
const char S45487[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11";
const char S45488[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11";
const char S45489[] = "Mozilla/5.0 (Linux; U; Android2.2; en-us; myTouch4G Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45490[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; myTouch4GBuild/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45491[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; myTouch4G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45492[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; myTouch4G Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S45493[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;myTouch4G Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45494[] = "Mozilla/5.0 (Linux; U;Android 2.2; en-us; myTouch4G Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45495[] = "Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; myTouch4G Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45496[] = "Mozilla/5.0 (Linux; U; Android2.2.1; en-us; myTouch4G Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45497[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; myTouch4GBuild/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45498[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; myTouch4G Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45499[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; myTouch4G Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45500[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9370; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.755 Mobile Safari/534.11+";
const char S45501[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45502[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45503[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45504[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; SPH-M920 Build/FROYO)";
const char S45505[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SPH-M920 Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S45506[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S45507[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-N7000 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S45508[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T879 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S45509[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000B Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S45510[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160L Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S45511[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S45512[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160K Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S45513[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S45514[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-05D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S45515[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M480W Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S45516[] = "Mozilla/5.0 (Linux; Android 4.0.3; SHV-E160S Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S45517[] = "Mozilla/5.0 (Series40; Nokia311/05.92; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S45518[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5) Presto/2.12.388 Version/12.15";
const char S45519[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT875 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45520[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45521[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S45522[] = "Mozilla/5.0 (Series40; Nokia205/03.18; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.34";
const char S45523[] = "Mozilla/5.0 (Series40; Nokia205.1/03.18; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.34";
const char S45524[] = "Mozilla/5.0 (Series40; Nokia2055/03.20; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.34";
const char S45525[] = "Mozilla/5.0 (Series40; Nokia205.3/03.19; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.34";
const char S45526[] = "Mozilla/5.0 (Series40; Nokia2052/03.20; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.34";
const char S45527[] = "Mozilla/5.0 (Series40; Nokia2051/03.20; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.34";
const char S45528[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S45529[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC VLE_U Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S45530[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S45531[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Special Edition Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S45532[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.5 Mobile/11A4435d Safari/8536.25";
const char S45533[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A465 Safari/8536.25";
const char S45534[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A4449d Safari/8536.25";
const char S45535[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A4449a Safari/8536.25";
const char S45536[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A501 Safari/8536.25";
const char S45537[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S45538[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710VMUB Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S45539[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710BST Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S45540[] = "Mozilla/5.0 (Linux; Android 4.0.3; SPH-D710 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S45541[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X+ Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45542[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC EVARE_UL Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45543[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Venture Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45544[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Venture Build/GRJ90)";
const char S45545[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/11.10 Chromium/14.0.835.202 Chrome/14.0.835.202 Safari/535.1";
const char S45546[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/11.10 Chromium/14.0.835.202 Chrome/14.0.835.202 Safari/535.1";
const char S45547[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/11.10 Chromium/14.0.835.202 Chrome/14.0.835.202 Safari/535.1,gzip(gfe) (via translate.google.com)";
const char S45548[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ALCATEL ONE TOUCH 4010X Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45549[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; ALCATEL ONE TOUCH 4010A Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45550[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; ALCATEL ONE TOUCH 4010D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45551[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; ALCATEL ONE TOUCH 4010E Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45552[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; SHW-M440S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45553[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHW-M440S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45554[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHW-M440S Build/JZO54K)";
const char S45555[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SHW-M440S Build/JRO03C)";
const char S45556[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I925 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45557[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I925U Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 USCC-I925U";
const char S45558[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I925 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45559[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-I925 Build/JZO54K)";
const char S45560[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:23.0) Gecko/X Firefox/23.0 SeaMonkey/2.20";
const char S45561[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:23.0) Gecko/X SeaMonkey/2.20";
const char S45562[] = "Mozilla/5.0 (Linux; Android 4.3; Galaxy Nexus Build/JWR66V) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S45563[] = "Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Safari/535.19 Silk-Accelerated=true";
const char S45564[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/532.6 (KHTML, like Gecko) Chrome/4.0.266.0 Safari/532.6";
const char S45565[] = "Mozilla/5.0 (X11; U; Linux x86_64; fr-FR) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/4.0.202.0 Safari/532.0";
const char S45566[] = "Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US) AppleWebKit/532.2 (KHTML, like Gecko) Chrome/4.0.221.8 Safari/532.2";
const char S45567[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/4.0.202.0 Safari/532.0";
const char S45568[] = "Mozilla/5.0 (X11; U; Slackware Linux x86_64; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5";
const char S45569[] = "Mozilla/5.0 (Linux; Android 4.0.4; Next7P12 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S45570[] = "Mozilla/5.0 (Linux; Android 4.0.4; Next7P12-8G Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S45571[] = "Mozilla/5.0 (Linux; Android 4.0.3; Next7P12 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S45572[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; ViewPad 7a Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45573[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; ViewPad7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45574[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; ViewPad7 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45575[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; ViewPad7 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45576[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ru-; ViewPad7 Build/FRG83) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S45577[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; ViewPad7 Build/FRF91)";
const char S45578[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; ViewPad7 Build/FRG83G)";
const char S45579[] = "Mozilla/5.0 (Linux; Android 4.1.1; C6602 Build/10.1.A.1.78) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45580[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6602 Build/10.1.A.1.434) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45581[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-02E Build/10.1.D.0.343) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45582[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NexusHD2 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45583[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC HD2 Build/IMM76L; CyanogenMod-BlueOrange) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45584[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC HD2 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45585[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NexusHD2 Build/IMM76L; CyanogenMod-CM9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45586[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC HD2 Build/IMM76L; CyanogenMod-huhjun iCyanogen v1.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45587[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC HD2 Build/IMM76D; CyanogenMod-Cm9ight_leo_v1.1a) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45588[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; HD2 Build/IMM76I; CyanogenMod-9.1.0 Final) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45589[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; N861 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45590[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; N861 Build/JZO54K)";
const char S45591[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S45592[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S45593[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S45594[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Special Edition Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S45595[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC VLE_U Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S45596[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/10.04 Chromium/14.0.835.202 Chrome/14.0.835.202 Safari/535.1";
const char S45597[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/10.04 Chromium/14.0.813.0 Chrome/14.0.813.0 Safari/535.1";
const char S45598[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8160 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45599[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8160 Build/JZO54K)";
const char S45600[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; GT-I8160P Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45601[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; GT-I8160L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45602[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8160P Build/JZO54K)";
const char S45603[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8160L Build/JZO54K)";
const char S45604[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S45605[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; NXM726 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45606[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; NXM726HN Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45607[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; NXM726HN_HASTINGS Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45608[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.1; NXM726HN Build/GINGERBREAD)";
const char S45609[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-; NXM726HN Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45610[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Comet Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45611[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; Comet Build/FRF91)";
const char S45612[] = "Mozilla/5.0 (Linux; U;Android 2.2; en-us; Comet Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45613[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us;Comet Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45614[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Comet Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S45615[] = "Mozilla/5.0 (Linux; U; Android2.2; en-us; Comet Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45616[] = "Mozilla/5.0 (Symbian/3; Series60/5.3 NokiaC6-01; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.4.2.6 Mobile Safari/533.4 3gpp-gba";
const char S45617[] = "Mozilla/5.0 (Symbian/3; Series60/5.3 NokiaC6-01/111.040.1511; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.1.4 Mobile Safari/535.1 3gpp-gba";
const char S45618[] = "Nokia6303c_CMCC/2.0 (08.90) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S45619[] = "Nokia6303c/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S45620[] = "Nokia6303classic/2.0 (09.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S45621[] = "Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303classic";
const char S45622[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S45623[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S45624[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S45625[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S45626[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S45627[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S45628[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S45629[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en; rv:1.9.2.28) Gecko/X Camino/2.1.2 (like Firefox/3.6.28)";
const char S45630[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:16.0) Gecko/X Firefox/16.0 AlexaToolbar/alxf-2.17";
const char S45631[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.0.19) Gecko/2010031218 Firefox/3.0.19";
const char S45632[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6.1; en-US; rv:1.9) Gecko/X Firefox/3.0";
const char S45633[] = "SkimBot/1.0 (www.skimlinks.com <dev@skimlinks.com>)";
const char S45634[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS870 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45635[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9205 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S45636[] = "Mozilla/5.0 (Linux; Android 4.2.2; fr-fr; SAMSUNG GT-I9205/I9205XXUAMG7 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S45637[] = "Mozilla/5.0 (Linux; Android 4.2.2; fr-fr; SAMSUNG GT-I9205/I9205XXUAMG4 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S45638[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/1.0.154.36 Safari/525.19";
const char S45639[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/1, 6, 0, 6 Safari/533.3";
const char S45640[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/1, 6, 0, 4 Safari/533.3";
const char S45641[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.3 (KHTML, like Gecko) Chrome/1, 6, 0, 4 Safari/535.3";
const char S45642[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/1, 6, 0, 3 Safari/533.3";
const char S45643[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/1, 6, 2, 0 Safari/533.3";
const char S45644[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.3 (KHTML, like Gecko) Chrome/1, 6, 0, 4 Safari/535.3";
const char S45645[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/1, 6, 1, 0 Safari/533.3";
const char S45646[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S45647[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989D Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S45648[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S45649[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S45650[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-T989 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S45651[] = "Mozilla/5.0 (Linux; U;Android 1.5; en-us; MB200 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S45652[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB200Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S45653[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB200-demo Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S45654[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us;MB200 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S45655[] = "Mozilla/5.0 (Linux; U; Android1.5; en-us; MB200 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S45656[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-; MB200 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S45657[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB200 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S45658[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB200 Build/CUPCAKE) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S45659[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB200 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mob";
const char S45660[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT29i Build/9.1.B.0.411) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45661[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT29i Build/9.1.B.0.411) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45662[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LT29i Build/9.1.B.1.67)";
const char S45663[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT29i Build/9.1.B.1.67) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45664[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT29i Build/9.1.B.1.67) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45665[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45666[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7100 Build/IMM76D)";
const char S45667[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P7100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45668[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P7100 Build/HTJ85B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45669[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-; GT-P7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45670[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S45671[] = "Biblia/2.2.7 (Linux; U; Android 4.0.4; es-MX; GT-P7100 Build/IMM76D.BULG3)";
const char S45672[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7100 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45673[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-P7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45674[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; GT-P7100 Build/HTJ85B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45675[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 820)";
const char S45676[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 820; ANZ825)";
const char S45677[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 820; Vodafone)";
const char S45678[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 820; Orange)";
const char S45679[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Nokia 820)";
const char S45680[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 820; 1328-3)";
const char S45681[] = "Mozilla/5.0 (Series40; NokiaC2-03/07.48; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S45682[] = "Mozilla/4.0 (PSP (PlayStation Portable); 2.00)";
const char S45683[] = "Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S45684[] = "Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S45685[] = "Mozilla/5.0 (Linux; Android 4.1.2; MB886 Build/9.8.2Q-8_MB886_FFW-11) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S45686[] = "Mozilla/5.0 (Linux; U; Android 1.5; zh-cn; ME600 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S45687[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-ca; MB300 Build/Blur_Version.0.13.1145.MB300.TELUS.en.CA Flex/P009) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S45688[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-ca; MB300 Build/Blur_Version.0.13.1129.MB300.TELUS.en.CA Flex/P008) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S45689[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB300 Build/Blur_Version.0.12.1151.MB300.ATT.en.US Flex/P012) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S45690[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB300 Build/Blur_Version.0.12.1199.MB300.ATT.en.US Flex/P014) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S45691[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB300 Build/Blur_Version.0.13.37.MB300.ATT.en.US Flex/P014) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S45692[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB300 Build/Blur_Version.0.13.35.MB300.ATT.en.US Flex/P014) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S45693[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB300 Build/Blur_Version.0.12.1214.MB300.ATT.en.US Flex/P014) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S45694[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB300 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S45695[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293";
const char S45696[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S45697[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S45698[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293";
const char S45699[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S45700[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S45701[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/6531.22.7 iLunascape/3203";
const char S45702[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293";
const char S45703[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293";
const char S45704[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S45705[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S45706[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/6531.22.7 iLunascape/3205";
const char S45707[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S45708[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_2 like Mac OS X) SparqEye/1.2.2 AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A293 Safari/6531.22.7";
const char S45709[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_4 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S45710[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_4 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293";
const char S45711[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X) SparqEye/1.2.2 AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A293 Safari/6531.22.7";
const char S45712[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_4 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S45713[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/10B329 Safari/6531.22.7";
const char S45714[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/10B146 Safari/6531.22.7";
const char S45715[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/6531.22.7 iLunascape/3212";
const char S45716[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_4 like Mac OS X) SparqEye/1.2.2 AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A293 Safari/6531.22.7";
const char S45717[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45718[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Micromax A110 Build/IMM76D)";
const char S45719[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; Micromax A110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45720[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A110 Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S45721[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S45722[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S45723[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus S Build/JRO03E) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45724[] = "Mozilla/5.0 (Linux; Android 4.1; Nexus S Build/JRN83C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45725[] = "Mozilla/5.0 (Linux; Android 4.1.1; Full Android on Crespo Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45726[] = "Mozilla/5.0 (Linux; Android 4.1.1;  Nexus S Build/JRO03E) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S45727[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45728[] = "Mozilla/5.0 (Linux; Android 4.1.2;  Nexus S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S45729[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; N850 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45730[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; N850 Build/GINGERBREAD)";
const char S45731[] = "Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Safari/535.19 Silk-Accelerated=true";
const char S45732[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; one touch T10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45733[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; CT10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45734[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45735[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; one touch T10 Build/IML74K)";
const char S45736[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; EVOAT10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45737[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HYUNDAI T10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45738[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P5100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S45739[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P5100 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S45740[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T779 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S45741[] = "Mozilla/5.0 (Windows NT 6.2; rv:2.0) Gecko/X Firefox/4.0";
const char S45742[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:2.0b12pre) Gecko/X Firefox/4.0b12pre";
const char S45743[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; WOW64; en-US; rv:2.0b1) Gecko/X Firefox/4.0b1";
const char S45744[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 AvantBrowser/Tri-Core";
const char S45745[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 AvantBrowser/Tri-Core";
const char S45746[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; Avant TriCore) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.96 Safari/537.4";
const char S45747[] = "Mozilla/5.0 (Windows NT 6.2; Avant TriCore) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.96 Safari/537.4";
const char S45748[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; Avant TriCore) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S45749[] = "Mozilla/5.0 (Windows NT 6.2; Avant TriCore) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S45750[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; Avant TriCore) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S45751[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; Avant TriCore) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S45752[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; Avant TriCore) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S45753[] = "Mozilla/5.0 (Windows NT 6.2; Avant TriCore) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S45754[] = "Mozilla/5.0 (Windows NT 6.2; Avant TriCore) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S45755[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; Avant TriCore) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S45756[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36";
const char S45757[] = "Mozilla/5.0 (Windows NT 6.2; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36";
const char S45758[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36";
const char S45759[] = "Mozilla/5.0 (Windows NT 6.2; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36";
const char S45760[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36";
const char S45761[] = "Mozilla/5.0 (Windows NT 6.2; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36";
const char S45762[] = "Mozilla/5.0 (Linux; Android 4.3; HTC One Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S45763[] = "Mozilla/5.0 (Linux; Android 4.3; One Build/JWR66V.H10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S45764[] = "Mozilla/5.0 (Linux; Android 4.3; HTC One 801e Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S45765[] = "Mozilla/5.0 (Linux; Android 4.3; HTC One with MoDaCo.SWITCH Beta 13 Build/JWR66V.H10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S45766[] = "Mozilla/5.0 (Linux; Android 4.3; HTCONE Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S45767[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A5376e Safari/7534.48.3";
const char S45768[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A403 Safari/7534.48.3";
const char S45769[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A405 Safari/7534.48.3";
const char S45770[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A523 Safari/7534.48.3";
const char S45771[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A525 Safari/7534.48.3";
const char S45772[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A551 Safari/7534.48.3";
const char S45773[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5; U; en) Presto/2.10.289 Version/12.02";
const char S45774[] = "Mozilla/5.0 (iPod; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/3A100a Safari/419.3";
const char S45775[] = "Mozilla/5.0 (iPod; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/4A102 Safari/419.3";
const char S45776[] = "Mozilla/5.0 (iPod; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/4B1 Safari/419.3";
const char S45777[] = "Mozilla/5.0 (iPod; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/4A93 Safari/419.3";
const char S45778[] = "Mozilla/5.0 (iPod; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/3B48b Safari/419.3";
const char S45779[] = "Mozilla/5.0 (iPod; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/3A110a Safari/419.3";
const char S45780[] = "Mozilla/5.0 (iPod; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML like Gecko) Version/3.0 Mobile/3A101a Safari/419.3";
const char S45781[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PMP5080B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45782[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PMP5080CPRO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45783[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; PMP5080CPRO Build/IML74K)";
const char S45784[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S45785[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S45786[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8005 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S45787[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8020 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S45788[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8020 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S45789[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LG-L40G/V100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45790[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LG-L40G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45791[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-L40G Build/IML74K)";
const char S45792[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 UCBrowser/9.0.1.284 Mobile";
const char S45793[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 UCBrowser/9.0.1.284 Mobile";
const char S45794[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 UCBrowser/9.0.1.284 Mobile";
const char S45795[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 UCBrowser/9.0.1.284 Mobile";
const char S45796[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 UCBrowser/9.0.1.284 Mobile";
const char S45797[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 UCBrowser/9.0.1.284 Mobile";
const char S45798[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145 UCBrowser/9.0.1.284 Mobile";
const char S45799[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-US) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 UCBrowser/9.0.0.260 Mobile";
const char S45800[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X; en-US) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B329 Safari/7534.48.3 UCBrowser/9.0.0.260";
const char S45801[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en; rv:1.9.2.28) Gecko/X Camino/2.1.2 (like Firefox/3.6.28)";
const char S45802[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en; rv:1.9.2.28) Gecko/X Camino/2.1.2 (like Firefox/13.0)";
const char S45803[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Light Tab 2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45804[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; MegaFon V9+ Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45805[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; V9A Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45806[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Light Tab 2 Build/GINGERBREAD)";
const char S45807[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; th-th; V9+ Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45808[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-za; ZTE V9A Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45809[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Light Tab 2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45810[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; V9A Build/GINGERBREAD)";
const char S45811[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT881 Build/6.7.3-130_YUS-11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45812[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT881 Build/6.7.3-85_YTZUSC-LE-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45813[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT881 Build/6.7.3-144) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45814[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; XT881 Build/6.7.3-130_YUS-11)";
const char S45815[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S45816[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300TL Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S45817[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300TG Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S45818[] = "Mozilla/5.0 (Linux; Android 4.2.2; TF300T Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S45819[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Pad TF300T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S45820[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.0; en) Opera 9.5";
const char S45821[] = "Opera/9.52 (Windows NT 6.0; U; en)";
const char S45822[] = "Opera/9.51 (Windows NT 6.0; U; en)";
const char S45823[] = "Opera/9.50 (Windows NT 6.0; U; en)";
const char S45824[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.0; en) Opera 9.50";
const char S45825[] = "Opera/9.5 (Windows NT 6.0; U; en)";
const char S45826[] = "Mozilla/5.0 (Windows NT 5.1; rv:19.0) Gecko/X Firefox/19.0 SeaMonkey/2.16.2";
const char S45827[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:19.0) Gecko/X Firefox/19.0 SeaMonkey/2.16.2";
const char S45828[] = "Mozilla/5.0 (Windows NT 5.1; rv:19.0) Gecko/X SeaMonkey/2.16.2";
const char S45829[] = "Mozilla/5.0 (Series40; Nokia311/03.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S45830[] = "Mozilla/5.0 (Series40; Nokia311/03.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5";
const char S45831[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Twitter for iPhone";
const char S45832[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Twitter for iPhone";
const char S45833[] = "SAMSUNG-GT-S5610K/S5610KDDKK2 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S45834[] = "SAMSUNG-GT-S5610/S5610XXLF4 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S45835[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-R730 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45836[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SCH-R730 Build/GINGERBREAD)";
const char S45837[] = "Mozilla/5.0 (PLAYSTATION 3 4.31) AppleWebKit/531.22.8 (KHTML, like Gecko)";
const char S45838[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45839[] = "Instagram 3.4.1 (iPhone4,1; iPhone OS 6.0.1; en_US; en) AppleWebKit/420+";
const char S45840[] = "Instagram 3.4.1 (iPhone5,2; iPhone OS 6.0.2; en_US; en) AppleWebKit/420+";
const char S45841[] = "Instagram 3.4.2 (iPhone4,1; iPhone OS 6.0; en_US; en) AppleWebKit/420+";
const char S45842[] = "SonyEricssonU1i/R2AA; Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525";
const char S45843[] = "SonyEricssonU1iv/R2AA; Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525";
const char S45844[] = "SonyEricssonU1a/R2AA; Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525";
const char S45845[] = "Opera/9.80 (BREW; Opera Mini/5.1/27.2338; U; en) Presto/2.8.119 240X400 Pantech CDM8992";
const char S45846[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S45847[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S45848[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Iron/25.0.1400.0 Chrome/25.0.1400.0 Safari/537.22";
const char S45849[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML, like Gecko) Iron/25.0.1400.0 Chrome/25.0.1400.0 Safari/537.22";
const char S45850[] = "Mercury/7.4 CFNetwork/609.1.4 Darwin/13.0.0";
const char S45851[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; IM-A830L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45852[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; IM-A830S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45853[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; IM-A830K Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45854[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; IM-A830L Build/IML74K)";
const char S45855[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; IM-A830S Build/IML74K)";
const char S45856[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; IM-A830K Build/IML74K)";
const char S45857[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; IM-A830K Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45858[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; IM-A830S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45859[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; IM-A830L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45860[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; SKY-A830 Build/LTE) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S45861[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; SKY-A830S/L/K Build/LTE) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S45862[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; IM-A830S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S45863[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; IM-A830 Build/Vesion 1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45864[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SKY-A830 Build/LTE) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45865[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; IM-A830K Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S45866[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SKY-A830S/L/K Build/LTE) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45867[] = "Mozilla/5.0 (X11; Linux x86_64; rv:22.0) Gecko/X Firefox/22.0 Iceweasel/22.0";
const char S45868[] = "Mozilla/5.0 (X11; Linux i686 on x86_64; rv:22.0) Gecko/X Firefox/22.0";
const char S45869[] = "Mozilla/5.0 (X11; Linux x86_64; rv:22.0) Gecko/X Iceweasel/22.0";
const char S45870[] = "Nokia200/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia200) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S45871[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB611 Build/BGN_1.1.23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45872[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/BGN_1.0.23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S45873[] = "Mozilla/5.0 (Linux; U; Android2.2; en-us; MB611 Build/BGN_1.0.23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45874[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/STAB2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45875[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us;MB611 Build/BGN_1.0.23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45876[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB611 Build/BGN_1.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45877[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/S246) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45878[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/BGN_1.0.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45879[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB611 Build/BGN_1.1.18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45880[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/1.0.11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45881[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/BGN_1.0.23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45882[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/1.0.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45883[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB611 Build/BGN_1.1.29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45884[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/BGN_1.0.24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45885[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB611 Build/BGN_1.1.30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S45886[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/1.0.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45887[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/S266) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45888[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB611 Build/BGN_1.1.25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45889[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/BGN_1.0.15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45890[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;MB611 Build/BGN_1.1.30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45891[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/S269) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45892[] = "Mozilla/5.0 (Linux; U;Android 2.2.2; en-us; MB611 Build/BGN_1.1.30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45893[] = "Mozilla/5.0 (Linux; U; Android2.2.2; en-us; MB611 Build/BGN_1.1.30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45894[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB611 Build/BGN_1.1.27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45895[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB611 Build/BGN_1.1.30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45896[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/BGN_1.0.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45897[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/S275) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45898[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB611 Build/BGN_1.1.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45899[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; MB611 Build/BGN_1.1.30)";
const char S45900[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; MB611 Build/BGN_1.0.23)";
const char S45901[] = "Mozilla/5.0 (Linux; Android 4.0.4; C5170 Build/IML77) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45902[] = "Mozilla/5.0 (Linux; Android 4.0.4; Hydro Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S45903[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45904[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45905[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45906[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45907[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-R530U Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45908[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45909[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45910[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I747 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45911[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L710 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45912[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530U Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45913[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8010 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S45914[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8010 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S45915[] = "Opera/9.80 (X11; Linux x86_64; U; ru) Presto/2.10.229 Version/11.64";
const char S45916[] = "Mozilla/5.0 (X11; Linux i686; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.64";
const char S45917[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-S950C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45918[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-S950C Build/GINGERBREAD)";
const char S45919[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID8048 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S45920[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; MID8048 Build/IML74K)";
const char S45921[] = "Mozilla/5.0 (Linux; U; Android 3.2; it-it; SmartTab10-MSM8260-V03a-MarX-Vodafone-IT) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45922[] = "Mozilla/5.0 (Linux; U; Android 3.2; tr-tr; SmartTab10-MSM8260-V03a-MarX-Vodafone-TR) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45923[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-ie; SmartTab10-MSM8260-V03a-MarX-Vodafone-IE) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45924[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; SmartTab10-MSM8260-V02d-DecX-Vodafone-ZA) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45925[] = "Mozilla/5.0 (Linux; U; Android 3.2; sr-rs; SmartTab10-MSM8260-V02d-DecX-Vodafone-RS) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45926[] = "Mozilla/5.0 (Linux; U; Android 3.2; es-es; SmartTab10-MSM8260-V02d-DecX-Vodafone-ES) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45927[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-nz; SmartTab10-MSM8260-V02d-DecX-Vodafone-NZ) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45928[] = "Mozilla/5.0 (Linux; U; Android 3.2; tr-tr; SmartTab10-MSM8260-V02d-DecX-Vodafone-TR) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45929[] = "Mozilla/5.0 (Linux; U; Android 3.2; hu-hu; SmartTab10-MSM8260-V03a-MarX-Vodafone-HU) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45930[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; SmartTab10-MSM8260-V03b-AprX-Vodafone-ZA) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45931[] = "Mozilla/5.0 (Linux; U; Android 3.2; nl-nl; SmartTab10-MSM8260-V03b-AprX-Vodafone-NL) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45932[] = "Mozilla/5.0 (Linux; U; Android 3.2; it-it; SmartTab10-MSM8260-V03b-AprX-Vodafone-IT) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45933[] = "Mozilla/5.0 (Linux; U; Android 3.2; tr-tr; SmartTab10-MSM8260-V03b-MayX-Vodafone-TR) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45934[] = "Mozilla/5.0 (Linux; U; Android 3.2; hu-hu; SmartTab10-MSM8260-V02d-DecX-Vodafone-HU) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45935[] = "Mozilla/5.0 (Linux; U; Android 3.2; es-es; SmartTab10-MSM8260-V03z-AprX-Vodafone-ES-GotaTest) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45936[] = "Mozilla/5.0 (Linux; U; Android 3.2; cs-cz; SmartTab10-MSM8260-V03a-MarX-Vodafone-CZ) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45937[] = "Mozilla/5.0 (Linux; U; Android 3.2; hu-hu; SmartTab10-MSM8260-V03b-AprX-Vodafone-HU) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45938[] = "Mozilla/5.0 (Linux; U; Android 3.2; ro-ro; SmartTab10-MSM8260-V03a-MarX-Vodafone-RO) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45939[] = "Mozilla/5.0 (Linux; U; Android 3.2; ro-ro; SmartTab10-MSM8260-V03b-AprX-Vodafone-RO) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45940[] = "Mozilla/5.0 (Linux; U; Android 3.2; nl-nl; SmartTab10-MSM8260-V03a-MarX-Vodafone-NL) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45941[] = "Mozilla/5.0 (Linux; U; Android 3.2; el-gr; SmartTab10-MSM8260-V03a-MarX-Vodafone-GR) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45942[] = "Mozilla/5.0 (Linux; U; Android 3.2; es-es; SmartTab10-MSM8260-V03b-AprX-Vodafone-ES) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45943[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; SmartTab10-MSM8260-V03b-AprX-Vodafone-RS) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45944[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; SmartTab10-MSM8260-V03a-MarX-Vodafone-LK) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S45945[] = "Mozilla/5.0 (Linux; U; Android 3.2; el-gr; SmartTab10-MSM8260-V03b-AprX-Vodafone-GR) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45946[] = "Mozilla/5.0 (Linux; U; Android 3.2; it-it; SmartTab10-MSM8260-V03b-AprX-Vodafone-IT) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S45947[] = "Mozilla/5.0 (Linux; U; Android 3.2; vi-vn; SmartTab10-MSM8260-V03a-FebX-Vodafone-DE) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S45948[] = "Opera/9.10 (Windows NT 5.1; U; en)";
const char S45949[] = "Opera/9.10 (Windows NT 5.2; U; en)";
const char S45950[] = "Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.8.0) Gecko/X Firefox/1.5.0 Opera 9.10";
const char S45951[] = "Opera/9.12 (Windows NT 5.1; U; ru)";
const char S45952[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; en) Opera 9.12";
const char S45953[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N9100 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S45954[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; N9100 Build/IMM76I)";
const char S45955[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8013 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S45956[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8013 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S45957[] = "Mozilla/5.0 (Series40; NokiaC3-00/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S45958[] = "Mozilla/5.0 Firefox/3.5.6";
const char S45959[] = "Mozilla/5.0 Firefox/3.5.6 Tapatalk/";
const char S45960[] = "Mozilla/5.0 Firefox/3.5.6 Tapatalk/1.14.2";
const char S45961[] = "Mozilla/5.0 Firefox/3.5.6 Tapatalk/2.4";
const char S45962[] = "Mozilla/5.0 Firefox/3.5.6 Tapatalk/1.14.3";
const char S45963[] = "Mozilla/5.0 Firefox/3.5.6 Tapatalk/2.4.5";
const char S45964[] = "Mozilla/5.0 Firefox/3.5.6 Tapatalk/1.15.0";
const char S45965[] = "Mozilla/5.0 Firefox/3.5.6 Tapatalk/1.15.1";
const char S45966[] = "Mozilla/5.0 Firefox/3.5.6 Tapatalk/2.4.7";
const char S45967[] = "Mozilla/5.0 Firefox/3.5.6 Tapatalk/2.4.10";
const char S45968[] = "Mozilla/5.0 Firefox/3.5.6 Tapatalk/1.16.0";
const char S45969[] = "Mozilla/5.0 Firefox/3.5.6 Tapatalk/2.4.11";
const char S45970[] = "Mozilla/5.0 Firefox/3.5.6 Tapatalk/2.4.13";
const char S45971[] = "Mozilla/5.0 Firefox/3.5.6 Tapatalk/1.17.1";
const char S45972[] = "Mozilla/5.0 Firefox/3.5.6 Tapatalk/2.0.2";
const char S45973[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; GT-I8350)";
const char S45974[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; Omnia W; Orange)";
const char S45975[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; GT-I8350T)";
const char S45976[] = "AppleCoreMedia/1.0.0.10B141 (iPad; U; CPU OS 6_1 like Mac OS X; en_us)";
const char S45977[] = "AppleCoreMedia/1.0.0.10B146 (iPad; U; CPU OS 6_1_2 like Mac OS X; en_us)";
const char S45978[] = "AppleCoreMedia/1.0.0.10B329 (iPad; U; CPU OS 6_1_3 like Mac OS X; en_us)";
const char S45979[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; Milestone XT720 Build/STR_U2_02.1E.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45980[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; Milestone XT720 Build/Itfunz_XT720_NewBegin) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45981[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Milestone XT720 Build/STR_U2_03.20.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45982[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Milestone XT720 Build/STR_U2_01.1E.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45983[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; Motorola XT720 Build/STR_U2_05.1F.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45984[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-ca; Motorola XT720 Build/STR_U2_05.1F.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45985[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-ru; Milestone XT720 Build/STR_U2_03.20.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45986[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Milestone XT720 Build/STR_U2_01.18.3) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45987[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1_Hiapk_1.1_XXKJ; zh-cn; Milestone XT720 Build/STR_U2_01.18.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45988[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Milestone XT720 Build/STR_U2_00.18.0D) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45989[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Milestone XT720 Build/STR_U2_01.18.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45990[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Milestone XT720 Build/STR_U2_04.1E.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45991[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Milestone XT720 Build/STR_U2_02.18.3) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45992[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Milestone XT720 Build/STR_U2_04.1E.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45993[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Milestone XT720 Build/STR_U2_01.1E.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45994[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Milestone XT720 Build/STR_U2_00.15.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45995[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-kr; XT720 Build/STSKT_N_79.11.39R) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45996[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-kr; XT720 Build/KyuHyuk ROM for XT720 1.5.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45997[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT720 Build/STSKT_N_79.11.36R1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45998[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT720 Build/STSKT_N_79.22.44R) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S45999[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Milestone XT720 Build/STR_U2_01.1E.0)";
const char S46000[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-kr; XT720 Build/STSKT_N_79.22.39R) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S46001[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT720 Build/STSKT_N_79.11.41R) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S46002[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT720 Build/STSKT_N_79.11.44R) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S46003[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 521)";
const char S46004[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 10 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S46005[] = "Mozilla/5.0 (Linux; Android 4.2; Nexus 10 Build/JVP15P) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S46006[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 10 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S46007[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; SPV M3100; OpVer 14.123.1.614)";
const char S46008[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x240)";
const char S46009[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 240x320)";
const char S46010[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; SPV M700; OpVer 19.123.1.615)";
const char S46011[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 240x320; SPV C600; OpVer 11.2.1.5)";
const char S46012[] = "TyTN Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)";
const char S46013[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Sprint:PPC-6700; PPC; 240x320)";
const char S46014[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; SPV M600; OpVer 13.10.1.107)";
const char S46015[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 240x320; SPV C600; OpVer 11.2.1.8)";
const char S46016[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 480x640; SPV M5000; OpVer 7.130.1.145)";
const char S46017[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; SDA/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1)";
const char S46018[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; SDA/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1)";
const char S46019[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)";
const char S46020[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 176x220; Smartphone; SDA/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1)";
const char S46021[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 176x220; SPV C500; OpVer 4.1.1.4)";
const char S46022[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 176x220; SPV C500; OpVer 4.1.2.0)";
const char S46023[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; SPV M2000; OpVer 5.31.1.124)";
const char S46024[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; SPV M2000; OpVer 5.40.2.223)";
const char S46025[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; 240x320)";
const char S46026[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; SPV M500; OpVer 6.06.3.123)";
const char S46027[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; SPV M600; Configuration/CLDC-1.1)";
const char S46028[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; SPV M5000; Configuration/CLDC-1.1)";
const char S46029[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)";
const char S46030[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; SPV M600; OpVer 13.10.2.112)";
const char S46031[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; SPV M1500; OpVer 3.11.1.167)";
const char S46032[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 240x320; SPV C550; OpVer 8.2.1.10)";
const char S46033[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 320x240; SPV E600; OpVer 20.118.1.615)";
const char S46034[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Sprint:SPH-ip830w; PPC; 240x320)";
const char S46035[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 176x220)";
const char S46036[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows NT Windows CE)";
const char S46037[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; MSN Companion 2.0; 800x600; Compaq)";
const char S46038[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID9742 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46039[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; MID9742 Build/IML74K)";
const char S46040[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:2.0.1) Gecko/X Firefox/19.0.2";
const char S46041[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SC-02C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46042[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SC-02C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46043[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-; SC-02C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46044[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ja-jp; SC-02C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46045[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SC-02C Build/GINGERBREAD)";
const char S46046[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-; SC-02C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46047[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ja-; SC-02C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46048[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SC-02C Build/GINGERBREAD)";
const char S46049[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SC-02C Build/GINGERBREAD)";
const char S46050[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja_jp; SC-02C Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)";
const char S46051[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-P880 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46052[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P880 Build/JZO54K)";
const char S46053[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P880/P88020a-EUR-XXX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46054[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; LG-P880/V20a Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46055[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P880/P88020b-EUR-XXX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46056[] = "Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10 UCBrowser/2.3.2.300";
const char S46057[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46058[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; KFJWI Build/IMM76D)";
const char S46059[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; KFJWI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46060[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; KFJWI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46061[] = "Nokia3110c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S46062[] = "Nokia3110c/2.0 (06.02) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia3110c";
const char S46063[] = "Nokia3110c_CMCC/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S46064[] = "Nokia3110";
const char S46065[] = "SAMSUNG-S8000-Bouygues/S8000AGIG4 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S46066[] = "SAMSUNG-S8000/S8000XPJA1 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 SS-Widget/S8000-FM";
const char S46067[] = "SAMSUNG-S8000-ORANGE/S8000BVIG4 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S46068[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; cm_tenderloin Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46069[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; cm_tenderloin Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46070[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PG8600 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46071[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; skz_tenderloin Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46072[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; pa_tenderloin Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46073[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.126 Safari/533.4";
const char S46074[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 928)";
const char S46075[] = "NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-01) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S46076[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-P970 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46077[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P970 Build/IMM76L)";
const char S46078[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; LG-P970/P97030b-EUR-XX Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46079[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-sa; LG-P970h Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46080[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; LG-P970-Orange Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46081[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-P970g-parrot Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46082[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-P970g Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46083[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; LG-P970/P97030c-208-10 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46084[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; LG-P970/P97030c Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46085[] = "Mozilla/5.0 (iPad; U; CPU OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B141 Safari/7534.48.3";
const char S46086[] = "Mozilla/5.0 (iPad; U; CPU OS 6_1_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B146 Safari/7534.48.3";
const char S46087[] = "Mozilla/5.0 (iPad; U; CPU OS 6_1_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B329 Safari/7534.48.3";
const char S46088[] = "Mozilla/5.0 (iPad; U; CPU OS 6_1_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B147 Safari/7534.48.3";
const char S46089[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:8.0.1) Gecko/X Firefox/8.0.1";
const char S46090[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGPT13 Build/TID0142) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S46091[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGPT12 Build/TID0142) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S46092[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Twitter for iPhone";
const char S46093[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Twitter for iPhone";
const char S46094[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Twitter for iPhone";
const char S46095[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9Z999 Twitter for iPhone";
const char S46096[] = "BlackBerry9500/5.0.0.841 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/120";
const char S46097[] = "BlackBerry9630fts/4.7.1.40 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/104";
const char S46098[] = "BlackBerry9630/4.7.1.61 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105";
const char S46099[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; HD7 T9292)";
const char S46100[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; HD7)";
const char S46101[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; T9296)";
const char S46102[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/ 7.0; HTC; HD7 T9292)";
const char S46103[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Gold)";
const char S46104[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/ 7.0; WpsLondonTest; HTC; HD7 T9292)";
const char S46105[] = "Mozilla/4.0 (compatible; MSIE 8.0;";
const char S46106[] = "MSIE 8.0";
const char S46107[] = "Mozilla/4.0 (compatible; MSIE 8.0; ; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Policy)";
const char S46108[] = "Mozilla/5.0 (compatible; MSIE 8.0; PPC; Trident/4.0)";
const char S46109[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident/4.0)";
const char S46110[] = "Mozilla/5.0 (compatible; MSIE 8.0; PPC; SV1; Trident/1.0)";
const char S46111[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P5110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S46112[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P5110 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S46113[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI U8950N-51 Build/HuaweiU8950N-51) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46114[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI U8950D Build/HuaweiU8950D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46115[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HUAWEI U8950N-1 Build/HuaweiU8950N-1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46116[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U8950D Build/HuaweiU8950D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S46117[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI U8950-1 Build/HuaweiU8950-1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46118[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-ve; HUAWEI U8950-51 Build/HuaweiU8950-51) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46119[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI U8950D Build/HuaweiU8950D)";
const char S46120[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI U8950-1 Build/HuaweiU8950-1)";
const char S46121[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI U8950N-51 Build/HuaweiU8950N-51)";
const char S46122[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P7500 Build/HTJ85B) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S46123[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S46124[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500R Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S46125[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500M Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S46126[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S46127[] = "Mozilla/5.0 (Linux; Android 4.0.3;  GT-P7500 Build/HTJ85B) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S46128[] = "Mozilla/5.0 (Linux; Android 4.0.4;  GT-P7500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S46129[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S46130[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 wI-fI & 3g Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S46131[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 WIFI & 3G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S46132[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.11 (KHTML, like Gecko) Ubuntu/11.04 Chromium/17.0.963.79 Chrome/17.0.963.79 Safari/535.11";
const char S46133[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.11 (KHTML, like Gecko) Ubuntu/11.04 Chromium/17.0.963.65 Chrome/17.0.963.65 Safari/535.11";
const char S46134[] = "Firefox/7.0.1";
const char S46135[] = "RebelMouse/0.1 Mozilla/5.0 (compatible; http://rebelmouse.com) Gecko/X Firefox/7.0.1";
const char S46136[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_C525c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46137[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C525c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46138[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; C525c Build/JZO54K)";
const char S46139[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46140[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; i9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46141[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46142[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; i9220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46143[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46144[] = "Mozilla/5.0 (Linux; U; Android 4.0.6; en-us; i9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46145[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; I9220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46146[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9220 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S46147[] = "Mozilla/5.0 (Linux; U; Android 4.0.5; en-us; HK-I9220 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46148[] = "Mozilla/5.0 (Linux; U; Android 4.0.6; en-us; i9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46149[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; GT-I9220 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46150[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9220 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S46151[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; GT-I9220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46152[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46153[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.6; i9220 Build/IML74K)";
const char S46154[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9220 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46155[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9220 Build/IMM76D)";
const char S46156[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46157[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; GT_I9220 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46158[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; i9220 Build/IML74K)";
const char S46159[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I9220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46160[] = "GT-I9220 Linux/3.0.13 Android/4.1.1 Release/07.21.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S46161[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; i9220 Build/IML74K)";
const char S46162[] = "Mozilla/5.0 (Linux; U; Android 4.0.6; en-us; Angelobiz v3.2 for Haipai Noble I9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46163[] = "Mozilla/5.0 (Linux; U; Android 4.0.6; en-us; I9220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46164[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; i9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46165[] = "Dalvik/1.4.0 (Linux; U; Android 4.0.9; GT_I9220 Build/GINGERBREAD.XXKL3)";
const char S46166[] = "Mozilla/5.0 (Linux; U; Android 4.0.6; it-it; i92200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46167[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-i705 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46168[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I705 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46169[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-i705 Build/IMM76D)";
const char S46170[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; SCH-i705 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46171[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; XT316 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46172[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT316 Build/V4.26F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46173[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT316 Build/V4.26M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46174[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT316 Build/V4.26H) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46175[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT316 Build/V4.26P) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46176[] = "MOT-XT316/Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; XT316 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46177[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT316 Build/V4.26K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46178[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT316 Build/V4.26O) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46179[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT316 Build/V4.26U) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46180[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; XT316 Build/V4.620) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46181[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT316 Build/V4.34A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46182[] = "MOT-XT316/Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; XT316 Build/V4.480) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46183[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; XT316 Build/V4.630) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46184[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT316 Build/V4.340) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46185[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT316 Build/V4.26Q) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46186[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; XT316 Build/V4.26R) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46187[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT316 Build/V4.26T) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46188[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; XT316 Build/V4.500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46189[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; XT316 Build/V4.650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46190[] = "MOT-XT316/1.0 Android/2.3.4 Release/9.15.2011 Browser/AppleWebKit533.1 Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; XT316 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46191[] = "Mozilla/5.0 (Linux; U; Android 2.3.5 Ambras Build; es-us; XT316 Build/V4.620) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46192[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S46193[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:16.0) Gecko/X Thunderbird/16.0.2";
const char S46194[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P5113 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S46195[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P5113 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S46196[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; HTC-Vivo Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46197[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC-Vivo Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46198[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_H3000C Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46199[] = "SAMSUNG-GT-C6712/C6712DDKK3 SHP/VPP/R5 Dolfin/2.0 NexPlayer/3.0 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B";
const char S46200[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8552 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46201[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8552 Build/JZO54K)";
const char S46202[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; GT-I8552B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46203[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-I8552 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46204[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8552B Build/JZO54K)";
const char S46205[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S46206[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S46207[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S46208[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S46209[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S46210[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S46211[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S46212[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S46213[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S46214[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S46215[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S46216[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46217[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; ja-jp; SonyEricssonSO-01C Build/3.0.D.2.79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46218[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; ja-jp; SO-01C Build/3.0.D.2.79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46219[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; ja-jp; SonyEricssonSO-01C Build/3.0.1.D.2.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46220[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SO-01C Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46221[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonSO-01C Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46222[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SO-01C Build/3.0.1.F.0.131) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46223[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonSO-01C Build/3.0.1.F.0.131) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46224[] = "Mozilla/5.0 (Linux; U; Android 2.3; ja_jp; SO-01C Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.2 Mobile (Dragonfruit)";
const char S46225[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SO-01C Build/3.0.1.F.0.126) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46226[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonSO-01C Build/3.0.1.F.0.126) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46227[] = "Mozilla/5.0 (Linux; U;Android 2.3.2; ja-jp; SonyEricssonSO-01C Build/3.0.D.2.79)AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46228[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-01C Build/4.0.1.C.1.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46229[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SO-01C Build/4.0.1.C.1.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46230[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-01C Build/4.0.1.C.1.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46231[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SO-01C Build/4.0.1.C.1.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46232[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SO-01C Build/3.0.1.F.0.131)";
const char S46233[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-01C Build/4.0.1.C.1.24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46234[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SO-01C Build/4.0.1.C.1.24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46235[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.2; SO-01C Build/3.0.D.2.79)";
const char S46236[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-01C Build/4.0.1.C.1.31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46237[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SO-01C Build/4.0.1.C.1.31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46238[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SO-01C Build/4.0.1.C.1.31)";
const char S46239[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46240[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710BST Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46241[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710VMUB Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46242[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8013 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S46243[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8013 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S46244[] = "SAMSUNG-SGH-A667/A667UCLD2; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.20822; U; en-US) Opera 9.50";
const char S46245[] = "Mozilla/5.0 (Linux; U; Android 2.2; de-de; LG-E720/V10b; Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S46246[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; it-it; LG-E720 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S46247[] = "Mozilla/5.0 (Linux; U; Android 2.2; it-it; LG-E720 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S46248[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-ca; LG-E720b Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S46249[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; LG-E720/V10d; Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2";
const char S46250[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; LG-E720b Build/FRF91)";
const char S46251[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; LG-E720 Build/FRF91)";
const char S46252[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530C Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S46253[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S46254[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I747M Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S46255[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-L710 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S46256[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530U Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S46257[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S46258[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210L Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S46259[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210K Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S46260[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530C Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S46261[] = "Mozilla/5.0 (Linux; Android 4.0.4; Galaxy S III Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S46262[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S46263[] = "Mozilla/5.0 (Linux; U;Android 1.6; en-us; LG-GT540; Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S46264[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; GT540 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S46265[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; LG-GT540/V10b; Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S46266[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; LG-GT540g; Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S46267[] = "LG-GT540f/v10a Mozilla/5.0 (Linux; U; Android 1.6; en-us; GT540f; Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S46268[] = "Mozilla/5.0 (Linux; U; Android1.6; en-us; LG-GT540; Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S46269[] = "LG-GT540 Mozilla/5.0 (Linux; U; Android 1.6; en-us; GT540; Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S46270[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; LG-GT540f; Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S46271[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; LG-GT540; Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S46272[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; LG-GT540 ; Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S46273[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46274[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970C Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46275[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970X Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46276[] = "Mozilla/5.0 (webOS/1.0; U; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Pre/1.0";
const char S46277[] = "Mozilla/5.0 (webOS/1.1; U; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Pre/1.0";
const char S46278[] = "Mozilla/5.0 (webOS/1.2; U; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Pre/1.0";
const char S46279[] = "Mozilla/5.0 (webOS/1.3.1; U; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Pre/1.0";
const char S46280[] = "Mozilla/5.0 (webOS/1.3.5; U; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Pre/1.0";
const char S46281[] = "Mozilla/5.0 (webOS/1.4.1.1; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0";
const char S46282[] = "Mozilla/5.0 (webOS/1.4.5; U; de-DE) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0";
const char S46283[] = "Mozilla/5.0 (webOS/1.4.5; U; en-GB) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0";
const char S46284[] = "Mozilla/5.0 (webOS/1.4.5; U; en-CA) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0";
const char S46285[] = "Mozilla/5.0 (webOS/1.4.5; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0";
const char S46286[] = "Mozilla/5.0 (webOS/1.4.5; U; es-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0";
const char S46287[] = "Mozilla/5.0 (webOS/1.4.5; U; fr-CA) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0";
const char S46288[] = "Mozilla/5.0 (webOS/1.4.0; U; es-MX) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0";
const char S46289[] = "Mozilla/5.0 (webOS/1.4.0; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0";
const char S46290[] = "Mozilla/5.0 (webOS/1.4.5; U; es-ES) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0";
const char S46291[] = "Mozilla/5.0 (webOS/1.4.5; U; en-IE) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0";
const char S46292[] = "Mozilla/5.0 (webOS/2.1.0; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0";
const char S46293[] = "Mozilla/5.0 (webOS/1.4.1.1; U; es-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0";
const char S46294[] = "Mozilla/5.0 (webOS/1.3.5.1; U; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Pre/1.0";
const char S46295[] = "Mozilla/5.0 (webOS/2.1.0; U; en-GB) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0";
const char S46296[] = "Mozilla/5.0 (webOS/1.4.5; U; it-IT) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0";
const char S46297[] = "Mozilla/5.0 (webOS/2.1.0; U; de-DE) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0";
const char S46298[] = "Mozilla/5.0 (webOS/1.4.5; U; zh-CN) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0";
const char S46299[] = "Mozilla/5.0 (webOS/2.1.0; U; es-MX) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0";
const char S46300[] = "Mozilla/5.0 (webOS/1.4.5; U; es-MX) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0";
const char S46301[] = "Mozilla/5.0 (webOS/1.1; U; en-CA) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Pre/1.0";
const char S46302[] = "Mozilla/5.0 (webOS/1.4.5; U; fr-FR) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0";
const char S46303[] = "Mozilla/5.0 (webOS/1.4.5; U; en-US) AppleWebKit/532.2 (KHTML	 like Gecko) Version/1.0 Safari/532.2 Pre/1.0";
const char S46304[] = "Mozilla/5.0 (webOS/1.4.0; U; es-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0";
const char S46305[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46306[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46307[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46308[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46309[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46310[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M480W Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46311[] = "Mozilla/5.0 (Linux; Android 4.1.2; SC-05D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46312[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46313[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46314[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_DesireHD/1.0 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46315[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Desire HD Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46316[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Desire HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46317[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_DesireHD_A9191 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46318[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Desire HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46319[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Desire HD Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46320[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Desire HD Build/IML74K)";
const char S46321[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Desire HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46322[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Desire_HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46323[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Desire HD Build/IMM76L)";
const char S46324[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Desire HD Build/HuaweiU8815) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46325[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_DesireHD_A9191 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46326[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Desire HD Build/IML74K)";
const char S46327[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Desire_HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46328[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_Desire_HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46329[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_DesireHD Build/ViperDHD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46330[] = "Mozilla/5.0 (Linux; U; Android 4.0.3 (B952); en-gb; Desire HD Build/HuaweiU8815) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46331[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_DesireHD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46332[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Desire HD Build/ViperDHD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46333[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Desire HD Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46334[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; HTC_DesireHD_X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46335[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Desire HD Build/SVHD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46336[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ua; HTC Desire HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46337[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; HTC_DesireHD_A9191 Build/ucxl) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46338[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Desire_HD Build/SVHD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46339[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P5110 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S46340[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P5110 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S46341[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36";
const char S46342[] = "Nokia6230i/2.0 (03.70) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6230i";
const char S46343[] = "Nokia6230i/2.0 (03.25) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S46344[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; USCC-US730 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46345[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-US730 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46346[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-US730 Build/IMM76L)";
const char S46347[] = "Opera/9.80 (X11; Linux i686) Presto/2.12.388 Version/12.12";
const char S46348[] = "Mozilla/5.0 (X11; Linux x86_64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.12";
const char S46349[] = "Mozilla/5.0 (compatible; MSIE 9.0; X11; Linux i686) Opera 12.12";
const char S46350[] = "Opera/9.80 (X11; Linux x86_64; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)) Presto/2.12.388 Version/12.12";
const char S46351[] = "Opera/9.80 (X11; Linux i686) Presto/2.12.388 Version/12.12,gzip(gfe)";
const char S46352[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; HTC Panache Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46353[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Panache Build/GRI40)";
const char S46354[] = "Mozilla/5.0 (Linux; Android 4.0.4; MB886 Build/7.7.1Q-144_MB886_MR1-22) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S46355[] = "SAMSUNG-GT-S3802 Opera/9.80 (J2ME/MIDP; Opera Mini/7.1.32830/30.3697; U; en) Presto/2.8.119 Version/11.10";
const char S46356[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S46357[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S46358[] = "Mozilla/5.0 (Nintendo WiiU) AppleWebKit/534.52 (KHTML, like Gecko) NX/2.1.0.10.9 NintendoBrowser/1.5.0.8047.EU";
const char S46359[] = "Mozilla/5.0 (Nintendo WiiU) AppleWebKit/534.52 (KHTML, like Gecko) NX/2.1.0.10.9 NintendoBrowser/1.5.0.8047.US";
const char S46360[] = "Mozilla/5.0 (Nintendo WiiU) AppleWebKit/534.52 (KHTML, like Gecko) NX/2.1.0.10.9 NintendoBrowser/1.5.0.8047.JP";
const char S46361[] = "SAMSUNG-GT-E2202 Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.32420/31.1475; U; en) Presto/2.8.119 Version/11.10";
const char S46362[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 822)";
const char S46363[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LS860 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46364[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-LS860 Build/IMM76I)";
const char S46365[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mach Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46366[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S46367[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970C Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S46368[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970X Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S46369[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8010 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S46370[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8010 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S46371[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-sa; LG-E615f Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46372[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-E615 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46373[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-E615 Build/IMM76D)";
const char S46374[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-E615f Build/IMM76D)";
const char S46375[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P5110 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S46376[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P5110 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S46377[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5303 Build/12.0.A.1.257) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S46378[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P5100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S46379[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P5100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S46380[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T779 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S46381[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; DROIDX Build/VZW) AppleWebKit/525.10  (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S46382[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; DROIDX Build/VZW) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 480X854 motorola DROIDX companydatatrees";
const char S46383[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.0.200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S46384[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LS970 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S46385[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9000 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46386[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-I9000 Build/JOP40D)";
const char S46387[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9000 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46388[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9000 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46389[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; GT-I9000 Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46390[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9000 Build/JDQ39E)";
const char S46391[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9000 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46392[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46393[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; IdeaTabS2109A-F Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S46394[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IdeaTabS2109A-F Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S46395[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; IdeaTabS2109A-F Build/IMM76D)";
const char S46396[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; PantechP4100 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S46397[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; PantechP4100 Build/HTK75)";
const char S46398[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46399[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8013 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S46400[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8013 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S46401[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT890 Build/9.8.2I-50_SMI-26) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46402[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCEVOV4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46403[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTCEVOV4G Build/IML74K)";
const char S46404[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCEVOV4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S46405[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; HTCEVOV4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46406[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S46407[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S46408[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S46409[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S46410[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S46411[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M480W Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S46412[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S46413[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S46414[] = "Mozilla/5.0 (Linux; Android 4.1.2; SC-05D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S46415[] = "Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S46416[] = "Mozilla/5.0 (FreeBSD; U; Viera; en-GB) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46417[] = "Mozilla/5.0 (FreeBSD; U; Viera; de-DE) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46418[] = "Mozilla/5.0 (FreeBSD; U; Viera; en-US) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1";
const char S46419[] = "Mozilla/5.0 (FreeBSD; U; Viera; nl-NL) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46420[] = "Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.804.0 Safari/535.1";
const char S46421[] = "Mozilla/5.0 (FreeBSD; U; Viera; en-00) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1";
const char S46422[] = "Mozilla/5.0 (FreeBSD; U; Viera; es-ES) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46423[] = "Mozilla/5.0 (FreeBSD; U; Viera; en-NZ) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1";
const char S46424[] = "Mozilla/5.0 (FreeBSD; U; Viera; en-AU) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1";
const char S46425[] = "Mozilla/5.0 (FreeBSD; U; Viera; en-IE) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46426[] = "Mozilla/5.0 (FreeBSD; U; Viera; it-IT) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46427[] = "Mozilla/5.0 (FreeBSD; U; Viera; pl-PL) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46428[] = "Mozilla/5.0 (FreeBSD; U; Viera; fr-FR) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46429[] = "Mozilla/5.0 (FreeBSD; U; Viera; fr-CA) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1";
const char S46430[] = "Mozilla/5.0 (FreeBSD; U; Viera; da-DK) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46431[] = "Mozilla/5.0 (FreeBSD; U; Viera; es-BR) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1";
const char S46432[] = "Mozilla/5.0 (FreeBSD; U; Viera; cs-CZ) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46433[] = "Mozilla/5.0 (FreeBSD; U; Viera; tr-TR) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46434[] = "Mozilla/5.0 (FreeBSD; U; Viera; es-US) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1";
const char S46435[] = "Mozilla/5.0 (FreeBSD; U; Viera; bg-BG) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46436[] = "Mozilla/5.0 (FreeBSD; U; Viera; fi-FI) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46437[] = "Mozilla/5.0 (FreeBSD; U; Viera; el-GR) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46438[] = "Mozilla/5.0 (FreeBSD; U; Viera; sv-SE) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46439[] = "Mozilla/5.0 (FreeBSD; U; Viera; hr-HR) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46440[] = "Mozilla/5.0 (FreeBSD; U; Viera; ru-RU) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46441[] = "Mozilla/5.0 (FreeBSD; U; Viera; pt-BR) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1";
const char S46442[] = "Mozilla/5.0 (FreeBSD; U; Viera; no-NO) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46443[] = "Mozilla/5.0 (FreeBSD; U; Viera; th-00) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1";
const char S46444[] = "Mozilla/5.0 (FreeBSD; U; Viera; hu-HU) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46445[] = "Mozilla/5.0 (FreeBSD; U; Viera; sk-SK) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46446[] = "Mozilla/5.0 (FreeBSD; U; Viera; pt-PT) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46447[] = "Mozilla/5.0 (FreeBSD; U; Viera; ja-JP) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1";
const char S46448[] = "Mozilla/5.0 (FreeBSD; U; Viera; en-SG) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1";
const char S46449[] = "Mozilla/5.0 (FreeBSD; U; Viera; ro-RO) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46450[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; en-GB) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46451[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; it-IT) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46452[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; en-US) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1";
const char S46453[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; en-NZ) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1";
const char S46454[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; en-AU) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1";
const char S46455[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; nl-NL) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46456[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; fr-FR) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46457[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; de-DE) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46458[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; cs-CZ) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46459[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; es-ES) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46460[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; en-00) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1";
const char S46461[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; pl-PL) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46462[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; tr-TR) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46463[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; en-IE) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46464[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; pt-BR) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1";
const char S46465[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; da-DK) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46466[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; sv-SE) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1";
const char S46467[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; es-BR) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1";
const char S46468[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One X Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46469[] = "Mozilla/5.0 (Linux; Android 4.2.2; One X Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46470[] = "Mozilla/5.0 (X11; Linux i686; rv:8.0) Gecko/X Firefox/8.0 Cunaguaro/8.0";
const char S46471[] = "Mozilla/5.0; (X11; Linux x86_64; rv:8.0) Gecko/X Firefox/8.0";
const char S46472[] = "Mozilla/5.0 (X11; Linux i686 on x86_64; rv:8.0) Gecko/X Firefox/8.0 Iceweasel/8.0";
const char S46473[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46474[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S46475[] = "Mozilla 5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build JRO03L) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S46476[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2S Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S46477[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2C Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46478[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2S Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46479[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2SC Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46480[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2A Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46481[] = "AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; MI 2 Build/JRO03L)";
const char S46482[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2SC Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S46483[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2A Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S46484[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; MI 2S MIUI/JLB21.0)";
const char S46485[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HTC_Butterfly Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46486[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HTC Butterfly Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46487[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC Butterfly Build/JRO03C)";
const char S46488[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC_X920e Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46489[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC X920e Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46490[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC X920e Build/JRO03C)";
const char S46491[] = "Opera/9.80 (Series 60; Opera Mini/5.1.22784/23.334; U; en) Presto/2.5.25 Version/10.54";
const char S46492[] = "Opera/9.80 (Series 60; Opera Mini/5.1.22783/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S46493[] = "NokiaX2-02/2.0 (11.57) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S46494[] = "NokiaX2-02/2.0 (11.79) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-02";
const char S46495[] = "NokiaX2-02/2.0 (11.79) Profile/MIDP-2.1 Configuration/CLDC-1.1 NokiaX2-02/2.0 (11.79) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S46496[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT890 Build/9.8.2I-50_SMI-26) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S46497[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I915 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46498[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-I915 Build/JZO54K)";
const char S46499[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S46500[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S46501[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ADR910L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46502[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ADR910L 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46503[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; ADR910L Build/JZO54K)";
const char S46504[] = "Opera/9.80 (Android 2.3.3; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.10.254 Version/12.00";
const char S46505[] = "Opera/9.80 (Android 2.3.5; Linux; Opera Mobi/ADR-X; U; de) Presto/2.10.254 Version/12.00";
const char S46506[] = "Opera/9.80 (Android 2.3.7; Linux; Opera Mobi/ADR-X; U; es-ES) Presto/2.10.254 Version/12.00";
const char S46507[] = "Opera/9.80 (Android 2.3.6; Linux; Opera Mobi/ADY-X; U; ru) Presto/2.10.254 Version/12.00";
const char S46508[] = "Opera/9.80 (Android 2.3.4; Linux; Opera Mobi/ADY-X; U; ru) Presto/2.10.254 Version/12.00";
const char S46509[] = "Opera/9.80 (Android 2.3.2; Linux; Opera Mobi/ADR-X; U; es-ES) Presto/2.10.254 Version/12.00";
const char S46510[] = "Opera/9.80 (Android 2.3.9; Linux; Opera Mobi/ADR-X; U; en) Presto/2.10.254 Version/12.00";
const char S46511[] = "Opera/9.80 (Android 2.3.1; Linux; Opera Mobi/ADR-X; U; en) Presto/2.10.254 Version/12.00";
const char S46512[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9100; es) AppleWebKit/534.8+ (KHTML, like Gecko) Version/6.0.0.534 Mobile Safari/534.8+";
const char S46513[] = "BlackBerry9100/6.0.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/331";
const char S46514[] = "SAMSUNG-GT-C3312R Opera/9.80 (J2ME/MIDP; Opera Mini/7.0.32584/30.3697; U; en) Presto/2.8.119 Version/11.10";
const char S46515[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PantechP9090 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46516[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; PantechP9090 Build/IMM76I)";
const char S46517[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; 007HW Build/HuaweiU8850) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46518[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HUAWEI-U8850 Build/HuaweiU8850) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46519[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; 007HW Build/HuaweiU8850) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46520[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HUAWEI-U8850 Build/HuaweiU8850) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46521[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HUAWEI-U8850 Build/HuaweiU8850)";
const char S46522[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; 007HW Build/HuaweiU8850) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46523[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; HUAWEI-U8850 Build/HuaweiU8850)";
const char S46524[] = "SoftBank/1.0/705NK/NKJ001 Series60/3.0 NokiaN73/3.0650 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S46525[] = "NokiaN73-2/3.0638.0.0.1_rm132 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S46526[] = "NokiaN73-5/3.0628.0.0.6 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S46527[] = "NokiaN73-1/4.0839.42.2.1Series60/3.0 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S46528[] = "NokiaN73-1/4.0839.42.2.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S46529[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.236; en-us; 9700/6.6.0.236) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46530[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.124; en-us; 9700/6.6.0.124) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46531[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.248; en-us; 9700/6.6.0.248) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46532[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.241; en-us; 9700/6.6.0.241) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46533[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.175; en-us; 9700/5.1.0.175) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46534[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.207; en-us; 9700/6.6.0.207) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46535[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.212; en-us; 9700/6.6.0.212) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46536[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.50; en-us; 9700/6.6.0.50) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46537[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.195; en-us; 9700/6.6.0.195) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46538[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.98; en-us; 9700/5.1.0.98) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46539[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.146; en-us; 9700/5.1.0.146) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46540[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.246; en-us; 9700/6.6.0.246) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46541[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.200; en-us; 9700/6.6.0.200) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46542[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.223; en-us; 9700/6.6.0.223) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46543[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.216; en-us; 9700/6.6.0.216) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46544[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.177; en-us; 9700/5.1.0.177) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46545[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.105; en-us; 9700/6.6.0.105) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46546[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.137; en-us; 9700/6.6.0.137) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46547[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.86; en-us; 9700/6.6.0.86) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46548[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.147; en-us; 9700/5.1.0.147) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46549[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.153; en-us; 9700/5.1.0.153) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46550[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.151; en-us; 9700/6.6.0.151) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46551[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.121; en-us; 9700/5.1.0.121) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46552[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.63; en-us; 9700/6.6.0.63) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46553[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.189; en-us; 9700/6.6.0.189) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46554[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.109; en-us; 9700/6.6.0.109) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46555[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.116; en-us; 9700/5.1.0.116) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46556[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.160; en-us; 9700/5.1.0.160) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46557[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.90; en-us; 9700/5.1.0.90) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46558[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.165; en-us; 9700/5.1.0.165) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46559[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.250; en-us; 9700/6.6.0.250) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46560[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.112; en-us; 9700/5.1.0.112) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46561[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.91; en-us; 9700/5.1.0.91) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S46562[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; G100W Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S46563[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; G100W Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S46564[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; G100W Build/HTK55D)";
const char S46565[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I957 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46566[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I957R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46567[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I957M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46568[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I957D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46569[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I957 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46570[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I957R Build/IMM76D)";
const char S46571[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SAMSUNG-SGH-I957 Build/IMM76D)";
const char S46572[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.2 Mobile/8H7 Safari/6533.18.5";
const char S46573[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.2 Mobile/8L1 Safari/6533.18.5";
const char S46574[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.2 Mobile/8J2 Safari/6533.18.5";
const char S46575[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; HTC-X710a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46576[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC-X710a Build/IML74K)";
const char S46577[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SO-02C Build/3.0.1.F.0.131) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46578[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonSO-02C Build/3.0.1.F.0.131) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46579[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonSO-02C Build/3.0.1.F.0.99) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46580[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SO-02C Build/3.0.1.F.0.126) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46581[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonSO-02C Build/3.0.1.F.0.126) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46582[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-02C Build/4.0.1.C.1.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46583[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-02C Build/4.0.1.C.1.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46584[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SO-02C Build/4.0.1.C.1.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46585[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SO-02C Build/4.0.1.C.1.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46586[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SO-02C Build/3.0.1.F.0.131)";
const char S46587[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SO-02C Build/4.0.1.C.1.24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46588[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-02C Build/4.0.1.C.1.24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46589[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonSO-02C Build/3.0.1.F.0.28)";
const char S46590[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SO-02C Build/4.0.1.C.1.24)";
const char S46591[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; A500 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S46592[] = "Dalvik/1.5.0 (Linux; U; Android 3.0.1; A500 Build/HRI66)";
const char S46593[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; he-il; A500 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S46594[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Touchpad Build/HLK75F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46595[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Touchpad Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46596[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TOUCHPAD 7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46597[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CnM-TOUCHPAD 7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46598[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CnM-TOUCHPAD 9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46599[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Touchpad7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46600[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; CnM-TOUCHPAD7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46601[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; CnM Touchpad 9.7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46602[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; CnM-TOUCHPAD9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46603[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VS TOUCHPAD 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46604[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Versus TouchPad 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46605[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; CnM TOUCHPAD 9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46606[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; CnM TouchPad 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46607[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; CnM-TOUCHPAD 7 Build/IML74K)";
const char S46608[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VS-TOUCHPAD 9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46609[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; TOUCHPAD 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46610[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Versus Touchpad 9.7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46611[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; CnM-TOUCHPAD 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46612[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Vs TOUCHPAD 9.7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46613[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Versus TouchPad 9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46614[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; Blueron TouchPad10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46615[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Versus TouchPad 7 Build/IMM76D)";
const char S46616[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; CnM Touchpad 4.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46617[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; CnM TouchPad 7 Build/IMM76D)";
const char S46618[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; CnM Touchpad 9.7 Build/IMM76D)";
const char S46619[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-I747 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S46620[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I747 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S46621[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I747 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S46622[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L710 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S46623[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I747 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S46624[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I747 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S46625[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L710 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S46626[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530U Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S46627[] = "Mozilla/5.0 (Linux; Android 4.2.1; SAMSUNG-SGH-I747 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S46628[] = "Mozilla/5.0 (Linux; Android 4.2.1; SPH-L710 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S46629[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9305 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S46630[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530C Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S46631[] = "Apple-iPad1C1/902.206";
const char S46632[] = "Apple-iPad2C1/1001.523";
const char S46633[] = "Apple-iPad2C2/1001.523";
const char S46634[] = "Apple-iPad3C1/1001.523";
const char S46635[] = "Apple-iPad3C4/1001.523";
const char S46636[] = "Apple-iPad3C6/1001.8426";
const char S46637[] = "Apple-iPad2C1/1002.329";
const char S46638[] = "Apple-iPad2C5/1002.329";
const char S46639[] = "Apple-iPad3C3/1002.329";
const char S46640[] = "Apple-iPad3C4/1002.329";
const char S46641[] = "Apple-iPad2C2/1002.329";
const char S46642[] = "Apple-iPad3C1/1002.329";
const char S46643[] = "Apple-iPad2C4/1002.329";
const char S46644[] = "Apple-iPad3C6/1002.329";
const char S46645[] = "Apple-iPad2C7/1002.329";
const char S46646[] = "gamed/4.10.17.1.6.13.5.2.1 (iPad2,5; 6.1.3; 10B329; GameKit-781.18)";
const char S46647[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; SonyEricssonX10a Build/R2CA016) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S46648[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:22.0) Gecko/X Firefox/22.0 SeaMonkey/2.19";
const char S46649[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:22.0) Gecko/X SeaMonkey/2.19";
const char S46650[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:22.0) Gecko/X Firefox/22.0 SeaMonkey/2.19";
const char S46651[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710BST Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S46652[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S46653[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S46654[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710VMUB Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S46655[] = "Mozilla/5.0 (Linux; Android 4.0.3; SPH-D710 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S46656[] = "SAMSUNG-GT-B3410R/B3410UXJC4 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S46657[] = "SAMSUNG-GT-B3410/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S46658[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I717R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46659[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I717R Build/IMM76D)";
const char S46660[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SGH-I717R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46661[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6603 Build/10.3.A.0.423) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S46662[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6603 Build/10.3.1.A.0.244) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S46663[] = "Mozilla/5.0 (compatible; MSIE 3.0; Linux i686; Trident/3.0)";
const char S46664[] = "Mozilla/5.0 (compatible; MSIE 3.0; Linux x86_64; Win64; IA64; Trident/5.0)";
const char S46665[] = "Mozilla/5.0 (compatible; MSIE 3.0; Linux i686; WOW64; Trident/5.0)";
const char S46666[] = "Mozilla/5.0 (compatible; MSIE 3.0; Linux i686; SV1; Trident/5.0)";
const char S46667[] = "Mozilla/5.0 (compatible; MSIE 3.0; Linux i686; Tablet PC; Trident/3.0)";
const char S46668[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Ubuntu/11.10 Chromium/15.0.874.106 Chrome/15.0.874.106 Safari/535.2";
const char S46669[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.2 (KHTML, like Gecko) Ubuntu/11.10 Chromium/15.0.874.120 Chrome/15.0.874.120 Safari/535.2";
const char S46670[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.2 (KHTML	 like Gecko) Ubuntu/11.10 Chromium/15.0.874.120 Chrome/15.0.874.120 Safari/535.2";
const char S46671[] = "Mozilla/5.0 (Android; Tablet; rv:24.0) Gecko/24.0 Firefox/24.0";
const char S46672[] = "Mozilla/5.0 (Android; Tablet; rv:24.0a2) Gecko/24.0a2 Firefox/24.0a2";
const char S46673[] = "Mozilla/5.0 (Android; Tablet; rv:24.0) Gecko/24.0 Firefox/24.0,gzip(gfe)";
const char S46674[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 510)";
const char S46675[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Nokia 510)";
const char S46676[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 510; Orange)";
const char S46677[] = "Mozilla/5.0 (Series40; Nokia303/14.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S46678[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; U8500 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46679[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-tw; HUAWEI_IDEOS_U8500 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46680[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; U8500 Build/HuaweiU8500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46681[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-in; HuaweiU8500 Build/HuaweiU8500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46682[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pl-pl; U8500 Build/HuaweiU8500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46683[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-ph; ICE Build/HuaweiU8500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46684[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; U8500 Build/FRF91)";
const char S46685[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; U8500 Build/HuaweiU8500)";
const char S46686[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ru-by; MTC Evo Build/HuaweiU8500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46687[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; HUAWEI_IDEOS_U8500 Build/HuaweiU8500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46688[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-kh; MF8503 Build/HuaweiU8500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46689[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729) chromeframe/5.0.317.0";
const char S46690[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET4.0C; .NET4.0E; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729) chromeframe/5.0.317.0";
const char S46691[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729) chromeframe/5.0.317.0";
const char S46692[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PantechP8010 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46693[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; PantechP8010 Build/JZO54K)";
const char S46694[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=oscar;devm=PantechP8010;man=PANTECH;brw=Android;shwd=540x888;shwa=540x851;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S46695[] = "SAMSUNG-GT-C5212i/C5212iJPKD1 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S46696[] = "SAMSUNG-C5212/C5212DDIK1 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S46697[] = "NetFront/3.5.1(BREW 5.0.2.1; U; en-us; SAMSUNG; NetFront/3.5.1/WAP) Boost M390 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S46698[] = "Mozilla/5.0 (compatible; MSIE 10.0; Linux x86_64; Trident/2.0)";
const char S46699[] = "Mozilla/5.0 (compatible; MSIE 10.0; Linux x86_64; WOW64; Trident/2.0)";
const char S46700[] = "Mozilla/5.0 (compatible; MSIE 10.0; Linux i686; SV1; Trident/2.0)";
const char S46701[] = "Mozilla/5.0 (compatible; MSIE 10.0; X11; U; Linux x86_64; Trident/6.0)";
const char S46702[] = "Mozilla/5.0 (compatible; MSIE 10.0; Linux x86_64; Win64; IA64; Trident/2.0)";
const char S46703[] = "Mozilla/5.0 (compatible; MSIE 10.0; Linux i686; Tablet PC; Trident/4.0)";
const char S46704[] = "Mozilla/5.0 (Linux; Android 4.1.2; SM-T210 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46705[] = "Mozilla/5.0 (Linux; Android 4.1.2; SM-T210R Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46706[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19 AvantBrowser/Tri-Core";
const char S46707[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11 AvantBrowser/Tri-Core";
const char S46708[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 AvantBrowser/Tri-Core";
const char S46709[] = "Mozilla/5.0 (Windows NT 6.0; Avant TriCore) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S46710[] = "Mozilla/5.0 (Windows NT 6.0; Avant TriCore) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.96 Safari/537.4";
const char S46711[] = "Mozilla/5.0 (Windows NT 6.0; Avant TriCore) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S46712[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5 AvantBrowser/Tri-Core";
const char S46713[] = "Mozilla/5.0 (Windows NT 6.0; Avant TriCore) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S46714[] = "Mozilla/5.0 (Windows NT 6.0; Avant TriCore) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S46715[] = "Mozilla/5.0 (Windows NT 6.0; Avant TriCore) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S46716[] = "Mozilla/5.0 (Windows NT 6.0; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36";
const char S46717[] = "Mozilla/5.0 (Windows NT 6.0; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36";
const char S46718[] = "Mozilla/5.0 (Windows NT 6.0; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36";
const char S46719[] = "Mozilla/5.0 (Series40; Nokia305/03.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.13";
const char S46720[] = "Mozilla/5.0 (Series40; Nokia3050/03.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.13";
const char S46721[] = "Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Series40; Nokia305/03.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.13";
const char S46722[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I815 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46723[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I815 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46724[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-I815 Build/IMM76D)";
const char S46725[] = "iTunes/10.2.2 (Macintosh; Intel Mac OS X 10.6.7) AppleWebKit/533.21.1";
const char S46726[] = "iTunes/10.1.1 (Macintosh; Intel Mac OS X 10.6.1) AppleWebKit/531.9";
const char S46727[] = "iTunes/10.0.1 (Macintosh; Intel Mac OS X 10.6.4) AppleWebKit/533.18.1";
const char S46728[] = "iTunes/10.0.1 (Macintosh; Intel Mac OS X 10.6.6) AppleWebKit/533.18.1";
const char S46729[] = "iTunes/10.7 (Macintosh; Intel Mac OS X 10.6.8) AppleWebKit/534.57.2";
const char S46730[] = "iTunes/10.1 (Macintosh; Intel Mac OS X 10.6.5) AppleWebKit/533.18.1";
const char S46731[] = "iTunes/9.0.3 (Macintosh; U; Intel Mac OS X 10_6_2; en-ca)";
const char S46732[] = "iTunes/10.6.3 (Macintosh; Intel Mac OS X 10.6.3) AppleWebKit/531.21.11";
const char S46733[] = "iTunes/11.0.1 (Macintosh; OS X 10.6.8) AppleWebKit/534.57.2";
const char S46734[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46735[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S5830 Build/ICECREAMSANDWICH) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46736[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-gb; GT-S5830 Build/ICECREAMSANDWICH) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46737[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-S5830 Build/ICECREAMSANDWICH) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46738[] = "Dalvik/1.4.0 (Linux; U; Android 4.0.1; GT-S5830 Build/ICECREAMSANDWICH)";
const char S46739[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46740[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; es-es; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46741[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; es-; GT-S5830 Build/ICECREAMSANDWICH) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46742[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S5830 Build/JELLYBEAN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46743[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S5830 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46744[] = "Mozilla/5.0 (Linux; U; Android 4.0.5; en-us; S5830/Galaxy Ace Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46745[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S5830 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46746[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:7.0.1) Gecko/X Firefox/7.0.1";
const char S46747[] = "Mozilla/5.0 (Series40; Nokia206/03.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.36";
const char S46748[] = "Mozilla/5.0 (Series40; Nokia206.1/03.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.36";
const char S46749[] = "Nokia206/2.0 (03.59) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Series40; Nokia206/03.59; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.36";
const char S46750[] = "server-bag [iPhone OS,5.0.1,9A405,iPod4,1]";
const char S46751[] = "server-bag [iPhone OS,5.1.1,9B206,iPod4,1]";
const char S46752[] = "Apple iPod v5.1.1 YouTube v1.0.0.9B206";
const char S46753[] = "server-bag [iPhone OS,4.3.3,8J2,iPod4,1]";
const char S46754[] = "Appcelerator Titanium/1.7.2 (iPod touch/6.1.3; iPhone OS; en_US;)";
const char S46755[] = "Mozilla/5.0 (Windows NT 4.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S46756[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P769 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S46757[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S46758[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P768 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S46759[] = "Mozilla/5.0 (Linux; Android 4.1.2; LGMS769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S46760[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Safari/535.19 Silk-Accelerated=false";
const char S46761[] = "Opera/9.80 (Windows NT 5.1; U; Edition Next; en) Presto/2.11.310 Version/12.50";
const char S46762[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I497 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46763[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-I497 Build/JZO54K)";
const char S46764[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S46765[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S46766[] = "SAMSUNG-SGH-E250/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S46767[] = "SAMSUNG-SGH-E250-VODA/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S46768[] = "SAMSUNG-SGH-E250i-ORANGE/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S46769[] = "SAMSUNG-SGH-E250-ORANGE/E250DBVHE2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S46770[] = "Vodafone/1.0/SAMSUNG-SGH-E250V/BUGB2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S46771[] = "Vodafone/1.0/SAMSUNG-SGH-E250i/BAIC2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S46772[] = "Vodafone/1.0/SAMSUNG-SGH-E250V/DBUHC1/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S46773[] = "Vodafone/1.0/SAMSUNG-SGH-E250V/DBUGK1/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S46774[] = "Vodafone/1.0/SAMSUNG-SGH-E250V/BUGD1/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S46775[] = "SAMSUNG-SGH-E250i/E250iJAKD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S46776[] = "Vodafone/1.0/SAMSUNG-SGH-E250V/AMGC1/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S46777[] = "SonyEricssonK800iv/R1JC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S46778[] = "SonyEricssonK800i/R8BF Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S46779[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46780[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46781[] = "Mozilla/5.0 (compatible; MSIE 9.0; Linux i686; Trident/1.0)";
const char S46782[] = "Mozilla/5.0 (compatible; MSIE 9.0; Linux x86_64; SV1; Trident/4.0)";
const char S46783[] = "Mozilla/5.0 (compatible; MSIE 9.0; Linux x86_64; WOW64; Trident/2.0)";
const char S46784[] = "Mozilla/5.0 (compatible; MSIE 9.0; Linux x86_64; Win64; IA64; Trident/2.0)";
const char S46785[] = "Mozilla/5.0 (compatible; MSIE 9.0; Linux x86_64; Tablet PC; Trident/3.0)";
const char S46786[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT890 Build/8.7.1I-110_IFW-39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S46787[] = "Mozilla/5.0 (Symbian/3; Series60/5.3 NokiaE6-00/111.140.0058; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.1.4 Mobile Safari/535.1 3gpp-gba";
const char S46788[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300T Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S46789[] = "Mozilla/5.0 (Linux; Android 4.2.2; TF300T Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S46790[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300TL Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S46791[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300TG Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S46792[] = "Mozilla/5.0 (Linux; U; Android 3.1; ja-jp; MZ604 Build/H.6.4-15) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S46793[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; MZ604 Build/H.6.4-20) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S46794[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; MZ604 Build/H.6.4-20)";
const char S46795[] = "Mozilla/5.0 (Linux; U; Android 3.1; de-de; MZ604 Build/H.6.4-20-3) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S46796[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-au; MZ604 Build/H.6.4-16) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S46797[] = "Mozilla/5.0 (Linux; Android 4.1.1; PC36100 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S46798[] = "Mozilla/5.0 (Linux; Android 4.1.2; EVO Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S46799[] = "Mozilla/5.0 (Linux; Android 4.1.2; PC36100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S46800[] = "Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S46801[] = "Mozilla/5.0 (Linux; Android 4.1.1; EVO 4G LTE Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S46802[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT881 Build/9.8.3O-118_USC-14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46803[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT881 Build/9.8.3O-118_USC-14)";
const char S46804[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT881 Build/9.8.3O-118_USC-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46805[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; GT-S6010 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46806[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; GT-S6010L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46807[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-S6010 Build/IMM76D)";
const char S46808[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID PRO Build/V2_2.26.99) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46809[] = "MOT-XT610/Blur_Version.0.22.2.XT610.Personal.en.AR Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; XT610 Build/0.22.0_R02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46810[] = "MOT-XT610/Blur_Version.0.26.1.XT610.Movistar.en.01 Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; XT610 Build/0.26.0_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46811[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; XT610 Build/V2U_3.4.2-154) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46812[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; XT610 Build/V2U_3.4.2-179) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46813[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID PRO Build/S275) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S46814[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID PRO Build/V2_2.26.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46815[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROID PRO Build/V2_2.26.60) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46816[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROID PRO Build/3.4.2_80_V2_87) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46817[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; DROID PRO Build/3.4.2_80_V2_87)";
const char S46818[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROID PRO Build/3.4.2_80_V2_86) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46819[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROID PRO Build/3.4.2_80_V2_85) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46820[] = "MOT-XT610/Blur_Version.0.17.62.XT610.Latam.en.01 Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; XT610 Build/0.17.0_R062) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46821[] = "MOT-XT610/Blur_Version.0.17.9.XT610.AmericaMovil.en.MX Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; XT610 Build/0.17.0_R09) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46822[] = "MOT-XT610/Blur_Version.0.23.1.XT610.AmericaMovil.en.01 Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; XT610 Build/0.23.0_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46823[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROID PRO Build/V2.26.61) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46824[] = "Mozilla/5.0 (Android; Tablet; rv:10.0.4) Gecko/10.0.4 Firefox/10.0.4 Fennec/10.0.4";
const char S46825[] = "Mozilla/5.0 (Android; Tablet; rv:10.0.5) Gecko/10.0.5 Firefox/10.0.5 Fennec/10.0.5";
const char S46826[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P3110 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S46827[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3110 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S46828[] = "Mozilla/5.0 (Linux; Android 4.0.3;  GT-P3110 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S46829[] = "Mozilla/5.0 (Linux; Android 4.0.4;  GT-P3110 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S46830[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P999 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46831[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P999 Build/GRI40; CyanogenMod-Eaglesblood_build) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46832[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P999 Build/GWK74; CyanogenMod-HellFire Phoenix 2.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46833[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P999 Build/GRJ22; CyanogenMod-7.2.0; LG-P990 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46834[] = "Mozilla/5.0 (Windows NT 4.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S46835[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MZ609 4G Build/7.7.1-85_MZ609-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46836[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MZ609 Build/7.7.1-85_MZ609-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46837[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MZ609 Build/7.7.1-85_MZ609-31)";
const char S46838[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MZ609 4G Build/7.7.1-85_MZ609-27) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46839[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMP5580C Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S46840[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; PMP5580C Build/IMM76I)";
const char S46841[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; S7 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 S7V100R001C13B021";
const char S46842[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; HUAWEI-S7 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S46843[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Ideos S7 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S46844[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HUAWEI Ideos S7 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S46845[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; S7 Build/4749) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46846[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; S7 Build/4431) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46847[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; S7 Build/4626) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46848[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Ideos S7 Build/ERE27)";
const char S46849[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; S7 Build/4078) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46850[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; S7 Build/ERE27)";
const char S46851[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; S7 Build/3987) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46852[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; cs-cz; S7 Build/4625) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46853[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; cs-cz; S7 Build/4657) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46854[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P5113 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S46855[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P5113 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S46856[] = "Mozilla/5.0 (Linux; Android 4.0.3;  GT-P5113 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S46857[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P5113 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S46858[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P5113 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S46859[] = "Mozilla/5.0 (Windows NT 5.1; rv:17.0) Gecko/X Firefox/17.0 SeaMonkey/2.14.1";
const char S46860[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:17.0) Gecko/X Firefox/17.0 SeaMonkey/2.14.1";
const char S46861[] = "Mozilla/5.0 (Windows NT 5.1; rv:17.0) Gecko/X SeaMonkey/2.14.1";
const char S46862[] = "Mozilla/5.0 (Series40; Nokia303/14.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6";
const char S46863[] = "Mozilla/5.0 (Series40; Nokia303/14.87; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.62.10";
const char S46864[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A810K Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46865[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A810S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46866[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-A810S Build/IMM76I)";
const char S46867[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-A810K Build/IMM76I)";
const char S46868[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A810S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46869[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A810K Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46870[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A810K/A810K2.14 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46871[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A810K Build/VEGAVIET.COM ROM ICS V1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S46872[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A810S Build/VEGAVIET.COM ROM ICS V1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S46873[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A810S Build/VEGAVIET.COM ROM ICS V2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S46874[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A810K/A810K2.14 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S46875[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A810S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30";
const char S46876[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A810S Build/VEGAVIET.COM ROM ICS V4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S46877[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A810K/A810K2.14 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30";
const char S46878[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L720 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S46879[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L720 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S46880[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC_Flyer_P510e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46881[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_Flyer_P510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46882[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_Flyer_P512 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46883[] = "Logos/0.9.4 (Linux; U; Android 2.3.4; en_US; HTC Flyer Build/GRJ22)";
const char S46884[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Flyer P510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46885[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Flyer P512 Build/GRI40)";
const char S46886[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Flyer P510e Build/GRI40)";
const char S46887[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Flyer P510e Build/GRJ22)";
const char S46888[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC_Flyer Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S46889[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_Flyer_P512_NA Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46890[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC_Flyer_P512_NA Build/GRJ22)";
const char S46891[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Flyer P512 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46892[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; HTC_Flyer Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46893[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Flyer P512 Build/GRJ22)";
const char S46894[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; HTC Flyer Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46895[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_Flyer_P512 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46896[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Flyer Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46897[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Flyer P510e Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S46898[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; HTC_Flyer Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46899[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us-; HTC Flyer P510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46900[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Flyer P510e Build/GRJ22) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46901[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTCFlyerP510e Build/GRJ22) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46902[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTCFlyerP512 Build/GRI40) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46903[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Flyer P512 Build/GRI40) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46904[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTCFlyer Build/GRJ22) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46905[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTCFlyerP510e Build/GRI40) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S46906[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7;en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17 Skyfire/2.0";
const char S46907[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17 Skyfire/2.0";
const char S46908[] = "Mozilla/5.0(Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17 Skyfire/2.0";
const char S46909[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0 Safari/528.17 Skyfire/2.0";
const char S46910[] = "Mozilla/5.0 (Series40; Nokia112/03.32; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S46911[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S46912[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S46913[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5";
const char S46914[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5";
const char S46915[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/7534.48.3 TeamStream HD (iPad2,1 284 1.7)";
const char S46916[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 TeamStream HD (iPad2,1 284 1.7)";
const char S46917[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 TeamStream HD (iPad2,1 336 1.8.1)";
const char S46918[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3113 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S46919[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3113 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S46920[] = "Mozilla/5.0 (Linux; Android 4.2.1; v89_zlh_hd Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S46921[] = "Mozilla/5.0 (Linux; Android 4.2.2; R821 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46922[] = "Mozilla/5.0 (Linux; Android 4.2.2; M9 pro Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46923[] = "Mozilla/5.0 (Linux; Android 4.2.1; M470BSE Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46924[] = "Mozilla/5.0 (Linux; Android 4.2.1; 06_v89_jbla768_asx_s4lockscreen Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46925[] = "Mozilla/5.0 (Linux; Android 4.2.1; AT10LE-A Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46926[] = "Mozilla/5.0 (Linux; Android 4.2.2; S510 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46927[] = "Mozilla/5.0 (Linux; Android 4.2.1; W8s Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46928[] = "Mozilla/5.0 (Linux; Android 4.2.1; STAIRWAY Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46929[] = "Mozilla/5.0 (Linux; Android 4.2.1; Lenovo S820_ROW Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46930[] = "Mozilla/5.0 (Linux; Android 4.2.1; v89_jbl1a668 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46931[] = "Mozilla/5.0 (Linux; Android 4.2.2; M6 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46932[] = "Mozilla/5.0 (Linux; Android 4.2.1; ZTE V967S Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46933[] = "Mozilla/5.0 (Linux; Android 4.2.2; ME302KL Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46934[] = "Mozilla/5.0 (Linux; Android 4.2.2; X50D Build/X50D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46935[] = "Mozilla/5.0 (Linux; Android 4.2.1; A240 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46936[] = "Mozilla/5.0 (Linux; Android 4.2.2; U55GT Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46937[] = "Mozilla/5.0 (Linux; Android 4.2.2; CW-V88-QUAD Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46938[] = "Mozilla/5.0 (Linux; Android 4.2.2; EPAD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46939[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9197 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46940[] = "Mozilla/5.0 (Linux; Android 4.2.2; DLX Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46941[] = "Mozilla/5.0 (Linux; Android 4.2.1; W200 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46942[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SM-T217A Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46943[] = "Mozilla/5.0 (Linux; Android 4.2.1; IMO s89 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S46944[] = "Mozilla/5.0 (Linux; Android 4.2.1; DARKSIDE Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46945[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nocturnal Google Edition CDMA 5.0 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46946[] = "Mozilla/5.0 (Linux; Android 4.2.1; Q5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46947[] = "Mozilla/5.0 (Linux; Android 4.2.1; 1080P-N003 Build/1080P1080P-N003) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46948[] = "Mozilla/5.0 (Linux; Android 4.2.1; i6000 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46949[] = "Mozilla/5.0 (Linux; Android 4.2.2; M7 pro Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46950[] = "Mozilla/5.0 (Linux; Android 4.2.1; MyPhone Agua Iceberg Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46951[] = "Mozilla/5.0 (Linux; Android 4.2.1; v89_gq2008s Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46952[] = "Mozilla/5.0 (Linux; Android 4.2.1; v89_jbl1a698 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46953[] = "Mozilla/5.0 (Linux; Android 4.2.1; M470BSS Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46954[] = "Mozilla/5.0 (Linux; Android 4.2.1; AUXUS Nuclea N1 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46955[] = "Mozilla/5.0 (Linux; Android 4.2.2; B1-711 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46956[] = "Mozilla/5.0 (Linux; Android 4.2.1; 7102 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46957[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC_PO582 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46958[] = "Mozilla/5.0 (Linux; Android 4.2.1; IMO S89 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46959[] = "Mozilla/5.0 (Linux; Android 4.2.1; I9389 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46960[] = "Mozilla/5.0 (Linux; Android 4.2.1; B943 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46961[] = "Mozilla/5.0 (Linux; Android 4.2.1; ZP990 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46962[] = "Mozilla/5.0 (Linux; Android 4.2.2; TCL S950 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46963[] = "Mozilla/5.0 (Linux; Android 4.2.1; HY5001 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46964[] = "Mozilla/5.0 (Linux; Android 4.2.1; AT10PE-A Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46965[] = "Mozilla/5.0 (Linux; Android 4.2.1; 01v21_v89_jbl1a698_2g Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46966[] = "Mozilla/5.0 (Linux; Android 4.2.2; p4wifi Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46967[] = "Mozilla/5.0 (Linux; Android 4.2.2; U30GT C4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46968[] = "Mozilla/5.0 (Linux; Android 4.2.1; HUAWEI G700-U00 Build/HuaweiG700-U00) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46969[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R960 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46970[] = "Mozilla/5.0 (Linux; Android 4.2.1; ThL W9 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46971[] = "Mozilla/5.0 (Linux; Android 4.2.1; POMP W89 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46972[] = "Mozilla/5.0 (Linux; Android 4.2.1; GSmart Aku A1 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46973[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy-S4 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46974[] = "Mozilla/5.0 (Linux; Android 4.2.2; SM-T215 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46975[] = "Mozilla/5.0 (Linux; Android 4.2.1; Lenovo S750 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46976[] = "Mozilla/5.0 (Linux; Android 4.2.1; VOW V5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46977[] = "Mozilla/5.0 (Linux; Android 4.2.1; Glory 2 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46978[] = "Mozilla/5.0 (Linux; Android 4.2.1; S5-F Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46979[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337Z Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46980[] = "Mozilla/5.0 (Linux; Android 4.2.2; enrc2b Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46981[] = "Mozilla/5.0 (Linux; Android 4.2.1; M-PP2G530 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46982[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5220 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46983[] = "Mozilla/5.0 (Linux; Android 4.2.1; S4 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46984[] = "Mozilla/5.0 (Linux; Android 4.2.2; Slate 21 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46985[] = "Mozilla/5.0 (Linux; Android 4.2.2; ifiveX2 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46986[] = "Mozilla/5.0 (Linux; Android 4.2.1; Ban-G Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46987[] = "Mozilla/5.0 (Linux; Android 4.2.1; H920 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46988[] = "Mozilla/5.0 (Linux; Android 4.2.2; M9pro Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46989[] = "Mozilla/5.0 (Linux; Android 4.2.1; 01v21_v89_gq2008s Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46990[] = "Mozilla/5.0 (Linux; Android 4.2.2; Novo10 Catpain QuadCore Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46991[] = "Mozilla/5.0 (Linux; Android 4.2.2; ALCATEL ONE TOUCH 6040D Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46992[] = "Mozilla/5.0 (Linux; Android 4.2.1; H3088 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46993[] = "Mozilla/5.0 (Linux; Android 4.2.2; OZZY Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46994[] = "Mozilla/5.0 (Linux; Android 4.2.1; HS-U970 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46995[] = "Mozilla/5.0 (Linux; Android 4.2.1; T600 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46996[] = "Mozilla/5.0 (Linux; Android 4.2.2; V973 Core4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S46997[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASP-5000H Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S46998[] = "Mozilla/5.0 (compatible; MSIE 5.0; Linux x86_64; Trident/2.0)";
const char S46999[] = "Mozilla/5.0 (compatible; MSIE 5.0; Linux x86_64; SV1; Trident/1.0)";
const char S47000[] = "Mozilla/5.0 (compatible; MSIE 5.0; Linux x86_64; WOW64; Trident/1.0)";
const char S47001[] = "Mozilla/5.0 (compatible; MSIE 5.0; Linux x86_64; Win64; x64; Trident/5.0)";
const char S47002[] = "Mozilla/5.0 (compatible; MSIE 5.0; Linux x86_64; Tablet PC; Trident/5.0)";
const char S47003[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710VMUB Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S47004[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710BST Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S47005[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S47006[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-R760 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47007[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-R760 Build/IMM76D)";
const char S47008[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Amazon Kindle Fire Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47009[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amazon Kindle Fire Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47010[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amazon Kindle Fire2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47011[] = "Dalvik/1.6.0 (Linux; U; Android 4.2; Amazon Kindle Fire Build/JOP40C)";
const char S47012[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; Amazon Kindle Fire Build/JDQ39)";
const char S47013[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813/C8813V100R001C92B159; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S47014[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI C8813 Build/HuaweiC8813) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S47015[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813/C8813V100R001C92B162; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S47016[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI C8813 Build/HuaweiC8813) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47017[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813/C8813V100R001C92B167; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S47018[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813/C8813V100R001C92B166; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S47019[] = "Mozilla 5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI C8813 Build HuaweiC8813) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S47020[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813D/C8813DV100R001C92B168; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S47021[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813/C8813V100R001C92B169; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S47022[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HUAWEI C8813 Build/HuaweiC8813)";
const char S47023[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813D/C8813DV100R001C92B166; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S47024[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813D/C8813DV100R001C92B172; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S47025[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HUAWEI C8813D Build/HuaweiC8813D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47026[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_C8813Q/C8813QV100R001C92B186; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S47027[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HUAWEI C8813D Build/HuaweiC8813D)";
const char S47028[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_C8813Q/C8813QV100R001C92B185; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S47029[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813D/C8813DV100R001C419B172; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S47030[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI C8813D Build/HuaweiC8813D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S47031[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI C8813Q Build/HuaweiC8813Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47032[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_C8813Q/C8813QV100R001C92B189; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S47033[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813/C8813V100R001C92B173; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S47034[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI C8813Q Build/HuaweiC8813Q) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S47035[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HUAWEI C8813Q Build/HuaweiC8813Q)";
const char S47036[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_C8813Q/C8813QV100R001C92B191; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S47037[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_C8813Q/C8813QV100R001C92B193; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S47038[] = "Mozilla 5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI C8813Q Build HuaweiC8813Q) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S47039[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813/C8813V100R001C92B175; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S47040[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813/C8813V100R001C92B168; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S47041[] = "NokiaN70-1/5.0737.3.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.9.1.120/27/351/UCWEB";
const char S47042[] = "NokiaN70/ 5.0741.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.9.1.120/27/351/UCWEB";
const char S47043[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I717M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47044[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I717M Build/IMM76D)";
const char S47045[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SGH-I717M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47046[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S47047[] = "Mozilla/4.0 (Windows 98; US) Opera 10.00 [en]";
const char S47048[] = "Opera/9.80 (Windows 98; U; en) Presto/2.2.15 Version/10.00";
const char S47049[] = "Opera/9.80 (Windows 98; U; en) Presto/2.2.15 Version/10.01";
const char S47050[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; en) Opera 10.00";
const char S47051[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1.7) Gecko/X Ubuntu/9.10 (karmic) Firefox/3.5.7 GTB6";
const char S47052[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I537 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S47053[] = "Mozilla/5.0 (X11; SunOS i86pc; rv:10.0.3) Gecko/X Firefox/10.0.3";
const char S47054[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID2 Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola DROID2";
const char S47055[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID2 Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 480X854 motorola DROID2";
const char S47056[] = "Mozilla/5.0 (Linux; U;Android 2.2; en-us; DROID2 Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola DROID2";
const char S47057[] = "Mozilla/5.0 (Linux; U; Android2.2; en-us; DROID2 Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola DROID2";
const char S47058[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID2Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola DROID2";
const char S47059[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID2 Build/VZW) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S47060[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROID2 Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47061[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; DROID2 Build/VZW)";
const char S47062[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID2WE Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47063[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47064[] = "Mozilla/5.0 (Linux; Android 4.0.3;  SAMSUNG-SGH-I727 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S47065[] = "Mozilla/5.0 (Linux; Android 4.0.4;  SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S47066[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47067[] = "Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I727 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47068[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I727 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47069[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; LG-P690 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S47070[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LG-P690 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S47071[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; LG-P690 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S47072[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; LG-P690/${Version} Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S47073[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; LG-P690 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S47074[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P690 Build/GRJ22)";
const char S47075[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ar; LG-P690 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S47076[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P690 Build/GRI40; neko95@xda-developers) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47077[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; LG-P690-Net Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S47078[] = "NokiaX7-00.1/011.519 (Symbian/3; Series60/5.2 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.7.2 3gpp-gba";
const char S47079[] = "Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaX7-00/021.013; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.28 Mobile Safari/533.4 3gpp-gba";
const char S47080[] = "Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaX7-00.1/022.008; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.31 Mobile Safari/533.4 3gpp-gba";
const char S47081[] = "Mozilla/5.0 (PLAYSTATION 3 4.41) AppleWebKit/531.22.8 (KHTML, like Gecko)";
const char S47082[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Sensation XE with Beats Audio Z715e Build/GRJ22)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47083[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Sensation XE with Beats Audio Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Mobile Maxthon";
const char S47084[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Sensation XE with Beats Audio Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Mobile Maxthon Mobile Maxthon";
const char S47085[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.1b3pre) Gecko/X Firefox/3.1b3pre";
const char S47086[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; it; rv:1.9.1b2) Gecko/X Firefox/3.1b2";
const char S47087[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1 ; x64; en-US; rv:1.9.1b2pre) Gecko/X Firefox/3.1b2pre";
const char S47088[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9103 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47089[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9103 Build/IMM76D)";
const char S47090[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47091[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-I9103 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47092[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S47093[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S47094[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989D Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S47095[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S47096[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-T989 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S47097[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; blaze Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47098[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Mobile Safari/535.19 Silk-Accelerated=true";
const char S47099[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8) Presto/2.12.388 Version/12.15";
const char S47100[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6.8; rv:2.0) Gecko/X Firefox/4.0 Opera 12.15";
const char S47101[] = "Mozilla/5.0 (compatible; MSIE 4.0; Linux i686; Trident/1.0)";
const char S47102[] = "Mozilla/5.0 (compatible; MSIE 4.0; Linux x86_64; Win64; x64; Trident/3.0)";
const char S47103[] = "Mozilla/5.0 (compatible; MSIE 4.0; Linux i686; WOW64; Trident/4.0)";
const char S47104[] = "Mozilla/5.0 (compatible; MSIE 4.0; Linux i686; SV1; Trident/3.0)";
const char S47105[] = "Mozilla/5.0 (compatible; MSIE 4.0; Linux i686; Tablet PC; Trident/1.0)";
const char S47106[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Milestone Build/SHOLS_U2_02.31.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47107[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; Milestone Build/SHOLS_U2_02.34.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47108[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; Milestone Build/SHOLS_U2_03.10.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47109[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; Milestone Build/SHOLS_U2_03.11.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47110[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Milestone Build/SHOLS_U2_02.36.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47111[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Milestone Build/SHOLS_U2_02.27.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47112[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; Milestone Build/SHOLS_U2_02.36.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47113[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; Milestone Build/SHOLS_U2_02.38.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47114[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; Milestone Build/SGC03040) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47115[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; it-it; Milestone Build/SHOLS_U2_02.38.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47116[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; Milestone Build/SHOLS_U2_02.38.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47117[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; he-il; Milestone Build/SHOLS_U2_02.27.C) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47118[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; Milestone Build/SHOLS_U2_02.34.3) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47119[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-ru; Milestone Build/SHOLS_U2_02.27.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47120[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Milestone Build/SHOLS_U2_02.31.0)";
const char S47121[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Milestone Build/SHOLS_U2_02.27.B) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47122[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Milestone Build/SHOLS_U2_02.31.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47123[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-ru; Milestone Build/SHOLS_U2_02.27.9) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47124[] = "Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; Milestone Build/SHOLS_U2_02.36.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47125[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Milestone Build/SHOLS_U2_04.08.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47126[] = "Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; Milestone Build/SHOLS_U2_02.36.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S47127[] = "Mozilla/5.0 (Windows; Crawler; U; Windows NT 6.0; en-US; rv:1.9.0.7) Gecko/X Firefox/3.0.7 (.NET CLR 3.5.30729)";
const char S47128[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N762 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47129[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; N762 Build/GRJ22)";
const char S47130[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Milestone X Build/RETAIL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola Milestone X";
const char S47131[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Milestone X Build/AWCC) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola Milestone X";
const char S47132[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Milestone X Build/RETAIL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola Milestone X";
const char S47133[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Milestone X Build/NTEL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola Milestone X";
const char S47134[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Milestone X Build/CS) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola Milestone X";
const char S47135[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Milestone X Build/NTEL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola Milestone X";
const char S47136[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; Milestone X Build/RETAIL)";
const char S47137[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S47138[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; T7575)";
const char S47139[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Nokia; T7575)";
const char S47140[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I547 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47141[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I547C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47142[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SAMSUNG-SGH-I547 Build/IMM76D)";
const char S47143[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I547C Build/IMM76D)";
const char S47144[] = "Nokia200/2.0 (11.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia200) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S47145[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3113 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S47146[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3113 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S47147[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT300SE Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S47148[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT300 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S47149[] = "Mozilla/5.0 (Linux; Android 4.1.2; MB886 Build/9.8.2Q-8_MB886_FFW-11) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S47150[] = "Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S47151[] = "Mozilla/1.22 (compatible; MSIE 10.0; Windows 3.1)";
const char S47152[] = "Mozilla/5.0 (compatible; MSIE 10.0; Win3.11; Trident/2.0)";
const char S47153[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26i Build/6.1.A.2.55) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S47154[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26i Build/6.1.A.2.45) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S47155[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/534.6 (KHTML, like Gecko) Chrome/7.0.503.0 Safari/534.6";
const char S47156[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.28 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.28";
const char S47157[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S47158[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S47159[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545L Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S47160[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Instapaper/4.0 (+http://www.instapaper.com/)";
const char S47161[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; Padfone Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47162[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; PadFone Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47163[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PadFone Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47164[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; PadFone Build/IML74K)";
const char S47165[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; PadFone Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47166[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; PadFone Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47167[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; PadFone Build/IMM76I)";
const char S47168[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5750E-Orange/S5750EAFKE1; U; Bada/1.0; fr-fr) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 NexPlayer/3.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B";
const char S47169[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT25i Build/9.1.A.1.140) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47170[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; XT885 Build/6.7.3-135_YIC-8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47171[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; XT885 Build/6.7.3-179_YIC-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47172[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; XT885 Build/6.7.3-135_YIC-8)";
const char S47173[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT885 Build/6.7.2_GC-200-YTZU-6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47174[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT885 Build/6.7.2_GC-180-YTZU-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47175[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A760S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47176[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-A760S Build/IMM76D)";
const char S47177[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A760S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47178[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A760S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47179[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A760SXPERIA Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47180[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A760S_Optimus Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47181[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; IM-A760S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47182[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A760S/A770K FIX Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47183[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A760S/A770K Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47184[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 10 Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47185[] = "Dalvik/1.6.0 (Linux; U; Android 4.3; Nexus 10 Build/JWR66Y)";
const char S47186[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 10 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47187[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 10 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47188[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 10 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47189[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Twitter for iPhone";
const char S47190[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148a Twitter for iPhone";
const char S47191[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E200 Twitter for iPhone";
const char S47192[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_7 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E303 Twitter for iPhone";
const char S47193[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E401 Twitter for iPhone";
const char S47194[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_10 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E600 Twitter for iPhone";
const char S47195[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E128 Twitter for iPhone";
const char S47196[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_9 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E501 Twitter for iPhone";
const char S47197[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2 like Mac OS X; nl-nl) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C134 Twitter for iPhone";
const char S47198[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S47199[] = "Mozilla/5.0 (Series40; Nokia305/05.92; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S47200[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; GT-S5660M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47201[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5660M Build/GINGERBREAD)";
const char S47202[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ONE_TOUCH_960C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47203[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; ONE_TOUCH_960C Build/GRK39F)";
const char S47204[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I5700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47205[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; GT-I5700 Build/ECLAIR) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S47206[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; GT-I5700R Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47207[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; GT-I5700L Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47208[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-gb; GT-I5700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47209[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I5700 Build/ECLAIR)";
const char S47210[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-ca; GT-I5700R Build/ECLAIR) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S47211[] = "Mozilla/5.0 (Linux; U; Android 2.1; fr-fr; GT-I5700 Build/ECLAIR) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S47212[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; GT-I5700 Build/ECLAIR) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S47213[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.12.12934.201";
const char S47214[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.12.12934.201";
const char S47215[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.12.12934.201";
const char S47216[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.7 (KHTML, like Gecko) Chrome/20.0.1099.0 Safari/536.7 QQBrowser/6.12.13631.201";
const char S47217[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0) QQBrowser/6.12.12934.201";
const char S47218[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C) QQBrowser/6.12.12934.201";
const char S47219[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.12.12934.201,gzip(gfe) (via translate.google.com)";
const char S47220[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; QQBrowser/6.12)";
const char S47221[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; SV1; QQBrowser/6.12)";
const char S47222[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; eSobiSubscriber 1.0.0.40; BRI/2; MAAR; .NET4.0C) QQBrowser/6.12.12934.201";
const char S47223[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; SV1; QQBrowser/6.12)";
const char S47224[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; QQBrowser/6.12)";
const char S47225[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)";
const char S47226[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/6.12)";
const char S47227[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)";
const char S47228[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; QQBrowser/6.12)";
const char S47229[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)";
const char S47230[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; QQBrowser/6.12)";
const char S47231[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; QQBrowser/6.12)";
const char S47232[] = "Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.1; SV1; QQBrowser/6.12)";
const char S47233[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; QQBrowser/6.12)";
const char S47234[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; QQBrowser/6.12)";
const char S47235[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/6.12)";
const char S47236[] = "Mozilla/4.0 (compatible; MSIE 6.0; Linux 2.6.26-1-amd64) Lobo/0.98.3";
const char S47237[] = "Mozilla/5.0 (compatible; MSIE 6.0; Linux x86_64; Trident/1.0)";
const char S47238[] = "Mozilla/5.0 (compatible; MSIE 6.0; Linux x86_64; WOW64; Trident/4.0)";
const char S47239[] = "Mozilla/5.0 (compatible; MSIE 6.0; Linux i686; SV1; Trident/1.0)";
const char S47240[] = "Mozilla/5.0 (compatible; MSIE 6.0; Linux x86_64; Win64; x64; Trident/3.0)";
const char S47241[] = "Mozilla/5.0 (compatible; MSIE 6.0; Linux i686; Tablet PC; Trident/1.0)";
const char S47242[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.2 Mobile/8L1 Safari/6533.18.5";
const char S47243[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.2 Mobile/8H7 Safari/6533.18.5";
const char S47244[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.2 Mobile/8J2 Safari/6533.18.5";
const char S47245[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-R760 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47246[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R760X Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47247[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-R760X Build/JZO54K)";
const char S47248[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-R760 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47249[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S47250[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S47251[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S47252[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S47253[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47254[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A510 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47255[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; A510 Build/IML74K)";
const char S47256[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S47257[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S47258[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; M9300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47259[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; M9300 Build/GINGERBREAD)";
const char S47260[] = "SexyShelves/9.02.1 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47261[] = "ClassifiedAds/1.7.7 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47262[] = "GQMagazine/1.0 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47263[] = "AtomicBrowser/6.0.1 CFNetwork/485.12.30 Darwin/10.4.0";
const char S47264[] = "HOT%20HD%20WPs/9.19.1 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47265[] = "OfflinePagesPro/1.4.1 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47266[] = "Terra/1.3.6 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47267[] = "Celeb%20Wallpapers/9.13.1 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47268[] = "Clipish/9.29.1 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47269[] = "2%20Million%20WPs/9.16.1 CFNetwork/485.12.30 Darwin/10.4.0";
const char S47270[] = "EpicWallPaper2011/1.6 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47271[] = "InstapaperPro/4.0.1 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47272[] = "WOW%20WPs/9.26.1 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47273[] = "Google/2.0.1.10455 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47274[] = "ImageSearcherFreeS/1.0.5 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47275[] = "360%20Browser/2.1 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47276[] = "AtomicLite/5.6.0 CFNetwork/485.12.30 Darwin/10.4.0";
const char S47277[] = "ImageSearcherS/2.1.1 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47278[] = "Perfect%20Browser/7.2 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47279[] = "Mercury3Free/6.0.1 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47280[] = "AdSheet/1.0 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47281[] = "Hot%20Wallpapers/9.28.1 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47282[] = "Downloader%20copy/1.0 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47283[] = "Mercury3/6.0.1 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47284[] = "Love%20Pictures/3.6 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47285[] = "Funny%20Pics/2.92 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47286[] = "ImageSearch/1.4 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47287[] = "ysearch2/3.0.2 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47288[] = "Images/1.4 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47289[] = "browser/31759 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47290[] = "Twitter/4.3.2 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47291[] = "Holidays/9.13.1 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47292[] = "LinkedIn/5.1 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47293[] = "curl/7.21.2 (x86_64-apple-darwin10.4.0) libcurl/7.21.2 OpenSSL/1.0.1c zlib/1.2.7 libidn/1.19";
const char S47294[] = "iDownloaderPro/2.0 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47295[] = "Wallpapers/1.6 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47296[] = "Pic%20Collage/2.20.21 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47297[] = "123people/3.1.1 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47298[] = "ImageSearcherProS/2.1.2 CFNetwork/485.12.7 Darwin/10.4.0";
const char S47299[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:17.0) Gecko/X Thunderbird/17.0.8";
const char S47300[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300T Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S47301[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300TG Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S47302[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300TL Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S47303[] = "Mozilla/5.0 (Linux; Android 4.1.2; Transformer Pad TF300T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S47304[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/534.12 (KHTML, like Gecko) Chrome/9.0.576.0 Safari/534.12";
const char S47305[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_Lexikon Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47306[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/26.0.1410.43) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/537.31";
const char S47307[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/26.0.1410.64)";
const char S47308[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/26.0.1410.64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S47309[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; chromeframe/26.0.1410.64)";
const char S47310[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/26.0.1410.64)";
const char S47311[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; chromeframe/26.0.1410.64)";
const char S47312[] = "Mozilla/5.0 (Windows NT 6.2; chromeframe/26.0.1410.64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S47313[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1";
const char S47314[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1";
const char S47315[] = "Mozilla/5.0 (Linux; Android 4.0.3;  GT-P3113 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S47316[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P3113 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S47317[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3113 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S47318[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.236; en-us; 9300/6.6.0.236) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47319[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.207; en-us; 9300/6.6.0.207) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47320[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.212; en-us; 9300/6.6.0.212) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47321[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.195; en-us; 9300/6.6.0.195) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47322[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.124; en-us; 9300/6.6.0.124) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47323[] = "UCWEB/2.0(BlackBerry; U; 6.3.0.63; en-us; 9300/6.3.0.63) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47324[] = "UCWEB/2.0(BlackBerry; U; 6.3.0.39; en-us; 9300/6.3.0.39) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47325[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.241; en-us; 9300/6.6.0.241) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47326[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.223; en-us; 9300/6.6.0.223) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47327[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.248; en-us; 9300/6.6.0.248) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47328[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.246; en-us; 9300/6.6.0.246) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47329[] = "UCWEB/2.0(BlackBerry; U; 6.3.0.23; en-us; 9300/6.3.0.23) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47330[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.200; en-us; 9300/6.6.0.200) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47331[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.86; en-us; 9300/6.6.0.86) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47332[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.105; en-us; 9300/6.6.0.105) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47333[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.216; en-us; 9300/6.6.0.216) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47334[] = "UCWEB/2.0(BlackBerry; U; 6.3.0.31; en-us; 9300/6.3.0.31) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47335[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.50; en-us; 9300/6.6.0.50) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47336[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.137; en-us; 9300/6.6.0.137) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47337[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.250; en-us; 9300/6.6.0.250) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47338[] = "UCWEB/2.0(BlackBerry; U; 6.3.0.57; en-us; 9300/6.3.0.57) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47339[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.151; en-us; 9300/6.6.0.151) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47340[] = "UCWEB/2.0(BlackBerry; U; 6.3.0.34; en-us; 9300/6.3.0.34) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S47341[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727) yolinkBot";
const char S47342[] = "SNK Screenshot Bot/0.20 ( abuse at savenkeep dot_com ) (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0)";
const char S47343[] = "Peeplo Screenshot Bot/0.20 ( abuse at peeplo dot_com ) (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0)";
const char S47344[] = "Mozilla/5.0 (Symbian/3; Series60/5.4 Nokia808PureView/112.020.0309; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.2.1.22 Mobile Safari/535.1 3gpp-gba";
const char S47345[] = "Mozilla/5.0 (Symbian/3; Series60/5.5 Nokia808PureView/113.010.1507; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.2.21 Mobile Safari/535.1 3gpp-gba";
const char S47346[] = "Nokia808/113.010.1507 (Symbian/3; Series60/5.5 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) Version/3.0 NokiaBrowser/8.3.2.21 3gpp-gba";
const char S47347[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE N9120 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47348[] = "Puffin/9073 CFNetwork/548.1.4 Darwin/11.0.0";
const char S47349[] = "Puffin%20Free/9090 CFNetwork/548.1.4 Darwin/11.0.0";
const char S47350[] = "Mozilla/5.0 (Series40; Nokia305/03.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S47351[] = "Mozilla/5.0 (Series40; Nokia305/03.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5";
const char S47352[] = "Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Series40; Nokia305/03.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.31";
const char S47353[] = "Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S47354[] = "Mozilla/5.0 (Linux; Android 4.0.4; Transformer Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S47355[] = "Mozilla/5.0 (Linux; Android 4.0.3; TF101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S47356[] = "Mozilla/5.0 (compatible; MSIE 7.0; Linux i686; Trident/5.0)";
const char S47357[] = "Mozilla/5.0 (compatible; MSIE 7.0; Linux x86_64; WOW64; Trident/5.0)";
const char S47358[] = "Mozilla/5.0 (compatible; MSIE 7.0; Linux i686; Win64; IA64; Trident/2.0)";
const char S47359[] = "Mozilla/5.0 (compatible; MSIE 7.0; Linux x86_64; Tablet PC; Trident/1.0)";
const char S47360[] = "Mozilla/5.0 (compatible; MSIE 7.0; Linux x86_64; SV1; Trident/1.0)";
const char S47361[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S47362[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S47363[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S47364[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S47365[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S47366[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S47367[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S47368[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S47369[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S47370[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID8042 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47371[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; MID8042 Build/IML74K)";
const char S47372[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SAMSUNG-SGH-I857 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47373[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; SAMSUNG-SGH-I857 Build/FROYO)";
const char S47374[] = "Mozilla/5.0 (Linux; Android 4.1.1; C1505 Build/11.3.A.0.47) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47375[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S47376[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S47377[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Special Edition Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S47378[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S47379[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC VLE_U Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S47380[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC VLE_U Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S47381[] = "Mozilla/4.0 WebTV/2.6 (compatible; MSIE 4.0)";
const char S47382[] = "Mozilla/4.0 (compatible; MSIE 4.01; Mac_PowerPC)";
const char S47383[] = "Mozilla/4.0 (compatible; MSIE 4.0;";
const char S47384[] = "MSIE 4\\.0b2;";
const char S47385[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; fr; rv:1.9.1b1) Gecko/X Firefox/3.1b1";
const char S47386[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0 x64; en-US; rv:1.9.1b2pre) Gecko/X Firefox/3.1b2pre";
const char S47387[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0 ; x64; en-US; rv:1.9.1b2pre) Gecko/X Firefox/3.1b2pre";
const char S47388[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus One Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47389[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; Nexus One Build/GRK39F; JRsoft - CyanogenMod) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47390[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.215 Safari/534.10";
const char S47391[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; de-DE; rv:1.8.1.23) Gecko/X SeaMonkey/1.1.18";
const char S47392[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.2pre) Gecko/X SeaMonkey/1.1 Mnenhy/0.7.4.10005";
const char S47393[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.6) Gecko/X Firefox/2.0.0.11 SeaMonkey/1.1.8";
const char S47394[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.8.1.19) Gecko/X SeaMonkey/1.1.14";
const char S47395[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7000 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S47396[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47397[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S47398[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7000 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47399[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M480W Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S47400[] = "Mozilla/5.0 (Linux; Android 4.1.2; SC-05D Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47401[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47402[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47403[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47404[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000B Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47405[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M480S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S47406[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M480K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S47407[] = "Mozilla/5.0 (Linux; U; Android2.1-update1; en-us; T-Mobile_Espresso Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47408[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us;T-Mobile_Espresso Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47409[] = "Mozilla/5.0 (Linux; U;Android 2.1-update1; en-us; T-Mobile_Espresso Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47410[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; T-Mobile_Espresso Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 MobileSafari/530.17";
const char S47411[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; T-Mobile_EspressoBuild/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47412[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; T-Mobile_Espresso Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47413[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; HTC Espresso Build/ERD35B) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S47414[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/11.0.696.57) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.57 Safari/534.24";
const char S47415[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/11.0.696.71) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24";
const char S47416[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/11.0.696.71; .NET CLR 2.0.50727; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 3.0.04506.30; .NET4.0C; .NET4.0E)";
const char S47417[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/11.0.696.65) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.65 Safari/534.24";
const char S47418[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.1; chromeframe/11.0.696.65)";
const char S47419[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/11.0.696.77) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Safari/534.24";
const char S47420[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB6.6; BTRS31753; chromeframe/11.0.696.71; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; BRI/1)";
const char S47421[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.1; .NET4.0C; .NET4.0E; chromeframe/11.0.696.65)";
const char S47422[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/11.0.696.68) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S47423[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/11.0.696.77) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Safari/534.24";
const char S47424[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/11.0.696.60) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.60 Safari/534.24";
const char S47425[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/11.0.696.68; EmbeddedWB 14,52 from: http://www.bsalsa.com/ EmbeddedWB 14.52; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; BOIE8;ENUS)";
const char S47426[] = "Mozilla/5.0 (Linux; Android 4.0.3;  A500 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S47427[] = "Mozilla/5.0 (Linux; Android 4.0.4;  Iconia A500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S47428[] = "Mozilla/5.0 (Linux; Android 4.0.3; A500 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S47429[] = "Mozilla/5.0 (Linux; Android 4.0.4; Iconia A500 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S47430[] = "Mozilla/5.0 (Linux; Android 4.0.4; A500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S47431[] = "Mozilla/5.0 (Linux; Android 4.0.3; Acer Iconia A500 Build/MR1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S47432[] = "Mozilla/5.0 (Linux; Android 4.0.4; A500 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47433[] = "(compatible; MSIE 4.01; MSN 2.5; AOL 4.0; Windows 98)";
const char S47434[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows 98)";
const char S47435[] = "Mozilla/4.0(compatible; MSIE 4.01; MSN 2.5; AOL 4.0; Windows 98)";
const char S47436[] = "Mozilla/4.0 (compatible; MSIE 4.0; Windows 98)";
const char S47437[] = "Mozilla/4.0 (compatible; MSIE 4.0; Windows 98 )";
const char S47438[] = "Mozilla/2.0 (compatible; MSIE 4.0; Windows 98)";
const char S47439[] = "Mozilla/4.5 (compatible; MSIE 4.01; Windows 98)";
const char S47440[] = "Mozilla/5.0 (compatible; MSIE 4.0; Win 9x 4.90; Trident/1.0)";
const char S47441[] = "Mozilla/5.0 (compatible; MSIE 4.0; Win98; Trident/5.0)";
const char S47442[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; A500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S47443[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; A500 Build/HMJ37)";
const char S47444[] = "Mozilla/5.0 (Linux; U; Android 3.1; he-il; A500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S47445[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; Acer Iconia Tab A500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S47446[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S47447[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3110 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S47448[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3110 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S47449[] = "Mozilla/5.0 (Linux; Android 4.1.1;  GT-P3110 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S47450[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq edison Build/1.1.1 IMM76D X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47451[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison Build/1.0.0 c1006-user 4.0.4 IMM76D eng.xmypf.X.X test-keys) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47452[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq edison Build/1.1.1 bq_edison_c1006_X_user) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47453[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison Build/1.1.4 X-10:36) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47454[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison Build/1.1.5 X-11:03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47455[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison Build/1.1.7 X-11:59) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47456[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison Build/1.1.2 X-14:11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47457[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison Build/1.1.8 X-16:29) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47458[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison Build/1.1.8-1015 X-10:20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47459[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison Build/1.1.10-1015 X-18:00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47460[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison Build/1.1.11-1015 X-10:30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47461[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; bq Edison Build/1.1.8 X-16:29)";
const char S47462[] = "NokiaC2-00/2.0 (03.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-00";
const char S47463[] = "NokiaC2-00/2.0 (03.82) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S47464[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP9070 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47465[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-N7100 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47466[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7105 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47467[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7105 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47468[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7105 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47469[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L900 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47470[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7100 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47471[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-N7100 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47472[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L900 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47473[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7105 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47474[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T889 Build/JDQ39; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47475[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I317 Build/JDQ39; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47476[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I317 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/t0lteatt";
const char S47477[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7100 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/n7100";
const char S47478[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T889 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/t0ltetmo";
const char S47479[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I317 Build/JDQ39E; CyanogenMod-10.1.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47480[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; md-us; Nook Tablet Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47481[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Nook Tablet Build/GINGERBREAD; CyanogenMod-7.1.0 XDA Team) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47482[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; BNTV250 Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47483[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.1 Mobile/10B141 Safari/8536.25";
const char S47484[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.1 Mobile/10B146 Safari/8536.25";
const char S47485[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/6.1 Mobile/9B176 Safari/7534.48.3";
const char S47486[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.1 Mobile/10B147 Safari/8536.25";
const char S47487[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.1 Mobile/10B329 Safari/8536.25";
const char S47488[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.1.3 Mobile/10B329 Safari/8536.25";
const char S47489[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF700T Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S47490[] = "Mozilla/5.0 (Linux; Android 4.2.2; ASUS Transformer Pad TF700T Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S47491[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9000 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47492[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9000 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47493[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; GT-I9000 Build/JRO03E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47494[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-I9000 Build/JRO03L)";
const char S47495[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9000 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47496[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47497[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9000 Build/JellyHeff) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47498[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9000 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47499[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9000 Build/JZO54K)";
const char S47500[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9000 Build/By lucainno) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47501[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; GT-I9000 Build/JRO03U) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47502[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; GT-I9000 Build/JRO03L by muchopoli83) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 M?vil Safari/534.30";
const char S47503[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9000 Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S47504[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.0.489) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S47505[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.0.489) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S47506[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xperia T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S47507[] = "MobileRSSFree-iPad/3.1.4 CFNetwork/609 Darwin/13.0.0";
const char S47508[] = "Dolphin%20Browser%20for%20iPad/5.4.1 CFNetwork/609 Darwin/13.0.0";
const char S47509[] = "MobileRSS-iPad/3.1.4 CFNetwork/609.1.4 Darwin/13.0.0";
const char S47510[] = "Perfect%20Browser-iPad/7.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S47511[] = "EvernoteWebClip-iPad/2.2.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S47512[] = "YandexSearch-iPad/683 CFNetwork/609.1.4 Darwin/13.0.0";
const char S47513[] = "browseripad/36901 CFNetwork/609.1.4 Darwin/13.0.0";
const char S47514[] = "CitrixReceiver-iPad/172 CFNetwork/609.1.4 Darwin/13.0.0";
const char S47515[] = "Mozilla/5.0 (compatible; MSIE 8.0; Linux x86_64; Trident/4.0)";
const char S47516[] = "Mozilla/5.0 (compatible; MSIE 8.0; Linux x86_64; Win64; IA64; Trident/3.0)";
const char S47517[] = "Mozilla/5.0 (compatible; MSIE 8.0; Linux i686; WOW64; Trident/4.0)";
const char S47518[] = "Mozilla/5.0 (compatible; MSIE 8.0; Linux x86_64; Tablet PC; Trident/2.0)";
const char S47519[] = "Mozilla/5.0 (compatible; MSIE 8.0; Linux x86_64; SV1; Trident/4.0)";
const char S47520[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire X Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47521[] = "Mozilla/4.76 [en] (X11; U; FreeBSD 4.4-STABLE i386)";
const char S47522[] = "Mozilla/3.0 (WorldGate Gazelle 3.5.1 build 11; FreeBSD2.2.8-STABLE)";
const char S47523[] = "Mozilla/5.0 (compatible; Konqueror/3.2; FreeBSD) (KHTML, like Gecko)";
const char S47524[] = "Mozilla/5.0 (X11; U; FreeBSD i386; en-US; rv:1.5) Gecko/X";
const char S47525[] = "python-requests/1.1.0 CPython/2.7.2 FreeBSD/9.0-RELEASE-p3";
const char S47526[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SHARP-ADS1 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47527[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SHARP-ADS1 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47528[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; SHARP-ADS1 Build/FRG83G)";
const char S47529[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; PB99400 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47530[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; PB99400 Build/FRF91)";
const char S47531[] = "Mozilla/5.0 (Series40; Nokia306/03.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.13.1";
const char S47532[] = "Mozilla/5.0 (Linux; U; Android2.2; en-us; SGH-T849 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47533[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T849 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S47534[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T849Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47535[] = "Mozilla/5.0 (Linux; U;Android 2.2; en-us; SGH-T849 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47536[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T849 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47537[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us;SGH-T849 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47538[] = "Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; SGH-T849 Build/TOSHIBA_FOLIO_AND_A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47539[] = "Logos/0.9.4 (Linux; U; Android 2.2; en_US; SGH-T849 Build/FROYO.UVJJB)";
const char S47540[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; SGH-T849 Build/FROYO)";
const char S47541[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;ADR6400L 4G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47542[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; ADR6400L 4GBuild/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47543[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; ADR6400L 4G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47544[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; ADR6400L Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47545[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; ADR6400L Build/FRG83D)";
const char S47546[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ADR6400L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47547[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; ADR6400L Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47548[] = "Mozilla/5.0 (Windows NT 6.0; rv:25.0) Gecko/X Firefox/25.0";
const char S47549[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:25.0) Gecko/X Firefox/25.0";
const char S47550[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaN82/20.0.062; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S47551[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; LG-P698f Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S47552[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LG-P698 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S47553[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P698 Build/GRJ22)";
const char S47554[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ar; LG-P698 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S47555[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P698f Build/GRJ22)";
const char S47556[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.3 Mobile/10A523 Safari/8536.25";
const char S47557[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.3 Mobile/10A403 Safari/8536.25";
const char S47558[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I717D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47559[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I717D Build/IMM76D)";
const char S47560[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SGH-I717D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47561[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47562[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47563[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; it-it) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47564[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; es-es) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47565[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; de-de) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47566[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fr-fr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47567[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; zh-cn) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47568[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ru-ru) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47569[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; cs-cz) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47570[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pl-pl) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47571[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nl-nl) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47572[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-au) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47573[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; hu-hu) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47574[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; es-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47575[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-ca) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47576[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; tr-tr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47577[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; zh-tw) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47578[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; de-at) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47579[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; el-gr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47580[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ar-ae) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47581[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fr-ca) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47582[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; sv-se) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47583[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-in) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47584[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; vi-vn) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47585[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fa-fa) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47586[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; de-ch) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47587[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nl-be) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47588[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; in-id) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47589[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pt-br) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47590[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-ph) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47591[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; da-dk) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47592[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-ie) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47593[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nb-no) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47594[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ja-jp) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon";
const char S47595[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SCH-I100 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 USCC-I100";
const char S47596[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SCH-I100 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47597[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SCH-I100 Build/FROYO)";
const char S47598[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; SCH-I100 Build/FROYO)";
const char S47599[] = "Mozilla/5.0 (Series40; NokiaC2-02/06.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S47600[] = "Mozilla/5.0 (Series40; NokiaC2-02.1/06.52; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S47601[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S47602[] = "Mozilla/5.0 (X11; CrOS i686 0.13.507) AppleWebKit/534.35 (KHTML, like Gecko) Chrome/13.0.763.0 Safari/534.35";
const char S47603[] = "Mozilla/5.0 (X11; CrOS i686 0.13.509) AppleWebKit/534.35 (KHTML, like Gecko) Chrome/13.0.764.0 Safari/534.35";
const char S47604[] = "Mozilla/5.0 (X11; CrOS i686 13.587.100) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.108 Safari/535.1";
const char S47605[] = "Mozilla/5.0 (X11; CrOS i686 13.587.135) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1";
const char S47606[] = "Mozilla/5.0 (X11; CrOS i686 0.13.587) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.14 Safari/535.1";
const char S47607[] = "Mozilla/5.0 (X11; CrOS i686 13.587.113) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.213 Safari/535.1";
const char S47608[] = "Mozilla/5.0 (X11; CrOS i686 13.587.48) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.43 Safari/535.1";
const char S47609[] = "Mozilla/5.0 (X11; CrOS i686 13.587.129) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.218 Safari/535.1";
const char S47610[] = "Mozilla/5.0 (X11; CrOS i686 0.13.477) AppleWebKit/534.33 (KHTML, like Gecko) Chrome/13.0.753.0 Safari/534.33";
const char S47611[] = "Mozilla/5.0 (X11; CrOS i686 13.587.126) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.216 Safari/535.1";
const char S47612[] = "Mozilla/5.0 (X11; CrOS i686 13.587.93) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1";
const char S47613[] = "Mozilla/5.0 (X11; CrOS i686 13.587.65) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.86 Safari/535.1";
const char S47614[] = "Mozilla/5.0 (X11; CrOS i686 13.587.80) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.99 Safari/535.1";
const char S47615[] = "Mozilla/5.0 (X11; CrOS i686 13.587.43) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.37 Safari/535.1";
const char S47616[] = "Mozilla/5.0 (X11; CrOS i686 13.587.32) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.27 Safari/535.1";
const char S47617[] = "Mozilla/5.0 (X11; CrOS i686 0.13.558) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.775.2 Safari/535.1";
const char S47618[] = "Mozilla/5.0 (X11; CrOS i686 13.587.40) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.34 Safari/535.1";
const char S47619[] = "Mozilla/5.0 (X11; CrOS i686 13.587.44) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.38 Safari/535.1";
const char S47620[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-SU540 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47621[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-P940 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47622[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-KU5400 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47623[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-P940/V20d Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47624[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-P940-Orange Build/V20d) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47625[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P940 Build/IMM76D)";
const char S47626[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-KU5400 Build/IMM76D)";
const char S47627[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-KU5400/20i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47628[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Safari/535.19 Silk-Accelerated=false";
const char S47629[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S47630[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195L Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S47631[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195T Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S47632[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; X8 Build/GingerDX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47633[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; X8 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47634[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; X8 Build/GingerDX; GingerDX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47635[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Xperia X8 Build/V10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47636[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15i Build/4.0.1.A.0.283; GingerXperia) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47637[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15i Build/3.0.1.A.0.145; GingerXperia) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47638[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; X8 Build/V12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47639[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; X8 Build/GingerDX)";
const char S47640[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; X8 Build/GingeeXperia; GingerXperia) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47641[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15i Build/4.0.2.A.0.69; GingerXperia) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47642[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Xperia_X8 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47643[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; X8 Build/GingerDX)";
const char S47644[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; X8 Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47645[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; Xperia X8 Build/V8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47646[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; pl-pl; X8 Build/V13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47647[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; Xperia X8 Build/V5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47648[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; E15i Build/3.0.1.A.0.145)";
const char S47649[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15i Build/2.0.0; STORNMIX-v 2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47650[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; E15a Build/Xperience stock) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47651[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; E15a Build/4.0.1.A.0.283; Sony Xperia X8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47652[] = "Mozilla/5.0 (Linux; U; Android Gingerbread 2.3.7; es-es; E15a Build/3.0.1.A.0.145; Sony Xperia X8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47653[] = "Mozilla/5.0 (Linux; U; Android 2.3.7-XPE; es-es; E15a Build/4.0.1.A.0.283; XPERIENCE STK2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47654[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; X8 Build/3.0.1.A.0.145; GingerDX-v029) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47655[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Xperia X8 Build/SEMC V3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47656[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15i Build/GingerZaraki; GingerZaraki) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47657[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15i Build/GingerSony; GingerSony) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47658[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15i Build/AltisV7.3; Xperia-Altis) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47659[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15i Build/10.0; PrimeHD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47660[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15i Build/fde +6  yousef-p and Ahmad.H; FDE +6 yousef-p and Ahmad.H) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47661[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ru-ru; E15 Build/GRH55; Leopard-X8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47662[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; E15i Build/4.0.2.A.0.42; Xperia-Altis.V7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47663[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5) Presto/2.12.388 Version/12.14";
const char S47664[] = "Opera/9.80 (Windows NT 6.2; WOW64; U; en) Presto/2.10.289 Version/12.00";
const char S47665[] = "Opera/9.80 (Windows NT 6.2; WOW64; U; en) Presto/2.10.289 Version/12.01";
const char S47666[] = "Mozilla/5.0 (Linux; Android 4.2.1; SCH-I535 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47667[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-T999 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47668[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9300 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47669[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T999 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47670[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I535 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47671[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47672[] = "Mozilla/5.0 (Linux; Android 4.2.2; it-it; SAMSUNG GT-I9300 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S47673[] = "Mozilla/5.0 (Series40; Nokia300/07.57; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S47674[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SCH-R915 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47675[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SCH-R915 Build/FROYO)";
const char S47676[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-I535 Build/JOP40D; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47677[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-T999 Build/JOP40D; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47678[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T999 Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47679[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I535 Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47680[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47681[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I535 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/d2vzw";
const char S47682[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T999 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/d2tmo";
const char S47683[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/i9300";
const char S47684[] = "Mozilla/5.0 (Linux; Android 4.1.1; ME172V Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S47685[] = "ASTRO36_TD/v3 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10";
const char S47686[] = "Spice M6688 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10";
const char S47687[] = "Symphony FT45 (MRE\\2.3.00(20480) resolution\\X chipset\\MT6255 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10";
const char S47688[] = "PhilipsX623 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; ru-RU) Presto/2.5.28 Version/10.10";
const char S47689[] = "ASTRO36_TD/v3 (MRE\\2.3.00(3072) resolution\\X chipset\\MT6255 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10";
const char S47690[] = "ASTRO36_TD/v3 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera (HTTPS) Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10";
const char S47691[] = "ASTRO36_TD/v3 (MRE\\2.3.00(5120) resolution\\X chipset\\MT6255 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10";
const char S47692[] = "ASTRO36_TD/v3 (MRE\\2.4.00(20480) resolution\\X chipset\\MT6255 touch\\0 tpannel\\0 camera\\1 gsensor\\0 keyboard\\qwerty) MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10";
const char S47693[] = "ASTRO36_TD/v3 (MRE&#92;2.3.00(20480) resolution&#92;X chipset&#92;MT6255 touch&#92;1 tpannel&#92;1 camera&#92;0 gsensor&#92;0 keyboard&#92;reduced) MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; id-ID) Presto/2.5.28 Version/10.10";
const char S47694[] = "Mozilla/5.0 (Linux; Android 4.0.3; U9200 Build/HuaweiU9200) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47695[] = "Mozilla/5.0 (Linux; Android 4.0.4; U9202L-1 Build/HuaweiU9202L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47696[] = "Mozilla/5.0 (Linux; Android 4.0.3; U9200E Build/HuaweiU9200E) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47697[] = "Mozilla/5.0 (Linux; Android 4.0.3; GS03 Build/HuaweiU9200-92) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47698[] = "Mozilla/5.0 (Linux; Android 4.0.4; U9202L-3 Build/HuaweiU9202L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47699[] = "Mozilla/5.0 (Linux; Android 4.0.4; U9202L-4 Build/HuaweiU9202L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47700[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en-US; rv:1.9.1.5) Gecko/X Firefox/3.5.5";
const char S47701[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190L Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47702[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47703[] = "Pingdom.com_bot_version_1.4_(http://www.pingdom.com/)";
const char S47704[] = "Pingdom.com_bot_version_1.4_(http://www.pingdom.com)";
const char S47705[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; HTC_Wildfire_A3333 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47706[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC_Wildfire_A3333 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S47707[] = "Mozilla/5.0 (Linux; Android 4.3; GT-I9300 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S47708[] = "Mozilla/5.0 (Linux; Android 4.3; SCH-I535 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S47709[] = "Mozilla/5.0 (Linux; Android 4.3; SGH-T999 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S47710[] = "Mozilla/5.0 (Linux; Android 4.3; SAMSUNG-SGH-T999 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S47711[] = "Mozilla/5.0 (compatible; heritrix/1.8.0 +http://yourwebsite.domain.com/)";
const char S47712[] = "Mozilla/5.0 (compatible; heritrix/2.0.2-SNAPSHOT-X.X +http://www.expertsearch.nl +@OPERATOR_FROM@)";
const char S47713[] = "Mozilla/5.0 (compatible; heritrix/3.0.0 +http://www.yell.com)";
const char S47714[] = "Mozilla/5.0 (compatible; heritrix/1.14.0+http://www.nationallibrary.fi/)";
const char S47715[] = "Mozilla/5.0 (compatible; heritrix/1.7.0 +http://www.greaterera.com/)";
const char S47716[] = "Mozilla/5.0 (compatible; heritrix/1.5.0-X http://pandora.nla.gov.au/crawl.html)";
const char S47717[] = "Mozilla/5.0 (compatible; heritrix/3.1.0 +http://internetmemory.org)";
const char S47718[] = "Mozilla/5.0 (compatible; heritrix/1.6.0 http://www.worio.com/)";
const char S47719[] = "Mozilla/5.0 (compatible; heritrix/1.4t http://www.truveo.com/)";
const char S47720[] = "Mozilla/5.0 (compatible; heritrix/@VERSION@ +http://www.mibka.com)";
const char S47721[] = "Nokia2700c/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S47722[] = "Nokia2700c-2/2.0 (09.97) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S47723[] = "Nokia2700c-2/2.0 (09.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2700c-2";
const char S47724[] = "Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabA2109A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S47725[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.3 Mobile/10A523 Safari/8536.25";
const char S47726[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.3 Mobile/10A403 Safari/8536.25";
const char S47727[] = "Mozilla/5.0 (Series40; NokiaC2-06/07.48; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S47728[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/533.3 (KHTML, like Gecko) rekonq Safari/533.3";
const char S47729[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) rekonq Safari/534.34";
const char S47730[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) rekonq Safari/534.X-10-16 20:21:01";
const char S47731[] = "SAMSUNG-GT-S5292 Opera/9.80 (J2ME/MIDP; Opera Mini/7.0.32584/31.1475; U; en) Presto/2.8.119 Version/11.10";
const char S47732[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S47733[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I535 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S47734[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-06D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S47735[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S47736[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300T Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S47737[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999V Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S47738[] = "Mozilla/5.0 (Linux; Android 4.0.4; i9300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S47739[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S47740[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-l9300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S47741[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9300 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S47742[] = "Opera/9.80 (S60; SymbOS; Opera Mobi/1178; U; en-GB) Presto/2.5.28 Version/10.1";
const char S47743[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyLT30at Build/7.0.B.0.165) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47744[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyLT30at Build/7.0.B.1.104) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47745[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyLT30at Build/VRC-X-1604-1052-ATT) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47746[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyLT30at Build/7.0.B.1.68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47747[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyLT30at Build/7.0.B.1.152) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47748[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT30at Build/7.0.B.1.152) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47749[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT30at Build/7.0.B.1.152)";
const char S47750[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; NookColor Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47751[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NookColor Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47752[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; NookColor Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47753[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; NookColor Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47754[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NookColor Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47755[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; NookColor Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47756[] = "Logos/0.9.3 (Linux; U; Android 2.3.7; en_US; NookColor Build/GWK74)";
const char S47757[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; NookColor Build/GRK39F) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.1 Mobile Safari/534.30";
const char S47758[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; NookColor Build/GRK39F)";
const char S47759[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; NookColor Build/GRI40)";
const char S47760[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; NookColor Build/GRI40)";
const char S47761[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; NookColor Build/GRJ22)";
const char S47762[] = "Mozilla/5.0 (Linux; U; Android 2.3.5.1; en-us; Nook Color Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47763[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8013 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S47764[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8013 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S47765[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile G1 Build/DMD64) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S47766[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us;T-Mobile G1 Build/DMD64) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S47767[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile G1 Build/DMD64) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 MobileSafari/525.20.1";
const char S47768[] = "Mozilla/5.0 (Linux; U; Android 1.6; de-; T-Mobile G1 Build/DRC92) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S47769[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile G1 Build/Super D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S47770[] = "Mozilla/5.0 (Linux; U; Android1.6; en-us; T-Mobile G1 Build/DMD64) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S47771[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; T-Mobile G1 Build/DRC92) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S47772[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile G1 Build/DRC83) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S47773[] = "Mozilla/5.0 (Linux; U;Android 1.6; en-us; T-Mobile G1 Build/DMD64) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S47774[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile G1 Build/DMD64) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S47775[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-fr; T-Mobile G1 Build/DRC83) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2";
const char S47776[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile G1 Build/DRC63) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S47777[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; tr-tr; GM FOX Build/HuaweiU8350) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47778[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; U8350 Build/HuaweiU8350) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47779[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; nl-be; Barcelona Build/HuaweiU8350) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47780[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; U8350 Build/HuaweiU8350)";
const char S47781[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-ar; Personal U8350 Build/HuaweiU8350) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47782[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-ar; U8350-51 Build/HuaweiU8350) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47783[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XOOM 2 Build/7.7.1-85_MZ615-26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47784[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XOOM 2 Build/7.7.1-128_MZ615-12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47785[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; XOOM 2 Build/7.7.1_128) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47786[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; XOOM 2 Build/7.7.1-85_MZ615-26)";
const char S47787[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XOOM 2 Build/7.7.1-128_MZ615-6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S47788[] = "Opera/9.50 (J2ME/MIDP; Opera Mini/7.5.33466/29.3757; U; en)";
const char S47789[] = "Mozilla/5.0 (Windows NT 4.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S47790[] = "Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 4.1; Maxthon)";
const char S47791[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Maxthon)";
const char S47792[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; Maxthon)";
const char S47793[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 4.1; Maxthon)";
const char S47794[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Maxthon)";
const char S47795[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; Maxthon)";
const char S47796[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; Avant Browser; Maxthon)";
const char S47797[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Maxthon; Maxthon)";
const char S47798[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; Maxthon; SV1)";
const char S47799[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; Maxthon; Avant Browser)";
const char S47800[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; Maxthon; Maxthon)";
const char S47801[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; SV1; Maxthon)";
const char S47802[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; SV1; Maxthon)";
const char S47803[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; Maxthon; Maxthon)";
const char S47804[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Avant Browser; Maxthon)";
const char S47805[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.10; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)";
const char S47806[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Maxthon; Avant Browser)";
const char S47807[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Maxthon; SV1)";
const char S47808[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Maxthon; Maxthon)";
const char S47809[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.10; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)";
const char S47810[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1";
const char S47811[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1";
const char S47812[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1";
const char S47813[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1";
const char S47814[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1";
const char S47815[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1";
const char S47816[] = "Mozilla/5.0 (Series40; NokiaC2-05/08.69; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6";
const char S47817[] = "Mozilla/5.0 (Series40; NokiaC2-05/08.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S47818[] = "Mozilla/5.0 (Series40; NokiaC2-05.1/08.69; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6";
const char S47819[] = "Mozilla/5.0 (Series40; NokiaC2-05.1/08.60b; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S47820[] = "Mozilla/5.0 (Linux; Android 4.0.4;  SGH-I777 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S47821[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I777 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47822[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-I777 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47823[] = "Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I777 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47824[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I777 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47825[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:19.0) Gecko/19.0 Firefox/19.0";
const char S47826[] = "Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S47827[] = "Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5130c-2";
const char S47828[] = "Nokia5130/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Nokia5130/07.95";
const char S47829[] = "Nokia5130/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S47830[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.2 Safari/535.19 Silk-Accelerated=true";
const char S47831[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; fj_vortex Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47832[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; fj_vortex Build/IML74K)";
const char S47833[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9a1) Gecko/X GranParadiso/3.0a1";
const char S47834[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9a1) Gecko/X GranParadiso/2.0";
const char S47835[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9.1.1) Gecko/X Firefox/3.0.10";
const char S47836[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; C771 Build/C771M030) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47837[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; C771 Build/C771M050) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47838[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; C771 Build/C771M050)";
const char S47839[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; C771 Build/C771M020) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47840[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-AS680 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47841[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-AS680 Build/GRJ22)";
const char S47842[] = "Mozilla/5.0 (Linux; Android 4.1.2; VS870 4G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47843[] = "Mozilla/5.0 (Linux; Android 4.1.1; LG-VS870 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47844[] = "Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S47845[] = "Mozilla/5.0 (Linux; Android 4.0.3; TF101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S47846[] = "Mozilla/5.0 (Linux; Android 4.0.4; Transformer Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S47847[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF700T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S47848[] = "Mozilla/5.0 (Linux; Android 4.2.2; ASUS Transformer Pad TF700T Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S47849[] = "SAMSUNG-GT-E2550/E2550DXXKC3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S47850[] = "SAMSUNG-GT-E2550L/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S47851[] = "Vodafone/1.0/SAMSUNG-GT-E2550/XXJF5 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S47852[] = "Vodafone/1.0/SAMSUNG-GT-E2550/XXJI1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S47853[] = "Vodafone/1.0/SAMSUNG-GT-E2550/XXJH3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S47854[] = "Vodafone/1.0/SAMSUNG-GT-E2550/XXJL2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S47855[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; SonyEricssonR800iv Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47856[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-nz; SonyEricssonR800iv Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47857[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; SonyEricssonR800iv Build/3.0.1.A.0.146) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47858[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-za; SonyEricssonR800iv Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47859[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ie; SonyEricssonR800iv Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47860[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonR800iv Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47861[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonR800iv Build/4.0.2.A.0.79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47862[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonR800iv Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47863[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonR800iv Build/4.0.2.A.0.79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47864[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5233) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S47865[] = "Nokia5233/UCWEB8.8.0.212/50/800";
const char S47866[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia 5233) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S47867[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Sensation Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47868[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Sensation Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47869[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Sensation Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47870[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Sensation Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47871[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Sensation Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47872[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Sensation mEMBER's Choice Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47873[] = "Reeder/1020.09.00 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookPro8%2C2)";
const char S47874[] = "Reeder/1020.09.00 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacPro3%2C1)";
const char S47875[] = "Reeder/1020.09.00 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (iMac12%2C2)";
const char S47876[] = "Reeder/1020.09.00 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookAir4%2C2)";
const char S47877[] = "Reeder/1020.09.00 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBook5%2C1)";
const char S47878[] = "Reeder/1020.09.00 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (Macmini5%2C1)";
const char S47879[] = "Mozilla/5.0 (Linux; Android 4.2.1; SAMSUNG-SGH-I747 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47880[] = "Mozilla/5.0 (Linux; Android 4.2.1; SPH-L710 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47881[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-I747 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47882[] = "Mozilla/5.0 (Linux; Android 4.2.1; SCH-R530U Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47883[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I747 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47884[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L710 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47885[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I747 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47886[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530U Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47887[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9305 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47888[] = "Mozilla/5.0 (Linux; Android 4.0.4;  Xoom Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S47889[] = "Mozilla/5.0 (Linux; Android 4.0.3;  Xoom Build/IML77) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S47890[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xoom Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S47891[] = "Mozilla/5.0 (Linux; Android 4.0.3; Xoom Build/IML77) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S47892[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ505 Build/Moto_MZ505) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S47893[] = "Opera/9.80 (X11; Linux i686; U; en) Presto/2.6.30 Version/10.63";
const char S47894[] = "Opera/9.80 (X11; Linux i686; U; en) Presto/2.6.30 Version/10.60";
const char S47895[] = "Opera/9.80 (X11; Linux i686; U; en) Presto/2.6.30 Version/10.62";
const char S47896[] = "Opera/9.80 (X11; Linux i686; U; es-ES) Presto/2.6.30 Version/10.61";
const char S47897[] = "Opera/10.60 (Linux armv6l ; U; CE-HTML/1.0 NETTV/3.0.1;; en) Presto/2.6.33 Version/10.60";
const char S47898[] = "Opera/9.80 (Linux armv6l ; U; CE-HTML/1.0 NETTV/3.0.1;; en) Presto/2.6.33 Version/10.60";
const char S47899[] = "Opera/9.80 (Linux armv7l ; U; CE-HTML/1.0 NETTV/3.2.2; en) Presto/2.6.33 Version/10.60";
const char S47900[] = "Opera/9.80 (iPhone; Opera Mini/6.0.1/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S47901[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-I747M Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.2/552,2.0.2.552,2013-05-22 23:05, -0400, ca";
const char S47902[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I747M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, ca";
const char S47903[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us";
const char S47904[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0500, us";
const char S47905[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9305 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb";
const char S47906[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us";
const char S47907[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530U Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0500, us";
const char S47908[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-L300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47909[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SPH-L300 Build/IMM76D)";
const char S47910[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S47911[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S47912[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S47913[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S47914[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S47915[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S47916[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S47917[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S47918[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S47919[] = "Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S47920[] = "Mozilla/5.0 (Linux; Android 4.1.2; MB886 Build/9.8.2Q-50_MB886_NII_TA-2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S47921[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.231 Safari/534.10";
const char S47922[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10";
const char S47923[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_0; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10";
const char S47924[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10";
const char S47925[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10";
const char S47926[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10";
const char S47927[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.215 Safari/534.10";
const char S47928[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.231 Safari/534.10";
const char S47929[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.10 (KHTML	 like Gecko) Chrome/8.0.552.237 Safari/534.10";
const char S47930[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Dell Streak 7 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47931[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Dell Streak 7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S47932[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; Dell Streak 7 Build/FRG83G)";
const char S47933[] = "Mozilla/5.0 (Linux; U;Android 2.2.2; en-us; Dell Streak 7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S47934[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; DellStreak 7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S47935[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;Dell Streak 7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S47936[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Dell Streak 7Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S47937[] = "Mozilla/5.0 (Linux; U; Android2.2.2; en-us; Dell Streak 7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S47938[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.60";
const char S47939[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4876); ru) Presto/2.10.229 Version/11.60";
const char S47940[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4139); ru) Presto/2.10.229 Version/11.60";
const char S47941[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.60";
const char S47942[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4664); ru) Presto/2.10.229 Version/11.60";
const char S47943[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.60";
const char S47944[] = "Opera/9.80 (Windows NT 6.0; U; en) Presto/2.10.229 Version/11.60";
const char S47945[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.4 (build 02647); ru) Presto/2.10.229 Version/11.60";
const char S47946[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4598); ru) Presto/2.10.229 Version/11.60";
const char S47947[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.60";
const char S47948[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.60";
const char S47949[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; en) Opera 11.60";
const char S47950[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03790); ru) Presto/2.10.229 Version/11.60";
const char S47951[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4947); ru) Presto/2.10.229 Version/11.60";
const char S47952[] = "Opera/9.80 (Windows NT 5.1; U; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0); en) Presto/2.10.229 Version/11.60";
const char S47953[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.6 (build 03278); ru) Presto/2.10.229 Version/11.60";
const char S47954[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4157); ru) Presto/2.10.229 Version/11.60";
const char S47955[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4848); ru) Presto/2.10.229 Version/11.60";
const char S47956[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4133); ru) Presto/2.10.229 Version/11.60";
const char S47957[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF700T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S47958[] = "Mozilla/5.0 (Linux; Android 4.2.2; ASUS Transformer Pad TF700T Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S47959[] = "NokiaN70-1/5.0737.3.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S47960[] = "NokiaN70/ 5.0741.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S47961[] = "NokiaN70/SymbianOS/8.0 Series60/2.6";
const char S47962[] = "NokiaN70-1/SymbianOS/8.0 Series60/2.6";
const char S47963[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Mobile/11A4414e Twitter for iPhone";
const char S47964[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A4400f Twitter for iPhone";
const char S47965[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4435d Twitter for iPhone";
const char S47966[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449a Twitter for iPhone";
const char S47967[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d Twitter for iPhone";
const char S47968[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Twitter for iPhone";
const char S47969[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Twitter for iPhone";
const char S47970[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; USCC-US760 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S47971[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; USCC-US760 Build/FRG83G)";
const char S47972[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P880 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47973[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P880g Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S47974[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F180L Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47975[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F180S Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47976[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F180K Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S47977[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-F180L Build/IMM76L)";
const char S47978[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-F180S Build/IMM76L)";
const char S47979[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Flipboard/2.0";
const char S47980[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Flipboard/2.0";
const char S47981[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0";
const char S47982[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.1";
const char S47983[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Flipboard/2.0.1";
const char S47984[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.2";
const char S47985[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Flipboard/2.0.2";
const char S47986[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Flipboard/2.0.2";
const char S47987[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.3";
const char S47988[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.4";
const char S47989[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Flipboard/2.0.4";
const char S47990[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Flipboard/2.0.4";
const char S47991[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.5";
const char S47992[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.6";
const char S47993[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.7";
const char S47994[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-US) AppleWebKit/530.5 (KHTML, like Gecko) Chrome/ Safari/530.5";
const char S47995[] = "Mozilla/5.0 (Macintosh; U; Mac OS X 10_5_7; en-US) AppleWebKit/530.5 (KHTML, like Gecko) Chrome/ Safari/530.5";
const char S47996[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-US) AppleWebKit/530.9 (KHTML, like Gecko) Chrome/ Safari/530.9";
const char S47997[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-US) AppleWebKit/530.6 (KHTML, like Gecko) Chrome/ Safari/530.6";
const char S47998[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337M Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S47999[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8500/S8500XXJL2; U; Bada/1.2; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.2 Mobile WVGA SMM-MMS/1.2.0 OPN-B";
const char S48000[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; XOOM 2 Build/7.7.1-128_MZ616-11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S48001[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XOOM 2 Build/7.7.1-128_MZ616-12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S48002[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MZ616 Build/7.7.1-141-6-PAST-UMTS-LA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S48003[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ616-12)";
const char S48004[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:9.0.1) Gecko/X Firefox/9.0.1";
const char S48005[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; ONE TOUCH 983 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48006[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; ALCATEL_one_touch_983 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48007[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; ONE TOUCH 983 Build/GINGERBREAD)";
const char S48008[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xoom Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S48009[] = "Mozilla/5.0 (Linux; Android 4.1.1; Xoom Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S48010[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-i9100 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36";
const char S48011[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48012[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48013[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48014[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-i9100G Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48015[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_EVO3D_X515m; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S48016[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_EVO3D_X515a; en-ca) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S48017[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_X515a V1.24.841.3; en-au) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S48018[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_X515m; zh-cn) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S48019[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:10.0.7) Gecko/X Firefox/10.0.7 TenFourFox/7450";
const char S48020[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S48021[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710VMUB Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S48022[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710BST Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S48023[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC HD2 Build/GRI40; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48024[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC HD2 Build/GRI40; HyperDroid-CM7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48025[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HD2 Build/GRK39F; HyperDroid-CM7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48026[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC HD2 Build/GRI40; cope & warrenb present: AmeriCanAndroid GB237 CyanogenMod7.2 HTC LEO X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48027[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HD2 Build/GRI40; HyperDroid-CM7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48028[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HD2 Build/GRI40; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48029[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; HTC HD2 Build/GRI40; Negro90 V.18 CyanogenMod 7.1 MAGLDR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48030[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; HTC HD2 Build/GWK74; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48031[] = "Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S48032[] = "Mozilla/5.0 (Linux; Android 4.0.4; Transformer Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S48033[] = "Mozilla/5.0 (Linux; Android 4.0.3; TF101 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S48034[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Ubuntu/10.04 Chromium/6.0.472.62 Chrome/6.0.472.62 Safari/534.3";
const char S48035[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Ubuntu/10.04 Chromium/6.0.472.53 Chrome/6.0.472.53 Safari/534.3";
const char S48036[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-sa; LG-P720h Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S48037[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-P725 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48038[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; LG-P720 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S48039[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; LG-P720/V10e Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S48040[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-P725 Build/GRK39F)";
const char S48041[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; LG-P725/V10f Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48042[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; LG-P720/V10a Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S48043[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; LG-P725/V10d Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48044[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P5110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S48045[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P5110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S48046[] = "Opera/9.70 (WindowsNT 5.1; U; en) TMO-US_LEO";
const char S48047[] = "Opera/9.70 (Windows NT5.1; U; en) TMO-US_LEO";
const char S48048[] = "Opera/9.70 (Windows NT 5.1; U; en) TMO-US_LEO";
const char S48049[] = "Opera 9.7 (Windows NT 5.2; U; en)";
const char S48050[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT890 Build/9.8.2I-50_SMI-26) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48051[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID7048 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S48052[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; MID7048 Build/IML74K)";
const char S48053[] = "Mozilla/5.0 (Linux; U; Android 1.5; fr-fr; HTC Hero-Orange Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S48054[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-gb; HTC Hero Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S48055[] = "Mozilla/5.0 (Linux; U; Android 1.5; de-ch; HTC Hero Build/CUPCAKE) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S48056[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; ALCATEL_one_touch_985 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48057[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; ALCATEL ONE TOUCH 985N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48058[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; ALCATEL ONE TOUCH 985D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48059[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; ALCATEL ONE TOUCH 985 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48060[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LW770 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48061[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-LW770 Build/IMM76I)";
const char S48062[] = "Opera/9.80 (X11; Linux i686) Presto/2.12.388 Version/12.11";
const char S48063[] = "Mozilla/5.0 (X11; Linux i686; rv:2.0) Gecko/X Firefox/4.0 Opera 12.11";
const char S48064[] = "Mozilla/5.0 (compatible; MSIE 9.0; X11; Linux i686) Opera 12.11";
const char S48065[] = "Opera/9.80 (Linux armv7l; Opera TV Store/5521) Presto/2.12.362 Version/12.11";
const char S48066[] = "Opera/9.80 (Linux armv7l; Model/Sony-KDL-46EX650 SonyCEBrowser/1.0 (KDL-46EX650; CTV/1.0; JPN; en)) Presto/2.12.362 Version/12.11";
const char S48067[] = "Opera/9.80 (Linux armv7l; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-50R550A; CC/USA) Presto/2.12.362 Version/12.11";
const char S48068[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48069[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC6435LVW Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48070[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.0.400) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48071[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X+ Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48072[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC EVARE_UL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48073[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8010 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S48074[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8010 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S48075[] = "NetFront/3.5.1 (BREW 5.0.1.2; U; en-us; LG; NetFront/3.5.1/AMB) Sprint LN510 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S48076[] = "NetFront/3.5.1 (BREW 5.0.1.2; U; en-us; LG; NetFront/3.5.1/AMB) Sprint LN510 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1, NetFront/3.5.1 (BREW 5.0.1.2; U; en-us; LG; NetFront/3.5.1/AMB) Sprint LN510 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S48077[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9295 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S48078[] = "Mozilla/5.0 (Linux; Android 4.2.2; de-de; SAMSUNG GT-I9295/I9295XXUAMF5 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S48079[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; GT-I9210T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48080[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9210T Build/IML74K)";
const char S48081[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-I9210T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48082[] = "UCWEB/2.0 (iOS; U; iPh OS 6_1_3; zh-CN; iPh4,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile";
const char S48083[] = "UCWEB/2.0 (iOS; U; iPh OS 6_1_4; zh-CN; iPh5,2) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile";
const char S48084[] = "UCWEB/2.0 (iOS; U; iPh OS 6_1_3; zh-CN; iPh3,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile";
const char S48085[] = "UCWEB/2.0 (iOS; U; iPh OS 6_1_3; en-US; iPh2,1) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S48086[] = "UCWEB/2.0 (iOS; U; iPh OS 6_1_3; zh-CN; iPh3,2) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile";
const char S48087[] = "UCWEB/2.0 (iOS; U; iPh OS 6_1_4; zh-CN; iPh5,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile";
const char S48088[] = "UCWEB/2.0 (iOS; U; iPh OS 6_1_3; en-US; iPh3,3) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S48089[] = "UCWEB/2.0 (iOS; U; iPh OS 6_1_3; vi; iPh3,1) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S48090[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9981; en-GB) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.342 Mobile Safari/534.11+";
const char S48091[] = "Mozilla/5.0 (X11; U; Linux armv7l; en-GB; rv:1.9.2.3pre) Gecko/20100723 Firefox/3.5 Maemo Browser 1.7.4.8 RX-51 N900";
const char S48092[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE Plus Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S48093[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; MI-ONE Plus Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48094[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE Plus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48095[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; MI-ONE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48096[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE Plus Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S48097[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48098[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; MI-ONE Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S48099[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S48100[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; zh-cn; MI-ONE Plus Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48101[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; MI-ONE Plus Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S48102[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S48103[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MI-ONE Plus Build/IMM76D)";
const char S48104[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; MI-ONE Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S48105[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE Plus Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48106[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MI-ONE Plus MIUI/ICS24.0)";
const char S48107[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) konqueror/4.9.5 Safari/534.34";
const char S48108[] = "Mozilla/5.0 (X11; Linux i686) KHTML/4.9.5 (like Gecko) Konqueror/4.9";
const char S48109[] = "Mozilla/5.0 (X11; Linux) KHTML/4.9.1 (like Gecko) Konqueror/4.9";
const char S48110[] = "Mozilla/5.0 (X11) KHTML/4.9.1 (like Gecko) Konqueror/4.9";
const char S48111[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/14.0.835.186)";
const char S48112[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/14.0.835.202) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S48113[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/14.0.835.202; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S48114[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/14.0.835.202)";
const char S48115[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/14.0.835.163) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.163 Safari/535.1";
const char S48116[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/14.0.835.202; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S48117[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/14.0.835.109) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.109 Safari/535.1";
const char S48118[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/14.0.835.202; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S48119[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/14.0.835.186) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1";
const char S48120[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/14.0.835.187) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.187 Safari/535.1";
const char S48121[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/14.0.835.94) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.94 Safari/535.1";
const char S48122[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/14.0.835.202; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)";
const char S48123[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/14.0.835.15) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.15 Safari/535.1";
const char S48124[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/14.0.803.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S48125[] = "Mozilla/5.0 (Windows NT 5.1; chromeframe/14.0.803.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.803.0 Safari/535.1";
const char S48126[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE75-2/202.12.01; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.1.19424";
const char S48127[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2NokiaE75-1/100.48.78 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S48128[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 NokiaE75-1 /110.48.125 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S48129[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE75-1/211.12.01; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.1.1";
const char S48130[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405,Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S48131[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S48132[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S48133[] = "Mozilla/5.0 (iPad; U; CPU OS 5_0_1 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.10";
const char S48134[] = "Mozilla/5.0 (iPad; U; CPU OS 5_0_1 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S48135[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-M950 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48136[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SPH-M950 Build/IMM76I)";
const char S48137[] = "Mozilla/5.0 (Linux; Android 4.0.3;  ADR6425LVW Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S48138[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR6425LVW Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48139[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6425LVW Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48140[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315";
const char S48141[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S48142[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P6800 Build/JRO03H)";
const char S48143[] = "Logos/2.1.2 (Linux; U; Android 4.1.1; en-US; GT-P6800 Build/pa_p6800-userdebug 4.1.1 JRO03L eng.androguide.X.X test-keys)";
const char S48144[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; GT-P6800 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S48145[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P6800 Build/JRO03L; 800*1216) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S48146[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; GT-P6800 Build/JR003H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S48147[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6800 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S48148[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6800 Build/IMML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S48149[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P6800 Build/JZO54K)";
const char S48150[] = "Logos/2.2.3 (Linux; U; Android 4.1.2; en-SG; GT-P6800 Build/cm_p6800-userdebug 4.1.2 JZO54K eng.aorth.X.X test-keys)";
const char S48151[] = "GoogleAnalytics/2.0 (Linux; U; Android 4.1.1; en-us; GT-P6800 Build/JRO03R)";
const char S48152[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P6800 Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S48153[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P6800 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S48154[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P6800 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S48155[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P6800 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48156[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I537 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48157[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9A334 Safari/7534.48.3";
const char S48158[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9A406 Safari/7534.48.3";
const char S48159[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9A405 Safari/7534.48.3";
const char S48160[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S48161[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S48162[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S48163[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989D Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S48164[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; LIFETAB_S9512 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S48165[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; LIFETAB_S9512 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S48166[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LIFETAB_S9512 Build/IMM76D)";
const char S48167[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC first Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48168[] = "Mozilla/5.0 (X11; CrOS i686 12.433.257) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.130 Safari/534.30";
const char S48169[] = "Mozilla/5.0 (X11; CrOS i686 12.433.114) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.105 Safari/534.30";
const char S48170[] = "Mozilla/5.0 (X11; CrOS i686 12.433.231) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.113 Safari/534.30";
const char S48171[] = "Mozilla/5.0 (X11; CrOS i686 12.433.109) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.93 Safari/534.30";
const char S48172[] = "Mozilla/5.0 (X11; CrOS i686 12.0.742.91) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.93 Safari/534.30";
const char S48173[] = "Mozilla/5.0 (X11; CrOS i686 12.433.216) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.105 Safari/534.30";
const char S48174[] = "Mozilla/5.0 (X11; CrOS i686 0.12.433) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.77 Safari/534.30";
const char S48175[] = "Mozilla/5.0 (X11; CrOS i686 0.0.0) AppleWebKit/534.28 (KHTML, like Gecko) Chrome/12.0.723.1 Safari/534.28";
const char S48176[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; D930 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48177[] = "Influencebot/0.9; (Automatic classification of websites; http://www.influencebox.com/; info@influencebox.com)User-Agent: Mozilla/5.0 (X11; Linux i686; rv:9.0) Gecko/X Firefox/9.0";
const char S48178[] = "Influencebot/0.9; (Automatic classification of websites; http://www.influencebox.com/; info@influencebox.com)Mozilla/5.0 (X11; Linux i686; rv:9.0) Gecko/X Firefox/9.0";
const char S48179[] = "Mozilla/5.0 (PLAYSTATION 3 4.30) AppleWebKit/531.22.8 (KHTML, like Gecko)";
const char S48180[] = "NewsBlur Page Fetcher (1 subscriber) - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)";
const char S48181[] = "NewsBlur Page Fetcher (2 subscribers) - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)";
const char S48182[] = "NewsBlur Feed Finder (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)";
const char S48183[] = "NewsBlur Page Fetcher (3 subscribers) - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)";
const char S48184[] = "NewsBlur Page Fetcher (4 subscribers) - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)";
const char S48185[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/536.2.3 (KHTML, like Gecko) Version/5.2 (NewsBlur Feed Fetcher - 56 subscribers - http://www.newsblur.com)";
const char S48186[] = "NewsBlur Feed Fetcher - 1 subscriber - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/536.2.3 (KHTML, like Gecko) Version/5.2)";
const char S48187[] = "NewsBlur Feed Fetcher - 3 subscribers - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/536.2.3 (KHTML, like Gecko) Version/5.2)";
const char S48188[] = "NewsBlur Feed Fetcher - 2 subscribers - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/536.2.3 (KHTML, like Gecko) Version/5.2)";
const char S48189[] = "NewsBlur Page Fetcher (6 subscribers) - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)";
const char S48190[] = "NewsBlur Page Fetcher (5 subscribers) - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)";
const char S48191[] = "NewsBlur Page Fetcher (0 subscribers) - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)";
const char S48192[] = "NewsBlur Page Fetcher (7 subscribers) - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)";
const char S48193[] = "NewsBlur Page Fetcher (56 subscribers) - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)";
const char S48194[] = "NewsBlur Content Fetcher - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)";
const char S48195[] = "NewsBlur Feed Fetcher - 4 subscribers - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/536.2.3 (KHTML, like Gecko) Version/5.2)";
const char S48196[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337M Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48197[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337M Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S48198[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonIS11S Build/3.0.1.C.1.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48199[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11S Build/3.0.1.C.1.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48200[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11S Build/3.0.1.C.2.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48201[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonIS11S Build/3.0.1.C.2.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48202[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11S Build/3.0.1.C.1.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48203[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonIS11S Build/3.0.1.C.1.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48204[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11S Build/3.0.1.C.0.80) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48205[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonIS11S Build/3.0.1.C.0.80) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48206[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonIS11S Build/3.0.1.C.3.31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48207[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11S Build/3.0.1.C.3.31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48208[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonIS11S Build/4.0.1.B.0.112) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48209[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; IS11S Build/4.0.1.B.0.112) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48210[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; IS11S Build/4.0.1.B.0.117) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48211[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonIS11S Build/4.0.1.B.0.117) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48212[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonIS11S Build/4.0.1.B.0.118) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48213[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; IS11S Build/4.0.1.B.0.118) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48214[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; IS11S Build/4.0.1.B.0.118)";
const char S48215[] = "Mozilla/2.0 (compatible; MSIE 3.0B; Win32)";
const char S48216[] = "Mozilla/2.0 (compatible; MSIE 3.0B; Windows NT)";
const char S48217[] = "Mozilla/2.0 (compatible; MSIE 3.02; Update a; AK; Windows NT)";
const char S48218[] = "Mozilla/2.0 (compatible; MSIE 3.01; Windows NT)";
const char S48219[] = "Mozilla/5.0 (compatible; MSIE 3.0; 68K; Trident/1.0)";
const char S48220[] = "Mozilla/5.0 (compatible; MSIE 3.0; 68K; WOW64; Trident/1.0)";
const char S48221[] = "Mozilla/5.0 (compatible; MSIE 3.0; 68K; Win64; x64; Trident/2.0)";
const char S48222[] = "Mozilla/5.0 (compatible; MSIE 3.0; 68K; SV1; Trident/5.0)";
const char S48223[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-GB; rv:1.9.2.13) Gecko/X Ubuntu/9.10 (karmic) Firefox/3.6.13";
const char S48224[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One X Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48225[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One X Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48226[] = "Opera/9.80 (BlackBerry; Opera Mini/6.0.24209/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S48227[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9B176 Safari/6533.18.5";
const char S48228[] = "Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X; zh-cn Model:iPad2,1) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S48229[] = "Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X; 6534.46) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5";
const char S48230[] = "Mozilla/5.0 (iPad; U; CPU OS 5_1_1 like Mac OS X; en_us) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5";
const char S48231[] = "Mozilla/5.0 (iPad; U; CPU OS 5_1_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S48232[] = "Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X; en_us) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5";
const char S48233[] = "Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S48234[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9B206 Safari/6533.18.5";
const char S48235[] = "Mozilla/5.0 (iPad; U; CPU OS 5_1_1 like Mac OS X; 6534.46) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5";
const char S48236[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/6533.18.5 DiigoBrowser";
const char S48237[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/6533.18.5 DiigoBrowser Safari/6533.18.5 DiigoBrowser";
const char S48238[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/6533.18.5 DiigoBrowser";
const char S48239[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/6533.18.5 DiigoBrowser Safari/6533.18.5 DiigoBrowser Safari/6533.18.5 DiigoBrowser";
const char S48240[] = "Mozilla/5.0 (iPad; U; CPU OS 5_1_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5";
const char S48241[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; 801a Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48242[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ALCATEL ONE TOUCH 918N Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48243[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ALCATEL ONE TOUCH 918N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48244[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; ALCATEL_one_touch_918N Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48245[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8005 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S48246[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8020 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S48247[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8000 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S48248[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8020 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S48249[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8000 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S48250[] = "Mozilla/5.0 (Linux; Android 4.0.4; N861 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48251[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MOT-XT535 Build/V1.540) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48252[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; MOT-XT535 Build/V1.60A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48253[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT535 Build/V1.540) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48254[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT535 Build/V1.47G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S48255[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; XT535 Build/V1.50A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48256[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; MOT-XT535 Build/V1.57A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48257[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MOT-XT535 Build/V1.47G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48258[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MOT-XT535 Build/V1.50A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48259[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MOT-XT535 Build/V1.50D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48260[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; XT535 Build/V1.57A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48261[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; MOT-XT535 Build/V1.610) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48262[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; XT535 Build/V1.60A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48263[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT535 Build/V1.47G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48264[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; XT535 Build/V1.540)";
const char S48265[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MOT-XT535 Build/V1.47I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48266[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; XT535 Build/V1.610) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48267[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MOT-XT535 Build/V1.47H) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48268[] = "Facebook 4110.0 (iPhone; iPhone OS 6.0; en_US)";
const char S48269[] = "Appcelerator Titanium/1.7.2 (iPhone/6.0.1; iPhone OS; en_US;)";
const char S48270[] = "FeeddlerRSS 1.12.5 (iPhone; iPhone OS 6.0.1; en_US)";
const char S48271[] = "Pulse 3.0.3 (iPhone; iPhone OS 6.0.2; en_US)";
const char S48272[] = "Nokia5800 XpressMusic/UCWEB8.7.0.187/50/999";
const char S48273[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5800 XpressMusic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S48274[] = "Nokia5800 XpressMusicUCBrowser/8.7.1.234/50/444/UCWEB";
const char S48275[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5800d-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba";
const char S48276[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Maxthon 2.0)";
const char S48277[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; InfoPath.2; .NET CLR 3.5.20706; .NET CLR 3.0.04506; MAXTHON 2.0)";
const char S48278[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Maxthon 2.0)";
const char S48279[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; MAXTHON 2.0)";
const char S48280[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; Maxthon 2.0)";
const char S48281[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; SLCC2; Maxthon 2.0; DigExt; Zune 4.7)";
const char S48282[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SV1; .NET CLR 1.1.4322; MAXTHON 2.0)";
const char S48283[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.04506; Media Center PC 5.0; .NET CLR 1.1.4322; .NET CLR 3.5.21022; MAXTHON 2.0)";
const char S48284[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; InfoPath.2; eMusic DLM/3; MAXTHON 2.0)";
const char S48285[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET4.0C; .NET CLR 3.0.30729; Maxthon 2.0)";
const char S48286[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB0.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MAXTHON 2.0)";
const char S48287[] = "Mozilla/5.0 (Linux; Android 4.0.4; BNTV600 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S48288[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A523 Safari/8536.25";
const char S48289[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A403 Safari/8536.25";
const char S48290[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A550 Safari/8536.25";
const char S48291[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A8426 Safari/8536.25";
const char S48292[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A407 Safari/8536.25";
const char S48293[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A8500 Safari/8536.25";
const char S48294[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A406 Safari/8536.25";
const char S48295[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT28at Build/6.1.C.1.111) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48296[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT28h Build/6.1.E.3.7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48297[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT28i Build/6.1.E.2.68) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48298[] = "AppleCoreMedia/1.0.0.10A403 (iPhone; U; CPU OS 6_0 like Mac OS X; en_us)";
const char S48299[] = "AppleCoreMedia/1.0.0.10A523 (iPhone; U; CPU OS 6_0_1 like Mac OS X; en_us)";
const char S48300[] = "AppleCoreMedia/1.0.0.10A551 (iPhone; U; CPU OS 6_0_2 like Mac OS X; en_us)";
const char S48301[] = "Mozilla/5.0 (Linux; Android 4.0.4; C5155 Build/IML77) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48302[] = "Mozilla/5.0 (Linux; Android 4.0.4; Rise Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48303[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48304[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One S Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48305[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC VLE_U Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48306[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC VLE_U Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48307[] = "Mozilla/5.0 (Series40; NokiaC2-03/07.48; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.9";
const char S48308[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.0 Mobile/10A403 Safari/8536.25";
const char S48309[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-S738C Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48310[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-mx; SonyEricssonU20a Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48311[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; U20a Build/2.0.1.A.0.47) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48312[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-mx; U20a Build/2.0.A.0.504) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48313[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-co; U20a Build/2.0.2.A.0.24) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48314[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-mx; U20a Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48315[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; U20a Build/2.1.1.A.0.6)";
const char S48316[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5210 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S48317[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I5503 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48318[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; GT-I5503T Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48319[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I5503 Build/ERE27)";
const char S48320[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I5503 Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S48321[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I5503T Build/ERE27)";
const char S48322[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; GT-I5503 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48323[] = "Opera/9.80 (Android 2.3.4; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S48324[] = "Opera/9.80 (Android 2.3.3; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S48325[] = "Opera/9.80 (Android 2.3.5; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S48326[] = "Opera/9.80 (Android 2.3.6; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S48327[] = "Opera/9.80 (Android 2.3.2; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S48328[] = "Opera/9.80 (Android 2.3.7; Linux; Opera Mobi/46154) Presto/2.11.355 Version/12.10";
const char S48329[] = "Opera/9.80 (Android 2.3.1; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S48330[] = "Opera/9.80 (Android 2.3.8; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S48331[] = "Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2730c-1";
const char S48332[] = "Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S48333[] = "Nokia2730c/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S48334[] = "Nokia2730c-1b/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S48335[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-AS730 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48336[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-AS730 Build/IMM76L)";
const char S48337[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-L710 Build/JOP40D; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48338[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I747 Build/JOP40D; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48339[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I747 Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48340[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L710 Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48341[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530U Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48342[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; GT-I9305 Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48343[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I747 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/d2att";
const char S48344[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-R530U Build/JOP40D; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48345[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L710 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/d2spr";
const char S48346[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9305 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/i9305";
const char S48347[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315";
const char S48348[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315";
const char S48349[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S48350[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S48351[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S48352[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S48353[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S48354[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S48355[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S48356[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S48357[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S48358[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R950 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S48359[] = "Mozilla/5.0 (Linux; Android 4.0.3;  HTC One V Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S48360[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One V Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48361[] = "Mozilla/5.0 (Linux; Android 4.0.4; One V Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48362[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One V Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48363[] = "Mozilla/5.0 (Linux; Android 4.0.3;  HTC PH39100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S48364[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC PH39100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48365[] = "Mozilla/5.0 (SymbianOS/9.1; U; en-us) AppleWebKit/413 (KHTML, like Gecko) Safari/413 es65";
const char S48366[] = "Mozilla/5.0 (SymbianOS/9.1; U; ar) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S48367[] = "Mozilla/5.0 (SymbianOS/9.1; U; id) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S48368[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A854 Build/MILER_X1_01.22.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48369[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A854 Build/MILER_X1_00.26.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48370[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; A854 Build/MILER_X1_37.10.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48371[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A854 Build/MILER_X1_01.22.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48372[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A854 Build/MILER_X1_38.31.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48373[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A854 Build/MILER_X1_00.26.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48374[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; A854 Build/MILER_X1_37.10.0)";
const char S48375[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A854 Build/MILER_X1_00.26.1) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S48376[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; LG-V909 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S48377[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; LG-V909 Build/HMJ37)";
const char S48378[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en) Presto/2.10.289 Version/12.02";
const char S48379[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6.8; en; rv:2.0) Gecko/X Firefox/4.0 Opera 12.02";
const char S48380[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S48381[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S48382[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747M Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S48383[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S48384[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S48385[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S48386[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S48387[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S48388[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S48389[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S48390[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S48391[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; MT15a Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48392[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; SonyEricssonMT15a Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48393[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; pt-br; SonyEricssonMT15a Build/3.0.A.2.130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48394[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; sv-se; SonyEricssonMT15a Build/3.0.A.2.15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48395[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15a Build/4.0.A.2.286) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48396[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-cr; MT15a Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48397[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-cr; SonyEricssonMT15a Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48398[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; SonyEricssonMT15a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48399[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; MT15a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48400[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonMT15a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48401[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MT15a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48402[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; MT15a Build/4.0.2.A.0.42)";
const char S48403[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; MT15a Build/3.0.1.A.0.145)";
const char S48404[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; SonyEricssonMT15a Build/3.0.A.2.182) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48405[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0";
const char S48406[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0)";
const char S48407[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; ZTE; Windows Phone - Internet 7; SFR)";
const char S48408[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; PC40200)";
const char S48409[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Nokia; Champagne)";
const char S48410[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; ZTE; USCCN859; USC-US)";
const char S48411[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Nokia; Radar C110e)";
const char S48412[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; samsung; i917)";
const char S48413[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; ZTE; V965W)";
const char S48414[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Nokia; 7 Trophy T8686)";
const char S48415[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; ZTE; Mimosa)";
const char S48416[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Nokia; mwp6985)";
const char S48417[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; 909)";
const char S48418[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; HTCPO881; SPRINT)";
const char S48419[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; SCH-R860U)";
const char S48420[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E120S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48421[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SHV-E120K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48422[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SHV-E120L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48423[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E120L Build/JZO54K)";
const char S48424[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E120K/KKJMD2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48425[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E120S Build/JZO54K)";
const char S48426[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E120K Build/JZO54K)";
const char S48427[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SHV-E120K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48428[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E120K/KKJMH1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48429[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; T8788)";
const char S48430[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Mondrian)";
const char S48431[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Surround)";
const char S48432[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; 7 Surround)";
const char S48433[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; 7 Mondrian T8788)";
const char S48434[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S48435[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8005 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S48436[] = "Mozilla/5.0 (Linux; Android 4.0.4;  GT-N8000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S48437[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A770K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48438[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-A770K Build/IMM76D)";
const char S48439[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A770K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48440[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A770K Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48441[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A770K/2.23 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48442[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A770K/2.23 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48443[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Sensation_Z710e; en-in) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S48444[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Sensation_4G; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S48445[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC Z710e; zh-cn) AndroidWebKit/533.16 (KHTML, Like Gecko) Version/5.0 Safari/533.1";
const char S48446[] = "iTunes/10.6.1 (Windows; Microsoft Windows Vista Home Basic Edition Service Pack 1 (Build 6001)) AppleWebKit/534.54.16";
const char S48447[] = "iTunes/10.6.1 (Windows; Microsoft Windows Vista Home Basic Edition (Build 6000)) AppleWebKit/534.54.16";
const char S48448[] = "iTunes/10.6.3 (Windows; Microsoft Windows Vista Business Edition Service Pack 1 (Build 6001)) AppleWebKit/534.57.2";
const char S48449[] = "iTunes/10.7 (Windows; Microsoft Windows Vista Home Premium Edition Service Pack 2 (Build 6002)) AppleWebKit/536.26.9";
const char S48450[] = "iTunes/10.7 (Windows; Microsoft Windows Vista x64 x64 Home Premium Edition Service Pack 2 (Build 6002)) AppleWebKit/536.26.9";
const char S48451[] = "iTunes/10.7 (Windows; Microsoft Windows Vista Business Edition Service Pack 2 (Build 6002)) AppleWebKit/536.26.9";
const char S48452[] = "iTunes/10.7 (Windows; Microsoft Windows Vista Ultimate Edition Service Pack 2 (Build 6002)) AppleWebKit/536.26.9";
const char S48453[] = "iTunes/10.7 (Windows; Microsoft Windows Vista Home Basic Edition Service Pack 2 (Build 6002)) AppleWebKit/536.26.9";
const char S48454[] = "iTunes/10.7 (Windows; Microsoft Windows Vista Home Premium Edition Service Pack 1 (Build 6001)) AppleWebKit/536.26.9";
const char S48455[] = "iTunes/10.7 (Windows; Microsoft Windows Vista Home Premium Edition (Build 6000)) AppleWebKit/536.26.9";
const char S48456[] = "iTunes/10.7 (Windows; Microsoft Windows Vista x64 x64 Ultimate Edition Service Pack 2 (Build 6002)) AppleWebKit/536.26.9";
const char S48457[] = "iTunes/10.7 (Windows; Microsoft Windows Vista x64 x64 Business Edition Service Pack 2 (Build 6002)) AppleWebKit/536.26.9";
const char S48458[] = "iTunes/11.0.1 (Windows; Microsoft Windows Vista x64 x64 Home Premium Edition Service Pack 1 (Build 6001)) AppleWebKit/536.27.1";
const char S48459[] = "Opera/9.80 (Windows NT 6.0; U; en) Presto/2.9.168 Version/11.52";
const char S48460[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.9.168 Version/11.52";
const char S48461[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.9.168 Version/11.52";
const char S48462[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.9.168 Version/11.52";
const char S48463[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4157); ru) Presto/2.9.168 Version/11.52";
const char S48464[] = "Mozilla/5.0 (Windows NT 6.0; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.52";
const char S48465[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4876); ru) Presto/2.9.168 Version/11.52";
const char S48466[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03790); ru) Presto/2.9.168 Version/11.52";
const char S48467[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4664); ru) Presto/2.9.168 Version/11.52";
const char S48468[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03686); ru) Presto/2.9.168 Version/11.52";
const char S48469[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.4 (build 02647); ru) Presto/2.9.168 Version/11.52";
const char S48470[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.6 (build 03278); ru) Presto/2.9.168 Version/11.52";
const char S48471[] = "Opera/11.52 (Windows NT 6.0; U; en) Presto/2.9.168 Version/11.52";
const char S48472[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5 Comodo_Dragon/19.2.0.0";
const char S48473[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5 Comodo_Dragon/19.2.0.0";
const char S48474[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0";
const char S48475[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Flipboard/2.0";
const char S48476[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.1";
const char S48477[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Flipboard/2.0.1";
const char S48478[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Flipboard/2.0.1";
const char S48479[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.2";
const char S48480[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Flipboard/2.0.2";
const char S48481[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Flipboard/2.0.3";
const char S48482[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.3";
const char S48483[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Flipboard/2.0.4";
const char S48484[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.4";
const char S48485[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.5";
const char S48486[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Flipboard/2.0.5";
const char S48487[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Flipboard/2.0.6";
const char S48488[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.6";
const char S48489[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9A334 Safari/6533.18.5";
const char S48490[] = "Mozilla/5.0 (iPad; U; CPU OS 5 like Mac OS X; zh-cn Model:iPad2,1) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S48491[] = "Mozilla/5.0 (iPad; U; CPU OS 5_0_1 like Mac OS X; en_us) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5";
const char S48492[] = "Mozilla/5.0 (iPad; U; CPU OS 5_0 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5 DiigoBrowser";
const char S48493[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9A405 Safari/6533.18.5";
const char S48494[] = "Mozilla/5.0 (iPad; U; CPU OS 5_0 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S48495[] = "Mozilla/5.0 (iPad; U; CPU OS 5_0_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S48496[] = "Mozilla/5.0 (iPad; U; CPU OS 5_0_1 like Mac OS X; 6534.46) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5";
const char S48497[] = "Mozilla/5.0 (iPad; U; CPU OS 5_0 like Mac OS X; en_us) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5";
const char S48498[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9A5259f Safari/6533.18.5";
const char S48499[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/6533.18.5 DiigoBrowser";
const char S48500[] = "Mozilla/5.0 (iPad; U; CPU OS 5_0 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J3 Safari/6533.18.5";
const char S48501[] = "Mozilla/5.0(iPad; U; CPU OS 5_0 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F191 Safari/6533.18.5";
const char S48502[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0 Mobile/9A405 Safari/7534.48.3";
const char S48503[] = "Mozilla/5.0 (iPad; U; CPU OS 5_0 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9A5248d Safari/6533.18.5";
const char S48504[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; XT311 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48505[] = "MOT-XT311/Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; XT311 Build/V4.36M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48506[] = "MOT-XT311/Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; XT311 Build/V4.33B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48507[] = "MOT-XT311/Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; XT311 Build/V4.36J) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48508[] = "MOT-XT311/Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; XT311 Build/V4.40C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48509[] = "MOT-XT311/Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; XT311 Build/V4.53B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48510[] = "MOT-XT311/Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; XT311 Build/V4.36B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48511[] = "MOT-XT311/Mozilla/5.0 (Linux; U; Android 2.3.4; ro-ro; XT311 Build/V4.36H) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48512[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; XT311 Build/V4.480) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48513[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; XT311 Build/V4.530) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48514[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT311 Build/V4.26Q) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48515[] = "MOT-XT311/Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; XT311 Build/V4.54B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48516[] = "MOT-XT311/Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; XT311 Build/V4.33A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48517[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B144 Safari/7534.48.3";
const char S48518[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B142 Safari/7534.48.3";
const char S48519[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B141 Safari/7534.48.3";
const char S48520[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B143 Safari/7534.48.3";
const char S48521[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B145 Safari/7534.48.3";
const char S48522[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B146 Safari/7534.48.3";
const char S48523[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B329 Safari/7534.48.3";
const char S48524[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_4 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B350 Safari/7534.48.3";
const char S48525[] = "Mozilla/5.0 (Linux; U; en-US) AppleWebKit/528.5+ (KHTML, like Gecko, Safari/528.5+) Version/4.0 Kindle/3.0 (screen 600x800; rotate)";
const char S48526[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.1.140) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S48527[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S5282 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48528[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S5282 Build/JZO54K)";
const char S48529[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-S5282 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48530[] = "SonyEricssonW995/R1HA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.5";
const char S48531[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16 AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2";
const char S48532[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-M950 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48533[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SPH-M950 Build/JZO54K)";
const char S48534[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I415 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48535[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I415 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48536[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-I415 Build/IMM76D)";
const char S48537[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530M Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48538[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-R530M Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48539[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/21.0.1180.75 Safari/537.1";
const char S48540[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1; Maxthon (4.0.4.1000);";
const char S48541[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.22 (KHTML, like Gecko) Maxthon/4.0.6.3000 Chrome/25.0.1364.99 Safari/537.22";
const char S48542[] = "Nokia303/5.0 (14.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S48543[] = "Mozilla/5.0 (Linux; Android 4.1.1; A110 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S48544[] = "Mozilla/5.0 (Linux; Android 4.1.2; A110 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S48545[] = "Mozilla/5.0 (Linux; Android 4.1.1; Micromax A110 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48546[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-L710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48547[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-L710 Build/JZO54K)";
const char S48548[] = "Mozilla/5.0 (Linux; Android 4.1.2; VS870 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S48549[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaC5-00.2) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S48550[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaC5-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S48551[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac5-00) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba";
const char S48552[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac5-00.2) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba";
const char S48553[] = "Mozilla/5.0 (Linux; Android 4.0.3;  ASUS Transformer Pad TF300T Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S48554[] = "Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300T Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S48555[] = "Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300TG Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S48556[] = "Mozilla/5.0 (Linux; Android 4.0.4; Transformer Pad TF300T Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S48557[] = "Mozilla/5.0 (Linux; Android 4.0.4; Transformer Prime TF300T Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S48558[] = "Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300TL Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S48559[] = "Mozilla/5.0 (Linux; Android 4.0.3; ASUS Pad TF300T Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S48560[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SCH-I100 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48561[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SCH-I100 Build/ECLAIR)";
const char S48562[] = "Mozilla/5.0 (Linux; Android 4.1.2; B1-A71 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S48563[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; AZ210A-ORANGE Build/AZ210A_ICS_02.19I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48564[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; AZ210A-ORANGE Build/AZ210A_ICS_02.27I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48565[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; AZ210A Build/AZ210A_ICS_02.19I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48566[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; AZ210B-ORANGE Build/AZ210B_ICS_02.27I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48567[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; AZ210A Build/AZ210A_ICS_02.27I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48568[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; AZ210A Build/AZ210A_ICS_02.27I)";
const char S48569[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; AZ210B Build/AZ210B_ICS_02.27I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48570[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; AZ210A-ORANGE Build/AZ210A_ICS_02.34I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48571[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; AZ210A Build/AZ210A_ICS_02.34I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48572[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; AZ210B-ORANGE Build/AZ210B_ICS_02.34I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48573[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; AZ210B Build/AZ210B_ICS_02.34I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48574[] = "Mozilla/5.0 (Linux; Android 4.0.4;  SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S48575[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48576[] = "Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I717 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48577[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S48578[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB525 Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48579[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; MB525 Build/WhiteRabbitEdition-1.5.2; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48580[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; MB525 Build/WhiteRabbitEdition-1.7.1; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48581[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LG730 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48582[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE5-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S48583[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiae5-00) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba";
const char S48584[] = "NOKIAE5-00/UCWEB8.7.0.187/28/999";
const char S48585[] = "NOKIAE5-00UCBrowser/8.7.1.234/28/444/UCWEB";
const char S48586[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE5-00.2) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S48587[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I747 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48588[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L710 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48589[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I747 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48590[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-I747 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48591[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530U Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48592[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9305 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48593[] = "Mozilla/5.0 (Linux; Android 4.2.1; SAMSUNG-SGH-I747 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S48594[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I747 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S48595[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L710 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S48596[] = "Mozilla/5.0 (Linux; Android 4.2.1; SPH-L710 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48597[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530C Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48598[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I747 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S48599[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337M Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48600[] = "Mozilla/5.0 (Linux; Android 4.1.2; VS870 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48601[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LS970 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48602[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-LS970 Build/IMM76L)";
const char S48603[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S48604[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S48605[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48606[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S48607[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SCH-I400 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48608[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SCH-I400 Build/ECLAIR)";
const char S48609[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SCH-I400 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 MobileSafari/530.17";
const char S48610[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-; SCH-I400 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48611[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us;SCH-I400 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48612[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SCH-I400Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48613[] = "Mozilla/5.0 (Linux; U;Android 2.1-update1; en-us; SCH-I400 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48614[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Vodafone 875 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30 SVN/040HEG1";
const char S48615[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 520)";
const char S48616[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 520; Vodafone)";
const char S48617[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 520; ANZ914)";
const char S48618[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 520; Orange)";
const char S48619[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 520; Meteor)";
const char S48620[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Nokia 520)";
const char S48621[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 520; ANZ915)";
const char S48622[] = "Mozilla/5.0 (Linux; Android 4.0.4; Event Build/IML77) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48623[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21i2 Build/11.0.A.1.8) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48624[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21a2 Build/11.0.A.1.8) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48625[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21a Build/11.0.A.0.16) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48626[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21i Build/11.0.A.4.22) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48627[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S48628[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03R; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S48629[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S48630[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03L; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48631[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Nexus 7 Build/JRO03H; CyanogenMod-znsj.taobao.com jiji) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S48632[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48633[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:17.0) Gecko/X Thunderbird/17.0.8";
const char S48634[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S48635[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T859 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S48636[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P7510 Build/HTJ85B) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S48637[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HW-HUAWEI_C8812/C8812V100R001C92B913SP02; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S48638[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S48639[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48640[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S48641[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HUAWEI C8812 Build/HuaweiC8812)";
const char S48642[] = "Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build HuaweiC8812) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S48643[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8812E/C8812EV100R001C92B942; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S48644[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8812E/C8812EV100R001C92B939; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S48645[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8812E Build/HuaweiC8812E) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S48646[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8812E Build/HuaweiC8812E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48647[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8812E/C8812EV100R001C92B937; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S48648[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8812E/C8812EV100R001C92B943SP01; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S48649[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8812E Build/HuaweiC8812E) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S48650[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8812E/C8812EV100R001C92B949; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S48651[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8812E/C8812EV100R001C92B942SP02; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S48652[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI C8812E Build/HuaweiC8812E)";
const char S48653[] = "Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8812E Build HuaweiC8812E) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S48654[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8812E/C8812EV100R001C92B946; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S48655[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8812 Build/IMM76) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48656[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8812E/C8812EV100R001C92B938; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S48657[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8812E/C8812EV100R001C117B945SP01; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S48658[] = "Mozilla/5.0 (Linux; Android 4.1.2; C2105 Build/15.0.A.1.31) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48659[] = "Mozilla/5.0 (Linux; Android 4.1.1; C2105 Build/15.0.A.0.20) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48660[] = "SAMSUNG Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.30421/30.3697; U; en) Presto/2.8.119 Version/11.10";
const char S48661[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S48662[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S48663[] = "NetFront/3.5.1(BREW 3.1.5; U; en-us; SAMSUNG; NetFront/3.1.5/WAP) Sprint M350 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S48664[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; Milestone Build/SHOLS_U2_05.06.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48665[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Milestone Build/FRG22D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48666[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Milestone Build/R.U.R.1920) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48667[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Milestone Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48668[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; Milestone Build/SHOLS_U2_05.26.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48669[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Milestone Build/SHOLS_U2_05.26.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48670[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; Milestone Build/SHOLS_U2_05.26.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48671[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pl-pl; Milestone Build/SHOLS_U2_05.26.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48672[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; Milestone Build/SHOLS_U2_05.27.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48673[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; Milestone Build/SHOLS_U2_05.27.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48674[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; Milestone Build/SHOLS_U2_05.27.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48675[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; fr-ca; Milestone Build/ SHOLS_U2_05.26.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48676[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Milestone Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48677[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; Milestone Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48678[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; Milestone Build/FRG83G)";
const char S48679[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; Milestone Build/SHOLS_U2_05.20.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48680[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; Milestone Build/SHOLS_U2_05.26.4)";
const char S48681[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; Milestone Build/R.U.R.1920) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S48682[] = "SAMSUNG-SGH-T669/1.0 SHP/VPP/R5 Dolfin/1.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S48683[] = "SAMSUNG-SGH-T669Y/T669USKC5 SHP/VPP/R5 Dolfin/1.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S48684[] = "SAMSUNG-SGH-T669-MKB/T669UEKG4 SHP/VPP/R5 Dolfin/1.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S48685[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HTC-A6366/1.0 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48686[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HTC Liberty Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48687[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us;HTC-A6366/1.0 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S48688[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; HTC Liberty Build/ERE27)";
const char S48689[] = "Mozilla/5.0 (Windows NT 4.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S48690[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-gb; MZ601 Build/H.6.1-25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S48691[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-gb; MZ601 Build/H.6.1-38-1) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S48692[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-gb; MZ601 Build/H.6.1-17) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S48693[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; sv-se; MZ601 Build/H.6.1-38-13) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S48694[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-au; MZ601 Build/H.6.1-23) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S48695[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; de-de; MZ601 Build/H.6.1-38-2) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S48696[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-sg; MZ601 Build/H.6.1-40) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S48697[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; de-de; MZ601 Build/H.6.1-38-5) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S48698[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-au; MZ601 Build/H.6.1-38-9) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S48699[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-gb; MZ601 Build/H.6.1-38) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S48700[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; tr-tr; MZ601 Build/H.6.1-38-11) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S48701[] = "Mozilla/5.0 (Linux; U; Android 3.0; de-de; MZ601 Build/H.6.1-11) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S48702[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; MZ601 Build/H.6.1-37.1) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S48703[] = "Dalvik/1.5.0 (Linux; U; Android 3.0.1; MZ601 Build/H.6.1-38-9)";
const char S48704[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X+ Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48705[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC EVARE_UL Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48706[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X+ Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S48707[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S48708[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.24 (KHTML, like Gecko) Ubuntu/10.04 Chromium/11.0.696.68 Chrome/11.0.696.68 Safari/534.24";
const char S48709[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Ubuntu/10.04 Chromium/11.0.696.0 Chrome/11.0.696.0 Safari/534.24";
const char S48710[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.21 (KHTML, like Gecko) Ubuntu/10.04 Chromium/11.0.674.0 Chrome/11.0.674.0 Safari/534.21";
const char S48711[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.21 (KHTML, like Gecko) Ubuntu/10.04 Chromium/11.0.680.0 Chrome/11.0.680.0 Safari/534.21";
const char S48712[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989D Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S48713[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S48714[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S48715[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S48716[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-T989 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S48717[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S48718[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48719[] = "PantechP2020/JIUS05172010R; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.19039; U; en-US) Opera 9.50";
const char S48720[] = "PantechP2020/JIUS05172010R; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.19039; U; es) Opera 9.50";
const char S48721[] = "PantechP2020/JIUSX; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.19039; U; en-US) Opera 9.50";
const char S48722[] = "PantechP2020/JIUSX; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.19039; U; es) Opera 9.50";
const char S48723[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; ALCATEL ONE TOUCH 997D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48724[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sr-rs; ALCATEL ONE TOUCH 997 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48725[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ALCATEL ONE TOUCH 997D Build/IMM76D)";
const char S48726[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LG730 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48727[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T779 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S48728[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P5100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S48729[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P5100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S48730[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8) Presto/2.12.388 Version/12.14";
const char S48731[] = "Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Nokia5250/30.0.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413 3gpp-gba";
const char S48732[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S48733[] = "AppleCoreMedia/1.0.0.10A403 (iPad; U; CPU OS 6_0 like Mac OS X; en_us)";
const char S48734[] = "AppleCoreMedia/1.0.0.10A523 (iPad; U; CPU OS 6_0_1 like Mac OS X; en_us)";
const char S48735[] = "AppleCoreMedia/1.0.0.10A550 (iPad; U; CPU OS 6_0_2 like Mac OS X; en_us)";
const char S48736[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S48737[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T859 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S48738[] = "Mozilla/5.0 (Linux; Android 4.0.3;  HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S48739[] = "Mozilla/5.0 (Linux; Android 4.0.3;  HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S48740[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48741[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48742[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715a Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48743[] = "Mozilla/5.0 (Linux; Android 4.0.3; Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48744[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S48745[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Avant Browser)";
const char S48746[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; Avant Browser)";
const char S48747[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Avant Browser)";
const char S48748[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; Avant Browser)";
const char S48749[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Avant Browser; Avant Browser)";
const char S48750[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; Avant Browser; Avant Browser)";
const char S48751[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Avant Browser; SV1)";
const char S48752[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; SV1; Avant Browser)";
const char S48753[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Avant Browser; SV1)";
const char S48754[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; Avant Browser; Avant Browser)";
const char S48755[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Avant Browser; Avant Browser)";
const char S48756[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; SV1; Avant Browser)";
const char S48757[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-AS695 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48758[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-AS695 Build/GRK39F)";
const char S48759[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:23.0) Gecko/X Firefox/23.0 AlexaToolbar/alxf-2.18";
const char S48760[] = "SAMSUNG-GT-B7722/DDJH6 SHP/VPP/R5 Dolfin/1.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S48761[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P760 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48762[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P769 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48763[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P768 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48764[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P769 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S48765[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-B5330 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48766[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-B5330 Build/JZO54K)";
const char S48767[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-B5330L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48768[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-B5330 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48769[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; GT-B5330B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48770[] = "Mozilla/5.0 (Linux; U; Android2.2.1; en-us; T-Mobile myTouch 3G Slide Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48771[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3G Slide Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48772[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3G SlideBuild/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48773[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; T-Mobile myTouch 3G Slide Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48774[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3G Slide Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48775[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; T-Mobile myTouch 3G Slide Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48776[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; T-Mobile myTouch 3G Slide Build/FRG83D)";
const char S48777[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3G Slide Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S48778[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch3G Slide Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48779[] = "Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; T-Mobile myTouch 3G Slide Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48780[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3GSlide Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48781[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;T-Mobile myTouch 3G Slide Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48782[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-MobilemyTouch 3G Slide Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48783[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:25.0) Gecko/X Firefox/25.0";
const char S48784[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:25.0) Gecko/X Firefox/25.0 SeaMonkey/2.22a2";
const char S48785[] = "<![CDATA[Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:25.0) Gecko/X Firefox/25.0";
const char S48786[] = "Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0) Undrip Bot";
const char S48787[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48788[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48789[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48790[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I747M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48791[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-L710 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48792[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48793[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530U Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48794[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48795[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT26ii Build/6.2.B.0.200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48796[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT26ii Build/6.2.B.0.200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48797[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT26ii Build/6.2.B.0.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48798[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LT26ii Build/6.2.B.0.200)";
const char S48799[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT26ii Build/6.2.B.0.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48800[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT26ii Build/6.2.B.1.96) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48801[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT26ii Build/6.2.B.1.96) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48802[] = "iTunes/10.5.1 (Windows; Microsoft Windows XP Professional Service Pack 2 (Build 2600)) AppleWebKit/534.51.22";
const char S48803[] = "iTunes/10.6.1 (Windows; Microsoft Windows XP Home Edition Dodatek Service Pack 3 (Build 2600)) AppleWebKit/534.54.16";
const char S48804[] = "iTunes/10.7 (Windows; Microsoft Windows XP Professional Service Pack 3 (Build 2600)) AppleWebKit/536.26.9";
const char S48805[] = "iTunes/10.7 (Windows; Microsoft Windows XP Home Edition Service Pack 3 (Build 2600)) AppleWebKit/536.26.9";
const char S48806[] = "iTunes/10.6.3 (Windows; Microsoft Windows XP Service Pack 2 (Build 2600)) AppleWebKit/534.57.2";
const char S48807[] = "iTunes/10.6.1 (Windows; Microsoft Windows XP Home Edition Service Pack 2 (Build 2600)) AppleWebKit/534.54.16";
const char S48808[] = "Mozilla/5.0 (X11; Linux i686; rv:9.0) Gecko/X Firefox/9.0";
const char S48809[] = "Mozilla/5.0 (X11; Linux x86_64; rv:9.0.1) Gecko/X Firefox/9.0.1 Firefox/9.0.1";
const char S48810[] = "Mozilla/5.0 (X11; Linux i686; rv:9.0) Gecko/X Firefox/9.0 Iceweasel/9.0";
const char S48811[] = "Mozilla/5.0 (Linux; rv:9.0.1) Gecko/X Firefox/9.0.1";
const char S48812[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; P771A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48813[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; P771A Build/IMM76D)";
const char S48814[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SO-03D Build/6.0.A.5.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48815[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonSO-03D Build/6.0.A.5.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48816[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonSO-03D Build/6.0.A.5.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48817[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SO-03D Build/6.0.A.5.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48818[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonSO-03D Build/6.0.A.5.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48819[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SO-03D Build/6.0.A.5.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48820[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonSO-03D Build/6.0.A.5.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48821[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SO-03D Build/6.0.A.5.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48822[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; SO-03D Build/6.0.A.5.14)";
const char S48823[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT28at Build/6.1.C.1.105) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48824[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT28h Build/6.1.E.3.7) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48825[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT28i Build/6.1.E.2.68) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48826[] = "Flipboard/2.0.4 CFNetwork/609.1.4 Darwin/13.0.0";
const char S48827[] = "Mozilla/5.0 (Series40; NokiaX2-05/08.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S48828[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID1125 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48829[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; MID1125 Build/GRI40) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S48830[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID1125 Build/GRI40)";
const char S48831[] = "Logos/0.9.4 (Linux; U; Android 2.3.3; en_US; MID1125 Build/MID1125 X)";
const char S48832[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; MID1125A Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48833[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PMP5097CPRO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S48834[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; PMP5097CPRO Build/IML74K)";
const char S48835[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5 Comodo_Dragon/19.1.0.0";
const char S48836[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5 Comodo_Dragon/19.1.0.0";
const char S48837[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-05D Build/V16R24C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48838[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-05D Build/V18R26A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48839[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-05D Build/F0001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48840[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-05D Build/V17R25B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48841[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-05D Build/V19R28A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48842[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-05D Build/V20R29A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48843[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; F-05D Build/V20R29A)";
const char S48844[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-05D Build/V21R30C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48845[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-05D Build/V22R41A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48846[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.30 (KHTML, like Gecko) Ubuntu/10.04 Chromium/12.0.742.112 Chrome/12.0.742.112 Safari/534.30";
const char S48847[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.30 (KHTML, like Gecko) Ubuntu/10.04 Chromium/12.0.742.112 Chrome/12.0.742.112 Safari/534.30";
const char S48848[] = "Mozilla/5.0 (Linux; Android 4.0.3;  HUAWEI MediaPad Build/HuaweiMediaPad) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S48849[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI MediaPad Build/HuaweiMediaPad) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S48850[] = "Mozilla/5.0 (Linux; Android 4.0.3; SpringBoard Build/HuaweiMediaPad) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S48851[] = "Mozilla/5.0 (Linux; Android 4.0.3; MediaPad 7 Lite Build/HuaweiMediaPad) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S48852[] = "Mozilla/5.0 (Linux; Android 4.0.4; MediaPad 10 LINK Build/HuaweiMediaPad) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S48853[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT01 Build/HuaweiMediaPad) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S48854[] = "Mozilla/5.0 (Linux; Android 4.1.2; VS870 4G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S48855[] = "Mozilla/5.0 (Linux; Android 4.1.2; VS870 4G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S48856[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710e Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48857[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48858[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Z710e Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48859[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G with Beats Audio Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48860[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation 4G Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S48861[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9315; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.912 Mobile Safari/534.11+";
const char S48862[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9B176 Safari/7534.48.3";
const char S48863[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9B206 Safari/7534.48.3";
const char S48864[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 511 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/19.0.1084.60 Mobile/9B206 Safari/7534.48.3";
const char S48865[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 51 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/19.0.1084.60 Mobile/9B176 Safari/7534.48.3";
const char S48866[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; A100 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S48867[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; A100 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S48868[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; A100 Build/HTK55D)";
const char S48869[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; A100 Build/HTJ85B)";
const char S48870[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3100 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S48871[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P3100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S48872[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3100B Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S48873[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P3100B Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S48874[] = "Nexian NX-T905/MTK Release/10.01.2009 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1";
const char S48875[] = "NexianNX-G915/MTK Release/27.11.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S48876[] = "Nexian-NXG777/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S48877[] = "NexianNX-D981/MTK Release/23.3.2010 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S48878[] = "NexianNX-T901/MTK Release/19.04.2011 Browser/MAUI Profile/MIDP2.0Configuration/CLDC-1";
const char S48879[] = "NexianNX-A725/MTK Release/12.07.2011 Browser/MAUI Profile/MIDP2.0Configuration/CLDC-1";
const char S48880[] = "Nexian NX-T908/MTK Release/10.01.2009 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1";
const char S48881[] = "NexianNX-G269/MTK Release/01.01.2012 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1";
const char S48882[] = "NexianNX-G855/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1";
const char S48883[] = "Jigsaw/2.3.0 W3C_CSS_Validator_JFouffa/2.0 (See <http://validator.w3.org/services>)";
const char S48884[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; HTC Sensation Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48885[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; HTC Sensation Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48886[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Sensation Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48887[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC Sensation Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48888[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC Sensation Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48889[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48890[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LS970 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S48891[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Twitter for iPhone";
const char S48892[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A406 Twitter for iPhone";
const char S48893[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Twitter for iPhone";
const char S48894[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N5120 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S48895[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N5120 Build/JZO54K)";
const char S48896[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P505/V20f Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48897[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P505 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48898[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; LG-P505R Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48899[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P505 Build/GRJ22)";
const char S48900[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; LG-P505CH Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48901[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; LG-P505 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Versi?n/4.0 Mobile Safari/533.1";
const char S48902[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMP5880D Build/PMP5880D_v1.0.1_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S48903[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMP5880D Build/PMP5880D_v1.0.2_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S48904[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMP5880D Build/PMP5880D_v1.0.3_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S48905[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; PMP5880D Build/PMP5880D_v1.0.3_X)";
const char S48906[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5330/1.0; U; Bada/1.0; es) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 OPN-B";
const char S48907[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-M840 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48908[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SPH-M840 Build/JZO54K)";
const char S48909[] = "Nokia200/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S48910[] = "Nokia200/2.0 (11.64) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia200";
const char S48911[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; U30GT-H Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S48912[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; U30GT-H Build/JRO03H)";
const char S48913[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; U30GT-H Wanden.co.uk Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S48914[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; U30GT-H_www.wanden.co.uk Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S48915[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; U30GT-HS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S48916[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_HD2_T8585-Orange; Windows Phone 6.5)";
const char S48917[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_HD2_T8585; Windows Phone 6.5.3.5)";
const char S48918[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD2/3.14.172.3 (04666); Windows Phone 6.5)";
const char S48919[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD2/1.44.162.6 (70494); Windows Phone 6.5)";
const char S48920[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_HD2; T-Mobile_LEO; Windows Phone 6.5.3.5)";
const char S48921[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HD2_T8585; Windows Phone 6.5)";
const char S48922[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_HD2_T8585; Windows Phone 6.5)";
const char S48923[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; T-Mobile_LEO; Windows Phone 6.5)";
const char S48924[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5 HTC_HD2/1.0)";
const char S48925[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD2/1.72.172.0 (82124); Windows Phone 6.5)";
const char S48926[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD2/1.72.162.0 (82124); Windows Phone 6.5)";
const char S48927[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD2/1.72.161.3 (82124); Windows Phone 6.5)";
const char S48928[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD2/3.14.162.5 (04666); Windows Phone 6.5)";
const char S48929[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD2/3.14.164.4 (04666); Windows Phone 6.5)";
const char S48930[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD2/1.44.163.3 (70494); Windows Phone 6.5)";
const char S48931[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD2/1.49.164.2 (71528); Windows Phone 6.5)";
const char S48932[] = "HTC_HD2_T9193 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; V1.72.841.0 (82124); Windows Phone 6.5)";
const char S48933[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5 HTC_HD2/1.0; Windows Phone 6.5.3.5)";
const char S48934[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD2/3.14.161.3 (04666); Windows Phone 6.5)";
const char S48935[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD2/3.14.163.3 (04666); Windows Phone 6.5)";
const char S48936[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; T-Mobile_LEO; Windows Phone 6.5.3.5)";
const char S48937[] = "HTC_HD2_T9193 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; V1.49.841.1 (71528); Windows Phone 6.5)";
const char S48938[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_HD2; Windows Phone 6.5)";
const char S48939[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_HD2_T8585_SOD; Windows Phone 6.5)";
const char S48940[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 Build/1.6.0_243-MZ616) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S48941[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 Build/1.6.0_268.1-MZ616) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S48942[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 Build/1.6.0_258.1-MZ616) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S48943[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 Build/1.6.0_268.3-MZ616) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S48944[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 Build/1.6.0_236-MZ616) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S48945[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-au; XOOM 2 Build/1.6.0_268.4-MZ616) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S48946[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-au; XOOM 2 Build/1.6.0_281.4-MZ616) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S48947[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; iw-il; XOOM 2 Build/1.6.0_281.8-MZ616) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S48948[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; de-de; XOOM 2 Build/1.6.0_268.6-MZ616) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S48949[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; pt-br; MZ616 Build/1.6.0M_272.18_MZ616) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S48950[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-au; XOOM 2 Build/1.6.0_268.4-MZ616) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S48951[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; pt-br; MZ616 Build/1.6.0M_272.18_MZ616) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S48952[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; de-de; XOOM 2 Build/1.6.0_281.4-MZ616) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S48953[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; de-de; XOOM 2 Build/1.6.0_268.6-MZ616) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S48954[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.2; XOOM 2 Build/1.6.0_268.4-MZ616)";
const char S48955[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/BLUE1.0-DECOUPLED-X-1500) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48956[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/TESTFEATURE-SUPERSTAMINA-X-0856) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48957[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/7.0.A.0.735) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48958[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SonyLT29i Build/ICS-BLUE-QCT-MW-X-1944) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48959[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyLT29i Build/BLUE1.0-DECOUPLED-X-1530) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48960[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/9.0.A.0.226) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48961[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyLT29i Build/TESTFEATURE-SUPERSTAMINA-X-0839) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48962[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/7.0.A.1.155) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48963[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/BLUE1.0-DECOUPLED-X-1750) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48964[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/7.0.A.1.107) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48965[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/7.0.A.1.291) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48966[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/7.0.A.1.300) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48967[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; SonyLT29i Build/ICS-BLUE-QCT-MW-X-0654) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48968[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyLT29i Build/BLUE1.0-DECOUPLED-X-1600) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48969[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/VRC-X-1342-1050) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48970[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/BLUE1.0-DECOUPLED-X-1700) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48971[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/7.0.D.1.119) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48972[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/9.0.A.0.189) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48973[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyLT29i Build/7.0.A.2.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48974[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyLT29i Build/7.0.A.1.303) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48975[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; SonyLT29i Build/7.0.A.1.307) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48976[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; SonyLT29i Build/7.0.A.3.197) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48977[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; SonyLT29i Build/7.0.A.3.212) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48978[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT29i Build/7.0.A.3.197)";
const char S48979[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT29i Build/7.0.A.3.212) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48980[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT29i Build/7.0.A.3.197) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S48981[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; LT29i Build/7.0.A.1.307) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48982[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT29i Build/7.0.A.3.197) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48983[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT29i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48984[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/7.0.A.3.193) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48985[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT29i Build/7.0.A.1.303) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48986[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; DELL; Venue Pro)";
const char S48987[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LS970 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S48988[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC One X+ Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48989[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC One X+ Build/JDQ39)";
const char S48990[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC_OneXplus Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48991[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC_OneXplus/2.17.161.2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S48992[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-I9210T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48993[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9210T Build/GINGERBREAD)";
const char S48994[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; USCCADR6275US Carrier ID 45 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48995[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Milestone PLUS Build/3.4.2_80_V2_25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48996[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Milestone PLUS Build/3.4.2_80_V2_26) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48997[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; Milestone PLUS Build/3.4.2_80_V2_25)";
const char S48998[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Milestone PLUS Build/3.4.2_80_V2_11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S48999[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS840 4G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S49000[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS840PP Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S49001[] = "Mozilla/5.0 (X11; Linux x86_64) KHTML/4.8.4 (like Gecko) Konqueror/4.8";
const char S49002[] = "Mozilla/5.0 (X11; Linux i686) KHTML/4.8.5 (like Gecko) Konqueror/4.8";
const char S49003[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) konqueror/4.8.5 Safari/534.34";
const char S49004[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/533.3 (KHTML, like Gecko) konqueror/4.8.5 Safari/533.3";
const char S49005[] = "Mozilla/5.0 (X11) KHTML/4.8.5 (like Gecko) Konqueror/4.8";
const char S49006[] = "IUC(U;iOS 5.1.1;Zh-cn;320*480;)/UCWEB8.7.1.201/42/800";
const char S49007[] = "IUC(U;iOS 5.0.1;Zh-cn;320*480;)/UCWEB8.7.1.201/42/800";
const char S49008[] = "IUC(U;iOS 4.1;Zh-cn;320*480;)/UCWEB8.7.1.201/41/800";
const char S49009[] = "IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB8.7.1.201/41/800";
const char S49010[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; opera) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S49011[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; mstar) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S49012[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; maui) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S49013[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; j2me) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S49014[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; spice) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S49015[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; generic) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S49016[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; mstar) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S49017[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; opera) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S49018[] = "IUC(U;iOS 4.2.1;Zh-cn;320*480;)/UCWEB8.7.1.201/41/800";
const char S49019[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; mozilla) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S49020[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; mstar platform) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S49021[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sunmicro) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S49022[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sunmicro) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S49023[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; generic) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S49024[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; karbonn) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S49025[] = "UCWEB/2.0(Java; U; MIDP-2.0; id; maui) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S49026[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; bird) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S49027[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; jbed-fastbcc) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S49028[] = "UCWEB/2.0(Java; U; MIDP-2.0; id; maui) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S49029[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; huawei) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S49030[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; 11a wbg3505a_0302) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S49031[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; maui browser) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S49032[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; jbed) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S49033[] = "1PasswordThumbs/1 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (iMac12%2C2)";
const char S49034[] = "1PasswordThumbs/1 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookAir4%2C2)";
const char S49035[] = "1PasswordThumbs/1 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacPro3%2C1)";
const char S49036[] = "1PasswordThumbs/1 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBook5%2C1)";
const char S49037[] = "1PasswordThumbs/1 CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (MacBookPro10%2C1)";
const char S49038[] = "1PasswordThumbs/1 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (Macmini5%2C1)";
const char S49039[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB632 Build/5.5.1Q-83_ELW-FIT2-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49040[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB632 Build/5.5.1Q-110_ELW-TA-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49041[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; MB632 Build/5.5.1Q-110_ELWLA-TA-23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49042[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MB632 Build/5.5.1Q-110_ELW-TA-36) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49043[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; MB632 Build/5.5.1Q-110_ELWLA-TA-28) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49044[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; MB632 Build/5.5.1Q-110_ELWLA-TA-39) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49045[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; MB632 Build/5.5.1Q-110_ELW-TA-21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49046[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB632 Build/5.5.1Q-110-L28_ELWUS-TA-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49047[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; MB632 Build/5.5.1Q-110_ELW-TA-29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49048[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MB632 Build/5.5.1Q-110_ELW-TA-38) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49049[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MB632 Build/5.5.1Q-110_ELW-TA-41) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49050[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; MB632 Build/5.5.1Q-110_ELWLA-TA-38) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49051[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; MB632 Build/5.5.1Q-110_ELWLA-TA-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49052[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MB632 Build/5.5.1Q-110_ELWMEA-TA-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49053[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; MB632 Build/5.5.1Q-110_ELWLA-TA-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49054[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; MB632 Build/5.5.1Q-110_ELWLA-TA-29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49055[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-LS840 Build/ZVC.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49056[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-LS840 Build/ZVD.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49057[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; LG-LS840 Build/ZVD.GWK74)";
const char S49058[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S49059[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S49060[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S49061[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747M Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S49062[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S49063[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S49064[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S49065[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530C Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S49066[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S49067[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S49068[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P5110 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49069[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P5110 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49070[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P5110 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49071[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P5110 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49072[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-; GT-P5110 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49073[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P5110 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1.1/p5110";
const char S49074[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49075[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S49076[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire XC dual sim Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S49077[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X dual sim Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S49078[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S49079[] = "SAMSUNG-SGH-F480I/BOIG2 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S49080[] = "SAMSUNG-SGH-F480I-ORANGE/AFIJ1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S49081[] = "SAMSUNG-SGH-F480I-Bouygues/AGII1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S49082[] = "SAMSUNG-SGH-F480I-Vodafone/BUIJ2 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S49083[] = "SAMSUNG-SGH-F480i/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S49084[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-au; GT-I9000T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49085[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; GT-I9000T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49086[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I9000T Build/FROYO)";
const char S49087[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-I9000T Build/FROYO)";
const char S49088[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID7036 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49089[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S49090[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S49091[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC6435LVW Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S49092[] = "Nokia5233/UCWEB8.6.0.180/50/800";
const char S49093[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5233) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile";
const char S49094[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia 5233) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/352/UCWEB Mobile";
const char S49095[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5303 Build/12.0.A.1.257) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49096[] = "Mozilla/5.0 (Linux; Android 4.1.2; MOTOROLA RAZR Build/9.8.2O-72_VZW-8) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S49097[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S49098[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; EK-GC100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49099[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; EK-GC100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49100[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-EK-GC100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49101[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; EK-GC100 Build/JRO03C)";
const char S49102[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; EK-GC100 Build/JZO54K)";
const char S49103[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-EK-GC100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49104[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P5110 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S49105[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P5110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S49106[] = "DoCoMo/2.0 F704i(c100;TB;W23H12)";
const char S49107[] = "DoCoMo/2.0 L704i(c100;TB;W24H14)";
const char S49108[] = "DoCoMo/2.0 L705iX(c100;TB;W24H14)";
const char S49109[] = "DoCoMo/2.0 L852i(c100;TB;W24H14)";
const char S49110[] = "DoCoMo/2.0 L03C(c500;TB;W24H16)";
const char S49111[] = "DoCoMo/2.0 N03C(c500;TB;W24H16)";
const char S49112[] = "DoCoMo/2.0 P03C(c500;TB;W24H16)";
const char S49113[] = "DoCoMo/2.0 SH04C(c500;TB;W24H16)";
const char S49114[] = "DoCoMo/2.0 P705i(c100;TB;W24H15)";
const char S49115[] = "DoCoMo/2.0 F04A(c100;TB;W24H17)";
const char S49116[] = "DoCoMo/2.0 L03B(c100;TB;W24H16)";
const char S49117[] = "DoCoMo/2.0 SH903i(c100;TB;W24H16)";
const char S49118[] = "DoCoMo/2.0 N02B(c500;TB;W24H16)";
const char S49119[] = "DoCoMo/2.0 SH04B(c500;TB;W24H14)";
const char S49120[] = "DoCoMo/2.0 SO906i(c100;TB;W24H18)";
const char S49121[] = "DoCoMo/2.0 P704i(c100;TB;W24H12)";
const char S49122[] = "DoCoMo/2.0 D705i(c100;TB;W24H16)";
const char S49123[] = "DoCoMo/2.0 D903i(c100;TB;W28H20) (compatible; Hatena-Mobile-Gateway/1.2; +http://mgw.hatena.ne.jp/help)";
const char S49124[] = "DoCoMo/2.0 F705i(c100;TB;W24H17)";
const char S49125[] = "DoCoMo/2.0 P706imyu(c100;TB;W24H15)";
const char S49126[] = "DoCoMo/2.0 P06A(c100;TB;W24H15)";
const char S49127[] = "DoCoMo/2.0 SH05B(c500;TB;W24H16)";
const char S49128[] = "DoCoMo/2.0 L06A(c100;TB;W24H15)";
const char S49129[] = "DoCoMo/2.0 SO706i(c100;TB;W24H16)";
const char S49130[] = "DoCoMo/2.0 N01A(c100;TB;W24H16)";
const char S49131[] = "DoCoMo/2.0 F904i(c100;TB;W24H17)";
const char S49132[] = "DoCoMo/2.0 SH904i(c100;TB;W24H16)";
const char S49133[] = "DoCoMo/2.0 N05A(c100;TB;W24H16)";
const char S49134[] = "DoCoMo/2.0 SH11C(c500;TB;W24H16)";
const char S49135[] = "DoCoMo/2.0 SO705i(c100;TB;W24H16)";
const char S49136[] = "DoCoMo/2.0 SH07A3(c500;TB;W24H14)";
const char S49137[] = "DoCoMo/2.0 SH10C(c500;TB;W24H16)";
const char S49138[] = "DoCoMo/2.0 F10C(c500;TB;W24H16)";
const char S49139[] = "DoCoMo/2.0 F11C(c500;TB;W24H16)";
const char S49140[] = "DoCoMo/2.0 P06C(c500;TB;W24H16)";
const char S49141[] = "DoCoMo/2.0 N05C(c500;TB;W30H20)";
const char S49142[] = "DoCoMo/2.0 P904i(c100;TB;W24H15)";
const char S49143[] = "DoCoMo/2.0 SH02B(c500;TB;W24H16)";
const char S49144[] = "DoCoMo/2.0 N03B(c500;TB;W24H16)";
const char S49145[] = "DoCoMo/2.0 SO905i(c100;TB;W24H18)";
const char S49146[] = "DoCoMo/2.0 N09A3(c500;TB;W24H16)";
const char S49147[] = "DoCoMo/2.0 N04A(c100;TB;W30H20)";
const char S49148[] = "DoCoMo/2.0 F09C(c500;TB;W24H16)";
const char S49149[] = "DoCoMo/2.0 F906i(c100;TB;W24H17)";
const char S49150[] = "DoCoMo/2.0 P09A3(c500;TB;W24H15)";
const char S49151[] = "DoCoMo/2.0 P05C(c500;TB;W24H16)";
const char S49152[] = "DoCoMo/2.0 N904i(c100;TB;W24H16)";
const char S49153[] = "DoCoMo/2.0 F706i(c100;TB;W24H17)";
const char S49154[] = "DoCoMo/2.0 CA01C(c500;TB;W30H20)";
const char S49155[] = "DoCoMo/2.0 P04C(c500;TB;W24H16)";
const char S49156[] = "DoCoMo/2.0 F02D(c500;TB;W24H16)";
const char S49157[] = "DoCoMo/2.0 N03D(c500;TB;W24H16)";
const char S49158[] = "DoCoMo/2.0 P705imyu(c100;TB;W24H15)";
const char S49159[] = "DoCoMo/2.0 F04D(c500;TB;W24H16)";
const char S49160[] = "DoCoMo/2.0 SH706i(c100;TB;W24H16)";
const char S49161[] = "DoCoMo/2.0 SH05C(c500;TB;W30H15)";
const char S49162[] = "DoCoMo/2.0 P03D(c500;TB;W24H16)";
const char S49163[] = "DoCoMo/2.0 N02D(c500;TB;W24H16)";
const char S49164[] = "DoCoMo/2.0 SH03D(c500;TB;W24H16)";
const char S49165[] = "DoCoMo/2.0 SH703i(c100;TB;W24H12)";
const char S49166[] = "DoCoMo/2.0 SH705i(c100;TB;W24H16)";
const char S49167[] = "DoCoMo/2.0 N703iD(c100;TB;W24H12)";
const char S49168[] = "DoCoMo/2.0 SH704i(c100;TB;W24H16)";
const char S49169[] = "DoCoMo/2.0 L04A(c100;TB;W24H15)";
const char S49170[] = "DoCoMo/2.0 F07A(c100;TB;W20H08)";
const char S49171[] = "DoCoMo/2.0 SH05D(c500;TB;W24H16)";
const char S49172[] = "DoCoMo/2.0 F06D(c500;TB;W24H16)";
const char S49173[] = "DoCoMo/2.0 P01E(c500;TB;W24H16)";
const char S49174[] = "DoCoMo/2.0 F01E(c500;TB;W24H16)";
const char S49175[] = "DoCoMo/2.0 L01B(c500;TB;W24H16)";
const char S49176[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows 2000)";
const char S49177[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows 2000)";
const char S49178[] = "Mozilla/5.0 (Linux; Android 4.0.4; BNTV400 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S49179[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L720 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S49180[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; LG-E617G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49181[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-E617G Build/IML74K)";
const char S49182[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:11.0) Gecko/X Firefox/11.0";
const char S49183[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:11.0) Gecko/X Firefox/11.0";
const char S49184[] = "Mozilla/5.0 (X11; CrOS x86_64 2268.0.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.1 Safari/536.11";
const char S49185[] = "Mozilla/5.0 (X11; CrOS i686 2237.0.2012) AppleWebKit/536.10 (KHTML, like Gecko) Chrome/20.0.1125.0 Safari/536.10";
const char S49186[] = "Mozilla/5.0 (X11; CrOS i686 2268.16.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.11 Safari/536.11";
const char S49187[] = "Mozilla/5.0 (X11; CrOS i686 2268.23.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.15 Safari/536.11";
const char S49188[] = "Mozilla/5.0 (X11; CrOS i686 2268.42.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.22 Safari/536.11";
const char S49189[] = "Mozilla/5.0 (X11; CrOS x86_64 2268.42.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.22 Safari/536.11";
const char S49190[] = "Mozilla/5.0 (X11; CrOS i686 2268.70.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.34 Safari/536.11";
const char S49191[] = "Mozilla/5.0 (X11; CrOS i686 2268.85.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.37 Safari/536.11";
const char S49192[] = "Mozilla/5.0 (X11; CrOS x86_64 2268.85.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.37 Safari/536.11";
const char S49193[] = "Mozilla/5.0 (X11; CrOS i686 2268.105.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.54 Safari/536.11";
const char S49194[] = "Mozilla/5.0 (X11; CrOS x86_64 2268.105.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.54 Safari/536.11";
const char S49195[] = "Mozilla/5.0 (X11; CrOS i686 2268.111.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S49196[] = "Mozilla/5.0 (X11; CrOS x86_64 2268.111.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S49197[] = "Mozilla/5.0 (X11; CrOS i686 2268.124.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.59 Safari/536.11";
const char S49198[] = "Mozilla/5.0 (X11; CrOS x86_64 2268.124.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.59 Safari/536.11";
const char S49199[] = "Mozilla/5.0 (X11; CrOS i686 2141.0.2012) AppleWebKit/536.8 (KHTML, like Gecko) Chrome/20.0.1102.0 Safari/536.8";
const char S49200[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC HD2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49201[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC HD2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49202[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC HD2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49203[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HD2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49204[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HD2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49205[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC HD2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49206[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC HD2 Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S49207[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC HD2 Build/MIUI)";
const char S49208[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; HD2 Build/MIUI)";
const char S49209[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC HD2 Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S49210[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC HD2 Build/MIUI)";
const char S49211[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC HD2 Build/MIUIAndroid 2.3.4a) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49212[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC HD2 Build/MIUI)";
const char S49213[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; HTC HD2 Build/MIUIclk) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49214[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC HD2 Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S49215[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC HD2 Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S49216[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A13-MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49217[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A13-MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49218[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A13-MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49219[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; A13-MID Build/IML74K)";
const char S49220[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; A13-MID Build/IMM76D)";
const char S49221[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A13MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49222[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; mid-A13 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49223[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A13MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49224[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; A13MID Build/IMM76D)";
const char S49225[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; mid-A13 Build/IMM76D)";
const char S49226[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A13_MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49227[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; A13_MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49228[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID-A13 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49229[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8262 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49230[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8262 Build/JZO54K)";
const char S49231[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-I8262 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49232[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGPT12 Build/TJS0166) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S49233[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGPT13 Build/TJS0166) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S49234[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S49235[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T859 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S49236[] = "Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S49237[] = "Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S49238[] = "Mozilla/5.0 (Linux; Android 4.1.2; EVO Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S49239[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A523 Safari/8536.25";
const char S49240[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A403 Safari/8536.25";
const char S49241[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A550 Safari/8536.25";
const char S49242[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A407 Safari/8536.25";
const char S49243[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A8500 Safari/8536.25";
const char S49244[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A8426 Safari/8536.25";
const char S49245[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A406 Safari/8536.25";
const char S49246[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S49247[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36";
const char S49248[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_SensationXE_Beats_Z715e; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S49249[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_SensationXE_Beats_Z715a; en-au) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S49250[] = "Mozilla/5.0 (Windows NT 6.0; rv:11.0) Gecko/X Firefox/11.0 CometBird/11.0";
const char S49251[] = "Mozilla/5.0 (Windows NT 6.0; rv:11.0) Gecko/X Firefox/11.0 CometBrowser/11.0";
const char S49252[] = "Mozilla/5.0 (Linux; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S49253[] = "Mozilla/5.0 (Linux; Android 4.0.4; ME865 Build/6.7.2_GC-220-EDS-3) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S49254[] = "Mozilla/5.0 (Linux; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S49255[] = "Mozilla/5.0 (Unknown; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) CasperJS/1.0.0-RC1+Phantomjs/1.6.0 Safari/534.34";
const char S49256[] = "Mozilla/5.0 (Unknown; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) CasperJS/1.0.0-RC1+Phantomjs/1.6.0 Safari/534.34";
const char S49257[] = "SAMSUNG-GT-S3770/S3770AHKG3 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S49258[] = "SAMSUNG-GT-S3770K/S3770KDDKK2 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S49259[] = "SAMSUNG-GT-S3770L/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S49260[] = "SAMSUNG-GT-S3770M/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S49261[] = "SAMSUNG-GT-S3770Y/S3770YXXLC1 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S49262[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; SGH-T589W Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49263[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-T589W Build/GINGERBREAD)";
const char S49264[] = "Mozilla/5.0 (Nintendo WiiU) AppleWebKit/534.52 (KHTML, like Gecko) NX/2.1.0.8.23 NintendoBrowser/1.1.0.7579.EU";
const char S49265[] = "Mozilla/5.0 (Nintendo WiiU) AppleWebKit/534.52 (KHTML, like Gecko) NX/2.1.0.8.23 NintendoBrowser/1.1.0.7579.US";
const char S49266[] = "Mozilla/5.0 (Nintendo WiiU) AppleWebKit/534.52 (KHTML, like Gecko) NX/2.1.0.8.23 NintendoBrowser/1.1.0.7579.JP";
const char S49267[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-; LS670 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49268[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LS670 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49269[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LS670 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49270[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LS670 Build/ZVC) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49271[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; LS670 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49272[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LS670 Build/FRG83) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S49273[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LS670 Build/FRG83G)";
const char S49274[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; LS670 Build/FRG83)";
const char S49275[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LS670 Build/FRG83G) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S49276[] = "Nokia202/2.0 (20.28) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S49277[] = "Nokia203/2.0 (20.36) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S49278[] = "Nokia202/2.0 (20.28) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia202";
const char S49279[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Liquid MT Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49280[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Liquid MT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49281[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Liquid MT Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49282[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Liquid MT Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49283[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; Liquid MT Build/GRK39F)";
const char S49284[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Liquid MT Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0  Safari/533.1";
const char S49285[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.0.200) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S49286[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT890 Build/9.8.2I-50_SMI-26) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S49287[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT890 Build/9.8.2I-50_SMI-29) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22";
const char S49288[] = "Opera/9.80 (Macintosh; Intel Mac OS X; U; en) Presto/2.6.30 Version/10.61";
const char S49289[] = "Opera/9.80 (Macintosh; Intel Mac OS X; U; en) Presto/2.6.30 Version/10.62";
const char S49290[] = "Opera/9.80 (Macintosh; PPC Mac OS X; U; en) Presto/2.6.30 Version/10.63";
const char S49291[] = "Opera/9.80 (Macintosh; Intel Mac OS X; U; en) Presto/2.6.30 Version/10.60";
const char S49292[] = "Apple Mac OS X v10.6.2 CoreMedia v1.0.0.10C540";
const char S49293[] = "Apple Mac OS X v10.6.4 CoreMedia v1.0.0.10F569";
const char S49294[] = "Apple Mac OS X v10.6.5 CoreMedia v1.0.0.10H574";
const char S49295[] = "Apple Mac OS X v10.6.4 CoreMedia v1.0.0.10F2108";
const char S49296[] = "Apple Mac OS X v10.6.6 CoreMedia v1.0.0.10J567";
const char S49297[] = "Apple Mac OS X v10.6.7 CoreMedia v1.0.0.10J869";
const char S49298[] = "Apple Mac OS X v10.6.7 CoreMedia v1.0.0.10J3250";
const char S49299[] = "Apple Mac OS X v10.6.6 CoreMedia v1.0.0.10J3210";
const char S49300[] = "Apple Mac OS X v10.6.3 CoreMedia v1.0.0.10D2125";
const char S49301[] = "Apple Mac OS X v10.6.7 CoreMedia v1.0.0.10J4139";
const char S49302[] = "Apple Mac OS X v10.6.8 CoreMedia v1.0.0.10K540";
const char S49303[] = "Apple Mac OS X v10.6.8 CoreMedia v1.0.0.10K549";
const char S49304[] = "Apple Mac OS X v10.6 CoreMedia v1.0.0.10A432";
const char S49305[] = "Apple Mac OS X v10.6.3 CoreMedia v1.0.0.10D575";
const char S49306[] = "Apple Mac OS X v10.6.4 CoreMedia v1.0.0.10F3061";
const char S49307[] = "Mozilla/5.0 (PLAYSTATION 3 4.40) AppleWebKit/531.22.8 (KHTML, like Gecko)";
const char S49308[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49309[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; A10 Build/unknown) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49310[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; A10 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49311[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49312[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; A10 Build/IMM76D)";
const char S49313[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-eg; A10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49314[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; Treq A10 Pocket Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49315[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; A10 Build/R13.4-rc1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49316[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5) Presto/2.12.388 Version/12.11";
const char S49317[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I5503 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49318[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-nz; GT-I5503T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49319[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; GT-I5503T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49320[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I5503T Build/FROYO)";
const char S49321[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I5503 Build/FROYO)";
const char S49322[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-I5503T Build/FROYO)";
const char S49323[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S49324[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I467 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49325[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I467M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49326[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-I467 Build/JZO54K)";
const char S49327[] = "NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S49328[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; AS740 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49329[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; US740 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49330[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; US740 Build/FRG83G)";
const char S49331[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; AS740 Build/FRG83G)";
const char S49332[] = "Mozilla/5.0 (Linux; Android 4.1.1; Transformer Prime TF201 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S49333[] = "Mozilla/5.0 (Linux; Android 4.1.2; Transformer Prime TF201 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S49334[] = "Mozilla/5.0 (Linux; Android 4.1.1; TF201 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S49335[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1 solfo-linkchecker/1.0 (http://solfo.com/linkbot.html)";
const char S49336[] = "GoogleEarth/7.0.1.8244(Windows;Microsoft Windows (5.1.2600.3);en;kml:2.2;client:Free;type:default)";
const char S49337[] = "GoogleEarth/7.0.2.8415(Windows;Microsoft Windows (5.1.2600.3);en;kml:2.2;client:Free;type:default)";
const char S49338[] = "GoogleEarth/7.0.2.8415(Windows;Microsoft Windows (5.1.2600.3);de;kml:2.2;client:Free;type:default)";
const char S49339[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.3);en;kml:2.2;client:Free;type:default)";
const char S49340[] = "Mozilla/5.0 (Windows; N; Windows NT 5.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/7.0.3.8542 Safari/532.4";
const char S49341[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.2);en;kml:2.2;client:Free;type:default)";
const char S49342[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.3);de;kml:2.2;client:Free;type:default)";
const char S49343[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.3);it;kml:2.2;client:Free;type:default)";
const char S49344[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.3);ru;kml:2.2;client:Free;type:default)";
const char S49345[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.3);fr;kml:2.2;client:Free;type:default)";
const char S49346[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.3);zh-Hans;kml:2.2;client:Free;type:default)";
const char S49347[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.3);nl;kml:2.2;client:Free;type:default)";
const char S49348[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.3);es;kml:2.2;client:Free;type:default)";
const char S49349[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.3);es-419;kml:2.2;client:Free;type:default)";
const char S49350[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.3);pt;kml:2.2;client:Free;type:default)";
const char S49351[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.2);de;kml:2.2;client:Free;type:default)";
const char S49352[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; Garmin-Asus A50 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17; A50-V5.0.73-user-20101105";
const char S49353[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Garmin-Asus A50 Build/ERE27)";
const char S49354[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; Garmin-Asus A50 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17; A50-V5.0.73-user-X";
const char S49355[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P5110 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S49356[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P5110 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S49357[] = "Mozilla/5.0 (X11; Linux i686; rv:20.0) Gecko/X Firefox/20.0 SeaMonkey/2.17.1";
const char S49358[] = "Mozilla/5.0 (X11; Linux i686; rv:20.0) Gecko/X SeaMonkey/2.17.1";
const char S49359[] = "Mozilla/5.0 (X11; Linux x86_64; rv:20.0) Gecko/X SeaMonkey/2.17.1";
const char S49360[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ALCATEL_one_touch_909S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49361[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ALCATEL_one_touch_909B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49362[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; ALCATEL_one_touch_909A Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49363[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; USCC_ALCATEL_one_touch_909B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49364[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-ca; SAMSUNG-SGH-I896 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49365[] = "Mozilla/5.0 (Linux; U; Android 2.2; ru-; SAMSUNG-SGH-I896 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49366[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; SAMSUNG-SGH-I896 Build/FROYO)";
const char S49367[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Safari/535.19 Silk-Accelerated=false";
const char S49368[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:20.0) Gecko/X Firefox/20.0 AlexaToolbar/alxf-2.18";
const char S49369[] = "SAMSUNG-SGH-A847/A847UCJK2; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.17574; U; en-US) Opera 9.50";
const char S49370[] = "Nokia200/2.0 (11.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia200) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S49371[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3110 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S49372[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3110 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S49373[] = "Mozilla/5.0 (Linux; Android 4.2.1; ME301T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S49374[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7";
const char S49375[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7";
const char S49376[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8L1 Safari/7534.48.3";
const char S49377[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8F191 Safari/7534.48.3";
const char S49378[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8J2 Safari/7534.48.3";
const char S49379[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8G4 Safari/7534.48.3";
const char S49380[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8F190 Safari/7534.48.3";
const char S49381[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8J3 Safari/7534.48.3";
const char S49382[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8H7 Safari/7534.48.3";
const char S49383[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8K2 Safari/7534.48.3";
const char S49384[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8H8 Safari/7534.48.3";
const char S49385[] = "SAMSUNG-GT-C3510/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S49386[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-VS410PP Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49387[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-VS410PP Build/GRK39F)";
const char S49388[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9082 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49389[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9082 Build/JDQ39)";
const char S49390[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.6 Safari/535.19 Silk-Accelerated=true";
const char S49391[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S49392[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; MZ604 Build/H.6.2-21-demo) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S49393[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; ja-jp; MZ604 Build/H.6.2-20) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S49394[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-ca; MZ604 Build/H.6.2-21) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S49395[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; MZ604 Build/H.6.2-24) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S49396[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; MZ604 Build/H.6.2-26) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S49397[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; MZ604 Build/H.6.2-28) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S49398[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; de-de; MZ604 Build/H.6.2-35.1) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S49399[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-gb; MZ604 Build/H.6.2-34) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S49400[] = "Dalvik/1.5.0 (Linux; U; Android 3.0.1; MZ604 Build/H.6.2-24)";
const char S49401[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.2/552,2.0.2.552,2013-05-22 23:05, +0100, gb";
const char S49402[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T999 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us";
const char S49403[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I535 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us";
const char S49404[] = "Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49405[] = "Mozilla/5.0 (Linux; Android 4.1.2; EVO Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49406[] = "iTunes/10.4.1 (Macintosh; Intel Mac OS X 10.7.1) AppleWebKit/534.48.3";
const char S49407[] = "iTunes/10.3.1 (Macintosh; Intel Mac OS X 10.7) AppleWebKit/534.48.3";
const char S49408[] = "iTunes/10.6.1 (Macintosh; Intel Mac OS X 10.7.3) AppleWebKit/534.53.11";
const char S49409[] = "iTunes/10.4.1 (Macintosh; Intel Mac OS X 10.7.2) AppleWebKit/534.51.22";
const char S49410[] = "iTunes/10.6.3 (Macintosh; Intel Mac OS X 10.7.4) AppleWebKit/534.56.5";
const char S49411[] = "iTunes/10.7 (Macintosh; Intel Mac OS X 10.7.5) AppleWebKit/534.57.7";
const char S49412[] = "iTunes/11.0 (Macintosh; OS X 10.7.4) AppleWebKit/534.56.5";
const char S49413[] = "iTunes/11.0.1 (Macintosh; OS X 10.7.5) AppleWebKit/534.57.7";
const char S49414[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.8.1.20) Gecko/X BRI/1 Firefox/2.0.0.20";
const char S49415[] = "Mozilla/5.0 (Windows NT 6.1; U; en; rv:1.8.1) Gecko/X Firefox/2.0.0";
const char S49416[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.8.1.3) Gecko/X BonEcho/2.0.0.3";
const char S49417[] = "Mozilla/5.0 (Windows NT 6.1; rv:17.0) Gecko/X Firefox/20.6.14";
const char S49418[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/6533.18.5 iLunascape/3201";
const char S49419[] = "Mozilla/5.0 (iPad; U; CPU OS 6_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S49420[] = "Mozilla/5.0 (iPad; U; CPU OS 6_1_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S49421[] = "Mozilla/5.0 (iPad; U; CPU OS 6_1 like Mac OS X; ko_kr) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5";
const char S49422[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/6533.18.5 iLunascape/3203";
const char S49423[] = "Mozilla/5.0 (iPad; U; CPU OS 6_1_2 like Mac OS X; ko_kr) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5";
const char S49424[] = "User-Agent: Mozilla/5.0 (iPad; U; CPU OS 6_1_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5";
const char S49425[] = "Mozilla/5.0 (iPad; U; CPU OS 6_1_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S49426[] = "Mozilla/5.0 (iPad; U; CPU OS 6_1_3 like Mac OS X; ko_kr) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5";
const char S49427[] = "Mozilla/5.0 (iPad; U; CPU OS 6_1_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5";
const char S49428[] = "Mozilla/5.0 (iPad; U; CPU OS 6_1 like Mac OS X; zh-cn Model:iPad2,1) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S49429[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/6533.18.5 iLunascape/3212";
const char S49430[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 1.1.4322; MSIECrawler)";
const char S49431[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; MSIECrawler)";
const char S49432[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; MSIECrawler)";
const char S49433[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; NN4.1.0.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; MSIECrawler)";
const char S49434[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; MSIECrawler)";
const char S49435[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; InfoPath.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; MSIECrawler)";
const char S49436[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSIECrawler)";
const char S49437[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; MSIECrawler)";
const char S49438[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; InfoPath.2; MSIECrawler)";
const char S49439[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; PeoplePal 3.0; MSIECrawler)";
const char S49440[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; MSIECrawler)";
const char S49441[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; GTB7.1; MSIECrawler)";
const char S49442[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSIECrawler)";
const char S49443[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; MSIECrawler)";
const char S49444[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; InfoPath.2; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSIECrawler)";
const char S49445[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; InfoPath.2; MSIECrawler)";
const char S49446[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; InfoPath.1; MSIECrawler)";
const char S49447[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; InfoPath.2; .NET CLR 2.0.50727; MSIECrawler)";
const char S49448[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 1.0.3705; MSIECrawler)";
const char S49449[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.0.3705; MSIECrawler)";
const char S49450[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; {73D5DF9F-E500-BB8D-C9C1-7F04A4EB0838}; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSIECrawler)";
const char S49451[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; MSIECrawler)";
const char S49452[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; MSIECrawler)";
const char S49453[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; GTB7.2; MSIECrawler)";
const char S49454[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 2.0.50727; MSIECrawler)";
const char S49455[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSIECrawler)";
const char S49456[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; InfoPath.2; MSIECrawler)";
const char S49457[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; GTB6; Ant.com Toolbar 1.6; MSIECrawler)";
const char S49458[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; FunWebProducts; MSIECrawler)";
const char S49459[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; MSIECrawler)";
const char S49460[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; GTB6.6; MSIECrawler)";
const char S49461[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; FunWebProducts; MSIECrawler)";
const char S49462[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Embedded Web Browser from: http://bsalsa.com/; MSIECrawler)";
const char S49463[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Crayon Crawler; snprtz|TX; (R1 1.5))";
const char S49464[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; GTB7.3; MSIECrawler)";
const char S49465[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; FunWebProducts; ezPeer+ v1.0 Beta (0.4.1.98); ezPeer+ v1.0 (0.5.0.00); .NET CLR 1.1.4322; MSIECrawler)";
const char S49466[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; MSIECrawler)";
const char S49467[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; BTRSX; .NET CLR 2.0.50727; MSIECrawler)";
const char S49468[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322; MSIECrawler)";
const char S49469[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSIECrawler)";
const char S49470[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2; MSIECrawler)";
const char S49471[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QQDownload 708; MSIECrawler)";
const char S49472[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; qdesk 2.3.1186.202; MSIECrawler)";
const char S49473[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; BTRSX; SIMBAR={4CDE8DFA-3984-11E2-8CE6-XDA9311}; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; MSIECrawler)";
const char S49474[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; BTRSX; SIMBAR={4CDE8DFA-3984-11E2-8CE6-XDA9311}; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; MSIECrawler)";
const char S49475[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; SIMBAR={4CDE8DFA-3984-11E2-8CE6-XDA9311}; BTRSX; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; MSIECrawler)";
const char S49476[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; SIMBAR={4CDE8DFA-3984-11E2-8CE6-XDA9311}; BTRS99959; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; MSIECrawler)";
const char S49477[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; SIMBAR={4CDE8DFA-3984-11E2-8CE6-XDA9311}; AlexaToolbar/amzni-3.0; BTRS99956; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; AlexaToolbar/amzni-3.0; MSIECrawler)";
const char S49478[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; GTB7.4; MSIECrawler)";
const char S49479[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; MSIECrawler);c2VvLWNyYXdsZXI=";
const char S49480[] = "Mozilla/5.0 (Windows NT 4.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S49481[] = "SAMSUNG-GT-E2252/E2252DDLG2 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S49482[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A850L Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49483[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A850S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49484[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A850K Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49485[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-A850K Build/IMM76I)";
const char S49486[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-A850L Build/IMM76I)";
const char S49487[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-A850S Build/IMM76I)";
const char S49488[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A850L Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49489[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A850L Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S49490[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A850K Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49491[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A850S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49492[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N9500 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49493[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; N9500 Build/IMM76I)";
const char S49494[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Garminfone Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17; A50-V5.0.70-user-20101025";
const char S49495[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Garminfone Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 MobileSafari/530.17; A50-V5.0.70-user-X";
const char S49496[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; GarminfoneBuild/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17; A50-V5.0.70-user-X";
const char S49497[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Garminfone Build/ERE27)";
const char S49498[] = "Mozilla/5.0 (Linux; U;Android 2.1-update1; en-us; Garminfone Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17; A50-V5.0.70-user-X";
const char S49499[] = "Mozilla/5.0 (Linux; U; Android2.1-update1; en-us; Garminfone Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17; A50-V5.0.70-user-X";
const char S49500[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us;Garminfone Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17; A50-V5.0.70-user-X";
const char S49501[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-R740C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49502[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-R740C Build/IMM76D)";
const char S49503[] = "Mozilla/5.0 (Linux; U; Android 4.1.5; en-us; I9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49504[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-i9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49505[] = "Mozilla/5.0 (Linux; U; Android 4.1.5; en-gb; I9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49506[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; I9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49507[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; i9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49508[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-gb; I9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49509[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; i92200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49510[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; i9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49511[] = "Mozilla/5.0 (Linux; U; Android 4.1.5; es-es; I9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49512[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9220 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49513[] = "Mozilla/5.0 (Linux; U; Android 4.1.5; es-us; I9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49514[] = "Mozilla/5.0 (Linux; U; Android 4.1.5; it-it; I9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49515[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49516[] = "Mozilla/5.0 (Linux; U; Android 4.1.6; en-us; GT-i9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49517[] = "Mozilla/5.0 (Linux; U; Android 4.1.5; it-it; I92200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49518[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; i92200 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49519[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49520[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9220 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S49521[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9220 Build/JZO54K)";
const char S49522[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; fr-fr; I9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49523[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; es-es; I9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49524[] = "Mozilla/5.0 (Linux; U; Android 4.1.5; en-us; I92200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49525[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; nl-nl; I9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49526[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HK-I9220 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49527[] = "Mozilla/5.0 (Linux; U; Android 4.1.5; en-gb; I92200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49528[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-i9220 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49529[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S49530[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S49531[] = "Mozilla/5.0 (Linux; Android 4.1.1; Transformer Prime TF201 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S49532[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID7012 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49533[] = "Logos/0.9.2 (Linux; U; Android 2.3.3; en_US; MID7012 Build/MID7012 X)";
const char S49534[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID7012 Build/GRI40)";
const char S49535[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A830L Build/JZO54K)";
const char S49536[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A830S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49537[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A830L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49538[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A830K/2.09 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49539[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A830K Build/JZO54K)";
const char S49540[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30";
const char S49541[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S49542[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A830S Build/JZO54K)";
const char S49543[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S49544[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830K Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S49545[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30";
const char S49546[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830S Build/Phi?n b?n Galaxy S4 - By Hanzoo1504) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30";
const char S49547[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830K/2.12 Build/Hanzoo1504) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30";
const char S49548[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830K/2.09 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30";
const char S49549[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830L Build/Hanzoo1504) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30";
const char S49550[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A830K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 NAVER(inapp; search; 230; 4.7.7)";
const char S49551[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830K/2.11 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30";
const char S49552[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830KE/1.06 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49553[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; IM-A830K/2.12 Build/Hanzoo1504) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49554[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; IM-A830KE/1.07 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49555[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A830K/2.13 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49556[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; X325a Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49557[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; X325a-parrot Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49558[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9B206 Safari/7534.48.3";
const char S49559[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9B176 Safari/7534.48.3";
const char S49560[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI H881C Build/HuaweiH881C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49561[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI H881C Build/HuaweiH881C)";
const char S49562[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S49563[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-P895 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49564[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P895 Build/IMM76D)";
const char S49565[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-sa; LG-P895qb Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49566[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S49567[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S49568[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) 480x800; XV6975; Window Mobile 6.5 Professional;";
const char S49569[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) RhodiumW";
const char S49570[] = "8900b/MR2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S49571[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE)";
const char S49572[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) 480x800; XV6875; Window Mobile 6.1 Professional;";
const char S49573[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Sprint S511";
const char S49574[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Sprint-SPH-i350";
const char S49575[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) T-01A_6.5";
const char S49576[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Sprint:PPC6800";
const char S49577[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) T-01A";
const char S49578[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 320x240)";
const char S49579[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S49580[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 480x640; DX900/2.0 Profile";
const char S49581[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC)";
const char S49582[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) 480x800; XV6875.1; Window Mobile 6.5 Professional;";
const char S49583[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone)";
const char S49584[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 320x240; Smartphone)";
const char S49585[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE, BrailleNote)";
const char S49586[] = "SonyEricssonJ10/R7CA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2";
const char S49587[] = "SonyEricssonJ10i2/R7CA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2";
const char S49588[] = "SonyEricssonJ10i/R7CA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2";
const char S49589[] = "Mozilla/5.0 (Linux; Android 4.0.4;  HTC Evo 4G Build/GRI40) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S49590[] = "Mozilla/5.0 (Linux; Android 4.0.3;  EVO Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S49591[] = "Mozilla/5.0 (Linux; Android 4.0.3; EVO Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S49592[] = "Mozilla/5.0 (Linux; Android 4.0.4; PC36100 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S49593[] = "Mozilla/5.0 (Linux; Android 4.0.4; EVO Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S49594[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Evo 4G Build/GRI40) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S49595[] = "Opera/9.80 (Android 4.0.3; Linux; Opera Mobi/ADR-X; U; de) Presto/2.10.254 Version/12.00";
const char S49596[] = "Opera/9.80 (Android 4.0.4; Linux; Opera Mobi/ADR-X; U; en) Presto/2.10.254 Version/12.00";
const char S49597[] = "Opera/9.80 (Android 4.0.2; Linux; Opera Mobi/ADR-X; U; de) Presto/2.10.254 Version/12.00";
const char S49598[] = "Opera/9.80 (Android 4.0.1; Linux; Opera Mobi/ADR-X; U; en-GB) Presto/2.10.254 Version/12.00";
const char S49599[] = "Opera/9.80 (Android 4.0.9; Linux; Opera Mobi/ADR-X; U; en) Presto/2.10.254 Version/12.00";
const char S49600[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S49601[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S49602[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S49603[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-i9100G Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S49604[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B142 Version/6.1 Safari/8536.25 Safari/6533.18.5";
const char S49605[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B143 Version/6.1 Safari/8536.25 Safari/6533.18.5";
const char S49606[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B146 Version/6.1 Safari/8536.25 Safari/6533.18.5";
const char S49607[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; ko_kr) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5";
const char S49608[] = "User-Agent: Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5";
const char S49609[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_2 like Mac OS X; ko_kr) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5";
const char S49610[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B141 Version/6.1 Safari/8536.25 Safari/6533.18.5";
const char S49611[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B329 Version/6.1 Safari/8536.25 Safari/6533.18.5";
const char S49612[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B144 Version/6.1 Safari/8536.25 Safari/6533.18.5";
const char S49613[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; ko_kr) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5";
const char S49614[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5";
const char S49615[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5";
const char S49616[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B350 Version/6.1 Safari/8536.25 Safari/6533.18.5";
const char S49617[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B145 Version/6.1 Safari/8536.25 Safari/6533.18.5";
const char S49618[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_4 like Mac OS X; ko_kr) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5";
const char S49619[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B329 Safari/6533.18.5";
const char S49620[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; 6536.26) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5";
const char S49621[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC_Holiday Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49622[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-kr; HTC_Raider_X710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49623[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Holiday Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49624[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; -au; HTC_Holiday Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49625[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Raider X710e Build/IML74K)";
const char S49626[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-kr; HTC Raider X710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49627[] = "Mozilla/5.0 (Linux; Android 4.3; GT-I9505 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S49628[] = "Mozilla/5.0 (Linux; Android 4.3; GT-I9505G Build/JWR66Y.S003) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S49629[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; SCH-I905 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 USCC-I905";
const char S49630[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; SCH-I905 4G Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S49631[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; SCH-I905 Build/HTJ85B)";
const char S49632[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; SCH-I905 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S49633[] = "curl/7.18.2 (x86_64-pc-linux-gnu) libcurl/7.18.2 OpenSSL/0.9.8g zlib/1.2.3.3 libidn/1.8 (FlipboardProxy/1.1; +http://flipboard.com/browserproxy)";
const char S49634[] = "Opera/9.80 (BREW; Opera Mini/5.0/27.2338; U; en) Presto/2.8.119 240X320 Samsung SCH-U660";
const char S49635[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S49636[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S49637[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100B Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S49638[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S49639[] = "Mozilla/5.0 (Linux; Android 4.1.1;  GT-P3100 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S49640[] = "Mozilla/5.0 (Linux; Android 4.1.0; GT-P3100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S49641[] = "Mozilla/5.0 (Linux; Android 4.1.2;  GT-P3100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S49642[] = "Mozilla/5.0 (Linux; Android 4.1.1;  GT-P3100B Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S49643[] = "Mozilla/5.0 (Linux; Android 4.1.2 Jelly Bean; Galaxy Tab 2 7.0 GT-P3100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S49644[] = "Mozilla/5.0 (Windows NT 4.0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S49645[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49646[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I535 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49647[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-l9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49648[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49649[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-06D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49650[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300T Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49651[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999V Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49652[] = "Mozilla/5.0 (Linux; Android 4.0.4; I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49653[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ARCHOS 80 COBALT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49654[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ARCHOS 80 COBALT Build/IMM76D)";
const char S49655[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S49656[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S49657[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S49658[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S49659[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S49660[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S49661[] = "Mozilla/5.0 (Windows; N; Windows NT 5.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.1.0.5001 Safari/532.4";
const char S49662[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);es-ES;kml:2.2;client:Free;type:default)";
const char S49663[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);en-US;kml:2.2;client:Free;type:default)";
const char S49664[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);pt-BR;kml:2.2;client:Free;type:default)";
const char S49665[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);en-GB;kml:2.2;client:Free;type:default)";
const char S49666[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);it-IT;kml:2.2;client:Free;type:default)";
const char S49667[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);en-AU;kml:2.2;client:Free;type:default)";
const char S49668[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);en;kml:2.2;client:Free;type:default)";
const char S49669[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.2);en-US;kml:2.2;client:Free;type:default)";
const char S49670[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);es-419;kml:2.2;client:Free;type:default)";
const char S49671[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);fr-FR;kml:2.2;client:Free;type:default)";
const char S49672[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);de;kml:2.2;client:Free;type:default)";
const char S49673[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);de-DE;kml:2.2;client:Free;type:default)";
const char S49674[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);ru-RU;kml:2.2;client:Free;type:default)";
const char S49675[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);nl-NL;kml:2.2;client:Free;type:default)";
const char S49676[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);en-CA;kml:2.2;client:Free;type:default)";
const char S49677[] = "Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/345) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49678[] = "Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/262) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49679[] = "Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/2.0.1.30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49680[] = "Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/2.0.0.56) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49681[] = "Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/2.0.1.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49682[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PlayBook Build/GINGERBREAD) AppleWebKit/535.1+ (KHTML, like Gecko) Version/7.2.0.0 Safari/535.1+";
const char S49683[] = "Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/2.1.0.48) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49684[] = "Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/2.1.0.123) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49685[] = "Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/2.1.0.131) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49686[] = "Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-gb; PlayBook Build/10.0.0.279) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49687[] = "Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/10.0.0.336) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49688[] = "Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/10.1.0.103) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49689[] = "Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/10.0.0.347) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49690[] = "Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-gb; PlayBook Build/10.1.0.123) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49691[] = "Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/10.1.0.83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49692[] = "Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/10.1.0.147) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49693[] = "Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/2.1.0.147) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49694[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S49695[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S49696[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S49697[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S49698[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100G Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S49699[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xoom Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S49700[] = "Mozilla/5.0 (Linux; Android 4.1.1; Xoom Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S49701[] = "NokiaC2-02/2.0 (06.51) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-02";
const char S49702[] = "NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S49703[] = "NokiaC2-02.1/2.0 (06.52) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S49704[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Desire HD Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49705[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Desire HD Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49706[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Desire HD Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49707[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Desire HD Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49708[] = "Mozilla/5.0 (Series40; Nokia112/03.32; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.5.16.9";
const char S49709[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C811 4G Build/C811M010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49710[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C811 4G Build/C811M020) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49711[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; C811 4G Build/C811M020)";
const char S49712[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C811 4G Build/C811M021_DU) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49713[] = "Nokia2700c-2/2.0 (09.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia2700c-2) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S49714[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT28at Build/6.1.C.1.111) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S49715[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT28i Build/6.1.E.2.68) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S49716[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT28h Build/6.1.E.3.7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S49717[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; KIS Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49718[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; KIS PLUS Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49719[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; KIS Build/GRK39F)";
const char S49720[] = "NokiaC3-00/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac3-00) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S49721[] = "NokiaC2-03/2.0 (07.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-03) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S49722[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.4.2.3000 Chrome/18.0.966.0 Safari/535.12";
const char S49723[] = "Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.4.2.3000 Chrome/18.0.966.0 Safari/535.12";
const char S49724[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.4.2.3000 Chrome/18.0.966.0 Safari/535.12";
const char S49725[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.4.2.3000 Chrome/18.0.966.0 Safari/535.12";
const char S49726[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Safari/535.19 Silk-Accelerated=false";
const char S49727[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Safari/535.19 Silk-Accelerated=true";
const char S49728[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49729[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-N7000 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49730[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T879 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49731[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49732[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49733[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M480W Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S49734[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-05D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49735[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49736[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000B Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49737[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0) chromeframe/5.0.317.0";
const char S49738[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3) chromeframe/5.0.375.62";
const char S49739[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0) chromeframe/5.0.317.0";
const char S49740[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0) chromeframe/5.0.375.62";
const char S49741[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MDDR; InfoPath.2; OfficeLiveConnector.1.4; OfficeLivePatch.1.3) chromeframe/5.0.317.0";
const char S49742[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E) chromeframe/5.0.375.62";
const char S49743[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0) chromeframe/5.0.317.0";
const char S49744[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0) chromeframe/5.0.317.0";
const char S49745[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S49746[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S49747[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari Version/5.1";
const char S49748[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/5.1 Mobile/11A465 Safari/7537.51.1";
const char S49749[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Version/5.1 Safari/7534.48.3 NAVER(inapp; search; 103; 4.7.6)";
const char S49750[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S49751[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Safari Version/5.1";
const char S49752[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari/7534.48.3";
const char S49753[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Version/5.1 Safari/7534.48.3 NAVER(inapp; search; 103; 4.7.6)";
const char S49754[] = "Mozilla/5.0 (iPad; CPU OS 7 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Version/5.1 Safari/7534.48.3 Kikin/1.9.5";
const char S49755[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; GT-I5510M Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49756[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; GT-I5510M Build/FROYO)";
const char S49757[] = "Voyager/1.0";
const char S49758[] = "parsijoo-crawler";
const char S49759[] = "Mozilla/5.0 (compatible; parsijoo; +http://www.parsijoo.ir/; ehsan.mousakazemi@gmail.com)";
const char S49760[] = "Voyager/1.0;c2VvLWNyYXdsZXI=";
const char S49761[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49762[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T989 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49763[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S49764[] = "Mozilla/5.0 (Linux; Android 4.1.1; A210 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S49765[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.2 Mobile/9B206 Safari/7534.48.3";
const char S49766[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.2 Mobile/9B176 Safari/7534.48.3";
const char S49767[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S49768[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S49769[] = "Mozilla/5.0 (Symbian/3; Series60/5.3 NokiaX7-00; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.4.2.6 Mobile Safari/533.4 3gpp-gba";
const char S49770[] = "Mozilla/5.0 (Symbian/3; Series60/5.3 NokiaX7-00/111.040.1511; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.1.4 Mobile Safari/535.1 3gpp-gba";
const char S49771[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokian8-00) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba";
const char S49772[] = "NokiaN8-00/UCWEB8.7.0.187/51/999";
const char S49773[] = "Mozilla/5.0 (Symbian3; U; en-us; NokiaN8-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile";
const char S49774[] = "NokiaN8-00UCBrowser/8.7.1.234/51/355/UCWEB";
const char S49775[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P504 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S49776[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P504 Build/FRG83G)";
const char S49777[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;LG-P504 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S49778[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P504 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S49779[] = "Mozilla/5.0 (Linux; U; Android2.2.2; en-us; LG-P504 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S49780[] = "Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.5.22 Version/10.51";
const char S49781[] = "Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.5.24 Version/10.53";
const char S49782[] = "Opera/9.80 (Windows NT 6.0; U; en) Presto/2.5.24 Version/10.52";
const char S49783[] = "Opera/9.80 (Windows NT 6.0; U; en) Presto/2.5.24 Version/10.54";
const char S49784[] = "Opera/9.80 (Windows NT 6.0; U; en) Presto/2.5.22 Version/10.50";
const char S49785[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile myTouch 3G Build/DRC92) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S49786[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile myTouch 3G Build/DMD64) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S49787[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile myTouch 3G Build/DRD35) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S49788[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile myTouch 3G Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S49789[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; WOWMobile myTouch 3G Build/unknown) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S49790[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile myTouch 3G Build/DRC83) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S49791[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile myTouch 3G Build/Super D beta) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S49792[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S49793[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599N Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S49794[] = "Nokia201/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S49795[] = "Nokia201/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia201";
const char S49796[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; ARCHOS 101G9 Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S49797[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; ARCHOS 101G9 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S49798[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; ARCHOS 101G9 Build/HTK75D)";
const char S49799[] = "Mozilla/5.0 (Linux; Android 4.1.1; HP Slate 7 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S49800[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4400f Safari/8536.25";
const char S49801[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4414e Safari/8536.25";
const char S49802[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4449d Safari/8536.25";
const char S49803[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A465 Safari/8536.25";
const char S49804[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A501 Safari/8536.25";
const char S49805[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_0 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5A345 Safari/525.20";
const char S49806[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_0 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5A347 Safari/525.20";
const char S49807[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_0 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5A347 Safari/525.200";
const char S49808[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_0_2 like Mac OS X; es-es) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5C1 Safari/525.20";
const char S49809[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_0_1 like Mac OS X; es-es) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5B108 Safari/525.20";
const char S49810[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_0 like Mac OS X; en-us) AppleWebKit/528.4.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5A347 Safari/525.20";
const char S49811[] = "Mozilla/5.0 (Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_0_1 like Mac OS X; haw-US) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5G77 Safari/525.20";
const char S49812[] = "'Mozilla/5.0 (iPhone; U; CPU iOS 2_0 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/XXXXX Safari/525.20'";
const char S49813[] = "Mozilla/5.0 (SymbianOS/9.1; U; en-us) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S49814[] = "Mozilla/5.0 (Linux; Android 4.1.1; Transformer Prime TF201 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S49815[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-05D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S49816[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000B Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S49817[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160L Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S49818[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T879 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S49819[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S49820[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-N7000 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S49821[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S49822[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S49823[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160K Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S49824[] = "Mozilla/5.0 (Linux; Android 4.0.3; SHV-E160S Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S49825[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M480W Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S49826[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49827[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49828[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49829[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49830[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P1000 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49831[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; GT-P1000T Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49832[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT-P1000L Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49833[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; GT-P1000T Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49834[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P1000N Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49835[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT-P1000L Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49836[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; GT-P1000N Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49837[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P1000 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49838[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49839[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P5113 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S49840[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P5113 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S49841[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999V Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S49842[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S49843[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I535 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S49844[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S49845[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-06D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S49846[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-l9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S49847[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300T Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S49848[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0) SlimBrowser/6.01.069";
const char S49849[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; SlimBrowser/6.01.100)";
const char S49850[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Avant Browser; SlimBrowser/6.01.101)";
const char S49851[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S49852[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S49853[] = "Mozilla/5.0 (Linux; Android 4.2.3; I9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S49854[] = "Mozilla/5.0 (Linux; Android 4.2.2; I9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S49855[] = "Mozilla/5.0 (Linux; Android 4.2.9; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S49856[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Desire Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49857[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Desire Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49858[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Desire Build/MIUIAndroid 2.3.4a) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49859[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Desire Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49860[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Desire Build/MIUI-XJ) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49861[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire Build/MIUI-XJ) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49862[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49863[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC Desire Build/MIUI HTC_Desire_A8183 V2.26.841.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49864[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Desire Build/MIUI)";
const char S49865[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Desire CDMA Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49866[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Desire Build/MIUI)";
const char S49867[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Desire Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S49868[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Desire CDMA Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49869[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; HTC Desire Build/MIUI HTC_Desire_A8183 V2.26.841.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49870[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Desire CDMA Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49871[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Desire Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S49872[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S49873[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; HTC Desire Build/MIUIes) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49874[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; HTC Desire Build/MIUIExt) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49875[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8010 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S49876[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8010 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S49877[] = "NokiaN81 SymbianOS/9.2 Series60/3.1 Release/10.1.028 Mozilla/5.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S49878[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaN81-3/11.0.045 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S49879[] = "NokiaN81/ SymbianOS/9.2 Series60/3.1 Release/20.1.009 Mozilla/5.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S49880[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaN81-1/11.0.045 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S49881[] = "NokiaN81";
const char S49882[] = "NokiaN81(14.01)/SymbianOS/9.1 Series60/3.0";
const char S49883[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A523 Safari/8536.25";
const char S49884[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A403 Safari/8536.25";
const char S49885[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A406 Safari/8536.25";
const char S49886[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC6435LVW Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S49887[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S49888[] = "SAMSUNG-GT-S5233A/1.0 SHP/VPP/R5 Jasmine/0.8 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S49889[] = "Mozilla/5.0 (Windows; N; ; en-US) AppleWebKit/527+ (KHTML, like Gecko, Safari/419.3) Qt/4.5.3";
const char S49890[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8013 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S49891[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8013 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S49892[] = "LG-P690f/V10c Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; LG-P690f Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S49893[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru; LG-P690f Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S49894[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P690f Build/GRJ22)";
const char S49895[] = "Mozilla/5.0 (MSIE 7.0; Macintosh; U; SunOS; X11; gu; SV1; InfoPath.2; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648)";
const char S49896[] = "Mozilla/5.0 (MSIE 7.0; Macintosh; U; Trident/4.0; SunOS; X11; gu; SV1; InfoPath.2; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648)";
const char S49897[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/1";
const char S49898[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/1.0.154.48 Safari/525.19";
const char S49899[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/1.0.154.42 Safari/525.19	Chrome 1, Win Vista";
const char S49900[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/1, 6, 0, 6 Safari/533.3";
const char S49901[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.3 (KHTML, like Gecko) Chrome/1, 6, 0, 4 Safari/535.3";
const char S49902[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S49903[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S49904[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S49905[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M480W Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S49906[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S49907[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S49908[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S49909[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S49910[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-LU6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49911[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LG-LU6200 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S49912[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-LU6200 Build/IMM76D)";
const char S49913[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; LG-LU6200/V20c Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49914[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LU6200/V20e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49915[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; LG-LU6200/V-V2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49916[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; LG-LU6200/VNov Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49917[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; LG-LU6200/V20F Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49918[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LG-LU6200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49919[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LU6200/V20f Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49920[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LU6200 Build/Sovandyphone X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49921[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; LG-LU6200/BELL-P930-V20F-FOR-LU6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49922[] = "Opera/9.27 (Macintosh; PPC Mac OS X; U; en)";
const char S49923[] = "Opera/9.23 (Macintosh; Intel Mac OS X; U; en)";
const char S49924[] = "Opera/9.21 (Macintosh; Intel Mac OS X; U; fr)";
const char S49925[] = "Opera/9.22 (Macintosh; PPC Mac OS X; U; en)";
const char S49926[] = "Opera/9.20 (Macintosh; Intel Mac OS X; U; en)";
const char S49927[] = "Opera/9.24 (Macintosh; Intel Mac OS X; U; en)";
const char S49928[] = "Opera/9.26 (Macintosh; PPC Mac OS X; U; en)";
const char S49929[] = "Opera/9.25 (Macintosh; Intel Mac OS X; U; en)";
const char S49930[] = "Opera/9.23 (Mac OS X; ru)";
const char S49931[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X; U; en; rv:1.8.0) Gecko/X Firefox/1.5.0 Opera 9.27";
const char S49932[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-E430-Orange Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49933[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; hr-hr; LG-E430/V10c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49934[] = "LG-E430 Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E430 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49935[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E430 Build/JZO54K)";
const char S49936[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-E430/E43010a-EUR-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49937[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; LG-E430/V10b Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49938[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; LG-E430/V10d Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49939[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E430/E43010a-TSC-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49940[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; 7 Pro T7576)";
const char S49941[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; USCCHTC-PC93100; USC-US)";
const char S49942[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; PC93100)";
const char S49943[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ARCHOS 97 CARBON Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49944[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; ARCHOS 97 CARBON Build/IML74K)";
const char S49945[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8S by HTC)";
const char S49946[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8S by HTC; Orange)";
const char S49947[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; A620e)";
const char S49948[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; A620b)";
const char S49949[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; A620m)";
const char S49950[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8S by HTC; 1.04.161.01)";
const char S49951[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8S by HTC; 1.04.163.03)";
const char S49952[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8S by HTC; 2.00.161.01)";
const char S49953[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8S by HTC; 2.00.163.02)";
const char S49954[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8S by HTC; 3.29.161.01)";
const char S49955[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8S by HTC; 2.00.166.01)";
const char S49956[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8S by HTC; 3.29.163.01)";
const char S49957[] = "Mozilla/5.0 (Linux; Android 4.0.4; LGL86C Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S49958[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S7250D/S7250DXXKK1; U; Bada/2.0; it-it) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile HVGA SMM-MMS/1.2.0 OPN-B";
const char S49959[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S7250-BOUYGUES/S7250AGLA3; U; Bada/2.0; fr-fr) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile HVGA SMM-MMS/1.2.0 OPN-B";
const char S49960[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Kurio7 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49961[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-; Kurio7 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49962[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:17.0) Gecko/X Thunderbird/17.0.8";
const char S49963[] = "Mozilla/5.0 (Linux; Android 4.0.3;  A200 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S49964[] = "Mozilla/5.0 (Linux; Android 4.0.3; A200 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S49965[] = "Mozilla/5.0 (Linux; Android 4.0.4; A200 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S49966[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S6810 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49967[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-S6810L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49968[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S6810 Build/JZO54K)";
const char S49969[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; GT-S6810B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49970[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S6810L Build/JZO54K)";
const char S49971[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-S6810M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49972[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S6810M Build/JZO54K)";
const char S49973[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; SAMSUNG; SCH-I930)";
const char S49974[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Avant Browser; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)";
const char S49975[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Avant Browser)";
const char S49976[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Avant Browser)";
const char S49977[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Avant Browser; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; Avant Browser)";
const char S49978[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 6.0; Avant Browser)";
const char S49979[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Avant Browser)";
const char S49980[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Avant Browser; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; Avant Browser)";
const char S49981[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Avant Browser; Avant Browser; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; Tablet PC 2.0)";
const char S49982[] = "Mozilla/5.0 (Series40; NokiaC2-02/06.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.62.10";
const char S49983[] = "Mozilla/5.0 (Series40; NokiaC2-02.1/06.52; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.62.10";
const char S49984[] = "Mozilla/5.0 (Series40; NokiaC2-02/07.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6";
const char S49985[] = "Mozilla/5.0 (Series40; NokiaC2-02.1/07.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6";
const char S49986[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; S7 Build/Smartfren.androtab.v25.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49987[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; S7 Build/Smartfren.andromaxtab7.0.v.27.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49988[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; S7 Build/Smartfren.androtab.v19.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49989[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49990[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; S7 Build/Smartfren.andromaxtab7.0.v.37.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49991[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S7 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S49992[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S7 Build/Smartfren.andromaxtab7.0.v.38.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49993[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S7 Build/Smartfren.andromaxtab7.0.v.32.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49994[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ENERGY s7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49995[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; S7 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S49996[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; YTONE S7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S49997[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; ELIYA_S7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S49998[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; vivo S7 Build/IMM76D)";
const char S49999[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; S7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50000[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50001[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50002[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-L710 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50003[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50004[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50005[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530U Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50006[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I747M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50007[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50008[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S50009[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S50010[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100B Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S50011[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-au; GT-P6810 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50012[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P6810 Build/HTJ85B)";
const char S50013[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-au; GT-P6810 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S50014[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6810 Build/HTJ85B) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S50015[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P6810 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50016[] = "Vyrso/2.2.3 (Linux; U; Android 3.2; en-US; GT-P6810 Build/HTJ85B.ZSKL2)";
const char S50017[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-au; GT-P6810 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.11;FBDM/{density=1.0,width=800,height=1232};FBLC/en_AU;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P6810;FBSV/3.2;]";
const char S50018[] = "Vyrso/2.2.7 (Linux; U; Android 3.2; en-AU; GT-P6810 Build/HTJ85B)";
const char S50019[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-; A6277 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 Sprint APA6277KT";
const char S50020[] = "Mozilla/5.0 (Linux; U; Android 2.1-update2 (freshhh); en-us; A6277 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 Sprint APA6277KT";
const char S50021[] = "Mozilla/5.0 (Linux; U; Android 2.1-fresh; en-us; A6277 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 Sprint APA6277KT";
const char S50022[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A6277 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 Sprint APA6277KT";
const char S50023[] = "Opera/9.80 (SpreadTrum; Opera Mini/6.5.28991/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S50024[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S50025[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; LG; LG-C900B)";
const char S50026[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; LG; LG-C900)";
const char S50027[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; LG; LG-C900k)";
const char S50028[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/25.0.1364.97) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22";
const char S50029[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/25.0.1364.172)";
const char S50030[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; chromeframe/25.0.1364.172)";
const char S50031[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/25.0.1364.172) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S50032[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/25.0.1364.152) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22";
const char S50033[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; chromeframe/25.0.1364.172)";
const char S50034[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/25.0.1364.172)";
const char S50035[] = "Mozilla/5.0 (Windows NT 6.2; chromeframe/25.0.1364.172) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S50036[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.2 Chrome/15.0.900.2 Safari/535.2";
const char S50037[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.2 Chrome/15.0.900.2 Safari/535.2";
const char S50038[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6810 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50039[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P6810 Build/IMM76D)";
const char S50040[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; lt-lt; GT-P6810 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50041[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6810 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S50042[] = "com.google.android.apps.plus/X (Linux; U; Android 4.0.4; en_GB; GT-P6810; Build/IMM76D) (gzip)";
const char S50043[] = "FaithlifeStudyBible/2.2.8.1 (Linux; U; Android 4.0.4; en-AU; GT-P6810 Build/IMM76D.XXLPK)";
const char S50044[] = "FaithlifeStudyBible/2.2.10 (Linux; U; Android 4.0.4; zh-HK; GT-P6810 Build/IMM76D.ZSLP8)";
const char S50045[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P6810 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50046[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6810 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S50047[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GT-P6810 Build/IMM76D) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.4 Version/4.0 Mobile Safari/530.17 baidubrowser/061_1.8.4.2_diordna_257_0821/gnusmas_51_4.0.4_0186P-TG/Xc/42F751AF98F8EAD052B3E00D9F3F4F35%7C0/1";
const char S50048[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6810 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 GoogleGoggles-Android/1.9; gzip";
const char S50049[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E610 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S50050[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612f Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S50051[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S50052[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E610v Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S50053[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612g Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S50054[] = "Mozilla/5.0 (Linux; Android 4.0.3;  LG-E612 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S50055[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Sony Tablet S Build/THMAS10000) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50056[] = "Mozilla/5.0 (Linux; U; Android 3.1; de-de; Sony Tablet S Build/THMASD10000) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50057[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; Sony Tablet S Build/THMAS10000)";
const char S50058[] = "Mozilla/5.0 (Linux; Android 4.1.2; MB886 Build/9.8.2Q-8_MB886_FFW-11) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50059[] = "Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50060[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50061[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-N7000 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50062[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50063[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50064[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50065[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T879 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50066[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-05D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50067[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000B Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50068[] = "sam-r380 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S50069[] = "Mozilla/5.0 (Linux; U; Android 3.0; en-us; Xoom Build/HRI39) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S50070[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Xoom Build/HWI69) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50071[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Xoom Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50072[] = "Mozilla/5.0 (Linux; U; Android 3.0; en-us; Xoom Build/HRI39) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50073[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Xoom Build/HRI62) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50074[] = "Mozilla/5.0 (Linux; U; Android 3.0; en-us; Xoom Build/HRI35F) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50075[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Xoom Build/HWI63B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50076[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Xoom Build/HWI61B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50077[] = "Dalvik/1.5.0 (Linux; U; Android 3.0.1; Xoom Build/HWI69)";
const char S50078[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Xoom Build/HRI60B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50079[] = "Mozilla/5.0 (Linux; U; Android 3.0; en-us; Xoom Build/HWI59) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50080[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Xoom Build/HWI73) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50081[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; fr-ca; Xoom Build/HONEYCOMB) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50082[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Xoom Build/HRI82B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50083[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; ru-; Xoom Build/HWI69) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50084[] = "Mozilla/5.0 (Linux; U; Android 3.0; en-us; Xoom Build/HRI53) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50085[] = "Mozilla/5.0 (Linux; U; Android 3.0; en-us; Xoom Build/HRI35C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50086[] = "Mozilla/5.0 (Linux; U; Android 3.0; en-us; Xoom Build/HRI39) AppleWebKit/525.10  (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S50087[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SM-T211 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50088[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SM-T211 Build/JZO54K)";
const char S50089[] = "Opera/9.80 (Windows NT 6.0; U; en) Presto/2.9.168 Version/11.50";
const char S50090[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.9.168 Version/11.50";
const char S50091[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.9.168 Version/11.50";
const char S50092[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94; ru) Presto/2.9.168 Version/11.50";
const char S50093[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.9.168 Version/11.50";
const char S50094[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4157); ru) Presto/2.9.168 Version/11.50";
const char S50095[] = "Opera/9.80 (Windows NT 6.0; U; en) Presto/2.9.168 Version/11.50,gzip(gfe) (via translate.google.com)";
const char S50096[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.4 (build 02647); ru) Presto/2.9.168 Version/11.50";
const char S50097[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; en) Opera 11.50";
const char S50098[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; en) Presto/2.10.289 Version/12.02";
const char S50099[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/A.B (KHTML, like Gecko) Chrome/X.Y.Z.W Safari/A.B.";
const char S50100[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9152 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S50101[] = "Mozilla/5.0 (Series40; Nokia306/03.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S50102[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2) Presto/2.12.388 Version/12.11";
const char S50103[] = "Mozilla/5.0 (Series40; NokiaX2-01/08.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S50104[] = "Mozilla/5.0 (Series40; NokiaX2-01.1/07.22; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S50105[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5110 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S50106[] = "Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF700T Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S50107[] = "Mozilla/5.0 (Linux; Android 4.0.4; ASUS Transformer Pad TF700KL Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S50108[] = "Mozilla/5.0 (Linux; Android 4.0.3; ASUS Pad TF700T Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S50109[] = "Mozilla/5.0 (Linux; Android 4.1.1; MD7305 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50110[] = "Mozilla/5.0 (Linux; Android 4.1.2; A600 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50111[] = "Mozilla/5.0 (Linux; Android 4.1.2; I110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50112[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E455g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50113[] = "Mozilla/5.0 (Linux; Android 4.1.1; MiSS Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50114[] = "Mozilla/5.0 (Linux; Android 4.1.2; G526-L22 Build/HuaweiG526-L22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50115[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E451g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50116[] = "Mozilla/5.0 (Linux; Android 4.1.1; Camelio Family tablet Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50117[] = "Mozilla/5.0 (Linux; Android 4.1.1; H100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50118[] = "Mozilla/5.0 (Linux; Android 4.1.2; B943 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50119[] = "Mozilla/5.0 (Linux; Android 4.1.2; EG98 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50120[] = "Mozilla/5.0 (Linux; Android 4.1.1; EGS102 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50121[] = "Mozilla/5.0 (Linux; Android 4.1.1; EGP008 Build/EGP108FW1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50122[] = "Mozilla/5.0 (Linux; Android 4.1.2; A20 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50123[] = "Mozilla/5.0 (Linux; Android 4.1.2; N810 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50124[] = "Mozilla/5.0 (Linux; Android 4.1.1; PLT8223G Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50125[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nivo Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S50126[] = "Mozilla/5.0 (Linux; Android 4.1.1; 9920 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S50127[] = "Mozilla/5.0 (Linux; Android 4.1.1; PMP3970B Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50128[] = "Mozilla/5.0 (Linux; Android 4.1.2; Boston 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50129[] = "Mozilla/5.0 (Linux; Android 4.1.1; Bouygues Telecom Bs 351 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50130[] = "Mozilla/5.0 (Linux; Android 4.1.1; y7053 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50131[] = "Mozilla/5.0 (Linux; Android 4.1.1; WPT005 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50132[] = "Mozilla/5.0 (Linux; Android 4.1.2; G526-L11 Build/HuaweiG526-L11) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50133[] = "Mozilla/5.0 (Linux; Android 4.1.1; G1 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50134[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P712 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50135[] = "Mozilla/5.0 (Linux; Android 4.1.1; Positivo Ypy L700 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50136[] = "Mozilla/5.0 (Linux; Android 4.1.2; Thrive Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50137[] = "Mozilla/5.0 (Linux; Android 4.1.1; MTC 970H Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50138[] = "Mozilla/5.0 (Linux; Android 4.1.1; SN97T41W Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50139[] = "Mozilla/5.0 (Linux; Android 4.1.1; XOLO Play Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50140[] = "Mozilla/5.0 (Linux; Android 4.1.1; XO Learning tablet Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50141[] = "Mozilla/5.0 (Linux; Android 4.1.1; TBDG773 Build/Zeki.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50142[] = "Mozilla/5.0 (Linux; Android 4.1.2; A950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50143[] = "Mozilla/5.0 (Linux; Android 4.1.1; PRO7D Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50144[] = "Mozilla/5.0 (Linux; Android 4.1.2; ICON Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50145[] = "Mozilla/5.0 (Linux; Android 4.1.2; F600 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50146[] = "Mozilla/5.0 (Linux; Android 4.1.2; V72A Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50147[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E415f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50148[] = "Mozilla/5.0 (Linux; Android 4.1.1; AML-MX REF Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50149[] = "Mozilla/5.0 (Linux; Android 4.1.2; iQTab 3G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50150[] = "Mozilla/5.0 (Linux; Android 4.1.2; A9600 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50151[] = "Mozilla/5.0 (Linux; Android 4.1.2; iQTab Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50152[] = "Mozilla/5.0 (Linux; Android 4.1.1; smart_a17 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50153[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGP351 Build/10.1.1.A.1.307) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50154[] = "Mozilla/5.0 (Linux; Android 4.1.1; ACGM816KC Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50155[] = "Mozilla/5.0 (Linux; Android 4.1.1; MD1005 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50156[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S7392 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50157[] = "Mozilla/5.0 (Linux; Android 4.1.1; MaxiIQ Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50158[] = "Mozilla/5.0 (Linux; Android 4.1.1; Positivo Ypy L1050E Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50159[] = "Mozilla/5.0 (Linux; Android 4.1.2; Z998 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50160[] = "Mozilla/5.0 (Linux; Android 4.1.1; MTC 970 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50161[] = "Mozilla/5.0 (Linux; Android 4.1.1; SZENIO 2500 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50162[] = "Mozilla/5.0 (Linux; Android 4.1.1; Tablet-PC-4 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50163[] = "Mozilla/5.0 (Linux; Android 4.1.2; Infinity_GN708 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50164[] = "Mozilla/5.0 (Linux; Android 4.1.1; HS_7DTB14 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50165[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E410B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50166[] = "Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI P2-6070 Build/HuaweiP2-6070) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50167[] = "Mozilla/5.0 (Linux; Android 4.1.1; XTAB-781+ Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50168[] = "Mozilla/5.0 (Linux; Android 4.1.1; Joyz Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50169[] = "Mozilla/5.0 (Linux; Android 4.1.2; Boost Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50170[] = "Mozilla/5.0 (Linux; Android 4.1.2; ThinkPadTablet Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50171[] = "Mozilla/5.0 (Linux; Android 4.1.2; Casper_VIA_A6108 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50172[] = "Mozilla/5.0 (Linux; Android 4.1.1; MT7A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50173[] = "Mozilla/5.0 (Linux; Android 4.1.2; MX Enjoy TV BOX Build/V1.03.04MX01_X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50174[] = "Mozilla/5.0 (Linux; Android 4.1.2; Z995 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50175[] = "Mozilla/5.0 (Linux; Android 4.1.2; Forward_Infinity Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50176[] = "Mozilla/5.0 (Linux; Android 4.1.2; N800 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50177[] = "Mozilla/5.0 (Linux; Android 4.1.2; Huawei Y301A2 Build/HuaweiY301A2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50178[] = "Nokia6500s-1/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6500s-1";
const char S50179[] = "Nokia6500s-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S50180[] = "Nokia6500s/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S50181[] = "SCH-U365/1.0 NetFront/3.0.22.2.23 (GUI) MMP/2.0";
const char S50182[] = "Mozilla/5.0 (Windows NT 5.1; rv:15.3) Gecko/X Firefox/15.3.2 PaleMoon/15.3.2";
const char S50183[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:15.3) Gecko/X Firefox/15.3.2 PaleMoon/15.3.2";
const char S50184[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R830 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S50185[] = "Nokia200/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia200) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile";
const char S50186[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000B Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50187[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T879 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50188[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50189[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-N7000 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50190[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50191[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50192[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50193[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-05D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50194[] = "Mozilla/5.0 (Linux; Android 4.0.3; SHV-E160S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50195[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M480W Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S50196[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P5100 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S50197[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P5100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S50198[] = "Mozilla/5.0 (Linux; Android 4.1.1;  GT-P5100 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S50199[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T779 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S50200[] = "Mozilla/5.0 (Linux; Android 4.1.2;  GT-P5100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S50201[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-mx; E10a Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S50202[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-mx; SonyEricssonE10a Build/2.0.A.0.504) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S50203[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; E10a Build/2.0.1.A.0.47) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S50204[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-co; E10a Build/2.0.2.A.0.24) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S50205[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-mx; E10a Build/2.0.A.0.504) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S50206[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; E10a Build/2.1.1.A.0.6)";
const char S50207[] = "Mozilla/2.0 (compatible; MSIE 2.1; Mac_PowerPC)";
const char S50208[] = "Mozilla/5.0 (compatible; MSIE 1.0; PPC; Trident/3.0)";
const char S50209[] = "Mozilla/5.0 (compatible; MSIE 2.0; PPC; Trident/4.0)";
const char S50210[] = "Mozilla/5.0 (compatible; MSIE 1.0; PPC; SV1; Trident/1.0)";
const char S50211[] = "Mozilla/5.0 (compatible; MSIE 2.0; PPC; SV1; Trident/5.0)";
const char S50212[] = "Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Nokia5235-1d/12.8.092; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S50213[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5235-1b/50.8.001; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.25 Mobile Safari/533.4 3gpp-gba";
const char S50214[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5235-1d/50.8.001; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.25 Mobile Safari/533.4 3gpp-gba";
const char S50215[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5235-1b/20.8.007; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.3";
const char S50216[] = "Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Nokia5235/12.6.092; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S50217[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5235/51.6.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 Mobile Safari/533.4 3gpp-gba";
const char S50218[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S50219[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaN86-3/30.009;; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.1.19424";
const char S50220[] = "NokiaN86/30.101 (SymbianOS/9.3; U; Series60/3.2 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/425 (KHTML, like Gecko) Vesion/3.0 Browser NG/7.1.13036";
const char S50221[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaN86-1/30.009;; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.1.19424";
const char S50222[] = "NokiaN868MP/SymbianOS/9.1 Series60/3.0";
const char S50223[] = "Mozilla/5.0 (Series40; NokiaC3-00/08.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S50224[] = "Mozilla/5.0 (Series40; NokiaC3-00/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5";
const char S50225[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE63) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile";
const char S50226[] = "NokiaE63UCBrowser/8.7.1.234/28/352/UCWEB";
const char S50227[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiae63-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S50228[] = "NokiaE63/UCWEB8.7.0.187/28/999";
const char S50229[] = "Lenovo-A60/S100 Linux/2.6.35.7 Android/2.3.3 Release/04.19.2011 Browser/AppleWebKit533.1 Profile/ Configuration/";
const char S50230[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Lenovo A60 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50231[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Lenovo A60 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50232[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Lenovo A60 Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S50233[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; Lenovo A60 Build/GRI40)";
const char S50234[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Lenovo A60 Build/GRJ90)";
const char S50235[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A60 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S50236[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A60 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S50237[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Lenovo A60 Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S50238[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Lenovo A60+ Build/LenovoLePhone) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50239[] = "Lenovo A60+ Linux/2.6.35 Android/2.3.6 Release/05.04.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1";
const char S50240[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; Lenovo A60+ Build/LenovoLePhone)";
const char S50241[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A60+ Build/LenovoLePhone) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S50242[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S50243[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S50244[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S50245[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S50246[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747M Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S50247[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S50248[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S50249[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S50250[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530C Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S50251[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S50252[] = "Mozilla/5.0 (X11; CrOS i686 1625.0.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1017.3 Safari/535.19";
const char S50253[] = "Mozilla/5.0 (X11; CrOS i686 1660.20.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.29 Safari/535.19";
const char S50254[] = "Mozilla/5.0 (X11; CrOS i686 1660.34.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.32 Safari/535.19";
const char S50255[] = "Mozilla/5.0 (X11; CrOS i686 1660.44.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.39 Safari/535.19";
const char S50256[] = "Mozilla/5.0 (X11; CrOS i686 1660.57.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.46 Safari/535.19";
const char S50257[] = "Mozilla/5.0 (X11; CrOS i686 1660.71.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.52 Safari/535.19";
const char S50258[] = "Mozilla/5.0 (X11; CrOS i686 1660.112.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19";
const char S50259[] = "Mozilla/5.0 (X11; CrOS i686 1494.0.2012) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.983.0 Safari/535.15";
const char S50260[] = "Mozilla/5.0 (X11; CrOS i686 1660.84.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.98 Safari/535.19";
const char S50261[] = "Mozilla/5.0 (X11; CrOS i686 1660.94.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.120 Safari/535.19";
const char S50262[] = "Mozilla/5.0 (X11; CrOS i686 1660.132.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.165 Safari/535.19";
const char S50263[] = "Mozilla/5.0 (X11; CrOS i686 1660.143.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S50264[] = "Mozilla/5.0 (X11; CrOS i686 1424.0.2011) AppleWebKit/535.12 (KHTML, like Gecko) Chrome/18.0.966.0 Safari/535.12";
const char S50265[] = "Mozilla/5.0 (X11; CrOS i686 1533.0.2012) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.995.0 Safari/535.15";
const char S50266[] = "Mozilla/5.0 (X11; CrOS i686 1602.0.2012) AppleWebKit/535.18 (KHTML, like Gecko) Chrome/18.0.1012.1 Safari/535.18";
const char S50267[] = "Mozilla/5.0 (X11; CrOS i686 1494.0.2011) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.983.0 Safari/535.15";
const char S50268[] = "Mozilla/5.0 (X11; CrOS i686 1563.0.2012) AppleWebKit/535.16 (KHTML, like Gecko) Chrome/18.0.1003.0 Safari/535.16";
const char S50269[] = "Mozilla/5.0 (X11; CrOS i686 1660.98.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.140 Safari/535.19";
const char S50270[] = "Mozilla/5.0 (X11; CrOS i686 1465.0.2011) AppleWebKit/535.14 (KHTML, like Gecko) Chrome/18.0.973.0 Safari/535.14";
const char S50271[] = "Mozilla/5.0 (X11; CrOS i686 1660.99.2012) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.145 Safari/535.19";
const char S50272[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E970 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S50273[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID1042 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50274[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; MID1042 Build/IML74K)";
const char S50275[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S50276[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S50277[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7000 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S50278[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S50279[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M480W Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S50280[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S50281[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S50282[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000B Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S50283[] = "SAMSUNG-GT-S3350/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S50284[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT901 Build/9.8.2Q-50_SLS-7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50285[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50286[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13)";
const char S50287[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT901 Build/9.8.2Q-50_SLS-14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50288[] = "stagefright/1.2 (Linux;Android 4.1.2;motorola XT901 Build/9.8.2Q-50_SLS-13)";
const char S50289[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50290[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT28h Build/6.1.E.3.7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50291[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT28at Build/6.1.C.1.111) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50292[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT28i Build/6.1.E.3.7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50293[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50294[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/0";
const char S50295[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.3.154.9 Safari/525.19";
const char S50296[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.4.154.29 Safari/525.19";
const char S50297[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.2.152.0 Safari/525.19";
const char S50298[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; de) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.2.149.27 Safari/525.13";
const char S50299[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/0.0.0.0 Safari/534.24";
const char S50300[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; de) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.4.154.29 Safari/525.19";
const char S50301[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/0.0.0.0 Safari/533.3";
const char S50302[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A551 Safari/8536.25";
const char S50303[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A403 Safari/8536.25";
const char S50304[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A525 Safari/8536.25";
const char S50305[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A523 Safari/8536.25";
const char S50306[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A405 Safari/8536.25";
const char S50307[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S50308[] = "NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-03) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S50309[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 NokiaN79-3/11.049; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S50310[] = "NokiaN79/SymbianOS/9.3 Series60/3.2 Release/32.001; Mozilla/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML,like Gecko) Safari/413";
const char S50311[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 NokiaN79-1/32.001; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S50312[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710VMUB Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50313[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710BST Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50314[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50315[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 NokiaN96-3/3.00; Profile/MIDP-2.1 Configuration/CLDC-1.1;) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S50316[] = "NokiaN96/SymbianOS/9.3 Series60/3.2 Release/30.033; Mozilla/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML,like Gecko) Safari/413";
const char S50317[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 NokiaN96-1/3.00; Profile/MIDP-2.1 Configuration/CLDC-1.1;) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S50318[] = "Nokia N96";
const char S50319[] = "Mozilla/5.0 BilgiSisPhone (Linux; U; Android ; en-us; Generic Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50320[] = "Mozilla/5.0 (Linux; U; Android 0.5; en-us) AppleWebKit/522+ (KHTML, like Gecko) Safari/419.3";
const char S50321[] = "Mozilla/5.0 (Linux; U; Android unknown; zh-tw; VITA V8 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50322[] = "Mozilla/5.0 (Linux; U; Android 0.5; en-us) AppleWebKit/522  (KHTML, like Gecko) Safari/419.3";
const char S50323[] = "Player/LG Player 1.0 for Android(stagefright alternative)";
const char S50324[] = "LG Player 1.0; Android";
const char S50325[] = "Android Application:";
const char S50326[] = "Android ClemBrowser/1.0 Clempad_CPT";
const char S50327[] = "Mozilla/5.0 (Linux; U; Android 5.3.4(Android 5.1.1); en-us; iPhone 4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50328[] = "Mozilla/5.0 (Linux; U; Android 5.3.3(Android 5.1.1); en-us; iPhone 4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50329[] = "Mozilla/5.0 (Linux; U; Android 5.3.9(Android 5.1.1); en-us; iPhone_4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50330[] = "Mozilla/5.0 (Linux; U; Android 5.4.4(Android 5.1.1); en-us; iPhone_4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50331[] = "Android-Wifi/0.1";
const char S50332[] = "Mozilla/5.0 (Linux; U; Android 5.3.8(Android 5.1.1); en-us; iPhone_4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50333[] = "Mozilla/5.0 (Linux; U; Android 5.4.8(Android 5.1.1); en-us; iPhone_4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50334[] = "Mozilla/5.0 (Linux; U; Android 5.4.7(Android 5.1.1); en-us; iPhone_4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50335[] = "Mozilla/5.0 (Linux; U; Android 5.5.0(Android 5.1.1); en-us; iPhone_4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50336[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Desire HD Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50337[] = "LG-E460 Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E460 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50338[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; LG-E460/V10c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50339[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E460 Build/JZO54K)";
const char S50340[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-E460/V10d Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50341[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-E460/V10f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50342[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; LG-E460/E46010a-SFR-FR Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50343[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-E460/E46010f-EUR-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50344[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; LG-E460-Orange/E46010b-ORG-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50345[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; et; LG-E460 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50346[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-E460/E46010c-EUR-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50347[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E460f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50348[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; LG-E460/E46010b-SFR-FR Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50349[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E460/E46010b-O2-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50350[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; hu-hu; LG-E460/V10a Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50351[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ro-ro; LG-E460-Orange/E46010e-ORG-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50352[] = "Mozilla/5.0 (Series40; Nokia308/05.80; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.13.8";
const char S50353[] = "Nokia308/2.0 (05.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Series40; Nokia308/05.80; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.13.8";
const char S50354[] = "Mozilla/5.0 (X11; Linux x86_64) KHTML/4.10.5 (like Gecko) Konqueror/4.10";
const char S50355[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.21 (KHTML, like Gecko) konqueror/4.10.5 Safari/537.21";
const char S50356[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) konqueror/4.10.5 Safari/534.34";
const char S50357[] = "Mozilla/5.0 (X11) KHTML/4.10.5 (like Gecko) Konqueror/4.10";
const char S50358[] = "Mozilla/5.0 (X11; Linux i686) KHTML/4.10.5 (like Gecko) Konqueror/4.10";
const char S50359[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9A406 Safari/7534.48.3";
const char S50360[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9A405 Safari/7534.48.3";
const char S50361[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9A334 Safari/7534.48.3";
const char S50362[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; T9295)";
const char S50363[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.0 Mobile/10A403 Safari/8536.25";
const char S50364[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M532 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50365[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; M532 Build/IML74K)";
const char S50366[] = "Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S50367[] = "Mozilla/5.0 (Linux; Android 4.1.2; MB886 Build/9.8.2Q-8_MB886_FFW-11) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S50368[] = "Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S50369[] = "NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac1-01) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S50370[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.7.4; U; ru) Presto/2.10.289 Version/12.00";
const char S50371[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5; U; en) Presto/2.10.289 Version/12.01";
const char S50372[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T779 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S50373[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P5100 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S50374[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P5100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S50375[] = "Opera/9.25 (Windows NT 5.0; U; en)";
const char S50376[] = "Opera/9.23 (Windows NT 5.0; U; en)";
const char S50377[] = "Opera/9.22 (Windows NT 5.0; U; en)";
const char S50378[] = "Opera/9.21 (Windows NT 5.0; U; de)";
const char S50379[] = "Opera/9.26 (Windows NT 5.0; U; fr)";
const char S50380[] = "Opera/9.27 (Windows NT 5.0; U; en)";
const char S50381[] = "Opera/9.20 (Windows NT 5.0; U; en)";
const char S50382[] = "Opera/9.24 (Windows NT 5.0; U; ru)";
const char S50383[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; Micromax A116 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50384[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Micromax A116 Build/JZO54K)";
const char S50385[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; Micromax A116 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50386[] = "Mozilla/5.0 (Linux; Android 4.1.1; Galaxy X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S50387[] = "Mozilla/5.0 (Linux; Android 4.1.1; Galaxy Nexus Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S50388[] = "Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S50389[] = "Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S50390[] = "Mozilla/5.0 (Linux; Android 4.1; Galaxy Nexus Build/JRN84D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S50391[] = "Mozilla/5.0 (Linux; Android 4.1.1; Galaxy Nexus Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S50392[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0";
const char S50393[] = "Mozilla/5.0 (Macintosh; U; PowerPC Mac OS X 10_5_8; en-US) AppleWebKit/531.21.8+(KHTML, like Gecko, Safari/528.16) Version/5.10.3 OmniWeb/622.14.0";
const char S50394[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0";
const char S50395[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-US) AppleWebKit/528.16 (KHTML, like Gecko, Safari/528.16) OmniWeb/v622.8.0";
const char S50396[] = "Mozilla/5.0 (Macintosh; U; PowerPC Mac OS X 10_5_8; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0";
const char S50397[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0";
const char S50398[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en; rv:1.9.2.24) Gecko/X Camino/2.1 (like Firefox/3.6.24)";
const char S50399[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en; rv:1.9.2.28) Gecko/X Camino/2.1.2 (like Firefox/13.0)";
const char S50400[] = "NokiaX2-02/2.0 (11.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-02) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S50401[] = "Nokia5230/UCWEB8.7.0.187/50/999";
const char S50402[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5230) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S50403[] = "Nokia5230UCBrowser/8.7.1.234/50/444/UCWEB";
const char S50404[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5230) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba";
const char S50405[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-AS855 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50406[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-AS855 Build/GRJ22)";
const char S50407[] = "SuperBot/4.7.0.74 (Windows XP)";
const char S50408[] = "Mozilla/4.0 (Add Catalog; Windows NT 5.1; infobot.4942@yahoo.com)";
const char S50409[] = "SuperBot/4.4.0.60 (Windows XP)";
const char S50410[] = "WebAlta Crawler/1.3.12 (http://www.webalta.ru/bot.html) (Windows; U; Windows NT 5.1; ru-RU)";
const char S50411[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.6) Gecko/X Firefox/2.0.0.6  - WebCrawler http://cognitiveseo.com/bot.html";
const char S50412[] = "yacybot (x86 Windows XP 5.1; java 1.6.0_17; Europe/de) http://yacy.net/bot.html;c2VvLWNyYXdsZXI=";
const char S50413[] = "WebAlta Crawler/1.3.11 (http://www.webalta.ru/bot.html) (Windows; U; Windows NT 5.1; ru-RU)";
const char S50414[] = "yacybot (x86 Windows XP 5.1; java 1.6.0_19; America/en) http://yacy.net/bot.html;c2VvLWNyYXdsZXI=";
const char S50415[] = "yacybot (freeworld/global; x86 Windows XP 5.1; java 1.7.0_25; America/en) http://yacy.net/bot.html";
const char S50416[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:11.0) Gecko/X Firefox/11.0 CometBird/11.0";
const char S50417[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:11.0) Gecko/X Firefox/11.0 CometBrowser/11.0";
const char S50418[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E970 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S50419[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S50420[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S50421[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X dual sim Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S50422[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire XC dual sim Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S50423[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; it-it; Garmin-Asus A10 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17; A10-V5.0.67-user-20101018";
const char S50424[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; Garmin-Asus A10 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17; A10-V5.0.71-user-X";
const char S50425[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Garmin-Asus A10 Build/ERE27)";
const char S50426[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50427[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50428[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5)";
const char S50429[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5.3.5)";
const char S50430[] = "NokiaC3-00/5.0 (08.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac3-00) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S50431[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; CnM TouchPad 9DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50432[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; CnM TouchPad 10.1DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50433[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; CnM TouchPad 10.1DC Build/JRO03H)";
const char S50434[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; CnM TouchPad 9DC Build/JRO03H)";
const char S50435[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; CnM TouchPad 7DC Build/JRO03H)";
const char S50436[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; CnM TouchPad 7DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50437[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Safari Version/5.1";
const char S50438[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 Safari Version/5.1";
const char S50439[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Safari Version/5.1";
const char S50440[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari Version/5.1";
const char S50441[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145 Safari Version/5.1";
const char S50442[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari Version/5.1";
const char S50443[] = "Mozilla/5.0 (iPhone; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B146 Safari/7536.26";
const char S50444[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari Version/5.1";
const char S50445[] = "Mozilla/5.0 (iPhone; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B329 Safari/7536.26";
const char S50446[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari Version/5.1";
const char S50447[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/10B329 Safari/7534.48.3";
const char S50448[] = "Mozilla/5.0 (iPhone; CPU OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B350 Safari/7536.26";
const char S50449[] = "Mozilla/5.0 (iPhone; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B144 Safari/7536.26";
const char S50450[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Transformer TF101 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50451[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S50452[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; Transformer TF101 Build/HMJ37)";
const char S50453[] = "Mozilla/5.0 (Linux; U; Android 3.1; de-de; Transformer TF101 Build/HMJ37) AppleWebKit/534.13+ (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50454[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Status Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50455[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Status Build/GRI40)";
const char S50456[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Status Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50457[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Status Build/GRJ90)";
const char S50458[] = "Android";
const char S50459[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50460[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-06D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50461[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I535 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50462[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50463[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300T Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50464[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-l9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50465[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999V Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50466[] = "Mozilla/5.0 (Linux; Android 4.0.4; i9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50467[] = "Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S50468[] = "Mozilla/5.0 (Linux; Android 4.0.4; Transformer Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S50469[] = "Mozilla/5.0 (Linux; Android 4.0.3; TF101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S50470[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P5100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S50471[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T779 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S50472[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P5100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S50473[] = "Mozilla/5.0 (Linux; Android 4.0.3;  MZ604 Build/I.7.1-34) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S50474[] = "Mozilla/5.0 (Linux; Android 4.0.3; MZ604 Build/I.7.1-34) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S50475[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ604 Build/I.7.1-45) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S50476[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGP311 Build/10.1.C.0.370) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S50477[] = "AppleCoreMedia/1.0.0.8H7 (iPad; U; CPU OS 4_3_2 like Mac OS X; en_us)";
const char S50478[] = "AppleCoreMedia/1.0.0.8G4 (iPad; U; CPU OS 4_3_1 like Mac OS X; en_us)";
const char S50479[] = "AppleCoreMedia/1.0.0.8J2 (iPad; U; CPU OS 4_3_3 like Mac OS X; en_us)";
const char S50480[] = "AppleCoreMedia/1.0.0.8K2 (iPad; U; CPU OS 4_3_4 like Mac OS X; en_us)";
const char S50481[] = "AppleCoreMedia/1.0.0.8L1 (iPad; U; CPU OS 4_3_5 like Mac OS X; en_us)";
const char S50482[] = "AppleCoreMedia/1.0.0.8F191 (iPad; U; CPU OS 4_3 like Mac OS X; en_us)";
const char S50483[] = "Mozilla/5.0 AppleCoreMedia/1.0.0.8J2 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F190 Safari/6533.18.5";
const char S50484[] = "Mozilla/5.0 AppleCoreMedia/1.0.0.8J2 (iPad; U; CPU OS 4_3like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/6533.18.5";
const char S50485[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5) Presto/2.12.388 Version/12.12";
const char S50486[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-P1010 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50487[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-P1010 Build/GINGERBREAD)";
const char S50488[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-P1010 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.7 Mobile Safari/534.30";
const char S50489[] = "Vyrso/2.1.2 (Linux; U; Android 2.3.6; en-US; GT-P1010 Build/GINGERBREAD.XXKPI)";
const char S50490[] = "Logos/2.1.2 (Linux; U; Android 2.3.6; en-US; GT-P1010 Build/GINGERBREAD.XXKPO)";
const char S50491[] = "Logos/2.1.2 (Linux; U; Android 2.3.6; cs-CZ; GT-P1010 Build/GINGERBREAD.XXKPG)";
const char S50492[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT-P1010 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50493[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; GT-P6210 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50494[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-P1010 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S50495[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-; GT-P1010 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50496[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-P1010 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 GoogleGoggles-Android/1.9; gzip";
const char S50497[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID7022 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50498[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID7022 Build/GRI40)";
const char S50499[] = "Logos/0.9.3 (Linux; U; Android 2.3.3; en_US; MID7022 Build/MID7022 X)";
const char S50500[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700T Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S50501[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700KL Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S50502[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7000 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50503[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7000 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50504[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7000 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50505[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-T879 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50506[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7000 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50507[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2; facebookexternalhit)AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1309.0Safari/537.17";
const char S50508[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S6312 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50509[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S6312 Build/JZO54K)";
const char S50510[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-S6312 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50511[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X+ Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S50512[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC EVARE_UL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S50513[] = "NokiaC1-01/2.0 (04.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac1-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile";
const char S50514[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7105 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50515[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7105 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50516[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7105 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50517[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-L900 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50518[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L900 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50519[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I317 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50520[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7100 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50521[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Galaxy Note II Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50522[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T889 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50523[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7105 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50524[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7100 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50525[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-N7100 Build/JDQ39E)";
const char S50526[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-N7100 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50527[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50528[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; N7100+ Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50529[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50530[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I317 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50531[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; N7100 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50532[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-N7105 Build/JDQ39E)";
const char S50533[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SAMSUNG-SGH-I317 Build/JDQ39E)";
const char S50534[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50535[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T889 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50536[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I317 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50537[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L900 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50538[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; NXM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50539[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; it-it; DATAM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50540[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.1; NXM803HC Build/GINGERBREAD)";
const char S50541[] = "IUC(U;iOS 5.0.1;Zh-cn;320*480;)/UCWEB8.5.0.163/42/800";
const char S50542[] = "IUC(U;iOS 5.1.1;Zh-cn;320*480;)/UCWEB8.5.1.178/42/997";
const char S50543[] = "Mozilla/5.0 (Java; U; en-us; mstar) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S50544[] = "IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB8.5.0.163/41/800";
const char S50545[] = "IUC(U;iOS 4.1;Zh-cn;320*480;)/UCWEB8.5.0.163/41/800";
const char S50546[] = "Mozilla/5.0 (Java; U; en-us; maui) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S50547[] = "Mozilla/5.0 (Java; U; en-us; opera) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S50548[] = "Mozilla/5.0 (Java; U; en-us; j2me) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S50549[] = "Mozilla/5.0 (Java; U; en-us; spice) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S50550[] = "IUC(U;iOS 4_1;en-us;320*480;)/UCWEB8.5.1.228/41/355";
const char S50551[] = "Mozilla/5.0 (Java; U; en-us; generic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S50552[] = "Mozilla/5.0 (Java; U; id; maui) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S50553[] = "Mozilla/5.0 (Java; U; en-us; mozilla) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/405/UCWEB Mobile";
const char S50554[] = "Mozilla/5.0 (Java; U; en-us; jbed-fastbcc) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S50555[] = "Mozilla/5.0 (Java; U; en-us; mstar platform) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S50556[] = "Mozilla/5.0 (Java; U; en-us; maui browser) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S50557[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7105 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S50558[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S50559[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I317 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S50560[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T889 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S50561[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L900 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S50562[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I317 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S50563[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Note II Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S50564[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Note II Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S50565[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R950 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S50566[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S50567[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 NokiaN78-5/11.043; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S50568[] = "NokiaN78/SymbianOS/9.3 Series60/3.2 Release/30.014; Mozilla/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML,like Gecko) Safari/413";
const char S50569[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 NokiaN78-1/12.046; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S50570[] = "(C)NokiaN78/SymbianOS/9.1 Series60/3.0";
const char S50571[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S50572[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S50573[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xperia P Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S50574[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S50575[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; SAMSUNG-SGH-I927R Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50576[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SAMSUNG-SGH-I927R Build/GINGERBREAD)";
const char S50577[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SoftwinerEvb Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50578[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; SoftwinerEvb Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50579[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SoftwinerEvb Build/JRO03C)";
const char S50580[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SoftwinerEvb Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50581[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S50582[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; K1 Build/HMJ37) AppleWebKit/534.13(KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50583[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; K1 Build/K1_A301_03_03_X_US) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50584[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; K1 Build/K1_A301_04_04_X_US) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50585[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; K1 Build/K1_A301_02_02_X_CA) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50586[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; K1 Build/K1_A301_02_02_X_US) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50587[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; K1 Build/K1_A301_01_01_X_US) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50588[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; K1 Build/K1_A301_03_02_X_CN) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50589[] = "Mozilla/5.0 (Linux; U; Android 3.1; es-es; K1 Build/K1_A301_01_01_X_SG) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50590[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; K1 Build/K1_A301_02_02_X_CA)";
const char S50591[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; K1 Build/K1_A301_03_03_X_UK) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S50592[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S50593[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100G Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S50594[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S50595[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S50596[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100G Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S50597[] = "AppleCoreMedia/1.0.0.11C74 (Macintosh; U; Intel Mac OS X 10_7_2; en_us)";
const char S50598[] = "AppleCoreMedia/1.0.0.11D50 (Macintosh; U; Intel Mac OS X 10_7_3; en_us)";
const char S50599[] = "AppleCoreMedia/1.0.0.11D50b (Macintosh; U; Intel Mac OS X 10_7_3; en_us)";
const char S50600[] = "AppleCoreMedia/1.0.0.11D50d (Macintosh; U; Intel Mac OS X 10_7_3; en_us)";
const char S50601[] = "AppleCoreMedia/1.0.0.11E53 (Macintosh; U; Intel Mac OS X 10_7_4; en_us)";
const char S50602[] = "AppleCoreMedia/1.0.0.11C2002 (Macintosh; U; Intel Mac OS X 10_7_2; en_us)";
const char S50603[] = "AppleCoreMedia/1.0.0.11D2001 (Macintosh; U; Intel Mac OS X 10_7_3; en_us)";
const char S50604[] = "AppleCoreMedia/1.0.0.11E2620 (Macintosh; U; Intel Mac OS X 10_7_4; en_us)";
const char S50605[] = "AppleCoreMedia/1.0.0.11E2705 (Macintosh; U; Intel Mac OS X 10_7_4; en_us)";
const char S50606[] = "AppleCoreMedia/1.0.0.11E2617 (Macintosh; U; Intel Mac OS X 10_7_4; en_us)";
const char S50607[] = "AppleCoreMedia/1.0.0.11G63 (Macintosh; U; Intel Mac OS X 10_7_5; en_us)";
const char S50608[] = "AppleCoreMedia/1.0.0.11G56 (Macintosh; U; Intel Mac OS X 10_7_5; en_us)";
const char S50609[] = "AppleCoreMedia/1.0.0.11G63b (Macintosh; U; Intel Mac OS X 10_7_5; en_us)";
const char S50610[] = "AppleCoreMedia/1.0.0.11G63 (Macintosh; U; Intel Mac OS X 10_7_5; de_de)";
const char S50611[] = "AppleCoreMedia/1.0.0.11G63 (Macintosh; U; Intel Mac OS X 10_7_5; fr_fr)";
const char S50612[] = "iTunes/10.6.3 (Macintosh; Intel Mac OS X 10.8) AppleWebKit/536.25";
const char S50613[] = "iTunes/10.7 (Macintosh; Intel Mac OS X 10.8.1) AppleWebKit/536.25";
const char S50614[] = "iTunes/10.7 (Macintosh; Intel Mac OS X 10.8.2) AppleWebKit/536.26.14";
const char S50615[] = "iTunes/11.0 (Macintosh; OS X 10.8) AppleWebKit/536.25";
const char S50616[] = "iTunes/11.0.1 (Macintosh; OS X 10.8.2) AppleWebKit/536.26.14";
const char S50617[] = "iTunes/11.0.2 (Macintosh; OS X 10.8.3) AppleWebKit/536.28.10";
const char S50618[] = "iTunes/11.0.4 (Macintosh; OS X 10.8.4) AppleWebKit/536.30.1";
const char S50619[] = "iTunes/11.1 (Macintosh; OS X 10.8.5) AppleWebKit/536.30.1";
const char S50620[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50621[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50622[] = "GT-N7100 Linux/3.0.13 Android/4.1.1 Release/02.01.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S50623[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N9550/N7100 PAD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50624[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50625[] = "Mozilla/5.0 (Linux; U; Android 4.0.5; en-us; N7100 Build/MocorDroid4.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50626[] = "BlackBerry8100/4.5.0.182 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102";
const char S50627[] = "Mozilla/5.0 (Windows NT 5.1; rv:19.0) Gecko/X Firefox/19.0 PaleMoon/19.0.2";
const char S50628[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:19.0) Gecko/X Firefox/19.0 PaleMoon/19.0.2";
const char S50629[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:20.0) Gecko/X Firefox/20.0 AlexaToolbar/alxf-2.18";
const char S50630[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/10.10 Chromium/14.0.835.202 Chrome/14.0.835.202 Safari/535.1";
const char S50631[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/10.10 Chromium/14.0.808.0 Chrome/14.0.808.0 Safari/535.1";
const char S50632[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U1 Build/HCL ME Tablet U1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50633[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; U1 Build/HCL ME Tablet U1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50634[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; U1 Build/HCL ME Tablet U1)";
const char S50635[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:22.0) Gecko/X Firefox/22.0 AlexaToolbar/alxf-2.18";
const char S50636[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:22.0) Gecko/X Firefox/22";
const char S50637[] = "Mozilla/5.0 (Series40; Nokia205/03.18; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S50638[] = "Mozilla/5.0 (Series40; Nokia205.1/03.19; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S50639[] = "Mozilla/5.0 (Series40; Nokia205.3/03.19; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S50640[] = "Mozilla/5.0 (Series40; Nokia2055/03.20; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S50641[] = "Mozilla/5.0 (Series40; Nokia2052/03.20; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S50642[] = "Mozilla/5.0 (Series40; Nokia2051/03.20; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S50643[] = "Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A465 Safari/8536.25";
const char S50644[] = "Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A4449d Safari/8536.25";
const char S50645[] = "Mozilla/5.0 (iPod; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A501 Safari/8536.25";
const char S50646[] = "SAMSUNG-SGH-T749-MKB/T749UEJJ7 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S50647[] = "SAMSUNG-SGH-T749/T749UVIK1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S50648[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB860 Build/4.5.91; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50649[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB860 Build/4.5.91; CM7.2 RC1 Base) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50650[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB860 Build/4.5.141; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50651[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MB860 Build/4.5.141; CM7 X [yyyymmdd]) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50652[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9A334 Safari/7534.48.3";
const char S50653[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9A405 Safari/7534.48.3";
const char S50654[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S50655[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S50656[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Prestige 7L_4.1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50657[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Prestige 7L_4.1 Build/Prestige) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50658[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Prestige 7L_4.1 Build/JRO03C)";
const char S50659[] = "Mozilla/5.0 (Linux; Android 4.1.1; Galaxy Nexus Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S50660[] = "Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S50661[] = "Mozilla/5.0 (Linux; Android 4.1.1; Galaxy X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S50662[] = "Mozilla/5.0 (Linux; Android 4.1; Galaxy Nexus Build/JRN84D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S50663[] = "jBrowser/J2ME Profile/MIDP-1.0 Configuration/CLDC-1.0 (Google WAP Proxy/1.0)";
const char S50664[] = "Mozilla/5.0 (compatible; CRI/1.3; +http://www.creativeresearchindexer.com)";
const char S50665[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ja-jp; SC-03D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50666[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ja-; SC-03D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50667[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SC-03D Build/gingerbread)";
const char S50668[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ja-jp; SC-03D Build/gingerbread) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 YJApp-ANDROID jp.co.yahoo.android.yjtop/1.7.2";
const char S50669[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC first Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50670[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P1000 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50671[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P1000 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50672[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P1000 Build/JRO03R; CyanogenMod-Sapir) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50673[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P1000 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50674[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; GT-P1000L Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50675[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P1000 Build/JRO03R; CyanogenMod-Sapir) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50676[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P1000L Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50677[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; GT-P1000L Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50678[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; GT-P1000N Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50679[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P1000 Build/JZO54K; CyanogenMod-Sapir) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50680[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-P1000N Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50681[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-; GT-P1000N Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50682[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; GT-P1000N Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50683[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; XT886 Build/6.7.3-135_YIC-17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50684[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT886 Build/6.7.3-179_YIC-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50685[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-ca; XT886 Build/6.7.3-179_YIC-11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50686[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT886 Build/6.7.3-169) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50687[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; XT886 Build/6.7.3-135_YIC-17)";
const char S50688[] = "NokiaX2-02/2.0 (11.79) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-02) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S50689[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID2 GLOBAL Build/S273) AppleWebKit/525.10  (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S50690[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID2 GLOBAL Build/S273) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50691[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; DROID2 GLOBAL Build/S273)";
const char S50692[] = "Mozilla/5.0 (Linux; U; Android2.2; en-us; DROID2 GLOBAL Build/S273) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50693[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROID2 GLOBAL Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50694[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID2 GLOBALBuild/S273) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50695[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us;DROID2 GLOBAL Build/S273) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50696[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID2GLOBAL Build/S273) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50697[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID2 GLOBAL Build/S273) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S50698[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROID2 GLOBAL Build/3.4.2_80_D2G_16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50699[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-S730M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50700[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-S730M Build/IMM76D)";
const char S50701[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-S730M-parrot Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50702[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia6210Navigator/05.16; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S50703[] = "NOKIA6210 Navigator";
const char S50704[] = "Nokia6210s/05.16 (SymbianOS/9.3; U; Series60/3.2 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S50705[] = "Opera/9.80 (Series 60; Opera Mini/6.7.29868/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S50706[] = "Mozilla/5.0 (compatible; MSIE 10.0; Win64; Trident/6.0)";
const char S50707[] = "Mozilla/5.0 (compatible; MSIE 10.0; Win32; Trident/6.0)";
const char S50708[] = "Mozilla/5.0 (compatible; MSIE 10.0; 68K; Trident/5.0)";
const char S50709[] = "Mozilla/5.0 (compatible; MSIE 10.0; PPC; Win64; IA64; Trident/5.0)";
const char S50710[] = "Mozilla/5.0 (compatible; MSIE 10.0; PPC; WOW64; Trident/2.0)";
const char S50711[] = "Mozilla/5.0 (compatible; MSIE 10.0; 68K; SV1; Trident/3.0)";
const char S50712[] = "Mozilla/5.0 (compatible; MSIE 10.0; PPC; .NET CLR; Trident/1.0)";
const char S50713[] = "Mozilla/5.0 (compatible; MSIE 10.0; 68K; WOW64; Trident/1.0)";
const char S50714[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; LG-P925G Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50715[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; fr-ca; LG-P925g Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50716[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P925g Build/FRG83G)";
const char S50717[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; B1-710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Safari/534.30";
const char S50718[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; B1-710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50719[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; B1-710 Build/JZO54K)";
const char S50720[] = "Mozilla/5.0 (Series40; NokiaC2-05/08.69; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S50721[] = "Mozilla/5.0 (Series40; NokiaC2-05.1/08.69; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S50722[] = "Mozilla/5.0 (Series40; NokiaC2-05/08.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5";
const char S50723[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Prism II Build/HuaweiU8686) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50724[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Prism II Build/HuaweiU8686)";
const char S50725[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I777 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50726[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I777 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50727[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I777 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50728[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGH-I777 Build/JDQ39)";
const char S50729[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-US; SGH-i777 Build/JOP40D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50730[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I777 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50731[] = "SAMSUNG-GT-C3262/C3262DDLHA NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S50732[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID7042 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50733[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; MID7042 Build/IML74K)";
const char S50734[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X+ Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50735[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC EVARE_UL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50736[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S50737[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S50738[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G with Beats Audio Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S50739[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Z710e Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S50740[] = "Opera/9.52 (X11; Linux i686; U; fr)";
const char S50741[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/24.0.1312.52) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17";
const char S50742[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; chromeframe/24.0.1312.57)";
const char S50743[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/24.0.1312.56) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17";
const char S50744[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/24.0.1312.57)";
const char S50745[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/24.0.1312.57)";
const char S50746[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/24.0.1312.57) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S50747[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; chromeframe/24.0.1312.57)";
const char S50748[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; NookColor Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50749[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Mobile Safari/535.19 Silk-Accelerated=true";
const char S50750[] = "Mozilla/5.0 (Series40; Nokia309/05.85; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S50751[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S50752[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S50753[] = "SAMSUNG-GT-S5233S/S5233SDDJJ1 SHP/VPP/R5 Jasmine/0.8 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S50754[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3113 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S50755[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3113 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S50756[] = "HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; fr)";
const char S50757[] = "HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; de)";
const char S50758[] = "HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; ru)";
const char S50759[] = "HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; zh-tw)";
const char S50760[] = "HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; nl)";
const char S50761[] = "HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; es-ES)";
const char S50762[] = "HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; da)";
const char S50763[] = "HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; el)";
const char S50764[] = "HTC_HD2_T8585 Opera/9.70 (Windows NT 5.1; U; de)";
const char S50765[] = "HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; pt)";
const char S50766[] = "HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; sv)";
const char S50767[] = "HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; nb)";
const char S50768[] = "HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; it)";
const char S50769[] = "HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; en)";
const char S50770[] = "SAMSUNG-GT-S5600-ORANGE/S5600BVIH3 SHP/VPP/R5 Jasmine/0.8 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S50771[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S50772[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S50773[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970C Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S50774[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970C Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S50775[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970X Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S50776[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970X Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S50777[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S50778[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SH80F Build/A8024) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50779[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SH-12C Build/S5068) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50780[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SH-12C Build/S5050) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50781[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SH-12C Build/S7140) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50782[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SH-12C Build/S9300) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50783[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SH-12C Build/S4040) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50784[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; SH80F Build/S9104) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50785[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SH80F Build/SA060) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50786[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SH-12C Build/S7140)";
const char S50787[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SH80F Build/RA060) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50788[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SH80F Build/R9102) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50789[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SH-12C Build/S7170) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50790[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SH-12C Build/S5050)";
const char S50791[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; GT-P7320T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50792[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nb-no; GT-P7320 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50793[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7320 Build/IMM76D)";
const char S50794[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7320T Build/IMM76D)";
const char S50795[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; GT-P7320 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.4.1-100 (Android 4.0.4; samsung GT-P7320 Build/IMM76D.XXLPI)";
const char S50796[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:23.0) Gecko/X Firefox/23.0 AlexaToolbar/alxf-2.18";
const char S50797[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A551 Safari/8536.25";
const char S50798[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A523 Safari/8536.25";
const char S50799[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A403 Safari/8536.25";
const char S50800[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A405 Safari/8536.25";
const char S50801[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A525 Safari/8536.25";
const char S50802[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26i Build/6.1.A.2.55) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S50803[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26i Build/6.1.A.2.45) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S50804[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S50805[] = "Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S50806[] = "Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5310xpressmusic";
const char S50807[] = "Nokia5310/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S50808[] = "Nokia5310_CMCC/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S50809[] = "Nokia5310XpressMusic_CMCC/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S50810[] = "SAMSUNG-GT-B5722/B5722DDJA1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S50811[] = "Opera/9.80 (Android 4.0.4; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00";
const char S50812[] = "Opera/9.80 (Android 4.0.3; Linux; Opera Tablet/ADR-X; U; de) Presto/2.10.254 Version/12.00";
const char S50813[] = "Opera/9.80 (Android 4.0.3 + Den Eds Update 2; Linux; Opera Tablet/ADR-X; U; nl) Presto/2.10.254 Version/12.00";
const char S50814[] = "Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.2.15 Version/10.10";
const char S50815[] = "Mozilla/5.0 (Windows NT 6.0; U; tr; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 10.10";
const char S50816[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.0; tr) Opera 10.10";
const char S50817[] = "Opera/10.1 (Windows NT 6.0; U; en)";
const char S50818[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ISW13HT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50819[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; HTC_J_Z321e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50820[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; ISW13HT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50821[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; HTC J Z321e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50822[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC J Z321e Build/IMM76D)";
const char S50823[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ISW13HT Build/IMM76D)";
const char S50824[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; ISW13HT Build/IML74K)";
const char S50825[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HUAWEI P6-U06 Build/HuaweiP6-U06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50826[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; P6-U06 Build/HuaweiP6-U06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50827[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; HUAWEI P6-U06 Build/HuaweiP6-U06)";
const char S50828[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HUAWEI P6-U06 Build/HuaweiP6-U06) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S50829[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; HUAWEI P6-U06-orange Build/HuaweiP6-U06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50830[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE-90-1/07.24.0.3; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S50831[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE90-1/400.34.93 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S50832[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9190 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S50833[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5F136 Safari/525.20";
const char S50834[] = "Mozilla/5.0 (iPhone; U; CPU iPhone 2_1 like Mac OS X; en) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile/5F136 Safari/525.20.1";
const char S50835[] = "Mozilla/5.0 (iPhone Simulator; U; CPU iPhone OS 2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5F135 Safari/525.20";
const char S50836[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5F136";
const char S50837[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 21 like Max OS X; en-us) AppleWebKit/525.18.1 (KHTML like Geko) BB2C/1.3.33 Version/3.1.1 Mobile/5F136 Safari/525.20";
const char S50838[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_1 like Mac OS X; es-US) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5F90 Safari/525.20";
const char S50839[] = "SAMSUNG-SGH-A687/A687UCJH1; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.16479; U; en-US) Opera 9.50";
const char S50840[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_PH39100/1.63.502.4; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S50841[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; VS910 4G Build/VS910ZV4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50842[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; VS910 4G Build/VS910ZV6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50843[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; VS910 4G Build/VS910ZV6)";
const char S50844[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; VS910 4G Build/VS910V6A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50845[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; VS910 4G Build/VS910ZV1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50846[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; calgary Build/DRC92) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S50847[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; calgary Build/DRC92) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S50848[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Mobile Safari/535.19 Silk-Accelerated=true";
const char S50849[] = "SAMSUNG-SGH-T359/T359UVJI2 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S50850[] = "Mozilla/5.0 (compatible; PRTG Network Monitor (www.paessler.com); Windows)";
const char S50851[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; LG-LU6200 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50852[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; LG-LU6200 Build/GRJ90)";
const char S50853[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LG-LU6200 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S50854[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LG-LU6200 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S50855[] = "Mozilla/4.0 (Brew MP 1.0.2; U; en-us; Sanyo; NetFront/3.5.1/AMB) Sprint SCP-6780";
const char S50856[] = "Mozilla/4.0 (Brew MP 1.0.2; U; en-us; Sanyo; NetFront/3.5.1/AMB) Boost SCP6780";
const char S50857[] = "Mozilla/5.0 (Windows NT 5.1; rv:15.0; Avant TriCore) Gecko/X Firefox/15.0.1";
const char S50858[] = "Mozilla/5.0 (Windows NT 5.1; rv:16.0; Avant TriCore) Gecko/X Firefox/16.0";
const char S50859[] = "Mozilla/5.0 (Windows NT 5.1; rv:17.0; Avant TriCore) Gecko/X Firefox/17.0";
const char S50860[] = "Mozilla/5.0 (Windows NT 5.1; rv:18.0; Avant TriCore) Gecko/X Firefox/18.0";
const char S50861[] = "Mozilla/5.0 (Windows NT 5.1; rv:19.0; Avant TriCore) Gecko/X Firefox/19.0";
const char S50862[] = "Mozilla/5.0 (Windows NT 5.1; rv:20.0; Avant TriCore) Gecko/X Firefox/20.0";
const char S50863[] = "Mozilla/5.0 (Windows NT 5.1; rv:21.0; Avant TriCore) Gecko/X Firefox/21.0";
const char S50864[] = "Mozilla/5.0 (Windows NT 5.1; rv:22.0; Avant TriCore) Gecko/X Firefox/22.0";
const char S50865[] = "Mozilla/5.0 (Windows NT 5.1; rv:23.0; Avant TriCore) Gecko/X Firefox/23.0";
const char S50866[] = "Mozilla/5.0 (Linux; Android 4.2.2; A1-810 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S50867[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Nexus S 4G Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50868[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Full Android on Crespo4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50869[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus S 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50870[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus S 4G Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50871[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S 4G Build/IMM26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50872[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus S 4G Build/IMM26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50873[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50874[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Nexus S 4G Build/IMM76D)";
const char S50875[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus S 4G Build/GWK74) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S50876[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50877[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S 4G Build/IMM76B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50878[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S 4G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50879[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Crespo4g Build/IMLM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50880[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S 4G Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50881[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Nexus S 4G Build/IML74K)";
const char S50882[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; Nexus S 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50883[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; LG-P690b Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S50884[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P690b Build/GRJ22)";
const char S50885[] = "BlackBerry8120/4.5.0.124 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100";
const char S50886[] = "Mozilla/5.0 (iPad; U; CPU OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9A405 Safari/7534.48.3";
const char S50887[] = "Mozilla/5.0 (iPad; U; CPU OS 5_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9A334 Safari/7534.48.3";
const char S50888[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-us; Micromax A89 Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S50889[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A89 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50890[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; A89 Build/IMM76D)";
const char S50891[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaC5-06/20.6.039; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.27 Mobile Safari/533.4 3gpp-gba";
const char S50892[] = "NokiaC5-06/22.6.007 (SymbianOS/9.4; U; Series60/5.0 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Safari/525 3gpp-gba";
const char S50893[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Xbox)";
const char S50894[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3 TeamStream (iPhone4,1 276 2.2)";
const char S50895[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3 TeamStream (iPhone5,1 279 2.2.1)";
const char S50896[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 TeamStream (iPhone4,1 293 2.2.2)";
const char S50897[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Safari/7534.48.3 TeamStream (iPhone5,1 293 2.2.2)";
const char S50898[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari/7534.48.3 TeamStream (iPhone5,1 306 2.3.1)";
const char S50899[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ONDA MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50900[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ONDA_MID Build/IML74K; 768*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S50901[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; ONDA MID Build/IML74K)";
const char S50902[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ONDA MID Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S50903[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ONDA MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50904[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ONDA MID Build/ICS.g12refM1006.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50905[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ONDA MID Build/ICS.g12refM708A2.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50906[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ONDA MID Build/ICS.g12refM704A1.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50907[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ONDA MID Build/ICS.g12refM703A1.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50908[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ONDA MID Build/ICS.g12refM1005.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50909[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ONDA MID Build/ICS.g12refM806A1SPI.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50910[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ONDA MID Build/ICS.g12refM806A2.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50911[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ONDA_MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50912[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ONDA_MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50913[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; ONDA MID Build/ICS.g12refM805.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50914[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; ONDA MID Build/ICS.g12refM708A1.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50915[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; HTC6990LVW)";
const char S50916[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50917[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190L Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S50918[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9A405 Safari/7534.48.3";
const char S50919[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9A334 Safari/7534.48.3";
const char S50920[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9A406 Safari/7534.48.3";
const char S50921[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S50922[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S50923[] = "Mozilla/5.0 (Windows NT 5.1; rv:22.0) Gecko/X Firefox/22.0 SeaMonkey/2.19";
const char S50924[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:22.0) Gecko/X Firefox/22.0 SeaMonkey/2.19";
const char S50925[] = "Mozilla/5.0 (Windows NT 5.1; rv:22.0) Gecko/X SeaMonkey/2.19";
const char S50926[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ZTE Grand X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50927[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ZTE U970 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S50928[] = "Mozilla/5.0 (Linux; U; Android 4.0.4;zh-cn; ZTE V970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50929[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE V970 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S50930[] = "Mozilla/5.0 (Linux; U; Android 4.0.4;es-es; ZTE V970M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50931[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; ZTE V970 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50932[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ZTE V970 Build/IMM76D)";
const char S50933[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Grand X(M) Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50934[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE V970 Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S50935[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; bg-bg; Grand X Pro Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50936[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; Grand X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50937[] = "SAMSUNG-SGH-A887/A887UCIJ1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S50938[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:20.0) Gecko/X Firefox/20.0 AlexaToolbar/alxf-2.18";
const char S50939[] = "Incredimail 1.2.1 rv:4897 (iPad; iPhone OS 6.1.3; en_US)";
const char S50940[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Incredimail/1.2.1.4897";
const char S50941[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Incredimail/1.2.1.4897";
const char S50942[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Incredimail/1.2.4.5649";
const char S50943[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5228/21.1.004; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.5.2 3gpp-gba";
const char S50944[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5228/51.1.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 Mobile Safari/533.4 3gpp-gba";
const char S50945[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; INQ Cloud Touch Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50946[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; INQ Cloud Touch -parrot Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50947[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; INQ Cloud Touch Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S50948[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; INQ Cloud Touch Build/FRG83)";
const char S50949[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; INQ Cloud Touch Build/FRG83G)";
const char S50950[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8010 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S50951[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SAMSUNG-SGH-T989 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50952[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-T989 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50953[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-T989 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/hercules";
const char S50954[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; ALCATEL_one_touch_995 Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50955[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ALCATEL_one_touch_995 Build/IceCreamSandwich)";
const char S50956[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ALCATEL_one_touch_995S Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50957[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ALCATEL_one_touch_995A Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50958[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS12S Build/6.1.D.0.170) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50959[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS12S Build/6.1.D.1.74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50960[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonyEricssonIS12S Build/6.1.D.1.91) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50961[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS12S Build/6.1.D.1.91) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50962[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS12S Build/6.1.D.1.103) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50963[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; IS12S Build/6.1.D.1.103)";
const char S50964[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; PadFone Build/JRO03L)";
const char S50965[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PadFone Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50966[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PadFone Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50967[] = "MobileRSSFree-iPad/3.1.4 CFNetwork/548.1.4 Darwin/11.0.0";
const char S50968[] = "MobileRSS-iPad/3.1.4 CFNetwork/548.1.4 Darwin/11.0.0";
const char S50969[] = "browseripad/32992 CFNetwork/548.1.4 Darwin/11.0.0";
const char S50970[] = "CitrixReceiver-iPad/166 CFNetwork/548.1.4 Darwin/11.0.0";
const char S50971[] = "Perfect%20Browser-iPad/6.0.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S50972[] = "Dolphin%20Browser%20for%20iPad/5.4.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S50973[] = "MSN%20for%20iPad/1.0.2 CFNetwork/548.0.4 Darwin/11.0.0";
const char S50974[] = "YandexSearch-iPad/474 CFNetwork/548.1.4 Darwin/11.0.0";
const char S50975[] = "WordsWithFriendsFreeiPad/4.12 CFNetwork/485.13.9 Darwin/11.0.0";
const char S50976[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; SlimBrowser/7.00.035)";
const char S50977[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; SlimBrowser/7.00.035)";
const char S50978[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Avant Browser; SlimBrowser/7.00.035)";
const char S50979[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X Firefox/15.0.1 SeaMonkey/2.12.1";
const char S50980[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X SeaMonkey/2.12";
const char S50981[] = "Mozilla/5.0 (Linux; Android 4.1.1; IdeaTabA2109A Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S50982[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Torque Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S50983[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Torque Build/IMM76I)";
const char S50984[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100G Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S50985[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S50986[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S50987[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S50988[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100G Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S50989[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.1.140) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S50990[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.1.140) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S50991[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xperia T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S50992[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; GT-P6200 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50993[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P6200 Build/JZO54K)";
const char S50994[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6200 Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50995[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P6200 Build/JRO03R)";
const char S50996[] = "Logos/2.2.8.1 (Linux; U; Android 4.1.2; en-GB; GT-P6200 Build/cm_p6200-userdebug 4.1.2 JZO54K eng.aorth.X.X test-keys)";
const char S50997[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; GT-P6200 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50998[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P6200 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S50999[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; GT-P6200L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51000[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P6200L Build/JZO54K)";
const char S51001[] = "Mozilla/5.0 (Linux; Android 4.0.4; N9100 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51002[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S51003[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S51004[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S51005[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P3100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S51006[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SAMSUNG GT-S7560/S7560XXAME9 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51007[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7560 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51008[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-S7560 Build/IMM76I)";
const char S51009[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-S7560-ORANGE/S7560XXAMG7 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51010[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; SAMSUNG GT-S7560/S7560XXAMGF Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51011[] = "Opera/9.80 (Windows NT 6.0; U; en) Presto/2.9.168 Version/11.51";
const char S51012[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.9.168 Version/11.51";
const char S51013[] = "Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.9.168 Version/11.51,gzip(gfe) (via translate.google.com)";
const char S51014[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.9.168 Version/11.51";
const char S51015[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.9.168 Version/11.51";
const char S51016[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4598); ru) Presto/2.9.168 Version/11.51";
const char S51017[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.6 (build 03278); ru) Presto/2.9.168 Version/11.51";
const char S51018[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4664); ru) Presto/2.9.168 Version/11.51";
const char S51019[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.4 (build 02647); ru) Presto/2.9.168 Version/11.51";
const char S51020[] = "Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03686); ru) Presto/2.9.168 Version/11.51";
const char S51021[] = "Opera/9.80 (X11; Linux x86_64; Edition Linux Mint) Presto/2.12.388 Version/12.15";
const char S51022[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaC5-00.2) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S51023[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac5-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile  3gpp-gba";
const char S51024[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaC5-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S51025[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac5-00.2) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile  3gpp-gba";
const char S51026[] = "Mozilla/5.0 (Linux; Android 4.1.1; Orange Daytona Build/HuaweiG510-0200) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51027[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0200 Build/HuaweiG510-0200) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51028[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0100 Build/HuaweiG510-0100) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51029[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51030[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0251 Build/HuaweiG510-0251) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51031[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105P Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51032[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51033[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.7)";
const char S51034[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.6)";
const char S51035[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.7)";
const char S51036[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.7)";
const char S51037[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.9)";
const char S51038[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.10)";
const char S51039[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.9)";
const char S51040[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.9)";
const char S51041[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.8)";
const char S51042[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.11)";
const char S51043[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.12)";
const char S51044[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.14)";
const char S51045[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.3)";
const char S51046[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; Sleipnir/2.9.8)";
const char S51047[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; Sleipnir/2.9.3)";
const char S51048[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 2.0.50727; Sleipnir/2.9.2)";
const char S51049[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.15)";
const char S51050[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; InfoPath.1; Sleipnir/2.9.4)";
const char S51051[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322; Sleipnir/2.9.2)";
const char S51052[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB0.0; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 1.1.4322; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; OfficeLiveConnector.1.3; OfficeLivePatch.0.0; Sleipnir/2.9.6)";
const char S51053[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.16)";
const char S51054[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.17)";
const char S51055[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.18)";
const char S51056[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF700T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S51057[] = "Mozilla/5.0 (Linux; Android 4.2.2; ASUS Transformer Pad TF700T Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S51058[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; GT-I9000T Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51059[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I9000T Build/ECLAIR)";
const char S51060[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2) Presto/2.12.388 Version/12.12";
const char S51061[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, gb";
const char S51062[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:38, us";
const char S51063[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I535 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, -0400, us";
const char S51064[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, +0100, gb";
const char S51065[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-T999 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2tmo";
const char S51066[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9300 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i9300";
const char S51067[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I535 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2vzw";
const char S51068[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-gb; GT-I9300 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i9300";
const char S51069[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-T999 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2tmo";
const char S51070[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I535 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2vzw";
const char S51071[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I535 Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2vzw";
const char S51072[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I535 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2vzw";
const char S51073[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-T999 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2tmo";
const char S51074[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9300 Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i9300";
const char S51075[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-T999 Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2tmo";
const char S51076[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-gb; GT-I9300 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i9300";
const char S51077[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN7DG3ST-CP Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51078[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN7DG3-CP Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51079[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; AN7DG3-LZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51080[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN7DG3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51081[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN7DG3B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51082[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S51083[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC6435LVW Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S51084[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-ca; SGH-I337M Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51085[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGH-I337M Build/JDQ39)";
const char S51086[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-; SGH-I337M Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51087[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT300 Build/SESGC_U3_00.35.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51088[] = "MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; XT300 Build/SESLA_U3_01.71.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51089[] = "MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; XT300 Build/SESLA_U3_01.71.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51090[] = "MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; XT300 Build/SESLA_U3_01.71.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51091[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT300 Build/SESGC_U3_10.10.0_RT) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51092[] = "MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; XT300 Build/SESLA_U3_01.72.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51093[] = "MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT300 Build/SESLA_U3_01.49.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51094[] = "MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; XT300 Build/SESLA_U3_01.44.5) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51095[] = "MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; XT300 Build/SESLA_U3_01.44.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51096[] = "MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; XT300 Build/SESLA_U3_01.72.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51097[] = "MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT300 Build/SESLA_U3_01.60.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51098[] = "MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; XT300 Build/SESLA_U3_01.44.4) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51099[] = "MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; XT300 Build/SESLA_U3_01.53.1_R01) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51100[] = "MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; XT300 Build/SESLA_U3_01.71.3) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51101[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; XT300 Build/SESLA_U3_01.24.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51102[] = "Dalvik/1.2.0 (Linux; U; Android 2.1-update1; XT300 Build/SESLA_U3_01.71.0)";
const char S51103[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT300 Build/SESGC_U3_10.10.0_CU) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51104[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT300 Build/SESGC_U3_00.35.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S51105[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; IDEOS S7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51106[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-au; S7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 S7V100R001C13B119";
const char S51107[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; S7 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 S7V100R001C13B115";
const char S51108[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; IDEOS S7 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51109[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; IDEOS S7 Build/FRG83G)";
const char S51110[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ru-ru; Oysters SmaKit S7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51111[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; SGH-I896 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51112[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SGH-I896 Build/GINGERBREAD)";
const char S51113[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P5110 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S51114[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P5110 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S51115[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9305 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/135,1.9.6.135,2012-08-29 19:33, gb";
const char S51116[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:38, us";
const char S51117[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:38, us";
const char S51118[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-I747M Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, -0400, ca";
const char S51119[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, -0400, us";
const char S51120[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9305 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, +0100, gb";
const char S51121[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530U Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, -0500, us";
const char S51122[] = "NokiaN85/SymbianOS/9.3 Series60/3.2 Release/31.002; Mozilla/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML,like Gecko) Safari/413";
const char S51123[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 NokiaN85-3/11.047; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S51124[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 NokiaN85-1/31.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S51125[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; MD_LIFETAB_P9516 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51126[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; MD_LIFETAB_P9516 Build/IML74K)";
const char S51127[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGP321 Build/10.1.1.A.1.253) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S51128[] = "SAMSUNG-GT-S3370C/1.0 SHP/VPP/R5 Dolfin/1.5 Qtv/5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-N";
const char S51129[] = "SAMSUNG-GT-S3370L/1.0 SHP/VPP/R5 Dolfin/1.5 Qtv/5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-N";
const char S51130[] = "SAMSUNG-GT-S3370B/S3370DO SHP/VPP/R5 Dolfin/1.5 Qtv/5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-N";
const char S51131[] = "SAMSUNG-GT-S3370/S3370DDJE1 SHP/VPP/R5 Dolfin/1.5 Qtv/5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-N";
const char S51132[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R970 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51133[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SCH-R970 Build/JDQ39)";
const char S51134[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R970C Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51135[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SCH-R970C Build/JDQ39)";
const char S51136[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R970X Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51137[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SCH-R970X Build/JDQ39)";
const char S51138[] = "Opera/9.80 (BlackBerry; Opera Mini/4.5.33868/31.1475; U; en) Presto/2.8.119 Version/11.10";
const char S51139[] = "Mozilla/5.0 (Linux; Android 4.2.1; ME301T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S51140[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XOLO A1000 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51141[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; IdeaTabA1000-F Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51142[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; IdeaTabA1000-T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51143[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; IdeaTabA1000-F Build/JZO54K)";
const char S51144[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; IdeaTabA1000-G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51145[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; IdeaTabA1000-G Build/JZO54K)";
const char S51146[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S51147[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S51148[] = "Mozilla/5.0 (Linux; Android 4.2.1; ME173X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S51149[] = "Mozilla/5.0 (Linux; Android 4.2.2; ME173X Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S51150[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-04E Build/10.1.1.D.0.179) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51151[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-04E Build/10.1.1.D.0.179) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51152[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-04E Build/10.1.1.D.2.26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51153[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-04E Build/10.1.1.D.2.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51154[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SO-04E Build/10.1.1.D.2.31)";
const char S51155[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-04E Build/10.1.1.D.2.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51156[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-04E Build/10.1.1.D.2.26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51157[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows 95)";
const char S51158[] = "Mozilla/4.0 (compatible; MSIE 4.0; Windows 95)";
const char S51159[] = "Mozilla/5.0 (compatible; MSIE 4.0; Win95; Trident/1.0)";
const char S51160[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A800S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51161[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-A800S Build/IMM76I)";
const char S51162[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A800S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51163[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A800S Build/A800S 216 V6.5 VH by atulaphan) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51164[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IM-A800S Build/A800S 221 v6.7 by atulaphan) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51165[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A800S Build/PHIEN BAN GALAXY S A800S BY HANZOO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51166[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IM-A800S Build/A800S 216 V6.4 by atulaphan) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51167[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IM-A800S Build/A800S 222 v6.8 by atulaphan) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51168[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A800S Build/VEGAVIET.COM ROM ICS V2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S51169[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A800S Build/VEGAVIET.COM ROM ICS V1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S51170[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A800S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51171[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A800S Build/www.tkvietnam.com) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51172[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IM-A800S Build/A800S 219 V6.6 by atulaphanI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51173[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A800S Build/wwww.avio.vn) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S51174[] = "Mozilla/5.0 (PlayStation Vita 2.12) AppleWebKit/536.26 (KHTML, like Gecko) Silk/3.2";
const char S51175[] = "SAMSUNG-GT-E2652/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S51176[] = "SAMSUNG-GT-E2652W/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S51177[] = "SAMSUNG-GT-E2652/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1, SAMSUNG-GT-E2652/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S51178[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A820L Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51179[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S51180[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-A820L Build/IMM76I)";
const char S51181[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A820L Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51182[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/HOANGMINHHUU) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S51183[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/Vegaviet.com) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S51184[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/di?n) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S51185[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IM-A820L Build/HOANGMINHHUU) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51186[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/Versions Rom Galaxy S.Production by Hanzoo1504) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51187[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/Super Rom Optimus HD Pro. Production By Hanzoo1054) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51188[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/MINHHAI-A820L-VEGAVIET) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S51189[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/ChoiMobile.vn) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51190[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/tungon_ blackberry@yahoo.com) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S51191[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/Tkvietnam) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51192[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6603 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S51193[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6603 Build/10.3.A.0.423) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S51194[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT25i Build/9.0.D.0.235) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51195[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT25i Build/9.0.D.0.219) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51196[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT25i Build/9.0.D.0.484) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51197[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT25i Build/9.0.1.D.0.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51198[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT25i Build/9.0.D.0.484)";
const char S51199[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; LT25i Build/9.0.1.D.0.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51200[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SonyLT25i Build/9.0.D.0.457) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51201[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; LT25i Build/9.0.D.0.484) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51202[] = "BlackBerry8350i/5.0.0.1051 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/103";
const char S51203[] = "Logos/0.9.3 (Linux; U; Android 2.3.3; en_US; MID8127 Build/MID8127 X)";
const char S51204[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID8127 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51205[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID8127 Build/GRI40)";
const char S51206[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I5801 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51207[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I5801 Build/ECLAIR)";
const char S51208[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I5801 Build/ECLAIR) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S51209[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; GT-I5801 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51210[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Dell Venue Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51211[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Dell Venue Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51212[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; Dell Venue Build/FRF91)";
const char S51213[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; Dell Venue Build/FRG83G)";
const char S51214[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I915 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51215[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I915 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51216[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-I915 Build/IMM76D)";
const char S51217[] = "HUAWEI-M635/001.00 WAP/OBIGO/Q05A";
const char S51218[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51219[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; A210 Build/IMM76D)";
const char S51220[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo_A2105 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51221[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S51222[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S51223[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Z710e Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S51224[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G with Beats Audio Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S51225[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S51226[] = "Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S51227[] = "Mozilla/5.0 (Linux; Android 4.1.1; Galaxy X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S51228[] = "Mozilla/5.0 (Linux; Android 4.1.1; Galaxy Nexus Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S51229[] = "Mozilla/5.0 (Linux; Android 4.1; Galaxy Nexus Build/JRN84D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S51230[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; LePanII Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S51231[] = "Dalvik/151 (Linux; U; Android 3.2.1; LePanII Build/HTK75)";
const char S51232[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I727 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51233[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I727 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51234[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I727 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51235[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I727 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51236[] = "Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S51237[] = "Mozilla/5.0 (Linux; Android 4.1.2; EVO Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S51238[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT925 Build/7.7.1Q-144_VQL_S3-58) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S51239[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR HD Build/7.7.1Q-144_VQW_S334-25) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S51240[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT925 Build/7.7.1Q-144_VQL_S3-58) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S51241[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR HD Build/7.7.1Q-144_VQW_S334-25) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S51242[] = "Mozilla/5.0 (Googlebot compatible; Dayviewbot/2.0; http://www.dayview.com/)";
const char S51243[] = "Mozilla/5.0 (compatible; Googlebot/2.1; http://www.semetrical.com/bot.html)";
const char S51244[] = "Googlebot/2.1 (http://www.googlebot.com/bot.html);c2VvLWNyYXdsZXI=";
const char S51245[] = "Mozilla/5.0 (compatible; GoogleApps script; +http://script.google.com/bot.html)";
const char S51246[] = "Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html) (support@google.com)";
const char S51247[] = "Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html) support@google.com";
const char S51248[] = "* Googlebot/2.1 (+http://www.googlebot.com/bot.html)";
const char S51249[] = "Mozilla/5.0 (compatible; Googlebot/2.1";
const char S51250[] = "Googlebot/2.1; +http://www.google.com/bot.html";
const char S51251[] = "Mozilla/5.0 (compatible; Googlebot/2.1; +http://import.io)";
const char S51252[] = "Googlebot-News (2.3.3, ruby 1.9.3 (2013-02-22))";
const char S51253[] = "Opera/9.24 (Windows NT 6.1; U; en)";
const char S51254[] = "Opera/9.26 (Windows NT 6.1; U; en)";
const char S51255[] = "Opera/9.27 (Windows NT 6.1; U; en)";
const char S51256[] = "Opera/9.25 (Windows NT 6.1; U; en)";
const char S51257[] = "Opera/9.20 (Windows NT 6.1; U; en)";
const char S51258[] = "Opera/9.23 (Windows NT 6.1; U; en)";
const char S51259[] = "Opera/9.21 (Windows NT 6.1; U; en)";
const char S51260[] = "Opera/9.22 (Windows NT 6.1; U; en)";
const char S51261[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11";
const char S51262[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11";
const char S51263[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11";
const char S51264[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-T989 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51265[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-T989 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51266[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-T989 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51267[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-T989 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51268[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51269[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51270[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; A700 Build/IMM76D)";
const char S51271[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MA700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51272[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-T989 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51273[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T989 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51274[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51275[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T989 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51276[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989D Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51277[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P7501 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51278[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7501 Build/IMM76D)";
const char S51279[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P7501 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51280[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-; GT-P7501 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51281[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SAMSUNG GT-P7501/P7501XXLPD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51282[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC first Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S51283[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9A405 Safari/7534.48.3";
const char S51284[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9A406 Safari/7534.48.3";
const char S51285[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9A334 Safari/7534.48.3";
const char S51286[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XOOM 2 ME Build/7.7.1-128_MZ607-14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51287[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XOOM 2 ME Build/7.7.1-85_MZ607-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51288[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ607-14)";
const char S51289[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.2.1 (like Version/10B329 Mobile/6.1.3 Safari/8536.25)";
const char S51290[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.2.1 (like Version/10B141 Mobile/6.1 Safari/8536.25)";
const char S51291[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.2.1 (like Version/10B146 Mobile/6.1.2 Safari/8536.25)";
const char S51292[] = "Mozilla/5.0 (Linux; Android 4.0.3;  HTC Sensation Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S51293[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC-Z710a Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51294[] = "Mozilla/5.0 (Linux; Android 4.0.3; Sensation Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51295[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710a Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51296[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51297[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51298[] = "Mozilla/5.0 (Linux; Android 4.0.4; Sensation Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51299[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51300[] = "Mozilla/5.0 (Series40; Nokia501/10.0.2; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.0.0.0.73";
const char S51301[] = "Mozilla/5.0 (Series40; Nokia501s/10.0.12; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.0.0.0.75";
const char S51302[] = "Mozilla/5.0 (PLAYSTATION 3 4.25) AppleWebKit/531.22.8 (KHTML, like Gecko)";
const char S51303[] = "NokiaC5-03/UCWEB8.7.0.187/50/999";
const char S51304[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaC5-03) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S51305[] = "NokiaC5-03UCBrowser/8.7.1.234/50/444/UCWEB";
const char S51306[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; NookTablet Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51307[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Barnes & Noble Nook Tablet Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51308[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; NookTablet Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51309[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; NookTablet Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51310[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Barnes & Noble Nook Tablet Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51311[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Barnes & Noble Nook Tablet Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51312[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-CA; Barnes & Noble Nook Tablet Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51313[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; NookTablet Build/JDQ39E; CyanogenMod-acclaim_10.1_21APR2013_HD_SDC) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51314[] = "ASTRO36_TD/v3 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10";
const char S51315[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MOT-XT303 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51316[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT303 Build/07.17.03R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51317[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT303 Build/07.1E.00R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51318[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT303 Build/07.22.00R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51319[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT303 Build/07.17.04R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51320[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; XT303 Build/07.17.03R_S)";
const char S51321[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT303 Build/07.1B.05R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51322[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT303 Build/07.1F.01R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51323[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S51324[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S51325[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T989 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S51326[] = "Mozilla/5.0 (Linux; Android 4.2.1; en-us; SAMSUNG GT-I9200 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0.X Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S51327[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG GT-I9200 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S51328[] = "SAMSUNG-GT-E2220/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S51329[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Amazon Kindle Fire Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51330[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Amazon Kindle Fire Build/JRO03C)";
const char S51331[] = "Logos/2.0.2 (Linux; U; Android 4.1.1; en-US; Amazon Kindle Fire Build/full_otter-userdebug 4.1.1 JRO03C eng.hashcode.X.X test-keys)";
const char S51332[] = "Logos/1.1.3 (Linux; U; Android 4.1.1; en-US; Amazon Kindle Fire Build/cm_otter-userdebug 4.1.1 JRO03C eng.twagner.X.X test-keys)";
const char S51333[] = "Logos/1.1.3 (Linux; U; Android 4.1.1; en-US; Amazon Kindle Fire Build/full_otter-userdebug 4.1.1 JRO03C eng.0xD34D.X.X test-keys)";
const char S51334[] = "Logos/1.1.3 (Linux; U; Android 4.1.1; en-US; Amazon Kindle Fire Build/cm_otter-userdebug 4.1.1 JRO03H eng.twagner.X.X test-keys)";
const char S51335[] = "Logos/2.0.8 (Linux; U; Android 4.1.1; en-US; Amazon Kindle Fire Build/cm_otter-userdebug 4.1.1 JRO03L eng.twagner.X.X test-keys)";
const char S51336[] = "Logos/2.0.11 (Linux; U; Android 4.1.1; en-US; Amazon Kindle Fire Build/cm_otter-userdebug 4.1.1 JRO03L eng.hashcode.X.X test-keys)";
const char S51337[] = "Logos/2.1.2 (Linux; U; Android 4.1.1; en-US; Amazon Kindle Fire Build/cm_otter-userdebug 4.1.1 JRO03R eng.build.X.X test-keys)";
const char S51338[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Amazon Kindle Fire Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S51339[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Amazon Kindle Fire Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51340[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Amazon Kindle Fire Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S51341[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Amazon Kindle Fire Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51342[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Amazon Kindle Fire Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51343[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Amazon Kindle Fire Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S51344[] = "FaithlifeStudyBible/2.0.11 (Linux; U; Android 4.1.1; en-US; Amazon Kindle Fire Build/cm_otter-userdebug 4.1.1 JRO03L eng.build.X.X test-keys)";
const char S51345[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Amazon Kindle Fire Build/JZO54K)";
const char S51346[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A523 Safari/8536.25";
const char S51347[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A403 Safari/8536.25";
const char S51348[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A406 Safari/8536.25";
const char S51349[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS870 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S51350[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.0 Safari/533.3";
const char S51351[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHW-M440S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51352[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHW-M440S Build/IMM76D)";
const char S51353[] = "SonyEricssonC510/R1HA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.5";
const char S51354[] = "SonyEricssonC510a/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3";
const char S51355[] = "Mozilla/4.0 SonyEricssonC510v/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3";
const char S51356[] = "Mozilla/4.0 SonyEricssonC510av/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3";
const char S51357[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; MS-RTC LM 8)";
const char S51358[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S51359[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; Zune 3.0)";
const char S51360[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)";
const char S51361[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; .NET4.0E)";
const char S51362[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S51363[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ617 4G Build/1.6.0M-218.14.Demo2-MZ617) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S51364[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ617 Build/1.6.0M-218.14-MZ617) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S51365[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ617 4G Build/1.6.0_190.10-MZ617) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S51366[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; MZ617 4G Build/1.6.0M_279_MZ617) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S51367[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ617 4G Build/1.6.0M-218.14-MZ617) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S51368[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; MZ617 Build/1.6.0M_279_MZ617) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S51369[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.2; MZ617 Build/1.6.0M_279_MZ617)";
const char S51370[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; MZ617 4G Build/1.6.0M_280_MZ617) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S51371[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HTC-A6366/1.0 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51372[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-au; HTC Liberty Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51373[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; HTC-A6366/1.0 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51374[] = "Mozilla/5.0 (Linux; U; Android2.2.2; en-us; HTC-A6366/1.0 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51375[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; HTC Liberty Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51376[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Liberty Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51377[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Liberty Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51378[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; HTC Liberty Build/FRF91)";
const char S51379[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.32) Gecko/X Firefox/3.6.32 (Palemoon/3.6.32)";
const char S51380[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2) Gecko/X Palemoon/3.6.0.5";
const char S51381[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.2.32) Gecko/X Firefox/3.6.32 (Palemoon/3.6.32)";
const char S51382[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/23.0.1271.64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S51383[] = "Mozilla/5.0 (Windows NT 6.2; chromeframe/23.0.1271.64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11";
const char S51384[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/23.0.1271.95) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11";
const char S51385[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/23.0.1271.97)";
const char S51386[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; chromeframe/23.0.1271.97)";
const char S51387[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; chromeframe/23.0.1271.97)";
const char S51388[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/23.0.1271.97) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S51389[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/23.0.1271.97)";
const char S51390[] = "Mozilla/5.0 (Windows NT 6.2; chromeframe/23.0.1271.97) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S51391[] = "Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S51392[] = "Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S51393[] = "Mozilla/5.0 (FreeBSD; U; Viera; en-GB) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.0.4 Chrome/23.0.1271.97 Safari/537.11";
const char S51394[] = "Mozilla/5.0 (FreeBSD; U; Viera; pl-PL) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.0.4 Chrome/23.0.1271.97 Safari/537.11";
const char S51395[] = "Mozilla/5.0 (FreeBSD; U; Viera; de-DE) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.0.4 Chrome/23.0.1271.97 Safari/537.11";
const char S51396[] = "Mozilla/5.0 (FreeBSD; U; Viera; nl-NL) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.0.4 Chrome/23.0.1271.97 Safari/537.11";
const char S51397[] = "Mozilla/5.0 (FreeBSD; U; Viera; en-AU) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.0.4 Chrome/23.0.1271.97 Safari/537.11";
const char S51398[] = "Mozilla/5.0 (FreeBSD; U; Viera; it-IT) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.0.4 Chrome/23.0.1271.97 Safari/537.11";
const char S51399[] = "Mozilla/5.0 (FreeBSD; U; Viera; fr-FR) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.0.4 Chrome/23.0.1271.97 Safari/537.11";
const char S51400[] = "Mozilla/5.0 (FreeBSD; U; Viera; es-ES) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.0.4 Chrome/23.0.1271.97 Safari/537.11";
const char S51401[] = "Mozilla/5.0 (FreeBSD; U; Viera; en-US) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11";
const char S51402[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; en-AU) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11";
const char S51403[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; en-US) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11";
const char S51404[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; en-GB) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11";
const char S51405[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; de-DE) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11";
const char S51406[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; fr-FR) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11";
const char S51407[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; it-IT) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11";
const char S51408[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; nl-NL) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11";
const char S51409[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; es-ES) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11";
const char S51410[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; en-NZ) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11";
const char S51411[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; pl-PL) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11";
const char S51412[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; sv-SE) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11";
const char S51413[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; da-DK) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11";
const char S51414[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; cs-CZ) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11";
const char S51415[] = "Mozilla/5.0 (X11; FreeBSD; U; Viera; en-IE) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11";
const char S51416[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; nl-be; San Francisco Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51417[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-pt; Optimus San Francisco Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51418[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; BASE lutea Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51419[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Orange San Francisco Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51420[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Orange San Francisco Build/ERE27)";
const char S51421[] = "Mozilla/5.0 (Series40; Nokia206/03.59; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S51422[] = "Mozilla/5.0 (Series40; Nokia206.1/03.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S51423[] = "SAMSUNG-C3053/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S51424[] = "SAMSUNG-C3050/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S51425[] = "SAMSUNG-C3050-ORANGE/C3050BVIH2 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S51426[] = "SAMSUNG-C3050/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S51427[] = "SAMSUNG-GT-C3050C/1.0 Release/1.19.2009 Browser/Openwave6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S51428[] = "SAMSUNG-C3053/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S51429[] = "Vodafone/1.0/SamsungC3050/C3050BUIC6 Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S51430[] = "Vodafone/1.0/SamsungC3050/C3050BUII1 Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S51431[] = "Vodafone/1.0/SamsungC3050/C3050BUJI1 Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S51432[] = "Vodafone/1.0/SamsungC3050/C3050BUIJ1 Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S51433[] = "Vodafone/1.0/SamsungC3050/C3050BUKC1 Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S51434[] = "Vodafone/1.0/SamsungC3050/C3050BUIG1 Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S51435[] = "Vodafone/1.0/SamsungC3050/C3050BUJK1 Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S51436[] = "Vodafone/1.0/SamsungC3050/C3050BUIF3 Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S51437[] = "Vodafone/1.0/SamsungC3050/C3050BUID2 Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S51438[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S51439[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 620)";
const char S51440[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 620; Vodafone)";
const char S51441[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 620; ANZ846)";
const char S51442[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 620; Meteor)";
const char S51443[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI MT1-U06 Build/HuaweiMT1-U06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51444[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HUAWEI MT1-U06 Build/HuaweiMT1-U06)";
const char S51445[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HUAWEI MT1-U06 Build/HuaweiMT1-U06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51446[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HUAWEI MT1-U06 Build/HuaweiMT1-U06C02B105) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51447[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LG-E971 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51448[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E971 Build/JZO54K)";
const char S51449[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51450[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830C Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51451[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/4.1.0)";
const char S51452[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/4.1.0)";
const char S51453[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/4.1.2)";
const char S51454[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/4.1.2)";
const char S51455[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 Sleipnir/4.1.3";
const char S51456[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 Sleipnir/4.1.3";
const char S51457[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/4.1.3)";
const char S51458[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/4.1.3)";
const char S51459[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36 Sleipnir/4.1.4";
const char S51460[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36 Sleipnir/4.1.4";
const char S51461[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/4.1.4)";
const char S51462[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/4.1.4)";
const char S51463[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.16) Gecko/X Ubuntu/8.04 (hardy) Firefox/3.6.16";
const char S51464[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51465[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Karbonn A2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51466[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; A2 Build/GRK39F)";
const char S51467[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; Karbonn A2 Build/GRK39F)";
const char S51468[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Callbar A2 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51469[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A4372q Safari/8536.25";
const char S51470[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A4400f Safari/8536.25";
const char S51471[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11A4414e Safari/8536.25";
const char S51472[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/11A4435d Safari/8536.25";
const char S51473[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; LG-C555-parrot Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S51474[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; LG-C555 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S51475[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-C555 Build/FRG83)";
const char S51476[] = "Facebook/4110.0 CFNetwork/609 Darwin/13.0.0";
const char S51477[] = "Tumblr/117 CFNetwork/609.1.4 Darwin/13.0.0";
const char S51478[] = "Mozilla/5.0 (Linux; Android 4.0.3;  LT18i Build/4.1.A.0.562) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S51479[] = "Mozilla/5.0 (Linux; Android 4.0.3; LT18i Build/4.1.A.0.562) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51480[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT18i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51481[] = "Mozilla/5.0 (Linux; Android 4.0.3; LT18a Build/4.1.A.0.562) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51482[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT18a Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51483[] = "Mozilla/5.0 (Linux; Android 4.0.4;  LT18i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S51484[] = "Mozilla/5.0 (Linux; Android 4.1.1; HP Slate 7 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S51485[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B329 Safari/8536.25";
const char S51486[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B144 Safari/8536.25";
const char S51487[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B146 Safari/8536.25";
const char S51488[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B141 Safari/8536.25";
const char S51489[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SPH-M900 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51490[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SPH-M900 Build/ECLAIR) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S51491[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SPH-M900 Build/ECLAIR)";
const char S51492[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_0; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0";
const char S51493[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_0; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0";
const char S51494[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_1; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0";
const char S51495[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_2; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0";
const char S51496[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_2; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0";
const char S51497[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_2; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0";
const char S51498[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_1; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0";
const char S51499[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_2; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0";
const char S51500[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_3; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0";
const char S51501[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_3; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0";
const char S51502[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_3; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0";
const char S51503[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_3; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0";
const char S51504[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_4; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0";
const char S51505[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_2; en-US) AppleWebKit/531.21.8+(KHTML, like Gecko, Safari/528.16) Version/5.10.3 OmniWeb/622.14.0";
const char S51506[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_4; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0";
const char S51507[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_4; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0";
const char S51508[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_4; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0";
const char S51509[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_5; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0";
const char S51510[] = "Mozilla/5.0 (Linux; Android 4.0.3; Le Pan S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S51511[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:19.0) Gecko/X Firefox/19.0 PaleMoon/19.0.2";
const char S51512[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:19.0) Gecko/X Firefox/19.0 PaleMoon/19.0.2-x64";
const char S51513[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; E350 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51514[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; E350 Build/IML74K)";
const char S51515[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; E350 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51516[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-; E350 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51517[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; E350 Build/IMM76D)";
const char S51518[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; 909)";
const char S51519[] = "Mozilla/5.0 (Windows NT 4.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S51520[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S51521[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S51522[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-L710 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S51523[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I747M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S51524[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S51525[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S51526[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530U Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S51527[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S51528[] = "Pixray-Seeker/2.0 (Pixray-Seeker; http://www.pixray.com/pixraybot; crawler@pixray.com)";
const char S51529[] = "Pixray-Seeker/2.0 (Pixray-Seeker; +http://www.pixray.com/pixraybot; +crawler@pixray.com)";
const char S51530[] = "Pixray-Seeker/2.0 (http://www.pixray.com/pixraybot; crawler@pixray.com)";
const char S51531[] = "Pixray-Seeker/2.0 (http://www.pixray.com/pixraybot; +crawler@pixray.com)";
const char S51532[] = "NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac2-03) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S51533[] = "LG-C395/V10e Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S51534[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S51535[] = "1PasswordThumbs/1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro10%2C1)";
const char S51536[] = "1PasswordThumbs/1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookAir5%2C2)";
const char S51537[] = "1PasswordThumbs/1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (iMac12%2C2)";
const char S51538[] = "1PasswordThumbs/1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (Macmini6%2C2)";
const char S51539[] = "1PasswordThumbs/1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacPro5%2C1)";
const char S51540[] = "1PasswordThumbs/1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBook5%2C1)";
const char S51541[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S51542[] = "BlackBerry9500/4.7.0.181 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/120";
const char S51543[] = "ALCATEL-OT871A/980 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S51544[] = "ALCATEL-OT871A/984 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S51545[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A840S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51546[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-A840S Build/IMM76I)";
const char S51547[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A840S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51548[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A840S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S51549[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A840S Build/VEGAVIET.COM ROM ICS V4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S51550[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A840S Build/VEGAVIET.COM ROM ICS V5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S51551[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IM-A840S Build/VEGAVIET.COM ROM ICS V4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51552[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A840S Build/VEGAVIET.COM ROM ICS V2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S51553[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A840S Build/VEGAVIET.COM ROM ICS V6.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?i Safari/534.30";
const char S51554[] = "Mozilla/5.0 (Series40; Nokia110/03.33; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S51555[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT28i Build/6.1.E.2.68) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S51556[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT28h Build/6.1.E.3.7) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S51557[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT28at Build/6.1.C.1.111) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S51558[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S51559[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500R Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S51560[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S51561[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LS970 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S51562[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S51563[] = "Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 8.0; Windows NT 5.0) Chrome/22.0.1229.79";
const char S51564[] = "Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 7.0; Windows NT 5.0) Chrome/22.0.1229.79";
const char S51565[] = "Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 9.0; Windows NT 5.0WOW64) Chrome/22.0.1229.79";
const char S51566[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:20.0) Gecko/X Firefox/20.0 IceDragon/20.0.1.14";
const char S51567[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.1b3) Gecko/X Firefox/3.1b3 GTB5";
const char S51568[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S51569[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8815 Build/HuaweiU8815) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51570[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8818 Build/HuaweiU8818) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51571[] = "Mozilla/5.0 (Linux; Android 4.0.4; Ascend G300 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51572[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI Ascend G 300 Build/HuaweiU8815) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51573[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8815N Build/HuaweiU8815NC02B942) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51574[] = "Mozilla/5.0 (Linux; Android 4.0.3; U8815-51 Build/HuaweiU8815) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51575[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A840S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51576[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A840S Build/JZO54K)";
const char S51577[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A840S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30";
const char S51578[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A840SP Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51579[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A840S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S51580[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; IM-A840S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51581[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A840SP Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30";
const char S51582[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A840S Build/GALAXY_S5_PRO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30";
const char S51583[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Mobile Safari/535.19 Silk-Accelerated=true";
const char S51584[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; ME173X Build/JOP40D)";
const char S51585[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ME173X Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51586[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; SAMSUNG-GT-N7108_TD/1.0 Android/4.1.1 Release/08.30.2012 Browser/AppleWebKit534.30 Build/JRO03C) ApplelWebkit/534.30 (KHTML,like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51587[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7108 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S51588[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7108 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51589[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-N7108 Build/JRO03C)";
const char S51590[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7108 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51591[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N7108 Build/JZO54K)";
const char S51592[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7108 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S51593[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One X Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S51594[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S51595[] = "Mozilla/5.0 (X11; Linux i686; rv:6.0.2) Gecko/X Abrowser/6.0.2";
const char S51596[] = "Mozilla/5.0 (X11; Linux i686; rv:6.0a2) Gecko/X Firefox/6.0a2 Iceweasel/6.0a2";
const char S51597[] = "Mozilla/5.0 (X11; Linux armv7l; rv:6.0.2) Gecko/X Firefox/6.0.2 Version.5.9.905.XT875.Verizon.en.US";
const char S51598[] = "Mozilla/5.0 (Windows NT 5.1; rv:15.0) Gecko/X Firefox/15.0.1 SeaMonkey/2.12.1";
const char S51599[] = "Mozilla/5.0 (Windows NT 5.1; rv:15.0) Gecko/X SeaMonkey/2.12.1";
const char S51600[] = "Mozilla/5.0 (Windows NT 5.2; rv:15.0) Gecko/X Firefox/15.0.1 SeaMonkey/2.12.1";
const char S51601[] = "SAMSUNG-GT-S5560/1.0 SHP/VPP/R5 Dolfin/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S51602[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:22.0) Gecko/X Firefox/22.0 AlexaToolbar/alxf-2.18";
const char S51603[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.13) Gecko/X Ubuntu/7.10 (gutsy) Firefox/2.0.0.13 (Linux Mint)";
const char S51604[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.16) Gecko/X Ubuntu/7.10 (gutsy) Firefox/2.0.0.16 (Linux Mint)";
const char S51605[] = "Mozilla/5.0 (X11; U; Linux i686; en-GB; rv:1.8.1.10) Gecko/X Ubuntu/7.10 (gutsy) Firefox/2.0.0.10";
const char S51606[] = "JUC (Linux; U; 2.3.6; zh-cn; GT-S5360; 240*320) UCWEB7.9.0.94/139/444";
const char S51607[] = "Openwave (Linux; U; 2.3.6; zh-cn; GT-S5360; 240*320) UCWEB7.9.0.94/139/444";
const char S51608[] = "Opera (Linux; U; 2.3.6; zh-cn; GT-S5360; 240*320) UCWEB7.9.0.94/139/444";
const char S51609[] = "JUC (Linux; U; 2.3.6; zh-cn; GT-S5360L; 240*320) UCWEB7.9.0.94/139/444";
const char S51610[] = "JUC (Linux; U; 2.3.6; zh-cn; GT-S5360B; 240*320) UCWEB7.9.0.94/139/444";
const char S51611[] = "Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S51612[] = "Mozilla/5.0 (Linux; Android 4.1.2; EVO Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S51613[] = "IUC(U;iOS 5.1.1;Zh-cn;320*480;)/UCWEB8.9.0.238/42/997";
const char S51614[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S51615[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mstar) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S51616[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; generic) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S51617[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui browser) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S51618[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mstar platform) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S51619[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; jbed-fastbcc) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S51620[] = "IUC(U;iOS 5.0.1;Zh-cn;320*480;)/UCWEB8.9.1.271/42/800";
const char S51621[] = "IUC(U;iOS 6_1_2;en-us;320*480;)/UCWEB8.9.0.238/44/355";
const char S51622[] = "UCWEB/2.0 (Java; U; MIDP-2.0; vi; maui) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S51623[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sunmicro) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S51624[] = "UCWEB/2.0 (Java; U; MIDP-2.0; id; maui) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S51625[] = "IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB8.9.1.271/41/800";
const char S51626[] = "UCWEB/2.0 (Java; U; MIDP-2.0; id; mstar) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S51627[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile /AJSC/1.5";
const char S51628[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; karbonn) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S51629[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile TRUSTED/1.0";
const char S51630[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; opera) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile TRUSTED/1.0";
const char S51631[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; spice) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile TRUSTED/1.0";
const char S51632[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile,";
const char S51633[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0";
const char S51634[] = "UCWEB/2.0 (Java; U; MIDP-2.0; Pt-BR; mozilla) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S51635[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; jbed) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S51636[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui wap browser) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S51637[] = "UCWEB/2.0 (Java; U; MIDP-2.0; vi; mstar) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S51638[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5233)/UC Browser8.2.0.132/50/355/UCWEB Mobile";
const char S51639[] = "Nokia5233/UCWEB8.2.0.116/50/800";
const char S51640[] = "Mozilla/5.0 (Java; U; en-in; nokia5233) UCBrowser8.2.1.144/69/444/UCWEB Mobile  3gpp-gba";
const char S51641[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-06D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S51642[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S51643[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I535 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S51644[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S51645[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300T Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S51646[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S51647[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9300 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S51648[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999V Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S51649[] = "Mozilla/5.0 (Linux; Android 4.0.4; I9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S51650[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-l9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S51651[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I939 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S51652[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9370; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.374 Mobile Safari/534.11+";
const char S51653[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13";
const char S51654[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.84 Safari/534.13";
const char S51655[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_0; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.107 Safari/534.13";
const char S51656[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.107 Safari/534.13";
const char S51657[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.107 Safari/534.13";
const char S51658[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13";
const char S51659[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.107 Safari/534.13";
const char S51660[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13";
const char S51661[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST23i Build/11.0.A.5.5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S51662[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST23a Build/11.0.A.2.16) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S51663[] = "Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S51664[] = "Mozilla/5.0 (Linux; Android 4.1.1; Galaxy Nexus Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S51665[] = "Mozilla/5.0 (Linux; Android 4.1.1; Galaxy X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S51666[] = "Mozilla/5.0 (Linux; Android 4.1; Galaxy Nexus Build/JRN84D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S51667[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; Nexus One Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51668[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Nexus One Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51669[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Nexus One Build/EPE54B) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51670[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; Nexus One Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile";
const char S51671[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; Nexus One Build/ERD78) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S51672[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; Nexus One Build/ERD72) AppleWebKit/525.10  (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S51673[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; Nexus One Build/ERE19) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51674[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en_us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51675[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Nexus One Build/EPF30) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51676[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; Nexus One Build/ERD79) AppleWebKit/530.17 (KHTML like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51677[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Nexus One Build/ERE36B) AppleWebKit/530.17 (KHTML like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51678[] = "Mozilla/5.0 (Linux; U; Android 2.1; Nexus One Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51679[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonyEricssonSO-03D Build/6.1.F.0.106) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51680[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-03D Build/6.1.F.0.106) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51681[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SO-03D Build/6.1.F.0.106)";
const char S51682[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-03D Build/6.1.F.0.128) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51683[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One SV Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51684[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P925/P92530b Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51685[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P925 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51686[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P925 Build/ICS)";
const char S51687[] = "Mozilla/5.0 (Series40; NokiaC3-00/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6";
const char S51688[] = "Mozilla/5.0 (Series40; NokiaC3-00/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S51689[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; SAMSUNG SGH-I757M/I757MUGLCA Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51690[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; SGH-I757M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51691[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I757 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51692[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-I757M Build/GINGERBREAD)";
const char S51693[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; AS740 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51694[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; US740 Build/ERE27) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (innerActive-ANDROID-206)";
const char S51695[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; US740 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51696[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; US740 Build/ERE27)";
const char S51697[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; AS740 Build/ERE27)";
const char S51698[] = "Firefox";
const char S51699[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; Acer E320 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51700[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; Acer E320-orange Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51701[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; Acer E320 Build/GRJ22)";
const char S51702[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8) Presto/2.12.388 Version/12.12";
const char S51703[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; onetouch EVO7 Build/WIND_01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51704[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; onetouch EVO7 Build/MAIN_01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51705[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; onetouch EVO7 Build/TMOBILE_PL) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51706[] = "Mozilla/5.0 (iPad; U; CPU OS 6_0 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S51707[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/6533.18.5 DiigoBrowser";
const char S51708[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/6533.18.5 DiigoBrowser Safari/6533.18.5 DiigoBrowser";
const char S51709[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/6533.18.5 DiigoBrowser Safari/6533.18.5 DiigoBrowser Safari/6533.18.5 DiigoBrowser";
const char S51710[] = "Mozilla/5.0 (iPad; U; CPU OS 6_0 like Mac OS X; en_us) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5";
const char S51711[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10A403 Safari/6533.18.5";
const char S51712[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/6533.18.5 DiigoBrowser Safari/6533.18.5 DiigoBrowser Safari/6533.18.5 DiigoBrowser Safari/6533.18.5 DiigoBrowser";
const char S51713[] = "Mozilla/5.0 (iPad; U; CPU OS 6_0_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S51714[] = "Mozilla/5.0 (iPad; U; CPU OS 6 like Mac OS X; zh-cn Model:iPad2,1) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S51715[] = "Mozilla/5.0 (iPad; U; CPU OS 6_0_1 like Mac OS X; en_us) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5";
const char S51716[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/6533.18.5 iLunascape/3200";
const char S51717[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10A523 Safari/6533.18.5";
const char S51718[] = "Mozilla/5.0 (iPad; U; CPU OS 6_0_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S51719[] = "Mozilla/5.0 (iPad; U; CPU OS 6_0_1 like Mac OS X; 6536.26) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5";
const char S51720[] = "Mozilla/5.0 (iPad; U; CPU OS 6_0_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5";
const char S51721[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26i Build/6.1.A.2.55) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S51722[] = "Mozilla/5.0 (Linux; Android 4.0.3; LT26i Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S51723[] = "LG/U8120/v1.0";
const char S51724[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Mobile Safari/535.19 Silk-Accelerated=true";
const char S51725[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC One X Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51726[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; One X Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51727[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; One X Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51728[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC One X Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51729[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC One X Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51730[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; One X Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51731[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One X Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51732[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; EndeavorU Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51733[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1";
const char S51734[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1";
const char S51735[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1";
const char S51736[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3110 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51737[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3110 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51738[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3110 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51739[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3110 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51740[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-; GT-P3110 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51741[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; es-; GT-P3110 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51742[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3110 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1.2/p3110";
const char S51743[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/537.35 (KHTML, like Gecko) Chrome/27.0.1445.2 Safari/537.35";
const char S51744[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S51745[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T859 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S51746[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.4.2 Mobile/9B206 Safari/7534.48.3";
const char S51747[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.4.2 Mobile/9B176 Safari/7534.48.3";
const char S51748[] = "Mozilla/5.0 (Vodafone-W1/V02c_19Jan2011_ZA; U; Linux; en-ZA) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51749[] = "Mozilla/5.0 (Linux; U; en-us; EBRD1101; EXT) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51750[] = "Mozilla/5.0 (Linux; U; 2.2; zh-cn; sdk Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51751[] = "Mozilla/5.0 (Linux; U; en-us; EBRD1201; EXT) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51752[] = "Mozilla/5.0 (Linux; U; en-us; EBRD1101(P); INT1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51753[] = "Mozilla/5.0(Linux;U;zh-tw)AppleWebKit/530.17(KHTML,like Gecko)Version/4.0 Mobile Safari/530.17";
const char S51754[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:17.0) Gecko/X Firefox/17.0 IceDragon/18.0.1.0";
const char S51755[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:18.0) Gecko/X Firefox/18.0 IceDragon/18.0.3.1";
const char S51756[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Mobile Safari/535.19 Silk-Accelerated=true";
const char S51757[] = "SAMSUNG-GT-S5222R Opera/9.80 (J2ME/MIDP; Opera Mini/7.0.32584/30.3697; U; en) Presto/2.8.119 Version/11.10";
const char S51758[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F240L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51759[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F240L Build/JZO54K)";
const char S51760[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; WM8850-mid Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51761[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; WM8850-mid Build/IML74K)";
const char S51762[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; WM8850_X6-7V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51763[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; WM8850_10V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51764[] = "Firefox/1.0 (Windows; U; Win98; en-US; Localization; rv:1.4) Gecko/X Netscape/7.1 (ax)";
const char S51765[] = "Mozilla/5.0 (Windows; U; Win98; es-AR; rv:1.7.6) Gecko/X Firefox/1.0.2";
const char S51766[] = "Mozilla/5.0 (Windows; U; Win 9x 4.90; es-AR; rv:1.7.6) Gecko/X Firefox/1.0.2";
const char S51767[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Safari/535.19 Silk-Accelerated=false";
const char S51768[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PC36100 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51769[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5333/S5333DDJI5; U; Bada/1.0; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 OPN-B";
const char S51770[] = "Opera/9.80 (X11; Linux i686; Edition Linux Mint) Presto/2.12.388 Version/12.14";
const char S51771[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:23.0) Gecko/X Firefox/23.0 AlexaToolbar/alxf-2.18";
const char S51772[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Acer Liquid Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51773[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; Liquid Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Acer_S100/Acer_LiquidE_4.008.08_EMEA_VFIT";
const char S51774[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Acer Liquid Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51775[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Liquid Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51776[] = "Mozilla/5.0 (Linux; U; Android 2.2; cs-; Liquid Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51777[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; Liquid Build/MASTER)";
const char S51778[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; Acer Liquid Build/FRG83G)";
const char S51779[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SHW-M110S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51780[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SHW-M110S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51781[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SHW-M110S Build/FROYO)";
const char S51782[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; SHW-M110S Build/FROYO)";
const char S51783[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; SHW-M110S Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51784[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; SHW-M110S Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S51785[] = "SonyEricssonJ108i/R7EA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2";
const char S51786[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Twitter for iPad";
const char S51787[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Twitter for iPad";
const char S51788[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Twitter for iPad";
const char S51789[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3) AppleWebKit/534.31 (KHTML, like Gecko) Chrome/17.0.558.0 Safari/534.31 UCBrowser/2.3.2.300";
const char S51790[] = "SAMSUNG-GT-S5222/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S51791[] = "Mozilla/5.0 (Windows NT 6.0; rv:20.0) Gecko/X Firefox/20.0 SeaMonkey/2.17.1";
const char S51792[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-10D Build/V16R45C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51793[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-10D Build/V18R46F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51794[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-10D Build/V15R43D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51795[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-10D Build/V20R47F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51796[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-10D Build/V21R48A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51797[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; F-10D Build/V21R48A)";
const char S51798[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-10D Build/V22R49C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51799[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus S Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51800[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus S Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51801[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus S Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51802[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus S Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/crespo";
const char S51803[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S51804[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500R Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S51805[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 WIFI_3G Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22";
const char S51806[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S51807[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500M Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S51808[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S51809[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P7500 Build/HTJ85B) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S51810[] = "Mozilla/5.0 (Series40; Nokia302/14.26; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.11";
const char S51811[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8010 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S51812[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8010 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S51813[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; AT1S0 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S51814[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; AT1S0 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S51815[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; AT1S0 Build/HTJ85B)";
const char S51816[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; AT1S0 Build/HTK55D)";
const char S51817[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S51818[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S51819[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S51820[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S51821[] = "Mozilla/5.0 (Linux;U;Linux2.6; it-it; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML,like Gecko) Version/4.0 Safari/530.17";
const char S51822[] = "Mozilla/5.0 (Linux; U; Linux 2.6;  en-gb; TOSHIBA_AC_AND_AZ) AppleWebKit/530.17(KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S51823[] = "Mozilla/5.0 (Linux;U;Linux2.6; en-gb; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML,like Gecko) Version/4.0 Safari/530.17";
const char S51824[] = "Mozilla/5.0 (Linux;U;Linux2.6; fr-fr; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1";
const char S51825[] = "Mozilla/5.0 (Linux;U;Linux2.6; nl-nl; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1";
const char S51826[] = "Mozilla/5.0 (Linux;U;Linux2.6; en-gb; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1";
const char S51827[] = "Mozilla/5.0 (Linux;U;Linux2.6; nl-nl; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML,like Gecko) Version/4.0 Safari/530.17";
const char S51828[] = "Mozilla/5.0 (Linux;U;Linux2.6; en-us; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML,like Gecko) Version/4.0 Safari/530.17";
const char S51829[] = "Mozilla/5.0 (Linux;U;Linux2.6; en-us; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1";
const char S51830[] = "Mozilla/5.0 (Linux;U;Linux2.6; fr-; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1";
const char S51831[] = "Mozilla/5.0 (Linux;U;Linux2.6; es-es; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1";
const char S51832[] = "Mozilla/5.0 (Linux;U;Linux2.6; it-it; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1";
const char S51833[] = "Mozilla/5.0 (Linux;U;Linux2.6; fr-; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML,like Gecko) Version/4.0 Safari/530.17";
const char S51834[] = "Mozilla/5.0 (Linux;U;Linux2.6; nl-; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1";
const char S51835[] = "Mozilla/5.0 (Linux;U;Linux2.6; fr-fr; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML,like Gecko) Version/4.0 Safari/530.17";
const char S51836[] = "Mozilla/5.0 (Linux;U;Linux2.6; de-de; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1";
const char S51837[] = "Mozilla/5.0 (Linux;U;Linux2.6; es-; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1";
const char S51838[] = "Mozilla/5.0 (Linux;U;Linux2.6; cs-cz; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1";
const char S51839[] = "Mozilla/5.0 (Linux;U;Linux2.6; sv-se; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1";
const char S51840[] = "Mozilla/5.0 (Linux;U;Linux2.6; zh-cn; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML,like Gecko) Version/4.0 Safari/530.17";
const char S51841[] = "Mozilla/5.0 (Linux;U;Linux2.6; es-es; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML,like Gecko) Version/4.0 Safari/530.17";
const char S51842[] = "Mozilla/5.0 (Linux;U;Linux2.6; de-de; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML,like Gecko) Version/4.0 Safari/530.17";
const char S51843[] = "Mozilla/5.0 (Linux;U;Linux2.6; en-; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML,like Gecko) Version/4.0 Safari/530.17";
const char S51844[] = "Mozilla/5.0 (Linux; U; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S51845[] = "Mozilla/5.0 (Linux; U; Linux 2.6;  de; TOSHIBA_AC_AND_AZ) AppleWebKit/530.17(KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S51846[] = "Mozilla/5.0 (Linux; U;) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S51847[] = "Mozilla/5.0 (Linux; U; Linux 2.6;  it; TOSHIBA_AC_AND_AZ) AppleWebKit/530.17(KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S51848[] = "Mozilla/5.0 (Linux; U; Bigdroid_2.0; zh-cn; I600/FRF85B; 1024*600) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17 ThunderSoft_BigDroid/2.0";
const char S51849[] = "Mozilla/5.0 (Linux; U; Linux 2.6;  en-us; TOSHIBA_AC_AND_AZ) AppleWebKit/530.17(KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S51850[] = "Mozilla/5.0 (Linux; U; Linux 2.6;  fr; TOSHIBA_AC_AND_AZ) AppleWebKit/530.17(KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S51851[] = "Mozilla/5.0 (Linux; U; Linux 2.6;  nl; TOSHIBA_AC_AND_AZ) AppleWebKit/530.17(KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S51852[] = "Mozilla/5.0 (Linux; U; Linux 2.6;  es; TOSHIBA_AC_AND_AZ) AppleWebKit/530.17(KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S51853[] = "Mozilla/5.0 (Linux; U; Bigdroid_2.0; zh-cn; SMB-1002-WIFI/FRF91; 1024*600) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17 ThunderSoft_BigDroid/2.0";
const char S51854[] = "Mozilla/5.0 (Linux; U; Linux ; en-US) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.4 en-US;L6200 Version/4.0 Safari/531.2+";
const char S51855[] = "Mozilla/5.0 (Linux; U; Bigdroid_2.0; zh-cn; SMB-1002-3G/FRF91; 1024*600) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17 ThunderSoft_BigDroid/2.0";
const char S51856[] = "Mozilla/5.0 (Linux; U; Linux ; fr-CA) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.4 fr-CA;L6200 Version/4.0 Safari/531.2+";
const char S51857[] = "Mozilla/5.0 (Linux; U; Linux ; en-CA) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.4 en-CA;L6200 Version/4.0 Safari/531.2+";
const char S51858[] = "Mozilla/5.0 (Linux; U; Cosmote_My_mini_Tab 4.0.4; ro-ro; Cosmote_My_mini_Tab Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51859[] = "Mozilla/5.0 (Linux; U; Linux 2.6;  ru; TOSHIBA_AC_AND_AZ) AppleWebKit/530.17(KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S51860[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T699 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51861[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T699 Build/JZO54K)";
const char S51862[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11";
const char S51863[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11";
const char S51864[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9B176 Safari/7534.48.3";
const char S51865[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9B206 Safari/7534.48.3";
const char S51866[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC_S510b Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51867[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-hk; HTC_Rhyme_S510b Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51868[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC S510b Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S51869[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC S510b Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S51870[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-hk; HTC Rhyme S510b Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51871[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC S510b Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51872[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Rhyme S510b Build/IML74K)";
const char S51873[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Rhyme S510b Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S51874[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Comodo_Dragon/15.0.3.0 Chrome/15.0.874.102 Safari/535.2";
const char S51875[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.2 (KHTML, like Gecko) Comodo_Dragon/15.0.3.0 Chrome/15.0.874.102 Safari/535.2";
const char S51876[] = "SAMSUNG-GT-E3210L/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S51877[] = "SAMSUNG-GT-E3210/E3210ODFKK1 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S51878[] = "SAMSUNG-GT-E3210B/E3210BDOKJ1 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S51879[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S51880[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293";
const char S51881[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S51882[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/6531.22.7 iLunascape/3165";
const char S51883[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293";
const char S51884[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S51885[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S51886[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/6531.22.7 iLunascape/3195";
const char S51887[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/6531.22.7 iLunascape/3200";
const char S51888[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293";
const char S51889[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S51890[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari/6531.22.7 iLunascape/3200";
const char S51891[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S51892[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X) SparqEye/1.2.2 AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A293 Safari/6531.22.7";
const char S51893[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.11 (KHTML, like Gecko) Ubuntu/10.10 Chromium/17.0.963.79 Chrome/17.0.963.79 Safari/535.11";
const char S51894[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.11 (KHTML, like Gecko) Ubuntu/10.10 Chromium/17.0.963.65 Chrome/17.0.963.65 Safari/535.11";
const char S51895[] = "NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-01) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S51896[] = "NokiaX2-01.1/5.0 (07.22) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-01.1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S51897[] = "SAMSUNG-GT-S5230W/S523WXIJK1 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S51898[] = "Mozilla/5.0 (Linux; U; Android 2.0; en-us; Droid Build/ERD15) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51899[] = "Mozilla/5.0 (Linux; U; Android 2.0.1; en-us; Droid Build/ESD56) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51900[] = "Mozilla/5.0 (Linux; U; Android 2.0; en-us; Droid Build/ESD20) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S51901[] = "Mozilla/5.0 (Linux; U; Android 2.0; en-us; Droid Build/ESD20) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S51902[] = "Mozilla/5.0 (Linux; U; Android Eclair; en-us; sholes) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S51903[] = "Mozilla/5.0 (Linux; U; Android 2.0; en-us; Droid Build/ESD20) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/ 530.17";
const char S51904[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S51905[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S51906[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Z710e Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S51907[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G with Beats Audio Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S51908[] = "Opera/9.80 (S60; SymbOS; Opera Mobi/SYB-X; U; es-LA) Presto/2.10.254 Version/12.00";
const char S51909[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-R940 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51910[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-R940 Build/GINGERBREAD)";
const char S51911[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Twitter for iPhone";
const char S51912[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Twitter for iPhone";
const char S51913[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; en) Opera 8.01";
const char S51914[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; en) Opera 8.0";
const char S51915[] = "Opera/8.54 (Windows NT 5.0; U; en)";
const char S51916[] = "Opera/8.50 (Windows NT 5.0; U; en)";
const char S51917[] = "Opera/8.52 (Windows NT 5.0; U; en)";
const char S51918[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; en) Opera 8.02";
const char S51919[] = "Opera/8.01 (Windows NT 5.0; U; de)";
const char S51920[] = "Opera/8.51 (Windows NT 5.0; U; en)";
const char S51921[] = "Opera/8.53 (Windows NT 5.0; U; en)";
const char S51922[] = "Opera/8.0 (Windows NT 5.0; U; en)";
const char S51923[] = "Opera/8.00 (Windows NT 5.0; U; en)";
const char S51924[] = "Opera 8.0 (Win 2000): Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; en) Opera 8.0";
const char S51925[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:15.0) Gecko/X Firefox/15.1.1-x64 PaleMoon/15.1.1-x64";
const char S51926[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X Firefox/15.1.1 PaleMoon/15.1.1";
const char S51927[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS840PP Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S51928[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS840 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S51929[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Desire HD Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51930[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Desire HD Build/JRO03R; CyanogenMod-JellyTime_R13_Unlucky_Edition2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51931[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Desire HD Build/JZO54K; CyanogenMod-JellyTime_R30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51932[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Desire HD Build/JRO03L; CyanogenMod-JellyTime_R10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51933[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; USCC_ALCATEL_one_touch_988 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51934[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ALCATEL_one_touch_988 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51935[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; USCC_ALCATEL_one_touch_988 Build/GINGERBREAD)";
const char S51936[] = "Opera/9.80 (iPhone; Opera Mini/5.0.X/22.414; U; en) Presto/2.5.25 Version/10.54";
const char S51937[] = "Opera/9.80 (J2ME/iPhone;Opera Mini/5.0.X/886; U; ja)Presto/2.4.15";
const char S51938[] = "Opera/9.80 (iPhone; Opera Mini/5.0.0176/764; U; en) Presto/2.4.154.15";
const char S51939[] = "Opera/9.80 (iPhone; Opera Mini/5.0/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S51940[] = "Mozilla/5.0 (Linux; Android 4.1.1; ME172V Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S51941[] = "NokiaC3-00/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac3-00) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S51942[] = "Mozilla/5.0 (PlayStation Vita 1.80) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2";
const char S51943[] = "Mozilla/5.0 (PlayStation Vita 1.81) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2";
const char S51944[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.1 Chrome/13.0.800.1 Safari/535.1";
const char S51945[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.1 Chrome/13.0.800.1 Safari/535.1";
const char S51946[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Optimus 2X Build/GRJ22; CyanogenMod-7.2.0; LG-P990 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51947[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; Optimus 2X Build/GRJ22; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51948[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7510 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51949[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7510 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51950[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7510 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51951[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-; GT-P7510 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51952[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7510 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51953[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710e Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S51954[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S51955[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Z710e Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S51956[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G with Beats Audio Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S51957[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation 4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S51958[] = "Mozilla/5.0 (Series40; NokiaC2-02/06.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S51959[] = "Mozilla/5.0 (Series40; NokiaC2-02.1/06.52; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S51960[] = "NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac1-01) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S51961[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; IM-A760S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51962[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; IM-A760S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51963[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; IM-A760S Build/GINGERBREAD)";
const char S51964[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; IM-A760S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51965[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; IM-A760S Build/GINGERBREAD)";
const char S51966[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; IM-A760S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51967[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; IM-A760S/770K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S51968[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4";
const char S51969[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4";
const char S51970[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4";
const char S51971[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M975 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51972[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M975 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51973[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; M975 Build/JRO03H)";
const char S51974[] = "UCWEB/2.0(BlackBerry; U; 6.4.0.105; en-us; 9800/6.4.0.105) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S51975[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.223; en-us; 9800/6.6.0.223) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S51976[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.207; en-us; 9800/6.6.0.207) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S51977[] = "UCWEB/2.0(BlackBerry; U; 6.4.0.120; en-us; 9800/6.4.0.120) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S51978[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.236; en-us; 9800/6.6.0.236) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S51979[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.212; en-us; 9800/6.6.0.212) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S51980[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.249; en-us; 9800/6.6.0.249) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S51981[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.246; en-us; 9800/6.6.0.246) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S51982[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.195; en-us; 9800/6.6.0.195) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S51983[] = "UCWEB/2.0(BlackBerry; U; 6.4.0.210; en-us; 9800/6.4.0.210) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S51984[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.241; en-us; 9800/6.6.0.241) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S51985[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.200; en-us; 9800/6.6.0.200) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S51986[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.248; en-us; 9800/6.6.0.248) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S51987[] = "UCWEB/2.0(BlackBerry; U; 6.4.0.64; en-us; 9800/6.4.0.64) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S51988[] = "UCWEB/2.0(BlackBerry; U; 6.4.0.142; en-us; 9800/6.4.0.142) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S51989[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.216; en-us; 9800/6.6.0.216) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S51990[] = "UCWEB/2.0(BlackBerry; U; 6.4.0.211; en-us; 9800/6.4.0.211) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S51991[] = "UCWEB/2.0(BlackBerry; U; 6.4.0.99; en-us; 9800/6.4.0.99) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S51992[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S51993[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S51994[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S51995[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S51996[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-S738C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S51997[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Amazon Kindle Fire Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S51998[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; Amazon Kindle Fire Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S51999[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Amazon Kindle Fire Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52000[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Amazon Kindle Fire Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52001[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-CA; Amazon Kindle Fire Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52002[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Sprint APA7373KT Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52003[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8) Presto/2.12.388 Version/12.11";
const char S52004[] = "Mozilla/5.0 (Series40; Nokia206/03.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S52005[] = "Mozilla/5.0 (Series40; Nokia206.1/03.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S52006[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F240K/10g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52007[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F240K/10f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52008[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F240K/10h Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52009[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F240K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52010[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F240K/10l Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52011[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F240K/10n Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52012[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F240K/10o Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52013[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F240K Build/JZO54K)";
const char S52014[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F240K/10r Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52015[] = "Mozilla/5.0 (Linux; Android 4.1.1; Transformer Prime TF201 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S52016[] = "Mozilla/5.0 (Linux; Android 4.1.2; Transformer Prime TF201 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S52017[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; 7 Mozart; Orange)";
const char S52018[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; T8697)";
const char S52019[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; 7 Mozart T8698)";
const char S52020[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; 7 Mozart)";
const char S52021[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Mozart)";
const char S52022[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; PD67100)";
const char S52023[] = "Opera/9.80 (Android; Opera Mini/7.7.33426/29.3551; U; zh) Presto/2.8.119 Version/11.10";
const char S52024[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S52025[] = "Mozilla/5.0 (compatible; TweetmemeBot/2.11; +http://tweetmeme.com/)";
const char S52026[] = "sam-r390 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S52027[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-02E Build/SB140) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52028[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-02E Build/SC110) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52029[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-02E Build/S2130) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52030[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SH-02E Build/SC110)";
const char S52031[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-02E Build/S4100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52032[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S52033[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52034[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7000 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52035[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52036[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52037[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52038[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M480W Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S52039[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000B Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52040[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LS970 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52041[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30at Build/9.1.F.1.120) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52042[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P5113 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S52043[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52044[] = "Mozilla/5.0 (Linux; Android 4.0.4; Desire S Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52045[] = "Mozilla/5.0 (Linux; Android 4.0.3;  HTC Desire S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S52046[] = "Mozilla/5.0 (Linux; Android 4.0.4;  Desire S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S52047[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52048[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire SV Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52049[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; M1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S52050[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; M1 Build/JRO03H)";
const char S52051[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; M1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52052[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Lenovo P700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52053[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo P700 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S52054[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Lenovo P700 Build/IML74K)";
const char S52055[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo P700 Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S52056[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; Lenovo P700i; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52057[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lenovo P700i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52058[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo P700i Build/IMM76D)";
const char S52059[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo P700i Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S52060[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; P700i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52061[] = "LGE-VM510 NetFront/3.5.1 (GUI) MMP/2.0";
const char S52062[] = "Mozilla/5.0 (Linux; U; Android 2.1; nl-nl; HTC Legend Build/ERD72) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52063[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; de-at; HTC Legend 2.05.177.2 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52064[] = "Mozilla/5.0 (Linux; U; Android 2.1; fr-fr; HTC Legend 1.32.163.1 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52065[] = "Mozilla/5.0 (Linux; U; Android 2.1; fr-fr; HTC Legend 1.23.163.1 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52066[] = "Mozilla/5.0 (Linux; U; Android 2.1; fr-fr; HTC Legend 1.32.163.2 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52067[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-gb; HTC Legend 1.32.161.4 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52068[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-gb; HTC Legend 1.23.161.1 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52069[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; de-at; HTC Legend 1.31.177.2 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52070[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Legend 1.32.161.4 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52071[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Legend 2.05.161.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52072[] = "Mozilla/5.0 (Linux; U; Android 2.1; de-at; HTC Legend 1.31.177.2 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52073[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; HTC Legend Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52074[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Legend 1.23.161.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52075[] = "Mozilla/5.0 (Linux; U; Android 2.1 Legend; zh-cn; HTC Legend Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52076[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; HTC Legend 2.05.163.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52077[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; HTC Legend 1.32.163.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52078[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-gb; HTC Legend Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52079[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; HTC Legend 1.23.163.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52080[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; HTC Legend Build/cupcake) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52081[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; HTC Legend Build/ERE27)";
const char S52082[] = "Dalvik/1.1.0 (Linux; U; Android 2.1; HTC Legend Build/ERD79)";
const char S52083[] = "Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; HTC Legend Build/ERD79) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S52084[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Sony Tablet P Build/TISU0085) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S52085[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Sony Tablet P Build/TISU0093) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S52086[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; Sony Tablet P Build/TISU0124) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S52087[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Sony Tablet P Build/TISU0144) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S52088[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Sony Tablet P Build/TISU0144)";
const char S52089[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST27i Build/6.2.A.1.100) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52090[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST27a Build/6.2.A.1.100) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52091[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530U Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52092[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52093[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-L710 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52094[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I747M Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52095[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530C Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52096[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210K Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52097[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52098[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210L Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52099[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ARCHOS 80 TITANIUM Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S52100[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ARCHOS 80 TITANIUM Build/JRO03H)";
const char S52101[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530M Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S52102[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530M Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S52103[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC VLE_U Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S52104[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S52105[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S52106[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC VLE_U Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S52107[] = "Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia305) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S52108[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T989 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52109[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52110[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52111[] = "Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S52112[] = "Mozilla/5.0 (Linux; Android 4.0.4; Transformer Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S52113[] = "Mozilla/5.0 (Linux; Android 4.0.3; TF101 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S52114[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; SonyC1504 Build/11.3.A.0.47) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52115[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; C1504 Build/11.3.A.0.47) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52116[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; SonyC1504 Build/11.3.A.0.58) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52117[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; SonyC1504 Build/11.3.A.0.56) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52118[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; C1504 Build/11.3.A.0.47)";
const char S52119[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; SonyC1504 Build/11.3.A.2.13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52120[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; SonyC1504 Build/11.3.A.0.59) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52121[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; C1504 Build/11.3.A.2.13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52122[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; C1504 Build/11.3.A.0.59) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52123[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-mx; C1504 Build/11.3.A.0.58) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52124[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SAMSUNG-SGH-I727 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52125[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I727 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52126[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I727 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/skyrocket";
const char S52127[] = "Mozilla/4.0 (BREW 3.1.5; U; en-us; Sanyo; NetFront/3.5.1/AMB) Boost SCP6760, Mozilla/4.0 (BREW 3.1.5; U; en-us; Sanyo; NetFront/3.5.1/AMB) Boost SCP6760";
const char S52128[] = "Mozilla/4.0 (BREW 3.1.5; U; en-us; Sanyo; NetFront/3.5.1/AMB) Boost SCP6760";
const char S52129[] = "Mozilla/4.0 (BREW 3.1.5; U; en-us; Sanyo; NetFront/3.5.1/AMB) Sprint SCP-6760";
const char S52130[] = "Mozilla/4.0 (BREW 3.1.5; U; en-us; Sanyo; NetFront/3.5.1/AMB) Sanyo  SCP-6760";
const char S52131[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; Blade Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52132[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; hu-hu; ZTE-Blade Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52133[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; Blade Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52134[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Blade Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52135[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Blade Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52136[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; ZTE-Blade Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52137[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; Blade Build/GRJ22)";
const char S52138[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Blade Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S52139[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; Blade Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52140[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; Blade Build/FRG83)";
const char S52141[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Blade Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S52142[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S52143[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S52144[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100B Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S52145[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P769 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52146[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P768 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52147[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P760 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52148[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID7033 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S52149[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LS970 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S52150[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LS970 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52151[] = "Nokia5233/UC Browser8.0.4.121/50/444/UCWEB";
const char S52152[] = "Nokia5233/UCWEB8.0.3.99/50/800";
const char S52153[] = "Nokia 5233/UC Browser8.0.4.121/50/351/UCWEB";
const char S52154[] = "BlackBerry8110/4.5.0.182 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102";
const char S52155[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; el-gr; ALCATEL_one_touch_908 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52156[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ALCATEL_one_touch_908S Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52157[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ALCATEL_one_touch_908A Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52158[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; ALCATEL_one_touch_908 Build/FRG83G)";
const char S52159[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; U30GT-H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S52160[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; U30GT-H Build/IMM76D)";
const char S52161[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U30GT-H Build/IMM76D; 1280*752) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S52162[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-I9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52163[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; BP-I9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52164[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; I9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52165[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; BP-I9300 Build/GRK39F)";
const char S52166[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Grand X In Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52167[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Grand X In Build/GXI_TATV1.0.0B03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52168[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; Grand X In Build/GXI_TTRV1.0.0B03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52169[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Grand X In Build/GXI_TATV1.0.0B02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52170[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; Grand X In Build/GXI_TTRV1.0.0B04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52171[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Grand X In Build/GXI_TPLV1.0.0B04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52172[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; Grand X In Build/GXI_THUV1.0.0B04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52173[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ro-ro; Grand X In Build/GXI_TROV1.0.0B01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52174[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sk-sk; Grand X In Build/GXI_OSKV1.0.0B02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52175[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Grand X In Build/GXI_TMKV1.0.0B04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52176[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Grand X In Build/GXI_V1.0.0B02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52177[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Grand X In Build/GXI_V1.0.0B01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52178[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sk-sk; Grand X In Build/GXI_TMSKV1.0.0B01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52179[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Grand X In Build/GXI_TPLV1.0.0B06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52180[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; Grand X In Build/GXI_TTRV1.0.0B06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52181[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; Grand X In Build/GXI_THUV1.0.0B06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52182[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Grand X In Build/GXI_GERV1.0.0B01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52183[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Grand X In Build/GXI_GERV1.0.0B02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52184[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Grand X In Build/GXI_TMKV1.0.0B06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52185[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; Grand X In Build/GXI_V1.0.0B04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52186[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; Grand X In Build/GXI_TSKV1.0.0B02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52187[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sk-sk; Grand X In Build/GXI_TMSKV1.0.0B02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52188[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; Grand X In Build/GXI_TSKV1.0.0B03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52189[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Grand X In Build/GXI_ATBPV1.0.0B01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52190[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Grand X In Build/GXI_ESPV1.0.0B01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52191[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; Grand X In Build/GXI_V1.0.0B05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52192[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9A405 Safari/7534.48.3";
const char S52193[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9A334 Safari/7534.48.3";
const char S52194[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.10.229 Version/11.61";
const char S52195[] = "Mozilla/5.0 (IE 11.0; Windows NT 6.2; WOW64; Trident/7.0; rv:11.0) like Gecko";
const char S52196[] = "Mozilla/5.0 (IE 11.0; Windows NT 6.2; Trident/7.0; rv:11.0) like Gecko";
const char S52197[] = "Mozilla/5.0 (IE 11.0; Windows NT 6.2; Win64; x64; Trident/7.0; rv:11.0) like Gecko";
const char S52198[] = "Windows-RSS-Platform/2.0 (IE 11.0; Windows NT 6.2)";
const char S52199[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; Trident/7.0; rv:11.0) like Gecko";
const char S52200[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; Trident/7.0; WebView/1.0; rv:11.0) like Gecko";
const char S52201[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; Trident/7.0; rv:11.0; MSN 9.0;MSN 9.1;MSN 9.6;MSN 10.0;MSN 10.2;MSN 10.5; MSNbMSNI; MSNmen-us; MSNcOTH)";
const char S52202[] = "Mozilla/5.0 (AOL 9.7; AOLBuild 4343.1028; Windows NT 6.2; WOW64; Trident/7.0; rv:11.0) like Gecko";
const char S52203[] = "Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Safari/535.19 Silk-Accelerated=true";
const char S52204[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S52205[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S52206[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S52207[] = "Mozilla/5.0 (Series40; Nokia305/05.92; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S52208[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/21.0.1180.75 Safari/537.1";
const char S52209[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.3.6000 Chrome/21.0.1180.75 Safari/537.1";
const char S52210[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.22 (KHTML, like Gecko) Maxthon/4.0.4.3000 Chrome/25.0.1364.99 Safari/537.22";
const char S52211[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.22 (KHTML, like Gecko) Maxthon/4.0.6.3000 Chrome/25.0.1364.99 Safari/537.22";
const char S52212[] = "Opera/9.64 (Macintosh; PPC Mac OS X; U; en) Presto/2.1.1";
const char S52213[] = "Opera/9.63 (Macintosh; Intel Mac OS X; U; en) Presto/2.1.1";
const char S52214[] = "Opera/9.62 (Macintosh; PPC Mac OS X; U; en) Presto/2.1.1";
const char S52215[] = "Opera/9.60 (Macintosh; Intel Mac OS X; U; de) Presto/2.1.1";
const char S52216[] = "Opera/9.61 (Macintosh; Intel Mac OS X; U; de) Presto/2.1.1";
const char S52217[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5";
const char S52218[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5";
const char S52219[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5";
const char S52220[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5";
const char S52221[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5";
const char S52222[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5";
const char S52223[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5";
const char S52224[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/4.0.202.0 Safari/532.0";
const char S52225[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/4.0.211.2 Safari/532.0";
const char S52226[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6606 Build/10.1.1.B.0.166) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52227[] = "Mozilla/5.0 (Linux; Android 2.3.5; GT-I9100 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S52228[] = "Mozilla/5.0 (Linux; Android 2.3.3; GT-I9100 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S52229[] = "Mozilla/5.0 (Linux; Android 2.3.4; GT-I9100 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S52230[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-I9100 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S52231[] = "Mozilla/5.0 (Linux; Android 2.3.6; SGH-S959G Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S52232[] = "Mozilla/5.0 (Linux; Android 2.3.5; GT-I9100T Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S52233[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-I9100G Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S52234[] = "Mozilla/5.0 (Linux; Android 2.3.3; GT-I9100M Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S52235[] = "Mozilla/5.0 (Linux; Android 2.3.3; GT-I9100T Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453";
const char S52236[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/21.0.1180.60) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.60 Safari/537.1";
const char S52237[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/21.0.1180.60) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.60 Safari/537.1";
const char S52238[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/21.0.1180.83) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1";
const char S52239[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/21.0.1180.79) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.79 Safari/537.1";
const char S52240[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/21.0.1180.79) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.79 Safari/537.1";
const char S52241[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/21.0.1180.77) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.77 Safari/537.1";
const char S52242[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/21.0.1180.89)";
const char S52243[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/21.0.1180.89)";
const char S52244[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/21.0.1180.89) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S52245[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; chromeframe/21.0.1180.89; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S52246[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/21.0.1180.89; GTB7.4; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)";
const char S52247[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/21.0.1180.89) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S52248[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; chromeframe/21.0.1180.79; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MS-RTC LM 8; InfoPath.3)";
const char S52249[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; chromeframe/21.0.1180.89; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)";
const char S52250[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/21.0.1180.89; SLCC1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.5.21022; InfoPath.2; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S52251[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/21.0.1180.89; GTB7.4; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729)";
const char S52252[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/21.0.1180.89; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 1.1.4322; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.1; .NET4.0C; .NET4.0E)";
const char S52253[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:21.0) Gecko/X Firefox/21.0 AlexaToolbar/alxf-2.18";
const char S52254[] = "Mozilla/5.0 (X11; U; FreeBSD i386; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S52255[] = "Mozilla/5.0 (X11; U; FreeBSD x86_64; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S52256[] = "Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Safari/535.19 Silk-Accelerated=true";
const char S52257[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; C6602 Build/10.3.A.0.423) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52258[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; C6602 Build/10.3.1.A.0.244)";
const char S52259[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; C6602 Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52260[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; SonyL36h Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52261[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; ALCATEL ONE TOUCH 5020D Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S52262[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ALCATEL ONE TOUCH 5020E Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S52263[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ALCATEL ONE TOUCH 5020D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S52264[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; ALCATEL ONE TOUCH 5020A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S52265[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ALCATEL ONE TOUCH 5020X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S52266[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ALCATEL ONE TOUCH 5020W Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S52267[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Nexus S 4G Build/GRJ06D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52268[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Nexus S 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52269[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Nexus S 4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52270[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; Nexus S 4G Build/GWK74)";
const char S52271[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus S 4G Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52272[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus S 4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52273[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; Nexus S 4G Build/GRJ22)";
const char S52274[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Nexus S 4G Build/GRJ90)";
const char S52275[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Nexus S 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52276[] = "Mozilla/5.0 (Linux; U; Android 2.3; xx-xx; Nexus S 4G Build/GRJ90) AppleWebKit/533.1  (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52277[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9.1.9) Gecko/X Firefox/3.5.9 WinNT-EVI 23.04.2010";
const char S52278[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; de-DE; rv:1.8) Gecko/X Firefox/3.5";
const char S52279[] = "SAMSUNG-GT-B5310/1.0 SHP/VPP/R5 Dolfin/1.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S52280[] = "SAMSUNG-GT-B5310U/1.0 SHP/VPP/R5 Dolfin/1.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S52281[] = "SAMSUNG-GT-B5310R/B5310UXJB5 SHP/VPP/R5 Dolfin/1.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S52282[] = "SAMSUNG-GT-B5310L/1.0 SHP/VPP/R5 Dolfin/1.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S52283[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T959 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52284[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T959 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52285[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52286[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SGH-T959 Build/IML74K)";
const char S52287[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; SGH-T959 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52288[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52289[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52290[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959 Build/IMM30B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52291[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52292[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959 Build/IMM26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52293[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.0 Safari/533.3";
const char S52294[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.1 Safari/533.3";
const char S52295[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-ca; SAMSUNG SGH-M919V Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S52296[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P5110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S52297[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P5110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S52298[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S5303 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52299[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT-S5303B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52300[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-S5303 Build/IMM76D)";
const char S52301[] = "SAMSUNG-GT-E2121/E2121JBJF1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S52302[] = "SAMSUNG-GT-E2121B/E2121BXXKC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S52303[] = "SAMSUNG-GT-E2121N/E2121NJCJJ1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S52304[] = "SAMSUNG-GT-E2121L/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S52305[] = "Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.1.9) Gecko/X Fedora/3.5.9-1.fc11 Firefox/3.5.9";
const char S52306[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IdeaTab_A1107 Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S52307[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; IdeaTab_A1107 Build/MR1)";
const char S52308[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyC1605 Build/11.1.A.0.64) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52309[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C1605 Build/11.1.A.0.68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52310[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C1605 Build/11.1.A.0.64) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52311[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; C1605 Build/11.1.A.0.68)";
const char S52312[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315";
const char S52313[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S52314[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S52315[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S52316[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S52317[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID7034 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S52318[] = "SAMSUNG-GT-E2152L/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S52319[] = "SAMSUNG-GT-E2152/E2152DDJH1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S52320[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; SHW-M130L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52321[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; SHW-M130L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52322[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M130K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52323[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SHW-M130K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52324[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SHW-M130K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52325[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHW-M130L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52326[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SHW-M130L Build/GINGERBREAD)";
const char S52327[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M130K Build/GINGERBREAD)";
const char S52328[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M130L Build/GINGERBREAD)";
const char S52329[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHW-M130K Build/GINGERBREAD)";
const char S52330[] = "AppleCoreMedia/1.0.0.9B176 (iPad; U; CPU OS 5_1 like Mac OS X; en_us)";
const char S52331[] = "AppleCoreMedia/1.0.0.9B206 (iPad; U; CPU OS 5_1_1 like Mac OS X; en_us)";
const char S52332[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S52333[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 30 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S52334[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Safari/535.19 Silk-Accelerated=false";
const char S52335[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I5510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52336[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I5510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52337[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-I5510T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52338[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I5510 Build/GINGERBREAD)";
const char S52339[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-I5510L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52340[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; GT-I5510L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52341[] = "Mozilla/5.0 (X11; Linux i686; rv:18.0) Gecko/X Firefox/18.0 SeaMonkey/2.15.2";
const char S52342[] = "Mozilla/5.0 (X11; Linux i686; rv:18.0) Gecko/X SeaMonkey/2.15.1";
const char S52343[] = "Mozilla/5.0 (Linux; Android 4.0.4; cm_tenderloin Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S52344[] = "Mozilla/5.0 (Linux; Android 4.0.3; cm_tenderloin Build/GWK74) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S52345[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Barnes & Noble Nook Tablet Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52346[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Barnes & Noble Nook Tablet Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52347[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Barnes & Noble Nook Tablet Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52348[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; NookTablet Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52349[] = "Opera/9.80 (Android; Opera Mini/16.0.1212/31.1475; U; en) Presto/2.8.119 Version/11.10";
const char S52350[] = "Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; NexusHD2 Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52351[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; NexusHD2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52352[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC HD2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52353[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC HD2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52354[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; NexusHD2 Build/IML74K)";
const char S52355[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC HD2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52356[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HTC HD2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52357[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC HD2 Build/GRK39F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52358[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC HD2 Build/IMM76I)";
const char S52359[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; NexusHD2 Build/IMM76L)";
const char S52360[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC HD2 EvoHd2 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52361[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; NexusHD2 Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52362[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; zh-tw; Hd2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52363[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; NexusHD2 Build/GRK39F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52364[] = "Mozilla/5.0 (Linux; U; Android 4.0.4RC; en-us; Evo Hd2 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52365[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyLT30a Build/9.1.A.0.489) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52366[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LT30a Build/9.1.A.0.489) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52367[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LT30a Build/9.1.A.0.489)";
const char S52368[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyLT30a Build/9.1.A.1.140) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52369[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LT30a Build/9.1.A.1.140) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52370[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyLT30a Build/9.1.A.0.492) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52371[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyLT30a Build/9.1.A.1.141) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52372[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9A405 Safari/7534.48.3";
const char S52373[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9A406 Safari/7534.48.3";
const char S52374[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9A334 Safari/7534.48.3";
const char S52375[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Vision Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52376[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Vision Build/GRI40; ILWT-CM7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52377[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; sdk Build/GSI11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52378[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52379[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; google_sdk Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52380[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; sdk Build/GRI34)";
const char S52381[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; rk29sdk Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52382[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S52383[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; sdk Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52384[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; google_sdk Build/GINGERBREAD)";
const char S52385[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.1; sdk Build/GSI11)";
const char S52386[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; google_sdk Build/GSI11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52387[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.1; rk29sdk Build/GINGERBREAD)";
const char S52388[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; rk29sdk Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S52389[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 BaDoinkDownloader_Android_2.0";
const char S52390[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 BaDoinkDownloader_Android_2.0";
const char S52391[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 BaDoinkDownloader_Android_2.0";
const char S52392[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 BaDoinkDownloader_Android_2.0";
const char S52393[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U9508 Build/HuaweiU9508) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S52394[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52395[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI U9508 Build/HuaweiU9508)";
const char S52396[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S52397[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11";
const char S52398[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11";
const char S52399[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500R Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S52400[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S52401[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S52402[] = "Nokia2600c-2/2.0 (05.61) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2600c-2";
const char S52403[] = "Nokia2600c-2/2.0 (05.61) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S52404[] = "Nokia2600c/2.0 (05.21) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S52405[] = "Nokia2600c/2.0 (05.22) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2600c";
const char S52406[] = "Nokia2600c-2b/2.0 (06.45) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S52407[] = "Nokia2600c-2_CMCC/2.0 (05.61) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S52408[] = "Nokia2600c_CMCC/2.0 (05.22) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S52409[] = "Mozilla/5.0 (Linux; Android 4.1.2; EVO Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52410[] = "Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S52411[] = "Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52412[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X+ Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S52413[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X+ Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52414[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC EVARE_UL Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52415[] = "Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Safari/535.19 Silk-Accelerated=true";
const char S52416[] = "Mozilla/5.0 (Linux; Android 4.0.4; P771A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S52417[] = "Mozilla/5.0 (Linux; Android 4.0.4; P771A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52418[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I537 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52419[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SAMSUNG-SGH-I537 Build/JDQ39)";
const char S52420[] = "label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.2.2;dev=jactivelteatt;devm=SAMSUNG-SGH-I537;man=samsung;brw=Android;shwd=1080x1920;shwa=1080x1845;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;";
const char S52421[] = "Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHW-M380W Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52422[] = "Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHW-M380K Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52423[] = "Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHW-M380S Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52424[] = "Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHW-M380S Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52425[] = "Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHW-M380W Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52426[] = "Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHW-M380K Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52427[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; SHW-M380W Build/HTK75D)";
const char S52428[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E980 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S52429[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT300 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S52430[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT300SE Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S52431[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X Thunderbird/15.0.1";
const char S52432[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Flipboard/1.9.7";
const char S52433[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Flipboard/1.9.6";
const char S52434[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Flipboard/1.9.8";
const char S52435[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/1.9.8";
const char S52436[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/1.9.7";
const char S52437[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8426 Flipboard/1.9.8";
const char S52438[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A407 Flipboard/1.9.8";
const char S52439[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A406 Flipboard/1.9.8";
const char S52440[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550 Flipboard/1.9.8";
const char S52441[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8500 Flipboard/1.9.8";
const char S52442[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/1.9.6";
const char S52443[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-S738C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S52444[] = "Mozilla/5.0 (Linux; Android 4.3; SAMSUNG-SGH-I747 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S52445[] = "Mozilla/5.0 (Linux; Android 4.3; SPH-L710 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S52446[] = "Mozilla/5.0 (Linux; Android 4.3; GT-I9305 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S52447[] = "Mozilla/5.0 (Linux; Android 4.3; SCH-R530U Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S52448[] = "Mozilla/5.0 (Linux; Android 4.3; SGH-I747 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S52449[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1b3pre) Gecko/X SeaMonkey/2.0a3pre";
const char S52450[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.13) Gecko/X Mnenhy/0.8.3 SeaMonkey/2.0.8";
const char S52451[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-CA; rv:1.9.1.11pre) Gecko/X SeaMonkey/2.0.6pre";
const char S52452[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ie; Vodafone 845 Build/B220) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52453[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Vodafone 845 Build/B219) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52454[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Vodafone 845 Build/B226) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52455[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-nz; Vodafone 845 Build/B233) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52456[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-nz; Vodafone 845 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52457[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; Vodafone 845 Build/B221) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52458[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; tr-tr; Vodafone 845 Build/B232) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52459[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Vodafone 845 Build/B223) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52460[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; U8120 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52461[] = "Mozilla/5.0 (Linux; U; Android 2.1; pt-pt; Vodafone 845 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52462[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; el-gr; Vodafone 845 Build/B264) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52463[] = "Mozilla/5.0 (Linux; U; Android 2.1-update2; tr-tr; U8120 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52464[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; lt-lt; Vodafone 845 Build/B236) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S52465[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Vodafone 845 Build/B233)";
const char S52466[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Droid Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52467[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Droid Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52468[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Droid Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52469[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Droid Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52470[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Droid Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52471[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52472[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; Droid Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52473[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; Droid Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52474[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Droid Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52475[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Droid Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52476[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; Droid Build/FRG83D)";
const char S52477[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Droid Build/GRI54) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52478[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID Build/FRG83G; liquid.sholes.X.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52479[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; Droid Build/FRG83G)";
const char S52480[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; Droid Build/FRG83D)";
const char S52481[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; Momodesign MD Droid Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52482[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Droid Build/FRG83G)";
const char S52483[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Droid Build/GRH78C; 11000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52484[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; MEDION LIFE P4310 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52485[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; MEDION LIFE P4310 Build/GRJ26) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52486[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; da-dk; MEDION LIFE P4310 Build/GRJ25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52487[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; MEDION LIFE P4310 Build/GRJ29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52488[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; nl-be; MEDION LIFE P4310 Build/GRJ27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52489[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-at; MEDION LIFE P4310 Build/GRJ30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52490[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; MEDION LIFE P4310 Build/GRJ32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52491[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; MEDION LIFE P4310 Build/GRJ34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52492[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-ch; MEDION LIFE P4310 Build/GRJ31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52493[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; MEDION LIFE P4310 Build/GRJ28) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52494[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A101IT Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52495[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; A101IT Build/FROYO)";
const char S52496[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A101B Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52497[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A101C Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52498[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.2 (KHTML, like Gecko) Ubuntu/10.04 Chromium/15.0.874.106 Chrome/15.0.874.106 Safari/535.2";
const char S52499[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Ubuntu/10.04 Chromium/15.0.874.106 Chrome/15.0.874.106 Safari/535.2";
const char S52500[] = "Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52501[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-D710 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52502[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-D710 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52503[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-D710 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52504[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-D710 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52505[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3113 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S52506[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3113 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S52507[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; A501 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52508[] = "Dalvik/1.5.0 (Linux; U; Android 3.0.1; A501 Build/HRI66)";
const char S52509[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6300 Build/FRF91; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52510[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6300 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52511[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6300 Build/GRJ22; Condemned CM7 V15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52512[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; SC-03E Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52513[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SC-03E Build/JRO03C)";
const char S52514[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; YP-GB70 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52515[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; YP-GB70D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52516[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; YP-GB70 Build/GINGERBREAD)";
const char S52517[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5800 XpressMusic) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S52518[] = "Nokia5800 XpressMusic/UCWEB8.8.0.212/50/999";
const char S52519[] = "UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; nokia5800d-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile  3gpp-gba";
const char S52520[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5800w XpressMusic) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S52521[] = "Nokia5800w XpressMusic/UCWEB8.8.0.212/50/800";
const char S52522[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S52523[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190L Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S52524[] = "<![CDATA[Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:24.0) Gecko/X Firefox/24.0";
const char S52525[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One X Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52526[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52527[] = "Mozilla/5.0 (X11; Linux x86_64; PantechP4100 Build/IMM76I) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S52528[] = "python-requests/1.2.3 CPython/2.7.1 Darwin/11.4.2";
const char S52529[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Amazon Kindle Fire Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52530[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Amazon Kindle Fire Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52531[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amazon Kindle Fire Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52532[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Amazon Kindle Fire2 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52533[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amazon Kindle Fire2 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52534[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amazon Kindle Fire Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52535[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amazon Kindle Fire Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S52536[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amazon Kindle Fire2 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52537[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Mobile Safari/535.19 Silk-Accelerated=true";
const char S52538[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI U9510E Build/HuaweiU9510E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52539[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI U9510 Build/HuaweiU9510) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52540[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U9510E Build/HuaweiU9510E) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S52541[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI U9510E Build/HuaweiU9510E)";
const char S52542[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; HUAWEI U9510E Build/HuaweiU9510E) AppleWebKit/534.30 (KHTML, like Gecko) Phi?n b?n/4.0 Mobile Safari/534.30";
const char S52543[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAE5-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S52544[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiae5-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile  3gpp-gba";
const char S52545[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAE5-00.2) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S52546[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAE5-00u) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S52547[] = "NOKIAE5-00/UCWEB8.8.0.212/28/999";
const char S52548[] = "Nokia5233/UCWEB8.9.0.253/50/999";
const char S52549[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5233) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S52550[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia 5233) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S52551[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-02E Build/10.1.D.0.343) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52552[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6602 Build/10.1.1.A.1.253) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52553[] = "Mozilla/5.0 (Linux; Android 4.0.3; A500 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S52554[] = "Mozilla/5.0 (Linux; Android 4.0.4; Iconia A500 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S52555[] = "Mozilla/5.0 (Linux; Android 4.0.4; A500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S52556[] = "Mozilla/5.0 (Series40; Nokia200/11.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S52557[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S52558[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S52559[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S7233E/1.0; U; Bada/1.0; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 NexPlayer/3.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B";
const char S52560[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S52561[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E975K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S52562[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT30at Build/7.0.B.1.152) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52563[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; SlimBrowser/6.01.101)";
const char S52564[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; SlimBrowser/6.01.101)";
const char S52565[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Avant Browser; SlimBrowser/6.01.101)";
const char S52566[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8013 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S52567[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8013 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S52568[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM670 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52569[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM670 Build/Harmonia 2.03 GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52570[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM670 Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52571[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; VM670 Build/Harmonia 2 GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52572[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; VM670 Build/GRI40; CyanogenMod-7.2.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52573[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; ThinkPad Tablet Build/ThinkPadTablet_A310_02) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52574[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; ThinkPad Tablet Build/ThinkPadTablet_A310_02)";
const char S52575[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; ThinkPad Tablet Build/univofaz_A310_02) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52576[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; ThinkPad Tablet Build/OCSAdmin_A310_02) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52577[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; ThinkPad Tablet Build/bladenct_A310_02) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52578[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; ThinkPad Tablet Build/bladencs_A310_02) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52579[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 810)";
const char S52580[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID4 Build/6.7.2-180_DR4-16_M2-37) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52581[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID4 Build/6.7.2-180_DR4-16_M2-37) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S52582[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; M470BSA Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S52583[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; M470BSA Build/JOP40D)";
const char S52584[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_Touch_Pro2/1.86.161.1 (67102)";
const char S52585[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) HTC_Touch_Pro2_T7379";
const char S52586[] = "HTC_Touch_Pro2_T7373 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_S200";
const char S52587[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_Touch_Pro2/1.14.161.5 (50207)";
const char S52588[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_Touch_Pro2/1.14.161.6 (50207)";
const char S52589[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_Touch_Pro2/1.86.161.0 (67102)";
const char S52590[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Sprint T7380";
const char S52591[] = "HTC_Touch_Pro2_T7373 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S52592[] = "HTC_Touch_Pro2_T7381/V1.19.841.3 (51489) Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S52593[] = "Mozilla/4.0 (compatible; MSIE 5.01; Windows CE; IEMobile 8.12; MSIEMobile6.0) Sprint T7380";
const char S52594[] = "HTC_Touch_Pro2_T7373/480x800 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S52595[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 480x800; HTC_Touch_Pro2_T7385-Orange; OpVer 122.190.1.610";
const char S52596[] = "HTC_Touch_Pro2_T7373/Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S52597[] = "HTC_Touch_Pro2_T7381 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S52598[] = "HTC_Touch_Pro2_T7381 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) V1.93.841.0 (81210)";
const char S52599[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Swisscom/1.0/HTC Touch Pro2/1.86.166.1 (67102)";
const char S52600[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 480x800; HTC_Touch_Pro2_T7385-Orange; OpVer 122.190.2.731";
const char S52601[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile6.0) Sprint T7380";
const char S52602[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Swisscom/1.0/HTC Touch Pro2/1.86.167.2 (67102)";
const char S52603[] = "Mozilla/5.0 (Windows NT 6.0; rv:9.0.1) Gecko/X Firefox/9.0.1 Firefox/9.0.1";
const char S52604[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:9.0) Gecko/X Firefox/9.0";
const char S52605[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC6435LVW Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52606[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52607[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S52608[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8013 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S52609[] = "Mozilla/5.0 (Linux;U;Android 4.0.3;en-us;Digitac Ultratab;C725 Build/ICS) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S52610[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C725 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S52611[] = "Mozilla/5.0 (Linux;U;Android 4.0.4;en-us;Digitac Ultratab;C725 Build/ICS) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S52612[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Digitac Ultratab;C725 Build/ICS) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52613[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Digitac Ultratab;C725 Build/ICS) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52614[] = "Mozilla/5.0 (Linux; Android 4.0.4; LePanII Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S52615[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I317 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52616[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7105 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52617[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52618[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T889 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52619[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Note II Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52620[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L900 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52621[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7105 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S52622[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R950 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52623[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I317 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52624[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S52625[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P3100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S52626[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S52627[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP9090 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52628[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT890 Build/9.8.2I-50_SMI-26) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52629[] = "Nokia2630/2.0 (06.82) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S52630[] = "Nokia2630/2.0 (06.82) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2630";
const char S52631[] = "Nokia2630_CMCC/2.0 (05.62) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S52632[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID7035 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S52633[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; MID7035C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S52634[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LG730 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S52635[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Maxthon/3.0)";
const char S52636[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Maxthon/3.0)";
const char S52637[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Maxthon/3.0)";
const char S52638[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19";
const char S52639[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; A511 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S52640[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; A511 Build/JZO54K)";
const char S52641[] = "Mozilla/5.0 (PlayBook; U; RIM Tablet OS 1.0.0; en-US) AppleWebKit/534.8+ (KHTML, like Gecko) Version/0.0.1 Safari/534.8+";
const char S52642[] = "Mozilla/5.0 (PlayBook; U; RIM Tablet OS 1.0.0; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/0.0.1 Safari/534.11+";
const char S52643[] = "Mozilla/5.0 (PlayBook; U; RIM Tablet OS 1.0.0; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.7 Safari/534.11+";
const char S52644[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P768 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52645[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P769 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52646[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P760 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52647[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P769 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S52648[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S52649[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; T-Hub2 Build/TVA301TELBG3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52650[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; T-Hub2 Build/TC-301-L27.G.3-EVT-) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52651[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; T-Hub2 Build/TC-301-L27.G.7-DVT-) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52652[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; T-Hub2 Build/TVA301TELBG3)";
const char S52653[] = "Mozilla/5.0 (Linux; Android 4.0.3; A500 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S52654[] = "Mozilla/5.0 (Linux; Android 4.0.4; A500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S52655[] = "Mozilla/5.0 (Linux; Android 4.0.4; Iconia A500 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S52656[] = "Mozilla/5.0 (Linux; Android 4.0.4; A500 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S52657[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR6410LVW Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52658[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LVW Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52659[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR6410LRA Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52660[] = "HTC_Magic Mozilla/5.0 (Linux; U; Android 1.5; en-ca; Build/2.17.631.2) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S52661[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-hk; HTC Magic_MR Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S52662[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-tw; HTC Magic MR Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S52663[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-ie; HTC Magic Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S52664[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-in; HTC_Magic Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S52665[] = "HTC_Magic Mozilla/5.0 (Linux; U; Android 1.5; en-ca; Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S52666[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-gb; HTC Magic Build/CRC1) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S52667[] = "Mozilla/5.0 (Linux; U; Android 1.5; pl-pl; HTC Magic Build/2.17.401.2) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S52668[] = "Mozilla/5.0 (Linux; U; Android 1.5; it-; HTC Magic Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S52669[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; sapphire; HTC_Magic) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S52670[] = "Mozilla/5.0 (Linux; U; Android 1.5; fr-fr; HTC Magic Build/CRB17) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S52671[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-au; HTC Magic Build/CRB43) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S52672[] = "Mozilla/5.0 (Linux; U; Android 1.5; de-de; HTC Magic Build/CRC24) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S52673[] = "Mozilla/5.0 (Linux; U; Android 1.5; de-de; HTC Magic Build/PLAT-RC33) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 FirePHP/0.3";
const char S52674[] = "Mozilla/5.0 (Linux; U; Android 1.5; fr-fr; Mozilla/5.0 (Linux; U; Android 1.5; fr-fr; HTC Magic Build/CRB43) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S52675[] = "Mozilla/5.0 (Linux; U; Android 1.5; de-de; HTC Magic Build/PLAT-RC33) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 FirePHP/0.3";
const char S52676[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC One Build/JWR66V.H10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52677[] = "Dalvik/1.6.0 (Linux; U; Android 4.3; HTC One Build/JSS15J)";
const char S52678[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC_One Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52679[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC One Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52680[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52681[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC One Build/BlackJelly 1.1 JWR66V.H10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52682[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC One Build/JWR66Y.H1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52683[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC_PN071 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52684[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC_PN07120 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52685[] = "Mozilla/5.0 (Linux; U; Android 4.3; zh-tw; HTC One 801e Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52686[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTCONE Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52687[] = "Dalvik/1.6.0 (Linux; U; Android 4.3; HTC One 801e Build/JSS15J)";
const char S52688[] = "Dalvik/1.6.0 (Linux; U; Android 4.3; HTCONE Build/JSS15J)";
const char S52689[] = "Mozilla/5.0 (Linux; Android 4.0.3; Sony Tablet S Build/TISU0143) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S52690[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; MDA_Compact_V/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S52691[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) T-Mobile_Rhodium";
const char S52692[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) T-Mobile_Maple";
const char S52693[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) T-Mobile_LEO";
const char S52694[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) USCCHTC6875";
const char S52695[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) GSmart-S1205;Mozilla/4.0(compatible;MSIE6.0;PPC; 240x400)";
const char S52696[] = "Mozilla/5.0 (compatible; MSIE 6.0; PPC; Tablet PC; Trident/4.0)";
const char S52697[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySO-01E Build/9.0.G.0.238) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52698[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySO-01E Build/9.0.G.0.247) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52699[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-01E Build/9.0.G.0.247) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52700[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySO-01E Build/9.0.G.1.76) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52701[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-01E Build/9.0.G.0.238) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52702[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-01E Build/9.0.G.1.76) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52703[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySO-01E Build/9.0.G.1.108) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52704[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-01E Build/9.0.G.1.108) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52705[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SO-01E Build/9.0.G.1.108)";
const char S52706[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-ca; GT-I5700R Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S52707[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-gb; GT-I5700 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S52708[] = "Mozilla/5.0 (Linux; U; Android 1.5; fr-fr; GT-I5700-ORANGE/I5700OFRIK1 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S52709[] = "Mozilla/5.0 (Linux; U; Android 1.5; es-us; GT-I5700L Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S52710[] = "Mozilla/5.0 (Linux; U; Android 1.5; ar-; GT-I5700 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S52711[] = "Mozilla/5.0 (Linux; U; Android 1.5; iw-; GT-I5700L Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S52712[] = "Mozilla/5.0 (Linux; U; Android 1.5; fr-fr; GT-I5700 Build/CUPCAKE) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S52713[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-S730G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52714[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-S730G Build/IMM76D)";
const char S52715[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3";
const char S52716[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.13 (KHTML, like Gecko) Version/5.1 Mobile/10A5338d Safari/7534.48.3";
const char S52717[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari Version/5.1";
const char S52718[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Safari Version/5.1";
const char S52719[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari Version/5.1";
const char S52720[] = "Mozilla/5.0 (iPhone; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10A403 Safari/7536.26";
const char S52721[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Version/5.1 Safari/7534.48.3";
const char S52722[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari Version/5.1";
const char S52723[] = "Mozilla/5.0 (iPhone; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10A523 Safari/7536.26";
const char S52724[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/10A403 Safari/7534.48.3";
const char S52725[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Version/5.1 Safari/7534.48.3";
const char S52726[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari Version/5.1";
const char S52727[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/10A523 Safari/7534.48.3";
const char S52728[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 60 like Mac OS X) AppleWebKit/536.13 (KHTML like Gecko) Version/5.1 Mobile/10A5338d Safari/7534.48.3";
const char S52729[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B179 Safari/7534.48.3";
const char S52730[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 601 like Mac OS X) AppleWebKit/536.26 (KHTML like Gecko) Mobile/10A525 Safari Version/5.1";
const char S52731[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S52732[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; Juggernaut; Semaphore)";
const char S52733[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; id104)";
const char S52734[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; Qualcomm; Fluid)";
const char S52735[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; id302)";
const char S52736[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; RM-825_developer_249)";
const char S52737[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; SAMSUNG; SCH-R860U)";
const char S52738[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; HTCPO881; SPRINT)";
const char S52739[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia 925)";
const char S52740[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; RIO_U)";
const char S52741[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; id308)";
const char S52742[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; SAMSUNG; SGH-I187)";
const char S52743[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia 925T)";
const char S52744[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; 909; ANZ875)";
const char S52745[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; id307)";
const char S52746[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 1520)";
const char S52747[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; 909; Vodafone)";
const char S52748[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia 625H)";
const char S52749[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 929)";
const char S52750[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; 909; ANZ892)";
const char S52751[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; HTCPO881; X)";
const char S52752[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S52753[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52754[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Velocitymicro/T410) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52755[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.29";
const char S52756[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S52757[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.28 (KHTML, like Gecko) Version/3.2.2 Safari/525.28.1";
const char S52758[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; hu-HU) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.13";
const char S52759[] = "python-requests/1.1.0 CPython/2.7.2 Darwin/12.3.0";
const char S52760[] = "python-requests/1.2.3 CPython/2.7.2 Darwin/12.3.0";
const char S52761[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PMP3370B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52762[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; PMP3370B Build/IML74K)";
const char S52763[] = "Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; XOOM 2 Build/1.6.0_211.04) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S52764[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; XOOM 2 Build/1.6.0_218.3-MZ615) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S52765[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; XOOM 2 Build/1.6.0_218.3-MZ615) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52766[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; XOOM 2 Build/1.6.0_218.3-MZ615)";
const char S52767[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; XOOM 2 Build/1.6.0M_268-MZ615) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S52768[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; XOOM 2 Build/1.6.0_222) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S52769[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 Build/1.6.0_268.4-MZ615) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S52770[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; de-de; XOOM 2 Build/1.6.0_249.1-MZ615) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S52771[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; XOOM 2 Build/1.6.0M_279_MZ615) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S52772[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; XOOM 2 Build/1.6.0M_278-MZ615) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S52773[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; XOOM 2 Build/1.6.0M_280_MZ615) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S52774[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; XOOM 2 Build/1.6.0M_278-MZ615) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52775[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 Build/1.6.0_268.4-MZ615) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52776[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.2; XOOM 2 Build/1.6.0_268.4-MZ615)";
const char S52777[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S52778[] = "Mozilla/5.0 (Linux; Android 4.1; Nexus S 4G Build/JRN84D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52779[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus S 4G Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52780[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus S 4G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52781[] = "Mozilla/5.0 (Linux; Android 4.1.1; Full Android on Crespo4G Build/PHONELAB_JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52782[] = "Opera/9.80 (X11; Linux i686; U; en) Presto/2.2.15 Version/10.00";
const char S52783[] = "Opera/9.80 (X11; Linux i686; U; en) Presto/2.2.15 Version/10.01";
const char S52784[] = "Opera/10.00 (X11; Linux i686 ; U; en) Presto/2.2.0";
const char S52785[] = "Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux i686 ; en) Opera 10.00";
const char S52786[] = "Opera/10.0 (X11; Linux i686; U)";
const char S52787[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One X+ Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S52788[] = "Mozilla/5.0 (Series40; Nokia200/10.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.11";
const char S52789[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S6012 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52790[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT-S6012B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52791[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-S6012 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52792[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-S6012 Build/IMM76D)";
const char S52793[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-S960L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52794[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-S960L Build/JZO54K)";
const char S52795[] = "Mozilla/5.0 (Linux; Android 4.0.4; cm_tenderloin Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S52796[] = "Mozilla/5.0 (Linux; Android 4.0.3; cm_tenderloin Build/GWK74) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S52797[] = "Mozilla/5.0 (Series40; Nokia306/03.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S52798[] = "Mozilla/5.0 (Series40; Nokia306/03.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5";
const char S52799[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT605 Build/5.5.1Q-JORIAN-108-TA-261) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52800[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT605 Build/5.5.1Q-JORIAN-108-TA-262) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52801[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT605 Build/5.5.1Q-JORIAN-108-TA-270) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52802[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.6.31 Version/10.70";
const char S52803[] = "Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.6.31 Version/10.70";
const char S52804[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2NokiaE73/044.008; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.9 3gpp-gba";
const char S52805[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE73/044.008; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.9 3gpp-gba";
const char S52806[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.0.400) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52807[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS03 Build/S3251) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52808[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS03 Build/S9090) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52809[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS03 Build/S5160) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52810[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS03 Build/S3080) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52811[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS03 Build/S4110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52812[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS03 Build/S8301) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52813[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; IS03 Build/S8301)";
const char S52814[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S52815[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S52816[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S52817[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S52818[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S52819[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S52820[] = "iTunes-iPhone/6.1.3 (4; 16GB; dt:27)";
const char S52821[] = "iTunes-iPhone/6.1.3 (6; 16GB; dt:73)";
const char S52822[] = "iTunes-iPhone/6.1.3 (4; 8GB; dt:27)";
const char S52823[] = "iTunes-iPhone/6.1.3 (6; 32GB; dt:73)";
const char S52824[] = "iTunes-iPhone/6.1.3 (4; 32GB; dt:27)";
const char S52825[] = "iTunes-iPhone/6.1.3 (3; 16GB; dt:24)";
const char S52826[] = "iTunes-iPhone/6.1.3 (6; 64GB; dt:73)";
const char S52827[] = "iTunes-iPhone/6.1.3 (5; 8GB; dt:70)";
const char S52828[] = "iTunes-iPhone/6.1.3 (3; 8GB; dt:24)";
const char S52829[] = "iTunes-iPhone/6.1.4 (6; 32GB; dt:82)";
const char S52830[] = "iTunes-iPhone/6.1.4 (6; 16GB; dt:82)";
const char S52831[] = "iTunes-iPhone/6.1.3 (8GB; dt:27)";
const char S52832[] = "iTunes-iPhone/6.1.4 (6; 16GB; dt:79)";
const char S52833[] = "iTunes-iPhone/6.1.3 (5; 16GB; dt:70)";
const char S52834[] = "iTunes-iPhone/6.1.4 (6; 64GB; dt:82)";
const char S52835[] = "iTunes-iPhone/6.1.4 (6; 32GB; dt:79)";
const char S52836[] = "iTunes-iPhone/6.1.3 (3; 32GB; dt:24)";
const char S52837[] = "iTunes-iPhone/6.1.4 (6; 64GB; dt:79)";
const char S52838[] = "iTunes-iPhone/6.1.3 (5; 32GB; dt:70)";
const char S52839[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaE63) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S52840[] = "NokiaE63/UCWEB8.8.0.212/28/999";
const char S52841[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiae63-1) U2/1.0.0 UCBrowser/8.8.0.227 U2/1.0.0 Mobile";
const char S52842[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; ZTE-BLADE Build/OUK_B05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52843[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; ZTE-BLADE Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52844[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; sv-se; ZTE-BLADE Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52845[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; ZTE-BLADE Build/GRJ22)";
const char S52846[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; ZTE-BLADE Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52847[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; ZTE-BLADE Build/GRJ22)";
const char S52848[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE-BLADE Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S52849[] = "Mozilla/5.0 Firefox/3.0.5";
const char S52850[] = "Firefox/3.0.5";
const char S52851[] = "Mozilla/5.0 Firefox/3.0.4";
const char S52852[] = "Firefox/3.0.7";
const char S52853[] = "Mozilla/5.0 ( en-US; rv:1.9.0.3) Gecko/X Firefox/3.0.3";
const char S52854[] = "Mozilla/5.0 (www.vouchers.im; en-US; rv:1.9) Gecko/X Firefox/3.0";
const char S52855[] = "Firefox/3.05";
const char S52856[] = "Firefox/3.0.3";
const char S52857[] = "Mozilla/5.0 (SaferSurf rv:1.9) Gecko/X Firefox/3.0";
const char S52858[] = "Firefox/3.0";
const char S52859[] = "Firefox/3.0.15";
const char S52860[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9A405 Safari/7534.48.3";
const char S52861[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9A334 Safari/7534.48.3";
const char S52862[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52863[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC VLE_U Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52864[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC VLE_U Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52865[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One S Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S52866[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; LG-P700 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52867[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P700/P700 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52868[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P700 Build/JZO54K)";
const char S52869[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-P700-Orange Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52870[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; LG-P700/V20c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52871[] = "Mozilla/5.0 (X11; Linux) AppleWebKit/534.26+ Midori/0.4";
const char S52872[] = "Mozilla/5.0 (X11; Linux) AppleWebKit/535.22 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/535.22 Midori/0.4";
const char S52873[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.22 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/535.22 Midori/0.4";
const char S52874[] = "Mozilla/5.0 (X11; Linux) AppleWebKit/535.22 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/535.22 Midori/\"0\".\"4\"";
const char S52875[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.22 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/535.22 Midori/0.4";
const char S52876[] = "Mozilla/5.0 (X11; Linux; rv:2.0.1) Gecko/X Firefox/4.0.1 Midori/\"0\".\"4\"";
const char S52877[] = "iTunes-iPad-M/4.3.5 (64GB)";
const char S52878[] = "iTunes-iPad-M/4.3.5 (2; 32GB)";
const char S52879[] = "iTunes-iPad-M/5.0.1 (2; 64GB)";
const char S52880[] = "iTunes-iPad-M/5.1.1 (64GB; dt:28)";
const char S52881[] = "iTunes-iPad-M/5.1.1 (2; 16GB; dt:75)";
const char S52882[] = "iTunes-iPad-M/5.1.1 (16GB; dt:28)";
const char S52883[] = "iTunes-iPad-M/4.3.5 (2; 16GB)";
const char S52884[] = "iTunes-iPad-M/4.2.1 (16GB)";
const char S52885[] = "iTunes-iPad-M/4.2.1 (32GB)";
const char S52886[] = "iTunes-iPad-M/5.1.1 (32GB; dt:28)";
const char S52887[] = "iTunes-iPad-M/6.0.1 (5; 64GB; dt:85)";
const char S52888[] = "iTunes-iPad-M/6.1.3 (5; 32GB; dt:78)";
const char S52889[] = "iTunes-iPad-M/6.1.3 (2; 32GB; dt:75)";
const char S52890[] = "iTunes-iPad-M/5.0.1 (3; 32GB)";
const char S52891[] = "iTunes-iPad-M/6.1.3 (2; 64GB; dt:75)";
const char S52892[] = "iTunes-iPad-M/6.1.3 (5; 64GB; dt:78)";
const char S52893[] = "iTunes-iPad-M/6.1.3 (5; 16GB; dt:78)";
const char S52894[] = "iTunes-iPad-M/6.1.3 (5; 32GB; dt:85)";
const char S52895[] = "iTunes-iPad-M/6.1.3 (5; 16GB; dt:88)";
const char S52896[] = "iTunes-iPad-M/6.1.3 (5; 16GB; dt:85)";
const char S52897[] = "iTunes-iPad-M/6.1.3 (3; 64GB; dt:76)";
const char S52898[] = "iTunes-iPad-M/6.1.3 (5; 32GB; dt:87)";
const char S52899[] = "iTunes-iPad-M/6.1.3 (4; 64GB; dt:78)";
const char S52900[] = "iTunes-iPad-M/6.1.3 (5; 32GB; dt:84)";
const char S52901[] = "iTunes-iPad-M/6.1.3 (5; 32GB; dt:88)";
const char S52902[] = "iTunes-iPad-M/6.1.3 (5; 64GB; dt:88)";
const char S52903[] = "iTunes-iPad-M/6.1.3 (5; 16GB; dt:87)";
const char S52904[] = "iTunes-iPad-M/6.1.3 (4; 32GB; dt:78)";
const char S52905[] = "iTunes-iPad-M/6.1.3 (3; 32GB; dt:76)";
const char S52906[] = "iTunes-iPad-M/6.1.3 (3; 16GB; dt:76)";
const char S52907[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8J2 Safari/7534.48.3";
const char S52908[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8L1 Safari/7534.48.3";
const char S52909[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8F190 Safari/7534.48.3";
const char S52910[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8G4 Safari/7534.48.3";
const char S52911[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8K2 Safari/7534.48.3";
const char S52912[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8H7 Safari/7534.48.3";
const char S52913[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 10 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S52914[] = "Mozilla/5.0 (Linux; Android 4.2; Nexus 10 Build/JVP15P) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S52915[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 10 Build/JOP40F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S52916[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; GALAXY_Tab Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52917[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SCH-I800 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52918[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; SCH-I800 Build/FROYO)";
const char S52919[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; U8800Pro Build/HuaweiU8800Pro) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S52920[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; U8800Pro Build/HuaweiU8800Pro) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52921[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; U8800Pro Build/HuaweiU8800Pro)";
const char S52922[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; U8800Pro Build/HuaweiU8800Pro) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S52923[] = "Mozilla/5.0 (Linux; Android 4.0.3;  GT-I9100P Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S52924[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100P Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52925[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100P Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52926[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.0.200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52927[] = "Mozilla/5.0 (Linux; Android 4.2.2; One Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52928[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52929[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One 801e Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52930[] = "Mozilla/5.0 (Linux; Android 4.2.1; HTC_One Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52931[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC_PN071 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S52932[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-P5200 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S52933[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P5200 Build/JDQ39)";
const char S52934[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; cs-cz; SAMSUNG GT-P5200/P5200XXUAMFC Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S52935[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Mobile Safari/535.19 Silk-Accelerated=true";
const char S52936[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Internet TV Box Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52937[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Internet TV Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52938[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; EPAD Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52939[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; MUSN-T800 Build/HTK55D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S52940[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Thrive Build/v_1.2) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52941[] = "Mozilla/5.0 (Linux; U; Android 3.2; es-es; T1001 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52942[] = "Mozilla/5.0 (Linux; U; Android 3.2; he-il; Liberty Tab G100 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52943[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Speed_MB-Z4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52944[] = "Mozilla/5.0 (Linux; U; Android 3.2; tr-tr; Nirvana_Tablet Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52945[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; THRiVE Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52946[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; ja-jp; EB-A71GJ Build/HTK55D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S52947[] = "Mozilla/5.0 (Linux; U; Android 3.2; tr-tr; P10MAN Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52948[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Huawei S7-312u Build/HuaweiS7-312uB021) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52949[] = "Mozilla/5.0 (Linux; U; Android 3.2; pt-br; TA1013 Build/SEMP_X_001) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52950[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; zh-cn; M1101AS Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52951[] = "Mozilla/5.0 (Linux; U; Android 3.2; ru-ru; ts9708b Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52952[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; pt-br; YPY10FTA Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52953[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; T91 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52954[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Revue Build/KA2X-X.X) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52955[] = "Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; Camangi-Mangrove7 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S52956[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; ZY Z6330S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S52957[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; pt-br; YPY_10FTA Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52958[] = "Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHV-E140K Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52959[] = "Mozilla/5.0 (Linux; U; Android 3.2; ru-ru; ts9705b Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52960[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; NSZ-GS7/GX70 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52961[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; M97B Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52962[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; V66 4G Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52963[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-tw; IGS-1000 Build/IGS_X_001) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52964[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; zh-cn; M170AT Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52965[] = "Mozilla/5.0 (Linux; U; Android 3.2; ru-ru; ts1009b Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52966[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us;USCCADT71US Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52967[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; th-th; FE6 Build/HTK55D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S52968[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; zh-cn; IdeaTabV2010A Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52969[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; zh-cn; M1101AT Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52970[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; VAP430 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S52971[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Transformer TF101 Build/JB_MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S52972[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Transformer TF101 Build/JB_MR1.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S52973[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Transformer TF101 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S52974[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; Transformer TF101 Build/JDQ39)";
const char S52975[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Transformer Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52976[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Transformer Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52977[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Safari/535.19 Silk-Accelerated=false";
const char S52978[] = "Mozilla/5.0 (Series40; NokiaC2-03/07.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S52979[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Ubuntu/10.04 Chromium/10.0.648.133 Chrome/10.0.648.133 Safari/534.16";
const char S52980[] = "Mozilla/5.0 (X11; U; Linux armv7l; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Ubuntu/10.04 Chromium/10.0.648.133 Chrome/10.0.648.133 Safari/534.16";
const char S52981[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.15 (KHTML, like Gecko) Ubuntu/10.04 Chromium/10.0.612.3 Chrome/10.0.612.3 Safari/534.15";
const char S52982[] = "Mozilla/5.0 (Windows NT 5.1; rv:15.4) Gecko/X Firefox/15.4.1 PaleMoon/15.4.1";
const char S52983[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:15.4) Gecko/X Firefox/15.4.1 PaleMoon/15.4.1";
const char S52984[] = "Mozilla/5.0 (Series40; NokiaX2-05/08.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S52985[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52986[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310N Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52987[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52988[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S52989[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xoom Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S52990[] = "Mozilla/5.0 (Linux; Android 4.1.1; Xoom Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S52991[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; QQBrowser/7.3.8581.400)";
const char S52992[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; QQBrowser/7.3.11251.400)";
const char S52993[] = "Mozilla/5.0 (compatible; MSIE 9.0; qdesk 2.4.1264.203; Windows NT 6.2; WOW64; Trident/6.0; QQBrowser/7.3.8581.400)";
const char S52994[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-N7000 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52995[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-N7000 Build/JOP40D)";
const char S52996[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7000 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52997[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7000 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52998[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7000 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S52999[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-N7000 Build/JDQ39E)";
const char S53000[] = "SAMSUNG-GT-S5270L/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S53001[] = "SAMSUNG-GT-S5270K/S5270KDDKK1 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S53002[] = "SAMSUNG-GT-S5270Y/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S53003[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows ME) Opera 5.11 [en]";
const char S53004[] = "Opera/5.11 (Windows ME; U) [ru]";
const char S53005[] = "Mozilla/6.0 (compatible; MSIE 7.03; Windows ME) Opera 5.19 [jp]";
const char S53006[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows ME) Opera 5.12 [de]";
const char S53007[] = "Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; fr_crawler)";
const char S53008[] = "Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; fr_crawler)/V1";
const char S53009[] = "Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; FeedFinder-2.0; http://bloggz.se/crawler)";
const char S53010[] = "Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; BlogCrawler-Fetchy; http://bloggz.se/crawler)";
const char S53011[] = "Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; MSIECrawler)";
const char S53012[] = "Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; YComp 5.0.2.6; MSIECrawler)";
const char S53013[] = "Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; FeedFetcher-2.0; http://bloggz.se/crawler)";
const char S53014[] = "Nokia2700c-2/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia2700c-2) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S53015[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P3100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S53016[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3100 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S53017[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT27i Build/6.1.1.B.1.54) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53018[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xperia Sola Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53019[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.402 Mobile Safari/537.35+";
const char S53020[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.483 Mobile Safari/537.35+";
const char S53021[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.552 Mobile Safari/537.35+";
const char S53022[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.519 Mobile Safari/537.35+";
const char S53023[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.810 Mobile Safari/537.35+";
const char S53024[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.852 Mobile Safari/537.35+";
const char S53025[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1337 Mobile Safari/537.35+";
const char S53026[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.341 Mobile Safari/537.35+";
const char S53027[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.936 Mobile Safari/537.35+";
const char S53028[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1023 Mobile Safari/537.35+";
const char S53029[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1047 Mobile Safari/537.35+";
const char S53030[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1323 Mobile Safari/537.35+";
const char S53031[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1425 Mobile Safari/537.35+";
const char S53032[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1371 Mobile Safari/537.35+";
const char S53033[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1155 Mobile Safari/537.35+";
const char S53034[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1443 Mobile Safari/537.35+";
const char S53035[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1113 Mobile Safari/537.35+";
const char S53036[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1629 Mobile Safari/537.35+";
const char S53037[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1581 Mobile Safari/537.35+";
const char S53038[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1719 Mobile Safari/537.35+";
const char S53039[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1671 Mobile Safari/537.35+";
const char S53040[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1725 Mobile Safari/537.35+";
const char S53041[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1737 Mobile Safari/537.35+";
const char S53042[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1731 Mobile Safari/537.35+";
const char S53043[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1761 Mobile Safari/537.35+";
const char S53044[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1743 Mobile Safari/537.35+";
const char S53045[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1767 Mobile Safari/537.35+";
const char S53046[] = "Mozilla/5.0 (Symbian3; U; en-us; Nokia500) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile";
const char S53047[] = "Nokia500UCBrowser/8.7.1.234/51/355/UCWEB";
const char S53048[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia500) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba";
const char S53049[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/531.0 (KHTML, like Gecko) Chrome/3.0.190.1 Safari/531.0";
const char S53050[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M-270 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S53051[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M-270 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S53052[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; M-270 Build/IML74K)";
const char S53053[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; X325a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53054[] = "Mozilla/5.0 (Windows NT 6.0; rv:18.0) Gecko/X Firefox/18.0 SeaMonkey/2.15.2";
const char S53055[] = "Mercury/7.3 CFNetwork/609.1.4 Darwin/13.0.0";
const char S53056[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; Huawei; H883G; HuaweiH883G)";
const char S53057[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ609 4G Build/1.6.0M-218.14.Demo2-MZ609) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S53058[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ609 4G Build/1.6.0M-218.14-MZ609) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S53059[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ609 Build/1.6.0M-218.14-MZ609) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S53060[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; MZ609 4G Build/1.6.0M_279_MZ609) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S53061[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; MZ609 Build/1.6.0M_279_MZ609) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S53062[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.2; MZ609 Build/1.6.0M_279_MZ609)";
const char S53063[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; MZ609 Build/1.6.0M-218.14-MZ609)";
const char S53064[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC HD2 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53065[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; NexusHD2 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53066[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC HD2 Build/JRO03R; CyanogenMod-cmtight_leo) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53067[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZeJstersHD2 Build/JZO54K; CyanogenMod-JellyTime_R32) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53068[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; SGH-i917R)";
const char S53069[] = "Mozilla/5.0 (Linux; Android 4.1.2; SM-T210 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S53070[] = "Mozilla/5.0 (Linux; Android 4.1.2; SM-T210R Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S53071[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/3.0.4 Safari/523.12";
const char S53072[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; AZ210A-ORANGE Build/AZ210A_GB_01.01I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53073[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; AZ210A Build/AZ210A_GB_01.01I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53074[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; AZ210B-ORANGE Build/AZ210B_GB_01.01I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53075[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; AZ210B Build/AZ210B_GB_01.01I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53076[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; AZ210A-ORANGE Build/AZ210A_GB_01.06I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53077[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; AZ210B-ORANGE Build/AZ210B_GB_01.06I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53078[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; AZ210A Build/AZ210A_GB_01.06I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53079[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; AZ210A Build/AZ210A_GB_01.01I)";
const char S53080[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S53081[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S53082[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S53083[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S53084[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S53085[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8L1 Safari/6531.22.7";
const char S53086[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8J2 Safari/6531.22.7";
const char S53087[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8K2 Safari/6531.22.7";
const char S53088[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293";
const char S53089[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293";
const char S53090[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8H7 Safari/6531.22.7";
const char S53091[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S53092[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S53093[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S53094[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293";
const char S53095[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S53096[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S53097[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8J2 Safari/6531.22.7";
const char S53098[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293";
const char S53099[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; vi-vn) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8J2 Safari/6531.22.7";
const char S53100[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8K2 Safari/6531.22.7";
const char S53101[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; vi-vn) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8F190 Safari/6531.22.7";
const char S53102[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293";
const char S53103[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-gb) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8k2 Safari/6531.22.7";
const char S53104[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; es-es) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S53105[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; vi-vn) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8F190 Safari/6531.22.7";
const char S53106[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:21.0) Gecko/X Firefox/21.0 AlexaToolbar/alxf-2.18";
const char S53107[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI H881C Build/HuaweiH881C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53108[] = "Mozilla/5.0 (Windows NT 5.1; rv:15.0) Gecko/X Firefox/15.2 PaleMoon/15.2";
const char S53109[] = "Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:15.2) Gecko/X Firefox/15.2.1-x64 PaleMoon/15.2.1-x64";
const char S53110[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:15.2) Gecko/X Firefox/15.2.1 PaleMoon/15.2.1";
const char S53111[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.0; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727)";
const char S53112[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.0; Trident/6.0)";
const char S53113[] = "Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.0; SV1)";
const char S53114[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.0; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S53115[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.0; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S53116[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100G Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53117[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53118[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-i9100 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53119[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100G Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53120[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39E) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53121[] = "Mozilla/5.0 (Linux; Android 4.1.2; ME371MG Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S53122[] = "Mozilla/5.0 (Windows NT 5.0; rv:10.0.12) Gecko/X Firefox/10.0.12";
const char S53123[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53124[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; QMobile A8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53125[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; A8 Build/IMM76D)";
const char S53126[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; Garminfone Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 MobileSafari/525.20.1; A50-V4.0.12-user-20100414";
const char S53127[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; Garminfone Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1; A50-V4.0.21-user-20100604";
const char S53128[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us;Garminfone Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1; A50-V4.0.12-user-X";
const char S53129[] = "Mozilla/5.0 (Linux; U;Android 1.6; en-us; Garminfone Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1; A50-V4.0.21-user-X";
const char S53130[] = "Mozilla/5.0 (Linux; U; Android1.6; en-us; Garminfone Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1; A50-V4.0.12-user-X";
const char S53131[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; GarminfoneBuild/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1; A50-V4.0.12-user-X";
const char S53132[] = "Mozilla/5.0 (Linux; U; Android 1.6; fr-ca; n?vifone A50 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1; A50-V4.0.29-user-X";
const char S53133[] = "NSPlayer/12.0.7600.16385 WMFSDK/12.0";
const char S53134[] = "NSPlayer/12.0.7600.16385";
const char S53135[] = "NSPlayer/12.0.7601.17514";
const char S53136[] = "NSPlayer/12.00.9200.16384 WMFSDK/12.00.9200.16384";
const char S53137[] = "NSPlayer/12.00.9200.16420 WMFSDK/12.00.9200.16420";
const char S53138[] = "NSPlayer/12.00.9200.16437 WMFSDK/12.00.9200.16437";
const char S53139[] = "NSPlayer/12.00.9200.16409 WMFSDK/12.00.9200.16409";
const char S53140[] = "NSPlayer/12.0.9200.16384 WMFSDK/12.0";
const char S53141[] = "NSPlayer/12.0.9200.16384";
const char S53142[] = "NSPlayer/12.00.7601.17514 WMFSDK/12.00.7601.17514 SE 2.X MetaSr 1.0";
const char S53143[] = "NSPlayer/12.0.7601.17514 WMFSDK/12.0 SE 2.X MetaSr 1.0";
const char S53144[] = "NSPlayer/12.00.7600.16597 WMFSDK/12.00.7600.16597 SE 2.X MetaSr 1.0";
const char S53145[] = "NSPlayer/12.0.7600.16385 WMFSDK/12.0 SE 2.X MetaSr 1.0";
const char S53146[] = "NSPlayer/12.00.7600.16385 WMFSDK/12.00.7600.16385 SE 2.X MetaSr 1.0";
const char S53147[] = "NSPlayer/12.00.9200.16550 WMFSDK/12.00.9200.16550";
const char S53148[] = "NokiaX2-02/2.0 (11.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-02) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S53149[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53150[] = "Mozilla/5.0 (Series40; NokiaX3-02/07.16; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.34";
const char S53151[] = "Mozilla/5.0 (Series40; NokiaX3-02.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S53152[] = "NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiax2-01) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S53153[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B144 Safari/7534.48.3";
const char S53154[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B141 Safari/7534.48.3";
const char S53155[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B146 Safari/7534.48.3";
const char S53156[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B329 Safari/7534.48.3";
const char S53157[] = "NokiaC3-00/5.0 (08.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac3-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile";
const char S53158[] = "Mozilla/5.0 (Series40; Nokia205/03.18; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S53159[] = "Mozilla/5.0 (Series40; Nokia205.1/03.18; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S53160[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGP321 Build/10.3.1.A.0.244) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S53161[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S53162[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I535 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S53163[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T999 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S53164[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S53165[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-I9300T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S53166[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-T999V Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S53167[] = "SAMSUNG-GT-C3520/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S53168[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E970 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S53169[] = "NokiaX2-02/2.0 (11.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiax2-02) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S53170[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC VLE_U Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53171[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53172[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53173[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC VLE_U Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53174[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SAMSUNG-SGH-I717 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53175[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I717 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53176[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I717 Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53177[] = "Mozilla/5.0 (Series40; NokiaC2-02/06.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.2.26.8";
const char S53178[] = "Mozilla/5.0 (Series40; NokiaC2-02.1/07.48; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.9";
const char S53179[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Maxthon/4.0.6.2000)";
const char S53180[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Maxthon/4.0.6.2000)";
const char S53181[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Maxthon/4.0.6.2000)";
const char S53182[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Maxthon/4.0.6.2000)";
const char S53183[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Maxthon/4.0.6.2000)";
const char S53184[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Maxthon/4.0.6.2000)";
const char S53185[] = "Mozilla/3.0 (X11; U; Linux 2.4.2-2 i586; en-US; m18) Gecko/X Netscape6/6.01";
const char S53186[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53187[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S53188[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S53189[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; MI-ONE Plus Build/GINGERBREAD)";
const char S53190[] = "Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S53191[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53192[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MI-ONE Plus Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S53193[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53194[] = "Nokia6220c-1";
const char S53195[] = "Nokia6220c/04.13 (SymbianOS/9.3; U; Series60/3.2 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S53196[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia6220c-1/05.15; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S53197[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; 7 Trophy)";
const char S53198[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; 7 Trophy T8686)";
const char S53199[] = "Mozilla/4.0 (compatible: MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; 7 Trophy)";
const char S53200[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; PC40100)";
const char S53201[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Spark)";
const char S53202[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaC6-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S53203[] = "NokiaC6-00UCBrowser/8.7.1.234/50/444/UCWEB";
const char S53204[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac6-00) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba";
const char S53205[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Flipboard/1.9.6";
const char S53206[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Flipboard/1.9.7";
const char S53207[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Flipboard/1.9.7";
const char S53208[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Flipboard/1.9.8";
const char S53209[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Flipboard/1.9.8";
const char S53210[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/1.9.8";
const char S53211[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Flipboard/1.9.8";
const char S53212[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/1.9.7";
const char S53213[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Flipboard/1.9.8";
const char S53214[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC Desire Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53215[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC Desire Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53216[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC Desire Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53217[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Desire Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53218[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC Desire Build/CM10)";
const char S53219[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC Desire Build/unknown) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53220[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; Desire_L_by_HTC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53221[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_Desire_600_dual_sim Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53222[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; HTC_Desire_600 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53223[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; HTC_Desire_500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53224[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF700T Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S53225[] = "Mozilla/5.0 (Linux; Android 4.2.2; ASUS Transformer Pad TF700T Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S53226[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HUAWEI; Ascend W1)";
const char S53227[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HUAWEI; HUAWEI Ascend W1 A4BA5B9A-2CD4-430D-B216-EA57BAF03A15)";
const char S53228[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HUAWEI; W1-U00)";
const char S53229[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; Huawei; W1-U34; Huawei W1-U34)";
const char S53230[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HUAWEI; W1-C00)";
const char S53231[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53232[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E970 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53233[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0";
const char S53234[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0";
const char S53235[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0";
const char S53236[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/531.21.8+(KHTML, like Gecko, Safari/528.16) Version/5.10.3 OmniWeb/622.14.0";
const char S53237[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/531.21.8+(KHTML, like Gecko, Safari/528.16) Version/5.10.3 OmniWeb/622.14.0";
const char S53238[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0";
const char S53239[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0";
const char S53240[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0";
const char S53241[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0";
const char S53242[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/531.21.8+(KHTML, like Gecko, Safari/528.16) Version/5.10.3 OmniWeb/622.14.0";
const char S53243[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.17.0";
const char S53244[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0";
const char S53245[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/531.21.8+(KHTML, like Gecko, Safari/528.16) Version/5.10.3 OmniWeb/622.14.0";
const char S53246[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0";
const char S53247[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0";
const char S53248[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0";
const char S53249[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0";
const char S53250[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9.0.2) Gecko/X Firefox/3.1.6";
const char S53251[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT626 Build/5.5.1Q-391_IR_TA-26) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53252[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; XT626 Build/5.5.1Q-391_IR_TA-30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53253[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; XT626 Build/5.5.1Q-391_IR_TA-72) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53254[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT626 Build/5.5.1Q-391_IR_TA-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53255[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; XT626 Build/5.5.1Q-391_IR_TA-82) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53256[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CMP741d Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53257[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; CMP741d Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53258[] = "Mozilla/5.0 (X11; U; FreeBSD amd64; en-US; rv:1.9.0.7) Gecko/X Firefox/3.0.7";
const char S53259[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53260[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53261[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53262[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/GLAZE6-16; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53263[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76L; CyanogenMod-Glazed ICS 7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S53264[] = "Nokia5320/04.13 (SymbianOS/9.3; U; Series60/3.2 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S53265[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia5320d-1/04.13; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S53266[] = "Nokia5320di/05.302 SymbianOS/9.3 Series60/3.2 Release/03.19.2009 Browser/Mozilla5.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S53267[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 NokianOS/9.3; U; Series60/3.2 Nokia5320d-1/04.13; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S53268[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia5320d-1b/03.08; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S53269[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia 5320di XpressMusic/05.302; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S53270[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia5320di XpressMusic/06.203; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S53271[] = "Nokia5320diXpressMusic(19.01)/SymbianOS/9.1 Series60/3.0";
const char S53272[] = "Nokia5320_CMCC/06.103 (SymbianOS/9.3; U; Series60/3.2 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S53273[] = "Nokia5320(19.01)/SymbianOS/9.1 Series60/3.0";
const char S53274[] = "sam-r360 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S53275[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-gb; AT200 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S53276[] = "Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; AT200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S53277[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; AT200 Build/HTK55D)";
const char S53278[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-US730 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53279[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB508 Build/SAGE_U3_10.23.20) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S53280[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB508 Build/Blur_Version.1.23.20.MB508.ATT.en.US Flex/P033) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S53281[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; MB508 Build/SAGE_U3_10.23.20)";
const char S53282[] = "LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S53283[] = "LG-KP500-Orange/V10i Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S53284[] = "LG-KP500-CH Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S53285[] = "Vodafone/1.0/LG-KP500/V10b/Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S53286[] = "Mozilla/5.0 (Series40; NokiaC2-06/07.48; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S53287[] = "Mozilla/5.0 (Series40; NokiaC2-06/06.97; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5";
const char S53288[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S53289[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S53290[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100B Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S53291[] = "Mozilla/5.0 (X11; Linux x86_64; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.62";
const char S53292[] = "Opera/9.80 (X11; Linux i686; U; en) Presto/2.10.229 Version/11.62";
const char S53293[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; Micromax A25 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53294[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A25 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53295[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; A25 Build/GRK39F)";
const char S53296[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A25 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S53297[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.1.3) Gecko/X Firefox/3.5.3";
const char S53298[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S53299[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S53300[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S53301[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S53302[] = "Mozilla/5.0 (Series40; Nokia311/05.92; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S53303[] = "Opera/9.80 (X11; Linux i686; U; de) Presto/2.2.15 Version/10.11";
const char S53304[] = "Opera/9.80 (X11; Linux i686; ADR-X; U; TSB_CLOUD_COMPANION;TOSHIBA_AC_AND_AZ; ru) Presto/2.5.28 Version/10.1";
const char S53305[] = "Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux i686; de) Opera 10.10";
const char S53306[] = "Opera/9.80 (X11; U; Linux i686; en-US; rv:1.9.2.3) Presto/2.2.15 Version/10.10";
const char S53307[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19";
const char S53308[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19";
const char S53309[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19";
const char S53310[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5) Presto/2.12.388 Version/12.10";
const char S53311[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS870 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53312[] = "SonyEricssonC902c/R3EG Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.3";
const char S53313[] = "SonyEricssonC902/R3EA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.3";
const char S53314[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.8.0)";
const char S53315[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.8.1)";
const char S53316[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.8.1)";
const char S53317[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.8.2)";
const char S53318[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.8.2)";
const char S53319[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.8.3)";
const char S53320[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.8.3)";
const char S53321[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.8.4)";
const char S53322[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 Sleipnir/3.8.4";
const char S53323[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 Sleipnir/3.8.4";
const char S53324[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.8.4)";
const char S53325[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.8.0)";
const char S53326[] = "Opera/9.80 (BREW; Opera Mini/5.1/27.2338; U; en) Presto/2.8.119 240X320 Casio C781";
const char S53327[] = "NokiaC2-03/2.0 (07.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-03) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile";
const char S53328[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_H3000C Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53329[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53330[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E975K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53331[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E975T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53332[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.21) Gecko/X Thunderbird/2.0.0.21";
const char S53333[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.8.1.23) Gecko/X Thunderbird/2.0.0.23";
const char S53334[] = "Mozilla/4.0 (compatible; Crawler; MSIE 7.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727)";
const char S53335[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSIECrawler)";
const char S53336[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.1) (TARGUSinfo Web Crawler www.targusinfo.com)";
const char S53337[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; MSIECrawler)";
const char S53338[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; MSIECrawler)";
const char S53339[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSIECrawler)";
const char S53340[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Crayon Crawler; .NET CLR 1.1.4322);c2VvLWNyYXdsZXI=";
const char S53341[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; iOpus-I-M; Infocrawler; SU 3.23; SIMBAR={29BC6696-626E-44F1-91BE-215A3CX}; Mozill;c2VvLWNyYXdsZXI=";
const char S53342[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Velocitymicro/T408) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53343[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-T989 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53344[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T989 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53345[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-T989 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53346[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGH-T989 Build/JDQ39)";
const char S53347[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-T989 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53348[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SAMSUNG-SGH-T989 Build/JDQ39E)";
const char S53349[] = "Reeder/1020.09.00 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro10%2C1)";
const char S53350[] = "Reeder/1020.09.00 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (iMac12%2C2)";
const char S53351[] = "Reeder/1020.09.00 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookAir5%2C2)";
const char S53352[] = "Reeder/1020.09.00 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBook5%2C1)";
const char S53353[] = "Reeder/1020.09.00 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacPro5%2C1)";
const char S53354[] = "Reeder/1020.09.00 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (Macmini3%2C1)";
const char S53355[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS840 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53356[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS840PP Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53357[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S53358[] = "Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 8.0; Windows NT 5.0) Chrome/21.0.1229.79";
const char S53359[] = "Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 7.0; Windows NT 5.0) Chrome/21.0.1229.79";
const char S53360[] = "Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 9.0; Windows NT 5.0WOW64) Chrome/21.0.1229.79";
const char S53361[] = "Mozilla/5.0 (compatible; MSIE 10.0; Macintosh; Intel Mac OS X 10_7_3; Trident/6.0)";
const char S53362[] = "Mozilla/5.0 (Linux; Android 4.1.1; Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S53363[] = "Mozilla/5.0 (Linux; Android 4.1.2; cm_tenderloin Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S53364[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbla668_fwvga Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53365[] = "Mozilla/5.0 (Linux; Android 4.1.2; skz_tenderloin Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S53366[] = "Mozilla/5.0 (Linux; Android 4.1.1; Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53367[] = "Mozilla/5.0 (Linux; Android 4.1.1; V1013 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S53368[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1911_v77_gq2000_a41_6628_v20 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53369[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbl1 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53370[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1909l_v77_gq1008_a41_v20 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53371[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1908_v77_jbl1_9p017_6628 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53372[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1920_v77_zlh_9p017_asx Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53373[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_gq2002tc Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53374[] = "Mozilla/5.0 (Linux; Android 4.1.5; iPhone5 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53375[] = "Mozilla/5.0 (Linux; Android 4.1.2; Motorola Galaxy s3 Build/9.8.2O-72_VZW-8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53376[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbla668_9p017_fwvga Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53377[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_gq2008 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53378[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1920_v77_zlh_fwvga Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53379[] = "Mozilla/5.0 (Linux; Android 4.1.2; e2001_v89_gq2008s Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53380[] = "Mozilla/5.0 (Linux; Android 4.1.2; e2001_v89_jbl1a698 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53381[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5233)/UC Browser8.5.0.183/50/355/UCWEB Mobile";
const char S53382[] = "Nokia5233/UCWEB8.5.0.163/50/800";
const char S53383[] = "Mozilla/5.0 (Java; U; en-us; nokia5233) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile  3gpp-gba";
const char S53384[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT914 Build/2_32A_2012) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53385[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SM-T311 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S53386[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SM-T311 Build/JDQ39)";
const char S53387[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-AS730 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53388[] = "Opera/9.80 (X11; Linux i686; U; Edition Linux Mint; en) Presto/2.10.289 Version/12.02";
const char S53389[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR HD 4G Build/7.7.1Q-78_VQW_LE-27) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53390[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR HD Build/7.7.1Q-78_VQW_LE-27) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53391[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR HD 4G Build/7.7.1Q-126_VQW_RL-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53392[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR HD Build/7.7.1Q_83) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53393[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR HD 4G Build/7.7.1Q-78_VQW_LE-25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53394[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR HD Build/7.7.1Q-144_VQW_S3-34) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53395[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR HD Build/7.7.1Q-144_VQW_S334-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53396[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR HD Build/7.7.1Q-144_VQW_S334-25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53397[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR HD 4G Build/7.7.1Q_140) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53398[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; XT925 Build/7.7.1Q-144_VQU_S3-35-19) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53399[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR HD Build/7.7.1Q-144_VQW_S33425-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53400[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; DROID RAZR HD Build/7.7.1Q-144_VQW_S33425-2)";
const char S53401[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; XT925 Build/7.7.1Q-144_VQL_S3-49.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53402[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT925 Build/7.7.1Q-144_VQL_S3-58) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53403[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; XT925 Build/7.7.1Q_144_VQU_DE_S3-35-26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53404[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; XT925 Build/7.7.1Q-144_VQL_S3-49) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53405[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; XT925 Build/7.7.1Q-144_VQU_S3-35-19)";
const char S53406[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; XT925 Build/7.7.1Q-144_VQU_S3-35-1101) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53407[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT925 Build/7.7.1Q-144_VQL_S3-61) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53408[] = "stagefright/1.2 (Linux;Android 4.0.4;motorola DROID RAZR HD Build/7.7.1Q-144_VQW_S33425-2)";
const char S53409[] = "Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia305) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S53410[] = "Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia305) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S53411[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Lenovo S880 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53412[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S880 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S53413[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Lenovo S880 Build/IML74K)";
const char S53414[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-mx; M4TEL SS880 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53415[] = "Mozilla/5.0 (Linux; Android 4.1.1; ME301T Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S53416[] = "Mozilla/5.0 (Android; Mobile; rv:19.0) Gecko/19.0 K9Browser/19.0.0";
const char S53417[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One X Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S53418[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One X Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S53419[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; IM-A820L Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53420[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; IM-A820L Build/GRJ90)";
const char S53421[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; IM-A820L Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53422[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; IM-A820L Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S53423[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; IM-A820L Build/viethoa boi galeri) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53424[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; IM-A820L Build/GRJ90-1.53-B2 Vi?t h?a, Mod By Ho?ng H?i) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53425[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-cn; IM-A820L Build/Ver 1.53 B3 Final by Ho?ng H?i) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53426[] = "Mozilla/5.0 (Series40; Nokia303/13.47; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.10";
const char S53427[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7005 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53428[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-N7005 Build/IMM76D)";
const char S53429[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS870 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S53430[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS870 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S53431[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S53432[] = "Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I727 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S53433[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S53434[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T859 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S53435[] = "Mozilla/5.0 (PlayStation Vita 2.61) AppleWebKit/536.26 (KHTML, like Gecko) Silk/3.2";
const char S53436[] = "Mozilla/5.0 (Series40; Nokia300/07.03; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S53437[] = "Mozilla/5.0 (Series40; Nokia300/07.03; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5";
const char S53438[] = "Mozilla/5.0 (PlayStation Vita 2.05) AppleWebKit/536.26 (KHTML, like Gecko) Silk/3.2";
const char S53439[] = "Mozilla/5.0 (Linux; Android 4.0.3; Sony Tablet S Build/TISU0143) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S53440[] = "SCH-U460L/1.0 NetFront/3.0.22.2.21 (GUI) MMP/2.0";
const char S53441[] = "SCH-U460P/1.0 NetFront/3.0.22.2.21 (GUI) MMP/2.0";
const char S53442[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A818 Duo Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53443[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MyPhone A818 Slim Duo Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53444[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; A818 Duo Build/GRK39F)";
const char S53445[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaN73) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile";
const char S53446[] = "NokiaN73UCBrowser/8.7.1.234/28/444/UCWEB";
const char S53447[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Acer-Z110 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53448[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; Z110 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53449[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; Z110 Build/GRK39F)";
const char S53450[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3110 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S53451[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3110 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S53452[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730M Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53453[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; ZTE V791 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/ZTE-Android-MMS-V2.0";
const char S53454[] = "Mozilla/5.0 (Linux; U; Android 2.3;vi-vn;ZTE V791 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53455[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; ZTE V791 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53456[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; ZTE V791 Build/GRK39F)";
const char S53457[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; ZTE V791 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53458[] = "Mozilla/5.0 (Linux; U; Android 2.3;en_us; ZTE V791 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53459[] = "Mozilla/5.0 (Linux; Android 4.1.1; C1505 Build/11.3.A.2.13) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53460[] = "SonyEricssonU10a/R7BA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2";
const char S53461[] = "SonyEricssonU10i/R7BA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2";
const char S53462[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB860 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53463[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB860 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53464[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB860 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53465[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MB860 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53466[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Karbonn A9+ Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53467[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A9+ Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53468[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; KPT A9PLUS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53469[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A9+(Flare S100) Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53470[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; KPT A9PLUS Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S53471[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S100/A9+ Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53472[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CELKON A9+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53473[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; A9+ Build/IMM76I)";
const char S53474[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.0.3) Gecko/X Firefox/3.0.3";
const char S53475[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I727 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53476[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I727 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53477[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53478[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I727 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53479[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE N9120 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S53480[] = "KWC-Buckle/ UP.Browser/7.2.7.2.541 (GUI) MMP/2.0";
const char S53481[] = "KWC-Buckle/ UP.Browser/7.2.7.2.549 (GUI) MMP/2.0";
const char S53482[] = "KWC-Buckle/ABN1004 UP.Browser/7.2.7.2.551 (GUI) MMP/2.0";
const char S53483[] = "KWC-Buckle/ABO1008 UP.Browser/7.2.7.2.551 (GUI) MMP/2.0";
const char S53484[] = "KWC-Buckle/ABO1102 UP.Browser/7.2.7.2.551 (GUI) MMP/2.0";
const char S53485[] = "KWC-Buckle/ABO1107 UP.Browser/7.2.7.2.551 (GUI) MMP/2.0";
const char S53486[] = "KWC-Buckle/ABN1104 UP.Browser/7.2.7.2.551 (GUI) MMP/2.0";
const char S53487[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Sensation Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53488[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Sensation Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53489[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Sensation Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53490[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Sensation Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53491[] = "Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:15.0) Gecko/X";
const char S53492[] = "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:13.0)";
const char S53493[] = "Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E300K/KKUAMDK Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S53494[] = "Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E300K/KKUAME6 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S53495[] = "Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E300K/KKUAME7 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S53496[] = "Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E300K/KKUAMF3 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S53497[] = "Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E300K/KKUAMG2 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S53498[] = "Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E300K/KKUAMH2 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S53499[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGP311 Build/10.1.C.0.370) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S53500[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; sdk Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53501[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; google_sdk Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53502[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; google_sdk Build/FRF91)";
const char S53503[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; rk2818sdk Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53504[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; sdk Build/FRF91)";
const char S53505[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; sdk Build/FRF42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53506[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; rk2818sdk Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S53507[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; tr-tr; sdk Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53508[] = "Mozilla/5.0 (Linux;U; Android 2.2; en-us; sdk Build/FRF91) AppleWebKit/533.1 BOLT/3.02 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53509[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53510[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One X Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53511[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; el-gr; RACERII Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53512[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; RacerII Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53513[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; RACERII Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53514[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; de-at; RacerII Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53515[] = "Nokia311/5.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia311) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S53516[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; E310 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53517[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; E310 Build/GRJ90)";
const char S53518[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; E310 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53519[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F240S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53520[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F240S Build/JZO54K)";
const char S53521[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT621 Build/5.5.1Q_PRIMUS_TA-207) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53522[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT621 Build/5.5.1Q_PRIMUS_TA-215) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53523[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT621 Build/5.5.1Q_PRIMUS_TA-214) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53524[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT621 Build/5.5.1Q_PRIMUS_TA-229) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53525[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT621 Build/5.5.1Q_PRIMUS_MR-217) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53526[] = "SonyEricssonW595c/R3EG Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.3";
const char S53527[] = "SonyEricssonW595a/R3DA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.2";
const char S53528[] = "SonyEricssonW595/R3EG Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.3";
const char S53529[] = "Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S53530[] = "Nokia3050/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S53531[] = "Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S53532[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; One X Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53533[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One X Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53534[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; HTC One X Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53535[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC One X Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53536[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One X Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53537[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC One X Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/endeavoru";
const char S53538[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC One X Build/4.75; CyanogenMod-4.75) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53539[] = "Mozilla/4.76 (Windows 98; U) Opera 5.12 [en]";
const char S53540[] = "Opera/5.12 (Windows 98; U) [en]";
const char S53541[] = "Opera/5.02 (Windows 98; U)  [en]";
const char S53542[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows 98) Opera 5.02 [en]";
const char S53543[] = "Opera/5.11 (Windows 98; U) [en]";
const char S53544[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows 98) Opera 5.12 [en]";
const char S53545[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GEM10313BK Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S53546[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GEM10313BK Build/JRO03H)";
const char S53547[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus 10 Build/JWR66V) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.111 Safari/537.36";
const char S53548[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P769 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36";
const char S53549[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P768 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S53550[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P760 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S53551[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.7.7) NimbleCrawler 1.11 obeys UserAgent NimbleCrawler For problems contact: crawler_at_dataalchemy.com";
const char S53552[] = "Mozilla/5.0 (compatible; Mozilla/5.0 (Windows NT 5.0; rv:21.0) Gecko/X Firefox/21.0; +http://wiki.github.com/bixo/bixo/bixocrawler; bixo-dev@yahoogroups.com)";
const char S53553[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P3110 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S53554[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3110 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S53555[] = "Mozilla/5.0 (Linux; Android 4.1.2; Transformer Prime TF201 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S53556[] = "Mozilla/5.0 (Linux; Android 4.1.1; Transformer Prime TF201 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S53557[] = "Mozilla/5.0 (Linux; Android 4.2.2; ASUS Transformer Pad TF700T Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S53558[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF700T Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S53559[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S53560[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S53561[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53562[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970C Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53563[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Touchpad Build/GRJ22; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53564[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Touchpad Build/GRJ22; XRON/CM7 vB2.8 Odexed and bloated) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53565[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F100L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53566[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-F100L Build/IMM76D)";
const char S53567[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/19.0.1084.46) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5";
const char S53568[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/19.0.1084.52)";
const char S53569[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/19.0.1084.52)";
const char S53570[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/19.0.1084.52) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5";
const char S53571[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/19.0.1084.52) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5";
const char S53572[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/19.0.1084.52; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506)";
const char S53573[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; chromeframe/19.0.1084.52; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S53574[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/19.0.1084.56) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S53575[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.3; chromeframe/19.0.1084.56; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)";
const char S53576[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/19.0.1084.52; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S53577[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; chromeframe/19.0.1084.52; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.21022; .NET CLR 3.0.30729)";
const char S53578[] = "Mozilla/5.0 (Windows NT 5.1; rv:16.0) Gecko/X Thunderbird/16.0.2";
const char S53579[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:16.0) Gecko/X Thunderbird/16.0.2";
const char S53580[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko)";
const char S53581[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A293";
const char S53582[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A400";
const char S53583[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko)";
const char S53584[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_0 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod2,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.0;FBSS/1; FBCR/;FBID/phone;FBLC/en_US;FBSF/1.0]";
const char S53585[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_0_2 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod3,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.0.2;FBSS/1; FBCR/;FBID/phone;FBLC/en_US;FBSF/1.0]";
const char S53586[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A293 iPod2,1 BingWeb/3.03.1428.X";
const char S53587[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; SC-02C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53588[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-; SC-02C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53589[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SC-02C Build/IML74K)";
const char S53590[] = "Nokia2700c-2/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2700c-2) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S53591[] = "Nokia2700c/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2700c) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S53592[] = "SAMSUNG-SGH-L700/DDID4 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S53593[] = "Mozilla/5.0 (Linux; Android 4.0.3;  ARCHOS 101G9 Build/MR1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S53594[] = "Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 101G9 Build/MR1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S53595[] = "Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 101G9 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S53596[] = "SonyEricssonCK15i/R3AE017 TelecaBrowser/Q07C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S53597[] = "SonyEricssonCK15a/R3AC020 TelecaBrowser/Q07C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S53598[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Next10P12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S53599[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; KM-S200 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53600[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; KM-S220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53601[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; KM-S200 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53602[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; KM-S220H Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53603[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; KM-S220 Build/GINGERBREAD)";
const char S53604[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; KM-S200 Build/GINGERBREAD)";
const char S53605[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Micromax A87 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53606[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A87 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53607[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Micromax A87 Build/GINGERBREAD)";
const char S53608[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Micromax A87 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S53609[] = "Mozilla/5.0 (Series40; NokiaX2-02/11.79; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S53610[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE71) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile";
const char S53611[] = "NokiaE71UCBrowser/8.7.1.234/28/352/UCWEB";
const char S53612[] = "NokiaE71/UCWEB8.7.0.187/28/999";
const char S53613[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiae71-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S53614[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE71x) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile";
const char S53615[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiae71-2) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S53616[] = "Nokia7210Supernova/2.0 (04.86) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7210supernova";
const char S53617[] = "Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S53618[] = "Nokia7210c_CMCC/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S53619[] = "Nokia7210c/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S53620[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG-SGH-I527 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S53621[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-ca; SGH-I527M Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S53622[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-ca; SGH-I527M-parrot Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S53623[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.7pre) Gecko/X Firefox/2.0.0.6 Navigator/9.0b3";
const char S53624[] = "Browser/Opera Spice M6868n Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10";
const char S53625[] = "Lenovo-TD39t_TD/S100 LMP/LML Release/2011.04.15 Browser/Opera Profile/MIDP2.0 Configuration/CLDC1.1 MBBMS/2.0 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; zh-CN) Presto/2.5.28 Version/10.10";
const char S53626[] = "Huawei/1.0/HUAWEI-G6609   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10";
const char S53627[] = "Huawei/1.0/HUAWEI-G6609   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; ro-RO) Presto/2.5.28 Version/10.10";
const char S53628[] = "Huawei/1.0/HUAWEI-G6609   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; es-SA) Presto/2.5.28 Version/10.10";
const char S53629[] = "Mot-WX308/v3 BTSTLE_G_0A.22.09RPS Release/10.11.2011 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10";
const char S53630[] = "Huawei/1.0/HUAWEI-G6609   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; sr-SR) Presto/2.5.28 Version/10.10";
const char S53631[] = "Fly_E154 ASTRO36_TD/v3 (MRE\\2.3.00(20480) resolution\\X chipset\\MT6255 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; ru-RU) Presto/2.5.28 Version/10.10";
const char S53632[] = "Huawei/1.0/HUAWEI-G6609/G6609V100R001C221B201SP01   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; pt-PT) Presto/2.5.28 Version/10.10";
const char S53633[] = "Fly_E154 ASTRO36_TD/v3 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; ru-RU) Presto/2.5.28 Version/10.10";
const char S53634[] = "Browser/Opera Spice M6868n Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; hi-IN) Presto/2.5.28 Version/10.10";
const char S53635[] = "Huawei/1.0/HUAWEI-G6609   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; fr-FR) Presto/2.5.28 Version/10.10";
const char S53636[] = "Huawei/1.0/HUAWEI-G6609   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; de-DE) Presto/2.5.28 Version/10.10";
const char S53637[] = "Huawei/1.0/HUAWEI-G6609   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; bg-BG) Presto/2.5.28 Version/10.10";
const char S53638[] = "Huawei/1.0/HUAWEI-G6609   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; it-IT) Presto/2.5.28 Version/10.10";
const char S53639[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53640[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Torque Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53641[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Torque Build/JZO54K)";
const char S53642[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.28) Gecko/X Thunderbird/3.1.20";
const char S53643[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ODYS-NOON Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S53644[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; NOON Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S53645[] = "AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; ODYS-NOON Build/JRO03H)";
const char S53646[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; NOON970 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S53647[] = "Mozilla/5.0 (Linux; Android 4.0.3; Sony Tablet S Build/TISU0143) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S53648[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMP5770D Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S53649[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SC-02B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53650[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ja-jp; SC-02B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53651[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SC-02B Build/GINGERBREAD)";
const char S53652[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SC-02B Build/GINGERBREAD)";
const char S53653[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I537 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S53654[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I537 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S53655[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53656[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830C Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53657[] = "Opera/9.80 (Linux mips ; U; HbbTV/1.1.1 (; Philips; ; ; ; ) CE-HTML/1.0 NETTV/3.2.1; en) Presto/2.6.33 Version/10.70";
const char S53658[] = "Opera/9.80 (Linux mips; U; HbbTV/1.1.1 (; Philips; ; ; ; ) CE-HTML/1.0 NETTV/4.0.2; en) Presto/2.10.250 Version/11.60";
const char S53659[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9A405 Safari/7534.48.3";
const char S53660[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9A334 Safari/7534.48.3";
const char S53661[] = "Mozilla/5.0 (iPod; CPU iPhone OS 501 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/21.0.1180.80 Mobile/9A405 Safari/7534.48.3";
const char S53662[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PAP4040_DUO Build/PrestigioPAP4040DUO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53663[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; PAP4040_DUO Build/PrestigioPAP4040DUO)";
const char S53664[] = "BlackBerry8830/4.5.0.186 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105";
const char S53665[] = "NOKIA5320/UC Browser8.0.4.121/28/444/UCWEB";
const char S53666[] = "Nokia5320di/UCWEB8.0.3.99/28/999";
const char S53667[] = "Mozilla/5.0 (Macintosh; U; PowerPC Mac OS X 10_4_11; en-US) AppleWebKit/531.21.8+(KHTML, like Gecko, Safari/528.16) Version/5.10.3 OmniWeb/622.14.0";
const char S53668[] = "Mozilla/5.0 (Macintosh; U; PowerPC Mac OS X 10_4_11; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0";
const char S53669[] = "Mozilla/5.0 (Macintosh; U; PowerPC Mac OS X 10_4_11; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0";
const char S53670[] = "Mozilla/5.0 (Macintosh; U; PowerPC Mac OS X 10_4_11; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0";
const char S53671[] = "Mozilla/5.0 (Macintosh; U; PowerPC Mac OS X 10_4_11; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0";
const char S53672[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_8; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0";
const char S53673[] = "Reeder/3.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S53674[] = "Mozilla/5.0 (Linux; Android 4.0.3; A500 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S53675[] = "Mozilla/5.0 (Linux; Android 4.0.4; A500 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S53676[] = "Mozilla/5.0 (Linux; Android 4.0.4; A500 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S53677[] = "Mozilla/5.0 (Linux; Android 4.0.4; Iconia A500 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S53678[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A523 Safari/8536.25";
const char S53679[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A406 Safari/8536.25";
const char S53680[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A403 Safari/8536.25";
const char S53681[] = "Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S53682[] = "Nokia3120classic/2.0 (07.16) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia3120classic";
const char S53683[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315";
const char S53684[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315";
const char S53685[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S53686[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S53687[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100T Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S53688[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S53689[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100M Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S53690[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S53691[] = "SAMSUNG-S8300-Vodafone/S8300BUIC6 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S53692[] = "SAMSUNG-S8300-Bouygues/S8300AGIC1 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S53693[] = "SAMSUNG-S8300T/1.0 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S53694[] = "SAMSUNG-S8300-ORANGE/S8300BVIE1 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S53695[] = "SAMSUNG-S8300H/1.0 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S53696[] = "SAMSUNG-S8300/1.0 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S53697[] = "Mozilla/5.0 (Linux; Android 4.0.4; N8000_USA_Cricket Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S53698[] = "Mozilla/5.0 (Linux; Android 4.0.4; V8000_USA_Cricket Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S53699[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-us; Tuna Build/IFK77E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.67 commoncrawl.org/research//Nutch-1.7-SNAPSHOT";
const char S53700[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN9G3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S53701[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337M Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53702[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710BST Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S53703[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S53704[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710VMUB Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S53705[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S53706[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710VMUB Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S53707[] = "Mozilla/5.0 (Linux; Android 4.1.1; SPH-D710 Build/JRO03R) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S53708[] = "Opera/8.01 (J2ME/MIDP; Opera Mini/1.2.3214/1724; en; U; ssr)";
const char S53709[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; M650 Build/HuaweiM650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53710[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; M650 Build/HuaweiM650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53711[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-S5830 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53712[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-CA; GT-S5830 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53713[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-S5830 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53714[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-S5830 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/cooper";
const char S53715[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6602 Build/10.1.1.A.1.253) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S53716[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-02E Build/10.1.D.0.343) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S53717[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC_Amaze_4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53718[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Ruby Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53719[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Amaze 4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53720[] = "Mozilla/5.0 (Linux; Android 4.0.3; Amaze Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53721[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Ruby Build/Speed Rom 8.0 By EclipzeRemix) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53722[] = "Opera/9.80 (BREW; Opera Mini/4.4.29939/28.3392; U; id) Presto/2.8.119 Version/11.10";
const char S53723[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I777 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53724[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I777 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53725[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I777 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53726[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I777 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53727[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.2 Mobile/9B206 Safari/7534.48.3";
const char S53728[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.2 Mobile/9B176 Safari/7534.48.3";
const char S53729[] = "Mozilla/5.0 (Unknown; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.10.0 (development) Safari/534.34";
const char S53730[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.10.0 (development) Safari/534.34";
const char S53731[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A406 Safari/8536.25";
const char S53732[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A523 Safari/8536.25";
const char S53733[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A403 Safari/8536.25";
const char S53734[] = "Mozilla/5.0 (Series40; Nokia112/03.32; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S53735[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Karbonn A21 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53736[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A21 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53737[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Karbonn A21 Themed and Tweaked by Santo_Rock Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53738[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; A21 Build/IMM76I)";
const char S53739[] = "SAMSUNG-GT-S5220 Opera/9.80 (J2ME/MIDP; Opera Mini/6.5.28754/30.3214; U; en) Presto/2.8.119 Version/11.10";
const char S53740[] = "Mozilla/5.0 (X11; Linux i686; rv:19.0) Gecko/X Firefox/19.0 SeaMonkey/2.16";
const char S53741[] = "Mozilla/5.0 (Series40; NokiaX3-02/07.16; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S53742[] = "Mozilla/5.0 (Series40; NokiaX3-02.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S53743[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; ADR6350 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53744[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; pcdadr6350 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53745[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; pcdadr6350 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S53746[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; pcdadr6350Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53747[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ko-; pcdadr6350 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53748[] = "Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; pcdadr6350 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53749[] = "Mozilla/5.0 (Linux; U; Android2.2.1; en-us; pcdadr6350 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53750[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;pcdadr6350 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53751[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; ADR6350 Build/FRG83D)";
const char S53752[] = "Mozilla/5.0 (Linux;U; Android 4.0.4; engb; Build/IMM76D) AppleWebKit/534.30 (KHTML,like Gecko) Version/4.0 Safari/534.30;SurfTab_7.0";
const char S53753[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; ST70104-1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53754[] = "Mozilla/5.0 (Linux;U; Android 4.0.4; engb; SurfTab_7.0 Build/IMM76D) AppleWebKit/534.30 (KHTML,like Gecko) Version/4.0 Safari/534.30";
const char S53755[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-B5510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S53756[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-B5510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S53757[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:19.0) Gecko/X Firefox/19.0 IceDragon/19.0.1.0";
const char S53758[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Iron/12.0.750.0 Chrome/12.0.750.0 Safari/534.30";
const char S53759[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.30 (KHTML, like Gecko) Iron/12.0.750.0 Chrome/12.0.750.0 Safari/534.30";
const char S53760[] = "Opera/9.80 (X11; Linux i686; U; en) Presto/2.10.229 Version/11.61";
const char S53761[] = "Mozilla/5.0 (X11; Linux i686; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.61";
const char S53762[] = "Mozilla/5.0 (PlayStation Vita 2.10) AppleWebKit/536.26 (KHTML, like Gecko) Silk/3.2";
const char S53763[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; Avant Browser; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; .NET CLR 3.5.21022; InfoPath.2)";
const char S53764[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Avant Browser)";
const char S53765[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Avant Browser)";
const char S53766[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Avant Browser; SV1)";
const char S53767[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Avant Browser; Avant Browser)";
const char S53768[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; SV1; Avant Browser)";
const char S53769[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Avant Browser; Avant Browser)";
const char S53770[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S5310 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53771[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-S5310/S5310XXAME2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53772[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S5310 Build/JZO54K)";
const char S53773[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-S5310L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53774[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; GT-S5310B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53775[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ie; SAMSUNG GT-S5310/S5310XXAMF1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53776[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S5310G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53777[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; el-gr; SAMSUNG GT-S5310/S5310XXAME5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53778[] = "Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; SC-02B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53779[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SC-02B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53780[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SC-02B Build/FROYO)";
const char S53781[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; SC-02B Build/FROYO)";
const char S53782[] = "Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; SC-02B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile";
const char S53783[] = "Mozilla/5.0 (Linux; Android 4.1.2; VS870 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53784[] = "sam-r350 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S53785[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Version/3.1.2 Safari/525.21";
const char S53786[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13.3";
const char S53787[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.17";
const char S53788[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.6.30 Version/10.62";
const char S53789[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.6.30 Version/10.63";
const char S53790[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.6.30 Version/10.60";
const char S53791[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.6.30 Version/10.61";
const char S53792[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S53793[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS870 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53794[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR HD Build/7.7.1Q-144_VQW_S334-25) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53795[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT925 Build/7.7.1Q-144_VQL_S3-58) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S53796[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SPH-D700 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53797[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SPH-D700 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53798[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SPH-D700 Build/FROYO)";
const char S53799[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.4.1.1000 Chrome/18.0.966.0 Safari/535.12";
const char S53800[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.4.1.1000 Chrome/18.0.966.0 Safari/535.12";
const char S53801[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1";
const char S53802[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.29";
const char S53803[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/525.28 (KHTML, like Gecko) Version/3.2.2 Safari/525.28.1";
const char S53804[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.13";
const char S53805[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/531.0 (KHTML, like Gecko) Version/3.2.1 Safari/531.0";
const char S53806[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/531.3 (KHTML, like Gecko) Version/3.2.1 Safari/531.3 Awesomium/1.5.1";
const char S53807[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Version/3.2.1 Safari/532.0";
const char S53808[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTCEVOV4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53809[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17";
const char S53810[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17";
const char S53811[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17";
const char S53812[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; GT-S7530)";
const char S53813[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; GT-S7530E)";
const char S53814[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; GT-S7530L)";
const char S53815[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT29i Build/9.1.B.1.67) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53816[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9195 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53817[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9195 Build/JDQ39)";
const char S53818[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; es-us; GT-I9195L Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53819[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; GT-I9195T Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53820[] = "Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5130c-2) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S53821[] = "Nokia5130/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia5130) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S53822[] = "Mozilla/5.0 (Linux; Android 4.3; Galaxy Nexus Build/JWR66V) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53823[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; USCCADR6275US Carrier ID 45 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S53824[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.2.1 (like Version/10B350 Mobile/6.1.4 Safari/8536.25)";
const char S53825[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.2.1 (like Version/10B329 Mobile/6.1.3 Safari/8536.25)";
const char S53826[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.2.1 (like Version/10B146 Mobile/6.1.2 Safari/8536.25)";
const char S53827[] = "1PasswordThumbs/1 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro10%2C1)";
const char S53828[] = "1PasswordThumbs/1 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (iMac12%2C2)";
const char S53829[] = "1PasswordThumbs/1 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookAir5%2C2)";
const char S53830[] = "1PasswordThumbs/1 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacPro3%2C1)";
const char S53831[] = "1PasswordThumbs/1 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (Macmini6%2C2)";
const char S53832[] = "1PasswordThumbs/1 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBook5%2C1)";
const char S53833[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9B176 Safari/7534.48.3";
const char S53834[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9B206 Safari/7534.48.3";
const char S53835[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Safari/535.19 Silk-Accelerated=false";
const char S53836[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MotoA953 Build/MILS2_U6_4.1-22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53837[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; A953 Build/4.5.2-51_M2L-27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53838[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; A953 Build/4.5.2-51_M2L-27_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53839[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; pt-br; MotoA953 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53840[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; MotoA953 Build/MILS2_U6_4.1-22)";
const char S53841[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; A953 Build/WajkIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53842[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; A953 Build/4.5.2-51_M2L-27_R02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53843[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; A953 Build/4.5.2-51_M2L-27_R03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53844[] = "Mozilla/5.0 (Linux; Android 4.0.4; V8000_USA_Cricket Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53845[] = "Mozilla/5.0 (Linux; Android 4.0.4; N8000_USA_Cricket Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53846[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-P7511 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S53847[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7511 Build/HTJ85B)";
const char S53848[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-P7511 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S53849[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-P7511 Build/HTJ85B.XXLA4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53850[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9192 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S53851[] = "Nokia2330c-2/2.0 (08.20) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S53852[] = "Nokia2330c-2/2.0 (09.85) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2330c-2";
const char S53853[] = "Nokia2330c/2.0 (08.20) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S53854[] = "Nokia2330c-2b/2.0 (08.13) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S53855[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Safari/535.19 Silk-Accelerated=false";
const char S53856[] = "Mozilla/5.0 (Linux; Android 4.1.1; Orange Daytona Build/HuaweiG510-0200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53857[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0200 Build/HuaweiG510-0200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53858[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0100 Build/HuaweiG510-0100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53859[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0251 Build/HuaweiG510-0251) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53860[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53861[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.11 (KHTML, like Gecko) Ubuntu/10.04 Chromium/17.0.963.79 Chrome/17.0.963.79 Safari/535.11";
const char S53862[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.11 (KHTML, like Gecko) Ubuntu/10.04 Chromium/17.0.963.79 Chrome/17.0.963.79 Safari/535.11";
const char S53863[] = "Mozilla/5.0 (Android; Mobile; rv:21.0) Gecko/21.0 Firefox/21.0 commoncrawl.org/research//Nutch-1.7-SNAPSHOT";
const char S53864[] = "HUAWEI-M735/001.00 Opera/9.70";
const char S53865[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One XL Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53866[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC EVA_UTL Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53867[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT300SE Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S53868[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT300 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S53869[] = "SAMSUNG-SGH-T139/T139UVJC1 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S53870[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I897 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53871[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I897 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53872[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I897 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53873[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I897 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/captivatemtd";
const char S53874[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID704 Build/MID704) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S53875[] = "AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; MID704 Build/MID704)";
const char S53876[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; MID704 Build/Tab746) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S53877[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; Tab746 Build/MID704) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S53878[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Transformer TF101G Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S53879[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Transformer TF101G Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S53880[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; Transformer TF101G Build/HTK75)";
const char S53881[] = "Mozilla/5.0 (Linux; U; Android 3.2.1 KRAKD by jcarrz1; en-us; Transformer TF101G Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S53882[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2 x64; en-US; rv:1.9.0.7) Gecko/X Minefield/3.0.7";
const char S53883[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9a8pre) Gecko/X Minefield/3.0a8pre";
const char S53884[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9a1) Gecko/X Minefield/3.0a1";
const char S53885[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293 Safari/6531.22.7 commoncrawl.org/research//Nutch-1.7-SNAPSHOT";
const char S53886[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11";
const char S53887[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11";
const char S53888[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11";
const char S53889[] = "Nokia6233/2.0 (05.60) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S53890[] = "Mozilla/5.0 (X11; SunOS i86pc; rv:17.0) Gecko/X Firefox/17.0";
const char S53891[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190 Twitter for iPhone";
const char S53892[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 Twitter for iPhone";
const char S53893[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Twitter for iPhone";
const char S53894[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2 Twitter for iPhone";
const char S53895[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Twitter for iPhone";
const char S53896[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 Twitter for iPhone";
const char S53897[] = "Opera/9.80 (Macintosh; Intel Mac OS X; U; en-Us) Presto/2.5.24 Version/10.53";
const char S53898[] = "Opera/9.80 (Macintosh; PPC Mac OS X; U; de) Presto/2.5.24 Version/10.54";
const char S53899[] = "Opera/9.80 (Macintosh; Intel Mac OS X; U; en) Presto/2.5.24 Version/10.52";
const char S53900[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN9G2I Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S53901[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; AN9G2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S53902[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; AN9G2I Build/IML74K)";
const char S53903[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0) commoncrawl.org/research//Nutch-1.7-SNAPSHOT";
const char S53904[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1";
const char S53905[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1";
const char S53906[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5230) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S53907[] = "Nokia5230/UCWEB8.8.0.212/50/800";
const char S53908[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5230-c) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S53909[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5230-1c) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S53910[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia 5230) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S53911[] = "Mozilla/5.0 (Windows; N; Windows NT 5.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.2.2.6613 Safari/532.4";
const char S53912[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);en;kml:2.2;client:Free;type:default)";
const char S53913[] = "GoogleEarth/6.2.1.6014(Windows;Microsoft Windows (5.1.2600.3);en;kml:2.2;client:Free;type:default)";
const char S53914[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);zh-Hans;kml:2.2;client:Free;type:default)";
const char S53915[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.2);en;kml:2.2;client:Free;type:default)";
const char S53916[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);fr;kml:2.2;client:Free;type:default)";
const char S53917[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);de;kml:2.2;client:Free;type:default)";
const char S53918[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);ru;kml:2.2;client:Free;type:default)";
const char S53919[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);ko;kml:2.2;client:Free;type:default)";
const char S53920[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);es;kml:2.2;client:Free;type:default)";
const char S53921[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);it;kml:2.2;client:Free;type:default)";
const char S53922[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);es-419;kml:2.2;client:Free;type:default)";
const char S53923[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);nl;kml:2.2;client:Free;type:default)";
const char S53924[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);pt;kml:2.2;client:Free;type:default)";
const char S53925[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);en;kml:2.2;client:Pro;type:default)";
const char S53926[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);cs;kml:2.2;client:Free;type:default)";
const char S53927[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TD-1010 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S53928[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; TD-1010 Build/IMM76D)";
const char S53929[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-M950 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53930[] = "Mozilla/5.0 (SmartHub; SMART-TV; U; Linux/SmartTV) AppleWebKit/531.2+ (KHTML, like Gecko) WebBrowser/1.0 SmartTV Safari/531.2+";
const char S53931[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; LG-P935 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S53932[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; LG-P935 Build/GRJ90)";
const char S53933[] = "Mozilla/5.0 (Symbian3; U; en-us; NokiaC7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile";
const char S53934[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac7-00) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba";
const char S53935[] = "NokiaC7-00UCBrowser/8.7.1.234/51/355/UCWEB";
const char S53936[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST27i Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53937[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST27a Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53938[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53939[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S53940[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B329 Safari/8536.25";
const char S53941[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B146 Safari/8536.25";
const char S53942[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B141 Safari/8536.25";
const char S53943[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B144 Safari/8536.25";
const char S53944[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; ARCHOS 70it2 Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S53945[] = "Mozilla/5.0 (Linux; Android 4.1.1; SPH-D710 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53946[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53947[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710VMUB Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53948[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710BST Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53949[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P659 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53950[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P659 Build/JZO54K)";
const char S53951[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Incredible S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53952[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Incredible S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53953[] = "Mozilla/5.0 (Linux; Android 4.0.3; Incredible S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S53954[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; I9377 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53955[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; GT-i9377 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53956[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; I9377 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S53957[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-i9377 Build/JRO03C)";
const char S53958[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; I9377 Build/JRO03C)";
const char S53959[] = "Mozilla/5.0 (Windows NT 6.0; rv:19.0) Gecko/X Firefox/19.0 SeaMonkey/2.16.2";
const char S53960[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8) Presto/2.12.388 Version/12.14";
const char S53961[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LG730 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S53962[] = "Nokia1680c-2/2.0 (06.82) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia1680c-2";
const char S53963[] = "Nokia1680c-2/2.0 (07.60) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S53964[] = "Nokia1680c-2b/2.0 (05.61) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S53965[] = "Nokia1680c-2b_CMCC/2.0 (06.82) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S53966[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.3.4641 Chrome/7.0.517.450 Safari/534.7";
const char S53967[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Micromax A27 Build/IML74K) AppleWebKit/534.30(KHTML,like Gecko) 			Version/4.0 Mobile Safari/534.30";
const char S53968[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Micromax A27 Build/MocorDroid2.3.5)";
const char S53969[] = "Opera/9.80 (Macintosh; Intel Mac OS X; U; en) Presto/2.2.15 Version/10.00";
const char S53970[] = "Opera/9.80 (Macintosh; PPC Mac OS X; U; en) Presto/2.2.15 Version/10.01";
const char S53971[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X; U; en; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 10.00";
const char S53972[] = "Opera/10.00 (Macintosh; Intel Mac OS X; U; en) Presto/2.2.0";
const char S53973[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-ie; HTC Tattoo 1.67.174.27 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S53974[] = "Mozilla/5.0 (Linux; U; Android 1.6; es-es; HTC Tattoo 1.67.164.18 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S53975[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; HTC Tattoo 1.67.161.18 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S53976[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; HTC Tattoo 1.67.161.5 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S53977[] = "Mozilla/5.0 (Linux; U; Android 1.6; tr-tr; HTC Tattoo 1.67.184.60 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S53978[] = "Mozilla/5.0 (Linux; U; Android 1.6; pt-pt; HTC Tattoo 1.67.173.37 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S53979[] = "Mozilla/5.0 (Linux; U; Android 1.6; es-es; HTC Tattoo 1.65.161.2 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S53980[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; HTC Tattoo 1.64.161.1 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S53981[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; HTC_TATTOO_A3288 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S53982[] = "Mozilla/5.0 (Linux; U; Android 1.6; fr-ch; HTC Tattoo-Orange Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S53983[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; HTC Tattoo Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S53984[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; HTC_TATTOO_A3288 Build/DRC79) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S53985[] = "BlackBerry9530/4.7.0.148 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105";
const char S53986[] = "Opera/9.80 (Macintosh; PPC Mac OS X; U; en) Presto/2.2.15 Version/10.10";
const char S53987[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.1)";
const char S53988[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S53989[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows XP Prof.)";
const char S53990[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.2; .NET CLR 1.1.4322; InfoPath.2; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; FDM)";
const char S53991[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
const char S53992[] = "Mozilla/5.0 (compatible; MSIE 5.5; Windows NT 5.1; SV1)";
const char S53993[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.2; .NET CLR 1.1.4322) (Compatible; ; ; Trident/4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.0.3705; .NET CLR 1.1.4322)";
const char S53994[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; MSIE 5.5; Windows NT 5.0; GTB7.4; EasyBits GO v1.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; OfficeLiveConnector.1.3; OfficeLivePatch.0.0; InfoPath.2; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; AskTbAVR-4/5.15.11.30498)";
const char S53995[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; CT1010 Build/IMM76I.v006) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S53996[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; CT1010 Build/IMM76I.v007) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S53997[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; CT1010 Build/IMM76I.v007S) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S53998[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-be; CT1010 Build/IMM76I.v007B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S53999[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-be; CT1010 Build/IMM76I.v008B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54000[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; CT1010 Build/IMM76I.v008) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54001[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac5-00.2) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba";
const char S54002[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac5-00) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba";
const char S54003[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaC5-00) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S54004[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaC5-00.2) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S54005[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36";
const char S54006[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36";
const char S54007[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36";
const char S54008[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMP5880D Build/PMP5880D_v1.0.0_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54009[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTCEVOV4G Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54010[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:20.0) Gecko/X Firefox/20.0 SeaMonkey/2.17.1";
const char S54011[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A870S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54012[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A870L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54013[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A870K/1.27 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54014[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A870K/1.20 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54015[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A870L Build/JZO54K)";
const char S54016[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A870K/1.32 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54017[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A870L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30";
const char S54018[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A870K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 NAVER(inapp; search; 230; 4.7.7)";
const char S54019[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A870S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30";
const char S54020[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A870S Build/JZO54K)";
const char S54021[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A870K Build/JZO54K)";
const char S54022[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A870K/1.44 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30";
const char S54023[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; IM-A770K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54024[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; IM-A770K Build/GINGERBREAD)";
const char S54025[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; IM-A770K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54026[] = "Mozilla/5.0 (Series40; Nokia311/05.92; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S54027[] = "UCWEB/2.0(BlackBerry; U; 6.5.0.54; en-us; 9780/6.5.0.54) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S54028[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.207; en-us; 9780/6.6.0.207) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S54029[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.248; en-us; 9780/6.6.0.248) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S54030[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.236; en-us; 9780/6.6.0.236) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S54031[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.246; en-us; 9780/6.6.0.246) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S54032[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.223; en-us; 9780/6.6.0.223) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S54033[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.212; en-us; 9780/6.6.0.212) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S54034[] = "UCWEB/2.0(BlackBerry; U; 6.5.0.136; en-us; 9780/6.5.0.136) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S54035[] = "UCWEB/2.0(BlackBerry; U; 6.5.0.153; en-us; 9780/6.5.0.153) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S54036[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.241; en-us; 9780/6.6.0.241) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S54037[] = "UCWEB/2.0(BlackBerry; U; 6.5.0.93; en-us; 9780/6.5.0.93) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S54038[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.216; en-us; 9780/6.6.0.216) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S54039[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.200; en-us; 9780/6.6.0.200) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S54040[] = "UCWEB/2.0(BlackBerry; U; 6.5.0.147; en-us; 9780/6.5.0.147) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S54041[] = "UCWEB/2.0(BlackBerry; U; 6.6.0.250; en-us; 9780/6.6.0.250) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S54042[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Safari/535.19 Silk-Accelerated=false";
const char S54043[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54044[] = "Mozilla/5.0 (Linux; Android 4.1.1; C1504 Build/11.3.A.0.47) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S54045[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.3.9.1000 Chrome/18.0.966.0 Safari/535.12";
const char S54046[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.3.9.1000 Chrome/18.0.966.0 Safari/535.12";
const char S54047[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.3 (KHTML, like Gecko) Maxthon/3.3.3.1000 Chrome/16.0.883.0 Safari/535.3";
const char S54048[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.3 (KHTML, like Gecko) Maxthon/3.3.3.1000 Chrome/16.0.883.0 Safari/535.3";
const char S54049[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Twitter for iPad";
const char S54050[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Twitter for iPad";
const char S54051[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550 Twitter for iPad";
const char S54052[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8426 Twitter for iPad";
const char S54053[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A407 Twitter for iPad";
const char S54054[] = "Opera/9.80 (BREW; Opera Mini/5.1/27.2338; U; en) Presto/2.8.119 320X240 Pantech TXT8045";
const char S54055[] = "Mozilla/5.0 (X11; Linux i686; rv:17.0) Gecko/X Firefox/17.0 SeaMonkey/2.14.1";
const char S54056[] = "Mozilla/5.0 (X11; Linux i686; rv:17.0) Gecko/X SeaMonkey/2.14.1";
const char S54057[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9B206 Safari/7534.48.3";
const char S54058[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9B176 Safari/7534.48.3";
const char S54059[] = "Yahoo! (iPhone Inquisitor; 1.0)";
const char S54060[] = "iPhone Vox";
const char S54061[] = "WhatsEye (iPhone) AppleWebKit (KHTML, like Gecko) Mobile/8F192";
const char S54062[] = "Apple iPhone v9B206 Maps v5.1.1";
const char S54063[] = "com.apple.invitation-registration [iPhone OS,5.1.1,9B206,iPhone3,3]";
const char S54064[] = "Apple iPhone v8C148 Maps v4.2.1";
const char S54065[] = "Apple iPhone v9A405 Maps v5.0.1";
const char S54066[] = "server-bag [iPhone OS,5.0.1,9A406,iPhone4,1]";
const char S54067[] = "Apple iPhone v5.1.1 YouTube v1.0.0.9B206";
const char S54068[] = "Apple iPhone v5.1.1 Stocks v3.0.9B206";
const char S54069[] = "server-bag [iPhone OS,5.1.1,9B208,iPhone3,1]";
const char S54070[] = "Apple iPhone v5.0.1 YouTube v1.0.0.9A405";
const char S54071[] = "Apple iPhone v9B176 Maps v5.1";
const char S54072[] = "Tribune App iPhone v1.0";
const char S54073[] = "Apple iPhone v9B179 Maps v5.1";
const char S54074[] = "Apple iPhone v5.0 YouTube v1.0.0.9A334";
const char S54075[] = "com.apple.invitation-registration [iPhone OS,5.1.1,9B206,iPhone4,1]";
const char S54076[] = "eBayiPhone/2.8.0";
const char S54077[] = "eBayiPhone/2.7.2";
const char S54078[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P880g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S54079[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P880 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S54080[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SHW-M180S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54081[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SHW-M180L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54082[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; SHW-M180K Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54083[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; SHW-M180W Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54084[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SHW-M180S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54085[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SHW-M180L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54086[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; SHW-M180S Build/FROYO)";
const char S54087[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; SHW-M180L Build/FROYO)";
const char S54088[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SHW-M180S Build/FROYO)";
const char S54089[] = "HUAWEI-M636/001.00 WAP/OBIGO/Q05A";
const char S54090[] = "HUAWEI-M636/001.00 Browser/Obigo-Browser/Q05A MMS/UNIBOX/V1.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S54091[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I997 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54092[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I997 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54093[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I997 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54094[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I997 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54095[] = "1PasswordThumbs/1 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C2)";
const char S54096[] = "1PasswordThumbs/1 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (iMac12%2C2)";
const char S54097[] = "1PasswordThumbs/1 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookAir4%2C2)";
const char S54098[] = "1PasswordThumbs/1 CFNetwork/520.5.1 Darwin/11.4.2 (i386) (MacPro1%2C1)";
const char S54099[] = "1PasswordThumbs/1 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBook5%2C1)";
const char S54100[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ601 Build/I.7.1-45) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S54101[] = "Mozilla/5.0 (Linux; Android 4.0.3; MZ601 Build/I.7.1-34) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S54102[] = "Mozilla/5.0 (masking-agent; rv:14.0) Gecko/X Firefox/14.0";
const char S54103[] = "Mozilla/5.0 (masking-agent; rv:14.0) Gecko/X Firefox/14.0.1";
const char S54104[] = "Mozilla/5.0 (SaferSurf rv:14.0) Gecko/X Firefox/14.0";
const char S54105[] = "Mozilla/5.0 (Unknown; rv:14.0) Gecko/X Firefox/14.0.1";
const char S54106[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 7 Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.10/221,1.9.10.221,2012-11-19 13:37, us";
const char S54107[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Nexus 7 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0000, gb";
const char S54108[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, -0400, us";
const char S54109[] = "Mozilla/5.0 (Series40; Nokia200/11.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S54110[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54111[] = "SAMSUNG-SGH-T479/T479UVJH1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S54112[] = "SonyEricssonWT13i/R3AF010 TelecaBrowser/Q07C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S54113[] = "Opera/9.80 (BlackBerry; Opera Mini/5.1.22303/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S54114[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Blade Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54115[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Blade Build/GRJ22; Eco-CM7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54116[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S54117[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P708g Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S54118[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P705 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S54119[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P705g Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S54120[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P705f Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S54121[] = "Opera/9.80 (Android; Opera Mini/7.6.32764/28.3234; U; zh) Presto/2.8.119 Version/11.10";
const char S54122[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AN7FG3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54123[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; AN7FG3 Build/IML74K)";
const char S54124[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; PC1038 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54125[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; PC1038 Build/PC1038) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54126[] = "AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; PC1038 Build/JRO03H)";
const char S54127[] = "python-requests/1.2.3 CPython/2.7.2 Darwin/12.4.0";
const char S54128[] = "python-requests/1.2.3 CPython/2.7.5 Darwin/12.4.0";
const char S54129[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; INFOBAR A01 Build/S7142) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54130[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; INFOBAR A01 Build/S6160) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54131[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; INFOBAR A01 Build/S9081) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54132[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; INFOBAR A01 Build/SB240) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54133[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; INFOBAR A01 Build/S3081) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54134[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; INFOBAR A01 Build/SA271) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54135[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; INFOBAR A01 Build/S8270) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54136[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; INFOBAR A01 Build/S8020) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54137[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; INFOBAR A01 Build/S7050) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54138[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; INFOBAR A01 Build/S8020)";
const char S54139[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xoom Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54140[] = "Mozilla/5.0 (Linux; Android 4.1.1; Xoom Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54141[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; E310 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54142[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; E310 Build/FRG83G)";
const char S54143[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2) Presto/2.12.388 Version/12.10";
const char S54144[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)UCBrowser/8.7.1.234/50/444/UCWEB";
const char S54145[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54146[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xperia P Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54147[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S54148[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; LIFETAB_S9714 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54149[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; LIFETAB_S9714 Build/JRO03R)";
const char S54150[] = "BlackBerry8310/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102";
const char S54151[] = "Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320) BlackBerry8310/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/107";
const char S54152[] = "Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320) BlackBerry8310/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102";
const char S54153[] = "Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320) BlackBerry8310/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/154";
const char S54154[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8730 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S54155[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8730T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S54156[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.0.1-x64";
const char S54157[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.0.1";
const char S54158[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54159[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC VLE_U Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54160[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54161[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC VLE_U Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54162[] = "MT6515M-A1+/1.0 Linux/2.6.35.7 Android/2.3.6 Release/10.30.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S54163[] = "MT6515M-P5/1.0 Linux/2.6.35.7 Android 2.3.6 Release/11.29.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S54164[] = "MT6515M-A1+/1.0 Linux/2.6.35.7 Android/2.3.6 Release/10.29.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S54165[] = "MT6515M-A1+/1.0 Linux/2.6.35.7 Android 2.3.6 Release/12.09.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S54166[] = "MT6515M-A1+/1.0 Linux/2.6.35.7 Android/2.3.6 Release/gf.if.hfgh Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S54167[] = "MT6515M-GS2/1.0 Linux/2.6.35.7 Android 2.3.6 Release/11.13.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S54168[] = "MT6515M-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android 2.3.6 Release/12.03.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S54169[] = "MT6515M-PH350/1.0 Linux/2.6.35.7 Android 2.3.6 Release/11.22.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S54170[] = "MT6515M-A1+/1.0 Linux/2.6.35.7 Android 2.3.6 Release/11.01.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S54171[] = "MT6515M-P5/1.0 Linux/2.6.35.7 Android 2.3.6 Release/12.08.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S54172[] = "MT6515M-A1+/1.0 Linux/2.6.35.7 Android 2.3.6 Release/gh.fo.hfgh Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S54173[] = "Mozilla/5.0 (Linux; U; Android 2.3.9; zh-cn; MT6515M-HAPPLE/1.0 Android/2.3.9 Release/04.18.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54174[] = "MT6515-I-T-3002/1.0 Linux/Apps_4.0.32.9-perf Android/2.3.5 Release/11.07.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S54175[] = "MT6515M-HTH668/1.0 Linux/2.6.35.7 Android/2.3.6 Release/09.22.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S54176[] = "MT6515M-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/2.3.6 Release/10.27.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S54177[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire X Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54178[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire X Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S54179[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5302 Build/12.0.A.2.254) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54180[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2690";
const char S54181[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S54182[] = "SAMSUNG-SGH-A797/A797UCJC1; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.17793; U; en-US) Opera 9.50";
const char S54183[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR6425LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S54184[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Ubuntu/11.04 Chromium/15.0.871.0 Chrome/15.0.871.0 Safari/535.2";
const char S54185[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.2 (KHTML, like Gecko) Ubuntu/11.04 Chromium/15.0.874.106 Chrome/15.0.874.106 Safari/535.2";
const char S54186[] = "coccoc";
const char S54187[] = "Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; SC-02D Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S54188[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-P6201 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S54189[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; SC-02D Build/HTJ85B)";
const char S54190[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P6201 Build/HTJ85B)";
const char S54191[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6201 Build/HTJ85B) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S54192[] = "Mozilla/5.0 (Linux; U; Android 3.2; id-id; GT-P6201 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S54193[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6201 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.12;FBDM/{density=1.0,width=600,height=976};FBLC/en_US;FBCR/Smart;FBPN/com.facebook.katana;FBDV/GT-P6201;FBSV/3.2;]";
const char S54194[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P6201 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S54195[] = "com.google.android.youtube/4.3.9(Linux; U; Android 3.2; en_US; GT-P6201 Build/HTJ85B) gzip";
const char S54196[] = "Mozilla/5.0 (Series40; Nokia110/03.04; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S54197[] = "Mozilla/5.0 (Series40; Nokia110/03.26; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5";
const char S54198[] = "AppleCoreMedia/1.0.0.9B176 (iPhone; U; CPU OS 5_1 like Mac OS X; en_us)";
const char S54199[] = "AppleCoreMedia/1.0.0.9B206 (iPhone; U; CPU OS 5_1_1 like Mac OS X; en_us)";
const char S54200[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 505)";
const char S54201[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) PPC; 240x320; CHT 9100/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S54202[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12; en-US; KIN.Two 1.0)";
const char S54203[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12; es-US; KIN.Two 1.0)";
const char S54204[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12; en-US; KIN.One 1.0)";
const char S54205[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12; es-US; KIN.One 1.0)";
const char S54206[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Smartphone; 240x320; SPV E650; OpVer 22.122.1.613";
const char S54207[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Smartphone; 240x320; SPV E650; OpVer 22.114.1.612";
const char S54208[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Sprint:PPC6800";
const char S54209[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) S01SH";
const char S54210[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) WS011SH";
const char S54211[] = "Lenovo_ET600/Lenovo WindowsMobile/6 Release/4.22.2007 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S54212[] = "Techfaith Moffett/2.0 WAP2.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S54213[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S54214[] = "BlackBerry8220/4.6.0.180 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100";
const char S54215[] = "Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:10.0a1) Gecko/X Firefox/10.0a1";
const char S54216[] = "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:10.0.2) Gecko/X Firefox/10.0.2 Firefox/10.0.2";
const char S54217[] = "Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:10.0) Gecko/X Abrowser/10.0";
const char S54218[] = "Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:10.0.5) Gecko/X Firefox/10.0.5";
const char S54219[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:15.3) Gecko/X Firefox/15.3.2-x64 PaleMoon/15.3.2-x64";
const char S54220[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:15.3) Gecko/X Firefox/15.3.2 PaleMoon/15.3.2";
const char S54221[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S54222[] = "Chrome/1";
const char S54223[] = "Chrome/1.0.154.53 Safari/525.19";
const char S54224[] = "Spice M5100";
const char S54225[] = "Spice QT53";
const char S54226[] = "Spice M-5180";
const char S54227[] = "Spice M5460";
const char S54228[] = "Spice M6450";
const char S54229[] = "Spice M5456";
const char S54230[] = "Spice M-5455";
const char S54231[] = "Spice M5225";
const char S54232[] = "Spice M5665";
const char S54233[] = "Spice_M-5400/WapBrowserProfile";
const char S54234[] = "Spice M5670";
const char S54235[] = "Spice M5800";
const char S54236[] = "Spice QT59";
const char S54237[] = "Spice M5915 /Spreadtrum/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0Config/CLD-1.1";
const char S54238[] = "Spice Mi-354";
const char S54239[] = "Opera/9.51 (Windows NT 6.1; U; en)";
const char S54240[] = "Opera/9.52 (Windows NT 6.1; U; en)";
const char S54241[] = "Opera/9.50 (Windows NT 6.1; U; en)";
const char S54242[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; San Francisco II Build/OUK_B03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54243[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; San Francisco II Build/OUK_B04) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54244[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; San Francisco II Build/OUK_B03)";
const char S54245[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; sk-sk; San Francisco II Build/OSK_B04) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54246[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ro-ro; San Francisco II Build/OMD_B01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54247[] = "Mozilla/5.0 (Linux; Android 4.3; SAMSUNG-SGH-I317 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54248[] = "Mozilla/5.0 (Linux; Android 4.3; GT-N7100 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54249[] = "Mozilla/5.0 (Linux; Android 4.3; SPH-L900 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54250[] = "Mozilla/5.0 (Linux; Android 4.3; SGH-T889 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54251[] = "Mozilla/5.0 (Linux; Android 4.3; GT-N7105 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54252[] = "Mozilla/5.0 (Linux; Android 4.3; Galaxy Note II Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54253[] = "Mozilla/5.0 (Linux; Android 4.3; SGH-I317 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54254[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SonyEricssonMK16a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54255[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MK16a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54256[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MK16a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54257[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MK16a Build/4.1.B.0.587)";
const char S54258[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54259[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAD1001 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54260[] = "Mozilla/5.0 (Linux; Android 4.0.4; 7102 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54261[] = "Mozilla/5.0 (Linux; Android 4.0.4; 9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54262[] = "Mozilla/5.0 (Linux; Android 4.0.4; AMI-TT2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54263[] = "Mozilla/5.0 (Linux; Android 4.0.3; Explosion Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54264[] = "Mozilla/5.0 (Linux; Android 4.0.3; CTA Build/ICS.casper.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54265[] = "Mozilla/5.0 (Linux; Android 4.0.4; GLORY Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54266[] = "Mozilla/5.0 (Linux; Android 4.0.4; BASE_Lutea_3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54267[] = "Mozilla/5.0 (Linux; Android 4.0.4; A11Note Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54268[] = "Mozilla/5.0 (Linux; Android 4.0.4; I110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54269[] = "Mozilla/5.0 (Linux; Android 4.0.4; S7916E Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54270[] = "Mozilla/5.0 (Linux; Android 4.0.4; N1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54271[] = "Mozilla/5.0 (Linux; Android 4.0.4; Spark Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54272[] = "Mozilla/5.0 (Linux; Android 4.0.4; tmn smart a18 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54273[] = "Mozilla/5.0 (Linux; Android 4.0.4; MOBHIG0080 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54274[] = "Mozilla/5.0 (Linux; Android 4.0.4; N900 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54275[] = "Mozilla/5.0 (Linux; Android 4.0.4; P6 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54276[] = "Mozilla/5.0 (Linux; Android 4.0.4; T7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54277[] = "Mozilla/5.0 (Linux; Android 4.0.4; CJ ThL Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54278[] = "Mozilla/5.0 (Linux; Android 4.0.4; BT210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54279[] = "Mozilla/5.0 (Linux; Android 4.0.3; Alpha GT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54280[] = "Mozilla/5.0 (Linux; Android 4.0.3; Infinity Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54281[] = "Mozilla/5.0 (Linux; Android 4.0.3; 1040RS Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54282[] = "Mozilla/5.0 (Linux; Android 4.0.4; JY139 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54283[] = "Mozilla/5.0 (Linux; Android 4.0.4; A13-DLR Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54284[] = "Mozilla/5.0 (Linux; Android 4.0.4; MD-0697 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54285[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB9008GB Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54286[] = "Mozilla/5.0 (Linux; Android 4.0.3; 740RS Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54287[] = "Mozilla/5.0 (Linux; Android 4.0.4; MIRA-2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54288[] = "Mozilla/5.0 (Linux; Android 4.0.3; S5-N Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54289[] = "Mozilla/5.0 (Linux; Android 4.0.3; StarPAD 7 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54290[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID_Robin Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54291[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMURAI10(QuadCore) Build/SAMURAI10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54292[] = "Mozilla/5.0 (Linux; Android 4.0.4; SMP53-210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54293[] = "Mozilla/5.0 (Linux; Android 4.0.4; POM727MC Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54294[] = "Mozilla/5.0 (Linux; Android 4.0.4; Avvio_790 Build/AvvioAvvio_790) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54295[] = "Mozilla/5.0 (Linux; Android 4.0.4; M1005 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54296[] = "Mozilla/5.0 (Linux; Android 4.0.4; STXS Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54297[] = "Mozilla/5.0 (Linux; Android 4.0.4; P4i Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54298[] = "Mozilla/5.0 (Linux; Android 4.0.4; HD II Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54299[] = "Mozilla/5.0 (Linux; Android 4.0.4; SMP45-210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54300[] = "Mozilla/5.0 (Linux; Android 4.0.4; A13 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54301[] = "Mozilla/5.0 (Linux; Android 4.0.4; 5I Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54302[] = "Mozilla/5.0 (Linux; Android 4.0.4; GO Live Dual Core Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54303[] = "Mozilla/5.0 (Linux; Android 4.0.3; Clear Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54304[] = "Mozilla/5.0 (Linux; Android 4.0.4; M1005D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54305[] = "Mozilla/5.0 (Linux; Android 4.0.4; hexing77_ics2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54306[] = "Mozilla/5.0 (Linux; Android 4.0.4; K-BOOK PLUS Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54307[] = "Mozilla/5.0 (Linux; Android 4.0.4; G1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54308[] = "Mozilla/5.0 (Linux; Android 4.0.4; CT710 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54309[] = "Mozilla/5.0 (Linux; Android 4.0.3; Freelander I20 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54310[] = "Mozilla/5.0 (Linux; Android 4.0.3; Netbook PC 12700 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54311[] = "Mozilla/5.0 (Linux; Android 4.0.4; M1001 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54312[] = "Mozilla/5.0 (Linux; Android 4.0.4; V8502 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54313[] = "Mozilla/5.0 (Linux; Android 4.0.4; magcomm77_cu_ics2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54314[] = "Mozilla/5.0 (Linux; Android 4.0.4; X714 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54315[] = "Mozilla/5.0 (Linux; Android 4.0.4; VL-109 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54316[] = "Mozilla/5.0 (Linux; Android 4.0.4; BC1077 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54317[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26w Build/6.2.B.1.96) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54318[] = "Mozilla/5.0 (Linux; Android 4.0.3;  ARCHOS 80G9 Build/MR1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S54319[] = "Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 80G9 Build/MR1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S54320[] = "Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80G9 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S54321[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB526 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54322[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MB526 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54323[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One V Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54324[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One V Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54325[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST23i Build/11.0.A.5.5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54326[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST23a Build/11.0.A.2.16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54327[] = "KWC-S2300/ABR1003 UP.Browser/7.2.7.2.551 (GUI) MMP/2.0";
const char S54328[] = "KWC-S2300-C/ACF1002 UP.Browser/7.2.7.2.549 (GUI) MMP/2.0";
const char S54329[] = "KWC-S2300/ABQ1000 UP.Browser/7.2.7.2.549 (GUI) MMP/2.0";
const char S54330[] = "KWC-S2300/ABQ1103 UP.Browser/7.2.7.2.549 (GUI) MMP/2.0";
const char S54331[] = "KWC-S2300/ABU1001 UP.Browser/7.2.7.2.552 (GUI) MMP/2.0";
const char S54332[] = "KWC-S2300/ABR1001 UP.Browser/7.2.7.2.551 (GUI) MMP/2.0";
const char S54333[] = "KWC-S2300/ABR1004 UP.Browser/7.2.7.2.551 (GUI) MMP/2.0";
const char S54334[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:12.3) Gecko/X Firefox/12.3-x64 PaleMoon/12.3-x64";
const char S54335[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.3) Gecko/X Firefox/12.3r2 PaleMoon/12.3r2";
const char S54336[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC PH39100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54337[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S54338[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; XOOM 2 ME Build/1.6.0M_278-MZ607) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S54339[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; XOOM 2 ME Build/1.6.0MW-211.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S54340[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; XOOM 2 ME Build/1.6.0_218.3-MZ607) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S54341[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; XOOM 2 ME Build/1.6.0_218.3-MZ607) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S54342[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 ME Build/1.6.0_268.3-MZ607) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S54343[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; XOOM 2 ME Build/1.6.0_218.3-MZ607)";
const char S54344[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; XOOM 2 ME Build/1.6.0MW-211.7) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S54345[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; XOOM 2 ME Build/1.6.0_218.3-MZ607) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S54346[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 ME Build/1.6.0_268.4-MZ607) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S54347[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 ME Build/1.6.0_249.1-MZ607) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S54348[] = "Mozilla/5.0 (Linux; U; Android 3.2; pt-br; XOOM 2 ME Build/1.6.0_218.11-MZ607) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S54349[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; XOOM 2 ME Build/1.6.0M_278-MZ607) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S54350[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; XOOM 2 ME Build/1.6.0M_268-MZ607) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S54351[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 ME Build/1.6.0_268.4-MZ607) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S54352[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 ME Build/1.6.0M_280_MZ607) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S54353[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S54354[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S54355[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S54356[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I535 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S54357[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T999 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S54358[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-I9300T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S54359[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30; Maxthon (4.0.4.1000);";
const char S54360[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54361[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S54362[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:16.0) Gecko/X Firefox/16.0 AlexaToolbar/alxf-2.17";
const char S54363[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) rekonq/1.1 Safari/534.34";
const char S54364[] = "SAMSUNG-GT-S5263/S5263DDKE1 SHP/VPP/R5 Dolfin/2.0 NexPlayer/3.0 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B";
const char S54365[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE72-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S54366[] = "NokiaE72-1/UCWEB8.7.0.187/28/999";
const char S54367[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiae72-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba";
const char S54368[] = "NOKIAE72-1UCBrowser/8.7.1.234/28/444/UCWEB";
const char S54369[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.35 (KHTML, like Gecko) Ubuntu/10.10 Chromium/13.0.764.0 Chrome/13.0.764.0 Safari/534.35";
const char S54370[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/10.10 Chromium/13.0.782.99 Chrome/13.0.782.99 Safari/535.1";
const char S54371[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; MID_Serials Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54372[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; MID_Serials Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54373[] = "Dalvik/1.4.0 (Linux; U; Android 2.3; MID_Serials Build/GINGERBREAD)";
const char S54374[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.1; MID_Serials Build/GINGERBREAD)";
const char S54375[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Parrot Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54376[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; MID_Serials Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54377[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Avant Browser)";
const char S54378[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Avant Browser)";
const char S54379[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Avant Browser)";
const char S54380[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Avant Browser)";
const char S54381[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Avant Browser)";
const char S54382[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S54383[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300TG Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S54384[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300TL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S54385[] = "Mozilla/5.0 (Linux; Android 4.1.2; Transformer Pad TF300T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S54386[] = "Mozilla/5.0 (Linux; Android 4.0.3;  HTC One XL Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S54387[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One XL Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S54388[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One XL Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S54389[] = "Mozilla/5.0 (Android; Mobile; rv:14.0) Gecko/14.0 K9Browser/14.0.b5";
const char S54390[] = "Mozilla/5.0 (Android; Mobile; rv:14.0) Gecko/14.0 K9Browser/14.0.1";
const char S54391[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySO-04D Build/7.0.D.1.117) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54392[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-04D Build/7.0.D.1.117) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54393[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SO-04D Build/7.0.D.1.117)";
const char S54394[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySO-04D Build/7.0.D.1.130) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54395[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-04D Build/7.0.D.1.130) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54396[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySO-04D Build/7.0.D.1.137) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54397[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-04D Build/7.0.D.1.137) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54398[] = "Nokia6280/2.0 (03.60) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S54399[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGP321 Build/10.1.1.A.1.307) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S54400[] = "NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-00) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S54401[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; WIKO-CINK SLIM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54402[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; WIKO-CINK SLIM Build/IMM76D)";
const char S54403[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST25i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54404[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST25a Build/6.1.1.B.1.10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54405[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S54406[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7; en-us) AppleWebKit/533.4 (KHTML, like Gecko) Version/4.1 Safari/533.4";
const char S54407[] = "Opera/9.80 (X11; Linux i686; Edition Linux Mint) Presto/2.12.388 Version/12.12";
const char S54408[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; dtab01 Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54409[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MediaPad 10 LINK Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54410[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fi-fi; MediaPad 7 Vogue Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54411[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MediaPad 7 Youth Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54412[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Telpad Dual S Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54413[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; dtab01 Build/HuaweiMediaPad)";
const char S54414[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; MediaPad 10 LINK Build/HuaweiMediaPad)";
const char S54415[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; SGH-T999 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S54416[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9300 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S54417[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SCH-I535 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S54418[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; GT-I9300 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S54419[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SGH-T999 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S54420[] = "BlackBerry8820/4.5.0.110 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102";
const char S54421[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P5100 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S54422[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P5100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S54423[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T779 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S54424[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; es-ES; rv:1.8.0.3) Gecko/X Firefox/1.5.0.3";
const char S54425[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.13.13719.201";
const char S54426[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0) QQBrowser/6.13.13719.201";
const char S54427[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) QQBrowser/6.13.13719.201";
const char S54428[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HW-HUAWEI_C8650/C8650V100R001C92B825; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S54429[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; C8650 Build/HuaweiC8650) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S54430[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8650+ Build/V100R001C92B861) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54431[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8650+ Build/V100R001C92B861) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S54432[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-id; C8650 Build/HuaweiC8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54433[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; C8650 Build/HuaweiC8650) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S54434[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; C8650 Build/HuaweiC8650)";
const char S54435[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; c8650 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S54436[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; C8650 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54437[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; C8650 Build/HuaweiC8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54438[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8650+ Build/V100R001C92B861) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S54439[] = "Mozilla 5.0 (Linux; U; Android 2.3.3; zh-cn; C8650 Build HuaweiC8650) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S54440[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en_id; C8650 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.2.0 Mobile (Dragonfruit)";
const char S54441[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; C8650 Build/HuaweiC8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54442[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; HUAWEI C8650+ Build/V100R001C92B861)";
const char S54443[] = "Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8650+ Build V100R001C92B861) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S54444[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; C8650/C8650+ Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S54445[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC PH39100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S54446[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire C Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S54447[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire C Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S54448[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9205 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54449[] = "NokiaN72/5.0819.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.9.1.120/27/351/UCWEB";
const char S54450[] = "NokiaN72/ 5.0741.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UCWEB7.9.0.94/27/999";
const char S54451[] = "Mozilla/5.0 (Series40; NokiaX2-01/08.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S54452[] = "Mozilla/5.0 (Series40; NokiaX2-01.1/07.22; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S54453[] = "Mozilla/5.0 (Series40; NokiaX2-01/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5";
const char S54454[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID9740 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54455[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID9740 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54456[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; ISW11K Build/134.3.0005) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54457[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; ISW11K Build/136.1.0000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54458[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; ISW11K Build/138.2.0001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54459[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; ISW11K Build/143.2.0001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54460[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; ISW11K Build/142.2.0000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54461[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; ISW11K Build/144.2.0000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54462[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; ISW11K Build/145.0.0002) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54463[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; ISW11K Build/145.0.0002)";
const char S54464[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8L1 Safari/6533.18.5";
const char S54465[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8J2 Safari/6533.18.5";
const char S54466[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8G4 Safari/6533.18.5";
const char S54467[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8F190 Safari/6533.18.5";
const char S54468[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8H7 Safari/6533.18.5";
const char S54469[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8K2 Safari/6533.18.5";
const char S54470[] = "Opera/9.80 (Android; Opera Mini/6.24212/26.1305; U; en) Presto/2.8.119 Version/10.54";
const char S54471[] = "Opera/9.80 (Android; Opera Mini/6.25979/26.1670; U; es) Presto/2.8.119 Version/10.54";
const char S54472[] = "Opera/9.80 (Android; Opera Mini/6.25174/24.902; U; de) Presto/2.5.25 Version/10.54";
const char S54473[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.2 Safari/535.19 Silk-Accelerated=true";
const char S54474[] = "Mozilla/5.0 (PlayStation Vita 2.11) AppleWebKit/536.26 (KHTML, like Gecko) Silk/3.2";
const char S54475[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LS970 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54476[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LS970 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S54477[] = "Nokia2700c-2/2.0 (09.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia2700c-2) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S54478[] = "Nokia2700c/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia2700c) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S54479[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT28h Build/6.2.B.0.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54480[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT28h Build/6.2.B.0.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54481[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyLT28i Build/6.2.B.0.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54482[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LT28h Build/6.2.B.0.211)";
const char S54483[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LT28i Build/6.2.B.0.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54484[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LT28i Build/6.2.B.0.211)";
const char S54485[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR6425LVW Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54486[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6425LVW Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54487[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGP312 Build/10.1.C.0.370) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S54488[] = "Mozilla/5.0 (PlayStation Vita 2.60) AppleWebKit/536.26 (KHTML, like Gecko) Silk/3.2";
const char S54489[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500R Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54490[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54491[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54492[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54493[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1S Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S54494[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54495[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1SC Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S54496[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54497[] = "Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1S Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S54498[] = "Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1SC Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S54499[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1S Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S54500[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MI 1S Build/IMM76D)";
const char S54501[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MI 1S MIUI/ICS24.0)";
const char S54502[] = "Mozilla/5.0(iPad; U;CPU OS 5_1 like Mac OS X; zh-CN; iPad3,1) AppleWebKit/534.46 (KHTML, like Gecko) UCBrowser/2.0.1.280 U3/0.8.0 Safari/7543.48.3";
const char S54503[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5233) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S54504[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia 5233) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S54505[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID8125 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54506[] = "Logos/0.9.2 beta 2 (Linux; U; Android 2.3.3; es_US; MID8125 Build/MID8125 X)";
const char S54507[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID8125 Build/GRI40)";
const char S54508[] = "NokiaC1-01/2.0 (04.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac1-01) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S54509[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E425 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54510[] = "LG-E425f-AB/E425f10d Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; LG-E425f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54511[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E425g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54512[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E425 Build/JZO54K)";
const char S54513[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E425f Build/JZO54K)";
const char S54514[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E425g Build/JZO54K)";
const char S54515[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S54516[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S54517[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:21.0) Gecko/X Firefox/21.0 AlexaToolbar/alxf-2.18";
const char S54518[] = "Mozilla/5.0 (Linux; Android 4.0.3; cm_tenderloin Build/GWK74) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54519[] = "Mozilla/5.0 (Linux; Android 4.0.4; cm_tenderloin Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S54520[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S54521[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830C Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S54522[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-L710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54523[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-L710 Build/IMM76D)";
const char S54524[] = "Mozilla/5.0 (Linux; Android 4.3; Galaxy Nexus Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S54525[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-H Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S54526[] = "Mozilla/5.0 (Linux; Android 4.0.4; SmartTabII7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Safari/537.36";
const char S54527[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-F Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S54528[] = "Mozilla/5.0 (Linux; Android 4.0.4; A2107A-H Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S54529[] = "SonyEricssonW20i/R7DA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2";
const char S54530[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT881 Build/9.8.3O-118_USC-14) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S54531[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; N90 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54532[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N90 DUAL CORE2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54533[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N90 DUAL CORE2 V12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54534[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N90 DUAL CORE 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54535[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; N90 Build/IML74K)";
const char S54536[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; N90 Build/IML74K; 768*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S54537[] = "FreeRSSReader/1.9.5 CFNetwork/609 Darwin/13.0.0";
const char S54538[] = "MobileRSSFree/3.4 CFNetwork/609 Darwin/13.0.0";
const char S54539[] = "RSSRadio/2.07 CFNetwork/609 Darwin/13.0.0";
const char S54540[] = "MobileRSS/3.4.1 CFNetwork/609 Darwin/13.0.0";
const char S54541[] = "FeeddlerRSS/1.12.5 CFNetwork/609.1.4 Darwin/13.0.0";
const char S54542[] = "MusicDownloaderSSA/1.0.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S54543[] = "Pics4Facebook/9.15.1 CFNetwork/609.1.4 Darwin/13.0.0";
const char S54544[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Safari/535.19 Silk-Accelerated=false";
const char S54545[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3100 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54546[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3100 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54547[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3100 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54548[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; fr-CA; GT-P3100 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Safari/534.30";
const char S54549[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3100 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1.2/p3100";
const char S54550[] = "Mozilla/5.0 (Linux; Android 4.1.2; cm_tenderloin Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S54551[] = "Mozilla/5.0 (Linux; Android 4.1.2; skz_tenderloin Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S54552[] = "Mozilla/5.0 (Linux; Android 4.1.1; Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S54553[] = "Mozilla/5.0 (Linux; Android 4.1.1; V1013 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S54554[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbl1 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54555[] = "Mozilla/5.0 (Linux; Android 4.1.2; Motorola Galaxy s3 Build/9.8.2O-72_VZW-8) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54556[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1911_v77_gq2000_a41_6628_v20 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54557[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbla668_fwvga Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54558[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbla668_9p017_fwvga Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54559[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1909l_v77_gq1008_a41_v20 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54560[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1908_v77_jbl1_9p017_6628 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54561[] = "Mozilla/5.0 (Linux; Android 4.1.1; Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54562[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1920_v77_zlh_9p017_asx Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54563[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_gq2002 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54564[] = "Mozilla/5.0 (Linux; Android 4.1.5; iPhone5 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54565[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_gq2008_ov8825 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54566[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jdt1_9p017 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54567[] = "Mozilla/5.0 (Linux; Android 4.1.2; e2001_v89_gq2008s Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54568[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1920_v77_zlh_fwvga Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54569[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1921_v77_cxq1 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54570[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbla668_9p017 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54571[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54572[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562L Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54573[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST25i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S54574[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST25a Build/6.1.1.B.1.10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S54575[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST26a Build/11.2.A.0.21) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54576[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST26i Build/11.2.A.0.31) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54577[] = "Mozilla/5.0 (X11; Linux i686 on x86_64; rv:24.0) Gecko/X Firefox/24.0";
const char S54578[] = "Mozilla/5.0 (X11; Linux x86_64; rv:24.0) Gecko/X Firefox/24.0 Iceweasel/24.0";
const char S54579[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT30p Build/7.0.A.3.195) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54580[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT30p Build/7.0.A.3.195) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S54581[] = "IUC(U;iOS 5.1.1;Zh-cn;320*480;)/UCWEB8.8.0.212/42/997";
const char S54582[] = "IUC(U;iOS 5.0.1;Zh-cn;320*480;)/UCWEB8.8.0.212/42/800";
const char S54583[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S54584[] = "IUC(U;iOS 6_0_1;en-us;320*480;)/UCWEB8.8.4.247/44/355";
const char S54585[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; mstar) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S54586[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; maui) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S54587[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; spice) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S54588[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; opera) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S54589[] = "UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; maui) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S54590[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; mozilla) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S54591[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; generic) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S54592[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; jbed-fastbcc) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S54593[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0";
const char S54594[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; mstar platform) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S54595[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; sunmicro) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S54596[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; maui browser) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S54597[] = "IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB8.8.0.212/41/800";
const char S54598[] = "UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; mstar) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S54599[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LW770 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S54600[] = "Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; F-01D Build/F0001) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S54601[] = "Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; F-01D Build/V27R41C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S54602[] = "Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; F-01D Build/V28R43A) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S54603[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; F-01D Build/V28R43A)";
const char S54604[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:9.0) Gecko/X Firefox/9.0";
const char S54605[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:9.0) Gecko/X Firefox/9.0";
const char S54606[] = "Mozilla/5.0 (Windows NT 6.2; rv:9.0.1) Gecko/X Firefox/9.0.1 Firefox 9.0";
const char S54607[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP4100 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S54608[] = "Mozilla/5.0 (Windows NT 4.0; rv:15.0) Gecko/X Firefox/15.0.1";
const char S54609[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Light Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54610[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Light Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54611[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; Light Build/FRF91)";
const char S54612[] = "Mozilla/5.0 (Series40; NokiaC3-00/08.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.0.9.17.2";
const char S54613[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5800 XpressMusic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile";
const char S54614[] = "Nokia5800 XpressMusic/UCWEB8.6.0.180/50/800";
const char S54615[] = "Mozilla/5.0 (Java; U; en-us; nokia5800d-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/84/352/UCWEB Mobile  3gpp-gba";
const char S54616[] = "Nokia5800w XpressMusic/UCWEB8.6.0.180/50/800";
const char S54617[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaC5-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile";
const char S54618[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaC5-00.2) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/355/UCWEB Mobile";
const char S54619[] = "Mozilla/5.0 (Java; U; en-us; nokiac5-00.2) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile  3gpp-gba";
const char S54620[] = "Mozilla/5.0 (Java; U; en-us; nokiac5-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile  3gpp-gba";
const char S54621[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyC6506 Build/10.1.A.1.395) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54622[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; C6506 Build/10.1.A.1.395)";
const char S54623[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; C6506 Build/10.1.A.1.395) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54624[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C6506 Build/10.1.A.1.434) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54625[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyC6506 Build/10.1.A.1.350) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54626[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C6506 Build/10.1.1.A.1.253) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54627[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-mx; C6506 Build/10.1.A.1.350) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54628[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LGMS659 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54629[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LGMS659 Build/JZO54K)";
const char S54630[] = "Mozilla/5.0 (iPad; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S54631[] = "Mozilla/5.0 (iPad; U; Intel Mac OS X 10_6_8) AppleWebKit/534.30 (KHTML, like Gecko) Version/5.0.5 Safari/534.30";
const char S54632[] = "Mozilla/5.0 (iPad; U; Intel Mac OS X 10_6_8) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26";
const char S54633[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; LG-P925g Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54634[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; LG-P925g Build/GRJ90)";
const char S54635[] = "Mozilla/5.0 (Linux; Android 4.2.2; A1-810 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S54636[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9B206 Safari/7534.48.3";
const char S54637[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9B208 Safari/7534.48.3";
const char S54638[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9B176 Safari/7534.48.3";
const char S54639[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9B179 Safari/7534.48.3";
const char S54640[] = "PANTECH-P7000/JDUS10052009; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.15645; U; en-US) Opera 9.50";
const char S54641[] = "PANTECH-P7000/JDUS10052009; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.15645; U; es) Opera 9.50";
const char S54642[] = "PANTECH-P7000/JDUSX; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.15645; U; fr) Opera 9.50";
const char S54643[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/11A465 Safari/8536.25";
const char S54644[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/11A501 Safari/8536.25";
const char S54645[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S54646[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Iron/25.0.1400.0 Chrome/25.0.1400.0 Safari/537.22";
const char S54647[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Iron/25.0.1400.0 Chrome/25.0.1400.0 Safari/537.22";
const char S54648[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.22 (KHTML, like Gecko) Iron/25.0.1400.0 Chrome/25.0.1400.0 Safari/537.22";
const char S54649[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D2-721 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54650[] = "AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; D2-721 Build/JRO03H)";
const char S54651[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D2-721G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54652[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; D2-721G Build/JRO03H)";
const char S54653[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54654[] = "Nokia6030/2.0 (5.11) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S54655[] = "Nokia6030b/2.0 (5.25) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S54656[] = "Nokia6030/2.0 (5.40) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2), Nokia6030/2.0 (5.40) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S54657[] = "Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_5) AppleWebKit (KHTML, like Gecko) Chrome/6";
const char S54658[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.53 Safari/534.3";
const char S54659[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.55 Safari/534.3";
const char S54660[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.2 (KHTML, like Gecko) Chrome/6.0.453.1 Safari/534.2";
const char S54661[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S54662[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S54663[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S54664[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus 10 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S54665[] = "SAMSUNG-SGH-A157/A157UCLD1 SHP/VPP/R5 NetFront/4.2 SMM-MMS/1.2.0";
const char S54666[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-P500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54667[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SPH-P500 Build/JZO54K)";
const char S54668[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54669[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; U8800Pro Build/HuaweiU8800Pro) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54670[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; el-gr; U8800 Pro Build/HuaweiU8800Pro) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54671[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; U8800Pro Build/HuaweiU8800ProB032) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54672[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; IDEOS X5 Build/HuaweiU8800Pro) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54673[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-cn; U8800+ Build/HuaweiU8800Pro) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54674[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-za; RBM_HD Build/HuaweiU8800Pro) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54675[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ru-by; MTC Neo Build/HuaweiU8800Pro) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54676[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M340S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54677[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHW-M340S Build/GINGERBREAD)";
const char S54678[] = "NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-00) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S54679[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S54680[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus 7 Build/JRO03S) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S54681[] = "Mozilla/5.0 (Linux; Android 4.1; Nexus 7 Build/JRN84D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S54682[] = "NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S54683[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S54684[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S54685[] = "Mozilla/5.0 (Linux; Android 4.0.4; Hydro Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S54686[] = "Mozilla/5.0 (Linux; Android 4.0.4; C5170 Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S54687[] = "Mozilla/5.0 (Linux; Android 4.0.4; LGL86C Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S54688[] = "Reeder/1.5.7 CFNetwork/609 Darwin/13.0.0";
const char S54689[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.162.11; es-es) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54690[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.162.11; de-de) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54691[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.162.11; nl-nl) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54692[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.161.11; en-gb) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54693[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.161.11; en-ie) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54694[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710a; en-au) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54695[] = "Mozilla/5.0 (X11; Linux x86_64; HTC-Z710a; en-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54696[] = "Mozilla/5.0 (X11; Linux x86_64; HTC-Z710a; fr-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54697[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.163.12; fr-fr) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54698[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.166.11; de-ch) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54699[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.165.11; it-it) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54700[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.162.11; en-de) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54701[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.166.11; fr-ch) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54702[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.162.52; de-de) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54703[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.161.52; en-gb) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54704[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.162.52; nl-nl) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54705[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.161.52; en-ie) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54706[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.162.52; es-es) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54707[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.166.52; de-ch) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54708[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.163.53; fr-fr) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54709[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.165.52; it-it) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54710[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.166.52; fr-ch) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54711[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.163.12; de-at) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S54712[] = "Mozilla/4.0 (compatible; MidEx 0.01; Midori 0.01)";
const char S54713[] = "Mozilla/4.0 (compatible; MidEx 0.01; Midori 0.01;)";
const char S54714[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Micromax A57 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54715[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Micromax A57 Build/GINGERBREAD)";
const char S54716[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7100 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S54717[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N7100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S54718[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-N719 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S54719[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SGH-T889 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S54720[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7105 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S54721[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; SCH-N719 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S54722[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT29i Build/7.0.A.3.212) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S54723[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; BLU Studio 5.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54724[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; BLU Studio 5.3 Build/GRJ90)";
const char S54725[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST26i Build/11.0.A.7.5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54726[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST26a Build/11.0.A.7.5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54727[] = "Mozilla/5.0 (Linux; Android 4.1.2; N861 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S54728[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID1126 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54729[] = "Logos/0.9.3 (Linux; U; Android 2.3.3; en_US; MID1126 Build/MID1126 X)";
const char S54730[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID1126 Build/GRI40)";
const char S54731[] = "SAMSUNG-GT-B2710/B2710XXJK1 SHP/VPP/R5 Dolfin/2.0 QTV/5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B";
const char S54732[] = "SAMSUNG-GT-B2710D/B2710DDDJJ2 SHP/VPP/R5 Dolfin/2.0 QTV/5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B";
const char S54733[] = "SAMSUNG-SGH-A877/A877UCID3 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S54734[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P7510 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S54735[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7510 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S54736[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; USCC-C6721 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54737[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; USCC-C6721 Build/JZO54K)";
const char S54738[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9A405 Safari/7534.48.3";
const char S54739[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9A334 Safari/7534.48.3";
const char S54740[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9295 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54741[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I467 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S54742[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I467M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S54743[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LS860 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S54744[] = "Mozilla/5.0 (Linux; Android 4.0.4; Mach Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S54745[] = "KDDI-KC46 UP.Browser/6.2_7.2.7.1.K.7.108 (GUI) MMP/2.0";
const char S54746[] = "KDDI-SA3B UP.Browser/6.2_7.2.7.1.K.1.5.1.120 (GUI) MMP/2.0";
const char S54747[] = "KDDI-SH35 UP.Browser/6.2_7.2.7.1.K.2.207 (GUI) MMP/2.0";
const char S54748[] = "KDDI-SN3C UP.Browser/6.2_7.2.7.1.K.1.5.1.120 (GUI) MMP/2.0";
const char S54749[] = "KDDI-HI38 UP.Browser/6.2.0.11.1.3.110 (GUI) MMP/2.0";
const char S54750[] = "KDDI-TS3K UP.Browser/6.2_7.2.7.1.K.2.234 (GUI) MMP/2.0";
const char S54751[] = "KDDI-TS3A UP.Browser/6.2.0.11.2.1e (GUI) MMP/2.0";
const char S54752[] = "KDDI-SH3C UP.Browser/6.2_7.2.7.1.K.3.352 (GUI) MMP/2.0";
const char S54753[] = "KDDI-CA3B UP.Browser/6.2_7.2.7.1.K.2.234 (GUI) MMP/2.0";
const char S54754[] = "KDDI-TS3Y UP.Browser/6.2_7.2.7.1.K.7.112 (GUI) MMP/2.0";
const char S54755[] = "KDDI-TS3Z UP.Browser/6.2_7.2.7.1.K.7.112 (GUI) MMP/2.0";
const char S54756[] = "KDDI-HI3F UP.Browser/6.2_7.2.7.1.K.3.354 (GUI) MMP/2.0";
const char S54757[] = "KDDI-SN3U UP.Browser/6.2_7.2.7.1.K.7.112 (GUI) MMP/2.0";
const char S54758[] = "KDDI-SH37 UP.Browser/6.2_7.2.7.1.K.2.232 (GUI) MMP/2.0";
const char S54759[] = "KDDI-HI3G UP.Browser/6.2_7.2.7.1.K.3.354 (GUI) MMP/2.0";
const char S54760[] = "KDDI-CA3K UP.Browser/6.2_7.2.7.1.K.7.112 (GUI) MMP/2.0";
const char S54761[] = "KDDI-TS3M UP.Browser/6.2_7.2.7.1.K.2.234 (GUI) MMP/2.0";
const char S54762[] = "KDDI-SA3D UP.Browser/6.2_7.2.7.1.K.1.5.1.120 (GUI) MMP/2.0";
const char S54763[] = "KDDI-SA3E UP.Browser/6.2_7.2.7.1.K.2.232 (GUI) MMP/2.0";
const char S54764[] = "KDDI-CA3D UP.Browser/6.2_7.2.7.1.K.3.354 (GUI) MMP/2.0";
const char S54765[] = "KDDI-FJ31 UP.Browser/6.2_7.2.7.1.K.8.160 (GUI) MMP/2.0";
const char S54766[] = "KDDI-SN3V UP.Browser/6.2_7.2.7.1.K.7.113 (GUI) MMP/2.0";
const char S54767[] = "KDDI-TS3L UP.Browser/6.2_7.2.7.1.K.2.234 (GUI) MMP/2.0";
const char S54768[] = "KDDI-KC38 UP.Browser/6.2.0.11.1.2.2e (GUI) MMP/2.0";
const char S54769[] = "KDDI-KC4A UP.Browser/6.2_7.2.7.1.K.7.1.104 (GUI) MMP/2.0";
const char S54770[] = "KDDI-PT37 UP.Browser/6.2.0.17.2.4.1.2 (GUI) MMP/2.0";
const char S54771[] = "KDDI-KC45 UP.Browser/6.2.0.17.2 (GUI) MMP/2.0";
const char S54772[] = "Mozilla/5.0 (Windows NT 6.0; rv:17.0) Gecko/X Firefox/17.0 SeaMonkey/2.14.1";
const char S54773[] = "FeeddlerRSS 1.12.5 (iPad; iPhone OS 6.1; en_US)";
const char S54774[] = "FeeddlerRSS 1.12.5 (iPad; iPhone OS 6.1.2; en_US)";
const char S54775[] = "Appcelerator Titanium/1.7.2 (iPad/6.1.3; iPhone OS; en_US;)";
const char S54776[] = "FeeddlerRSS 1.12.5 (iPad; iPhone OS 6.1.3; en_US)";
const char S54777[] = "Mozilla/5.0 (Series40; NokiaC3-01.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S54778[] = "Mozilla/5.0 (Series40; NokiaC3-01/07.15; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S54779[] = "Mozilla/5.0 (Series40; Nokia305/05.92; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S54780[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Avant Browser)";
const char S54781[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Avant Browser)";
const char S54782[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Avant Browser; SV1)";
const char S54783[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Avant Browser; SV1)";
const char S54784[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Avant Browser; Avant Browser)";
const char S54785[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Avant Browser; Avant Browser)";
const char S54786[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26w Build/6.1.A.2.45) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S54787[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Note II Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S54788[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54789[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7105 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54790[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I317 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54791[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T889 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54792[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L900 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54793[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R950 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S54794[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C5215 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54795[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; C5215 Build/JZO54K)";
const char S54796[] = "BlackBerry8520/5.2.0.64 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/159/444";
const char S54797[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E450 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54798[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E450f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54799[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E450g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54800[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E450f Build/JZO54K)";
const char S54801[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E450 Build/JZO54K)";
const char S54802[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LG-E450B/E450B11e-BLM-CA Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54803[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LG-E450B/E450B11c-BLM-CA Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54804[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E450g Build/JZO54K)";
const char S54805[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LG-E450B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54806[] = "Mozilla/4.0 (BREW 3.1.5; U; en-us; Sanyo; Polaris/6.0/AMB) Boost SCP-2700";
const char S54807[] = "Mozilla/4.0 (BREW 3.1.5; U; en-us; Sanyo; Polaris/6.0/AMB) Sprint SCP-2700";
const char S54808[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S54809[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S54810[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SCH-i509 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S54811[] = "CybEye.com/2.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/4.0; GTB6.4)";
const char S54812[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T711 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54813[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; T711 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54814[] = "Nokia5300/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S54815[] = "Nokia5300/2.0 (05.00) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia5300";
const char S54816[] = "Mozilla/4.0 (compatible; MSIE 5.0;";
const char S54817[] = "Mozilla/5.0 (compatible; MSIE 5.0)";
const char S54818[] = "Mozilla/5.0 (compatible; MSIE 5.0; PPC; Trident/1.0)";
const char S54819[] = "Mozilla/5.0 (compatible; MSIE 5.0; PPC; SV1; Trident/3.0)";
const char S54820[] = "Mozilla/5.0 (Linux; Android 4.2.1; M470BSA Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S54821[] = "iTunes/10.5 (Downcast v2.5.5 (Build 1001))";
const char S54822[] = "iTunes/10.5 (Downcast v2.5.2 (Build 1006))";
const char S54823[] = "iTunes/10.6 (Downcast v2.7.0 (Build 1001))";
const char S54824[] = "iTunes/10.0 (Downcast v2.1.3 (Build 1004))";
const char S54825[] = "iTunes/10.6 (Downcast v2.7.1 (Build 1029))";
const char S54826[] = "iTunes/10.5 (Downcast v2.5.6 (Build 1015))";
const char S54827[] = "iTunes/10.6 (Downcast v2.7.2 (Build 1012))";
const char S54828[] = "iTunes/10.6 (Downcast v2.7.3 (Build 1004))";
const char S54829[] = "iTunes/10.6 (Downcast v2.7.4 (Build 1008))";
const char S54830[] = "iTunes/10.5.2 (PodCruncher 1.9.2)";
const char S54831[] = "iTunes/10.6 (Downcast v2.7.5 (Build 1013))";
const char S54832[] = "iTunes/10.6 (Downcast v2.7.6 (Build 1035))";
const char S54833[] = "iTunes/10.6 (Downcast v2.7.7 (Build 1000))";
const char S54834[] = "iTunes/10.6 (Downcast v2.7.8 (Build 1053))";
const char S54835[] = "iTunes/10.6 (Downcast v2.7.9 (Build 1004))";
const char S54836[] = "iTunes/10.6 (Downcast v2.7.10 (Build 1015))";
const char S54837[] = "iTunes/10.6 (Downcast v2.7.11 (Build 1032))";
const char S54838[] = "iTunes/10.5.2 (PodCruncher 2.0)";
const char S54839[] = "iTunes/10.7 (Downcast/2.7.12 (Build 1005))";
const char S54840[] = "iTunes/10.7 Downcast/2.7.15.1007";
const char S54841[] = "iTunes/10.7 Downcast/2.8.0.1025";
const char S54842[] = "iTunes/10.7 Downcast/2.8.1.1034";
const char S54843[] = "iTunes/10.7 Downcast/2.8.2.2001";
const char S54844[] = "iTunes/10.7 Downcast/2.8.3.2097";
const char S54845[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36 OPR/16.0.1196.55 (Edition Next)";
const char S54846[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36 OPR/16.0.1196.73";
const char S54847[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.11 Safari/537.36  OPR/16.0.1146.1";
const char S54848[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.49 Safari/537.36 OPR/16.0.1196.41 (Edition Next),gzip(gfe)";
const char S54849[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.49 Safari/537.36 OPR/16.0.1196.41 (Edition Next)";
const char S54850[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36 OPR/16.0.1196.55 (Edition Next)";
const char S54851[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36 OPR/16.0.1196.62,gzip(gfe)";
const char S54852[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36 OPR/16.0.1196.73";
const char S54853[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36 OPR/16.0.1196.62 (Edition Freesoft)";
const char S54854[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36 OPR/16.0.1196.73";
const char S54855[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36 OPR/16.0.1196.73 (Edition Rambler)";
const char S54856[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; HS-9DTB4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54857[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; HS-9DTB4-4GB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54858[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HS-9DTB4 Build/IMM76D)";
const char S54859[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; ALCATEL_one_touch_916 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54860[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; ALCATEL ONE TOUCH 916D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54861[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; ALCATEL ONE TOUCH 916 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54862[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-pt; Alcatel_one_touch_916_Helsinki Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54863[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; ALCATEL ONE TOUCH 916A Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54864[] = "Opera/9.50 (Macintosh; Intel Mac OS X; U; en)";
const char S54865[] = "Opera/9.51 (Macintosh; Intel Mac OS X; U; en)";
const char S54866[] = "Opera/9.52 (Macintosh; Intel Mac OS X; U; en)";
const char S54867[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E170K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54868[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E170S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54869[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E170L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54870[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHV-E170K Build/IMM76D)";
const char S54871[] = "Mozilla/5.0 (PlayStation Vita 2.02) AppleWebKit/536.26 (KHTML, like Gecko) Silk/3.2";
const char S54872[] = "Mozilla/5.0 (Linux; Android 4.1.1; H100 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54873[] = "Mozilla/5.0 (Linux; Android 4.1.1; MiSS Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S54874[] = "Mozilla/5.0 (Linux; Android 4.1.2; ThinkPadTablet Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S54875[] = "Mozilla/5.0 (Linux; Android 4.1.1; y7053 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S54876[] = "Mozilla/5.0 (Linux; Android 4.1.2; B943 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54877[] = "Mozilla/5.0 (Linux; Android 4.1.2; A9600 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54878[] = "Mozilla/5.0 (Linux; Android 4.1.1; YOUPU U6 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54879[] = "Mozilla/5.0 (Linux; Android 4.1.2; M7 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54880[] = "Mozilla/5.0 (Linux; Android 4.1.4; Note3 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54881[] = "Mozilla/5.0 (Linux; Android 4.1.1; X720D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54882[] = "Mozilla/5.0 (Linux; Android 4.1.1; 9930 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54883[] = "Mozilla/5.0 (Linux; Android 4.1.1; CrystalQuadCore Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S54884[] = "Mozilla/5.0 (Linux; Android 4.1.1; G1 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S54885[] = "Mozilla/5.0 (Linux; Android 4.1.1; MD7305 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54886[] = "Mozilla/5.0 (Linux; Android 4.1.2; S9 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S54887[] = "Mozilla/5.0 (Linux; Android 4.1.1; M71 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S54888[] = "Mozilla/5.0 (Linux; Android 4.1.1; Wb7-L Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S54889[] = "Mozilla/5.0 (Linux; Android 4.1.1; MT7A Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S54890[] = "Mozilla/5.0 (Linux; Android 4.1.1; ST70216-1 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S54891[] = "Mozilla/5.0 (Linux; Android 4.1.2; Starmobile JUMP Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54892[] = "Mozilla/5.0 (Linux; Android 4.1.1; MTC 970 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54893[] = "Mozilla/5.0 (Linux; Android 4.1.1; TBQC1063 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S54894[] = "Mozilla/5.0 (Linux; Android 4.1.1; Digital2 Platinum Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S54895[] = "Mozilla/5.0 (Linux; Android 4.1.1; PLT8223G Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S54896[] = "Mozilla/5.0 (Linux; Android 4.1.1; MTC 970H Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54897[] = "Mozilla/5.0 (Linux; Android 4.1.2; Boston 4G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54898[] = "Mozilla/5.0 (Linux; Android 4.1.2; G526-L22 Build/HuaweiG526-L22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54899[] = "Mozilla/5.0 (Linux; Android 4.1.2; Mi-491 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54900[] = "Mozilla/5.0 (Linux; Android 4.1.1; Positivo Ypy L700 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S54901[] = "Mozilla/5.0 (Linux; Android 4.1.1; EGS102 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S54902[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E415f Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54903[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S7392 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54904[] = "Mozilla/5.0 (Linux; Android 4.1.1; Camelio Family tablet Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S54905[] = "Mozilla/5.0 (Linux; Android 4.1.2; N810 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54906[] = "Mozilla/5.0 (Linux; Android 4.1.2; N9510 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54907[] = "Mozilla/5.0 (Linux; Android 4.1.2; WHAM WT72 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54908[] = "Mozilla/5.0 (Linux; Android 4.1.1; XO Learning tablet Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S54909[] = "Mozilla/5.0 (Linux; Android 4.1.2; Huawei Y301A2 Build/HuaweiY301A2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54910[] = "Mozilla/5.0 (Linux; Android 4.1.2; N800 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S54911[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54912[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X dual sim Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54913[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666N Build/HuaweiU8666N) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S54914[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E Build/HuaweiU8666E) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S54915[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E-51 Build/HuaweiU8666E-51) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S54916[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-H Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S54917[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-F Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S54918[] = "Mozilla/5.0 (Linux; Android 4.0.4; SmartTabII7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S54919[] = "Mozilla/5.0 (Linux; Android 4.0.4; A2107A-H Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S54920[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-LG870 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54921[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-LG870 Build/JZO54K)";
const char S54922[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; OMNIA7)";
const char S54923[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; OMNIA7; Orange)";
const char S54924[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; GT-i8700)";
const char S54925[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST25i Build/6.1.1.B.1.10) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54926[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST25a Build/6.1.1.B.1.10) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S54927[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; I10A-LE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S54928[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-D710 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54929[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-D710 Build/JRO03R; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54930[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-D710 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54931[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; YP-GB1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54932[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; YP-GB1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54933[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; YP-G1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54934[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; YP-GB1 Build/FROYO)";
const char S54935[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; YP-G1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S54936[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; YP-G1 Build/FROYO)";
const char S54937[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB300 Build/GINtech_Systems_DeBlurred Flex/P007) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S54938[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; ME600 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S54939[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB300 Build/Blur_Version.3.0.1390.MB300.ATT.en.US Flex/P007) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S54940[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB300 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S54941[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; ME600 Build/Blur_Version.3.0.1390.MB300.ATT.en.US Flex/P020) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S54942[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB300 Build/iaio72) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S54943[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; MB300 Build/ERD79)";
const char S54944[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LG730 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S54945[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; SGH-I897 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54946[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-I897 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54947[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-I897 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54948[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I897 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54949[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I897 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54950[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I897 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54951[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SGH-I897 Build/IML74K)";
const char S54952[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I897 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54953[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I897 Build/IML76K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54954[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I897 Build/IMM76L)";
const char S54955[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.4.2.34167 (Firestorm-Release; firestorm skin) Safari/533.3";
const char S54956[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S54957[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S54958[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7000 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54959[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7000 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54960[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54961[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7000 Build/JRO03R; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54962[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; SC-05D Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54963[] = "iTunes-iPhone/6.0 (6; 32GB; dt:73)";
const char S54964[] = "iTunes-iPhone/6.0 (6; 64GB; dt:82)";
const char S54965[] = "iTunes-iPhone/6.0.1 (6; 16GB; dt:82)";
const char S54966[] = "iTunes-iPhone/6.0.1 (6; 16GB; dt:73)";
const char S54967[] = "iTunes-iPhone/6.0.1 (4; 16GB; dt:27)";
const char S54968[] = "iTunes-iPhone/6.0.1 (4; 32GB; dt:27)";
const char S54969[] = "iTunes-iPhone/6.0.1 (6; 32GB; dt:82)";
const char S54970[] = "iTunes-iPhone/6.0.1 (4; 8GB; dt:27)";
const char S54971[] = "iTunes-iPhone/6.0.1 (6; 64GB; dt:73)";
const char S54972[] = "iTunes-iPhone/6.0.1 (3; 8GB; dt:24)";
const char S54973[] = "iTunes-iPhone/6.0.1 (3; 16GB; dt:24)";
const char S54974[] = "iTunes-iPhone/6.0.1 (5; 16GB; dt:70)";
const char S54975[] = "iTunes-iPhone/6.0.1 (5; 8GB; dt:70)";
const char S54976[] = "iTunes-iPhone/6.0.2 (6; 16GB; dt:79)";
const char S54977[] = "iTunes-iPhone/6.0 (6; 32GB; dt:79)";
const char S54978[] = "iTunes-iPhone/6.0.1 (3; 32GB; dt:24)";
const char S54979[] = "Mozilla/5.0 (Series40; NokiaC2-03/06.51; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.1.21.8.2";
const char S54980[] = "JUC (Linux; U; 2.2.1; zh-cn; GT-S5570; 240*320) UCWEB7.9.0.94/139/444";
const char S54981[] = "JUC (Linux; U; 2.3.6; zh-cn; GT-S5570; 240*320) UCWEB7.9.0.94/139/444";
const char S54982[] = "JUC (Linux; U; 2.3.6; zh-cn; GT-S5570I; 240*320) UCWEB7.9.0.94/139/444";
const char S54983[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AD683G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S54984[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; AD683G Build/IMM76I)";
const char S54985[] = "Mozilla/5.0 (Linux; Android 4.1.2; ME371MG Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S54986[] = "Mozilla/5.0 (X11; Linux i686; rv:16.0) Gecko/X Firefox/16.0 SeaMonkey/2.13.1";
const char S54987[] = "Mozilla/5.0 (PlayStation Vita 2.06) AppleWebKit/536.26 (KHTML, like Gecko) Silk/3.2";
const char S54988[] = "Nokia200/2.0 (11.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia200) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S54989[] = "NetFront/3.5.1(BREW 5.0.2.1; U; en-us; Samsung ; NetFront/3.5.1/WAP) Boost M260 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S54990[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT300 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S54991[] = "Mozilla/5.0 (Linux; Android 4.1.1; XTAB-781+ Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S54992[] = "Mozilla/5.0 (Linux; Android 4.1.1; AN7IG3 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S54993[] = "Mozilla/5.0 (Linux; Android 4.1.2; F600 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S54994[] = "Mozilla/5.0 (Linux; Android 4.1.1; SZENIO 2500 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S54995[] = "Mozilla/5.0 (Linux; Android 4.1.1; MD7305 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S54996[] = "Mozilla/5.0 (Linux; Android 4.1.2; B943 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S54997[] = "Mozilla/5.0 (Linux; Android 4.1.2; A600 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S54998[] = "Mozilla/5.0 (Linux; Android 4.1.1; MiSS Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S54999[] = "Mozilla/5.0 (Linux; Android 4.1.2; Boost Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55000[] = "Mozilla/5.0 (Linux; Android 4.1.1; MTC 970 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55001[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E451g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55002[] = "Mozilla/5.0 (Linux; Android 4.1.1; 9920 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55003[] = "Mozilla/5.0 (Linux; Android 4.1.2; Boston 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55004[] = "Mozilla/5.0 (Linux; Android 4.1.1; MaxiIQ Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55005[] = "Mozilla/5.0 (Linux; Android 4.1.2; A9600 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55006[] = "Mozilla/5.0 (Linux; Android 4.1.1; PMP3970B Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55007[] = "Mozilla/5.0 (Linux; Android 4.1.1; SN97T41W Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55008[] = "Mozilla/5.0 (Linux; Android 4.1.1; YOUPU U6 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55009[] = "Mozilla/5.0 (Linux; Android 4.1.1; it900 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55010[] = "Mozilla/5.0 (Linux; Android 4.1.1; Amazing A4 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55011[] = "Mozilla/5.0 (Linux; Android 4.1.1; WPT005 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55012[] = "Mozilla/5.0 (Linux; Android 4.1.2; A20 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55013[] = "Mozilla/5.0 (Linux; Android 4.1.1; Bouygues Telecom Bs 351 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55014[] = "Mozilla/5.0 (Linux; Android 4.1.1; MD7305 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55015[] = "Mozilla/5.0 (Linux; Android 4.1.2; iQTab 3G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55016[] = "Mozilla/5.0 (Linux; Android 4.1.1; CrystalQuadCore Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55017[] = "Mozilla/5.0 (Linux; Android 4.1.1; Joyz Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55018[] = "Mozilla/5.0 (Linux; Android 4.1.1; PLT8223G Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55019[] = "Mozilla/5.0 (Linux; Android 4.1.1; V8 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55020[] = "Mozilla/5.0 (Linux; Android 4.1.2; ThinkPadTablet Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55021[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E455g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55022[] = "Mozilla/5.0 (Linux; Android 4.1.1; H100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55023[] = "Mozilla/5.0 (Linux; Android 4.1.1; XOLO Play Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55024[] = "Mozilla/5.0 (Linux; Android 4.1.2; M7 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55025[] = "Mozilla/5.0 (Linux; Android 4.1.1; U35GT2 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55026[] = "Mozilla/5.0 (Linux; Android 4.1.2; A950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55027[] = "Mozilla/5.0 (Linux; Android 4.1.1; y7053 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55028[] = "Mozilla/5.0 (Linux; Android 4.1.2; V72A Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55029[] = "Mozilla/5.0 (Linux; Android 4.1.2; W700 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55030[] = "Mozilla/5.0 (Linux; Android 4.1.2; MD700 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55031[] = "Mozilla/5.0 (Linux; Android 4.1.2; Forward_Infinity Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55032[] = "Mozilla/5.0 (Linux; Android 4.1.2; G526-L22 Build/HuaweiG526-L22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55033[] = "Mozilla/5.0 (Linux; Android 4.1.2; H9500+ Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55034[] = "Mozilla/5.0 (Linux; Android 4.1.1; 9930 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55035[] = "Mozilla/5.0 (Linux; Android 4.1.2; I110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55036[] = "Mozilla/5.0 (Linux; Android 4.1.2; one touch D920 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55037[] = "Mozilla/5.0 (Linux; Android 4.1.1; ACGM816KC Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55038[] = "Mozilla/5.0 (Linux; Android 4.1.1; ACGM727KC_N Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55039[] = "Mozilla/5.0 (Linux; Android 4.1.4; Note3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55040[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S7392 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55041[] = "Mozilla/5.0 (Linux; Android 4.1.1; MID-X15 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55042[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6506 Build/10.1.A.1.395) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55043[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.28) Gecko/X Thunderbird/3.1.20";
const char S55044[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Cynus T2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55045[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Cynus T2 Build/IMM76D)";
const char S55046[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55047[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; PMP7280C3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55048[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; PMP7280C3G Build/JZO54K)";
const char S55049[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; PMP7280C3G_DUO Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55050[] = "Nokia6230/2.0 (03.15) Profile/MIDP-2.0 Configuration/CLDC-1.1, Nokia6230/2.0 (03.15) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55051[] = "Nokia6230/2.0 (03.15) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55052[] = "Mozilla/5.0 (Linux; Android 4.0.3; A500 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S55053[] = "Mozilla/5.0 (Linux; Android 4.0.4; A500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S55054[] = "Mozilla/5.0 (Linux; Android 4.0.4; Iconia A500 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S55055[] = "Mozilla/5.0 (Linux; Android 4.0.4; A500 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S55056[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3;) AppleWebKit/523.10.3 (KHTML, like Gecko) Version/3.0.4 Safari/523.10";
const char S55057[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.0.4 Safari/523.10";
const char S55058[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.0.4 Safari/523.15";
const char S55059[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_2; en-us) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.0.4 Safari/523.15";
const char S55060[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.0.4 Safari/523.15";
const char S55061[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.0.4 Safari/523.10";
const char S55062[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.0.4 Safari/523.15";
const char S55063[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.0.4 Safari/523.10";
const char S55064[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.0.4 Safari/523.15";
const char S55065[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.0.4 Safari/523.15";
const char S55066[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.0.4 Safari/523.10";
const char S55067[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.0.4 Safari/523.10";
const char S55068[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.0.4 Safari/523.10";
const char S55069[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.0.4 Safari/523.13.2";
const char S55070[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_3; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.0.4 Safari/523.15";
const char S55071[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5; en-us) AppleWebKit/528.7+ (KHTML, like Gecko) Version/3.0.4 Safari/523.10";
const char S55072[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-us) AppleWebKit/525.8+ (KHTML, like Gecko) Version/3.0.4 Safari/523.10.6";
const char S55073[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.0.4 Safari/523.15";
const char S55074[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.0.4 Safari/523.10";
const char S55075[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-nz; HUAWEI U8666N Build/HuaweiU8666N) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55076[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HUAWEI U8666E Build/HuaweiU8666E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55077[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HUAWEI U8666E-51 Build/HuaweiU8666E-51) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55078[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HUAWEI U8666E Build/HuaweiU8666E)";
const char S55079[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HUAWEI U8666E-51 Build/HuaweiU8666E-51)";
const char S55080[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PLT7035-PL Build/Curtis International Ltd) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55081[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PLT7035 Build/Curtis International Ltd) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55082[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PLT7035-B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55083[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; DROID X2 Build/4.4.1_274_DTN-14.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55084[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; DROID X2 Build/4.4.1_274_DTN-14.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55085[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROID X2 Build/4.4.1_274_DTN-14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55086[] = "Mozilla/5.0 (Linux; U;Android 2.2.2; en-us; DROID X2 Build/4.4.1_274_DTN-14.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55087[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; DROIDX2 Build/4.4.1_274_DTN-14.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55088[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;DROID X2 Build/4.4.1_274_DTN-14.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55089[] = "Mozilla/5.0 (Linux; U; Android2.2.2; en-us; DROID X2 Build/4.4.1_274_DTN-14.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55090[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; DROID X2 Build/4.4.1_274_DTN-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55091[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; DROID X2Build/4.4.1_274_DTN-14.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55092[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; DROID X2 Build/4.4.1_274_DTN-14.8)";
const char S55093[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT100 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S55094[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae5-00) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba";
const char S55095[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE5-00) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S55096[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE5-00.2) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S55097[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae5-00.2) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba";
const char S55098[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE5-00u) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S55099[] = "HuaweiSymantecSpider/1.0+DSE-support@huaweisymantec.com+(compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR ; http://www.huaweisymantec.com/en/IRL/spider)";
const char S55100[] = "Mozilla/5.0(Windows; U; Windows NT 5.2; rv:1.9.2) Gecko/X Firefox/6.0";
const char S55101[] = "Mozilla/5.0 (Windows NT 5.1; rv:6.0) Gecko/X Firefox/6.0 Firefox/6.0";
const char S55102[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; ru-RU; rv:5.0) Gecko/false  Firefox/6.0";
const char S55103[] = "Mozilla/5.0 (Windows NT 5.1; rv:6.0.2) Gecko/X Firefox/6.0.2,gzip(gfe)";
const char S55104[] = "Mozilla/5.0 (Series40; Nokia300/07.03; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S55105[] = "Mozilla/5.0 (Series40; Nokia300/07.57; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6";
const char S55106[] = "Mozilla/5.0 (Windows NT 4.10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36";
const char S55107[] = "Mozilla/5.0 (Windows NT 4.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S55108[] = "Mozilla/5.0 (;;; en-us; Huawei-U8651S Build/U8651SV100R001USAC85B843) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55109[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-M840 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S55110[] = "Mozilla/5.0 (X11; Linux x86_64) Ubuntu/12.04 Codebot/1.0";
const char S55111[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST27i Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55112[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST27a Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55113[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S55114[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.385 Chrome/16.0.912.77 Safari/535.7";
const char S55115[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S55116[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LGL85C Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55117[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LGL85C Build/GRJ22)";
const char S55118[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; TPA60W Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S55119[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; TPA60W Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S55120[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; TPA60W Build/HTK55D)";
const char S55121[] = "Nokia200/2.0 (11.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia200) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S55122[] = "Nokia6630/1.0 (5.03.08) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55123[] = "SonyEricssonJ20/R7CA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2";
const char S55124[] = "SonyEricssonJ20i/R7CA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2";
const char S55125[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P880 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55126[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P880g Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55127[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P880 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S55128[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT919 Build/2_290_2012) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55129[] = "Mozilla/5.0 (Linux; Android 4.1.1; A700 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S55130[] = "Mozilla/5.0 (Linux; Android 4.1.2; A700 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S55131[] = "Mozilla/5.0 (Linux; Android; 4.1.2; GT-I9100 Build/X) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1234.12 Mobile Safari/537.22 OPR/14.0.123.123";
const char S55132[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Inspire 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55133[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Inspire 4G Build/IML74K)";
const char S55134[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC-A9192 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55135[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Inspire 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55136[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Inspire 4G Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55137[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Inspire 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55138[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC-A9192/1.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55139[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_Inspire_4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55140[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_Inspire_4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55141[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Inspire 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55142[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Inspire 4G Build/IML74K)";
const char S55143[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-GB; rv:1.9.1.17) Gecko/X (like Firefox/3.x) SeaMonkey/2.0.12";
const char S55144[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.1b3pre) Gecko/X SeaMonkey/2.0a3pre";
const char S55145[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-S5367 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55146[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5367 Build/GINGERBREAD)";
const char S55147[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.8)";
const char S55148[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.7)";
const char S55149[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.6)";
const char S55150[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.4)";
const char S55151[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Sleipnir/2.9.6)";
const char S55152[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.10)";
const char S55153[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.9)";
const char S55154[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Sleipnir/2.9.9)";
const char S55155[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Sleipnir/2.9.9)";
const char S55156[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Sleipnir/2.9.10)";
const char S55157[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Sleipnir/2.9.8)";
const char S55158[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Sleipnir/2.9.7)";
const char S55159[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.11)";
const char S55160[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.12)";
const char S55161[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.3)";
const char S55162[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Sleipnir/2.9.11)";
const char S55163[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.14)";
const char S55164[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Sleipnir/2.9.14)";
const char S55165[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Sleipnir/2.9.12)";
const char S55166[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Sleipnir/2.9.14)";
const char S55167[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Sleipnir/2.9.15)";
const char S55168[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.15)";
const char S55169[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Sleipnir/2.9.4)";
const char S55170[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.16)";
const char S55171[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Sleipnir/2.9.16)";
const char S55172[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.17)";
const char S55173[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Sleipnir/2.9.14)";
const char S55174[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Sleipnir/2.9.17)";
const char S55175[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.18)";
const char S55176[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55177[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55178[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55179[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus S Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55180[] = "LGE-MN270/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S55181[] = "Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; Lenovo-P770/S100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S55182[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Lenovo P770 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55183[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Lenovo P770 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S55184[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Lenovo P770 Build/JRO03C)";
const char S55185[] = "Mozilla/5.0 (Linux; U; Android 4.1; vi-vn; Lenovo-P770_ROW/S100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S55186[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Lenovo P770 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55187[] = "Mozilla/5.0 (Windows NT 4.10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S55188[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7310 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S55189[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P7310 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S55190[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_9p017 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55191[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_jbl_253 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55192[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_ov5647 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55193[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_5647 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55194[] = "Mozilla/5.0 (Linux; Android 4.0.9; e1809c_v77_gq1008_5647 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55195[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_5640 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55196[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_jbl_5647 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55197[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_253 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55198[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_9p017 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55199[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_253_6628 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55200[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_gq1008_9p017 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55201[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_gq1008_9p017_6628 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55202[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_5640 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55203[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_ov5647_uta Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55204[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_gq1008_5647_6628 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55205[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_jdt1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55206[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_9p017_6628 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55207[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:16.1) Gecko/X Firefox/16.1 IceDragon/16.1.0.1";
const char S55208[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; M9300 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55209[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; M9300 Build/FRG83)";
const char S55210[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.32) Gecko/X Firefox/3.6.32 (Palemoon/3.6.32)";
const char S55211[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Stealth 9.7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55212[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; Stealth9.7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55213[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Stealth 9.7 Build/IMM76D)";
const char S55214[] = "NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiax2-00) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S55215[] = "SAMSUNG-GT-S3600i/S3600iJPIL1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55216[] = "SonyEricssonW810i/R4EA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55217[] = "SonyEricssonW810iv/R4EA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55218[] = "SonyEricssonW810c/R4EA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55219[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; ElocityA7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S55220[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; ElocityA7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55221[] = "Mozilla/5.0 (Series40; NokiaC2-06/07.48; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S55222[] = "Mozilla/5.0 (Series40; NokiaC2-06/07.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6";
const char S55223[] = "YelpWebView/1.0 ipad/6_1_2 YelpApp/6.5.0 (x-screen-size 1.0;)";
const char S55224[] = "YelpWebView/1.0 ipad/6_1_3 YelpApp/6.5.1 (x-screen-size 1.0;)";
const char S55225[] = "YelpWebView/1.0 ipad/6_1_3 YelpApp/6.6.1 (x-screen-size 1.0;)";
const char S55226[] = "iPad2,1/6.1.3 (10B329)";
const char S55227[] = "server-bag [iPhone OS,6.1.3,10B329,iPad3,1]";
const char S55228[] = "YelpWebView/1.0 ipad/6_1_3 YelpApp/6.7.0 (x-screen-size 2.0;)";
const char S55229[] = "YelpWebView/1.0 ipad/6_1_3 YelpApp/6.7.0 (x-screen-size 1.0;)";
const char S55230[] = "iPad2,5/6.1.3 (10B329)";
const char S55231[] = "YelpWebView/1.0 ipad/6_1_3 YelpApp/6.8.0 (x-screen-size 2.0;)";
const char S55232[] = "YelpWebView/1.0 ipad/6_1_3 YelpApp/6.8.0 (x-screen-size 1.0;)";
const char S55233[] = "YelpWebView/1.0 ipad/6_1_3 YelpApp/6.9.1 (x-screen-size 1.0;)";
const char S55234[] = "YelpWebView/1.0 ipad/6_1_3 YelpApp/6.9.1 (x-screen-size 2.0;)";
const char S55235[] = "YelpWebView/1.0 ipad/6_1_3 YelpApp/6.9.2 (x-screen-size 1.0;)";
const char S55236[] = "YelpWebView/1.0 ipad/6_1_3 YelpApp/7.0.1 (x-screen-size 1.0;)";
const char S55237[] = "YelpWebView/1.0 ipad/6_1_3 YelpApp/7.0.1 (x-screen-size 2.0;)";
const char S55238[] = "YelpWebView/1.0 ipad/6_1_3 YelpApp/7.1.0 (x-screen-size 2.0;)";
const char S55239[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55240[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S55241[] = "Mozilla/5.0 (Linux; Android 4.2.2; One X Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55242[] = "Mozilla/5.0 (Linux; Android 4.2.1; HTC One X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55243[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One X Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55244[] = "Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_5) AppleWebKit (KHTML, like Gecko) Chrome/7";
const char S55245[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S55246[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S55247[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S55248[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S55249[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S55250[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_0; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S55251[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S55252[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.7 (KHTML	 like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S55253[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; TECNO N3 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55254[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; TECNO N3S Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55255[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; TECNO N3 Build/master)";
const char S55256[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S55257[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S55258[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SCH-i509 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S55259[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5360L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S55260[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5360B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S55261[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5360 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S55262[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; enspert_e201 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55263[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; enspert_e201 Build/FRF91)";
const char S55264[] = "Mozilla/5.0 ( Windows; U; Windows NT4.0; FireFox )";
const char S55265[] = "Mozilla/5.0 (Windows; U; WinNT4.0; en-US; rv:0.9.5) Gecko/X";
const char S55266[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Safari/535.19 Silk-Accelerated=true";
const char S55267[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST26i Build/11.2.A.0.21) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55268[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST26i Build/11.2.A.0.21) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S55269[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55270[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55271[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715a Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55272[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55273[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One V Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55274[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One V Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55275[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; HTC Magic Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55276[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Magic Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55277[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Magic Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55278[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; HTC Magic Build/FRG83D)";
const char S55279[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; HTC Magic Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S55280[] = "Mozilla/5.0 (Series40; Nokia201/11.21; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.11";
const char S55281[] = "SAMSUNG-GT-C3212i/C3212IDDJF2 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55282[] = "SAMSUNG-GT-C3212/C3212DDIL1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55283[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 9300+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55284[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; 9300+ Build/IMM76D)";
const char S55285[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; 9300+ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55286[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; zh-tw; 9300+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55287[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L900 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55288[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7100 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55289[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7105 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S55290[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Note II Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55291[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7105 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55292[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I317 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55293[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7100 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S55294[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T889 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55295[] = "Nokia306/2.0 (03.63) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S55296[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55297[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S55298[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T989 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55299[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989D Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55300[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T989 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S55301[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989D Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S55302[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MediaPad 10 FHD Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55303[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MediaPad 10 FHD Build/HuaweiMediaPad)";
const char S55304[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P5110 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S55305[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P5110 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S55306[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P5100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S55307[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P5100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S55308[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T779 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S55309[] = "SAMSUNG-GT-S3653W/S3653WJPJC4 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S55310[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaE71) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S55311[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAE71x) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S55312[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiae71-1) U2/1.0.0 UCBrowser/8.8.0.227 U2/1.0.0 Mobile";
const char S55313[] = "NokiaE71/UCWEB8.8.0.212/28/800";
const char S55314[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:23.0) Gecko/X Firefox/23.0";
const char S55315[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E980 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55316[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT890 Build/8.7.1I-110_IFW-39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S55317[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT890 Build/8.7.1I-110_IFW-39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22";
const char S55318[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S55319[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S55320[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S55321[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S55322[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S55323[] = "Nokia5200/2.0 (05.53) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55324[] = "Nokia5200/2.0 (03.93) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia5200";
const char S55325[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55326[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One 801e Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55327[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC_PN071 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55328[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S55329[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One 801e Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S55330[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC_PN071 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S55331[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One dual sim Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55332[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; V370 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55333[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; V370 Build/JOP40D)";
const char S55334[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; nl-nl; V370 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55335[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; V370 Build/JDQ39)";
const char S55336[] = "Mozilla/5.0 (Linux; Android 4.0.3; A200 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S55337[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID_Serials Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55338[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID_Serials Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55339[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Tablet PC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55340[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; tablet Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55341[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Tablet PC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55342[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; M71 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55343[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 1688-CJ-C30Q8-31S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55344[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Tablet Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55345[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; MID970 Build/IML74K)";
const char S55346[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Tablet 7.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55347[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MID900SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55348[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Tablet 7'' Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55349[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Android Tablet 7'' Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55350[] = "JUC (Linux; U; 4.0.4; zh-cn; GT-I9100G; 480*800) UCWEB7.9.3.103/139/32092";
const char S55351[] = "JUC (Linux; U; 2.3.4; zh-cn; GT-I9108; 480*800) UCWEB7.9.3.103/139/800";
const char S55352[] = "JUC (Linux; U; 4.0.3; zh-cn; GT-I9100; 480*800) UCWEB7.9.0.94/139/444";
const char S55353[] = "JUC (Linux; U; 2.3.5; zh-cn; GT-I9100; 480*800) UCWEB7.9.0.94/139/444";
const char S55354[] = "JUC (Linux; U; 2.3.4; zh-cn; GT-I9100G; 480*800) UCWEB7.9.0.94/139/444";
const char S55355[] = "JUC (Linux; U; 2.3.5; zh-cn; GT-I9100T; 480*800) UCWEB7.9.0.94/139/444";
const char S55356[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S55357[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S55358[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S55359[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S55360[] = "Mozilla/5.0 (Linux; Android 4.1.1; ME172V Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55361[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9082 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S55362[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S100 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55363[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; S100 Build/IMM76I)";
const char S55364[] = "NokiaN72/2.0617.1.0.3 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55365[] = "NokiaN72/2.0617.1.0.3 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1, NokiaN72/2.0617.1.0.3 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55366[] = "NokiaN72/SymbianOS/8.0 Series60/2.6";
const char S55367[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; ADR6225 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55368[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-uk; Xolo_X900 Build/AZ510_ICS_02.19I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55369[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55370[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-uk; Xolo_X900 Build/AZ510_ICS_02.24I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55371[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-uk; Xolo_X900 Build/AZ510_ICS_02.31I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55372[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-uk; Xolo_X900 Build/AZ510_ICS_02.36I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55373[] = "Mozilla/5.0 (compatible; Teleca Q7; Brew 3.1.5; U; en) 320X240 LGE VX9200";
const char S55374[] = "Mozilla/5.0 (Linux; Android 4.0.4; C5170 Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S55375[] = "Mozilla/5.0 (Linux; Android 4.0.4; Hydro Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S55376[] = "Mozilla/5.0 (Linux; Android 4.0.4; V8000_USA_Cricket Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S55377[] = "Mozilla/5.0 (Linux; Android 4.0.4; N8000_USA_Cricket Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S55378[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.29 Safari/537.36 OPR/15.0.1147.24 (Edition Next)";
const char S55379[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153";
const char S55380[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153";
const char S55381[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S55382[] = "Mozilla/5.0 (Linux; Android 4.0.3; LT22i Build/6.2.A.0.400) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S55383[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xperia P Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S55384[] = "KCI-S2100-VMUB/ACP1000 UP.Browser/7.2.7.2.564 (GUI) MMP/2.0";
const char S55385[] = "KCI-S2100/ACM1000 UP.Browser/7.2.7.2.565 (GUI) MMP/2.0";
const char S55386[] = "KCI-S2100/ACQ1007 UP.Browser/7.2.7.2.565 (GUI) MMP/2.0";
const char S55387[] = "KCI-S2100/ACM1002 UP.Browser/7.2.7.2.565 (GUI) MMP/2.0";
const char S55388[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Dell Streak Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/streak.407.0";
const char S55389[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Dell Streak Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S55390[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; Dell Streak Build/GRI40)";
const char S55391[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Dell Streak Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55392[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Dell Streak Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55393[] = "sprd-GT-A7100/1.0 Linux/2.6.35.7 Android/4.0.4 Release/01.14.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S55394[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.1.17) Gecko/X Firefox/3.5.17";
const char S55395[] = "MQQBrowser/43 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3";
const char S55396[] = "MQQBrowser/43 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3";
const char S55397[] = "MQQBrowser/43 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3";
const char S55398[] = "MQQBrowser/43 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Safari/7534.48.3";
const char S55399[] = "MQQBrowser/43 Mozilla/5.0 (iPhone 4; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Safari/7534.48.3";
const char S55400[] = "MQQBrowser/43 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 Safari/7534.48.3";
const char S55401[] = "MQQBrowser/43 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145 Safari/7534.48.3";
const char S55402[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S55403[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S55404[] = "Mozilla/5.0 (Series40; NokiaC2-03/07.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S55405[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CLP281A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55406[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CLP281X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55407[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GOCLEVER Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55408[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GOCLEVER Build/JRO03H)";
const char S55409[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.9.0 Safari/534.34";
const char S55410[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) CasperJS/1.0.2+Phantomjs/1.9.0 Safari/534.34";
const char S55411[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.9.1 Safari/534.34";
const char S55412[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE N9120 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S55413[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Avant Browser)";
const char S55414[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Avant Browser)";
const char S55415[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; SV1; Avant Browser)";
const char S55416[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; SV1; Avant Browser)";
const char S55417[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Avant Browser; Avant Browser)";
const char S55418[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Avant Browser; Avant Browser)";
const char S55419[] = "MQQBrowser/42 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3";
const char S55420[] = "MQQBrowser/42 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3";
const char S55421[] = "MQQBrowser/42 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Safari/7534.48.3";
const char S55422[] = "MQQBrowser/42 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 Safari/7534.48.3";
const char S55423[] = "MQQBrowser/421 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3";
const char S55424[] = "MQQBrowser/42 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145 Safari/7534.48.3";
const char S55425[] = "MQQBrowser/421 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3";
const char S55426[] = "MQQBrowser/42 Mozilla/5.0 (iPhone 4; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Safari/7534.48.3";
const char S55427[] = "MQQBrowser/421 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3";
const char S55428[] = "MQQBrowser/421 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 Safari/7534.48.3";
const char S55429[] = "MQQBrowser/421 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Safari/7534.48.3";
const char S55430[] = "MQQBrowser/421 Mozilla/5.0 (iPhone 4; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Safari/7534.48.3";
const char S55431[] = "MQQBrowser/421 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145 Safari/7534.48.3";
const char S55432[] = "MQQBrowser/42 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3";
const char S55433[] = "Mozilla/5.0 (X11; U; Linux i686; fr; rv:1.8.1.12) Gecko/X Fedora/2.0.0.12-1.fc8 Firefox/2.0.0.12";
const char S55434[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Acer Liquid Metal Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55435[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Liquid Metal Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55436[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Acer Liquid Metal Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55437[] = "Mozilla/5.0 (Linux; U; Android 2.2; it-it; Liquid Metal Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Acer_S120/Acer_LiquidMetal_1.000.14_EMEA_VFIT";
const char S55438[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; Acer Liquid Metal Build/FRG83)";
const char S55439[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; Liquid Metal Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55440[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; Liquid Metal Build/MASTER)";
const char S55441[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A5376e Safari/8536.25 (compatible; Googlebot-Mobile/2.1; +http://www.google.com/bot.html)";
const char S55442[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/537.4 (KHTML, like Gecko; Google Page Speed Insights) Version/6.0 Mobile/10A525 Safari/8536.25 GoogleBot/2.1";
const char S55443[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I717 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55444[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I717 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55445[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I717 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55446[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21i Build/11.0.A.4.22) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55447[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21a Build/11.0.A.0.16) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55448[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21i2 Build/11.0.A.6.5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55449[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21a2 Build/11.0.A.1.8) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55450[] = "Mozilla/5.0 (Series40; NokiaX2-02/11.79; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S55451[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55452[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55453[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Celkon_A101 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55454[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; A101 Build/IML74K)";
const char S55455[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Mobile Safari/535.19 Silk-Accelerated=true";
const char S55456[] = "Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320) BlackBerry8330/4.3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/104";
const char S55457[] = "BlackBerry8330/4.3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105";
const char S55458[] = "Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320) BlackBerry8330/4.3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/126";
const char S55459[] = "Mozilla/5.0 (Linux; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55460[] = "Mozilla/5.0 (Linux; Android 4.0.4; ME865 Build/6.7.2_GC-220-EDS-3) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55461[] = "Mozilla/5.0 (Linux; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S55462[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR6425LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S55463[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6425LVW Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S55464[] = "LG/KU990-Orange/v10ci Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55465[] = "LG/KU990/v10a Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55466[] = "LG-KU990/v10a Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55467[] = "Vodafone/1.0/LG-KU990/V10a Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55468[] = "Vodafone/1.0/LG-KU990/V10i Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55469[] = "Vodafone/1.0/LG-KU990/V10h Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55470[] = "Vodafone/1.0/LG-KU990/V10c Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55471[] = "Vodafone/1.0/LG-KU990/v10b Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55472[] = "Vodafone/1.0/LG-KU990/V10j Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55473[] = "Vodafone/1.0/LG-KU990/V10f Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55474[] = "Vodafone/1.0/LG-KU990/V10g Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55475[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Avant Browser)";
const char S55476[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Avant Browser)";
const char S55477[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Avant Browser; SV1)";
const char S55478[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Avant Browser; SV1)";
const char S55479[] = "Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Avant Browser; Avant Browser)";
const char S55480[] = "Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Avant Browser; Avant Browser)";
const char S55481[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I897 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55482[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I897 Build/JRO03H; CyanogenMod-helly_bean_cappy) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55483[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I897 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55484[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I897 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55485[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293";
const char S55486[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S55487[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S55488[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293";
const char S55489[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S55490[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S55491[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/6531.22.7 iLunascape/4";
const char S55492[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/9B206 Safari/6531.22.7";
const char S55493[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/6531.22.7 iLunascape/3165";
const char S55494[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/6531.22.7 iLunascape/4";
const char S55495[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; vi-vn) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/9B176 Safari/6531.22.7";
const char S55496[] = "Mozilla/5.0 (Windows NT 5.0; rv:15.0) Gecko/X Firefox/15.0.1";
const char S55497[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; U8666-1 Build/HuaweiU8666) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55498[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-by; MTC Fit Build/HuaweiU8666) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55499[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; U8666-51 Build/HuaweiU8666-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55500[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; U8666-51 Build/HuaweiU8666) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55501[] = "Opera/9.7 (Windows NT 5.1; HTC_HD2/1.0; U; en)";
const char S55502[] = "HTC_HD_T8585 Opera/9.7 (Windows NT 5.1; U; en)";
const char S55503[] = "HTC_HD2_T9193 Opera/9.7 (Windows NT 5.1; U; en) V1.49.841.1 (71528)";
const char S55504[] = "HTC_HD2_T9193 Opera/9.7 (Windows NT 5.1; U; en) V1.72.841.0 (82124)";
const char S55505[] = "PPC; 480x800; HTC_HD2_T8585-Orange; OpVer 114.143.2.731 Opera/9.7 (Windows NT 5.1; U;fr)";
const char S55506[] = "HTC_Touch_HD2_T8585 Opera/9.70 (Windows NT 5.1; U; en)";
const char S55507[] = "Vodafone/1.0/HTC_HD2/1.49.164.2 (71528) Opera/9.7 (Windows NT 5.1; U; es-ES)";
const char S55508[] = "Vodafone/1.0/HTC_HD2/1.72.162.0 (82124) Opera/9.7 (Windows NT 5.1; U; de)";
const char S55509[] = "Vodafone/1.0/HTC_HD2/1.72.172.0 (82124) Opera/9.7 (Windows NT 5.1; U; nl)";
const char S55510[] = "Vodafone/1.0/HTC_HD2/1.72.161.3 (82124) Opera/9.7 (Windows NT 5.1; U; en)";
const char S55511[] = "Vodafone/1.0/HTC_HD2/1.44.163.3 (70494) Opera/9.7 (Windows NT 5.1; U; fr)";
const char S55512[] = "Vodafone/1.0/HTC_HD2/3.14.162.5 (04666) Opera/9.7 (Windows NT 5.1; U; de)";
const char S55513[] = "Vodafone/1.0/HTC_HD2/3.14.161.3 (04666) Opera/9.7 (Windows NT 5.1; U; en)";
const char S55514[] = "Vodafone/1.0/HTC_HD2/3.14.163.3 (04666) Opera/9.7 (Windows NT 5.1; U; fr)";
const char S55515[] = "Vodafone/1.0/HTC_HD2/3.14.172.3 (04666) Opera/9.7 (Windows NT 5.1; U; nl)";
const char S55516[] = "Vodafone/1.0/HTC_HD2/1.72.164.4 (82124) Opera/9.7 (Windows NT 5.1; U; es-ES)";
const char S55517[] = "Vodafone/1.0/HTC_HD2/1.44.172.7 (70494) Opera/9.7 (Windows NT 5.1; U; nl)";
const char S55518[] = "Vodafone/1.0/HTC_HD2/1.44.162.6 (70494) Opera/9.7 (Windows NT 5.1; U; de)";
const char S55519[] = "Vodafone/1.0/HTC_HD2/3.14.164.4 (04666) Opera/9.7 (Windows NT 5.1; U; es-ES)";
const char S55520[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8730 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S55521[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8730T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S55522[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 480x800; HTC_HD2_T8585-Orange; OpVer 114.143.2.731";
const char S55523[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 480x800; HTC_HD2_T8585-Orange; OpVer 156.314.2.732";
const char S55524[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 480x800; HTC_HD2_T8585-Orange; OpVer 114.172.1.612";
const char S55525[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; HTC_HD2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55526[] = "HTC_Leo Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S55527[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/1.49.164.2 (71528)";
const char S55528[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/3.14.161.3 (04666)";
const char S55529[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/3.14.163.3 (04666)";
const char S55530[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 480x800; HTC_HD2_T8585-Orange; OpVer 114.172.2.733";
const char S55531[] = "HD2_T8585 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S55532[] = "HTC_HD2_T8585/480x800 4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S55533[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) HTC_HD2_T8585/480x800";
const char S55534[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/1.72.172.0 (82124)";
const char S55535[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/1.44.163.3 (70494)";
const char S55536[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/1.44.172.7 (70494)";
const char S55537[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/1.72.161.3 (82124)";
const char S55538[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/1.72.162.0 (82124)";
const char S55539[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/1.72.164.4 (82124)";
const char S55540[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/3.14.172.3 (04666)";
const char S55541[] = "HTC_HD2_T9193 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) V1.49.841.1 (71528)";
const char S55542[] = "HTC_HD2_T9193 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) V1.72.841.0 (82124)";
const char S55543[] = "HTC_HD2_Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S55544[] = "HTC_HD2_T8585 Mozilla\\4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S55545[] = "HTC_HD2_T9193 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S55546[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/3.14.162.5 (04666)";
const char S55547[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/3.14.164.4 (04666)";
const char S55548[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 480x800; HTC_HD2_T8585-Orange; OpVer 156.314.1.612";
const char S55549[] = "HTC_HD2_T8585/1.64.621.6 (77873) Mozilla/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S55550[] = "HTC_HD2_T8585 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S55551[] = "Mozilla/5.0 (Unknown; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.5.0 Safari/534.34";
const char S55552[] = "Mozilla/5.0 (Unknown; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.5.0 Safari/534.34";
const char S55553[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.5.0 (development) Safari/534.34";
const char S55554[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; sdk Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S55555[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-us; rk2818sdk Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S55556[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; sdk Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S55557[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; google_sdk Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S55558[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; sdk Build/ECLAIR)";
const char S55559[] = "Dalvik/1.1.0 (Linux; U; Android 2.1; google_sdk Build/ERD79)";
const char S55560[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Vision Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55561[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Vision Build/IMM76I; CyanogenMod-Audacity Beta 4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55562[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Vision Build/IMM76L; CyanogenMod-mimicry) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55563[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-us; HTC Vision Build/IMM76I; CyanogenMod-QuadraSense 1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55564[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100 (Edition Next)";
const char S55565[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153";
const char S55566[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153";
const char S55567[] = "SAMSUNG-GT-E2232/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S55568[] = "SAMSUNG-GT-E2232/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55569[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Triumph Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55570[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Triumph Build/GRJ22; CM7 Illuminatus v0.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55571[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE5-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/355/UCWEB Mobile";
const char S55572[] = "Mozilla/5.0 (Java; U; en-us; nokiae5-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/84/352/UCWEB Mobile  3gpp-gba";
const char S55573[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE5-00.2) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile";
const char S55574[] = "SAMSUNG-SGH-i900Orange/BVIB2 Opera 9.5";
const char S55575[] = "SAMSUNG-SGH-i900V/JBHH1 Opera 9.5";
const char S55576[] = "SAMSUNG-SGH-i900/1.0 Opera 9.5";
const char S55577[] = "Mozilla/5.0 (SAMSUNG-SGH-i900/1.0 Opera 9.5; U; (null); rv:1.8.1) Gecko/20061208 Firefox/2.0.0 Opera 9.5";
const char S55578[] = "Mozilla/5.0 (masking-agent; rv:13.0) Gecko/X Firefox/13.0";
const char S55579[] = "Mozilla/5.0 (masking-agent; rv:13.0) Gecko/X Firefox/13.0.1";
const char S55580[] = "Mozilla/5.0 (SaferSurf rv:13.0) Gecko/X Firefox/13.0";
const char S55581[] = "Mozilla/5.0 (Series40; NokiaX2-01/08.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S55582[] = "Mozilla/5.0 (Series40; NokiaX2-01/07.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6";
const char S55583[] = "Mozilla/5.0 (Series40; NokiaX2-01.1/07.22; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.62.10";
const char S55584[] = "SonyEricssonJ105i/R1HA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.5";
const char S55585[] = "SonyEricssonJ105a/R1HA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.5";
const char S55586[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26i Build/6.1.A.2.55) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55587[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5";
const char S55588[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5";
const char S55589[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5";
const char S55590[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; HUAWEI Y300-0151 Build/HuaweiY300-0151) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55591[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HUAWEI Y300-0151 Build/HuaweiY300-0151)";
const char S55592[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; T7575)";
const char S55593[] = "Mozilla/5.0 (Linux; U; Android 2.2.1-rc8; en-us; T301 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55594[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T301 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55595[] = "Mozilla/5.0 (Linux; U; Android 2.2.1-rc7; en-us; T301 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55596[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; T301 Build/FRG83)";
const char S55597[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8000 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S55598[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8005 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S55599[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55600[] = "Mozilla/5.0 (Linux; Android 4.0.3; LT22i Build/6.2.A.0.400) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55601[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; AMID-9743G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55602[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; AMID-9743G Build/JZO54K)";
const char S55603[] = "Opera/9.80 (X11; Linux i686) Presto/2.12.388 Version/12.13";
const char S55604[] = "Mozilla/5.0 (X11; Linux x86_64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.13";
const char S55605[] = "Nokia200/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia200) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S55606[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; SGH-T869 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S55607[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; SGH-T869 Build/HTJ85B)";
const char S55608[] = "NokiaC5-03/UCWEB8.8.0.212/50/999";
const char S55609[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaC5-03) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S55610[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; SO-01B Build/2.0.B.0.138) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S55611[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; SO-01B Build/2.0.1.B.0.19) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S55612[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; SO-01B Build/2.0.2.B.0.34) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S55613[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; SO-01B Build/2.0.2.B.0.29) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S55614[] = "Mozilla/5.0 (Linux; U; Android 2.1; ja_jp; SO-01B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)";
const char S55615[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; SonyEricssonSO-01B Build/2.0.2.B.0.37) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S55616[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; SO-01B Build/2.0.2.B.0.37) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S55617[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SO-01B Build/2.0.2.B.0.37)";
const char S55618[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; IM-A800S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55619[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; IM-A800S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55620[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; IM-A800S Build/GRJ90)";
const char S55621[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; IM-A800S Build/V5.4 (TV) by atulaphan) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55622[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; IM-A800S Build/VEGAVIET.COM ROM GB V2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55623[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17";
const char S55624[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17";
const char S55625[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SGH-M919N Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S55626[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 10 Build/1.1.5 X-15:30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55627[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 10 Build/1.1.7 X-14:18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55628[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 10 Build/1.1.4 X-13:45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55629[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 10 Build/1.1.8 X-15:22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55630[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 10 Build/1.1.8-1015 X-16:30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55631[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 10 Build/1.1.10-1015 X-18:00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55632[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 10 Build/1.1.11-1015 X-10:30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55633[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 10 3G Build/1.1.9-1015 X-15:48) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55634[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 10 3G Build/1.1.10-1015 X-18:00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55635[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 10 3G Build/1.1.11-1015 X-16:17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55636[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS840 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S55637[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS840PP Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S55638[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; A101 Build/HTJ85) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S55639[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; A101 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S55640[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; A101 Build/HTK55D)";
const char S55641[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; PantechP8000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55642[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; PantechP8000 Build/GRK39F)";
const char S55643[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PantechP8000 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55644[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:15.4) Gecko/X Firefox/15.4.1 PaleMoon/15.4.1";
const char S55645[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:15.4) Gecko/X Firefox/15.4.1-x64 PaleMoon/15.4.1-x64";
const char S55646[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbla668_fwvga Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55647[] = "Mozilla/5.0 (Linux; Android 4.1.2; cm_tenderloin Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S55648[] = "Mozilla/5.0 (Linux; Android 4.1.2; skz_tenderloin Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S55649[] = "Mozilla/5.0 (Linux; Android 4.1.1; Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S55650[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbl1 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55651[] = "Mozilla/5.0 (Linux; Android 4.1.1; V1013 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S55652[] = "Mozilla/5.0 (Linux; Android 4.1.2; Motorola Galaxy s3 Build/9.8.2O-72_VZW-8) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55653[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1911_v77_gq2000_a41_6628_v20 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55654[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1909l_v77_gq1008_a41_v20 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55655[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1908_v77_jbl1_9p017_6628 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55656[] = "Mozilla/5.0 (Linux; Android 4.1.1; Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55657[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbla668_9p017_fwvga Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55658[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_gq2002tc Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55659[] = "Mozilla/5.0 (Linux; Android 4.1.2; e2001_v89_gq2008s Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55660[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1920_v77_zlh_9p017_asx Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55661[] = "Mozilla/5.0 (Linux; Android 4.1.5; iPhone5 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55662[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1921_v77_cxq1 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55663[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1920_v77_zlh_fwvga Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55664[] = "Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jdt1_9p017_fwvga Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S55665[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:8.0.1) Gecko/X Firefox/8.0.1";
const char S55666[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; M040 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55667[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; M040 Build/JRO03H) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S55668[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; M040 Build/JRO03H)";
const char S55669[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; M040 Build/JRO03H) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.4 Version/4.0 Mobile Safari/530.17 baidubrowser/023_1.8.4.2_diordna_0821_008/uzieM_61_1.1.4_040M/Xb/35BC577D8B1D44338D9CE3C5EE1466B2%7CX/1";
const char S55670[] = "Mozilla/5.0 (Linux; Android 4.0.4; Torque Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55671[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ALCATEL_one_touch_990 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55672[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; ALCATEL ONE TOUCH 990 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55673[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-cl; ALCATEL_one_touch_990A Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55674[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; ALCATEL one touch 990 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55675[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ALCATEL_one_touch_990S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55676[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; TheMOONSTONE ALCATEL OT-990 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55677[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-ec; ALCATEL ONE TOUCH 990A Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55678[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-mn; ALCATEL one touch 990C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55679[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; SlimBrowser/7.00.035)";
const char S55680[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; SlimBrowser/7.00.035)";
const char S55681[] = "Mozilla/5.0 (X11; U; OpenBSD i386; en-US; rv:1.8.1.14) Gecko/X Firefox/2.0.0.14";
const char S55682[] = "Mozilla/5.0 (X11; U; OpenBSD ppc; en-US; rv:1.8.1.9) Gecko/X BonEcho/2.0.0.9";
const char S55683[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; N9101 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55684[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; N9101 Build/JZO54K)";
const char S55685[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; ISW11SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55686[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ISW11SC Build/IMM76D)";
const char S55687[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-; ISW11SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55688[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; Avant Browser)";
const char S55689[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; Avant Browser)";
const char S55690[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; Avant Browser; Avant Browser)";
const char S55691[] = "Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; SV1; Avant Browser)";
const char S55692[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; SV1; Avant Browser)";
const char S55693[] = "Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; Avant Browser; Avant Browser)";
const char S55694[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T959 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55695[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T959 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55696[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T959 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55697[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T959 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55698[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T959 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55699[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8) Presto/2.12.388 Version/12.15";
const char S55700[] = "Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia305) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S55701[] = "Mozilla/5.0 (Series40; NokiaC2-05/08.69; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S55702[] = "Mozilla/5.0 (Series40; NokiaC2-05.1/08.69; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S55703[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S55704[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S55705[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3492; U; en) Presto/2.8.119 Version/11.10";
const char S55706[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3492; U; en) Presto/2.8.119 Version/11.10";
const char S55707[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; H7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55708[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; H7100 Build/JRO03C)";
const char S55709[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; SonyEricssonWT19a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55710[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; WT19a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55711[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-ar; WT19a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55712[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; WT19a Build/4.1.B.0.587)";
const char S55713[] = "MOT-WX430/00.72 UP.Browser/7.2.7.5.708 (GUI) MMP/2.0";
const char S55714[] = "8900b/MR2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S55715[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Touch 3G T3232";
const char S55716[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) 480x640; XV6950; Window Mobile 6.1 Professional;";
const char S55717[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) S21HT";
const char S55718[] = "A3100 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S55719[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) S22HT";
const char S55720[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) e811_CU/1.0 CEOS/5.2.20269 Release/11.24.2009 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55721[] = "default (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) default";
const char S55722[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Traveler 117;VGA;480x640";
const char S55723[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) AnyDATA ASP-210C";
const char S55724[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Sprint:PPC6800";
const char S55725[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Sprint PPC6850SP";
const char S55726[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) WS020SH";
const char S55727[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) MOVISTAR/1.0/e810/VEN_TM_SE_P180A2V1.0.1 IE Mobile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55728[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Sprint:SPH-i325 320x240";
const char S55729[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S55730[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) 320x320; XV6900; Window Mobile 6.1 Professional;";
const char S55731[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Glacier Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55732[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Glacier Build/GRI40; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55733[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Glacier Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55734[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.4 Safari/534.34";
const char S55735[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.2 Safari/534.34";
const char S55736[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaX6-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S55737[] = "NokiaX6-00/UCWEB8.7.0.187/50/999";
const char S55738[] = "NokiaX6-00UCBrowser/8.7.1.234/50/444/UCWEB";
const char S55739[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; C6506 Build/10.3.A.0.423)";
const char S55740[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-ca; SonyC6506 Build/10.3.A.0.423) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55741[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-ca; C6506 Build/10.3.A.0.423) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55742[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; C6506 Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55743[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; C6506 Build/10.3.1.A.2.67) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55744[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7A400";
const char S55745[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko)";
const char S55746[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko)";
const char S55747[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7A341";
const char S55748[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; Android edition by sfr STARADDICT Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55749[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; Android edition by sfr STARADDICT Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55750[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; Smartphone Android by SFR STARADDICT II Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55751[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; Microsoft; Virtual)";
const char S55752[] = "Mozilla/5.0 (Windows NT 6.0; rv:16.0) Gecko/X Firefox/16.0 SeaMonkey/2.13.2";
const char S55753[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; VS870 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55754[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; VS870 4G Build/JZO54K)";
const char S55755[] = "Mozilla/5.0 (Linux; Android 4.0.4; LGL86C Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S55756[] = "NetFront/3.5.1 (BREW 3.1.5; U; en-us; LG; NetFront/3.5.1/WAP) Sprint LN240 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S55757[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MID8024 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55758[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; MID8024 Build/FRF91)";
const char S55759[] = "Logos/0.9.2 (Linux; U; Android 2.2; en_US; MID8024 Build/MID Serials 2.2 FRF91 X.X)";
const char S55760[] = "Logos/0.9.4 (Linux; U; Android 2.2; en_US; MID8024 Build/MID Serilas 2.2 FRF91 X.X)";
const char S55761[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; SAMSUNG-SGH-I896 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S55762[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-; SAMSUNG-SGH-I896 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S55763[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SAMSUNG-SGH-I896 Build/ECLAIR)";
const char S55764[] = "Mozilla/5.0 (X11; CrOS i686 1193.65.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.44 Safari/535.7";
const char S55765[] = "Mozilla/5.0 (X11; CrOS i686 1193.15.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.10 Safari/535.7";
const char S55766[] = "Mozilla/5.0 (X11; CrOS i686 1193.105.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S55767[] = "Mozilla/5.0 (X11; CrOS i686 1193.117.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.66 Safari/535.7";
const char S55768[] = "Mozilla/5.0 (X11; CrOS i686 1193.158.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7";
const char S55769[] = "Mozilla/5.0 (X11; CrOS i686 1193.194.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S55770[] = "Mozilla/5.0 (X11; CrOS i686 1193.45.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.32 Safari/535.7";
const char S55771[] = "Mozilla/5.0 (X11; CrOS i686 1144.0.2011) AppleWebKit/535.6 (KHTML, like Gecko) Chrome/16.0.903.0 Safari/535.6";
const char S55772[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36";
const char S55773[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36";
const char S55774[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5753E/S5753EDDJK6; U; Bada/1.0; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 OPN-B";
const char S55775[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; IM-A810S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55776[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; IM-A810K Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55777[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; IM-A810S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55778[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; IM-A810S Build/GRJ90)";
const char S55779[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; IM-A810K Build/GRJ90)";
const char S55780[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-; IM-A810K Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55781[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; IM-A810K Build/PHIEN BAN GALAXY S SKY A810K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55782[] = "Mozilla/5.0 (Linux; Android 4.1.1; Amazon Kindle Fire Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S55783[] = "Mozilla/5.0 (Linux; Android 4.1.1; Amazon Kindle Fire Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55784[] = "Mozilla/5.0 (Linux; Android 4.1.2; Amazon Kindle Fire Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S55785[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC first Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S55786[] = "Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Safari/535.19 Silk-Accelerated=true";
const char S55787[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; u8800 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55788[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; U8800 Build/HuaweiU8800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55789[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Huawei u8800 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55790[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; U8800 Build/HuaweiU8800)";
const char S55791[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; U8800 Build/HuaweiU8800) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S55792[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; U8800pro Build/HuaweiU8800pro) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55793[] = "Instagram 3.4.2 (iPad2,1; iPhone OS 6.1; en_US; en) AppleWebKit/420+";
const char S55794[] = "Instagram 3.4.2 (iPad2,1; iPhone OS 6.1.2; en_US; en) AppleWebKit/420+";
const char S55795[] = "Instagram 3.4.2 (iPad2,1; iPhone OS 6.1.3; en_US; en) AppleWebKit/420+";
const char S55796[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5210 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55797[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Full Android on S5PV210 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55798[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; Full Android on S5PV210 Build/HONEYCOMB) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55799[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Full Android on S5PV210 Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55800[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.1; Full Android on S5PV210 Build/GINGERBREAD)";
const char S55801[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.1.1.0 Chrome/16.0.912.63 Safari/535.7";
const char S55802[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.1.1.0 Chrome/16.0.912.63 Safari/535.7";
const char S55803[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT925 Build/7.7.1Q-144_VQL_S3-58) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55804[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR HD Build/7.7.1Q-144_VQW_S334-25) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55805[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7100 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S55806[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-N719 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S55807[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N7100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S55808[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7105 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S55809[] = "IUC(U;iOS 5.0.1;Zh-cn;320*480;)/UCWEB8.6.0.180/42/800";
const char S55810[] = "IUC(U;iOS 5.1.1;Zh-cn;320*480;)/UCWEB8.6.0.180/42/800";
const char S55811[] = "IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB8.6.0.180/41/800";
const char S55812[] = "IUC(U;iOS 4.1;Zh-cn;320*480;)/UCWEB8.6.0.180/41/800";
const char S55813[] = "Mozilla/5.0 (Java; U; en-us; j2me) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/84/352/UCWEB Mobile";
const char S55814[] = "Mozilla/5.0 (Java; U; en-us; maui) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S55815[] = "Mozilla/5.0 (Java; U; en-us; mstar) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S55816[] = "Mozilla/5.0 (Java; U; en-us; spice) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S55817[] = "Mozilla/5.0 (Java; U; en-us; generic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile";
const char S55818[] = "Mozilla/5.0 (Java; U; en-us; mstar platform) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S55819[] = "Mozilla/5.0 (Java; U; en-us; opera) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S55820[] = "Mozilla/5.0 (Java; U; en-us; sunmicro) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S55821[] = "Mozilla/5.0 (Java; U; en-us; maui browser) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S55822[] = "Mozilla/5.0 (Java; U; vi; maui) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S55823[] = "Mozilla/5.0 (Java; U; en-us; karbonn) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S55824[] = "UCWEB/2.0(Java; U; MIDP-2.0; zh-cn; maui) U2/1.0.0 UCBrowser/8.6.1.237 U2/1.0.0 Mobile";
const char S55825[] = "Mozilla/5.0 (Java; U; en-us; mozilla) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/405/UCWEB Mobile";
const char S55826[] = "Mozilla/5.0 (Linux; U; en-gb) AppleWebKit/418.9.1 (KHTML, like Gecko) SiteCon/8.10.9";
const char S55827[] = "Feedly/1.0 (Linux x86_64; rv:1.0)";
const char S55828[] = "Needle/0.4.8 (Node.js v0.8.22; linux x64)";
const char S55829[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; Vodafone SmartTab II 10 Build/SmartTabII10_A403_068_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55830[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; Vodafone SmartTab II 10 Build/SmartTabII10_A403_V72_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55831[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; Vodafone SmartTab II 10 Build/SmartTabII10_A403_K85_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55832[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; SmartTabII10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55833[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; Vodafone SmartTab II 10 Build/SmartTabII10_A403_K83_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55834[] = "UCWEB/2.0(BlackBerry; U; 9.6.0.51; en-us; 9360/9.6.0.51) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S55835[] = "UCWEB/2.0(BlackBerry; U; 8.0.0.388; en-us; 9360/8.0.0.388) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S55836[] = "UCWEB/2.0(BlackBerry; U; 8.0.0.499; en-us; 9360/8.0.0.499) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S55837[] = "UCWEB/2.0(BlackBerry; U; 9.6.0.111; en-us; 9360/9.6.0.111) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S55838[] = "UCWEB/2.0(BlackBerry; U; 8.0.0.510; en-us; 9360/8.0.0.510) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S55839[] = "UCWEB/2.0(BlackBerry; U; 9.6.0.80; en-us; 9360/9.6.0.80) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S55840[] = "UCWEB/2.0(BlackBerry; U; 8.0.0.359; en-us; 9360/8.0.0.359) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S55841[] = "UCWEB/2.0(BlackBerry; U; 8.0.0.520; en-us; 9360/8.0.0.520) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S55842[] = "UCWEB/2.0(BlackBerry; U; 9.6.0.39; en-us; 9360/9.6.0.39) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S55843[] = "UCWEB/2.0(BlackBerry; U; 9.6.0.86; en-us; 9360/9.6.0.86) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S55844[] = "UCWEB/2.0(BlackBerry; U; 9.6.0.26; en-us; 9360/9.6.0.26) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S55845[] = "UCWEB/2.0(BlackBerry; U; 9.6.0.112; en-us; 9360/9.6.0.112) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S55846[] = "UCWEB/2.0(BlackBerry; U; 8.0.0.464; en-us; 9360/8.0.0.464) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S55847[] = "UCWEB/2.0(BlackBerry; U; 8.0.0.391; en-us; 9360/8.0.0.391) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S55848[] = "UCWEB/2.0(BlackBerry; U; 8.0.0.419; en-us; 9360/8.0.0.419) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S55849[] = "UCWEB/2.0(BlackBerry; U; 9.6.0.24; en-us; 9360/9.6.0.24) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S55850[] = "UCWEB/2.0(BlackBerry; U; 9.6.0.93; en-us; 9360/9.6.0.93) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S55851[] = "Mozilla/5.0 (Android; Mobile; rv:25.0) Gecko/25.0 Firefox/25.0";
const char S55852[] = "Mozilla/5.0 (Android; Mobile; rv:25.0a1) Gecko/25.0a1 Firefox/25.0a1";
const char S55853[] = "Mozilla/5.0 (Android; Mobile; rv:25.0a2) Gecko/25.0a2 Firefox/25.0a2";
const char S55854[] = "Mozilla/5.0 (Windows NT 5.0; rv:17.0) Gecko/X Firefox/17.0";
const char S55855[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; Turkcell Maxi Plus 5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55856[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-; Turkcell Maxi Plus 5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55857[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Turkcell Maxi Plus 5 Build/IMM76D)";
const char S55858[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; Behold2 Build/DONUT) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S55859[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; Behold II Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S55860[] = "Mozilla/5.0 (Linux; Android 4.0.3;  A100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S55861[] = "Mozilla/5.0 (Linux; Android 4.0.3; A100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S55862[] = "Mozilla/5.0 (Linux; Android 4.0.4; A100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55863[] = "Mozilla/5.0 (Windows NT 5.0; rv:16.0) Gecko/X Firefox/16.0";
const char S55864[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Mobile Safari/535.19 Silk-Accelerated=true";
const char S55865[] = "Mozilla/5.0 (Series40; Nokia202/20.28; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15";
const char S55866[] = "Mozilla/5.0 (Series40; Nokia203/20.26; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15";
const char S55867[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT25i Build/9.1.A.1.140) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S55868[] = "Mozilla/8.0(Windows; U; Windows 7; en-Us; rv : 1.7) Gecko/X Firefox/15.1";
const char S55869[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 521)";
const char S55870[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55871[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.2 Chrome/15.0.900.2 Safari/535.2";
const char S55872[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.1 Chrome/15.0.900.1 Safari/535.2";
const char S55873[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.2 Chrome/15.0.900.2 Safari/535.2";
const char S55874[] = "Opera/9.80 (Windows NT 6.0; U; Edition Next; en) Presto/2.11.310 Version/12.50";
const char S55875[] = "Mozilla/5.0 (Windows; U; Windows 98; en-US; rv:1.8.1.5pre) Gecko/X Firefox/2.0.0.4 Navigator/9.0b2";
const char S55876[] = "Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.8.1.8pre) Gecko/X Firefox/2.0.0.7 Navigator/9.0RC1";
const char S55877[] = "Mozilla/5.0 (Windows; U; Win 9x 4.90; en-US; rv:1.8.1.8pre) Gecko/X Firefox/2.0.0.7 Navigator/9.0";
const char S55878[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.2.1) Gecko/X Firefox/12.2.1 PaleMoon/12.2.1";
const char S55879[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:12.2.1) Gecko/X Firefox/12.2.1-x64 PaleMoon/12.2.1-x64";
const char S55880[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTCEVOV4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55881[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E975K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55882[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55883[] = "Mozilla/5.0 (Series40; Nokia309/05.85; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.13.8";
const char S55884[] = "SAMSUNG-GT-C6112/C6112DDJB2 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55885[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55886[] = "Mozilla/5.0 (Linux; Android 4.0.4; TWD_MID Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55887[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55888[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S55889[] = "Mozilla/5.0 (Linux; Android 4.0.3; Build/X) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S55890[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbl1 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55891[] = "Mozilla/5.0 (Linux; Android 4.0.4; Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55892[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1808_v75_jbl1_5640 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55893[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1808_v75_jbl2_6628 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55894[] = "Mozilla/5.0 (Linux; Android 4.0.9; e1909c_v77_gq1008_9p017_6228 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55895[] = "Mozilla/5.0 (Linux; Android 4.0.4; Tender Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55896[] = "Mozilla/5.0 (Linux; Android 4.0.4; Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S55897[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1808_v75_jbl2 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55898[] = "Mozilla/5.0 (Linux; Android 4.0.3; V1013 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S55899[] = "Mozilla/5.0 (Linux; Android 4.0.4; EBM727KC Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S55900[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S55901[] = "Mozilla/5.0 (Linux; Android 4.0.4; Pad Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55902[] = "Mozilla/5.0 (Linux; Android 4.0.4; INFINITY_POWER Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55903[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1809l_v75_gq1008_ov5647_v20 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55904[] = "Mozilla/5.0 (Linux; Android 4.0.3; A200 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55905[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.10.289 Version/12.00";
const char S55906[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.10.289 Version/12.01";
const char S55907[] = "Mozilla/5.0 (Linux; Android 4.2.1; AT10LE-A Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55908[] = "Mozilla/5.0 (Linux; Android 4.2.2; M6 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55909[] = "Mozilla/5.0 (Linux; Android 4.2.2; S1S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55910[] = "Mozilla/5.0 (Linux; Android 4.2.1; T600 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55911[] = "Mozilla/5.0 (Linux; Android 4.2.1; C3 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55912[] = "Mozilla/5.0 (Linux; Android 4.2.1; 06_v89_jbla768_asx Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55913[] = "Mozilla/5.0 (Linux; Android 4.2.2; CW-V88-QUAD Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55914[] = "Mozilla/5.0 (Linux; Android 4.2.1; v89_jbl1a668 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55915[] = "Mozilla/5.0 (Linux; Android 4.2.1; Z5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55916[] = "Mozilla/5.0 (Linux; Android 4.2.1; 7102 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55917[] = "Mozilla/5.0 (Linux; Android 4.2.1; v89_gq2008s Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55918[] = "Mozilla/5.0 (Linux; Android 4.2.1; Lenovo S820_ROW Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55919[] = "Mozilla/5.0 (Linux; Android 4.2.1; AT10PE-A Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55920[] = "Mozilla/5.0 (Linux; Android 4.2.1; IMO S89 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55921[] = "Mozilla/5.0 (Linux; Android 4.2.1; B943 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55922[] = "Mozilla/5.0 (Linux; Android 4.2.2; Transformer Prime Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55923[] = "Mozilla/5.0 (Linux; Android 4.2.2; V818mini Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55924[] = "Mozilla/5.0 (Linux; Android 4.2.2; redhookbay Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55925[] = "Mozilla/5.0 (Linux; Android 4.2.1; v89_zlh_hd Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55926[] = "Mozilla/5.0 (Linux; Android 4.2.1; 06v20_v89_jbla768_asx Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55927[] = "Mozilla/5.0 (Linux; Android 4.2.1; X508 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55928[] = "Mozilla/5.0 (Linux; Android 4.2.2; S510 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55929[] = "Mozilla/5.0 (Linux; Android 4.2.1; H3088 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55930[] = "Mozilla/5.0 (Linux; Android 4.2.1; VOW V5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55931[] = "Mozilla/5.0 (Linux; Android 4.2.1; I9389 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55932[] = "Mozilla/5.0 (Linux; Android 4.2.2; EPAD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55933[] = "Mozilla/5.0 (Linux; Android 4.2.2; TCL S950 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55934[] = "Mozilla/5.0 (Linux; Android 4.2.1; v89_gq3008s Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55935[] = "Mozilla/5.0 (Linux; Android 4.2.1; ZTE V967S Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55936[] = "Mozilla/5.0 (Linux; Android 4.2.1; M470BSE Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55937[] = "Mozilla/5.0 (Linux; Android 4.2.1; Q5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55938[] = "Mozilla/5.0 (Linux; Android 4.2.1; HY5001 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55939[] = "Mozilla/5.0 (Linux; Android 4.2.2; V811 Core4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55940[] = "Mozilla/5.0 (Linux; Android 4.2.1; 06_v89_jbla768_asx_s4lockscreen Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55941[] = "Mozilla/5.0 (Linux; Android 4.2.1; AUXUS Nuclea N1 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55942[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-p6810 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55943[] = "Mozilla/5.0 (Linux; Android 4.2.2; R821 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55944[] = "Mozilla/5.0 (Linux; Android 4.2.2; Z2 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55945[] = "Mozilla/5.0 (Linux; Android 4.2.1; v89_jbla768_asx Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55946[] = "Mozilla/5.0 (Linux; Android 4.2.2; x909 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55947[] = "Mozilla/5.0 (Linux; Android 4.2.2; U8 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55948[] = "Mozilla/5.0 (Linux; Android 4.2.2; HYUNDAI T7 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S55949[] = "Mozilla/5.0 (Linux; Android 4.2.1; HX-9299A Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55950[] = "Mozilla/5.0 (Linux; Android 4.2.1; HUAWEI G610-U00 Build/HuaweiG610-U00) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S55951[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8) Presto/2.12.388 Version/12.10";
const char S55952[] = "SonyEricssonCK13i/R5AA032 TelecaBrowser/Q07C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S55953[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; PMP5080B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55954[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; ru-ru; PMP5080BRU Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55955[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTCEVODesign4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55956[] = "Mozilla/5.0 (Linux; Android 4.0.3; PH44100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S55957[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Cyclone Voyager Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55958[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Cyclone Voyager Build/JRO03H)";
const char S55959[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Cyclone Voyager 2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55960[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT25i Build/9.1.A.0.490) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S55961[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT25i Build/9.1.A.0.490) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S55962[] = "LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/06.14.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S55963[] = "LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/07.06.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S55964[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A288t Build/MocorDroid2.3.5) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S55965[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A288t Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55966[] = "Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A288t Build MocorDroid2.3.5) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S55967[] = "LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/08.01.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S55968[] = "LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/08.16.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S55969[] = "LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/10.22.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S55970[] = "LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/06.20.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S55971[] = "Lenovo-A288t_TD/S100 Linux/2.6.35 Android/2.3.5 Release/02.29.2012 Browser/AppleWebkit533.1 Mobile Safari/533.1";
const char S55972[] = "LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/11.23.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S55973[] = "LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/09.28.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S55974[] = "LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/10.09.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S55975[] = "LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/12.28.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S55976[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Lenovo A288t Build/MocorDroid2.3.5)";
const char S55977[] = "LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/09.20.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S55978[] = "LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/03.27.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S55979[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Cynus T1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S55980[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Cynus T1 Build/IMM76D)";
const char S55981[] = "Mozilla/5.0 (Linux; Android 4.1.2; A510 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S55982[] = "Mozilla/5.0 (Linux; Android 4.1.1; A510 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S55983[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ505 Build/Moto_MZ505) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S55984[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xoom Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S55985[] = "Mozilla/5.0 (Linux; Android 4.0.3; Xoom Build/IML77) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S55986[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; SGH-I717R Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55987[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-I717R Build/GINGERBREAD)";
const char S55988[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SGH-I717R Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S55989[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7500 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55990[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7500 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55991[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7500 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55992[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7500 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55993[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Xoom Build/JB_MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55994[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Xoom Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55995[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Xoom Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55996[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Xoom Build/JB_MR1.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S55997[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; Xoom Build/JDQ39)";
const char S55998[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.7.3; U; en) Presto/2.10.229 Version/11.61";
const char S55999[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE63) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile";
const char S56000[] = "NokiaE63/UCWEB8.6.0.180/28/999";
const char S56001[] = "Mozilla/5.0 (Java; U; en-us; nokiae63-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S56002[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S56003[] = "Mozilla/5.0 (Linux; U; Android 1.0.0; en-us; A70BHT Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56004[] = "Mozilla/5.0 (Linux; U; Android 1.0.1; md-us; A70BHT Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56005[] = "Mozilla/5.0 (Linux; U; Android 1.1.1; en-us; A70BHT Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56006[] = "Mozilla/5.0 (Linux; U; Android 1.0.0; es-es; A70BHT-LZ Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56007[] = "Mozilla/5.0 (Linux; U; Android 1.0.4; fr-fr; A70BHT-SC Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56008[] = "Mozilla/5.0 (Linux; U; Android 1.0.0; us-us; A70BHT-PR Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56009[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaN73) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S56010[] = "Mozilla/5.0 (Windows NT 6.0; rv:15.3) Gecko/X Firefox/15.3.2 PaleMoon/15.3.2";
const char S56011[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:15.3) Gecko/X Firefox/15.3.2-x64 PaleMoon/15.3.2-x64";
const char S56012[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S56013[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S56014[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S56015[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S56016[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S56017[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S56018[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S56019[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S56020[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-I9300 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S56021[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9300 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S56022[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-I939 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S56023[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SGH-T999 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S56024[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SCH-I535 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S56025[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-I939D Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S56026[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Iron/10.0.650.1 Chrome/10.0.650.1 Safari/534.16";
const char S56027[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Mobile Safari/535.19 Silk-Accelerated=true";
const char S56028[] = "Opera/9.64 (Windows NT 5.0; U; en) Presto/2.1.1";
const char S56029[] = "Opera/9.62 (Windows NT 5.0; U; en) Presto/2.1.1";
const char S56030[] = "Opera/9.63 (Windows NT 5.0; U; en) Presto/2.1.1";
const char S56031[] = "Opera/9.60 (Windows NT 5.0; U; de) Presto/2.1.1";
const char S56032[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; de) Opera 9.64";
const char S56033[] = "Opera/9.61 (Windows NT 5.0; U; en) Presto/2.1.1";
const char S56034[] = "HTC_Wildfire_Mozilla/5.0 (Linux; U; Android 2.1; it-it; HTC Buzz Build/ERD79) AppleWebKit/ 530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56035[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Wildfire 1.14.161.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56036[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; nl-nl; HTC Wildfire 1.14.172.2 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56037[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Wildfire 1.26.161.3 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56038[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; HTC_Wildfire-orange-LS Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56039[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; HTC Wildfire 1.13.163.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56040[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; HTC Wildfire 1.14.162.3 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56041[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; HTC Wildfire 1.25.162.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56042[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; HTC Wildfire 1.36.164.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56043[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; HTC Wildfire 1.29.163.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56044[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; nl-nl; HTC Wildfire 1.25.172.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56045[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; de-ch; HTC Wildfire 1.29.163.2 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56046[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; HTC Wildfire 1.17.164.3 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56047[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; HTC Buzz 1.02.161.2 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56048[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Wildfire Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56049[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; HTC Wildfire Build/ERE27)";
const char S56050[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; HTC Wildfire Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S56051[] = "curl/7.24.0 (x86_64-apple-darwin12.0) libcurl/7.24.0 OpenSSL/0.9.8r zlib/1.2.5";
const char S56052[] = "curl/7.24.0 (x86_64-apple-darwin12.0) libcurl/7.24.0 OpenSSL/0.9.8x zlib/1.2.5";
const char S56053[] = "curl/7.24.0 (x86_64-apple-darwin12.0) libcurl/7.24.0 OpenSSL/0.9.8y zlib/1.2.5";
const char S56054[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT925 Build/7.7.1Q-144_VQL_S3-58) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S56055[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR HD Build/7.7.1Q-144_VQW_S334-25) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S56056[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT925 Build/7.7.1Q-144_VQL_S3-58) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S56057[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR HD Build/7.7.1Q-144_VQW_S334-25) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S56058[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6506 Build/10.3.A.0.423) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S56059[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Maxthon/4.1.0.4000)";
const char S56060[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Maxthon/4.1.0.4000)";
const char S56061[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Maxthon/4.1.0.4000)";
const char S56062[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Maxthon/4.1.0.4000)";
const char S56063[] = "Mozilla/5.0 (Linux; Android 4.0.4; Hydro Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S56064[] = "Mozilla/5.0 (Linux; Android 4.0.4; C5170 Build/IML77) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S56065[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; LG-LU6800 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56066[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-LU6800 Build/FRG83G)";
const char S56067[] = "Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.65 Safari/535.11";
const char S56068[] = "Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.83 Safari/535.11";
const char S56069[] = "Mozilla/5.0 (Linux; Android 4.0.3;  PG86100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S56070[] = "Mozilla/5.0 (Linux; Android 4.0.3; PG86100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56071[] = "Mozilla/5.0 (Linux; Android 4.0.4; PG86100 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56072[] = "Mozilla/5.0 (Linux; Android 4.0.4; Evo 3D GSM Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56073[] = "Mozilla/5.0 (Linux; Android 4.0.3; ISW12HT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56074[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN95) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S56075[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN95 8GB) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S56076[] = "UCWEB/2.0(Java; U; MIDP-2.0; fa; nokian95) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S56077[] = "NOKIAN95UCBrowser/8.7.1.234/28/444/UCWEB";
const char S56078[] = "NOKIAN95 8GBUCBrowser/8.7.1.234/28/444/UCWEB";
const char S56079[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokian95_8gb) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S56080[] = "Mozilla/5.0 (Series40; NokiaX2-00/08.25; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S56081[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P7500 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.26 Safari/537.31";
const char S56082[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S56083[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500R Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S56084[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500M Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S56085[] = "Nokia311/5.0 (03.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia311) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S56086[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S56087[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300TG Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S56088[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300TL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S56089[] = "Mozilla/5.0 (Linux; Android 4.1.2; Transformer Pad TF300T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S56090[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S56091[] = "Mozilla/4.0 (BREW 3.1.5; U; en-us; Sanyo; NetFront/3.5.1/AMB) Boost SCP3810";
const char S56092[] = "Mozilla/4.0 (BREW 3.1.5; U; en-us; Sanyo; NetFront/3.5.1/AMB) Sprint SCP-3810";
const char S56093[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; AN7CG2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56094[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; AN7CG2 Build/GINGERBREAD)";
const char S56095[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-nz; U8186 Build/HuaweiU8186) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56096[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; U8186 Build/HuaweiU8186)";
const char S56097[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; H2000+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56098[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; H2000+ Build/IMM76D)";
const char S56099[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300TL Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S56100[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300T Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S56101[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300TG Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S56102[] = "Mozilla/5.0 (Linux; Android 4.1.2; Transformer Pad TF300T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S56103[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT919 Build/2_290_2012) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S56104[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT300SE Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S56105[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT300 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S56106[] = "Mozilla/5.0 (Linux; Android 4.1.1; HP Slate 7 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S56107[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I897 Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56108[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I897 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56109[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I897 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56110[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC PH39100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S56111[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC PH39100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S56112[] = "Mozilla/5.0 (Linux; Android 4.3; GT-I9505G Build/JWR66V.S014) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S56113[] = "Mozilla/5.0 (Linux; Android 4.3; GT-I9505 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S56114[] = "Mozilla/5.0 (Linux; Android 4.3; en-gb; SAMSUNG GT-I9505 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Version/1.5 Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S56115[] = "SAMSUNG-GT-E2250 Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.29465/30.3745; U; en) Presto/2.8.119 Version/11.10";
const char S56116[] = "NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac1-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S56117[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11";
const char S56118[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11";
const char S56119[] = "Mozilla/5.0 (Linux; Android 4.0.4; cm_tenderloin Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S56120[] = "Mozilla/5.0 (Linux; Android 4.0.3; cm_tenderloin Build/GWK74) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S56121[] = "Opera/9.80 (X11; Linux i686; U; en) Presto/2.9.168 Version/11.51";
const char S56122[] = "Opera/9.80 (X11; Linux i686; U; ru) Presto/2.9.168 Version/11.51 (ru-RU,ru;q=0.9,en;q=0.8)";
const char S56123[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; E330 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56124[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; GT-S5360 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S56125[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5360 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S56126[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-LA; GT-S5360L Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S56127[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; SCH-i509 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S56128[] = "NokiaX2-00/5.0 (04.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile";
const char S56129[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100B Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S56130[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S56131[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S56132[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S56133[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S56134[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S56135[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S56136[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S56137[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S56138[] = "Mozilla/5.0 (Linux; Android 4.0.3; LT22i Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S56139[] = "Mozilla/5.0 (Linux; U; Android 2.0; en-us; pandigital9hr) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56140[] = "Mozilla/5.0 (Linux; U; Android 2.0; en-us; pandigital9hr2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56141[] = "IUC(U;iOS 5.0.1;Zh-cn;320*480;)/UCWEB8.4.1.169/42/997";
const char S56142[] = "IUC(U;iOS 5.1.1;Zh-cn;320*480;)/UCWEB8.4.1.169/42/997";
const char S56143[] = "IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB8.4.1.169/41/997";
const char S56144[] = "IUC(U;iOS 5_1_1;en-us;320*480;)/UCWEB8.4.0.159/42/355";
const char S56145[] = "IUC(U;iOS 4.2.1;Zh-cn;320*480;)/UCWEB8.4.1.169/41/997";
const char S56146[] = "Mozilla/5.0 (Java; U; en-in; spice) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S56147[] = "Mozilla/5.0 (Java; U; en-us; opera) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S56148[] = "Mozilla/5.0 (Java; U; en-us; maui) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S56149[] = "Mozilla/5.0 (Java; U; en-in; j2me) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S56150[] = "Mozilla/5.0 (Java; U; en-in; mstar) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S56151[] = "Mozilla/5.0 (Java; U; en-in; mstar platform) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S56152[] = "Mozilla/5.0 (Java; U; en-us; generic) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S56153[] = "IUC(U;iOS 4.1;Zh-cn;320*480;)/UCWEB8.4.1.169/41/997";
const char S56154[] = "Mozilla/5.0 (Java; U; en-us; mozilla) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S56155[] = "Mozilla/5.0 (Java; U; kau; maui) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S56156[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ADR930L 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56157[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ADR930L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56158[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; ADR930L Build/JZO54K)";
const char S56159[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; A211 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S56160[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; A211 Build/JRO03H)";
const char S56161[] = "Mozilla/5.0 (Windows NT 4.0; rv:12.0) Gecko/X Firefox/12.0";
const char S56162[] = "Mozilla/5.0 (Series40; Nokia111/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.5.16.9";
const char S56163[] = "Mozilla/5.0 (Series40; Nokia111.1/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.5.16.9";
const char S56164[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.5) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.5";
const char S56165[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.11) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.11";
const char S56166[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.3) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.17";
const char S56167[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.13) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.13";
const char S56168[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.18) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.17";
const char S56169[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.7) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.7";
const char S56170[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.8) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.8";
const char S56171[] = "Mozilla/5.0 (X11; U; Linux i686; fi-FI; rv:1.9.0.13) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.13";
const char S56172[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.17) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.17";
const char S56173[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.6) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.17";
const char S56174[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.5) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.4";
const char S56175[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.8) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.8 (.NET CLR 3.5.30729)";
const char S56176[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE N9120 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S56177[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Slider SL101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S56178[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Slider SL101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S56179[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; Slider SL101 Build/HTK75)";
const char S56180[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PantechP9090 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56181[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; PantechP9090 Build/JZO54K)";
const char S56182[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; V9 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56183[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; V9 Build/ERE27)";
const char S56184[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11CA Build/01.08.00) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56185[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11CA Build/01.10.20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56186[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11CA Build/01.12.00) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56187[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11CA Build/01.09.00) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56188[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11CA Build/01.12.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56189[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11CA Build/01.13.60) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56190[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11CA Build/01.14.50) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56191[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11CA Build/01.11.30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56192[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11CA Build/01.15.20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56193[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11CA Build/01.16.00) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56194[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S56195[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S56196[] = "LG-GR500R/V10a Teleca/Q7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S56197[] = "LG-GR500F/V10a Teleca/Q7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S56198[] = "LG-GR500/V10j Teleca/Q7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S56199[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E270K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56200[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E270K/KKAMC3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56201[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E270K/KKAMD1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56202[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E270K Build/JZO54K)";
const char S56203[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E270K/KKAME2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56204[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E270K/KKAMF2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56205[] = "Nokia5610/2.0 (08.20) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S56206[] = "Nokia5610d-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S56207[] = "Nokia5610 XpressMusic/2.0 (08.23) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S56208[] = "Nokia5610_CMCC/2.0 (09.70) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S56209[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Iron/8.0.555.0 Chrome/8.0.555.0 Safari/534.10";
const char S56210[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S56211[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S56212[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830C Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S56213[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830C Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S56214[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-S738C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S56215[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S56216[] = "Mozilla/5.0 (compatible; MSIE 9.0; ; Trident/5.0)";
const char S56217[] = "Mozilla/4.0 (compatible; MSIE 9.0;";
const char S56218[] = "Mozilla/5.0 (compatible; MSIE 9.0; PPC; Trident/3.0)";
const char S56219[] = "Mozilla/5.0 (compatible; MSIE 9.0; PPC; SV1; Trident/4.0)";
const char S56220[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAB10-201 Build/Xenta) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S56221[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; Xenta TAB10-201 Build/Xenta TAB10-201) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S56222[] = "AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; Xenta TAB10-201 Build/Xenta TAB10-201)";
const char S56223[] = "Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Safari/535.19 Silk-Accelerated=true";
const char S56224[] = "sam-r261 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S56225[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; GT-I9001 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56226[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; GT-i9001 Build/JZO54K; CyanogenMod-EHNDROIX II i9001) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56227[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/11.04 Chromium/13.0.782.215 Chrome/13.0.782.215 Safari/535.1";
const char S56228[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/11.04 Chromium/13.0.782.41 Chrome/13.0.782.41 Safari/535.1";
const char S56229[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ARCHOS 101G10 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S56230[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ARCHOS 101G10 Build/JRO03L)";
const char S56231[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) iLivid/1.0 Safari/534.34";
const char S56232[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.34 (KHTML, like Gecko) iLivid/1.0 Safari/534.34";
const char S56233[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Superbird/24.0.1312.57 Safari/537.17";
const char S56234[] = "Mozilla/5.0 (N; Windows NT 6.2; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) Raptr Safari/534.34";
const char S56235[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.17 (KHTML, like Gecko) Superbird/24.0.1312.57 Safari/537.17";
const char S56236[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Superbird/27.0.1454.94 Safari/537.22";
const char S56237[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.15 (KHTML, like Gecko) Steam Safari/535.15";
const char S56238[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.22 (KHTML, like Gecko) Superbird/27.0.1454.94 Safari/537.22";
const char S56239[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Superbird/28.0.1500.71 Safari/537.36";
const char S56240[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) SuperBird/23.0 Safari/537.11";
const char S56241[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Steam Safari/535.19";
const char S56242[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Superbird/28.0.1500.71 Safari/537.36";
const char S56243[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC PH39100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S56244[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC PH39100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S56245[] = "Nokia5630_CMCC/011.023 (SymbianOS/9.3; U; Series60/3.2.3 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S56246[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2Nokia5630d-1/013.009; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S56247[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia5630d-1/013.009; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S56248[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2.3 Nokia5630d-1/011.001; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S56249[] = "Nokia6790s-1b/ATT.03.22 Mozilla/5.0 SymbianOS/9.3; U; [en]; Series60/3.2; Profile/MIDP-2.1 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S56250[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia6790s-1c/20.007; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S56251[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB200 Build/brwnMOD v3.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56252[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-; MB200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56253[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us;MB200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56254[] = "Mozilla/5.0 (Linux; U; Android2.1-update1; en-us; MB200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56255[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56256[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB200Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56257[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 MobileSafari/530.17";
const char S56258[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; MB200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56259[] = "Mozilla/5.0 (Linux; U;Android 2.1-update1; en-us; MB200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56260[] = "Mozilla/5.0 (Linux; U; Android 2.1.5; es-es; MB200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56261[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB200 Build/ERD91_INT) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56262[] = "Dalvik/1.2.0 (Linux; U; Android 2.1-update1; MB200 Build/ERD79)";
const char S56263[] = "NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-01) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S56264[] = "Mozilla/5.0 (Linux; Android 4.0.4; N9500 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S56265[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR6400L Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56266[] = "Mozilla/5.0 (Linux; Android 4.0.4; ThunderBolt Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56267[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6400L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56268[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML, like Gecko) Iron/11.0.700.2 Chrome/11.0.700.2 Safari/534.24";
const char S56269[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Iron/11.0.700.2 Chrome/11.0.700.2 Safari/534.24";
const char S56270[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S56271[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S56272[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S56273[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100T Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S56274[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S56275[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100M Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S56276[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F100S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56277[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-F100S Build/IMM76D)";
const char S56278[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LG-F100S Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S56279[] = "Mozilla/5.0 (Windows NT 5.1; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.0.1";
const char S56280[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.0.1";
const char S56281[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; SGH-T899M)";
const char S56282[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; GT-I8750)";
const char S56283[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 920)";
const char S56284[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 920; ANZ821)";
const char S56285[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 920; Vodafone)";
const char S56286[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 920; Orange)";
const char S56287[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Nokia 920T)";
const char S56288[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Nokia 920)";
const char S56289[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6602 Build/10.1.A.1.434) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S56290[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6602 Build/10.1.A.1.434) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S56291[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-02E Build/10.1.D.0.343) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S56292[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-P5100 Build/JOP40D)";
const char S56293[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; GT-P5100 Build/JOP40D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Safari/534.30";
const char S56294[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P5100 Build/JDQ39E)";
const char S56295[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; GT-P5100 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S56296[] = "Mozilla/4.0 SonyEricssonW910iv/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S56297[] = "SonyEricssonW910i/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S56298[] = "Nokia200/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia200) UCBrowser8.4.0.159/70/444/UCWEB Mobile";
const char S56299[] = "GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (5.1.2600.3);en;kml:2.2;client:Free;type:default)";
const char S56300[] = "GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (5.1.2600.3);zh-Hans;kml:2.2;client:Free;type:default)";
const char S56301[] = "GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (5.1.2600.2);en;kml:2.2;client:Free;type:default)";
const char S56302[] = "Mozilla/5.0 (Windows; N; Windows NT 5.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/7.1.1.1888 Safari/532.4";
const char S56303[] = "GoogleEarth/7.1.1.1888(Windows;Microsoft Windows (5.1.2600.3);en;kml:2.2;client:Free;type:default)";
const char S56304[] = "GoogleEarth/7.1.1.1888(Windows;Microsoft Windows (5.1.2600.2);en;kml:2.2;client:Free;type:default)";
const char S56305[] = "GoogleEarth/7.1.1.1888(Windows;Microsoft Windows (5.1.2600.3);de;kml:2.2;client:Free;type:default)";
const char S56306[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-; SCH-R880 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56307[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SCH-R880 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56308[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SCH-R880 Build/FROYO)";
const char S56309[] = "Mozilla/5.0 (Linux; Android 4.0.3; ZTE Grand X Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56310[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56311[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970M Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56312[] = "Mozilla/5.0 (Linux; Android 4.0.3; ZTE V970 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56313[] = "Mozilla/5.0 (Linux; Android 4.0.4; Grand X(M) Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56314[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE-U V970M Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56315[] = "Mozilla/5.0 (Linux; Android 4.0.4; Grand X Pro Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56316[] = "Mozilla/5.0 (Linux; Android 4.0.4; Grand X Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56317[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HUAWEI-U9000 Build/HuaweiU9000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56318[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; HUAWEI-U9000 Build/HuaweiU9000)";
const char S56319[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9B206 Safari/7534.48.3";
const char S56320[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9B176 Safari/7534.48.3";
const char S56321[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT25i Build/9.0.1.D.0.10) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56322[] = "Mozilla/5.0 (Linux; Android 4.2.1; ME301T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S56323[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (yLr7EYRY+BPE6cyYr8SmFfIYdh8JYq1AqYls1MG3Xsk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56324[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (eXOS9+r/txvf7GWAR83bE831NJ1A0skcz/3ifI/hSRg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56325[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (WwP6TFR8dkb3nE2b3iz6/+6FvKRTq9DcAiq91iNmRRA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56326[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (eQgPy3Q2MPSoxbaOkDQ9rgyEnIm4AQzLO2w5u0yhK0w=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56327[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (rAarqQZ39qF9yogSsDjqVrcJDs77SqxPLdKD3G3RTVg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56328[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (e+JzrrzKBmWDkr+EEqA1qSoiT104X513i5C6nqA7rzE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56329[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (E6in5QOY71GVa82exQB8887za0VXlMNIevf4LRK0CBs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56330[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (UuCyn/MDeKIqEOTue1DYHMGflhX4w4+4cfNzdo9/ADA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56331[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (VyUcpeAR8Ai8spfwg6tb72hAeolwd7YD7IzHnQBYBN0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56332[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (vu02ioweEmbE1k05PbdvwLx/SYSGaWQhXdg3hlnQiKI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56333[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (wwzFDV3lHrr3kWautEAS4WWFNBGNIdsQ5Sr/xIy1o0Y=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56334[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (9nsdhJ/+S6cos8/qpISdjN34FLUX2gAPzMSYb+0S+p4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56335[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (9McWXSTeEmMMfA6OsMV9kep3k6cuVZuHY6NWIDd8I2c=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56336[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (g/GwcJbJeh+6nQ/8SRBrYyh23xhe/8NPibCGJRr9E1g=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56337[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (fC9aZpPd6Hn/OYBx2h81JRUl3rqmVvjwW9K9UqUEf78=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56338[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (8ENVDUFtaYrI+znmdpzFWFaKl5iYRPcxfQCK5iQ+Exg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56339[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ro7B4K6y6K5Rejh0OZPGHXzt2Vb8ilglPyaDOLXzgqA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56340[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (QkfdtMJ96CzPwvqRrtR+41uuFIWZeOaVmq+ds5tJ//w=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56341[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (QDsFFh/GywLoYHpNZElVJa1IXkbKb5uqLgqYi6JsZ0w=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56342[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (2VvZcHNQ0uUtnyVfS5XXpWpTW0uePqs8OMeQocRC54c=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56343[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (UMmnbVoi0nLSMEdfHgMFETIY4p7zr8Ss5bF+T3tJr2s=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56344[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (1j9Klv6ZovFXBK/nyrgG/iH+jY/bvSpyuNsuzCAL2kc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56345[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (oVaX3xu8lS6+dxqSTxJyCyJ+2By0urQwPAPZDhxZX8Q=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56346[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (659XPlOKKBiubejHMHV73+zxqMzVDxAZmvCKmHMwVgY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56347[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (RxDps3R7fTXYeIKbpiKNzqii48KMn3QVW+Id41TNiRY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56348[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Nszvs8ocD3cLViHON78UentvzX/lZBQFQeGypzznosI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56349[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (gzguGHikfSlBmx1Uvv5uki6ydx11N9G1XtqTJLgyJbk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56350[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Dc9XSTm++s9+r7kiONBrIAeCrm1I8yINUtZWOg4WxNY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56351[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (BiJcJ/NnWWNRCJbpE1IDAUARApQI13bBpXjUKIJ3NX0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56352[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (/fePx6eQbNOJc18VKF+tUCcbSw7dqUiyUeAWMr1Oe2E=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56353[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (NZhZ48QdASSwhLUCfDdPk788vO53KRHFxUX0yUzs8WY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56354[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (B4+CrD9j8S7HbuyOvQLVn3xRku3nUUZiU8mwvow+fF4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56355[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (HRfVdPGb2DXDoQv7j0jiZ7kumjupqYEZuuv+uUjYaXY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56356[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (EnLPfK3gBP87LW9Hd6JIkOZwJI246mpl+xkiH+y8vqA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56357[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (MAMz0qVOAVH1zmRrKL6LTwU7KrhI+1chexANAMUmFM4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56358[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (wwV7b+ehRKit4Z/Dcpf5Iyt+XWId08eJmLj0BeAM1ik=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56359[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (FgZ9ZDQJMtbyd+5UMDlLOlswH+0rh9Wzij5CW0gcD5g=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56360[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (27R4y1cxKja/+yOuL1iUKmJfspVxIpm4qgV/wcGnnng=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56361[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (emYYDLcpXdnGC4F7/lsxEyicjrvVS3AKJI6j/hO5EiE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56362[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (mEHSren5PNQEk2sVQQOzhnX0SPrYAQ6x+xwMaR4JcZg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56363[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (7zlzhARynJN0A3DR1n2rAVz6ogiv+TqieWs82IN6w7c=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56364[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (rB52Gi4rXs2Hd+lWYo6EBLZ6nlUb+W5M115D/RKUNpU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56365[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (/pvbdYxWRjdoWSNGyIEiz24A/TxU5NIpGCrI1c+6BO8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56366[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (0I21w8U5AvqqNw4mgaFumiiimI3WBMq9In3/oI8wSVo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56367[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (arfzlppEmoiA53tb+e6+XK5ub99P4csLo3yOjLfiryE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56368[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (5iKulOK5fSdkCAflu2pPnoc+mVMN1AGKenxJLFPYung=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56369[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (6PNbJqYPVfeNgfBReG2mSHUyWmtwktcmfHhI0M3t0O4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56370[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (EBwxSqWg5Wyih9g4o0X6Fnn3KtSCv25+Z+q3jvSY/4Y=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56371[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (MJlfzfmnC1aXm16FnVsEDvR7IPIhBO2LA7vphtTU+/Q=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56372[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (tn6Q5f9Ot0lhlMN1eraayuqNMoK2OuAKELMhxZQMzr0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56373[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (jYgJA+CQFZBTIh8fIZ8yjQSipb2lRoSHvR9EkMfGUKc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56374[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (9ueqGW6b1Fz57VroPB3Hd/4XUdOCNbcny4nMxcaCeE0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56375[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (04khbePl+otyC+t7Z1Pu2Rbap5uJ/LllvSrEAqSrJyo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56376[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (JfnV4ayfSahhDq3h8ZW6haERMx/9Vr37XmTYkOwRFC4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56377[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Sd3AqNoBhGZf18vsXJ0/8nH6XhxAVPOOtWHj+vYtE0E=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56378[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (7datnnrdtSacUg2wppZ/l12m0AoNxiaS++MDoWK56ts=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56379[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (/jRFM94SIl+0rwrScZlAfRpeVd1zX9LVAc31pppjmOQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56380[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (5JnbqYmhynI9cNOdpWUQdRtcgOmRf2qSRPfS2jgTqMU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56381[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (SY7Up89ngF6J8ZipjX/NoWXRdPkjrE9YRa4jUcMAnI0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56382[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (LmSzCLvkVtv8go56pkt0CddiEHAq3YDx2qKsCpola4o=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56383[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (eLxYEzaXKSGyVCtDG8d1ouh030x3nVxlEbl8rNaF8D8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56384[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (NYhSzv2V4fErCf970dbkWk1gTpEr/VQoLOcOKvbOAIw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56385[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (GBaT+dAKdb5ag0K1t73SVwzni3araYhmuJUEvFd+aLQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56386[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (102RA7gBLDVd58NEHncNUo8mdSgXueovHUHvzElkp7I=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56387[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (H9vuO55J/W/eNCjf7LhP6UHNesJkTSkpq/3U3wuSATg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56388[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (3OcK1N4hdsHIuNk6EOP87XJ4nHTQGDAiLkIKY2Bb3UQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56389[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (DfTxNfIc1SJWFW3YLg5NgpQppGO+uX/9ERxv4qLmHTM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56390[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (2087ant8VnOSsF8CFzumB4dSFXO1oFyqgTXTVG1EBz8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56391[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (KUyUZWf8ZTgrrfMVs9t1/AnFjVeflpi9+6XWpGMjI+A=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56392[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (1UuigJfJyxyQtlTlacrUKuXHRwy1OwqkhfAafGxOnoE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56393[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (F87GmG3U5UzGPls4nzXh8E0YdQDCbyzs7f3KnzyYCWY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56394[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (K+6qnOvOy9mHa8gypoOGCxZ0+9Q/XdToNVvpen9WhxA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56395[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (4blVetg+KABCZSS+P+rTlbEQ/omF/lPtkIZXO0oioAo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56396[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (MIMnM26aJxYcC7D6UcR/UbhgWJ7ZkH2LmIsIMSx2ZJM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56397[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (6Yw3OOeWeoGqtg/kx7EAagivd9LN2yOQ+V34bgG2p8E=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56398[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (+g+tKZzOq3m0Eugbze7XxqEERXmoJFBrjSDY0bcAZKc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56399[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (QOrVbdS+eL/r0Slz60PYt6HyAZGbaOIDfchqmBBpV6o=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56400[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (OIAQUKTlXuMbp8WFJ/ocZV18fzht3HjfhkuAmcorvLc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56401[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (MLXScUUnhU/hofqC9vs5OHYQ4TkagfNIkyUfE2GAmFo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56402[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (GnkUPP2lBJm/9lZYoP7uGAYRR0ms99IG/Q5qufwgfOQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56403[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (O/uMX6WZOdiKtdhCcmJ2uSeNg3Q0+oIwjDpAIRP3kNM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56404[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (PXapA5Z/MGQaDC7vjDXgexwbBdCzIkaopbjJAOsjbbM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56405[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Ojv/+Fixi9wu+PKtowPVhCpxwKZ8Wnx2KXh218/Uk0k=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56406[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Cz5aCNaTFtXXsKT2oFHQd3qPc+QLU0v4dHo1orRVM64=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56407[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (K4d71TAHukdLN+JPiJ11cONTHk+8LRVjt1q41xl4vy4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56408[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (F8mmnbiyVe0G9nvtIG8BSOM6dNSgqO9W4tJ7i9N3++s=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56409[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (I1LB8FMPASyXKcBjEBcL240oWtFG6JXhL+umWvuUcfc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56410[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Gb5K8kVWawT4LUxyaZLuen3KaKj8jtDJazcUmbrOWOE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56411[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (2rQeXPsC3ePKld67gekywWvCSzi71y7a/gNvje6OBok=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56412[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (EJifqhr2KN4J9xAlKJJdhadmpGTcwX8wR16iE7hNJbs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56413[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (60rM7YJMgxHqoCw68TlAJr5wr2bgrFIsvtEoy8jVELY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56414[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (QJe6dPVgINLoKdCHOq+uFSCo1D53BFcliDisLQrHUh4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56415[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (LhplWfaTuWxKYSG0sR7ZNNc5WYl1Aek4yz1jsXOPEMQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56416[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (6y+XVtkcmUcScnuMY4hWaj5HPWngNx88XntxfGm+3rs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56417[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (CXIaASHC25QDsRh4jhP3qQhlDSjFCtbcgeyZaw2BRQw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56418[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 () Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56419[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (KNsE1EASYyeCh9D38j3CdBbjRSgp1qgR3XU0PzTLYCk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56420[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ESKbJrIxO/OM+TIVZX/LEyPAwKTZ4XlxhvYo5y6FP1k=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56421[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (q2LVBpKdjbTyqvLqnu3cXX5gjjEx1Jq+0EvSmPd3Vpw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56422[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (GZTg5JsTFXdJASbcqxtJ4kc227JaAEjA9qCSMp3LpEM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56423[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (9hRsNiBmaMF8LcGaO6q7poecL5ltV06i1ehDP02K7xY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56424[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (HGAlINyHJ63WZpMfRSkYNnKwuad2cIqEK7jujdf8hn0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56425[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (MhomaEPz8o9QtTg/tWkYogXfAhagASQ8MA5iQzKAXks=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56426[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Dzon2AOH6VZ/H8TzCrtS+PkGAVMhxyeTIhm/Q11rGR0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56427[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (6Ax7j/13R6Z+KPeeg3aRWmnpPSgOnfx/HYFNi9rU+2M=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56428[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (P/JRMeIn4QwFCty0H+lik+8+BIbrthQEdtY89HAjZD0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56429[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (PSjeKGWdCylYhWGfjYvF0dKDtfIL4BxWi9vri2hRbT0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2 AlexaToolbar/alxg-3.1";
const char S56430[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (HCp7eVfyKxXZNDQvdaBJRfFA7oFojwRgXl2rHa4JnCk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56431[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (OSN90Wsfxm9WWBpVchbma031XNL8JAPoCLlVDDzOeNE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56432[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (M1IESb2ieydVOooaIxiujqJ0nx0cCRZISR5PbPbwffY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56433[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (0VlR2JAtqxZ6fkq0Y7BbxKf/TDooTZhhpdMuzAEu3yo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56434[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (1XiV8WgCLfifNzZmhfphbeGZxNPI8JR8bm8H24JHXRg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56435[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (A2DQFIrA3vbr+RZxhUCaOI0Jh6xBymHa1zqg2pShAM0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56436[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (/NX7InNn6d3CRfo3caRNV4mqCQJtRcZm1LIa9kNCw3s=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56437[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (FEDdaLoNUMVKNv9SOf3b8WwTXYqAtJCYcZNmS5gj7ME=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56438[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (4m6sVCI4nP2PkRhoYwuo1oUTC8xkiI8OMfdMKaooMog=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56439[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (CZ2zxrlGsSGJMJ0K9VCK0WmASFXw539bhAd3I7sgcFM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56440[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (NYUqLfyWlXYPLOEAxB+0NB9op+as7RylykItSXUTzdU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56441[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (2kD9OO3QeDnxgxb1LsKxCtrVVY6KXG3NIe99Nala+nY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56442[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (1eGZWrh/GNRuPXlJ6gdzFytm+ofPtfIEje3w3et5jIM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56443[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (PP6xmQBlu5tkR2DPw4D/PGYsePZAXxHD8IMHOEI50lY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56444[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (7zeHZopFpbXZh+fsx0qv92Y120A/VprIybj2ogv4BEc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56445[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (GrB9prK+u1lhwLXPUHW9+h6ktavdko1Cmpq24Fb0dUM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56446[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (24IOq23G4tDKCeD+daqEDDbrkF86sWfSwi+pDx8iKnE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56447[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (IpQ478U5sV2cpUIQluc4yCEzMpy0TUG6UBzIvvsrCXs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56448[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (PFtdHkTvl+DlzVDiw97aq8a9hi+E53Y61rHRkoKMJtw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56449[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (0xC+aZXjvyX3z7vZ6WQchghPLa0zEU855ZHU9K75bPg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56450[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (6tk33y5hhueCSio09/ceEZwcEtv31GHeKbrx8GSD8sM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56451[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (6y4KDAUZ4Yeu3B0wudIC/OxcUC2/x6g5MQ01IVviMK0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56452[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (KJvmEcjv1LthE68S1zu1N/UIUYsE/ZDYLGZ5Gw4eRiY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56453[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (J/fOmXi1yuShbr0lH505R5fBBaJd7oFZ++SvxADvX8g=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56454[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (h7ZC/WHQ9ZKuisFdHtvkNG9m5Fq4tBP+BVPku54TnqY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2 AlexaToolbar/alxg-3.1";
const char S56455[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (/Tt3hBeM8qsUV2uLIAg9pIXy7QvL8IBvhoxZVsTRuJE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56456[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (1HRiFjd9JskKRYL4nyP2KXm139CqELAxiDBxxGEWpEc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56457[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (E2v+q782bYW0z2LhHmbj5nbRRpt5n+NIiUNai9sV+os=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56458[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (JC+r/fEBSdQnnr/4V3OO2w971TESd/38xS1sOuRD9yU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56459[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Abow04eTqUYMazUU2A9MlFSENbcHAe9l2owj1V+oa3I=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56460[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (0o5+iTBLyJBsBjjbQUwsVcmGcuTqU1iX4fERm5FJV0Q=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56461[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (HSuiFJLlNVl1mLogg0+c1vXyQwvXI8WFvdYPp4OpdKE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56462[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (FlFrGmq8hPjvjnvAwYbvDUW1X4sEmLSAWUNGmLsZ6PU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56463[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (AkOL++mrzrNymKlj+LjvrDXGA20mHjXpK2CaEl/GE/8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56464[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (OD/YP4Evvmz5Q3QPJlFM5b7mb8mDdI+b/yED5cWl9KU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56465[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (NMPAH+H2Deh9P/6B3fPHBlUUE8Rb0A/q7exPmi3Ur4Y=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56466[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (IV9Hs5mQaN/R0K+zLm63tnr3ZJNQbeB4Dk1GQQNc2GQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56467[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (GqeZ/RtN8BLyxeYOHOxkaY4KRe8VeSSCcYwSFXqJ5ws=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56468[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (OGxyVI07puJEIltprVOx+nKUtFBW0X1nDLz6GIOU7TM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56469[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (08NM6fWO0spYbbrISyT640jdADTaDrcwt7LJUmRJA84=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56470[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (--USERAGENTOMITTEDOMGVESS--) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56471[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (J3Nh6dtKZAw7Cxyd8WJ4+OrIdcooZ8AJlO4IDvCnkts=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56472[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (M6a+B/nOIHuCUB2BQqmNvoEviZIg+t2ZHrFWHXzK+7g=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56473[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (9D6Sa+VnLKjIy6fis+4kAk0XRz4VCJcB4HW9rPam/mU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56474[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (2fRebjeePe+IThJAi9oQVdnQFS9Kq724/uCmKHxzufY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56475[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (1hU8GbFimod1nJypd07LoGVb4Rfp0XSPzKRPjsmCI24=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56476[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Q3g5Tl3qaAFbveuPIfDyEUoQt4Nq2243JCb/v1x9ZWk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56477[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (JAMkPgy34jtg5uC/PTfQye1NRFZmn+kq6LJvRXI+l8A=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56478[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (1YWAElGXJ0rYPS9OR/1Y8e4X/ZVQbpRu/WZgXxoC+Kc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56479[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (HE/1yDoqWxsCUFZOqUsWhH9FzepmDyjWvn2MfkcgD4Q=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56480[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (6e5YUf22go55jLeYCrxeLl4VBU4hHaD1vd6RkSY0CUE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56481[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (xfMcA+UNukSoKxiQo4oK63DE5u5v+y6FWE3ME12jJoM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56482[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (PAsw5rddaZSz7ApwnjzKfYerpKku+1vEeA33lIeYJrU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56483[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Nnzjz7RTQiSfsQM2njoNSsyxiiWpADCyA+qu2iFqKSo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56484[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (H3Md1XIJnTbE+bu2rA68p1zm1RLiEeeTztPpy0c371E=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56485[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (g3kuz6HvwD1E+pHG1Nwxvc1pwnRrGwY8opYv4mY7fRs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56486[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (fhKsIQ7I0F/5bX2vyefYcXT+g29812oL5z5q7M4WQT8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56487[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (5JwzvCez8rvUtaD9kCenNCNNepcLraRJjgWelsjuYy4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56488[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (g64W3wbVizIE1HlD2HNCBMVgzXLIdbrQexhlzTdIzLw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56489[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (syHhPbBQAJ9JLQbRWYzB34hFtYYmA6BvGnWJ5TZ/ZAg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56490[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (XPokjOYxFHsUdRJ3O2EuYjHwfjohEnE9z01PffnbHEI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56491[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (o8DUVnIvuRUEqRNFk3Vi22zpr6r87h5RilbXlOl5JmI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56492[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (XkKA8LspQ7HAPCh3iWpgQ8bGpHzXRROGeMnxx60F5Mc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56493[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (YjVMepHWYwA48Iw9s1Wt71D3KHixEXQZSSaLao3egQk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56494[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (RO9oj+u2gYTRUfZifbLi9HK8Bk0GiQbZWK6TsLivua8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56495[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (xokYZOXYGKSLMXnZxQkcjAxr4E3a8+0X3xBe/nw/XKA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56496[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (jBSQpmdOVOtqd7mo8YyM6kWhFZK5tDpS4ZMcSkcmGLY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56497[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ZmEufAdvOJ6+jpzS29ugr8sbtxJwbNEwTvj+UjrNdn8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56498[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (rt4R8kudpZt4cdRTDJpP4Dml+DiDYOmChp3nTiVQ9us=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56499[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (vFojr9LdxbndzTN7qN3F42k6KXC7MpQtw5IRLvnZMwU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56500[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (nuS3iPfF+4oBZXsWfuw2wYSgxrH/3AxDEPP29Ubp/qM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56501[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (uehqSy5ycXDRBKP+jbwUI1YPVd1yTBKlCXkDiKHDrj4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56502[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (SOoQfsPbm1lAUwyU4IeKc7/J6f1R/CRljpFkYKaSSUI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56503[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (jggpP0hQtttQ7LSCMvnVZXoDT+E1L1TbRG9xP1kRKfI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56504[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (t7uBc2IWaOPWwcmDKIxzdAhY3o7s1/OSI9TrQHtDr8I=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56505[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (R/8lZYo/R4I9wlH+sCKID7do032Kv5VfatEvYt02tgA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56506[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (402ZbSXLcKvG/yLyMRNi4dNkuf9vKU7noCMU6FN1e2U=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56507[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (XT28UluKEur58RliQm9ffzA9Ed0PI9ZHkAaWZudRZQc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56508[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Yo0INpJdWDNv/vcPYCxyRy1H7gyy40OQW+o58kOLROA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56509[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (seO21gpOtp8uTe8EjZClilM5rgzyrlWqV92C7FPS+dA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56510[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (n0erFnR5T93tQbcRdc/KEZPIAMVtMqew1kqMHu6BIV4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56511[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (cnSggKKdlaO+LI2nd8S44ATEvyTvlMEScWLzZqKu508=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56512[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (qTjnaiLnccJjRdPHGnzmGJn9JEUIy6xlHH8iU6mzucs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56513[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (bTReBGAmyU0p9EZguECS3zebxG/O7ntBasc9XCGRZt4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56514[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (fA7uvGuHN3kN7vCweg7Zs3esLnIVDCJzy7vZPZDzq4w=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56515[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (t5mBoP0o3e1bIQMnWqz/r5moBZD074zuu0ecMlND88c=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56516[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (XbePOaXVC76N7gVPYtb6cDOKV9UmpBN/1T0lsb1CZTc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56517[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (VAGxVr3aE18uG9UGNhxfYQKo8MfRq62rziav6QZT9EQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56518[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (de25zPN9VnTBgaGnFSBrZfvLPthsE6zX+tHYaeeeyFc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56519[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (gzU1iOuRlXSBqvWxux/rXl70e13x6Y73FslOAlL67gg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56520[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (r60KQyMXvoJrV1J2MHgd/PaRSVOMjfujl16LDOj4fWo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56521[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (2STuB04lUCSCz+ci9Sh9LJc6f08JB8Rnvw6U8nyrOYY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56522[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (t8KTdxr3CCQJs0qCrxcYV8EfKDYQAn9KVgD6PdMFVoo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56523[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (jFStrNCKwLCtSNpH0kaAT8gmOTLStyWawS580arwYhU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56524[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (hTcsqC3sfa1UOOX2GKI3MWYCYllaTVDaI1A/hICTkOk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56525[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (z+fvGPOyIDB7AlyW1GAPcCji/Jny6v/TzN9NQqOAIiI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56526[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (zusPZowyBv5A5OSo+4DJD8FSIhUCdGoPm6C4OJ8yMRk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56527[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (xR36Tq9pb/2KNCkXC3Nu1HBrhQMzwBt+vlts4pxjmLs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56528[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (hQvjPP1taKyzkeKiRUPZ3NTlquLiYxP+qELySAzKmOM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56529[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (O7daEPPR2YhzQal6PXK9cr2EkanWTzKcqJRvh58/wgI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56530[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (WglbkfnoMaByhxZ1oJq8mu6cScKsBTVgnaJsO2yb+DY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56531[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (gJLwyb+WNaIFI2tk6C5dBKHyqjk2e2ena4NbpsbRcYI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56532[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (1Hh3oX+JxLNpJl6v5FCKXwE+1kcRk2Xl8l+7ZGPFiSU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56533[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (HtfMH4ztVEkcy+JAR7uq5dxwuGNyDc+BklZPJOWo44o=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56534[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (nnZZUZ+GX+YEpGIGlOJa5427vfm1tUJWFTEPhReqhZs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56535[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (HaeTNjDSX4UdHMWq9aS3jL9drc626A8mXIkMMny+Mso=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56536[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (a0limEdCixWcisKTaGJRl+qgelJUYrCO3lpRT0UZyjI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56537[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (8r8TI2g8Oosihh+RPpg4HTUQ3JsSWh1I6GqrKsFVUvg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56538[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (4zEpNsdsk56kxPCiX/fn/exXDBjxEaJutLKRld+tnsg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56539[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (k2d90rHgrieCxWZ7j7LEwgJtAttkMnZE2P+0KY+tAnI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56540[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (yNfKQihQ3vhQpjkr4G538EERM2gOcht/OpHxMyECjjY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56541[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Vn9aWhTXlKjYxWa1AWKrDjQV4GolAIAA337m5tXcGPo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56542[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (QdGHGwV6DDLxhFQ5VF1VBo6UHURoMyWCYOYFqlg8icA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56543[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (kUTIS1zqXnVReNICAdbwrjEMW5ZfYPu1ee+TBmvCDuk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56544[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (uIrp5lLCuxa/l8XOMZgYRvQmAFNWx7Ra4iNpD1RndyE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56545[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (jtol5VG/jdIaur/ZLzPGeTwP0rwaQ1HlN3YGEguVM0E=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56546[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (CJZKkKysm/LkU+fZlIfebm7a9VNMuopZW8bqizzu1lI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56547[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (iffEFlsDsdqaBuxLflOhK0ZAKZdV93wIQ++3QmooXvY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56548[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (dkywy/UnKmkuNFgW2Aaoh+pQkL2+hgY0KnwsezA9PYg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56549[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (w7A7KBPuA1dSEatlDUAw2vfxuwXIuBUClMa13UCb5OU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56550[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (o+FY3XzStuX79VZqANaMtweaN+NmEO15VbhkMaMkAVg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56551[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (0+Zhtkn1WHTLRgjrx5iFmgjETLJ4PrLntWQjqzfxU00=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56552[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (XhiVp9ukLvJhUkb7XXwouDGXbNA+JHdu5Lm3x79INNY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56553[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (eAzY+kYN0idgz3M1TZsokaJ1cYrypz8oiwQaYqFHcR4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56554[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (JbfcMZkRZLHrEVQewXKvYc7wFwc7MUNRwhQqL1BXcys=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56555[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (x3958fvVfFCU21B657tQahv9dsJWT9nbCCztNA43fKI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56556[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (M3hkW9nUg6Dp8U7nLeE08moI2QB7qI/hkerVZ364Pw0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56557[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (VMipwgxElVUUmp0W1xF5HMIRPfgiOpvnuUFIWGElMiA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56558[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ekLz1M/BXVWgI0h9U0OQLG8D1dQihLpRf5wxUVxHV/g=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56559[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (YJhmyxXWiDJcbjw5ehHYTrZ24rSkO/qPAWVgBYeBRls=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56560[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Yw7CXMh97LNm2m2Avt/kFEczoO2FO5Ti04gpAhzIqSk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56561[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (VrodkL6YAZXTdclpYFNRzqNAzvnqKUGpnrtNEcHQiLY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56562[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (X4CqHbr60EuXxPPS5gLxgDnOSuuM8kkrHItBDazl9qY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56563[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (rxKhWmzaHoxTw/h67C+jnzGPXhvThAo8hQl/0UDc3Fw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56564[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (kQ2Jmk0yidFt8RYmQK5Owzr+Vs2qrPq6WpcUhJZMahc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56565[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (5zWCDjxZBKthbAcjYKkCbjb4p13zjVOwN7wRNmt2tKk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56566[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (fqpxMAp4qobzduuLJD1YDLgCYhpjzwBtd6CAKRgP0SU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56567[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (aQkIWuVoJXWrBe68DMKAc1iw/iekakAOvLpsk0zlkck=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56568[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Z1LEme1Oit3lvLHyrOwekP1UyjSj4OzR7xoPAuHnmfo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56569[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (IOyPaUr1QAPbOxHyJsvuGAzfwFgMiroZPdN7bEvJpas=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56570[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (RbtcTs/JkhUvpTymuH3B9PRX2yQLJLm0x5kl13fSinI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56571[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (QRvfESr+Z96Hj/zLUrdHRgvv+b7YAKJoHlt4Ib2Nhz0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56572[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (TNk5OrHC1cdBwbsxSf9GDYba1ebTTeB0Lv91smUY0BU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56573[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ws5oNBzDH1lFhC4gYNEtj68K0ZiorScjR7mBwJdGHco=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56574[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (YJxLL7kX0A4Or/0URhJklkH+e4kL6zwjoChsP/AYteg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56575[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (tmcE6pNlnWk/bsOM7/ghgQFOf6UC9tGzNg3xgrBPB0k=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56576[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ZzcZ9AYfEOcJ9lhSYF9ZdA3wxO7FaGsplEU1IZWRH/Q=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56577[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (hBr6ukyHIg1eYJTvD8htVXhrUvsfOu89RV5J7bE6yK0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56578[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (rwOnC8kMMVUR72abFcdENgIXAxvJ4PSjR6536Ap0pWg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56579[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (aQz97QYZtsMgm5KZZqvKzxyXvG4yuecbUC+HENETmaM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56580[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (TC0JkyfLsQ88z/VhLsmfJsVs8ox+9y/8X9lg+iiM47E=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56581[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (k1lo7MMfGRzz+H9kX0m1ErWwKP6WX9rAl1+BEAT0HxM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56582[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (SHVSI35bhyK4PTHi6tmODwin+msZjjYMJ0VZJYGDF1U=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56583[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (o8u5P/w47yiT9EAgMQTYbosdzSOTsLwW+Jodpmz0mIg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56584[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (UigEF+mmriU2Mz/7g2pS545JqUmoJHzl1DTo+i/EArY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56585[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (yRb20x6RvBeMpZgTV+JJ41tbBp51y2q/uvIlx+OsQXI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56586[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (c/mSum0BxDohjtRBryehWKdiFpjkT7uOUL5ndY3KN2M=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56587[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ZBySmGBP70JM5ZAqpAdVILhGsw1KxaTJuifqSR7FxyU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56588[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (GMFsA4xDuwpFwtioxE5wjlqKVF18n1xHlxKo0Ys03Jk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56589[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (WKKNEiGPExM+NZzLS3xwOi2UAiCvCuH+EZWhrH36F8w=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56590[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (unmw3fwrmzUnYwK6osi874tmNS4FQEgXz+BM20YXnWQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56591[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (umzQg4lvRq0uV3OeMw9GXajP/1qhhS1umsC93VoFHOc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56592[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (aSsd9DXCDv1IuI9SaZ5OALZTXvXyV+pmJoD2fZ/QEEo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56593[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Z9ayOgYdpkziBpFWAHurb2+NdhUEMkal0Coi48WhYSg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56594[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (5sCy/OHleEkDvQA7JlBWNnGm5LTU5eOLa5zM3ojgyHk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56595[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (qhZpk3sOWiUQtaqqf/xeZbClL8ldmNsFneCI8zp87QY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56596[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (+LXkPnc/5BJU4J4T9RUDaEgikXxIlIJg8343auYBxHM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56597[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (jgyvZY491AZPD+uUM2Pd0i4hLNZWXHX9buI5w8Mi8co=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56598[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (6kbLVUoR17n+3ZtBm/D/QiBKuOGhC7co4TuOtdJOqjs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56599[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (iCKPpRbdNJgnRPe/onFHWYRI+JFtOSS02ggdX5embdg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56600[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (yF8vrzhY/PIrceenqGk9VxD9j4CrunW/Xo4a7cr6gF8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56601[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (QdJ5XIU6jKRGUoJffaSuKszcaZvKV86NGLZyebgBWhs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56602[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (V1ZZpCcb9C9BO5DutYZMhcgpgKXaOyqs7wZF01Vpg+4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56603[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (nYIctuRB/beFyVtOo9EwVcX1/eSHY36Pcc9ZK/F6mzI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56604[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (oSFMNOWJ7mee1o3PNlu10IDyy3flDA9esJ/3OXAKdeg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56605[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (UFaPbkUuaQOsgoDza0t7vrvV7hcA3g9Hj4tSHymtEJw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56606[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (xZ27BdtV8deE4szZoEIQjShfFfQIpNVCawxEKsfokQ4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56607[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (1g6f596zPKJ5ypdiLPyAiRnz+JdQJLbtqC+5Mgi+wFk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56608[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (rqZhywrLlRmI6qan4QYva0VIv9pNu29JPwgQ3VhVnlU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56609[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (z6t3OSr4heg/B3fj1rYFX4jH3ShUYvzxcuq0pNSPAq8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56610[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (7lvLBTtP67zeagboNKTYLkipG7bSruoFQr2Wz3nsLWc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56611[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (cVXDIxup5VPssh1LjEq2w4bQxzKLIVWsBSNmPTkYZyA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56612[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (knZfiemMM6RyO9NlESomR2atx1NZTWArdUMoQO2D2wA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56613[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ku9IcPmDzDD7Jto9RmYJJ9+wxVwgQdfYf3pUtTJK1F8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56614[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Wo5/LgfN39aUMbvU8NGie8UFL7Zf9lAv6UpvjEHn6t8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56615[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (meNfYiy6I3ppmw9xO1YdLt/65DtxIg6HxfjVaeSNgTo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56616[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (u2Z4xCgkXKKJgGda3H6R1VbXZMkH+zXH5YdYURonh/0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56617[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (yMcNLr07ESmuK0cFsyM3Zd7Gf2T+bweeGAlc6jXz/Zc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56618[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (lWfaUqAQOJMuqvzUqT9VuXt9zmTPlhfNotkj/HLjPrs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56619[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (XT2dowopqZqR8BylfK4xdBjMa4skVqLx431RySaPQbQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56620[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (aNh9Fm6zp7LXk9aXx1RHU2fQrtgv+YzFi6fW1JfM7H4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56621[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (hxaQY8SX8wbwE43zLRqXWDvqJXsJM89gNxRqWy/TxKI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56622[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (VKixsQPwnrpBozTTnkCTM3GrVtsL7a+mT9Gyk8jkqcY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56623[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (RxBH3wYjb6Sly1R+1xH14F0wxWgTxcLI3INNl2shFOg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56624[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (pFriUtpOaEm9C4moKeOBbGau7kSnh412mKWf7QG8E1s=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56625[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (edrtrjeUeJ/HBNh0MpeJ6QRATQPQ5zSVkFZ8BtDcquI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56626[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (mzGgN/HzNhhgcvWYTn7WA7LyCbALi8EHImgUE1eIqNw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56627[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (UnvLVuRInG8yeNZq/pppEmui6kHjzlGTb/58O/trQQ0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56628[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (iV5YOqAYc6rvNVSKpp0NBNV91yqFt1Mx6Fm4wk6SnnE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56629[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (hTEc1eA2BcEa3yh/c2Qxi8jOWzcdaD5lI+mvSto3zsc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56630[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (321hm9c0TRNP7JOZEHRzLLxKdfRoD3NHAZ+x3kmS3YE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56631[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ef5l5ZXucdFpSpM5gtvZHnL6gwLIdKahXdXzPyufCrI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56632[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (zfVFJuVlanQy+j7b5igmxdrQBuM7tLLoyB/15AnGSbI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56633[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (dvOQjkz7JIYiNMyng/VswHoLQitlbOzxfgkwaqISpFE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56634[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (sQYjERz1ubvf65h3weMYRRjvhbcmcr6ag/WtrJyeXrs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56635[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (u9QzCRtRKC6sc2ZwyEgxahuVpobPf1KT8Tv30x2xIaw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56636[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (0ThAtahPUpxeDvmtAfpu7xqmR3Wwga7KCQ53WWxOJ6M=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56637[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (W/ZfIKAvgoX0iKDoanNFSQ4ACAdqbGggBE6bPCMbXZw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56638[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (guASzxOkmpjI9kKu7RwJpgxQiJ3TYboX8iK1bzYJd8o=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56639[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (WycBmyeI9G4eXQcHsrILjwZzEi3rHMPr4R2UQIpjw6o=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56640[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (bym/xt2RJ8DKOe4+SZX6g1RU7uZ2HTr+ZBZvREFub6Y=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56641[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (qssOqQLCTdajZgLd7qBK74liefWIj4LdnbH4gdLZwlk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56642[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (h2Fzqc2mOkW+AbtBovdqFFsqJpddHF3tTZH4n1TpMv8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56643[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (feJvHBLFG/ji20JUMOg/s+nAZZGPVC0N55W+RVEOHGY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56644[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (wOEMDNSLp57AsSjYXxyOpqo9t30vj8mQCFTAiZ/0lrs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56645[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (UK6V7Py2kcVw/dnKbuymGiNThXGxPoNf0MfUd5E0K5Y=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56646[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (XslUDt22kn7C4dJjfK0MegbfsuCzRALbq2DN4Ak8n7Q=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56647[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (mmq2AxmpN6mTMn6dp5NSJ7jB+K+UrXVCF1weHJXwOKA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56648[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ylmEa52W207SG1GnYzMlpW/W6ng1RFzD+I7wxcuPKuQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56649[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ydp6PmWhcHtHxSicI9MrDBi3hJ4bWjQvsJnmTiFeosM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56650[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (JOTkjMjEbQDYvULeoLEl5PRAXJ0sWDZI1gpvWEufgiY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56651[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (jwe5xXmWM+FABSn/IX52P5t787xuw1HOrpnGICks5Ig=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56652[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (jwD2y6YFkctosT5FELDx07yy1LfQUWIvcaRaHNV882U=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56653[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (n16iuKG0oiIZnNtFgMy/mOWoqu1fpgvKVGcJqPYizhk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56654[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (wtOEhI1rrZiiNRw5yqCC7E77WKPb191krsNJXz4Ewns=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56655[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (nk+7B0vEnuz9VvsyAkLTldkqNT8gyT9s2jnHz5TJZ5Q=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56656[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (VeMx9Kg9TSc6N+NLIDDJQk+yAs2Jc88hDOAWC581Q9s=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56657[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (YfGJjXRFhbCAoZyY/ZjJxzMXEnDXZ8i/UufR5exOpDI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56658[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (i4DjSJEBwdLJDDtV37i4IpGJXfG7l1GcnLuPNHSOyBQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56659[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (kSamzH/nGjnKspyMoq+MPnFVFkSdI+FUde17GDT6ReQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56660[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (vHypc+b66opo5xuvH61R/KYC1CgrxNFSSnAMWuJR+Dc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56661[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Qs9BpIh4b7R5/ukfRa6di+OpQmwuF9qP8XEjo8ajS9E=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56662[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (g6nvFfchIynZi8ANz6XPf+CbzWUMVLJqJcEp+hN8JyU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56663[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (p5usOvJvGacb2+FJcN3rghOYHJYwhX+I17yMH4Yx6Vw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56664[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (sEs/QgeTfo2EUcNTXfMGw96mktdIjjns6eMcXPzoLLA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56665[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (f3ps+xVnjEglJhFnHdOHBktKl4Qxxl+w8rUrkP4Rzbs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56666[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (VO3Cbt53bR4mF1f/KXCifMsVISUWSMiP5HnS+7W7Y7I=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56667[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (TvD5+NJC4hh3xHHnjJnpt8Qe1olxdCk8mJQ9D+lsjOI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56668[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Uqh8UtattBH59jvLUNGh4NMNm8n3kbZ4BJkSZdeOWg4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56669[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Wrc3hJH1W1SeI011OGGbYem0vXXKezLwAYmzDIcEEsA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56670[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ryJBKeu+uiLV4QGqbHOToZWHPvVlbJJXgguWpuINYr0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56671[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (iX6d3X3xDayNQLOZwAQiEIMEp4QZYD9P6BQ9jzQ4ZeM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56672[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (a5awpctAWy8CBe3R9TQKINjJZ7kwi/vK+qc8jRZTTiY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56673[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (MsPsoLwb2Ir3YkLDG0+RDOlxNCEeQUSUQHcS7+G7Tlo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56674[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (iEd5/mrBMp3jquOCBoKF4chHs0LdCG9clqZ+PMIDmu8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56675[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (FoXF9MbUulGnnbL+qVXRpDt9NfLHG5jgsokcwJDkvqc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56676[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Y6J0A4AypvhLrYtEDuplyc/ynkWaSb7/uFFTeDesFfw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56677[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (+vFlTvo3odzKW4iJxF+oWfftiUgjiSVun4xBZ+hcJ4g=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56678[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (kZs/CGP3zxi2UauiBWnApUogcOo8hj2GUJ9Y4VZYek0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56679[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (sPHxgDKpO8H7xz7UJx+863KHtKvj0XxHSH5oNWkOt+Y=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56680[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (TfYrHRyrSD4Kn2M0+GHEmoJTgX6OLxy4kW1ZuuQLIDM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56681[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (xgdGEK/fdvz2DGSE6Cf120dXoeFmqXKs/HQR+fraLIg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56682[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (agP4pt2yF6VEAl/KmMDs3f1CXhclqRUfJNuiR+MRv7g=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56683[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ihqEMjN167zr2GmPJWGvZCFgJH/tSQBYSmR17H/gkyA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56684[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Vy2KMyWEwGGvp3cl/rtuqCv3GP+HgkVPsX958DCddZQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56685[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (eo6cGIfmNqYAHrPM09mzURU8F/OcgdJuC+LA96Z73uY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56686[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (SZeodRuzUSRiq9cM5cnj8UCpB1uBuIiqyXA6fqD+ieI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56687[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (SPU+RE2VQzssJ5t5NWWLsxt4x06naEd0rksRNlcG7jI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56688[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (dUMw/7yTgSAJ/xiEUvTtzrkGWfo5Xctzi8sRdllzPpw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56689[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (3Qfs5yafqd3YgrMNJbg2WHLWmXoeGjurTwEsdjqo1mo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56690[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (hILhIG4TGDutA2R4AllG0WfhAxA9J0OdjZGDgjTAvWA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56691[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (UPtiktRoWcoIMGlKY3cW/HdQR+kiog1z4OVVSu03eaw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56692[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (SCUcFEIJSvh5LwRRoCzbzZDWYDIFqr3QsfHM7TATKlM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56693[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (sml52qM/QbK84BElcdbkmKv9EsJ3w8gorvqi+VXblNA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56694[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (LtGWyc+Jvdn7blfLvV1xn3l/Ak60aKawjq/pNVDPrZ0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56695[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (5b63Fxx1SW/mYX4ICILj5mcuri4RsZCX0NwYVs4/p2Y=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56696[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (sKBlqb3ROanUFyNonRs/B7i7iApx2iN1aOE8r0n9ULY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56697[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ztiL7JhKyXAD2V+3UOUlRJoNKMoCLO9o3ualW0A3AyI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56698[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (qsClFhjoEOQSmG/K9NJNCM0QWiAyyK4u3mc1rIinWEg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56699[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (31JHh2fQ2msj2U0nKQqQqjMuJr1f6XDxBpzeELAc++o=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S56700[] = "Mozilla/5.0 (Linux; Android 4.0.3;  Transformer Prime TF201 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S56701[] = "Mozilla/5.0 (Linux; Android 4.0.3; Transformer Prime TF201 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S56702[] = "Mozilla/5.0 (Linux; Android 4.0.4; Transformer Prime TF201 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S56703[] = "Mozilla/5.0 (Linux; Android 4.0.3; TF201 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S56704[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID1045 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S56705[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; MID1045 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S56706[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; LG; LG-E900)";
const char S56707[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; LG; LG-E900; Orange)";
const char S56708[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; LG; LG-E900h)";
const char S56709[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0) LG;LG-E900";
const char S56710[] = "Mozilla/5.0 (Linux; Android 4.1.1; Xoom Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S56711[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xoom Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S56712[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Mobile Safari/535.19 Silk-Accelerated=true";
const char S56713[] = "Mozilla/5.0 (PlayStation Vita 2.01) AppleWebKit/536.26 (KHTML, like Gecko) Silk/3.2";
const char S56714[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Desire HD Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56715[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Desire HD Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56716[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; Desire HD Build/JOP40D)";
const char S56717[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Desire HD Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56718[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; Desire HD Build/JDQ39)";
const char S56719[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Desire HD Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56720[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S56721[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S56722[] = "Mozilla/5.0 (Linux; Android 4.1.1; SPH-D710 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S56723[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710VMUB Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S56724[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710BST Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S56725[] = "Mozilla/5.0 (X11; Linux) AppleWebKit/535.22 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/535.22 Midori/0.5";
const char S56726[] = "Mozilla/5.0 (X11; Linux) AppleWebKit/537.6+ Midori/0.5";
const char S56727[] = "Mozilla/5.0 (X11; Linux; rv:2.0.1) Gecko/X Firefox/4.0.1 Midori/0.5";
const char S56728[] = "Opera/9.80 (Linux armv7l; Opera Mobi/1; MeeGo) Presto/2.11.355 Version/12.10";
const char S56729[] = "Opera/9.80 (Linux i686; Opera Mobi/46154; MeeGo) Presto/2.11.355 Version/12.10";
const char S56730[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MID7015 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56731[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MID7015A Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56732[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; MID7015 Build/ECLAIR)";
const char S56733[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; MID7015A Build/ECLAIR)";
const char S56734[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; MID7015A Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56735[] = "Vyrso/0.9.3 (Linux; U; Android 2.1-update1; en_US; MID7015 Build/MID7015-eng 2.1-update1 ECLAIR eng.enzo.X.X test-keys)";
const char S56736[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5; U; Edition MAS; en) Presto/2.10.229 Version/11.64";
const char S56737[] = "Mozilla/5.0 (Windows; U; Windows NT   5.1; en-US; rv:1.8) Gecko/X SeaMonkey/1.0b";
const char S56738[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.0.4) Gecko/X SeaMonkey/1.0.2";
const char S56739[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; pandigitalopp12/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S56740[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:9.0) Gecko/X Firefox/9.0";
const char S56741[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:9.0.1) Gecko/X Firefox/9.0.1 Firefox/9.0.1";
const char S56742[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Wildfire Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56743[] = "NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiax2-01) UCBrowser8.4.0.159/70/352/UCWEB Mobile";
const char S56744[] = "Nokia5130c-2/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5130c-2) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S56745[] = "Nokia5130/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5130) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S56746[] = "MQQBrowser/40 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 Safari/7534.48.3";
const char S56747[] = "MQQBrowser/40 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Safari/7534.48.3";
const char S56748[] = "MQQBrowser/40 Mozilla/5.0 (iPhone 4; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Safari/7534.48.3";
const char S56749[] = "MQQBrowser/40 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3";
const char S56750[] = "MQQBrowser/40 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145 Safari/7534.48.3";
const char S56751[] = "MQQBrowser/40 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3";
const char S56752[] = "MQQBrowser/40 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3";
const char S56753[] = "Mozilla/5.0 (en-us) AppleWebKit/525.13 (KHTML, like Gecko; Google Wireless Transcoder) Version/3.1 Safari/525.13";
const char S56754[] = "Mozilla/5.0 (SaferSurf) AppleWebKit/525.19 (KHTML, like Gecko) Version/3.1.2 Safari/525.21";
const char S56755[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; A701 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S56756[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; A701 Build/JRO03H)";
const char S56757[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; sdk Build/JB_MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56758[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; sdk Build/JB_MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S56759[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; google_sdk Build/JB_MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56760[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; sdk Build/JB_MR1.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56761[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; sdk Build/JB_MR1.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S56762[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; google_sdk Build/JB_MR1.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56763[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; sdk Build/JB_MR1.1)";
const char S56764[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; rk31sdk Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S56765[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 BaDoinkDownloader_Android_2.0";
const char S56766[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; rk30sdk Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S56767[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LG730 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S56768[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LG730 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S56769[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; C2105 Build/15.0.A.0.33) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S56770[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C2105 Build/15.0.A.1.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56771[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; SonyC2105 Build/15.0.A.1.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56772[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C2105 Build/15.0.A.1.36) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56773[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; C2105 Build/15.0.A.2.17)";
const char S56774[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C2105 Build/15.0.A.2.17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56775[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; SonyC2105 Build/15.0.A.2.17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56776[] = "Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E300S Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S56777[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AT270 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S56778[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; AT270 Build/IML74K)";
const char S56779[] = "Mozilla/5.0 (Linux; Android 4.1.1; Transformer TF101 Build/MASTER) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S56780[] = "Mozilla/5.0 (Linux; Android 4.1.1; Transformer Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S56781[] = "Mozilla/5.0 (Linux; Android 4.1.2; Transformer TF101 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S56782[] = "Mozilla/5.0 (Linux; Android 4.1.2; Transformer Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S56783[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS840 4G Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S56784[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56785[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P710 Build/JZO54K)";
const char S56786[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-P710/P71010a-VDF-ES Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56787[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P710/V10a Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56788[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P710/P71010d-EUR-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56789[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P710/V10c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56790[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P710/P71010e-EUR-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56791[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-P710/P71010b-VDF-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56792[] = "Mozilla/5.0 (Linux; Android 4.0.3; Sony Tablet S Build/TISU0143) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S56793[] = "Mozilla/5.0 (Series40; Nokia300/07.57; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S56794[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8F190 Safari/7534.48.3";
const char S56795[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8J2 Safari/7534.48.3";
const char S56796[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8L1 Safari/7534.48.3";
const char S56797[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8H7 Safari/7534.48.3";
const char S56798[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8G4 Safari/7534.48.3";
const char S56799[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8K2 Safari/7534.48.3";
const char S56800[] = "SonyEricssonU8i/R2EA; Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525";
const char S56801[] = "Mozilla/5.0 (Linux; Android 4.2; en-US; SONY XperiaTablet Build/JOP22F) AppleWebKit/535.19 (KHTML like Gecko) Chrome/18.0.1025.464 Safari/535.19";
const char S56802[] = "Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Safari/535.19 Silk-Accelerated=false";
const char S56803[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I717 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56804[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I717 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56805[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56806[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56807[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S56808[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Micromax A35 Build/IML74K) AppleWebKit/534.30(KHTML,like Gecko) 			Version/4.0 Mobile Safari/534.30";
const char S56809[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Micromax A35 Build/MocorDroid2.3.5)";
const char S56810[] = "Mozilla/5.0 (Linux; Android 4.1.1; C5302 Build/12.0.A.0.224) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56811[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5302 Build/12.0.A.1.211) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56812[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E970 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S56813[] = "PantechP2030/JSUS06132011; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.22250; U; en-US) Opera 9.50";
const char S56814[] = "PantechP2030/JSUSX; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.22250; U; es) Opera 9.50";
const char S56815[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en) AppleWebKit/522.11.3 (KHTML, like Gecko) Version/3.0 Safari/522.11.3";
const char S56816[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/523.15 (KHTML, like Gecko) Version/3.0 Safari/523.15";
const char S56817[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en) AppleWebKit/522.15.5 (KHTML, like Gecko) Version/3.0.3 Safari/522.15.5";
const char S56818[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en) AppleWebKit/522.12.1 (KHTML, like Gecko) Version/3.0.1 Safari/522.12.2";
const char S56819[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; nl) AppleWebKit/522.13.1 (KHTML, like Gecko) Version/3.0.2 Safari/522.13.1";
const char S56820[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en) AppleWebKit/525+ (KHTML, like Gecko) Version/3.0.4 Safari/523.11";
const char S56821[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; da-DK) AppleWebKit/523.12.9 (KHTML, like Gecko) Version/3.0 Safari/523.12.9";
const char S56822[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; sv-SE) AppleWebKit/523.13 (KHTML, like Gecko) Version/3.0 Safari/523.13";
const char S56823[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ZTE-Z992 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56824[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Z992 Build/JRO03C)";
const char S56825[] = "Mozilla/5.0 (Linux; Android 4.0.3;  HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S56826[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56827[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315b Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56828[] = "Mozilla/5.0 (Linux; Android 4.0.3; Sensation XL with Beats Audio Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56829[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-P708g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56830[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-P705 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56831[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LG-P705g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56832[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P705 Build/JZO54K)";
const char S56833[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P708g Build/JZO54K)";
const char S56834[] = "Mozilla/5.0 (Linux; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S56835[] = "Mozilla/5.0 (Linux; Android 4.0.4; ME865 Build/6.7.2_GC-220-EDS-3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S56836[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S56837[] = "Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I727 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S56838[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.10; SV1)";
const char S56839[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.10; Trident/4.0)";
const char S56840[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.10; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S56841[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.10; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)";
const char S56842[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.10; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S56843[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia6760s-1/03.45; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S56844[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; HTC Incredible S Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56845[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC_IncredibleS_S710e Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56846[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-tw; HTC_IncredibleS_S710e Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56847[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; HTC Incredible S Build/FRG83D)";
const char S56848[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; HTC Incredible S Build/FRG83D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S56849[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; HTC Incredible S Build/FRG83D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S56850[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S56851[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P500 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56852[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; LG-P500 Build/GRI40; NightlyFourE Based CyanogenMod 7 Nightly) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56853[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S56854[] = "Mozilla/5.0 (Series40; Nokia301.1/02.03b; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.48";
const char S56855[] = "Mozilla/5.0 (Series40; Nokia301/02.33; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.54";
const char S56856[] = "Mozilla/5.0 (Series40; Nokia301.2/02.51; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.55";
const char S56857[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZP900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56858[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZP900S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56859[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ZP900 Build/IMM76D)";
const char S56860[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT897 Build/7.7.1Q-6_SPR-125_ASA-14) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S56861[] = "Mozilla/5.0 (Windows NT 5.0; rv:19.0) Gecko/X Firefox/19.0";
const char S56862[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S56863[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC_PN071 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S56864[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56865[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562L Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56866[] = "Mozilla/5.0 (Linux; Android 4.0.3; A200 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S56867[] = "NokiaC2-00/2.0 (03.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-00) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S56868[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S56869[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus 7 Build/JRO03S) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S56870[] = "Mozilla/5.0 (Linux; Android 4.1; Nexus 7 Build/JRN84D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S56871[] = "Mozilla/5.0 (Series40; Nokia113/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.5.16.9";
const char S56872[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTCEVOV4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S56873[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EM543 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56874[] = "Mozilla/5.0 (Windows NT 4.10) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S56875[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R740C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S56876[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB525 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56877[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MB525 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56878[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MB525 Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S56879[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; MB525 Build/MIUI Es) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56880[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; MB525 Build/MIUI Es) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56881[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; MB525 Build/MIUI)";
const char S56882[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MB525 Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S56883[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MB525 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56884[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; MB525 Build/MIUI Es) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56885[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ME525 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56886[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ME525+ Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56887[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ME525 Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S56888[] = "NOKIA6120cUCBrowser/8.7.1.234/28/352/UCWEB";
const char S56889[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA6120c) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S56890[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6120c) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S56891[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I777 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S56892[] = "Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I777 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S56893[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I777 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S56894[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-I777 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S56895[] = "SAMSUNG-SGH-T469V/T469VTJH1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S56896[] = "SAMSUNG-SGH-T469W/T469UQIK5 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S56897[] = "SAMSUNG-SGH-T469/T469UVIG4 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S56898[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SHW-M190S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56899[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M190S Build/GINGERBREAD)";
const char S56900[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M190S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56901[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHW-M190S Build/GINGERBREAD)";
const char S56902[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PMID701C Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56903[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; LG-F100L Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56904[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-F100L Build/GRK39F)";
const char S56905[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:14.0) Gecko/X Firefox/14.0.1 SeaMonkey/2.11";
const char S56906[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:14.0) Gecko/X SeaMonkey/2.11";
const char S56907[] = "Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5130c-2) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S56908[] = "Nokia5130/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5130) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S56909[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21i Build/11.0.A.4.22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S56910[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21i2 Build/11.0.A.6.5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S56911[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21a2 Build/11.0.A.1.8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S56912[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21a Build/11.0.A.0.16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S56913[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; MWP6985)";
const char S56914[] = "Mozilla/5.0 (compatible; Konqueror/3.5; Linux) KHTML/3.5.5 (like Gecko) (Kubuntu)";
const char S56915[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.8.0.4) Gecko/X Camino/1.0.2";
const char S56916[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-US; rv:1.8) Gecko/X Camino/1.0b1";
const char S56917[] = "Mozilla/5.0 (Linux; Android 4.0.3;  HTC Desire HD Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S56918[] = "Mozilla/5.0 (Linux; Android 4.0.4; Desire HD Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56919[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire HD Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56920[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire HD Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56921[] = "Mozilla/5.0 (Linux; Android 4.0.3; Desire HD Build/HuaweiU8815) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S56922[] = "BlackBerry8800/4.5.0.69 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/123";
const char S56923[] = "iTunes-iPod/4.3.3 (3; 64GB)";
const char S56924[] = "iTunes-iPod/5.0.1 (4; 32GB)";
const char S56925[] = "iTunes-iPod/4.2.1 (2; 8GB)";
const char S56926[] = "iTunes-iPod/4.3.5 (4; 32GB)";
const char S56927[] = "iTunes-iPod/3.1.3 (2)";
const char S56928[] = "iTunes-iPod/4.2.1 (4; 8GB)";
const char S56929[] = "iTunes-iPod/4.1 (4; 64GB)";
const char S56930[] = "iTunes-iPod/4.3.5 (3; 32GB)";
const char S56931[] = "iTunes-iPod/4.2.1 (2; 16GB)";
const char S56932[] = "iTunes-iPod/5.0.1 (4; 8GB)";
const char S56933[] = "iTunes-iPod/4.3.5 (4; 8GB)";
const char S56934[] = "iTunes-iPod/4.2.1 (2; 32GB)";
const char S56935[] = "iTunes-iPod/4.1 (4; 8GB)";
const char S56936[] = "iTunes-iPod/3.1.3";
const char S56937[] = "iTunes-iPod/5.0.1 (3; 32GB)";
const char S56938[] = "iTunes-iPod/5.1.1 (4; 32GB; dt:71)";
const char S56939[] = "iTunes-iPod/5.1.1 (3; 32GB; dt:25)";
const char S56940[] = "iTunes-iPod/5.1.1 (4; 8GB; dt:71)";
const char S56941[] = "iTunes-iPod/5.1.1 (4; 64GB; dt:71)";
const char S56942[] = "iTunes-iPod/5.1.1 (3; 64GB; dt:25)";
const char S56943[] = "iTunes-iPod/4.0 (2; 8GB)";
const char S56944[] = "iTunes-iPod/4.1 (2; 8GB)";
const char S56945[] = "iTunes-iPod/6.0 (4; 32GB; dt:71)";
const char S56946[] = "iTunes-iPod/4.1 (2; 32GB)";
const char S56947[] = "iTunes-iPod/4.1 (4; 32GB)";
const char S56948[] = "iTunes-iPod/4.2.1 (4; 32GB)";
const char S56949[] = "iTunes-iPod/6.0.1 (4; 8GB; dt:71)";
const char S56950[] = "iTunes-iPod/5.0.1 (4; 64GB)";
const char S56951[] = "iTunes-iPod/6.0.1 (4; 32GB; dt:81)";
const char S56952[] = "iTunes-iPod/6.0.1 (4; 16GB; dt:71)";
const char S56953[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; T-Mobile Vivacity Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56954[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; T-Mobile Vivacity Build/GRJ22)";
const char S56955[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Vivacity Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56956[] = "Lynx";
const char S56957[] = "FavOrg";
const char S56958[] = "RSS";
const char S56959[] = "VisuRss";
const char S56960[] = "Feedbin";
const char S56961[] = "Mozilla/5.0 (Windows NT 6.0; rv:16.0) Gecko/X Thunderbird/16.0.2";
const char S56962[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LVW Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S56963[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR6410LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S56964[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LRA Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S56965[] = "Mozilla/5.0 (Linux; Android 4.0.4; Hydro Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S56966[] = "Mozilla/5.0 (Linux; Android 4.0.4; C5170 Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S56967[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56968[] = "Mozilla/5.0 (Linux; U; Android 4.0.0; en-us; I9500 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56969[] = "GT-I9500 Linux/3.0.13 Android/4.2.2 Release/07.22.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S56970[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MID7024 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56971[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-es; CFW-MID7024 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56972[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; MID7024 Build/FRF91)";
const char S56973[] = "Logos/0.9.3 (Linux; U; Android 2.2; en_US; MID7024 Build/MID Serials 2.2 FRF91 X.X)";
const char S56974[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Next7S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S56975[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337M Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S56976[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337M Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S56977[] = "Mozilla/5.0 (Linux; Android 4.2; Galaxy Nexus Build/IMM96K) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/30.0.1207.1 Mobile Safari/8536.25";
const char S56978[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S56979[] = "Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S56980[] = "Mozilla/5.0 (Series40; Nokia210/04.12; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.53";
const char S56981[] = "Mozilla/5.0 (Series40; Nokia210.2/04.12; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.53";
const char S56982[] = "Mozilla/5.0 (Series40; Nokia210.4/04.11; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.53";
const char S56983[] = "Mozilla/5.0 (Series40; Nokia210.5/04.11; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.53";
const char S56984[] = "Mozilla/5.0 (Series40; Nokia210.3/04.12; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.53";
const char S56985[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; BLU DASH 3.5 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56986[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; BMOBILE DASH 3.5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56987[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; BLU DASH 3.5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56988[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; BLU DASH 3.5 VIVA Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56989[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; DASH 3.5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S56990[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S56991[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562L Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S56992[] = "Mozilla/5.0 (Linux; Android 4.1.1; C1505 Build/11.3.A.2.13) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S56993[] = "Mozilla/5.0 (X11; Linux armv7l; rv:7.0.1; HERCULES; EC-SLIM-EX) Gecko/X Firefox/7.0.1";
const char S56994[] = "Mozilla/5.0 (X11; Linux ppc; rv:7.0) Gecko/X Firefox/7.0 Iceweasel/7.0";
const char S56995[] = "Mozilla/5.0 (X11; Linux i686; rv:7.0.1) Gecko/X Firefox/7.0.1 Firefox/7.0.1";
const char S56996[] = "Mozilla/5.0X11Linuxi686rv7.0.1Gecko/XFirefox/7.0.1";
const char S56997[] = "Mozilla/5.0 (X11; Linux i686; rv:7.0.1) Gecko/X Abrowser/7.0.1";
const char S56998[] = "Mozilla/5.0 (Windows NT 4.0; rv:16.0) Gecko/X Firefox/16.0";
const char S56999[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30at Build/9.1.F.1.120) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57000[] = "LCT_L300B_01/L3PM510/WAP2.0 Profile";
const char S57001[] = "PM/3.3.5";
const char S57002[] = "Mozilla/4.0 (MobilePhone PM-8200/US/1.0) NetFront/3.x MMP/2.0";
const char S57003[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.2.3) Gecko/X Namoroka/3.6.3";
const char S57004[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko)";
const char S57005[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:12.0) Gecko/ /12.0";
const char S57006[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/533+ (KHTML, like Gecko)";
const char S57007[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.57.2 (KHTML, like Gecko)";
const char S57008[] = "Mozilla/4.0 (Windows 8 6.2) Java/1.7.0_25";
const char S57009[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S57010[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Ubuntu/10.04 Chromium/9.0.597.94 Chrome/9.0.597.94 Safari/534.13";
const char S57011[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Ubuntu/10.04 Chromium/9.0.595.0 Chrome/9.0.595.0 Safari/534.13";
const char S57012[] = "Mozilla/5.0 (X11; U; Linux armv7l; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Ubuntu/10.04 Chromium/9.0.597.94 Chrome/9.0.597.94 Safari/534.13";
const char S57013[] = "HUAWEI-M570/001.00 WAP/OBIGO/Q05A";
const char S57014[] = "NokiaX2-02/2.0 (11.79) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-02) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S57015[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-T959 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57016[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-T959 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57017[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T959 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57018[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T959 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57019[] = "NokiaX2-02/2.0 (11.79) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-02) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S57020[] = "SAMSUNG-SGH-T379/1.0 SHP/VPP/R5 NetFront/4.2 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S57021[] = "NetFront/3.5.1(BREW 3.1.5; U; en-us; SAMSUNG; NetFront/3.1.5/WAP) Sprint M380 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S57022[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/9.5/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S57023[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaC5-05) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S57024[] = "NokiaC5-05UCBrowser/8.7.1.234/50/444/UCWEB";
const char S57025[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8) Presto/2.12.388 Version/12.12";
const char S57026[] = "Opera/9.80 (Android 2.2; Linux; Opera Mobi/ADR-X; U; es-ES) Presto/2.10.254 Version/12.00";
const char S57027[] = "Opera/9.80 (Android 2.2.3; Linux; Opera Mobi/ADR-X; U; en) Presto/2.10.254 Version/12.00";
const char S57028[] = "Opera/9.80 (Android 2.2.1; Linux; Opera Mobi/ADR-X; U; pl) Presto/2.10.254 Version/12.00";
const char S57029[] = "Opera/9.80 (Android 2.2.2; Linux; Opera Mobi/ADR-X; U; pl) Presto/2.10.254 Version/12.00";
const char S57030[] = "LGE-MN240/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S57031[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-04E Build/V10R27E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57032[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-04E Build/V11R29D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57033[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-04E Build/V13R30C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57034[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-04E Build/V14R31C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57035[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; F-04E Build/V14R31C)";
const char S57036[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-04E Build/V15R32D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57037[] = "Mozilla/5.0 (Windows NT 6.3; Windows NT 6.3; Win64; x64; Trident/7.0; rv:11.0) like Gecko/X Firefox/12.0";
const char S57038[] = "Mozilla/5.0 (Windows NT 6.3; Windows NT 6.3; Trident/7.0; rv:11.0) like Gecko/X Firefox/12.0";
const char S57039[] = "Mozilla/5.0 (Windows NT 6.3; Win64; x64; Trident/7.0; rv:11.0) like Gecko/X Firefox/12.0";
const char S57040[] = "Mozilla/5.0 (Windows NT 6.3; Trident/7.0; rv:11.0) like Gecko/X Firefox/12.0";
const char S57041[] = "Mozilla/5.0 (Windows NT 5.0; rv:18.0) Gecko/X Firefox/18.0";
const char S57042[] = "Mozilla/5.0 (Linux; Android 4.0.3; Build/JWQ87D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.49 Mobile Safari/537.31";
const char S57043[] = "Mozilla/5.0 (Linux; Android 4.0.4; TWD_MID Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57044[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S57045[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57046[] = "Mozilla/5.0 (Linux; Android 4.0.4; INFINITY_POWER Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57047[] = "Mozilla/5.0 (Linux; Android 4.0.4; Pad Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57048[] = "Mozilla/5.0 (Linux; Android 4.0.4; Tender Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57049[] = "Mozilla/5.0 (Linux; Android 4.0.4; Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57050[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbl1 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57051[] = "Mozilla/5.0 (Linux; Android 4.0.3; Build/X) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S57052[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID Serials Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S57053[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1808_v75_jbl2 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57054[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1809l_v75_gq1008_ov5647_v20 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57055[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1808_v75_jbl1_5640 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57056[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57057[] = "Mozilla/5.0 (Linux; Android 4.0.4; Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S57058[] = "Mozilla/5.0 (Linux; Android 4.0.9; e1909c_v77_gq1008_9p017_6228 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57059[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1808_v75_jbl2_6628 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57060[] = "Mozilla/5.0 (Linux; Android 4.0.3; V1013 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S57061[] = "Mozilla/5.0 (Linux; Android 4.0.4; EBM727KC Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S57062[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S57063[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbaol1 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57064[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S57065[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus S Build/JRO03E) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S57066[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S57067[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS870 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57068[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS870 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S57069[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P970 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57070[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P970h Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57071[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P970g Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57072[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lenco CoolTab-70 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57073[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5303 Build/12.0.A.1.257) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57074[] = "Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia305) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S57075[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A1+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57076[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; A1+ Build/MocorDroid2.3.5)";
const char S57077[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 928)";
const char S57078[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGPT13 Build/TJS0166) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S57079[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGPT12 Build/TJS0166) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S57080[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One V Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57081[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One V Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57082[] = "Nokia 6680";
const char S57083[] = "Nokia6680/1.0 (4.04.07) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0Configuration/CLDC-1.1, Nokia6680/1.0 (4.04.07) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0Configuration/CLDC-1.1";
const char S57084[] = "Nokia6680/1.0 (4.04.07) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S57085[] = "Nokia6680";
const char S57086[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HW-T18D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57087[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HW-T18 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57088[] = "Mozilla/5.0 (Linux; Android 4.0.3; Build/JWR11G) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.49 Mobile Safari/537.36";
const char S57089[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S57090[] = "Mozilla/5.0 (Linux; Android 4.0.4; TWD_MID Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57091[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57092[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57093[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1808_v75_jbl2 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57094[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbl1 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57095[] = "Mozilla/5.0 (Linux; Android 4.0.4; Pad Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57096[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1808_v75_jbl1_5640 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57097[] = "Mozilla/5.0 (Linux; Android 4.0.4; EBM727KC Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S57098[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1809l_v75_gq1008_ov5647_v20 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57099[] = "Mozilla/5.0 (Linux; Android 4.0.3; Build/ICS.g04ref.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S57100[] = "Mozilla/5.0 (Linux; Android 4.0.4; Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57101[] = "Mozilla/5.0 (Linux; Android 4.0.4; Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S57102[] = "Mozilla/5.0 (Linux; Android 4.0.4; INFINITY_POWER Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57103[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_hjy1a6x Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57104[] = "Mozilla/5.0 (Linux; Android 4.0.4; Tender Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57105[] = "Mozilla/5.0 (Linux; Android 4.0.9; e1909c_v77_gq1008_9p017_6228 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57106[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID Serials Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S57107[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbl1a535 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57108[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S57109[] = "Mozilla/5.0 (Linux; Android 4.0.9; e1909c_v77_gq1008_5647 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57110[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbaol1 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57111[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG SM-C101 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S57112[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; BLU STUDIO 5.0 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57113[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Studio 5.0s Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57114[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; BLU STUDIO 5.0 Build/JZO54K)";
const char S57115[] = "Mozilla/5.0 (Series40; NokiaC1-01/05.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.5.16.11";
const char S57116[] = "Mozilla/5.0 (Series40; NokiaC1-01.1/04.40a; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.5.16.11";
const char S57117[] = "Mozilla/5.0 (X11; Linux x86_64; NOOK BNTV400 BUILD/ICS) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S57118[] = "Mozilla/5.0 (iPad; U; CPU OS 4_0 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S57119[] = "Mozilla/5.0 (iPad; U; CPU iPad OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293 Safari/6531.22.7";
const char S57120[] = "Mozilla/5.0 (iPad; U; CPU OS 4_0 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B500 Safari/531.21.10";
const char S57121[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST25i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57122[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST25a Build/6.1.1.B.1.10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57123[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xperia U Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57124[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTCONE Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S57125[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S57126[] = "LG-GM360i/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S57127[] = "LG-GM360/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S57128[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4";
const char S57129[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4";
const char S57130[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Incredible S Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57131[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Incredible S Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57132[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Incredible S Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57133[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Incredible S S710e Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/vivo";
const char S57134[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; N9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57135[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; N9500 Build/JOP40D)";
const char S57136[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-03D Build/V21R28E) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57137[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-03D Build/F0001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57138[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-03D Build/V22R30A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57139[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-03D Build/V23R31Ac) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57140[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-03D Build/V22R29Bc) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57141[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; F-03D Build/F0001)";
const char S57142[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-03D Build/V23R32C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57143[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-03D Build/V24R33C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57144[] = "Mozilla/5.0 (Linux; Android 4.1.2; B1-710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S57145[] = "Android 1.5;";
const char S57146[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S57147[] = "Mozilla/5.0 (Linux; U; Android 1.5; vi-vn Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S57148[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S57149[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; Pole1 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S57150[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; N10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57151[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N10 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57152[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N10 Build/R13.4-rc1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57153[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57154[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; MID8024 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57155[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; CFW-MID8024 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57156[] = "Dalvik/1.4.0 (Linux; U; Android 2.3; MID8024 Build/GINGERBREAD)";
const char S57157[] = "Dalvik/1.4.0 (Linux; U; Android 2.3; CFW-MID8024 Build/GINGERBREAD)";
const char S57158[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/527+ (KHTML, like Gecko, Safari/419.3) Arora/0.6 (Change: )";
const char S57159[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Kobo Desktop Edition Safari/532.4";
const char S57160[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) VIDEODOWNLOAD Safari/533.3";
const char S57161[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.1 (KHTML, like Gecko) CEF/0.0.0.0 Safari/534.1";
const char S57162[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.3 (KHTML, like Gecko) Lunascape/6.8.0.25815 Safari/535.3";
const char S57163[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.34 (KHTML, like Gecko) iLivid/1.0 Safari/534.34";
const char S57164[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/528+ (KHTML, like Gecko, Safari/528.0) Lunascape/5.1.2.0";
const char S57165[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/529 (KHTML, like Gecko, Safari/529.0) Lunascape/4.9.9.100";
const char S57166[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) iLivid/1.0 Safari/534.34";
const char S57167[] = "Mozilla/5.0 (N; Windows NT 6.0) AppleWebKit/534.34 (KHTML, like Gecko) Raptr Safari/534.34";
const char S57168[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Superbird/27.0.1450.0 Safari/537.36";
const char S57169[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.17 (KHTML, like Gecko) Superbird/24.0.1312.57 Safari/537.17";
const char S57170[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.22 (KHTML, like Gecko) Superbird/27.0.1453.93 Safari/537.22";
const char S57171[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-M840 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57172[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-M840 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S57173[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57174[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S57175[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195T Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57176[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195L Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57177[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.3 (KHTML, like Gecko) Maxthon/3.3.3.1000 Chrome/16.0.883.0 Safari/535.3";
const char S57178[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.3 (KHTML, like Gecko) Maxthon/3.3.3.1000 Chrome/16.0.883.0 Safari/535.3";
const char S57179[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.3.9.1000 Chrome/18.0.966.0 Safari/535.12";
const char S57180[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.3.9.2000 Chrome/18.0.966.0 Safari/535.12";
const char S57181[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-B7510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57182[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; iw-il; GT-B7510B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57183[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-B7510 Build/GINGERBREAD)";
const char S57184[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:17.0) Gecko/17.0 Firefox/17.0 IceDragon/17.0.1.0";
const char S57185[] = "Mozilla/5.0 (Linux; Android 4.0.3; A500 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S57186[] = "Mozilla/5.0 (Linux; Android 4.0.4; Iconia A500 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S57187[] = "Mozilla/5.0 (Linux; Android 4.0.4; A500 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S57188[] = "Mozilla/5.0 (Linux; Android 4.0.4; A500 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57189[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.3) Gecko/20061201 Firefox/2.0.0.3 (Ubuntu-feisty)";
const char S57190[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC Velocity 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57191[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Velocity 4G Build/IML74K)";
const char S57192[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-hk; HTC_Velocity_4G_X710s Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57193[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-hk; HTC Velocity 4G X710s Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57194[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Velocity 4G X710s Build/IML74K)";
const char S57195[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Huawei Ascend Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57196[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Huawei Ascend Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57197[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; M860 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57198[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; M860 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57199[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; M860 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57200[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HUAWEI-M860 Build/HuaweiM860) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57201[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.10) Gecko/X Linux Mint/5 (Elyssa) Firefox/3.0.10";
const char S57202[] = "Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/X Linux Mint/5 (Elyssa) Firefox/3.0.11";
const char S57203[] = "Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.10) Gecko/X Linux Mint/5 (Elyssa) Firefox/3.0.10";
const char S57204[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.12) Gecko/X Linux Mint/5 (Elyssa) Firefox/3.0.12";
const char S57205[] = "Mozilla/5.0 (X11; U; Linux i686; es-AR; rv:1.9.0.4) Gecko/X Linux Mint/5 (Elyssa) Firefox/3.0.4";
const char S57206[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.5) Gecko/X Linux Mint/5 (Elyssa) Firefox/3.0.5";
const char S57207[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.7) Gecko/X Linux Mint/5 (Elyssa) Firefox/3.0.7";
const char S57208[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.8) Gecko/X Linux Mint/5 (Elyssa) Firefox/3.0.8";
const char S57209[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S57210[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S57211[] = "Mozilla/5.0 (Linux; Android 4.2.2; Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57212[] = "Mozilla/5.0 (Linux; Android 4.2.2; skz_tenderloin Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S57213[] = "Mozilla/5.0 (Linux; Android 4.2.1; e2001_v89_jbl1a698 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57214[] = "Mozilla/5.0 (Linux; Android 4.2.2; Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S57215[] = "Mozilla/5.0 (Linux; Android 4.2.2; cm_tenderloin Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S57216[] = "Mozilla/5.0 (Linux; Android 4.2.1; e2001_v89_gq2008s Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57217[] = "Mozilla/5.0 (Linux; Android 4.2.1; e2001v21_v89_zlh_hd Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57218[] = "Mozilla/5.0 (Linux; Android 4.2.1; v89_jbl1a698 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57219[] = "Mozilla/5.0 (Linux; Android 4.2; iPhone5 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57220[] = "Mozilla/5.0 (Linux; Android 4.2.1; e2001v21_v89_jbl1a698_2g Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57221[] = "Mozilla/5.0 (Linux; Android 4.2.1; e2001_v89_jbl1a668_fwvga Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57222[] = "Mozilla/5.0 (Linux; Android 4.3; GT-I9100 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57223[] = "Mozilla/5.0 (Linux; Android 4.3; GT-I9100G Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57224[] = "Logos/1.0 Beta 11 (Linux; U; Android 2.2; en_US; SGH-I987 Build/FROYO.UCJK1)";
const char S57225[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-I987 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57226[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; SGH-I987 Build/FROYO)";
const char S57227[] = "Mozilla/5.0 (Linux; U; Android 2.2; de-de; GT-I9010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57228[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SHW-M240S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57229[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SHW-M240S Build/FROYO)";
const char S57230[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6800 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S57231[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P6800 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S57232[] = "Mozilla/5.0 (Linux; Android 4.0.4;  GT-P6800 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S57233[] = "Mozilla/5.0 (Linux; Android 4.1.2; VS930 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57234[] = "SAMSUNG-GT-C3200/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S57235[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I467 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S57236[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I467M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S57237[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-tw; LiquidE Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S57238[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Acer Liquid Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S57239[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Liquid Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S57240[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Liquid Build/ECLAIR)";
const char S57241[] = "Opera/9.80 (Android 4.0.3; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S57242[] = "Opera/9.80 (Android 4.0.4; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S57243[] = "Opera/9.80 (Android 4.0.1; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S57244[] = "Opera/9.80 (Android 4.0.9; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S57245[] = "Opera/9.80 (Android 4.0; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S57246[] = "Opera/9.80 (Android 4.0.2; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S57247[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PLT7044K Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57248[] = "AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; PLT7044K Build/JRO03H)";
const char S57249[] = "Mozilla/5.0 (Linux; U; Android 2.2; vi-; HTC HD2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57250[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Android Powered HD2 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57251[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; vi-vn; HTC_HD2 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57252[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC HD2 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57253[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC HD2 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57254[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HD2_T8585 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57255[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; HD2_T8585 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57256[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HD2_T8585 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57257[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-tw; HTC_HD2_T8585 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57258[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC_HD2_T8585 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57259[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC HD2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57260[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC HD2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57261[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; HTC HD2 Build/FRF91)";
const char S57262[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC HD2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57263[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; HTC HD2 Build/FRG83D)";
const char S57264[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC HD2 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S57265[] = "NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-01) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S57266[] = "Mozilla/5.0 (PLAYSTATION 3 4.21) AppleWebKit/531.22.8 (KHTML, like Gecko)";
const char S57267[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One X Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57268[] = "Mozilla/5.0 (Linux; Android 4.2.1; HTC One X Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57269[] = "Mozilla/5.0 (Linux; Android 4.2.2; One X Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57270[] = "Mozilla/5.0 (Linux; Android 4.2.2; Full AOSP on EndeavorU Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57271[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One X Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S57272[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.7 (KHTML, like Gecko) Ubuntu/11.10 Chromium/16.0.912.77 Chrome/16.0.912.77 Safari/535.7";
const char S57273[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.7 (KHTML, like Gecko) Ubuntu/11.10 Chromium/16.0.912.77 Chrome/16.0.912.77 Safari/535.7";
const char S57274[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.7 (KHTML, like Gecko) Ubuntu/11.10 Chromium/16.0.912.77 Chrome/16 0.912.77 Safari/535.7";
const char S57275[] = "Nokia2760/2.0 (05.20) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S57276[] = "Nokia2760_CMCC/2.0 (05.20) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S57277[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ617 Build/7.7.1-85_MZ617-27) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S57278[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; HTL21 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57279[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTL21 Build/JRO03C)";
const char S57280[] = "Mozilla/5.0 (Windows NT 4.0; rv:17.0) Gecko/X Firefox/17.0";
const char S57281[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-51 Build/HuaweiU8950N-51) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57282[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-1 Build/HuaweiU8950-1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57283[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950D Build/HuaweiU8950D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57284[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-1 Build/HuaweiU8950N-1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57285[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-51 Build/HuaweiU8950-51) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57286[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/toro";
const char S57287[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/toro";
const char S57288[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/toro";
const char S57289[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/toro";
const char S57290[] = "Mozilla/4.0 (compatible; Polaris 6.2; Brew 3.1.5; en)/240X320 Samsung sam-r631";
const char S57291[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-GB; rv:1.8.1.12) Gecko/X Firefox/2.0.0.12";
const char S57292[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NovoCrystal Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57293[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NovoCrystal Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57294[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; NovoCrystal Build/JRO03C)";
const char S57295[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6506 Build/10.3.A.0.423) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57296[] = "Mozilla/5.0 (Linux; Android 4.0.4; Event Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57297[] = "Mozilla/5.0 (X11; U; Linux i686; de-DE; rv:99.0) Gecko/X Firefox/25.0.0";
const char S57298[] = "Mozilla/5.0 (X11; Linux x86_64; rv:25.0) Gecko/X Firefox/25.0 Iceweasel/25.0a2";
const char S57299[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-I9001 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57300[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-i9001 Build/IMM76L; CyanogenMod-EHNDROIX i9001) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57301[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-I9001 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57302[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Maxwell Plus Build/1.0.0 X-10:39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57303[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Maxwell Plus Build/1.0.2 X-14:58) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57304[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Maxwell Plus Build/1.0.3 X-14:07) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57305[] = "Mozilla/5.0 (Linux; Android 4.0.4; ME865 Build/6.7.2_GC-220-EDS-3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57306[] = "Mozilla/5.0 (Linux; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57307[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_DesireHD_A9191; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S57308[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_DesireHD_A9191/1.0; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S57309[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Desire_HD; en-hk) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S57310[] = "MOT-EX119 MIDP-2.0/CLDC-1.1 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U) Presto/2.5.28 Version/10.10";
const char S57311[] = "MOT-EX139 MIDP-2.0/CLDC-1.1 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U) Presto/2.5.28 Version/10.10";
const char S57312[] = "MOT-EX226 MIDP-2.0/CLDC-1.1 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10";
const char S57313[] = "MOT-EX505 MIDP-2.0/CLDC-1.1 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10";
const char S57314[] = "NokiaX2-02/2.0 (11.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-02) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S57315[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/11A465 Safari/7534.48.3";
const char S57316[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/11A501 Safari/7534.48.3";
const char S57317[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SPH-M580 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57318[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; SPH-M580 Build/FROYO)";
const char S57319[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57320[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; tr-tr; Trident A7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57321[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; A7* Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57322[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; A7 Build/GRK39F)";
const char S57323[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 Nokia6730c-1/021.009; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.1.13126";
const char S57324[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 Nokia6730c/031.022; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.3.1";
const char S57325[] = "NOKIA6730c-1";
const char S57326[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-I5500 Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57327[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-I5500 Build/GINGERBREAD; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57328[] = "Mozilla/5.0 (Windows; N; Windows NT 6.0; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/7.0.3.8542 Safari/532.4";
const char S57329[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.0.6000.0);en;kml:2.2;client:Free;type:default)";
const char S57330[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus S Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57331[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; Nexus S Build/JOP40D)";
const char S57332[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus S Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57333[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57334[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; crespo Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57335[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus S Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57336[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; Nexus S Build/JDQ39E)";
const char S57337[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT300SE Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S57338[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT300 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S57339[] = "Mozilla/5.0 (Windows NT 4.10) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31";
const char S57340[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; T8788)";
const char S57341[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Surround)";
const char S57342[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Mondrian)";
const char S57343[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7";
const char S57344[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7";
const char S57345[] = "Android/1.0 (daytona 4.5.1A-DTN-150-30)";
const char S57346[] = "Android/1.0 (VS700 GRJ22)";
const char S57347[] = "Mozilla/5.0 (Linux; U; Android 1.0; en-us; generic) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S57348[] = "Android/1.0 (pyramid IML74K)";
const char S57349[] = "Android/1.0 (holiday IML74K)";
const char S57350[] = "Android/1.0 (aloha FRG83D)";
const char S57351[] = "Android/1.0 (kingdom GRJ22)";
const char S57352[] = "Android/1.0 (cdma_targa 5.5.1_84_DBN-74)";
const char S57353[] = "Android/1.0 (cdma_droid2 4.5.1_57_DR4-51)";
const char S57354[] = "Android/1.0 (cdma_maserati 6.5.1_167_DR4-1_M1-219)";
const char S57355[] = "Android/1.0 (shooter IML74K)";
const char S57356[] = "Android/1.0 (kingdom IML74K)";
const char S57357[] = "Android/1.0 (mecha IMM76D)";
const char S57358[] = "Android/1.0 (d2att IMM76D)";
const char S57359[] = "Android/1.0 (stingray IMM76L)";
const char S57360[] = "Android/1.0 (e0 GRK39F)";
const char S57361[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.6.6.X (Second Life Release; default skin) Safari/533.3";
const char S57362[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.9.1";
const char S57363[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.1";
const char S57364[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.9.2";
const char S57365[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.2";
const char S57366[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.3";
const char S57367[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.4";
const char S57368[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.9.4";
const char S57369[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.5";
const char S57370[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.9.5";
const char S57371[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.6";
const char S57372[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.9.6";
const char S57373[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.7";
const char S57374[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.9.7";
const char S57375[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.8";
const char S57376[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.9.8";
const char S57377[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57378[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57379[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Z715e Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57380[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57381[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JY-G3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57382[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; JY-G3 Build/IMM76D)";
const char S57383[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Z120 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57384[] = "Mozilla/5.0 (Windows NT 6.1; rv:16.0) Gecko/X Firefox/16.0 epic/16.0";
const char S57385[] = "Mozilla/5.0 (X11; Linux i686; rv:22.0) Gecko/X Firefox/22.0 SeaMonkey/2.19";
const char S57386[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/mako";
const char S57387[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/mako";
const char S57388[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/mako";
const char S57389[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/mako";
const char S57390[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; iBall Slide i9702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57391[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iBall Slide 3G 7334 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57392[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; iBall Slide 3G 7307 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57393[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iBall Slide 3G7271 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57394[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; iBall Slide i5715 Build/tablet-c) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57395[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; iBall Slide i6030 Build/iBall-Slide-i6030) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57396[] = "Mozilla/5.0 (Linux; Android 4.0.4; N9500 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57397[] = "Mozilla/5.0 (Linux; Android 4.0.4; N9500 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S57398[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E973 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57399[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F200L Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57400[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F200K Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57401[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F200S Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57402[] = "Mozilla/5.0 (Linux; Android 4.1.1; EK-GC100 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57403[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-EK-GC100 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57404[] = "Mozilla/5.0 (Linux; Android 4.1.2; EK-GC100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57405[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-EK-GC100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57406[] = "SonyEricssonK770i/R8BC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S57407[] = "SonyEricssonK770iv/R8BC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S57408[] = "Mozilla/5.0 (Series40; NokiaX2-05/08.35; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.11";
const char S57409[] = "Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10 UCBrowser/2.2.2.259";
const char S57410[] = "LG-GW300-Bouygues Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S57411[] = "LG-GW300FD/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S57412[] = "LG-GW300-orange/V09a Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S57413[] = "LG-GW300-Orange/V10c Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S57414[] = "LG-GW300/V100 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S57415[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E970 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S57416[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E970 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S57417[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_2; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0";
const char S57418[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_0; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0";
const char S57419[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_2; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0";
const char S57420[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_3; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0";
const char S57421[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0";
const char S57422[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_3; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0";
const char S57423[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0";
const char S57424[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0";
const char S57425[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0";
const char S57426[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/531.21.8+(KHTML, like Gecko, Safari/528.16) Version/5.10.3 OmniWeb/622.14.0";
const char S57427[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0";
const char S57428[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0";
const char S57429[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0";
const char S57430[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0";
const char S57431[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-US) AppleWebKit/531.21.8+(KHTML, like Gecko, Safari/528.16) Version/5.10.3 OmniWeb/622.14.0";
const char S57432[] = "Mozilla/5.0 (Linux; Android 4.0.4;  Amazon Kindle Fire Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S57433[] = "Mozilla/5.0 (Linux; Android 4.0.4; Amazon Kindle Fire Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S57434[] = "Mozilla/5.0 (Linux; Android 4.0.4; Kindle Fire Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S57435[] = "Mozilla/5.0 (Linux; Android 4.0.3; Amazon Kindle Fire Build/MR1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S57436[] = "Mozilla/5.0 (Linux; Android 4.0.4-2.4.20; Amazon Kindle Fire Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S57437[] = "Mozilla/5.0 (Linux; Android 4.0.4; Amazon Kindle Fire Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57438[] = "Mozilla/5.0 (Linux; Android 4.0.3; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S57439[] = "Mozilla/5.0 (Linux; Android 4.0.4; KF012 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57440[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E973 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57441[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F200S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57442[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F200K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57443[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F200L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57444[] = "Opera/9.80 (Android 4.1.1; Linux; Opera Mobi/ADR-X; U; fr) Presto/2.10.254 Version/12.00";
const char S57445[] = "Opera/9.80 (Android 4.1; Linux; Opera Mobi/ADR-X; U; en) Presto/2.10.254 Version/12.00";
const char S57446[] = "Opera/9.80 (Android 4.1.2; Linux; Opera Mobi/ADY-X; U; ru) Presto/2.10.254 Version/12.00";
const char S57447[] = "Nokia7610/2008.05.06 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S57448[] = "Nokia7610/2.0 (7.0642.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S57449[] = "Nokia7610/SymbianOS/8.0 Series60/2.6";
const char S57450[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHW-M380W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57451[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHW-M380S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57452[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHW-M380K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57453[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHW-M380W Build/IMM76D)";
const char S57454[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T769 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57455[] = "Mozilla/5.0 (Linux;U; Android 4.1; en-gb; ST10216-1Build/ST10216-1) AppleWebKit/534.30 (KHTML,like Gecko) Version/4.0 Safari/534.30";
const char S57456[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ST10216-1 Build/ST10216-1)";
const char S57457[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ST10216-2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30;SurfTab_10.1";
const char S57458[] = "Mozilla/5.0 (iPhone; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.10";
const char S57459[] = "Mozilla/5.0(iPhone; U; CPU iPhone OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B314 Safari/531.21.10";
const char S57460[] = "Mozilla/5.0 (iPhone Simulator; U; CPU iPhone OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7D11 Safari/531.21.10";
const char S57461[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; N90 DUAL CORE2 CZ Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57462[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; N90 DUAL CORE2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57463[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N90DC_S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57464[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; N90DC_S Build/JRO03H)";
const char S57465[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; N90 DUAL CORE FHD Build/N90 DUAL CORE FHD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57466[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; N90 DUAL CORE2 V12 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57467[] = "NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac1-01) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S57468[] = "Nokia3500c/2.0 (05.51) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S57469[] = "Nokia3500c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia3500c";
const char S57470[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5; U; Edition MAS; en) Presto/2.10.229 Version/11.67";
const char S57471[] = "Mozilla/5.0 (X11; FreeBSD amd64; rv:17.0) Gecko/17.0 Firefox/17.0";
const char S57472[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LVW Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S57473[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR6410LRA Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S57474[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR6410LVW Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S57475[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LVW Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S57476[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S57477[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus 7 Build/JRO03S) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S57478[] = "Mozilla/5.0 (Linux; Android 4.1; Nexus 7 Build/JRN84D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S57479[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; LG-V900 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57480[] = "Mozilla/5.0 (Linux; U; Android 3.1; de-de; LG-V900/v10e Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57481[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; LG-V900 Build/HMJ37)";
const char S57482[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9000 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57483[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9000 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57484[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, gb";
const char S57485[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T999 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, us";
const char S57486[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I535 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, us";
const char S57487[] = "Mozilla/5.0 (Windows NT 4.0; rv:18.0) Gecko/X Firefox/18.0";
const char S57488[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-L710 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57489[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; A6277 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S57490[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; A6277 Build/CUPCAKE) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S57491[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Sony Tablet P Build/THMAAS0045) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57492[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-gb; Sony Tablet P Build/THMAS0042) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57493[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; Sony Tablet P Build/THMAS11001) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57494[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-gb; Sony Tablet P Build/THMASU0035) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57495[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; Sony Tablet P Build/THMD01900) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57496[] = "Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; Sony Tablet P Build/THMAS11002) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57497[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57498[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57499[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S57500[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S57501[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9220 Build/GINGERBREAD)";
const char S57502[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; I9220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57503[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; I9220 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57504[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NWZ-Z1000Series Build/C121) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57505[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NWZ-Z1000Series Build/B100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57506[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; NWZ-Z1000Series Build/C121)";
const char S57507[] = "NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiax2-01) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S57508[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US; rv:1.8.1.16) Gecko/X Firefox/3.0.1";
const char S57509[] = "Mozilla/6.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:2.0.0.0) Gecko/X Firefox/3.0	Firefox 3.0";
const char S57510[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; rv:1.7.2) Gecko/X Firefox/3.0";
const char S57511[] = "Mozilla/5.0 (Linux; Android 4.0.4; Motorola RAZR MAXX Build/6.7.2-180_DHD-16_NA-27) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57512[] = "Opera/9.80 (BREW; Opera Mini/6.0.0/27.2338; U; en) Presto/2.8.119 240X320 Samsung SCH-U660";
const char S57513[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ604 Build/I.7.1-45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S57514[] = "Mozilla/5.0 (Linux; Android 4.0.3; MZ604 Build/I.7.1-34) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S57515[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; ISW11M Build/4.5.1A-1_KDI-92_LE-24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57516[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; ISW11M Build/4.5.1A-1_KDI-95_MR1-47) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57517[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; ISW11M Build/4.5.1A-1_KDI-95_MR3-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57518[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; ISW11M Build/4.5.1A-1_KDI-95_MR2-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57519[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; ISW11M Build/4.5.1A-1_KDI-95_MR4-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57520[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; ISW11M Build/4.5.1A-1_KDI-95_MR4-3)";
const char S57521[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2) Gecko/X Ubuntu/9.04 (jaunty) Firefox/3.6";
const char S57522[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V1277 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57523[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; V1277 Build/IMM76D)";
const char S57524[] = "Mozilla/5.0 (Windows NT 4.0; rv:19.0) Gecko/X Firefox/19.0";
const char S57525[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Twitter for iPhone";
const char S57526[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PAP4300 DUO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57527[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR6425LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57528[] = "SCH-U360/1.0 NetFront/3.0.22.2.22 (GUI) MMP/2.0";
const char S57529[] = "TIANYU-KTOUCH/A930/Screen-240X320";
const char S57530[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One S Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57531[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One S Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57532[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One S Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57533[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT7033 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57534[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E610 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57535[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E610v Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57536[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57537[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57538[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612f Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57539[] = "BlackBerry8130/4.5.0.175 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105";
const char S57540[] = "NokiaC1-01/2.0 (04.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac1-01) UCBrowser8.4.0.159/70/444/UCWEB Mobile";
const char S57541[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-E971 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57542[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-E971 Build/IMM76L)";
const char S57543[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.0.400) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S57544[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.0.400) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S57545[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_EVO3D_X515a; en-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S57546[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_EVO3D_X515m; zh-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S57547[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_EVO3D_X515m; en-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S57548[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_EVO3D_X515m; en-hk) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S57549[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_EVO3D_X515m; en-gb) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S57550[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_EVO3D_X515a; en-au) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S57551[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_X515a V3.29.841.1; en-au) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S57552[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_EVO3D_X515m; zh-cn) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S57553[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_EVO3D_X515m; de-de) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S57554[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_EVO3D_X515a; fr-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S57555[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_EVO3D_X515m; vi-vn) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S57556[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_X515a V1.24.841.3; en-au) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S57557[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST26i Build/11.2.A.0.21) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57558[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST26i Build/11.2.A.0.21) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S57559[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST26a Build/11.2.A.0.33) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57560[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SH-01D Build/SB082) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57561[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SH-01D Build/S1110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57562[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-01D Build/S2040) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57563[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-01D Build/S5100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57564[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-01D Build/S6270) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57565[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SH-01D Build/S6270)";
const char S57566[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN8CG3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57567[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; VS TOUCHTAB 7DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57568[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; VS TOUCHTAB 7DC V Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57569[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; VS TOUCHTAB 7DC Build/JRO03H)";
const char S57570[] = "Mozilla/5.0 (Linux; Android 4.0.4; C525c Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57571[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; B15 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57572[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; B15 Build/JZO54K)";
const char S57573[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; WX442 Build/REV) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 240X320 motorola WX442";
const char S57574[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57575[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190L Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57576[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E610 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S57577[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612g Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S57578[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612f Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S57579[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S57580[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E610v Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S57581[] = "Mozilla/5.0 (Linux; Android 4.1.2; C525c Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57582[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; Behold2 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S57583[] = "Mozilla/2.0 (compatible; MSIE 3.0; AK; Windows 95)";
const char S57584[] = "Mozilla/2.0 (compatible; MSIE 3.02; Update a; AOL 3.0; Windows 95)";
const char S57585[] = "Mozilla/2.0 (compatible; MSIE 3.01; Windows 95)";
const char S57586[] = "Mozilla/5.0 (compatible; MSIE 3.0; Win95; Trident/1.0)";
const char S57587[] = "Facebook/4110.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S57588[] = "Tumblr/117 CFNetwork/548.1.4 Darwin/11.0.0";
const char S57589[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57590[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57591[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57592[] = "Mozilla/4.0 (Brew MP 1.0.2; U; en-us; Kyocera; NetFront/3.5.1/AMB) Sprint S3015";
const char S57593[] = "Mozilla/4.0 (Brew MP 1.0.2; U; en-us; Kyocera; NetFront/3.5.1/AMB) payLo S3015-PLB";
const char S57594[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Mobile Safari/535.19 Silk-Accelerated=true";
const char S57595[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGPT12 Build/TID0142) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S57596[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGPT13 Build/TID0142) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S57597[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; USCC-US780 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57598[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-US780 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57599[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-US780 Build/JZO54K)";
const char S57600[] = "Nokia5230/UCWEB8.6.0.180/50/800";
const char S57601[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5230) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile";
const char S57602[] = "Mozilla/5.0 (Linux; Android 4.1.1; C6502 Build/10.1.A.0.233) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57603[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6502 Build/10.1.A.1.434) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57604[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57605[] = "PantechP6020/JXUSX BMP/1.0.4 DeviceId/X NetFront/4.1 OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S57606[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315";
const char S57607[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S57608[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 7 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S57609[] = "Mozilla/5.0 (Series40; NokiaC3-00/08.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S57610[] = "NokiaC3-00/5.0 (08.71) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac3-00) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S57611[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I5801 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57612[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I5801 Build/FROYO)";
const char S57613[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I5801 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S57614[] = "Mozilla/5.0 (Windows NT) AppleWebKit/534.20 (KHTML, like Gecko) Chrome/11.0.672.2 Safari/534.20";
const char S57615[] = "Mozilla/5.0 (Windows NT) AppleWebKit/534.20 (KHTML like Gecko) Chrome/11.0.672.2 Safari/534.20";
const char S57616[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-I9080L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57617[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9080L Build/JZO54K)";
const char S57618[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; IdeaTab S6000-F Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57619[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; IdeaTab S6000-F Build/JOP40D)";
const char S57620[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; pl-pl; IdeaTab S6000-H Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57621[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; IdeaTab S6000-F Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57622[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57623[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57624[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57625[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; VodafoneSmartChat-MSM7225A-V02c-SepX-Vodafone-IT/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57626[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ro-ro; VodafoneSmartChat-MSM7225A-V02c-SepX-Vodafone-RO/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57627[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; VodafoneSmartChat-MSM7225A-V02c-SepX-Vodafone-IE/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57628[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; VodafoneSmartChat-MSM7225A-V02c-SepX-Vodafone-ES/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57629[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; VodafoneSmartChat-MSM7225A-V02c-SepX-Vodafone-UK/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57630[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; VodafoneSmartChat-MSM7225A-V03a-DecX-Vodafone-ES/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57631[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; VodafoneSmartChat-MSM7225A-V03a-DecX-Vodafone-IT/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57632[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ro-ro; VodafoneSmartChat-MSM7225A-V03a-DecX-Vodafone-RO/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57633[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-pt; VodafoneSmartChat-MSM7225A-V03a-DecX-Vodafone-PT/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57634[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; VodafoneSmartChat-MSM7225A- V02a-SepX-Vodafone-NZ/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57635[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; VodafoneSmartChat-MSM7225A-V03a-DecX-Vodafone-UK/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57636[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; el-gr; VodafoneSmartChat-MSM7225A-V03a-DecX-Vodafone-GR/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57637[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-pt; VodafoneSmartChat-MSM7225A-V03a-MarX-Vodafone-PT/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57638[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 720; ANZ885)";
const char S57639[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 720; Vodafone)";
const char S57640[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia 720)";
const char S57641[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia 720T; CMCC)";
const char S57642[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 720; Orange)";
const char S57643[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R740C Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57644[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R740C Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S57645[] = "SAMSUNG-GT-B3210/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S57646[] = "SAMSUNG-GT-B3210/1.0 Release/10.19.2009 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S57647[] = "Mozilla/5.0 (Series40; Nokia2700c-2/09.97; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S57648[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.1 Safari/535.19 Silk-Accelerated=true";
const char S57649[] = "Opera/9.80 (Android 2.3.4; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00";
const char S57650[] = "Opera/9.80 (Android 2.3.3; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00";
const char S57651[] = "Opera/9.80 (Android 2.3.1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00";
const char S57652[] = "Opera/9.80 (Android 2.3.5; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00";
const char S57653[] = "Opera/9.80 (Android 2.3.6; Linux; Opera Tablet/ADR-X; U; de) Presto/2.10.254 Version/12.00";
const char S57654[] = "Opera/9.80 (Android 2.3; Linux; Opera Tablet/ADR-X; U; en-GB) Presto/2.10.254 Version/12.00";
const char S57655[] = "Opera/9.80 (Android 2.3.7; Linux; Opera Tablet/35779; U; en) Presto/2.10.254 Version/12.00";
const char S57656[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S57657[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus 7 Build/JRO03S) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S57658[] = "Mozilla/5.0 (Linux; Android 4.1; Nexus 7 Build/JRN84D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S57659[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57660[] = "Mozilla/5.0 (Series40; NokiaX2-01/08.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.0.9.17.2";
const char S57661[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.26.2 (KHTML, like Gecko) Version/6.1 Safari/537.26.2";
const char S57662[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.28.2 (KHTML, like Gecko) Version/6.1 Safari/537.28.2";
const char S57663[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.30.1 (KHTML, like Gecko) Version/6.1 Safari/537.30.1";
const char S57664[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.32 (KHTML, like Gecko) Version/6.1 Safari/537.32";
const char S57665[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.31.15 (KHTML, like Gecko) Version/6.1 Safari/537.31.15";
const char S57666[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.33 (KHTML, like Gecko) Version/6.1 Safari/537.33";
const char S57667[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.34 (KHTML, like Gecko) Version/6.1 Safari/537.34";
const char S57668[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.37 (KHTML, like Gecko) Version/6.1 Safari/537.37";
const char S57669[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.36 (KHTML, like Gecko) Version/6.1 Safari/537.36";
const char S57670[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.35.11 (KHTML, like Gecko) Version/6.1 Safari/537.35.11";
const char S57671[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.38 (KHTML, like Gecko) Version/6.1 Safari/537.38";
const char S57672[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.39.2 (KHTML, like Gecko) Version/6.1 Safari/537.39.1";
const char S57673[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.1 Safari/537.40";
const char S57674[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.41 (KHTML, like Gecko) Version/6.1 Safari/537.41";
const char S57675[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.42.1 (KHTML, like Gecko) Version/6.1 Safari/537.42.1";
const char S57676[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.43 (KHTML, like Gecko) Version/6.1 Safari/537.43";
const char S57677[] = "Mozilla/5.0 (X11; U; Linux i686; pt-BR; rv:1.8.0.3) Gecko/X Ubuntu/dapper Firefox/1.5.0.3";
const char S57678[] = "SAMSUNG-GT-S5220/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S57679[] = "SAMSUNG-GT-S5220R/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S57680[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 Nokia6710s/031.022; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.3.1";
const char S57681[] = "NOKIA6710s";
const char S57682[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; SmartTab7-MSM8260-V02d-DecX-Vodafone-ZA) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57683[] = "Mozilla/5.0 (Linux; U; Android 3.2; es-es; SmartTab7-MSM8260-V02d-DecX-Vodafone-ES) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57684[] = "Mozilla/5.0 (Linux; U; Android 3.2; ro-ro; SmartTab7-MSM8260-V02d-DecX-Vodafone-RO) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57685[] = "Mozilla/5.0 (Linux; U; Android 3.2; es-es; SmartTab7-MSM8260-V03a-FebX-Vodafone-ES) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57686[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-nz; SmartTab7-MSM8260-V02d-DecX-Vodafone-NZ) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57687[] = "Mozilla/5.0 (Linux; U; Android 3.2; hu-hu; SmartTab7-MSM8260-V03b-AprX-Vodafone-HU) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57688[] = "Mozilla/5.0 (Linux; U; Android 3.2; es-es; SmartTab7-MSM8260-V03b-AprX-Vodafone-ES) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57689[] = "Mozilla/5.0 (Linux; U; Android 3.2; ro-ro; SmartTab7-MSM8260-V03b-AprX-Vodafone-RO) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57690[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; SmartTab7-MSM8260-V03b-AprX-Vodafone-ZA) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57691[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; SmartTab7-MSM8260-V03a-JanX-Vodafone-DE) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57692[] = "Mozilla/5.0 (Linux; U; Android 3.2; hu-hu; SmartTab7-MSM8260-V03a-JanX-Vodafone-HU) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57693[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; SmartTab7-MSM8260-V03b-AprX-Vodafone-DE) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57694[] = "Mozilla/5.0 (Linux; U; Android 3.2; cs-cz; SmartTab7-MSM8260-V03b-AprX-Vodafone-CZ) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57695[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; es-es; SmartTab7 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57696[] = "Mozilla/5.0 (Linux; U; Android 3.2; pt-pt; SmartTab7-MSM8260-V03b-AprX-Vodafone-PT) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57697[] = "Mozilla/5.0 (en-US) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229 Safari/537.4";
const char S57698[] = "Mozilla/5.0 (Linux; U; Android 3.1; ko-kr; SHW-M380K Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57699[] = "Mozilla/5.0 (Linux; U; Android 3.1; ko-kr; SHW-M380W Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57700[] = "Mozilla/5.0 (Linux; U; Android 3.1; ko-kr; SHW-M380S Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S57701[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; SHW-M380W Build/HMJ37)";
const char S57702[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; SHW-M380S Build/HMJ37)";
const char S57703[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Micromax A54 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57704[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; A54 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57705[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; A54 Build/GINGERBREAD)";
const char S57706[] = "SonyEricssonC905/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3";
const char S57707[] = "Opera/9.80 (Android 2.3.3; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S57708[] = "Opera/9.80 (Android 2.3.4; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S57709[] = "Opera/9.80 (Android 2.3.1; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S57710[] = "Opera/9.80 (Android 2.3.6; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S57711[] = "Opera/9.80 (Android 2.3.5; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S57712[] = "Opera/9.80 (Android 2.3; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S57713[] = "Opera/9.80 (Android 2.3.7; Linux; Opera Tablet/46154) Presto/2.11.355 Version/12.10";
const char S57714[] = "Mozilla/5.0 (Symbian3; U; en-us; NokiaC6-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile";
const char S57715[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac6-01) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba";
const char S57716[] = "NokiaC6-01UCBrowser/8.7.1.234/51/355/UCWEB";
const char S57717[] = "Mozilla/5.0 (Linux; Android 4.1.1; E270BSA Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S57718[] = "Nokia2610/2.0 (04.50) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia2610";
const char S57719[] = "Nokia2610/2.0 (04.90) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S57720[] = "JUC (Linux; U; 2.3.4; zh-cn; GT-S5830; 320*480) UCWEB7.9.4.145/139/800";
const char S57721[] = "JUC (Linux; U; 2.2.1; zh-cn; GT-S5830; 320*480) UCWEB7.9.0.94/139/444";
const char S57722[] = "JUC (Linux; U; 2.3.6; zh-cn; GT-S5830M; 320*480) UCWEB7.9.0.94/139/444";
const char S57723[] = "JUC (Linux; U; 2.3.4; zh-cn; GT-S5830L; 320*480) UCWEB7.9.0.94/139/444";
const char S57724[] = "NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-03) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S57725[] = "Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Safari/535.19 Silk-Accelerated=true";
const char S57726[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-LS720 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57727[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-LS720 Build/JZO54K)";
const char S57728[] = "FeeddlerRSS 1.12.4 (iPad; iPhone OS 6.0; en_US)";
const char S57729[] = "Appcelerator Titanium/1.7.2 (iPad/6.0.1; iPhone OS; en_US;)";
const char S57730[] = "FeeddlerRSS 1.12.5 (iPad; iPhone OS 6.0.1; en_US)";
const char S57731[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; J-Q88B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57732[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; J-Q88B? Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57733[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/2.8.1 Mobile/10B141 Safari/7534.48.3";
const char S57734[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/2.8.1 Mobile/10B146 Safari/7534.48.3";
const char S57735[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/2.8.1 Mobile/10B147 Safari/7534.48.3";
const char S57736[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/2.8.1 Mobile/10B329 Safari/7534.48.3";
const char S57737[] = "Mozilla/5.0 (Series40; Nokia113/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S57738[] = "UCWEB/2.0(BlackBerry; U; 9.49.0.31; en-us; 9320/9.49.0.31) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S57739[] = "UCWEB/2.0(BlackBerry; U; 9.49.0.76; en-us; 9320/9.49.0.76) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S57740[] = "UCWEB/2.0(BlackBerry; U; 9.49.0.77; en-us; 9320/9.49.0.77) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S57741[] = "UCWEB/2.0(BlackBerry; U; 9.49.0.47; en-us; 9320/9.49.0.47) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S57742[] = "UCWEB/2.0(BlackBerry; U; 9.49.0.98; en-us; 9320/9.49.0.98) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S57743[] = "UCWEB/2.0(BlackBerry; U; 9.49.0.53; en-us; 9320/9.49.0.53) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S57744[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I527 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57745[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I527M Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57746[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MZ601 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57747[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MZ601 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57748[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MZ601 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57749[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MZ601 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57750[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; MZ601 Build/JZO54K)";
const char S57751[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; MZ601 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57752[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Mobile Safari/535.19 Silk-Accelerated=true";
const char S57753[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac5-00.2) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile  3gpp-gba";
const char S57754[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaC5-00.2) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S57755[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaC5-00) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S57756[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac5-00) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile  3gpp-gba";
const char S57757[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S57758[] = "Mozilla/5.0 (Linux; Android 4.0.3;  GT-I9000 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S57759[] = "Mozilla/5.0 (Linux; Android 4.0.4;  GT-I9000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S57760[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9000 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57761[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9000 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57762[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC_Amaze_4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57763[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Ruby Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57764[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Amaze 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57765[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-hk; LG-P936 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57766[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; LG-P936/V10c Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57767[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; LG-P936/V10e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57768[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; sv; LG-P936 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57769[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-P936 Build/GRJ90)";
const char S57770[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-pt; LG-P936/V10f Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57771[] = "Mozilla/5.0 (Series40; NokiaC2-00/03.82; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.5.16.11";
const char S57772[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE N9120 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S57773[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57774[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57775[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57776[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57777[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation XE with Beats Audio z715e Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57778[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Z715e Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57779[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.6 Safari/535.19 Silk-Accelerated=true";
const char S57780[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:22.0) Gecko/X Firefox/22.0";
const char S57781[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; QQBrowser/7.0.4350.400)";
const char S57782[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; QQBrowser/7.0.4350.400)";
const char S57783[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/7.0.4350.400)";
const char S57784[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; QQBrowser/7.0.4350.400)";
const char S57785[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/7.0.4350.400)";
const char S57786[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/7.0.4350.400)";
const char S57787[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR6425LVW Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57788[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6425LVW Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S57789[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.3; WOW64; Trident/7.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729)";
const char S57790[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.3; Trident/7.0; .NET4.0E; .NET4.0C)";
const char S57791[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.3; WOW64; Trident/7.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729)";
const char S57792[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.3; Win64; x64; Trident/7.0; .NET4.0E; .NET4.0C)";
const char S57793[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.3; WOW64; Trident/7.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3)";
const char S57794[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT886 Build/6.7.3-179_YIC-10) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57795[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 720)";
const char S57796[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 720; ANZ885)";
const char S57797[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 720; Vodafone)";
const char S57798[] = "SAMSUNG-SGH-i900/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900/1.0";
const char S57799[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900/1.0";
const char S57800[] = "SAMSUNG-SGH-i900Orange/AFHH1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S57801[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/JBHG5";
const char S57802[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/BUHJ2";
const char S57803[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/ARHG5";
const char S57804[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/ARHJ2";
const char S57805[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/ACHH1";
const char S57806[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/AEIB2";
const char S57807[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/AEHJ2";
const char S57808[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/ACHH3";
const char S57809[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/AMHK1";
const char S57810[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/AMHJ1";
const char S57811[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/AUHJ2";
const char S57812[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/AEHG4";
const char S57813[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/ACHJ2";
const char S57814[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/AMHG5";
const char S57815[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/MJHJ2";
const char S57816[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/NVHH2";
const char S57817[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/MIHH3";
const char S57818[] = "SAMSUNG-SGH-i900/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S57819[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/BAHK2";
const char S57820[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/JBHH1";
const char S57821[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/AEHH2";
const char S57822[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V";
const char S57823[] = "Mozilla/5.0 (compatible; Konqueror/3.4; Linux) KHTML/3.4.0 (like Gecko)";
const char S57824[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A43 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57825[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; A43 Build/FROYO)";
const char S57826[] = "Opera/9.80 (X11; Linux i686; U; en) Presto/2.10.229 Version/11.60";
const char S57827[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2";
const char S57828[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Version/5.1.1 Safari/536.26.14";
const char S57829[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_2; en-us) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3";
const char S57830[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7";
const char S57831[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22";
const char S57832[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) Version/5.1.1 Safari/536.28.10";
const char S57833[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) Version/5.1.1 Safari/536.30.1";
const char S57834[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_4; en-us) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3";
const char S57835[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) Version/5.1.9 Safari/534.59.8";
const char S57836[] = "Mercury/7.2 CFNetwork/609 Darwin/13.0.0";
const char S57837[] = "TBD753B Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S57838[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57839[] = "Mozilla/5.0 (Series40; Nokia308/05.85; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S57840[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/10B329 Safari/7534.48.3";
const char S57841[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/10B146 Safari/7534.48.3";
const char S57842[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/10B147 Safari/7534.48.3";
const char S57843[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/10B141 Safari/7534.48.3";
const char S57844[] = "NOKIA6720c";
const char S57845[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia6720c-1b/012.008 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S57846[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia6720c/012.008 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S57847[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; de; rv:1.9.2.17) Gecko/X Thunderbird/3.1.10 ThunderBrowse/3.3.5";
const char S57848[] = "Mozilla/5.0 (Linux; Android 4.0.4; LGL86C Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57849[] = "SAMSUNG-B2100/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S57850[] = "SAMSUNG-B2100-ORANGE/B2100AFIF3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S57851[] = "Vodafone/1.0/SAMSUNG-B2100/BUIC6 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S57852[] = "Vodafone/1.0/SAMSUNG-B2100/BUJA1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S57853[] = "Vodafone/1.0/SAMSUNG-B2100/BUIF2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S57854[] = "Mozilla/5.0 (Linux; Android 4.2; toroplus Build/JOP40C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57855[] = "Mozilla/5.0 (Linux; Android 4.2.1; i777 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57856[] = "Mozilla/5.0 (Linux; Android 4.2.1; UTOOPadM80E Build/E8HD-ELITE-1280x768) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S57857[] = "Mozilla/5.0 (Linux; Android 4.2.1; YOUPU U6 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57858[] = "Mozilla/5.0 (Linux; Android 4.2; Full AOSP on Manta Build/JVP15P) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S57859[] = "Mozilla/5.0 (Linux; Android 4.2.2; X2-Lite Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S57860[] = "Mozilla/5.0 (Linux; Android 4.2.2; MBX reference board (g18ref) Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S57861[] = "Mozilla/5.0 (Linux; Android 4.2.2; X2-16GB Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S57862[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9506 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S57863[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SGH-T399 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S57864[] = "Mozilla/5.0 (Linux; Android 4.2.1; X130_HD Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57865[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SPH-L520 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S57866[] = "Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E310L Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S57867[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-au; SAMSUNG GT-I9197 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S57868[] = "Mozilla/5.0 (Linux; Android 4.2.2; STB9097 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S57869[] = "Mozilla/5.0 (Linux; Android 4.2.1; Glory 2 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57870[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG-SM-C105A Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S57871[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG-SGH-I257 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S57872[] = "Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E310S Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S57873[] = "Mozilla/5.0 (Linux; Android 4.2.2; CRXT 1131 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S57874[] = "Mozilla/5.0 (Linux; Android 4.2.1; ZTE V880H Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57875[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SCH-R960 USCC Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S57876[] = "Mozilla/5.0 (Linux; Android 4.2; Full JellyBean on Mako Build/LGEP19) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57877[] = "Mozilla/5.0 (Linux; Android 4.2.1; signal77_cu_jb Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57878[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SPH-L600 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S57879[] = "Mozilla/5.0 (Linux; Android 4.2.2; S1A Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S57880[] = "Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E310K/KKUAMH3 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S57881[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG-SGH-I337Z Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S57882[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-au; SAMSUNG GT-I9507 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S57883[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGP321 Build/10.1.1.A.1.307) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S57884[] = "NokiaC2-05/2.0 (08.69) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S57885[] = "NokiaC2-05.1/2.0 (08.69) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S57886[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus 7 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S57887[] = "SAMSUNG-SGH-A897/A897UCJC1; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.15304; U; en-US) Opera 9.50";
const char S57888[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Iron/7.0.520.0 Chrome/7.0.520.0 Safari/534.7";
const char S57889[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR910L Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57890[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR910L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57891[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaC6-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S57892[] = "NokiaC6-00/UCWEB8.8.0.212/50/800";
const char S57893[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac6-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile  3gpp-gba";
const char S57894[] = "Mozilla/5.0 (Linux; Android 4.2.2; V370 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57895[] = "Mozilla/5.0 (Linux; Android 4.2.1; V370 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57896[] = "Opera/9.80 (Windows NT 6.2; WOW64; U; en) Presto/2.10.229 Version/11.64";
const char S57897[] = "Mozilla/5.0 (Linux; Android 4.2.2; HUAWEI P6-U06 Build/HuaweiP6-U06) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57898[] = "Mozilla/5.0 (Linux; Android 4.2.2; HUAWEI P6-U06-orange Build/HuaweiP6-U06) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57899[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/534.53.11 (KHTML, like Gecko) Version 4.0.5 Safari/534.53.11";
const char S57900[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/534.52.7 (KHTML, like Gecko) Version 4.0.5 Safari/534.52.7";
const char S57901[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en_US) AppleWebKit/530.19.2 (KHTML, like Gecko) Dreamweaver/11.0.4.4993 Version/4.0.2 Safari/530.19.2";
const char S57902[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/534.55.3 (KHTML, like Gecko) Version 4.0.5 Safari/534.55.3";
const char S57903[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/534.56.5 (KHTML, like Gecko) Version 4.0.5 Safari/534.56.5";
const char S57904[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/533.21.1 (KHTML, like Gecko) Version 4.0.5 Safari/533.21.1";
const char S57905[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/536.25 (KHTML, like Gecko) Version 4.0.5 Safari/536.25";
const char S57906[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/536.26.14 (KHTML, like Gecko) Version 4.0.5 Safari/536.26.14";
const char S57907[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/534.57.7 (KHTML, like Gecko) Version 4.0.5 Safari/534.57.7";
const char S57908[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/536.28.10 (KHTML, like Gecko) Version 4.0.5 Safari/536.28.10";
const char S57909[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/536.30.1 (KHTML, like Gecko) Version 4.0.5 Safari/536.30.1";
const char S57910[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/534.59.8 (KHTML, like Gecko) Version 4.0.5 Safari/534.59.8";
const char S57911[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; IM-A710K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57912[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; IM-A710K Build/GINGERBREAD)";
const char S57913[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; IM-A710K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57914[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; IM-A710K Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S57915[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; IM-A710K Build/GINGERBREAD--XiaoXingGe) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57916[] = "Nokia3710fold/5.0 (04.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S57917[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonIS12S Build/6.0.D.0.272) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57918[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonIS12S Build/6.0.D.0.270) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57919[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonIS12S Build/6.0.D.0.271) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57920[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; IS12S Build/6.0.D.0.272) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57921[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; IS12S Build/6.0.D.0.271) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57922[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonIS12S Build/6.0.D.0.274) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57923[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; IS12S Build/6.0.D.0.274) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57924[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonIS12S Build/6.0.D.0.275) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57925[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; IS12S Build/6.0.D.0.275) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57926[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; IS12S Build/6.0.D.0.274)";
const char S57927[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:15.0) Gecko/X Firefox/15.0-x64 PaleMoon/15.0-x64";
const char S57928[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X Firefox/15.0 PaleMoon/15.0";
const char S57929[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae63-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S57930[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE63) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S57931[] = "NokiaE63/UCWEB8.9.0.253/28/999";
const char S57932[] = "Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Safari/535.19 Silk-Accelerated=false";
const char S57933[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One V Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57934[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Iron/25.0.1400.0 Chrome/25.0.1400.0 Safari/537.22";
const char S57935[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.22 (KHTML, like Gecko) Iron/25.0.1400.0 Chrome/25.0.1400.0 Safari/537.22";
const char S57936[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T15A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57937[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T15A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57938[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P880 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57939[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P880g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S57940[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:25.0) Gecko/X Firefox/25.0";
const char S57941[] = "Mozilla/5.0 (Windows NT 5.1; rv:25.0) Gecko/X Firefox/25.0 AlexaToolbar/amznf-3.0.X";
const char S57942[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S57943[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P3110 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S57944[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-M840 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57945[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en) Presto/2.10.289 Version/12.00";
const char S57946[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en) Presto/2.10.289 Version/12.01";
const char S57947[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EGP007 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57948[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; HTC Desire Build/VJ CM10.1; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57949[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC Desire Build/VJ CM10.1; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57950[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC Desire Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57951[] = "Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Safari/535.19 Silk-Accelerated=true";
const char S57952[] = "BlackBerry8350i/4.6.1.313 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/103";
const char S57953[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7510 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57954[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7510 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57955[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7510 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57956[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7510 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57957[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-P7510 Build/JRO03L; CyanogenMod-AlexandrOid 4.1.2 Win 8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57958[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-P7510 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57959[] = "Flipboard/1.9.8 CFNetwork/609 Darwin/13.0.0";
const char S57960[] = "Mozilla/5.0 (Linux; Android 4.0.4; C5155 Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57961[] = "Mozilla/5.0 (Linux; Android 4.0.4; Rise Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S57962[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21i Build/11.0.A.4.22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57963[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21i2 Build/11.0.A.6.5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57964[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21a Build/11.0.A.0.16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57965[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21a2 Build/11.0.A.1.8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S57966[] = "Mozilla/5.0 (Linux; Android 4.0.3;  HTC EVO 3D X515m Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S57967[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515m Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57968[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515a Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57969[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC_X515E Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S57970[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9308 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S57971[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9308 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57972[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9308 Build/IMM76D)";
const char S57973[] = "Opera/9.80 (X11; Linux x86_64; Edition Linux Mint) Presto/2.12.388 Version/12.11";
const char S57974[] = "Lenovo A390_ROW:Mozilla/5.0 (Linux; U;  Android 4.0.4; zh-cn; Lenovo A390_ROW/S030) AppleWebKit534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57975[] = "Lenovo-A390t_TD/S100 Release/11.2012 Mozilla/5.0 (Linux; U; Android 4.0.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57976[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo A390t Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57977[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A390 Build/LenovoLePhone) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57978[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lenovo A390_ROW Build/LenovoLePhone) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57979[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo A390_ROW Build/IMM76I)";
const char S57980[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo A390t Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S57981[] = "Lenovo A390 Linux/3.0.13 Android/4.0.4 Release/04.03.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S57982[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lenovo A390_ROW Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57983[] = "Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo A390t Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S57984[] = "Mozilla/5.0 (Linux; U;  Android 4.0.4; zh-cn; Lenovo A390_ROW/S030) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57985[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A390 Build/LenovoLePhone) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S57986[] = "Lenovo A390_ROW:Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A390_ROW/S030) AppleWebKit534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57987[] = "Mozilla/5.0 (compatible; Konqueror/2.2.2; Linux 2.4.14-xfs; X11; i686)";
const char S57988[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; GT-P1000 Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57989[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57990[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57991[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S57992[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P1000 Build/IMM76L)";
const char S57993[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P1000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57994[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P1000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57995[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S57996[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-P1000 Build/IML74K)";
const char S57997[] = "Logos/1.1.3.2 (Linux; U; Android 4.0.4; ko-KR; GT-P1000N Build/cm_p1n-eng 4.0.4 IMM76I eng.humberos.X.X test-keys)";
const char S57998[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; GT-P1000 Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S57999[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; GT-P1000 Build/MjhieuS3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58000[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P1000L Build/IMM76L)";
const char S58001[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; GT-P1000 Build/Smartfren.andromaxtab7.0.v.27.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58002[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S58003[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; vi-vn; GT-P1000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58004[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58005[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P1000 Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S58006[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Bu AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58007[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S58008[] = "Logos/2.2.3 (Linux; U; Android 4.0.4; en-US; GT-P1000 Build/IMM76I)";
const char S58009[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P1000 Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58010[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P1000 Build/IMM76L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S58011[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P1000 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S58012[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IMM76L) Daily Mail Online/1.8.3";
const char S58013[] = "Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-US; GT-P1000 Build/cm_p1-userdebug 4.0.4 IMM76L eng.jenkins.X.X test-keys)";
const char S58014[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; GT-P1000 Build/Smartfren.androtab.v25.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58015[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; GT-P1000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58016[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P1000 Build/IMM76L) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)";
const char S58017[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/Smartfren.andromaxtab7.0.v.37.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58018[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58019[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P1000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58020[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; GT-P1000 Build/MjhieuGalaxyNote) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58021[] = "Groupon/2.1.2483 (Android 4.0.4; Samsung GT-P1000 / Samsung GT-P1000; )[preload=false;locale=en_US;clientidbase=android-google]";
const char S58022[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P1000 Build/R13.4-rc2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58023[] = "Flipboard/1.9.8 CFNetwork/548.1.4 Darwin/11.0.0";
const char S58024[] = "NetFront/3.5.1(BREW 5.0.2.1; U; en-us; SAMSUNG; NetFront/3.5.1/WAP) PayLo M270-PLB";
const char S58025[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) rekonq/2.2.1 Safari/534.34";
const char S58026[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.21 (KHTML, like Gecko) rekonq/2.2.1 Safari/537.21";
const char S58027[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Huawei-H867G Build/HuaweiH867G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58028[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S58029[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L900 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S58030[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S58031[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T889 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S58032[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7105 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S58033[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I317M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S58034[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI H868C Build/HuaweiH868C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58035[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI H868C Build/HuaweiH868C)";
const char S58036[] = "Mozilla/5.0 (Linux; Android 4.1.2; PantechP8010 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S58037[] = "SAMSUNG-GT-E2120L/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S58038[] = "SAMSUNG-GT-E2120B/E2120BDDIL1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S58039[] = "SAMSUNG-GT-E2120i/E2120iDDJE1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S58040[] = "SAMSUNG-GT-E2120/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S58041[] = "Mozilla/5.0 (Windows NT 4.0; rv:14.0) Gecko/X Firefox/14.0.1";
const char S58042[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SPH-M820-BST Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58043[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SPH-M820 Build/Build_Ver) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58044[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; SPH-M820-BST Build/FROYO)";
const char S58045[] = "Mozilla/5.0 (Windows NT 4.0; rv:13.0) Gecko/X Firefox/13.0.1";
const char S58046[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-78) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S58047[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/11A4372q Safari/8536.25";
const char S58048[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/11A465 Safari/8536.25";
const char S58049[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/11A501 Safari/8536.25";
const char S58050[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID7125 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58051[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID7125 Build/GRI40)";
const char S58052[] = "Logos/0.9.4 (Linux; U; Android 2.3.3; en_US; MID7125 Build/MID7125 X)";
const char S58053[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D700 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58054[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D700 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58055[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58056[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58057[] = "Mozilla/5.0 (Series40; NokiaC2-01/11.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S58058[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9100G Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i9100g";
const char S58059[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9100 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i9100";
const char S58060[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9100 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i9100";
const char S58061[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9100G Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i9100g";
const char S58062[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9100 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i9100";
const char S58063[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9100 Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i9100";
const char S58064[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-gb; GT-I9100 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2.0/i9100";
const char S58065[] = "SAMSUNG-SGH-G600-VODA/G600ACHF1 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S58066[] = "SAMSUNG-SGH-G600/G600MSGH2 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S58067[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P7511 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58068[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7511 Build/IMM76D)";
const char S58069[] = "NokiaC3-00/5.0 (08.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac3-00) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S58070[] = "Mozilla/16.0 (Windows NT 6.2; Win64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/32.0.1297.0 Safari/8536.25";
const char S58071[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/32.0.1659.2 Safari/537.36";
const char S58072[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/32.0.1659.2 Safari/537.36";
const char S58073[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5303 Build/12.0.A.2.245) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S58074[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5303 Build/12.0.A.2.245) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S58075[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0100 Build/HuaweiG510-0100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S58076[] = "Mozilla/5.0 (Linux; Android 4.1.1; Orange Daytona Build/HuaweiG510-0200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S58077[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0200 Build/HuaweiG510-0200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S58078[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0251 Build/HuaweiG510-0251) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S58079[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S58080[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58081[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Transformer Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58082[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58083[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaE72-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S58084[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiae72-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile  3gpp-gba";
const char S58085[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaE72-2) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S58086[] = "NOKIAE72-1/UCWEB8.8.0.212/28/999";
const char S58087[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaE72) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S58088[] = "Mozilla/5.0 Firefox/3.6.8";
const char S58089[] = "Firefox/3.6.3";
const char S58090[] = "Mozilla/5.0 Firefox/3.6.12";
const char S58091[] = "Mozilla/5.0 (compatible; rv:1.9.2) Gecko/X Firefox/3.6";
const char S58092[] = "Mozilla/5.0 Gecko/X Firefox/3.6.10";
const char S58093[] = "Mozilla/5.0 (OS/2; U; Warp 4.5; en-US; rv:1.9.2.13) Gecko/X Firefox/3.6.13";
const char S58094[] = "Firefox/3.6.27";
const char S58095[] = "AppleCoreMedia/1.0.0.8G4 (iPhone; U; CPU OS 4_3_1 like Mac OS X; en_us)";
const char S58096[] = "AppleCoreMedia/1.0.0.8F190 (iPhone; U; CPU OS 4_3 like Mac OS X; en_us)";
const char S58097[] = "AppleCoreMedia/1.0.0.8H7 (iPhone; U; CPU OS 4_3_2 like Mac OS X; en_us)";
const char S58098[] = "AppleCoreMedia/1.0.0.8J2 (iPhone; U; CPU OS 4_3_3 like Mac OS X; en_us)";
const char S58099[] = "AppleCoreMedia/1.0.0.8L1 (iPhone; U; CPU OS 4_3_5 like Mac OS X; en_us)";
const char S58100[] = "AppleCoreMedia/1.0.0.8K2 (iPhone; U; CPU OS 4_3_4 like Mac OS X; en_us)";
const char S58101[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-S5570 Build/GRI41; Cyanogen-MUSIC2.1ro.setupwizard.mode=OPTIONAL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58102[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5570 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58103[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5570 Build/GRI41; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58104[] = "Mozilla/5.0 (Linux; Android 4.1.2; N861 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S58105[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26i Build/6.1.A.2.45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S58106[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; Titanium Build/ECLAIR) AppleWebKit/530.17+ (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58107[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; Titanium Build/ESE81) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58108[] = "Dalvik/1.2.0 (Linux; U; Android 2.1-update1; Titanium Build/ESE81)";
const char S58109[] = "SPH-M575-VMUB NetFront/3.5.1 MMP/2.0";
const char S58110[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MID1024 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58111[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; MID1024 Build/FRF91)";
const char S58112[] = "Logos/0.9.1 (Linux; U; Android 2.2; en_US; MID1024 Build/MID Serials 2.2 FRF91 X.X)";
const char S58113[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; AN10G2-LN Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58114[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; AN10G2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58115[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.1; AN10G2 Build/GINGERBREAD)";
const char S58116[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; fr-fr; AN10G2-LZ Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58117[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; AN10G2-KN Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58118[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I777 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58119[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I777 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58120[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I777 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58121[] = "Mozilla/5.0 (Linux; Android 4.1.1; A210 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S58122[] = "JUC(Linux;U;Android2.3.5;Zh_cn;GT-S5360;240*320;)UCWEB7.8.0.95/139/444";
const char S58123[] = "JUC(Linux;U;Android2.3.6;Zh_cn;GT-S5360;240*320;)UCWEB7.8.0.95/139/444";
const char S58124[] = "Mozilla/5.0(Linux;U;Android2.3.6;Zh_cn;GT-S5360;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/444";
const char S58125[] = "Mozilla/5.0 (compatible; Windows NT 6.0) Gecko/X Firefox/3.5 NjuiceBot";
const char S58126[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-GB; rv:1.9.1.2) Firefox/3.5.5 spellr.usBot/4.0 (http://checkdog.com/faqs.php) Gecko/X";
const char S58127[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-GB; rv:1.9.1.2) Firefox/3.5.5 spellr.usBot/4.4 (http://checkdog.com/faqs.php) Gecko/X";
const char S58128[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-GB; rv:1.9.1.2) Firefox/3.5.5 spellr.usBot/4.6 (http://checkdog.com/faqs.php) Gecko/X";
const char S58129[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-GB; rv:1.9.1.2) Firefox/3.5.5 spellr.usBot/4.9 (http://checkdog.com/faqs.php) Gecko/X";
const char S58130[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-GB; rv:1.9.1.2) Firefox/3.5.5 CheckDogBot/4.9 (http://checkdog.com/faqs.php) Gecko/X";
const char S58131[] = "Nokia110/2.0 (03.04) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S58132[] = "Nokia110/2.0 (03.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia110";
const char S58133[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S58134[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xoom Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S58135[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ505 Build/Moto_MZ505) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S58136[] = "Mozilla/5.0 (Linux; Android 4.0.3; Xoom Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S58137[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Kindle Fire Build/IMM76D; CyanogenMod-CM9 Reloaded) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58138[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Amazon Kindle Fire Build/IMM76D; CyanogenMod-CM9 Reloaded v1.3 OTTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58139[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Amazon Kindle Fire Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58140[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Amazon Kindle Fire Build/IML74k; CyanogenMod-Energy CM9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58141[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Amazon Kindle Fire Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58142[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Amazon Kindle Fire Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58143[] = "Mozilla/5.0 (Linux; Android 4.0.3;  Slider SL101 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S58144[] = "Mozilla/5.0 (Linux; Android 4.0.3; Slider SL101 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S58145[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-H Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58146[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-F Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58147[] = "Mozilla/5.0 (Linux; Android 4.0.4; A2107A-H Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58148[] = "Mozilla/5.0 (Linux; Android 4.0.4; SmartTabII7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58149[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26w Build/6.2.B.0.200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S58150[] = "Mozilla/5.0 (Linux; Android 4.1.2; N861 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S58151[] = "Mozilla/5.0 (Linux; Android 4.1.2; N861 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S58152[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT914 Build/2_32A_2012) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S58153[] = "Mozilla/4.0 (compatible; MSIE 5.0; Linux 2.4.18-rmk7-pxa3-embedix armv5tel; 480x640) Opera 6.0 [en]";
const char S58154[] = "Opera/6.x (Linux 2.4.8-26mdk i686; U) [en]";
const char S58155[] = "Mozilla/5.0 (Linux 2.4.18-18.7.x i686; U) Opera 6.03 [en]";
const char S58156[] = "Mozilla/4.0 (compatible; MSIE 5.0; Linux 2.4.20-4GB i686) Opera 6.03 [en]";
const char S58157[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; A200 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S58158[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; A200 Build/HTK55D)";
const char S58159[] = "Opera/9.80 (Windows NT 6.2; U; en) Presto/2.10.229 Version/11.61";
const char S58160[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Incredible 2 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58161[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Incredible 2 Build/GRI40; Condemned CM7 V1.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58162[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.1pre) Gecko/X Ubuntu/8.10 (intrepid) Namoroka/3.6.1pre";
const char S58163[] = "Mozilla/5.0 (Windows NT 6.0; rv:19.0) Gecko/X Firefox/19.0 PaleMoon/19.0.2";
const char S58164[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:19.0) Gecko/X Firefox/19.0 PaleMoon/19.0.2-x64";
const char S58165[] = "Mozilla/5.0 (Linux; Android 4.2.2; ja-jp; SC-04E Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S58166[] = "Xda_Mantle/240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S58167[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) USCCHTC6850";
const char S58168[] = "Treo850/v0100 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S58169[] = "Xda_diamond/480x640 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S58170[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) i-mate 9502; 480x640";
const char S58171[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) T-Mobile_Cleopatra";
const char S58172[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) ASUS-P552w/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S58173[] = "HPiPAQVoiceMessenger/1.0/Mozilla/4.0  (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) C11G25L/1.0";
const char S58174[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) i-mate 8150/ 480x640";
const char S58175[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Sprint:MotoQ9c";
const char S58176[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) 320x320; VZW; UTStar-XV6800; Window Mobile 6.1 Professional;";
const char S58177[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) ASUSP750/1.0";
const char S58178[] = "HPiPAQVoiceMessenger/1.0/Mozilla/4.0  (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Smartphone;240x320";
const char S58179[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Alltel:MotoQ9c";
const char S58180[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) ASUS-P565/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S58181[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; X610/2.0 Profile";
const char S58182[] = "HTC_Touch_3G Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S58183[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE, BrailleNote; IEMobile 7.11)";
const char S58184[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PMI920 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58185[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; PMI920 Build/GRJ22)";
const char S58186[] = "Mozilla/5.0 (Linux; Android 4.0.3; Clear Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S58187[] = "Mozilla/5.0 (Linux; Android 4.0.4; N900 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58188[] = "Mozilla/5.0 (Linux; Android 4.0.4; MOBHIG0080 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58189[] = "Mozilla/5.0 (Linux; Android 4.0.4; 9300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58190[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAD1001 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S58191[] = "Mozilla/5.0 (Linux; Android 4.0.4; M1005D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S58192[] = "Mozilla/5.0 (Linux; Android 4.0.4; A13-DLR Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58193[] = "Mozilla/5.0 (Linux; Android 4.0.4; mfld_dv10 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22";
const char S58194[] = "Mozilla/5.0 (Linux; Android 4.0.4; M1005 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S58195[] = "Mozilla/5.0 (Linux; Android 4.0.4; tmn smart a18 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58196[] = "Mozilla/5.0 (Linux; Android 4.0.4; N1 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58197[] = "Mozilla/5.0 (Linux; Android 4.0.4; T7 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S58198[] = "Mozilla/5.0 (Linux; Android 4.0.3; Explosion Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58199[] = "Mozilla/5.0 (Linux; Android 4.0.3; 740RS Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58200[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB9008GB Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58201[] = "Mozilla/5.0 (Linux; Android 4.0.4; BASE_Lutea_3 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58202[] = "Mozilla/5.0 (Linux; Android 4.0.4; BT210 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S58203[] = "Mozilla/5.0 (Linux; Android 4.0.3; Infinity Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58204[] = "Mozilla/5.0 (Linux; Android 4.0.3; CTA Build/ICS.casper.X) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S58205[] = "Mozilla/5.0 (Linux; Android 4.0.3; AN13FP Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S58206[] = "Mozilla/5.0 (Linux; Android 4.0.4; QW TB-1217 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S58207[] = "Mozilla/5.0 (Linux; Android 4.0.4; A11Note Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58208[] = "Mozilla/5.0 (Linux; Android 4.0.3; StarPAD 7 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58209[] = "Mozilla/5.0 (Linux; Android 4.0.4; M1001 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S58210[] = "Mozilla/5.0 (Linux; Android 4.0.4; INFINITY_PASSION Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58211[] = "Mozilla/5.0 (Linux; Android 4.0.4; Q75 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58212[] = "Mozilla/5.0 (Linux; Android 4.0.3; SmartPadv10 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S58213[] = "Mozilla/5.0 (Linux; Android 4.0.4; TM450 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58214[] = "Mozilla/5.0 (Linux; Android 4.0.4; JY139 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58215[] = "Mozilla/5.0 (Linux; Android 4.0.4; R10S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22";
const char S58216[] = "Mozilla/5.0 (Linux; Android 4.0.4; CT710 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S58217[] = "Mozilla/5.0 (Linux; Android 4.0.3; QW TB-1207 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S58218[] = "Mozilla/5.0 (Linux; Android 4.0.4; GEM9212 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S58219[] = "Mozilla/5.0 (Linux; Android 4.0.4; Genius G1 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S58220[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB292 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S58221[] = "Mozilla/5.0 (Linux; Android 4.0.4; K-BOOKMW Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58222[] = "Mozilla/5.0 (Linux; Android 4.0.4; A721 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58223[] = "Mozilla/5.0 (Linux; Android 4.0.4; MTC_968 Build/ICECREAM) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58224[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; ZP980 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58225[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; ZP980 Build/JOP40D)";
const char S58226[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; ZP980H Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58227[] = "Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5130c-2) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S58228[] = "Nokia5130/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5130) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S58229[] = "Mozilla/5.0 (Windows NT 4.0; rv:20.0) Gecko/X Firefox/20.0";
const char S58230[] = "Mozilla/5.0 (Linux; Android 4.2.2; One X Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58231[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One X Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58232[] = "Mozilla/5.0 (Linux; Android 4.2.1; HTC One X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58233[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; V9C Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58234[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; V9C Build/GRJ22)";
const char S58235[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Light Tab Build/GRJ22) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58236[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ALCATEL ONE TOUCH 4030D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S58237[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ALCATEL ONE TOUCH 4030D Build/JRO03C)";
const char S58238[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A500 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58239[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Iconia A500 Build/JRO03H; CyanogenMod-Tegraowners JB ROM v8 (thor & digetx)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58240[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A500 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58241[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Iconia A500 Build/JRO03L; CyanogenMod-Tegraowners JB ROM v12 (thor & digetx)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58242[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A500 Build/JZO54K; CyanogenMod-Android 4.1.2 CM10 build) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58243[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9400 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58244[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; i9400 Build/IMM76D)";
const char S58245[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; PP4MT-7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58246[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; PP4MT-7 Build/IML74K)";
const char S58247[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4";
const char S58248[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4";
const char S58249[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One V Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S58250[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One V Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S58251[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; HTC_Lexikon Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58252[] = "Mozilla/5.0 (BREW; U; BREW 3.1.5; en) AppleWebKit/525.26 (KHTML, like Gecko) Polaris/7.0 Safari/525.26 480X800 LGE VX11000";
const char S58253[] = "Mozilla/4.0 (compatible; MSIE 10.0;";
const char S58254[] = "Mozilla/5.0 (compatible; MSIE 10.0; PPC; Trident/3.0)";
const char S58255[] = "Mozilla/5.0 (compatible; MSIE 10.0; ; Trident/6.0)";
const char S58256[] = "Mozilla/5.0 (compatible; MSIE 10.0; PPC; SV1; Trident/4.0)";
const char S58257[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE5-00)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S58258[] = "NOKIAE5-00/UCWEB8.2.0.116/28/800";
const char S58259[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE5-00.2)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S58260[] = "Mozilla/5.0 (Java; U; en-in; nokiae5-00) UCBrowser8.2.1.144/69/444/UCWEB Mobile  3gpp-gba";
const char S58261[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Vodafone 975 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30 SVN/130HVG2";
const char S58262[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L710 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2spr";
const char S58263[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I747 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2att";
const char S58264[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I747 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2att";
const char S58265[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L710 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2spr";
const char S58266[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-au; GT-I9305 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i9305";
const char S58267[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I747 Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2att";
const char S58268[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I747 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2.0/d2att";
const char S58269[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L710 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2spr";
const char S58270[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L710 Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2spr";
const char S58271[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I747 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2att";
const char S58272[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-05D Build/V08R31C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58273[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-05D Build/V09R32B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58274[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; F-05D Build/V08R31C)";
const char S58275[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-05D Build/V11R40A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58276[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-05D Build/V12R41A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58277[] = "Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58278[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E971 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58279[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6602 Build/10.3.1.A.0.244) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S58280[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6602 Build/10.3.1.A.0.244) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S58281[] = "Mozilla/5.0 (Linux; Android 4.0.3; MZ604 Build/I.7.1-34) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S58282[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ604 Build/I.7.1-45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S58283[] = "Nokia5000d-2/1.0 (04.89) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S58284[] = "Nokia5000d-2b/1.0 (03.54) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S58285[] = "Nokia5000/1.0 (06.32) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S58286[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM003SH Build/S4030) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58287[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM003SH Build/S4040) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58288[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM003SH Build/S4080) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58289[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM003SH Build/S4080) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30";
const char S58290[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM003SH Build/S4070) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58291[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SBM003SH Build/S4080)";
const char S58292[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM003SH Build/S4090) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58293[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM003SH Build/S4110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58294[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Barnes & Noble Nook HD+ Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58295[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Barnes & Noble Nook HD+ Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58296[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; BN NookHD+ Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1/ovation";
const char S58297[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/10B329 Safari/7534.48.3";
const char S58298[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/10B141 Safari/7534.48.3";
const char S58299[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/10B146 Safari/7534.48.3";
const char S58300[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/10B147 Safari/7534.48.3";
const char S58301[] = "NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac1-01) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S58302[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ604 Build/I.7.1-45) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S58303[] = "Mozilla/5.0 (Linux; Android 4.0.3; MZ604 Build/I.7.1-34) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S58304[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58305[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58306[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ALCATEL ONE TOUCH 4030A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S58307[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ALCATEL ONE TOUCH 4030A Build/JRO03C)";
const char S58308[] = "Mozilla/5.0 (X11; U; Linux i686; nl; rv:1.8.1.3) Gecko/X Firefox/2.0.0.3 (Ubuntu-edgy)";
const char S58309[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-GB; rv:1.8.1b1) Gecko/X BonEcho/2.0b1 (Ubuntu-edgy)";
const char S58310[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; OPPOX907 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58311[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; th-TH; OPPOX9017 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58312[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; X907 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S58313[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; X907 Build/IML74K)";
const char S58314[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; X9017 Build/IML74K)";
const char S58315[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru; OPPOX907 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58316[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Wildfire Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58317[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Wildfire Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58318[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Wildfire Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58319[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Wildfire Build/GRI40)";
const char S58320[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Wildfire Build/GRI40)";
const char S58321[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Wildfire Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58322[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Wildfire Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S58323[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Wildfire Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58324[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ARCHOS 101G10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58325[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xoom Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58326[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ505 Build/Moto_MZ505) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58327[] = "Mozilla/5.0 (Linux; Android 4.0.3; Xoom Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58328[] = "Mozilla/5.0 (Linux; Android 4.1.2; ME371MG Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S58329[] = "Mozilla/5.0 (Linux; Android 4.1.1; IdeaTabA2109A Build/JRO03R) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S58330[] = "Mozilla/5.0 (Linux; Android 4.0.3; U9200 Build/HuaweiU9200) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58331[] = "Mozilla/5.0 (Linux; Android 4.0.4; U9202L-1 Build/HuaweiU9202L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58332[] = "Mozilla/5.0 (Linux; Android 4.1.1; H100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58333[] = "Mozilla/5.0 (Linux; Android 4.1.1; CrystalQuadCore Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58334[] = "Mozilla/5.0 (Linux; Android 4.1.2; ThinkPadTablet Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58335[] = "Mozilla/5.0 (Linux; Android 4.1.2; A9600 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58336[] = "Mozilla/5.0 (Linux; Android 4.1.1; MiSS Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58337[] = "Mozilla/5.0 (Linux; Android 4.1.1; SZENIO 2500 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58338[] = "Mozilla/5.0 (Linux; Android 4.1.1; y7053 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58339[] = "Mozilla/5.0 (Linux; Android 4.1.2; B943 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58340[] = "Mozilla/5.0 (Linux; Android 4.1.1; G1 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58341[] = "Mozilla/5.0 (Linux; Android 4.1.2; H9500+ Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58342[] = "Mozilla/5.0 (Linux; Android 4.1.1; MD7305 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58343[] = "Mozilla/5.0 (Linux; Android 4.1.1; PMID720 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58344[] = "Mozilla/5.0 (Linux; Android 4.1.1; SZENIO 2000 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58345[] = "Mozilla/5.0 (Linux; Android 4.1.1; YOUPU U6 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58346[] = "Mozilla/5.0 (Linux; Android 4.1.1; AN7IG3 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58347[] = "Mozilla/5.0 (Linux; Android 4.1.2; Boost Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58348[] = "Mozilla/5.0 (Linux; Android 4.1.2; M7 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58349[] = "Mozilla/5.0 (Linux; Android 4.1.4; Note3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58350[] = "Mozilla/5.0 (Linux; Android 4.1.2; Thrive Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58351[] = "Mozilla/5.0 (Linux; Android 4.1.1; XTAB-781+ Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58352[] = "Mozilla/5.0 (Linux; Android 4.1.2; W125 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58353[] = "Mozilla/5.0 (Linux; Android 4.1.1; it900 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58354[] = "Mozilla/5.0 (Linux; Android 4.1.1; eZee Tab1004 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58355[] = "Mozilla/5.0 (Linux; Android 4.1.1; DPS Dual 7 Build/X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58356[] = "Mozilla/5.0 (Linux; Android 4.1.1; V2 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58357[] = "Mozilla/5.0 (Linux; Android 4.1.2; F600 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58358[] = "Mozilla/5.0 (Linux; Android 4.1.2; V72A Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58359[] = "Mozilla/5.0 (Linux; Android 4.1.1; Tablet-PC-4 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58360[] = "Mozilla/5.0 (Linux; Android 4.1.2; I110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58361[] = "Mozilla/5.0 (Linux; Android 4.1.1; MaxiIQ Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58362[] = "Mozilla/5.0 (Linux; Android 4.1.1; AMOI N820 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58363[] = "Mozilla/5.0 (Linux; Android 4.1.2; iQTab 3G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58364[] = "Mozilla/5.0 (Linux; Android 4.1.1; Engage 8 Build/Engage 8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S58365[] = "Mozilla/5.0 (Linux; Android 4.1.2; EG98 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58366[] = "Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI D2-0082 Build/HuaweiD2-0082) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58367[] = "Mozilla/5.0 (X11; Linux i686; rv:23.0) Gecko/X Firefox/23.0 SeaMonkey/2.20";
const char S58368[] = "Mozilla/5.0 (X11; Linux x86_64; rv:23.0) Gecko/X SeaMonkey/2.20";
const char S58369[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT30p Build/7.0.A.1.303) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S58370[] = "Mozilla/5.0 (Linux; Android 4.0.4; Event Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S58371[] = "Mozilla/5.0 (Windows; N; Windows NT 6.0; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.1.0.5001 Safari/532.4";
const char S58372[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6000.0);en-US;kml:2.2;client:Free;type:default)";
const char S58373[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6000.0);de-DE;kml:2.2;client:Free;type:default)";
const char S58374[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6000.0);en;kml:2.2;client:Free;type:default)";
const char S58375[] = "Mozilla/5.0 (Windows NT 6.1; rv:10.0.3) Gecko/X Firefox/10.0.3 Epic/10.0.3";
const char S58376[] = "NokiaC2-00/2.0 (03.82) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac2-00) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S58377[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58378[] = "Dalvik/1.4.0 (Linux; U; Android 4.1.1; GT-S5360 Build/GINGERBREAD)";
const char S58379[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; MyPhone A848 Duo Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58380[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; MyPhone A848i Duo Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58381[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; BLU STUDIO 5.3 II Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58382[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; BLU STUDIO 5.3 II Build/JZO54K)";
const char S58383[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.2) Gecko/X Firefox/3.6.2 CometBird/3.6.2";
const char S58384[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN; rv:1.9.2.13) Gecko/X Firefox/3.6.13 CometBrowser/3.6.13";
const char S58385[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.2.16) Gecko/X Firefox/3.6.16 CometBird/3.6.16";
const char S58386[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A860K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58387[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A860K/A860K1.20 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58388[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A860K/1.29 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58389[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A860K/1.38 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58390[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A860K Build/JZO54K)";
const char S58391[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A860K/1.43 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58392[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810P Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58393[] = "Mozilla/5.0 (Series40; NokiaX2-02/10.91; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.11";
const char S58394[] = "Mozilla/5.0 (Windows NT 5.0; rv:14.0) Gecko/X Firefox/14.0.1";
const char S58395[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.10.289 Version/12.02";
const char S58396[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; NookColor Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58397[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; T-01D Build/V41R39C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58398[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; T-01D Build/F0001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58399[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; T-01D Build/V39R38A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58400[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; T-01D Build/V43R41C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58401[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; T-01D Build/V40R39B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58402[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; T-01D Build/V43R41C)";
const char S58403[] = "SAMSUNG-GT-i8000T Opera 9.5 (Windows CE; Opera Mobi; U; en) DVJG1";
const char S58404[] = "SAMSUNG-GT-i8000T/1.0 (Windows CE; Opera Mobi; U; en) Opera 9.5";
const char S58405[] = "SAMSUNG-GT-i8000/1.0 Opera 9.5 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S58406[] = "SAMSUNG-GT-i8000V/NXXJD6 (Windows CE; Opera Mobi; U; en) Opera 9.5";
const char S58407[] = "SAMSUNG-GT-i8000H/1.0 (Windows CE; Opera Mobi; U; en) Opera 9.5";
const char S58408[] = "SAMSUNG-GT-i8000/1.0 (Windows CE; Opera Mobi; U; en) Opera 9.5";
const char S58409[] = "SAMSUNG-GT-i8000Orange/BVIJ1 (Windows CE; Opera Mobi; U; en) Opera 9.5";
const char S58410[] = "Mozilla/5.0 (Windows NT 4.10) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S58411[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; HTC Butterfly Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58412[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-tw; HTC_Butterfly Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58413[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC Butterfly Build/JDQ39)";
const char S58414[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-tw; HTC Butterfly Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58415[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HTC_X920e Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58416[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HTC X920e Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58417[] = "Nokia2680s-2/1.0 (05.28) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S58418[] = "Nokia2680s-2/1.0 (06.17) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2680s-2";
const char S58419[] = "Nokia2680s-2b/1.0 (05.28) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S58420[] = "Nokia2680s/1.0 (06.17) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S58421[] = "Nokia2680s_CMCC/1.0 (06.17) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S58422[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21i Build/11.0.A.4.22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58423[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21i2 Build/11.0.A.6.5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58424[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21a Build/11.0.A.0.16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58425[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21a2 Build/11.0.A.1.8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58426[] = "Reeder/1.5.7 CFNetwork/548.1.4 Darwin/11.0.0";
const char S58427[] = "Mozilla/5.0 (X11; U; FreeBSD amd64; en-US; rv:1.8.1.20) Gecko/X Firefox/2.0.0.20 Kazehakase/0.5.4";
const char S58428[] = "NokiaC2-00/2.0 (03.82) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac2-00) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S58429[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B117 Twitter for iPhone";
const char S58430[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Moto_ME860 Build/Blur_Version.45.0.25.ME860.Retail.en.05 Flex/P011) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58431[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ME860 Build/4.5.3-118_OLY-14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58432[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; ME860 Build/4.5.2A-74_OLH-25)";
const char S58433[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ME860 Build/4.5.2A-74_OLH-25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58434[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ME860 Build/4.5.3-118_OLY-14) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S58435[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; Moto_ME860 Build/Blur_Version.45.29.0.ME860.Retail.en.05 Flex/P011) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58436[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ME860 Build/4.5.2A-74_OLH-25) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S58437[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME860 Build/4.5.3-118_OLY-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58438[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME860 Build/4.5.2A-74_OLH-25) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S58439[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ME860 Build/4.5.3-118_OLY-14) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S58440[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; ME860 Build/4.5.2A-74_OLH-29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58441[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315";
const char S58442[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S58443[] = "Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S58444[] = "Mozilla/5.0 (Windows NT 5.0; rv:13.0) Gecko/X Firefox/13.0.1";
const char S58445[] = "Mozilla/5.0 (Linux; Android 4.0.3; NS-13T001 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S58446[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S58447[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S58448[] = "KWC-Torino/ UP.Browser/7.2.7.2.561 (GUI) MMP/2.0";
const char S58449[] = "AppleCoreMedia/1.0.0.12A269 (Macintosh; U; Intel Mac OS X 10_8; en_us)";
const char S58450[] = "AppleCoreMedia/1.0.0.12B19 (Macintosh; U; Intel Mac OS X 10_8_1; en_us)";
const char S58451[] = "AppleCoreMedia/1.0.0.12C54 (Macintosh; U; Intel Mac OS X 10_8_2; en_us)";
const char S58452[] = "AppleCoreMedia/1.0.0.12C60 (Macintosh; U; Intel Mac OS X 10_8_2; en_us)";
const char S58453[] = "AppleCoreMedia/1.0.0.12C3006 (Macintosh; U; Intel Mac OS X 10_8_2; en_us)";
const char S58454[] = "AppleCoreMedia/1.0.0.12C3012 (Macintosh; U; Intel Mac OS X 10_8_2; en_us)";
const char S58455[] = "AppleCoreMedia/1.0.0.12C2034 (Macintosh; U; Intel Mac OS X 10_8_2; en_us)";
const char S58456[] = "AppleCoreMedia/1.0.0.12C60 (Macintosh; U; Intel Mac OS X 10_8_2; de_de)";
const char S58457[] = "AppleCoreMedia/1.0.0.12C3103 (Macintosh; U; Intel Mac OS X 10_8_2; en_us)";
const char S58458[] = "AppleCoreMedia/1.0.0.12D78 (Macintosh; U; Intel Mac OS X 10_8_3; en_us)";
const char S58459[] = "AppleCoreMedia/1.0.0.12E55 (Macintosh; U; Intel Mac OS X 10_8_4; en_us)";
const char S58460[] = "AppleCoreMedia/1.0.0.12E55 (Macintosh; U; Intel Mac OS X 10_8_4; de_de)";
const char S58461[] = "AppleCoreMedia/1.0.0.12E3067 (Macintosh; U; Intel Mac OS X 10_8_4; en_us)";
const char S58462[] = "AppleCoreMedia/1.0.0.12E55 (Macintosh; U; Intel Mac OS X 10_8_4; fr_fr)";
const char S58463[] = "AppleCoreMedia/1.0.0.12F37 (Macintosh; U; Intel Mac OS X 10_8_5; en_us)";
const char S58464[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMID4311 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58465[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Mobile Safari/535.19 Silk-Accelerated=true";
const char S58466[] = "Mozilla/5.0 (Linux; Android 4.0.3; LIFETAB_P9514 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S58467[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9260 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58468[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9260 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58469[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9260 Build/JZO54K)";
const char S58470[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-i9260 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58471[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-I9260 Build/JRO03C)";
const char S58472[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9260 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S58473[] = "Mozilla/5.0 (compatible; TourlentaScanner/0.6; +http://tourlenta.com)";
const char S58474[] = "Mozilla/5.0 (compatible; parsijoo-search-engine +http://www.parsijoo.ir/)";
const char S58475[] = "AskQuickly v2 (http://askquickly.org/)";
const char S58476[] = "mb-searchengine 1.0";
const char S58477[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I777 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S58478[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I777 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S58479[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11";
const char S58480[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11";
const char S58481[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11";
const char S58482[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11";
const char S58483[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58484[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S58485[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S58486[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC T328w Build/IML74K)";
const char S58487[] = "Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S58488[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JOP40D; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58489[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39; LiquidSmooth-Liquid-JB-v2.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58490[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/toro";
const char S58491[] = "Nokia2626/2.0 (06.80) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S58492[] = "Opera/9.80 (BREW; Opera Mini/5.1/27.2338; U; en) Presto/2.8.119 320X240 ZTE F-450";
const char S58493[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS920 4G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58494[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT901 Build/7.7.1Q-144_SLS_S240-58) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58495[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT901 Build/7.7.1Q-144_SLS_S240-58) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S58496[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST23i Build/11.0.A.5.5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S58497[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST23a Build/11.0.A.2.16) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S58498[] = "Mozilla/5.0 (Linux; Android 4.1.1; Prism II Build/HuaweiU8686) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58499[] = "Mozilla/5.0 (Linux; U; Android 1.6; es-mx; U20a Build/1.2.A.1.181) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S58500[] = "Mozilla/5.0 (Linux; U; Android 1.6; pt-br; U20a Build/1.2.A.1.174) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S58501[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-au; SonyEricssonU20a Build/1.1.A.0.8) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S58502[] = "Mozilla/5.0 (Linux; U; Android 1.6; es-mx; U20a Build/1.1.A.0.8) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S58503[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-au; U20a Build/1.1.A.0.12) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S58504[] = "Mozilla/5.0 (Linux; U; Android 1.6; es-mx; U20a Build/1.1.A.0.9) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S58505[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; U20a Build/1.2.A.1.183) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S58506[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; KM-E100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58507[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; KM-E100 Build/IMM76D)";
const char S58508[] = "Mozilla/5.0 (LG-GD880/V10b Browser/AppleWebKit/531 Widget/LGMW/3.0 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S58509[] = "Mozilla/5.0 LG-GD880 Browser/AppleWebKit/531 Widget/LGMW/3.0 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1Configuration/CLDC-1.1";
const char S58510[] = "Mozilla/5.0 (LG/GD880/V10a Browser/AppleWebKit/531 Widget/LGMW/3.0 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S58511[] = "Mozilla/5.0 (LG-GD880-Orange/V10d Browser/AppleWebKit/531 Widget/LGMW/3.0 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S58512[] = "Mozilla/5.0 (LG-GD880-Orange/V10b Browser/AppleWebKit/531 Widget/LGMW/3.0 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S58513[] = "Mozilla/5.0 (Vodafone/1.0/LG-GD880/V10e Browser/AppleWebKit/531 Widget/LGMW/3.0 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S58514[] = "Mozilla/5.0 (LG/GD880/V10e Browser/AppleWebKit/531 Widget/LGMW/3.0 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S58515[] = "Mozilla/5.0 (Vodafone/1.0/LG-GD880/V10a Browser/AppleWebKit/531 Widget/LGMW/3.0 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S58516[] = "Mozilla/5.0 (Vodafone/1.0/LG-GD880/V10b Browser/AppleWebKit/531 Widget/LGMW/3.0 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S58517[] = "Mozilla/5.0 (LG/GD880 Browser/AppleWebKit/531 Widget/LGMW/3.0 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S58518[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; pandigitalsprnova1/sourceidDLX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58519[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P5100 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58520[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P5100 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58521[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P5100 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58522[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P5100 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1.2/p5100";
const char S58523[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E610 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S58524[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S58525[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S58526[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E610v Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S58527[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612f Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S58528[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-05D Build/7.0.D.1.117) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58529[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySO-05D Build/7.0.D.1.117) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58530[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySO-05D Build/7.0.D.1.130) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58531[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-05D Build/7.0.D.1.130) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58532[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySO-05D Build/7.0.D.1.137) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58533[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-05D Build/7.0.D.1.137) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58534[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; MyPhone A919 Duo Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58535[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MyPhone A919 Duo Build/IMM76D)";
const char S58536[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58537[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58538[] = "Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I727 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58539[] = "BlackBerry8330/5.0.0.438 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/389";
const char S58540[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LePanII Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58541[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LePanII Build/IMM76D)";
const char S58542[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT925 Build/7.7.1Q-144_VQL_S3-58) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S58543[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR HD Build/7.7.1Q-144_VQW_S334-25) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S58544[] = "LGE-VN251P/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S58545[] = "LGE-VN251L/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S58546[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I905 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S58547[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MW0711 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58548[] = "Mozilla/5.0 (Linux; Android 4.0.3; Sony Tablet S Build/TISU0143) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S58549[] = "Mozilla/5.0(iPad; U;CPU OS 6 like Mac OS X; zh-CN; iPad3,4) AppleWebKit/534.46 (KHTML, like Gecko) UCBrowser/2.0.1.280 U3/0.8.0 Safari/7543.48.3";
const char S58550[] = "NokiaC2-00/2.0 (03.82) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile";
const char S58551[] = "Mozilla/5.0 (compatible; MSIE 3.0; PPC; Trident/5.0)";
const char S58552[] = "Mozilla/5.0 (compatible; MSIE 3.0; PPC; SV1; Trident/2.0)";
const char S58553[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NABIXD-NV10A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58554[] = "Mozilla/5.0 (Series40; Nokia310/07.35; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.36";
const char S58555[] = "BlackBerry8700/4.5.0.182 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102";
const char S58556[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAB462 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58557[] = "iPhone3,1/6.1.2 (10B146)";
const char S58558[] = "server-bag [iPhone OS,6.1.3,10B329,iPhone4,1]";
const char S58559[] = "iPhone4,1/6.1.2 (10B146)";
const char S58560[] = "iPhone4,1/6.1.1 (10B145)";
const char S58561[] = "iPhone5,2/6.1.2 (10B146)";
const char S58562[] = "iPhone5,2/6.1 (10B143)";
const char S58563[] = "iPhone4,1/6.1.3 (10B329)";
const char S58564[] = "iPhone4,1/6.1 (10B142)";
const char S58565[] = "iPhone3,1/6.1 (10B144)";
const char S58566[] = "iPhone3,1/6.1.3 (10B329)";
const char S58567[] = "iPhone2,1/6.1.2 (10B146)";
const char S58568[] = "iPhone5,2/6.1.3 (10B329)";
const char S58569[] = "iPhone5,2/6.1.4 (10B350)";
const char S58570[] = "iPhone2,1/6.1.3 (10B329)";
const char S58571[] = "iPhone3,3/6.1.3 (10B329)";
const char S58572[] = "iPhone5,1/6.1.4 (10B350)";
const char S58573[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae5-00) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile  3gpp-gba";
const char S58574[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE5-00) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S58575[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE5-00.2) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S58576[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N7100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S58577[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7100 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S58578[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-N719 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S58579[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7105 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S58580[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SPH-L900 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S58581[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7105 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S58582[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; SCH-N719 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S58583[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BLU VIVO 4.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58584[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VIVO 4.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58585[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; VIVO 4.3 Build/IMM76D)";
const char S58586[] = "Nokia6070/2.0 (03.20) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S58587[] = "Mozilla/4.0 (PS3 (PlayStation 3); 1.00)";
const char S58588[] = "Mozilla/5.0 (PLAYSTATION 3; 1.00)";
const char S58589[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58590[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58591[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S58592[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S58593[] = "Mozilla/5.0 (Windows NT 5.0; rv:20.0) Gecko/X Firefox/20.0";
const char S58594[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NXM805ND_EN Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58595[] = "IUC(U;iOS 6.1;Zh-cn;320*568;)/UCWEB8.9.0.238/44/997";
const char S58596[] = "IUC(U;iOS 6.1.2;Zh-cn;320*480;)/UCWEB8.9.0.238/44/997";
const char S58597[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; Galaxy Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S58598[] = "Mozilla/5.0 (Linux; U; Android 1.5; fr-fr; Galaxy Build/CUPCAKE) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S58599[] = "Mozilla/5.0 (Linux; U; Android 1.5; tr-; Galaxy Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S58600[] = "Mozilla/5.0 (Linux; U; Android 1.5; de-de; Galaxy Build/CUPCAKE) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S58601[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I777 Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58602[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I777 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58603[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I777 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58604[] = "NokiaC2-03/2.0 (07.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac2-03) UCBrowser8.4.0.159/70/444/UCWEB Mobile";
const char S58605[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:17.0) Gecko/17.0 Firefox/17.0 TenFourFox/G5";
const char S58606[] = "Opera/9.80 (BREW; Opera Mini/6.5/30.3214; U; en) Presto/2.8.119 Version/11.10";
const char S58607[] = "Mozilla/5.0 (iPad; U; CPU OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S58608[] = "SAMSUNG-GT-S5233W/1.0 SHP/VPP/R5 Jasmine/0.8 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S58609[] = "Mozilla/5.0 (Linux; Android 4.0.4; U9202L-1 Build/HuaweiU9202L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58610[] = "Mozilla/5.0 (Linux; Android 4.0.3; U9200 Build/HuaweiU9200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58611[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.33 (KHTML, like Gecko) Ubuntu/9.10 Chromium/13.0.752.0 Chrome/13.0.752.0 Safari/534.33";
const char S58612[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.36 (KHTML, like Gecko) Ubuntu/9.10 Chromium/13.0.768.0 Chrome/13.0.768.0 Safari/534.36";
const char S58613[] = "Mozilla/5.0 (Linux; Android 4.0.4; V8000_USA_Cricket Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58614[] = "Mozilla/5.0 (Linux; Android 4.0.4; N8000_USA_Cricket Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58615[] = "SonyEricssonP1i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 768) Opera 8.65 [en]";
const char S58616[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.0) Gecko/X Thunderbird/12.0.1";
const char S58617[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S58618[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S58619[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I727 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S58620[] = "Instagram 3.4.2 (iPod4,1; iPhone OS 6.1; en_US; en) AppleWebKit/420+";
const char S58621[] = "Instagram 3.4.2 (iPod4,1; iPhone OS 6.1.2; en_US; en) AppleWebKit/420+";
const char S58622[] = "Instagram 3.4.2 (iPod4,1; iPhone OS 6.1.3; en_US; en) AppleWebKit/420+";
const char S58623[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One XL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S58624[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC EVA_UTL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S58625[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One XL Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S58626[] = "Nokia6110Navigator/4.24 (SymbianOS/9.2; U; Series60/3.1 Nokia6110Navigator/4.24; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) Mozilla/5.0 AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S58627[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Nokia6110Navigator/4.22; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S58628[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Nokia6110/3.01; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S58629[] = "Mozilla/5.0 (Windows NT 6.0; rv:15.0; Avant TriCore) Gecko/X Firefox/15.0.1";
const char S58630[] = "Mozilla/5.0 (Windows NT 6.0; rv:16.0; Avant TriCore) Gecko/X Firefox/16.0";
const char S58631[] = "Mozilla/5.0 (Windows NT 6.0; rv:17.0; Avant TriCore) Gecko/X Firefox/17.0";
const char S58632[] = "Mozilla/5.0 (Windows NT 6.0; rv:19.0; Avant TriCore) Gecko/X Firefox/19.0";
const char S58633[] = "Mozilla/5.0 (Windows NT 6.0; rv:20.0; Avant TriCore) Gecko/X Firefox/20.0";
const char S58634[] = "Mozilla/5.0 (Windows NT 6.0; rv:18.0; Avant TriCore) Gecko/X Firefox/18.0";
const char S58635[] = "Mozilla/5.0 (Windows NT 6.0; rv:21.0; Avant TriCore) Gecko/X Firefox/21.0";
const char S58636[] = "Mozilla/5.0 (Windows NT 6.0; rv:22.0; Avant TriCore) Gecko/X Firefox/22.0";
const char S58637[] = "Mozilla/5.0 (Windows NT 6.0; rv:23.0; Avant TriCore) Gecko/X Firefox/23.0";
const char S58638[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.13) Gecko Fedora/1.9.2.13-12.fc14 Firefox/3.6.13";
const char S58639[] = "Mozilla/5.0 (compatible; FlipboardRSS/1.1; +http://flipboard.com/browserproxy)";
const char S58640[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S58641[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9190 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S58642[] = "Opera/9.80 (Windows NT 6.2; U; en) Presto/2.8.131 Version/11.10";
const char S58643[] = "Opera/9.80 (Windows NT 6.2; U; en) Presto/2.8.131 Version/11.11";
const char S58644[] = "Opera/11.1 (Windows NT 6.2: U; en)";
const char S58645[] = "Nokia311/5.0 (03.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia311) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S58646[] = "Mozilla/5.0 (Linux; Android 4.0.4; N861 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58647[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-P765 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58648[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P765 Build/IMM76L)";
const char S58649[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Iron/9.0.600.2 Chrome/9.0.600.2 Safari/534.13";
const char S58650[] = "Opera/9.50 (J2ME/MIDP; Opera Mini/5.21463/27.1659; U; en)";
const char S58651[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/5.22371/26.1670; U; es) Presto/2.8.119 Version/10.54";
const char S58652[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6210 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S58653[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P6210 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S58654[] = "Mozilla/5.0 (Linux; Android 4.0.3 LeoROM; GT-P6210 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S58655[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Desire HD Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58656[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Desire HD Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58657[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Desire HD Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58658[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Desire HD Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58659[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire HD Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58660[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Desire HD Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S58661[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Desire HD Build/MIUI)";
const char S58662[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Desire HD Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58663[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Desire HD Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S58664[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:7.0.1) Gecko/X Postbox/3.0.8";
const char S58665[] = "iTunes-iPad/6.1.3 (2; 16GB; dt:74)";
const char S58666[] = "iTunes-iPad/6.1.3 (4; 32GB; dt:77)";
const char S58667[] = "iTunes-iPad/6.1.3 (5; 16GB; dt:86)";
const char S58668[] = "iTunes-iPad/6.1.3 (4; 16GB; dt:77)";
const char S58669[] = "iTunes-iPad/6.1.3 (5; 32GB; dt:86)";
const char S58670[] = "iTunes-iPad/6.1.3 (2; 32GB; dt:74)";
const char S58671[] = "iTunes-iPad/6.1.3 (5; 64GB; dt:83)";
const char S58672[] = "iTunes-iPad/6.1.3 (4; 64GB; dt:77)";
const char S58673[] = "iTunes-iPad/6.1.3 (5; 16GB; dt:83)";
const char S58674[] = "iTunes-iPad/6.1.3 (2; 64GB; dt:74)";
const char S58675[] = "iTunes-iPad/6.1.3 (5; 32GB; dt:83)";
const char S58676[] = "iTunes-iPad/6.1.3 (5; 64GB; dt:86)";
const char S58677[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17";
const char S58678[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17";
const char S58679[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17";
const char S58680[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17";
const char S58681[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17";
const char S58682[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30at Build/9.1.F.1.120) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S58683[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; PC36100 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58684[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; Sprint APA9292KT Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58685[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; PC36100 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58686[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Sprint APA9292KT Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58687[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Sprint APA9292KT Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko)";
const char S58688[] = "Mozilla/5.0 (Linux; U; Android 2.1; xx-xx; PC36100 Build/ERE27) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S58689[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; PC36100 Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58690[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:6.0) Gecko/X Firefox/6.0";
const char S58691[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; X909 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58692[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; X909 Build/JRO03L)";
const char S58693[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-nz; GT-I8190T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58694[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8190T Build/JZO54K)";
const char S58695[] = "Mozilla/5.0 (Linux; Android 4.2.1; ME301T Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S58696[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I777 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58697[] = "Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I777 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58698[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I777 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58699[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-I777 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58700[] = "Mozilla/5.0 (Series40; NokiaX2-05/08.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S58701[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.4.2 Mobile/9B206 Safari/7534.48.3";
const char S58702[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.4.2 Mobile/9B176 Safari/7534.48.3";
const char S58703[] = "NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-03) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S58704[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S58705[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562L Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S58706[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S58707[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8) Presto/2.12.388 Version/12.11";
const char S58708[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/2.8 Mobile/10A523 Safari/7534.48.3";
const char S58709[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/2.8 Mobile/10A403 Safari/7534.48.3";
const char S58710[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/2.8 Mobile/10A407 Safari/7534.48.3";
const char S58711[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/2.8 Mobile/10A8426 Safari/7534.48.3";
const char S58712[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/2.8.1 Mobile/10A550 Safari/7534.48.3";
const char S58713[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/2.8.1 Mobile/10A8500 Safari/7534.48.3";
const char S58714[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E970 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S58715[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E970 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S58716[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SCH-I589 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58717[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SCH-I589 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58718[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i589 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58719[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AT1S0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58720[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; AT1S0 Build/IMM76D)";
const char S58721[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P5110 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58722[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-P5110 Build/JOP40D)";
const char S58723[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P5110 Build/JDQ39)";
const char S58724[] = "com.google.android.youtube/4.2.16(Linux; U; Android 4.2.1; en_GB; GT-P5110 Build/JOP40G) gzip";
const char S58725[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; GT-P5110 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58726[] = "NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-01) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S58727[] = "NokiaX2-01.1/5.0 (07.22) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-01.1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S58728[] = "UCWEB/2.0 (Java; U; MIDP-2.1; en-US; NokiaX2-01) U2/1.0.0 UCBrowser/9.2.1.331 U2/1.0.0 Mobile Opera Mini/7.1.32052/30.3697";
const char S58729[] = "Nokia6020/2.0 (04.90) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S58730[] = "Nokia6020/2.0 (04.90) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6020";
const char S58731[] = "Mozilla/5.0 (X11; U; Linux i686; zh-TW; rv:1.8.0.10) Gecko/X Fedora/1.5.0.10-1.fc5 Firefox/1.5.0.10";
const char S58732[] = "Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S58733[] = "Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S58734[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58735[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58736[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810M Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58737[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810B Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58738[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810E Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58739[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; NXM908HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58740[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T10A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58741[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T10A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58742[] = "Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.12 Mobile/11A4414e Safari/8536.25";
const char S58743[] = "Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A4435d Safari/8536.25";
const char S58744[] = "Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A4449a Safari/8536.25";
const char S58745[] = "Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A4449d Safari/8536.25";
const char S58746[] = "Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A465 Safari/8536.25";
const char S58747[] = "Mozilla/5.0 (iPod; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A501 Safari/8536.25";
const char S58748[] = "Mozilla/5.0 (Linux; Android 4.1.2; C525c Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S58749[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; LG-SU640 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58750[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; LG-SU640 Build/GRJ90)";
const char S58751[] = "Mozilla/5.0 (Series40; Nokia111.1/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S58752[] = "Mozilla/5.0 (Series40; Nokia111/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S58753[] = "Cricket-A310/1.0 UP.Browser/6.3.0.7 (GUI) MMP/2.0";
const char S58754[] = "Mercury/7.4 CFNetwork/548.1.4 Darwin/11.0.0";
const char S58755[] = "Nokia2220s/2.0 (09.55) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2220s";
const char S58756[] = "Nokia2220s/2.0 (09.55) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S58757[] = "Nokia2220s-b/2.0 (10.20) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S58758[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1;	Trident/4.0; SLCC2;	.NET CLR 2.0.50727; .NET CLR 3.5.30729;	.NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E) QQBrowser/6.0";
const char S58759[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.0";
const char S58760[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.16; QQBrowser/6.0.8082.201 (webkit)";
const char S58761[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1) QQBrowser/6.0";
const char S58762[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.0.8405.201";
const char S58763[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01E Build/S9130) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58764[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01E Build/SA310) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58765[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01E Build/S9132) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58766[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01EVW Build/SB020) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58767[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01EVW Build/S1100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58768[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01E Build/S3110) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58769[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01EVW Build/S3120) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58770[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01E Build/S5270) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58771[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01EVW Build/S5270) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58772[] = "Mozilla/5.0 (Linux; U; Android 3.0; en-us; MID_Serials Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58773[] = "Mozilla/5.0 (Linux; U; Android 3.0UI; en-us; MID_Serials Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58774[] = "Dalvik/1.4.0 (Linux; U; Android 3.0; MID_Serials Build/GINGERBREAD)";
const char S58775[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.10.229 Version/11.64";
const char S58776[] = "Mozilla/5.0 (Windows NT 7.1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30";
const char S58777[] = "Mozilla/5.0 (Windows NT 7.1; en-us) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30";
const char S58778[] = "Mozilla/5.0 (Windows NT 7.1; es-mx) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30";
const char S58779[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; BLADEII Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58780[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison 3G Build/1.1.7 X-17:05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58781[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison 3G Build/1.1.8-1015 X-14:40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58782[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison 3G Build/1.1.9-1015 X-17:37) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58783[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison 3G Build/1.1.9 X-10:23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58784[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison 3G Build/1.1.10-1015 X-18:00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58785[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison 3G Build/1.1.11-1015 X-15:24) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58786[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison 3G Build/1.1.11 X-10:07) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58787[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison 3G Build/1.1.8 X-17:02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58788[] = "Nokia5330/5.0 (09.85) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S58789[] = "Nokia5330-1d/5.0 (06.88) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S58790[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LVW Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58791[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LRA Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58792[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR6410LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S58793[] = "Mozilla/5.0 (Series40; Nokia311/05.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S58794[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E220S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58795[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E220S Build/JZO54K)";
const char S58796[] = "Nokia5230/UCWEB8.9.0.253/50/999";
const char S58797[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5230) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S58798[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5230-c) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S58799[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5230-1c) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S58800[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E988 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58801[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E988 Build/JZO54K)";
const char S58802[] = "Mozilla/5.0 (Linux; Android 4.1.2; SM-T210 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S58803[] = "Mozilla/5.0 (Linux; Android 4.1.2; SM-T210R Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S58804[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus One Build/GRK39F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58805[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus One Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58806[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en_us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58807[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus One Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58808[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus One Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58809[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus One Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58810[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus One Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58811[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58812[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus One Build/FRF91) AppleWebKit/534.3 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.3";
const char S58813[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100P Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58814[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100P Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58815[] = "Nokia202/2.0 (20.28) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia202) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S58816[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E450 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58817[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E450f Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58818[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E450g Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58819[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E450B Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58820[] = "NokiaX2-01/5.0 (08.71) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-01) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S58821[] = "NokiaX2-01.1/5.0 (07.22) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-01.1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S58822[] = "LG-GW370/V10c; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.17086; U; en) Opera 9.50";
const char S58823[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.8.1.17) Gecko/X SeaMonkey/1.1.12";
const char S58824[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; de-AT; rv:1.8.1.21) Gecko/X SeaMonkey/1.1.15 Mnenhy/0.7.6.0";
const char S58825[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Q-Smart S18 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58826[] = "Mozilla/5.0 (Linux;U; Android 4.0;en-US;Q-Smart S18 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58827[] = "Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80 COBALT Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S58828[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire C Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58829[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire C Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58830[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;HTC Glacier Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58831[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Glacier Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1";
const char S58832[] = "Mozilla/5.0 (Linux; U; Android2.2.1; en-us; HTC Glacier Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58833[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Glacier Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58834[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; HTC Glacier Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58835[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Glacier Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58836[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Glacier Build/FRG83) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S58837[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; HTC Glacier Build/FRG83)";
const char S58838[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; HTC Glacier Build/FRF91)";
const char S58839[] = "Mozilla/5.0 (compatible; MSIE 6.0; PPC; Trident/1.0)";
const char S58840[] = "Mozilla/5.0 (compatible; MSIE 6.0; PPC; SV1; Trident/4.0)";
const char S58841[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.3 Mobile/9B206 Safari/7534.48.3";
const char S58842[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.3 Mobile/9B176 Safari/7534.48.3";
const char S58843[] = "NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac2-01) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S58844[] = "sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/11.05.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S58845[] = "sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/11.07.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S58846[] = "sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/10.20.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S58847[] = "sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/12.06.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S58848[] = "sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/12.05.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S58849[] = "sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/09.26.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S58850[] = "sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/ab.`e.b`ab Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S58851[] = "sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/02.26.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S58852[] = "sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/03.13.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S58853[] = "sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/12.11.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S58854[] = "sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/03.08.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S58855[] = "sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/`b.bf.b`ac Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S58856[] = "sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/12.10.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S58857[] = "sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/ab.`f.b`ab Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S58858[] = "sprd-S9300/1.0 Linux/2.6.35.7 Android/4.0.3 Release/12.06.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S58859[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; VegaComb Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S58860[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Flashback 8.0 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S58861[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; GtabComb Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S58862[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Flashback 10.0 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S58863[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Flashback 9.0 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S58864[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; gTablet Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S58865[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Android Illuminate Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S58866[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Flashback 7.1 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S58867[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-gb; Flashback 8.1 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S58868[] = "Mozilla/5.0 (Linux; U; Android 3.0UI; pt-br; Tabra QAV801 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58869[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; GtabComb Build/NIHCPX) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S58870[] = "Mozilla/5.0 (Linux; U; Android 3.0UI; en-us; REL-7305B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58871[] = "Mozilla/5.0 (Linux; U; Android 3.0UI; tr-tr; F4S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58872[] = "Mozilla/5.0 (Linux; U; Android 3.0UI; en-us; mTAB NEO2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58873[] = "Mozilla/5.0 (Linux; U; Android 3.0; en-us; SPH-M828C Build/HONEYCOMB; HONEYCOMB PRECEDENT) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58874[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58875[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552B Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58876[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1";
const char S58877[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1";
const char S58878[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu/12.04 Chromium/25.0.1364.172 Chrome/25.0.1364.172 Safari/537.22";
const char S58879[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu/12.04 Chromium/25.0.1364.172 Chrome/25.0.1364.172 Safari/537.22";
const char S58880[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMP7100D3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58881[] = "Mozilla/5.0 (Linux; Android 4.1.2; VS930 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S58882[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58883[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58884[] = "SonyEricssonJ108a/R7EA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2";
const char S58885[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaC5-00)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S58886[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaC5-00.2)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S58887[] = "Mozilla/5.0 (Java; U; en-in; nokiac5-00.2) UCBrowser8.2.1.144/69/444/UCWEB Mobile  3gpp-gba";
const char S58888[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1";
const char S58889[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1";
const char S58890[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1";
const char S58891[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1";
const char S58892[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1";
const char S58893[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1";
const char S58894[] = "NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-03) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S58895[] = "Mozilla/5.0 (Linux; U; Android 2.0.1; en-us; Droid (AdamZ Enhanced) Build/ESD56) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58896[] = "Mozilla/5.0 (Linux; U; Android 1.0.0; es-es; STM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58897[] = "Mozilla/5.0 (Linux; U; Android 1.0.0; sv-se; 7CK100 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58898[] = "Mozilla/5.0 (Linux; U; Android 1.0.0; nl-nl; TP_10D8 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58899[] = "Mozilla/5.0 (Linux; U; Android 2.0; en-us;) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58900[] = "Mozilla/5.0 (Linux; U; Android v1.02.11-2010.12.17_English; en-us; TR718D Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58901[] = "Mozilla/5.0 (Linux; U; Android v1.02.09-2010.11.22_English; en-us; TR718D Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58902[] = "Mozilla/5.0 (Linux; U; Android v1.02.15_14-M_EN-2011.03.29; en-us; M8002R Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58903[] = "Mozilla/5.0 (Linux; U; Android 1.0.0; ld-us; Android-for-Rockchip-2818 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58904[] = "Mozilla/5.0 (Linux; U; Android v1.02_14.13-EN-2011.01.11; en-en; TR718D Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58905[] = "Mozilla/5.0 (Linux; U; Android v1.02.15_14-M_EN-2011.03.18; en-us; M8002R Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58906[] = "Mozilla/5.0 (Linux; U; Android v1.02_14.13-M_EN-2011.03.18; en-en; TR718D Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58907[] = "Mozilla/5.0 (Linux; U; Android v1.02_15.13-ES-2011.04.19; es-es; N6701 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58908[] = "Mozilla/5.0 (Linux; U; Android v1.02_14.13_CH-2011.03.31; es-es; MID1002 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58909[] = "Mozilla/5.0 (Linux; U; Android 1.0.0; es-es; STM7UW Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58910[] = "Mozilla/5.0 (Linux; U; Android Eclair; fr-fr; Tablet-P27 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58911[] = "Mozilla/5.0 (Linux; U; Android 1.0.0; es-es; ARCHM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58912[] = "Mozilla/5.0 (Linux; U; Android v1.02_15.14-M_GB@X; en-gb; A93KQ Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58913[] = "Mozilla/5.0 (Linux; U; Android 1.0.1; en-us; A7EC Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58914[] = "Mozilla/5.0 (Linux; U; Android 1.0.0; fr-fr; CT701 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58915[] = "Mozilla/5.0 (Linux; U; Android 1.0.0; es-es; STM7UC Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58916[] = "Mozilla/5.0 (Linux; U; Android v1.02_14.13-M_EN-2011.01.13; es-es; TR718D Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58917[] = "Mozilla/5.0 (Linux; U; Android v1.02_14.13-M_EN-2011.01.28; en-en; TR718D Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58918[] = "Mozilla/5.0 (Linux; U; Android 1.0.4; fr-fr; A7EB-FLFR Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58919[] = "Dalvik/1.1.0 (Linux; U; Android Eclair Build/ECLAIR)";
const char S58920[] = "Mozilla/5.0 (Linux; U; Android Android_2.1-1.1.0; es-es; T300 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58921[] = "Mozilla/5.0 (Linux; U; Android v1.02_15.14-EN-2011.03.23; en-za; TR718D Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58922[] = "Mozilla/5.0 (Linux; U; Android Eclair; tr-tr; FENOMEN Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58923[] = "Mozilla/5.0 (Linux; U; Android v1.02_15.13-ES-2011.03.29; es-es; AIRIS-OnePAD Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58924[] = "Mozilla/5.0 (Linux; U; Android v1.02_14.13-ES-2011.01.15; es-es; M721 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58925[] = "Mozilla/5.0 (Linux; U; Android v1.02_15.14-FR--2011.02.21; fr-fr; MFC150FR_01 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58926[] = "Mozilla/5.0 (Linux; U; Android v1.02_14.13-ES-2011.02.21; es-es; M721 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58927[] = "Mozilla/5.0 (Linux; U; Android Eclair; es-es; Taicho Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58928[] = "Mozilla/5.0 (Linux; U; Android 1.0.5; cs-cz; A7EB-WK Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58929[] = "Mozilla/5.0 (Linux; U; Android 1.0.0; da-dk; NTM7U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S58930[] = "Mozilla/5.0 (Linux; Android 4.0.3; Marquis_MP977 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S58931[] = "Mozilla/5.0 (Linux; Android 4.0.4; Rise Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58932[] = "Mozilla/5.0 (Linux; Android 4.0.4; C5155 Build/IML77) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58933[] = "Mozilla/5.0 (Android; Tablet; rv:25.0) Gecko/25.0 Firefox/25.0";
const char S58934[] = "Mozilla/5.0 (Linux; Android 4.1.1; XOLO A1000 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S58935[] = "Mozilla/5.0 (Linux; Android 4.1.2; IdeaTabA1000-F Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S58936[] = "Mozilla/5.0 (Linux; Android 4.1.2; IdeaTabA1000-G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S58937[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire C Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S58938[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S58939[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; A777 Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58940[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; WT7.0 Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58941[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A7 ART Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58942[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; GUNIUS713 Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58943[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; GOCLEVER TAB T75 Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58944[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; TAKARA MID72 Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58945[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A777-C-MID-general Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58946[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A777 Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S58947[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Prology T-710T Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58948[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Bliss Pad T7012 Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58949[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Prology iMap-7000 Tab Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58950[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Note-700 Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58951[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; TAKARA MID72 Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58952[] = "Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320) BlackBerry8100/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100";
const char S58953[] = "BlackBerry8100/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100";
const char S58954[] = "LG-C550 Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; LG-C550 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S58955[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; LG-C550 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S58956[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en; LG-C550 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S58957[] = "Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; L-04C Build/FRF91) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58958[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; L-04C Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S58959[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; L-04C Build/FRG83G)";
const char S58960[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Maxthon/3.0)";
const char S58961[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Maxthon/3.0)";
const char S58962[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Maxthon/3.0)";
const char S58963[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Maxthon/3.0)";
const char S58964[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; Maxthon/3.0)";
const char S58965[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Maxthon/3.0)";
const char S58966[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Maxthon/3.0)";
const char S58967[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LS970 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S58968[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LS970 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S58969[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; LG-E435f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58970[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E435 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58971[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E435 Build/JZO54K)";
const char S58972[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E435g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S58973[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E435f Build/JZO54K)";
const char S58974[] = "SAMSUNG-SGH-A597/A597UCJG1; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.19903; U; en-US) Opera 9.50";
const char S58975[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.424; en-us; 9630/4.2.0.424) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S58976[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.451; en-us; 9630/4.2.0.451) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S58977[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.456; en-us; 9630/4.2.0.456) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S58978[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.284; en-us; 9630/4.2.0.284) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S58979[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.433; en-us; 9630/4.2.0.433) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S58980[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.334; en-us; 9630/4.2.0.334) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S58981[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.179; en-us; 9630/4.2.0.179) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S58982[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.285; en-us; 9630/4.2.0.285) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S58983[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.494; en-us; 9630/4.2.0.494) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S58984[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.214; en-us; 9630/4.2.0.214) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S58985[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.477; en-us; 9630/4.2.0.477) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S58986[] = "UCWEB/2.0(BlackBerry; U; 4.1.0.40; en-us; 9630/4.1.0.40) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S58987[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTCEVOV4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S58988[] = "Tiphone T78/1.0 Browser/wap2.0 Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S58989[] = "Tiphone MY1/1.0 Browser/wap2.0 Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S58990[] = "Tiphone T33/1.0 Browser/wap2.0 Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S58991[] = "tiphoneE88/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S58992[] = "Mozilla/5.0 (iOS; U; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.1";
const char S58993[] = "Mozilla/5.0 (iOS; U; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.2";
const char S58994[] = "Mozilla/5.0 (iOS; U; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.5";
const char S58995[] = "Mozilla/5.0 (iOS; U; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.4";
const char S58996[] = "Mozilla/5.0 (iOS; U; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.6";
const char S58997[] = "/media/sandisk/af_mobile/user_agent/iphone";
const char S58998[] = "Mozilla/5.0 (iOS; U; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.3";
const char S58999[] = "Mozilla/5.0 (iOS; U; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.7";
const char S59000[] = "Mozilla/5.0 (iOS; U; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.8";
const char S59001[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; IM-A780L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59002[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; IM-A780L Build/GINGERBREAD)";
const char S59003[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A780 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59004[] = "Lenovo A780/1.0 Linux/2.6.35.7 Android/2.3.5 Browser/AppleWebKit533.1 Configuration";
const char S59005[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A780 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S59006[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Lenovo A780 Build/GRJ90)";
const char S59007[] = "SonyEricssonW705/R1GA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.4";
const char S59008[] = "SonyEricssonW705u/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 model-orange JavaPlatform/JP-8.4.3";
const char S59009[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I537 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S59010[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-us; NXM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59011[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.6; en-us; NXM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59012[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.6; en-us; NXM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59013[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.14; en-us; NXM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59014[] = "SonyEricssonW580c_CMCC/R8BE SEMC-SW/R8BE Release/01.15.2008 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S59015[] = "SonyEricssonW580i/R8BE Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S59016[] = "SonyEricssonW580c/R8BE Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S59017[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T999 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59018[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9300 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59019[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59020[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC first Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59021[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC first Build/JZO54K)";
const char S59022[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P769 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S59023[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P768 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S59024[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P760 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S59025[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.1 Chrome/17.0.1000.1 Safari/535.11";
const char S59026[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.2 Chrome/17.0.1000.2 Safari/535.11";
const char S59027[] = "Opera/9.80 (S60; SymbOS; Opera Mobi/SYB-X; U; en-US) Presto/2.9.201 Version/11.50";
const char S59028[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA6120c) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S59029[] = "SAMSUNG-GT-M8800/M8800ADIF2 SHP/VPP/R5 NetFront/3.5 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S59030[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; ALCATEL one touch 890D Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59031[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ALCATEL_one_touch_890D Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59032[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; ALCATEL one touch 890D Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S59033[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; ALCATEL one touch 890D Build/FRG83G)";
const char S59034[] = "Facebook 4110.0 (iPhone; iPhone OS 5.1.1; en_US)";
const char S59035[] = "Facebook 4110.0 (iPhone; iPhone OS 5.1; en_US)";
const char S59036[] = "Instagram 2.5.1 (iPhone; iPhone OS 5.1.1; en_US) AppleWebKit/420+";
const char S59037[] = "com.cnn.iphone/150.85 (unknown, iPhone OS 5.1.1, iPhone, Scale/2.X)";
const char S59038[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXL_Beats_X315e; zh-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S59039[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXL_Beats_X315b; en-au) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S59040[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXL_Beats_X315e; en-gb) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S59041[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXL_Beats_X315e; en-hk) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S59042[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXL_Beats_X315e; en-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S59043[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXL_Beats_X315e; en-in) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S59044[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/SensationXL_Beats/2.32.161.2; de-ch) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S59045[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXL_Beats_X315b; en-nz) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S59046[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/SensationXL_Beats/2.32.161.2; de-at) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S59047[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/SensationXL_Beats/2.32.161.2; fr-ch) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S59048[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXL_Beats_X315e; ru-ru) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S59049[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXL_Beats_X315e; zh-cn) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S59050[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/SensationXL_Beats/2.32.161.2; en-gb) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S59051[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXL_Beats_X315e; vi-vn) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S59052[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MZ605 Build/I.7.1-39.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59053[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MZ605 Build/I.7.1-39.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59054[] = "NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-02) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S59055[] = "Nokia201/2.0 (11.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia201) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S59056[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Vodafone/1.0/HTC_HD_mini/1.11.172.2 (87652)";
const char S59057[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Vodafone/1.0/HTC_HD_mini/1.10.161.1 (86611)";
const char S59058[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Vodafone/1.0/HTC_HD_mini/1.41.161.1 (10904)";
const char S59059[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Vodafone/1.0/HTC_HD_mini/1.36.184.2 (94538)";
const char S59060[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Vodafone/1.0/HTC_HD_mini/1.11.164.1 (87733)";
const char S59061[] = "HD_mini/1.36.621.4 (94538) Mozilla/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S59062[] = "HTC_HD_mini_T5555 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S59063[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) HD_mini_T5555/320x480";
const char S59064[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Vodafone/1.0/HTC_HD_mini/1.41.162.1 (10904)";
const char S59065[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Swisscom/1.0/HTC_HD_mini/1.11.166.1 (87652)";
const char S59066[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Vodafone/1.0/HTC_HD_mini/1.41.164.1 (10904)";
const char S59067[] = "HD_mini_T5555 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S59068[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Vodafone/1.0/HTC_HD_mini/1.11.162.1 (87652)";
const char S59069[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Swisscom/1.0/HTC_HD_mini/1.41.166.1 (10904)";
const char S59070[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Swisscom/1.0/HTC_HD_mini/1.11.167.1 (87652)";
const char S59071[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Vodafone/1.0/HTC_HD_mini/1.00.164.1 (80896)";
const char S59072[] = "Mozilla/5.0 (Series40; NokiaX3-02.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S59073[] = "Mozilla/5.0 (Series40; NokiaX3-02/05.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S59074[] = "Mozilla/5.0 (Series40; NokiaX3-02.5/07.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6";
const char S59075[] = "Mozilla/5.0 (Series40; NokiaX3-02/07.16; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6";
const char S59076[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5530) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S59077[] = "Nokia5530UCBrowser/8.7.1.234/50/444/UCWEB";
const char S59078[] = "Nokia5530/UCWEB8.7.0.187/50/999";
const char S59079[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.24 (KHTML, like Gecko) Ubuntu/11.04 Chromium/11.0.696.71 Chrome/11.0.696.71 Safari/534.24";
const char S59080[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Ubuntu/11.04 Chromium/11.0.696.71 Chrome/11.0.696.71 Safari/534.24";
const char S59081[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.21 (KHTML, like Gecko) Ubuntu/11.04 Chromium/11.0.676.0 Chrome/11.0.676.0 Safari/534.21";
const char S59082[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae63-1) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S59083[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE63) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S59084[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-D700 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59085[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-D700 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59086[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-D700 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59087[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-D700 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59088[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-D700 Build/JZ054K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59089[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I200 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59090[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I200 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S59091[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I200 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S59092[] = "Mozilla/5.0 (Windows NT 5.0; rv:21.0) Gecko/X Firefox/21.0";
const char S59093[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7105 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.5/580,2.0.5.580,2013-06-26 22:40, +0100, gb";
const char S59094[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L900 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0500, us";
const char S59095[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us";
const char S59096[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I317M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, ca";
const char S59097[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T889 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us";
const char S59098[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7105 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb";
const char S59099[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb";
const char S59100[] = "LG-A340/V10d; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.22250; U; en-US) Opera 9.50";
const char S59101[] = "Mozilla/5.0 (Linux; Android 4.2.1; M470BSA Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S59102[] = "Mozilla/5.0 (Linux; Android 4.1.1; A210 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S59103[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; SPH-M900 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S59104[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; SPH-M900 Build/CUPCAKE) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S59105[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; SPH-M900) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S59106[] = "Mozilla/5.0 (iDEN; U; CPU iDEN Android Opus One; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5G77 Safari/525.20";
const char S59107[] = "Mozilla/5.0 (Linux; U; Android v2.2; en-us; WM8650 Build/FRG83D_v2.2 Android Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59108[] = "Mozilla/5.0 (Linux; U; Android Cyberus 3.5; en-us; Cyberus Tablet Build/DRD20) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S59109[] = "Mozilla/5.0 (Linux; U; Android DG2.30; md-us; SYTAB7MX Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59110[] = "Mozilla/5.0 (Linux; U; Android BoosT3.5; es-us; HTC D/S Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59111[] = "Dalvik/1.2.0 (Linux; U; Android v2.2; WM8650 Build/FRG83D_v2.2 Android Froyo)";
const char S59112[] = "Mozilla/5.0 (Linux; U; Android unknown; zh-tw; GETEK-A8 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59113[] = "Mozilla/5.0 (Linux; U; Android 5.2.6(Android 5.1.1); en-us; mtk6515_6626_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59114[] = "Mozilla/5.0 (Linux; U; Android 5.2.9(Android 5.1.1); en-us; mtk6515_6626_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59115[] = "Mozilla/5.0 (Linux; U; Android unknown; zh-cn; W99 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59116[] = "Mozilla/5.0 (Linux; U; Android GFIVE-4.0; en-us; GFIVE A77 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59117[] = "Mozilla/5.0 (Linux; U; Android unknown; zh-cn; NOAIN A900 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59118[] = "Mozilla/5.0 (Linux; U; Android unknown; zh-cn; TETC-W318 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59119[] = "Mozilla/5.0 (Linux; U; Android unknown; zh-cn; SOP-W168 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59120[] = "Mozilla/5.0 (Linux; U; Android unknown; vi-vn; Sen S1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59121[] = "Mozilla/5.0 (Linux; U; Android 5.2.8(Android 5.1.1); en-us; mtk6515_6626_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59122[] = "Mozilla/5.0 (Linux; U; Android 5.2.1(Android 5.1.1); en-us; mtk6515_6626_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59123[] = "Mozilla/5.0 (Linux; U; Android unknown; zh-cn; HOSIN U98 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59124[] = "Mozilla/5.0 (Linux; U; Android unknown; en-us; MAXX MA-808 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59125[] = "Mozilla/5.0 (Linux; U; Android MASSTEL 4.0; vi-vn; MASSTEL (M18) Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59126[] = "Mozilla/5.0 (Linux; U; Android Freakenstien 1.1; en-us; Hotrod Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59127[] = "Mozilla/5.0 (Linux; U; Android GFIVE 4.0; es-es; GFIVE Spark(A1) Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59128[] = "mSpot Android Aria";
const char S59129[] = "Mozilla/5.0 (Linux; U; Android Android OS,v2.3.6; en-; DTC GT6  Build/GT830) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59130[] = "Mozilla/5.0 (Linux; U; Android 5.2.3(Android 5.1.1); en-us; mtk6515_6626_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59131[] = "Mozilla/5.0 (Linux; U; Android GFIVE-4.0; en-us; GFIVE Blade X (F600) Build/F600-V1.0-2012.11.13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59132[] = "Mozilla/5.0 (Linux; U; Android QuadCore-4.0.0; vi-vn; M180 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59133[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:19.0) Gecko/X Firefox/19.0 SeaMonkey/2.16.2";
const char S59134[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1";
const char S59135[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1";
const char S59136[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I727R Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59137[] = "Mozilla/5.0 (Linux; Android 4.0.3; SHV-E110S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59138[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E110S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59139[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9210 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59140[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T779 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59141[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P5100 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59142[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P5100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59143[] = "Mozilla/5.0 (Series40; NokiaC2-02/06.51; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.1.21.8.2";
const char S59144[] = "Mozilla/5.0 (Series40; NokiaC2-02.1/06.52; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.1.21.8.2";
const char S59145[] = "Mozilla/5.0 (Windows NT 6.1; rv:2.0.1) Gecko/X Firefox/7.0.1/Nutch-1.4";
const char S59146[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HUAWEI U8681 Build/HuaweiU8681) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59147[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HUAWEI U8681 Build/HuaweiU8681)";
const char S59148[] = "Nokia202/2.0 (20.52) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia202) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S59149[] = "Nokia203/2.0 (20.52) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia203) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S59150[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-F Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S59151[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-H Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S59152[] = "Mozilla/5.0 (Linux; Android 4.0.4; SmartTabII7 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S59153[] = "Mozilla/5.0 (Linux; Android 4.0.4; A2107A-H Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S59154[] = "Mozilla/5.0 (Linux; Android 4.2.1; M470BSA Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S59155[] = "Mozilla/5.0 (Windows; N; Windows NT 5.1; en-US) AppleWebKit/527+ (KHTML, like Gecko, Safari/419.3) Qt/4.5.3";
const char S59156[] = "Mozilla/5.0 (Windows; N; Windows NT 5.2; en-US) AppleWebKit/527+ (KHTML, like Gecko, Safari/419.3) Qt/4.5.3";
const char S59157[] = "Mozilla/5.0 (Windows; N; Windows NT 5.2; en-US) AppleWebKit/523.15 (KHTML, like Gecko, Safari/419.3) Qt/4.4.0";
const char S59158[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Qt/4.6.3 Safari/532.4";
const char S59159[] = "Mozilla/5.0 (Linux; Android 4.0.3; E350 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59160[] = "Mozilla/5.0 (Linux; Android 4.0.4; E350 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59161[] = "SAMSUNG-GT-S5603/1.0 SHP/VPP/R5 Jasmine/0.8 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S59162[] = "SAMSUNG-GT-S5603T/1.0 SHP/VPP/R5 Jasmine/0.8 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S59163[] = "Mozilla/5.0 (Linux; Android 4.1.2; USCC-C6721 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59164[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE63)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S59165[] = "NokiaE63/UCWEB8.2.0.116/28/800";
const char S59166[] = "Mozilla/5.0 (Java; U; en-in; nokiae63-1) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S59167[] = "Mozilla/5.0 (Windows NT 4.0; rv:21.0) Gecko/X Firefox/21.0";
const char S59168[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SHV-E300S Build/JDQ39)";
const char S59169[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ko-kr; SHV-E300S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 NAVER(inapp; search; 230; 4.7.7)";
const char S59170[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I497 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S59171[] = "pcdc751/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S59172[] = "Amoi-F90/Plat-F/WAP2.0/MIDP1.0/CLDC1.0 UP.Browser/6.2.2.6.f.1.100 (GUI) MMP/1.0";
const char S59173[] = "WP8990/T05_0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S59174[] = "OPWV-SDK/62 UP.Browser/6.2.2.1.208 (GUI) MMP/2.0";
const char S59175[] = "KWC-KX18/10073 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S59176[] = "M1000-2X0/1.0.11 UP.Browser/6.2.3.9.g.1.111 (GUI) MMP/2.0";
const char S59177[] = "pcdc731/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S59178[] = "UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S59179[] = "r270v1 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S59180[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SHW-M130L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59181[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SHW-M130K Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59182[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SHW-M130K Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59183[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SHW-M130L Build/FROYO)";
const char S59184[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SHW-M130L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59185[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SHW-M130K Build/FROYO)";
const char S59186[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; ADR6225 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59187[] = "NetFront/3.5.1 (BREW 3.1.5; U; en-us; LG; NetFront/3.5.1/AMB) Sprint LX610 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S59188[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; Avant Browser)";
const char S59189[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Win64; x64; Trident/6.0; Touch; Avant Browser)";
const char S59190[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; Touch; Avant Browser)";
const char S59191[] = "Mozilla/5.0 (Linux; Android 4.1.2; C2104 Build/15.0.A.1.31) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59192[] = "SonyEricssonK700c/R2AE SEMC-Browser/4.0.3Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S59193[] = "SonyEricssonK700c/R2E SEMC-Browser/4.0 Profile/MIDP-1.0 MIDP-2.0 Configuration/CLDC-1.1";
const char S59194[] = "SonyEricssonK700c/R2L SEMC-Browser/4.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S59195[] = "SonyEricssonK700c/R2AA SEMC-Browser/4.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S59196[] = "SonyEricssonK700c/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S59197[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; F-12C Build/F0001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59198[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; F-12C Build/F0001)";
const char S59199[] = "Mozilla/5.0 (compatible; heritrix/1.14.4 +http://parsijoo.ir)";
const char S59200[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59201[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59202[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST25i Build/6.1.1.B.1.54) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59203[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST25a Build/6.1.1.B.1.10) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59204[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xperia U Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59205[] = "Android (Linux; U; Android 2.1-update1; zh-cn; mooncake) AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S59206[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-by; MTC_Android Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59207[] = "Android (Linux; U; Android 2.1-update1; zh-cn; C8500) AppleWebKit/528.5+ (KHTML) Version/3.1.2";
const char S59208[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fa_ir) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky/1.5.0 Safari/533.1";
const char S59209[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-ua; life:) Android Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59210[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Android for Telechips TCC8900 Evaluation Board (US) Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59211[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Android for Telechips TCC8902 MID Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59212[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Android for Telechips TCC8900 Release Board (US) Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59213[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Android for Telechips TCC8900 Evaluation Board (multi-language) Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59214[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; Android for Telechips TCC8902 Tablet-PC (DE) Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59215[] = "CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 2.1-update1)";
const char S59216[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MID Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59217[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Android MID Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59218[] = "Mozilla/5.0 (Linux; U; Android 2.1.2; ru-ru Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59219[] = "Mozilla/5.0 (Linux; U; Android 2.1.1; ru-ru Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59220[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; OMAP_SS Build/ECLAIR) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S59221[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-us; MID Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59222[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; generic Build/EPE54B) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59223[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S59224[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-kr Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59225[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; sr-rs; Vip_MiniDroid Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59226[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; it-it; TT-7000H Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59227[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ThL W8 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59228[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; it-it; ThL W8 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59229[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; ThL W8 Build/JOP40D)";
const char S59230[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; ThL W8+ Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59231[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; HTC-X710a Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59232[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC-X710a Build/GRJ22)";
const char S59233[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Marquis_MP977 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59234[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; VS TOUCHTAB 10.1DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59235[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; VS TOUCHTAB 10.1DC Build/JRO03H)";
const char S59236[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A860S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59237[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A860S Build/JZO54K)";
const char S59238[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-I8190 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59239[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-I8190 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59240[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-I8190 Build/GRK39F)";
const char S59241[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-79) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S59242[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT925 Build/9.8.2Q-8-XT925_VQL-12.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S59243[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGP312 Build/10.3.1.C.0.136) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S59244[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB508 Build/Blur_Version.2.112.12.MB508.ATT.en.US Flex/P007) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59245[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB508 Build/Blur_Version.2.159.17.MB508.ATT.en.US Flex/P011) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59246[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB508 Build/3.4.2-159_SAG-17_SecATT) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59247[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; WS802C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59248[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Penta WS802C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59249[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Penta WS802C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59250[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Novo7Fire Build/Xqm) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59251[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Novo7Fire Build/Xtm) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59252[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Novo7Fire Build/X.qm) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59253[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Novo7Fire Build/X.tm) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59254[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS840 4G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59255[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS840 4G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59256[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS840PP Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59257[] = "KWC-Rio/ACI1005 UP.Browser/7.2.7.2.563 (GUI) MMP/2.0";
const char S59258[] = "KWC-Rio/ACI1003 UP.Browser/7.2.7.2.563 (GUI) MMP/2.0";
const char S59259[] = "KWC-Rio/ACJ1005 UP.Browser/7.2.7.2.562 (GUI) MMP/2.0";
const char S59260[] = "KWC-Rio/ABY1009 UP.Browser/7.2.7.2.562 (GUI) MMP/2.0";
const char S59261[] = "KWC-Rio/ACK10006 UP.Browser/7.2.7.2.563 (GUI) MMP/2.0";
const char S59262[] = "JUC (Linux; U; 2.1-update1; zh-cn; C8500; 240*320) UCWEB7.9.3.103/139/999";
const char S59263[] = "JUC (Linux; U; 2.3.5; zh-cn; TCL_A919; 320*480) UCWEB7.9.3.103/139/32463";
const char S59264[] = "JUC (Linux; U; 2.3.6; zh-cn; Nexus_S; 480*800) UCWEB7.9.3.103/139/800";
const char S59265[] = "JUC (Linux; U; 2.3.5; zh-cn; 7260; 480*800) UCWEB7.9.3.103/139/32781";
const char S59266[] = "JUC (Linux; U; 2.3.5; zh-cn; M9; 640*960) UCWEB7.9.3.103/139/800";
const char S59267[] = "JUC (Linux; U; 2.3.4; zh-cn; GN380; 480*800) UCWEB7.9.3.103/139/32702";
const char S59268[] = "JUC (Linux; U; 2.3.4; zh-cn; GN106; 320*480) UCWEB7.9.3.103/139/32702";
const char S59269[] = "JUC (Linux; U; 2.3.4; zh-cn; GN205; 480*800) UCWEB7.9.3.103/139/32702";
const char S59270[] = "JUC (Linux; U; 2.3.5; zh-cn; MI-ONE_Plus; 480*854) UCWEB7.9.4.145/139/800";
const char S59271[] = "JUC (Linux; U; 2.1-update1; zh-cn; OMAP_SS; 480*800) UCWEB7.9.3.103/139/800";
const char S59272[] = "JUC (Linux; U; 2.1; zh-cn; D539; 320*480) UCWEB7.9.3.103/139/800";
const char S59273[] = "JUC (Linux; U; 2.1-update1; zh-cn; XT702; 480*854) UCWEB7.9.3.103/139/32160";
const char S59274[] = "JUC (Linux; U; 1.5; zh-cn; ME600; 320*480) UCWEB7.9.3.103/139/800";
const char S59275[] = "JUC (Linux; U; 2.3.5; zh-cn; HOSIN_U3; 320*480) UCWEB7.9.3.103/139/32684";
const char S59276[] = "JUC (Linux; U; 2.3.5; zh-cn; TCL_W939; 320*480) UCWEB7.9.3.103/139/32870";
const char S59277[] = "JUC (Linux; U; 2.3.6; zh-cn; Z3Z6Z7; 320*480) UCWEB7.9.4.145/139/800";
const char S59278[] = "JUC (Linux; U; 2.3.4; zh-cn; G1; 480*800) UCWEB7.9.0.94/139/352";
const char S59279[] = "JUC (Linux; U; 2.3.6; zh-cn; GLX_G1; 320*480) UCWEB7.9.0.94/139/352";
const char S59280[] = "JUC (Linux; U; 2.3.7; zh-cn; X8; 320*480) UCWEB7.9.0.94/139/444";
const char S59281[] = "JUC (Linux; U; 2.3.6; zh-cn; YUSUN_-_W_800; 320*480) UCWEB7.9.3.103/139/800";
const char S59282[] = "JUC (Linux; U; 2.3.5; zh-cn; ZTE_N960; 480*800) UCWEB7.9.3.103/139/32602";
const char S59283[] = "JUC (Linux; U; 2.3.6; zh-cn; signal75_gb2; 320*480) UCWEB7.9.0.94/139/800";
const char S59284[] = "JUC (Linux; U; 2.3.6; zh-cn; C208; 320*480) UCWEB7.9.4.145/139/800";
const char S59285[] = "JUC (Linux; U; 2.3.5; zh-cn; Coolpad_7005; 320*480) UCWEB7.9.3.103/139/32765";
const char S59286[] = "JUC (Linux; U; 2.2.2; zh-cn; Coolpad_W706+; 320*480) UCWEB7.9.4.145/139/32411";
const char S59287[] = "JUC (Linux; U; 2.3.6; zh-cn; A811; 320*480) UCWEB7.9.4.145/139/800";
const char S59288[] = "JUC (Linux; U; 2.3.6; zh-cn; W58SX; 320*480) UCWEB7.9.4.145/139/800";
const char S59289[] = "JUC (Linux; U; 4.0.4; zh-cn; MI_1S; 480*854) UCWEB7.9.4.145/139/800";
const char S59290[] = "JUC (Linux; U; 2.3.6; zh-cn; A7; 320*480) UCWEB7.9.0.94/139/355";
const char S59291[] = "JUC (Linux; U; 2.3.3; zh-cn; Z10; 768*1280) UCWEB7.9.0.94/139/352";
const char S59292[] = "JUC (Linux; U; 2.3.6; zh-cn; TSC_Z; 320*480) UCWEB7.9.4.145/139/800";
const char S59293[] = "JUC (Linux; U; 2.3.6; zh-cn; EBEST_F5; 320*480) UCWEB7.9.4.145/139/800";
const char S59294[] = "JUC (Linux; U; 2.3.4; zh-cn; T-Mobile_G2; 480*800) UCWEB7.9.0.94/139/444";
const char S59295[] = "JUC (Linux; U; 2.3.6; zh-cn; Epade_S3; 320*480) UCWEB7.9.0.94/139/800";
const char S59296[] = "JUC (Linux; U; 2.3.5; zh-cn; SPHS_on_Hsdroid; 320*480) UCWEB7.9.4.145/139/800";
const char S59297[] = "JUC (Linux; U; 4.0.4; zh-cn; A21; 540*960) UCWEB7.9.0.94/139/444";
const char S59298[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59299[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; M3 Build/JRO03H)";
const char S59300[] = "http://www.msai.in/uaprof/micromax/X455.xml";
const char S59301[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-06D Build/S3130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59302[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-06D Build/S2281) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59303[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-06D Build/S5081) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59304[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-06DNERV Build/S6050) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59305[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-06DNERV Build/S7231) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59306[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-06D Build/S7231) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59307[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SH-06D Build/S7231)";
const char S59308[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-06DNERV Build/S8310) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59309[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315";
const char S59310[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S59311[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC PH39100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59312[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC PH39100 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59313[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; tr-tr; Turkcell T20 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59314[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; tr-tr; Turkcell T20 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59315[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; tr-tr; Turkcell T20 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59316[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; tr-tr; Turkcell T20 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59317[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A780L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59318[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-A780L Build/IMM76D)";
const char S59319[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Twitter for iPhone";
const char S59320[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Twitter for iPhone";
const char S59321[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; SonyEricssonMT15a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59322[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MT15a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59323[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MT15a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59324[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM107SH Build/S0006) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59325[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM107SH Build/S0010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59326[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SBM107SH Build/S0010)";
const char S59327[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM107SHB Build/S0006) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59328[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM107SH Build/S0024) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59329[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM107SH Build/S0026) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59330[] = "Mozilla/5.0 (Linux; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S59331[] = "Mozilla/5.0 (Linux; Android 4.0.4; ME865 Build/6.7.2_GC-220-EDS-3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S59332[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9152 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S59333[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Digitac Ultratab;C925 Build/ICS) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59334[] = "Mozilla/5.0 (Linux;U; Android 4.0.3; en-us;Digitac Ultratab;C925 Build/ICS) AppleWebKit/534.30 (KHTML,like Gecko) Version/4.0 Safari/534.30;SurfTab_7.0";
const char S59335[] = "Opera/9.7 (Windows Mobile; PPC; Opera Mobi/35166; U; en) Presto/2.2.1";
const char S59336[] = "Opera/9.7 (WindowsMobile; PPC; Opera Mobi/35267; U; en; Presto/2.1.1)";
const char S59337[] = "Mozilla/5.0 (Linux; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59338[] = "Mozilla/5.0 (Linux; Android 4.0.4; ME865 Build/6.7.2_GC-220-EDS-3) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59339[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT30p Build/7.0.A.3.195) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S59340[] = "Opera/9.80 (BREW; Opera Mini/5.0/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S59341[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30a Build/9.1.A.0.489) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59342[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GEM10312BK Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59343[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GEM10312BK Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59344[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; C5302 Build/12.0.A.1.211)";
const char S59345[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; C5302 Build/12.0.A.1.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59346[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C5302 Build/12.0.A.1.257) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59347[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; C5302 Build/12.0.A.2.245) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59348[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C5302 Build/12.0.A.1.284) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59349[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; C5302 Build/12.0.A.2.254) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59350[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59351[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59352[] = "Mozilla/5.0 (Series40; NokiaC2-06/07.48; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.9";
const char S59353[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; IS05 Build/S9290) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59354[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; IS05 Build/S3080) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59355[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-; IS05 Build/S9290) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59356[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; IS05 Build/S8221) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59357[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN7BG3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59358[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XOOM 2 ME Build/7.7.1-128_MZ608-14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59359[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MZ608 Build/7.7.1-141-7-FLEM-UMTS-LA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59360[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S59361[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S59362[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S59363[] = "iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A1+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59364[] = "Mozilla/5.0 (compatible; MSIE 4.0; PPC; Trident/4.0)";
const char S59365[] = "Mozilla/5.0 (compatible; MSIE 4.0; PPC; SV1; Trident/1.0)";
const char S59366[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E460 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59367[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E460f Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59368[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu/12.10 Chromium/25.0.1364.172 Chrome/25.0.1364.172 Safari/537.22";
const char S59369[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu/12.10 Chromium/25.0.1364.172 Chrome/25.0.1364.172 Safari/537.22";
const char S59370[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.13.13719.201";
const char S59371[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.13.13719.201";
const char S59372[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.13.13719.201";
const char S59373[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C) QQBrowser/6.13.13719.201";
const char S59374[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MALN; .NET4.0C) QQBrowser/6.13.13719.201";
const char S59375[] = "Mozilla/4.0 (compatible; MSIE 9.0; qdesk 2.3.1186.202; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.13.13719.201";
const char S59376[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; MALN) QQBrowser/6.13.13719.201";
const char S59377[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; QQDownload 716; Mozilla/4.0(Compatible-EmbeddedWB 14.59 http://bsalsa.com/ ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0) QQBrowser/6.13.13719.201";
const char S59378[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; ALCATEL ONE TOUCH 992D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59379[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ALCATEL ONE TOUCH 992D Build/IMM76D)";
const char S59380[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P715 Build/JZO54K)";
const char S59381[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-P715 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59382[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I497 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S59383[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30at Build/9.1.F.1.120) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S59384[] = "Dolphin http client/10.1.5(105) (Android)";
const char S59385[] = "Dolphin http client/10.1.5(104) (Android)";
const char S59386[] = "Dolphin http client/10.1.4(103) (Android)";
const char S59387[] = "Dolphin http client/10.1.0 Beta 3(247) (Android)";
const char S59388[] = "Dolphin http client/10.1.0(248) (Android)";
const char S59389[] = "Dolphin http client/10.1.0 Beta 2(246) (Android)";
const char S59390[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari Version/5.1";
const char S59391[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari Version/5.1";
const char S59392[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; zh-cn Model:iPad2,1) UC AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7543.48.3";
const char S59393[] = "Mozilla/5.0 (iPad; CPU OS 4_3_3 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S59394[] = "Mozilla/5.0 (iPad; CPU OS 4_3_5 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S59395[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 Safari Version/5.1";
const char S59396[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 Safari Version/5.1";
const char S59397[] = "Mozilla/5.0 (iPad; CPU OS 4_3_2 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S59398[] = "Mozilla/5.0 (iPad; CPU OS 4_3 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S59399[] = "Mozilla/5.0 (iPad; CPU OS 4_3_1 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S59400[] = "Mozilla/5.0 (iPad; CPU OS 4_3_4 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S59401[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Safari/535.19 Silk-Accelerated=false";
const char S59402[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-H9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59403[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-H9500 Build/JOP40D)";
const char S59404[] = "SAMSUNG-SGH-A777/A777UCIF2; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.14315; U; en-US) Opera 9.50";
const char S59405[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Novo10 Hero Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59406[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Novo 10 Hero QuadCore Build/JRO03C)";
const char S59407[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Novo10 Hero Build/X)";
const char S59408[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Novo 10 Hero QuadCore Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59409[] = "SAMSUNG-GT-C3312R/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S59410[] = "Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E330S Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S59411[] = "Mozilla/5.0 (Linux; Android 4.0.4; BT210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59412[] = "Mozilla/5.0 (Linux; Android 4.0.4; Avvio_790 Build/AvvioAvvio_790) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59413[] = "Mozilla/5.0 (Linux; Android 4.0.4; 5I Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59414[] = "Mozilla/5.0 (Linux; Android 4.0.3; CTA Build/ICS.casper.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S59415[] = "Mozilla/5.0 (Linux; Android 4.0.4; A11Note Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59416[] = "Mozilla/5.0 (Linux; Android 4.0.4; CA-201L Build/CA201LU410_00) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59417[] = "Mozilla/5.0 (Linux; Android 4.0.4; magcomm77_cu_ics2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59418[] = "Mozilla/5.0 (Linux; Android 4.0.4; JY139 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59419[] = "Mozilla/5.0 (Linux; Android 4.0.3; 740RS Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59420[] = "Mozilla/5.0 (Linux; Android 4.0.4; EverShine Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59421[] = "Mozilla/5.0 (Linux; Android 4.0.4; SMP53-210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59422[] = "Mozilla/5.0 (Linux; Android 4.0.4; 9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59423[] = "Mozilla/5.0 (Linux; Android 4.0.4; N900 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59424[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAD1001 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S59425[] = "Mozilla/5.0 (Linux; Android 4.0.4; tmn smart a18 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59426[] = "Mozilla/5.0 (Linux; Android 4.0.4; M1001 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S59427[] = "Mozilla/5.0 (Linux; Android 4.0.4; TM450 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59428[] = "Mozilla/5.0 (Linux; Android 4.0.4; N8000_USA_RS Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59429[] = "Mozilla/5.0 (Linux; Android 4.0.4; P6 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59430[] = "Mozilla/5.0 (Linux; Android 4.0.4; 7102 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59431[] = "Mozilla/5.0 (Linux; Android 4.0.4; I110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59432[] = "Mozilla/5.0 (Linux; Android 4.0.4; BASE_Lutea_3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59433[] = "Mozilla/5.0 (Linux; Android 4.0.4; SMP45-210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59434[] = "Mozilla/5.0 (Linux; Android 4.0.4; ViewPad 7D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59435[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST70104-2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59436[] = "Mozilla/5.0 (Linux; Android 4.0.4; T9002 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S59437[] = "Mozilla/5.0 (Linux; Android 4.0.4; M980 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S59438[] = "Mozilla/5.0 (Linux; Android 4.0.4; A13 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59439[] = "Mozilla/5.0 (Linux; Android 4.0.4; GLORY Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59440[] = "Mozilla/5.0 (Linux; Android 4.0.4; S7916E Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S59441[] = "Mozilla/5.0 (Linux; Android 4.0.4; MOBHIG0080 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59442[] = "Mozilla/5.0 (Linux; Android 4.0.4; M9100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59443[] = "Mozilla/5.0 (Linux; Android 4.0.4; PC7058ME Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59444[] = "Mozilla/5.0 (Linux; Android 4.0.4; HX-M02 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S59445[] = "Mozilla/5.0 (Linux; Android 4.0.3; Infinity Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59446[] = "Mozilla/5.0 (Linux; Android 4.0.4; 3G7271 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59447[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT25i Build/9.1.A.1.140) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59448[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT25i Build/9.1.A.1.140) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59449[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaX5-01.2/043.016; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.7.4 3gpp-gba";
const char S59450[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaX5-01/043.005; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.3 3gpp-gba";
const char S59451[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; GT-I5800L Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59452[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I5800L Build/ECLAIR)";
const char S59453[] = "Motorola_ES405B/191 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5.3.5)";
const char S59454[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Motorola_ES405B_19103; Windows Phone 6.5.3.5)";
const char S59455[] = "Motorola_ES405B/20 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5.3.5)";
const char S59456[] = "Motorola_ES405B/27 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Motorola_ES405B_256; Windows Phone 6.5.3.5)";
const char S59457[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Motorola_ES405B_312; Windows Phone 6.5.3.5)";
const char S59458[] = "Motorola_ES405B/22 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5.3.5)";
const char S59459[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Motorola_ES405B_256; Windows Phone 6.5.3.5)";
const char S59460[] = "Motorola_ES405B/28 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Motorola_ES405B_256; Windows Phone 6.5.3.5)";
const char S59461[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Motorola_ES405B_44.000; Windows Phone 6.5.3.5)";
const char S59462[] = "NokiaC5-03/UCWEB8.6.0.180/50/800";
const char S59463[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaC5-03) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile";
const char S59464[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One XL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S59465[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC EVA_UTL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S59466[] = "SAMSUNG-GT-E2600/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S59467[] = "SAMSUNG-GT-E2600_ORANGE/E2600BVKL5 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S59468[] = "Mozilla/5.0 (Linux; Android 4.0.4; C1605 Build/11.1.A.0.68) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59469[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; HTC_A3335 V2.38.841.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59470[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; HTC_A3335 V1.21.841.3 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59471[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; HTC_A3335 V1.30.841.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59472[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; Micromax A52 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59473[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A52 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59474[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; Micromax-Xzoom A52 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59475[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; A52 Build/GRK39F)";
const char S59476[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; Micromax-Xtreme Droid A52 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59477[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; S5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59478[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S5 Build/S5-V1.0-2012.11.30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59479[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; S5 Build/S5-V1.0-2012.12.01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59480[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59481[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S5 Build/S5-V1.0-2013.04.12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59482[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTCEVOV4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59483[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTCEVOV4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59484[] = "Mozilla/5.0 (Linux; U; Android 1.5; pt-br; LG Eve) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S59485[] = "Mozilla/5.0 (Linux; U; Android 1.5; LG Eve-Orange) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 (LG_UA; AD_LOGON=LGE.NET)";
const char S59486[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; LG Eve-Orange) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S59487[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-ca; LG GW620R Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S59488[] = "Mozilla/5.0 (Linux; U; Android 1.5; LG Eve) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S59489[] = "LGE-UN272/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S59490[] = "LGE-AN272/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S59491[] = "Mozilla/5.0 (Linux; Android 4.0.4; MOBHIG0080 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59492[] = "Mozilla/5.0 (Linux; Android 4.0.4; tmn smart a18 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59493[] = "Mozilla/5.0 (Linux; Android 4.0.4; 7102 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59494[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAD1001 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59495[] = "Mozilla/5.0 (Linux; Android 4.0.4; CJ ThL Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59496[] = "Mozilla/5.0 (Linux; Android 4.0.4; N1 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59497[] = "Mozilla/5.0 (Linux; Android 4.0.4; BASE_Lutea_3 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59498[] = "Mozilla/5.0 (Linux; Android 4.0.4; T7 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59499[] = "Mozilla/5.0 (Linux; Android 4.0.3; CTA Build/ICS.casper.X) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59500[] = "Mozilla/5.0 (Linux; Android 4.0.4; N900 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59501[] = "Mozilla/5.0 (Linux; Android 4.0.4; A11Note Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59502[] = "Mozilla/5.0 (Linux; Android 4.0.3; Infinity Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59503[] = "Mozilla/5.0 (Linux; Android 4.0.3; Explosion Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59504[] = "Mozilla/5.0 (Linux; Android 4.0.4; Alpha GT Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59505[] = "Mozilla/5.0 (Linux; Android 4.0.4; P6 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59506[] = "Mozilla/5.0 (Linux; Android 4.0.4; 1688-CJ-C30Q8-31S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59507[] = "Mozilla/5.0 (Linux; Android 4.0.4; A76 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59508[] = "Mozilla/5.0 (Linux; Android 4.0.4; 9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59509[] = "Mozilla/5.0 (Linux; Android 4.0.3; Clear Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59510[] = "Mozilla/5.0 (Linux; Android 4.0.4; GLORY Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59511[] = "Mozilla/5.0 (Linux; Android 4.0.4; INFINITY_PASSION Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59512[] = "Mozilla/5.0 (Linux; Android 4.0.4; A26 Build/CROSSCROSS_A26) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59513[] = "Mozilla/5.0 (Linux; Android 4.0.4; M1005D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59514[] = "Mozilla/5.0 (Linux; Android 4.0.4; s4502 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59515[] = "Mozilla/5.0 (Linux; Android 4.0.4; cyclone Titan Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59516[] = "Mozilla/5.0 (Linux; Android 4.0.4; S7916E Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59517[] = "Mozilla/5.0 (Linux; Android 4.0.4; AMI-TT2 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59518[] = "Mozilla/5.0 (Linux; Android 4.0.4; hexing77_ics2 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59519[] = "Mozilla/5.0 (Linux; Android 4.0.4; Strike Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59520[] = "Mozilla/5.0 (Linux; Android 4.0.4; Q75 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59521[] = "Mozilla/5.0 (Linux; Android 4.0.4; M1005 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59522[] = "Mozilla/5.0 (Linux; Android 4.0.3; X9015 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59523[] = "Mozilla/5.0 (Linux; Android 4.0.4; HD II Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59524[] = "Mozilla/5.0 (Linux; Android 4.0.4; JY139 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59525[] = "Mozilla/5.0 (Linux; Android 4.0.4; GHIA 27154P Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59526[] = "Mozilla/5.0 (Linux; Android 4.0.4; V8200plus Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59527[] = "Mozilla/5.0 (Linux; Android 4.0.4; PICOpad GEW Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59528[] = "Mozilla/5.0 (Linux; Android 4.0.4; QT-70BW Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59529[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGP312 Build/10.1.C.0.370) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S59530[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia2690) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S59531[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-T769 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59532[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SHL21 Build/SA132) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59533[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SHL21 Build/SB070) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59534[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SHL21 Build/SB300) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59535[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SHL21 Build/SC260) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59536[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SHL21 Build/S2140) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59537[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHL21 Build/S2140)";
const char S59538[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/24.0.648.205 Safari/534.16";
const char S59539[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/24.0.697.0 Safari/534.24";
const char S59540[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/24.0.648.205 Safari/534.16";
const char S59541[] = "SonyEricssonW100i/R1CA009 TelecaBrowser/Q07C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S59542[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Wildfire S Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59543[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Wildfire S Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59544[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Wildfire S Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59545[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Wildfire S A510e Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59546[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.4.2.X (Second Life Release; default skin) Safari/533.3";
const char S59547[] = "Mozilla/5.0 (Linux; Android 4.0.4; C5155 Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S59548[] = "Mozilla/5.0 (Linux; Android 4.0.4; Rise Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S59549[] = "Opera/8.01 (J2ME/MIDP; Opera Mini/3.0.6306/1724; en; U; ssr)";
const char S59550[] = "Mozilla/5.0 (Windows; U; Win 9x 4.90; nl; rv:1.8.1.20) Gecko/X Firefox/2.0.0.20";
const char S59551[] = "Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.8.1.3) Gecko/X Firefox/2.0.0.3";
const char S59552[] = "Mozilla/5.0 (Windows NT 5.1; rv:14.0) Gecko/X Firefox/14.0.1 SeaMonkey/2.11";
const char S59553[] = "Mozilla/5.0 (Windows NT 5.1; rv:14.0) Gecko/X SeaMonkey/2.11";
const char S59554[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:14.0) Gecko/X Firefox/14.0.1 SeaMonkey/2.11";
const char S59555[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SH-13C Build/S7160) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59556[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SH-13C Build/S9090) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59557[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SH-13C Build/SC190) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59558[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SH-13C Build/S4090) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59559[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SH-13C Build/S8100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59560[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SH-13C Build/S4090)";
const char S59561[] = "Mozilla/5.0 (Linux; Android 4.0.4; Arc S Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59562[] = "Mozilla/5.0 (Linux; Android 4.0.4; Arc Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S59563[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT15a Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59564[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaX6-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S59565[] = "NokiaX6-00/UCWEB8.8.0.212/50/800";
const char S59566[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P1000 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59567[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-us; GT-P1000N Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59568[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P1000 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59569[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P1000 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59570[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; GT-P1000 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59571[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P1000 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59572[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P1000 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59573[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S59574[] = "Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 8.0; Windows NT 5.0) Chrome/20.0.1229.79";
const char S59575[] = "Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 7.0; Windows NT 5.0) Chrome/20.0.1229.79";
const char S59576[] = "Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 9.0; Windows NT 5.0WOW64) Chrome/20.0.1229.79";
const char S59577[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S59578[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySOL21 Build/9.0.F.0.195) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59579[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySOL21 Build/9.0.F.0.226) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59580[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SOL21 Build/9.0.F.0.195) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59581[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SOL21 Build/9.0.F.2.71) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59582[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySOL21 Build/9.0.F.2.71) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59583[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySOL21 Build/9.0.F.2.103) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59584[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SOL21 Build/9.0.F.2.103) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59585[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySOL21 Build/9.0.F.2.128) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59586[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SOL21 Build/9.0.F.2.128) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59587[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SOL21 Build/9.0.F.0.226) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59588[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59589[] = "Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I727 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59590[] = "Mozilla/2.0 (compatible; MSIE 3.01; Windows 98)";
const char S59591[] = "Mozilla/5.0 (compatible; MSIE 3.0; Win 9x 4.90; Trident/1.0)";
const char S59592[] = "Mozilla/5.0 (compatible; MSIE 3.0; Win98; Trident/2.0)";
const char S59593[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID7014 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59594[] = "Logos/0.9.4 (Linux; U; Android 2.3.3; en_US; MID7014 Build/MID7014 X)";
const char S59595[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; P-02D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59596[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; P-02D Build/GRJ90)";
const char S59597[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2) Presto/2.12.388 Version/12.13";
const char S59598[] = "Mozilla/5.0 (Linux; Android 4.1.2; ADR910L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59599[] = "Alcatel-OT-606A/1.0 ObigoInternetBrowser/Q03C";
const char S59600[] = "Alcatel-OT-606A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S59601[] = "Alcatel-OT-606/010_05 1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S59602[] = "Alcatel-OT-606/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S59603[] = "Alcatel-OT-606/1.0 model-orange Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S59604[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SCH-I510 4G Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59605[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SCH-I510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59606[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SCH-I510 Build/FROYO)";
const char S59607[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.2 (KHTML, like Gecko) Comodo_Dragon/15.0.3.0 Chrome/15.0.874.102 Safari/535.2";
const char S59608[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Comodo_Dragon/15.0.3.0 Chrome/15.0.874.102 Safari/535.2";
const char S59609[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; CT705 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59610[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; CT705FR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59611[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; YP-GP1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59612[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; YP-GP1 Build/IMM76D)";
const char S59613[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire C Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S59614[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S59615[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; ALCATEL ONE TOUCH 993D Build/ICECREAM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59616[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ALCATEL ONE TOUCH 993 Build/ICECREAM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59617[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ALCATEL ONE TOUCH 993D Build/ICECREAM)";
const char S59618[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/26.0.648.205 Safari/534.16";
const char S59619[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.27 (KHTML, like Gecko) Chrome/26.0.712.0 Safari/534.27";
const char S59620[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/26.0.648.205 Safari/534.16";
const char S59621[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/537.6+ Ubuntu/12.10 (3.6.0-0ubuntu1) Epiphany/3.6.0";
const char S59622[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/537.6+ Ubuntu/12.10 (3.6.0-0ubuntu1) Epiphany/3.6.0";
const char S59623[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/537.6+ Ubuntu/12.10 (3.6.1-0ubuntu1) Epiphany/3.6.1";
const char S59624[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/537.6+ Ubuntu/12.10 (3.6.1-0ubuntu1) Epiphany/3.6.1";
const char S59625[] = "LGE-VN150/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S59626[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Twitter for iPad";
const char S59627[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Twitter for iPad";
const char S59628[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.3.4641 Chrome/7.0.517.450 Safari/534.7";
const char S59629[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.3.4641 Chrome/7.0.517.450 Safari/534.7";
const char S59630[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-N7000 Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59631[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-N7000 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59632[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-N7000 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59633[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-N7000 Build/IML74K; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59634[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SHV-E160S Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59635[] = "Mozilla/5.0 (Linux; Android 4.0.4; Touchpad Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S59636[] = "Mozilla/5.0 (Linux; Android 4.0.3; TOUCHPAD 7 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59637[] = "Mozilla/5.0 (Linux; Android 4.0.3; CnM-TOUCHPAD 7 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59638[] = "Mozilla/5.0 (Linux; Android 4.0.4; CnM-TOUCHPAD 9 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59639[] = "Mozilla/5.0 (Linux; Android 4.0.3; Touchpad7 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59640[] = "Mozilla/5.0 (Linux; Android 4.0.3; CnM-TOUCHPAD7 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59641[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS-TOUCHPAD 9 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59642[] = "Mozilla/5.0 (Linux; Android 4.0.4; CnM TOUCHPAD 9 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59643[] = "Mozilla/5.0 (Linux; Android 4.0.4; CnM-TOUCHPAD9 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59644[] = "Mozilla/5.0 (Linux; Android 4.0.4; CnM Touchpad 9.7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S59645[] = "Mozilla/5.0 (Linux; Android 4.0.4; TOUCHPAD 7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59646[] = "Mozilla/5.0 (Linux; Android 4.0.4; Versus TouchPad 7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59647[] = "Mozilla/5.0 (Linux; Android 4.0.4; CnM TouchPad 7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59648[] = "Mozilla/5.0 (Linux; Android 4.0.4; Touchpad10 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S59649[] = "Mozilla/5.0 (Linux; Android 4.0.4; CnM-TOUCHPAD 7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59650[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SC-03D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59651[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-; SC-03D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59652[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SC-03D Build/IMM76D)";
const char S59653[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5302 Build/12.0.A.1.257) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59654[] = "Mozilla/5.0 (Windows NT 4.10) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S59655[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire X Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59656[] = "Mozilla/5.0 (Linux; U; Linux Ventana; en-us; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59657[] = "Mozilla/5.0 (Linux; U; Linux Ventana; zh-tw; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59658[] = "Mozilla/5.0 (Linux; U; Linux Ventana; en-gb; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59659[] = "Mozilla/5.0 (Linux; U; Linux Ventana; de-ch; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59660[] = "Mozilla/5.0 (Linux; U; Linux Ventana; en-ca; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59661[] = "Mozilla/5.0 (Linux; U; Linux Ventana; zh-cn; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59662[] = "Mozilla/5.0 (Linux; U; Linux Ventana; sv-se; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59663[] = "Mozilla/5.0 (Linux; U; Linux Ventana; nl-nl; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59664[] = "Mozilla/5.0 (Linux; U; Linux Ventana; nb-no; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59665[] = "Mozilla/5.0 (Linux; U; Linux Ventana; fr-fr; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59666[] = "Mozilla/5.0 (Linux; U; Linux Ventana; de-de; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59667[] = "Mozilla/5.0 (Linux; U; Linux Ventana; fr-ch; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59668[] = "Mozilla/5.0 (Linux; U; Linux Ventana; fr-ca; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59669[] = "Mozilla/5.0 (Linux; U; Linux Ventana; pt-pt; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59670[] = "Mozilla/5.0 (Linux; U; Linux Ventana; cs-cz; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59671[] = "Mozilla/5.0 (Linux; U; Linux Ventana; da-dk; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59672[] = "Mozilla/5.0 (Linux; U; Linux Ventana; fi-fi; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59673[] = "Mozilla/5.0 (Linux; U; Linux Ventana; zh-tw; Transformer TF101 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59674[] = "Mozilla/5.0 (Linux; U; Linux Ventana; en-us; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59675[] = "Mozilla/5.0 (Linux; U; Linux Ventana; es-es; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59676[] = "Mozilla/5.0 (Linux; U; Linux Ventana; it-it; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59677[] = "Mozilla/5.0 (Linux; U; Linux Ventana; en-us; Transformer TF101 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59678[] = "Mozilla/5.0 (Linux; U; Linux Ventana; en-us; Transformer TF101 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59679[] = "Mozilla/5.0 (Linux; U; Linux Ventana; en-gb; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59680[] = "Mozilla/5.0 (Linux; U; Linux Ventana; nl-nl; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59681[] = "Mozilla/5.0 (Linux; U; Linux Ventana; fr-fr; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59682[] = "Mozilla/5.0 (Linux; U; Linux Ventana; zh-tw; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59683[] = "Mozilla/5.0 (Linux; U; Linux Ventana; nl-nl; Transformer TF101 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59684[] = "Mozilla/5.0 (Linux; U; Linux Ventana; en-ca; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59685[] = "Mozilla/5.0 (Linux; U; Linux Ventana; pl-pl; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59686[] = "Mozilla/5.0 (Linux; U; Linux Ventana; pt-br; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59687[] = "Mozilla/5.0 (Linux; U; Linux Ventana; en-gb; Transformer TF101 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59688[] = "Mozilla/5.0 (Linux; U; Linux Ventana; ru-ru; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59689[] = "Mozilla/5.0 (Linux; U; Linux Ventana; it-it; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59690[] = "Mozilla/5.0 (Linux; U; Linux Ventana; en-ca; Transformer TF101 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59691[] = "Mozilla/5.0 (Linux; U; Linux Ventana; de-de; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59692[] = "Mozilla/5.0 (Linux; U; Linux Ventana; sv-se; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59693[] = "Mozilla/5.0 (Linux; U; Linux Ventana; es-es; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59694[] = "Mozilla/5.0 (Linux; U; Linux Ventana; de-de; Transformer TF101 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59695[] = "Mozilla/5.0 (Linux; U; Linux Ventana; nl-be; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59696[] = "Mozilla/5.0 (Linux; U; Linux Ventana; zh-cn; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S59697[] = "Mozilla/5.0 (Linux; U; Android Eclair; es-us Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59698[] = "Mozilla/5.0 (Linux; U; Android Eclair; en- Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59699[] = "Mozilla/5.0 (Linux; U; Android Eclair; es-es Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59700[] = "Mozilla/5.0 (Linux; U; Android Eclair; pt-br Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59701[] = "Mozilla/5.0 (Linux; U; Android Eclair; nl-nl Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59702[] = "Mozilla/5.0 (Linux; U; Android Eclair; en-in Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59703[] = "Mozilla/5.0 (Linux; U; Android Eclair; en-gb Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59704[] = "Mozilla/5.0 (Linux; U; Android Eclair; vi-vn Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59705[] = "Mozilla/5.0 (Linux; U; Android Eclair; en-sg Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59706[] = "Mozilla/5.0 (Linux; U; Android Eclair; en-nz Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59707[] = "Mozilla/5.0 (Linux; U; Android Eclair; ca-es Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59708[] = "Mozilla/5.0 (Linux; U; Android Eclair; bg-bg Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59709[] = "Dalvik/1.1.0 (Linux; U; Android Eclair Build/pandigitalopc1/sourceidDLX)";
const char S59710[] = "Mozilla/5.0 (Linux; U; Android Eclair; de-de Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59711[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; JY-G2 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S59712[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; JY-G2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59713[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JY-G2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59714[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; JY-G2 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S59715[] = "Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia305) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S59716[] = "Mozilla/5.0 (Series40; NokiaC2-06/07.48; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S59717[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; IS04 Build/FFK300) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59718[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; IS04 Build/FFK301) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59719[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; IS04 Build/FFK300)";
const char S59720[] = "Mozilla/5.0 (Windows NT 4.0; rv:22.0) Gecko/X Firefox/22.0";
const char S59721[] = "SAMSUNG-SGH-J700/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S59722[] = "SAMSUNG-SGH-J700-ORANGE/J700ABHB2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S59723[] = "Opera/9.80 (MTK; Nucleus; U; en-US) Presto/2.4.18 Version/10.00";
const char S59724[] = "Fly_E190 Wi-Fi    Opera/9.80 (MTK; Nucleus; U; ru-RU) Presto/2.4.18 Version/10.00";
const char S59725[] = "Fly E185  Opera/9.80 (MTK; Nucleus; U; ru-RU) Presto/2.4.18 Version/10.00";
const char S59726[] = "MVL G99  Opera/9.80 (MTK; Nucleus; U; en-US) Presto/2.4.18 Version/10.00";
const char S59727[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID7016 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59728[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID7016 Build/GRI40)";
const char S59729[] = "Logos/0.9.3 (Linux; U; Android 2.3.3; en_US; MID7016 Build/MID7016 X)";
const char S59730[] = "Mozilla/5.0 (X11; U; Linux i686; it-IT; rv:1.9.0.2) Gecko/X Ubuntu/9.04 (jaunty) Firefox/3.5";
const char S59731[] = "Mozilla/5.0 (X11;U; Linux i686; en-GB; rv:1.9.1) Gecko/X Ubuntu/9.04 (jaunty) Firefox/3.5";
const char S59732[] = "SAMSUNG-GT-S3100/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S59733[] = "SAMSUNG-SGH-A817/A817UCLD2 SHP/VPP/R5 Dolfin/1.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S59734[] = "SAMSUNG-GT-C3322/C3322DDKK1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S59735[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-S960L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59736[] = "Mozilla/5.0 (Linux; Android 4.1.2; USCC-C6721 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59737[] = "Mozilla/5.0 (Linux; Android 4.1.2; USCC-C6721 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59738[] = "NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile";
const char S59739[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LS840 Build/ZVK.IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59740[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Arc S Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59741[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; WL-101GQC Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59742[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; WL-101GQC Build/JZO54K)";
const char S59743[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530M Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59744[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E430 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59745[] = "SAMSUNG-SGH-A867/A867UCHJ3 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S59746[] = "SAMSUNG-SGH-A697/A697UCJB2 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S59747[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S59748[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6400L Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S59749[] = "Mozilla/5.0 (Linux; Android 4.0.4; ThunderBolt Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S59750[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6312 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59751[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020A Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59752[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020D Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59753[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020E Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59754[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020X Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59755[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020W Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59756[] = "Mozilla/5.0 (Linux; Android 4.0.4; N861 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S59757[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT30p Build/7.0.A.1.303) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59758[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; XOOM 2 ME Build/1.6.0_268.1-MZ608) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S59759[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 ME Build/1.6.0_268.4-MZ608) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S59760[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 ME Build/1.6.0_258.1-MZ608) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S59761[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; sv-se; XOOM 2 ME Build/1.6.0_281.4-MZ608) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S59762[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; pt-br; MZ608 Build/1.6.0M_272.4_MZ608) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S59763[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; pt-br; MZ608 Build/1.6.0M_272.12_MZ608) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S59764[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; pt-br; MZ608 Build/1.6.0M_272.18_MZ608) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S59765[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; pt-br; MZ608 Build/1.6.0M_272.12_MZ608) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S59766[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; de-de; XOOM 2 ME Build/1.6.0_268.6-MZ608) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S59767[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; pt-br; MZ608 Build/1.6.0M_272.24_MZ608) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S59768[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.0) Gecko/X Firefox/12.0 SeaMonkey/2.9.1";
const char S59769[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.0) Gecko/X SeaMonkey/2.9.1";
const char S59770[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; U8600 Build/HuaweiU8600) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59771[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:10.0) Gecko/X Thunderbird/10.0";
const char S59772[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DEM752HCF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59773[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S59774[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6102 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S59775[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.9 (KHTML, like Gecko) Iron/4.0.280.0 Chrome/4.0.280.0 Safari/532.9";
const char S59776[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1";
const char S59777[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1";
const char S59778[] = "MT6517-MT6517-H369X/1.0 Linux/Apps_4.0.32.9-perf Android/4.1.1 Release/01.16.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S59779[] = "MT6517-MT6517-H369X/1.0 Linux/Apps_4.0.32.9-perf Android/4.1.1 Release/03.20.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S59780[] = "CoolPad8730_CMCC_TD/1.0 Linux/3.1.10 Android/4.1.1 Release/12.28.2012 Browser/AppleWebkit534.31";
const char S59781[] = "MT-GT-T7100/1.0 Linux/Apps_2.6.32.9-perf Android/4.1.1 Release/03.21.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S59782[] = "sprd-NOTE2/1.0 Linux/2.6.35.7 Android/4.1 Release/03.26.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S59783[] = "E9000 Linux/2.6.35 Android/4.1.1 Release/03.19.2013 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1";
const char S59784[] = "sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/4.1 Release/05.03.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S59785[] = "sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/4.1 Release/03.28.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S59786[] = "sprd-HT-G2/1.0 Linux/2.6.35.7 Android/4.1.3 Release/11.28.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S59787[] = "ZTEU956_TD/1.0 Linux/3.4.0 Android/4.1 Release/1.18.2013 Browser/AppleWebKit534.30";
const char S59788[] = "sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/4.1 Release/05.09.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S59789[] = "HS-T958_TD/1.0 Android/4.1.2 Release/15.03.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1;";
const char S59790[] = "sprd-NOTE2/1.0 Linux/2.6.35.7 Android/4.1 Release/03.21.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S59791[] = "sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/4.1 Release/04.10.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S59792[] = "sprd-SKK-A7/1.0 Linux/2.6.35.7 Android/4.1 Release/05.06.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S59793[] = "sprd-NAIDE/1.0 Linux/2.6.35.7 Android/4.1.0 Release/05.06.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S59794[] = "sprd-NOTE2/1.0 Linux/2.6.35.7 Android/4.1.1 Release/05.14.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S59795[] = "samsung-SKK-A3/1.0 Linux/2.6.35.7 Android/4.1 Release/04.16.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S59796[] = "ZTEU819_TD/1.0 Linux/3.4.0 Android/4.1 Release/2.8.2013 Browser/AppleWebKit534.30";
const char S59797[] = "Coolpad8185_CMCC_TD/1.0 Linux/3.4.5 Android/4.1.2 Release/3.15.2013 Browser/AppleWebkit533.1";
const char S59798[] = "sprd-haowanda/1.0 Linux/2.6.35.7 Android/4.1 Release/01.15.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S59799[] = "Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Samsung/I8910-ORANGE/BVORAIE4; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525";
const char S59800[] = "Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Samsung/I8910-ORANGE/BVIDEIE2; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525";
const char S59801[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Samsung/I8910;; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/";
const char S59802[] = "Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Samsung/I8910-ORANGE/BVORAIF3; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525";
const char S59803[] = "Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Samsung/I8910-ORANGE/BVORAII1; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525";
const char S59804[] = "Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Samsung/I8910-ORANGE/BVORAIE2; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525";
const char S59805[] = "Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Samsung/I8910; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525";
const char S59806[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; TouchPad Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1/tenderloin";
const char S59807[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB520 Build/Blur_Version.3.5.470.MB520.ATT.en.US Flex/P047) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59808[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB520 Build/Blur_Version.3.4.120.MB520.ATT.en.US Flex/P031) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59809[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB520 Build/Blur_Version.3.2.810.MB520.ATT.en.US Flex/P032) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59810[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB520 Build/KOBE_U3_5.35.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59811[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB520 Build/Blur_Version.3.5.350.MB520.ATT.en.US Flex/P044) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59812[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB520 Build/Blur_Version.3.5.340.MB520.ATT.en.US Flex/P044) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59813[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB520 Build/KOBE_U3_2.144.0) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S59814[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB520 Build/KOBE_U3_5.47.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S59815[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:2.0b5) Gecko/X Firefox/4.0b5";
const char S59816[] = "Mozilla/5.0 (Windows NT 6.0; rv:2.0.1) Gecko/X Firefox/4.0.1 Firefox/4.0.1";
const char S59817[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:2.0b12pre) Gecko/X Firefox/4.0b12pre";
const char S59818[] = "Mozilla/5.0 (Windows; Windows NT 6.0; rv:2.0b3) Gecko/X Firefox/4.0b3 (.NET CLR 3.5.30729)";
const char S59819[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; Light Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59820[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Light Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59821[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC_Amaze_4G Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S59822[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Amaze 4G Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S59823[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Ruby Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S59824[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59825[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59826[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59827[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715a Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59828[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Z715e Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59829[] = "Mozilla/5.0 (Windows NT 6.0; rv:22.0) Gecko/X Firefox/22.0 SeaMonkey/2.19";
const char S59830[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; EVO Build/IMM76D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59831[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; EVO Build/JRO03C; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59832[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; EVO Build/JRO03C; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59833[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; PC36100 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59834[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; Opus One Build/RBE.00.00) AppleWebKit/528.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile Safari/525.20.1";
const char S59835[] = "Huawei/1.0/HUAWEI-G7300   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10";
const char S59836[] = "Huawei/1.0/HUAWEI-G7300   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; es-SA) Presto/2.5.28 Version/10.10";
const char S59837[] = "Huawei/1.0/HUAWEI-G7300   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; es) Presto/2.5.28 Version/10.10";
const char S59838[] = "Huawei/1.0/HUAWEI-G7300   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; vi-VN) Presto/2.5.28 Version/10.10";
const char S59839[] = "Huawei/1.0/HUAWEI-G7300   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; it-IT) Presto/2.5.28 Version/10.10";
const char S59840[] = "Huawei/1.0/HUAWEI-G7300   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; pt-PT) Presto/2.5.28 Version/10.10";
const char S59841[] = "Huawei/1.0/HUAWEI-G7300   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; lt-LT) Presto/2.5.28 Version/10.10";
const char S59842[] = "Huawei/1.0/HUAWEI-G7300 Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; es-SA) Presto/2.5.28 Version/10.10";
const char S59843[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-kr; HTC_Raider_X710e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59844[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Raider X710e Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59845[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; HTC_Holiday Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59846[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-kr; HTC_Raider_X710e Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59847[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Raider X710e Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S59848[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Raider 4G Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59849[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Raider X710e Build/GWK74)";
const char S59850[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-kr; HTC Raider X710e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59851[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-T100K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59852[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-T100K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59853[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-T100K Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59854[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-T100K Build/IMM76D)";
const char S59855[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-T100K/2.21 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59856[] = "MQQBrowser/2.0,Nokia6681/2.0";
const char S59857[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5110 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59858[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SCH-i559 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59859[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; SCH-I559 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59860[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SCH-i559 Build/FROYO)";
const char S59861[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; SCH-I559 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59862[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SCH-i559 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S59863[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SCH-i559 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S59864[] = "UCWEB/2.0(BlackBerry; U; 4.4.0.560; en-us; 9650/4.4.0.560) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S59865[] = "UCWEB/2.0(BlackBerry; U; 4.4.0.438; en-us; 9650/4.4.0.438) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S59866[] = "UCWEB/2.0(BlackBerry; U; 4.4.0.315; en-us; 9650/4.4.0.315) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S59867[] = "UCWEB/2.0(BlackBerry; U; 4.4.0.314; en-us; 9650/4.4.0.314) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S59868[] = "UCWEB/2.0(BlackBerry; U; 4.4.0.486; en-us; 9650/4.4.0.486) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S59869[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.318; en-us; 9650/4.2.0.318) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S59870[] = "UCWEB/2.0(BlackBerry; U; 4.4.0.147; en-us; 9650/4.4.0.147) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S59871[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.437; en-us; 9650/4.2.0.437) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S59872[] = "UCWEB/2.0(BlackBerry; U; 4.4.0.348; en-us; 9650/4.4.0.348) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S59873[] = "UCWEB/2.0(BlackBerry; U; 4.4.0.234; en-us; 9650/4.4.0.234) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S59874[] = "UCWEB/2.0(BlackBerry; U; 4.4.0.185; en-us; 9650/4.4.0.185) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S59875[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.424; en-us; 9650/4.2.0.424) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S59876[] = "UCWEB/2.0(BlackBerry; U; 4.4.0.562; en-us; 9650/4.4.0.562) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S59877[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.334; en-us; 9650/4.2.0.334) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S59878[] = "UCWEB/2.0(BlackBerry; U; 4.4.0.376; en-us; 9650/4.4.0.376) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S59879[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Barnes & Noble Nook HD+ Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59880[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Barnes & Noble Nook HD+ Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59881[] = "Mozilla/5.0 (Linux; Android 4.0.3; PadFone Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59882[] = "Mozilla/5.0 (Linux; Android 4.0.3; PadFone Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S59883[] = "Mozilla/5.0 (Linux; Android 4.0.4; PadFone Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S59884[] = "Mozilla/5.0 (Linux; Android 4.0.4; PadFone Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59885[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Butterfly Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59886[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Safari/535.19 Silk-Accelerated=false";
const char S59887[] = "Mozilla/5.0 (Linux; Android 4.0.3; ThinkPad Tablet Build/ThinkPadTablet_A400_03) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S59888[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NABIJR-NV5A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59889[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59890[] = "Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I717 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59891[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59892[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Flipboard/1.9.8";
const char S59893[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Flipboard/1.9.8";
const char S59894[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B147 Flipboard/1.9.8";
const char S59895[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/1.9.8";
const char S59896[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Twitter for iPad";
const char S59897[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Twitter for iPad";
const char S59898[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Twitter for iPad";
const char S59899[] = "Mozilla/5.0 (Linux; Android 4.0.3; A200 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S59900[] = "LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S59901[] = "Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; Lenovo-S890/S100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S59902[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Lenovo S890 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59903[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Lenovo S890 Build/JRO03C)";
const char S59904[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Lenovo S890 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59905[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOR801 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59906[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-TH; R8015 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59907[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; R801 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S59908[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; R801 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S59909[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; OPPOR8015 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59910[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; R801 Build/GRK39F)";
const char S59911[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; R8015 Build/GRK39F)";
const char S59912[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700T Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59913[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700KL Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59914[] = "Mozilla/5.0 (Linux; Android 4.1.2; ASUS Transformer Pad TF700T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59915[] = "Mozilla/5.0 (Linux; Android 4.1.2; PantechP8010 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S59916[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5800 XpressMusic) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S59917[] = "Mozilla/5.0 (Linux; U; Android 1.6; pt-br; E10a Build/1.2.A.1.174) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S59918[] = "Mozilla/5.0 (Linux; U; Android 1.6; es-mx; SonyEricssonE10a Build/1.2.A.1.181) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S59919[] = "Mozilla/5.0 (Linux; U; Android 1.6; es-mx; E10a Build/1.1.A.0.8) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S59920[] = "Mozilla/5.0 (Linux; U; Android 1.6; es-mx; E10a Build/1.2.A.1.181) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S59921[] = "Mozilla/5.0 (Linux; U; Android 1.6; pt-br; E10a Build/1.1.A.0.9) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S59922[] = "Mozilla/5.0 (Linux; U; Android 1.6; es-co; E10a Build/1.0.A.1.38) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S59923[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; E10a Build/1.1.A.0.12) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S59924[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S59925[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ARCHOS 101 Titanium Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59926[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ARCHOS 101 Titanium Build/JRO03H)";
const char S59927[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I777 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59928[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I777 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59929[] = "Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I777 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59930[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-I777 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59931[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ZiiLABS ZiiO7  Build/v2.02.04) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile Safari/528.16";
const char S59932[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; Zio Build/DRC92) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S59933[] = "Mozilla/5.0 (Linux; U; Android 1.6; es-us; Zio Build/DRC92)";
const char S59934[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EVO Build/IML74K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59935[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PC36100 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59936[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; EVO Build/IML74K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59937[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; EVO Build/IMM76D; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59938[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PC36100 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59939[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PC36100 Build/JRO03R; CyanogenMod-Team D.I.R.T RemICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59940[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.19 (KHTML, like Gecko) Maxthon/1.0.3.1000 Chrome/18.0.1025.142 Safari/535.19";
const char S59941[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/535.19 (KHTML, like Gecko) Maxthon/1.0.3.1000 Chrome/18.0.1025.142 Safari/535.19";
const char S59942[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/1.0.4.1000 Chrome/21.0.1180.75 Safari/537.1";
const char S59943[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/1.0.4.2000 Chrome/21.0.1180.75 Safari/537.1";
const char S59944[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/1.0.4.2000 Chrome/21.0.1180.75 Safari/537.1";
const char S59945[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.19 (KHTML, like Gecko) Maxthon/1.0.3.1000 Chrome/18.0.1025.142 Safari/535.19";
const char S59946[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/1.0.4.2000 Chrome/21.0.1180.75 Safari/537.1";
const char S59947[] = "Mozilla/5.0 (X11; Linux x86_64; rv:16.0) Gecko/X Thunderbird/16.0.2";
const char S59948[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PMP3170B Build/PMP3170B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59949[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PMP3170BEN Build/PMP3170BEN) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59950[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Safari/535.19 Silk-Accelerated=false";
const char S59951[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:12.0) Gecko/X Firefox/12.0-x64 PaleMoon/12.0-x64";
const char S59952[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.0) Gecko/X Firefox/12.0 PaleMoon/12.0";
const char S59953[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825-1 Build/HuaweiU8825-1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59954[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59955[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI C8825D Build/HuaweiC8825D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59956[] = "Mozilla/5.0 (Linux; Android 4.0.4; Bs 401 Build/HuaweiU8825-1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S59957[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Mobile Safari/535.19 Silk-Accelerated=false";
const char S59958[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Amaze 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S59959[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC_Amaze_4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S59960[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Ruby Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S59961[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; LG-F120K Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59962[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-F120K Build/GRK39F)";
const char S59963[] = "Opera/12.80 (Windows NT 6.2; WOW64; U; en)Presto/22.9.168 Version/12.80";
const char S59964[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae71-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S59965[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE71) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S59966[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE71x) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S59967[] = "NokiaE71/UCWEB8.9.0.253/28/999";
const char S59968[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-au; V9 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59969[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; V9 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59970[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; V9 Build/FRF91)";
const char S59971[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T989 Build/IML74K; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59972[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T989 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S59973[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-T989 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59974[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S59975[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S59976[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S59977[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437Z Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S59978[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59979[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59980[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59981[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; Edition MAS; en) Presto/2.10.229 Version/11.64";
const char S59982[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; URBANO PROGRESSO Build/008.2.2400) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59983[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; URBANO PROGRESSO Build/008.4.2480) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59984[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; URBANO PROGRESSO Build/010.0.3000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59985[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; URBANO PROGRESSO Build/011.0.3100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S59986[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59987[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus 7 Build/JRO03S) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59988[] = "Mozilla/5.0 (Linux; Android 4.1; Nexus 7 Build/JRN84D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S59989[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S59990[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB611 Build/BGN_2.0.83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59991[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB611 Build/BGN_2.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59992[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MB611 Build/BGN_2.0.41) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59993[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB611 Build/4.5.1-149-BGN-2.0-87) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59994[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB611 Build/4.5.1-149-BGN-2.0-84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S59995[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9A405 Safari/7534.48.3";
const char S59996[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9A334 Safari/7534.48.3";
const char S59997[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 925)";
const char S59998[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 925; Vodafone)";
const char S59999[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 925; ANZ892)";
const char S60000[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 925; 1326-2)";
const char S60001[] = "Mozilla/5.0 (Linux; Android 4.1.2; Torque Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60002[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:7.0.1) Gecko/X Postbox/3.0.8";
const char S60003[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 TeamStream HD (iPad2,1 284 1.7)";
const char S60004[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3 TeamStream HD (iPad2,1 284 1.7)";
const char S60005[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550 Safari/7534.48.3 TeamStream HD (iPad2,5 284 1.7)";
const char S60006[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8426 Safari/7534.48.3 TeamStream HD (iPad3,6 284 1.7)";
const char S60007[] = "Nokia311/5.0 (03.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia311) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S60008[] = "Opera/9.51 Beta (Microsoft Windows; PPC; Opera Mobi/1718; U; en)";
const char S60009[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8681 Build/HuaweiU8681) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S60010[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; Blade S Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60011[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E615 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S60012[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E615f Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S60013[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; SGH-I717M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60014[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-I717M Build/GINGERBREAD)";
const char S60015[] = "Nokia308/2.0 (05.85) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S60016[] = "Nokia308/2.0 (05.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 Nokia308/05.80 CLDC-1.1";
const char S60017[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.4.2.3000 Chrome/18.0.966.0 Safari/535.12";
const char S60018[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.4.2.3000 Chrome/18.0.966.0 Safari/535.12";
const char S60019[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60020[] = "Mozilla/5.0 (Linux; Android 4.1.1; IdeaTabA2109A Build/JRO03R) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S60021[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT28h Build/6.2.B.0.211) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60022[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT28i Build/6.2.B.0.211) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60023[] = "Mozilla/5.0 (Linux; Android 4.0.4; N9500 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60024[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; IM-T100K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60025[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; IM-T100K Build/GINGERBREAD)";
const char S60026[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; IM-T100K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60027[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; IM-T100K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60028[] = "Safari5533.22.3 CFNetwork/438.16 Darwin/9.8.0 (Power%20Macintosh) (PowerBook5%2C6)";
const char S60029[] = "Safari5533.18.5 CFNetwork/438.14 Darwin/9.8.0 (i386) (MacBook4%2C1)";
const char S60030[] = "Safari5533.19.4 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac7%2C1)";
const char S60031[] = "Safari5533.21.1 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook5%2C2)";
const char S60032[] = "Safari5533.22.3 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacPro1%2C1)";
const char S60033[] = "Safari5533.22.3 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBookPro5%2C4)";
const char S60034[] = "Safari5533.22.3 CFNetwork/438.16 Darwin/9.8.0 (i386) (Macmini2%2C1)";
const char S60035[] = "Safari5533.21.1 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBookPro5%2C1)";
const char S60036[] = "Safari5533.22.3 CFNetwork/438.16 Darwin/9.8.0 (Power%20Macintosh) (PowerMac12%2C1)";
const char S60037[] = "Safari5533.21.1 CFNetwork/438.16 Darwin/9.8.0 (Power%20Macintosh) (PowerMac11%2C2)";
const char S60038[] = "Safari5533.22.3 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBookAir1%2C1)";
const char S60039[] = "Safari5533.21.1 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac8%2C1)";
const char S60040[] = "Safari5533.19.4 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook5%2C1)";
const char S60041[] = "Safari5533.22.3 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac5%2C1)";
const char S60042[] = "Safari5533.19.4 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBookPro4%2C1)";
const char S60043[] = "Safari5533.20.27 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBookPro4%2C1)";
const char S60044[] = "Safari5533.20.27 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook5%2C2)";
const char S60045[] = "Safari5533.21.1 CFNetwork/438.16 Darwin/9.8.0 (i386) (Macmini2%2C1)";
const char S60046[] = "Safari5533.20.27 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac9%2C1)";
const char S60047[] = "Safari5533.22.3 CFNetwork/438.14 Darwin/9.8.0 (i386) (MacBook5%2C1)";
const char S60048[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Maxthon/3.0)";
const char S60049[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Maxthon/3.0)";
const char S60050[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Maxthon/3.0)";
const char S60051[] = "Mozilla/5.0 (Series40; Nokia303/13.47; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S60052[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; GT-P7100 Build/HRI83) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S60053[] = "Dalvik/1.5.0 (Linux; U; Android 3.0.1; GT-P7100 Build/HRI83)";
const char S60054[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; ru-; GT-P7100 Build/HRI83) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S60055[] = "Mozilla/5.0 (Linux; U; Android 3.0; xx-xx; GT-P7100 Build/HRI83) AppleWebkit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S60056[] = "Mozilla/5.0 (Linux; U; Android 3.0; en-us; GT-P7100 Build/HRI53) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S60057[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SHW-M290K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60058[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M290K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60059[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M290K Build/GINGERBREAD)";
const char S60060[] = "NokiaC2-06/2.0 (07.48) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S60061[] = "NokiaC2-06/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-06";
const char S60062[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LG-VM670 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60063[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-VM670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60064[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-VM670 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60065[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LG Optimus V Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60066[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-VM670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60067[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-VM670 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60068[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM670 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60069[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; VM670 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60070[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM670 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60071[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM670 Build/ICS19; ICSandwich2.3.7 - Final Version) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60072[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; M670 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60073[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM670 Build/ICS40; ICSandwich2.3.7-FINAL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60074[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM670 Build/ICS43; ICSandwich v3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60075[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM670 Build/ICS23; ICSandwich v2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60076[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; LG-VM670 Build/GRI40)";
const char S60077[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11";
const char S60078[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11";
const char S60079[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAN95) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S60080[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAN95 8GB) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S60081[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:22.0) Gecko/X Firefox/22.0 SeaMonkey/2.19";
const char S60082[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E973 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60083[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F200S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60084[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F200K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60085[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F200L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60086[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7510 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60087[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7510 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60088[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7510 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60089[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7510 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60090[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P7510 Build/JZO54K)";
const char S60091[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7510 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60092[] = "Vyrso/1.1.3 (Linux; U; Android 4.1.1; en-US; GT-P7510 Build/cm_p4wifi-userdebug 4.1.1 JRO03H eng.root.X.X test-keys)";
const char S60093[] = "Logos/2.0.9 (Linux; U; Android 4.1.1; en-US; GT-P7510 Build/full_GT-P7510-userdebug 4.1.1 JRO03L eng.maplesci.X.X test-keys)";
const char S60094[] = "Logos/2.0.11 (Linux; U; Android 4.1.1; en-GB; GT-P7510 Build/cm_p4wifi-userdebug 4.1.1 JRO03L eng.matt.X.X test-keys)";
const char S60095[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P7510 Build/JRO03H)";
const char S60096[] = "Logos/2.1.2 (Linux; U; Android 4.1.1; en-US; GT-P7510 Build/cm_p4wifi-userdebug 4.1.1 JRO03H eng.maplesci.X.X test-keys)";
const char S60097[] = "Groupon/2.0.2414 (Android 4.1.1; Samsung P4wifi / Samsung GT-P7510; )[preload=false;locale=en_US;clientidbase=android-google]";
const char S60098[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7510 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/1.9.11;FBDM/{density=1.0,width=1280,height=752};FBLC/en_US;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P7510;FBSV/4.1.1;]";
const char S60099[] = "com.google.android.youtube/4.1.23(Linux; U; Android 4.1.1; en_US; GT-P7510 Build/JRO03H) gzip";
const char S60100[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7510 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S60101[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-P7510 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60102[] = "FaithlifeStudyBible/2.2.3 (Linux; U; Android 4.1.2; en-US; GT-P7510 Build/cm_p4wifi-userdebug 4.1.2 JZO54K eng.root.X.X test-keys)";
const char S60103[] = "com.google.android.apps.plus/X (Linux; U; Android 4.1.2; en_US; GT-P7510; Build/JZO54K) (gzip)";
const char S60104[] = "Logos/2.2.7 (Linux; U; Android 4.1.2; en-CA; GT-P7510 Build/LIQUID PURE CM10 31.10 RC1)";
const char S60105[] = "FaithlifeStudyBible/2.2.8.1 (Linux; U; Android 4.1.1; en-US; GT-P7510 Build/cm_p4wifi-userdebug 4.1.1 JRO03L eng.matt.X.X release-keys)";
const char S60106[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P7510 Build/JZO05K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60107[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7510 Build/JZO54K) Mint.com/2.6.1";
const char S60108[] = "NokiaC3-00/5.0 (07.20) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac3-00) UCBrowser8.4.0.159/70/352/UCWEB Mobile";
const char S60109[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Micromax A90S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60110[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; A90S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60111[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; A90S Build/IMM76D)";
const char S60112[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; CINK PEAX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60113[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Titanium S5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60114[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Titanium S5 Build/JZO54K)";
const char S60115[] = "Mozilla/5.0 (Linux; Android 4.3; HTC One Build/JWR66V.H10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60116[] = "Mozilla/5.0 (Linux; Android 4.3; One Build/JWR66V.H10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60117[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:18.0) Gecko/X Firefox/18.0 SeaMonkey/2.15.2";
const char S60118[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)";
const char S60119[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)";
const char S60120[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)";
const char S60121[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC_One_XL/5.08.111.2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60122[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; de-de; HTC/One_XL/5.06.162.1 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60123[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; de-ch; HTC/One_XL/5.06.166.1 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60124[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC One XL Build/JDQ39)";
const char S60125[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; HTC One XL Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60126[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-sg; HTC_One_XL Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60127[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; HTC/One_XL/5.08.163.2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60128[] = "Mozilla/Nutch-1.5.1 (Mozilla/5.0 (X11; Linux i686; rv:15.0) Gecko/X Firefox/15.0)";
const char S60129[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S60130[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S60131[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT875 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S60132[] = "Nokia200/2.0 (11.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia200) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S60133[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S60134[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P3110 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S60135[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.1 Chrome/13.0.800.1 Safari/535.1";
const char S60136[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.1 Chrome/13.0.800.1 Safari/535.1";
const char S60137[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60138[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4m";
const char S60139[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3 Sleipnir/2.4.2m";
const char S60140[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.2m";
const char S60141[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3 Sleipnir/2.4.1m";
const char S60142[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.1m";
const char S60143[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.4.2m";
const char S60144[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3 Sleipnir/2.4m";
const char S60145[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3 Sleipnir/2.4.3m";
const char S60146[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.3m";
const char S60147[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.4m";
const char S60148[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3 Sleipnir/2.4.4m";
const char S60149[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.4.4m";
const char S60150[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3 Sleipnir/2.4.5m";
const char S60151[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.5m";
const char S60152[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.4.5m";
const char S60153[] = "Mozilla/5.0 (Linux; Android 4.0.3; PG86100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60154[] = "Mozilla/5.0 (Linux; Android 4.0.3; ISW12HT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60155[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; LG-SU660 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S60156[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; LG-SU660 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S60157[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-SU660 Build/FRG83G)";
const char S60158[] = "Mozilla/5.0 (Linux; Android 4.0.4; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S60159[] = "Mozilla/5.0 (Linux; U; Android 2.0; en-us;) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 (Kobo Touch)";
const char S60160[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; T-02D Build/V15R47B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60161[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; T-02D Build/V17R48D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60162[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; T-02D Build/V18R49C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60163[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; T-02D Build/V19R50B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60164[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; T-02D Build/V19R50B)";
const char S60165[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; T-02D Build/V20R52A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60166[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.1 Mobile Safari/535.19 Silk-Accelerated=false";
const char S60167[] = "Mozilla/5.0 (Linux; Android 4.0.3; PG86100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S60168[] = "Mozilla/5.0 (Linux; Android 4.0.3; ISW12HT Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S60169[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190L Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S60170[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S60171[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S60172[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190L Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S60173[] = "Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60174[] = "Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S60175[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60176[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60177[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S60178[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_3; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S60179[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S60180[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S60181[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S60182[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S60183[] = "Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 8.0; Windows NT 5.0) Chrome/18.0.1229.79";
const char S60184[] = "Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 7.0; Windows NT 5.0) Chrome/18.0.1229.79";
const char S60185[] = "Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 9.0; Windows NT 5.0WOW64) Chrome/18.0.1229.79";
const char S60186[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 UCBrowser/9.0.1.284 Mobile";
const char S60187[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 UCBrowser/9.0.1.284 Mobile";
const char S60188[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 UCBrowser/9.0.1.284 Mobile";
const char S60189[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 UCBrowser/9.0.1.284 Mobile";
const char S60190[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 UCBrowser/9.0.1.284 Mobile";
const char S60191[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC5-03) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S60192[] = "NokiaC5-03/UCWEB8.9.0.253/50/999";
const char S60193[] = "iTunes/10.6.3 (Macintosh; Intel Mac OS X 10.5.8) AppleWebKit/534.50.2";
const char S60194[] = "iTunes/10.6.3 (Macintosh; PPC Mac OS X 10.5.8) AppleWebKit/534.50.2";
const char S60195[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-T989 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60196[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T989 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60197[] = "Mozilla/5.0 (Mobile; Nexus 10; rv:18.1) Gecko/18.1 Firefox/18.1";
const char S60198[] = "BlackBerry8300/4.5.0.182 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102";
const char S60199[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; NGM Legend Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60200[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NGM Legend2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60201[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; novo7Legend Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60202[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NGM LegendXL Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60203[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; Legend Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60204[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; NGM_Legend2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60205[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; NGM_LegendXL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60206[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; novo7Legend Build/IMM76D)";
const char S60207[] = "Mozilla/5.0 (Linux; Android 4.0.4; Hydro Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S60208[] = "Mozilla/5.0 (Linux; Android 4.0.4; C5170 Build/IML77) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S60209[] = "Mozilla/5.0 (Linux; Android 4.0.4; C5170 Build/IML77) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S60210[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP9070 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60211[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6503 Build/10.3.1.A.0.244) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60212[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60213[] = "Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I717 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60214[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; M-MP101S2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60215[] = "SonyEricssonK530i/R8BB Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S60216[] = "SonyEricssonK530c/R8BA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S60217[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT100 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S60218[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9192 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60219[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60220[] = "Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I727 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60221[] = "Opera/9.80 (BREW; Opera Mini/6.0.3/27.2338; U; en) Presto/2.8.119 240X400 Pantech CDM8992";
const char S60222[] = "Mozilla/5.0 (Linux; Android 4.0.4; Event Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60223[] = "SonyEricssonW150i/R2BA006 TelecaBrowser/Q07C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S60224[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.14) Gecko/X Firefox/2.0.0.14 Flock/1.2.2";
const char S60225[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.8.1.14) Gecko/X Firefox/2.0.0.14 Flock/1.2.1";
const char S60226[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGP311 Build/10.1.C.0.370) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S60227[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; AT100 Build/HRI83) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S60228[] = "Nokia2700c-2/2.0 (09.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia2700c-2) UCBrowser8.4.0.159/70/444/UCWEB Mobile";
const char S60229[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P880 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S60230[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P880g Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S60231[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P880 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S60232[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S60233[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S60234[] = "Opera 8.54 SonyEricssonP990i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 513) Opera 8.65 [en]";
const char S60235[] = "Opera 7.0 SonyEricssonP990i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 513) Opera 8.65 [en]";
const char S60236[] = "SonyEricssonP990i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 598) Opera 8.65 [en]";
const char S60237[] = "SonyEricssonP990i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 306) Opera 8.60 [en]";
const char S60238[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; Droid Build/FRG22D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60239[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Droid Build/ESE81) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S60240[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; Droid Build/ESE81) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S60241[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; Droid Build/ESE53) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S60242[] = "Mozilla/5.0 (Linux; U; Android 2.1 (ESE 53); en-us; Droid Build/ESE53) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S60243[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Droid Build/ESE53) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S60244[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; Droid Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S60245[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Vip Droid Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S60246[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Incredimail/1.0.4.3811";
const char S60247[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Incredimail/1.0.4.3811";
const char S60248[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B147 Incredimail/1.0.4.3811";
const char S60249[] = "Incredimail 1.0.4 rv:3811 (iPad; iPhone OS 6.1.2; en_US)";
const char S60250[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Incredimail/1.0.5.4111";
const char S60251[] = "Incredimail 1.0.5 rv:4111 (iPad; iPhone OS 6.1.3; en_US)";
const char S60252[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.11 (KHTML, like Gecko) Ubuntu/12.04 Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.11";
const char S60253[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.11 (KHTML, like Gecko) Ubuntu/12.04 Chromium/17.0.963.79 Chrome/17.0.963.79 Safari/535.11";
const char S60254[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Ubuntu/12.04 (3.4.1-0ubuntu1) Epiphany/3.4.1";
const char S60255[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Ubuntu/12.04 (3.4.1-0ubuntu1) Epiphany/3.4.1";
const char S60256[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Ubuntu/12.04 (3.4.0.1-2ubuntu1) Epiphany/3.4.0.1";
const char S60257[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Ubuntu/12.04 (3.4.0.1-2ubuntu1) Epiphany/3.4.0.1";
const char S60258[] = "N400 Linux/3.0.13 Android/4.0.4 Release/11.23.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S60259[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N400 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60260[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60261[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60262[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us;Sprint ATP515CKIT Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S60263[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60264[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ja-jp; ISW11SC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60265[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ja-; ISW11SC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60266[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; ISW11SC Build/GINGERBREAD)";
const char S60267[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1.2 (like Version/10B141 Mobile/6.1 Safari/8536.25)";
const char S60268[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1.2 (like Version/10B146 Mobile/6.1.2 Safari/8536.25)";
const char S60269[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1.2 (like Version/10B329 Mobile/6.1.3 Safari/8536.25)";
const char S60270[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT920 Build/2_32D_2009) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60271[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:13.0) Gecko/X Firefox/13.0.1 SeaMonkey/2.10.1";
const char S60272[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:13.0) Gecko/X SeaMonkey/2.10.1";
const char S60273[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E425 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S60274[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E425f Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S60275[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E425g Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S60276[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7310 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S60277[] = "Mozilla/5.0 (compatible; Konqueror/4.4; Linux 2.6.32-22-generic; X11; en_US) KHTML/4.4.3 (like Gecko) Kubuntu";
const char S60278[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S60279[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S60280[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293";
const char S60281[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S60282[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/6531.22.7 iLunascape/4";
const char S60283[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S60284[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293";
const char S60285[] = "Mozilla/5.0 (iPhone U CPU iPhone OS 5_0 like Mac OS X en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293 Safari/6531.22.7";
const char S60286[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/9A405 Safari/6531.22.7";
const char S60287[] = "Mozilla/5.0 (iPhone; U; CPU OS 5_0 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0 Mobile/8C148 Safari/528.16";
const char S60288[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Coast/1.1.0.00000 Mobile/11A4449b Safari/7534.48.3";
const char S60289[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Coast/1.1.1.64324 Mobile/11A501 Safari/7534.48.3";
const char S60290[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Coast/1.1.0.64021 Mobile/11A465 Safari/7534.48.3";
const char S60291[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Coast/1.1.0.64021 Mobile/11A4449d Safari/7534.48.3";
const char S60292[] = "JUC (Linux; U; 2.3.6; zh-cn; GT-S5830i; 320*480) UCWEB7.9.0.94/139/444";
const char S60293[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; S500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60294[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; S500 Build/JZO54K)";
const char S60295[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; S300 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0  Safari/533.1";
const char S60296[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; S300 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60297[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; S300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60298[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; KM-S300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60299[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; S300 Build/GRI40)";
const char S60300[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; KM-S300 Build/GINGERBREAD)";
const char S60301[] = "Mozilla/5.0 (Series40; Nokia202/20.28; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S60302[] = "Mozilla/5.0 (Series40; Nokia203/20.52; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S60303[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I727R Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60304[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E110S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60305[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9210 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60306[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A45 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60307[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; A45 Build/GRK39F)";
const char S60308[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T1-D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60309[] = "SonyEricssonW395/R1EA Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1, SonyEricssonW395/R1EA Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S60310[] = "SonyEricssonW395/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1, SonyEricssonW395/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1, SonyEricssonW395/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S60311[] = "SonyEricssonW395/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S60312[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS12SH Build/S7141) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60313[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS12SH Build/S6160) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60314[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS12SH Build/S9081) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60315[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS12SH Build/SB220) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60316[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS12SH Build/S3081) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60317[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS12SH Build/S3081) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.1 Mobile Safari/534.30";
const char S60318[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS12SH Build/S8030) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60319[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; IS12SH Build/S8030)";
const char S60320[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5";
const char S60321[] = "Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 8.0; Windows NT 5.0) Chrome/19.0.1229.79";
const char S60322[] = "Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 7.0; Windows NT 5.0) Chrome/19.0.1229.79";
const char S60323[] = "Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 9.0; Windows NT 5.0WOW64) Chrome/19.0.1229.79";
const char S60324[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST26i Build/11.0.A.7.5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60325[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST26a Build/11.0.A.7.5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60326[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST27i Build/6.2.A.0.400) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60327[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST27a Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60328[] = "NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac2-02) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S60329[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-D700 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60330[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-D700 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60331[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-D700 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/epicmtd";
const char S60332[] = "Mozilla/5.0 (Linux; U; Android 4.3.0; en-us; I9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60333[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-gb; GT-I9300 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60334[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I535 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60335[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I535 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60336[] = "Dalvik/1.6.0 (Linux; U; Android 4.3; GT-I9300 Build/JSS15Q)";
const char S60337[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-T999 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60338[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-T999 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60339[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-gb; GT-I9300 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60340[] = "Dalvik/1.6.0 (Linux; U; Android 4.3; SCH-I535 Build/JSS15Q)";
const char S60341[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P3110 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60342[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P3110 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S60343[] = "Mozilla/5.0 (Series40; Nokia302/14.26; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S60344[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LVW Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60345[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LRA Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60346[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR6410LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60347[] = "Mozilla/5.0 (X11; CrOS i686 2650.0.0) AppleWebKit/537.2 (KHTML, like Gecko) Chrome/22.0.1215.0 Safari/537.2";
const char S60348[] = "Mozilla/5.0 (X11; CrOS i686 2628.0.0) AppleWebKit/537.2 (KHTML, like Gecko) Chrome/22.0.1210.0 Safari/537.2";
const char S60349[] = "Mozilla/5.0 (X11; CrOS x86_64 2628.0.0) AppleWebKit/537.2 (KHTML, like Gecko) Chrome/22.0.1210.0 Safari/537.2";
const char S60350[] = "Mozilla/5.0 (X11; CrOS i686 2694.0.0) AppleWebKit/537.3 (KHTML, like Gecko) Chrome/22.0.1222.0 Safari/537.3";
const char S60351[] = "Mozilla/5.0 (X11; CrOS x86_64 2723.18.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.8 Safari/537.4";
const char S60352[] = "Mozilla/5.0 (X11; CrOS i686 2723.25.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.12 Safari/537.4";
const char S60353[] = "Mozilla/5.0 (X11; CrOS i686 2723.18.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.8 Safari/537.4";
const char S60354[] = "Mozilla/5.0 (X11; CrOS x86_64 2723.0.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.0 Safari/537.4";
const char S60355[] = "Mozilla/5.0 (X11; CrOS i686 2723.0.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.0 Safari/537.4";
const char S60356[] = "Mozilla/5.0 (X11; CrOS x86_64 2723.37.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.21 Safari/537.4";
const char S60357[] = "Mozilla/5.0 (X11; CrOS i686 2723.37.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.21 Safari/537.4";
const char S60358[] = "Mozilla/5.0 (X11; CrOS i686 2723.50.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.26 Safari/537.4";
const char S60359[] = "Mozilla/5.0 (X11; CrOS i686 2723.50.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.26 Safari/537.4";
const char S60360[] = "Mozilla/5.0 (X11; CrOS x86_64 2723.50.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.26 Safari/537.4";
const char S60361[] = "Mozilla/5.0 (X11; CrOS x86_64 2723.50.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.26 Safari/537.4";
const char S60362[] = "Mozilla/5.0 (X11; CrOS i686 2723.52.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.26 Safari/537.4";
const char S60363[] = "Mozilla/5.0 (X11; CrOS x86_64 2723.63.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.33 Safari/537.4";
const char S60364[] = "Mozilla/5.0 (X11; CrOS i686 2723.63.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.33 Safari/537.4";
const char S60365[] = "Mozilla/5.0 (X11; CrOS i686 2556.0.2012) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/22.0.1198.0 Safari/537.1";
const char S60366[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Google Earth/7.0.3.8542 Safari/533.3";
const char S60367[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60368[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60369[] = "Mozilla/5.0 (Series40; Nokia300/07.03; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.9";
const char S60370[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Xoom Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60371[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Xoom Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60372[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Xoom Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60373[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Xoom Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60374[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Xoom Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60375[] = "SAMSUNG-GT-E2230/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S60376[] = "SAMSUNG-GT-E2230/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S60377[] = "SAMSUNG-GT-E2230L/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S60378[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MOMO11Speed Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60379[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; MOMO11Speed Build/JRO03H)";
const char S60380[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CUBE U9GT 2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60381[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; U9GT2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60382[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; CUBE U9GT 2 Build/IML74K)";
const char S60383[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; CUBE_U9GT_2 Build/IML74K; 768*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S60384[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60385[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);nl-NL;kml:2.2;client:Free;type:default)";
const char S60386[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);de;kml:2.2;client:Free;type:default)";
const char S60387[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);en-US;kml:2.2;client:Free;type:default)";
const char S60388[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);en;kml:2.2;client:Free;type:default)";
const char S60389[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6001.1);en-US;kml:2.2;client:Free;type:default)";
const char S60390[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);en-GB;kml:2.2;client:Free;type:default)";
const char S60391[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);de-DE;kml:2.2;client:Free;type:default)";
const char S60392[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);it-IT;kml:2.2;client:Free;type:default)";
const char S60393[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6001.1);en;kml:2.2;client:Free;type:default)";
const char S60394[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);fr-FR;kml:2.2;client:Free;type:default)";
const char S60395[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);en-CA;kml:2.2;client:Free;type:default)";
const char S60396[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);nl;kml:2.2;client:Free;type:default)";
const char S60397[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);fr;kml:2.2;client:Free;type:default)";
const char S60398[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);en-AU;kml:2.2;client:Free;type:default)";
const char S60399[] = "GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);pt-BR;kml:2.2;client:Free;type:default)";
const char S60400[] = "Mozilla/5.0 (Series40; Nokia306/03.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S60401[] = "iTunes/10.7 (Windows; (Build 9200)) AppleWebKit/536.26.9";
const char S60402[] = "iTunes/10.7 (Windows; (Build 8400)) AppleWebKit/536.26.9";
const char S60403[] = "iTunes/10.7 (Windows; (Build 8250)) AppleWebKit/536.26.9";
const char S60404[] = "iTunes/11.0.2 (Windows; Microsoft Windows 8 x64 Business Edition (Build 9200)) AppleWebKit/536.27.1";
const char S60405[] = "iTunes/11.0.2 (Windows; Microsoft Windows 8 x64 Home Premium Edition (Build 9200)) AppleWebKit/536.27.1";
const char S60406[] = "iTunes/11.0.2 (Windows; Microsoft Windows 8 Business Edition (Build 9200)) AppleWebKit/536.27.1";
const char S60407[] = "MOT-EX225 MIDP-2.0/CLDC-1.1 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10";
const char S60408[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LG-P990 Build/GRJ22; CyanogenMod-9.9.9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60409[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P990 Build/GRJ22; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60410[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P990 Build/IMM76L; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60411[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC_Amaze_4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60412[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Amaze 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60413[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Ruby Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60414[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E460 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60415[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E460f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60416[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; Lenovo A800; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60417[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lenovo A800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60418[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo A800 Build/IMM76D)";
const char S60419[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lenovo A800 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60420[] = "Samsung-SPHM540BST Polaris/6.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S60421[] = "Samsung-SPHM540 Polaris/6.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S60422[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; HTC Magic Build/DRC92) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S60423[] = "Mozilla/5.0 (Linux; U; Android 1.6; zh-cn; HTC Magic Build/DRC83) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S60424[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; HTC Magic Build/DRD35) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S60425[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; M532 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60426[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; M532 Build/JRO03C)";
const char S60427[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T989 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S60428[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S60429[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T989 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S60430[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S60431[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989D Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S60432[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6400L Build/FRG83D; ThunderShed-v1.5_CM7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60433[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6400L Build/FRG83D; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60434[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6400L Build/GRJ22; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60435[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6400L Build/GRJ22; ThunderShed-v1.6_CM7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60436[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; IM-A730S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60437[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; IM-A730S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60438[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; IM-A730S Build/GINGERBREAD)";
const char S60439[] = "Mozilla/5.0 (Linux; U; Android 2.3.7 (link_ke); vi-vn; IM-A730S Build/Vegaviet.com) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60440[] = "Mozilla/5.0 (Linux; U; Android 2.3.7 (link_ke); en-; IM-A730S Build/Vegaviet.com) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60441[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT907 Build/7.7.1Q-144_SMQ_S136-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60442[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; XT907 Build/7.7.1Q-144_SMQ_S136-10)";
const char S60443[] = "stagefright/1.2 (Linux;Android 4.0.4;motorola XT907 Build/7.7.1Q-144_SMQ_S136-10)";
const char S60444[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; XT905 Build/7.7.1Q_GCIRD-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60445[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; XT905 Build/7.7.1Q_GCIRD-16)";
const char S60446[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S7562 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S60447[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-S7562i Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S60448[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E973 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60449[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F200K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60450[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F200L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60451[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F200S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60452[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SBM203SH Build/S0012) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60453[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SBM203SH Build/S0018) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60454[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SBM203SH Build/S0018)";
const char S60455[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SBM203SH Build/S0022) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60456[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SBM203SH Build/S0024) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60457[] = "Mozilla/5.0 (Series40; Nokia201/11.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S60458[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MAXWEST Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60459[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MAXWEST Build/MAXWEST-V03.00.00_2013.01.05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60460[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MAXWEST Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60461[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MAXWEST Build/MAXWEST-V03.00.01_2013.03.15) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60462[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; A2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60463[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; Cross A2 Build/ICS_IMM76D_B156) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60464[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Karbonn A2+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60465[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; A2 Build/IMM76I)";
const char S60466[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-gb; LG-V900 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S60467[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; de-de; LG-V900/v10d Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S60468[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; tr-tr; LG-V901 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S60469[] = "Dalvik/1.5.0 (Linux; U; Android 3.0.1; LG-V900 Build/HRI66)";
const char S60470[] = "Opera/9.80 (Windows 98; U; en) Presto/2.6.30 Version/10.63";
const char S60471[] = "Opera/9.80 (Windows 98; U; en) Presto/2.6.30 Version/10.62";
const char S60472[] = "Opera/9.80 (Windows 98; U; de) Presto/2.6.30 Version/10.61";
const char S60473[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S60474[] = "Mozilla/5.0 (Series40; Nokia310/07.35; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S60475[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; HTC PG09410 Build/HMJ15) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S60476[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us;HTC_PG09410/1.17.502.3 Build/HMJ15) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S60477[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us;HTC_PG09410/1.30.502.1 Build/HMJ15) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S60478[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-ca; HTC-P715a Build/HMJ15) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S60479[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-ca;HTC-P715a Build/HMJ15) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S60480[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; HTC-P715a Build/HMJ15)";
const char S60481[] = "Android/4.6.1 (Linux; U; Android 3.1; en_CA; HTC-P715a)";
const char S60482[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; HTC PG09410 Build/HMJ15)";
const char S60483[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One XL Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S60484[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One XL Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S60485[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:20.0) Gecko/X Firefox/20.0 SeaMonkey/2.17.1";
const char S60486[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:20.0) Gecko/X SeaMonkey/2.17.1";
const char S60487[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E610 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60488[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60489[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E610v Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60490[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60491[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612f Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60492[] = "Mozilla/5.0 (Linux; Android 4.1.2; Torque Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S60493[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Alcatel OT-980 Build/DR4) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S60494[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Alcatel OT-980 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S60495[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Alcatel OT-980A Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S60496[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:11.0) Gecko/X Firefox/11.0 AlexaToolbar/alxf-2.15";
const char S60497[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:11.0) Gecko/X Firefox/11.0 Firefox/11.0";
const char S60498[] = "Mozilla/5.0 (Series40; Nokia310/07.35; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S60499[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S60500[] = "Mozilla/5.0 (X11; U; OpenBSD i386; en-US; rv:1.8.0.5) Gecko/X Firefox/1.5.0.5";
const char S60501[] = "Mozilla/5.0 (Windows NT 6.0; rv:15.4) Gecko/X Firefox/15.4.1 PaleMoon/15.4.1";
const char S60502[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:15.4) Gecko/X Firefox/15.4.1-x64 PaleMoon/15.4.1-x64";
const char S60503[] = "Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320) BlackBerry8830/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105";
const char S60504[] = "BlackBerry8830/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105";
const char S60505[] = "Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320) BlackBerry8830/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/104";
const char S60506[] = "PantechP6010/JNUS06062011D BMP/1.0.2 DeviceId/141020 NetFront/4.1 OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S60507[] = "PantechP6010/JNUSX BMP/1.0.2 DeviceId/X NetFront/4.1 OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S60508[] = "PantechP9020/JEUS04262010; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.18216; U; en-US) Opera 9.50";
const char S60509[] = "PantechP9020/JEUSX; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.18216; U; es) Opera 9.50";
const char S60510[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; Edition MAS; en) Presto/2.10.229 Version/11.67";
const char S60511[] = "Mozilla/4.0(compatible;Polaris 6.2;Brew 3.1.5;U;en)/400x240 Samsung SCH-U820";
const char S60512[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Ubuntu/12.04 Chromium/23.0.1271.64 Chrome/23.0.1271.64 Safari/537.11";
const char S60513[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.11 (KHTML, like Gecko) Ubuntu/12.04 Chromium/23.0.1271.64 Chrome/23.0.1271.64 Safari/537.11";
const char S60514[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; OP0118-12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60515[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en_us-; OP0118-12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60516[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5) Presto/2.12.388 Version/12.13";
const char S60517[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ALCATEL ONE TOUCH 5035A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60518[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; ALCATEL ONE TOUCH 5035D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60519[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; ALCATEL ONE TOUCH 5035X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60520[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; ND-8014RK Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60521[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S5280 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60522[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S5280 Build/JZO54K)";
const char S60523[] = "Mozilla/5.0 (Linux;U;Android 4.0.4; en-us; Nuqleo; Quantum 7 Build/ICS) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S60524[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nuqleo; Quantum 7 Build/ICS) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S60525[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Quantum 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60526[] = "Nokia6500c_CMCC/2.0 (08.20) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S60527[] = "Nokia6500c/2.0 (06.61) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S60528[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HUAWEI U8661 Build/HuaweiU8661) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60529[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI U8661 Build/HuaweiU8661) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S60530[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; HUAWEI U8661 Build/HuaweiU8661)";
const char S60531[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT300 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S60532[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT300SE Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S60533[] = "HUAWEI_G520-5000_TD/1.0 Android/4.1.2 (Linux; U; Android 4.1.2; zh-cn) Release/01.31.2013 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30";
const char S60534[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HUAWEI G520-0000 Build/HuaweiG520-0000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60535[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI G520-5000 Build/HuaweiG520-5000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60536[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HUAWEI G520-0000 Build/HuaweiG520-0000)";
const char S60537[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HUAWEI G520-5000 Build/HuaweiG520-5000)";
const char S60538[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI G520-0000 Build/HuaweiG520-0000) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S60539[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI G520-5000 Build/HuaweiG520-5000) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S60540[] = "NokiaC3-00/5.0 (08.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac3-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S60541[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; LG-P990 Build/IMM76L; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60542[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-P990 Build/IMM76L; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60543[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; MWP6885)";
const char S60544[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; L-05D Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60545[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; L-05D Build/IMM76I)";
const char S60546[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 2_1_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5F138 Safari/525.20";
const char S60547[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5F137 Safari/525.20";
const char S60548[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Htcclay's SuperCM7 G2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60549[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Inspire 4g Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60550[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Liquid Mt Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60551[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Precedent Build/GINGERBREAD; CM7.1.0-PrecedentR1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60552[] = "Mozilla/5.0 (Linux; U; Android 2.3.7 Beta 5 modded by cas_xp; en-us; TC970 Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60553[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Htcclay's SuperCM7 NS2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60554[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; Folio 100 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60555[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Crescent Build/GRJ22; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60556[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; u8510 Build/ICS CM7 - teammat3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60557[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; M865 Build/GINGERBREAD; CM7.2-M865) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60558[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; Swift Me Build/GRJ90; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60559[] = "Mozilla/5.0 (Linux; U; Android 2.3.7 Beta 4 modded by cas_xp; en-us; TC970 Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60560[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; TC970 Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60561[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; a107 Build/GINGERBREAD; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60562[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; tr-tr; Perfect Rom v4 Build/FRG83; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60563[] = "Mozilla/5.0 (Linux; U; Android 2.3.7 Xperia Edition v.2; tr-tr; Xperia 945 Build/FRG83; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60564[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; IdeaPad A1 Build/GRK393; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60565[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; Geeksphone ZERO Build/FRG83; ZerO_CR 3.0 CyanogenMod) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60566[] = "Mozilla/5.0 (Linux; U; Android 2.3.7+; es-es; Nexus One+ Build/GRK39F; CyanogenMod-7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60567[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; LG-SU660 Build/GRJ22; CyanogenMod-7; LG-P990 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60568[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; m865 Build/Ultimate CM7 V2; Ultiamte CM7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60569[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; SBM104SH Build/S0024) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60570[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; SBM104SH Build/S0026) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60571[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; SBM104SH Build/S0018) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60572[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; SBM104SH Build/S0028) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60573[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; SBM104SH Build/S0030) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60574[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SBM104SH Build/S0030)";
const char S60575[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; SBM104SH Build/S0032) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60576[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; SBM104SH Build/S0034) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60577[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; SBM104SH Build/S0038) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60578[] = "Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; SonyEricssonSO-01B Build/R1EA018) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 (compatible; ichiro/mobile goo;+http://search.goo.ne.jp/option/use/sub4/sub4-1/)";
const char S60579[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60580[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; KFJWA Build/IMM76D)";
const char S60581[] = "Reeder/1020.09.00 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (iMac12%2C2)";
const char S60582[] = "Reeder/1020.09.00 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro10%2C1)";
const char S60583[] = "Reeder/1020.09.00 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookAir5%2C2)";
const char S60584[] = "Reeder/1020.09.00 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBook5%2C1)";
const char S60585[] = "Reeder/1020.09.00 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacPro3%2C1)";
const char S60586[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-tw; HTC_Butterfly_S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60587[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC Butterfly s Build/JDQ39)";
const char S60588[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-tw; HTC Butterfly s Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60589[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-sg; HTC_Butterfly_s Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60590[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; GT-I9000 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60591[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-I9000 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60592[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT-I9000 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60593[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-I9000 Build/MIUIAR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60594[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; GT-I9000 Build/MIUI)";
const char S60595[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-I9000 Build/MIUI)";
const char S60596[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-I9000 Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S60597[] = "Mozilla/5.0 (Linux; U; Android 2.3.9; en-us; GT-I9000 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60598[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-I9000 Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S60599[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-P920 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60600[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P920 Build/ICS)";
const char S60601[] = "Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S60602[] = "Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S60603[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; Archos5 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S60604[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S60605[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S60606[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-I747M Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S60607[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530U Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S60608[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-I9305T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S60609[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9305 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S60610[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I747M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S60611[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S60612[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I547 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60613[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I547C Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60614[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I547 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60615[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AN8G3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60616[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S60617[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S60618[] = "Nokia3720c/2.0 (10.16) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S60619[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lenovo K860 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60620[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo K860 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S60621[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo K860 Build/IMM76D)";
const char S60622[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo K860i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60623[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo K860 Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S60624[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae72-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba";
const char S60625[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE72-1) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S60626[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE72-2) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S60627[] = "NokiaE72-1/UCWEB8.9.0.253/28/999";
const char S60628[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Twitter for iPad";
const char S60629[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C134b Twitter for iPad";
const char S60630[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C134 Twitter for iPad";
const char S60631[] = "Mozilla/5.0 (Windows NT 5.0; rv:22.0) Gecko/X Firefox/22.0";
const char S60632[] = "Nokia3600slide/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S60633[] = "Nokia3600s/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S60634[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-09D Build/S6130) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60635[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-09D Build/S7090) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60636[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-09D Build/S9070) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60637[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-09D Build/SB220) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60638[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-09D Build/S4060) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60639[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SH-09D Build/S9070)";
const char S60640[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5360) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S60641[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; SCH-i509) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S60642[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5360B) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S60643[] = "Mozilla/5.0 (Linux; Android 4.0.3; PG86100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60644[] = "Mozilla/5.0 (Linux; Android 4.0.3; ISW12HT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60645[] = "Mozilla/5.0 (Linux; Android 4.0.4; PG86100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60646[] = "TBDC1093 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S60647[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus S Build/JRO03E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60648[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60649[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60650[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315b Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60651[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; fusion5 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60652[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; fusion 5 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60653[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; fusion5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60654[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; fusion5 Build/FR-MX68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60655[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; FUSION5 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60656[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ALCATEL ONE TOUCH 6030X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S60657[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; ALCATEL_ONE_TOUCH_6030X_Orange Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S60658[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60659[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315b Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60660[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HP Touchpad Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60661[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Touchpad Build/GRJ22; XRONified-Odexed) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60662[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; Touchpad Build/GRJ22)";
const char S60663[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; HP Touchpad Build/GRJ22)";
const char S60664[] = "Mozilla/5.0 (Series40; Nokia5130c-2/07.95; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S60665[] = "Mozilla/5.0 (Series40; Nokia5130/07.97; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S60666[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-10D Build/S8130) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60667[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-10D Build/S9130) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60668[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-10D Build/SB010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60669[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-10D Build/S3070) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60670[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-10D Build/S4040) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60671[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-10D Build/S4180) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60672[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaN8-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S60673[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.67; en-us; 8900/5.2.0.67) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S60674[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.104; en-us; 8900/5.2.0.104) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S60675[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.31; en-us; 8900/5.2.0.31) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S60676[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.96; en-us; 8900/5.2.0.96) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S60677[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.102; en-us; 8900/4.2.0.102) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S60678[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.76; en-us; 8900/5.2.0.76) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S60679[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.127; en-us; 8900/4.2.0.127) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S60680[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.58; en-us; 8900/5.2.0.58) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S60681[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.94; en-us; 8900/4.2.0.94) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S60682[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.47; en-us; 8900/5.2.0.47) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S60683[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.113; en-us; 8900/4.2.0.113) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S60684[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.111; en-us; 8900/5.2.0.111) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S60685[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.108; en-us; 8900/4.2.0.108) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S60686[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.102; en-us; 8900/5.2.0.102) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S60687[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919N Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60688[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11";
const char S60689[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11";
const char S60690[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11";
const char S60691[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11";
const char S60692[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11";
const char S60693[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11";
const char S60694[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HTC_J_Z321e Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60695[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HTC J Z321e Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60696[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC J Z321e Build/JRO03C)";
const char S60697[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9981; en-GB) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.579 Mobile Safari/534.11+";
const char S60698[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; LG-F100S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60699[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-F100S Build/GRK39F)";
const char S60700[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60701[] = "Mozilla/5.0 (PLAYSTATION 3 4.11) AppleWebKit/531.22.8 (KHTML, like Gecko)";
const char S60702[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3110 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S60703[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P3110 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S60704[] = "Opera/9.80 (Android; Opera Mini/7.8.33965/31.1386; U; zh) Presto/2.8.119 Version/11.10";
const char S60705[] = "Mozilla/5.0 (Linux; Android 4.0.4;  Nexus S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S60706[] = "Mozilla/5.0 (Linux; Android 4.0.3;  Nexus S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S60707[] = "Mozilla/5.0 (Linux; Android 4.0.4; Nexus S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S60708[] = "Mozilla/5.0 (Linux; Android 4.0.3; Nexus S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S60709[] = "Mozilla/5.0 (Linux; Android 4.0.4; Crespo Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S60710[] = "Mozilla/5.0 (Linux; Android 4.0.1; Nexus S Build/GWK74) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S60711[] = "Mozilla/5.0 (Linux; Android 4.0.3; Full Android on Crespo Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S60712[] = "Mozilla/5.0 (Windows NT 4.0; rv:11.0) Gecko/X Firefox/11.0";
const char S60713[] = "Mozilla/5.0 (X11; SunOS i86pc; rv:15.0) Gecko/X Firefox/15.0.1";
const char S60714[] = "Mozilla/5.0 (Linux; Android 4.1.1; A210 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S60715[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S60716[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S60717[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S60718[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S60719[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S60720[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S60721[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S60722[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S60723[] = "SAMSUNG-GT-C3303/C3303DDJG4 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S60724[] = "SAMSUNG-GT-C3303i/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S60725[] = "SAMSUNG-GT-C3303K/C3303KJPKA1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S60726[] = "Mozilla/5.0 (Linux; U; Android 2.1-update2; tr-tr; U8100 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S60727[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; Huawei_8100-9 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S60728[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; U8100 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S60729[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; U8100 Build/ERE27)";
const char S60730[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-I9000 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60731[] = "Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia305) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S60732[] = "Mozilla/5.0 (compatible; Teleca Q7; Brew 3.1.5; U; en) 480X800 LGE VX11000";
const char S60733[] = "Mozilla/5.0(compatible; Teleca Q7; Brew 3.1.5; U; en) 480X800 LGE VX11000";
const char S60734[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.2.15 Version/10.20";
const char S60735[] = "Opera/10.2 (Windows NT 6.1; WOW64; U; en)";
const char S60736[] = "SonyEricssonK810a/R1KC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S60737[] = "SonyEricssonK810i/R8BA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S60738[] = "SonyEricssonK810c/R1KC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S60739[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM006SH Build/S0012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60740[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM006SH Build/S0018) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60741[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM006SH Build/S0026) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60742[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM006SH Build/S0038) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60743[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM006SH Build/S0042) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60744[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SBM006SH Build/S0042)";
const char S60745[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM006SH Build/S0044) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60746[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM006SH Build/S0064) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60747[] = "Mozilla/5.0 (Android; Tablet; rv:14.0) Gecko/14.0 Firefox/14.0a2";
const char S60748[] = "Mozilla/5.0 (Android; Tablet; rv:14.0) Gecko/14.0 Firefox/14.0";
const char S60749[] = "Mozilla/5.0 (Android; Tablet; rv:14.0) Gecko/14.0 Firefox/14.0 Fennec/14.0";
const char S60750[] = "Mozilla/5.0 (Android; Tablet; rv:14.0) Gecko/14.0 Firefox/14.0.1";
const char S60751[] = "Mozilla/5.0 (Android; Tablet; rv:14.0) Gecko/14.0 Fennec/14.0";
const char S60752[] = "Mozilla/5.0 (Android; Tablet; rv:14.0) Gecko/14.0 Firefox/14.0a1";
const char S60753[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Windows Phone 8X by HTC)";
const char S60754[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; PM23300)";
const char S60755[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Windows Phone 8X by HTC; 2.00.161.04)";
const char S60756[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Windows Phone 8X by HTC; Orange)";
const char S60757[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Windows Phone 8X by HTC; 2.01.161.00)";
const char S60758[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Windows Phone 8X by HTC; 3.41.161.02)";
const char S60759[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; PMP5570C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60760[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; KYL21 Build/015.3.3510) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60761[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; KYL21 Build/017.0.3a30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60762[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; KYL21 Build/018.0.3a90) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60763[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; KYL21 Build/019.0.3b00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60764[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; KYL21 Build/019.0.3b00)";
const char S60765[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Micromax A72 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60766[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; Micromax A72 Build/GRK39F)";
const char S60767[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P895 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S60768[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P895qb Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S60769[] = "Mozilla/5.0 (Linux; Android 4.1.2; VS930 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60770[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S60771[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8005 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S60772[] = "PantechP2000/JGUS03292010; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.16479; U; en-US) Opera 9.50";
const char S60773[] = "PantechP2000/JGUSX; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.16479; U; es) Opera 9.50";
const char S60774[] = "Nokia2700c-2/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia2700c-2) UCBrowser8.2.0.132/69/452/UCWEB Mobile";
const char S60775[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One S Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60776[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One S Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60777[] = "Nokia6085/2.0 (05.03) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6085";
const char S60778[] = "Nokia6085/2.0 (06.00) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S60779[] = "Nokia5233/UCWEB7.9.0.94/50/999";
const char S60780[] = "Nokia5233/UC Browser7.9.0.102/50/355/UCWEB";
const char S60781[] = "SAMSUNG-GT-S3572 Opera/9.80 (J2ME/MIDP; Opera Mini/6.5.30242/30.3593; U; en) Presto/2.8.119 Version/11.10";
const char S60782[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60783[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-style Q2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60784[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i-mobile i-STYLE Q2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60785[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-STYLE Q2 DUO Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60786[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; i-mobile i-STYLE Q2 DUO Build/IMM76D)";
const char S60787[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515m Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60788[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60789[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC_X515E Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60790[] = "Mozilla/5.0 (Linux; Android 4.0.4; SMP45-210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S60791[] = "Mozilla/5.0 (Linux; Android 4.0.4; N8000_USA_RS Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60792[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAP5430 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60793[] = "Mozilla/5.0 (Linux; Android 4.0.4; 9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60794[] = "Mozilla/5.0 (Linux; Android 4.0.4; tmn smart a18 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60795[] = "Mozilla/5.0 (Linux; Android 4.0.4; BASE_Lutea_3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60796[] = "Mozilla/5.0 (Linux; Android 4.0.4; A11Note Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60797[] = "Mozilla/5.0 (Linux; Android 4.0.4; Pad Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60798[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAD1001 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S60799[] = "Mozilla/5.0 (Linux; Android 4.0.4; V8200plus Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60800[] = "Mozilla/5.0 (Linux; Android 4.0.4; SMP53-210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60801[] = "Mozilla/5.0 (Linux; Android 4.0.4; INFINITY_POWER Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60802[] = "Mozilla/5.0 (Linux; Android 4.0.4; BT210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60803[] = "Mozilla/5.0 (Linux; Android 4.0.4; M980 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S60804[] = "Mozilla/5.0 (Linux; Android 4.0.4; 7102 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60805[] = "Mozilla/5.0 (Linux; Android 4.0.4; N900 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60806[] = "Mozilla/5.0 (Linux; Android 4.0.4; cyclone Titan Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S60807[] = "Mozilla/5.0 (Linux; Android 4.0.3; Clear Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S60808[] = "Mozilla/5.0 (Linux; Android 4.0.4; TR101 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S60809[] = "Mozilla/5.0 (Linux; Android 4.0.4; TR71 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S60810[] = "Mozilla/5.0 (Linux; Android 4.0.4; A13 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60811[] = "Mozilla/5.0 (Linux; Android 4.0.4; Next7D12 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S60812[] = "Mozilla/5.0 (Linux; Android 4.0.4; GLORY Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60813[] = "Mozilla/5.0 (Linux; Android 4.0.4; 5I Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60814[] = "Mozilla/5.0 (Linux; Android 4.0.4; TPC7102 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S60815[] = "Mozilla/5.0 (PLAYSTATION 3 4.50) AppleWebKit/531.22.8 (KHTML, like Gecko)";
const char S60816[] = "Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Opera 8.65; Nokia N70/5.0638.3.0.1; 9414) Opera 8.65 [en]";
const char S60817[] = "Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Nokia N70/5.0741.4.0.1; 9404) Opera 8.65 [en]";
const char S60818[] = "Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Nokia N70/5.0737.3.0.1; 1657) Opera 8.60 [en]";
const char S60819[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9300 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S60820[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-I9300 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S60821[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SCH-I535 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S60822[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SGH-T999 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S60823[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-I939 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S60824[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-I939D Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S60825[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U9510E Build/HuaweiU9510E) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S60826[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U9510 Build/HuaweiU9510) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S60827[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I497 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S60828[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LG-P999 Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60829[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60830[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60831[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60832[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LG-P999 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60833[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60834[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P999 Build/GRI40)";
const char S60835[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-P999 Build/GRI40)";
const char S60836[] = "IUC(U;iOS 6.0.1;Zh-cn;320*480;)/UCWEB8.8.0.212/44/997";
const char S60837[] = "IUC(U;iOS 6.0.1;Zh-cn;320*568;)/UCWEB8.8.0.212/44/997";
const char S60838[] = "Mozilla/4.0 (compatible; MSIE 5.0; SunOS 5.9 sun4u; X11)";
const char S60839[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC Desire Build/JRO03C; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60840[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC Desire Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60841[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC Desire Build/CM10; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60842[] = "Mozilla/5.0 (compatible; MSIE 1.0; 68K; Tablet PC; Trident/2.0)";
const char S60843[] = "Mozilla/5.0 (compatible; MSIE 1.0; PPC; Tablet PC; Trident/5.0)";
const char S60844[] = "Mozilla/5.0 (compatible; MSIE 1.0; WindowsCE; Trident/5.0)";
const char S60845[] = "Mozilla/5.0 (compatible; MSIE 2.0; 68K; Tablet PC; Trident/3.0)";
const char S60846[] = "Mozilla/5.0 (compatible; MSIE 2.0; WindowsCE; Trident/4.0)";
const char S60847[] = "Mozilla/5.0 (compatible; MSIE 2.0; PPC; Tablet PC; Trident/4.0)";
const char S60848[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21a Build/11.0.A.0.16) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S60849[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21i2 Build/11.0.A.6.5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S60850[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21i Build/11.0.A.4.22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S60851[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21a2 Build/11.0.A.1.8) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S60852[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30at Build/9.1.F.1.120) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S60853[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30at Build/9.1.F.1.120) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S60854[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko; Google Page Speed Insights) Chrome/22.0.1229 Safari/537.4 GoogleBot/2.1";
const char S60855[] = "Mozilla/5.0 (Linux; Android 4.1.2; USCC-C6721 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60856[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; Micromax A75 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60857[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; A75 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60858[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; A75 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S60859[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; A75 Build/GRJ22)";
const char S60860[] = "Opera/9.80 (Windows NT 6.2; Win64; x64; Edition Next) Presto/2.12.378 Version/12.50";
const char S60861[] = "NokiaX2-01/5.0 (08.71) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-01) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S60862[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-ve; CM980 Build/HuaweiCM980) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60863[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; CM980 Build/HuaweiCM980)";
const char S60864[] = "Mozilla/5.0 (Linux; U; Android 2.3.5 Plus; es-ve; CM980 Build/HuaweiCM980) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60865[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26ii Build/6.1.A.2.45) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S60866[] = "HUAWEI_T8950_TD/1.0 Android/4.0.4 (Linux; U; Android 4.0.4; zh-cn) Release/05.31.2012 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30";
const char S60867[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8950D Build/HuaweiC8950D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S60868[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8950D Build/HuaweiC8950D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60869[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8950D/C8950DV100R001C92B944SP01; 540*960; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S60870[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8950 Build/HuaweiT8950) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S60871[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8950D/C8950DV100R001C92B941SP02; 540*960; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S60872[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI C8950D Build/HuaweiC8950D)";
const char S60873[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8950D/C8950DV100R001C92B948; 540*960; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S60874[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8950 Build/HuaweiT8950) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60875[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8950D/C8950DV100R001C92B955; 540*960; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S60876[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8950D/C8950DV100R001C92B941; 540*960; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S60877[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI T8950 Build/HuaweiT8950)";
const char S60878[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8010 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S60879[] = "Mozilla/5.0 (Series40; Nokia114/03.33; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S60880[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S60881[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S60882[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X dual sim Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S60883[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire XC dual sim Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S60884[] = "Mozilla/5.0 (Windows NT 5.1; rv:15.0) Gecko/X Thunderbird/15.0.1";
const char S60885[] = "Mozilla/5.0 (Windows NT 5.2; rv:15.0) Gecko/X Thunderbird/15.0.1";
const char S60886[] = "TECNO Q1 Linux/3.0.13 Android/4.0.4 Release/04.13.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S60887[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TECNO Q1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60888[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; TECNO Q1 Build/IMM76D)";
const char S60889[] = "Nokia311/5.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia311) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S60890[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT29i Build/9.1.B.1.67) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60891[] = "Mozilla/5.0 (Linux; Android 4.1.2; A110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S60892[] = "Mozilla/5.0 (Linux; Android 4.1.1; Micromax A110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S60893[] = "Mozilla/5.0 (Linux; Android 4.1.1; A110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S60894[] = "Mozilla/4.0 (compatible; Polaris 6.2; BREW 3.1.5; U; en)/240X400 LGE VX8575";
const char S60895[] = "Opera/9.80 (Windows NT 6.2; U; en) Presto/2.10.229 Version/11.60";
const char S60896[] = "Mozilla/5.0 (Series40; NokiaC2-05/08.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15";
const char S60897[] = "Mozilla/5.0 (Series40; NokiaC2-05.1/08.60b; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15";
const char S60898[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S60899[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S60900[] = "Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Safari/535.19 Silk-Accelerated=true";
const char S60901[] = "Mozilla/5.0 (Linux; Android 4.0.3; MZ604 Build/I.7.1-34) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S60902[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ604 Build/I.7.1-45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S60903[] = "SAMSUNG-SGH-I637/UCID4 Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S60904[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST27i Build/6.1.1.B.1.10) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S60905[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST27a Build/6.1.1.B.1.54) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S60906[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11";
const char S60907[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11";
const char S60908[] = "I9300 Linux/2.6.35 Android/4.0.4 Release/10.12.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1";
const char S60909[] = "samsung-GT-I9300/1.0 Linux/2.6.35.7 Android/4.0.4 Release/03.25.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S60910[] = "samsung-I9300/1.0 Linux/2.6.35.7 Android/4.0.4 Release/12.25.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S60911[] = "I9300-I9300/1.0 Linux/2.6.35.7 Android/4.0.3 Release/02.28.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S60912[] = "I9300-I9300/1.0 Linux/2.6.35.7 Android/4.0.3 Release/01.21.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S60913[] = "sprd-I9300/1.0 Linux/2.6.35.7 Android/4.0 Release/04.11.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S60914[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S60915[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9205 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60916[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S60917[] = "NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-02) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S60918[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1;	Trident/4.0; SLCC2;	.NET CLR 2.0.50727; .NET CLR 3.5.30729;	.NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E) QQBrowser/6.0";
const char S60919[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) QQBrowser/6.0.8405.201";
const char S60920[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.0";
const char S60921[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.0.8405.201";
const char S60922[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.16; QQBrowser/6.0.8082.201 (webkit)";
const char S60923[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1) QQBrowser/6.0";
const char S60924[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2) QQBrowser/6.0";
const char S60925[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; LIFETAB_P9514 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S60926[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; LIFETAB_P9514 Build/HTJ85B)";
const char S60927[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ALCATEL ONE TOUCH 6033X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S60928[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-T328d/1.23.1401.1) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60929[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328d Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S60930[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328d Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S60931[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-T328d/1.48.1401.5) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60932[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-T328d/1.40.1401.2) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60933[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-T328d/1.55.1401.2) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60934[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328d) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60935[] = "Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328d Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S60936[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-T328d/1.55.1401.4) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60937[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-T328d/1.67.1401.3) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60938[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC T328d Build/IML74K)";
const char S60939[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-T328d/1.67.1401.6) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60940[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; DATAM819HD_A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60941[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; DATAM819HD_B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60942[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; DATAM819HD_C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60943[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BC1003 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60944[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; BC1003 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60945[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; BC1003 Build/IMM76D)";
const char S60946[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I897 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60947[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I897 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60948[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I897 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60949[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I897 Build/JRO03U) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60950[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I897 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60951[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I897 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60952[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I897 Build/JZO54K)";
const char S60953[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I897 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60954[] = "Mozilla/5.0 (Windows NT 6.0; rv:15.0) Gecko/X Firefox/15.2 PaleMoon/15.2";
const char S60955[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:15.0) Gecko/X Firefox/15.2-x64 PaleMoon/15.2-x64";
const char S60956[] = "LG-GT365/V11c Teleca/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S60957[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; ZTE T28 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60958[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; ZTE T28 Prepaid Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S60959[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; P-06D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60960[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; P-06D Build/IMM76D)";
const char S60961[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ISW13F Build/V51R37G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60962[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ISW13F Build/V53R39C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60963[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ISW13F Build/V55R41F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60964[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ISW13F Build/V61R47A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60965[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ISW13F Build/V69R51I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60966[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ISW13F Build/V67R49E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60967[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ISW13F Build/V71R54A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60968[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ISW13F Build/V75R58A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60969[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG-GT-i8000V/BUIJ1";
const char S60970[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG-GT-i8000V/NXXJD6";
const char S60971[] = "SAMSUNG-GT-i8000H/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S60972[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG-GT-i8000V/BUII1";
const char S60973[] = "SAMSUNG-GT-i8000T Mozilla (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S60974[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG-GT-i8000V/BUJA3";
const char S60975[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG-GT-i8000V/NXXIL5";
const char S60976[] = "SAMSUNG-GT-i8000Orange/BVJB3 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S60977[] = "SAMSUNG-GT-i8000/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S60978[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG-GT-i8000V/NXXJE2";
const char S60979[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.3.4641 Chrome/7.0.517.450 Safari/534.7";
const char S60980[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaC6-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile";
const char S60981[] = "NokiaC6-00/UCWEB8.6.0.180/50/800";
const char S60982[] = "Mozilla/5.0 (Java; U; en-us; nokiac6-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/84/352/UCWEB Mobile  3gpp-gba";
const char S60983[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8730 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60984[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8730T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S60985[] = "Mozilla/5.0 (compatible; Konqueror/4.4; Linux) KHTML/4.4.1 (like Gecko) Fedora/4.4.1-1.fc12";
const char S60986[] = "Mozilla/5.0 (Windows NT 5.1; rv:13.0) Gecko/X Firefox/13.0.1 SeaMonkey/2.10.1";
const char S60987[] = "Mozilla/5.0 (Windows NT 5.1; rv:13.0) Gecko/X SeaMonkey/2.10.1";
const char S60988[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:13.0) Gecko/X Firefox/13.0.1 SeaMonkey/2.10.1";
const char S60989[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Luna TAB474 Build/LunaTAB474) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S60990[] = "Nokia110/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia110) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S60991[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8020 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S60992[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S60993[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S60994[] = "Nokia3230/2.0 (5.0614.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0Configuration/CLDC-1.0, Nokia3230/2.0 (5.0614.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0Configuration/CLDC-1.0";
const char S60995[] = "Nokia3230/2.0 (5.0614.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0Configuration/CLDC-1.0";
const char S60996[] = "Nokia3230";
const char S60997[] = "SHARP-STX2/MC.40; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.19039; U; en-US) Opera 9.50";
const char S60998[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Full Android on Supersonic Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S60999[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Crescent Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61000[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Crescent Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61001[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Wildfire Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61002[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Ray Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61003[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Play Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61004[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Pro Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61005[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Neo V Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61006[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; EHNDROIX-i9003 Build/IMM76L; CyanogenMod-EHNDROIX 12.9.3 Based on 9.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61007[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; novo7 Build/IML74K; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61008[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Mini Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61009[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Neo Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61010[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-be; MP907C Build/IML74K; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61011[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I200 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61012[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.0.400) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S61013[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.0.400) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S61014[] = "SAMSUNG-GT-S7070/S7070ADJD1 SHP/VPP/R5 Jasmine/1.0 Nextreaming/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S61015[] = "1PasswordThumbs/1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro6%2C2)";
const char S61016[] = "1PasswordThumbs/1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (iMac10%2C1)";
const char S61017[] = "1PasswordThumbs/1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookAir3%2C2)";
const char S61018[] = "1PasswordThumbs/1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBook4%2C1)";
const char S61019[] = "1PasswordThumbs/1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (Macmini3%2C1)";
const char S61020[] = "1PasswordThumbs/1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacPro1%2C1)";
const char S61021[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; IS703C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61022[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Penta IS703C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61023[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IS703C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61024[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Penta IS703C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61025[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I727R Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61026[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E110S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61027[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9001 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61028[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9001 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/ariesve";
const char S61029[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; it-it; GT-i9001 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/EHNDROIX III i9001/ariesve";
const char S61030[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM106SH Build/S0010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61031[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM106SH Build/S0020) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61032[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SBM106SH Build/S0020)";
const char S61033[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM106SH Build/S0026) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61034[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Kogan-Agora Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61035[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KHPHN05ANDA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61036[] = "KHPHN05ANDA Linux/3.0.13 Android/4.0.4 Release/04.16.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S61037[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Karbonn A6 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61038[] = "Mozilla/5.0 (Series40; Nokia114/03.33; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S61039[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ601 Build/I.7.1-45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S61040[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; aggregator VocusBot 0.4; +http://www.vocus.com/vnhs.html)";
const char S61041[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) Jayde NicheBot";
const char S61042[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; PTAB7200 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61043[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A690/S001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61044[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A690 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S61045[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Lenovo A690 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61046[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; Lenovo A690 Build/GRK39F)";
const char S61047[] = "PantechP7040/JLUS05242010 Browser/Obigo/Q05A OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S61048[] = "PantechP7040/JFUS03152010F Browser/Obigo/Q05A OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S61049[] = "PantechP7040/JLUS04042011 Browser/Obigo/Q05A OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S61050[] = "Nokia5233/UCWEB8.4.0.150/50/800";
const char S61051[] = "Mozilla/5.0 (S60V5; U; en-in; Nokia5233)/UC Browser8.4.0.159/50/352/UCWEB Mobile";
const char S61052[] = "Mozilla/5.0 (S60V5; U; en-in; Nokia 5233)/UC Browser8.4.0.159/50/352/UCWEB Mobile";
const char S61053[] = "Mozilla/5.0 (Java; U; en-in; nokia5233) UCBrowser8.4.0.159/70/352/UCWEB Mobile  3gpp-gba";
const char S61054[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 Flipboard/1.9.8";
const char S61055[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Flipboard/1.9.8";
const char S61056[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Flipboard/1.9.8";
const char S61057[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145 Flipboard/1.9.8";
const char S61058[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Flipboard/1.9.8";
const char S61059[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Flipboard/1.9.8";
const char S61060[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/1.9.8";
const char S61061[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Flipboard/1.9.8";
const char S61062[] = "SonyEricssonK750i/R1L Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S61063[] = "SAMSUNG-GT-C3010/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S61064[] = "SAMSUNG-GT-C3010/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S61065[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S500 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61066[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; TCL S500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61067[] = "TCL S500 Linux/3.0.13 Android/4.0.4 Release/ Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.4;";
const char S61068[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Nexus 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.2/552,2.0.2.552,2013-05-22 23:05, +0100, gb";
const char S61069[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61070[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E975K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61071[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61072[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6506 Build/10.1.A.1.395) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61073[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-P506 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61074[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P506 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61075[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; LG-P506 Build/FRG83)";
const char S61076[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P506 Build/FRG83G)";
const char S61077[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-P506 Build/FRG85) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61078[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-P506 Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61079[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; zh-TW; rv:1.9.2.16) Gecko/X Firefox/3.6.16 CometBrowser/3.6.16";
const char S61080[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; bg; rv:1.9.2.16) Gecko/X Firefox/3.6.16 CometBird/3.6.16";
const char S61081[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; A511 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61082[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I727R Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S61083[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I727R Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S61084[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E110S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S61085[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; AT-AS43D2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61086[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; AT-AS43D2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61087[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A15 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61088[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MITO A15 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61089[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; A15 Build/IMM76I)";
const char S61090[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One XL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61091[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One XL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61092[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC EVA_UTL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61093[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919V Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S61094[] = "SAMSUNG-GT-C5010E/C5010EDWJI1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S61095[] = "SAMSUNG-GT-C5010D/C5010DBGJG3 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S61096[] = "SAMSUNG-GT-C5010L/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S61097[] = "SAMSUNG-GT-C5010B/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S61098[] = "SAMSUNG-GT-C5010/C5010DDJF3 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S61099[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-P935 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61100[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; LG OPTIMUS LTE HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61101[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P935 Build/IMM76D)";
const char S61102[] = "Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Safari/535.19 Silk-Accelerated=false";
const char S61103[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T769 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S61104[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC_X515E Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S61105[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515m Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S61106[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S61107[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5215 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S61108[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM102SH Build/S0008) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61109[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM102SH Build/S0018) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61110[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM102SH Build/S0012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61111[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM102SH Build/S0020) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61112[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM102SH Build/S0022) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61113[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SBM102SH Build/S0022)";
const char S61114[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM102SH2 Build/S0002) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61115[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SBM102SH2 Build/S0002)";
const char S61116[] = "Mozilla/5.0 (Linux; Android 4.0.4; U9202L-1 Build/HuaweiU9202L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S61117[] = "Mozilla/5.0 (Linux; Android 4.0.3; U9200 Build/HuaweiU9200) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S61118[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LVW Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S61119[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR6410LVW Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S61120[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LRA Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S61121[] = "Mozilla/5.0 (Series40; NokiaX2-05/08.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S61122[] = "SonyEricssonC903a/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3";
const char S61123[] = "SonyEricssonC903/R1GA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.4";
const char S61124[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B367";
const char S61125[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B500";
const char S61126[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_2_1 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B405";
const char S61127[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7W367a Safari/531.21.10";
const char S61128[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.20 (KHTML, like Gecko) Mobile/7B298g";
const char S61129[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SH Build/S0020) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61130[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SH Build/S0038) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61131[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SH Build/S0026) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61132[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SHJ Build/S0012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61133[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SHJ Build/S0006) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61134[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SH Build/S0048) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61135[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SH Build/S0040) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61136[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SHJ Build/S0024) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61137[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SHJ Build/S0016) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61138[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SHK Build/S0008) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61139[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SHK Build/S0006) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61140[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SHK Build/S0016) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61141[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SBM007SH Build/S0048)";
const char S61142[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SH Build/S0058) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61143[] = "Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Safari/535.19 Silk-Accelerated=true";
const char S61144[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; Edition MacAppStore; en) Presto/2.10.229 Version/11.61";
const char S61145[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6.8; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.61";
const char S61146[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:9.0) Gecko/X Firefox/9.0";
const char S61147[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire C Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61148[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61149[] = "Mozilla/5.0 (Linux; Android 4.1.2; Torque Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S61150[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-SU640 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61151[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-SU640 Build/IMM76D)";
const char S61152[] = "SAMSUNG-GT-S3570/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S61153[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-US780 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61154[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T989 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61155[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-T989 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61156[] = "Opera/9.80 (Linux armv7l; Maemo; Opera Mobi/14; U; en-GB) Presto/2.9.201 Version/11.50";
const char S61157[] = "Opera/9.80 (Linux armv6l; Maemo; Opera Mobi/14; U; en-GB) Presto/2.9.201 Version/11.50";
const char S61158[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S61159[] = "Instagram 3.2.0 (iPhone4,1; iPhone OS 6.0.1; en_US) AppleWebKit/420+";
const char S61160[] = "Instagram 3.2.0 (iPhone4,1; iPhone OS 6.0; en_US) AppleWebKit/420+";
const char S61161[] = "Instagram 3.2.0 (iPhone5,1; iPhone OS 6.0.2; en_US) AppleWebKit/420+";
const char S61162[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl_NL; V360 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61163[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; V360 Build/JRO03C)";
const char S61164[] = "Mozilla/5.0 (Linux; Android 4.1.2; C2105 Build/15.0.A.2.17) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S61165[] = "Mozilla/5.0 (Linux; Android 4.1.2; C2105 Build/15.0.A.2.17) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S61166[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61167[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; A6 Build/HuaqinA6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61168[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S61169[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S61170[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315";
const char S61171[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S61172[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus S Build/JRO03E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S61173[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S61174[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S61175[] = "Mozilla/5.0 (Linux; Android 4.2.1; ME301T Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S61176[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S61177[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2S Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S61178[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2A Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S61179[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2SC Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S61180[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2C Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S61181[] = "NokiaX2-02/2.0 (11.79) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-02) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S61182[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7310 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S61183[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Iron/10.0.650.1 Chrome/10.0.650.1 Safari/534.16";
const char S61184[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5800 XpressMusic)/UC Browser8.2.0.132/50/355/UCWEB Mobile";
const char S61185[] = "Nokia5800 XpressMusic/UCWEB8.2.0.116/50/800";
const char S61186[] = "Nokia5800w XpressMusic/UCWEB8.2.0.116/50/800";
const char S61187[] = "Mozilla/5.0 (Java; U; en-us; nokia5800d-1) UCBrowser8.2.1.144/69/352/UCWEB Mobile  3gpp-gba";
const char S61188[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/22.0.1183.0 Safari/537.1";
const char S61189[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/22.0.648.205 Safari/534.16";
const char S61190[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/22.0.648.205 Safari/534.16";
const char S61191[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I8150 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61192[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5235) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S61193[] = "Nokia5235UCBrowser/8.7.1.234/50/444/UCWEB";
const char S61194[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9200 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S61195[] = "LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S61196[] = "LG-GD510-Bouygues/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S61197[] = "LG-GD510-Orange/V10d Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S61198[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12; Microsoft ZuneHD 4.3)";
const char S61199[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12; Microsoft ZuneHD 4.5)";
const char S61200[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12; Microsoft ZuneHD 4.1)";
const char S61201[] = "LG/KU990i-Orange/v10a Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S61202[] = "LG/KU990i/v10a Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S61203[] = "LG/KU990i-Bouygues/v10a Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S61204[] = "Vodafone/1.0/LG-KU990i/V10a Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S61205[] = "Vodafone/1.0/LG-KU990i/V10c Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S61206[] = "Vodafone/1.0/LG-KU990i/V10d Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S61207[] = "Vodafone/1.0/LG-KU990i/V10b Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S61208[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8) Presto/2.12.388 Version/12.13";
const char S61209[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.276; en-us; 9900/5.1.0.276) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S61210[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.532; en-us; 9900/5.1.0.532) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S61211[] = "UCWEB/2.0(BlackBerry; U; 5.0.0.734; en-us; 9900/5.0.0.734) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S61212[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.230; en-us; 9900/5.1.0.230) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S61213[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.393; en-us; 9900/5.1.0.393) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S61214[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.546; en-us; 9900/5.1.0.546) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S61215[] = "UCWEB/2.0(BlackBerry; U; 5.0.0.496; en-us; 9900/5.0.0.496) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S61216[] = "UCWEB/2.0(BlackBerry; U; 5.0.0.464; en-us; 9900/5.0.0.464) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S61217[] = "UCWEB/2.0(BlackBerry; U; 5.0.0.735; en-us; 9900/5.0.0.735) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S61218[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.415; en-us; 9900/5.1.0.415) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S61219[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.181; en-us; 9900/5.1.0.181) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S61220[] = "UCWEB/2.0(BlackBerry; U; 5.0.0.694; en-us; 9900/5.0.0.694) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S61221[] = "UCWEB/2.0(BlackBerry; U; 5.0.0.733; en-us; 9900/5.0.0.733) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S61222[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.338; en-us; 9900/5.1.0.338) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S61223[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.75; en-us; 9900/5.1.0.75) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S61224[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.641; en-us; 9900/5.1.0.641) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S61225[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S61226[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.8.0.4) Gecko/X Firefox/1.5.0.4";
const char S61227[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LG730 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S61228[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LG730 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S61229[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S61230[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I997 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61231[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I997 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61232[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I997 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61233[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I997 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61234[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; chagall Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61235[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MIDC410 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61236[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MIDC410PR Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61237[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; LG-P765 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61238[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-P765 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61239[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P765 Build/JZO54K)";
const char S61240[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; i-mate JAMin PPC; 240x320; PPC; 240x320)";
const char S61241[] = "Dopod900 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 480x640; Dopod900)";
const char S61242[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; i-mate JASJAM V2 PPC; 240x320; i-mate JASJAM PPC; 240x320; PPC; 240x320)";
const char S61243[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; i-mate JASJAR PPC; 640x480; PPC; 240x320)";
const char S61244[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; i-mate JASJAM PPC; 240x320; PPC; 240x320)";
const char S61245[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; MDA Pro/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1)";
const char S61246[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; HP iPAQ h6300)";
const char S61247[] = "Mozilla/4.0 Sprint:MotoQ (compatible; MSIE 4.01; Windows CE; Smartphone; 176x220)";
const char S61248[] = "GIGABYTE-g-Smart Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; GIGABYTE-g-Smart)";
const char S61249[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC i-mate PDAL; 240x320; PPC)";
const char S61250[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; i-mate JAQ PPC; 240x320; PPC; 240x320)";
const char S61251[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE)";
const char S61252[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 800x480)";
const char S61253[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC;   240x320)";
const char S61254[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPS; 240x320)";
const char S61255[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone)";
const char S61256[] = "Mozilla/4.0 PPC (compatible; MSIE 4.01; Windows CE; PPC; 240x320; Sprint:PPC-6700; PPC; 240x320)";
const char S61257[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Sprint:SPH-ip320; Smartphone; 176x220)";
const char S61258[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; PPC)";
const char S61259[] = "Mozilla/4.0(compatible; MSIE 4.01; Windows CE; PPC; 240x320)";
const char S61260[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Sprint;PPC-i830; PPC; 240x320)";
const char S61261[] = "Mozilla/5.0 (compatible; MSIE 4.0; PPC; Tablet PC; Trident/1.0)";
const char S61262[] = "Vodafone/1.0/HTC_prophet/2.15.3.113/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)";
const char S61263[] = "NSPlayer/11.0.5721.5262";
const char S61264[] = "NSPlayer/11.0.5721.5262 WMFSDK/11.0";
const char S61265[] = "NSPlayer/11.0.6002.18005";
const char S61266[] = "NSPlayer/11.0.5721.5245 WMFSDK/11.0";
const char S61267[] = "NSPlayer/11.0.6001.7000 WMFSDK/11.0";
const char S61268[] = "NSPlayer/11.0.5705.5043";
const char S61269[] = "NSPlayer/11.0.6000.6346";
const char S61270[] = "NSPlayer/11.0.6001.7006 WMFSDK/11.0";
const char S61271[] = "NSPlayer/11.0.6000.6351 WMFSDK/11.0";
const char S61272[] = "NSPlayer/11.0.5721.5269";
const char S61273[] = "NSPlayer/11.00.6001.7000 WMFSDK/11.00.6001.7000";
const char S61274[] = "NSPlayer/11.0.5721.5275 WMFSDK/11.0 SE 2.X MetaSr 1.0";
const char S61275[] = "NSPlayer/11.0.5721.5262 WMFSDK/11.0 SE 2.X MetaSr 1.0";
const char S61276[] = "NSPlayer/11.0.5721.5275 WMFSDK/11.0 LBBROWSER";
const char S61277[] = "NSPlayer/11.0.5721.5145 WMFSDK/11.0 SE 2.X MetaSr 1.0";
const char S61278[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.1.8) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S61279[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.4.27) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S61280[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.6.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S61281[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.0.16) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S61282[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.6.8) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S61283[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.4.22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S61284[] = "Mozilla/5.0 (Linux; Android 4.1.1; Micromax A110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S61285[] = "Mozilla/5.0 (Linux; Android 4.1.2; A110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S61286[] = "Mozilla/5.0 (Linux; Android 4.1.1; A110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S61287[] = "Mozilla/5.0 (Linux; Android 4.1.1; Micromax A110 (Canvas 2) Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S61288[] = "SAMSUNG-SGH-C414Y/C414USKA4 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S61289[] = "SAMSUNG-SGH-C414R/C414UXKD1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S61290[] = "SAMSUNG-SGH-C414M/C414BMKD3 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S61291[] = "SAMSUNG-SGH-C414/C414TLJK9 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S61292[] = "SAMSUNG-SGH-C414W/C414UQKD1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S61293[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-08D Build/F0001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61294[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-08D Build/V10R21B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61295[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-08D Build/V12R23A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61296[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; F-08D Build/V12R23A)";
const char S61297[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; de; rv:1.8.1.24) Gecko/X Thunderbird/2.0.0.24";
const char S61298[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-SU870 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61299[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-SU870 Build/GRK39F)";
const char S61300[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E971 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61301[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-F Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S61302[] = "Mozilla/5.0 (Linux; Android 4.0.4; SmartTabII7 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S61303[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-H Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S61304[] = "Mozilla/5.0 (Linux; Android 4.0.4; A2107A-H Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S61305[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL_one_touch_995 Build/IceCreamSandwich) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61306[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL_one_touch_995S Build/IceCreamSandwich) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61307[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL_one_touch_995A Build/IceCreamSandwich) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61308[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-E400 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61309[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A70S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61310[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; A70S Build/FROYO)";
const char S61311[] = "IUC(U;iOS 5.0.1;Zh-cn;320*480;)/UCWEB8.3.1.151/42/997";
const char S61312[] = "IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB8.3.1.151/41/997";
const char S61313[] = "IUC(U;iOS 4.1;Zh-cn;320*480;)/UCWEB8.3.1.151/41/800";
const char S61314[] = "Mozilla/5.0 (Java; U; en-in; j2me) UCBrowser8.3.0.154/69/352/UCWEB Mobile";
const char S61315[] = "Mozilla/5.0 (Java; U; en-in; mstar) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S61316[] = "IUC(U;iOS 5.1.1;Zh-cn;320*480;)/UCWEB8.3.1.151/42/997";
const char S61317[] = "Mozilla/5.0 (Java; U; en-in; mstar platform) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S61318[] = "Mozilla/5.0 (Java; U; en-in; spice) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S61319[] = "IUC(U;iOS 4.2.1;Zh-cn;320*480;)/UCWEB8.3.1.151/41/997";
const char S61320[] = "Mozilla/5.0 (Java; U; en-in; opera) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S61321[] = "Mozilla/5.0 (Java; U; en-us; maui) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S61322[] = "IUC(U;iOS 5_1_1;en-us;320*480;)/UCWEB8.3.1.173/42/355";
const char S61323[] = "Mozilla/5.0 (Java; U; en-us; generic) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S61324[] = "Mozilla/5.0 (Java; U; kau; mozilla) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S61325[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61326[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; A5 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61327[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HOOW A5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61328[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; id-id; CROSS A5 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61329[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TOUCHMATE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61330[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TOUCHMATE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61331[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SH-04E Build/S1090) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61332[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SH-04E Build/S2130) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61333[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SH-04E Build/S5170) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61334[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SH-04E Build/S6181) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61335[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SH-04E Build/S6181)";
const char S61336[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; PadFone Infinity Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61337[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; PadFone Infinity Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61338[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; PadFone Infinity Build/JZO54K)";
const char S61339[] = "Opera/6.x (Windows NT 4.0; U) [de]";
const char S61340[] = "Opera/6.02 (Windows NT 4.0; U) [de]";
const char S61341[] = "Opera/6.04 (Windows NT 4.0; U) [de]";
const char S61342[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 4.0) Opera 6.01 [en]";
const char S61343[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 4.0) Opera 6.04 [en]";
const char S61344[] = "Opera/6.05 (Windows NT 4.0; U) [de]";
const char S61345[] = "Mozilla/5.0 (Windows NT 4.0; U) Opera 6.05 [en]";
const char S61346[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 4.0) Opera 6.0 [de]";
const char S61347[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 4.0) Opera 6.02 [en]";
const char S61348[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyC5306 Build/12.0.A.1.284) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61349[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; C5306 Build/12.0.A.1.284) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61350[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; C5306 Build/12.0.A.1.284)";
const char S61351[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE71) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile";
const char S61352[] = "NokiaE71/UCWEB8.6.0.180/28/999";
const char S61353[] = "Mozilla/5.0 (Java; U; en-us; nokiae71-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/84/352/UCWEB Mobile";
const char S61354[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-05E Build/V12R34G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61355[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-05E Build/V13R36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61356[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-05E Build/V16R39A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61357[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-05E Build/V17R40A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61358[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-I9100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S61359[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-I9100 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S61360[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-I9100G Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S61361[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9100G Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S61362[] = "Opera/7.54 (Linux)";
const char S61363[] = "Mozilla/5.0 (Linux 2.4.21-0.13mdk i686; U) Opera 7.11 [en]";
const char S61364[] = "Opera/7.51 (Linux) [en]";
const char S61365[] = "Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux i686) Opera 7.54 [en]";
const char S61366[] = "Mozilla/5.0 (X11; Linux i686; U) Opera 7.54 [en]";
const char S61367[] = "Mozilla/5.0 (X11; Linux i386; U) Opera 7.60  [en-GB]";
const char S61368[] = "MSIE (MSIE 6.0; X11; Linux; i686) Opera 7.23";
const char S61369[] = "Opera/7.50 (X11; Linux i686; U)  [en]";
const char S61370[] = "Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux i686) Opera 7.23";
const char S61371[] = "Opera/7.23 (X11; Linux i686; U)  [en]";
const char S61372[] = "Opera/7.54 (X11; Linux i686; U)  [en]";
const char S61373[] = "JUC (Linux; U; 2.1-update1; zh-cn; E15i; 320*480) UCWEB7.9.0.94/139/352";
const char S61374[] = "JUC (Linux; U; 2.1-update1; zh-cn; E15a; 320*480) UCWEB7.9.0.94/139/444";
const char S61375[] = "JUC (Linux; U; 2.3.7; zh-cn; E15i; 320*480) UCWEB7.9.0.94/139/352";
const char S61376[] = "Opera (Linux; U; 2.1-update1; zh-cn; E15i; 320*480) UCWEB7.9.0.94/139/352";
const char S61377[] = "Mozilla/5.0 (Linux; Android 4.0.3;  MT11i Build/4.1.A.0.562) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S61378[] = "Mozilla/5.0 (Linux; Android 4.0.3; MT11i Build/4.1.A.0.562) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61379[] = "Mozilla/5.0 (Linux; Android 4.0.4;  MT11i Build/4.1.B.0.431) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S61380[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT11i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61381[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT11a Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61382[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26w Build/6.1.A.2.45) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S61383[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I547 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S61384[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I547C Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S61385[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I547 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S61386[] = "SAMSUNG-GT-C3330/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S61387[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:11.0) Gecko/X Firefox/11.0.1-x64 PaleMoon/11.0.1-x64";
const char S61388[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:11.0) Gecko/X Firefox/11.0.1 PaleMoon/11.0.1";
const char S61389[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A860L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61390[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A860L Build/JZO54K)";
const char S61391[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A860L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30";
const char S61392[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; ALCATEL ONE TOUCH 6030D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S61393[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ALCATEL ONE TOUCH 6030D Build/JRO03C)";
const char S61394[] = "Mozilla/5.0 (Linux; Android 4.2.2; cm_tenderloin Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S61395[] = "Mozilla/5.0 (Linux; Android 4.2.2; Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61396[] = "Mozilla/5.0 (Linux; Android 4.2.1; e2001_v89_gq2008s Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61397[] = "Mozilla/5.0 (Linux; Android 4.2.1; e2001_v89_jbl1a698 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61398[] = "Mozilla/5.0 (Linux; Android 4.2.1; v89_jbl1a698 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61399[] = "Mozilla/5.0 (Linux; Android 4.2.2; skz_tenderloin Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S61400[] = "Mozilla/5.0 (Linux; Android 4.2.2; Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S61401[] = "Mozilla/5.0 (Linux; Android 4.2.1; e2001_v89_jdt1qhd Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61402[] = "Mozilla/5.0 (Linux; Android 4.2.1; e2001v21_v89_zlh_hd Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61403[] = "Mozilla/5.0 (Linux; Android 4.2; iPhone5 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61404[] = "Mozilla/5.0 (Linux; Android 4.2.1; e2001_v89_zlh_hd Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61405[] = "Opera/9.80 (Windows NT 6.1; Opera Tablet/15165; U; en) Presto/2.8.149 Version/11.1";
const char S61406[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 1_2_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5";
const char S61407[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySOL21 Build/9.1.D.0.395) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61408[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SOL21 Build/9.1.D.0.395) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61409[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySOL21 Build/9.1.D.0.401) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61410[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SOL21 Build/9.1.D.0.401) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61411[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SOL21 Build/9.1.D.0.401)";
const char S61412[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.2.1.0 Chrome/16.0.912.75 Safari/535.7";
const char S61413[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.2.1.0 Chrome/16.0.912.75 Safari/535.7";
const char S61414[] = "Mozilla/5.0 (Linux; U; Android 2.3;es_MX; ZTE-U V856 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61415[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; ZTE-V856 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61416[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; ZTE-V856 Build/GRJ22)";
const char S61417[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia2690) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S61418[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xperia Acro S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61419[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26w Build/6.2.B.0.200) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61420[] = "Mozilla/5.0 (Linux; Android 4.0.4; N9100 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61421[] = "JUC (Linux; U; 2.3.6; zh-cn; GT-S6102; 240*320) UCWEB7.9.0.94/139/444";
const char S61422[] = "Mozilla/5.0 (Linux; U; Android 1.1.7; md-us; A101B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S61423[] = "Mozilla/5.0 (Linux; U; Android 1.1.20; md-us; A101C Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S61424[] = "Mozilla/5.0 (Linux; U; Android 6.1.8; md-us; A101B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S61425[] = "Mozilla/5.0 (Linux; U; Android 1.1.5; fr-fr; A101B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S61426[] = "Mozilla/5.0 (Linux; U; Android 6.1.8; nl-nl; A101B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S61427[] = "Mozilla/5.0 (Linux; U; Android 6.1.8; fr-fr; A101B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S61428[] = "Mozilla/5.0 (Linux; U; Android 6.1.8; es-es; A101B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S61429[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:20.0) Gecko/X Firefox/20.0 SeaMonkey/2.17.1";
const char S61430[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; ISW11HT Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61431[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; IM-A740S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61432[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; IM-A750K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61433[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; IM-A750K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61434[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; IM-A750K Build/GINGERBREAD)";
const char S61435[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; IM-A740S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61436[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SH Build/S0008) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61437[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SH Build/S0012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61438[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SH Build/S0018) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61439[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SH Build/S0016) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61440[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SH Build/S0022) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61441[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SHY Build/S0024) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61442[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SHY Build/S0014) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61443[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SHY Build/S0020) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61444[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SHY Build/S0016) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61445[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SBM009SH Build/S0022)";
const char S61446[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SH Build/S0026) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61447[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SH Build/S0032) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61448[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SHY Build/S0028) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61449[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SHY Build/S0030) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61450[] = "Mozilla/5.0 (HW-HUAWEI_U2801;U;REX/1.0;BREW/5.0.4.1;Profile/MIDP-2.0 Configuration/CLDC-1.1;128*160) Opera/9.70";
const char S61451[] = "Mozilla/5.0 (HW-HUAWEI_U3220;U;REX/1.0;BREW/5.0.4.1;Profile/MIDP-2.0 Configuration/CLDC-1.1;240*320) Opera/9.70";
const char S61452[] = "Mozilla/5.0 (HW-HUAWEI_U5200/;U;REX/1.0;BREW/5.0.4.1;Profile/MIDP-2.0 Configuration/CLDC-1.1;240*320) Opera/9.70";
const char S61453[] = "Mozilla/5.0 (HW-HUAWEI_U5300/;U;REX/1.0;BREW/5.0.4.1;Profile/MIDP-2.0 Configuration/CLDC-1.1;240*320) Opera/9.70";
const char S61454[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A403 Safari/7534.48.3";
const char S61455[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A523 Safari/7534.48.3";
const char S61456[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 60 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/19.0.1084.60 Mobile/10A403 Safari/7534.48.3";
const char S61457[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:14.0) Gecko/X Thunderbird/14.0";
const char S61458[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Edison Build/2.0.0 X-15:47) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61459[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Edison Build/2.1.0 X-12:38) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61460[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; bq Edison Build/2.1.0 X-12:38)";
const char S61461[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Edison 2 Build/1.0.0_X-18:36) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61462[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Edison 2 Build/1.0.1_X-14:02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61463[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; bq Edison 2 Build/1.0.1_X-14:02)";
const char S61464[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7310 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S61465[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P7310 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S61466[] = "Mozilla/5.0 (Linux; Android 4.0.3; PH44100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S61467[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTCEVODesign4G Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S61468[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-P713 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61469[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; LG-P713GO Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61470[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P713 Build/JZO54K)";
const char S61471[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/5.0.354.0 Safari/533.3";
const char S61472[] = "Mozilla/5.0 (Linux; Android 4.0.3; ST18i Build/4.1.A.0.562) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61473[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST18a Build/4.1.B.0.431) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61474[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST18i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61475[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190L Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S61476[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S61477[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S61478[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190L Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S61479[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SGP311 Build/10.1.C.0.344) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61480[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGP311 Build/10.1.C.0.370) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61481[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGP311 Build/10.1.C.0.370)";
const char S61482[] = "SonySGP311 Build/10.1.C.0.370 stagefright/1.2 (Linux;Android 4.1.2)";
const char S61483[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9A405 Safari/7534.48.3";
const char S61484[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9A334 Safari/7534.48.3";
const char S61485[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 501 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/19.0.1084.60 Mobile/9A405 Safari/7534.48.3";
const char S61486[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 50 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/19.0.1084.60 Mobile/9A334 Safari/7534.48.3";
const char S61487[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT30a Build/7.0.A.1.300) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61488[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SonyLT30a Build/7.0.A.1.303) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61489[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SonyLT30a Build/7.0.A.3.195) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61490[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LT30a Build/7.0.A.3.195) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61491[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT30a Build/7.0.A.3.195)";
const char S61492[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; WX04K Build/143.3.0000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61493[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; WX04K Build/145.2.0000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61494[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; WX04K Build/145.0.0000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61495[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; WX04K Build/146.2.0000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61496[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; WX04K Build/147.2.0000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61497[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; WX04K Build/147.2.0000)";
const char S61498[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; B92 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61499[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; B92 Build/IMM76D)";
const char S61500[] = "Nokia2720a-2/2.0 (09.55) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S61501[] = "Nokia2720a-2b/2.0 (08.64) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S61502[] = "Nokia2720a/2.0 (09.55) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S61503[] = "sam-r570 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S61504[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-ca; LG-V905R Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S61505[] = "Dalvik/1.5.0 (Linux; U; Android 3.0.1; LG-V905R Build/HRI66)";
const char S61506[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; XT720 Build/SHOLS_U2_05.26.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61507[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; XT720 Build/STSKT_N_79.33.50R) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61508[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Milestone XT720 Build/R.U.R.1920) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61509[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; XT720 Build/STSKT_N_79.33.50R)";
const char S61510[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; Milestone XT720 Build/R.U.R.1920) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61511[] = "NetFront/3.5.1 (BREW 5.0.2.1; U; en-us; Samsung ; NetFront/3.5.1/AMB) Sprint SPH-M570 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S61512[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SC-01C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61513[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; SC-01C Build/GINGERBREAD)";
const char S61514[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ja-jp; SC-01C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61515[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P700 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61516[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; i-mobile IQ 6 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61517[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; i-mobile IQ 6A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61518[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; i-mobile IQ 6 Build/JRO03C)";
const char S61519[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61520[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus S Build/JRO03E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61521[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/10B141 Safari/7534.48.3";
const char S61522[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/10B329 Safari/7534.48.3";
const char S61523[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/10B146 Safari/7534.48.3";
const char S61524[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/10B147 Safari/7534.48.3";
const char S61525[] = "SonyEricssonK550i//R6BC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S61526[] = "SonyEricssonK550c/R1JD Release/Jan-23-2007 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S61527[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.9 (KHTML, like Gecko) Iron/4.0.280.0 Chrome/4.0.280.0 Safari/532.9";
const char S61528[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/532.8 (KHTML, like Gecko) Iron/4.0.275.2 Chrome/4.0.275.2 Safari/532.8";
const char S61529[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.11; en-us; NXM736 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S61530[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.19; en-us; NXM736 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S61531[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.14; en-us; NXM736 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S61532[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S61533[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S61534[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S61535[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9305 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S61536[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-I747M Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S61537[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530U Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S61538[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; W100 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61539[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; W100 Build/JOP40D)";
const char S61540[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; LG-C710; Build/Eclair) AppleWebKit/530.17(KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S61541[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; LG-C710h Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S61542[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; LG-C710h; Build/Eclair) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S61543[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; LG-C710h Build/ERE27)";
const char S61544[] = "Mozilla/5.0 (Windows; U; Windows CE 5.2; rv:1.8.1.4pre) Gecko/20070327 Minimo/0.020";
const char S61545[] = "Mozilla/6.0 (compatible; MSIE 7.01; Windows NT Windows CE)";
const char S61546[] = "Mozilla/1.1 (compatible; MSPIE 2.0; Windows CE)";
const char S61547[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT30p Build/7.0.A.3.195) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S61548[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT30p Build/7.0.A.3.195) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S61549[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; haw-US; KFJWI Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.1 Mobile Safari/535.19 Silk-Accelerated=false";
const char S61550[] = "J2ME/UCWEB7.2.2.54/139/352";
const char S61551[] = "J2ME/UCWEB7.2.2.54/139/351";
const char S61552[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U9500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61553[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U9500 Build/HuaweiU9500) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61554[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; U9500 Build/HuaweiU9500) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S61555[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; U9500 Build/HuaweiU9500)";
const char S61556[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; U9500E Build/HuaweiU9500E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61557[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; SGH-I717D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61558[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-I717D Build/GINGERBREAD)";
const char S61559[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01D Build/S8230) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61560[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01D Build/S2050) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61561[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SH-01D Build/S8230)";
const char S61562[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01D Build/S8060) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61563[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; VS TOUCHTAB 8DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61564[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; VS TOUCHTAB 8DC Build/JRO03H)";
const char S61565[] = "Mozilla/5.0 (Linux; Android 4.1.1; Arc Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S61566[] = "RealPlayer%20Downloader949 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac7%2C1)";
const char S61567[] = "RealPlayer%20Downloader949 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBookPro4%2C1)";
const char S61568[] = "Spector%20Pro3731 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook5%2C1)";
const char S61569[] = "RealPlayer%20Downloader1116 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook5%2C1)";
const char S61570[] = "Spector%20Pro3731 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac9%2C1)";
const char S61571[] = "RealPlayer%20Downloader1569 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBookPro5%2C5)";
const char S61572[] = "Spector%20Pro2657 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac8%2C1)";
const char S61573[] = "Spector%20Pro3950 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook4%2C1)";
const char S61574[] = "Spector%20Pro3987 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac9%2C1)";
const char S61575[] = "RealPlayer%20Downloader1116 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac9%2C1)";
const char S61576[] = "RealPlayer%20Downloader884 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook4%2C1)";
const char S61577[] = "RealPlayer%20Downloader949 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook3%2C1)";
const char S61578[] = "RealPlayer%20Downloader1569 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook4%2C1)";
const char S61579[] = "Spector%20Pro3950 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac9%2C1)";
const char S61580[] = "RealPlayer%20Downloader884 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac8%2C1)";
const char S61581[] = "RealPlayer%20Downloader1444 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook4%2C1)";
const char S61582[] = "RealPlayer%20Downloader884 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBookPro4%2C1)";
const char S61583[] = "GrandReporterUpdater1.1.0 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook5%2C2)";
const char S61584[] = "Spector%20Pro4195 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac8%2C1)";
const char S61585[] = "RealPlayer%20Downloader1725 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook4%2C1)";
const char S61586[] = "Spector%20Pro4413 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac8%2C1)";
const char S61587[] = "RealPlayer%20Downloader1116 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBookPro4%2C1)";
const char S61588[] = "Spector%20Pro2743 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac9%2C1)";
const char S61589[] = "RealPlayer%20Downloader1569 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac7%2C1)";
const char S61590[] = "Spector%20Pro3987 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook4%2C1)";
const char S61591[] = "iCoreService (unknown version) CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook4%2C1)";
const char S61592[] = "iCoreService (unknown version) CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac8%2C1)";
const char S61593[] = "RealPlayer%20Downloader1725 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac8%2C1)";
const char S61594[] = "iCoreService (unknown version) CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBookPro5%2C5)";
const char S61595[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.7) Gecko/X Firefox/3.5.7 Flock/2.5.6 (.NET CLR 3.5.30729)";
const char S61596[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.16) Gecko/X Firefox/3.0.16 Flock/2.5.6 FBSMTWB";
const char S61597[] = "Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E300L Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S61598[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGP321 Build/10.1.1.A.1.253)";
const char S61599[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonySGP321 Build/10.1.1.A.1.307) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61600[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SGP321 Build/10.1.1.A.1.253) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61601[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonySGP321 Build/10.1.1.A.1.253) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61602[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SGP321 Build/10.1.1.A.1.307) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61603[] = "Midori/0.2.2 (X11; Linux i686; U; en-us) WebKit/531.2+";
const char S61604[] = "Midori/0.2 (X11; Linux; U; en-us) WebKit/531.2+";
const char S61605[] = "Midori/0.2 (X11; Linux; U; c) WebKit/531.2+";
const char S61606[] = "Mozilla/5.0 (X11; U; Linux; en-us) AppleWebKit/534+ (KHTML, like Gecko) Safari/534.7+ Midori/0.2";
const char S61607[] = "Midori/0.2 (X11; Linux; U; de-de) WebKit/531.2+";
const char S61608[] = "Midori/0.2 (X11; Linux; U; en-gb) WebKit/531.2+";
const char S61609[] = "Midori/0.2.2 (X11; Linux i686; U; en-gb) WebKit/531.2+";
const char S61610[] = "Midori/0.2.2 (X11; Linux i686; U; de-de) WebKit/531.2+";
const char S61611[] = "Midori/0.2.2 (X11; Linux x86_64; U; en-us) WebKit/531.2+";
const char S61612[] = "Midori/0.2 (X11; Linux; U; es-ve) WebKit/531.2+";
const char S61613[] = "Midori/0.2 (X11; Linux; U; fr-fr) WebKit/531.2+";
const char S61614[] = "Midori/0.2 (X11; Linux; U; es-es) WebKit/531.2+";
const char S61615[] = "Midori/0.2.0 (X11; Linux i686; U; en-us) WebKit/531.2+";
const char S61616[] = "Midori/0.2.2 (X11; Linux i686; U; ru-ru) WebKit/531.2+";
const char S61617[] = "Mozilla/5.0 (X11; U; Linux; es-ve; rv:1.9.0.2) Gecko/X Firefox/3.8 Midori/0.2";
const char S61618[] = "Midori/0.2.2 (X11; Linux i686; U; ja-jp) WebKit/531.2+";
const char S61619[] = "Midori/0.2 (X11; Linux; U; it-it) WebKit/531.2+";
const char S61620[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9300 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S61621[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-I9300 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S61622[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; SGH-T999 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S61623[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; SCH-I535 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S61624[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; SCH-I535 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S61625[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; SGH-T999 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S61626[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9300 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S61627[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-us ; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S61628[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/4.0.0)";
const char S61629[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/4.0.0)";
const char S61630[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22 Sleipnir/4.0.1";
const char S61631[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/4.0.1)";
const char S61632[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/4.0.1)";
const char S61633[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22 Sleipnir/4.0.1";
const char S61634[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/4.0.0)";
const char S61635[] = "SAMSUNG-GT-E2370/E1055GJVKB3 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S61636[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-i909 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61637[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SCH-i909 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61638[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SCH-i909 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S61639[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-i909 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S61640[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-I9088 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61641[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; zh-tw; SCH-i909 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61642[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SCH-i909 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S61643[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; zh-cn; SCH-i909 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S61644[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-i909 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S61645[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-I9088 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S61646[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; SCH-i909 Build/FROYO)";
const char S61647[] = "Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300T Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S61648[] = "Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300TG Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S61649[] = "Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300TL Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S61650[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S7562 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S61651[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-S7562i Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S61652[] = "Mozilla/5.0 (Linux; Android 4.2.2; SM-T311 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S61653[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 Nokia5730s-1/200.12.87; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.1.1";
const char S61654[] = "Mozilla/4.0 (compatible; MSIE 5.5;";
const char S61655[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CMP741E Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61656[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; CMP741E Build/ITL41D) AppleWebKit/434.50 (KHTML, like Gecko) Version/4.0 Mobile";
const char S61657[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xoom Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S61658[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ505 Build/Moto_MZ505) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S61659[] = "Mozilla/5.0 (Linux; Android 4.0.3; Xoom Build/IML77) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S61660[] = "Mozilla/5.0 (Linux; U; Android 4.2 jellybean # ariel-tbc /UCHIHA; en-CA ; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S61661[] = "Mozilla/5.0 (Linux; Android 4.1.1; Vodafone 875 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S61662[] = "Mozilla/5.0 (Linux; Android 4.1.1; Vodafone 875 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S61663[] = "Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia305) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S61664[] = "Opera/9.80 (BlackBerry; Opera Mini/4.3.25172/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S61665[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I727 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61666[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I727 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61667[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I727 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61668[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SAMSUNG-SGH-I727 Build/JDQ39E)";
const char S61669[] = "SAMSUNG-GT-I6410-Vodafone/I6410BUJC1 Linux/X2/R1 Opera/9.6 SMS-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S61670[] = "Opera/9.62 (X11;Linux i686; U; en) Presto/2.2.0/SAMSUNG-GT-I6410-Vodafone/";
const char S61671[] = "Instagram 3.4.2 (iPad2,1; iPhone OS 6.0.1; en_US; en) AppleWebKit/420+";
const char S61672[] = "Instagram 3.4.2 (iPad2,5; iPhone OS 6.0.2; en_US; en) AppleWebKit/420+";
const char S61673[] = "Instagram 3.4.2 (iPad2,1; iPhone OS 6.0; en_US; en) AppleWebKit/420+";
const char S61674[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S61675[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S61676[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36";
const char S61677[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36";
const char S61678[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36";
const char S61679[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36";
const char S61680[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S61681[] = "Nokia202/2.0 (20.28) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia202) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S61682[] = "UCWEB/2.0 (Java; U; MIDP-2.0; Nokia203/20.37) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S61683[] = "Nokia203/2.0 (20.36) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia203) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S61684[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One SV Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S61685[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S61686[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S61687[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S61688[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S61689[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S61690[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S61691[] = "Mozilla/5.0 (Linux; Android 4.2.1; IdeaTab S6000-F Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S61692[] = "Mozilla/5.0 (Linux; Android 4.2.2; IdeaTab S6000-H Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S61693[] = "Mozilla/5.0 (Linux; Android 4.2.2; IdeaTab S6000-F Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S61694[] = "ZDM/4.0; Windows Mobile 8.0";
const char S61695[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P880 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61696[] = "Flipboard/2.0.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S61697[] = "LGE-MN180/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S61698[] = "GoogleEarth/7.0.1.8244(Windows;Microsoft Windows (6.0.6002.2);en;kml:2.2;client:Free;type:default)";
const char S61699[] = "GoogleEarth/7.0.2.8415(Windows;Microsoft Windows (6.0.6002.2);en;kml:2.2;client:Free;type:default)";
const char S61700[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.0.6002.2);en;kml:2.2;client:Free;type:default)";
const char S61701[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.0.6001.1);en;kml:2.2;client:Free;type:default)";
const char S61702[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.0.6002.2);de;kml:2.2;client:Free;type:default)";
const char S61703[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.0.6002.2);nl;kml:2.2;client:Free;type:default)";
const char S61704[] = "GoogleEarth/7.0.2.8415(Windows;Microsoft Windows (6.0.6002.2);de;kml:2.2;client:Free;type:default)";
const char S61705[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.0.6002.2);fr;kml:2.2;client:Free;type:default)";
const char S61706[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.0.6002.2);it;kml:2.2;client:Free;type:default)";
const char S61707[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.0.6002.2);es;kml:2.2;client:Free;type:default)";
const char S61708[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID7120 Build/GRI40)";
const char S61709[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID7120 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61710[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID7120A Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61711[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Y3 Build/HCL) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61712[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Connect-3G Build/HCL) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61713[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; Y3 Build/HCL) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61714[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE N9120 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S61715[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE N9120 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S61716[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Sensation Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61717[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Sensation Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61718[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Sensation Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61719[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Sensation Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61720[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC Sensation Build/JZO54K)";
const char S61721[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Sensation Build/JZO54K)";
const char S61722[] = "Mozilla/5.0 (Windows NT 4.10; rv:15.0) Gecko/X Firefox/15.0.1";
const char S61723[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S61724[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8005 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S61725[] = "Mozilla/5.0 (Linux; Android 4.3; Galaxy Nexus Build/JWR66V) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61726[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.4.11; U; en) Presto/2.8.131 Version/11.10";
const char S61727[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.4.11; U; en) Presto/2.8.131 Version/11.11";
const char S61728[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lenovo A660 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61729[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; a660 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61730[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; Lenovo A660; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61731[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo A660 Build/IMM76D)";
const char S61732[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S61733[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S61734[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S61735[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire SV Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S61736[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGP312 Build/10.1.C.0.370) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Safari/537.36";
const char S61737[] = "LGE-VN270L/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S61738[] = "LGE-VN270P/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S61739[] = "LGE-VN270/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S61740[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A403 Safari/8536.25";
const char S61741[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S61742[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S61743[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S61744[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B143 Safari/8536.25";
const char S61745[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S61746[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S61747[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P3110 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S61748[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P3110 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S61749[] = "Mozilla/5.0 (Linux; Android 4.2; GT-P3110 Build/JOP40C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S61750[] = "Mozilla/5.0 (Linux; Android 4.0.4; N8000_USA_Cricket Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S61751[] = "Mozilla/5.0 (Linux; Android 4.0.4; V8000_USA_Cricket Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S61752[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus 4 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S61753[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML, like Gecko) Iron/12.0.750.0 Chrome/12.0.750.0 Safari/534.30";
const char S61754[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Iron/12.0.750.0 Chrome/12.0.750.0 Safari/534.30";
const char S61755[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.30 (KHTML, like Gecko) Iron/12.0.750.0 Chrome/12.0.750.0 Safari/534.30";
const char S61756[] = "Mozilla/5.0 (Windows NT 4.10) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S61757[] = "Opera/9.80 (Android 4.0.4; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S61758[] = "Opera/9.80 (Android 4.0.3; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S61759[] = "Opera/9.80 (Android 4.0.3; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10,gzip(gfe)";
const char S61760[] = "Opera/9.80 (Android 4.0.4; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10,gzip(gfe)";
const char S61761[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S61762[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9300 Build/_miui.polimorph_ v5.c) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S61763[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; GT-I9300 Build/miui.es JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S61764[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9300 Build/By Rohit) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S61765[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, zoals Gecko) Versie/4.0 MobielSafari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S61766[] = "NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac1-01) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S61767[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SHV-E300K Build/JDQ39)";
const char S61768[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ko-kr; SHV-E300K Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 NAVER(inapp; search; 230; 4.7.7)";
const char S61769[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S61770[] = "SAMSUNG-GT-C3530/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S61771[] = "Mozilla/5.0 (Linux; U; Android 2.3;en-au; ZTE T760 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61772[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; ZTE T760 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61773[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5; U; Edition MAS; en) Presto/2.10.229 Version/11.66";
const char S61774[] = "Nokia2700c-2/2.0 (09.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia2700c-2) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S61775[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCL21 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61776[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCL21 Build/IMM76D)";
const char S61777[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-02E Build/10.1.D.0.322) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S61778[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6602 Build/10.1.A.1.350) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S61779[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6602 Build/10.1.A.1.434) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S61780[] = "Mozilla/5.0 (MSIE 9.0; Windows NT 6.3; WOW64; Trident/7.0; rv:11.0) like Gecko";
const char S61781[] = "Mozilla/5.0 (MSIE 9.0; Windows NT 6.3; Win64; x64; Trident/7.0; rv:11.0) like Gecko";
const char S61782[] = "Mozilla/5.0 (MSIE 9.0; Windows NT 6.3; Trident/7.0; rv:11.0) like Gecko";
const char S61783[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.3; WOW64; Trident/7.0)";
const char S61784[] = "NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-00) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S61785[] = "SAMSUNG-GT-S3770K/S3770KDDKK2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S61786[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGPT13 Build/TJS0166) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S61787[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGPT12 Build/TJS0166) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S61788[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S61789[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT8088 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61790[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S61791[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S61792[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I727R Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61793[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E110S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61794[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP9090 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S61795[] = "Mozilla/5.0 (Linux; Android 4.1.1; MI 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S61796[] = "Mozilla/5.0 (Linux; Android 4.1.1; MI 2A Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S61797[] = "LG-KS360-Orange/V10g Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S61798[] = "LG-KS360 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S61799[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.8.1.7pre) Gecko/X Firefox/2.0.0.6 Navigator/9.0b3";
const char S61800[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.11pre) Gecko/X Firefox/2.0.0.11 Navigator/9.0.0.5 GTB5";
const char S61801[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS950 4G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61802[] = "Mozilla/5.0 (PlayStation Vita 2.00) AppleWebKit/536.26 (KHTML, like Gecko) Silk/3.2";
const char S61803[] = "Mozilla/4.0 (Brew MP 1.0.4; U; en-us; Kyocera; NetFront/4.1/AMB) Sprint S2151-BST";
const char S61804[] = "Mozilla/4.0 (Brew MP 1.0.4; U; en-us; Kyocera; NetFront/4.1/AMB) Sprint S2151-PLB";
const char S61805[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Motorola-XT5 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S61806[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Motorola-XT502 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S61807[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; Motorola-XT500 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S61808[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Motorola-XT502 Build/ERE27)";
const char S61809[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; Motorola-XT502 Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S61810[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P500 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61811[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P500 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61812[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P500 Build/IMM76L; CyanogenMod-ICySnap) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61813[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P500 Build/NightlyFourE; CyanogenMod-9.1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61814[] = "Nokia308/2.0 (05.85) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia308) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S61815[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZP950 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61816[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZP950+ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61817[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZP950H Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61818[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I777 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S61819[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I777 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S61820[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I727 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61821[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I727 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61822[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959V Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61823[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959V Build/RemICS-UX; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61824[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S61825[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SPH-D700 Build/ECLAIR)";
const char S61826[] = "MyPhoneS21/MTK Release/V0.6 Mar-06-2008 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S61827[] = "MyPhoneB22Duo/MTK Release/V0.44 Apr-01-2008 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S61828[] = "MyPhoneTV21Duo/MTK Release/V6 Jan-12-2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S61829[] = "MyPhoneB23DuoAM/MTK Release/V0.1 Aug-25-2008 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S61830[] = "MyPhoneMS28Duo/MTK Release/V7 May-04-2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S61831[] = "MyPhoneB21AM/MTK Release/V0.3 Aug-23-2008 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S61832[] = "MyPhoneQ21Duo/MTK Release/V7 Apl-01-2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S61833[] = "MyPhoneQ19iDuo/MTK Release/V3 AUG-3-2010 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S61834[] = "MyphoneTW1DUO/MTKRelease/2011/02/25Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0";
const char S61835[] = "MyphoneTW8DUO/MTKRelease/2011/03/03Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0";
const char S61836[] = "MyphoneQW29 DUO/MTKRelease/MAY-27-2011Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0";
const char S61837[] = "MyphoneQP29Duo/MTKRelease/NOV-11-2010Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0";
const char S61838[] = "MyPhoneQTV20iDuo/MTK Release/V1.8 JULY-22-2010 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S61839[] = "MyphoneT18TV/MTKRelease/2011/09/19Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0";
const char S61840[] = "MyphoneQW29 DUO/MTKRelease/DEC-18-2010Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0";
const char S61841[] = "MyphoneT23 Duo/MTKRelease/JULY-08-2010Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0";
const char S61842[] = "MyphoneTW28Duo/MTK Release/Feb-14-2012Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0";
const char S61843[] = "MyPhoneB12 Duo/MTKRelease/V2.0 MAY-14-2010Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0";
const char S61844[] = "MyPhoneT28TVDuo/MTKRelease/2011/09/29Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0";
const char S61845[] = "MyphoneTW1Duo/MTKRelease/2011/11/24Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0";
const char S61846[] = "MyphoneTW29TVDuo/MTK Release/Feb-20-2012Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0";
const char S61847[] = "MyPhoneQTV20Duo/MTK Release/V1.8 JULY-22-2010 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S61848[] = "MyPhoneQTV20iDuo/MTK Release/V7 AUG-25-2011 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S61849[] = "MyPhoneB16TV Duo/MTKRelease/V0.2 AUG-22-2011Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0";
const char S61850[] = "MyPhoneB16TV Duo/MTKRelease/V4 AUG-26-2011Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0";
const char S61851[] = "MyphoneQW30Duo/MTKRelease/FEB-22-2011Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0";
const char S61852[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KB901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61853[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; KB901 Build/IMM76D)";
const char S61854[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KB901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61855[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61856[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562L Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61857[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; ARM; Touch; WPDesktop) UCBrowser/3.0.0.285";
const char S61858[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; ARM; Touch; WPDesktop) UCBrowser/3.0.1.302";
const char S61859[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ601 Build/I.7.1-45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S61860[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I777 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S61861[] = "Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I777 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S61862[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I777 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S61863[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-I777 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S61864[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LS720 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S61865[] = "BlackBerry8700/4.1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S61866[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-I535 Build/JOP40G; Carbon-Carbon-JB-v1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61867[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I535 Build/JDQ39; Carbon-Carbon-JB-v1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61868[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I535 Build/JDQ39; Carbon-CARBON-JB-v1.6.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61869[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/JDQ39; Carbon-CARBON-JB-v1.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61870[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T999 Build/JDQ39; Carbon-CARBON-JB-v1.6.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61871[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I535 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61872[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T999 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61873[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61874[] = "TECNO D5 Linux/3.0.13 Android/4.0.4 Release/01.28.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S61875[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TECNO D5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61876[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; TECNO D5 Build/IMM76D)";
const char S61877[] = "Mozilla/5.0 (Linux; Android 4.1.2; C2105 Build/15.0.A.1.31) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61878[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-GB; rv:1.7.6) Gecko/X Firefox/1.0.1";
const char S61879[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; SCH-R880 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S61880[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SCH-R880 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S61881[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SCH-R880 Build/ECLAIR)";
const char S61882[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S61883[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; T3020 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S61884[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; T3020 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S61885[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; T3020 Build/ERE27)";
const char S61886[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaN73) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile";
const char S61887[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID9042 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61888[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HUAWEI P2-6011 Build/HuaweiP2-6011) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61889[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HUAWEI P2-6011 Build/HuaweiP2-6011)";
const char S61890[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; HUAWEI P2-6011-orange Build/hwP2-6011C109B007) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61891[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S61892[] = "Mozilla/5.0 (Linux; Android 4.1.1; Arc Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S61893[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; Micromax A50 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61894[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; Micromax A50 Build/GRK39F)";
const char S61895[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID8065 Build/ICS.MID8065.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61896[] = "SAMSUNG-GT-S5511T/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 S5511AJKD4";
const char S61897[] = "SAMSUNG-GT-S5511/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 S5511JJKI1";
const char S61898[] = "NokiaE63/UC Browser8.0.4.121/28/444/UCWEB";
const char S61899[] = "NokiaE63/UCWEB8.0.3.99/28/999";
const char S61900[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S61901[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S61902[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919V Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S61903[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; T9295)";
const char S61904[] = "LG-P692/Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; LG-P692 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S61905[] = "Mozilla/5.0 (Windows NT 5.1; rv:17.0) Gecko/X Firefox/17.0 IceDragon/18.0.1.0";
const char S61906[] = "Mozilla/5.0 (Windows NT 5.1; rv:18.0) Gecko/X Firefox/18.0 IceDragon/18.0.3.1";
const char S61907[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17";
const char S61908[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17";
const char S61909[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:13.0) Gecko/X Thunderbird/13.0.1";
const char S61910[] = "Mozilla/5.0 (Linux; U; Android 4.0.6; en-us; A9550 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61911[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; A9550 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61912[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.6; A9550 Build/IML74K)";
const char S61913[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN7G2I Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61914[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; ALCATEL ONE TOUCH 6010D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S61915[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ALCATEL ONE TOUCH 6010D Build/JRO03C)";
const char S61916[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9505G Build/JWR66N.S005) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61917[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9505G Build/JWR66V.S001) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61918[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9505G Build/JWR66V.S014) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61919[] = "Dalvik/1.6.0 (Linux; U; Android 4.3; GT-I9505G Build/JWR66V.S014)";
const char S61920[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9505G Build/JWR66Y.S003) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S61921[] = "Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6545; Pro)";
const char S61922[] = "Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.4518; Pro)";
const char S61923[] = "Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6425; Pro)";
const char S61924[] = "Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6554; Pro)";
const char S61925[] = "Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6562; Pro)";
const char S61926[] = "Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6535; Pro)";
const char S61927[] = "Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6529; Pro)";
const char S61928[] = "Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6213; Pro)";
const char S61929[] = "Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6607; Pro)";
const char S61930[] = "Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6320; Pro)";
const char S61931[] = "Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6662; Pro)";
const char S61932[] = "Microsoft Office/12.0 (Windows NT 5.2; Microsoft Office Outlook 12.0.6662; Pro)";
const char S61933[] = "Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6020; Pro)";
const char S61934[] = "Microsoft Office/12.0 (Windows NT 5.2; Microsoft Office Outlook 12.0.6607; Pro)";
const char S61935[] = "Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6683; Pro)";
const char S61936[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.1 Safari/532.0";
const char S61937[] = "Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.198.0 Safari/532.0";
const char S61938[] = "Mozilla/5.0 (Linux; U; LeOS1.0; zh-cn; lepad_001b Build/PQXU100.4.0091.X) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.33 Safari/532.0";
const char S61939[] = "Mozilla/5.0 (Linux; U; LeOS1.0; zh-cn; lepad_001b Build/PQXU100.4.0097.X) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.33 Safari/532.0";
const char S61940[] = "Mozilla/5.0 (Linux; U; LeOS1.0; zh-cn; lepad_001n Build/PQXU100.4.0073.X) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.33 Safari/532.0";
const char S61941[] = "Mozilla/5.0 (Linux; Android 4.0.3; SoftwinerEvb Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S61942[] = "Mozilla/5.0 (Linux; Android 4.0.4; SoftwinerEvb Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61943[] = "Mozilla/5.0 (Linux; Android 4.0.4; SoftwinerEvb Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S61944[] = "Mozilla/5.0 (Linux; Android 4.0.3; SoftwinerEvb Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61945[] = "Mozilla/5.0 (Linux; Android 4.0.4; Softwinerf900 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61946[] = "Mozilla/5.0 (Linux; Android 4.0.4; Softwinerf721 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61947[] = "Mozilla/5.0 (Linux; Android 4.0.4; Softwinerf761 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S61948[] = "Mozilla/5.0 (Linux; Android 4.0.3; Softwinerq702 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S61949[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-M950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S61950[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; u8820 Build/HuaweiU8820) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61951[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; U8820 Build/HuaweiU8820) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61952[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; U8820 Build/HuaweiU8820)";
const char S61953[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; CT1002 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S61954[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE63)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S61955[] = "NokiaE63/UCWEB8.5.0.163/28/999";
const char S61956[] = "Mozilla/5.0 (Java; U; en-us; nokiae63-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile";
const char S61957[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100P Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61958[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100P Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S61959[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP9070 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S61960[] = "BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 90005; Lang: zhcn) ucweb-squid";
const char S61961[] = "BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 2497; Lang: zhcn) ucweb-squid";
const char S61962[] = "BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 39054; Lang: zhcn) ucweb-squid";
const char S61963[] = "BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 600; Lang: zhcn; Profile/MIDP-2.0_Configuration/CLDC-1.1) ucweb-squid";
const char S61964[] = "Nokia2700c-2/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2700c-2) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S61965[] = "Nokia2700c/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2700c) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S61966[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; SHV-E230S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61967[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; SHV-E230K Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61968[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; SHV-E230L Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61969[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E230K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61970[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E230S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61971[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E230L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61972[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E230K/KKVME2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61973[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E230K/KKVMF2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61974[] = "Mozilla/5.0 (Linux; Android 4.1.2; PadFone Infinity Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S61975[] = "Mozilla/5.0 (Linux; Android 4.1.2; PadFone Infinity Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S61976[] = "Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 10 LINK Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S61977[] = "Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 7 Vogue Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S61978[] = "Mozilla/5.0 (Linux; Android 4.1.2; Telpad Dual S Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S61979[] = "Mozilla/5.0 (Linux; Android 4.1.2; dtab01 Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S61980[] = "Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 7 Youth Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S61981[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8J2 Safari/7534.48.3";
const char S61982[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8G4 Safari/7534.48.3";
const char S61983[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8F190 Safari/7534.48.3";
const char S61984[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8L1 Safari/7534.48.3";
const char S61985[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8H7 Safari/7534.48.3";
const char S61986[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8K2 Safari/7534.48.3";
const char S61987[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 435 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/19.0.1084.60 Mobile/8L1 Safari/7534.48.3";
const char S61988[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 433 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/19.0.1084.60 Mobile/8J2 Safari/7534.48.3";
const char S61989[] = "SAMSUNG-GT-I9308_TD/1.0 Android/4.0.3 Release/02.15.2012 Browser/AppleWebKit534.30";
const char S61990[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP9070 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S61991[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9205 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S61992[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9205 Build/JDQ39)";
const char S61993[] = "Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S61994[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S61995[] = "Mozilla/5.0 (Windows; N; ; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/7.0.3.8542 Safari/532.4";
const char S61996[] = "Nokia302/5.0 (14.78) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia302) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S61997[] = "Mozilla/5.0 (Linux; Android 4.0.3; A200 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S61998[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; ZTE; Tania)";
const char S61999[] = "SAMSUNG-S8003/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S62000[] = "SAMSUNG-GT-S8003/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S62001[] = "AppleCoreMedia/1.0.0.9A334 (iPhone; U; CPU OS 5_0 like Mac OS X; en_us)";
const char S62002[] = "AppleCoreMedia/1.0.0.9A405 (iPhone; U; CPU OS 5_0_1 like Mac OS X; en_us)";
const char S62003[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Iron/8.0.555.0 Chrome/8.0.555.0 Safari/534.10";
const char S62004[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Iron/8.0.555.0 Chrome/8.0.555.0 Safari/534.10";
const char S62005[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Microsoft; XDeviceEmulator)";
const char S62006[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Microsoft; CEPC)";
const char S62007[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaC5-05) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S62008[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE C1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62009[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE C1 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S62010[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE C1 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S62011[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; MI-ONE C1 Build/GINGERBREAD)";
const char S62012[] = "Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE C1 Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S62013[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.1 Safari/535.19 Silk-Accelerated=true";
const char S62014[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Novo7Flame Build/X.qm) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62015[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Novo7Flame Build/X.tm) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62016[] = "Nokia5220XpressMusic/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S62017[] = "Nokia5220/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S62018[] = "Opera/9.80 (Windows NT 6.2; U; en) Presto/2.7.62 Version/11.01";
const char S62019[] = "Opera/9.80 (Windows NT 6.2; U; en) Presto/2.7.62 Version/11.00";
const char S62020[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R811 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62021[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-Th; OPPOR8111 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62022[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R811 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S62023[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; R8111 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62024[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; R811 Build/IMM76D)";
const char S62025[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; R8111 Build/IMM76D)";
const char S62026[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R811W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62027[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-T989 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S62028[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T989 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S62029[] = "Android/4.0.4-EAS-1.3";
const char S62030[] = "Android/4.0.3-EAS-1.3";
const char S62031[] = "Windowshop/1.50.03 (Android/4.0.3/KFTT)";
const char S62032[] = "Windowshop/1.51.09 (Android/4.0.3/KFTT)";
const char S62033[] = "Windowshop/1.51.09 (Android/4.0.3/KFOT)";
const char S62034[] = "Windowshop/1.51.06 (Android/4.0.4/KFJWI)";
const char S62035[] = "Windowshop/1.51.42 (Android/4.0.3/KFTT)";
const char S62036[] = "Windowshop/1.51.43 (Android/4.0.3/KFTT)";
const char S62037[] = "Windowshop/1.51.43 (Android/4.0.3/KFOT)";
const char S62038[] = "Xtreme Labs Android HTTP Framework on device with Android OS 4.0.4";
const char S62039[] = "Windowshop/1.51.46 (Android/4.0.3/KFTT)";
const char S62040[] = "Windowshop/1.51.46 (Android/4.0.3/KFOT)";
const char S62041[] = "Windowshop/1.51.61 (Android/4.0.3/KFTT)";
const char S62042[] = "Windowshop/1.51.61 (Android/4.0.3/KFOT)";
const char S62043[] = "Windowshop/1.51.62 (Android/4.0.3/KFTT)";
const char S62044[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62045[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62046[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62047[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:16.0; Avant TriCore) Gecko/X Firefox/16.0";
const char S62048[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:17.0; Avant TriCore) Gecko/X Firefox/17.0";
const char S62049[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:18.0; Avant TriCore) Gecko/X Firefox/18.0";
const char S62050[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:19.0; Avant TriCore) Gecko/X Firefox/19.0";
const char S62051[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:20.0; Avant TriCore) Gecko/X Firefox/20.0";
const char S62052[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:21.0; Avant TriCore) Gecko/X Firefox/21.0";
const char S62053[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:22.0; Avant TriCore) Gecko/X Firefox/22.0";
const char S62054[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:23.0; Avant TriCore) Gecko/X Firefox/23.0";
const char S62055[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; i940 Build/ECLAIR) AppleWebKit/530.17+ (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S62056[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; i940 Build/ESE81) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S62057[] = "Dalvik/1.2.0 (Linux; U; Android 2.1-update1; i940 Build/ESE81)";
const char S62058[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62059[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315b Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62060[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I727 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S62061[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I727 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S62062[] = "Mozilla/5.0 (Windows; U; WinNT4.0; en-US; rv:1.7.9) Gecko/X Firefox/1.0.5";
const char S62063[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; POV_TAB-PROTAB30-IPS10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62064[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; POV_TAB-PROTAB30-IPS10 Build/JRO03H)";
const char S62065[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BLU DASH 4.0 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62066[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Dash 4.0 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62067[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-S5360 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62068[] = "Mozilla/5.0 (Series40; Nokia114/03.33; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.5.16.9";
const char S62069[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Iron/7.0.520.1 Chrome/7.0.520.1 Safari/534.7";
const char S62070[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Iron/7.0.520.1 Chrome/7.0.520.1 Safari/534.7";
const char S62071[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Iron/7.0.520.1 Safari/534.7";
const char S62072[] = "NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiax2-00) UCBrowser8.4.0.159/70/444/UCWEB Mobile";
const char S62073[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; X507 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62074[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; X507S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62075[] = "SonyEricssonT700/R3EA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.3";
const char S62076[] = "Opera/9.80 (Android 2.3.7; Linux; Opera Mobi/ADR-X; U; en-GB) Presto/2.8.149 Version/11.10";
const char S62077[] = "Opera/9.80 (Android 2.3.5; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.8.149 Version/11.10";
const char S62078[] = "Opera/9.80 (Android 2.3.3; Linux; Opera Mobi/ADR-X; U; it) Presto/2.8.149 Version/11.10";
const char S62079[] = "Opera/9.80 (Android 2.3.2; Linux; Opera Mobi/ADR-X; U; en-GB) Presto/2.8.149 Version/11.10";
const char S62080[] = "Opera/9.80 (Android 2.3.4; Linux; Opera Mobi/build-X; U; en-GB) Presto/2.8.149 Version/11.10";
const char S62081[] = "Opera/9.80 (Android 2.3.6; Linux; Opera Mobi/ADY-X; U; en) Presto/2.8.149 Version/11.10";
const char S62082[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 TeamStream (iPhone 199 1.6)";
const char S62083[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 TeamStream (iPhone 199 1.6)";
const char S62084[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3 TeamStream (iPhone4,1 215 1.6.1)";
const char S62085[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3 TeamStream (iPhone3,1 215 1.6.1)";
const char S62086[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 TeamStream (iPhone4,1 267 2.1)";
const char S62087[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 TeamStream (iPhone 199 1.6)";
const char S62088[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3 TeamStream (iPhone3,1 293 2.2.2)";
const char S62089[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19";
const char S62090[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19";
const char S62091[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S62092[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S7562 Build/IMM76I) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S62093[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-in; Micromax A116 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62094[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; Micromax A116 Build/JOP40D)";
const char S62095[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Micromax A116 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62096[] = "Nokia5800 XpressMusic/UCWEB8.5.0.163/50/999";
const char S62097[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5800 XpressMusic)/UC Browser8.5.0.183/50/355/UCWEB Mobile";
const char S62098[] = "Mozilla/5.0 (Java; U; en-us; nokia5800d-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile  3gpp-gba";
const char S62099[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.4.3 Safari/534.34";
const char S62100[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.21 (KHTML, like Gecko) QupZilla/1.4.3 Safari/537.21";
const char S62101[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S62102[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S62103[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S62104[] = "Mozilla/5.0 (Linux; Android 4.1.2; SM-T211 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S62105[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SC-01E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62106[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SC-01E Build/IMM76D)";
const char S62107[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Infectious Rezound Build/IML74K; CyanogenMod-INFECTION) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62108[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Rezound on TRON-2.0 Build/IML74K; CyanogenMod-A product of the Umbrella Corporation) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62109[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Infected HTC Rezound Build/IML74K; CyanogenMod-INFECTION 2.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62110[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Rezound on TRON 2.1 Build/IML74K; CyanogenMod-By IAmTheOneTheyCallNeo) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62111[] = "Mozilla/5.0 (Linux; U; Android 4.2.2 Jelly Bean; en-us; Rezound on TRON Build/IML74K; CyanogenMod-IAmTheOneTheyCallNeo) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62112[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Infected Rezound Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/INFECTION 2.9.1/vigor";
const char S62113[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6400L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S62114[] = "Mozilla/5.0 (Linux; Android 4.0.4; Thunderbolt Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S62115[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT919 Build/2_290_2012) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S62116[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; fr; rv:1.8.0.12) Gecko/X Firefox/1.5.0.12";
const char S62117[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; es-ES; rv:1.8.0.3) Gecko/X Firefox/1.5.0.3";
const char S62118[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xenta TAB9-200 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62119[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAB9-200 Build/Xenta) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62120[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GOCLEVER TAB T76 Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62121[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; GOCLEVER TAB T76 Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62122[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S62123[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2S Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S62124[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2SC Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S62125[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2C Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S62126[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2A Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S62127[] = "Mozilla/5.0 (Series40; Nokia206/03.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S62128[] = "Mozilla/5.0 (Series40; Nokia206.1/03.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S62129[] = "SonyEricssonT715a/R1GA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.4";
const char S62130[] = "SonyEricssonT715/R1HA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.5";
const char S62131[] = "HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; es-ES)";
const char S62132[] = "HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; pt)";
const char S62133[] = "HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; el)";
const char S62134[] = "HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; en)";
const char S62135[] = "HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; zh-tw)";
const char S62136[] = "HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; fr)";
const char S62137[] = "HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; nl)";
const char S62138[] = "HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; ru)";
const char S62139[] = "HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; de)";
const char S62140[] = "HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; it)";
const char S62141[] = "HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; nb)";
const char S62142[] = "HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; sv)";
const char S62143[] = "HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; da)";
const char S62144[] = "HTC_Touch3G_T3333 Opera/9.50 (Windows NT 5.1; U; en)";
const char S62145[] = "Vodafone/1.0/HTC_Touch2/1.29.176.2 (67024) Opera/9.50 (Windows NT 5.1; U; es-ES)";
const char S62146[] = "Opera/9.50 (Windows NT 5.1; HTC_Touch2/1.0; U; en)";
const char S62147[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Twitter for iPhone";
const char S62148[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Twitter for iPhone";
const char S62149[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-L710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, us";
const char S62150[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, us";
const char S62151[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I747M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, ca";
const char S62152[] = "Nokia205/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S62153[] = "Nokia205.1/2.0 (03.19) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S62154[] = "Nokia2055/2.0 (03.20) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S62155[] = "Nokia205.3/2.0 (03.19) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S62156[] = "SonyEricssonW350iv/R10BA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S62157[] = "SonyEricssonW350i/R11CA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S62158[] = "Mozilla/5.0 (Linux; U; Android 2.1-update-1; en-us; Dell_Aero Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari.530.17";
const char S62159[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; Dell Aero Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S62160[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Dell Aero Build/ECLAIR)";
const char S62161[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; ZTE T60 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62162[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; ZTE T60 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62163[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; ZTE T60 Build/GRJ22)";
const char S62164[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAC-97032 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62165[] = "Opera/9.80 (BREW; Opera Mini/6.0.3/27.2338; U; en) Presto/2.8.119 320X240 Pantech TXT8045";
const char S62166[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-A7100 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62167[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; sprd-GT-A7100/1.0 Android/4.0.4 Release/04.07.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62168[] = "Dalvik/1.4.0 (Linux; U; Android 4.0.4; GT-A7100 Build/MocorDroid2.3.5)";
const char S62169[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; sprd-GT-A7100/1.0 Android/4.0.4 Release/04.25.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62170[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; sprd-GT-A7100/1.0 Android/4.0.4 Release/06.15.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62171[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; sprd-GT-A7100/1.0 Android/4.0.4 Release/06.01.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62172[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; sprd-GT-A7100/1.0 Android/4.0.4 Release/`d.be.b`ac Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62173[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; sprd-GT-A7100/1.0 Android/4.0.4 Release/06.28.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62174[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; sprd-GT-A7100/1.0 Android/4.0.4 Release/05.31.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62175[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; sprd-GT-A7100/1.0 Android/4.0.4 Release/07.24.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62176[] = "Mozilla/5.0 (X11; Linux x86_64; X325a; en-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S62177[] = "Mozilla/5.0 (X11; Linux x86_64; X325a; fr-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S62178[] = "Mozilla/5.0 (X11; Linux x86_64; X325a; en-us) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S62179[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT920 Build/2_310_2009) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S62180[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17; Maxthon (4.0.4.1000);";
const char S62181[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.18 (KHTML,like Gecko) Version/3.1.2 Safari/525.20.1; Maxthon (4.0.4.1000);";
const char S62182[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LG730 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S62183[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LG730 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S62184[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62185[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S6800 Build/GINGERBREAD)";
const char S62186[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; PMP7170B3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62187[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMP7170B3GCE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62188[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S62189[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9108 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31";
const char S62190[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S62191[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9100G Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S62192[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S62193[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S62194[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-I9100G Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S62195[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; MB861 Build/4.5.91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62196[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB861 Build/4.5.2A-KT-75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62197[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; MB861 Build/4.5.2A-KT-167) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62198[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; MB861 Build/4.5.2A-KT-75)";
const char S62199[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; MB861 Build/4.5.2A-KT-167)";
const char S62200[] = "Mozilla/5.0 (compatible; Konqueror/4.5; FreeBSD) KHTML/4.5.4 (like Gecko)";
const char S62201[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile";
const char S62202[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile";
const char S62203[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile";
const char S62204[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile";
const char S62205[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fa-fa) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181 Mobile";
const char S62206[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; A78 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62207[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; A78 Build/JRO03H)";
const char S62208[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; A78 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62209[] = "Opera/10.60 (Linux i686 ; U; HbbTV/1.1.1 (;;;;;); CE-HTML; TechniSat MultyVision ISIO; de) Presto/2.6.33 Version/10.60";
const char S62210[] = "Opera/9.80 (Linux sh4; U; HbbTV/1.1.1(;;;;;); CE-HTML; TechniSat DIGIT ISIO S; en) Presto/2.8.115 Version/11.10";
const char S62211[] = "Opera/10.60 (Linux sh4 ; U; HbbTV/1.1.1 (;;;;;); CE-HTML; TechniSat DIGIT ISIO S; de) Presto/2.6.33 Version/10.60";
const char S62212[] = "Opera/9.80 (Linux i686; U; HbbTV/1.1.1 (;;;;;); CE-HTML; TechniSat MultyVision ISIO; de) Presto/2.9.167 Version/11.50";
const char S62213[] = "Opera/10.60 (Linux sh4 ; U; HbbTV/1.1.1 (+PVR; Loewe; SL150; LOH/3.10;;) CE-HTML/1.0 Config(L:deu,CC:DEU); en) Presto/2.6.33 Version/10.60";
const char S62214[] = "Opera/9.80 (Linux sh4; U; HbbTV/1.1.1 (+PVR; Loewe; SL150; LOH/3.10;;) CE-HTML/1.0 Config(L:deu,CC:DEU); en) Presto/2.10.250 Version/11.60";
const char S62215[] = "Opera/9.80 (Linux sh4; U; HbbTV/1.1.1 (;;;;;); CE-HTML; TechniSat Digit ISIO S; de) Presto/2.9.167 Version/11.50";
const char S62216[] = "Opera/9.80 (Linux armv7l; U; HbbTV/1.1.1 (;tv2n;videoweb;1.0.0;1.0;); en) Presto/2.8.115 Version/11.10";
const char S62217[] = "Opera/10.60 (Linux sh4 ; U; HbbTV/1.1.1 (; LOH/2.01; -----;;;) CE-HTML/1.0 Config(L:de,CC:DE); en) Presto/2.6.33 Version/10.60";
const char S62218[] = "Opera/9.80 (Linux sh4; U; HbbTV/1.1.1 (+PVR; Loewe; SL150; LOH/3.10;;) CE-HTML/1.0 Config(L:nld,CC:NLD); en) Presto/2.10.250 Version/11.60";
const char S62219[] = "Opera/9.80 (Linux armv6l; U; NETRANGEMMH;HbbTV/1.1.1;CE-HTML/1.0; en) Presto/2.8.115 Version/11.10";
const char S62220[] = "Opera/9.80 (Linux armv6l; U; NETRANGEMMH;HbbTV/1.1.1;CE-HTML/1.0;THOMSON LF1V394; en) Presto/2.10.250 Version/11.60";
const char S62221[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A306 Twitter for iPhone";
const char S62222[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A293 Twitter for iPhone";
const char S62223[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A400 Twitter for iPhone";
const char S62224[] = "Nokia5800 XpressMusic/UCWEB8.0.0.89/50/351";
const char S62225[] = "Nokia5800 XpressMusic/UC Browser8.0.4.121/50/444/UCWEB";
const char S62226[] = "Nokia5800w XpressMusic/UCWEB8.0.3.99/50/800";
const char S62227[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.22 (KHTML, like Gecko) Iron/25.0.1400.0 Chrome/25.0.1400.0 Safari/537.22";
const char S62228[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Iron/25.0.1400.0 Chrome/25.0.1400.0 Safari/537.22";
const char S62229[] = "Mozilla/5.0 (Windows NT 4.10; rv:21.0) Gecko/X Firefox/21.0";
const char S62230[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S6812 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62231[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; GT-S6812B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62232[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S6812 Build/JZO54K)";
const char S62233[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-S6812i Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62234[] = "Mozilla/5.0 (Series40; Nokia303/14.38; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S62235[] = "SAMSUNG-GT-C3303K/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S62236[] = "SAMSUNG-GT-C3303/C3303DDJG4 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S62237[] = "SAMSUNG-GT-C3303i/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S62238[] = "Opera/9.5 (Microsoft Windows; Windows CE; Opera Mobi/9.5; U; en) 480x800 SAMSUNG SCH-i920 PPC";
const char S62239[] = "Opera/9.5 (Microsoft Windows; Windows CE; OperaMobi/9.5; U; en) 480x800 SAMSUNG SCH-i920 PPC";
const char S62240[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11SH Build/S6160) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62241[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11SH Build/S7150) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62242[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11SH Build/S9081) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62243[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11SH Build/SB280) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62244[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11SH Build/S3085) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62245[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11SH Build/S8270) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62246[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11SH Build/S8030) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62247[] = "Mozilla/5.0 (Series40; NokiaC3-01.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S62248[] = "Mozilla/5.0 (Series40; NokiaC3-01/07.15; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S62249[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 820)";
const char S62250[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 820; Vodafone)";
const char S62251[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 820; ANZ825)";
const char S62252[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 820; 1328-3)";
const char S62253[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Nokia 820)";
const char S62254[] = "Mozilla/5.0 (Linux; Android 4.2.1; V370 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S62255[] = "Mozilla/5.0 (Linux; Android 4.2.2; V370 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S62256[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LS860 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S62257[] = "Mozilla/5.0 (Linux; Android 4.0.4; Mach Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S62258[] = "Mozilla/5.0 (Linux; Android 4.0.4; N9500 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S62259[] = "LG-GS170 Browser/Obigo-Q7.0 MMS/LG-MMS-V1.0/1.2 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S62260[] = "Mozilla/5.0 (X11; Linux i686; rv:11.0) Gecko/X Firefox/11.0 SeaMonkey/2.8";
const char S62261[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Intex Aqua 3.2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62262[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5830i Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S62263[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S62264[] = "YahooFeedSeeker/2.0 (compatible; Mozilla 4.0; MSIE 5.5; http://publisher.yahoo.com/rssguide)";
const char S62265[] = "Y!J-BRO/YFSJ crawler (compatible; Mozilla 4.0; MSIE 5.5; http://help.yahoo.co.jp/help/jp/search/indexing/indexing-15.html; YahooFeedSeekerJp/2.0)";
const char S62266[] = "Y!J-BRO/YFSJ crawler (compatible; Mozilla 4.0; MSIE 5.5; http://help.yahoo.co.jp/help/jp/search/indexing/indexing-15.html; YahooFeedSeekerJp/2.0; users 0; views 28)";
const char S62267[] = "Mozilla/5.0 (Windows; U; ; en-NZ) AppleWebKit/527+ (KHTML, like Gecko, Safari/419.3) Arora/0.8.0";
const char S62268[] = "Mozilla/5.0 (Windows; U; ; cs-CZ) AppleWebKit/528+ (KHTML, like Gecko, Safari/528.0) Lunascape/5.1.2.0";
const char S62269[] = "Mozilla/5.0 (Windows; U; ; cs-CZ) AppleWebKit/532+ (KHTML, like Gecko, Safari/532.0) Lunascape/5.1.2.3";
const char S62270[] = "Mozilla/5.0 (Windows; U; ; en-US) AppleWebKit/533.3 (KHTML, like Gecko) VIDEODOWNLOAD Safari/533.3";
const char S62271[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62272[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S62273[] = "SAMSUNG-SGH-A667TL-parrot/A667TLKA2 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S62274[] = "SAMSUNG-SGH-A667/A667UCJI2 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S62275[] = "SAMSUNG-SGH-A667TL/A667TLKA2 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S62276[] = "Nokia6350/2.0(ATT.12.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 Nokia6350/12.40; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.16803; U; en-US) Opera 9.50";
const char S62277[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS13SH Build/SC100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62278[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS13SH Build/S2150) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62279[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS13SH Build/S3300) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62280[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS13SH Build/SA310) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62281[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS13SH Build/S6291) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62282[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS13SH Build/S9100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62283[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS13SH Build/S3060) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62284[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT912 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/spyder";
const char S62285[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S62286[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S62287[] = "Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S62288[] = "Nokia200/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia200) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S62289[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SCH-I535, 39, 1.8.2)";
const char S62290[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SGH-T999, 39, 1.8.2)";
const char S62291[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I535 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SCH-I535, 42, 1.9.1)";
const char S62292[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (GT-I9300, 42, 1.9.1)";
const char S62293[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T999 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SGH-T999, 47, 1.9.7)";
const char S62294[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ThL W8 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62295[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; ThL W8 Build/JZO54K)";
const char S62296[] = "Nokia311/5.0 (07.36) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia311) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S62297[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E450g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S62298[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E450B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S62299[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E450f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S62300[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E450 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S62301[] = "SAMSUNG-GT-S3353/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S62302[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V1 Build/HCL ME Tablet V1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62303[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CONNECT 2G Build/HCL ME Tablet CONNECT 2G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62304[] = "Opera/9.62 (X11;Linux i686; U; en) Presto/2.2.0/SAMSUNG-GT-I8320-Vodafone/";
const char S62305[] = "SAMSUNG-GT-I8320-Vodafone/I8320BUJC1 Linux/X2/R1 Opera/9.6 SMS-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S62306[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Opera/9.80 (Android; Opera Mini/6.5.28192/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S62307[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26i Build/6.1.A.2.55) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S62308[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-SU660 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62309[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-SU660 Build/GRJ22)";
const char S62310[] = "Mozilla/5.0 (Maemo; Mobile; rv:15.0) Gecko/X Firefox/15.0 Fennec/15.0";
const char S62311[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S62312[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925U Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S62313[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-I925 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S62314[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID4 Build/6.7.2-180_DR4-16_M2-37) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62315[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T769 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S62316[] = "LG-GR700/V10g; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.15814; U; en-US) Opera 9.50";
const char S62317[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SimTab T700A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62318[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ET7006C-P12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62319[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; WEXLER.BOOK_T7008 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62320[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; MT7003 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62321[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-be; CT700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62322[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT7001E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62323[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; T7004 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62324[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; EvoPAD T7000 Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62325[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; MT7005 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62326[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; T7003 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62327[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ET7008C-H12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62328[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.2 Safari/535.19 Silk-Accelerated=false";
const char S62329[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36";
const char S62330[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36";
const char S62331[] = "Mozilla/5.0 (Series40; Nokia110/03.04; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S62332[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.3.1.31155 (Firestorm-Release; firestorm skin) Safari/533.3";
const char S62333[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S62334[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S62335[] = "Nokia5530/UCWEB8.8.0.212/50/999";
const char S62336[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5530) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S62337[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S62338[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5839i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S62339[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5830i Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S62340[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S62341[] = "MQQBrowser/44 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3";
const char S62342[] = "MQQBrowser/44 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3";
const char S62343[] = "MQQBrowser/44 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3";
const char S62344[] = "MQQBrowser/44 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 Safari/7534.48.3";
const char S62345[] = "MQQBrowser/44 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Safari/7534.48.3";
const char S62346[] = "MQQBrowser/44 Mozilla/5.0 (iPhone 4; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Safari/7534.48.3";
const char S62347[] = "Mozilla/5.0 (compatible; Konqueror/4.4; Linux) KHTML/4.4.5 (like Gecko)";
const char S62348[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOMO9 plus III Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62349[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOMO9-3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62350[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MOMO9-3S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62351[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MOMO9-3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62352[] = "Mozilla/5.0 (Linux; Android 4.0.4; CT1010 Build/IMM76I.v007) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S62353[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; ARCHOS 80XSK Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62354[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; ALCATEL ONE TOUCH 6030A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S62355[] = "Mozilla/5.0 (Series40; Nokia306/05.93; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S62356[] = "Mozilla/5.0 (masking-agent; rv:19.0) Gecko/X Firefox/19.0";
const char S62357[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/11A465 Safari/8536.25";
const char S62358[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/11A501 Safari/8536.25";
const char S62359[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; E210 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 ACER_E210/1.006.00";
const char S62360[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; E210 Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1ACER_E210/1.0";
const char S62361[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) 320x240; VZW; UTStar-XV6175.1; Windows Mobile 6.5 Standard;";
const char S62362[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) 320x240; VZW; UTStar-XV6175; Window Mobile 6.1 Standard;";
const char S62363[] = "Mozilla/5.0 (Linux; Android 4.0.3;  Transformer TF101G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S62364[] = "Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S62365[] = "Mercury/7.0.1 CFNetwork/609 Darwin/13.0.0";
const char S62366[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.1.96) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S62367[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.1.96) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S62368[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.4.11; U; en) Presto/2.7.62 Version/11.00";
const char S62369[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.4.11; U; en) Presto/2.7.62 Version/11.01";
const char S62370[] = "Opera/9.80 (Android 3.2.1; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.10.254 Version/12.00";
const char S62371[] = "Opera/9.80 (Android 3.2.6; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00";
const char S62372[] = "Opera/9.80 (Android 3.2; Linux; Opera Tablet/ADY-X; U; ru) Presto/2.10.254 Version/12.00";
const char S62373[] = "Opera/9.80 (Android 3.2.2; Linux; Opera Tablet/ADR-X; U; en-GB) Presto/2.10.254 Version/12.00";
const char S62374[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-in; Micromax A110Q Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62375[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; Micromax A110Q Build/JOP40D)";
const char S62376[] = "Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI MT1-U06 Build/HuaweiMT1-U06) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S62377[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB870 Build/4.5.1A-DTN-150-30; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62378[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:17.0) Gecko/17.0 Firefox/17.0 TenFourFox/7450";
const char S62379[] = "Instagram 3.4.2 (iPhone4,1; iPhone OS 5.1.1; en_US; en) AppleWebKit/420+";
const char S62380[] = "Instagram 3.4.2 (iPhone4,1; iPhone OS 5.1; en_US; en) AppleWebKit/420+";
const char S62381[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo S720 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62382[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo S720 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S62383[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo S720 Build/IMM76D)";
const char S62384[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; Lenovo S720; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62385[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E980 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S62386[] = "Mozilla/5.0 (Series40; Nokia110/03.33; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S62387[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; fr; rv:1.9.0.19) Gecko/X Firefox/3.0.19";
const char S62388[] = "NokiaC2-00/2.0 (03.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S62389[] = "Mozilla/5.0 (masking-agent; rv:17.0) Gecko/X Firefox/17.0";
const char S62390[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; LIFETAB_S9714 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62391[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP9070 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S62392[] = "Opera/9.80 (Android; Opera Mini/444.66/30.3214; U; en) Presto/2.8.119 Version/11.10";
const char S62393[] = "Opera/9.80 (Android; Opera Mini//30.3214; U; en) Presto/2.8.119 Version/11.10";
const char S62394[] = "Opera/9.80 (Android; Opera Mini/1963.55/31.1475; U; en) Presto/2.8.119 Version/11.10";
const char S62395[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.3.0 Safari/534.34";
const char S62396[] = "Mozilla/5.0 (X11; U; Linux x86_64; C) AppleWebKit/533.3 (KHTML, like Gecko) PhantomJS/1.3.0 Safari/533.3";
const char S62397[] = "Mozilla/5.0 (X11; U; Linux; C -) AppleWebKit/532.4 (KHTML, like Gecko) PhantomJS/1.3.0 Safari/532.4";
const char S62398[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P930 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S62399[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P930 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S62400[] = "Mozilla/5.0 (Linux; Android 4.1.2; B1-A71 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S62401[] = "Mozilla/5.0 (Linux; Android 4.1.1; ME172V Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S62402[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 710;7.10;7740)";
const char S62403[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 710;7.10;8107)";
const char S62404[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 710;7.10;8773)";
const char S62405[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 710;7.10;8779)";
const char S62406[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 710;7.10;8112)";
const char S62407[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae71-1) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S62408[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE71) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S62409[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE71x) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S62410[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S62411[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S62412[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S62413[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S62414[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S62415[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA6120c) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile";
const char S62416[] = "Mozilla/5.0 (Linux; Android 4.2; Amazon Kindle Fire Build/JOP40C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S62417[] = "Mozilla/5.0 (Linux; Android 4.2.1; Amazon Kindle Fire Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S62418[] = "Mozilla/5.0 (Linux; Android 4.2.1; Amazon Kindle Fire2 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S62419[] = "Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S62420[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I717 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62421[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I717 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62422[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I717 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62423[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SAMSUNG-SGH-I717 Build/JDQ39E)";
const char S62424[] = "Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Safari/535.19 Silk-Accelerated=false";
const char S62425[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S62426[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Iron/3.0.197.0 Safari/532.0";
const char S62427[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Iron/3.0.197.0 Safari/532.0";
const char S62428[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/531.0 (KHTML, like Gecko) Iron/3.0.189.0 Safari/531.0";
const char S62429[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/0.0.0 Safari/532.0 Iron/3.0.197.0";
const char S62430[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1) AppleWebKit/532.0 (KHTML, like Gecko) Iron/3.0.197.0 Safari/532.0";
const char S62431[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.13) Gecko Fedora/1.9.2.13-3.el5";
const char S62432[] = "Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US; rv:1.9.2.18) Gecko Fedora/1.9.2.18-1.fc14";
const char S62433[] = "Mozilla/5.0 (Linux; Android 4.2.2; One X Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62434[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One X Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62435[] = "Mozilla/5.0 (Linux; Android 4.2.1; HTC One X Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62436[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; MOT-XT390 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62437[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT390 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S62438[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; XT390 Build/07.15.04R_D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62439[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT390 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62440[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-uk; @TAB-NINE-X2-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62441[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; @TAB-NINE-X2-B Build/JRO03H)";
const char S62442[] = "Mozilla/5.0 (Linux; Android 4.0.4; Rise Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S62443[] = "Mozilla/5.0 (Linux; Android 4.0.4; C5155 Build/IML77) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S62444[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612g Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S62445[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E610v Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S62446[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612f Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S62447[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S62448[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E610 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S62449[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us;T-Mobile G2 Build/FRF91) AppleWebKit/533.1(KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62450[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; T-MobileG2 Build/FRF91) AppleWebKit/533.1(KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62451[] = "Mozilla/5.0 (Linux; U;Android 2.2; en-us; T-Mobile G2 Build/FRF91) AppleWebKit/533.1(KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62452[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; T-Mobile G2 Build/FRF91) AppleWebKit/533.1(KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62453[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; T-Mobile G2 Build/FRF91)";
const char S62454[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile G2 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62455[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7105 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62456[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7100 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62457[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L900 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62458[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7105 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S62459[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-N7100 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62460[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Note II Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62461[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919V Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62462[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919V Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S62463[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS920 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S62464[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.17 Mobile Safari/537.36";
const char S62465[] = "Mozilla/5.0 (Windows NT 5.1; rv:12.0) Gecko/X Firefox/12.0 SeaMonkey/2.9.1";
const char S62466[] = "Mozilla/5.0 (Windows NT 5.1; rv:12.0) Gecko/X SeaMonkey/2.9.1";
const char S62467[] = "Mozilla/5.0 (Windows NT 5.2; rv:12.0) Gecko/X Firefox/12.0 SeaMonkey/2.9.1";
const char S62468[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaC5-00)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S62469[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaC5-00.2)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S62470[] = "Mozilla/5.0 (Java; U; en-us; nokiac5-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile  3gpp-gba";
const char S62471[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; E860 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62472[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; E860 Build/IML74K)";
const char S62473[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; M-MP875S2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62474[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:18.0) Gecko/X Firefox/18.0 SeaMonkey/2.15.2";
const char S62475[] = "HTC_Touch_Pro2_T7381 Opera/9.50 (Windows NT 5.1; U; en)";
const char S62476[] = "PPC; 480x800; HTC_Touch_Pro2_T7385-Orange; OpVer 94.119.2.733 Opera/9.5 (Microsoft Windows 6.1; Opera Mobi/16071; U; fr)";
const char S62477[] = "Opera/9.5(Microsoft Windows; PPC; Opera Mobi/16643; U; en) HTC_Touch_Pro2_T7379";
const char S62478[] = "HTC_Touch_Pro2_T7373 Opera/9.50 (Windows NT 5.1; U; de)";
const char S62479[] = "HTC_Touch_Pro2_T7381 Opera/9.50 (Windows NT 5.1; U; en) V1.19.841.3 (51489)";
const char S62480[] = "HTC_Touch_Pro2_T7381 Opera/9.50 (Windows NT 5.1; U; en) V1.93.841.0 (81210)";
const char S62481[] = "PPC; 480x800; HTC_Touch_Pro2_T7385-Orange; OpVer 122.190.2.731 Opera/9.5 (Microsoft Windows 6.5; Opera Mobi/17518; U; fr)";
const char S62482[] = "Swisscom/1.0/HTC_Touch_Pro2/ Opera/9.50 (Windows NT 5.1; U; en)";
const char S62483[] = "Swisscom/1.0/HTC_Touch_Pro2/ Opera/9.50 (Windows NT 5.1; U; de)";
const char S62484[] = "HTC_Touch_Pro2_T7373 Opera/9.50 (Windows NT 5.1; U; en)";
const char S62485[] = "Vodafone/1.0/HTC_Touch_Pro2/1.86.161.1 (67102) Opera/9.50 (Windows NT 5.1; U; en)";
const char S62486[] = "HTC_Touch_Pro2_T7373 Opera/9.50 (Windows NT 5.1; U; nl)";
const char S62487[] = "Vodafone/1.0/HTC_Touch_Pro2/1.86.161.0 (67102) Opera/9.50 (Windows NT 5.1; U; en)";
const char S62488[] = "Vodafone/1.0/HTC_Touch_Pro2/1.14.161.6 (50207) Opera/9.50 (Windows NT 5.1; U; en)";
const char S62489[] = "HTC_Touch_Pro2_T7373 Opera/9.50 (Windows NT 5.1; U; nb)";
const char S62490[] = "HTC_Touch_Pro2_T7373 Opera/9.50 (Windows NT 5.1; U; it)";
const char S62491[] = "Vodafone/1.0/HTC_Touch_Pro2/1.14.161.5 (50207) Opera/9.50 (Windows NT 5.1; U; en)";
const char S62492[] = "HTC_Touch_Pro2_T7373 Opera/9.50 (Windows NT 5.1; U; fr)";
const char S62493[] = "LG-CU920/V1.0s Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S62494[] = "Mozilla/5.0 (compatible; Konqueror/2.2.2)";
const char S62495[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; PocketBook A10 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62496[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PocketBook A10 Build/GRJ90) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62497[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7/X (like Mobile/8C148 Safari/6533.18.5)";
const char S62498[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)";
const char S62499[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)";
const char S62500[] = "Mozilla/5.0 (Linux; Android 4.0.4; N861 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S62501[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E971 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S62502[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C6606 Build/10.1.1.B.0.166) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62503[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; C6606 Build/10.1.1.B.0.166)";
const char S62504[] = "SonyC6606 Build/10.1.1.B.0.166 stagefright/1.2 (Linux;Android 4.1.2)";
const char S62505[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/X1i/1.03.175.7";
const char S62506[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/X1i/1.05.175.13";
const char S62507[] = "SonyEricssonX1i/R3AA Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SonyEricssonX1i/R3AA Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S62508[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Swisscom/1.0/X1i/R3AA";
const char S62509[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/X1i/1.02.175.3";
const char S62510[] = "X1i Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S62511[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Swisscom/1.0/X1i/1.02.166.7";
const char S62512[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SonyEricssonX1i/R3AA Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S62513[] = "Vodafone/1.0/SEX1i/R3AA Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) UP.Link/6.3.1.20.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S62514[] = "Vodafone/1.0/SEX1i/R1AA Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)  Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S62515[] = "Mozilla/5.0 (Linux; Android 4.0.3; U8860 Build/HuaweiU8860) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S62516[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; CINK PEAX Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62517[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; Cink Peax Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62518[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; CINK PEAX Build/JRO03C)";
const char S62519[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; CINK PEAX 2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62520[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729) Sleipnir/2.8.3";
const char S62521[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Sleipnir/2.8.1";
const char S62522[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727) Sleipnir/2.8.4";
const char S62523[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.8.5)";
const char S62524[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022) Sleipnir/2.8.4";
const char S62525[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648) Sleipnir/2.8.0";
const char S62526[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729) Sleipnir/2.8.4";
const char S62527[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.1 (KHTML, like Gecko) Ubuntu/12.04 Chromium/21.0.1180.79 Chrome/21.0.1180.79 Safari/537.1";
const char S62528[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.1 (KHTML, like Gecko) Ubuntu/12.04 Chromium/21.0.1180.79 Chrome/21.0.1180.79 Safari/537.1";
const char S62529[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5230) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S62530[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5230-c) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S62531[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S62532[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One XL Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S62533[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Enjoy 7 Plus Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62534[] = "Lenovo-A278t_TD/S100 Linux/2.6.35 Android/2.3.5 Release/09.03.2012 Browser/AppleWebkit533.1 Mobile Safari/533.1LENOVO-Lenovo-A278t/1.0 Linux/2.6.35.7 Android 2.3.5 Release/12.27.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S62535[] = "Lenovo-A278t_TD/S100 Linux/2.6.35 Android/2.3.5 Release/09.03.2012 Browser/AppleWebkit533.1 Mobile Safari/533.1";
const char S62536[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A278t Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62537[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A278t Build/MocorDroid2.3.5) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S62538[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Lenovo A278t Build/MocorDroid2.3.5)";
const char S62539[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTCEVODesign4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S62540[] = "Mozilla/5.0 (Linux; Android 4.0.3; PH44100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S62541[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.1.0.5001 Safari/532.4";
const char S62542[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62543[] = "Mozilla/5.0 (Linux; Android 4.0.3; Lenovo P700 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S62544[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo P700i Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S62545[] = "Mozilla/5.0 (X11; Linux x86_64; PadFone 2) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S62546[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/11A465 Safari/8536.25";
const char S62547[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/11A501 Safari/8536.25";
const char S62548[] = "MOT-V8/R601_G_80.44.2BR Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola V8; 3288) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.50 [zh-tw]";
const char S62549[] = "MOT-V8/R601_G_80.56.1BRP Mozilla/4.0 (compatible; MSIE 6.0 Linux; Motorola V8;nnn) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.50[yy]";
const char S62550[] = "MOT-V8/R601_G_80.54.0CR Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola V8; 3109) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.50 [zh-tw]";
const char S62551[] = "MOT-MOTORAZRV8_CMCC/1.0 LinuxOS/2.6.10 Release/06.30.2007 Browser/Opera8.50 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R601_G_80.44.2DR";
const char S62552[] = "MOT-MOTORAZRV82GB/1.0 LinuxOS/2.6.10 Release/08.30.2007 Browser/Opera8.50 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R601_G_80.54.60R";
const char S62553[] = "MOT-V8/R601_G_80.41.1BR Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola V8; 2961) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.50 [zh-tw]";
const char S62554[] = "motorazrV8/R601_G_80.56.1BR Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.50 [es-LA]";
const char S62555[] = "MOT-MOTORAZRV82GB_CMCC/1.0 LinuxOS/2.6.10 Release/08.30.2007 Browser/Opera8.50 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R601_G_80.56.15R_B";
const char S62556[] = "motorazrV8/R601_G_80.44.2BRP Mozilla/4.0 (compatible; MSIE 6.0 Linux; Motorola V8;nnn) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.50[yy]";
const char S62557[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Skate Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62558[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB525 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62559[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB525 Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62560[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MB525 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62561[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MB525+ Build/IMM76L; CyanogenMod-Fast Edition Black 2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62562[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S62563[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P1000 Build/GINGERBREAD) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S62564[] = "Mozilla/5.0 (Linux; Android 4.0.3;  GT-P1000 Build/GINGERBREAD) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S62565[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000L Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S62566[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P1000 Build/MjhieuS3) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S62567[] = "Mozilla/5.0 (Linux; Android 4.0.4;  GT-P1000 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S62568[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S62569[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000N Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S62570[] = "Mozilla/5.0 (Linux; Android 4.0.9; GT-P1000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S62571[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ601 Build/I.7.1-45) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S62572[] = "Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; SC-01D Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S62573[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; SC-01D Build/MASTER)";
const char S62574[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; INO_ONE Build/INO_ONEINO_ONE) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62575[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; INO_ONE_PLUS Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62576[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; INO_ONE Build/IMM74D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62577[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS920 4G Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62578[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ604 Build/I.7.1-45) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S62579[] = "Mozilla/5.0 (Linux; Android 4.0.3; MZ604 Build/I.7.1-34) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S62580[] = "Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; SC-01C Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62581[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; SC-01C Build/FROYO)";
const char S62582[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T699 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S62583[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaN97) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S62584[] = "NokiaN97UCBrowser/8.7.1.234/50/444/UCWEB";
const char S62585[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokian97-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S62586[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62587[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562L Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62588[] = "Mozilla/5.0 (Series40; NokiaX2-01/08.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S62589[] = "Mozilla/5.0 (Series40; NokiaX2-01.1/07.22; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S62590[] = "Mozilla/5.0 (Windows NT 4.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11";
const char S62591[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I500 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62592[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I500 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62593[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I500 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62594[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I500 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62595[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SPH-L710, 39, 1.8.2)";
const char S62596[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SPH-L710, 42, 1.9.1)";
const char S62597[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SAMSUNG-SGH-I747, 42, 1.9.1)";
const char S62598[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-I747M Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SGH-I747M, 42, 1.9.1)";
const char S62599[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I747M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SGH-I747M, 47, 1.9.7)";
const char S62600[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530U Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SCH-R530U, 42, 1.9.1)";
const char S62601[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SAMSUNG-SGH-I747, 54, 2.0.1)";
const char S62602[] = "Mozilla/5.0 (Linux; Android 4.1.1; TBD753B Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S62603[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S62604[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-I9100 Build/miui.es JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S62605[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9100 Build/JZO54K.ASNET) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S62606[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9100G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S62607[] = "BDFetch";
const char S62608[] = "Moreoverbot/5.1 (+http://www.moreover.com; webmaster@moreover.com) Mozilla/5.0";
const char S62609[] = "Mozilla/5.0 Moreoverbot/5.1 (+http://www.moreover.com; webmaster@moreover.com)";
const char S62610[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.0)";
const char S62611[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.0; Trident/4.0)";
const char S62612[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.0; SV1)";
const char S62613[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.0; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S62614[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)";
const char S62615[] = "Mozilla/5.0 (compatible; MSIE 8.0; WinNT4.0; Trident/4.0)";
const char S62616[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S62617[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A100/Micromax_A100_V1.0; 480*854;  CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62618[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A100/Micromax_A100_V1.1; 480*854;  CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62619[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-I500 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62620[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I500 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62621[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-I500 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62622[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I500 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62623[] = "Mozilla/5.0 (Series40; Nokia2700c-2/09.97; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S62624[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515m Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62625[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515a Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62626[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC_X515E Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62627[] = "yacybot (freeworld/global; amd64 Windows 8 6.2; java 1.6.0_41; Europe/de) http://yacy.net/bot.html";
const char S62628[] = "yacybot (freeworld/global; amd64 Windows 8 6.2; java 1.7.0_25; Europe/de) http://yacy.net/bot.html";
const char S62629[] = "yacybot (freeworld/global; amd64 Windows 8 6.2; java 1.7.0_25; America/en) http://yacy.net/bot.html";
const char S62630[] = "SAMSUNG-SGH-D880/D880XAHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S62631[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9620; es) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.807 Mobile Safari/534.11+";
const char S62632[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5360) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S62633[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; vi; GT-S5360) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S62634[] = "iPad";
const char S62635[] = "/media/sandisk/af_mobile/user_agent/ipad";
const char S62636[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N70-S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62637[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; N70-S Build/JRO03H)";
const char S62638[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Iris 349 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62639[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Iris 349+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62640[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us";
const char S62641[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S62642[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700KL Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S62643[] = "SAMSUNG-SGH-T369R/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S62644[] = "SAMSUNG-SGH-T369/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S62645[] = "lb-spider/Mozilla/5.0 Gecko/X Firefox/10.0.2 (lb-spider; http://www.linkbutler.de/spider; spider@linkbutler.de)";
const char S62646[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGPT12 Build/TJS0166) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S62647[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGPT13 Build/TJS0166) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S62648[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0151 Build/HuaweiY300-0151) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S62649[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP9090 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S62650[] = "LGE-UN200/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S62651[] = "User-Agent:LG-LX600 Polaris/6.0 MMP/2.0 Profilie/MIDP-2.1 Configuration/CLDC-1.1";
const char S62652[] = "LG-LX600 Polaris/6.0 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S62653[] = "Microsoft Office/12.0 (Windows NT 6.1; Microsoft Office Outlook 12.0.6545; Pro)";
const char S62654[] = "Microsoft Office/12.0 (Windows NT 6.1; Microsoft Office Outlook 12.0.4518; Pro)";
const char S62655[] = "Microsoft Office/12.0 (Windows NT 6.1; Microsoft Office Outlook 12.0.6554; Pro)";
const char S62656[] = "Microsoft Office/12.0 (Windows NT 6.1; Microsoft Office Outlook 12.0.6562; Pro)";
const char S62657[] = "Microsoft Office/12.0 (Windows NT 6.1; Microsoft Office Outlook 12.0.6607; Pro)";
const char S62658[] = "Microsoft Office/12.0 (Windows NT 6.1; Microsoft Office Outlook 12.0.6320; Pro)";
const char S62659[] = "Microsoft Office/12.0 (Windows NT 6.1; Microsoft Office Outlook 12.0.6425; Pro)";
const char S62660[] = "Microsoft Office/12.0 (Windows NT 6.1; Microsoft Office Outlook 12.0.6535; Pro)";
const char S62661[] = "Microsoft Office/12.0 (Windows NT 6.1; Microsoft Office Outlook 12.0.6662; Pro)";
const char S62662[] = "Microsoft Office/12.0 (Windows NT 6.1; Microsoft Office Outlook 12.0.6213; Pro)";
const char S62663[] = "Microsoft Office/12.0 (Windows NT 6.1; Microsoft Office Outlook 12.0.6683; Pro)";
const char S62664[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Scroll Excel Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62665[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Micromax A65 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62666[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8005 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S62667[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8000 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S62668[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S62669[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire SV Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S62670[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S62671[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE72-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/355/UCWEB Mobile";
const char S62672[] = "NokiaE72-1/UCWEB8.6.0.180/28/800";
const char S62673[] = "Mozilla/5.0 (Java; U; en-us; nokiae72-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/84/352/UCWEB Mobile  3gpp-gba";
const char S62674[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62675[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire SV Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62676[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire S Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62677[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS920 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S62678[] = "Nokia7610Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S62679[] = "Nokia7610s/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S62680[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1";
const char S62681[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1";
const char S62682[] = "Mozilla/5.0 (X11; U; Linux amd64) Iron/21.0.1200.0 Chrome/21.0.1200.0 Safari/537.1";
const char S62683[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A7* Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62684[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; epade A7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62685[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; A7* Build/IMM76D)";
const char S62686[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A7 3G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62687[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2690) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S62688[] = "Mozilla/5.0 (Windows NT 5.1; rv:20.0) Gecko/X Firefox/20.0 IceDragon/20.0.1.14";
const char S62689[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6102 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S62690[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)  480x800 LGE VS750";
const char S62691[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)UCBrowser/8.6.0.199/50/352/UCWEB";
const char S62692[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.21 (KHTML, like Gecko) rekonq/2.3.2 Safari/537.21";
const char S62693[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) rekonq/2.3.2 Safari/534.34";
const char S62694[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6400L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S62695[] = "Mozilla/5.0 (Linux; Android 4.0.4; ThunderBolt Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S62696[] = "Flipboard/2.0.7 CFNetwork/672.0.2 Darwin/14.0.0";
const char S62697[] = "Mozilla/5.0 (Linux; Android 4.2.2; Transformer Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S62698[] = "Mozilla/5.0 (Linux; Android 4.2.2; Transformer TF101 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S62699[] = "Mozilla/5.0 (Linux; Android 4.2.1; Transformer Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S62700[] = "Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S62701[] = "Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S62702[] = "Mozilla/5.0 (Linux; Android 4.1.1; C1504 Build/11.3.A.0.47) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S62703[] = "Mozilla/5.0 (Symbian3; U; en-us; NokiaN8-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/51/355/UCWEB Mobile";
const char S62704[] = "NokiaN8-00/UCWEB8.6.0.180/51/800";
const char S62705[] = "Mozilla/5.0 (Java; U; en-us; nokian8-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile  3gpp-gba";
const char S62706[] = "Mozilla/5.0 (Linux; Android 4.0.3; PH44100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S62707[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTCEVODesign4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S62708[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; SH-03C Build/S3240) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62709[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; SH-03C Build/S9130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62710[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; SH-03C Build/S3010) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62711[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; SH-03C Build/S9130)";
const char S62712[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; SH-03C Build/S8100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62713[] = "SAMSUNG-GT-I9228_TD/1.0 Android/2.3.5 Release/9.15.2011 Browser/AppleWebKit533.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S62714[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9228 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62715[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9228 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S62716[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9228 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S62717[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9228 Build/GINGERBREAD)";
const char S62718[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire C Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S62719[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire C Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S62720[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT29i Build/9.1.B.1.67) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S62721[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-01E Build/9.1.C.0.473) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62722[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-01E Build/9.1.C.0.473) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62723[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-01E Build/9.1.C.0.475) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62724[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-01E Build/9.1.C.0.475) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62725[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SO-01E Build/9.1.C.0.475)";
const char S62726[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-01E Build/9.1.C.1.103) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62727[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-01E Build/9.1.C.1.103) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62728[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I897 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62729[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I897 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62730[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I897 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62731[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I897 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62732[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGH-I897 Build/JDQ39E)";
const char S62733[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P7510 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S62734[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7510 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S62735[] = "LG-CF360/V10h Teleca/Q7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S62736[] = "Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows 98) Opera 7.02 [en]";
const char S62737[] = "Opera/7.02 (Windows 98; U) [en]";
const char S62738[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98) Opera 7.21";
const char S62739[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98) Opera 7.54";
const char S62740[] = "Opera/7.03 (Windows 98; U) [en]";
const char S62741[] = "Opera/7.23 (Windows 98; U) [en]";
const char S62742[] = "Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows 98) Opera 7.03 [de]";
const char S62743[] = "Opera/7.0 (Windows 98; U) [en]";
const char S62744[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98) Opera 7.55";
const char S62745[] = "Opera/7.54 (Windows 98; U)  [en]";
const char S62746[] = "Opera/7.11 (Windows 98; U)  [nl]";
const char S62747[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98) Opera 7.20 [ru]";
const char S62748[] = "Opera/7.21 (Windows 98; U) [en]";
const char S62749[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98) Opera 7.10 [en]";
const char S62750[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98) Opera 7.23 [ru]";
const char S62751[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98) Opera 7.53 [en]";
const char S62752[] = "Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows 98) Opera 7.01 [ru]";
const char S62753[] = "Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows 98) Opera 7.0 [ru]";
const char S62754[] = "Opera/7.01 (Windows 98; U) [fr]";
const char S62755[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.10) Gecko/X Linux Mint/10 (Julia) Firefox/3.6.10 GTB7.1";
const char S62756[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.12) Gecko/X Linux Mint/10 (Julia) Firefox/3.6.12";
const char S62757[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.24) Gecko/X Linux Mint/10 (Julia) Firefox/3.6.24";
const char S62758[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.24) Gecko/X Linux Mint/10 (Julia) Firefox/3.6.24";
const char S62759[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.24) Gecko/X Linux Mint/10 (Julia) Firefox/3.6.24 GTB7.1";
const char S62760[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.17) Gecko/X Linux Mint/10 (Julia) Firefox/3.6.17 GTB7.1";
const char S62761[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.18) Gecko/X Linux Mint/10 (Julia) Firefox/3.6.18 GTB7.1";
const char S62762[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.23) Gecko/X Linux Mint/10 (Julia) Firefox/3.6.23 GTB7.1";
const char S62763[] = "Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5130c-2) UCBrowser8.4.0.159/70/352/UCWEB Mobile";
const char S62764[] = "Nokia5130/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia5130) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S62765[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; XOLO_Q800 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62766[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Q800 Build/JZO54K)";
const char S62767[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; LG-P875 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62768[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P875 Build/JZO54K)";
const char S62769[] = "SAMSUNG-GT-C3350/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S62770[] = "SAMSUNG-GT-C3350_ORANGE/C3350BVKL5 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S62771[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:19.0) Gecko/X Firefox/19.0 SeaMonkey/2.16.2";
const char S62772[] = "Mozilla/5.0 (Linux; Android 4.1.1; IdeaTabA2109A Build/JRO03R) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S62773[] = "SAMSUNG-SGH-J700i/J700IXAIE1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S62774[] = "Vodafone/1.0/SAMSUNG-SGH-J700i/BUIE2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S62775[] = "Vodafone/1.0/SAMSUNG-SGH-J700i/JBID1/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S62776[] = "Vodafone/1.0/SAMSUNG-SGH-J700i/BUIC2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S62777[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; P-07C Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62778[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; P-07C Build/GRI40)";
const char S62779[] = "Nokia5230/UCWEB8.0.3.99/50/999";
const char S62780[] = "Nokia5230/UC Browser8.0.4.121/50/444/UCWEB";
const char S62781[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/523.12.9 (KHTML, like Gecko) Version/3.0 Safari/523.12.9";
const char S62782[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; da) AppleWebKit/522.15.5 (KHTML, like Gecko) Version/3.0.3 Safari/522.15.5";
const char S62783[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en) AppleWebKit/522.13.1 (KHTML, like Gecko) Version/3.0.2 Safari/522.13.1";
const char S62784[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/523.15 (KHTML, like Gecko) Version/3.0 Safari/523.15";
const char S62785[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT901 Build/7.7.1Q-144_SLS_S240-55) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62786[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT901 Build/7.7.1Q-144_SLS_S240-58) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62787[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; XT901 Build/7.7.1Q-144_SLS_S240-58)";
const char S62788[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; cm_tenderloin Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62789[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; skz_tenderloin Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62790[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; skz_tenderloin Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62791[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; skz_tenderloin Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62792[] = "ZTE-F160/1.3.0; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.20822; U; en-US) Opera 9.50";
const char S62793[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-SU760 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62794[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; LG-SU760 Build/GRJ90)";
const char S62795[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE5-00)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S62796[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE5-00.2)/UC Browser8.5.0.183/28/444/UCWEB Mobile";
const char S62797[] = "Mozilla/5.0 (Java; U; en-us; nokiae5-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile  3gpp-gba";
const char S62798[] = "NOKIAE5-00/UCWEB8.5.0.163/28/800";
const char S62799[] = "NokiaC2-00/2.0 (03.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac2-00) UCBrowser8.4.0.159/70/444/UCWEB Mobile";
const char S62800[] = "Mozilla/5.0 (Windows; N; ; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/7.1.1.1888 Safari/532.4";
const char S62801[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ZiiLABS ZiiO10  Build/v2.00.14-FROYO) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile Safari/528.16";
const char S62802[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710e; zh-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S62803[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710e; sv-se) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S62804[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_4G; en-us) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S62805[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710e; en-us) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S62806[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710e; en-gb) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S62807[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710e; nl-nl) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S62808[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710e; en-in) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S62809[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710e; en-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S62810[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710e; ru-ru) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S62811[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710e; en-hk) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S62812[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710e; fr-fr) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S62813[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710e; zh-cn) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S62814[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; B3000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62815[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; CINK KING Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62816[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; CINK KING Build/IMM76D)";
const char S62817[] = "Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Safari/535.19 Silk-Accelerated=true";
const char S62818[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I777 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S62819[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I777 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S62820[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT890 Build/8.7.1I-110_SML-20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S62821[] = "Mozilla/6.0 (Macintosh; U; Amiga-AWeb) Safari 3.1";
const char S62822[] = "QuickTime/7.6.2 (qtver=7.6.2;cpu=IA32;os=Mac 10.5.8)";
const char S62823[] = "QuickTime/7.6 (qtver=7.6;cpu=IA32;os=Mac 10.5.8)";
const char S62824[] = "MacNetwork/1.0 (Macintosh)";
const char S62825[] = "QuickTime/7.6.3 (qtver=7.6.3;cpu=IA32;os=Mac 10.6)";
const char S62826[] = "QuickTime.7.6.6 (qtver=7.6.6;cpu=IA32;os=Mac 10.6.8).";
const char S62827[] = "QuickTime/7.6.3 (qtver=7.6.3;cpu=IA32;os=Mac 10.6.1)";
const char S62828[] = "QuickTime/7.6.3 (qtver=7.6.3;cpu=IA32;os=Mac 10.6.2)";
const char S62829[] = "Mozilla/4.0 (compatible; WebCapture 3.0; Macintosh)";
const char S62830[] = "QuickTime/7.6 (qtver=7.6;cpu=IA32;os=Mac 10.5.6)";
const char S62831[] = "QuickTime/7.5.5 (qtver=7.5.5;cpu=IA32;os=Mac 10.5.8)";
const char S62832[] = "TuneUpApp/2.4.2 (Mac; OSX 10.7)";
const char S62833[] = "QuickTime/7.6.1 (qtver=7.6.1;cpu=IA32;os=Mac 10.5.8)";
const char S62834[] = "QuickTime.7.6.9 (qtver=7.6.9;cpu=IA32;os=Mac 10.5.8).";
const char S62835[] = "TuneUpApp/2.4.2 (Mac; OSX 10.6)";
const char S62836[] = "QuickTime.7.7 (qtver=7.7;cpu=IA32;os=Mac 10.5.8).";
const char S62837[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-P3100 Build/JOP40D)";
const char S62838[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P3100 Build/JDQ39)";
const char S62839[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ru-ru; GT-P3100 Build/JDQ39) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 GoogleGoggles-Android/1.9; gzip";
const char S62840[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3100 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62841[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3100 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62842[] = "SCH-U450/1.0 NetFront/3.0.22.2.20 (GUI) MMP/2.0";
const char S62843[] = "Mozilla/5.0 (Windows NT 6.0; rv:15.0) Gecko/X Firefox/15.0.1 SeaMonkey/2.12.1";
const char S62844[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Mobile Safari/535.19 Silk-Accelerated=true";
const char S62845[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9000 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S62846[] = "Mozilla/5.0 (Linux; Android 4.2; GT-I9000 Build/JOP40C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S62847[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9000 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S62848[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Mobile Safari/535.19 Silk-Accelerated=true";
const char S62849[] = "Opera/9.80 (Android 4.1.1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00";
const char S62850[] = "Opera/9.80 (Android 4.1.2; Linux; Opera Tablet/ADY-X; U; ru) Presto/2.10.254 Version/12.00";
const char S62851[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; ALCATEL_one_touch_890 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S62852[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; ALCATEL one touch 890 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S62853[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; ALCATEL_one_touch_890 Build/ERE27)";
const char S62854[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Ally Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S62855[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Ally Build/ERE27) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S62856[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/17.0.963.46) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.46 Safari/535.11";
const char S62857[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/17.0.963.56) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11";
const char S62858[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/17.0.963.56)";
const char S62859[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/17.0.963.79) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11";
const char S62860[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/17.0.963.79) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11";
const char S62861[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/17.0.963.56)";
const char S62862[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/17.0.963.83) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.83 Safari/535.11";
const char S62863[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/17.0.963.56; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)";
const char S62864[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/17.0.963.83; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E)";
const char S62865[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SV1; GTB7.2; chromeframe/17.0.963.46; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.30618; .NET CLR 3.5.30729)";
const char S62866[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/4.0; GTB7.2; chromeframe/17.0.963.46; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.30618; .NET CLR 3.5.30729)";
const char S62867[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.2; chromeframe/17.0.963.46; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; InfoPath.2; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S62868[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5230)/UC Browser8.2.0.132/50/355/UCWEB Mobile";
const char S62869[] = "Nokia5230/UCWEB8.2.0.116/50/800";
const char S62870[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5230-c)/UC Browser8.2.0.132/50/355/UCWEB Mobile";
const char S62871[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T769 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62872[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 UCBrowser/9.0.1.284 Mobile";
const char S62873[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 UCBrowser/9.0.1.284 Mobile";
const char S62874[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 UCBrowser/9.0.1.284 Mobile";
const char S62875[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 UCBrowser/9.0.1.284 Mobile";
const char S62876[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L720 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S62877[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG-SCH-i220";
const char S62878[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG-SCH-i220a";
const char S62879[] = "Mail/1283 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)";
const char S62880[] = "Mail/1283 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (iMac12%2C1)";
const char S62881[] = "Mail/1283 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookAir4%2C2)";
const char S62882[] = "Mail/1283 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBook5%2C1)";
const char S62883[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; pt-br; XT317 Build/V4.33B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62884[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; pt-br; XT317 Build/V4.36C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62885[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT317 Build/V4.36A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62886[] = "Mozilla/5.0 (Linux; Android 4.0.3; Lenovo S880 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S62887[] = "sam-r270 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S62888[] = "sam-r270v1 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S62889[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; ISW11F Build/FGK500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62890[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; ISW11F Build/FGK500)";
const char S62891[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62892[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710a Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62893[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC-Z710a Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62894[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62895[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; fr_crawler)";
const char S62896[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; MSIECrawler)";
const char S62897[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; .NET CLR 1.1.4322; MSIECrawler)";
const char S62898[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; MSIECrawler)";
const char S62899[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; .NET CLR 2.0.50727; MSIECrawler)";
const char S62900[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; fr_crawler);c2VvLWNyYXdsZXI=";
const char S62901[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; FunWebProducts; Seekmo 11.0.96.0; MSIECrawler);c2VvLWNyYXdsZXI=";
const char S62902[] = "Mozilla/5.0 (X11; U; SunOS i86pc; en-US; rv:1.9.2.2) Gecko/X Firefox/3.6.2";
const char S62903[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; AN7G2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62904[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.1; AN7G2 Build/GINGERBREAD)";
const char S62905[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; DATAM752HCX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62906[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; DATAM752HCX_A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62907[] = "Mozilla/5.0 (Linux; U; Android 4.0.3 mod by Anninaone; it-it; DATAM752HCX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62908[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; M-MP715I Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S62909[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:21.0) Gecko/X Firefox/21.0 SeaMonkey/2.18";
const char S62910[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT881 Build/9.8.3O-118_USC-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S62911[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R740C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S62912[] = "LG-T300-Orange/V10c Obigo/Q7.3 MMS/LG-MMS-V1.1/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S62913[] = "LG-T300/V100 Obigo/Q7.3 MMS/LG-MMS-V1.1/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S62914[] = "SAMSUNG-SGH-T459/T459UVIA1 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S62915[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; IS03 Build/SB060) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S62916[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; IS03 Build/S2080) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S62917[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; IS03 Build/SC240) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S62918[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; IS03 Build/S5180) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S62919[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; IS03 Build/S9211) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S62920[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; IS03 Build/S4120) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S62921[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; IS03 Build/S2080)";
const char S62922[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko)  QtWeb Internet Browser/3.7 http://www.QtWeb.net";
const char S62923[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko)  QtWeb Internet Browser/3.7.3 http://www.QtWeb.net";
const char S62924[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.3 (KHTML, like Gecko)  QtWeb Internet Browser/3.7.3 http://www.QtWeb.net";
const char S62925[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko)  QtWeb Internet Browser/3.7.5 http://www.QtWeb.net";
const char S62926[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko)  QtWeb Internet Browser/3.7.2 http://www.QtWeb.net";
const char S62927[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.3 (KHTML, like Gecko)  QtWeb Internet Browser/3.7.5 http://www.QtWeb.net";
const char S62928[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S5830 Build/CCGH1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62929[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-S5830 Build/REVOLUTiON) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62930[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; GT-S5830C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62931[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-S5830 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62932[] = "Dalvik/1.4.0 (Linux; U; Android 4.1.2; GT-S5830 Build/CCGH1)";
const char S62933[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Mobile Safari/535.19 Silk-Accelerated=true";
const char S62934[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; STM722HCZ Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62935[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; STM722HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62936[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; STM722HZ Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62937[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HS-E910 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62938[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; E910 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62939[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-E910 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S62940[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.4) Gecko/X Firefox/10.0.4";
const char S62941[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.2) Gecko/X Firefox/10.0.2";
const char S62942[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.5) Gecko/X Firefox/10.0.5";
const char S62943[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0) Gecko/X Firefox/10.0";
const char S62944[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.6) Gecko/X Firefox/10.0.6";
const char S62945[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.7) Gecko/X Firefox/10.0.7";
const char S62946[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.9) Gecko/X Firefox/10.0.9";
const char S62947[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.10) Gecko/X Firefox/10.0.10";
const char S62948[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.11) Gecko/X Firefox/10.0.11";
const char S62949[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.12) Gecko/X Firefox/10.0.12";
const char S62950[] = "Mozilla/5.0 (Linux; Android 4.0.4; N9500 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S62951[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; SCH-I930)";
const char S62952[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-L710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S62953[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 800;7.10;7720)";
const char S62954[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 800;7.10;8107)";
const char S62955[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 800;7.10;7740)";
const char S62956[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 800;7.10;8773)";
const char S62957[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 800;7.10;8112)";
const char S62958[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 800;7.10;8779)";
const char S62959[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 800;7.10;8783)";
const char S62960[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M9000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62961[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; M9000 Build/IMM76D)";
const char S62962[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia500) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S62963[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One VX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62964[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PM36100/2.21.502.1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62965[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC One VX Build/JZO54K)";
const char S62966[] = "Mozilla/5.0 (Linux; Android 4.0.4; C5155 Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S62967[] = "Mozilla/5.0 (Linux; Android 4.0.4; Rise Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S62968[] = "LENOVO-Lenovo-A298t/1.0 Linux/2.6.35.7 Android 2.3.5 Release/10.30.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S62969[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A298t Build/MocorDroid2.3.5) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S62970[] = "Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A298t Build MocorDroid2.3.5) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S62971[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A298t Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62972[] = "LENOVO-Lenovo-A298t/1.0 Linux/2.6.35.7 Android 2.3.5 Release/10.22.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S62973[] = "LENOVO-Lenovo-A298t/1.0 Linux/2.6.35.7 Android 2.3.5 Release/12.13.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S62974[] = "LENOVO-Lenovo-A298t/1.0 Linux/2.6.35.7 Android 2.3.5 Release/11.22.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S62975[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Lenovo A298t Build/MocorDroid2.3.5)";
const char S62976[] = "LENOVO-Lenovo-A298t/1.0 Linux/2.6.35.7 Android 2.3.5 Release/10.11.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S62977[] = "Lenovo A298t_TD/1.0 Linux/2.6.35.7 Android 2.3.5 Release/04.07.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S62978[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100P Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S62979[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100P Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S62980[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SH-02E Build/S4260) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62981[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SH-02E Build/S4260)";
const char S62982[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SH-02E Build/S6210) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62983[] = "AppleCoreMedia/1.0.0.9A334 (iPad; U; CPU OS 5_0 like Mac OS X; en_us)";
const char S62984[] = "AppleCoreMedia/1.0.0.9A405 (iPad; U; CPU OS 5_0_1 like Mac OS X; en_us)";
const char S62985[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; md-us; Kindle Fire Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62986[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; md-us; Kindle Fire Build/GRK39F; CyanogenMod-HellFireKindle_V1.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62987[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Amazon Kindle Fire Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S62988[] = "Mozilla/5.0 (Windows NT 5.1; rv:15.0) Gecko/X Firefox/15.1.1 PaleMoon/15.1.1";
const char S62989[] = "Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:15.0) Gecko/X Firefox/15.1.1-x64 PaleMoon/15.1.1-x64";
const char S62990[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; P-02E Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S62991[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; P-02E Build/JZO54K)";
const char S62992[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R830 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S62993[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST26i Build/11.0.A.7.5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S62994[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST26a Build/11.0.A.7.5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S62995[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Amaze 4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S62996[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Ruby Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S62997[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC_Amaze_4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S62998[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac6-00) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba";
const char S62999[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC6-00) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S63000[] = "NokiaC6-00/UCWEB8.9.0.253/50/999";
const char S63001[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaN73) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S63002[] = "Mozilla/5.0 (Linux; Android 4.0.4; Huawei-H867G Build/HuaweiH867G) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63003[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; chromeframe/27.0.1453.116)";
const char S63004[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; Alcatel one touch 908F Orange Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63005[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; Alcatel_one_touch_908F_Orange Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63006[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; ALCATEL_one_touch_908F_Orange Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63007[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; pt-pt; ALCATEL_one_touch_908F Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63008[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; Alcatel_one_touch_908F Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63009[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Iron/9.0.600.2 Chrome/9.0.600.2 Safari/534.13";
const char S63010[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63011[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire V Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63012[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire VC Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63013[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire VC T328d Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63014[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire P Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63015[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire Q Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63016[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC_Desire_200 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63017[] = "Mozilla/5.0 (Java; U; en-us; gt-s5233s) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/84/352/UCWEB Mobile";
const char S63018[] = "Mozilla/5.0 (Java; U; vi; gt-s5233s) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S63019[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 10 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63020[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 10 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63021[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 10 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63022[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 10 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1.1/manta";
const char S63023[] = "Mozilla/5.0 (Linux; Android 4.1.1; MI 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S63024[] = "Mozilla/5.0 (Linux; Android 4.1.1; MI 2A Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S63025[] = "Mozilla/5.0 (Linux; Android 4.0.3; PG86100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63026[] = "Mozilla/5.0 (Linux; Android 4.0.3; ISW12HT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63027[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Ubuntu/10.10 Chromium/11.0.696.71 Chrome/11.0.696.71 Safari/534.24";
const char S63028[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.24 (KHTML, like Gecko) Ubuntu/10.10 Chromium/11.0.696.57 Chrome/11.0.696.57 Safari/534.24";
const char S63029[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.18 (KHTML, like Gecko) Ubuntu/10.10 Chromium/11.0.661.0 Chrome/11.0.661.0 Safari/534.18";
const char S63030[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.21 (KHTML, like Gecko) Ubuntu/10.10 Chromium/11.0.681.0 Chrome/11.0.681.0 Safari/534.21";
const char S63031[] = "Mozilla/5.0 (X11; U; Linux i686) AppleWebKit/534.21 (KHTML, like Gecko) Ubuntu/10.10 Chromium/11.0.682.0 Chrome/11.0.682.0 Safari/534.21";
const char S63032[] = "Nokia6630/1.0 (5.03.08) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.9.1.120/27/351/UCWEB";
const char S63033[] = "NokiaX2-02/2.0 (11.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiax2-02) UCBrowser8.4.0.159/70/352/UCWEB Mobile";
const char S63034[] = "iTunes-iPad/6.0.1 (2; 16GB; dt:74)";
const char S63035[] = "iTunes-iPad/6.0.1 (2; 32GB; dt:74)";
const char S63036[] = "iTunes-iPad/6.0.1 (4; 32GB; dt:77)";
const char S63037[] = "iTunes-iPad/6.0.1 (4; 16GB; dt:77)";
const char S63038[] = "iTunes-iPad/6.0.1 (5; 16GB; dt:83)";
const char S63039[] = "iTunes-iPad/6.0.1 (4; 64GB; dt:77)";
const char S63040[] = "iTunes-iPad/6.0.1 (2; 64GB; dt:74)";
const char S63041[] = "iTunes-iPad/6.0.2 (5; 16GB; dt:86)";
const char S63042[] = "iTunes-iPad/6.0.1 (5; 32GB; dt:83)";
const char S63043[] = "iTunes-iPad/6.0.2 (5; 32GB; dt:86)";
const char S63044[] = "iTunes-iPad/6.0.1 (5; 64GB; dt:83)";
const char S63045[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; K701 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63046[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; K701 Build/JRO03H)";
const char S63047[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Teraware K701H Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63048[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Mobile/11A4414e Twitter for iPhone";
const char S63049[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4435d Twitter for iPhone";
const char S63050[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449a Twitter for iPhone";
const char S63051[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d Twitter for iPhone";
const char S63052[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Twitter for iPhone";
const char S63053[] = "Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Twitter for iPhone";
const char S63054[] = "Nokia5700/6.01 (SymbianOS/9.2; U; Series60/3.1 Nokia5700/6.01; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) Mozilla/5.0 AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S63055[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.2 Nokia5700/1.00; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S63056[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Nokia5700/3.83.1; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S63057[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6503 Build/10.1.A.1.434) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63058[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-03E Build/10.1.E.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63059[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-03E Build/10.1.E.0.250) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63060[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-03E Build/10.1.E.0.265) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63061[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-03E Build/10.1.E.0.269) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63062[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-03E Build/10.1.E.0.269) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63063[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SO-03E Build/10.1.E.0.269)";
const char S63064[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-03E Build/10.1.E.0.305) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63065[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 754TPC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63066[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; 754TPC Build/IMM76D)";
const char S63067[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30at Build/9.1.F.1.120) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S63068[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30at Build/9.1.F.1.120) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S63069[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D2-927G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63070[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D2-927G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63071[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM200SH Build/S0004) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63072[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM200SH Build/S0016) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63073[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM200SH Build/S0028) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63074[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SBM200SH Build/S0028)";
const char S63075[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S63076[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT915 Build/2_32A_2008) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63077[] = "Nokia2700c-2/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2700c-2) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S63078[] = "Nokia2700c/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2700c) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S63079[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; V9 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63080[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; V9 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63081[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; V9 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63082[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; V9 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63083[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; V9 Build/GRJ22)";
const char S63084[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ThL V9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63085[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; RTK V9 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63086[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; V9 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63087[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; V9 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63088[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9a5pre) Gecko/X Minefield/3.0a5pre";
const char S63089[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0 x64; en-US; rv:1.9.0.3pre) Gecko/X Minefield/3.0.5pre";
const char S63090[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1";
const char S63091[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1";
const char S63092[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CMP745E Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63093[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT912 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63094[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT912 Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63095[] = "NokiaC3-00/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac3-00) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S63096[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROIDX Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63097[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63098[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROIDX Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63099[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; DROIDX Build/MIUI)";
const char S63100[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63101[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/IMM76I; CyanogenMod-9.0.0; Owains Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63102[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63103[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/GRJ22; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63104[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/IMM76L; CyanogenMod-9.0.0; Owains Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63105[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/GRI40; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63106[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/GRI40; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63107[] = "NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S63108[] = "NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-05";
const char S63109[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xoom Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63110[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xoom Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63111[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xoom Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63112[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; F-02E Build/V16R46A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63113[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; F-02E Build/V17R48A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63114[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; F-02E Build/V17R48A)";
const char S63115[] = "Mozilla/5.0 (Series40; NokiaC2-02/06.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S63116[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S63117[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S63118[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S63119[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S63120[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.2 Safari/534.34";
const char S63121[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.8.0 Safari/533.3";
const char S63122[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN7HG3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63123[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; AN7HG3 Build/IML74K)";
const char S63124[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T889 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S63125[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S63126[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S63127[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L900 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S63128[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7105 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S63129[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S63130[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T889 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S63131[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L900 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S63132[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30; Maxthon (4.0.4.1000);";
const char S63133[] = "SAMSUNG-SGH-A847M/A847BMJK4 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S63134[] = "SAMSUNG-SGH-A847R/A847UXJK4 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S63135[] = "SAMSUNG-SGH-A847D/A847TLJK4 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S63136[] = "SAMSUNG-SGH-A847/A847UCJK2 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S63137[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en; rv:1.8.1.14) Gecko/X Camino/1.6.1 (like Firefox/2.0.0.14)";
const char S63138[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en; rv:1.8.1.14) Gecko/X Camino/1.6 (like Firefox/2.0.0.14)";
const char S63139[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en; rv:1.8.1.4pre) Gecko/X Camino/1.6pre";
const char S63140[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U1 Build/HCL ME Tablet U1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63141[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 4.0.3; en-; U1 Build/HCL ME Tablet U1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63142[] = "HUAWEI_T8620_TD/1.0 Android/4.0.3 (Linux; U; Android 4.0.3; zh-cn) Release/05.31.2012 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30";
const char S63143[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; T8620 Build/HuaweiT8620) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S63144[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T8620 Build/HuaweiT8620) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63145[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; T8620 Build/HuaweiT8620)";
const char S63146[] = "Mozilla/5.0 (Windows NT 6.0; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.0.1";
const char S63147[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.0.1-x64";
const char S63148[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315b Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S63149[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S63150[] = "Mozilla/5.0 (Linux; Android 4.1.1; A210 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S63151[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5110 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S63152[] = "BlackBerry8320/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100";
const char S63153[] = "Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320) BlackBerry8320/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100";
const char S63154[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JOP40G; Carbon-Carbon-JB-v1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63155[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39; Carbon-Carbon-JB-v1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63156[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39; Carbon-CARBON-JB-v1.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63157[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63158[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; ADR6200 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S63159[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; ADR6200Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S63160[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; T3 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63161[] = "Mozilla/5.0 (Linux;U;Android 2.3.5;en-us;TECNO T3 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63162[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; Fantastic T3 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63163[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; in-id; T3 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63164[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SATC T3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63165[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; T3 Build/GRJ22)";
const char S63166[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Hosin T3 Build/MocorDroid2.3.5) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S63167[] = "GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.1.7600.0);en-US;kml:2.2;client:Free;type:default)";
const char S63168[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.0.3.2197 Safari/532.4";
const char S63169[] = "GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.1.7601.1);en-US;kml:2.2;client:Free;type:default)";
const char S63170[] = "GoogleEarth/6.0.1.2032(Windows;Microsoft Windows (6.1.7600.0);en-US;kml:2.2;client:Free;type:default)";
const char S63171[] = "GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.1.7600.0);de-DE;kml:2.2;client:Free;type:default)";
const char S63172[] = "GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.1.7601.1);en-GB;kml:2.2;client:Free;type:default)";
const char S63173[] = "GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.1.7601.1);de-DE;kml:2.2;client:Free;type:default)";
const char S63174[] = "GoogleEarth/6.0.1.2032(Windows;Microsoft Windows (6.1.7601.1);en-US;kml:2.2;client:Free;type:default)";
const char S63175[] = "GoogleEarth/6.0.2.2074(Windows;Microsoft Windows (6.1.7601.1);en-US;kml:2.2;client:Free;type:default)";
const char S63176[] = "GoogleEarth/6.0.2.2074(Windows;Microsoft Windows (6.1.7600.0);en-US;kml:2.2;client:Free;type:default)";
const char S63177[] = "GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.1.7601.1);en-CA;kml:2.2;client:Free;type:default)";
const char S63178[] = "GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.1.7601.1);en-AU;kml:2.2;client:Free;type:default)";
const char S63179[] = "GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.1.7601.1);en-US;kml:2.2;client:Pro;type:default)";
const char S63180[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957R Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63181[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957M Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63182[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63183[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I957 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63184[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63185[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; M-MP102S2B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63186[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; M-MP102S2W Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63187[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3) AppleWebKit/534.31 (KHTML, like Gecko) Chrome/17.0.558.0 Safari/534.31 UCBrowser/2.2.2.259";
const char S63188[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515m Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S63189[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S63190[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC_X515E Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S63191[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Vision Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63192[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Vision Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63193[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; HTC Vision Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63194[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Vision Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63195[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Vision Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63196[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HTC Vision Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63197[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Vision Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63198[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; HTC Vision Build/FRF91)";
const char S63199[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; HTC Vision Build/FRG83D)";
const char S63200[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Vision Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63201[] = "WIRE/0.22 (Linux; i686; Bot,Robot,Spider,Crawler)";
const char S63202[] = "WIRE/0.22 (Linux; x86_64; Bot,Robot,Spider,Crawler)";
const char S63203[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.19 (KHTML, like Gecko) Maxthon/1.0.3.1000 Chrome/18.0.1025.142 Safari/535.19";
const char S63204[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/1.0.4.1000 Chrome/21.0.1180.75 Safari/537.1";
const char S63205[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/1.0.4.2000 Chrome/21.0.1180.75 Safari/537.1";
const char S63206[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/1.0.4.2000 Chrome/21.0.1180.75 Safari/537.1";
const char S63207[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT885 Build/6.7.3-179_YIC-2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63208[] = "Opera/9.80 (Android 4.1.1; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S63209[] = "Opera/9.80 (Android 4.1.2; Linux; Opera Mobi/ADY-X) Presto/2.11.355 Version/12.10";
const char S63210[] = "Opera/9.80 (Android 4.1.9; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S63211[] = "Opera/9.80 (Android 4.1.5; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S63212[] = "Opera/9.80 (Android 4.1; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S63213[] = "UCWEB/2.0(BlackBerry; U; 9.50.0.28; en-us; 9220/9.50.0.28) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S63214[] = "UCWEB/2.0(BlackBerry; U; 9.50.0.50; en-us; 9220/9.50.0.50) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S63215[] = "UCWEB/2.0(BlackBerry; U; 9.50.0.36; en-us; 9220/9.50.0.36) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S63216[] = "UCWEB/2.0(BlackBerry; U; 9.50.0.80; en-us; 9220/9.50.0.80) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S63217[] = "UCWEB/2.0(BlackBerry; U; 9.50.0.79; en-us; 9220/9.50.0.79) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S63218[] = "UCWEB/2.0(BlackBerry; U; 9.50.0.99; en-us; 9220/9.50.0.99) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S63219[] = "SAMSUNG-GT-C3222/C3222DDLD4 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S63220[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5";
const char S63221[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5";
const char S63222[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ615-12) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63223[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-M830 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63224[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P705 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S63225[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P705g Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S63226[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P708g Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S63227[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P705f Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S63228[] = "Nokia3108/1.0 (03.01) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S63229[] = "SAMSUNG-GT-S3310/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S63230[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:16.0) Gecko/X Firefox/16.0 SeaMonkey/2.13.2";
const char S63231[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One XL Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S63232[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One XL Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S63233[] = "Mozilla/5.0 (Linux; Android 4.0.4; C525c Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S63234[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63235[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63236[] = "Mozilla/5.0 (Linux; Android 4.0.3;  SHW-M250S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S63237[] = "Mozilla/5.0 (Linux; Android 4.0.3; SHW-M250S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63238[] = "Mozilla/5.0 (Linux; Android 4.0.3; SHW-M250K Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63239[] = "Mozilla/5.0 (Linux; Android 4.0.3; SHW-M250L Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63240[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63241[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250K Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63242[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63243[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.6 Mobile Safari/535.19 Silk-Accelerated=true";
const char S63244[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6800 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S63245[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LS720 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63246[] = "SonyEricssonW880i/R8BA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S63247[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-06D Build/S9030) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63248[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-06DNERV Build/S9061) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63249[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-06D Build/S2060) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63250[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-06DNERV Build/S2060) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63251[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SH-06D Build/S9030)";
const char S63252[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-06D Build/S9090) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63253[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/2.8 Mobile/9B206 Safari/7534.48.3";
const char S63254[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/2.8 Mobile/9B176 Safari/7534.48.3";
const char S63255[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; ZTE-BLADE Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S63256[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-gb; ZTE-BLADE Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63257[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; ZTE-BLADE Build/ERE27)";
const char S63258[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; U8500 HiQQ Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S63259[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; U8500 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S63260[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-tw; HUAWEI IDEOS U8500 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S63261[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; U8500 Build/ERE27)";
const char S63262[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; U8500 Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S63263[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63264[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P6200 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63265[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Safari/535.19";
const char S63266[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:20.0) Gecko/X Firefox/20.0 IceDragon/20.0.1.14";
const char S63267[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaN73) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S63268[] = "Mozilla/5.0 (Linux; Android 4.2.1; ME173X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S63269[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; ISW16SH Build/S6151) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63270[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; ISW16SH Build/S7060) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63271[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; ISW16SH Build/S7260) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63272[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; ISW16SH Build/S8100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63273[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; ISW16SH Build/S9181) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63274[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; ISW16SH Build/S1150) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63275[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; ISW16SH Build/S2271) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63276[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; ISW16SH Build/S5082) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63277[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ISW16SH Build/S5082)";
const char S63278[] = "Opera/9.80 (Windows NT 6.2; U; en) Presto/2.6.30 Version/10.63";
const char S63279[] = "Opera/9.80 (Windows NT 6.2; U; en) Presto/2.6.30 Version/10.60";
const char S63280[] = "Opera/9.80 (Windows NT 6.2; U; en) Presto/2.6.30 Version/10.61";
const char S63281[] = "Opera/9.80 (Windows NT 6.2; U; en) Presto/2.6.30 Version/10.62";
const char S63282[] = "Nokia202/2.0 (20.28) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia202) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S63283[] = "Mozilla/4.0 (compatible; MSIE 4.01; MSIECrawler; Windows 95)";
const char S63284[] = "Mozilla/4.0 (compatible; MSIE 4.0; MSIECrawler; Windows 95)";
const char S63285[] = "Mozilla/5.0 (Series40; NokiaX2-00/08.25; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S63286[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; GN800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63287[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GN800 Build/IMM76D)";
const char S63288[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; GIONEE_GN800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63289[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN800 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S63290[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GPLUS_GN800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63291[] = "Mozilla/5.0 (Windows NT 4.0; rv:23.0) Gecko/X Firefox/23.0";
const char S63292[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; SmartPad7503G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63293[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Flytouch9 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63294[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Flytouch 9 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63295[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.21 (KHTML, like Gecko) QupZilla/1.4.3 Safari/537.21";
const char S63296[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.21 (KHTML, like Gecko) QupZilla/1.4.3 Safari/537.21";
const char S63297[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5120 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S63298[] = "Mozilla/5.0 (Linux; Android 4.1.2; N9101 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63299[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS05 Build/S2251) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63300[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS05 Build/S7210) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63301[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS05 Build/S9140) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63302[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS05 Build/S5160) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63303[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-; IS05 Build/S9140) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63304[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; IS05 Build/S9140)";
const char S63305[] = "Mozilla/2.0 (compatible; MSIE 3.0; Windows 3.1)";
const char S63306[] = "Mozilla/2.0 (compatible; MSIE 3.02; Windows 3.1)";
const char S63307[] = "Mozilla/2.0 (compatible; MSIE 3.03; Windows 3.1)";
const char S63308[] = "Mozilla/5.0 (compatible; MSIE 3.0; Win3.11; Trident/4.0)";
const char S63309[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MOMO8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63310[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; MOMO8 Build/JRO03H)";
const char S63311[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Luna TAB07-100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63312[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Luna TAB07-100 Build/JRO03C)";
const char S63313[] = "Mozilla/5.0 (Linux; Android 4.0.3; ISW12HT Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S63314[] = "Mozilla/5.0 (Linux; Android 4.0.3; PG86100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S63315[] = "Mozilla/5.0 (Linux;U;Linux2.6; en-us; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/6.0 Safari/533.1";
const char S63316[] = "Mozilla/5.0 (Linux;U;Linux2.6; en-us; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML,like Gecko) Version/6.0 Safari/530.17";
const char S63317[] = "Mozilla/5.0 (Linux; U; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/6.0 Safari/530.17";
const char S63318[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TABEO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63319[] = "Mozilla/5.0 (Linux; Android 4.2.1; Xoom Build/JB_MR1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63320[] = "Mozilla/5.0 (Linux; Android 4.2.2; Xoom Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63321[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID7046 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63322[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LG870 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63323[] = "NokiaN73-1/2.0628.0.0.1 S60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S63324[] = "NokiaN73-2/2.0626 S60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S63325[] = "NokiaN73";
const char S63326[] = "Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63327[] = "Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63328[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F260S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63329[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F260S Build/JZO54K)";
const char S63330[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S63331[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700KL Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S63332[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-SU760 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63333[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-SU760 Build/FRG83G)";
const char S63334[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ru-; LG-SU760 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63335[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SHV-E330S Build/JDQ39)";
const char S63336[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ko-kr; SHV-E330S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 NAVER(inapp; search; 230; 4.7.7)";
const char S63337[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F180L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63338[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F180K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63339[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F180 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63340[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F180S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63341[] = "MOT-MPx220/1.430 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 176x220)";
const char S63342[] = "MOT-MPx220/1.400 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone;";
const char S63343[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus One Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63344[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Nexus One Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63345[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Nexus One Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63346[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; Nexus One Build/MIUI)";
const char S63347[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Nexus One Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S63348[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ616-12) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63349[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ616 Build/7.7.1-141-6-PAST-UMTS-LA) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63350[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax P275 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63351[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Micromax P275 Build/IMM76D)";
const char S63352[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1.2 (like Version/10B143 Mobile/6.1 Safari/8536.25)";
const char S63353[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1.2 (like Version/10B146 Mobile/6.1.2 Safari/8536.25)";
const char S63354[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1.2 (like Version/10B145 Mobile/6.1.1 Safari/8536.25)";
const char S63355[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1.2 (like Version/10B329 Mobile/6.1.3 Safari/8536.25)";
const char S63356[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1.2 (like Version/10B350 Mobile/6.1.4 Safari/8536.25)";
const char S63357[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5360) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S63358[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; vi; GT-S5360) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S63359[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I777 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63360[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I777 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63361[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I777 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63362[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I777 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63363[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-uk; @TAB-TEN.1-X2-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63364[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; @TAB-TEN.1W-X2-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63365[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D2-912 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63366[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I200 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S63367[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I200 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S63368[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-M950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63369[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT897 Build/7.7.1Q-6_SPR-125_ASA-14) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S63370[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT897 Build/7.7.1Q-6_SPR-125_ASA-14) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S63371[] = "SAMSUNG-GT-S5350/S5350DDJA3 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-N";
const char S63372[] = "Opera/9.80 (Android; Opera Mini/6.7.29878/28.3234; U; zh) Presto/2.8.119 Version/11.10";
const char S63373[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s3850) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S63374[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP9070 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S63375[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP9070 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S63376[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GEM7012 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63377[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PMP5197DULTRA Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63378[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; PMP5197DULTRA Build/IML74K)";
const char S63379[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0151 Build/HuaweiY300-0151) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63380[] = "Opera/9.80 (Windows NT 6.2; WOW64; U; en) Presto/2.10.229 Version/11.62";
const char S63381[] = "Opera/9.80 (Windows NT 6.2; WOW64; U; MRA 5.10 (build 5300); ru) Presto/2.10.229 Version/11.62";
const char S63382[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; CINK+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63383[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; CINK FIVE Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63384[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; CINK FIVE Build/JZO54K)";
const char S63385[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; CINK+ Build/JRO03C)";
const char S63386[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PMP7074B3G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63387[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ru-ru; PMP7074B3GRU Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63388[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63389[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63390[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63391[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63392[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S63393[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63394[] = "Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:9.0.1) Gecko/X Firefox/9.0.1 Firefox/9.0.1";
const char S63395[] = "Mozilla/5.0 (Ubuntu; X11; Linux x86_64; rv:9.0) Gecko/X Firefox/9.0";
const char S63396[] = "Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:9.0) Gecko/X Firefox/9.0";
const char S63397[] = "Mozilla/5.0 (Ubuntu; X11; Linux x86_64; rv:9.0.1) Gecko/X Firefox/9.0.1,gzip(gfe) (via translate.google.com)";
const char S63398[] = "Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:9.0.1) Gecko/X Firefox/9.0.1,gzip(gfe) (via translate.google.com)";
const char S63399[] = "Mozilla/5.0 (Ubuntu; X11; Linux i686 on x86_64; rv:9.0.1) Gecko/X Firefox/9.0.1";
const char S63400[] = "Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:9.0.1) Gecko/X Abrowser/9.0.1";
const char S63401[] = "Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:9.0.1) Gecko/X Firefox/9.0.1 FirePHP/0.6";
const char S63402[] = "Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:9.0a2) Gecko/X Firefox/9.0a2";
const char S63403[] = "Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:9.0.1) Gecko/X Firefox/9.0.1,gzip(gfe)";
const char S63404[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; CT1002PLUS Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63405[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; CT1002 Build/ICS.b02ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63406[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; CT1002 Build/ICS.b02ref) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63407[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5210 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S63408[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SPH-P100 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63409[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SPH-P100 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63410[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SPH-P100 Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S63411[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SPH-P100 Build/FROYO)";
const char S63412[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; SPH-P100 Build/FROYO)";
const char S63413[] = "Mozilla/5.0 (compatible; MSIE 6.0; BREW 4.0.3; en )/800x480 Samsung sam-r900";
const char S63414[] = "sam-r250 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S63415[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I497 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S63416[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGP321 Build/10.1.1.A.1.253) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S63417[] = "NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-00) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S63418[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS11N Build/01.02.00) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63419[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS11N Build/01.04.20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63420[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS11N Build/01.03.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63421[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS11N Build/01.06.30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63422[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS11N Build/01.05.00) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63423[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS11N Build/01.07.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63424[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS11N Build/01.08.00) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63425[] = "Micromax X335/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S63426[] = "Micromax X335C/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S63427[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A9 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63428[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; A9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63429[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A9 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63430[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; A9 Build/JRO03C)";
const char S63431[] = "Mozilla/5.0 (Linux; U; Android 2.3;en-au; ZTE T12 Build/FRF91)AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63432[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; ZTE T12 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63433[] = "Nokia206/2.0 (03.59) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia206) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S63434[] = "Nokia206.1/2.0 (03.58) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia206.1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S63435[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26w Build/6.1.A.2.45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S63436[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7310 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S63437[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS950 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S63438[] = "MOT-MOTORAZRV9/A0.01.10R10 BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003266) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [en]";
const char S63439[] = "MOT-MOTORAZRV9/A0.03.18R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003291) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [en]";
const char S63440[] = "MOT-MOTORAZRV9x/9E.03.3AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003352) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.60 [en]";
const char S63441[] = "MOT-MOTORAZRV9x/9E.03.28R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003348) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [en]";
const char S63442[] = "MOT-MOTORAZRV9/A0.05.20R1 BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003348) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [en]";
const char S63443[] = "MOT-MOTORAZRV9/A0.03.2ER BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003321) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [en]";
const char S63444[] = "MOT-MOTORAZRV9/A0.02.0ER BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13223296) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [en]";
const char S63445[] = "MOT-MOTORAZRV9/A0.02.07R2 BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003275) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [en]";
const char S63446[] = "MOT-MOTORAZRV9x/9E.03.39R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; X) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [en]";
const char S63447[] = "MOT-MOTORAZRV9/A0.05.20R1 BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; X) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [en]";
const char S63448[] = "Mozilla/5.0 (Windows NT 5.1; rv:12.3) Gecko/X Firefox/12.3 PaleMoon/12.3";
const char S63449[] = "Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:12.3) Gecko/X Firefox/12.3-x64 PaleMoon/12.3-x64";
const char S63450[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:12.3) Gecko/X Firefox/12.3 PaleMoon/12.3";
const char S63451[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Tattoo Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63452[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.2 Mobile Safari/535.19 Silk-Accelerated=true";
const char S63453[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DA220HQL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63454[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; DA220HQL Build/IMM76D)";
const char S63455[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7";
const char S63456[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7";
const char S63457[] = "JUC (Linux; U; 2.3.4; zh-cn; GT-S5670; 240*320) UCWEB7.9.0.94/139/352";
const char S63458[] = "JUC (Linux; U; 2.2.1; zh-cn; GT-S5670; 240*320) UCWEB7.9.0.94/139/444";
const char S63459[] = "NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiax2-01) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S63460[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E980 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S63461[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E980 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S63462[] = "Nokia206/2.0 (03.59) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia206) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S63463[] = "Nokia206.1/2.0 (03.58) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia206.1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S63464[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T769 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63465[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; fr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S63466[] = "Mozilla/5.0 (X11; U; FreeBSD i386; en-US; rv:1.9.1) Gecko/X Firefox/3.5";
const char S63467[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.2.16) Gecko/X Firefox/3.6.16 CometBird/3.6.16";
const char S63468[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9100G Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S63469[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S63470[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Google Earth/7.1.1.1888 Safari/533.3";
const char S63471[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-L710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63472[] = "Opera/9.80 (X11; Linux i686; Edition Linux Mint) Presto/2.12.388 Version/12.10";
const char S63473[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M1062 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63474[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S63475[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S63476[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT890 Build/8.7.1I-110_SML-20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S63477[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROIDX Build/VZW; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63478[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E970 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S63479[] = "KWC-S1300/ UP.Browser/7.2.6.1.839 (GUI) MMP/2.0";
const char S63480[] = "KWC-S1300/1007 UP.Browser/7.2.6.1.840 (GUI) MMP/2.0";
const char S63481[] = "KWC-S1300/1002 UP.Browser/7.2.6.1.840 (GUI) MMP/2.0";
const char S63482[] = "KWC-S1300/1001 UP.Browser/7.2.6.1.841 (GUI) MMP/2.0";
const char S63483[] = "KWC-S1300/ UP.Browser/7.2.6.1.838 (GUI) MMP/2.0";
const char S63484[] = "KWC-S1300/1009 UP.Browser/7.2.6.1.839 (GUI) MMP/2.0";
const char S63485[] = "KWC-S1300/10081 UP.Browser/7.2.6.1.839 (GUI) MMP/2.0";
const char S63486[] = "KWC-S1300/1007 UP.Browser/7.2.6.1.839 (GUI) MMP/2.0";
const char S63487[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; T-01C Build/FFR002) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63488[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; T-01C Build/FFR002)";
const char S63489[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) WebClip/5534.49 Safari/5534.49";
const char S63490[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50";
const char S63491[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Cruz/0.4 Safari/534.50.2";
const char S63492[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/534.50.2 (KHTML, like Gecko) Fake/1.7.2 Safari/534.50.2";
const char S63493[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16";
const char S63494[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5) AppleWebKit/537.13  (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2";
const char S63495[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i929 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63496[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i929 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S63497[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i929 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S63498[] = "Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i929 Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S63499[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-i929 Build/GINGERBREAD)";
const char S63500[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M032 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63501[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; M032 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S63502[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; M032 Build/IML74K)";
const char S63503[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P7500 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63504[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7500 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63505[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7500M Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63506[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A789 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S63507[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A789 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63508[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo A789 Build/IMM76D)";
const char S63509[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LENOVO A789 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63510[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; Lenovo A789; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63511[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; LENOVO A789-orange Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63512[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lenovo A789-orange Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63513[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Lenovo A789 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63514[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7500 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63515[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7500 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63516[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7500 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63517[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-; GT-P7500 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63518[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.3; Trident/7.0; MSAppHost/1.0)";
const char S63519[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.3; WOW64; Trident/6.0)";
const char S63520[] = "Mozilla/5.0 (MSIE 10.0; Windows NT 6.3; WOW64; Trident/7.0; rv:11.0) like Gecko";
const char S63521[] = "Mozilla/5.0 (MSIE 10.0; Windows NT 6.3; Win64; x64; Trident/7.0; rv:11.0) like Gecko";
const char S63522[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.3; Win64; x64; Trident/6.0)";
const char S63523[] = "Mercury/7.3 CFNetwork/548.1.4 Darwin/11.0.0";
const char S63524[] = "SAMSUNG-GT-M5650/M5650XIJA1 SHP/VPP/R5 Jasmine/1.0 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S63525[] = "SAMSUNG-GT-M5650U/1.0 SHP/VPP/R5 Jasmine/1.0 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S63526[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; KM-S200 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63527[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; KM-S220 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63528[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; KM-S200 Build/IMM76I)";
const char S63529[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I407 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63530[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-I407 Build/JZO54K)";
const char S63531[] = "Mozilla/5.0 (masking-agent; rv:20.0) Gecko/X Firefox/20.0";
const char S63532[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26ii Build/6.2.B.0.200) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63533[] = "SAMSUNG-SGH-T559/T559UVIFA SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S63534[] = "Mozilla/5.0 (compatible; Thumbshots.ru; +http://thumbshots.ru/bot) Firefox/3";
const char S63535[] = "Mozilla/4.0 (Brew MP 1.0.2; U; en-us; Kyocera; NetFront/4.1/AMB) Sprint E4255";
const char S63536[] = "Mozilla/4.0 (Brew MP 1.0.2; U; en-us; Kyocera; NetFront/4.1/AMB) Sprint E4210";
const char S63537[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaN8-00) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S63538[] = "Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabA2109A Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S63539[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5230)/UC Browser8.5.0.183/50/355/UCWEB Mobile";
const char S63540[] = "Nokia5230/UCWEB8.5.0.163/50/999";
const char S63541[] = "Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Safari/535.19 Silk-Accelerated=true";
const char S63542[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T989 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S63543[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-T989 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S63544[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-T989 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S63545[] = "Mozilla/5.0 (Linux; Android 4.0.4; Next7P12 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Safari/537.36";
const char S63546[] = "Mozilla/5.0 (Linux; Android 4.0.4; Next7P12-8G Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S63547[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID7127 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63548[] = "Logos/0.9.3 (Linux; U; Android 2.3.3; en_US; MID7127 Build/MID7127 X)";
const char S63549[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; novo7_Aurora Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63550[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Novo7 Aurora Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63551[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; novo7_Aurora Build/IML74K)";
const char S63552[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Novo7Aurora Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63553[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; novo7_Aurora Build/IML74K; 1024*552) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S63554[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I699 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63555[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I699 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S63556[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-I699 Build/GINGERBREAD)";
const char S63557[] = "Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I699 Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S63558[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; VOYAGER Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63559[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-uk; VOYAGER-BT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63560[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; VOYAGER-BT Build/JRO03H)";
const char S63561[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SC-04E Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63562[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SC-04E Build/JDQ39)";
const char S63563[] = "Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_5) AppleWebKit (KHTML, like Gecko) Chrome/3";
const char S63564[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-US) AppleWebKit/531.0 (KHTML, like Gecko) Chrome/3.0.183 Safari/531.0";
const char S63565[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.198 Safari/532.0";
const char S63566[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.198 Safari/532.0";
const char S63567[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s3850) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S63568[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s3850) UCBrowser8.3.1.161/70/465/UCWEB Mobile";
const char S63569[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP8010 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63570[] = "Mozilla/5.0 (Series40; Nokia112/03.26; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S63571[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAB469 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63572[] = "Mozilla/5.0 (Android; Tablet; rv:19.0) Gecko/19.0 K9Browser/19.0.0";
const char S63573[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8000 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S63574[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8000 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S63575[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8020 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S63576[] = "Mozilla/5.0 (Linux; Android 4.1.1; IdeaTabA2109A Build/JRO03R) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S63577[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S63578[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-I9100T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S63579[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; GT-I9100G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S63580[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.60140";
const char S63581[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 7 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S63582[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S63583[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S63584[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S63585[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S63586[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S63587[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S63588[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S63589[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S63590[] = "NetFront/3.5.1(BREW 3.1.5; U; en-us; SAMSUNG; NetFront/3.1.5/AMB) Sprint M560 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S63591[] = "Mozilla/5.0 (masking-agent; rv:15.0) Gecko/X Firefox/15.0";
const char S63592[] = "Mozilla/5.0 (masking-agent; rv:15.0) Gecko/X Firefox/15.0.1";
const char S63593[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; QMobile A11Note Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63594[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; QMobile_A11Note Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63595[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.31 (KHTML, like Gecko) Chrome/17.0.558.0 Safari/534.31 UCBrowser/9.0.2.299";
const char S63596[] = "Mozilla/5.0 (Windows; N; Windows NT 6.0; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/7.1.1.1888 Safari/532.4";
const char S63597[] = "Mozilla/5.0 (Windows NT 5.0; rv:23.0) Gecko/X Firefox/23.0";
const char S63598[] = "NokiaC5-00/UCWEB8.0.3.99/28/999";
const char S63599[] = "NokiaC5-00/UC Browser8.0.4.121/28/444/UCWEB";
const char S63600[] = "NokiaC5-00.2/UC Browser8.0.4.121/28/444/UCWEB";
const char S63601[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63602[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; A9 Build/GRK39F)";
const char S63603[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6120c) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S63604[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26w Build/6.1.A.2.45) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S63605[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S63606[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S63607[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S63608[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC-Z710a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S63609[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; PG06100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63610[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; PG06100 Build/FRF91) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S63611[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; PG06100 Build/FRF91)";
const char S63612[] = "Mozilla/5.0 (Linux; Android 4.0.3; A501 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63613[] = "Mozilla/5.0 (Linux; Android 4.0.4; Iconia A501 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63614[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTCEVODesign4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S63615[] = "Mozilla/5.0 (Linux; Android 4.0.3; PH44100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S63616[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC6435LRA Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63617[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC6435LRA Build/JRO03C)";
const char S63618[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_Droid_DNA Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63619[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E973 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S63620[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F200K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S63621[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F200S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S63622[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F200L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S63623[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.4.22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S63624[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.6.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S63625[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.0.16) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S63626[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.6.8) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S63627[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.1.8) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S63628[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.4.27) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S63629[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-gb; Nexus 7 Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb";
const char S63630[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us";
const char S63631[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us";
const char S63632[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-gb; Nexus 7 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.8/620,2.0.8.620,2013-09-06 15:06, +0100, gb";
const char S63633[] = "Mozilla/5.0 (Linux; Android 4.3; One Build/JWR66V.H10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.38 Mobile Safari/537.36";
const char S63634[] = "Mozilla/5.0 (Linux; Android 4.3; HTC One Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S63635[] = "Mozilla/5.0 (Linux; Android 4.3; HTCONE Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S63636[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5200 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S63637[] = "SAMSUNG-GT-E2152i/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S63638[] = "SAMSUNG-GT-E2152/E2152DDJH1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S63639[] = "SAMSUNG-GT-E2152i/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S63640[] = "LGE-VN250P/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S63641[] = "LGE-VN250L/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S63642[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; nl-nl; MOVE Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63643[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; MOVE Build/FRG83G)";
const char S63644[] = "Mozilla/5.0 (compatible; Konqueror/2.1.1; X11)";
const char S63645[] = "Opera/9.00 (Windows NT 5.0; U; en)";
const char S63646[] = "Opera/9.02 (Windows NT 5.0; U; en)";
const char S63647[] = "Opera/9.01 (Windows NT 5.0; U; en)";
const char S63648[] = "Opera/9.0 (Windows NT 5.0; U; en)";
const char S63649[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer TF101 Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63650[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer TF101 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63651[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer TF101 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63652[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63653[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Transformer TF101 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63654[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Transformer Build/JZO54K)";
const char S63655[] = "Mozilla/5.0 (Linux; Android 4.1.2; A110 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S63656[] = "Mozilla/5.0 (Linux; Android 4.1.1; Micromax A110 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S63657[] = "Mozilla/5.0 (Linux; Android 4.1.1; A110 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S63658[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-M950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S63659[] = "Mozilla/5.0 (Windows NT 4.10; rv:12.0) Gecko/X Firefox/12.0";
const char S63660[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; U8650 Build/benru89; CM7.1.0-benru89) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63661[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Huawei u8650 Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63662[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; U8650 Build/cm7-benru89; CyanogenMod7.2-benru89) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63663[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; U8650 Build/CyanogenMod 7.2; CyanogenMod 7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63664[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; U8650 Build/CyanogenMod 2.3.7; CyanogenMod 7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63665[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A95 Build/CelkonCelkon_A95) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63666[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Celkon*A95*Pro Build/CelkonCelkon_A95) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63667[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403,Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S63668[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S63669[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S63670[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523,Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S63671[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 6_0_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S63672[] = "Mozilla/5.0 (iPad; U; CPU OS 6_0 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S63673[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AMI-TT4 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63674[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; AMI-TT4 Build/JRO03H)";
const char S63675[] = "LGE-VP200-PLB NetFront/3.5.1 (GUI) MMP/2.0";
const char S63676[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3 TeamStream HD (iPad2,1 232 1.6.1)";
const char S63677[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 TeamStream HD (iPad1,1 284 1.7)";
const char S63678[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ZP100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63679[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZP100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63680[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SN70T3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63681[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SN70T31 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63682[] = "Mozilla/4.0 (compatible; Polaris 6.2; Brew 3.1.5; en)/240X320 Samsung sam-r640";
const char S63683[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I500 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63684[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I500 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63685[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I500 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63686[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I500 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63687[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63688[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63689[] = "LG-GU295/V10i; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.15957; U; en-US) Opera 9.50";
const char S63690[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko) Iron/11.0.700.2 Chrome/11.0.700.2 Safari/534.24";
const char S63691[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; ARCHOS 97 TITANIUMHD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63692[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT907 Build/7.7.1Q-144_SMQ_S136-10) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S63693[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT905 Build/7.7.1Q_GCIRD-16) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S63694[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT907 Build/7.7.1Q-144_SMQ_S136-10) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S63695[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT905 Build/7.7.1Q_GCIRD-16) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S63696[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; C1604 Build/11.1.A.0.68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63697[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; SonyC1604 Build/11.1.A.0.64) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63698[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; C1604 Build/11.1.A.0.64) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63699[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; SonyC1604 Build/11.1.A.0.53) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63700[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I547 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S63701[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I547C Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S63702[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I547 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S63703[] = "Mozilla/5.0 (Linux; Android 4.1.1; Z992 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63704[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaX6-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile";
const char S63705[] = "NokiaX6-00/UCWEB8.6.0.180/50/800";
const char S63706[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ViewPad 10e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63707[] = "Mozilla/5.0 (Linux; U; Android 2.3;it-it; ZTE-U V857 Build/FRF91)AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63708[] = "Mozilla/5.0 (Linux; U; Android 2.3;sr-rs;  ZTE-U V857 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63709[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S63710[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia2690) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S63711[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Sensation Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63712[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Sensation Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63713[] = "Mozilla/5.0 (Linux; Android 4.1.2; Sensation Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63714[] = "Mozilla/5.0 (Linux; Android 4.2.2; zh-cn; SAMSUNG-GT-I9508_TD/1.0 Android/4.2.2 Release/03.15.2013 Browser/AppleWebKit535.19 Build/JDQ39) ApplelWebkit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S63715[] = "Mozilla/5.0 (Linux; Android 4.2.2; zh-cn; SAMSUNG GT-I9508 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S63716[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One SV Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S63717[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; A701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63718[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63719[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; A701 Build/IMM76D)";
const char S63720[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; ZTE T22 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63721[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; X506 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63722[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; X506 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63723[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63724[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63725[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A9 Build/ICS.g08refema9.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63726[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad7295 Build/4.1.015.X.7295; 540*960; CUCC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63727[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad7295 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S63728[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Coolpad7295 Build/JZO54K)";
const char S63729[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 7295+ Build/4.1.003.X.7295+; 540*960; CUCC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63730[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 7295+ Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S63731[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Coolpad 7295+ Build/JZO54K)";
const char S63732[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 7295A Build/4.1.003.X.7295A; 480*854; CUCC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63733[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P705f Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S63734[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P705 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63735[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P708g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63736[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P705g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63737[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; LG-SU880 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63738[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-SU880 Build/GRJ22)";
const char S63739[] = "Mozilla/5.0 (Windows NT 6.0; rv:15.0) Gecko/X Thunderbird/15.0.1";
const char S63740[] = "Nokia303/5.0 (14.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia303) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S63741[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:17.0) Gecko/X Firefox/17.0 SeaMonkey/2.14.1";
const char S63742[] = "Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S63743[] = "Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S63744[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315";
const char S63745[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-N7000 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S63746[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S63747[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N850L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63748[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N850 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63749[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; vi-vn; FPT Tablet Build/1.0.3900.0211)AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63750[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; vi-vn; FPT Tablet II Build/1.0.7044.0298)AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63751[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; vi-vn; FPT Tablet II Build/1.0.5259.0261)AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63752[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; FPT Tablet Build/1.0.4846.0245)";
const char S63753[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; FPT Tablet II Build/1.0.5779.0279)";
const char S63754[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; FPT Tablet Build/1.0.4846.0245)AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63755[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; FPT Tablet II Build/1.0.5779.0279)AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63756[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS17SH Build/S7070) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63757[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS17SH Build/S7251) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63758[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS17SH Build/S2200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63759[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC5-03) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S63760[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9500 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S63761[] = "DoCoMo/2.0 SH902i (compatible; Y!J-SRD/1.0; http://help.yahoo.co.jp/help/jp/search/indexing/indexing-27.html)";
const char S63762[] = "Opera/5.0 (Linux 2.0.38 i386; U) [en]";
const char S63763[] = "Mozilla/4.0 (compatible; MSIE 5.0; Linux) Opera 5.0 [en]";
const char S63764[] = "Mozilla/5.0 (X11; Litl OS i686) AppleWebKit/534.23 (KHTML, like Gecko) Chrome/11.0.689.0 Safari/534.23";
const char S63765[] = "Chrome/11";
const char S63766[] = "Mozilla/5.0 (compatible) AppleWebKit/534.21 (KHTML, like Gecko) Chrome/11.0.682.0 Safari/534.21";
const char S63767[] = "MQQBrowser/39 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3";
const char S63768[] = "MQQBrowser/39 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari/7534.48.3";
const char S63769[] = "MQQBrowser/39 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3";
const char S63770[] = "MQQBrowser/391 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3";
const char S63771[] = "MQQBrowser/391 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari/7534.48.3";
const char S63772[] = "MQQBrowser/39 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3";
const char S63773[] = "MQQBrowser/391 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3";
const char S63774[] = "MQQBrowser/391 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3";
const char S63775[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 610;7.10;8773)";
const char S63776[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 610;7.10;8779)";
const char S63777[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 610;7.10;8858)";
const char S63778[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 610;7.10;8862)";
const char S63779[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 610 NFC;7.10;8779)";
const char S63780[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 610;7.10;8783)";
const char S63781[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SM-T315 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63782[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SAMSUNG SM-T315/T315XXUAMG3 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63783[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SM-T315 Build/JDQ39)";
const char S63784[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63785[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus S Build/JRO03E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63786[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I727R Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S63787[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E110S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S63788[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I727R Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S63789[] = "Mozilla/5.0 (Series40; Nokia112/03.32; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S63790[] = "Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook HD+ Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S63791[] = "Mozilla/5.0 (Linux; Android 4.2.2; BN NookHD+ Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S63792[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nook HD+ Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S63793[] = "Mozilla/5.0 (Series40; Nokia5130c-2/07.95; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.4.34.8";
const char S63794[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I547 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63795[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I547C Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63796[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I547 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63797[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6400L Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S63798[] = "Mozilla/5.0 (Linux; Android 4.0.4; ThunderBolt Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S63799[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9152 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63800[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9152 Build/JDQ39)";
const char S63801[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63802[] = "Mozilla/5.0 (Series40; Nokia205/03.18; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S63803[] = "Mozilla/5.0 (Series40; Nokia205.1/03.18; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S63804[] = "Mozilla/5.0 (Linux; U; Android 1.0.0; nl-nl; A7EB Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S63805[] = "Mozilla/5.0 (Linux; U; Android 1.0.1; nl-nl; A7EB Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S63806[] = "Mozilla/5.0 (Linux; U; Android 1.0.4; en-us; A7EB Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S63807[] = "Mozilla/5.0 (Linux; U; Android 1.0.4; fr-fr; A7EB Build/ECLAIR.eng.root.X.X) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S63808[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M340K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63809[] = "Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabS2109A-F Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63810[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTabS2109A-F Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63811[] = "Mozilla/5.0 (Mobile; rv:18.0) Gecko/18.0 Firefox/18.0";
const char S63812[] = "Mozilla/5.0 (Linux; Android 4.2.2; Transformer TF101 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S63813[] = "Mozilla/5.0 (Linux; Android 4.2.1; Transformer Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S63814[] = "Mozilla/5.0 (Linux; Android 4.2.2; Transformer Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S63815[] = "NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S63816[] = "OneBrowser/3.0,Nokia6681/2.0";
const char S63817[] = "One Browser/3.0,Nokia6681/2.0";
const char S63818[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.8.1 Safari/534.34";
const char S63819[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT300 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S63820[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; Ivy Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S63821[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-kw; EQ U8110 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S63822[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; sr-rs; U8110 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S63823[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; U8110 Build/ERE27)";
const char S63824[] = "SAMSUNG-SGH-T919/T919UVHL3 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S63825[] = "Mozilla/5.0 (X11; U; SunOS sun4u; en-US; rv:1.8.1.1) Gecko/X Firefox/2.0.0.1";
const char S63826[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.7 (KHTML, like Gecko) Ubuntu/10.04 Chromium/16.0.912.77 Chrome/16.0.912.77 Safari/535.7";
const char S63827[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.7 (KHTML, like Gecko) Ubuntu/10.04 Chromium/16.0.912.77 Chrome/16.0.912.77 Safari/535.7";
const char S63828[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN95) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile";
const char S63829[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN95 8GB) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile";
const char S63830[] = "Mozilla/5.0 (Java; U; en-us; nokian95) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S63831[] = "Mozilla/5.0 (Java; U; en-us; nokian95_8gb) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S63832[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Edison Build/1.1.8 X-16:29) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63833[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq edison Build/1.1.1 bq_edison_c1006_X_user) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63834[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-I9100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S63835[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-I9100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S63836[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-I9100G Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S63837[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-I9100G Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S63838[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-I9100T Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S63839[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Incredible S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S63840[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT881 Build/9.8.3O-118_USC-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63841[] = "HTC_Touch_HD_T8282 Opera/9.50 (Windows NT 5.1; U; de)";
const char S63842[] = "HTC_Touch_HD_T8282 Opera/9.50 (Windows NT 5.1; U; nl)";
const char S63843[] = "HTC_Touch_HD_T8282 Opera/9.50 (Windows NT 5.1; U; pt)";
const char S63844[] = "HTC_Touch_HD_T8282 Opera/9.50 (Windows NT 5.1; U; zh-tw)";
const char S63845[] = "HTC_Touch_HD_T8282 Opera/9.50 (Windows NT 5.1; U; fr)";
const char S63846[] = "HTC_Touch_HD_T8282 Opera/9.50 (Windows NT 5.1; U; es-ES)";
const char S63847[] = "HTC_Touch_HD_T8282 Opera/9.50 (Windows NT 5.1; U; en)";
const char S63848[] = "HTC_Touch_HD_T8282 Opera/9.50 (Windows NT 5.1; U; it)";
const char S63849[] = "HTC_Touch_HD_T8282 Opera/9.50 (Windows NT 5.1; U; ru)";
const char S63850[] = "HTC_Touch_HD_T8282 Opera/9.50 (Windows NT 5.1; U; nb)";
const char S63851[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; 7 Pro T7576)";
const char S63852[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; USCCHTC-PC93100; USC-US)";
const char S63853[] = "NokiaE71/UCWEB8.2.0.116/28/800";
const char S63854[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE71)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S63855[] = "Mozilla/5.0 (Java; U; en-in; nokiae71-1) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S63856[] = "Lenovo-A65/S204 Linux/2.6.35.7 Android/2.3 Release/12.07.2011 Browser/AppleWebKit533.1 Profile/ Configuration/";
const char S63857[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Lenovo A65 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63858[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A65 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S63859[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A65 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S63860[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Lenovo A65 Build/GRJ90)";
const char S63861[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959V Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63862[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959V Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63863[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959V Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63864[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959V Build/cMIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63865[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-T959V Build/IMM76L)";
const char S63866[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT200 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63867[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT875 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63868[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; XT875 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63869[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT875 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/targa";
const char S63870[] = "Mozilla/5.0 (X11; FreeBSD amd64; rv:20.0) Gecko/X Firefox/20.0";
const char S63871[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXE_Beats_Z715e; zh-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S63872[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXE_Beats_Z715e; en-hk) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S63873[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXE_Beats_Z715e; en-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S63874[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXE_Beats_Z715e; en-gb) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S63875[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXE_Beats_Z715a; en-au) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S63876[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXE_Beats_Z715e; nl-nl) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S63877[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXE_Beats_Z715e; en-in) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S63878[] = "Mozilla/5.0 (Linux; Android 4.1.1; Micromax A110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S63879[] = "Mozilla/5.0 (Linux; Android 4.1.2; A110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S63880[] = "Mozilla/5.0 (Linux; Android 4.1.1; A110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S63881[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM102SH Build/S1010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63882[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM102SH2 Build/S1010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63883[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SBM102SH Build/S1010)";
const char S63884[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SBM102SH2 Build/S1010)";
const char S63885[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM102SH Build/S1020) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63886[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM102SH2 Build/S1020) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63887[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM102SH Build/S1040) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63888[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM102SH2 Build/S1040) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63889[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9100G Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S63890[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S63891[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9100T Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S63892[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Aquaris Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63893[] = "LG-GS390/V10p Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S63894[] = "HTC_Touch_HD_T8282/480x800 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S63895[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 480x800; HTC_Touch_HD_T8282; OpVer 34.119.1.611";
const char S63896[] = "HTC_Touch_HD_T8282 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S63897[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 480x800; HTC_Touch_HD_T8282; OpVer 34.159.1.612";
const char S63898[] = "HTC_Touch_HD_T8282-orange/PPC; 480x800 OpVer 34.119.2.731 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S63899[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Touch_HD_T8282/1.14.163.3 (32434)";
const char S63900[] = "HTC_Touch_HD_T8282/1.21.621.2 (38850) Mozilla/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S63901[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Touch_HD_T8282/1.14.163.2 (28253)";
const char S63902[] = "Mozilla/5.0 (compatible; Konqueror/2.0.1; X11); Supports MD5-Digest; Supports gzip encoding";
const char S63903[] = "Opera/9.80 (Windows; Opera Mobi/35779; U; en) Presto/2.10.254 Version/12.00";
const char S63904[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT15i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63905[] = "Mozilla/5.0 (Linux; Android 4.0.4;  LT15i Build/4.1.B.0.431) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S63906[] = "Nokia113/2.0 (03.32) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S63907[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Next9P Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S63908[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ASP-4500Z Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63909[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ASP-4500Z Build/JRO03C)";
const char S63910[] = "Mozilla/5.0 (Linux; Android 4.1.2; LGMS659 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S63911[] = "Mozilla/5.0 (Linux; Android 4.1.2; LGMS659 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S63912[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-1 Build/HuaweiU8950-1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63913[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-51 Build/HuaweiU8950-51) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63914[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-51 Build/HuaweiU8950N-51) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63915[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-1 Build/HuaweiU8950N-1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S63916[] = "Opera 9.4 (Windows NT 6.1; U; en)";
const char S63917[] = "Opera 9.4 (Windows NT 6.1; U; en)	Opera 9.4";
const char S63918[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.13) Gecko/X Linux Mint/9 (Isadora) Firefox/3.6.13 GTB7.1";
const char S63919[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.14) Gecko/X Linux Mint/9 (Isadora) Firefox/3.6.14";
const char S63920[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.24) Gecko/X Linux Mint/9 (Isadora) Firefox/3.6.24";
const char S63921[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.24) Gecko/X Linux Mint/9 (Isadora) Firefox/3.6.24";
const char S63922[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.15) Gecko/X Linux Mint/9 (Isadora) Firefox/3.6.15";
const char S63923[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.21) Gecko/X Linux Mint/9 (Isadora) Firefox/3.6.21";
const char S63924[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.6) Gecko/X Linux Mint/9 (Isadora) Firefox/3.6.6";
const char S63925[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.24) Gecko/X Linux Mint/9 (Isadora) Firefox/3.6.24 GTB7.1";
const char S63926[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.9) Gecko/X Linux Mint/9 (Isadora) Firefox/3.6.9";
const char S63927[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.17) Gecko/X Linux Mint/9 (Isadora) Firefox/3.6.17 GTB7.1";
const char S63928[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; 101F Build/V43R36H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63929[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; 101F Build/V45R37B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63930[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; 101F Build/V49R39H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63931[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; 101F Build/V47R38J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63932[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; 101F Build/V51R40E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63933[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; 101F Build/V51R40E)";
const char S63934[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993D Build/ICECREAM) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63935[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993 Build/ICECREAM) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S63936[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT901 Build/7.7.1Q-144_SLS_S240-58) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S63937[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT914 Build/2_32A_2012) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S63938[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; HTC PLS7373ADR Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S63939[] = "Nokia302/5.0 (14.53) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia302) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S63940[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S63941[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S63942[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.32) Gecko/X Firefox/3.6.32 (Palemoon/3.6.32)";
const char S63943[] = "Nokia110/2.0 (03.04) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia110) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S63944[] = "Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2730c-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S63945[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PAP4500TDUO Build/PrestigioPAP4500TDUO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63946[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PAP4500DUO Build/PrestigioPAP4500DUO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S63947[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/10B146 Safari/7534.48.3";
const char S63948[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/10B329 Safari/7534.48.3";
const char S63949[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/10B141 Safari/7534.48.3";
const char S63950[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/10B147 Safari/7534.48.3";
const char S63951[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGP312 Build/10.1.C.0.370) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S63952[] = "Mozilla/4.0 (compatible; MSIE 6.0; BREW 3.1.5; en )/800x480 Samsung SCH-U960";
const char S63953[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; 6534.46) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5";
const char S63954[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9B176 Safari/6533.18.5";
const char S63955[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9B179 Safari/6533.18.5";
const char S63956[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en_us) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5";
const char S63957[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; en_us) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5";
const char S63958[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9B206 Safari/6533.18.5";
const char S63959[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; 6534.46) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5";
const char S63960[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/6533.18.5 DiigoBrowser";
const char S63961[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9B206 Safari/6533.18.5";
const char S63962[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; es-es) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S63963[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S63964[] = "Mozilla/5.0 (Linux; Android 4.0.3; Penta WS802C Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63965[] = "Mozilla/5.0 (Linux; Android 4.0.4; Penta WS802C Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63966[] = "Mozilla/5.0 (Linux; Android 4.0.3; WS802C Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S63967[] = "SAMSUNG-GT-S3802/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S63968[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.22 (KHTML, like Gecko) Iron/25.0.1400.0 Chrome/25.0.1400.0 Safari/537.22";
const char S63969[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Iron/25.0.1400.0 Chrome/25.0.1400.0 Safari/537.22";
const char S63970[] = "AppleCoreMedia/1.0.0.11A465 (iPhone; U; CPU OS 7_0 like Mac OS X; en_us)";
const char S63971[] = "AppleCoreMedia/1.0.0.11A470a (iPhone; U; CPU OS 7_0_1 like Mac OS X; en_us)";
const char S63972[] = "AppleCoreMedia/1.0.0.11A501 (iPhone; U; CPU OS 7_0_2 like Mac OS X; en_us)";
const char S63973[] = "NokiaE61i Orange/1.0 Series60/3.0 Nokia E61i/2.0633.65.01 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S63974[] = "NokiaE61i-1/3.0 (1.0633.62.05) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S63975[] = "NokiaE61-1/3.0 (3.0633.09.04) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S63976[] = "Mozilla/5.0 (SymbianOS/9.1; U; [en-us]; Series60/3.0 NokiaE61/3.0633.09.04) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S63977[] = "SAMSUNG-SGH-A927/A927UCJF5 SHP/VPP/R5 Dolfin/1.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S63978[] = "NetFront/3.5.1(BREW 3.1.5; U; en-us; SAMSUNG; NetFront/3.1.5/AMB) Sprint M550 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S63979[] = "Mozilla/5.0 (X11; CrOS i686 1324.0.0) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/17.0.942.0 Safari/535.8";
const char S63980[] = "Mozilla/5.0 (X11; CrOS i686 1384.0.2011) AppleWebKit/535.10 (KHTML, like Gecko) Chrome/17.0.955.0 Safari/535.10";
const char S63981[] = "Mozilla/5.0 (X11; CrOS i686 1408.0.2011) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.962.0 Safari/535.11";
const char S63982[] = "Mozilla/5.0 (X11; CrOS i686 1412.64.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.27 Safari/535.11";
const char S63983[] = "Mozilla/5.0 (X11; CrOS i686 1412.87.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.33 Safari/535.11";
const char S63984[] = "Mozilla/5.0 (X11; CrOS i686 1412.121.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.38 Safari/535.11";
const char S63985[] = "Mozilla/5.0 (X11; CrOS i686 1412.142.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.43 Safari/535.11";
const char S63986[] = "Mozilla/5.0 (X11; CrOS i686 1412.150.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.46 Safari/535.11";
const char S63987[] = "Mozilla/5.0 (X11; CrOS i686 1412.205.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.60 Safari/535.11";
const char S63988[] = "Mozilla/5.0 (X11; CrOS i686 1412.223.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11";
const char S63989[] = "Mozilla/5.0 (X11; CrOS i686 1412.230.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.78 Safari/535.11";
const char S63990[] = "Mozilla/5.0 (X11; CrOS i686 1412.231.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.78 Safari/535.11";
const char S63991[] = "Mozilla/5.0 (X11; CrOS i686 1412.234.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.80 Safari/535.11";
const char S63992[] = "Mozilla/5.0 (X11; CrOS i686 1412.186.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.54 Safari/535.11";
const char S63993[] = "Mozilla/5.0 (X11; CrOS i686 1412.174.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.51 Safari/535.11";
const char S63994[] = "Mozilla/5.0 (X11; CrOS i686 1341.0.2011) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/17.0.942.0 Safari/535.8";
const char S63995[] = "Mozilla/5.0 (X11; CrOS i686 1403.0.2011) AppleWebKit/535.10 (KHTML, like Gecko) Chrome/17.0.955.0 Safari/535.10";
const char S63996[] = "Mozilla/5.0 (X11; CrOS i686 1319.0.2011) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/17.0.940.0 Safari/535.8";
const char S63997[] = "Mozilla/5.0 (X11; CrOS i686 1412.245.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.81 Safari/535.11";
const char S63998[] = "Nokia7610/2.0 (7.0642.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.9.1.120/27/351/UCWEB";
const char S63999[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1b3) Gecko/X Firefox/3.1b3";
const char S64000[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI H868C Build/HuaweiH868C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S64001[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E970 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S64002[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.28) Gecko/X Thunderbird/3.1.20";
const char S64003[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S64004[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S64005[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700KL Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S64006[] = "Mozilla/5.0 (Linux; Android 4.1.2; ASUS Transformer Pad TF700T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S64007[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S64008[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S64009[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; U8230 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S64010[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; U8230 Build/ERE27)";
const char S64011[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-pt; U8230 Build/HuaweiU8230) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S64012[] = "ZTE-F555/1.0.0 NetFront/4.2 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64013[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5215 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S64014[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5215 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S64015[] = "Opera/9.02 (Linux armv7l; U; ARCHOS; GOGI; G6H; Version 1.7.22    (WMDRMPD: 10.1) ; en)";
const char S64016[] = "Opera/9.02 (X11; Linux i686; U; en)";
const char S64017[] = "Opera/9.02 (Linux armv5tejl; U; ARCHOS; GOGI; a605; en)";
const char S64018[] = "Opera/9.00 (X11; Linux i686; U; en)";
const char S64019[] = "Opera/9.01 (X11; Linux i686; U; en)";
const char S64020[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.231 Safari/534.10";
const char S64021[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_2; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10";
const char S64022[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_3; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10";
const char S64023[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_0; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10";
const char S64024[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10";
const char S64025[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10";
const char S64026[] = "Mozilla/5.0 (X11; CrOS i686 14.811.89) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.157 Safari/535.1";
const char S64027[] = "Mozilla/5.0 (X11; CrOS i686 14.811.132) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1";
const char S64028[] = "Mozilla/5.0 (X11; CrOS i686 14.811.117) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1";
const char S64029[] = "Mozilla/5.0 (X11; CrOS i686 14.811.154) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.204 Safari/535.1";
const char S64030[] = "Mozilla/5.0 (X11; CrOS i686 14.805.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.832.0 Safari/535.1";
const char S64031[] = "Mozilla/5.0 (X11; CrOS i686 14.811.47) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.98 Safari/535.1";
const char S64032[] = "Mozilla/5.0 (X11; CrOS i686 14.811.61) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.110 Safari/535.1";
const char S64033[] = "Mozilla/5.0 (X11; CrOS i686 14.811.105) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.163 Safari/535.1";
const char S64034[] = "Mozilla/5.0 (X11; CrOS i686 14.774.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.825.0 Safari/535.1";
const char S64035[] = "Mozilla/5.0 (X11; CrOS i686 14.811.77) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.125 Safari/535.1";
const char S64036[] = "Mozilla/5.0 (X11; CrOS i686 14.642.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.789.0 Safari/535.1";
const char S64037[] = "Mozilla/5.0 (X11; CrOS i686 14.811.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.834.0 Safari/535.1";
const char S64038[] = "Mozilla/5.0 (X11; CrOS i686 14.811.18) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.19 Safari/535.1";
const char S64039[] = "Mozilla/5.0 (X11; CrOS i686 14.811.34) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.87 Safari/535.1";
const char S64040[] = "Mozilla/5.0 (X11; CrOS i686 14.811.26) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.25 Safari/535.1";
const char S64041[] = "Mozilla/5.0 (X11; CrOS i686 14.751.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.818.0 Safari/535.1";
const char S64042[] = "Mozilla/5.0 (X11; CrOS i686 14.811.15) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.16 Safari/535.1";
const char S64043[] = "Mozilla/5.0 (X11; CrOS i686 14.667.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.800.0 Safari/535.1";
const char S64044[] = "Mozilla/5.0 (X11; CrOS i686 14.751.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.818.0 Safari/535.1";
const char S64045[] = "Mozilla/5.0 (X11; CrOS i686 14.775.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.825.0 Safari/535.1";
const char S64046[] = "Mozilla/5.0 (X11; CrOS i686 14.787.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.827.10 Safari/535.1";
const char S64047[] = "Mozilla/5.0 (X11; CrOS i686 14.811.62) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.110 Safari/535.1";
const char S64048[] = "Mozilla/5.0 (X11; CrOS i686 14.688.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.804.0 Safari/535.1";
const char S64049[] = "Mozilla/5.0 (X11; CrOS i686 14.700.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.807.0 Safari/535.1";
const char S64050[] = "Mozilla/5.0 (X11; CrOS i686 14.712.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.807.0 Safari/535.1";
const char S64051[] = "Mozilla/5.0 (X11; CrOS i686 14.771.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.825.0 Safari/535.1";
const char S64052[] = "Mozilla/5.0 (X11; CrOS i686 14.811.64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.112 Safari/535.1";
const char S64053[] = "Mozilla/5.0 (X11; CrOS i686 14.759.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.822.0 Safari/535.1";
const char S64054[] = "Mozilla/5.0 (X11; CrOS i686 15.813.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.4 Safari/535.1";
const char S64055[] = "Mozilla/5.0 (X11; U; CrOS i686 0.13.507) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.94 Safari/535.1";
const char S64056[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9082 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S64057[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT890 Build/8.7.1I-110_SML-20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S64058[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire SV Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S64059[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S64060[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S64061[] = "NokiaC2-00/2.0 (03.99) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-00) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S64062[] = "Mozilla/5.0 (compatible; Konqueror/3.5; FreeBSD) KHTML/3.5.10 (like Gecko)";
const char S64063[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NGM Polaris Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64064[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NGM P0laris Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64065[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; NGM_P0laris Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64066[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; NGM_Polaris Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64067[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26ii Build/6.1.A.2.55) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S64068[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I777 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S64069[] = "Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I777 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S64070[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I777 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S64071[] = "Opera/9.80 (Android 2.2.2; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00";
const char S64072[] = "Opera/9.80 (Android 2.2; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.10.254 Version/12.00";
const char S64073[] = "Opera/9.80 (Android 2.2.1; Linux; Opera Tablet/ADR-X; U; en-GB) Presto/2.10.254 Version/12.00";
const char S64074[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A390_ROW Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S64075[] = "LG-T310-Orange/V10b Obigo/Q7.3 MMS/LG-MMS-V1.1/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64076[] = "LG-T310/V100 Obigo/Q7.3 MMS/LG-MMS-V1.1/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64077[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T769 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S64078[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A50 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64079[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; A50 Build/JRO03C)";
const char S64080[] = "Mozilla/5.0 (Series40; Nokia112/03.32; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S64081[] = "LG-KM900-Bouygues/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64082[] = "LG-KM900-orange/V10v Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64083[] = "LG-KM900F/V10a Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64084[] = "LG-KM900/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64085[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Transformer TF101 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S64086[] = "JUC (Linux; U; 2.3.6; zh-cn; GT-S5660; 320*480) UCWEB7.9.0.94/139/352";
const char S64087[] = "JUC (Linux; U; 2.2.1; zh-cn; GT-S5660; 320*480) UCWEB7.9.0.94/139/352";
const char S64088[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ601 Build/I.7.1-45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S64089[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.1) Gecko/X Firefox/3.1b3;MEGAUPLOAD 1.0 (.NET CLR 3.5.30729)";
const char S64090[] = "Mozilla/5.0 (Windows NT 5.1; rv:12.0) Gecko/X Thunderbird/12.0.1";
const char S64091[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:12.0) Gecko/X Thunderbird/12.0.1";
const char S64092[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC-X710a Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S64093[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.31782/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S64094[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.31782/28.3782; U; en) Presto/2.8.119 Version/11.10";
const char S64095[] = "NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac1-01) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S64096[] = "Opera/12.80 (Windows NT 6.1; U; en) Presto/2.9.168 Version/12.80";
const char S64097[] = "Opera/12.80 (Windows NT 6.1; U; en) Presto/2.10.289 Version/12.08";
const char S64098[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae72-1) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile  3gpp-gba";
const char S64099[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE72-1) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S64100[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE72-2) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S64101[] = "Opera/9.80 (Android 4.1.1; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S64102[] = "Opera/9.80 (Android 4.1.2; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S64103[] = "Opera/9.80 (Android 4.1.1; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10,gzip(gfe)";
const char S64104[] = "Opera/9.80 (Android 4.1; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S64105[] = "Opera/9.80 (Android 4.1.2; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10,gzip(gfe)";
const char S64106[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E988 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S64107[] = "Opera/9.80 (Windows NT 6.2; U; en) Presto/2.9.168 Version/11.50";
const char S64108[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4";
const char S64109[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4";
const char S64110[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; Lenovo S560; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64111[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lenovo S560 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64112[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo S560 Build/IMM76D)";
const char S64113[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64114[] = "Dalvik/1.4.0 (Linux; U; Android 4.1.1; GT-S5830i Build/GINGERBREAD)";
const char S64115[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; Xtreamer_Mobile_AiKi Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64116[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; Xtreamer Mobile Aiki4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64117[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Xtreamer_Mobile_AiKi Build/IML74K)";
const char S64118[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; Aiki-5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64119[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S64120[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S64121[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC-Z710a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S64122[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S64123[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; lenovo A820 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64124[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Lenovo A820t Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64125[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Lenovo A820t Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S64126[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Lenovo A820 Build/JZO54K)";
const char S64127[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; LNV-Lenovo_A820e/S-2-03; 960*540; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1.2 Mobile Safari/534.30";
const char S64128[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Lenovo A820t Build/JZO54K)";
const char S64129[] = "Mozilla/5.0 (Linux; U; Android 4.1.2;en-us; Lenovo A820 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30;";
const char S64130[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Lenovo A820e Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64131[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P708g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S64132[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P705 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S64133[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P705g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S64134[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P705f Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S64135[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; ARM; Trident/6.0; Touch)";
const char S64136[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; ARM; Trident/6.0; Touch; ARMBJS)";
const char S64137[] = "Mozilla/5.0 (Linux; Android 4.1.2; B1-A71 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S64138[] = "Mozilla/5.0 (Linux; Android 4.2.2; MB865) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S64139[] = "Mozilla/5.0 (Linux; Android 4.2.2; Transformer Prime Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S64140[] = "Mozilla/5.0 (Linux; Android 4.2.2; Transformer Pad Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S64141[] = "Mozilla/5.0 (Linux; Android 4.2.1; v89_zlh_hd Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S64142[] = "Mozilla/5.0 (Linux; Android 4.2.1; AT10LE-A Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S64143[] = "Mozilla/5.0 (Linux; Android 4.2.2; M6 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S64144[] = "Mozilla/5.0 (Linux; Android 4.2.2; R821 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S64145[] = "Mozilla/5.0 (Linux; Android 4.2.1; Lenovo S820_ROW Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S64146[] = "Mozilla/5.0 (Linux; Android 4.2.1; Primo_NX Build/WALTONWALTON) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S64147[] = "Mozilla/5.0 (Linux; Android 4.2.2; ME302KL Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S64148[] = "Mozilla/5.0 (Linux; Android 4.2.1; Aveo NOTE II+ Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S64149[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SM-T217A Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S64150[] = "Mozilla/5.0 (Linux; Android 4.2.1; A240 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S64151[] = "Mozilla/5.0 (Linux; Android 4.2.1; STAIRWAY Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S64152[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9197 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S64153[] = "Mozilla/5.0 (Linux; Android 4.2.2; S510 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S64154[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC_PO582 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S64155[] = "Mozilla/5.0 (Linux; Android 4.2.1; M470BSE Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S64156[] = "Mozilla/5.0 (Linux; Android 4.2.2; B1-711 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S64157[] = "Mozilla/5.0 (Linux; Android 4.2.2; X50D Build/X50D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S64158[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R960 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S64159[] = "SAMSUNG-GT-B7330/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG-GT-B7330V/XXIJA";
const char S64160[] = "SAMSUNG-GT-B7330R/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S64161[] = "SAMSUNG-GT-B7330B/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S64162[] = "SAMSUNG-GT-B7330Orange/BVJE2 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S64163[] = "SAMSUNG-GT-B7330/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S64164[] = "NokiaN73/UC Browser8.0.4.121/28/444/UCWEB";
const char S64165[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; IM-A690S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64166[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; IM-A690L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64167[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-; IM-A690S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64168[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; IM-A690S Build/GINGERBREAD)";
const char S64169[] = "Mozilla/5.0 (Windows NT 5.1; rv:10.0.3) Gecko/X Firefox/10.0.3 Epic/10.0.3";
const char S64170[] = "Mozilla/5.0 (Windows NT 5.2; rv:10.0.3) Gecko/X Firefox/10.0.3 Epic/10.0.3";
const char S64171[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8) Presto/2.12.388 Version/12.10";
const char S64172[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T679 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64173[] = "OneBrowser/3.1 (Nokia200/11.81)";
const char S64174[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaC7-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S64175[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S64176[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S64177[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S64178[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC VLE_U Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S64179[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S64180[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717R Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S64181[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S64182[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cherry w500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64183[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M1070 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64184[] = "Mozilla/5.0 (Windows NT 5.1; rv:19.0) Gecko/X Firefox/19.0 IceDragon/19.0.1.0";
const char S64185[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Verzio Envii";
const char S64186[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Sprint:PPC6800";
const char S64187[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Smartphone; 320x240; Mail/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S64188[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Amoi E78/R7.0 NF-Browser/3.2";
const char S64189[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Smartphone; 240x320; SPV E650; OpVer 22.135.1.611";
const char S64190[] = "8900a/1.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S64191[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Sprint:SPH-i325 320x240";
const char S64192[] = "PPC; 240x320; CHT 9100/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S64193[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S64194[] = "AppleCoreMedia/1.0.0.8C148 (iPhone; U; CPU OS 4_2_1 like Mac OS X; en_us)";
const char S64195[] = "AppleCoreMedia/1.0.0.8E200 (iPhone; U; CPU OS 4_2_6 like Mac OS X; en_us)";
const char S64196[] = "AppleCoreMedia/1.0.0.8E401 (iPhone; U; CPU OS 4_2_8 like Mac OS X; en_us)";
const char S64197[] = "AppleCoreMedia/1.0.0.8E600 (iPhone; U; CPU OS 4_2_10 like Mac OS X; en_us)";
const char S64198[] = "AppleCoreMedia/1.0.0.8E303 (iPhone; U; CPU OS 4_2_7 like Mac OS X; en_us)";
const char S64199[] = "AppleCoreMedia/1.0.0.8E128 (iPhone; U; CPU OS 4_2_5 like Mac OS X; en_us)";
const char S64200[] = "AppleCoreMedia/1.0.0.8E501 (iPhone; U; CPU OS 4_2_9 like Mac OS X; en_us)";
const char S64201[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; de; rv:1.9.1.8) Gecko/X SeaMonkey/2.0.3";
const char S64202[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; NookColor Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64203[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6503 Build/10.3.A.0.423) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S64204[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S64205[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S64206[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT901 Build/7.7.1Q-144_SLS_S240-58) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S64207[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_8; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) iCab/4.8 Safari/533.16";
const char S64208[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) iCab/4.8 Safari/533.16";
const char S64209[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ZT ICS Build/ICS.f04ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64210[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; ZT ICS Build/ICS.f04ref.X)";
const char S64211[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S64212[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I727 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S64213[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I727 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S64214[] = "Mozilla/5.0 (Linux; Android 4.1.1; HP Slate 7 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S64215[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9295 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64216[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9295 Build/JDQ39)";
const char S64217[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT300 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S64218[] = "SonyEricssonW850i/R1ED Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S64219[] = "SonyEricssonW850iv/R1ED Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S64220[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Sprint ATP515CKIT Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64221[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Sprint ATP515CKIT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64222[] = "sam-r100 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S64223[] = "Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.4) Gecko Netscape/7.1 (ax)";
const char S64224[] = "Mozilla/5.0 (Windows; U; Win 9x 4.90; en-US; rv:1.4) Gecko/X Netscape/7.1 (ax)";
const char S64225[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaC5-03)/UC Browser8.2.0.132/50/355/UCWEB Mobile";
const char S64226[] = "NokiaC5-03/UCWEB8.2.0.116/50/800";
const char S64227[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30a Build/9.1.A.0.489) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S64228[] = "SonyEricssonW890i/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64229[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; GT-P7100 Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S64230[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; ru-; GT-P7100 Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S64231[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; it-it; GT-P7100 Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S64232[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:15.0) Gecko/X Firefox/15.2 PaleMoon/15.2";
const char S64233[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:15.2) Gecko/X Firefox/15.2.1-x64 PaleMoon/15.2.1-x64";
const char S64234[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; ZTE T81 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64235[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P5113 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S64236[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P5113 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S64237[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus S 4G Build/JRO03R) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S64238[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus S 4G Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S64239[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus S 4G Build/JRO03R) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S64240[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.3 Mobile/9B206 Safari/7534.48.3";
const char S64241[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.3 Mobile/9B176 Safari/7534.48.3";
const char S64242[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/10B141 Safari/7534.48.3";
const char S64243[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/10B329 Safari/7534.48.3";
const char S64244[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/10B146 Safari/7534.48.3";
const char S64245[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Incredible S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S64246[] = "SAMSUNG-SGH-A107/A107UCJJ1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S64247[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:16.0) Gecko/X Thunderbird/16.0.2";
const char S64248[] = "NokiaC3-00/5.0 (08.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac3-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S64249[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.0.0.260";
const char S64250[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.0.1.303";
const char S64251[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.0.0.290";
const char S64252[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.0.1.317";
const char S64253[] = "SonyEricssonK850i/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64254[] = "MOT-RAZRV3xxv/99.41.04ER BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163189) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S64255[] = "MOT-RAZRV3xx/98.30.72AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12003079) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S64256[] = "MOT-RAZRV3xx/96.66.72AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11033063) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S64257[] = "MOT-RAZRV3xx/96.B0.0AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11073156) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [en]";
const char S64258[] = "MOT-RAZRV3xx/99.41.08R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163225) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S64259[] = "MOT-RAZRV3xx/99.31.08R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12153195) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S64260[] = "MOT-RAZRV3xx/96.71.95R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11063081) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [en]";
const char S64261[] = "MOT-RAZRV3xx/96.B0.0FR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11073256) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [en]";
const char S64262[] = "MOT-RAZRV3xx/96.A0.0ER BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11073121) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S64263[] = "MOT-RAZRV3xx/99.41.0CR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163234) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S64264[] = "MOT-RAZRV3xx/99.41.04ER2 BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163189) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [it]";
const char S64265[] = "MOT-RAZRV3xxv/98.50.11R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12093118) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S64266[] = "MOT-RAZRV3xx/96.80.55AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11073083) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S64267[] = "MOT-RAZRV3xx/96.80.53R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11073071) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S64268[] = "MOT-RAZRV3xxv/98.20.33BR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12043064) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S64269[] = "MOT-RAZRV3xxv/98.50.11R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; X) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S64270[] = "MOT-RAZRV3xx/96.B0.0FR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; X) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [vi]";
const char S64271[] = "BlackBerry8230/4.6.1.234 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105";
const char S64272[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Incredible 2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64273[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Incredible 2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64274[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Incredible 2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64275[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Incredible 2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64276[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Incredible 2 3D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64277[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Incredible 2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64278[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Incredible 2 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64279[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; Incredible 2 Build/GRI40)";
const char S64280[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Incredible 2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64281[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; AN9G2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64282[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.1; AN9G2 Build/GINGERBREAD)";
const char S64283[] = "JUC (Linux; U; 4.0.4; zh-cn; LT18i; 480*854) UCWEB7.9.0.94/139/352";
const char S64284[] = "JUC (Linux; U; 2.3.4; zh-cn; LT18i; 480*854) UCWEB7.9.0.94/139/352";
const char S64285[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-i705 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S64286[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Micromax P350 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64287[] = "Zeus 6003 Webster Pro V2.9 Win32";
const char S64288[] = "Zeus 74490 Webster Pro V2.9 Win32";
const char S64289[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT918 Build/2_26D_2009) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S64290[] = "Mozilla/5.0 (Linux; Android 4.1.2; A510 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S64291[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E971 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S64292[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SPH-M910 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S64293[] = "SAMSUNG-SGH-A897/A897UCJC1 SHP/VPP/R5 Jasmine/1.0 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S64294[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-style Q3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64295[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-style Q3i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64296[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; i-mobile i-style Q3 Build/IMM76D)";
const char S64297[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; it-IT; rv:1.7.5) Gecko/X Netscape/8.1";
const char S64298[] = "Opera/9.80 (Android 4.2; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S64299[] = "Opera/9.80 (Android 4.2.1; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S64300[] = "Opera/9.80 (Android 4.2.2; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S64301[] = "Mozilla/5.0 (Linux; Android 4.2.2; zh-cn; SAMSUNG GT-I9502 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S64302[] = "SAMSUNG-SGH-U900/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S64303[] = "SAMSUNG-SGH-U900-Vodafone/U900BUHD6 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S64304[] = "SAMSUNG-SGH-U900T/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S64305[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.8.0.14eol) Gecko/X Red Hat/1.0.9-40.el4 SeaMonkey/1.0.9";
const char S64306[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.0.7) Gecko/X SeaMonkey/1.0.5";
const char S64307[] = "Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabA2109A Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S64308[] = "Mozilla/5.0 (Vodafone/1.0/LG-GT505/V10b Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S64309[] = "Mozilla/5.0 (Vodafone/1.0/LG-GT505/V10c Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S64310[] = "LG/GT505/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64311[] = "LG-GT505-Orange/V10f Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64312[] = "Mozilla/5.0 (Vodafone/1.0/LG-GT505/V10d Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S64313[] = "Mozilla/5.0 (Vodafone/1.0/LG-GT505/V10e Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S64314[] = "LG/GT505e/v10c Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64315[] = "LG-GT505/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64316[] = "LG-GT505/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-gt505) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S64317[] = "LG-GT505/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; vi; lg-gt505) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S64318[] = "LG-GT505/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gt505) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S64319[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; dL1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64320[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17";
const char S64321[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17";
const char S64322[] = "NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-01) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S64323[] = "Nokia112/2.0 (03.32) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64324[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Curie Build/1.0.1 X-19:40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64325[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Curie Build/1.0.2 X-19:30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64326[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Curie Build/1.0.1 X-14:56) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64327[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Curie Build/1.0.4 X-16:15) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64328[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Curie Build/1.1.0 X-14:50) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64329[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; bq Curie Build/1.1.0 X-14:50)";
const char S64330[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Curie Build/1.0.0 X-17:25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64331[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Curie Build/1.0.3 X-13:48) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64332[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Curie 2 Build/1.0.0_X-18:13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64333[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Curie 2 3G Build/1.0.1_X-16:30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64334[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Curie 2 3G Build/1.0.0_X-14:13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64335[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; G3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64336[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; G3 Build/IMM76D)";
const char S64337[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; AOLE G3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64338[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E975K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S64339[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S64340[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S64341[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E975K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S64342[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S64343[] = "LG-GT350/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64344[] = "LG-GT350i/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64345[] = "LG-GT350F/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64346[] = "LG-GT350-Orange/v10b Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64347[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/12.04 Chromium/22.0.1229.79 Chrome/22.0.1229.79 Safari/537.4";
const char S64348[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/12.04 Chromium/22.0.1229.79 Chrome/22.0.1229.79 Safari/537.4";
const char S64349[] = "Nokia306/2.0 (05.93) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia306) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S64350[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS930 4G Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S64351[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S64352[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S64353[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S64354[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2S Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S64355[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2SC Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S64356[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2A Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S64357[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2C Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S64358[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Ubuntu/11.04 Chromium/10.0.648.205 Chrome/10.0.648.205 Safari/534.16";
const char S64359[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Ubuntu/11.04 Chromium/10.0.648.205 Chrome/10.0.648.205 Safari/534.16";
const char S64360[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-07D Build/V16R23B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64361[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-07D Build/V17R24B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64362[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-07D Build/F0001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64363[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; F-07D Build/V17R24B)";
const char S64364[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-07D Build/V18R25A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64365[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Vandroid T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64366[] = "HUAWEI_T8951_TD/1.0 Android/4.0.4 (Linux; U; Android 4.0.4; zh-cn) Release/05.31.2012 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30";
const char S64367[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8951 Build/HuaweiT8951) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S64368[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8951 Build/HuaweiT8951) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64369[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI T8951 Build/HuaweiT8951)";
const char S64370[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySOL22 Build/10.2.F.3.27) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64371[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonySOL22 Build/10.2.F.2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64372[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySOL22 Build/10.2.F.3.43) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64373[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SOL22 Build/10.2.F.3.43) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64374[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SOL22 Build/10.2.F.3.43)";
const char S64375[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySOL22 Build/10.2.F.3.81) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64376[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS950 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S64377[] = "PantechP2020/JIUS05172010R NetFront/3.5 OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64378[] = "PantechP2020/JIUSX NetFront/3.5 OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64379[] = "Mozilla/5.0 (Linux; U; Android Donut; en-us; HTC_Lexikon_C Build/Donut) AppleWebKit/528.2+(KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 CSOUTH-6325";
const char S64380[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; Dell Venue Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/venue.408.0";
const char S64381[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; Dell Venue Build/GRI40)";
const char S64382[] = "Mozilla/5.0 (Linux; U; Android 2.4.27; en-us; Dell Venue Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64383[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; Dell Venue Build/GWK74; CM-7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64384[] = "Mozilla/5.0 (Linux; Android 4.1.1; MZ601 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S64385[] = "Mozilla/5.0 (Linux; Android 4.1.2; MZ601 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S64386[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; T-17B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64387[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; T-17B Build/JRO03H)";
const char S64388[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMP3670B Build/PMP3670B_X_v1.0.9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64389[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMP3670B-WH Build/PMP3670B-WH_X_v2.0.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64390[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S64391[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; Eris Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S64392[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; Eris xtrROM5.0.1 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S64393[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-; Eris Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S64394[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; Eris Build/ERD79) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S64395[] = "Dalvik/1.1.0 (Linux; U; Android 2.1; Eris Build/ERD79)";
const char S64396[] = "NexianNX-G857/MTK Release/10.1.2009 Browser/MAUI Profile";
const char S64397[] = "NexianNX-G252/MTK Release/01.01.2010 Browser/MAUI Profile";
const char S64398[] = "NexianNX-G265/MTK Release/15.02.2011 Browser/MAUI Profile";
const char S64399[] = "NexianNX-G256/MTK Release/01.01.2010 Browser/MAUI Profile";
const char S64400[] = "NexianNX-G315/MTK Release/10.1.2009 Browser/MAUI Profile";
const char S64401[] = "NexianNX-G236/MTK Release/01.01.2010 Browser/MAUI Profile";
const char S64402[] = "NexianNX-G268/MTK Release/15.02.2011 Browser/MAUI Profile";
const char S64403[] = "NexianNX-G262/MTK Release/19.04.2011 Browser/MAUI Profile";
const char S64404[] = "NexianNX-G311/MTK Release/30.05.2011 Browser/MAUI Profile";
const char S64405[] = "Nexian G263/MTK Release/01.01.2012 Browser/MAUI Profile";
const char S64406[] = "NexianNX-G332/MTK Release/22.11.2007 Browser/MAUI Profile";
const char S64407[] = "NexianNXG311T/MTK Release/20.09.2011 Browser/MAUI Profile";
const char S64408[] = "Nexian G257/MTK Release/01.01.2012 Browser/MAUI Profile";
const char S64409[] = "NexianNX-G253/MTK Release/01.01.2010 Browser/MAUI Profile";
const char S64410[] = "NexianNX-G506/MTK Release/01.01.2011 Browser/MAUI Profile";
const char S64411[] = "CKT50_COSMOS_11B_HW (MRE\\2.5.00(2500) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) RECTANGLE-S03A_FLY_L2EN_206_X Release/2012.10.26 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US";
const char S64412[] = "CKT50_COSMOS_11B_HW (MRE\\2.5.00(2500) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) COCOA-S04A_FLY_L2EN_205_X Release/2012.10.15 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US";
const char S64413[] = "J2TECH50_FTE_6464_11B_HW (MRE\\3.0.00(3096) resolution\\X chipset\\MT6250 touch\\0 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) Fly_MV264_V12 Release/2012.10.16 WAP Browser/MAUI (HTTPS) Profile/  Q03C1-2.40 en-US";
const char S64414[] = "J2TECH50_FTE_6464_11B_HW (MRE\\3.0.00(3096) resolution\\X chipset\\MT6250 touch\\0 tpannel\\0 camera\\1 gsensor\\0 keyboard\\normal) Fly_MV264_V12 Release/2012.10.29 WAP Browser/MAUI (HTTPS) Profile/  Q03C1-2.40 en-US";
const char S64415[] = "Opera/9.80 (Android; Linux; Opera Mobi/ADR-X; U; de) Presto/2.5.28 Version/10.1";
const char S64416[] = "Opera/9.80 (Android; Linux; Opera Mobi/ADR-X; U; TSB_CLOUD_COMPANION;TOSHIBA_AC_AND_AZ; en) Presto/2.5.28 Version/10.1";
const char S64417[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; BLU DASH Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64418[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DASH MUSIC J101 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64419[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S64420[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fa-fa; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3392; U; fa) Presto/2.8.119 Version/11.10";
const char S64421[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3445; U; en) Presto/2.8.119 Version/11.10";
const char S64422[] = "Nokia6300/2.0 (07.00) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6300) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S64423[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6300) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S64424[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6603 Build/10.1.1.A.1.253) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S64425[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S64426[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S64427[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Acer; Allegro; bouygues)";
const char S64428[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Acer; Allegro)";
const char S64429[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Acer; Allegro) [UsableNet Lift Mobile]";
const char S64430[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Karbonn A30 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64431[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Karbonn A30 Build/IMM76D)";
const char S64432[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 Sleipnir/4.1.3";
const char S64433[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36 Sleipnir/4.1.4";
const char S64434[] = "Mozilla/5.0 (Windows NT 4.10; rv:22.0) Gecko/X Firefox/22.0";
const char S64435[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; pandigitalp1hr) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S64436[] = "MOT-EX118 MIDP-2.0/CLDC-1.1 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U) Presto/2.5.28 Version/10.10";
const char S64437[] = "Nokia300/5.0 (07.03) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia300) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S64438[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I415 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S64439[] = "Mercury/7.1 CFNetwork/609 Darwin/13.0.0";
const char S64440[] = "Mozilla/5.0 (Linux; Android 4.1.1; XOLO A1000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S64441[] = "Mozilla/5.0 (Linux; Android 4.1.2; IdeaTabA1000-F Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S64442[] = "Mozilla/5.0 (Linux; Android 4.1.2; IdeaTabA1000-G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S64443[] = "Nokia200/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; kau; nokia200) UCBrowser8.2.1.144/70/355/UCWEB Mobile";
const char S64444[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaC5-03)/UC Browser8.5.0.183/50/355/UCWEB Mobile";
const char S64445[] = "NokiaC5-03/UCWEB8.5.0.163/50/800";
const char S64446[] = "Mozilla/5.0 (Linux; Android 4.0.4; WIKO-CINK SLIM Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S64447[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XOLO A700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64448[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; XOLO A700 Build/IMM76D)";
const char S64449[] = "Mozilla/5.0 (Linux; Android 4.1.2; ME371MG Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S64450[] = "Mozilla/5.0 (Linux; Android 4.1.2; PadFone Infinity Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S64451[] = "Mozilla/5.0 (Linux; Android 4.1.2; PadFone Infinity Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S64452[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N7100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S64453[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7100 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S64454[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-N719 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S64455[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7105 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S64456[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-pt; soft stone Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S64457[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; ZTE-RACER Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S64458[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; ZTE-RACER Build/ERE27)";
const char S64459[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ALCATEL ONE TOUCH 5035X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S64460[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; ALCATEL ONE TOUCH 5035D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S64461[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ALCATEL ONE TOUCH 5035E Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S64462[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ZTE-Z993 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64463[] = "Mozilla/5.0 (Linux; Android 4.1.2; PantechP9090 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S64464[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8010 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S64465[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT560 Build/V1.67A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64466[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; pt-br; XT560 Build/V1.66A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64467[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; pt-br; XT560 Build/V1.59B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64468[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; pt-br; XT560 Build/V1.59D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64469[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; chromeframe/22.0.1229.94)";
const char S64470[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/22.0.1229.94)";
const char S64471[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/22.0.1229.94) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S64472[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; chromeframe/22.0.1229.94)";
const char S64473[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-I9100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S64474[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-I9100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S64475[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-I9100G Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S64476[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-I9100G Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S64477[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Micromax A30 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64478[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Micromax A30 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64479[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Evo 3D CDMA Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64480[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Evo 3D CDMA Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64481[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Evo 3D GSM Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64482[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Evo 3D CDMA Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64483[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Evo 3D CDMA Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64484[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Evo 3D GSM Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64485[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9300 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S64486[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-I9300 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S64487[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SCH-I535 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S64488[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SGH-T999 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S64489[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-I939D Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S64490[] = "Mozilla/5.0 (Vodafone/1.0/LG-KC910/V10d Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S64491[] = "Mozilla/5.0 (Vodafone/1.0/LG-KC910/V10b Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S64492[] = "Mozilla/5.0 (Vodafone/1.0/LG-KC910/V10c Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S64493[] = "Mozilla/5.0 (Vodafone/1.0/LG-KC910/V10g Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S64494[] = "Mozilla/4.0 (Vodafone/1.0/LG-KC910/V08e Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S64495[] = "Mozilla/5.0 (Vodafone/1.0/LG-KC910/V10h Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S64496[] = "LG-KC910-v10e Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64497[] = "LG-KC910-v10b Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64498[] = "LG-KC910-v10h Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64499[] = "LG-KC910Q Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64500[] = "LG-KC910Qa Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64501[] = "LG-KC910-Orange/V10e Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64502[] = "LG-KC910-v10d Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64503[] = "LG-KC910/V10a Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64504[] = "LG/KC910/V10a Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64505[] = "LG/KC910e/V10c Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64506[] = "Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.8.1.24) Gecko/X SeaMonkey/1.1.19";
const char S64507[] = "Mozilla/5.0 (Windows; U; Win 9x 4.90; en-GB; rv:1.8.1.6) Gecko/X SeaMonkey/1.1.4";
const char S64508[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T989 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S64509[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-T989 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S64510[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-T989 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S64511[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S64512[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S64513[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.0.200) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S64514[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.0.200) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S64515[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ONE TOUCH 4011X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S64516[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ONE TOUCH 4011X Build/JRO03C)";
const char S64517[] = "Nokia206/2.0 (03.59) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia206) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S64518[] = "Nokia206.1/2.0 (03.58) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia206.1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S64519[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-M830 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S64520[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT881 Build/9.8.3O-118_USC-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S64521[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; iOCEAN X7 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64522[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; iOCEAN X7 Build/JOP40D)";
const char S64523[] = "Mozilla/5.0 (PLAYSTATION 3 4.45) AppleWebKit/531.22.8 (KHTML, like Gecko)";
const char S64524[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.0.200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S64525[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:11.0) Gecko/X Firefox/11.0,gzip(gfe) (via translate.google.com)";
const char S64526[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:11.0) Gecko/X Firefox/11.0 Firefox/11.0";
const char S64527[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S64528[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141,Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S64529[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146,Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S64530[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S64531[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329,Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S64532[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S64533[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-i705 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S64534[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; LIFE PLAY Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64535[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; LIFE PLAY Build/JOP40D)";
const char S64536[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530X Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64537[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-R530X Build/JZO54K)";
const char S64538[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Incredible S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S64539[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Micromax A56 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64540[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; A56 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64541[] = "Mozilla/5.0 (X11; CrOS x86_64 3083.2.0) AppleWebKit/537.16 (KHTML, like Gecko) Chrome/24.0.1305.4 Safari/537.16";
const char S64542[] = "Mozilla/5.0 (X11; CrOS i686 3083.1.0) AppleWebKit/537.16 (KHTML, like Gecko) Chrome/24.0.1305.3 Safari/537.16";
const char S64543[] = "Mozilla/5.0 (X11; CrOS i686 3020.0.2012) AppleWebKit/537.13 (KHTML, like Gecko) Chrome/24.0.1290.1 Safari/537.13";
const char S64544[] = "Mozilla/5.0 (X11; CrOS i686 3083.2.0) AppleWebKit/537.16 (KHTML, like Gecko) Chrome/24.0.1305.4 Safari/537.16";
const char S64545[] = "Mozilla/5.0 (X11; CrOS x86_64 3083.1.0) AppleWebKit/537.16 (KHTML, like Gecko) Chrome/24.0.1305.3 Safari/537.16";
const char S64546[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:20.0) Gecko/X Firefox/20.0 SeaMonkey/2.17.1";
const char S64547[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; JY-G4 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64548[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; JY-G4 Build/JOP40D)";
const char S64549[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ONDA MID Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64550[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ONDA MID Build/JRO03C)";
const char S64551[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ONDA MID Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64552[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P5110 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S64553[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5110 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S64554[] = "MacOutlook/14.3.0.X (Intel Mac OS X 10.8.2)";
const char S64555[] = "MacOutlook/14.3.1.X (Intel Mac OS X 10.8.2)";
const char S64556[] = "MacOutlook/14.3.2.X (Intel Mac OS X 10.8.2)";
const char S64557[] = "MacOutlook/14.3.2.X (Intel Mac OS X 10.8.3)";
const char S64558[] = "MacOutlook/14.3.1.X (Intel Mac OS X 10.8.3)";
const char S64559[] = "MacOutlook/14.3.4.X (Intel Mac OS X 10.8.3)";
const char S64560[] = "MacOutlook/14.3.4.X (Intel Mac OS X 10.8.4)";
const char S64561[] = "MacOutlook/14.3.5.X (Intel Mac OS X 10.8.4)";
const char S64562[] = "MacOutlook/14.3.5.X (Intel Mac OS X 10.8.3)";
const char S64563[] = "MacOutlook/14.3.2.X (Intel Mac OS X 10.8.4)";
const char S64564[] = "MacOutlook/14.3.6.X (Intel Mac OS X 10.8.4)";
const char S64565[] = "MacOutlook/14.3.7.X (Intel Mac OS X 10.8.4)";
const char S64566[] = "MacOutlook/14.3.6.X (Intel Mac OS X 10.8.5)";
const char S64567[] = "MacOutlook/14.3.7.X (Intel Mac OS X 10.8.5)";
const char S64568[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-S5830 Build/CCGH1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64569[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64570[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-S5830 Build/CCGH2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64571[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-S5830 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64572[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; es-ve; GT-S5830M Build/JELLYBEAN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64573[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-AS730 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S64574[] = "Nokia5130c-2/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia5130c-2) UCBrowser8.2.0.132/69/452/UCWEB Mobile";
const char S64575[] = "SAMSUNG-GT-S5292/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S64576[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S5312 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64577[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S5312 Build/JZO54K)";
const char S64578[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; GT-S5312B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64579[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-S5312L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64580[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E430 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S64581[] = "HD_mini_T5555 Opera/9.7 (Windows NT 5.1; U; en)";
const char S64582[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) 480x800 SAMSUNG SCH-i920 PPC";
const char S64583[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-I9070 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S64584[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-I9070P Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S64585[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7510 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S64586[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P7510 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S64587[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT920 Build/2_310_2009) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S64588[] = "ZTE-F160-Orange/4.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64589[] = "ZTE-F160(Uganda)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S64590[] = "ZTE-F160(Brazil_VIVO)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S64591[] = "ZTE-F160/1.3.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64592[] = "ZTE-F160(orange-20-branches)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64593[] = "ZTE-F160(Venezuela_Movilnet)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S64594[] = "ZTE-F160-Free/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64595[] = "ZTE-F160(Rwanda_MTN)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64596[] = "Mozilla/4.0 (compatible; Linux 2.6.22) NetFront/3.4 Kindle/2.0 (screen 600x800)";
const char S64597[] = "Mozilla/4.0 (compatible; Linux 2.6.22) NetFront/3.4 Kindle/2.3 (screen 600x800; rotate)";
const char S64598[] = "Mozilla/4.0 (compatible; Linux 2.6.22) NetFront/3.4 Kindle/2.5 (screen 600x800; rotate)";
const char S64599[] = "Mozilla/5.0 (Linux; Android 4.0.4; MediaPad 10 FHD Build/HuaweiMediaPad) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S64600[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64601[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A3 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64602[] = "Mozilla/5.0 (Linux; U; Android 2.3;zh-tw; A3 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64603[] = "Karbonn A25 Linux/3.0.13 Android/4.0.4 Release/04.15.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S64604[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; Karbonn A25 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64605[] = "Mozilla/5.0 (Linux; Android 4.0.4; Torque Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S64606[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.2pre) Gecko/X SeaMonkey/1.1";
const char S64607[] = "Mozilla/5.0 (X11; U; OpenVMS COMPAQ_Professional_Workstation; en-US; rv:1.8.1.17) Gecko/X SeaMonkey/1.1.12";
const char S64608[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T01A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64609[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T01A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64610[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; T01A Build/IML74K)";
const char S64611[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; T01A Build/IMM76D)";
const char S64612[] = "Mozilla/5.0 (X11; Linux x86_64; rv:15.0) Gecko/X Thunderbird/15.0.1 Lightning/1.7";
const char S64613[] = "NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax3-02) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S64614[] = "NokiaX3-02.5/5.0 (06.75) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax3-02.5) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S64615[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LW770 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S64616[] = "LGE-UN270/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S64617[] = "iPhone";
const char S64618[] = "Mozilla/5.0 (X11; U; Linux i686; en-us) AppleWebKit/531.2+ (KHTML, like Gecko) Version/5.0 Safari/531.2+ Ubuntu/10.04 LTS () Epiphany/2.30.2";
const char S64619[] = "Mozilla/5.0 (Windows NT 4.10; rv:16.0) Gecko/X Firefox/16.0";
const char S64620[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A12 Build/RheaTK) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64621[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Karbonn A12 Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S64622[] = "Mozilla/5.0 (Linux; Android 4.0.4; T-Hub2 Build/TVA301TELBG3) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S64623[] = "NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-01) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S64624[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S64625[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC_PN071 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S64626[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTCONE Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S64627[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30a Build/9.1.A.0.489) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S64628[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Coast/1.0.0.00000 Mobile/11A4414e Safari/7534.48.3";
const char S64629[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/11A465 Safari/7534.48.3";
const char S64630[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/11A4449d Safari/7534.48.3";
const char S64631[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/11A4449a Safari/7534.48.3";
const char S64632[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/11A4435d Safari/7534.48.3";
const char S64633[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/11A501 Safari/7534.48.3";
const char S64634[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 2_2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Mobile/5H11";
const char S64635[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 2_2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko)";
const char S64636[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 2_2 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko)";
const char S64637[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 2_2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Mobile/5H11a";
const char S64638[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 2_2 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Mobile/5G77a";
const char S64639[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A44 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64640[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5360) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S64641[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; vi; GT-S5360) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S64642[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) Version/6.0.4 Safari/8536.29.13 Sleipnir/4.3.0.1";
const char S64643[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) Version/6.0.5 Safari/8536.30.1 Sleipnir/4.3.1";
const char S64644[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) Version/6.0.3 Safari/8536.28.10 Sleipnir/4.3.1";
const char S64645[] = "LG-GD570AW Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64646[] = "LG-GD570 Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64647[] = "LG-GD570Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64648[] = "Mozilla/5.0 (Windows NT;en-us) Firefox/3.5.9";
const char S64649[] = "Mozilla/5.0 (Windows; U; de) Gecko/X Firefox/3.5 (.NET CLR 3.5)";
const char S64650[] = "Mozilla/5.0 (compatible; MSIE 6.0b; Windows NT 5.0) Gecko/X Firefox/3.0.6 TweetmemeBot";
const char S64651[] = "Mozilla/5.0 (X11; SunOS i86pc; rv:8.0) Gecko/X Firefox/8.0";
const char S64652[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT300 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S64653[] = "Mozilla/5.0 (Linux; Android 4.1.1; E270BSA Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S64654[] = "SAMSUNG-SGH-A177/A177UCIC3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S64655[] = "Mozilla/5.0 (Linux; U; Android 3.1; ja-jp; L-06C Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S64656[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; MID1024 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64657[] = "Dalvik/1.4.0 (Linux; U; Android 2.3; MID1024 Build/GINGERBREAD)";
const char S64658[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; L-01D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64659[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; L-01D Build/GRJ90)";
const char S64660[] = "Liferea/1.8.3 (Linux; en_US.UTF-8; http://liferea.sf.net/)";
const char S64661[] = "Liferea/1.8.8 (Linux; en_GB.utf8; http://liferea.sf.net/)";
const char S64662[] = "Liferea/1.8.3 (Linux; en_GB.UTF-8; http://liferea.sf.net/)";
const char S64663[] = "Liferea/1.8.9 (Linux; en_GB.utf8; http://liferea.sf.net/)";
const char S64664[] = "Liferea/1.8.5 (Linux; en_US.UTF-8; http://liferea.sf.net/)";
const char S64665[] = "Liferea/1.8.3 (Linux; fr_FR.UTF-8; http://liferea.sf.net/)";
const char S64666[] = "Liferea/1.8.6 (Linux; en_US.UTF-8; http://liferea.sf.net/)";
const char S64667[] = "Liferea/1.8.10 (Linux; en_GB.utf8; http://liferea.sf.net/)";
const char S64668[] = "Liferea/1.8.6 (Linux; fr_FR.UTF-8; http://liferea.sf.net/)";
const char S64669[] = "Liferea/1.8.12 (Linux; en_US.UTF-8; http://liferea.sf.net/)";
const char S64670[] = "Liferea/1.8.3 (Linux; de_DE.UTF-8; http://liferea.sf.net/)";
const char S64671[] = "Liferea/1.8.7 (Linux; fr_FR.UTF-8; http://liferea.sf.net/)";
const char S64672[] = "Liferea/1.8.12 (Linux; fr_FR.UTF-8; http://liferea.sf.net/)";
const char S64673[] = "Liferea/1.8.12 (Linux; fr_FR.utf8; http://liferea.sf.net/)";
const char S64674[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-TH; OPPOU7011 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64675[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOU701 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64676[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; U701 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S64677[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; U701 Build/GRK39F)";
const char S64678[] = "Mozilla/5.0 (X11; SunOS i86pc; rv:16.0) Gecko/X Firefox/16.0";
const char S64679[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; FJL21 Build/V23R39X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64680[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; FJL21 Build/V27R40J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64681[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; FJL21 Build/V31R42A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64682[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; FJL21 Build/V35R45B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64683[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; FJL21 Build/V37R47A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64684[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; FJL21 Build/V39R48C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64685[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus S 4G Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64686[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus S 4G Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64687[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus S 4G Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64688[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus S 4G Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/crespo4g";
const char S64689[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.89; en-us; 9000/5.2.0.89) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S64690[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.93; en-us; 9000/5.2.0.93) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S64691[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.64; en-us; 9000/5.2.0.64) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S64692[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.95; en-us; 9000/5.2.0.95) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S64693[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.33; en-us; 9000/5.2.0.33) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S64694[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.57; en-us; 9000/5.2.0.57) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S64695[] = "UCWEB/2.0(BlackBerry; U; 4.0.0.248; en-us; 9000/4.0.0.248) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S64696[] = "UCWEB/2.0(BlackBerry; U; 4.0.0.155; en-us; 9000/4.0.0.155) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S64697[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.76; en-us; 9000/5.2.0.76) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S64698[] = "UCWEB/2.0(BlackBerry; U; 4.0.0.247; en-us; 9000/4.0.0.247) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S64699[] = "UCWEB/2.0(BlackBerry; U; 4.0.0.206; en-us; 9000/4.0.0.206) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S64700[] = "UCWEB/2.0(BlackBerry; U; 5.2.0.41; en-us; 9000/5.2.0.41) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S64701[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMP5780D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64702[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; PMP5780D Build/JRO03H)";
const char S64703[] = "HTC Streaming Player verizon_wwe / 1.0 / inc / 2.2";
const char S64704[] = "HTC Streaming Player htc_europe / 1.0 / htc_saga / 2.3.3";
const char S64705[] = "HTC Streaming Player o2_uk / 1.0 / htc_saga / 2.3.3";
const char S64706[] = "HTC Streaming Player vodafone_uk / 1.0 / htc_saga / 2.3.5";
const char S64707[] = "HTC Streaming Player htc_europe / 1.0 / htc_saga / 2.3.5";
const char S64708[] = "HTC Streaming Player tmous / 1.0 / ville / 4.0.3";
const char S64709[] = "HTC Streaming Player verizon_wwe / 1.0 / htc_blissc / 2.3.4";
const char S64710[] = "HTC Streaming Player cingular_us / 1.0 / evita / 4.0.4";
const char S64711[] = "HTC Streaming Player htc_europe / 1.0 / endeavoru / 4.0.4";
const char S64712[] = "HTC Streaming Player cingular_us / 1.0 / evita / 4.0.3";
const char S64713[] = "HTC Streaming Player htc_europe / 1.0 / endeavoru / 4.1.1";
const char S64714[] = "HTC Streaming Player htc_europe / 1.0 / htc_primou / 4.0.3";
const char S64715[] = "HTC Streaming Player tmous / 1.0 / ville / 4.0.4";
const char S64716[] = "HTC Streaming Player verizon_wwe / 1.0 / dlx / 4.1.1";
const char S64717[] = "HTC Streaming Player verizon_wwe / 1.0 / fireball / 4.0.4";
const char S64718[] = "HTC Streaming Player htc_asia_wwe / 1.0 / endeavoru / 4.1.1";
const char S64719[] = "HTC Streaming Player htc_asia_wwe / 1.0 / htc_primou / 4.0.3";
const char S64720[] = "HTC Streaming Player htc_europe / 1.0 / ville / 4.0.4";
const char S64721[] = "HTC Streaming Player htc_europe / 1.0 / enrc2b / 4.1.1";
const char S64722[] = "HTC Streaming Player htc / 1.0 / m7wls / 4.1.2";
const char S64723[] = "HTC Streaming Player cingular_us / 1.0 / m7 / 4.1.2";
const char S64724[] = "HTC Streaming Player htc_europe / 1.0 / protou / 4.0.4";
const char S64725[] = "HTC Streaming Player cingular_us / 1.0 / evita / 4.1.1";
const char S64726[] = "HTC Streaming Player tmous / 1.0 / m7 / 4.1.2";
const char S64727[] = "HTC Streaming Player htc_europe / 1.0 / ville / 4.1.1";
const char S64728[] = "HTC Streaming Player htc_europe / 1.0 / m7 / 4.1.2";
const char S64729[] = "HTC Streaming Player us_cellular_wwe / 1.0 / htc_lexikon / 2.3.4";
const char S64730[] = "HTC Streaming Player cingular_us / 1.0 / evitareul / 4.1.1";
const char S64731[] = "HTC Streaming Player htc / 1.0 / htc_europe / 4.2.2";
const char S64732[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.5; en-US; rv:1.9.1b3pre) Gecko/X Mozilla/5.0 (Windows; U; Windows NT 5.1; en) AppleWebKit/526.9 (KHTML, like Gecko) Version/4.0dp1 Safari/526.8";
const char S64733[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I717 Build/IMM76I; CyanogenMod-h0tw1r3.r5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64734[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I717 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64735[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS930 4G Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S64736[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; PMP7100D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64737[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID1048 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64738[] = "NokiaN76/CMCC SymbianOS/9.2 Series60/3.1 Release/21.1.004 Mozilla/5.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S64739[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaN76-1/31.0.014 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S64740[] = "NOKIAN76";
const char S64741[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; L-07C Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64742[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; L-07C Build/GRI40)";
const char S64743[] = "Mozilla/5.0 (Linux; Android 4.0.4; SK17i Build/4.1.B.0.431) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S64744[] = "Mozilla/5.0 (Linux; Android 4.0.4; SK17a Build/4.1.B.0.431) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S64745[] = "Mozilla/5.0 (Linux; Android 4.0.4; Sk17i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S64746[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; V8403 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64747[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; V8403 Build/GRK39F)";
const char S64748[] = "Mozilla/5.0 (Linux; Android 4.2.1; HTC One X Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S64749[] = "Mozilla/5.0 (Linux; Android 4.2.1; One X Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S64750[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One X Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S64751[] = "Mozilla/5.0 (Linux; Android 4.2.2; One X Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S64752[] = "Nokia112/2.0 (03.32) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia112) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S64753[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7 HTC_DesireHD-orange-LS; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S64754[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-mx; ALCATEL_one_touch_906Y Build/FSR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64755[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-ph; ALCATEL_one_touch_906 Build/FSR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64756[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.1";
const char S64757[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.2";
const char S64758[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.4";
const char S64759[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.5";
const char S64760[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Flipboard/1.9.4";
const char S64761[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.6";
const char S64762[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.9.6";
const char S64763[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Flipboard/1.9.6";
const char S64764[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.7";
const char S64765[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Flipboard/1.9.7";
const char S64766[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.8";
const char S64767[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Flipboard/1.9.8";
const char S64768[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Flipboard/1.9.8";
const char S64769[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.9.7";
const char S64770[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.9.8";
const char S64771[] = "Mozilla/5.0 (Windows NT 4.10; rv:14.0) Gecko/X Firefox/14.0.1";
const char S64772[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.0 Safari/535.19 Silk-Accelerated=true";
const char S64773[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PAP3500 DUO Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64774[] = "POLARIS/6.01 (BREW 3.1.5; U; en-us; LG; LX265; POLARIS/6.01/WAP) MMP/2.0 profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64775[] = "POLARIS/6.01(BREW 3.1.5;U;en-us;LG;LX265;POLARIS/6.01/WAP;)MMP/2.0 profile/MIDP-201 Configuration /CLDC-1.1";
const char S64776[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; CatNova8 Build/GRI40) AppleWebKit/533.16 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64777[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Polaroid Tablet Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64778[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Polaroid Tablet Build/IML74K)";
const char S64779[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64780[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; T1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64781[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST27i Build/6.1.1.B.1.54) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S64782[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST27a Build/6.1.1.B.1.54) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S64783[] = "Mozilla/5.0 (Linux; U; Android 1.5; es-; FBW1_4 Build/MASTER) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S64784[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; sdkDemo Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S64785[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; SLATE8 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S64786[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; KESAM7 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S64787[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; hero) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S64788[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; SLATE5 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S64789[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; htc_bahamas Build/CRB17) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S64790[] = "Mozilla/5.0 (Linux; U; Android 1.5; tr-tr; SiRF Dream Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S64791[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; Galaxy with MCR 1.2 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S64792[] = "Mozilla/5.0 (Linux; U; Android 1.5; es-us; GT-i5700 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S64793[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; 750ET Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S64794[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; AEBK08FB Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S64795[] = "Lenovo_O1plus_TD/1.0 Ophone/1.5 (Linux; Android 1.5) Release/9.1.2009 Browser/WAP 2.0 (AppleWebKit/528.5+) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S64796[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-; AEBK08FB Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S64797[] = "Mozilla/5.0 (Linux; U; Android 1.5.350; de-de; m7005 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S64798[] = "Mozilla/5.0 (Linux; U; Android 1.5.1.16-RT-X.X; zh-cn; K-Touch E619 Build/AliyunOs-2012) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S64799[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; htc_bahamas Build/CRB17) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S64800[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.1.1.0 Chrome/16.0.912.63 Safari/535.7";
const char S64801[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; ru-ru; PMP3384BRU Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64802[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; PMP3384B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64803[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/148,1.9.6.148,2012-09-07 16:07, gb";
const char S64804[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I317 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/148,1.9.6.148,2012-09-07 16:07, us";
const char S64805[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-I317M Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/148,1.9.6.148,2012-09-07 16:07, ca";
const char S64806[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7105 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/148,1.9.6.148,2012-09-07 16:07, gb";
const char S64807[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L900 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/148,1.9.6.148,2012-09-07 16:07, us";
const char S64808[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, +0100, gb";
const char S64809[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L900 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, -0400, us";
const char S64810[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I317M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, -0400, ca";
const char S64811[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, -0400, us";
const char S64812[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7105 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, +0800, sg";
const char S64813[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I717R Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64814[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I717R Build/JZO54K)";
const char S64815[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9082 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S64816[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I415 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S64817[] = "SonyEricssonW20/R7DA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2";
const char S64818[] = "Mozilla/5.0 Firefox/2.0";
const char S64819[] = "Firefox/2.0";
const char S64820[] = "Mozilla/5.0 (AS) Gecko/X Firefox/2.0";
const char S64821[] = "Mozilla/5.0 (BeOS; U; BeOS BePC; en-US; rv:1.8.1b2) Gecko/X Firefox/2.0b2";
const char S64822[] = "Mozilla/5.0 (X; 0; 000 000 00 0 X; 00000; X) Gecko/X Firefox/2.0.0.20";
const char S64823[] = "Mozilla/5.0 (en) Gecko/X Firefox/2.0.0.4";
const char S64824[] = "Mozilla/5.0 (OS/2; U; Warp 4.5; en-US; rv:1.8.1.9) Gecko/X Firefox/2.0.0.9";
const char S64825[] = "ALCATEL-OT510A/382 ObigoInternetBrowser/Q05A";
const char S64826[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 6.1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)";
const char S64827[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 6.1; SLCC2; .NET CLR 2.0.50727; .NET4.0C)";
const char S64828[] = "SAMSUNG-GT-C3310/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S64829[] = "SAMSUNG-GT-C3310R/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S64830[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.11 (KHTML, like Gecko) Ubuntu/12.10 Chromium/20.0.1132.47 Chrome/20.0.1132.47 Safari/536.11";
const char S64831[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.11 (KHTML, like Gecko) Ubuntu/12.10 Chromium/20.0.1132.47 Chrome/20.0.1132.47 Safari/536.11";
const char S64832[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03R; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64833[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I535 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64834[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T999 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64835[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03R; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64836[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64837[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Android T3i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64838[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Vandroid T3i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64839[] = "Opera/9.80 (X11; Linux x86_64; U; Ubuntu; fr) Presto/2.10.289 Version/12.01";
const char S64840[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Mobile Safari/535.19 Silk-Accelerated=false";
const char S64841[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; V972 Core4 Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64842[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; V972 Core4 Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64843[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; V972 Core4 Build/MASTER)";
const char S64844[] = "Mozilla/5.0 (Linux; Android 4.1.1; A700 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S64845[] = "Mozilla/5.0 (Linux; Android 4.0.4; C1604 Build/11.1.A.0.68) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S64846[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian95) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S64847[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian95_8gb) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S64848[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN95) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S64849[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN95 8GB) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S64850[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S64851[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S64852[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S64853[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5";
const char S64854[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27";
const char S64855[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S64856[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8";
const char S64857[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S64858[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4";
const char S64859[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S64860[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTabS2110AF Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S64861[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTabS2110AH Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S64862[] = "Nokia309/2.0 (05.85) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S64863[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P765 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S64864[] = "Nokia201/2.0 (11.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia201) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S64865[] = "Mozilla/5.0 (Linux; Android 4.2.1; Transformer Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S64866[] = "Mozilla/5.0 (Linux; Android 4.2.1; Transformer TF101 Build/JB_MR1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S64867[] = "Mozilla/5.0 (Linux; Android 4.2.2; Transformer TF101 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S64868[] = "Mozilla/5.0 (Linux; Android 4.2.2; Transformer Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S64869[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; I7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64870[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; I7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64871[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; es-es; I7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64872[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; I7100 Build/JRO03C)";
const char S64873[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-gb; I7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64874[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; de-de; I7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64875[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; HKPHONE-I7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64876[] = "Mozilla/5.0 (compatible; MSIE 6.0; PPC; Win64; IA64; Trident/5.0)";
const char S64877[] = "Mozilla/5.0 (compatible; MSIE 6.0; PPC; WOW64; Trident/4.0)";
const char S64878[] = "Mozilla/5.0 (compatible; MSIE 6.0; PPC; .NET CLR; Trident/1.0)";
const char S64879[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:21.0) Gecko/X Firefox/21.0";
const char S64880[] = "Mozilla/5.0 (Linux; Android 4.3; GT-I9300 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S64881[] = "Mozilla/5.0 (Linux; Android 4.3; SCH-I535 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S64882[] = "Mozilla/5.0 (Linux; Android 4.3; SGH-T999 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S64883[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI MediaPad Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S64884[] = "Mozilla/5.0 (Linux; Android 4.0.3; MediaPad 7 Lite Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S64885[] = "Mozilla/5.0 (Linux; Android 4.0.4; MediaPad 10 LINK Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S64886[] = "Mozilla/5.0 (Linux; Android 4.0.3; SpringBoard Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S64887[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; G100W Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S64888[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U8812D Build/HuaweiU8812D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64889[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Desire S Build/JOP40D; CyanogenMod-CM-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64890[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Desire S Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64891[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Desire S Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64892[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Desire S Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64893[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9152 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S64894[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S64895[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Ubuntu/10.10 Chromium/7.0.517.44 Chrome/7.0.517.44 Safari/534.7";
const char S64896[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Ubuntu/10.10 Chromium/7.0.517.44 Chrome/7.0.517.44 Safari/534.7";
const char S64897[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB525 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64898[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB525 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64899[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E435 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S64900[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E435f Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S64901[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E435g Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S64902[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-L710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S64903[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.43.58 (KHTML, like Gecko) Version/6.1 Safari/537.43.58";
const char S64904[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.48.2 (KHTML, like Gecko) Version/6.1 Safari/537.48.2";
const char S64905[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.46.3 (KHTML, like Gecko) Version/6.1 Safari/537.46.3";
const char S64906[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.51.3 (KHTML, like Gecko) Version/6.1 Safari/537.51.3";
const char S64907[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.54.1 (KHTML, like Gecko) Version/6.1 Safari/537.54.1";
const char S64908[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.59 (KHTML, like Gecko) Version/6.1 Safari/537.59";
const char S64909[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.66 (KHTML, like Gecko) Version/6.1 Safari/537.66";
const char S64910[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.71 (KHTML, like Gecko) Version/6.1 Safari/537.71";
const char S64911[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTCEVODesign4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S64912[] = "Mozilla/5.0 (Linux; Android 4.0.3; PH44100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S64913[] = "Mozilla/5.0 (Linux; Android 4.2.2; SM-C101 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S64914[] = "Mozilla/5.0 (Linux; U; Android 2.2.9; en-us; X8 Build/HACKDROID Floyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64915[] = "Mozilla/5.0 (Linux; U; Android 2.2.9; en-us; X8 Build/FroyoBread) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64916[] = "Mozilla/5.0 (Linux; U; Android 2.2.9; hr-hr; XPERIA X8 Build/FroyoPro) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64917[] = "Mozilla/5.0 (Linux; U; Android 2.2.9; en-us; X8 Build/unknown) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64918[] = "Mozilla/5.0 (Linux; U; Android 2.2.5; en-us; X8 Build/Floyo by Racht) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64919[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; X8 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64920[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; X8 Build/v.0.20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64921[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; X8 Build/RC1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64922[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; X8 Build/1.0.0-RC1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64923[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; X8 Build/1.0.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64924[] = "Mozilla/5.0 (Linux; U; Android 2.2.5; es-es; X8 Build/Floyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64925[] = "Mozilla/5.0 (Linux; U; Android 2.2.9; en-us; XPERIA X8 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64926[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; X8 Build/v5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S64927[] = "MicromaxX650 ASTRO36_TD/v3 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10";
const char S64928[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MT8000 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64929[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S64930[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5302 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S64931[] = "LGE-VM101-C/1.0 UP.Browser/7.2.7.2.543 (GUI) MMP/2.0";
const char S64932[] = "LGE-VM101/1.0 UP.Browser/7.2.7.2.543 (GUI) MMP/2.0";
const char S64933[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.7) Gecko/X Ubuntu/9.04 (jaunty) Shiretoko/3.5.7";
const char S64934[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1.1) Gecko/X Ubuntu/9.04 (jaunty) Shiretoko/3.5.1";
const char S64935[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S64936[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7105 Build/JOP40D; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64937[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7105 Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64938[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L900 Build/JDQ39; LiquidSmooth-Liquid-JB-v2.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S64939[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7105 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.8/t0lte";
const char S64940[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-N7100 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/n7100";
const char S64941[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L900 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/l900";
const char S64942[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.1m";
const char S64943[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.2m";
const char S64944[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.4.2m";
const char S64945[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.3m";
const char S64946[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.4m";
const char S64947[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/7534.48.3 Sleipnir/2.4.4m";
const char S64948[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.4.4m";
const char S64949[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.5m";
const char S64950[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S64951[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160LV Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S64952[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S64953[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S64954[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S64955[] = "SAMSUNG-GT-C3752/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S64956[] = "Opera/9.80 (Windows 98; U; en) Presto/2.2.15 Version/10.10";
const char S64957[] = "Opera/9.80 (S60; SymbOS; Opera Mobi/SYB-X; U; da) Presto/2.7.81 Version/11.00";
const char S64958[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.10.229 Version/11.62";
const char S64959[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; en) Presto/2.10.289 Version/12.01";
const char S64960[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; en) Presto/2.10.289 Version/12.00";
const char S64961[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Newsmy T3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64962[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Newsmy T3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64963[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S64964[] = "NokiaC5-00UCBrowser/8.7.1.234/28/444/UCWEB";
const char S64965[] = "NokiaC5-00.2UCBrowser/8.7.1.234/28/444/UCWEB";
const char S64966[] = "GoogleEarth/7.0.1.8244(Windows;Microsoft Windows (6.2.9200.0);en;kml:2.2;client:Free;type:default)";
const char S64967[] = "GoogleEarth/7.0.2.8415(Windows;Microsoft Windows (6.2.9200.0);en;kml:2.2;client:Free;type:default)";
const char S64968[] = "GoogleEarth/7.0.2.8415(Windows;Microsoft Windows (6.2.9200.0);de;kml:2.2;client:Free;type:default)";
const char S64969[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.2.9200.0);en;kml:2.2;client:Free;type:default)";
const char S64970[] = "GoogleEarth/7.0.1.8244(Windows;Microsoft Windows (6.2.9200.0);de;kml:2.2;client:Free;type:default)";
const char S64971[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.2.9200.0);de;kml:2.2;client:Free;type:default)";
const char S64972[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.2.9200.0);fr;kml:2.2;client:Free;type:default)";
const char S64973[] = "GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.2.9200.0);nl;kml:2.2;client:Free;type:default)";
const char S64974[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I497 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S64975[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-US780 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S64976[] = "PANTECH-C790/JAUS08312009; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.14866; U; en-US) Opera 9.50";
const char S64977[] = "PANTECH-C790/JAUSX; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.14866; U; es) Opera 9.50";
const char S64978[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322; InfoPath.2; .NET4.0C; .NET4.0E; MS-RTC LM 8) chromeframe/6.0.472.62";
const char S64979[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729) chromeframe/6.0.472.63";
const char S64980[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.1) chromeframe/6.0.472.63";
const char S64981[] = "Mozilla/6.0 (Windows; U; Windows NT 7.0; en-US; rv:1.9.0.8) Gecko/X Firefox/3.0.9 (.NET CLR 3.5.30729)";
const char S64982[] = "Mozilla/5.0 (Windows;U;Windows NT;pl;rv:1.9.0.12) Gecko/X Firefox/3.0.12";
const char S64983[] = "Mozilla/5.0 (Windows; U; Windows NT based; en-US; rv:1.9.0.5) Gecko/X Firefox/3.0.5";
const char S64984[] = "Mozilla/5.0 (Windows; U; Windows NT 7.0; en-US; rv:1.9.0.6) Gecko/X Firefox/3.0.6";
const char S64985[] = "Mozilla/5.0 (Windows; U; en) AppleWebKit/526.9+ (KHTML, like Gecko) FoozKids/1.0 Firefox/3.0";
const char S64986[] = "Mozilla/5.0 (Java; U; en-in; gt-s5233s) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S64987[] = "Mozilla/5.0 (Java; U; vi; gt-s5233s) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S64988[] = "Mozilla/5.0 (Series40; NokiaC2-00/03.82; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S64989[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7510 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S64990[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P7510 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S64991[] = "Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia305) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S64992[] = "HUAWEI-M750/001.00 ACS-NetFront/3.2";
const char S64993[] = "WinWAP/4.2 (Win32) WinWAP-X/4.2.0.290";
const char S64994[] = "WinWAP-SPBE/1.3 (1.3.0.2;Win32)";
const char S64995[] = "WinWAP/3.x (3.x.x.xx; Win32) (Google WAP Proxy/1.0)";
const char S64996[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; MediaPad Build/orange_Tahiti-LS) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S64997[] = "Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; HUAWEI MediaPad Build/orange_Tahiti-LS) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S64998[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.19pre) Gecko/X Prism zdesktop/7.2.2";
const char S64999[] = "Mozilla/5.0 (Symbian3; U; en-us; Nokia701) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile";
const char S65000[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia701) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba";
const char S65001[] = "Nokia701UCBrowser/8.7.1.234/51/355/UCWEB";
const char S65002[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS950 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S65003[] = "Mozilla/5.0 (Linux; U; Android 4.2.1;en-us; Lenovo_K900/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2.1 Mobile Safari/534.30";
const char S65004[] = "Mozilla/5.0 (Linux; U; Android 4.2.1;en-us; Lenovo_K900_ROW/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2.1 Mobile Safari/534.30";
const char S65005[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Lenovo K900 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65006[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Lenovo K900_ROW Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65007[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; Lenovo K900_ROW Build/JOP40D)";
const char S65008[] = "Mozilla/5.0 (Linux; U; Android 4.2.2;zh-cn; Lenovo_K900/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2.2 Mobile Safari/534.30";
const char S65009[] = "Mozilla/5.0 (Linux; Android 4.1.1; Vodafone 875 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S65010[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Ubuntu/10.04 Chromium/8.0.552.224 Chrome/8.0.552.224 Safari/534.10";
const char S65011[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Ubuntu/10.04 Chromium/8.0.552.237 Chrome/8.0.552.237 Safari/534.10";
const char S65012[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia2690) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S65013[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Mobile Safari/535.19 Silk-Accelerated=true";
const char S65014[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S65015[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; AT-AS40SE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65016[] = "AT-AS40SE Linux/3.0.13 Android/4.0.4 Release/04.10.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S65017[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P710 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S65018[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P880 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S65019[] = "LG-GS505Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S65020[] = "LG-GS505 Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S65021[] = "SCH-U350/1.0 NetFront/3.0.22.2.18 (GUI) MMP/2.0";
const char S65022[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.19 (KHTML, like Gecko) Maxthon/1.0.3.1000 Chrome/18.0.1025.142 Safari/535.19";
const char S65023[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/1.0.4.2000 Chrome/21.0.1180.75 Safari/537.1";
const char S65024[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; SmartPad970s2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65025[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; SmartPad970s23G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65026[] = "Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5130c-2) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S65027[] = "Nokia5130/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5130) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S65028[] = "TELECA-/2.0 (BREW 3.1.5; U; EN-US; SAMSUNG; SPH-M800; Teleca/Q05A/INT) MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S65029[] = "Facebook 4110.0 (iPad; iPhone OS 5.1; en_US)";
const char S65030[] = "FeeddlerRSS 1.12.5 (iPad; iPhone OS 5.1.1; en_US)";
const char S65031[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; TQ150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65032[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; TQ150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, como Gecko) versi?n/4.0 Mobile Safari/533.1";
const char S65033[] = "samsung-mini7100/1.0 Linux/2.6.35.7 Android/4.0.4 Release/03.25.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S65034[] = "Mozilla/5.0 (Linux; Android 4.1.1; C1505 Build/11.3.A.0.47) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S65035[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S65036[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S65037[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S65038[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9100G Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S65039[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Comodo_Dragon/4.1.1.11 Chrome/4.1.249.1042 Safari/532.5";
const char S65040[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Comodo_Dragon/4.1.1.12 Chrome/4.1.249.1042 Safari/532.5";
const char S65041[] = "Opera/9.64 (Windows 98; U; en) Presto/2.1.1";
const char S65042[] = "Opera/9.60 (Windows 98; U; en) Presto/2.1.1";
const char S65043[] = "Opera/9.63 (Windows 98; U; en) Presto/2.1.1";
const char S65044[] = "Opera/9.62 (Windows 98; U; en) Presto/2.1.1";
const char S65045[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T708 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65046[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-P500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65047[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I717D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65048[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I717D Build/JZO54K)";
const char S65049[] = "Nokia6131/2.0 (06.10) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S65050[] = "Nokia6131NFC/2.0 (05.12) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S65051[] = "SAMSUNG-SGH-T259/1.0 SHP/VPP/R5 NetFront/4.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S65052[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/50/354/UCWEB";
const char S65053[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/50/351/UCWEB";
const char S65054[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/50/355/UCWEB";
const char S65055[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/28/355/UCWEB";
const char S65056[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.1.120/27/352/UCWEB";
const char S65057[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/28/354/UCWEB";
const char S65058[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/27/351/UCWEB";
const char S65059[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.1.120/27/351/UCWEB";
const char S65060[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/27/354/UCWEB";
const char S65061[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/27/352/UCWEB";
const char S65062[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.1.120/27/354/UCWEB";
const char S65063[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/28/352/UCWEB";
const char S65064[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.1.120/27/401/UCWEB";
const char S65065[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.1.120/27/402/UCWEB";
const char S65066[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.1.120/26/351/UCWEB";
const char S65067[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS14SH Build/SC100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65068[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS14SH Build/S3300) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65069[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS14SH Build/S2200) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65070[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS14SH Build/S6290) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65071[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS14SH Build/S9100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65072[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS14SH Build/S3060) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65073[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5250) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S65074[] = "Nokia5250UCBrowser/8.7.1.234/50/444/UCWEB";
const char S65075[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5250) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile  3gpp-gba";
const char S65076[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9295 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65077[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S65078[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S65079[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S65080[] = "Opera/9.10 (Windows NT 5.0; U; ru)";
const char S65081[] = "Opera/9.12 (Windows NT 5.0; U; ru)";
const char S65082[] = "Mozilla/5.0 (Windows NT 4.10; rv:13.0) Gecko/X Firefox/13.0.1";
const char S65083[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11 Comodo_Dragon/20.1.1.0";
const char S65084[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11 Comodo_Dragon/20.1.1.0";
const char S65085[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; T-10B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65086[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TECNO_N7 Build/TECNOTECNO_N7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65087[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; TECNO_N7 Build/TECNOTECNO_N7)";
const char S65088[] = "NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-00) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S65089[] = "NokiaC2-00/2.0 (03.82) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-00) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S65090[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8010 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S65091[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E970 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S65092[] = "stagefright/1.1 (Linux;Android 3.2.4)";
const char S65093[] = "stagefright/1.1 (Linux;Android 3.2)";
const char S65094[] = "stagefright/1.1 (Linux;Android 3.2.1)";
const char S65095[] = "Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; FAR7 Build/F0001) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S65096[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; eeepc Build/HLK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S65097[] = "stagefright/1.1 (Linux;Android 3.2.2)";
const char S65098[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-nz; CORE_Speed_Z4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65099[] = "stagefright/1.1 (Linux;Android 3.2.6)";
const char S65100[] = "Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; FAR7 Build/V04R24A) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S65101[] = "Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; FAR7 Build/V03R22B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S65102[] = "Mozilla/5.0 (Android 3.2; en-us) AppleWebKit/1.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.1";
const char S65103[] = "stagefright/1.1 (Linux;Android 3.2.1) Model Number,V55 Software Version,V55V1.1.0B12";
const char S65104[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1";
const char S65105[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1";
const char S65106[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A5Classic Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65107[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Mobile Safari/535.19 Silk-Accelerated=true";
const char S65108[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S65109[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S65110[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mstar) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S65111[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; generic) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S65112[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; opera) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S65113[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; spice) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S65114[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mozilla) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S65115[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S65116[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S65117[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mstar) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S65118[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/9.0.0.295 U2/1.0.0 Mobile";
const char S65119[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; generic) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S65120[] = "UCWEB/2.0 (Java; U; MIDP-2.0; vi; maui) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S65121[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; jbed-fastbcc) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S65122[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui browser) U2/1.0.0 UCBrowser/9.0.0.295 U2/1.0.0 Mobile";
const char S65123[] = "UCWEB/2.0 (Java; U; MIDP-2.0; id; maui) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S65124[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-S7270 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S65125[] = "Mozilla/5.0 (Linux; Android 4.2.2; th-th; SAMSUNG GT-S7270L Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S65126[] = "Mozilla/5.0 (X11; U; Linux; C -) AppleWebKit/532.4 (KHTML, like Gecko) PhantomJS/1.4.1 Safari/532.4";
const char S65127[] = "Mozilla/5.0 (X11; U; Linux x86_64; C) AppleWebKit/533.3 (KHTML, like Gecko) PhantomJS/1.4.1 Safari/533.3";
const char S65128[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.4.0 Safari/534.34";
const char S65129[] = "Mercury/7.0.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S65130[] = "Nokia3110c/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia3110c) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S65131[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:19.0) Gecko/X Firefox/19.0 SeaMonkey/2.16.2";
const char S65132[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S65133[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.53463";
const char S65134[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; PMP7280C Build/PMP7280C_X_v1.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65135[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; PMP7280C Build/PMP7280C_X_v1.0.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65136[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; PMP7280C Build/PMP7280C_X_v1.0.0)";
const char S65137[] = "Nokia201/2.0 (11.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia201) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S65138[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I747 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65139[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I747 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65140[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L710 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65141[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L710 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65142[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I747 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65143[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I747 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65144[] = "Dalvik/1.6.0 (Linux; U; Android 4.3; SAMSUNG-SGH-I747 Build/JSS15Q)";
const char S65145[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I747 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65146[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML, like Gecko) Comodo_Dragon/11.4.0.0 Chrome/11.0.696.71 Safari/534.24";
const char S65147[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Comodo_Dragon/11.4.0.0 Chrome/11.0.696.71 Safari/534.24";
const char S65148[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; HS-7DTB6-4GB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65149[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ENERGY s7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65150[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S7 Build/Smartfren.andromaxtab7.0.v.41.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65151[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65152[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; ELIYA_S7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65153[] = "Mozilla/5.0 (Linux; Android 4.0.4; LePanII Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S65154[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P3113 Build/JOP40G) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S65155[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P3113 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S65156[] = "Mozilla/5.0 (Linux; Android 4.1.1; Orange Daytona Build/HuaweiG510-0200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S65157[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0100 Build/HuaweiG510-0100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S65158[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0200 Build/HuaweiG510-0200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S65159[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0251 Build/HuaweiG510-0251) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S65160[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S65161[] = "Mozilla/5.0 (Series40; NokiaX3-02/07.51; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S65162[] = "Mozilla/5.0 (Series40; NokiaX3-02.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S65163[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I535 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.17 Mobile Safari/537.36";
const char S65164[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S65165[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T999 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S65166[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile";
const char S65167[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile";
const char S65168[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I8150 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65169[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile IQ 5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65170[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; i-mobile IQ 5 Build/IMM76D)";
const char S65171[] = "Mozilla/5.0 (Linux; Android 4.2.2; Xoom Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S65172[] = "Mozilla/5.0 (Linux; Android 4.2.1; Xoom Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S65173[] = "N285/P130D10/Obigo Q03C/MIDP-2.0 Configuration/CLDC-1.1";
const char S65174[] = "TCL602D/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S65175[] = "T-mobile Atlas/EFS-TMO-EE-P107A20V1.0.5 Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S65176[] = "ZMTFTV20/R01 Browser/Obigo/Q03C/WAP2.0";
const char S65177[] = "OPPO A201/4.02/12 Release/2010.6.02 Browser/Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S65178[] = "OPPO A520/4.02/12 Release/2010.6.02 Browser/Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S65179[] = "Grameenphone Q100/X990_V2_Z22_EN_D18F104 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C";
const char S65180[] = "OPPOA125/4.02/12 Release/2010.8.10 Browser/Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S65181[] = "OPPO A115/4.02/12 Release/2010.6.02 Platform/MTK Browser/Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S65182[] = "OPPOF15/4.02/12 Release/2010.8.10 Browser/Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S65183[] = "OPPOT15/4.02/12 Release/2010.8.10 Browser/Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S65184[] = "R228/Obigo Q03C/MIDP-2.0 Configuration/CLDC-1.1";
const char S65185[] = "Vega/P130D10/Obigo Q03C/MIDP-2.0 Configuration/CLDC-1.1";
const char S65186[] = "OPPOA209/4.02/12 Release/2010.8.10 Browser/Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S65187[] = "OPPOA115K/4.02/12 Release/2010.8.10 Browser/Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S65188[] = "Mozilla/5.0 (Linux; U; Android 1.0.15; en-us; A70HB Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S65189[] = "ZTE-Jack 3G-Orange/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S65190[] = "ZTE-E821S/1.0 NetFront/3.5 QTV2.4 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S65191[] = "ZTE-Jack 3G(Telenor)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S65192[] = "ZTE-E821-South-Africa-Vodacom/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S65193[] = "ZTE-E821(Mexico_Iusacell)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S65194[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; SCH-I815 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S65195[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; SCH-I815 4G Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S65196[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; SCH-I815 Build/HTJ85B)";
const char S65197[] = "SAMSUNG Opera/9.80 (J2ME/MIDP; Opera Mini/5.1.25449/30.3558; U; en) Presto/2.8.119 Version/11.10";
const char S65198[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; C8660 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65199[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Mobile Safari/535.19 Silk-Accelerated=true";
const char S65200[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-i705 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S65201[] = "NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac1-02";
const char S65202[] = "NokiaC1-02_CMCC/2.0 (05.40) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S65203[] = "NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S65204[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; ADR6300 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S65205[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5233) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S65206[] = "LG-LGC300/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S65207[] = "LG-C300-Orange/V10a Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S65208[] = "LG-C300/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S65209[] = "SAMSUNG-SGH-A767/A767UCHI4 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S65210[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-in; XCD35 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65211[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-in; XCD35 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S65212[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; XCD35 Build/FRF91)";
const char S65213[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; XCD35 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65214[] = "Mozilla/5.0 (Linux; Android 4.1.1; Barnes & Noble Nook Tablet Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S65215[] = "Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook Tablet Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S65216[] = "Mozilla/5.0 (Linux; Android 4.1.2; NookTablet Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S65217[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; L-01E Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65218[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; L-01E Build/IMM76L)";
const char S65219[] = "Mozilla/5.0 (Windows NT 4.10; rv:17.0) Gecko/X Firefox/17.0";
const char S65220[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3113 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S65221[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P3113 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S65222[] = "NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-03) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S65223[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65224[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65225[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65226[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65227[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SCH-I500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65228[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-I500 Build/IML74K)";
const char S65229[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65230[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65231[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65232[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:17.0) Gecko/X Firefox/17.0 SeaMonkey/2.14.1";
const char S65233[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile IQ 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65234[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile IQ 2A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65235[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; i-mobile IQ 2 Build/IMM76D)";
const char S65236[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:19.0) Gecko/X Firefox/19.0 IceDragon/19.0.1.0";
const char S65237[] = "Mozilla/5.0 (X11; FreeBSD amd64; rv:21.0) Gecko/X Firefox/21.0";
const char S65238[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S65239[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S65240[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-N7000 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S65241[] = "Mozilla/5.0 (Linux; Android 4.0.4; N861 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S65242[] = "Mozilla/5.0 (compatible; Konqueror/4.0; Linux) KHTML/4.0.5 (like Gecko)";
const char S65243[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10_7_2; U; en) Presto/2.9.168 Version/11.60";
const char S65244[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5; U; Edition MacAppStore; en) Presto/2.10.229 Version/11.60";
const char S65245[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F180L Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S65246[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F180K Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S65247[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F180S Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S65248[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B146 Version/6.1 Safari/8536.25 Safari/6533.18.5";
const char S65249[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B144 Version/6.1 Safari/8536.25 Safari/6533.18.5";
const char S65250[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B329 Version/6.1 Safari/8536.25 Safari/6533.18.5";
const char S65251[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5";
const char S65252[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/10b329 Safari/6533.18.5";
const char S65253[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-N5110 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65254[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-N5110 Build/JDQ39)";
const char S65255[] = "Mozilla/5.0 (Linux; Android 4.2.2; BN NookHD+ Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S65256[] = "Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook HD+ Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S65257[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nook HD+ Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S65258[] = "SAMSUNG-SGH-A837/A837UCHH1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S65259[] = "NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-01/UC Browser8.0.3.107/69/444";
const char S65260[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717M Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S65261[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I757M Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S65262[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S65263[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S65264[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S65265[] = "server-bag [iPhone OS,6.0.1,10A523,iPhone4,1]";
const char S65266[] = "iPhone4,1/6.0.1 (10A523)";
const char S65267[] = "iPhone3,1/6.0.1 (10A523)";
const char S65268[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/6.0 Mobile/1A543a Safari/419.3";
const char S65269[] = "iPhone5,2/6.0.1 (10A525)";
const char S65270[] = "iPhone4,1/6.0 (10A403)";
const char S65271[] = "iPhone3,1/6.0 (10A403)";
const char S65272[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/6.0 Mobile/1A522a Safari/419.3";
const char S65273[] = "Mozilla/5.0 (X11; FreeBSD amd64; rv:16.0) Gecko/X Firefox/16.0";
const char S65274[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; GO Live S1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65275[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GO Live S1 Build/GRK39F)";
const char S65276[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; HTC Desire Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65277[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC Desire Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65278[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC Desire Build/VJ AOKP) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65279[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC Desire Build/JDQ39E)";
const char S65280[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LS840 Build/ZVK.IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65281[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; Android Dev Phone 1 Build/DRC83) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S65282[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; Android Dev Phone 1 Build/DRC83) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S65283[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM005SH Build/S4050) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65284[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM005SH Build/S4080) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65285[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM005SH Build/S4040) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65286[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM005SH Build/S4090) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65287[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SBM005SH Build/S4090)";
const char S65288[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM005SH Build/S4100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65289[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM005SH Build/S4120) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65290[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Milestone Build/SHOLS_U2_05.26.3; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65291[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Milestone Build/SHOLS_U2_05.26.3; HO!NO!CM7.2.0-RC0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65292[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; ZTE_U N720 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65293[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; N720 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65294[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; ZTE_U N720 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65295[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ZTE-U N720 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65296[] = "Opera/9.80 (Android 2.2.2; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.8.149 Version/11.10";
const char S65297[] = "Opera/9.80 (Android 2.2; Linux; Opera Mobi/ADR-X; U; pt-BR) Presto/2.8.149 Version/11.10";
const char S65298[] = "Opera/9.80 (Android 2.2.1; Linux; Opera Mobi/ADR-X; U; sv) Presto/2.8.149 Version/11.10";
const char S65299[] = "Opera/9.80 (Android 2.2.3; Linux; Opera Mobi/ADR-X; U; en) Presto/2.8.149 Version/11.10";
const char S65300[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S65301[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S65302[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-N7000 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S65303[] = "Mozilla/5.0 (Linux; Android 4.0.3; A100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S65304[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; Edition MAS; en) Presto/2.10.229 Version/11.67";
const char S65305[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CT920 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65306[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-; Garmin-Asus A10 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1; A10-V3.11.3-user-20100620";
const char S65307[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-au; Garmin-Asus A10 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1; A10-V3.7.5-user-20100525";
const char S65308[] = "Mozilla/4.0 (compatible; MSIE 6.0; Nitro) Opera 8.50 [en]";
const char S65309[] = "Mozilla/5.0 (compatible; OpenWeb 5.7.2-10) Opera 8.54";
const char S65310[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MID_Serails Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65311[] = "Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; Slidepad Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65312[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; lepad_001b Build/PQXU100.4.0097.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65313[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; lepad_001b Build/PQXU100.4.0091.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65314[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; lepad_001b Build/PQXU100.4.0073.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65315[] = "Instagram 3.4.2 (iPod4,1; iPhone OS 6.0.1; en_US; en) AppleWebKit/420+";
const char S65316[] = "Instagram 3.4.2 (iPod4,1; iPhone OS 6.0; en_US; en) AppleWebKit/420+";
const char S65317[] = "yacybot (freeworld/global; x86_64 Mac OS X 10.8.2; java 1.6.0_37; Europe/de) http://yacy.net/bot.html";
const char S65318[] = "yacybot (freeworld/global; x86_64 Mac OS X 10.8.3; java 1.6.0_43; Europe/en) http://yacy.net/bot.html";
const char S65319[] = "yacybot (freeworld/global; x86_64 Mac OS X 10.8.3; java 1.6.0_45; Europe/en) http://yacy.net/bot.html";
const char S65320[] = "RSS Bot/1.0 (Mac OS X Version 10.8.3 (Build 12D78))";
const char S65321[] = "RSS Bot/1.1 (Mac OS X Version 10.8.4 (Build 12E55))";
const char S65322[] = "Mozilla/5.0 (Windows NT 4.10; rv:19.0) Gecko/X Firefox/19.0";
const char S65323[] = "Mozilla/5.0 (Linux; Android 2.3.5; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S65324[] = "Mozilla/5.0 (Linux; Android 2.3.3; Desire HD Build/GRI40) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453";
const char S65325[] = "Mozilla/5.0 (Linux; Android 2.3.5; Desire HD Build/GRJ90) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S65326[] = "Mozilla/5.0 (Linux; Android 2.3.7; Desire HD Build/GRI40) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453";
const char S65327[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E460 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65328[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E460f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65329[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5310 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S65330[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5310B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S65331[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5310L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S65332[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5310G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S65333[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/11A465 Safari/7534.48.3";
const char S65334[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/11A501 Safari/7534.48.3";
const char S65335[] = "Nokia3220/2.0 (05.50) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S65336[] = "Nokia6600i-1c/2.0 (36.16) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S65337[] = "SAMSUNG-SGH-I617/UCHJ1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S65338[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-T959 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65339[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T959 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65340[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-T959 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65341[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T959 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65342[] = "Mozilla/5.0 (X11; Linux i686; rv:13.0) Gecko/X Thunderbird/13.0 Lightning/1.5.1";
const char S65343[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN10DG3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65344[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; AN10DG3 Build/IML74K)";
const char S65345[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE66) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/355/UCWEB Mobile";
const char S65346[] = "NokiaE66UCBrowser/8.7.1.234/28/352/UCWEB";
const char S65347[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiae66-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S65348[] = "NokiaE66/UCWEB8.7.0.187/28/800";
const char S65349[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S65350[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S65351[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LG730 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S65352[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LG730 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S65353[] = "Mozilla/5.0 (Linux; Android 4.1.2; USCC-C6721 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S65354[] = "Mozilla/4.0 (Brew MP 1.0.2; U; en-us; Sanyo; NetFront/3.5.1/AMB) Sprint SCP-3820";
const char S65355[] = "Mozilla/4.0 (Brew MP 1.0.2; U; en-us; Sanyo; NetFront/3.5.1/AMB) Sanyo  SCP-3820";
const char S65356[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; LG-KU5900 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65357[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-KU5900 Build/FRG83G)";
const char S65358[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I619 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65359[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I619 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S65360[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-I619 Build/GINGERBREAD)";
const char S65361[] = "MQQBrowser/37 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3";
const char S65362[] = "MQQBrowser/371 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3";
const char S65363[] = "MQQBrowser/371 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Safari/7534.48.3";
const char S65364[] = "MQQBrowser/371 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3";
const char S65365[] = "MQQBrowser/371 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3";
const char S65366[] = "MQQBrowser/37 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3";
const char S65367[] = "MQQBrowser/37 Mozilla/5.0 ( ; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Safari/7534.48.3";
const char S65368[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Intex Aqua Marvel Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65369[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Intex Aqua Marvel+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65370[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A850K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S65371[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A850L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S65372[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A850S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S65373[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A850S/L/K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S65374[] = "Mozilla/5.0 (Linux; Android 4.1.2; A510 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S65375[] = "Mozilla/5.0 (compatible; MSIE 4.0; PPC; .NET CLR; Trident/5.0)";
const char S65376[] = "Mozilla/5.0 (compatible; MSIE 4.0; PPC; Win64; x64; Trident/1.0)";
const char S65377[] = "Mozilla/5.0 (compatible; MSIE 4.0; PPC; WOW64; Trident/4.0)";
const char S65378[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS920 4G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S65379[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E977 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65380[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E977 Build/JZO54K)";
const char S65381[] = "Mozilla/5.0 (Linux; Android 4.2.2; Transformer TF101 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S65382[] = "Mozilla/5.0 (Linux; Android 4.2.2; Transformer Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S65383[] = "Mozilla/5.0 (Linux; Android 4.2.1; Transformer Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S65384[] = "Nokia6288/2.0 (06.43) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S65385[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; zh-cn; M9 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S65386[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; M9 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S65387[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; M9 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65388[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; M9 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S65389[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; M9 Build/GRJ90)";
const char S65390[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; zh-cn; M9 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S65391[] = "Mozilla/5.0 (Linux; Android 4.0.3;  HTC Velocity 4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S65392[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S65393[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G X710s Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S65394[] = "Lenovo-A750/S102 Linux/2.6.35.7 Android/2.3 Release/12.12.2011 Browser/AppleWebKit533.1 Profile/ Configuration/";
const char S65395[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A750 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S65396[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A750 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65397[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A750 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S65398[] = "Lenovo A750/S102 Linux/2.6.35.7 Android/2.3 Release/12.12.2011 Browser/AppleWebKit533.1 Profile/ Configuration/";
const char S65399[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; Lenovo A750 Build/GRK39F)";
const char S65400[] = "Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A750 Build GRK39F) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S65401[] = "Mozilla/5.0 (Linux; Android 4.1.1; A210 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S65402[] = "IUC(U;iOS 5.0.1;Zh-cn;320*480;)/UCWEB8.2.1.132/42/997";
const char S65403[] = "IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB8.2.1.132/41/997";
const char S65404[] = "IUC(U;iOS 4.1;Zh-cn;320*480;)/UCWEB8.2.2.137/41/800";
const char S65405[] = "Mozilla/5.0 (Java; U; en-in; opera) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S65406[] = "Mozilla/5.0 (Java; U; en-in; mstar) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S65407[] = "IUC(U;iOS 4.2.1;Zh-cn;320*480;)/UCWEB8.2.1.132/41/997";
const char S65408[] = "Mozilla/5.0 (Java; U; en-in; mstar platform) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S65409[] = "Mozilla/5.0 (Java; U; en-us; maui) UCBrowser8.2.0.132/69/352/UCWEB Mobile";
const char S65410[] = "Mozilla/5.0 (Java; U; en-in; j2me) UCBrowser8.2.0.132/69/452/UCWEB Mobile";
const char S65411[] = "IUC(U;iOS 5.1.1;Zh-cn;320*480;)/UCWEB8.2.1.132/42/997";
const char S65412[] = "Mozilla/5.0 (Java; U; kau; maui) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S65413[] = "NokiaC3-00/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac3-00) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S65414[] = "Mozilla/5.0 (Series40; Nokia303/14.38; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15";
const char S65415[] = "Opera/9.80 (X11; Linux i686; U; Edition Linux Mint; en) Presto/2.10.289 Version/12.00";
const char S65416[] = "Opera/9.80 (X11; Linux i686; U; Edition Linux Mint; en) Presto/2.10.289 Version/12.01";
const char S65417[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT18i Build/4.1.B.0.587) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S65418[] = "Mozilla/5.0 (Linux; Android 4.0.3; LT18i Build/4.1.A.0.562) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S65419[] = "Mozilla/5.0 (Linux; Android 4.0.4; N861 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65420[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5780-ORANGE/S5780BVKJ1; U; Bada/1.1; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 NexPlayer/3.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B";
const char S65421[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5780D/S5780DXEKG2; U; Bada/1.1; ru-ru) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 NexPlayer/3.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B";
const char S65422[] = "Mozilla/5.0 (Linux; U; Android 3.1; it-it; OP110 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S65423[] = "Mozilla/5.0 (X11; Linux x86_64; HTC-X710a; en-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S65424[] = "Mozilla/5.0 (X11; Linux x86_64; HTC-X710a; fr-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S65425[] = "Mozilla/5.0 (X11; Linux x86_64; HTC-X710a; es-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S65426[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11";
const char S65427[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11";
const char S65428[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11";
const char S65429[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11";
const char S65430[] = "Mozilla/5.0 (Android; Linux armv7l; rv:2.0b5pre) Gecko/X Namoroka/4.0b4pre Fennec/2.0a1";
const char S65431[] = "Mozilla/5.0 (Android; Linux armv7l; rv:2.0.1) Gecko/X Firefox/4.0.1 Fennec/2.0.1";
const char S65432[] = "Mozilla/5.0 (Android; Mobile; rv:14.0) Gecko/14.0 Firefox/2.0.1.0";
const char S65433[] = "Mozilla/5.0 (Android; Mobile; rv:14.0) Gecko/14.0 Firefox/2.0.1.1";
const char S65434[] = "Mozilla/5.0 (Android; Mobile; rv:2.0.2.1) Gecko/2.0.2.1 Firefox/2.0.2.1";
const char S65435[] = "Mozilla/5.0 (Android; Mobile; rv:2.0.2.2) Gecko/2.0.2.2 Firefox/2.0.2.2";
const char S65436[] = "Mozilla/5.0 (Android; Mobile; rv:2.0.2.4) Gecko/2.0.2.4 Firefox/2.0.2.4";
const char S65437[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax A110 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S65438[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17 Sleipnir/3.9.1";
const char S65439[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17 Sleipnir/3.9.1";
const char S65440[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.9.1)";
const char S65441[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.9.1)";
const char S65442[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/3.9.1)";
const char S65443[] = "Alcatel-OT-800/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S65444[] = "Alcatel-OT-800A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S65445[] = "Alcatel-OT-800A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A, Alcatel-OT-800A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S65446[] = "Alcatel-OT-800/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A, Alcatel-OT-800/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S65447[] = "Alcatel-OT-800/1.0 model-orange Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S65448[] = "Mozilla/5.0 (Linux; U; Android 2.2; ko-kr; IM-A690L Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65449[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; IM-A690L Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65450[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; IM-A690S Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65451[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; IM-A690S Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65452[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; IM-A690L Build/FRG83)";
const char S65453[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-; IM-A690S Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65454[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; IM-A690L Build/FRF91)";
const char S65455[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; IM-A690S Build/FRG83)";
const char S65456[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-; IM-A690S Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65457[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC-P715a; en-ca) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S65458[] = "Mozilla/5.0 (Linux; U; Android 2.0; en-us; pandigital92ghr) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S65459[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LGL46C Build/CV2.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65460[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaX6-00) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S65461[] = "NokiaX6-00/UCWEB8.9.0.253/50/800";
const char S65462[] = "Mozilla/5.0 (Series40; Nokia309/05.85; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S65463[] = "SAMSUNG-GT-S5233A/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S65464[] = "SAMSUNG-SGH-D780/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S65465[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.8.1.12) Gecko/X Firefox/2.0.0.12 Navigator/9.0.0.6";
const char S65466[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6810 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S65467[] = "Nokia302/5.0 (14.53) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia302) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S65468[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; H9500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65469[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; H9500 Build/IMM76D)";
const char S65470[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65471[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; M8 Build/JRO03H)";
const char S65472[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6120c) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S65473[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-I9100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S65474[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-I9100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S65475[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-I9100G Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S65476[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P6800 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65477[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6800 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65478[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6800 Build/JRO03H; CyanogenMod-Nexness PRO cm10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65479[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6800 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65480[] = "Mozilla/5.0 (compatible; MSIE 2.0; Win 9x 4.90; Trident/4.0)";
const char S65481[] = "Mozilla/5.0 (compatible; MSIE 1.0; Win 9x 4.90; Trident/3.0)";
const char S65482[] = "Mozilla/5.0 (compatible; MSIE 1.0; Win98; Trident/2.0)";
const char S65483[] = "Mozilla/5.0 (compatible; MSIE 2.0; Win98; Trident/2.0)";
const char S65484[] = "Mozilla/5.0 (Linux; Android 4.0.4; Motorola RAZR MAXX Build/6.7.2-180_DHD-16_NA-27) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65485[] = "Mozilla/5.0 (Windows NT 5.1; rv:12.2.1) Gecko/X Firefox/12.2.1 PaleMoon/12.2.1";
const char S65486[] = "Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:12.2.1) Gecko/X Firefox/12.2.1-x64 PaleMoon/12.2.1-x64";
const char S65487[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L710 Build/JDQ39; Carbon-Carbon-JB-v1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65488[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I747 Build/JDQ39; Carbon-Carbon-JB-v1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65489[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L710 Build/JDQ39; Carbon-CARBON-JB-v1.6.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65490[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I747 Build/JDQ39; Carbon-CARBON-JB-v1.6.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65491[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L710 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65492[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I747 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65493[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-L710 Build/JOP40G; Carbon-Carbon-JB-v1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65494[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P3110 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S65495[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P3110 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S65496[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P3110 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S65497[] = "Mozilla/5.0 (Linux; Android 4.2; GT-P3110 Build/JOP40C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S65498[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S65499[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P769 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S65500[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S65501[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P768 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S65502[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0";
const char S65503[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0.1";
const char S65504[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0.2";
const char S65505[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0.3";
const char S65506[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0.4";
const char S65507[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0.5";
const char S65508[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0.6";
const char S65509[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5830 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S65510[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; GT-S5830 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S65511[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-S5830 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S65512[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-S5830 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S65513[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-LA; GT-S5830M Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S65514[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP8010 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S65515[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9152 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S65516[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9152 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S65517[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-us; NXM703U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S65518[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.1.0; en-us; NXM703U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S65519[] = "Opera/9.80 (Android 4.0.1; Linux; Opera Mobi/ADR-X; U; en-GB) Presto/2.9.201 Version/11.50";
const char S65520[] = "Opera/9.80 (Android 4.0.3; Linux; Opera Mobi/ADY-X; U; ru) Presto/2.9.201 Version/11.50";
const char S65521[] = "Opera/9.80 (Android 4.0.4; Linux; Opera Mobi/ADY-X; U; ru) Presto/2.9.201 Version/11.50";
const char S65522[] = "Nokia6300/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6300) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S65523[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6300) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S65524[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.8.1 Safari/534.34";
const char S65525[] = "Mozilla/5.0 (X11; FreeBSD amd64; rv:19.0) Gecko/X Firefox/19.0";
const char S65526[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8815 Build/HuaweiU8815) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S65527[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8815N Build/HuaweiU8815NC02B942) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S65528[] = "Mozilla/5.0 (Linux; Android 4.0.3; EVO Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S65529[] = "Mozilla/5.0 (Linux; Android 4.0.4; EVO Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S65530[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.64.347 Chrome/13.0.782.218 Safari/535.1";
const char S65531[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.64.361 Chrome/13.0.782.218 Safari/535.1";
const char S65532[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.71.128 Chrome/14.0.835.202 Safari/535.1";
const char S65533[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML, like Gecko) RockMelt/0.9.56.357 Chrome/11.0.696.71 Safari/534.24";
const char S65534[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.72.615 Chrome/14.0.835.202 Safari/535.1";
const char S65535[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) RockMelt/0.9.50.549 Chrome/10.0.648.205 Safari/534.16";
const char S65536[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) RockMelt/0.9.58.494 Chrome/11.0.696.71 Safari/534.24";
const char S65537[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) RockMelt/0.9.48.51 Chrome/9.0.597.107 Safari/534.13";
const char S65538[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.30 (KHTML, like Gecko) RockMelt/0.9.62.313 Chrome/12.0.742.122 Safari/534.30";
const char S65539[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML	 like Gecko) RockMelt/0.9.72.698 Chrome/14.0.835.202 Safari/535.1";
const char S65540[] = "HUAWEI_T8830Pro_TD/1.0 Android/4.0.4 (Linux; U; Android 4.0.4; zh-cn) Release/08.31.2012 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30";
const char S65541[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; T8830Pro Build/HuaweiT8830Pro) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S65542[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; T8830Pro Build/HuaweiT8830Pro) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65543[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; T8830Pro Build/HuaweiT8830Pro)";
const char S65544[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; X210 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65545[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:24.0) Gecko/X Firefox/24.0 AlexaToolbar/alxf-2.19";
const char S65546[] = "Mozilla/5.0 (Nintendo WiiU) AppleWebKit/536.28 (KHTML, like Gecko) NX/3.0.3.12.6 NintendoBrowser/2.0.0.9362.US";
const char S65547[] = "Mozilla/5.0 (Nintendo WiiU) AppleWebKit/536.28 (KHTML, like Gecko) NX/3.0.3.12.6 NintendoBrowser/2.0.0.9362.JP";
const char S65548[] = "Mozilla/5.0 (Nintendo WiiU) AppleWebKit/536.28 (KHTML, like Gecko) NX/3.0.3.12.6 NintendoBrowser/2.0.0.9362.EU";
const char S65549[] = "BlackBerry8120/4.3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100";
const char S65550[] = "Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320) BlackBerry8120/4.3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100";
const char S65551[] = "JUC (Linux; U; 2.3.6; zh-cn; GT-S5300; 240*320) UCWEB7.9.0.94/139/444";
const char S65552[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/23.0.648.205 Safari/534.16";
const char S65553[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/23.0.648.205 Safari/534.16";
const char S65554[] = "Nokia6270/2.0 (03.90) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S65555[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; E140 Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1ACER_E140/1.0";
const char S65556[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; E140 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1ACER_E140/1.0";
const char S65557[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; Excellent8 Android 4.0 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65558[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; Excellent8 Android 4.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65559[] = "NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac2-01) UCBrowser8.4.0.159/70/444/UCWEB Mobile";
const char S65560[] = "iCoreService (unknown version) CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacBookPro9%2C2)";
const char S65561[] = "WebProcess/8536.25 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacBookPro9%2C2)";
const char S65562[] = "iCoreService (unknown version) CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacBookAir5%2C2)";
const char S65563[] = "WebProcess/8536.25 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacBookAir5%2C2)";
const char S65564[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A618 TV Duo Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65565[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Enjoy 10 Plus Build/1.14ST) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65566[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Enjoy 10 Plus Build/1.11ST) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65567[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Enjoy 10 Plus Build/1.11PP) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65568[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Enjoy 10 Plus Build/1.14PP) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65569[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Enjoy 10 Plus Build/1.13ST) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65570[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Enjoy 10 Plus Build/1.18ST) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65571[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65572[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI U9508 Build/HuaweiU9508) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S65573[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HUAWEI U9508 Build/HuaweiU9508)";
const char S65574[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ALCATEL ONE TOUCH 4030E Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S65575[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ALCATEL ONE TOUCH 4030E Build/JRO03C)";
const char S65576[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; bq Elcano Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65577[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; bq Elcano Build/JZO54K)";
const char S65578[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S65579[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710VMUB Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S65580[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710BST Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S65581[] = "Mozilla/5.0 (Linux;U;Android 4.0.4;en-us;I10-LE Build/IML74K) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S65582[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I415 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S65583[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LW770 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S65584[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; DROIDX Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65585[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65586[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROIDX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65587[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDX Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65588[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Droid X Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65589[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDX Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65590[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Droid X Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65591[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Droid X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65592[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S65593[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P7100 Build/HTJ85B) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S65594[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-01D Build/V08R31A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65595[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-01D Build/V10R32D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65596[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI Y300-0000 Build/HuaweiY300-0000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65597[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HUAWEI Y300-0000 Build/HuaweiY300-0000)";
const char S65598[] = "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:19.0) Gecko/X Firefox/19.0 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19";
const char S65599[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LS860 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65600[] = "Mozilla/5.0 (Linux; Android 4.0.4; Mach Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65601[] = "SAMSUNG-GT-M3710/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S65602[] = "Mozilla/5.0 (Series40; NokiaC2-01/11.20; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.1.21.8";
const char S65603[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352";
const char S65604[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352";
const char S65605[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP9090 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S65606[] = "Mozilla/5.0 (Linux; Android 4.1.2; PantechP8010 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S65607[] = "Mozilla/5.0 (Linux; Android 4.1.2; PantechP8010 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S65608[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I727 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S65609[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I727 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S65610[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; T-Mobile myTouch 3G Slide Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S65611[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; T-Mobile myTouch 3G Slide Build/ERE27) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S65612[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; T-Mobile myTouch 3G Slide Build/ERE27)";
const char S65613[] = "Mozilla/5.0 (Series40; Nokia201/11.95; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S65614[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MyTouch 4g Build/AwesomeICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65615[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MyTouch 4G Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65616[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; myTouch4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65617[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC myTouch 4G slide Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65618[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MyTouch 4G Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65619[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; myTouch4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65620[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC myTouch 4g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65621[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC myTouch 4g Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65622[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_C8810 Build/C8810V100R001C92B862) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65623[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8810 Build/C8810V100R001C92B862) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S65624[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8810 Build/C8810V100R001C92B862) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S65625[] = "HUAWEI_Y220T_TD/1.0 Android/2.3.5 SC8810_CMCC/W12.06.21 Release/06.22.2012 Browser/AppleWebkit533.1";
const char S65626[] = "Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8810 Build C8810V100R001C92B862) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S65627[] = "Bird_T9108_TD/T9108_V1.00 Android/2.3.5 SC8810_CMCC/W12.06.10 Release/04.09.2012 Browser/AppleWebKit533.1";
const char S65628[] = "HUAWEI_Y220-T10_TD/1.0 Android/2.3.5 SC8810_CMCC/W12.06.21 Release/02.01.2013 Browser/AppleWebkit533.1";
const char S65629[] = "Mozilla/5.0 (Linux; Android 4.1.1; Lenovo P770 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S65630[] = "Mozilla/5.0 (Linux; Android 4.1.1; Lenovo P770_ROW Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S65631[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; TECNO P3 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65632[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S65633[] = "Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5130c-2) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S65634[] = "Mozilla/5.0 (Symbian3; U; en-us; NokiaE7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile";
const char S65635[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiae7-00) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba";
const char S65636[] = "NokiaE7-00UCBrowser/8.7.1.234/51/355/UCWEB";
const char S65637[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IdeaTabA2207A-H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65638[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; IdeaTabA2207A-H Build/IMM76D)";
const char S65639[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; IdeaTabA2207A-H Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S65640[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; LG-F100L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65641[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-F100L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65642[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F100L Build/JZO54K)";
const char S65643[] = "Mozilla/5.0 (Linux; Android 4.0.3; Slider SL101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S65644[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS920 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S65645[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus 7 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S65646[] = "SAMSUNG-SGH-I637/UCIJ2 Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S65647[] = "Chrome/8";
const char S65648[] = "Mozilla/5.0 AppleWebKit/534.10 Chrome/8.0.552.215 Safari/534.10";
const char S65649[] = "T-Mobile_LEO (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S65650[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)/UC Browser7.8.0.95";
const char S65651[] = "480x800; XV6975; Window Mobile 6.5 Professional; (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S65652[] = "480x800; XV6875.1; Window Mobile 6.5 Professional; (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S65653[] = "USCCHTC6875 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S65654[] = "T-Mobile_Rhodium (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S65655[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Sprint:SPH-ip830w)/UC Browser7.8.0.95";
const char S65656[] = "Sprint-SPH-i350 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S65657[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95/31/352";
const char S65658[] = "T-Mobile_Maple (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S65659[] = "RSSOwl/2.1.2.X (X11; U; en)";
const char S65660[] = "RSSOwl/2.1.1.X (Windows; U; en)";
const char S65661[] = "RSSOwl/2.1.4.X (Windows; U; en)";
const char S65662[] = "RSSOwl/2.1.4.X (X11; U; en)";
const char S65663[] = "RSSOwl/2.1.6.X (Windows; U; en)";
const char S65664[] = "RSSOwl/2.1.6.X (X11; U; en)";
const char S65665[] = "SonyEricssonS312/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1, SonyEricssonS312/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1, SonyEricssonS312/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S65666[] = "SonyEricssonS312/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1, SonyEricssonS312/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S65667[] = "SonyEricssonS312/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S65668[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-nz; Huawei IDEOS X3 Build/HuaweiU8510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65669[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; Huawei IDEOS X3 Build/HuaweiU8510)";
const char S65670[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; pt-pt; Huawei Ideos X3 Build/joaquimorg; CM-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65671[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; Ultimate10-Android4.0 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65672[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; ULTIMATE10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65673[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.1) Gecko/X Firefox/12.1 PaleMoon/12.1";
const char S65674[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:12.1) Gecko/X Firefox/12.1-x64 PaleMoon/12.1-x64";
const char S65675[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U8800 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65676[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; U8800 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65677[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; U8800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65678[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; U8800 Build/IMM76I)";
const char S65679[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315b Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S65680[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S65681[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT915 Build/2_32A_2008) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S65682[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; PG41200 Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S65683[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; PG41200 Build/HTK75C)";
const char S65684[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 900;7.10;8107)";
const char S65685[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 900;7.10;8112)";
const char S65686[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 900;7.10;8773)";
const char S65687[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 900;7.10;8779)";
const char S65688[] = "Opera/9.80 (Android 2.3.3; Linux; Opera Mobi/ADR-X; U; fr) Presto/2.9.201 Version/11.50";
const char S65689[] = "Opera/9.80 (Android 2.3.7; Linux; Opera Mobi/ADR-X; U; es-ES) Presto/2.9.201 Version/11.50";
const char S65690[] = "Opera/9.80 (Android 2.3.4; Linux; Opera Mobi/ADR-X; U; ko) Presto/2.9.201 Version/11.50";
const char S65691[] = "Opera/9.80 (Android 2.3.5; Linux; Opera Mobi/ADY-X; U; ru) Presto/2.9.201 Version/11.50";
const char S65692[] = "Opera/9.80 (Android 2.3; Linux; Opera Mobi/ADR-X; U; en) Presto/2.9.201 Version/11.50";
const char S65693[] = "Opera/9.80 (Android 2.3.2; Linux; Opera Mobi/ADR-X; U; en-GB) Presto/2.9.201 Version/11.50";
const char S65694[] = "Opera/9.80 (Android 2.3.1; Linux; Opera Mobi/ADR-X; U; en) Presto/2.9.201 Version/11.50";
const char S65695[] = "Opera/9.80 (Android 2.3.6; Linux; Opera Mobi/ADR-X; U; uk) Presto/2.9.201 Version/11.50";
const char S65696[] = "Opera/9.80 (X11; Linux x86_64; Edition Next) Presto/2.12.378 Version/12.50";
const char S65697[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaC5-05) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile";
const char S65698[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BLU Touch Book 7.0 LITE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65699[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-D710 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65700[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-D710 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65701[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-D710 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65702[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-D710 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65703[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SPH-D710 Build/JDQ39E)";
const char S65704[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI H881C Build/HuaweiH881C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S65705[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; HP eStation/1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65706[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:17.0) Gecko/X Firefox/17.0 IceDragon/18.0.1.0";
const char S65707[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:18.0) Gecko/X Firefox/18.0 IceDragon/18.0.3.1";
const char S65708[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI H881C Build/HuaweiH881C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65709[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I905 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S65710[] = "KDDI-CA31 UP.Browser/6.2.0.7.3.129 (GUI) MMP/2.0 (compatible; ichiro/mobile goo;+http://search.goo.ne.jp/option/use/sub4/sub4-1/)";
const char S65711[] = "Mozilla/5.0 (Windows NT 4.10; rv:18.0) Gecko/X Firefox/18.0";
const char S65712[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5235) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S65713[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Micromax A62 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65714[] = "UCWEB/2.0 (iOS; U; iPh OS 6_0_1; zh-CN; iPh3,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile";
const char S65715[] = "UCWEB/2.0 (iOS; U; iPh OS 6_0_1; zh-CN; iPh4,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile";
const char S65716[] = "UCWEB/2.0 (iOS; U; iPh OS 6_0_2; zh-CN; iPh5,2) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile";
const char S65717[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-P714 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65718[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P714 Build/JZO54K)";
const char S65719[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID7015 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65720[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID7015 Build/GRI40)";
const char S65721[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5530) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile";
const char S65722[] = "Nokia5530/UCWEB8.6.0.180/50/800";
const char S65723[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I437 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S65724[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; 201M Build/7.7.1Q-164_SMJ-133) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65725[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; 201M Build/7.7.1Q-164_SMJ-143) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65726[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; 201M Build/7.7.1Q-164_SMJ-165) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65727[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; 201M Build/7.7.1Q-164_SMJ-165)";
const char S65728[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; T2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65729[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; T2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65730[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; T2 Build/JRO03H)";
const char S65731[] = "Opera/9.80 (Windows NT 5.0) Presto/2.12.388 Version/12.15";
const char S65732[] = "Opera/9.80 (Windows NT 5.0; Edition Yx) Presto/2.12.388 Version/12.15";
const char S65733[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E980 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S65734[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F180L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65735[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F180K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65736[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F180S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65737[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F180 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65738[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6506 Build/10.1.A.1.350) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65739[] = "SAMSUNG-GT-S5560i/S5560IXXKA5 SHP/VPP/R5 Dolfin/2.0 NexPlayer/3.0 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B";
const char S65740[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i917R)";
const char S65741[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.10.229 Version/11.64";
const char S65742[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 Comodo_Dragon/21.1.1.0";
const char S65743[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 Comodo_Dragon/21.1.1.0";
const char S65744[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; C2 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65745[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; C2 Build/JOP40D)";
const char S65746[] = "NokiaE65-1 Orange/1.0 (1.0633.18.02) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S65747[] = "NokiaE65/3.0 (1.0633.58.02) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S65748[] = "NokiaE65-1/3.0 (2.0633.65.01) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S65749[] = "SAMSUNG-GT-B3310/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S65750[] = "Mozilla/5.0 (Linux; Android 4.1.2; Titanium S5 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S65751[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S65752[] = "Nokia6080b/2.0 (04.22) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S65753[] = "Nokia6080/2.0 (04.20) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S65754[] = "Mozilla/5.0 (Windows 98; U; en; rv:1.8.0) Gecko/X Firefox/1.5.0";
const char S65755[] = "Mozilla/5.0 (Windows; U; Win 9x 4.90; en-US; rv:1.8.0.3) Gecko/X Firefox/1.5.0.3";
const char S65756[] = "Mozilla/5.0 (Windows; U; Win98; ru; rv:1.8.0.4) Gecko/X Firefox/1.5.0.4";
const char S65757[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; CINK Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65758[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SC-01D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65759[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.6 Safari/535.19 Silk-Accelerated=false";
const char S65760[] = "WhatsApp/2.8.7 iPhone_OS/6.0.1 Device/iPhone_4";
const char S65761[] = "WhatsApp/2.8.7 iPhone_OS/5.1.1 Device/iPhone_4";
const char S65762[] = "WhatsApp/2.8.7 iPhone_OS/6.1 Device/iPhone_4";
const char S65763[] = "WhatsApp/2.8.7 iPhone_OS/6.1 Device/iPhone_5";
const char S65764[] = "WhatsApp/2.8.7 iPhone_OS/6.1.2 Device/iPhone_5";
const char S65765[] = "WhatsApp/2.8.7 iPhone_OS/6.1.2 Device/iPhone_4";
const char S65766[] = "WhatsApp/2.8.7 iPhone_OS/6.1.2 Device/iPhone_4S";
const char S65767[] = "WhatsApp/2.8.7 iPhone_OS/5.1.1 Device/iPhone_4S";
const char S65768[] = "WhatsApp/2.8.7 iPhone_OS/6.1.3 Device/iPhone_4S";
const char S65769[] = "WhatsApp/2.8.7 iPhone_OS/6.1.3 Device/iPhone_4";
const char S65770[] = "WhatsApp/2.8.7 iPhone_OS/6.0.1 Device/iPhone_4S";
const char S65771[] = "WhatsApp/2.8.7 iPhone_OS/6.1 Device/iPhone_4S";
const char S65772[] = "WhatsApp/2.8.7 iPhone_OS/6.1.3 Device/iPhone_5";
const char S65773[] = "WhatsApp/2.8.7 iPhone_OS/6.1.4 Device/iPhone_5";
const char S65774[] = "WhatsApp/2.8.7 iPhone_OS/6.1.3 Device/iPhone_3GS";
const char S65775[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; GT-I5700 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65776[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; GT-I5700 Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65777[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; GT-I5700 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65778[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-I5700 Build/FRG83)";
const char S65779[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; GT-I5700 Build/FRG83G)";
const char S65780[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; GT-I5700 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65781[] = "NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac1-01) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S65782[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; HTC Velocity 4G Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65783[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-hk; HTC_Velocity_4G_X710s Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65784[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; HTC/Velocity 4G/2.38.162.3 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65785[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Velocity 4G Build/GWK74)";
const char S65786[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; HTC Velocity 4G X710s Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65787[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-at; HTC/Velocity 4G/2.39.169.1 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65788[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; HTC/Velocity 4G/2.38.162.5 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65789[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; HTC/Velocity 4G/2.38.162.1 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65790[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-M950 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S65791[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65792[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; U1 Build/GRK39F)";
const char S65793[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; 3305-1030 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65794[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65795[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65796[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160LV Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65797[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65798[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65799[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160S/K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65800[] = "Mozilla/5.0 (Linux; Android 4.3; GT-I9300 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.30 Mobile Safari/537.36";
const char S65801[] = "Mozilla/5.0 (Linux; Android 4.3; SCH-I535 Build/JLS36G) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S65802[] = "NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S65803[] = "JUC (Linux; U; 2.3.6; zh-cn; GT-S5302; 240*320) UCWEB7.9.0.94/139/444";
const char S65804[] = "Opera (Linux; U; 2.3.6; zh-cn; GT-S5302; 240*320) UCWEB7.9.0.94/139/444";
const char S65805[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M870 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65806[] = "Nokia206/2.0 (03.58) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S65807[] = "Nokia206.1/2.0 (p) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S65808[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; U2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65809[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; U2 Build/IMM76I)";
const char S65810[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT30at Build/7.0.B.1.152) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S65811[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID7038 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65812[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 Sleipnir/4.1.0";
const char S65813[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 Sleipnir/4.1.3";
const char S65814[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36 Sleipnir/4.1.4";
const char S65815[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36 Sleipnir/4.1.4";
const char S65816[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; JY-G3 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65817[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; JY-G3 Build/JOP40D)";
const char S65818[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65819[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Iron/5.0.381.0 Chrome/5.0.381 Safari/533.4";
const char S65820[] = "Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia2730c-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S65821[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F180S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S65822[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F180L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S65823[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F180K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S65824[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I547 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S65825[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I547C Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S65826[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I547 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S65827[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; T-Mobile_G2_Touch Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S65828[] = "SAMSUNG-SGH-T929/T929UVIA2 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S65829[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; STM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65830[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.1; STM803HC Build/GINGERBREAD)";
const char S65831[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S 4G Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65832[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S 4G Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65833[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S 4G Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65834[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus S 4G Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65835[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia S Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65836[] = "AQUA 5.0 Linux/3.0.13 Android/4.0.4 Release/12.26.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S65837[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AQUA 5.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65838[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAB-840_G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65839[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; TAB-840_G Build/JRO03H)";
const char S65840[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; Win 9x 4.90; MSIECrawler)";
const char S65841[] = "Opera/9.80 (Windows NT 7.0; U; en) Presto/2.9.211 Version/12.00";
const char S65842[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X Firefox/15.0.2.0 IceDragon/15.0.2.0";
const char S65843[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; vivo X1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65844[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; vivo X1 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S65845[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; vivo X1 Build/JRO03C)";
const char S65846[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; vivo X1w Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65847[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; F-06E Build/V17R45B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65848[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; F-06E Build/V19R47F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65849[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; F-06E Build/V17R45B)";
const char S65850[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; F-06E Build/V21R48D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65851[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6400L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S65852[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID8120 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65853[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID8120 Build/GRI40)";
const char S65854[] = "Mozilla/5.0 (Linux; Android 4.1.2; N9101 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S65855[] = "Mozilla/5.0 (Linux; Android 4.1.2; N9101 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S65856[] = "HTC_P3700 Opera/9.50 (Windows NT 5.1; U; ru)";
const char S65857[] = "HTC_Diamond Opera/9.50 (Windows NT 5.1; U; en) P3720";
const char S65858[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Opera/9.5) Alltel HTC Touch Diamond";
const char S65859[] = "HTC_Diamond Opera/9.50 (Windows NT 5.1; U; en) P3720/V2.07.841.2";
const char S65860[] = "HTC_P3700 Opera/9.50 (Windows NT 5.1; U; en)";
const char S65861[] = "Vodafone/1.0/HTC_Diamond Opera/9.50 (Windows NT 5.1; U; en)";
const char S65862[] = "HTC_P3700 Opera/9.50 (Windows NT 5.1; U; nb)";
const char S65863[] = "HTC_P3700 Opera/9.50 (Windows NT 5.1; U; da)";
const char S65864[] = "HTC_P3700 Opera/9.50 (Windows NT 5.1; U; sv)";
const char S65865[] = "HTC_P3700 Opera/9.50 (Windows NT 5.1; U; en-GB)";
const char S65866[] = "HTC_P3700 Opera/9.50 (Windows NT 5.1; U; es-ES)";
const char S65867[] = "HTC_P3700 Opera/9.50 (Windows NT 5.1; U; nl)";
const char S65868[] = "HTC_P3700 Opera/9.50 (Windows NT 5.1; U; pt)";
const char S65869[] = "HTC_P3700 Opera/9.50 (Windows NT 5.1; U; fr)";
const char S65870[] = "HTC_P3700 Opera/9.50 (Windows NT 5.1; U; it)";
const char S65871[] = "HTC_P3700 Opera/9.50 (Windows NT 5.1; U; de)";
const char S65872[] = "HTC_P3700 Opera/9.50 (Windows NT 5.1; U; zh-tw)";
const char S65873[] = "Vodafone/1.0/HTC_Diamond Opera/9.50 (Windows NT 5.1; U; fr)";
const char S65874[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; VTAB1008 Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65875[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.2; VTAB1008 Build/GRH78C)";
const char S65876[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; iw-il; Orange_infinity_996 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65877[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; B92 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65878[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5530) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S65879[] = "Nokia5530/UCWEB8.9.0.253/50/800";
const char S65880[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:24.0) Gecko/X Firefox/24.0 AlexaToolbar/alxf-2.19";
const char S65881[] = "Mozilla/5.0 (Series40; Nokia114/03.33; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S65882[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; V96A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Build/IMM76I";
const char S65883[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 101 Titanium Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S65884[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515a Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S65885[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515m Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S65886[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC_X515E Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S65887[] = "Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36";
const char S65888[] = "Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36";
const char S65889[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I905 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S65890[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6800 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S65891[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;Radar 4G;7.10;7720)";
const char S65892[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;Radar 4G;7.10;7740)";
const char S65893[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;Radar 4G;7.10;8107)";
const char S65894[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;Radar C110e;7.10;7720)";
const char S65895[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;Radar;7.10;7720)";
const char S65896[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;Radar C110e;7.10;7740)";
const char S65897[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;Radar 4G;7.10;8773)";
const char S65898[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;Radar;7.10;7740)";
const char S65899[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;Radar C110e;7.10;8107)";
const char S65900[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;Radar;7.10;8107)";
const char S65901[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;Radar C110e;7.10;8773)";
const char S65902[] = "Mozilla/5.0 (Windows NT 4.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S65903[] = "Mozilla/5.0 (Windows NT 4.0) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5";
const char S65904[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; sr-rs; Telenor_Smart_HD Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65905[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; mauie800) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S65906[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; mauie800) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S65907[] = "UCWEB/2.0(Java; U; MIDP-2.0; id; mauie800) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S65908[] = "UCWEB/2.0(Java; U; MIDP-2.0; id; mauie800) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S65909[] = "UCWEB/2.0(Java; U; MIDP-2.0; vi; mauie800) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S65910[] = "UCWEB/2.0(Java; U; MIDP-2.0; ar-sa; mauie800) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S65911[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5800d-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba";
const char S65912[] = "Nokia5800w XpressMusic/UCWEB8.9.0.253/50/999";
const char S65913[] = "Nokia5800 XpressMusic/UCWEB8.9.0.253/50/999";
const char S65914[] = "UCWEB/2.0 (Symbian; U; S60 V5; id; Nokia5800 XpressMusic) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S65915[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5800w XpressMusic) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S65916[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S65917[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315";
const char S65918[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC VLE_U Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S65919[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S65920[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTCONE Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S65921[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC_PN071 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S65922[] = "Mozilla/5.0 (Linux; Android 4.2.1; ME173X Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S65923[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Safari/535.19 Silk-Accelerated=false";
const char S65924[] = "Mozilla/5.0 (Linux; Android 4.2.1; Micromax A110Q Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S65925[] = "Mozilla/5.0 (Linux; Android 4.2.1; Micromax A110Q Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22";
const char S65926[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT28h Build/6.2.B.0.211) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65927[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT28i Build/6.2.B.0.211) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65928[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; GT-I8260 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65929[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8260 Build/JZO54K)";
const char S65930[] = "Mozilla/5.0 (X11; Linux x86_64; PadFone) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S65931[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I997 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65932[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I997 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65933[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I997 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65934[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; OnePAD 970 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65935[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MOMO7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65936[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MOMO7 Talent Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65937[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65938[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; GT-I9300 Build/JRO03C.I9300ZNDLK4;720*1280)";
const char S65939[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; GT-I9300 Build/JRO03C.I9300ZNDLL1;720*1280)";
const char S65940[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65941[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-I9300 Build/JZO54K.I9300ZNEMB2;720*1280)";
const char S65942[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-I9300 Build/JZO54K.I9300ZNEMD2;720*1280)";
const char S65943[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-I9300 Build/JZO54K.I9300ZNEMB1;720*1280)";
const char S65944[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-I9300 Build/JZO54K.I9300ZNEMA2;720*1280)";
const char S65945[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; GT-I9300 Build/JRO03C.I9300ZCDLL1;720*1280)";
const char S65946[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-I9300 Build/JZO54K.I9300ZCEMD2;720*1280)";
const char S65947[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; GT-I9300 Build/JRO03C.I9300ZCDLK1;720*1280)";
const char S65948[] = "Nokia303/5.0 (14.87) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia303) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S65949[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7310 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S65950[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5 UCBrowser/9.0.1.275";
const char S65951[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 4_3_3 like Mac OS X; zh-CN) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 UCBrowser/9.0.1.284 Mobile";
const char S65952[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 4_3_5 like Mac OS X; zh-CN) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 UCBrowser/9.0.1.284 Mobile";
const char S65953[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5570I Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S65954[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5570 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S65955[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-S5570 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S65956[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; GT-S5570 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S65957[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; V972 Core4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65958[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; V972 Core4 Build/JDQ39)";
const char S65959[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; Micromax A60 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S65960[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; Micromax A60 Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S65961[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Micromax A60 Build/ERE27)";
const char S65962[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US; rv:1.8.1.14) Gecko/X Firefox/2.0.0.14 Flock/1.2.1";
const char S65963[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.8.1.15) Gecko/X Firefox/2.0.0.15 Flock/1.2.3";
const char S65964[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D2-711 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65965[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; D2-711 Build/IMM76D)";
const char S65966[] = "Nokia2700c-2/2.0 (09.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia2700c-2) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S65967[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One VX Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S65968[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SKY IM-A650S Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65969[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-; SKY IM-A650S Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S65970[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SKY IM-A650S Build/FRG83)";
const char S65971[] = "Mozilla/5.0 (compatible; Firefox/12.0; Windows NT 5.1;) Vbot/1303";
const char S65972[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:16.0) Gecko/X Thunderbird/16.0.2";
const char S65973[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:16.0) Gecko/X Firefox/16.0 SeaMonkey/2.13.2";
const char S65974[] = "Nokia202/2.0 (20.28) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia202) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S65975[] = "Mozilla/5.0 (compatible; MSIE 5.0; Windows NT 6.1; Trident/4.0)";
const char S65976[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 6.1; Trident/3.0)";
const char S65977[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HTC_802w Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65978[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HTC 802w Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65979[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC 802w Build/JZO54K)";
const char S65980[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; HTC_One_dual_sim Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65981[] = "Lenovo-A830/S100 Linux/3.4.0 Android/4.2 Release/01.23.2013 Browser/AppleWebKit534.30 Profile/ Configuration/ Mobile Safari/534.30";
const char S65982[] = "SAMSUNG-GT-I9158_TD/1.0 Android/4.2.2 Release/04.03.2013 Browser/AppleWebKit535.19";
const char S65983[] = "sprd-GT-T9500/1.0 Linux/2.6.35.7 Android/4.2.0 Release/07.18.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S65984[] = "Coolpad7269_CUCC_WCDMA/1.0 Linux/3.4.5 Android/4.2 Release/6.28.2013 Browser/AppleWebkit533.1";
const char S65985[] = "AppleCoreMedia/1.0.0.11A465 (iPad; U; CPU OS 7_0 like Mac OS X; en_us)";
const char S65986[] = "AppleCoreMedia/1.0.0.11A501 (iPad; U; CPU OS 7_0_2 like Mac OS X; en_us)";
const char S65987[] = "NokiaE72-1/UCWEB8.2.0.116/28/800";
const char S65988[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE72-1)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S65989[] = "Mozilla/5.0 (Java; U; en-in; nokiae72-1) UCBrowser8.2.1.144/69/444/UCWEB Mobile  3gpp-gba";
const char S65990[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1811_v75_jbaol1_g88 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65991[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1811_v75_jbaol1_g58 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65992[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1811_v75_hjy1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65993[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1811_v75_jyuany1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65994[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; e1811_v75_zlh1_5647 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S65995[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; HS-10DTB2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S65996[] = "Mozilla/5.0 (Windows NT 5.1; rv:16.0) Gecko/X Firefox/16.0 epic/16.0";
const char S65997[] = "Mozilla/5.0 (Windows NT 5.2; rv:16.0) Gecko/X Firefox/16.0 Epic/16.0";
const char S65998[] = "Mozilla/5.0 (X11; FreeBSD amd64; rv:18.0) Gecko/X Firefox/18.0";
const char S65999[] = "NokiaC2-06/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-06) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S66000[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; Micromax A88 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66001[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Micromax A88 Build/JRO03C)";
const char S66002[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9205 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66003[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9205 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S66004[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ605 Build/H.6.5-14) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S66005[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ605 Build/H.6.5-12) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S66006[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ605 Build/H.6.5-8) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S66007[] = "Mozilla/5.0 (Linux; U; Android 3.2; es-us; MZ605 Build/H.6.5-18.5) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S66008[] = "Mozilla/5.0 (Linux; U; Android 3.2; pt-br; MZ605 Build/H.6.5-18.4) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S66009[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SHW-M190S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66010[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SHW-M190S Build/FROYO)";
const char S66011[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; IS11PT Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66012[] = "AppleCoreMedia/1.0.0.9B176 (iPod; U; CPU OS 5_1 like Mac OS X; en_us)";
const char S66013[] = "AppleCoreMedia/1.0.0.9B206 (iPod; U; CPU OS 5_1_1 like Mac OS X; en_us)";
const char S66014[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; I_7520 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66015[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I_7520 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66016[] = "OneBrowser/3.1 (NokiaX2-02/11.79)";
const char S66017[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; I10A-LE Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66018[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; I10A-LE Build/JRO03H)";
const char S66019[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S66020[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S66021[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC-Z710a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S66022[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S66023[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9A405 Safari/7534.48.3";
const char S66024[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9A406 Safari/7534.48.3";
const char S66025[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9A334 Safari/7534.48.3";
const char S66026[] = "SonyEricssonZ610i/R1KH Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S66027[] = "SAMSUNG-SGH-L760/JAGH2 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S66028[] = "SAMSUNG-SGH-L760-Vodafone/BUHC1 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S66029[] = "Alcatel-OT-802A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S66030[] = "Alcatel-OT-802Y/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S66031[] = "Alcatel-OT-802/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S66032[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.16) Gecko/X Thunderbird/2.0.0.16";
const char S66033[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.8) Gecko/X Firefox/3.6.8 CometBird/3.6.8";
const char S66034[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; zh-CN; rv:1.9.2.16) Gecko/X Firefox/3.6.16 CometBrowser/3.6.16";
const char S66035[] = "NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-02) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile";
const char S66036[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-STYLE Q4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66037[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-STYLE Q4A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66038[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; i-STYLE Q4 Build/IMM76D)";
const char S66039[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Micromax A26 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66040[] = "Mozilla/5.0 (Linux; Android 4.1.1; Arc Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S66041[] = "Mozilla/5.0 (Linux; Android 4.2.2; A1-810 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S66042[] = "SAMSUNG-S3600/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S66043[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; U8800-51 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66044[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; ZTE-U V880 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66045[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-U V880 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66046[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-U V880 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S66047[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; ZTE-U V880 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S66048[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-U V880 Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S66049[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; ZTE-U V880 Build/FRF91)";
const char S66050[] = "HUAWEI-M228/001.00 ACCESS-NetFront/3.2(Mobile) Mozilla/5.0";
const char S66051[] = "HUAWEI-M228/001.00 ACS-NetFront/3.2";
const char S66052[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; chromeframe/26.0.1410.64)";
const char S66053[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; Touch; chromeframe/26.0.1410.64)";
const char S66054[] = "Nokia2700c-2/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2700c-2) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S66055[] = "Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S66056[] = "Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire2 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S66057[] = "Mozilla/5.0 (Linux; Android 4.2; Amazon Kindle Fire Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S66058[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S66059[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710BST Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S66060[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710VMUB Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S66061[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7510 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S66062[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P7510 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S66063[] = "Nokia6600s/2.0 (05.60) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S66064[] = "NOKIAE5-00/UC Browser8.0.4.121/28/444/UCWEB";
const char S66065[] = "Mozilla/5.0 (U; Win32 i686; ru; rv:1.9.2.13) Gecko/X 3.6.13-1.fc14 Firefox/3.6.13";
const char S66066[] = "Mozilla/5.0 (Windows; U; Windows NT 7.1; en-US; rv:1.9.2.13) Gecko/X Firefox/3.6.13";
const char S66067[] = "Mozilla/5.0 (Windows; U; Windows; en-US) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16 Gecko/X Firefox/3.6.4 EvoHtmlToPdf/3.5";
const char S66068[] = "Mozilla/5.0 (Windows; Windows NT) Firefox/3.6";
const char S66069[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; XT687 Build/V2.27D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66070[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; XT687 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66071[] = "Nokia201/2.0 (11.64) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia201) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S66072[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GEM7008 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66073[] = "Nokia303/5.0 (14.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia303) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S66074[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-1 Build/HuaweiU8950-1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S66075[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-51 Build/HuaweiU8950N-51) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S66076[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-51 Build/HuaweiU8950-51) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S66077[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950D Build/HuaweiU8950D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S66078[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-1 Build/HuaweiU8950N-1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S66079[] = "Mozilla/5.0 (webOS/2.1.1; U; de-DE) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 P160U/1.0";
const char S66080[] = "Mozilla/5.0 (webOS/2.1.0; U; en-GB) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.1";
const char S66081[] = "Mozilla/5.0 (webOS/1.3; U; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Desktop/1.0";
const char S66082[] = "Mozilla/5.0 (webOS/1.4.5; U; en-IE) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.1";
const char S66083[] = "Mozilla/5.0 (webOS/2.1.0; U; fr-FR) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.2";
const char S66084[] = "Mozilla/5.0 (webOS/1.4.5; U; zh-CN) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.1";
const char S66085[] = "Mozilla/5.0 (webOS/2.1.0; U; de-DE) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.1";
const char S66086[] = "Mozilla/5.0 (webOS/2.1.1; U; en-GB) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 P160U/1.0";
const char S66087[] = "Mozilla/5.0 (webOS/2.1.0; U; es-ES) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.2";
const char S66088[] = "Mozilla/5.0 (webOS/2.1.2; U; es-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 P160UNA/1.0";
const char S66089[] = "Mozilla/5.0 (webOS/2.1.2; U; zh-CN) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 P160UNA/1.0";
const char S66090[] = "Mozilla/5.0 (webOS/1.4.0; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.1";
const char S66091[] = "Mozilla/5.0 (webOS/2.1.0; U; fr-FR) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.1";
const char S66092[] = "Mozilla/5.0 (webOS/2.1.1; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 P160U/1.0";
const char S66093[] = "Mozilla/5.0 (webOS/2.1.0; U; es-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.2";
const char S66094[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; HS-7DTB4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66095[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; HS-7DTB4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66096[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hr-hr; move 2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66097[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.276; en-us; 9810/5.1.0.276) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S66098[] = "UCWEB/2.0(BlackBerry; U; 5.0.0.734; en-us; 9810/5.0.0.734) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S66099[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.532; en-us; 9810/5.1.0.532) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S66100[] = "UCWEB/2.0(BlackBerry; U; 5.0.0.469; en-us; 9810/5.0.0.469) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S66101[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.230; en-us; 9810/5.1.0.230) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S66102[] = "UCWEB/2.0(BlackBerry; U; 5.0.0.736; en-us; 9810/5.0.0.736) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S66103[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.507; en-us; 9810/5.1.0.507) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S66104[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.546; en-us; 9810/5.1.0.546) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S66105[] = "UCWEB/2.0(BlackBerry; U; 5.0.0.496; en-us; 9810/5.0.0.496) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S66106[] = "UCWEB/2.0(BlackBerry; U; 5.0.0.451; en-us; 9810/5.0.0.451) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S66107[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.181; en-us; 9810/5.1.0.181) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S66108[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.401; en-us; 9810/5.1.0.401) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S66109[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9430 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66110[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-M830 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66111[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Incredible S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66112[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26w Build/6.2.B.0.200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66113[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xperia Acro S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66114[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66115[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S66116[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66117[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310N Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66118[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66119[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310B Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66120[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L900 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/l900";
const char S66121[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-T889 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/t0ltetmo";
const char S66122[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L900 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/l900";
const char S66123[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-N7105 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/t0lte";
const char S66124[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I317 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/t0lteatt";
const char S66125[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-T889 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/t0ltetmo";
const char S66126[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-N7100 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/n7100";
const char S66127[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-N7105 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/t0lte";
const char S66128[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I317 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2.0/t0lte";
const char S66129[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I317 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2.0/t0lte";
const char S66130[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-T889 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/t0ltetmo";
const char S66131[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L900 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/l900";
const char S66132[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I317 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2.0/t0lteatt";
const char S66133[] = "Mozilla/5.0 (Series40; NokiaX2-00/08.25; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.0.9.17.2";
const char S66134[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.2 (KHTML, like Gecko) Comodo_Dragon/15.0.3.0 Chrome/15.0.874.102 Safari/535.2";
const char S66135[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Comodo_Dragon/15.0.3.0 Chrome/15.0.874.102 Safari/535.2";
const char S66136[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.10.229 Version/11.61";
const char S66137[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-P6211 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S66138[] = "Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P6211 Build/HTJ85B)";
const char S66139[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P6211 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S66140[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-B5512 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S66141[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TAB-730 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66142[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAB-730 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66143[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; TAB-730 Build/IMM76D)";
const char S66144[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S66145[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7100 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S66146[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-N719 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S66147[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; SCH-N719 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S66148[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7105 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S66149[] = "Mozilla/5.0 (Linux; Android 4.1.2; B1-A71 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S66150[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP9090 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S66151[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en) Presto/2.9.168 Version/11.50";
const char S66152[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.0.2.0 Chrome/16.0.912.59 Safari/535.7";
const char S66153[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.0.2.0 Chrome/16.0.912.59 Safari/535.7";
const char S66154[] = "NetFront/3.5.1(BREW 5.0.2.1; U; en-us; SAMSUNG; NetFront/3.5.1/WAP) Sprint M370 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S66155[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; ZP900S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66156[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; ZP900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66157[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZP900H Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66158[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; ZP900 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66159[] = "NokiaX2-02/2.0 (11.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiax2-02) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S66160[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT901 Build/7.7.1Q-144_SLS_S240-58) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66161[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT901 Build/7.7.1Q-144_SLS_S240-58) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S66162[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-i705 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S66163[] = "Mozilla/5.0 (Series40; Nokia111/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S66164[] = "Mozilla/5.0 (Series40; Nokia111.1/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S66165[] = "Mozilla/5.0 (Linux; Android 4.1.1; ME301T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S66166[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KUNO3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66167[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03R; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66168[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Galaxy Nexus Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66169[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN79) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S66170[] = "NOKIAN79UCBrowser/8.7.1.234/28/352/UCWEB";
const char S66171[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokian79-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S66172[] = "SAMSUNG-SGH-A997/A997UCMD5 SHP/VPP/R5 NetFront/4.2 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S66173[] = "SAMSUNG-SGH-A997M/A997MUGMC2 SHP/VPP/R5 NetFront/4.2 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S66174[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6210 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S66175[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P6210 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S66176[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ601 Build/I.7.1-45) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S66177[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9080L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66178[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LS840 Build/ZVK.IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66179[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT912 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S66180[] = "Mozilla/5.0 (Linux; Android 4.2.2; Droid Razr M Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S66181[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; M-MP860S2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66182[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1b3pre) Gecko/X Minefield/3.1b3pre";
const char S66183[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1b3pre) Gecko/X Minefield/3.1b2pre Firefox/3.0.3";
const char S66184[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.1b2pre) Gecko/X Minefield/3.1b2pre";
const char S66185[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1a1pre) Gecko/X Firefox Minefield/3.1a1pre";
const char S66186[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S66187[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S66188[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S66189[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S66190[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S66191[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5830L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S66192[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; GT-S5830 Build/GRWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S66193[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5830 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S66194[] = "Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 101G9 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S66195[] = "Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 101G9 Build/MR1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S66196[] = "Mozilla/5.0 (compatible; MSIE 3.0; PPC; Win64; x64; Trident/5.0)";
const char S66197[] = "Mozilla/5.0 (compatible; MSIE 3.0; PPC; .NET CLR; Trident/1.0)";
const char S66198[] = "Mozilla/5.0 (compatible; MSIE 3.0; PPC; WOW64; Trident/3.0)";
const char S66199[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I727 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66200[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I727 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66201[] = "SonyEricssonW518a/R1EB009 Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.2";
const char S66202[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.0.3.2197 Safari/532.4";
const char S66203[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.0.1.0 Chrome/14.0.835.163 Safari/535.1";
const char S66204[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.0.1.0 Chrome/14.0.835.163 Safari/535.1";
const char S66205[] = "Opera/9.80 (Android; Opera Mini/7.29530/27.1407; U; en) Presto/2.8.119 Version/11.10";
const char S66206[] = "Opera/9.80 (Android; Opera Mini/7.29703/27.1573; U; ru) Presto/2.8.119 Version/11.10";
const char S66207[] = "Opera/9.80 (Android; Opera Mini/7.29952/27.1741; U; es) Presto/2.8.119 Version/11.10";
const char S66208[] = "Opera/9.80 (Android; Opera Mini/7.28879/27.1662; U; en-us) Presto/2.8.119 Version/11.10";
const char S66209[] = "Mozilla/5.0 (Windows NT 5.1; rv:16.1) Gecko/X Firefox/16.1 IceDragon/16.1.0.1";
const char S66210[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Rikomagic MK802IIIS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66211[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8) Presto/2.12.388 Version/12.13";
const char S66212[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST21i2 Build/11.0.A.1.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S66213[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST21i Build/11.0.A.0.16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S66214[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.1.8) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S66215[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.4.27) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S66216[] = "SAMSUNG-GT-S5222R/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S66217[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT916 Build/2_26D_2014) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66218[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One SV Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66219[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Vodafone 975N Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30 SVN/070HFN2";
const char S66220[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; IdeaTab A3000-F Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66221[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; IdeaTab A3000-H Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66222[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; IdeaTab A3000-H Build/JDQ39)";
const char S66223[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One X Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S66224[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Legend Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66225[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_Legend Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66226[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; Legend Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66227[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Legend Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66228[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; Legend Build/GRH55)";
const char S66229[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-SU370 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66230[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-SU370 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66231[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-SU370 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66232[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; LG-SU370 Build/FRF91)";
const char S66233[] = "Opera/8.52 (X11; Linux x86_64; U; en)";
const char S66234[] = "Opera/8.5 (GOGI; Linux sh4; U) [en] Amino";
const char S66235[] = "Opera/8.54 (X11; Linux i686; U; pl)";
const char S66236[] = "Opera/8.51 (X11; Linux i686; U; en)";
const char S66237[] = "Opera/8.0 (X11; Linux i686; U; cs)";
const char S66238[] = "Opera/8.51 (GOGI; Linux armv5EJl; U) [en]";
const char S66239[] = "Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux i686; en) Opera 8.02";
const char S66240[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P870 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66241[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAB07-200 Build/Xenta) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66242[] = "Mozilla/5.0 (Series40; Nokia5130c-2/07.95; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S66243[] = "Mozilla/5.0 (Series40; Nokia5130/07.97; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S66244[] = "Mastone_G9_TD/V2.00 Release/3.19.2012 Mozilla/5.0 (Linux; U; Android 2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66245[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SHV-E300L Build/JDQ39)";
const char S66246[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ko-kr; SHV-E300L Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 NAVER(inapp; search; 230; 4.7.7)";
const char S66247[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020A Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66248[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020D Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66249[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66250[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020A Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S66251[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S66252[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020D Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S66253[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/flo";
const char S66254[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/flo";
const char S66255[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.2/grouper";
const char S66256[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/flo";
const char S66257[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E425f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S66258[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E425g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S66259[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E425 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S66260[] = "Mozilla/4.1 (compatible; Teleca Q7; BMP 1.0.1; U; en) 240X400 LG-MN510";
const char S66261[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1 ; en-us; ThinkPad Tablet Build/ThinkPadTablet_A310_02) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S66262[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; NOOK BNRV300 Build/ERD79) Apple WebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S66263[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; NOOK BNRV350 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S66264[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; NOOK BNRV300 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S66265[] = "SonyEricssonS500i/R8BE Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S66266[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; DAMAGEDHERO200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S66267[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HERO200 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S66268[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HERO200 Build/ERE27) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S66269[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; HERO200 Build/ERE27)";
const char S66270[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; HERO200 Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S66271[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; T-Mobile myTouch 3G Slide Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66272[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; T-Mobile myTouch 3G Slide Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66273[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; T-Mobile myTouch 3G Slide Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66274[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; T-Mobile myTouch 3G Slide Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66275[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; T-Mobile myTouch 3G Slide Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66276[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; T-Mobile myTouch 3G Slide Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S66277[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T05A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66278[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; T05A Build/IML74K)";
const char S66279[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T05A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66280[] = "Mozilla/4.0 (Brew MP 1.0.2; U; en-us; Kyocera; NetFront/4.1/AMB) Sprint E4277";
const char S66281[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S66282[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P7300 Build/HTJ85B) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S66283[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7300B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S66284[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A858 Duo Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66285[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-SU660 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66286[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-SU660 Build/IMM76L)";
const char S66287[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9082 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S66288[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9082 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S66289[] = "Mozilla/5.0 (Linux; Android 4.0.3; A100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S66290[] = "Mozilla/5.0 (PlayStation Vita 2.12) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S66291[] = "Mozilla/5.0 (Linux; Android 4.1.1; Prism II Build/HuaweiU8686) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S66292[] = "SimplePie/1.0 b3.2 (Feed Parser; http://simplepie.org/; Allow like Gecko) Build/X";
const char S66293[] = "SimplePie/1.1 (Feed Parser; http://simplepie.org; Allow like Gecko) Build/X";
const char S66294[] = "SimplePie/1.0 (Feed Parser; http://simplepie.org/; Allow like Gecko) Build/X";
const char S66295[] = "SimplePie/1.1.3 (Feed Parser; http://simplepie.org; Allow like Gecko) Build/X (+curl)";
const char S66296[] = "SimplePie/1.2.1 (Feed Parser; http://simplepie.org; Allow like Gecko) Build/X";
const char S66297[] = "SimplePie/1.3.1 (Feed Parser; http://simplepie.org; Allow like Gecko) Build/X";
const char S66298[] = "SimplePie/1.4-dev (Feed Parser; http://simplepie.org; Allow like Gecko) Build/X";
const char S66299[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; VIEWPIA-DR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66300[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; VIEWPIA-DR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66301[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S66302[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S66303[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S66304[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S66305[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5570 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S66306[] = "Mozilla/5.0 (X11; FreeBSD amd64; rv:22.0) Gecko/X Firefox/22.0";
const char S66307[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC6-00) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S66308[] = "SAMSUNG-GT-i8000T Opera 9.5";
const char S66309[] = "SAMSUNG-GT-i8000/1.0 Opera 9.5";
const char S66310[] = "Opera/9.5 (Windows NT 5.1; U; en) SAMSUNG-GT-i8000";
const char S66311[] = "Mozilla/5.0 (S60V3; U; en-in; NokiaN73)/UC Browser8.2.0.132/28/444/UCWEB Mobile";
const char S66312[] = "NokiaN73/UCWEB8.2.0.116/28/800";
const char S66313[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ609 Build/7.7.1-85_MZ609-31) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S66314[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ANGB Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66315[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; SlimBrowser/6.01.091)";
const char S66316[] = "Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S66317[] = "Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S66318[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100B Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315";
const char S66319[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315";
const char S66320[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.53005";
const char S66321[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S66322[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S66323[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6800 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S66324[] = "MOT-VA76r/R63715_U_71.01.82R BER/2.0 Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S66325[] = "Mozilla/5.0 (Windows NT 4.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7";
const char S66326[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.2 Mobile/9A405 Safari/7534.48.3";
const char S66327[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.2 Mobile/9A334 Safari/7534.48.3";
const char S66328[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-us; Micromax A91 Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S66329[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A91 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66330[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5120 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S66331[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; Maxwell Lite Build/v1.0.0.ICS.maxwell.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66332[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Maxwell Lite Build/v1.0.2 X-08:55) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66333[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Maxwell Lite Build/1.0.2.X-17:05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66334[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Maxwell_Lite Build/1.0.2-X-19:27) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66335[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:18.0) Gecko/X Firefox/18.0 SeaMonkey/2.15.2";
const char S66336[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaN97) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S66337[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E971 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66338[] = "Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300T Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S66339[] = "Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300TG Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S66340[] = "Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300TL Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S66341[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Sensation Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S66342[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S66343[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S66344[] = "Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabA2109A Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S66345[] = "Mozilla/5.0 (Linux; Android 4.0.3; A100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S66346[] = "NokiaN93-1/20.0.041 SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1, NokiaN93-1/20.0.041 SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S66347[] = "NokiaN93/11.0.034 SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S66348[] = "NokiaN93-1/20.0.041";
const char S66349[] = "NokiaN93-1/20.0.041 SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S66350[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Iconia A500 Build/IMM76D; CyanogenMod-Tegraowners ICS ROM v158 (thor & digetx)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66351[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Iconia A500 Build/IMM76I; CyanogenMod-Tegraowners ICS ROM v169 (thor & digetx)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66352[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Iconia A500 Build/IMM76L; CyanogenMod-Tegraowners ICS ROM v170 (thor & digetx)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66353[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A500 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66354[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; A500 Build/IML77; CyanogenMod-TeamDRH) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66355[] = "Mozilla/5.0 (X11; Linux x86_64; U; de; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.62";
const char S66356[] = "Mozilla/4.0 (compatible; MSIE 8.0; X11; Linux x86_64; de) Opera 10.62";
const char S66357[] = "Mozilla/5.0 (Linux; Android 4.0.4; LIFETAB_S9512 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S66358[] = "Mozilla/5.0 (Linux; Android 4.0.3; LIFETAB_S9512 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S66359[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9003 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66360[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; GT-I9003 Build/JRO03L; CyanogenMod-remicsjb v2.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66361[] = "UCWEB/2.0(BlackBerry; U; 9.16.0.158; en-us; 9790/9.16.0.158) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S66362[] = "UCWEB/2.0(BlackBerry; U; 9.16.0.57; en-us; 9790/9.16.0.57) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S66363[] = "UCWEB/2.0(BlackBerry; U; 9.16.0.67; en-us; 9790/9.16.0.67) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S66364[] = "UCWEB/2.0(BlackBerry; U; 9.16.0.128; en-us; 9790/9.16.0.128) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S66365[] = "UCWEB/2.0(BlackBerry; U; 9.16.0.129; en-us; 9790/9.16.0.129) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S66366[] = "UCWEB/2.0(BlackBerry; U; 9.16.0.33; en-us; 9790/9.16.0.33) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S66367[] = "UCWEB/2.0(BlackBerry; U; 9.16.0.102; en-us; 9790/9.16.0.102) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S66368[] = "UCWEB/2.0(BlackBerry; U; 9.16.0.159; en-us; 9790/9.16.0.159) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S66369[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SPHS on Hsdroid Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66370[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-us; SPHS on Hsdroid Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66371[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; SPHS on Hsdroid Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66372[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-us; SPHS on Hsdroid Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66373[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; SPHS on Hsdroid Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66374[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; SPHS on Hsdroid Build/Android4.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66375[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Barnes & Noble Nook HD Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66376[] = "UCWEB/2.0 (iOS; U; iPh OS 5_1_1; zh-CN; iPh4,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile";
const char S66377[] = "UCWEB/2.0 (iOS; U; iPh OS 5_1_1; zh-CN; iPh3,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile";
const char S66378[] = "UCWEB/2.0 (iOS; U; iPh OS 5_1_1; en-US; iPh2,1) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S66379[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66380[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66381[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S66382[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S66383[] = "Mozilla/5.0 (Linux; Android 4.1.2; ADR910L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S66384[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5306 Build/12.0.A.1.284) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66385[] = "Mozilla/5.0 (Linux; Android 4.2.2; TouchPad Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S66386[] = "HTC_Touch_Pro2_T7373 Opera/9.7 (Windows NT 5.1; U; nl)";
const char S66387[] = "HTC_Touch_Pro2_T7373 Opera/9.7 (Windows NT 5.1; U; da)";
const char S66388[] = "HTC_Touch_Pro2_T7373 Opera/9.7 (Windows NT 5.1; U; de)";
const char S66389[] = "HTC_Touch_Pro2_T7373 Opera/9.7 (Windows NT 5.1; U; ru)";
const char S66390[] = "HTC_Touch_Pro2_T7373 Opera/9.7 (Windows NT 5.1; U; nb)";
const char S66391[] = "HTC_Touch_Pro2_T7373 Opera/9.7 (Windows NT 5.1; U; it)";
const char S66392[] = "HTC_Touch_Pro2_T7373 Opera/9.7 (Windows NT 5.1; U; pt)";
const char S66393[] = "HTC_Touch_Pro2_T7373 Opera/9.7 (Windows NT 5.1; U; es-ES)";
const char S66394[] = "HTC_Touch_Pro2_T7373 Opera/9.7 (Windows NT 5.1; U; en)";
const char S66395[] = "HTC_Touch_Pro2_T7373 Opera/9.7 (Windows NT 5.1; U; sv)";
const char S66396[] = "HTC_Touch_Pro2_T7373 Opera/9.7 (Windows NT 5.1; U; fr)";
const char S66397[] = "Nokia6111/2.0 (03.82) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S66398[] = "Mozilla/4.0 (compatible; MSIE 6.0; PPC Mac OS X 10.6.5; Tasman 1.0)";
const char S66399[] = "Mozilla/4.0 (compatible; MSIE 6.0; PPC Mac OS X 10.6.7; Tasman 1.0)";
const char S66400[] = "Mozilla/4.0 (compatible; MSIE 6.0; PPC Mac OS X 10.6.8; Tasman 1.0)";
const char S66401[] = "Mozilla/4.0 (compatible; MSIE 6.0; PPC Mac OS X 10.6.4; Tasman 1.0)";
const char S66402[] = "Mozilla/4.0 (compatible; MSIE 6.0; PPC Mac OS X 10.6.6; Tasman 1.0)";
const char S66403[] = "Mozilla/4.0 (compatible; MSIE 6.0; PPC Mac OS X 10.6.3; Tasman 1.0)";
const char S66404[] = "Mozilla/4.0 (compatible; MSIE 6.0; PPC Mac OS X 10.6.2; Tasman 1.0)";
const char S66405[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.10";
const char S66406[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.2 Mobile/8C148 Safari/6533.18.5";
const char S66407[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8C148 Safari/6531.22.7";
const char S66408[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148,Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S66409[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S66410[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S66411[] = "Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S66412[] = "Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire2 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S66413[] = "Mozilla/5.0 (Linux; Android 4.2; Amazon Kindle Fire Build/JOP40C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S66414[] = "Mozilla/5.0 (Linux; Android 4.2.1; Amazon Kindle Fire Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S66415[] = "Mozilla/5.0 (Linux; Android 4.1.1; IdeaTabA2109A Build/JRO03R) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S66416[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-AS730 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66417[] = "SAMSUNG-SGH-A187/A187UCJH2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S66418[] = "Apple-PubSub/65.32";
const char S66419[] = "Mozilla/5.0 (Java; U; en-in; gt-s5233s) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S66420[] = "Mozilla/5.0 (Java; U; vi; gt-s5233s) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S66421[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT30a Build/7.0.A.3.195) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66422[] = "Mercury/7.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S66423[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Tablet PC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66424[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Tablet PC 3000 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66425[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PAD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66426[] = "Mozilla/5.0 (Linux; Android 4.2.1; v89_zlh_hd Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66427[] = "Mozilla/5.0 (Linux; Android 4.2.2; p4wifi Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S66428[] = "Mozilla/5.0 (Linux; Android 4.2.1; v89_gq2008s Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66429[] = "Mozilla/5.0 (Linux; Android 4.2.2; Transformer Prime Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S66430[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-l900 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66431[] = "Mozilla/5.0 (Linux; Android 4.2.1; Q508 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66432[] = "Mozilla/5.0 (Linux; Android 4.2.1; X508 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66433[] = "Mozilla/5.0 (Linux; Android 4.2.2; MB865) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66434[] = "Mozilla/5.0 (Linux; Android 4.2.1; v89_gq3008s Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66435[] = "Mozilla/5.0 (Linux; Android 4.2.2; Z2 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66436[] = "Mozilla/5.0 (Linux; Android 4.2.1; 7102 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66437[] = "Mozilla/5.0 (Linux; Android 4.2.1; H3088 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66438[] = "Mozilla/5.0 (Linux; Android 4.2.1; B943 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66439[] = "Mozilla/5.0 (Linux; Android 4.2.1; H920 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66440[] = "Mozilla/5.0 (Linux; Android 4.2.1; v89_jbl1a668 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66441[] = "Mozilla/5.0 (Linux; Android 4.2.1; TS4 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66442[] = "Mozilla/5.0 (Linux; Android 4.2.1; A19S Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66443[] = "Mozilla/5.0 (Linux; Android 4.2.1; Aveo NOTE II+ Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66444[] = "Mozilla/5.0 (Linux; Android 4.2.2; One S C2 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66445[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5220 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S66446[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5360) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S66447[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; vi; GT-S5360) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S66448[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM103SH Build/S0006) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66449[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM103SH Build/S0012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66450[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SBM103SH Build/S0012)";
const char S66451[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM103SH Build/S0014) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66452[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; myTouch_4G_Slide Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66453[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; myTouch_4G_Slide Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66454[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ZP500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66455[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZP500+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66456[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PAP4500DUO Build/PrestigioPAP4500DUO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66457[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I777 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66458[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-I777 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66459[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P3110 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S66460[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P3110 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S66461[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P3110 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66462[] = "Mozilla/5.0 (Linux; Android 4.1.2; PantechP8010 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66463[] = "Mozilla/5.0 (PLAYSTATION 3 4.20) AppleWebKit/531.22.8 (KHTML, like Gecko)";
const char S66464[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T869 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66465[] = "Nokia308/2.0 (05.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia308) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S66466[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SH530U Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66467[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SH530U Build/JZO54K)";
const char S66468[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; WM8850-mid Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66469[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; WM8850-mid Build/JRO03H)";
const char S66470[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; WM8850:X6-10V Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66471[] = "Mozilla/5.0 (Linux; Android 4.3; Galaxy Nexus Build/JWR66V) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S66472[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT28h Build/6.2.B.0.211) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66473[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT28i Build/6.2.B.0.211) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66474[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S66475[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; jbed-fastbcc) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S66476[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mozilla) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S66477[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; spice) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S66478[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; generic) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S66479[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sunmicro) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S66480[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; karbonn) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S66481[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; opera) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile TRUSTED/1.0";
const char S66482[] = "UCWEB/2.0 (Java; U; MIDP-2.0; Pt-BR; mozilla) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S66483[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; s8000) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S66484[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; spice) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile TRUSTED/1.0";
const char S66485[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile TRUSTED/1.0";
const char S66486[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; spreadtrum6530) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S66487[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; 11a h700tecno) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S66488[] = "Mozilla/5.0 (Series40; NokiaX2-00/08.25; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S66489[] = "JUC (Linux; U; 2.3.6; zh-cn; GT-N7000; 800*1280) UCWEB7.9.0.94/139/444";
const char S66490[] = "JUC (Linux; U; 4.0.4; zh-cn; GT-N7000; 800*1280) UCWEB7.9.0.94/139/444";
const char S66491[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT7033E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66492[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC One Build/JZO54K; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66493[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/m7att";
const char S66494[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6802 Build/14.1.B.0.461) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66495[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6802 Build/14.1.B.0.461) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S66496[] = "Mozilla/5.0 (Linux; Android 4.1.2; N9101 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S66497[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/11A465 Safari/8536.25";
const char S66498[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/11A501 Safari/8536.25";
const char S66499[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.8.1.20) Gecko/X Firefox/2.0.0.20";
const char S66500[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; M032 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66501[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; M032 Build/JRO03H)";
const char S66502[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E970 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66503[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; R8113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66504[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; R8113 Build/JRO03C)";
const char S66505[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I927 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66506[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:16.0) Gecko/X Thunderbird/16.0.2";
const char S66507[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; ZTE T82 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66508[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; T-17P Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66509[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; T-17P Build/JRO03H)";
const char S66510[] = "Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_S9714 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S66511[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I717M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66512[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6502 Build/10.1.1.A.1.253) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66513[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; M9 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66514[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; M9 Build/JDQ39)";
const char S66515[] = "Mozilla/4.0 (compatible; MSIE 5.0; Series80/2.0 Nokia9300/05.22 Profile/MIDP-2.0 Configuration/CLDC-1.1)";
const char S66516[] = "Mozilla/4.0 (compatible; MSIE 5.0; Series80/2.0 Nokia9300i/5.50 Profile/MIDP-2.0 Configuration/CLDC-1.1)";
const char S66517[] = "Mozilla/4.0 (compatible; MSIE 5.0; Series80/2.0 Nokia9300i Profile/MIDP-2.0 Configuration/CLDC-1.1)";
const char S66518[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Stainless/0.8 like Version/5.1 Safari/534.48.3";
const char S66519[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Stainless/0.8 like Version/5.1 Safari/534.48.3";
const char S66520[] = "Mozilla/4.0 (compatible; MSIE 5.01; Windows CE)";
const char S66521[] = "Mozilla/5.0 (compatible; MSIE 5.0; WindowsCE; Trident/5.0)";
const char S66522[] = "Mozilla/5.0 (compatible; MSIE 5.0; 68K; Tablet PC; Trident/2.0)";
const char S66523[] = "Mozilla/5.0 (compatible; MSIE 5.0; PPC; Tablet PC; Trident/2.0)";
const char S66524[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11";
const char S66525[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11";
const char S66526[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S66527[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; JY-G3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66528[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; JY-G3 Build/JZO54K)";
const char S66529[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; JY-G3N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66530[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; SmartPad 85S23G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66531[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; M-MP85S23G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66532[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7510 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S66533[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P7510 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S66534[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S7562 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S66535[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-S7562i Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S66536[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9100G Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S66537[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-I9100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S66538[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-I9100 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S66539[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Kaiser/3.28.161.1";
const char S66540[] = "HTC_P4550 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S66541[] = "HTC_TyTN_II Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S66542[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Kaiser/3.28.162.2";
const char S66543[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Kaiser/3.08.161.0";
const char S66544[] = "HTC_P4550/TYTN_II Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S66545[] = "HTC_TyTN_II Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) HTC_TyTN_II Mozilla/4.0";
const char S66546[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC P4550; OpVer 24.328.1.610";
const char S66547[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC P4550; OpVer 24.181.1.612";
const char S66548[] = "Mozilla/5.0 (Vodafone/1.0/LG-GT400/V10d Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S66549[] = "Mozilla/5.0 (Vodafone/1.0/LG-GT400/V10e Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S66550[] = "LG-GT400-Bouygues/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S66551[] = "LG-GT400-Orange/V10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S66552[] = "LG-GT400/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S66553[] = "Mozilla/5.0 (Vodafone/1.0/LG-GT400/V10g Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S66554[] = "Mozilla/5.0 (Vodafone/1.0/LG-GT400/V10c Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S66555[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Version/3.1.2 Safari/525.21";
const char S66556[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.17";
const char S66557[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13";
const char S66558[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; dL1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66559[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P705g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66560[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P705 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66561[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P708g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66562[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P705f Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66563[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; GT7000DC Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66564[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; MT7008 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66565[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MT7007 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66566[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MT7001 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66567[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; MT7001 Build/JRO03C)";
const char S66568[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; T7005 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66569[] = "Nokia202/2.0 (20.52) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia202) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S66570[] = "Nokia203/2.0 (20.52) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia203) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S66571[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7560M Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S66572[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/Chicken Tits by Bloodawn)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66573[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/Chicken-Tits by Bloodawn)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66574[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66575[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S66576[] = "Mozilla/5.0 (X11; FreeBSD x86_64; rv:2.0) Gecko/X Firefox/3.6.12";
const char S66577[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:16.0) Gecko/X Thunderbird/16.0.2";
const char S66578[] = "Mozilla/5.0 (Symbian3; U; en-us; NokiaC7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/51/352/UCWEB Mobile";
const char S66579[] = "NokiaC7-00/UCWEB8.6.0.180/51/800";
const char S66580[] = "Mozilla/5.0 (Java; U; en-us; nokiac7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile  3gpp-gba";
const char S66581[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; W732 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66582[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; W732 Build/IML74K)";
const char S66583[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MITO_A300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66584[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MITO A300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66585[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MITO_A300 Build/IMM76D)";
const char S66586[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MITO A300 Build/IMM76D)";
const char S66587[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S66588[] = "Mozilla/5.0 (Linux; Android 4.0.4; Next7P12-8G Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S66589[] = "Mozilla/5.0 (Linux; Android 4.0.4; Next7P12 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S66590[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9190 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66591[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9190 Build/JDQ39)";
const char S66592[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; PC36100 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66593[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; PC36100 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66594[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; EVO Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66595[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; EVO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66596[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; EVO Build/JRO03C)";
const char S66597[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Evo 4G LTE Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66598[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; PC36100 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66599[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L300 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66600[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L300 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S66601[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac5-00) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile  3gpp-gba";
const char S66602[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac5-00.2) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile  3gpp-gba";
const char S66603[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaC5-00.2) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S66604[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaC5-00) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S66605[] = "Mozilla/5.0 R800 like Gecko/20060426 Netfront/3.4";
const char S66606[] = "Yoigo F233/1.0 ACS-NF/3.4 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S66607[] = "samr810 Netfront/3.4 Mozilla/5.0 like Gecko/20060426";
const char S66608[] = "Mozilla/5.0 (DigitalMediaPlayer; Walkman/NWZ-X1001 Series/1.00; like Gecko; wireless) NetFront/3.4";
const char S66609[] = "Mozilla/3.0(WILLCOM;KYOCERA/WX340K/2;3.0.3.11.X/1/C256) NetFront/3.4";
const char S66610[] = "Mozilla/5.0 (DigitalMediaPlayer; Walkman/NWZ-X1000 Series/1.00; like Gecko; wireless) NetFront/3.4";
const char S66611[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-c6712) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile";
const char S66612[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 7 Build/1.1.3 X-11:40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66613[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 7 Build/1.0.3 X-12:51) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66614[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 7 Build/1.0.2 X-15:15) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66615[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 7 Build/1.0.0 X-15:30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66616[] = "Mozilla/5.0 (Windows; N; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Qt/4.6.2 Safari/532.4";
const char S66617[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; PC906 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66618[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; PC906 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66619[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; PAD703 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66620[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; PAD703 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66621[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; PAD703 Build/IML74K)";
const char S66622[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TECNO F7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66623[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6506 Build/10.1.A.1.350) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S66624[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/11A465 Safari/8536.25";
const char S66625[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/11A501 Safari/8536.25";
const char S66626[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; IS11T Build/FGK400) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66627[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; IS11T Build/FGK401) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66628[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; L-02D Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66629[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; L-02D Build/GWK74)";
const char S66630[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; de; rv:1.9.1.16) Gecko/X Thunderbird/3.0.11";
const char S66631[] = "Mozilla/5.0 (SaferSurf rv:12.0) Gecko/X Firefox/12.0";
const char S66632[] = "Mozilla/5.0 (masking-agent; rv:12.0) Gecko/X Firefox/12.0";
const char S66633[] = "Firefox/12.0 YB/5.2.2";
const char S66634[] = "Mozilla/6.0 (compatible;MSIE 5.0;  Windows NT 6.0; QQDownload)";
const char S66635[] = "Mozilla/3.0 (compatible;MSIE 5.0;  Windows NT 6.0; QQDownload)";
const char S66636[] = "Mozilla/5.0 (compatible; MSIE 5.0; Windows NT 6.0; Trident/4.0)";
const char S66637[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 6.0; Trident/4.1)";
const char S66638[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC407 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66639[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MIDC407 Build/eng.glwx.X.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66640[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC407 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66641[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MIDC407 Build/MIDC407) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66642[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MIDC407 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66643[] = "Opera/9.80 (Linux armv7l; Maemo; Opera Mobi/16) Presto/2.11.355 Version/12.10";
const char S66644[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9000 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S66645[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9000 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S66646[] = "Mozilla/5.0 (Linux; Android 4.0.4; Motorola RAZR MAXX Build/6.7.2-180_DHD-16_NA-27) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S66647[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS950 4G Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S66648[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC-Z710a Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66649[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66650[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66651[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710a Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66652[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.12 (KHTML, like Gecko) Puffin/1.3.4232M Safari/534.12";
const char S66653[] = "FreeRSSReader/1.9.5 CFNetwork/548.1.4 Darwin/11.0.0";
const char S66654[] = "MobileRSSFree/3.4 CFNetwork/548.1.4 Darwin/11.0.0";
const char S66655[] = "MobileRSS/3.4 CFNetwork/548.1.4 Darwin/11.0.0";
const char S66656[] = "Pics4Facebook/9.15.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S66657[] = "RSSReader/1.04 CFNetwork/548.1.4 Darwin/11.0.0";
const char S66658[] = "RSSRadio/2.07 CFNetwork/548.1.4 Darwin/11.0.0";
const char S66659[] = "FeeddlerRSS/1.12.5 CFNetwork/548.1.4 Darwin/11.0.0";
const char S66660[] = "Mozilla/5.0 (Windows; N; Windows NT 6.0; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.2.2.6613 Safari/532.4";
const char S66661[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.0.6000.0);en;kml:2.2;client:Free;type:default)";
const char S66662[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; K-Touch W619 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S66663[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; K-Touch_W619 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66664[] = "Mozilla/5.0 (Linux; U; Android 2.3 YunOs 1.0.0.3; zh-cn; K-Touch W619 Build/AliyunOs-2012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66665[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; W619 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66666[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; K-Touch W619 Build/GRK39F)";
const char S66667[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; K-Touch W619 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66668[] = "Mozilla/5.0 (Linux; Android 4.0.3; Smartfren Tab 7 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S66669[] = "Mozilla/5.0 (Linux; Android 4.0.4; Tab 7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S66670[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GEM10313S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66671[] = "MOT-V3re/0E.43.0AR MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S66672[] = "MOT-V3r/08.BD.43R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S66673[] = "MOT-V3e/08.03.05R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S66674[] = "MOT-V3/0E.42.10R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S66675[] = "MOT-V3v/08.BD.B3R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S66676[] = "MOT-V3t/0E.C8.0DR MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S66677[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-I897 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66678[] = "Mozilla/5.0 (Windows NT 4.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11";
const char S66679[] = "Reeder/1020.09.00 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookAir4%2C2)";
const char S66680[] = "Reeder/1020.09.00 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (iMac10%2C1)";
const char S66681[] = "Reeder/1020.09.00 CFNetwork/520.5.1 Darwin/11.4.2 (i386) (MacBook2%2C1)";
const char S66682[] = "Reeder/1020.09.00 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)";
const char S66683[] = "Reeder/1020.09.00 CFNetwork/520.5.1 Darwin/11.4.2 (i386) (MacPro1%2C1)";
const char S66684[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; i-mobile IQ 5.1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66685[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; i-mobile IQ 5.1A Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66686[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; i-mobile IQ 5.1A Build/JZO54K)";
const char S66687[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9737AP Mobile";
const char S66688[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP Mobile";
const char S66689[] = "Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5130c-2) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S66690[] = "Nokia5130/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5130) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S66691[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I777 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S66692[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-i777 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S66693[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Butterfly Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S66694[] = "Nokia201/2.0 (11.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia201) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S66695[] = "AmigaVoyager/3.4.4 (MorphOS/PPC native)";
const char S66696[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T100 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66697[] = "J2ME/UCWEB7.0.3.45/139/7687";
const char S66698[] = "J2ME/UCWEB7.0.3.45/140/7704";
const char S66699[] = "J2ME/UCWEB7.0.3.45/140/7665";
const char S66700[] = "J2ME/UCWEB7.0.3.45/140/7689";
const char S66701[] = "J2ME/UCWEB7.0.3.45/139/7602";
const char S66702[] = "J2ME/UCWEB7.0.3.45/139/7685";
const char S66703[] = "J2ME/UCWEB7.0.3.45/139/7672";
const char S66704[] = "J2ME/UCWEB7.0.3.45/139/7694";
const char S66705[] = "J2ME/UCWEB7.0.3.45/139/7705";
const char S66706[] = "J2ME/UCWEB7.0.3.45/139/7695";
const char S66707[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I577 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66708[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; DIT9720 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66709[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; kingdom Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66710[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; tervigon Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66711[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Crescent Build/JRO03C; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66712[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Flyer Build/JZO54K; CyanogenMod-Flyer) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66713[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ThinkPadTablet Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66714[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Atrix 4G Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66715[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Live with Walkman Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66716[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Notion Ink ADAM Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66717[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-i9003 Build/JZO54K; CyanogenMod-EHNDROIX II i9003) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66718[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; warp2 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66719[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N51 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66720[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire SV Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66721[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66722[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire S Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66723[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaN8-00) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S66724[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; S5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66725[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; S5 Build/JOP40D)";
const char S66726[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S66727[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-N7000 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S66728[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OSX 10_5_7; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S66729[] = "Safari/6531.21.10 CFNetwork/454.5 Darwin/10.2.0 (i386) (MacBookPro5)";
const char S66730[] = "Safari/6531.9 CFNetwork/454.4 Darwin/10.1.1 (i386) (MacBook6%2C1)";
const char S66731[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; A111 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66732[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; A111 Build/JZO54K)";
const char S66733[] = "Mozilla/5.0 (Windows; U; ; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.6.6.X (Second Life Release; default skin) Safari/533.3";
const char S66734[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.7 (KHTML, like Gecko) Ubuntu/11.04 Chromium/16.0.912.77 Chrome/16.0.912.77 Safari/535.7";
const char S66735[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.7 (KHTML, like Gecko) Ubuntu/11.04 Chromium/16.0.912.77 Chrome/16.0.912.77 Safari/535.7";
const char S66736[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; ZTE R22 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66737[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID7032 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66738[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Opera/9.80 (Android; Opera Mini/6.5.26969/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S66739[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Opera/9.80 (Android; Opera Mini/6.5.26969/28.3126; U; en) Presto/2.8.119 Version/11.10";
const char S66740[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Fly IQ441; Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66741[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; IQ441 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66742[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; IQ441 Build/IMM76D)";
const char S66743[] = "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:24.0) Gecko/X Firefox/24.0";
const char S66744[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SYTABEX7-2 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66745[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; SYTABEX7-2 Build/FRF85B)";
const char S66746[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PC36100 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66747[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PC36100 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66748[] = "Mozilla/5.0 (Linux; U; Android 4.0.4 SM 2.0.3; en-us; PC36100 Build/IMM76L; CyanogenMod-SmurfMod 9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66749[] = "Mozilla/5.0 (Series40; NokiaC2-01/11.20; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S66750[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:16.0) Gecko/X Thunderbird/16.0.2";
const char S66751[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P3100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315";
const char S66752[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3100 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315";
const char S66753[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3100B Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315";
const char S66754[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3100 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S66755[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P3100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S66756[] = "Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10 UCBrowser/2.4.2.348";
const char S66757[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Palm-Treo850e BM BMC";
const char S66758[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Telus Treo850e";
const char S66759[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Alltel_Treo850e";
const char S66760[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Sprint Treo850e";
const char S66761[] = "Treo850e/v0100 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Sprint Treo850e";
const char S66762[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) 320X320 Palm Treo850e";
const char S66763[] = "Treo850e/v0100 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S66764[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko)";
const char S66765[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_2 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko)";
const char S66766[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_2 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Mobile/5G77";
const char S66767[] = "Palm680/RC1 (iPhone; U; CPU iPhone OS 2_2_1 like Mac OS X; zh-cn)/UC Browser 7.1.1.53/39/352";
const char S66768[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Mobile/5H11";
const char S66769[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; U20i Build/4.0.A.2.335) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66770[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; U20i Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66771[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; U20i Build/2.3.5.SB.V7.SP2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66772[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U20i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66773[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; U20i Build/GingerDX Pro) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66774[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; U20i Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66775[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; U20i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66776[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; U20i Build/3.0.1.A.0.145)";
const char S66777[] = "Mozilla/5.0 (Linux; Android 4.0.4; S500 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66778[] = "Mozilla/5.0 (Linux; Android 4.0.4; TCL S500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66779[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 Sleipnir/3.8.4";
const char S66780[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A78 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66781[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A78 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66782[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A78 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66783[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Mobile Safari/535.19 Silk-Accelerated=false";
const char S66784[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3110 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315";
const char S66785[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3110 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315";
const char S66786[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3110 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S66787[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3110 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S66788[] = "Mozilla/5.0 (Linux; Android 4.0.4; Motorola RAZR MAXX Build/6.7.2-180_DHD-16_NA-27) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66789[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.1.0.0 Chrome/12.0.742.91 Safari/534.30";
const char S66790[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.1.0.0 Chrome/12.0.742.91 Safari/534.30";
const char S66791[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5660 Build/GRWK74; CyanogenMod-7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66792[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5660 Build/GWK74; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66793[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5660 Build/GINGERBREAD; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66794[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; GT-S5660 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66795[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT611 Build/V4.55B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66796[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT611 Build/V4.53A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66797[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT611 Build/V4.580) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66798[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT611 Build/V4.650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66799[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT611 Build/V4.54A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66800[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT611 Build/V4.570) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66801[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Comodo_Dragon/21.0.2.0";
const char S66802[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Comodo_Dragon/21.0.2.0";
const char S66803[] = "Mozilla/5.0 (Linux; Android 4.0.4; Inspire 4G Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66804[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Inspire 4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66805[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Inspire 4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66806[] = "Nokia311/5.0 (03.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia311) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S66807[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-F120L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66808[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Mobile Safari/535.19 Silk-Accelerated=false";
const char S66809[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P715 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66810[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; C2104 Build/15.0.A.1.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66811[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; C2104 Build/15.0.A.1.31)";
const char S66812[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyC2104 Build/15.0.A.1.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66813[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-ca; SonyC2104 Build/15.0.A.1.36) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66814[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; C2104 Build/15.0.A.2.17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66815[] = "Mozilla/5.0 (Linux; Android 4.1.2; Micromax A116 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S66816[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI MediaPad Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S66817[] = "Mozilla/5.0 (Linux; Android 4.0.3; MediaPad 7 Lite Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S66818[] = "Mozilla/5.0 (Linux; Android 4.0.4; MediaPad 10 LINK Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S66819[] = "Mozilla/5.0 (Linux; Android 4.0.3; SpringBoard Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S66820[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5360 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S66821[] = "NokiaE63/UCWEB7.9.0.94/28/800";
const char S66822[] = "NokiaE63/UC Browser7.9.0.102/28/355/UCWEB";
const char S66823[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66824[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaN97 mini) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S66825[] = "NokiaN97 miniUCBrowser/8.7.1.234/50/444/UCWEB";
const char S66826[] = "Mozilla/5.0 (Linux; Android 4.1.1; V360 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66827[] = "Mozilla/5.0 (Linux; Android 4.1.1; CINK+ Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66828[] = "Mozilla/5.0 (Linux; Android 4.1.2; CINK FIVE Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66829[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8010 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S66830[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8010 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S66831[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Starmobile ASTRA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66832[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Mobile Safari/535.19 Silk-Accelerated=false";
const char S66833[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT300 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S66834[] = "Mozilla/5.0 (SunOS 5.8 sun4u; U) Opera 5.0 [en]";
const char S66835[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S66836[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S66837[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S66838[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S66839[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S66840[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S66841[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LG-P999 Build/GRI40; CyanogenMod-9.9.9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66842[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P999 Build/GRI40; CyanogenMod-9.9.9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66843[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.2 Mobile/9A405 Safari/7534.48.3";
const char S66844[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.2 Mobile/9A334 Safari/7534.48.3";
const char S66845[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT27i Build/6.1.1.B.1.54) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S66846[] = "Mozilla/5.0 (Series40; Nokia206/03.59; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S66847[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S66848[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S66849[] = "Mozilla/5.0 (Linux; Android 4.0.3; Le Pan S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S66850[] = "Mozilla/5.0 (Linux; Android 4.2.1; Micromax A110Q Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S66851[] = "SAMSUNG-SGH-A737/UCGI3 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S66852[] = "Mozilla/5.0(Linux;U;Android2.1-update1;Zh_cn;E15i;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S66853[] = "JUC(Linux;U;Android2.1-update1;Zh_cn;E15i;320*480;)UCWEB7.8.0.95/139/444";
const char S66854[] = "LG-C105/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S66855[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/14.0.835.202)";
const char S66856[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/14.0.835.202) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S66857[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/14.0.835.186) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1";
const char S66858[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/14.0.835.202) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S66859[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/14.0.835.202; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)";
const char S66860[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/14.0.835.186)";
const char S66861[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/14.0.835.202; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)";
const char S66862[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/14.0.835.187) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.187 Safari/535.1";
const char S66863[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/14.0.835.186) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1";
const char S66864[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/14.0.835.202; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C; .NET4.0E)";
const char S66865[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/14.0.835.163) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.163 Safari/535.1";
const char S66866[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/14.0.835.202; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3)";
const char S66867[] = "Mozilla/5.0 (Windows NT 6.1; chromeframe/14.0.835.94) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.94 Safari/535.1";
const char S66868[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/14.0.835.202; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)";
const char S66869[] = "Mozilla/5.0 (X11; U; Linux i686; en-GB; rv:1.9.2.8) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.8";
const char S66870[] = "Mozilla/5.0 (X11; U; Linux i686; nl; rv:1.9.2.17) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.17";
const char S66871[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.12) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.12";
const char S66872[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.11) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.11";
const char S66873[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-GB; rv:1.9.2.17) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.17";
const char S66874[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.17) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.17 GTB7.1";
const char S66875[] = "Mozilla/5.0 (X11; U; Linux i686; nb-NO; rv:1.9.2.17) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.17";
const char S66876[] = "Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.2.17) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.17";
const char S66877[] = "Mozilla/5.0 (X11; U; Linux i686; en-GB; rv:1.9.2.15) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.15";
const char S66878[] = "Mozilla/5.0 (X11; U; Linux i686; it; rv:1.9.2.17) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.17";
const char S66879[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.9) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.9";
const char S66880[] = "Mozilla/5.0 (X11; U; Linux i686; fi-FI; rv:1.9.2.17) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.17";
const char S66881[] = "Mozilla/5.0 (X11; U; Linux x86_64; de; rv:1.9.2.17) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.17";
const char S66882[] = "Mozilla/5.0 (X11; U; Linux i686; fr; rv:1.9.2.13) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.13";
const char S66883[] = "Mozilla/5.0 (X11; U; Linux i686; pl-PL; rv:1.9.2.17) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.17";
const char S66884[] = "Mozilla/5.0 (Series40; NokiaC2-03/06.51; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15";
const char S66885[] = "Nokia311/5.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia311) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S66886[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6602 Build/10.3.A.0.423) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66887[] = "LG-CU720/V1.0l Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S66888[] = "Nokia6600/1.0 (5.53.0) SymbianOS/7.0s Series60/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S66889[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; Girafabot; girafabot at girafa dot com; http://www.girafa.com)";
const char S66890[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.0 Chrome/13.0.800.0 Safari/535.1";
const char S66891[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.1 Chrome/13.0.800.1 Safari/535.1";
const char S66892[] = "Mozilla/5.0 (PlayStation Vita 1.69) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2";
const char S66893[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ADR6425LVW Build/IML74K; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66894[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ADR6425LVW Build/IML74K; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66895[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 7 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S66896[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S66897[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);";
const char S66898[] = "Mozilla/5.0 (compatible; MSIE 8.0; WindowsCE; Trident/4.0)";
const char S66899[] = "Mozilla/5.0 (compatible; MSIE 8.0; PPC; Tablet PC; Trident/3.0)";
const char S66900[] = "Mozilla/5.0 (compatible; MSIE 8.0; 68K; Tablet PC; Trident/2.0)";
const char S66901[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9505 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb";
const char S66902[] = "Mozilla/5.0 (Linux; Android 4.2.2; One S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66903[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S66904[] = "Mozilla/5.0 (Linux; Android 4.0.4; Next7P12-8G Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S66905[] = "Mozilla/5.0 (Linux; Android 4.0.4; Next7P12 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S66906[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8020 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S66907[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S66908[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE55-1/034.001; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.1.5";
const char S66909[] = "Nokia5130c-2/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5130c-2/UC Browser8.0.3.107/69/444";
const char S66910[] = "Nokia5130/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5130/UC Browser8.0.3.107/69/444";
const char S66911[] = "Nokia5130c-2/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/69/444";
const char S66912[] = "Mozilla/5.0 (Windows NT 4.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S66913[] = "SPH-M340/1.0 UP.Browser/7.2.7.2.510 (GUI) MMP/2.0";
const char S66914[] = "SPH-M340-C/1.0 UP.Browser/7.2.7.2.510 (GUI) MMP/2.0";
const char S66915[] = "sam-r560 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S66916[] = "Mozilla/3.0 (Planetweb/2.100 JS SSL US; Dreamcast US)";
const char S66917[] = "SAMSUNG-GT-E2222/E2222DDKE5 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S66918[] = "iOS/6.1 (10B143) dataaccessd/1.0";
const char S66919[] = "iOS/6.1 (10B142) dataaccessd/1.0";
const char S66920[] = "iOS/6.1 (10B141) dataaccessd/1.0";
const char S66921[] = "iOS/6.1 (10B144) dataaccessd/1.0";
const char S66922[] = "iOS/6.1.1 (10B145) dataaccessd/1.0";
const char S66923[] = "iOS/6.1.2 (10B146) dataaccessd/1.0";
const char S66924[] = "iOS/6.1.3 (10B329) dataaccessd/1.0";
const char S66925[] = "iOS/6.1.4 (10B350) dataaccessd/1.0";
const char S66926[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C6502 Build/10.1.A.1.434) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66927[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; C6502 Build/10.1.A.1.434)";
const char S66928[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C6502 Build/10.1.1.A.1.253) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66929[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-co; C6502 Build/10.1.A.1.350) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66930[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyC6502 Build/10.1.A.1.350) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66931[] = "Mozilla/5.0 (Linux; Android 4.1.2; C525c Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S66932[] = "Mozilla/5.0 (Linux; Android 4.2.2; Xoom Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S66933[] = "Nokia6680/1.0 (5.04.40) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.9.1.120/27/352/UCWEB";
const char S66934[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; P-01D Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66935[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; P-01D Build/GRH78)";
const char S66936[] = "Mozilla/5.0 (Linux; Android 4.0.4; i9220 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66937[] = "Mozilla/5.0 (Linux; Android 4.0.3; i9220 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66938[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9220 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66939[] = "Mozilla/5.0 (Linux; Android 4.0.6; i9220 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66940[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9220 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66941[] = "Mozilla/5.0 (Linux; Android 4.0.4; I9220 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66942[] = "Mozilla/5.0 (Linux; Android 4.0.9; i9220 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66943[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 992D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S66944[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P7510 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S66945[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7510 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S66946[] = "Mozilla/5.0 (Windows NT 4.10; rv:20.0) Gecko/X Firefox/20.0";
const char S66947[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia500) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S66948[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S66949[] = "Nokia6060/2.0 (4.10) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S66950[] = "Nokia6060v/2.0 (3.03) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S66951[] = "Mozilla/5.0 (MOT-XT800/TITA_M2_20.08.0;U;Android/2.1-update1;480*854;CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S66952[] = "S8500 UCWEB6.0/UC Browser7.7.0.81";
const char S66953[] = "S8500 UCWEB6.0/UC Browser7.7.0.81/179/352";
const char S66954[] = "Opera/9.80 (Windows NT 5.0; U; ru) Presto/2.2.15 Version/10.00";
const char S66955[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.2.15 Version/10.01";
const char S66956[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC409 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66957[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC409 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66958[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s3653) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/405/UCWEB Mobile";
const char S66959[] = "Mozilla/5.0 (Symbian3; U; en-us; Nokia500) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/51/355/UCWEB Mobile";
const char S66960[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; TAD-97052 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66961[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; TAC-97052 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66962[] = "K-Touch_T619/X_8510_V0101 Mozilla/5.0 (Linux; U; Android 2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S66963[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; K-Touch T619 Build/MocorDroid2.3.5) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S66964[] = "Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; K-Touch T619 Build MocorDroid2.3.5) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S66965[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; K-Touch T619 Build/MocorDroid2.3.5)";
const char S66966[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S66967[] = "Mozilla/5.0 (compatible; Konqueror/4.2; Linux) KHTML/4.2.2 (like Gecko)";
const char S66968[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)";
const char S66969[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)";
const char S66970[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)";
const char S66971[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)";
const char S66972[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)";
const char S66973[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-in; Micromax P362 Build) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S66974[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; C6502 Build/10.3.A.0.423)";
const char S66975[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; C6502 Build/10.3.A.0.423) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66976[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; C6502 Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S66977[] = "Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 4.1; Alexa Toolbar)";
const char S66978[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 4.1; Alexa Toolbar)";
const char S66979[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 4.1; SV1)";
const char S66980[] = "Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 4.1; SV1)";
const char S66981[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 4.10; Trident/4.0)";
const char S66982[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 4.10; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)";
const char S66983[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 4.10; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; SE 2.x)";
const char S66984[] = "NSPlayer/10.0.0.3705 WMFSDK/10.0";
const char S66985[] = "NSPlayer/10.0.0.4005 WMFSDK/10.0";
const char S66986[] = "NSPlayer/10.0.0.4007 WMFSDK/10.0";
const char S66987[] = "NSPlayer/10.0.0.4362";
const char S66988[] = "NSPlayer/10.0.0.4054 WMFSDK/10.0";
const char S66989[] = "NSPlayer/10.0.0.4374 WMFSDK/10.0";
const char S66990[] = "NSPlayer/10.0.0.3703 WMFSDK/10.0";
const char S66991[] = "NSPlayer/10.0.0.3702 WMFSDK/10.0";
const char S66992[] = "NSPlayer/10.0.0.4078 WMFSDK/10.0 SE 2.X MetaSr 1.0";
const char S66993[] = "NSPlayer/10.0.0.3650";
const char S66994[] = "NSPlayer/10.0.0.3700 WMFSDK/10.0";
const char S66995[] = "NSPlayer/10.0.0.3700";
const char S66996[] = "NSPlayer/10.0.0.3802 WMFSDK/10.0 SE 2.X MetaSr 1.0";
const char S66997[] = "NSPlayer/10.0.0.4001";
const char S66998[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; EXLP2B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S66999[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; eGlide XL PRO II Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67000[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; eGlide XL PRO II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67001[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-TH; OPPOU7011 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67002[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U701T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67003[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; OPPOU701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67004[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th; U7015 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67005[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U701T Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S67006[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; OPPOU7015 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67007[] = "Mozilla/5.0 (Linux; Android 4.0.3; ThinkPad Tablet Build/ThinkPadTablet_A400_03) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S67008[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; DIT1010 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67009[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T510a Build/M) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67010[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T510a Build/M41882) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67011[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T510 Build/M41467) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67012[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T510 Build/M41703) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67013[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T510a Build/M41930) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67014[] = "MOT-EX430 MIDP-2.0/CLDC-1.1 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10";
const char S67015[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; PadFone 2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67016[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; PadFone 2 Build/IMM76I)";
const char S67017[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-sg; PadFone 2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67018[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-AS730 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67019[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67020[] = "Nokia7310/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7310";
const char S67021[] = "Nokia7310/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S67022[] = "Nokia7310_CMCC/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S67023[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.10.229 Version/11.62";
const char S67024[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.2.2.29837 (Firestorm-Release; firestorm skin) Safari/533.3";
const char S67025[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5110 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S67026[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P5110 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S67027[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5110 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S67028[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5302 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S67029[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S67030[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One SV Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S67031[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67032[] = "Mozilla/5.0 (Linux; Android 4.1.2; Micromax A116 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67033[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-us; MotoroiX Build/IUS) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola MotoroiX";
const char S67034[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; MotoroiX Build/IUS)";
const char S67035[] = "Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia2730c-1) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S67036[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S7562) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S67037[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; M-MP75S23G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67038[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT905 Build/7.7.1Q_GCIRD-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67039[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT907 Build/7.7.1Q-144_SMQ_S136-10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67040[] = "Mozilla/5.0 (Linux; Android 4.0.4; BNTV600 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S67041[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; Karbonn_A1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67042[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; Karbonn_A1 Build/FRF91)";
const char S67043[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; Karbonn_A1 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S67044[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S5830 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67045[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S5830 Build/IMM76L; CyanogenMod-CyanogenMod9 RC1 by MaclawStudios) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67046[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-au; MZ601 Build/JB_MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67047[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MZ601 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67048[] = "Mozilla/5.0 (Series40; Nokia205/03.18; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S67049[] = "Mozilla/5.0 (Series40; Nokia205.1/03.18; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S67050[] = "SonyEricssonM1i/R1BA Browser/Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S67051[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7/X (like Mobile/8C148 Safari/6533.18.5)";
const char S67052[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)";
const char S67053[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)";
const char S67054[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; MOT-XT305 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67055[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; XT305 Build/07.19.03R_D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67056[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; XT305 Build/07.18.06R_D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67057[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Explorer A310 Build/IMLK74; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67058[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Explorer A310e Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67059[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Explorer A310 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67060[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4";
const char S67061[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4";
const char S67062[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4";
const char S67063[] = "Mozilla/5.0 (Nintendo WiiU) AppleWebKit/534.52 (KHTML, like Gecko) NX/2.1.0.8.21 NintendoBrowser/1.0.0.7494.US";
const char S67064[] = "Mozilla/5.0 (Nintendo WiiU) AppleWebKit/534.52 (KHTML, like Gecko) NX/2.1.0.8.21 NintendoBrowser/1.0.0.7494.EU";
const char S67065[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S67066[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67067[] = "Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF700T Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S67068[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LS840 Build/ZVK.IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S67069[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I777 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S67070[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-i777 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S67071[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I777 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S67072[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LW770 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S67073[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-04D Build/9.1.C.0.475) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67074[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-04D Build/9.1.C.0.475) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67075[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SO-04D Build/9.1.C.0.475)";
const char S67076[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-04D Build/9.1.C.1.103) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67077[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-04D Build/9.1.C.1.103) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67078[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Diamond/1.37.172.3";
const char S67079[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Diamond/1.97.164.3";
const char S67080[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Alltel HTC Touch Diamond";
const char S67081[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_P3700/2.03.168.3";
const char S67082[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Diamond/1.37.163.4";
const char S67083[] = "HTC_P3700 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) HTC_P3700 Mozilla/4.0";
const char S67084[] = "HTC-P3700-orange/PPC; 480x640; OpVer 30.204.2.731 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S67085[] = "HTC_Diamond/V2.07.841.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S67086[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Diamond/1.97.161.4";
const char S67087[] = "HTC_Diamond Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S67088[] = "HTC-P3700-orange/PPC; 480x640; OpVer 30.137.2.732 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S67089[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Diamond/1.37.161.2";
const char S67090[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Diamond/1.97.163.4";
const char S67091[] = "HTC_P3700 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S67092[] = "HTC-P3700-orange/PPC; 480x640; OpVer 30.193.2.734 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S67093[] = "HTC-P3700-orange/PPC; 480x640; OpVer 30.126.2.732 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S67094[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Diamond/%ROM_VERSION%";
const char S67095[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Diamond/1.37.164.2";
const char S67096[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.7.12) Gecko/X Firefox/1.0.7 (Ubuntu package 1.0.7)";
const char S67097[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I905 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S67098[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE Blade III Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67099[] = "Mozilla/5.0 (Linux; Android 4.0.4; BLADE III Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67100[] = "Mozilla/5.0 (Linux; Android 4.0.4; Vip Racer III Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67101[] = "Mozilla/5.0 (Linux; Android 4.0.4; BLADE III_IL Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67102[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; SPNG9708 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67103[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; SPNG9708 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67104[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E455 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67105[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E455 Build/JZO54K)";
const char S67106[] = "Mozilla/5.0 (Linux; Android 4.1.2; Andromax U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67107[] = "Mozilla/5.0(iPad; U;CPU OS 7 like Mac OS X; zh-CN; iPad3,4) AppleWebKit/534.46 (KHTML, like Gecko) UCBrowser/2.0.1.280 U3/0.8.0 Safari/7543.48.3";
const char S67108[] = "Mozilla/5.0 (Linux; U; Android 1.0.3; de-de; A80KSC Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S67109[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SHW-M110S Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S67110[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SHW-M110S Build/ECLAIR)";
const char S67111[] = "Opera/9.80 (Windows NT 6.2; U; en) Presto/2.9.168 Version/11.52";
const char S67112[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One VX Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67113[] = "Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.2 Safari/535.19 Silk-Accelerated=true";
const char S67114[] = "Mozilla/5.0 (Windows NT 5.1; rv:17.0) Gecko/17.0 Firefox/17.0 IceDragon/17.0.1.0";
const char S67115[] = "Mozilla/5.0 (Linux; Android 4.1.2; A511 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S67116[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 101G10 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S67117[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E170K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67118[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E170K/KKJMC1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67119[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E170L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67120[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E170S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67121[] = "Mozilla/5.0 (Linux; Android 4.1.2; VS930 4G Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S67122[] = "Mozilla/5.0 (Linux; Android 4.1.2; VS930 4G Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S67123[] = "SAMSUNG-SGH-C270/C270JPIL1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S67124[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; U8300 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S67125[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; HuaweiU8300 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S67126[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; U8300 Build/ERE27)";
const char S67127[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 95; Alexa Toolbar)";
const char S67128[] = "Mozilla/4.0 (compatible; MSIE 6.01; Windows 95; Alexa Toolbar)";
const char S67129[] = "Mozilla/5.0 (compatible; MSIE 6.0; Win95; Trident/2.0)";
const char S67130[] = "Mozilla/5.0 (Linux; U; Android AOSP; en-us Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67131[] = "Mozilla/5.0 (Linux; U; Android AOSP; en-gb Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67132[] = "Mozilla/5.0 (Linux; U; Android AOSP; it-it Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67133[] = "Mozilla/5.0 (Linux; U; Android AOSP; pt-br Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67134[] = "Mozilla/5.0 (Linux; U; Android AOSP; nl-nl Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67135[] = "Mozilla/5.0 (Linux; U; Android AOSP; ru-ru Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67136[] = "Mozilla/5.0 (Linux; U; Android AOSP; es-es Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67137[] = "Mozilla/5.0 (Linux; U; Android AOSP; fr-fr Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67138[] = "Mozilla/5.0 (Linux; U; Android AOSP; de-de Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67139[] = "Mozilla/5.0 (Linux; U; Android AOSP; es-us Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67140[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; GT-I9100T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S67141[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S67142[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S67143[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S67144[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6802 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S67145[] = "Mozilla/5.0 (Linux; Android 4.0.3; Slider SL101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S67146[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800; 800 BOUYGUES)";
const char S67147[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800; Vodafone)";
const char S67148[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800; T-Mobile)";
const char S67149[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800; Orange)";
const char S67150[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800; SFR)";
const char S67151[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800) Microsoft.Consulting.Phone.SkySportsNews.App";
const char S67152[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800)";
const char S67153[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:7.0.1) Gecko/X Postbox/3.0.7";
const char S67154[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11 Comodo_Dragon/20.0.1.0";
const char S67155[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11 Comodo_Dragon/20.0.1.0";
const char S67156[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S67157[] = "Mozilla/5.0 (Linux; Android 4.1.1; PLT7223G Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S67158[] = "NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-02) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S67159[] = "LG-X330T Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S67160[] = "LG-X330 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S67161[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; MOT-XT882/SWDFS_M7_4.60.4; 540*960; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67162[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; XT882 Build/SWDFS_M7_4.60.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67163[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MOT-XT882/SWDFS_M7_4.80.0; 540*960; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67164[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT882 Build/SWDFS_M7_4.80.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67165[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT882 Build/SWDFS_M7_4.97.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S67166[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-XT882/SWDFS_M7_4.97.0; 540*960; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67167[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; XT882 Build/SWDFS_M7_4.60.4) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S67168[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT882 Build/SWDFS_M7_4.80.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S67169[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT882 Build/SWDFS_M7_4.97.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67170[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT882 Build/SWDFS_M7_4.97.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S67171[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT882 Build/SWDFS_M7_4.80.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S67172[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.14) Gecko/X Linux Mint/7 (Gloria) Firefox/3.0.17";
const char S67173[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.16) Gecko/X Linux Mint/7 (Gloria) Firefox/3.0.16";
const char S67174[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.16) Gecko/X Linux Mint/7 (Gloria) Firefox/3.0.15";
const char S67175[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.17) Gecko/X Linux Mint/7 (Gloria) Firefox/3.0.17";
const char S67176[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.12) Gecko/X Linux Mint/7 (Gloria) Firefox/3.0.12";
const char S67177[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.15) Gecko/X Linux Mint/7 (Gloria) Firefox/3.0.17";
const char S67178[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.0.10) Gecko/X Linux Mint/7 (Gloria) Firefox/3.0.10";
const char S67179[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.19) Gecko/X Linux Mint/7 (Gloria) Firefox/3.0.17 GTB7.1";
const char S67180[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.0.11) Gecko/X Linux Mint/7 (Gloria) Firefox/3.0.11";
const char S67181[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.13) Gecko/X Linux Mint/7 (Gloria) Firefox/3.0.17";
const char S67182[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.11) Gecko/X Linux Mint/7 (Gloria) Firefox/3.0.17";
const char S67183[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; TAB-PROTAB2XXL(4) Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67184[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; TAB-PROTAB2XXL Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67185[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-c6712) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S67186[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-c6712) UCBrowser8.3.1.161/69/405/UCWEB Mobile";
const char S67187[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; B79 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67188[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; B79 Build/IML74K)";
const char S67189[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I717 Build/JRO03L; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67190[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I717 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67191[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I717 Build/JRO03R; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67192[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I717 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67193[] = "Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S67194[] = "Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S67195[] = "Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/537.4 (KHTML like Gecko) Chrome/22.0.1229.79 Safari/537.4";
const char S67196[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 TeamStream (iPod4,1 313 2.4)";
const char S67197[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Safari/7534.48.3 TeamStream (iPod4,1 313 2.4)";
const char S67198[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 TeamStream (iPod4,1 379 2.7.2)";
const char S67199[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ617 Build/7.7.1-85_MZ617-27) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S67200[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P708g Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S67201[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P705 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S67202[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P705g Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S67203[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P705f Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S67204[] = "Mozilla/5.0 (Linux; Android 4.1.1; ME301T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S67205[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; HTC_A3335 V1.21.841.3 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S67206[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; HTC_A3335 V1.30.841.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S67207[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0) chromeframe/10.0.648.204";
const char S67208[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0) chromeframe/10.0.648.205";
const char S67209[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/10.0.648.204; SLCC1; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 1.1.4322; .NET CLR 3.0.30729; .NET CLR 3.5.30729; MS-RTC LM 8; .NET4.0C; .NET4.0E)";
const char S67210[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P5110 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67211[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67212[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GT-P5110 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67213[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; GT-P5110 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67214[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; GT-P5110 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67215[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-; GT-P5110 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67216[] = "NokiaX2-00/5.0 (04.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiax2-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S67217[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT27i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S67218[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6800 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S67219[] = "SonyEricssonK310iv/R4EA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S67220[] = "SonyEricssonK310i/R4EA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S67221[] = "Opera/9.60 (X11; Linux i686; U; Linux Mint; en) Presto/2.1.1";
const char S67222[] = "Opera/9.64 (X11; Linux i686; U; Linux Mint; nb) Presto/2.1.1";
const char S67223[] = "Opera/9.62 (X11; Linux i686; U; Linux Mint; en) Presto/2.1.1";
const char S67224[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1 x64; en-US; rv:1.9.3a1pre) Gecko/X Minefield/3.7a1pre";
const char S67225[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; ru; rv:1.9.3a4pre) Gecko/X Minefield/3.7a4pre";
const char S67226[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-F120L Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67227[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LW770 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S67228[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGPT12 Build/TID0142) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S67229[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGPT13 Build/TID0142) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S67230[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5110 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S67231[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P5110 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S67232[] = "Mozilla/5.0 (Linux; Android 4.2.2; One S Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S67233[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One S Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S67234[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-D710 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S67235[] = "Mozilla/5.0 (Linux; Android 4.2.1; SPH-D710 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S67236[] = "Mozilla/5.0 (Linux; Android 4.1.1; C1504 Build/11.3.A.0.47) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S67237[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-M830 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67238[] = "(DreamPassport/3.0; isao/MyDiGiRabi)";
const char S67239[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE71)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S67240[] = "NokiaE71/UCWEB8.5.0.163/28/800";
const char S67241[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE71x)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S67242[] = "Mozilla/5.0 (Java; U; en-us; nokiae71-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile";
const char S67243[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P1000 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S67244[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S67245[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P1000 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67246[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P1000N Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S67247[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67248[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P1000L Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S67249[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S67250[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000N Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S67251[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67252[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ARCHOS GAMEPAD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67253[] = "Mozilla/5.0 (Linux; Android 4.0.3; LT18i Build/4.1.A.0.562) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S67254[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT18i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S67255[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT18a Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S67256[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; i5350 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67257[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8010 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S67258[] = "Opera/9.70 (BREW 4.0.3; U; en; Samsung; Opera Mobi; Presto/2.2.1/INT) Sprint M850 SprintTitan/OSGi-4.1 Profile/MIDP-2.1 Configuration/CDC-1.1";
const char S67259[] = "NokiaN-GageQD/1.0 SymbianOS/6.1 Series60/1.2 Profile/MIDP-1.0 Configuration/CLDC-1.0/UC Browser7.9.1.120/26/351/UCWEB";
const char S67260[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; QQBrowser/7.0.4350.400)";
const char S67261[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; QQBrowser/7.0.4350.400)";
const char S67262[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; QQBrowser/7.0.4350.400)";
const char S67263[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QQBrowser/7.0.5173.400)";
const char S67264[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IRIS_501 Build/LAVAIRIS501) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67265[] = "Mozilla/5.0 (Linux; Android 4.0.3; Le Pan S Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S67266[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; GT-S7562 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67267[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-gb; GT-S7562 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67268[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; th-th; GT-S7562 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67269[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; pt-br; GT-S7562 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67270[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; fr-fr; GT-S7562 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67271[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; vi-vn; GT-S7562 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67272[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-au; GT-S7562 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67273[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; ar-eg; GT-S7562 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67274[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; es-es; GT-S7562 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67275[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; tr-tr; GT-S7562 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67276[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S67277[] = "Mozilla/5.0 (Linux; Android 4.1.1; CINK PEAX Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67278[] = "Mozilla/5.0 (Linux; Android 4.1.2; CINK PEAX 2 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67279[] = "SAMSUNG-C6625/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S67280[] = "Vodafone/1.0/SamsungC6625/BUIF2 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S67281[] = "Vodafone/1.0/SamsungC6625/BUID1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S67282[] = "Vodafone/1.0/SamsungC6625/BUIE1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S67283[] = "Vodafone/1.0/SamsungC6625/BUIH4 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S67284[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S67285[] = "Mozilla/5.0 (Windows NT 5.1; rv:15.0) Gecko/X Firefox/15.0 PaleMoon/15.0";
const char S67286[] = "Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:15.0) Gecko/X Firefox/15.0-x64 PaleMoon/15.0-x64";
const char S67287[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Le Pan S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67288[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Le Pan S Build/IML74K)";
const char S67289[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaN97 mini) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S67290[] = "NokiaN97 mini/UCWEB8.8.0.212/50/999";
const char S67291[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LT8088 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67292[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PLT8088 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67293[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 520)";
const char S67294[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 520; Vodafone)";
const char S67295[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 520; ANZ914)";
const char S67296[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 520; Orange)";
const char S67297[] = "Mozilla/5.0 (Linux; U; Android 1.0; en-us; dream)";
const char S67298[] = "Mozilla/5.0 (Linux; U; Android 1.0; en-us; dream) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S67299[] = "Mozilla/5.0 (Linux; U; Android 1.0; en-us; HTC_Dream) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S67300[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; K1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67301[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Ideapad K1 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67302[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; K1 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67303[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S67304[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105P Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S67305[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S67306[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105P Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S67307[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6506 Build/10.1.A.1.395) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S67308[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6506 Build/10.1.A.1.395) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S67309[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919N Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S67310[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaC6-00)/UC Browser8.2.0.132/50/355/UCWEB Mobile";
const char S67311[] = "NokiaC6-00/UCWEB8.2.0.116/50/800";
const char S67312[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaC6-00)/UC Browser8.5.0.183/50/355/UCWEB Mobile";
const char S67313[] = "NokiaC6-00/UCWEB8.5.0.163/50/999";
const char S67314[] = "Mozilla/5.0 (Java; U; en-us; nokiac6-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile  3gpp-gba";
const char S67315[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P970 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67316[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ThL W3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67317[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6800 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S67318[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S67319[] = "Mozilla/5.0 (Linux; Android 4.2.1; HTC Butterfly Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S67320[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SCH-M828C[X] Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67321[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; SCH-M828C Build/FROYO)";
const char S67322[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; SCH-M828C Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67323[] = "Mozilla/5.0 (Windows NT 5.0; rv:12.0) Gecko/X Firefox/12.0 SeaMonkey/2.9.1";
const char S67324[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SA3CNT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67325[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Legend Build/GRH55; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67326[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; pcdadr6350 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67327[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Incredible2_ADR6350 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67328[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6350 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67329[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; pcdadr6350 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67330[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6350 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67331[] = "Mozilla/5.0 (Linux; Android 4.0.3; SHV-E120L Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67332[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67333[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120K Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67334[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67335[] = "Mozilla/5.0 (Linux; Android 4.0.3; SHV-E120S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67336[] = "Mozilla/5.0 (Linux; Android 4.0.9; E120L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67337[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ607-14) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S67338[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; WALKMAN Build/icx1227_D101) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67339[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; WALKMAN Build/icx1227_E110) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67340[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; WALKMAN Build/icx1227_E111) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67341[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-N7100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S67342[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-N7100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S67343[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N7100 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S67344[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-N7105 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S67345[] = "Mozilla/5.0 (Windows NT 6.1; compatible; BDCbot/1.0; +http://ecommerce.bigdatacorp.com.br/faq.aspx) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1";
const char S67346[] = "Mozilla/5.0 (Windows NT 6.1; compatible; BDCbot/1.0; +http://www.bigdatacorp.com.br/bigweb/bdcbot.html) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1";
const char S67347[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S67348[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5306 Build/12.0.A.1.284) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67349[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; CRESTA.CTP888 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67350[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; CRESTA.CTP888 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67351[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS15SH Build/S6230) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67352[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS15SH Build/S7120) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67353[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS15SH Build/S7260) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67354[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS15SH Build/S2201) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67355[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Inspire 4G Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67356[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Inspire 4G Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67357[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Inspire 4G Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67358[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Inspire 4G Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67359[] = "Mozilla/5.0 (Linux; Android 4.1.1; Z120 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67360[] = "Mozilla/5.0 (Linux; Android 4.0.4; LGL86C Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S67361[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID7047(3G)-4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67362[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID7047 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67363[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:22.0) Gecko/X Firefox/22.0 SeaMonkey/2.19";
const char S67364[] = "LG-GX200 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S67365[] = "Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; SBM003SH Build/S0800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67366[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM003SH Build/S1400) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67367[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM003SH Build/S1810) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67368[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM003SH Build/S1900) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67369[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM003SH Build/S1510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67370[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM003SH Build/S1700) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67371[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM003SH Build/S1600) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67372[] = "Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; SBM003SH Build/S1100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67373[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM003SH Build/S1201) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67374[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SBM003SH Build/S1900)";
const char S67375[] = "Opera/9.80 (Windows NT 6.1; Opera Mobi/49; U; en) Presto/2.4.18 Version/10.00";
const char S67376[] = "Nokia2700c-2/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2700c-2/UC Browser8.0.3.107/69/444";
const char S67377[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6.1";
const char S67378[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Candy TV Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67379[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Candy TV  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67380[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Sensation Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S67381[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) 1Password/4.2.1 (like Version/11A4372q Mobile/7.0 Safari/8536.25)";
const char S67382[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) 1Password/4.2.2 (like Version/11A4414e Mobile/7.0 Safari/8536.25)";
const char S67383[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) 1Password/4.2.7 (like Version/11A465 Mobile/7.0 Safari/8536.25)";
const char S67384[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) 1Password/4.2.7 (like Version/11A501 Mobile/7.0.2 Safari/8536.25)";
const char S67385[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) 1Password/4.2.7 (like Version/11A470a Mobile/7.0.1 Safari/8536.25)";
const char S67386[] = "Nokia308/2.0 (05.85) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia308) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S67387[] = "Mozilla/5.0 (Linux; Android 4.1.2; ADR910L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67388[] = "SAMSUNG-SGH-T249/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S67389[] = "SAMSUNG-SGH-T249R/ NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S67390[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; D50 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67391[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D50 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67392[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC_T528w Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67393[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T528w Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S67394[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T528w Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67395[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC T528w Build/IMM76D)";
const char S67396[] = "Nokia300/5.0 (07.03) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia300) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S67397[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I757M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67398[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I757M Build/JZO54K)";
const char S67399[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; ALCATEL_ONE_TOUCH_6010X_Orange Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S67400[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ALCATEL ONE TOUCH 6010X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S67401[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S67402[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9200 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S67403[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-in; Micromax A210 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67404[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; A210 Build/JOP40D)";
const char S67405[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919N Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S67406[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919N Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S67407[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; de-de; ME302C Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67408[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; ME302C Build/JDQ39)";
const char S67409[] = "SAMSUNG-SGH-T746/T746BMJB3 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S67410[] = "Alcatel-OT-355A/1.0 ObigoInternetBrowser/Q03C";
const char S67411[] = "Alcatel-OT-355D/1.0 ObigoInternetBrowser/Q03C";
const char S67412[] = "Alcatel-OT-355/1.0 model-bouygues ObigoInternetBrowser/Q03C";
const char S67413[] = "Alcatel-OT-355/1.0 model-orange ObigoInternetBrowser/Q03C";
const char S67414[] = "Alcatel-OT-355/1.0 ObigoInternetBrowser/Q03C";
const char S67415[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; MZ605 Build/H.6.3-7) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S67416[] = "Mozilla/5.0 (Linux; U; Android 3.1; pt-br; MZ605 Build/H.6.3-28.6) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S67417[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; MZ605 Build/H.6.3-28) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S67418[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; MZ605 Build/H.6.3-25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S67419[] = "Mozilla/5.0 (Linux; U; Android 3.1; es-us; MZ605 Build/H.6.3-28.4) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S67420[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; MZ605 Build/H.6.3-23) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S67421[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-ca; MZ605 Build/H.6.3-16) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S67422[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; MZ605 Build/H.6.3-18) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S67423[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; MZ605 Build/H.6.3-21) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S67424[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; MZ605 Build/H.6.1-45) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S67425[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; MZ605 Build/H.6.3-29) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S67426[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; MZ605 Build/H.6.3-27) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S67427[] = "NokiaE71/UC Browser8.0.4.121/28/444/UCWEB";
const char S67428[] = "NokiaE71/UCWEB8.0.3.99/28/800";
const char S67429[] = "NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac1-01/UC Browser8.0.3.107/69/444";
const char S67430[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID1060 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67431[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT901 Build/7.7.1Q-144_SLS_S240-58) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67432[] = "Mozilla/5.0 (compatible; Konqueror/3.5) KHTML/3.5.4 (like Gecko)";
const char S67433[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.2.11) Gecko/X Thunderbird/3.1.5";
const char S67434[] = "SpiceM5600/MTK Release/01.01.2012 Browser/wap2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S67435[] = "Spice M5600";
const char S67436[] = "Spice M5600n";
const char S67437[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/6.5.28192/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S67438[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/6.5.28192/28.3234; U; en) Presto/2.8.119 Version/11.10";
const char S67439[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/6.5.28192/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S67440[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67441[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; A26 Build/CROSSCROSS_A26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67442[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T753 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67443[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; Maxtron T1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67444[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; CROSS A28 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67445[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; A7# Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67446[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; A22 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67447[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MAXTRON VENTUS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67448[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MOVIMAX P9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67449[] = "Mozilla/5.0 (Linux; Android 4.0.2; Galaxy Nexus Build/ICL53F) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S67450[] = "Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S67451[] = "Mozilla/5.0 (Linux; Android 4.0.1; Galaxy Nexus Build/ITL41F) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S67452[] = "Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S67453[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9A405 Safari/7534.48.3";
const char S67454[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9A334 Safari/7534.48.3";
const char S67455[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN95 8GB) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S67456[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN95) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S67457[] = "Nokia110/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia110) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S67458[] = "Mozilla/5.0 (Vodafone/1.0/LG-BL40/V10c Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S67459[] = "LG-BL40-Orange/V10b Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S67460[] = "LG/BL40g/V10a Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S67461[] = "LG-BL40f/v1.0 Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S67462[] = "Mozilla/5.0 (Vodafone/1.0/LG-BL40/V10b Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S67463[] = "LG-BL40/V10a Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S67464[] = "LG-BL40-V10d Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S67465[] = "LG/BL40/V10a Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S67466[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Droid Build/FRG83G; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67467[] = "Mozilla/5.0 (Series40; NokiaC3-01.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S67468[] = "Mozilla/5.0 (Series40; NokiaC3-01/06.00; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S67469[] = "Mozilla/5.0 (Series40; NokiaC3-01.5/07.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6";
const char S67470[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA6120c)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S67471[] = "Mozilla/5.0 (Java; U; en-us; nokia6120c) UCBrowser8.2.0.132/69/452/UCWEB Mobile";
const char S67472[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IT-M809RC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67473[] = "Mozilla/5.0 (X11; Linux x86_64; rv:10.0.12) Gecko/X conkeror/1.0pre (Debian-1.0~~pre+gitX-1)";
const char S67474[] = "Mozilla/5.0 (X11; Linux i686; rv:10.0.12) Gecko/X conkeror/1.0pre (Debian-1.0~~pre+gitX-1)";
const char S67475[] = "Mozilla/5.0 (X11; Linux x86_64; rv:22.0) Gecko/X Firefox/22.0 conkeror/1.0pre";
const char S67476[] = "Mozilla/5.0 (Linux; Android 4.0.3; SmartTabII10 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S67477[] = "Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S67478[] = "Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire2 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S67479[] = "Mozilla/5.0 (Linux; Android 4.2; Amazon Kindle Fire Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S67480[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P970 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67481[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P970h Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67482[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P970g Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67483[] = "Opera/9.63 (X11; FreeBSD 7.1-RELEASE i386; U; en) Presto/2.1.1";
const char S67484[] = "Opera/9.64 (X11; FreeBSD 7.4-RELEASE-p9 amd64; U; en) Presto/2.1.1";
const char S67485[] = "Mozilla/5.0 (Linux; Android 4.1.1; Iconia A500 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S67486[] = "Mozilla/5.0 (Linux; Android 4.1.1; A500 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S67487[] = "Mozilla/5.0 (Linux; Android 4.1.2; A500 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S67488[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; One V Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67489[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; One V Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67490[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One V Build/JZO54K; CyanogenMod-CM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67491[] = "Mozilla/5.0 (Linux; Android 4.1.1; A211 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S67492[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ThL W1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67493[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ThL W1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67494[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S67495[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200L Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S67496[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P6200 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S67497[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67498[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Butterfly Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S67499[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Butterfly Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S67500[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC X920e Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S67501[] = "Mozilla/5.0 (Linux; U; Android 2.3;en-us; ZTE V793 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67502[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; ZTE V793 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/ZTE-Android-MMS-V2.0";
const char S67503[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ZTE V793 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67504[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26w Build/6.1.A.2.45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S67505[] = "BlackBerry8100/4.2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100";
const char S67506[] = "Opera/9.25 (Windows ME; U; en)";
const char S67507[] = "Opera/9.20 (Windows ME; U; en)";
const char S67508[] = "Mozilla/5.0 (Linux; Android 4.1.1; A701 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S67509[] = "Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia2730c-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile";
const char S67510[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T699 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S67511[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; G2S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67512[] = "Mozilla/5.0 (Windows Phone 8.0; Trident/7.0; rv:11.0; IEMobile/11.0; ARM; Touch; NOKIA; Nokia) like Gecko";
const char S67513[] = "HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; en)";
const char S67514[] = "PPC; 480x800; HTC_Touch_Diamond2_T5353-Orange; OpVer 93.172.2.732 Opera/9.5 (Microsoft Windows 6.1; Opera Mobi/16071; U; fr)";
const char S67515[] = "HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; zh-cn)";
const char S67516[] = "HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; zh-tw)";
const char S67517[] = "HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; pt)";
const char S67518[] = "Vodafone/1.0/HTC_Touch_Diamond2 Opera/9.50 (Windows NT 5.1; U; de)";
const char S67519[] = "HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; sv)";
const char S67520[] = "HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; nl)";
const char S67521[] = "HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; ru)";
const char S67522[] = "HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; es-ES)";
const char S67523[] = "Vodafone/1.0/HTC_Touch_Diamond2 Opera/9.50 (Windows NT 5.1; U; nl)";
const char S67524[] = "HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; nb)";
const char S67525[] = "HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; de)";
const char S67526[] = "HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; fr)";
const char S67527[] = "Vodafone/1.0/HTC_Touch_Diamond2/2.16.164.1 (67041) Opera/9.50 (Windows NT 5.1; U; es-ES)";
const char S67528[] = "HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; el)";
const char S67529[] = "Nokia6350-1d/2.0(ATT.5.93) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S67530[] = "Nokia6350/2.0(ATT.4.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S67531[] = "Liferea/0.x.x (Linux; en_US.UTF-8; http://liferea.sf.net/)";
const char S67532[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:2.0pre) Gecko/X Firefox/4.0pre";
const char S67533[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X401 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67534[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; X401 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67535[] = "ALCATEL_ONE_TOUCH_838/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1ObigoInternetBrowser/Q05A";
const char S67536[] = "Mozilla/5.0 (Symbian3; U; en-us; Nokia603) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile";
const char S67537[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE52-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S67538[] = "NokiaE52-1UCBrowser/8.7.1.234/28/444/UCWEB";
const char S67539[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiae52-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba";
const char S67540[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ZTE V790 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67541[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E973 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S67542[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F200K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S67543[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F200S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S67544[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E973 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S67545[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F200L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S67546[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SonyC1605 Build/11.3.A.2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67547[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; C1605 Build/11.3.A.2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67548[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; C1605 Build/11.3.A.2.1)";
const char S67549[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; New Andromax-i Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67550[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; New Andromax-i Build/JZO54K)";
const char S67551[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; LG-E510 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67552[] = "Mozilla/5.0 (Linux; Android 4.2.1; SAMSUNG-SGH-T989 Build/JOP40G) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67553[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T989 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67554[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-T989 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67555[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-T989 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67556[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.21 (KHTML, like Gecko) QupZilla/1.4.3 Safari/537.21";
const char S67557[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.3; WOW64; Trident/7.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3)";
const char S67558[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.3; Win64; x64; Trident/7.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3)";
const char S67559[] = "SAMSUNG-S5200/S5200DDIK3 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S67560[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Glacier Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67561[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Glacier Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67562[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Glacier Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67563[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Glacier Build/MIUI demonspeed!) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67564[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; demonSPEED! Glacier Build/MIUI dS 1.7.15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67565[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Glacier Build/MIUIr1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67566[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; demonSPEED! Glacier Build/MIUI 1.6.17 dS v1.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67567[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; demonSPEED! Glacier Build/MIUI 1.6.24 dS v1.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67568[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Glacier Build/MIUI dS_MOD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67569[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaN73)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S67570[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; IMO S88 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67571[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Amaze 4G Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67572[] = "Mozilla/5.0 (Linux; Android 4.1.1; Micromax A110 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S67573[] = "Mozilla/5.0 (Linux; Android 4.1.2; A110 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S67574[] = "Mozilla/5.0 (Windows NT 4.10; rv:23.0) Gecko/X Firefox/23.0";
const char S67575[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; id-id; T1C Build/I700T_P4_11A_AC73_FTN_R_0_03_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67576[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; id-id; T1C Build/I700T_P6_20_AC73_FTN_R_0_01_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67577[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; id-id; T1C Build/I700T_P6_T02_AC73_FT_H44_R_0_01_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67578[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; id-id; T1C Build/I700T_P6_20_AC73_FT_S42_R_0_01_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67579[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-uk; GEM10212 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67580[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AT330 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67581[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; ZXY-ZTE_V6700 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67582[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5302) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S67583[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26ii Build/6.1.A.2.55) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S67584[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC5-05) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S67585[] = "Mozilla/5.0 (Linux; Android 4.1.2; VS930 4G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S67586[] = "Mozilla/5.0 (Linux; Android 4.1.2; VS930 4G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S67587[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ617 Build/7.7.1-85_MZ617-27) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S67588[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6102) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S67589[] = "Mozilla/5.0 (Linux; Android 4.2; Nexus 7 Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S67590[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S67591[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 7 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S67592[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Vivo Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67593[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; HTC-Vivo Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67594[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; HTC_Vivo Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67595[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; XT862 Build/5.5.1_84_D3G-55) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67596[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; XT862 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67597[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT862 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67598[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT862 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67599[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67600[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67601[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; LGL21 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67602[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S67603[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67604[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:21.0) Gecko/X Firefox/21.0 slurp";
const char S67605[] = "Mozilla/5.0 (Windows NT 6.1; rv:21.0) Gecko/X Firefox/21.0 slurp";
const char S67606[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:21.0) Gecko/X Firefox/21.0 slurp slurp";
const char S67607[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHW-M500W Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67608[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8010 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S67609[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ALCATEL ONE TOUCH 8008D Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2 Mobile Safari/534.30";
const char S67610[] = "LGE-VX5600/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S67611[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; Edition MacAppStore; en) Presto/2.9.168 Version/11.51";
const char S67612[] = "Mozilla/5.0 (Linux; Android 4.0.4; SOL21 Build/9.0.F.2.128) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67613[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Archos 50 Platinum Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67614[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Archos 50 Platinum Build/JZO54K)";
const char S67615[] = "Cricket-A410/1.0 Polaris/v6.17";
const char S67616[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; novo7_Advanced2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67617[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; novo7_Advanced2 Build/IML74K)";
const char S67618[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T679 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67619[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T679 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67620[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T679 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67621[] = "Nokia7230/5.0 (10.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7230) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S67622[] = "Mozilla/5.0 (Linux; Android 4.1.2; A510 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S67623[] = "Mozilla/3.01 (compatible; AmigaVoyager/2.95; AmigaOS/MC680x0)";
const char S67624[] = "Mozilla/5.0 (Symbian3; U; en-us; NokiaN8-00)/UC Browser8.5.0.183/51/355/UCWEB Mobile";
const char S67625[] = "NokiaN8-00/UCWEB8.5.0.163/51/800";
const char S67626[] = "Mozilla/5.0 (Java; U; fa; nokian8-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile  3gpp-gba";
const char S67627[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.31 (KHTML, like Gecko) Ubuntu Chromium/26.0.1410.63 Chrome/26.0.1410.63 Safari/537.31";
const char S67628[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.31 (KHTML, like Gecko) Ubuntu Chromium/26.0.1410.63 Chrome/26.0.1410.63 Safari/537.31";
const char S67629[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amazon Kindle Fire HD Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67630[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amazon Kindle Fire HD Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67631[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/532.9 (KHTML, like Gecko) Iron/4.0.280.0 Chrome/4.0.280.0 Safari/532.9";
const char S67632[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.11 (KHTML, like Gecko) Ubuntu/12.10 Chromium/23.0.1271.64 Chrome/23.0.1271.64 Safari/537.11";
const char S67633[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Ubuntu/12.10 Chromium/23.0.1271.64 Chrome/23.0.1271.64 Safari/537.11";
const char S67634[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/23.0.1271.95 Chrome/23.0.1271.95 Safari/537.6+ Ubuntu/12.10 (3.6.1-0ubuntu1) Epiphany/3.6.1";
const char S67635[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; N320 Build/LAVALAVA_N320) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67636[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S67637[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830M) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S67638[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; GT-S5830L) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S67639[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; sr-rs; ZTE Kis Lite Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67640[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGP321 Build/10.3.1.A.0.244)";
const char S67641[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SGP321 Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67642[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SonySGP321 Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67643[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9A405 Safari/7534.48.3";
const char S67644[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9A334 Safari/7534.48.3";
const char S67645[] = "Mozilla/4.1 (U; BREW 3.1.5; en-US; Teleca/Q05A/INT/SAM-M800)";
const char S67646[] = "Mozilla/4.1 (U; BREW 3.1.5; en-US; Teleca/Q05A/INT)";
const char S67647[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Comodo_Dragon/8.0.0.4 Chrome/8.0.552.215 Safari/534.10";
const char S67648[] = "NokiaX6-00/UC Browser8.0.4.121/50/444/UCWEB";
const char S67649[] = "NokiaX6-00/UCWEB8.0.3.99/50/800";
const char S67650[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Desire Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67651[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Desire Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67652[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Desire CDMA Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67653[] = "IUC(U;iOS 6.0.1;Zh-cn;320*480;)/UCWEB8.9.0.238/44/997";
const char S67654[] = "IUC(U;iOS 6.0.2;Zh-cn;320*568;)/UCWEB8.9.0.238/44/997";
const char S67655[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT912 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S67656[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMID901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67657[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8160 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67658[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8160P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67659[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8160L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67660[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT918 Build/2_26D_2009) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67661[] = "Mozilla/5.0 (Linux; Android 4.1.2; CINK FIVE Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67662[] = "Mozilla/5.0 (Linux; Android 4.1.1; CINK+ Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67663[] = "BlackBerry8800/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102";
const char S67664[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7D11 Twitter for iPhone";
const char S67665[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Twitter for iPhone";
const char S67666[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7C144 Twitter for iPhone";
const char S67667[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:18.0) Gecko/X Firefox/18.0 SeaMonkey/2.15.2";
const char S67668[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Woxter Tablet PC 101IPS DUAL Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67669[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Woxter Tablet PC 101IPS DUAL Build/JRO03H)";
const char S67670[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Woxter Tablet PC 101 IPS DUAL Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67671[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710e Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S67672[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S67673[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Comodo_Dragon/10.0.0.2 Chrome/10.0.648.204 Safari/534.16";
const char S67674[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; N12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67675[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N12 Build/N12_Android 4.0.4_X fwd.test-keys) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67676[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; N12 Build/IML74K)";
const char S67677[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N12 Build/rk29sdk_Android 4.0.4_X.X fwd.test-keys) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67678[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Inspire 4G Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67679[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Inspire 4G Build/JRO03R; CyanogenMod-JellyTime_R15) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67680[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Inspire 4G Build/JZO54K; CyanogenMod-JellyTime_R30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67681[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Inspire 4G Build/JRO03L; CyanogenMod-JellyTime_R10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67682[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GoTab GBT9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67683[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S67684[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S67685[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S67686[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S67687[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S67688[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S67689[] = "Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire2 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S67690[] = "Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S67691[] = "Mozilla/5.0 (Linux; Android 4.2; Amazon Kindle Fire Build/JOP40C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S67692[] = "Mozilla/5.0 (Linux; Android 4.2.1; Amazon Kindle Fire Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S67693[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26ii Build/6.1.A.2.55) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S67694[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC7-00) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S67695[] = "Mozilla/5.0 (Linux; Android 4.1.1; MT7001 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S67696[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5 HTC_Touch2/1.0)";
const char S67697[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch2_T3333; Windows Phone 6.5)";
const char S67698[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Mega; Windows Phone 6.5)";
const char S67699[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch3G_T3333; Windows Phone 6.5)";
const char S67700[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch2/1.0; Windows Phone 6.5)";
const char S67701[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch2_T3333; Windows Phone 6.5.3.5)";
const char S67702[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SH530U Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67703[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SH530U Build/IMM76D)";
const char S67704[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; LOOX Plus Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67705[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; LOOX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67706[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A7+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67707[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VL-110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67708[] = "OneBrowser/3.1 (NokiaX2-01/08.63)";
const char S67709[] = "Mozilla/5.0 (Linux; Android 4.2.2; SM-T310 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S67710[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-KU3700 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67711[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; LG-KU3700 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67712[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-KU3700 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67713[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; LG-KU3700 Build/FRF91)";
const char S67714[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Iron/12.0.750.0 Chrome/12.0.750.0 Safari/534.30";
const char S67715[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.30 (KHTML, like Gecko) Iron/12.0.750.0 Chrome/12.0.750.0 Safari/534.30";
const char S67716[] = "Safari/7534.57.2 CFNetwork/520.4.3 Darwin/11.4.0 (i386) (MacPro1%2C1)";
const char S67717[] = "Safari/7534.57.2 CFNetwork/520.4.3 Darwin/11.4.0 (x86_64) (Macmini5%2C1)";
const char S67718[] = "Safari/7534.57.2 CFNetwork/520.4.3 Darwin/11.4.0 (x86_64) (MacBook6%2C1)";
const char S67719[] = "Safari/7534.57.7 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)";
const char S67720[] = "Safari/7534.57.7 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookAir4%2C1)";
const char S67721[] = "Safari/7534.57.7 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (iMac12%2C1)";
const char S67722[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; V9 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67723[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T959 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67724[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T959 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67725[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; SGH-T959 Build/JRO03L)";
const char S67726[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T959 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67727[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T959 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67728[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T959 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67729[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T959 Build/JZO54K)";
const char S67730[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ETAB 801 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67731[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SANSUI ETAB 801VP Build/ICS.f08refem86.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67732[] = "JUC (Linux; U; 2.3.4; zh-cn; ST15i; 320*480) UCWEB7.9.0.94/139/352";
const char S67733[] = "JUC (Linux; U; 4.0.4; zh-cn; ST15i; 320*480) UCWEB7.9.0.94/139/352";
const char S67734[] = "SAMSUNG-GT-C3262/C3262DDLHA Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S67735[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6102) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S67736[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-S7572 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67737[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S7572 Build/JZO54K)";
const char S67738[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-S7572 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S67739[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP8010 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S67740[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax A110 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S67741[] = "Mozilla/5.0 (SymbianOS/9.2; U; [en]; Series60/3.2 Nokia3250/1.00; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67742[] = "Mozilla/5.0 (SymbianOS/9.2; Series60/3.2 Nokia3250/1.00; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.1 Mobile Safari/533.4 3gpp-gba";
const char S67743[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Nokia3250/1.00; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525";
const char S67744[] = "Nokia3250/2.0 (04.60) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S67745[] = "Nokia3250";
const char S67746[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_Wildfire_S_A510e; 320*480) UCWEB7.9.3.103/139/800";
const char S67747[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_Wildfire_S_A510e; 320*480) UCWEB7.9.1.100/139/800";
const char S67748[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_A510e; 320*480) UCWEB7.9.3.103/139/800";
const char S67749[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_Wildfire_S_A510e; 320*480) UCWEB7.9.3.103/139/999";
const char S67750[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_Wildfire_S_A510e; 320*480) UCWEB7.9.0.94/139/352";
const char S67751[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_Wildfire_S_A510e; 320*480) UCWEB7.9.0.94/139/352";
const char S67752[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_Wildfire_S_A510e; 320*480) UCWEB7.9.0.94/139/444";
const char S67753[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_Wildfire_S_A510e; 320*480) UCWEB7.9.0.94/139/355";
const char S67754[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_Wildfire_S_A510e; 320*480) UCWEB7.9.4.145/139/800";
const char S67755[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_Wildfire_S_A510e; 320*480) UCWEB7.9.0.94/139/444";
const char S67756[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_A510e; 320*480) UCWEB7.9.4.145/139/800";
const char S67757[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_Wildfire_S_A510e; 320*480) UCWEB7.9.0.94/139/351";
const char S67758[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_Wildfire_S_A510e; 320*480) UCWEB7.9.0.94/140/444";
const char S67759[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S67760[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S67761[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S67762[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; A100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67763[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Mi-A100 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67764[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A100 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67765[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; RoverPad Air A100 Build/MID) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67766[] = "Opera/9.80 (Android 2.1-update1; Linux; Opera Mobi/ADR-1107051709; U; en) Presto/2.8.149 Version/11.10";
const char S67767[] = "BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 32054; Lang: zhcn) ucweb-squid";
const char S67768[] = "BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 600; Lang: zhcn) ucweb-squid";
const char S67769[] = "BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 90197; Lang: zhcn) ucweb-squid";
const char S67770[] = "BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 40105; Lang: zhcn) ucweb-squid";
const char S67771[] = "BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 32060; Lang: zhcn) ucweb-squid";
const char S67772[] = "BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 40118; Lang: zhcn) ucweb-squid";
const char S67773[] = "BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 40065; Lang: zhcn) ucweb-squid";
const char S67774[] = "BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 90242; Lang: zhcn) ucweb-squid";
const char S67775[] = "BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 32065; Lang: zhcn) ucweb-squid";
const char S67776[] = "BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 90199; Lang: zhcn) ucweb-squid";
const char S67777[] = "BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 1199; Lang: zhcn) ucweb-squid";
const char S67778[] = "BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 90243; Lang: zhcn) ucweb-squid";
const char S67779[] = "BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 40155; Lang: zhcn) ucweb-squid";
const char S67780[] = "BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: Xe; Lang: hi; Profile/MIDP-2.0_Configuration/CLDC-1.1) ucweb-squid";
const char S67781[] = "BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 2520; Lang: zhcn) ucweb-squid";
const char S67782[] = "BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 40175; Lang: zhcn) ucweb-squid";
const char S67783[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US; rv:1.8.1.13) Gecko/X SeaMonkey/1.1.9";
const char S67784[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.8.1.21) Gecko/X SeaMonkey/1.1.15";
const char S67785[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; XT532 Build/V4.37B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67786[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT532 Build/V4.36A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67787[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT532 Build/V4.36C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67788[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; XT532 Build/V4.37C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67789[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT532 Build/V4.36D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67790[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7560M Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67791[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MN-1000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67792[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; MNB1000S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67793[] = "Mozilla/5.0 (Linux; Android 4.2.2; Transformer TF101 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S67794[] = "Mozilla/5.0 (Linux; Android 4.2.1; Transformer Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S67795[] = "Mozilla/5.0 (Linux; Android 4.2.2; Transformer Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S67796[] = "Mozilla/5.0 (Linux; Android 4.2.1; Transformer TF101 Build/JB_MR1.2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S67797[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Enjoy 71 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67798[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; N9810 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67799[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; N9810 Build/JZO54K)";
const char S67800[] = "Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300TG Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S67801[] = "Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300T Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S67802[] = "Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300TL Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S67803[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One VX Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S67804[] = "Reeder/3.2 CFNetwork/672.0.2 Darwin/14.0.0";
const char S67805[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S67806[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S67807[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; TECNO_N9 Build/TECNO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67808[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; TECNO_N9 Build/TECNO)";
const char S67809[] = "Mozilla/5.0 (Linux; Android 4.2.2; ME302C Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S67810[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Iron/10.0.650.0 Chrome/10.0.650.0 Safari/534.16";
const char S67811[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MEIZU MX Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67812[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MEIZU MX Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S67813[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-; MEIZU MX Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67814[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; MEIZU MX Build/GRJ90)";
const char S67815[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MEIZU MX Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S67816[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) iCab/5.0 Safari/533.16";
const char S67817[] = "Mac OS X/10.8.2 (12C60); ExchangeWebServices/3.0 (157); Mail/6.2 (1499)";
const char S67818[] = "Mac OS X/10.8.2 (12C3006); ExchangeWebServices/3.0 (157); Mail/6.2 (1499)";
const char S67819[] = "Mac OS X/10.8.2 (12C2034); ExchangeWebServices/3.0 (157); Mail/6.2 (1499)";
const char S67820[] = "Mac OS X/10.8.2 (12C3012); ExchangeWebServices/3.0 (157); Mail/6.2 (1499)";
const char S67821[] = "Mac OS X/10.8.2 (12C3103); ExchangeWebServices/3.0 (157); Mail/6.2 (1499)";
const char S67822[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S67823[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S67824[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One 801e Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S67825[] = "Mozilla/5.0 (Linux; Android 4.2.2; Transformer Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S67826[] = "Mozilla/5.0 (Linux; Android 4.2.2; Transformer TF101 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S67827[] = "Mozilla/5.0 (Linux; Android 4.2.1; Transformer Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S67828[] = "Mozilla/5.0 (Linux; Android 4.0.3; Le Pan S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S67829[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LS840 Build/ZVK.IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67830[] = "SonyEricssonW705a/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3";
const char S67831[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; A853 Build/SHLA_U2_03.05.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S67832[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A853 Build/SHLA_U2_03.06.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S67833[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A853 Build/SUSKT_U1_00.10.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S67834[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; A853 Build/SHLA_U2_03.06.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S67835[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; A853 Build/SHLA_U2_03.05.0)";
const char S67836[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-kr; A853 Build/SUSKT_U1_00.14.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S67837[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; sr-rs; Telenor_OneTouch Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67838[] = "LGE-AN270/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S67839[] = "Mozilla/5.0 (Linux; U; Android 2.3.9; en-us; Celkon A 85 Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67840[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MEDION Smartphone LIFE E3501 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67841[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; MEDION Smartphone LIFE E3501 Build/GRJ34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67842[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaE66) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S67843[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; MediaPad 10 FHD Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67844[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.1.8) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S67845[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.0.16) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S67846[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.4.27) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S67847[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.6.8) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S67848[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.4.22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S67849[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.6.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S67850[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S67851[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S67852[] = "JUC (Linux; U; 2.3.6; zh-cn; GT-I9003; 480*800) UCWEB7.9.0.94/139/444";
const char S67853[] = "JUC (Linux; U; 2.2.1; zh-cn; GT-I9003; 480*800) UCWEB7.9.0.94/139/444";
const char S67854[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; Penta IS701C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67855[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; Etisalat Smartphone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67856[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I8150 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67857[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I8150 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67858[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I8150 Build/Touchwi5; CyanogenMod-Touchwi5v2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67859[] = "Mozilla/5.0 (Linux; Android 4.3; SCH-I545 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67860[] = "HD2_T8585 Opera/9.7 (Windows NT 5.1; U; zh-tw)";
const char S67861[] = "HD2_T8585 Opera/9.7 (Windows NT 5.1; U; en)";
const char S67862[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-I9100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67863[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/MIUI 1.9.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67864[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/MIUI 1.9.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67865[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; GT-I9100 Build/MIUI)";
const char S67866[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/MIUI 1.7.29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67867[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/MIUI 1.9.16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67868[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/MIUI 1.8.26) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67869[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-I9100 Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S67870[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.8.1.21) Gecko/X SeaMonkey/1.1.15";
const char S67871[] = "Mozilla/5.0 (Linux; Android 4.1.1; Desire HD Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67872[] = "Mozilla/5.0 (Linux; Android 4.1.2; Desire HD Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67873[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus S 4G Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67874[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus S 4G Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67875[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus S 4G Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67876[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus S 4G Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67877[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; MegaFon SP-AI Build/1.2.10.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67878[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; MegaFon_SP-AI Build/1.2.10.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67879[] = "Mozilla/5.0 (rv:1.3; BREW 3.1.5; en)/400x240 sam-r860 like Gecko/20080531 NetFront/3.5";
const char S67880[] = "NetFront/3.5.1 (BREW 5.0.2.1; U; en-us; Samsung ; NetFront/3.5.1/WAP) Sprint M360 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S67881[] = "portalmmm/2.0 S500i(c20;TB)";
const char S67882[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/28/351/UCWEB";
const char S67883[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/50/351/UCWEB";
const char S67884[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/50/352/UCWEB";
const char S67885[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/28/352/UCWEB";
const char S67886[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/28/355/UCWEB";
const char S67887[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/50/355/UCWEB";
const char S67888[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/50/444/UCWEB";
const char S67889[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/50/405/UCWEB";
const char S67890[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/28/444/UCWEB";
const char S67891[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/28/354/UCWEB";
const char S67892[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/50/354/UCWEB";
const char S67893[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/28/405/UCWEB";
const char S67894[] = "SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonj108i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S67895[] = "AppleCoreMedia/1.0.0.10B144 (iPod; U; CPU OS 6_1 like Mac OS X; en_us)";
const char S67896[] = "AppleCoreMedia/1.0.0.10B146 (iPod; U; CPU OS 6_1_2 like Mac OS X; en_us)";
const char S67897[] = "AppleCoreMedia/1.0.0.10B329 (iPod; U; CPU OS 6_1_3 like Mac OS X; en_us)";
const char S67898[] = "Mozilla/5.0 (Linux; Android 4.1.1; TBDC1093 Build/X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S67899[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:22.0) Gecko/X Firefox/22.0 slurp";
const char S67900[] = "Mozilla/5.0 (Windows NT 6.1; rv:22.0) Gecko/X Firefox/22.0 slurp";
const char S67901[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:22.0) Gecko/X Firefox/22.0 slurp slurp";
const char S67902[] = "Mozilla/5.0 (Linux; Android 4.0.4; KFJWI Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S67903[] = "Mozilla/5.0 (Linux; Android 4.2.1; A210 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S67904[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XXHHa; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67905[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/DJHHb; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67906[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XXHH7; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67907[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/DJHJ4; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67908[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XEHI5; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67909[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/OXXHG3 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67910[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67911[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/DJHI4; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67912[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XXHH6 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67913[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XXHJ4; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67914[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XEHJ2; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67915[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XXHH9; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67916[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/DJHJ1; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67917[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XXHK1; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67918[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/CBHK1; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67919[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XXHG5; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67920[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/DJHK2; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67921[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XEHJ4; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67922[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XXHL2; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67923[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XEHL2; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67924[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XXHJ5; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67925[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/DJHI3; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67926[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 SAMSUNG-GT-I8510C/1.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67927[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XEHI2; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67928[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/DJHI1; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67929[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510L/UBHL3 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67930[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/DJHI2; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67931[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/DJHL1; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67932[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XXHJ3; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67933[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/DJHK4; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S67934[] = "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.2.15 Version/10.20";
const char S67935[] = "Opera/10.2 (Windows NT 5.1; WOW64; U; en)";
const char S67936[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 4 Build/JOP40D; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67937[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67938[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/mako";
const char S67939[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT29i Build/7.0.A.3.212) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S67940[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-S735C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67941[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; ld-us; AN8G2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67942[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; fr-fr; AN8G2DT Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67943[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.10.289 Version/12.00";
const char S67944[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.10.289 Version/12.01";
const char S67945[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A111 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67946[] = "Mozilla/5.0 (Linux; Android 4.0.3; Slider SL101 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S67947[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ThL W7 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67948[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; ThL W7 Build/JOP40D)";
const char S67949[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Spice Mi-502 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S67950[] = "Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 101G9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S67951[] = "Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 101G9 Build/MR1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S67952[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S67953[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9000 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S67954[] = "Nokia2320c/2.0 (08.20) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S67955[] = "Nokia2320c-2/2.0 (08.20) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S67956[] = "Nokia2320c_CMCC/2.0 (09.73) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S67957[] = "Nokia2320c-2b/2.0(ATT.11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S67958[] = "SAMSUNG-SGH-U800/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S67959[] = "SAMSUNG-SGH-U800E/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S67960[] = "Mozilla/5.0 (compatible; Teleca Q7; Brew 3.1.5; U; en) 240X400 LGE VX9700";
const char S67961[] = "JUC(Linux;U;Android2.2.1;Zh_cn;GT-S5570;240*320;)UCWEB7.8.0.95/140/444";
const char S67962[] = "Mozilla/5.0(Linux;U;Android2.2.1;Zh_cn;GT-S5570;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/140/444";
const char S67963[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; LG-C710h Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S67964[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; LG-C710h Build/GRI40) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S67965[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 710; T-Mobile)";
const char S67966[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 710)";
const char S67967[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 710; Vodafone)";
const char S67968[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 710; Orange)";
const char S67969[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67970[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S67971[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC J Z321e Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67972[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919V Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S67973[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.2 Chrome/15.0.900.2 Safari/535.2";
const char S67974[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.2 Chrome/15.0.900.2 Safari/535.2";
const char S67975[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; A73 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67976[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Micromax A73 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S67977[] = "Mozilla/5.0 (Windows NT 4.10) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S67978[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R760 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S67979[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:19.0) Gecko/X Firefox/19.0 SeaMonkey/2.16.2";
const char S67980[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9300 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S67981[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-I9300 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S67982[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-I939 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S67983[] = "Nokia110/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia110) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S67984[] = "User-Agent: Mozilla/5.0 (Ubuntu; X11; Linux x86_64; rv:8.0) Gecko/X Firefox/8.0";
const char S67985[] = "Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:8.0) Gecko/X Firefox/8.0 Firefox/8.0";
const char S67986[] = "Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:8.0) Gecko/X Firefox/8.0,gzip(gfe) (via translate.google.com)";
const char S67987[] = "Mozilla/5.0 (Ubuntu; X11; Linux x86_64; rv:8.0) Gecko/X Firefox/8.0,gzip(gfe) (via translate.google.com)";
const char S67988[] = "Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:8.0) Gecko/X Firefox/8.0 FirePHP/0.6";
const char S67989[] = "Mozilla/5.0 (Ubuntu; X11; Linux x86_64; rv:8.0) Gecko/X Firefox/8.0 FirePHP/0.6";
const char S67990[] = "Mozilla/5.0 (Ubuntu; X11; Linux x86_64; rv:8.0) Gecko/X Firefox/8.0 Firefox/8.0";
const char S67991[] = "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:8.0) Gecko/X Firefox/8.0";
const char S67992[] = "Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:8.0) Gecko/X Firefox/8.0,gzip(gfe)";
const char S67993[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; Microsoft; 0)";
const char S67994[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; Juggernaut; 0)";
const char S67995[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; NOKIA; )";
const char S67996[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0)";
const char S67997[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia)";
const char S67998[] = "Mozilla/5.0 (Windows NT 5.1; rv:14.0) Gecko/X Thunderbird/14.0";
const char S67999[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:14.0) Gecko/X Thunderbird/14.0";
const char S68000[] = "Mozilla/5.0 (X11; Linux i686; rv:10.0.12) Gecko/X Icedove/10.0.12";
const char S68001[] = "Mozilla/5.0 (X11; Linux x86_64; rv:10.0.12) Gecko/X Icedove/10.0.12 Lightning/1.2b1";
const char S68002[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; T-07B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68003[] = "MQQBrowser/38 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3";
const char S68004[] = "MQQBrowser/38 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3";
const char S68005[] = "MQQBrowser/381 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3";
const char S68006[] = "MQQBrowser/381 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3";
const char S68007[] = "MQQBrowser/381 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3";
const char S68008[] = "MQQBrowser/38 Mozilla/5.0 (iPhone 4; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3";
const char S68009[] = "Mozilla/5.0 (compatible; MSIE 10.0; PPC; Tablet PC; Trident/4.0)";
const char S68010[] = "Mozilla/5.0 (compatible; MSIE 10.0; WindowsCE; Trident/5.0)";
const char S68011[] = "Mozilla/5.0 (compatible; MSIE 10.0; 68K; Tablet PC; Trident/4.0)";
const char S68012[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9070 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68013[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9070P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68014[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9B206 Safari/7534.48.3";
const char S68015[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9B176 Safari/7534.48.3";
const char S68016[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S68017[] = "SAMSUNG-GT-C3011/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S68018[] = "SAMSUNG-GT-C3011/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S68019[] = "JUC(Linux;U;Android2.3.6;Zh_cn;GT-S5570I;240*320;)UCWEB7.8.0.95/139/444";
const char S68020[] = "JUC(Linux;U;Android2.3.6;Zh_cn;GT-S5570;240*320;)UCWEB7.8.0.95/139/444";
const char S68021[] = "JUC(Linux;U;Android2.3.4;Zh_cn;GT-S5570;240*320;)UCWEB7.8.0.95/139/444";
const char S68022[] = "JUC(Linux;U;Android2.3.5;Zh_cn;GT-S5570;240*320;)UCWEB7.8.0.95/139/444";
const char S68023[] = "Mozilla/5.0 (X11; Linux x86_64; rv:15.0) Gecko/X Firefox/15.0.1 SeaMonkey/2.12.1";
const char S68024[] = "Reeder/1010.79.00 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBook5%2C1)";
const char S68025[] = "Reeder/1010.79.00 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (iMac12%2C2)";
const char S68026[] = "Reeder/1010.79.00 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookPro8%2C2)";
const char S68027[] = "Reeder/1010.79.00 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookAir5%2C2)";
const char S68028[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/6533.18.5 DiigoBrowser";
const char S68029[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; en_us) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5";
const char S68030[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/6533.18.5 DiigoBrowser Safari/6533.18.5 DiigoBrowser";
const char S68031[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10A403 Safari/6533.18.5";
const char S68032[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; ko_kr) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5";
const char S68033[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; 6536.26) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5";
const char S68034[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10A523 Safari/6533.18.5";
const char S68035[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; 6536.26) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5";
const char S68036[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5";
const char S68037[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XOLO_A800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68038[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; PMP7100D3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68039[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; PMP7100D3G Build/JRO03H)";
const char S68040[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT29i Build/9.1.B.0.411) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S68041[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT29i Build/9.1.B.0.411) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S68042[] = "Mozilla/5.0 (Linux; Android 4.1.2; VS930 4G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68043[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9200 Build/JDQ39)";
const char S68044[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9200 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68045[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT27i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68046[] = "Nokia7100s-2/1.0 (06.31) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S68047[] = "Nokia7100s/1.0 (06.32) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S68048[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Stainless/0.8 like Version/5.1 Safari/534.48.3";
const char S68049[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; my Pad P3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68050[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; myPad P3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68051[] = "Mozilla/5.0 (Android; Mobile; rv:13.0a1) Gecko/13.0a1 Firefox/13.0a1";
const char S68052[] = "Mozilla/5.0 (Android; Mobile; rv:13.0) Gecko/13.0 Firefox/13.0 Fennec/13.0";
const char S68053[] = "Mozilla/5.0 (Android; Mobile; rv:13.0) Gecko/13.0 Firefox/13.0a2";
const char S68054[] = "Mozilla/5.0 (Android; Mobile; rv:13.0) Gecko/13.0 Firefox/13.0a1";
const char S68055[] = "Mozilla/5.0 (Android; Mobile; rv:13.0) Gecko/13.0 Firefox/13.0";
const char S68056[] = "Mozilla/5.0 (Linux; U; en-us; Velocitymicro/T408) AppleWebKit/530.17(KHTML, like Gecko) Version/4.0Safari/530.17";
const char S68057[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; POV_TAB-PROTAB26 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68058[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; POV_TAB-PROTAB26XL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68059[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; POV_TAB-PROTAB26-IPS10 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68060[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMP5597D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68061[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TBD753B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68062[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; TBD753B Build/JRO03C)";
const char S68063[] = "Mozilla/5.0 (Linux; Android 4.0.3; A100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S68064[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S68065[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S68066[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-N719 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S68067[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; SCH-N719 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S68068[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7105 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S68069[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E430 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S68070[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6312 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68071[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7560M Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68072[] = "Mozilla/5.0 (Linux; Android 4.1.1; MiSS Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S68073[] = "Mozilla/5.0 (Linux; Android 4.1.2; A9600 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S68074[] = "Mozilla/5.0 (Linux; Android 4.1.1; y7053 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S68075[] = "Mozilla/5.0 (Linux; Android 4.1.2; ThinkPadTablet Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S68076[] = "Mozilla/5.0 (Linux; Android 4.1; 9300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S68077[] = "Mozilla/5.0 (Linux; Android 4.1.1; Tablet PC 3000 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S68078[] = "Mozilla/5.0 (Linux; Android 4.1.1; MID-709S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S68079[] = "Mozilla/5.0 (Linux; Android 4.1.1; eZee Tab973 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S68080[] = "Mozilla/5.0 (Linux; Android 4.1.1; JPAD730 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S68081[] = "Mozilla/5.0 (Linux; Android 4.1.1; M6 Build/M6) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S68082[] = "Mozilla/5.0 (Linux; Android 4.1.2; PMP7380D3G Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S68083[] = "Mozilla/5.0 (Linux; Android 4.1.2; Casper_VIA_A6108 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S68084[] = "Mozilla/5.0 (Linux; Android 4.1.1; MTC 970H Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.20.1364.172 Mobile Safari/537.22";
const char S68085[] = "Mozilla/5.0 (Linux; Android 4.1.2; Walton Primo H1 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S68086[] = "AppleCoreMedia/1.0.0.8C148 (iPad; U; CPU OS 4_2_1 like Mac OS X; en_us)";
const char S68087[] = "AppleCoreMedia/1.0.0.8C134 (iPad; U; CPU OS 4_2 like Mac OS X; en_us)";
const char S68088[] = "Mozilla/5.0 AppleCoreMedia/1.0.0.8J2 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S68089[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-F160L Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68090[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-F160K Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68091[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F160L Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68092[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-F160S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68093[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F160LV Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68094[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-F160 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68095[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; O+_8.75_Android Build/OplusOplus_A100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68096[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; O+ 8.75 Android Build/OplusOplus_A100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68097[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; HTC6435LVW 4G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68098[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; HTC6435LVW Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68099[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC6435LVW Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68100[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC6435LVW 4G Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68101[] = "SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj108i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S68102[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu/12.04 Chromium/27.0.1453.6 Chrome/27.0.1453.6 Safari/537.36";
const char S68103[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu/12.04 Chromium/27.0.1453.6 Chrome/27.0.1453.6 Safari/537.36";
const char S68104[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; SPH-L710Android 4.2.2; en-US; SCH-R530U Build Build/JRO03L) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68105[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S68106[] = "Mozilla/5.0 (Linux; Android 4.2.2; IdeaTab A3000-H Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S68107[] = "Mozilla/5.0 (Linux; Android 4.2.2; IdeaTab A3000-F Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S68108[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N5110 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S68109[] = "SonyEricssonW995a/R1HA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.5";
const char S68110[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.2.2.7373 Safari/532.4";
const char S68111[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Ultimate)";
const char S68112[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; V55 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S68113[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; V55 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S68114[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; V55 Build/HTK75)";
const char S68115[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; SP704C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68116[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; SP704CE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68117[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; HTX21 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68118[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTX21 Build/JRO03C)";
const char S68119[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I727 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S68120[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I727 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S68121[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; SCH-I959 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68122[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; SCH-I959 Build/JDQ39) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S68123[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SCH-I959 Build/JDQ39)";
const char S68124[] = "Opera/9.80 (BREW; Opera Mini/6.0.3/27.2338; U; en) Presto/2.8.119 320X240 LG VN530";
const char S68125[] = "BlackBerry8110/4.3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102";
const char S68126[] = "SAMSUNG-SGH-I617/1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S68127[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Desire Z Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68128[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Desire Z Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68129[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Desire Z Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68130[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Desire Z Build/MIUI)";
const char S68131[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Desire Z Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S68132[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Desire Z Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S68133[] = "Mozilla/5.0 (X11; CrOS x86_64 2046.13.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.9 Safari/536.5";
const char S68134[] = "Mozilla/5.0 (X11; CrOS x86_64 2046.30.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.23 Safari/536.5";
const char S68135[] = "Mozilla/5.0 (X11; CrOS x86_64 2046.68.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.41 Safari/536.5";
const char S68136[] = "Mozilla/5.0 (X11; CrOS x86_64 2046.20.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.17 Safari/536.5";
const char S68137[] = "Mozilla/5.0 (X11; CrOS i686 2046.106.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5";
const char S68138[] = "Mozilla/5.0 (X11; CrOS i686 2046.107.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5";
const char S68139[] = "Mozilla/5.0 (X11; CrOS i686 2046.137.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.57 Safari/536.5";
const char S68140[] = "Mozilla/5.0 (X11; CrOS x86_64 2046.137.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.57 Safari/536.5";
const char S68141[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.5.1 (development) Safari/534.34";
const char S68142[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6800 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S68143[] = "Nokia2710c-2/2.0 (06.13) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S68144[] = "Nokia2710c/2.0 (06.21) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S68145[] = "Nokia2710c-2/2.0 (06.21) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2710c-2";
const char S68146[] = "SAMSUNG-SGH-A687/A687UCJH1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S68147[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-GB; rv:1.8.1.18) Gecko/X Thunderbird/2.0.0.18";
const char S68148[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Spice_Mi-425 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68149[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xolo_X900 Build/AZ510_ICS_02.24I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68150[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68151[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68152[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68153[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160LV Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68154[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P700 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S68155[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S68156[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S68157[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9108 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S68158[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S68159[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9100G Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S68160[] = "LG-GW520-Bouygues Teleca/WAP2.0 MIDP-2.1/CLDC-1.1";
const char S68161[] = "LG-GW520-H3G/V10f Teleca/WAP2.0 MIDP-2.1/CLDC-1.1";
const char S68162[] = "LG-GW520 Teleca/WAP2.0 MIDP-2.1/CLDC-1.1";
const char S68163[] = "LG-GW520-orange/V10d Teleca/WAP2.0 MIDP-2.1/CLDC-1.1";
const char S68164[] = "SonyEricssonC901/R1GA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.4";
const char S68165[] = "LG/U8138/v1.0";
const char S68166[] = "ZTE-Sydney-Orange/EFS-EE-ORG-P107A20V1.0.3 Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S68167[] = "ZTE-Sydney-Orange/EFS-EE-ORG-P107A20V1.0.5 Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S68168[] = "Mozilla/5.0 (Windows NT 4.0; rv:10.0) Gecko/X Firefox/10.0";
const char S68169[] = "NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiax3-02) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S68170[] = "NokiaX3-02.5/5.0 (06.75) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiax3-02.5) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S68171[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; SlidePad704CE Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68172[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; 704CE Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68173[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAP4500TDUO Build/PrestigioPAP4500TDUO) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68174[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAP4500DUO Build/PrestigioPAP4500DUO) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68175[] = "Celkon A98 Linux/3.0.13 Android/4.0.4 Release/04.08.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S68176[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Celkon A98 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68177[] = "Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; BLU Tank4.5 Build/ JB_001_00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68178[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TANK 4.5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68179[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-L710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S68180[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-L710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S68181[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HUAWEI; 4Afrika)";
const char S68182[] = "Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI MT1-U06 Build/HuaweiMT1-U06) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S68183[] = "Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S68184[] = "Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S68185[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-02D Build/S1110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68186[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-02D Build/S2201) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68187[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-02D Build/S3151) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68188[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-02D Build/SB180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68189[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-02D Build/S6130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68190[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SH-02D Build/S6130)";
const char S68191[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-02D Build/S1220) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68192[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us; android4.0; SYTABA73C 11.2.7.8) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S68193[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us;Android4.0;SYMETA10 11.10.1.1) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S68194[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; U8836D Build/HuaweiU8836D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68195[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U8836D Build/HuaweiU8836D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S68196[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; U8836D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68197[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Fly IQ440; Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68198[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; IQ440 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68199[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; IQ440 Build/IMM76D)";
const char S68200[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S68201[] = "NokiaC2-01/5.0 (11.20) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-01) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S68202[] = "[FBAN/FB4A;FBAV/3.1;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/T-Mobile;FBPN/com.facebook.katana;FBDV/SGH-T999;FBSV/4.1.1;]";
const char S68203[] = "[FBAN/FB4A;FBAV/2.3;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/T-Mobile;FBPN/com.facebook.katana;FBDV/SGH-T999;FBSV/4.1.1;]";
const char S68204[] = "[FBAN/FB4A;FBAV/2.2.1;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/T-Mobile;FBPN/com.facebook.katana;FBDV/SGH-T999;FBSV/4.1.1;]";
const char S68205[] = "[FBAN/FB4A;FBAV/3.2.1;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/T-Mobile;FBPN/com.facebook.katana;FBDV/SGH-T999;FBSV/4.1.2;]";
const char S68206[] = "[FBAN/FB4A;FBAV/3.3;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/T-Mobile;FBPN/com.facebook.katana;FBDV/SGH-T999;FBSV/4.1.2;]";
const char S68207[] = "[FBAN/FB4A;FBAV/3.5;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/T-Mobile;FBPN/com.facebook.katana;FBDV/SGH-T999;FBSV/4.1.2;]";
const char S68208[] = "[FBAN/FB4A;FBAV/3.4;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/T-Mobile;FBPN/com.facebook.katana;FBDV/SGH-T999;FBSV/4.1.2;]";
const char S68209[] = "[FBAN/FB4A;FBAV/3.6.1;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/SCH-I535;FBSV/4.1.2;FBCA/armeabi-v7a:armeabi;]";
const char S68210[] = "[FBAN/FB4A;FBAV/2.3;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/T-Mobile;FBPN/com.facebook.katana;FBDV/SGH-T999;FBSV/4.1.2;]";
const char S68211[] = "[FBAN/FB4A;FBAV/3.6.1;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/T-Mobile;FBPN/com.facebook.katana;FBDV/SGH-T999;FBSV/4.1.2;FBCA/armeabi-v7a:armeabi;]";
const char S68212[] = "Mozilla/5.0 (Linux; Android 4.2.1; M470BSA Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S68213[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F220K/10c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68214[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F220K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68215[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F220K/10d Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68216[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F220K Build/JZO54K)";
const char S68217[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5120 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S68218[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae5-00) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile  3gpp-gba";
const char S68219[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE5-00) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S68220[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE5-00.2) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S68221[] = "SAMSUNG-SGH-G800/JBGL1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S68222[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A70H Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68223[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; A70H Build/FROYO)";
const char S68224[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PTM1005HN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68225[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; STM1007HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68226[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-pt; MTM1005HN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68227[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PTM1005NT Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68228[] = "Mozilla/5.0 (X11; FreeBSD amd64; rv:15.0) Gecko/X Firefox/15.0.1";
const char S68229[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/9/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S68230[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/9/30.3214; U; en) Presto/2.8.119 Version/11.10";
const char S68231[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:17.0) Gecko/X Firefox/17.0 SeaMonkey/2.14.1";
const char S68232[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT881 Build/9.8.3O-118_USC-14) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S68233[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT881 Build/9.8.3O-118_USC-14) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S68234[] = "Mozilla/5.0 (Linux; Android 4.1.1; A700 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S68235[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I605 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S68236[] = "Mozilla/5.0 (Linux; Android 4.1.2; SM-T211 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S68237[] = "Mozilla/5.0 (Linux; Android 4.0.4; Motorola RAZR MAXX Build/6.7.2-180_DHD-16_NA-27) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68238[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S68239[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S68240[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC VLE_U Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S68241[] = "Mozilla/5.0 (Series40; NokiaX3-02.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.8";
const char S68242[] = "Mozilla/5.0 (Series40; NokiaX3-02/07.51; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.11";
const char S68243[] = "Mozilla/5.0 (Linux; U; Android 2.2; de-de; BASE lutea Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68244[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; BASE lutea Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68245[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; pt-pt; Optimus San Francisco Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68246[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; Orange San Francisco Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68247[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ro-ro; Orange San Francisco Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68248[] = "MOT-24.1_/00.62 UP.Browser/6.2.3.4.c.1.120 (GUI) MMP/2.0";
const char S68249[] = "MOT-COOL0/00.62 UP.Browser/6.2.3.4.c.1.128 (GUI) MMP/2.0";
const char S68250[] = "MOT-NCR_0/00.62 UP.Browser/6.2.3.4.c.1.115 (GUI) MMP/2.0";
const char S68251[] = "MOT-24.0_/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0";
const char S68252[] = "MOT-V9mm/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0";
const char S68253[] = "MOT-MILLA/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0";
const char S68254[] = "MOT-E815_/00.62 UP.Browser/6.2.3.4.c.1.106 (GUI) MMP/2.0";
const char S68255[] = "MOT-GATW_/00.62 UP.Browser/6.2.3.4.c.1.115 (GUI) MMP/2.0";
const char S68256[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) 320x320; XV6900; Window Mobile 6.0 Professional;";
const char S68257[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) S11HT";
const char S68258[] = "Lenovo_ET860/Lenovo WindowsMobile/6 Release/4.22.2007 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S68259[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S68260[] = "Mozilla/5.0 (X11; U; FreeBSD amd64; en-US; rv:1.8.1.21) Gecko/X SeaMonkey/1.1.16";
const char S68261[] = "JUC(Linux;U;Android2.3.6;Zh_cn;GT-S6102;240*320;)UCWEB7.8.0.95/139/444";
const char S68262[] = "Mozilla/5.0(Linux;U;Android2.3.6;Zh_cn;GT-S6102;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/444";
const char S68263[] = "Opera/9.80 (Android 2.1-update1; Linux; Opera Mobi/ADR-X; U; pl) Presto/2.10.254 Version/12.00";
const char S68264[] = "MT6515M-6515/1.0 Linux/2.6.35.7 Android 4.0 Release/11.13.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S68265[] = "Mozilla/5.0 (Linux; U; Android 4.0; zh-cn; MT6515M Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68266[] = "MT6515M-Connspeed-AS9/1.0 Linux/2.6.35.7 Android 4.0.4 Release/12.11.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S68267[] = "MT6515M-226J/1.0 Linux/2.6.35.7 Android 4.0 Release/11.13.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S68268[] = "MT6515M-6515/1.0 Linux/2.6.35.7 Android 4.0 Release/11.15.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S68269[] = "MT6515M-6515/1.0 Linux/2.6.35.7 Android 4.0 Release/11.14.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S68270[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; MT6515-MT6515-H003/1.0 Android/4.0.3 Release/04.02.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68271[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; MT6515-MT6515-H003/1.0 Android/4.0.3 Release/05.09.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68272[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; MT6515-sphs-on-hsdroi/1.0 Android/4.0.3 Release/07.18.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68273[] = "MT6515M-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android 4.0 Release/11.12.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S68274[] = "Mozilla/5.0 (compatible; MSIE 9.0; WindowsCE; Trident/1.0)";
const char S68275[] = "Mozilla/5.0 (compatible; MSIE 9.0; PPC; Tablet PC; Trident/5.0)";
const char S68276[] = "Mozilla/5.0 (compatible; MSIE 9.0; 68K; Tablet PC; Trident/3.0)";
const char S68277[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6800 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S68278[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:22.0) Gecko/X Firefox/22.0 SeaMonkey/2.19";
const char S68279[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68280[] = "Motorola_ES405B/191 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S68281[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Motorola_ES405B_19103";
const char S68282[] = "Motorola_ES405B/20 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S68283[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Motorola_ES405B_312";
const char S68284[] = "Motorola_ES405B/27 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Motorola_ES405B_256";
const char S68285[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Motorola_ES405B_256";
const char S68286[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Motorola_ES405B_44.000";
const char S68287[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:34, us";
const char S68288[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Nexus 7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, gb";
const char S68289[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:34, us";
const char S68290[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03S) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:34, sg";
const char S68291[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Micromax A90 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68292[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; A90 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68293[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; A90 Build/IML74K)";
const char S68294[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M7088 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68295[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M7088_ Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68296[] = "Mozilla/5.0 (Windows Phone 8.0; Trident/7.0; rv:11.0; IEMobile/11.0; ARM; Touch; Microsoft; Virtual) like Gecko";
const char S68297[] = "Mozilla/5.0 (Linux; Android 4.1.1; MI 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68298[] = "Mozilla/5.0 (Linux; Android 4.1.1; MI 2A Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68299[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68300[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S68301[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I577 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S68302[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; PalmSource/Palm-D050; Blazer/4.3) 16;448x320";
const char S68303[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; PalmSource/Palm-D050; Blazer/4.3) 16;320x448";
const char S68304[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; PalmSource/Palm-D050; Blazer/4.3) 16;320x320";
const char S68305[] = "Opera/9.5 (Microsoft Windows; PPC; Opera Mobi; U) SonyEricssonX1i/R3AA Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S68306[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s3850) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S68307[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S68308[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5300 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S68309[] = "Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; en-us; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/6.0 Safari/534.13";
const char S68310[] = "Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.1; en-us; Xoom Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/6.0 Safari/534.13";
const char S68311[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT907 Build/7.7.1Q-144_SMQ_S136-10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S68312[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT905 Build/7.7.1Q_GCIRD-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S68313[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5 UCBrowser/9.2.3.324";
const char S68314[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 4_3_3 like Mac OS X; zh-CN) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 UCBrowser/9.2.0.312 Mobile";
const char S68315[] = "Mozilla/5.0 (Series40; NokiaC2-02/06.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S68316[] = "BlackBerry8820/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102";
const char S68317[] = "BlackBerry8300/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102";
const char S68318[] = "SEC-SGHM620/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S68319[] = "SEC-SGHM620/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S68320[] = "Mozilla/5.0 (Windows NT 5.2; rv:10.0.2) Gecko/X Thunderbird/10.0.2";
const char S68321[] = "Mozilla/5.0 (Windows NT 5.1; rv:10.0.11) Gecko/X Thunderbird/10.0.11";
const char S68322[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE-N880E Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68323[] = "Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE N880E Build GRK39F) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S68324[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE N880E Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S68325[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; ZTE N880E Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68326[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; ZTE N880E Build/GRK39F)";
const char S68327[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I897 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68328[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I897 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68329[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S68330[] = "Mozilla/5.0 (Linux; Android 4.0.4; KFJWI Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S68331[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST27i Build/6.1.1.B.1.54) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S68332[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST27a Build/6.1.1.B.1.54) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S68333[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xperia Go Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S68334[] = "Mozilla/5.0 (Linux; Android 4.2.2; SC-04E Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68335[] = "Mozilla/4.0 (Brew MP 1.0.2; U; en-us; Sanyo; NetFront/3.5.1/AMB) Sprint E4100";
const char S68336[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; en) Opera 8.53";
const char S68337[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; en) Opera 8.54";
const char S68338[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; en) Opera 8.52";
const char S68339[] = "Mozilla/5.0 (Windows NT 5.0; U; de) Opera 8.50";
const char S68340[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; ru) Opera 8.50";
const char S68341[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; ru) Opera 8.51";
const char S68342[] = "IBrowse/2.2 (Windows 3.1)";
const char S68343[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B117 Twitter for iPhone";
const char S68344[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B118 Twitter for iPhone";
const char S68345[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire S Build/GRJ90; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68346[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I605 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68347[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I605 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68348[] = "NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-02) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S68349[] = "SonyEricssonK510i/R4EA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S68350[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) iCab/5.0 Safari/533.16";
const char S68351[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) iCab/5.0 Safari/533.16";
const char S68352[] = "Mozilla/5.0 (Linux; Android 4.0.4; WT19i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68353[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.5.0.183/50/352/UCWEB";
const char S68354[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.5.0.183/50/351/UCWEB";
const char S68355[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.5.0.183/28/352/UCWEB";
const char S68356[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.5.0.183/28/351/UCWEB";
const char S68357[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.5.0.183/51/352/UCWEB";
const char S68358[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.5.0.183/50/444/UCWEB";
const char S68359[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.5.0.183/50/354/UCWEB";
const char S68360[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.5.0.183/50/355/UCWEB";
const char S68361[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.5.0.183/28/444/UCWEB";
const char S68362[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.5.0.183/28/355/UCWEB";
const char S68363[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.5.0.183/51/355/UCWEB";
const char S68364[] = "Mozilla/5.0 (Linux; Android 4.0.4; AZ210A Build/AZ210A_ICS_02.27I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68365[] = "Mozilla/5.0 (Linux; Android 4.0.4; AZ210B Build/AZ210B_ICS_02.27I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68366[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; MB860 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68367[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; MB860 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68368[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MB860 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68369[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MB860 Build/JDQ39; CyanogenMod-BeanStalk1.179) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68370[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E270S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68371[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E270S Build/JZO54K)";
const char S68372[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68373[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9000 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68374[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; QMobile_A8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68375[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; A8 Build/JRO03C)";
const char S68376[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5282 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S68377[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5282 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S68378[] = "samsung-GT-I9500/1.0 Linux/2.6.35.7 Android/4.2.2 Release/06.06.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S68379[] = "sprd-I9500/1.0 Linux/2.6.35.7 Android/4.2.2 Release/07.10.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S68380[] = "samsung-GT-mini-I9500/1.0 Linux/2.6.35.7 Android/4.2.2 Release/07.17.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S68381[] = "sprd-I9500/1.0 Linux/2.6.35.7 Android/4.2.2 Release/07.18.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S68382[] = "sprd-I9500/1.0 Linux/2.6.35.7 Android/4.2.2 Release/07.29.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S68383[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68384[] = "LG-C305/V10a Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S68385[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; iw-il; U8160 Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68386[] = "Mozilla/5.0 (Symbian3; U; en-us; NokiaC6-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/51/352/UCWEB Mobile";
const char S68387[] = "Mozilla/5.0 (Java; U; en-us; nokiac6-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile  3gpp-gba";
const char S68388[] = "NokiaC6-01/UCWEB8.6.0.180/51/800";
const char S68389[] = "Opera/9.80 (Android; Opera Mini/7.1.32102/28.3234; U; zh) Presto/2.8.119 Version/11.10";
const char S68390[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NGM Wilco Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68391[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; NGM_Wilco Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68392[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cruize W280 Build/Cherry_Cruize_W280_2012.12.30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68393[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cruize W280 Build/V1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68394[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cruize W280 Build/V1.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68395[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; AT-AS45IPS Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68396[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; AT-AS45IPS Build/JRO03C)";
const char S68397[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30a Build/9.1.A.0.489) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68398[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S68399[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9100G Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S68400[] = "Mozilla/5.0 (Series40; Nokia113/03.32; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S68401[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ617 Build/7.7.1-85_MZ617-27) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S68402[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A850L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68403[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A850K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68404[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A850S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68405[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A850S/L/K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68406[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Alcatel OT-981A Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S68407[] = "Mozilla/5.0 (Linux; U; Android 2.0; en-us; Milestone Build/SHOLS_U2_01.03.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 NewsRob (http://newsrob.com) gzip";
const char S68408[] = "Mozilla/5.0 (Linux; U; Android 2.0.1; de-de; Milestone Build/SHOLS_U2_01.14.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S68409[] = "Mozilla/5.0 (Linux; U; Android 2.0.1; en-ca; Milestone Build/SHOLS_U2_03.06.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S68410[] = "Mozilla/5.0 (Linux; U; Android 2.0; en-us; Milestone Build/ SHOLS_U2_01.03.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S68411[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; EV-S100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68412[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; EV-S100 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68413[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; EV-S100 Build/FRF91)";
const char S68414[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; EV-S100 Build/FRG83G)";
const char S68415[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; rv:1.7.3) Gecko/X Firefox/0.10";
const char S68416[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; fr; rv:1.7) Gecko/X Firefox/0.9";
const char S68417[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_2 like Mac OS X; en-us)Maxthon AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5G77 Safari/525.20";
const char S68418[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Stainless/0.8 like Version/5.1 Safari/534.48.3";
const char S68419[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Windows Phone 8S by HTC)";
const char S68420[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; A620m)";
const char S68421[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Windows Phone 8S by HTC; Orange)";
const char S68422[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; A620b)";
const char S68423[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Windows Phone 8S by HTC; 2.00.161.01)";
const char S68424[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Windows Phone 8S by HTC; 1.04.161.01)";
const char S68425[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; A620e)";
const char S68426[] = "Mozilla/5.0 (Linux; Android 4.1.1; TAB-840_G Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S68427[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT27i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68428[] = "Mozilla/5.0 (Linux; Android 4.1.2; A111 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68429[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I717 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S68430[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I717 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S68431[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035A Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68432[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68433[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68434[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG-E906)";
const char S68435[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA6120c)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S68436[] = "NOKIA6120c/UCWEB8.5.0.163/28/800";
const char S68437[] = "Mozilla/5.0 (Java; U; en-us; nokia6120c) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile";
const char S68438[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; pt-br; XT682 Build/V1.51D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68439[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; MB526 Build/GWK74; CyanogenMod-72) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68440[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; MB526 Build/WhiteRabbitEdition-1.7.1-Plus; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68441[] = "Mozilla/5.0 (Linux; Android 4.0.4; MB886 Build/7.7.1Q-144_MB886_MR1-22) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S68442[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S68443[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S68444[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26ii Build/6.2.B.0.200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68445[] = "Mozilla/5.0 (Linux; Android 4.1.2; PantechP9090 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S68446[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9080L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68447[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows 98) Opera 6.01  [en]";
const char S68448[] = "Opera/6.03 (Windows 98; U) [en]";
const char S68449[] = "Opera/6.05 (Windows 98; U) [de]";
const char S68450[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows 98) Opera 6.03 [en]";
const char S68451[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows 98) Opera 6.0 [de]";
const char S68452[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows 98) Opera 6.04";
const char S68453[] = "Opera/6.06 (Windows 98; U)  [en]";
const char S68454[] = "Opera/6.01 (Windows 98; U)  [en]";
const char S68455[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.2) Gecko/X Firefox/3.0.2 Flock/2.0b3";
const char S68456[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ALLVIEW P5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68457[] = "MQQBrowser/40 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3";
const char S68458[] = "MQQBrowser/40 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari/7534.48.3";
const char S68459[] = "MQQBrowser/40 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3";
const char S68460[] = "MQQBrowser/40 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3";
const char S68461[] = "MQQBrowser/40 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Safari/7534.48.3";
const char S68462[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; U30GT 2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68463[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7560M Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68464[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SH-06E Build/S5110) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68465[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SH-06E Build/S5310) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68466[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SH-06E Build/S6170) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68467[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SH-06E Build/S7190) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68468[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SH-06E Build/S7190)";
const char S68469[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S68470[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC_PN071 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S68471[] = "Nokia112/2.0 (03.32) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia112) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S68472[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S68473[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8C148 Safari/6531.22.7";
const char S68474[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S68475[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8C148 Safari/6531.22.7";
const char S68476[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8E401 Safari/6531.22.7";
const char S68477[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S68478[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293";
const char S68479[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8E200 Safari/6531.22.7";
const char S68480[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8C148a Safari/6531.22.7";
const char S68481[] = "Mozilla/5.0 (iPhone; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0 Mobile/8C148 Safari/528.16";
const char S68482[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S68483[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_10 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S68484[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S68485[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en; rv:1.9.0.8pre) Gecko/X Camino/2.0b3pre";
const char S68486[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.5; en; rv:1.9.0.19) Gecko/X Camino/2.0.6 (MultiLang) (like Firefox/3.0.19)";
const char S68487[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S68488[] = "480x800; XV6875.1; Window Mobile 6.5 Professional; (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S68489[] = "480x800; XV6975; Window Mobile 6.5 Professional; (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S68490[] = "USCCHTC6875 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S68491[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)/UC Browser7.7.1.88";
const char S68492[] = "T-Mobile_LEO (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S68493[] = "T-Mobile_Rhodium (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S68494[] = "Sprint-SPH-i350 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S68495[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_5210/5210_V1.10; 320*480; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68496[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; YL-Coolpad_5210/5210_V1.10; 320*480; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68497[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 5210 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68498[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_5210S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68499[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 5210 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S68500[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad 5210S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68501[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 5210D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68502[] = "SAMSUNG-GT-S3572/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S68503[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax A110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S68504[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax_A110 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S68505[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/588.88 (KHTML, like Gecko) Version/5.1.3 Safari/588.88";
const char S68506[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/599.99 (KHTML, like Gecko) Version/5.1.3 Safari/599.99";
const char S68507[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Evo V 4G Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68508[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Evo V 4G Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68509[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Evo V 4G Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68510[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-uk; @TAB-SEVEN-X2-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68511[] = "Mozilla/5.0 (Linux; Android 4.0.4; Event Build/IML77) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S68512[] = "Mozilla/5.0 (Linux; Android 4.2.2; Xoom Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S68513[] = "Mozilla/5.0 (Linux; Android 4.2.1; Xoom Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S68514[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT18i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68515[] = "Mozilla/5.0 (Linux; Android 4.0.3; LT18i Build/4.1.A.0.562) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68516[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT18a Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68517[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S68518[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; SGH-T999 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S68519[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; SCH-I535 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S68520[] = "Mozilla/5.0 (Linux; U; Android 2.2;en-us;Huawei-U8520 build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68521[] = "Mozilla/5.0 (Linux; U; Android 2.2.2;pt-pt;U8520 build/HuaweiU8520) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68522[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-au; U8520 Build/HuaweiU8520) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68523[] = "Mozilla/5.0 (Linux; U; Android 2.2.2;de-de;Dual build/HuaweiU8520) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68524[] = "Nokia6300/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia6300) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S68525[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP8010 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S68526[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) 1Password/4.2.2 (like Version/11A4414e Mobile/7.0 Safari/8536.25)";
const char S68527[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) 1Password/4.2.7 (like Version/11A465 Mobile/7.0 Safari/8536.25)";
const char S68528[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) 1Password/4.2.7 (like Version/11A501 Mobile/7.0.2 Safari/8536.25)";
const char S68529[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ505 Build/Moto_MZ505) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S68530[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xoom Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S68531[] = "Mozilla/5.0 (Linux; Android 4.0.3; Xoom Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S68532[] = "SAMSUNG-GT-B7320L/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S68533[] = "SAMSUNG-GT-B7320/1.0 MIDP/2.1 CLDC/1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG-GT-B7320/1.0 MIDP/2.1 CLDC/1.1 Mozilla/4.0";
const char S68534[] = "SAMSUNG-GT-B7320/1.0 MIDP/2.1 CLDC/1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S68535[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.2.3 (KHTML, like Gecko) Version/5.2 Safari/536.2.3";
const char S68536[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.10.3 (KHTML, like Gecko) Version/5.2 Safari/536.10.3";
const char S68537[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/536.6.1 (KHTML, like Gecko) Version/5.2 Safari/536.6.1";
const char S68538[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.18.5 (KHTML, like Gecko) Version/5.2 Safari/535.18.5";
const char S68539[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT25i Build/4.1.B.0.631) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68540[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N8000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S68541[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9300 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S68542[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Incredible S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S68543[] = "Mozilla/5.0 (Linux; Android 4.2.2; A1-810 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S68544[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26w Build/6.1.A.2.45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68545[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/XXGK4 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68546[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/XBGK3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68547[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DZHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68548[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DZHF1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68549[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/XBHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68550[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DFHB5 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68551[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DTHB3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68552[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DZGL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68553[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/AIGI4 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68554[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/JAGL4 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68555[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/JCGL4 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68556[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DCGL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68557[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/AIHA2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68558[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/XXHC2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68559[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/XXHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68560[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/XXHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68561[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/XXGK6 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68562[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/JVGL1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68563[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DZHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68564[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/XEGK5 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68565[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450V/BUHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68566[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450V/BUHB3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68567[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DTHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68568[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/JDGL4 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68569[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/JPHB1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68570[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/XXHB1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68571[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/JPHC2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68572[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/MBHC3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68573[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/BGGK4 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68574[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450V/BUHC2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68575[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450V/JBJG5 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68576[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DBHF1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68577[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DBGL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68578[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/JPGL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68579[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/AIHD3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68580[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/JVHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68581[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DDHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68582[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/XEGL1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68583[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DBHD2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68584[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DDHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68585[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450V/BUGJ6 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68586[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DDHB4 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S68587[] = "BlackBerry8330/4.5.0.186 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/126 UP.Browser/5.0.3.3";
const char S68588[] = "BlackBerry8330m/4.5.0.186 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/104 UP.Browser/5.0.3.3";
const char S68589[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; XV6875.1; Windows Phone 6.5)";
const char S68590[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch_Pro2_T7373; Windows Phone 6.5)";
const char S68591[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch_Pro2_T7373; Windows Phone 6.5.3.5)";
const char S68592[] = "HTC_Touch_Pro2_T7381 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; V1.93.841.0 (81210); Windows Phone 6.5)";
const char S68593[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch_Pro2_T7373; HD2_T8585; Windows Phone 6.5)";
const char S68594[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; XV6875; Windows Phone 6.5)";
const char S68595[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT540 Build/GRI40; SwiftDroid v2.0 ASIS (Based on CyanogenMod-7.1.0)) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68596[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; pl-pl; GT540 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68597[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; pl-pl; LG GT540 Swift Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68598[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT540 Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68599[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; pl-pl; LG GT540 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68600[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19";
const char S68601[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19";
const char S68602[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:0.9.2) Gecko/X Netscape6/6.1";
const char S68603[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5263) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S68604[] = "Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80G9 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S68605[] = "Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 80G9 Build/MR1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S68606[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; XT885 Build/9.8.3O-148_IRD-3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68607[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; XT885 Build/6.7.2_GC-385) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68608[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT885 Build/9.8.3O-148_IRD-3)";
const char S68609[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; GT-P3100 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S68610[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-P3100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S68611[] = "Mozilla/5.0 (Series40; NokiaC3-01/07.51; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S68612[] = "Mozilla/5.0 (Series40; NokiaC3-01.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S68613[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Butterfly Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68614[] = "Mozilla/5.0(compatible; Access NetFront/3.5; BREW 3.1.5; en,es) 400x240 PCD C741";
const char S68615[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6.8; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.60";
const char S68616[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en) Presto/2.10.229 Version/11.60";
const char S68617[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X708S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68618[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; A500 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68619[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; A500 Build/JDQ39; CyanogenMod-Tegraowners JB-MR1 ROM v10 (thor & digetx)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68620[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; A500 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68621[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; A500 Build/JDQ39E; CyanogenMod-Tegraowners JB-MR1 ROM v11 (thor & digetx)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68622[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; iBall Andi3.5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68623[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Andi3.5r Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68624[] = "Mozilla/5.0 (Linux; U; Android 2.3.8; en-us; Andi3.5KKe Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68625[] = "Mozilla/5.0 (X11; SunOS i86pc; rv:20.0) Gecko/X Firefox/20.0";
const char S68626[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P880 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S68627[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P880 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S68628[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T699 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68629[] = "Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 101G9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S68630[] = "Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 101G9 Build/MR1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S68631[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-tw; HTC Magic Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S68632[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HTC Magic Build/EPE54B) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S68633[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; HTC Magic Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S68634[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HTC Magic Build/Super E) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S68635[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; HTC Magic Build/ERE27)";
const char S68636[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; HTC Magic Build/EPE54B) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S68637[] = "Mozilla/5.0 (Vodafone/1.0/LG-GS290/V10f Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S68638[] = "Mozilla/5.0 (Vodafone/1.0/LG-GS290/V10b Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S68639[] = "Mozilla/5.0 (Vodafone/1.0/LG-GS290/V10g Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S68640[] = "Mozilla/5.0 (Vodafone/1.0/LG-GS290/V10c Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S68641[] = "Mozilla/5.0 (Vodafone/1.0/LG-GS290/V10d Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S68642[] = "Nokia6234/2.0 (05.10) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S68643[] = "SAMSUNG-SGH-A797/A797UCJC1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S68644[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; DELL; Venue Pro)";
const char S68645[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-VM670 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68646[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-VM670 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68647[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB870 Build/4.5.1A-DTN-130; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68648[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MB870 Build/4.5.1A-DTN-130; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68649[] = "Micromax X456/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S68650[] = "Mozilla/5.0 (Windows NT 5.1; rv:21.0) Gecko/X Firefox/21.0 SeaMonkey/2.18";
const char S68651[] = "Mozilla/5.0 (Linux; Android 4.2.2; One S Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68652[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68653[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-GB; rv:1.8.0.14) Gecko/X Thunderbird/1.5.0.14";
const char S68654[] = "HS-U8/1.0 Linux/2.6 Android/2.3 Browser/AppleWebKit533.1 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S68655[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-U8 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68656[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-U8 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S68657[] = "HS-U8 Linux Android/2.3.5 Browser/AppleWebKit/533.1 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S68658[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-U8 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S68659[] = "HS-U860/1.0 Linux/2.6 Android/2.3 Browser/AppleWebKit533.1 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S68660[] = "Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; HS-U8 Build GRK39F) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S68661[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; HS-U820 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68662[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-S5830i Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68663[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile IQ 1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68664[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; i-mobile IQ 1 Build/IMM76D)";
const char S68665[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Cynus T2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68666[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Cynus T2 Build/JRO03C)";
const char S68667[] = "Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI MT1-U06 Build/HuaweiMT1-U06) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68668[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4435d Safari/7534.48.3 TeamStream (iPhone5,2 379 2.7.2)";
const char S68669[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d Safari/7534.48.3 TeamStream (iPhone5,2 389 2.8)";
const char S68670[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari/7534.48.3 TeamStream (iPhone5,2 408 2.9.1)";
const char S68671[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Safari/7534.48.3 TeamStream (iPhone4,1 408 2.9.1)";
const char S68672[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Incredimail/1.2.6.6361";
const char S68673[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Incredimail/1.2.6.6361";
const char S68674[] = "SonyEricssonU100i/R7BA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2";
const char S68675[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7 HTC_Sensation-orange-LS; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S68676[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68677[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 baiduboxapp/042_1.4_diordna_008_084/gnusmas_61_2.1.4_D2628I-TG/Xe/A074C84EXDDB007D93FX%7CX/1";
const char S68678[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8262D Build/JZO54K)";
const char S68679[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S68680[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I8262D Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68681[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P3110 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S68682[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P3110 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S68683[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P3110 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S68684[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P5113 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S68685[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P5113 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S68686[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P870 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S68687[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P895 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68688[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P895qb Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68689[] = "Mozilla/5.0 (Linux; Android 4.3; GT-N7000 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68690[] = "Nokia6151/2.0 (04.10) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S68691[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; fr; rv:1.9b4pre) Gecko/X SeaMonkey/2.0a1pre";
const char S68692[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; Cosmo Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68693[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; Micromax P500(Funbook) Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68694[] = "Mozilla/5.0 (X11; SunOS i86pc; rv:19.0) Gecko/X Firefox/19.0";
const char S68695[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MPDC110 BT IPS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68696[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HUAWEI; W1-U00)";
const char S68697[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Huawei; W1-U34; Huawei W1-U34)";
const char S68698[] = "Mozilla/5.0 (Linux; Android 4.2.2; zh-cn; SCH-I959 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S68699[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Q700 Build/XOLOQ700) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68700[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; Q700 Build/XOLOQ700)";
const char S68701[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Z788G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68702[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.0.3.2197 Safari/532.4";
const char S68703[] = "GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (5.1.2600.3);en-US;kml:2.2;client:Free;type:default)";
const char S68704[] = "GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (5.1.2600.3);en;kml:2.2;client:Free;type:default)";
const char S68705[] = "GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (5.1.2600.2);en-US;kml:2.2;client:Free;type:default)";
const char S68706[] = "GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (5.1.2600.3);en-GB;kml:2.2;client:Free;type:default)";
const char S68707[] = "GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (5.1.2600.3);de-DE;kml:2.2;client:Free;type:default)";
const char S68708[] = "GoogleEarth/6.0.2.2074(Windows;Microsoft Windows (5.1.2600.3);en-US;kml:2.2;client:Free;type:default)";
const char S68709[] = "GoogleEarth/6.0.1.2032(Windows;Microsoft Windows (5.1.2600.3);en-US;kml:2.2;client:Free;type:default)";
const char S68710[] = "GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (5.1.2600.3);zh-Hans-CN;kml:2.2;client:Free;type:default)";
const char S68711[] = "GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (5.1.2600.3);es-419;kml:2.2;client:Free;type:default)";
const char S68712[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; IM-A725L Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68713[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; IM-A725L Build/FRG83)";
const char S68714[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:5.0) Gecko/X Firefox/5.0,gzip(gfe) (via translate.google.com)";
const char S68715[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, lik0) Gecko/X Firefox/5.0";
const char S68716[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-F120S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68717[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F120K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68718[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A11 Build/KarbonnA11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68719[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP9090 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S68720[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID7051 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68721[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5300 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S68722[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9505 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S68723[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E610 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68724[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E612 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68725[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E612f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68726[] = "Mozilla/5.0 (compatible; MSIE 7.0; MSIE 6.0; ScanAlert; +http://www.scanalert.com/bot.jsp) Firefox/2.0.0.3";
const char S68727[] = "Mozilla/5.0 (compatible; MSIE 7.0; MSIE 6.0; Site Scanner Bot; +http://www.websiteprotection.com) Firefox/2.0.0.3";
const char S68728[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; KidzStar Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68729[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; KidzStar Build/IML74K)";
const char S68730[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-SU760 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68731[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-SU760 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68732[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-SU760 Build/ICS)";
const char S68733[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.2.4) Gecko/X FireDownload/2.0.1 Firefox/3.6.4 Wyzo/3.6.4.1 FireTorrent/2.0.1";
const char S68734[] = "Mozilla/5.0 (Linux; Android 4.1.1; X909 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68735[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A3+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68736[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S68737[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mstar) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S68738[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S68739[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; opera) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S68740[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui browser) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S68741[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mstar platform) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S68742[] = "UCWEB/2.0 (Java; U; MIDP-2.0; vi; maui) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S68743[] = "UCWEB/2.0 (Java; U; MIDP-2.0; id; maui) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S68744[] = "Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:25.0) Gecko/X Firefox/25.0";
const char S68745[] = "Mozilla/5.0 (Linux; Android 4.0.3; LIFETAB_P9514 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S68746[] = "Opera/9.5 (Microsoft Windows; Windows CE; Opera Mobi/9.5; U; en) Samsung-SCHI910 PPC 240x400";
const char S68747[] = "Opera/9.5 (Microsoft Windows; Windows CE; Opera Mobi/9.5; U; en) Samsung-SCHI910 BMC PPC 240x400";
const char S68748[] = "Opera/9.5 (Microsoft Windows; Windows CE; Opera Mobi/9.5; U; en) VZW:SCH-i910 PPC 240x400";
const char S68749[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7 HTC_DesireS-orange-LS; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S68750[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24 XiaoMi/MiuiBrowser/1.0";
const char S68751[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-L710 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S68752[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-L710 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S68753[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; novo9-Spark Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68754[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; novo9-Spark Build/MASTER)";
const char S68755[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; novo9-Spark Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68756[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-1 Build/HuaweiU8950-1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68757[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-51 Build/HuaweiU8950N-51) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68758[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-51 Build/HuaweiU8950-51) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68759[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-1 Build/HuaweiU8950N-1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68760[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950D Build/HuaweiU8950D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68761[] = "UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-914_im_india_269) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile";
const char S68762[] = "UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-885_im_india_249) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile";
const char S68763[] = "UCWEB/2.0 (Windows; U; wds 8.0; zh-CN; NOKIA; RM-822_apac_prc_204) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile";
const char S68764[] = "UCWEB/2.0 (Windows; U; wds 8.0; zh-CN; NOKIA; RM-867_apac_prc_277) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile";
const char S68765[] = "UCWEB/2.0 (Windows; U; wds 8.0; vi; NOKIA; RM-914_apac_vietnam_228) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile";
const char S68766[] = "UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-914_im_south_africa_243) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile";
const char S68767[] = "UCWEB/2.0 (Windows; U; wds 8.0; zh-CN; NOKIA; RM-914_apac_prc_227) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile";
const char S68768[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Mi-315) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S68769[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Celkon_A_59) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S68770[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; POCKET) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S68771[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; A80) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S68772[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Idea_Zeal) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S68773[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; B51+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S68774[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; P8) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S68775[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; TECNO_T3) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S68776[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A87) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S68777[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Lemon_P3) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S68778[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; A56) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S68779[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; Spice_Mi280) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S68780[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_4010X) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S68781[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Symphony_W15) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S68782[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_4010D) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S68783[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Fly_F40) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S68784[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5369) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S68785[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; A19) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S68786[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; AQUA_4.0) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S68787[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S68788[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; rv:1.7.3) Gecko/X Firefox/0.10";
const char S68789[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.8a) Gecko/X Firefox/0.8.0+";
const char S68790[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.7) Gecko/X Firefox/0.9.2";
const char S68791[] = "iOS/5.1 (9B176) dataaccessd/1.0";
const char S68792[] = "iOS/5.1 (9B179) dataaccessd/1.0";
const char S68793[] = "iOS/5.0.1 (9A405) dataaccessd/1.0";
const char S68794[] = "iOS/5.1.1 (9B206) dataaccessd/1.0";
const char S68795[] = "iOS/6.0 (10A403) dataaccessd/1.0";
const char S68796[] = "iOS/6.0 (10A5376e) dataaccessd/1.0";
const char S68797[] = "iOS/6.0.1 (10A523) dataaccessd/1.0";
const char S68798[] = "iOS/6.0 (10A405) dataaccessd/1.0";
const char S68799[] = "iOS/6.0.1 (10A525) dataaccessd/1.0";
const char S68800[] = "iOS/6.0.2 (10A551) dataaccessd/1.0";
const char S68801[] = "iOS/6.0.2 (10A550) dataaccessd/1.0";
const char S68802[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonyEricssonSO-02D Build/6.1.F.0.106) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68803[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-02D Build/6.1.F.0.106) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68804[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-02D Build/6.1.F.0.117) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68805[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I915 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S68806[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID7065 Build/ICS.MID7065.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68807[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MID7065 Build/ICS.MID7065.X)";
const char S68808[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus S 4G Build/JRO03R) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S68809[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus S 4G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S68810[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; QMobile A2_Classic Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68811[] = "Mozilla/5.0 (Linux; Android 4.0.2; Galaxy Nexus Build/ICL53F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68812[] = "Mozilla/5.0 (Linux; Android 4.0.1; Galaxy Nexus Build/ITL41F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68813[] = "Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68814[] = "Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S68815[] = "Mozilla/5.0 (Linux; Android 4.1.2; B1-710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S68816[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:23.0) Gecko/X Firefox/23.0 slurp";
const char S68817[] = "Mozilla/5.0 (Windows NT 6.1; rv:23.0) Gecko/X Firefox/23.0 slurp";
const char S68818[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:23.0) Gecko/X Firefox/23.0 slurp slurp";
const char S68819[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.7.0)";
const char S68820[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Sleipnir/3.7.3";
const char S68821[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Sleipnir/3.7.3";
const char S68822[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.7.3)";
const char S68823[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.7.1)";
const char S68824[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4 Sleipnir/3.7.4";
const char S68825[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4 Sleipnir/3.7.4";
const char S68826[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.7.4)";
const char S68827[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-D710 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68828[] = "Mozilla/5.0 (Linux; Android 4.2.1; SPH-D710 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68829[] = "LGE-AN200/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S68830[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SKYPAD Alpha 2 Build/X.01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68831[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SX-SP715A Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68832[] = "Logos/2.2.3 (Linux; U; Android 2.3.4; en-US; SX-SP715A Build/Skypad Alpha2.X.A1)";
const char S68833[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SX-SP715A Build/GRJ22)";
const char S68834[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SX-SP715A Build/GRJ22) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.0 Version/4.0 Mobile Safari/530.17";
const char S68835[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 822)";
const char S68836[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PantechP4100 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68837[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; PantechP4100 Build/IMM76I)";
const char S68838[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; KFTT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-17 12:06, +0000, gb";
const char S68839[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-17 12:06, -0500, us";
const char S68840[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 620)";
const char S68841[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 620; Vodafone)";
const char S68842[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 620; ANZ846)";
const char S68843[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT905 Build/7.7.1Q_GCIRD-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68844[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT907 Build/7.7.1Q-144_SMQ_S136-10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68845[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I905 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S68846[] = "Mozilla/5.0 (Linux; U; Android 1.6; fr-fr; Liquid Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S68847[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; Liquid Build/Donut) AppleWebkit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S68848[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A850K Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68849[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A850S Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68850[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A850L Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68851[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-N7100 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68852[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-N7105 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68853[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7100 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68854[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7105 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68855[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L900 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68856[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; SAMSUNG-GT-I9268_TD/1.0 Android/4.1.1 Release/08.30.2012 Browser/AppleWebKit534.30 Build/JRO03C) ApplelWebkit/534.30 (KHTML,like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68857[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9268 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S68858[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9268 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68859[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9268 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68860[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9268 Build/JZO54K)";
const char S68861[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9268 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S68862[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-N5120 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68863[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-N5120 Build/JDQ39)";
const char S68864[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; Orange_Boston Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S68865[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-pt; Boston Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S68866[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Orange_Boston Build/ERE27)";
const char S68867[] = "sam-r450 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S68868[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.2.1.0 Chrome/16.0.912.75 Safari/535.7";
const char S68869[] = "Opera/9.80 (Windows NT 5.0) Presto/2.12.388 Version/12.14";
const char S68870[] = "Opera/9.80 (Windows NT 5.0; Edition Yx) Presto/2.12.388 Version/12.14";
const char S68871[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A830L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68872[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A830K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68873[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A830S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68874[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A830KE Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68875[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Karbonn A4  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68876[] = "Lenovo-A820/S100 Linux/3.4.0 Android/4.1 Release/11.14.2012 Browser/AppleWebKit534.30 Profile/ Configuration/ Safari/534.30";
const char S68877[] = "Lenovo-A820t_TD/S100 Linux/3.4.0 Android/4.1 Release/1.4.2013 Browser/AppleWebKit534.30 Mobile Safari/534.30";
const char S68878[] = "NokiaC2-00/2.0 (03.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-00) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S68879[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P714 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68880[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC5-05) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S68881[] = "StumbleUpon/2.60 CFNetwork/548.1.4 Darwin/11.0.0";
const char S68882[] = "Bing/3.03 CFNetwork/548.1.4 Darwin/11.0.0";
const char S68883[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; KIRA N10040 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68884[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Woxter Tablet PC 97IPS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68885[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Woxter Tablet PC 97IPS DUAL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68886[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Woxter Tablet PC 97IPS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68887[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S68888[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S68889[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S68890[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5830L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S68891[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S68892[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S68893[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Android2.x) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S68894[] = "Mozilla/5.0 (Linux; Android 4.1.2; Micromax A116 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68895[] = "SAMSUNG-SGH-A257/A257UCIC6 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S68896[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; hu-hu; WayteQ Libra Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68897[] = "Mozilla/5.0 (Linux; U; Android 2.2; ru-ru; ZTE Libra Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68898[] = "Mozilla/5.0 (Linux; U; Android 2.2; pt-pt; a5 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68899[] = "Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; Android Edition LIBRA Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68900[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; pt-pt; a5 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68901[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; ZTE-Libra Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68902[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Desire HD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68903[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Desire HD Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68904[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Desire HD Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68905[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Desire HD Build/JZO54K)";
const char S68906[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Desire HD Build/JellyMIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68907[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5235) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile";
const char S68908[] = "Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Safari/535.19 Silk-Accelerated=false";
const char S68909[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Mi363 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68910[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Mi363/S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S68911[] = "Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 10 LINK Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S68912[] = "Mozilla/5.0 (Linux; Android 4.1.2; dtab01 Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S68913[] = "Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 7 Vogue Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S68914[] = "Mozilla/5.0 (Linux; Android 4.1.2; Telpad Dual S Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S68915[] = "Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 7 Youth Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S68916[] = "BlackBerry8700/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100";
const char S68917[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Ubuntu/10.10 Chromium/6.0.472.63 Chrome/6.0.472.63 Safari/534.3";
const char S68918[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Ubuntu/10.10 Chromium/6.0.472.63 Chrome/6.0.472.63 Safari/534.3";
const char S68919[] = "Mozilla/5.0 (compatible; Jobrapido/1.1; +http://www.jobrapido.com)";
const char S68920[] = "Mozilla/5.0 (Linux; Android 4.0.3; A13-MID Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S68921[] = "Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68922[] = "Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S68923[] = "Mozilla/5.0 (Linux; Android 4.0.4; mid-A13 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68924[] = "Mozilla/5.0 (Linux; Android 4.0.4; A13MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68925[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GEM8112 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68926[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GEM8112 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68927[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaC6-01) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S68928[] = "NokiaC6-01/UCWEB8.8.0.212/51/800";
const char S68929[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9000 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S68930[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9000 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S68931[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I739 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68932[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I739 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S68933[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-I739 Build/JZO54K)";
const char S68934[] = "Mozilla 5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I739 Build JZO54K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S68935[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; M-MP5303G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68936[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; M-MP5303G Build/JZO54K)";
const char S68937[] = "Mozilla/5.0 (IE 11.0; Windows NT 6.2; ARM; Trident/7.0; Touch; rv:11.0) like Gecko";
const char S68938[] = "Mozilla/5.0 (Windows NT 6.2; ARM; Trident/7.0; Touch; WebView/1.0; rv:11.0) like Gecko";
const char S68939[] = "Mozilla/5.0 (Linux; Android 4.1.2; Andromax U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S68940[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.1.0.0 Chrome/12.0.742.91 Safari/534.30";
const char S68941[] = "Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.1.0.0 Chrome/12.0.742.91 Safari/534.30";
const char S68942[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia2690) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S68943[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VSD220 Build/IMM76D.UI22E332_VSC) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68944[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VSD220 Build/IMM76D.UI23E802_VSC) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68945[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VSD220 Build/IMM76D.UI23EC62_VSC) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68946[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VSD220 Build/IMM76D.UI23ED12_VSC) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68947[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VSD220 Build/IMM76D.UI23ED02_VSC) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S68948[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P7500 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S68949[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7500 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S68950[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax A110 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S68951[] = "Mozilla/5.0 (Linux; Android 4.0.3; ThinkPad Tablet Build/ThinkPadTablet_A400_03) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S68952[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT881 Build/9.8.3O-118_USC-14) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S68953[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT881 Build/9.8.3O-118_USC-14) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S68954[] = "Mozilla/5.0 (Linux; Android 4.2.2; HUAWEI P6-U06 Build/HuaweiP6-U06) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S68955[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Vodafone/1.0/HTC_Kaiser/1.56.162.5";
const char S68956[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC P4550; OpVer 24.181.1.612";
const char S68957[] = "HTC_TyTN_II Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S68958[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC P4550; OpVer 24.181.9.693";
const char S68959[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC P4550; OpVer 24.181.8.683";
const char S68960[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Vodafone/1.0/HTC_Kaiser/1.56.161.4";
const char S68961[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Vodafone/1.0/HTC_Kaiser/1.56.172.3";
const char S68962[] = "HTC P4550/1.83.621.0 Mozilla/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S68963[] = "HTC-P4550-orange/PPC; 240x320; OpVer 24.181.2.731 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S68964[] = "HTC_P4550 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S68965[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Vodafone/1.0/HTC_Kaiser/1.56.165.5";
const char S68966[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; HD2)";
const char S68967[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; HD2 T8585)";
const char S68968[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.2; Trident/5.0; IEMobile/9.0;  ; LEO70)";
const char S68969[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; LEO)";
const char S68970[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; HD2 LEO)";
const char S68971[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s3653) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile";
const char S68972[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.10) Gecko/X Ubuntu/10.10 (maverick) Firefox/3";
const char S68973[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I997 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68974[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I997 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68975[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I997 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68976[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; SGH-I997 Build/JOP40D)";
const char S68977[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGH-I997 Build/JDQ39E)";
const char S68978[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-N7000 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68979[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-N7000 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S68980[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S68981[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S68982[] = "Nokia205/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia205) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S68983[] = "Nokia205.1/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia205.1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S68984[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S68985[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S68986[] = "Mozilla/5.0 (Linux; Android 4.1.1; A700 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S68987[] = "Mozilla/5.0 (Linux; Android 4.1.2; A700 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S68988[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; IM-A820L Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68989[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A820L Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S68990[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A820L Build/JZO54K; CyanogenMod-CyanogenMod 10 ) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68991[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; IM-A820L Build/Hoang Nguyen; CyanogenMod-EF84L_JellyBean) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68992[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A820L Build/Ho?ng Minh H?u _ VegavietTeam; CyanogenMod-JB_LR_VGV_V5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S68993[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus S 4G Build/JRO03R) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S68994[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0151 Build/HuaweiY300-0151) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S68995[] = "MOT-RAZRV3xv/85.97.CDP MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S68996[] = "MOT-RAZRV3xM/85.9B.E5P MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S68997[] = "MOT-RAZRV3xvM/85.97.CDP MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S68998[] = "MOT-RAZRV3x/01.2A.01 MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S68999[] = "Mozilla/5.0 (Windows NT 6.0; rv:12.0) Gecko/X Thunderbird/12.0.1";
const char S69000[] = "ZTE-TU812_TD/1.0 Linux/2.6.32 Android/2.2 Release/07.25.2011 Browser/AppleWebKit533.1";
const char S69001[] = "CoolPad8810_CMCC_TD/1.0 Linux/2.6.32 Android/2.2 Release/6.30.2011 Browser/AppleWebkit533.1";
const char S69002[] = "HuaweiT8100_TD/1.0 Android/2.2 Release/12.25.2010 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 AppleWebKit/533.1";
const char S69003[] = "CoolPad8811_CMCC_TD/1.0 Android/2.2 Release/4.10.2011 Browser/WAP 2.0 (Apple Webkit/530.18)";
const char S69004[] = "ZTE-TU802_TD/1.0 Linux/2.6.32 Android/2.2 Release/1.1.2011 Browser/AppleWebKit530.17 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S69005[] = "BLUA73/1.0 Linux/2.6.35 Android/2.2.2 Release/5.26.2011 Browser/WebKit533.1";
const char S69006[] = "eBayAndroid/2.2.1.0";
const char S69007[] = "eBayAndroid/2.2.0.25";
const char S69008[] = "4G_W4_TD/1.0 Android/2.2 MocorDroid/W11.32 Release/2011 Browser/AppleWebKit5333.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S69009[] = "TD818_TD/1.0 Android_OS/2.2 Mocor_Droid/W11 Release/2011 Browser/AppleWebKit5333.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S69010[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LT7052 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69011[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; pl-PL; rv:1.9a1) Gecko/X SeaMonkey/1.5a";
const char S69012[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8010 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S69013[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S69014[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S69015[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5502 Build/10.1.1.A.1.310) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69016[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9190 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S69017[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69018[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Hero Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69019[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Hero Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69020[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; HTC Hero Build/FRG83)";
const char S69021[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-cn; HTC Hero Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69022[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13";
const char S69023[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_2; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13";
const char S69024[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_3; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.107 Safari/534.13";
const char S69025[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13";
const char S69026[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_0; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.107 Safari/534.13";
const char S69027[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13";
const char S69028[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; Micromax P250(Funbook) Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69029[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S69030[] = "Mozilla/5.0 (Linux; U; Android 4.1.2;en-us; Lenovo_A706_ROW/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1.2 Mobile Safari/534.30";
const char S69031[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Lenovo A706_ROW Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69032[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Lenovo A706_ROW Build/JZO54K)";
const char S69033[] = "Mozilla/5.0 (Linux; Android 4.0.4; Grand X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S69034[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S69035[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69036[] = "Mozilla/5.0 (Linux; Android 4.0.4; Grand X(M) Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69037[] = "Mozilla/5.0 (Linux; Android 4.0.4; Grand X Pro Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69038[] = "Mozilla/5.0 (iPod; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko)";
const char S69039[] = "Apple-iPod2C1/803.148";
const char S69040[] = "Apple-iPod4C1/901.405";
const char S69041[] = "Apple-iPod/705.18";
const char S69042[] = "Apple-iPod4C1/902.206";
const char S69043[] = "Apple-iPod4C1/1001.523";
const char S69044[] = "gamed/4.10.17.1.6.13.5.2.1 (iPod4,1; 6.1.3; 10B329; GameKit-781.18)";
const char S69045[] = "Apple-iPod4C1/1002.329";
const char S69046[] = "Apple-iPod4C1/1001.403";
const char S69047[] = "Apple-iPod4C1/1002.144";
const char S69048[] = "Apple-iPod5C1/1002.329";
const char S69049[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SHW-M100S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69050[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M100S Build/GINGERBREAD)";
const char S69051[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; PMID700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69052[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) Maxthon AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.10";
const char S69053[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-I9300 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S69054[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9300 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S69055[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; SCH-I535 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S69056[] = "NokiaC2-06/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac2-06) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S69057[] = "Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6303iclassic) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S69058[] = "Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5130c-2) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S69059[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Mobile Safari/535.19 Silk-Accelerated=false";
const char S69060[] = "Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.7 Mobile/11A4435d Safari/8536.25";
const char S69061[] = "Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A4449d Safari/8536.25";
const char S69062[] = "Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A465 Safari/8536.25";
const char S69063[] = "Mozilla/5.0 (iPod; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A501 Safari/8536.25";
const char S69064[] = "SonyEricssonTM506/R3DA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.2";
const char S69065[] = "NokiaC5-03/UCWEB8.0.3.99/50/800";
const char S69066[] = "NokiaC5-03/UC Browser8.0.4.121/50/444/UCWEB";
const char S69067[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:6.0) Gecko/X Firefox/6.0 BLNGBAR";
const char S69068[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.3) Gecko/X Firefox/6.0 (.NET CLR 3.5.30729)";
const char S69069[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:8.0) Gecko/X Firefox/8.0 TenFourFox/G5";
const char S69070[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P7310 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S69071[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7310 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S69072[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; Aqua Wonder Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69073[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; Aqua Wonder Quad Core Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69074[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ICE 350e Build/FSV01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69075[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ICE 350e Build/FSV06) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69076[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ICE 350e Build/SKV01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69077[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ICE 350e Build/FSV07) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69078[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69079[] = "LG-GU230/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S69080[] = "SonyEricssonU8a/R2EA; Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525";
const char S69081[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-LU3700 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69082[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; LG-LU3700 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69083[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-LU3700 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69084[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; LG-LU3700 Build/FRF91)";
const char S69085[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-LU3700 Build/FRG83G)";
const char S69086[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; HERO200 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S69087[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia2690) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S69088[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC801 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69089[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570I) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S69090[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S69091[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT894 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69092[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; XT894 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69093[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT894 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/maserati";
const char S69094[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-ca; SGH-M919V Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69095[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGH-M919V Build/JDQ39)";
const char S69096[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9502 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69097[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9502 Build/JDQ39)";
const char S69098[] = "Mozilla/5.0 (Linux; U; Android 4.2.9; en-us; GT-I9502 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69099[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; pt-br; GT-I5500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69100[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; GT-I5500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69101[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; Galaxy5 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69102[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; GT-I5500 Build/GRK39F)";
const char S69103[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT-I5500 Build/GRK39F; Stock) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69104[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.2.28) Gecko/X Thunderbird/3.1.20";
const char S69105[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Maxthon/3.0)";
const char S69106[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Maxthon/3.0)";
const char S69107[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Maxthon/3.0)";
const char S69108[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Maxthon/3.0)";
const char S69109[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Arc S Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69110[] = "AppleCoreMedia/1.0.0.10A403 (iPod; U; CPU OS 6_0 like Mac OS X; en_us)";
const char S69111[] = "AppleCoreMedia/1.0.0.10A523 (iPod; U; CPU OS 6_0_1 like Mac OS X; en_us)";
const char S69112[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S69113[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S69114[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S69115[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; A18 Build/CrossCross_A18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69116[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; A18 Build/CrossCross_A18)";
const char S69117[] = "Mozilla/5.0 (Linux; Android 4.1.2; A510 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S69118[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HTC_802w Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69119[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HTC 802w Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69120[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ru-ru; HTC_One_dual_sim Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69121[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC 802w Build/JDQ39)";
const char S69122[] = "Mozilla/5.0 (Linux; Android 4.3; SGH-I337 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69123[] = "SonyEricssonW300c/R9A Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S69124[] = "SonyEricssonW300i/R4EA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S69125[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; md-us; GTablet Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69126[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GTablet Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69127[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; md-us; GTablet Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69128[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GTablet Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69129[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GTablet Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69130[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; GTablet Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69131[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; md-us; GTablet Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69132[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.7.62 Version/11.00";
const char S69133[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.7.62 Version/11.01";
const char S69134[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.4; fr; rv:1.9.0.19) Gecko/X Camino/2.0.7 (MultiLang) (like Firefox/3.0.19)";
const char S69135[] = "Opera/9.80 (Android 4.0.4; Linux; Opera Mobi/ADR-X; U; en) Presto/2.8.149 Version/11.10";
const char S69136[] = "Opera/9.80 (Android 4.0.3; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.8.149 Version/11.10";
const char S69137[] = "Opera/9.80 (Android 4.0; Linux; Opera Mobi/ADY-X; U; ru) Presto/2.8.149 Version/11.10";
const char S69138[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.7 (KHTML, like Gecko) Chrome/20.0.1099.0 Safari/536.7 QQBrowser/6.14.15493.201";
const char S69139[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C) QQBrowser/6.14.14517.201";
const char S69140[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/536.25 (KHTML, like Gecko) FluidApp Version/1.6 Safari/536.25";
const char S69141[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) FluidApp Version/1.6.1 Safari/536.26.14";
const char S69142[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) FluidApp Version/1.6.1 Safari/536.28.10";
const char S69143[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) FluidApp Version/1.6.1 Safari/536.30.1";
const char S69144[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC-T528d/1.05.1401.1) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69145[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T528d Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S69146[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC-T528d/1.19.1401.7) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69147[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T528d) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69148[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC-T528d/2.14.1401.1) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69149[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC-T528d/1.19.1401.2) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69150[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC-T528d/1.05.1401.2) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69151[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC-T528d/2.18.1401.1) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69152[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC T528d Build/IMM76D)";
const char S69153[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; da-dk; TAD-10023 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69154[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PAP5000TDUO Build/PrestigioPAP5000TDUO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69155[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; PAP5000TDUO Build/PrestigioPAP5000TDUO)";
const char S69156[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu/12.10 Chromium/27.0.1453.6 Chrome/27.0.1453.6 Safari/537.36";
const char S69157[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu/12.10 Chromium/27.0.1453.6 Chrome/27.0.1453.6 Safari/537.36";
const char S69158[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Huawei; H883G; HuaweiH883G)";
const char S69159[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Mobile Safari/535.19 Silk-Accelerated=false";
const char S69160[] = "Mozilla/5.0 (Linux; Android 4.0.4; Torque Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S69161[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P713 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69162[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P713GO Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69163[] = "SAMSUNG-GT-B6520/1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S69164[] = "Mozilla/5.0 (Series40; NokiaX2-01/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S69165[] = "Mozilla/5.0 (Series40; NokiaX2-01.1/07.22; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S69166[] = "Mozilla/5.0 (Linux; Android 4.0.3;  A510 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S69167[] = "Mozilla/5.0 (Linux; Android 4.0.3; A510 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S69168[] = "Mozilla/5.0 (Windows NT 5.1; rv:13.0) Gecko/X Thunderbird/13.0.1";
const char S69169[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; TAB-PROTAB25 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69170[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT881 Build/6.7.3-130_YUS-11) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69171[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P713 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69172[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P713GO Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69173[] = "Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S69174[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Pico_A310b; en-au) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S69175[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC/Explorer/1.17.161.3; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S69176[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC/Explorer/1.29.161.3; it-it) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S69177[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC/Explorer/1.41.161.3; es-es) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S69178[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus S Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69179[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus S Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69180[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0000, gb";
const char S69181[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-P5110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.27/381,1.9.27.381,2013-03-21 11:39, +0100, fr";
const char S69182[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-nz; ALCATEL ONE TOUCH 922 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69183[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5110 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S69184[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S69185[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly s Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69186[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; tr-tr; LG-P503 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S69187[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; LG-P503 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S69188[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; tr-tr; LG-P503 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S69189[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; LG-P503 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S69190[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; LG-P503 Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S69191[] = "Nokia5800w XpressMusic/UCWEB7.9.0.94/50/999";
const char S69192[] = "Nokia5800 XpressMusic/UCWEB7.9.0.94/50/999";
const char S69193[] = "Nokia5800 XpressMusic/UC Browser7.9.0.102/50/355/UCWEB";
const char S69194[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11 Comodo_Dragon/20.1.1.0";
const char S69195[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaC5-05)/UC Browser8.5.0.183/50/355/UCWEB Mobile";
const char S69196[] = "Mozilla/5.0 (Linux; Android 4.0.4; PadFone 2 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69197[] = "Mozilla/5.0 (Linux; Android 4.0.4; PadFone 2 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S69198[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7500 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S69199[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P7500 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S69200[] = "Mozilla/5.0 (Linux; Android 4.0.4; C525c Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S69201[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; P774A Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69202[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; P774A Build/JRO03H)";
const char S69203[] = "Mozilla/5.0 (Linux; Android 4.3; One X Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69204[] = "Mozilla/5.0 (Linux; Android 4.3; HTC One X Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69205[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; LG-LU6500 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69206[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-LU6500 Build/GRJ22)";
const char S69207[] = "NokiaX6-00/UCWEB8.5.0.163/50/800";
const char S69208[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaX6-00)/UC Browser8.5.0.183/50/355/UCWEB Mobile";
const char S69209[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; MID1125 Build/GreenDemon_v1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69210[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T1Ci Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69211[] = "TECHAIN50_11B_HW (MRE\\3.0.00(2500) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) Cherry_W7_T712V_M01_S10_X Release/2012.10.25 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US";
const char S69212[] = "TECHAIN50_11B_HW (MRE/3.0.00(2500);MAUI/y_W7_T712V_M01_S13_X;BDATE/2013/01/30 15:29;LCD/X;CHIP/MT6250;KEY/Reduced;TOUCH/1;CAMERA/0;SENSOR/0;DEV/TECHAIN50_11B_HW;WAP Browser/MAUI (HTTP PGDL;HTTPS);GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/005;GAMELOFT/00) Cherry_W7_T712V_M01_S13_X Release/2013.01.30 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US";
const char S69213[] = "TECHAIN50_11B_HW (MRE\\3.0.00(2500) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) Cherry_W7_T712V_M01_S06_X Release/2012.09.28 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US";
const char S69214[] = "TECHAIN50_11B_HW (MRE\\3.0.00(2500) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) Cherry_W7_T712V_M01_S11S_X Release/2012.11.23 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US";
const char S69215[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZP300+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69216[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Lenovo K860 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69217[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Lenovo K860i Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69218[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Lenovo K860i Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S69219[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Lenovo K860 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S69220[] = "iTunes-iPod/6.1.3 (4; 8GB; dt:71)";
const char S69221[] = "iTunes-iPod/6.1.3 (4; 32GB; dt:81)";
const char S69222[] = "iTunes-iPod/6.1.3 (4; 32GB; dt:71)";
const char S69223[] = "iTunes-iPod/6.1.3 (4; 16GB; dt:71)";
const char S69224[] = "iTunes-iPod/6.1.3 (4; 64GB; dt:71)";
const char S69225[] = "iTunes-iPod/6.1.3 (4; 64GB; dt:81)";
const char S69226[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S69227[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E975K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S69228[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S69229[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; NexusHD2 Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69230[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; NexusHD2 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69231[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; TouchWizMOD Build/TouchWizMOD_HD2; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69232[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; XBLWP7; ZuneWP7) UCBrowser/3.0.0.285";
const char S69233[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; XBLWP7; ZuneWP7) UCBrowser/3.0.1.302";
const char S69234[] = "Nokia6021/2.0 (04.90) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S69235[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Xoom MZ605 Build/H.6.1-29) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S69236[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; MZ605 Build/H.6.1-32) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S69237[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; MZ605 Build/H.6.1-32-1) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S69238[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; MZ605 Build/H.6.1-17) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S69239[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; pt-br; MZ605 Build/H.6.1-38.82) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S69240[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; MZ605 Build/H.6.1-25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S69241[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; pt-br; MZ605 Build/H.6.1-38.4) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S69242[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; MZ605 Build/H.6.1-40) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S69243[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; es-us; MZ605 Build/H.6.1-38.8) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S69244[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; VL-107 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69245[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VL-107 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69246[] = "Opera/10.53 (Windows NT 5.1; U; en)";
const char S69247[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; M-MP720I Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69248[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Butterfly Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S69249[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Butterfly Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S69250[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L720 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us";
const char S69251[] = "Mozilla/5.0 (Linux; Android 4.0.3; A501 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S69252[] = "SAMSUNG-B5702/B5702JVIK1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S69253[] = "Opera/9.80 (Android 4.0.3; Linux; Opera Tablet/ADY-X; U; ru) Presto/2.9.201 Version/11.50";
const char S69254[] = "Opera/9.80 (Android 4.0.4; Linux; Opera Tablet/ADR-X; U; en-GB) Presto/2.9.201 Version/11.50";
const char S69255[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; K1 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69256[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; K1 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69257[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; K1 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69258[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Xelio 7 pro Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69259[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:20.0) Gecko/X Firefox/20.0";
const char S69260[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST25i Build/6.0.B.3.188) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S69261[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST25i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S69262[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST25i Build/6.0.B.1.564) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S69263[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Aqua.Active Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69264[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-US780 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S69265[] = "Mozilla/5.0 (Linux; Android 4.0.3; ThinkPad Tablet Build/ThinkPadTablet_A400_03) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S69266[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I527 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69267[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SAMSUNG-SGH-I527 Build/JDQ39)";
const char S69268[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-ca; SGH-I527M Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69269[] = "Mozilla/5.0 (Series40; NokiaC2-00/03.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S69270[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P875 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69271[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; W900 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69272[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T989 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69273[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-M830 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S69274[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P930 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S69275[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P5113 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S69276[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5113 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S69277[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AX2_Frenzy Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69278[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Videocon A15 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69279[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6506 Build/10.3.A.0.423) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S69280[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6506 Build/10.3.A.0.423) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S69281[] = "Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S69282[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E450f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S69283[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E450 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S69284[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E450g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S69285[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E450B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S69286[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-gb; T-Mobile G1 Build/CRC37) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S69287[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; T-Mobile G1 Build/CRB43) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S69288[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; T-Mobile G1 Build/CRC1) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S69289[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-gb; T-Mobile G1 Build/CRB17) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S69290[] = "Mozilla/5.0 (Linux; U; Android 1.5; es-; T-Mobile G1 Build/CRB43) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S69291[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; T-Mobile G1 Build/CRB43) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari 525.20.1";
const char S69292[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB860 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69293[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB860 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69294[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB860 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69295[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB860 Build/MROM; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69296[] = "Mozilla/5.0 (Windows NT 4.10; rv:11.0) Gecko/X Firefox/11.0";
const char S69297[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; M030 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69298[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; M030 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S69299[] = "SAMSUNG-SGH-I747/100.40004";
const char S69300[] = "SAMSUNG-SGH-I747M/100.40004";
const char S69301[] = "SAMSUNG-SPH-L710/100.40102";
const char S69302[] = "[FBAN/FB4A;FBAV/2.3;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/AT&amp-T;FBPN/com.facebook.katana;FBDV/SAMSUNG-SGH-I747;FBSV/4.1.1;]";
const char S69303[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; AT100 Build/JZO54K; CyanogenMod-CM10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69304[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mauie800) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S69305[] = "UCWEB/2.0 (Java; U; MIDP-2.0; id; mauie800) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S69306[] = "UCWEB/2.0 (Java; U; MIDP-2.0; vi; mauie800) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S69307[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mauie800) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0";
const char S69308[] = "UCWEB/2.0 (Java; U; MIDP-2.0; ar-SA; mauie800) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S69309[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; U30GT 2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69310[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Pulse Mini Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S69311[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Pulse Mini Build/ERE27)";
const char S69312[] = "Mozilla/5.0 (Windows NT 4.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.187 Safari/535.1";
const char S69313[] = "JUC(Linux;U;Android2.3.6;Zh_cn;GT-S5830i;320*480;)UCWEB7.8.0.95/139/444";
const char S69314[] = "Mozilla/5.0(Linux;U;Android2.3.6;Zh_cn;GT-S5830i;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S69315[] = "Mozilla/5.0 (Series40; NokiaC2-05/08.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.11";
const char S69316[] = "Mozilla/5.0 (Series40; NokiaC2-05.1/08.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.11";
const char S69317[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5";
const char S69318[] = "Nokia300/5.0 (07.44) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia300) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S69319[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia500) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S69320[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S69321[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S69322[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGP321 Build/10.3.1.A.0.244) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S69323[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One SV Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69324[] = "SonyEricssonTM717/R1FC Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3";
const char S69325[] = "BlackBerry8520/5.2.0.67 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S69326[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; fr-FR; rv:1.0.2) Gecko/X Netscape/7.02";
const char S69327[] = "Mozilla/5.0 (X11; FreeBSD i386; rv:10.0.3) Gecko/X Firefox/10.0.3";
const char S69328[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; Nokia;N70)";
const char S69329[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s5253) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S69330[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-SU870 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69331[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S69332[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One VX Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S69333[] = "Mozilla/5.0 (Linux; U; Android 4.1;en-us; ZTE V970M Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30;";
const char S69334[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZTE V970 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69335[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ZTE V970 Build/JRO03C)";
const char S69336[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ZTE V970 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69337[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI 1S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31";
const char S69338[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI 1SC Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S69339[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PAP4055DUO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69340[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I467 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S69341[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9082 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S69342[] = "Mozilla/5.0 (Linux; Android 4.2.2; TouchPad Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S69343[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LG-P500 Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69344[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P500 Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69345[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LG-P500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69346[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-P500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69347[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P500 Build/IMM76L)";
const char S69348[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.0.1) Gecko/X Netscape/7.0";
const char S69349[] = "Mozilla/7.0 (Windows; U; Windows NT 5.1; en-US; rv:0.9.2) Gecko/X Netscape/7.0";
const char S69350[] = "Mozilla/5.0 (Linux;U; Android 4.0.4; en-us; MP1010 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69351[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69352[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69353[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-gb; GT-S5360 Build/JELLY BEAN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69354[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-LU6200 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69355[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-lu6200 Build/JZO54K)";
const char S69356[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGH-M919N Build/JDQ39)";
const char S69357[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-M919N Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69358[] = "Alcatel-OT-808/010_02 1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S69359[] = "Alcatel-OT-808/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A, Alcatel-OT-808/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S69360[] = "Alcatel-OT-808A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S69361[] = "Alcatel-OT-808/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S69362[] = "TELECA-/2.0 (BREW 3.1.5; U; EN-US; SAMSUNG; SPH-M810; Teleca/Q05A/INT) MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S69363[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PC36100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69364[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PC36100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69365[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69366[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PC36100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69367[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; EVO 4G Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69368[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; PC36100 Build/MIUI)";
const char S69369[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; EVO 4G Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S69370[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/5.0 (Windows; U; Windows NT 5.1; en) AppleWebKit/886; U; en) Presto/2.4.15";
const char S69371[] = "Opera/9.80 (Windows NT 5.1; U; Opera/9.80 (J2ME/MIDP; Opera Mini/5.0.18635/1030; U; en) Presto/2.4.15; ru) Presto/2.8.99 Version/11.10";
const char S69372[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/5.0(Windows; U; Windows NT 5.1; en-US)/23.390; U; en) Presto/2.5.25 Version/10.54";
const char S69373[] = "Nokia3110c/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia3110c) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S69374[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB855 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69375[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB855 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69376[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MB855 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69377[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB855 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69378[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I915 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S69379[] = "Mot-WX292/18.01 Browser/Obigo/Q03C MMP/2.0";
const char S69380[] = "MOT-WX416/03.03.0F Obigo/Q03C MMP/2.0";
const char S69381[] = "Mot-WX294/18.01 Browser/Obigo/Q03C MMP/2.0";
const char S69382[] = "Mot-WX306/1B.3 Browser/Obigo/Q03C MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S69383[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; Fly IQ245 Plus Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69384[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; Fly IQ245Plus Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69385[] = "Mozilla/5.0 (Linux; Android 4.0.4; LIFETAB_S9714 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S69386[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 800;7.10;8858)";
const char S69387[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Nokia 800;7.10;8862)";
const char S69388[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 800;7.10;8862)";
const char S69389[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 800;7.10;8860)";
const char S69390[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; AT-AS53N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69391[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; AT-AS53N Build/JZO54K)";
const char S69392[] = "Mozilla/5.0 (Linux; Android 4.0.1; Galaxy Nexus Build/ITL41F) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S69393[] = "Mozilla/5.0 (Linux; Android 4.0.2; Galaxy Nexus Build/ICL53F) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S69394[] = "Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S69395[] = "Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S69396[] = "SAMSUNG-SGH-A777/A777UCIF2 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S69397[] = "Mozilla/5.0 (Linux; U; Android 1.0.1; ld-us; A70CHT Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S69398[] = "Mozilla/5.0 (Linux; U; Android 1.0.3; ld-us; A70CHT Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S69399[] = "Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Safari/535.19 Silk-Accelerated=true";
const char S69400[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S69401[] = "SAMSUNG-GT-E1500/E1500JVLH6 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S69402[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Scroll Plus Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69403[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-D710 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S69404[] = "Mozilla/5.0 (Linux; Android 4.2.1; SPH-D710 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S69405[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGP321 Build/10.3.1.A.0.244) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S69406[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8730 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S69407[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8730T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S69408[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-i705 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S69409[] = "Mozilla/5.0 (HW-HUAWEI_U6150;U;REX/1.0;BREW/5.0.4.1;Profile/MIDP-2.0 Configuration/CLDC-1.1;320*240) Opera/9.70 (KHTML, like Gecko) Version/7.0 Mobile Opera/7.0";
const char S69410[] = "Mozilla/5.0 (Series40; NokiaC2-02.1/so7.22; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.12";
const char S69411[] = "Mozilla/5.0 (Series40; NokiaC2-02/06.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15";
const char S69412[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-T959 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69413[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LNV-Lenovo A790e Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69414[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LNV-Lenovo A790e Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S69415[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LNV-Lenovo A790e Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S69416[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; LNV-Lenovo A790e Build/GINGERBREAD)";
const char S69417[] = "Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; LNV-Lenovo A790e Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S69418[] = "Mozilla/5.0 (Linux;U; Android 4.0.4; en-us; MP700 Build/IMM76D) AppleWebKit/534.30 (KHTML,like Gecko) Version/4.0 Safari/534.30";
const char S69419[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Ray Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69420[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035E Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69421[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035X Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69422[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69423[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69424[] = "Mozilla/5.0 (compatible; MSIE 7.0; 68K; Tablet PC; Trident/3.0)";
const char S69425[] = "Mozilla/5.0 (compatible; MSIE 7.0; WindowsCE; Trident/2.0)";
const char S69426[] = "Mozilla/5.0 (compatible; MSIE 7.0; PPC; Tablet PC; Trident/2.0)";
const char S69427[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S69428[] = "Mozilla/5.0 (Linux; Android 4.0.4; DA220HQL Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S69429[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC Sensation Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/pyramid";
const char S69430[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC Sensation Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/pyramid";
const char S69431[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC Sensation Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/pyramid";
const char S69432[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC Sensation Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/pyramid";
const char S69433[] = "SonyEricssonW960i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 768) Opera 8.65 [en]";
const char S69434[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2_1 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B405 Twitter for iPad";
const char S69435[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B500 Twitter for iPad";
const char S69436[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B367 Twitter for iPad";
const char S69437[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9) Gecko/X Firefox/3.0 (Linux Mint)";
const char S69438[] = "Mozilla/5.0 (X11; U; Linux i686; it-IT; rv:1.9.0.1) Gecko/X Firefox/3.0.1 (Linux Mint)";
const char S69439[] = "Mozilla/5.0 (X11; U; Linux i686; en-GB; rv:1.9) Gecko/X Firefox/3.0 (Linux Mint)";
const char S69440[] = "Mozilla/5.0 (Linux; Android 4.0.4;  SGH-I897 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S69441[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I897 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69442[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-I897 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69443[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s5253) UCBrowser8.3.0.154/69/352/UCWEB Mobile";
const char S69444[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5253) UCBrowser8.3.1.161/69/405/UCWEB Mobile";
const char S69445[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; STM803HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69446[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; STM803HCZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69447[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; M712 Build/HRA701S) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69448[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; M712 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69449[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; M712 Build/M712) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69450[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M712 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69451[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8013 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S69452[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; QMobile A12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69453[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; QMobile_A12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69454[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3113 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S69455[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P3113 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S69456[] = "Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF700T Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S69457[] = "Mozilla/5.0 (Linux; Android 4.1.2; AS870 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69458[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P714 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69459[] = "Mozilla/5.0 (Windows NT 4.0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S69460[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i919 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69461[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i919 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S69462[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i919 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S69463[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I919U Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69464[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-I919U Build/GINGERBREAD)";
const char S69465[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I919U Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S69466[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I779 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69467[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I779 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S69468[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-I779 Build/GINGERBREAD)";
const char S69469[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69470[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:17.0) Gecko/X Firefox/17.0 SeaMonkey/2.14.1";
const char S69471[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; X10.Dual Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69472[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; X10.Dual+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69473[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT907 Build/7.7.1Q-144_SMQ_S136-10) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S69474[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT907 Build/7.7.1Q-144_SMQ_S136-10) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S69475[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT905 Build/7.7.1Q_GCIRD-16) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S69476[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT905 Build/7.7.1Q_GCIRD-16) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S69477[] = "Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_E10310 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S69478[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810P Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S69479[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810P Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S69480[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8010 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S69481[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N8010 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S69482[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8260 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69483[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT916 Build/2_26D_2014) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69484[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Comodo_Dragon/10.0.0.2 Chrome/10.0.648.204 Safari/534.16";
const char S69485[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/69/444";
const char S69486[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/28/444";
const char S69487[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/27/444";
const char S69488[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/50/444";
const char S69489[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/50/352";
const char S69490[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/50/351";
const char S69491[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.2.91/50/355";
const char S69492[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/28/351";
const char S69493[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/28/352";
const char S69494[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/27/351";
const char S69495[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/27/352";
const char S69496[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/50/354";
const char S69497[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/50/350";
const char S69498[] = "Bing%20for%20iPad/1.1.3 CFNetwork/548.1.4 Darwin/11.0.0";
const char S69499[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SHW-M220L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69500[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHW-M220L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69501[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.8.1.14) Gecko/X Firefox/2.0.0.14 Flock/1.2.1";
const char S69502[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; MTS-SP101 Build/HuaweiC8511) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69503[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-KU5900 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69504[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-KU5900 Build/GRJ22)";
const char S69505[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/528.5 (KHTML, like Gecko) Iron/0.4.155.0 Safari/528.5";
const char S69506[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/528.4 (KHTML, like Gecko) Iron/0.3.155.0 Safari/X.528";
const char S69507[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Iron/0.2.152.0 Safari/X.525";
const char S69508[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN95)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S69509[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN95 8GB)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S69510[] = "Mozilla/5.0 (Java; U; en-us; nokian95) UCBrowser8.2.1.144/69/352/UCWEB Mobile";
const char S69511[] = "Mozilla/5.0 (Java; U; en-us; nokian95_8gb) UCBrowser8.2.0.132/69/452/UCWEB Mobile";
const char S69512[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Malata SMBA1002 Build/JRO03L; CyanogenMod-TEAMDRH) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69513[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Malata SMBA1002 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69514[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-KU5900 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69515[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-KU5900 Build/IMM76L)";
const char S69516[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; ATP7683 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69517[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PG86100 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69518[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Evo 3D CDMA Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69519[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Evo 3D GSM Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69520[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I437 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S69521[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I717 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S69522[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I717 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S69523[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 Sleipnir/4.1.0";
const char S69524[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 Sleipnir/4.1.3";
const char S69525[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36 Sleipnir/4.1.4";
const char S69526[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36 Sleipnir/4.1.4";
const char S69527[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S69528[] = "Mozilla/5.0 (Linux; Android 4.0.3; Le Pan S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S69529[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LS860 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S69530[] = "Mozilla/5.0 (Linux; Android 4.0.4; Mach Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S69531[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5530) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S69532[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9190 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S69533[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9190 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S69534[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S69535[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S69536[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S7562 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S69537[] = "Mozilla/4.0 SonyEricssonW715v/R1GA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.4";
const char S69538[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 900)";
const char S69539[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; M3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69540[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; M3 media box board Build/X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69541[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; m3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69542[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; M3 Atom Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69543[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Aqua Sx Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69544[] = "Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia5130c-2) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S69545[] = "Nokia5130/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia5130) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S69546[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Elcano Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69547[] = "Mozilla/5.0 (Windows NT 6.1; rv:19.0) Gecko/X Firefox/19.0 slurp";
const char S69548[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:19.0) Gecko/X Firefox/19.0 slurp";
const char S69549[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:19.0) Gecko/X Firefox/19.0 slurp slurp";
const char S69550[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST27i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S69551[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST27a Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S69552[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8013 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S69553[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.3.4641 Chrome/7.0.517.450 Safari/534.7";
const char S69554[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.3.4628 Chrome/7.0.517.450 Safari/534.7";
const char S69555[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.3.4628 Chrome/7.0.517.450 Safari/534.7";
const char S69556[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.0.4568 Chrome/7.0.517.440 Safari/534.7";
const char S69557[] = "NokiaC6-00/UC Browser8.0.4.121/50/444/UCWEB";
const char S69558[] = "NokiaC6-00/UCWEB8.0.3.99/50/800";
const char S69559[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaN97) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/444/UCWEB Mobile";
const char S69560[] = "Mozilla/5.0 (Java; U; en-us; nokian97-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S69561[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; CROSS A10 Build/VENUS_00.03.109.H1059) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69562[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; CROSS A10 Build/VENUS_00.03.133.H1059) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69563[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; CROSS A10 Build/VENUS_00.03.161.H1059) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69564[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; CROSS A10 Build/VENUS_00.03.161.25.02.H1059) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69565[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S69566[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S69567[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S69568[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P970 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S69569[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P970h Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S69570[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire 600 dual sim Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69571[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire 600 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69572[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; Commtiva-N700 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69573[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Commtiva-N700 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69574[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Commtiva-N700 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69575[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; Commtiva-N700 Build/FRF91)";
const char S69576[] = "LGE-VM265/1.0 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0";
const char S69577[] = "Nokia5230/UCWEB7.9.0.94/50/999";
const char S69578[] = "Nokia5230/UC Browser7.9.0.102/50/355/UCWEB";
const char S69579[] = "Nokia5230-c/UC Browser7.9.0.102/50/355/UCWEB";
const char S69580[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en; rv:1.9.0.19) Gecko/X Camino/2.0.8 (MultiLang) (like Firefox/3.0.19)";
const char S69581[] = "NokiaX6-00/UCWEB8.2.0.116/50/800";
const char S69582[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaX6-00)/UC Browser8.2.0.132/50/355/UCWEB Mobile";
const char S69583[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; INFOBAR C01 Build/S4030) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69584[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; INFOBAR C01 Build/S2200) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69585[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; INFOBAR C01 Build/S1190) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69586[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; INFOBAR C01 Build/S6290) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69587[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; INFOBAR C01 Build/S2270) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69588[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-pt; tmn smart a7 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69589[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaN8-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile";
const char S69590[] = "Opera/9.80 (Windows NT 5.0) Presto/2.12.388 Version/12.12";
const char S69591[] = "Opera/9.80 (Windows NT 5.0; Edition Yx) Presto/2.12.388 Version/12.12";
const char S69592[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L900 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SPH-L900, 42, 1.9.1)";
const char S69593[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SAMSUNG-SGH-I317, 42, 1.9.1)";
const char S69594[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T889 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SGH-T889, 42, 1.9.1)";
const char S69595[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I317M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SGH-I317M, 42, 1.9.1)";
const char S69596[] = "Mozilla/5.0(iPad; U;CPU OS 5 like Mac OS X; zh-CN; iPad2,1) AppleWebKit/534.46 (KHTML, like Gecko) UCBrowser/2.0.1.280 U3/0.8.0 Safari/7543.48.3";
const char S69597[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S69598[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I535 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S69599[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; LG-P716 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69600[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P716 Build/JZO54K)";
const char S69601[] = "Nokia112/2.0 (03.32) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia112) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S69602[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337M Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S69603[] = "Nokia6681/2.0 (6.09.00) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S69604[] = "Opera/9.80 (Windows NT 6.2; U; en) Presto/2.9.168 Version/11.51";
const char S69605[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; LG-LU6800 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69606[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-LU6800 Build/GRJ22)";
const char S69607[] = "SAMSUNG-GT-S5610K/S5610KDDKK2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S69608[] = "Samsung5610/3.83; Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S69609[] = "SAMSUNG-GT-S5610/S5610XXLF4 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S69610[] = "NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax3-02) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S69611[] = "NokiaX3-02.5/5.0 (06.75) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax3-02.5) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S69612[] = "Mercury/7.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S69613[] = "Mozilla/5.0 (Linux; Android 4.0.4; SO-01E Build/9.0.G.0.247) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69614[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S69615[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S69616[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-N7000 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S69617[] = "sprd-TSM_T3/1.0 Linux/2.6.35.7 Android/2.3.7 Release/01.07.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S69618[] = "sprd-TSM_T3/1.0 Linux/2.6.35.7 Android/2.3.7 Release/02.28.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S69619[] = "sprd-TSM_T3/1.0 Linux/2.6.35.7 Android/2.3.7 Release/01.23.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S69620[] = "sprd-TSM_T3/1.0 Linux/2.6.35.7 Android/2.3.7 Release/04.22.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S69621[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; TSM_T3 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69622[] = "sprd-TSM_T3/1.0 Linux/2.6.35.7 Android/2.3.7 Release/05.03.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S69623[] = "sprd-TSM_T3/1.0 Linux/2.6.35.7 Android/2.3.7 Release/04.07.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S69624[] = "sprd-TSM_T3/1.0 Linux/2.6.35.7 Android/2.3.7 Release/06.09.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S69625[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; M-MP855i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69626[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-M919 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us";
const char S69627[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaX6-00) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S69628[] = "Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80G9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S69629[] = "Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 80G9 Build/MR1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S69630[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S69631[] = "Mozilla/5.0 (Linux; Android 4.3; Transformer TF101 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S69632[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SK-S100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69633[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SK-S100 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69634[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; SK-S100 Build/FRG83G)";
const char S69635[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:6.0.2) Gecko/X Firefox/6.0.2 AlexaToolbar/alxf-2.15";
const char S69636[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:6.0.2) Gecko/X Firefox/6.0.2 Firefox/6.0.2";
const char S69637[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST15i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69638[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST15a Build/4.1.B.0.431) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69639[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; PP4MT-9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69640[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GEM10312S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69641[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GEM10312S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69642[] = "OneBrowser/3.1 (Nokia5130c-2/07.95)";
const char S69643[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; N-03E Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69644[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; N-03E Build/AX)";
const char S69645[] = "Mozilla/5.0 (Windows NT 6.0; rv:17.0) Gecko/X Firefox/17.0 IceDragon/18.0.1.0";
const char S69646[] = "Mozilla/5.0 (Windows NT 6.0; rv:18.0) Gecko/X Firefox/18.0 IceDragon/18.0.3.1";
const char S69647[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; PAD-FMD700P Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69648[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; PAD-FMD700HD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69649[] = "HUAWEI_T8833_TD/1.0 Android/4.0.4 (Linux; U; Android 4.0.4; zh-cn) Release/05.31.2012 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30";
const char S69650[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8833 Build/HuaweiT8833) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69651[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8833 Build/HuaweiT8833) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S69652[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI T8833 Build/HuaweiT8833)";
const char S69653[] = "Mozilla/5.0 (Linux; Android 4.1.1; CINK PEAX Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69654[] = "Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S69655[] = "Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S69656[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S69657[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S69658[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P715 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69659[] = "Mozilla/5.0 (Series40; Nokia300/07.57; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S69660[] = "MOT-Q9 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 320x240/01.06.08R) Opera 8.65 [en]";
const char S69661[] = "MOT-Q9/01.09.28R Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 320x240) Opera 8.65";
const char S69662[] = "MOT-Q9/01.07.37R Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 240x320) Opera 8.65";
const char S69663[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.4.121/28/351/UCWEB";
const char S69664[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.4.121/50/351/UCWEB";
const char S69665[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.4.121/28/352/UCWEB";
const char S69666[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/50/444/UCWEB";
const char S69667[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/50/355/UCWEB";
const char S69668[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.4.121/50/444/UCWEB";
const char S69669[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/28/444/UCWEB";
const char S69670[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/50/351/UCWEB";
const char S69671[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/28/352/UCWEB";
const char S69672[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/50/354/UCWEB";
const char S69673[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/28/354/UCWEB";
const char S69674[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.4.121/50/352/UCWEB";
const char S69675[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/50/352/UCWEB";
const char S69676[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.4.121/28/444/UCWEB";
const char S69677[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.4.121/50/355/UCWEB";
const char S69678[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; ADM712HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69679[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Luna TAB274 Build/LunaTAB274) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69680[] = "UCWEB/2.0(BlackBerry; U; 9.32.0.88; en-us; 9380/9.32.0.88) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S69681[] = "UCWEB/2.0(BlackBerry; U; 9.32.0.51; en-us; 9380/9.32.0.51) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S69682[] = "UCWEB/2.0(BlackBerry; U; 9.32.0.59; en-us; 9380/9.32.0.59) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S69683[] = "UCWEB/2.0(BlackBerry; U; 9.32.0.40; en-us; 9380/9.32.0.40) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S69684[] = "UCWEB/2.0(BlackBerry; U; 9.32.0.143; en-us; 9380/9.32.0.143) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S69685[] = "UCWEB/2.0(BlackBerry; U; 9.32.0.114; en-us; 9380/9.32.0.114) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S69686[] = "UCWEB/2.0(BlackBerry; U; 9.32.0.144; en-us; 9380/9.32.0.144) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S69687[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; POV_TAB-P701 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69688[] = "Mozilla/5.0 (Windows NT 6.0; rv:20.0) Gecko/X Firefox/20.0 IceDragon/20.0.1.14";
const char S69689[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P895 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S69690[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P895qb Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S69691[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; YP-GB70 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69692[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; YP-GB70 Build/FROYO)";
const char S69693[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; YP-GB70 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69694[] = "NewzCrawler/1.9 (compatible; MSIE 6.00; Newz Crawler 1.9; http://www.newzcrawler.com/ )";
const char S69695[] = "Mozilla/4.0 (compatible; MSIE 6.0; Sitekit.net Linkchecker Bot;";
const char S69696[] = "Mozilla/4.0 (compatible; MSIE 6.0) WormlyBot; (+http://wormly.com)";
const char S69697[] = "Mozilla/5.0 (compatible; BecomeBot/2.x; MSIE 6.0 compatible; http://www.become.com/site_owners.html)";
const char S69698[] = "Mozilla/5.0 (compatible; MSIE 6.0; Podtech Network; crawler_admin@podtech.net)";
const char S69699[] = "Mozilla/5.0 (compatible; BecomeJPBot/2.3; MSIE 6.0 compatible; +http://www.become.co.jp/site_owners.html)";
const char S69700[] = "Mozilla/5.0 (compatible; BecomeBot/1.xx; MSIE 6.0 compatible; http://www.become.com/webmasters.html)";
const char S69701[] = "Mozilla/5.0 (compatible; iaskspider/1.0; MSIE 6.0)";
const char S69702[] = "IRLbot/2.0 (compatible; MSIE 6.0; http://irl.cs.tamu.edu/crawler)";
const char S69703[] = "Opera/9.52 (Windows NT 5.0; U; sv)";
const char S69704[] = "Opera/9.51 (Windows NT 5.0; U; en)";
const char S69705[] = "Opera/9.50 (Windows NT 5.0; U; en)";
const char S69706[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/9.80 (S60; SymbOS; Opera Mobi/23.348; U; en) Presto/2.5.25 Version/10.54";
const char S69707[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MID74C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69708[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID1065 Build/ICS.MID1065.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69709[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S69710[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69711[] = "Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ Epiphany/2.30.6";
const char S69712[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; 101K Build/155.0.0228) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69713[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; 101K Build/158.2.0237) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69714[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; 101K Build/159.3.0247) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69715[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; 101K Build/160.4.0252) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69716[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; 101K Build/160.4.0252)";
const char S69717[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; 101K Build/161.5.0254) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69718[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; 101K Build/162.6.0256) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69719[] = "Opera/5.40 (Windows NT 5.1; U; pl) Version/10.5";
const char S69720[] = "Opera/5.10 (Windows NT 5.1; U; pl) Version/10.5";
const char S69721[] = "Opera/5.30 (Windows NT 5.1; U; pl) Presto/2.3.15 Version/10.10";
const char S69722[] = "Opera/5.30 (Windows NT 5.1; U; pl) Version/10.5";
const char S69723[] = "Opera/5.20 (Windows NT 5.1; U; pl) Version/10.5";
const char S69724[] = "Opera/5.10 (Windows NT 5.1; U; pl) Presto/2.1.15 Version/10.10";
const char S69725[] = "Opera/5.40 (Windows NT 5.1; U; pl) Presto/2.4.15 Version/10.10";
const char S69726[] = "Opera/5.20 (Windows NT 5.1; U; pl) Presto/2.2.15 Version/10.10";
const char S69727[] = "JUC (Linux; U; 2.3.4; zh-cn; WT19i; 320*480) UCWEB7.9.0.94/139/444";
const char S69728[] = "JUC (Linux; U; 4.0.4; zh-cn; WT19i; 320*480) UCWEB7.9.0.94/139/444";
const char S69729[] = "Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Safari/535.19 Silk-Accelerated=false";
const char S69730[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-US730 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S69731[] = "AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; GT-P6210 Build/JRO03H)";
const char S69732[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-ca; GT-P6210 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69733[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P6210 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69734[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P6210 Build/JZO54K)";
const char S69735[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; GT-P6210 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69736[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P6210 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69737[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ617 Build/7.7.1-85_MZ617-27) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S69738[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I717 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S69739[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5360) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S69740[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; vi; GT-S5360) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S69741[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5282 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69742[] = "Mozilla/5.0 (Linux; U; Android 3.6.0; md-us; A101C Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S69743[] = "Mozilla/5.0 (MeeGo; U; en-AU) AppleWebKit/534.3 (KHTML, like Gecko) Qt/4.7.4 Safari/534.3";
const char S69744[] = "Mozilla/5.0 (MeeGo; U; fi-FI) AppleWebKit/534.3 (KHTML, like Gecko) Qt/4.7.4 Safari/534.3";
const char S69745[] = "Mozilla/5.0 (MeeGo; U; en-GB) AppleWebKit/534.3 (KHTML, like Gecko) Qt/4.7.4 Safari/534.3";
const char S69746[] = "Mozilla/5.0 (MeeGo) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.7.4 Mobile Safari/534.34";
const char S69747[] = "Mozilla/5.0 (MeeGo; U; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Qt/4.7.4 Safari/534.3";
const char S69748[] = "Mozilla/5.0 (MeeGo; U; en-ZA) AppleWebKit/534.3 (KHTML, like Gecko) Qt/4.7.4 Safari/534.3";
const char S69749[] = "Lenovo A789/S210 Linux/3.0.13 Android/4.0 Release/05.11.2012 Browser/AppleWebKit533.1 Profile/ Configuration/";
const char S69750[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; vi-vn; Limo X2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69751[] = "Mozilla/5.0 (Linux; U; Android 4.1.1_jelly_bean; vi-vn; Limo X2+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69752[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; Limo X2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69753[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; Limo X2+ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69754[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I815 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S69755[] = "Mozilla/5.0 (compatible; MSIE 2.0; Windows NT 5.2; Trident/5.0)";
const char S69756[] = "Mozilla/5.0 (compatible; MSIE 1.0; Windows NT 5.2; Trident/1.0)";
const char S69757[] = "Mozilla/5.0 (compatible; MSIE 2.0; Windows NT 5.1; Trident/3.0)";
const char S69758[] = "Mozilla/5.0 (compatible; MSIE 1.0; Windows NT 5.1; Trident/3.0)";
const char S69759[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-US730 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S69760[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9295 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S69761[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9295 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S69762[] = "Mozilla/5.0 (Linux; Android 4.3; Galaxy Nexus Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S69763[] = "Mozilla/5.0 (Linux; U; Linux mips; en) AppleWebKit/531.2+ (KHTML, like Gecko) Version/4.1.4 Safari/531.2+";
const char S69764[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:9.2) Gecko/X Firefox/9.2-x64 PaleMoon/9.2-x64";
const char S69765[] = "Mozilla/5.0 (Windows NT 6.1; rv:9.2) Gecko/X Firefox/9.2 PaleMoon/9.2";
const char S69766[] = "Mozilla/5.0 (Series40; NokiaC2-01/11.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S69767[] = "Mozilla/5.0 (S60V3; U; en-in; NokiaC5-00)/UC Browser8.4.0.159/28/352/UCWEB Mobile";
const char S69768[] = "Mozilla/5.0 (S60V3; U; en-in; NokiaC5-00.2)/UC Browser8.4.0.159/28/352/UCWEB Mobile";
const char S69769[] = "Mozilla/5.0 (Java; U; en-in; nokiac5-00.2) UCBrowser8.4.0.159/69/444/UCWEB Mobile  3gpp-gba";
const char S69770[] = "Mozilla/5.0 (Java; U; en-in; nokiac5-00) UCBrowser8.4.0.159/70/352/UCWEB Mobile  3gpp-gba";
const char S69771[] = "MQQBrowser/1.3,Nokia6681/2.0";
const char S69772[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; U9GT2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69773[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; U9GT4 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69774[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; U9GT V Build/U9GT V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69775[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Cube U9GT2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69776[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; U9GT V_Core 4 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69777[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:20.0) Gecko/X Firefox/20.0 slurp";
const char S69778[] = "Mozilla/5.0 (Windows NT 6.1; rv:20.0) Gecko/X Firefox/20.0 slurp";
const char S69779[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:20.0) Gecko/X Firefox/20.0 slurp slurp";
const char S69780[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030X Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69781[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL_ONE_TOUCH_6030X_Orange Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69782[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-US730 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S69783[] = "Mozilla/5.0 (Linux; Android 4.0.4; C811 4G Build/C811M020) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69784[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_IncredibleS_S710e; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S69785[] = "Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/10.54";
const char S69786[] = "Opera/9.80 (X11; Linux i686; U; en-GB) Presto/2.5.24 Version/10.53";
const char S69787[] = "Opera/9.80 (Linux i686; U; en) Presto/2.5.22 Version/10.51";
const char S69788[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM009SH Build/S4070) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69789[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM009SH Build/S4030) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69790[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM009SH Build/S4080) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69791[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM009SH Build/S4040) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69792[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; DM009SH Build/S4080)";
const char S69793[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM009SH Build/S4090) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69794[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM009SH Build/S4110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69795[] = "NokiaX2-00/5.0 (08.25) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-00/UC Browser8.0.3.107/69/444";
const char S69796[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Advent Vega Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69797[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Vega Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69798[] = "Mozilla/5.0 (Windows NT 4.10; rv:10.0.2) Gecko/X Firefox/10.0.2";
const char S69799[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; aokp_tenderloin Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69800[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; aokp_tenderloin Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69801[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; aokp_tenderloin Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69802[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1";
const char S69803[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1";
const char S69804[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID2 Build/VZW; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69805[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SCH-I500 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69806[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-R915 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69807[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-R915 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69808[] = "Lenovo-A366t_TD/S100 Linux/2.6.35 Android/2.3 Release/11.01.2011 Browser/AppleWebkit533.1 Mobile Safari/533.1 MBBMS/2.0";
const char S69809[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A366t Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S69810[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A366t Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S69811[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A366t Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69812[] = "Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A366t Build GRK39F) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S69813[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; Lenovo A366t Build/GRK39F)";
const char S69814[] = "Mozilla/5.0 (Android; Tablet; rv:14.0) Gecko/14.0 K9Browser/14.0.b5";
const char S69815[] = "Mozilla/5.0 (Android; Tablet; rv:14.0) Gecko/14.0 K9Browser/14.0.1";
const char S69816[] = "NetFront/4.2 (BREW 1.0.4; U; en-us; Samsung; NetFront/4.2/WAP) Sprint M400 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S69817[] = "Mozilla/5.0 (Symbian3; U; en-us; NokiaE6-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile";
const char S69818[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AT270 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69819[] = "Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S69820[] = "Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S69821[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-i705 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S69822[] = "Mozilla/5.0 (Linux; Android 4.1.2; B15 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69823[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S7562) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S69824[] = "Nokia302/5.0 (14.78) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia302) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S69825[] = "SAMSUNG-GT-S5233T/1.0 SHP/VPP/R5 Jasmine/0.8 NexPlayer/2.12 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S69826[] = "Cricket-A300/1.0 UP.Browser/6.3.0.7 (GUI) MMP/2.0";
const char S69827[] = "Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.6) Gecko/X Firefox/0.8";
const char S69828[] = "Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.7) Gecko/X Firefox/0.9.3";
const char S69829[] = "Mozilla/5.0 (Windows; U; Win 9x 4.90; rv:1.7) Gecko/X Firefox/0.9.3";
const char S69830[] = "Mozilla/5.0 (Windows; U; Win 9x 4.90; rv:1.7.3) Gecko/X Firefox/0.10.1";
const char S69831[] = "Mozilla/5.0 (Windows; U; Win 9x 4.90; en-US; rv:1.6) Gecko/X Firefox/0.8";
const char S69832[] = "Mozilla/5.0 (Windows; U; Win98; rv:1.7.3) Gecko/X Firefox/0.10.1 StumbleUpon/1.9993";
const char S69833[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID7030 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69834[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3110 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69835[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3110 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69836[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3110 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69837[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; GT-P3110 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69838[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GT-P3110 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69839[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69840[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ja-; GT-P3110 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69841[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; GPLUS_GN708W Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69842[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GN708W Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69843[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GN708W Build/JZO54K)";
const char S69844[] = "Mozilla/5.0 (Linux; Android 4.1.1; ME301T Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S69845[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT26i Build/6.1.A.2.55) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S69846[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LT26i Build/6.1.A.2.45) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S69847[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LT26i Build/6.1.A.0.452) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S69848[] = "SonyEricssonK700i/R2N SEMC-Browser/4.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S69849[] = "SonyEricssonK700i/R2AG SEMC-Browser/4.0.3 Profile/MIDP-1.0 MIDP-2.0 Configuration/CLDC-1.1, SonyEricssonK700i/R2AG SEMC-Browser/4.0.3 Profile/MIDP-1.0 MIDP-2.0 Configuration/CLDC-1.1";
const char S69850[] = "SonyEricssonK700i/R2AA SEMC-Browser/4.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S69851[] = "SonyEricssonK700i/R2AY SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S69852[] = "SonyEricssonK700i/R2A SEMC-Browser/4.0 Profile/MIDP-1.0 MIDP-2.0 Configuration/CLDC-1.1";
const char S69853[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; IM-A720L Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69854[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; IM-A720L Build/FRG83)";
const char S69855[] = "OneBrowser/3.1 (Nokia2700c-2/09.95)";
const char S69856[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaE52-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S69857[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Spice MI352 Build/GRJ90) AppleWebKit/5.33.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69858[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Spice MI-352 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69859[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9400 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69860[] = "Mozilla/5.0 (Linux; Android 4.0.3; Slider SL101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S69861[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5302 Build/12.0.A.1.211) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S69862[] = "Mozilla/5.0 (Linux; Android 4.1.1; Transformer Prime TF201 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S69863[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari Version/5.1";
const char S69864[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A470a Safari Version/5.1";
const char S69865[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Safari Version/5.1";
const char S69866[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d Safari Version/5.1";
const char S69867[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A466 Safari Version/5.1";
const char S69868[] = "SAMSUNG-SGH-F490/F490XEHB2 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S69869[] = "SAMSUNG-SGH-F490-ORANGE/F490AFHD1 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S69870[] = "SAMSUNG-SGH-F490-Vodafone/F490BUHB2 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S69871[] = "Mozilla/5.0 (Vodafone/1.0/LG-BL20/V10b Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S69872[] = "LG/BL20/v10a Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S69873[] = "LG-BL20-Orange/v10e Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S69874[] = "Mozilla/5.0 (Vodafone/1.0/LG-BL20/V10d Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S69875[] = "LG-BL20/v10a Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S69876[] = "SonyEricssonK320c/R4GC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S69877[] = "SonyEricssonK320i/R4GC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S69878[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; IM-A710K Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69879[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; IM-A710K Build/FRG83)";
const char S69880[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-; IM-A710K Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69881[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaC5-05)/UC Browser8.2.0.132/50/355/UCWEB Mobile";
const char S69882[] = "Mozilla/5.0 (Linux; Android 4.0.3; ISW13HT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69883[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC J Z321e Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69884[] = "Mozilla/5.0 (Linux; Android 4.0.4; ISW13HT Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69885[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KATBL10A16DA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69886[] = "MicromaxX292/MstarBrowser1.0/MIDP2.0Configuration/CLDC1.1";
const char S69887[] = "MicromaxX292i/MstarBrowser1.0/MIDP2.0Configuration/CLDC1.1";
const char S69888[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X6-7A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69889[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X6-7A-Q88 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69890[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X6-7A MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69891[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X6-7A-Q88 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69892[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us;BLU Quattro 4.5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69893[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BLU Quattro 4.5 HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69894[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P765 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69895[] = "Mozilla/5.0 (Series40; NokiaC2-01/11.40; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S69896[] = "NokiaN70-1/5.0737.3.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.7.1.88/27/444";
const char S69897[] = "NokiaN70/ 5.0741.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.7.1.88/27/444";
const char S69898[] = "Mozilla/5.0 (S60V3; U; en-in; NOKIAE5-00)/UC Browser8.4.0.159/28/352/UCWEB Mobile";
const char S69899[] = "Mozilla/5.0 (Java; U; en-in; nokiae5-00) UCBrowser8.4.0.159/69/444/UCWEB Mobile  3gpp-gba";
const char S69900[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; sr-rs; Telenor Touch Plus Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69901[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; sr-rs; Telenor Touch Pad Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69902[] = "MQQBrowser/37 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S69903[] = "MQQBrowser/371 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S69904[] = "MQQBrowser/37 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3";
const char S69905[] = "MQQBrowser/371 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3";
const char S69906[] = "MQQBrowser/371 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3";
const char S69907[] = "MQQBrowser/371 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3";
const char S69908[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VL-107L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69909[] = "Nokia110/2.0 (03.04) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia110) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile";
const char S69910[] = "OneBrowser/3.1 (NokiaC3-00/08.65)";
const char S69911[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; HTC_DesireU Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69912[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC_Desire_U_dual_sim Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69913[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC Desire U dual sim Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69914[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; HTC Desire U Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69915[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; de-de; LG-P880 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/p880";
const char S69916[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P930 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S69917[] = "Nokia2323c/2.0 (08.20) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S69918[] = "Nokia2323c-2/2.0 (08.20) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S69919[] = "SAMSUNG-SCH-W259/W259DDDA2 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S69920[] = "LG-GT550/V11d Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S69921[] = "ZTE-N295-Sydney-Optimus/Obigo Q03C/MIDP-2.0 Configuration/CLDC-1.1";
const char S69922[] = "zte-N295/P107A20/Obigo Q03C/MIDP-2.0 Configuration/CLDC-1.1";
const char S69923[] = "ZTE/ZTE-G N295 Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S69924[] = "ZTE/N295 Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S69925[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s5253) UCBrowser8.2.0.132/69/452/UCWEB Mobile";
const char S69926[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s5253) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S69927[] = "samsung-gt-s5253/UCWEB8.2.0.116/69/352";
const char S69928[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Transformer TF101 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69929[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Transformer TF101 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69930[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Transformer Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69931[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; hr-hr; Tele2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69932[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hr-hr; Tele2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69933[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7500 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69934[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69935[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; GT-P7500 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69936[] = "Logos/1.1.3 (Linux; U; Android 4.1.1; sk-SK; GT-P7500 Build/cm_p4-userdebug 4.1.1 JRO03C eng.maplesci.X.X test-keys)";
const char S69937[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P7500 Build/JRO03L)";
const char S69938[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; da-dk; GT-P7500 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S69939[] = "Mozilla/5.0 (Linux; U; Android 4.1; vi-vn; GT-P7500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69940[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P7500 Build/JZO54K)";
const char S69941[] = "Vyrso/2.2.3 (Linux; U; Android 4.1.1; en-GB; GT-P7500 Build/cm_p4-userdebug 4.1.1 JRO03H eng.maplesci.X.X test-keys)";
const char S69942[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; lt-lt; GT-P7500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S69943[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 GSA/2.0.0.X";
const char S69944[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69945[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fi-fi; GT-P7500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S69946[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P7500M Build/JZO54K)";
const char S69947[] = "Groupon/2.2.2572 (Android 4.1.2; Samsung P4 / Samsung GT-P7500; )[preload=false;locale=en_CA;clientidbase=android-samsung]";
const char S69948[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6800 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S69949[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P6800 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S69950[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P6800 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69951[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6800 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S69952[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; A878 Duo Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69953[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; M2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S69954[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8815 Build/HuaweiU8815) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S69955[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8815N Build/HuaweiU8815NC02B942) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S69956[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; Micromax A115 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69957[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Micromax A115 Build/JZO54K)";
const char S69958[] = "Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabA2109A Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S69959[] = "Mozilla/5.0 (Linux; Android 4.1.1; ME301T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S69960[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; H120 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S69961[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; H120 Build/JOP40D)";
const char S69962[] = "Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4372q Safari/8536.25";
const char S69963[] = "Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4400f Safari/8536.25";
const char S69964[] = "Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4414e Safari/8536.25";
const char S69965[] = "Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A465 Safari/8536.25";
const char S69966[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.9.168 Version/11.52";
const char S69967[] = "Mozilla/5.0 (Windows NT 6.0; rv:7.0.1; BrowserMob RBU) Gecko/X Firefox/7.0.1";
const char S69968[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:7.0a1) Gecko/X Firefox/7.0a1";
const char S69969[] = "Spice M5885   Browser/Opera  Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10";
const char S69970[] = "Spice M5885   Browser/Opera  Opera/9.80 (MTK; U; hi-IN) Presto/2.5.28 Version/10.10";
const char S69971[] = "Spice M5885 Browser/Opera Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10";
const char S69972[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; QMobile A5 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69973[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 7019 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69974[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 7019 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S69975[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 7019A Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69976[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A520/S101) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69977[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A520 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S69978[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A520 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69979[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A520 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S69980[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; Lenovo A520 Build/GRJ90)";
const char S69981[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A520GRAY Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69982[] = "Opera/9.80 (BREW; Opera Mini/5.0/27.2338; U; en) Presto/2.8.119 320X240 LG VX9200";
const char S69983[] = "JUC (Linux; U; 4.0.4; zh-cn; GT-I9300; 720*1280) UCWEB7.9.0.94/139/444";
const char S69984[] = "Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6303iclassic) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S69985[] = "Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80 COBALT Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S69986[] = "VLC/2.0.6 LibVLC/2.0.6";
const char S69987[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S7562) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S69988[] = "Mozilla/5.0 (Linux; Android 4.0.4; LePanII Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S69989[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I337 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/jflteatt";
const char S69990[] = "Mozilla/5.0 (Series40; NokiaC2-06/07.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S69991[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One VX Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S69992[] = "SCH-U640/1.0 NetFront/3.0.22.1.19 (GUI) MMP/2.0";
const char S69993[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MOT-XT883/SLNCT-57.1.40; 540*960; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69994[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-XT883/SLNCT-60.3.250; 540*960; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69995[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; XT883 Build/SLNCT-57.1.60) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69996[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MOT-XT883/SLNCT-57.1.60; 540*960; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S69997[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT883 Build/SLNCT-57.1.40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S69998[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT883 Build/SLNCT-57.1.40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S69999[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT883 Build/SLNCT-60.3.250) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S70000[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT883 Build/SLNCT-60.3.250) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70001[] = "ZTE-Me/Mobile";
const char S70002[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Transformer Prime TF201 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S70003[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; ja-jp; TF201 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S70004[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70005[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70006[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70007[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70008[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-P7510 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.2.0-57 (Android 4.0.4; samsung GT-P7510 Build/cm_p4wifi-userdebug 4.0.4 IMM76L eng.root.X.X test-keys)";
const char S70009[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-be; Lenco TAB-1014 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70010[] = "Mozilla/5.0 (Series40; NokiaC1-01/06.15; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S70011[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S70012[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; SonyC1604 Build/11.3.A.2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70013[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; C1604 Build/11.3.A.2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70014[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; C1604 Build/11.3.A.2.1)";
const char S70015[] = "Mozilla/5.0 (compatible; Konqueror/4.5; Linux) KHTML/4.5.3 (like Gecko) Kubuntu";
const char S70016[] = "Mozilla/5.0 (Series40; Nokia2730c-1/10.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S70017[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOR807 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70018[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; R807 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S70019[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; R807 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S70020[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; R807 Build/GRK39F)";
const char S70021[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s3653) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S70022[] = "Mozilla/5.0 (Linux; Android 4.0.4; BC1003 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S70023[] = "Mozilla/5.0 (Linux; Android 4.0.3; BC1003 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S70024[] = "Mozilla/5.0 (Linux; Android 4.0.4; BC1003 EUS Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S70025[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P3100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S70026[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-P3100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S70027[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-P3100 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.276 Mobile";
const char S70028[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; RC9731C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70029[] = "Mozilla/5.0 (Linux; Android 4.0.3; LIFETAB_P9514 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S70030[] = "Mozilla/5.0 (Linux; Android 4.2.2; Xoom Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70031[] = "Mozilla/5.0 (Linux; Android 4.2.1; Xoom Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70032[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; i-mobile i-style 7.1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70033[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; i-mobile i-style 7.1 Build/JRO03C)";
const char S70034[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HUAWEI G525-U00 Build/HuaweiG525-U00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70035[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HUAWEI G525-U00 Build/HuaweiG525-U00)";
const char S70036[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; T959 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70037[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; T959 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70038[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; T959 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70039[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; T959 Build/MIUI)";
const char S70040[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; T959 Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S70041[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SGH-T959 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70042[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 4.0)";
const char S70043[] = "Mozilla/6.0 (compatible;MSIE 7.0;  Windows NT 4.0; QQLOADOW)";
const char S70044[] = "Mozilla/4.0 (compatible;MSIE 7.0;  Windows NT 4.0;  MyIE2)";
const char S70045[] = "Mozilla/5.0 (compatible;MSIE 7.0;  Windows NT 4.0;  MyIE2)";
const char S70046[] = "Mozilla/3.0 (compatible;MSIE 7.0;  Windows NT 4.0; QQDownload)";
const char S70047[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 4.0; SV1)";
const char S70048[] = "Mozilla/5.0 (compatible; MSIE 7.0; WinNT4.0; Trident/2.0)";
const char S70049[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)";
const char S70050[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.4; en-US; rv:1.9.2.28) Gecko/X Thunderbird/3.1.20";
const char S70051[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; RK702 Build/RK2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70052[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; RK702 Build/2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70053[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; RK702 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70054[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-ca; RK702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70055[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; Brava_DM-990 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70056[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; Brava DM-990 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70057[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9A405 Safari/7534.48.3";
const char S70058[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9A334 Safari/7534.48.3";
const char S70059[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT27i Build/6.1.1.B.1.54) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S70060[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6802 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S70061[] = "Mozilla/5.0 (Linux; Android 4.2.1; IdeaTab S6000-F Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S70062[] = "Mozilla/5.0 (Linux; Android 4.2.2; IdeaTab S6000-H Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S70063[] = "SAMSUNG-SGH-D900i-ORANGE/D900iABGF2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S70064[] = "SAMSUNG-SGH-D900i/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S70065[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SHW-M100S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70066[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SHW-M100S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70067[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SHW-M100S Build/FROYO)";
const char S70068[] = "Spice Mi350n Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; huaqin73_gb Build/ALPS.GB.FDD2.MP.V2.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70069[] = "Spice Mi350n Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Spice Mi-350n Build/ALPS.GB.FDD2.MP.V2.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70070[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.7 (KHTML, like Gecko) Ubuntu/10.10 Chromium/16.0.912.77 Chrome/16.0.912.77 Safari/535.7";
const char S70071[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.7 (KHTML, like Gecko) Ubuntu/10.10 Chromium/16.0.912.77 Chrome/16.0.912.77 Safari/535.7";
const char S70072[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.4) Gecko/X FireDownload/2.0.1 Firefox/3.6.4 Wyzo/3.6.4.1 FireTorrent/2.0.1";
const char S70073[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_Desire_HD_A9191; 480*800) UCWEB7.9.3.103/139/800";
const char S70074[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_Desire_HD_A9191; 480*800) UCWEB7.9.3.103/139/999";
const char S70075[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_Desire_HD_A9191; 480*800) UCWEB7.9.4.145/139/800";
const char S70076[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_Desire_HD_A9191; 480*800) UCWEB7.9.0.94/139/352";
const char S70077[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_Desire_HD_A9191; 480*800) UCWEB7.9.0.94/139/444";
const char S70078[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_Desire_HD_A9191; 480*800) UCWEB7.9.0.94/139/351";
const char S70079[] = "JUC (Linux; U; 2.2; zh-cn; Desire_HD; 480*800) UCWEB7.9.0.94/139/352";
const char S70080[] = "JUC (Linux; U; 2.3.3; zh-cn; Desire_HD; 480*800) UCWEB7.9.0.94/139/444";
const char S70081[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_Desire_HD_A9191; 480*800) UCWEB7.9.0.94/139/355";
const char S70082[] = "Mozilla/5.0 (compatible; iCab 3.0.3; Macintosh; U; PPC Mac OS X)";
const char S70083[] = "iCab/3.0.2 (Macintosh; U; PPC Mac OS X)";
const char S70084[] = "iCab/3.0.5 (Macintosh; U; PPC Mac OS X)";
const char S70085[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; NGM Orion Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70086[] = "Mozilla/5.0 (Symbian3; U; en-us; NokiaX7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile";
const char S70087[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiax7-00) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba";
const char S70088[] = "NokiaX7-00UCBrowser/8.7.1.234/51/355/UCWEB";
const char S70089[] = "Mozilla/5.0 (Symbian3; U; en-us; NokiaX7-00.1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile";
const char S70090[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; MyPhone A888 Duo Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70091[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P5113 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S70092[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P5113 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S70093[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; S100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70094[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; S100 Build/JZO54K)";
const char S70095[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/10.04 Chromium/13.0.782.215 Chrome/13.0.782.215 Safari/535.1";
const char S70096[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.33 (KHTML, like Gecko) Ubuntu/10.04 Chromium/13.0.752.0 Chrome/13.0.752.0 Safari/534.33";
const char S70097[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.10.229 Version/11.60";
const char S70098[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; MID7024 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70099[] = "Dalvik/1.4.0 (Linux; U; Android 2.3; MID7024 Build/GINGERBREAD)";
const char S70100[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; MID7024 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70101[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Ideapad K1 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70102[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I727 Build/JRO03R; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70103[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I727 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70104[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I727 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70105[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I727 Build/JRO03L; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70106[] = "NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac2-03) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S70107[] = "MQQBrowser/40 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S70108[] = "MQQBrowser/40 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3";
const char S70109[] = "MQQBrowser/40 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3";
const char S70110[] = "MQQBrowser/40 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3";
const char S70111[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TD-1010 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70112[] = "Mozilla/5.0 (Windows NT 6.0; rv:19.0) Gecko/X Firefox/19.0 IceDragon/19.0.1.0";
const char S70113[] = "Mozilla/5.0 (Linux; Android 4.0.4; LGL86C Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S70114[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS950 4G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S70115[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I545 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/jfltevzw";
const char S70116[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; SH-03C Build/SB110) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S70117[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; SH-03C Build/S1130) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S70118[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; SH-03C Build/S2140) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S70119[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SH-03C Build/S2140)";
const char S70120[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Iron/8.0.555.0 Chrome/8.0.555.0 Safari/534.10";
const char S70121[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS12F Build/FGK600) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70122[] = "Bing%20for%20iPad/1.1.3 CFNetwork/609 Darwin/13.0.0";
const char S70123[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:16.0) Gecko/X Firefox/16.0 SeaMonkey/2.13.2";
const char S70124[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DII PAD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70125[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5100 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70126[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P5100 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70127[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5100 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70128[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5100 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70129[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; GT-P5100 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70130[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I717 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S70131[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I717 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S70132[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LS840 Build/ZVK.IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S70133[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S70134[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S70135[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S70136[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/6531.22.7 iLunascape/3212";
const char S70137[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8815 Build/HuaweiU8815) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70138[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Android2.x) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S70139[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; vi; Android2.x) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S70140[] = "Mozilla/5.0 (Linux; Android 4.1.1; I10A-LE Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S70141[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R760X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70142[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R760 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70143[] = "Mozilla/5.0 (Linux; Android 4.2.1; M470BSA Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70144[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E-51 Build/HuaweiU8666E-51) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S70145[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E Build/HuaweiU8666E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S70146[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666N Build/HuaweiU8666N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S70147[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.2.0.0 Chrome/12.0.742.112 Safari/534.30";
const char S70148[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.2.0.0 Chrome/12.0.742.112 Safari/534.30";
const char S70149[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Stainless/0.8 like Version/5.1 Safari/534.48.3";
const char S70150[] = "MicromaxX263/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S70151[] = "MicromaxX263/Q03C MAUI-browser/";
const char S70152[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NM7RK-C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70153[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E455 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70154[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; H7500+ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70155[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; H7500+ Build/JZO54K)";
const char S70156[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; H7500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70157[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-79) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S70158[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S70159[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P768 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S70160[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S70161[] = "Mozilla/5.0 (Linux; Android 4.1.2; LGMS769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S70162[] = "JUC (Linux; U; 2.3.3; zh-cn; X10i; 480*854) UCWEB7.9.0.94/139/444";
const char S70163[] = "JUC (Linux; U; 2.1-update1; zh-cn; X10i; 480*854) UCWEB7.9.0.94/139/444";
const char S70164[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Spice_Mi-355 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70165[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; AX515 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70166[] = "SAMSUNG-SGH-T159/1.0 SHP/VPP/R5 NetFront/4.2 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S70167[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST21i2 Build/11.0.A.1.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S70168[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST21i Build/11.0.A.0.16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S70169[] = "Mozilla/5.0 (Linux; Android 4.0.4; SH-02E Build/SC110) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70170[] = "Mozilla/5.0 (Linux; Android 4.0.4; CINK KING Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70171[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.3m";
const char S70172[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.3m";
const char S70173[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.3.1m";
const char S70174[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3 Sleipnir/2.3.2m";
const char S70175[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.3.2m";
const char S70176[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.3.2m";
const char S70177[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.3.1m";
const char S70178[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S70179[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S70180[] = "Mozilla/5.0 (Linux; Android 4.0.4; Grand X(M) Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70181[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70182[] = "Mozilla/5.0 (Linux; Android 4.0.3; ZTE Grand X Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70183[] = "Mozilla/5.0 (Linux; Android 4.0.4; Grand X Pro Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70184[] = "Mozilla/5.0 (Linux; Android 4.0.4; Grand X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70185[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70186[] = "Mozilla/5.0 (Linux; Android 4.2.2; ME302C Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S70187[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; OT107 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70188[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; OT107 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70189[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; Edition MAS; en) Presto/2.10.229 Version/11.66";
const char S70190[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ608-14) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S70191[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ608 Build/7.7.1-141-7-FLEM-UMTS-LA) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S70192[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S70193[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S70194[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S70195[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S70196[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; POLYTRON W2430 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70197[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; W2430 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70198[] = "NokiaX3-00/5.0 (11.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax3-00) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S70199[] = "Mozilla/5.0 (Linux; Android 4.0.4; KFJWI Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S70200[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9737AP Mobile";
const char S70201[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AT Mobile";
const char S70202[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP Mobile";
const char S70203[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70204[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; L-06D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70205[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; L-06DJOJO Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70206[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:16.0) Gecko/X Firefox/16.0 slurp";
const char S70207[] = "Mozilla/5.0 (Windows NT 6.1; rv:16.0) Gecko/X Firefox/16.0 slurp";
const char S70208[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:16.0) Gecko/X Firefox/16.0 slurp slurp";
const char S70209[] = "Mozilla/5.0 (Windows NT 6.1; rv:16.0) Gecko/X Firefox/16.0 slurp slurp";
const char S70210[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; HTC Vision Build/JOP40D; CyanogenMod-CM-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70211[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC Vision Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70212[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC Vision Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70213[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70214[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925U Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70215[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9070 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S70216[] = "Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 101G9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S70217[] = "Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 101G9 Build/MR1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S70218[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7560 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S70219[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7500 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S70220[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P7500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S70221[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21i Build/11.0.A.4.22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S70222[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21i2 Build/11.0.A.1.8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S70223[] = "Nokia5236/51.9.002 (SymbianOS/9.4; U; Series60/5.0; Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/413 (KHTML,like Gecko) Safari/413 3gpp-gba";
const char S70224[] = "JUC (Linux; U; 2.3.3; zh-cn; GT-I9000; 480*800) UCWEB7.9.0.94/139/444";
const char S70225[] = "JUC (Linux; U; 2.2; zh-cn; GT-I9000; 480*800) UCWEB7.9.0.94/139/444";
const char S70226[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I547 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70227[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I547C Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70228[] = "Opera/9.64 (Windows NT 6.2; U; en) Presto/2.1.1";
const char S70229[] = "Opera/9.63 (Windows NT 6.2; U; en) Presto/2.1.1";
const char S70230[] = "Opera/9.62 (Windows NT 6.2; U; en) Presto/2.1.1";
const char S70231[] = "Opera/9.60 (Windows NT 6.2; U; en) Presto/2.1.1";
const char S70232[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Titanium S1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70233[] = "Mozilla/5.0 (Linux; Android 4.1.1; Arc Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70234[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70235[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70236[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0151 Build/HuaweiY300-0151) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S70237[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0151 Build/HuaweiY300-0151) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70238[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; TECNO_P3) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S70239[] = "stagefright/1.2 (Linux;Android 4.3)";
const char S70240[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70241[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70242[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70243[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-i777 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S70244[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I777 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S70245[] = "Mozilla/5.0 (Linux; Android 4.0.3; W732 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70246[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; ZTE V790 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70247[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V790 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70248[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; Aqua Star Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70249[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70250[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ615-12) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S70251[] = "Mozilla/5.0 (Linux; U; Android 4.2.1;en-us; Lenovo_P780_ROW/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2.1 Mobile Safari/534.30";
const char S70252[] = "Mozilla/5.0 (Linux; U; Android 4.2.1;en-us; Lenovo_P780/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2.1 Mobile Safari/534.30";
const char S70253[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Lenovo P780_ROW Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70254[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; Lenovo P780_ROW Build/JOP40D)";
const char S70255[] = "SAMSUNG-GT-B3410W/1.0 SHP/VPP/R5 Dolfin/1.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-N";
const char S70256[] = "SAMSUNG-S3500/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S70257[] = "SAMSUNG-GT-S3500C/1.0 Release/10.23.2008 Browser/NetFront3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S70258[] = "Vodafone/1.0/SamsungS3500/S3500BUIB3 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S70259[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; IM-A730S Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70260[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; IM-A730S Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70261[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; IM-A730S Build/FRG83)";
const char S70262[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-; IM-A730S Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70263[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-; IM-A730S Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70264[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID7010 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70265[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID7010 Build/GRI40)";
const char S70266[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID7010A Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70267[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Revue Build/KA2X-X.X) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S70268[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Tostab03 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S70269[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; THRiVE Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S70270[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom 4G Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S70271[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Haipai_Noble_I9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70272[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-cn;HTCP715 /1.0 Android/3.1 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/HMJ15) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S70273[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-in; VOX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70274[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Videocon A26 Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S70275[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Swipe Xylus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70276[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9003 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70277[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/6.5.27431/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S70278[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; A898 Duo Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70279[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; iBuddy Connect Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70280[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; iBuddy Connect Build/IMM76D)";
const char S70281[] = "Mozilla/5.0 (Linux; Android 4.1.1; ME301T Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S70282[] = "Mozilla/5.0 (compatible; MSIE 2.0; Windows NT 6.0; Trident/3.0)";
const char S70283[] = "Mozilla/5.0 (compatible; MSIE 1.0; Windows NT 6.0; Trident/3.0)";
const char S70284[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E971 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S70285[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E971 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70286[] = "Mozilla/5.0 (en-US) AppleWebKit/[WEBKIT_VERSION] (KHTML, like Gecko) Chrome/[CHROME_VERSION] Safari/[WEBKIT_VERSION] pss-webkit-request";
const char S70287[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I915 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S70288[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I497 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S70289[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5310 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S70290[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5310B Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S70291[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5310L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S70292[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5310 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70293[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I927 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70294[] = "H1-1/20.0.041 SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S70295[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 E71-3/410.21.010; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S70296[] = "Mozilla/5.0 (SymbianOS/9.1; U; [en]; SymbianOS/9.1 Series60/3.0) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S70297[] = "E63/SymbianOS/9.1 Series60/3.0";
const char S70298[] = "E63-1/SymbianOS/9.1 Series60/3.0";
const char S70299[] = "(C)NokiaNXX/SymbianOS/9.1 Series60/3.0 3gpp-gba";
const char S70300[] = "E71/SymbianOS/9.1 Series60/3.0";
const char S70301[] = "E66/SymbianOS/9.1 Series60/3.0";
const char S70302[] = "E71-1/SymbianOS/9.1 Series60/3.0";
const char S70303[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_7260 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70304[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_7260/2.3.002.X.7260; 480*800; CTC/2.0) CoolpadWebkit/533.1";
const char S70305[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn;YL-Coolpad_7260A/2.3.002.X.7260+; 480*800; CUCC/3.0) CoolpadWebkit/533.1";
const char S70306[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_7260A Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70307[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-07D Build/S5230) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30";
const char S70308[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-07D Build/S5230) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70309[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-07D Build/S6151) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70310[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-07D Build/S7190) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70311[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-07D Build/S7230) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70312[] = "Mozilla/5.0 (S60V5; U; en-in; Nokia5800 XpressMusic)/UC Browser8.4.0.159/50/352/UCWEB Mobile";
const char S70313[] = "Nokia5800 XpressMusic/UCWEB8.4.0.150/50/999";
const char S70314[] = "Mozilla/5.0 (Java; U; en-us; nokia5800d-1) UCBrowser8.4.0.159/69/352/UCWEB Mobile  3gpp-gba";
const char S70315[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Tab 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70316[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Smartfren Tab 7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70317[] = "Mozilla/5.0 (Linux; U; Android 4.0.11; en-us; Tab 7G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70318[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; PIRANHA Pro II Tab 7.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70319[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; StarTab 715 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70320[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TAB10DUALC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70321[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TAB10DUALC 8GB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70322[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Android2.x) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S70323[] = "Mozilla/5.0 (Linux; Android 4.2.2; EVO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70324[] = "Mozilla/5.0 (Linux; Android 4.2.2; Evo 4G LTE Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70325[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae63-1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S70326[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE63) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S70327[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/m7spr";
const char S70328[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; One Build/JWR66V.H10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/m7ul";
const char S70329[] = "Mozilla/4.0 SonyEricssonW595v/R3EE Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.3";
const char S70330[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Iron/3.0.197.0 Safari/532.0";
const char S70331[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/531.0 (KHTML, like Gecko) Iron/3.0.189.0 Safari/531.0";
const char S70332[] = "Links (2.2; OpenBSD 4.8 i386; x)";
const char S70333[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; NW-Z1000Series Build/A100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70334[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; NW-Z1000Series Build/C121) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70335[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; NW-Z1000Series Build/B111) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70336[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; NW-Z1000Series Build/B110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70337[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; IS11LG Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70338[] = "Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.0.2) Gecko/X Netscape/7.02";
const char S70339[] = "Mozilla/5.0 (Windows; U; Win 9x 4.90; ja-JP; rv:1.0.1) Gecko/X Netscape/7.0";
const char S70340[] = "NokiaE63/UCWEB8.4.0.150/28/800";
const char S70341[] = "Mozilla/5.0 (S60V3; U; en-in; NokiaE63)/UC Browser8.4.0.159/28/352/UCWEB Mobile";
const char S70342[] = "Mozilla/5.0 (Java; U; en-in; nokiae63-1) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S70343[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; BASE Tab 7.1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70344[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Wildfire S A510e Build/ASN01; CyanogenMod-CM9_last_modified_by_BIT_liumiao) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70345[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Wildfire S A510e Build/ASNX; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70346[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Marvelc Build/ASNX; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70347[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7310 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70348[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7310 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70349[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; iball Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70350[] = "SAMSUNG-GT-E3300/1.0 NETFRONT/4.2 PROFILE/MIDP-2.0 CONFIGURATION/CLDC-1.1";
const char S70351[] = "SAMSUNG-GT-E3300L/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S70352[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I547 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S70353[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I547C Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S70354[] = "Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I547 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70355[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID4331 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Tablet Safari/534.30";
const char S70356[] = "Mozilla/5.0 (Linux; Android 4.1.2; dtab01 Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70357[] = "Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 10 LINK Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70358[] = "Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 7 Vogue Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70359[] = "Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 7 Youth Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70360[] = "Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 7 Lite II Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70361[] = "Mozilla/5.0 (Linux; Android 4.1.2; Telpad Dual S Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70362[] = "SAMSUNG-GT-C3222/C3222DDJL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S70363[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.3.0.0 Chrome/17.0.963.46 Safari/535.11";
const char S70364[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.3.0.0 Chrome/17.0.963.46 Safari/535.11";
const char S70365[] = "Mozilla/5.0 (X11; U; SunOS i86pc; en-US; rv:1.7.12) Gecko/X Firefox/1.0.7 (Nexenta package 1.0.7)";
const char S70366[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V889D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70367[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V889D Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S70368[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE V889D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70369[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V889D Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S70370[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; ZTE V889D Build/GRK39F)";
const char S70371[] = "Dolphin http client/9.0.0(184) (Android)";
const char S70372[] = "Dolphin http client/9.0.1(185) (Android)";
const char S70373[] = "Dolphin http client/9.0.2(190) (Android)";
const char S70374[] = "Dolphin http client/9.0.3(191) (Android)";
const char S70375[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIANokia N81) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile";
const char S70376[] = "NOKIANokia N81UCBrowser/8.7.1.234/28/352/UCWEB";
const char S70377[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokian81-3) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S70378[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIANokia N81 8GB) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S70379[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; UBISLATE7C+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70380[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; UBISLATE 7C+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70381[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; UBISLATE7C+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70382[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; UBISLATE7C+ Build/IMM76D)";
const char S70383[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Barnes & Noble Nook HD Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70384[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Barnes & Noble Nook HD Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70385[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Barnes & Noble Nook HD Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70386[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; BN Nook HD Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1/hummingbird";
const char S70387[] = "Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S70388[] = "Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70389[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5360 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S70390[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; GT-S5360 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S70391[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E988 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70392[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G X710s Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S70393[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S70394[] = "Mozilla/4.1 (compatible; Teleca Q7; BMP 1.0.1; U; en) 240X400 LG-AN510";
const char S70395[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; Win64; x64; en-US; rv:1.9.3a5pre) Gecko/X Minefield/3.7a5pre";
const char S70396[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.3a5pre) Gecko/X Minefield/3.7a5pre";
const char S70397[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/527+ (KHTML, like Gecko) QtWeb Internet Browser/1.2 http://www.QtWeb.net";
const char S70398[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE72-1)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S70399[] = "NOKIAE72-1/UCWEB8.5.0.163/28/800";
const char S70400[] = "Mozilla/5.0 (Java; U; en-us; nokiae72-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile  3gpp-gba";
const char S70401[] = "Mozilla/5.0 (Linux; Android 4.1.1; Ideapad K1 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S70402[] = "Mozilla/5.0 (Linux; Android 4.1.2; K1 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S70403[] = "Mozilla/5.0 (Linux; Android 4.1.1; K1 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S70404[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; OPPOR817 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70405[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R817 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S70406[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R817T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70407[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; R817 Build/IMM76D)";
const char S70408[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; R817T Build/IMM76D)";
const char S70409[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V790 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70410[] = "Mozilla/5.0 (Linux; Android 4.0.4; V790 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70411[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; VIETTEL V8404 Build/HuaweiY210-0200) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70412[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; VIETTEL V8404 Unlock Build/HuaweiY210-0200) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70413[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; VIETTEL V8404 - Y210 Build/HuaweiY210-0010) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70414[] = "Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Safari/535.19 Silk-Accelerated=false";
const char S70415[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZTE V70 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70416[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T699 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S70417[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PAP4322DUO Build/PrestigioPAP4322DUO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70418[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC-X710a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70419[] = "SAMSUNG-SGH-D980/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S70420[] = "Pulp/1.5.2 (iPad; http://www.acrylicapps.com/pulp/)";
const char S70421[] = "Apple iPad v5.1.1 YouTube v1.0.0.9B206";
const char S70422[] = "server-bag [iPhone OS,5.1.1,9B206,iPad1,1]";
const char S70423[] = "eBayiPad/2.2.1";
const char S70424[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonSO-02D Build/6.0.A.4.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70425[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonSO-02D Build/6.0.A.5.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70426[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SO-02D Build/6.0.A.5.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70427[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SO-02D Build/6.0.A.4.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70428[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonSO-02D Build/6.0.A.5.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70429[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SO-02D Build/6.0.A.5.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70430[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; SO-02D Build/6.0.A.5.12)";
const char S70431[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonSO-02D Build/6.0.A.5.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70432[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SO-02D Build/6.0.A.5.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70433[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11";
const char S70434[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11";
const char S70435[] = "Mozilla/5.0 (X11; OpenBSD amd64; rv:18.0) Gecko/X Firefox/18.0";
const char S70436[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Ascend G300 Build/JOP40C; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70437[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Ascend G300 Build/JOP40C; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70438[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B179 Maxthon/4.0.4.1000 Safari/537.4";
const char S70439[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6812 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70440[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6812B Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70441[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A11) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S70442[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; iBuddy_Connect) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S70443[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Videocon_A30) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S70444[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; V1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S70445[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax_A101) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S70446[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MID) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S70447[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; Penta_IS701C) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S70448[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Y3) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S70449[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; U1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S70450[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; VP10) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S70451[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Connect-3G) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S70452[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; W60) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S70453[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Cloud_Z5) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S70454[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; CT-1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S70455[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; W70) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S70456[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; Lafone) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S70457[] = "Mozilla/5.0 (Linux; Android 4.1.1; V360 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S70458[] = "DoCoMo/2.0 SH01B(c500;TB;W24H16)";
const char S70459[] = "DoCoMo/2.0 SH01C(c500;TB;W24H16)";
const char S70460[] = "DoCoMo/2.0 SH01A(c100;TB;W24H16)";
const char S70461[] = "SAMSUNG-GT-E2530/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S70462[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOR805 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70463[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; R805 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S70464[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; R805 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S70465[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; R805 Build/GRK39F)";
const char S70466[] = "Mozilla/5.0 (X11; SunOS i86pc; rv:14.0) Gecko/X Firefox/14.0.1";
const char S70467[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.4.2 Mobile/9A405 Safari/7534.48.3";
const char S70468[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.4.2 Mobile/9A334 Safari/7534.48.3";
const char S70469[] = "Mozilla/5.0 (Linux; Android 4.1.2; B1-710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70470[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I577 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S70471[] = "Mozilla/5.0 (Linux; Android 4.1.1; Lenovo P770 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S70472[] = "Mozilla/5.0 (Linux; Android 4.1.2; Lenovo P770 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S70473[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en; rv:1.8.1.8) Gecko/X Camino/1.5.2";
const char S70474[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X Mach-O; en; rv:1.8.1.12) Gecko/X Camino/1.5.5";
const char S70475[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.152 Safari/535.19 Comodo_Dragon/18.1.2.0";
const char S70476[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.152 Safari/535.19 Comodo_Dragon/18.1.2.0";
const char S70477[] = "CoolPad8150_CMCC_TD/1.0 Linux/2.6.35 Android/2.3 Release/12.25.2011 Browser/AppleWebkit533.1";
const char S70478[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8150 Build/GWK74) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S70479[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8150 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70480[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8150 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S70481[] = "Mozilla 5.0 (Linux; U; Android 2.3.7; zh-cn; 8150 Build GWK74) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S70482[] = "Mozilla/5.0 (Windows; U; Windows NT based; en-US) AppleWebKit/533.3 (KHTML, like Gecko)  QtWeb Internet Browser/3.7.5 http://www.QtWeb.net";
const char S70483[] = "Mozilla/5.0 (Windows; U; Windows NT based; en-US) AppleWebKit/533.3 (KHTML, like Gecko)  QtWeb Internet Browser/3.7.3 http://www.QtWeb.net";
const char S70484[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MIDC497 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70485[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC497 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70486[] = "Mozilla/5.0 (Linux; Android 4.0.3; E350 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S70487[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +1300, nz";
const char S70488[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.20/286,1.9.20.286,2013-01-25 13:38, +0100, de";
const char S70489[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0100, es";
const char S70490[] = "Mozilla/5.0 (Linux; Android 4.2.2; EVO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70491[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70492[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; E270BSA Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70493[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; E270BSA Build/JRO03H)";
const char S70494[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E-51 Build/HuaweiU8666E-51) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70495[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E Build/HuaweiU8666E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70496[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666N Build/HuaweiU8666N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70497[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; AT10-A Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70498[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; AT10-A Build/JOP40D)";
const char S70499[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I815 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S70500[] = "MOT-W845/00.72 UP.Browser/7.2.7.5.663 (GUI) MMP/2.0";
const char S70501[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-E739 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70502[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-E739 Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70503[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; LG-P350 Build/GRJ90; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70504[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; LG-P350 Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70505[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; LG-P350 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70506[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; Optimus Me Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70507[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9103 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70508[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cloudfone Thrill 430g Build/SKV05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70509[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cloudfone Thrill 430g Build/FSV04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70510[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P1000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70511[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P1000 Build/JZO54K)";
const char S70512[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P1000 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70513[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P1000 Build/JRO03R)";
const char S70514[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; GT-P1000 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70515[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P1000 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S70516[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P1000 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70517[] = "GoogleAnalytics/2.0 (Linux; U; Android 4.1.2; ko-kr; GT-P1000 Build/JZO54K)";
const char S70518[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P1000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30";
const char S70519[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P1000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70520[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P1000N Build/JZO54K)";
const char S70521[] = "Logos/2.2.8.1 (Linux; U; Android 4.1.2; en-GB; GT-P1000 Build/cm_p1-userdebug 4.1.2 JZO54K eng..X.X test-keys)";
const char S70522[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P1000 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70523[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P1000) antivirusAM/2.4.5 (KHTML, like Gecko) Version/Version 2.4.5/antivirus";
const char S70524[] = "Mozilla/5.0 (Linux; U; Android 4.1.0; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70525[] = "Groupon/2.3.2787 (Android 4.1.2; Samsung GT-P1000 / Samsung GT-P1000; )[preload=false;locale=en_CA;clientidbase=android-google]";
const char S70526[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P1000 Build/Smartfren.andromaxtab7.0.v.41.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70527[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-P1000N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70528[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6210 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70529[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Transformer Prime TF201 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70530[] = "Mozilla/5.0 (Linux; Android 4.1.2; Andromax U Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S70531[] = "Mozilla/5.0 (Linux; Android 4.1.2; Andromax U Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S70532[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/2.0";
const char S70533[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Flipboard/2.0";
const char S70534[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/2.0.1";
const char S70535[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/2.0.2";
const char S70536[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Flipboard/2.0.2";
const char S70537[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Flipboard/2.0.1";
const char S70538[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/2.0.3";
const char S70539[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/2.0.4";
const char S70540[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Flipboard/2.0.4";
const char S70541[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/2.0.6";
const char S70542[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SBM200SH Build/S1002) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70543[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SBM200SH Build/S1002)";
const char S70544[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SBM200SH Build/S1004) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70545[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Micromax A116 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S70546[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6503 Build/10.1.A.1.434) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70547[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S70548[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S70549[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S70550[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989D Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S70551[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax A110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70552[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S70553[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9082 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70554[] = "SAMSUNG-SGH-A707/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S70555[] = "sam-r260 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S70556[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.2.2.0 Chrome/14.0.835.202 Safari/535.1";
const char S70557[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.2.2.0 Chrome/14.0.835.202 Safari/535.1";
const char S70558[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P940 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70559[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-KU5400 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70560[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-SU540 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70561[] = "SAMSUNG-GT-I9050_TD/1.0 Android/2.3.6 Release/01.15.2012 Browser/AppleWebKit533.1";
const char S70562[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9050 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S70563[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9050 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70564[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9050 Build/GINGERBREAD)";
const char S70565[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; I9250 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70566[] = "Mozilla/5.0 (Linux; Android 4.1.1;  ONDA MID Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S70567[] = "Mozilla/5.0 (Linux; Android 4.1.1; ONDA MID Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S70568[] = "Mozilla/5.0 (Linux; Android 4.1.1; ONDA MID Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70569[] = "Mozilla/5.0 (Linux; Android 4.1.1;  ONDA MID Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S70570[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E15i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S70571[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E15a) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S70572[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SC-91JB Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70573[] = "Mozilla/5.0 (PlayStation Vita 2.10) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S70574[] = "Mozilla/5.0 (Linux; Android 4.0.3; Slider SL101 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70575[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P970 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70576[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P970h Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70577[] = "Mozilla/5.0 (Linux; Android 4.1.2; Andromax U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70578[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I815 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S70579[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST27i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70580[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST27a Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70581[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7500 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S70582[] = "Mozilla/5.0 (Linux; Android 4.0.4; BNTV400 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S70583[] = "LG-GS500-parrot/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S70584[] = "LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S70585[] = "ZTE NOVA/G-R3100-Hollywood-Orange/R3100_V4_Z0_FR_D18GOUK104 Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S70586[] = "ZTE NOVA/G-R3100-Hollywood-Tmobile/R3100_V4_Z1_FR_D18GTMO101 Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S70587[] = "ZTE NOVA/G-R3100-Hollywood-Orange/R3100_V7_Z3_ES_D18GOSP101 Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S70588[] = "LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-kp500 teleca) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S70589[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A27) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S70590[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; 201M Build/9.8.2Q-34_SMJ-102)";
const char S70591[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; 201M Build/9.8.2Q-34_SMJ-102) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70592[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I605 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70593[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9500 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S70594[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T5-A Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70595[] = "T5-A Linux/3.0.13 Android/4.0.4 Release/05.16.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S70596[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A830L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S70597[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A830K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S70598[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A830S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S70599[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A830KE Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S70600[] = "Mozilla/5.0 (Linux; U; Android 1.5; es-es; htc_dream Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S70601[] = "HTC_Dream Mozilla/5.0 (Linux; U; Android 1.5; en-ca; Build/1.86.631.1) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S70602[] = "HTC_Dream Mozilla/5.0 (Linux; U; Android 1.5; en-ca; Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S70603[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-za; HTC_Dream Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S70604[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-sg; HTC Dream Build/CRC28) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S70605[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-za; HTC Dream Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S70606[] = "HTC_Dream Mozilla/5.0 (Linux; U; Android 1.5; en-ca; Build/CUPCAKE) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S70607[] = "SonyEricssonW508a/R1GA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.4";
const char S70608[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; myTouch4G; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S70609[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_myTouch4G; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S70610[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.4.0.0 Chrome/13.0.782.220 Safari/535.1";
const char S70611[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.4.0.0 Chrome/13.0.782.220 Safari/535.1";
const char S70612[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I997 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70613[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I997 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70614[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I997 Build/4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70615[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I997 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70616[] = "Mozilla/5.0 (Linux; Android 4.0.4; NexusHD2 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70617[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC HD2 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70618[] = "Mozilla/5.0 (Linux; Android 4.0.9; Hd2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70619[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC HD2 EvoHd2 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70620[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia500) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile";
const char S70621[] = "JUC (Linux; U; 4.0.4; zh-cn; ST18i; 480*854) UCWEB7.9.0.94/139/352";
const char S70622[] = "JUC (Linux; U; 2.3.4; zh-cn; ST18i; 480*854) UCWEB7.9.0.94/139/444";
const char S70623[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P6201 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70624[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SC-02D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70625[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P6201 Build/IMM76D)";
const char S70626[] = "com.google.android.youtube/4.2.16(Linux; U; Android 4.0.4; de_DE; GT-P6201 Build/IMM76D) gzip";
const char S70627[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P6201 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70628[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Starmobile CRYSTAL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70629[] = "Mozilla/5.0 (Linux; Android 4.1.1; Arc Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S70630[] = "Mozilla/5.0 (Series40; Nokia310/07.35; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S70631[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E610 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70632[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E612f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70633[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E612 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70634[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E610v Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70635[] = "ZTE-F102 H3G/1.0 ACS-NF/3.4 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S70636[] = "ZTE-F102i/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S70637[] = "ZTE-F102/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S70638[] = "ZTE-F102 H3G/2.0 ACS-NF/3.4 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S70639[] = "SAMSUNG-SGH-A767/A767UCIF2; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.14315; U; en-US) Opera 9.50";
const char S70640[] = "MOT-KINGFISHER Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S70641[] = "MOT-EX116 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S70642[] = "Mozilla/5.0 (Java; U; en-in; gt-s5233s) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S70643[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; AN10BG2DT Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70644[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; fr-fr; AN10BG2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70645[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A200 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70646[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; a200 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70647[] = "NokiaX3-00/5.0 (11.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiax3-00) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S70648[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17";
const char S70649[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I415 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S70650[] = "Mozilla/5.0 (Linux; Android 4.1.1; ME172V Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70651[] = "Mozilla/5.0 (Linux; Android 4.0.3; A100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70652[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S70653[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I535 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S70654[] = "Nokia7500/2.0 (04.62) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S70655[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS06 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70656[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-; IS06 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70657[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; IS06 Build/FRG83)";
const char S70658[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Blade Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70659[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Blade Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70660[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Blade Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70661[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ZTE Blade Build/ITL41D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70662[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZTE Blade Build/IMM76D; CyanogenMod-Eco) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70663[] = "SAMSUNG-SGH-T999/100.40101";
const char S70664[] = "SAMSUNG-GT-I9300/100.40102";
const char S70665[] = "SAMSUNG-SCH-I535/100.40102";
const char S70666[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Mobiistar_Touch_S01 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70667[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; Mobiistar Touch S01 Build/GRK39F)";
const char S70668[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia500) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S70669[] = "Mozilla/5.0 (Linux; Android 4.1.1; MI 2 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70670[] = "Mozilla/5.0 (Linux; Android 4.1.1; MI 2S Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70671[] = "Mozilla/5.0 (Linux; Android 4.1.1; MI 2SC Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70672[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TA-FONE A34 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70673[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TA-FONE A34 HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70674[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; U7015 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70675[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; U7015 Build/JRO03C)";
const char S70676[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ Ubuntu/10.10 (2.30.6-1ubuntu5) Epiphany/2.30.6";
const char S70677[] = "Mozilla/5.0 (X11; U; Linux i686; en-us) AppleWebKit/531.2+ (KHTML, like Gecko) Version/5.0 Safari/531.2+ Ubuntu/10.10 () Epiphany/2.30.2";
const char S70678[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M1050S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70679[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M1050S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70680[] = "Lenovo P700/S100 Linux/3.0.13 Android/4.0.3 Release/12.12.2011 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30";
const char S70681[] = "Lenovo P700iLenovo-P700i/S101 Linux/3.0.13 Android/4.0 Release/05.28.2012 Browser/AppleWebKit533.1 Profile/ Configuration/";
const char S70682[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70683[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Vsun i1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70684[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaE51) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S70685[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; 201F Build/V31R11K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70686[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; 201F Build/V33R12A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70687[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; 201F Build/V33R12A)";
const char S70688[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; 201F Build/V35R13E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70689[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P930 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70690[] = "SAMSUNG-GT-B7300Orange/BVIJ1 (Windows CE; Opera Mobi; U; en) Opera 9.5";
const char S70691[] = "SAMSUNG-GT-B7300/1.0 (Windows CE; Opera Mobi; U; en) Opera 9.5";
const char S70692[] = "SonyEricssonK790i/R8BF Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S70693[] = "Mozilla/4.1 (compatible; Teleca Q7; BMP 1.0.1; U; en) 240X400 LGE UN510";
const char S70694[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11 Comodo_Dragon/17.5.2.0";
const char S70695[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11 Comodo_Dragon/17.5.2.0";
const char S70696[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959 Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70697[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70698[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70699[] = "Mozilla/5.0 (Windows NT 5.1; rv:12.0) Gecko/X Firefox/12.0 PaleMoon/12.0";
const char S70700[] = "Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:12.0) Gecko/X Firefox/12.0-x64 PaleMoon/12.0-x64";
const char S70701[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B144 Safari/7534.48.3";
const char S70702[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B146 Safari/7534.48.3";
const char S70703[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B329 Safari/7534.48.3";
const char S70704[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT912 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70705[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; XT912 Build/JDQ39E)";
const char S70706[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Droid Razr M Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70707[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6800 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S70708[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P875 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70709[] = "Mozilla/5.0 (Series40; Nokia114/03.33; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S70710[] = "SonyEricssonG705u/R1BA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 model-orange JavaPlatform/JP-8.4.0";
const char S70711[] = "SonyEricssonG705/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3";
const char S70712[] = "LG-GW525-parrot/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S70713[] = "LG-GW525 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S70714[] = "LG-GW525g/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S70715[] = "TIANYU-KTOUCH/D772/Screen-176X220";
const char S70716[] = "TIANYU-KTOUCH/DT200/MIDP2.0/CLDC1.1/Screen-240x320";
const char S70717[] = "TIANYU-KTOUCH/A610/Screen-176X220";
const char S70718[] = "TIANYU-KTOUCH/D159/Screen-128X128";
const char S70719[] = "TIANYU-KTOUCH/D175/Screen-128X160";
const char S70720[] = "TIANYU-KTOUCH/X90/MIDP2.0/CLDC1.1/Screen-320X240";
const char S70721[] = "TIANYU-KTOUCH/A927/MIDP2.0/CLDC1.1/Screen-240x320";
const char S70722[] = "TIANYU-KTOUCH/FT05/MIDP2.0/CLDC1.1/Screen-240X320";
const char S70723[] = "TIANYU-KTOUCH/B858/Screen-240X320";
const char S70724[] = "TIANYU-KTOUCH/V320/MIDP2.0/CLDC1.1/Screen-240X320";
const char S70725[] = "TIANYU-KTOUCH/B922/MIDP2.0/CLDC1.1/Screen-240X320";
const char S70726[] = "TIANYU-KTOUCH/C350/MIDP2.0/CLDC1.1/Screen-240X320";
const char S70727[] = "TIANYU-KTOUCH/B2012/Screen-128X128";
const char S70728[] = "TIANYU-KTOUCH/B2016/Screen-128X128";
const char S70729[] = "TIANYU-KTOUCH/D172/Screen-128x160";
const char S70730[] = "TIANYU-KTOUCH/A660/MIDP2.0/CLDC1.1/Screen-240x320";
const char S70731[] = "TIANYU-KTOUCH/A662/MIDP2.0/CLDC1.1/Screen-240X320";
const char S70732[] = "TIANYU-KTOUCH/H711/Screen-176X220";
const char S70733[] = "TIANYU-KTOUCH/A662/MIDP2.0/CLDC1.1/Screen-240X400";
const char S70734[] = "TIANYU-KTOUCH/A168/MIDP2.0/CLDC1.1/Screen-320X430";
const char S70735[] = "TIANYU-KTOUCH/V760/MIDP2.0/CLDC1.1/Screen-240X400";
const char S70736[] = "TIANYU-KTOUCH/DT28/MIDP2.0/CLDC1.1/Screen-240x320";
const char S70737[] = "TIANYU-KTOUCH/W990/MIDP2.0/CLDC1.1/Screen-240x320";
const char S70738[] = "TIANYU-KTOUCH/A995/MIDP2.0/CLDC1.1/Screen-320x430";
const char S70739[] = "TIANYU-KTOUCH/D173/Screen-128x160";
const char S70740[] = "TIANYU-KTOUCH/A903/MIDP2.0/CLDC1.1/Screen-240X320";
const char S70741[] = "TIANYU-KTOUCH/C800/MIDP2.0/CLDC1.1/Screen-240x320";
const char S70742[] = "TIANYU-KTOUCH/H800/MIDP2.0/CLDC1.1/Screen-176X220";
const char S70743[] = "TIANYU-KTOUCH/Q981/Screen-240X320";
const char S70744[] = "TIANYU-KTOUCH/M600/MIDP2.0/CLDC1.1/Screen-240X320";
const char S70745[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; CTP810 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70746[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; CTP810 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70747[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s5380k) UCBrowser8.3.0.154/69/405/UCWEB Mobile";
const char S70748[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5380k) UCBrowser8.3.1.161/69/405/UCWEB Mobile";
const char S70749[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB855 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70750[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB855 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70751[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB855 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70752[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MB855 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70753[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P3100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S70754[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S70755[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3100B Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S70756[] = "Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia305) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S70757[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9001 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70758[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-i9001 Build/JZO54K)";
const char S70759[] = "Mozilla/5.0 (Linux; Android 4.1.1; Micromax A88 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S70760[] = "Mozilla/5.0 (Linux; Android 4.1.1; Micromax A88 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22";
const char S70761[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S70762[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; SGH-T889 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S70763[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S70764[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; SPH-L900 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S70765[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.1; en-gb; GT-N7100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S70766[] = "Mozilla/5.0 (Linux; Android 4.1.2; CINK FIVE Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70767[] = "Mozilla/5.0 (Linux; Android 4.1.1; CINK+ Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70768[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; LG-E440 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70769[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E440 Build/JZO54K)";
const char S70770[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-E440/E44010a-EUR-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70771[] = "Mozilla/5.0 (Series40; NokiaX2-00/08.25; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S70772[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S70773[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5; garmin-asus-Nuvifone-M10/1.0)";
const char S70774[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5) garmin-asus-Nuvifone-M10/1.0 Opera9.5";
const char S70775[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5; garmin-asus-Nuvifone-M10/2.0)";
const char S70776[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; SCH-I905 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S70777[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; SCH-I905 4G Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S70778[] = "Dalvik/1.5.1 (Linux; U; Android 3.1; SCH-I905 Build/HMJ37)";
const char S70779[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; T-01C Build/TER018) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S70780[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; T-01C Build/TER018)";
const char S70781[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-i809 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70782[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SCH-i809 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70783[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-i809 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S70784[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; zh-cn; SCH-i809 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70785[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SCH-i809 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S70786[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-i809 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S70787[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; zh-cn; SCH-i809 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S70788[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; CBW Blaze Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S70789[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.7.2) Gecko/X Camino/0.8.1";
const char S70790[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows NT 5.0)";
const char S70791[] = "Mozilla/4.0 (compatible; MSIE 4.0; Windows NT 5.0)";
const char S70792[] = "Mozilla/4.0 (compatible; MSIE 4.0; WindowsNT 5.0)";
const char S70793[] = "Mozilla/5.0 (compatible; MSIE 4.0; Windows NT 5.0; Trident/1.0)";
const char S70794[] = "JUC(Linux;U;Android2.3.4;Zh_cn;GT-S5830;320*480;)UCWEB7.8.0.95/139/444";
const char S70795[] = "JUC(Linux;U;Android2.3.3;Zh_cn;GT-S5830;320*480;)UCWEB7.8.0.95/139/444";
const char S70796[] = "JUC(Linux;U;Android2.3.6;Zh_cn;GT-S5830;320*480;)UCWEB7.8.0.95/139/444";
const char S70797[] = "JUC(Linux;U;Android2.3.5;Zh_cn;GT-S5830;320*480;)UCWEB7.8.0.95/139/352";
const char S70798[] = "server-bag [iPhone OS,6.0,10A403,iPad3,1]";
const char S70799[] = "iPad2,1/6.0.1 (10A523)";
const char S70800[] = "iPad3,1/6.0.1 (10A523)";
const char S70801[] = "iPad3,1/6.0 (10A403)";
const char S70802[] = "iPad3,3/6.0.1 (10A523)";
const char S70803[] = "MicromaxX279/MstarBrowser1.0/MIDP2.0Configuration/CLDC1.1";
const char S70804[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P870 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70805[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; VL-110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70806[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.1; PD_Novel Build/GINGERBREAD)";
const char S70807[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PD_Novel Build/PR71_A_BN_W_A_C_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70808[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PD_Novel Build/PR72_A_BN_W_A_C_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70809[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PD_Novel Build/PR77_A_KZ_W_A_C_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70810[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PD_Novel Build/PR73_A_BN_W_A_C_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70811[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PD_Novel Build/PR74_A_AM_W_A_C_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70812[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PD_Novel Build/PR76_A_KZ_W_A_C_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70813[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB526 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70814[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB526 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70815[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.0 Safari/535.19 Silk-Accelerated=true";
const char S70816[] = "JUC (Linux; U; 4.1.2; zh-cn; GT-I9300; 720*1280) UCWEB7.9.0.94/139/444";
const char S70817[] = "Mozilla/5.0 (Linux; Android 4.0.4; Grand X In Build/GXI_TPLV1.0.0B04) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70818[] = "sprd-TSM_T9i/1.0 Linux/2.6.35.7 Android/2.3.7 Release/01.05.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S70819[] = "sprd-TSM_T9i/1.0 Linux/2.6.35.7 Android/2.3.7 Release/12.18.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S70820[] = "sprd-TSM_T9i/1.0 Linux/2.6.35.7 Android/2.3.7 Release/01.28.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S70821[] = "sprd-TSM_T9i/1.0 Linux/2.6.35.7 Android/2.3.7 Release/01.07.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S70822[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; TSM_T9i Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70823[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; sprd-TSM_T9i/1.0 Android/2.3.7 Release/05.31.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70824[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.7/210,1.9.7.210,2012-10-25 19:53, in";
const char S70825[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +0700, kh";
const char S70826[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.22/288,1.9.22.288,2013-01-31 13:09, +0700, id";
const char S70827[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0000, gb";
const char S70828[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGP312 Build/10.1.C.0.370) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70829[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGP312 Build/10.1.C.0.370)";
const char S70830[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGP312 Build/10.1.C.0.344) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70831[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-I9100 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S70832[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9100G Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S70833[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-I9100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S70834[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5303 Build/12.0.A.2.254) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S70835[] = "LG-GW620 Mozilla/5.0 (Linux; U; Android 1.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S70836[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-; LG GW620 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S70837[] = "Mozilla/5.0 (Linux; U; Android 1.5; fr-fr; LG GW620 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S70838[] = "Motorola-QA30 UP.Browser/7.2.7.5.605 (GUI) MMP/2.0";
const char S70839[] = "MOT-QA30/00.72 UP.Browser/7.2.7.5.598 (GUI) MMP/2.0";
const char S70840[] = "MOT-QA30/00.72 UP.Browser/7.2.7.5.610 (GUI) MMP/2.0";
const char S70841[] = "MOT-QA30/00.72 UP.Browser/7.2.7.5.565 (GUI) MMP/2.0";
const char S70842[] = "MOT-QA30/00.72 UP.Browser/7.2.7.5.619 (GUI) MMP/2.0";
const char S70843[] = "MOT-QA30/00.72 UP.Browser/7.2.7.5.546 (GUI) MMP/2.0";
const char S70844[] = "MOT-QA30/00.72 UP.Browser/7.2.7.5.622 (GUI) MMP/2.0";
const char S70845[] = "SAMSUNG-GT-S5510T/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S70846[] = "SAMSUNG-GT-S5510/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S70847[] = "Mozilla/6.0 (compatible; MSIE 7.01; Windows 95)";
const char S70848[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows 95)";
const char S70849[] = "Mozilla/5.0 (compatible; MSIE 7.0; Win95; Trident/1.0)";
const char S70850[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70851[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S70852[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ru-ru; PocketBook A7 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70853[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11";
const char S70854[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11";
const char S70855[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11";
const char S70856[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5380k) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile";
const char S70857[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.8.0.6) Gecko/X SeaMonkey/1.0.4";
const char S70858[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A800 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70859[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530M Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S70860[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.53463";
const char S70861[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S70862[] = "Mozilla/5.0 (Linux; Android 4.0.4; Motorola RAZR MAXX Build/6.7.2-180_DHD-16_NA-27) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S70863[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S70864[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.53463";
const char S70865[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R760X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70866[] = "Mozilla/5.0 (Linux; Android 4.2.1; V370 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S70867[] = "Mozilla/5.0 (Linux; Android 4.2.2; V370 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S70868[] = "Mozilla/5.0 (Linux; Android 4.2.1; V370 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70869[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70870[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CLP285 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70871[] = "Mozilla/4.0 (BREW 3.1.5; U; en-us; Sanyo; Polaris/6.0/AMB) PL2700";
const char S70872[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S70873[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70874[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; GT-P7500 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70875[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; GT-P7500 Build/GWK74; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70876[] = "JUC (Linux; U; 4.0.4; zh-cn; GT-S7562; 480*800) UCWEB7.9.0.94/139/444";
const char S70877[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; YL-Coolpad_5860S/4.0.011.X.5860S; 480*800; CTC/2.0) CoolpadWebkit/534.30";
const char S70878[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; 5860S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70879[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; 5860S Build/IMM76I)";
const char S70880[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; 5860S Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S70881[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70882[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; T970 Build/IMM76D)";
const char S70883[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PAP4020DUO Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70884[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; NGM Miracle Build/IMM76D) AppleWebKit/534.30 KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70885[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; NGM_Miracle Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70886[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ph; A919i Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70887[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; A919i Build/JZO54K)";
const char S70888[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F180L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70889[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F180K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70890[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F180S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70891[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F180 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70892[] = "Nokia306/2.0 (05.93) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia306) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S70893[] = "Mozilla/5.0 (Linux; Android 4.2.2; SC-04E Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70894[] = "Mozilla/5.0 (Linux; Android 4.1.1; Vodafone 875 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S70895[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; NOVO7PALADIN Build/IML74K)";
const char S70896[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; NOVO7PALADIN Build/ICS_MR0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70897[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; NOVO7PALADIN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70898[] = "Mozilla/5.0 (X11; Linux i686; rv:2.0pre) Gecko/X Firefox/4.0pre SeaMonkey/2.1b3";
const char S70899[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.3a4pre) Gecko/X SeaMonkey/2.1a1pre";
const char S70900[] = "Mozilla/5.0 (Series40; Nokia5130c-2/07.97; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S70901[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ASTROPLUS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70902[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ASTROPLUS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70903[] = "Mozilla/5.0 (Linux; Android 4.0.4; 201M Build/7.7.1Q-164_SMJ-165) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70904[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Butterfly Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S70905[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Butterfly Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70906[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Butterfly Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S70907[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6102) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S70908[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M9 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70909[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; M9 Build/JRO03H)";
const char S70910[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE T790 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70911[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9200 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S70912[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9200 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70913[] = "INQ/Q1-200H/1.0.65 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S70914[] = "INQ/Q1-200H/1.0.70 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S70915[] = "INQ/Q1-200H/1.0.56 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S70916[] = "INQ/Q1-200H/1.0.48 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S70917[] = "INQ/Q1-200H/1.0.63 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S70918[] = "INQ/Q1-200H/1.0.45 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S70919[] = "NOKIAE5-00/UC Browser7.9.0.102/28/355/UCWEB";
const char S70920[] = "NOKIAE5-00.2/UC Browser7.9.0.102/28/355/UCWEB";
const char S70921[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:10.0.2) Gecko/X Yoono/1.8.32";
const char S70922[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ja-jp; N-06D Build/AX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70923[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; N-06D Build/AX)";
const char S70924[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_5860/2.3.002.X.5860; 480*800; CTC/2.0) CoolpadWebkit/533.1";
const char S70925[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; 5860 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S70926[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; 5860 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70927[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_5860+/2.3.009.X.5860+; 480*800; CTC/2.0) CoolpadWebkit/533.1";
const char S70928[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; 5860 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S70929[] = "Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; 5860 Build GRJ90) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S70930[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_5860 /2.3.009.X.5860 ; 480*800; CTC/2.0) CoolpadWebkit/533.1";
const char S70931[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC HD2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70932[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC HD2 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70933[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; NexusHD2 Build/JZO54K)";
const char S70934[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC HD2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70935[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HD2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70936[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; Infinity HD2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70937[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NexusHD2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70938[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; Infinity HD2+ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70939[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HD2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70940[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; HTC HD2 Build/DI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70941[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GEM7112 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S70942[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-I500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70943[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70944[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70945[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-I500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70946[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70947[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I500 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70948[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030E Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S70949[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S70950[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7300B Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S70951[] = "Opera/9.80 (X11; FreeBSD 9.1-STABLE amd64) Presto/2.12.388 Version/12.15";
const char S70952[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAC-70051 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70953[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Mobile Safari/535.19 Silk-Accelerated=false";
const char S70954[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax P560 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70955[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70956[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70957[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70958[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160LV Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70959[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160S/K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70960[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI H881C Build/HuaweiH881C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S70961[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9508 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70962[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9508 Build/JDQ39)";
const char S70963[] = "ZTE-T95 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1 firmware/T95 T03";
const char S70964[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC HD2 Build/IMM76I; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70965[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC HD2 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70966[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC HD2 Leo Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70967[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11";
const char S70968[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11";
const char S70969[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; TOOKY A19 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70970[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; TOOKY A9 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70971[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TOOKY T1982 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70972[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TOOKY T1982 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S70973[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; TOOKY A81 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70974[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TOOKY T1982 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S70975[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TOOKY T1981 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70976[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TOOKY T85 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70977[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.0.3/X (like Mobile/10A525 Safari/8536.25)";
const char S70978[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.0.3/X (like Mobile/10A525 Safari/8536.25)";
const char S70979[] = "Mozilla/5.0 (PlayStation Vita 2.05) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S70980[] = "Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101G Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S70981[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A27 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70982[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; CROSS A27 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70983[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6502 Build/10.3.A.0.423) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S70984[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6502 Build/10.3.A.0.423) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70985[] = "Mozilla/5.0 (Linux; Android 4.0.3; LIFETAB_P9514 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S70986[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; SPX-5_3G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S70987[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; HTC6990LVW)";
const char S70988[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Fly_IQ442 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70989[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Fly_IQ442 Build/IMM76D)";
const char S70990[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7560M Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S70991[] = "Mozilla/5.0 (Linux; Android 4.1.1; MI 2 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S70992[] = "Mozilla/5.0 (Linux; Android 4.1.1; MI 2 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S70993[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9500 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S70994[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9A405 Safari/7534.48.3";
const char S70995[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9A334 Safari/7534.48.3";
const char S70996[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; QMobile Noir A10 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70997[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; QMobile A10 Noir Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70998[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Karbonn A10 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S70999[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A10 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71000[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-05D Build/9.1.C.0.475) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71001[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-05D Build/9.1.C.0.475) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71002[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SO-05D Build/9.1.C.0.475)";
const char S71003[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-05D Build/9.1.C.1.103) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71004[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-05D Build/9.1.C.1.103) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71005[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S71006[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S71007[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-N719 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S71008[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7105 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S71009[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-D710 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71010[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P870 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71011[] = "SAMSUNG-SGH-A197/A197UCJH2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S71012[] = "Mozilla/4.0 (compatible; MSIE 6.0; PPC Mac OS X 10.4.9; Tasman 1.0)";
const char S71013[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.0) Gecko/X Firefox/12.0 Kylo/1.1.1";
const char S71014[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.3.5 Safari/534.34";
const char S71015[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.21 (KHTML, like Gecko) QupZilla/1.3.5 Safari/537.21";
const char S71016[] = "Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia305) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S71017[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DEM752NC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71018[] = "Mozilla/5.0 (Linux; Android 4.1.1; Orange Daytona Build/HuaweiG510-0200) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71019[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0200 Build/HuaweiG510-0200) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71020[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0100 Build/HuaweiG510-0100) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71021[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71022[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5503 Build/10.1.1.A.1.310) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71023[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I927 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S71024[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030A Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71025[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030A Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S71026[] = "Mozilla/5.0 (Linux; Android 4.0.3; A501 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S71027[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; GT-I9500 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S71028[] = "NOKIA6120c/UC Browser8.0.4.121/28/444/UCWEB";
const char S71029[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-S5820 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71030[] = "SAMSUNG-GT-S5820_TD/1.0 Android/2.3.4 Release/12.15.2010 Browser/AppleWebKit533.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S71031[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5820 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S71032[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-S5820 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S71033[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5820 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71034[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5820 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S71035[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-S5820 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S71036[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71037[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71038[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Xoom Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71039[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Liberty Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71040[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71041[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71042[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3113 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71043[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71044[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71045[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.1 Mobile Safari/535.19 Silk-Accelerated=true";
const char S71046[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GoTab GBT9 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71047[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5235) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S71048[] = "Mozilla/5.0 (Linux; Android 4.0.3; ThinkPad Tablet Build/ThinkPadTablet_A400_03) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S71049[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R760 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S71050[] = "Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI P2-6011 Build/HuaweiP2-6011) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71051[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; SAMSUNG; SPH-I800; SPRINT)";
const char S71052[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; spice) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S71053[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; generic) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S71054[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mozilla) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S71055[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sunmicro) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S71056[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; jbed-fastbcc) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S71057[] = "UCWEB/2.0 (Java; U; MIDP-2.0; Pt-BR; mozilla) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S71058[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; karbonn) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S71059[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile TRUSTED/1.0";
const char S71060[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; 11a h700tecno) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S71061[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; bird) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S71062[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I527 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71063[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I527 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S71064[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I527M Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71065[] = "Nokia6101/2.0 (04.90) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S71066[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Nokia6124c/4.34; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S71067[] = "NOKIA6124c";
const char S71068[] = "Mozilla/5.0 (compatible; Konqueror/3.4) KHTML/3.4.2 (like Gecko)";
const char S71069[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Dell Streak 10 Pro Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S71070[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; Dell_Streak_10_Pro Build/HTJ85B; 1280*752) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S71071[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.17 (KHTML, like Gecko) Ubuntu/12.04 Chromium/24.0.1312.2 Chrome/24.0.1312.2 Safari/537.17";
const char S71072[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.17 (KHTML, like Gecko) Ubuntu/12.04 Chromium/24.0.1312.2 Chrome/24.0.1312.2 Safari/537.17";
const char S71073[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I777 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71074[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I777 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S71075[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I777 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71076[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P700 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S71077[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/10A523 Safari/7534.48.3";
const char S71078[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/10A403 Safari/7534.48.3";
const char S71079[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/10A550 Safari/7534.48.3";
const char S71080[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/10A407 Safari/7534.48.3";
const char S71081[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/10A8426 Safari/7534.48.3";
const char S71082[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/10A8500 Safari/7534.48.3";
const char S71083[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.31 (KHTML, like Gecko) Chrome/17.0.558.0 Safari/534.31 UCBrowser/9.1.0.297";
const char S71084[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-78) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S71085[] = "Nokia114/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia114) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S71086[] = "Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S71087[] = "Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71088[] = "Mozilla/5.0 (Linux; Android 4.0.3; E350 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71089[] = "Nokia6670/2.0 (6.0522.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S71090[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; Dell Streak Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S71091[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; Dell Streak Build/Donut AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/ 525.20.1";
const char S71092[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC-X710a; en-ca) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S71093[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7";
const char S71094[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7";
const char S71095[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7";
const char S71096[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7";
const char S71097[] = "Mozilla/5.0 (X11; U; Linux x86; sv-SE; rv:1.8.1.12) Gecko/X Ubuntu/8.04 (hardy) Firefox/2.0.0.12";
const char S71098[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; P-05D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71099[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; P-05D Build/GRJ90)";
const char S71100[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I500 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71101[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I500 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71102[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I500 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71103[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; ZMCK900 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71104[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5";
const char S71105[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5";
const char S71106[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5";
const char S71107[] = "Mozilla/5.0 (Java; U; en-us; mauie800) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S71108[] = "Mozilla/5.0 (Java; U; id; mauie800) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S71109[] = "Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia7210supernova) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S71110[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE51) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/355/UCWEB Mobile";
const char S71111[] = "ZTEU790_TD/1.0 Linux/2.6.35 Android/2.3 Release/7.5.2012 Browser/AppleWebKit533.1";
const char S71112[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE U790 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71113[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE U790 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S71114[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; ZTE U790 Build/GRK39F)";
const char S71115[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; SPNG9716DC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71116[] = "Mozilla/5.0 (compatible; MSIE 9.0; Opera 11.0; Windows Server 2003; SV1) Admin search system: nikolaydovydov@km.ru";
const char S71117[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I415 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71118[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT300 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S71119[] = "Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF700T Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S71120[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; UMI X2 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71121[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; UMI X2 Build/JOP40D)";
const char S71122[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E440 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71123[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X; zh-CN) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d UCBrowser/9.0.1.284 Mobile";
const char S71124[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X; zh-CN) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 UCBrowser/9.0.1.284 Mobile";
const char S71125[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X; zh-CN) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 UCBrowser/9.0.1.284 Mobile";
const char S71126[] = "LGE-VX9100/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S71127[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; GS01 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71128[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)";
const char S71129[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; O+_8.9_Android Build/OplusOplus_A1000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71130[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30a Build/9.1.A.0.489) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71131[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30a Build/9.1.A.0.489) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S71132[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC7-00) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S71133[] = "Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S71134[] = "Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71135[] = "Mozilla/5.0 (Linux; Android 4.0.2; Galaxy Nexus Build/ICL53F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71136[] = "Mozilla/5.0 (Linux; Android 4.0.1; Galaxy Nexus Build/ITL41F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71137[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5282 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S71138[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A390_ROW Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71139[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6300 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71140[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ADR6300 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71141[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6300 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71142[] = "Opera/9.80 (Android 3.1; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.10.254 Version/12.00";
const char S71143[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I777 Build/Task650 & Ktoonsez AOKP) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.1 Mobile Safari/534.30";
const char S71144[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I777 Build/Task650 & Ktoonsez AOKP) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71145[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I777 Build/Task650 & Ktoonsez AOKP)";
const char S71146[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Celkon A88 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71147[] = "Mozilla/5.0 (masking-agent; rv:16.0) Gecko/16.0 Firefox/16.0";
const char S71148[] = "Mozilla/5.0 (compatible; MSIE 9.0; Gecko/X Firefox/16.0; Trident/5.0)";
const char S71149[] = "Mozilla/4.0 (compatible; MSIE 8.0; Gecko/X Firefox/16.0; Trident/4.0)";
const char S71150[] = "Mozilla/5.0 (compatible; MSIE 10.0; Gecko/X Firefox/16.0; Trident/6.0)";
const char S71151[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I915 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S71152[] = "Nokia309/2.0 (05.85) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia309) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S71153[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030D Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71154[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030A Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71155[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S71156[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; pt-br; C6503 Build/10.3.A.0.423) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71157[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; C6503 Build/10.3.1.A.0.244)";
const char S71158[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; C6503 Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71159[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2690) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S71160[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S71161[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus S 4G Build/JRO03R) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71162[] = "Mozilla/5.0 (Linux; Android 4.1.1; E270BSA Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S71163[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT912 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71164[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; tr-tr; ZTE-JOE Build/VF945_V02c_PT) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S71165[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; tr-tr; Vodafone 945 Build/VF945_V02e_TR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S71166[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Vodafone 945 Build/VF945_V01a_QA) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S71167[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; cs-cz; Vodafone 945 Build/VF945_V02d_CZ) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S71168[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-za; Vodafone 945 Build/VF945_V02b_SA) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S71169[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-pt; Vodafone 945 Build/VF945_V02c_PT) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S71170[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; it-it; Vodafone 945 Build/VF945_V02d_DE) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S71171[] = "LG-GC900-Bouygues/V10a Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S71172[] = "LG-GC900/V10a Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S71173[] = "LG-GC900-orange/V10n Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S71174[] = "SCH-U750L/1.0 NetFront/3.0.22.2.17 (GUI) MMP/2.0";
const char S71175[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.83 Safari/535.11";
const char S71176[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T02A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71177[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T02A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71178[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.8b5) Gecko/X Firefox/1.4.1";
const char S71179[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; it; rv:1.8b5) Gecko/X Firefox/1.4.1";
const char S71180[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-1 Build/HuaweiU8950-1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71181[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-51 Build/HuaweiU8950N-51) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71182[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950D Build/HuaweiU8950D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71183[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-51 Build/HuaweiU8950-51) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71184[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-1 Build/HuaweiU8950N-1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71185[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, -0200, br";
const char S71186[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +0000, gb";
const char S71187[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PNDPP4MT9G3 Build/ICS.g08refmid1061.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71188[] = "Mozilla/5.0 (Linux; Android 4.2.2; One S Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71189[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One S Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71190[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I757M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71191[] = "Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_S9714 Build/JRO03R) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S71192[] = "SonyEricssonV600i/R2K Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S71193[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/5.2.1.1588 Safari/532.4";
const char S71194[] = "GoogleEarth/5.2.1.1588(Windows;Microsoft Windows (6.1.7600.0);es-419;kml:2.2;client:Free;type:default)";
const char S71195[] = "GoogleEarth/5.2.1.1588(Windows;Microsoft Windows (6.1.7600.0);en-US;kml:2.2;client:Free;type:default)";
const char S71196[] = "GoogleEarth/5.2.1.1588(Windows;Microsoft Windows (6.1.7601.1);en-US;kml:2.2;client:Free;type:default)";
const char S71197[] = "GoogleEarth/5.2.1.1588(Windows;Microsoft Windows (6.1.7601.1);de-DE;kml:2.2;client:Free;type:default)";
const char S71198[] = "Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; SMT-i9100 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71199[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; SMT-i9100 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71200[] = "Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia7210supernova) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S71201[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71202[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71203[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; G2 Build/ICS.Trio_Stealth_G2.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71204[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; G2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71205[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; G2 Build/ICS.Trio_Stealth_G2.X)";
const char S71206[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; EK-GC110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71207[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6802 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S71208[] = "Mozilla/5.0 (Linux; Android 4.0.4; MB886 Build/7.7.1Q-144_MB886_MR1-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71209[] = "SAMSUNG-SGH-M200/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S71210[] = "Vodafone/1.0/SAMSUNG-SGH-M200/AMHI4/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S71211[] = "SAMSUNG-GT-C5130/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S71212[] = "SAMSUNG-GT-C5130S/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S71213[] = "SAMSUNG-GT-C5130U/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S71214[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) SAMSUNG-SGH-i780/1.0";
const char S71215[] = "SAMSUNG-SGH-i780/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) SAMSUNG-SGH-i780/1.0";
const char S71216[] = "SAMSUNG-SGH-i780ORANGE/BVGK3 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S71217[] = "SAMSUNG-SGH-i780TIM/AIHA3 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S71218[] = "SAMSUNG-SGH-i780SFR/AEHC6_HD2 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S71219[] = "SAMSUNG-SGH-i780V/MJHA3 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S71220[] = "SAMSUNG-SGH-i780/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S71221[] = "LG-GU297/V10d Teleca/Q7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S71222[] = "LG-GU290f/V10f Teleca/Q7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S71223[] = "Mozilla/4.0 (compatible; MSIE 6.0; PPC Mac OS X 10.5.8; Tasman 1.0)";
const char S71224[] = "Mozilla/4.0 (compatible; MSIE 6.0; PPC Mac OS X 10.5.6; Tasman 1.0)";
const char S71225[] = "AppleCoreMedia/1.0.0.8G4 (iPod; U; CPU OS 4_3_1 like Mac OS X; en_us)";
const char S71226[] = "AppleCoreMedia/1.0.0.8J2 (iPod; U; CPU OS 4_3_3 like Mac OS X; en_us)";
const char S71227[] = "AppleCoreMedia/1.0.0.8L1 (iPod; U; CPU OS 4_3_5 like Mac OS X; en_us)";
const char S71228[] = "AppleCoreMedia/1.0.0.8K2 (iPod; U; CPU OS 4_3_4 like Mac OS X; en_us)";
const char S71229[] = "AppleCoreMedia/1.0.0.8H7 (iPod; U; CPU OS 4_3_2 like Mac OS X; en_us)";
const char S71230[] = "AppleCoreMedia/1.0.0.8F190 (iPod; U; CPU OS 4_3 like Mac OS X; en_us)";
const char S71231[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.24 (KHTML, like Gecko) Iron/11.0.700.2 Chrome/11.0.700.2 Safari/534.24";
const char S71232[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Iron/11.0.700.3 Chrome/11.0.700.3 Safari/534.24";
const char S71233[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; TT102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71234[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S7230E/S723EXXLB3; U; Bada/2.0; en-us) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile WQVGA SMM-MMS/1.2.0 OPN-B";
const char S71235[] = "Mozilla/5.0 (Series40; Nokia113/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S71236[] = "Mozilla/5.0 (Series40; Nokia113/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5";
const char S71237[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4";
const char S71238[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4";
const char S71239[] = "Mozilla/5.0 (Linux; Android 4.0.4; N9100 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71240[] = "Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 101G9 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S71241[] = "Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 101G9 Build/MR1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S71242[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8160 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71243[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8160L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71244[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8160P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71245[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SHL22 Build/S6290) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71246[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SHL22 Build/S6290)";
const char S71247[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R760X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71248[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.0.5) Gecko/X Firefox/3.0.5 Flock/2.0.3";
const char S71249[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.1) Gecko/X Firefox/3.0.1 Flock/2.0b2";
const char S71250[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:2.0.1) Gecko/X Firefox/4.0.1 Camino/2.2.1";
const char S71251[] = "MQQBrowser/37 (iOS 4S; U; CPU like Mac OS X; zh-cn)";
const char S71252[] = "MQQBrowser/37 (iOS 4; U; CPU like Mac OS X; zh-cn)";
const char S71253[] = "MQQBrowser/37 (iOS 5; U; CPU like Mac OS X; zh-cn)";
const char S71254[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A97i Build/CelkonA97i_Celkon) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71255[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; X21 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71256[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5800d-1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile  3gpp-gba";
const char S71257[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5800 XpressMusic) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S71258[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S71259[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S71260[] = "DoCoMo/1.0/N505i/c20/TB/W24H12";
const char S71261[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0) Asus;Galaxy6";
const char S71262[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/ 7.0; Asus;Galaxy6)";
const char S71263[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/ 7.0; WpsLondonTest; Asus;Galaxy6)";
const char S71264[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.1.1.0 Chrome/16.0.912.63 Safari/535.7";
const char S71265[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.1.1.0 Chrome/16.0.912.63 Safari/535.7";
const char S71266[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/3.1.1 Mobile/7A280f Safari/525.20";
const char S71267[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/525.181 (KHTML, like Gecko) Version/3.1.1 Mobile/5H11 Safari/525.20";
const char S71268[] = "Opera/8.50 (Windows 98; U; en)";
const char S71269[] = "Opera/8.53 (Windows 98; U; en)";
const char S71270[] = "Opera/8.54 (Windows 98; U; en)";
const char S71271[] = "Opera/8.51 (Windows 98; U; en)";
const char S71272[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; en) Opera 8.0";
const char S71273[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; ru) Opera 8.02";
const char S71274[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; ru) Opera 8.01";
const char S71275[] = "Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S71276[] = "Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S71277[] = "Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/536.5 (KHTML like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S71278[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S560 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71279[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:17.0) Gecko/17.0 Firefox/17.0 slurp";
const char S71280[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:17.0) Gecko/X Firefox/17.0 slurp";
const char S71281[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:17.0) Gecko/X Firefox/17.0 slurp slurp";
const char S71282[] = "Mozilla/5.0 (Windows NT 6.1; rv:17.0) Gecko/X Firefox/17.0 slurp";
const char S71283[] = "Mozilla/5.0 (Linux; Android 4.1.1; ST10 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S71284[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; myTouch 4g Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71285[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; myTouch 4g Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71286[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; myTouch 4G Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71287[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71288[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71289[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-D710 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71290[] = "Mozilla/5.0 (Linux; Android 4.2.1; SPH-D710 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71291[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-D710 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S71292[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; w30hd(QuadCore) Build/W30hd) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71293[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT29i Build/9.1.B.0.411) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71294[] = "Mozilla/5.0 (Linux; Android 4.0.4; Next7P12 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S71295[] = "Mozilla/5.0 (Linux; Android 4.0.4; Next7P12-8G Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S71296[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N7000 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S71297[] = "Mozilla/5.0 (Series40; Nokia308/07.55; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.36";
const char S71298[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6102) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S71299[] = "Nokia200/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia200/UC Browser8.0.3.107/69/444";
const char S71300[] = "Mozilla/5.0 (X11; Linux i686; rv:8.0.1) Gecko/X Firefox/8.0.1 SeaMonkey/2.5";
const char S71301[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Scroll Excel Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71302[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.5) Gecko/X Firefox/10.0.5 SeaMonkey/2.7.2";
const char S71303[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.2) Gecko/X Firefox/10.0.2 SeaMonkey/2.7";
const char S71304[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.4) Gecko/X Firefox/10.0.4 SeaMonkey/2.7.2";
const char S71305[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.7) Gecko/X Firefox/10.0.7 SeaMonkey/2.7.2";
const char S71306[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.11) Gecko/X Firefox/10.0.11 SeaMonkey/2.7.2";
const char S71307[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.12) Gecko/X Firefox/10.0.12 SeaMonkey/2.7.2";
const char S71308[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.10) Gecko/X Firefox/10.0.10 SeaMonkey/2.7.2";
const char S71309[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.9) Gecko/X Firefox/10.0.9 SeaMonkey/2.7.2";
const char S71310[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Karbonn_A18 Build/KarbonnA18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71311[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5253) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/405/UCWEB Mobile";
const char S71312[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTL21 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71313[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A1+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S71314[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; W30(QuadCore) Build/W30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71315[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SC-74JB Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71316[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P970 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71317[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A35) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S71318[] = "Mozilla/5.0 (Linux; Android 4.0.3; E350 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71319[] = "Mozilla/5.0 (Linux; Android 4.0.4; E350 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71320[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71321[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G X710s Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71322[] = "Mozilla/5.0 (Linux; Android 4.1.2; Titanium S5 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71323[] = "Mozilla/5.0 (Series40; NokiaC3-00/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S71324[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; T2V Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71325[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Aurora-II Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71326[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; Aurora-II Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71327[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Aurora-II Build/ICS.Aurora-II.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71328[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P5100 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S71329[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5100 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S71330[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAN79) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S71331[] = "Mozilla/5.0 (X11; SunOS i86pc; rv:18.0) Gecko/X Firefox/18.0";
const char S71332[] = "Mozilla/5.0 (PlayStation Vita 2.11) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S71333[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S71334[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; A1-811 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71335[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N7000 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S71336[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) BlackBerry8707/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/150";
const char S71337[] = "Nokia5800 XpressMusic/UCBrowser7.7.0.81/50/999";
const char S71338[] = "Nokia5800 XpressMusic";
const char S71339[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.8.1.5pre) Gecko/X Firefox/2.0.0.4 Navigator/9.0b2";
const char S71340[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US; rv:1.8.1.8pre) Gecko/X Firefox/2.0.0.8 Navigator/9.0.0.1";
const char S71341[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; LG-LU3000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71342[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; LG-LU3000 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71343[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sr-rs; Telenor_OneTouch Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71344[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; Telenor_OneTouch Build/qltsar Telenor One Touch Minimal Edition v1.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71345[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P990 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71346[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P990 Build/GRJ22; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71347[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; LG-P990 Build/IMM76L; CyanogenMod-9.0.0; Owains Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71348[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; X710d Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71349[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; POV_TAB-PROTAB25XXL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71350[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; POV_TAB-PROTAB25XXL8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71351[] = "Mozilla/5.0 (Linux; Android 4.0.4; KFJWI Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S71352[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Cynus F3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71353[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Cynus F3 Build/JRO03C)";
const char S71354[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R830 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S71355[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 UCBrowser/9.1.0.287 Mobile";
const char S71356[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 UCBrowser/9.1.0.287 Mobile";
const char S71357[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 UCBrowser/9.1.0.287 Mobile";
const char S71358[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 UCBrowser/9.1.0.287 Mobile";
const char S71359[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 UCBrowser/9.1.0.287 Mobile";
const char S71360[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145 UCBrowser/9.1.0.287 Mobile";
const char S71361[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 UCBrowser/9.1.0.287 Mobile";
const char S71362[] = "Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF700T Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S71363[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax A110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71364[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S71365[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9100G Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S71366[] = "Opera/9.80 (Windows ME; U; en) Presto/2.6.30 Version/10.63";
const char S71367[] = "Opera/9.80 (Windows ME; U; en) Presto/2.6.30 Version/10.62";
const char S71368[] = "Opera/9.80 (Windows ME; U; en) Presto/2.6.30 Version/10.61";
const char S71369[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; BLU_Elite_3.8 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71370[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:18.0) Gecko/X Firefox/18.0 slurp";
const char S71371[] = "Mozilla/5.0 (Windows NT 6.1; rv:18.0) Gecko/X Firefox/18.0 slurp";
const char S71372[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:18.0) Gecko/X Firefox/18.0 slurp slurp";
const char S71373[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; GIONEE_Gpad_G1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71374[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I777 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71375[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-i777 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71376[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I467 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S71377[] = "Mozilla/5.0 (Linux; Android 4.0.4; C525c Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36";
const char S71378[] = "Mozilla/5.0 (Linux; Android 4.2.1; Lenovo P780 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71379[] = "Mozilla/5.0 (Linux; Android 4.2.1; Lenovo P780_ROW Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71380[] = "Mozilla/4.0 (compatible; MSIE 4.5; Mac_PowerPC)";
const char S71381[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19";
const char S71382[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19";
const char S71383[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19";
const char S71384[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 Sleipnir/3.8.4";
const char S71385[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 Sleipnir/3.8.4";
const char S71386[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/3.8.4)";
const char S71387[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-B5330 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71388[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-B5330L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71389[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Desire S Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71390[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC Saga Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71391[] = "Nokia7230/5.0 (10.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia7230) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S71392[] = "Mozilla/5.0 (Linux; Android 4.1.1; CT920 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S71393[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0000, gb";
const char S71394[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3110 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, us";
const char S71395[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, -0500, us";
const char S71396[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-P3110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0100, fr";
const char S71397[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC first Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71398[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC first Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S71399[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian97-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S71400[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaN97) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S71401[] = "Mozilla/5.0 (Linux; Android 4.1.2; ADR910L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S71402[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ALCATEL ONE TOUCH 4030X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S71403[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ALCATEL ONE TOUCH 4030X Build/JRO03C)";
const char S71404[] = "HTC_Snap_S521 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S71405[] = "LG-S310/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S71406[] = "NokiaC5-00.2/UC Browser7.9.0.102/28/355/UCWEB";
const char S71407[] = "NokiaC5-00/UC Browser7.9.0.102/28/355/UCWEB";
const char S71408[] = "Mozilla/5.0   (X11; U; FreeBSD i386; en-US; rv:1.7.8) Gecko/X Firefox/1.0.4";
const char S71409[] = "Mozilla/5.0 (X11; U; FreeBSD i386; en-US; rv:1.7.6) Gecko/X Firefox/1.0.1";
const char S71410[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; Amazing A1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71411[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; Amazing A1 Build/GRK39F)";
const char S71412[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Incredible 2 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71413[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Incredible 2 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71414[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Incredible 2 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71415[] = "Mozilla/5.0 (Linux; Android 4.0.3; MD_LIFETAB_P9516 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S71416[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:15.0) Gecko/X Thunderbird/15.0.1";
const char S71417[] = "Mozilla/5.0 (Linux; Android 4.0.4; KYL21 Build/019.0.3b00) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71418[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.2; Software Development; QQ:X; http://www.zk1t.com/)";
const char S71419[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.2; Win64; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; IDCRL 8.0.1132.0; IDCRL-cfg 13.500.20639.0; App svchost.exe, 6.2.9200.16420, {DF60E2DF-88AD-4526-AE21-83D130EF0F68})";
const char S71420[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.2; WOW64; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729)";
const char S71421[] = "Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 6.2; Trident/3.0)";
const char S71422[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F100S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71423[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S71424[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I927 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71425[] = "SEC-SGHU600-ORANGE/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S71426[] = "SEC-SGHU600B-ORANGE/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S71427[] = "SEC-SGHU600/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S71428[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9A5248d Safari/6533.18.5";
const char S71429[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9A405 Safari/6533.18.5";
const char S71430[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en_us) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5";
const char S71431[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; 6534.46) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5";
const char S71432[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5 DiigoBrowser";
const char S71433[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X; en_us) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5";
const char S71434[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9A334 Safari/6533.18.5";
const char S71435[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9A406 Safari/6533.18.5";
const char S71436[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0 Mobile/9A334 Safari/7534.48.3";
const char S71437[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5";
const char S71438[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S71439[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71440[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SPH-D700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71441[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71442[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D700 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71443[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71444[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SPH-D700 Build/IML74K)";
const char S71445[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; SPH-D700 Build/IMM76L)";
const char S71446[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDX Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71447[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDX Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71448[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDX Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71449[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; S22 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71450[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; S22 Build/IMM76D)";
const char S71451[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A89 Build/CelkonCelkon_A89) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71452[] = "Micromax X446/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S71453[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SYMPHONY_W90 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71454[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SYMPHONY W90 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71455[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; M030 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71456[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; fnac 4.5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71457[] = "Mozilla/5.0 (Linux; Android 4.2.2; skz_tenderloin Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S71458[] = "Mozilla/5.0 (Linux; Android 4.2.2; cm_tenderloin Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S71459[] = "Mozilla/5.0 (Linux; Android 4.2.1; e2001_v89_jbl1a698 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71460[] = "Mozilla/5.0 (Linux; Android 4.2.2; Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71461[] = "Mozilla/5.0 (Linux; Android 4.2.1; e2001_v89_gq2008s Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71462[] = "Mozilla/5.0 (Linux; Android 4.2.1; v89_jbl1a698 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71463[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Spark TV Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71464[] = "Mozilla/5.0 (Linux; Android 4.0.4; EVO Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S71465[] = "Mozilla/5.0 (Linux; Android 4.0.3; EVO Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S71466[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P765 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71467[] = "Mercury3/4.3.3 CFNetwork/485.10.2 Darwin/10.3.1";
const char S71468[] = "SpringBoard/50 CFNetwork/485.2 Darwin/10.3.1";
const char S71469[] = "Million%20HD%20WPs/8.80.1 CFNetwork/485.10.2 Darwin/10.3.1";
const char S71470[] = "Web/1.0 CFNetwork/467.12 Darwin/10.3.1";
const char S71471[] = "Terra/1.3.5 CFNetwork/467.12 Darwin/10.3.1";
const char S71472[] = "GQMagazine/1.0 CFNetwork/485.10.2 Darwin/10.3.1";
const char S71473[] = "AtomicBrowser/6.0.1 CFNetwork/467.12 Darwin/10.3.1";
const char S71474[] = "Hot%20Wallpapers/9.12.1 CFNetwork/485.10.2 Darwin/10.3.1";
const char S71475[] = "AtomicLite/5.6.0 CFNetwork/485.10.2 Darwin/10.3.1";
const char S71476[] = "Clipish/9.29.1 CFNetwork/485.10.2 Darwin/10.3.1";
const char S71477[] = "Perfect%20Browser/5.96 CFNetwork/485.2 Darwin/10.3.1";
const char S71478[] = "Celeb%20Wallpapers/9.13.1 CFNetwork/485.10.2 Darwin/10.3.1";
const char S71479[] = "HOT%20HD%20WPs/9.19.1 CFNetwork/485.10.2 Darwin/10.3.1";
const char S71480[] = "360%20Browser/3.1.1 CFNetwork/485.10.2 Darwin/10.3.1";
const char S71481[] = "2%20Million%20WPs/9.16.1 CFNetwork/485.2 Darwin/10.3.1";
const char S71482[] = "browser/31759 CFNetwork/485.10.2 Darwin/10.3.1";
const char S71483[] = "LinkedIn/5.0.1 CFNetwork/485.10.2 Darwin/10.3.1";
const char S71484[] = "Mercury3Free/6.0.1 CFNetwork/485.10.2 Darwin/10.3.1";
const char S71485[] = "Images/1.4 CFNetwork/485.10.2 Darwin/10.3.1";
const char S71486[] = "Wallpapers/1.6 CFNetwork/485.10.2 Darwin/10.3.1";
const char S71487[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; 101P Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71488[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; 101P Build/GRJ90)";
const char S71489[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; P300(Funbook) Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71490[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; P300(Funbook) Build/IML74K)";
const char S71491[] = "Opera/9.80 (Linux; Opera Mobi/8; MeeGo; U; en-GB) Presto/2.10.254 Version/12.00";
const char S71492[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Mobiistar Touch S03 Build/VENUS_00.03.142.H1091) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71493[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Mobiistar Touch S03 Build/VENUS_00.03.155.H1091A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71494[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Mobiistar Touch S03 Build/VENUS_00.03.166.H1091A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71495[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) MQQBrowser/2.8 Mobile/8J2 Safari/7534.48.3";
const char S71496[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) MQQBrowser/2.8 Mobile/8L1 Safari/7534.48.3";
const char S71497[] = "Tweetcaster for iPhone 3.0.0.13133";
const char S71498[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; AT-AS43D3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71499[] = "Mozilla/5.0 (Linux; Android 2.3.5; HTC Desire S Build/GRJ90) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S71500[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; th-th; i-mobile IQ 5.3 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71501[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; i-mobile IQ 5.3 Build/JOP40D)";
const char S71502[] = "Mozilla/5.0 (Linux; Android 4.1.2; N861 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S71503[] = "Mozilla/5.0 (Linux; Android 4.0.4; C811 4G Build/C811M020) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71504[] = "MOT-L7e/08.01.0AR MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/1.0";
const char S71505[] = "MOT-L7_MinhThuc/00.00.00 MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S71506[] = "MOT-L7/08.B7.DCR MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S71507[] = "MOT-L7_V2.1/08.B7.ACR MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S71508[] = "MOT-L7i/AAUG2103AA 08.02.06R/10.21.2005 MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/1.0";
const char S71509[] = "MOT-L7 .T.H/08.B7.DCR MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S71510[] = "MOT-L7_Itunes/00.00.00 MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S71511[] = "MOT-L7v/08.B7.ABR MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S71512[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; Motorola_i1 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S71513[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I747 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71514[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03R; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71515[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L710 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71516[] = "AT-AS45SE Linux/3.0.13 Android/4.0.4 Release/09.28.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S71517[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; AT-AS45SE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71518[] = "Mobiistar Touch Kem 452 Linux/3.0.13 Android/4.0.4 Release/09.21.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S71519[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar_Touch_KEM_452_HDMI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71520[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar Touch Kem 452 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71521[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-; Mobiistar_Touch_KEM_452_HDMI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71522[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I777 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71523[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I777 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71524[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I415 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71525[] = "Nokia201/2.0 (11.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia201) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S71526[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; O+ 8.36 Android Build/Oplus8.36_Oplus) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71527[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; O+ 8.36 Android Build/Oplus8.36_Oplus)";
const char S71528[] = "Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80G9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S71529[] = "Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 80G9 Build/MR1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S71530[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9.3a4pre) Gecko/X Minefield/3.7a4pre";
const char S71531[] = "Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: wow-proxy)";
const char S71532[] = "Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: xxdaili4)";
const char S71533[] = "Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: s~scroll-down-hrd)";
const char S71534[] = "Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: s~surf6009-hrd)";
const char S71535[] = "Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: s~proxy6002-hrd)";
const char S71536[] = "Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: s~appcodes-hrd)";
const char S71537[] = "Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: s~proxydisk9-hrd)";
const char S71538[] = "Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: s~surfproxy4-hrd)";
const char S71539[] = "Mozilla/5.0 (compatible; bingbot/2.0 +http://www.bing.com/bingbot.htm);c2VvLWNyYXdsZXI=";
const char S71540[] = "Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: s~surf702-hrd)";
const char S71541[] = "Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: s~swedishdanish-hrd)";
const char S71542[] = "Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: s~surfproxy6-hrd)";
const char S71543[] = "Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: s~surf603-hrd)";
const char S71544[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71545[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71546[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71547[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5215 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71548[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Micromax A116 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S71549[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST27i Build/6.1.1.B.1.10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71550[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST27a Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71551[] = "BlackBerry8130/4.3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105";
const char S71552[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; BASE Tab Build/FRF91) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71553[] = "NokiaC1-01/2.0 (05.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac1-01/UC Browser7.9.0.102/69/352";
const char S71554[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; YP-G50 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S71555[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; YP-G50 Build/ECLAIR)";
const char S71556[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; CT704 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71557[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s5620) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S71558[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5620) UCBrowser8.3.1.161/69/405/UCWEB Mobile";
const char S71559[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71560[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71561[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S720 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71562[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS930 4G Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71563[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ617 Build/7.7.1-85_MZ617-27) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S71564[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9192 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71565[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9192 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S71566[] = "Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 80G9 Build/MR1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Safari/537.36";
const char S71567[] = "Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80G9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S71568[] = "Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80 COBALT Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S71569[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; C8800 Build/HuaweiC8800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71570[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HW-HUAWEI_C8800/C8800V100R001C92B818; 480*800; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S71571[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; C8800 Build/HuaweiC8800) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S71572[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; C8800 Build/HuaweiC8800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71573[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; DISCO10 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71574[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; DISCO10 Build/FRF85B)";
const char S71575[] = "Mozilla/5.0 (Linux; U; Android 3.2; it-it; OP110 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S71576[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; ViewSonic-V350 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71577[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; ODYS Space Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71578[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; ODYS Space_TKI_BRR233v9 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71579[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fa-ir; GLX G1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71580[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; fa-ir; GLX G1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71581[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GLX G111+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71582[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11";
const char S71583[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11";
const char S71584[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11";
const char S71585[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11";
const char S71586[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11";
const char S71587[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-M950 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71588[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile";
const char S71589[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile";
const char S71590[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; JXD-P1000 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71591[] = "SAMSUNG-C5220/1.0 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S71592[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaN8-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S71593[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.8.1 Safari/534.34";
const char S71594[] = "Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7210supernova) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S71595[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Burst S280 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71596[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; PIONEER_P1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71597[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Pioneer_P1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71598[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I879 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71599[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-I879 Build/JZO54K)";
const char S71600[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S71601[] = "Mozilla/5.0 (Linux; Android 4.1.1; M97 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S71602[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26i Build/6.1.A.2.55) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S71603[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6312 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71604[] = "Mozilla/5.0 (Windows NT 6.3) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S71605[] = "Mozilla/5.0 (Windows NT 6.3; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S71606[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-US730 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71607[] = "Mozilla/4.0 (compatible; MSIE 6.0; ; Linux x86_64) Opera 7.50 [en]";
const char S71608[] = "Mozilla/5.0 (X11; Linux i686; U) Opera 7.50  [en]";
const char S71609[] = "Mozilla/5.0 (OS/2; U; Warp 4.5; en-US; rv:1.8.0.7) Gecko/X Firefox/1.5.0.7";
const char S71610[] = "Mozilla/5.0 (compatible; MSIE 5.5;) Firefox/1.5";
const char S71611[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7510 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S71612[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P7510 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S71613[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919V Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71614[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919V Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S71615[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; ARNOVA 101 G4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71616[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320; SPV M650; OpVer 21.138.9.693";
const char S71617[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320;SPV M3100; OpVer 14.211.1.613";
const char S71618[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320; SPV M650; OpVer 21.184.2.732";
const char S71619[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320; CHT 9100/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S71620[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8)";
const char S71621[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320;SPV M3100; OpVer 14.211.2.732";
const char S71622[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320;SPV M3100; OpVer 14.211.1.611";
const char S71623[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320;SPV M3100; OpVer 14.211.1.612";
const char S71624[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) Smartphone; 320x240; Mail/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S71625[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) G500";
const char S71626[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) SPV E610 /SW2.1.1 MSIE/6.0";
const char S71627[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; FujitsuToshibaMobileCommun; IS12T; KDDI)";
const char S71628[] = "Mozilla/5.0 (X11; U; OpenBSD amd64; en-US; rv:1.9.0.1) Gecko/X Firefox/3.0.1";
const char S71629[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; miTab NEO Build/ICS.g08refem618.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71630[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; miTab NEO Build/ICS.miTab NEO.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71631[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5250) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S71632[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID7065 Build/ICS.MID7065.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S71633[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; GT-N7000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31";
const char S71634[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-N7000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S71635[] = "Swisscom/1.0/HTC_HD_mini/Opera 9.7/1.11.170.1 (87652) (Windows NT 5.1; U; en)";
const char S71636[] = "HTC_HD_mini_T5555 Opera/9.7 (Windows NT 5.1; U; zh-tw)";
const char S71637[] = "HTC_HD_mini_T5555 Opera/9.7 (Windows NT 5.1; U; en)";
const char S71638[] = "Vodafone/1.0/HTC_HD_mini/Opera 9.7/1.11.164.1 (87733)/(Windows NT 5.1; U; es-ES)";
const char S71639[] = "Vodafone/1.0/HTC_HD_mini/Opera 9.7/1.10.161.1 (86611)/(Windows NT 5.1; U; en)";
const char S71640[] = "Vodafone/1.0/HTC_HD_mini/Opera 9.7/1.41.161.1 (10904)/(Windows NT 5.1; U; en)";
const char S71641[] = "Vodafone/1.0/HTC_HD_mini/Opera 9.7/1.11.162.1 (87652)/(Windows NT 5.1; U; de)";
const char S71642[] = "Vodafone/1.0/HTC_HD_mini/Opera 9.7/1.41.162.1 (10904)/(Windows NT 5.1; U; de)";
const char S71643[] = "Vodafone/1.0/HTC_HD_mini/Opera 9.7/1.36.184.2 (94538)/(Windows NT 5.1; U; en)";
const char S71644[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; EV-S100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71645[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-I997 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71646[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-I997 Build/GINGERBREAD; SuperCM7-v1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71647[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s3850) UCBrowser8.2.0.132/69/452/UCWEB Mobile";
const char S71648[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s3850) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S71649[] = "samsung-gt-s3850/UCWEB8.2.0.116/69/352";
const char S71650[] = "Mozilla/5.0 (PlayStation Vita 1.67) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2";
const char S71651[] = "MQQBrowser/2.5.1 Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S71652[] = "MQQBrowser/2.5.1 Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3";
const char S71653[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/528.4 (KHTML, like Gecko) Iron/0.3.155.0 Safari/X.528";
const char S71654[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Iron/0.2.152.0 Safari/X.525";
const char S71655[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/528.5 (KHTML, like Gecko) Iron/0.4.155.0 Safari/528.5";
const char S71656[] = "Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU5) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S71657[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S71658[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S71659[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S71660[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S71661[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; U20GT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71662[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; U20GT_WS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71663[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D2-712 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71664[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.4.2 Mobile/9A405 Safari/7534.48.3";
const char S71665[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.4.2 Mobile/9A334 Safari/7534.48.3";
const char S71666[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71667[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9000 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71668[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x400) Opera VZW:SCH-i910";
const char S71669[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) VZW:SCH-i910 PPC 240x400";
const char S71670[] = "NokiaX2-01/5.0 (08.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-01/UC Browser7.9.0.102/69/352";
const char S71671[] = "Mozilla/5.0 (DTV) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.4 AQUOSBrowser/1.0 (US00DTV;V;0001;0001)";
const char S71672[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; MIDC010PR001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71673[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 Comodo_Dragon/21.1.1.0";
const char S71674[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 Comodo_Dragon/21.1.1.0";
const char S71675[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Blade Build/JRO03C; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71676[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Blade Build/JRO03C; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71677[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X Firefox/15.0 slurp";
const char S71678[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X Firefox/15.0.1 slurp";
const char S71679[] = "Mozilla/5.0 (Windows NT 6.1; rv:15.0) Gecko/X Firefox/15.0.1 slurp";
const char S71680[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X Firefox/15.0.1 slurp slurp";
const char S71681[] = "Mozilla/5.0 (Windows NT 6.1; rv:15.0) Gecko/X Firefox/15.0 slurp";
const char S71682[] = "Nokia6300/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia6300) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S71683[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S71684[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.2 Safari/535.19 Silk-Accelerated=false";
const char S71685[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; zh-CN) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19.1";
const char S71686[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9.1";
const char S71687[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S71688[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10";
const char S71689[] = "OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; W20 Build/Symphony_W20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71690[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ALCATEL ONE TOUCH 930D Build/1.2.10.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71691[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010D Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71692[] = "Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S71693[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S71694[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S71695[] = "Mozilla/5.0 (Linux; Android 4.1.1; C1604 Build/11.3.A.2.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71696[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9082 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71697[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9082 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S71698[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E425f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71699[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E425g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71700[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E425 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71701[] = "SAMSUNG-B2700/XXIB2 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S71702[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-gb; T-Mobile G2 Touch Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S71703[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-gb; T-Mobile_G2_Touch Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S71704[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-gb; T-Mobile G2 Touch Build/CUPCAKE) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S71705[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-gb; T-Mobile_G2_Touch Build/CUPCAKE) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S71706[] = "SAMSUNG-C5212/C5212DDIK1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S71707[] = "SAMSUNG-GT-C5212i/C5212iJPKD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S71708[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11";
const char S71709[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11";
const char S71710[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11";
const char S71711[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11";
const char S71712[] = "Mozilla/5.0 (X11; Linux i686; rv:14.0) Gecko/X Firefox/14.0.1 SeaMonkey/2.11";
const char S71713[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA5320) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S71714[] = "NOKIA5320UCBrowser/8.7.1.234/28/444/UCWEB";
const char S71715[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5320d-1) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S71716[] = "Mozilla/5.0 (Linux; Android 4.0.4; SoftwinerEvb Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71717[] = "Mozilla/5.0 (Linux; Android 4.0.4; SoftwinerEvb Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S71718[] = "Mozilla/5.0 (Linux; Android 4.0.4; Softwinerf900 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71719[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.53463";
const char S71720[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S71721[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S71722[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A5000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71723[] = "RSSOwl/2.2.0.X (Windows; U; en)";
const char S71724[] = "RSSOwl/2.2.0.X (X11; U; en)";
const char S71725[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI MediaPad Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S71726[] = "Mozilla/5.0 (Linux; Android 4.0.3; SpringBoard Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S71727[] = "Mozilla/5.0 (Linux; Android 4.0.3; MediaPad 7 Lite Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S71728[] = "Mozilla/5.0 (Linux; Android 4.0.4; MediaPad 10 LINK Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S71729[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.6.5.X (Second Life Release; default skin) Safari/533.3";
const char S71730[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S71731[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP8010 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71732[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R830 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71733[] = "HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; nb)";
const char S71734[] = "HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; fr)";
const char S71735[] = "HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; sv)";
const char S71736[] = "HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; nl)";
const char S71737[] = "HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; zh-tw)";
const char S71738[] = "HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; ru)";
const char S71739[] = "HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; de)";
const char S71740[] = "HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; da)";
const char S71741[] = "HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; it)";
const char S71742[] = "HTC_Touch_Diamond2_T5353 Opera/9.70 (Windows NT 5.1; U; en)";
const char S71743[] = "HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; pt)";
const char S71744[] = "HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; es-ES)";
const char S71745[] = "HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; el)";
const char S71746[] = "HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; en)";
const char S71747[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; tr-tr; Turkcell T11 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71748[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SC-90JB Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71749[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I927 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71750[] = "Mozilla/5.0 (compatible; MSIE 4.0; WindowsCE; Trident/4.0)";
const char S71751[] = "Mozilla/5.0 (compatible; MSIE 4.0; 68K; Tablet PC; Trident/5.0)";
const char S71752[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.9.1 Safari/534.34";
const char S71753[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; PMP5570C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71754[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A4+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71755[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; in-id; PICOpad-QGN Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71756[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; PICOpad-QGN Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71757[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; L102 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71758[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VL-879 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71759[] = "Mozilla/5.0 (Java; U; en-us; gt-s5233s) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile";
const char S71760[] = "SonyEricssonJ10i2/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; fa; sonyericssonj10i2) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S71761[] = "SonyEricssonJ10i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; fa; sonyericssonj10i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S71762[] = "Mozilla/5.0 (IE 11.0; Windows NT 6.2; WOW64; Trident/7.0; Touch; rv:11.0) like Gecko";
const char S71763[] = "Mozilla/5.0 (IE 11.0; Windows NT 6.2; Win64; x64; Trident/7.0; Touch; rv:11.0) like Gecko";
const char S71764[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; Trident/7.0; Touch; WebView/1.0; rv:11.0) like Gecko";
const char S71765[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; Trident/7.0; Touch; WebView/1.0; rv:11.0) like Gecko";
const char S71766[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-S5570 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71767[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-S5570 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71768[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-S5570 Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71769[] = "OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Symphony Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71770[] = "OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; W70 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71771[] = "OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; W80 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71772[] = "OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xplorer T8i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71773[] = "OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xplorer T7i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71774[] = "Mozilla/5.0 (Linux; Android 4.0.3; EVO Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71775[] = "Mozilla/5.0 (Linux; Android 4.0.4; EVO Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71776[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Explorer A310e Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S71777[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 UCBrowser/9.0.1.284 Mobile";
const char S71778[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 UCBrowser/9.0.1.284 Mobile";
const char S71779[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire V Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71780[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire P Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71781[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire VC Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71782[] = "Mozilla/5.0 (Linux; Android 4.3; SCH-I605 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71783[] = "Facebook 4100.0 (iPhone; iPhone OS 5.0; en_US)";
const char S71784[] = "Instagram 2.4.0 (iPhone; iPhone OS 5.0.1; en_US)";
const char S71785[] = "Instagram 2.5.1 (iPhone; iPhone OS 5.0; en_US) AppleWebKit/420+";
const char S71786[] = "MicromaxQ50/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S71787[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; A853 Build/SHLA_U2_05.14.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71788[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; A853 Build/SHLA_U2_05.12.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71789[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; A853 Build/SHLA_U2_05.15.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71790[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A853 Build/SUSKT_U1_01.19.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71791[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; A853 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71792[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; A853 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71793[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; A853 Build/SHLA_U2_05.15.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71794[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; A853 Build/SUSKT_U1_01.19.0)";
const char S71795[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaN8-00)/UC Browser8.2.0.132/50/355/UCWEB Mobile";
const char S71796[] = "Nokia7230/5.0 (10.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia7230) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S71797[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I997 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71798[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I997 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71799[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I997 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71800[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-i997 Build/SGH-i997) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71801[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I997 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71802[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I997 Build/JZO54K)";
const char S71803[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I997 Build/SGH-I977) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71804[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-03E Build/V16R24Ac) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71805[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-03E Build/V18R26Dc) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71806[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-03E Build/V16R24A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71807[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-03E Build/V18R26D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71808[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-03E Build/V20R29Bc) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71809[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-03E Build/V20R29B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71810[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-03E Build/V21R31A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71811[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-03E Build/V21R31Ac) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71812[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B179 Safari/537.4 Maxthon";
const char S71813[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31";
const char S71814[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID8065 Build/ICS.MID8065.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S71815[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A27) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S71816[] = "Mozilla/5.0 (Linux; Android 4.1.2; C2104 Build/15.0.A.2.17) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71817[] = "Mozilla/5.0 (Linux; Android 4.1.2; C2104 Build/15.0.A.2.17) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S71818[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9192 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71819[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9192 Build/JDQ39)";
const char S71820[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-S7270 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71821[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-S7270L Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71822[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71823[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5360) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S71824[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; vi; GT-S5360) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S71825[] = "Nokia3109c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S71826[] = "Nokia3109c_CMCC/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S71827[] = "Mozilla/5.0 (Linux; U; Android 2.0; en-us; sdk Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17</string>";
const char S71828[] = "Mozilla/5.0 (Linux; U; Android 1.0.0; en-us; rk2818sdk Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S71829[] = "Mozilla/5.0 (Linux; U; Android Android_2.1-1.1.3; es-es; rk2818sdk Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S71830[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; CT1002 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71831[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; OP111 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71832[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaN97 mini) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S71833[] = "NokiaN97 mini/UCWEB8.9.0.253/50/999";
const char S71834[] = "SAMSUNG-GT-C3312R Opera/9.80 (J2ME/MIDP; Opera Mini/7.1.33680/31.1475; U; en) Presto/2.8.119 Version/11.10";
const char S71835[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71836[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71837[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71838[] = "Amoi 8512/R23.0 NF-Browser/3.3";
const char S71839[] = "Amoi 8512/R21.0 NF-Browser/3.3";
const char S71840[] = "Amoi 8512/R18.0 NF-Browser/3.3";
const char S71841[] = "Amoi 8512/R22.0 NF-Browser/3.3";
const char S71842[] = "Amoi 8512/R18.5 NF-Browser/3.3";
const char S71843[] = "ACS-NF/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 Qtv/4.3";
const char S71844[] = "Red Mobile F188/1.0 ACS-NF/3.3 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S71845[] = "Amoi 8512/R18.7 NF-Browser/3.3";
const char S71846[] = "Amoi 8512/R28.0 NF-Browser/3.3";
const char S71847[] = "Mozilla/4.0 (MobilePhone PL6650/US/1.0) NetFront/3.3L MMP/2.0";
const char S71848[] = "W186/Qualcom6246/X/NetFront/3.3Profile/MIDP-2.0Configuration/CLDC1.1";
const char S71849[] = "Palm850/V0100 Mozilla/4.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S71850[] = "palm850/V0100 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S71851[] = "SAMSUNG-SGH-U700-Vodafone/BUGE7 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S71852[] = "SAMSUNG-SGH-U700/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S71853[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HW-HUAWEI_C8500S/C8500SV100R001C92B627SP01; 240*320; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S71854[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HW-HUAWEI_C8500S/C8500SV100R001C92B655; 240*320; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S71855[] = "Mozilla/4.1 (compatible; MSIE 6.0; ) 400x240 LGE VX10000";
const char S71856[] = "LGE-MX9700_VEZ/1.0 Mozilla/4.1 (compatible; MSIE 6.0; ) 240X400";
const char S71857[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC-A9192; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S71858[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC-A9192/1.0; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S71859[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:15.0) Gecko/X Firefox/15.1.1-x64 PaleMoon/15.1.1-x64";
const char S71860[] = "Mozilla/5.0 (Windows NT 6.0; rv:15.0) Gecko/X Firefox/15.1.1 PaleMoon/15.1.1";
const char S71861[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; W60 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71862[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; iBall Andi4.5q Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71863[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N78 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71864[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N78 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71865[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N78 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71866[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L300 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71867[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; eZee' Tab 706 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71868[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Smartfren Tab 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71869[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PIRANHA Ultra Tab 7.0 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71870[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; StarTab 769 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71871[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Tab 705M Build/PD200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71872[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Smartfren Tab 7 Build/JRO03H)";
const char S71873[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One VX Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71874[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P5113 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S71875[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P5113 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S71876[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; TA-FONE A37 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71877[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; TA-FONE A37 HD Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71878[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One V Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S71879[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7102 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71880[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7102 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71881[] = "Mozilla/5.0 (Linux; U; Android 4.2.1Jelly Bean; vi-vn; GT-N7102 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71882[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/6.0 Mobile/9B206 Safari/7534.48.3 YaBrowser/2.1.1364.172";
const char S71883[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S71884[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Version/6.0 Safari/8536.25";
const char S71885[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A406 Safari/8536.25";
const char S71886[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S71887[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/6.0 Mobile/9B176 Safari/7534.48.3 YaBrowser/2.1.1364.172";
const char S71888[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ616-12) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S71889[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ616 Build/7.7.1-141-6-PAST-UMTS-LA) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S71890[] = "SAMSUNG-S7350-Bouygues/S7350AGIC3 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S71891[] = "SAMSUNG-S7350/1.0 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S71892[] = "SAMSUNG-S7350i/1.0 SHP/VPP/R5 NetFront/3.5 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S71893[] = "SAMSUNG-GT-B7320L/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S71894[] = "SAMSUNG-GT-B7320/1.0 MIDP/2.1 CLDC/1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S71895[] = "LG-C310/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S71896[] = "UTSTARCOM-GTX75/UC1.88 POLARIS/6.00 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S71897[] = "Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.1.1";
const char S71898[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 Comodo_Dragon/19.0.3.0";
const char S71899[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 Comodo_Dragon/19.0.3.0";
const char S71900[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A840S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71901[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A840SP Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71902[] = "Mozilla/5.0 (Linux; Android 4.0.4; LePanII Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S71903[] = "Mozilla/5.0 (Linux; Android 4.1.1; C1605 Build/11.3.A.2.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71904[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IRULU-MID Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71905[] = "Mozilla/5.0 (Linux; Android 4.1.1; Lenovo P770 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71906[] = "Mozilla/5.0 (Linux; Android 4.1.2; Lenovo P770 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71907[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71908[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT915 Build/2_32A_2008) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71909[] = "SonyEricssonG700/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 958) Opera 8.65 [en]";
const char S71910[] = "AppleCoreMedia/1.0.0.7B367 (iPad; U; CPU OS 3_2 like Mac OS X)";
const char S71911[] = "AppleCoreMedia/1.0.0.7B405 (iPad; U; CPU OS 3_2_1 like Mac OS X)";
const char S71912[] = "AppleCoreMedia/1.0.0.7B500 (iPad; U; CPU OS 3_2_2 like Mac OS X)";
const char S71913[] = "Mozilla/5.0 AppleCoreMedia/1.0.0.8J2 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S71914[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7";
const char S71915[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.7 (KHTML	 like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7";
const char S71916[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; KPN Smart 200 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71917[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; NGM_WINN Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71918[] = "MQQBrowser/2.7 Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S71919[] = "MQQBrowser/2.7 Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3";
const char S71920[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-L300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71921[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAB07-100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S71922[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-AS730 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S71923[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-1 Build/HuaweiU8950-1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71924[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-51 Build/HuaweiU8950-51) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71925[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-51 Build/HuaweiU8950N-51) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71926[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950D Build/HuaweiU8950D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71927[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-1 Build/HuaweiU8950N-1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71928[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SBM206SH Build/S0004) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71929[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SBM206SH Build/S0008) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71930[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SBM206SH Build/S0008)";
const char S71931[] = "SAMSUNG-SGH-F250L/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S71932[] = "SAMSUNG-SGH-F250/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S71933[] = "LG-GX500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S71934[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/20.0.1132.43) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.43 Safari/536.11";
const char S71935[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/20.0.1132.47) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11";
const char S71936[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/20.0.1132.47) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11";
const char S71937[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/20.0.1132.57)";
const char S71938[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/20.0.1132.57)";
const char S71939[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/20.0.1132.57) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S71940[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/20.0.1132.57; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S71941[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/20.0.1132.57) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S71942[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/20.0.1132.57; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)";
const char S71943[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/20.0.1132.57; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618)";
const char S71944[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.3; chromeframe/20.0.1132.57; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)";
const char S71945[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/20.0.1132.57; SLCC1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.5.21022; InfoPath.2; .NET CLR 3.5.30729; .NET CLR 3.0.30729)";
const char S71946[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; chromeframe/20.0.1132.47; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)";
const char S71947[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; ODYS Space Build/I700T_P7_T04_TSCL_FT_R_0_03_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71948[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; NGM Action Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71949[] = "Mozilla/5.0 (Linux; Android 4.1.5; I9220 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71950[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9220 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71951[] = "Mozilla/5.0 (Linux; Android 4.1.1; i9220 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71952[] = "Mozilla/5.0 (Linux; Android 4.1.1; i92200 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71953[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9220 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71954[] = "Mozilla/5.0 (Linux; Android 4.1.5; I92200 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71955[] = "Mozilla/5.0 (Linux; Android 4.1.9; I9220 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71956[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9210T Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71957[] = "Mozilla/5.0 (Linux; Android 4.2.1; SAMSUNG-SGH-I727 Build/JOP40G) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71958[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I727 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71959[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I727 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S71960[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Orbit_5.1DC Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71961[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MSD7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71962[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MSD7Android Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71963[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MSD7Smarty Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71964[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MSD7 Android Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S71965[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-M950 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S71966[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; VT75C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71967[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E977 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71968[] = "Mozilla/5.0 (Linux; Android 4.1.2; EK-GC100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71969[] = "Mozilla/5.0 (Linux; Android 4.1.1; EK-GC100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71970[] = "Mozilla/5.0 (Linux; Android 4.1.2; Lenovo A706_ROW Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S71971[] = "Nokia7360/2.0 (04.90) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S71972[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; S3C6410ZYOF Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S71973[] = "Mozilla/5.0 (Linux; U; Edge DINGO-1.11; Android 1.6; en-us; edge Build/DRD20) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S71974[] = "Mozilla/5.0 (Linux; U; Android 1.6; fr-fr; BYT-P726G; Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1, gzip";
const char S71975[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; Dell Mini 5 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S71976[] = "Mozilla/5.0 (Linux; U; Android 1.6; zh-cn; Lenovo 3GC101/S-5-02;320*533;CTC/2.0) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S71977[] = "Mozilla/5.0 (Linux; U; Android 1.6; th-th; COSMOS_IM8500 Build/DRD35) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S71978[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Ubuntu/10.04 Chromium/7.0.517.44 Chrome/7.0.517.44 Safari/534.7";
const char S71979[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.9 (KHTML, like Gecko) Ubuntu/10.04 Chromium/7.0.530.0 Chrome/7.0.530.0 Safari/534.9";
const char S71980[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/2.8 Mobile/9A405 Safari/7534.48.3";
const char S71981[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/2.8.1 Mobile/9A334 Safari/7534.48.3";
const char S71982[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T769 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71983[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T769 Build/Baked-T769-JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71984[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-T769 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S71985[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-T769 Build/JZO54K)";
const char S71986[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 900;7.10;8858)";
const char S71987[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 900;7.10;8862)";
const char S71988[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 900;7.10;8860)";
const char S71989[] = "Mozilla/5.0 (Linux; Android 4.0.3; A501 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S71990[] = "Mozilla/5.0 (Linux; Android 4.0.4; SoftwinerEvb Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S71991[] = "Mozilla/5.0 (Linux; Android 4.0.4; Softwinerkf026 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S71992[] = "Mozilla/5.0 (Linux; Android 4.0.4; SoftwinerEvb Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S71993[] = "Mozilla/5.0 (Linux; Android 4.0.4; Softwinerf900 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S71994[] = "Mozilla/5.0 (Linux; Android 4.0.4; Softwinerf761 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S71995[] = "Mozilla/5.0 (Linux; Android 4.0.4; MB886 Build/7.7.1Q-144_MB886_MR1-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S71996[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ615-12) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S71997[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; i867 Build/ESE81) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S71998[] = "Dalvik/1.2.0 (Linux; U; Android 2.1-update1; i867 Build/ESE81)";
const char S71999[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328t Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S72000[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328t Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72001[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328t Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S72002[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC T328t Build/IML74K)";
const char S72003[] = "Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328t Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S72004[] = "NokiaN97 mini/UCWEB8.6.0.180/50/999";
const char S72005[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaN97 mini) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/352/UCWEB Mobile";
const char S72006[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; sr-rs; Telenor_Smart Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72007[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S72008[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9100G Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S72009[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9100 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S72010[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; GT-I9100) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S72011[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; GT-I9100G) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S72012[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-mobile i-STYLE 6 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72013[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-mobile i-STYLE 6A Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72014[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Ctrl_V1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72015[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GIONEE_Ctrl_V1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72016[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010X Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72017[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL_ONE_TOUCH_6010X_Orange Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72018[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S72019[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S72020[] = "Mozilla/5.0 (Linux; Android 4.1.1; Prism II Build/HuaweiU8686) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72021[] = "Nokia6300/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6300) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S72022[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6300) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S72023[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Zio Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S72024[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Zio Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S72025[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; GT-P7510 Build/HRI83) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S72026[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; GT-P7510 Build/HONEYCOMB) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S72027[] = "Mozilla/5.0(Linux;U;Android 3.0.1; zh-tw; GT-P7510 Build/HRI83) AppleWebKit/534.13(KHTML,?? Gecko)?/4.0 ???Safari/534.13";
const char S72028[] = "Dalvik/1.5.0 (Linux; U; Android 3.0.1; GT-P7510 Build/HRI83)";
const char S72029[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Comodo_Dragon/9.0.0.3 Chrome/9.0.597.98 Safari/534.13";
const char S72030[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-I9100 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72031[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; GT-I9100 Build/XB-CM7-DEV-24; CyanogenMod-7.2.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72032[] = "JUC(Linux;U;Android2.3.4;Zh_cn;GT-S5670;240*320;)UCWEB7.8.0.95/139/444";
const char S72033[] = "JUC(Linux;U;Android2.3.6;Zh_cn;GT-S5670;240*320;)UCWEB7.8.0.95/139/444";
const char S72034[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Droid Incredible Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72035[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Droid Incredible Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72036[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Droid Incredible Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72037[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Droid Incredible Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72038[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S72039[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200L Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S72040[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P700 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S72041[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I577 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S72042[] = "Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_S9714 Build/JRO03R) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S72043[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8552 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S72044[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P930 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S72045[] = "SonyEricssonX2i/R3AA Browser/Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S72046[] = "SonyEricssonX2/R3AA Browser/Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S72047[] = "MOT-EX223 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10";
const char S72048[] = "MOT-EX223 MIDP-2.0/CLDC-1.1 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U) Presto/2.5.28 Version/10.10";
const char S72049[] = "MOT-EX223 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U; es-SA) Presto/2.5.28 Version/10.10";
const char S72050[] = "MOT-EX223 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U; pt-BZ) Presto/2.5.28 Version/10.10";
const char S72051[] = "LG-T310i/V100 Obigo/Q7.3 MMS/LG-MMS-V1.1/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S72052[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Motorola_MC659B_312; Windows Phone 6.5.3.5)";
const char S72053[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Motorola_MC659B_301; Windows Phone 6.5.3.5)";
const char S72054[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Motorola_MC659B_21103; Windows Phone 6.5.3.5)";
const char S72055[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Motorola_MC659B_40211; Windows Phone 6.5.3.5)";
const char S72056[] = "Motorola_MC659B/200 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5.3.5)";
const char S72057[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; Fly IQ245 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72058[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID SPYDER Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72059[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID SPYDER Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72060[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID SPYDER Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72061[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID SPYDER 4G Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72062[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT928 Build/6.7.2_GC-158-DNRCG-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72063[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT928 Build/6.7.2_GC-158-DNRCG-10) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S72064[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT928 Build/6.7.2_GC-203-DNRCG-8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72065[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT928 Build/6.7.2_GC-203-DNRCG-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72066[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT928 Build/6.7.2_GC-203-DNRCG-9) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S72067[] = "Mozilla/5.0 (Linux; Android 4.0.4; A10 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S72068[] = "Mozilla/5.0 (Linux; Android 4.0.4; A10 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72069[] = "Mozilla/5.0 (Linux; Android 4.0.3; A10 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S72070[] = "Mozilla/5.0 (Linux; Android 4.0.3; A10 MID Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72071[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Andi4.5h Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72072[] = "Mozilla/5.0 (Linux; Android 4.0.4; U2 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S72073[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D2-727 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72074[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; D2-727 Build/JRO03H)";
const char S72075[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A850K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72076[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A850L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72077[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A850S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72078[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6102 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S72079[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT897 Build/7.7.1Q-6_SPR-125_CSP-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S72080[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S72081[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM005SH Build/S0500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72082[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM005SH Build/S0900) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72083[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM005SH Build/S0810) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72084[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM005SH Build/S0610) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72085[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM005SH Build/S0700) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72086[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SBM005SH Build/S0900)";
const char S72087[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Transformer TF101 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S72088[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Sleipnir/2.9.14)";
const char S72089[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; Sleipnir/2.9.14)";
const char S72090[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Sleipnir/2.9.15)";
const char S72091[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; Sleipnir/2.9.15)";
const char S72092[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Sleipnir/2.9.16)";
const char S72093[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Sleipnir/2.9.17)";
const char S72094[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Sleipnir/2.9.18)";
const char S72095[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0 x64; en-US; rv:1.9.1b2pre) Gecko/X Minefield/3.1b2pre";
const char S72096[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.1b2pre) Gecko/X Minefield/3.1b2pre";
const char S72097[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9260 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72098[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9260 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72099[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; O+_8.5_Android Build/OplusOplus_A51) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72100[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-I777 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72101[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I777 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72102[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033X Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72103[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Vivid 4G Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72104[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC-X710a Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S72105[] = "Tweetcaster for iPhone 3.1.0.13306";
const char S72106[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Q7A Build/ICS_IMM76D_B206) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72107[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; BLU_VIVO 4.3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72108[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6102) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S72109[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; HTC_Touch2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S72110[] = "HTC_Mega/1.28.621.4 (66092) Mozilla/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S72111[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 240x320; HTC_Mega-T3333-Orange; OpVer 113.128.10.702";
const char S72112[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_Touch2/1.29.176.2 (67024)";
const char S72113[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 240x320; HTC_Mega-T3333-Orange; OpVer 113.128.2.733";
const char S72114[] = "HTC_Touch3G_T3333 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S72115[] = "HTC_Touch2_T3333 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S72116[] = "NokiaC3-00/5.0 (04.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac3-00/UC Browser8.0.3.107/69/444";
const char S72117[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SYNET7LP-DS Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72118[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; p7901a Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72119[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; p7901ui Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72120[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; SYNET7LP-DS Build/FRF85B)";
const char S72121[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.86 Safari/533.4";
const char S72122[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:12.3) Gecko/X Firefox/12.3-x64 PaleMoon/12.3-x64";
const char S72123[] = "Mozilla/5.0 (Windows NT 6.0; rv:12.3) Gecko/X Firefox/12.3r2 PaleMoon/12.3r2";
const char S72124[] = "SAMSUNG-GT-S5229-Vodafone/S5229BULC3 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S72125[] = "Mozilla/5.0 (Linux; Android 4.1.1; SPH-D700 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72126[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D700 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72127[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5620) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/405/UCWEB Mobile";
const char S72128[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Andi3n Build/iBalliBall_Andi3n) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72129[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; NS-13T001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72130[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; NS-13T001 Build/IML74K)";
const char S72131[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TP10.1-1500DC-metal Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72132[] = "Instagram 3.4.2 (iPad1,1; iPhone OS 5.1.1; en_US; en) AppleWebKit/420+";
const char S72133[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MOMO12 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72134[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SC-72JB Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72135[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-i705 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S72136[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae66-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S72137[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE66) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S72138[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5302 Build/12.0.A.2.245) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S72139[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5302 Build/12.0.A.2.245) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S72140[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-N7000 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S72141[] = "Mozilla/5.0 (Linux; Android 2.3.5; GT-N7000 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453";
const char S72142[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S72143[] = "Mozilla/5.0 (Linux; Android 4.2.2; Desire HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72144[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S72145[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S72146[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S72147[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PG41200 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72148[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PG41200 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72149[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; PG41200 Build/GRJ22)";
const char S72150[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko) Comodo_Dragon/11.4.0.0 Chrome/11.0.696.71 Safari/534.24";
const char S72151[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Comodo_Dragon/11.4.0.0 Chrome/11.0.696.71 Safari/534.24";
const char S72152[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.1.16) Gecko/X Thunderbird/3.0.11";
const char S72153[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; es-ES; rv:1.9.1.7) Gecko/X Thunderbird/3.0.1 ThunderBrowse/3.2.8.1";
const char S72154[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.11) Gecko/X Firefox/2.0.0.11 Flock/1.0.5";
const char S72155[] = "Mozilla/5.0 (Linux; Ubuntu 9.10; ru-RU; rv:7.0.1) Gecko/X Firefox/7.0.1";
const char S72156[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-uk; X900 Build/AZ510_GB_01.12I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72157[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-uk; X900 Build/AZ510_GB_01.23I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72158[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-uk; X900 Build/AZ510_GB_01.09I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72159[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC One V Build/IML74K) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S72160[] = "OneBrowser/3.1 (NokiaX2-00/08.25)";
const char S72161[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S72162[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; HTL22 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72163[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTL22 Build/JZO54K)";
const char S72164[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S72165[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S72166[] = "Mozilla/5.0 (Linux; Android 4.0.4; Grand X(M) Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S72167[] = "Mozilla/5.0 (Linux; Android 4.0.3; ZTE Grand X Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S72168[] = "Mozilla/5.0 (Linux; Android 4.0.4; Grand X Pro Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S72169[] = "Mozilla/5.0 (Linux; Android 4.0.4; Grand X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S72170[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT886 Build/6.7.3-179_YIC-10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72171[] = "Mozilla/5.0 (Linux; Android 4.2.1; ZP980 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S72172[] = "Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300K Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S72173[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-KU3700 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72174[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-KU3700 Build/GRJ22)";
const char S72175[] = "SonyEricssonF100i/R1HA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.5";
const char S72176[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s5263) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S72177[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5263) UCBrowser8.3.1.161/69/405/UCWEB Mobile";
const char S72178[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; W626 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72179[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Philips W626 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S72180[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Philips W626 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72181[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-T959 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72182[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T959 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72183[] = "Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 101G10 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S72184[] = "Nokia302/5.0 (14.53) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia302) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S72185[] = "Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 10 Build/1.1.11-1015 X-10:30) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S72186[] = "Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 10 3G Build/1.1.11-1015 X-16:17) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S72187[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ST10 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72188[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ST10 Build/JRO03C)";
const char S72189[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; DTC GT10 Build/Bullet) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72190[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I605 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72191[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I605 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72192[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SCH-I605 Build/JDQ39E)";
const char S72193[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S72194[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S72195[] = "Mozilla/5.0 (Linux; Android 4.1.1; Transformer Prime TF201 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S72196[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I957 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S72197[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957R Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S72198[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S72199[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S72200[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72201[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; W8 beyond Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72202[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5302) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S72203[] = "Mozilla/5.0 (Linux; Android 4.1.1; WM8850:X6-10V Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S72204[] = "Mozilla/5.0 (Linux; Android 4.1.1; WM8850-mid Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S72205[] = "Mozilla/5.0 (Linux; Android 4.1.1; WM8850-mid Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S72206[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5; SonyEricssonM1i/R1BA; Profile/MIDP-2.1; Configuration/CLDC-1.1; Windows Phone 6.5.3.5)";
const char S72207[] = "Nokia7260/2.0 (05.90) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S72208[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; Racer Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S72209[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; ZTE-U X850 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S72210[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; HTC Dream Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72211[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Dream Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72212[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; HTC Dream Build/FRG83)";
const char S72213[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Comodo_Dragon/8.0.0.4 Chrome/8.0.552.215 Safari/534.10";
const char S72214[] = "NokiaC5-00.2";
const char S72215[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaC5-00";
const char S72216[] = "NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiax2-00) UCBrowser8.2.0.132/69/452/UCWEB Mobile";
const char S72217[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19";
const char S72218[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-c6712) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S72219[] = "JUC (Linux; U; 4.0.4; zh-cn; ST21i2; 320*480) UCWEB7.9.0.94/139/444";
const char S72220[] = "JUC (Linux; U; 4.0.4; zh-cn; ST21i; 320*480) UCWEB7.9.0.94/139/444";
const char S72221[] = "MicromaxX324/MstarBrowser1.0/MIDP2.0Congiguration/CLDC1.1";
const char S72222[] = "Micromax X324i/MstarBrowser1.0/MIDP-2.0 Configuration/CLDC1.1";
const char S72223[] = "iBrowser/2.0,Nokia6681/2.0";
const char S72224[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6802) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S72225[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.68; en-us; 8310/2.7.0.68) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S72226[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.106; en-us; 8310/2.7.0.106) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S72227[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.105; en-us; 8310/2.7.0.105) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S72228[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.90; en-us; 8310/2.7.0.90) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S72229[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.109; en-us; 8310/2.7.0.109) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S72230[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.92; en-us; 8310/2.7.0.92) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S72231[] = "Micromax X295/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S72232[] = "Micromax X295i/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S72233[] = "Mozilla/5.0 (Linux; Android 4.1.2; Transformer Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S72234[] = "Mozilla/5.0 (Linux; Android 4.1.2; Transformer TF101 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S72235[] = "Mozilla/5.0 (Linux; Android 4.2.2; Xoom Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S72236[] = "Mozilla/5.0 (Linux; Android 4.2.1; Xoom Build/JB_MR1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S72237[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S72238[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I957 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S72239[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957R Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S72240[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S72241[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S72242[] = "Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80 COBALT Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S72243[] = "Nokia303/5.0 (14.87) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia303) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S72244[] = "SAMSUNG-GT-E2130/E2130DDIL1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S72245[] = "MOT-WX400/00.72 UP.Browser/7.2.7.5.708 (GUI) MMP/2.0";
const char S72246[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5";
const char S72247[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5";
const char S72248[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5";
const char S72249[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5";
const char S72250[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5";
const char S72251[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-c6712) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S72252[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN95)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S72253[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN95 8GB)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S72254[] = "Mozilla/5.0 (Java; U; fa; nokian95_8gb) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S72255[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11";
const char S72256[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11";
const char S72257[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; it-it; SmartTab10 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S72258[] = "Mozilla/5.0 (Linux; U; Android 3.2; es-es; SmartTab10 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S72259[] = "Dalvik/1.5.1 (Linux; U; Android 3.2.1; SmartTab10 Build/HTK75)";
const char S72260[] = "JUC (Linux; U; 2.3.6; zh-cn; GT-S6802; 320*480) UCWEB7.9.0.94/139/444";
const char S72261[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCL21 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72262[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I605 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S72263[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I605 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S72264[] = "Nokia202/2.0 (20.52) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia202) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S72265[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5235) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S72266[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; tr-tr; Discovery Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72267[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; Discovery Build/JOP40D)";
const char S72268[] = "Mozilla/5.0 (compatible; Konqueror/4.6; Linux) KHTML/4.6.0 (like Gecko) SUSE";
const char S72269[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_2; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S72270[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_3; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.41 Safari/534.7";
const char S72271[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S72272[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S72273[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_0; en-US) AppleWebKit/534.6 (KHTML, like Gecko) Chrome/7.0.500.0 Safari/534.6";
const char S72274[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.1b3pre) Gecko/X SeaMonkey/2.0a2";
const char S72275[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-gb; ELONEX765ET Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72276[] = "Mozilla/5.0 (X11; SunOS i86pc; rv:11.0) Gecko/X Firefox/11.0";
const char S72277[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Q-Smart S1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72278[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7300 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72279[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7300 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72280[] = "GoogleEarth/7.0.2.8415(Macintosh;Mac OS X (10.8.2);en;kml:2.2;client:Free;type:default)";
const char S72281[] = "GoogleEarth/7.0.3.8542(Macintosh;Mac OS X (10.8.2);en;kml:2.2;client:Free;type:default)";
const char S72282[] = "GoogleEarth/7.0.3.8542(Macintosh;Mac OS X (10.8.3);en;kml:2.2;client:Free;type:default)";
const char S72283[] = "GoogleEarth/7.0.3.8542(Macintosh;Mac OS X (10.8.3);es;kml:2.2;client:Free;type:default)";
const char S72284[] = "GoogleEarth/7.0.3.8542(Macintosh;Mac OS X (10.8.3);de;kml:2.2;client:Free;type:default)";
const char S72285[] = "GoogleEarth/7.0.3.8542(Macintosh;Mac OS X (10.8.4);en;kml:2.2;client:Free;type:default)";
const char S72286[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X403 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72287[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; GT-S5670) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S72288[] = "Mozilla/5.0 (Linux; Android 4.0.3; U8860 Build/HuaweiU8860) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S72289[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; QUANTUM7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72290[] = "Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabA2109A Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S72291[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9A405 Safari/7534.48.3";
const char S72292[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9A334 Safari/7534.48.3";
const char S72293[] = "Mozilla/5.0 (Linux; Android 4.2.2; SM-T311 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S72294[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P716 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S72295[] = "JUC(Linux;U;Android2.3.3;Zh_cn;GT-I9100;480*800;)UCWEB7.8.0.95/139/444";
const char S72296[] = "JUC(Linux;U;Android2.3.5;Zh_cn;GT-I9100;480*800;)UCWEB7.8.0.95/139/351";
const char S72297[] = "JUC(Linux;U;Android2.3.4;Zh_cn;GT-I9100G;480*800;)UCWEB7.8.0.95/139/351";
const char S72298[] = "Mozilla/5.0(Linux;U;Android2.3.3;Zh_cn;GT-I9100;480*800;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S72299[] = "JUC(Linux;U;Android2.3.6;Zh_cn;GT-I9100G;480*800;)UCWEB7.8.0.95/139/444";
const char S72300[] = "JUC(Linux;U;Android2.3.6;Zh_cn;GT-I9100;480*800;)UCWEB7.8.0.95/139/444";
const char S72301[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15i Build/3.0.1.A.0.145; MiniCM7-2.2.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72302[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15i Build/GRH55; CyanogenMod 7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72303[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:7.0.1) Gecko/X Postbox/3.0.8";
const char S72304[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; Ilium_S200 Build/ALPS.GB2.MP.V1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72305[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; Ilium_S200 Build/HuaqinIlium_S200) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72306[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; Aqua flash Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72307[] = "Mozilla/5.0 (Linux; Android 4.0.3; KFOT Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S72308[] = "Mozilla/5.0 (Linux; Android 4.0.4; KF012 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S72309[] = "Mozilla/5.0 (Linux; Android 4.0.4; Amazon Kindle Fire Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S72310[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7500 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S72311[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P7500 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S72312[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T1-E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72313[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E615 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S72314[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E615f Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S72315[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT200 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S72316[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11";
const char S72317[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:6.0.2) Gecko/X Firefox/6.0.2";
const char S72318[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; DM012SH Build/S0004) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72319[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; DM012SH Build/S0008) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72320[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; DM012SH Build/S0008)";
const char S72321[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; DM012SH Build/S0010) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72322[] = "Mozilla/5.0 (Windows; U; en_US) AppleWebKit/533.19.4 (KHTML, like Gecko) Dreamweaver/12.0.0.5808 Version/5.0.3 Safari/533.19.4";
const char S72323[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT15i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72324[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Spice Mi-500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72325[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72326[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72327[] = "Mozilla/5.0 (Linux; Android 4.0.4; WALKMAN Build/icx1227_E110) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72328[] = "Mozilla/5.0 (Linux; Android 4.1.1; SC-03E Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72329[] = "Mozilla/5.0 (Linux; Android 4.0.4; Grand X In Build/GXI_TPLV1.0.0B04) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S72330[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; i-mobile IQ 3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72331[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; i-mobile IQ 3 Build/JRO03C)";
const char S72332[] = "Mozilla/5.0 (Linux; Android 4.0.4; Torque Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S72333[] = "Mozilla/5.0 (Linux; Android 4.1.1; SC-03E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72334[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.1.141) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S72335[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT18i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72336[] = "Mozilla/5.0 (Linux; Android 4.0.3; LT18i Build/4.1.A.0.562) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72337[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax A110 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S72338[] = "Mozilla/5.0 (Linux; Android 4.2.1; Micromax A116 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S72339[] = "Mozilla/5.0 (Linux; Android 4.2.1; A116 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S72340[] = "Mozilla/5.0 (SymbianOS/9.3 U; [en]; Series60/3.2 Nokia6650d-1bH/1.00 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S72341[] = "Nokia6650d-1bh/ATT.3.26 Mozilla/5.0 SymbianOS/9.3; U; [en]; Series60/3.2; Profile/MIDP-2.1 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S72342[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia6650d-1c/03.09; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S72343[] = "QQBrowser (Linux; U; zh-cn; HTC Hero Build/FRF91)";
const char S72344[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; IM-A720L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72345[] = "Mozilla/5.0 (Windows NT 6.1; Intel Mac OS X 10.6; rv:7.0.1) Gecko/X Firefox/7.0.1";
const char S72346[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:16.0) Gecko/X Firefox/16.0 SeaMonkey/2.13.2";
const char S72347[] = "JUC (Linux; U; 2.3.6; zh-cn; A25; 240*320) UCWEB7.9.0.94/139/444";
const char S72348[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M729 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72349[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; M729 Build/IML74K)";
const char S72350[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Mobile Safari/535.19 Silk-Accelerated=false";
const char S72351[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAN82) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S72352[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Thunder S220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72353[] = "Guzzle/3.3.1 curl/7.22.0 PHP/5.3.10-1ubuntu3.6";
const char S72354[] = "Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80G9 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S72355[] = "Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 80G9 Build/MR1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S72356[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Sensation Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S72357[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire C Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S72358[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax A110 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S72359[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S72360[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ARCHOS FAMILYPAD 2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72361[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ALCATEL ONE TOUCH 5021E Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S72362[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP Mobile";
const char S72363[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9500 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72364[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-i9500 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72365[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72366[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC_P3450/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S72367[] = "PPC; 240x320; HTC_P3450/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S72368[] = "HTC P3450/2.20.621.1 Mozilla/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S72369[] = "HTC-P3450-Ten/PPC; 240x320; OpVer 23.227.2.792 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S72370[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC P3450; OpVer 23.221.1.611";
const char S72371[] = "HTC-P3450-orange/PPC; 240x320; OpVer 23.221.2.741 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S72372[] = "HTC-Touch/Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S72373[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC P3450 White; OpVer 23.227.1.801";
const char S72374[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC P3450; OpVer 23.221.15.751";
const char S72375[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) chromeframe/9.0.597.98";
const char S72376[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13 chromeframe/9.0.597.98";
const char S72377[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3) chromeframe/9.0.597.98";
const char S72378[] = "ZTE-T50 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1 firmware/T50 T03";
const char S72379[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E) chromeframe/6.0.472.63";
const char S72380[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I997 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72381[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I997 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72382[] = "NokiaC2-06/2.0 (07.29) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-06) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S72383[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; TAD-97052 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72384[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9000 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S72385[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9000 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S72386[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI MediaPad Build/HuaweiMediaPad) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S72387[] = "Mozilla/5.0 (Linux; Android 4.0.3; SpringBoard Build/HuaweiMediaPad) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S72388[] = "Mozilla/5.0 (Linux; Android 4.0.3; MediaPad 7 Lite Build/HuaweiMediaPad) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S72389[] = "Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF700T Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S72390[] = "Mozilla/5.0 (Series40; NokiaC3-00/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S72391[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Lenovo A3000-H Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72392[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Lenovo A3000-H Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72393[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; Lenovo A3000-H Build/JDQ39)";
const char S72394[] = "Mozilla/5.0 (Linux; Android 4.1.1; Z992 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S72395[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S72396[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2S Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S72397[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2A Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S72398[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2SC Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S72399[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2C Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S72400[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S72401[] = "Nokia7020/2.0 (08.10) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S72402[] = "LG-AN430 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S72403[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.0.1.0 Chrome/14.0.835.163 Safari/535.1";
const char S72404[] = "Mozilla/5.0 (X11; U; SunOS sun4u; en-US; rv:1.8.0.1) Gecko/X Firefox/1.5.0.1";
const char S72405[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en; rv:1.9.0.19) Gecko/X Camino/2.0.7 (MultiLang) (like Firefox/3.0.19)";
const char S72406[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; Scroll Essential Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72407[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Scroll Essential Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72408[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U8800Pro Build/HuaweiU8800Pro) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72409[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9300) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S72410[] = "JUC (Linux; U; 4.1.9; zh-cn; GT-I9300; 480*800) UCWEB8.2.2.135/145/800";
const char S72411[] = "Mozilla/5.0 (Linux; Android 4.1.2; Micromax A116 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S72412[] = "Mozilla/5.0 (Linux; Android 4.1.2; Micromax A116 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S72413[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I905 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S72414[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ARCHOS 97B TITANIUM Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72415[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) FluidApp Version/1842 Safari/536.30.1";
const char S72416[] = "Mozilla/5.0 (Linux; Android 4.1.1; WM8850:X6-10V Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S72417[] = "Mozilla/5.0 (Linux; Android 4.1.1; WM8850-mid Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72418[] = "Mozilla/5.0 (Linux; Android 4.1.1; WM8850-mid Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S72419[] = "Mozilla/5.0 (X11; Linux x86_64; rv:10.0.12) Gecko/X Thunderbird/10.0.12";
const char S72420[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5530)/UC Browser8.5.0.183/50/352/UCWEB Mobile";
const char S72421[] = "Nokia5530/UCWEB8.5.0.163/50/999";
const char S72422[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; L-01D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72423[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; NetTAB THOR Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72424[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; NetTAB THOR mini Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72425[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; KM-E100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72426[] = "Mozilla/5.0 (Series40; NokiaX3-00/11.00; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S72427[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LAVA iris 351 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72428[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; LAVA iris 351 Build/GRJ90)";
const char S72429[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI-ONE Plus Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S72430[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI-ONE Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S72431[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-N7000 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S72432[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-N7000 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S72433[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-M830 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72434[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; KYY21 Build/100.0.2480) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72435[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; KYY21 Build/205.0.0d00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72436[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; KYY21 Build/100.0.2480)";
const char S72437[] = "Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2730c-1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S72438[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC6435LVW Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72439[] = "Mozilla/5.0 (Linux; Android 4.2; HTC6435LVW Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72440[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P768 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S72441[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S72442[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S72443[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LS720 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72444[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; SPH-M920 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S72445[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SPH-M920 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S72446[] = "SCH-R430 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S72447[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; ZTE Blade Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72448[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Blade Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72449[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; ZTE Blade Build/FRF91)";
const char S72450[] = "PantechP9050/JJUS09132010 NetFront/3.5 OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S72451[] = "PantechP9050/JJUSX NetFront/3.5 OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1 Novarra-Vision/8.0";
const char S72452[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/530.1 (KHTML, like Gecko) Iron/2.0.168.0 Safari/530.1";
const char S72453[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/530.9 (KHTML, like Gecko) Iron/2.0.178.0 Safari/530.9";
const char S72454[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en) Presto/2.10.229 Version/11.62";
const char S72455[] = "Opera/9.80 (Android 2.3.4; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.7.81 Version/11.00";
const char S72456[] = "Opera/9.80 (Android 2.3.6; Linux; Opera Mobi/ADR-X; U; en-GB) Presto/2.7.81 Version/11.00";
const char S72457[] = "Opera/9.80 (Android 2.3.5; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.7.81 Version/11.00";
const char S72458[] = "Opera/9.80 (Android 2.3.3; Linux; Opera Mobi/ADR-X; U; pl) Presto/2.7.81 Version/11.00";
const char S72459[] = "Opera/9.80 (Android 2.3.7; Linux; Opera Mobi/ADR-X; U; en) Presto/2.7.81 Version/11.00";
const char S72460[] = "Opera/9.80 (X11; Linux x86_64; U; Edition Linux Mint; en) Presto/2.10.229 Version/11.64";
const char S72461[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-N7000 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S72462[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-N7000 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S72463[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)";
const char S72464[] = "Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.2; SV1; MAXTHON)";
const char S72465[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; MAXTHON)";
const char S72466[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAXTHON)";
const char S72467[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Blade Build/JRO03C; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72468[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Blade Build/JRO03C; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72469[] = "Mozilla/5.0 (Linux; Android 4.1.1; Lenovo S890 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72470[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7500 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S72471[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P7500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S72472[] = "Mozilla/5.0 (Linux; Android 4.1.1; X909 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S72473[] = "NokiaN90-1/2.0527.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S72474[] = "NokiaN90";
const char S72475[] = "NokiaN90-1/3.0545.5.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1 (en-US; rv:9.3.2) Clecko/X Classilla/CFM";
const char S72476[] = "Mozilla/5.0 (Series40; NokiaC2-03/06.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S72477[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S72478[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; NGM Quasar Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72479[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; Quasar Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72480[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; AN8G2I Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72481[] = "Mozilla/5.0 (X11; Linux i686) KHTML/4.9.3 (like Gecko) Konqueror/4.9 Fedora/4.9.3-2.fc17";
const char S72482[] = "Mozilla/5.0 (X11; Linux x86_64) KHTML/4.9.5 (like Gecko) Konqueror/4.9 Fedora/4.9.5-2.fc17";
const char S72483[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ThL V11 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72484[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; CJ-ThL V11 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72485[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S72486[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC_Explorer_A310e Build/GRJ90) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S72487[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cloudfone Thrill 430d Build/SKV02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72488[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cloudfone Thrill 430d Build/FSV02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72489[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PAP5000DUO Build/PrestigioPAP5000DUO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72490[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39; Carbon-Carbon-JB-v1.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72491[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39; Carbon-CARBON-JB-v1.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72492[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72493[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7560M Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S72494[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S72495[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035A Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72496[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72497[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72498[] = "Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2730c-1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S72499[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.9.168 Version/11.51";
const char S72500[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows 98; SV1; Avant Browser)";
const char S72501[] = "Mozilla/5.0 (compatible; MSIE 7.0; Windows 98; Avant Browser; Avant Browser)";
const char S72502[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; Avant Browser)";
const char S72503[] = "Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; Avant Browser)";
const char S72504[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; Avant Browser; SV1)";
const char S72505[] = "Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; Avant Browser; Avant Browser)";
const char S72506[] = "Mozilla/5.0 (Windows; U; Windows NT 7.0; en-US; rv:1.9.2.11) Gecko/X Firefox/5.0.10";
const char S72507[] = "Nokia3110c/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia3110c) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S72508[] = "Instacast/2.2.4 (like iTunes/10.1.2)";
const char S72509[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; Avant Browser)";
const char S72510[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; Trident/6.0; Avant Browser)";
const char S72511[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; Win64; x64; Trident/6.0; Avant Browser)";
const char S72512[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:16.0) Gecko/X Firefox/16.0 SeaMonkey/2.13.2";
const char S72513[] = "Mozilla/5.0 (Series40; Nokia7230/10.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S72514[] = "Mozilla/5.0 (Series40; Nokia7230-1c/10.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S72515[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S72516[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S72517[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S72518[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160LV Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S72519[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAP5000TDUO Build/PrestigioPAP5000TDUO) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72520[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S7500 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S72521[] = "Mozilla/5.0 (Linux; Android 4.1.2; iris405 Build/LAVAIRIS405) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72522[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/9B206 Safari/7534.48.3";
const char S72523[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/9B176 Safari/7534.48.3";
const char S72524[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; G928 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72525[] = "Mozilla/5.0 (Linux; Android 4.2.1; Micromax A110Q Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72526[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9192 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72527[] = "Mozilla/5.0 (Linux; Android 4.2.2; SHV-E330S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S72528[] = "Nokia5238/51.9.002 (SymbianOS/9.4; U; Series60/5.0 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Safari/525 3gpp-gba";
const char S72529[] = "ZTE-TU880_TD/1.0 Linux/2.6.32 Android/2.2 Release/5.25.2011 Browser/AppleWebKit533.1";
const char S72530[] = "Mozilla/5.0 (compatible; Teleca Q7; Brew 3.1.5; U; en) 240X480 LGE VX9600";
const char S72531[] = "LG-GW305/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S72532[] = "NokiaE72-1/UC Browser8.0.4.121/28/444/UCWEB";
const char S72533[] = "Mail/1085 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (iMac10%2C1)";
const char S72534[] = "Mail/1085 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro7%2C1)";
const char S72535[] = "Mail/1085 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBook7%2C1)";
const char S72536[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia500)/UC Browser8.2.0.132/50/355/UCWEB Mobile";
const char S72537[] = "Reeder/3.0.5 CFNetwork/548.1.4 Darwin/11.0.0";
const char S72538[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E15i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S72539[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E15a Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S72540[] = "Mozilla/5.0 (Java; U; en-us; nokiae5-00) UCBrowser8.3.1.161/70/352/UCWEB Mobile  3gpp-gba";
const char S72541[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-N7000 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S72542[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-N7000 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S72543[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Woxter Tablet PC 97IPS DUAL Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72544[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Woxter 97 IPS DUAL 3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72545[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Woxter Tablet PC 97IPS DUAL Build/JRO03H)";
const char S72546[] = "LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-kp500 teleca) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S72547[] = "Mozilla/5.0 (Linux; Android 4.0.3; ThinkPad Tablet Build/ThinkPadTablet_A400_03) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S72548[] = "Mozilla/5.0 (Linux; Android 4.0.4; DA220HQL Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S72549[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I545 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us";
const char S72550[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Q1000 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72551[] = "Mozilla/5.0 (Linux; Android 4.1.1; CINK PEAX Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72552[] = "Mozilla/5.0 (Linux; Android 4.1.1; SC-03E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S72553[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ616-12) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S72554[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ616 Build/7.7.1-141-6-PAST-UMTS-LA) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S72555[] = "SAMSUNG-M7600-Vodafone/M7600BUIF2 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S72556[] = "SAMSUNG-M7600H/1.0 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1/*MzU1MTcxMDMwMDUwODY4";
const char S72557[] = "SAMSUNG-M7600/M7600XXIDA SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S72558[] = "SAMSUNG-M7600-ORANGE/M7600BVIF1 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S72559[] = "Nokia7373/2.0 (05.50) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S72560[] = "LG-T325/V100 Obigo/Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S72561[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Liberty Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72562[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Liberty Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72563[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; Liberty Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72564[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Liberty Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72565[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Liberty Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72566[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; Liberty Build/GRK39F)";
const char S72567[] = "Opera/9.80 (Android 2.2.1; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.9.201 Version/11.50";
const char S72568[] = "Opera/9.80 (Android 2.2; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.9.201 Version/11.50";
const char S72569[] = "Opera/9.80 (Android 2.2.2; Linux; Opera Mobi/ADR-X; U; fr) Presto/2.9.201 Version/11.50";
const char S72570[] = "Firefox 0.8 (Linux)\" useragent=\"Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.6) Gecko/X Firefox/0.8";
const char S72571[] = "Mozilla/5.0 (X11; U; NetBSD i386; en-GB; rv:1.7) Gecko/X Firefox/0.9.1";
const char S72572[] = "Mozilla/5.0 (X11; U; Linux i686; rv:1.7.3) Gecko/X Firefox/0.10.1libwww-perl/6.04";
const char S72573[] = "Nokia6681/2.0 (6.09.00) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.9.1.120/27/351/UCWEB";
const char S72574[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I800 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72575[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I800 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72576[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I800 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72577[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC S610d Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72578[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC S610d Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S72579[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-cn; HTC_S610d Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72580[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC S610d Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S72581[] = "Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia2730c-1) UCBrowser8.4.0.159/70/444/UCWEB Mobile";
const char S72582[] = "YahooMobile/1.0 (Football '12; 4.0) (Apple; iPhone; iPhone OS/5.1.1)";
const char S72583[] = "YahooMobile/1.0 (Baseball '12; 2.2) (Apple; iPhone; iPhone OS/5.1.1)";
const char S72584[] = "YahooMobileMessenger/1.0 (iPhone Messenger; 2.2.7.5443) (6693; Apple; iPhone; iPhone OS/5.1.1)";
const char S72585[] = "YahooMobile/1.0 (iPhone Sportacular; 95) (Apple; iPhone; iPhone OS/5.1.1)";
const char S72586[] = "Opera/9.80 (Windows NT 5.0) Presto/2.12.388 Version/12.11";
const char S72587[] = "Opera/9.80 (Windows NT 5.0; Edition Yx) Presto/2.12.388 Version/12.11";
const char S72588[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72589[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72590[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72591[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72592[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72593[] = "MQQBrowser/3.7/Adr (Linux; U; 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD.ZSKI3;480*800)";
const char S72594[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9108 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72595[] = "MQQBrowser/3.7/Adr (Linux; U; 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD.ZCLC2;480*800)";
const char S72596[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72597[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72598[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S72599[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S72600[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9070 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S72601[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S72602[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; YL-Coolpad_5890/4.1.007.X.5890; 540*960; CTC/2.0) CoolpadWebkit/534.30";
const char S72603[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 5890 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72604[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 5890 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S72605[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Coolpad 5890 Build/JZO54K)";
const char S72606[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Andromax U Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72607[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Andromax U Build/JZO54K)";
const char S72608[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; ARNOVA 97G4 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72609[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S72610[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U9510E Build/HuaweiU9510E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S72611[] = "Mozilla/5.0 (Windows NT 4.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1";
const char S72612[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; NookColor Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72613[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NookColor Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72614[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; NookColor Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72615[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; novo7_ELF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72616[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; novo7_ELF Build/IML74K)";
const char S72617[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Novo7 ELF Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72618[] = "Mozilla/5.0 (Windows NT 5.1; rv:12.1) Gecko/X Firefox/12.1 PaleMoon/12.1";
const char S72619[] = "MQQBrowser/39 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S72620[] = "MQQBrowser/39 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3";
const char S72621[] = "MQQBrowser/391 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S72622[] = "MQQBrowser/391 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3";
const char S72623[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7102 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72624[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; GT-N7102 Build/JRO03C.N7102ZCAMB3;720*1280)";
const char S72625[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; GT-N7102 Build/JRO03C.N7102ZNAMB1;720*1280)";
const char S72626[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7102 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72627[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; GT-N7102 Build/JRO03C.N7102ZNAMA6;720*1280)";
const char S72628[] = "Mozilla/5.0 (Linux; Android 4.0.4; PadFone Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S72629[] = "Mozilla/5.0 (Linux; Android 4.0.4; PadFone Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S72630[] = "Mozilla/5.0 (Linux; Android 4.0.3; PadFone Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S72631[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717R Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S72632[] = "Mozilla/5.0 (Linux; U; Android 4.0.4(6577); en-us; Celkon A27  Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72633[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; CROSS A27 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72634[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A27 Build/BCM21654AndroidTK_SystemRel_1.3.2P.05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72635[] = "NokiaC3-01/5.0 (06.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac3-01) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S72636[] = "NokiaC3-01.5/5.0 (06.75) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac3-01.5) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S72637[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S72638[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; Desire HD Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S72639[] = "Nokia300/5.0 (07.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia300) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S72640[] = "Mozilla/5.0 (Linux; Android 4.0.4; KFJWI Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Safari/537.36";
const char S72641[] = "UCWEB/2.0 (iOS; U; iPh OS 7_0; zh-CN; iPh5,2) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile";
const char S72642[] = "UCWEB/2.0 (iOS; U; iPh OS 7_0; zh-CN; iPh4,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile";
const char S72643[] = "UCWEB/2.0 (iOS; U; iPh OS 7_0; zh-CN; iPh3,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile";
const char S72644[] = "UCWEB/2.0 (iOS; U; iPh OS 7_0; zh-CN; iPh5,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile";
const char S72645[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; PB99400 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S72646[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; PB99400 Build/ERE27)";
const char S72647[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-in; iBall Slide i7011 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72648[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-in; iBall Slide i7011 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72649[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-in; iBall Slide i7119 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72650[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.0.6002.2);en;kml:2.2;client:Free;type:default)";
const char S72651[] = "GoogleEarth/6.2.1.6014(Windows;Microsoft Windows (6.0.6002.2);en;kml:2.2;client:Free;type:default)";
const char S72652[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.0.6001.1);en;kml:2.2;client:Free;type:default)";
const char S72653[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.0.6002.2);de;kml:2.2;client:Free;type:default)";
const char S72654[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.0.6002.2);nl;kml:2.2;client:Free;type:default)";
const char S72655[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.0.6002.2);fr;kml:2.2;client:Free;type:default)";
const char S72656[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5";
const char S72657[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5";
const char S72658[] = "Mozilla/5.0 (X11; OpenBSD amd64; rv:13.0) Gecko/X Firefox/13.0.1";
const char S72659[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S72660[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; Xenta TAB9-200 Build/Xenta TAB9-200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72661[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-L710 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72662[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; W100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72663[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GHONG W100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72664[] = "Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia7210supernova) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S72665[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ADR6300 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72666[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ADR6300 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72667[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ADR6300 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72668[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; PROG95B Build/G95-V1.0-2012.12.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72669[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; PROG95B Build/G95-V1.0-2012.12.14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72670[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; PROG95B Build/G95-V1.0-2013.03.22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72671[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; PROG95B Build/G95-V1.0-2012.12.07) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72672[] = "Nokia114/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia114) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S72673[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105P Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S72674[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S72675[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian8-00) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba";
const char S72676[] = "NokiaN8-00/UCWEB8.9.0.253/51/999";
const char S72677[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A818i Duo Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72678[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire 600 dual sim Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72679[] = "Mozilla/5.0 (Windows; U; ; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.4.2.34167 (Firestorm-Release; firestorm skin) Safari/533.3";
const char S72680[] = "Mozilla/5.0 (Linux; Android 4.1.2; B15 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S72681[] = "SAMSUNG-GT-B7350Orange/BVJI1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S72682[] = "SAMSUNG-GT-B7350/1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S72683[] = "Vodafone/1.0/SAMSUNG-GT-B7350/BUJG5 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S72684[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; zt180 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S72685[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; zt180 Build/ERE27)";
const char S72686[] = "NOKIAN95 8GB/UC Browser8.0.4.121/28/444/UCWEB";
const char S72687[] = "NOKIAN95/UC Browser8.0.4.121/28/444/UCWEB";
const char S72688[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s5380k) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S72689[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s5380k) UCBrowser8.2.0.132/69/352/UCWEB Mobile";
const char S72690[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; Android Edition StarText Build/V1.0.0B06) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72691[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; Android Edition StarText Build/V1.0.0B07) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72692[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Symphony W10 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72693[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB860 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72694[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB860 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72695[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB860 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72696[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB860 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72697[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MB860 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72698[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; MB860 Build/JZO54K)";
const char S72699[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; STM722HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72700[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; STM722HCZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72701[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; STM722HZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72702[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S72703[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ELONEX1012ET Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72704[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOMO11Speed Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72705[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72706[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72707[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; GT-N7100 Build/JRO03C.N7100ZCALK5;720*1280)";
const char S72708[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-N7100 Build/JZO54K.N7100ZCDMD3;720*1280)";
const char S72709[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; GT-N7100 Build/JRO03C.N7100ZCALL4;720*1280)";
const char S72710[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; da-dk; EN-TAB419 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72711[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMID705 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72712[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMID705X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72713[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; PMID705 Build/IMM76D)";
const char S72714[] = "Mozilla/4.0 (compatible; MSIE; Windows NT 6.1; Trident/4.0)";
const char S72715[] = "Mozilla/5.0 (compatible; MSIE 2.0; Windows NT 6.1; Trident/5.0)";
const char S72716[] = "Mozilla/5.0 (compatible; MSIE 1.0; Windows NT 6.1; Trident/5.0)";
const char S72717[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T720 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72718[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; T720 Build/IMM76D)";
const char S72719[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I757M Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S72720[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MyPhone A888 Duo Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72721[] = "RunboX5-W Linux/3.0.13 Android/4.0.4 Release/05.20.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S72722[] = "Mozilla/5.0 (Linux; Android 4.1.2; Micromax A115 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72723[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax A110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S72724[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A830S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72725[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A830K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72726[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A830L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72727[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S7562) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S72728[] = "Mozilla/5.0 (Linux; Android 4.3; SPH-L720 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S72729[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I957 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S72730[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957R Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S72731[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S72732[] = "SonyEricssonW380i/R11CA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S72733[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.4.1.0 Chrome/17.0.963.56 Safari/535.11";
const char S72734[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.4.1.0 Chrome/17.0.963.56 Safari/535.11";
const char S72735[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S72736[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.2.139/145/355";
const char S72737[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5360 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S72738[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Scroll Extreme Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72739[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Scroll Extreme ll Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72740[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST17i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72741[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST17a Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72742[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:15.0) Gecko/X Firefox/15.0.1 SeaMonkey/2.12.1";
const char S72743[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; ST97216-1 Build/ST97216-1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72744[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S72745[] = "Mozilla/5.0 (Series40; Nokia109/04.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.35";
const char S72746[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Maxthon (4.0.4.1000) Safari/537.4";
const char S72747[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; AN10G2I Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72748[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; V812 Core4 Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72749[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC One S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72750[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72751[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; One S Build/JDQ39)";
const char S72752[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC One S Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72753[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC_One_S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72754[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST27i Build/6.2.A.0.400) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S72755[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST27i Build/6.2.A.0.400) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S72756[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515m Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S72757[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515a Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S72758[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-mx; ALCATEL ONE TOUCH 6033A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S72759[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT897 Build/7.7.1Q-6_SPR-89_ASA-36) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72760[] = "Mozilla/5.0 (Linux; Android 4.0.4; AZ210B Build/AZ210B_ICS_02.34I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S72761[] = "Mozilla/5.0 (Linux; Android 4.0.4; AZ210A Build/AZ210A_ICS_02.34I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S72762[] = "MQQBrowser/44 Mozilla/5.0 (iPhone 5; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d Safari/7534.48.3";
const char S72763[] = "MQQBrowser/44 Mozilla/5.0 (iPhone 5; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari/7534.48.3";
const char S72764[] = "MQQBrowser/44 Mozilla/5.0 (iPhone 5; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Safari/7534.48.3";
const char S72765[] = "Mozilla/5.0 (Linux; Android 4.2.2; A500 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S72766[] = "Mozilla/5.0 (Linux; Android 4.2.1; A500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S72767[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; in-id; CSL_Spice_MI700 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72768[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; CSL_Spice_MI700 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72769[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-LU3000 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72770[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-LU3000 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72771[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; LG-LU3000 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72772[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-LU3000 Build/FRG83G)";
const char S72773[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; ZTE-U N721 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72774[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; ZTE_U N721 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72775[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; N721 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72776[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i677;7.10;7720)";
const char S72777[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i677;7.10;8773)";
const char S72778[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i677;7.10;8107)";
const char S72779[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE66) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/355/UCWEB Mobile";
const char S72780[] = "Mozilla/5.0 (Java; U; en-us; nokiae66-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/84/352/UCWEB Mobile";
const char S72781[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5380k) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S72782[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Mobile Safari/535.19 Silk-Accelerated=true";
const char S72783[] = "JUC (Linux; U; 2.1-update1; zh-cn; E10i; 240*320) UCWEB7.9.0.94/139/352";
const char S72784[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ISW11F Build/FIK700) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72785[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5228) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S72786[] = "Nokia5228UCBrowser/8.7.1.234/50/444/UCWEB";
const char S72787[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6200 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72788[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P6200 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72789[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6200 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72790[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P6200 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72791[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; GT-I8750)";
const char S72792[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-T899M)";
const char S72793[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R760 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S72794[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; BLU DASH 3.2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72795[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5110 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72796[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; TAB1050 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72797[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC-X710a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S72798[] = "Mozilla/5.0 (Linux; Android 2.3.6; MB860 Build/4.5.1A-129_OLY-145) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S72799[] = "Mozilla/5.0 (Linux; Android 2.3.4; MB860 Build/4.5.2A-74_OLE-31) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S72800[] = "Mozilla/5.0 (Linux; Android 2.3.7; MB860 Build/4.5.141) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S72801[] = "Nokia205/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia205) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S72802[] = "Nokia205.1/2.0 (03.19) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia205.1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S72803[] = "Mozilla/5.0 (Linux; Android 4.1.1; V360 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72804[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Maxthon/4.1.0.4000)";
const char S72805[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; Maxthon/4.1.0.4000)";
const char S72806[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9300 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S72807[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9300 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S72808[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I939 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S72809[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I939D Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S72810[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.1.141) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S72811[] = "SonyEricssonS302/R1BB Browser/OpenWave/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1, SonyEricssonS302/R1BB Browser/OpenWave/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1, SonyEricssonS302/R1BB Browser/OpenWave/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S72812[] = "SonyEricssonS302/R1BB Browser/OpenWave/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1, SonyEricssonS302/R1BB Browser/OpenWave/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S72813[] = "SonyEricssonS302/R1AB Browser/OpenWave/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.FENGPROD.104 (GUI)";
const char S72814[] = "SonyEricssonS302/R1BB Browser/OpenWave/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S72815[] = "BlackBerry8230f/4.6.1.309 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/294";
const char S72816[] = "SAMSUNG-GT-C3500/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S72817[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe; SLCC1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.5.30729; .NET CLR 3.0.30729)";
const char S72818[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; InfoPath.2; .NET CLR 3.0.30729; WWTClient2)";
const char S72819[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; 102P Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72820[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; 102P Build/GRJ90)";
const char S72821[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11";
const char S72822[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11";
const char S72823[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; Micromax A68 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72824[] = "NokiaC2-00/2.0 (03.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac2-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S72825[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P3100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S72826[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S72827[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-P3100 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S72828[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; VP10 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72829[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; VP10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72830[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i2) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S72831[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S72832[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3113 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315";
const char S72833[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P3113 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S72834[] = "samsung-I8160/1.0 Linux/2.6.35.7 Android/4.0.4 Release/01.29.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S72835[] = "Mozilla/5.0 (Linux; Android 4.0.4; Smartphone Android by SFR STARADDICT II Build/IceCreamSandwich) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72836[] = "Mozilla/5.0 (Linux; Android 4.0.4; STARADDICT II Plus Build/GXI_ZTEV1.0.0B02) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72837[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T679 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72838[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T679 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72839[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P970 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S72840[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P970h Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S72841[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S72842[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S72843[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I815 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S72844[] = "Mozilla/5.0 (Linux; Android 4.0.4; MB886 Build/7.7.1Q-144_MB886_MR1-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S72845[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9300 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S72846[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9300 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S72847[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I939D Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S72848[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S72849[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S72850[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S72851[] = "SAMSUNG-SGH-D900-ORANGE/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S72852[] = "SAMSUNG-SGH-D900-VODA/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S72853[] = "SAMSUNG-SGH-D900/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S72854[] = "Alcatel-OT-710A/1.0 ObigoInternetBrowser/Q03C";
const char S72855[] = "Alcatel-OT-710A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S72856[] = "Alcatel-OT-710/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C, Alcatel-OT-710/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S72857[] = "Alcatel-OT-710/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S72858[] = "Alcatel-OT-710D/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S72859[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2 x64; en-US; rv:1.9a1) Gecko/X Firefox/1.6a1";
const char S72860[] = "Mozilla/5.0 (Windows NT 5.1; rv:1.9a1) Gecko/X Firefox/1.6a1";
const char S72861[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9a1) Gecko/X Firefox/1.6a1";
const char S72862[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Incredible S Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72863[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-pt; Optimus Barcelona Build/OPT_B01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72864[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; U20i Build/3.0.1.A.0.145; MiniCM7-2.2.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72865[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; U20i Build/MiniCM7 Pro V7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72866[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Mobile Safari/535.19 Silk-Accelerated=false";
const char S72867[] = "Nokia111/2.0 (03.24) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S72868[] = "Nokia111.1/2.0 (03.24) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S72869[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac6-01) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba";
const char S72870[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC6-01) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S72871[] = "NokiaC6-01/UCWEB8.9.0.253/51/999";
const char S72872[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36 slurp";
const char S72873[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36 slurp";
const char S72874[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36 slurp";
const char S72875[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36 slurp";
const char S72876[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36 slurp";
const char S72877[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; de-de; Cynus T5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72878[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5302) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S72879[] = "Mozilla/5.0 (Linux; Android 4.2.2; SM-T315 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S72880[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/11A465 Safari/7534.48.3";
const char S72881[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/11A501 Safari/7534.48.3";
const char S72882[] = "SAMSUNG-SGH-M150/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S72883[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_Hero Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72884[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Hero CDMA Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72885[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Hero Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72886[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Hero Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72887[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; Hero Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72888[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC Hero CDMA Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72889[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Hero Build/unknown) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72890[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Hero Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72891[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; Hero Build/GRH78)";
const char S72892[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Hero Build/GRI40; LiGux-Beta4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72893[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Hero Build/GRH78) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S72894[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; MID7026 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72895[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Motorola Triumph Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72896[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Motorola Triumph Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72897[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PocketBook A10 3G Build/GRJ90) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72898[] = "Mozilla/5.0 (Linux; Android 4.0.3; M532 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S72899[] = "Mac OS X/10.7.5 (11G56); ExchangeWebServices/2.2 (127); Mail/5.3 (1283)";
const char S72900[] = "Mac OS X/10.7.5 (11G63); ExchangeWebServices/2.2 (127); Mail/5.3 (1283)";
const char S72901[] = "Mac OS X/10.7.5 (11G63b); ExchangeWebServices/2.2 (127); Mail/5.3 (1283)";
const char S72902[] = "Mac OS X/10.7.4 (11E2617); ExchangeWebServices/2.1 (125); Mail/5.3 (1280)";
const char S72903[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Kindle Fire Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.19/283,1.9.19.283,2013-01-17 12:06, -0500, us";
const char S72904[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Transformer Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72905[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Transformer Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72906[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Transformer Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72907[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Transformer Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/tf101";
const char S72908[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Android2.x) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S72909[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT200 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S72910[] = "Mozilla/5.0 (Linux; Android 4.0.4; EVO Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72911[] = "Mozilla/5.0 (Linux; Android 4.0.3; EVO Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72912[] = "Mozilla/5.0 (Linux; Android 4.0.3; Transformer Prime TF201 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S72913[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N5120 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S72914[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SonySO-04E Build/10.3.1.B.0.224) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72915[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SO-04E Build/10.3.1.B.0.224) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72916[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SO-04E Build/10.3.1.B.0.224)";
const char S72917[] = "Mozilla/5.0 (Linux; Android 4.3; en-us; SAMSUNG SM-N900V Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Version/1.5 Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72918[] = "Mozilla/5.0 (Linux; Android 4.3; en-ca; SM-N900W8 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Version/1.5 Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72919[] = "Mozilla/5.0 (Linux; Android 4.3; en-us; SAMSUNG SM-N900P Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Version/1.5 Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72920[] = "Mozilla/5.0 (Linux; Android 4.3; ko-kr; SAMSUNG SM-N900K/KKUBMI8 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Version/1.5 Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72921[] = "Mozilla/5.0 (Linux; Android 4.3; ko-kr; SAMSUNG SM-N900L Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Version/1.5 Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72922[] = "Mozilla/5.0 (Linux; Android 4.3; en-gb; SAMSUNG SM-N9005/N9005XXUBMI7 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Version/1.5 Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S72923[] = "LG265/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S72924[] = "LG265-parrot/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S72925[] = "LGE-LG265/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S72926[] = "Mozilla/5.0 (Linux; U; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.2.149.27 Safari/525.13";
const char S72927[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; CT704 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72928[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; XT550 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72929[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-co; XT550 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Versi?n/4.0 Mobile Safari/533.1";
const char S72930[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; XT550 Build/GRK39F)";
const char S72931[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; tr-tr; PLANET Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72932[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_Explorer_A310e; 320*480) UCWEB7.9.0.94/139/352";
const char S72933[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_Explorer_A310e; 320*480) UCWEB7.9.0.94/139/444";
const char S72934[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_Explorer_A310e; 320*480) UCWEB7.9.0.94/139/355";
const char S72935[] = "Lenovo S720/S100 Linux/3.0.13 Android/4.0.3 Release/12.12.2011 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30";
const char S72936[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae52-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba";
const char S72937[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE52-1) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S72938[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ616-12) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S72939[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ616 Build/7.7.1-141-6-PAST-UMTS-LA) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S72940[] = "Mozilla/5.0 (Linux; Android 4.0.3; Transformer Prime TF201 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S72941[] = "Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S72942[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; V-T100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72943[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; V-T100 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72944[] = "SonyEricssonM1a/R1BA Browser/Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S72945[] = "Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; Android Edition StarTrail Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72946[] = "NokiaC3-00/5.0 (08.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac3-00/UC Browser7.9.0.102/69/352";
const char S72947[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6300 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72948[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6300 Build/GRJ22; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72949[] = "websitepulse checker/1.1 (compatible; MSIE 5.5; Netscape 4.75; Linux)";
const char S72950[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MP948 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S72951[] = "Mozilla/5.0 (Linux; Android 4.0.4; MK16i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72952[] = "Mozilla/5.0 (Windows NT 6.0; rv:14.0) Gecko/X Firefox/14.0.1 SeaMonkey/2.11";
const char S72953[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3 Sleipnir/2.1.1m";
const char S72954[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3 Sleipnir/2.1.2m";
const char S72955[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Safari/7534.48.3 Sleipnir/2.1.2m";
const char S72956[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 Sleipnir/2.1.2m";
const char S72957[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 Sleipnir/2.1.3m";
const char S72958[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3 Sleipnir/2.1.3m";
const char S72959[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3 Sleipnir/2.1.4m";
const char S72960[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 Sleipnir/2.1.4m";
const char S72961[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 Sleipnir/2.1.5m";
const char S72962[] = "Opera/9.80 (Android 2.2.1; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S72963[] = "Opera/9.80 (Android 2.2; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S72964[] = "Opera/9.80 (Android 2.2.2; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S72965[] = "Opera/9.80 (Android 2.2.3; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S72966[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P990 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72967[] = "Mozilla/5.0 (Linux; Android 4.0.4; FJL21 Build/V23R39X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S72968[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S72969[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5570 Build/FROYO) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S72970[] = "Mozilla/5.0 (Linux; Android 4.2.2; One S Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S72971[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One S Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S72972[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; GL07S Build/HuaweiU9700L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S72973[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GL07S Build/HuaweiU9700L)";
const char S72974[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R830 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S72975[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT886 Build/6.7.3-179_YIC-10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S72976[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E-51 Build/HuaweiU8666E-51) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S72977[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E Build/HuaweiU8666E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S72978[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666N Build/HuaweiU8666N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S72979[] = "Mozilla/5.0 (Linux; Android 4.3; SGH-M919 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S72980[] = "sam-u450 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S72981[] = "sam-r351 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S72982[] = "sam-r550 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S72983[] = "SCH-U340C UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S72984[] = "LG-A133/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S72985[] = "SonyEricssonZ530c/R6AC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S72986[] = "SonyEricssonZ530i/R6DA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S72987[] = "KWC-Glide/ UP.Browser/7.2.7.2.541 (GUI) MMP/2.0";
const char S72988[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; ALCATEL_one_touch_910A Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72989[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; sl-si; ALCATEL_one_touch_910 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72990[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; Dell_Grappa Build/CUPCAKE) AppleWebKit/528.5+ Version/3.1.2";
const char S72991[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.8.131 Version/11.10";
const char S72992[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.8.131 Version/11.11";
const char S72993[] = "Mozilla/5.0 (X11; U; SunOS i86pc; en-US; rv:1.9.1) Gecko/X Firefox/3.5";
const char S72994[] = "Mozilla/2.0 (compatible; MSIE 3.0; Windows NT 5.0)";
const char S72995[] = "Mozilla/3.0 (compatible; MSIE 3.0; Windows NT 5.0)";
const char S72996[] = "Mozilla/5.0 (compatible; MSIE 3.0; Windows NT 5.0; Trident/1.0)";
const char S72997[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM101SH Build/S0028) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72998[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM101SH Build/S0018) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S72999[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM101SH Build/S0020) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73000[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; SBM101SH Build/S0028)";
const char S73001[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM101SH Build/S0030) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73002[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM101SH Build/S0032) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73003[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KM10C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73004[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN82) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S73005[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1 (like Version/10A523 Mobile/6.0.1 Safari/8536.25)";
const char S73006[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1 (like Version/10A550 Mobile/6.0.2 Safari/8536.25)";
const char S73007[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ615-12) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S73008[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7568 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73009[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-S7568 Build/IMM76D)";
const char S73010[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P930 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73011[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A89 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S73012[] = "Mozilla/5.0 (Series40; Nokia309/07.55; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.36";
const char S73013[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; vivo X1St Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73014[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; vivo X1S Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73015[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; vivo X1St Build/JOP40D)";
const char S73016[] = "Mozilla/5.0 (Linux; Android 4.1.2; PadFone Infinity Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S73017[] = "Mozilla/5.0 (Linux; Android 4.1.2; PadFone Infinity Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S73018[] = "MQQBrowser/43 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3";
const char S73019[] = "MQQBrowser/43 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3";
const char S73020[] = "MQQBrowser/43 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari/7534.48.3";
const char S73021[] = "MQQBrowser/43 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3";
const char S73022[] = "MQQBrowser/43 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Safari/7534.48.3";
const char S73023[] = "LG-T320/V100 Obigo/Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S73024[] = "LG-T320g/V100 Obigo/Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S73025[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; BLU Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73026[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-I5800 Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73027[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TP10A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73028[] = "MQQBrowser/2.7 Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3";
const char S73029[] = "MQQBrowser/2.7 Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3";
const char S73030[] = "MQQBrowser/2.7 Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A407 Safari/7534.48.3";
const char S73031[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; es-es; GT-S5360L Build/ICE CREAM SANDWICH) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73032[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73033[] = "Mozilla/5.0 (Linux; U; Android 4.0.6; pt-pt; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73034[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; L-02E Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73035[] = "Nokia205/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia205) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S73036[] = "Nokia205.1/2.0 (03.19) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia205.1) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S73037[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AIRIS TM500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73038[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; AIRIS_TM500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73039[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Micromax A51 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73040[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Micromax A51 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73041[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; U9200 Build/HuaweiU9200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73042[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S73043[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTCONE Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S73044[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:22.0) Gecko/X Firefox/22.0 SeaMonkey/2.19";
const char S73045[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A35) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S73046[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S73047[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A870K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S73048[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A870S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S73049[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A870L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S73050[] = "Mozilla/5.0 (X11; U; BSD Four; C) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.3 Safari/533.3";
const char S73051[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LS670 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73052[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LS670 Build/GWK74; X-CM7.1.01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73053[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LS670 Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73054[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-LS670 Build/GWK74; X-CM7.1.01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73055[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; SCH-i569 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73056[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SCH-i569 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73057[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-STYLE 1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73058[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-STYLE 1 i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73059[] = "Mozilla/5.0 (Linux; Android 4.0.3; IM-A830L Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S73060[] = "Mozilla/5.0 (Linux; Android 4.0.3; IM-A830S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S73061[] = "Mozilla/5.0 (Linux; Android 4.0.3; IM-A830K Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S73062[] = "Mozilla/5.0 (Linux; Android 4.0.3; SKY-A830 Build/LTE) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S73063[] = "Mozilla/5.0 (Linux; Android 4.1.1; MB526 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S73064[] = "Mozilla/5.0 (Linux; Android 4.1.2; MB526 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S73065[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; QMobile_A7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73066[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S73067[] = "Nokia306/2.0 (05.93) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia306) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S73068[] = "Mozilla/4.0 (compatible; Polaris 6.2; BREW 3.1.5; U; en)/240X400 LGE LG-LG8575";
const char S73069[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SH-04D Build/S1230) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73070[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SH-04D Build/S4130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73071[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SH-04D Build/S4130)";
const char S73072[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SH-04D Build/S8100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73073[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; MIDC700PR001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73074[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; MIDC700PR001 Build/Polaroid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73075[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; MIDC700PR002 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73076[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; MIDC700PR001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73077[] = "Mozilla/5.0 (Linux; Android 4.0.4; IdeaTab_A1107 Build/MR1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S73078[] = "Mozilla/5.0 (Linux; Android 4.0.3; PMP5197DULTRA Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S73079[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST21i Build/11.0.A.0.16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S73080[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST21i2 Build/11.0.A.1.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S73081[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; TAB-PROTAB25XXL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73082[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TAB-PROTAB25XXL8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73083[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; G1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73084[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; E1-B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73085[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 3G7271 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73086[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AMAZE TAB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73087[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SIMM-X722 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73088[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GOCLEVER TAB R106 Build/FR-MX68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73089[] = "GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (6.2.9200.0);en;kml:2.2;client:Free;type:default)";
const char S73090[] = "GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (6.2.9200.0);de;kml:2.2;client:Free;type:default)";
const char S73091[] = "GoogleEarth/7.1.1.1871(Windows;Microsoft Windows (6.2.9200.0);en;kml:2.2;client:Free;type:default)";
const char S73092[] = "GoogleEarth/7.1.1.1888(Windows;Microsoft Windows (6.2.9200.0);en;kml:2.2;client:Free;type:default)";
const char S73093[] = "Mozilla/5.0 (Linux; Android 2.3.5; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S73094[] = "Mozilla/5.0 (Linux; Android 2.3.5; Motorola Electrify Build/4.5.1A_SUN_USC_19.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453";
const char S73095[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S73096[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HD_mini_T5555; Windows Phone 6.5.3.5)";
const char S73097[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_HD_mini_T5555; Windows Phone 6.5.3.5)";
const char S73098[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD_mini/1.41.162.1 (10904); Windows Phone 6.5.3.5)";
const char S73099[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD_mini/1.11.164.1 (87733); Windows Phone 6.5.3.5)";
const char S73100[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD_mini/1.11.162.1 (87652); Windows Phone 6.5.3.5)";
const char S73101[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_HD_mini-orange-LS; Windows Phone 6.5.3.5)";
const char S73102[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD_mini/1.10.161.1 (86611); Windows Phone 6.5.3.5)";
const char S73103[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD_mini/1.36.184.2 (94538); Windows Phone 6.5.3.5)";
const char S73104[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HD_mini; Windows Phone 6.5.3.5)";
const char S73105[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD_mini/1.41.161.1 (10904); Windows Phone 6.5.3.5)";
const char S73106[] = "Akregator/1.6.5; syndication";
const char S73107[] = "Akregator/1.6.6; syndication";
const char S73108[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T959V Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S73109[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; en; rv:2.0) Gecko/X Firefox/4.0 Opera 12.01";
const char S73110[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; en) Opera 12.01";
const char S73111[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Incredible 2 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73112[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Incredible 2 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73113[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Incredible 2 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73114[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Incredible 2 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73115[] = "ZTE U930_TD/1.0 Linux/2.6.39 Android/4.0 Release/3.5.2012 Browser/AppleWebKit534.30";
const char S73116[] = "ZTEU930HD_TD/1.0 Linux/3.0.8 Android/4.0 Release/10.18.2012 Browser/AppleWebKit534.30";
const char S73117[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Genio Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73118[] = "NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac2-01) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S73119[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; PMP3270B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73120[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; PMP3270B Build/IML74K)";
const char S73121[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; GLADIATOR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73122[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S73123[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S73124[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Smartphone Android by SFR STARADDICT II Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73125[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; STARADDICT II Plus Build/GXI_ZTEV1.0.0B02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73126[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; STARADDICT II Plus Build/GXI_ZTEV1.0.0B03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73127[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R760 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73128[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; CINK SLIM Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73129[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT919 Build/2_290_2012) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S73130[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaN97) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S73131[] = "NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax3-02) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S73132[] = "NokiaX3-02.5/5.0 (07.58) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax3-02.5) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S73133[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710BST Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S73134[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710VMUB Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S73135[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S73136[] = "Opera 8.54 SonyEricssonM600i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 513) Opera 8.65 [en]";
const char S73137[] = "Opera 7.0 SonyEricssonM600i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 513) Opera 8.65 [en]";
const char S73138[] = "SonyEricssonM600i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 306) Opera 8.60 [en]";
const char S73139[] = "SonyEricssonM600i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 513) Opera 8.65 [en]";
const char S73140[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-ve; UM840 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S73141[] = "HTC_Touch_Pro2_T7373 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S73142[] = "Sprint T7380 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S73143[] = "NokiaC5-03";
const char S73144[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; XT800W Build/TTSKT_U_80.33.41R) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73145[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; XT800 Build/TITA_M2_17.19.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S73146[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MOT-XT800/TITA_M2_17.19.0; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73147[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; XT800W Build/TTSKT_U_80.33.41R)";
const char S73148[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MOT-XT800/TITA_M2_17.16.0; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73149[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; XT800 Build/TITA_M2_17.19.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73150[] = "Mozilla/5.0 (compatible; monitis - premium monitoring service; http://www.monitis.com)";
const char S73151[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/527+ (KHTML, like Gecko) QtWeb Internet Browser/1.7 http://www.QtWeb.net";
const char S73152[] = "Nokia7230/5.0 (06.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia7230) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile";
const char S73153[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I415 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S73154[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MID77C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73155[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID77C One Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73156[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HEU004D Build/HEU004D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73157[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5300) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S73158[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S73159[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G X710s Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S73160[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LS860 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S73161[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Dialog i35 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73162[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9070 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73163[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9070P Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73164[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30a Build/9.1.A.0.489) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73165[] = "MQQBrowser/42 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3";
const char S73166[] = "MQQBrowser/421 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3";
const char S73167[] = "MQQBrowser/421 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari/7534.48.3";
const char S73168[] = "MQQBrowser/421 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3";
const char S73169[] = "MQQBrowser/421 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3";
const char S73170[] = "MQQBrowser/42 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3";
const char S73171[] = "MQQBrowser/42 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari/7534.48.3";
const char S73172[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030A Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S73173[] = "Mozilla/5.0 (Linux; Android 4.2.1; JY-G3 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S73174[] = "Nokia6600f/2.0 (05.60) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S73175[] = "SAMSUNG-GT-S3550/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S73176[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; Devour Build/DRC92) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S73177[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; Devour Build/DRC92) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S73178[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Triumph Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73179[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; Triumph Build/GRJ22)";
const char S73180[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Triumph Build/GRJ22; Starship_Triumph) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73181[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB860 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73182[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MB860 Build/4.5.91) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73183[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MB860 Build/4.5.91) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30";
const char S73184[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MB860 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73185[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB860 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73186[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB860 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73187[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB860 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73188[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB860 Build/6.7.7_95) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73189[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MB860 Build/IMM76L)";
const char S73190[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC Desire S.M.S; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S73191[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_DesireS_S510e; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S73192[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Desire_S_S510e; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S73193[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; MTS-SP100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73194[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; MTS-SP100 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S73195[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A502 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73196[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; A502+ Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73197[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Comodo_Dragon/21.0.2.0";
const char S73198[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Comodo_Dragon/21.0.2.0";
const char S73199[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; U30GT-M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73200[] = "JUC (Linux; U; 4.1.1; zh-cn; GT-N7100; 720*1280) UCWEB7.9.0.94/139/444";
const char S73201[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA5320) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S73202[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Mobile Safari/535.19 Silk-Accelerated=true";
const char S73203[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73204[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73205[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73206[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160LV Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73207[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F160L/LV Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73208[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36 slurp";
const char S73209[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36 slurp";
const char S73210[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 slurp";
const char S73211[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 slurp";
const char S73212[] = "Zte-tu235B_TD/1.0 ThreadX/4.0b Larena/2.40 Release/10.10.2010 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S73213[] = "Zte-tu230/1.0 ThreadX/4.0b Larena/2.40 Release/8.15.2009 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S73214[] = "Zte-tu219_TD/1.0 ThreadX/4.0b L2K/4.0 Release/3.15.2011 Browser/NetFront3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1s";
const char S73215[] = "ZTE-F116-Moldova-Moldtelecom/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S73216[] = "ZTE-S50/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S73217[] = "ZTE-Beeline_C100/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S73218[] = "ZTE-F116(ITA_H3G)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S73219[] = "ZTE-Z222/1.0.5 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S73220[] = "NokiaE71/UC Browser7.9.0.102/28/355/UCWEB";
const char S73221[] = "NokiaE71/UCWEB7.9.0.94/28/999";
const char S73222[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PG06100 Build/GRI40; CM7.2.0-RM) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73223[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PG06100 Build/GRI40; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73224[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PG06100 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73225[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; S170 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73226[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; S170 Build/GINGERBREAD)";
const char S73227[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:7.0.1) Gecko/X Postbox/3.0.8";
const char S73228[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; GT-I9008L Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S73229[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; GT-I9008L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73230[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; GT-I9008L Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S73231[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; zh-cn; GT-I9008L Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S73232[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; zh-cn; GT-I9008L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73233[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; GT-I9008L Build/FROYO)";
const char S73234[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; M9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73235[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; M9 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S73236[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; PLAYTAB Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73237[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ605 Build/I.7.1-39.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S73238[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; MW0831 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73239[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9128V Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73240[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SAMSUNG-GT-I9128_TD/1.0 Android/4.1.2 Release/11.15.2012 Browser/AppleWebKit534.30 Build/JRO03L) ApplelWebkit/534.30 (KHTML,like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73241[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9128V Build/JZO54K)";
const char S73242[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.0.400) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S73243[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599V Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73244[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599V Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S73245[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S73246[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320T Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S73247[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9300 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S73248[] = "Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire2 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S73249[] = "Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S73250[] = "SEC-SGHE900/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S73251[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; LG-CX670 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73252[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P3100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S73253[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S73254[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3100B Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S73255[] = "Nokia112/2.0 (03.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia112) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S73256[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100P Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S73257[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Maxthon/4.0.6.2000)";
const char S73258[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Maxthon/4.0.6.2000)";
const char S73259[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; Maxthon/4.0.6.2000)";
const char S73260[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E270L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73261[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; L-05D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73262[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; L-05D Build/JZO54K)";
const char S73263[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U9510E Build/HuaweiU9510E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S73264[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; nl-nl; M785 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73265[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; nl-nl; M785K Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73266[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Sensation Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S73267[] = "Mozilla/5.0 (Linux; Android 4.1.1; Transformer Prime TF201 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S73268[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2690/UC Browser8.0.3.107/69/444";
const char S73269[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/69/444";
const char S73270[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; m7206 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73271[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; m7206 Build/GRI40)";
const char S73272[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; PS47 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73273[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Yan 8.1 Android 4.0 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73274[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Yan 8.1A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73275[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Yan8.1G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73276[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Yan 8.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73277[] = "Mail/53 CFNetwork/609.1.4 Darwin/13.0.0";
const char S73278[] = "webMailReader/1.3.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S73279[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T679 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73280[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T679 Build/JZO86L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73281[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T679 Build/JZO54K)";
const char S73282[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; POLYTRON W2500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73283[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; POLYTRON_W2500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73284[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U705T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73285[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; U705T Build/IMM76D)";
const char S73286[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N78 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73287[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C5502 Build/10.1.1.A.1.89) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73288[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C5502 Build/10.1.1.A.1.273) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73289[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C5502 Build/10.1.1.A.1.210) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73290[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C5502 Build/10.1.1.A.1.164) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73291[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; C5502 Build/10.1.1.A.1.164) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S73292[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C5502 Build/10.1.1.A.1.191) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73293[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; C5502 Build/10.1.1.A.1.310)";
const char S73294[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; C5502 Build/10.1.1.A.1.310) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73295[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.68; en-us; 8100/2.7.0.68) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S73296[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.105; en-us; 8300/2.7.0.105) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S73297[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.105; en-us; 8100/2.7.0.105) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S73298[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.92; en-us; 8100/2.7.0.92) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S73299[] = "UCWEB/2.0(BlackBerry; U; 2.6.0.53; en-us; 8110/2.6.0.53) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S73300[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.110; en-us; 8800/2.7.0.110) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S73301[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.106; en-us; 8110/2.7.0.106) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S73302[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.105; en-us; 8700/2.7.0.105) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S73303[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.110; en-us; 8700/2.7.0.110) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S73304[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.105; en-us; 8110/2.7.0.105) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S73305[] = "UCWEB/2.0(BlackBerry; U; 6.2.0.94; en-us; 9105/6.2.0.94) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S73306[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.137; en-us; 9850/5.1.0.137) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S73307[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.92; en-us; 8120/2.7.0.92) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S73308[] = "Mozilla/5.0 (Linux; Android 4.1.1; P774A Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S73309[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6810 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S73310[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; BLU DASH 4.5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73311[] = "Mozilla/5.0 (Linux; Android 4.1.1; I10A-LE Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S73312[] = "LGE-MT375/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S73313[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-T959 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S73314[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T959 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S73315[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S73316[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; Lenovo A690 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/33482";
const char S73317[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Mobile Safari/535.19 Silk-Accelerated=false";
const char S73318[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970M Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S73319[] = "Mozilla/5.0 (Linux; Android 4.0.4; Grand X(M) Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S73320[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S73321[] = "Mozilla/5.0 (Linux; Android 4.0.3; ZTE Grand X Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S73322[] = "Mozilla/5.0 (Linux; Android 4.0.3; ZTE V970 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S73323[] = "Mozilla/5.0 (Linux; Android 4.1.1; Smartfren Tab 7 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S73324[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; QQBrowser/7.3.11251.400)";
const char S73325[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; AT-AS40D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73326[] = "Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook HD+ Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S73327[] = "Mozilla/5.0 (Linux; Android 4.2.2; BN NookHD+ Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S73328[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S73329[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515m Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S73330[] = "NokiaC2-06/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-06) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S73331[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus 4 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S73332[] = "HTC_S740 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S73333[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Smartphone; 240x320; HTC S740; OpVer 32.174.1.611";
const char S73334[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Smartphone; 240x320; HTC S740; OpVer 32.117.1.612";
const char S73335[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Swisscom/1.0/HTC_S740/1.14.166.0";
const char S73336[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Smartphone; 240x320; HTC S740; OpVer 32.117.2.732";
const char S73337[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_S740/1.14.172.1";
const char S73338[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Mazaa)";
const char S73339[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.9.168 Version/11.52";
const char S73340[] = "ZTEU795_TD/1.0 Linux/2.6.39 Android/4.0 Release/6.10.2012 Browser/AppleWebKit534.30";
const char S73341[] = "ZTEU795+_TD/1.0 Linux/3.0.13 Android/4.0 Release/7.10.2012 Browser/AppleWebKit534.30";
const char S73342[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZTE V889M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73343[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ZTE V889M Build/IMM76D)";
const char S73344[] = "MacOutlook/14.3.0.X (Intel Mac OS X 10.7.5)";
const char S73345[] = "MacOutlook/14.3.1.X (Intel Mac OS X 10.7.5)";
const char S73346[] = "MacOutlook/14.3.2.X (Intel Mac OS X 10.7.5)";
const char S73347[] = "MacOutlook/14.3.4.X (Intel Mac OS X 10.7.5)";
const char S73348[] = "MacOutlook/14.3.5.X (Intel Mac OS X 10.7.5)";
const char S73349[] = "MacOutlook/14.3.6.X (Intel Mac OS X 10.7.5)";
const char S73350[] = "MacOutlook/14.3.7.X (Intel Mac OS X 10.7.5)";
const char S73351[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; P4013 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73352[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Spice Mi-351 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73353[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.3 Mobile/8L1 Safari/6533.18.5";
const char S73354[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.3 Mobile/8J2 Safari/6533.18.5";
const char S73355[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.3 Mobile/8H7 Safari/6533.18.5";
const char S73356[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ615-12) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S73357[] = "Mozilla/5.0 (Linux; Android 4.0.3; E350 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S73358[] = "Mozilla/5.0 (Linux; Android 4.0.4; E350 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S73359[] = "UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 610) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile";
const char S73360[] = "Mozilla/5.0 (compatible; MSIE 6.0; WindowsCE; Trident/2.0)";
const char S73361[] = "Mozilla/5.0 (compatible; MSIE 6.0; 68K; Tablet PC; Trident/2.0)";
const char S73362[] = "Nokia300/5.0 (07.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia300) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S73363[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S73364[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC-X710a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S73365[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E435 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S73366[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E435f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S73367[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E435g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S73368[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.1.96) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S73369[] = "Mozilla/5.0 (Linux; Android 4.1.2; SM-T210R Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S73370[] = "Mozilla/5.0 (Linux; Android 4.1.2; SM-T210 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S73371[] = "Nokia6301/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S73372[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; XV6175.1)";
const char S73373[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-pt; Optimus Monte Carlo Build/OPT_B02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73374[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-pt; Optimus Monte Carlo Build/OPT_B03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73375[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-pt; Optimus Monte Carlo Build/OPT_B04) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73376[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-pt; Optimus Monte Carlo Build/OPT_B05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73377[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-pt; Optimus Monte Carlo Build/OPT_B06) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73378[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.2 Chrome/15.0.900.2 Safari/535.2";
const char S73379[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.2 Chrome/15.0.900.2 Safari/535.2";
const char S73380[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6";
const char S73381[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6.2";
const char S73382[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-pt; Optimus_Madrid Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73383[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EXP8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73384[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; EXP8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73385[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Thrill 430x Build/FSV04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73386[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Thrill 430x Build/FSV06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73387[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Thrill 430x Build/FSV10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73388[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5228) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S73389[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; F7HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73390[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GOCLEVER TAB M713G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73391[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MP888 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73392[] = "Mozilla/5.0 (Linux; Android 4.0.3; A501 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S73393[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-S960L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S73394[] = "SAMSUNG-GT-B7610/1.0 Opera 9.5";
const char S73395[] = "Vodafone/1.0/SAMSUNG-GT-B7610/NBUIK3 Browser/Opera/9.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S73396[] = "Opera/9.80 (Android 2.1-update1; Linux; Opera Mobi/ADR-1105231029; U; en) Presto/2.7.81 Version/11.00";
const char S73397[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; LENCM900HZ Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73398[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; T-Mobile myTouch 3G Slide Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73399[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; T-01D Build/V09R40A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73400[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; T-01D Build/V11R42A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73401[] = "SAMSUNG-GT-C3322/C3322DDLF3 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0 Opera/9.80 (J2ME/MIDP; Opera Mini/6.5.28159/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S73402[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko, M5WebKit/1.0.1, M5HybridSDK/1.0.0) Version/5.1.0 Mobile/6.1 Mammoth/5.1.0";
const char S73403[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko, M5WebKit/1.0.1, M5HybridSDK/1.0.0) Version/5.1.0 Mobile/6.1.2 Mammoth/5.1.0";
const char S73404[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko, M5WebKit/1.0.2, M5HybridSDK/1.0.0) Version/5.1.1 Mobile/6.1.3 Mammoth/5.1.1";
const char S73405[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko, M5WebKit/1.0.2, M5HybridSDK/1.0.0) Version/5.1.1 Mobile/6.1.2 Mammoth/5.1.1";
const char S73406[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko, M5WebKit/1.0.2, M5HybridSDK/1.0.0) Version/5.1.1 Mobile/6.1 Mammoth/5.1.1";
const char S73407[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko, M5WebKit/1.0.2, M5HybridSDK/1.0.0) Version/5.1.1 Mobile/6.1.4 Mammoth/5.1.1";
const char S73408[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; chromeframe/25.0.1364.172)";
const char S73409[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; DIT8070 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73410[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Version/6.0.2 Safari/8536.26.17 Sleipnir/4.1";
const char S73411[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) Version/6.0.3 Safari/8536.28.10 Sleipnir/4.1.2";
const char S73412[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) Version/6.0.4 Safari/8536.29.13 Sleipnir/4.1.2";
const char S73413[] = "Mozilla/5.0 (Linux; Android 3.2; GT-P6200 Build/HTJ85B) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.53463";
const char S73414[] = "Mozilla/5.0 (Linux; Android 3.2; GT-P6200 Build/HTJ85B) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S73415[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P895 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S73416[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P895qb Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S73417[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S73418[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S73419[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X+ Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S73420[] = "Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S73421[] = "Nokia6300/2.0 (07.00) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6300) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S73422[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6300) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S73423[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE66) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S73424[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT918 Build/2_26D_2009) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S73425[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP8010 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S73426[] = "Mozilla/5.0 (Linux; Android 4.1.2; S500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S73427[] = "SAMSUNG-SGH-i900/1.0 (compatible; MSIE 6.0; Windows CE; PPC) Opera 9.5";
const char S73428[] = "LG-C100/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S73429[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7 HTC_WildfireS-orange-LS; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S73430[] = "Mozilla/5.0 (X11; Linux i686; rv:13.0) Gecko/X Firefox/13.0 SeaMonkey/2.10";
const char S73431[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s5263) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S73432[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR930L 4G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73433[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR930L Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73434[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S73435[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S73436[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5250) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/444/UCWEB Mobile";
const char S73437[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Q-Smart S12 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73438[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Q-Smart S12 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73439[] = "Mozilla/5.0 (S60V3; U; en-us; Nokia N96) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile";
const char S73440[] = "Nokia N96UCBrowser/8.7.1.234/28/444/UCWEB";
const char S73441[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokian96-1) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S73442[] = "Lenovo S890/S100 Linux/3.0.13 Android/4.0.3 Release/12.12.2011 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30";
const char S73443[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Safari/7534.48.3 Sleipnir/2.2m";
const char S73444[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 Safari/7534.48.3 Sleipnir/2.2m";
const char S73445[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.2m";
const char S73446[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145 Safari/7534.48.3 Sleipnir/2.2m";
const char S73447[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Safari/7534.48.3 Sleipnir/2.2m";
const char S73448[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.2m";
const char S73449[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT200 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S73450[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 4_3_3 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S73451[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 4_3_5 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S73452[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 4_3_1 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S73453[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 4_3 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S73454[] = "Mozilla/5.0 (Linux; U; Allview_TX1_Quasar 4.0.4; ro-ro; ALLVIEW_TX1_Quasar Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73455[] = "Mozilla/5.0 (Linux; U; Allview_TX1_Quasar 4.0.4; ru-ru; Perfeo_7777W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73456[] = "Mozilla/5.0 (Linux; U; Allview_TX1_Quasar 4.0.4; ru-ru; Inch Regulus (ITWGN7004) Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73457[] = "Mozilla/5.0 (Linux; U; Allview_TX1_Quasar 4.1.2; ru-ru; M73 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73458[] = "Mozilla/5.0 (Linux; U; Allview_TX1_Quasar 4.0.4; bg-bg; ALLVIEW_TX1_Quasar Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73459[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LW770 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73460[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6312 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S73461[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8190 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S73462[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; GT-I8190 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S73463[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ARCHOS 70 Titanium Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73464[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP4100 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S73465[] = "Huaweisymantecspider (compatible; MSIE 8.0; DSE-support@huaweisymantec.com)";
const char S73466[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5; U; en) Presto/2.10.229 Version/11.62";
const char S73467[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5530)/UC Browser8.2.0.132/50/355/UCWEB Mobile";
const char S73468[] = "Nokia5530/UCWEB8.2.0.116/50/999";
const char S73469[] = "JUC (Linux; U; 2.3.5; zh-cn; GT-I9001; 480*800) UCWEB7.9.0.94/139/444";
const char S73470[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, gb";
const char S73471[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I535 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, us";
const char S73472[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; GT-I9300T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, au";
const char S73473[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T999 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, us";
const char S73474[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-12D Build/V15R28C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73475[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-12D Build/V18R30C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73476[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-12D Build/V28R34B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73477[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-12D Build/V30R35B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73478[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-12D Build/V31R36B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73479[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; M-MP815I Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73480[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZP900 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S73481[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZP900S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S73482[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-mobile i-STYLE 4 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73483[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-mobile i-STYLE 4i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73484[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaC7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S73485[] = "Mozilla/5.0 (Symbian3; U; en-us; Nokia700) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile";
const char S73486[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; T-909 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73487[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; T-909 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73488[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; T-909S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73489[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Maxwell Build/v1.0.0.ICS.maxwell.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73490[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Maxwell Build/v1.0.2 X-09:29) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73491[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Maxwell Build/1.0.2.X-17:05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73492[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; SKYTEX-SP712 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73493[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233s) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S73494[] = "UCWEB/2.0 (Java; U; MIDP-2.0; vi; gt-s5233s) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S73495[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233s) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0";
const char S73496[] = "UCWEB/2.0 (Java; U; MIDP-2.0; ?en-US; gt-s5233s) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S73497[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S73498[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710BST Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S73499[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP4100 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S73500[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Idea Zeal Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73501[] = "Mozilla/5.0 (Linux; Android 4.2.2; One X+ Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S73502[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One X+ Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S73503[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9103 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S73504[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-mx; HTC_IncredibleC Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73505[] = "LCT_X880B_01/1.0 Nucleus/1.13.21 MTK6225/07A Release/5.1.2008 Browser/TelecaQ03C1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S73506[] = "Beetel GD777/1.0 Nucleus/1.13.21 MTK6236/11A Release/03.30.2011 Browser/TelecaQ03C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S73507[] = "LCT_D800B_01/1.0 Nucleus/1.13.21 MTK6225/07A Release/5.1.2008 Browser/TelecaQ03C1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S73508[] = "LENOVO-E156/S005 Nucleus/1.15 MTK/23C Release/07.01.2010 Browser/Obigo03C";
const char S73509[] = "azumi l2 OBIGO/Q03C (MTK; Nucleus; U; es-GT)";
const char S73510[] = "HUAWEI-G5520/1.0 MTK6252C/Nucleus 1.0 Release/05.21.2012 Browser/OBIGO-Q03C WAP-Browser 1.0  Profile/MIDP-2.0 Config/CLDC-1.1";
const char S73511[] = "Azumi Tzuki_Slim Obigo/Q03C (MTK; Nucleus; es-CL) 		 Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S73512[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 4.0; obot)";
const char S73513[] = "Opera/9.70 (Linux mips ; U; CE-HTML/1.0 (<profilelist><ui_profile name=\"PHILIPS_OLS_2010\"/></profilelist>); en) Presto/2.2.1";
const char S73514[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; M005 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73515[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-08D Build/V05R11A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73516[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-08D Build/V07R13A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73517[] = "Nokia311/5.0 (03.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia311) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S73518[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73519[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73520[] = "Mozilla/5.0 (Linux; Android 4.2.2; EVO Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73521[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S73522[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S73523[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S73524[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-tw; GT-I5508 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73525[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-I5508 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S73526[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-I5508 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73527[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I5508 Build/FROYO)";
const char S73528[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-I5508 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S73529[] = "Mozilla/5.0 (Windows NT 5.0; rv:5.0) Gecko/X Firefox/5.0 SmartLinksAddon";
const char S73530[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; eo; rv:5.0) Gecko/X Firefox/5.0";
const char S73531[] = "JUC (Linux; U; 2.2.1; zh-cn; HTC_Wildfire; 240*320) UCWEB7.9.3.103/139/800";
const char S73532[] = "JUC (Linux; U; 2.2.1; zh-cn; HTC_Wildfire; 240*320) UCWEB7.9.3.103/140/999";
const char S73533[] = "JUC (Linux; U; 2.2.1; zh-cn; HTC_Wildfire; 240*320) UCWEB7.9.0.94/139/352";
const char S73534[] = "JUC (Linux; U; 2.2.1; zh-cn; HTC_Wildfire; 240*320) UCWEB7.9.0.94/139/444";
const char S73535[] = "JUC (Linux; U; 2.2.1; zh-cn; HTC_Wildfire; 240*320) UCWEB7.9.0.94/139/355";
const char S73536[] = "JUC (Linux; U; 2.3.7; zh-cn; HTC_Wildfire; 240*320) UCWEB7.9.0.94/139/352";
const char S73537[] = "JUC (Linux; U; 2.2.1; zh-cn; HTC_Wildfire; 240*320) UCWEB7.9.4.145/139/800";
const char S73538[] = "JUC (Linux; U; 2.3.7; zh-cn; HTC_Wildfire; 240*320) UCWEB7.9.0.94/139/444";
const char S73539[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s3850) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile";
const char S73540[] = "Bing/3.03 CFNetwork/609 Darwin/13.0.0";
const char S73541[] = "StumbleUpon/2.52 CFNetwork/609 Darwin/13.0.0";
const char S73542[] = "CoolPad8020_CMCC_TD/1.0 Linux/2.6.35 Android/2.3 Release/6.15.2012 Browser/AppleWebkit533.1";
const char S73543[] = "CoolPad8020+_CMCC_TD/1.08 Linux/2.6.35 Android/2.3.5 Release/03.29.2012 Mozilla/5.0 AppleWebKit/533.1 Version/4.0 Mobile Safari/533.1";
const char S73544[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; HTC Vision Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73545[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC Vision Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73546[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC Vision Build/JDQ39)";
const char S73547[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; W110 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73548[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5100 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S73549[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P5100 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S73550[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5100 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S73551[] = "Mozilla/5.0 (Linux; U; Android 4.1.X; de-de; Xelio 10 Pro Build/ODYS_Xelio) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73552[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One V Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S73553[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S5282) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S73554[] = "Mozilla/5.0 (Linux; Android 4.2.1; JY-G4 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S73555[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 101 Titanium Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S73556[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ru-ru; Liquid Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73557[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; hd-us; Liquid Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73558[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; Liquid Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73559[] = "MOT-VE440/00.72 UP.Browser/7.2.7.5.634 (GUI) MMP/2.0";
const char S73560[] = "MOT-VE440-parrot/1.0 UP.Browser/7.2.7.5.656 (GUI) MMP/2.0";
const char S73561[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC-S710d/2.13.1401.4) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73562[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-cn; HTC-S710d/3.06.1401.0) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73563[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC S710d) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73564[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC S710d Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S73565[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC-S710d/3.06.1401.3) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73566[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC-S710d/3.07.1401.3) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73567[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; HTC_IncredibleS_S710d Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73568[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC S710d Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S73569[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; HTC IncredibleS S710d Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73570[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Iron/7.0.520.0 Chrome/7.0.520.0 Safari/534.7";
const char S73571[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.8.1.23) Gecko/X Thunderbird/2.0.0.23";
const char S73572[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.4; de; rv:1.9.1.19) Gecko/X SeaMonkey/2.0.14";
const char S73573[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Andi107 Build/iBalliBall_Andi107) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73574[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn;YL-Coolpad_7230/2.3.006.X.7230; 480*800; CUCC/3.0) CoolpadWebkit/533.1";
const char S73575[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_7230-B Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73576[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Coolpad 7230 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) FlyFlow/2.2 Version/4.0 Mobile Safari/533.1";
const char S73577[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_7230 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73578[] = "Mozilla/5.0 (Linux; U; Android 2.3.9; en-us; SPHS on Hsdroid Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73579[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SPHS on Hsdroid Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73580[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPHS on Hsdroid Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73581[] = "Instagram 3.4.2 (iPod4,1; iPhone OS 5.1.1; en_US; en) AppleWebKit/420+";
const char S73582[] = "Instagram 3.4.2 (iPod4,1; iPhone OS 5.1; en_US; en) AppleWebKit/420+";
const char S73583[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Magic Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73584[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Magic Build/IMM76D)";
const char S73585[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E973 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S73586[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F200K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S73587[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F200S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S73588[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E973 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S73589[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.3m";
const char S73590[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.3.1m";
const char S73591[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.3.2m";
const char S73592[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.3m";
const char S73593[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.3.2m";
const char S73594[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT8000 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S73595[] = "Mozilla/5.0 (Linux; Android 4.2.2; A500 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S73596[] = "Mozilla/5.0 (Linux; Android 4.2.1; A500 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73597[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Stealth 9.7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73598[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT920 Build/2_290_2009) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S73599[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P765 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S73600[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PAP3540DUO Build/PrestigioPAP3540DUO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73601[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5200 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S73602[] = "Mozilla/5.0 (Linux; Android 4.3; GT-I9305 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S73603[] = "Mozilla/5.0 (Linux; Android 4.3; SAMSUNG-SGH-I747 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S73604[] = "Mozilla/5.0 (Linux; Android 4.3; SPH-L710 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S73605[] = "Nokia5233/UC Browser7.7.1.88/50/444";
const char S73606[] = "Opera/9 (Windows NT 6.1) Presto";
const char S73607[] = "Opera/9.00 (Windows NT 6.1; U; en)";
const char S73608[] = "Opera/9.02 (Windows NT 6.1; U; en)";
const char S73609[] = "Opera/9.0 (Windows NT 6.1; U; en)";
const char S73610[] = "Opera 9.4 (Windows NT 5.3; U; en)";
const char S73611[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.11) Gecko/X Lightning/1.0b1 Thunderbird/3.0.6";
const char S73612[] = "Mozilla/5.0 (X11; Linux x86_64) KHTML/4.8.3 (like Gecko) Konqueror/4.8 Fedora/4.8.3-1.fc17";
const char S73613[] = "Mozilla/5.0 (X11; Linux i686) KHTML/4.8.3 (like Gecko) Konqueror/4.8 Fedora/4.8.3-1.fc17";
const char S73614[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GFIVE Bravo Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73615[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GFIVE Bravo(G95) Build/GFIVE_BRAVO_G95-V1.0-2012.12.21) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73616[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; GFIVE G95 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73617[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IRULU-AL101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73618[] = "Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3120classic) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S73619[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9070 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S73620[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T1-B Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73621[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT30at Build/7.0.B.1.152) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73622[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2690) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S73623[] = "Mozilla/5.0 (Linux; Android 4.0.3; LIFETAB_P9514 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S73624[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E976 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73625[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E976 Build/JZO54K)";
const char S73626[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9505 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S73627[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Transformer TF101 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73628[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Transformer TF101 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73629[] = "Mozilla/5.0 (Series40; Nokia310/07.35; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S73630[] = "Alcatel-OT-255/1.0 ObigoInternetBrowser/Q03C, Alcatel-OT-255/1.0 ObigoInternetBrowser/Q03C";
const char S73631[] = "Alcatel-OT-255A/1.0 ObigoInternetBrowser/Q03C";
const char S73632[] = "Alcatel-OT-255/1.0 ObigoInternetBrowser/Q03C";
const char S73633[] = "Alcatel-OT-255D/1.0 ObigoInternetBrowser/Q03C";
const char S73634[] = "SonyEricssonW550c/R4CB Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S73635[] = "SonyEricssonW550i/R4CB Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S73636[] = "NokiaC2-03/2.0 (06.51) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-03/UC Browser8.0.3.107/69/444";
const char S73637[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.3) Gecko/X Firefox/3.0.3 Flock/2.0";
const char S73638[] = "MQQBrowser/35 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S73639[] = "MQQBrowser/35 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3";
const char S73640[] = "MQQBrowser/35 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3";
const char S73641[] = "MQQBrowser/35 Mozilla/5.0 ( ; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S73642[] = "MQQBrowser/35 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3";
const char S73643[] = "Mozilla/5.0 (Symbian3; U; en-us; Nokia500)/UC Browser8.5.0.183/51/355/UCWEB Mobile";
const char S73644[] = "Mozilla/5.0 (Java; U; en-us; nokia500) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile  3gpp-gba";
const char S73645[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Motorola MOT-XT681 Build/IRMXCT_9_02.19.05RPS) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73646[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Motorola MOT-XT681 Build/IRMXCT_9_02.19.05RPS) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S73647[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; Motorola MOT-XT681 Build/IRM90B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73648[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Motorola MOT-XT681 Build/IRMXCT_9_02.19.05RPS) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S73649[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Motorola MOT-XT681 Build/IRMXCT_9_02.30.00RPS) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73650[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Motorola MOT-XT681 Build/IRMXCT_9_02.24.00RPS) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73651[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Motorola MOT-XT681 Build/IRMXCT_9_02.30.00RPS) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S73652[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Motorola MOT-XT681 Build/IRMXCT_9_02.30.00RPS) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S73653[] = "IUC(U;iOS 6.0.1;Zh-cn;320*480;)/UCWEB8.4.1.169/42/997";
const char S73654[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5750E-BOUYGUES/S5750EAGLE1; U; Bada/2.0; fr-fr) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile WQVGA SMM-MMS/1.2.0 OPN-B";
const char S73655[] = "Mozilla/5.0 (Windows NT 6.0; rv:17.0) Gecko/17.0 Firefox/17.0 IceDragon/17.0.1.0";
const char S73656[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Fly F350 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73657[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Fly F350s build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73658[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ThL W5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73659[] = "Mozilla/5.0 (Linux; Android 4.1.2; S500 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S73660[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P870 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S73661[] = "Mozilla/5.0 (Linux; Android 4.1.2; ME371MG Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22";
const char S73662[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; OPPOX907 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73663[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; X907 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73664[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; X907 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73665[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7102 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S73666[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7102 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S73667[] = "Mozilla/5.0 (Linux; Android 4.1.9; GT-N7102 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S73668[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT914 Build/2_28B_2005) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S73669[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S73670[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S73671[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S73672[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S73673[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S73674[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9108 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S73675[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S73676[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S73677[] = "Mozilla/5.0 (Linux; Android 4.1.2; Titanium S5 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S73678[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9080L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S73679[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GTablet Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73680[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GTablet Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73681[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11 Comodo_Dragon/20.0.1.0";
const char S73682[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AN7G3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73683[] = "ZTE-T116a NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1 firmware/T116a T04";
const char S73684[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer Pad TF300T Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73685[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer Pad TF300T Build/JRO03R; CyanogenMod-1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73686[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Transformer Pad TF300T Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73687[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Spice_Mi-495 Build/SpiceSpice_Mi-495) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73688[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A798t Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S73689[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A798t Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S73690[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A798t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73691[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo A798t Build/IMM76D)";
const char S73692[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Andromax U Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73693[] = "Mozilla/5.0 (iPad; U; CPU OS 6_1 like Mac OS X; zh-cn Model:iPad2,2) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7543.48.3 UCBrowser/1.0.0.192/43/355";
const char S73694[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I727R Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S73695[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9210 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S73696[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E110S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S73697[] = "Mozilla/5.0 (Linux; Android 4.0.3; SHV-E110S Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S73698[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830L Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73699[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830K Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73700[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830S Build/JZO54K; CyanogenMod-JB_LR_VGV_V2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73701[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830K Build/JZO54K; CyanogenMod-Cyanogenmod Version 10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73702[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830 Build/Ho?ng Minh H?u - Ho?ng H?i Bily; CyanogenMod-JB_LR_VGV_SLIM_V1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73703[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830L Build/JZO54K; CyanogenMod-Cyanogenmod Version 10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73704[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36";
const char S73705[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9103 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S73706[] = "Mozilla/4.0 SonyEricssonC902v/R3DA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.2";
const char S73707[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.12.12934.201";
const char S73708[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)";
const char S73709[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; SV1; QQBrowser/6.12)";
const char S73710[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; QQBrowser/6.12)";
const char S73711[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)";
const char S73712[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; QQDownload 718; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; QQBrowser/6.12)";
const char S73713[] = "Mozilla/5.0 (compatible; Konqueror/3.5; SunOS)";
const char S73714[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-N7000 Build/IMM76I; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73715[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-N7000 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73716[] = "Mozilla/5.0 (Windows NT 6.0; rv:14.0) Gecko/X Thunderbird/14.0";
const char S73717[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; Philips W632 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73718[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE75-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S73719[] = "Mozilla/5.0 (PlayStation Vita 2.01) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S73720[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; SonyEricssonU5) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S73721[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-note 2 Build/ICS_IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73722[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; HTC Sensation Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73723[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC Sensation Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73724[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC Sensation Build/JDQ39E)";
const char S73725[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC Sensation Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73726[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Spice Mi-435 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73727[] = "Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5310xpressmusic) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S73728[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TM400 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73729[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AIRIS TM400 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73730[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S73731[] = "Mozilla/5.0 (iPad; U; CPU OS 6_1_3 like Mac OS X; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT Mobile";
const char S73732[] = "Mozilla/5.0 (iPad; U; CPU OS 6_1_3 like Mac OS X; en-GB) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT Mobile";
const char S73733[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; L-01E Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73734[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; L-01E Build/JZO54K)";
const char S73735[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST25i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S73736[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST25i Build/6.0.B.3.188) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S73737[] = "Mozilla/5.0 (Linux; Android 4.2.2; Desire HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S73738[] = "Mozilla/5.0 (Linux; Android 4.0.4; WIKO-CINK SLIM Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S73739[] = "INQ/Q1-218H/1.0.74 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S73740[] = "INQ/Q1-218H/1.0.74parrot Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S73741[] = "Mozilla/5.0 (PDA; NF35WMPRO/1.0; like Gecko) NetFront/3.5";
const char S73742[] = "MD-Cube/R01 NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S73743[] = "MD-302/1.0/B000 Browser/NetFront/3.5 MMS/Obigo-MMS/Q05A SyncML/HW-SyncML/1.0 Java/HWJa/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Player/QTV-Player/5.3";
const char S73744[] = "Mozilla/5.0 (DigitalMediaPlayer; Walkman/NWZ-X1000 Series/1.00; like Gecko; wireless) NetFront/3.5";
const char S73745[] = "Mozilla/5.0 (rv:1.3) like Gecko/X Profile/MIDP-2.0 Configuration/CLDC-1.1 NetFront/3.5";
const char S73746[] = "SAMSUNG-GT-S7550B/1.0 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S73747[] = "SAMSUNG-GT-S7550/S7550XXJD2 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S73748[] = "PANTECH-C740/ISUS09012008 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S73749[] = "PANTECH-C740/R01 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S73750[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HERO200 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73751[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HERO200 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73752[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HERO200 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73753[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HERO200 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73754[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; HERO200 Build/GRI40)";
const char S73755[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ar-eg; HERO200 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73756[] = "Mozilla/5.0 (X11; Linux i686; rv:7.0.2) Gecko/X Firefox/12.0.1 http://repskan.com/bots.repskan";
const char S73757[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; WX435 Build/IML77; CyanogenMod-9.0.0.Triumph) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73758[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; WX435 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73759[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; P-07D Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73760[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.7.5) Gecko/X Netscape/8.1.2";
const char S73761[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.0 Safari/535.19 Silk-Accelerated=false";
const char S73762[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; STM805ND Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73763[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5880D Build/PMP5880D_v1.0.0_X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S73764[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9308 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S73765[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; E2001 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73766[] = "Mozilla/5.0 (compatible; MSIE 2.0; WinNT4.0; Trident/5.0)";
const char S73767[] = "Mozilla/5.0 (compatible; MSIE 1.0; WinNT4.0; Trident/1.0)";
const char S73768[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P715 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S73769[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7102 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S73770[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7102 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S73771[] = "Mozilla/5.0 (Linux; Android 4.1.9; GT-N7102 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S73772[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7102 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S73773[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7102 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S73774[] = "Mozilla/5.0 (Linux; Android 4.1.9; GT-N7102 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S73775[] = "Mozilla/5.0 (Vodafone/1.0/LG-KM900/V10o Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S73776[] = "Mozilla/5.0 (Vodafone/1.0/LG-KM900/V10v Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S73777[] = "Mozilla/5.0 (Vodafone/1.0/LG-KM900/V10j Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S73778[] = "Mozilla/5.0 (Vodafone/1.0/LG-KM900/V10m Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S73779[] = "LG-KM900/V10g Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S73780[] = "Mozilla/5.0 (Vodafone/1.0/LG-KM900/V10w Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S73781[] = "Mozilla/5.0 (Vodafone/1.0/LG-KM900/V10l Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S73782[] = "Mozilla/5.0 (Vodafone/1.0/LG-KM900/V10t Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S73783[] = "Opera/9.80 (Android 2.2; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.9.201 Version/11.50";
const char S73784[] = "Opera/9.80 (Android 2.2.1; Linux; Opera Tablet/ADR-X; U; es-ES) Presto/2.9.201 Version/11.50";
const char S73785[] = "Opera/9.80 (Android 2.2.2; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.9.201 Version/11.50";
const char S73786[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.9.2";
const char S73787[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.9.1";
const char S73788[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.9.4";
const char S73789[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.9.5";
const char S73790[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.9.6";
const char S73791[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.9.6";
const char S73792[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.9.1";
const char S73793[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.9.7";
const char S73794[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.9.7";
const char S73795[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.9.2";
const char S73796[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.9.5";
const char S73797[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.9.8";
const char S73798[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.9.8";
const char S73799[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 Flipboard/1.9.8";
const char S73800[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 Flipboard/1.9.8";
const char S73801[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.1 Safari/535.19 Silk-Accelerated=false";
const char S73802[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; U20GT Build/U20GT) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73803[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile IQ 5A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73804[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; i-mobile IQ 5A Build/IMM76D)";
const char S73805[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S73806[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; HTC Wildfire S A510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S73807[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC_Wildfire_S_A510e Build/GRJ90) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S73808[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Studio 5.3s Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73809[] = "Mozilla/5.0 (Linux; Android 4.2.2; HUAWEI P6-U06 Build/HuaweiP6-U06) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S73810[] = "Mozilla/5.0 (Linux; Android 4.2.2; HUAWEI P6-U06 Build/HuaweiP6-U06) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73811[] = "SonyEricssonC905a/R1BB Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.0";
const char S73812[] = "NexianNX-G922/MTK Release/7.15.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S73813[] = "Mozilla/5.0 (X11; SunOS i86pc; rv:13.0) Gecko/X Firefox/13.0.1";
const char S73814[] = "Pulp/2.5.2 (Mac; http://www.acrylicapps.com/pulp/)";
const char S73815[] = "iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73816[] = "LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-kp500 teleca) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile";
const char S73817[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; A19 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73818[] = "SamsungI8910/SymbianOS/9.1 Series60/3.0";
const char S73819[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; i-mobile i-STYLE 2i  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73820[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; i-mobile i-STYLE 2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73821[] = "JUC (Linux; U; 2.3.6; zh-cn; GT-I9070; 480*800) UCWEB7.9.0.94/139/444";
const char S73822[] = "G607_MAIN_V1.0 (MRE\\2.5.00(2072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\1 gsensor\\0 keyboard\\reduced) INTEX_TURBO_3.5_V07_INDIAN Release/2013.01.04 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US";
const char S73823[] = "G607_MAIN_V1.0 (MRE\\2.5.00(2072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\1 gsensor\\0 keyboard\\reduced) INTEX_TURBO_3.5_V08_INDIAN Release/2013.03.08 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US";
const char S73824[] = "G607_MAIN_V1.0 (MRE\\2.5.00(2072) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\1 gsensor\\0 keyboard\\reduced) INTEX_TURBO_3.5_V10_INDIAN Release/2013.04.26 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US";
const char S73825[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/Sleipnir 2.4.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73826[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/Sleipnir 2.4.0.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73827[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/Sleipnir 2.4.0.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73828[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/Sleipnir 2.4.0.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73829[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300";
const char S73830[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S73831[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-P3100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S73832[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-P3100 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S73833[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E460 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73834[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E460 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S73835[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S73836[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT200 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S73837[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S73838[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:23.0) Gecko/X Thunderbird/23.0";
const char S73839[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; CROSS A7S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73840[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT914 Build/2_28B_2005) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73841[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT914 Build/2_32A_2012)";
const char S73842[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT914 Build/2_32A_2012) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73843[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT914 Build/2_28B_2012) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73844[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT914 Build/2_32A_2035) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73845[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTCEVOV4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S73846[] = "Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI P2-6011 Build/HuaweiP2-6011) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S73847[] = "Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S73848[] = "SAMSUNG-GT-S5233T/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S73849[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.68.1420 Chrome/14.0.835.202 Safari/535.1";
const char S73850[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) RockMelt/0.9.50.549 Chrome/10.0.648.205 Safari/534.16";
const char S73851[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko) RockMelt/0.9.58.494 Chrome/11.0.696.71 Safari/534.24";
const char S73852[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.64.319 Chrome/13.0.782.215 Safari/535.1";
const char S73853[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML, like Gecko) RockMelt/0.9.62.351 Chrome/12.0.742.122 Safari/534.30";
const char S73854[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) RockMelt/0.9.48.51 Chrome/9.0.597.107 Safari/534.13";
const char S73855[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SPH-D700 Build/GRJ22; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73856[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SPH-D700 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73857[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-c6712) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S73858[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-c6712) UCBrowser8.2.0.132/69/352/UCWEB Mobile";
const char S73859[] = "Opera/9.80 (Macintosh; Intel Mac OS X; Opera Mobi/27; U; en) Presto/2.4.18 Version/10.00";
const char S73860[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Ex Note Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73861[] = "Mozilla/5.0 (Series40; Nokia2700c-2/09.97; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S73862[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Ascend G300 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73863[] = "Mozilla/5.0 (Linux; Android 4.0.4; JY-G3 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S73864[] = "Mozilla/5.0 (PlayStation Vita 2.02) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S73865[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ616-12) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S73866[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ616 Build/7.7.1-141-6-PAST-UMTS-LA) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S73867[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9082) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S73868[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-M830 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73869[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xperia Acro S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73870[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26w Build/6.2.B.1.96) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73871[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26w Build/6.2.B.1.96) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S73872[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S73873[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S73874[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT875 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S73875[] = "Mozilla/5.0 (Linux; Android 4.1.2; IdeaTabA1000-F Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S73876[] = "Mozilla/5.0 (Linux; Android 4.1.1; XOLO A1000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S73877[] = "Mozilla/5.0 (Linux; Android 4.1.2; IdeaTabA1000-G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S73878[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I537 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S73879[] = "SAMSUNG-GT-B3313/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S73880[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; 003P Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73881[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; 003P Build/GRI40)";
const char S73882[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11";
const char S73883[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11";
const char S73884[] = "HUAWEI_T8300_TD/1.0 OPhone/2.5 (Linux;Android 2.2) Release/03.20.2011 Browser/WAP2.0(AppleWebKit/533.1) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S73885[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HUAWEI T8300 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S73886[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HUAWEI T8300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73887[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HUAWEI T8300 Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S73888[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:15.0) Gecko/X Thunderbird/15.0.1";
const char S73889[] = "JUC (Linux; U; 2.1-update1; zh-cn; E16i; 320*480) UCWEB7.9.0.94/139/444";
const char S73890[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo P700i Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S73891[] = "Mozilla/5.0 (Linux; Android 4.0.3; Lenovo P700 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S73892[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S73893[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S73894[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae71-1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S73895[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE71) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S73896[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae72-1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile  3gpp-gba";
const char S73897[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE72-1) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S73898[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Explorer A310e Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S73899[] = "MOT-V3iv/08.B5.10R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S73900[] = "MOT-V3i/08.B4.34R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S73901[] = "MOT-V3ire/08.00.33R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S73902[] = "MOT-V3i /08.03.05R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S73903[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) MOT-Q9/01.07.28R";
const char S73904[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) MOT-Q9/01.06.08R";
const char S73905[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) MOT-Q9/01.07.05R";
const char S73906[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) MOT-Q9/01.07.37R";
const char S73907[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) 320x240; VZW; Motorola-Q9m; Windows Mobile 6.0 Standard";
const char S73908[] = "SonyEricssonW150a/R2AD032 TelecaBrowser/Q07C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S73909[] = "SAMSUNG-SGH-T919/T919UVHI5 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S73910[] = "Mozilla/5.0 (Windows NT 6.0; rv:12.0) Gecko/X Firefox/12.0 SeaMonkey/2.9.1";
const char S73911[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:12.0) Gecko/X SeaMonkey/2.9.1";
const char S73912[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.20) Gecko/X Firefox(2.0.0.20)";
const char S73913[] = "Mozilla/5.0 (X11; Linux i686; rv:2.0b10pre) Gecko/X Firefox-4.0/4.0b10pre";
const char S73914[] = "Mozilla/5.0 (X11; ; Linux x86_64; rv:1.8.1.6) Gecko/X Firefox";
const char S73915[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; th-TH; OPPOX9015 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73916[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; U19GT-W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73917[] = "Mozilla/5.0 (Linux; Android 4.0.4; NABI2-NV7A Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S73918[] = "Mozilla/5.0 (Linux; Android 4.0.4; NABI2-NV7A-UK Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S73919[] = "Mozilla/5.0 (Linux; Android 4.0.4; NABI2-NV7A-IE Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S73920[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MP843 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73921[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB192 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S73922[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB0192 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S73923[] = "Mozilla/5.0 (Linux; Android 4.1.1; bq Curie Build/1.0.2 X-19:30) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S73924[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; A953 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73925[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; A953 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73926[] = "Mozilla/5.0 (Linux; Android 4.1.2; Transformer Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S73927[] = "Mozilla/5.0 (Linux; Android 4.1.2; Transformer TF101 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S73928[] = "Mozilla/5.0 (Linux; Android 4.1.1; Transformer TF101 Build/MASTER) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S73929[] = "UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 710) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile";
const char S73930[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8013 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S73931[] = "Mozilla/5.0 (Linux; Android 4.0.3; U8860 Build/HuaweiU8860) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S73932[] = "SAMSUNG-GT-M8910/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S73933[] = "ZTE-E520/1.0 SMIT-Browser/2.0.0";
const char S73934[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; Galaxy Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S73935[] = "Mozilla/5.0 (Linux; U; Android 1.6 - Galaxo V1.6.3.3; en-gb; Galaxy Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S73936[] = "Mozilla/5.0 (Linux; U; Android 1.6 - Galaxo V1.6.3.4; en-gb; Galaxy Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S73937[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; T-Mobile myTouch 3G Build/COC10) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S73938[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; T-Mobile myTouch 3G Build/CRB57) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S73939[] = "Mozilla/5.0 (Linux; U; Android 1.5; es-; T-Mobile myTouch 3G Build/COC10) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S73940[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/525.18 (KHTML, like Gecko, Safari/525.20) OmniWeb/v622.6.0.X";
const char S73941[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/528.16 (KHTML, like Gecko, Safari/528.16) OmniWeb/v622.8.0.X";
const char S73942[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/525.18 (KHTML, like Gecko, Safari/525.20) OmniWeb/v622.4.0.X";
const char S73943[] = "Mozilla/5.0 (Linux; Android 4.1.1; SCH-I500 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S73944[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I500 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S73945[] = "ZTEU880s_TD/1.0 Linux/2.6.35 Android/2.3 Release/2.10.2012 Browser/AppleWebKit533.1";
const char S73946[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE U880s Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S73947[] = "Mozilla 5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE U880s Build GWK74) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S73948[] = "ZTEU880s2_TD/1.0 Linux/2.6.35 Android/2.3.5 Release/03.29.2012 Browser/AppleWebKit533.1";
const char S73949[] = "JUC (Linux; U; 4.0.4; zh-cn; GT-P3100; 600*976) UCWEB7.9.0.94/139/444";
const char S73950[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaE75-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S73951[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaE75-2) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S73952[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI 1S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S73953[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI 1SC Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S73954[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax P600 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0  Safari/534.30";
const char S73955[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, -0500, us";
const char S73956[] = "Mozilla/5.0 (compatible; MSIE 3.0; WindowsCE; Trident/1.0)";
const char S73957[] = "Mozilla/5.0 (compatible; MSIE 3.0; 68K; Tablet PC; Trident/1.0)";
const char S73958[] = "Mozilla/5.0 (Linux; Android 4.1.2; Andromax U Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73959[] = "Mozilla/5.0 (Linux; Android 4.1.2; Andromax U Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S73960[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-US730 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S73961[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; GOCLEVER TAB M723G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73962[] = "UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 510) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile";
const char S73963[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; vivo Xplay Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73964[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; vivo Xplay Build/JDQ39)";
const char S73965[] = "Mozilla/5.0 (Linux; Android 4.2.2; SM-T311 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S73966[] = "Mozilla/5.0 (Linux; Android 4.0.4; T-Hub2 Build/TVA301TELBG3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S73967[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire 500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S73968[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire 500 dual sim Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S73969[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC_Desire_500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S73970[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) 320x240; VZW; Motorola-Q9c; Windows Mobile 6.1 Standard";
const char S73971[] = "Mozilla/4.0 SonyEricssonC905v/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3";
const char S73972[] = "Nokia2700c-2/2.0 (09.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2700c-2/UC Browser7.9.0.102/70/355";
const char S73973[] = "Nokia2700c-2/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/69/352";
const char S73974[] = "Mozilla/6.0 (compatible; MSIE 7.01; Windows CE; 760x412; Samsung)";
const char S73975[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8L1 Safari/6531.22.7";
const char S73976[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8J2 Safari/6531.22.7";
const char S73977[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8J3 Safari/6531.22.7";
const char S73978[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S73979[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S73980[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3,Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S73981[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S73982[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S73983[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S73984[] = "Mozilla/5.0 (X11; U; Linux armv7l; it-IT; rv:1.9.2a1pre) Gecko/X Firefox/3.5 Maemo Browser 1.5.6 RX-51 N900";
const char S73985[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; PC1007 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73986[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; PC1007 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S73987[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Incredible 2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73988[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Incredible 2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73989[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Incredible 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73990[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Incredible 2 Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73991[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Incredible 2 Build/Kangdemnation_V0.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73992[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Incredible 2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73993[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Incredible 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73994[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Droid Incredible 2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73995[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Incredible 2 Build/Kangdemnation_V0.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S73996[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:15.0) Gecko/X Thunderbird/15.0.1";
const char S73997[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i909 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73998[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-en; SGH-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S73999[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i909 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S74000[] = "Mozilla/5.0 (Linux; Android 4.0.4; SH530U Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74001[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; ZTE V880E Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74002[] = "Lemon T99 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10";
const char S74003[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; starTIM tab 7 Build/TIM_01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74004[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:16.0) Gecko/X Firefox/16.0 epic/16.0";
const char S74005[] = "Mozilla/5.0 (Windows NT 6.2; rv:16.0) Gecko/X Firefox/16.0 Epic/16.0";
const char S74006[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730M Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S74007[] = "Mozilla/5.0 (Linux; Android 4.0.4; SoftwinerEvb Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S74008[] = "Mozilla/5.0 (Linux; Android 4.0.4; Softwinerf761 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S74009[] = "Mozilla/5.0 (Linux; Android 4.0.4; SoftwinerEvb Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S74010[] = "Mozilla/5.0 (Linux; Android 4.0.4; Softwinerf900 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S74011[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S74012[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GenxDroid7 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74013[] = "Mozilla/5.0 (X11; SunOS i86pc; rv:22.0) Gecko/X Firefox/22.0";
const char S74014[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; iris504Q Build/iris504Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74015[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8262 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S74016[] = "ZTE-G-N281/Obigo/Q03";
const char S74017[] = "ZTE-G-N281-Miami-Orange/ORANGE_ENG_EN_P107A3V1.0.7 Obigo/Q03C";
const char S74018[] = "ZTE-G-N281-LeTactile-Orange/ORANGE_OFR_FR_P107A3V1.0.8 Obigo/Q03C";
const char S74019[] = "ZTE-G-N281-Miami-Orange/ORANGE_OSP_SP_P107A3V1.0.3 Obigo/Q03C";
const char S74020[] = "ZTE-G-N281-Miami-Orange/ORANGE_OSP_SP_P107A3V1.0.7 Obigo/Q03C";
const char S74021[] = "ZTE-G-N281-Miami-Orange/ORANGE_ENG_EN_P107A3V1.0.4 Obigo/Q03C";
const char S74022[] = "ZTE-G-N281-Miami-Orange/ORANGE_ENG_EN_P107A3V1.0.8 Obigo/Q03C";
const char S74023[] = "ZTE-G_R221/WAP2.0";
const char S74024[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-I997 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74025[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; SGH-I997 Build/MIUI)";
const char S74026[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-I997 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74027[] = "Mozilla/5.0 (Linux; U; Android 2.3.9; en-us; SGH-I997 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74028[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5 Comodo_Dragon/19.2.0.0";
const char S74029[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; PP803 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74030[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; tmn smart a15 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74031[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; ZTE V880E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74032[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 80XSK Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S74033[] = "sprd-TSM_T6/1.0 Linux/Apps_2.6.32.9-perf Android/2.3.7 Release/01.19.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S74034[] = "sprd-TSM_T6/1.0 Linux/Apps_2.6.32.9-perf Android/2.3.7 Release/01.24.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S74035[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; sprd-TSM_T6/1.0 Android/2.3.7 Release/04.22.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74036[] = "Mozilla/5.0 (Linux; Android 4.0.3; Transformer Prime TF201 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S74037[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; TAD-10021 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74038[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; OV-BaseCore10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74039[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6502 Build/10.1.A.1.434) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S74040[] = "LAVA50_11B_HW (MRE\\3.0.00(2500) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) LAVA_DISCOVER135_S208_X Release/2012.12.18 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US";
const char S74041[] = "LAVA50_11B_HW (MRE\\3.0.00(2500) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) LAVA_DISCOVER132_S210_X Release/2012.12.14 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US";
const char S74042[] = "VSUN50_FTE_11B_HW (MRE/3.0.00(512);MAUI/C11_S422_X;BDATE/2013/06/05 15:05;LCD/X;CHIP/MT6250;KEY/Normal;TOUCH/1;CAMERA/1;SENSOR/0;DEV/VSUN50_FTE_11B_HW;WAP Browser/MAUI ();GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/005;GAMELOFT/006;MOBIM/007;KKFUN/008;MT) LAVA_C11_S422_X Release/2013.06.05 WAP Browser/MAUI Profile/  Q03C1-2.40 en-US";
const char S74043[] = "VSUN50_FTE_11B_HW (MRE/3.0.00(512);MAUI/C11_S418_X;BDATE/2013/05/10 13:45;LCD/X;CHIP/MT6250;KEY/Normal;TOUCH/1;CAMERA/1;SENSOR/0;DEV/VSUN50_FTE_11B_HW;WAP Browser/MAUI ();GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/005;GAMELOFT/006;MOBIM/007;KKFUN/008;MT) LAVA_C11_S418_X Release/2013.05.10 WAP Browser/MAUI Profile/  Q03C1-2.40 en-US";
const char S74044[] = "LAVA50_11B_HW (MRE\\3.0.00(2500) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) LAVA_DISCOVER135_S214_X Release/2013.04.28 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US";
const char S74045[] = "FARSIGHTED60A_12864_COSMOS_BU1_11B_HW (MRE/3.1.00(1536);MAUI/DISCOVER135_S106_X;BDATE/2013/08/09 11:20;LCD/X;CHIP/MT6260;KEY/Reduced;TOUCH/1;CAMERA/0;SENSOR/0;DEV/FARSIGHTED60A_12864_COSMOS_BU1_11B_HW;WAP Browser/MAUI (HTTP PGDL);GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/) LAVA_DISCOVER135_S106_X Release/2013.08.09 WAP Browser/MAUI (HTTP PGDL) Profile/  Q03C1-2.40 en-US";
const char S74046[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S74047[] = "SAMSUNG-S7220/S7220ACIB1 SHP/VPP/R5 NetFront/3.5 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S74048[] = "Nokia6555c/2.0 (03.41) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S74049[] = "Nokia6555b/2.0 (03.31) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S74050[] = "Nokia6555/2.0 (03.46) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S74051[] = "Mozilla/5.0 (Linux; U; Android 1.5; de-; sdk Build/CUPCAKE) AppleWebkit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S74052[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; sdk Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S74053[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; sdk Build/CUPCAKE) AppleWebkit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S74054[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; google_sdk Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S74055[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Incredible 2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74056[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Incredible 2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74057[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.2.0.0 Chrome/12.0.742.112 Safari/534.30";
const char S74058[] = "Mozilla/5.0 (PLAYSTATION 3 4.10) AppleWebKit/531.22.8 (KHTML, like Gecko)";
const char S74059[] = "Mozilla/5.0 (PLAYSTATION 3 4.10) AppleWebKit/531.22.8 (KHTML	 like Gecko)";
const char S74060[] = "Mozilla/5.0 (X11; U; SunOS i86pc; en-US; rv:1.9.0.8) Gecko/X Firefox/3.0.8";
const char S74061[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; HKPHONE H8-3G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74062[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; H8-3G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74063[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; H8-3G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74064[] = "iTunes/10.5.2";
const char S74065[] = "iTunes/10.6";
const char S74066[] = "iTunes (unknown version) CFNetwork/520.2.6";
const char S74067[] = "iTunes/10.0";
const char S74068[] = "iTunes/10.1.6";
const char S74069[] = "MicromaxX444/Q03C MAUI-browser; Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S74070[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Schubert)";
const char S74071[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; 7 Schubert T9292)";
const char S74072[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-09D Build/V19R26A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74073[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-09D Build/V20R27B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74074[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-09D Build/V21R28A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74075[] = "Mozilla/5.0 (Windows NT 5.1; rv:15.0) Gecko/X Firefox/15.0.2.0 IceDragon/15.0.2.0";
const char S74076[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XT912 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74077[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT912 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74078[] = "NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-05) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S74079[] = "Mozilla/5.0 (Windows NT 6.0; rv:16.1) Gecko/X Firefox/16.1 IceDragon/16.1.0.1";
const char S74080[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PG86100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74081[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PG86100 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74082[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Evo 3D CDMA Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74083[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN700W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74084[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN700W Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S74085[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GN700W Build/IMM76D)";
const char S74086[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn;GiONEE-GN700W/GN700W_T15 Build/IMM76D) AppleWebKit534.30(KHTML,like Gecko)Version/4.0 Mobile Safari/534.30";
const char S74087[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn;GiONEE-GN700W/GN700W_T16 Build/IMM76D) AppleWebKit534.30(KHTML,like Gecko)Version/4.0 Mobile Safari/534.30";
const char S74088[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn;GiONEE-GN700W/GN700W_T11 Build/IMM76D) AppleWebKit534.30(KHTML,like Gecko)Version/4.0 Mobile Safari/534.30";
const char S74089[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74090[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562i Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74091[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-I997 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74092[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I997 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74093[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E460 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S74094[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Celkon A107 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74095[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; PMP5870C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74096[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5230) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S74097[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R830 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S74098[] = "HTC_Touch_T2223 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S74099[] = "HTC_Touch_Viva_T2223 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S74100[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC_Touch_Viva_T2223; OpVer 86.157.15.753";
const char S74101[] = "LGE-VX8360/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S74102[] = "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaC5-04.1/10.1.031; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.9 3gpp-gba";
const char S74103[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; tegav2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74104[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; tegav2 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74105[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.152 Safari/535.19 Comodo_Dragon/18.1.2.0";
const char S74106[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; fr-FR; rv:1.7.5) Gecko/X Thunderbird/1.0";
const char S74107[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; SPX-5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74108[] = "Mozilla/5.0 (X11; SunOS i86pc; rv:12.0) Gecko/X Firefox/12.0";
const char S74109[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; PTAB4300 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74110[] = "JUC (Linux; U; 2.2; zh-cn; GT-P1000; 600*1024) UCWEB7.9.0.94/139/444";
const char S74111[] = "JUC (Linux; U; 2.3.3; zh-cn; GT-P1000; 600*1024) UCWEB7.9.0.94/139/444";
const char S74112[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5";
const char S74113[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5";
const char S74114[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S74115[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; BLU_P100i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74116[] = "Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.6 Safari/535.19 Silk-Accelerated=true";
const char S74117[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MAXWEST Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74118[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MAXWEST Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74119[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MAXWEST_Orbit4150 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74120[] = "MT6515-MT6515-C909/1.0 Linux/Apps_4.0.32.9-perf Android/4.0.3 Release/11.20.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S74121[] = "MT6515-MT6515-H003/1.0 Linux/Apps_4.0.32.9-perf Android/4.0.3 Release/11.30.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S74122[] = "MT6515-MT6515-H003/1.0 Linux/Apps_4.0.32.9-perf Android/4.0.3 Release/01.11.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S74123[] = "MT6515-MT6515-H003/1.0 Linux/Apps_4.0.32.9-perf Android/4.0.3 Release/01.12.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S74124[] = "MT6515-MT6515-H003/1.0 Linux/Apps_4.0.32.9-perf Android/4.0.3 Release/03.20.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S74125[] = "MT6515-MT6515-C909/1.0 Linux/Apps_4.0.32.9-perf Android/4.0.3 Release/03.05.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S74126[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; LG-E455f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74127[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E455f Build/JZO54K)";
const char S74128[] = "ultrafone 303 Linux/3.0.13 Android/4.0.4 Release/04.11.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S74129[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ultrafone 303 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74130[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S74131[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E610 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S74132[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S74133[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612f Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S74134[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S74135[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320T Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S74136[] = "Mozilla/5.0 (Series40; NokiaX2-01/08.70; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S74137[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.0.0.0 Chrome/13.0.782.107 Safari/535.1";
const char S74138[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.0.0.0 Chrome/13.0.782.107 Safari/535.1";
const char S74139[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.8.1.8pre) Gecko/X Firefox/2.0.0.8 Navigator/9.0.0.1";
const char S74140[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) chromeframe/7.0.517.44";
const char S74141[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB0.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729) chromeframe/7.0.517.44";
const char S74142[] = "Mozilla/5.0 (Linux; Android 4.0.4;  SPH-D700 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S74143[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D700 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74144[] = "Mozilla/5.0 (Linux; Android 4.0.3; SPH-D700 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74145[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S74146[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; AT500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74147[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; AT500a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74148[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; iBall Andi3e Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74149[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; MEDION LIFE P4012 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74150[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; LT-TLA Build/LTLMR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74151[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.2 Mobile Safari/535.19 Silk-Accelerated=true";
const char S74152[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; id-id; IMO Z7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74153[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; IMO Z7 Build/JRO03C)";
const char S74154[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; IRIS355 Build/IRIS355) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74155[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Novo7 Venus Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74156[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Novo7 Venus Build/JRO03C)";
const char S74157[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3113 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S74158[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P3113 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S74159[] = "Mozilla/5.0 (Linux; Android 4.0.4; StarTrail III Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74160[] = "Mozilla/5.0 (Linux; Android 4.1.2; TA-FONE A37 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74161[] = "Mozilla/5.0 (Linux; Android 4.1.2; TA-FONE A37 HD Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74162[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; Aqua Glow Build/MocorDroid2.3.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74163[] = "Mozilla/5.0 (Linux; Android 4.0.4; WIKO-CINK SLIM Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S74164[] = "Mozilla/5.0 (Linux; Android 4.1.1; D2-721G Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S74165[] = "SAMSUNG-SGH-E250i/E250iJBIB2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S74166[] = "SAMSUNG-SGH-E250/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S74167[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en; rv:1.7.12) Gecko/X Firefox/1.0.7 Madfox/3.0";
const char S74168[] = "Mozilla/5.0 (Symbian3; U; en-us; NokiaC7-00)/UC Browser8.5.0.183/51/355/UCWEB Mobile";
const char S74169[] = "NokiaC7-00/UCWEB8.5.0.163/51/999";
const char S74170[] = "Mozilla/5.0 (Java; U; en-us; nokiac7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile  3gpp-gba";
const char S74171[] = "JUC (Linux; U; 2.3.7; zh-cn; ST25i; 480*854) UCWEB7.9.0.94/139/444";
const char S74172[] = "JUC (Linux; U; 4.0.4; zh-cn; ST25i; 480*854) UCWEB7.9.0.94/139/444";
const char S74173[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax A110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S74174[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.6 Mobile Safari/535.19 Silk-Accelerated=true";
const char S74175[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Allview P5-Mini Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74176[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ro-ro; P5-Mini Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74177[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I815 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S74178[] = "Mozilla/5.0 (Linux; Android 4.0.4; Next7P12 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S74179[] = "Mozilla/5.0 (Linux; Android 4.0.4; Next7P12-8G Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S74180[] = "Mozilla/5.0 (Linux; Android 4.0.3; Le Pan S Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S74181[] = "skk Phoenix Linux/3.0.13 Android/4.0.4 Release/03.13.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S74182[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; skk Phoenix Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74183[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Vandroid T3-A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74184[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax A110 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S74185[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ME860 Build/OLHKT_U4_0.50.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74186[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; ME860 Build/OLHKT_U4_0.56.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74187[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; Moto_ME860 Build/Blur_Version.4.0.560.ME860.Retail.en.05 Flex/P019) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74188[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME860 Build/OLYGC_U4_1.27.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74189[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Moto_ME860 Build/Blur_Version.4.0.500.ME860.Retail.en.05 Flex/P017) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74190[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME860 Build/OLYGC_U4_1.21.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74191[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME860 Build/OLYGC_U4_1.33.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74192[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME860 Build/OLYGC_U4_1.31.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74193[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME860 Build/OLYGC_U4_1.33.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S74194[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; SHW-M220L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74195[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SHW-M220L Build/FROYO)";
const char S74196[] = "Mozilla/5.0 (X11; U; SunOS sun4u; en-US; rv:1.0.1) Gecko/X Netscape/7.0";
const char S74197[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ZP300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74198[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ENERGY s10 DUAL Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74199[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5570) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S74200[] = "Mozilla/5.0 (Linux; Android 4.1.2; A511 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S74201[] = "Mozilla/5.0 (Linux; Android 4.1.1; MI 2 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S74202[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LNV-Lenovo_A600e Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74203[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LNV-Lenovo A600e Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74204[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LNV-Lenovo A600e Build/ICS) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S74205[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LNV-Lenovo A600e Build/ICS)";
const char S74206[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTCEVOV4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S74207[] = "Mozilla/5.0 (Linux; Android 2.3.6; SGH-T989 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S74208[] = "Mozilla/5.0 (Linux; Android 2.3.5; SGH-T989D Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S74209[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT30at Build/7.0.B.1.152) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S74210[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5021E Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74211[] = "Mozilla/5.0 (Linux; Android 4.1.2; A500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S74212[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ609 Build/7.7.1-85_MZ609-31) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S74213[] = "Mozilla/5.0 (Linux; Android 4.2.1; A210 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S74214[] = "Mozilla/5.0 (Linux; Android 4.2.1; A210 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S74215[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTL21 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S74216[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I717 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S74217[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.1.0.64021 Mobile/10B141 Safari/7534.48.3";
const char S74218[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.1.0.64021 Mobile/10B147 Safari/7534.48.3";
const char S74219[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.1.1.64324 Mobile/10B329 Safari/7534.48.3";
const char S74220[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.1.1.64324 Mobile/10B146 Safari/7534.48.3";
const char S74221[] = "SAMSUNG-SGH-B130-ORANGE/B130ABHI2 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S74222[] = "SAMSUNG-SGH-B130/B130XAHF4 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S74223[] = "SAMSUNG-SGH-B130L/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S74224[] = "Mozilla/5.0 (Windows NT 4.0) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S74225[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/28/351";
const char S74226[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/50/351";
const char S74227[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/28/355";
const char S74228[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/50/355";
const char S74229[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/159/352";
const char S74230[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/50/352";
const char S74231[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/159/444";
const char S74232[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/27/351";
const char S74233[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/160/352";
const char S74234[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/159/351";
const char S74235[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/159/350";
const char S74236[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/27/352";
const char S74237[] = "Mozilla/5.0 (Nokia; Qt; MeeGo)";
const char S74238[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Comodo_Dragon/1.0.0.5 Safari/532.0";
const char S74239[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Comodo_Dragon/1.0.0.9 Chrome/ Version/3.2.1 Safari/532.0";
const char S74240[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Comodo_Dragon/1.0.0.5 Safari/532.0";
const char S74241[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1.16) Gecko/X Icedove/3.0.11";
const char S74242[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.16) Gecko/X Icedove/3.0.11";
const char S74243[] = "LGCMF 0.8.1.5";
const char S74244[] = "LGMC-LGEAS/5.074IV";
const char S74245[] = "LGMC-LGEAS/5.23IV";
const char S74246[] = "LGCMF 0.8.1.7";
const char S74247[] = "LGMC-LGEAS/5.09L1A";
const char S74248[] = "LGMC-LGEAS/6.44.4";
const char S74249[] = "LGE-bryce-VS910 4G";
const char S74250[] = "LGMC-LGEAS/6.44.9";
const char S74251[] = "Mozilla/5.0 (Linux; Android 4.0.4; SO-04D Build/7.0.D.1.130) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74252[] = "Micromax X294/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S74253[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:18.0) Gecko/X Thunderbird/18.0";
const char S74254[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TP10.1-1500DC Dark grey metal Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74255[] = "Mozilla/5.0 (Linux; Android 4.1.2; iris502 Build/LAVAIRIS502) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74256[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P895 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S74257[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P895qb Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S74258[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S74259[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR910L Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S74260[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR910L Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S74261[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S74262[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2S Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S74263[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2A Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S74264[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2C Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S74265[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2SC Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S74266[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) T-Mobile Dash";
const char S74267[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SKY IM-A600S Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74268[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SKY IM-A600S Build/FRG83)";
const char S74269[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-; SKY IM-A600S Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74270[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT319 Build/V4.37B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74271[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; XT319 Build/V4.39A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74272[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; XT319 Build/V4.50A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74273[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; XT319 Build/V4.50A) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S74274[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; XT319 Build/V4.560) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74275[] = "Opera/9.64 (Windows ME; U; en) Presto/2.1.1";
const char S74276[] = "Opera/9.60 (Windows ME; U; en) Presto/2.1.1";
const char S74277[] = "Opera/9.62 (Windows ME; U; en) Presto/2.1.1";
const char S74278[] = "Opera/9.63 (Windows ME; U; de) Presto/2.1.1";
const char S74279[] = "Mozilla/5.0 (Series40; Nokia6300/07.21; Profile/MIDP-2.0 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S74280[] = "Mozilla/5.0 (Linux; U; Android 3.2; pt-br; MZ607 Build/1.6.0_218.11-MZ607) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S74281[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; pt-br; MZ607 Build/1.6.0M_272.15_MZ607) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S74282[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; pt-br; MZ607 Build/1.6.0M_272.15_MZ607) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S74283[] = "MacOutlook/14.2.0.X (Intel Mac OS X 10.7.3)";
const char S74284[] = "MacOutlook/14.2.1.X (Intel Mac OS X 10.7.3)";
const char S74285[] = "MacOutlook/14.2.2.X (Intel Mac OS X 10.7.4)";
const char S74286[] = "MacOutlook/14.2.2.X (Intel Mac OS X 10.7.3)";
const char S74287[] = "MacOutlook/14.2.1.X (Intel Mac OS X 10.7.4)";
const char S74288[] = "MacOutlook/14.2.3.X (Intel Mac OS X 10.7.4)";
const char S74289[] = "MacOutlook/14.2.4.X (Intel Mac OS X 10.7.5)";
const char S74290[] = "MacOutlook/14.2.5.X (Intel Mac OS X 10.7.5)";
const char S74291[] = "MacOutlook/14.2.3.X (Intel Mac OS X 10.7.5)";
const char S74292[] = "MacOutlook/14.2.2.X (Intel Mac OS X 10.7.5)";
const char S74293[] = "Mozilla/5.0 (X11; Linux i686; rv:12.0) Gecko/X Firefox/12.0 SeaMonkey/2.9";
const char S74294[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; TAB-1012 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74295[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS12M Build/6.7.5-25_LE-8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74296[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS12M Build/6.7.5-50_MR1-11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74297[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, gb";
const char S74298[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, gb";
const char S74299[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:17.0) Gecko/17.0 Firefox/17.0 IceDragon/17.0.1.0";
const char S74300[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S74301[] = "NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-02) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S74302[] = "JUC (Linux; U; 2.3.5; zh-cn; Micromax_A87; 480*800) UCWEB7.9.0.94/139/444";
const char S74303[] = "Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6303classic) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S74304[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cnm TOUCHPAD 9.7 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S74305[] = "Mozilla/5.0 (Linux; Android 4.0.4; Versus TouchPad 7 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S74306[] = "Mozilla/5.0 (Linux; Android 4.0.4; CnM TOUCHPAD 9 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S74307[] = "Mozilla/5.0 (Linux; Android 4.0.4; Vs TOUCHPAD 9.7 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S74308[] = "Mozilla/5.0 (Linux; Android 4.0.4; Versus TouchPad 9 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S74309[] = "Mozilla/5.0 (Series40; Nokia2730c-1/10.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S74310[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.1.140) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S74311[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; KUNO4 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74312[] = "Huawei/1.0/0HuaweiG3610/WAP2.0/Obigo-Browser/Q03C MMS/Obigo-MMS/1.2";
const char S74313[] = "Huawei/1.0/0HuaweiG3610/WAP2.0/Obigo-Browser/Q03C MMS/Obigo-MMS/1.2, Huawei/1.0/0HuaweiG3610/WAP2.0/Obigo-Browser/Q03C MMS/Obigo-MMS/1.2";
const char S74314[] = "LG-KM555/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S74315[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S74316[] = "JUC(Linux;U;Android2.3.4;Zh_cn;LT18i;480*854;)UCWEB7.8.0.95/139/351";
const char S74317[] = "Mozilla/5.0(Linux;U;Android2.3.4;Zh_cn;LT18i;480*854;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S74318[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, us";
const char S74319[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I747M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, ca";
const char S74320[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-L710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, us";
const char S74321[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-R530U Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, us";
const char S74322[] = "Mozilla/5.0 (X11; FreeBSD amd64; rv:14.0) Gecko/X Firefox/14.0.1";
const char S74323[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Safari/535.19 Silk-Accelerated=false";
const char S74324[] = "SAMSUNG-GT-E2250/E2250XXLG2 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S74325[] = "Mozilla/5.0 (Linux; Android 4.0.4; S100 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74326[] = "Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook Tablet Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S74327[] = "Mozilla/5.0 (Linux; Android 4.1.2; NookTablet Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S74328[] = "NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-05) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S74329[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22 Sleipnir/4.0.1";
const char S74330[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F180L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S74331[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F180S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S74332[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F180K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S74333[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X+ Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S74334[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9505 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/jfltexx";
const char S74335[] = "Mozilla/5.0 (Linux; Android 4.1.2; S500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S74336[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S74337[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S74338[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S74339[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT30at Build/7.0.B.1.152) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S74340[] = "Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_E10310 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S74341[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S74342[] = "Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; SonyEricssonSO-01B Build/R1EA029) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S74343[] = "Opera/9.10 (X11; Linux i686; U; en)";
const char S74344[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.1.0.0 Chrome/17.0.963.38 Safari/535.11";
const char S74345[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.1.0.0 Chrome/17.0.963.38 Safari/535.11";
const char S74346[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ViewPhone3 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74347[] = "Spice Mi280 Linux/2.6.35 Android/2.3.4 Release/12.09.2011 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1";
const char S74348[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Spice Mi280 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74349[] = "Mozilla/5.0 (Windows NT 6.0; rv:13.0) Gecko/X Firefox/13.0.1 SeaMonkey/2.10.1";
const char S74350[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P7300 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S74351[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7300 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S74352[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; ALCATEL ONE TOUCH 918N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S74353[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ME171 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74354[] = "MOT-EX440 MIDP-2.0/CLDC-1.1 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10";
const char S74355[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; SGH-T989 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74356[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MID1018 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74357[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; FZ-A1B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74358[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; DPS Dream 7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74359[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6810 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S74360[] = "MQQBrowser/42 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S74361[] = "MQQBrowser/421 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S74362[] = "MQQBrowser/42 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3";
const char S74363[] = "MQQBrowser/421 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3";
const char S74364[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ616-12) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S74365[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ616 Build/7.7.1-141-6-PAST-UMTS-LA) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S74366[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-AS730 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S74367[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE51) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S74368[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I8160 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S74369[] = "Mozilla/5.0 (Linux; Android 4.0.3; KFOT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S74370[] = "Mozilla/5.0 (Linux; Android 4.0.4; KF012 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S74371[] = "SAMSUNG-E1360B-Bouygues/E1360BAGJC1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S74372[] = "SAMSUNG-E1360B/E1360BXAID3 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S74373[] = "SAMSUNG-E1360M/E1360MXEID2 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S74374[] = "SAMSUNG-E1360B-ORANGE/E1360BBVID1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S74375[] = "SAMSUNG-E1360/E1360JAIC1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S74376[] = "Vodafone/1.0/SAMSUNG-E1360B/ARID1/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S74377[] = "Nokia1680c_CMCC/2.0 (05.61) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S74378[] = "Nokia1680c/2.0 (05.63) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S74379[] = "Huawei-U2800/U2800ACBXB119 Browser/Obigo-Browser/Q05A";
const char S74380[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.1.1.0 Chrome/14.0.835.202 Safari/535.1";
const char S74381[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.1.1.0 Chrome/14.0.835.202 Safari/535.1";
const char S74382[] = "ZTE-Z221/1.3.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S74383[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800; SFR)";
const char S74384[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800; T-Mobile)";
const char S74385[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800)";
const char S74386[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800; Orange)";
const char S74387[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800; Vodafone)";
const char S74388[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Nokia 800)";
const char S74389[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800; Meteor)";
const char S74390[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800; 800 BOUYGUES)";
const char S74391[] = "Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia6303iclassic) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S74392[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAC-10011 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74393[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT912 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S74394[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5670 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S74395[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-S5670 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S74396[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S74397[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S74398[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-N719 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S74399[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E977 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74400[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S74401[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S74402[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S74403[] = "SonyEricssonT303c/R2CC001 TelecaBrowser/Q04C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S74404[] = "SonyEricssonT303/R2CC001 TelecaBrowser/Q04C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S74405[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.50 [en]";
const char S74406[] = "Nokia6600/1.0 (5.53.0) SymbianOS/7.0s Series60/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.9.1.120/27/351/UCWEB";
const char S74407[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LU6200 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74408[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-lu6200 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74409[] = "HTCT328t_TD/1.0 Android/4.0 release/2012 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S74410[] = "MQQBrowser/2.6 Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S74411[] = "MQQBrowser/2.6 Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3";
const char S74412[] = "IUC(U;iOS 6.0;Zh-cn;320*480;)/UCWEB8.7.1.201/44/997";
const char S74413[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LG-P500 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74414[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P500 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74415[] = "Mozilla/5.0 (Linux; Android 4.0.4; KB901 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74416[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SIMM-X720 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74417[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SIMM-X720 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74418[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; X720 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74419[] = "Mozilla/5.0 (Linux; Android 4.1.2; Huawei MediaPad Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S74420[] = "Mozilla/5.0 (Linux; Android 4.1.2; dtab01 Build/HuaweiMediaPad) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S74421[] = "Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 10 LINK Build/HuaweiMediaPad) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S74422[] = "Mozilla/5.0 (PlayStation Vita 2.06) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S74423[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR910L Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S74424[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR910L Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S74425[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S74426[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; ST26i Build/11.2.A.0.21) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S74427[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; ST26i Build/11.2.A.0.31) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S74428[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N7102 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74429[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S74430[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G X710s Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S74431[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT18i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S74432[] = "Mozilla/5.0 (Linux; Android 4.0.3; LT18i Build/4.1.A.0.562) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S74433[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S74434[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S74435[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 925)";
const char S74436[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 925; Vodafone)";
const char S74437[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 925; 1326-2)";
const char S74438[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 925; ANZ892)";
const char S74439[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AURUS III Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S74440[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC6435LVW Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S74441[] = "Mozilla/4.0 (compatible; Linux 2.6.22) NetFront/3.4 Kindle/2.1 (screen 824x1200; rotate)";
const char S74442[] = "Mozilla/4.0 (compatible; Linux 2.6.22) NetFront/3.4 Kindle/2.5 (screen 824x1200; rotate)";
const char S74443[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5838 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74444[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5838 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S74445[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5838 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S74446[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5838 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74447[] = "Mozilla/5.0 (Linux; U; Android 2.3;en-us; ViewSonic-ViewPad7e build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74448[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.1; ViewPad7e Build/GINGERBREAD)";
const char S74449[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-MOBILE i-STYLE 5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74450[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-MOBILE i-STYLE 5i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74451[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MIDC497 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74452[] = "Mozilla/5.0 (Linux; Android 4.0.3; KFOT Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S74453[] = "Mozilla/5.0 (Linux; Android 4.0.4; KF012 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S74454[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-66) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S74455[] = "Nokia205/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia205) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S74456[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R760 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S74457[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaN73) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S74458[] = "SonyEricssonW902/R3EA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.3";
const char S74459[] = "Mozilla/4.0 SonyEricssonW902v/R3EE Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.3";
const char S74460[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.64.347 Chrome/13.0.782.218 Safari/535.1";
const char S74461[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.68.1565 Chrome/14.0.835.202 Safari/535.1";
const char S74462[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.71.128 Chrome/14.0.835.202 Safari/535.1";
const char S74463[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.72.698 Chrome/14.0.835.202 Safari/535.1";
const char S74464[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/534.24 (KHTML, like Gecko) RockMelt/0.9.58.494 Chrome/11.0.696.71 Safari/534.24";
const char S74465[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.30 (KHTML, like Gecko) RockMelt/0.9.62.313 Chrome/12.0.742.122 Safari/534.30";
const char S74466[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/534.13 (KHTML, like Gecko) RockMelt/0.9.48.51 Chrome/9.0.597.107 Safari/534.13";
const char S74467[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.13 (KHTML, like Gecko) RockMelt/0.9.48.59 Chrome/9.0.597.107 Safari/534.13";
const char S74468[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.24 (KHTML, like Gecko) RockMelt/0.9.58.494 Chrome/11.0.696.71 Safari/534.24";
const char S74469[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.71.128 Chrome/14.0.835.202 Safari/535.1";
const char S74470[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/534.16 (KHTML, like Gecko) RockMelt/0.9.50.518 Chrome/10.0.648.205 Safari/534.16";
const char S74471[] = "Mozilla/5.0 (Windows NT 6.0; rv:6.0) Gecko/X Thunderbird/6.0 Lightning/1.0b5";
const char S74472[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5368 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S74473[] = "SAMSUNG-GT-S5368_TD/1.0 Android/2.3.6 Release/12.15.2010 Browser/AppleWebKit533.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S74474[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5368 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74475[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5368 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S74476[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-E739 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74477[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ko-kr; LG-LU5400 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74478[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-E920 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74479[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-E920 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S74480[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; E920 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74481[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; DOEASY E920 B Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74482[] = "Mozilla/5.0 (Linux; Android 4.0.4; ONDA MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S74483[] = "Mozilla/5.0 (Linux; Android 4.0.3; ONDA MID Build/ICS.g12refM1006.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S74484[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN79) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile";
const char S74485[] = "Mozilla/5.0 (Java; U; en-us; nokian79-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile";
const char S74486[] = "Opera/9.80 (Android 4.2; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S74487[] = "Opera/9.80 (Android 4.2.1; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S74488[] = "Opera/9.80 (Android 4.2.2; Linux; Opera Tablet/ADY-X) Presto/2.11.355 Version/12.10";
const char S74489[] = "HTCT528t_TD/1.0 Android/4.0 release/2012 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S74490[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Y3 Build/HCL) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74491[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Connect-3G Build/HCL) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74492[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-P7510 Build/JOP40D)";
const char S74493[] = "FaithlifeStudyBible/2.2.10 (Linux; U; Android 4.2.1; en-US; GT-P7510 Build/cm_p4wifi-userdebug 4.2.1 JOP40D eng..X.X test-keys)";
const char S74494[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7510 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MantanoReader";
const char S74495[] = "com.google.android.youtube/4.3.9(Linux; U; Android 4.2.2; en_US; GT-P7510 Build/JDQ39) gzip";
const char S74496[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7510 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74497[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P7510 Build/JDQ39E)";
const char S74498[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI 1S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S74499[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI 1SC Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S74500[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I927 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S74501[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cloud Z5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74502[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S5282) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S74503[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SHL21 Build/S5170) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74504[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHL21 Build/S5170)";
const char S74505[] = "Mozilla/5.0 (Linux; Android 4.2.2; HUAWEI P6-U06 Build/HuaweiP6-U06) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74506[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Glass 1 Build/XRR82) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74507[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Glass 1 Build/XRR72D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74508[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Glass 1 Build/IMM76L; XE7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74509[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-P3100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S74510[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; GT-P3100 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S74511[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I545 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltevzw";
const char S74512[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I545 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltevzw";
const char S74513[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I545 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltevzw";
const char S74514[] = "Mozilla/5.0 (Linux; Android 4.1.2; Q800 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S74515[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S74516[] = "Mozilla/5.0 (Linux; Android 4.0.3; SHW-M250S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S74517[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S74518[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S74519[] = "Huawei/1.0/G6600/Browser/obigo-browser/q03c";
const char S74520[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;HD7;7.10;7720)";
const char S74521[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;HD7 T9292;7.10;8107)";
const char S74522[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;HD7;7.10;8107)";
const char S74523[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;HD7 T9292;7.10;7740)";
const char S74524[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;HD7 T9292;7.10;7720)";
const char S74525[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;HD7 T9292;7.10;8773)";
const char S74526[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;T9296;7.10;8107)";
const char S74527[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;HD7;7.10;7740)";
const char S74528[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Ubuntu/9.10 Chromium/9.0.592.0 Chrome/9.0.592.0 Safari/534.13";
const char S74529[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaC7-00)/UC Browser8.2.0.132/50/355/UCWEB Mobile";
const char S74530[] = "Mozilla/5.0 (X11; U; Linux armv61; en-US; rv:1.9.1b2pre) Gecko/X Fennec/1.0a1";
const char S74531[] = "Mozilla/5.0 (X11; U; Linux armv7l; en-US; rv:1.9.2.1) Gecko/X Firefox/3.6.1 Fennec/1.0.1";
const char S74532[] = "Mozilla/4.0 (compatible; MSIE 2.0; Windows NT 5.0; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)";
const char S74533[] = "Mozilla/5.0 (compatible; MSIE 2.0; Windows NT 5.0; Trident/3.0)";
const char S74534[] = "Mozilla/5.0 (compatible; MSIE 1.0; Windows NT 5.0; Trident/3.0)";
const char S74535[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74536[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74537[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-gb; ZTE-Skate Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74538[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Skate Pro Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74539[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Skate Acqua Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74540[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Neo V Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74541[] = "Mozilla/5.0 (Linux; Android 4.1.1; M532 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S74542[] = "Mozilla/5.0 (Linux; Android 4.0.3; Marquis_MP977 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S74543[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIANokia N81) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S74544[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIANokia N81 8GB) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S74545[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MPDC88 BT IPS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74546[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST23i Build/11.0.A.5.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S74547[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S74548[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S74549[] = "Mozilla/5.0 (Linux; Android 4.1.2; AS870 4G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74550[] = "Mozilla/5.0 (Linux; U; Android 4.1 ;en-us; ZTE V807 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30;";
const char S74551[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ZTE V807 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74552[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P875 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S74553[] = "Mozilla/5.0 (Linux; Android 4.1.2; SM-T211 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S74554[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S74555[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8020 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S74556[] = "Mozilla/5.0 (Linux; U; Android SenseHERO 2.1; en-gb; HTC Hero Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S74557[] = "Mozilla/5.0 (Linux; U; Android VillainROM-v13; en-gb; HTC Hero Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S74558[] = "Mozilla/5.0 (Linux; U; Android Kimera 2.1; en-gb; HTC Hero Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S74559[] = "Mozilla/5.0 (Linux; U; Android VillainROM9.0.0; en-gb; HTC Hero Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S74560[] = "Mozilla/5.0 (Linux; U; Android Hero_2.19_Hiapk; zh-cn; HTC Hero Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S74561[] = "Mozilla/5.0 (Linux; U; Android Hero_2.16_Hiapk; zh-cn; HTC Hero Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S74562[] = "Dalvik/1.1.0 (Linux; U; Android VillainROM9.0.0; HTC Hero Build/ERE27)";
const char S74563[] = "Mozilla/5.0 (Linux; U; Android Hero_2.19_Hiapk; zh-cn; HTC Hero Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S74564[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) HTC-PPC6850 BM BMC";
const char S74565[] = "HTC_Touch_Pro_T7272-orange/PPC; 480x640; OpVer 31.190.2.734 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S74566[] = "HTC_Touch_Pro_T7272 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S74567[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Touch_Pro/1.90.172.4";
const char S74568[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Touch_Pro/5.11.162.1";
const char S74569[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Touch_Pro/1.90.161.5";
const char S74570[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Swisscom/1.0/HTC_Touch_Pro/1.90.166.6";
const char S74571[] = "HTC_Touch_Pro Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S74572[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Touch_Pro/%ROM_VERSION%";
const char S74573[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Touch_Pro/5.10.161.1";
const char S74574[] = "HTC_Touch_Pro_T7276 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S74575[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SO-03C Build/4.0.D.2.61) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74576[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SO-03C Build/4.0.1.C.1.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74577[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SO-03C Build/4.0.1.C.1.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74578[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SO-03C Build/4.0.1.C.1.31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74579[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; SO-03C Build/4.0.1.C.1.31)";
const char S74580[] = "MOT-MT870_TD/1.0 Android/2.3 (Linux; U; Android 2.3.3; zh-cn) Release/5.31.2011 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74581[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MT870 Build/04.30.51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74582[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MT870 Build/04.30.54) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S74583[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MT870 Build/04.30.59) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S74584[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MT870 Build/04.30.62) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S74585[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MT870 Build/04.30.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74586[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MT870 Build/04.30.59) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S74587[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MT870 Build/04.30.59) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74588[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; HP eStation/1.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S74589[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.218 Safari/535.1";
const char S74590[] = "Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; en-us; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S74591[] = "Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.1; en-us; Xoom Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S74592[] = "Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; en-gb; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S74593[] = "Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; fr-fr; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S74594[] = "Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; es-us; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S74595[] = "Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; en-nz; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S74596[] = "Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; es-es; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S74597[] = "Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; en-ca; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S74598[] = "Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; ar-eg; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S74599[] = "Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; zh-tw; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S74600[] = "Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; zh-cn; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S74601[] = "Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; nl-nl; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S74602[] = "Nokia5235/UC Browser8.0.4.121/50/444/UCWEB";
const char S74603[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8b5) Gecko/X Flock/0.4 Firefox/1.0+";
const char S74604[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S74605[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CT705 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74606[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 70 Titanium Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S74607[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S74608[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S74609[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S74610[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S74611[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT897 Build/7.7.1Q-6_SPR-125_ASA-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S74612[] = "Mac OS X/10.8.4 (12E55); ExchangeWebServices/3.0.1 (158); Mail/6.5 (1508)";
const char S74613[] = "Mac OS X/10.8.4 (12E3067); ExchangeWebServices/3.0.1 (158); Mail/6.5 (1508)";
const char S74614[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; QMobile_A20 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74615[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S74616[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; Desire HD Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S74617[] = "Mozilla/5.0 (Linux; U; Android 1.6; de-de; Garmin-Asus A50 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1; A50-V4.1.29-user-20100608";
const char S74618[] = "Spice M6800 Opera/9.80 (MTK; Nucleus; U; en-US) Presto/2.4.18 Version/10.00";
const char S74619[] = "Spice M6800  Opera/9.80 (MTK; Nucleus; U; en-US) Presto/2.4.18 Version/10.00";
const char S74620[] = "ZTE-T54 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 firmware/T54 T04";
const char S74621[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; KM-S120 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74622[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; KM-S120 Build/FRG83)";
const char S74623[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.9.168 Version/11.50";
const char S74624[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Desire S Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74625[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; DesireS Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74626[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Desire S Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S74627[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Desire S Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74628[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Desire S Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S74629[] = "Mozilla/5.0 (Linux; Android 4.0.4; T1004 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S74630[] = "Mozilla/5.0 (Java; U; en-us; gt-s5233a) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S74631[] = "Mozilla/5.0 (Java; U; fa; gt-s5233a) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile";
const char S74632[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S74633[] = "Mozilla/5.0 (iPad; U; CPU OS 6 like Mac OS X; zh-cn Model:iPad3,3) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7543.48.3 UCBrowser/1.0.0.192/43/355";
const char S74634[] = "Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia3120classic) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S74635[] = "Mozilla/5.0 (Linux; U; Android 4.0.4(6577); en-us; Blaze S180  Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74636[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E15i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S74637[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E15i Build/2.1.1.A.0.6) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.276 Mobile";
const char S74638[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I407 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74639[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT885 Build/6.7.3-135_YIC-8) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S74640[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9103 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S74641[] = "SonyEricssonJ10i2/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj10i2) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S74642[] = "SonyEricssonJ10i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; fa; sonyericssonj10i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S74643[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Excite_352g Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74644[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Excite 352g Build/SKV06) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74645[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LS860 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S74646[] = "Mozilla/5.0 (Linux; Android 4.0.4; Mach Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S74647[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; H7100 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74648[] = "Mozilla/5.0 (Linux; Android 4.1.2; Transformer Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S74649[] = "Mozilla/5.0 (Linux; Android 4.1.2; Transformer TF101 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S74650[] = "Mozilla/5.0 (Linux; Android 4.1.2; EK-GC100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S74651[] = "Mozilla/5.0 (Linux; Android 4.1.1; EK-GC100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S74652[] = "SAMSUNG-SGH-D500E/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S74653[] = "SAMSUNG-SGH-D500/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S74654[] = "PantechP9020/JEUS04262010 NetFront/3.5 OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S74655[] = "PantechP9020/JEUSX NetFront/3.5 OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1 Novarra-Vision/8.0";
const char S74656[] = "Mozilla/5.0 (X11; 78; CentOS; US-en) AppleWebKit/527+ (KHTML, like Gecko) Bolt/0.862 Version/3.0 Safari/523.15";
const char S74657[] = "Version/3.0 Mobile/1A477d Safari/419.3";
const char S74658[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_2; en-US) AppleWebKit/533.2 (KHTML, like Gecko) Chrome/5.0.342.9 Safari/533.2";
const char S74659[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_0; en-US) AppleWebKit/533.2 (KHTML, like Gecko) Chrome/5.0.342.7 Safari/533.2";
const char S74660[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_3; en-US) AppleWebKit/533.2 (KHTML, like Gecko) Chrome/5.0.342.7 Safari/533.2";
const char S74661[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/533.2 (KHTML, like Gecko) Chrome/5.0.342.9 Safari/533.2";
const char S74662[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-US) AppleWebKit/533.2 (KHTML, like Gecko) Chrome/5.0.342.9 Safari/533.2";
const char S74663[] = "Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHV-E150S Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S74664[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.21) Gecko/X SeaMonkey/1.1.15 (Ubuntu-1.1.15+nobinonly-0ubuntu2)";
const char S74665[] = "Mozilla/5.0 (Linux; Android 4.0.4; A700 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S74666[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M886 Build/HuaweiM886) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74667[] = "Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia2730c-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S74668[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One XL Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S74669[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One XL Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S74670[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E971 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S74671[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E971 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S74672[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I915 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S74673[] = "[FBAN/FB4A;FBAV/2.3;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/SPH-L710;FBSV/4.1.2;]";
const char S74674[] = "[FBAN/FB4A;FBAV/3.1;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/SPH-L710;FBSV/4.1.2;]";
const char S74675[] = "[FBAN/FB4A;FBAV/3.2.1;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/SPH-L710;FBSV/4.1.2;]";
const char S74676[] = "[FBAN/FB4A;FBAV/3.3;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/SPH-L710;FBSV/4.1.2;]";
const char S74677[] = "[FBAN/FB4A;FBAV/3.4;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/SPH-L710;FBSV/4.1.2;]";
const char S74678[] = "[FBAN/FB4A;FBAV/3.5;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/SPH-L710;FBSV/4.1.2;]";
const char S74679[] = "[FBAN/FB4A;FBAV/3.6.1;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/SPH-L710;FBSV/4.1.2;FBCA/armeabi-v7a:armeabi;]";
const char S74680[] = "Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300K Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S74681[] = "Nokia310/2.0 (07.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia310) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S74682[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A870S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S74683[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A870K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S74684[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A870L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S74685[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP4100 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S74686[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S74687[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570I) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S74688[] = "Mozilla/5.0 (Linux; Android 4.0.3; U8860 Build/HuaweiU8860) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S74689[] = "Mozilla/5.0 (compatible; MSIE or Firefox mutant; not on Windows server; +http://ws.daum.net/aboutWebSearch.html) Daumoa/2.0";
const char S74690[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7";
const char S74691[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NookColor Build/IMM76D; CyanogenMod-CyanogenMod) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74692[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NookColor Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74693[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74694[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.3a3pre) Gecko/X Minefield/3.7a3pre";
const char S74695[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; SAMBM816ND Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74696[] = "NOKIA-Lumia 710/UCWEB2.8.0.249/47/355";
const char S74697[] = "Mozilla/5.0 (compatible; MSIE 4.0; Windows NT 5.1; Trident/5.0)";
const char S74698[] = "Mozilla/5.0 (compatible; MSIE 4.0; Windows NT 5.2; Trident/1.0)";
const char S74699[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; eZee'Tab805 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74700[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; eZee Tab805 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74701[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT920 Build/2_310_2009) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74702[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT920 Build/2_290_2009) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74703[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT920 Build/2_290_2014) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74704[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT920 Build/2_310_2014) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74705[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT920 Build/2_310_2014)";
const char S74706[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT920 Build/2_32D_2009) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74707[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT920 Build/2_32D_2014) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74708[] = "Mozilla/5.0 (Series40; NokiaX2-01/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S74709[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; L3 Build/TECNOL3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74710[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; L3 Build/TECNOL3)";
const char S74711[] = "Nokia7510a/2.0 (06.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S74712[] = "Nokia7510a_CMCC/2.0 (06.65) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S74713[] = "Mozilla/5.0 ( Nokia7510a-b/2.0 (03.90) Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S74714[] = "Nokia7510Supernova/2.0 (06.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S74715[] = "NokiaN97mini/30.0.004 (SymbianOS/9.4; Series60/5.0 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebkit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.28 3gpp-gba";
const char S74716[] = "NokiaN97mini/12.0.110 (SymbianOS/9.4; Series60/5.0 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebkit/525 (KHTML, like Gecko) BrowserNG/7.1.4";
const char S74717[] = "NokiaN97 mini/UCBrowser7.7.0.81/50/800";
const char S74718[] = "NokiaN97 mini";
const char S74719[] = "MOT-EX115 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S74720[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; LG-LU3100 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74721[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-LU3100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74722[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-LU3100 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74723[] = "NokiaX6-00/UCWEB7.9.0.94/50/999";
const char S74724[] = "NokiaX6-00/UC Browser7.9.0.102/50/355/UCWEB";
const char S74725[] = "Opera/9.80 (X11; Linux i686; U; Edition Linux Mint; en) Presto/2.10.229 Version/11.61";
const char S74726[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-MS690 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74727[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-MS690 Build/GRJ90-LG-MS690) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74728[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LG Optimus M Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74729[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-MS690 Build/GRJ90-LG-MS690) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74730[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-MS690 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74731[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID2 Global Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74732[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID2 Global Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74733[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID2 GLOBAL Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74734[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; YL-Coolpad_5832/2.3.011.X.5832; 480*320; CTC/2.0) CoolpadWebkit/533.1";
const char S74735[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 5832 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74736[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 5832 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S74737[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 5832 Build/GWK74) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S74738[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E150S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74739[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xperia S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74740[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.0.3/X (like Mobile/10A525 Safari/8536.25)";
const char S74741[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.0.3/X (like Mobile/10A525 Safari/8536.25)";
const char S74742[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F240L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74743[] = "Mozilla/5.0 (Linux; Android 4.0.4; WIKO-CINK SLIM Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S74744[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5580C Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S74745[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MPDC8 BT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74746[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ615-12) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S74747[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S74748[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I535 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S74749[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T999 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S74750[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTL21 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S74751[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; A500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74752[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; A500 Build/JOP40D)";
const char S74753[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8190 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S74754[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; GT-I8190 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S74755[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8013 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S74756[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire V Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S74757[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire VC Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S74758[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP4100 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S74759[] = "MOT-A1200eam/1.0/R541L7_G_11.00.16R Mozilla/4.0 (compatible; MSIE 6.0; Linux; A1200eam; 781) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00";
const char S74760[] = "MOT-A1200w/1.0/R541L7_G_11.10.11R Mozilla/4.0 (compatible; MSIE 6.0; Linux; A1200w; 781) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00";
const char S74761[] = "MOT-A1200i/R532L4_G_11.40.1AR Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola A1200i;nnn) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [es-la]";
const char S74762[] = "MOT-A1200r/1.0 LinuxOS/2.4.20 Release/8.22.2006 Browser/Opera8.00 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R532_G_11.00.59P";
const char S74763[] = "MOT-A1200eam/1.0/R541L7_G_11.00.13R Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00";
const char S74764[] = "MOT-A1200e/R541_G_11.xx.yyR Mozilla/4.0 (compatible; MSIE 6.0 Linux; A1200e;nnn) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00[yy]";
const char S74765[] = "MOT-A1200/1.0 LinuxOS/2.4.20 Release/6.20.2006 Browser/Opera8.00 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R532_G_11.00.53P";
const char S74766[] = "MOT-A1200e/1.0 LinuxOS/2.4.20 Release/1.31.2007 Browser/Opera8.00 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R541_G_11.56.14R";
const char S74767[] = "MOT-A1200/R532_G_11.00.53P Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola A1200; 1862) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [en]";
const char S74768[] = "MOT-A1200/R532H2_G_11.20.08P Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola A1200; 781) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [en]";
const char S74769[] = "MOT-A1200/R532_G_11.00.44P Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola A1200; 1555) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [en]";
const char S74770[] = "sam-r810 NetFront/3.4 Mozilla/5.0 like Gecko/20060426";
const char S74771[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia2690) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S74772[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; Alldro3Speed DUO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74773[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.393; en-us; 9860/5.1.0.393) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S74774[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.276; en-us; 9860/5.1.0.276) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S74775[] = "UCWEB/2.0(BlackBerry; U; 5.0.0.496; en-us; 9860/5.0.0.496) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S74776[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.546; en-us; 9860/5.1.0.546) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S74777[] = "UCWEB/2.0(BlackBerry; U; 5.0.0.734; en-us; 9860/5.0.0.734) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S74778[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.532; en-us; 9860/5.1.0.532) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S74779[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BLU VIVO 4.65 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74780[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BLU_VIVO 4.65 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74781[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ro-ro; ALLVIEW_P5_Quad Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74782[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) Version/6.0.4 Safari/8536.29.13 Sleipnir/4.2.1";
const char S74783[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) Version/6.0.5 Safari/8536.30.1 Sleipnir/4.2.1";
const char S74784[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LT18i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S74785[] = "Mozilla/5.0 (Linux; Android 4.2.1; V370 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S74786[] = "Mozilla/5.0 (Linux; Android 4.2.2; V370 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S74787[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9300 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S74788[] = "Mozilla/5.0 (Linux; Android 4.1.1; ONE TOUCH 4011X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S74789[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9082 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S74790[] = "SAMSUNG-SGH-D600 /1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S74791[] = "SAMSUNG-SGH-D600E/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S74792[] = "SAMSUNG-SGH-D600/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S74793[] = "SAMSUNG-SGH-T239/T239UVIC6 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S74794[] = "Mozilla/5.0 (Linux; U; Linux 1.6; es-es; AirLife100) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S74795[] = "Mozilla/5.0 (Linux; U; Linux 1.6; en-us; AirLife100) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S74796[] = "Mozilla/5.0 (Linux; U; Linux 1.6; en-gb; AirLife100) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S74797[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S 4G Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74798[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S 4G Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74799[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S 4G Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74800[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer Prime TF201 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74801[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer Prime TF201 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74802[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Transformer Prime TF201 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74803[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6425LVW Build/IML74K; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74804[] = "Mozilla/5.0/Nutch-1.4 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0)";
const char S74805[] = "Mozilla/5.0 (Linux; Android 4.0.4; U8836D Build/HuaweiU8836D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74806[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S74807[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S7500 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S74808[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Sensation Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S74809[] = "Mozilla/5.0 (Linux; Android 4.1.2; Sensation Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S74810[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax_A110) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S74811[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S74812[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S74813[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; HTC Wildfire S A510e Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S74814[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-P925g Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74815[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S74816[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE52-1) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S74817[] = "Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Safari/535.19 Silk-Accelerated=false";
const char S74818[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S74819[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9305 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S74820[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SPH-L710 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S74821[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S74822[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6812 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S74823[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6812B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S74824[] = "Nokia5070b/2.0 (04.20) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S74825[] = "Nokia5070/2.0 (04.22) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S74826[] = "LG-KF750-v10c Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S74827[] = "LG-KF750-Orange/v10c Browser/Obigo-Q05A/3.12MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S74828[] = "LG-KF750-v10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S74829[] = "LG/KF750/v10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S74830[] = "LG-KF750/v10a Browser/Obigo-Q05A/3.11 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S74831[] = "Mozilla/4.0 (Vodafone/1.0/LG-KF750/V10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S74832[] = "Mozilla/4.0 (Vodafone/1.0/LG-KF750/V10b Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S74833[] = "LG-KF750-v10b Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S74834[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari Version/5.1";
const char S74835[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari Version/5.1";
const char S74836[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 Safari Version/5.1";
const char S74837[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190 Safari Version/5.1";
const char S74838[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3";
const char S74839[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 Safari Version/5.1";
const char S74840[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; Fly_IQ235 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74841[] = "Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5310xpressmusic) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S74842[] = "Opera/9.80 (Android 4.1.1; Linux; Opera Tablet/ADY-X; U; ru) Presto/2.9.201 Version/11.50";
const char S74843[] = "Opera/9.80 (Android 4.1.2; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.9.201 Version/11.50";
const char S74844[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Maxthon/4.0.6.2000)";
const char S74845[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-M919 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/jfltetmo";
const char S74846[] = "Mozilla/5.0 (Linux; Android 4.2.2; Touchpad Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S74847[] = "MQQBrowser/43 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S74848[] = "MQQBrowser/43 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3";
const char S74849[] = "MQQBrowser/43 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3";
const char S74850[] = "MQQBrowser/43 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3";
const char S74851[] = "Mozilla/5.0 (Linux; Android 4.2.1; Lenovo S920_ROW Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S74852[] = "NokiaC5-03/UCWEB7.9.0.94/50/999";
const char S74853[] = "NokiaC5-03/UC Browser7.9.0.102/50/355/UCWEB";
const char S74854[] = "Mozilla/4.0 (compatible; MSIE 6.0; Mac_PowerPC Mac OS X; en) Opera 8.02";
const char S74855[] = "Opera/8.51 (Macintosh; PPC Mac OS X; U; de)";
const char S74856[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X; U; en) Opera 8.0";
const char S74857[] = "Opera/8.0 (Macintosh; PPC Mac OS X; U; en)";
const char S74858[] = "Opera/8.5 (Macintosh; PPC Mac OS X; U; en)";
const char S74859[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74860[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74861[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; MB300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74862[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; MB300 Build/GRJ22)";
const char S74863[] = "SuperBot/4.7.0.74 (Windows Vista)";
const char S74864[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.4) Gecko/X http://ow.ly web crawler (.NET CLR 3.5.30729);c2VvLWNyYXdsZXI=";
const char S74865[] = "yacybot (webportal-global; x86 Windows Vista 6.0; java 1.7.0_25; Europe/en) http://yacy.net/bot.html";
const char S74866[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; ZTE-Mimosa X Build/ICS_P943FV1.0.0B06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74867[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; ZTE-Mimosa X Build/ICS_P943FV1.0.0B07) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74868[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-11D Build/V21R36A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74869[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-11D Build/V24R40A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74870[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-11D Build/V26R42B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74871[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S74872[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320T Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S74873[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.11 (KHTML, like Gecko) Ubuntu/10.04 Chromium/23.0.1271.64 Chrome/23.0.1271.64 Safari/537.11";
const char S74874[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Ubuntu/10.04 Chromium/23.0.1271.64 Chrome/23.0.1271.64 Safari/537.11";
const char S74875[] = "Mozilla/5.0 (Linux; Android 4.2.1; One S Build/JOP40G) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74876[] = "Mozilla/5.0 (Linux; Android 4.2.2; One S Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74877[] = "Mozilla/5.0 (Linux; Android 4.2.1; HTC One S Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74878[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One S Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74879[] = "Mozilla/5.0 (Linux; Android 4.2.1; EVO Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74880[] = "Mozilla/5.0 (Linux; Android 4.2.2; EVO Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S74881[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Next7C12F Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74882[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; VS920 4G Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74883[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; VS920 4G Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74884[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:21.0) Gecko/X Thunderbird/21.0";
const char S74885[] = "Mozilla/5.0 (compatible; MSIE 1.0; Win3.11; Trident/1.0)";
const char S74886[] = "Mozilla/5.0 (compatible; MSIE 2.0; Win3.11; Trident/2.0)";
const char S74887[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT25i Build/4.1.B.0.631) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S74888[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S74889[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S74890[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S74891[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S74892[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.0.200) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S74893[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S74894[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S74895[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; Desire HD Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S74896[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Desire HD Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S74897[] = "HTC Streaming Player htc_wwe / 1.0 / htc_ace / 2.2.1";
const char S74898[] = "HTC Streaming Player twm_tw / 1.0 / htc_ace / 2.2.1";
const char S74899[] = "HTC Streaming Player htc_asia_wwe / 1.0 / htc_ace / 2.2";
const char S74900[] = "HTC Streaming Player htc_wwe / 1.0 / htc_ace / 2.2";
const char S74901[] = "HTC Streaming Player htc_asia_wwe / 1.0 / htc_ace / 2.3.3";
const char S74902[] = "HTC Streaming Player cingular_us / 1.0 / htc_ace / 2.3.3";
const char S74903[] = "HTC Streaming Player h3g_uk / 1.0 / htc_ace / 2.2";
const char S74904[] = "HTC Streaming Player tmo_de / 1.0 / htc_ace / 2.2.1";
const char S74905[] = "HTC Streaming Player cingular_us / 1.0 / htc_ace / 2.2.1";
const char S74906[] = "HTC Streaming Player htc_wwe / 1.0 / htc_ace / 2.3.3";
const char S74907[] = "HTC Streaming Player htc_tur / 1.0 / htc_ace / 2.3.3";
const char S74908[] = "HTC Streaming Player hutch_aus / 1.0 / htc_ace / 2.3.3";
const char S74909[] = "HTC Streaming Player orange_uk / 1.0 / htc_ace / 2.3.3";
const char S74910[] = "HTC Streaming Player vodafone_au / 1.0 / htc_ace / 2.3.3";
const char S74911[] = "HTC Streaming Player vodafone_uk / 1.0 / htc_ace / 2.3.5";
const char S74912[] = "HTC Streaming Player vodafone_au / 1.0 / htc_ace / 2.3.5";
const char S74913[] = "HTC Streaming Player htc_wwe / 1.0 / htc_ace / 2.3.5";
const char S74914[] = "HTC Streaming Player cingular_us / 1.0 / htc_ace / 2.3.5";
const char S74915[] = "HTC Streaming Player telus_wwe / 1.0 / htc_ace / 2.3.5";
const char S74916[] = "SonyEricssonV630iv/R1JC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S74917[] = "NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-01/UC Browser8.0.3.107/69/444";
const char S74918[] = "Mozilla/5.0 (compatible; Konqueror/4.1; OpenBSD) KHTML/4.1.4 (like Gecko)";
const char S74919[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Fluid/0.9.4 Safari/525.13";
const char S74920[] = "Akregator/4.9.1; syndication";
const char S74921[] = "Akregator/4.9.2; syndication";
const char S74922[] = "Akregator/4.9.3; syndication";
const char S74923[] = "Akregator/4.9.4; syndication";
const char S74924[] = "Akregator/4.9.5; syndication";
const char S74925[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S74926[] = "Nokia110/2.0 (03.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia110) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S74927[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN86 8MP) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S74928[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0";
const char S74929[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; 11a h700tecno) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S74930[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; t23) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S74931[] = "UCWEB/2.0 (Java; U; MIDP-2.0; es-LA; mozilla) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S74932[] = "UCWEB/2.0 (Java; U; MIDP-2.0; id; generic) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S74933[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US;  mozilla) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S74934[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; s8000) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S74935[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; t50) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S74936[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; beetel gd777) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S74937[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sjt_e861a) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S74938[] = "UCWEB/2.0 (Java; U; MIDP-2.0; fr-FR; mozilla) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S74939[] = "UCWEB/2.0 (Java; U; MIDP-2.0; fr-FR; generic) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S74940[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; bird) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S74941[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; zen-a60) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S74942[] = "UCWEB/2.0 (Java; U; MIDP-2.0; vi; generic) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S74943[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; inq) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S74944[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mstar) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0";
const char S74945[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile, ,";
const char S74946[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; OV-SteelCore10+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74947[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; OV-STEELCORE 10+ (Z) Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74948[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC-X710a Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S74949[] = "Mozilla/5.0 (Linux; Android 4.1.1; Lenovo P770 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S74950[] = "Mozilla/5.0 (Linux; Android 4.1.2; Lenovo P770 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S74951[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6503 Build/10.3.A.0.423) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S74952[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; TRUE BEYOND 3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74953[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; TRUE BEYOND 3G Build/JZO54K)";
const char S74954[] = "Nokia114/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia114) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S74955[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S74956[] = "Mozilla/5.0 (Linux; Android 4.0.3; SHW-M250S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S74957[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S74958[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S74959[] = "Mozilla/5.0 (Linux; Android 4.1.2; ADR930L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S74960[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 UCBrowser/9.2.0.312 Mobile";
const char S74961[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 UCBrowser/9.2.0.312 Mobile";
const char S74962[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 UCBrowser/9.2.0.312 Mobile";
const char S74963[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 UCBrowser/9.2.0.312 Mobile";
const char S74964[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 UCBrowser/9.2.0.312 Mobile";
const char S74965[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 UCBrowser/9.2.0.312 Mobile";
const char S74966[] = "SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj108i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S74967[] = "NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax3-02) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S74968[] = "Mozilla/5.0 (Linux; Android 4.1.1; Z120 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S74969[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF700T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S74970[] = "NSPlayer/9.0.0.4504 WMFSDK/9.0";
const char S74971[] = "NSPlayer/9.0.0.3268 WMFSDK/9.0";
const char S74972[] = "NSPlayer/9.0.0.2980 WMFSDK/9.0";
const char S74973[] = "NSPlayer/9.0.0.3270 WMFSDK/9.0";
const char S74974[] = "NSPlayer/9.0.0.3267 WMFSDK/9.0";
const char S74975[] = "NSPlayer/9.0.0.3268";
const char S74976[] = "NSPlayer/9.0.0.3265 WMFSDK/9.0";
const char S74977[] = "NSPlayer/9.0.0.4509 WMFSDK/9.0 SE 2.X MetaSr 1.0";
const char S74978[] = "NSPlayer/9.00.00.4504";
const char S74979[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S74980[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S74981[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S74982[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S74983[] = "Mozilla/5.0 (Series40; Nokia300/07.44; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.7";
const char S74984[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; EasyPad 970 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74985[] = "Opera/9.80 (Android 1.6; Linux; Opera Mobi/ADR-X; U; en) Presto/2.10.254 Version/12.00";
const char S74986[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; N-08D Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74987[] = "Nokia300/5.0 (07.03) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia300) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile";
const char S74988[] = "Mozilla/5.0 (Linux; Android 4.0.4; TD-1010 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S74989[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74990[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ThL W6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S74991[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST23i Build/11.0.A.5.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S74992[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; TR730 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S74993[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E-51 Build/HuaweiU8666E-51) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S74994[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E Build/HuaweiU8666E) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S74995[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, -0500, us";
const char S74996[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One X+ Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S74997[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7102 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S74998[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7102 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S74999[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5250) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S75000[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 97B TITANIUM Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S75001[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6503 Build/10.3.A.0.423) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S75002[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6503 Build/10.3.A.0.423) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S75003[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Mobile Safari/535.19 Silk-Accelerated=false";
const char S75004[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8013 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S75005[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC One Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S75006[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75007[] = "Mozilla/5.0 (Linux; Android 4.1.2; A511 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S75008[] = "SonyEricssonZ550i/R6CA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S75009[] = "ZTE-G-X991-Rio-orange/X991_V1_Z2_FRES_D18F108 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C";
const char S75010[] = "ZTE-G-X991-Rio-orange/X991_V1_Z2_FRES_D18F109 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C";
const char S75011[] = "Mozilla/5.0 (PlayStation Vita 1.61) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2";
const char S75012[] = "Mozilla/5.0 (Linux; Android 4.0.4;  Nexus S 4G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S75013[] = "Mozilla/5.0 (Linux; Android 4.0.4; Nexus S 4G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75014[] = "Mozilla/5.0 (Linux; Android 4.0.3;  Nexus S 4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S75015[] = "Mozilla/5.0 (Linux; Android 4.0.3; Nexus S 4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75016[] = "Mozilla/5.0 (X11; Linux i686; rv:12.0) Gecko/X Thunderbird/12.0.1";
const char S75017[] = "Opera/9.80 (compatible; U) Presto/2.7.39 Version/11.00";
const char S75018[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5235)/UC Browser8.5.0.183/50/355/UCWEB Mobile";
const char S75019[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V788D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75020[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; V788D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75021[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V788D Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S75022[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Titan TV S320 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75023[] = "libhttp/1.80 (PS Vita)";
const char S75024[] = "libhttp/2.06 (PS Vita)";
const char S75025[] = "libhttp/2.10 (PS Vita)";
const char S75026[] = "libhttp/2.11 (PS Vita)";
const char S75027[] = "libhttp/2.12 (PS Vita)";
const char S75028[] = "libhttp/2.00 (PS Vita)";
const char S75029[] = "libhttp/2.60 (PS Vita)";
const char S75030[] = "libhttp/2.61 (PS Vita)";
const char S75031[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT920 Build/2_290_2009) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S75032[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT920 Build/2_32D_2009) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S75033[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7108 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31";
const char S75034[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7108 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S75035[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i2) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S75036[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S75037[] = "Mozilla/5.0 (Linux; Android 4.0.3; Transformer Prime TF201 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S75038[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75039[] = "Mozilla/5.0 (Linux; Android 4.2.1; N9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75040[] = "Mozilla/5.0 (Linux; Android 4.2.1; N986 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75041[] = "Mozilla/5.0 (Linux; Android 4.1.1; MT7001 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S75042[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; netphone 701 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75043[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Netphone 701 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75044[] = "SAMSUNG-GT-B7300C_CMCC/1.0 Release/01.24.2009 Browser/IE6 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S75045[] = "SAMSUNG-GT-B7300Orange/BVIJ1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S75046[] = "SAMSUNG-GT-B7300/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S75047[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.60 Safari/534.24";
const char S75048[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.0.2.0 Chrome/16.0.912.59 Safari/535.7";
const char S75049[] = "NokiaC2-00/2.0 (03.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac2-00) UCBrowser8.2.1.144/70/352/UCWEB Mobile";
const char S75050[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1.1) Gecko/X Linux Mint/7 (Gloria) Firefox/3.5.1";
const char S75051[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MID84C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75052[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHW-M340L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75053[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S75054[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; M-MP722I Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75055[] = "Opera/9.80 (Android 3.2; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S75056[] = "Opera/9.80 (Android 3.2.2; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S75057[] = "Opera/9.80 (Android 3.2.1; Linux; Opera Tablet/ADY-X) Presto/2.11.355 Version/12.10";
const char S75058[] = "Opera/9.80 (Android 3.2; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10,gzip(gfe)";
const char S75059[] = "Opera/9.80 (Windows NT 6.2; U; en) Presto/2.5.24 Version/10.53";
const char S75060[] = "Opera/9.80 (Windows NT 6.2; U; en) Presto/2.5.22 Version/10.51";
const char S75061[] = "Opera/9.80 (Windows NT 6.2; U; en) Presto/2.5.24 Version/10.54";
const char S75062[] = "Opera/9.80 (Windows NT 6.2; U; en) Presto/2.5.22 Version/10.50";
const char S75063[] = "Opera/9.80 (Windows NT 6.2; U; en) Presto/2.5.24 Version/10.52";
const char S75064[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; BC9710AM Build/IMM76D) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S75065[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; BC9710AM  Build/IMM76D) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S75066[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; BC9710AM Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75067[] = "Mozilla/ (Android; Mobile;) Gecko/ Firefox/";
const char S75068[] = "Mozilla/5.0 (Linux; Android 4.0.4; GSmart GS202+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75069[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; Movistar Motion Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75070[] = "Mozilla/5.0 (Linux; Android 4.1.1; ME172V Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S75071[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; TAC-90012 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75072[] = "Mozilla/5.001 (windows; U; NT4.0; en-US; rv:1.0) Gecko/X - crawler@dhoondho.com";
const char S75073[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PLT7035-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75074[] = "Mozilla/5.0 (Linux; Android 4.2.1; Micromax A116 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75075[] = "Mozilla/5.0 (Linux; Android 4.2.1; A116 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75076[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (xokYZOXYGKSLMXnZxQkcjAxr4E3a8+0X3xBe/nw/XKA=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75077[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (iGtY0hJmov5Fk7HDajUIFHSFd99uX/+KKi5qRkOtGC0=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75078[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (X6aa0cxTmTyzcR7tJg3bMFMUGU3xsdq29+eu6XdAkHE=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75079[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (k2d90rHgrieCxWZ7j7LEwgJtAttkMnZE2P+0KY+tAnI=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75080[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (SfAMt3UVsKT+8oRuuNpdWI4PkbnvCpKxcacjaYU/lm0=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75081[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (TC0JkyfLsQ88z/VhLsmfJsVs8ox+9y/8X9lg+iiM47E=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75082[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (guKh6oaFgD+xMc5oGFJI1fq3vzOdjGaqg6BZG0redCA=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75083[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (dfIqXopg5WI32DlAoLIVAGgDN2xkKJEYxAkRgUGHN9Y=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75084[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Ls7mTa8UpuMpngc0acQipSbiVKIFJVQxlPjHo2joYE0=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75085[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (0bMPZuK/pZ/MGsoXWZnpxDZS+/EYc5Dxlku2joAQv8U=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75086[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Mqv5sOlSrVW+dUzqWqRlD6AARXq7ovO3vtJH/+u/Rvc=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75087[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (08NM6fWO0spYbbrISyT640jdADTaDrcwt7LJUmRJA84=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75088[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ETTxHyN01okH+hHakfiU9rtc9S5CI4VpReEtqpkuang=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75089[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (GnkUPP2lBJm/9lZYoP7uGAYRR0ms99IG/Q5qufwgfOQ=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75090[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (HJGPZi6aUMHsUq4HW5Pl1O87AJ1GCmyvBmmNLkeH6Eo=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75091[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (62Xq1n+WrUMgx2SQALk+4WgRJH1O4FXJTIniPlv6xpU=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75092[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (6tk33y5hhueCSio09/ceEZwcEtv31GHeKbrx8GSD8sM=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75093[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (9McWXSTeEmMMfA6OsMV9kep3k6cuVZuHY6NWIDd8I2c=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75094[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (BiJcJ/NnWWNRCJbpE1IDAUARApQI13bBpXjUKIJ3NX0=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75095[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (A0EOyAcA49CvJQmybQ9R7LGJppl1vSQUAMNKtbYbklA=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75096[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 () Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75097[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (g6nvFfchIynZi8ANz6XPf+CbzWUMVLJqJcEp+hN8JyU=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75098[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (th82DnmgCskucwSlw5YfKzAa7euPJMHwiXiw3NS2uio=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75099[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (jYgJA+CQFZBTIh8fIZ8yjQSipb2lRoSHvR9EkMfGUKc=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75100[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (IzWk1OkDfyfiLtNbMheKn0Z/1RN3ZhU2J+0yhWKGggQ=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75101[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (O7daEPPR2YhzQal6PXK9cr2EkanWTzKcqJRvh58/wgI=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75102[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (/pvbdYxWRjdoWSNGyIEiz24A/TxU5NIpGCrI1c+6BO8=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75103[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (/pvbdYxWRjdoWSNGyIEiz24A/TxU5NIpGCrI1c+6BO8=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2,gzip(gfe)";
const char S75104[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (TfYrHRyrSD4Kn2M0+GHEmoJTgX6OLxy4kW1ZuuQLIDM=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75105[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (q2LVBpKdjbTyqvLqnu3cXX5gjjEx1Jq+0EvSmPd3Vpw=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75106[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (XkKA8LspQ7HAPCh3iWpgQ8bGpHzXRROGeMnxx60F5Mc=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75107[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (l7dXj3d5pf5os4ewfPWqDq6O2BKbpVb83l7mJzmt/u4=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75108[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (rt4R8kudpZt4cdRTDJpP4Dml+DiDYOmChp3nTiVQ9us=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75109[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (bTReBGAmyU0p9EZguECS3zebxG/O7ntBasc9XCGRZt4=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75110[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (uze4z7Jwafdyx1Rmk3l3JfpPUIl/6aSpXF7DxANxl8I=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75111[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (7256e72oXTNB/RApA4sEF/AKB7dnIX8dCCLChdrqcgs=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75112[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (seO21gpOtp8uTe8EjZClilM5rgzyrlWqV92C7FPS+dA=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75113[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (VKixsQPwnrpBozTTnkCTM3GrVtsL7a+mT9Gyk8jkqcY=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75114[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Y6J0A4AypvhLrYtEDuplyc/ynkWaSb7/uFFTeDesFfw=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75115[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (tmcE6pNlnWk/bsOM7/ghgQFOf6UC9tGzNg3xgrBPB0k=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75116[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (sKBlqb3ROanUFyNonRs/B7i7iApx2iN1aOE8r0n9ULY=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75117[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (oSFMNOWJ7mee1o3PNlu10IDyy3flDA9esJ/3OXAKdeg=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75118[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (wRP81FUXe8n0nEyvvzSY5n8m1S3JRPPA/egWmkaZT40=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2";
const char S75119[] = "Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola MOTOROKR E6; 2647) MOT-MOTOROKR E6/R533_G_11.12.10P Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [vi]";
const char S75120[] = "MOT-MOTOROKRE6/R533_G_11.12.08P Mozilla/4.0 (compatible; MSIE 6.0; Linux; MOTOROKRE6; 781) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [en]";
const char S75121[] = "MOT-MOTOROKRE6/1.0 LinuxOS/2.4.20 Release/8.4.2006 Browser/Opera8.00 Profile/MIDP2.0 Configuration/CLDC1.1 Software/R533_G_11.11.04P";
const char S75122[] = "MOT-MOTOROKRE6e/1.0 LinuxOS/2.4.20 Release/31.3.2007 Browser/Opera8.00 Profile/MIDP2.0 Configuration/CLDC1.1 Software/R535_G_11.81.02P";
const char S75123[] = "MOT-MOTOROKR E6/1.0/R533_G_11.12.06P Mozilla/4.0 (compatible; MSIE 6.0; Linux; MOTOROKR E6; 781) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [";
const char S75124[] = "HTC_Touch_Viva_T2223 Opera/9.50 (Windows NT 5.1; U; en)";
const char S75125[] = "HTC_Touch_T2223 Opera/9.50 (Windows NT 5.1; U; pt)";
const char S75126[] = "HTC_Touch_Viva_T2223 Opera/9.50 (Windows NT 5.1; U; ru)";
const char S75127[] = "HTC_Touch_Viva_T2223 Opera/9.50 (Windows NT 5.1; U; es-ES)";
const char S75128[] = "HTC_Touch_Viva_T2223 Opera/9.50 (Windows NT 5.1; U; it)";
const char S75129[] = "LGE-LG230 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S75130[] = "LG-LG230 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S75131[] = "Nokia5233/UC Browser7.8.0.95/50/351";
const char S75132[] = "AppleCoreMedia/1.0.0.8C148 (iPod; U; CPU OS 4_2_1 like Mac OS X; en_us)";
const char S75133[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.8.1.6) Gecko/X SeaMonkey/1.1.4";
const char S75134[] = "Mozilla/5.0 (Windows NT 5.1; rv:2.0b11) Gecko/X Firefox/4.0b11 SeaMonkey/2.1b2";
const char S75135[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-CA; rv:1.9.3a3pre) Gecko/X SeaMonkey/2.1a1pre";
const char S75136[] = "Firefox/1.0.4";
const char S75137[] = "Mozilla/5.0 (OS/2; U; Warp 4.5; en-US; rv:1.7.12) Gecko/X Firefox/1.0.7";
const char S75138[] = "Mozilla/5.0 (SkyOS; U; Intel x86; en-US; rv:1.7.5) Gecko/X Firefox/1.0";
const char S75139[] = "NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac2-01) UCBrowser8.2.1.144/70/355/UCWEB Mobile";
const char S75140[] = "JUC(Linux;U;Android2.3.6;Zh_cn;GT-S5300;240*320;)UCWEB7.8.0.95/139/444";
const char S75141[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-mobile i-style 3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75142[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-mobile i-style 3i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75143[] = "Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A21 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75144[] = "Mozilla/5.0 (Linux; Android 4.0.4; A21 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75145[] = "JUC (Linux; U; 2.3.5; zh-cn; Micromax_A57; 320*480) UCWEB7.9.0.94/139/444";
const char S75146[] = "SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonj105i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S75147[] = "Mozilla/5.0 (Linux; Android 4.1.2; AMID-9743G Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S75148[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.3 Mobile/9A405 Safari/7534.48.3";
const char S75149[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.3 Mobile/9A334 Safari/7534.48.3";
const char S75150[] = "Mozilla/5.0 (Linux; Android 4.0.3; A501 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S75151[] = "Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S75152[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030D Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75153[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300";
const char S75154[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SAMSUNG-GT-I8268_TD/1.0 Android/4.1.2 Release/11.15.2012 Browser/AppleWebKit534.30 Build/JRO03L) ApplelWebkit/534.30 (KHTML,like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75155[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8268 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75156[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7108 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S75157[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7108 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S75158[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5 UCBrowser/9.1.0.297";
const char S75159[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S75160[] = "Mozilla/5.0 (Linux; Android 4.1.2; AS870 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75161[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I337 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jflteatt";
const char S75162[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I337 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jflteatt";
const char S75163[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I337 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jflteatt";
const char S75164[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970C Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S75165[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S75166[] = "NokiaN80-1/3.0 (4.0632.0.10) Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S75167[] = "NokiaN80/1.0.0 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S75168[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; Android Dev Phone 1 Build/PLAT-RC33) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S75169[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; Android Dev Phone 1 Build/CRB21) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S75170[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; Android Dev Phone 1 Build/CRB43) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S75171[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; Android Dev Phone 1 Build/CRC1) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S75172[] = "Mozilla/5.0 (X11; U; CrOS i686 0.9.130; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.344 Safari/534.10";
const char S75173[] = "Mozilla/5.0 (X11; U; CrOS i686 0.9.128; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.343 Safari/534.10";
const char S75174[] = "stagefright/1.1 (Linux;Android 3.0.1)";
const char S75175[] = "Mozilla/5.0 (Linux; U; Android 3.0UI; en-us; MID Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75176[] = "Mozilla/5.0 (Linux; U; Android 3.0UI; en-us; MID serials 3.0ui honeycomb Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75177[] = "Mozilla/5.0 (Linux; U; Android 3.0UI; en-in; iBall 3G 7316 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75178[] = "Mozilla/5.0 (Linux; U; Android 3.0; en-us) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S75179[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) iCab/4.9 Safari/533.16";
const char S75180[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Maxwell Plus Build/1.0.3 X-14:07) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S75181[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6358 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S75182[] = "SAMSUNG-GT-S6358_TD/1.0 Android/2.3.6 Release/1.15.2012 Browser/AppleWebKit533.1";
const char S75183[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6358 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75184[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U795 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S75185[] = "Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U795 Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S75186[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ZTE U795 Build/IMM76D)";
const char S75187[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U795+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75188[] = "Nokia306/2.0 (03.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia306) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S75189[] = "Mozilla/5.0 (Linux; Android 4.1.1; ST8 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S75190[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; iDxD10 3G Build/ICS.g02ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75191[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; iDxD10 3G Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75192[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; iDxD10 3G Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75193[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAN86 8MP) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S75194[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S75195[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S75196[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S75197[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ609 Build/7.7.1-85_MZ609-31) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S75198[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; Tab1020 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75199[] = "Nokia310/2.0 (07.35) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S75200[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Fly F40 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75201[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035X Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75202[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035E Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75203[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035D Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75204[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E615 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75205[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E615f Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75206[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75207[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75208[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75209[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus S 4G Build/JRO03R) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S75210[] = "SAMSUNG-SGH-A167/A167UCIB3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S75211[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0)";
const char S75212[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0) AdMob;Octagon";
const char S75213[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; 909)";
const char S75214[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; HTCPO881; SPRINT)";
const char S75215[] = "Opera/9.80 (Android 2.3.3; Linux; Opera Tablet/ADR-X; U; de) Presto/2.9.201 Version/11.50";
const char S75216[] = "Opera/9.80 (Android 2.3.2; Linux; Opera Tablet/ADR-X; U; en) Presto/2.9.201 Version/11.50";
const char S75217[] = "Opera/9.80 (Android 2.3.4; Linux; Opera Tablet/ADR-X; U; de) Presto/2.9.201 Version/11.50";
const char S75218[] = "Opera/9.80 (Android 2.3.1; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.9.201 Version/11.50";
const char S75219[] = "Opera/9.80 (Android 2.3; Linux; Opera Tablet/ADR-X; U; en) Presto/2.9.201 Version/11.50";
const char S75220[] = "Opera/9.80 (Android 2.3.6; Linux; Opera Tablet/ADR-X; U; en-GB) Presto/2.9.201 Version/11.50";
const char S75221[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.3.5 Safari/534.34";
const char S75222[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.3.5 Safari/534.34";
const char S75223[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7501 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S75224[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaC5-06) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S75225[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Wildfire_S_A510e) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S75226[] = "UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; HTC_Wildfire_S_A510e) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S75227[] = "SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj105i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S75228[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; SH837W Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75229[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SH837W Build/JZO54K)";
const char S75230[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I829 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75231[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I829 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S75232[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/23.0.1271.95 Chrome/23.0.1271.95 Safari/537.6+ Ubuntu/13.04 (3.6.1-2ubuntu1) Epiphany/3.6.1";
const char S75233[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/23.0.1271.95 Chrome/23.0.1271.95 Safari/537.6+ Ubuntu/13.04 (3.6.1-2ubuntu1) Epiphany/3.6.1";
const char S75234[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I757M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S75235[] = "Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Safari/535.19 Silk-Accelerated=false";
const char S75236[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT916 Build/2_26D_2014) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75237[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6810 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S75238[] = "MOT-L6/0A.52.45R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S75239[] = "MOT-L6i/0A.65.07R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S75240[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Nokia6121c/5.11; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S75241[] = "Opera/9.10 (Windows NT 6.1; U; en)";
const char S75242[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s5263) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S75243[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5263) UCBrowser8.2.0.132/69/352/UCWEB Mobile";
const char S75244[] = "Lenovo S880/S100 Linux/3.0.13 Android/4.0.3 Release/12.12.2011 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30";
const char S75245[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; S153G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75246[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; IQ255 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75247[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; SHW-M110S Build/JELLYBEAN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75248[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ZTE N880E Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75249[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE N880E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75250[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ZTE N880E Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S75251[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE N880E Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S75252[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N880E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75253[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS GAMEPAD Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S75254[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ST8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75255[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; ST8 Build/JRO03C)";
const char S75256[] = "Dolphin http client/9.3.1(200) (Android)";
const char S75257[] = "Dolphin http client/9.3.2(203) (Android)";
const char S75258[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ607-14) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S75259[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717M Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S75260[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; eZee'TAB707 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75261[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; miTab FUNK Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75262[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC One X Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S75263[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5302) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S75264[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P713 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75265[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P713GO Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75266[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.9.1 Safari/534.34";
const char S75267[] = "Mozilla/5.0 (Series40; NokiaC2-05/08.69; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S75268[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 640x480; HTC_Advantage/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1) Opera 8.65 [en]";
const char S75269[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x240) Opera 8.65 [en]";
const char S75270[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320) Opera 8.65 [nl]";
const char S75271[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 320x320) Opera 8.65 [en]";
const char S75272[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x240) Opera 8.60 [en]";
const char S75273[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320) Opera 8.65 [zh-cn]";
const char S75274[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 480x640) Opera 8.60 [en]";
const char S75275[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Sprint:PPC-6700) Opera 8.65 [en]";
const char S75276[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 320x320)Opera 8.65 [en]";
const char S75277[] = "Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; Android Edition Starnaute Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75278[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; Android Edition Starnaute Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75279[] = "IUC(U;iOS 4.3.5;Zh-cn;320*480;)/UCWEB8.1.0.104/41/997";
const char S75280[] = "IUC(U;iOS 5.0.1;Zh-cn;320*480;)/UCWEB8.1.4.125/42/997";
const char S75281[] = "IUC(U;iOS 4.1;Zh-cn;320*480;)/UCWEB8.1.2.111/41/800";
const char S75282[] = "IUC(U;iOS 4.2.1;Zh-cn;320*480;)/UCWEB8.1.2.111/41/997";
const char S75283[] = "IUC(U;iOS 5.1.1;Zh-cn;320*480;)/UCWEB8.1.2.111/42/997";
const char S75284[] = "BlackBerry9780/6.5.0.54 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/159/352";
const char S75285[] = "BlackBerry9780/6.6.0.207 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/159/444";
const char S75286[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; maxamine.com-robot) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.127 Safari/533.4";
const char S75287[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19";
const char S75288[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19";
const char S75289[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; Hd1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75290[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; Hd1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75291[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/84/352";
const char S75292[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/69/352";
const char S75293[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/70/352";
const char S75294[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/69/444";
const char S75295[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/84/444";
const char S75296[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/69/405";
const char S75297[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/69/352  3gpp-gba";
const char S75298[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/70/444";
const char S75299[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/70/405";
const char S75300[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/69/444  3gpp-gba";
const char S75301[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/84/352  3gpp-gba";
const char S75302[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaC7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile";
const char S75303[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; myTouch_4G_Slide Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75304[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MyTouch 4G Slide Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75305[] = "Mozilla/5.0 (Linux; Android 4.0.4; Arc Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S75306[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT15a Build/4.1.B.0.587) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S75307[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5120 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S75308[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N7000 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S75309[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT890 Build/9.8.2I-50_SMI-26) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S75310[] = "Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabS2109A-F Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S75311[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia603) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S75312[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S75313[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ609 Build/7.7.1-85_MZ609-31) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S75314[] = "MOT-RAZRV3XXR_J/97.04.30R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003290) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.60 [en]";
const char S75315[] = "MOT-RAZRV3XXR_J/97.04.30R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; X) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [en]";
const char S75316[] = "SAMSUNG-GT-S8500T/1.0 SHP/VPP/R5 Dolfin/2.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 S8500DVJF3";
const char S75317[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; Eris Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S75318[] = "Mozilla/5.0 (Linux; like iPhone; U; en-US) AppleWebKit/528.5+ (KHTML, like Gecko, Safari/528.5+) Version/4.0 Kindle/3.0 (screen 600x800; rotate)";
const char S75319[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Twitter for iPhone";
const char S75320[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7D11 Twitter for iPhone";
const char S75321[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7C145 Twitter for iPhone";
const char S75322[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S75323[] = "Mozilla/5.0 (compatible; Konqueror/3.5; Darwin) KHTML/3.5.6 (like Gecko)";
const char S75324[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; S8600 Build/HuaweiS8600) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S75325[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HW-HUAWEI_S8600/S8600V100R001C92B119; 480*800;CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S75326[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; S8600 Build/HuaweiS8600) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S75327[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; S8600 Build/HuaweiS8600) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75328[] = "Safari/6533.21.1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBook5%2C1)";
const char S75329[] = "Safari/6533.21.1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (Macmini3%2C1)";
const char S75330[] = "Safari/6533.21.1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro4%2C1)";
const char S75331[] = "Safari/6533.21.1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookAir3%2C1)";
const char S75332[] = "Safari/6533.21.1 CFNetwork/454.12.4 Darwin/10.8.0 (x86_64) (iMac12%2C1)";
const char S75333[] = "Safari/6533.21.1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacPro3%2C1)";
const char S75334[] = "Flipboard/1.8.4 beta 1/65,1.8.4,2012-05-30 18:15, us, Android";
const char S75335[] = "Flipboard/1.8.4 beta 1/65,1.8.4,2012-05-30 18:15, Android";
const char S75336[] = "Flipboard/1.8.4 beta 1/65,1.8.4,2012-05-30 18:15, gb, Android";
const char S75337[] = "Flipboard/1.8.4/65,1.8.4,2012-05-30 18:17, gb, Android";
const char S75338[] = "Flipboard/1.8.4 beta 3/86,1.8.4.86,2012-06-14 14:31, gb, Android";
const char S75339[] = "Flipboard/1.8.4/65,1.8.4,2012-05-30 18:17, Android";
const char S75340[] = "Flipboard/1.8.4/73,1.8.4.73,2012-06-06 12:32, us, Android";
const char S75341[] = "Flipboard/1.8.4/73,1.8.4.73,2012-06-06 12:32, Android";
const char S75342[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Qmobile-S11 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75343[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; TURKCELL MaxiPRO5 Build/HuaweiU8860) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75344[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s3850) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S75345[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-N7100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S75346[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-N7100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S75347[] = "Mozilla/5.0 (Linux; Android 4.2.1; SPH-D710 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75348[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-D710 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75349[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3653) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S75350[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; F8 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75351[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; FPT F8 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75352[] = "Mozilla/5.0 (Linux; Android 4.1.1; GEM7020 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S75353[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7100) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S75354[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; AN10CG3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75355[] = "Mozilla/5.0 (Linux; U; Android 4.1.4; zh-tw; HD88+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75356[] = "Mozilla/5.0 (Linux; U; Android 4.1.4; en-us; HD88+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75357[] = "Mozilla/5.0 (Linux; U; Android 4.1.4; en-gb; HD88+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75358[] = "Mozilla/5.0 (Linux; U; Android 4.1.4; zh-cn; HD88+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75359[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire V Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S75360[] = "Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabS2109A-F Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S75361[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac7-00) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba";
const char S75362[] = "NokiaC7-00/UCWEB8.9.0.253/51/999";
const char S75363[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S75364[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S75365[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9108 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S75366[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9100G Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S75367[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.1.258 U3/0.8.0 Mobile Safari/534.31";
const char S75368[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Rockmelt/3.0.3.7 Mobile/10A525 Safari/8536.25";
const char S75369[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6313T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75370[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-S738C Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S75371[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire V Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S75372[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire VC Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S75373[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; C5302) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S75374[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; A27) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S75375[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; IRIS_455) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S75376[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; C2104) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S75377[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Nexus_S) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S75378[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; LG-E455) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S75379[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Connect-2G-2.0) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S75380[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Symphony_Xplorer_W35) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S75381[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; XOLO_A1000) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S75382[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT885 Build/6.7.3-135_YIC-8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75383[] = "Mozilla/5.0 (Linux; Android 4.1.1; A211 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S75384[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.4.27) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S75385[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.0.16) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S75386[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.6.8) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S75387[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.4.22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S75388[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.1.8) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S75389[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.6.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S75390[] = "Opera 7.0 SonyEricssonW950i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 513) Opera 8.65 [en]";
const char S75391[] = "SonyEricssonW950i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 323) Opera 8.60 [en]";
const char S75392[] = "SonyEricssonW950i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 598) Opera 8.65 [en]";
const char S75393[] = "Opera 8.54 SonyEricssonW950i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 513) Opera 8.65 [en]";
const char S75394[] = "Mozilla/5.0 Vodafone/1.0/LG-GM750/V10b Browser/IE8.12 MMS/LGMMSv1.0/1.2 Java/LGVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S75395[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) T-Mobile_Juno";
const char S75396[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) Vodafone 1520";
const char S75397[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) Sprint:MotoQ9c";
const char S75398[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) Alltel:MotoQ9c";
const char S75399[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Milestone Build/R.U.R.1920) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75400[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; Milestone Build/R.U.R.1920) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75401[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; Milestone Build/R.U.R.1920)";
const char S75402[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; Milestone Build/R.U.R.1920) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75403[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; Milestone Build/SHOLS_U2_05.26.3)";
const char S75404[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; es-es; Milestone Build/R.U.R.1920) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75405[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Milestone Build/R.U.R.1920) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75406[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Milestone Build/SHOLS_U2_05.26.3) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S75407[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/5.2.1.1588 Safari/532.4";
const char S75408[] = "Opera/9.80 (Windows NT 5.0; U; Edition United States Local; en) Presto/2.9.168 Version/11.50";
const char S75409[] = "Mozilla/5.0 (Nokia; Qt; Symbian)";
const char S75410[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.4; en-US; rv:1.9) Gecko/X Firefox/3.0 Flock/2.0b1";
const char S75411[] = "Mozilla/5.0 (X11; FreeBSD amd64; rv:13.0) Gecko/X Firefox/13.0.1";
const char S75412[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E617G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75413[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A100 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75414[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A100 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75415[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A100 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75416[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A100 Build/JZO54K; CyanogenMod-Android 4.1.2 CM10 build) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75417[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; TAB 804 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75418[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5620) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S75419[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; GT-P3105 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75420[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P3105 Build/IMM76D)";
const char S75421[] = "Nokia206/2.0 (03.59) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia206) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S75422[] = "Nokia206.1/2.0 (03.58) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia206.1) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S75423[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; LG-E400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S75424[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-E400 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S75425[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iris454 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75426[] = "Mozilla/5.0 (Linux; Android 4.1.1; Arc Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S75427[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax P360 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75428[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S75429[] = "Micromax X321/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S75430[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-CN; LT18i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S75431[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_Sensation_Z710e Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75432[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC_Sensation_4G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75433[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC Sensation Z710e Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75434[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Sensation 4G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75435[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9082 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S75436[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5282 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75437[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75438[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75439[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75440[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A840SP Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75441[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A840S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75442[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S75443[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-hk; HTC Aria A6380 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S75444[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; HTC_Aria_A6380 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S75445[] = "LG-A165/V100 Obigo/Q05A MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S75446[] = "sam-r580 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S75447[] = "DoCoMo/2.0 F01C(c500;TB;W24H16)";
const char S75448[] = "DoCoMo/2.0 F01B(c500;TB;W24H16)";
const char S75449[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5235)/UC Browser8.2.0.132/50/355/UCWEB Mobile";
const char S75450[] = "Mozilla/5.0 (Series40; Nokia6303iclassic/10.80; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S75451[] = "Mozilla/5.0 (Linux; Android 4.1.1; 9300+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75452[] = "SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonj20i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S75453[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:19.0) Gecko/X Thunderbird/19.0";
const char S75454[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S75455[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9210T Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S75456[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S75457[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530M Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S75458[] = "Mozilla/5.0 (Linux; Android 4.2.1; ZP980 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75459[] = "Mozilla/5.0 (Linux; Android 4.2.2; SM-C101 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75460[] = "Mozilla/5.0 (Linux; Android 4.3; GT-N7100 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75461[] = "Mozilla/5.0 (Linux; Android 4.3; GT-N7105 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75462[] = "Mozilla/5.0 (Linux; Android 4.3; SPH-L900 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75463[] = "Mozilla/5.0 (Linux; Android 4.3; SAMSUNG-SGH-I317 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75464[] = "Mozilla/5.0 (Linux; Android 4.3; Galaxy Note II Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75465[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo P700i Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75466[] = "Mozilla/5.0 (Linux; Android 4.0.3; Lenovo P700 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75467[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5210 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S75468[] = "Spice M6900 Opera/9.80 (MTK; Nucleus; U; en-US) Presto/2.4.18 Version/10.00";
const char S75469[] = "Spice M6900  Opera/9.80 (MTK; Nucleus; U; en-US) Presto/2.4.18 Version/10.00";
const char S75470[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT800W Build/TTSKT_U_80.10.24R) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S75471[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT800W Build/TTSKT_U_80.10.28R) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S75472[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT800 Build/TITA_M2_16.22.7) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S75473[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT800 Build/TITA_M2_16.22.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S75474[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT800 Build/TITA_M2_16.35.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S75475[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT800 Build/TITA_M2_16.35.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S75476[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT800 Build/TITA_M2_16.30.3) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S75477[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; XT800W Build/TTSKT_U_80.10.28R)";
const char S75478[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT800 Build/TITA_M2_16.30.3) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S75479[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT800 Build/TITA_M2_20.08.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S75480[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT800 Build/TITA_M2_16.35.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S75481[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Iron/9.0.600.2 Chrome/9.0.600.2 Safari/534.13";
const char S75482[] = "Browser/Opera  Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10";
const char S75483[] = "Mozilla/5.0 (X11; U; Linux i686; en-GB; rv:1.7.13) Gecko/X Fedora/1.0.8-1.1.fc4 Firefox/1.0.8";
const char S75484[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT-AS43D2 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75485[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT-AS43D2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75486[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3 Sleipnir/2.1.1m";
const char S75487[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3 Sleipnir/2.1.2m";
const char S75488[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 Sleipnir/2.1.3m";
const char S75489[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3 Sleipnir/2.1.3m";
const char S75490[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 Sleipnir/2.1.4m";
const char S75491[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 Sleipnir/2.1.5m";
const char S75492[] = "Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6303classic) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S75493[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAB-1030 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75494[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TAB-1030 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75495[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I577 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S75496[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7100 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S75497[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P7100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S75498[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Sweet M971 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75499[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Sweet M971 Build/JRO03H)";
const char S75500[] = "Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80 COBALT Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S75501[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PNDPP48GP Build/ICS.PNDPP48GP.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75502[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Micromax_A72) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S75503[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ONE TOUCH TAB 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75504[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ONE TOUCH TAB 7 Build/GSDT011) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75505[] = "UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-914_im_india_269) U2/1.0.0 UCBrowser/3.1.1.343 U2/1.0.0 Mobile";
const char S75506[] = "UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-885_im_india_249) U2/1.0.0 UCBrowser/3.1.1.343 U2/1.0.0 Mobile";
const char S75507[] = "LG-GX300/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S75508[] = "NokiaC6-00";
const char S75509[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A601 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75510[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; A601 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75511[] = "Nokia5530/UC Browser8.0.4.121/50/444/UCWEB";
const char S75512[] = "Nokia5530/UCWEB8.0.3.99/50/999";
const char S75513[] = "Mozilla/5.0 (Series40; NokiaX3-02/05.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S75514[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19 Comodo_Dragon/18.3.0.0";
const char S75515[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19 Comodo_Dragon/18.3.0.0";
const char S75516[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE C1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75517[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE C1 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S75518[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE C1 Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S75519[] = "SAMSUNG-GT-I8250_TD/1.0 Android/2.3.6 Release/11.11.2011 Browser/AppleWebKit533.1";
const char S75520[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8250 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S75521[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8250 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75522[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; MID8127 Build/GreenDemon_v1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75523[] = "JUC (Linux; U; 2.1-update1; zh-cn; U20i; 240*320) UCWEB7.9.0.94/139/444";
const char S75524[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.31270/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S75525[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Etel i3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75526[] = "MQQBrowser/40 (iOS 4S; U; CPU like Mac OS X; zh-cn)";
const char S75527[] = "MQQBrowser/40 (iOS 4; U; CPU like Mac OS X; zh-cn)";
const char S75528[] = "MQQBrowser/40 (iOS 5; U; CPU like Mac OS X; zh-cn)";
const char S75529[] = "Mozilla/5.0 (Linux; Android 4.2.2; EVO Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S75530[] = "ZTE-T96 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1 firmware/T96 T02";
const char S75531[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21i Build/11.0.A.4.22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S75532[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21i2 Build/11.0.A.6.5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S75533[] = "Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_S9714 Build/JRO03R) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S75534[] = "OPPO_R815T/1.0 Linux/3.4.0 Android/4.2.1  Release/12.24.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2 System/Android 4.2.1;";
const char S75535[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; R815T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75536[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; R815T Build/JOP40D)";
const char S75537[] = "Mozilla/5.0 (Linux; Android 3.2; GT-P6200 Build/HTJ85B) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S75538[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S75539[] = "Mozilla/5.0 (Linux; Android 4.0.1; Galaxy Nexus Build/ITL41F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75540[] = "Mozilla/5.0 (Linux; Android 4.0.2; Galaxy Nexus Build/ICL53F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75541[] = "Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75542[] = "Mozilla/5.0 (Series40; NokiaX2-05/08.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S75543[] = "SAMSUNG-GT-C3560/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S75544[] = "LG-KP502-V10b Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S75545[] = "LG-KP502 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S75546[] = "SAMSUNG-SGH-I607/I607FG1 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 320x240)";
const char S75547[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Incredible S Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S75548[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Incredible S Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75549[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Incredible S Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S75550[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Incredible S Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75551[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s5620) UCBrowser8.2.0.132/69/452/UCWEB Mobile";
const char S75552[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s5620) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S75553[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SmartTab1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75554[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:15.0) Gecko/X Firefox/15.0.1 SeaMonkey/2.12.1";
const char S75555[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/6.5.28192/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S75556[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MP969 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75557[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TOUCH96 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75558[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus S Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S75559[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus S Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S75560[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xperia U Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75561[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Flame2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75562[] = "Nokia110/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia110) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S75563[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A9+ Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S75564[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A9+ Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S75565[] = "Mozilla/5.0 (Linux; Android 4.0.4; C1605 Build/11.1.A.0.68) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75566[] = "KDDI-SN3S UP.Browser/6.2_7.2.7.1.K.6.212 (GUI) MMP/2.0";
const char S75567[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; BLU Magic Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75568[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i917.;7.10;7720)";
const char S75569[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i917;7.10;7720)";
const char S75570[] = "JUC (Linux; U; 2.2; zh-cn; HTC_Desire; 480*800) UCWEB7.9.3.103/139/800";
const char S75571[] = "JUC (Linux; U; 2.2; zh-cn; HTC_Desire; 480*800) UCWEB7.9.3.103/139/999";
const char S75572[] = "JUC (Linux; U; 2.3.7; zh-cn; HTC_Desire; 480*800) UCWEB7.9.3.103/139/800";
const char S75573[] = "JUC (Linux; U; 2.2; zh-cn; HTC_Desire; 480*800) UCWEB7.9.4.145/139/800";
const char S75574[] = "JUC (Linux; U; 2.2; zh-cn; HTC_Desire; 480*800) UCWEB7.9.3.103/139/1002";
const char S75575[] = "JUC (Linux; U; 2.2; zh-cn; HTC_Desire; 480*800) UCWEB7.9.1.100/139/800";
const char S75576[] = "JUC (Linux; U; 2.2; zh-cn; HTC_Desire; 480*800) UCWEB7.9.0.94/139/352";
const char S75577[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC_Desire_V; 480*800) UCWEB7.9.0.94/139/355";
const char S75578[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC_Desire_V; 480*800) UCWEB7.9.0.94/139/444";
const char S75579[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC_Desire_V; 480*800) UCWEB7.9.0.94/139/352";
const char S75580[] = "JUC (Linux; U; 2.2; zh-cn; HTC_Desire; 480*800) UCWEB7.9.0.94/139/444";
const char S75581[] = "Mozilla/5.0 (Linux; U; Windows NT 6.1; en-us; dream) DoggCatcher";
const char S75582[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0;	Trident/4.0; SLCC2;	.NET CLR 2.0.50727; .NET CLR 3.5.30729;	.NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E) QQBrowser/6.0";
const char S75583[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.0";
const char S75584[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0) QQBrowser/6.0";
const char S75585[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-gb; Silk/1.0.13.81_X)Maxthon AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16 Silk-Accelerated=true";
const char S75586[] = "HUAWEI_T8600_TD/1.0 OPhone/2.5 (Linux;Android 2.2) Release/11.01.2011 Browser/WAP2.0(AppleWebKit/533.1) Mobile Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S75587[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HUAWEI T8600 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75588[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HUAWEI T8600 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S75589[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HUAWEI T8600 Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S75590[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5670 Build/GRI40; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75591[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT-S5670 Build/GRI40; CyanogenMod 7.2.0 Stable) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75592[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ETAB 701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75593[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75594[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75595[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75596[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S6T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75597[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S6 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S75598[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T769 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75599[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.0.5) Gecko/X Gentoo Firefox/3.0.5 (Dot TK - spider 4.1)";
const char S75600[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; zh-cn; MI-ONE Plus Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S75601[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S75602[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S75603[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; SCH-N719 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S75604[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-N719 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S75605[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ThL W7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75606[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TM-MID720 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75607[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; TM-MID720 Build/JRO03H)";
const char S75608[] = "Nokia201/2.0 (11.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia201) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S75609[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I815 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S75610[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75611[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75612[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6312 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S75613[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5580C Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S75614[] = "SAMSUNG-SGH-T669B/T669BMJK1 SHP/VPP/R5 Dolfin/1.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S75615[] = "Nokia6260s-1/2.0 (07.20) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S75616[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.8.131 Version/11.11";
const char S75617[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.8.119 Version/11.10";
const char S75618[] = "LG-350 MIDP-2.0/CLDC-1.1 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U; pt-BZ) Presto/2.5.28 Version/10.10";
const char S75619[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-us; EBW404B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S75620[] = "Mozilla/5.0 (Unknown; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) CasperJS/1.0.0-RC2+Phantomjs/1.7.0 Safari/534.34";
const char S75621[] = "Mozilla/5.0 (Linux; U; Android 4.1.4; zh-tw; iTouch-HD88 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75622[] = "Mozilla/5.0 (Linux; U; Android 4.1.4; en-us; iTouch-HD88 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75623[] = "Mozilla/5.0 (Linux; U; Android 4.1.4; en-gb; iTouch-HD88 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75624[] = "Mozilla/5.0 (Linux; U; Android 4.1.4; zh-cn; iTouch-HD88 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75625[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; spice) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S75626[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mozilla) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S75627[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I957 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S75628[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957R Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S75629[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S75630[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Q-Smart model S6 Build/S6_S406_YA860_V01.01B26) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75631[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Q-Smart model S6 Build/S6_S406_YA860_V01.01B30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75632[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Q-Smart model S6 Build/S6_S406_YA860_V01.01B31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75633[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Q-Smart model S6 Build/S6_S406_YA860_V01.01B32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75634[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Q-Smart model S6 Build/S6_S406_YA860_V01.01B28) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75635[] = "Mozilla/5.0 (Linux; Android 4.1.2; Transformer Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S75636[] = "Mozilla/5.0 (Linux; Android 4.1.2; Transformer TF101 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S75637[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC Incredible S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S75638[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22";
const char S75639[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9210T Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S75640[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5282 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S75641[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT18i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S75642[] = "Mozilla/5.0 (Linux; Android 4.0.3; LT18i Build/4.1.A.0.562) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S75643[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; SPH-L710 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S75644[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S75645[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-ca; SGH-I747M Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S75646[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-I9305 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S75647[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S75648[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E455 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75649[] = "Mozilla/5.0 (Linux; Android 4.1.2; ADR930L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75650[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT18i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75651[] = "Mozilla/5.0 (Linux; Android 4.0.3; LT18i Build/4.1.A.0.562) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75652[] = "ZTE-F100(Zimbabwe)/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S75653[] = "ZTE-F100/0.1 ACS-NF/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S75654[] = "ZTE-F100(Jordan Orange)/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S75655[] = "ZTE-F100(DGT)/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S75656[] = "ZTE-F100/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S75657[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.4.0.0 Chrome/13.0.782.220 Safari/535.1";
const char S75658[] = "NokiaC2-03/2.0 (06.51) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-03/UC Browser7.9.0.102/69/352";
const char S75659[] = "Mozilla/5.0 (Java; U; en-US; mauie800) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S75660[] = "Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia6303iclassic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile";
const char S75661[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 Sleipnir/3.8.4";
const char S75662[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Sleipnir/3.8.4)";
const char S75663[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 Sleipnir/3.8.4";
const char S75664[] = "JUC (Linux; U; 2.3.4; zh-cn; MT11i; 480*854) UCWEB7.9.0.94/139/444";
const char S75665[] = "JUC (Linux; U; 4.0.4; zh-cn; MT11i; 480*854) UCWEB7.9.0.94/139/444";
const char S75666[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari/7534.48.3 Sleipnir/2.2m";
const char S75667[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3 Sleipnir/2.2m";
const char S75668[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 Sleipnir/2.2m";
const char S75669[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Safari/7534.48.3 Sleipnir/2.2m";
const char S75670[] = "Mozilla/5.0 (Linux; Android 4.0.3; NS-13T001 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S75671[] = "Mozilla/5.0 (Linux; Android 4.0.3; LIFETAB_P9514 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S75672[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/10A523 Safari/7534.48.3";
const char S75673[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/10A403 Safari/7534.48.3";
const char S75674[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/10A550 Safari/7534.48.3";
const char S75675[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/10A8426 Safari/7534.48.3";
const char S75676[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/10A407 Safari/7534.48.3";
const char S75677[] = "Mozilla/5.0 (Linux; Android 4.0.3; U8860 Build/HuaweiU8860) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S75678[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cnm TOUCHPAD 9.7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S75679[] = "Mozilla/5.0 (Linux; Android 4.0.4; Versus TouchPad 7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S75680[] = "Mozilla/5.0 (Linux; Android 4.0.4; Versus TouchPad 9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S75681[] = "Mozilla/5.0 (Linux; Android 4.0.4; CnM TouchPad 7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S75682[] = "Mozilla/5.0 (Linux; Android 4.0.4; CnM TouchPad 9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S75683[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291";
const char S75684[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.319";
const char S75685[] = "Mozilla/5.0 (Linux; U; Android 4.2.1;en_us; N986 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30;";
const char S75686[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; N986 Build/JOP40D)";
const char S75687[] = "Mozilla/5.0 (Linux; Android 4.0.4; LIFETAB_S9512 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S75688[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT30at Build/7.0.B.1.152) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75689[] = "Mozilla/5.0(Linux;U;Android2.1-update1;Zh_cn;E16i;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/444";
const char S75690[] = "JUC(Linux;U;Android2.1-update1;Zh_cn;E16i;320*480;)UCWEB7.8.0.95/139/444";
const char S75691[] = "SAMSUNG-SGH-i900/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S75692[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; de-de; MD_LIFETAB_P9516 Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S75693[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-W999 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75694[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-W999 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S75695[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S75696[] = "Instagram 3.1.2 (iPhone; iPhone OS 6.0; en_US) AppleWebKit/420+";
const char S75697[] = "Instagram 3.1.2 (iPhone; iPhone OS 6.0.1; en_US) AppleWebKit/420+";
const char S75698[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:16.1) Gecko/X Firefox/16.1 IceDragon/16.1.0.1";
const char S75699[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.4.3.X (Second Life Release; default skin) Safari/533.3";
const char S75700[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Luna TAB374 Build/LunaTAB374) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75701[] = "Nokia6300/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia6300) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S75702[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Magicon Mnote Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75703[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Magicon Mnote Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S75704[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.21 (KHTML, like Gecko) rekonq/2.1 Safari/537.21";
const char S75705[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) rekonq/2.1 Safari/534.34";
const char S75706[] = "Mozilla/5.0 (Linux; Android 4.0.4; Q7A Build/ICS_IMM76D_B206) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75707[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT25i Build/9.0.D.0.484) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S75708[] = "Mozilla/5.0 (Linux; Android 4.1.2; Titanium S1 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75709[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IRIS 455 Build/LAVAIRIS455) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75710[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT885 Build/6.7.3-135_YIC-8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S75711[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S75712[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P769 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S75713[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P768 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S75714[] = "samsung-GT-N7100/1.0 Linux/2.6.35.7 Android/4.0.4 Release/03.27.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S75715[] = "N7100-N7100/1.0 Linux/2.6.35.7 Android/4.0 Release/01.21.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S75716[] = "N7100-N7100/1.0 Linux/2.6.35.7 Android/4.0 Release/01.20.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S75717[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; TAB750 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75718[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6506 Build/10.3.A.0.423) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S75719[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo P700i Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75720[] = "Mozilla/5.0 (Linux; Android 4.0.3; Lenovo P700 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75721[] = "Mozilla/5.0 (Linux; Android 4.3; GT-I9100 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75722[] = "Mozilla/5.0 (Linux; Android 4.3; GT-I9100G Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75723[] = "Mozilla/5.0 (Linux; Android 4.1.1; Lenovo S890 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75724[] = "Mozilla/5.0 (Linux; Android 4.2.1; A900 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75725[] = "ZTE-F107(3UK)/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S75726[] = "Mozilla/5.0 (Linux; U; Linux Ventana; zh-tw; Transformer TF101G Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S75727[] = "Mozilla/5.0 (Linux; U; Linux Ventana; en-us; Transformer TF101G Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S75728[] = "Mozilla/5.0 (Linux; U; Linux Ventana; zh-tw; Transformer TF101G Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S75729[] = "Mozilla/5.0 (Linux; U; Linux Ventana; it-it; Transformer TF101G Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S75730[] = "Mozilla/5.0 (Linux; U; Linux Ventana; en-gb; Transformer TF101G Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S75731[] = "Mozilla/5.0 (Linux; U; Linux Ventana; de-de; Transformer TF101G Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S75732[] = "Mozilla/5.0 (Linux; U; Linux Ventana; en-ca; Transformer TF101G Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S75733[] = "Mozilla/5.0 (Linux; U; Linux Ventana; ru-ru; Transformer TF101G Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S75734[] = "Mozilla/5.0 (Linux; U; Linux Ventana; en-us; Transformer TF101G Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S75735[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; SKY IM-A630K Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75736[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-; SKY IM-A630K Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75737[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; SKY IM-A630K Build/FRG83)";
const char S75738[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S75739[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S75740[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352";
const char S75741[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S75742[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S75743[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S75744[] = "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 4.0)";
const char S75745[] = "Mozilla/5.0 (compatible; MSIE 9.0; WinNT4.0; Trident/4.0)";
const char S75746[] = "Mozilla/5.0 (compatible; MSIE 9.0; AOL 9.7; AOLBuild 4343.19; Windows NT 4.0; Trident/5.0)";
const char S75747[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-I897 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75748[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I897 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75749[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Q-Smart S20 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75750[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S75751[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G X710s Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S75752[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A13-MID Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75753[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; A13-MID Build/JRO03C)";
const char S75754[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-E400 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S75755[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A850L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S75756[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A850S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S75757[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A850K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S75758[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S6010 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75759[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-S6010L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75760[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75761[] = "Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S75762[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75763[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL_ONE_TOUCH_6010X_Orange Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75764[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LS970 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S75765[] = "NokiaE50-2/3.0 (07.36.0.0) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S75766[] = "NokiaE50-1/3.0 (06.41.3.0) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S75767[] = "SAMSUNG-GT-C5510/C5510DFII1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S75768[] = "SAMSUNG-GT-C5510H/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1/*MzU0NzUyMDMwMTEwNDg4";
const char S75769[] = "Facebook/4110.0 CFNetwork/485.12.7 Darwin/10.4.0";
const char S75770[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-ve; UM840 Build/HuaweiUM840) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75771[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_Incredible_S; 480*800) UCWEB7.9.3.103/139/800";
const char S75772[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_Incredible_S; 480*800) UCWEB7.9.3.103/139/999";
const char S75773[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_Incredible_S; 480*800) UCWEB7.9.1.100/139/800";
const char S75774[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_Incredible_S; 480*800) UCWEB7.9.3.103/139/800";
const char S75775[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_Incredible_S; 480*800) UCWEB7.9.3.103/139/32160";
const char S75776[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_Incredible_S; 480*800) UCWEB7.9.4.145/139/800";
const char S75777[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_Incredible_S; 480*800) UCWEB7.9.4.145/139/800";
const char S75778[] = "JUC (Linux; U; 4.0.4; zh-cn; HTC_Incredible_S; 480*800) UCWEB7.9.0.94/139/352";
const char S75779[] = "JUC (Linux; U; 4.0.4; zh-cn; HTC_Incredible_S; 480*800) UCWEB7.9.0.94/139/444";
const char S75780[] = "JUC (Linux; U; 4.0.4; zh-cn; HTC_Incredible_S; 480*800) UCWEB7.9.4.145/139/800";
const char S75781[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_Incredible_S; 480*800) UCWEB7.9.0.94/139/800";
const char S75782[] = "Mozilla/5.0 (Java; U; en-us; nokian8-00) UCBrowser8.3.0.154/69/444/UCWEB Mobile  3gpp-gba";
const char S75783[] = "Mozilla/5.0 (PLAYSTATION 3 4.22) AppleWebKit/531.22.8 (KHTML, like Gecko)";
const char S75784[] = "Mozilla/5.0 (Linux; Android 4.1.2; NookColor Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S75785[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; MITO A200 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75786[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A200 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75787[] = "Mozilla/5.0 (Linux; Android 4.0.4; F-04E Build/V14R31C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75788[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; Nokia N96) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S75789[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-N7000 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S75790[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-N7000 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S75791[] = "Opera/12.02 (Android 4.1; Linux; Opera Mobi/ADR-X; U; en-US) Presto/2.9.201 Version/12.02";
const char S75792[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MAXWEST_ORBIT6100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75793[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; WESTMAX_ORBIT6100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75794[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL_one_touch_995 Build/IceCreamSandwich) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S75795[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE N880G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75796[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE N880F Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75797[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaC5-06) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S75798[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; A880 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75799[] = "[FBAN/FB4A;FBAV/2.3;FBBV/X;FBDM/{density=1.5,width=540,height=960};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/DROID RAZR;FBSV/4.1.2;]";
const char S75800[] = "[FBAN/FB4A;FBAV/3.1;FBBV/X;FBDM/{density=1.5,width=540,height=960};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/DROID RAZR;FBSV/4.1.2;]";
const char S75801[] = "[FBAN/FB4A;FBAV/3.2.1;FBBV/X;FBDM/{density=1.5,width=540,height=960};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/DROID RAZR;FBSV/4.1.2;]";
const char S75802[] = "[FBAN/FB4A;FBAV/3.3;FBBV/X;FBDM/{density=1.5,width=540,height=960};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/DROID RAZR;FBSV/4.1.2;]";
const char S75803[] = "[FBAN/FB4A;FBAV/3.4;FBBV/X;FBDM/{density=1.5,width=540,height=960};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/DROID RAZR;FBSV/4.1.2;]";
const char S75804[] = "[FBAN/FB4A;FBAV/3.5;FBBV/X;FBDM/{density=1.5,width=540,height=960};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/DROID RAZR;FBSV/4.1.2;]";
const char S75805[] = "[FBAN/FB4A;FBAV/3.6.1;FBBV/X;FBDM/{density=1.5,width=540,height=960};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/DROID RAZR;FBSV/4.1.2;FBCA/armeabi-v7a:armeabi;]";
const char S75806[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7102 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S75807[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7102 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S75808[] = "Mozilla/5.0 (Linux; Android 4.1.9; GT-N7102 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S75809[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire HD Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S75810[] = "Mozilla/5.0 (Linux; Android 4.0.4; Desire HD Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S75811[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire HD Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S75812[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; iris 401e Build/LAVA-iris401e-S109) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75813[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; iris 401e Build/LAVA-iris401e-S111) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75814[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; iris 401e Build/LAVA-iris401e-S114) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75815[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9070 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S75816[] = "Mozilla/5.0 (Linux; Android 4.1.2; A511 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S75817[] = "Mozilla/5.0 (Linux; Android 4.1.2; WL-101GQC Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S75818[] = "SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj108i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S75819[] = "Mozilla/5.0 (Linux; Android 4.1.1; M532 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S75820[] = "Mozilla/5.0 (Linux; Android 4.1.2; A500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S75821[] = "sam-r420 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S75822[] = "Alcatel-OH5/1.0 UP.Browser/6.1.0.7.7 (GUI) MMP/1.0";
const char S75823[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-es; Blu Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75824[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; BLU Build/1.0.6330.0359) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75825[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; BLU Build/1.0.7264.0392) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75826[] = "Opera/9.0 (Macintosh; PPC Mac OS X; U; en)";
const char S75827[] = "Opera/9.00 (Macintosh; PPC Mac OS X; U; en)";
const char S75828[] = "Opera/9.01 (Macintosh; PPC Mac OS X; U; en)";
const char S75829[] = "Opera/9.02 (Macintosh; PPC Mac OS X; U; en)";
const char S75830[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; OPPOX9015 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75831[] = "Mozilla/5.0 (Windows NT 6.0; rv:13.0) Gecko/X Thunderbird/13.0.1";
const char S75832[] = "iBrowser/2.7 (Nokia5233;SymbianOS/9.1 Series60/3.0)";
const char S75833[] = "MQQBrowser/2.6 Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3";
const char S75834[] = "MQQBrowser/2.6 Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3";
const char S75835[] = "Mozilla/5.0 (Linux; Android 4.0.3; U9500 Build/HuaweiU9500) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75836[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S75837[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S75838[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S75839[] = "Mozilla/5.0 (Java; U; en-us; nokian70-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S75840[] = "Mozilla/5.0 (Java; U; en-us; nokian70) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S75841[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE65) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile";
const char S75842[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiae65-1) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S75843[] = "NOKIAE65UCBrowser/8.7.0.218/28/352/UCWEB";
const char S75844[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHL21 Build/S2140) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75845[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia603) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S75846[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530M Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75847[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-R530M Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75848[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530M Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/d2mtr";
const char S75849[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/528.8 (KHTML, like Gecko) Chrome/2.0.156.1 Safari/528.8";
const char S75850[] = "TSM-T8i Linux/3.0.13 Android/4.0.4 Release/05.29.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S75851[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TSM-T8i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75852[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SM-G730A Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75853[] = "Mozilla/5.0 (Series40; NokiaC2-06/07.48; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S75854[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC) Opera 8.65 [en]";
const char S75855[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC) Opera 8.65 [ru]";
const char S75856[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; (null)) Opera 8.65 [en]";
const char S75857[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone) Opera 8.65";
const char S75858[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320) Opera 8.65 [en]";
const char S75859[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC) Opera 8.65 [de]";
const char S75860[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; G500) Opera 8.65";
const char S75861[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 240x320) Opera 8.65 [en]";
const char S75862[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320) Opera 8.60 [en]";
const char S75863[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; 320x320; XV6900; Window Mobile 6.1 Professional;) Opera 8.65 [en]";
const char S75864[] = "Opera/9.5";
const char S75865[] = "Opera/9.50";
const char S75866[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB855 Build/4.5.1A-1_SUN-198_6; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75867[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Photonic Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75868[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LS670 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75869[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LS670 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75870[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LS670 Build/IMM76L; CyanogenMod-9.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75871[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E200 Safari Version/5.1";
const char S75872[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E401 Safari Version/5.1";
const char S75873[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_10 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E600 Safari Version/5.1";
const char S75874[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari Version/5.1";
const char S75875[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148a Safari Version/5.1";
const char S75876[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Haier HW-W910 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75877[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Haier HW-W910 Build/IMM76I)";
const char S75878[] = "Mozilla/5.0 (Java; U; en-in; nokia500) UCBrowser8.3.0.154/69/444/UCWEB Mobile  3gpp-gba";
const char S75879[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) rekonq/1.3 Safari/534.34";
const char S75880[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE N881E Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75881[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE N881E Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S75882[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ZTE N881E Build/IMM76I)";
const char S75883[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; One V Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75884[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; One V Build/JZO54K)";
const char S75885[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_One_VX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75886[] = "Mozilla/5.0 (Linux; Android 4.0.4; Andromax U Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75887[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6233) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S75888[] = "Nokia6233/2.0 (05.60) Profile/MIDP-2.0 Configuration/CLDC-1.1 , UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6233) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S75889[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT885 Build/6.7.3-135_YIC-8) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S75890[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E455f Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75891[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Spice Mi-450 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75892[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Mi450 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75893[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-S735C Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75894[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S75895[] = "Mozilla/5.0 (Linux; Android 4.0.4; C1605 Build/11.1.A.0.68) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S75896[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I8160 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S75897[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S7562) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S75898[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I757M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75899[] = "SonyEricssonT280a/R1CB002 TelecaBrowser/Q04C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S75900[] = "SonyEricssonT280i/R1CB002 TelecaBrowser/Q04C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S75901[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 1.0/SamsungSGHi560/I560DTHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S75902[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Vodafone/1.0/SamsungSGHi560/I560BUGK1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S75903[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Vodafone/1.0/SamsungSGHi560/I560BUHF1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S75904[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 1.0/SamsungSGHi560/I560JDHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S75905[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 1.0/SamsungSGHi560/I560DFHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S75906[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 1.0/SamsungSGHi560/I560XXGL1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S75907[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 1.0/SamsungSGHi560/I560XEGL2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S75908[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 1.0/SamsungSGHi560/I560JVHB3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S75909[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 1.0/SamsungSGHi560/I560XXHE1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S75910[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Vodafone/1.0/SamsungSGHi560/I560BUGJ5 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S75911[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Vodafone/1.0/SamsungSGHi560/I560BUHD3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S75912[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Vodafone/1.0/SamsungSGHi560/I560BUHB1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S75913[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Vodafone/1.0/SamsungSGHi560/I560JBHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S75914[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Vodafone/1.0/SamsungSGHi560/I560AEHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S75915[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Vodafone/1.0/SamsungSGHi560/I560JBHB1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S75916[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 1.0/SamsungSGHi560/I560JCHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S75917[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 1.0/SamsungSGHi560/I560DTHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S75918[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 1.0/SamsungSGHi560/I560JAHD2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S75919[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung-SGH-i560/BGHA1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S75920[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Vodafone/1.0/SamsungSGHi560/I560BUHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S75921[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; E130 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S75922[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; E130 Build/ERE27)";
const char S75923[] = "SAMSUNG-E2210BT/E2210BZDIG1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S75924[] = "SAMSUNG-E2210B/E2210BXAIJ1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S75925[] = "SAMSUNG-E2210L/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S75926[] = "SAMSUNG-E2210/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S75927[] = "SAMSUNG-E2210B-Bouygues/E2210BAGIF3 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S75928[] = "SAMSUNG-GT-C3750/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S75929[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 710)";
const char S75930[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 710; Vodafone)";
const char S75931[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 710; Orange)";
const char S75932[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 710; T-Mobile)";
const char S75933[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 710; Meteor)";
const char S75934[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; AT570 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75935[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; AT570 Build/IML74K)";
const char S75936[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CMP743d Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75937[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CMP743d Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75938[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; ODYS-Q Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75939[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE U880E Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S75940[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE U880E Build/GWK74) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S75941[] = "ZTEU880E_TD/1.0 Linux/2.6.35 Android/2.3 Release/12.15.2011 Browser/AppleWebKit533.1";
const char S75942[] = "Mozilla 5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE U880E Build GWK74) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S75943[] = "Mozilla/3.0 (compatible; WebMon 1.0.12; Windows 7)";
const char S75944[] = "RssBandit/1.9.0.1002 (.NET CLR 2.0.50727.5466; WinNT 6.1.7601 Service Pack 1; http://www.rssbandit.org)";
const char S75945[] = "SharpReader/0.9.7.0 (.NET CLR 2.0.50727.5466; WinNT 6.1.7601 Service Pack 1)";
const char S75946[] = "MQQBrowser/38 (iOS 4S; U; CPU like Mac OS X; zh-cn)";
const char S75947[] = "MQQBrowser/38 (iOS 4; U; CPU like Mac OS X; zh-cn)";
const char S75948[] = "MQQBrowser/38 (iOS 5; U; CPU like Mac OS X; zh-cn)";
const char S75949[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S75950[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; X8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75951[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; X8 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75952[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; Brava DM-993 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75953[] = "Mozilla/5.0 (Linux; Android 4.2.1; Transformer Prime TF201 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S75954[] = "Mozilla/5.0 (Linux; Android 4.2.2; ASUS Transformer Prime TF201 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S75955[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T699 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S75956[] = "Mozilla/5.0 (Linux; Android 4.2.1; V370 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75957[] = "Mozilla/5.0 (Linux; Android 4.1.2; N9810 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75958[] = "Mozilla/5.0 (Linux; Android 4.3; GT-I9505 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.30 Mobile Safari/537.36";
const char S75959[] = "Mozilla/5.0 (Linux; Android 4.3; GT-I9505G Build/JWR66Y.S003) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S75960[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I605 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S75961[] = "MOT-V360v/08.B7.AFR MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S75962[] = "MOT-V360/08.B7.2ER MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S75963[] = "Mozilla/5.0 (compatible; Teleca Q7; Brew 3.1.5; U; en) 240X400 LGE UX700";
const char S75964[] = "DoCoMo/2.0 P07B(c500;TB;W24H16)";
const char S75965[] = "JUC(Linux;U;Android2.2.1;Zh_cn;GT-S5670;240*320;)UCWEB7.8.0.95/139/444";
const char S75966[] = "Mozilla/5.0(Linux;U;Android2.2.1;Zh_cn;GT-S5670;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355";
const char S75967[] = "Mozilla/5.0 (Series40; NokiaX2-00/04.90; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S75968[] = "NokiaE63/UCWEB8.3.0.133/28/999";
const char S75969[] = "Mozilla/5.0 (Java; U; en-in; nokiae63-1) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S75970[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; 009Z Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75971[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; 009Z Build/GINGERBREAD)";
const char S75972[] = "Mozilla/5.0 (Windows; U; Windows NT 3.51; en-GB; rv:1.9.0.1) Gecko/X Firefox/3.0.1";
const char S75973[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; GPLUS GN105 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75974[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; GIONEE GN105 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75975[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GN105 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S75976[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GN105 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S75977[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GN105 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S75978[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GN105 Build/GRJ22)";
const char S75979[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; M031 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S75980[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; M031 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S75981[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; M031 Build/IML74K) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 baidubrowser/023_1.41.3.2_diordna_069_046/uzieM_51_3.0.4_130M/Xe/1EXCEB87A57CEEE0B84F875E1A%7CX/1";
const char S75982[] = "Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; M031 Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S75983[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch)";
const char S75984[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM;Touch)";
const char S75985[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MZ607 Build/7.7.1-122-4-FLEM-WIFI-LA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75986[] = "Mozilla/5.0 (Linux; Android 4.0; Swipe_Fablet_F1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S75987[] = "JUC (Linux; U; 2.3.6; zh-cn; A1+; 320*480) UCWEB7.9.0.94/139/444";
const char S75988[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Opera/9.80 (Android; Opera Mini/6.0.25270/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S75989[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Opera/9.80 (Android; Opera Mini/6.0.25270/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S75990[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Mitab FLY Build/ICS.g08ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S75991[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaE6-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S75992[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I9070) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S75993[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia500) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba";
const char S75994[] = "Nokia500/UCWEB8.9.0.253/51/999";
const char S75995[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S75996[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320T Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S75997[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5100 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S75998[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P5100 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S75999[] = "Mozilla/5.0 (Linux; Android 4.0.3; SpringBoard Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S76000[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI MediaPad Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S76001[] = "Mozilla/5.0 (Linux; Android 4.0.3; MediaPad 7 Lite Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S76002[] = "MQQBrowser/3.4/Adr (Linux; U; 4.0.4; zh-cn; GT-S7562 Build/IMM76I.S7562ZCALI1;480*800)";
const char S76003[] = "MQQBrowser/3.4/Adr (Linux; U; 4.0.4; zh-cn; GT-S7562 Build/IMM76I.S7562ZCAMB2;480*800)";
const char S76004[] = "MQQBrowser/3.4/Adr (Linux; U; 4.0.4; zh-cn; GT-S7562i Build/IMM76I.S7562iZCALL1;480*800)";
const char S76005[] = "MQQBrowser/3.4/Adr (Linux; U; 4.0.4; zh-cn; GT-S7562i Build/IMM76I.S7562iZCALI6;480*800)";
const char S76006[] = "MQQBrowser/3.4/Adr (Linux; U; 4.0.4; zh-cn; GT-S7562 Build/IMM76I.S7562ZCALK2;480*800)";
const char S76007[] = "MQQBrowser/3.4/Adr (Linux; U; 4.0.4; zh-cn; GT-S7562 Build/IMM76I.S7562ZCAME1;480*800)";
const char S76008[] = "MQQBrowser/3.4/Adr (Linux; U; 4.0.4; zh-cn; GT-S7562 Build/IMM76I.S7562ZCALL1;480*800)";
const char S76009[] = "MQQBrowser/3.4/Adr (Linux; U; 4.0.4; zh-cn; GT-S7562 Build/IMM76I.S7562ZCAMG1;480*800)";
const char S76010[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; id301)";
const char S76011[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; id300)";
const char S76012[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; id300; Vodafone)";
const char S76013[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; GIO-GiONEE_C800/CBD7602LT_0301_V0327; 540*960; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76014[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; GIO-GiONEE_C800/CBD7602LT_0301_V0335; 540*960; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76015[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GIO-GiONEE_C800/CBD7602GT_0301_V0324; 540*960; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76016[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; GIO-GiONEE_C800/CBD7602LT_0301_V0358; 540*960; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76017[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; GIO-GiONEE_C800/CBD7602LT_0301_V0361; 540*960; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76018[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MB526 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76019[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MB526 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/mb526";
const char S76020[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-I9001 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S76021[] = "Mozilla/5.0 (Linux; Android 2.3.5; GT-I9001 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453";
const char S76022[] = "Mozilla/5.0 (Linux; Android 2.3.3; GT-I9001 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S76023[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Spice Mi-505 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76024[] = "Mozilla/5.0 (Series40; Nokia300/07.03; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S76025[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC5-03) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S76026[] = "UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-S5360) U2/1.0.0 UCBrowser/9.3.0.321 Mobile";
const char S76027[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A523 Safari/8536.25";
const char S76028[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A403 Safari/8536.25";
const char S76029[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A406 Safari/8536.25";
const char S76030[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; HTC Click-Orange Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S76031[] = "Opera/9.80 (Windows ME; U; en) Presto/2.2.15 Version/10.10";
const char S76032[] = "Mozilla/5.0 (X11; Linux x86_64) KHTML/4.7.4 (like Gecko) Konqueror/4.7";
const char S76033[] = "Mozilla/5.0 (X11; Linux i686) KHTML/4.7.4 (like Gecko) Konqueror/4.7";
const char S76034[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11 Comodo_Dragon/17.5.2.0";
const char S76035[] = "Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11 Comodo_Dragon/17.5.2.0";
const char S76036[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Symphony_W25 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76037[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Scroll Engage Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76038[] = "NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac2-02) UCBrowser8.4.0.159/70/444/UCWEB Mobile";
const char S76039[] = "OneBrowser/3.1 (Nokia2690/10.10)";
const char S76040[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; OfficeLiveConnector.1.3; OfficeLivePatch.1.3) Sleipnir/2.8.4";
const char S76041[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/4.0; GTB6; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; Sleipnir/2.8.5)";
const char S76042[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; FDM; Sleipnir/2.8.5)";
const char S76043[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MIDC410 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76044[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; MIDC410 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76045[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOMO9 STAR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76046[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOMO9star Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76047[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Philips W536 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76048[] = "Micromax X325/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S76049[] = "Mozilla/5.0 (Linux; Android 4.0.4; LIFETAB_S9512 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S76050[] = "OneBrowser/3.1 (SAMSUNG-GT-S3653/S3653JPJD1)";
const char S76051[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; GSmart GS202+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76052[] = "Nokia310/2.0 (07.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia310) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S76053[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian79-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S76054[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN79) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S76055[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76056[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5580C Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S76057[] = "Mozilla/5.0 (Linux; Android 4.1.1; A211 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S76058[] = "Mozilla/5.0 (Linux; Android 4.2.1; A500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S76059[] = "Mozilla/5.0 (Linux; Android 4.2.2; A500 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S76060[] = "Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S76061[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID-A13 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S76062[] = "Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S76063[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8500R/S8500RUXJF3; U; Bada/1.0; en-ca) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WVGA SMM-MMS/1.2.0 OPN-B";
const char S76064[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; IS04 Build/TEK008) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S76065[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; IS04 Build/FEK100) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S76066[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; IS04 Build/FEK200) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S76067[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5; U; Edition MacAppStore; en) Presto/2.9.168 Version/11.51";
const char S76068[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; es-ES; rv:1.8.0.12) Gecko/X Firefox/1.5.0.12";
const char S76069[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Polaroid PMID701i Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76070[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; Polaroid PMID701i Build/IML74K)";
const char S76071[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; PMID701DC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76072[] = "Mozilla/5.0 (X11; MOT-Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S76073[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6233) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S76074[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Sensation Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S76075[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 101G10 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S76076[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E970 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S76077[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E970 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S76078[] = "G51 Wynncom Linux/3.0.13 Android/4.0.4 Release/01.23.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S76079[] = "Micromax X272/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S76080[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; sv-se; C5503 Build/10.3.A.0.317) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76081[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; xh-us; C5503 Build/10.3.1.A.0.15) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76082[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; sv-se; C5503 Build/10.3.A.0.356) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76083[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; C5503 Build/10.3.1.A.0.244)";
const char S76084[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; sv-se; C5503 Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76085[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; GT-S6313T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76086[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S6313T Build/JZO54K)";
const char S76087[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S76088[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE51) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S76089[] = "Nokia310/2.0 (07.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia310) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S76090[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTabS2110AF Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S76091[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTabS2110AH Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S76092[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE Blade III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S76093[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE BLADE III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S76094[] = "Mozilla/5.0 (Linux; Android 4.0.4; Blade III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S76095[] = "Mozilla/5.0 (Linux; Android 4.0.4; BLADE III_IL Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S76096[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S76097[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-I9100 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S76098[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9100G Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S76099[] = "MOT-ZN5/R6637_G_81.11.2CR_128 Mozilla/5.0 (compatible; MSIE 6.0; Linux; MOTOZINE ZN5) Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony/1.0";
const char S76100[] = "MOT-ZN5/R6637_G_81.03.05R Mozilla/4.0 (compatible; OSS/1.0; Linux MOTOZINE ZN5) Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony 1.0";
const char S76101[] = "HTC_Touch_3G_T3232 Opera/9.50 (Windows NT 5.1; U; zh-tw)";
const char S76102[] = "HTC_Touch_3G_T3232 Opera/9.50 (Windows NT 5.1; U; de)";
const char S76103[] = "HTC_Touch_3G_T3232 Opera/9.50 (Windows NT 5.1; U; en)";
const char S76104[] = "HTC_Touch_3G_T3232 Opera/9.50 (Windows NT 5.1; U; pt)";
const char S76105[] = "HTC_Touch_3G_T3232 Opera/9.50 (Windows NT 5.1; U; nl)";
const char S76106[] = "HTC_Touch_3G_T3232 Opera/9.50 (Windows NT 5.1; U; ru)";
const char S76107[] = "LG/GD580/v1.0";
const char S76108[] = "LG-GD580/v1.0";
const char S76109[] = "Mozilla/5.0 (Series40; NokiaC2-01/11.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.0.9.17.2";
const char S76110[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; Movistar Link Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S76111[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.1)";
const char S76112[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.3)";
const char S76113[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.0.4)";
const char S76114[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.4)";
const char S76115[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.5)";
const char S76116[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.15)";
const char S76117[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.0rc3)";
const char S76118[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.0.5)";
const char S76119[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.7)";
const char S76120[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.6)";
const char S76121[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.0.9)";
const char S76122[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.9)";
const char S76123[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.0.16)";
const char S76124[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.16)";
const char S76125[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.0.17)";
const char S76126[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.17)";
const char S76127[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.8)";
const char S76128[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.12)";
const char S76129[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.0.15)";
const char S76130[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.11)";
const char S76131[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.14)";
const char S76132[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.0.14)";
const char S76133[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.13)";
const char S76134[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.10)";
const char S76135[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.2)";
const char S76136[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.0.10)";
const char S76137[] = "Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2730c-1/UC Browser8.0.3.107/69/444";
const char S76138[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.140 Safari/535.19 Comodo_Dragon/18.0.3.0";
const char S76139[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.140 Safari/535.19 Comodo_Dragon/18.0.3.0";
const char S76140[] = "Mozilla/5.0 (Series40; NokiaC3-00/08.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S76141[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:7.0.1) Gecko/X Firefox/7.0.1 AlexaToolbar/alxf-2.15";
const char S76142[] = "MeeGo/6.0";
const char S76143[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TM-MID880 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76144[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; chromeframe/24.0.1312.57)";
const char S76145[] = "Mozilla/5.0 (Linux; Android 4.0.3; AN10DG3 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S76146[] = "Mozilla/5.0 (Linux; Android 4.0.4; CnM Touchpad 9.7 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S76147[] = "Mozilla/5.0 (Linux; Android 4.0.4; Versus TouchPad 7 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76148[] = "Mozilla/5.0 (Linux; Android 4.0.4; CnM TouchPad 7 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76149[] = "Mozilla/5.0 (Linux; Android 4.0.4; CnM TouchPad 9 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76150[] = "Mozilla/5.0 (Linux; Android 4.0.4; Versus TouchPad 9 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76151[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9103 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76152[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9210 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76153[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I727R Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76154[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E110S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76155[] = "Mozilla/5.0 (Linux; Android 4.0.3; SHV-E110S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76156[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:22.0) Gecko/X Thunderbird/22.0";
const char S76157[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 86VEBC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76158[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; 86VEBC Build/IMM76D)";
const char S76159[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Mobile Safari/535.19 Silk-Accelerated=false";
const char S76160[] = "Mozilla/5.0 (Linux; Android 4.0.4; A10 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S76161[] = "Mozilla/5.0 (Linux; Android 4.0.4; A10 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S76162[] = "Mozilla/5.0 (Series40; NokiaC3-01/07.15; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.11";
const char S76163[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-XT928/6.5.1_GC-159-DNRCG-8; 720*1280; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76164[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT928 Build/6.5.1_GC-159-DNRCG-8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76165[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-XT928/6.5.1_GC-179-DNRCG-12; 720*1280; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76166[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-XT928/6.5.1_GC-179-DNRCG-9; 720*1280; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76167[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-XT928/6.5.1_GC-122-DNRCG-15; 720*1280; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76168[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT928 Build/6.5.1_GC-159-DNRCG-8) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S76169[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT928 Build/6.5.1_GC-179-DNRCG-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76170[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT928 Build/6.5.1_GC-179-DNRCG-12) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S76171[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM010SH Build/S0012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76172[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM010SH Build/S0010) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76173[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM010SH Build/S0014) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76174[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM010SH Build/S0016) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76175[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM010SH Build/S0018) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76176[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM010SH Build/S0020) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76177[] = "WebProcess/7534.53.11 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookPro8%2C1)";
const char S76178[] = "RealPlayer%20Downloader/1725 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookPro8%2C2)";
const char S76179[] = "WebProcess/7534.53.11 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookAir4%2C2)";
const char S76180[] = "ThumbnailService/39400.005 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookPro8%2C2)";
const char S76181[] = "ocspd (unknown version) CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookPro8%2C1)";
const char S76182[] = "WebProcess/7534.53.11 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (iMac12%2C1)";
const char S76183[] = "iCoreService (unknown version) CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookPro8%2C1)";
const char S76184[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; C8600 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76185[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-ve; C8600 Build/HuaweiC8600) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76186[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.1";
const char S76187[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.2";
const char S76188[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.4";
const char S76189[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.5";
const char S76190[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.6";
const char S76191[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.7";
const char S76192[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.8";
const char S76193[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Flipboard/1.9.8";
const char S76194[] = "Mozilla/5.0 (Linux; Android 4.0.3; PMP5097CPRO Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S76195[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MIDC407 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76196[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MIDC407 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76197[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; AN7CG3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76198[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; GIONEE_Ctrl_V3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76199[] = "Mozilla/5.0 (DTV) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.1.1 AQUOSBrowser/1.1 (US01DTV;V;0001;0001)";
const char S76200[] = "Akregator/4.10.1; syndication";
const char S76201[] = "Akregator/4.10.2; syndication";
const char S76202[] = "Akregator/4.10.3; syndication";
const char S76203[] = "Akregator/4.10.4; syndication";
const char S76204[] = "Akregator/4.10.5; syndication";
const char S76205[] = "Mozilla/5.0 (Linux; Android 4.0.4; Grand X(M) Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76206[] = "Mozilla/5.0 (Linux; Android 4.0.3; ZTE Grand X Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76207[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970M Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76208[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76209[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu/13.04 Chromium/27.0.1453.6 Chrome/27.0.1453.6 Safari/537.36";
const char S76210[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu/13.04 Chromium/27.0.1453.6 Chrome/27.0.1453.6 Safari/537.36";
const char S76211[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire V Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S76212[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire VC Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S76213[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76214[] = "Mozilla/5.0 (Linux; Android 4.0.3; SHW-M250S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76215[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76216[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76217[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire U Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S76218[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire U dual sim Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S76219[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; AT-AS40SE Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76220[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Connect-2G-2.0 Build/HCL ME Tablet V2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76221[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Micromax A110Q Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S76222[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5 UCBrowser/9.3.0.321";
const char S76223[] = "HTC-ST7377/1.59.502.3 (67150) Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S76224[] = "HTC-ST7377/2.10.502.4 (93577) Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S76225[] = "SAMSUNG-M3510/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S76226[] = "SAMSUNG-M3510L/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S76227[] = "MicromaxX210/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S76228[] = "MicromaxX210+/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S76229[] = "Nokia5230/UCWEB8.4.0.150/50/999";
const char S76230[] = "Mozilla/5.0 (S60V5; U; en-in; Nokia5230)/UC Browser8.4.0.159/50/352/UCWEB Mobile";
const char S76231[] = "JUC(Linux; U;2.2.2;zh_cn;Coolpad W706+;320*480;)UCWEB7.6.0.75/139/32411";
const char S76232[] = "JUC(Linux; U;2.3.4;zh_cn;Lenovo P70;320*480;)UCWEB7.6.0.75/139/32101";
const char S76233[] = "JUC(Linux; U;2.2.2;zh_cn;Coolpad W706;320*480;)UCWEB7.6.0.75/139/32411";
const char S76234[] = "JUC(Linux;U;2.1;Zh_cn;E239;320*480;)UCWEB7.6.0.75/139/800";
const char S76235[] = "JUC(Linux;U;2.1;Zh_cn;D539;320*480;)UCWEB7.6.0.75/139/800";
const char S76236[] = "JUC(Linux; U;2.2.2;zh_cn;5855;320*480;)UCWEB7.6.0.75/139/32521";
const char S76237[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Vision Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76238[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Vision Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76239[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N860 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76240[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N860 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S76241[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_9p017 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S76242[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_ov5647 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S76243[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_9p017 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S76244[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_253_6628 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S76245[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_5640 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S76246[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_253 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S76247[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; AMOI N821 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76248[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; AMOI N821 Build/JRO03C)";
const char S76249[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT885 Build/9.8.3O-148_IRD-3) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S76250[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Optimus G Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76251[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717R Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76252[] = "Mozilla/5.0 (Linux; Android 4.1.2; SM-T210 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S76253[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S76254[] = "Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabS2109A-F Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S76255[] = "MOT-Z6/R60_G_80.xx.yyI Mozilla/4.0 (compatible; MSIE 6.0 Linux; MOTOROKR Z6;nnn) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.50[yy]";
const char S76256[] = "MOTOROKR Z6/R60_G_80.33.03R Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.50[es-LA]";
const char S76257[] = "MOTOROKR Z6/R60_G_80.xx.yyI Mozilla/4.0 (compatible; MSIE 6.0 Linux; MOTOROKR Z6;nnn) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.50[yy]";
const char S76258[] = "MOT-MOTOROKRZ6_CMCC/1.0 LinuxOS/2.6.10 Release/04.30.2007 Browser/Opera8.50 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R60_G_80.31.11R";
const char S76259[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Pulse Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S76260[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-in; Pulse Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S76261[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Pulse Build/ERE27)";
const char S76262[] = "SAMSUNG-SGH-J800/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S76263[] = "LG-KP570 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S76264[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; N880 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76265[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; N880 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76266[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; BLADE_N880 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76267[] = "Mozilla/5.0 (Linux; U; Android 2.2; en_us; N880 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.2.0 Mobile (Dragonfruit)";
const char S76268[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; ZTE-C_N880/N880V1.0.0B05; 480*800; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76269[] = "MAUI";
const char S76270[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; LG-P970 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76271[] = "Mozilla/5.0 (Windows; U; Win95; rv:1.8.1) Gecko/X Firefox/2.0";
const char S76272[] = "Mozilla/5.0 (X11; U; Win95; en-US; rv:1.8.1) Gecko/X BonEcho/2.0";
const char S76273[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618)";
const char S76274[] = "Mozilla/5.0 (Linux; Android 4.0.4; Incredible 2 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S76275[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Incredible 2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S76276[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Droid Incredible 2 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S76277[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; AMOI_N808 Build/ALPS.GB2.MP.V1.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76278[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; GPLUS-N808 Build/ALPS.GB2.MP.V1.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76279[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; AMOI N808 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76280[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; AMOI N808 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S76281[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; chromeframe/23.0.1271.97)";
const char S76282[] = "JUC (Linux; U; 4.0.4; zh-cn; LT26i; 720*1280) UCWEB7.9.0.94/139/444";
const char S76283[] = "JUC (Linux; U; 2.3.7; zh-cn; LT26i; 720*1280) UCWEB7.9.0.94/139/444";
const char S76284[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S76285[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.3 (KHTML, like Gecko) Chrome/1, 6, 0, 4 Safari/535.3";
const char S76286[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/1.0.154.36 Safari/525.19";
const char S76287[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825-1 Build/HuaweiU8825-1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S76288[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S76289[] = "Mozilla/5.0 (Linux; Android 4.1.2; Amazon Kindle Fire Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S76290[] = "Mozilla/5.0 (Linux; Android 4.1.1; Amazon Kindle Fire Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S76291[] = "HUAWEI Y310-5000_TD/1.0 Linux/2.6.35 Android/2.3.5 Release/7.16.2012 Mozilla/5.0 (Linux; U; Android 2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76292[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A89 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S76293[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A89 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S76294[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S76295[] = "Mozilla/5.0 (Linux; Android 4.0.3; SHW-M250S Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S76296[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250K Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S76297[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250L Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S76298[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus S Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76299[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus S Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76300[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5570 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S76301[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax A110) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S76302[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S76303[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S76304[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-CN; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S76305[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Mobile Safari/535.19 Silk-Accelerated=false";
const char S76306[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P3100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S76307[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P3100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76308[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC6435LVW Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76309[] = "Mozilla/5.0 (Linux; Android 4.2; HTC6435LVW Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76310[] = "Mozilla/5.0 (Linux; Android 4.0.4; AZ210A Build/AZ210A_ICS_02.34I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S76311[] = "Mozilla/5.0 (Linux; Android 4.0.4; AZ210B Build/AZ210B_ICS_02.34I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S76312[] = "Alcatel-OT-806/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A SVN/04E";
const char S76313[] = "Alcatel-OT-806A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S76314[] = "Alcatel-OT-806D/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S76315[] = "Alcatel-OT-806/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S76316[] = "Vodafone/1.0/SAMSUNG-SGH-J700V/BUHA3/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S76317[] = "Vodafone/1.0/SAMSUNG-SGH-J700V/JBHC1/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S76318[] = "Vodafone/1.0/SAMSUNG-SGH-J700V/BUHG1/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S76319[] = "Vodafone/1.0/SAMSUNG-SGH-J700V/BUHB2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S76320[] = "Vodafone/1.0/SAMSUNG-SGH-J700V/BUHB1/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S76321[] = "Vodafone/1.0/SAMSUNG-SGH-J700V/AMHB2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S76322[] = "Vodafone/1.0/SAMSUNG-SGH-J700V/AEHE1/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S76323[] = "Microsoft Office/12.0 (Windows NT 6.0; Microsoft Office Outlook 12.0.6562; Pro)";
const char S76324[] = "Microsoft Office/12.0 (Windows NT 6.0; Microsoft Office Outlook 12.0.6607; Pro)";
const char S76325[] = "Microsoft Office/12.0 (Windows NT 6.0; Microsoft Office Outlook 12.0.6662; Pro)";
const char S76326[] = "Microsoft Office/12.0 (Windows NT 6.0; Microsoft Office Outlook 12.0.4518; Pro)";
const char S76327[] = "Microsoft Office/12.0 (Windows NT 6.0; Microsoft Office Outlook 12.0.6554; Pro)";
const char S76328[] = "Microsoft Office/12.0 (Windows NT 6.0; Microsoft Office Outlook 12.0.6683; Pro)";
const char S76329[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76330[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76331[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76332[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76333[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76334[] = "MQQBrowser/3.5/Adr (Linux; U; 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD.ZCLC2;480*800)";
const char S76335[] = "MQQBrowser/3.5/Adr (Linux; U; 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD.ZSKI3;480*800)";
const char S76336[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76337[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9108 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76338[] = "MQQBrowser/3.5/Adr (Linux; U; 2.3.5; zh-cn; GT-I9100G Build/GINGERBREAD.ZCKI1;480*800)";
const char S76339[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE N855D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76340[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE N855D Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S76341[] = "Micromax X445/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S76342[] = "Mozilla/5.0 (Linux; Android 4.0.4; MI 1S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S76343[] = "Mozilla/5.0 (Linux; Android 4.0.4; MI 1SC Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S76344[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I-Buddy 7.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76345[] = "KKT38_WAP_Browser";
const char S76346[] = "Mozilla/5.0 (Linux; Android 4.0.4; TOUCH90 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S76347[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; ST25i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S76348[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; ST25a) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S76349[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76350[] = "IUC(U;iOS 6.1.3;Zh-cn;320*480;)/UCWEB8.4.1.169/42/997";
const char S76351[] = "Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_S9714 Build/JRO03R) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S76352[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.14/271,1.9.14.271,2012-12-20 13:41, -0500, ca";
const char S76353[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0200, za";
const char S76354[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, -0400, ca";
const char S76355[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, -0400, ca";
const char S76356[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; AIRIS TM350 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76357[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; TM350 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76358[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T8700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76359[] = "Mozilla/5.0 (Linux; Android 4.2.1; PMP5588C Build/E8HD-ELITE-1280x768) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S76360[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S76361[] = "Mozilla/5.0 (Linux; Android 4.2.1; W100 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S76362[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/10A523 Safari/7534.48.3";
const char S76363[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/10A8426 Safari/7534.48.3";
const char S76364[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/10A403 Safari/7534.48.3";
const char S76365[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/10A550 Safari/7534.48.3";
const char S76366[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/10A407 Safari/7534.48.3";
const char S76367[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/10A406 Safari/7534.48.3";
const char S76368[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/10A8500 Safari/7534.48.3";
const char S76369[] = "AppleCoreMedia/1.0.0.8B117 (iPhone; U; CPU OS 4_1 like Mac OS X; en_us)";
const char S76370[] = "MOT-V860/00.72 UP.Browser/7.2.7.5.630 (GUI) MMP/2.0";
const char S76371[] = "MOT-V860/00.72 UP.Browser/7.2.7.5.665 (GUI) MMP/2.0";
const char S76372[] = "Mozilla/5.0 (compatible; MSIE 7.0; MSIE 6.0; DoCoMo; Nokia; SymbianOS; NetFront; XENIUM; PPC; ScanAlert; +http://www.mcafeesecure.com/bot.jsp )";
const char S76373[] = "Mozilla/5.0 (compatible; Konqueror/4.5; Linux) KHTML/4.5.5 (like Gecko)";
const char S76374[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5; U; Edition MacAppStore; en) Presto/2.9.168 Version/11.52";
const char S76375[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.5.24 Version/10.53";
const char S76376[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.5.22 Version/10.51";
const char S76377[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.5.22 Version/10.50";
const char S76378[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.5.24 Version/10.54";
const char S76379[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.10) Gecko/X Ubuntu Firefox/3.6.10";
const char S76380[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Optimus 2X Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76381[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; Optimus 2X Build/MIUI)";
const char S76382[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG Optimus 2X Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76383[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB860 Build/MIUI-2.2.17.0-OLYMPUS-EN; LG-P990 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76384[] = "Mozilla/5.0 (Windows NT 6.0; rv:10.0.3) Gecko/X Firefox/10.0.3 Epic/10.0.3";
const char S76385[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 610)";
const char S76386[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 610; Vodafone)";
const char S76387[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; B79 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76388[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; RPSPG3201 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76389[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; AQUA 4.0 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76390[] = "Mozilla/5.0 (Linux; Android 4.0.3; GEM10312BK Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S76391[] = "Mozilla/5.0 (Linux; Android 4.0.4; GEM10312BK Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S76392[] = "Nokia201/2.0 (11.64) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia201) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S76393[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ENERGY i8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76394[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Viettel I8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76395[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-S5830 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76396[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-S5830 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76397[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT26i Build/6.1.A.2.55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S76398[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ID720WTA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76399[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MT25i Build/4.1.B.0.631) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S76400[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT25i Build/4.1.B.0.631) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S76401[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/10A403 Safari/7534.48.3";
const char S76402[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/10A523 Safari/7534.48.3";
const char S76403[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/10A550 Safari/7534.48.3";
const char S76404[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/10A407 Safari/7534.48.3";
const char S76405[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/10A8426 Safari/7534.48.3";
const char S76406[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire HD Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76407[] = "Mozilla/5.0 (Linux; Android 4.0.4; Desire HD Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76408[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire HD Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76409[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9082) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S76410[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Symphony_W125 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76411[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax A110 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S76412[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S76413[] = "Mozilla/5.0 (Linux; Android 4.0.3; Lenovo S880 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S76414[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S76415[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S76416[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S76417[] = "Mozilla/5.0 (Linux; Android 4.0.3; AN10DG3 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S76418[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI 1S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S76419[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI 1SC Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S76420[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-04E Build/10.1.1.D.2.31) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S76421[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S76422[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Stream Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76423[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; Stream Build/MASTER)";
const char S76424[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC_Touch_3G_T3232-Orange; OpVer 33.171.1.6116";
const char S76425[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Jade/1.40.164.2 (21259)";
const char S76426[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC_Touch_3G_T3232-Orange; OpVer 33.171.1.6112";
const char S76427[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC_Touch_3G_T3232-Orange; OpVer 33.171.2.7311";
const char S76428[] = "HTC_Touch_3G_T3232 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S76429[] = "SonyEricssonK618i/R1JG Release/Feb-15-2007 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S76430[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-es; Light Pro Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76431[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; tr-tr; Turkcell T10 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S76432[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Turkcell T10 Build/ERE27)";
const char S76433[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Incredible Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76434[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Incredible Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76435[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Incredible Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76436[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Incredible Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76437[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Droid Incredible Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76438[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; Incredible Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76439[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Incredible Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76440[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Incredible Build/FRF91; liquid.htcinc.X.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76441[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; MWP6885)";
const char S76442[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; Mac App Store Edition; en) Presto/2.7.62 Version/11.01";
const char S76443[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en) Presto/2.7.62 Version/11.00";
const char S76444[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; I897 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76445[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; I897 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76446[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; I897 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76447[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SGH-I897 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76448[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; I897 Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S76449[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P999 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76450[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P999 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76451[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P999 Build/MIUI; LG-P990 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76452[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P999 Build/MIUIandroid 1.8.19) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76453[] = "Mozilla/5.0 (Linux; U; Android 2.3.9; en-us; LG-P999 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76454[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; P-04D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76455[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; P-04D Build/GRJ90)";
const char S76456[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; i-mobile i-note WiFi Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76457[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-note WiFi 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76458[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-NOTE WIFI 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76459[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; i-mobile i-note WiFi 7 Build/IMM76D)";
const char S76460[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; i-mobile i-NOTE WIFI 2 Build/IMM76D)";
const char S76461[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN82) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/355/UCWEB Mobile";
const char S76462[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; RPSPE4301 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76463[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P990 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S76464[] = "JUC (Linux; U; 2.3.4; zh-cn; LT15i; 480*854) UCWEB7.9.0.94/139/352";
const char S76465[] = "JUC (Linux; U; 4.0.4; zh-cn; LT15i; 480*854) UCWEB7.9.0.94/139/444";
const char S76466[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-L300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S76467[] = "Rockmelt/3.0.0.328 CFNetwork/609.1.4 Darwin/13.0.0";
const char S76468[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amaze 4G Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76469[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amaze 4G Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/ruby";
const char S76470[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Titan TV S320 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76471[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-M950 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76472[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P765 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76473[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S76474[] = "Mozilla/5.0 (Linux; Android 4.0.4; LIFETAB_S9512 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S76475[] = "Mozilla/5.0 (Linux; Android 4.2.1; Micromax A110Q Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76476[] = "Mozilla/5.0 (Linux; Android 4.2.1; Micromax A110Q Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S76477[] = "SAMSUNG-E2100/E2100JAIC1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S76478[] = "Mozilla/5.0 (Linux; U; Android 1.5; nl-; Pulse Build/CRB17) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S76479[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-gb; Pulse Build/CRB17) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S76480[] = "SAMSUNG-SGH-T659/T659UVIG9 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S76481[] = "SAMSUNG-SGH-C160L/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S76482[] = "SAMSUNG-SGH-C160/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S76483[] = "NokiaC6-00/UC Browser7.9.0.102/50/355/UCWEB";
const char S76484[] = "NokiaC6-00/UCWEB7.9.0.94/50/800";
const char S76485[] = "Midori/0.1.9 (X11; Linux i686; U; en-us) WebKit/532+";
const char S76486[] = "Midori/0.1.10 (X11; Linux i686; U; en-us) WebKit/(531).(2)+";
const char S76487[] = "Midori/0.1.10 (X11; Linux i686; U; en-us) WebKit/(531).(2)";
const char S76488[] = "Midori/0.1.7 (X11; Linux x86_64; U; en-us) WebKit/532+";
const char S76489[] = "Midori/0.1.20 (X11; Linux i686; U; fr-fr) WebKit/532.1+";
const char S76490[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ZTE-C_N760/N760V1.0.0B03; 320*480; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76491[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE-C_N760/N760V1.0.0B04; 320*480; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76492[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ZTE-C_N760/N760V1.0.0B02; 320*480; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76493[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ZTE-C N760 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S76494[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE-C N760 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S76495[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE-C_N760/N760V1.0.0B05; 320*480; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76496[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ZTE-C N760 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S76497[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE-C N760 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S76498[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE-C_N760/N760V1.0.0B07; 320*480; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76499[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; DATAM803HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76500[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ZP200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76501[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ZP200+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76502[] = "Mozilla/5.0 (Linux; Android 4.1.1; SHW-M440S Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S76503[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M440S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S76504[] = "Mozilla/5.0 (Java; U; en-us; gt-s5233w) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S76505[] = "Mozilla/5.0 (Java; U; vi; gt-s5233w) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S76506[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo K860 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S76507[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; AVA757 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76508[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; SPH-L710 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S76509[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S76510[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; SPH-L710 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S76511[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; SGH-I747M Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S76512[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9305 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S76513[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.442; en-us; 9550/4.2.0.442) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S76514[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.487; en-us; 9550/4.2.0.487) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S76515[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.323; en-us; 9550/4.2.0.323) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S76516[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.124; en-us; 9550/4.2.0.124) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S76517[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; POV_TAB_NAVI7_3G_M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76518[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; POV_TAB-NAVI7-3G-M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76519[] = "iPod4,1/6.1 (10B144)";
const char S76520[] = "server-bag [iPhone OS,6.1.3,10B329,iPod4,1]";
const char S76521[] = "iPod4,1/6.1.3 (10B329)";
const char S76522[] = "iPod4,1/6.1.2 (10B146)";
const char S76523[] = "iPod5,1/6.1.3 (10B329)";
const char S76524[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P870 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76525[] = "Mozilla/5.0 (X11; SunOS i86pc; rv:21.0) Gecko/X Firefox/21.0";
const char S76526[] = "Nokia308/2.0 (05.85) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia308) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S76527[] = "Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI MT1-U06 Build/HuaweiMT1-U06) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76528[] = "Mozilla/5.0 (Linux; Android 4.0.3; KFOT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S76529[] = "Mozilla/5.0 (Linux; Android 4.0.4; KF012 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76530[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8681 Build/HuaweiU8681) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76531[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI A199 Build/HuaweiA199) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76532[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI A199 Build/HuaweiA199) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S76533[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_A199/A199V100R001C92B123; 720*1280; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S76534[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HUAWEI A199 Build/HuaweiA199)";
const char S76535[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_A199/A199V100R001C92B128; 720*1280; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S76536[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_A199/A199V100R001C92B121SP05; 720*1280; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30";
const char S76537[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6802 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S76538[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-N7000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S76539[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; GT-N7000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S76540[] = "Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6303iclassic) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S76541[] = "Mozilla/5.0 (Linux; Android 4.0.3; AN10DG3 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S76542[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S76543[] = "Nokia3100/1.0 (06.31) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S76544[] = "Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2730c-1/UC Browser7.9.0.102/69/352";
const char S76545[] = "Mozilla/5.0 (Windows; U; en_US) AppleWebKit/530.19.2 (KHTML, like Gecko) Dreamweaver/11.5.0.5315 Version/4.0.2 Safari/530.19.2";
const char S76546[] = "Mozilla/5.0 (Windows; U; Windows NT based; en-US) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Safari/528.17";
const char S76547[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.0.4) Gecko/X Firefox/3.0.4 CometBird/3.0.4";
const char S76548[] = "Mozilla/5.0 (Android; Mobile; rv:12.0a2) Gecko/12.0a2 Firefox/12.0a2";
const char S76549[] = "Mozilla/5.0 (Android; Mobile; rv:12.0a2) Gecko/12.0 Firefox/12.0a2";
const char S76550[] = "Mozilla/5.0 (Android; Mobile; rv:12.0) Gecko/12.0 Firefox/12.0 Fennec/12.0";
const char S76551[] = "Mozilla/5.0 (Android; Mobile; rv:12.0) Gecko/12.0 Firefox/12.0";
const char S76552[] = "Mozilla/5.0 (X11; Linux x86_64; rv:10.0.2) Gecko/X Firefox/10.0.2 SeaMonkey/2.7.2";
const char S76553[] = "Mozilla/5.0 (X11; Linux i686; rv:10.0.2) Gecko/X SeaMonkey/2.7.2";
const char S76554[] = "samsung-gt-c6712/UC Browser8.0.3.107/69/444";
const char S76555[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s3653) UCBrowser8.3.0.154/69/352/UCWEB Mobile";
const char S76556[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s3653) UCBrowser8.3.1.161/69/405/UCWEB Mobile";
const char S76557[] = "Mozilla/5.0 (Series40; NokiaC3-01.5/07.32; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.7";
const char S76558[] = "Mozilla/5.0 (Series40; NokiaC3-01/06.00; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15";
const char S76559[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; th-th; A100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76560[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; MITO A100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76561[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SHT21 Build/SB221) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76562[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SHT21 Build/SC170) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76563[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SHT21 Build/S2150) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76564[] = "Mozilla/5.0 (Linux; Android 4.1.1; TANK 4.5 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S76565[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ609 Build/7.7.1-85_MZ609-31) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S76566[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xtouch X405 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76567[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300";
const char S76568[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Aqua I-5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76569[] = "Mozilla/5.0 (Series40; NokiaC1-01/06.15; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S76570[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ Ubuntu/11.04 (3.0.4-1ubuntu1) Epiphany/3.0.4";
const char S76571[] = "Mozilla/5.0 (Windows NT 6.0; rv:6.0) Gecko/X Firefox/6.0 SeaMonkey/2.3.1";
const char S76572[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Z710t Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76573[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Z710t Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S76574[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 810)";
const char S76575[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Mini Pro Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76576[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; X10 Build/JZO54K; CyanogenMod-AlienMOD_Solar_Flare_SeriesV) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76577[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE V889D Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S76578[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE V889D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76579[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; ZTE-U V889D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76580[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MOT-XT788 Build/IRPMCT_B_02.60.00RPD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76581[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MOT-XT788 Build/IRPMCT_B_02.60.00RPD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S76582[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MOT-XT788 Build/IRPMCT_B_02.64.00RPD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76583[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC One V Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S76584[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; W120 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76585[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76586[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76587[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76588[] = "NokiaC2-06/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-06) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S76589[] = "Mozilla/5.0 (Linux; Android 4.1.2; B15 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S76590[] = "Mozilla/5.0 (Linux; Android 4.0.3; Marquis_MP977 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S76591[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari/7534.48.3 TeamStream HD (iPad2,1 370 1.8.7)";
const char S76592[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Safari/7534.48.3 TeamStream HD (iPad3,4 411 1.9)";
const char S76593[] = "Mozilla/5.0 (Linux; Android 4.2.2; ME173X Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S76594[] = "Mozilla/5.0 (Linux; Android 4.2.1; ME173X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S76595[] = "LG-GB270 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S76596[] = "SEC-SGHB100/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S76597[] = "SEC-SGHB100/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S76598[] = "SAMSUNG-SGH-A597/A597UCJG1 SHP/VPP/R5 Jasmine/1.0 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S76599[] = "Alcatel-OT-799/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S76600[] = "Alcatel-OT-799A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S76601[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; Next2 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S76602[] = "Mozilla/5.0 (Windows; U; Windows NT 4.0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S76603[] = "GoogleEarth/6.1.0.5001(Macintosh;Mac OS X (10.6.8);en;kml:2.2;client:Free;type:default)";
const char S76604[] = "GoogleEarth/6.1.0.5001(Macintosh;Mac OS X (10.6.8);de;kml:2.2;client:Free;type:default)";
const char S76605[] = "GoogleEarth/6.1.0.5001(Macintosh;Mac OS X (10.6.8);fr;kml:2.2;client:Free;type:default)";
const char S76606[] = "Mozilla/5.0 (Series40; NokiaX3-02.5/06.73; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.1.21.8";
const char S76607[] = "Mozilla/5.0 (Series40; NokiaX3-02/07.16; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.1.21.8";
const char S76608[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A500 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S76609[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo A500/S100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76610[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A500 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76611[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A500 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S76612[] = "Mozilla/5.0 (Java; U; en-in; mauie800) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S76613[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; E10i Build/3.0.1.A.0.145; MiniCM7-2.2.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76614[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaN97)/UC Browser8.5.0.183/50/352/UCWEB Mobile";
const char S76615[] = "iBrowser/2.5 (NokiaE5-00;SymbianOS/9.3 Series60/3.2)";
const char S76616[] = "Nokia302/5.0 (14.78) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia302) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S76617[] = "SAMSUNG-GT-E2350B Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.29475/29.3417; U; en) Presto/2.8.119 Version/11.10";
const char S76618[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; Moii E801 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76619[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Moii E801 Build/IMM76I)";
const char S76620[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Spice Mi-530 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76621[] = "Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.8.1.24) Gecko/X Firefox/3.6";
const char S76622[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-S5660 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76623[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-S5660 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76624[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-S5660 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76625[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Z7C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76626[] = "Mozilla/5.0 (Linux; Android 4.0.3; E350 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76627[] = "Mozilla/5.0 (Linux; Android 4.0.4; E350 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76628[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; Karbonn_A6) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S76629[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A6) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S76630[] = "Mozilla/5.0 (Linux; Android 4.1.1; WM8850-mid Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76631[] = "Mozilla/5.0 (Linux; Android 4.1.1; WM8850-mid Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S76632[] = "Mozilla/5.0 (Linux; Android 4.1.1; WM8850:X6-10V Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S76633[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A523 Safari/8536.25";
const char S76634[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A403 Safari/8536.25";
const char S76635[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S76636[] = "SonyEricssonZ310iv/R1JC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S76637[] = "SonyEricssonZ310a/R8BA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S76638[] = "SonyEricssonZ310i/R6BC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S76639[] = "LG-GD710/V10f; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.14812; U; en) Opera 9.50";
const char S76640[] = "LG-C320/V100 Obigo/Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S76641[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.52.7 (KHTML, like Gecko) Fluid/0.9.6 Safari/534.52.7";
const char S76642[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Fluid/0.9.6 Safari/534.57.2";
const char S76643[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Fluid/0.9.6 Safari/533.21.1";
const char S76644[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.50 (KHTML, like Gecko) Fluid/0.9.6 Safari/534.50";
const char S76645[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.51.22 (KHTML, like Gecko) Fluid/0.9.6 Safari/534.51.22";
const char S76646[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.55.3 (KHTML, like Gecko) Fluid/0.9.6 Safari/534.55.3";
const char S76647[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) Fluid/0.9.6 Safari/533.20.25";
const char S76648[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; nl-nl) AppleWebKit/532.3+ (KHTML, like Gecko) Fluid/0.9.6 Safari/532.3+";
const char S76649[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.58.2 (KHTML, like Gecko) Fluid/0.9.6 Safari/534.58.2";
const char S76650[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.59.8 (KHTML, like Gecko) Fluid/0.9.6 Safari/534.59.8";
const char S76651[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s3653) UCBrowser8.2.0.132/69/352/UCWEB Mobile";
const char S76652[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s3653) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S76653[] = "samsung-gt-s3653/UCWEB8.2.0.116/69/352";
const char S76654[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.7.5) Gecko/X Netscape/8.1.3";
const char S76655[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:12.0) Gecko/X Firefox/12.0 AlexaToolbar/alxf-2.17";
const char S76656[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ELONEX765ET Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76657[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/528.16 (KHTML, like Gecko) Stainless/0.5.3 Safari/525.20.1";
const char S76658[] = "JUC (Linux; U; 2.3.7; zh-cn; LT22i; 540*960) UCWEB7.9.0.94/139/352";
const char S76659[] = "JUC (Linux; U; 4.0.4; zh-cn; LT22i; 540*960) UCWEB7.9.0.94/139/444";
const char S76660[] = "OneBrowser/3.1 (NokiaC2-00/03.45)";
const char S76661[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CloudPad 700d Build/FSV02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76662[] = "WhatsApp/2.8.5023 Android/3.1 Device/samsung-GT-P7500";
const char S76663[] = "Mozilla/5.0 (compatible; MSIE 9.0; Win3.11; Trident/5.0)";
const char S76664[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GiONEE_Dream_D1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76665[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (DROID RAZR, 42, 1.9.1)";
const char S76666[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; L-04E Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76667[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76668[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S76669[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S76670[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; GOCLEVER TAB M813G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76671[] = "Mozilla/5.0 (Linux; Android 4.1.1; C1504 Build/11.3.A.0.47) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76672[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaN97 mini) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S76673[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire C Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S76674[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN79) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S76675[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Opera/9.5) HTC-PPC6850 BM BMC";
const char S76676[] = "HTC_Touch_Pro_T7272 Opera/9.50 (Windows NT 5.1; U; en)";
const char S76677[] = "HTC_Touch_Pro_T7272 Opera/9.50 (Windows NT 5.1; U; zh-tw)";
const char S76678[] = "Swisscom/1.0/HTC_Touch_Pro/ Opera/9.50 (Windows NT 5.1; U; de)";
const char S76679[] = "HTC_Touch_Pro_T7272 Opera/9.50 (Windows NT 5.1; U; pt)";
const char S76680[] = "HTC_Touch_Pro_T7272 Opera/9.50 (Windows NT 5.1; U; nl)";
const char S76681[] = "HTC_Touch_Pro_T7272 Opera/9.50 (Windows NT 5.1; U; it)";
const char S76682[] = "Vodafone/1.0/HTC_Touch_Pro Opera/9.50 (Windows NT 5.1; U; en)";
const char S76683[] = "HTC_Touch_Pro_T7272 Opera/9.50 (Windows NT 5.1; U; nb)";
const char S76684[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; E400 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S76685[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; E400 Build/ECLAIR)";
const char S76686[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; E400 Build/ECLAIR)";
const char S76687[] = "SAMSUNG-GT-S5230C/1.0 SHP/R5 Release/02.01.2009 Browser/NetFront3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S76688[] = "Mozilla/5.0 (Linux; U; Android 2.2; it-it; TT101 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76689[] = "HTC_Maple_S520 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S76690[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; rv:1.7.3) Gecko/X Waterunicorn/0.10 StumbleUpon/1.998";
const char S76691[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.4b) Gecko/X Mozilla Firebird/0.5+ StumbleUpon/1.63";
const char S76692[] = "Mozilla/3.0 (compatible; WebMon 1.0.12; Windows XP)";
const char S76693[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; GSmart G1345 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76694[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ANS-01 Build/2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76695[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ANS-01 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76696[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ANS-01 Build/RK2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76697[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-S710d/4.09.1401.0) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76698[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-S710d/4.08.1401.7) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76699[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC S710d) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76700[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC S710d Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S76701[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC S710d Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S76702[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-S710d/4.10.1401.2) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76703[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; vivo E1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76704[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; vivo E1 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S76705[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76706[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P5113 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76707[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76708[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76709[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76710[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-P5113 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76711[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; K-Touch_W650 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76712[] = "Mozilla/5.0 (Java; U; en-us; nokian73-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile";
const char S76713[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia701) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S76714[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo Y3t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76715[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo Y3t Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S76716[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; vivo Y3t Build/IMM76D)";
const char S76717[] = "Mac OS X/10.8.3 (12D78); ExchangeWebServices/3.0 (157); Mail/6.3 (1503)";
const char S76718[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-03E Build/10.1.E.0.269) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S76719[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I957 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S76720[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957R Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S76721[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957M Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S76722[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S76723[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8160 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76724[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8160P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76725[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus S Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76726[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S76727[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710VMUB Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S76728[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710BST Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S76729[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I497 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S76730[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A390_ROW Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S76731[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I747 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.17 Mobile Safari/537.36";
const char S76732[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I747 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.30 Mobile Safari/537.36";
const char S76733[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L710 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S76734[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-P500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S76735[] = "Mozilla/5.0 (compatible; Teleca Q7; Brew 3.1.5; U; en) 240X400 LGE UX840";
const char S76736[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.7.4; U; Edition MAS; en) Presto/2.10.229 Version/11.63";
const char S76737[] = "Nokia200/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia200/UC Browser7.9.0.102/69/352";
const char S76738[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE52-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile";
const char S76739[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76740[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F120S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76741[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-I9300 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S76742[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9300 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S76743[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; EVOPad_R8010 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76744[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; BlissR8015 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76745[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R801T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76746[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-Th; OPPOR8015 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76747[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Celkon A 200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76748[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Edison 3G Build/1.1.11-1015 X-15:24) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S76749[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:20.0) Gecko/X Thunderbird/20.0";
const char S76750[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-STYLE Q 5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76751[] = "Mozilla/5.0 (Linux; Android 4.1.2; Sensation Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76752[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Sensation Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76753[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U807 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S76754[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U807 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76755[] = "Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U807 Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S76756[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ZTE U807 Build/IMM76D)";
const char S76757[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L300 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S76758[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A21 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S76759[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A21 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S76760[] = "GT-N7102 Linux/3.0.13 Android/4.1.9 Release/02.01.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S76761[] = "Mozilla/5.0 (Linux; Android 4.1.1; M532 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S76762[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E977 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S76763[] = "SAMSUNG-SGH-J750AFHA1/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S76764[] = "SAMSUNG-SGH-J750BVGG1/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S76765[] = "SAMSUNG-SGH-J750MJHL1/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S76766[] = "SAMSUNG-SGH-J750AFHC1/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S76767[] = "SAMSUNG-SGH-J750MJGK4/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S76768[] = "SAMSUNG-SGH-J750/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S76769[] = "SAMSUNG-SGH-J750BNGI4/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S76770[] = "SAMSUNG-SGH-J750MJGJ4/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S76771[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.2.15 Version/10.10";
const char S76772[] = "NokiaN73/UC Browser7.9.0.102/28/355/UCWEB";
const char S76773[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; HTC Vision Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76774[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Vision Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76775[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Vision Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76776[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Vision Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76777[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Vision Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76778[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Vision Build/IMM76L)";
const char S76779[] = "Mozilla/7.0 (Windows; U; Windows NT 5.1; pl; rv:1.7.1.7) Gecko/X Firefox/3.7.7 (.NET CLR 3.2.37777)\"";
const char S76780[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Micromax A80 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76781[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; A80 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76782[] = "MQQBrowser/38 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S76783[] = "MQQBrowser/381 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S76784[] = "MQQBrowser/38 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3";
const char S76785[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SPH-D710, 42, 1.9.1)";
const char S76786[] = "Mozilla/5.0 (Linux; Android 4.0.4; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S76787[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaE7-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S76788[] = "Mozilla/5.0 (Linux; Android 4.0.4; JY-G3 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S76789[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S76790[] = "Mozilla/5.0 (Windows NT 5.1; rv:21.0) Gecko/X Firefox/21.0 slurp";
const char S76791[] = "Mozilla/5.0 (Windows NT 5.1; rv:21.0) Gecko/X Firefox/21.0 slurp slurp";
const char S76792[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Android2.x) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S76793[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT875 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S76794[] = "Mozilla/5.0 (Linux; Android 4.0.4; A10 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S76795[] = "Mozilla/5.0 (Linux; Android 4.1.1; Lenovo S890 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S76796[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia701) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S76797[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; OPPOT703 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76798[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; T703 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S76799[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; T703 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S76800[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; T703 Build/GRJ22)";
const char S76801[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.1b3) Gecko/X Firefox/3.1b3";
const char S76802[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; KU9500 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76803[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SU950 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76804[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; KU9500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76805[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SU950 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76806[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; YL-Coolpad_5820/5820_V6.20; 320*480; CTC/2.0) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S76807[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; YL-Coolpad_5820/5820_V3.50; 320*480; CTC/2.0) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S76808[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; YL-Coolpad_5820/5820_V5.23; 320*480; CTC/2.0) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S76809[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad 5820 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76810[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; YL-Coolpad_5820/v1.0; 320*480) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S76811[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; YL-Coolpad_5820/5820_V6.10; 320*480; CTC/2.0) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S76812[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; YL-Coolpad_5820/5820_V5.70; 320*480; CTC/2.0) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S76813[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad 5820 Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S76814[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T320e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76815[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T320e Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S76816[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19";
const char S76817[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19";
const char S76818[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19";
const char S76819[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19";
const char S76820[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19";
const char S76821[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76822[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; vivo S9 Build/IMM76D)";
const char S76823[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S9 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S76824[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MK808 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76825[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; MK808 Build/JRO03H)";
const char S76826[] = "OneBrowser/3.1 (NokiaC1-01/06.15)";
const char S76827[] = "Mozilla/5.0 (Linux; Android 4.0.4; Philips W536 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S76828[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Tablet Hexen Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76829[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Hexen10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76830[] = "Mozilla/5.0 (Linux; Android 4.0.4; Event Build/IML77) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S76831[] = "Lenovo-A800/S100 Linux/3.0.13 Android/4.0 Release/07.19.2012 Browser/AppleWebKit533.1 Profile/ Configuration/ Safari/533.1";
const char S76832[] = "Lenovo A800/S100 Linux/3.0.13 Android/4.0.3 Release/12.12.2011 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30";
const char S76833[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B146 Safari/7534.48.3 UCBrowser8.9.0.238/44/355";
const char S76834[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B144 Safari/7534.48.3 UCBrowser8.9.0.238/44/355";
const char S76835[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B329 Safari/7534.48.3 UCBrowser8.9.0.238/44/355";
const char S76836[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B142 Safari/7534.48.3 UCBrowser8.9.0.238/44/355";
const char S76837[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B143 Safari/7534.48.3 UCBrowser8.9.0.238/44/355";
const char S76838[] = "Mozilla/5.0 (Linux; Android 4.0.4; Desire HD Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S76839[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire HD Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S76840[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire HD Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S76841[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/6.1 Mobile/9B176 Safari/7534.48.3";
const char S76842[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.1 Mobile/10B329 Safari/8536.25";
const char S76843[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.1 Mobile/10B350 Safari/8536.25";
const char S76844[] = "Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80 COBALT Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S76845[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I757M Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76846[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9205 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76847[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; 3G7334i Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76848[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L720 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S76849[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-gb; GT-I9505 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltexx";
const char S76850[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9505 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltexx";
const char S76851[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-gb; GT-I9505 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltexx";
const char S76852[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F240L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S76853[] = "Mozilla/5.0 (Linux; Android 4.1.2; VS870 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S76854[] = "SAMSUNG-SGH-F700/F700XXHB1 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S76855[] = "SAMSUNG-SGH-F700-Vodafone/F700ACGK2 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S76856[] = "Nokia6125/2.0 (03.71) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S76857[] = "LGE-AX310/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S76858[] = "LGE-LW310/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S76859[] = "SAMSUNG-SCH-W169/W169DDDK3 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S76860[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Huawei 858 Build/HuaweiU8160) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76861[] = "POLARIS/6.1 (BREW 3.1.5; U; en-us; LG; POLARIS/6.1/WAP) Boost LG102 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S76862[] = "Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; IS01 Build/SA180) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S76863[] = "Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; IS01 Build/S9200) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S76864[] = "Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; IS01 Build/SC030) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S76865[] = "Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; IS01 Build/S2151) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S76866[] = "Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; IS01 Build/S3082) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S76867[] = "Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; IS01 Build/S2151) AppleWebKit/528.5 (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S76868[] = "Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; IS01 Build/S9210) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S76869[] = "Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; IS01 Build/S6191) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S76870[] = "Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; IS01 Build/S5090) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S76871[] = "HTC-ST7377/1.59.502.3 (67150) Opera/9.50 (Windows NT 5.1; U; en)";
const char S76872[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; PD_Novel Build/MID) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76873[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; PD_Novel Build/MID)";
const char S76874[] = "Mozilla/5.0 (Windows NT 6.0; rv:12.0) Gecko/X Firefox/12.0 PaleMoon/12.0";
const char S76875[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:12.0) Gecko/X Firefox/12.0-x64 PaleMoon/12.0-x64";
const char S76876[] = "SAMSUNG-GT-I9008L_TD/1.0 Android/2.2.1 Release/12.15.2010 Browser/AppleWebKit533.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S76877[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; eZee' Tab702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76878[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-pt; eZee'Tab702 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76879[] = "Mozilla/5.0 (Linux; Android 4.0.4; M9000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S76880[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5360 Build/GINGERBREAD; 240*320) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444";
const char S76881[] = "K-Touch_T760_TD/T760_MocorDroid2.2_W11.41_V1.00 Release/06.26.2012 Mozilla/5.0 (Linux; U; Android 2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76882[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 Sleipnir/2.2m";
const char S76883[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3 Sleipnir/2.2m";
const char S76884[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550 Safari/7534.48.3 Sleipnir/2.2m";
const char S76885[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Fly_IQ450 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76886[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Dialog i43 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76887[] = "Mozilla/5.0 (compatible; MSIE 10.0; Win 9x 4.90; Trident/4.0)";
const char S76888[] = "Mozilla/5.0 (compatible; MSIE 10.0; Win98; Trident/4.0)";
const char S76889[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Infinix X450 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76890[] = "Mozilla/5.0 (Series40; Nokia6303iclassic/10.80; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S76891[] = "Dolphin http client/9.4.1(208) (Android)";
const char S76892[] = "Dolphin http client/9.4.2(212) (Android)";
const char S76893[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC_T528w Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76894[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC T528w Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76895[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC T528w Build/JRO03H)";
const char S76896[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; HS-U950 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76897[] = "SAMSUNG-GT-S5550/JCJB3 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S76898[] = "SAMSUNG-S5550U SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S76899[] = "DoCoMo/2.0 N02A(c100;TB;W24H16)";
const char S76900[] = "DoCoMo/2.0 N02C(c500;TB;W24H16)";
const char S76901[] = "SAMSUNG-GT-B7610/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S76902[] = "Vodafone/1.0/SAMSUNG-GT-B7610/NBUIK3 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S76903[] = "Vodafone/1.0/SAMSUNG-GT-B7610/NBUIK1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S76904[] = "Vodafone/1.0/SAMSUNG-GT-B7610/NBUIL1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S76905[] = "Mozilla/5.0 (Windows NT 4.0) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S76906[] = "Mozilla/5.0 (Windows NT 6.0; rv:10.0.1) Gecko/X Thunderbird/10.0.1 Lightning/1.2.1";
const char S76907[] = "Mozilla/5.0 (Windows NT 5.1; WOW64; rv:10.0.2) Gecko/X Firefox/10.0.2 SeaMonkey/2.7.2";
const char S76908[] = "Mozilla/5.0 (Windows NT 5.1; rv:10.0.2) Gecko/X SeaMonkey/2.7.2";
const char S76909[] = "Mozilla/5.0 (Windows NT 5.2; rv:10.0.1) Gecko/X Firefox/10.0.1 SeaMonkey/2.7.1";
const char S76910[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; vivo V1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76911[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; vivo V1 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S76912[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; vivo V1 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S76913[] = "JUC(Linux;U;Android2.3.3;Zh_cn;HTC Wildfire S A510e;320*480;)UCWEB7.8.0.95/139/355";
const char S76914[] = "JUC(Linux;U;Android2.3.5;Zh_cn;HTC Wildfire S A510e;320*480;)UCWEB7.8.0.95/139/355";
const char S76915[] = "Mozilla/5.0(Linux;U;Android2.3.3;Zh_cn;HTC Wildfire S A510e;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S76916[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_IncredibleS_S710e; zh-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S76917[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_IncredibleS_S710e; en-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S76918[] = "Mozilla/5.0 (Linux; Android 4.0.4; S7 Build/Smartfren.andromaxtab7.0.v.37.3) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S76919[] = "Mozilla/5.0 (Linux; Android 4.0.3; S7 Build/Smartfren.andromaxtab7.0.v.27.3) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S76920[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; E620 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76921[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-STYLE Q6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76922[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; FPT HD Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76923[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; ATLANTIS Build/NGMNGM_ATLANTIS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76924[] = "OneBrowser/3.1 (Nokia201/11.81)";
const char S76925[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10_5_8) AppleWebKit/537.26+ (KHTML, like Gecko) Version/6.0.2 Safari/536.26.17";
const char S76926[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/534.50.2 (KHTML, like Gecko) (like Version/6.0 Safari/534.50.2) Roccat/3.0 Roccat/534.50.2";
const char S76927[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10_5_8) AppleWebKit/536.30.1 (KHTML, like Gecko) Version/6.0.5 Safari/536.30.1";
const char S76928[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Andi 3.5i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76929[] = "FONE 500 Linux/3.0.13 Android/4.0.4 Release/02.27.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S76930[] = "Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabS2109A-F Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S76931[] = "Mozilla/5.0 (Linux; Android 4.1.2; A500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S76932[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIANokia N81) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S76933[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian81-3) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S76934[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIANokia N81 8GB) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S76935[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaN81) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S76936[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320T Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S76937[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S76938[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8190N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb";
const char S76939[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_ov5647 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76940[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_9p017 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76941[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_253_6628 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76942[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_253 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76943[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_5640 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S76944[] = "Mozilla/5.0 (Linux; Android 4.1.1; ONE TOUCH 4011X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S76945[] = "Mozilla/5.0 (Linux; Android 4.1.1; ONE TOUCH 4011X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S76946[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6102 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S76947[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-L40G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S76948[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S76949[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2S Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S76950[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2SC Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S76951[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2A Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S76952[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2C Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S76953[] = "Mozilla/5.0 (Linux; Android 4.2.1; iOCEAN X7 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S76954[] = "Mozilla/4.0 (BREW 3.1.5; U; en-us; Samsung; PLS_M330; POLARIS/6.1/WAP) MMP/2.0 Configuration/CLDC-1.1";
const char S76955[] = "Mozilla/4.0 (BREW 3.1.5; U; en-us; Samsung; SPH_M330; POLARIS/6.1/WAP) MMP/2.0 Configuration/CLDC-1.1";
const char S76956[] = "NokiaN91/3.0 (1.10.030) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S76957[] = "NokiaN91-2/3.0 (3.10.023) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S76958[] = "NokiaN91-1/3.0 (2.10.013) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S76959[] = "NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-00/UC Browser7.9.0.102/69/352";
const char S76960[] = "NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/69/352";
const char S76961[] = "NokiaC5-00";
const char S76962[] = "MicromaxX285/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S76963[] = "System Center Operations Manager 2007 6.0.6278.100";
const char S76964[] = "System Center Operations Manager 2007 6.1.7221.81";
const char S76965[] = "System Center Operations Manager 2007 6.1.7221.0";
const char S76966[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) 1Password/3.6.5/X (like Mobile/8C148 Safari/6533.18.5)";
const char S76967[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) 1Password/3.6.5/X (like Mobile/8C148 Safari/6533.18.5)";
const char S76968[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) iCab/5.0 Safari/533.16";
const char S76969[] = "Mozilla/5.0 (Linux; Android 4.0.3; T05A Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S76970[] = "Mozilla/5.0 (Linux; Android 4.0.4; T05A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S76971[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S5660 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76972[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S5660 Build/IMM76L; CyanogenMod-CyanogenMod9 RC1 by MaclawStudios) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76973[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; DaVinci Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S76974[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-pt; moche smart a8 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S76975[] = "Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD+ Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S76976[] = "Mozilla/5.0 (Linux; Android 4.0.3; FZ-A1B Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S76977[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9070 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S76978[] = "HTCT329t_TD/1.0 Android/4.0 release/2012 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S76979[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; TOOKY A9PLUS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76980[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; TOOKY A19PLUS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76981[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; TOOKY T83 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76982[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; TOOKY A981+ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76983[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; TOOKY T88 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S76984[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_918N) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S76985[] = "UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; GT-I9000) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S76986[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI 1S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S76987[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI 1SC Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S76988[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Lenovo A390_ROW Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33482";
const char S76989[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-N7000 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/n7000";
const char S76990[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-N7000 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/n7000";
const char S76991[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-N7000 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/n7000";
const char S76992[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36 slurp";
const char S76993[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.62 Safari/537.36 slurp";
const char S76994[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36 slurp";
const char S76995[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36 slurp";
const char S76996[] = "Mozilla/5.0 (Linux; Android 4.2.1; C2 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S76997[] = "Nokia6750-1c/2.0(ATT.5.93) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S76998[] = "Nokia6750-1b/2.0(ATT.4.99) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S76999[] = "LGE-MT310/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S77000[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/5.2.1.1588 Safari/532.4";
const char S77001[] = "GoogleEarth/5.2.1.1588(Windows;Microsoft Windows (5.1.2600.3);en-US;kml:2.2;client:Free;type:default)";
const char S77002[] = "GoogleEarth/5.2.1.1588(Windows;Microsoft Windows (5.1.2600.3);de-DE;kml:2.2;client:Free;type:default)";
const char S77003[] = "Mozilla/5.0 (Linux; U; Android 2.2; vi-vn; Q-mobile S10 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77004[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; Q-mobile S10 Build/FRF91)";
const char S77005[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Iron/9.0.600.2 Chrome/9.0.600.2 Safari/534.13";
const char S77006[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Iron/9.0.600.2 Chrome/9.0.600.2 Safari/534.13";
const char S77007[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Iron/9.0.600.2 Chrome/9.0.600.2 Safari/534.13";
const char S77008[] = "Opera/9.80 (S60; SymbOS; Opera Mobi/SYB-X; U; fa) Presto/2.8.149 Version/11.10";
const char S77009[] = "Liferea/1.6.4 (Linux; en_US.UTF-8; http://liferea.sf.net/)";
const char S77010[] = "Liferea/1.6.5 (Linux; en_US.utf8; http://liferea.sf.net/)";
const char S77011[] = "Liferea/1.6.6b (Linux; en_US.UTF-8; http://liferea.sf.net/)";
const char S77012[] = "Liferea/1.6.2 (Linux; en_US.utf8; http://liferea.sf.net/)";
const char S77013[] = "Liferea/1.6.4 (Linux; en_US.utf8; http://liferea.sf.net/)";
const char S77014[] = "Liferea/1.6.3 (Linux; en_US.utf8; http://liferea.sf.net/)";
const char S77015[] = "Liferea/1.6.2 (Linux; en_US.UTF-8; http://liferea.sf.net/)";
const char S77016[] = "Liferea/1.6.4 (Linux; fr_FR.UTF-8; http://liferea.sf.net/)";
const char S77017[] = "Liferea/1.6.5 (Linux; fr_FR.utf8; http://liferea.sf.net/)";
const char S77018[] = "Liferea/1.6.4 (Linux; fr_FR.utf8; http://liferea.sf.net/)";
const char S77019[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; vivo S3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77020[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; vivo S3 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S77021[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; LNV-Lenovo_S850e/S-3-03; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77022[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S850e Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S77023[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S850e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77024[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S850e Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S77025[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar_Touch_S02i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77026[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar Touch S02 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77027[] = "NokiaC3-01/5.0 (07.51) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac3-01) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S77028[] = "NokiaC3-01.5/5.0 (06.75) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac3-01.5) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S77029[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/7534.48.3 Sleipnir/2.2m";
const char S77030[] = "Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.2m";
const char S77031[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.2m";
const char S77032[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; W30 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77033[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730M Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77034[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI 1S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77035[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI 1S Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S77036[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI 1SC Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77037[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9300 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S77038[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9300 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S77039[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 720)";
const char S77040[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 720; ANZ885)";
const char S77041[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9003 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S77042[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E430 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77043[] = "Mozilla/5.0 (Linux; Android 4.0.4; A10 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77044[] = "Mozilla/5.0 (Linux; Android 4.0.4; A10 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S77045[] = "Mozilla/5.0 (Linux; Android 4.1.1; SC-03E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77046[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Sensation Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77047[] = "Mozilla/5.0 (Linux; Android 4.1.2; Sensation Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77048[] = "Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77049[] = "Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S77050[] = "Mozilla/5.0 (Linux; Android 4.0.4; mid-A13 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77051[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7102 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S77052[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7102 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S77053[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Orbit Z50 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77054[] = "Mozilla/5.0 (Linux; Android 4.1.2; PMP7280C3G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S77055[] = "Mozilla/5.0 (iPad; U; CPU OS 7_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/11A465 Safari/7534.48.3";
const char S77056[] = "Mozilla/5.0 (iPad; U; CPU OS 7_0_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/11A501 Safari/7534.48.3";
const char S77057[] = "Nokia6103/2.0 (04.90) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77058[] = "Facebook 4110.0 (iPhone; iPhone OS 4.2.1; en_US)";
const char S77059[] = "Facebook 4110.0 (iPhone; iPhone OS 4.2.6; en_US)";
const char S77060[] = "Facebook 4110.0 (iPhone; iPhone OS 4.2.10; en_US)";
const char S77061[] = "Facebook 4110.0 (iPhone; iPhone OS 4.2.8; en_US)";
const char S77062[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch_Diamond2_T5353; Windows Phone 6.5)";
const char S77063[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch_Diamond2_T5360; Windows Phone 6.5)";
const char S77064[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch_Diamond2_T5353; Windows Phone 6.5.3.5)";
const char S77065[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XCD 28 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S77066[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XCD 28 Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S77067[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Comodo_Dragon/8.0.0.4 Chrome/8.0.552.215 Safari/534.10";
const char S77068[] = "SonyEricssonK800i/R1ED Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 (Mediobot/1.0 +http://bot.medio.com)";
const char S77069[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; MW0812 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77070[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; MW0812_V2.0 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77071[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; en) Presto/2.10.229 Version/11.64";
const char S77072[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; miTab LION Build/ICS.g08refem611.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77073[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; VICTORY Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77074[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Wynncom_G1 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77075[] = "Mozilla/5.0 (Linux; U; Android 2.3;en-us;[Wynncom G11] build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77076[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Desire C Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S77077[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A1+ Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S77078[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6810 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S77079[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL_one_touch_995 Build/IceCreamSandwich) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77080[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL_one_touch_995S Build/IceCreamSandwich) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77081[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One X Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S77082[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S77083[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/9B206 Safari/7534.48.3";
const char S77084[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/9B176 Safari/7534.48.3";
const char S77085[] = "Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S77086[] = "Mozilla/5.0 (Linux; Android 4.2.2; A1-811 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S77087[] = "MOT-WX345/03.38.01 Obigo/Q03C MMP/2.0";
const char S77088[] = "NokiaX6-00";
const char S77089[] = "NokiaX6-00/UCBrowser7.7.0.81/50/999";
const char S77090[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Iron/5.0.381.0 Chrome/5.0.381 Safari/533.4";
const char S77091[] = "Mozilla/5.0 (iPhone) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S77092[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/534.10 (KHTML, like Gecko) RockMelt/0.8.40.147 Chrome/8.0.552.231 Safari/534.10";
const char S77093[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.7 (KHTML, like Gecko) RockMelt/0.8.36.116 Chrome/7.0.517.44 Safari/534.7";
const char S77094[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/534.7 (KHTML, like Gecko) RockMelt/0.8.36.116 Chrome/7.0.517.44 Safari/534.7";
const char S77095[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.10 (KHTML, like Gecko) RockMelt/0.8.40.147 Chrome/8.0.552.231 Safari/534.10";
const char S77096[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.3 (KHTML, like Gecko) RockMelt/0.8.34.820 Chrome/6.0.472.63 Safari/534.3";
const char S77097[] = "Mozilla/5.0 (Sonim-XP3400/E343B_1900B04_B_N1900;U;REX/1.0;BREW/3.1.5;240*320;CTC/2.0) POLARIS/6.2";
const char S77098[] = "Mozilla/5.0 (Sonim-XP3400/E343B_2100B00_N1900;U;REX/1.0;BREW/3.1.5;240*320;CTC/2.0) POLARIS/6.2";
const char S77099[] = "Mozilla/5.0 (Sonim-XP3400/E343B_1900B03_B_N1900;U;REX/1.0;BREW/3.1.5;240*320;CTC/2.0) POLARIS/6.2";
const char S77100[] = "Mozilla/5.0 (Series40; NokiaX2-05/08.60a; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15";
const char S77101[] = "Mozilla/5.0 (Linux; Android 4.0.4; A210 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S77102[] = "Opera/9.80 (Windows 95; U; ru) Presto/2.10.289 Version/12.02";
const char S77103[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar Touch Kem 350 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77104[] = "JUC (Linux; U; 4.0.4; zh-cn; MT25i; 480*854) UCWEB7.9.0.94/139/444";
const char S77105[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EGL26BL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77106[] = "Mozilla/5.0 (Linux; Android 4.2.1; Desire HD Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77107[] = "Mozilla/5.0 (Linux; Android 4.2.2; Desire HD Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77108[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V889M Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77109[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E16i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S77110[] = "UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; GT-I9100) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S77111[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I9100G) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S77112[] = "Mozilla/5.0 (Linux; Android 4.0.3; Transformer Prime TF201 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S77113[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) Nokia5233";
const char S77114[] = "Nokia5233;c2VvLWNyYXdsZXI=";
const char S77115[] = "Mozilla/5.0 (Linux; Android 4.0.4; WIKO-CINK SLIM Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77116[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ607-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S77117[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGPT12 Build/TISU0105) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S77118[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGPT13 Build/TID0129) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S77119[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly s Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S77120[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire 600 dual sim Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S77121[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire 600 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S77122[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-S7270 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77123[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-S7270L Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S77124[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-S7270L Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77125[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-S7270 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S77126[] = "SAMSUNG GT-S3330 Opera/9.80 (J2ME/MIDP; Opera Mini/7.1.32840/31.1475; U; en) Presto/2.8.119 Version/11.10";
const char S77127[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-L40G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S77128[] = "Mozilla/5.0 (Linux; Android 4.3; HTC Sensation Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S77129[] = "Mozilla/4.0 (compatible; MSIE 5.0; Series90/1.1 Nokia7710/4.10.0 Profile/MIDP-2.0 Configuration/CLDC-1.0)";
const char S77130[] = "XV6875.1 Opera/9.70 (Windows NT  5.1; U; en)";
const char S77131[] = "Mozilla/5.0 (HW-HUAWEI_C8500/C8500V100R001C92B234;U;Android/2.1;240*320;CTC/2.0) AppleWebKit/530.17 Mobile Safari/530.17";
const char S77132[] = "LGE-UN150/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S77133[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/530.9 (KHTML, like Gecko) Iron/2.0.178.0 Safari/530.9";
const char S77134[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/530.1 (KHTML, like Gecko) Iron/2.0.168.0 Safari/530.1";
const char S77135[] = "Mozilla/5.0 (X11; Linux i686; rv:11.0) Gecko/X Thunderbird/11.0.1";
const char S77136[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; OnePAD 970 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77137[] = "Mozilla/5.0 (Linux; Android 4.0.3; PC906 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S77138[] = "Mozilla/5.0 (Linux; Android 4.0.4; PC906 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S77139[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; MID7022 Build/GreenDemon_v1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77140[] = "Mozilla/5.0 (Linux; Android 4.0.3; Sony Tablet P Build/TISU0144) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S77141[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SP-140 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77142[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; UMI X1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77143[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.5.3.X (Second Life Release; default skin) Safari/533.3";
const char S77144[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P870 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77145[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6312 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77146[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6312 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S77147[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT18i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S77148[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; LT18i Build/4.1.A.0.562) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S77149[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire C Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S77150[] = "Mozilla/5.0 (Linux; Android 4.1.1; CINK PEAX Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77151[] = "Mozilla/5.0 (Linux; Android 4.1.2; CINK PEAX Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77152[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BLU Quattro 5.7HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77153[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S77154[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; GT-I8262B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77155[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC J Z321e Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S77156[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac6-00) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile  3gpp-gba";
const char S77157[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC6-00) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S77158[] = "Mozilla/5.0 (Linux; Android 4.1.1; C1605 Build/11.3.A.2.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S77159[] = "MOT-E8/R6713_G_71.03.05R Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 [es-co]";
const char S77160[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.14.1ER BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77161[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.14.1ER_B BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77162[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.01.2DR BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77163[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.02.07R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77164[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.14.0EI BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77165[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/Final Myth Rokr E8 BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77166[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/Final Myth Rokr E8 SP1 BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77167[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.04.06R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77168[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/Nik_Stallion_E8 BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77169[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.14.1AR BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77170[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.14.1FI BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77171[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.01.2ER BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77172[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6718_LIZU_V1.0 BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77173[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.14.18R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77174[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.14.1CR_A BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77175[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.01.8BR BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77176[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.01.2FR BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77177[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.14.1CR BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77178[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8_CMCC/R6718_G_71.10.13R_C BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77179[] = "HTC_Touch Cruise Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S77180[] = "HTC-P3650-orange/PPC; 240x320; OpVer 28.319.2.732 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S77181[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC_P3650/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77182[] = "HTC_Touch_Cruise_T4242 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S77183[] = "HTC_P3650 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S77184[] = "SAMSUNG-SGH-T479B/T479UGKB1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S77185[] = "HTC_P3650 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S77186[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC P3650; OpVer 28.129.15.751";
const char S77187[] = "HTC_P3650t/1.27.621.1 Mozilla/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S77188[] = "HTC-P3650-orange/PPC; 240x320; OpVer 28.129.2.732 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S77189[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC_P3650/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77190[] = "SAMSUNG-GT-C3630/1.0 SHP/VPP/R5 NetFront/3.4 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S77191[] = "HTC_HD2_T8585 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S77192[] = "HD2_T8585 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S77193[] = "PPC; HTC_HD2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S77194[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.2.0.0 Chrome/13.0.782.218 Safari/535.1";
const char S77195[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.2.0.0 Chrome/13.0.782.218 Safari/535.1";
const char S77196[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; nl-nl; MID-S7 Build/GR144) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77197[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Connspeed_AS7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77198[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; epade S7 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S77199[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Hiya Smart S7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77200[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; epade S7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77201[] = "Mozilla/5.0 (Linux; Android 4.0.4; SO-05D Build/7.0.D.1.130) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77202[] = "Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia305/UC Browser8.0.3.107/69/444";
const char S77203[] = "Mozilla/5.0 (Linux; Android 4.0.3; chagall Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S77204[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; INFINITY_LOTUS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77205[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GIONEE INFINITY_LOTUS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77206[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A87) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S77207[] = "Mozilla/5.0 (Linux; Android 4.1.1; ZTE V807 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77208[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S77209[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S7562) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S77210[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Micromax A116 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S77211[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT919 Build/2_290_2012)";
const char S77212[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT919 Build/2_290_2012) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77213[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT919 Build/2_290_2005) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77214[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT919 Build/2_290_2004) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77215[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F240S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S77216[] = "Mozilla/5.0 (Linux; Android 4.2.1; N9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S77217[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT26i Build/6.2.B.0.200) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S77218[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT26i Build/6.2.B.1.96) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S77219[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC One V Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S77220[] = "Mozilla/5.0 (Series40; Nokia501/10.0.2; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S77221[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB300 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77222[] = "Opera/9.80 (Android 4.1.1; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.9.201 Version/11.50";
const char S77223[] = "Opera/9.80 (Android 4.1.2; Linux; Opera Mobi/ADR-X; U; nb) Presto/2.9.201 Version/11.50";
const char S77224[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s3653) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S77225[] = "Nokia303/5.0 (14.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia303) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S77226[] = "iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A11 Build/KarbonnA11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77227[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S77228[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S77229[] = "Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 10 FHD Build/HuaweiMediaPad) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S77230[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LT8036 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77231[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST18i Build/4.1.B.0.587) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S77232[] = "OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; W30 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77233[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Mobile Safari/535.19 Silk-Accelerated=false";
const char S77234[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; myTouch 4G Slide Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77235[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MyTouch 4G Slide Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77236[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/2.0";
const char S77237[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/2.0.1";
const char S77238[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/2.0.2";
const char S77239[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Flipboard/2.0.1";
const char S77240[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Flipboard/2.0.2";
const char S77241[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/2.0.3";
const char S77242[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/2.0.4";
const char S77243[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Flipboard/2.0.4";
const char S77244[] = "Mozilla/5.0 (Linux; Android 4.0.3; U8860 Build/HuaweiU8860) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77245[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I8150 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S77246[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC6435LVW Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/dlx";
const char S77247[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Cherry Sonic  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77248[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; ALCATEL ONE TOUCH 8000D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77249[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5300) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S77250[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9308 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S77251[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Toshiba/TG01/02/ PPC; 480x800";
const char S77252[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Toshiba-TG01-orange/ 02/ PPC; 480x800";
const char S77253[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Toshiba/TG01/ PPC; 480x800";
const char S77254[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Toshiba-TG01-orange/ 01/ PPC; 480x800";
const char S77255[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Toshiba/TG01";
const char S77256[] = "LG-GT500-v10h Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S77257[] = "LG-GT500-v10k Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S77258[] = "LG-GT500-v10j Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S77259[] = "LG-GT500-v10f Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S77260[] = "LG-GT500-v10b Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S77261[] = "LG/GT500/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S77262[] = "LG/GT500-Bouygues/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S77263[] = "LG-GT500/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S77264[] = "LG-GT500-v10e Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S77265[] = "LG-GT500-v10i Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S77266[] = "LG-GT500-v10d Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S77267[] = "LG-GT500-v10c Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S77268[] = "Mozilla/5.0 (Series40; NokiaC2-06/06.52; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.1.21.8.4";
const char S77269[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:12.0) Gecko/X Thunderbird/12.0.1";
const char S77270[] = "Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S77271[] = "Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S77272[] = "Mozilla/5.0 (Windows NT 6.1; rv:2.0.1) Gecko/X Firefox/6.0.1/Nutch-1.4";
const char S77273[] = "Mozilla/5.0 (Linux; Android 4.0.4; MI-ONE Plus Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77274[] = "Mozilla/5.0 (Linux; Android 4.0.4;  MI-ONE Plus Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S77275[] = "Mozilla/5.0 (Linux; Android 4.0.4; MI-ONE Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77276[] = "Mozilla/5.0 (BD-C5500; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5";
const char S77277[] = "Mozilla/5.0 (Stopper; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S77278[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en_US) AppleWebKit/533.19.4 (KHTML, like Gecko) Dreamweaver/12.0.0.5808 Version/5.0.3 Safari/533.19.4";
const char S77279[] = "JUC (Linux; U; 2.3.6; zh-cn; ZTE_N880E; 480*800) UCWEB7.9.3.103/139/32873";
const char S77280[] = "JUC (Linux; U; 4.0.3; zh-cn; ZTE_N880E; 480*800) UCWEB7.9.3.103/139/32873";
const char S77281[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9001 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77282[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E16i Build/2.1.1.A.0.53) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S77283[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S77284[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; HTC Wildfire S A510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S77285[] = "MicromaxX454/MstarBrowser1.0/MIDP2.0Configuration/CLDC1.1";
const char S77286[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE51) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/355/UCWEB Mobile";
const char S77287[] = "SAMSUNG-GT-E2350B/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S77288[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Kurio10 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77289[] = "Spice_M-5900/WapBrowserProfile";
const char S77290[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; TSM-T8i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77291[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N7100 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77292[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I317 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77293[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L900 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77294[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.128; en-us; 9530/4.2.0.128) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S77295[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.179; en-us; 9530/4.2.0.179) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S77296[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.456; en-us; 9530/4.2.0.456) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S77297[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 97 TITANIUMHD Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S77298[] = "Nokia201/2.0 (11.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia201) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S77299[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Explorer A310e Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77300[] = "Mozilla/5.0 (compatible; MSIE 1.0; WinNT3.51; Trident/5.0)";
const char S77301[] = "Mozilla/5.0 (compatible; MSIE 2.0; WinNT3.51; Trident/5.0)";
const char S77302[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8730 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77303[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8730 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S77304[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE BLADE III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77305[] = "Mozilla/5.0 (Linux; Android 4.0.4; Blade III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77306[] = "Mozilla/5.0 (Linux; Android 4.0.4; BLADE III_IL Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77307[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR910L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77308[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR910L Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77309[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; eZee'TAB706 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77310[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5830i Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S77311[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus S Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S77312[] = "Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_E10310 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S77313[] = "Mozilla/5.0 (Linux; Android 4.1.2; Q800 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S77314[] = "Mozilla/5.0 (Linux; Android 4.1.2; WL-101GQC Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S77315[] = "Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300L Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S77316[] = "Mozilla/5.0 (Linux; Android 4.1.2; Transformer Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S77317[] = "Mozilla/5.0 (Linux; Android 4.1.2; Transformer TF101 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S77318[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N7000 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S77319[] = "HP_KB1/0.90 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC;320x240";
const char S77320[] = "HP_KB1/1.02_1.02 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC;320x240";
const char S77321[] = "Nokia6820/2.0 (4.83) Profile/MIDP-1.0 Configuration/CLDC-1.0 AppEngine-Google; (+http://code.google.com/appengine; appid: s~google.com:mobile-checker)";
const char S77322[] = "SonyEricssonW100a/R1CA009 TelecaBrowser/Q07C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77323[] = "Opera/9.80 (Windows 98; U; en) Presto/2.5.24 Version/10.53";
const char S77324[] = "Opera/9.80 (Windows 98; U; en) Presto/2.5.24 Version/10.52";
const char S77325[] = "Opera/9.80 (Windows 98; U; en) Presto/2.5.22 Version/10.51";
const char S77326[] = "Opera/9.80 (Windows 98; U; en) Presto/2.5.24 Version/10.54";
const char S77327[] = "Opera/9.80 (Windows 98; U; en) Presto/2.5.22 Version/10.50";
const char S77328[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.5b) Gecko/X Camino/0.7+";
const char S77329[] = "Mozilla/5.0 (Linux; U; Android 3.0UI; en-in; i7218 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77330[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 Comodo_Dragon/19.0.3.0";
const char S77331[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; X310e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77332[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A760S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77333[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; VORTEX COLOR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77334[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.3.5 Safari/534.34";
const char S77335[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A789 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77336[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S77337[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P6200 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S77338[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S77339[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; ALCATEL ONE TOUCH 918N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S77340[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; ALCATEL_ONE_TOUCH_918N Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S77341[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Qmobile S13 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77342[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P3100 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S77343[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P3100 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S77344[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P3100 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S77345[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; AMAZE S200 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77346[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MyPhone A818g Duo Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77347[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; MyPhone A818g Duo Build/JRO03C)";
const char S77348[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8013 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S77349[] = "Bmobile_AX530 Linux/2.6.35.7 Android/2.3.6 Release/10.10.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1";
const char S77350[] = "Mozilla/5.0 (Linux; Android 4.1.2; Torque Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77351[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-04E Build/10.1.1.D.2.31) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S77352[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S77353[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717R Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S77354[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE Blade III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36";
const char S77355[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE BLADE III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S77356[] = "Mozilla/5.0 (Linux; Android 4.0.4; Blade III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S77357[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7102 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S77358[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7102 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S77359[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) T-Mobile_Atlas";
const char S77360[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; MDA Touch/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77361[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) ASUSP750/1.0";
const char S77362[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Vodafone/1.0/VPA_Touch/2.24.162.52";
const char S77363[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; MDA Touch Plus/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77364[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Vodafone/1.0/SFR_v3650/1.25.163.3";
const char S77365[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) i-mate JASJAM";
const char S77366[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) i-mate 9502; 480x640";
const char S77367[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 480x640; X650/2.0 Profile";
const char S77368[] = "Mozilla/5.0 (Linux; U; Android 1.6; es-es; Orange_Boston Build/DONUT) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S77369[] = "Mozilla/5.0 (Linux; U; Android 1.6; pt-pt; Boston Build/DONUT) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S77370[] = "BlackBerry8703e/4.1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105";
const char S77371[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Google Nexus S Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77372[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus S Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77373[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Nexus S Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77374[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Google Nexus S Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77375[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Google Nexus S Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77376[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Google Nexus S Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S77377[] = "Mozilla/5.0 (Linux; U; Android 2.3; zh-cn; HTCA9188_TD/1.0 Android/2.3 release/4.1.2011 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77378[] = "Mozilla/5.0 (Linux; U; Android 2.3; zh-cn; HTC A9188 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77379[] = "Mozilla/5.0 (Linux; U; Android 2.3; zh-cn; HTC A9188 Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S77380[] = "Mozilla/5.0 (Linux; U; Android 2.3; zh-cn; HTC A9188 Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S77381[] = "NokiaC2-03/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac2-03) UCBrowser8.2.1.144/70/355/UCWEB Mobile";
const char S77382[] = "MicromaxX288/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S77383[] = "JUC(Linux;U;Android2.3.3;Zh_cn;GT-S5660;320*480;)UCWEB7.8.0.95/139/352";
const char S77384[] = "JUC(Linux;U;Android2.3.6;Zh_cn;GT-S5660;320*480;)UCWEB7.8.0.95/139/444";
const char S77385[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.0.1) Gecko/X Firefox/3.0.1 Flock/2.0b2";
const char S77386[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9a1) Gecko/X Firefox/1.6a1 (Swiftfox)";
const char S77387[] = "MicromaxX276/MstarBrowser1.0/MIDP2.0Congiguration/CLDC1.1";
const char S77388[] = "Mozilla/5.0 (Series40; NokiaX3-00/11.00; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S77389[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TAB466EUK Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77390[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; WT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S77391[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:15.0) Gecko/X Firefox/15.0.1 SeaMonkey/2.12.1";
const char S77392[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S3+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77393[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S3+ Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S77394[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9228 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S77395[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9228 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77396[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9228 Build/IMM76D)";
const char S77397[] = "NokiaC3-01/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac3-01) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S77398[] = "OneBrowser/3.1 (Nokia305/03.60)";
const char S77399[] = "YahooMobileFinance/1.0 (Android Finance; 1.1.5) (cdma_spyder;motorola;DROID RAZR;4.0.4/6.7.2-180_DHD-16_M4-31)";
const char S77400[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/99.0 Safari/537.1 YottaaMonitor";
const char S77401[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/99.0 Safari/537.11 YottaaMonitor";
const char S77402[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; ARCHOS 97 XENON Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77403[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; PG86100 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77404[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; PG86100 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77405[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; AMOI N828 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77406[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; AMOI N828 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77407[] = "Mozilla/5.0 (Linux; Android 4.1.1; I10A-LE Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S77408[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26ii Build/6.2.B.1.96) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77409[] = "Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S77410[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; One X Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/evita";
const char S77411[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; One X Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/evita";
const char S77412[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC One X Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/endeavoru";
const char S77413[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-gb; HTC One X Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/endeavoru";
const char S77414[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; One X Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/evita";
const char S77415[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F240S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S77416[] = "Mozilla/5.0 (Linux; Android 4.0.3; M532 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S77417[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5280 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S77418[] = "Motorola-V950 Obigo/Q05A1 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77419[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.2.2.0 Chrome/14.0.835.202 Safari/535.1";
const char S77420[] = "Nokia5233/UCWEB8.3.0.133/50/800";
const char S77421[] = "Mozilla/5.0 (Java; U; en-in; nokia5233) UCBrowser8.3.0.154/69/444/UCWEB Mobile  3gpp-gba";
const char S77422[] = "Mozilla/5.0 (Linux; Android 4.0.4; aokp_tenderloin Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S77423[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Dell V04B Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77424[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Dell Streak Pro Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77425[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; BB EH7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77426[] = "JUC (Linux; U; 2.3.7; zh-cn; MT27i; 480*854) UCWEB7.9.0.94/139/352";
const char S77427[] = "JUC (Linux; U; 4.0.4; zh-cn; MT27i; 480*854) UCWEB7.9.0.94/139/352";
const char S77428[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I800 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77429[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/6.5.27431/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S77430[] = "Lenovo S880i/S100 Linux/3.0.13 Android/4.0.3 Release/12.12.2011 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30";
const char S77431[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7102 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77432[] = "Mozilla/5.0 (Linux; Android 4.1.9; GT-N7102 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77433[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7102 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77434[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S77435[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; Desire HD Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S77436[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S77437[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; 201K Build/117.1.1c00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77438[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; 201K Build/117.1.1c00)";
const char S77439[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; 201K Build/118.0.1f00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77440[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Fly_IQ446 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77441[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; C6503 Build/10.1.A.1.434)";
const char S77442[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; C6503 Build/10.1.A.1.434) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77443[] = "Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Safari/535.19 Silk-Accelerated=false";
const char S77444[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PantechP9070 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77445[] = "Mozilla/5.0 (Linux; Android 4.1.2; PadFone Infinity Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77446[] = "Mozilla/5.0 (Linux; Android 4.1.2; PadFone Infinity Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S77447[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100P Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S77448[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100P Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S77449[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 101G10 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S77450[] = "Mozilla/5.0 (Linux; Android 4.1.2; K1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S77451[] = "Mozilla/5.0 (Linux; Android 4.1.1; Ideapad K1 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S77452[] = "Mozilla/5.0 (Linux; Android 4.1.1; K1 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S77453[] = "Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S77454[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S77455[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P714 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S77456[] = "Mozilla/5.0 (Linux; Android 4.1.2; A111 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S77457[] = "Mozilla/5.0 (Linux; Android 4.1.1; A701 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S77458[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P705 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S77459[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P708g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S77460[] = "SAMSUNG-SGH-F400/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S77461[] = "MICROMAX Q5";
const char S77462[] = "MICROMAXQ5";
const char S77463[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8500L/1.0; U; Bada/1.0; es) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WVGA SMM-MMS/1.2.0 OPN-B";
const char S77464[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Comodo_Dragon/10.0.0.2 Chrome/10.0.648.204 Safari/534.16";
const char S77465[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) RockMelt/0.8.34.841 Chrome/6.0.472.63 Safari/534.3";
const char S77466[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) RockMelt/0.8.36.116 Chrome/7.0.517.44 Safari/534.7";
const char S77467[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) RockMelt/0.8.40.147 Chrome/8.0.552.231 Safari/534.10";
const char S77468[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; myTouch_4G_Slide Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77469[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC myTouch_4G_Slide Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77470[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; myTouch 4G Slide Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77471[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; myTouch_4G_Slide Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77472[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; myTouch_4G_Slide Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77473[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:2.0b2pre) Gecko/X Minefield/4.0b2pre";
const char S77474[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0b4pre) Gecko/X Minefield/4.0b4pre";
const char S77475[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77476[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S77477[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S77478[] = "Mozilla/5.0 (Series40; NokiaX3-02/05.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15";
const char S77479[] = "Mozilla/5.0 (Series40; NokiaX3-02.5/07.32; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.7";
const char S77480[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) iCab/5.0 Safari/533.16";
const char S77481[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77482[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P1000 Build/MIUI)";
const char S77483[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-; GT-P1000 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77484[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P1000 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 ????????? Safari/534.30";
const char S77485[] = "Biblia/2.2.7 (Linux; U; Android 4.0.4; es-ES; GT-P1000 Build/MIUI)";
const char S77486[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; POLYTRON W3430 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77487[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; POLYTRON W3430 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77488[] = "Mozilla/5.0 (Linux; U; Android 4.0; zh-cn; K-Touch W806+ Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77489[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch W806+ Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S77490[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; K-Touch W806+ Build/IMM76I)";
const char S77491[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch W806+ Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77492[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; SonyEricssonU1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S77493[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; Sony EricssonU1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S77494[] = "Mozilla/5.0 (Windows NT 5.1; rv:19.0) Gecko/X Firefox/19.0 slurp";
const char S77495[] = "Mozilla/5.0 (Windows NT 5.1; rv:19.0) Gecko/X Firefox/19.0 slurp slurp";
const char S77496[] = "Mozilla/5.0 (Linux; Android 4.1.2; PadFone Infinity Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S77497[] = "Mozilla/5.0 (Linux; Android 4.1.2; PadFone Infinity Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77498[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9300 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S77499[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT11i Build/4.1.B.0.587) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S77500[] = "Mozilla/5.0 (Linux; Android 4.0.3; MT11i Build/4.1.A.0.562) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S77501[] = "SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj20i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S77502[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; KPN Smart 300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77503[] = "Mozilla/5.0 (Linux; Android 4.2.2; Desire HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77504[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST25i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S77505[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST25i Build/6.0.B.3.188) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S77506[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570I) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S77507[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S77508[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9082) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S77509[] = "NokiaX3-00/5.0 (11.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax3-00) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S77510[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-gb; GT-i9100 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77511[] = "Dalvik/1.6.0 (Linux; U; Android 4.3; GT-I9100 Build/JSS15Q)";
const char S77512[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-gb; GT-I9100 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77513[] = "Mozilla/5.0 (Linux; U; Android 4.3; es-es; GT-i9100 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77514[] = "Mozilla/5.0 (Linux; Android 4.2.1; Transformer Prime TF201 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S77515[] = "Mozilla/5.0 (Linux; Android 4.0.4; Torque Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S77516[] = "SAMSUNG-SGH-i600/WM534 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8)";
const char S77517[] = "SAMSUNG-SGH-i600ORANGE/BVGD2 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8)";
const char S77518[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) SAMSUNG-SGH-i600/1.0";
const char S77519[] = "SAMSUNG-SGH-i600TIM/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8)";
const char S77520[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) SAMSUNG-SGH-i600V/1.0";
const char S77521[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) SAMSUNG-SGH-i600/WM534";
const char S77522[] = "LG-GD900 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S77523[] = "ZTE-G S213/WAP2.0";
const char S77524[] = "ZTE-G-S213/WAP2.0";
const char S77525[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Pyramid Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77526[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Pyramid Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77527[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-tw; HTC_Pyramid Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari";
const char S77528[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_Pyramid Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77529[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-tw; HTC Pyramid Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77530[] = "NokiaN72/5.0819.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.7.1.88/27/444";
const char S77531[] = "Mozilla/5.0 (X11; OpenBSD amd64; rv:9.0.1) Gecko/X Firefox/9.0.1";
const char S77532[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TM-MID780 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77533[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaN8-00)/UC Browser8.5.0.183/50/352/UCWEB Mobile";
const char S77534[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; 5860A Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S77535[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; 5860A Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77536[] = "Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; 5860A Build GRJ90) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S77537[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; 5860A Build/GRJ90)";
const char S77538[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S77539[] = "Nokia202/2.0 (20.28) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia202) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S77540[] = "OneBrowser/for Blackberry3.0.0 (BlackBerry8520/5.0.0.592)";
const char S77541[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MMU002D Build/MMU002D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77542[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; Optimus_Madrid Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77543[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9300 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S77544[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9300 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S77545[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LNV-Lenovo_A765e/S-3-02; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77546[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A765e Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77547[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo A765e Build/IMM76I)";
const char S77548[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC_Z560e Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77549[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC Z560e Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77550[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC Z560e Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S77551[] = "Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_S9714 Build/JRO03R) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S77552[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; WT19i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S77553[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; WT19i Build/4.0.2.A.0.58) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S77554[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S5303 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S77555[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S77556[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST26i Build/11.2.A.0.21) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S77557[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P880 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S77558[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT25i Build/9.0.D.0.484) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77559[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; iris458Q Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77560[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I545 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SCH-I545, 47, 1.9.7)";
const char S77561[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570I) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S77562[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S77563[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S77564[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i2) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S77565[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; id; ST21i) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Ponsel";
const char S77566[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-M919 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltetmo";
const char S77567[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-M919 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltetmo";
const char S77568[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-M919 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltetmo";
const char S77569[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A35) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S77570[] = "Mozilla/5.0 (Linux; Android 4.1.1; C1604 Build/11.3.A.2.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S77571[] = "SAMSUNG-GT-S7330/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S77572[] = "KWC-Lightpipe/ UP.Browser/7.2.7.2.531 (GUI) MMP/2.0";
const char S77573[] = "KWC-Lightpipe/1002 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0";
const char S77574[] = "KWC-Lightpipe/1003 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0";
const char S77575[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1";
const char S77576[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.4.1.0 Chrome/17.0.963.56 Safari/535.11";
const char S77577[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ERIS Build/GRI40; GSB.v4.5-FINAL_ODEXED_CM7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77578[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ERIS Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77579[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.6.1.82/69/351";
const char S77580[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.6.1.82/50/351";
const char S77581[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.6.1.82/28/351";
const char S77582[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.6.1.82/50/355";
const char S77583[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.6.1.82/50/352";
const char S77584[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.6.1.82/28/352";
const char S77585[] = "Opera/9.80 (Android 2.2.2; Linux; Opera Mobi/ADR-X; U; en-GB) Presto/2.7.81 Version/11.00";
const char S77586[] = "Opera/9.80 (Android 2.2; Linux; Opera Mobi/ADR-X; U; de) Presto/2.7.81 Version/11.00";
const char S77587[] = "Opera/9.80 (Android 2.2.1; Linux; Opera Mobi/ADR-X; U; es-ES) Presto/2.7.81 Version/11.00";
const char S77588[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/528.7 (KHTML, like Gecko) Iron/1.0.155.0 Safari/528.7";
const char S77589[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; TS1 Duo Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77590[] = "Mozilla/8.0 (Windows; U; Windows NT 5.1; pl; rv:1.8.1.8) Gecko/X Firefox/3.8.8 (.NET CLR 3.3.38888)\"";
const char S77591[] = "nokiac5-00.2/UC Browser8.0.3.107/69/444  3gpp-gba";
const char S77592[] = "nokiac5-00/UC Browser8.0.3.107/69/444  3gpp-gba";
const char S77593[] = "Mozilla/5.0 (Windows NT 5.0; rv:12.0) Gecko/X Thunderbird/12.0.1";
const char S77594[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:7.0.1) Gecko/X Firefox/7.0.1 AlexaToolbar/alxf-2.15";
const char S77595[] = "Mozilla/5.0 (Symbian3; U; en-us; NokiaC6-01)/UC Browser8.5.0.183/51/352/UCWEB Mobile";
const char S77596[] = "NokiaC6-01/UCWEB8.5.0.163/51/800";
const char S77597[] = "Mozilla/5.0 (Linux; Android 4.0.3; TAB466EUK Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S77598[] = "MicromaxX640ASTRO36_TD/v3 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10";
const char S77599[] = "Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia5310xpressmusic) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S77600[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; T-Mobile myTouch 4G Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77601[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; T-Mobile myTouch 4G Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77602[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; myTouch 4G Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77603[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; T-Mobile myTouch 4G Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77604[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; myTouch 4g Build/JZO54K; CyanogenMod-CM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77605[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; DISTAB9000R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77606[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Videocon A20 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77607[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 792TPC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77608[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenco CoolTab-70 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S77609[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Micromax_A72) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S77610[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S77611[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Spice Mi-535 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77612[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S77613[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S77614[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6502 Build/10.3.A.0.423) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77615[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A840S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S77616[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A840SP Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S77617[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6802) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S77618[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S77619[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S77620[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S77621[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S77622[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9100G Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S77623[] = "Nokia6267/2.0 (03.25) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S77624[] = "POLARIS/6.01 (BREW 3.1.5; U; en-us; LG; LX370; POLARIS/6.01/AMB) MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S77625[] = "Nokia3120/1.0 (06.31) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S77626[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB501 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77627[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB501 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77628[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB501 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77629[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.0.0.0 Chrome/13.0.782.107 Safari/535.1";
const char S77630[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; eGlide, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77631[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100 Build/IMM76I; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77632[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77633[] = "Mozilla/5.0 (Windows NT 6.0; rv:12.2.1) Gecko/X Firefox/12.2.1 PaleMoon/12.2.1";
const char S77634[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:12.2.1) Gecko/X Firefox/12.2.1-x64 PaleMoon/12.2.1-x64";
const char S77635[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; SPNG708 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77636[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U18GT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77637[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; U18GT2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77638[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U18GT-W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77639[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; TAB-PROTAB2-IPS9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77640[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S77641[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S77642[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S77643[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77644[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77645[] = "Mozilla/5.0 (Linux; Android 4.1.1; Desire L by HTC Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77646[] = "SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; fa; sonyericssonw20i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S77647[] = "Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia2730c-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S77648[] = "NOKIA-Lumia 610/UCWEB2.8.0.249/47/355";
const char S77649[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-i705 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S77650[] = "Mozilla/5.0 (Linux; Android 4.2.2; Desire HD Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77651[] = "Mozilla/5.0 (Linux; Android 4.2.1; Desire HD Build/JOP40G) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77652[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire V Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77653[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire VC Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77654[] = "Mozilla/5.0 (iPhone; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Rockmelt/3.0.3.7 Mobile/10A525 Safari/8536.25";
const char S77655[] = "Mozilla/5.0 (compatible; MSIE 10.0; WinNT4.0; Trident/4.0)";
const char S77656[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 928)";
const char S77657[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; HTC Desire S Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31";
const char S77658[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Desire S Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S77659[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo P700i Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77660[] = "Mozilla/5.0 (Linux; Android 4.0.3; Lenovo P700 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77661[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-I9000 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S77662[] = "Mozilla/5.0 (Linux; Android 2.3.3; GT-I9000 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S77663[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77664[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77665[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77666[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825-1 Build/HuaweiU8825-1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77667[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77668[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E610 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77669[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E612 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77670[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E612f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77671[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; M4TEL SS660 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77672[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Note II N7102 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77673[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5302) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S77674[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5282 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S77675[] = "Nokia303/5.0 (14.87) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia303) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S77676[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F240K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S77677[] = "BlackBerry8520/4.2.0.135 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/159/444";
const char S77678[] = "Opera/9.80 (Android 3.2; Linux; Opera Tablet/ADR-X; U; fr) Presto/2.9.201 Version/11.50";
const char S77679[] = "Opera/9.80 (Android 3.2.1; Linux; Opera Tablet/ADR-X; U; pl) Presto/2.9.201 Version/11.50";
const char S77680[] = "Opera/9.80 (Android 3.2.2; Linux; Opera Tablet/ADR-X; U; en) Presto/2.9.201 Version/11.50";
const char S77681[] = "Opera/9.80 (Android 3.2.6; Linux; Opera Tablet/ADR-X; U; en) Presto/2.9.201 Version/11.50";
const char S77682[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19 Comodo_Dragon/18.3.0.0";
const char S77683[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; M10 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S77684[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; ViewPad7x Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S77685[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; ViewPad7X Build/HTK55D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S77686[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; ViewPad7X Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S77687[] = "Mozilla/5.0 (Linux; Android 4.0.4; IS12S Build/6.1.D.1.74) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77688[] = "MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HUAWEI U8661 Build/HuaweiU8661) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77689[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S77690[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S77691[] = "Opera/9.80 (iPad; Opera Mini/6.5.2/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S77692[] = "JUC (Linux; U; 2.3.4; zh-cn; SK17i; 320*480) UCWEB7.9.0.94/139/444";
const char S77693[] = "JUC (Linux; U; 4.0.4; zh-cn; SK17i; 320*480) UCWEB7.9.0.94/139/352";
const char S77694[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77695[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; SCH-N719 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77696[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77697[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-N719 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77698[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GOCLEVER TAB A93.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77699[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/6.5.28192/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S77700[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/6.5.28192/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S77701[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ME865 Build/6.7.2_GC-384) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77702[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ME865 Build/6.7.2_GC-404) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77703[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Explorer_A310e) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S77704[] = "Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD+ Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S77705[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Edison Build/1.1.8 X-16:29) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S77706[] = "Mozilla/5.0 (Linux; Android 4.1.1; X909 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77707[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717R Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77708[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I577 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77709[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; PROG40C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77710[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A25 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S77711[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7510 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S77712[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P7510 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S77713[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P716 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77714[] = "Mozilla/5.0 (Linux; Android 4.1.1; Vodafone 975 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77715[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; E3 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77716[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; E3 Build/JOP40D)";
const char S77717[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; id; A1+) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Ponsel";
const char S77718[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A1+) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S77719[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; Karbonn A27+ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77720[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Karbonn A26 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77721[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Karbonn A26 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77722[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9082) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S77723[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7501 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S77724[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT885 Build/6.7.3-135_YIC-8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S77725[] = "SAMSUNG-SGH-E251/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S77726[] = "SAMSUNG-SGH-E251L/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S77727[] = "LG-GT405/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S77728[] = "LG-GT405-Orange/V10f Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S77729[] = "Huawei/1.0/G7010/Browser/obigo-browser/q03c";
const char S77730[] = "Huawei/G7010/1.0/Browser/obigo-browser/q03c";
const char S77731[] = "MOT-VE240/00.72 UP.Browser/7.2.7.5.541 (GUI) MMP/2.0";
const char S77732[] = "MOT-VE240/00.72 UP.Browser/7.2.7.5.548 (GUI) MMP/2.0";
const char S77733[] = "MOT-VE240/00.72 UP.Browser/7.2.7.5.599 (GUI) MMP/2.0";
const char S77734[] = "LGE-VX5500/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S77735[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; LG-SU370 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77736[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5 Comodo_Dragon/19.1.0.0";
const char S77737[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s5380k) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S77738[] = "Mozilla/5.0 (Linux; Android 4.0.4; rk30sdk Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S77739[] = "Mozilla/5.0 (Linux; Android 4.0.4; Full AOSP on Rk29sdk Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S77740[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T06A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77741[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AT107F Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77742[] = "Micromax X288/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77743[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar Touch Kem 432m Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77744[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; LG-E400) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S77745[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 710;7.10;8862)";
const char S77746[] = "Windows Phone Search (Windows Phone OS 7.10;Nokia;Lumia 710;7.10;8858)";
const char S77747[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6810 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S77748[] = "Mozilla/5.0 (compatible; MSIE 3.0; PPC; Tablet PC; Trident/4.0)";
const char S77749[] = "NokiaC2-05/2.0 (08.69) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-05) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S77750[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-P3100) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S77751[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S77752[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI C8812E Build/HuaweiC8812E) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31";
const char S77753[] = "Mozilla/5.0 (Linux; Android 4.1.2; SOL21 Build/9.1.D.0.401) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77754[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77755[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S77756[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-79) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S77757[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-79-1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S77758[] = "Mozilla/5.0 (Linux; Android 4.2.1; Transformer Prime TF201 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S77759[] = "Mozilla/5.0 (Linux; Android 4.0.3; Marquis_MP977 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S77760[] = "SAMSUNG-GT-E2330B/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S77761[] = "ZTE-T100/1.0 ACS-NF/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77762[] = "Nokia6300/2.0 (05.50) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6300/UC Browser7.9.0.102/70/352 Novarra-Vision/8.0";
const char S77763[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Radar; Orange)";
const char S77764[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Radar)";
const char S77765[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Radar C110e)";
const char S77766[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Radar 4G)";
const char S77767[] = "ZDM/4.0; Windows Mobile 7.0;";
const char S77768[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:2.0b1) Gecko/X Firefox/4.0b1";
const char S77769[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; AT-AS43D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77770[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB501 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77771[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SurfTab_8.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77772[] = "OneBrowser/3.1 (NokiaC2-01/11.10)";
const char S77773[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S77774[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S77775[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S77776[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S77777[] = "Mozilla/5.0 (Linux; Android 4.0.4; A10 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S77778[] = "Mozilla/5.0 (Linux; Android 4.0.4; A10 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S77779[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; SH930W Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77780[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S77781[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);";
const char S77782[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9000 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77783[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.137; en-us; 9930/5.1.0.137) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S77784[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.553; en-us; 9930/5.1.0.553) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S77785[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.429; en-us; 9930/5.1.0.429) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S77786[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.230; en-us; 9930/5.1.0.230) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S77787[] = "UCWEB/2.0(BlackBerry; U; 5.1.0.678; en-us; 9930/5.1.0.678) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S77788[] = "Mozilla/5.0 (Linux; Android 4.2.2; Infected Rezound Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77789[] = "Mozilla/5.0 (Linux; Android 4.2.2; Infected HTC Rezound Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77790[] = "Mozilla/5.0 (Linux; Android 4.2.2; Rezound on TRON 2.1 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77791[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A25) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S77792[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S77793[] = "Mozilla/5.0 (Linux; Android 4.0.4; JY-G3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S77794[] = "Mozilla/5.0 (Linux; Android 4.1.1; NS-13T001 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S77795[] = "SAMSUNG-SGH-T349/T349UVIC7 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77796[] = "Mozilla/5.0(Linux;U;Android2.1-update1;Zh_cn;E10i;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S77797[] = "JUC(Linux;U;Android2.1-update1;Zh_cn;E10i;240*320;)UCWEB7.8.0.95/139/444";
const char S77798[] = "MOT-RASPB/00.72 UP.Browser/7.2.7.5.622 (GUI) MMP/2.0";
const char S77799[] = "MOT-VEGAS/00.72 UP.Browser/7.2.6.1.835 (GUI) MMP/2.0";
const char S77800[] = "MOT-RASPB/00.72 UP.Browser/7.2.7.5.598 (GUI) MMP/2.0";
const char S77801[] = "MOT-VEGAS/00.72 UP.Browser/7.2.6.1.841 (GUI) MMP/2.0";
const char S77802[] = "MOT-DBLDV/00.72 UP.Browser/7.2.7.5.599 (GUI) MMP/2.0";
const char S77803[] = "MOT-A-A7/00.05 UP.Browser/7.2.7.2.542f (GUI) MMP/2.0 Push/PO";
const char S77804[] = "MOT-CRUSH/00.72 UP.Browser/7.2.7.5.678 (GUI) MMP/2.0";
const char S77805[] = "HTC Streaming Player htc_europe / 1.0 / htc_pyramid / 2.3.3";
const char S77806[] = "HTC Streaming Player tmous / 1.0 / htc_pyramid / 2.3.3";
const char S77807[] = "HTC Streaming Player tmous / 1.0 / htc_pyramid / 2.3.4";
const char S77808[] = "HTC Streaming Player orange_uk / 1.0 / htc_pyramid / 2.3.3";
const char S77809[] = "HTC Streaming Player telstra_wwe / 1.0 / htc_pyramid / 2.3.3";
const char S77810[] = "HTC Streaming Player vodafone_uk / 1.0 / htc_pyramid / 2.3.4";
const char S77811[] = "HTC Streaming Player htc_europe / 1.0 / htc_pyramid / 2.3.4";
const char S77812[] = "HTC Streaming Player htc_europe / 1.0 / htc_pyramid / 4.0.3";
const char S77813[] = "HTC Streaming Player tmous / 1.0 / htc_pyramid / 4.0.3";
const char S77814[] = "HTC Streaming Player htc_asia_wwe / 1.0 / htc_pyramid / 4.0.3";
const char S77815[] = "KDDI-SH3J UP.Browser/6.2_7.2.7.1.K.5.212 (GUI) MMP/2.0";
const char S77816[] = "NexianNX-G967/MTK  Release/01.01.2011/MAUI Browser/Opera Profile/MIDP2.0 Configuration/CLDC-1 Opera/9.80 (MTK; U; id-ID) Presto/2.5.28 Version/10.10";
const char S77817[] = "NexianNX-G963/MTK  Release/01.01.2011/MAUI Browser/Opera Profile/MIDP2.0 Configuration/CLDC-1 Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10";
const char S77818[] = "NexianNX-G963/MTK Release/01.01.2011/MAUI Browser/Opera Profile/MIDP2.0 Configuration/CLDC-1 Opera/9.80 (MTK; U; id-ID) Presto/2.5.28 Version/10.10";
const char S77819[] = "NexianNX-G963/MTK  Release/01.01.2011/MAUI Browser/Opera Profile/MIDP2.0 Configuration/CLDC-1 Opera/9.80 (MTK; U; id-ID) Presto/2.5.28 Version/10.10";
const char S77820[] = "NexianNX-G967/MTK  Release/01.01.2011/MAUI Browser/Opera Profile/MIDP2.0 Configuration/CLDC-1 Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10";
const char S77821[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.4.0.159/50/351/UCWEB";
const char S77822[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.4.0.159/50/352/UCWEB";
const char S77823[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.4.0.159/28/352/UCWEB";
const char S77824[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.4.0.159/28/351/UCWEB";
const char S77825[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.4.0.159/50/355/UCWEB";
const char S77826[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.4.0.159/50/444/UCWEB";
const char S77827[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.4.0.159/28/444/UCWEB";
const char S77828[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.4.0.159/28/354/UCWEB";
const char S77829[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.4.0.159/50/354/UCWEB";
const char S77830[] = "Nokia308/2.0 (05.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia308) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S77831[] = "Mozilla/5.0 (Series40; Nokia111/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S77832[] = "Mozilla/5.0 (Series40; Nokia111.1/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S77833[] = "Mozilla/5.0 (Linux; Android 4.0.4; tmn smart a15 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77834[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V880E Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77835[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Sensation XE Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77836[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_SensationXE_Beats_Z715e Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77837[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC Sensation XE with Beats Audio Z715e Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77838[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A5+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77839[] = "Micromax X336/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77840[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S77841[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S77842[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);";
const char S77843[] = "Mozilla/5.0 (Linux; Android 4.1.1; A701 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S77844[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SH631W Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77845[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-S7500 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S77846[] = "Mozilla/5.0 (Linux; Android 4.1.2; EK-GC110 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77847[] = "Mozilla/5.0 (Linux; Android 4.1.2; C2104 Build/15.0.A.1.31) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77848[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E976 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77849[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I545 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S77850[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ607-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S77851[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I757M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S77852[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5570) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S77853[] = "Mozilla/5.0 (Linux; Android 4.2.2; SHV-E330S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S77854[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5250) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S77855[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire C Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S77856[] = "Mozilla/5.0 (Linux; Android 4.1.2; PantechP9090 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77857[] = "Mozilla/5.0 (Linux; Android 4.1.2; PantechP9090 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S77858[] = "Nokia7230/5.0 (10.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7230) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S77859[] = "Mozilla/5.0 (Linux; Android 4.2.1; LIFE PLAY Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S77860[] = "Mozilla/5.0 (Linux; Android 4.3; EVO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S77861[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT886 Build/6.7.3-179_YIC-10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S77862[] = "Nokia6208c/2.0 (04.51) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S77863[] = "LG/U990/v1.0";
const char S77864[] = "Alcatel-OT-305/1.0 ObigoInternetBrowser/Q03C, Alcatel-OT-305/1.0 ObigoInternetBrowser/Q03C";
const char S77865[] = "Alcatel-OT-305A/1.0 ObigoInternetBrowser/Q03C";
const char S77866[] = "Alcatel-OT-305/1.0 ObigoInternetBrowser/Q03C";
const char S77867[] = "Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5130c-2/UC Browser7.9.0.102/70/355";
const char S77868[] = "Nokia5130/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5130/UC Browser7.9.0.102/69/352";
const char S77869[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.13) Gecko/X Firefox/2.0.0.13 Flock/1.1.1";
const char S77870[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; GT-P1000 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77871[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-P1000 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77872[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-P1000 Build/MIUI-XJ) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77873[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; GT-P1000 Build/MIUI-XJ)";
const char S77874[] = "Mozilla/5.0 (Linux; U; Android Android 2.3.5; en-gb; GT-P1000 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77875[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-P1000 Build/MIUI-XJ) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S77876[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ru-ru; GT-P1000 Build/MIUI-XJ) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-9.1.0 Mobile Safari/534.30";
const char S77877[] = "LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-kp500 teleca) UCBrowser8.2.1.144/70/352/UCWEB Mobile";
const char S77878[] = "Sprint T7380 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S77879[] = "HTC_Touch_Pro2_T7373 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S77880[] = "Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S77881[] = "Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S77882[] = "Mozilla/5.0 (Linux; Android 4.0.4; V1277 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77883[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko)/Nutch-1.5.1";
const char S77884[] = "Opera/9.80 (Windows NT 5.0) Presto/2.12.388 Version/12.10";
const char S77885[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID SPYDER Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77886[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID SPYDER Build/HDish) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77887[] = "NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiax2-05) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S77888[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3100 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77889[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; GT-P3100 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77890[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3100 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77891[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3100 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77892[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; GT-P3100 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77893[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/SPS05T; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77894[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GEM8113 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77895[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; UMI X1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77896[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Philips W832 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77897[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; WEXLER-TAB-7T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77898[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/4.6.32741/29.3551; U; zh) Presto/2.8.119 Version/11.10";
const char S77899[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7500 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S77900[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P7500 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S77901[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7501 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S77902[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:34, lv";
const char S77903[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +0000, gb";
const char S77904[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT11i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77905[] = "Mozilla/5.0 (Linux; Android 4.0.3; MT11i Build/4.1.A.0.562) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77906[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT11a Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77907[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP4100 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S77908[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E15i Build/2.0.1.A.0.47) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S77909[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E15i Build/2.1.1.A.0.6) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S77910[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S77911[] = "Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S77912[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105P Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S77913[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S77914[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030A Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77915[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030A Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S77916[] = "Mozilla/5.0 (Linux; Android 4.0.4; AZ210A Build/AZ210A_ICS_02.34I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S77917[] = "Mozilla/5.0 (Linux; Android 4.0.4; AZ210B Build/AZ210B_ICS_02.34I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S77918[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 101 Titanium Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S77919[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-LS970 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S77920[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E435g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S77921[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E435f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S77922[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E435 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S77923[] = "Mozilla/5.0 (Linux; Android 4.1.2; AS870 4G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77924[] = "Mozilla/5.0 (Linux; Android 4.1.2; AS870 4G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S77925[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ607-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S77926[] = "Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S77927[] = "LG-GD330-CH/V100 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77928[] = "LG-GD330/V100 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S77929[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/XEHE1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77930[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/XXGK4 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77931[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/DZGL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77932[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/JTHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77933[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/DZHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77934[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/XXHA1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77935[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/XBHB3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77936[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/JPHH2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77937[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/DDHB4 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77938[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/DZHF1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77939[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/XEGL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77940[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/AKHE2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77941[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/DBGL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77942[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/XEGK3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77943[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/NPGJ2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77944[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/PMHE3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77945[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/JVGK3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77946[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung-SGH-i550/AOGL2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77947[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/AKHA1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77948[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/JVGJ4 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77949[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/AIGI4 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77950[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/DBHD2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77951[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/XXGL2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77952[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/JPGL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77953[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/DDHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77954[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/DBHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77955[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/JVHB2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S77956[] = "sam-r460 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S77957[] = "Mozilla/5.0 (Series40; NokiaC3-00/04.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.6";
const char S77958[] = "Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; LT-NA7 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77959[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Skate Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77960[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Skate Build/IML74K; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77961[] = "Mozilla/5.0 (X11; U; Linux i686; pl; rv:1.8.0.1) Gecko/X Firefox/1.5.0.1 Ubuntu";
const char S77962[] = "NOKIA-Lumia 710/UCWEB2.2.0.157/47/355";
const char S77963[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:14.0) Gecko/X Firefox/14.0.3.0 IceDragon/14.0.3.0";
const char S77964[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.1 Safari/535.19 Silk-Accelerated=false";
const char S77965[] = "Mozilla/5.0 (Linux; Android 4.0.3; PAD703 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77966[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAD703 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77967[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo A698t Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S77968[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo A698t Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S77969[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo A698t Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77970[] = "Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo A698t Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S77971[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.27.1 (KHTML, like Gecko) Stainless/0.4 Safari/525.20.1";
const char S77972[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar Touch Kem 432 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77973[] = "LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-gs290) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S77974[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VS TOUCHTAB 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77975[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-L40G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S77976[] = "Nokia6300/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia6300) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S77977[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; A5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S77978[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KPT A5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77979[] = "Mozilla/5.0 (Linux; Android 4.0.4; CINK PEAX Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S77980[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A800S Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77981[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; vivo S9 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77982[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; vivo S9t Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77983[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; vivo S9 Build/JRO03C)";
const char S77984[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/537.31 slurp";
const char S77985[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 slurp";
const char S77986[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M040 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S77987[] = "UCWEB/2.0 (iOS; U; iPh OS 5_0_1; zh-CN; iPh3,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile";
const char S77988[] = "UCWEB/2.0 (iOS; U; iPh OS 5_0_1; zh-CN; iPh4,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile";
const char S77989[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN82) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S77990[] = "Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_E7310 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S77991[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030D Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S77992[] = "Mozilla/5.0 (Linux; Android 4.1.1; ONE TOUCH TAB 7HD Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S77993[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9500 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S77994[] = "MQQBrowser/4.3/Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77995[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ONE TOUCH TAB 7HD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S77996[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; GT-N7000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S77997[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-N7000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S77998[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Zeepad 7.0 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S77999[] = "Mozilla/5.0 (Windows NT 5.1; rv:22.0) Gecko/X Firefox/22.0 slurp";
const char S78000[] = "Mozilla/5.0 (Windows NT 5.1; rv:22.0) Gecko/X Firefox/22.0 slurp slurp";
const char S78001[] = "Nokia7230/5.0 (10.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7230) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S78002[] = "Mozilla/5.0 (Linux; Android 4.0.4; Desire HD Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S78003[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire HD Build/ViperDHD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S78004[] = "Mozilla/5.0 (Linux; Android 4.0.4; T-Hub2 Build/TVA301TELBG3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S78005[] = "Mozilla/5.0 (Linux; Android 4.0.3; Lenovo S880 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S78006[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A27) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S78007[] = "Mozilla/4.0 SonyEricssonV640iv/R1DA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S78008[] = "Mozilla/5.0 (Linux; U; Android 1.5; fr-fr; U8230 Build/CRB17) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S78009[] = "Midori/0.3 (X11; Linux; U; en-us) WebKit/534.21+";
const char S78010[] = "Mozilla/5.0 (X11; Linux) AppleWebKit/531.2+ Midori/0.3";
const char S78011[] = "Mozilla/5.0 (X11; U; Linux; en-us; rv:1.9.0.2) Gecko/X Firefox/3.8 Midori/0.3";
const char S78012[] = "Mozilla/5.0 (X11; U; Linux; en-us) AppleWebKit/531+ (KHTML, like Gecko) Safari/531.2+ Midori/0.3";
const char S78013[] = "Midori/0.3 (X11; Linux; U; en-us) WebKit/531.2+";
const char S78014[] = "Midori/0.3 (X11; Linux; U; de-de) WebKit/534.21+";
const char S78015[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MT917 Build/6.5.1_GC-146-DNRTD-8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78016[] = "MOT-MT917_TD/1.0 Android/2.3 (Linux; U; Android 2.3.5; zh-cn) Release/12.09.2011 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78017[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MT917 Build/6.5.1_GC-146-DNRTD-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78018[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ja-jp; IS12M Build/6.5.1-134_SWK-68_LE-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78019[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; zh-cn; S2005A-H Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78020[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; S2005A-H Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S78021[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; S2005A-H Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78022[] = "Mozilla/5.0 (Linux; Android 4.0.3; JY-G2 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78023[] = "Mozilla/5.0 (Linux; Android 4.0.4; JY-G2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78024[] = "LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gs290) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S78025[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P765 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S78026[] = "Mozilla/5.0 (Linux; Android 4.1.2; A500 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S78027[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Celkon A75 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78028[] = "Mozilla/5.0 (Linux; Android 4.1.1; SHV-E230S Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S78029[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E230S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S78030[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E230K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S78031[] = "Mozilla/5.0 (Linux; Android 4.1.1; SHV-E230K Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S78032[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E230L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S78033[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P895 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S78034[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6503 Build/10.1.A.1.434) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S78035[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 101G10 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S78036[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)";
const char S78037[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; QQBrowser/6.12)";
const char S78038[] = "Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.2; SV1; QQBrowser/6.12)";
const char S78039[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; QQBrowser/6.12)";
const char S78040[] = "Nokia206/2.0 (03.59) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia206) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S78041[] = "Mozilla/5.0 (Linux; Android 4.1.1; V360 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S78042[] = "Opera/9.80 (Linux armv7l; U;  CE-HTML/1.0 NETTV/4.1.1 PHILIPSTV/2.1.1; en) Presto/2.10.287 Version/12.00";
const char S78043[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.1 Safari/7536.29.13 Sleipnir/4.3.0.1";
const char S78044[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.1 Safari/7536.30.1 Sleipnir/4.3.1";
const char S78045[] = "Mozilla/5.0 (Linux; Android 4.1.1; I10A-LE Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S78046[] = "Mozilla/5.0 (Linux; Android 4.2.1; JY-G3 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S78047[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-in; ST21i Build/11.0.A.4.27) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S78048[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-in; ST21i2 Build/11.0.A.6.8) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S78049[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-in; ST21i2 Build/11.0.A.1.8) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S78050[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-in; ST21i Build/11.0.A.0.16) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S78051[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-in; ST21i Build/11.0.A.4.22) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S78052[] = "Mozilla/5.0 (Linux; Android 4.0.3; M532 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S78053[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HUAWEI C8813 Build/HuaweiC8813) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S78054[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI C8813Q Build/HuaweiC8813Q) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S78055[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-N7105 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78056[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-N7105 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78057[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L900 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78058[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I317 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78059[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-N7105 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78060[] = "Mozilla/5.0 (Linux; Android 4.2.1; ME301T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S78061[] = "MOT-RAZRV3xxR/97.04.2BR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003290) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.60 [en]";
const char S78062[] = "MOT-RAZRV3xxR/97.04.2BR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; X) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [en]";
const char S78063[] = "Nokia7310c/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S78064[] = "Nokia7310c_CMCC/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S78065[] = "LG/GT405/v1.0";
const char S78066[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC/Sensation/1.45.161.1; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S78067[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC/Sensation/1.45.162.1; de-de) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S78068[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC/Sensation/1.45.166.1; de-ch) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S78069[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Sensation_Z710a; en-au) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S78070[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC/Sensation/1.35.161.2; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S78071[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC/Sensation/1.45.163.4; fr-fr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S78072[] = "NokiaC5-05/UC Browser8.0.4.121/50/444/UCWEB";
const char S78073[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 610)";
const char S78074[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 610; Vodafone)";
const char S78075[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE66)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S78076[] = "NokiaE66/UCWEB8.2.0.116/28/800";
const char S78077[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; X10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78078[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; X10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78079[] = "Opera/9.80 (Android 4.0.3; Linux; Opera Mobi/ADR-X; U; en) Presto/2.7.81 Version/11.00";
const char S78080[] = "Opera/9.80 (Android 4.0.4; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.7.81 Version/11.00";
const char S78081[] = "Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU5) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/352/UCWEB Mobile";
const char S78082[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-b7722) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S78083[] = "Opera/12.80 (Windows NT 5.1; U; en) Presto/2.10.289 Version/12.08";
const char S78084[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Dslide 704 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78085[] = "Mozilla/5.0 (Linux; Android 4.0.4; F-05E Build/V13R36C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S78086[] = "Micromax Q76/Q03C   Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10";
const char S78087[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo S890 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78088[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo S890 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S78089[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo S890 Build/IMM76D)";
const char S78090[] = "Mozilla/5.0 (Linux; Android 4.0.4; C525c Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S78091[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7100 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S78092[] = "Nokia308/2.0 (05.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia308) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S78093[] = "Nokia6500s-1/2.0 (08.30) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6500s-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S78094[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; SonyEricssonU5) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S78095[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5228) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S78096[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U9510E Build/HuaweiU9510E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S78097[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P870 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S78098[] = "Mozilla/5.0 (Linux; Android 4.2.2; SC-04E Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S78099[] = "Mozilla/5.0 (Linux; Android 4.2.2; SC-04E Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S78100[] = "UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 800) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile";
const char S78101[] = "Mozilla/5.0 (Linux; Android 4.0.4; C1605 Build/11.1.A.0.68) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S78102[] = "Mozilla/5.0 (Linux; Android 4.1.1; Vodafone 975N Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78103[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8552 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S78104[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7501 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S78105[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC VLE_U Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S78106[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S78107[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S78108[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaE6-00) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S78109[] = "Mozilla/5.0 (Linux; Android 4.1.1; ONE TOUCH 4011X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S78110[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PAP4505DUO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78111[] = "Nokia6133/2.0 (05.60) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S78112[] = "MOT-W840/00.72 UP.Browser/7.2.7.5.673 (GUI) MMP/2.0";
const char S78113[] = "MOT-W840/00.72 UP.Browser/7.2.7.5.663 (GUI) MMP/2.0";
const char S78114[] = "Alcatel-OT-807A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S78115[] = "Alcatel-OT-807/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S78116[] = "Alcatel-OT-807/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A SVN/05S";
const char S78117[] = "Alcatel-OT-807/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A SVN/04N";
const char S78118[] = "Alcatel-OT-807D/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S78119[] = "LG-KE970 MIC/1.1.14 MIDP-2.0/CLDC-1.1";
const char S78120[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Comodo_Dragon/9.0.0.3 Chrome/9.0.597.98 Safari/534.13";
const char S78121[] = "Opera/9.27 (Windows 98; U; en)";
const char S78122[] = "Opera/9.25 (Windows 98; U; en)";
const char S78123[] = "Opera/9.21 (Windows 98; U; en)";
const char S78124[] = "Opera/9.24 (Windows 98; U; en)";
const char S78125[] = "Opera/9.22 (Windows 98; U; en)";
const char S78126[] = "Opera/9.26 (Windows 98; U; en)";
const char S78127[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.8.1.17; ns) Gecko/X Firefox/2.0.0.17 Flock/1.2.6";
const char S78128[] = "NokiaC5-03/UCWEB8.4.0.150/50/800";
const char S78129[] = "Mozilla/5.0 (S60V5; U; en-in; NokiaC5-03)/UC Browser8.4.0.159/50/352/UCWEB Mobile";
const char S78130[] = "JUC(Linux;U;Android2.3.4;Zh_cn;WT19i;320*480;)UCWEB7.8.0.95/139/444";
const char S78131[] = "Mozilla/5.0(Linux;U;Android2.3.4;Zh_cn;WT19i;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S78132[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5253) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/405/UCWEB Mobile";
const char S78133[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Smart_one Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78134[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; Nexus One Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 (compatible; Googlebot-Mobile/2.1; +http://www.google.com/bot.html)";
const char S78135[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; LT-7009 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78136[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT-7009 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78137[] = "Mozilla/5.0 (compatible; MSIE 4.0; Windows NT 6.2; Trident/3.0)";
const char S78138[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Surfer 8.31 3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78139[] = "Mozilla/5.0 (Linux; Android 4.0.4; MB886 Build/7.7.1Q-144_MB886_MR1-22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S78140[] = "Mozilla/5.0 (Linux; Android 4.1.1; CINK SLIM Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78141[] = "Mozilla/5.0 (Linux; U; Android 4.1.2;zh-cn; ZTE V987 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30;";
const char S78142[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; N80IPS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78143[] = "Mozilla/5.0 (X11; Linux x86_64; Orange Daytona Build/HuaweiG510-0200) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24 TT/4.1.1-eng.lX.X.X";
const char S78144[] = "Mozilla/5.0 (X11; Linux x86_64; HUAWEI G510-0200 Build/HuaweiG510-0200) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24 TT/4.1.1";
const char S78145[] = "Mozilla/5.0 (X11; Linux x86_64; HUAWEI G510-0100 Build/HuaweiG510-0100) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24 TT/4.1.1";
const char S78146[] = "Mozilla/5.0 (X11; Linux x86_64; rv:17.0) Gecko/X Icedove/17.0.5";
const char S78147[] = "Mozilla/5.0 (X11; Linux i686; rv:17.0) Gecko/X Icedove/17.0.7";
const char S78148[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7102 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S78149[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7102 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S78150[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-M840 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78151[] = "Mozilla/5.0 (Linux; Android 4.0.4; JY-G3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S78152[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A89 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S78153[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Titanium S5 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S78154[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC One X Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S78155[] = "SAMSUNG-M3200/M3200JCHJ1 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S78156[] = "LG-GU285g Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S78157[] = "LG-GU285f Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S78158[] = "LG-GU285/v10a Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S78159[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S78160[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) 480x800; XV6975; Window Mobile 6.5 Professional;";
const char S78161[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Mozilla/5.0 (HW-HUAWEI_C8300/C8300V100R001C00B102;U;Windows Mobile/6.5;320*480;CTC/2.0) IE/6.0";
const char S78162[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; SCH-i909 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S78163[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; GT-I9088 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S78164[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; SCH-i909 Build/ECLAIR) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S78165[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; SCH-i909 Build/ECLAIR) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S78166[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT; MS Search 4.0 Robot)";
const char S78167[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Iron/10.0.650.0 Chrome/10.0.650.0 Safari/534.16";
const char S78168[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Iron/10.0.650.0 Chrome/10.0.650.0 Safari/534.16";
const char S78169[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_Desire_S; 480*800) UCWEB7.9.3.103/139/800";
const char S78170[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_Desire_S; 480*800) UCWEB7.9.0.94/139/800";
const char S78171[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_Desire_S; 480*800) UCWEB7.9.3.103/139/1002";
const char S78172[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_Desire_S; 480*800) UCWEB7.9.3.103/139/800";
const char S78173[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_Desire_S; 480*800) UCWEB7.9.3.103/139/999";
const char S78174[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_Desire_S; 480*800) UCWEB7.9.1.100/139/800";
const char S78175[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_Desire_S; 480*800) UCWEB7.9.4.145/139/800";
const char S78176[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_Desire_S; 480*800) UCWEB7.9.4.145/139/800";
const char S78177[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_Desire_S; 480*800) UCWEB7.9.0.94/139/352";
const char S78178[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_Desire_S; 480*800) UCWEB7.9.0.94/139/444";
const char S78179[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Incredible Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78180[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Droid Incredible Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78181[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Incredible Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78182[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Incredible Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78183[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Incredible Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78184[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Droid Incredible Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78185[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Droid Incredible Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78186[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Droid Incredible Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78187[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.14) Gecko/X Firefox/2.0.0.14 Flock/1.2.1";
const char S78188[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Spice Mi-350 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78189[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Spice Mi-350 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78190[] = "Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHV-E140S Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S78191[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; id-id; T1C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78192[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; N-07D Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78193[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; N-07D Build/AX)";
const char S78194[] = "Mozilla/5.0 (Windows NT 5.1; rv:17.0) Gecko/17.0 Firefox/17.0 slurp";
const char S78195[] = "Mozilla/5.0 (Windows NT 5.1; rv:17.0) Gecko/X Firefox/17.0 slurp";
const char S78196[] = "Mozilla/5.0 (Windows NT 5.1; rv:17.0) Gecko/X Firefox/17.0 slurp slurp";
const char S78197[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Transformer Pad TF300T Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78198[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; TF300T Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78199[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; TF300T Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78200[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; TF300T Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78201[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HS-E926 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78202[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HS-E926 Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S78203[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HS-E926 Build/IMM76I)";
const char S78204[] = "Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; HS-E926 Build IMM76I) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S78205[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P990 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S78206[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7310 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78207[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-P7310 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78208[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7310 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78209[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7310 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78210[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ609 Build/7.7.1-85_MZ609-31) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S78211[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; @TAB-NINE.7-X2-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78212[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; LG-E612 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S78213[] = "Mozilla/5.0 (Linux; Android 4.1.2; S500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S78214[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTL21 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S78215[] = "Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300L Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S78216[] = "SAMSUNG-E2510/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S78217[] = "SAMSUNG-E2510-ORANGE/E2510CHIB1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S78218[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SCH-W899 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78219[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-W899 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78220[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-W899 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S78221[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-W899 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S78222[] = "SAMSUNG-SGH-I627/UCIC2 Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S78223[] = "Beetel GT415/2.0 (5.0614.0) NucleusOS/1.0 Profile/MIDP-2.0Configuration/CLDC-1.0";
const char S78224[] = "MX8/1.00 Nucleus RTOS/V1.11.19 MTK6225/07B Release/04.25.2008 Browser/Teleca Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S78225[] = "BIRD K298/1.00 Nucleus RTOS/V1.11.19 MTK6223/08A Release/07.01.2008 Browser/Teleca Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S78226[] = "BIRD /1.00 Nucleus RTOS/V1.11.19 MTK6235/08B Release/04.09.2009 Browser/Teleca Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S78227[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; ja-JP; rv:1.7.5) Gecko/X Netscape/8.0.4";
const char S78228[] = "Mozilla/5.0 (Windows NT 6.0; rv:9.0.1) Gecko/X Firefox/9.0.1 CometBird/9.0.1";
const char S78229[] = "Opera/8.50 (Windows ME; U; en)";
const char S78230[] = "Opera/8.52 (Windows ME; U; en)";
const char S78231[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows ME; pl) Opera 8.02";
const char S78232[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 8.0; Trident/5.0; IEMobile/9.0; SAMSUNG; SGH-i917)";
const char S78233[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 8.0; Trident/5.0; IEMobile/9.0; WpsLondonTest; SAMSUNG; SGH-i917)";
const char S78234[] = "Mozilla/5.0 (S60V3; U; en-in; NokiaN73)/UC Browser8.4.0.159/28/352/UCWEB Mobile";
const char S78235[] = "Mozilla/5.0 (Java; U; en-us; gt-s5233a) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S78236[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; SPNG808 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78237[] = "Mozilla/5.0 (Java; U; en-in; mauie800) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S78238[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S78239[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT1S0 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S78240[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Aqua S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78241[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ADR6300 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78242[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ADR6300 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78243[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; spice) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S78244[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P3100 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S78245[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P3100 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S78246[] = "Mozilla/5.0 (Linux; Android 4.0.4; Philips W832 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78247[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000";
const char S78248[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-I8190 Build/JRO03H) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S78249[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I8190 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S78250[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Intex_Aqua_3.2) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S78251[] = "Mozilla/5.0 (iPod; CPU iPhone OS 60 like Mac OS X) AppleWebKit/536.26 (KHTML like Gecko) Mobile/10A403 Safari/7534.48.3 Sleipnir/2.1.2m";
const char S78252[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Newman N2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78253[] = "FARSIGHTED50_6464_11B_HW (MRE\\2.5.00(800) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\1 gsensor\\0 keyboard\\normal) ZEN_P32_Software_V1.2 Release/2013.01.31 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US";
const char S78254[] = "FARSIGHTED50_6464_11B_HW (MRE\\2.5.00(800) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\1 gsensor\\0 keyboard\\normal) ZEN_P32_Software_V1.5 Release/2013.03.01 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US";
const char S78255[] = "FARSIGHTED50_6464_11B_HW (MRE\\2.5.00(800) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\1 gsensor\\0 keyboard\\normal) ZEN_P32_Software_V1.5 Release/2013.03.01 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 hi-IN";
const char S78256[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT890 Build/9.8.2I-50_SMI-26) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S78257[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E615 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S78258[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E615f Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S78259[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CMP749 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78260[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax A110 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S78261[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-N7000 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S78262[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-N7000 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S78263[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6120c) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S78264[] = "Mozilla/5.0 (Windows NT 5.1; rv:23.0) Gecko/X Firefox/23.0 slurp";
const char S78265[] = "Mozilla/5.0 (Windows NT 5.1; rv:23.0) Gecko/X Firefox/23.0 slurp slurp";
const char S78266[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; GT-N7000 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S78267[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-N7000 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S78268[] = "Mozilla/5.0 (Series40; Nokia109/04.12; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S78269[] = "SAMSUNG-SGH-T819/T819UVHB2 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S78270[] = "SonyEricssonW350c/R10BA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S78271[] = "SonyEricssonW350c_CMCC/R10BA SEMC-SW/R10BA Release/04.23.2008 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S78272[] = "SonyEricssonW350a/R11BA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S78273[] = "INQ/B1-200H/1.0.39 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S78274[] = "INQ/B1-200H/1.0.36 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S78275[] = "INQ/B1-200H/1.0.54 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S78276[] = "INQ/B1-200H/1.0.63 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S78277[] = "INQ/B1-200H/1.0.56 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S78278[] = "INQ/B1-200H/1.0.38 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S78279[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.0.4568 Chrome/7.0.517.440 Safari/534.7";
const char S78280[] = "Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux i686; en) Opera 8.51";
const char S78281[] = "Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux i686; en) Opera 8.52";
const char S78282[] = "Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux i686; en) Opera 8.54";
const char S78283[] = "Mozilla/5.0 (X11; Linux i686; U; en) Opera 8.52";
const char S78284[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; a9100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78285[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; A9100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78286[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; A9100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78287[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; nl-; A9100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78288[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; A9100C Build/GRJ90)";
const char S78289[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; A9100C Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78290[] = "JUC (Linux; U; 2.2.2; zh-cn; ME525; 480*854) UCWEB7.9.3.103/139/800";
const char S78291[] = "JUC (Linux; U; 2.2.2; zh-cn; MB525; 480*854) UCWEB7.9.4.145/139/800";
const char S78292[] = "JUC (Linux; U; 2.3.7; zh-cn; MB525; 480*854) UCWEB7.9.4.145/139/800";
const char S78293[] = "JUC (Linux; U; 2.3.6; zh-cn; ME525+; 480*854) UCWEB7.9.4.145/139/800";
const char S78294[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E15i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S78295[] = "Mozilla/5.0 (Linux; Android 4.0.3; SC-02C Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78296[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; A211 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78297[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; L-02D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78298[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-C800 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78299[] = "Nokia308/2.0 (05.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia308) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S78300[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Dslide 703R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78301[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EGS005 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78302[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78303[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78304[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78305[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; KM-S330 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78306[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Starmobile ENERGY Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78307[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Cynus T1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78308[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ThL W5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78309[] = "UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-914_im_india_269) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile";
const char S78310[] = "UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-885_im_india_249) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile";
const char S78311[] = "UCWEB/2.0 (Windows; U; wds 8.0; zh-CN; NOKIA; RM-822_apac_prc_204) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile";
const char S78312[] = "UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-821_im_india_443) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile";
const char S78313[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E Build/HuaweiU8666E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S78314[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E-51 Build/HuaweiU8666E-51) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S78315[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S78316[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SPH-L710 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S78317[] = "Mozilla/5.0(iPad; U;CPU OS 4_3 like Mac OS X; zh-CN; iPad2,1) AppleWebKit/534.46 (KHTML, like Gecko) UCBrowser/2.0.1.280 U3/0.8.0 Safari/7543.48.3";
const char S78318[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S78319[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L720 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/jfltespr";
const char S78320[] = "Mozilla/5.0 (Linux; Android 4.2.1; JY-G4 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S78321[] = "Mozilla/5.0 (Linux; Android 4.3; BN NookHD+ Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S78322[] = "NokiaE72";
const char S78323[] = "NokiaE72-1/UCBrowser7.7.0.81/28/800";
const char S78324[] = "NokiaE72-1";
const char S78325[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0) QQBrowser/6.11.12135.201";
const char S78326[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.11.12388.201";
const char S78327[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1 QQBrowser/6.11.11981.201";
const char S78328[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; qdesk 2.3.1186.202) QQBrowser/6.11.12388.201";
const char S78329[] = "Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia7210supernova) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S78330[] = "Nokia3110c/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia3110c) UCBrowser8.2.0.132/69/352/UCWEB Mobile";
const char S78331[] = "Mozilla/5.0 (X11; FreeBSD i386; rv:9.0.1) Gecko/X Firefox/9.0.1";
const char S78332[] = "Mozilla/4.0 (compatible; MSIE 4.0; Windows NT 4.0)";
const char S78333[] = "Mozilla/5.0 (compatible; MSIE 4.0; WinNT4.0; Trident/5.0)";
const char S78334[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HERO200 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78335[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78336[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9100 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78337[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78338[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S78339[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-S959G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S78340[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S78341[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; KIRA N9010 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78342[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; DEOX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78343[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; BioniQ 700 Build/ICS.f08refem32.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78344[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6.2";
const char S78345[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.2 Mobile Safari/535.19 Silk-Accelerated=false";
const char S78346[] = "Mozilla/5.0 (Java; U; en-us; gt-s5230) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S78347[] = "Mozilla/5.0 (Java; U; fa; gt-s5230) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile";
const char S78348[] = "Mozilla/5.0 (Java; U; en-us; samsung-s5230) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S78349[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Scroll Evoke Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78350[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TOUCH90 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78351[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A21 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S78352[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Karbonn A21 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S78353[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A21 Build/IMM76I) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S78354[] = "Mozilla/5.0 (Linux;U; Android 4.0.4; en-us; MID102C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78355[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE V955 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78356[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ZTE V955 Build/IMM76I)";
const char S78357[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7102 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S78358[] = "Mozilla/5.0 (Linux; Android 4.1.9; GT-N7102 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S78359[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F180L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S78360[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F180K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S78361[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F180S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S78362[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9103 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S78363[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; CTAB712 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78364[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; Cink King Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78365[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CINK KING Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78366[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/9B206 Safari/7534.48.3";
const char S78367[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/9B176 Safari/7534.48.3";
const char S78368[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Incredible S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S78369[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S78370[] = "Mozilla/5.0 (Linux; Android 4.2.2; A500 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S78371[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I605 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S78372[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; TRUE BEYOND TAB 3G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78373[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; TRUE BEYOND TAB 3G Build/JRO03C)";
const char S78374[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-M840 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S78375[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT910 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S78376[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 992D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S78377[] = "SAMSUNG-GT-E2330/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S78378[] = "SAMSUNG-M2710/M2710DDIF4 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S78379[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6.1 GTB7.1";
const char S78380[] = "Mozilla/5.0 (Linux; Android 4.0.4;  HTC Vision Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S78381[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Vision Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78382[] = "Mozilla/5.0 (Windows NT 6.1; rv:11.0) Gecko/X Firefox/11.0 slurp";
const char S78383[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:11.0) Gecko/X Firefox/11.0 slurp";
const char S78384[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:11.0) Gecko/X Firefox/11.0 slurp slurp";
const char S78385[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S78386[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MP959 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78387[] = "Mozilla/5.0 (Linux; Android 4.0.3; Woxter Tablet PC 97IPS Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S78388[] = "Mozilla/5.0 (Linux; Android 4.0.4; Woxter Tablet PC 97IPS Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S78389[] = "Mozilla/5.0 (Linux; Android 4.0.4; Woxter Tablet PC 97IPS DUAL Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S78390[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; LNV-Lenovo_A560e Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78391[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; LNV-Lenovo_A560e Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S78392[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; LNV-Lenovo_A560e Build/GINGERBREAD)";
const char S78393[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5620) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/405/UCWEB Mobile";
const char S78394[] = "OneBrowser/3.1 (Nokia302/14.26)";
const char S78395[] = "NOKIA-Lumia 800/UCWEB2.8.0.249/47/355";
const char S78396[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; LG-E612 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S78397[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; LG-E610 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S78398[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; LG-E612f Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S78399[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5100 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78400[] = "OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78401[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Sensation 4G Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S78402[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Sensation Z710e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S78403[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT26ii Build/6.1.A.2.55) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S78404[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LT26ii Build/6.1.A.2.45) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31";
const char S78405[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300";
const char S78406[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; M6589 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78407[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HYUNDAI T7S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78408[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S78409[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S78410[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E988 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S78411[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E988 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S78412[] = "Mozilla/5.0 (Linux; Android 4.1.2; PMP7280C3G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S78413[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E440 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S78414[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E440 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S78415[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-US; Nexus 4 Build/JWR66V) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S78416[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-US; Nexus 4 Build/JWR66Y) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S78417[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5360 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S78418[] = "Mozilla/5.0 (Series40; NokiaC2-00/03.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S78419[] = "Mozilla/5.0 (Series40; Nokia7230/10.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.0.9.17.1";
const char S78420[] = "Mozilla/5.0 (Series40; Nokia7230-1c/10.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.0.9.17.1";
const char S78421[] = "ZTE-R101/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S78422[] = "DoCoMo/2.0 SH07B(c500;TB;W24H16)";
const char S78423[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Comodo_Dragon/6.0.0.10 Chrome/6.0.472.63 Safari/534.3";
const char S78424[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SHW-M290K Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78425[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Micromax_A70 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78426[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonS51SE Build/4.0.2.A.0.61) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78427[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonS51SE Build/4.0.2.A.0.74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78428[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonS51SE Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78429[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; S51SE Build/4.0.2.A.0.74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78430[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9pre) Gecko/X Thunderbird/3.0a1";
const char S78431[] = "NokiaC2-02/2.0 (06.52) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac2-02) UCBrowser8.2.1.144/70/355/UCWEB Mobile";
const char S78432[] = "Mozilla/4.0 (compatible; MSIE 5.05; Windows NT 3.51)";
const char S78433[] = "Mozilla/5.0 (compatible; MSIE 5.0; WinNT3.51; Trident/1.0)";
const char S78434[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; MP700 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78435[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;7 Mozart;7.10;7740)";
const char S78436[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;T8697;7.10;7720)";
const char S78437[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;7 Mozart;7.10;7720)";
const char S78438[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;T8697;7.10;8107)";
const char S78439[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;7 Mozart;7.10;8107)";
const char S78440[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;7 Mozart T8698;7.10;7740)";
const char S78441[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;7 Mozart T8698;7.10;8107)";
const char S78442[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;T8697;7.10;8773)";
const char S78443[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;7 Mozart T8698;7.10;8773)";
const char S78444[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Raider X710e Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78445[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Holiday Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78446[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S78447[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S78448[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S78449[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-P1000T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/405";
const char S78450[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-P1000R Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S78451[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; SLCC2)/UCWEB7.4.0.57/31/999";
const char S78452[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; .NET CLR 2.0.50727)/UCWEB7.4.0.57/31/999";
const char S78453[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; SV1)/UCWEB7.4.0.57/31/999";
const char S78454[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; InfoPath.2)/UCWEB7.4.0.57/31/999";
const char S78455[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; .NET4.0E)/UCWEB7.4.0.57/31/999";
const char S78456[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB7.4.0.57/31/999";
const char S78457[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MP1000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78458[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Spice Mi-425 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78459[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-b5310) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S78460[] = "Instagram 3.2.0 (iPhone4,1; iPhone OS 5.1.1; en_US) AppleWebKit/420+";
const char S78461[] = "Instagram 3.2.0 (iPhone4,1; iPhone OS 5.1; en_US) AppleWebKit/420+";
const char S78462[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7511 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S78463[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; M2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78464[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; M2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78465[] = "Mozilla/5.0 (Linux; U; Android 4.0; zh-cn; M2 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78466[] = "Mozilla/5.0 (Windows NT 5.1; rv:18.0) Gecko/X Firefox/18.0 slurp";
const char S78467[] = "Mozilla/5.0 (Windows NT 5.1; rv:18.0) Gecko/X Firefox/18.0 slurp slurp";
const char S78468[] = "Mozilla/5.0 (Linux; Android 4.1.2; SH530U Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78469[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 930D Build/1.2.10.4) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78470[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V1 Build/HCL ME Tablet V1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78471[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CONNECT 2G Build/HCL ME Tablet CONNECT 2G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78472[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S78473[] = "Mozilla/5.0 (Linux; Android 4.2.1; ME301T Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S78474[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID7065 Build/ICS.MID7065.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S78475[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TBDC1093 Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78476[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S78477[] = "Mozilla/5.0 (Linux; Android 4.2.2; A1-810 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S78478[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) PPC; 240x320; HTC P3450; OpVer 23.114.15.752";
const char S78479[] = "PPC; 240x320; HTC_P3450/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S78480[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) PPC; 240x320; HTC P3450; OpVer 23.114.3.634";
const char S78481[] = "HTC-P3450-orange/PPC; 240x320; OpVer 23.114.2.741 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S78482[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) PPC; 240x320; HTC P3450; OpVer 23.116.1.611";
const char S78483[] = "SAMSUNG-GT-E3213K/E3213KJPKD1 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S78484[] = "Opera/9.50 (Linux mips ; U; CE-HTML/1.0 (<profilelist><ui_profile name=\"PHILIPS_OLS_2008\"/></profilelist>); en)";
const char S78485[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB200 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78486[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB200 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78487[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; MB200 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78488[] = "Mozilla/5.0 (Linux; U; PocketEdge ERMINE-0.9; Android 2.2.1; en-us; edgejr Build/FRG83) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S78489[] = "Mozilla/5.0 (Android; Mobile; rv:11.0) Gecko/11.0 Firefox/11.0 Fennec/11.0";
const char S78490[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; T9199+ Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78491[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; T9199+ Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78492[] = "Mozilla/5.0 (X11; Linux i686; rv:14.0) Gecko/X Thunderbird/14.0";
const char S78493[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TAB-711 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78494[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Precedent Build/AreaRom Test)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78495[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Precedent Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78496[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; HYPERION 8 Mod by echcon04 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78497[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S78498[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; Evodroid 7.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78499[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; eVodroid U7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78500[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; Evodroid W10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78501[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus S 4G Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78502[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus S 4G Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78503[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-be; Lenco TAB-712 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78504[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E615 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S78505[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E615f Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S78506[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMID705X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78507[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMID705 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78508[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; PMID705 Build/JRO03C)";
const char S78509[] = "Mozilla/5.0 (Linux; Android 4.1.1; IRIS 455 Build/LAVAIRIS455) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78510[] = "Mozilla/5.0 (Linux; Android 2.3.7; ST25i Build/6.0.B.3.184) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S78511[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; iBall Slide i9702 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78512[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; iBall Slide 6309i Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78513[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; iBall Slide 6318i Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78514[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Intex Aqua Superb Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78515[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-L40G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S78516[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P715 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S78517[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Karbonn S2 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78518[] = "Mail/1508 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro10%2C1)";
const char S78519[] = "Mail/1508 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookAir5%2C2)";
const char S78520[] = "Mail/1508 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (iMac13%2C1)";
const char S78521[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SGP312 Build/10.3.1.C.0.136) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78522[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGP312 Build/10.3.1.C.0.136)";
const char S78523[] = "SAMSUNG-SGH-E370-VODA/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S78524[] = "SAMSUNG-SGH-E370/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S78525[] = "Opera/9.80 (X11; Linux i686; U; Linux Mint; en) Presto/2.10.229 Version/11.60";
const char S78526[] = "Mozilla/5.0 (Windows; U; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.1.2 Safari/525.21";
const char S78527[] = "Mozilla/5.0 (Windows NT; U; en) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Iris/1.1.7 Safari/525.20";
const char S78528[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; de; rv:1.9.0.15) Gecko/X Firefox 2.1 (.NET CLR 3.5.30729)";
const char S78529[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; de; rv:1.9.0.15) Gecko/X Firefox 2.1 (.NET CLR 3.5.30729)	Firefox 2.1";
const char S78530[] = "Spice M6700";
const char S78531[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;mwp6985;7.10;7720)";
const char S78532[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;mwp6985;7.10;8107)";
const char S78533[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;mwp6985;7.10;8773)";
const char S78534[] = "Opera/9.80 (Android 3.1; Linux; Opera Tablet/ADR-X; U; fr) Presto/2.9.201 Version/11.50";
const char S78535[] = "samsung-gt-s5263/UC Browser8.0.3.107/69/444";
const char S78536[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11";
const char S78537[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11";
const char S78538[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11";
const char S78539[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11";
const char S78540[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; LG-E400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S78541[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78542[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Nexus 7 Build/IMM76D)";
const char S78543[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; Nexus 7 Build/XtmTR) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78544[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus 7 Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78545[] = "Mozilla/5.0 (Linux; Android 4.0.3; INO_ONE Build/INO_ONEINO_ONE) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78546[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT535 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78547[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; F9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78548[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9000 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S78549[] = "Mozilla/5.0 (Linux; Android 4.1.1; A211 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S78550[] = "Mozilla/5.0 (compatible; MSIE 8.0; WinNT3.51; Trident/5.0)";
const char S78551[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P870 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78552[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XOLO B700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78553[] = "UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-846_im_mea3_237) U2/1.0.0 UCBrowser/3.0.0.285 U2/1.0.0 Mobile";
const char S78554[] = "UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-846_im_india_233) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile";
const char S78555[] = "Mozilla/4.7C-CCK-MCD {C-UDP; EBM-APPLE} (Macintosh; I; PPC)','Mozilla/4.8 [en] (Windows NT 5.0; U)";
const char S78556[] = "Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6303iclassic) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S78557[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Incredible S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S78558[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC J Z321e Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S78559[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825-1 Build/HuaweiU8825-1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S78560[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S78561[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6102) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S78562[] = "Mozilla/5.0 (Linux; Android 4.2.2; U30GT 2 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S78563[] = "Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S78564[] = "LG-KM570-Bouygues/V10c Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S78565[] = "LG-KM570-orange/V10e Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S78566[] = "LG-KM570/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S78567[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) LG-GW550 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S78568[] = "LG-GW550 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S78569[] = "LG-GU220/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S78570[] = "SAMSUNG-SGH-i900/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S78571[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7A341";
const char S78572[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko)";
const char S78573[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/528.7 (KHTML, like Gecko) Iron/1.0.155.0 Safari/528.7";
const char S78574[] = "Nokia5235/UC Browser7.9.0.102/50/351/UCWEB";
const char S78575[] = "Mozilla/5.0 (S60V3; U; en-in; NokiaE71)/UC Browser8.4.0.159/28/352/UCWEB Mobile";
const char S78576[] = "NokiaE71/UCWEB8.4.0.150/28/800";
const char S78577[] = "Mozilla/5.0 (Java; U; en-us; nokiae71-1) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S78578[] = "Mozilla/5.0 (Windows; U; Win95; en-US; rv:1.7.5) Gecko/X Firefox/1.0";
const char S78579[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I997 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78580[] = "NokiaN97 mini/UCWEB8.5.0.163/50/800";
const char S78581[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaN97 mini)/UC Browser8.5.0.183/50/355/UCWEB Mobile";
const char S78582[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; BLU TOUCH BOOK 7.0 Build/1.0.8265.0448) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78583[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC_Z560e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78584[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Z560e Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S78585[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Z560e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78586[] = "Opera/9.80 (Windows NT 6.2; U; en) Presto/2.2.15 Version/10.10";
const char S78587[] = "Mozilla/5.0 (Series40; Nokia5130c-2/07.95; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S78588[] = "Mozilla/5.0 (Linux; Android 4.1.1; A200 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S78589[] = "Mozilla/5.0 (Linux; Android 4.1.2; a200 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S78590[] = "Mozilla/5.0 (Java; U; en-in; nokia500) UCBrowser8.4.0.159/69/444/UCWEB Mobile  3gpp-gba";
const char S78591[] = "Mozilla/5.0 (Linux; Android 4.0.4; Optimus_Madrid Build/IceCreamSandwich) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78592[] = "Opera/9.80 (Windows NT 6.1; WOW64; Opera Mobi/46154) Presto/2.11.355 Version/12.10";
const char S78593[] = "Mozilla/5.0 (X11; Linux x86_64; PantechP8010 Build/IMM76D) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S78594[] = "Mozilla/5.0 (X11; Linux x86_64; PantechP8010 Build/JZO54K) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S78595[] = "Lenovo-A798t_TD/S100 Linux/3.0.13 Android/4.0 Release/05.16.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2";
const char S78596[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.482; en-us; 9520/4.2.0.482) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S78597[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.189; en-us; 9520/4.2.0.189) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S78598[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.323; en-us; 9520/4.2.0.323) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S78599[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.423; en-us; 9520/4.2.0.423) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S78600[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.220; en-us; 9520/4.2.0.220) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S78601[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.422; en-us; 8530/4.2.0.422) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S78602[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.451; en-us; 8530/4.2.0.451) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S78603[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.298; en-us; 8530/4.2.0.298) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S78604[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.335; en-us; 8530/4.2.0.335) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S78605[] = "Mozilla/5.0 (Linux; Android 4.1.1; C1505 Build/11.3.A.0.47) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S78606[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P765 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S78607[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; eZee' Tab 804 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78608[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; eZee'Tab804 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78609[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; TAC-70072 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78610[] = "Mozilla/5.0 (Linux; Android 4.2.1; iOCEAN X7 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S78611[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5282 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S78612[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-S5282 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S78613[] = "Mozilla/5.0 (Series40; Nokia2700c-2/09.97; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S78614[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E986 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S78615[] = "Mozilla/5.0 (Linux; Android 4.1.1; M040 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S78616[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCL21 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S78617[] = "Mozilla/5.0 (HW-C8600/C8600V100R001C92B225;U;Android/2.1;320*480;CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S78618[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5; XV6175.1)";
const char S78619[] = "NokiaN8-00/UC Browser8.0.4.121/50/444/UCWEB";
const char S78620[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; fr; rv:1.9.0.5) Gecko/X Firefox/3.0.5 Flock/2.0.3";
const char S78621[] = "Mozilla/5.0 (iPad; CPU 0S 5_0_1 like Mac OS X) AppleWebkit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3";
const char S78622[] = "Mozilla/5.0 (iPad; CPU 6_1_3 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S78623[] = "YelpWebView/1.0 ipad/5_1_1 YelpApp/6.5.0 (x-screen-size 1.0;)";
const char S78624[] = "YelpWebView/1.0 ipad/5_1_1 YelpApp/6.7.0 (x-screen-size 1.0;)";
const char S78625[] = "YelpWebView/1.0 ipad/5_1_1 YelpApp/7.0.1 (x-screen-size 1.0;)";
const char S78626[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.1";
const char S78627[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.2";
const char S78628[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.4";
const char S78629[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.5";
const char S78630[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.6";
const char S78631[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Flipboard/1.9.6";
const char S78632[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Flipboard/1.9.6";
const char S78633[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.7";
const char S78634[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Flipboard/1.9.7";
const char S78635[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.8";
const char S78636[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Flipboard/1.9.8";
const char S78637[] = "MicromaxX271/1.1.2011 Browser/OBIGO_Q03C MMS/OBIGO_Q03C_MMS_V02";
const char S78638[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TAB464 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78639[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; TAB464 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78640[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; MT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S78641[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7005 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78642[] = "Mozilla/5.0 (Symbian3; U; en-us; Nokia701) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/51/355/UCWEB Mobile";
const char S78643[] = "Mozilla/5.0 (Linux; Android 4.0.4; B92 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78644[] = "Mozilla/5.0 (Linux; Android 4.0.4; AD683G Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78645[] = "SAMSUNG-GT-C3260/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S78646[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; eZee'Tab803 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78647[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAN78) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S78648[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/9B206 Safari/7534.48.3";
const char S78649[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/9B176 Safari/7534.48.3";
const char S78650[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-c3222) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S78651[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-c3222) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile, SAMSUNG-GT-C3222/C3222DDJL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S78652[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A850K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S78653[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A850L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S78654[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A850S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S78655[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT29i Build/7.0.A.3.212) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S78656[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; Fly IQ430 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78657[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; IQ430 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78658[] = "Mozilla/5.0 (Linux; Android 4.0.3; NS-13T001 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S78659[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T679 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78660[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T679 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78661[] = "Mozilla/5.0 (Linux; Android 4.3; GT-I9505G Build/JWR66N.S005) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S78662[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E445 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78663[] = "Mozilla/5.0 (Linux; Android 4.2.1; A210 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S78664[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-04D Build/9.1.C.0.475) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S78665[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; ZTE-U X850 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78666[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; ZTE X850 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78667[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.10 (KHTML, like Gecko) RockMelt/0.8.40.147 Chrome/8.0.552.231 Safari/534.10";
const char S78668[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.7 (KHTML, like Gecko) RockMelt/0.8.36.128 Chrome/7.0.517.44 Safari/534.7";
const char S78669[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i677)";
const char S78670[] = "Mozilla/4.0 (Brew MP 1.0.2; U; en-us; Kyocera; NetFront/4.1/AMB) Sprint E4233";
const char S78671[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-b5310) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S78672[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-b5310) UCBrowser8.3.1.161/70/405/UCWEB Mobile";
const char S78673[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyTouch 4g Build/AwesomeICS) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78674[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyTouch 4G Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78675[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC myTouch 4g Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78676[] = "Mozilla/5.0 (Linux; Android 4.0.4; T-Mobile myTouch 4G Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78677[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-LW690 Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78678[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-LW690 Build/GWK74; CyanogenMod 7.2 ported by: Primed and sammyz) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78679[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Lenovo A60 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78680[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A60+ Build/LenovoLePhone) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78681[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile";
const char S78682[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78683[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78684[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78685[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78686[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; DPS Dream 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78687[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78688[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC One V Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S78689[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2a1pre) Gecko/X Minefield/3.6a1pre";
const char S78690[] = "Mozilla/5.0 (Linux; Android 4.0.4; VSD220 Build/IMM76D.UI23E802_VSC) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S78691[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Viettel I9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78692[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78693[] = "Mozilla/5.0 (Windows NT 6.0; rv:16.0) Gecko/X Firefox/16.0 epic/16.0";
const char S78694[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:16.0) Gecko/X Firefox/16.0 Epic/16.0";
const char S78695[] = "Mozilla/5.0 (Linux; Android 4.2.1; SAMSUNG-SGH-I717 Build/JOP40G) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78696[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I717 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78697[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I605 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S78698[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I605 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S78699[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_One_X; en-us) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S78700[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_One_X; en-au) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S78701[] = "Mozilla/5.0 (Linux; Android 4.0.3; MD_LIFETAB_P9516 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S78702[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S78703[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000 Build/GINGERBREAD) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S78704[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000L Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S78705[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; GT-I9001) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S78706[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT875 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S78707[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT200 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S78708[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One V Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78709[] = "Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; Lenovo-A3000-H/S100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S78710[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6312 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S78711[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax A110 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S78712[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I997 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S78713[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-I997 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S78714[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAP4044DUO Build/PrestigioPAP4044DUO) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78715[] = "Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook Tablet Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S78716[] = "Mozilla/5.0 (Linux; Android 4.0.4; EVO Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S78717[] = "Mozilla/5.0 (Linux; Android 4.0.3; EVO Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S78718[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6102) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S78719[] = "HTC_Touch_Diamond2_T5353 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) HTC_Touch_Diamond2_T5353 Mozilla/4.0";
const char S78720[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_Touch_Diamond2/2.16.162.3 (67041)";
const char S78721[] = "HTC_Touch_Diamond2_T5353 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S78722[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_Touch_Diamond2/2.16.163.3 (67041)";
const char S78723[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_Touch_Diamond2/2.16.164.1 (67041)";
const char S78724[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 480x800; HTC_Touch_Diamond2_T5353-Orange; OpVer 93.172.15.752";
const char S78725[] = "LG-GB230/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S78726[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.7.62 Version/11.01";
const char S78727[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.7.62 Version/11.00";
const char S78728[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-P1000 Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78729[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; GT-P1000 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78730[] = "MicromaxX231/Q03C MAUI-browser/";
const char S78731[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 8.0; Trident/5.0; IEMobile/9.0; Asus;Galaxy6)";
const char S78732[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 8.0; Trident/5.0; IEMobile/9.0; WpsLondonTest; Asus;Galaxy6)";
const char S78733[] = "Mozilla/5.0 (Linux; Android 4.0.3; T10A Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S78734[] = "Mozilla/5.0 (Linux; Android 4.0.4; T10A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S78735[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; Edition MAS; en) Presto/2.10.229 Version/11.66";
const char S78736[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I757M Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78737[] = "JUC (Linux; U; 2.3.6; zh-cn; Micromax_A50; 240*320) UCWEB7.9.0.94/139/444";
const char S78738[] = "Mozilla/5.0 (Linux; Android 4.0.4; YP-GP1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78739[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SpiceMi1010 Build/SpiceMi1010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78740[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.3 Mobile/9A405 Safari/7534.48.3";
const char S78741[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.3 Mobile/9A334 Safari/7534.48.3";
const char S78742[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT15i Build/4.1.B.0.587) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S78743[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9308 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S78744[] = "Opera/9.80 (Android 4.1.1; Linux; Opera Mobi/ADR-X; U; en) Presto/2.8.149 Version/11.10";
const char S78745[] = "Opera/9.80 (Android 4.1.2; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.8.149 Version/11.10";
const char S78746[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT22i Build/6.1.1.B.1.54) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S78747[] = "NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-02) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S78748[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC5-06) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S78749[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; SKY HD9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78750[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; SKY HD9500 Plus Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78751[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S78752[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xenta TAB13-201 Build/Xenta TAB13-201) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78753[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6503 Build/10.1.A.1.434) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S78754[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S78755[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S78756[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9500 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S78757[] = "Mozilla/5.0 (Linux; Android 4.1.2; SOL22 Build/10.2.F.3.43) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S78758[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P870 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S78759[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB7.0.0.41/31/352";
const char S78760[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB7.0.2.37/31/800";
const char S78761[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB7.0.0.41/30/351";
const char S78762[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)/UCWEB7.0.0.41/30/351";
const char S78763[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB7.0.0.33/31/800";
const char S78764[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MOT-XT800+/TTUPG_M6_1.200.36.7; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78765[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MOT-XT800+/TTUPG_M6_2.200.15.1; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78766[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MOT-XT800+/TTUPG_M6_2.200.13; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78767[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MOT-XT800+/TTUPG_M6_2.200.21; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78768[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; XT800+ Build/TTUPG_M6_2.200.15.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78769[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; XT800+ Build/TTUPG_M6_1.200.36.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78770[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; XT800+ Build/TTUPG_M6_2.200.15.1) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S78771[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; XT800+ Build/TTUPG_M6_2.200.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78772[] = "Mozilla/5.0 (Series40; Nokia200/10.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15";
const char S78773[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SPICE Mi-720 Build/Spice_Mi-720_IND_V08_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78774[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Asus;Galaxy6)";
const char S78775[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; WpsLondonTest; Asus;Galaxy6)";
const char S78776[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Dell Streak Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78777[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Dell Streak Build/GRI40; PureCyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78778[] = "Mozilla/5.0 (Linux; Android 4.0.3; PMP5080CPRO Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S78779[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; RMD-1035 Build/ICS.g12refM1005RU.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78780[] = "Nokia112/2.0 (03.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia112) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S78781[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; iDxD7 3G Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78782[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; iDxD7 3G Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78783[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D2-911 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78784[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; HTC_Wildfire) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S78785[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A850K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S78786[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A850L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S78787[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A850S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S78788[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Karbonn_A2) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S78789[] = "Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 3 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78790[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-7204 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78791[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Micromax A116) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S78792[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S78793[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L900 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S78794[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S78795[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE75-1) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S78796[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE75-2) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S78797[] = "Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S78798[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9308 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S78799[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One XL Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78800[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; tr-tr; GT-N5105 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78801[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9300) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S78802[] = "SAMSUNG-E2100B/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S78803[] = "HTC-ST7377/2.10.502.4 (93577) Opera/9.7 (Windows NT 5.1; U; en)";
const char S78804[] = "Nokia5800 XpressMusic/UC Browser7.7.1.88/50/444";
const char S78805[] = "nokia5800d-1/UC Browser7.7.1.88/69/444  3gpp-gba";
const char S78806[] = "KDDI-CA3I UP.Browser/6.2_7.2.7.1.K.5.214 (GUI) MMP/2.0";
const char S78807[] = "NokiaX2-02/2.0 (11.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-02/UC Browser8.0.3.107/69/444";
const char S78808[] = "Mozilla/5.0 (Windows NT 5.1; WOW64; rv:11.0) Gecko/X Firefox/11.0 SeaMonkey/2.8";
const char S78809[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:11.0) Gecko/X Firefox/11.0 SeaMonkey/2.8";
const char S78810[] = "Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko/X SeaMonkey/2.8";
const char S78811[] = "Opera/9.80 (Linux armv6l; U; en) Presto/2.8.115 Version/11.10";
const char S78812[] = "Mozilla/5.0 (Linux; Android 4.0.4;  U8800 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19";
const char S78813[] = "Mozilla/5.0 (Linux; Android 4.0.4; U8800 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78814[] = "Mozilla/5.0 (Linux; Android 4.0.3; U8800 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78815[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352";
const char S78816[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352";
const char S78817[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S78818[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.4) Gecko/X Firefox/2.0.0.4 Flock/0.9.0";
const char S78819[] = "Mozilla/5.0 (Linux; Android 4.1.1; Malata SMBA1002 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S78820[] = "Mozilla/5.0 (Linux; Android 4.1.2; Malata SMBA1002 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S78821[] = "Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/352/UCWEB Mobile";
const char S78822[] = "SAMSUNG-GT-E3309/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S78823[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Inspire 4G Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78824[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Inspire 4G Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78825[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; SP704CE Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 TabletSafari/534.30";
const char S78826[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S78827[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P6800 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S78828[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6800 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S78829[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6800 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S78830[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; A54) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S78831[] = "Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S78832[] = "Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S78833[] = "Mozilla/5.0 (Linux; Android 4.0.4; mid-A13 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S78834[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One V Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78835[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One V Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78836[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NEBULA6.9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78837[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; TECNO N3 Build/master) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S78838[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S5303) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S78839[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; U705T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78840[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; U705T Build/JZO54K)";
const char S78841[] = "Mozilla/5.0 (Linux; Android 4.0.4; JY-G3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S78842[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; Micromax A110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S78843[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I747 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S78844[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L710 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S78845[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-be; Lenco CoolTab-80 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78846[] = "Mozilla/5.0 (Linux; Android 4.0.4; CINK PEAX Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S78847[] = "Mozilla/5.0 (Linux; Android 4.1.2; Micromax A116 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S78848[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S78849[] = "Mozilla/5.0 (Linux; Android 4.1.2; Lenovo A706_ROW Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S78850[] = "UCWEB/2.0 (iOS; U; iPh OS 6_1_2; zh-CN; iPh4,1) U2/1.0.0 UCBrowser/9.1.0.287 U2/1.0.0 Mobile";
const char S78851[] = "UCWEB/2.0 (iOS; U; iPh OS 6_1_2; zh-CN; iPh3,1) U2/1.0.0 UCBrowser/9.1.0.287 U2/1.0.0 Mobile";
const char S78852[] = "UCWEB/2.0 (iOS; U; iPh OS 6_1_2; zh-CN; iPh5,2) U2/1.0.0 UCBrowser/9.1.0.287 U2/1.0.0 Mobile";
const char S78853[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One XL Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S78854[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One XL Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S78855[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i2) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S78856[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S78857[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825-1 Build/HuaweiU8825-1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S78858[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S78859[] = "Mozilla/5.0 (Linux; Android 4.1.2; SH-02E Build/S6210) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S78860[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Micromax A116 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S78861[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S78862[] = "SonyEricssonK800iv/R1KG SEMC/NewsReader/1.0";
const char S78863[] = "SonyEricssonK800i/R8BF Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S78864[] = "SpiderSolitaireFree/1.5 CFNetwork/485.13.9 Darwin/11.0.0";
const char S78865[] = "SpiderSolitaireFree/2.1.3 CFNetwork/548.1.4 Darwin/11.0.0";
const char S78866[] = "NokiaC2-00/2.0 (03.99) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-00/UC Browser8.0.3.107/69/444";
const char S78867[] = "Mozilla/5.0 (Java; U; en-in; nokian8-00) UCBrowser8.2.1.144/69/444/UCWEB Mobile  3gpp-gba";
const char S78868[] = "NokiaN8-00/UCWEB8.2.0.116/51/999";
const char S78869[] = "Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7210supernova/UC Browser8.0.3.107/69/444";
const char S78870[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-01D Build/4.0.1.C.1.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78871[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-01D Build/4.0.1.C.0.86) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78872[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SO-01D Build/4.0.1.C.1.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78873[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-01D Build/4.0.1.C.1.31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78874[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaN97)/UC Browser8.2.0.132/50/355/UCWEB Mobile";
const char S78875[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-b7722) UCBrowser8.3.1.161/69/405/UCWEB Mobile";
const char S78876[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-b7722) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S78877[] = "Mozilla/5.0 (Linux; Android 4.0.4; E330 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78878[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S78879[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S78880[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.0.181/145/352";
const char S78881[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S78882[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78883[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78884[] = "MicromaxX278/MstarBrowser1.0/MIDP2.0Configuration/CLDC1.1";
const char S78885[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78886[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78887[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78888[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZIGO EON5I Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78889[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EON5I Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78890[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC_Desire_C; 320*480) UCWEB7.9.0.94/139/352";
const char S78891[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC_Desire_C; 320*480) UCWEB7.9.0.94/139/355";
const char S78892[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC_Desire_C; 320*480) UCWEB7.9.0.94/139/444";
const char S78893[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; ADT 2.5 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78894[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST70104-1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78895[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78896[] = "MQQBrowser/3.7/Adr (Linux; U; 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD.ZCLA4;320*480)";
const char S78897[] = "MQQBrowser/3.7/Adr (Linux; U; 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD.ZCLL3;320*480)";
const char S78898[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.2.149.27 Safari/525.13";
const char S78899[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.4.154.25 Safari/525.19";
const char S78900[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/0.0.0.0 Safari/534.24";
const char S78901[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I659 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78902[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M460D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78903[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-N7000 Build/IML74K; XPath_Offline) AppleWebKit/533.1 (KHTML, like Gecko) Mobile MQQBrowser/4.0 Safari/533.1";
const char S78904[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CT910 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78905[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Celkon A 225 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78906[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S78907[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; GT-P3100 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7";
const char S78908[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S78909[] = "Mozilla/5.0 (Linux; Android 4.1.1; R8113 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78910[] = "Nokia112/2.0 (03.32) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia112) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S78911[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A830K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S78912[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A830L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S78913[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A830S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S78914[] = "Mozilla/5.0 (compatible; MSIE 10.0; Win95; Trident/4.0)";
const char S78915[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-SU640 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78916[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; Archos 80 Xenon Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78917[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9505 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S78918[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 7_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/11A465 Safari/7534.48.3";
const char S78919[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 7_0_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/11A501 Safari/7534.48.3";
const char S78920[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Samsung-SCHI910 PPC 240x400";
const char S78921[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) VZW:SCH-i910 PPC 240x400";
const char S78922[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Samsung-SCHI910 BMC PPC 240x400";
const char S78923[] = "Treo750 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S78924[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) T-Mobile_Atlas";
const char S78925[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) PPC; MDA Touch/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S78926[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) HTC/X02HT";
const char S78927[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) 320x320; VZW; UTStar-XV6800; Window Mobile 6.0 Professional;";
const char S78928[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Holleycomm-H8800/2.0 WAP2.0";
const char S78929[] = "SonyEricssonZ710i/R1JC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S78930[] = "SonyEricssonW900i/R5BC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S78931[] = "Mozilla/5.0 (X11; U; Linux ARM; en-us) AppleWebKit/525.1+ (KHTML, like Gecko) Safari/525 Mobile/MotorolaEvoke";
const char S78932[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; en) Opera 8.53";
const char S78933[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; ru) Opera 8.50";
const char S78934[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; en) Opera 8.52";
const char S78935[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; ru) Opera 8.54";
const char S78936[] = "Mozilla/5.0 (Android; Tablet; rv:13.0) Gecko/13.0 Firefox/13.0a2";
const char S78937[] = "Mozilla/5.0 (Android; Tablet; rv:13.0) Gecko/13.0 Firefox/13.0 Fennec/13.0";
const char S78938[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11";
const char S78939[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11";
const char S78940[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; 101N Build/AX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78941[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; 101N Build/AX)";
const char S78942[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB200 Build/GRJ22; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78943[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; IM-A725L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78944[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; F8 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78945[] = "Mozilla/5.0 (Linux; Android 4.0.3; A101 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S78946[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78947[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; OPPOT29 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78948[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ASUS Transformer Pad TF700T Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78949[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ASUS Transformer Pad TF700T Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78950[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ASUS Transformer Pad TF700T Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1.2/tf700t";
const char S78951[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MiniPro Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S78952[] = "Mozilla/5.0 (Windows NT 5.1; rv:20.0) Gecko/X Firefox/20.0 slurp";
const char S78953[] = "Mozilla/5.0 (Windows NT 5.1; rv:20.0) Gecko/X Firefox/20.0 slurp slurp";
const char S78954[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9300) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S78955[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; Galaxy Nexus Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S78956[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Galaxy Nexus Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S78957[] = "Instagram 4.0.2 (iPhone4,1; iPhone OS 6_1_3; en_US; en) AppleWebKit/420+";
const char S78958[] = "Instagram 4.0.2 (iPhone5,2; iPhone OS 6_1_4; en_US; en) AppleWebKit/420+";
const char S78959[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I8160 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S78960[] = "Mozilla/5.0 (Linux; Android 4.1.2; JY-G3 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S78961[] = "Mozilla/5.0 (Linux; Android 4.1.2; JY-G3N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S78962[] = "Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook Tablet Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S78963[] = "Mozilla/5.0 (Linux; Android 4.2.2; NookTablet Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S78964[] = "Mozilla/5.0 (Linux; Android 4.0.3; MD_LIFETAB_P9516 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S78965[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; EVO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jewel";
const char S78966[] = "LGE-AX8370/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S78967[] = "LGE-MX200/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S78968[] = "LGE-MX500/1.0 UP.Browser/6.2.3.2 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S78969[] = "LGE-AX245/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S78970[] = "LGE-MX510/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S78971[] = "LG285/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S78972[] = "LGE-MX8550/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S78973[] = "LGE-UN160/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S78974[] = "HTC_P3490 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S78975[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Sprint MP6950SP";
const char S78976[] = "Treo800w/v0100 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE, PPC; 320x320) (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S78977[] = "Opera/6.11 (Linux 2.4.10-4GB i686; U) [en]";
const char S78978[] = "Opera/6.11 (Linux 2.4.18-4GB i686; U) [en]";
const char S78979[] = "Mozilla/4.0 (compatible; MSIE 5.0; Linux 2.4.20-4GB i686) Opera 6.12 [de]";
const char S78980[] = "Opera/6.12 (Linux 2.4.18-14cpq i686; U) [en]";
const char S78981[] = "Opera/6.12 (Linux 2.4.20-4GB i686; U) [en]";
const char S78982[] = "Mozilla/4.0 (compatible; MSIE 5.0; Linux 2.4.4 i686) Opera 6.11 [en]";
const char S78983[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-T U880 Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S78984[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-T U880 Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S78985[] = "Mozilla 5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-T U880 Build FRG83G) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S78986[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1.15) Gecko/X Fedora/2.0.10-1.fc12 SeaMonkey/2.0.10";
const char S78987[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.19) Gecko/X Fedora/2.0.14-1.fc14 SeaMonkey/2.0.14";
const char S78988[] = "Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHV-E140L Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S78989[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-gb; ELONEX1012ET Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78990[] = "iBrowser/2.5 (NokiaNXX;SymbianOS/9.3 Series60/3.2)";
const char S78991[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GN100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78992[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GN100 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S78993[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; GN100 Build/GRK39F)";
const char S78994[] = "Mozilla/5.0 (Linux; Android 4.0.3; X9017 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78995[] = "Mozilla/5.0 (Linux; Android 4.0.3; X907 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S78996[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; O+ 8.3 Android Build/OplusOplus_A100p) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78997[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; O+_8.3_Android Build/OplusOplus_A100p) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S78998[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; chagall Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S78999[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717M Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79000[] = "Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/2.1.0.131)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79001[] = "Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/10.0.0.336)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79002[] = "Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/10.1.0.103)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79003[] = "Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-gb; PlayBook Build/10.1.0.123)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79004[] = "Mozilla/5.0 (Linux; Android 4.2.2; Optimus G Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S79005[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S79006[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC One V Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S79007[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S79008[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0151 Build/HuaweiY300-0151) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S79009[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 992D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79010[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79011[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035D Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79012[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035E Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79013[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Orbit 5400 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79014[] = "Mozilla/5.0 (Linux; Android 4.0.3; MD_LIFETAB_P9516 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S79015[] = "Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79016[] = "Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S79017[] = "SAMSUNG-SGH-T339/T339UVHD8 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S79018[] = "Mozilla/5.0 (Linux; U; PocketEdge DINGO-1.13; Android 1.6; en-us; edgejr Build/DRD20) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S79019[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 2000)";
const char S79020[] = "Mozilla/5.0 (compatible; MSIE 6.0; Windows 2000)";
const char S79021[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MOT-ME811/SHDWR_X6_2.200.25; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79022[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MOT-ME811/SHDWR_X6_2.200.12; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79023[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MOT-ME811/SHDWR_X6_2.200.33; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79024[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MOT-ME811/SHDWR_X6_2.200.36; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79025[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; iBall i6012 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79026[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; iBall i6012 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79027[] = "Opera/9.80 (Windows NT 6.2; U; en) Presto/2.2.15 Version/10.00";
const char S79028[] = "Opera/9.80 (Windows NT 6.2; U; en) Presto/2.2.15 Version/10.01";
const char S79029[] = "Opera/10.00 (Windows NT 6.2; U; en) Presto/2.2.0";
const char S79030[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; MG-391 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79031[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; W80 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79032[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A326 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79033[] = "NokiaX3-00/5.0 (08.54) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax3-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile";
const char S79034[] = "NOKIA-Lumia 710/UCWEB2.7.0.223/47/355";
const char S79035[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T989 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S79036[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T989 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S79037[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17 Sleipnir/3.9.1";
const char S79038[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-x330t) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S79039[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-x330) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S79040[] = "LG-GW300/V100 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gw300) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S79041[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo P700i Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79042[] = "Mozilla/5.0 (Linux; Android 4.0.3; Lenovo P700 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79043[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MPDC100 BT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79044[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A15 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S79045[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Fusion Bolt Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79046[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; Fusion Bolt Build/JRO03C)";
const char S79047[] = "Mozilla/5.0 (Linux; Android 4.0.3; AN10DG3 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S79048[] = "Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook Tablet Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S79049[] = "Mozilla/5.0 (Linux; Android 4.2.2; NookTablet Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S79050[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; sv-se; C6833 Build/14.1.B.1.278) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79051[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-; C6833 Build/14.1.B.1.278) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79052[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; sv-se; C6833 Build/14.1.B.1.320) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79053[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; sv-se; C6833 Build/14.1.B.1.298) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79054[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; C6833 Build/14.1.B.1.510)";
const char S79055[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SonyC6833 Build/14.1.B.1.510) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79056[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; C6833 Build/14.1.B.1.510) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79057[] = "Micromax X351/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S79058[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mauie800) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S79059[] = "Mozilla/5.0 (Series40; NokiaC2-00/03.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S79060[] = "Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79061[] = "Mozilla/5.0 (Linux; Android 4.1.2; SH-02E Build/S4260) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79062[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Sensation Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79063[] = "Mozilla/5.0 (Linux; Android 4.1.2; Sensation Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79064[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.1 Safari/533.3";
const char S79065[] = "Mozilla/5.0 (Linux; Android 4.2.1; IdeaTab S6000-F Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S79066[] = "Mozilla/5.0 (Linux; Android 4.2.2; IdeaTab S6000-H Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S79067[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S79068[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; C1504 Build/11.3.A.0.47) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S79069[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; C1504 Build/11.3.A.0.59) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S79070[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; T1-E Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79071[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79072[] = "Nokia5140i/2.0 (03.80) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S79073[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB501 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S79074[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB501 Build/GroovyG's 2.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S79075[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB501 Build/ZPSKT_U3_21.24.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S79076[] = "HTC_Touch2_T3320 Opera/9.50 (Windows NT 5.1; U; en)";
const char S79077[] = "HTC_Touch2_T3330 Opera/9.50 (Windows NT 5.1; U; en)";
const char S79078[] = "SCH-R211 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S79079[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/69/352";
const char S79080[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/70/352";
const char S79081[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/69/444";
const char S79082[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/70/444";
const char S79083[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/69/352  3gpp-gba";
const char S79084[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/70/352  3gpp-gba";
const char S79085[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/69/444  3gpp-gba";
const char S79086[] = "Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia305) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S79087[] = "Mozilla/5.0 (Windows NT 5.1; rv:16.0) Gecko/X Firefox/16.0 slurp";
const char S79088[] = "Mozilla/5.0 (Windows NT 5.1; rv:16.0) Gecko/X Firefox/16.0 slurp slurp";
const char S79089[] = "Mozilla/5.0 (Linux; Android 4.2; Full Android on Grouper Build/JOP40C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S79090[] = "Mozilla/5.0 (Linux; Android 4.2; Build/JOP40C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79091[] = "Mozilla/5.0 (Linux; Android 4.2.2; Build/JDQ39E) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79092[] = "JUC (Linux; U; 2.3.6; zh-cn; LG-E400; 240*320) UCWEB7.9.0.94/139/444";
const char S79093[] = "OneBrowser/3.1 (Nokia311/03.81)";
const char S79094[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E15i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S79095[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Mobile Safari/535.19 Silk-Accelerated=true";
const char S79096[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Mini Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79097[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia X10 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79098[] = "GoogleEarth/7.0.3.8542(Macintosh;Mac OS X (10.7.5);en;kml:2.2;client:Free;type:default)";
const char S79099[] = "GoogleEarth/7.0.2.8415(Macintosh;Mac OS X (10.7.5);en;kml:2.2;client:Free;type:default)";
const char S79100[] = "GoogleEarth/7.0.3.8542(Macintosh;Mac OS X (10.7.5);de;kml:2.2;client:Free;type:default)";
const char S79101[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P1000 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79102[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-P1000 Build/JOP40D)";
const char S79103[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P1000 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79104[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P1000 Build/JDQ39E)";
const char S79105[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-sa; LG-P778 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79106[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu/11.10 Chromium/25.0.1364.172 Chrome/25.0.1364.172 Safari/537.22";
const char S79107[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu/11.10 Chromium/25.0.1364.172 Chrome/25.0.1364.172 Safari/537.22";
const char S79108[] = "Mozilla/5.0 (Linux; Android 4.1.1; PRAXIS 13 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S79109[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S79110[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus S Build/JRO03E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S79111[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; Spice_Mi-422 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79112[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S79113[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.3; ARM; Trident/7.0; Touch; .NET4.0E; .NET4.0C; Tablet PC 2.0)";
const char S79114[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5280 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79115[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5280 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S79116[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E455 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79117[] = "Mozilla/5.0 (Linux; Android 4.1.1; NS-13T001 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S79118[] = "Mozilla/5.0 (Linux; Android 4.2.2; SM-C101 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79119[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Micromax A116 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S79120[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One XL Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79121[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One XL Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79122[] = "Mozilla/4.0 (compatible; MSIE 5.0; Series80/2.0 Nokia9500/4.51 Profile/MIDP-2.0 Configuration/CLDC-1.1)";
const char S79123[] = "SAMSUNG-SGH-T639/UVGG1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S79124[] = "LG-LX260 POLARIS-LX260/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S79125[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/16.0.912.63)";
const char S79126[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.2; chromeframe/16.0.912.63; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; InfoPath.2)";
const char S79127[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/16.0.912.75) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7";
const char S79128[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/16.0.912.75; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S79129[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/16.0.912.77) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S79130[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.2; chromeframe/16.0.912.77; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S79131[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; BOIE9;ENUS; chromeframe/16.0.912.75)";
const char S79132[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/16.0.912.63) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S79133[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/16.0.912.63; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)";
const char S79134[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; chromeframe/16.0.912.77; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; MDDC; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C; BRI/1; X; Version/11.00302)";
const char S79135[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7";
const char S79136[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7";
const char S79137[] = "IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB7.9.0.94/41/997";
const char S79138[] = "IUC(U;iOS 4.2.1;Zh-cn;320*480;)/UCWEB7.9.0.94/41/997";
const char S79139[] = "IUC(U;iOS 5.0.1;Zh-cn;320*480;)/UCWEB7.9.0.94/41/997";
const char S79140[] = "MAUI_WAP_Browser/UCWEB7.9.0.114/109/33068";
const char S79141[] = "MAUI_WAP_Browser/UCWEB7.9.0.114/109/33224";
const char S79142[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC HD2 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79143[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC HD2 Build/JRO03R; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79144[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS11LG Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79145[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Dell Venue Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79146[] = "Mozilla/5.0 (X11; U; Darwin i386; en-US; rv:1.9.0.8) Gecko/X Firefox/3.0.8";
const char S79147[] = "Opera/9.80 (Android 1.6; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S79148[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Mobile Safari/535.19 Silk-Accelerated=false";
const char S79149[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC One X with Beats Audio Build/GRJ90) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79150[] = "Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5310xpressmusic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S79151[] = "Mozilla/5.0 (Linux; Android 4.0.4; A90S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79152[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-LU6800 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79153[] = "Mozilla/5.0 (Linux; Android 4.0.4; SH-01E Build/SA310) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79154[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ALLVIEW P6 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79155[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA6210 Navigator) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S79156[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E973 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S79157[] = "Mozilla/5.0 (Linux; Android 4.1.2; Micromax A116 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S79158[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22 Sleipnir/4.0.1";
const char S79159[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22 Sleipnir/4.0.1";
const char S79160[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S79161[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S79162[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8013 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S79163[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6312) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S79164[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26w Build/6.1.A.2.45) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S79165[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P940 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79166[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-KU5400 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79167[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-SU540 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79168[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax A110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S79169[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-H9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79170[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; A900 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79171[] = "Mozilla/5.0 (Linux; Android 4.0.4; CnM TouchPad 9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79172[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cnm TOUCHPAD 9.7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S79173[] = "Mozilla/5.0 (Linux; Android 4.0.4; CnM TouchPad 7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79174[] = "Mozilla/5.0 (Linux; Android 4.0.4; Versus TouchPad 7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79175[] = "Mozilla/5.0 (Linux; Android 4.2.2; ar-ae; SAMSUNG GT-S7272 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S79176[] = "SAMSUNG-SGH-i600ORANGE/BVGD2 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S79177[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) SAMSUNG-SGH-i600/WM534";
const char S79178[] = "SAMSUNG-SGH-i600/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S79179[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) SAMSUNG-SGH-i600V/1.0";
const char S79180[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; E130 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S79181[] = "Alcatel-OT-880/1.0 model-orange Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S79182[] = "Alcatel-OT-880A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S79183[] = "Alcatel-OT-880/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S79184[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S79185[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Build/IRK40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79186[] = "stagefright/1.1 (Linux;Android 3.1)";
const char S79187[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Build/HTJ70C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S79188[] = "HTC_P3400i-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S79189[] = "HTC_P3400-Mozilla/4.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S79190[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1) Gecko/X Firefox/3.5 CometBird/3.5";
const char S79191[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-b5310) UCBrowser8.2.1.144/69/352/UCWEB Mobile";
const char S79192[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-b5310) UCBrowser8.2.0.132/69/352/UCWEB Mobile";
const char S79193[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; IT-M809RC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S79194[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 8.0; Trident/5.0; IEMobile/9.0; HTC; HD7 T9292)";
const char S79195[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 8.0; Trident/5.0; IEMobile/9.0; WpsLondonTest; HTC; HD7 T9292)";
const char S79196[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; Micromax A78 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79197[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; A78 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79198[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-en) AppleWebKit/533.16 (KHTML, like Gecko) Version/4.1 Safari/533.16";
const char S79199[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; TECNO T1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79200[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; T1 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79201[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.8.1.13) Gecko/X Firefox/2.0.0.13 Flock/1.1.1";
const char S79202[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11 slurp";
const char S79203[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11 slurp";
const char S79204[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11 slurp";
const char S79205[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaC5-06) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/444/UCWEB Mobile";
const char S79206[] = "iBall Andi4.3J+ Linux/3.0.13 Android/4.0.4 Release/10.25.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S79207[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iBall Andi4.3J+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79208[] = "OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; W60 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79209[] = "NOKIA-Lumia 510/UCWEB2.8.0.249/47/355";
const char S79210[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S79211[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320T Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S79212[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S79213[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT886 Build/6.7.3-179_YIC-10) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79214[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P778 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79215[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; Nexus 4 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S79216[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT-7245 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79217[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT11i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S79218[] = "Mozilla/5.0 (Linux; Android 4.0.3; MT11i Build/4.1.A.0.562) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S79219[] = "Mozilla/5.0 (Linux; Android 4.2.2; SC-04E Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79220[] = "Mozilla/5.0 (Linux; Android 4.2.2; SC-04E Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S79221[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S79222[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-I9100G Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S79223[] = "Mozilla/5.0 (Linux; Android 4.1.1; AURUS III Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79224[] = "Micromax X088/R2AE  SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S79225[] = "Micromax X455/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S79226[] = "Micromax X455i/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S79227[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SM-C101 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79228[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; SM-C101 Build/JDQ39)";
const char S79229[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M040 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S79230[] = "Mozilla/5.0 (Linux; Android 4.2.2; M9 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S79231[] = "Mozilla/5.0 (Linux; Android 4.2.2; EVO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79232[] = "Mozilla/5.0 (compatible; Konqueror/4.3; Linux) KHTML/4.3.1 (like Gecko) Fedora/4.3.1-3.fc11";
const char S79233[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; de-de; Odys-Loox Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79234[] = "iCab/4.0 (Macintosh; U; Intel Mac OS X)";
const char S79235[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Mobiistar Touch S02 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79236[] = "YahooMobileMessenger/1.0 (Android Mail; 1.4.4) (mecha; HTC; ADR6400L; 2.3.4/GRJ22)";
const char S79237[] = "SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonj108i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S79238[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; LG-E400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S79239[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; C925 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79240[] = "Rockmelt/2.1.0.1310 CFNetwork/609 Darwin/13.0.0";
const char S79241[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Aqua Trendy Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79242[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; myTab 10 Dual Core Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79243[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7510 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S79244[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P7510 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S79245[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar Touch Kem 402 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79246[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-; Mobiistar Touch Kem 402 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79247[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; TECNO_B3 Build/TECNO_B3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79248[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79249[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; HUAWEI G510-0010 Build/G510-0010V100R001C17B168;480*854)";
const char S79250[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; HUAWEI G510-0010 Build/G510-0010V100R001C17B173;480*854)";
const char S79251[] = "Mozilla/5.0 (Linux; Android 4.1.1; O+ 8.52 Android Build/Oplus8.52_Android) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79252[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-M830 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S79253[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8013 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S79254[] = "Mozilla/5.0 (Linux; Android 4.2.2; SC-04E Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S79255[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-M919 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S79256[] = "Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5784)) Presto/";
const char S79257[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (HTC One, 47, 1.9.7)";
const char S79258[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTCONE Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (HTCONE, 47, 1.9.7)";
const char S79259[] = "Mozilla/5.0 (Linux; Android 4.1.1; A701 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S79260[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTabS2110AF Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S79261[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTabS2110AH Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S79262[] = "Mozilla/5.0 (Linux; Android 4.1.1; NS-13T001 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S79263[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT25i Build/4.1.B.0.631) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S79264[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717R Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79265[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-B5330 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S79266[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-B5330L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79267[] = "Mozilla/5.0 (Linux; Android 4.2.1; R815 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79268[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Stream Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S79269[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.140 Safari/535.19 Comodo_Dragon/18.0.3.0";
const char S79270[] = "Mozilla/5.0 (X11; FreeBSD amd64; rv:12.0) Gecko/X Firefox/12.0";
const char S79271[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7";
const char S79272[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7";
const char S79273[] = "Mozilla/5.0 (S60V5; U; en-in; NokiaC6-01)/UC Browser8.2.0.132/50/444/UCWEB Mobile";
const char S79274[] = "Mozilla/5.0 (Java; U; en-us; nokiac6-01) UCBrowser8.2.1.144/69/444/UCWEB Mobile  3gpp-gba";
const char S79275[] = "Mozilla/5.0 (Windows NT 5.0; rv:2.0.1) Gecko/X Firefox/4.0.1,gzip(gfe) (via translate.google.com)";
const char S79276[] = "Mozilla/5.0 Opera/9.80 (KDDI-KC46; BREW; Opera Mobi; U; ja) Presto/2.4.18 Version/10.00";
const char S79277[] = "Mozilla/5.0 Opera/9.80 (KDDI-CA3K; BREW; Opera Mobi; U; ja) Presto/2.4.18 Version/10.00";
const char S79278[] = "Mozilla/5.0 Opera/9.80 (KDDI-TS3Y; BREW; Opera Mobi; U; ja) Presto/2.4.18 Version/10.00";
const char S79279[] = "Mozilla/5.0 Opera/9.80 (KDDI-SN3V; BREW; Opera Mobi; U; ja) Presto/2.4.18 Version/10.00";
const char S79280[] = "Mozilla/5.0 Opera/9.80 (KDDI-KC4A; BREW; Opera Mobi; U; ja) Presto/2.4.18 Version/10.00";
const char S79281[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9.2.32) Gecko/X Firefox/3.6.32 (Palemoon/3.6.32)";
const char S79282[] = "JUC (Linux; U; 2.3.6; zh-cn; ALCATEL_ONE_TOUCH_918N; 320*480) UCWEB7.9.0.94/139/444";
const char S79283[] = "JUC (Linux; U; 2.3.6; zh-cn; ALCATEL_ONE_TOUCH_918N; 320*480) UCWEB7.9.0.94/139/355";
const char S79284[] = "JUC (Linux; U; 2.3.6; zh-cn; ALCATEL_ONE_TOUCH_918N; 320*480) UCWEB7.9.0.94/139/352";
const char S79285[] = "Mozilla/5.0 (Java; U; en-us; nokian73-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S79286[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; W200 Wil Fone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79287[] = "Mozilla/5.0 (Linux; Android 4.0.4; H2000+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79288[] = "Mozilla/5.0 (Linux; Android 4.2.1; MZ601 Build/JB_MR1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S79289[] = "Mozilla/5.0 (Linux; Android 4.2.2; MZ601 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S79290[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAE65) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S79291[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5580C Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S79292[] = "Mozilla/5.0 (Linux; Android 4.0.3; Marquis_MP977 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S79293[] = "Mozilla/5.0 (Linux; Android 4.1.2; EK-GC100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S79294[] = "Mozilla/5.0 (Linux; Android 4.1.1; EK-GC100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S79295[] = "Mozilla/5.0 (Linux; Android 4.0.4; C1604 Build/11.1.A.0.68) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79296[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5306 Build/12.0.A.1.284) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79297[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7100) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S79298[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M440S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79299[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-H9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79300[] = "Nokia8800e-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S79301[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; E110 Build/Cupcake) AppleWebKit/528.5+ (KHTML, like  Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S79302[] = "MOT-EX128 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S79303[] = "NokiaN70-1/5.0737.3.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/27/351";
const char S79304[] = "NokiaN70/ 5.0741.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/27/351";
const char S79305[] = "Mozilla/5.0 (Windows; U; Windows NT 4.0; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10";
const char S79306[] = "NokiaN97/UC Browser8.0.4.121/50/444/UCWEB";
const char S79307[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P930 Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79308[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P930 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79309[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P930 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79310[] = "Mozilla/5.0 (Series40; NokiaX2-01/08.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15";
const char S79311[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Huawei-U9000 Build/HuaweiU9000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79312[] = "Mozilla/5.0 (X11; U; FreeBSD i386; en-US; rv:1.7) Gecko/X Firefox/0.9.1";
const char S79313[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LG-P930 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79314[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P930 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79315[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; MID1126 Build/GreenDemon_v1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79316[] = "Instagram 3.4.2 (iPhone4,1; iPhone OS 5.0.1; en_US; en) AppleWebKit/420+";
const char S79317[] = "MQQBrowser/40 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3";
const char S79318[] = "MQQBrowser/40 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S79319[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-N7000 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S79320[] = "Mozilla/5.0 (Linux; Android 4.0.4; CT1010 Build/IMM76I.v007) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S79321[] = "Mozilla/5.0 (Linux; Android 4.0.4; GSmart Rio R1 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79322[] = "Mozilla/5.0 (Linux; Android 4.0.4; Rio R1 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79323[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3030; U; en) Presto/2.8.119 Version/11.10";
const char S79324[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S79325[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI-ONE Plus Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79326[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI-ONE Plus Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S79327[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GOCLEVER TAB I71 Build/GRI40) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.0.181/145/352";
const char S79328[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Phoenix Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79329[] = "Mozilla/5.0 (Linux; Android 4.0.3; SHW-M250S Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79330[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79331[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250K Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79332[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250L Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79333[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300";
const char S79334[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SC-1009JB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79335[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA5320) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S79336[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5320d-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S79337[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S79338[] = "UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-914_im_india_269) U2/1.0.0 UCBrowser/3.0.0.285 U2/1.0.0 Mobile";
const char S79339[] = "UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-885_im_india_249) U2/1.0.0 UCBrowser/3.0.0.285 U2/1.0.0 Mobile";
const char S79340[] = "Mozilla/5.0 (Linux; Android 2.3.4; GT-I9100P Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S79341[] = "Mozilla/5.0 (Linux; Android 2.3.5; GT-I9100P Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S79342[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-I9100P Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S79343[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; TM1004 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79344[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SPH-L710 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S79345[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S79346[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9305 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S79347[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E15i) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S79348[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F240L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79349[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; th-th; i-mobile IQ X Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79350[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79351[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC Incredible S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S79352[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9152 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S79353[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; tr-tr; LG-E986 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79354[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax A110 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S79355[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8552 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S79356[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; ME722 Build/MLS2GC_0.24.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79357[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; ME722 Build/MLS2GC_1.10.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79358[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME722 Build/MLS2GC_2.10.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79359[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME722 Build/MLS2GC_2.5.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79360[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME722 Build/MLS2GC_2.6.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79361[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; ME722 Build/MLS2GC_0.24.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S79362[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME722 Build/MLS2GC_2.6.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S79363[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME722 Build/MLS2GC_2.6.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S79364[] = "Dell_Mini 3iX/1.0 (Linux; Android 1.5) Browser/WAP 2.0 AppleWebKit/528.5+ Version/3.1.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S79365[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; md-us; p7901a Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S79366[] = "nokian8-00/UC Browser8.0.3.107/69/444  3gpp-gba";
const char S79367[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.1.3) Gecko/X Firefox/3.5.3 CometBird/3.5.3";
const char S79368[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I717 Build/JRO03H; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79369[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I717 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79370[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I717 Build/JRO03R; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79371[] = "Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia6303classic) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S79372[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT270 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S79373[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; M3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79374[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MOVI M3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79375[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB469 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S79376[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; WT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S79377[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; EasyPad 971 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79378[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Transformer Pad Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79379[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; olympus Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79380[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; sunfire Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79381[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo S880i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79382[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo S880i Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S79383[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo S880i Build/IMM76D)";
const char S79384[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Mobiistar Touch S06 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79385[] = "Mozilla/5.0 (Linux; Android 4.0.4; S100 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S79386[] = "Mozilla/5.0 (Linux; Android 4.0.4; WIKO-CINK SLIM Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79387[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A89) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S79388[] = "sprd-GT_I9300/1.0 Linux/2.6.35.7 Android/4.1.1 Release/09.28.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S79389[] = "MTK-GT_I9300/1.0 Linux/2.6.35.7 Android/4.1.9 Release/11.01.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S79390[] = "MTK-GT_I9300/1.0 Linux/2.6.35.7 Android/4.1.1 Release/10.24.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S79391[] = "MTK-GT_I9300/1.0 Linux/2.6.35.7 Android/4.1.1 Release/10.13.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S79392[] = "MTK-GT_I9300/1.0 Linux/2.6.35.7 Android/4.1.1 Release/10.09.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S79393[] = "MTK-GT_I9300/1.0 Linux/2.6.35.7 Android/4.1.1 Release/10.22.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S79394[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia603) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S79395[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S79396[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710VMUB Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S79397[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710BST Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S79398[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S79399[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F240K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79400[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N8013 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S79401[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S79402[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5303 Build/12.0.A.1.257) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S79403[] = "Mozilla/5.0 (Linux; Android 4.2.1; Q700 Build/XOLOQ700) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79404[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S79405[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8262 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S79406[] = "Nokia7390/2.0 (04.51) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S79407[] = "SAMSUNG-SGH-L170/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S79408[] = "HTC_Snap_S523/V1.22.841.5 (53520) Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S79409[] = "HTC_Snap_S523 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S79410[] = "HTC_Snap_S523/V2.03.841.1 (65854) Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S79411[] = "MOT-W230/0.0.31 UP.Browser/6.3.0.6.c.19 (GUI) MMP/2.0";
const char S79412[] = "modu T/1.0 Browser/Obigo/Q7 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S79413[] = "Mozilla/4.1 (compatible; Teleca/Q07/GS-CX700)";
const char S79414[] = "Alex-uo/1.0 Qualcom/6290 Release/3.20.2010 Browser/Obigo Q7 Profile/MIDP-2.0 Configuration/CLDC1.1";
const char S79415[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1";
const char S79416[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1";
const char S79417[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.4) Gecko/X Firefox/3.6.4 Wyzo/3.6.4.1 SearchToolbar/1.2";
const char S79418[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; KM-S120 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79419[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROIDX Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79420[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOR803 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79421[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; R803 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S79422[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; R803 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S79423[] = "Mozilla/5.0 (Linux; Android 4.1.1; PG86100 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79424[] = "Mozilla/5.0 (Linux; Android 4.1.2; PG86100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79425[] = "Mozilla/5.0 (Linux; Android 4.1.2; Evo 3D CDMA Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79426[] = "Mozilla/5.0 (Linux; Android 4.1.2; Evo 3D GSM Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79427[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ZTE U930 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S79428[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; ZTE U930 Build/IML74K)";
const char S79429[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U930A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79430[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ZTE U930HD Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S79431[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5233s) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S79432[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5233s) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S79433[] = "UCWEB/2.0(Java; U; MIDP-2.0; vi; gt-s5233s) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S79434[] = "UCWEB/2.0(Java; U; MIDP-2.0; vi; gt-s5233s) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S79435[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5233s) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0";
const char S79436[] = "Opera/9.80 (BREW; Opera Mini/4.2.99/28.3392; U; id) Presto/2.8.119 Version/11.10";
const char S79437[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; iDxD8 3G Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79438[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; iDxD8 3G Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79439[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/98 Safari/537.4 (StatusCake)";
const char S79440[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA6700s) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S79441[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5670) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S79442[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-be; Lenco TAB-813 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79443[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GoTab Gti8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79444[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; P-02D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79445[] = "Mozilla/5.0 (Linux; Android 4.1.1; BS471 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79446[] = "Mozilla/5.0 (Linux; U; Android 4.1.1;engb;Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 4.1.1; de-de; ST80208-2 Build/JRO03HSafari/534.30;SurfTab_8.0";
const char S79447[] = "Mozilla/5.0 (Linux; U; Android 4.1.1;engb;Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 4.1.1; de-de; ST80216-2 Build/JRO03HSafari/534.30;SurfTab_8.0";
const char S79448[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035A Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79449[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035X Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79450[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79451[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID07-D2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79452[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MID0714 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79453[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MID0738 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79454[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530M Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79455[] = "Mozilla/5.0 (Linux; Android 4.0.3; SmartTabII10 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S79456[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST27i Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S79457[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P765 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79458[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5200 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S79459[] = "Mozilla/5.0 (Linux; Android 4.0.4; CINK PEAX Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79460[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79461[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xoom Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S79462[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaN8-00) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S79463[] = "Mozilla/5.0 (Linux; Android 4.1.2; JY-G3 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79464[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One X+ Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S79465[] = "SAMSUNG-GT-B3410R/B3410UXJC4";
const char S79466[] = "SAMSUNG-GT-B3410/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S79467[] = "SAMSUNG-SGH-I907/UCHI5 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S79468[] = "SonyEricssonP910a/R3A SEMC-Browser/Symbian/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S79469[] = "SonyEricssonP910c/R2A SEMC-Browser/Symbian/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S79470[] = "SonyEricssonP910i/R5B SEMC-Browser/Symbian/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S79471[] = "SAMSUNG-SGH-I637/UCIJ2; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.14812; U; en-US) Opera 9.50";
const char S79472[] = "NokiaC2-01/5.0 (11.20) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-01/UC Browser7.9.0.102/69/352";
const char S79473[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Spice_Mi300 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S79474[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; CSL_Spice_MI300 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S79475[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/530.9 (KHTML, like Gecko) Iron/2.0.178.0 Safari/530.9";
const char S79476[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/530.1 (KHTML, like Gecko) Iron/2.0.168.0 Safari/530.1";
const char S79477[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.1.0.0 Chrome/17.0.963.38 Safari/535.11";
const char S79478[] = "Opera/9.80 (X11; Linux i686; U; Edition Linux Mint; en) Presto/2.10.229 Version/11.62";
const char S79479[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:0.9.4.1) Gecko/X Netscape6/6.2.2";
const char S79480[] = "Mozilla/5.0 (Windows NT 4.10) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5";
const char S79481[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S79482[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S79483[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79484[] = "JUC (Linux; U; 2.3.6; zh-cn; GT-S7500; 320*480) UCWEB7.9.0.94/139/444";
const char S79485[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC X720d Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S79486[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-X720d/1.34.1401.1) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79487[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-X720d/2.19.1401.1) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79488[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-X720d/1.27.1401.1) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79489[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-X720d/1.28.1401.1) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79490[] = "Nokia7230/5.0 (10.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia7230) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S79491[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T528t Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S79492[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T528t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79493[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC T528t Build/IMM76D)";
const char S79494[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I757M Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79495[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAC-80011 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79496[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; TAC-90011 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79497[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-SU880 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79498[] = "Mozilla/5.0 (Linux; Android 4.0.4; MEDION P4013 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79499[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A57) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S79500[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Oysters T34 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79501[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79502[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; AMOI N821 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79503[] = "AMOI N821 Linux/3.0.13 Android/4.0.4 Release/12.19.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S79504[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; CAPHG11-01 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79505[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; CAPHG11-01 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79506[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7105 Build/JDQ39; Carbon-CARBON-JB-v1.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79507[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L900 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79508[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T889 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79509[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7105 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79510[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.6 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/537.6 Midori/0.5";
const char S79511[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; fnac 10 v2 Build/1.0.0_X-15:17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79512[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; fnac 10 v2 Build/1.0.1_X-14:35) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79513[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7510 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S79514[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ608 Build/7.7.1-141-7-FLEM-UMTS-LA) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S79515[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ608-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S79516[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717R Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79517[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/10A523 Safari/7534.48.3";
const char S79518[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/10A550 Safari/7534.48.3";
const char S79519[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/10A403 Safari/7534.48.3";
const char S79520[] = "Mozilla/5.0 (Linux; Android 4.1.2; SOL21 Build/9.1.D.0.401) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79521[] = "Huawei/1.0/0HuaweiG6620/SP01 Browser/Obigo-Browser/Q03C MMS/Obigo-MMS/1.2, Huawei/1.0/0HuaweiG6620/SP01 Browser/Obigo-Browser/Q03C MMS/Obigo-MMS/1.2";
const char S79522[] = "Huawei/1.0/0HuaweiG6620/SP01 Browser/Obigo-Browser/Q03C MMS/Obigo-MMS/1.2";
const char S79523[] = "Huawei-G6620/1.0/0/SP01 Browser/Obigo-Browser/Q03C MMS/Obigo-MMS/1.2";
const char S79524[] = "NokiaE63/UC Browser7.7.1.88/28/444";
const char S79525[] = "nokiae63-1/UC Browser7.7.1.88/69/444";
const char S79526[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7";
const char S79527[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2";
const char S79528[] = "Mozilla/5.0 (Windows NT 5.0; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2";
const char S79529[] = "iTunes/9.0.2 (Windows; N)";
const char S79530[] = "NokiaX2-02/2.0 (10.91) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiax2-02) UCBrowser8.2.1.144/70/355/UCWEB Mobile";
const char S79531[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN79)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S79532[] = "NOKIAN79/UCWEB8.2.0.116/28/800";
const char S79533[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352";
const char S79534[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S79535[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5830L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S79536[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352";
const char S79537[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S79538[] = "SAMSUNG-GT-E2222/E2222JVKJ3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S79539[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Explay Infinity Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79540[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SC-72MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79541[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SC-72MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79542[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SC-72MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79543[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; ADT 2.0 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79544[] = "Mozilla/5.0 (S60V3; U; en-us; Nokia6220c-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S79545[] = "Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.6 Safari/535.19 Silk-Accelerated=false";
const char S79546[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E16i Build/2.1.1.A.0.53) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S79547[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E16i Build/2.1.1.A.0.53) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S79548[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; ADT 2.6 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79549[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P3100 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S79550[] = "Mozilla/5.0 (compatible; MSIE 6.0; Win3.11; Trident/1.0)";
const char S79551[] = "Mozilla/5.0 (compatible; MSIE 2.0; Windows NT 6.2; Trident/4.0)";
const char S79552[] = "Mozilla/5.0 (compatible; MSIE 1.0; Windows NT 6.2; Trident/4.0)";
const char S79553[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; O+ 8.35 Android Build/OplusOplus_A200) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79554[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; O+_8.35_Android Build/OplusOplus_A200) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79555[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Sensation Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S79556[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Sensation Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S79557[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R760X Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79558[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-MS770 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (LG-MS770, 42, 1.9.1)";
const char S79559[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P715 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79560[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Fly IQ451 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79561[] = "Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2730c-1) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S79562[] = "Mozilla/5.0 (Linux; Android 4.1.2; A511 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S79563[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-I9003 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S79564[] = "Mozilla/5.0 (Linux; Android 2.3.5; GT-I9003 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S79565[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT916 Build/2_26D_2014) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S79566[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S79567[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S79568[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S79569[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Orbit 5700T Build/ICS_IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79570[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaE7-00) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S79571[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A21 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S79572[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9000 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/galaxysmtd";
const char S79573[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9000 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/galaxysmtd";
const char S79574[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P940 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79575[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-KU5400 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79576[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-SU540 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79577[] = "NexianNX-G868/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S79578[] = "NexianNX-G868T/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP-2.0Configuration/CLDC-1.0";
const char S79579[] = "Mozilla/5.0(Linux;U;Android2.1-update1;Zh_cn;U20i;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S79580[] = "JUC(Linux;U;Android2.1-update1;Zh_cn;U20i;240*320;)UCWEB7.8.0.95/139/444";
const char S79581[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/528.7 (KHTML, like Gecko) Iron/1.0.155.0 Safari/528.7";
const char S79582[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT701 Build/STCU_41.07.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S79583[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT701 Build/STCU_31.05.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S79584[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT701 Build/STCU_31.05.4) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S79585[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT701 Build/STCU_41.07.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S79586[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; AN7 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79587[] = "Opera/9.80 (X11; Linux i686; U; de) Presto/2.1.1";
const char S79588[] = "Mozilla/5.0 (Series40; NokiaC2-02/06.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S79589[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; Novo7Basic Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S79590[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.8.0.1) Gecko/X Flock/0.5.13.2";
const char S79591[] = "JUC(Linux;U;Android2.3.5;Zh_cn;HTC Explorer A310e;320*480;)UCWEB7.8.0.95/139/444";
const char S79592[] = "Mozilla/5.0 (S60V3; U; en-in; NOKIA6120c)/UC Browser8.4.0.159/28/352/UCWEB Mobile";
const char S79593[] = "Mozilla/5.0 (Java; U; en-us; nokia6120c) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S79594[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.1 Mobile Safari/535.19 Silk-Accelerated=true";
const char S79595[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cynus T1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79596[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.0.16) Gecko/X Firefox/3.0.16 Flock/2.5.6";
const char S79597[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MFC155FR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79598[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; CTP828BT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79599[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HUAWEI Y310-5000 Build/HuaweiY310-5000) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S79600[] = "Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; HUAWEI Y310-5000 Build HuaweiY310-5000) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S79601[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HUAWEI Y310-5000 Build/HuaweiY310-5000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79602[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825-1 Build/HuaweiU8825-1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79603[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79604[] = "Opera/9.80 (Windows 95) Presto/2.12.388 Version/12.15";
const char S79605[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R813T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79606[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; R813T Build/IMM76D)";
const char S79607[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; Avea inTouch 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79608[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; X909T Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79609[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; X909 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79610[] = "Mozilla/5.0 (Linux; Android 4.2.1; AT10-A Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S79611[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Micromax A76 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79612[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Micromax A74 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79613[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC5-05) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S79614[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5282 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S79615[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.0.200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S79616[] = "HTC Streaming Player sprint / 1.0 / htc_supersonic / 2.3.3";
const char S79617[] = "HTC Streaming Player sprint / 1.0 / htc_supersonic / 2.2";
const char S79618[] = "HTC Streaming Player sprint / 1.0 / htc_supersonic / 2.3.5";
const char S79619[] = "KDDI-CA3F UP.Browser/6.2_7.2.7.1.K.4.310 (GUI) MMP/2.0";
const char S79620[] = "Mozilla/5.0 (compatible; Konqueror/4.1; Linux; X11)";
const char S79621[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Wyzo/3.5.6";
const char S79622[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:2.0) Gecko/X Firefox/4.0 TenFourFox/7450";
const char S79623[] = "Mozilla/5.0 (compatible; rv:2.0) Gecko/X Firefox/4.0";
const char S79624[] = "Mozilla/5.0 (Series40; Nokia5310XpressMusic/10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S79625[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; Edition MAS; en) Presto/2.10.229 Version/11.63";
const char S79626[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; 101DL Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79627[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; 101DL Build/GRJ90)";
const char S79628[] = "Mozilla/5.0 (X11; FreeBSD i386; rv:8.0) Gecko/X Firefox/8.0";
const char S79629[] = "samsung-gt-s3850/UC Browser8.0.3.107/69/444";
const char S79630[] = "Mozilla/5.0 (Linux; Android 4.0.3; Scroll Extreme Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S79631[] = "Mozilla/5.0 (Linux; Android 4.0.3;  Scroll Extreme Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S79632[] = "MacOutlook/14.2.3.X (Intel Mac OS X 10.8.0)";
const char S79633[] = "MacOutlook/14.2.3.X (Intel Mac OS X 10.8.1)";
const char S79634[] = "MacOutlook/14.2.4.X (Intel Mac OS X 10.8.2)";
const char S79635[] = "MacOutlook/14.2.3.X (Intel Mac OS X 10.8.2)";
const char S79636[] = "MacOutlook/14.2.5.X (Intel Mac OS X 10.8.2)";
const char S79637[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Lenovo A668t Build/GWK74) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S79638[] = "LenovoA668t_TD/1.0 Android/2.3 (Linux; Android 2.3) Release/08.20.2011 Browser/WAP 2.0 AppleWebKit/533.1";
const char S79639[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Lenovo A668t Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S79640[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Lenovo A668t Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79641[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; S15 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79642[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; PROG160 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79643[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P6211 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79644[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P6211 Build/IMM76D)";
const char S79645[] = "Mozilla/5.0 (Linux; Android 4.0.4; A8 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79646[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9b2pre) Gecko/X Minefield/3.0b2pre";
const char S79647[] = "SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonj108i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S79648[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Micromax_A50) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S79649[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Desire_HD_A9191) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S79650[] = "UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; Desire_HD) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S79651[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P6200 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.28 Safari/537.31";
const char S79652[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S79653[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I605 Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79654[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I605 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/i605";
const char S79655[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-L40G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79656[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fa-ir; G3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79657[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Jiayu G3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79658[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT28h Build/6.1.E.2.68) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S79659[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT28h Build/6.1.E.3.7) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S79660[] = "Mozilla/5.0 (Linux; Android 2.3.4; ADR6350 Build/GRJ22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453";
const char S79661[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; TAB950 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79662[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033A Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79663[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79664[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810B Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79665[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S79666[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN82) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S79667[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E450f Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79668[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E450g Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79669[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E450 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79670[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E450B Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79671[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; SonyEricssonU5) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S79672[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P778 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79673[] = "Mozilla/5.0 (Linux; Android 4.1.1; KUNO4 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S79674[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S5282) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S79675[] = "Mozilla/5.0 (Linux; Android 4.3; GT-I9100 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.30 Mobile Safari/537.36";
const char S79676[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One VX Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79677[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79678[] = "Mozilla/5.0 (Android; Mobile: SPH-L720; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S79679[] = "Mozilla/5.0 (Android; Tablet: GT-P3113; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S79680[] = "Mozilla/5.0 (Android; Tablet: SCH-I915; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S79681[] = "Mozilla/5.0 (Android; Mobile: SCH-I605; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S79682[] = "Mozilla/5.0 (Android; Mobile: Nexus 4; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S79683[] = "Mozilla/5.0 (Android; Mobile: LG-P769; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S79684[] = "PANTECH-P7000/JDUS10052009 Browser/Obigo/Q05A Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S79685[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Comodo_Dragon/6.0.0.10 Chrome/6.0.472.63 Safari/534.3";
const char S79686[] = "NokiaC2-00/2.0 (03.82) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-00/UC Browser7.9.0.102/69/352";
const char S79687[] = "Symphony M70";
const char S79688[] = "Symphony D48";
const char S79689[] = "Symphony D30";
const char S79690[] = "JUC(Linux;U;Android2.3.5;Zh_cn;HTC Desire HD A9191;480*800;)UCWEB7.8.0.95/139/444";
const char S79691[] = "JUC(Linux;U;Android2.3.3;Zh_cn;Desire HD;480*800;)UCWEB7.8.0.95/139/444";
const char S79692[] = "iCab/5.0 (Macintosh; U; Intel Mac OS X)";
const char S79693[] = "iCab/5.0 (Macintosh; U; PPC Mac OS X)";
const char S79694[] = "Mozilla/5.0 (Series40; Nokia2700c-2/07.80; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.3.23.12";
const char S79695[] = "Opera/9.80 (Windows 95; U; ru) Presto/2.10.289 Version/12.00";
const char S79696[] = "Opera/9.80 (Windows 95; U; ru) Presto/2.10.289 Version/12.01";
const char S79697[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; i-mobile i-style Q1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79698[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; i-mobile i-style Q1i Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79699[] = "SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; sonyericssonj105i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile";
const char S79700[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-x330t) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S79701[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-x330) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S79702[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-x330t) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S79703[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S79704[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355";
const char S79705[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-P1000L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355";
const char S79706[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S79707[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; XT875 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79708[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT875 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79709[] = "Mozilla/5.0 (Linux; Android 4.0.4; CINK KING Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S79710[] = "Mozilla/5.0 (Linux; Android 4.1.1; bq Curie Build/1.1.0 X-14:50) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S79711[] = "Mozilla/5.0 (Linux; Android 4.1.2; S500 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S79712[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000";
const char S79713[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Desire HD Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S79714[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire HD Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000";
const char S79715[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7108 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S79716[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7108 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S79717[] = "Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Safari/535.19 Silk-Accelerated=false";
const char S79718[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A21) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S79719[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Karbonn_A21) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S79720[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; Micromax A116 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S79721[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Micromax_A116 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S79722[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT15i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79723[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_ov5647 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79724[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_9p017 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79725[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_253 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79726[] = "Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_253_6628 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79727[] = "Mozilla/5.0 (Linux; Android 4.2.2; C5503 Build/10.3.A.0.317) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S79728[] = "Mozilla/5.0 (Linux; Android 4.2.2; C5503 Build/10.3.1.A.0.244) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79729[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-D700 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79730[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaE6-00) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S79731[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S79732[] = "Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook Tablet Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S79733[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT926 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/xt926";
const char S79734[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; ALLVIEW A4ALL Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79735[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One X Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S79736[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One X Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S79737[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SAMSUNG-SGH-I337, 47, 1.9.7)";
const char S79738[] = "Mozilla/5.0 (Linux; Android 4.1.2; C525c Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S79739[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5310 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79740[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S79741[] = "LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-kp500 teleca) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S79742[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7102 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S79743[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7102 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S79744[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S79745[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710VMUB Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S79746[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710BST Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S79747[] = "Nokia6263/2.0 (05.20) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S79748[] = "LG-GM730/v10a WM/6.1 IEMobile/6.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S79749[] = "IEMobile/v6.0 LG-GM730/V20b Browser/IE8.12 MMS/LGMMSv1.0/1.2 Java/LGVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1/Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S79750[] = "HTC_P3400i-Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S79751[] = "C5005/001.00 ACS-NetFront/3.2";
const char S79752[] = "C5330/001.00 ACS-NetFront/3.2";
const char S79753[] = "C5320/001.00 ACS-NetFront/3.2";
const char S79754[] = "C5588/001.00 ACS-NetFront/3.2";
const char S79755[] = "C5110/001.00 ACS-Netfront/3.2";
const char S79756[] = "C6100/001.00 ACS-Netfront/3.2";
const char S79757[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Iron/7.0.520.1 Chrome/7.0.520.1 Safari/534.7";
const char S79758[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Iron/7.0.520.1 Chrome/7.0.520.1 Safari/534.7";
const char S79759[] = "Nokia3110c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia3110c/UC Browser8.0.3.107/69/444";
const char S79760[] = "NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-02/UC Browser8.0.3.107/69/444";
const char S79761[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:12.0) Gecko/X Thunderbird/12.0.1";
const char S79762[] = "Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia2730c-1) UCBrowser8.2.0.132/69/452/UCWEB Mobile";
const char S79763[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Rhyme S510b Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79764[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Wildfire S A510e Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79765[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; HS-7DTB5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79766[] = "Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A9+ Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79767[] = "Mozilla/5.0 (Linux; Android 4.0.4; A9+ Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79768[] = "Mozilla/5.0 (Linux; Android 4.0.4; A9+(Flare S100) Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79769[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; ADT 1.0 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79770[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A810S Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79771[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A810K Build/VEGAVIET.COM ROM JELLYBEAN V1.0; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79772[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S79773[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S79774[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAE90) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S79775[] = "Mozilla/5.0 (Linux; Android 4.0.3; Lenovo S880 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S79776[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE BLADE III Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S79777[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE Blade III Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S79778[] = "Mozilla/5.0 (Linux; Android 4.0.4; Blade III Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S79779[] = "Mozilla/5.0 (Linux; Android 4.0.4; A21 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S79780[] = "Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A21 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S79781[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S79782[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S79783[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S79784[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0530, in";
const char S79785[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0545, np";
const char S79786[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0700, th";
const char S79787[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0800, ph";
const char S79788[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/527+ (KHTML, like Gecko) QtWeb Internet Browser/1.5 http://www.QtWeb.net";
const char S79789[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; DM014SH Build/S0014) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79790[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; DM014SH Build/S0010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79791[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; DM014SH Build/S0014)";
const char S79792[] = "Mozilla/5.0 (Linux; Android 4.2.2; Infected Rezound Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79793[] = "Mozilla/5.0 (Linux; Android 4.2.2; Infected HTC Rezound Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79794[] = "Mozilla/5.0 (Linux; Android 4.2.2; Rezound on TRON 2.1 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79795[] = "Mozilla/5.0 (Linux; Android 4.2.2; Rezound on TRON-2.0 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79796[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S79797[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Titanium S5 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S79798[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 521)";
const char S79799[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; Karbonn A6 Build/MocorDroid4.0.3) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S79800[] = "Mozilla/5.0 (Linux; Android 4.0.3; Lenovo S880 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S79801[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S79802[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; SX7-PEARL.GmbH Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79803[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; Galaxy Nexus Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S79804[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Galaxy Nexus Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S79805[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID7065 Build/ICS.MID7065.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Safari/537.36";
const char S79806[] = "Mozilla/5.0 (Linux; Android 3.1; GT-P7510 Build/HMJ37) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S79807[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Mobile Safari/535.19 Silk-Accelerated=true";
const char S79808[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia500) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S79809[] = "Mozilla/5.0 (Linux; Android 4.2.2; Micromax A76 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79810[] = "Mozilla/5.0 (Linux; Android 4.2.2; Micromax A74 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79811[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9082 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S79812[] = "ZTE-G-X990/X990_V2_Z13_EN_D18F101 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C";
const char S79813[] = "ZTE-G-X990-VIVO/VIVO_X990_V1.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C";
const char S79814[] = "ZTE-G-X990/X990_V2_Z14_PT_D18F100 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C";
const char S79815[] = "ZTE-G-X990/X990_V2_Z21_ES_D18F103 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C";
const char S79816[] = "ZTE-G-X990/X990_V2_Z15_PT_D18F108 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C";
const char S79817[] = "ZTE-G-X990-VIVO/X990_V2_Z10_PT_D18F100 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C";
const char S79818[] = "ZTE-G-X990/X990_V2_Z18_EN_D18F100 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C";
const char S79819[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S79820[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.2.1.0 Chrome/16.0.912.75 Safari/535.7";
const char S79821[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.2.1.0 Chrome/16.0.912.75 Safari/535.7";
const char S79822[] = "Akregator/4.8.3; syndication";
const char S79823[] = "Akregator/4.8.4; syndication";
const char S79824[] = "Akregator/4.8.2; syndication";
const char S79825[] = "Akregator/4.8.5; syndication";
const char S79826[] = "Akregator/4.8.5; syndication SUSE";
const char S79827[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SPH-M820 Build/GINGERBREAD; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79828[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-198_6)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79829[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Motorola Electrify Build/4.5.1A_SUN_USC_19.0)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79830[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S79831[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Motorola Electrify Build/4.5.1A_SUN_USC_19.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S79832[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; eGlide Steal Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79833[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; MIDC800PR001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79834[] = "Mozilla/5.0 (Linux; Android 4.0.4; A701 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S79835[] = "Mozilla/5.0 (Linux; Android 4.0.4; A701 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79836[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HS-EG906 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79837[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HS-EG906 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S79838[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HS-EG906 Build/IMM76D)";
const char S79839[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB865 Build/JRO03C; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79840[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MB865 Build/JRO03C; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79841[] = "Tweetbot/26000 CFNetwork/609 Darwin/13.0.0";
const char S79842[] = "Tweetbot/25100 CFNetwork/609 Darwin/13.0.0";
const char S79843[] = "Tweetbot/26200 CFNetwork/609 Darwin/13.0.0";
const char S79844[] = "Tweetbot/27301 CFNetwork/609.1.4 Darwin/13.0.0";
const char S79845[] = "Tweetbot/28000 CFNetwork/609.1.4 Darwin/13.0.0";
const char S79846[] = "Tweetbot/27100 CFNetwork/609.1.4 Darwin/13.0.0";
const char S79847[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Incredible S Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79848[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17 Sleipnir/3.9.1";
const char S79849[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17 Sleipnir/3.9.1";
const char S79850[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MT25i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S79851[] = "Mozilla/5.0 (Linux; Android 4.1.1; TSINTECH Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79852[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_985N) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S79853[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5300) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S79854[] = "Mozilla/5.0 (Linux; Android 4.3; Galaxy Nexus Build/JWR66V) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79855[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Viettel V8502 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79856[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST25i Build/6.1.1.B.1.10) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S79857[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-66) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S79858[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-78) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S79859[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-78-1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S79860[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M1062 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79861[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5670) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S79862[] = "Mozilla/5.0 (Linux; Android 4.2.1; S5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79863[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9300) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S79864[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT907 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79865[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S79866[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S79867[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S79868[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S79869[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79870[] = "Mozilla/5.0 (Linux; Android 4.1.2; AMID-9743G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S79871[] = "Mozilla/5.0 (Linux; Android 4.0.3; W732 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79872[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; BN NookHD+ Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.2/ovation";
const char S79873[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; BN NookHD+ Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.2/ovation";
const char S79874[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; BN NookHD+ Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.2/ovation";
const char S79875[] = "Mozilla/5.0 (Linux; Android 4.0.3; SmartTabII10 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S79876[] = "SAMSUNG-GT-M8800/M880BXXJA2 SHP/VPP/R5 Jasmine/0.8 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S79877[] = "Nokia5140/2.0 (3.10) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S79878[] = "LG-GR700/V10g Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S79879[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SKY IM-A650S Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S79880[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SKY IM-A650S Build/ERE27)";
const char S79881[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; SKY IM-A650S Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S79882[] = "NOKIA6120c/UC Browser7.9.0.102/28/355/UCWEB";
const char S79883[] = "Opera/9.80 (Windows NT 5.0; U; ru) Presto/2.9.168 Version/11.51";
const char S79884[] = "Mozilla/5.0 (Windows NT 6.0; rv:11.0) Gecko/X Firefox/11.0.1 PaleMoon/11.0.1";
const char S79885[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:11.0) Gecko/X Firefox/11.0.1-x64 PaleMoon/11.0.1-x64";
const char S79886[] = "nokiac5-00/UC Browser7.9.0.102/69/352  3gpp-gba";
const char S79887[] = "nokiac5-00.2/UC Browser7.9.0.102/69/352  3gpp-gba";
const char S79888[] = "Mozilla/5.0 (Series40; NokiaC2-01/11.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S79889[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaN97 mini)/UC Browser8.2.0.132/50/355/UCWEB Mobile";
const char S79890[] = "NokiaN97 mini/UCWEB8.2.0.116/50/800";
const char S79891[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; LAVA S12 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79892[] = "Mozilla/5.0 (PlayStation Vita 1.00) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2";
const char S79893[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; vivo Y1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79894[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; vivo Y1 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S79895[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile i-style Q2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79896[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile i-STYLE Q2 DUO Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S79897[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S79898[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S79899[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Ken Brown Synkom Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79900[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Ken Brown Synkom Build/tablet-c) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79901[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonk800i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S79902[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonk800i) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S79903[] = "UCWEB/2.0(Java; U; MIDP-2.0; fa; sonyericssonk800i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S79904[] = "UCWEB/2.0(Java; U; MIDP-2.0; fa; sonyericssonk800i) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S79905[] = "Nokia202/2.0 (20.28) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia202) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S79906[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S79907[] = "Opera/9.80 (Windows 95) Presto/2.12.388 Version/12.14";
const char S79908[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST23i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S79909[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A1+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S79910[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A1+ Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S79911[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC One V Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79912[] = "Mozilla/5.0 (X11; Linux x86_64) KHTML/4.10.1 (like Gecko) Konqueror/4.10 Fedora/4.10.1-1.fc17";
const char S79913[] = "Mozilla/5.0 (Linux; Android 4.1.1; OnePAD 1100x2 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S79914[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S79915[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S79916[] = "Mozilla/5.0 (Linux; Android 4.0.3; M532 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S79917[] = "Mozilla/5.0 (Linux; Android 4.0.4; S100 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S79918[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT918 Build/2_26C_2009) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79919[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT918 Build/2_26D_2009) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79920[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT918 Build/2_26D_2009)";
const char S79921[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT918 Build/2_32D_2009) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79922[] = "Mozilla/5.0 (Linux; Android 4.0.3; PG86100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S79923[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79924[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S79925[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; GT-1240 Build/GT-1240-v1.4-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79926[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; GT-1240 Build/GT-1240-v2.2-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79927[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; GT-1240 Build/GT-1240-v2.1-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79928[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; GT-1240 Build/GT-1240-v2.4-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79929[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5300) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S79930[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; ZP950+ Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79931[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ZP950H Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79932[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S79933[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-P500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S79934[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I437 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79935[] = "Mozilla/5.0 (Linux; Android 4.1.1; TAB-840_G Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S79936[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5302 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S79937[] = "Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook Tablet Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S79938[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S79939[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S79940[] = "SAMSUNG-GT-C3510T/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S79941[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; TAB-PLAYTABPRO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79942[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; TAB-PLAYTABPRO(4R) Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79943[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5250)/UC Browser8.5.0.183/50/352/UCWEB Mobile";
const char S79944[] = "Nokia5250/UCWEB8.5.0.163/50/800";
const char S79945[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MP1047 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79946[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Mobile Safari/535.19 Silk-Accelerated=true";
const char S79947[] = "Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia6303iclassic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S79948[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; D976D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79949[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S79950[] = "Instagram 3.4.2 (iPod4,1; iPhone OS 4.3.5; en_US; en) AppleWebKit/420+";
const char S79951[] = "Instagram 3.4.2 (iPod4,1; iPhone OS 4.3.3; en_US; en) AppleWebKit/420+";
const char S79952[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A9+ Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S79953[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Karbonn A9+ Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S79954[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; CELKON_A9+ Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S79955[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-P930 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79956[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-P930 Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79957[] = "Mozilla/5.0 (Linux; Android 4.0.3; AN10DG3 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S79958[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:21.0) Gecko/X Firefox/21.0 SeaMonkey/2.18";
const char S79959[] = "Mozilla/5.0 (Linux; Android 4.1.1; A211 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S79960[] = "Mozilla/5.0 (Linux; Android 4.0.4; LIFETAB_S9512 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S79961[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; Primo 91 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S79962[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-D710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S79963[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-D710BST Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S79964[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-D710VMUB Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S79965[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PAP4044DUO Build/PrestigioPAP4044DUO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79966[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HUAWEI C8813 Build/HuaweiC8813) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S79967[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI C8813Q Build/HuaweiC8813Q) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S79968[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HUAWEI C8813D Build/HuaweiC8813D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S79969[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11";
const char S79970[] = "Mozilla/5.0 (Linux; Android 4.1.1; M040 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79971[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAP4500TDUO Build/PrestigioPAP4500TDUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79972[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAP4500DUO Build/PrestigioPAP4500DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79973[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S79974[] = "Mozilla/5.0 (Linux; Android 4.0.4; S100 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79975[] = "Mozilla/5.0 (Linux; Android 4.1.1; Z120 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S79976[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC Desire X Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S79977[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC Desire X dual sim Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S79978[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530X Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S79979[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530X Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S79980[] = "Mozilla/5.0 (Linux; Android 4.0.4; STARADDICT II Plus Build/GXI_ZTEV1.0.0B03) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79981[] = "Mozilla/5.0 (Linux; Android 4.0.4; Smartphone Android by SFR STARADDICT II Build/IceCreamSandwich) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79982[] = "Mozilla/5.0 (Linux; Android 4.1.1; CINK SLIM Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S79983[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus 4 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1650.11 Mobile Safari/537.36";
const char S79984[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S79985[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax A110 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S79986[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6802 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S79987[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8010 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S79988[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.3.0.0 Chrome/17.0.963.46 Safari/535.11";
const char S79989[] = "BlackBerry9630/4.2.0.424 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/159/444";
const char S79990[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Glacier Build/Zips_Glacier) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79991[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Glacier Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79992[] = "Mozilla/5.0 (Linux; U; Android 4.0.5; en-us; HTC Glacier Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79993[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Glacier Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79994[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Glacier Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S79995[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Celkon_A97 Build/Celkon_A97Celkon_A97) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S79996[] = "Mozilla/5.0 (Windows NT 6.0; rv:15.0) Gecko/X Firefox/15.0 PaleMoon/15.0";
const char S79997[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:15.0) Gecko/X Firefox/15.0-x64 PaleMoon/15.0-x64";
const char S79998[] = "Mozilla/5.0 (Linux; Android 4.0.4; VIGO7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S79999[] = "Nokia110/2.0 (03.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia110) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S80000[] = "NokiaN8-00/UCWEB8.4.0.150/51/800";
const char S80001[] = "Mozilla/5.0 (Java; U; en-in; nokian8-00) UCBrowser8.4.0.159/69/444/UCWEB Mobile  3gpp-gba";
const char S80002[] = "Mozilla/5.0 (Series40; Nokia7230/10.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S80003[] = "Mozilla/5.0 (Series40; Nokia7230-1c/10.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S80004[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80005[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80006[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80007[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80008[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; PTL21 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80009[] = "Mozilla/5.0 (Linux; Android 4.0.3; TAB275 Build/TAB275) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S80010[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus One Build/VJ CM10.1; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80011[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus One Build/VJ CM10.1; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80012[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; U20i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S80013[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-P3100) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S80014[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; DM-992 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80015[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; DM-992a Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80016[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IdeaTabA2207A-H Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80017[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Intex Aqua Swadesh Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80018[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; Andi4v Build/MocorDroid2.3.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80019[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-LS970 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S80020[] = "Mozilla/5.0 (Linux; Android 4.1.2; Micromax A116 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S80021[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; TB311 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80022[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; ZP810 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80023[] = "Mozilla/5.0 (Linux; Android 4.2.2; U30GT 2 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S80024[] = "Mozilla/5.0 (Linux; Android 4.0.4; C1604 Build/11.1.A.0.64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S80025[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9502 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S80026[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S80027[] = "LG-KF700/v10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S80028[] = "LG-KF700-Orange/v10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S80029[] = "LG/KF700-O2/v10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S80030[] = "LG-KF700C Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S80031[] = "LG/KF700/v10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S80032[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SonyEricssonX2/R3AA; Profile/MIDP-2.1; Configuration/CLDC-1.1; Windows Phone 6.5.3.5)";
const char S80033[] = "SAMSUNG-SGH-Z170/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S80034[] = "Nokia6610I/1.0 (4.10) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S80035[] = "HQ1 UP.Browser/7.2.6.1.591 (GUI) MMP/2.0";
const char S80036[] = "odkyom2k/ZQ1007 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0";
const char S80037[] = "wskyok4kc/1004 UP.Browser/7.2.6.1.838 (GUI) MMP/2.0";
const char S80038[] = "http://mms.revol.us/uaprofs/kyo-e1100/1003 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0";
const char S80039[] = "KWC-E1000/1.0.12 UP.Browser/7.2.6.1.537 (GUI) MMP/2.0";
const char S80040[] = "http://mms.revol.us/uaprofs/M1400-AAR.xml/AAR1004 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0";
const char S80041[] = "http://mms.revol.us/uaprofs/s4000-VQ/10022 UP.Browser/7.2.6.1.838 (GUI) MMP/2.0";
const char S80042[] = "http://mms.revol.us/uaprofs/kyo-e2000/1013 UP.Browser/7.2.6.1.567 (GUI) MMP/2.0";
const char S80043[] = "KDDI-CA3J UP.Browser/6.2_7.2.7.1.K.5.207 (GUI) MMP/2.0";
const char S80044[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Comodo_Dragon/4.1.1.12 Chrome/4.1.249.1042 Safari/532.5";
const char S80045[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80046[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80047[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-gb; GT-I9100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80048[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-I9100 Build/MIUI)";
const char S80049[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; GT-I9100 Build/MIUI.es) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80050[] = "NokiaC2-02/2.0 (06.52) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-02/UC Browser7.9.0.102/69/352";
const char S80051[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.1.6) Gecko/X FireDownload/2.0.1 Firefox/3.5.6 Wyzo/3.5.6.1 FireTorrent/2.0.1";
const char S80052[] = "nokiae5-00/UC Browser8.0.3.107/69/444  3gpp-gba";
const char S80053[] = "nokia500/UC Browser8.0.3.107/69/444  3gpp-gba";
const char S80054[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;PI39100;7.10;7720)";
const char S80055[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;TITAN X310e;7.10;7720)";
const char S80056[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;TITAN X310e;7.10;7740)";
const char S80057[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;TITAN X310e;7.10;8107)";
const char S80058[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;TITAN X310e;7.10;8773)";
const char S80059[] = "LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-kp500 teleca) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S80060[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-E860 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80061[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-E860 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S80062[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SN10T1 Build/IML77; CyanogenMod-TeamDRH) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80063[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; SPNG9716 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80064[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; GIONEE A10 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80065[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; A10 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80066[] = "Mozilla/5.0 (Windows NT 6.2; rv:10.0.3) Gecko/X Firefox/10.0.3 Epic/10.0.3";
const char S80067[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Fluid/0.9.6 Safari/536.26.14";
const char S80068[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) Fluid/0.9.6 Safari/536.28.10";
const char S80069[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) Fluid/0.9.6 Safari/536.30.1";
const char S80070[] = "Nokia306/2.0 (03.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia306) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S80071[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:10.0.7) Gecko/X YextBot/Java Firefox/10.0.7";
const char S80072[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s8530) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S80073[] = "NokiaX3-00/5.0 (11.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax3-00) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S80074[] = "Mozilla/5.0 (Linux; Android 4.0.4; A2 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80075[] = "SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonj108i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S80076[] = "Mozilla/5.0 (Linux; Android 4.0.4; iBall Slide 3G 7334 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S80077[] = "Mozilla/5.0 (Linux; Android 4.0.4; iBall Slide i9702 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S80078[] = "Mozilla/5.0 (Linux; Android 4.0.3; iBall Slide 3G 7307 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80079[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; N-04D Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80080[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; ALCATEL ONE TOUCH 918N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S80081[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Star Mobile Bright Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80082[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT22i Build/6.1.1.B.1.54) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S80083[] = "Nokia3020/5.0 (14.53) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S80084[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ608 Build/7.7.1-141-7-FLEM-UMTS-LA) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S80085[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ608-14) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S80086[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +0800, my";
const char S80087[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAE61i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S80088[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAE61) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S80089[] = "Mozilla/5.0 (Linux; Android 4.0.4; A10 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S80090[] = "Mozilla/5.0 (Linux; Android 4.0.4; A10 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S80091[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A9+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S80092[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Karbonn_A9+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S80093[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; CELKON_A9+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S80094[] = "MacOutlook/14.3.2.X (Intel Mac OS X 10.6.8)";
const char S80095[] = "MacOutlook/14.3.1.X (Intel Mac OS X 10.6.8)";
const char S80096[] = "MacOutlook/14.3.4.X (Intel Mac OS X 10.6.8)";
const char S80097[] = "MacOutlook/14.3.5.X (Intel Mac OS X 10.6.8)";
const char S80098[] = "MacOutlook/14.3.6.X (Intel Mac OS X 10.6.8)";
const char S80099[] = "MacOutlook/14.3.7.X (Intel Mac OS X 10.6.8)";
const char S80100[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100P Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S80101[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7100) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S80102[] = "Mozilla/5.0 (Linux; Android 4.1.2; Titanium S5 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S80103[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S80104[] = "Mozilla/5.0 (Linux; Android 4.0.4; SK17i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S80105[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100 (Edition Next)";
const char S80106[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153";
const char S80107[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S7500 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S80108[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; MITO A355 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80109[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P713 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S80110[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P713GO Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S80111[] = "Nokia309/2.0 (05.85) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia309) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S80112[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S80113[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P716 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S80114[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Desire S Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S80115[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; HTC Desire S Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S80116[] = "MOT-MOTOZ9/9E.03.3AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003352) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.60 [en]";
const char S80117[] = "MOT-MOTOZ9/9E.01.16R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003306) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.60 [en]";
const char S80118[] = "MOT-MOTOZ9/9E.02.0FR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; X) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [es]";
const char S80119[] = "Nokia5230/UC Browser7.7.1.88/50/444";
const char S80120[] = "Nokia5230-c/UC Browser7.7.2.91/50/355";
const char S80121[] = "KDDI-TS3R UP.Browser/6.2_7.2.7.1.K.4.1.203 (GUI) MMP/2.0";
const char S80122[] = "HTC Streaming Player verizon_wwe / 1.0 / htc_mecha / 2.3.4";
const char S80123[] = "HTC Streaming Player verizon_wwe / 1.0 / htc_mecha / 2.2.1";
const char S80124[] = "HTC Streaming Player verizon_wwe / 1.0 / htc_vigor / 2.3.4";
const char S80125[] = "HTC Streaming Player verizon_wwe / 1.0 / htc_vigor / 4.0.3";
const char S80126[] = "HTC Streaming Player verizon_wwe / 1.0 / htc_mecha / 4.0.4";
const char S80127[] = "KDDI-CA3H UP.Browser/6.2_7.2.7.1.K.5.207 (GUI) MMP/2.0";
const char S80128[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MID7005 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S80129[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone)/UC Browser7.2.2.51";
const char S80130[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 176x220)/UC Browser7.2.2.51";
const char S80131[] = "T-Mobile_Maple (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.2.2.51";
const char S80132[] = "Sprint:SPH-i325 320x240 (compatible; MSIE 4.01; Windows CE; Smartphone)/UC Browser7.2.2.51";
const char S80133[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.2.2.51";
const char S80134[] = "J2ME/UCWEB7.4.0.57/139/999";
const char S80135[] = "J2ME/UCWEB7.4.0.57/139/800";
const char S80136[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.4) Gecko/X FireDownload/2.0.1 Firefox/3.6.4 Wyzo/3.6.4.1 SearchToolbar/1.2 KAT";
const char S80137[] = "NSPlayer/4.1.0.3856";
const char S80138[] = "Mozilla/5.0 (Series40; Nokia2700c-2/07.80; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S80139[] = "nokia5233/UC Browser8.0.3.107/69/444  3gpp-gba";
const char S80140[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:5.0.1) Gecko/X Firefox/5.0.1 Facicons";
const char S80141[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-i589 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80142[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-i589 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S80143[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; zh-cn; SCH-i589 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80144[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia500)/UC Browser8.5.0.183/50/444/UCWEB Mobile";
const char S80145[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo E3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80146[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo E3 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S80147[] = "Mozilla/5.0 (Windows NT 6.2; rv:15.0) Gecko/X Firefox/15.1.1 PaleMoon/15.1.1";
const char S80148[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:15.0) Gecko/X Firefox/15.1.1-x64 PaleMoon/15.1.1-x64";
const char S80149[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Skate Build/JRO03C; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80150[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Skate Build/JRO03C; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80151[] = "Mozilla/5.0 (Linux; Android 4.0.3; AN10DG3 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S80152[] = "Mozilla/5.0 (Linux; Android 4.1.2; AT100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S80153[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A919 Duo Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80154[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; gt-s5233s) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S80155[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; gt-s5233s) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0";
const char S80156[] = "UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; gt-s5233s) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S80157[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A8809 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80158[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MITO A220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80159[] = "Mozilla/5.0 (Linux; Android 4.2.2; Desire HD Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S80160[] = "Mozilla/5.0 (Linux; Android 4.2.1; Desire HD Build/JOP40G) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S80161[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I-Buddy 3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80162[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S80163[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-S959G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S80164[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000";
const char S80165[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, -0600, us";
const char S80166[] = "Mozilla/5.0 (compatible; MSIE 3.0; Windows NT 6.1; Trident/1.0)";
const char S80167[] = "Mozilla/5.0 (Linux; Android 4.0.4; CINK PEAX Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S80168[] = "Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook Tablet Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S80169[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9210T Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S80170[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3 XiaoMi/MiuiBrowser/1.0";
const char S80171[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; CTP828 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80172[] = "Opera/9.80 (Series 60; Opera Mini/7.6.32889/29.3551; U; zh) Presto/2.8.119 Version/11.10";
const char S80173[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N7200   Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80174[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; N7200 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80175[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Karbonn A2 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S80176[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A2 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S80177[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P940 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S80178[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-KU5400 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S80179[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-SU540 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S80180[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S80181[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8190 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S80182[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; GT-I8190 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S80183[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S80184[] = "Mozilla/5.0 (Linux; Android 4.1.2; AMID-9743G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S80185[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; XT912 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/spyder";
const char S80186[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; XT912 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/spyder";
const char S80187[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; XT912 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/spyder";
const char S80188[] = "Mozilla/5.0 (Linux; Android 4.0.4; GSmart GS202+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S80189[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-G810/XEHA3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S80190[] = "Mozilla/5.0 (SymbianOS/9.2; Series60/3.1 Samsung/SGH-G810/JVHE1 Profile/MIDP-2.0 Configuration/CLDC-1.1 U; ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S80191[] = "Mozilla/5.0 (SymbianOS/9.2; Series60/3.1 Samsung/SGH-G810/DUHF2 Profile/MIDP-2.0 Configuration/CLDC-1.1 U; ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S80192[] = "Mozilla/5.0 (SymbianOS/9.2; Series60/3.1 Samsung/SGH-G810/JCHE2 Profile/MIDP-2.0 Configuration/CLDC-1.1 U; ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S80193[] = "Mozilla/5.0 (SymbianOS/9.2; Series60/3.1 Samsung/SGH-G810/JPHE1 Profile/MIDP-2.0 Configuration/CLDC-1.1 U; ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S80194[] = "Mozilla/5.0 (SymbianOS/9.2; Series60/3.1 Samsung/SGH-G810/DZHD4 Profile/MIDP-2.0 Configuration/CLDC-1.1 U; ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S80195[] = "Mozilla/5.0 (SymbianOS/9.2; Series60/3.1 Samsung/SGH-G810/JXHE1 Profile/MIDP-2.0 Configuration/CLDC-1.1 U; ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S80196[] = "Mozilla/5.0 (SymbianOS/9.2; Series60/3.1 Samsung/SGH-G810/DBHD4 Profile/MIDP-2.0 Configuration/CLDC-1.1 U; ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S80197[] = "AppleCoreMedia/1.0.0.8A400 (iPhone; U; CPU OS 4_0_2 like Mac OS X; en_us)";
const char S80198[] = "AppleCoreMedia/1.0.0.8A293 (iPhone; U; CPU OS 4_0 like Mac OS X; en_us)";
const char S80199[] = "AppleCoreMedia/1.0.0.8A306 (iPhone; U; CPU OS 4_0_1 like Mac OS X; en_us)";
const char S80200[] = "BlackBerry8910_CMCC/5.0.0.608 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/204";
const char S80201[] = "BlackBerry8910/5.0.0.461 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/612";
const char S80202[] = "NokiaE63/UC Browser7.8.0.95/28/355";
const char S80203[] = "MicromaxQ75";
const char S80204[] = "KDDI-SN3Q UP.Browser/6.2_7.2.7.1.K.6.1.102 (GUI) MMP/2.0";
const char S80205[] = "Mozilla/5.0 (X11; Linux i686 on x86_64; rv:6.0.2) Gecko/X Firefox/6.0.2 SeaMonkey/2.3.3";
const char S80206[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S80207[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5830i Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S80208[] = "Mozilla/5.0 (Series40; Nokia200/10.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S80209[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-US) AppleWebKit/125.4 (KHTML, like Gecko, Safari) OmniWeb/v563.15";
const char S80210[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-US) AppleWebKit/125.4 (KHTML, like Gecko, Safari) OmniWeb/v563.57";
const char S80211[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-US) AppleWebKit/125.4 (KHTML, like Gecko, Safari) OmniWeb/v563.66";
const char S80212[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-US) AppleWebKit/125.4 (KHTML, like Gecko, Safari) OmniWeb/v563.34";
const char S80213[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE52-1)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S80214[] = "Mozilla/5.0 (Windows; U; Win 9x 4.90; de-DE; rv:0.9.2) Gecko/X Netscape6/6.1";
const char S80215[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M-285 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80216[] = "Mozilla/5.0 (Linux; Android 4.0.3; F-10D Build/V20R47F) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80217[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; LG-C660 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80218[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:15.0) Gecko/X Firefox/15.0.1 SeaMonkey/2.12.1";
const char S80219[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Y2 Build/HCL ME Tablet Y2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80220[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.1.1m";
const char S80221[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.1.2m";
const char S80222[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.1.3m";
const char S80223[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.1.4m";
const char S80224[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A360/S100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80225[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A360 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S80226[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A360 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80227[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80228[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S12 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S80229[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U1 Build/HCL ME Tablet U1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80230[] = "OneBrowser/3.1 (SAMSUNG-GT-C3312)";
const char S80231[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1 (like Version/10A551 Mobile/6.0.2 Safari/8536.25)";
const char S80232[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1 (like Version/10A523 Mobile/6.0.1 Safari/8536.25)";
const char S80233[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; HW-01E Build/HuaweiU9501L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80234[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S120 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80235[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5660) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S80236[] = "Mozilla/5.0 (Linux; Android 4.0.3; M532 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S80237[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6810 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80238[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6810 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80239[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6810 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80240[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P6810 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80241[] = "Mozilla/5.0 (Linux; Android 4.2.2; NookTablet Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S80242[] = "Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook Tablet Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S80243[] = "Mozilla/5.0 (Linux; Android 4.2.1; Barnes & Noble Nook Tablet Build/JOP40G) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S80244[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; LIFETAB_E10310 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80245[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; LIFETAB_E10310 Build/JRO03H)";
const char S80246[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Novo7Fire Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80247[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Novo7Fire Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80248[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Smartbook U25GT-S701 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80249[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Touchpad Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80250[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; TouchPad Build/JDQ39E)";
const char S80251[] = "UCWEB/2.0 (Windows; U; wds 8.0; zh-CN; NOKIA; RM-826_apac_prc_202) U2/1.0.0 UCBrowser/3.0.0.285 U2/1.0.0 Mobile";
const char S80252[] = "UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-820_nam_att_100) U2/1.0.0 UCBrowser/3.0.0.285 U2/1.0.0 Mobile";
const char S80253[] = "UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-826_im_india_201) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile";
const char S80254[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Celkon A119Q Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80255[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire C Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S80256[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P870 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S80257[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S5303 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S80258[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; PAP5044DUO Build/PrestigioPAP5044DUO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80259[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT885 Build/9.8.3O-148_IRD-3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S80260[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC 802w Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S80261[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P765 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S80262[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) HTCS620;Smartphone;320x240";
const char S80263[] = "LG/KC910i-Bouygues/V10a Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S80264[] = "Mozilla/5.0 (Vodafone/1.0/LG-KC910i/V10b Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S80265[] = "LG/KC910i/V10a Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S80266[] = "LG-KC910i-v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S80267[] = "Mozilla/5.0 (Vodafone/1.0/LG-KC910i/V10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S80268[] = "LG-KC910i-Orange/V10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S80269[] = "LG-KC910i/V10a Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S80270[] = "ztef350/1.0 Polaris/6.1.0.0 (GUI) MMP/2.0";
const char S80271[] = "R260/P107D10/Obigo Q03C/MIDP-2.0 Configuration/CLDC-1.1";
const char S80272[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; LU2300 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80273[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; LU2300 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80274[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.2; LU2300 Build/FRG83G)";
const char S80275[] = "JUC(Linux;U;Android2.2.1;Zh_cn;HTC Wildfire;240*320;)UCWEB7.8.0.95/139/444";
const char S80276[] = "Mozilla/5.0(Linux;U;Android2.2.1;Zh_cn;HTC Wildfire;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S80277[] = "Mozilla/5.0 (Windows NT 6.1; rv:9.1) Gecko/X Firefox/9.1 PaleMoon/9.1";
const char S80278[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E16i Build/4.0.1.A.0.283; GingerElite) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80279[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E16i Build/3.0.1.A.0.145; Ginger Ultimate) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80280[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E16i Build/3; GingerElite V3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80281[] = "Mozilla/5.0 (Windows NT 5.1; rv:15.0) Gecko/X Firefox/15.0 slurp";
const char S80282[] = "Mozilla/5.0 (Windows NT 5.1; rv:15.0) Gecko/X Firefox/15.0.1 slurp";
const char S80283[] = "Mozilla/5.0 (Windows NT 5.1; rv:15.0) Gecko/X Firefox/15.0.1 slurp slurp";
const char S80284[] = "MQQBrowser/37 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S80285[] = "MQQBrowser/37 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3";
const char S80286[] = "MQQBrowser/371 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S80287[] = "MQQBrowser/371 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3";
const char S80288[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TAB-831 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80289[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; Xteam 4.0 Smartpad 810c Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80290[] = "Mozilla/5.0 (Linux; U; Android 4.1.1-SDK_V1.00_X.X release-keys; it-it; Xteam Smartpad 810c Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80291[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; Xteam 4.1 Smartpad 810c Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80292[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; Xteam Smartpad 810c Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80293[] = "Mozilla/5.0 (Linux; Android 4.2; GT-P7310 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S80294[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P7310 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S80295[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7310 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S80296[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; NW-Z1000Series Build/E200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80297[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; NW-Z1000Series Build/E201) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80298[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; A350 TV Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80299[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; WT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S80300[] = "Mozilla/5.0 (X11; Linux x86_64; PadFone Infinity) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S80301[] = "Nokia114/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S80302[] = "Mozilla/5.0 (Linux; Android 4.1.1; Orange Daytona Build/HuaweiG510-0200) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S80303[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0200 Build/HuaweiG510-0200) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S80304[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTL21 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S80305[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I605 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S80306[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; folio100 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80307[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GOCLEVER TAB M703G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80308[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S80309[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S80310[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI C8812E Build/HuaweiC8812E) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S80311[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7";
const char S80312[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P5100 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7";
const char S80313[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Incredible 4G LTE Build/IMM76D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80314[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Celkon A 59 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80315[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mauie800) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S80316[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mauie800) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S80317[] = "Mozilla/5.0 (Linux; Android 4.1.1; U7015 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80318[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P700 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S80319[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.4.2.34167 (Firestorm-Release; firestorm skin) Safari/533.3";
const char S80320[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L720 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S80321[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S80322[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S80323[] = "Mozilla/5.0 (Linux; Android 3.2; GT-P7510 Build/HTJ85B) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S80324[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9128 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80325[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9128 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S80326[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9128 Build/JZO54K)";
const char S80327[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S80328[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT885 Build/9.8.3O-148_IRD-3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S80329[] = "Mozilla/5.0 (Linux; Android 4.0.4; StarTab 715 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S80330[] = "Mozilla/5.0 (Linux; Android 4.0.3; Smartfren Tab 7 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S80331[] = "Mozilla/5.0 (Linux; Android 4.0.4; StarTab 715x Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S80332[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5360 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S80333[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi; GT-S5360 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S80334[] = "Mozilla/5.0 (Linux; Android 4.2.1; E3 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S80335[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC first Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S80336[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i780/1.0";
const char S80337[] = "SAMSUNG-SGH-i780TIM/PAIHI4_IC3 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S80338[] = "SAMSUNG-SGH-i780ORANGE/PBVHK1_IB1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S80339[] = "SAMSUNG-SGH-i780/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S80340[] = "Samsung-SPHM350 Access-NetFront/3.1.5 MMP/2.0";
const char S80341[] = "Samsung-SPHM520 AU-MIC-M520/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S80342[] = "LG/KU990i/v1.0";
const char S80343[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; R800i Build/4.1.H.0.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80344[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonR800i Build/4.0.2.A.0.62) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80345[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; R800i Build/4.0.2.A.0.62) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80346[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P925 Build/GRJ90; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80347[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P925 Build/GRJ90; CyanogenMod 7.2+ v1.6.0 P925) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80348[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Dell Streak Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80349[] = "ZTE-R1/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S80350[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; APPTB701BV2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80351[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S80352[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A840S Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80353[] = "AppleCoreMedia/1.0.0.10A406e (Apple TV; U; CPU OS 6_0 like Mac OS X; en_us)";
const char S80354[] = "AppleCoreMedia/1.0.0.10A831 (Apple TV; U; CPU OS 6_0_1 like Mac OS X; en_us)";
const char S80355[] = "AppleCoreMedia/1.0.0.10B329a (Apple TV; U; CPU OS 6_1_3 like Mac OS X; en_us)";
const char S80356[] = "AppleCoreMedia/1.0.0.9B830 (Apple?TV; U; CPU OS 5_1_1 like Mac OS X; en_us)";
const char S80357[] = "AppleCoreMedia/1.0.0.10B809 (Apple TV; U; CPU OS 6_1_4 like Mac OS X; en_us)";
const char S80358[] = "JUC(Linux;U;Android4.0.3;Zh_cn;GT-P3100;600*976;)UCWEB7.8.0.95/139/444";
const char S80359[] = "JUC(Linux;U;Android4.0.4;Zh_cn;GT-P3100;600*976;)UCWEB7.8.0.95/139/444";
const char S80360[] = "Mozilla/5.0(Linux;U;Android4.0.3;Zh_cn;GT-P3100;600*976;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S80361[] = "Mozilla/5.0(Linux;U;Android4.0.4;Zh_cn;GT-P3100;600*976;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S80362[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; IdeaTabS2110AF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80363[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; IdeaTabS2110AF Build/IML74K)";
const char S80364[] = "Opera/9.80 (Android 3.0.1; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S80365[] = "Opera/9.80 (Android 3.0UI; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S80366[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; A606 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80367[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Viettel I6 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80368[] = "LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gd510) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S80369[] = "Opera/9.80 (X11; FreeBSD 9.1-RELEASE amd64) Presto/2.12.388 Version/12.14";
const char S80370[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E971 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S80371[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; TECNO D3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80372[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; TECNO D3 Build/GRK39F)";
const char S80373[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Samsung GT-S9081 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80374[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; Samsung GT-S9081 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80375[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5580C Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S80376[] = "Mozilla/5.0 (compatible; MSIE 4.0; Windows NT 6.1; Trident/5.0)";
const char S80377[] = "Skype/4.6.0.306 CFNetwork/609.1.4 Darwin/13.0.0";
const char S80378[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IRIS_501 Build/LAVAIRIS501) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80379[] = "Mozilla/5.0 (Linux; Android 4.2.1; ZP980 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S80380[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One XL Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S80381[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One XL Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S80382[] = "Mozilla/5.0 (Linux; Android 4.0.4; SK17i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S80383[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S80384[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL_ONE_TOUCH_6010X_Orange Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S80385[] = "Mozilla/5.0 (Linux; Android 4.1.2; A111 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S80386[] = "Mozilla/5.0 (Linux; Android 4.1.1; TAB-840_G Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S80387[] = "samsung-GT-I9500/1.0 Linux/2.6.35.7 Android/4.1.2 Release/07.18.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S80388[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaX6-00) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S80389[] = "LG-GR501/V10c Obigo/Q7.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S80390[] = "SCH-U750P/1.0 NetFront/3.0.22.2.17 (GUI) MMP/2.0";
const char S80391[] = "Mozilla/4.1 (compatible; Teleca/Q05A/GS-CX10000)";
const char S80392[] = "MTC/1.0/0MTC 733/B100 Browser/Obigo-Browser/Q05A MMS/Obigo-MMS/Q05A SyncML/HW-SyncML/1.0 Java/HWJa/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Player/QTV-Player/5.3";
const char S80393[] = "Panama-orange/Obigo Browser/Q05A Browser/Q03C MMS/1.0/MIDP-2.0 Configuration/CLDC-1.1";
const char S80394[] = "TCL-Q3/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S80395[] = "HTC-P4600/%ROM_VERSION% Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S80396[] = "HTC-P4600/5.11.502.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S80397[] = "HTC-P4600/1.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S80398[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; ZTE_GV821 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80399[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; HTC Dream G1 Build/DRD35) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S80400[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-sg; HTC Dream Build/DRD20) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S80401[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; HTC Dream Build/DRC83) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S80402[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Z71 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80403[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Z71 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80404[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Z71 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80405[] = "Mozilla/5.0 (Series40; Nokia2730c-1/10.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S80406[] = "Mozilla/5.0 (Windows NT 6.2; rv:9.0.1) Gecko/X Firefox/9.0.1 CometBird/9.0.1";
const char S80407[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SX-SP715A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80408[] = "Logos/2.2.8.1 (Linux; U; Android 4.0.3; en-US; SX-SP715A Build/ALPHA2 4.0.3 X.O A1)";
const char S80409[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; SX-SP715A Build/IML74K)";
const char S80410[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; SX-SP715A Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S80411[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NookTablet Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80412[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NookTablet Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80413[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S80414[] = "Mozilla/5.0 (Linux; Android 4.0.6; A9550 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80415[] = "LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-gs290) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S80416[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-I9210T Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80417[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; GT-I9210T Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80418[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-c6712) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S80419[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TAB275 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80420[] = "NOKIA-Lumia 610/UCWEB2.7.0.223/47/355";
const char S80421[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAD-97052 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S80422[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAC-97052 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S80423[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) rekonq/2.0 Safari/534.34";
const char S80424[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; TAD-10021 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80425[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Pro Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80426[] = "iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Karbonn A30 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80427[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S80428[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444";
const char S80429[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S80430[] = "ZEN P8 /Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S80431[] = "ZEN P8i /Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S80432[] = "Micromax X396/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S80433[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL_one_touch_995 Build/IceCreamSandwich) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S80434[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN86 8MP) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S80435[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; ZTE N909 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80436[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; ZTE N909 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S80437[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Starmobile DIAMOND Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80438[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; Starmobile DIAMOND V3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80439[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6606 Build/10.1.1.B.0.166) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S80440[] = "Mozilla/5.0 (Linux; Android 4.1.2; dtab01 Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S80441[] = "Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 10 LINK Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S80442[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Spice Mi-725 Build/Mi725) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80443[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT881 Build/6.7.3-130_YUS-11) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S80444[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Edison 3G Build/2.0.0 X-17:50) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80445[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Edison 2 3G Build/1.0.1_X-10:06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80446[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Edison 2 3G Build/1.0.2_X-08:25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80447[] = "Mozilla/5.0 (Linux; Android 4.2.2; Optimus G Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S80448[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S80449[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570I) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S80450[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAP4500TDUO Build/PrestigioPAP4500TDUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S80451[] = "Mozilla/5.0 (Linux; Android 4.1.1; HP Slate 7 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S80452[] = "BlackBerry7290/4.1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/104";
const char S80453[] = "Nokia7270/2.0 (03.24) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S80454[] = "Nokia3208c_CMCC/2.0 (09.25) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S80455[] = "Nokia3208c/2.0 (10.50) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S80456[] = "Nokia3208c/2.0 (10.50) Profile/MIDP-2.1 Configuration/CLDC-1.1 Nokia3208c/10.50";
const char S80457[] = "PANTECH-C520/IQUS04212008 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S80458[] = "PANTECH-C520/IQUS04132009 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S80459[] = "Nokia5230/UC Browser7.8.0.95/50/355";
const char S80460[] = "KDDI-SH3F UP.Browser/6.2_7.2.7.1.K.4.306 (GUI) MMP/2.0";
const char S80461[] = "JUC(Linux;U;2.3.3;Zh_cn;C8650;320*480;)UCWEB7.6.0.75/139/32046";
const char S80462[] = "WebProcess/7534.51.22 CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookPro8%2C1)";
const char S80463[] = "WebProcess/7534.51.22 CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookAir4%2C2)";
const char S80464[] = "WebProcess/7534.52.7 CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (iMac12%2C1)";
const char S80465[] = "RealPlayer%20Downloader/1725 CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookPro8%2C1)";
const char S80466[] = "ocspd (unknown version) CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookPro8%2C1)";
const char S80467[] = "iCoreService (unknown version) CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookPro8%2C1)";
const char S80468[] = "XProtectUpdater (unknown version) CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookPro8%2C1)";
const char S80469[] = "Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko/X Thunderbird/11.0";
const char S80470[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:11.0) Gecko/X Thunderbird/11.0.1";
const char S80471[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; 7DTB2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80472[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; STARTEXT II Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, comme Gecko) Version/4.0 Mobile Safari/533.1";
const char S80473[] = "System Center 2012 - Operations Manager 7.0.8560.0";
const char S80474[] = "System Center 2012 - Operations Manager 7.0.9538.0";
const char S80475[] = "Mac OS X/10.6.8 (10K549); ExchangeWebServices/1.3 (61); Mail/4.6 (1085)";
const char S80476[] = "Nokia3110c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia3110c) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S80477[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-gb; SonyEricssonLT18i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80478[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-gb; LT18i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80479[] = "Micromax X231/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S80480[] = "Mozilla/5.0 (Linux; Android 4.2.1; Barnes & Noble Nook Tablet Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S80481[] = "Mozilla/5.0 (Linux; Android 4.2.1; NookTablet Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S80482[] = "Mozilla/5.0 (Linux; Android 4.2; NookTablet Build/JOP40C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S80483[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT18i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S80484[] = "Mozilla/5.0 (compatible; MSIE 3.0; Windows NT 5.2; Trident/3.0)";
const char S80485[] = "Mozilla/5.0 (compatible; MSIE 3.0; Windows NT 5.1; Trident/4.0)";
const char S80486[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; M031 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80487[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; LG-P500) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S80488[] = "Mozilla/5.0 (Linux; Android 4.1.1; Cink Peax Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S80489[] = "Mozilla/5.0 (Linux; Android 4.1.2; Cink Peax Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S80490[] = "Mozilla/5.0 (Linux; Android 4.1.2; CINK PEAX 2 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S80491[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Blaze S180  Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80492[] = "Mozilla/5.0 (Linux; Android 4.1.1; A701 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S80493[] = "Mozilla/5.0 (Linux; Android 4.1.1; E270BSA Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S80494[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; th-th; i-mobile IQ XA Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80495[] = "Mozilla/5.0 (Linux; Android 4.1.1; D2-721G Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S80496[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT22i Build/6.2.A.1.100) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S80497[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT22i Build/6.2.A.0.400) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S80498[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; fnac 5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80499[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993D Build/ICECREAM) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S80500[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993 Build/ICECREAM) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S80501[] = "Mozilla/5.0 (Linux; Android 4.2.1; Q1000 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S80502[] = "MOT-K1/08.02.81R MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/1.0";
const char S80503[] = "SAMSUNG-SGH-i780ORANGE/PBVIC2 (compatible; MSIE 6.0; Windows CE; PPC) Opera 8.65 [fr]";
const char S80504[] = "SAMSUNG-SGH-i780/1.0 (compatible; MSIE 6.0; Windows CE; PPC) Opera 8.65";
const char S80505[] = "SAMSUNG-SGH-i780V/MJHA3 (compatible; MSIE 6.0; Windows CE; PPC) Opera 8.65";
const char S80506[] = "Nokia3610a_CMCC/2.0 (03.56) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S80507[] = "Nokia3610a/2.0 (03.56) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S80508[] = "SAMSUNG-SGH-Z400-Vodafone/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S80509[] = "SAMSUNG-SGH-Z400/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S80510[] = "SonyEricssonT707a/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3";
const char S80511[] = "KDDI-SN3T UP.Browser/6.2_7.2.7.1.K.6.1.102 (GUI) MMP/2.0";
const char S80512[] = "MicromaxX510";
const char S80513[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Cosmo Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80514[] = "Mozilla/5.0 (Linux; U; Android 2.2; ru-ru; Cosmo Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80515[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ru-ru; Cosmo Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80516[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; us-us; POV-Mobii-7 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S80517[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; us-us; POV Mobii 7 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S80518[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.1; es-es; POV-Mobii-7 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S80519[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.3; es-es; MERM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S80520[] = "Opera/9.80 (X11; Linux x86_64; U; Linux Mint; en) Presto/2.2.15 Version/10.10";
const char S80521[] = "Mozilla/5.0 (BeOS; U; BeOS BePC; en-US; rv:1.9a1) Gecko/X SeaMonkey/1.5a";
const char S80522[] = "Mozilla/5.0 (Windows ME; U; en) Opera 8.51";
const char S80523[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows ME; ru) Opera 8.50";
const char S80524[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; IM-A775C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80525[] = "NOKIA-Lumia 610/UCWEB2.2.0.157/47/355";
const char S80526[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-N7100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S80527[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-N7100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S80528[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1811_v75_jbaol1_g88 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80529[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1811_v75_hjy1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80530[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1811_v75_jyuany1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80531[] = "Mozilla/5.0 (Linux; Android 4.0.3; e1811_v75_jbaol1_g58 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80532[] = "JUC (Linux; U; 2.3.6; zh-cn; A45; 320*480) UCWEB7.9.0.94/139/444";
const char S80533[] = "Mozilla/5.0 (Linux; Android 4.0.3; PAP4300 DUO Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80534[] = "Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X; zh-cn Model:iPad3,3) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7543.48.3 UCBrowser/1.0.0.192/43/355";
const char S80535[] = "JUC (Linux; U; 4.0.4; zh-cn; Micromax_A110; 480*854) UCWEB7.9.0.94/139/444";
const char S80536[] = "Mozilla/5.0 (Linux; Android 4.1.1; M1070 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S80537[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80538[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80539[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80540[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80541[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3100B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80542[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80543[] = "Nokia3110c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3110c) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S80544[] = "Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia3120classic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S80545[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S80546[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Onyx Athena 10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80547[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AUXUS CoreX4 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80548[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7102 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S80549[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7102 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S80550[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; WALTON Primo_G1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80551[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Primo_G1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80552[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; WF7C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80553[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; XBLWP7; ZuneWP7) UCBrowser/2.9.0.263";
const char S80554[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC Explorer A310e) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S80555[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC Explorer A310e) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S80556[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A27 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S80557[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A840S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S80558[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A840SP Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S80559[] = "Mozilla/5.0 (Linux; Android 4.0.4; CINK KING Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S80560[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT918 Build/2_26D_2009) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S80561[] = "Nokia205/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia205) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S80562[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT910 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S80563[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; H7500+ Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80564[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST23i Build/11.0.A.5.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S80565[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-iNote-mini/1.0 Android/2.3.5 Release/05.18.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80566[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N5110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us";
const char S80567[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S80568[] = "Mozilla/5.0 (Linux; Android 4.1.1; C1605 Build/11.3.A.2.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S80569[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S80570[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S80571[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S6012 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S80572[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ608 Build/7.7.1-141-7-FLEM-UMTS-LA) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S80573[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ608-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S80574[] = "SAMSUNG-SGH-M610/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S80575[] = "MOT-EX130/1.0 REXL4/1.0 BrewMP/1.0.3 Release/6.15.2011 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S80576[] = "LG-A200/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S80577[] = "Mozilla/5.0 (Linux; U; Linux Ventana; en-us; Slider SL101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S80578[] = "Mozilla/5.0 (Linux; U; Linux Ventana; de-de; Slider SL101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S80579[] = "Mozilla/5.0 (Linux; U; Linux Ventana; zh-tw; Slider SL101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S80580[] = "Mozilla/5.0 (Linux; U; Linux Ventana; en-gb; Slider SL101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S80581[] = "Mozilla/5.0 (Linux; U; Linux Ventana; zh-tw; Slider SL101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S80582[] = "Mozilla/5.0 (Linux; U; Linux Ventana; zh-tw; Slider SL101 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S80583[] = "KDDI-TS3S UP.Browser/6.2_7.2.7.1.K.5.212 (GUI) MMP/2.0";
const char S80584[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Inspire 4G Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80585[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Inspire 4G Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80586[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Inspire 4G Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80587[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Sensation Build/GRJ22; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80588[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Sensation Build/GRJ22; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80589[] = "mstar platform/UC Browser7.9.0.102/69/352";
const char S80590[] = "opera/UC Browser7.9.0.102/69/352";
const char S80591[] = "mstar/UC Browser7.9.0.102/69/352";
const char S80592[] = "r620/UC Browser7.9.0.102/69/352 TRUSTED/1.0";
const char S80593[] = "JUC (Linux; U; 2.3.4; zh-cn; MT15i; 480*854) UCWEB7.9.0.94/139/444";
const char S80594[] = "JUC (Linux; U; 4.0.4; zh-cn; MT15i; 480*854) UCWEB7.9.0.94/139/444";
const char S80595[] = "Mozilla/5.0 (S60V5; U; en-in; NokiaC6-00)/UC Browser8.4.0.159/50/352/UCWEB Mobile";
const char S80596[] = "NokiaC6-00/UCWEB8.4.0.150/50/800";
const char S80597[] = "Mozilla/5.0 (Java; U; en-us; nokiac6-00) UCBrowser8.4.0.159/69/352/UCWEB Mobile  3gpp-gba";
const char S80598[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaX6-00)/UC Browser8.4.0.159/50/352/UCWEB Mobile";
const char S80599[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; NGM Vanity Smart Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80600[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; ST18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S80601[] = "Mozilla/5.0 (Linux; Android 4.1.2; Inspire 4G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80602[] = "Mozilla/5.0 (Linux; Android 4.1.1; Inspire 4G Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80603[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; W806 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80604[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; W806 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S80605[] = "Mozilla/5.0 (Linux; U;AliyunOS 1.6.1.16-RT-X.X; zh-cn; W806 Build/AliyunOs-2012;Android 2.3 Compatible) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80606[] = "Nokia7230/5.0 (10.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia7230) UCBrowser8.4.0.159/70/444/UCWEB Mobile";
const char S80607[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE75-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile";
const char S80608[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; MODECOM FreeTAB 9702 IPS X2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80609[] = "Mozilla/5.0 (Linux; Android 4.0.4; Telenor_One_Touch_S Build/IceCreamSandwich) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80610[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-S5570 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80611[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-S5570 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80612[] = "Microsoft Office/12.0 (Windows NT 6.2; Microsoft Office Outlook 12.0.6662; Pro)";
const char S80613[] = "Microsoft Office/12.0 (Windows NT 6.2; Microsoft Office Outlook 12.0.4518; Pro)";
const char S80614[] = "Microsoft Office/12.0 (Windows NT 6.2; Microsoft Office Outlook 12.0.6607; Pro)";
const char S80615[] = "Microsoft Office/12.0 (Windows NT 6.2; Microsoft Office Outlook 12.0.6683; Pro)";
const char S80616[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaX7-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S80617[] = "Mozilla/5.0 (Linux; Android 4.0.4; MI-ONE Plus Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S80618[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I915 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S80619[] = "Mozilla/5.0 (Windows; N; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth Pro/7.0.2.8415 Safari/532.4";
const char S80620[] = "Mozilla/5.0 (Linux; Android 4.0.4; SK17i Build/4.1.B.0.587) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S80621[] = "Mozilla/5.0 (Linux; Android 4.0.4; SK17a Build/4.1.B.0.431) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S80622[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; POV_TV-HDMI-200BT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80623[] = "Mozilla/5.0 (Linux; Android 4.0.3; NS-13T001 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S80624[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HUAWEI C8813 Build/HuaweiC8813) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31";
const char S80625[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Janetter/1.5.2";
const char S80626[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Janetter/1.5.2";
const char S80627[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Micromax A110) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S80628[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P765 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S80629[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ608 Build/7.7.1-141-7-FLEM-UMTS-LA) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S80630[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ608-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S80631[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A770K Build/JZO54K; CyanogenMod-CM10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80632[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A770K Build/update by fengdian; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80633[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A770K Build/->Wangsai_008; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80634[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I897 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S80635[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-I5700 Build/GWK74; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80636[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAC-70061 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80637[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-M919 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SGH-M919, 52, 1.9.9.2)";
const char S80638[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S80639[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337M Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S80640[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCL21 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S80641[] = "Mozilla/5.0 (Linux; Android 4.1.1; C1604 Build/11.3.A.2.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S80642[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Q600 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80643[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian95) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S80644[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN95) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S80645[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN95 8GB) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S80646[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-US; Nexus 4 Build/JWR66Y) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S80647[] = "MOT-MOTORAZRV9/9C.14.36R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 14003276) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S80648[] = "MOT-MOTORAZRV9/9C.14.1CR8 BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 14003262) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S80649[] = "MOT-MOTORAZRV9/9C.13.43R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 14183245) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S80650[] = "MOT-MOTORAZRV9/9C.13.37R3 BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 14183237) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [en]";
const char S80651[] = "MOT-MOTORAZRV9/9C.13.37R3 BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; X) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [en]";
const char S80652[] = "SAMSUNG-GT-B7350Orange/BVJI1 Opera/9.80 (Windows Mobile; Windows CE; Opera Mobi/ORS-75XXX; U) Presto/2,4,13 Version/10.00";
const char S80653[] = "SAMSUNG-GT-B7350/1.0 Opera/9.80 (Windows Mobile; Windows CE; Opera Mobi/ORS-75XXX; U) Presto/2,4,13 Version/10.00";
const char S80654[] = "Mozilla/5.0(Linux;U;Android2.2;Zh_cn;GT-P1000;600*1024;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355";
const char S80655[] = "JUC(Linux;U;Android2.2;Zh_cn;GT-P1000;600*1024;)UCWEB7.8.0.95/139/352";
const char S80656[] = "Mozilla/5.0(Linux;U;Android2.2;Zh_cn;GT-P1000T;600*1024;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/444";
const char S80657[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; 003Z Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80658[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; 003Z Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80659[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Iron/3.0.197.0 Safari/532.0";
const char S80660[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.1.1.0 Chrome/14.0.835.202 Safari/535.1";
const char S80661[] = "MicromaxX660/   Browser/Opera profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10";
const char S80662[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7 HTC_ChaCha-orange-LS; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S80663[] = "MQQBrowser/2.5.1 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3";
const char S80664[] = "MQQBrowser/2.5.1 Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3";
const char S80665[] = "MQQBrowser/2.5 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 Safari/7534.48.3";
const char S80666[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; WT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S80667[] = "MicromaxX274/Q03C MAUI-browser/CLDC-1.1";
const char S80668[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M440S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80669[] = "SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-in; sonyericssonj108i) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S80670[] = "LenovoS899t_TD/1.0 Android/4.0 Release/02.01.2012 Browser/WAP2.0 appleWebkit/534.30";
const char S80671[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; BOULM727MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80672[] = "Mozilla/5.0 (Linux; Android 4.0.4; N10 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S80673[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-c6712) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S80674[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I200 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S80675[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S7500) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S80676[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.78; en-us; 8320/2.7.0.78) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S80677[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.105; en-us; 8320/2.7.0.105) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S80678[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.106; en-us; 8320/2.7.0.106) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S80679[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.110; en-us; 8320/2.7.0.110) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S80680[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.92; en-us; 8320/2.7.0.92) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S80681[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; TECNO_N7 Build/TECNOTECNO_N7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/33567";
const char S80682[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST18i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S80683[] = "Mozilla/5.0 (Linux; Android 4.1.2; A511 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S80684[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E615 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S80685[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E615f Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S80686[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTL21 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S80687[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTL21 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S80688[] = "SonyEricssonW595/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw595) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S80689[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6802) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S80690[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; X506S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80691[] = "Mozilla/5.0 (Linux; Android 4.1.2; JY-G3 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S80692[] = "Mozilla/5.0 (Linux; Android 4.1.2; JY-G3N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S80693[] = "Mozilla/5.0 (X11; Linux x86_64; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24 TT/4.1.1";
const char S80694[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; IQ445 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80695[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; Fly_IQ445 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80696[] = "SAMSUNG-GT-S3800W/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S80697[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S80698[] = "Mozilla/5.0 (Linux; Android 4.1.2; SOL21 Build/9.1.D.0.401) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S80699[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE75-1) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S80700[] = "Mozilla/5.0 (Linux; U; Android 4.4; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.4 Release/04.11.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80701[] = "Mozilla/5.0 (Linux; U; Android 4.4; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.4 Release/04.19.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80702[] = "Mozilla/5.0 (Linux; U; Android 4.4; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.4 Release/04.02.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80703[] = "Mozilla/5.0 (Linux; U; Android 4.4; en-us; hsdroid) antivirusHttpClient/2.0 (KHTML, like Gecko) Version/Version 2.3/antivirus";
const char S80704[] = "Mozilla/5.0 (Linux; U; Android 4.4; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.4 Release/05.13.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80705[] = "Mozilla/5.0 (Linux; U; Android 4.4; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.4 Release/04.26.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80706[] = "Mozilla/5.0 (Linux; U; Android 4.4; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.4 Release/05.06.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80707[] = "Mozilla/5.0 (Linux; U; Android 4.4; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.4 Release/06.03.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80708[] = "Mozilla/5.0 (Linux; U; Android 4.4; zh-cn; sprd-N3L-U/1.0 Android/4.4 Release/04.23.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80709[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P716 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S80710[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P716 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S80711[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ORBIT 4600 Build/ORBIT-4600-V1.0-2013.06.23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80712[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ORBIT 4600 Build/ORBIT-4600-V1.0-2013.08.13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80713[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I605 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i605";
const char S80714[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I605 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i605";
const char S80715[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I605 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i605";
const char S80716[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L720 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltespr";
const char S80717[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L720 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltespr";
const char S80718[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L720 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltespr";
const char S80719[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; MID0714PGE01.133 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80720[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I8190 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S80721[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cnm TOUCHPAD 9.7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S80722[] = "Mozilla/5.0 (Linux; Android 4.0.4; Versus TouchPad 9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S80723[] = "Mozilla/5.0 (Linux; Android 4.0.4; CnM TouchPad 9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S80724[] = "Mozilla/5.0 (Linux; Android 4.0.4; Versus TouchPad 7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S80725[] = "Mozilla/5.0(Linux;U;Android2.3.3;Zh_cn;GT-I9000;480*800;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S80726[] = "JUC(Linux;U;Android2.3.3;Zh_cn;GT-I9000;480*800;)UCWEB7.8.0.95/139/444";
const char S80727[] = "LG-GT360 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S80728[] = "LG-P525/V100 Obigo/Q7.3 MMS/LG-MMS-V1.1/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S80729[] = "SAMSUNG-SGH-i900/1.0 (Windows NT 5.1; U; en) Opera 9.5";
const char S80730[] = "KDDI-HI3H UP.Browser/6.2_7.2.7.1.K.5.207 (GUI) MMP/2.0";
const char S80731[] = "HTC_P3400i-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S80732[] = "HTC_P3400-Mozilla/4.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S80733[] = "Mozilla/5.0 (Series40; NokiaC3-01.5/06.73; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.1.21.8";
const char S80734[] = "Mozilla/5.0 (Series40; NokiaC3-01/07.16; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.1.21.8";
const char S80735[] = "KDDI-SH3E UP.Browser/6.2_7.2.7.1.K.3.358 (GUI) MMP/2.0";
const char S80736[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.24 (KHTML, like Gecko) Iron/11.0.700.2 Chrome/11.0.700.2 Safari/534.24";
const char S80737[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/534.24 (KHTML, like Gecko) Iron/11.0.700.2 Chrome/11.0.700.2 Safari/534.24";
const char S80738[] = "KDDI-SN3R UP.Browser/6.2_7.2.7.1.K.6.1.103 (GUI) MMP/2.0";
const char S80739[] = "Mozilla/5.0 (X11; U; FreeBSD amd64; en-US; rv:1.8.0.8) Gecko/X Firefox/1.5.0.8";
const char S80740[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S80741[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6102 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S80742[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.4) Gecko/X Netscape/7.1 (ax; CDonDemand-Dom)";
const char S80743[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8L1 Safari/6531.22.7";
const char S80744[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8J2 Safari/6531.22.7";
const char S80745[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S80746[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S80747[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S80748[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S80749[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 900)";
const char S80750[] = "Mozilla/5.0 (Windows; N; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth Pro/6.2.2.6613 Safari/532.4";
const char S80751[] = "Mozilla/5.0 (Java; U; en-in; nokia500) UCBrowser8.2.1.144/69/444/UCWEB Mobile  3gpp-gba";
const char S80752[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; en) Opera 12.01";
const char S80753[] = "Mozilla/5.0 (Windows NT 5.1; ru; rv:2.0) Gecko/X Firefox/4.0 Opera 12.01";
const char S80754[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5670 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S80755[] = "SAMSUNG-GT-B9062_TD/1.0 Android/2.3.6 Release/09.15.2011 Browser/AppleWebKit533.1";
const char S80756[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-B9062 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S80757[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-B9062 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80758[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.19pre) Gecko/X Prism zdesktop/7.2.2";
const char S80759[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D97 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80760[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Explorer A310e Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80761[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Explorer Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80762[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Explorer A310e Build/IMM76L)";
const char S80763[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Barnes & Noble Nook Tablet Build/JZO54K)";
const char S80764[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Barnes & Noble Nook Tablet Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80765[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80766[] = "Mozilla/5.0 (Linux; Android 4.1.2; EK-GC120 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80767[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Celkon A79 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80768[] = "Mozilla/5.0 (X11; OpenBSD amd64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.68 Safari/537.17";
const char S80769[] = "Mozilla/5.0 (X11; OpenBSD i386) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.68 Safari/537.17";
const char S80770[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-us; GT-I8245 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0.X Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S80771[] = "Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I8245 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0.X Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S80772[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xperia Acro S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S80773[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26w Build/6.2.B.0.200) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S80774[] = "Mozilla/5.0 (Linux; Android 4.1.1; SC-03E Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S80775[] = "Mozilla/5.0 (Linux; Android 4.0.4; S100 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S80776[] = "sam-r631 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S80777[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.6 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/537.6 Midori/0.5";
const char S80778[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; MT27i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S80779[] = "Mozilla/5.0 (Linux; Android 4.0.3; MD_LIFETAB_P9516 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S80780[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC VLE_U Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S80781[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S80782[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One S Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S80783[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC One X Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S80784[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SH-07E Build/S5310) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80785[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SH-07E Build/S7120) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80786[] = "Mozilla/5.0 (Windows NT 6.3; Windows NT 6.3; ARM; Trident/7.0; rv:11.0) like Gecko/X Firefox/12.0";
const char S80787[] = "Mozilla/5.0 (Windows NT 6.3; ARM; Trident/7.0; rv:11.0) like Gecko/X Firefox/12.0";
const char S80788[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M440S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S80789[] = "Mozilla/5.0 (Linux; Android 4.1.2; AT100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S80790[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae5-00) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba";
const char S80791[] = "Mozilla/5.0 (Linux; Android 4.3; One S Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S80792[] = "Mozilla/5.0 (Linux; Android 4.3; SAMSUNG-SGH-I747 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.30 Mobile Safari/537.36";
const char S80793[] = "MOT-W375/0.0.64 UP.Browser/6.3.0.6.c.9 (GUI) MMP/2.0";
const char S80794[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HTC Bee Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S80795[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HTC Bee Build/ERE27) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S80796[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; HTC Bee Build/ERE27)";
const char S80797[] = "ZTE-F930(Yoigo)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S80798[] = "ZTE-F930(3UK)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S80799[] = "Palm750/v0000 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)";
const char S80800[] = "Palm750/v0003 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)";
const char S80801[] = "Palm750/v0100 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)";
const char S80802[] = "Spice M5445";
const char S80803[] = "Mozilla/5.0(Linux;U;Android2.3.3;Zh_cn;X10i;480*854;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355";
const char S80804[] = "JUC(Linux;U;Android2.3.3;Zh_cn;X10i;480*854;)UCWEB7.8.0.95/139/444";
const char S80805[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; V9C Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80806[] = "LG-A250/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S80807[] = "KDDI-SN3N UP.Browser/6.2_7.2.7.1.K.4.1.204 (GUI) MMP/2.0";
const char S80808[] = "NokiaE66/UC Browser8.0.4.121/28/351/UCWEB";
const char S80809[] = "NokiaE66/UCWEB8.0.3.99/28/800";
const char S80810[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; cm_tenderloin Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S80811[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; cm_tenderloin Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S80812[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; cm_tenderloin Build/GWK74) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S80813[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; cm_tenderloin Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S80814[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; cm_tenderloin Build/IMM76I) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S80815[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; cm_tenderloin Build/IMM76L) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7";
const char S80816[] = "iTunes/4.2 (Macintosh; U; PPC Mac OS X 10.2)";
const char S80817[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_Sensation_Z710e; 540*960) UCWEB7.9.3.103/139/800";
const char S80818[] = "JUC (Linux; U; 2.3.4; zh-cn; HTC_Sensation_Z710e; 540*960) UCWEB7.9.3.103/139/800";
const char S80819[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC_Sensation_Z710e; 540*960) UCWEB7.9.0.94/139/352";
const char S80820[] = "JUC (Linux; U; 2.3.4; zh-cn; HTC_Sensation_Z710e; 540*960) UCWEB7.9.0.94/139/352";
const char S80821[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC_Sensation_Z710e; 540*960) UCWEB7.9.0.94/139/444";
const char S80822[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_Sensation_Z710e; 540*960) UCWEB7.9.4.145/139/800";
const char S80823[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:8.0) Gecko/X Firefox/8.0 TenFourFox/7450";
const char S80824[] = "Mozilla/5.0 (X11; U; Linux i686; pl-PL; rv:1.9.0.7) Gecko/X Kubuntu/8.10 (intrepid) Firefox/3.0.9";
const char S80825[] = "Mozilla/5.0 (Linux; U; Kubuntu 8.4; de; rv:1.9) Gecko/X Firefox/3.0.1";
const char S80826[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352";
const char S80827[] = "Mozilla/5.0 (Windows NT 4.10) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S80828[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.23) Gecko/X YextBot/Java Firefox/3.6.23";
const char S80829[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) FluidApp Version/1.6.1 Safari/534.57.7";
const char S80830[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) FluidApp Version/1.6.1 Safari/534.56.5";
const char S80831[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-tw; myPadP4_V1.4) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S80832[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-tw; myPadP4_V1.6) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S80833[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; zh-tw; myPadP4 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S80834[] = "Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.0 Safari/535.19 Silk-Accelerated=false";
const char S80835[] = "JUC (Linux; U; 2.3.6; zh-cn; GT-I8150; 480*800) UCWEB7.9.0.94/139/444";
const char S80836[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CELKON CT2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80837[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Version/6.0.2 Safari/8536.26.17 Sleipnir/4.0.1";
const char S80838[] = "SonyEricssonC510/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonc510) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S80839[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IRIS_430 Build/LAVAIRIS430) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80840[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AUXUS CoreX2 3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80841[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S80842[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P1000 Build/JRO03R) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22";
const char S80843[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S80844[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000N Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S80845[] = "Mozilla/5.0 (Linux; Android 4.1.2; EK-GC100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S80846[] = "Mozilla/5.0 (Linux; Android 4.1.1; EK-GC100 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S80847[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; tr-tr; GT-P3105 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80848[] = "Mozilla/5.0 (Linux; Android 4.0.4; Haier HW-W910 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80849[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 511 like Mac OS X; en-us) AppleWebKit/534.35 (KHTML like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7055 Mobile";
const char S80850[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT918 Build/2_26C_2009) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S80851[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT918 Build/2_32D_2009) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S80852[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; W960 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80853[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KONKA W960 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80854[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; W960 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S80855[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; ELEMENT 8V3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80856[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XOLO_X1000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80857[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT15i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S80858[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTabS2110AF Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S80859[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTabS2110AH Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S80860[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S80861[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; BS471 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S80862[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-MS770 Build/JZO54K; CyanogenMod-10.0.0.l0.BobZhome.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80863[] = "Mozilla/5.0 (Linux; Android 4.2.1; N986 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S80864[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6503 Build/10.1.A.1.434) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S80865[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S80866[] = "LG-GW370/V10c Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S80867[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; TPA60W Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S80868[] = "Opera/9.10 (Windows NT 6.0; U; en)";
const char S80869[] = "sonyericssonk800i/UC Browser7.9.0.102/70/352";
const char S80870[] = "sonyericssonk800i/UC Browser7.9.0.102/69/352";
const char S80871[] = "Mozilla/5.0 (compatible; Konqueror/4.6; Linux) KHTML/4.6.5 (like Gecko) Fedora/4.6.5-8.fc14";
const char S80872[] = "Opera/9.80 (MTK; Nucleus; Opera Mobi/2167; U; en-US) Presto/2.4.18 Version/10.00";
const char S80873[] = "Opera/9.80 (Windows NT 5.1; U; Opera Mobi; de; CEB-42)";
const char S80874[] = "gt-s5233s/UC Browser8.0.3.107/69/352";
const char S80875[] = "gt-s5233s/UC Browser8.0.3.107/69/351";
const char S80876[] = "gt-s5233s/UC Browser8.0.3.107/70/352";
const char S80877[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-b7722) UCBrowser8.2.1.144/69/352/UCWEB Mobile";
const char S80878[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-b7722) UCBrowser8.2.0.132/69/352/UCWEB Mobile";
const char S80879[] = "Mozilla/5.0 (Windows NT 6.1; rv:8.0) Gecko/X Firefox/8.0 slurp";
const char S80880[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:8.0) Gecko/X Firefox/8.0 slurp";
const char S80881[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:8.0.1) Gecko/X Firefox/8.0.1 slurp";
const char S80882[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:8.0) Gecko/X Firefox/8.0 slurp slurp";
const char S80883[] = "Mozilla/5.0 (Windows NT 6.1; rv:8.0.1) Gecko/X Firefox/8.0.1 slurp";
const char S80884[] = "NokiaX2-02/2.0 (11.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-02/UC Browser7.9.0.102/69/352";
const char S80885[] = "Mozilla/5.0 (X11; U; OpenBSD i386; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/5.0.359.0 Safari/533.3";
const char S80886[] = "Mozilla/5.0 (Linux; Android 4.0.4; Kogan-Agora Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S80887[] = "Mozilla/5.0 (Linux; Android 4.0.4; KHPHN05ANDA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80888[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TM-MID720 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80889[] = "MQQBrowser/25/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HW-HUAWEI_C8812/C8812V100R001C92B913SP02; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30";
const char S80890[] = "MQQBrowser/25/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80891[] = "Mozilla/5.0 (Java; U; en-us; nokian70-1) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S80892[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC_One_V; 480*800) UCWEB7.9.0.94/139/352";
const char S80893[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC_One_V; 480*800) UCWEB7.9.0.94/139/355";
const char S80894[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC_One_V; 480*800) UCWEB7.9.0.94/139/444";
const char S80895[] = "Mozilla/5.0 (Linux; Android 4.1.1; I9377 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80896[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-i9377 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80897[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848 Duo Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80898[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848i Duo Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80899[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA6210 Navigator) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S80900[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT5 Build/GT3385) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80901[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-N7000) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S80902[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; INHON PAPILIO G1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80903[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; INHON PAPILIO G1 Build/JRO03C)";
const char S80904[] = "Mozilla/5.0 (compatible; MSIE 9.0; WinNT3.51; Trident/2.0)";
const char S80905[] = "Mozilla/5.0 (Linux; Android 4.0.4; C1605 Build/11.1.A.0.68) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S80906[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch V9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80907[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; CHUWI V9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80908[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ThL V9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80909[] = "Mozilla/5.0 (Linux; Android 4.1.1; WM8850-mid Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S80910[] = "Mozilla/5.0 (Linux; Android 4.1.1; WM8850-mid Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S80911[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; WX04K Build/324.4.0000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80912[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; WX04K Build/324.5.0000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80913[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; WX04K Build/325.3.0000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80914[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; O+ 8.52 Android Build/Oplus8.52_Android) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80915[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Micromax_A110) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S80916[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S6012 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S80917[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; i-mobile IQ 4 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80918[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7501 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S80919[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT25i Build/9.1.A.1.140) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S80920[] = "Mozilla/5.0 (X11; Linux x86_64; rv:21.0) Gecko/X Firefox/21.0 SeaMonkey/2.18";
const char S80921[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT907 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/xt907";
const char S80922[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/8J2 Safari/7534.48.3";
const char S80923[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/8L1 Safari/7534.48.3";
const char S80924[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Mobile Safari/535.19 Silk-Accelerated=true";
const char S80925[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S80926[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i2) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S80927[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 UCBrowser/9.1.0.287 Mobile";
const char S80928[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 UCBrowser/9.1.0.287 Mobile";
const char S80929[] = "Mozilla/5.0 (Linux; Android 4.2.1; C2 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S80930[] = "Mozilla/5.0 (Linux; Android 4.2.2; ADR6425LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S80931[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6312) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S80932[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One X Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S80933[] = "LG-KC550 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S80934[] = "Nokia5500d/2.0 (04.60) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S80935[] = "LG-CT810/V10x IEMobile/7.11 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S80936[] = "KDDI-SN3J UP.Browser/6.2_7.2.7.1.K.3.352 (GUI) MMP/2.0";
const char S80937[] = "SEC-SGHE490/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S80938[] = "Mozilla/5.0 (compatible; Konqueror/4.6; Linux) KHTML/4.6.5 (like Gecko) Kubuntu";
const char S80939[] = "JUC(Linux;U;Android2.3.3;Zh_cn;GT-I9001;480*800;)UCWEB7.8.0.95/139/351";
const char S80940[] = "JUC(Linux;U;Android2.3.5;Zh_cn;GT-I9001;480*800;)UCWEB7.8.0.95/139/444";
const char S80941[] = "Mozilla/5.0 (Windows NT 6.0; rv:11.0) Gecko/X Thunderbird/11.0 Lightning/1.3";
const char S80942[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; FPT F6 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80943[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; FPT F6 Build/GRJ90)";
const char S80944[] = "Opera/9.80 (Android 2.1-update1; Linux; Opera Mobi/ADR-X; U; ja) Presto/2.9.201 Version/11.50";
const char S80945[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Stainless/0.7.5 like Version/4.0 Safari/530.17";
const char S80946[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Stainless/0.7.5 like Version/4.0 Safari/530.17";
const char S80947[] = "JUC(Linux;U;Android2.3.4;Zh_cn;ST15i;320*480;)UCWEB7.8.0.95/139/444";
const char S80948[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; M.T.T. Smart Multimedia Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80949[] = "iBrowser/2.5 (E63-1;SymbianOS/9.2 Series60/3.1)";
const char S80950[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A800S Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80951[] = "Mozilla/5.0 (Linux; Android 4.1.2; NexusHD2 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80952[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC HD2 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80953[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC HD2 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S80954[] = "Nokia6300/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia6300) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S80955[] = "Mozilla/5.0 (Windows; U; ; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.4.2.X (Second Life Release; default skin) Safari/533.3";
const char S80956[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MODECOM FreeTab 1002 IPS X2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S80957[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Mobile Safari/535.19 Silk-Accelerated=true";
const char S80958[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5236) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S80959[] = "Mozilla/5.0 (Linux; Android 4.0.4; A8 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S80960[] = "Mozilla/5.0 (compatible; MSIE 4.0; Win3.11; Trident/5.0)";
const char S80961[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows 3.11)";
const char S80962[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ607-14) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S80963[] = "Miracle Linux/3.0.13 Android/4.0.4 Release/11.19.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S80964[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; Enspert_Miracle Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S80965[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia701) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S80966[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S80967[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8552 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S80968[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Explorer A310e Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S80969[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7000 Build/VeTW3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S80970[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S80971[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N7000 Build/VeTW3.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S80972[] = "Mozilla/5.0 (Linux; Android 4.1.1; Ideapad K1 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S80973[] = "Mozilla/5.0 (Linux; Android 4.1.2; K1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S80974[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Karbonn A50 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80975[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5228) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S80976[] = "Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD+ Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S80977[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S80978[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S80979[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S80980[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTL22 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S80981[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A760S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S80982[] = "Mozilla/5.0 (Linux; Android 4.1.2; Archos 80 Xenon Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S80983[] = "Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook Tablet Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S80984[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S80985[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S80986[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S80987[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S80988[] = "SonyEricssonK300i/R2BA SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S80989[] = "MicromaxX265/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S80990[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.4) Gecko/X  Firefoxlib/5.0\\0318  control/5.00318  X  ( .NET CLR 3.5.30729; .NET4.0C)";
const char S80991[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.1.16) Gecko/X AskTbFWV5/3.14.0.19709  Firefox[xSP_2:a6bb6b87c1124af321e1acd56a3bc3d0_208]  X  (.NET CLR 3.5.30729)";
const char S80992[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.25) Gecko/X  FirefoX X  (.NET CLR 3.5.30729)";
const char S80993[] = "NokiaN70-1/5.0737.3.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UCWEB7.8.0.87/27/352";
const char S80994[] = "NokiaN70/ 5.0741.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UCWEB7.8.0.87/27/352";
const char S80995[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; X10i Build/3.0.1.G.0.75; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S80996[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.5) Gecko/X Linux Mint/8 (Helena) Firefox/3.5.5";
const char S80997[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1.8) Gecko/X Linux Mint/8 (Helena) Firefox/3.5.8";
const char S80998[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1.7) Gecko/X Linux Mint/8 (Helena) Firefox/3.5.7";
const char S80999[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S81000[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S81001[] = "MQQBrowser/36 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S81002[] = "MQQBrowser/36 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3";
const char S81003[] = "MQQBrowser/36 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3";
const char S81004[] = "Mozilla/5.0 (Linux; Android 4.0.4; MK16a Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81005[] = "Mozilla/5.0 (Series40; Nokia7230/06.90; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S81006[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (DROID RAZR, 39, 1.8.2)";
const char S81007[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1";
const char S81008[] = "Mozilla/5.0 (X11; Linux x86_64; PantechP9070 Build/IMM76I) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S81009[] = "Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.2 Safari/535.19 Silk-Accelerated=false";
const char S81010[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5228) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile";
const char S81011[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cherry w500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81012[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAP4040_DUO Build/PrestigioPAP4040DUO) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81013[] = "SAMSUNG-GT-C3313T/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81014[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN78) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile";
const char S81015[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SH631W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81016[] = "Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia6303iclassic) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S81017[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3850) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S81018[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T329t Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S81019[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T329t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81020[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC T329t Build/IMM76D)";
const char S81021[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; POV_TAB-PROTAB30-IPS9 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81022[] = "Mozilla/5.0 (iPod; CPU iPhone OS 511 like Mac OS X) AppleWebKit/534.46 (KHTML like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.1.1m";
const char S81023[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; CTP989RX Build/CTP989RX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81024[] = "Mozilla/5.0 (Linux; Android 4.0.4; SH-02E Build/S2130) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S81025[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I605 Build/JDQ39; Carbon-CARBON-JB-v1.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81026[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I605 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81027[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; LG-P705 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S81028[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-S6500D Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S81029[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-S6500 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S81030[] = "Mozilla/5.0 (Linux; Android 4.1.2; B15 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81031[] = "Mozilla/5.0 (compatible; MSIE 10.0; WinNT3.51; Trident/3.0)";
const char S81032[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HTC Desire S Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S81033[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; HTC Desire S Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S81034[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S81035[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9505 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S81036[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; ALCATEL ONE TOUCH 8008X Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2 Mobile Safari/534.30";
const char S81037[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; he-il; orange infinity 8008X Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2 Mobile Safari/534.30";
const char S81038[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Cyclone Voyager 2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81039[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-uk; cyclone voyager Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81040[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Cyclone Voyager Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81041[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8260 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S81042[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8260 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S81043[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A35) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S81044[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac5-00.2) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba";
const char S81045[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac5-00) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba";
const char S81046[] = "Mozilla/5.0 (Linux; Android 4.2.2; IdeaTab A3000-H Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S81047[] = "Mozilla/5.0 (Linux; Android 4.2.2; IdeaTab A3000-F Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S81048[] = "Mozilla/5.0 (Linux; Android 4.1.2; N9810 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S81049[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6210 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S81050[] = "LG-A230/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S81051[] = "TXT8040/T05_0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S81052[] = "AppleCoreMedia/1.0.0.9A334 (iPod; U; CPU OS 5_0 like Mac OS X; en_us)";
const char S81053[] = "AppleCoreMedia/1.0.0.9A405 (iPod; U; CPU OS 5_0_1 like Mac OS X; en_us)";
const char S81054[] = "NokiaC7-00";
const char S81055[] = "Mozilla/5.0 (X11; CrOS i686 15.1011.43) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.51 Safari/535.2";
const char S81056[] = "Mozilla/5.0 (X11; CrOS i686 15.1011.62) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.83 Safari/535.2";
const char S81057[] = "Mozilla/5.0 (X11; CrOS i686 15.1011.107) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.116 Safari/535.2";
const char S81058[] = "Mozilla/5.0 (X11; CrOS i686 15.1011.118) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.117 Safari/535.2";
const char S81059[] = "Mozilla/5.0 (X11; CrOS i686 15.1011.137) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S81060[] = "Mozilla/5.0 (X11; CrOS i686 15.950.0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.862.1 Safari/535.2";
const char S81061[] = "Mozilla/5.0 (X11; CrOS i686 15.916.0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.855.0 Safari/535.2";
const char S81062[] = "Mozilla/5.0 (X11; CrOS i686 15.997.2011) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.872.0 Safari/535.2";
const char S81063[] = "Mozilla/5.0 (Linux; U; Android 2.2; md-us; Spice Mi-310 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81064[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID Pro Build/VZW; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81065[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID Pro Build/GRKLVL; CM7 by vanillaLVL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81066[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SH8188U Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81067[] = "Mozilla/5.0 (Linux; Android 4.0.4; myTouch_4G_Slide Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81068[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC MyTouch 4G Slide Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81069[] = "Mozilla/5.0 (Linux; Android 4.0.3; A511 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S81070[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S81071[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Iron/6.0.475.1 Chrome/6.0.475.1 Safari/534";
const char S81072[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S81073[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S81074[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S81075[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; GT-P1000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/352";
const char S81076[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S81077[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-P1000N Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S81078[] = "Mozilla/5.0 (Linux; Android 4.0.4; L-01E Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81079[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAC-97032 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S81080[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA6700s) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S81081[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; N12 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81082[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; LG-P705 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S81083[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E10i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S81084[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Droid Incredible Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81085[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Droid Incredible Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81086[] = "Mozilla/5.0 (Linux; Android 4.0.4; SH-02E Build/S2130) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S81087[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Neo Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81088[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I927 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81089[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I927 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81090[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I927 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81091[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7000 Build/JDQ39; Carbon-Carbon-JB-v1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81092[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-N7000 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81093[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; MITO t510 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81094[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC Wildfire S A510e Build/kantry123 v2.0; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81095[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Wildfire S A510e Build/ASNX; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81096[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Marvelc Build/ASNX; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81097[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Micromax_A116) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S81098[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT912 Build/JDQ39; LiquidSmooth-Liquid-JB-v2.2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81099[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT912 Build/JDQ39E; LiquidSmooth-Liquid-JB-v2.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81100[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT100 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S81101[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/VEGAVIETTEAM_HOANGMINHHUU; CyanogenMod-JB_LR_VGV_V4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81102[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/JZO54K; CyanogenMod-JB_LR_VGV_V3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81103[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/IMM76L; CyanogenMod-CM9_OPTIMUS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81104[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/Hoang Nguyen; CyanogenMod-EF84L_ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81105[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I437 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81106[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Symphony Xplorer W35 Build/UNKNOWN) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81107[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A89 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S81108[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; V812 Core4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81109[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-N7000 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S81110[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT926 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S81111[] = "Mozilla/5.0 (Linux; Android 4.2.1; Q700 Build/XOLOQ700) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S81112[] = "Mozilla/5.0 (Linux; Android 4.1.2; Amazon Kindle Fire Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S81113[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; L36h Build/10.1.1.A.1.253) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S81114[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; ZTE U956 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S81115[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI D2-2010 Build/HuaweiD2-2010) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S81116[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S81117[] = "UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-S6102) U2/1.0.0 UCBrowser/9.3.0.321 Mobile";
const char S81118[] = "SAMSUNG-SGH-J150/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S81119[] = "SAMSUNG-SGH-J150B/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S81120[] = "SonyEricssonW710i/R1JC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81121[] = "SAMSUNG-GT-S3600i/S3600iJPIL1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S81122[] = "NokiaE72-1/UCWEB7.9.0.94/28/999";
const char S81123[] = "NokiaE72-1/UC Browser7.9.0.102/28/355/UCWEB";
const char S81124[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Mobile/5F136";
const char S81125[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko)";
const char S81126[] = "Mozilla/5.0 (Windows; U; Windows NT 7.1; ru; rv:1.9.3.15) Gecko/X Firefox/7.0";
const char S81127[] = "Mozilla/5.0 (Windows NT) Firefox/7.0.1";
const char S81128[] = "Mozilla/5.0 (Java; U; en-us; gt-s5233a) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S81129[] = "gt-s5233a/UCWEB8.2.0.116/69/352";
const char S81130[] = "Mozilla/5.0 (Java; U; en-us; nokian70-1) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S81131[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE66)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S81132[] = "NokiaE66/UCWEB8.5.0.163/28/800";
const char S81133[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:12.3) Gecko/X Firefox/12.3 PaleMoon/12.3";
const char S81134[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:12.3) Gecko/X Firefox/12.3-x64 PaleMoon/12.3-x64";
const char S81135[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en-US; rv:1.9.2.16pre) Gecko/X Prism zdesktop/7.2.2";
const char S81136[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; Allview2SpeedDuo Build/SV1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81137[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; Dslide 702 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81138[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Dslide 702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81139[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; KM-S300 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81140[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-us; Spice Mi-424 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81141[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Micromax A84 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81142[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-ph; Cloudfone_Excite320g Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81143[] = "Mozilla/5.0 (Linux; Android 4.1.2; SH837W Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81144[] = "Mozilla/5.0 (Linux; Android 4.0.4; N9100 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S81145[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P6800 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Safari/537.31";
const char S81146[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6800 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S81147[] = "ZTEU807_TD/1.0 Linux/3.0.13 Android/4.0 Release/10.15.2012 Browser/AppleWebKit534.30";
const char S81148[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; Micromax A110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S81149[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax_A110 Build/JRO03C) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S81150[] = "Mozilla/5.0 (Linux; Android 4.1.2; S500 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S81151[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; C6602 Build/10.1.A.1.434) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S81152[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; C6602 Build/10.1.1.A.1.253) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S81153[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; eZee'Tab709 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81154[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT29i Build/7.0.A.3.212) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81155[] = "Spice M5364";
const char S81156[] = "LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-kp500 teleca) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S81157[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC One V Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S81158[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; sr-rs; ZTE Blade G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81159[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Micromax A72 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S81160[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One X Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S81161[] = "Mozilla/5.0 (Linux; Android 4.1.2; Titanium S1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S81162[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S81163[] = "sam-r810 NetFront/3.5 Mozilla/5.0 like Gecko/20060426";
const char S81164[] = "SAMSUNG-GT-S5628/ SHP/VPP/R5 Dolfin/1.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S81165[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-za; RBM_HD Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81166[] = "SAMSUNG-GT-S3778V/1.0 RTK-E/1.0 DF3G/1.0 Release/04.04.2011 Browser/NetFront4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81167[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Desire HD Build/GRI40)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81168[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S81169[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S81170[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Desire HD Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S81171[] = "AT-AS28D Linux/2.6.35 Android/2.3.5 Release/04.20.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1";
const char S81172[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; nl-nl; AT-AS28D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81173[] = "MQQBrowser/35 (iOS 4S; U; CPU like Mac OS X; zh-cn)";
const char S81174[] = "MQQBrowser/35 (iOS 4; U; CPU like Mac OS X; zh-cn)";
const char S81175[] = "MQQBrowser/35 (iOS 3GS; U; CPU like Mac OS X; zh-cn)";
const char S81176[] = "Mozilla/5.0 (Linux; Android 4.1.1; MB525 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81177[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MOT-XT800+/TTUPG_M7_2.45.0; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81178[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-XT800+/TTUPG_M7_2.52.0; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81179[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT800+ Build/TTUPG_M7_2.55.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S81180[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-XT800+/TTUPG_M7_2.55.0; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81181[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT800+ Build/TTUPG_M7_2.52.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81182[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.1.1.0 Chrome/16.0.912.63 Safari/535.7";
const char S81183[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.1.1.0 Chrome/16.0.912.63 Safari/535.7";
const char S81184[] = "Micromax X291/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81185[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-7230 Build/M7513) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81186[] = "Mozilla/5.0 (Linux; Android 4.0.4; ISW11SC Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81187[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A828 TV Duo Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81188[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.0 Safari/534.34";
const char S81189[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7";
const char S81190[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7";
const char S81191[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7";
const char S81192[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7";
const char S81193[] = "Mozilla/5.0 (PlayStation Vita 2.00) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S81194[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Desire C Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S81195[] = "LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-kp500 teleca) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S81196[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC One X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S81197[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; WT19i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S81198[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S81199[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; GT-I9100 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S81200[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S81201[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9100G Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S81202[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E220S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81203[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100P Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S81204[] = "MTk-GT_I9100/1.0 Linux/2.6.35.7 Android/4.1.1 Release/10.12.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S81205[] = "SAMSUNG-GT-I9108_TD/1.0 Android/4.1.2 Release/12.6.2012 Browser/AppleWebKit534.30";
const char S81206[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8190) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S81207[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT22i Build/6.2.A.0.400) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S81208[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT22i Build/6.2.A.1.100) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S81209[] = "Mozilla/5.0 (Linux; Android 4.0.4; CINK PEAX Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81210[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-P880 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81211[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS930 4G Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81212[] = "Mozilla/5.0 (Linux; Android 4.1.2; lenovo A820 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81213[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; Micromax A116 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300";
const char S81214[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9152 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81215[] = "Mozilla/5.0 (Linux; Android 4.1.2; SM-T210 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S81216[] = "Mozilla/5.0 (Linux; Android 4.1.2; SM-T210R Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S81217[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-P3100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S81218[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-P3100 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S81219[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I8530 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81220[] = "Mozilla/5.0 (Linux; Android 4.1.1; NS-14T004 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S81221[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233s) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S81222[] = "UCWEB/2.0 (Java; U; MIDP-2.0; vi; gt-s5233s) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S81223[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire V Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S81224[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT910 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S81225[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F260S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S81226[] = "Mozilla/5.0 (Linux; Android 4.1.1; MZ601 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S81227[] = "Mozilla/5.0 (Linux; Android 4.1.2; MZ601 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S81228[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S81229[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-S735C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S81230[] = "MOT-VE465/00.72 UP.Browser/7.2.7.5.616 (GUI) MMP/2.0";
const char S81231[] = "MOT-VE465/00.72 UP.Browser/7.2.7.5.659 (GUI) MMP/2.0";
const char S81232[] = "MOT-VE465/00.72 UP.Browser/7.2.7.5.541 (GUI) MMP/2.0";
const char S81233[] = "MOT-VE465/1.0 UP.Browser/7.2.7.5.593 (GUI) MMP/2.0";
const char S81234[] = "MOT-VE465/00.72 UP.Browser/7.2.7.5.643 (GUI) MMP/2.0";
const char S81235[] = "MOT-VE465-parrot/1.0 UP.Browser/7.2.7.5.593 (GUI) MMP/2.0";
const char S81236[] = "MOT-VE465/00.72 UP.Browser/7.2.7.5.599 (GUI) MMP/2.0";
const char S81237[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; 1000ET Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S81238[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME722 Build/4.5.3-109_MS2-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81239[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME722 Build/4.5.3-77) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81240[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME722 Build/4.5.3-109_MS2-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81241[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME722 Build/4.5.3-109_MS2-3) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S81242[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME722 Build/4.5.3-109_MS2-5) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S81243[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME722 Build/4.5.3-109_MS2-5) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S81244[] = "MicromaxX600/MIDP2.0/CLDC1.1/Screen-240X400";
const char S81245[] = "POLARIS/6.1 (BREW 3.1.5; U; en-us; LG; POLARIS/6.1/WAP) Sprint LX101 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S81246[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; SCH-i579 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81247[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; SCH-i579 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S81248[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i579 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81249[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; ATP7483 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81250[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; Excellent8 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81251[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9B206 Safari/6533.18.5";
const char S81252[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9B176 Safari/6533.18.5";
const char S81253[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S81254[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9B206 Safari/6533.18.5";
const char S81255[] = "Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S81256[] = "Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S81257[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T869 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S81258[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MT25i Build/4.1.B.0.631) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S81259[] = "Nokia110/2.0 (03.04) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia110) UCBrowser8.4.0.159/70/444/UCWEB Mobile";
const char S81260[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81261[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE-N788 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81262[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE N788 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S81263[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355";
const char S81264[] = "Kindle";
const char S81265[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81266[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81267[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81268[] = "Mozilla/5.0 (Linux; Android 4.0.3; AN10CG3 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S81269[] = "Instagram 3.2.0 (iPod4,1; iPhone OS 6.0.1; en_US) AppleWebKit/420+";
const char S81270[] = "Instagram 3.2.0 (iPod4,1; iPhone OS 6.0; en_US) AppleWebKit/420+";
const char S81271[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P3100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S81272[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-P3100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S81273[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; ELF-II Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81274[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ELF-II Build/ICS.ELF-II.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81275[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ELF-II Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81276[] = "NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac2-02) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S81277[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; pt-pt; AEG_AX410 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81278[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; Nokia6220c-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S81279[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; Micromax A87 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S81280[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A87 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S81281[] = "OneBrowser/3.1 (NokiaC2-03/07.63)";
const char S81282[] = "Mozilla/5.0 (Linux; Android 4.1.1; Ideapad K1 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S81283[] = "Mozilla/5.0 (Linux; Android 4.1.2; K1 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S81284[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Intex_Aqua_Marvel) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S81285[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Intex_Aqua_Marvel+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S81286[] = "UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 610) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile";
const char S81287[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A65) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S81288[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAB7250 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81289[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81290[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S81291[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570I) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S81292[] = "Mozilla/5.0 (Linux; Android 4.2.1; ThL W8 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81293[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E610 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S81294[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S81295[] = "Micromax X322/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81296[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT26i Build/6.1.A.2.55) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S81297[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LT26i Build/6.1.A.2.45) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S81298[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LT26i Build/6.1.A.0.452) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S81299[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT912 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/spyder";
const char S81300[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Spice Mi-515 Build/4.1.003.X.7295+; 540*960; CUCC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81301[] = "NokiaX3-00/5.0 (11.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax3-00) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S81302[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S81303[] = "Mozilla/5.0 (Linux; Android 4.2.1; W100 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S81304[] = "Mozilla/5.0 (Linux; Android 4.1.1; ONE TOUCH TAB 7 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S81305[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5670) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S81306[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Flytouch9 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81307[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Flytouch 9 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81308[] = "BlackBerry8830m/4.5.0.135 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/104";
const char S81309[] = "Alcatel-OT-708/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S81310[] = "Alcatel-OT-708A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S81311[] = "Alcatel-OT-708/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C, Alcatel-OT-708/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S81312[] = "Alcatel-OT-708/1.0 model-orange Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S81313[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_S200";
const char S81314[] = "LG-A155/V100 Obigo/Q05A MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S81315[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.127 Large Screen Safari/533.4 GoogleTV/b54202";
const char S81316[] = "Mozilla/5.0 (Windows NT 4.10) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S81317[] = "Nokia6303iclassic/5.0 (09.83) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303iclassic/UC Browser8.0.3.107/69/352";
const char S81318[] = "NokiaC7-00/UCWEB8.0.3.99/51/999";
const char S81319[] = "NokiaC7-00/UC Browser8.0.4.121/50/444/UCWEB";
const char S81320[] = "SAMSUNG-GT-C3332/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81321[] = "Mozilla/5.0 (Java; U; en-in; mauie800) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S81322[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.3.0.X (Second Life Release; default skin) Safari/533.3";
const char S81323[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MOT-ME811/SHDWR_X7_2.67.0; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81324[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-ME811/SHDWR_X7_2.71.0; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81325[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-ME811/SHDWR_X7_2.74.0; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81326[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TAB467 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81327[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAB467 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81328[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-b5310) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S81329[] = "Mozilla/5.0 (Linux; Android 4.0.3; M032 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81330[] = "JUC(Linux;U;Android4.0.4;Zh_cn;GT-I9300;720*1280;)UCWEB7.8.0.95/139/444";
const char S81331[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOMO11 bird Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81332[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MOMO11 bird Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81333[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOMO11bird3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81334[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I8150 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81335[] = "OneBrowser/3.0 (NokiaX2-01/08.63)";
const char S81336[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; miTab AIR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81337[] = "NOKIA-Lumia 710/UCWEB2.6.0.213/47/32079";
const char S81338[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; Tablet 723B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81339[] = "JUC (Linux; U; 2.3.7; zh-cn; ST27i; 320*480) UCWEB7.9.0.94/139/444";
const char S81340[] = "Mozilla/5.0 (Linux; Android 4.0.4; D2-711 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81341[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cynus T2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81342[] = "Opera/9.80 (Windows 95) Presto/2.12.388 Version/12.12";
const char S81343[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; myTouch 4g Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81344[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; myTouch 4g Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81345[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; myTouch 4G Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81346[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; PC906 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81347[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT886 Build/6.7.3-179_YIC-10) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S81348[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-nz; GT-P5110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S81349[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);";
const char S81350[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S81351[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; GT-P5110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);";
const char S81352[] = "Mozilla/5.0 (Linux; Android 4.1.2; A500 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S81353[] = "Mozilla/5.0 (Linux; Android 4.1.1; Lenovo P770 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S81354[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8681 Build/HuaweiU8681) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S81355[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One XL Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S81356[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One XL Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S81357[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Starmobile Hit Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81358[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A89) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S81359[] = "Mozilla/5.0 (Linux; Android 4.1.1; Lenovo S890 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81360[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-M830 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S81361[] = "Mozilla/5.0 (Linux; Android 4.0.4; MediaPad 10 FHD Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S81362[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID4 Build/6.7.2-180_DR4-16_M2-37) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81363[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ADR6350 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81364[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ADR6350 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81365[] = "GT-I9505 Linux/3.0.13 Android/4.2.2 Release/05.22.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S81366[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; Micromax A27 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S81367[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A27 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S81368[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT26i Build/6.2.B.0.200) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S81369[] = "Mozilla/5.0 (MSIE 9.0; Windows NT 6.3; WOW64; Trident/7.0; Touch; rv:11.0) like Gecko";
const char S81370[] = "Mozilla/5.0 (MSIE 9.0; Windows NT 6.3; Win64; x64; Trident/7.0; Touch; rv:11.0) like Gecko";
const char S81371[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Aquaris Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S81372[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia700) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S81373[] = "Mozilla/5.0 (Android; Mobile: SCH-I535; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S81374[] = "Mozilla/5.0 (Android; Mobile: SGH-T999; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S81375[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT18i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S81376[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S81377[] = "Mozilla/5.0 (Linux; Android 4.0.3; KFOT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S81378[] = "SonyEricssonW880iv/R1JC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81379[] = "LG-KF310-Orange/V10b Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81380[] = "LG-KF310 Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81381[] = "Mozilla/4.0 (Vodafone/1.0/LG-KF310/V10b Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1)";
const char S81382[] = "Mozilla/4.0 (Vodafone/1.0/LG-KF310/V10c Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1)";
const char S81383[] = "HTC_Touch_HD_T8282 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S81384[] = "Bing/2.2 CFNetwork/459 Darwin/10.0.0d3";
const char S81385[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Ubuntu/10.10 Chromium/15.0.874.106 Chrome/15.0.874.106 Safari/535.2";
const char S81386[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.2 (KHTML, like Gecko) Ubuntu/10.10 Chromium/15.0.874.106 Chrome/15.0.874.106 Safari/535.2";
const char S81387[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; HTC HD7 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81388[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; HTC HD7 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81389[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; HTC HD7 Pro/+ Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81390[] = "NokiaN97 mini/UCWEB8.0.3.99/50/800";
const char S81391[] = "NokiaN97 mini/UC Browser8.0.4.121/50/444/UCWEB";
const char S81392[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ME632 Build/5.5.1Q-131_HKT-8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81393[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ME632 Build/5.5.1Q-131_HKT-16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81394[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ME632 Build/5.5.2Q-CHN-111) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81395[] = "JUC(Linux;U;Android2.3.5;Zh_cn;TECNO T1;320*480;)UCWEB7.8.0.95/139/32929";
const char S81396[] = "Mozilla/5.0 (Linux; Android 4.0.4; WT19a Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81397[] = "Mozilla/5.0 (Linux; Android 4.0.4; LiveWithWalkman Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81398[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S680 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81399[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S680 Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S81400[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S680 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S81401[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MID1125 Build/CaribbeanSunset_v1.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81402[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81403[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81404[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo A300/S100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81405[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A300 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81406[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A300 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S81407[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81408[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5236) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S81409[] = "NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-05) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S81410[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A200 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81411[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; E-Boda Essential A200 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81412[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; eZee'Tab1001 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81413[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; Tablet726H Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81414[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xperia S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S81415[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I9003) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S81416[] = "HUAWEI_G6800 (MRE\\2.3.00(25600) resolution\\X chipset\\MT6255 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\qwerty) MAUI/MAUI.11B.W12.12.MP.V2 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; sr-SR) Presto/2.5.28 Version/10.10";
const char S81417[] = "HUAWEI_G6800 (MRE\\2.3.00(25600) resolution\\X chipset\\MT6255 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\qwerty) MAUI/MAUI.11B.W12.12.MP.V2 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; hr-HR) Presto/2.5.28 Version/10.10";
const char S81418[] = "HUAWEI_G6800 (MRE\\2.3.00(25600) resolution\\X chipset\\MT6255 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\qwerty) MAUI/MAUI.11B.W12.12.MP.V2 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; ro-RO) Presto/2.5.28 Version/10.10";
const char S81419[] = "HUAWEI_G6800 (MRE\\2.3.00(25600) resolution\\X chipset\\MT6255 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\qwerty) MAUI/MAUI.11B.W12.12.MP.V2 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; es-ES) Presto/2.5.28 Version/10.10";
const char S81420[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I897 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S81421[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-I897 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S81422[] = "Mozilla/5.0 (Linux; Android 4.1.1; A701 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S81423[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A1+) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S81424[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST27i Build/6.2.A.0.400) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S81425[] = "Nokia306/2.0 (05.93) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia306) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S81426[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.59.8 (KHTML, like Gecko) Version/5.1 Safari/6534.59.8 Sleipnir/4.3.1";
const char S81427[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-be; Lenco TAB-1030 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81428[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A760S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S81429[] = "Mozilla/5.0 (Linux; Android 4.2.2; Xperia S Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S81430[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTL22 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S81431[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Edison Build/1.1.11-1015 X-10:30) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S81432[] = "MQQBrowser/44 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S81433[] = "MQQBrowser/44 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3";
const char S81434[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5302) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S81435[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E455f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S81436[] = "Mozilla/5.0 (Linux; Android 4.1.1; M97 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S81437[] = "Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia305) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S81438[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S81439[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGP321 Build/10.3.1.A.0.244) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S81440[] = "SAMSUNG-SGH-L810-Vodafone/L810BUHG3 SHP/VPP/R5 Opera/9.5 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S81441[] = "HTC-P3470-orange/PPC; 240x320; OpVer 29.115.2.732 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S81442[] = "HTC-P3470-orange/PPC; 240x320; OpVer 29.113.2.731 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S81443[] = "HTC_P3470 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S81444[] = "SAMSUNG-SGH-C276L/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S81445[] = "SAMSUNG-SGH-C276/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S81446[] = "Mozilla/5.0 (compatible; Teleca Q7; Brew 4.0.3; U; en) 240X400 LGE LG840";
const char S81447[] = "LG-S365/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S81448[] = "Mozilla/5.0 (Series40; NokiaX2-00/04.80; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.6";
const char S81449[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.1 Chrome/15.0.900.2 Safari/535.2";
const char S81450[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; DM011SH Build/S0006) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81451[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; DM011SH Build/S0018) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81452[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; DM011SH Build/S0010) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81453[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; DM011SH Build/S0020) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81454[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; DM011SH Build/S0022) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81455[] = "nokia5233/UC Browser7.9.0.102/69/352  3gpp-gba";
const char S81456[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; eGlide Prism Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81457[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; eGlide Prism Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81458[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5250)/UC Browser8.2.0.132/50/355/UCWEB Mobile";
const char S81459[] = "Nokia5250/UCWEB8.2.0.116/50/800";
const char S81460[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:13.0) Gecko/X Firefox/13.0 slurp";
const char S81461[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:13.0) Gecko/X Firefox/13.0.1 slurp slurp";
const char S81462[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:13.0) Gecko/X Firefox/13.0.1 slurp";
const char S81463[] = "Mozilla/5.0 (Windows NT 6.1; rv:13.0) Gecko/X Firefox/13.0.1 slurp";
const char S81464[] = "Mozilla/5.0 (Windows NT 6.1; rv:13.0) Gecko/X Firefox/13.0 slurp";
const char S81465[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MOMO8 bird Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81466[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOMO8 bird Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81467[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.12) Gecko/X Firefox/3.6.12\",\"Mozilla/5.0 (Windows; U; Windows NT 5.1; pl-PL; rv:1.8.1.24pre) Gecko/X K-Meleon/1.5.4\",\"Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/540.0 (KHTML,like Gecko) Chrome/9.1.0.0 Safari/540.0\",\"Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Comodo_Dragon/4.1.1.11 Chrome/4.1.249.1042 Safari/532.5\",\"Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US; rv:1.9.0.";
const char S81468[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Comodo_Dragon/4.1.1.11 Chrome/4.1.249.1042 Safari/532.5";
const char S81469[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P920 Build/GRJ90; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81470[] = "Instagram 3.0.0 (iPhone; iPhone OS 5.1.1; en_US) AppleWebKit/420+";
const char S81471[] = "Instagram 3.0.0 (iPhone; iPhone OS 5.1; en_US) AppleWebKit/420+";
const char S81472[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7310 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81473[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7310 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81474[] = "AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; GT-P7310 Build/JRO03H)";
const char S81475[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7310 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30";
const char S81476[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7310 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S81477[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7310 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81478[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P7310 Build/JZO54K)";
const char S81479[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A770K Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81480[] = "Opera/9.80 (Android 2.2; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S81481[] = "Opera/9.80 (Android 2.2.2; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S81482[] = "Opera/9.80 (Android 2.2.1; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S81483[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2a1pre) Gecko/X Minefield/3.2a1pre";
const char S81484[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; el-gr; novaPad 100 C124 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81485[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Teclast P85??(A9D3) Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81486[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; P85(R8A1) Build/IMM76D; 1024*720) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S81487[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Teclast P85??(A9D3) Build/IMM76D)";
const char S81488[] = "server-bag [iPhone OS,6.0.1,10A523,iPod4,1]";
const char S81489[] = "iPod4,1/6.0.1 (10A523)";
const char S81490[] = "iPod4,1/6.0 (10A403)";
const char S81491[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; POV_TAB-PROTAB25 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81492[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; S18 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81493[] = "Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI MT1-U06 Build/HuaweiMT1-U06) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S81494[] = "Mozilla/5.0 (Linux; Android 4.2.1; Transformer Prime TF201 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S81495[] = "Mozilla/5.0 (Linux; Android 4.1.1; Ideapad K1 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S81496[] = "Mozilla/5.0 (Linux; Android 4.1.2; K1 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S81497[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; Q-Smart S16 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81498[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9070 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S81499[] = "Mozilla/5.0 (Linux; Android 4.1.2; Q800 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81500[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9080L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81501[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717R Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81502[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S81503[] = "Mozilla/5.0 (compatible; MSIE 4.0; WinNT3.51; Trident/3.0)";
const char S81504[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; O+ 8.15 Android Build/Oplus8.15_Android) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81505[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; S5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81506[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; S5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81507[] = "Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; K-Touch S5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81508[] = "Mozilla/5.0 (Linux; U; Android 4.1; zh-tw; iNO S5 Build/1.0.492_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81509[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; A19 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81510[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I869 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81511[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I869 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S81512[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S81513[] = "Mozilla/5.0 (Linux; Android 4.2.1; H120 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S81514[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LT18i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S81515[] = "SAMSUNG-GT-E2200 Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.32208/30.3697; U; en) Presto/2.8.119 Version/11.10";
const char S81516[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cynus T2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S81517[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE66-2/102.07.81; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S81518[] = "Mozilla/4.0(compatible; Polaris 6.2; Brew 3.1.5; U; en) 240X400 LGE LG-AX8575";
const char S81519[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; in-id; ZTE-U V852 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81520[] = "LG-P520/V100 Obigo/Q7.3 MMS/LG-MMS-V1.1/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S81521[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SHW-M130L Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S81522[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SHW-M130L Build/ECLAIR)";
const char S81523[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; ViewPad 10S Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81524[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-es; MID7105SR Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81525[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; MID7105SR Build/FRF85B)";
const char S81526[] = "Mozilla/5.0 (X11; U; FreeBSD i386; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/4.0.207.0 Safari/532.0";
const char S81527[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; CELKON_A_90 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81528[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P970 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81529[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Andi_4.3a Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81530[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s5620) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S81531[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81532[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81533[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81534[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81535[] = "SpiceM5365/MTK Release/01.01.2012 Browser/wap2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81536[] = "Mozilla/5.0 (Linux; Android 4.0.4; IRULU-AL101 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S81537[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S81538[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; STARNAUTE II Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81539[] = "Opera/9.80 (X11; Linux x86_64; Edition Linux Mint) Presto/2.12.388 Version/12.13";
const char S81540[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTX21 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81541[] = "stagefright/1.2 (Linux;Android ; Release/4.1.2, Manufacturer/motorola, Model/DROID RAZR, Build/9.8.2O-72_VZW-16)";
const char S81542[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I897 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S81543[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-I897 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S81544[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MPDC99 BT IPS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81545[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M040 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31";
const char S81546[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; N320) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S81547[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A45) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S81548[] = "Mozilla/5.0 (Linux; Android 4.1.2; F-02E Build/V17R48A) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81549[] = "Mozilla/5.0 (Linux; Android 4.1.1; A211 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S81550[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE Blade III Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S81551[] = "Mozilla/5.0 (Linux; Android 4.0.4; BLADE III Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S81552[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT25i Build/9.0.D.0.484) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S81553[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Lenovo A590 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S81554[] = "Mozilla/5.0 (Windows NT 6.0; rv:21.0) Gecko/X Firefox/21.0 SeaMonkey/2.18";
const char S81555[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; X10i Build/3.0.1.G.0.75) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S81556[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; A54 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S81557[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaE7-00) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S81558[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S81559[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100G Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S81560[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XOLO_A800 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81561[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Ktouch_A11 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81562[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST18i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S81563[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Tablet Rebel 8GB Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81564[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Skyfire2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81565[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT910 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81566[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; XT910 Build/JDQ39E)";
const char S81567[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-US780 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81568[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GIONEE_Gpad_G2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81569[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; M-MP876S2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81570[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PLT7223G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81571[] = "NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-05) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S81572[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26w Build/6.2.B.0.200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S81573[] = "Mozilla/5.0 (Linux; Android 4.1.2; MZ601 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S81574[] = "Mozilla/5.0 (Linux; Android 4.1.1; MZ601 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S81575[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A1+) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S81576[] = "Mozilla/5.0 (Linux; Android 4.1.2; bq Elcano Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S81577[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI H868C Build/HuaweiH868C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S81578[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XCD35 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S81579[] = "PANTECH-C530/ITUS10062008 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81580[] = "PANTECH-C530/R01 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81581[] = "PANTECH-C530/ITUSX Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1 Novarra-Vision/8.0";
const char S81582[] = "HPiPAQ910/1.0/Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) (PPC; 320x240)";
const char S81583[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) HPiPAQ910/1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 320x240)";
const char S81584[] = "LGE-GW370B Obigo/Q7.3 MMP/2.0";
const char S81585[] = "MOT-WX404/00.72 UP.Browser/7.2.7.5.705 (GUI) MMP/2.0";
const char S81586[] = "Mozilla/5.0 (compatible; Teleca Q7; Brew 3.1.5; U; en) 240X400 LGE LG700";
const char S81587[] = "Opera/9.5 (Microsoft Windows; Windows CE; Opera Mobi/9.5; U; en) VZW:SCH-i770 PPC 320x320";
const char S81588[] = "SonyEricssonX1i/R3AA Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S81589[] = "X1i Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S81590[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1 QQBrowser/6.10.11509.201";
const char S81591[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C) QQBrowser/6.10.11509.201";
const char S81592[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.10.11509.201";
const char S81593[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E) QQBrowser/6.10.11509.201";
const char S81594[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0) QQBrowser/6.10.11509.201";
const char S81595[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0) QQBrowser/6.10.11509.201";
const char S81596[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; MDDC) QQBrowser/6.10.11509.201";
const char S81597[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.10.11528.201";
const char S81598[] = "Mozilla/5.0 (Series40; Nokia6500s-1/10.00; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S81599[] = "nokiac7-00/UC Browser8.0.3.107/69/444  3gpp-gba";
const char S81600[] = "Mozilla/5.0(Linux;U;Android 2.1-update1;pandigital6gs)AppleWebKit/530.17(KHTML,like Gecko)Version/4.0 Mobile Safari/530.17";
const char S81601[] = "Mozilla/5.0 (Java; U; en-in; nokian73-1) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S81602[] = "LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-in; lg-gs290) UCBrowser8.3.0.154/69/405/UCWEB Mobile";
const char S81603[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S81604[] = "Videocon_V1545_Maui Wap Browser";
const char S81605[] = "Videocon V1511_Maui Wap Browser";
const char S81606[] = "Videocon_V1541_Maui Wap Browser";
const char S81607[] = "Videocon_V1521_Maui Wap Browser";
const char S81608[] = "Mozilla/5.0 (Windows NT 5.0; rv:11.0) Gecko/X Firefox/11.0 CometBird/11.0";
const char S81609[] = "CoolPad8022_CMCC_TD/1.0 Linux/2.6.35 Android/2.3 Release/5.10.2012 Browser/AppleWebkit533.1";
const char S81610[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:15.0) Gecko/X Thunderbird/15.0.1";
const char S81611[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KATBL10A08DA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81612[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Glacier Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81613[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Glacier Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81614[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; GT-B7510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S81615[] = "Opera/9.80 (Windows 95) Presto/2.12.388 Version/12.11";
const char S81616[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; M-MP1010i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81617[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en; rv:1.9.0.19) Gecko/X Camino/2.0.9 (like Firefox/3.0.19)";
const char S81618[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT26i Build/6.1.A.2.45) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S81619[] = "Mozilla/5.0 (Linux; Android 4.0.4; Philips W336 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81620[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TOUCH97 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81621[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MZ601 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81622[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S81623[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S81624[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81625[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81626[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81627[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA5630d-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S81628[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A15) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S81629[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A15 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S81630[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-S6012 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S81631[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S6012 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S81632[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; S5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81633[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5770D Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S81634[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; O+_8.12_Android Build/OplusOplus_A75) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81635[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Videocon A27 Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S81636[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; PMP7280C3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81637[] = "Mozilla/5.0 (Linux; Android 4.0.4; JY-G3 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S81638[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Hyper S250 Build/ICS_IMM76I_B479) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81639[] = "OneBrowser/3.1 (Nokia6303iclassic/10.80)";
const char S81640[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC J Z321e Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81641[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-M950 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S81642[] = "Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD+ Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S81643[] = "HONGYU50_COSMOS_11B_HW (MRE\\3.0.00(2500) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) CHERRY_W5_PH_C722L_CQ01_X Release/2013.05.02 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S81644[] = "HONGYU50_COSMOS_11B_HW (MRE\\3.0.00(2500) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) Cherry_W12_Ph_C722L_CQ01_X Release/2013.04.26 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S81645[] = "HONGYU50_COSMOS_11B_HW (MRE/3.0.00(2500);MAUI/Y_W20_C737E_S06_X;BDATE/2013/05/04 11:46;LCD/X;CHIP/MT6250;KEY/Reduced;TOUCH/1;CAMERA/0;SENSOR/0;DEV/HONGYU50_COSMOS_11B_HW;WAP Browser/MAUI (HTTP PGDL;HTTPS);GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/005;GAMELO) CHERRY_W20_C737E_S06_X Release/2013.05.04 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S81646[] = "Mozilla/5.0 (Linux; Android 4.0.3; PG86100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S81647[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax_A110) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S81648[] = "Mozilla/5.0 (Linux; U; Android 4.2.1;zh-cn; ZTE V987 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30;";
const char S81649[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; ZTE V987 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81650[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; Karbonn_A6) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S81651[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S81652[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; SCH-I959 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S81653[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-T989 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/hercules";
const char S81654[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-T989 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/hercules";
const char S81655[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-T989 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/hercules";
const char S81656[] = "Mozilla/5.0 (Linux; Android 4.1.1; Smartfren Tab 7 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S81657[] = "SAGEM-VF527/1.0/EVN_PS MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.416 (GUI)";
const char S81658[] = "SAGEM-VF527/1.0/EVN_PW MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.416 (GUI)";
const char S81659[] = "SAGEM-VF527/1.0/EVN_PV MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.416 (GUI)";
const char S81660[] = "SAGEM-VF527/1.0/EVN_PK MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.416 (GUI)";
const char S81661[] = "SAGEM-VF527/1.0/EVN_PI MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.410 (GUI)";
const char S81662[] = "SAGEM-VF527/1.0/E5N_PN MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.413 (GUI)";
const char S81663[] = "SAGEM-VF527/1.0/EVN_PL MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.416 (GUI)";
const char S81664[] = "SAGEM-VF527/1.0/E5N_PO MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.416 (GUI)";
const char S81665[] = "SAGEM-VF527/1.0/EVN_PH MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.410 (GUI)";
const char S81666[] = "SAGEM-VF527/1.0/EVN_PJ MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.410 (GUI)";
const char S81667[] = "Nokia6170/2.0 (03.24) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81668[] = "SonyEricssonK330/R1DA002 TelecaBrowser/Q04C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1,SonyEricssonK330/R1DA002 TelecaBrowser/Q04C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81669[] = "SonyEricssonK330/R1DA002 TelecaBrowser/Q04C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81670[] = "SonyEricssonK330a/R1CE002 TelecaBrowser/Q04C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81671[] = "ZTE-R100/1.0 ACS-NF/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81672[] = "MicromaxX228+/3.83; Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81673[] = "Mozilla/5.0 (Linux;U;Android 2.2;en-us;i7 Build/IML74K) AppleWebKit/533.16(KHTML, like Gecko) Chrome/11.0.696.34 Safari/533.16";
const char S81674[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.3) Gecko/X Ubuntu/8.04 (hardy) Firefox/3.5.3";
const char S81675[] = "Mozilla/5.0 (X11; U; OpenBSD i386; en-US; rv:1.7.10) Gecko/X (No IDN) Firefox/1.0.6";
const char S81676[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB870 Build/4.5.1A-DTN-130; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81677[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB870 Build/ParanoidAndroid; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81678[] = "MicromaxQ22/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S81679[] = "JUC(Linux;U;Android2.3.4;Zh_cn;ST18i;480*854;)UCWEB7.8.0.95/139/444";
const char S81680[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Version/5.1 Safari/7534.51.22 Sleipnir/3.8.3";
const char S81681[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Version/6.0.2 Safari/8536.26.17 Sleipnir/3.8.5";
const char S81682[] = "ELIYA S1 Linux/3.0.13 Android/4.0.4 Release/07.20.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S81683[] = "NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax3-02) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile";
const char S81684[] = "Nokia311/5.0 (03.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia311/UC Browser8.0.3.107/69/444";
const char S81685[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; CAL21 Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81686[] = "Mozilla/5.0 (Linux; Android 4.1.1; GEM10313BK Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S81687[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB-1030 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S81688[] = "Mozilla/5.0 (Linux; Android 4.0.3; TAB-1030 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S81689[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S81690[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-P1000N Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.0.211/145/352";
const char S81691[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355";
const char S81692[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-P1000T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S81693[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; GT-P1000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S81694[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-P1000M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S81695[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S81696[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/444";
const char S81697[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S81698[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC J Z321e Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S81699[] = "NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac1-02) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S81700[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; XT530 Build/V4.480) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81701[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; XT531 Build/V4.480) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81702[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-i9100 Build/JDQ39; Carbon-Carbon-JB-v1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81703[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-i9100 Build/JDQ39; Carbon-CARBON-JB-v1.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81704[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-i9100 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81705[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; WALTON Primo_R1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81706[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; WALTON_Primo_R1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81707[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3110 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315";
const char S81708[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P3110 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315";
const char S81709[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P3110 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S81710[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3110 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S81711[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 796TPC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81712[] = "MQQBrowser/42 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S81713[] = "MQQBrowser/42 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3";
const char S81714[] = "MQQBrowser/421 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S81715[] = "MQQBrowser/421 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3";
const char S81716[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S81717[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT916 Build/2_26C_2014) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81718[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT916 Build/2_26D_2014) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81719[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT916 Build/2_26D_2014)";
const char S81720[] = "Mozilla/5.0 (Linux; Android 4.1.1; M532 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S81721[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F240S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81722[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; TID-80042 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81723[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Orange Kivo Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S81724[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; Nexus 4 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S81725[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA5320) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S81726[] = "Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S81727[] = "Mozilla/5.0 (Linux; Android 4.2.2; PAD720 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81728[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I8150 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S81729[] = "Mozilla/5.0(Linux;U;Android2.2.1;Zh_cn;GT-I9003;480*800;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S81730[] = "JUC(Linux;U;Android2.2.1;Zh_cn;GT-I9003;480*800;)UCWEB7.8.0.95/139/444";
const char S81731[] = "Mozilla/5.0(Linux;U;Android2.2.1;Zh_cn;GT-S5830;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S81732[] = "JUC(Linux;U;Android2.2.1;Zh_cn;GT-S5830;320*480;)UCWEB7.8.0.95/139/444";
const char S81733[] = "LG-GM200 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S81734[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) VZW:SCH-i770 PPC 320x320";
const char S81735[] = "Nokia5235";
const char S81736[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1 QQBrowser/6.10.11509.201";
const char S81737[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0) QQBrowser/6.10.11509.201";
const char S81738[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) QQBrowser/6.10.11509.201";
const char S81739[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1) QQBrowser/6.10.11509.201";
const char S81740[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.1.15) Gecko/X SeaMonkey/2.0.10";
const char S81741[] = "Mozilla/5.0 (Series40; NokiaX3-00/08.54; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S81742[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; vi-vn; A8 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81743[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; th-th; A8 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81744[] = "Mozilla/5.0 (Linux; Android 4.0.4; MB525 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81745[] = "Mozilla/5.0 (Linux; Android 4.0.4; MB525+ Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81746[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-T959V Build/VUVKJ6; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81747[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ru-ru; Fly_IQ280 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81748[] = "Mozilla/5.0 (Linux; Android 4.0.3; E860 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81749[] = "iBrowser/2.7 (Nokia5800XpressMusic;SymbianOS/9.1 Series60/3.0)";
const char S81750[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Picasso Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81751[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S81752[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE90) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile";
const char S81753[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; 201HW Build/HuaweiU9201L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81754[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_2; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7";
const char S81755[] = "Spice Mi-315 Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Spice Build/SpiceMi315) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81756[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22 slurp";
const char S81757[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22 slurp";
const char S81758[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22 slurp";
const char S81759[] = "Mozilla/5.0 (Linux; Android 4.0.4; LIFETAB_S9714 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S81760[] = "YahooMobileMail/1.0 (Android Mail; 1.4.4) (GT-P7500;samsung;GT-P7500;3.2/HTJ85B)";
const char S81761[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; FABLET F3 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81762[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-S7572 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81763[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-S7572 Build/JZO54K.S7572ZCAMB2;480*800)";
const char S81764[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-S7572 Build/JZO54K.S7572ZCAMD2;480*800)";
const char S81765[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (Galaxy Nexus, 42, 1.9.1)";
const char S81766[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (Galaxy Nexus, 42, 1.9.1)";
const char S81767[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGPT12 Build/TJS0166) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S81768[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGPT13 Build/TJS0166) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S81769[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 101 Titanium Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S81770[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E425 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81771[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E425f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81772[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E425g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81773[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035A Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81774[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81775[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81776[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6312 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S81777[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E440 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81778[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9082 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S81779[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT890 Build/9.8.2I-50_SMI-26) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S81780[] = "SAMSUNG-M7603/1.0 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S81781[] = "LG-KS365/V100 Obigo/WAP2.0 Profile/MIDP-2.0/CLDC-1.1";
const char S81782[] = "SAMSUNG-SGH-Z240/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S81783[] = "SCH-S239/ POLARIS/6.15 (GUI; compatible; UP.Browser)";
const char S81784[] = "SAMSUNG-E1410/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S81785[] = "SAMSUNG-E1410/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81786[] = "HTC_Touch_Viva_T2223 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S81787[] = "DoCoMo/2.0 P04B(c500;TB;W24H16)";
const char S81788[] = "Mozilla/4.74 [en] (Windows NT 5.0; U; maxamine.com--robot)";
const char S81789[] = "Mozilla/5.0 (compatible; Windows NT 5.0; phpwebbrainBot/0.1 - http://www.monsterli.ch/phpwebbrain/)";
const char S81790[] = "NokiaN8-00/UC Browser7.9.0.102/50/355/UCWEB";
const char S81791[] = "LENOVO-P60/(2009.11.24) S004/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S81792[] = "Lenovo-S520/S100 LMP/LML Release/2010.01.25 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S81793[] = "LENOVO-A332/(2009.12.16) S003/WAP2.0 Profile";
const char S81794[] = "Lenovo-i350/S111 LMP/LML Release/2010.10.12 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S81795[] = "LENOVO-A312/(2009.12.09) S002/WAP2.0 Profile";
const char S81796[] = "Lenovo-TD16/S100 LMP/LML Release/2010.7.30 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S81797[] = "LENOVO-P60/(2010.02.23) S005/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S81798[] = "Lenovo-P301/S303 LMP/LML Release/2010.12.29 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S81799[] = "PantechP2030/JSUSX Browser/Obigo/Q05A OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S81800[] = "Mozilla/5.0 (X11; Linux i686 on x86_64; rv:5.0) Gecko/X Firefox/5.0 SeaMonkey/2.2";
const char S81801[] = "Mozilla/5.0 (X11; Linux i686; rv:2.2a1pre) Gecko/X SeaMonkey/2.2a1pre";
const char S81802[] = "MQQBrowser/35 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S81803[] = "MQQBrowser/35 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3";
const char S81804[] = "Mozilla/5.0 (Java; U; en-in; gt-s5233a) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S81805[] = "Benten A668 Linux/3.0.13 Android/4.0.4 Release/10.24.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S81806[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; SKnetworksE860 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81807[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M380W Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S81808[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M380S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S81809[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA5320) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile";
const char S81810[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MT11i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S81811[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.2.19pre) Gecko/X Prism zdesktop/7.2.2";
const char S81812[] = "Mozilla/5.0 (Symbian3; U; en-us; Nokia808 PureView) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile";
const char S81813[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A25 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S81814[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A25 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S81815[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 OverDrive Media Console/2.6.5";
const char S81816[] = "Mozilla/5.0 (Linux; Android 4.0.4; KB901 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S81817[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; ST26i Build/11.2.A.0.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S81818[] = "SND-N7100/1.0 Linux/2.6.35.7 Android/4.1.1 Release/12.13.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S81819[] = "SND-N7100/1.0 Linux/2.6.35.7 Android/4.1.2 Release/01.24.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S81820[] = "SND-N7100/1.0 Linux/2.6.35.7 Android/4.1.2 Release/03.08.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S81821[] = "SND-N7100/1.0 Linux/2.6.35.7 Android/4.1.1 Release/03.04.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S81822[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XT907 Build/9.8.1Q_27-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (XT907, 42, 1.9.1)";
const char S81823[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-66) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (XT907, 42, 1.9.1)";
const char S81824[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-78) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (XT907, 42, 1.9.1)";
const char S81825[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; WT19i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S81826[] = "Mozilla/5.0 (Linux; U; Android 4.1 ;zh-cn; ZTE V889S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30;";
const char S81827[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; ZTE V889S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81828[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; SonyEricssonU1) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S81829[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Sony EricssonU1) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S81830[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9070 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S81831[] = "Mozilla/5.0 (Linux; Android 4.0.4; C1604 Build/11.1.A.0.68) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81832[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-MS770 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S81833[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Micromax A72 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S81834[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17";
const char S81835[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S81836[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6210 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S81837[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P1000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S81838[] = "Nokia2690/2.0 (09.70) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2690) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S81839[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; GT-I9508 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S81840[] = "Mozilla/5.0 (Linux; Android 4.3; SAMSUNG-SGH-I727 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S81841[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A660 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S81842[] = "Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD+ Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S81843[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I407 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S81844[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) LG-KS20/V10a Browser/IEMobile/7.6 MMS/LG-MMS-WINCE-V1.0/1.2 Java/LGVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81845[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) LG-KS20/TIM/V10b Browser/IEMobile/7.6 MMS/LG-MMS-WINCE-V1.0/1.2 Java/LGVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81846[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Vodafone/1.0/LG-KS20/V10b Browser/IEMobile/7.6 MMS/LG-MMS-WINCE-V1.0/1.2 Java/LGVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81847[] = "SAMSUNG-SGH-A885/A885TLIK1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S81848[] = "LG-GW820/V10g IEMobile/8.12 Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S81849[] = "HTC Streaming Player htc_tur / 1.0 / htc_marvel / 2.3.3";
const char S81850[] = "HTC Streaming Player orange_uk / 1.0 / htc_marvel / 2.3.3";
const char S81851[] = "HTC Streaming Player htc_europe / 1.0 / htc_marvel / 2.3.3";
const char S81852[] = "HTC Streaming Player tmous / 1.0 / htc_marvel / 2.3.3";
const char S81853[] = "HTC Streaming Player telstra_wwe / 1.0 / htc_marvel / 2.3.3";
const char S81854[] = "HTC Streaming Player vodafone_uk / 1.0 / htc_marvel / 2.3.3";
const char S81855[] = "HTC Streaming Player vodafone_uk / 1.0 / htc_marvel / 2.3.5";
const char S81856[] = "HTC Streaming Player htc_europe / 1.0 / htc_marvel / 2.3.5";
const char S81857[] = "HTC Streaming Player tmous / 1.0 / htc_marvel / 2.3.5";
const char S81858[] = "HTC Streaming Player HTC / 1.0 / marvelc / 2.3.4";
const char S81859[] = "LGE-LG265 POLARIS/6.01 MMP/2.0";
const char S81860[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HW-HUAWEI_C8500/C8500V100R001C92B660; 240*320; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S81861[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; C8500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81862[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; C8500 Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S81863[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; C8500 Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S81864[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; cs-cz; GSmart G1355 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81865[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/18.0.1025.142) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19";
const char S81866[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/18.0.1025.142) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19";
const char S81867[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/18.0.1025.142; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618)";
const char S81868[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/18.0.1025.151) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19";
const char S81869[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/18.0.1025.151) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19";
const char S81870[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/18.0.1025.151; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)";
const char S81871[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/18.0.1025.162)";
const char S81872[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/18.0.1025.162)";
const char S81873[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/18.0.1025.162) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19";
const char S81874[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.3; chromeframe/18.0.1025.162; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S81875[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/18.0.1025.162; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; InfoPath.2; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S81876[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/18.0.1025.168) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S81877[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/18.0.1025.168) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S81878[] = "HTC-Android/1.0";
const char S81879[] = "SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonj108i/UC Browser8.0.3.107/69/444";
const char S81880[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) RockMelt/0.8.40.147 Chrome/8.0.552.231 Safari/534.10";
const char S81881[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) RockMelt/0.8.36.116 Chrome/7.0.517.44 Safari/534.7";
const char S81882[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) RockMelt/0.8.34.820 Chrome/6.0.472.63 Safari/534.3";
const char S81883[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; QW TB-1007 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81884[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11 Comodo_Dragon/20.1.1.0";
const char S81885[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11 Comodo_Dragon/20.1.1.0";
const char S81886[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i667;7.10;8112)";
const char S81887[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i667;7.10;8773)";
const char S81888[] = "Mozilla/5.0 (Windows; U; Win98; en-US; m18) Gecko/X Netscape6/6.01";
const char S81889[] = "JUC(Linux;U;Android2.3.6;Zh_cn;GT-S6802;320*480;)UCWEB7.8.0.95/139/444";
const char S81890[] = "Mozilla/5.0 (Symbian3; U; en-us; NokiaE7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/51/355/UCWEB Mobile";
const char S81891[] = "Mozilla/5.0 (Linux; Android 4.1.9; Limo X2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81892[] = "Mozilla/5.0 (Linux; Android 4.1.1_jelly_bean; Limo X2+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81893[] = "Mozilla/5.0 (Linux; Android 4.0.4; LGL21 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81894[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S81895[] = "OperaMini/7.0.5.45389 CFNetwork/609.1.4 Darwin/13.0.0";
const char S81896[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Livingstone 2 Build/1.1.7 X-10:33) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81897[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; MB526 Build/JZO54K)";
const char S81898[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; MB526 Build/JRO03L)";
const char S81899[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MB526 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81900[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A820L Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81901[] = "JUC (Linux; U; 4.0.4; zh-cn; ST23i; 320*480) UCWEB7.9.0.94/139/444";
const char S81902[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (SGH-T989, 42, 1.9.1)";
const char S81903[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAE50) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S81904[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; ZTE-U V889F Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81905[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE V889F Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81906[] = "Mozilla/5.0 (Linux; Android 4.1.2; SH-04E Build/S6181) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81907[] = "Mozilla/5.0 (iPad; U; CPU OS 6_1_2 like Mac OS X; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile";
const char S81908[] = "Mozilla/5.0 (iPad; U; CPU OS 6_1_3 like Mac OS X; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile";
const char S81909[] = "Mozilla/5.0 (iPad; U; CPU OS 6_1_3 like Mac OS X; en-GB) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile";
const char S81910[] = "Mozilla/5.0 (Linux; Android 4.0.3; W732 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S81911[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-L40G Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S81912[] = "Nokia205/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia205) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S81913[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.3 Mobile/8J2 Safari/6533.18.5";
const char S81914[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.3 Mobile/8L1 Safari/6533.18.5";
const char S81915[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.3 Mobile/8H7 Safari/6533.18.5";
const char S81916[] = "Mozilla/5.0 (Linux; Android 4.0.4; A89 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S81917[] = "Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabS2109A-F Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S81918[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7102 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S81919[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7102 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S81920[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S6012) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S81921[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-P3100) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S81922[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6312) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S81923[] = "Mozilla/5.0 (Linux; Android 4.1.2; AT100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S81924[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Andi5Li Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81925[] = "Mozilla/5.0 (Linux; Android 4.1.2; bq Elcano Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S81926[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S5282) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S81927[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7510 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S81928[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; HTL22 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81929[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S81930[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTCONE Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S81931[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-EM30/R6716_G_71.01.02R_B BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81932[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-EM30/R6716_G_71.03.04R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81933[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-EM30/R6716_G_71.04.06R_C BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81934[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-EM30/R6716_G_71.03.06R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81935[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-EM30/Imperfecto_Lite_v10.3 BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81936[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-EM30/R6716_G_71.01.02R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81937[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-EM30/IMPERFECTO_V.2.0_C BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81938[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-EM30/R6716_G_71.00.19R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81939[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-EM30/R6716_G_71.04.06R_A BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81940[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-EM30/R6716_G_71.03.02R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81941[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-EM30/R6716_G_71.01.02R_A BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S81942[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-; LG-GW620 Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S81943[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; LG-GW620 Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S81944[] = "SAMSUNG-SGH-A237/A237UCHG1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S81945[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Nokia6290/3.03; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S81946[] = "Mozilla/5.0 (SymbianOS/9.3 U; [en]; Series60/3.2 Nokia6290/1.00 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S81947[] = "NOKIAE5-00/UC Browser7.7.1.88/28/444";
const char S81948[] = "nokiae5-00/UC Browser7.7.1.88/69/444  3gpp-gba";
const char S81949[] = "Mozilla/4.0 SonyEricssonW890iv/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S81950[] = "KDDI-TS3V UP.Browser/6.2_7.2.7.1.K.6.210 (GUI) MMP/2.0";
const char S81951[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.1 Chrome/15.0.900.2 Safari/535.2";
const char S81952[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.1 Chrome/15.0.900.2 Safari/535.2";
const char S81953[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.1 Chrome/15.0.900.2 Safari/535.2";
const char S81954[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.1 Chrome/15.0.900.2 Safari/535.2";
const char S81955[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; sdk Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Maxthon";
const char S81956[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; es-ES; rv:1.9.1.7) Gecko/X Thunderbird/3.0.1 ThunderBrowse/3.2.8.1";
const char S81957[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SPICE Mi-720 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81958[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:14.0) Gecko/X Firefox/14.0.1 SeaMonkey/2.11";
const char S81959[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5578 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81960[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5578 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S81961[] = "JUC(Linux; U;2.3.4;zh_cn;T703;320*480;)UCWEB7.6.0.75/139/32534";
const char S81962[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOX905 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81963[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; X905 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S81964[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.6.2)";
const char S81965[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Sleipnir/3.6.2";
const char S81966[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.60 Safari/537.1 Sleipnir/3.6.1";
const char S81967[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.6.1)";
const char S81968[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-nz; VIGO7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81969[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; VIGO7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81970[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; N-05D Build/AX)";
const char S81971[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; N-05D Build/AX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81972[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A810K Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81973[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A810S Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81974[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Scroll Basic Plus v.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S81975[] = "JUC (Linux; U; 2.2.2; zh-cn; LG-P350; 240*320) UCWEB7.9.0.94/139/444";
const char S81976[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; KFOT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-17 12:06, +0000, gb";
const char S81977[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-17 12:06, -0500, us";
const char S81978[] = "NokiaC2-05/2.0 (08.69) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac2-05) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S81979[] = "Mozilla/5.0 (Linux; Android 4.1.2; Micromax A116 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81980[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5830) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S81981[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia700) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S81982[] = "Mozilla/5.0 (Linux; Android 4.0.3; SmartTabII10 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S81983[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch W619 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S81984[] = "Mozilla/5.0 (Linux; U; Android 4.0; zh-cn; K-Touch W619 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81985[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch_W619 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S81986[] = "Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S81987[] = "Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S81988[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Edison Build/1.1.8 X-16:29) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S81989[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xperia S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S81990[] = "Mozilla/5.0 (Linux; Android 4.1.1; Cink King Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S81991[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Andi4.5h Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81992[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Mobile Safari/535.19 Silk-Accelerated=true";
const char S81993[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; MT11i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S81994[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A90S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S81995[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Xperia S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81996[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Xperia S Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S81997[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT885 Build/9.8.3O-148_IRD-3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S81998[] = "Mozilla/5.0 (Linux; Android 4.1.1; MZ601 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S81999[] = "Mozilla/5.0 (Linux; Android 4.1.2; MZ601 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S82000[] = "Mozilla/5.0 (compatible; MSIE 9.0; Win95; Trident/5.0)";
const char S82001[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; X21 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82002[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S82003[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562i Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S82004[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9082) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S82005[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5302 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S82006[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST25i Build/6.1.1.B.1.10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S82007[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S82008[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; Karbonn A6 Build/MocorDroid4.0.3) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S82009[] = "MQQBrowser/44 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3";
const char S82010[] = "MQQBrowser/44 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3";
const char S82011[] = "MQQBrowser/44 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari/7534.48.3";
const char S82012[] = "MQQBrowser/44 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3";
const char S82013[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One XL Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S82014[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SPH-L710 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S82015[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S82016[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S82017[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9305 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S82018[] = "NokiaN-Gage/1.0 SymbianOS/6.1 Series60/1.2 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S82019[] = "NokiaN-GageQD/1.0 SymbianOS/6.1 Series60/1.2 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S82020[] = "SAMSUNG-GT-S8000L/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S82021[] = "SAMSUNG-GT-S8000B/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S82022[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ru-ru; GSmart G1315 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82023[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; LogicPD Zoom2 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S82024[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; NookColor Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S82025[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ERIS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82026[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ERIS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82027[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ERIS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82028[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ERIS-GSBv3.3 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82029[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Eris Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82030[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ERIS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82031[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; GSBv1.7-ERIS Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82032[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ERIS GSBv2.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82033[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.0.2.0 Chrome/16.0.912.59 Safari/535.7";
const char S82034[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.0.2.0 Chrome/16.0.912.59 Safari/535.7";
const char S82035[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; it-it; SMARTPAD810C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82036[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S82037[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S82038[] = "Mozilla/5.0 (Series40; NokiaC3-00/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.4.0.34.8";
const char S82039[] = "Mozilla/5.0 (Maemo; Linux armv7l; rv:12.0) Gecko/X Firefox/12.0 Fennec/12.0";
const char S82040[] = "ZTE-TU830_TD/1.0 OPhone/2.5 (Linux; Android 2.2) Release/08.15.2011 Browser/WAP2.0 (AppleWebKit/533.1) Mobile Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S82041[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-T U830 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S82042[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-T U830 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82043[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-T U830 Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S82044[] = "iBrowser/2.5 (NokiaE72-1;SymbianOS/9.3 Series60/3.2)";
const char S82045[] = "Opera/9.80 (Android; Opera Mini/6.6.29182/28.3234; U; zh) Presto/2.8.119 Version/11.10";
const char S82046[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.0.7) Gecko/X Firefox/1.5.0.7 Flock/0.7.6";
const char S82047[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.0.9) Gecko/X Flock/0.7.9.1";
const char S82048[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/527+ (KHTML, like Gecko)  QtWeb Internet Browser/3.0 http://www.QtWeb.net";
const char S82049[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/527+ (KHTML, like Gecko) QtWeb Internet Browser/3.0 http://www.QtWeb.net";
const char S82050[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus One Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82051[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82052[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus One Build/Slim VJ) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82053[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile i-style Q3 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82054[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile i-style Q3i Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82055[] = "Mozilla/5.0 (Linux; Android 4.0.4; Tablet PC Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S82056[] = "Mozilla/5.0 (Linux; Android 4.0.3; Tablet PC Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S82057[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID_Serials Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82058[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VS TOUCHTAB 9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82059[] = "AT-AQ28D Linux/2.6.35 Android/2.3.5 Release/11.15.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1";
const char S82060[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P1000 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S82061[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P1000N Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S82062[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P1000 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82063[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P1000L Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S82064[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P1000 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S82065[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P1000 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82066[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Energy x10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82067[] = "Micromax X293/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S82068[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; U25GT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82069[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; LG-P350) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S82070[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ro-ro; COSMOTE_MY_TAB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82071[] = "Mozilla/5.0 (Linux; Android 4.1.2; EK-GC100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S82072[] = "Mozilla/5.0 (Linux; Android 4.1.1; EK-GC100 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S82073[] = "Mozilla/5.0 (Linux; Android 4.0.3; MD_LIFETAB_P9516 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S82074[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; ST26i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S82075[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Wildfire S A510e Build/JDQ39; CyanogenMod-Sense 5T by djolivier) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82076[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Wildfire S A510e Build/CM10.1-Release-Candidate-v2.1; CyanogenMod-CM10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82077[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Wildfire S A510e Build/CM10.1-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/marvel";
const char S82078[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P870 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82079[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82080[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Infinix X500 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82081[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xperia ion Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82082[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT28h Build/6.2.B.0.211) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82083[] = "Mozilla/5.0 (Linux; Android 4.1.2; CINK FIVE Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82084[] = "Mozilla/5.0 (Linux; Android 4.1.1; CINK+ Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82085[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; HS-9DTB4 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S82086[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HTC-802d/1.11.1401.2) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82087[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HTC 802d Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82088[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HTC-802d/1.02.1401.1) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82089[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC One Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S82090[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT25i Build/4.1.B.0.631) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S82091[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A21 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S82092[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A21 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S82093[] = "Mozilla/5.0 (Linux; Android 4.1.1; CINK SLIM Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S82094[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Comodo_Dragon/1.0.0.9 Chrome/ Version/3.2.1 Safari/532.0";
const char S82095[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-I9082 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82096[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Edison Build/1.1.8 X-16:29) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S82097[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; LT26i Build/6.2.B.1.96) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S82098[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT26i Build/6.2.B.0.200) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S82099[] = "Nokia6100/1.0 (04.01) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S82100[] = "LG-TU515 Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S82101[] = "LG-CU515/V1.0g Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S82102[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.7.5) Gecko/X Netscape/8.0.1";
const char S82103[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.7.7) Gecko/X Firefox/1.0.3";
const char S82104[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us; HTC_P515E) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S82105[] = "Mozilla/5.0 (Windows NT 6.1; rv:6.0) Gecko/X Firefox/6.0 SeaMonkey/2.3";
const char S82106[] = "nokian73-1/UC Browser7.9.0.102/69/352";
const char S82107[] = "MQQBrowser/33 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3";
const char S82108[] = "MQQBrowser/33 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3";
const char S82109[] = "MQQBrowser/33 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S82110[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/GRJ90)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82111[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PC36100 Build/GRI40)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82112[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S82113[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PC36100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S82114[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:6.0.2) Gecko/X Firefox/6.0.2 slurp";
const char S82115[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:6.0) Gecko/X Firefox/6.0 slurp";
const char S82116[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:6.0.2) Gecko/X Firefox/6.0.2 slurp slurp";
const char S82117[] = "Mozilla/5.0 (Windows NT 6.1; rv:6.0.2) Gecko/X Firefox/6.0.2 slurp";
const char S82118[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; T3 Build/1.8.1371.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82119[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; T3 Build/1.9.1425.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82120[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; T3 Build/1.2.867.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82121[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; T3 Build/1.3.1018.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82122[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; T3 Build/1.7.1310.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82123[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; T3 Build/1.6.1250.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82124[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN86 8MP) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile";
const char S82125[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAE73) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S82126[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokian73-1) U2/1.0.0 UCBrowser/8.8.0.227 U2/1.0.0 Mobile";
const char S82127[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Videocon A30 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82128[] = "Mozilla/5.0 (Linux; Android 4.0.4; SBM200SH Build/S0016) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82129[] = "Nokia309/2.0 (05.85) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia309) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S82130[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NGM Absolute Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82131[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ThL A1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82132[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; DROID RAZR HD Build/9.8.1Q_39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S82133[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S82134[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT881 Build/6.7.3-130_YUS-11) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82135[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7511 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S82136[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Fly IQ444; Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82137[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Fly_IQ444 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82138[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ThL V12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82139[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ThL V12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82140[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; P5 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82141[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A95) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S82142[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Celkon*A95*Pro) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S82143[] = "UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 710) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile";
const char S82144[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaE63";
const char S82145[] = "Nokia303/5.0 (14.38) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia303) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S82146[] = "Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S82147[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 992D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82148[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Engage 7HD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82149[] = "Mozilla/5.0 (Linux; Android 4.2.2; Evo 3D CDMA Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82150[] = "Mozilla/5.0 (Linux; Android 4.2.2; Evo 3D GSM Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82151[] = "Mozilla/5.0 (Linux; Android 4.2.2; PG86100 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82152[] = "Mozilla/5.0 (Linux; Android 4.0.4; PadFone Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S82153[] = "Mozilla/5.0 (Linux; Android 4.0.4; PadFone Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82154[] = "Mozilla/5.0 (Linux; Android 4.0.3; PadFone Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82155[] = "Mozilla/5.0 (Linux; Android 4.0.3; PadFone Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S82156[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82157[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL_ONE_TOUCH_6030X_Orange Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82158[] = "iOS/7.0 (11A4372q) dataaccessd/1.0";
const char S82159[] = "iOS/7.0 (11A4449d) dataaccessd/1.0";
const char S82160[] = "iOS/7.0 (11A465) dataaccessd/1.0";
const char S82161[] = "iOS/7.0.2 (11A501) dataaccessd/1.0";
const char S82162[] = "Mozilla/5.0 (Linux; U; Android OUYA 4.1.2; en-us; OUYA  Build/JZO54L-OUYA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82163[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E445 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82164[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; vivo Y19t Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82165[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; vivo Y19t Build/JOP40D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S82166[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; vivo Y11 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82167[] = "Mozilla/5.0 (Linux; Android 4.1.2; MI-ONE Plus Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S82168[] = "Mozilla/5.0 (Linux; Android 4.1.2; 201M Build/9.8.2Q-34_SMJ-102) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S82169[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S82170[] = "Mozilla/5.0 (Linux; Android 4.0.3; W732 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S82171[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S5303) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S82172[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Amazon Kindle Fire Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/otter";
const char S82173[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Amazon Kindle Fire Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/otter";
const char S82174[] = "NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-05) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S82175[] = "Mozilla/5.0 (Linux; Android 4.2.2; Micromax A76 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S82176[] = "Mozilla/5.0 (Linux; Android 4.2.2; Micromax A74 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S82177[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S82178[] = "browseripad/40889 CFNetwork/672.0.2 Darwin/14.0.0";
const char S82179[] = "CitrixReceiver-iPad/179 CFNetwork/672.0.2 Darwin/14.0.0";
const char S82180[] = "Perfect%20Browser-iPad/10.0 CFNetwork/672.0.2 Darwin/14.0.0";
const char S82181[] = "MOT-RAZRV6/96.66.74R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11033063) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S82182[] = "MOT-RAZRV6/99.41.08R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163225) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S82183[] = "MOT-RAZRV6/98.30.80R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12093102) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S82184[] = "MOT-RAZRV6/96.80.55AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11073083) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S82185[] = "MOT-RAZRV6/99.41.0BR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163234) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S82186[] = "MOT-RAZRV6v/98.20.33BR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12043064) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S82187[] = "MOT-RAZRV6/96.80.51R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11073071) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S82188[] = "MOT-RAZRV6/96.A0.0ER BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11073121) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S82189[] = "MOT-RAZRV6/98.30.72AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12003079) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S82190[] = "MOT-RAZRV6/99.41.05R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163189) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S82191[] = "MOT-RAZRV6/96.66.71AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11033039) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S82192[] = "MOT-RAZRV6vb/99.41.04ER BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163189) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S82193[] = "MOT-RAZRV6v/98.50.11R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12093118) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S82194[] = "MOT-RAZRV6/98.60.02R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12093118) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [he]";
const char S82195[] = "SCH-R500 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S82196[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; LG-V909 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S82197[] = "Nokia5250";
const char S82198[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5; U; Edition MacAppStore; en) Presto/2.9.168 Version/11.50";
const char S82199[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-es; AIRIS_OnePAD700 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82200[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:12.0) Gecko/X Firefox/12.0 SeaMonkey/2.9";
const char S82201[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; PLOYER-MOMO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82202[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; PLOYER-MOMO Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82203[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.3; PLOYER-MOMO Build/IML74K)";
const char S82204[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s8530) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S82205[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S82206[] = "Nokia311/5.0 (03.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia311) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S82207[] = "Instagram 3.1.2 (iPhone; iPhone OS 5.1.1; en_US) AppleWebKit/420+";
const char S82208[] = "Instagram 3.1.2 (iPhone; iPhone OS 5.1; en_US) AppleWebKit/420+";
const char S82209[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7300 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82210[] = "Logos/1.1.3 (Linux; U; Android 4.1.1; en-US; GT-P7300 Build/cm_p5-userdebug 4.1.1 JRO03H eng.kalltkaffe.X.X test-keys)";
const char S82211[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P7300 Build/JRO03H)";
const char S82212[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82213[] = "Biblia/2.1.2 (Linux; U; Android 4.1.2; es-ES; GT-P7300 Build/cm_p5-userdebug 4.1.2 JZO54K eng.kalltkaffe.X.X test-keys)";
const char S82214[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P7300 Build/JZO54K)";
const char S82215[] = "Vyrso/2.1.2 (Linux; U; Android 4.1.1; en-US; GT-P7300 Build/cm_p5-userdebug 4.1.1 JRO03H eng.safariking.X.X test-keys)";
const char S82216[] = "NOKIA-Lumia 610/UCWEB2.6.0.213/47/32079";
const char S82217[] = "Dolphin http client/9.1.0(195) (Android)";
const char S82218[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; NGM EXPLORER Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82219[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT270 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S82220[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; S100 AMAZE Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82221[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST26i Build/11.0.A.7.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S82222[] = "Mozilla/5.0 (Linux; Android 4.0.4; ISW13HT Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82223[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC J Z321e Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82224[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_ChaCha_A810e) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S82225[] = "UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; HTC_ChaCha_A810e) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S82226[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MTV_volt Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82227[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT-I5503) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S82228[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Masstel M15 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82229[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9000 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S82230[] = "Mozilla/5.0 (Linux; Android 4.1.1; HP Slate 7 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S82231[] = "Mozilla/5.0 (Linux; Android 4.0.4; NABI2-NV7A Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S82232[] = "Mozilla/5.0 (Linux; Android 4.0.4; NABI2-NV7A-UK Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S82233[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A65 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S82234[] = "SND-I7100/1.0 Linux/2.6.35.7 Android/4.0.3 Release/03.05.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S82235[] = "SND-I7100/1.0 Linux/2.6.35.7 Android/4.0.3 Release/02.26.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S82236[] = "SND-I7100/1.0 Linux/2.6.35.7 Android/4.0.3 Release/01.21.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S82237[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT11i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S82238[] = "Mozilla/5.0 (Linux; Android 4.1.1; GOCLEVER-TERRA9o Build/GOCLEVER-TERRA9o-GU13045-0325) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S82239[] = "Mozilla/5.0 (Linux; Android 4.1.1; TP7-1000DC light Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S82240[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6833 Build/14.1.B.1.510) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S82241[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; ARM; Trident/7.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S82242[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; ARM; Trident/7.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)";
const char S82243[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T959V Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/galaxys4gmtd";
const char S82244[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Micromax_A72 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S82245[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire SV Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S82246[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S82247[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; A111 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S82248[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax A110) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S82249[] = "Nokia210/2.0 (04.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia210) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S82250[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; P2 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S82251[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-CN; R809T Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S82252[] = "Nokia200/2.0 (11.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311";
const char S82253[] = "Mozilla/5.0 (Linux; Android 4.1.2; K1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S82254[] = "Mozilla/5.0 (Linux; Android 4.1.1; Ideapad K1 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S82255[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGP312 Build/10.3.1.C.0.136) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S82256[] = "NokiaN77-1/3.0838.22.0.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S82257[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-za; RBM C Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S82258[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; de-at; RBM3 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S82259[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; hu-hu; RBM2 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S82260[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) HTC6175";
const char S82261[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) USCCHTC6175";
const char S82262[] = "SAMSUNG-GT-B7320/1.0 MIDP/2.1 CLDC/1.1 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.2.2.51";
const char S82263[] = "LGE-VX8300/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S82264[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; DM009SH Build/S0300) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82265[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; DM009SH Build/S0400) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82266[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; DM009SH Build/S0800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82267[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; DM009SH Build/S0700) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82268[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; DM009SH Build/S0500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82269[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16";
const char S82270[] = "Mozilla/5.0 (Windows NT 4.10) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19";
const char S82271[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.1.10) Gecko/X Postbox/1.0.1";
const char S82272[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; fr; rv:1.9.1.3) Gecko/X Firefox/3.5.3 CometBird/3.5.3";
const char S82273[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; u8800 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82274[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; u8800 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82275[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Huawei Ideos X5 1.12.9(ret4rt) Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82276[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Huawei Ideos X5 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82277[] = "Mozilla/5.0 (Windows NT 5.2; rv:7.0a1) Gecko/X SeaMonkey/2.4a1";
const char S82278[] = "Mozilla/5.0 (Windows NT 5.1; rv:7.0) Gecko/X Firefox/7.0 SeaMonkey/2.4 Lightning/1.0b7";
const char S82279[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB855 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82280[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB855 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82281[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB855 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82282[] = "Mozilla/5.0 (Windows NT 6.0; rv:2.0) Gecko/X Firefox/4.0 PaleMoon/4.0";
const char S82283[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8b5) Gecko/X Firefox/1.4.1";
const char S82284[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F100L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82285[] = "Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia305/UC Browser7.9.0.102/69/352";
const char S82286[] = "Mozilla/5.0 (Linux; Android 4.1.1; Incredible S Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82287[] = "Mozilla/5.0 (Linux; Android 4.1.2; Incredible S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82288[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; One V Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82289[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-03D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82290[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; PW1100S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82291[] = "Mozilla/5.0 (Linux; Android 4.0.4; dL1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82292[] = "Mozilla/5.0 (Linux; Android 4.0.4; SO-03D Build/6.1.F.0.106) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82293[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Spice Mi-320 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82294[] = "LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-gd510) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S82295[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5580C Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S82296[] = "MQQBrowser/39 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S82297[] = "MQQBrowser/39 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3";
const char S82298[] = "MQQBrowser/391 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S82299[] = "MQQBrowser/391 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3";
const char S82300[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; mauie800) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S82301[] = "UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; mauie800) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S82302[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; 11a h700tecno) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S82303[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; s8000) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S82304[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; beetel gd777) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S82305[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sjt_e861a) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S82306[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; 11a wbg3509a_0101) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S82307[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; 11a h700tecno) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S82308[] = "UCWEB/2.0(Java; U; MIDP-2.0; es-la; mozilla) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S82309[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17 slurp";
const char S82310[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17 slurp";
const char S82311[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17 slurp";
const char S82312[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D97 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82313[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-B5330 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S82314[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; ALCATEL ONE TOUCH 985N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S82315[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; ALCATEL_ONE_TOUCH_985N Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S82316[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCEVOV4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S82317[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S82318[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S82319[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S82320[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S82321[] = "Mozilla/5.0 (Linux; Android 4.0.4; S500 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82322[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5670 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S82323[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5670 Build/FROYO) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S82324[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M440S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82325[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7500 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S82326[] = "GoogleEarth/7.0.3.8542(Macintosh;Mac OS X (10.6.8);en;kml:2.2;client:Free;type:default)";
const char S82327[] = "Mozilla/5.0 (Linux; Android 4.0.4; CT1010 Build/IMM76I.v007) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S82328[] = "Mozilla/5.0 (Linux; Android 4.1.1; V360 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S82329[] = "Mozilla/5.0 (Linux; Android 4.2.2; Xperia S Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S82330[] = "Mozilla/5.0 (Linux; Android 4.0.4; PadFone Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S82331[] = "Mozilla/5.0 (Linux; Android 4.0.4; PadFone Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S82332[] = "Mozilla/5.0 (Linux; Android 4.0.3; PadFone Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S82333[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:20.0) Gecko/X Firefox/20.0";
const char S82334[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233s) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S82335[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233s) U2/1.0.0 UCBrowser/9.0.0.295 U2/1.0.0 Mobile";
const char S82336[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233s) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S82337[] = "UCWEB/2.0 (Java; U; MIDP-2.0; vi; gt-s5233s) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S82338[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A25) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S82339[] = "Mozilla/5.0 (Series40; Nokia307/07.55; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.36";
const char S82340[] = "Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300K Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82341[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROIDZ Sport Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82342[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Edison Build/1.1.8 X-16:29) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S82343[] = "Mozilla/5.0 (Linux; Android 4.2.1; N9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82344[] = "SAMSUNG-SGH-S275M/1.0 SHP/VPP/R5 NetFront/4.2 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S82345[] = "SAMSUNG-SGH-T189N/T189NUVMG1 SHP/VPP/R5 NetFront/4.2 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S82346[] = "SAMSUNG-GT-E1265/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S82347[] = "Mozilla/5.0 (Linux; Android 4.1.2; Lenovo A706_ROW Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S82348[] = "Mozilla/5.0 (Linux; Android 4.1.1; CINK KING Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S82349[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-04D Build/9.1.C.0.475) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S82350[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST18i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S82351[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iball Andi4di Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82352[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A9+ Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S82353[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SH-08E Build/S7200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82354[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SH-08E Build/S8220) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82355[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Q800 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S82356[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus S Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S82357[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; HUAWEI P6-C00 Build/HuaweiP6-C00) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S82358[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; P2 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S82359[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; HUAWEI P6-T00 Build/HuaweiP6-T00) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S82360[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-CN; HUAWEI G610-U00 Build/HuaweiG610-U00) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S82361[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SHW-M100S Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S82362[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SHW-M100S Build/ECLAIR)";
const char S82363[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; FroyoEris Build/unknown) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82364[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; FroyoEris Build/unknown) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82365[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; ERIS Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82366[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Eris FroshedYo v11 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82367[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Eris Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82368[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S82369[] = "Safari (unknown version) CFNetwork/454.11.5";
const char S82370[] = "Safari (unknown version) CFNetwork/520.2.6";
const char S82371[] = "Safari (unknown version) CFNetwork/454.10.13";
const char S82372[] = "Mozilla/5.0 (Sample; MFC Simulator/4.2; like Gecko) NetFront/4.2";
const char S82373[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; en) Opera 8.51";
const char S82374[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; en) Opera 8.50";
const char S82375[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; en) Opera 8.53";
const char S82376[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; en) Opera 8.54";
const char S82377[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) 1Password/3.6.5/X (like Mobile/8C148 Safari/6533.18.5)";
const char S82378[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) 1Password/3.6.2/X (like Mobile/8C148 Safari/6533.18.5)";
const char S82379[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PLOYER-MOMO Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82380[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; PLOYER-MOMO Build/GRJ22)";
const char S82381[] = "Mozilla/5.0 (Linux; Android 4.0.4; SH-09D Build/SB220) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82382[] = "InstacastHD/1.1.3 CFNetwork/609 Darwin/13.0.0";
const char S82383[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XT875 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82384[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XT875 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82385[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT875 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82386[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E140S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82387[] = "Mozilla/5.0 (Java; U; kau; lg-x330t) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S82388[] = "Mozilla/5.0 (Java; U; en-us; lg-x330t) UCBrowser8.3.0.154/69/352/UCWEB Mobile";
const char S82389[] = "MQQBrowser/2.7 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S82390[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5570C Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S82391[] = "Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 97 XENON Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S82392[] = "Mozilla/5.0 (Windows; U; WinNT4.0; rv:1.7.3) Gecko/X Firefox/0.10.1";
const char S82393[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN305 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S82394[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN305 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82395[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn;GiONEE-GN305/Phone Build/IMM76D) AppleWebKit534.30(KHTML,like Gecko)Version/4.0 Mobile Safari/534.30";
const char S82396[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN305 Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S82397[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us ; Nexus 7 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S82398[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; Nexus 7 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S82399[] = "Mozilla/5.0 (Linux; Android 4.0.3; PMP5197DULTRA Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S82400[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZP900 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82401[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZP900S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82402[] = "IUC(U;iOS 6.1.3;Zh-cn;320*480;)/UCWEB8.8.0.212/44/997";
const char S82403[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; el-gr; NovaPad80C104 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82404[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 505;7.10;8858)";
const char S82405[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 505;7.10;8862)";
const char S82406[] = "Mozilla/5.0 (Linux; Android 4.2.2; ASUS Eee Pad Transformer Prime TF201 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S82407[] = "Mozilla/5.0 (Linux; Android 4.2.1; Transformer Prime TF201 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S82408[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT910 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S82409[] = "Mozilla/5.0 (Linux; Android 4.0.4; LIFETAB_S9512 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S82410[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9210 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S82411[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I727R Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S82412[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E110S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S82413[] = "Mozilla/5.0 (X11; OpenBSD amd64; rv:20.0) Gecko/X Firefox/20.0";
const char S82414[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; E2001 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82415[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT11i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S82416[] = "SAMSUNG-SGH-I317/100.40102";
const char S82417[] = "SAMSUNG-SGH-T889/100.40101";
const char S82418[] = "SAMSUNG-SPH-L900/100.40102";
const char S82419[] = "SAMSUNG-GT-N7100/100.40102";
const char S82420[] = "[FBAN/FB4A;FBAV/3.5;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/AT&amp-T;FBPN/com.facebook.katana;FBDV/SAMSUNG-SGH-I317;FBSV/4.1.2;]";
const char S82421[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian96-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S82422[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; Nokia N96) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S82423[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaE71";
const char S82424[] = "Mozilla/5.0 (Linux; Android 2.3.5; PC36100 Build/GRJ90) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S82425[] = "Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S82426[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0";
const char S82427[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0.2";
const char S82428[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0.1";
const char S82429[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0.3";
const char S82430[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0.4";
const char S82431[] = "Spice M5385";
const char S82432[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M040 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S82433[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26ii Build/6.2.B.0.200) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82434[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; Revo_Lead Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82435[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P708g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S82436[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P705 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S82437[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; 202F Build/V16R45B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82438[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; 202F Build/V17R47F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82439[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; 202F Build/V17R47F)";
const char S82440[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC5-06) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S82441[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A65 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S82442[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S82443[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Sensation Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S82444[] = "Mozilla/5.0 (Linux; Android 4.1.2; Sensation Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S82445[] = "Nokia 5320/UCBrowser7.7.0.81/28/800";
const char S82446[] = "Nokia 5320";
const char S82447[] = "NOKIA5320";
const char S82448[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.71.128 Chrome/14.0.835.202 Safari/535.1";
const char S82449[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.64.361 Chrome/13.0.782.218 Safari/535.1";
const char S82450[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.72.698 Chrome/14.0.835.202 Safari/535.1";
const char S82451[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.64.319 Chrome/13.0.782.215 Safari/535.1";
const char S82452[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.24 (KHTML, like Gecko) RockMelt/0.9.56.357 Chrome/11.0.696.71 Safari/534.24";
const char S82453[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.13 (KHTML, like Gecko) RockMelt/0.9.46.126 Chrome/9.0.597.107 Safari/534.13";
const char S82454[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) RockMelt/0.9.50.518 Chrome/10.0.648.205 Safari/534.16";
const char S82455[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.30 (KHTML, like Gecko) RockMelt/0.9.62.313 Chrome/12.0.742.122 Safari/534.30";
const char S82456[] = "MicromaxX505";
const char S82457[] = "Mozilla/5.0 (Windows NT 6.1; rv:2.0.1) Gecko/X Firefox/4.0.1 SeaMonkey/2.1";
const char S82458[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.24) Gecko/X Fedora/3.1.16-1.fc14 Thunderbird/3.1.16";
const char S82459[] = "Mozilla/5.0 (Windows NT 5.0; rv:11.0) Gecko/X Firefox/11.0,gzip(gfe) (via translate.google.com)";
const char S82460[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I777 Build/Task650 & Ktoonsez AOKP) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82461[] = "JUC(Linux;U;Android4.0.3;Zh_cn;GT-I9100;480*800;)UCWEB7.8.0.95/139/444";
const char S82462[] = "JUC(Linux;U;Android4.0.3;Zh_cn;GT-I9100G;480*800;)UCWEB7.8.0.95/139/444";
const char S82463[] = "JUC(Linux;U;Android4.0.4;Zh_cn;GT-I9100;480*800;)UCWEB7.8.0.95/139/444";
const char S82464[] = "Opera/9.10 (Windows NT 5.1; U; pl) Version/10.9";
const char S82465[] = "Opera/9.10 (Windows NT 5.1; U; pl) Presto/2.1.15 Version/10.10";
const char S82466[] = "Opera/9.12 (Windows NT 5.1; U; ru) Presto/8.0.6.61 Version/10.10";
const char S82467[] = "iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Karbonn_A18 Build/KarbonnA18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82468[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Y2 Build/HCL ME Tablet Y2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82469[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE T22 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82470[] = "Mozilla/5.0 (Linux; Android 4.0.4; T-02D Build/V19R50B) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82471[] = "Nokia306/2.0 (03.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia306) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S82472[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Galaxy Precedent Build/AreaRom Test)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82473[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax A110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S82474[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en_us; Micromax A80 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82475[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S82476[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S82477[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S82478[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S82479[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; StarPAD 9 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82480[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-ph; Cloudfone_Excite350e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82481[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P880 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82482[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; ALCATEL; ONE TOUCH 5040X)";
const char S82483[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; TAC-90011 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82484[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GSmart Rio R1 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82485[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; LT18i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82486[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Xperia Arc S Build/JZO54K)";
const char S82487[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-gb; SonyEricssonLT18i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82488[] = "Mozilla/5.0 (Linux; Android 4.0.4; WT19i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S82489[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2; WOW64; Trident/4.0; Touch; .NET4.0E; .NET4.0C; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729)";
const char S82490[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S82491[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562i Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S82492[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S5301 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S82493[] = "Mozilla/5.0 (Linux; Android 4.0.3; SmartTabII10 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S82494[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xperia S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82495[] = "Mozilla/5.0 (Linux; Android 3.1; GT-P7510 Build/HMJ37) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S82496[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC-T329d/1.15.1401.1) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82497[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T329d) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82498[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F120K/30a Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82499[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F120K/30b Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82500[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S82501[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Explorer_A310e) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S82502[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Explorer_A310e) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S82503[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-B5330 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S82504[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-B5330L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S82505[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Micromax A116) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S82506[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 OverDrive Media Console/3.0.1";
const char S82507[] = "Mozilla/5.0 (Linux; Android 4.1.2; A200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S82508[] = "SAMSUNG-GT-C3322i Opera/9.80 (J2ME/MIDP; Opera Mini/7.1.33886/31.1475; U; en) Presto/2.8.119 Version/11.10";
const char S82509[] = "MOT-W388/0.1.41 UP.Browser/6.3.0.6.c.24 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S82510[] = "SonyEricssonK790a/R1KG Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S82511[] = "Mozilla/5.0 (Linux; U; Android 2.2; CMLeclair1.3; es-es; Boston Build/ERE27; Orange_Boston Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S82512[] = "Mozilla/5.0 (Linux; U; Android 2.2; CMLeclair1.2; pt-pt; CMLBoston Build/ERE27; Orange_Boston Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S82513[] = "Mozilla/5.0 (Linux; U; Android 2.2; 2.1CMLMod1.5; es-es; CMLMod1.5 Build/ERE27; Orange_Boston Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S82514[] = "Mozilla/5.0 (Linux; U; Android 2.2; firetrap Sugar1.2; pt-pt; Sugar1.2 Build/ERE27; Orange_Boston Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S82515[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Boston Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82516[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Boston Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82517[] = "LG-KE990/v10a Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S82518[] = "LG/KE990/v10a Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S82519[] = "LG-KF510 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S82520[] = "Motorola-w385 Obigo/Q04C1 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S82521[] = "SAMSUNG-GT-i8000/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S82522[] = "KDDI-SH38 UP.Browser/6.2_7.2.7.1.K.3.352 (GUI) MMP/2.0";
const char S82523[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; Win 9x 4.90; .NET CLR 1.1.4322; MSIECrawler)";
const char S82524[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; .NET CLR 1.1.4322; MSIECrawler)";
const char S82525[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; MSIECrawler)";
const char S82526[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; Win 9x 4.90; MSIECrawler)";
const char S82527[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; MSIECrawler);c2VvLWNyYXdsZXI=";
const char S82528[] = "PPC; 240x320; HTC_P3450/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S82529[] = "Nokia6300/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6300/UC Browser8.0.3.107/69/444";
const char S82530[] = "Safari/7534.53.10 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookPro8%2C1)";
const char S82531[] = "Safari/7534.53.10 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookAir4%2C1)";
const char S82532[] = "Safari/7534.53.10 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacPro3%2C1)";
const char S82533[] = "Safari/7534.55.3 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (iMac8%2C1)";
const char S82534[] = "Safari/7534.55.3 CFNetwork/520.3.2 Darwin/11.3.0 (i386) (MacBook4%2C1)";
const char S82535[] = "Safari/7534.53.10 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (Macmini5%2C1)";
const char S82536[] = "iTunes/9.2.1 (Macintosh; PPC Mac OS X 10.4.11) AppleWebKit/533.19.4";
const char S82537[] = "iTunes/9.2.1 (Macintosh; Intel Mac OS X 10.4.11) AppleWebKit/533.19.4";
const char S82538[] = "Mozilla/5.0 (Series40; Nokia2700c-2/09.97; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.4.0.34.8";
const char S82539[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID2 Global Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82540[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID2 Global Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82541[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; MZ606 Build/I.7.1-34) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82542[] = "Mozilla/5.0 (Linux; U; Android 3.0UI; en-in; 3G7307 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82543[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Lenovo A750 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82544[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo A750 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S82545[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A750 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82546[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PG86100 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82547[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCEVO3D Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82548[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; vivo V2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82549[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; vivo V2 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S82550[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Mobiistar_Touch_Lai_5.0 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82551[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; CloudfoneThrill Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82552[] = "Spice Mi-285 Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Spice Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82553[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82554[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82555[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.22 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/535.22 Midori/0.4";
const char S82556[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en-us) Midori/0.4";
const char S82557[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.6 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/537.6 Midori/\"0\".\"4\"";
const char S82558[] = "Mozilla/5.0 (Java; U; en-us; gt-s5233a) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile";
const char S82559[] = "Mozilla/5.0 (Java; U; fa; gt-s5233a) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile";
const char S82560[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.4.2.X (Second Life Release; default skin) Safari/533.3";
const char S82561[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; MT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S82562[] = "Opera/9.80 (X11; FreeBSD 8.1-RELEASE i386; Edition Next) Presto/2.12.388 Version/12.10";
const char S82563[] = "Mozilla/5.0 (compatible; U; Chumby; Linux) Flash Lite 3.1.7";
const char S82564[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; N-06D Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82565[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E140L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82566[] = "Mozilla/5.0 (X11; U; Linux i686; en-AU; rv:1.7) Gecko/X Firefox/0.9.3 (Ubuntu)";
const char S82567[] = "JUC (Linux; U; 4.1.1; zh-cn; GT-I9300; 720*1280) UCWEB8.7.4.225/145/800";
const char S82568[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9300) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S82569[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S82570[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I415 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82571[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Mobile Safari/535.19 Silk-Accelerated=true";
const char S82572[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar Touch Kem 462 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82573[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LU6200 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82574[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A760S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82575[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GEM7020 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82576[] = "Mozilla/5.0 (Linux; Android 4.0.4; Torque Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82577[] = "Mozilla/5.0 (Linux; Android 4.0.3; A101 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S82578[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82579[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Scroll Excel 3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82580[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; Lenovo P770 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33482";
const char S82581[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MITO A222 Build/MITO-A222-V00.00.01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82582[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I997 Build/JDQ39; Carbon-CARBON-JB-v1.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82583[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I997 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82584[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I997 Build/JDQ39; Carbon-Carbon-JB-v1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82585[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROIDZ Force Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82586[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E971 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82587[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; WALTON_Primo-N1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82588[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/9A405 Safari/7534.48.3";
const char S82589[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; MITOA800 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82590[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S5282) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S82591[] = "Mozilla/5.0 (Linux; Android 4.2.2; BN Nook HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S82592[] = "Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S82593[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N8000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S82594[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993D Build/ICECREAM) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S82595[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993 Build/ICECREAM) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S82596[] = "Mozilla/5.0 (Linux; Android 4.1.2; Micromax A115 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S82597[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5312 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S82598[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5312B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S82599[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8500T/1.0; U; Bada/1.0; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WVGA SMM-MMS/1.2.0 OPN-B";
const char S82600[] = "ZTE-F101(RDS)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S82601[] = "ZTE-F101(Optus) NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S82602[] = "ZTE-F101(Tigo)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S82603[] = "ZTE-G-X760/X760_Z10NL_GD_B07A202 Obigo/Q03C";
const char S82604[] = "ZTE-G-X760-vegas-orange/X760_Z15ES_FS_B07A104_ORANGE Obigo/Q03C";
const char S82605[] = "ZTE-G-X760/X760_Z10UK_GD_B07A201 Obigo/Q03C";
const char S82606[] = "ZTE-G-X760-vegas-orange/X760_Z15UK_FS_B07A105_ORANGE Obigo/Q03C";
const char S82607[] = "ZTE-G-X760-vegas-orange/X760_Z15UK_FS_B07A106_ORANGE Obigo/Q03C";
const char S82608[] = "ZTE-G-X760/X760_Z10UK_GD_B07A202 Obigo/Q03C";
const char S82609[] = "SAMSUNG-GT-I6712/1.0 SHP/VPP/R5 Dolfin/2.0 NexPlayer/3.0 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B";
const char S82610[] = "LG-GW382f/V10d Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S82611[] = "Opera/9.80 (S60; SymbOS; Opera Tablet/9174; U; en) Presto/2.7.81 Version/10.5";
const char S82612[] = "SAMSUNG-GT-I9100/100.40102";
const char S82613[] = "Mozilla/5.0 (Linux; Android 4.0.3; VIGO10 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S82614[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; da-dk; EN-TAB410 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82615[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9003 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82616[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9003 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82617[] = "MQQBrowser/2.7 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3";
const char S82618[] = "MQQBrowser/2.7 Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3";
const char S82619[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82620[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E16i Build/2.1.1.A.0.53) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S82621[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MID9106CM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82622[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Scroll Excel 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82623[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung gt-c3303i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile, SAMSUNG-GT-C3303i/C3303iDDKF2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S82624[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung gt-c3303i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S82625[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S82626[] = "Nokia300/5.0 (07.44) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia300) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S82627[] = "Mozilla/5.0 (Linux; Android 4.1.2; AMID-9743G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S82628[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One S Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82629[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One S Build/JDQ39E; LiquidSmooth-Liquid-JB-v2.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82630[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S82631[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5830 Build/FROYO) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S82632[] = "Mozilla/5.0 (Linux; Android 4.1.2; MZ601 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S82633[] = "Mozilla/5.0 (Linux; Android 4.1.1; MZ601 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S82634[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A820L Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82635[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GIONEE_Ctrl_V4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82636[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AX5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82637[] = "Mozilla/5.0 (Linux; Android 4.1.2; A27 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82638[] = "Mozilla/5.0 (Linux; Android 4.0.3; M532 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S82639[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E430 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S82640[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; JY-G2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82641[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; C1504 Build/11.3.A.0.47) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S82642[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xolo_X900 Build/AZ510_ICS_02.36I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S82643[] = "Mozilla/5.0 (Linux; Android 4.1.1; ZTE V970 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82644[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZTE V970 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82645[] = "Mozilla/5.0 (Linux; Android 4.1.2; SOL21 Build/9.1.D.0.395) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82646[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A820L Build/HOANG MINH H?U_VEGAVIETTEAM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82647[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A820L Build/HOA?NG MINH H??rU_VEGAVIETTEAM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82648[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S82649[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI C8812E Build/HuaweiC8812E) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S82650[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S82651[] = "sprd-Masstel-M11/1.0 Linux/2.6.35.7 Android/2.3.5 Release/03.21.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S82652[] = "sprd-Masstel-M11/1.0 Linux/2.6.35.7 Android/2.3.5 Release/04.03.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S82653[] = "sprd-Masstel-M11/1.0 Linux/2.6.35.7 Android/2.3.5 Release/05.16.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S82654[] = "Mozilla/5.0 (Linux; Android 4.0.4; A8 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S82655[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S82656[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P990 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S82657[] = "Mozilla/5.0 (Linux; Android 4.1.2; Archos 80 Xenon Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S82658[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S82659[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Mobile Safari/535.19 Silk-Accelerated=false";
const char S82660[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.3; WOW64; Trident/6.0; Touch)";
const char S82661[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.3; Win64; x64; Trident/6.0; Touch)";
const char S82662[] = "Mozilla/5.0 (Linux; Android 4.2.1; Celkon A119Q Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S82663[] = "Mozilla/5.0 (Linux; Android 4.1.1; U7015 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S82664[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 7_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293";
const char S82665[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 7_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S82666[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 7_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7";
const char S82667[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) HPiPAQ610/1.0 (PPC; 240x320)";
const char S82668[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) HPiPAQ610/1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320)";
const char S82669[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en; LG-P993 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S82670[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; LG-P993 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S82671[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; LG-P993 Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S82672[] = "Mozilla/5.0(Linux;U;Android2.2.2;Zh_cn;LG-P350;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355";
const char S82673[] = "JUC(Linux;U;Android2.2.2;Zh_cn;LG-P350;240*320;)UCWEB7.8.0.95/139/444";
const char S82674[] = "Nokia5233/UC Browser7.6.1.82/50/351";
const char S82675[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Dell Streak Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S82676[] = "NokiaC6-01/UC Browser8.0.4.121/50/352/UCWEB";
const char S82677[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; pl; rv:1.9.0.14; onet) Gecko/X Firefox/3.0.14 Flock/2.5.3";
const char S82678[] = "Mozilla/5.0 (Windows NT 5.0; rv:9.0.1) Gecko/X Firefox/9.0.1 SeaMonkey/2.6.1";
const char S82679[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; bq Verne Plus Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82680[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; bq Verne Plus 3G Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82681[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LNV-Lenovo_A68e/Lenovo A68e_S011_X; 480*320; CTC/2.0) Wangel/23";
const char S82682[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo A68e Build/Lenovo_A68e_S016_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82683[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo A68e Build/Lenovo_A68e_S018_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82684[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo A68e Build/Lenovo_A68e_S019_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82685[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; TAB-PROTAB2.4XL Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82686[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03H; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82687[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82688[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s8530) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S82689[] = "Opera/9.80 (Android 2.1-update1; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10";
const char S82690[] = "Mozilla/5.0 (Linux; Android 4.0.3; MIDC010PR001 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S82691[] = "NokiaC2-06/2.0 (07.48) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-06) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile";
const char S82692[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S82693[] = "Mozilla/5.0 (Linux; Android 4.0.4; A211 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S82694[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; BB EH7LE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82695[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; BB_EH7LE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82696[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s3850) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S82697[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; Onyx Calypso 9.7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82698[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EVO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S82699[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S82700[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MT11i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S82701[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT11i Build/4.1.B.0.587) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S82702[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ar-eg; S18 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82703[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; S18 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S82704[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; OmegaS200 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82705[] = "OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SYMPHONY_W90 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82706[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T769 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82707[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-T769 Build/JZO54K; LiquidSmooth-LiquidAllStar) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82708[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; IQ443 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82709[] = "Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7210supernova) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S82710[] = "Mozilla/5.0 (Linux; Android 4.1.2; AMID-9743G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S82711[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC PH39100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S82712[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZP950+ Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82713[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZP950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82714[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZP950H Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82715[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST23i Build/11.0.A.5.5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S82716[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-S6312 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S82717[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6312 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S82718[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; P-03E Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82719[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Razr HD Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82720[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Droid Razr HD Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82721[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC Incredible S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S82722[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A1+) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S82723[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5300 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S82724[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S82725[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; LG-E400) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S82726[] = "Mozilla/5.0 (Linux; Android 4.2.1; E3 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S82727[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A87) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S82728[] = "Mozilla/5.0 (Linux; Android 4.3; SPH-L720 Build/JWR66V.S014) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S82729[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S82730[] = "Mozilla/5.0 (Linux; Android 4.2.1; H120 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S82731[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-03E Build/10.1.E.0.305) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S82732[] = "SAMSUNG-SGH-T429/T429UVHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S82733[] = "SonyEricssonK600i/R2BA Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S82734[] = "SonyEricssonK600c/R1K Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S82735[] = "SAMSUNG-GT-E2230M/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S82736[] = "SAMSUNG-GT-E2230M/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S82737[] = "SonyEricssonZ780i/R3DA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.2";
const char S82738[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1";
const char S82739[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1";
const char S82740[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1";
const char S82741[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1";
const char S82742[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.7) Gecko/X Firefox/3.0.7 CometBird/3.0.7";
const char S82743[] = "Mozilla/5.0 (Maemo; Linux armv7l; rv:10.0a1) Gecko/X Firefox/10.0a1 Fennec/10.0a1";
const char S82744[] = "Mozilla/5.0 (Maemo; Linux armv7l; rv:10.0.2) Gecko/X Firefox/10.0.2 Fennec/10.0.2";
const char S82745[] = "Mozilla/5.0 (Maemo; Linux armv7l; rv:10.0.1) Gecko/X Firefox/10.0.1 Fennec/10.0.1";
const char S82746[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:10.0.1) Gecko/X Thunderbird/10.0.1";
const char S82747[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PC36100 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82748[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PC36100 Build/MIUIANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82749[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC EVO Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82750[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Evo 4G Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82751[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EVO Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82752[] = "SAMSUNG-GT-S5270K/S5270KDDKK1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S82753[] = "Mozilla/5.0 (Windows NT 4.0; rv:9.0.1) Gecko/X Firefox/9.0.1";
const char S82754[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en) AppleWebKit/522.12.1 (KHTML, like Gecko) Version/3.0.1 Safari/522.12.2";
const char S82755[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TeamDRH ICS for GTablet Build/IML77; CyanogenMod-TeamDRH) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82756[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-b7722) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S82757[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.1.1m";
const char S82758[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.1.2m";
const char S82759[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.1.3m";
const char S82760[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82761[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-L710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SPH-L710, 34, 1.6.0)";
const char S82762[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SAMSUNG-SGH-I747, 42, 1.9.1)";
const char S82763[] = "Mozilla/5.0 (Linux; Android 4.0.4; H9500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82764[] = "Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Safari/535.19 Silk-Accelerated=false";
const char S82765[] = "Mozilla/5.0 (Linux; Android 4.0.3; PMP3370B Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S82766[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5333) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S82767[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Ubuntu/11.10 Chromium/23.0.1271.64 Chrome/23.0.1271.64 Safari/537.11";
const char S82768[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.11 (KHTML, like Gecko) Ubuntu/11.10 Chromium/23.0.1271.64 Chrome/23.0.1271.64 Safari/537.11";
const char S82769[] = "NOKIA-Lumia 800/UCWEB2.7.0.223/47/355";
const char S82770[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC_Sensation_XE_with_Beats_Audio_Z715e; 540*960) UCWEB7.9.0.94/139/352";
const char S82771[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC_Sensation_XE_with_Beats_Audio_Z715e; 540*960) UCWEB7.9.0.94/139/444";
const char S82772[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC_Sensation_XE_with_Beats_Audio_Z715e; 540*960) UCWEB7.9.4.145/139/800";
const char S82773[] = "Opera/9.80 (Windows NT 5.0) Presto/2.12.388 Version/12.13";
const char S82774[] = "Nokia300/5.0 (07.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia300) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S82775[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian73-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S82776[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE73) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S82777[] = "Mozilla/5.0 (Linux; Android 4.0.4; SO-01E Build/9.0.G.1.108) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82778[] = "Mozilla/5.0 (Linux; Android 4.0.4; IQ440 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82779[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA6710s) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S82780[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A2) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S82781[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F240K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82782[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7000) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S82783[] = "Mozilla/5.0 (Linux; Android 4.0.3; IM-A830L Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82784[] = "Mozilla/5.0 (Linux; Android 4.0.3; IM-A830S Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82785[] = "Mozilla/5.0 (Linux; Android 4.0.3; IM-A830K Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82786[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P765 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S82787[] = "Mozilla/5.0 (Linux; Android 4.1.2; Amazon Kindle Fire Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S82788[] = "Mozilla/5.0 (Linux; Android 4.0.3; IM-A830L Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S82789[] = "Mozilla/5.0 (Linux; Android 4.0.3; IM-A830S Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S82790[] = "Mozilla/5.0 (Linux; Android 4.0.3; SKY-A830 Build/LTE) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S82791[] = "Mozilla/5.0 (Linux; Android 4.0.3; IM-A830K Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S82792[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A87 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S82793[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; ST25i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S82794[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S82795[] = "Mozilla/5.0 (Linux; Android 4.2.1; S5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82796[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S82797[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC_Amaze_4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S82798[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Amaze 4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S82799[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L720 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SPH-L720, 47, 1.9.7)";
const char S82800[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A35 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S82801[] = "Mozilla/5.0 (Linux; Android 4.1.2; SOL22 Build/10.2.F.3.43) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S82802[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-H Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S82803[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I997 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S82804[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xolo_X900 Build/AZ510_ICS_02.36I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S82805[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Rhyme S510b Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S82806[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xolo_X900 Build/AZ510_ICS_02.36I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S82807[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S82808[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S82809[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S82810[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S82811[] = "Mozilla/5.0 (Linux; Android 4.3; SM-N900V Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S82812[] = "Mozilla/5.0 (Linux; Android 4.3; SM-N900W8 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S82813[] = "Mozilla/5.0 (Linux; Android 4.3; SM-N900P Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S82814[] = "Mozilla/5.0 (Linux; Android 4.3; AOSP on geeb_att_us Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S82815[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S82816[] = "SAMSUNG-E1310M/E1310MXEIE2 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S82817[] = "SAMSUNG-E1310E/E1310EXBIE1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S82818[] = "Vodafone/1.0/SAMSUNG-E1310B/BUIE1/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S82819[] = "Vodafone/1.0/SAMSUNG-E1310B/AMIE1/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S82820[] = "SAMSUNG-GT-i8000Orange/NXXJE3 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S82821[] = "SAMSUNG-GT-i8000/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S82822[] = "DoCoMo/2.0 SH06A3(c500;TB;W24H14)";
const char S82823[] = "HTC_Touch_Diamond2_T5353 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S82824[] = "HTC_Touch_Diamond2_T5353 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95/31/352";
const char S82825[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.0.10) Gecko/X Thunderbird/1.5.0.10";
const char S82826[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; NX-A7500 Build/I700T_P4_11A_TSNE_FT_R_0_01_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82827[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; id-id; NX-A7500 Build/I700T_P7_T01_TSNE_FT_R_0_06_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82828[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; NX-A7500 Build/I700T_P4_11A_TSNE_FT_R_0_05_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82829[] = "SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonj108i/UC Browser7.9.0.102/69/352";
const char S82830[] = "Mozilla/5.0 (Series40; Nokia303/13.44; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S82831[] = "Mozilla/5.0 (X11; U; Linux i686; Ubuntu 7.04; de-CH; rv:1.8.1.5) Gecko/X Firefox/2.0.0.5";
const char S82832[] = "Opera/9.40 (Windows NT 5.1; U; pl) Version/10.9";
const char S82833[] = "Opera/9.40 (Windows NT 5.1; U; pl) Presto/2.4.15 Version/10.10";
const char S82834[] = "Opera/9.45 (Windows; U; Windows NT 5.1; ru; rv:1.8.1.68)";
const char S82835[] = "Mozilla/5.0 (Linux; U; Android 2.2; nl-nl; DIT7000 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82836[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; T7006 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82837[] = "Opera/9.80 (Android 4.0.3; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.8.149 Version/11.10";
const char S82838[] = "Opera/9.80 (Android 4.0.4; Linux; Opera Tablet/ADR-X; U; en) Presto/2.8.149 Version/11.10";
const char S82839[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; COUPLE Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82840[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S82841[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S82842[] = "Mozilla/5.0 (Linux; Android 4.0.4; T15A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S82843[] = "Mozilla/5.0 (Linux; Android 4.0.3; T15A Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S82844[] = "JUC (Linux; U; 4.0.3; zh-cn; GT-P5100; 800*1232) UCWEB7.9.0.94/139/352";
const char S82845[] = "Mozilla/5.0 (Linux; U; Android 3.8; es-es; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82846[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokian8-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile  3gpp-gba";
const char S82847[] = "NokiaN8-00/UCWEB8.8.0.212/51/800";
const char S82848[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AllviewSpeedDuo Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82849[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-17 12:06, -0500, us";
const char S82850[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; AigoPad M60 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82851[] = "Mozilla/5.0 (Linux; Android 4.0.4; AD683G Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82852[] = "Mozilla/5.0 (Linux; Android 4.1.2; JY-G3 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82853[] = "Mozilla/5.0 (Linux; Android 4.0.4; A89 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S82854[] = "HTCT327t_TD/1.0 Android/4.0 release/2012 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S82855[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-B5512 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S82856[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; ALCATEL ONE TOUCH 918N Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S82857[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; ST18i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S82858[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-B5330 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S82859[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A500 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S82860[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5670 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S82861[] = "Mozilla/5.0 (Linux; Android 4.0.4; A89 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82862[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Cloudfone Excite 400d Build/FSV04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82863[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E450 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82864[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E450f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S82865[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Sensation Z710e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S82866[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Sensation 4G Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S82867[] = "Mozilla/5.0 (Linux; Android 4.1.2; Archos 50 Platinum Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S82868[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; LG-P705 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S82869[] = "SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj105i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S82870[] = "Mozilla/5.0 (Linux; Android 4.2.2; MB860 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S82871[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5570) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S82872[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; Andromax-c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82873[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC7-00) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S82874[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S7562) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S82875[] = "Mozilla/5.0 (Series40; NokiaX3-02/07.16; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S82876[] = "Mozilla/5.0 (Series40; NokiaX3-02.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S82877[] = "PANTECH-C790/JAUS08312009 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S82878[] = "SEC-SGHE840-ORANGE/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S82879[] = "SEC-SGHE840/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S82880[] = "LG-GB220/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S82881[] = "KWC-M2000/AAN1004 UP.Browser/7.2.7.2.530 (GUI) MMP/2.0";
const char S82882[] = "KWC-M2000/AAN1002 UP.Browser/7.2.7.2.530 (GUI) MMP/2.0";
const char S82883[] = "KWC-M2000/ZQ1000 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0";
const char S82884[] = "KWC-M2000/ZQ1003 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0";
const char S82885[] = "LG/KC910/v1.0";
const char S82886[] = "KDDI-KC3O UP.Browser/6.2.0.15.1.1.1.2 (GUI) MMP/2.0";
const char S82887[] = "KDDI-KC3Z UP.Browser/6.2.0.16.1 (GUI) MMP/2.0";
const char S82888[] = "KDDI-SN3P UP.Browser/6.2_7.2.7.1.K.5.207 (GUI) MMP/2.0";
const char S82889[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Kindle Fire Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Mobile Maxthon";
const char S82890[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Kindle Fire Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82891[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Kindle Fire Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Mobile Maxthon Mobile Maxthon";
const char S82892[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; ) AppleWebKit/534.12 (KHTML, like Gecko) Maxthon/3.0 Safari/534.12";
const char S82893[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12";
const char S82894[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1";
const char S82895[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Stainless/0.7.5 like Version/4.0 Safari/530.17";
const char S82896[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Stainless/0.7.5 like Version/4.0 Safari/530.17";
const char S82897[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Stainless/0.7.5 like Version/4.0 Safari/530.17";
const char S82898[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; SH8188U Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82899[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SH8188U Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82900[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:6.0.2) Gecko/X Firefox/6.0.2 AlexaToolbar/alxf-2.15";
const char S82901[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; fr-fr; AN7BG2DT Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82902[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82903[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; M1 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S82904[] = "Mozilla/5.0 (Linux; Android 4.0.3; M886 Build/HuaweiM886) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82905[] = "cPro.iDevice [Mozilla/5.0 (Macintosh; Intel Mac OS X 10) Gecko/X Firefox/13]";
const char S82906[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S82907[] = "Mozilla/5.0 (Java; U; en-us; nokia6233) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile";
const char S82908[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Desire V Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S82909[] = "Nokia112/2.0 (03.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia112) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S82910[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PG06100 Build/GRI40; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82911[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MT25i Build/4.1.B.0.631) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S82912[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo S686 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82913[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo S686 Build/IMM76I)";
const char S82914[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo S686 Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S82915[] = "JUC (Linux; U; 2.3.6; zh-cn; GT-B5512; 240*320) UCWEB7.9.0.94/139/444";
const char S82916[] = "MQQBrowser/3.1/Adr (Linux; U; 2.3.6; zh-cn; Lenovo A60+ Build/A60+_S312_X;320*480)";
const char S82917[] = "MQQBrowser/3.1/Adr (Linux; U; 2.3.6; zh-cn; Lenovo A60+ Build/A60+_S314_X;320*480)";
const char S82918[] = "MQQBrowser/3.1/Adr (Linux; U; 2.3.6; zh-cn; Lenovo A60+ Build/A60+_S319_X;320*480)";
const char S82919[] = "MQQBrowser/3.1/Adr (Linux; U; 2.3.6; zh-cn; Lenovo A60+ Build/A60+_S315_X;320*480)";
const char S82920[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; POV_TV-HDMI-KB-01 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82921[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; TECNO_N3) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S82922[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; FPT Tablet III Build/ARES_00.01.61.H1127) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82923[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One X Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82924[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7501 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S82925[] = "Mozilla/5.0 (Linux; Android 4.0.3; Lenovo S880 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S82926[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; HTC Wildfire Build/FRG83D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S82927[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; HTC Desire X Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S82928[] = "MicromaxX259/Q03C MAUI-Browser Profile/MIDP-2.0Configuration/CLDC-1.1";
const char S82929[] = "Micromax X328/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S82930[] = "Micromax X282/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S82931[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Mobiistar Touch S30 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82932[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-; Mobiistar Touch S30 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82933[] = "LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-kp500 teleca) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S82934[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S82935[] = "Mozilla/5.0 (Linux; Android 3.2; GT-P7510 Build/HTJ85B) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S82936[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Karbonn_A2+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S82937[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Filla Sapphire 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82938[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID1065 Build/ICS.MID1065.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S82939[] = "LAVA.KKT40s.Maui Browser";
const char S82940[] = "Mozilla/5.0 (Linux; Android 4.2.2; MZ601 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S82941[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S82942[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S82943[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P925 Build/ICS) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S82944[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGP321 Build/10.1.1.A.1.307) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S82945[] = "NokiaX2-01/5.0 (08.71) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311";
const char S82946[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo K860 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S82947[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9260 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S82948[] = "Mozilla/5.0 (iPad; U; CPU OS 7_0 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S82949[] = "Mozilla/5.0 (iPad; U; CPU OS 7_0 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5";
const char S82950[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari/6533.18.5 iLunascape/3212";
const char S82951[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Safari/6533.18.5 iLunascape/3212";
const char S82952[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-; U8220 Build/CRB17) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S82953[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; U8220 Build/CRB17) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S82954[] = "SAMSUNG-SGH-A886/A886UXIJ9 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S82955[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; CSL-MI350 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82956[] = "KDDI-SN3O UP.Browser/6.2_7.2.7.1.K.5.207 (GUI) MMP/2.0";
const char S82957[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.9)";
const char S82958[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/13.0.782.112)";
const char S82959[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/13.0.782.215) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.215 Safari/535.1";
const char S82960[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/13.0.782.107)";
const char S82961[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/13.0.782.112) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1";
const char S82962[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/13.0.782.220) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1";
const char S82963[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/13.0.782.112) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1";
const char S82964[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/13.0.782.107; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; InfoPath.2; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; MS-RTC LM 8; .NET CLR 3.0.30729)";
const char S82965[] = "NOKIAN79/UC Browser8.0.4.121/28/444/UCWEB";
const char S82966[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Radar C110e)";
const char S82967[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Radar 4G)";
const char S82968[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Radar C110e; 1.08.161.01)";
const char S82969[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Radar; Orange)";
const char S82970[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Radar C110e; 1.08.164.02)";
const char S82971[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Radar C110e; 2.01.164.03)";
const char S82972[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Radar)";
const char S82973[] = "nokia5800d-1/UC Browser8.0.3.107/69/444  3gpp-gba";
const char S82974[] = "SAMSUNG-GT-S5260/1.0 SHP/VPP/R5 Dolfin/2.0 NexPlayer/3.0 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B NetFront/3.5";
const char S82975[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S82976[] = "JUC (Linux; U; 2.3.4; zh-cn; HTC_EVO_3D_X515m; 540*960) UCWEB7.9.3.103/139/800";
const char S82977[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC_EVO_3D_X515m; 540*960) UCWEB7.9.0.94/139/352";
const char S82978[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC_EVO_3D_X515m; 540*960) UCWEB7.9.0.94/139/444";
const char S82979[] = "JUC (Linux; U; 2.3.4; zh-cn; HTC_EVO_3D_X515m; 540*960) UCWEB7.9.4.145/139/800";
const char S82980[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC_EVO_3D_X515m; 540*960) UCWEB7.9.0.94/139/355";
const char S82981[] = "JUC (Linux; U; 2.3.4; zh-cn; HTC_EVO_3D_X515m; 540*960) UCWEB7.9.0.94/139/352";
const char S82982[] = "LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-kp500 teleca) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S82983[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE51)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S82984[] = "JUC(Linux;U;Android4.0.4;Zh_cn;LT18i;480*854;)UCWEB7.8.0.95/139/444";
const char S82985[] = "JUC(Linux;U;Android2.3.7;Zh_cn;ST25i;480*854;)UCWEB7.8.0.95/139/444";
const char S82986[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:15.0) Gecko/X Firefox/15.0.1 SeaMonkey/2.12.1";
const char S82987[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ja-jp; Panasonic SV-ME1000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82988[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Panasonic UN-W700 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S82989[] = "Nokia3110c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia3110c) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S82990[] = "Mozilla/5.0 (Linux; Android 4.0.4; IQ441 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82991[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S82992[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S6010 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S82993[] = "NokiaC3-01/5.0 (06.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac3-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S82994[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AN10BG3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S82995[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S82996[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1SC Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S82997[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NWZ-Z1000Series Build/E200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82998[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NWZ-Z1000Series Build/E201) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S82999[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S83000[] = "Mozilla/5.0 (Linux; Android 4.0.4; A90S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83001[] = "Mozilla/5.0 (Linux; Android 4.1.1; M532 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S83002[] = "Mozilla/5.0 (compatible; MSIE 8.0; Win95; Trident/4.0)";
const char S83003[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_Desire_V) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S83004[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_Desire_VC) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S83005[] = "Mozilla/5.0 (Linux; Android 4.1.1; Penta WS707C Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S83006[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P990 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83007[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83008[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83009[] = "Mozilla/5.0 (Linux; Android 4.0.4; CINK KING Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83010[] = "Mozilla/5.0 (Linux; Android 4.0.4; MI-ONE Plus Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83011[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; IMO S79 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83012[] = "MQQBrowser/2.7 Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/7534.48.3";
const char S83013[] = "MQQBrowser/2.7 Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3";
const char S83014[] = "MQQBrowser/2.7 Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3";
const char S83015[] = "OneBrowser/3.1 (Nokia206/03.59)";
const char S83016[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST26i Build/11.0.A.7.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S83017[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST26i Build/11.0.A.3.28) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S83018[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-US; GT-P1000 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S83019[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; Nexus 7 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S83020[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F240K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83021[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F240L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83022[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST18i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83023[] = "Mozilla/5.0 (Linux; Android 2.3.6; SAMSUNG-SGH-I777 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S83024[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83025[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9000 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83026[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233s) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S83027[] = "UCWEB/2.0 (Java; U; MIDP-2.0; vi; gt-s5233s) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S83028[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI Y300-0000 Build/HuaweiY300-0000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83029[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; HUAWEI Y300-0000 Build/Y300-0000V100R001C17B172;480*800)";
const char S83030[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; HUAWEI Y300-0000 Build/Y300-0000V100R001C17B170;480*800)";
const char S83031[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E971 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83032[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5570I Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S83033[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5570 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S83034[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5233) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba";
const char S83035[] = "Mozilla/5.0 (Linux; Android 4.1.2; 201M Build/9.8.2Q-34_SMJ-102) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83036[] = "Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S83037[] = "Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S83038[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) SP; 240x320; HTC_S710/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S83039[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) HTCS710;Smartphone;240x320";
const char S83040[] = "SonyEricssonW595s/R3EG Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 model-orange JavaPlatform/JP-8.3.3";
const char S83041[] = "SAMSUNG-GT-B7300/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S83042[] = "SonyEricssonM1i/R1BA Browser/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S83043[] = "nokiae63-1/UC Browser8.0.3.107/69/444";
const char S83044[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; U8150 Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83045[] = "Mozilla/4.0 (MobilePhone SCP-5500/US/1.0) NetFront/3.0 MMP/2.0 FAKE (compatible; Googlebot/2.1; http://www.google.com/bot.html)";
const char S83046[] = "Mozilla/4.0 (MobilePhone SCP-5500/US/1.0) NetFront/3.0 MMP/2.0 (compatible; Googlebot/2.1; http://www.google.com/bot.html)";
const char S83047[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; zh-CN) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19.1";
const char S83048[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T679 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83049[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; el-gr; NovaPad 80 C124 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83050[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P350 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S83051[] = "MQQBrowser/2.6 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3";
const char S83052[] = "MQQBrowser/2.6 Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3";
const char S83053[] = "Mozilla/5.0 (Linux; Android 4.0.3; Karbonn_A18 Build/KarbonnA18) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83054[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P935 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83055[] = "LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-kp500 teleca) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S83056[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s5263) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S83057[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-za; Titan Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83058[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; m865 Build/TITAN PRIME; TITAN PRIME) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83059[] = "JUC (Linux; U; 4.0.4; zh-cn; HTC_One_X; 720*1184) UCWEB7.9.0.94/139/352";
const char S83060[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC_One_X; 720*1184) UCWEB7.9.0.94/139/352";
const char S83061[] = "JUC (Linux; U; 4.0.4; zh-cn; HTC_One_X; 720*1184) UCWEB7.9.0.94/139/444";
const char S83062[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC_One_X; 720*1184) UCWEB7.9.0.94/139/444";
const char S83063[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-N7000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83064[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83065[] = "Mozilla/5.0 (Linux; Android 4.0.4; TOUCH 97 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S83066[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) Sleipnir/2.8.1";
const char S83067[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; .NET CLR 1.1.4322; Sleipnir/2.8.5)";
const char S83068[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; FDM) Sleipnir/2.8.4";
const char S83069[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; .NET CLR 1.1.4322) Sleipnir/2.8.3";
const char S83070[] = "Mozilla/5.0 (Linux; Android 4.0.3; X506 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83071[] = "JUC (Linux; U; 2.3.6; zh-cn; GT-I8160; 480*800) UCWEB7.9.0.94/139/444";
const char S83072[] = "LG-GW300/V100 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-gw300) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S83073[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; i88 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83074[] = "Mozilla/5.0 (Linux; Android 4.0.4; C1605 Build/11.1.A.0.64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83075[] = "Mozilla/5.0 (Linux; Android 4.0.4; U7015 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83076[] = "Mozilla/5.0 (Linux; Android 4.0.4; U7011 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83077[] = "Mozilla/5.0 (Linux; Android 4.0.4; IQ441 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83078[] = "LG-GW305/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gw305) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S83079[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-P7500 Build/JOP40D)";
const char S83080[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-P7500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 GSA/2.2.10.X";
const char S83081[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7500 Build/JOP40D) SPG/v1.0.3";
const char S83082[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; GT-P7500 Build/JOP40G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S83083[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83084[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P7500 Build/JDQ39E)";
const char S83085[] = "SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw20) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S83086[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F240S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83087[] = "Mozilla/5.0 (Linux; Android 4.1.2; SBM203SH Build/S0018) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83088[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-c3303i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S83089[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-c3303i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile, SAMSUNG-GT-C3303i/C3303iDDKF2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S83090[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; Infinity HD2+ Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83091[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; TouchWizMOD_HD2 Build/TouchWizMOD_HD2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83092[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; ITOUCH HD2S Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83093[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAP4500TDUO Build/PrestigioPAP4500TDUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83094[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAP4500DUO Build/PrestigioPAP4500DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83095[] = "Mozilla/5.0 (Linux; Android 4.2.2; Xperia S Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83096[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P1000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S83097[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S83098[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989D Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S83099[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; WX435 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83100[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; WX435 Build/IMM76I; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83101[] = "Mozilla/5.0 (compatible; MSIE 7.0; Win3.11; Trident/3.0)";
const char S83102[] = "Mozilla/5.0 (Linux; Android 4.0.4; N850L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83103[] = "Mozilla/5.0 (Linux; Android 4.0.4; N850 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83104[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/537.4 Maxthon";
const char S83105[] = "Mozilla/5.0 (Series40; NokiaC2-05/08.79; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S83106[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I8150 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S83107[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire V Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S83108[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire VC Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S83109[] = "Mozilla/5.0 (Linux; Android 4.2.1; Lenovo K900_ROW Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S83110[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S83111[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST18i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S83112[] = "Mozilla/5.0 (Linux; Android 4.0.3; NS-13T001 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Safari/537.36";
const char S83113[] = "Mozilla/5.0 (Linux; Android 4.0.4; GSmart GS202+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83114[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-S7270 Build/JDQ39)";
const char S83115[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-S7270 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83116[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5312 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83117[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5312B Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83118[] = "Mozilla/5.0 (Linux; Android 4.0.4; Connect-3G Build/HCL) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S83119[] = "Mozilla/5.0 (Linux; Android 4.0.4; Y3 Build/HCL) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S83120[] = "Mozilla/5.0 (Linux; Android 4.2.2; IdeaTab A3000-F Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S83121[] = "Mozilla/5.0 (Linux; Android 4.2.2; IdeaTab A3000-H Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S83122[] = "Mozilla/5.0 (Linux; Android 4.0.4; ISW13HT Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83123[] = "PPC; 240x320; HTC_TyTN/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S83124[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) PPC; 240x320; HTC_TyTN\\1.0 Profile\\MIDP-2.0 Configuration\\CLDC-1.1";
const char S83125[] = "HTC_TyTN Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S83126[] = "Nokia7070d-2/2.0 (06.20) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S83127[] = "LG-KP265 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S83128[] = "LGE-VX9900/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S83129[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.11.12388.201";
const char S83130[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1 QQBrowser/6.11.11981.201";
const char S83131[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0) QQBrowser/6.11.12388.201";
const char S83132[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.11.12388.201";
const char S83133[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0) QQBrowser/6.11.12388.201";
const char S83134[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; QQDownload 679; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C) QQBrowser/6.11.12135.201";
const char S83135[] = "Mozilla/5.0 (Android; Tablet; rv:12.0) Gecko/12.0 Firefox/12.0 Fennec/12.0";
const char S83136[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) SEOChat::Bot v1.1";
const char S83137[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; ZoomSpider.net bot; .NET CLR 1.1.4322)";
const char S83138[] = "MOT-A-D5/00.02 UP.Browser/7.0.2.2.c.1.126 (GUI) MMP/2.0";
const char S83139[] = "MOT-A-3D/01.04 UP.Browser/7.0.2.2.c.1.108 (GUI) MMP/2.0";
const char S83140[] = "MOT-A-C8/00.04 UP.Browser/7.0.2.2.c.1.126 (GUI) MMP/2.0";
const char S83141[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.11) Gecko/X Ubuntu/9.10 (karmic) Firefox/3.0.11";
const char S83142[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.0.4) Gecko/X Firefox/1.5.0.4 Flock/0.7.0.17.1";
const char S83143[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows ME)";
const char S83144[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN79)/UC Browser8.5.0.183/28/444/UCWEB Mobile";
const char S83145[] = "i-mobile S 301T   Browser/Opera  Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; th-TH) Presto/2.5.28 Version/10.10";
const char S83146[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-SU640 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83147[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U18GT-H Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83148[] = "iBrowser/2.7 (NokiaC5-03;SymbianOS/9.1 Series60/3.0)";
const char S83149[] = "UCWEB/2.0(Java; U; MIDP-2.0; fa; samsung-gt-s5253) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S83150[] = "SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonj20i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile";
const char S83151[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia603) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/444/UCWEB Mobile";
const char S83152[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Sen S3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83153[] = "Mozilla/5.0 (Linux; Android 4.0.4; SH631W Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83154[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; connspeed_AS5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83155[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxq5) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S83156[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxq5) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S83157[] = "JUC (Linux; U; 2.3.6; zh-cn; GT-S6500D; 320*480) UCWEB7.9.0.94/139/444";
const char S83158[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; D709 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83159[] = "Nokia7230/5.0 (10.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia7230) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S83160[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; LT18i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S83161[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; LG-P936/V20c Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83162[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; LG-P936 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83163[] = "TECHAIN50_11B_HW (MRE\\3.0.00(2500) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) Masscom_P19i_T716V_M00_S03_X Release/2012.11.16 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 vi-VN";
const char S83164[] = "TECHAIN50_11B_HW (MRE\\3.0.00(2500) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) Masscom_P19i_T716V_M00_S03_X Release/2012.11.29 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 vi-VN";
const char S83165[] = "TECHAIN50_11B_HW (MRE\\3.0.00(2500) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) Masscom_P19i_T716V_M00_S04_X Release/2013.01.04 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 vi-VN";
const char S83166[] = "Mozilla/5.0 (Linux; Android 4.0.4; SIMM-X720 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83167[] = "Mozilla/5.0 (Linux; Android 4.0.4; X720 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S83168[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22 Sleipnir/4.0.1";
const char S83169[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P925 Build/ICS) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83170[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S83171[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P6200 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S83172[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A44) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S83173[] = "Mozilla/5.0 (Linux; Android 4.1.1; M532 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S83174[] = "Mozilla/5.0 (Linux; Android 4.2.2; Optimus G Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83175[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120K Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83176[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83177[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120L Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83178[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-D700 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83179[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-D700 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83180[] = "Mozilla/5.0 (Linux; Android 4.1.2; 201M Build/9.8.2Q-34_SMJ-102) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83181[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E16i Build/2.1.1.A.0.53) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S83182[] = "Mozilla/5.0 (compatible; MSIE 4.0; Windows NT 6.0; Trident/2.0)";
const char S83183[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EVO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S83184[] = "MQQBrowser/43 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3";
const char S83185[] = "MQQBrowser/43 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S83186[] = "SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj105i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S83187[] = "Mozilla/5.0 (Linux; Android 4.1.2; G2S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83188[] = "Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S83189[] = "Mozilla/5.0 (Linux; Android 4.1.2; Micromax A115 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83190[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_918N) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S83191[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_918N) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S83192[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 UCBrowser/9.2.0.312 Mobile";
const char S83193[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 UCBrowser/9.2.0.312 Mobile";
const char S83194[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5235) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S83195[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A1+) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S83196[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P1000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S83197[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ605 Build/I.7.1-39.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S83198[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717R Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83199[] = "SAMSUNG-SGH-F330-TEN/F330AFHB1 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S83200[] = "SAMSUNG-SGH-F330/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S83201[] = "Nokia2660/2.0 (05.20) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S83202[] = "Nokia2660_CMCC/2.0 (05.25) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S83203[] = "SAMSUNG-SGH-T229/T229UVHC4 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S83204[] = "SHARP-STX2/MC.40 POLARIS/6.15 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S83205[] = "SAMSUNG_GT-I9008_TD/1.0 OPhone/2.0 (Linux; Android 2.1) Release/5.1.2010 Browser/WAP2.0 (AppleWebKit/530.17) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S83206[] = "NokiaX3-00/5.0 (08.54) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax3-00/UC Browser7.9.0.102/69/352";
const char S83207[] = "KDDI-SH3K UP.Browser/6.2_7.2.7.1.K.5.207 (GUI) MMP/2.0";
const char S83208[] = "MicromaxX266/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S83209[] = "DoCoMo/2.0 P10A(c100;TB;W24H15)";
const char S83210[] = "yacybot (freeworld/global; amd64 Windows Server 2008 R2 6.1; java 1.6.0_29; Europe/de) http://yacy.net/bot.html";
const char S83211[] = "feedfinder/1.371 Python-urllib/1.17 +http://www.aaronsw.com/2002/feedfinder/";
const char S83212[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; ZTE V875 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83213[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9308 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83214[] = "Mozilla/5.0 (Symbian3; U; en-us; Nokia603) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/51/352/UCWEB Mobile";
const char S83215[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83216[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; POLY PAD 9716HD Build/ICS.g12refM1005TR.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83217[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZP300+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83218[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Alpha GTR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83219[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.6 Mobile Safari/535.19 Silk-Accelerated=false";
const char S83220[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NEO_N002 Build/NeoNEO_N002) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83221[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; NEO_N002 Build/NeoNEO_N002)";
const char S83222[] = "Spice M5910";
const char S83223[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HS-EG950 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83224[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; HS-EG950 Build/IMM76I)";
const char S83225[] = "Mozilla/5.0 (Linux; Android 4.1.1; Lenovo P770 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83226[] = "Mozilla/5.0 (Linux; Android 4.0.4; WT19i Build/4.1.B.0.587) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83227[] = "Mozilla/5.0 (Linux; Android 4.0.4; N9500 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83228[] = "UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 510) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile";
const char S83229[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S5301) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S83230[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T699 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83231[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; WALTON Primo-D1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83232[] = "Mozilla/5.0 (Linux; Android 4.0.4; KPN Smart 300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83233[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P700 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83234[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR6425LVW Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S83235[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-lu6200 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83236[] = "Mozilla/5.0 (Linux; Android 4.1.2; SOL22 Build/10.2.F.3.43) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83237[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328w Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S83238[] = "Opera/9.80 (Android 4.1.2; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.7.81 Version/11.00";
const char S83239[] = "Opera/9.80 (Android 4.1.1; Linux; Opera Mobi/ADR-X; U; en) Presto/2.7.81 Version/11.00";
const char S83240[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100G Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S83241[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S83242[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S83243[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I605 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SCH-I605, 42, 1.9.1)";
const char S83244[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC PH39100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S83245[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5770D Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S83246[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83247[] = "Mozilla/5.0 (Linux; Android 4.1.2; Titanium S1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83248[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848i Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83249[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848 Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83250[] = "Mozilla/5.0 (Linux; Android 4.0.4; A21 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83251[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cynus T2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83252[] = "Mozilla/5.0 (Linux; Android 4.1.1; M97 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S83253[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Karbonn_A2+) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S83254[] = "Mozilla/5.0 (Linux; Android 4.1.2; ME371MG Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S83255[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; HTC One Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S83256[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; A210 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S83257[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (ZmEufAdvOJ6+jpzS29ugr8sbtxJwbNEwTvj+UjrNdn8=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83258[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (VAGxVr3aE18uG9UGNhxfYQKo8MfRq62rziav6QZT9EQ=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83259[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (oVaX3xu8lS6+dxqSTxJyCyJ+2By0urQwPAPZDhxZX8Q=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83260[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (J0uUUVb1lF9anOEASNkv+dk9J+E03avcau9WUGCrjqA=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83261[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (ekLz1M/BXVWgI0h9U0OQLG8D1dQihLpRf5wxUVxHV/g=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83262[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (seO21gpOtp8uTe8EjZClilM5rgzyrlWqV92C7FPS+dA=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83263[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (jwD2y6YFkctosT5FELDx07yy1LfQUWIvcaRaHNV882U=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83264[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (/fePx6eQbNOJc18VKF+tUCcbSw7dqUiyUeAWMr1Oe2E=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83265[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (kSamzH/nGjnKspyMoq+MPnFVFkSdI+FUde17GDT6ReQ=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83266[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (QOeq2ctm9O3zMuuvTkrEUt0UCx8jzBmtsr8W7cqsYZ8=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83267[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (WKKNEiGPExM+NZzLS3xwOi2UAiCvCuH+EZWhrH36F8w=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83268[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (Y+zhCxawR0sYLoffHaQoaUxQwBeXpqpbhXezjKAZqVc=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83269[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (HRfVdPGb2DXDoQv7j0jiZ7kumjupqYEZuuv+uUjYaXY=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83270[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (aQw7//dlIYE6iKTsPsmlK510hNgZeoA88gEzyy33PN8=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83271[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (i4DjSJEBwdLJDDtV37i4IpGJXfG7l1GcnLuPNHSOyBQ=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83272[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (6PNbJqYPVfeNgfBReG2mSHUyWmtwktcmfHhI0M3t0O4=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83273[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (GZTg5JsTFXdJASbcqxtJ4kc227JaAEjA9qCSMp3LpEM=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83274[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (EnLPfK3gBP87LW9Hd6JIkOZwJI246mpl+xkiH+y8vqA=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83275[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (H3Md1XIJnTbE+bu2rA68p1zm1RLiEeeTztPpy0c371E=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83276[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (YJxLL7kX0A4Or/0URhJklkH+e4kL6zwjoChsP/AYteg=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83277[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (toiqvGz7d4VX6nP0TXtuSfxAmWEMBplIGl6f4IRb+PA=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83278[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (ZzcZ9AYfEOcJ9lhSYF9ZdA3wxO7FaGsplEU1IZWRH/Q=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83279[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (OGxyVI07puJEIltprVOx+nKUtFBW0X1nDLz6GIOU7TM=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83280[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (2+5mbYbRy06RbV9AqKfDxK1OxQTSP/3ogwQnfKgRpe8=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83281[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (dPBmLrfYvwIweCRKYVi6ehCtlkvkj5qzhl3wX2U7t2E=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83282[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (o8u5P/w47yiT9EAgMQTYbosdzSOTsLwW+Jodpmz0mIg=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83283[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (wwV7b+ehRKit4Z/Dcpf5Iyt+XWId08eJmLj0BeAM1ik=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83284[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (iGHKmJCWM+S9hhlEe6w0Freohib2zGpsMp1/y8TZBkw=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83285[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (/HDwNKUP3bXS6+ZAwbPIIV7okuayQT2skTvAork+ngM=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83286[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (sPHxgDKpO8H7xz7UJx+863KHtKvj0XxHSH5oNWkOt+Y=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83287[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 () Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83288[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (XPokjOYxFHsUdRJ3O2EuYjHwfjohEnE9z01PffnbHEI=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2";
const char S83289[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Micromax_A72) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S83290[] = "Mozilla/5.0 (Linux; Android 4.1.1; ZTE V970 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83291[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Alltel HTC Snap";
const char S83292[] = "HTC_Snap Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S83293[] = "HTC_Snap_S522 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S83294[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HPiPAQDataMessenger/1.00.00 Browser/VF-Browser/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S83295[] = "HPiPAQDataMessenger/1.0/Mozilla/4.0  (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC;320x240";
const char S83296[] = "LG-A140/1.0/EUN_NK MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.1.582 (GUI)";
const char S83297[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) garmin-asus-Nuvifone-M10/2.0";
const char S83298[] = "SEC-SGHJ600/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S83299[] = "SEC-SGHJ600E-ORANGE/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S83300[] = "SEC-SGHJ600E/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S83301[] = "NokiaX3-00/5.0 (08.54) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax3-00/UC Browser8.0.3.107/69/444";
const char S83302[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;OMNIA7;7.10;8107)";
const char S83303[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;OMNIA7;7.10;7740)";
const char S83304[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;OMNIA7;7.10;7720)";
const char S83305[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;OMNIA7;7.10;8773)";
const char S83306[] = "Nokia500/UC Browser8.0.4.121/50/444/UCWEB";
const char S83307[] = "Opera/9.80 (X11; SunOS i86pc; U; en) Presto/2.2.15 Version/10.10";
const char S83308[] = "Opera/9.80 (X11; SunOS i86pc; U; en) Presto/2.2.15 Version/10.11";
const char S83309[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i937;7.10;7720)";
const char S83310[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i937;7.10;8773)";
const char S83311[] = "Opera/9.50 (J2ME/MIDP; Opera Mini/2.99/27.1659; U; en)";
const char S83312[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; NVSBL VORTEX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83313[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaC7-00)/UC Browser8.5.0.183/50/355/UCWEB Mobile";
const char S83314[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE52-1)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S83315[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; generic Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S83316[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:15.0) Gecko/X Thunderbird/15.0.1";
const char S83317[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S83318[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Desire C Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S83319[] = "Mozilla/5.0 (Java; U; en-us; nokia6120c) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S83320[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83321[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s5620) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S83322[] = "Mozilla/5.0 (Linux; Android 4.0.4; IQ440 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83323[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ZEN U4 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83324[] = "Nokia6500s-1/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6500s-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S83325[] = "ZTE U950_TD/1.0 Linux/3.1.10 Android/4.0 Release/8.1.2012 Browser/AppleWebKit534.30";
const char S83326[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; MT11i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S83327[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAN85) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S83328[] = "Opera/9.80 (Android 2.1; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S83329[] = "Opera/9.80 (Android 2.1-update1; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S83330[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; TECNO N3 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S83331[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; Micromax_P350) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S83332[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Xperia S Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83333[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Xperia S Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/nozomi";
const char S83334[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T769 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83335[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I997 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83336[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Celkon_A67 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83337[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A760S Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83338[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A760S Build/CM10-RUCRO; CyanogenMod-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83339[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Iris_349) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S83340[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Iris_349+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S83341[] = "Mozilla/5.0 (Linux; Android 4.1.1; U9200 Build/HuaweiU9200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83342[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N8013 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S83343[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PHICOMM i600 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83344[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; i600 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83345[] = "SCH-W279/ POLARIS/6.2 (GUI; compatible; UP.Browser)";
const char S83346[] = "Mozilla/5.0 (Linux; Android 4.0.4; Connect-3G Build/HCL) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S83347[] = "Mozilla/5.0 (Linux; Android 4.0.4; Y3 Build/HCL) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S83348[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; iBuddyConnect?_3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83349[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZP910 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83350[] = "Mozilla/5.0 (Linux; Android 4.0.4; ISW13HT Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83351[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-E975 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83352[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S83353[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT270 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S83354[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9502 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83355[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A789 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83356[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5570) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S83357[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-E400 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S83358[] = "Mozilla/5.0 (Linux; Android 4.1.1; P774A Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S83359[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A820L Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83360[] = "Mozilla/5.0 (Linux; Android 4.2.2; BN Nook HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S83361[] = "Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S83362[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S83363[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i2) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S83364[] = "Mozilla/5.0 (Linux; Android 4.1.1; BS471 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83365[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83366[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6313T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83367[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I757M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83368[] = "Mozilla/5.0 (Linux; Android 4.1.2; G2S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83369[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire HD Build/SVHD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83370[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-CN; X Build/HMX) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S83371[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; HUAWEI P6-T00 Build/HuaweiP6-T00) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S83372[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; P2 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S83373[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-CN; HUAWEI G610-U00 Build/HuaweiG610-U00) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S83374[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S83375[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-N7000 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S83376[] = "BlackBerry8320/4.5.0.81 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100 UP.Browser/5.0.3.3";
const char S83377[] = "LG9200/1.0 TELECA/Q7.2-1.1 MMP/2.0";
const char S83378[] = "HTC Streaming Player optus_au / 1.0 / htc_vivo / 2.3.3";
const char S83379[] = "HTC Streaming Player htc_wwe / 1.0 / htc_vivo / 2.3.3";
const char S83380[] = "HTC Streaming Player verizon_wwe / 1.0 / htc_vivow / 2.3.3";
const char S83381[] = "HTC Streaming Player verizon_wwe / 1.0 / htc_vivow / 2.3.4";
const char S83382[] = "ZTE-R54/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S83383[] = "ZTE-R54(South_Africa_MTN)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S83384[] = "NokiaE71/UC Browser7.7.1.88/28/444";
const char S83385[] = "nokiae71-1/UC Browser7.7.1.88/69/444";
const char S83386[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; hr-hr; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83387[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9001 Build/IMM76L)";
const char S83388[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-I9001 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83389[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-I9001 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83390[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-pk; DE88Plus Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83391[] = "Mozilla/5.0 (Linux; U; Android 2.2; in-id; DE88Plus Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83392[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; pl; rv:1.9.0.14; onet) Gecko/X Firefox/3.0.14 Flock/2.5.3";
const char S83393[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-GB; rv:0.9.4.1) Gecko/X Netscape6/6.2.3";
const char S83394[] = "Mozilla/5.0 (X11; Linux i686; rv:9.0) Gecko/X Thunderbird/9.0 Lightning/1.1.1";
const char S83395[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:10.0) Gecko/X Firefox/10.0 SeaMonkey/2.7 Lightning/1.2.1";
const char S83396[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:10.0.2) Gecko/X SeaMonkey/2.7.2";
const char S83397[] = "Flipboard/1.8.4/49,1.8.4,2012-05-11 16:30, gb";
const char S83398[] = "Flipboard/1.8.4/51,1.8.4,2012-05-16 03:48, gb";
const char S83399[] = "Flipboard/1.8.4/45,1.8.4,2012-05-09 22:44, gb";
const char S83400[] = "Flipboard/1.8.4/60,1.8.4,2012-05-24 22:32, sg";
const char S83401[] = "Flipboard/1.8.4/54,1.8.4,2012-05-16 21:29, us";
const char S83402[] = "Flipboard/1.8.4/55,1.8.4,2012-05-19 00:44, ca";
const char S83403[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:5.0) Gecko/X Firefox/5.0,gzip(gfe) (via translate.google.com)";
const char S83404[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; XT316 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83405[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3 UCBrowser8.3.1.173/42/355";
const char S83406[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3 UCBrowser8.3.1.173/42/355";
const char S83407[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I717 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83408[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I717 Build/IMM76I; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83409[] = "Mozilla/5.0 (Linux; Android 4.0.3; TAB464 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S83410[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB464 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S83411[] = "MicromaxX368/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S83412[] = "Micromax Q34/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S83413[] = "JUC (Linux; U; 4.0.3; zh-cn; N12; 480*764) UCWEB1.0.1.113/147/800";
const char S83414[] = "JUC (Linux; U; 4.0.4; zh-cn; P76a(K3G5); 800*444) UCWEB1.0.1.113/147/800";
const char S83415[] = "JUC (Linux; U; 4.0.4; zh-cn; CW-V17; 800*432) UCWEB1.0.1.113/147/800";
const char S83416[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10A403 Safari/7534.48.3 UCBrowser8.5.0.195/44/355";
const char S83417[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10A523 Safari/7534.48.3 UCBrowser8.5.0.195/44/355";
const char S83418[] = "Mozilla/5.0 (Java; U; en-us; nokiac7-00) UCBrowser8.4.0.159/69/352/UCWEB Mobile  3gpp-gba";
const char S83419[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I997 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83420[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.4.2.X (Second Life Release; default skin) Safari/533.3";
const char S83421[] = "Mozilla/5.0 (Windows; U; Win95; en-US; rv:1.7.2) Gecko/X Netscape/7.2 (ax)";
const char S83422[] = "Lenovo-A698t_TD/S100 Linux/3.0.13 Android/4.0 Release/01.05.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2";
const char S83423[] = "SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonw20) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S83424[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s3653) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S83425[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromax) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S83426[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromax) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S83427[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST23i Build/11.0.A.5.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S83428[] = "Micromax X269/Q03C MAUI-Browser Profile/MIDP-2.0Configuration/CLDC-1.1";
const char S83429[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; M1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83430[] = "Mozilla/5.0 (Linux;U;Android 2.3.5;zh-cn;GO M1 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83431[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; GO M1 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83432[] = "Mozilla/5.0 (Linux; Android 4.1.1; U9200 Build/HuaweiU9200) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83433[] = "Mozilla/5.0 (Linux; Android 4.1.1; X10.Dual+ Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S83434[] = "Mozilla/5.0 (Linux; Android 4.1.1; X10.Dual Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S83435[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; Brava DM-992 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83436[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8160 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83437[] = "AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b";
const char S83438[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PolyPad 1010IPS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83439[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; tablet Fnac 8 Build/1.0.4 X-11:52) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83440[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; tablet Fnac 8 Build/1.1.0 X-12:19) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83441[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; tablet Fnac 8 Build/1.0.1 X-10:20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83442[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Aqua Style Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83443[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-LS970 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83444[] = "Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300S Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83445[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S83446[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S83447[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S83448[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T699 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83449[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T699 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83450[] = "Mozilla/5.0 (Linux; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S83451[] = "Mozilla/5.0 (Linux; Android 4.0.3; Smartfren Tab 7 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S83452[] = "Mozilla/5.0 (Linux; Android 4.0.4; StarTab 715 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S83453[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P990 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83454[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-E970 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83455[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-H9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83456[] = "FPT-FPT-F2/1.0 Linux/2.6.35.7 Android/2.3.5 Release/03.30.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S83457[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; FPT-FPT-F2/1.0 Android/2.3.5 Release/05.23.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83458[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Videocon_A15) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S83459[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Videocon A27i Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S83460[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S83461[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S83462[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S83463[] = "MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U1 Build/HCL ME Tablet U1) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83464[] = "SAMSUNG GT-S3570 Opera/9.80 (J2ME/MIDP; Opera Mini/6.5.30607/30.3697; U; en) Presto/2.8.119 Version/11.10";
const char S83465[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax A110) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S83466[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7100) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S83467[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A919 Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83468[] = "Mozilla/5.0 (Linux; Android 4.1.1; Smartfren Tab 7 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S83469[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; M8 pro Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83470[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; KB901 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83471[] = "Opera/9.80 (BREW; Opera Mini/6.0.3/27.2338; U; en) Presto/2.8.119 240X400 Pantech CDM8999";
const char S83472[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9003 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S83473[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-B5330 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83474[] = "Mozilla/5.0 (Linux; Android 4.0.4; S100 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83475[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A919 Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83476[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5021E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83477[] = "Mozilla/5.0 (Linux; Android 4.2.1; i-mobile IQ X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83478[] = "MOT-A455/1.0 UP.Browser/7.2.7.5.646 (GUI) MMP/2.0";
const char S83479[] = "MOT-A455/00.72 UP.Browser/7.2.7.5.606 (GUI) MMP/2.0";
const char S83480[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; MB861 Build/Blur_Version.4.1.150.MB861.KT.en.KR Flex/P010) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83481[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; MB861 Build/OLYKT_U4_1.15.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83482[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; MB861 Build/OLYKT_U4_1.18.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83483[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; MB861 Build/Blur_Version.4.1.180.MB861.KT.en.KR Flex/P010) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83484[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; MB861 Build/OLYKT_U4_1.21.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83485[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; MB861 Build/Blur_Version.4.1.210.MB861.KT.en.KR Flex/P010) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83486[] = "SonyEricssonW20a/R7DA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2";
const char S83487[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) garmin-asus-Nuvifone-M10/1.0";
const char S83488[] = "Mozilla/5.0(Linux;U;Android2.1-update1;Zh_cn;X10i;480*854;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S83489[] = "JUC(Linux;U;Android2.1-update1;Zh_cn;X10i;480*854;)UCWEB7.8.0.95/139/444";
const char S83490[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SonyEricssonM1a/R1BA; Windows Phone 6.5; Profile/MIDP-2.1; Configuration/CLDC-1.1; Windows Phone 6.5.3.5)";
const char S83491[] = "Nokia5530/UCWEB7.9.0.94/50/999";
const char S83492[] = "Nokia5530/UC Browser7.9.0.102/50/355/UCWEB";
const char S83493[] = "MicromaxQ7/MIDP2.0/CLDC1.1/Screen-320X240";
const char S83494[] = "KDDI-TS3W UP.Browser/6.2_7.2.7.1.K.6.210 (GUI) MMP/2.0";
const char S83495[] = "Mozilla/5.0 (Windows NT 5.0; rv:8.0) Gecko/X Firefox/8.0,gzip(gfe) (via translate.google.com)";
const char S83496[] = "HD_mini_T5555 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S83497[] = "HTC_HD_mini_T5555 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S83498[] = "480x800 LGE VS750 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S83499[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:10.0.1) Gecko/X Firefox/10.0.1 slurp";
const char S83500[] = "Mozilla/5.0 (Windows NT 6.1; rv:10.0.2) Gecko/X Firefox/10.0.2 slurp";
const char S83501[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:10.0.2) Gecko/X Firefox/10.0.2 slurp slurp";
const char S83502[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:10.0) Gecko/X Firefox/10.0 slurp";
const char S83503[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19";
const char S83504[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Woxter Tablet PC 100CX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83505[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK BNTV250 Build/GINGERBREAD 1.4.3)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83506[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK BNTV250A Build/GINGERBREAD 1.4.3)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83507[] = "Mozilla/5.0 (Linux; Android 4.0.3; TAB-PROTAB2-IPS9 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S83508[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; Yummy Duo Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83509[] = "Mozilla/5.0 (Linux; Android 4.0.4; KM-E100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83510[] = "Mozilla/5.0 (Android; Tablet; rv:14.0) Gecko/14.0 Firefox/2.0.1.1";
const char S83511[] = "Mozilla/5.0 (Android; Tablet; rv:2.0.2.4) Gecko/2.0.2.4 Firefox/2.0.2.4";
const char S83512[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; tr-tr; Vodafone 958 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83513[] = "Mozilla/5.0 (Linux; Android 4.1.1; LG-P930 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83514[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P930 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83515[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; Micromax A87 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S83516[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6810 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S83517[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P6810 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S83518[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6810 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83519[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Rezound Build/IML74K; CyanogenMod-(/'o')/ _|____|_ 1.9.4 Linaro) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83520[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) Version/5 Safari/533.16";
const char S83521[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.8.1.6) Gecko/X Firefox/2.0 Wyzo/0.5.3";
const char S83522[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; one touch T20 Build/MAIN) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83523[] = "Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-50EX645; CC/USA; en) Presto/2.10.250 Version/11.60";
const char S83524[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA6710s) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S83525[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6200 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S83526[] = "OneBrowser/3.1 (Nokia2730c-1/10.45)";
const char S83527[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E270K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83528[] = "Mozilla/5.0 (Linux; Android 4.0.3; Marquis_MP977 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S83529[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8160 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83530[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-I8262D Build/JZO54K.I8262DZCAMC2;480*800)";
const char S83531[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-I8262D Build/JZO54K.I8262DZCAMA9;480*800)";
const char S83532[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-I8262D Build/JZO54K.I8262DZCAMB2;480*800)";
const char S83533[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83534[] = "Mozilla/5.0 (compatible; MSIE 3.0; Windows NT 6.2; Trident/4.0)";
const char S83535[] = "MQQBrowser/3.0/Adr (Linux; U; 4.0.4; zh-cn; HUAWEI U8825D Build/U8825DV100R001C17B951;480*800)";
const char S83536[] = "MQQBrowser/3.0/Adr (Linux; U; 4.0.4; zh-cn; HUAWEI U8825D Build/U8825DV100R001C17B935;480*800)";
const char S83537[] = "MQQBrowser/3.0/Adr (Linux; U; 4.0.4; zh-cn; HUAWEI U8825D Build/U8825DV100R001C17B956;480*800)";
const char S83538[] = "MQQBrowser/3.0/Adr (Linux; U; 4.0.4; zh-cn; HUAWEI U8825D Build/U8825DV100R001C17B933;480*800)";
const char S83539[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VERTU Ti Build/4.0.4.A.0.1.72) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83540[] = "Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_E10310 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S83541[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-LS970 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83542[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST27i Build/6.2.A.0.400) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S83543[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S83544[] = "Mozilla/5.0 (Linux; Android 4.0.4; WT19i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83545[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MB886 Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83546[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P714 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83547[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9003 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S83548[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; GT-I9003 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S83549[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9260 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83550[] = "Mozilla/5.0 (Linux; Android 4.0.4; KB901 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83551[] = "NokiaC3-01/5.0 (07.51) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac3-01) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S83552[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I897 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83553[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI MediaPad Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S83554[] = "Mozilla/5.0 (Linux; Android 4.0.3; MediaPad 7 Lite Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S83555[] = "Mozilla/5.0 (Linux; Android 4.2.1; A900 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83556[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6812 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S83557[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; MT11i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S83558[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S83559[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S83560[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC 802w Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83561[] = "Mozilla/5.0 (Series40; NokiaC2-01/11.40; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S83562[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X; zh-CN) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 UCBrowser/9.3.0.326 Mobile";
const char S83563[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X; zh-CN) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 UCBrowser/9.3.0.326 Mobile";
const char S83564[] = "SAMSUNG-SGH-F488/1.0 SHP/VPP/R5 NetFront/3.4 Qtv/5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S83565[] = "SEC-SGHM310L/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S83566[] = "SEC-SGHM310/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S83567[] = "SAMSUNG-SGH-T356-parrot/T356UWJG3 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S83568[] = "SAMSUNG-SGH-T356/T356TLJE4 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S83569[] = "Mozilla/4.0 (BREW 3.1.5; U; en-us; Samsung; PLS_M240; POLARIS/6.1/WAP) MMP/2.0 Configuration/CLDC-1.1";
const char S83570[] = "Mozilla/4.0 (BREW 3.1.5; U; en-us; Samsung; SPH_M240; POLARIS/6.1/WAP) MMP/2.0 Configuration/CLDC-1.1";
const char S83571[] = "SAMSUNG-SGH-A437/A437UCGDA Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S83572[] = "Mozilla/5.0 (Linux; U; Android 2.2; it-it; OliPad 100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83573[] = "Mozilla/5.0 (Linux; U; Android 2.2; it-; OliPad 100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S83574[] = "Mozilla/5.0 (Linux; U; Android 2.2; it-; OliPad 100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83575[] = "Mozilla/5.0 (Linux; U; Android 2.2; it-it; OliPad 100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S83576[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.0 Chrome/13.0.800.0 Safari/535.1";
const char S83577[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Iron/7.0.520.0 Chrome/7.0.520.0 Safari/534.7";
const char S83578[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Iron/7.0.520.0 Chrome/7.0.520.0 Safari/534.7";
const char S83579[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; SN10T1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83580[] = "Opera/9.80 (Windows NT 6.1; Opera Mobi/23731; U; en) Presto/2.9.201 Version/11.50";
const char S83581[] = "Mozilla/5.0 (Windows NT 4.10) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7";
const char S83582[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT862 Build/IML74K; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83583[] = "NokiaE71/UCWEB8.3.0.133/28/800";
const char S83584[] = "Mozilla/5.0 (Java; U; en-us; nokiae71-1) UCBrowser8.3.1.161/70/352/UCWEB Mobile";
const char S83585[] = "Mozilla/5.0 (S60V3; U; en-in; NokiaE72-1)/UC Browser8.4.0.159/28/352/UCWEB Mobile";
const char S83586[] = "NokiaE72-1/UCWEB8.4.0.150/28/999";
const char S83587[] = "MQQBrowser/2.5.1 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S83588[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-I9000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83589[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; SGH-I9000 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83590[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; W50 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83591[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; Fly_IQ275 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83592[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; IQ275 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83593[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S83594[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83595[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; A75 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S83596[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_ChaCha_A810e; 320*480) UCWEB7.9.0.94/139/352";
const char S83597[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_ChaCha_A810e; 320*480) UCWEB7.9.0.94/139/444";
const char S83598[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_ChaCha_A810e; 320*480) UCWEB7.9.0.94/139/352";
const char S83599[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A775C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83600[] = "Mozilla/5.0 (Windows NT 5.1; rv:7.0.1) Gecko/X Postbox/3.0.8";
const char S83601[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung gt-c3222) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S83602[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung gt-c3222) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile, SAMSUNG-GT-C3222/C3222DDJL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S83603[] = "contype chromeframe/24.0.1312.52";
const char S83604[] = "contype chromeframe/24.0.1312.56";
const char S83605[] = "contype chromeframe/24.0.1312.57";
const char S83606[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; ST15i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S83607[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; ST18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S83608[] = "Mozilla/5.0 (Linux; Android 4.0.4; HS-F1 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83609[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; Ezee'TAB901 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83610[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; AMAZE S200 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83611[] = "CoolPad8150D_CMCC_TD/1.0 Linux/3.0.8 Android/4.0 Release/10.15.2012 Browser/AppleWebkit534.3";
const char S83612[] = "Opera/9.80 (Linux mips; U; ; en) Presto/2.10.287 Version/12.00";
const char S83613[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Lenovo S720 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/33482";
const char S83614[] = "Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD+ Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S83615[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAC-70031 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83616[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S11t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83617[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C5503 Build/10.1.1.A.1.253) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83618[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C5503 Build/10.1.1.A.1.200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83619[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C5503 Build/10.1.1.A.1.210) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83620[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C5503 Build/10.1.1.A.1.191) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83621[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C5503 Build/10.1.1.A.0.72) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83622[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C5503 Build/10.1.1.A.1.89) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83623[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C5503 Build/10.1.1.A.1.259) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83624[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C5503 Build/10.1.1.A.1.164) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83625[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C5503 Build/LAGAN-JB-APPS-X-1000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83626[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; C5503 Build/10.1.1.A.1.310)";
const char S83627[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST23i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S83628[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-04E Build/10.1.1.D.0.179) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83629[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S83630[] = "Mozilla/5.0 (Linux; Android 4.0.3; Xtreamer_Mobile_AiKi Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83631[] = "Mozilla/5.0 (Linux; Android 4.0.3; Aiki-5 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83632[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T989 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SGH-T989, 42, 1.9.1)";
const char S83633[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T989 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SGH-T989, 42, 1.9.1)";
const char S83634[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9082 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S83635[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9082i Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S83636[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E988 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83637[] = "OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Symphony Xplorer W35 Build/UNKNOWN) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83638[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Mobile Safari/535.19 Silk-Accelerated=false";
const char S83639[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-E400 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S83640[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9300) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S83641[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT894 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83642[] = "Mozilla/5.0 (Linux; Android 4.1.1; Micromax A110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S83643[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LU6200 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83644[] = "Mozilla/5.0 (Linux; Android 4.1.1; ZTE V970 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83645[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-05D Build/9.1.C.0.475) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83646[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46HX750; CC/USA; en) Presto/2.10.250 Version/11.60";
const char S83647[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46EX650; CC/VNM; vi) Presto/2.10.250 Version/11.60";
const char S83648[] = "Opera/9.80 (Linux mips; Opera TV Store/4510; U; el) Presto/2.10.250 Version/11.60 Model/Sony-KDL-55HX750 SonyCEBrowser/1.0 (KDL-55HX750; CTV/PKG1.910EUA; GRC)";
const char S83649[] = "Opera/9.80 (Linux mips; U; Sony BRAVIA/PKG1.901GAA (KDL-46EX650,,en,MY); en) Presto/2.10.250 Version/11.60";
const char S83650[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL55HX850; CC/USA; en) Presto/2.10.250 Version/11.60";
const char S83651[] = "Opera/9.80 (Linux mips; Opera TV/19; U; pt-BR) Presto/2.9.201 Version/11.60";
const char S83652[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40HX853; CC/GBR; en) Presto/2.10.250 Version/11.60";
const char S83653[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A820L Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83654[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Micromax A110) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S83655[] = "LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gs290) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S83656[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC6435LVW Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/dlx";
const char S83657[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC6435LVW Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/dlx";
const char S83658[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC6435LVW Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/dlx";
const char S83659[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I997 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/infuse4g";
const char S83660[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I997 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/infuse4g";
const char S83661[] = "SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj20i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S83662[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S7562 Build/IMM76I) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S83663[] = "Mozilla/5.0 (Linux; Android 4.1.2; AT100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S83664[] = "Mozilla/5.0 (Linux; Android 4.0.4; KB901 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83665[] = "Mozilla/5.0 (Linux; Android 4.3; HTC6435LVW Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83666[] = "Nokia6086/2.0 (06.00) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S83667[] = "SAMSUNG-SGH-T456/T456UZJB4 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S83668[] = "BlackBerry8900/4.6.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100";
const char S83669[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 5.0; Avant Browser)";
const char S83670[] = "Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 5.0; Avant Browser)";
const char S83671[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.0; Avant Browser; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; Media Center PC 6.0; MS-RTC LM 8)";
const char S83672[] = "Micromax X450/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S83673[] = "Opera/9.81 (SaferSurf; de)";
const char S83674[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Novo7Basic Build/ICS_MR0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83675[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Novo7Basic Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83676[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SN10T1 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83677[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SN10T1 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83678[] = "Mozilla/5.0 (Java; U; en-us; nokian73-1) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S83679[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) rekonq/1.0 Safari/534.34";
const char S83680[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Sleipnir/3.7.3";
const char S83681[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4 Sleipnir/3.7.4";
const char S83682[] = "Mozilla/5.0 (Linux; Android 4.0.4; Rezound Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83683[] = "Mozilla/5.0 (Java; U; en-in; nokiac5-00.2) UCBrowser8.3.0.154/69/444/UCWEB Mobile  3gpp-gba";
const char S83684[] = "Mozilla/5.0 (Java; U; en-in; nokiac5-00) UCBrowser8.3.0.154/69/444/UCWEB Mobile  3gpp-gba";
const char S83685[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S83686[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; ST21i2 Build/11.0.A.1.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83687[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; ST21i Build/11.0.A.0.16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83688[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; TA7300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83689[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Ascend G300 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83690[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-T959 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83691[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T959 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83692[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A21 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S83693[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Karbonn A21 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S83694[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5380k) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S83695[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5380d) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S83696[] = "OneBrowser/Mini1.1 (Linux; U; en) Presto/2.8.119 Version/11.10";
const char S83697[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ZEN U5 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83698[] = "Mozilla/5.0 (Linux; Android 4.2.1; HTC Sensation Build/JOP40G) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83699[] = "Mozilla/5.0 (Linux; Android 4.2.2; Sensation Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83700[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Sensation Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83701[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Desire S Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83702[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Desire S Build/JellyFireBean v2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83703[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Desire S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83704[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MID9738G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83705[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ALLVIEW3CONECT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83706[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Lenovo A800 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/33482";
const char S83707[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAX5-01) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S83708[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83709[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83710[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83711[] = "Mozilla/5.0 (Linux; Android 4.0.4; KB901 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83712[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3653) U2/1.0.0 UCBrowser/9.0.0.295 U2/1.0.0 Mobile";
const char S83713[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASUS Tablet P1801-T Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S83714[] = "Nokia300/5.0 (07.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia300) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S83715[] = "Mozilla/5.0 (Linux; Android 4.1.2; Amazon Kindle Fire Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S83716[] = "Mozilla/5.0 (Linux; Android 4.0.4; HS-U950 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83717[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S83718[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S83719[] = "Mozilla/5.0 (Linux; Android 4.0.4; WT19i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S83720[] = "Mozilla/5.0 (Linux; Android 4.1.1; M040 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83721[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P880 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S83722[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; AS870 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83723[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; AS870 4G Build/JZO54K)";
const char S83724[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S5282) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S83725[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; STARXTREM Build/GM_ZTEV1.0.0B03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83726[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; STARXTREM Build/GM_ZTEV1.0.0B05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83727[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83728[] = "Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3120classic) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S83729[] = "Mozilla/5.0 (Linux; Android 4.2.2; MB860 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83730[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT11i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83731[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT26i Build/6.1.A.2.55) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S83732[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LT26i Build/6.1.A.2.45) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S83733[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-c6712) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S83734[] = "Mozilla/5.0 (Linux; Android 4.1.1; Orange Kivo Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83735[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F220K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83736[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S83737[] = "Nokia3711fold/5.0 (03.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S83738[] = "HTC_S730 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S83739[] = "ZTE-R109/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S83740[] = "DoCoMo/2.0 SH03B(c500;TB;W30H18)";
const char S83741[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; Lenovo 3GW100 Build/FRG83) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S83742[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; 3GW100 Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S83743[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; 3GW100 Build/FRG83) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S83744[] = "KDDI-KC3X UP.Browser/6.2_7.2.7.1.K.5.207 (GUI) MMP/2.0";
const char S83745[] = "Opera/9.80 (Windows NT 5.1; Opera Mobi/49; U; en) Presto/2.4.18 Version/10.00";
const char S83746[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Sensation 4G Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83747[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Sensation 4G with Beats Audio Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83748[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Sensation 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S83749[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S83750[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Adam Build/Beast2.2.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83751[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Adam Build/NIAX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83752[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; Adam Build/NIPX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83753[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Adam Build/Cromnia[Beta3]ClemsynKernel) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83754[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; PI39100)";
const char S83755[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; TITAN X310e)";
const char S83756[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S83757[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352";
const char S83758[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-mx; MID7108SC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83759[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MID7108SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83760[] = "Mozilla/5.0 (Linux; U; Android 4.0.5; es-es; MID7108SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83761[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I800 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S83762[] = "Mozilla/5.0 (Linux; Android 4.0.4; Turkcell Maxi Plus 5 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83763[] = "MyPhoneTW26-3D DUO/MTKRelease/2012/02/15Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0";
const char S83764[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83765[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; ST15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S83766[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT-AS45SE Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83767[] = "JUC(Linux;U;Android4.0.3;Zh_cn;GT-N7000;800*1280;)UCWEB7.8.0.95/139/444";
const char S83768[] = "JUC(Linux;U;Android4.0.4;Zh_cn;GT-N7000;800*1280;)UCWEB7.8.0.95/139/444";
const char S83769[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-P500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S83770[] = "Mozilla/5.0 (Java; U; en-us; sonyericssonwt13i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S83771[] = "Mozilla/5.0 (Java; U; fa; sonyericssonwt13i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S83772[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 BaDoinkDownloader_Android_2.0";
const char S83773[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 BaDoinkDownloader_Android_2.0";
const char S83774[] = "Nokia110/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia110) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S83775[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHT21 Build/SB221) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S83776[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT18i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S83777[] = "Mozilla/5.0 (Linux; Android 4.1.1; X909 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83778[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I577 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83779[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P940 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83780[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-KU5400 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83781[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID4 Build/6.7.2-180_DR4-16_M2-37) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83782[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5770D Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S83783[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; galaxy prime plus Build/1.0.459_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83784[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MPDC77 BT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83785[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; miTab NEO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83786[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S7500) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S83787[] = "Mozilla/5.0 (Linux; Android 4.2.2; MZ601 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S83788[] = "Mozilla/5.0 (Linux; Android 4.0.4; G3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83789[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T329w Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83790[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC_T329w Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83791[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A800 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83792[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83793[] = "Mozilla/5.0 (Linux; Android 2.3.4; HTC Glacier Build/GRJ22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S83794[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A9+ Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S83795[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A9+ Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S83796[] = "Mozilla/5.0 (Linux; Android 4.1.2; Archos 80 Xenon Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S83797[] = "Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6303classic) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S83798[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7511 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S83799[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-S7272 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83800[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5302) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S83801[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTL22 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83802[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E980 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S83803[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC_P3650) Opera 8.65 [sv]";
const char S83804[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC_P3650) Opera 8.65 [en]";
const char S83805[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC_P3650) Opera 8.65 [pt]";
const char S83806[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC_P3650) Opera 8.65 [ru]";
const char S83807[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC_P3650) Opera 8.65 [nl]";
const char S83808[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC_P3650) Opera 8.65 [es-es]";
const char S83809[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC_P3650) Opera 8.65 [de]";
const char S83810[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC_P3650) Opera 8.65 [zh-tw]";
const char S83811[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC_P3650) Opera 8.65 [it]";
const char S83812[] = "SAMSUNG-SGH-i200/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S83813[] = "SAMSUNG-SGH-i200-ORANGE/BVHE6 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S83814[] = "ZTE-F160(Zambia)/1.0 NetFront/3.4 QTV5.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S83815[] = "ZTE-F160(MTN)/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S83816[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.3.0.0 Chrome/17.0.963.46 Safari/535.11";
const char S83817[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.3.0.0 Chrome/17.0.963.46 Safari/535.11";
const char S83818[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE-T U880 Build/GWK74) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S83819[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE-T U880 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S83820[] = "Mozilla 5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE-T U880 Build GWK74) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S83821[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) 1Password/3.5.9/X (like Mobile/8C148 Safari/6533.18.5)";
const char S83822[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) 1Password/3.5.9/X (like Mobile/8C148 Safari/6533.18.5)";
const char S83823[] = "curl/7.16.4 (i386-apple-darwin9.0) libcurl/7.16.4 OpenSSL/0.9.7l zlib/1.2.3";
const char S83824[] = "curl/7.16.3 (powerpc-apple-darwin9.0) libcurl/7.16.3 OpenSSL/0.9.7l zlib/1.2.3";
const char S83825[] = "Windows Phone Search (Windows Phone OS 7.10;LG;LG-E900;7.10;7740)";
const char S83826[] = "Windows Phone Search (Windows Phone OS 7.10;LG;LG-E900h;7.10;7720)";
const char S83827[] = "Windows Phone Search (Windows Phone OS 7.10;LG;LG-E900;7.10;8107)";
const char S83828[] = "Windows Phone Search (Windows Phone OS 7.10;LG;LG-E900h;7.10;8107)";
const char S83829[] = "Windows Phone Search (Windows Phone OS 7.10;LG;LG-E900;7.10;7720)";
const char S83830[] = "Windows Phone Search (Windows Phone OS 7.10;LG;LG-E900;7.10;8773)";
const char S83831[] = "Windows Phone Search (Windows Phone OS 7.10;LG;LG-E900h;7.10;8773)";
const char S83832[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ro-ro; Allview_P2 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83833[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352";
const char S83834[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S83835[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S83836[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S83837[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-B5512 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S83838[] = "Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia7210supernova) UCBrowser8.4.0.159/70/444/UCWEB Mobile";
const char S83839[] = "Mozilla/5.0 (iPhone; CPU 6_1_3 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3";
const char S83840[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; generic Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S83841[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6200 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S83842[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6200L Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S83843[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.9) Gecko/X Firefox/3.0.9 Wyzo/3.0.3 FBSMTWB";
const char S83844[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.0.9) Gecko/X Firefox/3.0.9 Wyzo/3.0.3";
const char S83845[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) DV300 UP.Browser/7.2.7.2.559 (GUI)  MMP/2.0";
const char S83846[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) DV150 UP.Browser/7.2.7.2.559 (GUI)  MMP/2.0";
const char S83847[] = "Mozilla/5.0 (Linux; Android 4.0.3; ZP500 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83848[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZP500+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83849[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia701) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile";
const char S83850[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83851[] = "ZEN M8/1.0; Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S83852[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83853[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8812E Build/HuaweiC8812E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83854[] = "Mozilla/5.0 (Linux; Android 4.1.1; M040 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83855[] = "Dolphin http client/9.2.0(197) (Android)";
const char S83856[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S83857[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; HTC_Wildfire Build/FRG83D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S83858[] = "Mozilla/5.0 (Linux; Android 4.0.4; Andromax U Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83859[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.3.2.300";
const char S83860[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC Sensation XE Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83861[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Desire HD Build/JellyMIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S83862[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Desire HD Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S83863[] = "Mozilla/5.0 (Android 2.3.6; WOW64; rv:17.0) Gecko/X Firefox/17.0";
const char S83864[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCL21 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83865[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A820L Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83866[] = "Mozilla/5.0 (Linux; Android 4.0.4; A21 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83867[] = "Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A21 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S83868[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A26) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S83869[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; Micromax A110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300";
const char S83870[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T5-A Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83871[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T5-A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83872[] = "Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7210supernova) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S83873[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Micromax A72 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S83874[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; ST25i Build/6.2.A.0.400) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83875[] = "UCWEB/2.0 (iOS; U; iPh OS 5_1_1; zh-CN; iPh3,1) U2/1.0.0 UCBrowser/9.1.0.287 U2/1.0.0 Mobile";
const char S83876[] = "UCWEB/2.0 (iOS; U; iPh OS 5_1_1; zh-CN; iPh4,1) U2/1.0.0 UCBrowser/9.1.0.287 U2/1.0.0 Mobile";
const char S83877[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83878[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC 802w Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83879[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Xperia Arc S Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/anzu";
const char S83880[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Xperia Arc S Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83881[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Lenovo A830 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83882[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-tw; Lenovo A830 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83883[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; Lenovo A830 Build/JOP40D)";
const char S83884[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5620) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S83885[] = "UCWEB/2.0 (Linux; U; Adr 4.2.2; en-US; GT-I9500) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S83886[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6812 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83887[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6812B Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83888[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I337 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83889[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I337 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83890[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I337 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83891[] = "UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 710) U2/1.0.0 UCBrowser/3.1.1.343 U2/1.0.0 Mobile";
const char S83892[] = "Mozilla/5.0 (Linux; Android 4.1.1; ME172V Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S83893[] = "HTC Streaming Player htc_asia_wwe / 1.0 / htc_bravo / 2.2";
const char S83894[] = "HTC Streaming Player htc_wwe / 1.0 / htc_bravo / 2.2";
const char S83895[] = "HTC Streaming Player o2_de / 1.0 / htc_bravo / 2.2";
const char S83896[] = "HTC Streaming Player orange_uk / 1.0 / htc_bravo / 2.2";
const char S83897[] = "HTC Streaming Player tmo_uk / 1.0 / htc_bravo / 2.2";
const char S83898[] = "HTC Streaming Player orange_es / 1.0 / htc_bravo / 2.2";
const char S83899[] = "HTC Streaming Player telus_wwe / 1.0 / htc_bravo / 2.2";
const char S83900[] = "HTC Streaming Player h3g_uk / 1.0 / htc_bravo / 2.2";
const char S83901[] = "HTC Streaming Player virgin_mobile / 1.0 / htc_bravo / 2.2";
const char S83902[] = "HTC Streaming Player vodafone_uk / 1.0 / htc_bravo / 2.2";
const char S83903[] = "HTC Streaming Player htc_wwe / 1.0 / htc_bravo / 2.3.3";
const char S83904[] = "HTC Streaming Player htc_wwe / 1.0 / htc_bravo / 2.2.2";
const char S83905[] = "HTC Streaming Player telstra_wwe / 1.0 / htc_bravo / 2.2";
const char S83906[] = "HTC Streaming Player vodafone_uk / 1.0 / htc_bravo / 2.2.2";
const char S83907[] = "HTC Streaming Player vodafone_tr / 1.0 / htc_bravo / 2.2.2";
const char S83908[] = "Mozilla/5.0 (Linux; U; Android UbuntDroid-v4.5; en-us; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83909[] = "Mozilla/5.0 (Linux; U; Android UbuntDroid-v6; en-us; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83910[] = "Mozilla/5.0 (Linux; U; Android StockROM on Crack 1.2 (2.2.2 EC07); en-us; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83911[] = "Mozilla/5.0 (Linux; U; Android StockROM on Crack 1.1.2 (2.2.2 EC07); en-us; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83912[] = "Mozilla/5.0 (Linux; U; Android Ice Cream Froyo 1.2 (2.2.2 EC07); en-us; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83913[] = "Mozilla/5.0 (Linux; U; Android IcyPop1.0; en-us; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83914[] = "Mozilla/5.0 (Linux; U; Android 2.0.1; es-us; A853 Build/SHLA_U2_02.02.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S83915[] = "Mozilla/5.0 (Linux; U; Android 2.0; pt-br; A853 Build/SHLA_U2_01.03.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S83916[] = "Mozilla/5.0 (Linux; U; Android 2.0; es-us; A853 Build/SHLA_U2_01.03.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S83917[] = "Mozilla/5.0 (Linux; U; Android 2.0.1; es-us; A853 Build/SHLA_U2_02.01.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S83918[] = "LG-KP210 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S83919[] = "Mozilla/5.0 (Windows; U; Windows NT 4.0; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S83920[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.24 (KHTML, like Gecko) Comodo_Dragon/11.4.0.0 Chrome/11.0.696.71 Safari/534.24";
const char S83921[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Comodo_Dragon/11.4.0.0 Chrome/11.0.696.71 Safari/534.24";
const char S83922[] = "gt-s5230/UC Browser8.0.3.107/69/444";
const char S83923[] = "samsung-s5230/UC Browser8.0.3.107/69/352";
const char S83924[] = "gt-s5230/UC Browser8.0.3.107/70/352";
const char S83925[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID2 GLOBAL Build/S273; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83926[] = "Opera/9.80 (Android 2.2; Linux; Opera Tablet/ADR-X; U; de) Presto/2.8.149 Version/11.10";
const char S83927[] = "Opera/9.80 (Android 2.2.2; Linux; Opera Tablet/ADR-X; U; en-GB) Presto/2.8.149 Version/11.10";
const char S83928[] = "Opera/9.80 (Android 2.2.1; Linux; Opera Tablet/ADR-X; U; es-ES) Presto/2.8.149 Version/11.10";
const char S83929[] = "SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-in; sonyericssonj108i) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S83930[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE-T U960s Build/GWK74) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S83931[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE-T U960s Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S83932[] = "Mozilla 5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE-T U960s Build GWK74) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S83933[] = "Mozilla/5.0 (Linux; Android 4.1.1; Desire S Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83934[] = "Mozilla/5.0 (Linux; Android 4.1.2; Desire S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83935[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S83936[] = "MQQBrowser/2.6 (NokiaE5-00;SymbianOS/9.3 Series60/3.2)";
const char S83937[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.2.16pre) Gecko/X Prism zdesktop/7.2.2";
const char S83938[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo A580 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83939[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo A580 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S83940[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Symphony S101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83941[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; ST18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S83942[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S83943[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; ELEMENT10 1 Build/ICS.g08refem611.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S83944[] = "Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83945[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq50) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S83946[] = "Mozilla/5.0 (Linux; Android 4.1.1; SC-03E Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83947[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-S5360 Build/GINGERBREAD) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315";
const char S83948[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-S5360 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Mobile Safari/537.31 OPR/14.0.1074.54070";
const char S83949[] = "Mozilla/5.0 (Linux; Android 4.0.4; Telenor_Smart_HD Build/IceCreamSandwich) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83950[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I997 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83951[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-I997 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83952[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P940 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83953[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-KU5400 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83954[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT25i Build/4.1.B.0.631) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S83955[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7310 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S83956[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P7310 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S83957[] = "Mozilla/5.0 (Linux; Android 4.2; GT-P7310 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S83958[] = "Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76B) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229 Mobile Safari/537.4 pss-webkit-request";
const char S83959[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC J Z321e Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83960[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ608 Build/7.7.1-141-7-FLEM-UMTS-LA) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S83961[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ608-14) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S83962[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ThL W2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83963[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83964[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83965[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S83966[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S83967[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S83968[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S83969[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Lemon P9 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83970[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; CROSS A27 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S83971[] = "Mozilla/5.0 (Linux; Android 2.3.6; SGH-T769 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S83972[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZTE Blade G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S83973[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S83974[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo K860 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83975[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xperia S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83976[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-03E Build/10.1.E.0.269) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S83977[] = "Mozilla/5.0 (Linux; Android 4.0.4; A89 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S83978[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIANokia N81) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S83979[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIANokia N81 8GB) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S83980[] = "Mozilla/5.0 (Android; Mobile: SAMSUNG-SGH-I747; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S83981[] = "Mozilla/5.0 (Android; Mobile: SPH-L710; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S83982[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6802) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S83983[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; WT19i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S83984[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9502 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S83985[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530M Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S83986[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5770D Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S83987[] = "Nokia200/2.0 (11.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia200) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S83988[] = "ZTE-T2/1.0 ACS-NF/3.4 QTV2.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S83989[] = "MOT-R38.0/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0";
const char S83990[] = "NexianNX-G869/MTK Release/01.01.2011 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1";
const char S83991[] = "BlackBerry8900/5.2.0.104 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/159/444";
const char S83992[] = "Mozilla/5.0 (SaferSurf rv:10.0) Gecko/X Firefox/10.0";
const char S83993[] = "Mozilla/5.0 (compatible; rv:1.9b4) Gecko/X Firefox/10.0a4";
const char S83994[] = "SAMSUNG-GT-I8350/UCWEB2.2.0.157/47/355";
const char S83995[] = "NokiaC7-00/UCWEB8.3.0.133/51/999";
const char S83996[] = "Mozilla/5.0 (Java; U; en-in; nokiac7-00) UCBrowser8.3.0.154/69/444/UCWEB Mobile  3gpp-gba";
const char S83997[] = "Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU5)/UC Browser8.5.0.183/50/352/UCWEB Mobile";
const char S83998[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S83999[] = "Mozilla/5.0 (Windows NT 5.1; rv:14.0) Gecko/X Firefox/14.0.3.0 IceDragon/14.0.3.0";
const char S84000[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U8818 Build/HuaweiU8818) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84001[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; HWA580 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84002[] = "SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-in; sonyericssonj105i) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S84003[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax P500(Funbook) Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S84004[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5263) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile";
const char S84005[] = "Instagram 3.1.2 (iPod touch; iPhone OS 6.0; en_US) AppleWebKit/420+";
const char S84006[] = "Instagram 3.1.2 (iPod touch; iPhone OS 6.0.1; en_US) AppleWebKit/420+";
const char S84007[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Explay_Advance Build/ExplayExplay_Advance) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84008[] = "SpiceM5665/MTK Release/01.01.2012 Browser/wap2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84009[] = "Mozilla/5.0 (Linux; Android 4.0.4; ThL W3 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84010[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-LU6500 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84011[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84012[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84013[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Philips W336 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84014[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; SCH-I930)";
const char S84015[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-ph; Cloudfone Excite 350i Build/FSV02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84016[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17 Sleipnir/3.9.1";
const char S84017[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17 Sleipnir/3.9.1";
const char S84018[] = "Mozilla/5.0 (Linux; U; Android 4.0; zh-cn; K-Touch W719 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84019[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; K-Touch W719 Build/IMM76I)";
const char S84020[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-P500 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S84021[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 80XSK Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S84022[] = "OneBrowser/3.1 (NokiaX3-00/11.00)";
const char S84023[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N8013 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S84024[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5830 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S84025[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; ZP810 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84026[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-03E Build/10.1.E.0.269) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S84027[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-04E Build/10.1.1.D.0.179) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84028[] = "Mozilla/5.0 (Linux; Android 4.1.1; IRIS_501 Build/LAVAIRIS501) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84029[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S84030[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P705 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84031[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P708g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84032[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; TurboX-G310   Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84033[] = "Mozilla/5.0 (Linux; Android 4.1.1; Orange Zali Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84034[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT22i Build/6.2.A.1.100) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S84035[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT22i Build/6.2.A.0.400) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S84036[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; HTC One Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S84037[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S84038[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S84039[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-CN; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S84040[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:22.0) Gecko/X Firefox/22.0 TenFourFox/G5";
const char S84041[] = "Mozilla/5.0 (Linux; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S84042[] = "Mozilla/5.0 (Linux; Android 4.1.1; Novo 10 Hero QuadCore Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S84043[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Rhyme S510b Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84044[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.2; en-gb; GT-I9500 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S84045[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E425 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S84046[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E425f Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S84047[] = "Nokia210/2.0 (04.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia210) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S84048[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.4.2.34167 (Firestorm-Release; firestorm skin) Safari/533.3";
const char S84049[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Sensation Z710e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S84050[] = "Mozilla/5.0 (Linux; Android 4.0.4; C525c Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S84051[] = "Mozilla/5.0 (Linux; Android 4.0.4; A8 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S84052[] = "Mozilla/5.0 (Linux; Android 4.1.1; Transformer Prime TF201 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S84053[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)";
const char S84054[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)";
const char S84055[] = "BlackBerry8310/4.5.0.182 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102 UP.Browser/5.0.3.3";
const char S84056[] = "Spice S707/2008.07.09 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84057[] = "Spice S707/2008.09.26 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84058[] = "Spice S707/2008.05.29 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84059[] = "Spice S707/2008.05.16 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84060[] = "Spice S707/2009.03.03 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84061[] = "Spice S707/2009.05.31 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84062[] = "Spice S707/2009.04.28 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84063[] = "Spice S707/2008.09.02 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84064[] = "Spice S707/2009.04.10 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84065[] = "Spice S707/2008.08.20 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84066[] = "Spice S707/2009.07.15 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84067[] = "Spice S707/2009.06.29 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84068[] = "Spice S707/2008.09.27 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84069[] = "Spice S707/2008.12.03 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84070[] = "Spice S707/2009.03.25 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84071[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; it-it; PM1107 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84072[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.1.19; en-us; PMP3084B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84073[] = "MOT-JFJKN/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0";
const char S84074[] = "MOT-JFJK0/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0";
const char S84075[] = "MOT-W385/1.0 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0";
const char S84076[] = "SAMSUNG-SGH-C450/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S84077[] = "Micromax-X225";
const char S84078[] = "DoCoMo/2.0 N04B(c500;TB;W24H16)";
const char S84079[] = "MQQBrowser/28 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S84080[] = "MQQBrowser/28 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Safari/7534.48.3";
const char S84081[] = "MQQBrowser/28 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3";
const char S84082[] = "MQQBrowser/2801 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S84083[] = "Mozilla/5.0 (Windows NT 4.10) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11";
const char S84084[] = "Mozilla/5.0 (Windows NT 5.1; rv:10.0.2) Gecko/X Firefox/10.0.2 slurp";
const char S84085[] = "Mozilla/5.0 (Windows NT 5.1; rv:10.0) Gecko/X Firefox/10.0 slurp";
const char S84086[] = "Mozilla/5.0 (Windows NT 5.1; rv:10.0.1) Gecko/X Firefox/10.0.1 slurp";
const char S84087[] = "Mozilla/5.0 (Windows NT 5.1; rv:10.0.2) Gecko/X Firefox/10.0.2 slurp slurp";
const char S84088[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:12.0) Gecko/X Firefox/12.0 SeaMonkey/2.9";
const char S84089[] = "MicromaxX55 ASTRO36_TD/v3 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10";
const char S84090[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; CTP810 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S84091[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:6.0) Gecko/X Firefox/6.0-x64 PaleMoon/6.0-x64";
const char S84092[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/1.6.1.1691 (Phoenix Viewer Release Mesh; Firebird skin) Safari/533.3";
const char S84093[] = "Mozilla/5.0 (Maemo; Linux armv7l; rv:7.0.1) Gecko/X Firefox/7.0.1 Fennec/7.0.1";
const char S84094[] = "Mozilla/5.0 (Maemo; Linux armv7l; rv:7.0) Gecko/X Firefox/7.0 Fennec/7.0";
const char S84095[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN95 8GB)/UC Browser8.4.0.159/28/352/UCWEB Mobile";
const char S84096[] = "Mozilla/5.0 (S60V3; U; en-in; NOKIAN95)/UC Browser8.4.0.159/28/352/UCWEB Mobile";
const char S84097[] = "Mozilla/5.0 (Java; U; en-us; nokian95) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S84098[] = "Mozilla/5.0 (Java; U; en-us; nokian95_8gb) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S84099[] = "NokiaC7-00/UCWEB8.2.0.116/51/800";
const char S84100[] = "Mozilla/5.0 (Java; U; en-us; nokiac7-00) UCBrowser8.2.1.144/69/444/UCWEB Mobile  3gpp-gba";
const char S84101[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; Fly_IQ270 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84102[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:14.0) Gecko/X Thunderbird/14.0";
const char S84103[] = "curl/7.16.3 (powerpc-apple-darwin8.0) libcurl/7.16.3 OpenSSL/0.9.7l zlib/1.2.3";
const char S84104[] = "curl/7.13.1 (powerpc-apple-darwin8.0) libcurl/7.13.1 OpenSSL/0.9.7l zlib/1.2.3";
const char S84105[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.5.0.185/84/352";
const char S84106[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.5.0.185/82/352";
const char S84107[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.5.0.185/82/405";
const char S84108[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.5.0.185/83/352";
const char S84109[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.5.0.185/82/352  3gpp-gba";
const char S84110[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Motorola_MC659B_312";
const char S84111[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Motorola_MC659B_40211";
const char S84112[] = "Mozilla/5.0 (Linux; Android 4.0.4; L-06D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84113[] = "Mozilla/5.0 (Linux; Android 4.0.4; L-06DJOJO Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84114[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-SU760 Build/ICS) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84115[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-SU760 Build/ICS) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84116[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s3653w) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S84117[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; P300(Funbook) Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S84118[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Rockmelt/2.1.0.1310 Mobile/10A525 Safari/8536.25";
const char S84119[] = "Mozilla/5.0 (iPhone; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Rockmelt/2.1.0.1310 Mobile/10A525 Safari/8536.25";
const char S84120[] = "Mozilla/5.0 (Linux; Android 4.0.4; SH837W Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84121[] = "MQQBrowser/391 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 Safari/7534.48.3";
const char S84122[] = "MQQBrowser/391 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Safari/7534.48.3";
const char S84123[] = "MQQBrowser/391 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3";
const char S84124[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Essentielb-Black Diamond Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84125[] = "NokiaC7-00/UCWEB8.8.0.212/51/800";
const char S84126[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac7-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile  3gpp-gba";
const char S84127[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq5) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S84128[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOLO A700 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S84129[] = "Instagram 3.4.0 Android (15/4.0.4; 160dpi; 800x1232; samsung; GT-P7500; GT-P7500; p3; en_US)";
const char S84130[] = "Instagram 3.4.4 Android (17/4.2.2; 160dpi; 800x1232; samsung; GT-P7500; p4; p3; en_CA)";
const char S84131[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S84132[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S84133[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);";
const char S84134[] = "OPPO_R813T/1.0 Linux/3.0.35.7 Android/4.0 Release/11.15.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2";
const char S84135[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I8160) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S84136[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA6720c-1b) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S84137[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA6720c) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S84138[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Primo_F1 Build/WALTONPrimo_F1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84139[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S84140[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-S7568 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S84141[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax A91 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S84142[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CELKON CT 888 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84143[] = "SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj108i) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S84144[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033M Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84145[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT912 Build/JDQ39E; Carbon-CARBON-JB-NIGHTLY-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84146[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ultrafone_501 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84147[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S84148[] = "Mozilla/5.0 (Linux; Android 4.2.1; N986 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S84149[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-ca; SGH-I337M Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, ca";
const char S84150[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.60140";
const char S84151[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S84152[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A500 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S84153[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D44 Safari/9537.53";
const char S84154[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D46 Safari/9537.53";
const char S84155[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D47 Safari/9537.53";
const char S84156[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D48 Safari/9537.53";
const char S84157[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D49 Safari/9537.53";
const char S84158[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D50 Safari/9537.53";
const char S84159[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D52 Safari/9537.53";
const char S84160[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D51 Safari/9537.53";
const char S84161[] = "UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-S7562) U2/1.0.0 UCBrowser/9.3.0.321 Mobile";
const char S84162[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710BST Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S84163[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S84164[] = "MOT-Q11/01.00.49R-01 Software/WM6.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S84165[] = "Touch_HD_T8285/V1.59.841.6 (48753) Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S84166[] = "Touch_HD_T8285 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S84167[] = "Touch_HD_T8285/V1.59.841.5 (48753) Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S84168[] = "LG-CT810/V10h NetFront/v3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84169[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-gb; GT-P7500 Build/HONEYCOMB) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S84170[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; GT-P7500 Build/HRI83) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S84171[] = "LGE-VX8560/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S84172[] = "LGE-VX8550/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S84173[] = "DoCoMo/2.0 N06B(c100;TB;W24H16)";
const char S84174[] = "KDDI-CA3C UP.Browser/6.2_7.2.7.1.K.2.234 (GUI) MMP/2.0";
const char S84175[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus S Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84176[] = "Mozilla/4.0 (compatible; ipMonitor 9.0)";
const char S84177[] = "MQQBrowser/2.8 (Nokia5233;SymbianOS/9.1 Series60/3.0)";
const char S84178[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/69/352";
const char S84179[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/70/352";
const char S84180[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/69/444";
const char S84181[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/69/405";
const char S84182[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/69/352  3gpp-gba";
const char S84183[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.1.161/70/405";
const char S84184[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.1.161/70/352";
const char S84185[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/69/444  3gpp-gba";
const char S84186[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:13.0) Gecko/X Firefox/13.0 SeaMonkey/2.10";
const char S84187[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:14.0) Gecko/X Firefox/14.0.1 SeaMonkey/2.11";
const char S84188[] = "Opera/9.30 (Windows NT 5.1; U; pl) Version/10.9";
const char S84189[] = "Opera/9.30 (Windows NT 5.1; U; pl) Presto/2.3.15 Version/10.10";
const char S84190[] = "contype chromeframe/23.0.1271.64";
const char S84191[] = "contype chromeframe/23.0.1271.95";
const char S84192[] = "contype chromeframe/23.0.1271.97";
const char S84193[] = "Mozilla/5.0 AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11 iTunes/10.7 Downcast/2.7.13.1002";
const char S84194[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3 TeamStream HD (iPad2,1 284 1.7)";
const char S84195[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3 TeamStream HD (iPad2,1 284 1.7)";
const char S84196[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE61i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile";
const char S84197[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE61) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile";
const char S84198[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiae61i-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S84199[] = "NOKIAE61iUCBrowser/8.7.1.234/28/352/UCWEB";
const char S84200[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MODECOM FreeTAB 9702 IPS X2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84201[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Mobile Safari/535.19 Silk-Accelerated=false";
const char S84202[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; SH837W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84203[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P6210 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84204[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6210 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84205[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6210 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84206[] = "JUC (Linux; U; 2.3.5; zh-cn; TECNO_N3; 320*480) UCWEB7.9.0.94/139/444";
const char S84207[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S84208[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; ST18i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S84209[] = "HS-T930_TD/1.0 Android/2.3 Release/07.07.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84210[] = "Mozilla/5.0 (Linux; Android 4.0.3; PAP4300 DUO Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S84211[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NABI2-NV7A Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S84212[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST15i Build/4.1.B.0.587) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S84213[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S84214[] = "OPPO_R811/1.0 Linux/3.0.13 Android/4.0 Release/10.15.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2";
const char S84215[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_One_V) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S84216[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Mobile Safari/537.31";
const char S84217[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S84218[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; A75) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S84219[] = "Mozilla/5.0 (compatible; MSIE 3.0; Windows NT 6.0; Trident/5.0)";
const char S84220[] = "Mozilla/5.0 (Linux; Android 4.1.2; JY-G3 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S84221[] = "Mozilla/5.0 (Linux; Android 4.0.4; Arc Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S84222[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17; Maxthon (%s);";
const char S84223[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TM-MID730 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84224[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.380; en-us; 9500/4.2.0.380) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S84225[] = "UCWEB/2.0(BlackBerry; U; 4.0.0.174; en-us; 9500/4.0.0.174) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S84226[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.198; en-us; 9500/4.2.0.198) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S84227[] = "UCWEB/2.0(BlackBerry; U; 4.2.0.436; en-us; 9500/4.2.0.436) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S84228[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Karbonn_A4_) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S84229[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N8013 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);";
const char S84230[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N8013 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S84231[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST25i Build/6.1.1.B.1.10) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S84232[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE61i) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S84233[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE61) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S84234[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae61i-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S84235[] = "Mozilla/5.0 (Linux; Android 4.0.4; C5155 Build/IML77) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S84236[] = "Mozilla/5.0 (Linux; Android 4.0.3; W732 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84237[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Desire 600 dual sim Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84238[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC Desire 600 dual sim Build/JZO54K)";
const char S84239[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Aquaris Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84240[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M440S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84241[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S84242[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993D Build/ICECREAM) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84243[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993 Build/ICECREAM) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84244[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848i Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84245[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848 Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84246[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-ch; Lenco TAB-900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84247[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; E3 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33931";
const char S84248[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; iBall Andi4.5d quadro Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84249[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S84250[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; WT19i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S84251[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; WT19i Build/4.0.2.A.0.58) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S84252[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; GT-N8000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S84253[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N8000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);";
const char S84254[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S560 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84255[] = "Mozilla/5.0 (Linux; Android 4.1.2; P-02E Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84256[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I437 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S84257[] = "Mozilla/5.0 (Linux; Android 4.2.1; Starmobile KNIGHT Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S84258[] = "LG8700/1.1 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S84259[] = "LGE-LG8700 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S84260[] = "LG8700/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S84261[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; Videocon_V7400 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84262[] = "Nokia6260/2.0 (3.0448.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S84263[] = "MOT-MT620_TD/1.0 OPhone/2.5 (Linux; Android 2.2) Release/03.31.2011 Browser/WAP2.0 (AppleWebKit/533.1) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S84264[] = "Huawei/1.0/HUAWEI_G6005/Obigo Browser/Q03C Browser/Q03C MMS/1.2";
const char S84265[] = "HTC_HD2_T8585 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S84266[] = "HD2_T8585 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S84267[] = "HTC_HD2_T8585/480x800 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S84268[] = "Facebook 4110.0 (iPad; iPhone OS 5.0.1; en_US)";
const char S84269[] = "Facebook 4100.0 (iPad; iPhone OS 5.0; en_US)";
const char S84270[] = "FeedFetcher-Google-CoOp; (+http://www.google.com/coop/cse/cref)";
const char S84271[] = "StumbleUpon/2.60 CFNetwork/485.12.7 Darwin/10.4.0";
const char S84272[] = "Bing/3.03 CFNetwork/485.12.7 Darwin/10.4.0";
const char S84273[] = "nokia5800d-1/UC Browser7.9.0.102/69/352  3gpp-gba";
const char S84274[] = "Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia3120classic/UC Browser8.0.3.107/69/444";
const char S84275[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Glacier; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S84276[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1 x64; en-US; rv:1.9.0.10) Gecko/X Minefield/3.0.10";
const char S84277[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9a4pre) Gecko/X Minefield/3.0a4pre";
const char S84278[] = "LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-gs290) UCBrowser8.2.1.144/69/352/UCWEB Mobile";
const char S84279[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Woxter Tablet PC 85CXi Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84280[] = "Mozilla/5.0 (Linux; U; Android 4.0; vi-vn; HKPhone 4S-3G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84281[] = "Dolphin http client/8.8.1(171) (Android)";
const char S84282[] = "Dolphin http client/8.8.0(168) (Android)";
const char S84283[] = "Dolphin http client/8.8.2(177) (Android)";
const char S84284[] = "Rockmelt/2.0.1.4 CFNetwork/609 Darwin/13.0.0";
const char S84285[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) rekonq/1.2 Safari/534.34";
const char S84286[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT26i Build/6.1.A.2.45) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S84287[] = "Mozilla/5.0 (S60V3; U; en-us; Nokia N96) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile";
const char S84288[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; ALCATEL ONE TOUCH 918N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84289[] = "LG-S367/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S84290[] = "Mozilla/5.0 (Linux; Android 4.0.4; A6 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84291[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Mobile Safari/535.19 Silk-Accelerated=false";
const char S84292[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; U900 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84293[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA5630d-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S84294[] = "Mozilla/5.0 (Linux; Android 4.0.3; Xtreamer_Mobile_AiKi Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84295[] = "Mozilla/5.0 (Linux; Android 4.2.1; SPH-D700 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84296[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-D700 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84297[] = "Mozilla/5.0 (Linux; Android 4.1.1; tablet Fnac 8 Build/1.0.4 X-11:52) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S84298[] = "Mozilla/5.0 (Linux; Android 4.1.2; P-02E Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84299[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; F10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84300[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-MS770 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S84301[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9430 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84302[] = "UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; X10i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S84303[] = "Mozilla/5.0 (Linux; Android 4.0.3; INO_ONE Build/INO_ONEINO_ONE) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S84304[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0800, id";
const char S84305[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0800, my";
const char S84306[] = "Mozilla/5.0 (Linux; Android 4.0.4; NABI2-NV7A Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22";
const char S84307[] = "Mozilla/5.0 (Linux; Android 4.1.1; bq Curie Build/1.1.0 X-14:50) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S84308[] = "Mozilla/5.0 (Linux; Android 4.1.1; M040 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S84309[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI-ONE C1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S84310[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; EVO Build/JRO03C; Carbon-CARBON-JB-EXP-10APR2013) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84311[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; PC36100 Build/JDQ39E; Carbon-CARBON-JB-UNOFFICIAL-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84312[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; da-dk; TAB-80012 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84313[] = "Mozilla/5.0 (Linux; Android 4.1.1; Starmobile DIAMOND Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84314[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST27i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S84315[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S84316[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5570) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S84317[] = "Mozilla/5.0 (Linux; Android 4.1.1; bq Curie Build/1.1.0 X-14:50) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S84318[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cynus T2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84319[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I757M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84320[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S84321[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LU6200 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84322[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7108 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S84323[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7108 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S84324[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300";
const char S84325[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Aurora-II Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84326[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Aurora-II Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84327[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.11 (KHTML, like Gecko) Ubuntu/11.10 Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/537.36";
const char S84328[] = "Nokia301/2.0 (02.33) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S84329[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 UCBrowser/9.1.0.287 Mobile";
const char S84330[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 UCBrowser/9.1.0.287 Mobile";
const char S84331[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-i705 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S84332[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS930 4G Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84333[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E16i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S84334[] = "Mozilla/5.0 (Linux; Android 4.2.1; Q1000 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84335[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI H868C Build/HuaweiH868C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84336[] = "Mozilla/5.0 (Linux; Android 4.2.2; Razr HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84337[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT925 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84338[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; E3 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S84339[] = "Mozilla/5.0 (Linux; Android 4.2.1; W8 beyond Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S84340[] = "Mozilla/5.0 (Linux; Android 4.1.1; GSmart Maya M1 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S84341[] = "Mozilla/5.0 (Linux; Android 4.2.1; iris504Q Build/iris504Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S84342[] = "Mozilla/5.0 (Linux; Android 4.1.1; ST10 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S84343[] = "Mozilla/5.0 (Linux; U; Android noobnl Andromeda Galaxy ROM v1.0.3; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84344[] = "Mozilla/5.0 (Linux; U; Android Epic_Experience_v1.2.07a; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84345[] = "Mozilla/5.0 (Linux; U; Android Baked-Snack-1.2; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84346[] = "Mozilla/5.0 (Linux; U; Android SyndicateROM_1.0.1_(Sai); en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84347[] = "Mozilla/5.0 (Linux; U; Android Epic_Experience_v1.2.03; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84348[] = "Mozilla/5.0 (Linux; U; Android Epic_Experience_v1.2.07; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84349[] = "Mozilla/5.0 (Linux; U; Android Epic_Experience_v1.2.05; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84350[] = "Mozilla/5.0 (Linux; U; Android Epic_Experience_v1.2.04; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84351[] = "Mozilla/5.0 (Linux; U; Android Epic_Experience_v1.2.08; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84352[] = "Mozilla/5.0 (Linux; U; Android noobnl Andromeda Galaxy ROM v1.1.1; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84353[] = "Mozilla/5.0 (Linux; U; Android noobnl_Andromeda_Galaxy_ROM_v1.1.1a; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84354[] = "Mozilla/5.0 (Linux; U; Android SyndicateROM_1.0.2_(Shuriken); en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84355[] = "Mozilla/5.0 (Linux; U; Android Epic_Experience_v1.2.09; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84356[] = "Mozilla/5.0 (Linux; U; Android Heroic.Epic_v1.0.3; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84357[] = "Mozilla/5.0 (Linux; U; Android Heroic.Epic_v1.0.4.1; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84358[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Toshiba/TG01/02; Windows Phone 6.5)";
const char S84359[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Toshiba-TG01-orange/ 02; Windows Phone 6.5)";
const char S84360[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Toshiba_TG01; Windows Phone 6.5)";
const char S84361[] = "LG_P503/1.0 Android/2.2 Release/5.20.2010 Browser/KHTML (Mozilla/5.0 AppleWebKit/533.1 Version/4.0 Mobile Safari/533.1)";
const char S84362[] = "LG-KF350_CMCC Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S84363[] = "LG-KF350 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S84364[] = "HTC_Touch_Viva_T2223 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S84365[] = "WinWAP/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84366[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.0 Chrome/13.0.800.0 Safari/535.1";
const char S84367[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.0 Chrome/13.0.800.0 Safari/535.1";
const char S84368[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.0 Chrome/13.0.800.0 Safari/535.1";
const char S84369[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.0 Chrome/13.0.800.0 Safari/535.1";
const char S84370[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.68.1565 Chrome/14.0.835.202 Safari/535.1";
const char S84371[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.13 (KHTML, like Gecko) RockMelt/0.9.48.59 Chrome/9.0.597.107 Safari/534.13";
const char S84372[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.24 (KHTML, like Gecko) RockMelt/0.9.56.357 Chrome/11.0.696.71 Safari/534.24";
const char S84373[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.30 (KHTML, like Gecko) RockMelt/0.9.62.313 Chrome/12.0.742.122 Safari/534.30";
const char S84374[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.64.347 Chrome/13.0.782.218 Safari/535.1";
const char S84375[] = "Mozilla/5.0 (Windows; U; Windows NT 4.0; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13";
const char S84376[] = "NOKIAN95 8GB/UC Browser7.9.0.102/28/355/UCWEB";
const char S84377[] = "NOKIAN95/UC Browser7.9.0.102/28/355/UCWEB";
const char S84378[] = "Mozilla/4.0 (compatible; MSIE 5.0; Mac_PowerPC) Opera 5.0 [en]";
const char S84379[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; SHW-M290S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84380[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; myPad P2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84381[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:5.0) Gecko/X Thunderbird/5.0 Lightning/1.0b4";
const char S84382[] = "HTC Streaming Player sprint / 1.0 / htc_shooter / 2.3.4";
const char S84383[] = "HTC Streaming Player Sprint / 1.0 / htc_shooter / 4.0.3";
const char S84384[] = "HTC Streaming Player virgin_mobile / 1.0 / htc_shooter / 4.0.3";
const char S84385[] = "HTC Streaming Player htc / 1.0 / htc_shooter / 4.0.3";
const char S84386[] = "GoogleEarth/6.1.0.5001(Macintosh;Mac OS X (10.7.2);en;kml:2.2;client:Free;type:default)";
const char S84387[] = "GoogleEarth/6.1.0.5001(Macintosh;Mac OS X (10.7.3);en;kml:2.2;client:Free;type:default)";
const char S84388[] = "GoogleEarth/6.1.0.5001(Macintosh;Mac OS X (10.7.4);en;kml:2.2;client:Free;type:default)";
const char S84389[] = "GoogleEarth/6.1.0.5001(Macintosh;Mac OS X (10.7.5);en;kml:2.2;client:Free;type:default)";
const char S84390[] = "Mozilla/5.0 (Linux; U; Linux; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84391[] = "ZTE-Z331/1.5.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S84392[] = "JUC(Linux;U;Android2.3.4;Zh_cn;MT11i;480*854;)UCWEB7.8.0.95/139/444";
const char S84393[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.8.1.16) Gecko/X Firefox/2.0.0.16 Flock/1.2.4";
const char S84394[] = "SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-in; sonyericssonj105i) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S84395[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I500 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84396[] = "Mozilla/5.0 (Linux; Android 4.0.3; SCH-I500 Build/GINGERBREAD) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84397[] = "MOT-EX132/1.0 REXL4/1.0 BrewMP/1.0.3 Release/6.15.2011 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84398[] = "ZTE-T3 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1 firmware/T3 T04";
const char S84399[] = "ZTE-T3 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1 firmware/T3 M01";
const char S84400[] = "Mozilla/5.0 (X11) KHTML/4.8.4 (like Gecko) Konqueror/4.8";
const char S84401[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; RMD-1030 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84402[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; RMD-1030 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84403[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/555.5 (KHTML, like Gecko) Chrome/99.0.999.0 Safari/555.5";
const char S84404[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/99.9.999 Safari/534.24";
const char S84405[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/99.0 Safari/537.1 YottaaMonitor";
const char S84406[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; de) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/NA Chrome anonymized by Abelssoft X";
const char S84407[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/99.0 Safari/537.11 YottaaMonitor";
const char S84408[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:18.0) Gecko/18.0 Firefox/18.0";
const char S84409[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Skate Build/IML74K; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84410[] = "LG-A341 Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S84411[] = "LG-A341-parrot Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S84412[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S84413[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 TeamStream (iPod4,1 293 2.2.2)";
const char S84414[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3 TeamStream (iPod4,1 293 2.2.2)";
const char S84415[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-B5330 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S84416[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOLO A700 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84417[] = "Mozilla/5.0 (Linux; Android 4.1.1; SH930W Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84418[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; OBIGO_Q03C Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84419[] = "MOT-EX430/03.03.10 Obigo/Q03C MMP/2.0";
const char S84420[] = "SAMSUNG-GT-S6108_TD/1.0 Android/2.3.6 Release/1.5.2012 Browser/AppleWebKit533.1";
const char S84421[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6108 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S84422[] = "JUC(Linux;U;Android4.0.4;Zh_cn;GT-S7562;480*800;)UCWEB7.8.0.95/139/444";
const char S84423[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; Telenor_One_Touch_S Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84424[] = "Mozilla/5.0 (Linux; Android 4.0.3; U9500 Build/HuaweiU9500) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S84425[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SCH-i929 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84426[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; SmartPro Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84427[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I437 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S84428[] = "Mozilla/5.0 (Linux; Android 4.0.3; SmartTabII10 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S84429[] = "Mozilla/5.0 (Linux; Android 4.1.2; AMID-9743G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S84430[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A860L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84431[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A860S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84432[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V70 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S84433[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; WALTON Primo C1 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84434[] = "GoogleEarth/7.1.1.1580(Macintosh;Mac OS X (10.8.3);en;kml:2.2;client:Free;type:default)";
const char S84435[] = "GoogleEarth/7.1.1.1580(Macintosh;Mac OS X (10.8.4);en;kml:2.2;client:Free;type:default)";
const char S84436[] = "GoogleEarth/7.1.1.1888(Macintosh;Mac OS X (10.8.4);en;kml:2.2;client:Free;type:default)";
const char S84437[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530M Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84438[] = "Mozilla/5.0 (Linux; Android 4.0.3; U9200 Build/HuaweiU9200) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S84439[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S84440[] = "OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Symphony_W125 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84441[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8000 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S84442[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire 200 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84443[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; O+ 8.6 Android Build/Oplus8.6_Android) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84444[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9502 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S84445[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB192 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S84446[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB0192 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S84447[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-in; Micromax A116 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S84448[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S5303 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S84449[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; HTC_Wildfire) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S84450[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; HTC_Wildfire) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S84451[] = "Mozilla/5.0 (X11; OpenBSD amd64; rv:22.0) Gecko/X Firefox/22.0";
const char S84452[] = "Mozilla/5.0 (Linux; Android 4.1.1; bq Curie Build/1.1.0 X-14:50) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S84453[] = "Mozilla/5.0 (Linux; Android 4.0.4; AD683G Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84454[] = "Mozilla/5.0 (Linux; Android 4.0.4; A210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S84455[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9001 Build/JDQ39E)";
const char S84456[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; de-de; GT-I9001 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84457[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; N7889 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84458[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ultrafone 304 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84459[] = "Mozilla/5.0 (Linux; Android 4.2.2; Evo 3D CDMA Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84460[] = "Mozilla/5.0 (Linux; Android 4.2.2; PG86100 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84461[] = "Mozilla/5.0 (Linux; Android 4.2.2; Evo 3D GSM Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84462[] = "Mozilla/5.0 (Linux; Android 4.3; SCH-I545 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84463[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A90S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S84464[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6802) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S84465[] = "Mozilla/5.0 (Linux; Android 4.1.1; bq Curie Build/1.1.0 X-14:50) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S84466[] = "Mozilla/5.0 (Linux; Android 4.1.2; F-02E Build/V17R48A) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S84467[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST18i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S84468[] = "Mozilla/5.0 (Linux; Android 4.2.2; F-06E Build/V21R48D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S84469[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E220S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S84470[] = "Mozilla/5.0 (Linux; Android 4.2.2; Xperia S Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S84471[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Micromax A110Q Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S84472[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S84473[] = "Palm750/v0005 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S84474[] = "KDDI-SH3B UP.Browser/6.2_7.2.7.1.K.3.352 (GUI) MMP/2.0";
const char S84475[] = "DoCoMo/2.0 P05A(c100;TB;W24H15) (compatible; BaiduMobaider/1.0; +http://www.baidu.jp/spider/)";
const char S84476[] = "KDDI-SN3I UP.Browser/6.2_7.2.7.1.K.3.352 (GUI) MMP/2.0";
const char S84477[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.1.0.0 Chrome/12.0.742.91 Safari/534.30";
const char S84478[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.1.0.0 Chrome/12.0.742.91 Safari/534.30";
const char S84479[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ZTE Blade Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84480[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST3_Blade Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84481[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZTE Blade Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84482[] = "Million%20HD%20WPs/8.80.1 CFNetwork/459 Darwin/10.0.0d3";
const char S84483[] = "WOW%20WPs/8.80.1 CFNetwork/459 Darwin/10.0.0d3";
const char S84484[] = "InstapaperPro/3.0.3 CFNetwork/459 Darwin/10.0.0d3";
const char S84485[] = "GQMagazine/1.0 CFNetwork/459 Darwin/10.0.0d3";
const char S84486[] = "AtomicLite/5.6.0 CFNetwork/459 Darwin/10.0.0d3";
const char S84487[] = "Y!%20Search/2.0.3 CFNetwork/459 Darwin/10.0.0d3";
const char S84488[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.6 Safari/532.0";
const char S84489[] = "samsung-gt-s5263/UC Browser7.9.0.102/69/352";
const char S84490[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ONDA MID Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84491[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; ONDA MID Build/GRJ22)";
const char S84492[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ONDA MID Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S84493[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ONDA MID Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S84494[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; ja-jp; A01SH Build/HTK55D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S84495[] = "Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; A01SH Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S84496[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10";
const char S84497[] = "NOKIA-Lumia 800/UCWEB2.2.0.157/47/355";
const char S84498[] = "Mozilla/5.0 (X11; OpenBSD amd64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S84499[] = "Mozilla/5.0 (X11; OpenBSD i386) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11";
const char S84500[] = "Mozilla/5.0 (Windows NT 6.0; rv:15.0) Gecko/X Firefox/15.0.2.0 IceDragon/15.0.2.0";
const char S84501[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; fr; rv:1.8.1.9pre) Gecko/X Firefox/2.0.0.9 Navigator/9.0.0.3";
const char S84502[] = "HS-T96_TD/1.0 Android/4.0 Release/5.10.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84503[] = "Mozilla/5.0 (Windows NT 6.0; rv:16.0) Gecko/X Firefox/16.0 slurp";
const char S84504[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:16.0) Gecko/X Firefox/16.0 slurp";
const char S84505[] = "Mozilla/5.0 (X11; OpenBSD amd64; rv:16.0) Gecko/X Firefox/16.0";
const char S84506[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/5.1 Mobile/8B117 Safari/7534.48.3 UCBrowser8.5.1.228/41/355";
const char S84507[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT18i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S84508[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 1.1.4322) Sleipnir/2.7.0";
const char S84509[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30) Sleipnir/2.7.1";
const char S84510[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727) Sleipnir/2.7.2";
const char S84511[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP7100D3G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S84512[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84513[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84514[] = "JUC (Linux; U; 2.3.3; zh-cn; LG-P500; 320*480) UCWEB7.9.0.94/139/355";
const char S84515[] = "JUC (Linux; U; 2.2.2; zh-cn; LG-P500; 320*480) UCWEB7.9.0.94/139/444";
const char S84516[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en_us; Cloudfone Excite350g Build/FSV05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84517[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; Consul 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84518[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Desire V Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S84519[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; MT27i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S84520[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; LG-lu6200 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84521[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-LU6200 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84522[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; TM-9741 Build/V1.1.ICECREAMSANDWICH.rus.teXet.X.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 MYI/1.10";
const char S84523[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S560 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S84524[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LEMON P7  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84525[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, us";
const char S84526[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; GT-P3113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.22/288,1.9.22.288,2013-01-31 13:09, -0500, pe";
const char S84527[] = "UCWEB/2.0 (Linux; U; Adr 2.2; en-US; GT-I9000) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S84528[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_Desire_C) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S84529[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Penta WS707C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84530[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S84531[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S84532[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S84533[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S84534[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; YPY_AB10D Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84535[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; YPY_AB10DC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84536[] = "SAMSUNG-GT-I9228_TD/1.0 Android/4.1.2 Release/12.24.2012 Browser/AppleWebKit534.30";
const char S84537[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire V) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S84538[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire V) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S84539[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5660 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S84540[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-S5660 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S84541[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-CN; GT-S5660 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S84542[] = "Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300L Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84543[] = "Mozilla/5.0 (compatible; MSIE 5.0; Windows NT 6.2; Trident/5.0)";
const char S84544[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDZ Dash Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84545[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDZ Dash X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84546[] = "Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI P2-6011 Build/HuaweiP2-6011) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84547[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5620) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S84548[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5570) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S84549[] = "Mozilla/5.0 (Linux; Android 4.0.4; PNDPP48GP Build/ICS.PNDPP48GP.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S84550[] = "LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gs290) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S84551[] = "Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5310xpressmusic) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S84552[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S84553[] = "Mozilla/5.0 (Linux; Android 4.0.3; A100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.60140";
const char S84554[] = "Videocon_V1539N_Maui Wap Browser";
const char S84555[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S720 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84556[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S84557[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S84558[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E270K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S84559[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; ZTE U956 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S84560[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; L36h Build/10.1.1.A.1.253) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S84561[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI C8815 Build/HuaweiC8815) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S84562[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I8160 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S84563[] = "Mozilla/5.0 (Series40; NokiaX2-00/08.25; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S84564[] = "MOT-MOTOQA1/A2.00.52R1 BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003388) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.60 [es]";
const char S84565[] = "MOT-MOTOQA1/A2.00.52R1 BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; X) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.60 [en]";
const char S84566[] = "MOT-A3100 Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/16465; U; pt-BR)";
const char S84567[] = "MOT-A3100 Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/16263; U; en)";
const char S84568[] = "MOT-A3100 Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/16108; U; en)";
const char S84569[] = "MOT-A3100 Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/16747; U; en)";
const char S84570[] = "MOT-A3100/ Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/15405; U; zh-tw)";
const char S84571[] = "MOT-A3100/ Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/16747; U; zh-tw)";
const char S84572[] = "MOT-A3100/ Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/16263; U; zh-tw)";
const char S84573[] = "MOT-A3100 Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/16465; U; es-ES)";
const char S84574[] = "MOT-A3100 Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/16263; U; es-ES)";
const char S84575[] = "MOT-A3100 Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/15405; U; en)";
const char S84576[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; E120 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S84577[] = "Nokia6102/2.0 (04.50) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84578[] = "Nokia6102i/2.0 (04.68) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84579[] = "SEC-SGHE570/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84580[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch2_T3320; Windows Phone 6.5)";
const char S84581[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch2_T3330; Windows Phone 6.5)";
const char S84582[] = "SonyEricssonX2/R1AA Browser/Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S84583[] = "SCH-U430/1.0 NetFront/3.0.22.2.12 (GUI) MMP/2.0";
const char S84584[] = "HPiPAQhw6900/1.0/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x240)";
const char S84585[] = "KDDI-CA3G UP.Browser/6.2_7.2.7.1.K.2.234 (GUI) MMP/2.0";
const char S84586[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0) chromeframe/6.0.472.63";
const char S84587[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MS-RTC LM 8) chromeframe/6.0.472.63";
const char S84588[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MS-RTC LM 8) chromeframe/6.0.472.63";
const char S84589[] = "NokiaE52-1/UC Browser8.0.4.121/28/444/UCWEB";
const char S84590[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; CSL-MI410 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84591[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; swiper Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84592[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 3GC101 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84593[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; HD7 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84594[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SCH-I800 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84595[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I800 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84596[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:14.0) Gecko/X Thunderbird/14.0";
const char S84597[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Andi2 Build/iBall_HERA_00.04.79.17.43.H1019.M01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84598[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:15.0) Gecko/X Firefox/15.0-x64 PaleMoon/15.0-x64";
const char S84599[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:15.0) Gecko/X Firefox/15.0 PaleMoon/15.0";
const char S84600[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (PC36100, 42, 1.9.1)";
const char S84601[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P3100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S84602[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S84603[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; miTab URBAN Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84604[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; miTab URBAN Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84605[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xperia S Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84606[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA6700s) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile";
const char S84607[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; Micromax P350 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S84608[] = "Apple/4.0 Downcast/2.7.14.1133 iTunes/10.7";
const char S84609[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I8190 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/177,1.9.6.177,2012-10-17 16:38, gb";
const char S84610[] = "Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A30 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84611[] = "OneBrowser/3.1 (Nokia110/03.26)";
const char S84612[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84613[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; MT27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S84614[] = "A352 WAP-Browser/1.0.0";
const char S84615[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S84616[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8812E Build/HuaweiC8812E) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S84617[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Maxwell Plus Build/1.0.3 X-14:07) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S84618[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT912 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S84619[] = "Mozilla/5.0 (Linux; Android 4.1.1; Starmobile FLIRT Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84620[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:34, my";
const char S84621[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-P1000L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.7/210,1.9.7.210,2012-10-25 19:53, ar";
const char S84622[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.7/209,1.9.7.209,2012-10-24 23:31, au";
const char S84623[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000N Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, -0400, tt";
const char S84624[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-P1000M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, -0400, ca";
const char S84625[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Opera/9.80 (Android; Opera Mini/6.0.25270/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S84626[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Opera/9.80 (Android; Opera Mini/6.0.25270/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S84627[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MT11i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S84628[] = "Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 10 Build/1.1.11-1015 X-10:30) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S84629[] = "Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 10 3G Build/1.1.11-1015 X-16:17) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S84630[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I200 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S84631[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A57 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S84632[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S5303) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S84633[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A919 Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84634[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84635[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P925 Build/ICS) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84636[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5236) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S84637[] = "Mozilla/5.0 (Linux; Android 4.1.2; SH-02E Build/S4260) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84638[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3850) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S84639[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax A88 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S84640[] = "Mozilla/5.0 (Linux; Android 4.2.1; ME301T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.60140";
const char S84641[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P778 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84642[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84643[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-tw; HTC 802d Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84644[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HTC-802d/2.17.1401.2) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84645[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6210 Navigator) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S84646[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S84647[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5670) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S84648[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A810S Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84649[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A810K Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84650[] = "Mozilla/5.0 (Linux; Android 4.1.1; D2-721G Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S84651[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A25) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S84652[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Q700 Build/XOLOQ700) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S84653[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A27) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S84654[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; F-07E Build/V19R38A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84655[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S84656[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI C8812E Build/HuaweiC8812E) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S84657[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Micromax A116 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S84658[] = "UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-S5302) U2/1.0.0 UCBrowser/9.3.0.321 Mobile";
const char S84659[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S84660[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9205 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S84661[] = "Palm750/v0004 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S84662[] = "Palm750/v0101 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S84663[] = "Palm750/v0005 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S84664[] = "sam-r310 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S84665[] = "SAMSUNG-SGH-Z150/1.0 SHP/VPP/R5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S84666[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2690/UC Browser7.9.0.102/70/352";
const char S84667[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/69/352";
const char S84668[] = "Mozilla/5.0(Linux;U;Android2.3.4;Zh_cn;LT15i;480*854;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S84669[] = "JUC(Linux;U;Android2.3.2;Zh_cn;LT15i;480*854;)UCWEB7.8.0.95/139/355";
const char S84670[] = "JUC(Linux;U;Android2.3.4;Zh_cn;LT15i;480*854;)UCWEB7.8.0.95/139/444";
const char S84671[] = "JUC(Linux;U;Android2.3.3;Zh_cn;LT15i;480*854;)UCWEB7.8.0.95/139/444";
const char S84672[] = "KDDI-SH3L UP.Browser/6.2_7.2.7.1.K.6.210 (GUI) MMP/2.0";
const char S84673[] = "Mozilla/5.0 (X11; SunOS sun4u; rv:9.0.1) Gecko/X Firefox/9.0.1";
const char S84674[] = "MacOutlook/14.14.0.X (Intel Mac OS X 10.7.2)";
const char S84675[] = "MacOutlook/14.14.0.X (Intel Mac OS X 10.7.3)";
const char S84676[] = "MacOutlook/14.14.0.X (Intel Mac OS X 10.7.4)";
const char S84677[] = "MacOutlook/14.14.0.X (Intel Mac OS X 10.7.5)";
const char S84678[] = "Reeder/1010.79.00 CFNetwork/520.4.3 Darwin/11.4.0 (x86_64) (MacBookAir4%2C2)";
const char S84679[] = "Reeder/1010.79.00 CFNetwork/520.4.3 Darwin/11.4.0 (x86_64) (MacBookPro8%2C1)";
const char S84680[] = "Reeder/1010.79.00 CFNetwork/520.4.3 Darwin/11.4.0 (x86_64) (MacPro3%2C1)";
const char S84681[] = "Reeder/1010.79.00 CFNetwork/520.4.3 Darwin/11.4.0 (x86_64) (MacBook5%2C1)";
const char S84682[] = "Reeder/1010.79.00 CFNetwork/520.4.3 Darwin/11.4.0 (x86_64) (iMac11%2C1)";
const char S84683[] = "Nokia3500c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia3500c/UC Browser8.0.3.107/69/352";
const char S84684[] = "SAMSUNG-GT-i8000/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S84685[] = "JUC(Linux;U;Android2.3.6;Zh_cn;GT-N7000;800*1280;)UCWEB7.8.0.95/139/444";
const char S84686[] = "Mozilla/5.0(Linux;U;Android2.3.6;Zh_cn;GT-N7000;800*1280;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S84687[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) 1Password/3.6.1/X (like Mobile/8C148 Safari/6533.18.5)";
const char S84688[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:14.0) Gecko/X Firefox/14.0.1 slurp";
const char S84689[] = "Mozilla/5.0 (Windows NT 6.1; rv:14.0) Gecko/X Firefox/14.0.1 slurp";
const char S84690[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:14.0) Gecko/X Firefox/14.0.1 slurp slurp";
const char S84691[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; N-04C Build/AX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84692[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; N-04C Build/AX)";
const char S84693[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s5333) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S84694[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5333) UCBrowser8.3.1.161/69/405/UCWEB Mobile";
const char S84695[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; A606 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84696[] = "Mozilla/5.0 (Linux; U; Android Mi500; en-us; Mi500 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84697[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; th-th; i-mobile i698 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84698[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire Z Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84699[] = "Mozilla/5.0 (Linux; Android 4.0.3; Desire Z Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84700[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire Z Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84701[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84702[] = "MQQBrowser/3.5/Adr (Linux; U; 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD.ZCLF1;320*480)";
const char S84703[] = "MQQBrowser/3.5/Adr (Linux; U; 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD.ZCLA4;320*480)";
const char S84704[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0) chromeframe/5.0.317.0";
const char S84705[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; OfficeLiveConnector.1.4; .NET CLR 3.0.30729; OfficeLivePatch.1.3; FDM) chromeframe/5.0.342.0";
const char S84706[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; Edition Next; en) Presto/2.11.310 Version/12.50";
const char S84707[] = "Mozilla/5.0 (Symbian3; U; en-us; NokiaE7-00)/UC Browser8.5.0.183/51/355/UCWEB Mobile";
const char S84708[] = "ZEN M28; Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84709[] = "LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-gd510) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile";
const char S84710[] = "NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax3-02) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S84711[] = "Mozilla/5.0 (Series40; NokiaX3-00/11.00; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S84712[] = "Mozilla/5.0 (Linux; Android 4.0.4; V96A Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S84713[] = "Mozilla/5.0 (X11; Linux x86_64; en-US; rv:2.0b2pre) Gecko/X Minefield/4.0b2pre";
const char S84714[] = "Mozilla/5.0 (X11; Linux i686 (x86_64); rv:2.0b4pre) Gecko/X Minefield/4.0b4pre";
const char S84715[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MT917 Build/6.7.2_GC-205-DNRTD-4) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S84716[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MT917 Build/6.7.2_GC-205-DNRTD-4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84717[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MT917 Build/6.7.2_GC-131-DNRTD-20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84718[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_Sensation_XL_with_Beats_Audio_X315e; 480*800) UCWEB7.9.0.94/139/352";
const char S84719[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC_Sensation_XL_with_Beats_Audio_X315e; 480*800) UCWEB7.9.0.94/139/352";
const char S84720[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC_Sensation_XL_with_Beats_Audio_X315e; 480*800) UCWEB7.9.0.94/139/444";
const char S84721[] = "NokiaX3-00/5.0 (11.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiax3-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S84722[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo K860 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S84723[] = "JUC (Linux; U; 1.6; zh-cn; GT540; 320*480) UCWEB7.9.0.94/139/352";
const char S84724[] = "JUC (Linux; U; 2.1-update1; zh-cn; GT540; 320*480) UCWEB7.9.0.94/139/444";
const char S84725[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; R800i Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84726[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; PHCASIATAB10B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84727[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us ; Galaxy Nexus Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S84728[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; Galaxy Nexus Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S84729[] = "Mozilla/5.0 (iPhone; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 OverDrive Media Console/2.6.5";
const char S84730[] = "YahooMobileMail/1.0 (Android Mail; 1.4.5) (GT-P1000;samsung;GT-P1000;2.2/FROYO)";
const char S84731[] = "Mozilla/5.0 (Linux; Android 4.1.1; chagall Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S84732[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID7011 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84733[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; LG-E612) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S84734[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120L Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S84735[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S84736[] = "Mozilla/5.0 (Linux; Android 4.0.3; SHV-E120L Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S84737[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120K Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S84738[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HS-9DTB4 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/405";
const char S84739[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E435 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S84740[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E435f Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S84741[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; ST26i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S84742[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; VS840 4G Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84743[] = "Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300K Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S84744[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7511 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S84745[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-LS970 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84746[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-LS970 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/geehrc4g_spr_us";
const char S84747[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZP900 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84748[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZP900S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84749[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT894 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84750[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5770D Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S84751[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI-ONE Plus Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S84752[] = "Mozilla/5.0 (Linux; Android 4.2.1; Micromax A116 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84753[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; es-ES; rv:1.9.2.12) Gecko/X Firefox/3.7.12";
const char S84754[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTL22 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84755[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84756[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84757[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84758[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S84759[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; CROSS A7S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84760[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S84761[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Celkon A8+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84762[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S84763[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; C5302 Build/12.0.A.1.211) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S84764[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; C5302 Build/12.0.A.2.245) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S84765[] = "NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311";
const char S84766[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT907 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S84767[] = "SonyEricssonZ520a/R3F Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84768[] = "SonyEricssonZ520c/R3J Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84769[] = "SonyEricssonZ520i/R3F Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84770[] = "SCH-R210 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S84771[] = "MOT-EX245 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S84772[] = "DoCoMo/2.0 N906i(c100;TB;W24H16)";
const char S84773[] = "DoCoMo/2.0 N906imyu(c100;TB;W24H16)";
const char S84774[] = "DoCoMo/2.0 N906iL(c100;TB;W24H16)";
const char S84775[] = "SonyEricssonX2/R3AA Browser/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S84776[] = "Mozilla/5.0 (Windows; U; en_US) AppleWebKit/530.19.2 (KHTML, like Gecko) Dreamweaver/11.0.0.4909 Version/4.0.2 Safari/530.19.2";
const char S84777[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Desire Z Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84778[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_DesireZ_A7272 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84779[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Desire Z Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84780[] = "Opera/9.80 (Windows ME; U; en) Presto/2.2.15 Version/10.00";
const char S84781[] = "Opera/9.80 (Windows ME; U; en) Presto/2.2.15 Version/10.01";
const char S84782[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TeamDRH ICS for GTablet Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84783[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TeamDRH ICS for GTablet Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84784[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:13.0) Gecko/X Thunderbird/13.0.1";
const char S84785[] = "Mozilla/5.0 (Linux; Android 4.0.3; PC1007 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S84786[] = "Mozilla/5.0 (Linux; Android 4.0.4; PC1007 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S84787[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SCH-i559 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84788[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SCH-i559 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S84789[] = "Nokia302/5.0 (14.53) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia302) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S84790[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; Micromax A57 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S84791[] = "Mozilla/5.0 (Java; U; en-us; nokian72) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S84792[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; WT19i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S84793[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.2m";
const char S84794[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE T82 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84795[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; R97B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84796[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia701) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S84797[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; W900 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84798[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; W900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84799[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA5700) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile";
const char S84800[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5700) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S84801[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6200 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S84802[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6200L Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/352";
const char S84803[] = "Mozilla/5.0 (Linux; Android 4.1.1; INHON PAPILIO G1 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84804[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-I9300) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S84805[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Incredimail/1.0.4.3811";
const char S84806[] = "Mozilla/5.0 (Linux; Android 4.2.2; NookTablet Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S84807[] = "Mozilla/5.0 (Linux; Android 4.2.1; Barnes & Noble Nook Tablet Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S84808[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F180L Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S84809[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F180S Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S84810[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F180K Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S84811[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P3100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S84812[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-P3100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S84813[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; Micromax_A30) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S84814[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Celkon A119 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84815[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; C5502 Build/10.3.A.0.295) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84816[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; C5502 Build/10.3.1.A.2.67)";
const char S84817[] = "SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; fa; sonyericssonw20i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S84818[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I9070) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S84819[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) Nokia5230";
const char S84820[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Mobile Safari/535.19 Silk-Accelerated=false";
const char S84821[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC EVO 3D X515m Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S84822[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A62) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S84823[] = "Mozilla/5.0 (Linux; Android 4.1.1; TAB-840_G Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S84824[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xolo_X900 Build/AZ510_ICS_02.36I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84825[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; Celkon A58 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84826[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84827[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; Fly IQ238 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84828[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84829[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6400L Build/IMM76L; CyanogenMod-TouchWiz 5.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84830[] = "Mozilla/5.0 (Linux; Android 4.0.4; HYUNDAI T7S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S84831[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E973 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84832[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; UMI X1S Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84833[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT15i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84834[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT11i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S84835[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S84836[] = "Mozilla/5.0 (Linux; Android 4.2.2; MZ601 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S84837[] = "Mozilla/5.0 (Linux; Android 4.0.4; i9400 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S84838[] = "UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 510) U2/1.0.0 UCBrowser/3.1.1.343 U2/1.0.0 Mobile";
const char S84839[] = "Mozilla/5.0 (Series40; Nokia109/04.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S84840[] = "Mozilla/5.0 (Linux; Android 4.0.3; A500 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S84841[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE66-3/300.21.012; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S84842[] = "LG-KS660 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S84843[] = "LG-KS660_CMCC Release/01.02.2009 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S84844[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; Docomo HT-03A Build/DRD08) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S84845[] = "NokiaN70-1/5.0737.3.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.4.0.65/27/352";
const char S84846[] = "NokiaN70/ 5.0741.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.4.0.65/27/351";
const char S84847[] = "Mozilla/5.0(compatible; Teleca Q7; 4.0.3; U; en) 240X480 LGE LG-LG9600";
const char S84848[] = "HTC_Touch2_T3333 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S84849[] = "KDDI-TS3X UP.Browser/6.2_7.2.7.1.K.7.113 (GUI) MMP/2.0";
const char S84850[] = "Mozilla/5.0 (Linux; U; arm; en, Nuvifone) AppleWebKit/523.13 (KHTML, like Gecko) Version/3.0 Mobile Safari/419.3 Qt Qtopia Iris/1.1.2";
const char S84851[] = "Mozilla/5.0 (Linux; U; arm; en, Nuvi) AppleWebKit/523.13 (KHTML, like Gecko) Version/3.0 Mobile Safari/419.3 Qt Qtopia Iris/1.1.2";
const char S84852[] = "MicromaxX226+/3.83; Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84853[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; md-us; A70BHT Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84854[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; NookColor Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84855[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NookColor Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84856[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nook Color Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84857[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nook Color Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84858[] = "Mozilla/5.0 (WindowsCE 6.0; rv:2.0.1) Gecko/X Firefox/4.0.1";
const char S84859[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:8.0) Gecko/X Thunderbird/8.0 Lightning/1.0";
const char S84860[] = "Mozilla/5.0 (Series40; NokiaX2-00/04.90; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15";
const char S84861[] = "MQQBrowser/2.4 Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3";
const char S84862[] = "MQQBrowser/2.4 Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S84863[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.8.1.5pre) Gecko/X Firefox/2.0.0.4 Navigator/9.0b2";
const char S84864[] = "MQQBrowser/34 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S84865[] = "MQQBrowser/34 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3";
const char S84866[] = "MQQBrowser/34 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3";
const char S84867[] = "MQQBrowser/34 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3";
const char S84868[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Iron/6.0.475.1 Chrome/6.0.475.1 Safari/X.534";
const char S84869[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Iron/6.0.475 Safari/534";
const char S84870[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; i-mobile i-note S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84871[] = "Mozilla/5.0 (Windows; N; ; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.2.2.6613 Safari/532.4";
const char S84872[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.6.5/X (like Mobile/8C148 Safari/6533.18.5)";
const char S84873[] = "Mozilla/5.0 (Linux; Android 4.0.3; ZP100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84874[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZP100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84875[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZOPO ZP100 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84876[] = "Mozilla/5.0 (Linux; Android 4.0.3; A90 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84877[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.8.1.7) Gecko/X Firefox/2.0.0.7 Flock/0.9.1.2";
const char S84878[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN78) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile";
const char S84879[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9001 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84880[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT22i Build/6.1.1.B.1.54) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S84881[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; Tablet816HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84882[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; TAB-PROTAB26 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84883[] = "JUC (Linux; U; 4.0.3; zh-cn; Micromax_P350; 480*764) UCWEB7.9.0.94/139/444";
const char S84884[] = "Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Safari/535.19 Silk-Accelerated=false";
const char S84885[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355";
const char S84886[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S84887[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; LG-E730 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84888[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2a1pre) Gecko/X Minefield/3.2a1pre";
const char S84889[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6210 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S84890[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P6210 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S84891[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P6210 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84892[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6210 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84893[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; S930 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84894[] = "Dalvik/1.4.0 (Linux; U; Android 4.0.3; S930 Build/MocorDroid2.3.5)";
const char S84895[] = "Mozilla/5.0 (Windows NT 6.0; rv:19.0) Gecko/X Firefox/19.0 slurp";
const char S84896[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:19.0) Gecko/X Firefox/19.0 slurp";
const char S84897[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S5660 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84898[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LEMON P100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84899[] = "Mozilla/5.0 (Linux; Android 4.0.3; Smartfren Tab 7 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S84900[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-I9003) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S84901[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Xolo_X500 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84902[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; EVOLVE_FX500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84903[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; Micromax_A60) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S84904[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A840S Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S84905[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Galaxy Nexus Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S84906[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S84907[] = "UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 800) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile";
const char S84908[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A820L Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S84909[] = "Mozilla/5.0 (Linux; Android 4.0.3; A101 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S84910[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S84911[] = "Mozilla/5.0 (X11; OpenBSD amd64) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+";
const char S84912[] = "Mozilla/5.0 (X11; OpenBSD i386) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+";
const char S84913[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; SHV-E120S Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84914[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; SHV-E120L Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84915[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; ST15i Build/4.0.2.A.0.84) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S84916[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST15i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S84917[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84918[] = "Mozilla/5.0 (Linux; Android 4.0.4; SH-02E Build/S4100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84919[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S84920[] = "Mozilla/5.0 (Linux; Android 4.1.2; B1-A71 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S84921[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; Onyx Calypso 9.7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84922[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E977 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84923[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P700 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84924[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E455 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S84925[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One X+ Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84926[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:25.0) Gecko/X Firefox/25.0";
const char S84927[] = "Mozilla/5.0 (Linux; Android 4.1.2; N9101 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S84928[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S6012 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S84929[] = "Mozilla/5.0 (Linux; Android 4.1.2; A200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S84930[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT11i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S84931[] = "Mozilla/5.0 (Linux; Android 4.0.4; BNTV600 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S84932[] = "Mozilla/5.0 (Linux; Android 4.2.1; R815 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84933[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaX7-00) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S84934[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6812B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84935[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6812 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S84936[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S84937[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562i Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S84938[] = "Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7210supernova) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S84939[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID7065 Build/ICS.MID7065.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S84940[] = "Mozilla/5.0 (Linux; Android 4.0.3; SC-02C Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S84941[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-7204 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S84942[] = "Mozilla/5.0 (Linux; Android 4.3; Galaxy Nexus Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S84943[] = "Mozilla/5.0 (Linux; Android 4.1.2; B1-710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S84944[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; es-ve; U8220 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84945[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; U8220PLUS Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84946[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Pulse with FLB-MOD Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84947[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SonyEricssonX1a/R3AA Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84948[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; CBW Blaze Build/DONUT) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S84949[] = "DoCoMo/2.0 F09A3(c500;TB;W24H16)";
const char S84950[] = "KDDI-CA3E UP.Browser/6.2_7.2.7.1.K.2.234 (GUI) MMP/2.0";
const char S84951[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:9.0.1) Gecko/X Firefox/9.0.1 Epic/9.0.1";
const char S84952[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_0; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S84953[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_2; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S84954[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_3; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S84955[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S84956[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S84957[] = "Mozilla/5.0 (Windows NT 6.1; rv:12.0) Gecko/X Firefox/12.0 slurp";
const char S84958[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.0) Gecko/X Firefox/12.0 slurp";
const char S84959[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.0) Gecko/X Firefox/12.0 slurp slurp";
const char S84960[] = "samsung-gt-s5620/UC Browser8.0.3.107/69/352";
const char S84961[] = "Mac OS X/10.7.4 (11E53); ExchangeWebServices/2.1 (125); Mail/5.2 (1278)";
const char S84962[] = "Mac OS X/10.7.3 (11D50d); ExchangeWebServices/2.1 (125); Mail/5.2 (1257)";
const char S84963[] = "Mac OS X/10.7.3 (11D50b); ExchangeWebServices/2.1 (125); Mail/5.2 (1257)";
const char S84964[] = "Mac OS X/10.7.3 (11D50); ExchangeWebServices/2.1 (125); Mail/5.2 (1257)";
const char S84965[] = "Mac OS X/10.7.3 (11D2001); ExchangeWebServices/2.1 (125); Mail/5.2 (1257)";
const char S84966[] = "Mozilla/5.0 (Windows NT 5.1; rv:13.0) Gecko/X Firefox/13.0 slurp";
const char S84967[] = "Mozilla/5.0 (Windows NT 5.1; rv:13.0) Gecko/X Firefox/13.0.1 slurp";
const char S84968[] = "Mozilla/5.0 (Windows NT 5.1; rv:13.0) Gecko/X Firefox/13.0.1 slurp slurp";
const char S84969[] = "Mozilla/5.0 (Windows NT 5.1; rv:13.0) Gecko/X Firefox/13.0 slurp slurp";
const char S84970[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E16i Build/2.1.1.A.0.53) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S84971[] = "Mozilla/5.0 (Windows; U; Windows NT 4.10; zh-CN) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19.1";
const char S84972[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.16; en-gb; PMP3074B Build/ECLAIR.eng.root.X.X) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84973[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.19; hr-hr; PMP3074B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S84974[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; TAB465EUK Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84975[] = "iBrowser/2.5 (E71-1;SymbianOS/9.2 Series60/3.1)";
const char S84976[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.2.0 Safari/534.34";
const char S84977[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.2.0 Safari/534.34";
const char S84978[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIANokia N81) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile";
const char S84979[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; O+ TABLET Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84980[] = "Micromax Q23/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S84981[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; A500 Build/JZO54K)";
const char S84982[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84983[] = "Mozilla/5.0 (Linux; Android 4.0.4; Fly_IQ442 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84984[] = "Mozilla/5.0 (Linux; Android 4.0.4; NWZ-Z1000Series Build/E200) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S84985[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GOCLEVER TAB A104 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S84986[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; MT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S84987[] = "SonyEricssonC902/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonc902) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S84988[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax P275 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S84989[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT875 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S84990[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; S58 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84991[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8190N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.14/271,1.9.14.271,2012-12-20 15:49, +0000, gb";
const char S84992[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGP312 Build/10.1.C.0.344) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S84993[] = "Mozilla/5.0 (Linux; Android 4.1.2; Q800 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S84994[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A800S Build/CM9 V2 by atulaphanI; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84995[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A800S Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84996[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A800S Build/IMM76L; CyanogenMod-Rom_Cook_CM9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S84997[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST18i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S84998[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Celkon*A86 Build/Celkon_A86) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S84999[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Karbonn A6 Build/MocorDroid4.0.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85000[] = "Mozilla/5.0 (Linux; Android 4.0.4; HS-9DTB4 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S85001[] = "Mozilla/5.0 (Linux; Android 4.0.4; HS-9DTB4-4GB Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S85002[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; FLY F45s Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85003[] = "Mozilla/5.0 (Linux; Android 4.1.1; O+ 8.6 Android Build/Oplus8.6_Android) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85004[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/8J2 Safari/7534.48.3";
const char S85005[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/8L1 Safari/7534.48.3";
const char S85006[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S85007[] = "Mozilla/5.0 (Linux; Android 4.0.4; ISW13HT Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85008[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85009[] = "Mozilla/5.0 (Linux; Android 4.0.4; Nexus S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85010[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L720 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S85011[] = "Mozilla/5.0 (Linux; Android 4.1.2; FPT V Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85012[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; HTC Incredible S Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31";
const char S85013[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HTC Incredible S Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S85014[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOLO X910 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85015[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.4.2m";
const char S85016[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.4.4m";
const char S85017[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.4.5m";
const char S85018[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Maxwell 2 Lite Build/1.0.0_X-14:29) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85019[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Maxwell 2 Lite Build/1.0.1_X-09:59) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85020[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S85021[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N5110 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S85022[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ALCATEL ONE TOUCH 8008W Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2 Mobile Safari/534.30";
const char S85023[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A57) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S85024[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; id; Micromax_A57) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Ponsel";
const char S85025[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Micromax_A110) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S85026[] = "Mozilla/5.0 (Linux; Android 4.2.1; Lenovo S920_ROW Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85027[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax P362 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S85028[] = "SAMSUNG GT-S3332 Opera/9.80 (J2ME/MIDP; Opera Mini/7.1.32840/31.1475; U; en) Presto/2.8.119 Version/11.10";
const char S85029[] = "Mozilla/5.0 (Linux; Android 4.0.4; KYL21 Build/019.0.3b00) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85030[] = "Mozilla/5.0 (Linux; Android 4.1.2; P-02E Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85031[] = "Mozilla/5.0 (Linux; Android 4.1.2; i-mobile IQ 5.1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S85032[] = "Mozilla/5.0 (Linux; Android 4.1.2; i-mobile IQ 5.1A Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85033[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST25i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S85034[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST25i Build/6.0.B.3.188) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S85035[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Explorer A310e Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S85036[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9300 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S85037[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S85038[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-S738C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S85039[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S85040[] = "DoCoMo/2.0 SH06B(c500;TB;W24H16)";
const char S85041[] = "Opera/9.80 (Android 1.6; Linux; Opera Mobi/ADR-X; U; en) Presto/2.7.81 Version/11.00";
const char S85042[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Milestone Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85043[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Milestone Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85044[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Milestone Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85045[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; Milestone Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85046[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Milestone Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S85047[] = "Opera/9.80 (Linux armv7l; Maemo; Opera Mobi/4; U; en-GB) Presto/2.5.28 Version/10.1";
const char S85048[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Ubuntu/10.04 Chromium/5.0.375.127 Chrome/5.0.375.127 Safari/533.4";
const char S85049[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Ubuntu/10.04 Chromium/5.0.375.127 Chrome/5.0.375.127 Safari/533.4";
const char S85050[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; HD2)";
const char S85051[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0;  ; LEO70)";
const char S85052[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; Lenovo 3GW101 Build/FRG83) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S85053[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; 3GW101 Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S85054[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; 3GW101 Build/FRG83) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S85055[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; MotoA953 Build/GWK74; CyanogenMod-72) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85056[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MotoA953 Build/MILS2_U6_4.1-22; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85057[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S85058[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8C148 Safari/6531.22.7";
const char S85059[] = "Opera/9.80 (Android 3.1; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.8.149 Version/11.10";
const char S85060[] = "Mozilla/5.0 (Windows NT 5.1; rv:8.0) Gecko/X Firefox/8.0 slurp";
const char S85061[] = "Mozilla/5.0 (Windows NT 5.1; rv:8.0.1) Gecko/X Firefox/8.0.1 slurp";
const char S85062[] = "Mozilla/5.0 (Windows NT 5.1; rv:8.0) Gecko/X Firefox/8.0 slurp slurp";
const char S85063[] = "JUC(Linux;U;Android2.3.5;Zh_cn;TECNO T3;320*480;)UCWEB7.8.0.95/139/32929";
const char S85064[] = "Mozilla/5.0 (Linux; Android 4.0.3; GOCLEVER TAB A103 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S85065[] = "Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB A103 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S85066[] = "Nokia 5320/UCWEB7.4.0.57/28/800";
const char S85067[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGT-X40 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85068[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-style Q2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85069[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-STYLE Q2 DUO Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85070[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; MAGPAD9.7Pro Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85071[] = "Mozilla/5.0 (Linux; Android 4.0.4; Spice Mi-500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85072[] = "Mozilla/5.0 (Linux; Android 4.0.4; A878 Duo Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85073[] = "Opera/9.80 (Windows 95) Presto/2.12.388 Version/12.10";
const char S85074[] = "JUC(Linux;U;Android2.3.6;Zh_cn;GT-S5302;240*320;)UCWEB7.8.0.95/139/444";
const char S85075[] = "Mozilla/5.0 (Windows; U;) AppleWebKit/532.0 (KHTML, like Gecko) Iron/3.0.197.0 Safari/532.0";
const char S85076[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85077[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85078[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-B5330 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.30496/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S85079[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; MG-391 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85080[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; A20 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85081[] = "SonyEricssonC510/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonc510) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S85082[] = "Mozilla/5.0 (Linux; Android 4.0.4; Starmobile ASTRA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85083[] = "Mozilla/5.0 (Linux; Android 4.0.4; CINK PEAX Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85084[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A868 Duo Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85085[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I8150 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85086[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8150 Build/JZO54K)";
const char S85087[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; TH-1127M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85088[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S85089[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; SPH-D710 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S85090[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; SPH-D710BST Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S85091[] = "Mozilla/5.0 (Linux; Android 4.0.4; SO-04D Build/7.0.D.1.137) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S85092[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T769 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S85093[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7300 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S85094[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P7300 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S85095[] = "Mozilla/5.0 (Linux; Android 4.2.1; aigoPad M80E Build/E8HD-ELITE-1280x768) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S85096[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB 9 V8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85097[] = "OneBrowser/3.1 (SAMSUNG-GT-S3850/S3850JPKK2)";
const char S85098[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; AT570 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85099[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; AT570 Build/JRO03C)";
const char S85100[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A830L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S85101[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A830K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S85102[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A830S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S85103[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A760S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85104[] = "Mozilla/5.0 (Linux; Android 4.1.1; Cink King Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85105[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85106[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT926 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85107[] = "Mozilla/5.0 (Linux; U; Android 4.1.2 jellybean customized # ariel-tbc; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85108[] = "Mozilla/5.0 (Linux; Android 4.1.2; PMP7280C3G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S85109[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T699 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85110[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 5.0; Trident/2.0)";
const char S85111[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-VM696 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85112[] = "Mozilla/5.0 (Linux; Android 4.0.4; i9400 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85113[] = "Mozilla/5.0 (Linux; Android 4.2.2; Infected Rezound Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85114[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI G520-5000 Build/HuaweiG520-5000) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S85115[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI G520-0000 Build/HuaweiG520-0000) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S85116[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAP5000TDUO Build/PrestigioPAP5000TDUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85117[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5282 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S85118[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ605 Build/I.7.1-39.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S85119[] = "Mozilla/5.0 (Linux; Android 4.0.4; NXM805ND_EN Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S85120[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36";
const char S85121[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85122[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85123[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A9+) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S85124[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; CELKON_A9+) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S85125[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC_Amaze_4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S85126[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Amaze 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S85127[] = "Mozilla/5.0 (Linux; Android 4.0.4; iBall Slide 3G7271 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85128[] = "Mozilla/5.0 (Linux; Android 4.0.4; iBall Slide 3G 7334 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S85129[] = "Mozilla/5.0 (Linux; Android 4.1.1; NABIXD-NV10A Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S85130[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A860S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85131[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAP5000TDUO Build/PrestigioPAP5000TDUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85132[] = "Mozilla/5.0 (Linux; Android 4.0.4; Haier HW-W910 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85133[] = "SEC-SGHF210/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85134[] = "SEC-SGHF210L/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85135[] = "SonyEricssonK770i/R8BC Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85136[] = "Facebook 4110.0 (iPhone; iPhone OS 4.3.3; en_US)";
const char S85137[] = "Facebook 4110.0 (iPhone; iPhone OS 4.3.5; en_US)";
const char S85138[] = "Facebook 4110.0 (iPhone; iPhone OS 4.3.1; en_US)";
const char S85139[] = "Facebook 4110.0 (iPhone; iPhone OS 4.3; en_US)";
const char S85140[] = "Facebook 4110.0 (iPhone; iPhone OS 4.3.2; en_US)";
const char S85141[] = "LG-GB190 Obigo/WAP2.0";
const char S85142[] = "SAMSUNG-GT-S8000C/1.0 SHP/R5 Release/02.01.2009 Browser/NetFront3.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S85143[] = "SAMSUNG-GT-S8000H/1.0 SHP/R5 NetFront/3.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S85144[] = "SAMSUNG-SGH-A412/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S85145[] = "Micromax X395/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S85146[] = "JUC(Linux;U;2.2.2;Zh_cn;ZTE-U V880;480*800;)UCWEB7.6.0.75/139/32014";
const char S85147[] = "Opera/9.80 (Series 60; Opera Mini/6.24095/27.1169; U; en) Presto/2.8.119 Version/11.10";
const char S85148[] = "Opera/9.80 (Series 60; Opera Mini/6.24455/24.871; U; en) Presto/2.5.25 Version/10.54";
const char S85149[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:11.0) Gecko/X Thunderbird/11.0 Lightning/1.3";
const char S85150[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nook Tablet Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85151[] = "SonyEricssonJ10i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonj10i/UC Browser7.9.0.102/69/352";
const char S85152[] = "SonyEricssonJ10i2/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonj10i2/UC Browser7.9.0.102/69/352";
const char S85153[] = "Mozilla/5.0 (Series40; Nokia3120classic/09.41; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S85154[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s8530) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S85155[] = "Mozilla/5.0 (X11; U; SunOS sun4u; en-US; rv:1.8.1.4) Gecko/X Thunderbird/2.0.0.4";
const char S85156[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S85157[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; HTC Wildfire S A510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S85158[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:14.0) Gecko/X Thunderbird/14.0";
const char S85159[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; Viettel i5 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85160[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352";
const char S85161[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Desire S Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85162[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Desire S Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85163[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Desire S Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85164[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; Edition Next; en) Presto/2.11.310 Version/12.50";
const char S85165[] = "iBrowser/Mini2.8 (NokiaX2-01/08.63)";
const char S85166[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; PB-S3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85167[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:12.0) Gecko/X Thunderbird/12.0.1";
const char S85168[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ALLVIEW P4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85169[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3 UCBrowser8.5.0.195/42/355";
const char S85170[] = "Mozilla/5.0 (Java; U; en-us; gt-s5233w) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S85171[] = "Mozilla/5.0 (Java; U; vi; gt-s5233w) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S85172[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID1042 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S85173[] = "Mozilla/5.0 (Linux; Android 4.0.3; Legend Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85174[] = "Mozilla/5.0 (Linux; Android 4.0.4; Legend2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85175[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; POLY PAD 9708HD Build/ICS.g12refM1005TR.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85176[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; H9500S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85177[] = "JUC (Linux; U; 4.1.1; zh-cn; GT-N7100; 720*1280) UCWEB8.7.4.225/145/800";
const char S85178[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7100) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S85179[] = "Opera/9.80 (Linux mips ; U; HbbTV/1.1.1 (; Sharp; ; ; ; ) CE-HTML/1.0 NETTV/3.0.0; en) Presto/2.6.33 Version/10.70";
const char S85180[] = "Opera/9.80 (Linux mips; U; HbbTV/1.1.1 (; Opera Software; videowebtv; ; ; ); en) Presto/2.10.287 Version/12.00";
const char S85181[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL838; 51.3.31.2; t12; ) ; ToshibaTP/1.2.0 (+DRM) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S85182[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL838; 55.4.02.0; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S85183[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL838; 53.2.17.1; t12; ) ; ToshibaTP/1.2.0 (+DRM) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S85184[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL838; 48.4.72.8; t12; ) ; ToshibaTP/1.1.0 () ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S85185[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL838; 55.4.02.0; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D) ; pl) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S85186[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL838; 55.4.02.0; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D) ; en) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S85187[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL838; 73.0.51.8; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S85188[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL838; 51.3.31.2; t12; ) ; ToshibaTP/1.2.0 (+DRM) ; it) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S85189[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL838; 51.3.31.2; t12; ) ; ToshibaTP/1.2.0 (+DRM) ; pt) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S85190[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL838; 85.3.88.6; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S85191[] = "Mozilla/5.0 (Linux mips; U;HbbTV/1.1.1 (+RTSP;DMM;Dreambox;0.1a;1.0;) CE-HTML/1.0; en) AppleWebKit/535.19 no/Volksbox QtWebkit/2.2";
const char S85192[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAP4040_DUO Build/PrestigioPAP4040DUO) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S85193[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT26ii Build/6.1.A.2.55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S85194[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A919 Duo Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S85195[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT15i Build/4.1.B.0.587) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S85196[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-S5830i Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S85197[] = "Mozilla/5.0 (Linux; Android 4.0.4; A8 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S85198[] = "Mozilla/5.0 (Linux; Android 4.0.4; G3 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S85199[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 992D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S85200[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI T8951 Build/HuaweiT8951) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S85201[] = "Mozilla/5.0 (Linux; Android 4.1.2; ALCATEL ONE TOUCH 8000D Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85202[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire C) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S85203[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire C) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S85204[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; N-05E Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85205[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; X920e Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85206[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X920e Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85207[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA5630d-1) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S85208[] = "Mozilla/5.0 (Linux; Android 4.1.2; 201M Build/9.8.2Q-34_SMJ-102) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85209[] = "Mozilla/5.0 (Linux; Android 2.3.7; ST27i Build/6.0.B.3.184) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S85210[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; UNUSUAL Vortex Dual Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85211[] = "Mozilla/5.0 (Linux; Android 4.1.2; A200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S85212[] = "Mozilla/5.0 (Linux; Android 4.1.2; bq Elcano Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85213[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; TECNO Q1 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/33567";
const char S85214[] = "Mozilla/5.0 (Linux; Android 4.0.4; Miracle Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85215[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; WALTON_Primo-X1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85216[] = "Mozilla/5.0 (Linux; Android 4.0.4; NABI2-NV7A Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S85217[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-01E Build/9.1.C.0.475) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85218[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; AVEO X6 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85219[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T989 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S85220[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S85221[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5670) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S85222[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT18i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S85223[] = "Mozilla/5.0 (Linux; Android 4.1.2; Spice Mi-515 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85224[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; SKY_NANO Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85225[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5280 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85226[] = "Mozilla/5.0 (Linux; Android 4.0.3; SC-02C Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85227[] = "Mozilla/5.0 (Linux; U; Android 2.3;vi-vn; M9000 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85228[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; vi_vn; M9000 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.2.1 Mobile (Dragonfruit)";
const char S85229[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Micromax_A116) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S85230[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST27i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S85231[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E15i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S85232[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZTE Blade G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85233[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9082) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S85234[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9152 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S85235[] = "NokiaX2-02/2.0 (11.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; NokiaX2-02) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S85236[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9295 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S85237[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; MDA Vario/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85238[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; MDA Vario/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85239[] = "LG-KP170-orange/V10b Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S85240[] = "LG-KP170 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S85241[] = "Mot-EX300/B000 Browser/NetFront/3.5 MMS/Obigo-MMS/Q05A SyncML/HW-SyncML/1.0 Java/HWJa/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Player/QTV-Player/5.3";
const char S85242[] = "LG-GU295/V10i Teleca/Q7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S85243[] = "SAMSUNG-GT-I6220/1.0 SHP/VPP/R5 Jasmine/1.0 NexPlayer/2.12 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S85244[] = "LAVA-A10/SW1.0.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85245[] = "Perfect%20Browser-iPad/2.6 CFNetwork/485.12.7 Darwin/10.4.0";
const char S85246[] = "MobileRSSFree-iPad/3.1.4 CFNetwork/485.12.7 Darwin/10.4.0";
const char S85247[] = "browseripad/23286 CFNetwork/485.12.7 Darwin/10.4.0";
const char S85248[] = "MobileRSS-iPad/3.1.4 CFNetwork/485.12.7 Darwin/10.4.0";
const char S85249[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.1) chromeframe/4.0";
const char S85250[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E) chromeframe/4.0";
const char S85251[] = "Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.8.1.16) Gecko/X Firefox/2.0.0.16 Flock/1.2.4";
const char S85252[] = "Mozilla/5.0 (Windows; U; Win 9x 4.90; en-US; rv:1.8.1.16) Gecko/X Firefox/2.0.0.16 Flock/1.2.4";
const char S85253[] = "Mozilla/5.0 (Series40; Nokia7230/06.90; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S85254[] = "Mozilla/5.0 (Series40; NokiaC3-01/05.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S85255[] = "JUC (Linux; U; 2.3.3; zh-cn; C8650; 320*480) UCWEB7.9.4.145/139/800";
const char S85256[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; LT15i Build/4.0.2.A.0.42; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85257[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGT-A20 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85258[] = "SonyEricssonJ10i2/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonj10i2) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S85259[] = "SonyEricssonJ10i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonj10i) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S85260[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, gb";
const char S85261[] = "Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia6303iclassic) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S85262[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.0 Mobile Safari/535.19 Silk-Accelerated=true";
const char S85263[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LG-VM670 Build/JRO03L; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85264[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-VM670 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85265[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.0.181/145/352";
const char S85266[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6500D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S85267[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; M3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85268[] = "Mozilla/5.0 (Linux; Android 4.1.1; A953 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85269[] = "Mozilla/5.0 (Linux; Android 4.1.2; A953 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85270[] = "NOKIA-Lumia 800/UCWEB2.6.0.213/47/32079";
const char S85271[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; ALCATEL ONE TOUCH 985N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S85272[] = "NokiaC2-06/2.0 (07.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-06) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S85273[] = "Mozilla/5.0 (Linux; Android 4.0.4; KM-S200 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85274[] = "Mozilla/5.0 (Linux; Android 4.0.4; KM-S220 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85275[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; jblend) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile/HS-T39_TD/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0 Config/CLDC-1.1";
const char S85276[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; jblend) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile/HS-T39_TD/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0 Config/CLDC-1.1, /HS-T39_TD/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0 Config/CLDC-1.1";
const char S85277[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; jblend) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile/HS-T39_TD/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0 Config/CLDC-1.1,/HS-T39_TD/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0 Config/CLDC-1.1";
const char S85278[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; Tablet 816HDC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85279[] = "Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 7 Build/1.0.3 X-12:51) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S85280[] = "OneBrowser/3.1 (Nokia5233/51.1.002/sw_platform=S60;sw_platform_version=5.0;java_build_version=1.4.43)";
const char S85281[] = "MQQBrowser/2.7 Mozilla/5.0 (Linux; U; Android 2.2.2; in-id; A850 Build/SDB-P2_1.3.20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85282[] = "MQQBrowser/2.7 Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; GFIVE A57 Build/S21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85283[] = "Mail/1499 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookPro8%2C1)";
const char S85284[] = "Mail/1499 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (iMac12%2C2)";
const char S85285[] = "Mozilla/5.0 (Linux; Android 4.0.4; A89 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85286[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST25i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S85287[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PIRANHA BUSINESS TAB 7\" Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85288[] = "Mozilla/5.0 (Linux; Android 4.0.4; V1277 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S85289[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R760X Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S85290[] = "UCWEB/2.0 (Linux; U; Adr 2.3.9; en-US; Celkon_A_85) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S85291[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; GT-S5360 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S85292[] = "Instagram 3.4.2 (iPhone3,1; iPhone OS 4.3.3; en_US; en) AppleWebKit/420+";
const char S85293[] = "Instagram 3.4.2 (iPhone3,1; iPhone OS 4.3.5; en_US; en) AppleWebKit/420+";
const char S85294[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S85295[] = "Mozilla/5.0 (Linux; Android 4.2.2; ADR6425LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85296[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire X Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S85297[] = "Mozilla/5.0 (Linux; Android 4.1.2; MI-ONE Plus Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85298[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6602 Build/10.1.1.A.1.253) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S85299[] = "Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 10 FHD Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S85300[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; LG-E610 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85301[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-E612 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85302[] = "sprd-S5292/1.0 Linux/2.6.35.7 Android/4.0.4 Release/04.01.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S85303[] = "sprd-S5292/1.0 Linux/2.6.35.7 Android/4.0.4 Release/04.16.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S85304[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; SCH-I959 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S85305[] = "Mozilla/5.0 (Linux; Android 4.1.1; TBD753 Build/Zeki.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S85306[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Titanium S5 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S85307[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; vivo Y11 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85308[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Micromax A110Q Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S85309[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LS970 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S85310[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; STX ULTRA Build/STXSTX_ULTRA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85311[] = "Mozilla/5.0 (Linux; Android 4.0.4; KYL21 Build/019.0.3b00) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85312[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HTC Incredible S Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S85313[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; HTC Incredible S Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S85314[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Wildfire_S_A510e) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S85315[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Wildfire_S_A510e) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S85316[] = "Mozilla/5.0 (Linux; Android 4.1.2; N9810 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85317[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 10 Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us";
const char S85318[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 10 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.8/620,2.0.8.620,2013-09-06 15:06, -0400, us";
const char S85319[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-US; GT-P1000 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S85320[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S560 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85321[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A25) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S85322[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A860L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85323[] = "Mozilla/5.0 (Linux; Android 4.0.3; chagall Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S85324[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6312 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S85325[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:24.0) Gecko/X Firefox/24.0 slurp";
const char S85326[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S85327[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S85328[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S85329[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari/7534.48.3 Sleipnir/2.4.5m";
const char S85330[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Safari/7534.48.3 Sleipnir/2.4.5m";
const char S85331[] = "HTC_S620 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S85332[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) HTCS620;Smartphone;320x240";
const char S85333[] = "HTC-Excalibur Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S85334[] = "LG-KG290UAE Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S85335[] = "LG-KG290 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S85336[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-KH5200 Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S85337[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; LG-KH5200 Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S85338[] = "DoCoMo/2.0 P905i(c100;TB;W24H15)";
const char S85339[] = "DoCoMo/2.0 N08A3(c500;TB;W24H16)";
const char S85340[] = "alcatel-ot-807a";
const char S85341[] = "DoCoMo/2.0 F06B(c500;TB;W24H16)";
const char S85342[] = "Mozilla/5.0(Linux;U;Android2.3.4;Zh_cn;GT-I9003;480*800;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S85343[] = "JUC(Linux;U;Android2.3.4;Zh_cn;GT-I9003;480*800;)UCWEB7.8.0.95/139/352";
const char S85344[] = "JUC(Linux;U;Android2.3.6;Zh_cn;GT-I9003;480*800;)UCWEB7.8.0.95/139/444";
const char S85345[] = "Mozilla/5.0 (Windows NT 6.1; rv:6.0) Gecko/X Thunderbird/6.0 Lightning/1.0b5";
const char S85346[] = "nokia6120c/UC Browser8.0.3.107/69/444";
const char S85347[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.8b4) Gecko/X Firefox/1.4";
const char S85348[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-I9100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S85349[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-I9100G Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S85350[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-I9100T Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S85351[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:13.0) Gecko/X Thunderbird/13.0.1";
const char S85352[] = "Mozilla/5.0 (Linux; Android 4.0.4; S22 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85353[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE65) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile";
const char S85354[] = "Mozilla/5.0 (Java; U; en-us; nokiae65-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S85355[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-LU5400 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85356[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT687 Build/V2.27D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85357[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1";
const char S85358[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1";
const char S85359[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1";
const char S85360[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Malata SMBA1002 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85361[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S85362[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8650+ Build/V100R001C92B861) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85363[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HW-HUAWEI_C8650/C8650V100R001C92B825; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S85364[] = "Instapaper for Android 1.2";
const char S85365[] = "SpiceM5395/MTK Release/01.01.2012 Browser/wap2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85366[] = "MacOutlook/14.0.0.X (Intel Mac OS X 10.8.2)";
const char S85367[] = "MacOutlook/14.0.0.X (Intel Mac OS X 10.8.3)";
const char S85368[] = "MacOutlook/14.0.0.X (Intel Mac OS X 10.8.4)";
const char S85369[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_Rhyme_S510b; en-hk) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S85370[] = "Mozilla/5.0 (Linux; Android 4.0.4; SO-03D Build/6.1.F.0.106) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S85371[] = "TECHAIN50_11B_HW (MRE\\3.0.00(2500) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) Masscom_P12_T738_M00_S03_X Release/2012.11.16 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 vi-VN";
const char S85372[] = "TECHAIN50_11B_HW (MRE\\3.0.00(2500) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) Masscom_P12_T738_M00_S04_X Release/2013.01.11 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 vi-VN";
const char S85373[] = "Mozilla/5.0 (Windows; U; Windows NT 9.0; en-US; rv:1.9.1.11) Gecko/X SeaMonkey/2.0.6";
const char S85374[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7300 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85375[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7300 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85376[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; N400) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S85377[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S85378[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; Karbonn A2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S85379[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Karbonn_A2 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S85380[] = "UCWEB/2.0 (Linux; U; Adr 2.2; en-US; GT-I5510) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S85381[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT25i Build/4.1.B.0.631) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S85382[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328d Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31";
const char S85383[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; LG-P500 Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S85384[] = "Nokia7230/5.0 (10.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7230) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S85385[] = "Mozilla/5.0 (Linux; Android 4.1.1; GSmart Maya M1 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85386[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:20.0) Gecko/X Firefox/20.0 TenFourFox/7450";
const char S85387[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6700s) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S85388[] = "Mozilla/5.0 (Linux; Android 4.1.1; Z120 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85389[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S85390[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire SV Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S85391[] = "Tumblr/117 CFNetwork/646.4 Darwin/14.0.0";
const char S85392[] = "Facebook/X CFNetwork/672.0.2 Darwin/14.0.0";
const char S85393[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XOLO X910 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85394[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI G520-5000 Build/HuaweiG520-5000) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S85395[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI G520-0000 Build/HuaweiG520-0000) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S85396[] = "Mozilla/5.0 (Linux; Android 4.2.1; Lenovo K900_ROW Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85397[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-us; A9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S85398[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-S7568 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S85399[] = "Mozilla/5.0 (Linux; Android 4.0.4; V1277 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85400[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-S5303 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S85401[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S5303 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S85402[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8552) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S85403[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A89) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S85404[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E440 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85405[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A26) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S85406[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I8190 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/golden";
const char S85407[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I717 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/quincyatt";
const char S85408[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848i Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85409[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848 Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85410[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P990 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85411[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LU6200 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85412[] = "Mozilla/5.0 (Linux; Android 4.3; SGH-I337M Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85413[] = "Mozilla/5.0 (Linux; Android 4.1.2; PantechP9070 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85414[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S85415[] = "Mozilla/5.0 (Linux; Android 4.0.4; i9500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85416[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9082 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S85417[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) SP; 240x320; HTC_S710/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85418[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Vodafone/1.0/HTC_s710/1.22.172.3";
const char S85419[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Vodafone/1.0/HTC_s710/1.25.163.1";
const char S85420[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Vodafone/1.0/HTC_s710/1.15.163.2";
const char S85421[] = "Videocon V1760/V1760_00_V10A_IND_01_09_15_10/04.09.2009 Browser/Teleca Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S85422[] = "Videocon V1533/22.11.2007 Browser/wap2.0 Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85423[] = "Videocon V1635/22.11.2007 Browser/wap2.0 Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85424[] = "Videocon_V1655_Java/V1655-JAVA-91-V10a-MAR-30-2010 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85425[] = "SAMSUNG-GT-M2310/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S85426[] = "SonyEricssonZ770i/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S85427[] = "ZTE-G-S215/WAP2.0";
const char S85428[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-sg; GT-I8520 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S85429[] = "DoCoMo/2.0 P906i(c100;TB;W24H15)";
const char S85430[] = "LG-KD876/v1.0 Arena/L2.3.3 Release/5.2.2008 Browser/Teleca/QLine Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85431[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S7230B/1.0; U; Bada/1.0; pt-br) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 OPN-B";
const char S85432[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Cricket Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85433[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Cricket Build/FRG22D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85434[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.10.229 Version/11.60";
const char S85435[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows ME) Opera 6.01 [en]";
const char S85436[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows ME) Opera 6.0 [de]";
const char S85437[] = "Opera/6.05 (Windows ME; U) [de]";
const char S85438[] = "Opera/9.50 (Windows 98; U; en)";
const char S85439[] = "Opera/9.52 (Windows 98; U; en)";
const char S85440[] = "Opera/9.51 (Windows 98; U; en)";
const char S85441[] = "Mozilla/5.0 (Linux; U; Android 1.05; en-us; p10t Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S85442[] = "Mozilla/5.0 (Linux; U; Android 1.0; en-us; dream) AppleWebKit/5.25.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S85443[] = "Mozilla/5.0 (Linux; U; Android 1.0; en-us; dream) AppleWebKit/525.10  (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S85444[] = "GoogleEarth/6.0.3.2197(Macintosh;Mac OS X (10.7.2);en;kml:2.2;client:Free;type:default)";
const char S85445[] = "GoogleEarth/6.0.3.2197(Macintosh;Mac OS X (10.7.3);en;kml:2.2;client:Free;type:default)";
const char S85446[] = "GoogleEarth/6.0.3.2197(Macintosh;Mac OS X (10.7.4);en;kml:2.2;client:Free;type:default)";
const char S85447[] = "GoogleEarth/6.0.3.2197(Macintosh;Mac OS X (10.7.5);en;kml:2.2;client:Free;type:default)";
const char S85448[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_PH39100/1.63.502.4; en-us) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S85449[] = "Mozilla/5.0 (Maemo; Linux armv7l; rv:13.0) Gecko/X Firefox/13.0 Fennec/13.0";
const char S85450[] = "Mozilla/5.0 (Maemo; Linux armv7l; rv:13.0) Gecko/X Firefox/13.0.1 Fennec/13.0";
const char S85451[] = "GoogleEarth/6.2.2.7373(Macintosh;Mac OS X (10.7.4);en;kml:2.2;client:Free;type:default)";
const char S85452[] = "GoogleEarth/6.2.2.6613(Macintosh;Mac OS X (10.7.4);en;kml:2.2;client:Free;type:default)";
const char S85453[] = "GoogleEarth/6.2.2.7373(Macintosh;Mac OS X (10.7.5);en;kml:2.2;client:Free;type:default)";
const char S85454[] = "GoogleEarth/6.2.2.6613(Macintosh;Mac OS X (10.7.5);en;kml:2.2;client:Free;type:default)";
const char S85455[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Flipboard/1.9.7";
const char S85456[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/1.9.8";
const char S85457[] = "320x240; VZW; UTStar-XV6175.1; Windows Mobile 6.5 Standard; (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.2.2.51";
const char S85458[] = "320x240; VZW; UTStar-XV6175; Window Mobile 6.1 Standard; (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.2.2.51";
const char S85459[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2a1pre) Gecko/X Minefield/3.6a1pre";
const char S85460[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-01D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S85461[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/EVO_3D/3.28.161.3; es-es) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S85462[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/EVO_3D/3.28.161.3; de-de) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S85463[] = "Mozilla/5.0 (X11; Linux x86_64; HTC/EVO_3D/3.28.165.2; it-it) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S85464[] = "Mozilla/5.0 (Symbian3; U; en-us; NokiaE6-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/51/355/UCWEB Mobile";
const char S85465[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5260) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S85466[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.6) Gecko/X Firefox/2.0 Wyzo/0.5.3";
const char S85467[] = "Nokia5300/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5300) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S85468[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-us; NookTablet Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85469[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; NookTablet Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85470[] = "SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonj20i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S85471[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; iDx5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85472[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromax) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S85473[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6500D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S85474[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S85475[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; HS-F1 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85476[] = "Mozilla/5.0(Linux;U;Android4.0.3;Zh_cn;GT-P5100;800*1232;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S85477[] = "JUC(Linux;U;Android4.0.3;Zh_cn;GT-P5100;800*1232;)UCWEB7.8.0.95/139/351";
const char S85478[] = "JUC(Linux;U;Android4.0.4;Zh_cn;GT-P5100;800*1232;)UCWEB7.8.0.95/139/444";
const char S85479[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S85480[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S85481[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; LG-E405) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S85482[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua_Sx) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S85483[] = "OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85484[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A760S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S85485[] = "Mozilla/5.0 (Linux; Android 4.1.2; a200 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S85486[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P925 Build/ICS) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S85487[] = "sprd-N7000/1.0 Linux/2.6.35.7 Android/4.0.9 Release/08.15.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S85488[] = "sprd-N7000/1.0 Linux/2.6.35.7 Android/4.0.9 Release/10.11.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S85489[] = "sprd-N7000/1.0 Linux/2.6.35.7 Android/4.0.9 Release/09.13.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S85490[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9220 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85491[] = "MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-I9220 Build/JZO54K.I9220ZCLSF;800*1280)";
const char S85492[] = "Nokia5330-1d/5.0 (06.88) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5330-1d) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S85493[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-D700 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85494[] = "Mozilla/5.0 (Windows; U; ; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.5.3.X (Second Life Release; default skin) Safari/533.3";
const char S85495[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S7562 Build/IMM76I) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S85496[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S85497[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; FPT IV Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85498[] = "Mozilla/5.0 (Linux; Android 4.2.2; redhookbay Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S85499[] = "Mozilla/5.0 (Linux; Android 4.2.2; Transformer Prime Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S85500[] = "Mozilla/5.0 (Linux; Android 4.2.1; Mi-510 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S85501[] = "Mozilla/5.0 (Linux; Android 4.0.4; S500 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85502[] = "Mozilla/5.0 (Linux; Android 4.1.2; ThL W8 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85503[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EVO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (EVO, 42, 1.9.1)";
const char S85504[] = "Mozilla/5.0 (Linux; Android 4.1.1; CINK SLIM Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85505[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Mobiistar Touch S08 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85506[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-; Mobiistar Touch S08 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85507[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-B5330 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85508[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP Mobile";
const char S85509[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A390_ROW Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85510[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9308 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S85511[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P875 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85512[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; E120L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85513[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A870K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85514[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A870S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85515[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A870L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85516[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; DM014SH Build/S1004) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85517[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; DM014SH Build/S1008) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85518[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Micromax A115 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S85519[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; GT-I9508 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S85520[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HUAWEI; 4Afrika)";
const char S85521[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC EVO 3D X515m Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S85522[] = "Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6303classic) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S85523[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian8-00) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile  3gpp-gba";
const char S85524[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC 802w Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85525[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR6425LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S85526[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; Nokia N96) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S85527[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E617G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85528[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN86 8MP) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S85529[] = "Mozilla/5.0 (Linux; Android 4.1.1; ST10 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S85530[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE65) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S85531[] = "SAMSUNG-GT-C3322i/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85532[] = "Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3120classic) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S85533[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I777 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i777";
const char S85534[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I777 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i777";
const char S85535[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I777 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i777";
const char S85536[] = "Mozilla/5.0 (Linux; Android 4.3; GT-P7510 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S85537[] = "Mozilla/5.0 (Linux; Android 4.0.4; MI-ONE Plus Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85538[] = "Mozilla/5.0 (Linux; Android 4.2.2; SO-04E Build/10.3.1.B.0.224) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85539[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E970 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S85540[] = "Mozilla/5.0 (Linux; Android 4.0.3; Sony Tablet S Build/TISU0143) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S85541[] = "SAMSUNG-M7500/M7500ACHL1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S85542[] = "ZTE-R225/ES-ATT-USA-P103D13V1.1.5 OBIGO/Q03C WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85543[] = "ZTE-R225/ES-ATT-USA-P103D13V1.1.7 OBIGO/Q03C WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85544[] = "Huawei/1.0/G6603/Browser/obigo-browser/q03c";
const char S85545[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/Vodafone1230/ENX_P180A1V1.1.0 IE Mobile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85546[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/Vodafone1230/IE Mobile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85547[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/Vodafone1231/LSO_P180A1V1.0.1/MIDP-2.0 Configuration/CLDC-1.1";
const char S85548[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/Vodafone1231/ESL_P180A1V1.0.9 IE Mobile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85549[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/Vodafone1231/IE Mobile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85550[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/Vodafone1231/PTG_P180A1V1.1.0 IE Mobile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85551[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/Vodafone1230/INX_P180A1V1.1.4 IE Mobile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85552[] = "SAMSUNG-SGH-T739/T739UVHB2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S85553[] = "SCH-R600 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S85554[] = "Huawei/1.0/0HuaweiG3511/WAP2.0/Obigo-Browser/Q03C MMS/Obigo-MMS/1.2";
const char S85555[] = "LG-GR500FD/V10a Teleca/Q7.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S85556[] = "ZTE-T930 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 firmware/T930 T06";
const char S85557[] = "ZTE-G N290/WAP2.0 Profile/MIDP-2.0";
const char S85558[] = "DoCoMo/2.0 P01A(c100;TB;W24H15)";
const char S85559[] = "Nokia5233/UC Browser7.4.0.65/50/351";
const char S85560[] = "Nokia5800 XpressMusic/UC Browser7.8.0.95/50/355";
const char S85561[] = "NokiaN73/UC Browser7.8.0.95/28/351";
const char S85562[] = "DoCoMo/2.0 SH08B(c500;TB;W24H16)";
const char S85563[] = "DoCoMo/2.0 SH08C(c500;TB;W24H14)";
const char S85564[] = "DoCoMo/2.0 SH08A(c500;TB;W24H14)";
const char S85565[] = "Nokia5310XpressMusic/2.0 (09.42) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5310xpressmusic/UC Browser8.0.3.107/69/444";
const char S85566[] = "Opera/9.80 (Macintosh; Intel Mac OS X; Opera Tablet/23731; U; en) Presto/2.9.201 Version/11.50";
const char S85567[] = "Opera/9.80 (Macintosh; Intel Mac OS X; Opera Mobi/23731; U; en) Presto/2.9.201 Version/11.50";
const char S85568[] = "SonyEricssonJ10i2/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonj10i2/UC Browser8.0.3.107/69/444";
const char S85569[] = "JUC (Linux; U; 2.3.5; zh-cn; Lenovo_A520; 480*800) UCWEB7.9.3.103/139/32707";
const char S85570[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S85571[] = "Mozilla/5.0 (Mobile; rv:12.0) Gecko/12.0 Firefox/12.0";
const char S85572[] = "Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia6303classic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/352/UCWEB Mobile";
const char S85573[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5236) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/352/UCWEB Mobile";
const char S85574[] = "Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia3120classic) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S85575[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85576[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85577[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A900 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85578[] = "Mozilla/5.0 (Windows NT 5.1; rv:18.0) Gecko/X Thunderbird/18.0";
const char S85579[] = "JUC (Linux; U; 3.2; zh-cn; GT-P6200; 600*976) UCWEB7.9.0.94/139/444";
const char S85580[] = "JUC (Linux; U; 2.3.4; zh-cn; A75; 320*480) UCWEB7.9.0.94/139/444";
const char S85581[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5670 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S85582[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S85583[] = "Instagram 3.2.0 (iPhone4,1; iPhone OS 6.1; en_US) AppleWebKit/420+";
const char S85584[] = "Instagram 3.2.0 (iPhone4,1; iPhone OS 6.1.2; en_US) AppleWebKit/420+";
const char S85585[] = "Instagram 3.2.0 (iPhone4,1; iPhone OS 6.1.3; en_US) AppleWebKit/420+";
const char S85586[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6102 Build/GINGERBREAD; 240*320) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444";
const char S85587[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; WT19i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S85588[] = "Mozilla/5.0 (Linux; Android 4.0.4; Nexus S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S85589[] = "Mozilla/5.0 (Linux; Android 4.0.4; BC1003 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S85590[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8681 Build/HuaweiU8681) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S85591[] = "Mozilla/5.0 (Linux; Android 4.1.1; M040 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S85592[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100G Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S85593[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S85594[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S85595[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MAXIMUS max903 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85596[] = "JUC (Linux; U; 2.3.5; zh-cn; Micromax_A27; 320*480) UCWEB7.9.0.94/139/444";
const char S85597[] = "Mozilla/5.0 (Linux; Android 4.0.4; NXM805ND_EN Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S85598[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 101 Titanium Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S85599[] = "Mozilla/5.0 (Linux; Android 4.0.4; A21 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S85600[] = "Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A21 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S85601[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ605 Build/I.7.1-39.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S85602[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; U2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85603[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S85604[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2S Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S85605[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S85606[] = "Mozilla/5.0 (Linux; Android 4.0.4; A8 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85607[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT926 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85608[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT926 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85609[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S85610[] = "Mozilla/5.0 (Linux; Android 4.2.1; iOCEAN X7 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85611[] = "Mozilla/5.0 (Linux; Android 4.2.2; Evo V 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85612[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-i705 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S85613[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85614[] = "Mozilla/5.0 (Linux; Android 4.0.3; A101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S85615[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A12 Build/RheaTK) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S85616[] = "Mozilla/5.0 (Linux; Android 4.1.2; Primo-N1 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S85617[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E435 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85618[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E435f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85619[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; ZP800H Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85620[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S85621[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1SC Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S85622[] = "Nokia3110c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3110c) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S85623[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6802) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S85624[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5570 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S85625[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5570I Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S85626[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5830 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S85627[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-S5830 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S85628[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-7204 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85629[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F260S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85630[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAP4040_DUO Build/PrestigioPAP4040DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85631[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6312) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S85632[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I407 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85633[] = "Mozilla/5.0 (Linux; Android 4.0.4; WT19i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85634[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3653) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S85635[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-B5330 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S85636[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT875 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85637[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae66-1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S85638[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE66) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S85639[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A89 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S85640[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8190 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S85641[] = "SAMSUNG-SGH-J200/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S85642[] = "NokiaN93i-1/10.0.025 SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85643[] = "SonyEricssonT250i/R5CA005 TelecaBrowser/1.1.14.20";
const char S85644[] = "LG-GM600/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S85645[] = "ZTE-C_S265/VE_S265_NBTL25FV1.0.0B17 SMIT-BROWSER/V2.0.0 CTC/1.0";
const char S85646[] = "SAMSUNG-SGH-A411/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S85647[] = "Nokia6303iclassic/5.0 (09.83) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303iclassic/UC Browser7.9.0.102/69/352";
const char S85648[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; N-04C Build/A1010101) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85649[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; N-04C Build/AX)";
const char S85650[] = "MicromaxX550/MIDP2.0/CLDC1.1/Screen-240X400";
const char S85651[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.2.0.0 Chrome/13.0.782.218 Safari/535.1";
const char S85652[] = "Mozilla/4.0 (Windows; U; NT4.0;) NetFront/3.0";
const char S85653[] = "AppleCoreMedia/1.0.0.8B117 (iPod; U; CPU OS 4_1 like Mac OS X; en_us)";
const char S85654[] = "Mozilla/5.0 (Linux; U; Android 2.1-update2; en-; ZiiO7 Build/v1.00.13-ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S85655[] = "Mozilla/5.0 (Linux; U; Android 2.1-update2; en-us; ZiiO7 Build/v1.00.28-ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S85656[] = "Flipboard/1.8.3 CFNetwork/548.1.4 Darwin/11.0.0";
const char S85657[] = "MicromaxQ80/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S85658[] = "Mozilla/5.0 (Windows; U; Windows NT 4.10; zh-CN; rv:1.9.0.8) Gecko/X Firefox/3.0.8 (.NET CLR 3.5.30729)";
const char S85659[] = "Mozilla/5.0 (Windows NT 6.1; rv:5.0) Gecko/X Firefox/5.0 slurp";
const char S85660[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:5.0) Gecko/X Firefox/5.0 slurp";
const char S85661[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:12.1) Gecko/X Firefox/12.1 PaleMoon/12.1";
const char S85662[] = "Mozilla/5.0 (en-US) AppleWebKit/536.8 (KHTML, like Gecko) Chrome/19.0.1084.36 Safari/536.8";
const char S85663[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i667)";
const char S85664[] = "Mozilla/5.0 Gecko/X Firefox/6.0.2";
const char S85665[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; Mediacom 810C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85666[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; SmartPad810c Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85667[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; Mediacom 810C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85668[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; en) Presto/2.10.229 Version/11.61";
const char S85669[] = "MicromaxX261/Q03C MAUI-browser/";
const char S85670[] = "Mac OS X/10.8 (12A269); ExchangeWebServices/3.0 (157); Mail/6.0 (1485)";
const char S85671[] = "Mac OS X/10.8.1 (12B19); ExchangeWebServices/3.0 (157); Mail/6.0 (1486)";
const char S85672[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SHW-M290S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85673[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M290S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85674[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-za; Android Tablet PC Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S85675[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-C_N780/N780V1.0.0B06) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85676[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-C_N780/N780V1.0.0B04) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85677[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-C_N780/N780V1.0.0B05) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85678[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-C N780 Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S85679[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID9742 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S85680[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.2.0 Safari/534.34";
const char S85681[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; ZTE-BLADE Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S85682[] = "iBrowser/Mini2.8 (Nokia200/11.56)";
const char S85683[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; K-Touch_W610 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85684[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85685[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; ST25i Build/6.0.B.3.188) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85686[] = "Mozilla/5.0 (Symbian3; U; en-us; NokiaX7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/51/444/UCWEB Mobile";
const char S85687[] = "JUC (Linux; U; 2.2.2; zh-cn; Karbonn_A1; 240*320) UCWEB7.9.0.94/139/444";
const char S85688[] = "Instagram 3.2.0 (iPad2,1; iPhone OS 6.0.1; en_US) AppleWebKit/420+";
const char S85689[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T1004 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85690[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; G1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85691[] = "Mozilla/5.0 (Linux; Android 4.1.1; TM-8041HD Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S85692[] = "Mozilla/5.0 (Linux; Android 4.2.1; XT875 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85693[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT875 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85694[] = "Mozilla/5.0 (Linux; Android 4.0.3; Explay Surfer 7.02 Build/ICS.g12refM703A1HZ1.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S85695[] = "Mozilla/5.0 (Linux; Android 4.0.3;  Explay Surfer 7.02 Build/ICS.g12refM703A1HZ1.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19";
const char S85696[] = "SAMSUNG-GT-I8350/UCWEB2.8.0.249/47/355";
const char S85697[] = "HS-U850/1.0 Linux/2.6 Android/2.3 Browser/AppleWebKit533.1 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S85698[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-U850 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85699[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar Touch Lai 502 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85700[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia500) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile  3gpp-gba";
const char S85701[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA5700) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S85702[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S85703[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; HTC One X Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S85704[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S85705[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S85706[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9001 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S85707[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-i9001 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S85708[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TM-7026 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 MYI/1.10";
const char S85709[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-ph; Cloudfone_Excite320e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85710[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; Micromax A57 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S85711[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A57 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S85712[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cherry w500 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S85713[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCL21 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S85714[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3113, 39, 1.8.2)";
const char S85715[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3113, 42, 1.9.1)";
const char S85716[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S85717[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S85718[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S85719[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);";
const char S85720[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MITO A322 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85721[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; M2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85722[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; M2 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85723[] = "Mozilla/5.0 (Linux; Android 4.1.2; MI-ONE Plus Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S85724[] = "Mozilla/5.0 (Linux; Android 4.1.2; AT100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S85725[] = "OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85726[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; ALCATEL ONE TOUCH 985N Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S85727[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE90) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S85728[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6210 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S85729[] = "Mozilla/5.0 (Linux; Android 4.1.2; G2S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85730[] = "Mozilla/5.0 (Linux; Android 4.0.4; A210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S85731[] = "SAMSUNG-SCH-I545/101.40202";
const char S85732[] = "Mozilla/5.0 (Linux; Android 4.2.1; Micromax A110Q Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85733[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI 1S Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S85734[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI 1SC Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S85735[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.9.0) Presto/2.12.388 Version/12.15";
const char S85736[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Karbonn A25 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33756";
const char S85737[] = "Mozilla/5.0 (Linux; Android 4.1.2; STARXTREM Build/GM_ZTEV1.0.0B03) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85738[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Micromax A116) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S85739[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ALCATEL ONE TOUCH 4005D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85740[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; H9500S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85741[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; H9500S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85742[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; HTC Desire SV Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S85743[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC Desire S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S85744[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E455f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85745[] = "Mozilla/5.0 (Linux; Android 4.0.4; SK17i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85746[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-B5330 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85747[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5282 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S85748[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; LG-E400) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S85749[] = "Mozilla/5.0 (Linux; Android 4.0.4; ISW11SC Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85750[] = "Mozilla/5.0 (Linux; Android 4.0.3; Smartfren Tab 7 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S85751[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT270 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S85752[] = "Mozilla/5.0 (Linux; Android 4.3; XT912 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85753[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-N7000 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85754[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-N7000 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85755[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5530) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S85756[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6102 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S85757[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S5303B Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85758[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S5303 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85759[] = "Nokia3200/1.0 (5.34) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S85760[] = "SAMSUNG-SGH-A137/A137UCHG3 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S85761[] = "V6202/WAP2.0";
const char S85762[] = "SAMSUNG-SGH-E200/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S85763[] = "SAMSUNG-GT-C3322/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S85764[] = "MOT-EX122 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S85765[] = "SEC-SGHF300/1.0 NetFront/3.2";
const char S85766[] = "SEC-SGHD836/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85767[] = "SEC-SGHX836/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85768[] = "SEC-SGHE496/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85769[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) VZW:SCH-i760 PPC 240x320";
const char S85770[] = "SAMSUNG-GT-C3300i/C3300iXXKD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S85771[] = "SAMSUNG-GT-C3300K/C3300KXXJK2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S85772[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/15.0.874.106; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)";
const char S85773[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/15.0.874.121; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)";
const char S85774[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/15.0.874.121; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.04506)";
const char S85775[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/15.0.874.121; GTB7.2; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729)";
const char S85776[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/15.0.874.121)";
const char S85777[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/15.0.874.121)";
const char S85778[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/15.0.874.121) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S85779[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/15.0.874.121) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2";
const char S85780[] = "Mozilla/5.0 (compatible; Konqueror/3.4; Linux) KHTML/3.4.3 (like Gecko) (Kubuntu package 4:3.4.3-0ubuntu1)";
const char S85781[] = "KDDI-SN3L UP.Browser/6.2_7.2.7.1.K.4.306 (GUI) MMP/2.0";
const char S85782[] = "MicromaxX330";
const char S85783[] = "Safari/6533.21.1 CFNetwork/454.11.12 Darwin/10.7.0 (i386) (MacBookPro5%2C5)";
const char S85784[] = "Safari/6533.20.27 CFNetwork/454.11.12 Darwin/10.7.0 (i386) (MacBook2%2C1)";
const char S85785[] = "Safari/6533.21.1 CFNetwork/454.11.12 Darwin/10.7.0 (i386) (iMac10%2C1)";
const char S85786[] = "Safari/6533.21.1 CFNetwork/454.11.12 Darwin/10.7.0 (i386) (MacBookAir3%2C1)";
const char S85787[] = "Safari/6533.21.1 CFNetwork/454.11.12 Darwin/10.7.0 (i386) (MacPro4%2C1)";
const char S85788[] = "Safari/6533.21.1 CFNetwork/454.11.12 Darwin/10.7.0 (i386) (Macmini4%2C1)";
const char S85789[] = "Mozilla/5.0 (Windows NT 6.1; rv:2.0.1) Gecko/X Firefox/4.0.6 PaleMoon/4.0.6";
const char S85790[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.0) Gecko/X Firefox/4.0.5-x64 PaleMoon/4.0.5-x64";
const char S85791[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:6.0.2) Gecko/X Firefox/6.0.2 SeaMonkey/2.3.3";
const char S85792[] = "Mozilla/5.0 (Windows NT 5.1; rv:6.0.2) Gecko/X Firefox/6.0.2 SeaMonkey/2.3.3 Lightning/1.0b5";
const char S85793[] = "Mozilla/5.0 (Windows NT 5.1; rv:6.0.2) Gecko/X SeaMonkey/2.3.3";
const char S85794[] = "MacOutlook/14.2.2.X (Intel Mac OS X 10.6.8)";
const char S85795[] = "MacOutlook/14.2.3.X (Intel Mac OS X 10.6.8)";
const char S85796[] = "MacOutlook/14.2.5.X (Intel Mac OS X 10.6.8)";
const char S85797[] = "MacOutlook/14.2.4.X (Intel Mac OS X 10.6.8)";
const char S85798[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; i-mobile i693 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85799[] = "Mozilla/5.0 (S60V5; U; en-in; NokiaC5-05)/UC Browser8.4.0.159/50/352/UCWEB Mobile";
const char S85800[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:14.0) Gecko/X Thunderbird/14.0";
const char S85801[] = "SAMSUNG-GT-B6520/1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.2.2.51";
const char S85802[] = "KarbonnK15/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S85803[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; ALCATEL ONE TOUCH 918N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S85804[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; NEC-102 Build/AX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85805[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5; U; Edition Next; en) Presto/2.11.310 Version/12.50";
const char S85806[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; JY-G1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85807[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; JY-G1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85808[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; P300(Funbook) Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S85809[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GOCLEVER TAB A103 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85810[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; GOCLEVER TAB A103 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85811[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Rezound Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85812[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Rezound Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85813[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Rezound Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85814[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I8150 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85815[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I8150 Build/IMM76L)";
const char S85816[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I8150 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85817[] = "Mozilla/5.0 (Linux; U; Android 2.3.3) Gecko/X Firefox/8.0";
const char S85818[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-B5512 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355";
const char S85819[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9.2a1pre) Gecko/X Minefield/3.2a1pre";
const char S85820[] = "UCWEB/2.0(Java; U; MIDP-2.0; fa; sonyericssonu5i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S85821[] = "SonyEricssonU5UCBrowser/8.7.1.234/50/444/UCWEB";
const char S85822[] = "UCWEB/2.0(Java; U; MIDP-2.0; fa; sonyericssonu5i) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S85823[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonu5i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S85824[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Nexus 10 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0000, gb";
const char S85825[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Nexus 10 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, +0100, gb";
const char S85826[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85827[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 2A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85828[] = "LG-GW305/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-gw305) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S85829[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 7 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (Nexus 7, 39, 1.8.2)";
const char S85830[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (Nexus 7, 42, 1.9.1)";
const char S85831[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I847 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85832[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Advance TV Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85833[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S85834[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; vivo E5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85835[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; ST27i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S85836[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7310 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S85837[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P7310 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S85838[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-us; GT-P1000L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, mx";
const char S85839[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.7/210,1.9.7.210,2012-10-25 19:53, ve";
const char S85840[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.16/276,1.9.16.276,2012-12-21 16:37, +0800, ph";
const char S85841[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-P1000R Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, -0500, jm";
const char S85842[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; GT-P1000L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, -0200, br";
const char S85843[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-us; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, -0500, ec";
const char S85844[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.19/285,1.9.19.285,2013-01-24 11:19, +0800, china";
const char S85845[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LG-E615) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S85846[] = "Mozilla/5.0 (Linux; U; Android 3.1; vi-vn; IM-A840S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30";
const char S85847[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; IM-A840S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85848[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M440S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S85849[] = "Mozilla/5.0 (Linux; Android 4.0.4; SO-04D Build/7.0.D.1.137) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S85850[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:21.0) Gecko/X Firefox/21.0 SeaMonkey/2.18";
const char S85851[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST15i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S85852[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ONIDA i101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85853[] = "Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6303iclassic) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S85854[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; Optimus_San_Remo Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S85855[] = "Mozilla/5.0 (Linux; Android 4.0.3; SC-02C Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85856[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT27i Build/6.1.1.B.1.54) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S85857[] = "Mozilla/5.0 (Linux; Android 4.0.4; KB901 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85858[] = "Nokia114/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia114) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S85859[] = "Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S85860[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7108 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85861[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7108 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85862[] = "Mozilla/5.0 (Java; U; en-us; maui) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile /AJSC/1.5";
const char S85863[] = "Mozilla/5.0 (Java; U; en-US; sunmicro) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S85864[] = "Mozilla/5.0 (Java; U; en-us; generic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S85865[] = "Mozilla/5.0 (Java; U; en-us; karbonn) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S85866[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax A88 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S85867[] = "Mozilla/5.0 (Linux; Android 4.0.4; Smartphone Android by SFR STARADDICT II Build/IceCreamSandwich) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85868[] = "Mozilla/5.0 (Linux; Android 4.0.4; STARADDICT II Plus Build/GXI_ZTEV1.0.0B03) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85869[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9200 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85870[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/12.04 Chromium/18.0.1025.151 Chrome/18.0.1025.151 Safari/535.19 (compatible; PhantomJS/1.5.0; +http://cs.harding.edu)";
const char S85871[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9070 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S85872[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; POLYTRON_W3430 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85873[] = "Mozilla/5.0 (Linux; Android 4.0.3; LIFETAB_P9514 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S85874[] = "Mozilla/5.0 (Linux; Android 4.1.1; Micromax P362 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85875[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.3.2.300";
const char S85876[] = "Mozilla/5.0 (Linux; Android 4.2.2; Incredible 4G LTE Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85877[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; ST25i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S85878[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A62) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S85879[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus 7 Build/JWR66V) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.47 Safari/537.22";
const char S85880[] = "Mozilla/5.0 (Linux; Android 4.3; GT-N7000 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85881[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; zh-CN; MI 2) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S85882[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; zh-CN; MI 2S) U2/1.0.0 UCBrowser/9.2.4.329 U2/1.0.0 Mobile";
const char S85883[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9082 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S85884[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9082i Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S85885[] = "Mozilla/5.0 (Linux; Android 4.2.2; SM-T315 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S85886[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E973 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85887[] = "Mozilla/5.0 (Linux; Android 4.1.2; S100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85888[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LU6200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85889[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAP4040_DUO Build/PrestigioPAP4040DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85890[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E445 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S85891[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S85892[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S85893[] = "Mozilla/5.0 (Linux; Android 4.2.1; M470BSA Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S85894[] = "SEC-SGHX820/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85895[] = "PPC; 240x320; HTC_P3450/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S85896[] = "HTC-Touch/Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S85897[] = "HTC_P3400-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)";
const char S85898[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; google_sdk Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S85899[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-US; sdk Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Vesion/3.1.2 Mobile Safari/525.20.1";
const char S85900[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; sdk Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S85901[] = "ZTE-T108/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85902[] = "NokiaC6-00/UC Browser7.7.1.88/50/444";
const char S85903[] = "nokiac6-00/UC Browser7.7.1.88/69/444  3gpp-gba";
const char S85904[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Mobile/5F137";
const char S85905[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 2_1_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Mobile/5F138";
const char S85906[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko)";
const char S85907[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 2_1_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko)";
const char S85908[] = "NokiaC7-00/UC Browser7.9.0.102/50/355/UCWEB";
const char S85909[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Kindle Fire Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85910[] = "Mozilla/5.0 (Linux; U; Android v1.02_14.13-EN-2011.03.25; nl-nl; TAB210 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S85911[] = "Mozilla/5.0 (Linux; U; Android v1.02_15.14-EN@X; nl-nl; TAB210 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S85912[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.1.6) Gecko/X FireDownload/2.0.1 Firefox/3.5.6 Wyzo/3.5.6.1 FireTorrent/2.0.1 KAT";
const char S85913[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.19pre) Gecko/X Prism zdesktop/7.1.4";
const char S85914[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; SH8118U Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85915[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.13.13719.201";
const char S85916[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; X903 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85917[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; X903 Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S85918[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; X903 Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S85919[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Z710t Build/HTC_U8500L_R2D_EC07) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85920[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Z710t Build/HTC_U8500L_R2D_EC07) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S85921[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Z710t Build/HTC_U8500L_R2D_EC05) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S85922[] = "HTCZ710t_TD/1.0 Android/2.3 release/6.1.2011 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85923[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Z710t Build/HTC_U8500L_R2D_EC05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85924[] = "Mozilla/5.0 (S60V5; U; en-in; Nokia500)/UC Browser8.4.0.159/50/352/UCWEB Mobile";
const char S85925[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/532.3 (KHTML, like Gecko) Iron/4.0.227.0 Chrome/4.0.227.0 Safari/532.3";
const char S85926[] = "Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US) AppleWebKit/532.3 (KHTML, like Gecko) Iron/4.0.227.0 Chrome/4.0.227.0 Safari/532.3";
const char S85927[] = "nokiae71-1/UC Browser8.0.3.107/69/444";
const char S85928[] = "Mozilla/5.0 (Linux; Android 4.0.4; MB526 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85929[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; AMOI_N807 Build/ALPS.GB2.MP.V1.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85930[] = "Mozilla/5.0 (Linux; Android 4.0.3; Vega Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S85931[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959V Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85932[] = "iBrowser/2.7 (Nokia5230;SymbianOS/9.1 Series60/3.0)";
const char S85933[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8500L/1.0; U; Bada/2.0; es) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile WVGA SMM-MMS/1.2.0 OPN-B";
const char S85934[] = "HTC Streaming Player sprint_wwe / 1.0 / jewel / 4.0.3";
const char S85935[] = "HTC Streaming Player htc / 1.0 / jewel / 4.1.1";
const char S85936[] = "HTC Streaming Player sprint_wwe / 1.0 / jewel / 4.0.4";
const char S85937[] = "Micromax X312/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85938[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S5570 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85939[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN85) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S85940[] = "Micromax X331/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S85941[] = "Mozilla/5.0 (Linux; Android 4.1.1; Novo10 Hero Build/X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S85942[] = "Mozilla/5.0 (Linux; Android 4.1.1; Novo 10 Hero QuadCore Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S85943[] = "Mozilla/5.0 (Windows; U; Win 9x 4.90; en-US; rv:1.8.0.1) Gecko/X SeaMonkey/1.0";
const char S85944[] = "Mozilla/5.0 (Linux; Android 4.0.4; VL-110 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S85945[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; OPTIMA Smart Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85946[] = "Mozilla/5.0 (Linux; Android 4.0.3; GoTab Epic V Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S85947[] = "JUC (Linux; U; 2.3.6; zh-cn; A52; 240*320) UCWEB7.9.0.94/139/444";
const char S85948[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 5 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85949[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S85950[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxq7) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S85951[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxq7) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S85952[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s5253) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S85953[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Version/6.0.2 Safari/8536.26.17 Sleipnir/3.9.0.2";
const char S85954[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; SmartTab2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85955[] = "SonyEricssonW910i/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonw910i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S85956[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85957[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85958[] = "Opera/9.80 (Android 4.2.1; Linux; Opera Mobi/ADR-X; U; en) Presto/2.10.254 Version/12.00";
const char S85959[] = "Opera/9.80 (Android 4.2.2; Linux; Opera Mobi/ADR-X; U; en-GB) Presto/2.10.254 Version/12.00";
const char S85960[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5263) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S85961[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-note WiFi 9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85962[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; i-mobile i-note WiFi 9 Build/IMM76D)";
const char S85963[] = "Mozilla/5.0 (Linux; Android 4.0.3; Penta IS701C Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85964[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A789 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S85965[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; PC1007 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85966[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; PC1007 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S85967[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTabS2110AF Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S85968[] = "Mozilla/5.0 (Linux; Android 4.1.2; SH631W Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S85969[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S85970[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848i Duo Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S85971[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848 Duo Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S85972[] = "Mozilla/5.0 (Linux; Android 4.1.2; bq Elcano Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S85973[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/9A405 Safari/7534.48.3";
const char S85974[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S85975[] = "UCWEB/2.0 (Linux; U; Adr 2.2; en-US; HTC_Desire) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S85976[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; HTC_Desire) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S85977[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A35 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S85978[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7000) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S85979[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Infinix X401 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S85980[] = "NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax3-02) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S85981[] = "Mozilla/5.0 (Linux; Android 4.0.3; U9500 Build/HuaweiU9500) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85982[] = "Mozilla/5.0 (Linux; Android 4.2.2; Desire S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S85983[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.32 (KHTML, like Gecko) Chromium/25.0.1349.2 Chrome/25.0.1349.2 Safari/537.32 Ubuntu/13.04 (3.7.91-1ubuntu1~raring1) Epiphany/3.7.91";
const char S85984[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.32 (KHTML, like Gecko) Chromium/25.0.1349.2 Chrome/25.0.1349.2 Safari/537.32 Ubuntu/13.04 (3.7.91-1ubuntu1~raring1) Epiphany/3.7.91";
const char S85985[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.32 (KHTML, like Gecko) Chromium/25.0.1349.2 Chrome/25.0.1349.2 Safari/537.32 Ubuntu/13.04 (3.8.2-1ubuntu1~raring1) Epiphany/3.8.2";
const char S85986[] = "Lenovo-S920/S102 Linux/3.4.0 Android/4.2 Release/12.26.2012 Browser/AppleWebKit534.30 Profile/ Configuration/ Mobile Safari/534.30";
const char S85987[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S85988[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8812E Build/HuaweiC8812E) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S85989[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LT26ii Build/6.1.A.2.45) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S85990[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT26ii Build/6.1.A.2.55) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S85991[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI C8813 Build/HuaweiC8813) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S85992[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI C8813Q Build/HuaweiC8813Q) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S85993[] = "Mozilla/5.0 (GobyBot/1.0; Windows; U; Windows NT 5.1; en-US; +http://www.goby.com/; rv:1.8.0.1) Gecko/X Firefox/1.5.0.1";
const char S85994[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7100) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S85995[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Panasonic P51 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2 Mobile Safari/534.30";
const char S85996[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; L-05E Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2 Mobile Safari/534.30";
const char S85997[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; L-05E Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S85998[] = "Mozilla/5.0 (Linux; Android 4.0.4; MI-ONE Plus Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S85999[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P705 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S86000[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P705g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S86001[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN78) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S86002[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A65) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S86003[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3850) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S86004[] = "Mozilla/5.0 (Linux; Android 4.0.4; STARADDICT II Plus Build/GXI_ZTEV1.0.0B03) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86005[] = "Mozilla/5.0 (Linux; Android 4.0.4; Smartphone Android by SFR STARADDICT II Build/IceCreamSandwich) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86006[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_985N) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S86007[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_985N) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S86008[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E987 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86009[] = "Mozilla/5.0 (Linux; Android 4.1.2; Amazon Kindle Fire Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S86010[] = "Mozilla/5.0 (Linux; Android 4.2.2; Optimus G Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86011[] = "Mozilla/5.0 (Linux; Android 4.0.4; A89 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S86012[] = "Mozilla/5.0 (Linux; Android 4.1.2; BLU STUDIO 5.0 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86013[] = "Mozilla/5.0 (Linux; Android 4.1.2; Studio 5.0s Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86014[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Titanium S5 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S86015[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-ca; 801a Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86016[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-M840 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S86017[] = "SAMSUNG-SGH-L770/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S86018[] = "SAMSUNG-SGH-L770-Vodafone/L770BUHE1 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S86019[] = "SAMSUNG-SGH-L770-ORANGE/BVIG2 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S86020[] = "LG-GC900F/V10d Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S86021[] = "SAMSUNG-SGH-A256/A256UXIF8 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S86022[] = "Mozilla/5.0 (Linux; U; Android 2.1; zh-tw; YL-Coolpad_D530TW/1.01.012.100927.D530TW; 320*480; CTC/2.0) CoolpadWebkit/530.18";
const char S86023[] = "Mozilla/5.0 (Linux; U; Android 2.1; zh-tw; YL-Coolpad_D530TW/1.01.012.X.D530TW; 320*480; CTC/2.0) CoolpadWebkit/530.18";
const char S86024[] = "Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; D530 Build/ERD79) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S86025[] = "Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; D530 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S86026[] = "KDDI-SH3D UP.Browser/6.2_7.2.7.1.K.4.306 (GUI) MMP/2.0";
const char S86027[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 2_0 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5A345 Safari/525.20";
const char S86028[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 2_0 like Mac OS X; de-de) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5A347 Safari/525.20";
const char S86029[] = "NEC-N600/1.0 HopenOS/2.0";
const char S86030[] = "HTC-P4600/5.11.502.2 Opera/9.50 (Windows NT 5.1; U; en)";
const char S86031[] = "HTC-P4600/1.2 Opera/9.50 (Windows NT 5.1; U; en)";
const char S86032[] = "DoCoMo/2.0 P07A3(c500;TB;W24H15)";
const char S86033[] = "Mozilla/5.0 (Maemo; Linux armv7l; rv:11.0) Gecko/X Firefox/11.0 Fennec/11.0";
const char S86034[] = "Opera/9.80 (Android 3.0.1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00";
const char S86035[] = "Opera/9.80 (Android 3.0UI; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00";
const char S86036[] = "Opera/9.80 (Android 3.0; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00";
const char S86037[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:12.0) Gecko/X Thunderbird/12.0.1";
const char S86038[] = "UC Browser7.9.0.102/50/355/UCWEB";
const char S86039[] = "UC Browser7.9.1.120/27/352/UCWEB";
const char S86040[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; LG-LU3700 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86041[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaN8-00)/UC Browser8.4.0.159/50/352/UCWEB Mobile";
const char S86042[] = "SAMSUNG-GT-I9018_TD/1.0 Android/2.2.2 Release/12.15.2010 Browser/AppleWebKit533.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S86043[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; SCH-P739 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S86044[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; SCH-P739 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S86045[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_C8860E/C8860EV100R001C92B892; 480*854; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S86046[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8860E Build/HuaweiC8860E) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S86047[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_C8860E/C8860EV100R001C92B871; 480*854; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S86048[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_C8860E/C8860EV100R001C92B897; 480*854; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S86049[] = "Mozilla/5.0 (Linux; Android 4.0.3; URBANO PROGRESSO Build/011.0.3100) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86050[] = "Mozilla/5.0 (Java; U; en-us; nokian73-1) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S86051[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/3.1; Xbox)";
const char S86052[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; A1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86053[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86054[] = "Mozilla/5.0 (Series40; Nokia2730c-1/10.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S86055[] = "Mozilla/5.0 (Linux; Android 4.0.4; P5 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86056[] = "Mozilla/5.0 (Windows; U; WinNT3.51; en-US; rv:1.8.1.8) Gecko/X SeaMonkey/1.1.5";
const char S86057[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/5.0 (Windows NT 6.1; WOW64) AppleWebKit/23.411; U; en) Presto/2.5.25 Version/10.54";
const char S86058[] = "Mozilla/5.0 (Series40; Nokia5330-1d/06.88; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S86059[] = "Opera/9.80 (Series 60; Opera Mini/6.6.29184/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S86060[] = "Mozilla/5.0 (Java; U; en-us; lg-x330t) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile";
const char S86061[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia6233) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S86062[] = "Nokia6233/2.0 (05.60) Profile/MIDP-2.0 Configuration/CLDC-1.1 , UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia6233) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S86063[] = "Nokia114/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia114) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S86064[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V887 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86065[] = "Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S86066[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenco TAB-1014 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S86067[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; DJC Touchtab3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86068[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A800 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S86069[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T879 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86070[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7000 Build/JRO03L; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86071[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730G Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S86072[] = "Mozilla/5.0 (Linux; Android 4.1.1; Lenovo S890 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86073[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cynus T2 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86074[] = "Mozilla/5.0 (Linux; Android 4.2.2; ADR6425LVW Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86075[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A3) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S86076[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F260S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86077[] = "SonyEricssonW995/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw995) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S86078[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030D Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86079[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9100) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S86080[] = "Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300S Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86081[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; Micromax A116 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S86082[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-I8150 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453";
const char S86083[] = "Mozilla/5.0 (Linux; Android 2.3.4; LT18i Build/4.0.2.A.0.62) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453";
const char S86084[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; fr-fr; GT-N8000 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86085[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1811_v77_gq2000_ov5647 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86086[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT907 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86087[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S86088[] = "Mozilla/5.0 (Linux; Android 2.3.7; LG-VM696 Build/ZV5.GWK74) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S86089[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86090[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86091[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL_ONE_TOUCH_6010X_Orange Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86092[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-au; PNDPP41DG7BLK^ Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86093[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Spice MI-353 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86094[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One XL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S86095[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT881 Build/6.7.3-130_YUS-11) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86096[] = "Mozilla/5.0 (Linux; Android 4.2.1; Celkon A119Q Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86097[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 UCBrowser/9.1.0.287 Mobile";
const char S86098[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 UCBrowser/9.1.0.287 Mobile";
const char S86099[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 UCBrowser/9.1.0.287 Mobile";
const char S86100[] = "Mozilla/5.0 (Linux; Android 4.1.2; SH530U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86101[] = "Mozilla/5.0 (Linux; Android 4.2.2; F-06E Build/V21R48D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86102[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST25i Build/6.1.1.B.1.10) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S86103[] = "Mozilla/5.0 (Linux; Android 4.2.2; A1-811 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S86104[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; E3 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S86105[] = "Nokia6126/2.0 (04.28) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86106[] = "SAMSUNG-GT-M2510/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86107[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Sprint MP6900SP";
const char S86108[] = "LG-GS500/V100 OBIGO/Q7";
const char S86109[] = "DoCoMo/2.0 F01A(c100;TB;W24H17)";
const char S86110[] = "TXT8010/T05_0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S86111[] = "NOKIAN79";
const char S86112[] = "ZTE-T90/1.0 ACS-NF/3.4 QTV2.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86113[] = "DoCoMo/2.0 N07B(c500;TB;W24H16)";
const char S86114[] = "KDDI-TS3U UP.Browser/6.2_7.2.7.1.K.5.207 (GUI) MMP/2.0";
const char S86115[] = "DoCoMo/2.0 N905i(c100;TB;W24H16)";
const char S86116[] = "DoCoMo/2.0 N905imyu(c100;TB;W24H16)";
const char S86117[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.0.3.2197 Safari/532.4";
const char S86118[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:10.0.1) Gecko/X Thunderbird/10.0.1";
const char S86119[] = "Mozilla/5.0 (Windows NT 6.1; rv:11.0) Gecko/X Firefox/11.0 SeaMonkey/2.8 Lightning/1.3";
const char S86120[] = "JUC (Linux; U; 2.3.5; zh-cn; Lenovo_A65; 320*480) UCWEB7.9.3.103/139/32458";
const char S86121[] = "Opera/9.02 (Windows; U; nl)";
const char S86122[] = "Opera/9.00 (Windows; U)";
const char S86123[] = "JUC(Linux; U;2.3.3;zh_cn;Lenovo A60;320*480;)UCWEB7.6.0.75/139/32399";
const char S86124[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus One Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86125[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus One Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86126[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s3653w) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S86127[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s3653w) UCBrowser8.3.1.161/70/465/UCWEB Mobile";
const char S86128[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; ALCATEL_one_touch_908 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86129[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; el-gr; ALCATEL ONE TOUCH 908 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86130[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Comodo_Dragon/21.0.2.0";
const char S86131[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Comodo_Dragon/21.0.2.0";
const char S86132[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xperia S Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86133[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MID114C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86134[] = "Mozilla/5.0 (Linux; U; YunOs 1.0.0.3; zh-cn; K-Touch W619 Build/AliyunOs-2012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86135[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4 slurp";
const char S86136[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 slurp";
const char S86137[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 slurp";
const char S86138[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A52 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S86139[] = "Mozilla/5.0 (Linux; Android 4.0.3; Enjoy 7 Plus Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S86140[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; eZee'Tab705 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86141[] = "NOKIA-Lumia 510/UCWEB2.7.0.223/47/355";
const char S86142[] = "Mozilla/5.0 (Windows; U; ; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.3.1.31155 (Firestorm-Release; firestorm skin) Safari/533.3";
const char S86143[] = "SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonj105i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S86144[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; M9108 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86145[] = "Rockmelt/2.2.0.241 CFNetwork/609.1.4 Darwin/13.0.0";
const char S86146[] = "Lemon_T109; Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86147[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  SonyEricssonP1i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86148[] = "Instagram 3.4.2 (iPod4,1; iPhone OS 5.0.1; en_US; en) AppleWebKit/420+";
const char S86149[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; OnePAD 1100x2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86150[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I8150) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S86151[] = "Mozilla/5.0 (Windows; U; Windows CE 6.0; en-US; rv:1.9.2a1pre) Gecko/X Minefield/3.2a1pre";
const char S86152[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Skate Build/JRO03C; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86153[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; Skate Build/JRO03C; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86154[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; ZTE-BLADE) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S86155[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT22i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S86156[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI-ONE Plus Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S86157[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TOUCH102B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86158[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7310 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0000, gb";
const char S86159[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; GT-B7510) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S86160[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LU6200 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86161[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A800 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86162[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae65-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S86163[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE65) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S86164[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; A54 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S86165[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; A54 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S86166[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZP950 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86167[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZP950+ Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86168[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3850) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S86169[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Starmobile FLIRT Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86170[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530M Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86171[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-R530M Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86172[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9500 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S86173[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia700) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S86174[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S560 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86175[] = "Mozilla/5.0 (Linux; Android 4.0.4; CT1010 Build/IMM76I.v007) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S86176[] = "Mozilla/5.0 (Linux; Android 4.1.1; C1605 Build/11.3.A.2.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86177[] = "Mozilla/5.0 (Windows NT 5.1; rv:22.0) Gecko/X Thunderbird/22.0";
const char S86178[] = "Mozilla/5.0 (Linux; Android 4.1.2; B92 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86179[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A789 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86180[] = "Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S86181[] = "Mozilla/5.0 (Linux; Android 4.2.2; BN Nook HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S86182[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; FPT F11 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86183[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; @TAB-SEVEN-X1-B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86184[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar Touch Kem 351 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86185[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-; Mobiistar Touch Kem 351 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86186[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3653) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S86187[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; LG-E612 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S86188[] = "Mozilla/5.0 (Linux; Android 4.1.1; Orange Kivo Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86189[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT910 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/umts_spyder";
const char S86190[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Connect-3G-2.0 Build/HCL) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86191[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SH-10D Build/S7080) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86192[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; GT-I9502 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S86193[] = "Mozilla/5.0 (Linux; Android 4.2.1; Lenovo P780_ROW Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86194[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian8-00) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba";
const char S86195[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Sensation Z710e Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86196[] = "Mozilla/5.0 (Linux; Android 4.2.2; Infected Rezound Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86197[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-05D Build/9.1.C.0.475) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S86198[] = "Mozilla/5.0 (Linux; Android 4.1.2; MI-ONE Plus Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86199[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-E975 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86200[] = "Mozilla/5.0 (Linux; Android 4.0.4; U8836D Build/HuaweiU8836D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S86201[] = "Mozilla/5.0 (Linux; Android 4.3; Optimus G Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86202[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5670) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S86203[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE52-1) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S86204[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 7_0 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5";
const char S86205[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 7_0_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5";
const char S86206[] = "MOT-Z3/08.02.08R MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/1.0";
const char S86207[] = "MOT-MOTOROKREM30_CMCC/1.0 LinuxOS/2.6.10 Release/07.18.2008 Browser/OSS1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R6716_G_71.04.06R_B";
const char S86208[] = "MOT-MOTOROKREM30/1.0 LinuxOS/2.6.10 Release/07.18.2008 Browser/OSS1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R6716_G_71.01.23R_A";
const char S86209[] = "LGKU380/v1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86210[] = "LG/KU380/v10c Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86211[] = "LG-KU380/v1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86212[] = "Mozilla/4.0 (Vodafone/1.0/LG-GU280/v10b Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S86213[] = "LG-GU280f Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S86214[] = "LG-GU280/v09a Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S86215[] = "Mozilla/4.0 (Vodafone/1.0/LG-GU280/v10a Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S86216[] = "LG-GU280-Orange/v10e Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S86217[] = "SAMSUNG-GT-S5150/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86218[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SCH-i579 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86219[] = "Mozilla/5.0(Linux;U;Android2.2.2;Zh_cn;LG-P500;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355";
const char S86220[] = "Mozilla/5.0(Linux;U;Android2.2;Zh_cn;LG-P500;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355";
const char S86221[] = "JUC(Linux;U;Android2.2.2;Zh_cn;LG-P500;320*480;)UCWEB7.8.0.95/139/444";
const char S86222[] = "JUC(Linux;U;Android2.2;Zh_cn;LG-P500;320*480;)UCWEB7.8.0.95/139/444";
const char S86223[] = "JUC(Linux;U;Android2.2.1;Zh_cn;LG-P500;320*480;)UCWEB7.8.0.95/139/444";
const char S86224[] = "Mozilla/5.0 (Series40; NokiaX2-01/08.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.2.26.6";
const char S86225[] = "LG-GM205 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S86226[] = "NokiaN73/UC Browser7.7.1.88/28/444";
const char S86227[] = "NokiaN8-00/UC Browser7.7.1.88/50/444";
const char S86228[] = "nokian8-00/UC Browser7.7.1.88/69/444  3gpp-gba";
const char S86229[] = "NokiaE66/UCWEB7.9.0.94/28/999";
const char S86230[] = "NokiaE66/UC Browser7.9.0.102/28/355/UCWEB";
const char S86231[] = "Opera/9.80 (X11; Linux i686; U; Linux Mint; en) Presto/2.9.168 Version/11.52";
const char S86232[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Comodo_Dragon/9.0.0.3 Chrome/9.0.597.98 Safari/534.13";
const char S86233[] = "Nokia7230/5.0 (06.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7230/UC Browser8.0.3.107/69/444";
const char S86234[] = "Windows Phone Search (Windows Phone OS 7.10;LG;LG-C900;7.10;7720)";
const char S86235[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.53.11 (KHTML, like Gecko) Fluid/0.9.6 Safari/534.53.11";
const char S86236[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) Fluid/0.9.6 Safari/534.56.5";
const char S86237[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Fluid/0.9.6 Safari/534.57.7";
const char S86238[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:12.0) Gecko/X Firefox/12.0 SeaMonkey/2.9";
const char S86239[] = "Mozilla/5.0 (Series40; Nokia6303iclassic/07.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S86240[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; MT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S86241[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s5333) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S86242[] = "JUC(Linux;U;Android2.3.6;Zh_cn;Micromax A50;240*320;)UCWEB7.8.0.95/139/444";
const char S86243[] = "LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-gd510) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S86244[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; E15i Build/IMM76I; CyanogenMod-MiniCM9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86245[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; E15i Build/IMM76L; CyanogenMod-MiniCM9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86246[] = "MQQBrowser/Mini2.8 (NokiaX2-01/08.63)";
const char S86247[] = "GoogleEarth/6.1.0.5001(Macintosh;Mac OS X (10.8.2);en;kml:2.2;client:Free;type:default)";
const char S86248[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/9.80 (J2ME/22.478; U; en) Presto/2.5.25 Version/10.54";
const char S86249[] = "Mozilla/5.0 (Series40; Nokia6303classic/10.12; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S86250[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9210T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86251[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-I9210T Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86252[] = "Mozilla/5.0 (Linux; Android 4.0.3; TM901 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S86253[] = "Mozilla/5.0 (Linux; Android 4.0.4; TM901 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S86254[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; Artes D821 Build/ICS.g12refM805AST.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86255[] = "LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-gs500) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S86256[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9300 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S86257[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9300 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S86258[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S86259[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-c6712) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S86260[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86261[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86262[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.3.1.31155 (Firestorm-Release; firestorm skin) Safari/533.3";
const char S86263[] = "SonyEricssonU10i/R7BA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonu10i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S86264[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LU6200 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86265[] = "Mozilla/5.0 (Linux; Android 4.1.1; LG-LU6200 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86266[] = "Mozilla/5.0 (Linux; Android 4.1.1; TOUCH87 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S86267[] = "Mozilla/5.0 (Linux; Android 4.2.1; LG-P990 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86268[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-P990 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86269[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MID801 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86270[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S86271[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S86272[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MID Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S86273[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; INFINITY_POWER Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S86274[] = "Mozilla/5.0 (Linux; Android 4.1.2; 201F Build/V33R12A) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86275[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7005 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S86276[] = "Mozilla/5.0 (compatible; MSIE 6.0; WinNT3.51; Trident/1.0)";
const char S86277[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST26i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S86278[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; SK17i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S86279[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-B5512) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S86280[] = "Mozilla/5.0 (Linux; Android 4.0.4; BC1003 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S86281[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xperia Sola Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86282[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; PTL21 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86283[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI-ONE Plus Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S86284[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LG-P698 Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S86285[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-N8000 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S86286[] = "OneBrowser/3.1 (Nokia5800d-1/60.0.003/sw_platform=S60;sw_platform_version=5.0;java_build_version=1.4.43)";
const char S86287[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9260 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86288[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ASUS Transformer Pad TF300T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);";
const char S86289[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ASUS Transformer Pad TF300T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S86290[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; Micromax_A51) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S86291[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; jbed-fastbcc) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S86292[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; spice) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S86293[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sunmicro) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S86294[] = "Mozilla/5.0 (Linux; Android 4.1.1; CMP749 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S86295[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-01E Build/9.1.C.0.475) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86296[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Micromax A110Q Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S86297[] = "Mozilla/5.0 (Linux; Android 4.2.1; ALCATEL ONE TOUCH 8008D Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86298[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Tab700M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86299[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S86300[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID4 Build/9.8.2O-72_VZW-18-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S86301[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-ph; MyPhone A919i Duo Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86302[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; PMP7100D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86303[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5021E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86304[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; LG-E612 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S86305[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; HTC One Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S86306[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-LS970 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S86307[] = "Mozilla/5.0 (Linux; Android 4.0.4; NXM805ND_EN Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S86308[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC One V Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S86309[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515m Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S86310[] = "BlackBerry7130/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/179";
const char S86311[] = "LG-KF600 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S86312[] = "LG-KF600d Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S86313[] = "Spice M5700";
const char S86314[] = "PantechP2000/JGUS03292010 Browser/Obigo/Q05A OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S86315[] = "Nokia7610/2.0 (7.0642.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.7.1.88/27/444";
const char S86316[] = "NokiaE71/UC Browser7.8.0.95/28/355";
const char S86317[] = "HTC-ST6356/2.31.502.4 (66023) Opera/9.50 (Windows NT 5.1; U; en)";
const char S86318[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Spice Mi-270 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86319[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; Spice Mi-270 Build/MASTER)";
const char S86320[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Spice Mi-270 Build/MASTER) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S86321[] = "GoogleEarth/6.0.3.2197(Macintosh;Mac OS X (10.6.8);en;kml:2.2;client:Free;type:default)";
const char S86322[] = "GoogleEarth/6.0.1.2032(Macintosh;Mac OS X (10.6.8);en;kml:2.2;client:Free;type:default)";
const char S86323[] = "DoCoMo/2.0 P01B(c500;TB;W24H16)";
const char S86324[] = "Nokia6820/2.0 (4.83) Profile/MIDP-1.0 Configuration/CLDC-1.0 (compatible; Mediapartners-Google/2.1; +http://www.google.com/bot.html)";
const char S86325[] = "Mozilla/5.0 (compatible; Konqueror/4.1; DragonFly) KHTML/4.1.4 (like Gecko)";
const char S86326[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.2.149.27 Safari/525.13";
const char S86327[] = "HTC_TyTN_II Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S86328[] = "HTC_P4550 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S86329[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; 8810 Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S86330[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; 8810 Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S86331[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; 8810 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86332[] = "Mozilla 5.0 (Linux; U; Android 2.2.2; zh-cn; 8810 Build FRG83G) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S86333[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:13.0) Gecko/X Firefox/13.0.1 SeaMonkey/2.10.1";
const char S86334[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; TM-5200 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86335[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 Comodo_Dragon/21.1.1.0";
const char S86336[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 Comodo_Dragon/21.1.1.0";
const char S86337[] = "JUC(Linux;U;Android3.2;Zh_cn;GT-P6200;600*976;)UCWEB7.8.0.95/139/352";
const char S86338[] = "Mozilla/5.0(Linux;U;Android3.2;Zh_cn;GT-P6200;600*976;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S86339[] = "MQQBrowser/2.7 Mozilla/5.0 (Linux; U; Android A5000; in-id; A5000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86340[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari Version/5.1";
const char S86341[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; zh-cn Model:iPad1,1) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7543.48.3";
const char S86342[] = "LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-in; lg-gs290) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S86343[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; AIRIS OnePAD 725 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86344[] = "Mozilla/5.0 (Linux; Android 4.0.4; MOMO11Speed Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S86345[] = "OneBrowser/3.0 (Nokia5130c-2/07.95)";
const char S86346[] = "K-TouchT580_TD/1.0 Android/2.3 Release/02.01.2012 Browser/WAP2.0 appleWebkit/534.30";
const char S86347[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; cherry superion+ Build/gingerbread_house_r84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86348[] = "Opera/9.80 (BREW; Opera Mini/5.0/27.2239; U; en) Presto/2.8.119 320X240 LG VN250";
const char S86349[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; LG-E612 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S86350[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; Explay A350 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86351[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ607 Build/7.7.1-122-4-FLEM-WIFI-LA) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S86352[] = "OneBrowser/3.1 (SAMSUNG-GT-S5222)";
const char S86353[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonw580i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S86354[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonw580i) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S86355[] = "Mozilla/5.0 (Linux; Android 4.0.4; J-Q88B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86356[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE50) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile";
const char S86357[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; MT27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S86358[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Kindle Fire Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (Kindle Fire, 35, 1.7.0)";
const char S86359[] = "JUC (Linux; U; 2.3.6; zh-cn; Intex_Aqua_3.2; 240*320) UCWEB7.9.0.94/139/444";
const char S86360[] = "Mozilla/5.0 (Linux; Android 4.1.2; JY-G3 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86361[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Lemon P2  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86362[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Elcano Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S86363[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; WT19i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S86364[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT26i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S86365[] = "Opera/9.80 (Windows NT 4.0) Presto/2.12.388 Version/12.14";
const char S86366[] = "SonyEricssonW910i/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw910i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S86367[] = "Mozilla/5.0 (Linux; Android 4.1.1; WM8850-mid Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86368[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-N7000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S86369[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; GT-N7000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S86370[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993D Build/ICECREAM) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86371[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993 Build/ICECREAM) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86372[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A860K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86373[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI 1S Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31";
const char S86374[] = "Nokia3110c/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3110c) U2/1.0.0 UCBrowser/9.0.0.295 U2/1.0.0 Mobile";
const char S86375[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; HTC Desire S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S86376[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; HTC Desire SV Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S86377[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; N400 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S86378[] = "Mozilla/5.0 (Windows NT 6.0; rv:21.0) Gecko/X Firefox/21.0 slurp";
const char S86379[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; MI-ONE C1 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S86380[] = "Mozilla/5.0 (Linux; Android 4.1.1; Smartfren Tab 7 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S86381[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAP4040_DUO Build/PrestigioPAP4040DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86382[] = "Mozilla/5.0 (Linux; Android 4.0.4; SK17i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86383[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ThL W7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86384[] = "Mozilla/5.0 (Linux; Android 4.0.4; BC1003 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S86385[] = "Mozilla/5.0 (Linux; Android 2.3.6; SAMSUNG-SGH-I727 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S86386[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7 (compatible; PhantomJS/1.5.0; +http://cs.harding.edu)";
const char S86387[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; OPSSON IVO6600 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86388[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Videocon A22 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86389[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5105 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S86390[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Primux_Delta Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86391[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC6435LVW Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S86392[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Woxter Nimbus 97Q Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86393[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A25) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S86394[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; Nexus 7 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S86395[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; NEBULA6.9 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86396[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; Connect-3G Build/HCL) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S86397[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; Y3 Build/HCL) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S86398[] = "Mozilla/5.0 (Linux; Android 3.2.1; HTC Flyer P510e Build/HTK75C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S86399[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S86400[] = "UCWEB/2.0 (Linux; U; Adr 4.2.2; en-US; GT-I9500) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S86401[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTX21 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86402[] = "Mozilla/5.0 (Linux; Android 4.2.1; i-mobile IQ X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86403[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; Micromax_A51) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S86404[] = "Mozilla/5.0 (Windows NT 6.0; rv:23.0) Gecko/X Firefox/23.0 slurp";
const char S86405[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S86406[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; SPH-D710BST Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S86407[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 10 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.2/manta";
const char S86408[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 10 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.2/manta";
const char S86409[] = "Mozilla/5.0 (Linux; Android 4.2.1; i-mobile IQ XA Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86410[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus 7 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1650.11 Safari/537.36";
const char S86411[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire U dual sim Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86412[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC One X Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S86413[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S86414[] = "Mozilla/4.0 SonyEricssonK850iv/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S86415[] = "SAMSUNG-SGH-A561/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S86416[] = "HTC Streaming Player tmobile / 1.0 / htc_glacier / 2.2.1";
const char S86417[] = "HTC Streaming Player tmobile / 1.0 / htc_glacier / 2.3.4";
const char S86418[] = "NokiaN73/UC Browser7.6.1.82/28/351";
const char S86419[] = "NokiaN70-1/5.0638.3.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.6.1.82/27/354";
const char S86420[] = "NokiaN70/ 5.0741.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.6.1.82/27/351";
const char S86421[] = "Opera/9.80 (Android 1.6; Linux; Opera Mobi/ADR-X; U; en-GB) Presto/2.8.149 Version/11.10";
const char S86422[] = "Opera/9.80 (Linux armv7l; Maemo; Opera Mobi/9; U; en-GB) Presto/2.7.81 Version/11.00";
const char S86423[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Iron/8.0.555.0 Chrome/8.0.555.0 Safari/534.10";
const char S86424[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Iron/8.0.555.0 Chrome/8.0.555.0 Safari/534.10";
const char S86425[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID X2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86426[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB870 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86427[] = "Opera/9.80 (iPhone; Opera Mini/7.1.32694/27.1407; U; en) Presto/2.8.119 Version/11.10";
const char S86428[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.8pre) Gecko/X Ubuntu/10.04 (lucid) Namoroka/3.6.8pre";
const char S86429[] = "NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax3-02/UC Browser7.9.0.102/69/352";
const char S86430[] = "samsung-gt-b5310/UC Browser8.0.3.107/69/444";
const char S86431[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo P70 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S86432[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo P70 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86433[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo P70 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86434[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo P70 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S86435[] = "gt-s5233w/UC Browser8.0.3.107/69/352";
const char S86436[] = "gt-s5233w/UC Browser8.0.3.107/69/444";
const char S86437[] = "SAMSUNG-SPH-D710/100.40004";
const char S86438[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S86439[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S86440[] = "SonyEricssonM1i/R1BA Browser/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S86441[] = "Mozilla/5.0 (X11; SunOS i86pc; rv:6.0) Gecko/X Thunderbird/6.0 OracleBeehiveExtension/1.0.0.4-OracleInternal Lightning/1.0b5";
const char S86442[] = "JUC(Linux; U;2.3.6;zh_cn;GT-I9003;480*800;)UCWEB7.6.0.75/139/7694";
const char S86443[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT912 Build/IMM76D; CyanogenMod-Full Throttle Retribution V2 10/16/12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86444[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Droid Razr Build/IMM76D; CyanogenMod-Full Throttle CM9 Stronger Then All) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86445[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT912 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86446[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; in-id; Axioo-VIGO350 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86447[] = "Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia7210supernova) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S86448[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; Telenor_One_Touch_S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86449[] = "Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/444/UCWEB Mobile";
const char S86450[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST23i Build/11.0.A.2.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S86451[] = "Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-55EX640; CC/USA; en) Presto/2.8.115 Version/11.10";
const char S86452[] = "Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-55EX640; CC/USA; en) Presto/2.8.115 Version/11.10,gzip(gfe)";
const char S86453[] = "Mozilla/5.0 (Linux; Android 4.0.3; iball Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86454[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F100S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86455[] = "OneBrowser/3.0 (Nokia2700c-2/09.97)";
const char S86456[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86457[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86458[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86459[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3100B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86460[] = "Mozilla/5.0 (Linux; Android 4.0.4; MIDC409 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86461[] = "Mozilla/5.0 (Linux; Android 4.0.4; A11 Build/KarbonnA11) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86462[] = "GmailHybrid/2.0.1.7057 CFNetwork/609.1.4 Darwin/13.0.0";
const char S86463[] = "GmailPush/2.0.1 CFNetwork/609 Darwin/13.0.0";
const char S86464[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; ja-JP) AppleWebKit/529 (KHTML, like Gecko, Safari/529.0) Lunascape/4.9.9.96";
const char S86465[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530M Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86466[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; POLY PAD_8216HD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86467[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86468[] = "MQQBrowser/3.7/Adr (Linux; U; 2.3.6; zh-cn; GT-I8160 Build/GINGERBREAD.ZCLK3;480*800)";
const char S86469[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Scroll Essential II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86470[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Viettel_i67 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86471[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 992D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S86472[] = "Mozilla/5.0 (Linux; Android 4.0.4; w30hd(QuadCore) Build/W30hd) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S86473[] = "Mozilla/5.0 (Linux; Android 4.0.4; U8836D Build/HuaweiU8836D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S86474[] = "Mozilla/5.0 (Linux; Android 4.1.1; ZTE V970 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S86475[] = "YahooMobileMessenger/1.0 (Android Messenger; 1.8.3) (GT-P7500; samsung; GT-P7500; 4.0.4/IMM76D)";
const char S86476[] = "YahooMobileMessenger/1.0 (Android Messenger; 1.8.3) (p4; samsung; GT-P7500; 4.1.1/JRO03L)";
const char S86477[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S86478[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Malata SMBA1002 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86479[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Malata SMBA1002 Build/JDQ39; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86480[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZP900 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86481[] = "Mozilla/5.0 (Linux; Android 4.0.4; A9+ Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86482[] = "Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A9+ Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86483[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; LG-E730 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86484[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; LG-E730 Build/GRJ22; CM7-JBTaste) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86485[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HUAWEI; W1-U00)";
const char S86486[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GIONEE_Ctrl_V2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86487[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F240S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86488[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S86489[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaN73";
const char S86490[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN78) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S86491[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A15) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S86492[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AT100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);";
const char S86493[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AT100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S86494[] = "Mozilla/5.0 (Linux; Android 4.0.3; Sony Tablet P Build/TISU0144) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S86495[] = "Mozilla/5.0 (Linux; Android 4.1.2; Vivid 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86496[] = "Mozilla/5.0 (Linux; Android 4.0.4; Huawei-H867G Build/HuaweiH867G) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86497[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; 8190 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S86498[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; 8720 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.292 U3/0.8.0 Mobile Safari/534.31";
const char S86499[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo A630t Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.292 U3/0.8.0 Mobile Safari/534.31";
const char S86500[] = "SAMSUNG-GT-I8258_TD/1.0 Android/4.0.4 Release/09.24.2012 Browser/AppleWebKit534.30";
const char S86501[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A89 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.289";
const char S86502[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S6012) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S86503[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S86504[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; MT27i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S86505[] = "Mozilla/5.0 (Linux; Android 4.2.2; Evo V 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86506[] = "Mozilla/5.0 (Linux; Android 4.0.3; Xtreamer Joyz Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86507[] = "Mozilla/5.0 (Linux; Android 4.1.1; PLT7223G Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S86508[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MT25i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S86509[] = "NokiaC2-05/2.0 (08.69) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-05) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S86510[] = "Mozilla/5.0 (Linux; Android 4.0.4; A90S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86511[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8260 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86512[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-S735C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86513[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; GT-I9100) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S86514[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A12 Build/RheaTK) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S86515[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7005 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86516[] = "SonyEricssonC510/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonc510) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S86517[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I545 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86518[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I545 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86519[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT270 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S86520[] = "Mozilla/5.0 (Linux; Android 4.0.4; LENOVO A789 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86521[] = "Mozilla/5.0 (Linux; Android 4.2.2; vivo Xplay Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86522[] = "Mozilla/5.0 (Linux; Android 4.2.2; SHL22 Build/S6290) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86523[] = "Mozilla/5.0 (Linux; Android 4.2.1; Aqua I-5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86524[] = "NokiaC2-00/2.0 (03.99) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311";
const char S86525[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS870 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S86526[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows CE; PPC; 240x320)";
const char S86527[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows CE)";
const char S86528[] = "LG/KF310/v2.0";
const char S86529[] = "Nokia6212 classic/2.0 (06.20) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S86530[] = "SonyEricssonW830i_CMCC/R1KG SEMC-SW/R1KG Release/04.18.2007 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86531[] = "SonyEricssonW830i/R1KG Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86532[] = "LG-KF300 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S86533[] = "LG-KF300-Orange/V10c Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S86534[] = "LG-KF300e/CMCC Release/1.4.2008 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86535[] = "Mozilla/5.0 (Linux; U; Android Basic EB28 2.2.2; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86536[] = "Mozilla/5.0 (Linux; U; Android Platinum_2.2.2; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86537[] = "Mozilla/5.0 (Linux; U; Android SpeedRomBetaV2; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86538[] = "Mozilla/5.0 (Linux; U; Android Ultimate EB28 2.2.2; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86539[] = "Mozilla/5.0 (Linux; U; Android UbuntDroid-v6; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86540[] = "Mozilla/5.0 (Linux; U; Android Vanilla_2.2.2; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86541[] = "Mozilla/5.0 (Linux; U; Android TransGinger 2.2.2 EF09 v1.4; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86542[] = "Mozilla/5.0 (Linux; U; Android TransGinger 2.2.2 EF09 v2.0mg; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86543[] = "Mozilla/5.0 (Linux; U; Android TrasGingerOnCrack EF09 2.2.2 1.3.4 Vampirefo Edition; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86544[] = "Mozilla/5.0 (Linux; U; Android Blaze 2.2.2; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86545[] = "SEC-SGHE210/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86546[] = "ZTE-T870 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 firmware/T870 T06";
const char S86547[] = "NokiaN72/5.0819.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/27/351";
const char S86548[] = "Spice M5252";
const char S86549[] = "DoCoMo/2.0 F02A(c100;TB;W24H17)";
const char S86550[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-kr; SKY IM-A630K Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S86551[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SKY IM-A630K Build/ERE27)";
const char S86552[] = "PPC; 240x320; HTC_P3450/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S86553[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Malata SMBA1002 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86554[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Malata SMBA1002 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86555[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Malata SMBA1002 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86556[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.2.13) Gecko/X Thunderbird/3.1.7";
const char S86557[] = "Nokia5250/UC Browser8.0.4.121/50/444/UCWEB";
const char S86558[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en_us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S86559[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S86560[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S86561[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.2.139/145/355";
const char S86562[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S86563[] = "Mozilla/5.0 (Windows NT 4.0; rv:8.0.1) Gecko/X Firefox/8.0.1";
const char S86564[] = "Mozilla/5.0 (X11; FreeBSD i386; rv:6.0) Gecko/X Firefox/6.0";
const char S86565[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;GT-I8350;7.10;7720)";
const char S86566[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;GT-I8350;7.10;8107)";
const char S86567[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;GT-I8350;7.10;8773)";
const char S86568[] = "Opera/9.80 (Series 60; Opera Mini; U; en) Presto/2.8.119 Version/11.10";
const char S86569[] = "Mozilla/5.0 (Linux; Android 4.1.2; Incredible 2 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86570[] = "Mozilla/5.0 (Linux; Android 4.1.1; Incredible 2 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86571[] = "Mozilla/5.0 (Linux; Android 4.0.4; MOMO8 bird Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S86572[] = "Mozilla/5.0 (Linux; Android 4.0.3; MOMO8 bird Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S86573[] = "Mozilla/5.0 (Linux; Android 4.0.4; ISW16SH Build/S9181) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86574[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; zh-cn; HTC Rezound Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86575[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Rezound Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86576[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.1 Mobile Safari/535.19 Silk-Accelerated=false";
const char S86577[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P7100 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S86578[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S86579[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xenta TAB9-200 Build/Xenta TAB9-200) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S86580[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MT11i Build/4.1.B.0.431) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S86581[] = "WebProcess/8536.25 CFNetwork/596.1 Darwin/12.1.0 (x86_64) (MacBookPro9%2C2)";
const char S86582[] = "iCoreService (unknown version) CFNetwork/596.1 Darwin/12.1.1 (x86_64) (MacBookPro10%2C2)";
const char S86583[] = "Mozilla/5.0 (Linux; Android 4.1.1; LT18i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86584[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xperia Arc S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86585[] = "Mozilla/5.0 (Linux; Android 4.0.4; Thrill 430x Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86586[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S86587[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; POV_TAB-PROTAB27 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86588[] = "Mozilla/5.0 (Linux; Android 4.1.1; AN10BG3 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S86589[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; ALCATEL ONE TOUCH 985N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S86590[] = "LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-gs290) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S86591[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Celkon A 77 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86592[] = "Mozilla/5.0 (Linux; Android 4.0.3; SC-02C Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S86593[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993D Build/ICECREAM) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S86594[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993 Build/ICECREAM) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S86595[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST15i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S86596[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7300 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S86597[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P7300 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S86598[] = "Mozilla/5.0 (Linux; Android 4.1.2; NookColor Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S86599[] = "Mozilla/5.0 (Linux; Android 4.1.1; ST10 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S86600[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86601[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562i Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86602[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; X10i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S86603[] = "Mozilla/5.0 (Windows NT 6.0; rv:20.0) Gecko/X Firefox/20.0 slurp";
const char S86604[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9001 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86605[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; TECNO_D3) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S86606[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-e2222) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile,SAMSUNG-GT-E2222/E2222JVKJ3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S86607[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-e2222) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S86608[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID1042 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444";
const char S86609[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT916 Build/2_26C_2014) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86610[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A21) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S86611[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Q800 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S86612[] = "SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj105i) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S86613[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb";
const char S86614[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P778 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86615[] = "Mozilla/5.0 (Linux; Android 4.0.4; H2000+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86616[] = "Mozilla/5.0 (Linux; Android 4.0.4; V1277 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86617[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT270 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S86618[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; miTab DIAMOND Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86619[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID8065 Build/ICS.MID8065.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S86620[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6400L Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S86621[] = "Micromax X327/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86622[] = "UCWEB/2.0 (iOS; U; iPd OS 6_1_3; en-US; iPd4,1) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S86623[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; iris405 Build/LAVAIRIS405) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86624[] = "Mozilla/5.0 (Windows; U; Windows NT based; en-US) AppleWebKit/534.34 (KHTML, like Gecko)  QtWeb Internet Browser/3.8.4 http://www.QtWeb.net";
const char S86625[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.4.2m";
const char S86626[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.4.4m";
const char S86627[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.4.5m";
const char S86628[] = "Mozilla/5.0 (Linux; Android 4.1.1; Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S86629[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6312 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S86630[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT27i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S86631[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F220K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86632[] = "Mozilla/5.0 (Linux; Android 4.1.1; U7015 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86633[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A770k Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36";
const char S86634[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A770K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86635[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; i-note WiFi 1.1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86636[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; i-note WiFi 1.1 Build/i-mobile i-note WiFi 1.1 v0.2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86637[] = "Mozilla/5.0 (Linux; Android 4.1.1; BS471 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86638[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Intex_Aqua_3.2) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S86639[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9505 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S86640[] = "Mozilla/5.0 (Linux; Android 4.1.2; B92 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86641[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LG-E617G/E617G20a-BLM-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86642[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; WT19i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S86643[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-T989 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/BLACKLiquid-v2.9/hercules";
const char S86644[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6102 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S86645[] = "Mozilla/5.0 (Linux; Android 4.0.4; AD683G Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S86646[] = "Mozilla/5.0 (Linux; Android 4.3; LG-E975 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86647[] = "Mozilla/5.0 (Linux; Android 4.0.4; Optimus_Madrid Build/IceCreamSandwich) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86648[] = "Mozilla/5.0 (Linux; Android 4.1.2; H7500+ Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86649[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaN97) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S86650[] = "Mozilla/5.0 (Linux; Android 4.3; SAMSUNG-SGH-T989 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86651[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; C2104) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S86652[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; C1904) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S86653[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Connect-2G-2.0) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S86654[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E470f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86655[] = "SCH-F339/ POLARIS/6.15 (GUI; compatible; UP.Browser)";
const char S86656[] = "SAMSUNG-SCH-F339/1.0 POLARIS/6.15.WEB Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S86657[] = "SAMSUNG-SGH-T566/T566UXJG5 SHP/VPP/R5 Dolfin/1.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S86658[] = "DoCoMo/2.0 SH05A3(c500;TB;W24H14)";
const char S86659[] = "DoCoMo/2.0 N01B(c500;TB;W24H16)";
const char S86660[] = "MicromaxX560/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S86661[] = "Mozilla/5.0 (Windows; U; WinNT4.0; en-US) Netscape6/6.1";
const char S86662[] = "KDDI-SH3I UP.Browser/6.2_7.2.7.1.K.5.207 (GUI) MMP/2.0";
const char S86663[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VS660 Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86664[] = "Opera/9.80 (Windows ME; U; en) Presto/2.5.24 Version/10.53";
const char S86665[] = "Opera/9.80 (Windows ME; U; en) Presto/2.5.24 Version/10.54";
const char S86666[] = "Opera/9.80 (Windows ME; U; en) Presto/2.5.22 Version/10.50";
const char S86667[] = "NokiaC3-01/5.0 (06.05) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac3-01/UC Browser8.0.3.107/69/444";
const char S86668[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; HD7 T9292)";
const char S86669[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; HD7)";
const char S86670[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; T9296)";
const char S86671[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:7.0) Gecko/X Thunderbird/7.0";
const char S86672[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-LW690 Build/GRJ90-LG-LW690) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86673[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-LW690 Build/GRJ90-LG-LW690) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86674[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8650+ Build/V100R001C92B861) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86675[] = "Mozilla/5.0 (Android; Tablet; rv:11.0) Gecko/11.0 Firefox/11.0 Fennec/11.0";
const char S86676[] = "MQQBrowser/31 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S86677[] = "MQQBrowser/31 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3";
const char S86678[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; ; rv:1.8.0.7) Gecko/X Firefox/1.9.0.1";
const char S86679[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5530)/UC Browser8.4.0.159/50/352/UCWEB Mobile";
const char S86680[] = "Nokia5530/UCWEB8.4.0.150/50/999";
const char S86681[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I747 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86682[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-L710 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86683[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; th-th; WellcoM_A90; Build/ALPS.GB.FDD2.MP.V2.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86684[] = "MQQBrowser/2.6 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S86685[] = "iBrowser/Mini2.8 (Nokia2700c-2/09.98)";
const char S86686[] = "Instagram 3.0.0 (iPhone; iPhone OS 6.0; en_US) AppleWebKit/420+";
const char S86687[] = "MQQBrowser/Mini2.8 (NokiaC1-01/04.40)";
const char S86688[] = "Nokia3230/2.0 (5.0614.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.9.1.120/27/351/UCWEB";
const char S86689[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Desire V Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S86690[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT894 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86691[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA6210 Navigator) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile";
const char S86692[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; AIRIS OnePAD 730 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86693[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I800 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S86694[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86695[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86696[] = "Mozilla/5.0 (Linux; Android 4.1.1; ZP900 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86697[] = "Mozilla/5.0 (Linux; Android 4.1.1; ZP900S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86698[] = "JUC (Linux; U; 2.2.2; zh-cn; GT-B7510; 240*320) UCWEB7.9.0.94/139/444";
const char S86699[] = "Mozilla/5.0 (Linux; Android 4.0.4; GSmart GS202 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86700[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Transformer Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86701[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5253) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S86702[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/527+ (KHTML, like Gecko) QtWeb Internet Browser/1.7 http://www.QtWeb.net";
const char S86703[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA6760s-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S86704[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5230) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S86705[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-s5230) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S86706[] = "UCWEB/2.0 (Java; U; MIDP-2.0; ru; gt-s5230) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S86707[] = "UCWEB/2.0 (Java; U; MIDP-2.0; fa; gt-s5230) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S86708[] = "UCWEB/2.0 (Java; U; MIDP-2.0; es-LA; gt-s5230) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S86709[] = "UCWEB/2.0 (Java; U; MIDP-2.0; vi; gt-s5230) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S86710[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S86711[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; A100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S86712[] = "UCWEB/2.0 (Linux; U; Adr 2.2; en-US; GT-P1000) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S86713[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cynus T2 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S86714[] = "Mozilla/5.0 (Windows NT 6.2; rv:2.0.1) Gecko/X Firefox/4.0.1 CometBird/4.0.1";
const char S86715[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; myPadP4 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86716[] = "LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gs500) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S86717[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P6800 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86718[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P6800 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86719[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P7310 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.28 Safari/537.31";
const char S86720[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7310 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S86721[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ONIDA i099 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86722[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030E Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86723[] = "Mozilla/5.0 (Linux; Android 4.0.4; SK17i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86724[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9a1) Gecko/X SeaMonkey/1.5a";
const char S86725[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT-I5801) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S86726[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC6435LVW Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (HTC6435LVW, 42, 1.9.1)";
const char S86727[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOLO_X1000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86728[] = "Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300K Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S86729[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I8190 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86730[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I8190 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86731[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; i-mobile i-note WIFI3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86732[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) Nokia5800 XpressMusic";
const char S86733[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Titanium_S5) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S86734[] = "Mozilla/5.0 (Linux; Android 4.0.4; Starmobile GOAL Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86735[] = "Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/19.77.34.5 Mobile Safari/537.22";
const char S86736[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; iDnD7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86737[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86738[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I200 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86739[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E973 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86740[] = "Mail/1503 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro9%2C2)";
const char S86741[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LS970 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86742[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9105 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S86743[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; C2104 Build/15.0.A.1.31) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S86744[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S86745[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Sensation XL Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86746[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A15 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S86747[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; TECNO_P3) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S86748[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8190 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.5/580,2.0.5.580,2013-06-26 22:40, +0100, gb";
const char S86749[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; Xenta TAB10-211 Build/Xenta TAB10-211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86750[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7000) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S86751[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 80XSK Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S86752[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Raider X710e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86753[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Holiday Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86754[] = "Xaldon WebSpider;c2VvLWNyYXdsZXI=";
const char S86755[] = "Test Spider 0.2;c2VvLWNyYXdsZXI=";
const char S86756[] = "Norbert the Spider(Burf.com);c2VvLWNyYXdsZXI=";
const char S86757[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire V Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S86758[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S86759[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9100) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S86760[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I605 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86761[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I605 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86762[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI C8813Q Build/HuaweiC8813Q) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S86763[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI C8813 Build/HuaweiC8813) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S86764[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E971 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86765[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGP312 Build/10.3.1.C.0.136) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S86766[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S86767[] = "Xda_orbit_2; 240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S86768[] = "Xda_orbit_2/240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S86769[] = "LG-X335 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S86770[] = "SonyEricssonK500c/R2L SEMC-Browser/4.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86771[] = "SonyEricssonK500c/R2AA SEMC-Browser/4.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86772[] = "SonyEricssonK500c/R2AT SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86773[] = "SonyEricssonK500i/R2AA SEMC-Browser/4.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86774[] = "SonyEricssonK500i/R2L SEMC-Browser/4.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86775[] = "SonyEricssonK500i/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86776[] = "MOT-R974_/00.72 UP.Browser/7.2.7.5.656 (GUI) MMP/2.0";
const char S86777[] = "MOT-R974_/00.72 UP.Browser/7.2.7.5.671 (GUI) MMP/2.0";
const char S86778[] = "MOT-R974_/00.72 UP.Browser/7.2.7.5.658 (GUI) MMP/2.0";
const char S86779[] = "SAMSUNG-GT-M2520/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86780[] = "SAMSUNG-GT-E2652W/E2652WZHKF2 RTK-E/1.0 MMP/1.0 Release/Jun 27 2011 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86781[] = "Mozilla/5.0(Linux;U;Android2.3.3;Zh_cn;GT-P1000;600*1024;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S86782[] = "JUC(Linux;U;Android2.3.3;Zh_cn;GT-P1000;600*1024;)UCWEB7.8.0.95/139/352";
const char S86783[] = "Mozilla/5.0(Linux;U;Android2.3.6;Zh_cn;GT-P1000;600*1024;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S86784[] = "JUC(Linux;U;Android2.3.6;Zh_cn;GT-P1000;600*1024;)UCWEB7.8.0.95/139/444";
const char S86785[] = "JUC(Linux;U;Android2.3.4;Zh_cn;GT-P1000;600*1024;)UCWEB7.8.0.95/139/444";
const char S86786[] = "Flipboard/1.7.2 CFNetwork/548.1.4 Darwin/11.0.0";
const char S86787[] = "Mozilla/5.0 (Windows NT 5.1; rv:2.0b5pre) Gecko/X Minefield/4.0b5pre";
const char S86788[] = "Mozilla/5.0 (Windows; Windows NT 5.1; en-US; rv:2.0b2pre) Gecko/X Minefield/4.0b2pre";
const char S86789[] = "nokiac7-00/UC Browser7.9.0.102/69/352  3gpp-gba";
const char S86790[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7";
const char S86791[] = "JUC(Linux; U;2.3.3;zh_cn;HTC C510e;320*480;)UCWEB7.6.0.75/139/32317";
const char S86792[] = "JUC(Linux; U;2.3.5;zh_cn;HTC C510e;320*480;)UCWEB7.6.0.75/139/32317";
const char S86793[] = "SAMSUNG-SGH-T989/100.40004";
const char S86794[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; iberry AUXUS AX02 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86795[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7310 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86796[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P7310 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86797[] = "Micromax X246/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86798[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; Tablet-9738B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86799[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) 1Password/3.7/X (like Mobile/8C148 Safari/6533.18.5)";
const char S86800[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Pyramid Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86801[] = "Mozilla/5.0 (Linux; Android 4.0.4; QW TB-9718D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S86802[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.53) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S86803[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA5630d-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile";
const char S86804[] = "Mozilla/5.0 (Linux; Android 4.0.4; VIVO 4.3 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86805[] = "JUC (Linux; U; 2.1-update1; zh-cn; Micromax_A60; 240*320) UCWEB7.9.0.94/139/444";
const char S86806[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS TOUCHTAB 7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86807[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s5380k) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S86808[] = "Mozilla/5.0 (Linux; Android 4.1.2; Amaze 4G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86809[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2a1pre) Gecko/X Minefield/3.2a1pre";
const char S86810[] = "Mozilla/5.0 (Linux; Android 4.0.4; A15 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86811[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOLO_A800 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86812[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia603) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S86813[] = "Mozilla/5.0 (Linux; Android 4.0.4; ThL W1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86814[] = "Mozilla/5.0 (Linux; Android 4.0.3; ThL W1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86815[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86816[] = "ZTE U985_TD/1.0 Linux/3.1.10 Android/4.0 Release/8.1.2012 Browser/AppleWebKit534.30";
const char S86817[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; FLY IQ230 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86818[] = "OneBrowser/3.1 (Nokia308/05.80)";
const char S86819[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk530i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S86820[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Sensation 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S86821[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S86822[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cynus T1 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S86823[] = "Mozilla/5.0 (Linux; Android 4.2.2; MZ601 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S86824[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ro-ro; P5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86825[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T959V Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S86826[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3100 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86827[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86828[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7310 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S86829[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; CELKON A83 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86830[] = "Mozilla/5.0 (X11; Linux x86_64; PantechP9090 Build/IMM76I) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S86831[] = "Mozilla/5.0 (X11; Linux x86_64; PantechP9090 Build/JZO54K) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S86832[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A840S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86833[] = "Mozilla/5.0 (Linux; Android 4.2.2; MZ601 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S86834[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P700 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86835[] = "Mozilla/5.0 (Windows NT 5.1; rv:21.0) Gecko/X Thunderbird/21.0";
const char S86836[] = "Dolphin/7.3 CFNetwork/609.1.4 Darwin/13.0.0";
const char S86837[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5620) U2/1.0.0 UCBrowser/9.0.0.295 U2/1.0.0 Mobile";
const char S86838[] = "Mozilla/5.0 (Linux; Android 4.2.2; PG86100 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86839[] = "Mozilla/5.0 (Linux; Android 4.2.2; Evo 3D CDMA Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86840[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-c6712) U2/1.0.0 UCBrowser/9.0.0.295 U2/1.0.0 Mobile";
const char S86841[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC One V) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S86842[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC One V) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S86843[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MT25i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S86844[] = "Mozilla/5.0 (Series40; Nokia307/07.55; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S86845[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Celkon A62 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86846[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S86847[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L710 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S86848[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7005 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86849[] = "Mozilla/5.0 (Linux; Android 4.0.4; SO-03D Build/6.1.F.0.106) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86850[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Starmobile GOAL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86851[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT15i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S86852[] = "Mozilla/5.0 (Linux; Android 4.2.1; ALCATEL ONE TOUCH 8008D Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86853[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS840 4G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S86854[] = "Mozilla/5.0 (Linux; Android 4.0.4; Orange_infinity_996 Build/IceCreamSandwich) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86855[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.4.1 Safari/534.34";
const char S86856[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/537.21 (KHTML, like Gecko) QupZilla/1.4.3 Safari/537.21";
const char S86857[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT925 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86858[] = "Mozilla/5.0 (Linux; Android 4.2.2; Razr HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86859[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb";
const char S86860[] = "Mozilla/5.0 (Linux; Android 4.1.1; ultrafone_501 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86861[] = "Mozilla/5.0 (Linux; Android 4.0.4; WT19i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S86862[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-P990 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86863[] = "Mozilla/5.0 (Linux; Android 4.0.4; Enjoy 71 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S86864[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; L3 Build/TECNOL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/33567";
const char S86865[] = "Mozilla/5.0 (Linux; Android 4.0.3; A101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S86866[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S86867[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Karbonn A2+) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S86868[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A89) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S86869[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Micromax A72) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S86870[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI G520-0000 Build/HuaweiG520-0000) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S86871[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI G520-5000 Build/HuaweiG520-5000) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S86872[] = "Mozilla/5.0 (iPhone; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 OverDrive Media Console/3.0.1";
const char S86873[] = "Mozilla/5.0 (Series40; Nokia5130c-2/07.95; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S86874[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One X+ Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S86875[] = "Mozilla/5.0 (Linux; Android 4.0.4; Philips W536 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86876[] = "Mozilla/5.0 (Linux; Android 4.1.2; SH837W Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86877[] = "Mozilla/5.0 (Linux; Android 4.0.4; S500 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86878[] = "Opera/8.01 (J2ME/MIDP; Opera Mini/1.0.1479/HiFi; SonyEricsson P900; no; U; ssr)";
const char S86879[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; Karbonn_A6) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S86880[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86881[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86882[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S86883[] = "UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; A1+) U2/1.0.0 UCBrowser/9.3.0.321 Mobile";
const char S86884[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86885[] = "Mozilla/5.0 (Linux; U; Android 2.2.1-DL05-Beta-6; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86886[] = "Mozilla/5.0 (Linux; U; Android 2.2.1-DL05-Beta-5.5; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86887[] = "Mozilla/5.0 (Linux; U; Android Android 2.2.2|UberMoment 1.4.1.2 DL25; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86888[] = "Mozilla/5.0 (Linux; U; Android Android 2.2.2|UberMoment 1.4.1; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86889[] = "SAMSUNG-SGH-G400/1.0 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S86890[] = "SAMSUNG-SGH-G400L/1.0 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S86891[] = "BlackBerry7130e/4.1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/104";
const char S86892[] = "SAMSUNG-GT-S8500C/1.0 SHP/VPP/R5 Dolfin/2.0 Nextreaming SMM-MMS/1.2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S86893[] = "SAMSUNG-GT-S8500/S8500XXJF2 SHP/VPP/R5 Dolfin/2.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S86894[] = "DoCoMo/2.0 SH02A(c100;TB;W24H16)";
const char S86895[] = "Mozilla/5.0 440x240 Samsung SCH-U940 NetFront/3.4";
const char S86896[] = "SAMSUNG-SGH-M140/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S86897[] = "HTC_P3700 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S86898[] = "Mozilla/4.0 (compatible; MSIE 6.0; PPC Mac OS X 10.3.9; Tasman 1.0)";
const char S86899[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; EV-S110 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S86900[] = "NokiaX3-02/5.0 (05.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax3-02/UC Browser8.0.3.107/69/444";
const char S86901[] = "Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; Galaxy Nexus Build/ICL53F) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75 Mobile Safari/535.7";
const char S86902[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Galaxy Nexus Build/ITL41F) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75 Mobile Safari/535.7";
const char S86903[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM30B) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S86904[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76B) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S86905[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Galaxy Nexus Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S86906[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76I) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S86907[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75 Mobile Safari/535.7";
const char S86908[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; T-01B; Windows Phone 6.5.3.5)";
const char S86909[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; KDDI-TS01; Windows Phone 6.5.3.5)";
const char S86910[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; T-01A_6.5; Windows Phone 6.5)";
const char S86911[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; H-21; Windows Phone 6.5)";
const char S86912[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Toshiba/X02T; Windows Phone 6.5)";
const char S86913[] = "HTC_Touch_HD_T8282 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S86914[] = "Mail/53 CFNetwork/548.1.4 Darwin/11.0.0";
const char S86915[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:8.0.1) Gecko/X Firefox/8.0.1 SeaMonkey/2.5";
const char S86916[] = "Mozilla/5.0 (Windows NT 5.1; rv:8.0.1) Gecko/X Firefox/8.0.1 SeaMonkey/2.5 Lightning/1.0";
const char S86917[] = "Mozilla/5.0 (Windows NT 5.1; rv:8.0.1) Gecko/X SeaMonkey/2.5";
const char S86918[] = "NokiaC5-05/UC Browser7.9.0.102/50/355/UCWEB";
const char S86919[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.4+ (KHTML, like Gecko) Version/5.0 Safari/535.4+ Ubuntu/11.10 (3.2.0-1~webupd8~oneiric) Epiphany/3.2.0";
const char S86920[] = "Mozilla/5.0 (X11; U; Linux i686; fr-fr) AppleWebKit/531.2+ (KHTML, like Gecko) Version/5.0 Safari/531.2+ Ubuntu/11.10 (3.2.0-1~webupd8~oneiric) Epiphany/2.30.6";
const char S86921[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; PlayTabPro Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86922[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; TAB-PLAYTABPRO Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86923[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1";
const char S86924[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E15i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S86925[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E15i Build/2.1.1.A.0.6) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S86926[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; iDx7 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86927[] = "MicromaxX256/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S86928[] = "iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SmartTab1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86929[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Mytouch 4G Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86930[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC MyTouch4G Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86931[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN82)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S86932[] = "Mozilla/5.0 (Linux; Android 4.0.3; SBM104SH Build/S0030) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86933[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID2 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86934[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID2 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86935[] = "Mozilla/5.0 (Java; U; en-us; nokian72) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S86936[] = "Mozilla/5.0 (Linux; Android 4.0.3; U8800Pro Build/HuaweiU8800Pro) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86937[] = "HTC Streaming Player cingular_us / 1.0 / htc_holiday / 4.0.3";
const char S86938[] = "HTC Streaming Player cingular_us / 1.0 / htc_holiday / 2.3.4";
const char S86939[] = "MQQBrowser/25 (Linux; U; 4.0.3; zh-cn; HUAWEI C8812 Build/C8812V100R001C92B926;480*800)";
const char S86940[] = "MQQBrowser/25 (Linux; U; 4.0.3; zh-cn; HUAWEI C8812 Build/C8812V100R001C92B921;480*800)";
const char S86941[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; WT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86942[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-b7722) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S86943[] = "UCWEB/2.0(Java; U; MIDP-2.0; fa; samsung-gt-b7722i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S86944[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S86945[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S86946[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s3370) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S86947[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A52 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S86948[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxx600) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S86949[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxx600) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S86950[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:17.0) Gecko/X Firefox/17.0 SeaMonkey/2.14.1";
const char S86951[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; LG-E612 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S86952[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP7100D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S86953[] = "Mozilla/5.0 (Linux; Android 4.0.3; Enjoy 10 Plus Build/1.14ST) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S86954[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86955[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86956[] = "Mozilla/5.0 (Linux; Android 4.1.1; H7100 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S86957[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-Xperia S Build/GINGERBREAD_V13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86958[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-Xperia S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S86959[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; ST27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S86960[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P990 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S86961[] = "UCWEB/2.0(BlackBerry; U; 4.4.0.560; en-us; 9330/4.4.0.560) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S86962[] = "UCWEB/2.0(BlackBerry; U; 4.4.0.562; en-us; 9330/4.4.0.562) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S86963[] = "Orbit 5300 Linux/3.0.13 Android/4.0 Release/11.19.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S86964[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; PIRANHA BUSINESS II TAB 9\" Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86965[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; PIRANHA BUSINESS II TAB 9.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86966[] = "HS-T830_TD/1.0 Android/2.3 Release/09.27.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S86967[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A5) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S86968[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A8 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S86969[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-N7000) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S86970[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S720 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S86971[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-CN; LT18i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S86972[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LS970 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86973[] = "Mozilla/5.0 (Linux; Android 4.0.4; AD683G Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86974[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-D700 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86975[] = "Mozilla/5.0 (Linux; Android 4.1.2; PantechP9070 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S86976[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-US; LG-P350 Build/FRG83) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S86977[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9082 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S86978[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A8 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S86979[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6210 Navigator) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S86980[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-P500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S86981[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S86982[] = "Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 6 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86983[] = "Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 6A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86984[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P920 Build/ICS) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S86985[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; Orange_infinity_996 Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86986[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B146 Safari/7534.48.3 UCBrowser8.5.1.228/44/355";
const char S86987[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B329 Safari/7534.48.3 UCBrowser8.5.1.228/44/355";
const char S86988[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MPDC97 BT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S86989[] = "Mozilla/5.0 (Windows NT 6.0; rv:22.0) Gecko/X Firefox/22.0 slurp";
const char S86990[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8552 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S86991[] = "MQQBrowser/4.3/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8552 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S86992[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9103 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S86993[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; SonyEricssonU1) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S86994[] = "Appcelerator Titanium/1.7.2 (iPhone/7.0; iPhone OS; en_US;)";
const char S86995[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9003 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/galaxysl";
const char S86996[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT300 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.60140";
const char S86997[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT300SE Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S86998[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9100) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S86999[] = "Mozilla/5.0 (Linux; Android 4.1.2; ADR930L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87000[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E270K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87001[] = "Mozilla/5.0 (Linux; Android 4.2.1; Starmobile KNIGHT Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87002[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P920 Build/ICS) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87003[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A860L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87004[] = "UCWEB/2.0 (iOS; U; iPh OS 5_1_1; zh-CN; iPh4,1) U2/1.0.0 UCBrowser/9.2.0.312 U2/1.0.0 Mobile";
const char S87005[] = "UCWEB/2.0 (iOS; U; iPh OS 5_1_1; zh-CN; iPh3,1) U2/1.0.0 UCBrowser/9.2.0.312 U2/1.0.0 Mobile";
const char S87006[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I8190 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87007[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; C1604 Build/11.3.A.2.1) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S87008[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; F-05E Build/V08R32D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87009[] = "Mozilla/5.0 (Linux; Android 4.1.1; MT7001 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S87010[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI 1S Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S87011[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xperia S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87012[] = "Mozilla/5.0 (Linux; Android 4.0.4; A21 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87013[] = "Mozilla/5.0 (Linux; Android 4.1.1; Telenor Smart Pro 2 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87014[] = "Mozilla/5.0 (Linux; Android 4.1.2; SBM203SH Build/S0022) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87015[] = "Mozilla/5.0 (Linux; Android 4.0.4; A210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S87016[] = "Mozilla/5.0 (Series40; Nokia111/03.32; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S87017[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S87018[] = "OPWV-SDK UP.Browser/7.0.2.3.119 (GUI) MMP/2.0 Push/PO";
const char S87019[] = "SIE-CF75/10 UP.Browser/7.0.2.2.d.5(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87020[] = "SIE-S65/50 UP.Browser/7.0.2.2.d.3(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87021[] = "SIE-CF75/22 UP.Browser/7.0.2.2.d.7(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87022[] = "SIE-CX65/25 UP.Browser/7.0.0.1.c.3 (GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87023[] = "SIE-ME75/22 UP.Browser/7.0.2.2.d.7(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87024[] = "SIE-C65/43 UP.Browser/7.0.2.2.d.1.100(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87025[] = "SIE-C72/22 UP.Browser/7.0.2.2.d.7(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87026[] = "SIE-ME75/19 UP.Browser/7.0.2.2.d.7(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87027[] = "SIE-CF75/07 UP.Browser/7.0.2.2.d.5(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87028[] = "SIE-C72/16 UP.Browser/7.0.2.2.d.7(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87029[] = "SIE-C65/12 UP.Browser/7.0.0.1.c.3 (GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87030[] = "BlackBerry8980/5.0.0.978 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/1";
const char S87031[] = "BlackBerry8980_CMCC/5.0.0.1055 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/204";
const char S87032[] = "LG-GW825/V10c IEMobile/8.12 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S87033[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-i569 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87034[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-i569 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S87035[] = "zte-S2x/1.0 NetFront/3.5 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87036[] = "Nokia5800 XpressMusic/UC Browser7.6.1.82/50/352";
const char S87037[] = "Nokia5230/UC Browser7.6.1.82/50/351";
const char S87038[] = "Nokia5230-c/UC Browser7.6.1.82/50/355";
const char S87039[] = "DoCoMo/2.0 F08B(c500;TB;W24H16)";
const char S87040[] = "MicromaxQ5/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S87041[] = "NOKIAN95 8GB/UC Browser7.7.1.88/28/444";
const char S87042[] = "NOKIAN95/UC Browser7.7.1.88/28/444";
const char S87043[] = "nokian95/UC Browser7.7.1.88/69/444";
const char S87044[] = "nokian95_8gb/UC Browser7.7.1.88/69/444";
const char S87045[] = "Opera/9.80 (Windows NT 6.1; Opera Tablet/11648; U; en) Presto/2.7.81 Version/11.00";
const char S87046[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MID7015A Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87047[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; MID7015A Build/FRF85B)";
const char S87048[] = "MicromaxX270/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S87049[] = "Mozilla/5.0 (Windows NT 5.1; rv:9.0.1) Gecko/X SeaMonkey/2.6.1";
const char S87050[] = "Mozilla/5.0 (Windows NT 5.1; rv:9.0.1) Gecko/X Firefox/9.0.1 SeaMonkey/2.6.1 Lightning/1.1";
const char S87051[] = "Mozilla/5.0 (Windows NT 5.2; rv:9.0.1) Gecko/X Firefox/9.0.1 SeaMonkey/2.6.1";
const char S87052[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.152 Safari/535.19 Comodo_Dragon/18.1.2.0";
const char S87053[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.152 Safari/535.19 Comodo_Dragon/18.1.2.0";
const char S87054[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE51)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S87055[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; Ultimate10 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87056[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; ATP7526 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87057[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87058[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87059[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87060[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID2 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87061[] = "Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia6303classic) UCBrowser8.2.0.132/69/452/UCWEB Mobile";
const char S87062[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11";
const char S87063[] = "LenovoA66t_TD/1.0 Android/2.2 (Linux; Android 2.2) Release/12.20.2010 Browser/WAP 2.0 AppleWebKit/530.17";
const char S87064[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Lenovo A66t Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S87065[] = "InstacastHD/1.1.3 CFNetwork/548.1.4 Darwin/11.0.0";
const char S87066[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I997 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87067[] = "HTC_TyTN_II Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S87068[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; X10mini Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87069[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; X10mini Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87070[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S87071[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16";
const char S87072[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Woxter Tablet PC 75CXi Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87073[] = "Mozilla/5.0 (Windows NT 5.1; rv:12.0) Gecko/X Firefox/12.0 Kylo/1.1.1";
const char S87074[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87075[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Magnum2X Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87076[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87077[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S87078[] = "Mozilla/5.0 (Linux; Android 4.0.4; SBM106SH Build/S0020) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87079[] = "MQQBrowser/36 (iOS 4; U; CPU like Mac OS X; zh-cn)";
const char S87080[] = "MQQBrowser/36 (iOS 4S; U; CPU like Mac OS X; zh-cn)";
const char S87081[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S87082[] = "Mozilla/5.0 (compatible; Konqueror/3.5; Windows NT 6.0) KHTML/3.5.6 (like Gecko)";
const char S87083[] = "Nokia302/5.0 (14.78) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia302/UC Browser8.0.3.107/69/444";
const char S87084[] = "Mozilla/5.0 (Linux; Android 4.0.3; ZP300 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87085[] = "Nokia201/2.0 (11.21) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.2.1.144/70/355";
const char S87086[] = "Mozilla/5.0 (Linux; Android 4.1.1; MB865 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87087[] = "GoogleEarth/6.2.2.7373(Macintosh;Mac OS X (10.8.2);en;kml:2.2;client:Free;type:default)";
const char S87088[] = "GoogleEarth/6.2.2.6613(Macintosh;Mac OS X (10.8.2);en;kml:2.2;client:Free;type:default)";
const char S87089[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s5380k) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S87090[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s5380d) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S87091[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-B5330 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S87092[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.22 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/535.22 Midori/0.4";
const char S87093[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE R22 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87094[] = "MacOutlook/14.10.0.X (Intel Mac OS X 10.8.2)";
const char S87095[] = "MacOutlook/14.10.0.X (Intel Mac OS X 10.8.3)";
const char S87096[] = "MacOutlook/14.10.0.X (Intel Mac OS X 10.8.4)";
const char S87097[] = "Nokia7310/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia7310) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S87098[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S87099[] = "SAMSUNG-GT-C3520/C3520DDKK4 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S87100[] = "Mozilla/5.0 (Linux; Android 4.0.3; shooteru Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87101[] = "Mozilla/5.0 (Linux; Android 4.0.4; shooteru Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87102[] = "Mozilla/5.0 (Linux; Android 4.0.4; W30(QuadCore) Build/W30) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87103[] = "Mozilla/5.0 (Linux; Android 4.0.4; GEM10312S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87104[] = "Mozilla/5.0 (Linux; Android 4.0.3; GEM10312S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87105[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Sen S2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87106[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; LETAB1003 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87107[] = "Mozilla/5.0 (Linux; Android 4.2; HTC6435LVW Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87108[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MP1043 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87109[] = "Mozilla/5.0 (Macintosh; PowerPC MorphOS 3.1; Odyssey Web Browser; rv:1.18) AppleWebKit/536.9 (KHTML, like Gecko) OWB/1.18 Safari/536.9";
const char S87110[] = "Mozilla/5.0 (Macintosh; PowerPC MorphOS 3.1; Odyssey Web Browser; rv:1.19) AppleWebKit/537.25 (KHTML, like Gecko) OWB/1.19 Safari/537.25";
const char S87111[] = "Mozilla/5.0 (Macintosh; PowerPC MorphOS 3.1; Odyssey Web Browser; rv:1.20) AppleWebKit/537.33 (KHTML, like Gecko) OWB/1.20 Safari/537.33";
const char S87112[] = "Nokia201/2.0 (11.64) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia201/UC Browser8.0.3.107/69/444";
const char S87113[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S87114[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S87115[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Mobile Safari/535.19 Silk-Accelerated=false";
const char S87116[] = "Mozilla/5.0 (Linux; Android 4.0.4; B92 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S87117[] = "Mozilla/5.0 (Linux; Android 4.0.3; F-10D Build/V21R48A) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S87118[] = "YahooMobileMail/1.0 (Android Mail; 1.4.5) (GT-P1000;samsung;GT-P1000;2.3.3/GINGERBREAD)";
const char S87119[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; de; rv:1.8.1.8) Gecko/X SeaMonkey";
const char S87120[] = "SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonj105i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S87121[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A840S Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87122[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A840S Build/Version Metro Sky By Hanzoo1504 Vietnamese; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87123[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MK808B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87124[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A52) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S87125[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; N350) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S87126[] = "Mozilla/5.0 (Linux; Android 4.0.3; Smartfren Tab 7 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S87127[] = "Mozilla/5.0 (Linux; Android 4.0.4; StarTab 715 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S87128[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-B5330) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S87129[] = "Mozilla/5.0 (Series40; Nokia3020/14.53; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S87130[] = "Mozilla/5.0 (Linux; Android 4.0.3; INO_ONE Build/INO_ONEINO_ONE) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87131[] = "Mozilla/5.0 (Linux; Android 4.0.4; INO_ONE_PLUS Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87132[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-US; GT-I5510 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S87133[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Mobile Safari/535.19 Silk-Accelerated=true";
const char S87134[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A65) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S87135[] = "Mozilla/5.0 (Series40; Nokia6303classic/09.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S87136[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S87137[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCEVOV4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S87138[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 80XSK Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S87139[] = "DESAY_TS808_TD/1.0 Android/2.3.5 (Linux; Android 2.3.5) Release/7.25.2012 Mozilla/5.0(Linux;U;Android 2.3.5) AppleWebKit/533.1(KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87140[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; SK17i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S87141[] = "Mozilla/5.0 (Linux; Android 4.0.3; INO_ONE Build/INO_ONEINO_ONE) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87142[] = "Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87143[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-H Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S87144[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-F Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S87145[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Find 5 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/find5";
const char S87146[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; DIT4350 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87147[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8552) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S87148[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E987 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87149[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; iris405 Build/LAVAIRIS405) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S87150[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; MD-717 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87151[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Micromax A110) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S87152[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT270 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S87153[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8262 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S87154[] = "Mozilla/5.0 (Linux; Android 4.0.4; CAL21 Build/AX) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87155[] = "Mozilla/5.0 (Linux; Android 4.2.1; Lenovo S920_ROW Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87156[] = "Mozilla/5.0 (X11; Linux x86_64; Prism II Build/HuaweiU8686) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24 TT/4.1.1";
const char S87157[] = "Mozilla/5.0 (Linux; Android 4.2.1; A900 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87158[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Vivid 4G Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/holiday";
const char S87159[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Karbonn_A2) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S87160[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E220S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87161[] = "Mozilla/5.0 (Linux; Android 4.2.2; Desire S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87162[] = "Mozilla/5.0 (Linux; Android 4.0.4; UBISLATE7C+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87163[] = "Mozilla/5.0 (Linux; Android 4.0.4; UBISLATE 7C+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87164[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; AOSP on Toro Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87165[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; AOSP on geeb_att_us Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87166[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; VirtualBox Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87167[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8552) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S87168[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6233) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S87169[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N8013 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Safari/537.36";
const char S87170[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I200 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87171[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S87172[] = "Mozilla/5.0 (Linux; Android 4.0.4; KPN Smart 300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87173[] = "Mozilla/5.0 (Linux; Android 4.1.1; SH930W Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87174[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 80XSK Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S87175[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-i705 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S87176[] = "Mozilla/5.0 (Linux; Android 4.0.3; PMP5197DULTRA Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Safari/537.36";
const char S87177[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire 500 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87178[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9190 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S87179[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT300SE Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S87180[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT300 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S87181[] = "Opera/9.5 (Microsoft Windows; PPC; Opera Mobi; U) SonyEricssonX1a/R3AA Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87182[] = "LG-GW620 Mozilla/5.0 (Linux; U; Android 1.6) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S87183[] = "Nokia3510i/1.0 (05.30) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S87184[] = "SCH-M369/ POLARIS/6.15 (GUI; compatible; UP.Browser)";
const char S87185[] = "HTC_TouchDual Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S87186[] = "LG-GD350/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S87187[] = "Nokia5233/SymbianOS/9.1 Series60/3.0 3gpp-gba";
const char S87188[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; Vega Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87189[] = "Mozilla/5.0 (Windows NT 4.10) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.120 Safari/535.2";
const char S87190[] = "BlackBerry8520/4.2.0.135 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87191[] = "RSSOwl/2.1.2.X (Macintosh; U; en)";
const char S87192[] = "RSSOwl/2.1.4.X (Macintosh; U; en)";
const char S87193[] = "RSSOwl/2.1.6.X (Macintosh; U; en)";
const char S87194[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2) Gecko Firefox/3.5 (ITABot; robot@itasoftware.com)";
const char S87195[] = "Mozilla/5.0 (Windows NT 5.1; rv:2.0.1) Gecko/X Firefox/4.0.1 slurp";
const char S87196[] = "Mozilla/5.0 (Windows NT 5.1; rv:2.0) Gecko/X Firefox/4.0 slurp";
const char S87197[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i937)";
const char S87198[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/4.5.1_57_DX8-51)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87199[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S87200[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:14.0) Gecko/X Thunderbird/14.0";
const char S87201[] = "Mozilla/5.0 (Linux; Android 4.0.3; A777 Build/A777) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87202[] = "Mozilla/5.0 (Linux; Android 4.0.3; A7 ART Build/A777) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87203[] = "Mozilla/5.0 (Linux; Android 4.0.3; GOCLEVER TAB T75 Build/A777) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87204[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE75-1)/UC Browser8.5.0.183/28/444/UCWEB Mobile";
const char S87205[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; htc_jewel Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87206[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; htc_jewel Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87207[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; htc_jewel 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87208[] = "JUC(Linux;U;Android2.3.6;Zh_cn;A45;320*480;)UCWEB7.8.0.95/139/444";
const char S87209[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SmartTab2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87210[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; SK17i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S87211[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; JMI T970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87212[] = "Mozilla/5.0 (Linux; Android 4.1.2; One V Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87213[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A789 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87214[] = "Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB A104 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87215[] = "iBrowser/2.5 (NokiaN73(08);SymbianOS/9.1 Series60/3.0)";
const char S87216[] = "iBrowser/2.5 (NokiaN73(Z5.01);SymbianOS/9.1 Series60/3.0)";
const char S87217[] = "iBrowser/2.5 (NokiaN73(12);SymbianOS/9.1 Series60/3.0)";
const char S87218[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; S21 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87219[] = "Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB A104.2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87220[] = "NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiax3-02) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S87221[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; SK17i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S87222[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; IMO Z7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87223[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9308 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S87224[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5570C Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S87225[] = "Mozilla/5.0 (Linux; Android 4.0.3; Sony Tablet P Build/TISU0144) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S87226[] = "Mozilla/5.0 (Linux; Android 4.0.3; M729 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S87227[] = "Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 101G10 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S87228[] = "YahooMobileMail/1.0 (Android Mail; 1.3.10) (GT-P6200;samsung;GT-P6200;3.2/HTJ85B)";
const char S87229[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HUAWEI C8813 Build/HuaweiC8813) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S87230[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S87231[] = "Mozilla/5.0 (Linux; Android 4.0.4; V1277 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87232[] = "UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-846_im_india_233) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile";
const char S87233[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-i705 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S87234[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-03E Build/10.1.E.0.250) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S87235[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6233) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S87236[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-I9003 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S87237[] = "LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gs290) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S87238[] = "NokiaC2-06/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-06) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S87239[] = "Nokia302/5.0 (14.78) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia302) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S87240[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TSINTECH Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87241[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A5 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S87242[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaX7-00) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S87243[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P713 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87244[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ605 Build/I.7.1-39.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S87245[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A500) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S87246[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.2";
const char S87247[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0";
const char S87248[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.4";
const char S87249[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9505 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S87250[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax P362 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S87251[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenco CoolTab-80 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87252[] = "Mozilla/5.0 (Linux; Android 4.0.4; iBall Slide 3G7271 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87253[] = "Mozilla/5.0 (Linux; Android 4.0.4; iBall Slide 3G 7334 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S87254[] = "Mozilla/5.0 (Linux; Android 4.1.2; AS870 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87255[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N8013 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.5/580,2.0.5.580,2013-06-26 22:40, -0400, us";
const char S87256[] = "Mozilla/5.0 (Series40; NokiaX2-00/04.90; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S87257[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7005 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87258[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LU6200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87259[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; LG-MS770 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S87260[] = "Mozilla/5.0 (Linux; Android 4.0.4; ISW11SC Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87261[] = "Mozilla/5.0 (Linux; Android 4.0.4; SH-02E Build/S4100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87262[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT28h Build/6.1.E.3.7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S87263[] = "Mozilla/5.0 (Linux; Android 4.1.2; H7500+ Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87264[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S87265[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; LG-P350) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S87266[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S6012) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S87267[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S87268[] = "Mozilla/5.0 (Linux; Android 4.0.4; H2000+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87269[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST15i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87270[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6700s) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S87271[] = "Mozilla/5.0 (Linux; Android 4.2.1; Micromax A110Q Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S87272[] = "Mozilla/5.0 (Linux; Android 4.2.1; i-mobile IQ XA Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87273[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; TECNO N3 Build/master) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S87274[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9295 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S87275[] = "Googlebot/2.1 (+http://www.googlebot.com/bot.html; MSIE 7.0; AOL 9.1; AOLBuild 4334.5009; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.;c2VvLWNyYXdsZXI=";
const char S87276[] = "Googlebot/2.1 (+http://www.googlebot.com/bot.html; MSIE 7.0; Windows NT 5.1; IEMB3; InfoPath.2; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3;c2VvLWNyYXdsZXI=";
const char S87277[] = "Googlebot/2.1 (+http://www.googlebot.com/bot.html; MSIE 7.0; Windows NT 5.1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.5;c2VvLWNyYXdsZXI=";
const char S87278[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/8J2 Safari/7534.48.3";
const char S87279[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/8L1 Safari/7534.48.3";
const char S87280[] = "SonyEricssonJ10i2/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj10i2) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S87281[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S720 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87282[] = "Mozilla/5.0 (Linux; Android 4.1.2; A919i Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87283[] = "SAMSUNG-SGH-A747/A747UCHB3 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S87284[] = "SCH-U350 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S87285[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Nexus S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87286[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Nexus S Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87287[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87288[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Nexus S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87289[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Nexus S Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87290[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus S Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87291[] = "Huawei/1.0/G6150/Browser/obigo-browser/q03c";
const char S87292[] = "NokiaE63/UC Browser7.4.0.65/28/352";
const char S87293[] = "SEC-SGHX660/1.0 TSS/2.5";
const char S87294[] = "Videocon_VC1605_MAUI_Wap 2.0";
const char S87295[] = "Videocon_V1675_Wap MAUI WAP Browser2.0";
const char S87296[] = "Videocon V1510 MAUI WAP Browser";
const char S87297[] = "Nokia7230/5.0 (06.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7230/UC Browser7.9.0.102/69/352";
const char S87298[] = "SonyEricssonU5/UC Browser8.0.4.121/50/352/UCWEB";
const char S87299[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:10.0.1) Gecko/X Firefox/10.0.1 SeaMonkey/2.7.1";
const char S87300[] = "Instacast/1.5 CFNetwork/548.1.4 Darwin/11.0.0";
const char S87301[] = "MQQBrowser/2.3 Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3";
const char S87302[] = "MQQBrowser/2.3 Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S87303[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:12.0) Gecko/X Thunderbird/12.0.1";
const char S87304[] = "nokia500/UC Browser7.9.0.102/69/352  3gpp-gba";
const char S87305[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.4";
const char S87306[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.6";
const char S87307[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.7";
const char S87308[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.8";
const char S87309[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:7.0.1) Gecko/X Firefox/7.0.1 CometBird/7.0.1";
const char S87310[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; eZee'Tab701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87311[] = "Mozilla/5.0 (Linux; Android 4.0.3; P300(Funbook) Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87312[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HS-EG900 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87313[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, gb";
const char S87314[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIANokia N81)/UC Browser8.5.0.183/28/444/UCWEB Mobile";
const char S87315[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIANokia N81 8GB)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S87316[] = "iBrowser/2.7 (NokiaC6-00;SymbianOS/9.1 Series60/3.0)";
const char S87317[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.6.1 Safari/534.34";
const char S87318[] = "Nokia3500c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia3500c) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S87319[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/525.28 (KHTML, like Gecko) Version/3.2.2 Safari/525.28.1";
const char S87320[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Mobiistar Touch S05 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87321[] = "Mozilla/5.0 (Linux; Android 4.0.3; M729 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87322[] = "Mozilla/5.0 (Windows NT 5.1; rv:19.0) Gecko/X Thunderbird/19.0";
const char S87323[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; NEXO 3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87324[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S87325[] = "SonyEricssonJ10i2/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonj10i2) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S87326[] = "SonyEricssonJ10i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; sonyericssonj10i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S87327[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; FLY F410 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87328[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Sensation Z710e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S87329[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Sensation 4G Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S87330[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-P500 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S87331[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7511 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S87332[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MITO A220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87333[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx78) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S87334[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx78) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile TRUSTED/1.0";
const char S87335[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; U1 Build/HCL ME Tablet U1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444";
const char S87336[] = "Mozilla/5.0 (Linux; Android 4.0.3; W732 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87337[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F240L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87338[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus S 4G Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87339[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAP4040_DUO Build/PrestigioPAP4040DUO) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87340[] = "Mozilla/5.0 (compatible; MSIE 3.0; WinNT4.0; Trident/3.0)";
const char S87341[] = "Instagram 3.4.2 (iPad2,1; iPhone OS 4.3.5; en_US; en) AppleWebKit/420+";
const char S87342[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA6730c) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S87343[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA6730c-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S87344[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; AVA757 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87345[] = "Mozilla/5.0 (Linux; Android 4.0.4; Y3 Build/HCL) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S87346[] = "Mozilla/5.0 (Linux; Android 4.0.4; Connect-3G Build/HCL) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S87347[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F240K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87348[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (Nexus 4, 42, 1.9.1)";
const char S87349[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A52 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S87350[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A45 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S87351[] = "NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-05) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S87352[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT27i Build/6.1.1.B.1.54) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S87353[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A9 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S87354[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PHICOMM i370 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87355[] = "Mozilla/5.0 (Linux; Android 4.0.4; AD683G Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87356[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Rhyme S510b Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87357[] = "Instagram 3.5.1 (iPhone4,1; iPhone OS 6_1_3; en_US; en) AppleWebKit/420+";
const char S87358[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S87359[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T989 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S87360[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N8000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S87361[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-N8000 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S87362[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EX'NOTE ??? Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87363[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; EX'NOTE ? Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87364[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; MALATA_I50 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87365[] = "Mozilla/5.0 (Linux; Android 2.3.6; LG-MS840 Build/GRK39F) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S87366[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT270 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S87367[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F120L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87368[] = "Mozilla/5.0 (Linux; Android 4.2.2; SM-T310 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S87369[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S87370[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-in; A1+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S87371[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9082 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S87372[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; X Build/HMX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S87373[] = "Mozilla/5.0 (Linux; Android 4.0.3; U8860 Build/HuaweiU8860) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S87374[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE61i) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S87375[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE61) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S87376[] = "Mozilla/5.0 (Linux; Android 4.0.3; INO_ONE Build/INO_ONEINO_ONE) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87377[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT15i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S87378[] = "Mozilla/5.0 (Linux; Android 4.1.2; Titanium S5 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S87379[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; sr-rs; Telenor Smart Pro 2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S87380[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; Karbonn_A6 Build/MocorDroid4.0.3) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S87381[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P778 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87382[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC Explorer A310e) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S87383[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC Explorer A310e) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S87384[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; ST18i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S87385[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.22 (KHTML, like Gecko) Maxthon/4.0.6.3000 Chrome/25.0.1364.99 Safari/537.22";
const char S87386[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A9+) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S87387[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; CELKON A9+) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S87388[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_918N) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S87389[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Explorer_A310e) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S87390[] = "Mozilla/5.0 (Linux; Android 4.0.4; KHPHN05ANDA Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87391[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A770K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87392[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE T81 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87393[] = "Mozilla/5.0 (Linux; Android 4.0.4; A90S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87394[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAP4044DUO Build/PrestigioPAP4044DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87395[] = "Mozilla/5.0 (Linux; Android 4.2.1; i-mobile IQ 5.3 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87396[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN79) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S87397[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P920 Build/ICS) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87398[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; HTC One Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S87399[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S87400[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SM-G730A Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87401[] = "MOT-E770v/85.97.C3P MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87402[] = "MOT-E770/85.97.F1P MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87403[] = "SAMSUNG-SGH-X640/1.0 UP.Browser/6.2.2.6 (GUI) MMP/1.0";
const char S87404[] = "SAMSUNG-SGH-X640C/1.0 UP.Browser/6.2.2.6 (GUI) MMP/1.0";
const char S87405[] = "SAMSUNG-SGH-ZV60-Vodafone/BUGE4 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S87406[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; ISW11HT Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87407[] = "LG-CU500 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87408[] = "LG-TE365 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S87409[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; MDA Vario/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1)";
const char S87410[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; MDA Vario/1.2 Profile/MIDP-2.0 Configuration/CLDC-1.1)";
const char S87411[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; MDA Vario/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1)";
const char S87412[] = "Nokia-1006 UP.Browser/6.3.0.8.c.1.104 (GUI) MMP/2.0";
const char S87413[] = "NexianNX-G965/MTK Release/21.04.2011 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1  Opera/9.80 (MTK; Nucleus; U; en-US) Presto/2.4.18 Version/10.00";
const char S87414[] = "NexianNX-G965/MTK Release/21.04.2011 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1 Opera/9.80 (MTK; Nucleus; U; id-ID) Presto/2.4.18 Version/10.00";
const char S87415[] = "NexianNX-G965/MTK Release/21.04.2011 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1 Opera/9.80 (MTK; Nucleus; U; en-US) Presto/2.4.18 Version/10.00";
const char S87416[] = "NexianNX-G965/MTK Release/21.04.2011 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1  Opera/9.80 (MTK; Nucleus; U; id-ID) Presto/2.4.18 Version/10.00";
const char S87417[] = "Mozilla/5.0(compatible; Access NetFront/3.5; BREW 3.1.5; en,es)/240x320 PCD C721";
const char S87418[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; Orange Tactile internet 2 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S87419[] = "SCH-U490/1.0 NetFront/3.0.22.2.16 (GUI) MMP/2.0";
const char S87420[] = "SEC-SGHC260/1.0";
const char S87421[] = "BenQ-CF61/1.00/WAP2.0/MIDP2.0/CLDC1.0 UP.Browser/6.3.0.4.c.1.102 (GUI) MMP/2.0";
const char S87422[] = "BenQ-EF51/1.00/WAP2.0/MIDP2.0/CLDC1.0 UP.Browser/6.3.0.4.c.1.102 (GUI) MMP/2.0";
const char S87423[] = "BENQ UP.Browser/6.3.0.4.c.1.102 (GUI) MMP/2.0";
const char S87424[] = "DoCoMo/2.0 N01C(c500;TB;W24H16)";
const char S87425[] = "DoCoMo/2.0 F05C(c500;TB;W24H16)";
const char S87426[] = "DoCoMo/2.0 P02B(c500;TB;W24H16)";
const char S87427[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) QQBrowser/6.9.11079.201";
const char S87428[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0) QQBrowser/6.9.11153.201";
const char S87429[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1 QQBrowser/6.9.11206.201";
const char S87430[] = "NokiaN72/5.0819.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UCWEB7.8.0.87/27/352";
const char S87431[] = "Mozilla/5.0 (X11; Linux i686; rv:9.0) Gecko/X Firefox/9.0 SeaMonkey/2.6";
const char S87432[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:14.0) Gecko/X Firefox/14.0.1 SeaMonkey/2.11";
const char S87433[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; de-DE; rv:1.4) Gecko/X Netscape/7.1 (ax)";
const char S87434[] = "ZTE U970_TD/1.0 Linux/2.6.39 Android/4.0 Release/2.21.2012 Browser/AppleWebKit534.30";
const char S87435[] = "Mozilla/5.0 (Maemo; Linux armv7l; rv:14.0) Gecko/X Firefox/14.0.1 Fennec/14.0";
const char S87436[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Hero Build/GRH78; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87437[] = "CoolPad8180_CMCC_TD/1.0 Linux/2.6.35 Android/2.3 Release/1.20.2012 Browser/AppleWebkit533.1";
const char S87438[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5830 Build/MIUI; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87439[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5830 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87440[] = "Mozilla/5.0 (Linux; Android 4.0.4; M1050S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87441[] = "MQQBrowser/38 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S87442[] = "MQQBrowser/38 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3";
const char S87443[] = "MQQBrowser/381 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S87444[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS TOUCHTAB 9 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87445[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.3.1.31155 (Firestorm-Release; firestorm skin) Safari/533.3";
const char S87446[] = "Mozilla/5.0 (Linux; Android 4.0.3; LT-TLA Build/LTLMR1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87447[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ViewPad 10S Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87448[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S87449[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S87450[] = "Mozilla/5.0 (Linux; Android 4.0.4; N51 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87451[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; M-MP706i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87452[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; POLY PAD_8208HD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87453[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST25i Build/6.1.1.B.1.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S87454[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Eee Pad Transformer Prime TF201 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87455[] = "Mozilla/5.0 (Linux; Android 4.2.1; Transformer Prime TF201 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87456[] = "contype chromeframe/25.0.1364.97";
const char S87457[] = "contype chromeframe/25.0.1364.152";
const char S87458[] = "contype chromeframe/25.0.1364.172";
const char S87459[] = "OneBrowser/3.1 (SAMSUNG-GT-C6712/C6712DDKK3)";
const char S87460[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P350 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S87461[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-US; LG-P350 Build/FRG83) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S87462[] = "Mozilla/5.0 (Linux; Android 4.1.2; MB526 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S87463[] = "Mozilla/5.0 (Linux; Android 4.0.4; M3 Enjoy TV Box Build/V1.03.02MC03_X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87464[] = "Mozilla/5.0 (Linux; Android 4.0.4; M3 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87465[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Incredimail/1.0.5.4111";
const char S87466[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P5100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S87467[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P5100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S87468[] = "Mozilla/5.0 (Windows; U; Win98; en-US; rv:0.9.4) Gecko/X Netscape6/6.2.1";
const char S87469[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Celkon A 220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87470[] = "Mozilla/5.0 (Linux; Android 4.0.3; U9500 Build/HuaweiU9500) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87471[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT881 Build/6.7.3-130_YUS-11) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87472[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P920 Build/ICS) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87473[] = "Dolphin/6.3 CFNetwork/609.1.4 Darwin/13.0.0";
const char S87474[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCL21 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87475[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-P880 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87476[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT885 Build/9.8.3O-148_IRD-3) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87477[] = "Mozilla/5.0 (Series40; Nokia109/04.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S87478[] = "ZTE R270/P107D11/OPERA MINI 4.3/OBIGO Q03C/MIDP-2.0 CONFIGURATION/CLDC-1.1";
const char S87479[] = "ZTE-G X993/X993_V1_Z1_BR_D1EF104 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C";
const char S87480[] = "ZTE-G R255/Obigo Q03C";
const char S87481[] = "Mozilla/5.0 (X11; OpenBSD amd64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31";
const char S87482[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; SPH-D710 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S87483[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; XOLO A700 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S87484[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; IRIS_501 Build/LAVAIRIS501) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S87485[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6710s) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S87486[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-B5330 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S87487[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CloudPad 700w Build/FSV03) AppleWebKit/531.21.10(KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S87488[] = "Mozilla/5.0 (Linux; Android 4.0.4; A90S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87489[] = "Mozilla/5.0 (Linux; Android 2.3.5; PantechP9070 Build/GRJ90) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S87490[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; bq Maxwell Plus Build/2.0.0 X-11:45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87491[] = "Mozilla/5.0 (Windows; N; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth Pro/7.1.1.1580 Safari/532.4";
const char S87492[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6720c-1b) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S87493[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6720c) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S87494[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A3+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S87495[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ALCATEL ONE TOUCH 8000A Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87496[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; TAB-7D3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87497[] = "Mozilla/5.0 (X11; OpenBSD amd64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36";
const char S87498[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.1; en-in; Micromax A110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S87499[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Novo7 Venus Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87500[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; LG-P705 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S87501[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE N9120 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S87502[] = "Mozilla/5.0 (Linux; Android 4.1.1; R8113 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87503[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-03D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87504[] = "Mozilla/5.0 (Linux; Android 4.1.1; GSmart Maya M1 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87505[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A870S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87506[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A870L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87507[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A870K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87508[] = "Mozilla/5.0 (Linux; Android 4.2.1; ALCATEL ONE TOUCH 8008X Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87509[] = "Mozilla/5.0 (Linux; Android 4.2.1; Orange Infinity 8008X Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87510[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT907 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87511[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT907 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87512[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT-I5503) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S87513[] = "Nokia3110c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3110c) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S87514[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9082 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87515[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F180S Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87516[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One X+ Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S87517[] = "Mozilla/5.0 (Linux; Android 4.1.1; Micromax P362 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87518[] = "Mozilla/5.0 (Linux; Android 4.1.2; IQ446 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87519[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A87) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S87520[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A26) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S87521[] = "HTC_P4350 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S87522[] = "SonyEricssonGeneric A1/R8BB Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87523[] = "Mozilla/5.0 (Linux; U; Android 2.2; th-th; WellcoM-A800 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87524[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; th-th; WellcoM-A800 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87525[] = "ZTE-R6/1.0 ACS-NF/3.4 QTV2.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87526[] = "SonyEricssonV630i/R1KG Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87527[] = "MOT-U9/R6632_G_81.17.04R Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 [es-co]";
const char S87528[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-U9/R6632_G_81.24.07R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87529[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-U9/R6632_G_81.27.0DR BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87530[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-U9/R6632_G_81.27.0CR BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87531[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-U9/R6632_G_81.12.15R_B BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87532[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-U9/R6632_G_81.11.48R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87533[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-U9/R6632_G_81.27.05R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87534[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-U9/R6632_G_81.11.29R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87535[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-U9/R6632_G_81.17.04R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87536[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-U9/R6632_G_81.12.18R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87537[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-U9/R6632_G_81.27.0ER BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87538[] = "VM670";
const char S87539[] = "NokiaN75-3/3.0(20.1.172) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87540[] = "Mot-WX295/1.0 ObigoInternetBrowser/Q03C";
const char S87541[] = "SAMSUNG-GT-I6230/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S87542[] = "DoCoMo/2.0 P03B(c500;TB;W24H16)";
const char S87543[] = "LG-GB255/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S87544[] = "LG-GB255-parrot/V100 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87545[] = "Mozilla/5.0(Linux;U;Android2.2;Zh_cn;GT-I9000;480*800;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355";
const char S87546[] = "JUC(Linux;U;Android2.2.1;Zh_cn;GT-I9000;480*800;)UCWEB7.8.0.95/139/355";
const char S87547[] = "JUC(Linux;U;Android2.2;Zh_cn;GT-I9000;480*800;)UCWEB7.8.0.95/139/444";
const char S87548[] = "MicromaxQ1+/Q03C MAUI-browser/MIDP-2.0";
const char S87549[] = "IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB7.8.0.87/41/997";
const char S87550[] = "IUC(U;iOS 4.2.1;Zh-cn;320*480;)/UCWEB7.8.0.87/41/997";
const char S87551[] = "IUC(U;iOS 3.1.3;Zh-cn;320*480;)/UCWEB7.8.0.87/39/800";
const char S87552[] = "IUC(U;iOS 4.1;Zh-cn;320*480;)/UCWEB7.8.0.87/41/800";
const char S87553[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11 Comodo_Dragon/17.5.2.0";
const char S87554[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11 Comodo_Dragon/17.5.2.0";
const char S87555[] = "nokiac6-01/UC Browser8.0.3.107/69/444  3gpp-gba";
const char S87556[] = "SAMSUNG-SCH-F689/1.0 POLARIS/5.30.WAP CTC/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S87557[] = "Mozilla/5.0 (S60V5; U; en-in; Nokia5235)/UC Browser8.4.0.159/50/352/UCWEB Mobile";
const char S87558[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:12.2.1) Gecko/X Firefox/12.2.1-x64 PaleMoon/12.2.1-x64";
const char S87559[] = "Mozilla/5.0 (Windows NT 6.2; rv:12.2.1) Gecko/X Firefox/12.2.1 PaleMoon/12.2.1";
const char S87560[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; th-th; i-mobile i695 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87561[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN82)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S87562[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; u8800 Build/GRJ22; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87563[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; iw-il; U8800 Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87564[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; K8GT_H Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87565[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; K8GT_W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87566[] = "Mozilla/5.0 (Windows; U; WinNT3.51; en-US; rv:1.8.1.7) Gecko/X Firefox/2.0.0.7";
const char S87567[] = "MicromaxX207/Q03C MAUI-browser/";
const char S87568[] = "Mozilla/5.0 (Java; U; en-us; gt-s5230) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S87569[] = "Mozilla/5.0 (Java; U; ru; gt-s5230) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S87570[] = "iBrowser/Mini2.8 (NokiaX2-02/11.57)";
const char S87571[] = "GoogleEarth/6.2.2.7373(Macintosh;Mac OS X (10.6.8);en;kml:2.2;client:Free;type:default)";
const char S87572[] = "GoogleEarth/6.2.2.6613(Macintosh;Mac OS X (10.6.8);en;kml:2.2;client:Free;type:default)";
const char S87573[] = "GoogleEarth/6.2.1.6014(Macintosh;Mac OS X (10.6.8);en;kml:2.2;client:Free;type:default)";
const char S87574[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; GT-P7510 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87575[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; GT-P7510 Build/HTJ85B) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30";
const char S87576[] = "Mozilla/5.0 (Java; U; en-in; nokian72) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S87577[] = "iBrowser/2.5 (Nokia6120c(B20.03);SymbianOS/9.2 Series60/3.1)";
const char S87578[] = "iBrowser/2.5 (Nokia6120c(B20.01);SymbianOS/9.2 Series60/3.1)";
const char S87579[] = "iBrowser/2.5 (Nokia6120c(B21.03);SymbianOS/9.2 Series60/3.1)";
const char S87580[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; M5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87581[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I757M Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87582[] = "Mozilla/5.0 (Linux; Android 4.0.4; GoTab GBT9 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87583[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.2m";
const char S87584[] = "Mozilla/5.0 (Linux; Android 4.0.4; GN800 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87585[] = "Opera/9.80 (BREW; Opera Mini/5.0/27.2260; U; en) Presto/2.8.119 320X240 Samsung SCH-U750";
const char S87586[] = "Mozilla/5.0 (Linux; Android 4.1.1; I7100 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87587[] = "LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-gd510) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S87588[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87589[] = "MQQBrowser/3.7/Adr (Linux; U; 4.1.1; zh-cn; MI 2 Build/JRO03L;720*1280)";
const char S87590[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Surfer 8.31 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87591[] = "OPPO_R817T/1.0 Linux/3.0.35.7 Android/4.0 Release/11.15.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2";
const char S87592[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; HTC Desire Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S87593[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S87594[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848 Duo Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S87595[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848i Duo Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S87596[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.16/276,1.9.16.276,2012-12-21 16:37, +1300, nz";
const char S87597[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500D Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, -0800, ca";
const char S87598[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P5100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S87599[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P5100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S87600[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; MITO_A78 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87601[] = "Mozilla/5.0 (Linux; Android 4.0.4; SO-03D Build/6.1.F.0.106) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87602[] = "Mozilla/5.0 (Linux; Android 4.1.2; ThL W8 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87603[] = "OneBrowser/3.1 (Nokia7230/10.81)";
const char S87604[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; LG-P705) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S87605[] = "Opera/9.80 (Windows NT 4.0) Presto/2.12.388 Version/12.15";
const char S87606[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; C5170 Build/IML77) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S87607[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE50) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S87608[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT15i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S87609[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 510;7.10;8858)";
const char S87610[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 510;7.10;8862)";
const char S87611[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A9+) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S87612[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9220 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S87613[] = "Mozilla/5.0 (Linux; Android 4.0.3; PAP4300 DUO Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87614[] = "UCWEB/2.0 (Linux; U; Adr 4.2.2; en-US; GT-I9500) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S87615[] = "Mozilla/5.0 (Linux; Android 4.1.2; Titanium S1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87616[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LG730 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87617[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cynus T1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87618[] = "Mozilla/5.0 (Linux; Android 4.0.4; JY-G2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87619[] = "Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300L Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87620[] = "OneBrowser/3.1 (SonyEricssonJ108i/R7DA052)";
const char S87621[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iBall Andi5L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87622[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9082 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S87623[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (DROID BIONIC, 42, 1.9.1)";
const char S87624[] = "Mozilla/5.0 (Linux; Android 4.1.2; SH-02E Build/S4260) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87625[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8552) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S87626[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI-ONE Plus Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S87627[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; JK8000 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87628[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-04D Build/9.1.C.0.475) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87629[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Coolpad7295 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S87630[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Coolpad 7295+ Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S87631[] = "Mozilla/5.0 (X11; Linux x86_64; C811 4G) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S87632[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S87633[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-us; Motorola Electrify Build/4.5.1A_SUN_USC_19.0) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S87634[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LVW Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S87635[] = "Mozilla/5.0 (Linux; Android 4.2.2; A1-810 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S87636[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Superion TV 2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87637[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS840 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S87638[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I545 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/jfltevzw";
const char S87639[] = "Mozilla/5.0 (Linux; Android 4.2.1; Lenovo P780 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87640[] = "Mozilla/5.0 (Linux; Android 4.2.1; Lenovo P780_ROW Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87641[] = "Mozilla/5.0 (Linux; Android 4.2.1; iris504Q Build/iris504Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87642[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Amazon Kindle Fire Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87643[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-B5330 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S87644[] = "Mozilla/5.0 (Linux; Android 4.2.1; Micromax A116 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S87645[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F100L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87646[] = "Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 6 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87647[] = "Mozilla/5.0 (Linux; Android 4.0.3; Xtreamer Joyz Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87648[] = "Mozilla/5.0 (Linux; Android 4.1.1; R8113 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S87649[] = "Mozilla/5.0 (Linux; Android 4.0.4; Miracle Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87650[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9082 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S87651[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI-ONE Plus Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S87652[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S87653[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710VMUB Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S87654[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S87655[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI C8813Q Build/HuaweiC8813Q) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S87656[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HUAWEI C8813 Build/HuaweiC8813) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S87657[] = "Nokia8800SI/2.0 (04.20) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87658[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; cherry mobile-SUPERION Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87659[] = "SAMSUNG-SGH-J400/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S87660[] = "SAMSUNG-SGH-J408/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S87661[] = "Nokia6122c_CMCC/5.21 (SymbianOS/9.2; U; Series60/3.1 Nokia6122c_CMCC/5.21; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) Mozilla/5.0 AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S87662[] = "Nokia6122cCN01.02/SymbianOS/9.1 Series60/3.0";
const char S87663[] = "SAMSUNG-GT-E1390/E1390DDIG3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S87664[] = "SAMSUNG-SGH-P910/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S87665[] = "NokiaE63/UC Browser7.6.1.82/28/351";
const char S87666[] = "KDDI-SN3G UP.Browser/6.2.0.14.1.2 (GUI) MMP/2.0";
const char S87667[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) VZW:SCH-i770 PPC 320x320";
const char S87668[] = "Opera/8.01 (J2ME/MIDP; Opera Mini/1.1.5005/1662; en; U; ssr)";
const char S87669[] = "Mozilla/5.0 (Windows NT 4.10) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1";
const char S87670[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.2.0.0 Chrome/12.0.742.112 Safari/534.30";
const char S87671[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.2.0.0 Chrome/12.0.742.112 Safari/534.30";
const char S87672[] = "Opera/9.10 (Macintosh; PPC Mac OS X; U; en)";
const char S87673[] = "Opera/9.25 (Windows NT 4.0; U; en)";
const char S87674[] = "Opera/9.80 (Android 3.2; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.8.149 Version/11.10";
const char S87675[] = "Opera/9.80 (Android 3.2.1; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.8.149 Version/11.10";
const char S87676[] = "Opera/9.80 (Windows 95; U; en) Presto/2.9.168 Version/11.52";
const char S87677[] = "iBrowser/2.6 (Nokia5233;SymbianOS/9.1 Series60/3.0)";
const char S87678[] = "Mozilla/5.0 (Windows NT 5.1; rv:5.0) Gecko/X Firefox/5.0 slurp";
const char S87679[] = "Mozilla/5.0 (X11; U; OpenBSD amd64; en-US; rv:1.8.0.9) Gecko/X Firefox/3.5";
const char S87680[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.5.0)";
const char S87681[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 Sleipnir/3.5.1";
const char S87682[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.5.2)";
const char S87683[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 Sleipnir/3.5.1";
const char S87684[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11 Comodo_Dragon/20.0.1.0";
const char S87685[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11 Comodo_Dragon/20.0.1.0";
const char S87686[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; Monument)";
const char S87687[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;PI86100;7.10;8112)";
const char S87688[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-R760 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87689[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Onda Vi40 Elite - Top Notch Tablets Edition Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87690[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; Dslide 700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87691[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; N-04D Build/AX)";
const char S87692[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; N-04D Build/AX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87693[] = "Mozilla/5.0 (Linux; Android 4.0.3; TAB467 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87694[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB467 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87695[] = "Facebook 4020.0 (iPad; iPhone OS 4.2.1; en_US)";
const char S87696[] = "Mozilla/5.0 (Linux; Android 4.0.3; PLAYTAB Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87697[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; ZTE N789 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87698[] = "Mozilla/5.0 (Linux; Android 4.0.4; L-01D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87699[] = "Mozilla/5.0 (Linux; Android 4.0.4; IMO S88 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87700[] = "Mozilla/5.0 (Linux; Android 4.0.4; T-10B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87701[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxq50) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S87702[] = "Mozilla/5.0 (Linux; Android 4.0.4; NW-Z1000Series Build/E200) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87703[] = "Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia7210supernova) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S87704[] = "JUC (Linux; U; 4.1.1; zh-cn; HTC_One_X; 720*1184) UCWEB7.9.0.94/139/444";
const char S87705[] = "JUC (Linux; U; 4.1.1; zh-cn; HTC_One_X; 720*1184) UCWEB7.9.0.94/139/352";
const char S87706[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TOUCH87 Build/ICS.g08refem618.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87707[] = "Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Safari/535.19 Silk-Accelerated=false";
const char S87708[] = "Mozilla/5.0 (Linux; Android 4.1.1; TM-7043XD Build/V1.1.JELLYBEAN.rus.teXet.X.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87709[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S87710[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I547C Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S87711[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S87712[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6502 Build/10.1.A.1.434) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S87713[] = "WhatsApp/2.8.5610 Android/4.0.3 Device/samsung-GT-P5100";
const char S87714[] = "Nokia202/2.0 (20.36) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia202/UC Browser8.0.3.107/69/444";
const char S87715[] = "Mozilla/5.0 (Linux; Android 4.0.4; VERTU Ti Build/4.0.4.A.0.1.72) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87716[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HTC Incredible S Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S87717[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; HTC Incredible S Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S87718[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A789 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87719[] = "Mozilla/5.0 (Linux; Android 4.1.2; Vivid 4G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87720[] = "Mozilla/5.0 (Linux; Android 4.0.3; Sony Tablet P Build/TISU0144) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S87721[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Iron/12.0.750.0 Chrome/12.0.750.0 Safari/534.30";
const char S87722[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.30 (KHTML, like Gecko) Iron/12.0.750.0 Chrome/12.0.750.0 Safari/534.30";
const char S87723[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; novo7Legend Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87724[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; NGM_Legend2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87725[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Q-Smart S2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87726[] = "Micromax X340/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87727[] = "Mozilla/5.0 (Linux; Android 4.0.4; w30hd(QuadCore) Build/W30hd) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S87728[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; OlivePad V-T210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87729[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Intex Aqua 3.2 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S87730[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87731[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9070) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S87732[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC Wildfire S A510e) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S87733[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC Wildfire S A510e) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S87734[] = "Mozilla/5.0 (Linux; Android 4.0.4; MI-ONE Plus Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87735[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26ii Build/6.1.A.2.55) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S87736[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTCEVODesign4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S87737[] = "Mozilla/5.0 (Linux; Android 4.0.3; PH44100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S87738[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; INTEX_CLOUD_X1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S87739[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Raider X710e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87740[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Holiday Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87741[] = "Mozilla/5.0 (Linux; Android 4.0.4; T-Hub2 Build/TVA301TELBG3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S87742[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZTE V985 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87743[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P760 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S87744[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P768 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S87745[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P769 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S87746[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87747[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87748[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87749[] = "Mozilla/5.0 (Linux; Android 4.2.1; Celkon A119Q Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87750[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; TECNO N3 Build/master) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S87751[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; A111 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S87752[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Micromax A72) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S87753[] = "Mozilla/5.0 (Linux; Android 4.0.3; Slider SL101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.60140";
const char S87754[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F100L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87755[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly s Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87756[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly s Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S87757[] = "Mozilla/5.0 (Linux; Android 4.0.3; chagall Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S87758[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT25i Build/4.1.B.0.631) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87759[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; V4 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33931";
const char S87760[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; Lenovo S820_ROW Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33482";
const char S87761[] = "Mozilla/5.0 (Linux; Android 4.2.1; LIFE PLAY Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87762[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; A54) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S87763[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; Lenovo P770 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S87764[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S87765[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; CROSS A27 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87766[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SM-T211 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S87767[] = "Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5310xpressmusic) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S87768[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9300 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S87769[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-x330t) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S87770[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-x330) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S87771[] = "NokiaC3-01/5.0 (07.51) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac3-01) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S87772[] = "Mozilla/5.0 (Linux; Android 4.1.2; L-01E Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87773[] = "Mozilla/5.0 (Linux; Android 4.1.1; AURUS III Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87774[] = "Mozilla/5.0 (Linux; Android 4.2.2; M8 pro Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S87775[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E617G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87776[] = "Mozilla/5.0 (Linux; Android 4.1.2; 3G7334i Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87777[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F100L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87778[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E989 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87779[] = "Mozilla/5.0 (Linux; Android 4.2.1; Panasonic P51 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87780[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT22i Build/6.2.A.1.100) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S87781[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S6010 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87782[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S6010L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87783[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S87784[] = "SAMSUNG-SGH-L708/ZTIB1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S87785[] = "MOT-EX126 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S87786[] = "SEC-SGHE215L/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S87787[] = "SAMSUNG-GT-S6700T/1.0 SHP/VPP/R5 NetFront/3.5 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S87788[] = "SAMSUNG-GT-S6700/1.0 SHP/VPP/R5 NetFront/3.5 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S87789[] = "SAMSUNG-SWD-V100/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S87790[] = "SAMSUNG-S5200/S5200DDIK3 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S87791[] = "PantechP9050/JJUS09132010; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.21329; U; es) Opera 9.50";
const char S87792[] = "PantechP9050/JJUSX; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.21329; U; es) Opera 9.50";
const char S87793[] = "NokiaC5-00/UC Browser7.8.0.95/28/352";
const char S87794[] = "NokiaC5-00.2/UC Browser7.8.0.95/28/355";
const char S87795[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; phiremod for Nook Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87796[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87797[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87798[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87799[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.2.2.0 Chrome/14.0.835.202 Safari/535.1";
const char S87800[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.2.2.0 Chrome/14.0.835.202 Safari/535.1";
const char S87801[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC EVO 3D Build/MIUI) AppleWebKit/533.1 (KHTML	 like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87802[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.9.11153.201";
const char S87803[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1 QQBrowser/6.9.11206.201";
const char S87804[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.9.11206.201";
const char S87805[] = "iCab/4.8 (Macintosh; U; PPC Mac OS X)";
const char S87806[] = "iCab/4.8 (Macintosh; U; Intel Mac OS X)";
const char S87807[] = "Opera/9.80 (Windows 95; U; ru) Presto/2.10.229 Version/11.61";
const char S87808[] = "Mozilla/5.0 (Linux; U; Ubuntu; en-us) AppleWebKit/525.13 (KHTML, like Gecko) Version/2.2 Firefox/525.13	Safari 2.2";
const char S87809[] = "Mozilla/5.0 (Linux; U; Ubuntu; en-us) AppleWebKit/525.13 (KHTML, like Gecko) Version/2.2 Firefox/525.13";
const char S87810[] = "MQQBrowser/2.7 (Nokia5233;SymbianOS/9.1 Series60/3.0)";
const char S87811[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT875 Build/IML74K; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87812[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT875 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87813[] = "Mozilla/5.0 (Series40; NokiaX2-02/10.90; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15";
const char S87814[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S87815[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; 0000)";
const char S87816[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; <manufacturer>; <model> [;<operator])";
const char S87817[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Nokia)";
const char S87818[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-pt; Sapo a5 Build/tejosunhsine) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87819[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; ZTE-LIBRA Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87820[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9100 Build/JRO03H; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87821[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87822[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100G Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87823[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT18i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S87824[] = "SAMSUNG_KIES";
const char S87825[] = "[FBAN/FB4A;FBAV/3.6.1;FBBV/X;FBDM/{density=3.0,width=1080,height=1920};FBLC/en_US;FBCR/AT&amp-T;FBPN/com.facebook.katana;FBDV/SAMSUNG-SGH-I337;FBSV/4.2.2;FBCA/armeabi-v7a:armeabi;]";
const char S87826[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Next7P Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87827[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT15a Build/4.1.B.0.431) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87828[] = "UCWEB/2.0(Java; U; MIDP-2.0; ar-sa; nokian73-1) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S87829[] = "Mozilla/5.0 (Linux; Android 4.0.9; X710d Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87830[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) FluidApp Version/1.6.1 Safari/534.57.2";
const char S87831[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.59.8 (KHTML, like Gecko) FluidApp Version/1.6.1 Safari/534.59.8";
const char S87832[] = "Mozilla/5.0 (Linux; Android 4.0.3; F-05D Build/V08R31C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87833[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en-US; rv:1.9.1.6) Gecko/X FireDownload/1.2pre Firefox/3.5.6 Wyzo/3.5.6.1 FireTorrent/1.1pre";
const char S87834[] = "Nokia3110c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia3110c) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S87835[] = "SonyEricssonJ10i2/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; fa; sonyericssonj10i2) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile";
const char S87836[] = "SonyEricssonJ10i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; fa; sonyericssonj10i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile";
const char S87837[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Comodo_Dragon/15.0.3.0 Chrome/15.0.874.102 Safari/535.2";
const char S87838[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.2 (KHTML, like Gecko) Comodo_Dragon/15.0.3.0 Chrome/15.0.874.102 Safari/535.2";
const char S87839[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P7300 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87840[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7300 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87841[] = "Mozilla/5.0 (Java; U; en-us; nokian70-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S87842[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; N350 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87843[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Lenovo A60 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87844[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A60+ Build/LenovoLePhone) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87845[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/6.0.2 Safari/7536.26.17 Sleipnir/4.1";
const char S87846[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/6.0.3 Safari/7536.28.10 Sleipnir/4.1.2";
const char S87847[] = "Mozilla/5.0 (Linux; Android 4.0.4; Philips W536 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S87848[] = "Mozilla/5.0 (Linux; Android 4.1.1; rk30sdk Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87849[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S87850[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6200L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0700, th";
const char S87851[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0800, my";
const char S87852[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-7240 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87853[] = "Mozilla/5.0 (iPaddy; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/534.52 (KHTML, like Gecko) NX/2.1.0.8.21 NintendoBrowser/1.0.0.7494.US";
const char S87854[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5880D Build/PMP5880D_v1.0.0_X) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S87855[] = "Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S87856[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87857[] = "Mozilla/5.0 (Linux; Android 4.0.4; C1604 Build/11.1.A.0.64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87858[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAC-10011 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87859[] = "OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87860[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; C1604 Build/11.1.A.0.68) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S87861[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAX5-01) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S87862[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Maxwell Plus Build/1.0.3 X-14:07) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S87863[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo K860 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87864[] = "Mozilla/5.0 (Linux; Android 4.0.4; SO-04D Build/7.0.D.1.137) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87865[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Coolpad7295 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31";
const char S87866[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5880D Build/PMP5880D_v1.0.0_X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S87867[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT28h Build/6.1.E.3.7) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S87868[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Nexus 4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.2/552,2.0.2.552,2013-05-22 23:05, +0100, gb";
const char S87869[] = "Mozilla/5.0 (Linux; Android 4.0.4; w30hd(QuadCore) Build/W30hd) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S87870[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E617G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87871[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT875 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/targa";
const char S87872[] = "Mozilla/5.0 (Linux; Android 4.1.1; TECNO F7 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S87873[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6603 Build/10.3.A.0.423) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S87874[] = "JUC (Linux; U; 2.3.6; zh-cn; HUAWEI_Y210-2010; 320*480) UCWEB7.9.0.94/139/33572";
const char S87875[] = "JUC (Linux; U; 2.3.6; zh-cn; HUAWEI_Y210S; 320*480) UCWEB7.9.0.94/139/33572";
const char S87876[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6233) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S87877[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A35) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S87878[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A27) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S87879[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S87880[] = "Mozilla/5.0 (Linux; Android 4.1.2; PantechP9070 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87881[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; SPHS on Hsdroid Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33745";
const char S87882[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us ; SPHS on Hsdroid Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33745";
const char S87883[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A26) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S87884[] = "HUAWEI_G520-T10_TD/1.0 Android/4.2.1 (Linux; U; Android 4.2.1; zh-cn) Release/04.01.2013 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30";
const char S87885[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9308 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87886[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A660 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S87887[] = "Mozilla/5.0 (Linux; Android 4.2.2; ME302C Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S87888[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Aqua I-5 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S87889[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S87890[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SPH-L720 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S87891[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; AURUS III Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S87892[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia500) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba";
const char S87893[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S87894[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);";
const char S87895[] = "Mozilla/5.0 (Linux; Android 4.1.2; P5_Quad Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S87896[] = "Mozilla/5.0 (Linux; Android 4.1.1; Novo 10 Hero QuadCore Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S87897[] = "SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw20) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S87898[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE51) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S87899[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A9+) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S87900[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; CELKON_A9+) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S87901[] = "NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; NokiaC1-01) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S87902[] = "NokiaC3-00/5.0 (08.71) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; NokiaC3-00) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S87903[] = "SonyEricssonW580i/R8BE Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87904[] = "SAMSUNG-GT-C6620/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87905[] = "NokiaC3-00/5.0 (07.20) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac3-00/UC Browser7.8.0.95/70/355";
const char S87906[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GSMART G1310 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87907[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; GSmart G1310 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87908[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ru-ru; GSmart G1310 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87909[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) CellcomLBR/1.0/e811/CELLCOM_LBR_P180A2V1.0.1/MIDP-2.0 Configuration/CLDC-1.1";
const char S87910[] = "LG-KU250/v1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87911[] = "LG/KU250/v1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87912[] = "LG-KE850 MIC/1.1.14 MIDP-2.0/CLDC-1.1";
const char S87913[] = "KDDI-KC42 UP.Browser/6.2_7.2.7.1.K.6.210 (GUI) MMP/2.0";
const char S87914[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-I777 Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87915[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Droid Build/MIUI-1.11.25.0-SHOLES-EN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87916[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Droid Build/MIUI-1.8.26.0-SHOLES-EN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87917[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Droid Build/MIUI-1.12.30.0-SHOLES-EN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87918[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Droid Build/MIUI-2.2.10.0-SHOLES-EN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87919[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Droid Build/MIUI-1.9.30.0-SHOLES-EN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87920[] = "Opera/9.02 (Windows NT 6.0; U; en)";
const char S87921[] = "Opera/9.00 (Windows NT 6.0; U; en)";
const char S87922[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; MID7010 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87923[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/1.0.154.53 Safari/525.19";
const char S87924[] = "nokia6120c/UC Browser7.9.0.102/69/352";
const char S87925[] = "nokian70-1/UC Browser8.0.3.107/69/444";
const char S87926[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; Dell Streak 10 Pro Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S87927[] = "Mozilla/5.0 (Series40; NokiaX2-02/10.90; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S87928[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:2.0.1) Gecko/X Firefox/4.0.1 SeaMonkey/2.1";
const char S87929[] = "SonyEricssonX1i/R3AA Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S87930[] = "X1i Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S87931[] = "HS-T92_TD/1.0 Android/2.2 Release/12.10.2011 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87932[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0.1) Gecko/X Firefox/4.0.1 slurp";
const char S87933[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0) Gecko/X Firefox/4.0 slurp";
const char S87934[] = "Mozilla/5.0 (Android; Linux armv7l; rv:10.0a1) Gecko/X Firefox/10.0a1 MobiFilter/10.0a1";
const char S87935[] = "Mozilla/5.0 (Android; Linux armv7l; rv:10.0) Gecko/X Firefox/10.0 Spearmint/10.0";
const char S87936[] = "Mozilla/5.0 (Linux; Android 4.0.3; ASTROPLUS Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87937[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; De-Sensed HTC Vivid Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87938[] = "Safari/8536.25 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacBookAir4%2C2)";
const char S87939[] = "Safari/8536.25 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacBookPro9%2C2)";
const char S87940[] = "Safari/8536.25 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (iMac12%2C1)";
const char S87941[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; th-th; 697i Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87942[] = "Mozilla/5.0 (Symbian3; U; en-us; Nokia603)/UC Browser8.5.0.183/51/352/UCWEB Mobile";
const char S87943[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5228)/UC Browser8.5.0.183/50/444/UCWEB Mobile";
const char S87944[] = "Mozilla/5.0 (Linux; Android 4.0.4; WX435 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87945[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; MT27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S87946[] = "Mozilla/5.0 (Linux; Android 4.0.4; Novo7Flame Build/X.qm) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87947[] = "SAMSUNG-GT-S3770K/S3770KDDKL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S87948[] = "Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.1.161/69/355";
const char S87949[] = "GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.2.9200.0);en;kml:2.2;client:Free;type:default)";
const char S87950[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; W100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87951[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5753e) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S87952[] = "Mozilla/5.0 (Linux; Android 4.0.4; M-MP1010i Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S87953[] = "SAMSUNG-GT-S5222 Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10";
const char S87954[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:7.0a1) Gecko/X Firefox/7.0a1 (.NET CLR 3.5.30729)";
const char S87955[] = "HS-T860_TD/1.0 Android/2.3 Release/05.31.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87956[] = "Mozilla/5.0 (Windows NT 6.0; rv:18.0) Gecko/X Firefox/18.0 slurp";
const char S87957[] = "Mozilla/5.0 (Linux; Android 4.2.1; XT894 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87958[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT894 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87959[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonk800i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S87960[] = "UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; sonyericssonk800i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S87961[] = "JUC (Linux; U; 2.3.7; zh-cn; ZTE-T_U960s; 480*800) UCWEB7.9.3.103/139/800";
const char S87962[] = "JUC (Linux; U; 2.3.6; zh-cn; ALCATEL_ONE_TOUCH_985N; 320*480) UCWEB7.9.0.94/139/444";
const char S87963[] = "JUC (Linux; U; 2.3.6; zh-cn; ALCATEL_ONE_TOUCH_985N; 320*480) UCWEB7.9.0.94/139/355";
const char S87964[] = "JUC (Linux; U; 2.3.5; zh-cn; LG-P920; 480*800) UCWEB7.9.0.94/139/352";
const char S87965[] = "JUC (Linux; U; 2.2.2; zh-cn; LG-P920; 480*800) UCWEB7.9.0.94/139/352";
const char S87966[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I957 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87967[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar Touch S05 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S87968[] = "OPPO_U705T/1.0 Linux/3.0.35.7 Android/4.0 Release/11.15.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2";
const char S87969[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq7) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S87970[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N860 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S87971[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S87972[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562i Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S87973[] = "Mozilla/5.0 (Linux; Android 4.1.1; ST8 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S87974[] = "Mozilla/5.0 (Linux; Android 4.0.4; H2000+ Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S87975[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-ae; I200 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S87976[] = "SAMSUNG-GT-E2262/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S87977[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328w Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S87978[] = "Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A30 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S87979[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-S7562 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S87980[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-S7562i Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S87981[] = "contype chromeframe/26.0.1410.43";
const char S87982[] = "contype chromeframe/26.0.1410.64";
const char S87983[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo K860 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87984[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAP4500TDUO Build/PrestigioPAP4500TDUO) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87985[] = "Mozilla/5.0 (Linux; Android 4.0.4; SO-01E Build/9.0.G.1.108) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S87986[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.105; en-us; 8820/2.7.0.105) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S87987[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.109; en-us; 8820/2.7.0.109) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S87988[] = "UCWEB/2.0(BlackBerry; U; 2.7.0.106; en-us; 8820/2.7.0.106) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S87989[] = "JUC (Linux; U; 2.3.6; zh-cn; A89; 320*480) UCWEB7.9.0.94/139/444";
const char S87990[] = "JUC (Linux; U; 4.0.4; zh-cn; A89; 480*800) UCWEB7.9.0.94/139/444";
const char S87991[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; jblend) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile/HS-T39_TD/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0 Config/CLDC-1.1";
const char S87992[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; jblend) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile/HS-T39_TD/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0 Config/CLDC-1.1,/HS-T39_TD/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0 Config/CLDC-1.1";
const char S87993[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Mitab FLY Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S87994[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; W20) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S87995[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S87996[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; TECNO N3) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S87997[] = "Mozilla/5.0 (Linux; Android 4.1.1; Celkon A119 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S87998[] = "OneBrowser/3.2/Lenovo-A288t_TD/S100 Linux/2.6.35 Android/2.3.5 Release/02.29.2012 Browser/AppleWebkit533.1 Mobile Safari/533.1";
const char S87999[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTX21 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88000[] = "Mozilla/5.0 (Linux; Android 4.1.1; Optimus_San_Remo Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88001[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; HTC One Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S88002[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Tipo Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88003[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC One S Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S88004[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Thrill 430x Build/FSV14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88005[] = "Mozilla/5.0 (Linux; Android 4.1.1; Telenor Smart Pro 2 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88006[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; C1504 Build/11.3.A.0.47) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S88007[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A500 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S88008[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.6.6.X (Second Life Release; default skin) Safari/533.3";
const char S88009[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-in; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S88010[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A860S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S88011[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.7; en-in; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S88012[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.7; en-in; ST25i Build/6.0.B.3.188) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S88013[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-01E Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S88014[] = "Mozilla/5.0 (Linux; Android 4.1.2; SH837W Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S88015[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N8000 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S88016[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N8000 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1/n8000";
const char S88017[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC One S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S88018[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC VLE_U Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S88019[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A15) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S88020[] = "Maui Browser Karbonn K73";
const char S88021[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia808 PureView) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S88022[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F180S Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S88023[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; ST27i Build/6.2.A.1.100) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S88024[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; MT27i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S88025[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S88026[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Titanium_S5 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S88027[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; Titanium S5 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444";
const char S88028[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S88029[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; LG-E470f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88030[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTX21 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88031[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S88032[] = "Mozilla/5.0 (Linux; Android 4.3; Amazon Kindle Fire HD Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S88033[] = "Mozilla/5.0 (Linux; Android 4.0.4; CAL21 Build/AX) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88034[] = "Mozilla/5.0 (Linux; Android 4.0.4; V1277 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88035[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S88036[] = "NokiaX2-01/5.0 (08.71) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; NokiaX2-01) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S88037[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST27i Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S88038[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S88039[] = "HTC_Touch2_T3335 Opera/9.50 (Windows NT 5.1; U; en)";
const char S88040[] = "nokia2626";
const char S88041[] = "SAMSUNG-SGH-T329/T329UVFL8 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S88042[] = "Nokia7250I/1.0 (3.22) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S88043[] = "SAMSUNG-GT-C3510/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S88044[] = "BlackBerry8130m/4.5.0.127 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/294";
const char S88045[] = "NokiaN73/UC Browser7.4.0.65/28/351";
const char S88046[] = "Mot-WX290/1.0 ObigoInternetBrowser/Q03C";
const char S88047[] = "ZTE-F912/1.0 ACS-NF/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S88048[] = "KDDI-SH3H UP.Browser/6.2_7.2.7.1.K.5.207 (GUI) MMP/2.0";
const char S88049[] = "NOKIAE52-1";
const char S88050[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) iCab/4.8 Safari/533.16";
const char S88051[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/69/352";
const char S88052[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/69/444";
const char S88053[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/69/444  3gpp-gba";
const char S88054[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/70/352";
const char S88055[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/69/352  3gpp-gba";
const char S88056[] = "sonyericssonk800i/UC Browser8.0.3.107/70/352";
const char S88057[] = "sonyericssonk800i/UC Browser8.0.3.107/69/352";
const char S88058[] = "sonyericssonk800i/UC Browser8.0.3.107/69/444";
const char S88059[] = "NokiaN97/UC Browser7.9.0.102/50/355/UCWEB";
const char S88060[] = "MQQBrowser/31 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3";
const char S88061[] = "MQQBrowser/31 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3";
const char S88062[] = "MQQBrowser/31 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S88063[] = "Mozilla/5.0 (Windows; U; Windows NT 7.1; ru; rv:1.9.2.15) Gecko/X Firefox/4.0";
const char S88064[] = "Mozilla/5.0 (Windows NT 7.0; Win64; x64; rv:3.0b2pre) Gecko/X Firefox/4.0b12pre";
const char S88065[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:9.0) Gecko/X Thunderbird/9.0.1";
const char S88066[] = "JUC(Linux; U;2.3.6;zh_cn;U8860;480*854;)UCWEB7.6.0.75/139/7675";
const char S88067[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5670 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S88068[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-sg; Slicepad L8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S88069[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-sg; Slicepad L8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S88070[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; WT19i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S88071[] = "Nokia112/2.0 (03.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia112) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S88072[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_Pyramid Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88073[] = "Mozilla/5.0 (Linux; Android 4.0.4; A898 Duo Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88074[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST18i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S88075[] = "JUC (Linux; U; 2.3.6; zh-cn; A44; 240*320) UCWEB7.9.0.94/139/444";
const char S88076[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (HTC One X, 39, 1.8.2)";
const char S88077[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA6110) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S88078[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6110navigator) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S88079[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST18i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S88080[] = "OneBrowser/3.1 (SAMSUNG-GT-S5620/S5620DDKC1)";
const char S88081[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC Desire X Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S88082[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S88083[] = "Mozilla/5.0 (X11; Linux x86_64; LT30p-o) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S88084[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Mi-Book Mi7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S88085[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I8530 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S88086[] = "Mozilla/5.0 (Linux; Android 4.0.3; chagall Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S88087[] = "JUC (Linux; U; 4.1.2; zh-cn; GT-I9100; 480*800) UCWEB7.9.0.94/139/444";
const char S88088[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK BNTV250 Build/GINGERBREAD 1.4.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, -0500, us";
const char S88089[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I8530) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S88090[] = "UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; LG-P500) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S88091[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/8J2 Safari/7534.48.3";
const char S88092[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/8L1 Safari/7534.48.3";
const char S88093[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530M Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88094[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9260 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88095[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTabS2110AF Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S88096[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9003 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88097[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9003 Build/JZO54K)";
const char S88098[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9003 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88099[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5380k) U2/1.0.0 UCBrowser/9.0.0.295 U2/1.0.0 Mobile";
const char S88100[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID7065 Build/ICS.MID7065.X) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S88101[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; TM901 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S88102[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; TM901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S88103[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; ZTE-V6500 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88104[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; MB855 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88105[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; MB855 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88106[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S88107[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8812E Build/HuaweiC8812E) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S88108[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-P3100) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S88109[] = "Opera/9.80 (Windows 98) Presto/2.12.388 Version/12.15";
const char S88110[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (DROID RAZR HD, 42, 1.9.1)";
const char S88111[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-79) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (DROID RAZR HD, 52, 1.9.9.2)";
const char S88112[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; C1504) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S88113[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S88114[] = "Mozilla/5.0 (Linux; Android 4.1.1; rk30sdk Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S88115[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Ctrl_V1 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S88116[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST15i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88117[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.2/552,2.0.2.552,2013-05-22 23:05, +0100, gb";
const char S88118[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT15i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88119[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A30 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S88120[] = "Mozilla/5.0 (Linux; Android 2.3.6; SPH-M930BST Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S88121[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; TECNO D3 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S88122[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7300 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S88123[] = "Mozilla/5.0 (Linux; Android 4.0.4; G2 Build/ICS.Trio_Stealth_G2.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S88124[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI T8951 Build/HuaweiT8951) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S88125[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5502 Build/10.1.1.A.1.310) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88126[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Micromax A72) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S88127[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S88128[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S88129[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) FluidApp Version/1842 Safari/534.57.7";
const char S88130[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A65 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S88131[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A90S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S88132[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; WT19i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S88133[] = "Mozilla/5.0 (Linux; Android 4.0.4; C5155 Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S88134[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26w Build/6.1.A.2.45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S88135[] = "Mozilla/5.0 (Linux; Android 4.1.2; A919i Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S88136[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.60140";
const char S88137[] = "Mozilla/5.0 (Linux; Android 4.0.4; A9+ Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S88138[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7511 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S88139[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST18i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S88140[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5670) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S88141[] = "Mozilla/5.0 (Linux; Android 4.3; SGH-I337 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S88142[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9308 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S88143[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P778 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88144[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P925 Build/ICS) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88145[] = "Mozilla/5.0 (Linux; Android 4.0.4; StarTrail III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S88146[] = "Mozilla/5.0 (Linux; Android 4.2.1; H7500+ Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88147[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT11i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S88148[] = "Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D36 Safari/9537.53";
const char S88149[] = "Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D41 Safari/9537.53";
const char S88150[] = "Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D48 Safari/9537.53";
const char S88151[] = "Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D49 Safari/9537.53";
const char S88152[] = "Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D52 Safari/9537.53";
const char S88153[] = "Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D51 Safari/9537.53";
const char S88154[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One XL Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S88155[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaN97 mini) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S88156[] = "Mozilla/5.0 (Linux; Android 4.1.2; New Andromax-i Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88157[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9082 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S88158[] = "Mozilla/5.0 (Linux; Android 4.3; GT-N8013 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S88159[] = "UCWEB/2.0 (iOS; U; iPh OS 6_1_2; zh-CN; iPh4,1) U2/1.0.0 UCBrowser/9.2.0.312 U2/1.0.0 Mobile";
const char S88160[] = "UCWEB/2.0 (iOS; U; iPh OS 6_1_2; zh-CN; iPh5,2) U2/1.0.0 UCBrowser/9.2.0.312 U2/1.0.0 Mobile";
const char S88161[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S88162[] = "HTC Streaming Player htc_asia_india / 1.0 / htc_buzz / 2.2.1";
const char S88163[] = "HTC Streaming Player vodafone_uk / 1.0 / htc_buzz / 2.2.1";
const char S88164[] = "HTC Streaming Player htc_asia_wwe / 1.0 / htc_buzz / 2.2.1";
const char S88165[] = "HTC Streaming Player htc_wwe / 1.0 / htc_buzz / 2.2.1";
const char S88166[] = "HTC Streaming Player h3g_dan / 1.0 / htc_buzz / 2.2.1";
const char S88167[] = "HTC Streaming Player telstra_wwe / 1.0 / htc_buzz / 2.2.1";
const char S88168[] = "HTC Streaming Player vodafone_de_1und1 / 1.0 / htc_buzz / 2.2.1";
const char S88169[] = "HTC Streaming Player tmo_uk / 1.0 / htc_buzz / 2.2.1";
const char S88170[] = "HTC Streaming Player htc_wwe_tesco / 1.0 / htc_buzz / 2.2.1";
const char S88171[] = "HTC Streaming Player h3g_uk / 1.0 / htc_buzz / 2.2.1";
const char S88172[] = "HTC Streaming Player vodafone_spa / 1.0 / htc_buzz / 2.2.1";
const char S88173[] = "HTC Streaming Player htc_wwe-afk / 1.0 / htc_buzz / 2.2.1";
const char S88174[] = "HTC Streaming Player vodafone_fr / 1.0 / htc_buzz / 2.2.1";
const char S88175[] = "HTC Streaming Player vodafone_de / 1.0 / htc_buzz / 2.2.1";
const char S88176[] = "MOT-E8/R6713_G_71.14.1ER_B MotoWebKit/417.19 (KHTML, like Gecko) BER/2.0 Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/9.0";
const char S88177[] = "Spice M5170";
const char S88178[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; cs-cz; GSmart G1317D Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88179[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; cs-cz; GSmart G1317D Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88180[] = "LGE-LG260 POLARIS-LG260/2.0(GUI) MMP/2.0";
const char S88181[] = "LG-LG260 POLARIS-LG260/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S88182[] = "Mozilla/5.0(Linux;U;Android2.2.1;Zh_cn;GT-S5660;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S88183[] = "JUC(Linux;U;Android2.2.1;Zh_cn;GT-S5660;320*480;)UCWEB7.8.0.95/139/444";
const char S88184[] = "ZTE-ONIDA F970/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S88185[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SKY IM-A600S Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S88186[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:11.0) Gecko/X Thunderbird/11.0.1 Lightning/1.3";
const char S88187[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT702 Build/SGC03040) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S88188[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT702 Build/SGC03040) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S88189[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; X10mini Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88190[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; X10mini Build/FRG83)";
const char S88191[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID2 GLOBAL Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88192[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB855 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88193[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MB855 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88194[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB855 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88195[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB855 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88196[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB855 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88197[] = "nokiae5-00/UC Browser7.9.0.102/69/352  3gpp-gba";
const char S88198[] = "Mozilla/5.0 (X11; OpenBSD amd64; rv:11.0) Gecko/X Firefox/11.0";
const char S88199[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; iDx10 3G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88200[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-N7000 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S88201[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-N7000 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S88202[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; GT-I9018 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88203[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; GT-I9018 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S88204[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; GT-I9018 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S88205[] = "Mozilla/5.0 (Linux; U; Android 1.5.1.12-RT-X.X; zh-cn; K-Touch W619 Build/AliyunOs-2012) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S88206[] = "Mozilla/5.0 (Linux; U; Android 1.5.1.13-RT-X.X; zh-cn; K-Touch W619 Build/AliyunOs-2012) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S88207[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC Flyer P512 Build/JRO03C; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88208[] = "JUC (Linux; U; 2.3.6; zh-cn; Lenovo_A366t; 320*480)";
const char S88209[] = "480x800 LGE VS750 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S88210[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1";
const char S88211[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1";
const char S88212[] = "MQQBrowser/2.6 (NokiaNXX;SymbianOS/9.3 Series60/3.2)";
const char S88213[] = "Mozilla/5.0 (Linux; Android 4.0.4; MEDION LIFE P4012 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88214[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonk530i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S88215[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; LG-P705 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S88216[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S88217[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Rezound Build/IML74K; CyanogenMod-RAGE 2.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88218[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A770K Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S88219[] = "WhatsApp/2.8.5610 Android/4.0.4 Device/samsung-GT-P3100";
const char S88220[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0200, za";
const char S88221[] = "Mozilla/5.0 (Linux; Android 4.0.4; TCL S710 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88222[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I897 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S88223[] = "Mozilla/5.0 (Linux; Android 4.0.4; A210 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S88224[] = "Mozilla/5.0 (Linux; Android 4.0.4; A90S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88225[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A919 Duo Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88226[] = "Mozilla/5.0 (Linux; Android 4.0.4; Andromax U Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88227[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S88228[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; LG-P500 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S88229[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; LG-P500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S88230[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NB-1458 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88231[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ST3_Blade Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88232[] = "UCWEB/2.0 (Linux; U; Adr 3.2; en-US; GT-P6200) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S88233[] = "SonyEricssonC902/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonc902) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S88234[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A7+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S88235[] = "Mozilla/5.0 (Linux; Android 4.1.1; Novo 10 Hero QuadCore Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S88236[] = "Spice M6100";
const char S88237[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; GT-I9100) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S88238[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; zh-CN; GT-S5830) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile";
const char S88239[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; TECNO_N7 Build/TECNOTECNO_N7) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S88240[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; Nokia6220c-1) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S88241[] = "Mozilla/5.0 (Linux; Android 4.1.1; XOLO A1000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88242[] = "Mozilla/5.0 (Linux; Android 4.0.4; i9400 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88243[] = "Mozilla/5.0 (Linux; Android 4.2.1; ThL W7 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88244[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S88245[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-I8530 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S88246[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Titanium S1 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S88247[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI-ONE Plus Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S88248[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3110c) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S88249[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6131) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S88250[] = "UCWEB/2.0 (Java; U; MIDP-2.0; vi; nokia6085) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S88251[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GN700W Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88252[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GN700W Build/JRO03C)";
const char S88253[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; innos i6 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88254[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; i6 Build/BOWAYBOWAY_i6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88255[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC Desire X Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S88256[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC Desire X dual sim Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S88257[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; ST18i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S88258[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S88259[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-E970 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S88260[] = "Mozilla/5.0 (Mobile; rv:18.1) Gecko/18.1 Firefox/18.1";
const char S88261[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P760 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S88262[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P768 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S88263[] = "Mozilla/5.0 (Linux; Android 4.1.1; AMI-TT4 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S88264[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A1+ Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S88265[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT28h Build/6.1.E.3.7) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S88266[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT28h Build/6.1.E.2.68) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S88267[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E15i) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S88268[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; ST25i) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S88269[] = "NokiaX2-02/2.0 (11.79) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311";
const char S88270[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A35 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S88271[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGP321 Build/10.3.1.A.0.244) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S88272[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT894 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88273[] = "Mozilla/5.0 (Linux; Android 4.0.3) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/19.77.34.5 Safari/537.22";
const char S88274[] = "Mozilla/5.0 (Linux; Android 4.0.4) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/19.77.34.5 Safari/537.22";
const char S88275[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT926 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88276[] = "Mozilla/5.0 (Linux; Android 4.2.2; Desire S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88277[] = "Mozilla/5.0 (Linux; Android 4.1.1; A50 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88278[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S88279[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8550L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88280[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N8013 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S88281[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S88282[] = "Mozilla/5.0 (Linux; Android 4.1.2; ME371MG Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S88283[] = "Mozilla/5.0 (Linux; Android 4.0.3; A200 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S88284[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-cn; HTCA3366/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88285[] = "Mot-WX395/1.0 ObigoInternetBrowser/Q03C";
const char S88286[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_E101";
const char S88287[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-tw; myPad P1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88288[] = "LG-KM553/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S88289[] = "LGE-AX265/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S88290[] = "DoCoMo/2.0 P02A(c100;TB;W24H15)";
const char S88291[] = "Nokia5800 XpressMusic/UC Browser7.4.0.65/50/351";
const char S88292[] = "Nokia6630/1.0 (5.03.08) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.7.1.88/27/444";
const char S88293[] = "NokiaX6-00/UC Browser7.8.0.95/50/355";
const char S88294[] = "Mozilla/4.0 (compatible; Linux 2.6.10) NetFront/3.3 Kindle/1.0 (screen 600x800)";
const char S88295[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Vision Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88296[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Vision Build/MIUI OsiMood) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88297[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Vision Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88298[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Vision Build/MIUI OsiMood) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88299[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ja-jp; 001HT Build/GRH78C) AppleWebKit/533.16 (KHTML, like Gecko) Version/4.0 Safari/533.16";
const char S88300[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.7.10) Gecko/X Firefox/1.0.6";
const char S88301[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:8.0) Gecko/X Firefox/8.0-x64 PaleMoon/8.0-x64";
const char S88302[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:8.0) Gecko/X Firefox/8.0 PaleMoon/8.0";
const char S88303[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-U V880 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88304[] = "Mozilla/5.0 (Linux; U; Android 3.1; es-es; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88305[] = "Mozilla/5.0 (Linux; U; Android 3.1; es-es; GT-I9100 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88306[] = "Mozilla/5.0 (Linux; U; Android 3.1; es-es; GT-I9100 Build/IMM76I-RR) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88307[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US; rv:1.8.1.13) Gecko/X Firefox";
const char S88308[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; rv:1.8.1.16) Gecko/X Firefox";
const char S88309[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:13.0) Gecko/X Thunderbird/13.0.1";
const char S88310[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6300 Build/GRJ22) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88311[] = "JUC (Linux; U; 2.2.2; zh-cn; LG-P970; 480*800) UCWEB7.9.3.103/139/800";
const char S88312[] = "JUC (Linux; U; 2.3.4; zh-cn; LG-P970; 480*800) UCWEB7.9.0.94/139/352";
const char S88313[] = "Mozilla/5.0 (Java; U; en-us; gt-s5233w) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S88314[] = "Mozilla/5.0 (Java; U; vi; gt-s5233w) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S88315[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MID7022 Build/CaribbeanSunset_v1.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S88316[] = "Mozilla/5.0 (Linux; U; Android 2.3; es-es; OnePAD 725 Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88317[] = "Mozilla/5.0 (Windows NT 6.0; rv:14.0) Gecko/X Firefox/14.0.1 slurp";
const char S88318[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:14.0) Gecko/X Firefox/14.0.1 slurp";
const char S88319[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88320[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-I9003 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88321[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9003 Build/IMM76L)";
const char S88322[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; U8510 Build/supertoast; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88323[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID SPYDER Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88324[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID8042 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S88325[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; Maxthon/3.0)";
const char S88326[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.04506; Maxthon/3.0)";
const char S88327[] = "LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-gd510) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S88328[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; ST27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S88329[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88330[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHP-SH630E Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88331[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax A68 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88332[] = "Mozilla/5.0 (Linux; Android 4.2.1; A500 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S88333[] = "Mozilla/5.0 (Linux; Android 4.2.2; A500 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S88334[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233a) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S88335[] = "UCWEB/2.0 (Java; U; MIDP-2.0; fa; gt-s5233a) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S88336[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233a) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0";
const char S88337[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; iDx7 Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S88338[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; iDx7 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S88339[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Arc Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88340[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-B5512 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S88341[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S88342[] = "Mozilla/5.0 (Linux; Android 4.0.4; M9000 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S88343[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CYA-52 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88344[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT28h Build/6.1.E.3.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S88345[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT-AS43D2 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S88346[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT-AS43D2 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S88347[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 5 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S88348[] = "OneBrowser/3.1 (Nokia6300/07.21)";
const char S88349[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010D Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88350[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Fly_F350) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S88351[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S88352[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S88353[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S88354[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-E405 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S88355[] = "Mozilla/5.0 (Linux; Android 4.2.2; U30GT 2 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S88356[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-P990 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88357[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9295 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88358[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3370) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S88359[] = "contype chromeframe/27.0.1453.94";
const char S88360[] = "contype chromeframe/27.0.1453.110";
const char S88361[] = "contype chromeframe/27.0.1453.116";
const char S88362[] = "Mozilla/5.0 (Linux; Android 4.1.1; ST10 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S88363[] = "Mozilla/5.0 (Linux; Android 4.0.3; chagall Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S88364[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E15i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S88365[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);";
const char S88366[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S88367[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; N320 Build/LAVALAVA_N320) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S88368[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; ZTE U930 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S88369[] = "Mozilla/5.0 (Linux; Android 4.2.1; ultrafone 701HD Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88370[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; A54) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S88371[] = "Mozilla/5.0 (Linux; Android 4.0.4; A701 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88372[] = "Mozilla/5.0 (Linux; Android 4.0.4; A701 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S88373[] = "Mozilla/5.0 (Linux; Android 4.1.1; Vodafone 875 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88374[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-c6712) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S88375[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; MID1042 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S88376[] = "Mozilla/5.0 (Linux; Android 4.0.4; Grand X In Build/GXI_THUV1.0.0B06) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88377[] = "Mozilla/5.0 (Linux; Android 4.1.1; PLT7223G Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S88378[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88379[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88380[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; MT11i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S88381[] = "Mozilla/5.0 (Linux; Android 4.2.2; SH-06E Build/S6170) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88382[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sr-rs; Telenor_Smart_HD Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88383[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-x330t) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S88384[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-c3222) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S88385[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-c3222) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile, SAMSUNG-GT-C3222/C3222DDKH2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S88386[] = "Mozilla/5.0 (Linux; Android 4.0.4; Thunder S220 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S88387[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; Lenovo A706_ROW Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33482";
const char S88388[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.3; en-in; HTC One V Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S88389[] = "Mozilla/5.0 (Linux; Android 4.0.4; BC1003 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S88390[] = "Mozilla/5.0 (Linux; Android 4.1.1; MI 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S88391[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S88392[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S88393[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SH-09D Build/S7080) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88394[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-P778 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88395[] = "UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Micromax A110Q) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S88396[] = "LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gd510) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S88397[] = "Nokia311/5.0 (07.36) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311";
const char S88398[] = "Mozilla/5.0 (Linux; Android 4.2.2; ADR6425LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S88399[] = "Mozilla/5.0 (Linux; Android 4.3; SGH-I777 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88400[] = "Mozilla/5.0 (Linux; Android 4.2.1; Cynus T5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88401[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; B6000 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88402[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S88403[] = "NokiaC2-01/5.0 (11.20) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311";
const char S88404[] = "MOT-W396/0.1.43 UP.Browser/6.3.0.6.c.27 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S88405[] = "MOT-WX415/00.72 UP.Browser/7.2.7.5.708 (GUI) MMP/2.0";
const char S88406[] = "NokiaE62/3.0 (2.0618.06.32); SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S88407[] = "NokiaE62-1/3.0 (2.0618.06.17); SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1)";
const char S88408[] = "SonyEricssonK810i/R8BA Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S88409[] = "Nokia2700c-2/2.0 (09.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2700c-2/UC Browser7.8.0.95/70/355";
const char S88410[] = "Nokia5233/UC Browser7.0.0.41/50/351";
const char S88411[] = "NokiaE72-1/UC Browser7.7.1.88/28/444";
const char S88412[] = "nokiae72-1/UC Browser7.7.1.88/69/444  3gpp-gba";
const char S88413[] = "KDDI-CA3A UP.Browser/6.2.0.13.2 (GUI) MMP/2.0";
const char S88414[] = "TIANYU-KTOUCH/H699/Screen-176X220";
const char S88415[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.7 (KHTML, like Gecko) RockMelt/0.8.36.116 Chrome/7.0.517.44 Safari/534.7";
const char S88416[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.10 (KHTML, like Gecko) RockMelt/0.8.40.147 Chrome/8.0.552.231 Safari/534.10";
const char S88417[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.3 (KHTML, like Gecko) RockMelt/0.8.34.820 Chrome/6.0.472.63 Safari/534.3";
const char S88418[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Nexian NX-A890 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S88419[] = "Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Nexian NX-A890 Build/ERE27)";
const char S88420[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Magic/Dream Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88421[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Sapphire/Dream Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88422[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Sapphire/dream Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88423[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Dream/Sapphire Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88424[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC Magic/Dream Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88425[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Dream Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88426[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Dream/Sapphire Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88427[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:11.0) Gecko/X Thunderbird/11.0 Lightning/1.3";
const char S88428[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; MZ606 Build/H.6.2-9_CN-37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S88429[] = "nokian72/UC Browser8.0.3.107/69/444";
const char S88430[] = "Opera/8.54 (Windows NT 4.0; U; zh-cn)";
const char S88431[] = "Opera/8.50 (Windows NT 4.0; U; zh-cn)";
const char S88432[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-C N880S Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S88433[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh_TW; ZTE-C N880S Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88434[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-C N880S Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S88435[] = "Mozilla/5.0 (Linux; Android 4.0.3; QW TB-9718 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S88436[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3 UCBrowser8.4.0.159/42/355";
const char S88437[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B208 Safari/7534.48.3 UCBrowser8.4.0.159/42/355";
const char S88438[] = "MQQBrowser/25 Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE-N880E Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88439[] = "iBrowser/2.6 (Nokia5800XpressMusic;SymbianOS/9.1 Series60/3.0)";
const char S88440[] = "nokiac6-00/UC Browser8.0.3.107/69/444  3gpp-gba";
const char S88441[] = "JUC (Linux; U; 4.0.3; zh-cn; ONDA_MID; 800*444) UCWEB1.0.1.113/147/800";
const char S88442[] = "iBrowser/2.7 (NokiaC5-05;SymbianOS/9.1 Series60/3.0)";
const char S88443[] = "Mozilla/5.0 (Linux; Android 4.0.3; ISW13F Build/V69R51I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88444[] = "MQQBrowser/2.6 (E63-1;SymbianOS/9.2 Series60/3.1)";
const char S88445[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Rockmelt/2.0.3.516 Mobile/10A403 Safari/8536.2";
const char S88446[] = "Mozilla/4.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.4) Gecko/X Firefox/3.4.7";
const char S88447[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-D700 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88448[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; SPH-D700 Build/JZO54K)";
const char S88449[] = "HD_mini_T5555 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S88450[] = "Mozilla/5.0 (Linux; Android 4.0.3; Lenovo A750 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88451[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonk770i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S88452[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonk770i) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S88453[] = "Mozilla/5.0 (Linux; Android 4.1.1; A100 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S88454[] = "Mozilla/5.0 (Linux; Android 4.1.2; A100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S88455[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ADR6425LVW Build/IML74K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88456[] = "JUC (Linux; U; 2.3.6; zh-cn; A5; 320*480) UCWEB7.9.0.94/139/444";
const char S88457[] = "Lenovo S560/S100 Linux/3.0.13 Android/4.0.3 Release/12.12.2011 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30";
const char S88458[] = "Mozilla/5.0 (Java; U; en-us; nokia6233) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S88459[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Flare (S100) Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88460[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S88461[] = "Mozilla/5.0 (Windows NT 5.1; rv:20.0) Gecko/X Thunderbird/20.0";
const char S88462[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P509 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88463[] = "Mozilla/5.0 (compatible; MSIE 3.0; WinNT3.51; Trident/5.0)";
const char S88464[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XT907 Build/9.8.1Q_27-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S88465[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-66) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S88466[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk800i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S88467[] = "MALATA I8 Linux/3.0.13 Android/4.0.4 Release/07.24.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30";
const char S88468[] = "Mozilla/5.0 (Linux; Android 4.0.4; KYL21 Build/018.0.3a90) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S88469[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F160L Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S88470[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-F160K Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S88471[] = "Mozilla/5.0 (Linux; Android 4.1.2; AT100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S88472[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; GT-P3110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S88473[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S88474[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-E830 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88475[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S88476[] = "SonyEricssonW705/R1GA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw705) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S88477[] = "Nokia7310/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7310) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S88478[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; Karbonn_A1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S88479[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I535 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88480[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-CN; GT-S5830 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S88481[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC Incredible S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S88482[] = "Mozilla/5.0 (Linux; Android 4.0.4; i9500 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S88483[] = "Mozilla/5.0 (Linux; Android 4.0.4; ISW13HT Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88484[] = "Mozilla/5.0 (Linux; Android 4.0.3; PMP5197DULTRA Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S88485[] = "UCWEB/2.0(BlackBerry; U; 3.4.0.59; en-us; 8830/3.4.0.59) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S88486[] = "UCWEB/2.0(BlackBerry; U; 3.4.0.55; en-us; 8830/3.4.0.55) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile";
const char S88487[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; SKnetworksEG950 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88488[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; KM-S330 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88489[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT15i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88490[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E617G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88491[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGP321 Build/10.1.1.A.1.253) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S88492[] = "Mozilla/5.0 (Linux; Android 4.2.1; S5 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88493[] = "JUC (Linux; U; 2.3.5; zh-cn; Micromax_A35; 480*800) UCWEB7.9.0.94/139/444";
const char S88494[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; W30) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S88495[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6500D Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S88496[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua_Star) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S88497[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-LU6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S88498[] = "Mozilla/5.0 (Linux; Android 4.1.2; AT-AS53N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88499[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9001 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88500[] = "Mozilla/5.0 (Linux; Android 4.1.2; H7500+ Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88501[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; LG-KU5400 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88502[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A27) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S88503[] = "Mozilla/5.0 (Linux; Android 4.0.4; NXM805ND_EN Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S88504[] = "Mozilla/5.0 (Linux; Android 4.0.3; IM-A830L Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88505[] = "Mozilla/5.0 (Linux; Android 4.0.3; SKY-A830 Build/LTE) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88506[] = "Mozilla/5.0 (Linux; Android 4.0.4; GSmart GS202+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88507[] = "Mozilla/5.0 (Linux; Android 4.1.1; 9300+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88508[] = "Nokia6300/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6300) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S88509[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S88510[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; ST25i Build/6.0.B.3.188) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S88511[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A660 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88512[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC 802w Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88513[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT25i Build/4.1.B.0.631) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88514[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST27i Build/6.1.1.B.1.10) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S88515[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ultrafone 701HD Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88516[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S88517[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; TAB714 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S88518[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Scroll Essential II Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S88519[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; SlimBrowser/7.00.035)";
const char S88520[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Aqua flash Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S88521[] = "Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 10 Build/1.1.11-1015 X-10:30) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S88522[] = "Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 10 3G Build/1.1.11-1015 X-16:17) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S88523[] = "Mozilla/5.0 (Linux; Android 4.1.1; X507S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88524[] = "Mozilla/5.0 (Linux; Android 4.1.1; X507 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88525[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:19.0) Gecko/X Firefox/19.0";
const char S88526[] = "Mozilla/5.0 (X11; Linux x86_64; rv:19.0) Gecko/X K9Browser/19.0.0";
const char S88527[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S88528[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S88529[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A500) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S88530[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; GT-I8190 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S88531[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300";
const char S88532[] = "Mozilla/5.0 (Linux; Android 4.2.1; ME173X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S88533[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7568 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S88534[] = "Mozilla/5.0 (Linux; Android 4.0.4; Novo7Fire Build/Xtm) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S88535[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; Karbonn A6 Build/MocorDroid4.0.3) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S88536[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC T528d) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88537[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; Karbonn S2 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33756";
const char S88538[] = "Mozilla/5.0 (Linux; Android 4.0.3; F-10D Build/V22R49C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S88539[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cherry w500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S88540[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; Dell_Streak) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S88541[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; A55) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S88542[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; SGH-T839) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S88543[] = "SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; fa; sonyericssonj20i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S88544[] = "Mozilla/5.0 (Series40; NokiaX3-02/07.16; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S88545[] = "Mozilla/5.0 (Series40; NokiaX3-02.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S88546[] = "Mozilla/5.0 (Linux; Android 4.1.1; C1505 Build/11.3.A.0.47) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S88547[] = "MQQBrowser/44 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S88548[] = "MQQBrowser/44 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3";
const char S88549[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A21) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S88550[] = "Mozilla/5.0 (Linux; Android 4.1.2; Andromax U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S88551[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262B Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88552[] = "Mozilla/5.0 (Linux; Android 4.1.2; i-mobile IQ 5.1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S88553[] = "Mozilla/5.0 (Linux; Android 4.1.2; i-mobile IQ 5.1A Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S88554[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; LG-P500 Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S88555[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI-ONE Plus Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S88556[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:23.0) Gecko/X Firefox/23.0 slurp";
const char S88557[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/9A405 Safari/7534.48.3";
const char S88558[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S88559[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1SC Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S88560[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-03D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S88561[] = "Mozilla/5.0 (Linux; Android 4.0.4; HYUNDAI T7S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S88562[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7310 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S88563[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Karbonn_A2+) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S88564[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; A210 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S88565[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A26) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S88566[] = "UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; Micromax A26) U2/1.0.0 UCBrowser/9.3.0.321 Mobile";
const char S88567[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S88568[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 UCBrowser/9.3.0.326 Mobile";
const char S88569[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 UCBrowser/9.3.0.326 Mobile";
const char S88570[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 UCBrowser/9.3.0.326 Mobile";
const char S88571[] = "LG/HB620T/v10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S88572[] = "LG-HB620T/v10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S88573[] = "Vodafone/1.0/LG-HB620T/V10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S88574[] = "Vodafone/1.0/LG-HB620T/V10b Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S88575[] = "SAMSUNG-SGH-E740/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S88576[] = "LGE-LG370/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S88577[] = "LG-A225/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S88578[] = "ZTE-T6/1.0 ACS-NF/3.4 QTV2.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S88579[] = "Mozilla/5.0(Linux;U;Android2.3.3;Zh_cn;SK17i;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355";
const char S88580[] = "JUC(Linux;U;Android2.3.4;Zh_cn;SK17i;320*480;)UCWEB7.8.0.95/139/444";
const char S88581[] = "Nexian NX-G507/MTK Release/10.01.2009 Browser";
const char S88582[] = "DoCoMo/2.0 F03B(c500;TB;W24H16)";
const char S88583[] = "DoCoMo/2.0 P03A(c100;TB;W24H15)";
const char S88584[] = "DoCoMo/2.0 F02C(c500;TB;W24H16)";
const char S88585[] = "DoCoMo/2.0 SH02C(c500;TB;W24H16)";
const char S88586[] = "Mozilla/5.0 (Linux; U; Android UKB 2.0; en-us; SCH-I500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88587[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; PI86100)";
const char S88588[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.4; en-US; rv:1.9.0.16) Gecko/X Firefox/3.0.16 Flock/2.5.6";
const char S88589[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Sensation Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88590[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Sensation Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88591[] = "j2me/UC Browser8.0.3.107/69/444";
const char S88592[] = "maui/UC Browser8.0.3.107/69/444";
const char S88593[] = "maui/UC Browser8.0.3.107/69/352";
const char S88594[] = "j2me/UC Browser8.0.3.107/69/351";
const char S88595[] = "MQQBrowser/2.2 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S88596[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.0.1.27000 (Firestorm-Release; firestorm skin) Safari/533.3";
const char S88597[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s3653w) UCBrowser8.2.0.132/69/352/UCWEB Mobile";
const char S88598[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s3653w) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S88599[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9003 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88600[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; V11 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88601[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ThL V11 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88602[] = "Mozilla/5.0 (Linux; Android 4.0.4; L-05D Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88603[] = "Mozilla/5.0 (Linux; U; Android Final 2.3.9; en-gb; GT-S5830 Build/GINGER->_<-BREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88604[] = "Mozilla/5.0 (Linux; U; Android WP8; en-us; GT-S5830 Build/GRWK74; WP8.0_by_maniaczek1111) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88605[] = "Mozilla/5.0 (Symbian3; U; en-us; NokiaE6-00)/UC Browser8.5.0.183/51/355/UCWEB Mobile";
const char S88606[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.1 (KHTML, like Gecko) Ubuntu/11.10 Chromium/21.0.1180.89 Chrome/21.0.1180.89 Safari/537.1";
const char S88607[] = "Mozilla/5.0 (Linux; Android 4.0.3; F-01D Build/V08R31A) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S88608[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Desire Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88609[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; Micromax P350 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S88610[] = "Videocon_V1570_Maui Wap Browser/1.0; Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S88611[] = "NokiaX3-00/5.0 (11.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax3-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S88612[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.2.16pre) Gecko/X Prism zdesktop/7.2.1";
const char S88613[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88614[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88615[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; GOCLEVER TAB A73 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S88616[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-us; GOCLEVER TAB A73 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S88617[] = "JUC (Linux; U; 2.3.4; zh-cn; LG-P698; 320*480) UCWEB7.9.0.94/139/444";
const char S88618[] = "Mozilla/5.0 (Linux; Android 4.0.4; O+_8.5_Android Build/OplusOplus_A51) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88619[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P970 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S88620[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID7048 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S88621[] = "JUC (Linux; U; 4.0.4; zh-cn; GT-B5330; 240*320) UCWEB7.9.0.94/139/444";
const char S88622[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S88623[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Rockmelt/2.2.0.241 Mobile/10A525 Safari/8536.25";
const char S88624[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7102 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88625[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S88626[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Lenovo S920 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88627[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Lenovo S920_ROW Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88628[] = "Mozilla/5.0 (Linux; Android 4.0.4; Philips W832 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S88629[] = "Mozilla/5.0 (Linux; Android 4.1.1; ONDA MID Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S88630[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P1000 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S88631[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P1000 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S88632[] = "Mozilla/5.0 (Linux; Android 4.0.3; PAP4300 DUO Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88633[] = "Mozilla/5.0 (Linux; Android 4.0.4; MK16i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88634[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5253) U2/1.0.0 UCBrowser/9.0.0.295 U2/1.0.0 Mobile";
const char S88635[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S88636[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A10 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S88637[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I8530 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S88638[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LG-E615 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S88639[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A1+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S88640[] = "Mozilla/5.0 (Linux; Android 4.1.1; Novo 10 Hero QuadCore Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S88641[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88642[] = "Mozilla/5.0 (Linux; Android 4.2.2; MB860 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88643[] = "Mozilla/5.0 (Linux; Android 4.1.1; H7100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88644[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; E11 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88645[] = "Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 4 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88646[] = "Mozilla/5.0 (Linux; Android 4.1.2; i-mobile IQ 5.1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88647[] = "Mozilla/5.0 (Linux; Android 4.1.2; i-mobile IQ 5.1A Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88648[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I727R Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S88649[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Maxthon/4.0.6.2000)";
const char S88650[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; NoVa Pad 70C124 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88651[] = "UCWEB/2.0 (Linux; U; Adr 4.0.1; en-US; AMAZE_S200) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S88652[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I577 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88653[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I577 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88654[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; C1504) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S88655[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; i-mobile IQ 5.1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88656[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; i-mobile IQ 5.1A Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88657[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT22i Build/6.1.1.B.1.54) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S88658[] = "Mozilla/5.0 (Linux; Android 4.0.3; MD_LIFETAB_P9516 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S88659[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S88660[] = "Mozilla/5.0 (Linux; Android 4.1.2; P5_Quad Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S88661[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; U20a) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S88662[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; XT800) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S88663[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT-I5800) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S88664[] = "MQQBrowser/2.6 Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3";
const char S88665[] = "MQQBrowser/2.6 Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3";
const char S88666[] = "MQQBrowser/2.6 Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/7534.48.3";
const char S88667[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9300 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S88668[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; SCH-I605 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S88669[] = "Mozilla/5.0 (Linux; Android 4.0.4; C5170 Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S88670[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT25i Build/4.1.B.0.631) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S88671[] = "Mozilla/5.0 (Linux; Android 4.0.4; A89 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S88672[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E986 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S88673[] = "Mozilla/5.0 (PlayStation Vita 2.60) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S88674[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LG-MS770 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S88675[] = "Mozilla/5.0 (Linux; Android 4.0.3; NS-13T001 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S88676[] = "Mozilla/5.0 (Linux; Android 4.0.4; Philips W832 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88677[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT-AS45IPS Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88678[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax_A110) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S88679[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Sensation XE with Beats Audio Z715e Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88680[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Coast/1.0.1.62563 Mobile/9B206 Safari/7534.48.3";
const char S88681[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Coast/1.0.1.62563 Mobile/9B176 Safari/7534.48.3";
const char S88682[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5660) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S88683[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Coolpad7295 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S88684[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Coolpad 7295+ Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S88685[] = "Mozilla/5.0 (Linux; Android 4.0.4; WT19i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88686[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8262) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S88687[] = "Mozilla/5.0 (Linux; Android 4.1.2; MB860 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88688[] = "Mozilla/5.0 (Linux; Android 4.1.2; iris502 Build/LAVAIRIS502) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88689[] = "SAMSUNG-SGH-ZV40-Vodafone/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S88690[] = "SonyEricssonK608i/R2BA Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S88691[] = "LG-KG195/V01 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1 LG-KG195/V01 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S88692[] = "SAMSUNG-SGH-T539/T539UVHA1 Profile/MIDP-2.0 Configuration/CLDC-1.1 NetFront/3.2";
const char S88693[] = "SAMSUNG-S8000/S8000BOIK6 SHP/VPP/R5 Opera/9.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S88694[] = "SCH-A870 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S88695[] = "DoCoMo/2.0 F02B(c500;TB;W24H16)";
const char S88696[] = "DoCoMo/2.0 N06A3(c500;TB;W24H16)";
const char S88697[] = "SAMSUNG-SCH-W259/W259DDDF2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S88698[] = "KDDI-CA39 UP.Browser/6.2.0.13.1.5 (GUI) MMP/2.0";
const char S88699[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Treo700wx";
const char S88700[] = "DoCoMo/2.0 N05B(c500;TB;W24H16)";
const char S88701[] = "DoCoMo/2.0 F03C(c500;TB;W24H16)";
const char S88702[] = "SonyEricssonU5";
const char S88703[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; ZTE Crescent Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88704[] = "MQQBrowser/2.3 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S88705[] = "JUC(Linux;U;Android2.1-update1;Zh_cn;Micromax A60;240*320;)UCWEB7.8.0.95/140/444";
const char S88706[] = "gt-s5230/UC Browser7.9.0.102/69/352";
const char S88707[] = "nokiac5-00.2/UC Browser7.7.1.88/69/444  3gpp-gba";
const char S88708[] = "nokiac5-00/UC Browser7.7.1.88/69/444";
const char S88709[] = "MacOutlook/14.10.0.X (Intel Mac OS X 10.7.2)";
const char S88710[] = "MacOutlook/14.10.0.X (Intel Mac OS X 10.7.4)";
const char S88711[] = "MacOutlook/14.10.0.X (Intel Mac OS X 10.7.5)";
const char S88712[] = "Mozilla/5.0 (Windows NT 5.0; rv:10.0.1) Gecko/X Firefox/10.0.1 SeaMonkey/2.7.1";
const char S88713[] = "Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU5)/UC Browser8.2.0.132/50/444/UCWEB Mobile";
const char S88714[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-T989 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88715[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-T989 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88716[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-T989 Build/GRI40; CyanogenMod-DARKSIDE) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88717[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;T7575;7.10;7720)";
const char S88718[] = "Mozilla/5.0 (Windows NT 5.0; rv:9.0.1) Gecko/X Firefox/9.0.1,gzip(gfe) (via translate.google.com)";
const char S88719[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; ZTE V860 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88720[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5228)/UC Browser8.2.0.132/50/355/UCWEB Mobile";
const char S88721[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; WX435 Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88722[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; WX435 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88723[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; WX435 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88724[] = "Flipboard/1.9.8 CFNetwork/485.12.7 Darwin/10.4.0";
const char S88725[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:8.0.1) Gecko/X Firefox/8.0.1 SeaMonkey/2.5 Lightning/1.0";
const char S88726[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40EX520; CC/XGA; en) Presto/2.7.61 Version/11.00";
const char S88727[] = "Mozilla/5.0 (Windows; U; WinNT4.0; en-US; rv:1.9.2.28) Gecko/X Firefox/3.6.28";
const char S88728[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaC5-06)/UC Browser8.2.0.132/50/355/UCWEB Mobile";
const char S88729[] = "Dolphin http client/8.7.0(164) (Android)";
const char S88730[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA5320)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S88731[] = "Mozilla/5.0 (Windows; U; Windows NT 4.0; zh-CN) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19.1";
const char S88732[] = "MQQBrowser/2.8 (Nokia5800XpressMusic;SymbianOS/9.1 Series60/3.0)";
const char S88733[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; LG-P705 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S88734[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S88735[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S88736[] = "Mozilla/5.0 (Linux; Android 4.0.3; DEM752HCF Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S88737[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A780L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88738[] = "Opera/9.80 (Windows NT 4.0; U; en) Presto/2.10.289 Version/12.02";
const char S88739[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88740[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88741[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88742[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88743[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; maximus max901 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88744[] = "Mozilla/5.0 (Linux; Android 4.2.2; A500 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S88745[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S88746[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3100 Build/JRO03C; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444";
const char S88747[] = "JUC (Linux; U; 4.1.2; zh-cn; GT-P3100; 600*976) UCWEB7.9.0.94/139/444";
const char S88748[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M930BST Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88749[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M930BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S88750[] = "DoCoMo/2.0 N01E(c500;TB;W24H16)";
const char S88751[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT540) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S88752[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Beeline E600 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88753[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT910 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88754[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT910 Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88755[] = "Mozilla/5.0 (Linux; Android 4.0.3; chagall Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S88756[] = "Mozilla/5.0 (Linux; Android 4.1.2; NookColor Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S88757[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; E6 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88758[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; SK17i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S88759[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A7+ Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S88760[] = "Mozilla/5.0 (Linux; Android 4.1.2; P-02E Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88761[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT1S0 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S88762[] = "Mozilla/5.0 (Linux; Android 4.0.4; Enjoy 71 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S88763[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CRCB 2093 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88764[] = "Mozilla/5.0 (Linux; Android 4.1.1; Novo7 Venus Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S88765[] = "Mozilla/5.0 (Linux; Android 2.3.4; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S88766[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Allview_AX1_Shine Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S88767[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaE72-1";
const char S88768[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; TECNO D5 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/33567";
const char S88769[] = "Mozilla/5.0 (Linux; Android 4.1.1; Starmobile DIAMOND Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88770[] = "Mozilla/5.0 (Linux; Android 4.1.2; Starmobile DIAMOND V3 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88771[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Titanium S5) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S88772[] = "Mozilla/5.0 (Linux; Android 4.0.3; M729 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88773[] = "Mozilla/5.0 (Linux; Android 4.1.2; SOL22 Build/10.2.F.3.43) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88774[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9103 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S88775[] = "Nokia206/2.0 (03.59) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia206) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S88776[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; HTC Glacier Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S88777[] = "Mozilla/5.0 (Android 4.2; rv:19.0) Gecko/X Firefox/19.0";
const char S88778[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5282 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S88779[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI MediaPad Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S88780[] = "Mozilla/5.0 (Linux; Android 4.0.3; MediaPad 7 Lite Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S88781[] = "ZTE-T55 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S88782[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-D710 Build/JDQ39E; LiquidSmooth-Liquid-JB-v2.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88783[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88784[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6312) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S88785[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad7295 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S88786[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 7295+ Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S88787[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Lenovo A820 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S88788[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Lenovo A820t Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S88789[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E987 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88790[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6812 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S88791[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; A15 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S88792[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E15i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S88793[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; LG-LG-Optimus Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88794[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; LG-Optimus/mus Build/VDT-Rom) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88795[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAP4044DUO Build/PrestigioPAP4044DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S88796[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S88797[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; R821T Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S88798[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; R823T Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S88799[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5236) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S88800[] = "Mozilla/5.0 (Linux; Android 4.1.1; Z992 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S88801[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SGH-M919 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S88802[] = "Mozilla/5.0 (Linux; Android 4.3; SCH-I605 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S88803[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; HTC Wildfire Build/FRG83D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S88804[] = "Mozilla/5.0 (Linux; Android 4.0.4; HS-9DTB4 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S88805[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0530, in";
const char S88806[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 UCBrowser/9.2.0.312 Mobile";
const char S88807[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 UCBrowser/9.2.0.312 Mobile";
const char S88808[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A62) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S88809[] = "Mozilla/5.0 (Linux; Android 4.0.4; H2000+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88810[] = "Nokia200/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291";
const char S88811[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire C Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S88812[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033M Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88813[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A810S Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88814[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A810K Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88815[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; R821T Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S88816[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; HUAWEI G610-U00 Build/HuaweiG610-U00) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S88817[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HUAWEI P6-T00 Build/HuaweiP6-T00) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S88818[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Vodafone/1.0/HTC_TouchDual/1.31.172.16";
const char S88819[] = "HTC_TouchDual Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S88820[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Vodafone/1.0/HTC_TouchDual/1.31.163.09";
const char S88821[] = "SonyEricssonK750c/R1CA Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S88822[] = "SonyEricssonW205a/R1CA001 Browser/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S88823[] = "SAMSUNG-SGH-Z140/2.0 SHP/VPP/R5 SMB3.1 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S88824[] = "Nexian NX-G508/MTK Release/10.01.2009 Browser/MAUI profile/No Java";
const char S88825[] = "SonyEricssonT610/R201 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S88826[] = "SAMSUNG-SGH-A736/UXHB1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S88827[] = "LG-KG800 MIC/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S88828[] = "C5589/001.00 ACS-NetFront/3.2";
const char S88829[] = "Samsung-SPHM630 Polaris/6.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S88830[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/SGH-L870/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S88831[] = "Mozilla/5.0 SymbianOS/9.3; U; Series60/3.2 Samsung/SGH-L870-ORANGE/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S88832[] = "ZTE-G-X930-T-Mobile/X930_V1_Z1_EN_B18F103 Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S88833[] = "NokiaC5-00/UC Browser7.7.1.88/28/444";
const char S88834[] = "NokiaC5-00.2/UC Browser7.7.1.88/28/444";
const char S88835[] = "NokiaN97 mini/UC Browser7.9.0.102/50/355/UCWEB";
const char S88836[] = "NokiaN97 mini/UCWEB7.9.0.94/50/800";
const char S88837[] = "Mozilla/5.0 (PlayStation Vita 1.66) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2";
const char S88838[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; ru; rv:1.9.0.11) Gecko/X Firefox/3.0.11 CometBird/3.0.11";
const char S88839[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:10.0) Gecko/X Thunderbird/10.0";
const char S88840[] = "Mozilla/5.0 (Windows NT 5.1; rv:12.0) Gecko/X Firefox/12.0 slurp";
const char S88841[] = "Mozilla/5.0 (Windows NT 5.1; rv:12.0) Gecko/X Firefox/12.0 slurp slurp";
const char S88842[] = "Opera/9.80 (Android 2.2.1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.7.81 Version/11.00";
const char S88843[] = "Opera/9.80 (Android 2.2; Linux; Opera Tablet/ADR-X; U; de) Presto/2.7.81 Version/11.00";
const char S88844[] = "MQQBrowser/25 Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; 5860 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88845[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; th-th; i-mobile i691 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88846[] = "Mozilla/5.0(GM_UserLogon)AppleWebKit/535.1 (KHTML, like Gecko) Chrome/16.0.912.758 Safari/535.1";
const char S88847[] = "Mozilla/5.0 (Windows NT 6.0; rv:11.0) Gecko/X Firefox/11.0 slurp";
const char S88848[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:11.0) Gecko/X Firefox/11.0 slurp";
const char S88849[] = "Mozilla/4.0 (Windows; U; Windows NT 4.1; fr-FR; rv:1.8.1.3) Gecko/X Firefox/2.0.0.3";
const char S88850[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; Tablet-9728B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S88851[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S88852[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S88853[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGT SMA2+ Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88854[] = "Mozilla/5.0 (Linux; Android 4.0.3; De-Sensed HTC Vivid Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88855[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; HKPhone revo Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88856[] = "MQQBrowser/2.5.1 Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3";
const char S88857[] = "MQQBrowser/2.5.1 Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3";
const char S88858[] = "Mozilla/5.0 (Java; U; kau; lg-x330t) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S88859[] = "Mozilla/5.0 (Java; U; en-us; lg-x330t) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S88860[] = "Mozilla/5.0 (Linux; Android 4.0.3; A97i Build/CelkonA97i_Celkon) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88861[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S88862[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88863[] = "Mozilla/5.0 (Linux; Android 4.0.3; DATAM819HD_C Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S88864[] = "Mozilla/5.0 (Linux; Android 4.0.3; DATAM819HD_A Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S88865[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT330 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S88866[] = "OneBrowser/3.1 (Nokia7210Supernova/07.2)";
const char S88867[] = "LG-A340/V10d Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S88868[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E170K Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88869[] = "iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 4.1; en-us; SmartTab2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S88870[] = "MyPhoneQTW88TVDUO/MTKRelease/2012/04/17Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0";
const char S88871[] = "LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-gs290) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S88872[] = "Mozilla/5.0 (Linux; Android 4.0.4; Andi4.5h Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88873[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A25 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S88874[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; 101 CXi Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S88875[] = "Mozilla/5.0 (Linux; Android 4.1.1; AMOI N821 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88876[] = "Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia6303classic) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S88877[] = "Mozilla/5.0 (Linux; Android 4.0.4; HS-10DTB2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S88878[] = "Mozilla/5.0 (Linux; Android 4.0.4; V9S Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S88879[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Elcano Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88880[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TM-MID790 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S88881[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cloudfone Thrill 500g Build/FSV01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88882[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A90S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S88883[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PG86100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S88884[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t300) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S88885[] = "UCWEB/2.0 (Java; U; MIDP-2.0; vi; lg-t300) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S88886[] = "UCWEB/2.0 (Java; U; MIDP-2.0; Pt-BR; samsung-gt-s3650) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S88887[] = "Mozilla/5.0 (Linux; Android 4.1.2; B92 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S88888[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB462 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S88889[] = "YahooMobileMail/1.0 (Android Mail; 1.4.5) (espressorf;samsung;GT-P3100;4.0.4/IMM76D)";
const char S88890[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0800, my";
const char S88891[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-P5100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0100";
const char S88892[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S88893[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S88894[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P1000 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S88895[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile IQ 5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88896[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Holiday Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S88897[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Raider X710e Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S88898[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HTC Desire S Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S88899[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; HTC Desire S Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S88900[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Primo G2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88901[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E971 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88902[] = "Mozilla/5.0 (Linux; Android 4.2.2; Evo V 4G Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88903[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-P500 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S88904[] = "Mozilla/5.0 (Linux; Android 4.1.2; Lenovo A820 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88905[] = "Mozilla/5.0 (Linux; Android 4.0.3; Xtreamer_Mobile_AiKi Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88906[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xtreamer_Mobile_Aiki5s Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88907[] = "Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 101G10 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S88908[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 slurp";
const char S88909[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9080L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88910[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; Ideos) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S88911[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5302 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S88912[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI-ONE C1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88913[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax P275 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S88914[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-05E Build/S1160) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88915[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-05E Build/S3270) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88916[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; Micromax A110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S88917[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S88918[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N8013 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S88919[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; iris502 Build/LAVAIRIS502) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88920[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P700 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88921[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PHICOMM i800 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88922[] = "Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S88923[] = "Mozilla/5.0 (Linux; Android 4.0.3; A100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S88924[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (KFTT, 35, 1.7.0)";
const char S88925[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; KFTT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (KFTT, 35, 1.7.0)";
const char S88926[] = "Mozilla/5.0 (compatible; MSIE 7.0; WinNT3.51; Trident/1.0)";
const char S88927[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; HUAWEI_Y210-0200) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S88928[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; HUAWEI_Y210-0100) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S88929[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; UBISLATE7C+ Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S88930[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S88931[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Karbonn A4  Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S88932[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9220 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S88933[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; Huawei; H883G; HuaweiH883G)";
const char S88934[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Dialog i43(ICS) Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88935[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S88936[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P875 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S88937[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S88938[] = "Mozilla/5.0 (Linux; Android 4.0.4; Andromax U Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S88939[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5830) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S88940[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5312 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S88941[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC One V) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S88942[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC One V) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S88943[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A10 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S88944[] = "NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311";
const char S88945[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N8000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S88946[] = "Mozilla/5.0 (Linux; Android 4.0.4; A5Classic Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88947[] = "Mozilla/5.0 (Linux; Android 4.2.1; GSmart Maya M1 v2 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88948[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Micromax_A110) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S88949[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Celkon A107+ Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S88950[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9508 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S88951[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S88952[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570I) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S88953[] = "BlackBerry8130/4.5.0.169 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/126 UP.Browser/5.0.3.3";
const char S88954[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; E120 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S88955[] = "SAMSUNG-SGH-T219S/T219UVGJ1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S88956[] = "Mozilla/5.0 (compatible; Teleca Q7; Brew 3.1.5; U; en) 240X400 LGE AX840";
const char S88957[] = "MOT-K3/99.31.04R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12153168) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S88958[] = "MOT-K3m/99.51.05AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12173212) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S88959[] = "MOT-K3/99.13.11R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12133142) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S88960[] = "MOT-K3/99.13.14R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12143162) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S88961[] = "MOT-K3/99.41.08R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163225) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S88962[] = "MOT-K3/99.41.03R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163189) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S88963[] = "MOT-K3m/99.41.04AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163189) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S88964[] = "MOT-K3/99.13.11R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; X) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S88965[] = "Nokia6303ci/5.0 (07.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S88966[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) LG-C570 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S88967[] = "SAMSUNG-GT-S5600/1.0 SHP/VPP/R5 NetFront/3.5 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S88968[] = "ZTE-X990/X990_V2_Z12_ESFR_D18F100 Obigo/Q03C";
const char S88969[] = "ZTE-X990/X990_V2_Z5_FR_D13F104  Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C";
const char S88970[] = "ZTE-X990/X990_V1_Z2_ID_BFFC101  Profile/MIDP-1.0 Configuration/CLDC-1.0 Obigo/Q03C";
const char S88971[] = "MOT-ZN4/00.72 UP.Browser/7.2.7.5.597 (GUI) MMP/2.0";
const char S88972[] = "MOT-ZN4/00.72 UP.Browser/7.2.7.5.540 (GUI) MMP/2.0";
const char S88973[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; ZTE-RACER Build/chun) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88974[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; el-gr; ZTE-RACER Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88975[] = "LGE-VX8370/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S88976[] = "DoCoMo/2.0 P08A3(c500;TB;W24H15)";
const char S88977[] = "SAMSUNG-GT-M8800/1.0 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S88978[] = "KDDI-SH3G UP.Browser/6.2_7.2.7.1.K.5.207 (GUI) MMP/2.0";
const char S88979[] = "Mozilla/5.0 (compatible; EtaoSpider/1.0; http://open.etao.com/dev/EtaoSpider)";
const char S88980[] = "360Spider";
const char S88981[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Transformer Prime TF201 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S88982[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UCWEB7.2.2.54/139/352";
const char S88983[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.4.1.0 Chrome/17.0.963.56 Safari/535.11";
const char S88984[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.4.1.0 Chrome/17.0.963.56 Safari/535.11";
const char S88985[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.0.7) Gecko/X Firefox/1.5.0.7 SnapPreviewBot";
const char S88986[] = "Opera/9.80 (Linux i686; Opera Mobi/7457; U; en) Presto/2.5.28 Version/10.1";
const char S88987[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; fr-FR; rv:1.7.10) Gecko/X Thunderbird/1.0.6";
const char S88988[] = "Mozilla/5.0 (Windows NT 6.0; rv:9.2) Gecko/X Firefox/9.2 PaleMoon/9.2";
const char S88989[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; bq Voltaire Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88990[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-gb; ViewPad 10e Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S88991[] = "samsung-gt-s5380k/UC Browser7.9.0.102/69/352";
const char S88992[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:13.0) Gecko/X Firefox/13.0.3.0 IceDragon/13.0.3.0";
const char S88993[] = "Mozilla/5.0 (Linux; Android 4.0.4; U30GT-H Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S88994[] = "Mozilla/5.0 (Linux; Android 4.0.3; M030 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88995[] = "Mozilla/5.0 (Linux; Android 4.0.4; P-06D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S88996[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; el-gr; Turbo-X Ice Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S88997[] = "JUC (Linux; U; 2.3.7; zh-cn; XT615; 480*854) UCWEB7.9.4.145/139/999";
const char S88998[] = "Mozilla/5.0 (Java; U; kau; lg-x330t) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S88999[] = "Mozilla/5.0 (Java; U; en-us; lg-x330t) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S89000[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE90) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile";
const char S89001[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89002[] = "Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-46EX640; CC/USA; en) Presto/2.8.115 Version/11.10";
const char S89003[] = "Mozilla/5.0 (Linux; Android 4.0.3; I_7520 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89004[] = "Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia6303classic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S89005[] = "MicromaxX283/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S89006[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152) Sleipnir/2.5.13";
const char S89007[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729) Sleipnir/2.5.12";
const char S89008[] = "Mozilla/5.0 (Linux; Android 4.0.3; T1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S89009[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; U1 Build/HCL ME Tablet U1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S89010[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAX5-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S89011[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; gt-s5233a) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S89012[] = "UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; gt-s5233a) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S89013[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89014[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; id-id; CROSS A5 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89015[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; X10.mini Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89016[] = "Mozilla/5.0 (Linux; Android 4.2.1; HTC Desire Build/VJ CM10.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89017[] = "Mozilla/5.0 (X11; Linux x86_64; LT22i-o) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S89018[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-L710 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S89019[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; LG-lu6200 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89020[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; LG-LU6200 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89021[] = "Mozilla/5.0 (Linux; U; Android 4.2; en-gb; GT-P7310 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89022[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-P7310 Build/JOP40D)";
const char S89023[] = "Biblia/2.2.8.1 (Linux; U; Android 4.2.1; es-US; GT-P7310 Build/cm_p5wifi-userdebug 4.2.1 JOP40D eng.christian.X.X test-keys)";
const char S89024[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-P7310 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89025[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7310 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89026[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P7310 Build/JDQ39)";
const char S89027[] = "Instagram 3.4.0 Android (15/4.0.3; 160dpi; 800x1232; samsung; GT-P5100; espresso10rf; espresso10; en_GB)";
const char S89028[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST18i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S89029[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/9A405 Safari/7534.48.3";
const char S89030[] = "Mozilla/5.0 (Linux; Android 4.0.4; Fly_IQ442 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S89031[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S89032[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; Desire HD Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S89033[] = "Mozilla/5.0 (Linux; Android 4.0.4; JY-G2 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89034[] = "Mozilla/5.0 (Linux; Android 4.1.2; MB526 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89035[] = "Mozilla/5.0 (Linux; Android 4.0.3; SC-02C Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89036[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S89037[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S89038[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT28h) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S89039[] = "Mozilla/5.0 (Linux; Android 4.1.1; Novo7 Venus Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S89040[] = "Mozilla/5.0 (Linux; Android 4.1.1; H7100 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89041[] = "Mozilla/5.0 (Linux; Android 4.0.4; i9400 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89042[] = "Mozilla/5.0 (Linux; Android 4.0.4; H2000+ Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89043[] = "OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89044[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9103 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89045[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; P300(Funbook) Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S89046[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I747 Build/JROO3C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S89047[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9305 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S89048[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; LG-E510) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S89049[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAP5000TDUO Build/PrestigioPAP5000TDUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89050[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-01E Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S89051[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 10 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);";
const char S89052[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 10 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S89053[] = "Mozilla/5.0 (Linux; Android 4.0.4; KHPHN05ANDA Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89054[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S89055[] = "Mozilla/5.0 (Linux; Android 4.0.4; HYUNDAI T7S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S89056[] = "MicromaxX50/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S89057[] = "MicromaxQ56/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S89058[] = "MicromaxX490/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S89059[] = "LG-LGC440/V100 Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S89060[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P990 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S89061[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GOCLEVER TAB R70 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89062[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; GOCLEVER TAB R70 Build/JRO03H)";
const char S89063[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; C5302 Build/12.0.A.1.211) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S89064[] = "Mozilla/5.0 (Linux; Android 4.1.1; Micromax P362 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89065[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; POV_TAB-PL1015 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89066[] = "Mozilla/5.0 (Linux; Android 4.0.4; MK16i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89067[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Aqua Wonder Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S89068[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC T329t Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89069[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Masstel M25 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89070[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; ST25i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S89071[] = "Mozilla/5.0 (Linux; Android 4.2.2; U30GT 2 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S89072[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S89073[] = "Mozilla/5.0 (Linux; Android 4.2.2; SHV-E330S Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89074[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Orange Zali Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89075[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Karbonn A2+) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S89076[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT-AS45IPS Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89077[] = "Mozilla/5.0 (Linux; Android 4.0.4; i9500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89078[] = "Mozilla/5.0 (Linux; Android 4.1.2; F-02E Build/V17R48A) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89079[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-S5670 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S89080[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5670 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S89081[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M031 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S89082[] = "SonyEricssonJ10i2/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj10i2) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S89083[] = "SonyEricssonJ10i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; fa; sonyericssonj10i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S89084[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A500) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S89085[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A840S Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89086[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S6012 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89087[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5263) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S89088[] = "Mozilla/5.0 (Linux; Android 4.1.1; P774A Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S89089[] = "UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Micromax A116) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S89090[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; HTC Desire X Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S89091[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq5) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S89092[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S89093[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-I9300 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31";
const char S89094[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9300 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S89095[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus S Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89096[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus S Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89097[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-S7272 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89098[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I897 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S89099[] = "Mozilla/5.0 (Linux; Android 4.1.2; SH530U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S89100[] = "Mozilla/5.0 (Linux; Android 4.1.1; ONE TOUCH TAB 7HD Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S89101[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MT25i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S89102[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S89103[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8730 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S89104[] = "SAMSUNG-D900E-Bouygues/D900EAGIE1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S89105[] = "SAMSUNG-D900E/D900EXAID4 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S89106[] = "BlackBerry8703e/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105";
const char S89107[] = "MOT-W377/0.0.32 UP.Browser/6.3.0.6.c.19 (GUI) MMP/2.0";
const char S89108[] = "Samsung-SPHM320 Polaris/5.0 MMP/2.0";
const char S89109[] = "Mozilla/5.0 (PLAYSTATION 3; 2.00)";
const char S89110[] = "Micromax Q1";
const char S89111[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Comodo_Dragon/4.0.1.6 Chrome/4.0.249.78 Safari/532.5";
const char S89112[] = "Opera/9.80 (Windows 95; U; ru) Presto/2.10.229 Version/11.60";
const char S89113[] = "Mozilla/5.0 (Windows; U; WinNT4.0; en-US; rv:1.8.0.12) Gecko/X Firefox/1.5.0.12";
const char S89114[] = "Mozilla/5.0 (Windows; U; Windows NT 4.0; en-US; rv:1.8.0.2) Gecko/X Firefox/1.5.0.2;";
const char S89115[] = "Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 11.11";
const char S89116[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; ru) Opera 11.11 (ru,ru-RU;q=0.9,en;q=0.8)";
const char S89117[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; de) Opera 11.11";
const char S89118[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; cherry-MagnumHD Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89119[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; ja-JP; rv:1.7.2) Gecko/X Netscape/7.2";
const char S89120[] = "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.7.2) Gecko/X Netscape/7.2 (ax)";
const char S89121[] = "Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S89122[] = "Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19";
const char S89123[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19 Comodo_Dragon/18.3.0.0";
const char S89124[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19 Comodo_Dragon/18.3.0.0";
const char S89125[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.2.0 Safari/534.34";
const char S89126[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; SH8128U Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89127[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia603)/UC Browser8.2.0.132/50/355/UCWEB Mobile";
const char S89128[] = "Mozilla/5.0 (S60V5; U; en-in; NokiaC6-01)/UC Browser8.4.0.159/50/352/UCWEB Mobile";
const char S89129[] = "Mozilla/5.0 (Java; U; en-us; nokiac6-01) UCBrowser8.4.0.159/69/352/UCWEB Mobile  3gpp-gba";
const char S89130[] = "Mozilla/5.0 (Windows; U; Windows NT 4.0; zh-CN; rv:1.9.0.8) Gecko/X Firefox/3.0.8 (.NET CLR 3.5.30729)";
const char S89131[] = "HTC_P3700 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S89132[] = "Nokia 5320/UCWEB8.2.0.116/28/800";
const char S89133[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA5320)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S89134[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; ST18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S89135[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S89136[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5";
const char S89137[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46NX723; CC/GBR; en) Presto/2.7.61 Version/11.00";
const char S89138[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40NX720; CC/XGA; en) Presto/2.7.61 Version/11.00";
const char S89139[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46NX720; CC/USA; en) Presto/2.7.61 Version/11.00";
const char S89140[] = "Mozilla/5.0 (Java; U; en-us; sonyericssonu5i) UCBrowser8.2.1.144/69/352/UCWEB Mobile";
const char S89141[] = "LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-in; lg-gd510) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S89142[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MC7403A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89143[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-eg; MC7403A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89144[] = "Mozilla/5.0 (Linux; Android 4.0.3; Hd1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89145[] = "MQQBrowser/2.6 (NokiaE72-1;SymbianOS/9.3 Series60/3.2)";
const char S89146[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89147[] = "Mozilla/5.0 (Linux; Android 4.0.3; Scroll Engage Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S89148[] = "NokiaX3-00/5.0 (08.54) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax3-00) UCBrowser8.4.0.159/70/352/UCWEB Mobile";
const char S89149[] = "Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5310xpressmusic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S89150[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Micromax A57 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89151[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; Artes D821 Build/ICS.g12refM805AST.X) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S89152[] = "Mozilla/5.0 (Linux; Android 4.0.3; PROG160 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89153[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6680) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S89154[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I519 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89155[] = "Nokia308/2.0 (05.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia308) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S89156[] = "Mozilla/5.0 (Linux; Android 4.0.4; ThL W6 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89157[] = "Mozilla/5.0 (Linux; Android 4.0.3; TAB465EUK Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S89158[] = "Mozilla/5.0 (Linux; Android 4.0.4; w30hd(QuadCore) Build/W30hd) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S89159[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; SmartPad870 Build/E8HD-T364-768) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89160[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; SmartPad870 Build/E8HD-T364-1280x768) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89161[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; SH630E Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89162[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; LG-E405 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S89163[] = "Mozilla/5.0 (Linux; Android 4.0.4; SO-05D Build/7.0.D.1.130) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S89164[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; MITO A200 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89165[] = "Mozilla/5.0 (Linux; Android 4.0.4; ISW11SC Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S89166[] = "Mozilla/5.0 (Linux; Android 4.0.3; AN10CG3 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S89167[] = "SonyEricssonU10i/R7BA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonu10i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S89168[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.3m";
const char S89169[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.3.2m";
const char S89170[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T959 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89171[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-T959 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89172[] = "Mozilla/5.0 (Linux; Android 4.1.1; ZTE V970 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89173[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-i9377 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S89174[] = "Mozilla/5.0 (Linux; Android 4.1.1; I9377 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S89175[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 1030TPC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89176[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; LG-P698) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S89177[] = "Mozilla/5.0 (Linux; Android 4.0.4; Brava DM-993 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89178[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; X909 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S89179[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S89180[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Karbonn A2) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S89181[] = "Mozilla/5.0 (Linux; Android 4.0.4; U8836D Build/HuaweiU8836D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89182[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT894 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89183[] = "Lenovo-S720i/S103 Linux/3.4.0 Android/4.1 Release/10.29.2012 Browser/AppleWebKit534.30 Profile/ Configuration/ Mobile Safari/534.30";
const char S89184[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID7052 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89185[] = "Mozilla/5.0 (Linux; Android 4.1.2; Primo-X1 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S89186[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI A199 Build/HuaweiA199) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S89187[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Sensation XL Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89188[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A62 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S89189[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT11i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S89190[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Wildfire S A510e Build/AOKP-jb-m1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89191[] = "Mozilla/5.0 (Linux; Android 4.1.2; ALCATEL ONE TOUCH 8000A Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89192[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9100G Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S89193[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S89194[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LG-P970 Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S89195[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328w Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S89196[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A1+ Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S89197[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC One S Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S89198[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; MI-ONE C1 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S89199[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; P-04D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89200[] = "Mozilla/5.0 (Linux; Android 4.2.1; R815 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89201[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S89202[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI C8825D Build/HuaweiC8825D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S89203[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia500) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile  3gpp-gba";
const char S89204[] = "Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 6 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89205[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S89206[] = "Mozilla/5.0 (Linux; Android 4.2.2; V812 Core4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S89207[] = "Mozilla/5.0 (Linux; Android 4.1.1; Cynus T2 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89208[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-M830 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S89209[] = "Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A30 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89210[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-US; Nexus 7 Build/JWR66V) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S89211[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-US; Nexus 7 Build/JWR66Y) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S89212[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Ctrl_V1 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S89213[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT22i Build/6.1.1.B.1.54) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S89214[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Titanium S1 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S89215[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq50) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S89216[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Maxwell Plus Build/1.0.3 X-14:07) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S89217[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A800S Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S89218[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia701) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S89219[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; HTC Incredible S Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S89220[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HTC Incredible S Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S89221[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I727 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/skyrocket";
const char S89222[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I727 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/skyrocket";
const char S89223[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S89224[] = "Nokia1681c_CMCC/2.0 (06.82) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S89225[] = "Nokia1681c/2.0 (06.82) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S89226[] = "SonyEricssonZ555i/R10CA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S89227[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 LGKT610/V20A; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S89228[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 LGKT610/v10A; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S89229[] = "SAMSUNG-GT-I6220/1.0 SHP/VPP/R5 Jasmine/0.8 NexPlayer/2.12 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S89230[] = "LG-KS360/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S89231[] = "LG-KF900 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S89232[] = "NokiaX6-00/UC Browser7.7.1.88/50/444";
const char S89233[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/5.2.1.1588 Safari/532.4";
const char S89234[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-kr; KU9500 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S89235[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SU950 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S89236[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; P76TI Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89237[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; P76TI Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S89238[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; P76TI Build/GRJ22)";
const char S89239[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.1.9) Gecko/X Thunderbird/3.0.4";
const char S89240[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:6.0.1) Gecko/X Thunderbird/6.0.1";
const char S89241[] = "NokiaX2-01/5.0 (07.11) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-01/UC Browser7.7.1.88/69/444";
const char S89242[] = "MQQBrowser/32 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3";
const char S89243[] = "MQQBrowser/32 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3";
const char S89244[] = "MQQBrowser/32 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S89245[] = "JUC (Linux; U; 2.3.5; zh-cn; ZTE-C_N760; 320*480) UCWEB7.9.3.103/139/32139";
const char S89246[] = "SAMSUNG-GT-B7610/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S89247[] = "Mozilla/5.0 (X11; FreeBSD amd64; rv:11.0) Gecko/X Firefox/11.0";
const char S89248[] = "Mozilla/5.0 (Windows NT 6.2; rv:9.2) Gecko/X Firefox/9.2 PaleMoon/9.2";
const char S89249[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:9.2) Gecko/X Firefox/9.2-x64 PaleMoon/9.2-x64";
const char S89250[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Microsoft; Virtual)";
const char S89251[] = "Mozilla/5.0 (Linux; U; Android 2.2; th-th; i-mobile i692 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89252[] = "Nokia5200/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia5200/UC Browser8.0.3.107/69/352";
const char S89253[] = "Mozilla/5.0 (Linux; Android 4.0.3; htc_jewel Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89254[] = "Mozilla/5.0 (Linux; Android 4.0.4; htc_jewel Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89255[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; hu-hu; xTAB-70i Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89256[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; hu-hu; xTAB-70i Build/ICS.xTAB70i.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89257[] = "Mozilla/5.0 (Linux; U; Android 2.3.5;zh-cn;SHP-SH320T/2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89258[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SHP-SH320T/AD_230_E350_01006) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89259[] = "JUC(Linux;U;Android2.3.6;Zh_cn;GT-I8150;480*800;)UCWEB7.8.0.95/139/355";
const char S89260[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I9000 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89261[] = "iBrowser/Mini2.8 (NokiaC1-01/04.40)";
const char S89262[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Sleipnir/3.7.3";
const char S89263[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4 Sleipnir/3.7.4";
const char S89264[] = "contype chromeframe/22.0.1229.79";
const char S89265[] = "contype chromeframe/22.0.1229.94";
const char S89266[] = "Mozilla/4.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.2) Gecko/X Firefox/3.8.9";
const char S89267[] = "MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S89268[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S89269[] = "Mozilla/5.0 (PLAYSTATION 3 4.15) AppleWebKit/531.22.8 (KHTML, like Gecko)";
const char S89270[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus One Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89271[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus One Build/CM10; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89272[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s3653w) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S89273[] = "Mozilla/5.0 (Linux; Android 4.0.4; SO-02D Build/6.1.F.0.106) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89274[] = "Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Kem 432 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89275[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Explorer A310e Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89276[] = "Instagram 3.2.0 (iPod4,1; iPhone OS 5.1.1; en_US) AppleWebKit/420+";
const char S89277[] = "Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU8i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/352/UCWEB Mobile";
const char S89278[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S6012 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89279[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A25 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89280[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Next8P12 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S89281[] = "Opera/9.80 (Windows NT 4.0) Presto/2.12.388 Version/12.12";
const char S89282[] = "Mozilla/5.0 (Linux; Android 4.2.1; SCH-I605 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89283[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I605 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89284[] = "Mozilla/5.0 (Linux; Android 4.0.4; Skate Pro Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89285[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89286[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT30a Build/7.0.A.3.195) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S89287[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S5301 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89288[] = "Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-60EX645; CC/USA; en) Presto/2.10.250 Version/11.60";
const char S89289[] = "Mozilla/5.0 (Linux; Android 4.0.4; JY-G2 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S89290[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Edison 3G Build/1.1.11-1015 X-15:24) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S89291[] = "Mozilla/5.0 (Linux; Android 4.0.4; IdeaTab_A1107 Build/MR1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S89292[] = "Mozilla/5.0 (Linux; Android 4.1.2; C6503 Build/10.1.A.1.434) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S89293[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F240L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S89294[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21608/28.2957; U; en) Presto/2.8.119 Version/11.10";
const char S89295[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S89296[] = "Dalvik/1.2.0 (Linux; U; Android 2.2; GT-P1000 Build/FROYO) Opera/9.80 (Android; Opera Mini/5.1.21927/28.3445; U; en) Presto/2.8.119 Version/11.10";
const char S89297[] = "Mozilla/5.0 (Linux; U; Android GFIVE-2.3; en-us; GFIVE A68 Build/A68-V1.0-2012.08.08) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89298[] = "JUC (Linux; U; 4.0.4; zh-cn; LT28h; 720*1280) UCWEB7.9.0.94/139/444";
const char S89299[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; TCL S710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89300[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA6760s-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S89301[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; HTC_One_X) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S89302[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-E510 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S89303[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; LG-E510 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S89304[] = "JUC (Linux; U; 2.3.5; zh-cn; A54; 320*480) UCWEB7.9.0.94/139/444";
const char S89305[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 505)";
const char S89306[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A9) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S89307[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; V8503 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89308[] = "Mozilla/5.0 (Linux; Android 4.0.3; A510 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S89309[] = "Mozilla/5.0 (Linux; Android 4.0.4; A210 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S89310[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Aquaris Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89311[] = "Mozilla/5.0 (Linux; Android 4.1.2; B92 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89312[] = "Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire HD Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S89313[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.1 Safari/7536.29.13 Sleipnir/4.2.1";
const char S89314[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Ctrl_V1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S89315[] = "OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89316[] = "Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300L Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S89317[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; LG-E400) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S89318[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; VS920 4G Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89319[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Micromax A50 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S89320[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NOKIA6120c";
const char S89321[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:22.0) Gecko/X Thunderbird/22.0";
const char S89322[] = "NokiaX3-00/5.0 (11.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax3-00) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S89323[] = "Mozilla/5.0 (Linux; Android 4.2.2; Find 5 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89324[] = "Mozilla/5.0 (Linux; Android 4.0.4; M9000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89325[] = "Mozilla/5.0 (Linux; Android 4.0.3; PMP5197DULTRA Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S89326[] = "UCWEB/2.0 (Windows; U; wds 7.10; en-US; SAMSUNG; GT-I8350) U2/1.0.0 UCBrowser/3.0.0.285 U2/1.0.0 Mobile";
const char S89327[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A57) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S89328[] = "Mozilla/5.0 (Linux; Android 4.0.4; Thunder S220 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89329[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Videocon A51 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89330[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9228 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89331[] = "Mozilla/5.0 (Linux; Android 2.3.5; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S89332[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI H868C Build/HuaweiH868C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89333[] = "Mozilla/5.0 (Linux; Android 4.1.1; C1604 Build/11.3.A.2.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89334[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT15i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S89335[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8815 Build/HuaweiU8815) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S89336[] = "Mozilla/5.0 (Linux; Android 4.1.1; M97 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S89337[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F180S Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89338[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F180K Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89339[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A66/S001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89340[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-LS970 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S89341[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HTC 802t Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89342[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; Karbonn A6) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S89343[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9220 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S89344[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9220 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S89345[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-05D Build/9.1.C.0.475) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89346[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-US; GT-P1000 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S89347[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; X909 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S89348[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; F7HD Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89349[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; Karbonn A30 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S89350[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS870 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S89351[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S89352[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8190) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S89353[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE90) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S89354[] = "UCWEB/2.0 (Linux; U; Adr 2.2; en-US; GT-I5510) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S89355[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Sensation XE with Beats Audio Z715e Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89356[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89357[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A62) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S89358[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire C) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S89359[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire C) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S89360[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7000) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S89361[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A9 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S89362[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Titanium S5) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S89363[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-I8550L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89364[] = "Mozilla/5.0 (Linux; Android 4.1.1; Optimus_San_Remo Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S89365[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-01E Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S89366[] = "Mozilla/5.0 (Linux; Android 4.0.4; SH-02E Build/S4100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S89367[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT15i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S89368[] = "Mozilla/5.0 (Linux; Android 4.1.1; chagall Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S89369[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia603) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S89370[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9308 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S89371[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E440g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S89372[] = "Mozilla/5.0 (Linux; Android 4.3; LG-LS970 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S89373[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-79) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S89374[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT18i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S89375[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.5 Mobile Safari/533.1";
const char S89376[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S89377[] = "Mozilla/5.0 (Linux; Android 4.2.2; HUAWEI P6-U06 Build/HuaweiP6-U06) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S89378[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-N7000 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S89379[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S89380[] = "Mozilla/5.0 (Linux; Android 4.1.1; A210 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S89381[] = "Mozilla/5.0 (Linux; Android 4.0.4; N8000_USA_Cricket Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S89382[] = "Mozilla/5.0 (Linux; Android 4.0.4; V8000_USA_Cricket Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S89383[] = "MOT-L72/08.21.09R MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/1.0";
const char S89384[] = "MOT-L9/08.21.09R MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/1.0";
const char S89385[] = "BlackBerry8330/4.3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/126 UP.Browser/5.0.3.3";
const char S89386[] = "SAMSUNG-GT-B3410/B3410DDJC3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S89387[] = "SAMSUNG-SGH-A727/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S89388[] = "alcatel-ot-606a";
const char S89389[] = "SonyEricssonP900/R101 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S89390[] = "Nokia6220/2.0 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S89391[] = "Samsung-SPHA900P AU-MIC-A900P/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S89392[] = "Samsung-SPHA900 AU-MIC-A900/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S89393[] = "Samsung-SPHA900 AU-MIC/2.0 MMP/2.0";
const char S89394[] = "R236/P130D20/Obigo Q03C/MIDP-2.0 Configuration/CLDC-1.1";
const char S89395[] = "NokiaE63/UC Browser7.2.2.51/28/351";
const char S89396[] = "DoCoMo/2.0 F08A3(c500;TB;W24H16)";
const char S89397[] = "NokiaC5-00/UC Browser7.6.1.82/28/352";
const char S89398[] = "NokiaC5-00.2/UC Browser7.6.1.82/28/352";
const char S89399[] = "Vodafone/1.0/SEX1i/R3AA Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/15431; U; DE) UP.Link/6.3.1.20.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S89400[] = "Vodafone/1.0/SEX1i/R3AA Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/15431; U; UK)  Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S89401[] = "Vodafone/1.0/SEX1i/R1AA Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/2141; U; UK)  Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S89402[] = "TIANYU-KTOUCH/X260/Screen-176X220";
const char S89403[] = "Chrome/5";
const char S89404[] = "Mozilla/5.0 (compatible; U; WebPositive/533.4; Haiku) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.55 Safari/533.4";
const char S89405[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X) (compatible; Yeti-Mobile/0.1; +http://help.naver.com/robots/)";
const char S89406[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Iron/5.0.381.0 Chrome/5.0.381 Safari/533.4";
const char S89407[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Iron/5.0.381.0 Chrome/5.0.381 Safari/533.4";
const char S89408[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Iron/5.0.381.0 Chrome/5.0.381 Safari/533.4";
const char S89409[] = "SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonj105i/UC Browser7.9.0.102/69/352";
const char S89410[] = "HTC_Touch2_T3320 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S89411[] = "Nokia500/UC Browser7.9.0.102/50/355/UCWEB";
const char S89412[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; N-06C Build/AX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89413[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.3; N-06C Build/AX)";
const char S89414[] = "JUC(Linux;U;Android2.2.2;Zh_cn;GT-B7510;240*320;)UCWEB7.8.0.95/139/444";
const char S89415[] = "NokiaC2-02/2.0 (06.52) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-02/UC Browser7.7.1.88/69/444";
const char S89416[] = "Mozilla/5.0 (Windows NT 5.1; rv:10.0.2) Gecko/X Yoono/1.8.40";
const char S89417[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:11.0) Gecko/X Firefox/11.0 SeaMonkey/2.8";
const char S89418[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:12.0) Gecko/X Firefox/12.0-x64 PaleMoon/12.0-x64";
const char S89419[] = "Mozilla/5.0 (Windows NT 6.2; rv:12.0) Gecko/X Firefox/12.0 PaleMoon/12.0";
const char S89420[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus S 4G Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89421[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; GT-I5508 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S89422[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; GT-I5508 Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S89423[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; GT-I5508 Build/ERE27) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S89424[] = "Mozilla/5.0 (Linux; U; Linux mips; en-US) AppleWebKit/531.2+ (KHTML, like Gecko) Version/4.0 Safari/531.2+";
const char S89425[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) 1Password/3.5.9/X (like Mobile/8C148 Safari/6533.18.5)";
const char S89426[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) 1Password/3.5.9/X (like Mobile/8C148 Safari/6533.18.5)";
const char S89427[] = "Mozilla/5.0 (S60V3; U; en-in; NOKIAN79)/UC Browser8.4.0.159/28/352/UCWEB Mobile";
const char S89428[] = "Mozilla/5.0 (Java; U; en-us; nokian79-1) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S89429[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; SPV M700; OpVer 19.123.2.733) OrangeBot-Mobile 2008.0 (mobilesearch.support@orange-ftgroup.com)";
const char S89430[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth Pro/6.0.3.2197 Safari/532.4";
const char S89431[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL32CX520; CC/XGA; en) Presto/2.7.61 Version/11.00";
const char S89432[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Stellar Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89433[] = "Mozilla/4.0 (compatible; MSIE 5.0; NetNose-Crawler 2.0; A New Search Experience: http://www.netnose.com)";
const char S89434[] = "CoolPad8013_CMCC_TD/1.0 Linux/2.6.32 Android/2.2 Release/6.30.2011 Browser/AppleWebkit533.1";
const char S89435[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; QW TB-9718 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89436[] = "Mozilla/5.0 (Linux; Android 4.0.4; NookTablet Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S89437[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; U2 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89438[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; U2 Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S89439[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; U2 Build/FRG83) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S89440[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HW-HUAWEI_C8650/C8650V100R001C92B825; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S89441[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8650+ Build/V100R001C92B861) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89442[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; vivo S1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89443[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.6.5/X (like Mobile/8C148 Safari/6533.18.5)";
const char S89444[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.6.5/X (like Mobile/8C148 Safari/6533.18.5)";
const char S89445[] = "JUC (Linux; U; 2.3.4; zh-cn; T703; 320*480) UCWEB7.9.0.94/139/32534";
const char S89446[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s5753e) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S89447[] = "Mozilla/5.0 (Linux; Android 4.0.3; B79 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89448[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.7 (KHTML, like Gecko) Chrome/20.0.1099.0 Safari/536.7 QQBrowser/6.14.15493.201";
const char S89449[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-b7722) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile";
const char S89450[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT894 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89451[] = "Mozilla/5.0 (Linux; Android 4.0.3; Explay_Advance Build/ExplayExplay_Advance) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89452[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7300 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89453[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Desire Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89454[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Desire Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89455[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s5263) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S89456[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; U30GT-W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89457[] = "Mozilla/5.0 (Windows NT 4.0) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12";
const char S89458[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6201 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S89459[] = "Mozilla/5.0 (Linux; Android 4.0.4; UMI X1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89460[] = "Mozilla/5.0 (Linux; Android 4.0.4; myPadP4 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S89461[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A1+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89462[] = "JUC(Linux;U;Android4.0.4;Zh_cn;ST15i;320*480;)UCWEB7.8.0.95/139/444";
const char S89463[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40EX720; CC/XGA; en) Presto/2.7.61 Version/11.00";
const char S89464[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Y1 Build/HCL ME Tablet Y1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89465[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; Micromax P350 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S89466[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; P300(Funbook) Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S89467[] = "OneBrowser/3.1 (SAMSUNG-GT-S5253/S5253DDKJ2)";
const char S89468[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E617G Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S89469[] = "Mozilla/5.0 (Linux; Android 4.1.2; A200 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S89470[] = "YahooMobileMail/1.0 (Android Mail; 1.4.6) (GT-P6800;samsung;GT-P6800;3.2/HTJ85B)";
const char S89471[] = "YahooMobileMail/1.0 (Android Mail; 1.4.6) (p6800;Samsung;GT-P6800;4.1.2/JZO54K)";
const char S89472[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; Sen S3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89473[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia701) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba";
const char S89474[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; Nokia5730S-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S89475[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw580i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S89476[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328d Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S89477[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOLO A700 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89478[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N7005 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89479[] = "Mozilla/5.0 (Linux; Android 4.0.4; U8836D Build/HuaweiU8836D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89480[] = "Lemon B369_Maui Wap Browser";
const char S89481[] = "Lemon B499_Maui Browser";
const char S89482[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S89483[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Rhyme S510b Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89484[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Byond B66 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89485[] = "Mozilla/5.0 (Linux; Android 4.0.4; novo7Legend Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S89486[] = "Mozilla/5.0 (Linux; Android 4.0.4; Legend2 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S89487[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S89488[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; SPH-M930BST Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S89489[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN85) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S89490[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; Turbo-X G400 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89491[] = "Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3120classic) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S89492[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; U20i Build/2.1.1.A.0.6) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S89493[] = "Mozilla/5.0 (Linux; Android 4.1.1; Cynus T2 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89494[] = "Mozilla/5.0 (Linux; Android 4.0.4; P5 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89495[] = "Mozilla/5.0 (Linux; Android 4.0.4; IQ440 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89496[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS GAMEPAD Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S89497[] = "Mozilla/5.0 (Linux; Android 4.1.2; SH530U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89498[] = "Mozilla/5.0 (Linux; U; Android 2.3.7 (chip mod); vi-vn; IM-A840S Build/i_am_chip_pi) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89499[] = "Mozilla/5.0 (Linux; Android 4.1.2; SH837W Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89500[] = "Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 101G10 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S89501[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S89502[] = "Mozilla/5.0 (Linux; Android 4.0.4; C5170 Build/IML77) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S89503[] = "Mozilla/5.0 (Linux; Android 4.0.4; HS-9DTB4 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S89504[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GSmart GS202 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89505[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia500) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile  3gpp-gba";
const char S89506[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI C8825D Build/HuaweiC8825D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31";
const char S89507[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S89508[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; SENCOR ELEMENT 8V2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89509[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; T-Mobile G2 Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S89510[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S89511[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6730c) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S89512[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6730c-1) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S89513[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A810S Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89514[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A810K Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89515[] = "Mozilla/5.0 (Linux; Android 4.2.2; U9200 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89516[] = "UCWEB/2.0 (Java; U; MIDP-2.0; fa; samsung-gt-s5253) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S89517[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89518[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; it-it; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89519[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5570 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S89520[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9508 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S89521[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-B5330 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S89522[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.21 (KHTML, like Gecko) QupZilla/1.4.3 Safari/537.21";
const char S89523[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.21 (KHTML, like Gecko) QupZilla/1.4.3 Safari/537.21";
const char S89524[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6812 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S89525[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; A89 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S89526[] = "Mozilla/5.0 (Linux; Android 4.1.1; SH930W Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89527[] = "Mozilla/5.0 (Linux; Android 4.1.2; MB860 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89528[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq5) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S89529[] = "Mozilla/5.0 (Linux; Android 4.1.2;  i-mobile i-STYLE 7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89530[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LS970 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89531[] = "UCWEB/2.0 (iOS; U; iPh OS 5_0_1; zh-CN; iPh3,1) U2/1.0.0 UCBrowser/9.1.0.287 U2/1.0.0 Mobile";
const char S89532[] = "UCWEB/2.0 (iOS; U; iPh OS 5_0_1; zh-CN; iPh4,1) U2/1.0.0 UCBrowser/9.1.0.287 U2/1.0.0 Mobile";
const char S89533[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; TECNO_N3) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S89534[] = "Mozilla/5.0 (Linux; Android 4.1.2; iris502 Build/LAVAIRIS502) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89535[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; INTEX_CLOUD_X1) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S89536[] = "Mozilla/5.0 (Linux; Android 4.2.1; W8 beyond Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89537[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Aqua flash Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S89538[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-I9003 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S89539[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; HTC Desire SV Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S89540[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SCH-I545 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S89541[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT27i Build/6.1.1.B.1.54) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S89542[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N5120 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S89543[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; IRIS 455 Build/LAVAIRIS455) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S89544[] = "Mozilla/5.0 (Linux; Android 4.0.3; INO_ONE Build/INO_ONEINO_ONE) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S89545[] = "UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-S5570) U2/1.0.0 UCBrowser/9.3.0.321 Mobile";
const char S89546[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S89547[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S89548[] = "Nokia7600/2.0 (04.05) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S89549[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I7110/DJHK7 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S89550[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I7110/XNHL1 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S89551[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I7110/XNHJ8 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S89552[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I7110/XNHL2 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S89553[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I7110/XXHI5 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S89554[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I7110/XXHI3 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S89555[] = "MOT-VE66/R6743_G_74.31.03RPL Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MotoWebKit/417.19 (KHTML, like Gecko) BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/9.0";
const char S89556[] = "LG-GB250/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S89557[] = "Alcatel-OT-660/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S89558[] = "Alcatel-OT-660A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S89559[] = "SAMSUNG-SGH-D600/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S89560[] = "SAMSUNG-SGH-D608/1.0/WAP2.0 Profile/MIDP-2.0 Configuration/CL";
const char S89561[] = "MOT-W220/1.0 Release/6.30.2006 Browser/CMCS1.0 Software/0.280";
const char S89562[] = "LG-KP175 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S89563[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; Videocon-V7500 Build/DONUT) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S89564[] = "NOKIA6700s";
const char S89565[] = "DoCoMo/2.0 SH906i(c100;TB;W24H16)";
const char S89566[] = "DoCoMo/2.0 SH906iTV(c100;TB;W24H16)";
const char S89567[] = "Mac OS X/10.6.8 (10K549); ExchangeWebServices/1.3 (61); Mail/4.5 (1084)";
const char S89568[] = "Mac OS X/10.6.7 (10J869); ExchangeWebServices/1.3 (61); Mail/4.5 (1084)";
const char S89569[] = "Mozilla/5.0 (Linux; U; Android Eclair; en-us;pandigitalc1hr) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S89570[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Lenovo A390e Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S89571[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; LNV-Lenovo_A390e/Lenovo_A390e_S020_X; 320*480; CTC/2.0) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S89572[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; LNV-Lenovo_A390e/Lenovo_A390e_S024_X; 320*480; CTC/2.0) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S89573[] = "Mozilla/5.0 (Windows; U; Windows NT 4.10; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16";
const char S89574[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:11.0) Gecko/X Thunderbird/11.0 Lightning/1.3";
const char S89575[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:2.0.1) Gecko/X Firefox/4.0.1 SeaMonkey/2.1";
const char S89576[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:2.0b11) Gecko/X Firefox/ SeaMonkey/2.1b2";
const char S89577[] = "sonyericssonu5i/UC Browser7.9.0.102/69/352";
const char S89578[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.4; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6.1";
const char S89579[] = "Mozilla/5.0 (X11; U; Linux i686; ru; rv:1.9.2.15pre) Gecko/X Ubuntu/10.10 (maverick) Namoroka/3.6.15pre";
const char S89580[] = "Mozilla/5.0 (Linux; U; Bigdroid_2.0; zh-cn; AM7-A1/FRF85B; 1024*600) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17 ThunderSoft_BigDroid/2.0";
const char S89581[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE75-1)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S89582[] = "JUC (Linux; U; 2.3.4; zh-cn; KPT_A9; 480*800) UCWEB7.9.3.103/139/32503";
const char S89583[] = "JUC (Linux; U; 2.3.6; zh-cn; A9; 480*800) UCWEB7.9.0.94/139/444";
const char S89584[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; TAB264 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89585[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-18)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89586[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S89587[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; heritrix/1.3.0 http://www.cs.washington.edu/research/networking/websys/)";
const char S89588[] = "JUC (Linux; U; 2.3.4; zh-cn; GT-S5838; 320*480) UCWEB7.9.4.145/139/999";
const char S89589[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46EX520; CC/XGA; en) Presto/2.7.61 Version/11.00";
const char S89590[] = "Mozilla/5.0 (Symbian3; U; en-us; Nokia701)/UC Browser8.5.0.183/51/352/UCWEB Mobile";
const char S89591[] = "HTC Streaming Player sprint / 1.0 / htc_kingdom / 2.3.4";
const char S89592[] = "HTC Streaming Player us_cellular_wwe / 1.0 / htc_kingdom / 2.3.4";
const char S89593[] = "HTC Streaming Player boost / 1.0 / htc_kingdom / 4.0.3";
const char S89594[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID SPYDER Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89595[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; 760TPC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89596[] = "Mozilla/5.0 (S60V3; U; en-us; Nokia6220c-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile";
const char S89597[] = "Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-40EX640; CC/USA; en) Presto/2.8.115 Version/11.10";
const char S89598[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; U8180 Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89599[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89600[] = "Mozilla/5.0 (iPad; U; CPU OS 6_0_1 like Mac OS X; en-GB) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M Mobile";
const char S89601[] = "Mozilla/5.0 (iPad; U; CPU OS 6_0_1 like Mac OS X; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M Mobile";
const char S89602[] = "Mozilla/5.0 (Linux; Android 4.0.4; 101F Build/V49R39H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89603[] = "Mozilla/5.0 (Series40; Nokia6303iclassic/09.83; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33";
const char S89604[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; QW TB-9718D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89605[] = "Mozilla/5.0 (Linux; Android 4.1.1; Cube U9GT2 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S89606[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx600) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S89607[] = "Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Safari/535.19 Silk-Accelerated=false";
const char S89608[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; G-TiDE E50 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89609[] = "Swingtel SX3 Linux/3.0.13 Android/4.0.4 Release/11.27.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S89610[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Celkon CT7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89611[] = "Instagram 3.4.0 Android (15/4.0.4; 160dpi; 800x1232; samsung; GT-P6800; GT-P6800; smdk4210; en_US)";
const char S89612[] = "Instagram 3.4.2 Android (17/4.2.1; 213dpi; 800x1205; Samsung; GT-P6800; p6800; smdk4210; en_US)";
const char S89613[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile IQ 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89614[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; LT15i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S89615[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT26ii) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S89616[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LG-MS770 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S89617[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC Desire SV Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S89618[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC Desire S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S89619[] = "Mozilla/5.0 (Linux; Android 4.0.4; i9400 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S89620[] = "Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S89621[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 101 Titanium Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S89622[] = "OneBrowser/3.0 (NokiaX2-00/08.35)";
const char S89623[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; P5) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S89624[] = "UCWEB/2.0 (Linux; U; Adr 2.2; en-US; GT-I5801) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S89625[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; DIGMA iDsD10 3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89626[] = "Mozilla/5.0 (Linux; Android 4.1.2; GL07S Build/HuaweiU9700L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89627[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT270 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S89628[] = "Mozilla/5.0 (Linux; Android 4.0.4; S500 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89629[] = "Mozilla/5.0 (Linux; Android 4.0.4; dL1 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89630[] = "Mozilla/5.0 (Linux; Android 4.1.2; SH530U Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89631[] = "Mozilla/5.0 (Linux; Android 4.1.1; U9200 Build/HuaweiU9200) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89632[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID8042 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444";
const char S89633[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iBall Slide 3G7271 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89634[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-N7000 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S89635[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-x330t) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S89636[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-x330t) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S89637[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaC5-03";
const char S89638[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E270S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89639[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A770K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89640[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S720 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89641[] = "Mozilla/5.0 (Linux; Android 4.0.3; Xtreamer Joyz Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89642[] = "Mozilla/5.0 (Linux; Android 4.1.1; NovoCrystal Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S89643[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337M Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S89644[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Spice Mi-435 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S89645[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Aqua.Active) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S89646[] = "Mozilla/5.0 (Linux; Android 4.0.4; novo7Legend Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S89647[] = "Mozilla/5.0 (Linux; Android 4.0.4; Legend2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89648[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb";
const char S89649[] = "Mozilla/5.0 (Linux; Android 4.2.1; H120 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89650[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZTE Blade III Pro Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89651[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S89652[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S89653[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-T989 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S89654[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; Intex Aqua Superb Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33829";
const char S89655[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9152 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S89656[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT28h Build/6.1.E.3.7) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S89657[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT28h Build/6.1.E.2.68) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S89658[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8552) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S89659[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; SPH-D710 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S89660[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; SPH-D710BST Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S89661[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; L-02E Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89662[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; A21 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S89663[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-F120S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89664[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus S 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89665[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S89666[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-I8552 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S89667[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A800S Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89668[] = "Mozilla/5.0 (Linux; Android 3.2; GT-P6210 Build/HTJ85B) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.60140";
const char S89669[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-04E Build/10.1.1.D.2.31) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89670[] = "Mozilla/5.0 (Linux; Android 4.2.1; ZP810 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89671[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E440g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89672[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT1S0 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S89673[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; LG-E405) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S89674[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; Karbonn A6) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S89675[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; XOLO A700 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S89676[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-c3222) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S89677[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 UCBrowser/9.2.0.312 Mobile";
const char S89678[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 UCBrowser/9.2.0.312 Mobile";
const char S89679[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT925 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S89680[] = "Mozilla/5.0 (Linux; Android 4.1.1; M032 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S89681[] = "Mozilla/5.0 (PlayStation Vita 2.61) AppleWebKit/536.26 (KHTML, like Gecko)";
const char S89682[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; VL-879 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89683[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-CN; Galaxy Nexus Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S89684[] = "Mozilla/5.0 (Linux; Android 4.0.4; FJL21 Build/V39R48C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S89685[] = "Mozilla/5.0 (Linux; Android 4.3; XT926 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S89686[] = "Mozilla/5.0 (Linux; Android 4.0.4; SK17i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S89687[] = "UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; Micromax A110) U2/1.0.0 UCBrowser/9.3.0.321 Mobile";
const char S89688[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-78) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S89689[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S89690[] = "Nokia311/5.0 (07.36) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia311) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S89691[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LG730 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S89692[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One XL Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S89693[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; el-gr; Light Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S89694[] = "LG-CU575/V10d Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S89695[] = "SAMSUNG-SGH-A827/A827UCHC1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S89696[] = "SAMSUNG-SGH-T459A/T459UEIK2 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S89697[] = "SEC-SGHD840/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S89698[] = "NokiaN8-00/UC Browser7.8.0.95/50/355";
const char S89699[] = "nokian8-00/UC Browser7.8.0.95/69/351  3gpp-gba";
const char S89700[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; ViewSonic-V350 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89701[] = "Lenovo-P50/S041 LMP/LML Release/2009.12.10 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S89702[] = "Lenovo-P50/S045 LMP/LML Release/2010.03.08 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S89703[] = "Lenovo-P50-VN/S010 LMP/LML Release/2010.03.03 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S89704[] = "Lenovo-P50-VN/S014 LMP/LML Release/2010.05.06 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S89705[] = "DoCoMo/2.0 F07B(c500;TB;W24H16)";
const char S89706[] = "DoCoMo/2.0 P06B(c500;TB;W24H16)";
const char S89707[] = "NOKIA6120c/UC Browser7.6.1.82/28/352";
const char S89708[] = "Chrome/10";
const char S89709[] = "Mozilla/5.0 (compatible) AppleWebKit/534.14 (KHTML, like Gecko) Chrome/10.0.602.0 Safari/534.14";
const char S89710[] = "Facebook 4100.0 (iPad; iPhone OS 4.3.5; en_US)";
const char S89711[] = "Facebook 4110.0 (iPad; iPhone OS 4.3.3; en_US)";
const char S89712[] = "Mozilla/5.0 (compatible; Konqueror/4.2; Linux) KHTML/4.2.1 (like Gecko) Fedora/4.2.1-4.fc11";
const char S89713[] = "Mozilla/5.0(HW-C5700/C5700C58B107;U;REX/1.0;BREW/3.1.5;Profile/MIDP-2.0 Configuration/CLDC-1.1;240*320;CTC/2.0)ucweb/7.0";
const char S89714[] = "Mozilla/5.0 (HW-C5720/C5720C58B105;U;REX/1.0;BREW/3.1.5;Profile/MIDP-2.0 Configuration/CLDC-1.1;240*320;CTC/2.0) Ucweb/7.0";
const char S89715[] = "Mozilla/5.0 (HW-C5700/C5700C58B401;U;REX/1.0;BREW/3.1.5;Profile/MIDP-2.0 Configuration/CLDC-1.1;240*320;CTC/2.0) ucweb/7.0";
const char S89716[] = "Mozilla/5.0 (HW-C5720/C5720C58B106;U;REX/1.0;BREW/3.1.5;Profile/MIDP-2.0 Configuration/CLDC-1.1;240*320;CTC/2.0) Ucweb/7.0";
const char S89717[] = "Mozilla/5.0(HW-C5710/C5710C58B105;U;REX/1.0;BREW/3.1.5;Profile/MIDP-2.0 Configuration/CLDC-1.1;240*320;CTC/2.0)ucweb/7.0";
const char S89718[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3;) Gecko/X Firefox/2.0.0.17";
const char S89719[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.8.0.10) Gecko/X Thunderbird/1.5.0.10";
const char S89720[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; 008Z Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89721[] = "MQQBrowser/33 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3";
const char S89722[] = "MQQBrowser/33 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S89723[] = "samsung-gt-s5753e/UC Browser8.0.3.107/69/444";
const char S89724[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Iron/7.0.520.1 Chrome/7.0.520.1 Safari/534.7";
const char S89725[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:9.0.1) Gecko/X Firefox/9.0.1 slurp slurp";
const char S89726[] = "Mozilla/5.0 (Windows NT 6.1; rv:9.0.1) Gecko/X Firefox/9.0.1 slurp";
const char S89727[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaE7-00)/UC Browser8.2.0.132/50/355/UCWEB Mobile";
const char S89728[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:11.0) Gecko/X Firefox/11.0.1-x64 PaleMoon/11.0.1-x64";
const char S89729[] = "Mozilla/5.0 (Windows NT 6.2; rv:11.0) Gecko/X Firefox/11.0.1 PaleMoon/11.0.1";
const char S89730[] = "Opera/9.22 (X11; OpenBSD i386; U; en)";
const char S89731[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5 Comodo_Dragon/19.1.0.0";
const char S89732[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5 Comodo_Dragon/19.1.0.0";
const char S89733[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; JOYPAD_C81 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89734[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; JOYPAD C81 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S89735[] = "MicromaxM2/Q03C MAUI-browser";
const char S89736[] = "HTC_Snap_S521 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.2.2.51";
const char S89737[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P698 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S89738[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; A7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89739[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; A7* Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89740[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Karbonn_A18 Build/KarbonnA18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89741[] = "SAMSUNG-GT-I8350/UCWEB2.6.0.213/47/32079";
const char S89742[] = "Mozilla/5.0 (Linux; Android 4.0.3; ZP200 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89743[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonwt13i) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S89744[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonwt13i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S89745[] = "UCWEB/2.0(Java; U; MIDP-2.0; fa; sonyericssonwt13i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S89746[] = "UCWEB/2.0(Java; U; MIDP-2.0; fa; sonyericssonwt13i) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S89747[] = "Mozilla/5.0 (Linux; Android 4.0.3; F-11D Build/V24R40A) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89748[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.6 Mobile Safari/535.19 Silk-Accelerated=true";
const char S89749[] = "JUC (Linux; U; 2.1-update1; zh-cn; GT-I5503; 240*320) UCWEB7.9.0.94/139/355";
const char S89750[] = "Nokia306/2.0 (03.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia306) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S89751[] = "Opera/9.80 (X11; FreeBSD 9.1-RELEASE amd64) Presto/2.12.388 Version/12.12";
const char S89752[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.3a1pre) Gecko/X Minefield/3.7a1pre";
const char S89753[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.3a1pre) Gecko/X Minefield/3.7a1pre";
const char S89754[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; BLU P100 Build/1.0.8388.0486) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89755[] = "JUC (Linux; U; 4.0.4; zh-cn; ST26i; 480*854) UCWEB7.9.0.94/139/444";
const char S89756[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nb-no; KIS II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89757[] = "SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; sonyericssonj20i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S89758[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Desire S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89759[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89760[] = "SAMSUNG-GT-C3262 Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10";
const char S89761[] = "Mozilla/5.0 (Linux; Android 4.1.1; Novo7 Venus Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S89762[] = "SAMSUNG-GT-C3782/C3782XXLE1 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S89763[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC Incredible S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S89764[] = "RssBandit/1.9.0.1002 (.NET CLR 2.0.50727.6400; WinNT 6.2.9200.0; http://www.rssbandit.org)";
const char S89765[] = "SharpReader/0.9.7.0 (.NET CLR 2.0.50727.6400; WinNT 6.2.9200.0)";
const char S89766[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N8010 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0000, gb";
const char S89767[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N8010 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, +0100, gb";
const char S89768[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk770i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S89769[] = "Instagram 1.1.3 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7500; GT-P7500; p3; en_GB)";
const char S89770[] = "Opera/9.50 (Windows NT 6.2; U; en)";
const char S89771[] = "Opera/9.52 (Windows NT 6.2; U; en)";
const char S89772[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; GT-I5503 Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S89773[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; GT-I5503 Build/ERE27) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.276 Mobile";
const char S89774[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile";
const char S89775[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HTC One X Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S89776[] = "OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89777[] = "Mozilla/5.0 (Linux; Android 4.1.1; XOLO A1000 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89778[] = "Mozilla/5.0 (Linux; Android 4.1.1; Cynus T2 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89779[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N8013 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89780[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N8013 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1/n8013";
const char S89781[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT270 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S89782[] = "Mozilla/5.0 (Linux; Android 4.1.1; C1504 Build/11.3.A.0.47) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89783[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:21.0) Gecko/X Firefox/21.0 SeaMonkey/2.18";
const char S89784[] = "JUC (Linux; U; 2.3.6; zh-cn; T730; 320*480) UCWEB7.9.0.94/139/33183";
const char S89785[] = "SonyEricssonG502/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssong502) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S89786[] = "Mozilla/5.0 (Linux; Android 4.0.4; Smartphone Android by SFR STARADDICT II Build/IceCreamSandwich) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S89787[] = "Mozilla/5.0 (Linux; Android 4.0.4; STARADDICT II Plus Build/GXI_ZTEV1.0.0B02) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S89788[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S89789[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-US; GT-B7510 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S89790[] = "Mozilla/5.0 (Linux; Android 4.0.4; IQ441 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89791[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; SonyEricssonU8i) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S89792[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ607 Build/7.7.1-122-4-FLEM-WIFI-LA) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S89793[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOLO A700 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89794[] = "Mozilla/5.0 (Linux; Android 4.0.4; KYL21 Build/019.0.3b00) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89795[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9308 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89796[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-7204 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89797[] = "Mozilla/5.0 (Linux; Android 4.0.4; dL1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89798[] = "Mozilla/5.0 (Linux; Android 4.0.4; ISW11SC Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89799[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Nexus 10 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.2/552,2.0.2.552,2013-05-22 23:05, +0100, gb";
const char S89800[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gm360) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S89801[] = "LG-GM360/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; es-LA; lg-gm360) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S89802[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S89803[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Aqua_flash) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S89804[] = "Mozilla/5.0 (Linux; Android 4.0.3; U8860 Build/HuaweiU8860) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S89805[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E455 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89806[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lemon P101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89807[] = "Mozilla/5.0 (Linux; Android 4.0.3; T-909S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89808[] = "Mozilla/5.0 (Linux; Android 4.0.4; T-909 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S89809[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9001 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89810[] = "Mozilla/5.0 (Linux; Android 4.2.1; JY-G4 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89811[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9502 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89812[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S89813[] = "Mozilla/5.0 (Linux; Android 4.1.1; I9377 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89814[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-i9377 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89815[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I407 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89816[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I9070) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S89817[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9100) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S89818[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; ST15i Build/4.0.2.A.0.84) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S89819[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A89) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S89820[] = "Mozilla/5.0 (Linux; Android 4.0.4; GN800 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89821[] = "Mozilla/5.0 (Linux; Android 4.1.2; SBM203SH Build/S0022) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89822[] = "HS-U930/1.0 Linux/2.6 Android/2.3 Browser/AppleWebKit533.1 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S89823[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-U930 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89824[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Xperia Miro Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89825[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-in; ST26i Build/11.2.A.0.31) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S89826[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-in; ST26i Build/11.2.A.0.21) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S89827[] = "UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; X10i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S89828[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Starmobile KNIGHT Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89829[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Sensation Z710e Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89830[] = "Mozilla/5.0 (Linux; Android 4.1.1; A9 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S89831[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P769 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S89832[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; WT19i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S89833[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IP Mobile";
const char S89834[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; U20i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S89835[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT-I5801) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S89836[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; IdeaTab A2107A-H Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S89837[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A27 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S89838[] = "Mozilla/5.0 (Linux; Android 4.1.2; 201F Build/V35R13E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S89839[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; TECNO_P3) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S89840[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; iris504Q Build/iris504Q) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S89841[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Micromax_A72 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S89842[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; UBISLATE7C+ Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S89843[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; UBISLATE 7C+ Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S89844[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT25i Build/4.1.B.0.631) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S89845[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; SCH-I959 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S89846[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI 1S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S89847[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S89848[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S89849[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S89850[] = "Nokia6300i/2.0 (03.41) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S89851[] = "LG-GB280/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S89852[] = "LG-CU400/V1.0 Obigo/Q04C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S89853[] = "Xda_Stellar; 240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S89854[] = "SEC-SGHP520/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S89855[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HTC Dream Build/EPE54B) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S89856[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HTC Dream Build/Super E) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S89857[] = "Alcatel-OT-806A/1.0 ObigoInternetBrowser/Q03C";
const char S89858[] = "SCH-U410/1.0 NetFront/3.0.22.2.6 (GUI) MMP/2.0";
const char S89859[] = "SAMSUNG-SGH-E590/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S89860[] = "MOT-EX124g Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S89861[] = "SAMSUNG-GT-B7300/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S89862[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.41 Safari/534.7";
const char S89863[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-ru; MTC 916 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S89864[] = "JUC(Linux;U;Android2.1-update1;Zh_cn;GT-I5503;240*320;)UCWEB7.8.0.95/139/444";
const char S89865[] = "Mozilla/5.0 (Windows NT 5.1; rv:5.0) Gecko/X SeaMonkey/2.2";
const char S89866[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:5.0) Gecko/X Firefox/5.0 SeaMonkey/2.2";
const char S89867[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:9.0) Gecko/X Thunderbird/9.0.1 Lightning/1.1.1";
const char S89868[] = "Nexian NX-G733/MTK Release/10.01.2009/Browser/MAUI Profile";
const char S89869[] = "Nexian NX-G733/MTK Release/10.01.2009 Browser/MAUI Profile";
const char S89870[] = "JUC (Linux; U; 2.2.2; zh-cn; ZTE-U_V880; 480*800) UCWEB7.9.4.145/139/800";
const char S89871[] = "samsung-gt-s3370/UC Browser8.0.3.107/69/444";
const char S89872[] = "JUC (Linux; U; 2.3.5; zh-cn; Lenovo_A60; 320*480) UCWEB7.9.3.103/139/32399";
const char S89873[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; MP434 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89874[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;T8788;7.10;7720)";
const char S89875[] = "C5120/001.00 ACS-Netfront/3.2";
const char S89876[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76I; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89877[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89878[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Qt/4.6.0 Safari/532.4";
const char S89879[] = "SAMSUNG-SGH-i780/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S89880[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5 Comodo_Dragon/19.2.0.0";
const char S89881[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5 Comodo_Dragon/19.2.0.0";
const char S89882[] = "Mozilla/5.0 (Windows NT 5.1; rv:14.0) Gecko/X Firefox/14.0.1 slurp";
const char S89883[] = "Mozilla/5.0 (Windows NT 5.1; rv:14.0) Gecko/X Firefox/14.0.1 slurp slurp";
const char S89884[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN86 8MP)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S89885[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; en) Presto/2.10.229 Version/11.62";
const char S89886[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; onetouch T60 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89887[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; LG-P350 Build/GRI40; BeautySense 4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89888[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; Optimus Me Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89889[] = "Mozilla/5.0 (Linux; Android 4.0.3; R800i Build/4.1.H.0.4) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89890[] = "Mozilla/5.0 (Linux; Android 4.0.4; R800i Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89891[] = "Mozilla/5.0 (Linux; Android 4.0.3; PLOYER-MOMO Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S89892[] = "Mozilla/5.0 (Linux; Android 4.0.4; PLOYER-MOMO Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89893[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; PTT1026 Build/ICS.b02ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89894[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; PTT1026 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89895[] = "MQQBrowser/2.8 (Nokia5230;SymbianOS/9.1 Series60/3.0)";
const char S89896[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Incredible S Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89897[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Incredible S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89898[] = "JUC (Linux; U; 2.3.6; zh-cn; Lenovo_A366t; 320*480) UCWEB7.9.3.103/139/32908";
const char S89899[] = "Mozilla/5.0 (Linux; Android 4.0.4; SH-01D Build/S8230) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89900[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.2 Mobile Safari/535.19 Silk-Accelerated=false";
const char S89901[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST3_Blade Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89902[] = "NOKIA-Lumia 510/UCWEB2.6.0.213/47/32079";
const char S89903[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/10.10 Chromium/22.0.1229.79 Chrome/22.0.1229.79 Safari/537.4";
const char S89904[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ALCATEL one touch 986 Build/C986-2SALCN1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89905[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ALCATEL one touch 986+ Build/C986-2SALCN1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89906[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89907[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; HTC One X Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S89908[] = "Mozilla/5.0 (Linux; Android 4.0.4; SmartPad7503G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S89909[] = "Mozilla/5.0 (Linux; Android 4.1.1; POV_TAB-PROTAB26 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S89910[] = "Mozilla/5.0 (Linux; Android 4.0.4; i9500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89911[] = "Mozilla/5.0 (Linux; Android 4.2.1; XT912 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89912[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT912 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89913[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; TM-9747 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89914[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID8048 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S89915[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P970 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S89916[] = "Mozilla/5.0 (Linux; Android 4.2.2; Xperia S Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S89917[] = "Mozilla/5.0 (Linux; Android 4.0.4; SOL21 Build/9.0.F.2.128) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S89918[] = "Opera/9.80 (Windows 98) Presto/2.12.388 Version/12.14";
const char S89919[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; ST15i Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S89920[] = "Mozilla/5.0 (Linux; Android 4.0.4; H9500 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S89921[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; GT-P7100 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89922[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; GT-P7100 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89923[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; GT-P7100 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89924[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; GT-P7100 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.3.1-89 (Android 4.1.2; samsung GT-P7100 Build/cm_p3-userdebug 4.1.2 JZO54K eng..X.X test-keys)";
const char S89925[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P5100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S89926[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P5100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S89927[] = "Instagram 3.4.0 Android (13/3.2; 160dpi; 600x976; samsung; GT-P6200; GT-P6200; smdkc210; en_US)";
const char S89928[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7300 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S89929[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo E1t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89930[] = "SAMSUNG-SCH-I605/100.40102";
const char S89931[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S89932[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S89933[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-P990 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89934[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 80XSK Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S89935[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.7.2) Gecko/X Netscape/7.2 (ax)";
const char S89936[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5660) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S89937[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Wynncom_G58 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89938[] = "Mozilla/5.0 (Linux; Android 4.0.4; Nexus S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89939[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; LAVA_iris_351) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S89940[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S89941[] = "Mozilla/5.0 (Linux; Android 4.1.1; Cink King Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89942[] = "Mozilla/5.0 (Linux; Android 4.0.4; VL-110 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S89943[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V889D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89944[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE-U V889D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89945[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Spice_MI-352) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S89946[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC6435LVW Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S89947[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Raider X710e Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89948[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Holiday Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89949[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I757M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89950[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F180K Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89951[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F180L Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89952[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F180S Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89953[] = "Mozilla/5.0 (Linux; Android 4.1.1; WX04K Build/324.4.0000) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89954[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Nexus S Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S89955[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BLU Touch Book 9.7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S89956[] = "Mozilla/5.0 (Linux; Android 2.3.4; MT11i Build/4.0.2.A.0.62) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S89957[] = "Mozilla/5.0 (Linux; Android 4.1.2; Micromax A115 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89958[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenco TAB-900 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S89959[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S89960[] = "Mozilla/5.0 (Linux; Android 4.0.4; Avea inTouch 2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89961[] = "Mozilla/5.0 (Linux; Android 4.2.2; V972 Core4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S89962[] = "Mozilla/5.0 (Linux; Android 4.1.2; MB860 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S89963[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; PIONEER_P1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S89964[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M500W Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S89965[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I777 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S89966[] = "Mozilla/5.0 (Linux; U; Android 4.2 jellybean # ariel-tbc /UCHIHA; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89967[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Micromax_P362) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S89968[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; Fly IQ237 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89969[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; KFTT Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S89970[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8190) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S89971[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A789 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S89972[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A44 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S89973[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.21 (KHTML, like Gecko) QupZilla/1.4.3 Safari/537.21";
const char S89974[] = "NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-05) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S89975[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST25i Build/6.1.1.B.1.10) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S89976[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033M Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S89977[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Micromax_A88) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S89978[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; S735 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S89979[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LG-MS770 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S89980[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A12 Build/RheaTK) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S89981[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A62) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S89982[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 UCBrowser/2.3.2.300";
const char S89983[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S89984[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2S Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S89985[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E610 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89986[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E612 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S89987[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Lenovo A820 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S89988[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Lenovo A820t Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S89989[] = "Mozilla/5.0 (Linux; Android 4.0.4; MK16i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89990[] = "Mozilla/5.0 (Linux; Android 4.0.4; U8836D Build/HuaweiU8836D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89991[] = "Mozilla/5.0 (Linux; Android 4.2.1; Aqua I-5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S89992[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax A91 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S89993[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ph; MyPhone A848i Duo Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89994[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MyPhone A848g Duo Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S89995[] = "Mozilla/5.0 (Linux; Android 4.2.2; SH-06E Build/S7190) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S89996[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A57) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S89997[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Micromax_A116) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S89998[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHL21 Build/S5170) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S89999[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A65) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S90000[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A4+ Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S90001[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5670) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S90002[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A21 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S90003[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; GT-I9508 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S90004[] = "Mozilla/5.0 (Linux; Android 4.0.4; Connect-3G Build/HCL) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S90005[] = "NokiaC2-05/2.0 (08.69) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-05) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S90006[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-T989 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90007[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-T989 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90008[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One X+ Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S90009[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S90010[] = "Nokia6822/2.0 (4.52) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S90011[] = "SAMSUNG-SGH-I607/UCHA1 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S90012[] = "SAMSUNG-GT-S5600L/1.0 SHP/VPP/R5 Jasmine/0.8 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S90013[] = "SAMSUNG-SGH-Z230/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S90014[] = "SonyEricssonS700i/R3F SEMC-Browser/4.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S90015[] = "SonyEricssonS700i/R3B SEMC-Browser/4.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S90016[] = "SonyEricssonS700i/R3M SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S90017[] = "SAMSUNG-SGH-M110/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S90018[] = "SonyEricssonW810i/R4EA Java/SEMC-Java/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S90019[] = "SonyEricssonW810iv/R4CE Java/SEMC-Java/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S90020[] = "SonyEricssonW810";
const char S90021[] = "NokiaE61-1";
const char S90022[] = "NOKIAE61";
const char S90023[] = "NOKIAE61i";
const char S90024[] = "SAMSUNG-GT-C3300K/1.0 RTK-E/1.0 DF/1.0 Release/22.04.2010 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S90025[] = "DoCoMo/2.0 N03A(c100;TB;W24H16)";
const char S90026[] = "SEC-SGHB520/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S90027[] = "SEC-SGHB520/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S90028[] = "HTC-8900/1.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S90029[] = "NokiaC1-01/2.0 (05.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac1-01/UC Browser7.7.1.88/69/444";
const char S90030[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; HERO200 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90031[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HERO200 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90032[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HERO200 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90033[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65";
const char S90034[] = "MicromaxQ3+";
const char S90035[] = "JUC(Linux;U;Android2.2;Zh_cn;GT-I5510;240*400;)UCWEB7.8.0.95/139/444";
const char S90036[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; de-DE; rv:1.4) Gecko/X Netscape/7.1 (ax)";
const char S90037[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:2.2a1pre) Gecko/X Firefox/4.2a1pre";
const char S90038[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) iCab/4.8 Safari/533.16";
const char S90039[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) iCab/4.8 Safari/533.16";
const char S90040[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) iCab/4.8 Safari/533.16";
const char S90041[] = "Nokia5300/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia5300/UC Browser8.0.3.107/69/444";
const char S90042[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:9.0) Gecko/X Thunderbird/9.0.1 Lightning/1.1.1";
const char S90043[] = "Mozilla/5.0 (Maemo; Linux armv7l; rv:2.0.1) Gecko/X Firefox/4.0.1 Fennec/2.0.1";
const char S90044[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC Desire; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S90045[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC Desire Mefisto; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S90046[] = "opera/9.80 (j2me/midp; opera mini/6.24093/26.958; u; en) presto/2.8.119 version/10.54";
const char S90047[] = "Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko/X Firefox/11.0 slurp";
const char S90048[] = "Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko/X Firefox/11.0 slurp slurp";
const char S90049[] = "Opera/9.52 (Windows NT 6.0; U; Opera/9.52 (X11; Linux x86_64; U); en)";
const char S90050[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LINSAY COSMOS F-10HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90051[] = "JUC(Linux;U;Android2.3.6;Zh_cn;ALCATEL ONE TOUCH 918N;320*480;)UCWEB7.8.0.95/139/444";
const char S90052[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11";
const char S90053[] = "Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia5310xpressmusic) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S90054[] = "Mozilla/5.0 (Linux; Android 4.0.4; P-07D Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90055[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; Micromax A60 Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S90056[] = "garmin-asus-Nuvifone-M10/2.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S90057[] = "garmin-asus-Nuvifone-M10/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S90058[] = "Mozilla/5.0 (Java; U; en-us; sonyericssonwt13i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile";
const char S90059[] = "Mozilla/5.0 (Java; U; fa; sonyericssonwt13i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile";
const char S90060[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; MT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90061[] = "Mozilla/5.0 (Linux; Android 4.0.3; K1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S90062[] = "Mozilla/5.0 (Linux; Android 4.0.4; Teclast P85??(A9D3) Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S90063[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Micromax A87 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90064[] = "Mozilla/5.0 (Linux; Android 4.0.4; U20GT Build/U20GT) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S90065[] = "SAMSUNG-GT-I8350/UCWEB2.7.0.223/47/355";
const char S90066[] = "Mozilla/5.0 (Linux; Android 4.0.4; SH-10D Build/SB010) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90067[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; VS920 4G Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90068[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A888 Duo Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90069[] = "Nokia112/2.0 (03.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia112) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S90070[] = "Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia3120classic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S90071[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.68.1565 Chrome/14.0.835.202 Safari/535.1";
const char S90072[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.68.1565 Chrome/14.0.835.202 Safari/535.1";
const char S90073[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000";
const char S90074[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT910 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S90075[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S90076[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Andi107) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S90077[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; PHCASIATAB9D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90078[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ605 Build/I.7.1-39.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S90079[] = "Mozilla/5.0 (Linux; Android 4.0.4; MK16i Build/4.1.B.0.587) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S90080[] = "Mozilla/5.0 (Linux; Android 4.1.2; Q800 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S90081[] = "WhatsApp/2.8.6939 Android/4.0.4 Device/samsung-GT-P6800";
const char S90082[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; nl-nl; GT-P7100 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90083[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; nl-nl; GT-P7100 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90084[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; nl-nl; GT-P7100 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90085[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S90086[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S90087[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);";
const char S90088[] = "SAMSUNG-GT-C3312 Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10";
const char S90089[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; N860 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S90090[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA6110) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S90091[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; Galaxy Nexus Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S90092[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-CN; Galaxy Nexus Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S90093[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S560 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90094[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s8530) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S90095[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; DIGMA iDsD8 3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90096[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; U8180) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S90097[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT1S0 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S90098[] = "Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 10 Build/1.1.11-1015 X-10:30) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S90099[] = "Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 10 3G Build/1.1.11-1015 X-16:17) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S90100[] = "Mozilla/5.0 (Linux; Android 4.1.2; H7500+ Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90101[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S90102[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI C8825D Build/HuaweiC8825D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S90103[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N8000 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S90104[] = "Mozilla/5.0 (Linux; Android 4.0.4; iBall Slide 3G7271 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90105[] = "Mozilla/5.0 (Linux; Android 4.0.4; iBall Slide 3G 7334 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S90106[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S90107[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; X8) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S90108[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; E15i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S90109[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S90110[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI U9510E Build/HuaweiU9510E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90111[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S90112[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-P705 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S90113[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E455f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90114[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST15i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S90115[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; TECNO N3 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/33567";
const char S90116[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-E975 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90117[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9103 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90118[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; F8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90119[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; my-mm; SGT-X41 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90120[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; Lenovo S890 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/33482";
const char S90121[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LT15i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S90122[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E270K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90123[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S90124[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S90125[] = "Mozilla/5.0 (Linux; Android 4.1.2; Archos 50 Platinum Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90126[] = "Mozilla/5.0 (Linux; Android 4.0.4; A9+ Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90127[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S90128[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A95 Build/CelkonCelkon_A95) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S90129[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; FPT V Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90130[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; Paladin Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90131[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; ZTE U930 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S90132[] = "Tweetbot/28201 CFNetwork/609.1.4 Darwin/13.0.0";
const char S90133[] = "Tweetbot/28300 CFNetwork/609.1.4 Darwin/13.0.0";
const char S90134[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC Explorer A310e) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S90135[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Micromax A116 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S90136[] = "Mozilla/5.0 (Linux; Android 4.0.4; A2 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S90137[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5120 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S90138[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; E3 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S90139[] = "SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw20) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S90140[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.2; en-gb; GT-I9505 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S90141[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZTE Blade G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90142[] = "Mozilla/5.0 (Linux; Android 4.2.2; SM-T315 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S90143[] = "Mozilla/5.0 (Linux; Android 4.2.2; M9 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S90144[] = "Mozilla/5.0 (Linux; Android 4.0.4; KHPHN05ANDA Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S90145[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Aqua Wonder Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S90146[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST15i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S90147[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A8 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S90148[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S90149[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5570 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S90150[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; C1504) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S90151[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; ZP910 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90152[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S90153[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S90154[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8262) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S90155[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-e2202) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S90156[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9508 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S90157[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI MT1-U06 Build/HuaweiMT1-U06) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S90158[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5880D Build/PMP5880D_v1.0.0_X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S90159[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9100) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S90160[] = "Mozilla/5.0 (Linux; Android 4.3; SAMSUNG-SGH-I717 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S90161[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5228) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S90162[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; C1604 Build/11.1.A.0.68) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S90163[] = "Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A30 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S90164[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M500W Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S90165[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M040 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S90166[] = "Mozilla/5.0 (Linux; Android 4.1.1; PMP5780D Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S90167[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L720 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S90168[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S90169[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S90170[] = "MOT-W270/0.0.42 UP.Browser/6.3.0.6.c.23 (GUI) MMP/2.0";
const char S90171[] = "HTC_Touch2_T3320 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S90172[] = "HTC_Touch2_T3330 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S90173[] = "BlackBerry8707/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/245";
const char S90174[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) 320x240; VZW; Motorola-Q9c; Windows Mobile 6.0 Standard";
const char S90175[] = "ZTE-R220/1.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S90176[] = "SAMSUNG-GT-C5180/C5180ZWJL3 SHP/VPP/R5 Dolfin/2.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S90177[] = "SAMSUNG-SGH-T219/T219UVFL7 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S90178[] = "SAMSUNG-SGH-E390/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S90179[] = "LG-LX165/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S90180[] = "DoCoMo/2.0 P903i(c100;TB;W24H12)";
const char S90181[] = "DoCoMo/2.0 F905i(c100;TB;W24H17)";
const char S90182[] = "DoCoMo/2.0 N706i(c100;TB;W24H16)";
const char S90183[] = "DoCoMo/2.0 N706i2(c100;TB;W24H16)";
const char S90184[] = "KDDI-SN3F UP.Browser/6.2_7.2.7.1.K.2.234 (GUI) MMP/2.0";
const char S90185[] = "Nokia5530/UCBrowser7.7.0.81/50/999";
const char S90186[] = "Nokia5530/UC Browser7.7.1.88/50/444";
const char S90187[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; Microsoft; XDeviceEmulator)";
const char S90188[] = "Opera/9.5 (Windows NT 5.1; U; en) SAMSUNG SCH-i920";
const char S90189[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.1.1.0 Chrome/14.0.835.202 Safari/535.1";
const char S90190[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.1.1.0 Chrome/14.0.835.202 Safari/535.1";
const char S90191[] = "Mozilla/5.0 (Linux; U; Android 2.2; tr-tr; HT_PAD1051 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90192[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.8.10793.201";
const char S90193[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727) QQBrowser/6.8.10898.201";
const char S90194[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0) QQBrowser/6.8.10898.201";
const char S90195[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90196[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90197[] = "NOKIAN79/UC Browser7.9.0.102/28/355/UCWEB";
const char S90198[] = "Nokia5228/UC Browser8.0.4.121/50/444/UCWEB";
const char S90199[] = "NokiaC3-00/5.0 (07.20) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac3-00/UC Browser7.6.1.82/69/351";
const char S90200[] = "nokiae63-1/UC Browser7.9.0.102/69/352";
const char S90201[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:10.0) Gecko/X Firefox/10.0 SeaMonkey/2.7";
const char S90202[] = "Mozilla/5.0 (Windows NT 5.1; rv:9.0.1) Gecko/X Firefox/9.0.1 slurp";
const char S90203[] = "Mozilla/5.0 (Windows NT 5.1; rv:9.0.1) Gecko/X Firefox/9.0.1 slurp slurp";
const char S90204[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.1.6) Gecko/X FireDownload/1.2pre Firefox/3.5.6 Wyzo/3.5.6.1 FireTorrent/1.1pre";
const char S90205[] = "nokia701/UC Browser8.0.3.107/69/444  3gpp-gba";
const char S90206[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6.1";
const char S90207[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PG86100 Build/IML74K; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90208[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PG86100 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90209[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Evo 3D GSM Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90210[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-eg; SC3071N Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90211[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SC3071n Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90212[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; PB-S1 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90213[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P690 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90214[] = "sonyericssonwt13i/UC Browser8.0.3.107/69/444";
const char S90215[] = "OperaMini/7.0.5.45389 CFNetwork/548.1.4 Darwin/11.0.0";
const char S90216[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.6) Gecko/X Firefox/0.8";
const char S90217[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; rv:1.7.3) Gecko/X Firefox/0.10.1";
const char S90218[] = "SAMSUNG-GT-B7610/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S90219[] = "Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia6303classic) UCBrowser8.4.0.159/70/444/UCWEB Mobile";
const char S90220[] = "Mozilla/5.0 (Linux; Android 2.3.5; en-ph; ICE+Turbo Build/CloudfoneICE+Turbo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90221[] = "SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonw20) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile";
const char S90222[] = "OneBrowser/3.0 (NokiaX2-02/11.57)";
const char S90223[] = "Instagram 3.1.2 (iPod touch; iPhone OS 5.1.1; en_US) AppleWebKit/420+";
const char S90224[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I535 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SCH-I535, 36, 1.7.1)";
const char S90225[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (GT-I9300, 34, 1.6.0)";
const char S90226[] = "Mozilla/5.0 (Java; U; en-us; sonyericssonu5i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile";
const char S90227[] = "Mozilla/5.0 (Java; U; fa; sonyericssonu5i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile";
const char S90228[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; Allview3SpeedDuo Build/SV1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90229[] = "Nokia202/2.0 (20.28) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia202/UC Browser7.9.0.102/69/352";
const char S90230[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90231[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.2 Mobile Safari/535.19 Silk-Accelerated=true";
const char S90232[] = "Mozilla/5.0 (Linux; Android 4.0.3; FreeTAB 9701 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S90233[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-T769 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90234[] = "JUC (Linux; U; 4.0.3; zh-cn; NOVO7_Tornados; 800*444) UCWEB1.0.1.113/147/800";
const char S90235[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-b5310) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile";
const char S90236[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonk810i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S90237[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonk810i) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S90238[] = "Mozilla/5.0 (compatible; Konqueror/3.5; OpenBSD) KHTML/3.5.9 (like Gecko)";
const char S90239[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6211 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S90240[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; Nexus 7 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S90241[] = "Mozilla/5.0 (Linux; Android 4.0.3; RMD-1035 Build/ICS.g12refM1005RU.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S90242[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90243[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST26i Build/11.0.A.3.28) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S90244[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE T81 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90245[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Incredible S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90246[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Incredible S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90247[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; SamsungI8910) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S90248[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; AD683G Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S90249[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; MT11i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90250[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; MT11i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90251[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90252[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; DM013SH Build/S0006) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90253[] = "P16i; Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S90254[] = "Mozilla/5.0 (Linux; Android 4.2.2; Evo 3D CDMA Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S90255[] = "Mozilla/5.0 (Linux; Android 4.2.2; Evo 3D GSM Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S90256[] = "Instagram 3.2.0 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7500; GT-P7500; p3; en_GB)";
const char S90257[] = "Instagram 3.4.0 Android (8/2.2; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; GT-P1000; en_GB)";
const char S90258[] = "Instagram 3.4.0 Android (10/2.3.3; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; gt-p1000; en_GB)";
const char S90259[] = "Instagram 3.4.0 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P3100; espressorf; espresso; en_US)";
const char S90260[] = "Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Safari/535.19 Silk-Accelerated=false";
const char S90261[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-uk; GOCLEVER NETBOOK R103 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90262[] = "Mozilla/5.0 (Linux; Android 4.1.1; 9300+ Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S90263[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N8000 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S90264[] = "Mozilla/5.0 (Linux; Android 4.1.1; Limo X2+ Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S90265[] = "Mozilla/5.0 (Linux; Android 4.1.9; Limo X2 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S90266[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Maxwell Plus Build/1.0.3 X-14:07) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S90267[] = "Podkicker/1.2.4";
const char S90268[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cherry w500 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90269[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.8.1.16) Gecko/X Firefox/2.0.0.16 Flock/1.2.4";
const char S90270[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; GFIVE A68 Build/A68-V1.0-2012.05.29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90271[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; GFIVE A68 Build/A68-V1.0-2012.03.17) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90272[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Explorer A310e Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90273[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Explorer A310 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90274[] = "JUC (Linux; U; 4.0.3; zh-cn; LG-P705; 480*800) UCWEB7.9.0.94/139/444";
const char S90275[] = "Mozilla/5.0 (Linux; Android 4.0.4; Beeline E600 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90276[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/Sleipnir 2.3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90277[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/Sleipnir 2.3.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90278[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A89) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S90279[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC HD2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S90280[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7108 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90281[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N7108 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90282[] = "Mozilla/5.0 (Linux; Android 4.1.1; 9300+ Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90283[] = "Mozilla/5.0 (Linux; Android 4.1.2; ME865 Build/6.7.2_GC-384) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90284[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST70104-1 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90285[] = "Mozilla/5.0 (Linux; Android 4.0.3; X9017 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90286[] = "Mozilla/5.0 (Linux; Android 4.0.3; X907 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90287[] = "Mozilla/5.0 (Linux; Android 4.0.3; M729 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90288[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.3.2m";
const char S90289[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.3m";
const char S90290[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; TM-9747BT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90291[] = "OneBrowser/3.1 (Nokia6303classic/10.12)";
const char S90292[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAP4500DUO Build/PrestigioPAP4500DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90293[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 70 Titanium Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S90294[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HUAWEI Y 220T Build/HuaweiY220T) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S90295[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; HUAWEI Y 220T Build/HuaweiY220T)";
const char S90296[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-STYLE Q 5A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90297[] = "Mozilla/5.0 (Linux; Android 4.1.1; AMI-TT4 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S90298[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LVW Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S90299[] = "Mozilla/5.0 (Linux; Android 4.0.4; Novo7Fire Build/Xtm) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S90300[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Intex Aqua 3.2) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S90301[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Rezound Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90302[] = "Mozilla/5.0 (Linux; Android 2.3.7; LT26i Build/6.0.A.3.62) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S90303[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-M830 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S90304[] = "Mozilla/5.0 (Linux; Android 4.1.1; Micromax A88 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90305[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT-AS43D2 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90306[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT-AS43D2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90307[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A26 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S90308[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; iris405) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S90309[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT300SE Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S90310[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT300 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S90311[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT915 Build/2_32A_2008) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90312[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-S7572 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S90313[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F260S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90314[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I8150 Build/JDQ39E)";
const char S90315[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I8150 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90316[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; GT-I9502 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S90317[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S90318[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9268 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S90319[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I8160 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90320[] = "Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S90321[] = "Mozilla/5.0 (Linux; Android 4.2.1; Nexus 10 Build/JOP40D) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229 Safari/537.4 pss-webkit-request";
const char S90322[] = "Mozilla/5.0 (Linux; Android 3.2.1; AT100 Build/HTK55D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S90323[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S90324[] = "Mozilla/5.0 (Linux; Android 4.2.1; M6589 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S90325[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S90326[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT-AS45SE Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S90327[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; Mobiistar Touch Lai 502 HD Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90328[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS GAMEPAD Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S90329[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8552 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S90330[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I8552 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S90331[] = "Mozilla/5.0 (Linux; Android 4.2.2; A1-811 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S90332[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire V) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S90333[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire V) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S90334[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S90335[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S90336[] = "Mozilla/5.0 (Linux; Android 4.1.2; PantechP9090 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90337[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; GT-B7510) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S90338[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq7) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S90339[] = "Mozilla/5.0 (Linux; Android 4.1.1; Cynus T2 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S90340[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS GAMEPAD Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S90341[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9100G Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S90342[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S90343[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A1+ Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S90344[] = "SonyEricssonG502/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssong502) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S90345[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8262) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S90346[] = "Mozilla/5.0 (Linux; Android 4.1.2; Spice Mi-515 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S90347[] = "UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 800) U2/1.0.0 UCBrowser/3.1.1.343 U2/1.0.0 Mobile";
const char S90348[] = "AppleCoreMedia/1.0.0.11A465 (iPod touch; U; CPU OS 7_0 like Mac OS X; en_us)";
const char S90349[] = "AppleCoreMedia/1.0.0.11A501 (iPod touch; U; CPU OS 7_0_2 like Mac OS X; en_us)";
const char S90350[] = "Mozilla/5.0 (Linux; Android 4.3; Touchpad Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S90351[] = "Mozilla/5.0 (Linux; Android 4.3; TouchPad Build/JLS36G) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S90352[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I467 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S90353[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320; HTC_P3600/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S90354[] = "SAMSUNG-C5212/C5212DDIK1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S90355[] = "SAMSUNG-GT-C5212i/C5212iJVJI1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S90356[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x240; HP iPAQ hw6500/1.0)";
const char S90357[] = "HP iPAQ hw6500/1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x240)";
const char S90358[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x240; HP iPAQ hw6500)";
const char S90359[] = "MOT-C261/0.1.66 UP.Browser/6.2.3.9.c.9 (GUI) MMP/2.0";
const char S90360[] = "SEC-SGHB308/1.0 Release/2.28.2008 Browser/Openwave6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S90361[] = "SEC-SGHB300/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S90362[] = "SCH-U450-parrot/1.0 AU-MIC/2.0 (GUI) MMP/2.0";
const char S90363[] = "SCH-U450/1.0 AU-MIC/2.0 (GUI) MMP/2.0";
const char S90364[] = "ZTE-T165+ NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 firmware/T165+ T05";
const char S90365[] = "Spice M5570";
const char S90366[] = "POLARIS/6.01 (BREW 3.1.5; U; en-us; LG; POLARIS/6.01/WAP) Sprint LX290S MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S90367[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; in-id; Axioo-VIGO350 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90368[] = "Mozilla/4.0 (MobilePhone PL3800/US/1.0) NetFront/3.4 MMP/2.0";
const char S90369[] = "Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; SH-10B Build/S1110) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S90370[] = "Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; SH-10B Build/S7023) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S90371[] = "Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; SH-10B Build/S9190) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S90372[] = "Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; SH-10B Build/SB080) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S90373[] = "NokiaC7-00/UC Browser7.7.1.88/50/444";
const char S90374[] = "nokiac7-00/UC Browser7.7.1.88/69/444  3gpp-gba";
const char S90375[] = "Nokia7210Supernova/2.0 (06.51) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7210supernova/UC Browser7.9.0.102/69/352";
const char S90376[] = "Mozilla/5.0 (X11; U; Linux armv7l; fr-FR; rv:1.9.2a1pre) Gecko/X Firefox/3.5 Maemo Browser 1.4.1.22 RX-51 N900";
const char S90377[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Sensation 4G Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90378[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MIUI.us Sensation 4G Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90379[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.68.1565 Chrome/14.0.835.202 Safari/535.1";
const char S90380[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/534.30 (KHTML, like Gecko) RockMelt/0.9.62.351 Chrome/12.0.742.122 Safari/534.30";
const char S90381[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.72.615 Chrome/14.0.835.202 Safari/535.1";
const char S90382[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.68.1565 Chrome/14.0.835.202 Safari/535.1";
const char S90383[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.64.347 Chrome/13.0.782.218 Safari/535.1";
const char S90384[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.68.1565 Chrome/14.0.835.202 Safari/535.1";
const char S90385[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT; Avant Browser; Avant Browser; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2)";
const char S90386[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:11.0) Gecko/X Firefox/11.0 SeaMonkey/2.8";
const char S90387[] = "SAMSUNG-SGH-D780/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S90388[] = "HTC_Touch_3G_T3232 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S90389[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:5.0) Gecko/X Firefox/5.0 TenFourFox/7400";
const char S90390[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; zh-cn; MZ606 Build/H.6.2-9_CN-18) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S90391[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; zh-cn; MZ606 Build/H.6.2_CN-18-2) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S90392[] = "GoogleEarth/6.1.0.5001(Macintosh;Mac OS X (10.5.8);en;kml:2.2;client:Free;type:default)";
const char S90393[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:7.0) Gecko/X Firefox/7.0 TenFourFox/G3";
const char S90394[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90395[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Liberty Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90396[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Liberty Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90397[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Liberty Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90398[] = "Mozilla/5.0 (Linux; Android 4.0.3; TAB-PROTAB2XXL(4) Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S90399[] = "Mozilla/5.0 (Linux; Android 4.0.3; TAB-PROTAB2XXL Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S90400[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; GIONEE A9 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90401[] = "PantechP6030/JZUSX BMP/1.0.4 DeviceId/X NetFront/4.1 OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S90402[] = "Opera/9.80 (Windows 95; U; ru) Presto/2.10.229 Version/11.64";
const char S90403[] = "Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90404[] = "iBrowser/2.7 (NokiaX6-00;SymbianOS/9.1 Series60/3.0)";
const char S90405[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; Karbonn_A1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S90406[] = "Mozilla/5.0 (Symbian3; U; en-us; Nokia700)/UC Browser8.5.0.183/51/352/UCWEB Mobile";
const char S90407[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iberry AUXUS AX03G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90408[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; Micromax A50 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90409[] = "Mozilla/5.0 (Linux; Android 4.1.2; folio100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S90410[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN700W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90411[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S90412[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; QQBrowser/7.0.4350.400)";
const char S90413[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; QQBrowser/7.0.4350.400)";
const char S90414[] = "Mozilla/5.0 (Linux; Android 4.0.4; DII PAD Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90415[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8150 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90416[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-U V880 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90417[] = "Mozilla/5.0 (Linux; Android 4.0.4; H9500S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90418[] = "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.3.5 Safari/534.34";
const char S90419[] = "SonyEricssonG502/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssong502) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S90420[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A820L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90421[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90422[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A690 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90423[] = "Mozilla/5.0 (Linux; Android 4.0.3; MT7003 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90424[] = "Mozilla/5.0 (Linux; Android 4.0.3; WEXLER.BOOK_T7008 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S90425[] = "Mozilla/5.0 (Linux; Android 4.0.4; S120 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90426[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-b7722) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S90427[] = "UCWEB/2.0 (Java; U; MIDP-2.0; fa; samsung-gt-b7722i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S90428[] = "Mozilla/5.0 (X11; FreeBSD) AppleWebKit/534.26 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/534.26 Midori/0.4";
const char S90429[] = "MQQBrowser/3.1/Adr (Linux; U; 4.0.4; zh-cn; Lenovo A789 Build/A789_S228_X;480*800)";
const char S90430[] = "MQQBrowser/3.1/Adr (Linux; U; 4.0.4; zh-cn; Lenovo A789 Build/A789_S221_X;480*800)";
const char S90431[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SmartQT10 Build/GB.f02ref.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90432[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HUAWEI Y310-T10 Build/HuaweiY310-T10) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S90433[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A789 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90434[] = "NokiaC2-05/2.0 (08.69) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-05) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S90435[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaE7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S90436[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; TM-9740 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90437[] = "Mozilla/5.0 (Linux; Android 4.1.2; SH530U Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S90438[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ASUS Transformer Pad TF700T Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90439[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L900 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90440[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T889 Build/MIUI By AminLAmo) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90441[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonck15i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S90442[] = "Mozilla/5.0 (Linux; Android 4.0.4; P5 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S90443[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-S7568 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S90444[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P990 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90445[] = "Mozilla/5.0 (Linux; Android 4.0.3; Newman N2 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90446[] = "MQQBrowser/3.7/Adr (Linux; U; 4.1.2; zh-cn; GT-I8262D Build/JZO54K.I8262DZCAMB2;480*800)";
const char S90447[] = "MQQBrowser/3.7/Adr (Linux; U; 4.1.2; zh-cn; GT-I8262D Build/JZO54K.I8262DZCAMC2;480*800)";
const char S90448[] = "MQQBrowser/3.7/Adr (Linux; U; 4.1.2; zh-cn; GT-I8262D Build/JZO54K.I8262DZCAMA9;480*800)";
const char S90449[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90450[] = "Mozilla/5.0 (Linux; Android 4.1.2; B15 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90451[] = "Mozilla/5.0 (Linux; Android 4.0.4; M9000 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90452[] = "Mozilla/5.0 (Linux; Android 4.1.2; AT-AS53N Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90453[] = "Mozilla/5.0 (Linux; Android 4.1.2; Titanium S5 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90454[] = "OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90455[] = "Mozilla/5.0 (Linux; Android 4.2.2; C5502 Build/10.3.1.A.0.244) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90456[] = "MIKI50_6464_11B_24MP_HW (MRE\\2.5.00(800) resolution\\X chipset\\MT6250 touch\\0 tpannel\\0 camera\\1 gsensor\\0 keyboard\\qwerty) RLG_G51TV_V01_X Release/2012.09.08 WAP Browser/MAUI Profile/  Q03C1-2.40 en-US";
const char S90457[] = "MIKI50_6464_11B_24MP_HW (MRE\\2.5.00(800) resolution\\X chipset\\MT6250 touch\\0 tpannel\\0 camera\\1 gsensor\\0 keyboard\\qwerty) RLG_G51TV_V01_X Release/2012.09.19 WAP Browser/MAUI Profile/  Q03C1-2.40 en-US";
const char S90458[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Karbonn A30 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S90459[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; konka W830 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90460[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; tablet Fnac 10 Build/2.1.0 X-18:06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90461[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; AD683G Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S90462[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; Micromax P350 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S90463[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; R800i Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90464[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; P-05D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90465[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9105) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S90466[] = "Mozilla/5.0 (Linux; Android 4.0.4; Philips W536 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90467[] = "Mozilla/5.0 (Linux; Android 4.0.4; SO-01E Build/9.0.G.1.108) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90468[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo K800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90469[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE Grand Era Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90470[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90471[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90472[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E16i) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S90473[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cherry w500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90474[] = "Mozilla/5.0 (Linux; Android 4.1.1; SH930W Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90475[] = "Mozilla/5.0 (Linux; Android 4.1.2; A919i Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90476[] = "Mozilla/5.0 (Linux; Android 4.0.4; A6 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90477[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; GT-I9210T Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90478[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A6 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S90479[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTL22 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90480[] = "Mozilla/5.0 (Linux; Android 4.1.2; F-02E Build/V17R48A) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90481[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LS970 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S90482[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5110 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S90483[] = "Mozilla/5.0 (Linux; Android 2.3.6; SCH-S720C Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S90484[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S90485[] = "Mozilla/5.0 (Linux; Android 4.0.4; B92 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90486[] = "Mozilla/5.0 (Linux; Android 4.0.3; Xtreamer Joyz Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S90487[] = "Mozilla/5.0 (Linux; Android 4.0.4; Novo7Fire Build/Xtm) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S90488[] = "Mozilla/5.0 (Linux; Android 4.2.1; LT25i Build/9.1.A.1.140) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90489[] = "Mozilla/5.0 (Linux; Android 4.2.2; Xperia V Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90490[] = "Mozilla/5.0 (Linux; Android 4.2.2; Amaze 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90491[] = "Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A30 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90492[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Spice Mi-530 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90493[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S90494[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I405 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S90495[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SHT21 Build/S5140) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90496[] = "LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gs500) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S90497[] = "Mozilla/5.0 (Linux; Android 4.0.4; C811 4G Build/C811M020) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90498[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Q800 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S90499[] = "PAD-FMD700P Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S90500[] = "PAD-FMD700HD Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S90501[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; zh-CN; GT-N7100) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile";
const char S90502[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.1; id-id; T753 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90503[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; MITO t300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90504[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P708g Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90505[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P705 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90506[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; AMAZE S200 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S90507[] = "SonyEricssonC510/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonc510) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S90508[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn_A2+ Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S90509[] = "Mozilla/5.0 (Android; Mobile: XT907; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S90510[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S90511[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; iris454 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S90512[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-B5512) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S90513[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A21) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S90514[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC One S Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S90515[] = "Mozilla/5.0 (Linux; Android 4.2.2; M9 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S90516[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; LG-F240 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90517[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk530i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S90518[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9001 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S90519[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-T100K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S90520[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-E405 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S90521[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Karbonn_A2) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S90522[] = "Mozilla/5.0 (Linux; Android 4.0.4; BC1003 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S90523[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E16i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S90524[] = "Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S90525[] = "Mozilla/5.0 (Linux; Android 4.1.1; X507S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S90526[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S90527[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-N7000 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S90528[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; GT-N7000 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S90529[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9300 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.5 Mobile Safari/533.1";
const char S90530[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S90531[] = "SAMSUNG-GT-E3210/1.0 RTK-E/1.0 DF/1.0 Release/12.14.2010 Browser/NetFront4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S90532[] = "MOT-VE538/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.2, Browser/UP.Browser/7.2.7.2 (GUI) MMP/2.0 Push/PO";
const char S90533[] = "MOT-VE538/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.2, Browser/UP.Browser/7.2.7.2, Browser/UP.Browser/7.2.7.2 (GUI) MMP/2.0 Push/PO";
const char S90534[] = "MOT-VE538/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.2, Browser/UP.Browser/7.2.7.2, Browser/UP.Browser/7.2.7.2, Browser/UP.Browser/7.2.7.2 (GUI) MMP/2.0 Push/PO";
const char S90535[] = "MOT-VE538/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.2, Browser/UP.Browser/7.2.7.2, Browser/UP.Browser/7.2.7.2, Browser/UP.Browser/7.2.7.2, Browser/UP.Browser/7.2.7.2 (GUI) MMP/2.0 Push/PO";
const char S90536[] = "MOT-VE538/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.2 (GUI) MMP/2.0 Push/PO";
const char S90537[] = "LG-KP110 Obigo/WAP2.0";
const char S90538[] = "LG/KF755/v1.0";
const char S90539[] = "sonyericssonk530i";
const char S90540[] = "SonyEricssonK530i/R8BB Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S90541[] = "NokiaT7-00_TD/CMCC (SymbianOS/3; Release/028.004; Series60/5.2 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko)NokiaBrowser/7.3.1.37 Mobile Safari/533.4 3gpp-gba";
const char S90542[] = "SEC-SGHD830/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S90543[] = "NexianNX-G821/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP-2.0 configuration/CLDC-1.0";
const char S90544[] = "Nexian-NXG821/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S90545[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Liquid Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90546[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Liquid Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90547[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Liquid Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90548[] = "Nokia N96/UC Browser8.0.4.121/28/444/UCWEB";
const char S90549[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; MOT-XT301/CNACT_X2_01.20.3; 240*320; CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S90550[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; MOT-XT301/CNACT_X2_01.21.5; 240*320; CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S90551[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; MOT-XT301/CNACT_X2_01.51.0; 240*320; CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S90552[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.4; en-US; rv:1.9.0.9) Gecko/X Firefox/3.0.9 Wyzo/3.0.3";
const char S90553[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:6.0) Gecko/X Firefox/6.0 SeaMonkey/2.3.1";
const char S90554[] = "Mozilla/5.0 (X11; Linux i686; rv:6.0) Gecko/X Thunderbird/6.0";
const char S90555[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; 7 Mozart T8698)";
const char S90556[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; 7 Mozart)";
const char S90557[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; 7 Mozart; Orange)";
const char S90558[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; T8697)";
const char S90559[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 7.0) XSpider 7";
const char S90560[] = "HTC_Touch_Pro_T7272 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S90561[] = "Opera/9.01 (X11; OpenBSD i386; U; en)";
const char S90562[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:13.0) Gecko/X Firefox/13.0 SeaMonkey/2.10";
const char S90563[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S90564[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5570 Build/FROYO) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S90565[] = "Opera/9.80 (Android 2.1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00";
const char S90566[] = "Opera/9.80 (Android 2.1-update1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00";
const char S90567[] = "Opera/9.80 (Android 3.1; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S90568[] = "SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonj20i) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S90569[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID7042 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S90570[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, us";
const char S90571[] = "Mozilla/5.0 (S60V5; U; en-in; Nokia701)/UC Browser8.2.0.132/50/444/UCWEB Mobile";
const char S90572[] = "Mozilla/5.0 (Windows NT 6.2; rv:11.0) Gecko/X Firefox/11.0 SeaMonkey/2.8";
const char S90573[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/10.04 Chromium/22.0.1229.79 Chrome/22.0.1229.79 Safari/537.4";
const char S90574[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/10.04 Chromium/22.0.1229.79 Chrome/22.0.1229.79 Safari/537.4";
const char S90575[] = "Nokia110/2.0 (03.04) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia110/UC Browser7.9.0.102/69/352";
const char S90576[] = "Mozilla/5.0 (Linux; Android 4.0.4; U19GT-W Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S90577[] = "SAMSUNG-SGH-I727/100.40004";
const char S90578[] = "SAMSUNG-GT-E2220/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S90579[] = "iBrowser/2.5 (E66-1;SymbianOS/9.2 Series60/3.1)";
const char S90580[] = "iBrowser/2.5 (N79;SymbianOS/9.3 Series60/3.2)";
const char S90581[] = "Mozilla/5.0 (Linux; Android 4.0.4; NEO_N002 Build/NeoNEO_N002) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90582[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; i697 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S90583[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SY-Z19M4 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75 Mobile Safari/535.7";
const char S90584[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T679 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90585[] = "JUC (Linux; U; 2.2.2; zh-cn; ZTE-BLADE; 480*800) UCWEB7.9.0.94/139/355";
const char S90586[] = "Mozilla/5.0 (Linux; Android 4.2.1; SCH-I500 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90587[] = "Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia5310xpressmusic) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S90588[] = "Mozilla/5.0 (Linux; Android 4.0.4; L-02E Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90589[] = "Mozilla/5.0 (Linux; Android 4.0.4; R8111 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90590[] = "Dolphin http client/1.3.1(197) (Android)";
const char S90591[] = "OneBrowser/3.1 (Nokia3110c/07.21)";
const char S90592[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST25i Build/6.1.1.B.1.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S90593[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S90594[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST15i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S90595[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xolo_X500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90596[] = "HS-T92_TD/1.0 Android/2.3 Release/12.10.2011 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S90597[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5333) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S90598[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT1S0 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S90599[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-01E Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S90600[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315";
const char S90601[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S90602[] = "JUC (Linux; U; 4.0.4; zh-cn; LT26ii; 720*1280) UCWEB7.9.0.94/139/444";
const char S90603[] = "OPPO_U701T/1.0 Linux/3.0.13 Android/4.0 Release/10.15.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2";
const char S90604[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A45 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S90605[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A45 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S90606[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25";
const char S90607[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/6.0 Mobile/9A405 Safari/7534.48.3 YaBrowser/2.1.1364.172";
const char S90608[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; Micromax A50 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S90609[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Micromax_A50 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S90610[] = "Mozilla/5.0 (Linux; Android 4.0.4; A700 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S90611[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-N7000 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S90612[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; GT-N7000 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S90613[] = "Mozilla/5.0 (Linux; Android 4.0.4; IQ440 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90614[] = "Mozilla/5.0 (Linux; Android 4.0.4; P5 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90615[] = "Mozilla/5.0 (Linux; Android 4.1.2; P-02E Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90616[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Desire_S) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S90617[] = "Mozilla/5.0 (Linux; Android 4.1.2; MZ601 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S90618[] = "Mozilla/5.0 (Linux; Android 4.1.1; MZ601 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S90619[] = "Mozilla/5.0 (Linux; Android 4.1.2; SH837W Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S90620[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100G Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S90621[] = "OneBrowser/3.1 (LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1)";
const char S90622[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; A73) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S90623[] = "JUC (Linux; U; 4.0.4; zh-cn; ZTE_V955; 480*854) UCWEB7.9.0.94/139/33253";
const char S90624[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-P970 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90625[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-P970 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/p970";
const char S90626[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5300 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S90627[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S90628[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6280) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S90629[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I405 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000";
const char S90630[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaC6-00";
const char S90631[] = "SAMSUNG-SPH-L720/101.40202";
const char S90632[] = "[FBAN/FB4A-AZ;FBAV/2.0;FBDM/{density=1.5,width=800,height=1280};FBLC/en_US;FBCR/;FBPN/com.facebook.katana;FBDV/KFTT;FBSV/4.0.3;]";
const char S90633[] = "[FBAN/FB4A;FBAV/3.1;FBBV/X;FBDM/{density=1.5,width=540,height=960};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/DROID BIONIC;FBSV/4.1.2;]";
const char S90634[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Byond B65 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90635[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDZ Atom Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90636[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; LG-P920 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S90637[] = "SAMSUNG-SGH-I337/101.40202";
const char S90638[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA5700) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S90639[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE T81 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90640[] = "OneBrowser/3.1 (NokiaN8-00/111.040.1511/sw_platform=S60;sw_platform_version=5.3;java_build_version=2.3.24)";
const char S90641[] = "Mozilla/5.0 (Linux; Android 4.0.4; 754TPC Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S90642[] = "Mozilla/5.0 (Linux; Android 4.1.2; Desire HD Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90643[] = "Mozilla/5.0 (Linux; Android 4.2.1; ME301T Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S90644[] = "iDxD4 Linux/3.0.13 Android/4.0.4 Release/12.04.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S90645[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; zh-CN; MI 2) U2/1.0.0 UCBrowser/9.1.1.309 U2/1.0.0 Mobile";
const char S90646[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; W20 Build/Symphony_W20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90647[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; W20 Build/Symphony_W20)";
const char S90648[] = "UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 900) U2/1.0.0 UCBrowser/3.0.0.285 U2/1.0.0 Mobile";
const char S90649[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LGL35G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S90650[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90651[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; TECNO D5 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S90652[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromax) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S90653[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire C) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S90654[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST27i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S90655[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E16i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S90656[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-S7270 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S90657[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; T-Hub2 Build/TVA301TELBG3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90658[] = "MQQBrowser/3.6/Adr (Linux; U; 4.0.4; zh-cn; vivo S7t Build/ALPS.ICS2.TDD.MP.V1.2;480*800)";
const char S90659[] = "MQQBrowser/3.6/Adr (Linux; U; 4.2.1; zh-cn; vivo X1St Build/ALPS.JB2.TDD.MP.V1;720*1280)";
const char S90660[] = "Mozilla/5.0 (Linux; Android 4.1.2; ALCATEL ONE TOUCH 8000D Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90661[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A7* Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S90662[] = "SAMSUNG-GT-S3330/S3330JVMC5 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S90663[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; ru-ru; Fly IQ450 Quattro Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90664[] = "Mozilla/5.0 (Linux; Android 4.1.2; Q800 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S90665[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NS-13T001 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90666[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Ideapad K1 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1/k1";
const char S90667[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARNOVA 90G3 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S90668[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A87) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S90669[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-B5512) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S90670[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (/jRFM94SIl+0rwrScZlAfRpeVd1zX9LVAc31pppjmOQ=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90671[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (CJZKkKysm/LkU+fZlIfebm7a9VNMuopZW8bqizzu1lI=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90672[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (NKlZzYOCS63Iq04D5xmpu6GTWaUlr6F5E/V88R9Z2ro=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90673[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (PSjeKGWdCylYhWGfjYvF0dKDtfIL4BxWi9vri2hRbT0=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2 AlexaToolbar/alxg-3.1";
const char S90674[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (JAMkPgy34jtg5uC/PTfQye1NRFZmn+kq6LJvRXI+l8A=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90675[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (1j9Klv6ZovFXBK/nyrgG/iH+jY/bvSpyuNsuzCAL2kc=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90676[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (H9vuO55J/W/eNCjf7LhP6UHNesJkTSkpq/3U3wuSATg=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90677[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (QDsFFh/GywLoYHpNZElVJa1IXkbKb5uqLgqYi6JsZ0w=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90678[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (HtfMH4ztVEkcy+JAR7uq5dxwuGNyDc+BklZPJOWo44o=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90679[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (GnkUPP2lBJm/9lZYoP7uGAYRR0ms99IG/Q5qufwgfOQ=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90680[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (F87GmG3U5UzGPls4nzXh8E0YdQDCbyzs7f3KnzyYCWY=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90681[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (BiJcJ/NnWWNRCJbpE1IDAUARApQI13bBpXjUKIJ3NX0=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90682[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (H3Md1XIJnTbE+bu2rA68p1zm1RLiEeeTztPpy0c371E=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90683[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (LmSzCLvkVtv8go56pkt0CddiEHAq3YDx2qKsCpola4o=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90684[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (7zeHZopFpbXZh+fsx0qv92Y120A/VprIybj2ogv4BEc=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90685[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (xR36Tq9pb/2KNCkXC3Nu1HBrhQMzwBt+vlts4pxjmLs=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90686[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (q2LVBpKdjbTyqvLqnu3cXX5gjjEx1Jq+0EvSmPd3Vpw=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90687[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (XT28UluKEur58RliQm9ffzA9Ed0PI9ZHkAaWZudRZQc=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90688[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (seO21gpOtp8uTe8EjZClilM5rgzyrlWqV92C7FPS+dA=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90689[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (mEHSren5PNQEk2sVQQOzhnX0SPrYAQ6x+xwMaR4JcZg=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90690[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (rqZhywrLlRmI6qan4QYva0VIv9pNu29JPwgQ3VhVnlU=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90691[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (g3kuz6HvwD1E+pHG1Nwxvc1pwnRrGwY8opYv4mY7fRs=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90692[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (uehqSy5ycXDRBKP+jbwUI1YPVd1yTBKlCXkDiKHDrj4=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90693[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (z+fvGPOyIDB7AlyW1GAPcCji/Jny6v/TzN9NQqOAIiI=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90694[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (nuS3iPfF+4oBZXsWfuw2wYSgxrH/3AxDEPP29Ubp/qM=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90695[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (bTReBGAmyU0p9EZguECS3zebxG/O7ntBasc9XCGRZt4=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90696[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (xokYZOXYGKSLMXnZxQkcjAxr4E3a8+0X3xBe/nw/XKA=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90697[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (eLxYEzaXKSGyVCtDG8d1ouh030x3nVxlEbl8rNaF8D8=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90698[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (xfMcA+UNukSoKxiQo4oK63DE5u5v+y6FWE3ME12jJoM=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90699[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ps12oxi4NLcjkolTgLW+makwfh+qo4+Hyxd7hoj3zhs=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90700[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (t8KTdxr3CCQJs0qCrxcYV8EfKDYQAn9KVgD6PdMFVoo=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90701[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (lDmtjpKAQjTZX57klDy8yWr9AokW7gHOaG78SEH4bIM=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90702[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (nUzOdO5G4Zk1kpPNdPN0jyVHPVu1wLsdp7Vr9j1GedI=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90703[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (qrxHXoiLgqDAfXtTk8py4SJH9t6DzTJXbyXeGN77MgA=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90704[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (6y+XVtkcmUcScnuMY4hWaj5HPWngNx88XntxfGm+3rs=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90705[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (RO9oj+u2gYTRUfZifbLi9HK8Bk0GiQbZWK6TsLivua8=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90706[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (6e5YUf22go55jLeYCrxeLl4VBU4hHaD1vd6RkSY0CUE=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90707[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (jFStrNCKwLCtSNpH0kaAT8gmOTLStyWawS580arwYhU=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90708[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ZmEufAdvOJ6+jpzS29ugr8sbtxJwbNEwTvj+UjrNdn8=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90709[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (UuCyn/MDeKIqEOTue1DYHMGflhX4w4+4cfNzdo9/ADA=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2";
const char S90710[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-B5330 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S90711[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-S7500 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/trebon";
const char S90712[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-S7500 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90713[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-S7572 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S90714[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100G Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S90715[] = "LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gs500) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S90716[] = "Mozilla/5.0 (Linux; Android 4.2.1; Revo_Lead Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S90717[] = "Mozilla/5.0 (Linux; Android 4.0.4; IQ441 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S90718[] = "Mozilla/5.0 (Linux; Android 4.2.2; Evo V 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S90719[] = "Mozilla/5.0 (Linux; Android 4.2.2; ARNOVA 101 G4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S90720[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Spice Mi-435 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S90721[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 7295+ Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S90722[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad7295 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S90723[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC 802t Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S90724[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8552) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S90725[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6312) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S90726[] = "UCWEB/2.0 (Linux; U; Adr 4.2.2; en-US; P2) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S90727[] = "Mozilla/5.0 (Linux; Android 4.3; GT-N8000 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S90728[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9082) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S90729[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari/7534.48.3 Sleipnir/2.4.5m";
const char S90730[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S90731[] = "Mozilla/5.0 (Linux; Android 4.0.3; A101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S90732[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; GT-N7000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S90733[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-N7000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S90734[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830C Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S90735[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S90736[] = "SAMSUNG-SGH-F270/F270DDHG6 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S90737[] = "SAMSUNG-SGH-F270i/F270iDDIA1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S90738[] = "HTC_Leo Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S90739[] = "HTC_HD2_T8585 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S90740[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) PPC; HTC_HD2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S90741[] = "NokiaN92-2/2.0 (5.0748.2.1.4) S60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S90742[] = "NokiaN92-1/2.0 (5.0748.2.0.4) S60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S90743[] = "CDM8975/T05_0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S90744[] = "LG-KP199 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S90745[] = "LG-GT950/V10g Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S90746[] = "KWC-S4000/ UP.Browser/7.2.6.1.794 (GUI) MMP/2.0";
const char S90747[] = "KWC-S4000/10033 UP.Browser/7.2.6.1.644 (GUI) MMP/2.0";
const char S90748[] = "Mozilla/5.0(Linux;U;Android1.6;Zh_cn;GT540;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355";
const char S90749[] = "JUC(Linux;U;Android1.6;Zh_cn;GT540;320*480;)UCWEB7.8.0.95/139/351";
const char S90750[] = "Mozilla/5.0(Linux;U;Android2.3.3;Zh_cn;LG-P500;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355";
const char S90751[] = "JUC(Linux;U;Android2.3.3;Zh_cn;LG-P500;320*480;)UCWEB7.8.0.95/139/444";
const char S90752[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Orbit Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90753[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Orbit Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90754[] = "DoCoMo/2.0 P05B(c500;TB;W24H16)";
const char S90755[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I500 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90756[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SCH-I500 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90757[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SCH-I500 Build/MIUIAndroid) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90758[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.4.0.0 Chrome/13.0.782.220 Safari/535.1";
const char S90759[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.4.0.0 Chrome/13.0.782.220 Safari/535.1";
const char S90760[] = "MQQBrowser/28 (iOS; U; CPU like Mac OS X; zh-cn)";
const char S90761[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.0.3.0 Chrome/17.0.963.38 Safari/535.11";
const char S90762[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.0.3.0 Chrome/17.0.963.38 Safari/535.11";
const char S90763[] = "Mozilla/4.0 (compatible; MSIE 5.0; Mac_PowerPC) Opera 6.0  [en]";
const char S90764[] = "Opera/6.01 (X11; U; nn)";
const char S90765[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.140 Safari/535.19 Comodo_Dragon/18.0.3.0";
const char S90766[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.140 Safari/535.19 Comodo_Dragon/18.0.3.0";
const char S90767[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6.1";
const char S90768[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:0.9.4) Gecko/X Netscape6/6.2";
const char S90769[] = "Opera/9.70 (Linux armv7l ; U; turbotabbee/TSV2.0/1.02Q; fr) Presto/2.2.1";
const char S90770[] = "Opera/9.80 (Android 2.3.7; Linux; Opera Tablet/ADR-X; U; en) Presto/2.8.149 Version/11.10";
const char S90771[] = "Opera/9.80 (Android 2.3.3; Linux; Opera Tablet/ADR-X; U; en-GB) Presto/2.8.149 Version/11.10";
const char S90772[] = "Opera/9.80 (Android 2.3; Linux; Opera Tablet/ADR-X; U; en) Presto/2.8.149 Version/11.10";
const char S90773[] = "Nokia5800 XpressMusic/UCWEB8.3.0.133/50/800";
const char S90774[] = "Nokia5800w XpressMusic/UCWEB8.3.0.133/50/800";
const char S90775[] = "Mozilla/5.0 (Java; U; en-us; nokia5800d-1) UCBrowser8.3.0.154/69/352/UCWEB Mobile  3gpp-gba";
const char S90776[] = "JUC(Linux; U;2.3.4;zh_cn;ZTE-C N760;320*480;)UCWEB7.6.0.75/139/32139";
const char S90777[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; bq Darwin Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90778[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40CX520; CC/XGA; en) Presto/2.7.61 Version/11.00";
const char S90779[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46EX523; CC/USA; en) Presto/2.7.61 Version/11.00";
const char S90780[] = "SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-in; sonyericssonj105i) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S90781[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.0.16) Gecko/X Firefox/3.0.16 Flock/2.5.6";
const char S90782[] = "Mozilla/5.0 (Windows; U; Windows NT 4.0; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.1.249.1064 Safari/532.5";
const char S90783[] = "Mozilla/5.0 (Windows; U; Windows NT 4.0; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.78 Safari/532.5";
const char S90784[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S90785[] = "GoogleEarth/5.1.3533.1731(Windows;Microsoft Windows XP (Service Pack 3);en-US;kml:2.2;client:Free;type:default)";
const char S90786[] = "GoogleEarth/5.1.3535.3218(Windows;Microsoft Windows XP (Service Pack 3);en-US;kml:2.2;client:Free;type:default)";
const char S90787[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352";
const char S90788[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352";
const char S90789[] = "Mozilla/5.0 (Linux; Android 4.0.3; ME171 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S90790[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; A01SH Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90791[] = "iBrowser/Mini2.8 (NokiaC2-00/03.82)";
const char S90792[] = "MQQBrowser/3.1 (Nokia5233;SymbianOS/9.1 Series60/3.0)";
const char S90793[] = "Mozilla/5.0 (Windows NT 6.0; rv:15.0) Gecko/X Firefox/15.0.1 slurp";
const char S90794[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:15.0) Gecko/X Firefox/15.0.1 slurp";
const char S90795[] = "Mac OS X/10.8.2 (12C54); ExchangeWebServices/3.0 (157); Mail/6.1 (1498)";
const char S90796[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-gb) AppleWebKit/528.4+ (KHTML, like Gecko) Version/4.1 Safari/526.11.2";
const char S90797[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.1 Safari/530.18";
const char S90798[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_8; en-us) AppleWebKit/532.0+ (KHTML, like Gecko) Version/4.1 Safari/531.9";
const char S90799[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90800[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S90801[] = "Mozilla/5.0 (Java; U; en-us; micromaxq5) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S90802[] = "iBrowser/2.7 (NokiaN8-00;SymbianOS/9.1 Series60/3.0)";
const char S90803[] = "Mozilla/5.0 (Java; U; en-us; sonyericssonu5i) UCBrowser8.3.0.154/69/352/UCWEB Mobile";
const char S90804[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I5510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S90805[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; GT-I5503 Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S90806[] = "Videocon V1528/3.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S90807[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN85) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile";
const char S90808[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT18i Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90809[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Y2 Build/HCL ME Tablet Y2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90810[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-lg900g) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S90811[] = "Mozilla/5.0 (Linux; Android 4.0.4; N-03E Build/AX) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90812[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T508 Build/M) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90813[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMID4311 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90814[] = "Nokia201/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; kau; nokia201) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S90815[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-T100K Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90816[] = "Nokia302/5.0 (14.53) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia302) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S90817[] = "Mozilla/5.0 (Linux; U; Android 4.0.4 ICS; en-us; U8150 Build/GRK39F; ICSIdeos 2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90818[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Flipboard/1.9.8";
const char S90819[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Flipboard/1.9.6";
const char S90820[] = "Mozilla/5.0 (Linux; Android 4.0.4; starTIM tab 7 Build/TIM_01) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S90821[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90822[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.2; pt-br; GT-P1000L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90823[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90824[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.2; en-; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90825[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90826[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-P1000T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90827[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; EVO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90828[] = "SonyEricssonW705/R1GA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonw705) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S90829[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Explay Surfer 7.02 Build/ICS.g12refM703A1HZ1.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90830[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3 UCBrowser8.9.0.238/42/355";
const char S90831[] = "Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-46EX645; CC/USA; en) Presto/2.10.250 Version/11.60";
const char S90832[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LT26i Build/6.1.A.2.45) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S90833[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5880D Build/PMP5880D_v1.0.0_X) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S90834[] = "Android/1.0 (GT-P7500 HMJ37)";
const char S90835[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90836[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90837[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90838[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung gt-c3303) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S90839[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung gt-c3303) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile, SAMSUNG-GT-C3303/C3303DDJG4 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S90840[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; C5170 Build/IML77) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S90841[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT270 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S90842[] = "Mozilla/5.0 (Linux; Android 4.0.4; A9+ Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S90843[] = "Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A9+ Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S90844[] = "Mozilla/5.0 (Linux; Android 4.1.1; XOLO A1000 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S90845[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI T8951 Build/HuaweiT8951) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S90846[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A770K Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90847[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PIRANHA BUSINESS II TAB 9\" Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90848[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; U8185) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S90849[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T769 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S90850[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT885 Build/9.8.3O-148_IRD-3) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S90851[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID7048 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444";
const char S90852[] = "Mozilla/5.0 (Linux; Android 4.0.4; IdeaTab_A1107 Build/MR1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S90853[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MT27i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S90854[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V880G Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90855[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT926 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90856[] = "Mozilla/5.0 (Linux; Android 4.2.1; N9500 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S90857[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Spice_Mi-351) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S90858[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S90859[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; SHW-M250K Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90860[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; SHW-M250S Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90861[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; U8655 Build/HuaweiU8655; CyanogenMod 7.2 Unofficial) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90862[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A11 Build/KarbonnA11) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S90863[] = "Mozilla/5.0 (Linux; Android 4.0.4; Arc Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S90864[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-03D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90865[] = "RSSOwl/2.2.0.X (Macintosh; U; en)";
const char S90866[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; C6603 Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90867[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; UBISLATE7C+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S90868[] = "Mozilla/5.0 (Linux; Android 4.0.4; Penta IS703C Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90869[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A860S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90870[] = "Mozilla/5.0 (Linux; Android 4.1.1; ONDA MID Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S90871[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7310 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S90872[] = "Mozilla/5.0 (Linux; Android 2.3.5; HTC Rhyme S510b Build/GRJ90) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S90873[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; U18GT2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90874[] = "Nokia5200/2.0 (05.53) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5200) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S90875[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; 5860E Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S90876[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E976 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90877[] = "Mozilla/5.0 (Linux; Android 2.3.7; MT27i Build/6.0.B.3.184) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453";
const char S90878[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT29i Build/9.1.B.1.67) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S90879[] = "Mozilla/5.0 (Linux; Android 4.0.4; VIVO 4.3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90880[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9260 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90881[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9260 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90882[] = "Mozilla/5.0 (Linux; Android 4.2.1; JY-G3 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S90883[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; M040 Build/JRO03H) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S90884[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; mobiistar touch BEAN 454 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90885[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Goophone i9 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90886[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S90887[] = "Mozilla/5.0 (Series40; NokiaC1-01/06.15; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S90888[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; HTC Wildfire Build/FRG83D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S90889[] = "CKT50_COSMOS_11B_HW (MRE\\2.5.00(2500) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\0 gsensor\\0 keyboard\\reduced) COCOA-S05A_FLY_L2EN_203_X Release/2012.12.28 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US";
const char S90890[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; Kindle Fire Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S90891[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Infected HTC Incredible Build/JDQ39; CyanogenMod-INFECTION) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90892[] = "Mozilla/5.0 (Linux; Android 4.1.2; SBM200SH Build/S1002) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90893[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5670) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S90894[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A26) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S90895[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S90896[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC One V) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S90897[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S6012) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S90898[] = "LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gd510) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S90899[] = "Lenovo-A288t_TD/S100 Linux/2.6.35 Android/2.3.5 Release/02.29.2012 Browser/AppleWebkit533.1 Mobile Safari/533.1 UCBrowser/2.3.2.289";
const char S90900[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI C8950D Build/HuaweiC8950D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S90901[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI T8950 Build/HuaweiT8950) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S90902[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; Fly IQ246 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90903[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT875 Build/JDQ39E; Carbon-CARBON-JB-NIGHTLY-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90904[] = "Mozilla/5.0 (Linux; Android 4.1.1; PMP5780D Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S90905[] = "Mozilla/5.0 (Linux; Android 4.0.4; PadFone Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S90906[] = "Mozilla/5.0 (Linux; Android 4.0.4; PadFone Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S90907[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_Desire_C) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S90908[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_Desire_C) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S90909[] = "UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; X10i) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S90910[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI T8951 Build/HuaweiT8951) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S90911[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S90912[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire U dual sim Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S90913[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; XT926 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/xt926";
const char S90914[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; XT926 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/xt926";
const char S90915[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-1 Build/HuaweiU8950-1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S90916[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A25 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S90917[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC EVO 3D X515m Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S90918[] = "Mozilla/5.0 (Linux; Android 4.3; GT-I9100 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S90919[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8552 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S90920[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9220 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S90921[] = "Mozilla/5.0 (Linux; Android 4.0.3; IM-A830L Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S90922[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE T790 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S90923[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; LG-P350) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S90924[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; IM-A760S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S90925[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S90926[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Wildfire_S_A510e) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S90927[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A30 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S90928[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6210 Navigator) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S90929[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Iris_349) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S90930[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Iris_349+) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S90931[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire V Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S90932[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Desire S Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S90933[] = "MOT-A3100/ (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S90934[] = "MOT-A3100 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S90935[] = "BlackBerry8707/4.1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/125";
const char S90936[] = "Nokia6061/2.0 (3.03) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S90937[] = "HTC Streaming Player sprint / 1.0 / htc_speedy / 2.2";
const char S90938[] = "HTC Streaming Player sprint / 1.0 / htc_speedy / 2.3.3";
const char S90939[] = "HTC Streaming Player sprint / 1.0 / htc_speedy / 2.3.4";
const char S90940[] = "NexianNX-G353/MTK Release/01.01.2010 Browser/MAUI Profile";
const char S90941[] = "BlackBerry8120/4.5.0.174 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/303 UP.Browser/5.0.3.3";
const char S90942[] = "Nokia1682c_CMCC/2.0 (06.86) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S90943[] = "Nokia1682c/2.0 (06.86) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S90944[] = "Mozilla/5.0 (Linux; U; Android 1.6; pl-pl; Era G1 Build/DRC92) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S90945[] = "DoCoMo/2.0 N705i(c100;TB;W24H16)";
const char S90946[] = "DoCoMo/2.0 N705imyu(c100;TB;W24H16)";
const char S90947[] = "NOKIANokia N81";
const char S90948[] = "KDDI-TS3O UP.Browser/6.2_7.2.7.1.K.4.306 (GUI) MMP/2.0";
const char S90949[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.24 (KHTML, like Gecko) Iron/11.0.700.2 Chrome/11.0.700.2 Safari/534.24";
const char S90950[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:9.0.1) Gecko/X Firefox/9.0.1 SeaMonkey/2.6.1";
const char S90951[] = "gt-s5233a/UC Browser8.0.3.107/69/352";
const char S90952[] = "samsung-gt-s5333/UC Browser8.0.3.107/69/444";
const char S90953[] = "sonyericssonu5i/UC Browser8.0.3.107/69/352";
const char S90954[] = "sonyericssonu5i/UC Browser8.0.3.107/69/444";
const char S90955[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; GT-I8350)";
const char S90956[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; GT-I8350T)";
const char S90957[] = "JUC (Linux; U; 2.3.4; zh-cn; GN105; 320*480) UCWEB7.9.3.103/139/32702";
const char S90958[] = "Nokia300/5.0 (06.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia300/UC Browser8.0.3.107/69/444";
const char S90959[] = "Mozilla/5.0 (Java; U; en-in; nokian70-1) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S90960[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S90961[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:14.0) Gecko/X Firefox/14.0.1 SeaMonkey/2.11";
const char S90962[] = "Mozilla/5.0 (Linux; U; Android AcuraTeam 3D MAX V3.1; es-es; LG-P920 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S90963[] = "LG-A235/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S90964[] = "HTC-ST7377/2.10.502.4 (93577) Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S90965[] = "HTC-ST7377/1.59.502.3 (67150) Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S90966[] = "Mozilla/5.0 (Java; U; en-in; spice) UCBrowser8.3.0.154/69/405/UCWEB Mobile";
const char S90967[] = "Mozilla/5.0 (Java; U; en-in; j2me) UCBrowser8.3.0.154/70/352/UCWEB Mobile";
const char S90968[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S90969[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; HTC Desire Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S90970[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus one Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90971[] = "MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S90972[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S90973[] = "JUC(Linux;U;Android2.3.7;Zh_cn;MT27i;480*854;)UCWEB7.8.0.95/139/444";
const char S90974[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC Desire C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S90975[] = "Mozilla/5.0 (Linux; Android 4.0.3; ISW11F Build/FIK700) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90976[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.04506; Media Center PC 5.0) Sleipnir/2.7.2";
const char S90977[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; JMI T970 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90978[] = "Opera/9.80 (Windows NT 4.0) Presto/2.12.388 Version/12.11";
const char S90979[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F120L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90980[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One SC T528d Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S90981[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokian70-1) U2/1.0.0 UCBrowser/8.8.0.227 U2/1.0.0 Mobile";
const char S90982[] = "JUC (Linux; U; 4.0.4; zh-cn; GT-N8000; 800*1232) UCWEB7.9.0.94/139/444";
const char S90983[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Cloudfone Thrill 400g Build/VENUS_00.03.150.H1105) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S90984[] = "OneBrowser/3.1 (SAMSUNG-GT-E2252)";
const char S90985[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; eZeeTab707 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S90986[] = "Opera/9.80 (Linux mips ; U; en) Presto/2.6.33 Version/10.70";
const char S90987[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7102 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S90988[] = "Mozilla/5.0 (iPhone; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Rockmelt/2.2.0.241 Mobile/10A525 Safari/8536.25";
const char S90989[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT15i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S90990[] = "Nokia109/2.0 (04.10) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S90991[] = "Mozilla/5.0 (Linux; Android 4.0.4; U30GT-H Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S90992[] = "Mozilla/5.0 (Linux; Android 4.1.2; Desire HD Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S90993[] = "Instagram 1.1.1 Android (13/3.2; 160dpi; 600x976; samsung; GT-P6200; GT-P6200; smdkc210; en_GB)";
const char S90994[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.16/276,1.9.16.276,2012-12-21 16:37, +0300, kw";
const char S90995[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P6800 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.28 Safari/537.31";
const char S90996[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-p6800 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S90997[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_Velocity_4G_X710s; en-hk) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S90998[] = "Mozilla/5.0 (X11; Linux x86_64; HTC_Velocity_4G_X710s; zh-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S90999[] = "Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US; rv:1.9.2.15pre) Gecko/X Prism zdesktop/7.2.2";
const char S91000[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9070 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S91001[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9070 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S91002[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91003[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZP300+ Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91004[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-SU640 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91005[] = "Mozilla/5.0 (Linux; Android 4.0.4; ISW11SC Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91006[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Nexus 4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, +0100, gb";
const char S91007[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGT-A22 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91008[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S91009[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8825D Build/HuaweiC8825D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S91010[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A90S) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S91011[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk550i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S91012[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Andi3n) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S91013[] = "Mozilla/5.0 (Linux; Android 4.0.4; IS12S Build/6.1.D.1.91) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91014[] = "Mozilla/5.0 (Linux; Android 4.2.2; Desire S Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91015[] = "Mozilla/5.0 (Linux; Android 4.1.1; M032 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91016[] = "Mozilla/5.0 (Linux; Android 4.1.2; 201M Build/9.8.2Q-34_SMJ-102) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91017[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Desire Build/VJ CM10.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91018[] = "X-Client/AppexWin8Microsoft.BingNews X-Client-AppVersion/2.0.0.273";
const char S91019[] = "Mozilla/5.0 (Linux; Android 4.1.1; ZTE V970M Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91020[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOLO_X1000 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S91021[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S91022[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax_A91) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S91023[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; C6603 Build/10.1.A.1.434) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S91024[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; C6603 Build/10.1.1.A.1.253) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S91025[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian73-1) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S91026[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE73) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S91027[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; A75 Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S91028[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; N860 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S91029[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LG-P690 Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S91030[] = "Mozilla/5.0 (Linux; Android 4.0.4; Andromax U Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91031[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F100L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91032[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; SO-01D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91033[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; GT-P5100) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S91034[] = "Nokia309/2.0 (05.85) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia309) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S91035[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MK16i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S91036[] = "Mozilla/5.0 (Linux; Android 4.0.4; LGL21 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91037[] = "Mozilla/5.0 (Linux; Android 4.1.1; GSmart Maya M1 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91038[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax A91) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S91039[] = "Mozilla/5.0 (Linux; Android 4.0.4; U30GT-H Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S91040[] = "Mozilla/5.0 (Linux; Android 4.1.2; Amaze 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91041[] = "Mozilla/5.0 (Linux; Android 4.0.4; fnac 4.5 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91042[] = "Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S91043[] = "OneBrowser/3.1 (Nokia202/20.28)";
const char S91044[] = "OneBrowser/3.1 (Nokia300/07.57)";
const char S91045[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; bg-bg; Privileg MID-7M Build/Privileg_MID-7M) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91046[] = "Mozilla/5.0 (Series40; Nokia6700c-1/13.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S91047[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; iBall_Andi3.5) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S91048[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A87) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S91049[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; MT11i) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S91050[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT8000 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S91051[] = "Mozilla/5.0 (Linux; Android 4.1.2; A111 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91052[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; iris502 Build/LAVAIRIS502) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S91053[] = "Mozilla/5.0 (Linux; Android 4.0.4; TECNO_N7 Build/TECNOTECNO_N7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91054[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5380k) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S91055[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5380d) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S91056[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I747 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S91057[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T769 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S91058[] = "UCWEB/2.0 (Windows; U; wds 8.0; en-US; HTC;  8S by HTC) U2/1.0.0 UCBrowser/3.0.0.285 U2/1.0.0 Mobile";
const char S91059[] = "UCWEB/2.0 (Windows; U; wds 8.0; en-US; HTC;  8S by HTC) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile";
const char S91060[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S91061[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC One X+ Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S91062[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S91063[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; C5302 Build/12.0.A.1.211) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S91064[] = "Mozilla/5.0 (Linux; Android 4.1.1; Lenovo P770 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S91065[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.6 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/537.6 Midori/0.5";
const char S91066[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E445 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91067[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS920 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S91068[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA5630d-1) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S91069[] = "Mozilla/5.0 (Linux; Android 3.2.1; K1 Build/HTK75C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S91070[] = "Mozilla/5.0 (Linux; Android 4.0.4; IdeaTab_A1107 Build/MR1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S91071[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I337 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/jflteatt";
const char S91072[] = "Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S91073[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; ST15i Build/4.0.2.A.0.84) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S91074[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5380k) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S91075[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5380d) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S91076[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S6012) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S91077[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Micromax A115 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S91078[] = "Nokia2700c-2/2.0 (09.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311";
const char S91079[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5302 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S91080[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-S7572 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S91081[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; C2104) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S91082[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; AURUS_III) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S91083[] = "Mozilla/5.0 (Linux; Android 4.0.4; Enjoy 71 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S91084[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5250) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S91085[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7102 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S91086[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N5110 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S91087[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5570 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S91088[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5570I Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S91089[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I8150 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S91090[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-N7102 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S91091[] = "Mozilla/5.0 (Linux; Android 4.0.4; A9+ Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S91092[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE N9120 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S91093[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Smartphone; 176x220; HPiPAQ510/1.0";
const char S91094[] = "Nokia3650/1.0 SymbianOS/6.1 Series60/1.2 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S91095[] = "Nokia3650";
const char S91096[] = "Nokia7210/1.0 (3.08) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S91097[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-pt; a5 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S91098[] = "Mozilla/5.0 (Linux; U; Android Genocide_0.3a_EB13_OC/UV_Kernel; en-us; SPH-D700 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91099[] = "Mozilla/5.0 (Linux; U; Android Bonsai/Kernel_v3.0.0; en-us; SPH-D700 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91100[] = "Mozilla/5.0 (Linux; U; Android Frankenstein E=Bee13 v1.0.1; en-us; SPH-D700 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91101[] = "Mozilla/5.0 (Linux; U; Android Genocide Kernel 0.2a EB13 1.2GHz OC/UV; en-us; SPH-D700 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91102[] = "Mozilla/5.0 (Linux; U; Android Bonsai/Kernel_v4.0.1; en-us; SPH-D700 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91103[] = "Mozilla/5.0 (Linux; U; Android Bonsai/Kernel_v4.1.1; en-us; SPH-D700 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91104[] = "Mozilla/5.0 (Linux; U; Android Bonsai/SPH-D700_v4.1.3; en-us Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91105[] = "Mozilla/5.0 (Linux; U; Android BeigePhone Epic 4G v1.0; en-us; SPH-D700 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91106[] = "Motorola-V3m Obigo/Q04C1 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S91107[] = "Motorola-V3m-Red Obigo/Q04C1 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S91108[] = "ZTE-CG990/CG990_Z4_IN_BFFC107 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C";
const char S91109[] = "ZTE-CG990/CG990_Z0_ID_BFFC102 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C";
const char S91110[] = "ZTE-CG990/CG990_Z1_ID_BFFC102 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C";
const char S91111[] = "ZTE-CG990/CG990_Z11_SZ_BFFC100 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C";
const char S91112[] = "ZTE-CG990/CG990_Z10_NG_BFFC100 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C";
const char S91113[] = "ZTE-CG990/CG990_Z4_IN_BFFC101 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C";
const char S91114[] = "ZTE Bluebelt/v2/1.0.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S91115[] = "ZTE Bluebelt/1.0.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S91116[] = "DoCoMo/2.0 SH905iTV(c100;TB;W24H16)";
const char S91117[] = "DoCoMo/2.0 SH905i(c100;TB;W24H16)";
const char S91118[] = "Samsung-SCHU440 Infraware-Polaris/6.0 MMP/2.0";
const char S91119[] = "NOKIAE5-00/UC Browser7.6.1.82/28/352";
const char S91120[] = "NokiaE71/UC Browser7.6.1.82/28/351";
const char S91121[] = "KDDI-HI3E UP.Browser/6.2_7.2.7.1.K.2.234 (GUI) MMP/2.0";
const char S91122[] = "Opera/9.80 (Macintosh; Intel Mac OS X; U; en)";
const char S91123[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; HTC Mecha Build/MIUI-Icecream) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91124[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; HTC Mecha Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91125[] = "Mozilla/5.0 (Linux; U; Android 4.0.1 / 2.3.7 Hybrid; en-us; HTC Mecha Build/MIUI-Icecream) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91126[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618) chromeframe/8.0.552.224";
const char S91127[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB5; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; InfoPath.2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; OfficeLiveConnector.1.5; OfficeLivePatch.1.3) chromeframe/8.0.552.224";
const char S91128[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:5.0) Gecko/X Firefox/5.0-x64 PaleMoon/5.0-x64";
const char S91129[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:5.0) Gecko/X Firefox/5.0 PaleMoon/5.0";
const char S91130[] = "Mozilla/5.0 (Series40; NokiaC2-03/06.51; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.4.0.34.8";
const char S91131[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.2.139/145/355";
const char S91132[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; HTC Wildfire S A510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S91133[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC_Wildfire_S_A510e Build/GRJ90) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S91134[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; TAB460 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91135[] = "samsung-gt-s5260/UC Browser8.0.3.107/69/444";
const char S91136[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 Comodo_Dragon/19.0.3.0";
const char S91137[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 Comodo_Dragon/19.0.3.0";
const char S91138[] = "Dolphin http client/8.5.1(152) (Android)";
const char S91139[] = "Mozilla/5.0 (Windows NT 5.0; rv:6.0.2) Gecko/X Firefox/6.0.2 SeaMonkey/2.3.3";
const char S91140[] = "Mozilla/5.0 (Series40; NokiaC2-01/10.50; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.4.0.34.8";
const char S91141[] = "iBrowser/2.4 (NokiaE5-00;SymbianOS/9.1 Series60/3.0)";
const char S91142[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-VM670 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91143[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; LG-VM670 Build/ICS40; IceCreamSandwich-Final) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91144[] = "Opera/9.80 (Linux; Opera Mobi/2; U; en) Presto/2.10.254 Version/12.00";
const char S91145[] = "MQQBrowser/Mini2.7 (NokiaN70-1/5.0609.2.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1 )";
const char S91146[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; U8150 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91147[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Ideos Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91148[] = "MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91149[] = "MQQBrowser/24 (Linux; U; 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD.ZQKJ6;480*800)";
const char S91150[] = "Mozilla/5.0 (Linux; Android 4.0.3; X10 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S91151[] = "iBrowser/Mini2.8 (NokiaX2-00/08.25)";
const char S91152[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2) QQBrowser/6.0";
const char S91153[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; MB526 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91154[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MB526 Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S91155[] = "Mozilla/5.0 (Linux; Android 4.0.3; TURKCELL MaxiPRO5 Build/HuaweiU8860) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91156[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s3650) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S91157[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E10i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S91158[] = "JUC(Linux;U;Android2.3.6;Zh_cn;GT-I9070;480*800;)UCWEB7.8.0.95/139/444";
const char S91159[] = "iBrowser/Mini2.8 (NokiaC3-00/08.65)";
const char S91160[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; ZTE; MegaFon SP-W1)";
const char S91161[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91162[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-t300) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S91163[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-t300) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S91164[] = "UCWEB/2.0(Java; U; MIDP-2.0; vi; lg-t300) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S91165[] = "Mozilla/5.0 (X11; OpenBSD amd64; rv:17.0) Gecko/X Firefox/17.0";
const char S91166[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.1 Chrome/15.0.900.1 Safari/535.2";
const char S91167[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaC5-06)/UC Browser8.5.0.183/50/355/UCWEB Mobile";
const char S91168[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91169[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91170[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; LG-E400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91171[] = "Mozilla/5.0 (Linux; Android 4.1.1; Aurora-II Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S91172[] = "Mozilla/5.0 (Linux; Android 4.0.4; Starmobile CRYSTAL Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91173[] = "Mozilla/5.0 (Linux; Android 4.0.4; X6-7A MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91174[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10A523 Safari/7534.48.3 UCBrowser8.9.0.238/44/355";
const char S91175[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9308 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91176[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L710 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91177[] = "UCWEB/2.0 (Java; U; MIDP-2.0; vi; gt-s5233w) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S91178[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233w) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S91179[] = "UCWEB/2.0 (Java; U; MIDP-2.0; fr-FR; gt-s5233w) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S91180[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5603) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S91181[] = "Mozilla/5.0 (Linux; Android 4.0.4; BNTV600 Build/2.0.6.6008.magnesium02.ovation.rrdf.s1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S91182[] = "JUC (Linux; U; 2.3.6; zh-cn; GT-I8530; 480*800) UCWEB7.9.0.94/139/444";
const char S91183[] = "Mozilla/5.0 (Linux; Android 4.1.1; M032 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91184[] = "Mozilla/5.0 (Linux; Android 4.2.2; Infected Rezound Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91185[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-P990 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91186[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F100L Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91187[] = "WhatsApp/2.8.5023 Android/3.2 Device/samsung-GT-P6200";
const char S91188[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P7510 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S91189[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S91190[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);";
const char S91191[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT26w Build/6.1.A.2.55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S91192[] = "Mozilla/5.0 (Linux; Android 4.0.4; Spice Mi-530 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91193[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Tablet PC szenio 8100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91194[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N8100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91195[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 510)";
const char S91196[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS GAMEPAD Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S91197[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I915 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S91198[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD; 320*480) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444";
const char S91199[] = "UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; GT-P1000) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S91200[] = "Mozilla/5.0 (Linux; Android 4.1.9; I9220 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91201[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-i9220 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91202[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S91203[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; eZee'Tab1001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91204[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; eZee'Tab1001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91205[] = "TINNO50_6464_W1224_11B_HW (MRE\\2.5.00(800) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\1 gsensor\\0 keyboard\\normal) F7013A_T28TV_X_V10 Release/2012.10.20 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US";
const char S91206[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MB865 Build/unknown; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91207[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; SNM-LT25c Build/9.0.E.1.80) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91208[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; PIONEER) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S91209[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; huaqin73_gb) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S91210[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; SamsungI8910) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S91211[] = "Mozilla/5.0 (Linux; Android 4.0.4; IS12S Build/6.1.D.1.91) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91212[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A7 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S91213[] = "Mozilla/5.0 (Linux; Android 4.1.2; VS930 4G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S91214[] = "Mozilla/5.0 (Linux; Android 4.0.4; A700 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S91215[] = "Mozilla/5.0 (Linux; Android 4.0.4; HYUNDAI T10 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S91216[] = "Mozilla/5.0 (Linux; Android 4.1.2; Dream_D1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91217[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T869 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S91218[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; AirTab M975w Build/M975W) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91219[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E10i Build/2.1.1.A.0.6) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S91220[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC ChaCha A810e Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S91221[] = "Mozilla/5.0 (Linux; Android 2.3.4; WT19i Build/4.0.2.A.0.62) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453";
const char S91222[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I500 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91223[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S91224[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A35) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S91225[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S91226[] = "Nokia308/2.0 (05.85) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia308/UC Browser8.0.3.107/69/444";
const char S91227[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A111 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S91228[] = "Mozilla/5.0 (Series40; Nokia2010/11.95; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S91229[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; ALCATEL ONE TOUCH 4030D Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S91230[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A87) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S91231[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-B5330 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S91232[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Mobiistar S01 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91233[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/ZV5.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S91234[] = "Mozilla/5.0 (Linux; Android 4.0.4; Inspire 4G Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91235[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Inspire 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91236[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax P362 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S91237[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; YP-G70 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91238[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; YP-G70 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91239[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S91240[] = "Mozilla/5.0 (Linux; Android 4.2.1; i-mobile IQ X Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91241[] = "Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291";
const char S91242[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-I9070 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S91243[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; Nokia6220c-1) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S91244[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; C2104) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S91245[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; zh-CN; M040) U2/1.0.0 UCBrowser/9.1.1.309 U2/1.0.0 Mobile";
const char S91246[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.6.1.X (Second Life Release; default skin) Safari/533.3";
const char S91247[] = "NokiaX2-02/2.0 (11.79) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291";
const char S91248[] = "Mozilla/5.0 (Linux; Android 4.1.1; HP Slate 7 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S91249[] = "Mozilla/5.0 (Linux; Android 4.0.4; A21 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S91250[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST15i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S91251[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9000 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S91252[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP7170B3GCE Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S91253[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP7170B3G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S91254[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; MT27i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S91255[] = "Mozilla/5.0 (Linux; Android 4.2.1; PMP5588C Build/E8HD-ELITE-1280x768) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S91256[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; A54) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S91257[] = "[FBAN/FB4A;FBAV/3.6.1;FBBV/X;FBDM/{density=3.0,width=1080,height=1920};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/SCH-I545;FBSV/4.2.2;FBCA/armeabi-v7a:armeabi;]";
const char S91258[] = "Mozilla/5.0 (Linux; Android 4.0.4; P5 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S91259[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; One S Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/ville";
const char S91260[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AN10BG2I Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91261[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9308 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S91262[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LRA Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S91263[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; LG-LS970 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/ls970";
const char S91264[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; LG-LS970 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/ls970";
const char S91265[] = "Mozilla/5.0 (Series40; Nokia113/03.32; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S91266[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-US; Nexus 7 Build/JWR66Y) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.1.344 U3/0.8.0 Mobile Safari/534.31";
const char S91267[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS840 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S91268[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS840PP Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S91269[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5200 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S91270[] = "Fly DS160, Fly DS160";
const char S91271[] = "Fly DS160";
const char S91272[] = "Fly_DS170 Configuration/WAP2.0/Browser";
const char S91273[] = "Xda_orbit_2/240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S91274[] = "Xda Orbit; 240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S91275[] = "SAMSUNG-GT-B7620/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S91276[] = "Nokia2322c_CMCC/2.0 (08.20) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S91277[] = "SAMSUNG-SCH-M715/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S91278[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) ASUS-GalaxyMini/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S91279[] = "Mozilla/5.0 (YL-COOLPAD_E600/10.14.E600;U;REX/1.0;BREW/3.1.5SP01;240*320;CTC/2.0) POLARIS/V6.15.090923";
const char S91280[] = "Mozilla/5.0 (YL-Coolpad_D510/03.17.D510;U;REX/1.0;BREW/3.1.5SP01;240*320;CTC/2.0) POLARIS/v6.15.X";
const char S91281[] = "Mozilla/5.0 (YL-COOLPAD_E600/10.14.E600;U;REX/1.0;BREW/3.1.5SP01;240*320;CTC/2.0) POLARIS/V6.15.X";
const char S91282[] = "LG-KP501 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S91283[] = "VX8300v1";
const char S91284[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB300 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91285[] = "nokia6233/UC Browser8.0.3.107/69/444";
const char S91286[] = "BlackBerry9630/4.2.0.451 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S91287[] = "Mozilla/5.0 (X11; Linux i686; rv:7.0) Gecko/X Firefox/7.0 SeaMonkey/2.4";
const char S91288[] = "Mozilla/5.0 (Windows; U; Windows NT 4.10; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/1.0.154.53 Safari/525.19";
const char S91289[] = "Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6.2";
const char S91290[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:2.0) Gecko/X Firefox/4.0 CometBird/4.0";
const char S91291[] = "Mozilla/5.0 (Windows NT 5.1; rv:2.0) Gecko/X Firefox/4.0 CometBrowser/4.0";
const char S91292[] = "SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonw20i/UC Browser7.9.0.102/69/352";
const char S91293[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:9.1) Gecko/X Firefox/9.1 PaleMoon/9.1";
const char S91294[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:9.1) Gecko/X Firefox/9.1-x64 PaleMoon/9.1-x64";
const char S91295[] = "Mozilla/5.0 (X11; Linux i686) KHTML/4.7.2 (like Gecko) Konqueror/4.7 Fedora/4.7.2-5.fc16";
const char S91296[] = "Mozilla/5.0 (X11; Linux x86_64) KHTML/4.7.2 (like Gecko) Konqueror/4.7 Fedora/4.7.2-5.fc16";
const char S91297[] = "Mozilla/5.1 (X11; U; Linux i686; en-US; rv:1.9.0.1) Gecko/X Fedora/3.0.1-1.fc9 Firefox/9.0.1";
const char S91298[] = "MQQBrowser/28 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S91299[] = "MQQBrowser/28 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3";
const char S91300[] = "MQQBrowser/28 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3";
const char S91301[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Inspire 4G Build/GRI40; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91302[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:9.0.1) Gecko/X Firefox/9.0.1 PaleMoon/9.0.1";
const char S91303[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:9.0.1) Gecko/X Firefox/9.0.1-x64 PaleMoon/9.0.1-x64";
const char S91304[] = "Mozilla/5.0 (Linux; Android 4.0.4; MI-ONE C1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91305[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Elocity A7 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91306[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; th-th; A7 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91307[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; Panasonic SV-ME970 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S91308[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; Panasonic SV-MV100 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S91309[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S91310[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5570I Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S91311[] = "Mozilla/5.0 (Linux; U; Android 2.3; es-es; OnePAD 730 Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91312[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; ZP200 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91313[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; AT830 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91314[] = "MQQBrowser/2.8 (NokiaC5-03;SymbianOS/9.1 Series60/3.0)";
const char S91315[] = "nokiae72-1/UC Browser8.0.3.107/69/444  3gpp-gba";
const char S91316[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:13.0) Gecko/X Thunderbird/13.0.1";
const char S91317[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1b1pre) Gecko/X Minefield/3.1b1pre";
const char S91318[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:12.0) Gecko/X Firefox/12.0 SeaMonkey/2.9.1";
const char S91319[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/11.10 Chromium/22.0.1229.79 Chrome/22.0.1229.79 Safari/537.4";
const char S91320[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/11.10 Chromium/22.0.1229.79 Chrome/22.0.1229.79 Safari/537.4";
const char S91321[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.2.2.29837 (Firestorm-Release; firestorm skin) Safari/533.3";
const char S91322[] = "Mozilla/5.0 (Linux; Android 4.0.3; Onda Vi40 Elite - Top Notch Tablets Edition Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S91323[] = "Mozilla/5.0 (Symbian3; U; en-us; Nokia700) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/51/355/UCWEB Mobile";
const char S91324[] = "MQQBrowser/371 Mozilla/5.0 (iPhone 4; U; CPU iPhone OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3";
const char S91325[] = "MQQBrowser/37 Mozilla/5.0 (iPhone 4; U; CPU iPhone OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3";
const char S91326[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokian70-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S91327[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokian70) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S91328[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I8150 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91329[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S91330[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5233w) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S91331[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5233w) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S91332[] = "UCWEB/2.0(Java; U; MIDP-2.0; vi; gt-s5233w) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S91333[] = "UCWEB/2.0(Java; U; MIDP-2.0; vi; gt-s5233w) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S91334[] = "UCWEB/2.0(Java; U; MIDP-2.0; fr-fr; gt-s5233w) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S91335[] = "JUC (Linux; U; 4.0.3; zh-cn; Aurora-II; 1024*552) UCWEB1.0.1.113/147/800";
const char S91336[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; LG-KU8800 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91337[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5670 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91338[] = "Mozilla/5.0 (Linux; Android 4.0.4; Accord711 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91339[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxx78) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S91340[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; TM-3200R Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91341[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; T580 Build/X_8461_V0526) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S91342[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I957 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S91343[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91344[] = "Mozilla/5.0 (Linux; Android 4.0.4; Alpha GTR Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91345[] = "Mozilla/5.0 (Linux; Android 4.0.4; 201HW Build/HuaweiU9201L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91346[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; ST15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S91347[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; N81 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91348[] = "Mozilla/5.0 (Linux; Android 4.0.4(6577); Blaze S180  Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91349[] = "Mozilla/5.0 (Linux; Android 4.0.4; ThL W5 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91350[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAP4500DUO Build/PrestigioPAP4500DUO) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91351[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Woxter Tablet PC 50BL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91352[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A65 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91353[] = "JUC (Linux; U; 4.0.4; zh-cn; MI 1S; 480*854) UCWEB8.7.4.225/145/32693";
const char S91354[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw150i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S91355[] = "UCWEB/2.0 (Java; U; MIDP-2.0; fa; sonyericssonw150i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S91356[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-S5300 Build/GINGERBREAD) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315";
const char S91357[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-S5300 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Mobile Safari/537.31 OPR/14.0.1074.54070";
const char S91358[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A45 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S91359[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TM-4504 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91360[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; XOLO A700 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S91361[] = "Mozilla/5.0 (Linux; Android 4.0.3; Newman N2 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91362[] = "Mozilla/5.0 (Linux; Android 4.0.4; GSmart GS202+ Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91363[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT-AS40SE Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91364[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Desire Build/VJ CM10.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91365[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S91366[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91367[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P3108 Build/IMM76D)";
const char S91368[] = "Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S91369[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build/IMM76D; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S91370[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx368) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S91371[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5238) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S91372[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en-US; rv:1.9.2.13) Gecko/X Thunderbird/3.1.7";
const char S91373[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-us; Swipe_Fablet_F1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91374[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; SCH-I589) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S91375[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S720 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91376[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P778 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91377[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; FLY F41s Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91378[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T959V Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91379[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P920 Build/GRJ90; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91380[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; SK17i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S91381[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7108 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S91382[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7108 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S91383[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9220 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91384[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-i9220 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91385[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7568 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S91386[] = "JUC (Linux; U; 4.0.4; zh-cn; MOMO9_STAR; 800*432) UCWEB1.0.1.113/147/800";
const char S91387[] = "JUC (Linux; U; 4.0.4; zh-cn; MOMO9star; 800*432) UCWEB1.0.1.146/147/800";
const char S91388[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LG-MS770 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S91389[] = "Mozilla/5.0 (Linux; Android 4.1.2; IQ446 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91390[] = "Mozilla/5.0 (Linux; Android 4.0.4; AUXUS CoreX4 3G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S91391[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:21.0) Gecko/X Firefox/21.0 slurp";
const char S91392[] = "Mozilla/5.0 (Linux; Android 4.0.4; SO-05D Build/7.0.D.1.137) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91393[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; IRIS355) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S91394[] = "Mozilla/5.0 (Linux; Android 4.1.1; TP7-1000DC light Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S91395[] = "Mozilla/5.0 (Linux; U; Android Tomatoes Version II; vi-vn; IM-A820L Build/Tomatoes Version II) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91396[] = "Mozilla/5.0 (Linux; U; Android Tomatoes Final; vi-vn; IM-A820L Build/Tomatoes Version II) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91397[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-E930 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91398[] = "Mozilla/5.0 (Linux; Android 4.0.4; A701 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91399[] = "Mozilla/5.0 (Linux; Android 4.0.4; A701 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S91400[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Intex Aqua Marvel Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S91401[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LT7035-D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91402[] = "Mozilla/5.0 (Linux; Android 4.1.1; TSINTECH Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S91403[] = "Mozilla/5.0 (Linux; Android 4.0.3; X9017 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91404[] = "Mozilla/5.0 (Linux; Android 4.0.3; X907 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91405[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; LG-E612) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S91406[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaX6-00";
const char S91407[] = "Mozilla/5.0 (Linux; Android 4.1.2; Flame2 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91408[] = "Mozilla/5.0 (Linux; Android 4.2.1; UMI X2 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91409[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 7020 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91410[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GOCLEVER TAB A104.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91411[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC T528t Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91412[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo S720 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S91413[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S5301) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S91414[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; TECNO Q1 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S91415[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Win64; x64; Trident/6.0) (TARGUSinfo Web Crawler www.targusinfo.com)";
const char S91416[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S91417[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; Titanium S5 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S91418[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Primo H2 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91419[] = "Mozilla/5.0 (Linux; Android 4.2.1; SKY HD9500 Plus Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91420[] = "Mozilla/5.0 (Linux; Android 4.2.1; SKY HD9500 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91421[] = "Mozilla/5.0 (Windows Phone 8.0; ARM; Trident/7.0; Touch; rv:11.0; IEMobile/11.0; NOKIA; Lumia 920) like Gecko";
const char S91422[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A4+ Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33756";
const char S91423[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; myPad P4 Lite Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91424[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SP-140 DC Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91425[] = "Mozilla/5.0 (Series40; NokiaC2-01/11.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S91426[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-in; C2104 Build/15.0.A.1.31) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S91427[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-in; C2104 Build/15.0.A.2.17) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S91428[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D2-927G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S91429[] = "Mozilla/5.0 (Linux; Android 4.0.4; TA-M72CBT Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S91430[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT15i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S91431[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I200 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S91432[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A3+) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S91433[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033A Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91434[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-I8262 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S91435[] = "UCWEB/2.0 (Linux; U; Adr 2.3.9; en-US; Celkon_A_85) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S91436[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon";
const char S91437[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon";
const char S91438[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-I739 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S91439[] = "Mozilla/5.0 (Linux; Android 4.1.2; SM-T211 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S91440[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; MI-ONE C1 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S91441[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAP4500DUO Build/PrestigioPAP4500DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S91442[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9103 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S91443[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S91444[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S91445[] = "NokiaC3-00/5.0 (08.71) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311";
const char S91446[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT-AS45SE Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S91447[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S91448[] = "Mozilla/5.0 (Linux; Android 4.1.2; VS930 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S91449[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I527 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S91450[] = "HTC_P3300 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S91451[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)  PPC; 240x320; Toshiba  G810/RL1_081018_092ES2_61";
const char S91452[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)  PPC; 240x320; Toshiba G810";
const char S91453[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)  PPC; 240x320; Toshiba  G810/RL1_081105_093ES2_61";
const char S91454[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)  PPC; 240x320; Toshiba  G810";
const char S91455[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)  PPC; 240x320; Toshiba G810/ ver RL1_20090425_096IL1_61/ Windows CE; IE Mobile 7.11; Mozilla/4.0";
const char S91456[] = "Mozilla/5.0 (Linux; U; Android SDX.FROYO.DL25.OMEGA1_2.2.1; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91457[] = "Mozilla/5.0 (Linux; U; Android UbuntDroid_v5.1; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91458[] = "Mozilla/5.0 (Linux; U; Android Basic_DL05_2.2.1; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91459[] = "Mozilla/5.0 (Linux; U; Android SDX_FROYO_DL25_RC4_2.2.1; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91460[] = "Mozilla/5.0 (Linux; U; Android KeyLime09 v1.0; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91461[] = "Mozilla/5.0 (Linux; U; Android FROYO_DL25_RC4_2.2.1; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91462[] = "Mozilla/5.0 (Linux; U; Android Ginger Fresh EB28 V1.0; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91463[] = "Mozilla/5.0 (Linux; U; Android KeyLime28 v3.3; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91464[] = "Mozilla/5.0 (Linux; U; Android Ginger Fresh EB28 V1.5; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91465[] = "Mozilla/5.0 (Linux; U; Android KeyLime28 v4.0ro.build.date=2011. 06. 21. (i 22:11:41 KST; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91466[] = "Mozilla/5.0 (Linux; U; Android KeyLime28 v3.2.0; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91467[] = "Mozilla/5.0 (Linux; U; Android VULGARom Naked v1.1a; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91468[] = "Mozilla/5.0 (Linux; U; Android ICM v3.3 EB28; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91469[] = "Mozilla/5.0 (Linux; U; Android JROM EF09 2.2.2; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91470[] = "SonyEricssonW595/R3DA Profile/MIDP-2.1 Configuration/CLDC-1.1 UNTRUSTED/1.0";
const char S91471[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Dopod838Pro";
const char S91472[] = "Mot-WX390/1.0 ObigoInternetBrowser/Q03C";
const char S91473[] = "ZTE-MDTOUCHMINI(3ITA)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S91474[] = "SAMSUNG-SGH-D820/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S91475[] = "Alcatel-OT-505/1.0 model-Bouygues ObigoInternetBrowser/Q03C";
const char S91476[] = "Alcatel-OT-505/1.0 version 010 03 ObigoInternetBrowser/Q03C";
const char S91477[] = "Alcatel-OT-505/1.0 ObigoInternetBrowser/Q03C";
const char S91478[] = "NokiaN70-1/5.0737.3.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.2.2.51/27/351";
const char S91479[] = "LENOVO-i921/240A";
const char S91480[] = "ZTE-F900/1.0 ACS-NF/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S91481[] = "BlackBerry7100i/4.1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/103";
const char S91482[] = "LGKP501/v1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S91483[] = "NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-01/UC Browser7.6.1.82/69/351";
const char S91484[] = "NokiaC5-03/UC Browser7.7.1.88/50/444";
const char S91485[] = "HTC_Touch2_T3333 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S91486[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; Avant Browser)";
const char S91487[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; Avant Browser; Avant Browser)";
const char S91488[] = "TIANYU-KTOUCH/V908/MIDP2.0/CLDC1.1/Screen-240x320";
const char S91489[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Comodo_Dragon/6.0.0.10 Chrome/6.0.472.63 Safari/534.3";
const char S91490[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET CLR 3.0.30729) (TARGUSinfo Web Crawler www.targusinfo.com)";
const char S91491[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.2.8687.201";
const char S91492[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) QQBrowser/6.2.8687.201";
const char S91493[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0) QQBrowser/6.2.8687.201";
const char S91494[] = "Mozilla/5.0 (Windows; U; en_US) AppleWebKit/522.7 (KHTML, like Gecko) Dreamweaver/10.0.0.4117 Version/3.0 Safari/522.7";
const char S91495[] = "JUC(Linux;U;Android2.3.3;Zh_cn;HTC Desire S;480*800;)UCWEB7.8.0.95/139/444";
const char S91496[] = "JUC(Linux;U;Android2.3.5;Zh_cn;HTC Desire S;480*800;)UCWEB7.8.0.95/139/444";
const char S91497[] = "Mozilla/5.0 (X11; CrOS i686 0.0.0) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.57 Safari/534.24";
const char S91498[] = "Mozilla/5.0 (X11; CrOS i686 0.11.257) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.57 Safari/534.24";
const char S91499[] = "Mozilla/5.0 (X11; U; CrOS i686 0.0.0; en-US) AppleWebKit/534.21 (KHTML, like Gecko) Chrome/11.0.676.0 Safari/534.21";
const char S91500[] = "Mozilla/5.0 (Linux; U; Android Eclair; en-us; i-mobile i651 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S91501[] = "Mozilla/5.0 (Linux; U; Android Eclair; th-th; i-mobile i651 Build/ECLAIR) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S91502[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:5.0) Gecko/X Firefox/5.0 SeaMonkey/2.2";
const char S91503[] = "mauie800/UC Browser7.9.0.102/69/352";
const char S91504[] = "MQQBrowser/2.7 (Nokia5800XpressMusic;SymbianOS/9.1 Series60/3.0)";
const char S91505[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:9.0.1) Gecko/X Firefox/9.0.1 SeaMonkey/2.6.1";
const char S91506[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S91507[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S91508[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; TAB101F Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91509[] = "Nokia302/5.0 (14.53) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia302) UCBrowser8.2.1.144/70/355/UCWEB Mobile";
const char S91510[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; LG-P999 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91511[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91512[] = "Mozilla/5.0 (Windows; N; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth Pro/6.1.0.5001 Safari/532.4";
const char S91513[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; pt-pt; Boston Build/GRJ22; CM7.1_EAv7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91514[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE65)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S91515[] = "Opera/9.80 (Windows 95; U; ru) Presto/2.10.229 Version/11.62";
const char S91516[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; U20i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352";
const char S91517[] = "JUC(Linux;U;Android2.3.3;Zh_cn;HTC ChaCha A810e;320*480;)UCWEB7.8.0.95/139/352";
const char S91518[] = "JUC(Linux;U;Android2.3.5;Zh_cn;HTC ChaCha A810e;320*480;)UCWEB7.8.0.95/139/352";
const char S91519[] = "Mozilla/5.0 (S60V3; U; en-us; Nokia N96)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S91520[] = "MQQBrowser/36 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S91521[] = "MQQBrowser/36 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3";
const char S91522[] = "Nokia603";
const char S91523[] = "Mozilla/5.2 (Windows; U; Windows NT 5.2; en-EN) Gecko/X Firefox/1.8.7";
const char S91524[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; N-05D Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91525[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; chromeframe/22.0.1229.94)";
const char S91526[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.24) Gecko/X Icedove/3.1.16";
const char S91527[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.24) Gecko/X Icedove/3.1.16";
const char S91528[] = "JUC (Linux; U; 2.3.4; zh-cn; LG-E510; 320*480) UCWEB7.9.3.103/139/32430";
const char S91529[] = "Nokia300/5.0 (07.03) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia300/UC Browser7.9.0.102/69/352";
const char S91530[] = "Mozilla/5.0 (Linux; Android 4.0.4; SH-07D Build/S7190) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91531[] = "MQQBrowser/Mini2.8 (NokiaX2-02/11.57)";
const char S91532[] = "iBrowser/2.5 (NokiaE52-1;SymbianOS/9.3 Series60/3.2)";
const char S91533[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s5753e) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S91534[] = "Mozilla/5.0 (Linux; Android 4.0.4; MOMO9 STAR Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91535[] = "NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac1-02) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S91536[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; LG-P350 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91537[] = "OneBrowser/3.1 (Nokia205/03.18)";
const char S91538[] = "Mozilla/5.0 (Linux; Android 4.0.4; X401 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91539[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_Vision; 480*800) UCWEB7.9.0.94/139/352";
const char S91540[] = "JUC (Linux; U; 2.3.3; zh-cn; HTC_Vision; 480*800) UCWEB7.9.0.94/139/444";
const char S91541[] = "Mozilla/5.0 (Linux; Android 4.2.1; SCH-R530M Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91542[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S91543[] = "Mozilla/5.0 (Android; Linux armv7l; rv:21.0) Gecko/21.0 Firefox/21.0";
const char S91544[] = "Mozilla/5.0 (Linux; Android 4.2.1; HTC One X+ Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91545[] = "Mozilla/5.0 (Linux; Android 4.1.2; UMI X1 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91546[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia700) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S91547[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZP950 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91548[] = "Mozilla/5.0 (Java; U; en-us; lg-x330t) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S91549[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID8042 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S91550[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LiveWithWalkman Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91551[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7108 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S91552[] = "Opera/9.80 (Windows 95) Presto/2.12.388 Version/12.13";
const char S91553[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.19pre) Gecko/X Prism zdesktop/7.2.2";
const char S91554[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I997 Build/JOP40D; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91555[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; G-TIDE E8  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91556[] = "Mozilla/5.0 (Linux; Android 4.1.1; V360 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91557[] = "Mozilla/5.0 (Linux; Android 4.0.4; ThL W3 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91558[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT25c Build/9.0.E.1.80) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91559[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-au; GT-P6810 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91560[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; GT-P6810 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91561[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile IQ 1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91562[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91563[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP7100D3G Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S91564[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZP950 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91565[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZP950+ Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91566[] = "Mozilla/5.0 (Linux; Android 4.1.1; ST8 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S91567[] = "Mozilla/5.0 (Linux; Android 4.0.4; SOL21 Build/9.0.F.2.128) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91568[] = "iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ICS.f06ref848L.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91569[] = "DoCoMo/2.0 SH03E(c500;TB;W24H16)";
const char S91570[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia808 PureView) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S91571[] = "Mozilla/5.0 (Linux; Android 4.1.2; 201K Build/117.1.1c00) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91572[] = "Mozilla/5.0 (Linux; Android 4.0.4; S22 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91573[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; HTC_Desire_X) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S91574[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cynus T1 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91575[] = "Mozilla/5.0 (Linux; Android 4.1.1; CINK SLIM Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91576[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E973 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91577[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-01E Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S91578[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax A91 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91579[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9308 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91580[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; Lenovo A690 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S91581[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A7) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S91582[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A810K Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91583[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A810S Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91584[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; C6602) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S91585[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; TAB750 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91586[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-S7572 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S91587[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo A798t Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31";
const char S91588[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; Lenovo P770 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S91589[] = "Mozilla/5.0 (Linux; Android 4.1.1; WEXLER-TAB-7T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S91590[] = "Opera/8.01 (J2ME/MIDP; Opera Mini/10.1.000/1724; en; U; ssr)";
const char S91591[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Explorer A310e Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91592[] = "JUC (Linux; U; 4.1.2; zh-cn; GT-I9082; 480*800) UCWEB7.9.0.94/139/444";
const char S91593[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; Micromax A60 Build/ERE27) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S91594[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328t Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S91595[] = "Mozilla/5.0 (Linux; Android 4.0.4; MK16i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S91596[] = "Mozilla/5.0 (Linux; Android 4.0.4; N861 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S91597[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S91598[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S91599[] = "JUC (Linux; U; 4.0.4; zh-cn; GT-S5303; 240*320) UCWEB7.9.0.94/139/444";
const char S91600[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Gpad_G1 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S91601[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; X10i Build/2.1.A.0.435) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S91602[] = "Mozilla/5.0 (Linux; Android 4.1.1; Micromax A110 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S91603[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-lu6200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91604[] = "Mozilla/5.0 (Linux; Android 4.2.2; Ascend G300 Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91605[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MB865 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91606[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M97 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91607[] = "Mozilla/5.0 (Linux; Android 4.2.1; Primo H2 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91608[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S91609[] = "MQQBrowser/43 Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3";
const char S91610[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9502 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91611[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Huawei-U8665 Build/HuaweiU8665B037) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S91612[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; Karbonn A6) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S91613[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us";
const char S91614[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST15i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S91615[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk530i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S91616[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; A54) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S91617[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq50) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S91618[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P765 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S91619[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.1b2) Gecko/X Firefox/3.1b2";
const char S91620[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ALCATEL ONE TOUCH 918N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91621[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; BEN918N Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91622[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; FLY F8s Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91623[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.34 (KHTML, like Gecko)  QtWeb Internet Browser/3.8.4 http://www.QtWeb.net";
const char S91624[] = "UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Micromax_A110Q) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S91625[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Starmobile ASTRA Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91626[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S7t Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S91627[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; 8720 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S91628[] = "UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Micromax A110Q) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S91629[] = "Mozilla/5.0 (Linux; Android 4.0.3; PMP5197DULTRA Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S91630[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6812) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S91631[] = "Mozilla/5.0 (Linux; Android 4.1.2; S100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S91632[] = "Mozilla/5.0 (Android; Tablet: Nexus 7; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S91633[] = "Mozilla/5.0 (Linux; Android 4.1.2; BLU STUDIO 5.3 II Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S91634[] = "Mozilla/5.0 (Linux; Android 4.0.4; StarTrail III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S91635[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7310 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S91636[] = "Mozilla/5.0 (Linux; Android 4.2.2; vivo Xplay Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S91637[] = "Mozilla/5.0 (Linux; Android 4.2.1; i-mobile IQ 5.3 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S91638[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5253) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S91639[] = "Mozilla/5.0 (Linux; Android 4.2.2; SH-06E Build/S7190) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S91640[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LG-E615 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S91641[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; HTC Glacier Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S91642[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6720c-1b) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S91643[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6720c) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S91644[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S5303) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S91645[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A7* Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S91646[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Karbonn A10 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S91647[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9082 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S91648[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI A199 Build/HuaweiA199) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S91649[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6802 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S91650[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A25 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S91651[] = "UCWEB/2.0 (Linux; U; Adr 4.2.2; en-US; GT-I9500) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S91652[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; ST25i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S91653[] = "Mozilla/5.0 (Linux; Android 3.2; GT-P6200 Build/HTJ85B) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S91654[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A7+ Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S91655[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Q800 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S91656[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5302 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S91657[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-S5670 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S91658[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5670 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S91659[] = "Mozilla/5.0 (Linux; Android 4.1.2; Gpad_G2 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S91660[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S91661[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9500 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.5 Mobile Safari/533.1";
const char S91662[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A1+) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S91663[] = "Mozilla/5.0 (Series40; NokiaC3-01/07.51; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S91664[] = "Mozilla/5.0 (Series40; NokiaC3-01.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S91665[] = "MOT-ZN300/13.02.0BR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 17243441) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.65 [pt]";
const char S91666[] = "MOT-ZN300/13.02.14R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 17243462) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.65 [en]";
const char S91667[] = "NokiaE70-1/3.0 (1.0610.05.06) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S91668[] = "SonyEricssonK700/R1KG Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S91669[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Smartphone; 176x220; HPiPAQ510/1.0";
const char S91670[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; i-mate K-JAM PPC; 240x320; i-mate JAMin PPC; 240x320; PPC; 240x320)";
const char S91671[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; i-mate K-JAM PPC; 240x320; PPC; 240x320)";
const char S91672[] = "LG-KT520/v10a Browser/Obigo-Q05A/3.11 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S91673[] = "LG/KT520/v10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S91674[] = "SAMSUNG-SGH-A637/A637UCHH2 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S91675[] = "SAMSUNG-SGH-E730/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S91676[] = "SAMSUNG-SGH-T509S/T509UVID1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S91677[] = "SAMSUNG-SGH-T509/T509UVFC4 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S91678[] = "SAMSUNG-SGH-A796/A796UXJB1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S91679[] = "HTC_Touch_Cruise_T4242 Opera/9.50 (Windows NT 5.1; U; en)";
const char S91680[] = "HTC_Touch_Cruise_T4242 Opera/9.50 (Windows NT 5.1; U; zh-tw)";
const char S91681[] = "HTC_Touch_Cruise_T4242 Opera/9.50 (Windows NT 5.1; U; it)";
const char S91682[] = "HTC_Touch_Cruise_T4242 Opera/9.50 (Windows NT 5.1; U; nb)";
const char S91683[] = "UTSTARCOM-CDM-8964VM UP.Browser/7.2.7.2.398 (GUI) MMP/2.0";
const char S91684[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) 320x320; VZW; UTStar-SMT5800; Window Mobile 6.1 Standard;";
const char S91685[] = "SonyEricssonU1";
const char S91686[] = "SonyEricssonU10i";
const char S91687[] = "NokiaN72/5.0706.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.4.0.65/27/352";
const char S91688[] = "SEC-SGHC520/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S91689[] = "NOKIAE5-00/UC Browser7.8.0.95/28/355";
const char S91690[] = "NexianNX-G801/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S91691[] = "NexianNX-G801/MTK Release/5.12.2011 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S91692[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_DesireZ_A7272; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S91693[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/11.10 Chromium/13.0.782.215 Chrome/13.0.782.215 Safari/535.1";
const char S91694[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/11.10 Chromium/13.0.782.215 Chrome/13.0.782.215 Safari/535.1";
const char S91695[] = "Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/11.10 Chromium/13.0.782.215 Chrome/13.0.782.215 Safari/535.1";
const char S91696[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; R800i Build/4.0.2.A.0.42; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91697[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; T9199C Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91698[] = "Spice M9000";
const char S91699[] = "SAMSUNG-C6625/1.0 (compatible; MSIE 4.01; Windows CE; Smartphone)/UC Browser7.2.2.51";
const char S91700[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.7) Gecko/X Ubuntu Firefox/3.5.7";
const char S91701[] = "HTC_Touch_Diamond2_T5353 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S91702[] = "Nokia3110c/2.0 (07.01) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia3110c/UC Browser7.9.0.102/69/352";
const char S91703[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E16i Build/2.1.1.A.0.53) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S91704[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E16i Build/2.1.1.A.0.53) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S91705[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; TITAN X310e)";
const char S91706[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; PI39100)";
const char S91707[] = "PantechP5000/JTUSX BMP/1.0.2 DeviceId/X NetFront/4.1 OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S91708[] = "MQQBrowser/2.3 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3";
const char S91709[] = "MQQBrowser/2.3 Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3";
const char S91710[] = "MQQBrowser/2.3 Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 Safari/7534.48.3";
const char S91711[] = "SAMSUNG-SPH-D700/100.20306";
const char S91712[] = "Opera/9.80 (Windows NT 6.0; Opera Mobi/49; U; en) Presto/2.4.18 Version/10.00";
const char S91713[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE) Opera 8.0 [en]";
const char S91714[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6350 Build/GRJ22)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91715[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S91716[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:7.0b1) Gecko/X Firefox/7.0b1 TenFourFox/G5";
const char S91717[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; th-th; i-mobile I-Note Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91718[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MK16i Build/4.0.1.A.0.283; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91719[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ZP100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91720[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5620) UCBrowser8.4.0.159/69/405/UCWEB Mobile";
const char S91721[] = "Mozilla/5.0 (Linux; Android 4.0.3; T01A Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S91722[] = "Mozilla/5.0 (Linux; Android 4.0.4; T01A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S91723[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Ally Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91724[] = "iBrowser/Mini2.8 (Nokia2690/10.10)";
const char S91725[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:15.0) Gecko/X Firefox/15.0.2.0 IceDragon/15.0.2.0";
const char S91726[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91727[] = "Mozilla/5.0 (Java; U; en-in; lg-gu220) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S91728[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; HKPhone 4S-3G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91729[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Iron/6.0.475 Chrome/6.0.475.0 Safari/X.534";
const char S91730[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/70/352";
const char S91731[] = "Mozilla/5.0 (Java; U; en-us; lg-gu220) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S91732[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-01E Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S91733[] = "OneBrowser/3.1 (SAMSUNG-GT-E2652)";
const char S91734[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91735[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A660 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91736[] = "Mozilla/5.0 (Linux; Android 4.0.9; X21 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91737[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.2.6) Gecko/X Epic/1.0 Firefox/3.6.6";
const char S91738[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91739[] = "Mozilla/4.5 (compatible; iCab 2.9.1; Macintosh; U; PPC; Mac OS X)";
const char S91740[] = "iCab/2.9.5 (Macintosh; U; PPC; Mac OS X)";
const char S91741[] = "Mozilla/5.0 (Linux; Android 4.0.4; GSmart G1362 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91742[] = "Mozilla/5.0 (Linux; Android 4.0.4; MIDC901 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91743[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-x330t) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S91744[] = "TOUCH87 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S91745[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S91746[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U950 Build/IMM76L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S91747[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ZTE U950 Build/IMM76L)";
const char S91748[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; 5860E Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000";
const char S91749[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P698 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S91750[] = "Mozilla/5.0 (Linux; Android 4.1.2; Vivid 4G Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91751[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GSmart Maya M1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91752[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; GT-P5110 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7";
const char S91753[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P5110 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7";
const char S91754[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5110 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S91755[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.15/274,1.9.15.274,2012-12-21 15:38, +0000";
const char S91756[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, -0500, ca";
const char S91757[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; E15i Build/2.1.1.A.0.6; 320*480) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/352";
const char S91758[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Karbonn A30 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91759[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC VLE_U Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (HTC VLE_U, 42, 1.9.1)";
const char S91760[] = "MQQBrowser/3.4 (Nokia5233;SymbianOS/9.1 Series60/3.0)";
const char S91761[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9308 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91762[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9220 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S91763[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTX21 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91764[] = "Mozilla/5.0 (Linux; Android 4.0.4; IQ441 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91765[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F100L Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91766[] = "Mozilla/5.0 (Linux; Android 4.1.2; MB860 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91767[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.0.5) Gecko/X Firefox/3.0.5 Flock/2.0.3";
const char S91768[] = "Liferea/1.8.10 (Linux; fr_FR.UTF-8; http://liferea.sf.net/)";
const char S91769[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6280) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S91770[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; T-Mobile_G2) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S91771[] = "Mozilla/5.0 (Linux; Android 4.1.1; rk30sdk Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S91772[] = "Mozilla/5.0 (Linux; Android 4.0.4; NXM805ND_EN Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S91773[] = "OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91774[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; Poly Pad_7408IPS Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91775[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91776[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; CELKON_CT2) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S91777[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae7-00) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba";
const char S91778[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-7204 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91779[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N7100 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91780[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; MSD7) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S91781[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-M830 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91782[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F260S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91783[] = "Mozilla/5.0 (Linux; Android 4.0.4; Optimus_Madrid Build/IceCreamSandwich) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91784[] = "Mozilla/5.0 (Linux; Android 4.1.1; PRAXIS 13 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S91785[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia808 PureView) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S91786[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6760s-1) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S91787[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 5 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91788[] = "Mozilla/5.0 (Linux; Android 4.0.4; IRIS_501 Build/LAVAIRIS501) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91789[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARNOVA 90G3 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S91790[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.0; Maxthon)";
const char S91791[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)";
const char S91792[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Mini Plus 4G Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91793[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC X720d Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S91794[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC X720d) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91795[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  SonyEricssonP1i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S91796[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iDream_A75 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91797[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TPC0707 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91798[] = "Mozilla/5.0 (Linux; Android 4.0.4; RunboX5-W Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91799[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P935 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91800[] = "Mozilla/5.0 (Linux; Android 4.1.2; iris502 Build/LAVAIRIS502) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91801[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; ST27i Build/6.2.A.0.400) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S91802[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; ST27i Build/6.2.A.1.100) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S91803[] = "Mozilla/5.0 (Linux; Android 4.1.2; PantechP9070 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91804[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M500W Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S91805[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-MS870 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S91806[] = "Mozilla/5.0 (Linux; Android 4.2.1; E3 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91807[] = "Mozilla/5.0 (Linux; Android 4.2.1; Q1000 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91808[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; AMAZE S200 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91809[] = "Mozilla/5.0 (Linux; Android 4.1.2; BLU STUDIO 5.3 II Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91810[] = "Mozilla/5.0 (Linux; Android 4.2.1; C2 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91811[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; Q-Smart S16 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33829";
const char S91812[] = "Mozilla/5.0 (Linux; Android 4.1.1; Prism II Build/HuaweiU8686) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91813[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; baoxue Build/F7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33738";
const char S91814[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; 668 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33736";
const char S91815[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TSINTECH Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91816[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL ONE TOUCH 918N) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S91817[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL ONE TOUCH 918N) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S91818[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; IRIS 455 Build/LAVAIRIS455) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S91819[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A25 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S91820[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7000) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S91821[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST18i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S91822[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Q700 Build/XOLOQ700) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S91823[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.3; en-us; Karbonn_A18 Build/KarbonnA18) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S91824[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; ZP800H Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91825[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; ST21i2 Build/11.0.A.1.8) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S91826[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; ST21i Build/11.0.A.0.16) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S91827[] = "Mozilla/5.0 AppleCoreMedia/1.0.0.8J2 (iPad; U; CPU OS 4_0 like Mac OS X; en-us) AppleWebKit/531.21.10 (HTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10";
const char S91828[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.3; en-in; HTC Desire V Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S91829[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S5303) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S91830[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.60140";
const char S91831[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P990 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S91832[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Iris_349) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S91833[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHW-M500W Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S91834[] = "Mozilla/5.0 (Linux; Android 4.1.1; A50 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S91835[] = "Mozilla/5.0 (Linux; Android 4.2.2; MB886 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S91836[] = "Mozilla/5.0 (Linux; Android 4.0.4; S500 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S91837[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One VX Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S91838[] = "Mozilla/5.0 (Linux; Android 4.1.2; New Andromax-i Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S91839[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; N400 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S91840[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-gb; Nexus 4 Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb";
const char S91841[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9268 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S91842[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; iris502 Build/LAVAIRIS502) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S91843[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.8.1 Safari/534.34";
const char S91844[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-US; PD_Novel Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S91845[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; D709 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91846[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMID709 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91847[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC Desire C Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91848[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A5 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S91849[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E15i Build/2.1.1.A.0.6) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S91850[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I997 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S91851[] = "Mozilla/5.0 (Linux; Android 4.3; GT-N7100 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.24 Mobile Safari/537.36";
const char S91852[] = "Mozilla/5.0 (Linux; Android 4.3; GT-N7105 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.30 Mobile Safari/537.36";
const char S91853[] = "Nokia5130c-2/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311";
const char S91854[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; SK17i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S91855[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Intex Aqua 3.2) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S91856[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S91857[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; GT-I9300 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S91858[] = "Mozilla/5.0 (Linux; Android 4.1.2; Micromax A116 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S91859[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A34) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S91860[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_4010D) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S91861[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919V Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S91862[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6506 Build/10.3.A.0.423) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S91863[] = "LG9100/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S91864[] = "LG9100-parrot/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S91865[] = "LGE-VX8500/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S91866[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC Tattoo Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91867[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Tattoo Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91868[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; HTC Tattoo Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91869[] = "SAMSUNG-S8300/S8300BOIH3 SHP/VPP/R5 Opera/9.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S91870[] = "SAMSUNG-SGH-T629/T629UVFG8 Profile/MIDP-2.0 Configuration/CLDC-1.1 NetFront/3.2";
const char S91871[] = "SAMSUNG-SGH-T629R/T629UVFL2 Profile/MIDP-2.0 Configuration/CLDC-1.1 NetFront/3.2";
const char S91872[] = "HTC_P3450 Opera/9.50 (Windows NT 5.1; U; en)";
const char S91873[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) HTC-6900-MR2 BM";
const char S91874[] = "SAMSUNG-SGH-I616/1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S91875[] = "SAMSUNG-GT-M2513/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S91876[] = "SAMSUNG-GT-S3600i/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S91877[] = "MOT-W766/00.72 UP.Browser/7.2.7.5.630 (GUI) MMP/2.0";
const char S91878[] = "MOT-W766/00.72 UP.Browser/7.2.7.5.606 (GUI) MMP/2.0";
const char S91879[] = "SonyEricssonT68/R201A Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S91880[] = "DoCoMo/2.0 L03A(c100;TB;W20H11)";
const char S91881[] = "KDDI-HI3D UP.Browser/6.2_7.2.7.1.K.2.234 (GUI) MMP/2.0";
const char S91882[] = "KDDI-SH36 UP.Browser/6.2.0.14.2 (GUI) MMP/2.0";
const char S91883[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/6.24093/27.1324; U; ru) Presto/2.8.119 Version/11.10";
const char S91884[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/6.24288/25.729; U; en) Presto/2.5.25 Version/10.54";
const char S91885[] = "JUC(Linux;U;Android3.1;Zh_cn;GT-P7500;800*1280;)UCWEB7.8.0.95/139/355";
const char S91886[] = "Mozilla/5.0(Linux;U;Android3.1;Zh_cn;GT-P7500;800*1280;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S91887[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Comodo_Dragon/4.0.1.6 Chrome/4.0.249.78 Safari/532.5";
const char S91888[] = "MQQBrowser/2.2 Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3";
const char S91889[] = "MQQBrowser/2.2 Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S91890[] = "Mozilla/5.0 (compatible; Konqueror/4.4) KHTML/4.4.5 (like Gecko)";
const char S91891[] = "Opera/9.80 (X11; FreeBSD 8.2-RELEASE amd64; U; en) Presto/2.9.168 Version/11.51";
const char S91892[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E401,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S91893[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E200,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S91894[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_10 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E600,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S91895[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S91896[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148a,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S91897[] = "NokiaC3-01/5.0 (06.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac3-01/UC Browser7.9.0.102/69/352";
const char S91898[] = "Akregator/4.7.2; syndication SUSE";
const char S91899[] = "Akregator/4.7.3; syndication";
const char S91900[] = "MQQBrowser/2.2 Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3";
const char S91901[] = "MQQBrowser/2.2 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3";
const char S91902[] = "nokian72/UC Browser7.9.0.102/69/352";
const char S91903[] = "Mozilla/5.0 (Macintosh; PowerPC MorphOS 2.7; Odyssey Web Browser; rv:1.17) AppleWebKit/535.14 (KHTML, like Gecko) OWB/1.17 Safari/535.14";
const char S91904[] = "Mozilla/5.0 (Series40; NokiaX2-00/04.90; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.4.0.34.8";
const char S91905[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko)  QtWeb Internet Browser/3.3 http://www.QtWeb.net";
const char S91906[] = "Mozilla/5.0 (X11; OpenBSD i386; rv:12.0) Gecko/X Firefox/12.0";
const char S91907[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:6.0.1) Gecko/X Firefox/6.0.1 TenFourFox/7450";
const char S91908[] = "Mozilla/4.0 (compatible; MSIE 6.0 compatible; Asterias Crawler v4; +http://www.singingfish.com/help/spider.html; webmaster@singingfish.com); SpiderThread Revision: 3.10";
const char S91909[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U1 Build/HCL ME Tablet U1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91910[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; GT-S5830L Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91911[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S91912[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-S5830M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S91913[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; GPLUS-M600 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91914[] = "MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V889D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91915[] = "Mozilla/5.0 (Windows NT 6.0; rv:14.0) Gecko/X Firefox/14.0.3.0 IceDragon/14.0.3.0";
const char S91916[] = "Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU1)/UC Browser8.5.0.183/50/352/UCWEB Mobile";
const char S91917[] = "Mozilla/5.0 (S60V3; U; en-us; Nokia6220c-1)/UC Browser8.5.0.183/28/444/UCWEB Mobile";
const char S91918[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; T-Mobile myTouch 4G Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91919[] = "OneBrowser/3.0 (Nokia2690/10.10)";
const char S91920[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A200 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91921[] = "Mozilla/5.0 (Java; U; en-in; micromax) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S91922[] = "iBrowser/2.7 (Nokia5530;SymbianOS/9.1 Series60/3.0)";
const char S91923[] = "OneBrowser/3.0 (NokiaC1-01/04.40)";
const char S91924[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; E860 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S91925[] = "NOKIA-Lumia 510/UCWEB2.2.0.157/47/355";
const char S91926[] = "Mozilla/5.0 (Java; U; en-us; sonyericssonw150i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S91927[] = "Mozilla/5.0 (Linux; Android 4.0.4; CAL21 Build/AX) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91928[] = "Mozilla/5.0 (Linux; Android 4.0.3; Ex Note Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91929[] = "JUC(Linux;U;Android2.3.5;Zh_cn;Micromax A87;480*800;)UCWEB7.8.0.95/139/444";
const char S91930[] = "Mozilla/5.0 (Java; U; en-us; lg-t325) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S91931[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; ST15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91932[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; ST15i Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91933[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; One X Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91934[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC One X Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91935[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.1; MB525 Build/JRO03L)";
const char S91936[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB525 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91937[] = "JUC (Linux; U; 2.2.2; zh-cn; U8180; 240*320) UCWEB7.9.0.94/139/444";
const char S91938[] = "Mozilla/5.0 (Linux; Android 4.0.3; MTP-710 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91939[] = "Mozilla/5.0 (Linux; Android 4.0.4; NXM805ND_EN Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S91940[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A90S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S91941[] = "Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.6 Safari/535.19 Silk-Accelerated=true";
const char S91942[] = "Mozilla/5.0 (Linux; Android 4.1.1; MOMO11Speed Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S91943[] = "Mozilla/5.0 (Linux; Android 4.0.4; MIDC497 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S91944[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91945[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) MQQBrowser/2.8 Mobile/8C148 Safari/7534.48.3";
const char S91946[] = "Mozilla/5.0 (Linux; Android 4.0.4; O+_8.9_Android Build/OplusOplus_A1000) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S91947[] = "HS-TG88_TD/1.0 Android/2.3 Release/05.31.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S91948[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6352 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91949[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; ST27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S91950[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8530) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S91951[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian70-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S91952[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian70) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S91953[] = "JUC(Linux;U;Android4.0.4;Zh_cn;MT25i;480*854;)UCWEB7.8.0.95/139/444";
const char S91954[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S91955[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7500 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S91956[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Wildfire S A510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91957[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S91958[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; HTC_Incredible_S) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S91959[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-S5301 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S91960[] = "Mozilla/5.0 (Linux; Android 4.1.1; Cynus T2 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91961[] = "Nokia112/2.0 (03.32) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia112/UC Browser8.0.3.107/69/444";
const char S91962[] = "OneBrowser/3.1 (LG-T375)";
const char S91963[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI C8813 Build/HuaweiC8813) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S91964[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E610 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91965[] = "Mozilla/5.0 (Linux; Android 4.0.4; B92 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91966[] = "Mozilla/5.0 (Linux; Android 4.1.2; G2S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S91967[] = "OneBrowser/3.1 (SAMSUNG-GT-S5263/S5263JPKB4)";
const char S91968[] = "OneBrowser/for Blackberry3.0.0 (BlackBerry9320/7.1.0.398)";
const char S91969[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SPH-M930BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S91970[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SPH-M930 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S91971[] = "Mozilla/5.0 (Linux; Android 4.1.2; bq Elcano Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S91972[] = "Mozilla/5.0 (Series40; Nokia3020/14.53; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6";
const char S91973[] = "TINNO50_6464_W1224_11B_HW (MRE\\2.5.00(1536) resolution\\X chipset\\MT6250 touch\\1 tpannel\\1 camera\\1 gsensor\\0 keyboard\\reduced) F5100A_T29TV_X_V12 Release/2012.11.17 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US";
const char S91974[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I717 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000";
const char S91975[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TP10.1-1500DC kb Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S91976[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHL21 Build/S2140) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91977[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT-AS45IPS Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91978[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; IRIS_501) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S91979[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; CONNECT_2G) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S91980[] = "Mozilla/5.0 (Linux; Android 4.2.1; ZP810 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91981[] = "Mozilla/5.0 (Linux; Android 4.0.4; ThL W3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91982[] = "Mozilla/5.0 (Linux; Android 4.0.4; NexusHD2 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91983[] = "Mozilla/5.0 (Linux; Android 4.1.1; M032 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91984[] = "Mozilla/5.0 (Linux; Android 3.2.1; HTC Flyer P510e Build/HTK75C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S91985[] = "Mozilla/5.0 (Linux; Android 3.2; GT-P6210 Build/HTJ85B) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S91986[] = "Mozilla/5.0 (Linux; Android 4.0.4; IdeaTab_A1107 Build/MR1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S91987[] = "Mozilla/5.0 (Linux; Android 4.2.1; Q700 Build/XOLOQ700) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91988[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Next8P12 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444";
const char S91989[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-US; GT-P6200 Build/HTJ85B) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S91990[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A850K Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91991[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A850L Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91992[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5021E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91993[] = "Mozilla/5.0 (Linux; Android 4.2.2; Incredible 4G LTE Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91994[] = "Mozilla/5.0 (Linux; Android 4.1.1; BS471 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91995[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-D700 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S91996[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon";
const char S91997[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S91998[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; Q-Smart S21 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S91999[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI A199 Build/HuaweiA199) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S92000[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; HTC Desire X Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S92001[] = "Spider/2.3.0 CFNetwork/609.1.4 Darwin/13.0.0";
const char S92002[] = "Spider/2.4.3 CFNetwork/609.1.4 Darwin/13.0.0";
const char S92003[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A21 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.289";
const char S92004[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I497 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S92005[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A90S) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S92006[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; C1604 Build/11.1.A.0.68) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S92007[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Aqua Wonder Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S92008[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328d Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S92009[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I8530 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S92010[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; IRIS_501 Build/LAVAIRIS501) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S92011[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5830) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S92012[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A9+) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S92013[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.2.13) Gecko/X Epic/1.2 Firefox/3.6.10";
const char S92014[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6812) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S92015[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; A9+ Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S92016[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HTC Rhyme S510b Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S92017[] = "Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61647";
const char S92018[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; moii E996 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92019[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-S6312 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S92020[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; TECNO_N7 Build/TECNOTECNO_N7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/352";
const char S92021[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:22.0) Gecko/X Firefox/22.0 TenFourFox/7450";
const char S92022[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE Blade III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S92023[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE BLADE III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S92024[] = "Mozilla/5.0 (Linux; Android 3.2.1; G100W Build/HTK55D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S92025[] = "Mozilla/5.0 (Linux; Android 4.1.2; N861 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S92026[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8262) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S92027[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.1; en-in; Micromax A110Q Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S92028[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E610 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S92029[] = "Mozilla/5.0 (Linux; Android 4.0.4; KPN Smart 300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S92030[] = "Mozilla/5.0 (Linux; Android 4.0.4; Philips W536 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S92031[] = "Mozilla/5.0 (Linux; Android 4.0.4; Miracle Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S92032[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ADR6425LVW Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/vigor";
const char S92033[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; C1504) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S92034[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A12) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S92035[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; M040 Build/JRO03H) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S92036[] = "Mozilla/5.0 (Linux; Android 4.0.4; NABI2-NV7A Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S92037[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; QMobile_A700 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92038[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Spice Mi-535 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S92039[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9220 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S92040[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.20.1364.172 Mobile Safari/537.22";
const char S92041[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9152 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S92042[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; SCH-I959 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S92043[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_985N) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S92044[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; IRIS_501 Build/LAVAIRIS501) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S92045[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC_Explorer_A310e Build/GRJ90) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S92046[] = "Mozilla/5.0 (Linux; Android 4.0.4; TECNO_N7 Build/TECNOTECNO_N7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S92047[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S92048[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S92049[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC5-06) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S92050[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; XT875 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/targa";
const char S92051[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; XT875 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/targa";
const char S92052[] = "Mozilla/5.0 (Linux; Android 4.2.1; Q700 Build/XOLOQ700) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S92053[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9A405 Safari/7534.48.3";
const char S92054[] = "Mozilla/5.0 (Linux; Android 4.0.3; AN10CG3 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S92055[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; A111 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S92056[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOLO_X1000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S92057[] = "Mozilla/5.0 (Linux; Android 4.3; XT907 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S92058[] = "Nokia2700c-2/2.0 (09.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia2700c-2) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S92059[] = "Mozilla/5.0 (Linux; Android 4.0.4; LGL86C Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S92060[] = "Mozilla/5.0 (Linux; Android 4.1.1; Arc Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S92061[] = "HTC_P6500 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S92062[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC P6500; OpVer 27.157.1.614";
const char S92063[] = "BlackBerry7130/4.1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/120";
const char S92064[] = "MOT-EM35/R6743_G_74.12.01ROX Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MotoWebKit/417.19 (KHTML, like Gecko) BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/9.0";
const char S92065[] = "SAMSUNG-SGH-P960/P960ASHE1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S92066[] = "SAMSUNG-SGH-A117/A117UCGH2 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S92067[] = "Nokia7900/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S92068[] = "HTC_Touch_HD_T8282 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S92069[] = "SAMSUNG-GT-S5503/ SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S92070[] = "HTC-8500/1.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S92071[] = "SEC-SGHX830/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S92072[] = "SPICE M5161n";
const char S92073[] = "SPICE M5161n, SPICE M5161n";
const char S92074[] = "LGE-LG260/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S92075[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_F900";
const char S92076[] = "LG/BL42/v10a Browser/Teleca-Q7.3 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S92077[] = "LG-BL42k/v10b Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S92078[] = "SEC-SGHA127/A127UCGG5 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S92079[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; LG-P355 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S92080[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en; LG-P355 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S92081[] = "SAMSUNG-SGH-D880/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S92082[] = "MOT-CHARL/00.62 UP.Browser/6.2.3.4.c.1.124 (GUI) MMP/2.0";
const char S92083[] = "SAMSUNG-SGH-A701/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S92084[] = "SAGEM-myX5-2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.6.d.3 (GUI) MMP/1.0";
const char S92085[] = "SAGEM-myX5-2v/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.3.3.g.2.103 (GUI) MMP/2.0";
const char S92086[] = "SAGEM-myX5-2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.6.d.3.100 (GUI) MMP/1.0";
const char S92087[] = "KDDI-HI3B UP.Browser/6.2.0.13.2 (GUI) MMP/2.0";
const char S92088[] = "DoCoMo/2.0 F04C(c500;TB;W30H20)";
const char S92089[] = "KDDI-PT36 UP.Browser/6.2.0.17.2.4 (GUI) MMP/2.0";
const char S92090[] = "AppleWebKit/532.7 (KHTML, like Gecko) Chrome/4.0.271.1 Safari/532.7 TencentTraveler/5.0 QQBrowser/5.0.6642.400 (webkit)";
const char S92091[] = "Chrome/4";
const char S92092[] = "Mozilla/5.0 AppleWebKit/532.8 Chrome/4.0.288.0";
const char S92093[] = "Opera/9.80 (Windows NT 6.1; Opera Mobi/14316; U; en) Presto/2.7.81 Version/11.00";
const char S92094[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Iron/9.0.600.2 Chrome/9.0.600.2 Safari/534.13";
const char S92095[] = "Nokia6303classic/2.0 (09.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303classic/UC Browser8.0.3.107/69/352";
const char S92096[] = "MQQBrowser/2.1 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S92097[] = "MQQBrowser/2.1 Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Safari/7534.48.3";
const char S92098[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; Racer Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92099[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; ZTE Racer Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92100[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Racer Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92101[] = "Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.124 Safari/534.30";
const char S92102[] = "Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.91 Safari/534.30";
const char S92103[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9788; zh-CN) AppleWebKit/534.8+ (KHTML, like Gecko) Version/6.0.0.673 Mobile Safari/534.8+";
const char S92104[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Flock/3.0.0.3737 Chrome/4.1.249.1071 Safari/532.5";
const char S92105[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Flock/3.0.0.3989 Chrome/5.0.375.75 Safari/533.4";
const char S92106[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.13) Gecko/X Firefox/3.6.13 ( .NET CLR 3.5.30729; .NET4.0C) slurp";
const char S92107[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.17) Gecko/X Firefox/3.6.17 slurp";
const char S92108[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.13) Gecko/X Firefox/3.6.13 slurp";
const char S92109[] = "JUC(Linux;U;2.3.3;Zh_cn;HTC Desire S;480*800;)UCWEB7.6.0.75/139/800";
const char S92110[] = "JUC(Linux;U;2.3.3;Zh_cn;HTC Desire S;480*800;)UCWEB7.6.0.75/139/32154";
const char S92111[] = "JUC(Linux;U;2.3.5;Zh_cn;HTC Desire S;480*800;)UCWEB7.6.0.75/139/32154";
const char S92112[] = "samsung-gt-s8530/UC Browser7.9.0.102/69/352";
const char S92113[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; WT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S92114[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PG06100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92115[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PG06100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92116[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:8.0.1) Gecko/X Firefox/8.0.1";
const char S92117[] = "Mozilla/5.0 (PlayStation Vita 1.60) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2";
const char S92118[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.2.16pre) Gecko/X Prism zdesktop/7.1.4";
const char S92119[] = "SAMSUNGSCHI405/2.3.6-EAS-1.2";
const char S92120[] = "Opera/9.80 (Android 3.0.1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.7.81 Version/11.00";
const char S92121[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaN97)/UC Browser8.4.0.159/50/352/UCWEB Mobile";
const char S92122[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:13.0) Gecko/X Thunderbird/13.0.1";
const char S92123[] = "Mozilla/5.0 (Java; U; en-us; nokiac6-01) UCBrowser8.3.0.154/69/352/UCWEB Mobile  3gpp-gba";
const char S92124[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s5333) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S92125[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; id-id; E139 Build/I700T_P6_20_AC76_FTN_R_0_04_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92126[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-R760 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92127[] = "Opera/6.10 (Windows NT 5.1; U; pl) Presto/2.1.15 Version/10.10";
const char S92128[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5230) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S92129[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-s5230) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S92130[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5230) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S92131[] = "UCWEB/2.0(Java; U; MIDP-2.0; fa; gt-s5230) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S92132[] = "UCWEB/2.0(Java; U; MIDP-2.0; ru; gt-s5230) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S92133[] = "UCWEB/2.0(Java; U; MIDP-2.0; es-la; gt-s5230) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S92134[] = "UCWEB/2.0(Java; U; MIDP-2.0; fa; gt-s5230) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S92135[] = "UCWEB/2.0(Java; U; MIDP-2.0; vi; gt-s5230) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S92136[] = "Mozilla/5.0 (Linux; Android 4.0.4; IT-M809RC Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S92137[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC T328w Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92138[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A52 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92139[] = "Mozilla/5.0 (Java; U; id; tcl-c616) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S92140[] = "NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax3-02) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S92141[] = "Mozilla/5.0 (Linux; Android 4.0.4; T1Ci Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S92142[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-B5512 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92143[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S92144[] = "Mozilla/5.0 (Linux; Android 4.0.4; DaVinci Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S92145[] = "Mozilla/5.0 (Linux; Android 4.0.3; AUXUS AX01 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92146[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7510 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92147[] = "Mozilla/5.0 (Linux; Android 4.0.4; SH-06D Build/S9030) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92148[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S92149[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; th-th; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S92150[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S92151[] = "OneBrowser/3.1 (SAMSUNG GT-C3303)";
const char S92152[] = "Mozilla/5.0 (X11; Linux i686 on x86_64; rv:2.0.1) Gecko/X Firefox/4.0.1 Fennec/2.0.1";
const char S92153[] = "Mozilla/5.0 (Linux; Android 4.0.4; Woxter Tablet PC 85 HDS Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S92154[] = "JUC (Linux; U; 2.3.6; zh-cn; A2; 320*480) UCWEB7.9.0.94/139/444";
const char S92155[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Sensation Z710e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S92156[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax P275 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S92157[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:19.0) Gecko/X Thunderbird/19.0";
const char S92158[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Chacha A810 Build/ASN01; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92159[] = "Microsoft Internet Explorer; MSIE 7.3.2; Windows NT 5.4; Mozilla Network 2.0 XP Professional WOW64";
const char S92160[] = "Mozilla/5.0 (Linux; Android 4.0.4; i9400 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92161[] = "Mozilla/5.0 (Linux; Android 4.1.1; Genio Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S92162[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB520 Build/GWK74; CyanogenMod-72) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92163[] = "Mozilla/5.0 (Windows NT 4.0) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S92164[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Nokia)";
const char S92165[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; LG-E405 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S92166[] = "SonyEricssonW595/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonw595) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S92167[] = "Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB A972BK Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S92168[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC Desire Build/AOKP VJ 2.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92169[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; A7* Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92170[] = "NOKIA-Lumia 900/UCWEB2.8.0.249/47/355";
const char S92171[] = "Mozilla/5.0 (Linux; Android 4.0.4; 201M Build/7.7.1Q-164_SMJ-165) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S92172[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TM-MID755 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92173[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5770D Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S92174[] = "UCWEB/2.0(Symbian; U; S60 V5; en-US; SonyEricssonU8i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S92175[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST17i Build/4.1.B.0.587) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S92176[] = "Opera/9.80 (Android android4.0; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S92177[] = "Opera/9.80 (Android v2.2; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10";
const char S92178[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-03D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S92179[] = "Mozilla/5.0 (Linux; Android 4.0.4; Y3 Build/HCL) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S92180[] = "Mozilla/5.0 (Linux; Android 4.0.4; A6 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S92181[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T959 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S92182[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT270 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S92183[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8150 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92184[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S92185[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT15i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S92186[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAC-97032 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S92187[] = "Mozilla/5.0 (Linux; Android 4.0.4; TECNO_N7 Build/TECNOTECNO_N7) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92188[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP7100D3G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S92189[] = "Mozilla/5.0 (Linux; Android 4.0.4; Thunder S220 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92190[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-E970 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92191[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; MK16i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S92192[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST27i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S92193[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; C6602 Build/10.1.A.1.434) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S92194[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Celkon_A88) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S92195[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I9003) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S92196[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; A13-MID Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S92197[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A13-MID Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S92198[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) Nokia5235";
const char S92199[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST25i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S92200[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; SGH-T989 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S92201[] = "Mozilla/5.0 (Linux; Android 4.1.2; bq Elcano Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92202[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; HTC Desire X) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S92203[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; HTC Desire X) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S92204[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAC-97032 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S92205[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB462 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S92206[] = "Mozilla/5.0 (Linux; Android 4.1.2; SBM203SH Build/S0018) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92207[] = "Mozilla/5.0 (Linux; Android 4.1.1; U7015 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92208[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 97 TITANIUMHD Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S92209[] = "Mozilla/5.0 (Linux; Android 4.2.2; HUAWEI P6-U06 Build/HuaweiP6-U06) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92210[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PG86100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (PG86100, 42, 1.9.1)";
const char S92211[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-S5300 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92212[] = "LG-A447/V100 Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S92213[] = "Mozilla/5.0 (Linux; Android 4.1.1; tablet Fnac 8 Build/1.1.0 X-12:19) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S92214[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; A97i Build/CelkonA97i_Celkon) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S92215[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo A789 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S92216[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A89 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S92217[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP7170B3G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S92218[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-N7102 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92219[] = "Mozilla/5.0 (Linux; Android 4.1.2; iris405 Build/LAVAIRIS405) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92220[] = "Mozilla/5.0 (Linux; Android 4.1.2; ME371MG Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S92221[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S92222[] = "Mozilla/5.0 (Linux; Android 4.1.2; NookColor Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S92223[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A800S Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92224[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92225[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-04D Build/9.1.C.0.475) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92226[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-E400 Build/IMM76L)";
const char S92227[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-E400 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92228[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC PH39100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (HTC PH39100, 42, 1.9.1)";
const char S92229[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-B5330) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S92230[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC One V Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S92231[] = "Mozilla/5.0 (Linux; Android 4.2.1; iDxD10 3G Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S92232[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LG-P765 Build/IMM76L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S92233[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; LifeStyle 10DCIPS(BO-LF10DCI) Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92234[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A62) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S92235[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:22.0) Gecko/X Firefox/22.0 slurp";
const char S92236[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-US; HTC Desire Build/FRG83G) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S92237[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-US; HTC Desire Build/FRF91) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S92238[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-in; MT25i Build/4.1.B.0.631) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S92239[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A7*) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S92240[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; WT19i) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S92241[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-i9220 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92242[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9220 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92243[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZTE Blade III Pro Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92244[] = "UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; GT-I9100) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S92245[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P930 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S92246[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LG730 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S92247[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One dual 802d Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S92248[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC 802d Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S92249[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; GT-I9100) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S92250[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5830 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S92251[] = "Mozilla/5.0 (Linux; Android 4.3; BN NookHD+ Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S92252[] = "OneBrowser/3.1 (Nokia500/111.021.0028/sw_platform=S60;sw_platform_version=5.3;java_build_version=2.3.24)";
const char S92253[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9070) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S92254[] = "SAMSUNG-GT-S3332/S3332JVMD1 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S92255[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.2; en-us; SGH-M919 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S92256[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL ONE TOUCH 918N) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S92257[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL ONE TOUCH 918N) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S92258[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI G520-5000 Build/HuaweiG520-5000) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S92259[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae72-1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba";
const char S92260[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.2.1.0 Chrome/16.0.912.75 Safari/535.7";
const char S92261[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.2.1.0 Chrome/16.0.912.75 Safari/535.7";
const char S92262[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S5303) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S92263[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9103 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/i9103";
const char S92264[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN82) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S92265[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; A54) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S92266[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARNOVA 90G3 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S92267[] = "Mozilla/5.0 (Linux; Android 4.1.1; AMI-TT4 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S92268[] = "Mozilla/5.0 (Series40; Nokia208/03.39; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.55";
const char S92269[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5670) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S92270[] = "Mozilla/5.0 (Linux; Android 4.2.2; SM-T311 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S92271[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S92272[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E460 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S92273[] = "Nokia6136/2.0 (04.01) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S92274[] = "LG-GW520/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S92275[] = "ZTE-G N281/WAP2.0 Profile/MIDP-2.0";
const char S92276[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch2_T3335; Windows Phone 6.5)";
const char S92277[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-vn; V8401 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S92278[] = "LG-KP215 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S92279[] = "MOT-SPARK/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0";
const char S92280[] = "LG-GM310/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S92281[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.4.0.65/70/352";
const char S92282[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.4.0.65/27/351";
const char S92283[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.4.0.65/50/355";
const char S92284[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.4.0.65/28/352";
const char S92285[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.4.0.65/50/351";
const char S92286[] = "SAMSUNG-SGH-A657/A657UCIB5 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S92287[] = "SonyEricssonJ108i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S92288[] = "Mozilla/5.0 (Series40; NokiaC3-00/07.20; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.3.23.12";
const char S92289[] = "ZTE-R230/2.0 Profile/MIDP-2.0";
const char S92290[] = "DoCoMo/2.0 SH04A(c100;TB;W24H16)";
const char S92291[] = "NokiaN8-00/UC Browser7.6.1.82/50/355";
const char S92292[] = "nokian8-00/UC Browser7.6.1.82/69/351  3gpp-gba";
const char S92293[] = "NokiaN97/UC Browser7.7.1.88/50/444";
const char S92294[] = "nokian97-1/UC Browser7.7.1.88/69/444";
const char S92295[] = "KDDI-SN3E UP.Browser/6.2.0.13.2 (GUI) MMP/2.0";
const char S92296[] = "LG/U900/v1.0";
const char S92297[] = "KDDI-TS3Q UP.Browser/6.2_7.2.7.1.K.4.306 (GUI) MMP/2.0";
const char S92298[] = "Opera/9.80 (Android; Linux; Opera Mobi/49; U; en) Presto/2.4.18 Version/10.00";
const char S92299[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; Gemei G2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92300[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; Gemei G2 Build/GRJ22)";
const char S92301[] = "Opera/9.50 (Linux armv5tejl ; U; en)";
const char S92302[] = "Nokia5800 XpressMusic/UC Browser7.0.0.41/50/351";
const char S92303[] = "HTC_Touch_3G_T3232 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S92304[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:6.0) Gecko/X Firefox/6.0";
const char S92305[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:6.0.2) Gecko/X Firefox/6.0.2";
const char S92306[] = "gt-s5233s/UC Browser7.9.0.102/69/444";
const char S92307[] = "gt-s5233s/UC Browser7.9.0.102/69/351";
const char S92308[] = "NOKIANokia N81/UC Browser8.0.3.107/28/351/UCWEB";
const char S92309[] = "NOKIANokia N81 8GB/UC Browser8.0.4.121/28/352/UCWEB";
const char S92310[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.2.1.144/69/352";
const char S92311[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.2.1.144/69/444";
const char S92312[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.2.0.132/69/452";
const char S92313[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.2.1.144/69/405";
const char S92314[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.2.1.144/69/444  3gpp-gba";
const char S92315[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.2.0.132/70/352";
const char S92316[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.2.1.144/70/352";
const char S92317[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:8.0) Gecko/X Thunderbird/8.0 Lightning/1.0";
const char S92318[] = "nokian73-1/UC Browser8.0.3.107/69/352";
const char S92319[] = "nokiae71-1/UC Browser7.9.0.102/69/352";
const char S92320[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.0.1) Gecko/X Prism (zdesktop/1.0.4)";
const char S92321[] = "JUC(Linux;U;2.2.2;Zh_cn;ZTE-C N880S;480*800;)UCWEB7.6.0.75/139/32026";
const char S92322[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaC7-00)/UC Browser8.4.0.159/50/352/UCWEB Mobile";
const char S92323[] = "SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonj20i) UCBrowser8.3.0.154/69/352/UCWEB Mobile";
const char S92324[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC One X Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92325[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC One X Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92326[] = "Firefox/8.0";
const char S92327[] = "Mozilla/5.0 (SaferSurf rv:8.0) Gecko/X Firefox/8.0";
const char S92328[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S92329[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S92330[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Micromax A85 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92331[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A45 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S92332[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92333[] = "JUC(Linux;U;Android3.2;Zh_cn;GT-P6800;800*1232;)UCWEB7.8.0.95/139/352";
const char S92334[] = "Mozilla/5.0(Linux;U;Android3.2;Zh_cn;GT-P6800;800*1232;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S92335[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-uk; GEM8012 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92336[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Glacier Build/GRJ22)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92337[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S92338[] = "SonyEricssonJ10i2/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonj10i2) UCBrowser8.3.0.154/69/352/UCWEB Mobile";
const char S92339[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID2 Global Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92340[] = "Mozilla/5.0 (Windows) Gecko Firefox/12. -CareMediCom-";
const char S92341[] = "NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/69/444";
const char S92342[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaE7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile";
const char S92343[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S92344[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S92345[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; en-in; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92346[] = "Mozilla/5.0 (Linux; Android 4.1.1; Tablet 723B Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S92347[] = "Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Kem 350 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92348[] = "Mozilla/5.0 (Linux; Android 4.1.2; myTouch_4G_Slide Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92349[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Legend Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92350[] = "TE1303 Mozilla/5.0 (Windows; U; Windows NT 5.1; en; like Gecko) Obigo/Q7 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S92351[] = "Mozilla/5.0 (Linux; Android 4.1.2; Rezound Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92352[] = "Videocon V1548_Maui Wap Browser/3.83; Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S92353[] = "JUC (Linux; U; 3.2; zh-cn; GT-P6800; 800*1232) UCWEB7.9.0.94/139/352";
const char S92354[] = "Mozilla/5.0 (Linux; Android 4.0.3; GOCLEVER NETBOOK R103 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S92355[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CT704 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92356[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE-U V889F Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92357[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaE6-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/352/UCWEB Mobile";
const char S92358[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S92359[] = "Mozilla/5.0 (Linux; Android 4.1.1; H7100 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S92360[] = "YahooMobileMail/1.0 (Android Mail; 1.4.6) (GT-P7300;samsung;GT-P7300;3.1/HMJ37)";
const char S92361[] = "YahooMobileMail/1.0 (Android Mail; 1.4.5) (espresso10rf;samsung;GT-P5100;4.0.3/IML74K)";
const char S92362[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P6210 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92363[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P6210 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92364[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-P1000M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000";
const char S92365[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000";
const char S92366[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000";
const char S92367[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; GT-P1000L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000";
const char S92368[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; th-th; GT-P1000T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000";
const char S92369[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-P6800 Build/JOP40D)";
const char S92370[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-p6800 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92371[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; E15i Build/JRO03L; CyanogenMod-MiniCM10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92372[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; E15i Build/JZO54K; CyanogenMod-MiniCM10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92373[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; A7710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92374[] = "Mozilla/5.0 (X11; FreeBSD amd64) KHTML/4.9.3 (like Gecko) Konqueror/4.9";
const char S92375[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S92376[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A10 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S92377[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lava-a10) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S92378[] = "Mozilla/5.0 (Linux; Android 4.0.4; Enjoy 71 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S92379[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; MI-ONE C1 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S92380[] = "Mozilla/5.0 (Linux; Android 4.1.1; ST10 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S92381[] = "Mozilla/5.0 (Windows; U; ; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.1 Safari/533.3";
const char S92382[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7568 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S92383[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ADR6425LVW Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92384[] = "Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 6 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92385[] = "Mozilla/5.0 (Linux; Android 4.1.1; Smartfren Tab 7 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S92386[] = "Mozilla/5.0 (Linux; Android 4.0.4; i9500 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92387[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; eZee'Tab 971 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92388[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S92389[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A820L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92390[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A800S Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92391[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S92392[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; POV_TAB-P825 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92393[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T959V Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92394[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax_P275) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S92395[] = "Mozilla/5.0 (Linux; Android 4.0.4; i9500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92396[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaN8-00";
const char S92397[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N8000 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S92398[] = "Mozilla/5.0 (Linux; Android 4.0.4; A5Classic Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92399[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; R800x Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92400[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Informer 708 3G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92401[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Informer 708 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92402[] = "JUC (Linux; U; 4.0.4; zh-cn; LT26w; 720*1280) UCWEB7.9.0.94/139/352";
const char S92403[] = "Opera/9.80 (Android 4.2.1; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.10.254 Version/12.00";
const char S92404[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT30p Build/JZO54K; CyanogenMod-CM10 base (X unofficial build by quarx, compiled by sevenrock)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92405[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Xperia T Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92406[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; M9 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31";
const char S92407[] = "Mozilla/5.0 (Linux; Android 4.1.2; Q-Smart S21 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92408[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.5.3.X (Second Life Release; default skin) Safari/533.3";
const char S92409[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5670) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S92410[] = "UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; X10i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S92411[] = "Mozilla/5.0 (Linux; Android 4.0.3; KFOT Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/19.77.34.5 Safari/537.22";
const char S92412[] = "Mozilla/5.0 (Linux; Android 4.0.4; HS-U950 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92413[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Cherry w500 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92414[] = "Mozilla/5.0 (Linux; Android 4.2.1; W100 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92415[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; S7100 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33745";
const char S92416[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Wynncom G32 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92417[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S92418[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; HTC Glacier Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S92419[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk800i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S92420[] = "UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; X10i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S92421[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI U8950D Build/HuaweiU8950D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92422[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Xperia Play Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92423[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; ARNOVA 90G3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92424[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-us; Micromax A72 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S92425[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; Q-Smart S53 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92426[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE50) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S92427[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.4m";
const char S92428[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.5m";
const char S92429[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S92430[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR910L Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S92431[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9001 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S92432[] = "Mozilla/5.0 (Linux; Android 4.0.3; M729 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S92433[] = "Mozilla/5.0 (Linux; Android 4.2.2; SHL22 Build/S6290) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S92434[] = "Mozilla/5.0 (Linux; Android 4.1.2; 3G7334i Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S92435[] = "Mozilla/5.0 (Linux; Android 4.0.4; HYUNDAI T10 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S92436[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT15i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S92437[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Xperia Live with Walkman Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92438[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; EPAD Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1/tf201";
const char S92439[] = "[FBAN/FB4A;FBAV/3.5;FBBV/X;FBDM/{density=3.0,width=1080,height=1920};FBLC/en_US;FBCR/T-Mobile;FBPN/com.facebook.katana;FBDV/SGH-M919;FBSV/4.2.2;]";
const char S92440[] = "[FBAN/FB4A;FBAV/3.6.1;FBBV/X;FBDM/{density=3.0,width=1080,height=1920};FBLC/en_US;FBCR/T-Mobile;FBPN/com.facebook.katana;FBDV/SGH-M919;FBSV/4.2.2;FBCA/armeabi-v7a:armeabi;]";
const char S92441[] = "NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac1-02) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S92442[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac7-00) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba";
const char S92443[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; HTC_Wildfire) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S92444[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID0714PGE01.133 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92445[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F100S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S92446[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-P765 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S92447[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; A5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92448[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-B5330) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S92449[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST25i) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S92450[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-MS770 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S92451[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT915 Build/2_32A_2013) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92452[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT915 Build/2_32A_2008) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92453[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A111 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S92454[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; LG-E405) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S92455[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaE6-00) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S92456[] = "SAMSUNG-SGH-T289/T289UVME3 SHP/VPP/R5 Dolfin/1.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S92457[] = "Mozilla/5.0 (Linux; Android 4.1.2; FPT V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S92458[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S92459[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire C Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S92460[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOLO A700 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S92461[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT26i Build/6.1.A.2.55) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S92462[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-R740C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S92463[] = "Mozilla/5.0 (Vodafone/1.0/LG-GT350/V10a Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S92464[] = "Opera/9.50 (Windows NT 5.1; MDA_Vario_V/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1; U; de)";
const char S92465[] = "SAMSUNG-SGH-B270i/B270iBGID1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S92466[] = "SCH-U540/1.0 NetFront/3.0.22.1.5 (GUI) MMP/2.0";
const char S92467[] = "SCH-U540/1.0 NetFront/3.0.22.2.5 (GUI) MMP/2.0";
const char S92468[] = "SAMSUNG-GT-S5530/S5530XXJG8 SHP/VPP/R5 Dolfin/2.0 QTV/5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B";
const char S92469[] = "nokian72/UC Browser7.0.0.41/69/351";
const char S92470[] = "NokiaN72/5.0819.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.0.0.41/27/352";
const char S92471[] = "Mozilla/4.08 (740SC;SoftBank;SN354715035737340) NetFront/3.3";
const char S92472[] = "Mozilla/4.08 (740SC;SoftBank) NetFront/3.3";
const char S92473[] = "Mozilla/4.08 (740SC;SoftBank;SN354715035305338) NetFront/3.3";
const char S92474[] = "Mozilla/4.08 (740SC;SoftBank;SN354715035629794) NetFront/3.3";
const char S92475[] = "Mozilla/4.08 (740SC;SoftBank;SN354715034163506) NetFront/3.3";
const char S92476[] = "SoftBank/1.0/740SC/SCJ001 Browser/NetFront/3.3";
const char S92477[] = "SPICE-M6464/1.0 IAC/M6235 IAC/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S92478[] = "SPICE-M6464/1.0 IAC/M6235 IAC/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1, SPICE-M6464/1.0 IAC/M6235 IAC/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S92479[] = "SAMSUNG-SGH-Z560/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S92480[] = "SAMSUNG-SGH-Z560-Vodafone/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S92481[] = "SAMSUNG-S3600/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S92482[] = "NokiaE52-1/UCWEB7.9.0.94/28/999";
const char S92483[] = "NokiaE52-1/UC Browser7.9.0.102/28/444/UCWEB";
const char S92484[] = "LG-A255/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S92485[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.9) VZW:SCH-i760 PPC 240x320";
const char S92486[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A293 Twitter for iPhone";
const char S92487[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A400 Twitter for iPhone";
const char S92488[] = "NOKIA6120c/UC Browser7.7.1.88/28/444";
const char S92489[] = "SonyEricssonX2/R3AA Browser/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S92490[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Bottle of Smoke Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S92491[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)/UC Browser7.4.0.65";
const char S92492[] = "480x800; XV6875.1; Window Mobile 6.5 Professional; (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65";
const char S92493[] = "MicromaxX262/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S92494[] = "SAMSUNG-SGH-T959V/100.20306";
const char S92495[] = "MacOutlook/14.0.0.X (Intel Mac OS X 10.6.8)";
const char S92496[] = "Mozilla/5.0 (Windows NT 5.0; rv:6.0.2) Gecko/X Firefox/6.0.2,gzip(gfe) (via translate.google.com)";
const char S92497[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.2.16pre) Gecko/X Prism zdesktop/7.1.4";
const char S92498[] = "SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonj20i/UC Browser7.9.0.102/69/352";
const char S92499[] = "Mozilla/5.0 (Series40; NokiaX2-01/07.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.4.0.34.8";
const char S92500[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; hu-hu; xTAB-100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92501[] = "nokiac6-00/UC Browser7.9.0.102/69/352  3gpp-gba";
const char S92502[] = "Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU5)/UC Browser8.4.0.159/50/352/UCWEB Mobile";
const char S92503[] = "SonyEricssonW595/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonw595/UC Browser7.9.0.102/69/352";
const char S92504[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Salsa_C510b; en-au) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S92505[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A65 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92506[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE90)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S92507[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S92508[] = "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:12.1) Gecko/X Firefox/12.1-x64 PaleMoon/12.1-x64";
const char S92509[] = "Mozilla/5.0 (Windows NT 6.2; rv:12.1) Gecko/X Firefox/12.1 PaleMoon/12.1";
const char S92510[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; ZTE-BLADE Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S92511[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.60 Safari/537.1 Sleipnir/3.6.1";
const char S92512[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A1000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92513[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MA1000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92514[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MP747 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92515[] = "MQQBrowser/2.7 (NokiaX6-00;SymbianOS/9.1 Series60/3.0)";
const char S92516[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN86 8MP)/UC Browser8.5.0.183/28/444/UCWEB Mobile";
const char S92517[] = "Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia5310xpressmusic) UCBrowser8.2.0.132/69/452/UCWEB Mobile";
const char S92518[] = "iBrowser/2.6 (NokiaC5-03;SymbianOS/9.1 Series60/3.0)";
const char S92519[] = "MQQBrowser/26 (Linux; U; 2.3.6; zh-cn; HUAWEI C8650+ Build/C8650+V100R001C92B875;320*480)";
const char S92520[] = "MQQBrowser/26 (Linux; U; 2.3.6; zh-cn; HUAWEI C8650+ Build/C8650+V100R001C92B866SP01;320*480)";
const char S92521[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA6700s)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S92522[] = "Mozilla/5.0 (X11; OpenBSD amd64; rv:15.0) Gecko/X Firefox/15.0.1";
const char S92523[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9.2.4) Gecko/X FireDownload/2.0.1 Firefox/3.6.4 Wyzo/3.6.4.1 FireTorrent/2.0.1 SearchToolbar/1.2";
const char S92524[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03R; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92525[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92526[] = "samsung-gt-s5560i/UC Browser7.9.0.102/69/352";
const char S92527[] = "Mozilla/5.0 (X11; U; Linux i686; qcv; rv:1.9.0.5) Gecko/X Ubuntu/7.10 (gutsy) Firefox/3.0.5";
const char S92528[] = "JUC (Linux; U; 2.3.7; zh-cn; HTC_HD2; 480*800) UCWEB7.9.3.103/139/800";
const char S92529[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_HD2; 480*800) UCWEB7.9.0.94/139/352";
const char S92530[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_HD2; 480*800) UCWEB7.9.0.94/139/444";
const char S92531[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Iconia A501 Build/IMM76L; CyanogenMod-Tegraowners ICS ROM v170 (thor & digetx)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92532[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-SU660 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92533[] = "Mozilla/5.0 (Linux; Android 4.1.2; htc_jewel Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92534[] = "Mozilla/5.0 (Linux; Android 4.1.1; htc_jewel Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92535[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaE6-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile";
const char S92536[] = "MicromaxX650 ASTRO36_TD/v3 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10";
const char S92537[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT912 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S92538[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC One V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92539[] = "Mozilla/5.0 (Linux; Android 4.0.4; MAGPAD9.7Pro Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S92540[] = "Mozilla/5.0 (Linux; Android 4.0.3; CELKON CT2 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92541[] = "Mozilla/5.0 (Linux; Android 4.0.4; TOUCH90W Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92542[] = "Mozilla/5.0 (Linux; Android 4.0.4; P5-Mini Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92543[] = "Mozilla/5.0 (Linux; U; Android 3.2; ru-ru; V9S_V1.4) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S92544[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; X10i Build/2.1.A.0.435) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S92545[] = "Mozilla/5.0 (Linux; Android 4.0.4; Magic Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92546[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; AraTop AS-09 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92547[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8160 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92548[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; Explay Star Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92549[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S92550[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S92551[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC EVO 3D X515m Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S92552[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; N400 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S92553[] = "Mozilla/5.0 (Linux; Android 4.1.2; Amaze 4G Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S92554[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS GAMEPAD Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S92555[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus S 4G Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S92556[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92557[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92558[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.7.12) Gecko/X Firefox/1.0.7";
const char S92559[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID1042 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S92560[] = "JUC (Linux; U; 2.3.6; zh-cn; Karbonn_A2; 480*800) UCWEB7.9.0.94/139/444";
const char S92561[] = "Mozilla/5.0 (Linux; Android 4.1.1; OV-BaseCore7 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S92562[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LG-E615 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S92563[] = "Mozilla/5.0 (Java; U; en-us; nokia701) UCBrowser8.3.1.161/70/352/UCWEB Mobile  3gpp-gba";
const char S92564[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5660 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S92565[] = "Nokia308/2.0 (05.85) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia308) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S92566[] = "Mozilla/5.0 (Linux; Android 4.0.4; Starmobile ASTRA Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92567[] = "Mozilla/5.0 (Linux; Android 4.0.3; AN7HG3 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S92568[] = "Mozilla/5.0 (Linux; Android 4.0.4; L-01E Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92569[] = "Mozilla/5.0 (Linux; Android 4.0.3; X9017 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S92570[] = "UCWEB/2.0 (Windows; U; wds 7.10; en-US; SAMSUNG; GT-I8350) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile";
const char S92571[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT894 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S92572[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-L300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92573[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5263) U2/1.0.0 UCBrowser/9.0.0.295 U2/1.0.0 Mobile";
const char S92574[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I897 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92575[] = "eBayAndroid/2.1.1.0";
const char S92576[] = "eBayAndroid/2.1.3.1";
const char S92577[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_Sensation_XE_with_Beats_Audio_Z715e) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S92578[] = "[FBAN/FB4A;FBAV/2.3;FBBV/X;FBDM/{density=1.5,width=480,height=800};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/SPH-D710;FBSV/4.0.4;]";
const char S92579[] = "[FBAN/FB4A;FBAV/3.1;FBBV/X;FBDM/{density=1.5,width=480,height=800};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/SPH-D710;FBSV/4.0.4;]";
const char S92580[] = "UCWEB/2.0 (iOS; U; iPh OS 4_3_3; zh-CN; iPh3,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile";
const char S92581[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A850L Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S92582[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A850K Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S92583[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A9+ Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S92584[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5238) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S92585[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-P930 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92586[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP7100D3G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S92587[] = "Mozilla/5.0 (Linux; Android 4.1.1; ST8 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S92588[] = "Mozilla/5.0 (Linux; Android 4.1.2; MB526 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92589[] = "Mozilla/5.0 (Linux; Android 4.1.2; IQ451 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92590[] = "Mozilla/5.0 (Linux; Android 4.0.3; LIFETAB_P9514 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S92591[] = "Mozilla/5.0 (Linux; Android 4.0.4; Miracle Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92592[] = "Nokia306/2.0 (03.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia306) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S92593[] = "OneBrowser/for Blackberry3.0.0 (BlackBerry9300/6.0.0.668)";
const char S92594[] = "OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92595[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-P765 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S92596[] = "SAMSUNG-SGH-M919/101.40202";
const char S92597[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-I9103 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S92598[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; X907 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S92599[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T769 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S92600[] = "Mozilla/5.0 (Linux; Android 4.0.4; AZ210A Build/AZ210A_ICS_02.34I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92601[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; Android2.x) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S92602[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9260 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S92603[] = "Instagram 4.0.2 (iPod4,1; iPhone OS 6_1_3; en_US; en) AppleWebKit/420+";
const char S92604[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IQ442 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92605[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; s4502M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92606[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; LG-P500 Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S92607[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S92608[] = "Mozilla/5.0 (Linux; Android 4.2.2; F-06E Build/V17R45B) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92609[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire V) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S92610[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL ONE TOUCH 918N) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S92611[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S92612[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MT25i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S92613[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:17.0) Gecko/X Firefox/17.0";
const char S92614[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; Mobiistar Touch Bean 452 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92615[] = "MQQBrowser/43 (iOS 5; U; CPU like Mac OS X; zh-cn)";
const char S92616[] = "MQQBrowser/43 (iOS 4S; U; CPU like Mac OS X; zh-cn)";
const char S92617[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.2; en-us; SCH-I545 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S92618[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; MID9742 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S92619[] = "Mozilla/5.0 (Linux; Android 4.1.2; ADR910L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S92620[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; ru-ru; Fly IQ4410 Quad Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92621[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; MID8042 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S92622[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S92623[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN85) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S92624[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nook HD+ Build/JDQ39E; Carbon-CARBON-JB-UNOFFICIAL-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92625[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A45) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S92626[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E987 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S92627[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LRA Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S92628[] = "Mozilla/5.0 (Linux; Android 4.0.4; Optimus_Madrid Build/IceCreamSandwich) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S92629[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HTC Rhyme S510b Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S92630[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Nexus S Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S92631[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 slurp";
const char S92632[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; SPH-M930BST Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S92633[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5210 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S92634[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311";
const char S92635[] = "Mozilla/5.0 (Linux; Android 4.2.2; Evo 3D CDMA Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S92636[] = "Mozilla/5.0 (Linux; Android 4.1.2; PAP5300DUO Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92637[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC Wildfire S A510e) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S92638[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC Wildfire S A510e) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S92639[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ph; Cloudfone Excite 501d Build/SKV10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92640[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; TECNO D5 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S92641[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Connect-2G-2.0 Build/HCL+ME+Tablet+V2) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S92642[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; W125 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S92643[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6280) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile TRUSTED/1.0";
const char S92644[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST27i Build/6.1.1.B.1.10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S92645[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA5320) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S92646[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I337M Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltecan";
const char S92647[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I337M Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltecan";
const char S92648[] = "Mozilla/5.0 (Linux; Android 4.0.4; Magic Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S92649[] = "Mozilla/5.0 (Linux; Android 4.1.1; IRIS 455 Build/LAVAIRIS455) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S92650[] = "Mozilla/5.0 (Linux; Android 4.2.2; MB886 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S92651[] = "UCWEB/2.0 (Linux; U; Adr 4.2.2; zh-CN; GT-I9500) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S92652[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S6012 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S92653[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST21i Build/11.0.A.4.22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S92654[] = "Mozilla/5.0 (Linux; Android 4.2.2; V370 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S92655[] = "Mozilla/5.0 (Linux; Android 4.1.1; IdeaTabA2109A Build/JRO03R) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S92656[] = "SAMSUNG-SGH-C417 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S92657[] = "SAMSUNG-SGH-C417/C417UCFK3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S92658[] = "SAMSUNG-SGH-E830/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S92659[] = "Nokia6275/2.0 (BL290V0100.nep) UP.Browser/6.2.3.8 MMP/2.0";
const char S92660[] = "MOT-V3s/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0";
const char S92661[] = "Xda orbit; 240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S92662[] = "Samsung-SPHM510 AU-OBIGO/Q04C1-1.22 MMP/2.0";
const char S92663[] = "SAMSUNG-SGH-i900/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S92664[] = "Mozilla/4.1 (compatible; MSIE 6.0; Brew 3.1.5; U; en)/320x240 NOKIA 7705";
const char S92665[] = "SEC-SGHD520/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S92666[] = "LG_P350/1.0 Android/2.2 Release/5.20.2010 Browser/KHTML (Mozilla/5.0 AppleWebKit/533.1 Version/4.0 Mobile Safari/533.1)";
const char S92667[] = "PANTECH-C610/IRUS08252008 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S92668[] = "PANTECH-C610/IRUSX Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S92669[] = "Samsung-SPHM500 AU-MIC-M500/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S92670[] = "DoCoMo/2.0 F04B(c500;TB;W24H16)";
const char S92671[] = "NokiaC5-03/UC Browser7.8.0.95/50/351";
const char S92672[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.0.1.0 Chrome/14.0.835.163 Safari/535.1";
const char S92673[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.0.1.0 Chrome/14.0.835.163 Safari/535.1";
const char S92674[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.0.0.0 Chrome/13.0.782.107 Safari/535.1";
const char S92675[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.0.0.0 Chrome/13.0.782.107 Safari/535.1";
const char S92676[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.7.2) Gecko/X Netscape/7.2 (ax;nscd1)";
const char S92677[] = "NokiaC3-00/5.0 (08.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac3-00/UC Browser7.7.1.88/69/444";
const char S92678[] = "nokian8-00/UC Browser7.5.1.77/69/351  3gpp-gba";
const char S92679[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92680[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S92681[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-S5670 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S92682[] = "Facebook 4110.0 (iPod touch; iPhone OS 5.1; en_US)";
const char S92683[] = "Facebook 4110.0 (iPod touch; iPhone OS 5.1.1; en_US)";
const char S92684[] = "Instagram 2.5.1 (iPod touch; iPhone OS 5.1.1; en_US) AppleWebKit/420+";
const char S92685[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) 1Password/3.6.5/X (like Mobile/8C148 Safari/6533.18.5)";
const char S92686[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.3) Gecko/X Ubuntu/8.10 (intrepid) Firefox/3.6.3";
const char S92687[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9a1) Gecko/X Firefox/1.6a1";
const char S92688[] = "Instacast/2.1 (like iTunes/10.1.2)";
const char S92689[] = "Opera/9.80 (Windows NT 4.0; U; de) Presto/2.7.62 Version/11.00";
const char S92690[] = "Mozilla/5.0 (Windows; U; Windows NT 4.0; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/1.0.154.53 Safari/525.19";
const char S92691[] = "Mozilla/5.0 (Series40; Nokia2730c-1/09.41; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.4.0.34.8";
const char S92692[] = "SAMSUNG-GT-S7250/1.0 Bada/2.0 Dolfin/2.0 SMM-MMS/1.2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S92693[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; ko-kr; LG-V901 Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S92694[] = "Mozilla/5.0 (Linux; U; Android 1.0.0; es-us; E700R Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S92695[] = "HUAWEIU8520/1.0 Linux/2.6.35 Android/2.2.2 Release/5.26.2011 Browser/WebKit533.1";
const char S92696[] = "nokian95_8gb/UC Browser7.9.0.102/69/352";
const char S92697[] = "nokian95/UC Browser7.9.0.102/70/352";
const char S92698[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (DROID X2, 34, 1.6.0)";
const char S92699[] = "NOKIA-Lumia 900/UCWEB2.6.0.213/47/32079";
const char S92700[] = "Mozilla/5.0 (S60V5; U; en-in; NokiaE6-00)/UC Browser8.2.0.132/50/444/UCWEB Mobile";
const char S92701[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HS-T96 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S92702[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Sensation Z710e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S92703[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Sensation 4G Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S92704[] = "Nokia200/2.0 (11.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.1.161/69/355";
const char S92705[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2a1pre) Gecko/X Minefield/3.6a1pre";
const char S92706[] = "Mozilla/5.0 (Series40; NokiaC1-02/03.40; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.5.16.11";
const char S92707[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-nz; VIGO10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92708[] = "Nokia3500c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia3500c) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S92709[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; Micromax A60 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S92710[] = "Mozilla/5.0 (Linux; Android 4.0.4; JMI T970 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S92711[] = "Mozilla/5.0 (Linux; Android 4.1.2; Ascend G300 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92712[] = "Mozilla/5.0 (S60V5; U; en-us; SamsungI8910) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S92713[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT15i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S92714[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; micromaxq5) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S92715[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; micromax) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S92716[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92717[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92718[] = "JUC (Linux; U; 2.2; zh-cn; GT-I5510; 240*400) UCWEB7.9.0.94/139/355";
const char S92719[] = "Mozilla/5.0 (Linux; Android 4.0.4; N-06D Build/AX) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S92720[] = "Mozilla/5.0 (Linux; Android 4.1.2; Vivid 4G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92721[] = "Mozilla/5.0 (Linux; Android 4.0.4; AUXUS CoreX4 3G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S92722[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9220 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S92723[] = "Mozilla/5.0 (Linux; Android 4.0.4; INFINITY_LOTUS Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92724[] = "Mozilla/5.0 (Linux; Android 4.0.4; IS12S Build/6.1.D.1.91) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S92725[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nb-no; 726EP Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92726[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; Nexus S Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S92727[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ607 Build/7.7.1-122-4-FLEM-WIFI-LA) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S92728[] = "Mozilla/5.0 (Linux; Android 4.0.4; Optimus_Madrid Build/IceCreamSandwich) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S92729[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.16/276,1.9.16.276,2012-12-21 16:37, +0000";
const char S92730[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P5113, 42, 1.9.1)";
const char S92731[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P6200 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92732[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7300 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S92733[] = "Mozilla/5.0 (Linux; Android 4.1.1; ST18 Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92734[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xperia Ray Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92735[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; M040 Build/JRO03H) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S92736[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S92737[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S92738[] = "Mozilla/5.0 (Linux; Android 4.0.4; Titan TV S320 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S92739[] = "Mozilla/5.0 (Linux; Android 4.0.3; PAD703 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S92740[] = "Mozilla/5.0 (Linux; Android 4.0.4; SO-05D Build/7.0.D.1.137) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92741[] = "Mozilla/5.0 (Linux; Android 4.0.4; Optimus_Madrid Build/IceCreamSandwich) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92742[] = "Mozilla/5.0 (Linux; Android 4.0.4; A5Classic Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92743[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; SGH-T989 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S92744[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S92745[] = "Mozilla/5.0 (Linux; Android 4.0.4; GSmart GS202+ Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92746[] = "Mozilla/5.0 (Linux; Android 4.1.2; Dream_D1 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92747[] = "Mozilla/5.0 (Linux; Android 4.0.4; HYUNDAI T7S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S92748[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5503 Build/10.1.1.A.1.310) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92749[] = "Mozilla/5.0 (Linux; Android 4.1.1; I7100 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S92750[] = "Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_E10310 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S92751[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; NetTAB SPACE III Build/NetTAB SPACE III) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92752[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A100) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S92753[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-F160K Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92754[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F160L Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92755[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9308 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S92756[] = "Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_E7310 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S92757[] = "Mozilla/5.0 (Linux; Android 4.2.1; E2001 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92758[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Aqua_Wonder) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S92759[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; R800i Build/4.0.2.A.0.42) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S92760[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-p6800 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S92761[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S92762[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT-AS40SE Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92763[] = "Mozilla/5.0 (Linux; Android 4.0.3; KUNO3 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S92764[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A810S Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92765[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A810K Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92766[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S92767[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT18i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S92768[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; 5860E Build/GWK74) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S92769[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S92770[] = "Mozilla/5.0 (Linux; Android 4.0.4; myTouch_4G_Slide Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92771[] = "Mozilla/5.0 (Linux; Android 4.0.4; N10 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S92772[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire V Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S92773[] = "Mozilla/5.0 (Linux; Android 4.2.1; H7100 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92774[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; SCH-I699 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S92775[] = "Mozilla/5.0 (Linux; Android 4.2.1; AMOI N828 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92776[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T959 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92777[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile i-STYLE Q2 DUO Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92778[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile i-STYLE Q2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92779[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; ThL W6 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92780[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; hr-hr; MOVE Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92781[] = "Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 10 FHD Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S92782[] = "Mozilla/5.0 (Windows) Gecko/X Firefox/17.05 -CareMediCom-";
const char S92783[] = "Mozilla/5.0 (Windows) Gecko/X Firefox/17.08 -CareMediCom-";
const char S92784[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Spice_Mi-502) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S92785[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C5155 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S92786[] = "Mozilla/5.0 (Linux; Android 4.2.1; i-mobile IQ XA Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92787[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MIDC901 Build/MIDC901) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92788[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S92789[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax A91 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S92790[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A21) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S92791[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; C1604 Build/11.3.A.2.1) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S92792[] = "Mozilla/5.0 (Linux; Android 2.3.6; U8860 Build/HuaweiU8860) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453";
const char S92793[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC One V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S92794[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZTE N9120 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S92795[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; IM-A760S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S92796[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.3; en-in; HTC Desire C Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S92797[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A7* Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S92798[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-in; Micromax A110Q Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300";
const char S92799[] = "Mozilla/5.0 (Linux; Android 4.0.4; A5Classic Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S92800[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire U dual sim Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S92801[] = "Mozilla/5.0 (Linux; Android 4.1.1; IRIS 455 Build/LAVAIRIS455) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S92802[] = "Mozilla/5.0 (Linux; Android 4.3; SGH-M919 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S92803[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; Gpad_G2 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33886";
const char S92804[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A770K Build/->Wangsai_008) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92805[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A770K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92806[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-L38C Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S92807[] = "[FBAN/FB4A;FBAV/3.5;FBBV/X;FBDM/{density=3.0,width=1080,height=1920};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/SPH-L720;FBSV/4.2.2;]";
const char S92808[] = "[FBAN/FB4A;FBAV/3.6.1;FBBV/X;FBDM/{density=3.0,width=1080,height=1920};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/SPH-L720;FBSV/4.2.2;FBCA/armeabi-v7a:armeabi;]";
const char S92809[] = "[FBAN/FB4A;FBAV/3.5;FBBV/X;FBDM/{density=1.5,width=540,height=888};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/XT907;FBSV/4.1.2;]";
const char S92810[] = "[FBAN/FB4A;FBAV/3.6.1;FBBV/X;FBDM/{density=1.5,width=540,height=888};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/XT907;FBSV/4.1.2;FBCA/armeabi-v7a:armeabi;]";
const char S92811[] = "Mozilla/5.0 (Linux; Android 4.2.1; ME173X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S92812[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; xTAB-100dci-3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92813[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; DTR97 Build/xTAB-100 QCR) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92814[] = "Mozilla/5.0 (Linux; Android 4.2.2; Amaze 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S92815[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC_T329w Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92816[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC T329w Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92817[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-P715 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S92818[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Micromax A110) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S92819[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia501) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S92820[] = "Mozilla/5.0 (Linux; Android 4.1.2; SM-T211 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S92821[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9152 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S92822[] = "HTC-8100/1.2 Mozilla/4.0 (compatible; MSIE 5.5; Windows CE; PPC; 240x320)";
const char S92823[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) SAMSUNG-SGH-i640V/BUGI4";
const char S92824[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) SAMSUNG-SGH-i640V/BUGK1";
const char S92825[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) SAMSUNG-SGH-i640V/BUGJ2";
const char S92826[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; MDA_compact_IV/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S92827[] = "SonyEricssonZ800/R1X Browser/SEMC-Browser/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S92828[] = "SonyEricssonZ800i/R1X Browser/SEMC-Browser/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S92829[] = "MOT-E398/0E.20.99R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S92830[] = "Nokia5611/2.0 (09.70) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S92831[] = "SAMSUNG-GT-C3110C/1.0 Release/9.05.2008 Browser/Openwave6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S92832[] = "SAMSUNG-C3110/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S92833[] = "SAMSUNG-GT-C3110C/1.0 Release/9.05.2008 Browser/Openwave6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S92834[] = "UTSTARCOM-CDM7126/1.0 UP.Browser/6.3.0.7 (GUI) MMP/2.0";
const char S92835[] = "SonyEricssonW995/R1HA SEMC/NewsReader/1.0";
const char S92836[] = "LG/U960/v1.0";
const char S92837[] = "SAMSUNG-SGH-i908/1.0 Release/03.07 Browser/IE6 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S92838[] = "Mozilla/4.08 (821SC;SoftBank) NetFront/3.3";
const char S92839[] = "SoftBank/1.0/821SC/SCJ001 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S92840[] = "NokiaE72-1/UC Browser7.8.0.95/28/355";
const char S92841[] = "KDDI-KC3R UP.Browser/6.2_7.2.7.1.K.4.310 (GUI) MMP/2.0";
const char S92842[] = "TIANYU-KTOUCH/H999/MIDP2.0/CLDC1.1/Screen-320X240";
const char S92843[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/69/352";
const char S92844[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/70/352";
const char S92845[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/69/352  3gpp-gba";
const char S92846[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13";
const char S92847[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S92848[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S92849[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S92850[] = "Safari/6533.19.4 CFNetwork/454.11.5 Darwin/10.6.0 (i386) (MacBook7%2C1)";
const char S92851[] = "Safari/6533.19.4 CFNetwork/454.11.5 Darwin/10.6.0 (i386) (MacBookPro7%2C1)";
const char S92852[] = "Safari/6533.19.4 CFNetwork/454.11.5 Darwin/10.6.0 (i386) (iMac11%2C2)";
const char S92853[] = "Safari/6533.19.4 CFNetwork/454.11.5 Darwin/10.6.0 (i386) (MacBookAir3%2C1)";
const char S92854[] = "HTC_Touch2_T3320 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S92855[] = "Nokia5250/UC Browser7.9.0.102/50/355/UCWEB";
const char S92856[] = "Opera/9.00 (Windows 98; U; en)";
const char S92857[] = "Opera/9.02 (Windows 98; U; en)";
const char S92858[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:8.0) Gecko/X Thunderbird/8.0";
const char S92859[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S92860[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; MID Build/MID) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7";
const char S92861[] = "NokiaC2-03/2.0 (06.51) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-03/UC Browser7.7.1.88/69/444";
const char S92862[] = "SonyEricssonW910i/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonw910i/UC Browser8.0.3.107/69/352";
const char S92863[] = "SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonw20i/UC Browser8.0.3.107/69/444";
const char S92864[] = "Mozilla/5.0 (SaferSurf rv:11.0) Gecko/X Firefox/11.0";
const char S92865[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98) Opera 7.50 [ru\"]";
const char S92866[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-N7000 Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92867[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S92868[] = "Opera/9.80 (Android 3.0.1; Linux; Opera Tablet/ADR-X; U; en-GB) Presto/2.9.201 Version/11.50";
const char S92869[] = "NokiaC2-06/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-06/UC Browser8.0.3.107/69/444";
const char S92870[] = "Nokia303/5.0 (13.44) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia303/UC Browser8.0.3.107/70/355";
const char S92871[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S92872[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S92873[] = "Mozilla/5.0 (Java; U; en-in; nokia6233) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S92874[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92875[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S92876[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; ST15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S92877[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC One S Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92878[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC One S Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92879[] = "JUC(Linux;U;Android2.3.4;Zh_cn;MT15i;480*854;)UCWEB7.8.0.95/139/444";
const char S92880[] = "MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A780 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92881[] = "Mozilla/5.0 (Linux; U; Android 2.3; pl-pl; MID07 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92882[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s3653w) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S92883[] = "NOKIA-Lumia 710/UCWEB2.5.0.202/47/32079";
const char S92884[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5";
const char S92885[] = "MQQBrowser/Mini2.8 (NokiaC2-00/03.82)";
const char S92886[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-style Q3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92887[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5233a) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S92888[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5233a) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S92889[] = "UCWEB/2.0(Java; U; MIDP-2.0; fa; gt-s5233a) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S92890[] = "UCWEB/2.0(Java; U; MIDP-2.0; fa; gt-s5233a) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S92891[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92892[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Mobile Safari/535.19 Silk-Accelerated=true";
const char S92893[] = "Mozilla/5.0 (Linux; Android 4.0.4; SBM102SH Build/S1010) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92894[] = "MQQBrowser/3.2 (Nokia5233;SymbianOS/9.1 Series60/3.0)";
const char S92895[] = "Mozilla/5.0 (Linux; Android 4.0.4; N81 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S92896[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/6.7.29879/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S92897[] = "nokian8-00/UC Browser7.2.2.51/70/352  3gpp-gba";
const char S92898[] = "MQQBrowser/25 (Linux; U; 2.3.6; zh-cn; ZTE N880E Build/N880EV1.0.0B06;480*800)";
const char S92899[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; GT-I5801 Build/ECLAIR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S92900[] = "NOKIA-Lumia 900/UCWEB2.7.0.223/47/355";
const char S92901[] = "Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:17.0) Gecko/X Firefox/17.0 AdometryBot";
const char S92902[] = "Opera/9.80 (Macintosh; PPC Mac OS X 10.5.8; U; en) Presto/2.6.31 Version/10.70";
const char S92903[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxx510) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S92904[] = "Mozilla/5.0 (Linux; Android 4.0.4; N7100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92905[] = "NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiax2-05) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S92906[] = "Mozilla/5.0 (Linux; Android 4.0.3; M-270 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92907[] = "Mozilla/5.0 (Linux; Android 4.0.4; Fly_IQ450 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92908[] = "Mozilla/5.0 (Series40; NokiaX3-02/leCEX92_MP2.2.1_05.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.3.23.12";
const char S92909[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC Desire X Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S92910[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 5A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92911[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9001 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S92912[] = "Mozilla/5.0 (Linux; Android 4.0.4; PTL21 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92913[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25";
const char S92914[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A403 Safari/8536.25";
const char S92915[] = "Mozilla/5.0 (Linux; Android 4.0.4; i9430 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S92916[] = "Mozilla/5.0 (Linux; Android 4.0.3; Xtreamer_Mobile_AiKi Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S92917[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:20.0) Gecko/X Thunderbird/20.0";
const char S92918[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; J-Q88B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92919[] = "Android/1.0 (GT-P7510 HMJ37)";
const char S92920[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Opera/9.80 (Android; Opera Mini/6.5.27431/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S92921[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Opera/9.80 (Android; Opera Mini/6.5.27431/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S92922[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; RMD-755 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92923[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; X907 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S92924[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST27i Build/6.1.1.B.1.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S92925[] = "Mozilla/5.0 (Linux; Android 4.0.3; PMP3270B Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S92926[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Rhyme S510b Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S92927[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9070 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S92928[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAC-97032 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S92929[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S92930[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; T720 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92931[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S92932[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S92933[] = "Mozilla/5.0 (Linux; Android 4.1.1; N70-S Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S92934[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A500 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S92935[] = "Mozilla/5.0 (Linux; Android 4.1.2; Desire HD Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92936[] = "Mozilla/5.0 (Linux; Android 4.0.4; ThL W3 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92937[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 slurp";
const char S92938[] = "Mozilla/5.0 (Linux; Android 4.0.3; E860 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92939[] = "Mozilla/5.0 (Linux; Android 4.1.1; Limo X2+ Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92940[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:21.0) Gecko/X Thunderbird/21.0";
const char S92941[] = "OneBrowser/3.1 (SAMSUNG GT-S3600i)";
const char S92942[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_Sensation_XL_with_Beats_Audio_X315e) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S92943[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; KATBL10A16E Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92944[] = "Mozilla/5.0 (Linux; Android 4.0.3; F-05D Build/V08R31C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S92945[] = "Mozilla/5.0 (Linux; Android 4.2.2; A700 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S92946[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; C1604) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S92947[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Funtab Build/ICS.g12refM806A1SPISY.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92948[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Lenovo_A60+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S92949[] = "Mozilla/5.0 (iPad; U; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25 XiaoMi/MiuiBrowser/1.0";
const char S92950[] = "Mozilla/5.0 (Linux; U; Android 4.0; ru-ru; TM-7025 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92951[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; TM-7025 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92952[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S92953[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; HTC Inspire 4G Build/JOP40D; CyanogenMod-BlueBot-V3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92954[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3 Janetter/1.5.1";
const char S92955[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Safari/7534.48.3 Janetter/1.5.2";
const char S92956[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-7240 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S92957[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; GT-5660 Build/GRWK74; CyanogenMod-7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92958[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; XOLO_A700) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S92959[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M032 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S92960[] = "Mozilla/5.0 (Linux; Android 4.0.3; AN7HG3 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S92961[] = "Mozilla/5.0 (Linux; Android 4.0.3; Newman N2 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92962[] = "Mozilla/5.0 (Linux; Android 4.2.2; NookColor Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S92963[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; MT25i Build/4.1.B.0.631) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S92964[] = "Mozilla/5.0 (Linux; Android 4.0.3; AN10CG3 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S92965[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 97B TITANIUM Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S92966[] = "Mozilla/5.0 (Java; U; en-us; j2me) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/352/UCWEB Mobile";
const char S92967[] = "Mozilla/5.0 (Java; U; en-us; spice) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/352/UCWEB Mobile";
const char S92968[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GPLUS N930 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92969[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GPLUS N930 Build/IMM76D)";
const char S92970[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko, M5WebKit/1.0.1, M5HybridSDK/1.0.0) Version/5.1.0 Mobile/5.1.1 Mammoth/5.1.0";
const char S92971[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko, M5WebKit/1.0.2, M5HybridSDK/1.0.0) Version/5.1.1 Mobile/5.1.1 Mammoth/5.1.1";
const char S92972[] = "SonyEricssonT715/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssont715) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S92973[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-S5360 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S92974[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; HTC Ruby Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S92975[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; GT-I5503 Build/ERE27) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S92976[] = "Mozilla/5.0 (Linux; Android 4.1.2; AT-AS43D3 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92977[] = "Mozilla/5.0 (Linux; Android 4.1.1; CT920 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S92978[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax A91 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92979[] = "Mozilla/5.0 (Linux; Android 4.2.2; VS920 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92980[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-L710 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92981[] = "Mozilla/5.0 (Linux; Android 4.1.2; L-01E Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92982[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT30a Build/7.0.A.3.195) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92983[] = "Mozilla/5.0 (Linux; Android 4.0.3; E860 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92984[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Desire Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92985[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Tablet P1801-T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S92986[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw660i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S92987[] = "SonyEricssonC702/R3EK Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonc702) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S92988[] = "Mozilla/5.0 (Linux; Android 4.2.2; MB886 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S92989[] = "Mozilla/5.0 (Linux; Android 4.0.4; F-05E Build/V16R39A) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S92990[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; A111 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S92991[] = "UCWEB/2.0 (Linux; U; Adr 4.2.2; en-US; GT-I9500) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S92992[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8552 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S92993[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S92994[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; folio100 Build/HTJ85B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92995[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; folio100 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S92996[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-E739 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S92997[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; XOLO A1000 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S92998[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-ca; SGH-I337M Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S92999[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8190N Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S93000[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; vivo X1St Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S93001[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; ST26i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S93002[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; SPH-M930BST Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S93003[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9220 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S93004[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:16.0) Gecko/X Firefox/16.0";
const char S93005[] = "Mozilla/5.0 (Linux; Android 4.1.2; Q800 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93006[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S93007[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-in; ST25i Build/6.1.1.B.1.10) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S93008[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ultrafone_303) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S93009[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.1; en-in; Micromax A116 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S93010[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; SCH-I589 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S93011[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; XOLO Q800 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93012[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PG86100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S93013[] = "Mozilla/5.0 (Linux; Android 4.1.1; Orange Kivo Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93014[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D2-727 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S93015[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.2; en-us; SPH-L720 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S93016[] = "LG-GW305/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gw305) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S93017[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; A11 Build/KarbonnA11) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S93018[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Intex Aqua 3.2) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S93019[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac7-00) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile  3gpp-gba";
const char S93020[] = "Mozilla/5.0 (Linux; Android 4.2.1; H7500+ Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93021[] = "Mozilla/5.0 (Linux; Android 4.1.2; Vivid 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93022[] = "Mozilla/5.0 (Linux; Android 4.0.4; Haier HW-W910 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93023[] = "Mozilla/5.0 (Linux; Android 4.1.1; M032 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93024[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOLO_X1000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93025[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae63-1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S93026[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE63) U2/1.0.0 UCBrowser/9.2.0.336 U2/1.0.0 Mobile";
const char S93027[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S93028[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; ZTE U807 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.292 U3/0.8.0 Mobile Safari/534.31";
const char S93029[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; GT-S6012B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93030[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5670 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S93031[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A500) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S93032[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S93033[] = "Nokia109/2.0 (04.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia109) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S93034[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XOLO_X1000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93035[] = "Mozilla/5.0 (Linux; Android 4.3; SCH-I545 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.30 Mobile Safari/537.36";
const char S93036[] = "Mozilla/5.0 (Linux; Android 4.0.4; AD683G Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S93037[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; Micromax_A30) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S93038[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-B5512) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S93039[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; SonyEricssonU5) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S93040[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337M Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S93041[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A65 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S93042[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; Karbonn A6 Build/MocorDroid4.0.3) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S93043[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.1.141) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S93044[] = "Mozilla/5.0 (Linux; Android 4.2.1; Spice Mi-535 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S93045[] = "UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-I9082) U2/1.0.0 UCBrowser/9.3.0.321 Mobile";
const char S93046[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; C5302 Build/12.0.A.1.211) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S93047[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; C5302 Build/12.0.A.2.254) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S93048[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26w Build/6.2.B.1.96) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S93049[] = "SAMSUNG-GT-B7620/1.0 Browser/Opera/9.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93050[] = "Nokia6610/1.0 (5.52) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S93051[] = "ZTE-F952(METFONE)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S93052[] = "ZTE-F952(VMS)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S93053[] = "NexianNX-G889/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S93054[] = "SonyEricssonJ300i/R2BA SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93055[] = "SEC-SGHP310/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93056[] = "DoCoMo/2.0 D905i(c100;TB;W24H17)";
const char S93057[] = "Alcatel-OT-300/1.0 ObigoInternetBrowser/Q03C";
const char S93058[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; V8402 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S93059[] = "SonyEricssonWT18i_TD/1.0 OPhone/2.5 (Linux; Android 2.2) Release/5.23.2011 Browser/WAP2.0 (AppleWebKit/533.1) Mobile Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S93060[] = "SAMSUNG-GT-M7600L/1.0 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S93061[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i917)";
const char S93062[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i917.)";
const char S93063[] = "LG/U250/v1.0";
const char S93064[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; MEGAUPLOAD 1.0) Sleipnir/2.41";
const char S93065[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; GTB6; .NET CLR 1.1.4322; .NET CLR 2.0.50727) Sleipnir/2.49";
const char S93066[] = "Opera/9.80 (Windows 95; U; ru) Presto/2.9.168 Version/11.50";
const char S93067[] = "Mozilla/5.0 (Linux; U; Android 2.3; ru-ru; MID7027 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93068[] = "samsung-gt-s3650/UC Browser8.0.3.107/69/444";
const char S93069[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; en) Opera 8.0";
const char S93070[] = "Mozilla/5.0 (S60V3; U; en-in; NokiaE66)/UC Browser8.4.0.159/28/352/UCWEB Mobile";
const char S93071[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5 Comodo_Dragon/19.2.0.0";
const char S93072[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5 Comodo_Dragon/19.2.0.0";
const char S93073[] = "SonyEricssonW595/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonw595/UC Browser8.0.3.107/69/352";
const char S93074[] = "Nokia200/2.0 (10.58) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia200/UC Browser7.7.1.88/69/444";
const char S93075[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_ChaCha_A810b; en-au) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S93076[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T839 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93077[] = "Nokia303/5.0 (14.38) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia303) UCBrowser8.2.1.144/70/355/UCWEB Mobile";
const char S93078[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P350 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352";
const char S93079[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-MS690 Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93080[] = "Mozilla/5.0 (Linux; U; Android 2.2; en_us ; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S93081[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/352";
const char S93082[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-US; GT-P1000 Build/FROYO) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S93083[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ViewSonic-ViewPad4 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93084[] = "JUC(Linux;U;Android2.3.6;Zh_cn;GT-S7500;320*480;)UCWEB7.8.0.95/139/444";
const char S93085[] = "Mozilla/5.0 (Linux; Android 4.0.3; eZee'Tab971 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S93086[] = "Mozilla/5.0 (Linux; Android 4.0.4; eZee'Tab971 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S93087[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Lenovo A60 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93088[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93089[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_C8810 Build/C8810V100R001C92B862) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93090[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-B5512 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S93091[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s5753e) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S93092[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93093[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93094[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93095[] = "NOKIA-Lumia 610/UCWEB2.5.0.202/47/32079";
const char S93096[] = "Mozilla/5.0 (Linux; U; Android 3.0UI; es-us; Space Livepad 7.2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93097[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/70/352";
const char S93098[] = "SAMSUNG-SGH-I717/100.40004";
const char S93099[] = "Mozilla/5.0 (Java; U; en-us; samsung gt-c3303k) UCBrowser8.3.1.161/70/465/UCWEB Mobile, SAMSUNG-GT-C3303K/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S93100[] = "Mozilla/5.0 (Java; U; en-us; samsung gt-c3303i) UCBrowser8.3.1.161/70/465/UCWEB Mobile, SAMSUNG-GT-C3303i/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S93101[] = "JUC(Linux;U;2.3.4;Zh_cn;LT18i;480*854;)UCWEB7.6.0.75/139/32322";
const char S93102[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT894 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93103[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Mobile Safari/535.19 Silk-Accelerated=false";
const char S93104[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AUXUS AX01 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93105[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:18.0) Gecko/X Thunderbird/18.0";
const char S93106[] = "Mozilla/5.0 (Windows NT 6.0; rv:17.0) Gecko/X Firefox/17.0 slurp";
const char S93107[] = "Instagram 3.1.2 (iPad; iPhone OS 6.0.1; en_US) AppleWebKit/420+";
const char S93108[] = "Instagram 3.1.2 (iPad; iPhone OS 6.0; en_US) AppleWebKit/420+";
const char S93109[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93110[] = "Palm750/v0100 Mozilla/4.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S93111[] = "Palm750/v0005 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S93112[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; FreeTAB Build/RK2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93113[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-ks360 teleca) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S93114[] = "JUC (Linux; U; 4.0.4; zh-cn; Lenovo_A789; 480*800) UCWEB7.9.4.145/139/800";
const char S93115[] = "Mozilla/5.0 (Linux; Android 4.1.1; X507 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93116[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; TAB224 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93117[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ZTPAD ZT-282 (Slatedroid Market Script) Build/ICS.f04ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93118[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ZTPAD ZT-282 Build/ICS.f04ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93119[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; A660 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93120[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; NookColor Build/JZO54K)";
const char S93121[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Xperia S Build/JZO54K)";
const char S93122[] = "Mozilla/5.0 (Linux; Android 4.0.3; SOWA_S7700 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93123[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; D23 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93124[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; TCL_A966 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93125[] = "OneBrowser/3.0 (NokiaC3-00/08.65)";
const char S93126[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us ; Nexus 4 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S93127[] = "Mozilla/5.0 (Linux; Android 4.2.2; ADR6425LVW Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S93128[] = "WhatsApp/2.8.4559 Android/3.1 Device/samsung-GT-P7300";
const char S93129[] = "Instagram 3.4.0 Android (16/4.1.1; 160dpi; 1280x752; samsung; GT-P7300; p5; p3; en_US)";
const char S93130[] = "Instagram 3.4.0 Android (13/3.2; 160dpi; 800x1232; samsung; GT-P7300; GT-P7300; p3; en_GB)";
const char S93131[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B117 Safari Version/5.1";
const char S93132[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/5.1 Mobile/8B117 Safari/7534.48.3";
const char S93133[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; X1 Build/HCL ME Tablet X1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93134[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; AF9190 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93135[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P970 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S93136[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw350i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S93137[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I8150 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S93138[] = "Mozilla/5.0 (Linux; Android 4.0.4; iBall Slide 3G 7334 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S93139[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-03D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93140[] = "Mozilla/5.0 (X11; Linux x86_64; ST21i-o) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S93141[] = "Mozilla/5.0 (Linux; Android 4.0.4; VL-110 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S93142[] = "Mozilla/5.0 (Linux; Android 4.0.4; KYL21 Build/019.0.3b00) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93143[] = "Mozilla/5.0 (Linux; Android 4.0.3; ZT ICS Build/ICS.f04ref.X) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S93144[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Edison 3G Build/1.1.11-1015 X-15:24) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S93145[] = "Mozilla/5.0 (Linux; Android 4.0.3; AN10CG3 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S93146[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T959 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93147[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 70 Titanium Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S93148[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOLO_A800 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93149[] = "Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A30 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93150[] = "Mozilla/5.0 (Linux; Android 4.1.1; GEM10313BK Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S93151[] = "Mozilla/5.0 (Linux; Android 4.0.4; N10 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S93152[] = "Mozilla/5.0 (Linux; Android 4.0.3; ZT ICS Build/ICS.f04ref.X) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S93153[] = "Mozilla/5.0 (Linux; Android 4.0.4; G3 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S93154[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx510) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S93155[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; U8180 Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S93156[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; MT15i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S93157[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A8) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S93158[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; ZTE V880G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93159[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; WT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S93160[] = "Mozilla/5.0 (Linux; Android 4.0.4; GN800 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93161[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.8.0.10) Gecko/X Thunderbird/1.5.0.10";
const char S93162[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; PHCASIATAB8B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93163[] = "Mozilla/5.0 (Linux; Android 4.0.3; A510 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S93164[] = "Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Safari/537.31 OPR/14.0.1074.54070";
const char S93165[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A5Classic) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S93166[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xolo_X500 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S93167[] = "Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5310xpressmusic) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S93168[] = "Mozilla/5.0 (Linux; Android 4.1.2; GL07S Build/HuaweiU9700L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93169[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I927 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93170[] = "Mozilla/5.0 (Linux; Android 4.1.1; tablet Fnac 8 Build/1.1.0 X-12:19) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S93171[] = "Mozilla/5.0 (Linux; Android 4.1.1; Fusion Bolt Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S93172[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; TOOKY A6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93173[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A27 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S93174[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9220 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31";
const char S93175[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI G520-5000 Build/HuaweiG520-5000) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S93176[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI G520-0000 Build/HuaweiG520-0000) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S93177[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One X Build/JDQ39E; Carbon-CARBON-JB-UNOFFICIAL-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93178[] = "SAMSUNG-GT-I9505/101.40202";
const char S93179[] = "Mozilla/5.0 (Linux; Android 2.3.4; ADR6300 Build/GRJ22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S93180[] = "Mozilla/5.0 (Linux; Android 4.0.4; V96A Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S93181[] = "Mozilla/5.0 (Linux; Android 2.3.3; X10i Build/3.0.1.G.0.75) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453";
const char S93182[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T959V Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93183[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S93184[] = "SAMSUNG-GT-S3572 Opera/9.80 (J2ME/MIDP; Opera Mini/7.0.30356/30.3214; U; pt) Presto/2.8.119 Version/11.10";
const char S93185[] = "Mozilla/5.0 (Linux; Android 4.0.4; H9500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93186[] = "Mozilla/5.0 (Linux; Android 4.2.2; MB865 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93187[] = "Mozilla/5.0 (Linux; Android 4.1.2; M-MP5303G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93188[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I699 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S93189[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/11.04 Chromium/22.0.1229.79 Chrome/22.0.1229.79 Safari/537.4";
const char S93190[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8552 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93191[] = "MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI G520-0000 Build/HuaweiG520-0000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93192[] = "Nokia310/2.0 (07.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia310) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S93193[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-R720 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S93194[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I879 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S93195[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8190N Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S93196[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; TECNO_P3 Build/master) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S93197[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A15) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S93198[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian73-1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S93199[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE73) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S93200[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Videocon A15) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S93201[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Micromax A115 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S93202[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-E405 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S93203[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; XOLO A700 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S93204[] = "MQQBrowser/37 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3";
const char S93205[] = "MQQBrowser/371 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3";
const char S93206[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9505 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (GT-I9505, 42, 1.9.1)";
const char S93207[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ASUS Transformer Pad TF700T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);";
const char S93208[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-P715 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S93209[] = "Mozilla/5.0 (Linux; Android 4.0.4; NexusHD2 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36";
const char S93210[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-in; A90S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S93211[] = "Nokia109/2.0 (04.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia109) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S93212[] = "SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; fa; sonyericssonw20i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S93213[] = "Mozilla/5.0 (Linux; Android 4.0.4; FJL21 Build/V37R47A) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93214[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAX5-01) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S93215[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZTE Blade G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93216[] = "Mozilla/5.0 (Linux; Android 4.1.2; Desire HD Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93217[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S93218[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE T81 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93219[] = "Mozilla/5.0 (Linux; Android 4.1.2; IQ446 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93220[] = "UCWEB/2.0 (Java; U; MIDP-2.0; es-LA; samsung-gt-s5260) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S93221[] = "Mozilla/5.0 (Linux; Android 4.3; Transformer TF101 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S93222[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.10; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)";
const char S93223[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.10; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)";
const char S93224[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-B5330 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S93225[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S93226[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; N9500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93227[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia 909)";
const char S93228[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Spice Mi-502 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S93229[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7000) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S93230[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6812) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S93231[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT18i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S93232[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0000 Build/HuaweiY300-0000) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93233[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; MT27i Build/6.2.A.0.400) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93234[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S93235[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S93236[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; INTEX_CLOUD_X1) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S93237[] = "Mozilla/5.0 (Linux; Android 4.2.1; A10 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93238[] = "Mozilla/5.0 (Linux; Android 4.1.2; Vivid 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S93239[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A870S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93240[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A870K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93241[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; R815 Build/JOP40D)";
const char S93242[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; th-th; R815 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93243[] = "Nokia201/2.0 (11.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311";
const char S93244[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTCEVOV4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S93245[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax P362 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S93246[] = "Mozilla/5.0 (Linux; Android 4.2.2; A1-810 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S93247[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S93248[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S93249[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i2) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S93250[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S93251[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT30at Build/9.1.F.1.120) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S93252[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9192 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S93253[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 10 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S93254[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) HTC_Snap_S510";
const char S93255[] = "ZTE-G-X670-Lisbon-orange/X670_V1_OG_Z2_FRES_DFFA107 Obigo/Q03C";
const char S93256[] = "ZTE-G-X670-Lisbon-orange/X670_V1_OG_Z2_FRES_DFFA110 Obigo/Q03C";
const char S93257[] = "ZTE-G-X670-Lisboa-orange/X670_V1_OG_Z3_ESFR_DFFA109 Obigo/Q03C";
const char S93258[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 640x480; HTC_X7500/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1) Opera 8.65 [fr]";
const char S93259[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 640x480; HTC_X7500/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1) Opera 8.65 [en]";
const char S93260[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 640x480; HTC_X7500/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1) Opera 8.65 [nl]";
const char S93261[] = "Alcatel-OT-600/1.0 ObigoInternetBrowser/Q03C";
const char S93262[] = "PANTECH-C810/M044.33 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S93263[] = "PANTECH-C810/CB.WM01 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S93264[] = "PANTECH-C810/CT.WM02 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S93265[] = "PANTECH-C810/M043.32 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S93266[] = "SCH-U650/1.0 NetFront/3.0.22.2.15 (GUI) MMP/2.0";
const char S93267[] = "SEC-SGHB310/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S93268[] = "SEC-SGHB300-ORANGE/B300AFHG1 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S93269[] = "SEC-SGHC516/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S93270[] = "SEC-SGHC506/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S93271[] = "LGE-PM225/1.0 UP.Browser/6.2.3.7.e.1.101 (GUI) MMP/2.0";
const char S93272[] = "Samsung-SPHM800 AU-OBIGO/Q05A-3.13 MMP/2.0";
const char S93273[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG-SCH-i225";
const char S93274[] = "Spice QT68";
const char S93275[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; ru-ru; MTC 916 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93276[] = "SAMSUNG-SGH-X700/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S93277[] = "LG-KE770 MIC/1.1.14 MIDP-2.0/CLDC-1.1";
const char S93278[] = "ZTE-C70/1.0 SMIT-Browser/2.0.0";
const char S93279[] = "DoCoMo/2.0 P04A(c100;TB;W24H15)";
const char S93280[] = "Nokia6680/1.0 (5.04.40) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.7.1.88/27/444";
const char S93281[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; HTC click Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93282[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; HTC click Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93283[] = "NokiaC6-01/UC Browser7.7.1.88/50/444";
const char S93284[] = "nokiac6-01/UC Browser7.7.1.88/69/444  3gpp-gba";
const char S93285[] = "KDDI-MA33 UP.Browser/6.2.0.13.2 (GUI) MMP/2.0";
const char S93286[] = "KDDI-SN3M UP.Browser/6.2.0.14.1.2 (GUI) MMP/2.0";
const char S93287[] = "NexianNX-G912/MTK Release/27.11.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S93288[] = "DoCoMo/2.0 P01C(c500;TB;W24H16)";
const char S93289[] = "MicromaxX410/MIDP2.0/CLDC1.1/Screen-240X320";
const char S93290[] = "GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.0.6002.2);en-US;kml:2.2;client:Free;type:default)";
const char S93291[] = "GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.0.6002.2);en;kml:2.2;client:Free;type:default)";
const char S93292[] = "GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.0.6002.2);en-GB;kml:2.2;client:Free;type:default)";
const char S93293[] = "GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.0.6001.1);en-US;kml:2.2;client:Free;type:default)";
const char S93294[] = "GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.0.6002.2);de;kml:2.2;client:Free;type:default)";
const char S93295[] = "GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.0.6002.2);de-DE;kml:2.2;client:Free;type:default)";
const char S93296[] = "DoCoMo/2.0 N04A(c100;TB;W24H16)";
const char S93297[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0) chromeframe/9.0.597.94";
const char S93298[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0) chromeframe/9.0.597.107";
const char S93299[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.107 Safari/534.13 chromeframe/9.0.597.107";
const char S93300[] = "MQQBrowser/2.1 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3";
const char S93301[] = "MQQBrowser/2.1 Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3";
const char S93302[] = "samsung-gt-s5333/UC Browser7.9.0.102/69/352";
const char S93303[] = "Mozilla/5.0 (PRS-950; U; Linux; en-us) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2 PRS-950/2.0";
const char S93304[] = "MicromaxQ55/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S93305[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; SAMSUNG; OMNIA7)";
const char S93306[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; SAMSUNG; OMNIA7; Orange)";
const char S93307[] = "SAMSUNG-GT-B5722/B5722JPJJ1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S93308[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:11.0) Gecko/X Thunderbird/11.0.1";
const char S93309[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; MT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S93310[] = "Mozilla/5.0 (Windows NT 5.0; rv:11.0) Gecko/X Thunderbird/11.0.1";
const char S93311[] = "Nokia5610d-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5610d-1/UC Browser8.0.3.107/69/444";
const char S93312[] = "Mozilla/5.0 (Series40; Nokia5130c-2/07.91; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.4.0.34.8";
const char S93313[] = "J2ME/UCWEB7.3.1.56/139/999";
const char S93314[] = "Opera/7.0 (Windows 2000; U) [en]";
const char S93315[] = "Mozilla/5.0 (Windows 2000; U) Opera 7.0 [en]";
const char S93316[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.7.5) Gecko/X Netscape/8.1.3";
const char S93317[] = "Mozilla/5.0 (Java; U; en-us; gt-s5230) UCBrowser8.2.0.132/69/352/UCWEB Mobile";
const char S93318[] = "gt-s5230/UCWEB8.2.0.116/69/352";
const char S93319[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5250)/UC Browser8.4.0.159/50/352/UCWEB Mobile";
const char S93320[] = "Mozilla/5.0 (Windows NT 6.0; rv:5.0) Gecko/X Thunderbird/5.0 Lightning/1.0b4";
const char S93321[] = "Opera/9.80 (Android 3.0.1; Linux; Opera Tablet/ADR-X; U; en-GB) Presto/2.8.149 Version/11.10";
const char S93322[] = "Mozilla/5.0 (Windows NT 6.0; rv:8.0) Gecko/X Firefox/8.0 slurp";
const char S93323[] = "samsung-gt-s7233e/UC Browser8.0.3.107/69/444";
const char S93324[] = "SonyEricssonC902/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonc902/UC Browser8.0.3.107/69/444";
const char S93325[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6200L Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S93326[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6200 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S93327[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; X903 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93328[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93329[] = "MQQBrowser/25 Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LNV-Lenovo A790e Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93330[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93331[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93332[] = "samsung-gt-b5310/UC Browser7.9.0.102/69/351";
const char S93333[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s3650) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S93334[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40EX525; CC/BRA; en) Presto/2.7.61 Version/11.00";
const char S93335[] = "JUC (Linux; U; 4.0.3; zh-cn; HUAWEI_C8812; 480*800) UCWEB7.9.4.145/139/800";
const char S93336[] = "LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-gs500) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S93337[] = "Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU1)/UC Browser8.2.0.132/50/444/UCWEB Mobile";
const char S93338[] = "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0) METASpider";
const char S93339[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC One X Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355";
const char S93340[] = "MicromaxX1i_R/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S93341[] = "JUC(Linux;U;Android4.0.4;Zh_cn;WT19i;320*480;)UCWEB7.8.0.95/139/444";
const char S93342[] = "Mozilla/5.0 (Java; U; en-in; micromaxq5) UCBrowser8.3.0.154/69/352/UCWEB Mobile";
const char S93343[] = "Opera/9.80 (Windows 98; U; en) Presto/2.10.289 Version/12.02";
const char S93344[] = "Opera/9.80 (iPad; Opera Mini/6.0.1/29.3345; U; en) Presto/2.8.119 Version/11.10";
const char S93345[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; InfoPath.1) Sleipnir/2.5.17";
const char S93346[] = "Mozilla/5.0 (Linux; Android 4.0.4; SP-140 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93347[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile i-STYLE Q6 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93348[] = "Mozilla/5.0 (Linux; Android 4.2.1; Desire S Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93349[] = "NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/69/444";
const char S93350[] = "Mozilla/5.0 (Series40; Nokia6700c-1/13.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14";
const char S93351[] = "Mozilla/5.0 (Linux; Android 4.0.4; M870 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S93352[] = "Nokia3500c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3500c) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S93353[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; ST23i Build/11.0.A.5.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93354[] = "LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-gd510) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S93355[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; edenTABET701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93356[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9260 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S93357[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-SU640 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S93358[] = "YahooMobileMessenger/1.0 (Android Messenger; 1.8.3) (GT-P1000; samsung; GT-P1000; 2.2/FROYO)";
const char S93359[] = "Instagram 3.3.3 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7500; GT-P7500; p3; en_US)";
const char S93360[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; iDx10 Build/iDx10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93361[] = "Reeder/1020.09.00 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBook5%2C1)";
const char S93362[] = "Reeder/1020.09.00 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro7%2C1)";
const char S93363[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us  Build/HCL ME Tablet Y2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S93364[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Z7H Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S93365[] = "Mozilla/4.0 (compatible; ipMonitor 10.7)";
const char S93366[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A21 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93367[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Karbonn A21 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93368[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V900A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93369[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; TM-9743W Build/V1.1.ICECREAMSANDWICH.rus.teXet.X.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 MYI/1.10";
const char S93370[] = "Mozilla/5.0 (Linux; Android 4.0.4; BNTV400 Build/2.0.6.6008.magnesium02.hummingbird.rrdf.s1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S93371[] = "Mozilla/5.0 (Linux; Android 4.1.1; Lenovo S890 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S93372[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-D700 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S93373[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB469 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S93374[] = "Mozilla/5.0 (Linux; Android 4.0.4; Thunder S220 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S93375[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9082 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S93376[] = "Mozilla/5.0 (Linux; Android 4.0.4; Philips W536 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93377[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5260) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S93378[] = "LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-gs500) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S93379[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; miTab DIAMOND Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93380[] = "Mozilla/5.0 (Linux; Android 4.0.3; PAD703 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93381[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-I927 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93382[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5570C Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S93383[] = "iBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Mtag353 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93384[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 510)";
const char S93385[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93386[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9220 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S93387[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-D710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S93388[] = "Mozilla/5.0 (Linux; Android 4.0.4; N10 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S93389[] = "Mozilla/5.0 (Linux; Android 4.1.1; ONDA MID Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S93390[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M930BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S93391[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; NuqleoFusion Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93392[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; POLY PAD 9716HD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93393[] = "Mozilla/5.0 (Linux; Android 4.1.1; Samsung GT-S9081 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93394[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I425 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93395[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717R Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93396[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; HTC Desire Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S93397[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT26w) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S93398[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A1+ Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S93399[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-66) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S93400[] = "Mozilla/5.0 (compatible; MSIE 8.0; Win3.11; Trident/1.0)";
const char S93401[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; V9C) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S93402[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; SINUDYNE_SY970G2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93403[] = "Mozilla/5.0 (Linux; Android 4.0.3; MD_LIFETAB_P9516 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S93404[] = "SonyEricssonJ10i2/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; sonyericssonj10i2) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S93405[] = "SonyEricssonJ10i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; fa; sonyericssonj10i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S93406[] = "Mozilla/5.0 (Java; U; en-us; sonyericssonk800i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile";
const char S93407[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-S5830M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000";
const char S93408[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000";
const char S93409[] = "MQQBrowser/42 (iOS 4S; U; CPU like Mac OS X; zh-cn)";
const char S93410[] = "MQQBrowser/42 (iOS 4; U; CPU like Mac OS X; zh-cn)";
const char S93411[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; TSINTECH) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S93412[] = "Mozilla/5.0 (Linux; Android 4.0.3; A510 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S93413[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus S 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93414[] = "Mozilla/5.0 (Linux; Android 4.1.1; IQ442 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93415[] = "Mozilla/5.0 (Linux; Android 4.0.4; FJL21 Build/V37R47A) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93416[] = "Mozilla/5.0 (Linux; Android 4.2.2; MyTouch 4G Slide Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93417[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZP300+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93418[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST25i Build/6.2.A.0.400) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93419[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S93420[] = "Mozilla/5.0 (Linux; Android 4.0.3; ME171 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S93421[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; ALCATEL_ONE_TOUCH_4030D) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S93422[] = "Mozilla/5.0 (Linux; Android 4.2.2; vivo Xplay Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93423[] = "Mozilla/5.0 (Linux; Android 4.2.1; R815 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93424[] = "Mozilla/5.0 (Linux; Android 4.1.1; A50 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93425[] = "Mozilla/5.0 (Linux; Android 4.2.2; myTouch 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93426[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:18.0) Gecko/X Firefox/18.0";
const char S93427[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-S5670 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93428[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; Nokia5730S-1) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S93429[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB860 Build/4.5.1A-129_OLY-145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S93430[] = "Mozilla/5.0 (Linux; Android 4.0.4; StarTrail III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93431[] = "Mozilla/5.0 (Linux; Android 4.0.4; VL-110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S93432[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MOT-XT788 Build/IRPMCT_B_02.74.00RPD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93433[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; HW A880 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93434[] = "UCWEB/2.0 (Windows; U; wds 7.10; en-US; HTC; Radar C110e) U2/1.0.0 UCBrowser/3.0.0.285 U2/1.0.0 Mobile";
const char S93435[] = "UCWEB/2.0 (Windows; U; wds 7.10; en-US; HTC; Radar C110e) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile";
const char S93436[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S93437[] = "Mozilla/5.0 (Linux; U; AliyunOS 2.0; Android 4.0 Compatible; en-us; C2 Build/AliyunOs-2012) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93438[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S93439[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-ca; SGH-I337M Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SGH-I337M, 47, 1.9.7)";
const char S93440[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; E3 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S93441[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT27i Build/6.1.1.B.1.54) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S93442[] = "Nokia2710c-2/2.0 (06.13) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2710c-2) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S93443[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-US; HTC Desire Build/FRF91) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S93444[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL ONE TOUCH 985N) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S93445[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL ONE TOUCH 985N) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S93446[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; HTC One X Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S93447[] = "Mozilla/5.0 (Linux; Android 4.2.1; SKY HD9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93448[] = "Mozilla/5.0 (Linux; Android 4.2.1; SKY HD9500 Plus Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93449[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.289";
const char S93450[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; ALCATEL ONE TOUCH 6010 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S93451[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-in; ST23i Build/11.0.A.5.5) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S93452[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-US; LG-P500 Build/FRG83) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S93453[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-US; LG-P500 Build/FRF91) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S93454[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Spice Mi-502 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S93455[] = "Mozilla/5.0 (Linux; Android 4.2.2; Amaze 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93456[] = "Mozilla/5.0 (Linux; Android 4.0.4; IQ441 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93457[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S5303 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93458[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ607 Build/7.7.1-122-4-FLEM-WIFI-LA) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S93459[] = "Mozilla/5.0 (Linux; Android 4.1.2; L-01E Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93460[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9001 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93461[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S93462[] = "Mozilla/5.0 (Linux; Android 4.1.1; ONE TOUCH TAB 7HD Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S93463[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (Nexus 4, 52, 1.9.9.2)";
const char S93464[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (Nexus 4, 54, 2.0.1)";
const char S93465[] = "MQQBrowser/3.3 (NokiaNXX;SymbianOS/9.1 Series60/3.0)";
const char S93466[] = "MQQBrowser/3.3 (E66;SymbianOS/9.1 Series60/3.0)";
const char S93467[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8950D Build/HuaweiC8950D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S93468[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8950 Build/HuaweiT8950) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S93469[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Next8P12 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S93470[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; KM9001RK Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93471[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HTC 802t Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93472[] = "Mozilla/5.0 (Linux; Android 4.1.2; MID0738 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S93473[] = "Mozilla/5.0 (Series40; Nokia7230/10.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S93474[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI A199 Build/HuaweiA199) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S93475[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9205 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S93476[] = "Mozilla/5.0 (Linux; Android 4.3; Galaxy Nexus Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1650.11 Mobile Safari/537.36";
const char S93477[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P714 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93478[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A89) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S93479[] = "Mozilla/5.0 (Linux; Android 4.2.2; zh-cn; SAMSUNG-GT-I8558_TD/1.0 Android/4.2.2 Release/04.15.2013 Browser/AppleWebKit535.19 Build/JDQ39) ApplelWebkit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S93480[] = "MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MEEG_101 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93481[] = "MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HMIA5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93482[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6812 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S93483[] = "Mozilla/5.0 (Linux; Android 4.1.2; XOLO_X1000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S93484[] = "Mozilla/5.0 (Linux; Android 4.1.2; IQ451 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S93485[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S93486[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; H2000+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93487[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A57) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S93488[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; C1504 Build/11.3.A.0.47) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S93489[] = "Mozilla/5.0 (Linux; Android 4.1.2; EK-GC110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S93490[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT11i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S93491[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A45) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S93492[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S93493[] = "UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; Micromax A35) U2/1.0.0 UCBrowser/9.3.0.321 Mobile";
const char S93494[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A35) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S93495[] = "Mozilla/5.0 (Linux; Android 4.1.2; N861 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S93496[] = "SonyEricssonU10a/R7BA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2";
const char S93497[] = "SonyEricssonU10i/R7BA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2";
const char S93498[] = "HTC MAX 4G Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/1409; U; ru)";
const char S93499[] = "SAMSUNG-SGH-J210/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S93500[] = "Mozilla/5.0 (Linux;U;Android 2.1-update1;zh-cn;ZTE-C_R750/R750V1.0.0B05;320*480;CTC/2.0) AppleWebkit/530.17(KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S93501[] = "Mozilla/5.0 (Linux;U;Android 2.1-update1;zh-cn;ZTE-C_R750/R750V1.0.0B04;320*480;CTC/2.0) AppleWebkit/530.17(KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S93502[] = "Mozilla/5.0 (Linux;U;Android 2.1-update1;zh-cn;ZTE-C_R750/R750V1.0.0B03;320*480;CTC/2.0) AppleWebkit/530.17(KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S93503[] = "Mozilla/5.0 (Linux;U;Android 2.1-update1;zh-cn;ZTE-C_R750/VSF_R750V1.0.0B01;320*480;CTC/2.0) AppleWebkit/530.17(KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S93504[] = "MOT-C168i/1.0 Release/7.24.2006 Browser/CMCS1.0 Software/0.080";
const char S93505[] = "MOT-C168/ WAP.Browser/1.0";
const char S93506[] = "SonyEricssonW150i/R2AD034 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93507[] = "sonyericssonw150i";
const char S93508[] = "SonyEricssonS500i/R8BE Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93509[] = "sonyericssons500i";
const char S93510[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) HTC-6900-MR1";
const char S93511[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) HTC-6900";
const char S93512[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) PPC; 240x320; HTC_P3600i/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93513[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) PPC; 240x320; HTC_P3600/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93514[] = "LG-GM730f/IEMobile/v6.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S93515[] = "LG-KP115 Obigo/WAP2.0";
const char S93516[] = "LG/U890/v1.0";
const char S93517[] = "SEC-SGHB200/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S93518[] = "SonyEricssonK510a/R4EA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93519[] = "SonyEricssonK510c/R4EA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93520[] = "SonyEricssonZ558c/R4FA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93521[] = "SonyEricssonZ558i/R4FA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93522[] = "SAMSUNG-SGH-i900/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S93523[] = "Nokia6682/2.0 (4.65.0) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93524[] = "LG/U830/v1.0";
const char S93525[] = "LG/U8330/v1.0";
const char S93526[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; Dell Aero Build/MASTER) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S93527[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; Google Ion Build/DRC83) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S93528[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; phiremod Vibrant Build/XWJS5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93529[] = "Mot-EX212/18.01 Browser/Obigo/Q03C MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93530[] = "LGE-VX8350/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S93531[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320; HTC_TyTN/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93532[] = "NokiaX6-00/UC Browser7.6.1.82/50/351";
const char S93533[] = "KDDI-SH33 UP.Browser/6.2.0.13.1.5 (GUI) MMP/2.0";
const char S93534[] = "NOKIAN95/UC Browser7.6.1.82/28/352";
const char S93535[] = "NOKIAN95 8GB/UC Browser7.6.1.82/28/352";
const char S93536[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.6.1.82";
const char S93537[] = "NOKIAN95 8GB/UC Browser7.8.0.95/28/355";
const char S93538[] = "NOKIAN95/UC Browser7.8.0.95/28/352";
const char S93539[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; OliPad 100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93540[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; S2005A-H Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93541[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; S2005A-H Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/5.0 Mobile Safari/533.16";
const char S93542[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:5.0) Gecko/X Firefox/5.0 Kylo/1.0.1.76141";
const char S93543[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:10.0) Gecko/X Thunderbird/10.0";
const char S93544[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/2.1.0 Safari/533.1";
const char S93545[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/2.1.0 Safari/533.1";
const char S93546[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/2.1.0 Safari/533.1";
const char S93547[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/2.1.0 Safari/533.1";
const char S93548[] = "Mozilla/5.0 (Windows NT 5.0; rv:8.0.1) Gecko/X Firefox/8.0.1 SeaMonkey/2.5";
const char S93549[] = "JUC (Linux; U; 2.3.5; zh-cn; Lenovo_A500; 320*480) UCWEB7.9.3.103/139/32564";
const char S93550[] = "Opera/8.01 (J2ME/MIDP; MXit WebBot/6.2.1/1.8.5.168; U;) Opera Mini/3.1";
const char S93551[] = "Opera/8.01 (J2ME/MIDP; MXit WebBot/6.1.0/1.8.5.168; U;) Opera Mini/3.1";
const char S93552[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Radiant)";
const char S93553[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; PI86100)";
const char S93554[] = "SAMSUNG-GT-S5330/1.0 SHP/VPP/R5 Dolfin/2.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S93555[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40EX723; CC/GBR; en) Presto/2.7.61 Version/11.00";
const char S93556[] = "SonyEricssonC905/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonc905/UC Browser8.0.3.107/69/352";
const char S93557[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LG855 Build/CTMod 3.68)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93558[] = "Mozilla/5.0 (Java; U; en-us; gt-s5230) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S93559[] = "Mozilla/5.0 (S60V3; U; en-us; Nokia N96)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S93560[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-I9003 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93561[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; SK17i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S93562[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; id-id; T1 Build/I700T_P4_10_AC73_RE_R_0_03_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93563[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; SonyEricssonZ1i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93564[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Z1i Build/4.0.2.A.0.42) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S93565[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire S Build/GRJ90)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93566[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S93567[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-8230 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93568[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; MID7034C Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93569[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; A75 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S93570[] = "Mozilla/5.0 (Linux; Android 4.0.4; SN10T1 Build/IML77) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S93571[] = "Mozilla/5.0 (X11; U; Linux x86_64; de; rv:1.9.0.6) Gecko/X Ubuntu/8.10 (intrepid) Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.1) Opera 7.03 [de]";
const char S93572[] = "Nokia6120c/3.83; Profile/MIDP-2.0 Configuration/CLDC-1.1;Sansui_H50T_Maui Wap Browser";
const char S93573[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE61i)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S93574[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE61)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S93575[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; N860 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S93576[] = "Mozilla/5.0 (Java; U; en-in; micromax) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S93577[] = "Mozilla/5.0 (S60V5; U; en-in; Nokia5236)/UC Browser8.2.0.132/50/444/UCWEB Mobile";
const char S93578[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.0 Mobile Safari/535.19 Silk-Accelerated=false";
const char S93579[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Inspire 4G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93580[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Inspire 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93581[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; U20i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S93582[] = "SAMSUNG-GT-S5610K/S5610KJPKL1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S93583[] = "SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonw20i) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S93584[] = "Instacast/3.0.3 (like iTunes/10.1.2)";
const char S93585[] = "MQQBrowser/Mini2.8 (Nokia200/11.56)";
const char S93586[] = "Mozilla/5.0 (Linux; Android 4.0.4; IS12M Build/6.7.5-50_MR1-11) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93587[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; ZTE-BLADE Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S93588[] = "Mozilla/5.0 (Linux; Android 4.0.4; N-04D Build/AX) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93589[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.6; K900 Build/GRK39F)";
const char S93590[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; K900 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93591[] = "Mozilla/5.0 (X11; OpenBSD amd64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11";
const char S93592[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; SPH-D710 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S93593[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.2pre) Gecko/X Ubuntu/9.10 (karmic) Namoroka/3.6.2pre";
const char S93594[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.1 (KHTML, like Gecko) Ubuntu/10.04 Chromium/21.0.1180.89 Chrome/21.0.1180.89 Safari/537.1";
const char S93595[] = "MQQBrowser/3.7/ZTE-TU880_TD/1.0 Linux/2.6.32 Android/2.2 Release/5.25.2011 Browser/AppleWebKit533.1";
const char S93596[] = "Mozilla/5.0 (Linux; Android 4.0.4; N-05D Build/AX) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93597[] = "Mozilla/5.0 (Linux; Android 4.0.4; AVA757 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93598[] = "JUC(Linux;U;Android4.0.4;Zh_cn;LT26i;720*1280;)UCWEB7.8.0.95/139/355";
const char S93599[] = "OneBrowser/3.1 (SAMSUNG GT-C3303i)";
const char S93600[] = "Mozilla/5.0 (Linux; Android 4.0.3; iDx5 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93601[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s5250) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S93602[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonk530i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S93603[] = "Mozilla/5.0 (X11; U; SunOS i86pc; en-US; rv:1.8.1.12) Gecko/X SeaMonkey/1.1.8";
const char S93604[] = "JUC (Linux; U; 2.3.6; zh-cn; N320; 240*320) UCWEB7.9.0.94/139/444";
const char S93605[] = "Mozilla/5.0 (Linux; Android 4.0.4; IRIS_501 Build/LAVAIRIS501) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93606[] = "RssBandit/1.9.0.1002 (.NET CLR 2.0.50727.3643; WinNT 5.1.2600 Service Pack 3; http://www.rssbandit.org)";
const char S93607[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93608[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S93609[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; generic Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S93610[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-W899 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93611[] = "Mozilla/5.0 (Linux; Android 4.1.1; SH930W Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S93612[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P7100 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.37 Safari/537.22";
const char S93613[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7100 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S93614[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; HTC6435LVW Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93615[] = "OneBrowser/3.1 (NokiaX2-05/08.60)";
const char S93616[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S93617[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A789 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S93618[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LT15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S93619[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; X10i Build/2.1.A.0.435) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S93620[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530M Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93621[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; ZTE U795 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S93622[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAP4500DUO Build/PrestigioPAP4500DUO) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93623[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TM-5204 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93624[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; SmartTab1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93625[] = "ZTE-E821S-Movistar/1.0 NetFront/4.2 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S93626[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; FreeTAB 9701 HD X1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93627[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A840S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S93628[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; SHW-M110S) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S93629[] = "Mozilla/5.0 (Linux; Android 4.0.4; Enjoy 71 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S93630[] = "Mozilla/5.0 (Linux; Android 4.1.2; Amaze 4G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93631[] = "Mozilla/5.0 (Linux; Android 4.0.4; A6 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93632[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Elcano Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93633[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I8150 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S93634[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE T81 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93635[] = "Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 6 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93636[] = "OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93637[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI C8813 Build/HuaweiC8813) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S93638[] = "Mozilla/5.0 (compatible; Konqueror/4.1; Linux 2.6.27.7-134.fc10.x86_64; X11; x86_64) KHTML/4.1.3 (like Gecko) Fedora/4.1.3-4.fc10";
const char S93639[] = "Mozilla/5.0 (Linux; Android 4.0.4; HW-01E Build/HuaweiU9501L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93640[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia ion Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93641[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-7204 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S93642[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-MS770 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S93643[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT-AS45SE Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93644[] = "UCWEB/2.0 (Linux; U; Adr 2.2; en-US; XCD35) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S93645[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; TECNO_Q1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S93646[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Intex_Aqua_Swadesh) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S93647[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A15 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S93648[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A860K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93649[] = "Mozilla/5.0 (Linux; Android 4.2.1; ultrafone 701HD Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93650[] = "Mozilla/5.0 (Linux; Android 4.0.4; Wilco Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93651[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S93652[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40HX750; CC/VNM; vi) Presto/2.10.250 Version/11.60";
const char S93653[] = "Nokia2710c-2/2.0 (06.13) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia2710c-2) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S93654[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST15i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S93655[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A10) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S93656[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-e2202) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S93657[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Lenovo A590 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S93658[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-E970 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93659[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-E970 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/geeb_att_us";
const char S93660[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Transformer TF101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);";
const char S93661[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; myTouch 4G Build/JDQ39; LiquidSmooth-Liquid-JB-v2.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93662[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; RMD-757 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93663[] = "Mozilla/5.0 (Linux; Android 4.0.4; Gpad_G1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93664[] = "Mozilla/5.0 (Linux; Android 4.0.4; UBISLATE7C+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93665[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax P275 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S93666[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Edison 3G Build/1.1.11-1015 X-15:24) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S93667[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAP4055DUO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93668[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Titanium S5 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S93669[] = "Mozilla/5.0 (Linux; Android 4.1.2; P5_Quad Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93670[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9001 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93671[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC Desire X Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S93672[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Infinix X450 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/33567";
const char S93673[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S93674[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; X909 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S93675[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Spice Mi-424 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93676[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; zh-CN; GT-I9100G) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile";
const char S93677[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S93678[] = "Mozilla/5.0 (Linux; Android 4.1.1; ONE TOUCH 4011X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93679[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; TECNO_D5) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S93680[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-R820 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S93681[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A65) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S93682[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S93683[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTCEVODesign4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S93684[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7310 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S93685[] = "Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Kem 452 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93686[] = "Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar_Touch_KEM_452_HDMI Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93687[] = "Mozilla/5.0 (Android; Mobile: SCH-I545; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S93688[] = "Mozilla/5.0 (Linux; Android 4.0.4; P5 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93689[] = "Mozilla/5.0 (Linux; Android 4.2.2; Xperia T Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93690[] = "Mozilla/5.0 (Linux; Android 4.0.3; AN10CG3 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S93691[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5312 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93692[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOLO A700 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93693[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-b7722) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S93694[] = "Mozilla/5.0 (Linux; Android 4.3; HTC One X Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93695[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A44) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S93696[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Cloudfone Excite 402d Build/FSV08) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93697[] = "UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Micromax_A110Q) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S93698[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LS840 Build/ZVK.IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S93699[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; vivo X1St Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S93700[] = "Mozilla/5.0 (Linux; Android 4.2.1; PMP5588C Build/E8HD-ELITE-1280x768) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Safari/537.36";
const char S93701[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E470f Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93702[] = "Mozilla/5.0 (Linux; U; Android 4.0.4;es-us; ZTE V887 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93703[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT1S0 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S93704[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; A111) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S93705[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A52) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S93706[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Q700 Build/XOLOQ700) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S93707[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT26w Build/6.1.A.2.55) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S93708[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LT26w Build/6.1.A.2.45) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S93709[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGP321 Build/10.3.1.A.0.244) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S93710[] = "Mozilla/5.0 (Linux; Android 4.1.1; KUNO4 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S93711[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.2; en-gb; GT-I9082 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S93712[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A9+ Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S93713[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7310 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S93714[] = "SAMSUNG-SCH-M710/(null)ID4 (compatible; MSIE 6.0; Windows CE; PPC) Opera 9.5 Presto/2.2.1";
const char S93715[] = "NokiaN72/5.0706.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.6.1.82/27/354";
const char S93716[] = "HTC_Snap_S521 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S93717[] = "HTC-ST6356/2.31.502.4 (66023) Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S93718[] = "HTC_X7500 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S93719[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) PPC; 640x480; HTC_X7500/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93720[] = "Mozilla/5.0 (Linux;U;Android 2.1-update1;zh-cn;ZTE-C_N600/ZTE-C_N600V1.0.0B03;240*320;CTC/2.0) AppleWebkit/530.17(KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S93721[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; ZTE-C_N600+/ZTE-C_N600PLUSV1.0.0B02; 240*320; CTC/2.0) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S93722[] = "Mozilla/5.0 (Linux;U;Android 2.1-update1;zh-cn;ZTE-C_N600/ZTE-C_N600V1.0.0B02;240*320;CTC/2.0) AppleWebkit/530.17(KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S93723[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; ZTE-C N600 Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S93724[] = "LG-KP270V/V10g Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S93725[] = "LG-KP270/V10g Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S93726[] = "LG-KP270-V10a Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S93727[] = "Nokia3555b/2.0 (05.07) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S93728[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC_P3600/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93729[] = "HTC_Trinity_P3600 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC_P3600/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93730[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC_P3600i/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93731[] = "HTC_Trinity Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S93732[] = "HTC_P3600 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S93733[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; LG_GM750/1.0)";
const char S93734[] = "SAMSUNG-SGH-E360/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S93735[] = "LG/KU970/v10a Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93736[] = "LG-KU970/v1.0";
const char S93737[] = "NokiaN71-1/2.0613 Series60/3.0  Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93738[] = "LG9600/1.0 TELECA/Q7.0-1.6 MMP/2.0";
const char S93739[] = "Alcatel-OT-706/1.0 ObigoInternetBrowser/Q03C";
const char S93740[] = "Alcatel-OT-706A/1.0 ObigoInternetBrowser/Q03C";
const char S93741[] = "MOT-V235/0A.30.6CR MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93742[] = "NokiaN73/UC Browser7.2.2.51/28/351";
const char S93743[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; LG; GW910)";
const char S93744[] = "SonyEricssonZ320i/R1F003 Browser/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93745[] = "SonyEricssonZ320a/R1C003 Browser/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93746[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Schubert)";
const char S93747[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; ZTE-C_N600+/ZTE-C_N600PLUSV1.0.0B03; 240*320; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93748[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; ZTE-C N600+ Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S93749[] = "VIDEOCON V-1448/WAP2.0 Profile/MT6253/QVGA";
const char S93750[] = "NokiaE63/UCWEB7.5.0.66/28/999";
const char S93751[] = "NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-00/UC Browser7.7.1.88/69/444";
const char S93752[] = "NokiaN-GageQD/1.0 SymbianOS/6.1 Series60/1.2 Profile/MIDP-1.0 Configuration/CLDC-1.0/UC Browser7.7.1.88/26/444";
const char S93753[] = "NOKIAN79/UC Browser7.7.1.88/28/444";
const char S93754[] = "nokian79-1/UC Browser7.7.1.88/69/444";
const char S93755[] = "NokiaC6-00/UC Browser7.8.0.95/50/355";
const char S93756[] = "KDDI-TS3I UP.Browser/6.2_7.2.7.1.K.1.5.1.120 (GUI) MMP/2.0";
const char S93757[] = "KDDI-SN3K UP.Browser/6.2_7.2.7.1.K.3.352 (GUI) MMP/2.0";
const char S93758[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S93759[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S93760[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Nexus S 4G Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93761[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus S 4G Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93762[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/12.0.742.122)";
const char S93763[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/12.0.742.100)";
const char S93764[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/12.0.742.122) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S93765[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/12.0.742.112) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30";
const char S93766[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/12.0.742.100) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30";
const char S93767[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/12.0.742.112) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30";
const char S93768[] = "samsung-gt-s5620/UC Browser7.9.0.102/69/444";
const char S93769[] = "NOKIAE65/UC Browser8.0.4.121/28/352/UCWEB";
const char S93770[] = "MQQBrowser/31 (iOS; U; CPU like Mac OS X; zh-cn)";
const char S93771[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Comodo_Dragon/4.1.1.12 Chrome/4.1.249.1042 Safari/532.5";
const char S93772[] = "Safari/6531.21.10 CFNetwork/454.9.4 Darwin/10.3.2 (i386) (MacBookPro7%2C1)";
const char S93773[] = "Safari/6531.22.7 CFNetwork/454.9.4 Darwin/10.3.2 (i386) (MacBook7%2C1)";
const char S93774[] = "Mozilla/5.0 (Windows NT 5.1; rv:9.0) Gecko/X Thunderbird/9.0 Lightning/1.1.1";
const char S93775[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:9.0) Gecko/X Thunderbird/9.0.1";
const char S93776[] = "MQQBrowser/2.4 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3";
const char S93777[] = "MQQBrowser/2.4 Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3";
const char S93778[] = "MQQBrowser/2.4 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 Safari/7534.48.3";
const char S93779[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.6) Gecko/X Firefox/2.0.0.6 (Linux Mint)";
const char S93780[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9A405 Safari/6533.18.5";
const char S93781[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5";
const char S93782[] = "MQQBrowser/2.7 (NokiaC5-03;SymbianOS/9.1 Series60/3.0)";
const char S93783[] = "Mozilla/5.0 (Windows NT 5.1; rv:9.0.1) Gecko/X Firefox/9.0.1 CometBrowser/9.0.1";
const char S93784[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:9.0.1) Gecko/X Firefox/9.0.1 CometBird/9.0.1";
const char S93785[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; C8500 Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S93786[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; C8500 Build/ERE27) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S93787[] = "Mozilla 5.0 (Linux; U; Android 2.1-update1; zh-cn; C8500 Build ERE27) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31";
const char S93788[] = "Mozilla/5.0 (Windows NT 6.0; rv:12.0) Gecko/X Firefox/12.0 slurp";
const char S93789[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:12.0) Gecko/X Firefox/12.0 slurp";
const char S93790[] = "Mozilla/5.0 (X11; U; CrOS i686 9.10.0; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.253.0 Safari/532.5";
const char S93791[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MB525 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93792[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB525 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93793[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; MB525 Build/IMM76L)";
const char S93794[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i677)";
const char S93795[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG-E740h)";
const char S93796[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG-E740)";
const char S93797[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; Spice_Mi300 Build/DONUT) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S93798[] = "Opera/9.80 (X11; Linux i686; U; en) Presto/2.2.15 Version/10.20";
const char S93799[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:6.0.2) Gecko/X Firefox/6.0.2 SeaMonkey/2.3.3";
const char S93800[] = "Nokia6085/2.0 (06.00) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6085/UC Browser8.0.3.107/69/444";
const char S93801[] = "Mozilla/5.0 (Series40; NokiaC2-06/06.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S93802[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA6700s)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S93803[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation XE with Beats Audio Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93804[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S93805[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 Comodo_Dragon/21.1.1.0";
const char S93806[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 Comodo_Dragon/21.1.1.0";
const char S93807[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; eZee'Tab7c Build/V0.15-Storex-V971-114.GINGERBREAD.eng.chenxiang.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93808[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A100 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93809[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; th-th; A9 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93810[] = "MQQBrowser/26 (Linux; U; 2.2.2; zh-cn; ZTE-U V880 Build/UNI_CN_V880 2.1;480*800)";
const char S93811[] = "NokiaX2-02/2.0 (10.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-02/UC Browser7.7.1.88/69/444";
const char S93812[] = "lg-x330t/UC Browser7.9.0.102/69/352";
const char S93813[] = "Mozilla/5.0 (Linux; Android 4.0.3; SX-SP715A Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S93814[] = "Mozilla/5.0 (Linux; Android 4.0.3; GOCLEVER TAB A971 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S93815[] = "SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonw20) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S93816[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (ADR6425LVW, 42, 1.9.1)";
const char S93817[] = "Mozilla/5.0 (Linux; Android 4.0.3; S5 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93818[] = "Mozilla/5.0 (X11; Linux x86_64; HTC-A9192/1.0; en-gb) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S93819[] = "Mozilla/5.0 (X11; Linux x86_64; HTC-A9192/1.0; en-us) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S93820[] = "Mozilla/5.0 (Java; U; en-us; gt-s5233w) UCBrowser8.2.1.144/69/352/UCWEB Mobile";
const char S93821[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S93822[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC_Explorer_A310e Build/GRJ90) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S93823[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Desire C Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S93824[] = "Mozilla/5.0 (Linux; Android 4.0.3; T-01D Build/V09R40A) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93825[] = "NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.5.0.185/83/352";
const char S93826[] = "MQQBrowser/2.8 (NokiaX6-00;SymbianOS/9.1 Series60/3.0)";
const char S93827[] = "JUC(Linux;U;Android2.3.6;Zh_cn;A25;240*320;)UCWEB7.8.0.95/139/444";
const char S93828[] = "Mozilla/5.0 (Linux; Android 4.0.4; Stealth 9.7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S93829[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A15 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S93830[] = "iBrowser/Mini2.5 (LG-T375)";
const char S93831[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S93832[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cruize W280 Build/Cherry_Cruize_W280_2012.12.30) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S93833[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonwt13i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S93834[] = "UCWEB/2.0 (Java; U; MIDP-2.0; fa; sonyericssonwt13i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S93835[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL55HX853; CC/GBR; en) Presto/2.10.250 Version/11.60";
const char S93836[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V889D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93837[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S93838[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Samsung GT-S9081 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93839[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A52 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S93840[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530M Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S93841[] = "Mozilla/5.0 (Linux; Android 4.0.4; KHPHN05ANDA Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S93842[] = "Mozilla/5.0 (Linux; Android 4.2.2; Desire S Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S93843[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93844[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93845[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S93846[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android HDVs_Android2.2.1; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93847[] = "MQQBrowser/3.7/Adr (Linux; U; HDVs_Android2.2.1; zh-cn; GT-P1000 Build/HDVs-V2-1.4;600*1024)";
const char S93848[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3105 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S93849[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Iron/10.0.650.1 Chrome/10.0.650.1 Safari/534.16";
const char S93850[] = "JUC (Linux; U; 2.3.6; zh-cn; ZTE_N788; 320*480) UCWEB7.9.3.103/139/32846";
const char S93851[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; SHV-E120S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93852[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; Andy-3G-4.0 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93853[] = "Opera/9.80 (Windows 98) Presto/2.12.388 Version/12.11";
const char S93854[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7568 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93855[] = "Mozilla/5.0 (Linux; Android 4.0.4; A15 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S93856[] = "Mozilla/5.0 (Linux; Android 4.0.4; Thrill 430x Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S93857[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328t Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S93858[] = "Mozilla/5.0 (Linux; Android 4.0.3; AN7HG3 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S93859[] = "Mozilla/5.0 (Linux; Android 4.0.4; 201M Build/7.7.1Q-164_SMJ-165) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93860[] = "Nokia3500c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia3500c) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S93861[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; SGV-E120L Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93862[] = "Mozilla/5.0 (Linux; Android 4.1.1; X8 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S93863[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ607 Build/7.7.1-122-4-FLEM-WIFI-LA) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S93864[] = "Mozilla/5.0 (Linux; Android 4.1.1; Z120 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93865[] = "Mozilla/5.0 (Linux; Android 4.1.2; SH837W Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93866[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT-AS45SE Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93867[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST17i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93868[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xperia Arc S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93869[] = "Mozilla/5.0 (Linux; Android 4.0.4; A15 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93870[] = "Mozilla/5.0 (Linux; Android 4.1.1; Fusion Bolt Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S93871[] = "Mozilla/5.0 (Linux; Android 4.0.4; Philips W832 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93872[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MID Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S93873[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S93874[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N8013 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93875[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cloudpad 970g Build/FSV05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93876[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; KATBL08A16E Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93877[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8150 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S93878[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6210 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S93879[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-7240 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S93880[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; R800i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S93881[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Alpha GTR Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93882[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-en; MAPAN MX88R Build/MAPAN MX88R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93883[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; T-Mobile_myTouch_3G_Slide) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S93884[] = "Mozilla/5.0 (Linux; Android 4.0.4; F-04E Build/V14R31C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93885[] = "Mozilla/5.0 (Linux; Android 4.0.4; Philips W832 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93886[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.1.19) Gecko/X SeaMonkey/2.0.14";
const char S93887[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; LG-P990 Build/IMM76L)";
const char S93888[] = "Mozilla/5.0 (Linux; Android 4.0.4; Turkcell Maxi Plus 5 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93889[] = "Opera/9.01 (X11; FreeBSD 6 i386; U; en)";
const char S93890[] = "MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad7295 Build/4.1.015.X.7295; 540*960; CUCC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93891[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:21.0) Gecko/X Firefox/21.0 TenFourFox/7450";
const char S93892[] = "Mozilla/5.0 (Linux; Android 4.0.3; PAD703 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93893[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (DROIDX, 42, 1.9.1)";
const char S93894[] = "JUC (Linux; U; 2.2; zh-cn; GT-I5801; 240*400) UCWEB7.9.0.94/139/444";
const char S93895[] = "ZEN M2s CODE/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S93896[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; miTab BROOKLYN Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93897[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; AMAZE S200 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93898[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; XOLO_A800) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S93899[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9082 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S93900[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5580C Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S93901[] = "Mozilla/5.0 (Linux; Android 4.1.1; chagall Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S93902[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-P500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93903[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5303 Build/12.0.A.1.257) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S93904[] = "Mozilla/5.0 (Linux; Android 4.1.1; AMOI N821 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S93905[] = "Mozilla/5.0 (Linux; Android 4.0.4; Quantum 7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S93906[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; HTC_Glacier) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S93907[] = "Mozilla/5.0 (Linux; Android 4.0.4; Starmobile Engage7 3G Build/ICS_IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S93908[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID8048 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444";
const char S93909[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S93910[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; X907 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S93911[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; SPC Niobe7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93912[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S93913[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; hr-hr; Galaxy Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S93914[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A66 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93915[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S93916[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A9 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S93917[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Karbonn A2) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S93918[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; MITO t510 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93919[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPHS on Hsdroid Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93920[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E455f Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93921[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S93922[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; WALTON_Primo-G1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93923[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; A111) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S93924[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Softwinern01 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S93925[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; ALCATEL ONE TOUCH 6033M Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30";
const char S93926[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ipad mini Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93927[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Titanium S5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300";
const char S93928[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8190N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S93929[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-gb; GT-S5303 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S93930[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.7; en-us; LG-VM696 Build/ZV5.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S93931[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-gb; GT-S6012 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S93932[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6710s) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S93933[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia701) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile  3gpp-gba";
const char S93934[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F100L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93935[] = "Mozilla/5.0 (Linux; Android 4.1.1; X507S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93936[] = "Mozilla/5.0 (Linux; Android 4.2.1; Cynus T5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93937[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I959 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S93938[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Maxwell Plus Build/1.0.3 X-14:07) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S93939[] = "Mozilla/5.0 (Linux; Android 4.3; Amazon Kindle Fire Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S93940[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N5120 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S93941[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; N320) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S93942[] = "Mozilla/5.0 (Android; Mobile: DROID RAZR; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S93943[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Amazon Kindle Fire HD Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.2/tate";
const char S93944[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E977 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93945[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx78) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S93946[] = "UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Micromax_A110Q) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S93947[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5570 Build/FROYO) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S93948[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaE7-00) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S93949[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT-I5503) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S93950[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6312 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S93951[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S93952[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E467f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S93953[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS930 4G Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S93954[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S93955[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SM-G730A Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S93956[] = "Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia5130c-2) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S93957[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9200 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S93958[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919N Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S93959[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC PH39100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S93960[] = "SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S93961[] = "SCH-U700/1.0 NetFront/3.0.22.2.7 (GUI) MMP/2.0";
const char S93962[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_E200";
const char S93963[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_E200 1.049c.00";
const char S93964[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_E200 2.001.00";
const char S93965[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_E200 2.002.00";
const char S93966[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_E200 2.004.00";
const char S93967[] = "LG-TU720 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93968[] = "ZTE-F106(Vinaphone)/1.0 NetFront/3.5 QTV2.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93969[] = "HTC_Snap_S526 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S93970[] = "Lemon_W100/1.0SH NetFront/3.5 SMM-MMS/1.2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93971[] = "DoCoMo/2.0 SH03A(c100;TB;W24H16)";
const char S93972[] = "SPH-M310 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S93973[] = "LG-KP320 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S93974[] = "LG-LX550 AU-MIC-LX550/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93975[] = "ZTE-T7/1.0 ACS-NF/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93976[] = "NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-01/UC Browser7.8.0.95/70/355";
const char S93977[] = "DoCoMo/2.0 P02C(c500;TB;W24H16)";
const char S93978[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.0 Chrome/13.0.800.0 Safari/535.1";
const char S93979[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.0 Chrome/13.0.800.0 Safari/535.1";
const char S93980[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.30 (KHTML, like Gecko) Iron/12.0.750.0 Chrome/12.0.750.0 Safari/534.30";
const char S93981[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.30 (KHTML, like Gecko) Iron/12.0.750.0 Chrome/12.0.750.0 Safari/534.30";
const char S93982[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; HTC myTouch 4G Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93983[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; NOKIA; Lumia 710)";
const char S93984[] = "Opera/9.10 (Windows 98; U; en)";
const char S93985[] = "BlackBerry8900/5.2.0.96 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S93986[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S93987[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; HTC_Wildfire Build/FRG83D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S93988[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; DROIDX Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93989[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROIDX Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S93990[] = "MQQBrowser/33 (iOS; U; CPU like Mac OS X; zh-cn)";
const char S93991[] = "Mozilla/5.0 (X11; U; CrOS i686 0.0.0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.151 Safari/534.16";
const char S93992[] = "Mozilla/5.0 (X11; U; CrOS i686 0.10.146; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.642.2 Safari/534.16";
const char S93993[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.1 Chrome/15.0.900.2 Safari/535.2";
const char S93994[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.6) Gecko/X Linux Mint/4.0 (Daryna) Firefox/2.0.0.6 (Linux Mint)";
const char S93995[] = "Mozilla/5.0 (Windows NT 5.2; WOW64; rv:8.0) Gecko/X Thunderbird/8.0";
const char S93996[] = "Mozilla/5.0 (Windows NT 5.1; rv:8.0) Gecko/X Thunderbird/8.0 OracleBeehiveExtension/1.0.0.4-OracleInternal Lightning/1.0";
const char S93997[] = "IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB8.0.4.105/41/800";
const char S93998[] = "IUC(U;iOS 4.2.1;Zh-cn;320*480;)/UCWEB8.0.4.105/41/800";
const char S93999[] = "IUC(U;iOS 4.1;Zh-cn;320*480;)/UCWEB8.0.4.105/41/800";
const char S94000[] = "Mozilla/5.0 (MeeGo; NokiaN950-00/00) AppleWebKit/534.13 (KHTML, like Gecko) NokiaBrowser/8.5.0 Mobile Safari/534.13";
const char S94001[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11 Comodo_Dragon/20.0.1.0";
const char S94002[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11 Comodo_Dragon/20.0.1.0";
const char S94003[] = "Mozilla/5.0 NewsFox/1.0.8.4";
const char S94004[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11 Comodo_Dragon/20.1.1.0";
const char S94005[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11 Comodo_Dragon/20.1.1.0";
const char S94006[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; SAMSUNG; GT-I8350)";
const char S94007[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; SAMSUNG; GT-I8350T)";
const char S94008[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; SAMSUNG; Omnia W; Orange)";
const char S94009[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;T9295;7.10;7720)";
const char S94010[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;T9295;7.10;8107)";
const char S94011[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i917R;7.10;8107)";
const char S94012[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i917R;7.10;7720)";
const char S94013[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i917R;7.10;8773)";
const char S94014[] = "MicromaxX333/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S94015[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U10GT_A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S94016[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.5; N-01D Build/AX)";
const char S94017[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N-01D Build/AX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94018[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s3650) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S94019[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s3650) UCBrowser8.2.0.132/69/352/UCWEB Mobile";
const char S94020[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S94021[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-P1000T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S94022[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352";
const char S94023[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-P1000N Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352";
const char S94024[] = "MQQBrowser/20 Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ZTE-C_N760/N760V1.0.0B03; 320*480; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94025[] = "MQQBrowser/20 Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE-C_N760/N760V1.0.0B04; 320*480; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94026[] = "MQQBrowser/20 Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC A810e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94027[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Dell Venue Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94028[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-us; Tuna Build/IFK77E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.67";
const char S94029[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 3GW101 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94030[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 3GW101 Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S94031[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Transformer Prime TF300T Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S94032[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Transformer Pad TF300T Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S94033[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94034[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94035[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; eZee'Tab10c Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94036[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.8.0.5) Gecko/X Firefox/1.5.0.5 Flock/0.7.4.1";
const char S94037[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.2a1pre) Gecko/X Firefox/4.2a1pre";
const char S94038[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.2a1pre; Gecko/X Firefox/4.2a1pre)";
const char S94039[] = "Mozilla/5.0 (Java; U; en-us; sonyericssonk800i) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S94040[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94041[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.17 (KHTML, like Gecko) Ubuntu/12.10 Chromium/24.0.1312.2 Chrome/24.0.1312.2 Safari/537.17";
const char S94042[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.0.60 (Windows XP)/886; U; en) Presto/2.4.15";
const char S94043[] = "Nokia311/5.0 (03.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia311) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S94044[] = "iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SmartTab2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S94045[] = "maui/UC Browser7.2.2.51/69/352";
const char S94046[] = "Mozilla/5.0 (Windows; N; ; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.1.0.5001 Safari/532.4";
const char S94047[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; gt-s5230) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S94048[] = "UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; gt-s5230) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S94049[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-s5230) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S94050[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:12.0) Gecko/X Firefox/12.0 Kylo/1.1.1";
const char S94051[] = "LG-GX500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-gx500) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S94052[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S94053[] = "Mozilla/5.0 (Linux; Android 4.0.4; Advance TV Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94054[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/6.0.2 Safari/7536.26.17 Sleipnir/4.0.1";
const char S94055[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-b5310) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S94056[] = "Mozilla/5.0 (Linux; Android 4.0.4; O+ TABLET Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94057[] = "Mozilla/5.0 (Linux; Android 4.0.4; FPT HD Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94058[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V788D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94059[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; P711F Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S94060[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL55HX750; CC/USA; en) Presto/2.10.250 Version/11.60";
const char S94061[] = "Mozilla/5.0 (Linux; U; Android GFIVE-4.0; en-us; GFIVE Aurora(A79) Build/Aurora-A79-V1.0-2012.09.29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94062[] = "Mozilla/5.0 (Linux; Android 4.0.4; M1062 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94063[] = "MicromaxQ66/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S94064[] = "SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; sonyericssonw20) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S94065[] = "Mozilla/5.0 (Linux; Android 4.1.1; Artes D821 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94066[] = "Opera/9.80 (Windows 98) Presto/2.12.388 Version/12.12";
const char S94067[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTCEVOV4G Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S94068[] = "Instagram 3.4.4 Android (16/4.1.1; 160dpi; 600x976; samsung; GT-P3113; espressowifi; espresso; en_US)";
const char S94069[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T-Mobile myTouch 4G Build/IMM76L; CyanogenMod-mimicry) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94070[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lemon Tab LT29 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94071[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; SoftwinerEvb Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S94072[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_Sensation_Z710e) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S94073[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Full AOSP on Rk29sdk Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S94074[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N8013 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S94075[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D700 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S94076[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T679 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S94077[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAP4500TDUO Build/PrestigioPAP4500TDUO) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S94078[] = "Mozilla/5.0 (Linux; Android 4.0.4; AUXUS CoreX4 3G Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S94079[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; ZTE U930 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S94080[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9220 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S94081[] = "Mozilla/5.0 (Linux; Android 4.0.4; NexusHD2 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S94082[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P920 Build/ICS) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S94083[] = "Mozilla/5.0 (Windows; U; Windows NT based; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.A.B.C Safari/525.13";
const char S94084[] = "Windows Phone Search (Windows Phone OS 8.0;NOKIA;Lumia 920;8.0;10211)";
const char S94085[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F240S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S94086[] = "UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 900) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile";
const char S94087[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB469 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S94088[] = "Mozilla/5.0 (Linux; Android 4.0.4; novo7Legend Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S94089[] = "Mozilla/5.0 (Linux; Android 4.1.1; WEXLER-TAB-7T Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S94090[] = "Mozilla/5.0 (Linux; Android 4.0.4; NexusHD2 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94091[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; A75 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355";
const char S94092[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; Artes D821 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S94093[] = "Mozilla/5.0 (Linux; Android 4.0.3; TAB466EUK Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S94094[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; TAB950 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94095[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S94096[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB462 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S94097[] = "Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 7 Build/1.0.3 X-12:51) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S94098[] = "Mozilla/5.0 (Linux; Android 4.0.4; M1062 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S94099[] = "Mozilla/5.0 (Linux; Android 4.2.1; ThL W8 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94100[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; LG-P690) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S94101[] = "Mozilla/5.0 (Linux; Android 4.1.2; SOL21 Build/9.1.D.0.395) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94102[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T869 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S94103[] = "Mozilla/5.0 (Linux; Android 4.0.4; L-01E Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94104[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us ; PD_Novel Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444";
const char S94105[] = "Mozilla/5.0 (Linux; Android 4.2.2; A700 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S94106[] = "Mozilla/5.0 (Linux; Android 4.0.4; Starmobile ASTRA Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94107[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A25 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S94108[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A21 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S94109[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P990 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94110[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-STYLE Q4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94111[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; ELEMENT10 1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S94112[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; SK17i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S94113[] = "AMOI N807 Linux/3.0.13 Android/4.0.4 Release/11.06.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;";
const char S94114[] = "GIPO-W919/1.0 Linux/2.6.35.7 Android/Android_4.0 Release/12.03.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S94115[] = "Mozilla/5.0 (X11; Linux x86_64; ja-jp; dtab01 Build/HuaweiMediaPad) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24 TT/4.1.2-X.X";
const char S94116[] = "Mozilla/5.0 (Linux; Android 4.0.3; PMP3270B Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S94117[] = "Mozilla/5.0 (Linux; Android 4.2.1; M6589 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94118[] = "Mozilla/5.0 (Linux; Android 4.2.1; Celkon A119Q Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94119[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S94120[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOLO_X1000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94121[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E220S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94122[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT25i Build/4.1.B.0.631) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S94123[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26w Build/6.2.B.0.200) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S94124[] = "Mozilla/5.0 (Linux; Android 4.0.4; U8800 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94125[] = "Mozilla/5.0 (Linux; Android 4.1.2; JY-G2 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94126[] = "Mozilla/5.0 (Linux; Android 3.2.1; AT100 Build/HTK55D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S94127[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T679 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94128[] = "Reeder/1.5.7 CFNetwork/655 Darwin/14.0.0";
const char S94129[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Celkon_A98) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S94130[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Micromax_A115) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S94131[] = "Mozilla/5.0 (Linux; Android 4.2.1; i-mobile IQ XA Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94132[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID7051 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S94133[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; U30GT-MS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S94134[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; LT22i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S94135[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S94136[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T769 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S94137[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S94138[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8268 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S94139[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A10 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S94140[] = "Mozilla/5.0 (Linux; Android 4.1.1; ENERGY s10 DUAL Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94141[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ro-ro; ALLVIEW_P6_Quad Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94142[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E270K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94143[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S94144[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S94145[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; N400 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S94146[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-I9003 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S94147[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8552 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300";
const char S94148[] = "LG-GW300/V100 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gw300) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S94149[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6760s-1) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S94150[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT18i) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S94151[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Titanium S1 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S94152[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Ctrl_V1 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S94153[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Celkon A107 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S94154[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LG-P690 Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S94155[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5238) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S94156[] = "Mozilla/5.0 (Linux; Android 4.1.1; AURUS III Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S94157[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T959 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S94158[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-P930 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S94159[] = "Mozilla/5.0 (Linux; Android 4.0.4; Arc Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S94160[] = "Mozilla/5.0 (Linux; Android 4.0.4; TECNO_N7 Build/TECNOTECNO_N7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S94161[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P935 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S94162[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.2.1; en-gb; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S94163[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; A111) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S94164[] = "NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311";
const char S94165[] = "Mozilla/5.0 (Linux; Android 4.1.2; VS930 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S94166[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A21) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S94167[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E440g Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94168[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8262 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S94169[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; SonyEricssonU1) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S94170[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A44) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S94171[] = "Mozilla/5.0 (Linux; Android 4.0.3; F-10D Build/V22R49C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S94172[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; Lenovo A278t Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S94173[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGP312 Build/10.3.1.C.0.136) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S94174[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S94175[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5570) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S94176[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC Desire X Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S94177[] = "Mozilla/5.0 (Linux; Android 4.2.1; fnac 5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S94178[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S94179[] = "Mozilla/5.0 (Linux; Android 3.2; GT-P7510 Build/HTJ85B) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S94180[] = "Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S94181[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR6425LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S94182[] = "SAMSUNG-SGH-L768/ZUHK3 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S94183[] = "T-Mobile Dash Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 320x240)";
const char S94184[] = "T-Mobile Dash Mozilla/4.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 320x240)";
const char S94185[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Xda_Guide/240x320";
const char S94186[] = "Mozilla/5.0 (Linux; U; Android Amstel v2.0; en-us; SPH-M900 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S94187[] = "Mozilla/5.0 (Linux; U; Android noobnl's Radioactive FroYo v1.8.6; en-us; SPH-M900 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S94188[] = "Mozilla/5.0 (Linux; U; Android SDX-Samdroid turbo v4.4; en-us; SPH-M900 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S94189[] = "Mozilla/5.0 (Linux; U; Android Ultimate Dream 3.2; en-us; SPH-M900 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S94190[] = "Mozilla/5.0 (Linux; U; Android noobnl-Radioactive-Froyo-v2.0pre3; en-us; SPH-M900 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S94191[] = "Mozilla/5.0 (Linux; U; Android bourineROM v1.1.5; en-us; SPH-M900 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S94192[] = "Mozilla/5.0 (Linux; U; Android noobnl Radioactive FroYo v1.8.22; en-us; SPH-M900 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S94193[] = "Mozilla/5.0 (Linux; U; Android noobnl's Radioactive FroYo v1.8.7; en-us; SPH-M900 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S94194[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Sprint MP6900SP";
const char S94195[] = "SonyEricssonZ550c/R6BA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S94196[] = "SonyEricssonZ550a/R6GA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S94197[] = "Nokia7200/2.0 (3.110) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S94198[] = "SAMSUNG-SGH-J808/ZWHI1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S94199[] = "Mozilla/5.0(Linux;U;Android2.1-update1;Zh_cn;GT540;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355";
const char S94200[] = "JUC(Linux;U;Android2.1-update1;Zh_cn;GT540;320*480;)UCWEB7.8.0.95/139/444";
const char S94201[] = "HTC-8500/1.2 Mozilla/4.0 (compatible; MSIE 5.5; Windows CE; PPC; 240x320)";
const char S94202[] = "LG-A258/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S94203[] = "Mozilla/4.0 (MobilePhone SCP-3100/CA/1.0) NetFront/3.1 MMP/2.0";
const char S94204[] = "Mozilla/4.0 (MobilePhone SCP-3100/US/1.0) NetFront/3.1 MMP/2.0";
const char S94205[] = "Nexian NX-G505/MTK Release/10.1.2009 browser/MAUI Profile/MIDP-2.0 configuration/CLDC-1.0";
const char S94206[] = "ZTE-T106/1.0 ACS-NF/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S94207[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile G1 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94208[] = "TIANYU-KTOUCH/H677/Screen-176X220";
const char S94209[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.24 (KHTML, like Gecko) Iron/11.0.700.2 Chrome/11.0.700.2 Safari/534.24";
const char S94210[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Iron/11.0.700.2 Chrome/11.0.700.2 Safari/534.24";
const char S94211[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; LG; LG-E900)";
const char S94212[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; LG; LG-E900h)";
const char S94213[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; LG; LG-E900; Orange)";
const char S94214[] = "HTC Streaming Player tmous / 1.0 / htc_doubleshot / 2.3.4";
const char S94215[] = "nokiac6-01/UC Browser7.9.0.102/69/352  3gpp-gba";
const char S94216[] = "Mozilla/5.0 (X11; FreeBSD amd64) KHTML/4.7.3 (like Gecko) Konqueror/4.7";
const char S94217[] = "SonyEricssonU10i/R7BA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonu10i/UC Browser8.0.3.107/69/444";
const char S94218[] = "MQQBrowser/30 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S94219[] = "MQQBrowser/30 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3";
const char S94220[] = "Mozilla/5.0 Opera/9.5 (KDDI-SN3S; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00";
const char S94221[] = "Opera/9.24 (X11; SunOS i86pc; U; en)";
const char S94222[] = "Mozilla/5.0 (compatible; Konqueror/4.5; Linux) KHTML/4.5.5 (like Gecko) Fedora/4.5.5-2.fc13";
const char S94223[] = "Facebook 4110.0 (iPod touch; iPhone OS 4.2.1; en_US)";
const char S94224[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94225[] = "Mozilla/5.0 (Linux; Android 4.0.4; TeamDRH ICS for GTablet Build/IML77) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94226[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:14.0) Gecko/X Firefox/14.0.1 SeaMonkey/2.11";
const char S94227[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; Ypy 7 - TB07FTA Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94228[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; TB07FTA; pt-br) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94229[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A52 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S94230[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94231[] = "Mozilla/5.0 (Linux; Android 4.2; LT18i Build/4.1.B.0.431) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94232[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S94233[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaX7-00)/UC Browser8.2.0.132/50/355/UCWEB Mobile";
const char S94234[] = "480x800; XV6875.1; Window Mobile 6.5 Professional; (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.6.1.82";
const char S94235[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)/UC Browser7.6.1.82";
const char S94236[] = "Opera/9.80 (BREW; Opera Mini/5.1/27.2239; U; en) Presto/2.8.119 400X240 Samsung SCH-U820";
const char S94237[] = "MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S94238[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S94239[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:0.9.2) Gecko/X Netscape6/6.1";
const char S94240[] = "iBrowser/2.5 (NokiaN95(08.01);SymbianOS/9.2 Series60/3.1)";
const char S94241[] = "Opera/8.51 (Windows NT 6.0; U; en)";
const char S94242[] = "Mozilla/5.0 (Linux; Android 4.0.4; 932i Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94243[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; WT19i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94244[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.1; en-in; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94245[] = "MQQBrowser/2.6 (E71-1;SymbianOS/9.2 Series60/3.1)";
const char S94246[] = "Mozilla/5.0 (Linux; Android 4.0.4; X906 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94247[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lava-a10) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S94248[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F120K Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94249[] = "SAMSUNG-GT-S7508_TD/1.0 Android/2.3.6 Release/11.30.2011 Browser/AppleWebKit533.1";
const char S94250[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.3.1.31155 (Firestorm-Release; firestorm skin) Safari/533.3";
const char S94251[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID9106CM Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94252[] = "Mozilla/5.0 (Linux; Android 4.0.3; ATLANTIS Build/NGMNGM_ATLANTIS) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94253[] = "Mozilla/5.0 (Linux; Android 4.0.3; BC9710AM Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94254[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) Sleipnir/2.49";
const char S94255[] = "Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Kem 452 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94256[] = "Opera/9.80 (Windows NT 5.1; Opera Mobi/46154) Presto/2.11.355 Version/12.10";
const char S94257[] = "JUC (Linux; U; 2.3.6; zh-cn; N350; 320*480) UCWEB7.9.0.94/139/444";
const char S94258[] = "Mozilla/5.0 (Linux; Android 4.1.1; A8809 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94259[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; U20i Build/IMM76L; CyanogenMod-MiniCM9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94260[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; U20i Build/IMM76I; CyanogenMod-MiniCM9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94261[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5300 Build/GINGERBREAD; 240*320) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444";
const char S94262[] = "Opera/9.80 (Windows NT 6.2; WOW64; Opera Mobi/46154) Presto/2.11.355 Version/12.10";
const char S94263[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; pandigitalsprnova1/sourceidDLX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94264[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT-AS45IPS Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94265[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-SU870 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94266[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; MT25i Build/4.1.B.0.631) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94267[] = "MQQBrowser/40 Mozilla/5.0 (iPhone 4; U; CPU iPhone OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3 tencent(securitypay)";
const char S94268[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; S100 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S94269[] = "Mozilla/5.0 (Linux; Android 4.0.4; CAL21 Build/AX) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S94270[] = "JUC (Linux; U; 2.3.4; zh-cn; MK16i; 480*854) UCWEB7.9.0.94/139/444";
const char S94271[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC One X Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S94272[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; U20i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S94273[] = "Mozilla/5.0 (Linux; Android 4.0.4; L-01E Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S94274[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;Radar C110e;7.10;8862)";
const char S94275[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;Radar 4G;7.10;8862)";
const char S94276[] = "Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-55EX645; CC/CAN; en) Presto/2.10.250 Version/11.60";
const char S94277[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:19.0) Gecko/X Firefox/19.0 slurp";
const char S94278[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; UBISLATE7C+ Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S94279[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LT26ii Build/6.1.A.2.45) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S94280[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; SCH-I699 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S94281[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Desire HD Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (Desire HD, 34, 1.6.0)";
const char S94282[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (Desire HD, 42, 1.9.1)";
const char S94283[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; V9S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S94284[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk810i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S94285[] = "Mozilla/5.0 (Linux; Android 4.0.4; Fly_IQ442 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94286[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I8150 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94287[] = "Mozilla/5.0 (Linux; Android 4.0.4; Starmobile CRYSTAL Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94288[] = "Mozilla/5.0 (Linux; Android 4.1.2; UMI X1 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94289[] = "Mozilla/5.0 (Linux; Android 4.0.4; A700 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S94290[] = "iBrowser/2.6 (NokiaN8-00;SymbianOS/9.1 Series60/3.0)";
const char S94291[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAD-10021 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94292[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-be; Lenco TAB-832 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S94293[] = "Mozilla/5.0 (Linux; Android 4.0.3; Scroll Extreme Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S94294[] = "Mozilla/5.0 (Linux; Android 4.1.2; H7500+ Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S94295[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-i9377 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94296[] = "Mozilla/5.0 (Linux; Android 4.0.3; TAB275 Build/TAB275) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S94297[] = "Mozilla/5.0 (Linux; Android 4.0.4; Miracle Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94298[] = "OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S94299[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6802 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S94300[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-7240 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S94301[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALLVIEW_TX1_Quasar Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S94302[] = "LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gd510) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S94303[] = "Mozilla/5.0 (Linux; Android 4.0.4; N-07D Build/AX) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94304[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9300 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S94305[] = "Mozilla/5.0 (Linux; Android 4.0.4; TM-7024 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94306[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ALCATEL_ONE_TOUCH_930D) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S94307[] = "Mozilla/5.0 (Linux; Android 4.0.3; F-10D Build/V21R48A) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94308[] = "Mozilla/5.0 (Linux; Android 4.1.1; eZee Tab805 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S94309[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S94310[] = "Mozilla/5.0 (Linux; Android 2.3.6; LG-C800 Build/GRK39F) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S94311[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:5.0) Gecko/X Firefox/5.0 SeaMonkey/2.2";
const char S94312[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (SAMSUNG-SGH-I997, 42, 1.9.1)";
const char S94313[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94314[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94315[] = "UCWEB/2.0 (Linux; U; Adr 2.2; en-US; HTC_Legend) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S94316[] = "Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.6.5/X (like Mobile/8C148 Safari/6533.18.5)";
const char S94317[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Byond B61 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94318[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC Desire V Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S94319[] = "Mozilla/5.0 (Linux; Android 2.3.6; SCH-R760 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S94320[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7310 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S94321[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9000 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S94322[] = "Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 7 Build/1.0.3 X-12:51) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S94323[] = "Mozilla/5.0 (Linux; Android 4.0.4; A2 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94324[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB469 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S94325[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F220K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94326[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S94327[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S94328[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I847 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94329[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HS-EG909 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94330[] = "Mozilla/5.0 (Unknown; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) CasperJS/1.0.2+Phantomjs/1.8.1 Safari/534.34";
const char S94331[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GenxDroid7) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S94332[] = "Mozilla/5.0 (Linux; Android 4.2.1; ZP950+ Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94333[] = "Mozilla/5.0 (Linux; Android 4.1.2; N9810 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94334[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; Micromax A116 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S94335[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Maxthon/4.0.6.2000)";
const char S94336[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; ZTE U795 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S94337[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A57) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S94338[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A45) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S94339[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A8 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S94340[] = "MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S7t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94341[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030X Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94342[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A7* Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33756";
const char S94343[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7";
const char S94344[] = "Mozilla/5.0 (Linux; Android 4.2.1; iDxD8 3G Build/JB) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94345[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A4+ Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S94346[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5312 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S94347[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; A19 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94348[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; INTEX CLOUD X1) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S94349[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC Wildfire S A510e) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S94350[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S94351[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-us; PC36100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S94352[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; ALCATEL ONE TOUCH 4030D Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S94353[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; IRIS 455 Build/LAVAIRIS455) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S94354[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A62 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S94355[] = "Mozilla/5.0 (Linux; Android 4.0.4; PNDPP48GP Build/ICS.PNDPP48GP.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S94356[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A26 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S94357[] = "Mozilla/5.0 (Linux; Android 4.2.1; Spice Mi-535 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S94358[] = "Mozilla/5.0 (Linux; Android 4.0.4; YP-GP1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S94359[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E445 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S94360[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A52) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S94361[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw150i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S94362[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5230) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S94363[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-s5230) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S94364[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I437 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S94365[] = "Mozilla/5.0 (Linux; Android 4.1.2; NEBULA6.9 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S94366[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-in; A111 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S94367[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; Multilaser Titanium Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94368[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; Karbonn A50 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33756";
const char S94369[] = "Googlebot/2.1 (+http://www.googlebot.com/bot.html; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB6; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET C;c2VvLWNyYXdsZXI=";
const char S94370[] = "Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_S9714 Build/JRO03R) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S94371[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S94372[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S94373[] = "Mozilla/5.0 (Linux; Android 4.0.3; ThinkPad Tablet Build/ThinkPadTablet_A400_03) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S94374[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; ADR6425LVW Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/vigor";
const char S94375[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus 4 Build/JWR66Y) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.20.1364.172 Mobile Safari/537.22";
const char S94376[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; Micromax_A60) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S94377[] = "Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S94378[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S94379[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; GT-B7510) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S94380[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; Nokia N96) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S94381[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_One_V) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S94382[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S94383[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SPH-L720 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S94384[] = "Mozilla/5.0 (Linux; Android 4.0.4; YP-GP1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S94385[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9003 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S94386[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Micromax A116) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S94387[] = "Mozilla/5.0 (Series40; NokiaC1-01/06.15; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11";
const char S94388[] = "LG-KE970/V10e Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S94389[] = "SonyEricssonK750i";
const char S94390[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_E100";
const char S94391[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_E100 / 2.004.00_IT_vodafone";
const char S94392[] = "SAMSUNG-SGH-i710/MSIE4.01/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8)";
const char S94393[] = "HTC_P3700 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S94394[] = "SonyEricssonW600i/R7CA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S94395[] = "Nokia7250/1.0 (3.14) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S94396[] = "SAMSUNG-GT-S5520/S5520ZWKA1 SHP/VPP/R5 Dolfin/2.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S94397[] = "SonyEricssonK550i";
const char S94398[] = "SEC-SGHM600/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S94399[] = "MOT-W510/08.11.04R MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/1.0";
const char S94400[] = "LG-KP260 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S94401[] = "LG/U880/v1.0";
const char S94402[] = "SAMSUNG-SGH-T729/T729UVGF9 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S94403[] = "SAGEM-VF533/1.0/EVN_ZO MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.1.423 (GUI)";
const char S94404[] = "SAGEM-VF533/1.0/EVN_ZN MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.1.423 (GUI)";
const char S94405[] = "SAGEM-VF533/1.0/EVN_ZP MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.1.423 (GUI)";
const char S94406[] = "SAGEM-VF533/1.0/EVN_ZJ MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.1.423 (GUI)";
const char S94407[] = "SAGEM-VF533/1.0/EVN_ZI MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.1.423 (GUI)";
const char S94408[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; MDA Compact/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1)";
const char S94409[] = "MOT-A810/1.0 LinuxOS/2.4.20 Release/6.10.2008 Browser/Opera8.00 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/GSM_E28.24.1_G_11.02.18R";
const char S94410[] = "LG/U400/v1.0";
const char S94411[] = "NOKIA5630d-1";
const char S94412[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; Motorola-XT502 Build/DONUT) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S94413[] = "NokiaC7-00/UC Browser7.6.1.82/50/352";
const char S94414[] = "nokiac7-00/UC Browser7.6.1.82/69/351  3gpp-gba";
const char S94415[] = "Lenovo-P629/S118 LMP/LML Release/2010.04.22 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S94416[] = "TIANYU-KTOUCH/H688/Screen-176X220";
const char S94417[] = "Mozilla/5.0 (Linux; U; Android Honeycomb; en-us; Build/Nook) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S94418[] = "Mozilla/5.0 (Linux; U; Android 3.0; en-us; sdk Build/HONEYCOMB) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S94419[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; X10i Build/Cyanogen6.1.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94420[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; X10i Build/CyanogenMod 6.1.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94421[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; ASUS T20 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S94422[] = "ASUS_T20_TD/1.0 OPhone/2.5 (Linux; Android 2.2) Release/12.23.2009 Browser/WAP2.0 (AppleWebKit/533.1) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S94423[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.1.0.0 Chrome/17.0.963.38 Safari/535.11";
const char S94424[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; XT702 Build/SGC_U6_10.13.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94425[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; XT702 Build/SGC_U6_10.13.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S94426[] = "Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US; rv:1.8.1.13) Gecko/X Fedora/1.1.9-2.fc7 SeaMonkey/1.1.9";
const char S94427[] = "Mozilla/5.0 (Series40; Nokia5300/07.20; Profile/MIDP-2.0 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S94428[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.1.16) Gecko/X Thunderbird/3.0.11";
const char S94429[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6300 Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94430[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; ADR6300 Build/GRJ22)";
const char S94431[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) iCab/4.8 Safari/533.16";
const char S94432[] = "Mozilla/5.0 (Macintosh; U; Mac OS X 10_6_1; en-US) AppleWebKit/530.5 (KHTML, like Gecko) Chrome/ Safari/530.5";
const char S94433[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;USCCHTC-PC93100;7.10;7720)";
const char S94434[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;USCCHTC-PC93100;7.10;8107)";
const char S94435[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;7 Pro T7576;7.10;8107)";
const char S94436[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;7 Pro T7576;7.10;8773)";
const char S94437[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;7 Trophy;7.10;8107)";
const char S94438[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;7 Trophy T8686;7.10;8107)";
const char S94439[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;7 Trophy;7.10;7720)";
const char S94440[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;7 Trophy;7.10;7740)";
const char S94441[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; TR718DC_V21, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94442[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; KATBL08AND Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94443[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; KATBL08ANDC Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94444[] = "Reeder/1010.79.00 CFNetwork/596.1 Darwin/12.1.0 (x86_64) (MacBookAir5%2C2)";
const char S94445[] = "Reeder/1010.79.00 CFNetwork/596.1 Darwin/12.1.0 (x86_64) (iMac12%2C2)";
const char S94446[] = "Reeder/1010.79.00 CFNetwork/596.1 Darwin/12.1.0 (x86_64) (MacBookPro5%2C5)";
const char S94447[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn;Lenovo S760 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S94448[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94449[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94450[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94451[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Incredible S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94452[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Incredible S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94453[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A65 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94454[] = "JUC(Linux;U;Android2.3.7;Zh_cn;LT26i;720*1280;)UCWEB7.8.0.95/139/444";
const char S94455[] = "Mozilla/5.0 (Linux; Android 4.0.3; Scroll Excel Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94456[] = "nokia5230/UC Browser7.9.0.102/69/352  3gpp-gba";
const char S94457[] = "Nokia2700c-2/2.0 (09.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/70/352";
const char S94458[] = "HTC_Touch_Pro_T7272 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S94459[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC One X Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S94460[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P970 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S94461[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s8530) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S94462[] = "nokia5230/UC Browser8.0.3.107/69/444  3gpp-gba";
const char S94463[] = "MQQBrowser/3.2 (Nokia5230;SymbianOS/9.1 Series60/3.0)";
const char S94464[] = "JUC(Linux;U;Android2.3.5;Zh_cn;Micromax A57;320*480;)UCWEB7.8.0.95/139/444";
const char S94465[] = "Mozilla/5.0 (Linux; Android 4.0.4; KATBL10A16DA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94466[] = "Mozilla/5.0 (Linux; Android 4.0.4; Q-Smart S18 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94467[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94468[] = "SAMSUNG-GT-P3108_TD/1.0 Android/4.0.3 Release/02.15.2012 Browser/AppleWebKit534.30";
const char S94469[] = "Mozilla/5.0 (Linux; Android 4.0.4; SBM107SH Build/S0010) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94470[] = "Instagram 3.0.1 (iPod touch; iPhone OS 4.2.1; en_US) AppleWebKit/420+";
const char S94471[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E400 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94472[] = "Mozilla/5.0 (Linux; Android 4.1.1; M-MP101S2 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94473[] = "Mozilla/5.0 (Linux; Android 4.2.1; A700 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94474[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P698 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S94475[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-I9103 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94476[] = "Mozilla/5.0 (Linux; Android 4.0.4; V900A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94477[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000 Build/MIUI) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94478[] = "Mozilla/5.0 (Linux; Android 4.0.4; AQUA 5.0 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94479[] = "Mozilla/5.0 (Linux; Android 4.0.4; onetouch EVO7 Build/MAIN_01) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94480[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-STYLE Q4 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94481[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:19.0) Gecko/X Thunderbird/19.0";
const char S94482[] = "SAMSUNG-GT-C3520/1.0 RTK-E/1.0 DF/1.0 Release/07.15.2011 Browser/NetFront4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S94483[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT500 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94484[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT500a Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94485[] = "OneBrowser/3.1 (Nokia112/03.26)";
const char S94486[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; A75 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S94487[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GN100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94488[] = "JUC (Linux; U; 2.3.6; zh-cn; Aqua_Sx; 320*480) UCWEB7.9.0.94/139/444";
const char S94489[] = "JUC (Linux; U; 2.3.6; zh-cn; LG-E405; 240*320) UCWEB7.9.0.94/139/444";
const char S94490[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; ZTE-BLADE Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S94491[] = "Mozilla/5.0 (Linux; Android 4.2; HTC6435LVW Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S94492[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P7100 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94493[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7100 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94494[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P3110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S94495[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-P3110 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S94496[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Classpad Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S94497[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; TM-7023 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S94498[] = "Mozilla/5.0 (Linux; Android 4.0.3; Newman N2 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94499[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; R800i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S94500[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC ChaCha A810e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S94501[] = "Mozilla/5.0 (Linux; Android 4.0.4; LGL21 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S94502[] = "Mozilla/5.0 (Linux; Android 4.0.4; VIVO 4.3 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S94503[] = "Mozilla/5.0 (Linux; Android 4.0.4; Starmobile CRYSTAL Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S94504[] = "Mozilla/5.0 (Linux; Android 4.0.4; Starmobile ASTRA Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S94505[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V889M Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S94506[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I8262D Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S94507[] = "JUC (Linux; U; 4.1.1; zh-cn; HUAWEI_C8813; 480*854) UCWEB7.9.0.94/139/800";
const char S94508[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAN80) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S94509[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S94510[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA3250) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S94511[] = "Mozilla/5.0 (Linux; Android 4.1.2; IQ446 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94512[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZP300+ Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94513[] = "Mozilla/5.0 (Linux; Android 4.1.1; SH930W Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94514[] = "Mozilla/5.0 (Linux; Android 4.0.4; Andi4.5h Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94515[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P935 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94516[] = "Mozilla/5.0 (Windows NT 6.0; rv:21.0) Gecko/X Thunderbird/21.0";
const char S94517[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-S7560M Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:38, ca";
const char S94518[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; IdeaTab A2107A-H Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S94519[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:20.0) Gecko/X Firefox/20.0 slurp";
const char S94520[] = "Mozilla/5.0 (Linux; Android 4.2; HTC6435LVW Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94521[] = "Mozilla/5.0 (Linux; Android 4.1.1; chagall Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S94522[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax P275 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S94523[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V889M Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94524[] = "Mozilla/5.0 (Linux; Android 4.0.4; eZee'Tab1001 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94525[] = "Mozilla/5.0 (Linux; Android 4.0.3; eZee'Tab1001 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94526[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT-I5700) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S94527[] = "Mozilla/5.0 (Linux; Android 4.1.2; L-04E Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94528[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3653w) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S94529[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A1+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S94530[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_EVO_3D_X515m) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S94531[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk790i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S94532[] = "Mozilla/5.0 (Linux; Android 2.3.4; T-Mobile G2 Build/GRJ22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S94533[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; YP-G70 Build/IMM76L; CyanogenMod-Electric Catfish Mk 0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94534[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HIMAX PURE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94535[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N8000) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S94536[] = "Mozilla/5.0 (Linux; Android 4.0.4; IMO S88 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94537[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-SU640 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94538[] = "Mozilla/5.0 (Linux; Android 3.2.1; K1 Build/HTK75C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S94539[] = "Mozilla/5.0 (Linux; Android 4.1.1; Loox Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94540[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; RAPAX090-0508 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94541[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; i-mobile IQ 6 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94542[] = "Mozilla/5.0 (Linux; Android 4.0.4; 86VEBC Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94543[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC One XL Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S94544[] = "Mozilla/5.0 (Linux; Android 4.0.4; MEDION P4013 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94545[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS920 4G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S94546[] = "Mozilla/5.0 (Linux; Android 4.0.4; Titan TV S320 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94547[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; G1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S94548[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Karbonn_A30) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S94549[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT26i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S94550[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A840S Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94551[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB-730 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S94552[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5263) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S94553[] = "JUC (Linux; U; 2.3.5; zh-cn; Coolpad_7020; 320*480) UCWEB7.9.0.94/139/33619";
const char S94554[] = "Mozilla/5.0 (Linux; Android 4.1.2; L-01E Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94555[] = "Nokia303/5.0 (14.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia303) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S94556[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID1065 Build/ICS.MID1065.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S94557[] = "Mozilla/5.0 (Linux; Android 4.1.2; ADR910L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94558[] = "Mozilla/5.0 (Linux; Android 4.2.2; SHV-E330S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94559[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; T-Mobile G2 Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S94560[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST70104-1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94561[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; SCH-I959 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S94562[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R811 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S94563[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M032 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S94564[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-e2202) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S94565[] = "Mozilla/5.0 (Linux; Android 4.0.4; MK16i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S94566[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9152 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S94567[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; HS-9DTB4) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S94568[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PMP3470B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S94569[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Starmobile Engage7 3G Build/ICS_IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94570[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; ru-ru; ZTE Grand X Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94571[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S94572[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LT15i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S94573[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire U dual sim Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S94574[] = "Mozilla/5.0 (Linux; Android 4.2.1; ZTE Grand X Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94575[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; SPH-D710 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S94576[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL ONE TOUCH 985N) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S94577[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Titanium S5) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S94578[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9152 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94579[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A30 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S94580[] = "Nokia311/5.0 (07.36) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291";
const char S94581[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Aqua I-5 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S94582[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A65 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S94583[] = "Mozilla/5.0 (Linux; Android 4.0.4; U30GT-H Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S94584[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-MS770 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S94585[] = "Mozilla/5.0 (Linux; Android 4.0.4; Philips W832 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S94586[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6110) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S94587[] = "Mozilla/5.0 (Linux; Android 4.1.1; chagall Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S94588[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 5 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S94589[] = "Mozilla/5.0 (Linux; Android 4.2.2; M8 pro Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S94590[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian70-1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S94591[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian70) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S94592[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E986 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94593[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; HUAWEI_Y300-0100) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S94594[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC PH39100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S94595[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; ST18i) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S94596[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8262 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300";
const char S94597[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S94598[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.6) Gecko/X diffbot/0.3.7 (http://www.diffbot.com) Firefox/3.6.6 (.NET CLR 3.5.30729);c2VvLWNyYXdsZXI=";
const char S94599[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Titanium_S5) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S94600[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A789 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S94601[] = "Mozilla/5.0 (Linux; Android 4.1.1; ME172V Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S94602[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S94603[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx600) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S94604[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; TECNO D3 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S94605[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST15i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S94606[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S94607[] = "LG-GW300/V100 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gw300) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S94608[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; Micromax_A51) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S94609[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5113 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S94610[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; WT19i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S94611[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; U20i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S94612[] = "UCWEB/2.0 (Linux; U; Adr 2.2; en-US; GT-I5510) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S94613[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Q700 Build/XOLOQ700) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S94614[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A27) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S94615[] = "Mozilla/5.0 (Linux; Android 4.1.2; TRUE BEYOND 3G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S94616[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; GT-I9502 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S94617[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-P990 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S94618[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Micromax A72 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S94619[] = "Mozilla/5.0 (Linux; Android 4.1.1; KUNO4 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S94620[] = "Mozilla/5.0 (Linux; Android 4.1.2; PantechP8010 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S94621[] = "Mozilla/5.0 (Linux; Android 4.0.4; Event Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S94622[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC P5500; OpVer 25.131.1.6109";
const char S94623[] = "HTC-P5500-orange/PPC; 240x320; OpVer 25.131.2.7309 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S94624[] = "HTC_P5500 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S94625[] = "HTC P5500/1.31.621.09 Mozilla/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S94626[] = "SAMSUNG-SGH-X540/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S94627[] = "LG-KM335 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S94628[] = "SAMSUNG-SGH-E350E/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S94629[] = "SAMSUNG-SGH-E350/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S94630[] = "Alcatel-OT-600A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S94631[] = "Alcatel-GLAMPHONE-ELLE-A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S94632[] = "Alcatel-OT-S321A/1.0 ObigoInternetBrowser/Q03C";
const char S94633[] = "Alcatel-OT-S626A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S94634[] = "Alcatel-OT-C820/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S94635[] = "Alcatel-OT-360A/1.0 ObigoInternetBrowser/Q03C";
const char S94636[] = "LGE-AX585/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S94637[] = "PPC; 240x320; HTC_P3451/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S94638[] = "LG-KP130 Obigo/WAP2.0";
const char S94639[] = "LG-KP130-orange/V10c Obigo/WAP2.0";
const char S94640[] = "SIE-C75/11 UP.Browser/7.0.2.2.d.5(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S94641[] = "SIE-C75/13 UP.Browser/7.0.2.2.d.7(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S94642[] = "SIE-C75/24 UP.Browser/7.0.2.2.d.7(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S94643[] = "SIE-C75/22 UP.Browser/7.0.2.2.d.7(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S94644[] = "HTC_X7510 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S94645[] = "SonyEricssonT630/R601 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S94646[] = "HUAWEI-M328/001.00 ACS-NetFront/3.2";
const char S94647[] = "SAMSUNG-GT-S3600i/1.0 RTK-E/1.0 DF/1.0 Release/01.09.2009 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S94648[] = "SAMSUNG-SGH-A711/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S94649[] = "LG-LG100 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S94650[] = "NokiaE71/UC Browser7.2.2.51/28/352";
const char S94651[] = "SAMSUNG-GT-C3518/1.0 RTK-E/1.0 DF/1.0 Release/22.12.2009 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S94652[] = "SAMSUNG-GT-C3518/1.0 RTK-E/1.0 DF/1.0 Release/22.12.2009 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S94653[] = "SCH-U900/1.0 NetFront/3.0.22.2.9 (GUI) MMP/2.0";
const char S94654[] = "NokiaE71/UC Browser7.4.0.65/28/352";
const char S94655[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; Google Ion Build/CRB43) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S94656[] = "Samsung-SCHR351 Infraware-Polaris/6.0 MMP/2.0";
const char S94657[] = "DoCoMo/2.0 F03A(c100;TB;W24H17)";
const char S94658[] = "NokiaN8-00/SymbianOS/9.1 Series60/3.0 3gpp-gba";
const char S94659[] = "SEC-SGHX160/1.0";
const char S94660[] = "Nokia6681/2.0 (6.09.00) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.7.1.88/27/444";
const char S94661[] = "Nokia6303classic/2.0 (09.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303classic/UC Browser7.9.0.102/69/352";
const char S94662[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; O2 Xda II;PPC;240x320)";
const char S94663[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; O2 Xda 2mini; PPC; 240x320)";
const char S94664[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; O2 Xda 2s;PPC;240x320; PPC; 240x320)";
const char S94665[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; O2 Xda 2i; PPC; 240x320)";
const char S94666[] = "Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Nokia N70/2.0537.1.7; 6169) Opera 8.00 [en]";
const char S94667[] = "KDDI-SN3B UP.Browser/6.2.0.13.1.5 (GUI) MMP/2.0";
const char S94668[] = "KDDI-KC3B UP.Browser/6.2.0.13.2 (GUI) MMP/2.0";
const char S94669[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.8 (KHTML, like Gecko) Ubuntu/9.04 Chromium/7.0.529.0 Chrome/7.0.529.0 Safari/534.8";
const char S94670[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.9 (KHTML, like Gecko) Ubuntu/9.04 Chromium/7.0.530.0 Chrome/7.0.530.0 Safari/534.9";
const char S94671[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/1.9.1 Safari/533.1";
const char S94672[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en_gb) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/1.9.4 Safari/533.1";
const char S94673[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/1.9.4 Safari/533.1";
const char S94674[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en_gb) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/1.9.4 Safari/533.1";
const char S94675[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Xoom Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94676[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.19pre) Gecko/X Prism zdesktop/7.1.4";
const char S94677[] = "Apple-PubSub/65.24";
const char S94678[] = "Apple-PubSub/59";
const char S94679[] = "Mozilla/5.0 (Windows NT 6.1; U; en; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 11.11";
const char S94680[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; en) Opera 11.11";
const char S94681[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A200 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S94682[] = "Opera/9.00 (Nintendo Wii; U; ; 1309-9; en)";
const char S94683[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.0m";
const char S94684[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3 Sleipnir/2.0m";
const char S94685[] = "Opera/9.80 (X11; FreeBSD 8.2-RELEASE i386; U; en) Presto/2.8.131 Version/11.10";
const char S94686[] = "Mozilla/5.0 Opera/9.5 (KDDI-CA3I; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00";
const char S94687[] = "HTC_P3650 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; HTC_P3650)/UC Browser7.8.0.95";
const char S94688[] = "HTC_Touch_Cruise_T4242 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S94689[] = "Mozilla/5.0 (Macintosh; U; Macintosh; en-US; Valve Steam GameOverlay/X; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11";
const char S94690[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94691[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Desire HD Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94692[] = "JUC (Linux; U; 2.2.2; zh-cn; ZTE-C_N780; 320*480) UCWEB7.9.3.103/139/32580";
const char S94693[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; LG-E400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S94694[] = "Mozilla/5.0 (Linux; U; Android 1.0.0; es-es; Mobii 7 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S94695[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94696[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC ChaCha A810e Build/GRJ90; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94697[] = "JUC(Linux;U;Android2.3.6;Zh_cn;LG-E400;240*320;)UCWEB7.8.0.95/139/444";
const char S94698[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; DROIDZ Mate Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94699[] = "sonyericssonw150i/UC Browser8.0.3.107/69/444";
const char S94700[] = "MQQBrowser/2.1 Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3";
const char S94701[] = "MQQBrowser/2.1 Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S94702[] = "Mozilla/5.0 (Java; U; en-in; nokia6233) UCBrowser8.4.0.159/70/444/UCWEB Mobile";
const char S94703[] = "MQQBrowser/3.1 (Nokia5230;SymbianOS/9.1 Series60/3.0)";
const char S94704[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; de-DE; rv:1.0.2) Gecko/X Netscape/7.01";
const char S94705[] = "LANIX-LX11/R01/MT53 Browser/2.0/Obigo/Q03C Profile/MIDP-2.0/CLDC-1.1";
const char S94706[] = "iBrowser/2.7 (Nokia500;SymbianOS/9.1 Series60/3.0)";
const char S94707[] = "MQQBrowser/2.8 (NokiaN8-00;SymbianOS/9.1 Series60/3.0)";
const char S94708[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE65)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S94709[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia700) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile";
const char S94710[] = "JUC (Linux; U; 4.0.3; zh-cn; ELF-II; 1024*552) UCWEB1.0.1.113/147/800";
const char S94711[] = "Mozilla/5.0 (Linux; Android 4.0.3; STM803HC Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94712[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-KU5900 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94713[] = "Mozilla/5.0 (Java; U; en-us; micromaxq50) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S94714[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94715[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A100/Micromax_A100_V1.0; 480*854;  CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94716[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506) Sleipnir/2.6.1";
const char S94717[] = "iBrowser/2.6 (NokiaX6-00;SymbianOS/9.1 Series60/3.0)";
const char S94718[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94719[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94720[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SmartTab1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S94721[] = "Mozilla/5.0 (X11; FreeBSD amd64) KHTML/4.8.4 (like Gecko) Konqueror/4.8";
const char S94722[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46HX853; CC/GBR; en) Presto/2.10.250 Version/11.60";
const char S94723[] = "Mozilla/5.0 (Linux; Android 4.0.4; DM013SH Build/S0006) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94724[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Comodo_Dragon/10.0.0.2 Chrome/10.0.648.204 Safari/534.16";
const char S94725[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; Sleipnir/2.9.1)";
const char S94726[] = "JUC (Linux; U; 2.3.6; zh-cn; A95; 240*320) UCWEB7.9.0.94/139/444";
const char S94727[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94728[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A810S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94729[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:19.0) Gecko/X Firefox/19.0";
const char S94730[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7108 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94731[] = "OneBrowser/3.0 (Nokia200/11.56)";
const char S94732[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Incredible S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94733[] = "Mozilla/5.0 (Linux; Android 4.0.3; i-mobile i-style Q1i Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94734[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7108 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94735[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:8.0) Gecko/X Thunderbird/8.0";
const char S94736[] = "Mozilla/5.0 (Linux; Android 4.0.4; SO-02D Build/6.1.F.0.106) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S94737[] = "Mozilla/5.0 (Linux; Android 4.0.4; dL1 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S94738[] = "YahooMobileMessenger/1.0 (Android Messenger; 1.8.3) (GT-P6200; samsung; GT-P6200; 3.2/HTJ85B)";
const char S94739[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7510, 39, 1.8.2)";
const char S94740[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.0.2) Gecko/X Netscape/7.02";
const char S94741[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.2.32) Gecko/X Firefox/3.6.32 (Palemoon/3.6.32)";
const char S94742[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Woxter Tablet PC 85 HDS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S94743[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; CMP741E Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S94744[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; C5155 Build/IML77) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S94745[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A111 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S94746[] = "Mozilla/5.0 (Linux; Android 4.1.1; WEXLER-TAB-7T Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S94747[] = "Mozilla/5.0 (Linux; Android 4.1.2; NexusHD2 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S94748[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S94749[] = "Mozilla/5.0 (Linux; Android 4.0.4; CAL21 Build/AX) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94750[] = "Mozilla/5.0 (Linux; Android 4.1.1; POV_TAB-PROTAB30-IPS10 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94751[] = "Nokia5330-1d/5.0 (06.88) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5330-1d) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S94752[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t310i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S94753[] = "UCWEB/2.0 (Java; U; MIDP-2.0; vi; lg-t310i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S94754[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq80) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S94755[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8951 Build/HuaweiT8951) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S94756[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; A25 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94757[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3105 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S94758[] = "Mozilla/5.0 (Linux; Android 4.1.1; M032 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S94759[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Mobile Safari/535.19 Silk-Accelerated=false";
const char S94760[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAP5000TDUO Build/PrestigioPAP5000TDUO) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94761[] = "Mozilla/5.0 (Linux; Android 4.0.4; A5Classic Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94762[] = "Mozilla/5.0 (Linux; Android 4.0.3; ME171 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S94763[] = "Mozilla/5.0 (Linux; Android 4.1.2; Flame2 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94764[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MK16i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S94765[] = "Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 7 Build/1.0.3 X-12:51) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S94766[] = "Mozilla/5.0 (Linux; Android 4.0.6; A9550 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S94767[] = "Mozilla/5.0 (Linux; Android 4.0.4; IRIS_501 Build/LAVAIRIS501) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94768[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; S100) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S94769[] = "Mozilla/5.0 (Linux; Android 4.0.4; rk30sdk Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S94770[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; M040 Build/JRO03H) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S94771[] = "UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; HTC_Salsa_C510e) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S94772[] = "LG-GW525 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gw525 teleca) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S94773[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A111) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S94774[] = "Mozilla/5.0 (Linux; Android 4.1.1; O+ 8.52 Android Build/Oplus8.52_Android) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94775[] = "Mozilla/5.0 (Linux; Android 4.0.3; Slider SL101 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S94776[] = "Mozilla/5.0 (Windows NT 6.0; rv:22.0) Gecko/X Thunderbird/22.0";
const char S94777[] = "SAMSUNG-GT-C3780/C3780JVLF2 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S94778[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA6760s-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile";
const char S94779[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7568 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94780[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-E970 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94781[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC T528d Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S94782[] = "Mozilla/5.0 (Linux; Android 4.1.2; Novo7Fire Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S94783[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; AMAZE_S200) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S94784[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 505)";
const char S94785[] = "Mozilla/5.0 (Linux; Android 4.0.3; P300(Funbook) Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S94786[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6110) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S94787[] = "Mozilla/5.0 (Linux; Android 4.0.4; A89 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S94788[] = "Mozilla/5.0 (Linux; Android 4.1.1; Lenovo P770 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S94789[] = "Mozilla/5.0 (Linux; Android 4.1.1; Phoenix Build/X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S94790[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I8150 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94791[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I897 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94792[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6833 Build/14.1.B.1.320) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94793[] = "Mozilla/5.0 (Linux; Android 4.2.1; H7500+ Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94794[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xperia Arc S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94795[] = "Mozilla/5.0 (Linux; Android 4.1.1; T-17B Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S94796[] = "SAMSUNG-GT-S5698_TD/1.0 Android/2.3.6 Release/1.15.2012 Browser/AppleWebKit533.1";
const char S94797[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaC5-05";
const char S94798[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALLVIEW_TX1_Quasar Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S94799[] = "Mozilla/5.0 (Linux; Android 4.0.4; A15 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S94800[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; zh-CN; GT-S5570) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile";
const char S94801[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M380W Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S94802[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Micromax A88) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S94803[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5230) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S94804[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-s5230) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S94805[] = "UCWEB/2.0 (Java; U; MIDP-2.0; es-LA; gt-s5230) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S94806[] = "UCWEB/2.0 (Java; U; MIDP-2.0; fa; gt-s5230) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S94807[] = "Mozilla/5.0 (Linux; Android 4.1.2; GL07S Build/HuaweiU9700L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94808[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-M919 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.27/385,1.9.27.385,2013-03-31 15:31, +0000, us";
const char S94809[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717M Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94810[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; i-mobile i-style 8 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94811[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; VS930 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S94812[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT15i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S94813[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; ADR6350 Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S94814[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-B5512) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S94815[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Nexus S Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S94816[] = "Mozilla/5.0 (Linux; Android 4.2.2; KYY21 Build/100.0.2480) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94817[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-S5360 Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60015";
const char S94818[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; ZTE Grand Era Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94819[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ADR6350 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94820[] = "Mozilla/5.0 (X11; FreeBSD) AppleWebKit/535.22 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/535.22 Midori/0.5";
const char S94821[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; G2 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33886";
const char S94822[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S94823[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9128 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S94824[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq7) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S94825[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; SGH-T989 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S94826[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E10i) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S94827[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Karbonn A4 ) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S94828[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Fly_IQ444 Quattro Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94829[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-S6812 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S94830[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; A111) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S94831[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A12) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S94832[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA5700) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S94833[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E270S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S94834[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Aqua.Active) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S94835[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6313T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S94836[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOLO X910 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S94837[] = "Mozilla/5.0 (Linux; Android 4.1.2; EK-GC110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S94838[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT15i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S94839[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S6010 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S94840[] = "Mozilla/5.0 (Linux; Android 4.0.4; w30hd(QuadCore) Build/W30hd) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S94841[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T869 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S94842[] = "Mozilla/5.0 (Linux; Android 4.1.2; UMI X1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S94843[] = "Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_E7310 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S94844[] = "Mozilla/5.0 (Linux; Android 4.0.4; M9000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S94845[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC 802t Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S94846[] = "Mozilla/5.0 (Linux; Android 4.0.4; WALKMAN Build/icx1227_E111) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S94847[] = "UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; LG-P500) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S94848[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Optimus G Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/geeb";
const char S94849[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94850[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; mobiistar touch LAI 504 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94851[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:12.0) Gecko/X Firefox/12.0";
const char S94852[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-in; WT19i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S94853[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328w Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S94854[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5250) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S94855[] = "Mozilla/5.0 (Linux; Android 4.1.2; B92 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S94856[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9152 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S94857[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S94858[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9295 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S94859[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT300 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S94860[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A26 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S94861[] = "Mozilla/5.0 (Linux; Android 4.1.1; A9 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S94862[] = "Mozilla/5.0 (Linux; Android 4.2.1; ZTE Grand X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S94863[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S5303 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S94864[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax_A110 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S94865[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC Incredible S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S94866[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9300) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S94867[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax A110) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S94868[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; Lenovo P770 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S94869[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T959 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S94870[] = "Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S94871[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S94872[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S94873[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S94874[] = "Mozilla/5.0 (Linux; Android 4.2.2; ME302C Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S94875[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S94876[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) PPC; 240x320; HTC_P3300/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S94877[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; MDA Vario/3.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S94878[] = "ZTE-F866/1.0 ACS-NF/3.2 Qtv/4.3";
const char S94879[] = "MOT-PEBL U6/08.84.2DR MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S94880[] = "NexianNX-NXT901/MTKRelease/11-20-2010Browser/MAUIProfile";
const char S94881[] = "Nokia5130c-2/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5130c-2/UC Browser7.8.0.95/69/401";
const char S94882[] = "LG-LG510/1.0 Polaris/6.15.WEB";
const char S94883[] = "LG-T315i/V100 Obigo/Q7.3 MMS/LG-MMS-V1.1/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S94884[] = "NokiaN-GageQD/1.0 SymbianOS/6.1 Series60/1.2 Profile/MIDP-1.0 Configuration/CLDC-1.0/UC Browser7.2.2.51/26/352";
const char S94885[] = "LG-GD710/V10f Teleca/Q7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S94886[] = "SAMSUNG-S5200/S5200DDIK3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S94887[] = "ZTE-C77/1.0 SMIT-Browser/2.0.0";
const char S94888[] = "LG-CU915/V1.0h Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S94889[] = "Nokia5700";
const char S94890[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_0 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko)";
const char S94891[] = "Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Nokia 6680/5.04.07; 9399) Opera 8.65 [en]";
const char S94892[] = "DoCoMo/2.0 L04B(c100;TB;W24H12)";
const char S94893[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Adam Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S94894[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-gb; Adam Build/NIHCPX) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S94895[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.8.1";
const char S94896[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.8.2";
const char S94897[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.8.4";
const char S94898[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.8.4";
const char S94899[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.8.3";
const char S94900[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.8.3";
const char S94901[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:11.0) Gecko/X Firefox/11.0 SeaMonkey/2.8";
const char S94902[] = "Opera/9.80 (X11; FreeBSD 8.2-RELEASE amd64; U; en) Presto/2.7.62 Version/11.01";
const char S94903[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; 7 Trophy)";
const char S94904[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; 7 Trophy T8686)";
const char S94905[] = "PPC; 240x320; HTC_P3300/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S94906[] = "MQQBrowser/32 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3";
const char S94907[] = "MQQBrowser/32 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S94908[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:9.0) Gecko/X Thunderbird/9.0.1";
const char S94909[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; SGH-I777 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94910[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-us; SGH-I777 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94911[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-I777 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94912[] = "samsung-gt-s3653w/UC Browser8.0.3.107/69/444";
const char S94913[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Fluid/0.9.6 Safari/534.50.2";
const char S94914[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB8.1.0.104/31/800";
const char S94915[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB8.1.0.104/40/999";
const char S94916[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB8.1.0.104/31/999";
const char S94917[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB8.1.0.104/40/800";
const char S94918[] = "samsung-gt-s5253/UC Browser7.5.1.77/70/351";
const char S94919[] = "Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.9.168 Version/11.X-09-15 14:14:37";
const char S94920[] = "sonyericssonk770i/UC Browser7.9.0.102/69/352";
const char S94921[] = "JUC(Linux;U;Android2.2;Zh_cn;generic;480*800;)UCWEB7.8.0.95/139/444";
const char S94922[] = "JUC(Linux;U;Android2.2.2;Zh_cn;Vodafone 858;240*320;)UCWEB7.8.0.95/139/444";
const char S94923[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E10i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S94924[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S94925[] = "Mozilla/5.0 (Windows NT 5.1; rv:13.0) Gecko/X Firefox/13.0.3.0 IceDragon/13.0.3.0";
const char S94926[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; nl-nl; TAB450 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94927[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; hu-hu; xTAB-100 Build/ICS.b02ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S94928[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; ZTE-C_N700/N700V1.0.0B04; 240*400; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94929[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94930[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; SPH-D710 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S94931[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.50 (KHTML, like Gecko) FluidApp/1.2 Safari/534.50";
const char S94932[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) FluidApp/1.2 Safari/533.21.1";
const char S94933[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) FluidApp/1.2 Safari/533.21.1";
const char S94934[] = "Microsoft Office Outlook 2010";
const char S94935[] = "JUC(Linux;U;2.3.3;Zh_cn;C8800;480*800;)UCWEB7.5.0.66/139/7713";
const char S94936[] = "Mozilla/5.0 (Linux; Android 4.0.4; M-MP860S2 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94937[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Desire S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94938[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94939[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.1.1.28744 (Firestorm-Release; firestorm skin) Safari/533.3";
const char S94940[] = "samsung-gt-s5260/UC Browser7.9.0.102/69/352";
const char S94941[] = "Mozilla/5.0 (Linux; Android 4.0.4; MB860 Build/6.7.7_95) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94942[] = "Opera/9.12 (X11; Linux i686; U; en) (Ubuntu)";
const char S94943[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT570 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94944[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5603) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S94945[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5603) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S94946[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A45 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94947[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30) Sleipnir/2.6.0";
const char S94948[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E15i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S94949[] = "Mozilla/5.0 (Linux; Android 4.1.1; ViewPad 10S Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94950[] = "Mozilla/5.0 (Linux; Android 4.0.4; M9108 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94951[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3105 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S94952[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5570 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94953[] = "OneBrowser/3.1 (SAMSUNG-GT-C3262)";
const char S94954[] = "Mozilla/5.0 (Linux; Android 4.0.4; W2430 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94955[] = "Mozilla/5.0 (Linux; Android 4.0.4; Polaris Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94956[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC T528w Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94957[] = "Mozilla/5.0 (Linux; Android 4.0.4; N400 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94958[] = "Mozilla/5.0 (Linux; Android 4.1.2; KM-E100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94959[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; TM-7037W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S94960[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A44 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94961[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; en) Presto/2.9.168 Version/11.51";
const char S94962[] = "Opera/9.80 (Windows NT 4.0) Presto/2.12.388 Version/12.10";
const char S94963[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S94964[] = "Mozilla/5.0 (Linux; Android 4.0.4; ThL W7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94965[] = "iBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S94966[] = "Mozilla/5.0 (Linux; Android 4.0.3; Viettel I9 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94967[] = "iBrowser/2.7 (SonyEricssonU5;SymbianOS/9.1 Series60/3.0)";
const char S94968[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I8190 Build/JZO54K; CyanogenMod-JellyTime_R30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94969[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LG-P990 Build/IMM76L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S94970[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; orbit 4300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94971[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax P275 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S94972[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTX21 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S94973[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S94974[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 3.2; th-th; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S94975[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, -0600, us";
const char S94976[] = "WhatsApp/2.9.369 Android/3.2 Device/samsung-GT-P7500";
const char S94977[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40EX650; CC/VNM; vi) Presto/2.10.250 Version/11.60";
const char S94978[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I699 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S94979[] = "Mozilla/5.0 (Linux; Android 4.1.2; YP-G70 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S94980[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; LG-P920 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S94981[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s7250d) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S94982[] = "Mozilla/5.0 (Linux; Android 4.0.4; PC906 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S94983[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.58.2 (KHTML, like Gecko) Version/5.1.8 Safari/6534.58.2 Sleipnir/4.1.2";
const char S94984[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Karbonn A30 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S94985[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI C8813 Build/HuaweiC8813) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S94986[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328t Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S94987[] = "LG-GX500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gx500) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S94988[] = "UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-826_im_india_201) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile";
const char S94989[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A840S Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94990[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; X907 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S94991[] = "Mozilla/5.0 (Linux; Android 4.0.4; SH530U Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94992[] = "Mozilla/5.0 (Linux; Android 4.0.4; AUXUS CoreX4 3G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S94993[] = "Mozilla/5.0 (Linux; Android 4.1.1; tablet Fnac 8 Build/1.1.0 X-12:19) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S94994[] = "MQQBrowser/2.7 (NokiaN8-00;SymbianOS/9.1 Series60/3.0)";
const char S94995[] = "Lenovo A2105 Linux/3.0 Android/4.0 Release/04.12.2012 DXBrowser/2.0 Profile/ Configuration/";
const char S94996[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (HTC One X, 42, 1.9.1)";
const char S94997[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A660 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S94998[] = "Mozilla/5.0 (Linux; Android 4.0.3; P300(Funbook) Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S94999[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; eZeeTab1003 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S95000[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6500D) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S95001[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; zte-f970) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S95002[] = "OneBrowser/3.1 (SonyEricssonJ20i/R7CA064)";
const char S95003[] = "Mozilla/5.0 (Linux; Android 4.0.4; 792TPC Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S95004[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua_S) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S95005[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq5) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S95006[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T1-B Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95007[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S95008[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.6) Gecko/X Ubuntu/11.04 (natty) Firefox/3.6.6";
const char S95009[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S95010[] = "Mozilla/5.0 (Linux; Android 4.0.4; N-03E Build/AX) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95011[] = "Mozilla/5.0 (Linux; Android 4.0.4; Fly_IQ450 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95012[] = "Mozilla/5.0 (Linux; Android 4.2.1; ZP800H Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95013[] = "Mozilla/5.0 (Linux; Android 4.0.4; EM543 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S95014[] = "OneBrowser/3.1 (SonyEricssonK800i/R8BF003)";
const char S95015[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; eZee Tab972 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S95016[] = "JUC (Linux; U; 4.0.4; zh-cn; GT-S5301; 240*320) UCWEB7.9.0.94/139/444";
const char S95017[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; Ritmix RMD745 Build/FR-MX68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S95018[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MB860 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S95019[] = "Mozilla/5.0 (Linux; Android 4.2.2; Xperia T Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95020[] = "Mozilla/5.0 (Linux; Android 4.1.1; PMP5780D Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S95021[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Karbonn_A18 Build/KarbonnA18) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S95022[] = "Mozilla/5.0 (Linux; Android 4.0.4; M1062 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S95023[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I8150 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95024[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-I915 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S95025[] = "Mozilla/5.0 (Linux; Android 4.1.2; UMI X1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95026[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT300 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S95027[] = "Mozilla/5.0 (Linux; Android 4.2.2; Inspire 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95028[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P970 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S95029[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHL21 Build/S5170) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95030[] = "Mozilla/5.0 (Linux; Android 4.0.4; A878 Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95031[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZP810 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95032[] = "Mozilla/5.0 (Linux; Android 4.1.1; GOCLEVER TAB M723G Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S95033[] = "Mozilla/5.0 (Linux; Android 4.2.2; V812 Core4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S95034[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Vision Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95035[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LG730 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S95036[] = "Mozilla/5.0 (Linux; Android 4.0.4; O+_8.9_Android Build/OplusOplus_A1000) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95037[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE N9120 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S95038[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-MS770 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95039[] = "Mozilla/5.0 (Linux; Android 4.0.4; F7HD Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S95040[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36 slurp";
const char S95041[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8951 Build/HuaweiT8951) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S95042[] = "Mozilla/5.0 (Linux; Android 4.1.1; ME172V Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S95043[] = "iCab/4.7 (Macintosh; U; PPC Mac OS X)";
const char S95044[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus 4 Build/rs_v0.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95045[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9100G) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S95046[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; LG-E435) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S95047[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; T3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S95048[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V889D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S95049[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S95050[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I739 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S95051[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-I739 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S95052[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A7+ Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S95053[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T327t Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95054[] = "Mozilla/5.0 (Linux; Android 4.0.4; DM-992 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S95055[] = "SAMSUNG-GT-P3108_TD/1.0 Android/4.1.2 Release/12.21.2012 Browser/AppleWebKit534.30";
const char S95056[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6410LVW Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S95057[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VS920 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S95058[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCEVODesign4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S95059[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A810 Build/HOANG MINH H?U_VEGAVIETTEAM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95060[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_Amaze_4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S95061[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PantechP9070 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S95062[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S95063[] = "Mozilla/5.0 (Linux; Android 4.0.4; T1-D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95064[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S95065[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.3; en-us; HTCEVOV4G Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S95066[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.4; en-in; MT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S95067[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; SCH-I699 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S95068[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; A500 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S95069[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A2+ Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S95070[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; ZTE N9120 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S95071[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:22.0) Gecko/X Firefox/22.0 slurp";
const char S95072[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E15i Build/2.1.1.A.0.6) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S95073[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua Star) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S95074[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S95075[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; A100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S95076[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; iris504Q Build/iris504Q) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S95077[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A52) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S95078[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-I8160 Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60015";
const char S95079[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; iris502 Build/LAVAIRIS502) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S95080[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033M Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95081[] = "Mozilla/5.0 (Linux; Android 4.0.3; AN7HG3 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S95082[] = "Mozilla/5.0 (Linux; Android 4.2.1; Revo_Lead Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95083[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAP4055DUO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95084[] = "Mozilla/5.0 (Linux; Android 4.1.2; TRUE BEYOND 3G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95085[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHL21 Build/S2140) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95086[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAP4500DUO Build/PrestigioPAP4500DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95087[] = "Mozilla/5.0 (Linux; Android 4.1.2; BLU STUDIO 5.0 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95088[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT30a Build/7.0.A.3.195) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95089[] = "Mozilla/5.0 (Linux; Android 4.1.1; 9300+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95090[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5880D Build/PMP5880D_v1.0.0_X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S95091[] = "Mozilla/5.0 (Linux; Android 4.3; Optimus G Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95092[] = "Mozilla/5.0 (Linux; Android 4.2.1; Micromax A116 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S95093[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-P769 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/p769";
const char S95094[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq80) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S95095[] = "Mozilla/5.0 (Linux; Android 4.2.2; SM-T311 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S95096[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-78) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S95097[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; A52 Build/A52) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95098[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S6012) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S95099[] = "Mozilla/5.0 (Linux; Android 4.1.2; SM-T211 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S95100[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_Desire_V) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S95101[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; A10 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95102[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E270S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S95103[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT890 Build/9.8.2I-50_SMI-26) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S95104[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Micromax_A116 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S95105[] = "UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-S5830) U2/1.0.0 UCBrowser/9.3.0.321 Mobile";
const char S95106[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A25) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S95107[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT25i Build/4.1.B.0.631) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S95108[] = "Mozilla/5.0 (Linux; Android 4.1.1; ONE TOUCH TAB 7 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S95109[] = "Mozilla/5.0 (Linux; Android 4.0.4; UBISLATE7C+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S95110[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8262) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S95111[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7100) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S95112[] = "Mozilla/5.0 (Linux; Android 4.1.1; Orange Zali Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S95113[] = "Mozilla/5.0 (Linux; Android 4.0.4; Andromax U Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S95114[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S5282) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S95115[] = "Mozilla/5.0 (Linux; Android 4.0.4; A9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S95116[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S5303 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S95117[] = "UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Micromax A110Q) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S95118[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia2690) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S95119[] = "Bing/4.1.1 CFNetwork/672.0.2 Darwin/14.0.0";
const char S95120[] = "HTC Streaming Player htc_wwe / 1.0 / htc_vision / 2.2.1";
const char S95121[] = "HTC Streaming Player htc_wwe / 1.0 / htc_vision / 2.2";
const char S95122[] = "HTC Streaming Player htc_asia_wwe / 1.0 / htc_vision / 2.3.3";
const char S95123[] = "HTC Streaming Player htc_wwe / 1.0 / htc_vision / 2.3.3";
const char S95124[] = "HTC Streaming Player htc_asia_india / 1.0 / htc_vision / 2.3.3";
const char S95125[] = "Lenovo-i300_VN/S102 LMP/LML Release/2009.08.13 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S95126[] = "Lenovo-i300_VN/S101 LMP/LML Release/2009.08.03 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S95127[] = "Lenovo-i300/S003 LMP/LML Release/2009.03.19 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S95128[] = "SonyEricssonJ230c/R5J TelecaBrowser/4.08";
const char S95129[] = "SonyEricssonJ230i/R5N TelecaBrowser/4.08";
const char S95130[] = "LG-CE110/V1.0i Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S95131[] = "LG/U8360/v1.0";
const char S95132[] = "LG-KB775F Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S95133[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510M/UBHL2 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S95134[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510M Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S95135[] = "SAMSUNG-SGH-T439/T439UVGI7 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S95136[] = "HPiPAQrw6815/1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)";
const char S95137[] = "Mozilla/4.0 (MobilePhone PL6600/US/1.0) NetFront/3.1 MMP/2.0";
const char S95138[] = "Mozilla/4.0 (MobilePhone VI-2300/US/1.0) NetFront/3.1 MMP/2.0";
const char S95139[] = "LG-KS500/v10a Browser/Teleca-Q7.1/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S95140[] = "Mozilla/4.0 (Vodafone/1.0/LG-KS500/V10b Browser/Teleca-Q7.1/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S95141[] = "LG-GS205 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S95142[] = "SAMSUNG-SGH-F488I/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S95143[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; PD_Novel Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S95144[] = "LG-A130/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S95145[] = "KDDI-TS3D UP.Browser/6.2.0.13.1.5 (GUI) MMP/2.0";
const char S95146[] = "Q-mobile Q131";
const char S95147[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.2.0.0 Chrome/13.0.782.218 Safari/535.1";
const char S95148[] = "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.2.0.0 Chrome/13.0.782.218 Safari/535.1";
const char S95149[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.7.1";
const char S95150[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.7";
const char S95151[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Flipboard/1.7.1";
const char S95152[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.7.2";
const char S95153[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Flipboard/1.7.2";
const char S95154[] = "Dolphin http client/7.3.0(116) (Android)";
const char S95155[] = "Dolphin http client/7.3.1 Beta(118) (Android)";
const char S95156[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; ViewSonic-ViewPad4 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95157[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Viewsonic-ViewPad4 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95158[] = "MQQBrowser/2.0 (Nokia5233;SymbianOS/9.1 Series60/3.0)";
const char S95159[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/528.10 (KHTML, like Gecko) Chrome/2.0.157.2 Safari/528.10";
const char S95160[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/530.6 (KHTML, like Gecko) Chrome/2.0.174.0 Safari/530.6";
const char S95161[] = "Opera/4.02 (Windows 98; U) [en]";
const char S95162[] = "JUC(Linux;U;2.2.1;Zh_cn;HTC Incredible S;480*800;)UCWEB7.6.0.75/139/800";
const char S95163[] = "JUC(Linux; U;2.3.5;zh_cn;HTC Incredible S;480*800;)UCWEB7.6.0.75/139/32485";
const char S95164[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S95165[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9.0.16) Gecko/X Firefox/3.0.16 Flock/2.5.6";
const char S95166[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.7.5) Gecko/X Thunderbird/0.9";
const char S95167[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Nova A100 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S95168[] = "MacOutlook/14.10.0.X (Intel Mac OS X 10.6.8)";
const char S95169[] = "MQQBrowser/2.0 (E63-1;SymbianOS/9.1 Series60/3.0)";
const char S95170[] = "MQQBrowser/2.0 (E63;SymbianOS/9.1 Series60/3.0)";
const char S95171[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; GT-I9000 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95172[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9000 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95173[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; GT-I9000 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95174[] = "nokiae7-00/UC Browser8.0.3.107/69/444  3gpp-gba";
const char S95175[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40CX523; CC/GBR; en) Presto/2.7.61 Version/11.00";
const char S95176[] = "Mozilla/5.0 (X11; U; Linux; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.0.3.2197 Safari/532.4";
const char S95177[] = "JUC(Linux;U;2.3.4;Zh_cn;GT-I9100;480*800;)UCWEB7.6.0.75/139/32092";
const char S95178[] = "Reeder/1010.79.00 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacBookAir4%2C2)";
const char S95179[] = "Reeder/1010.79.00 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacBook5%2C1)";
const char S95180[] = "Reeder/1010.79.00 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (iMac12%2C2)";
const char S95181[] = "Reeder/1010.79.00 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacBookPro6%2C2)";
const char S95182[] = "Reeder/1010.79.00 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacPro3%2C1)";
const char S95183[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i937)";
const char S95184[] = "Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.8.1.24) Gecko/X Thunderbird/2.0.0.24";
const char S95185[] = "Opera/9.80 (Android 2.1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.9.201 Version/11.50";
const char S95186[] = "Opera/9.80 (Android 2.1-update1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.9.201 Version/11.50";
const char S95187[] = "samsung-gt-s5380k/UC Browser8.0.3.107/69/352";
const char S95188[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PC36100 Build/IMM76I; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95189[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PC36100 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95190[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S95191[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S95192[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN78)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S95193[] = "Mozilla/5.0 (Linux; U; Android 1.1.5; fr-fr; A101B2-LZ Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S95194[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire Build/MIUI-XJ) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Mobile Maxthon";
const char S95195[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; AraTop AS-03 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95196[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S95197[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL26EX320; CC/GBR; en) Presto/2.7.61 Version/11.00";
const char S95198[] = "JUC(Linux; U;2.3.4;zh_cn;HTC S610d;480*800;)UCWEB7.6.0.75/139/32449";
const char S95199[] = "Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU8i)/UC Browser8.5.0.183/50/352/UCWEB Mobile";
const char S95200[] = "Mozilla/5.0 (X11; OpenBSD amd64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S95201[] = "nokian79-1/UC Browser7.9.0.102/69/352";
const char S95202[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.8b2) Gecko Camino/0.9+";
const char S95203[] = "MQQBrowser/Mini2.8 (NokiaC3-00/07.20)";
const char S95204[] = "MQQBrowser/2.8 (NokiaC6-00;SymbianOS/9.1 Series60/3.0)";
const char S95205[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; Xbox)";
const char S95206[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; Karbonn_A1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95207[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; SK17i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95208[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonck15i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S95209[] = "Mozilla/5.0 (Linux; Android 4.1.2; VS920 4G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S95210[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-t300) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S95211[] = "iBrowser/2.6 (NokiaC6-00;SymbianOS/9.1 Series60/3.0)";
const char S95212[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-B5330 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95213[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A11 Build/KarbonnA11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95214[] = "MQQBrowser/3.7/Adr (Linux; U; 4.0.4; zh-cn; GT-I9100G Build/IMM76D.ZCLPL;480*800)";
const char S95215[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; N-04E Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95216[] = "MQQBrowser/2.0 (NokiaN8-00;SymbianOS/9.1 Series60/3.0)";
const char S95217[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; T700 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95218[] = "MQQBrowser/2.7 (SonyEricssonU5;SymbianOS/9.1 Series60/3.0)";
const char S95219[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S880 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95220[] = "Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-40EX645; CC/CAN; en) Presto/2.10.250 Version/11.60";
const char S95221[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO; 600*1024) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/800";
const char S95222[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO; 600*1024) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/352";
const char S95223[] = "Mozilla/5.0 (Windows; N; Windows NT 5.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth Pro/6.2.2.6613 Safari/532.4";
const char S95224[] = "MacOutlook/14.0.0.X (Intel Mac OS X 10.7.5)";
const char S95225[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; LG-E612 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95226[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A800S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S95227[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; V9C Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S95228[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I5510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S95229[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; Lenovo S880 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S95230[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; ADR6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S95231[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-01D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S95232[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAP4500DUO Build/PrestigioPAP4500DUO) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S95233[] = "Mozilla/5.0 (Linux; Android 4.0.3; E860 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S95234[] = "Mozilla/5.0 (Linux; Android 4.0.4; U8800 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S95235[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC Sensation Z710e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S95236[] = "Mozilla/5.0 (Linux; Android 4.0.4; E330 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S95237[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9228 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S95238[] = "Mozilla/5.0 (Linux; Android 4.1.2; G2S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S95239[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHL21 Build/S2140) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95240[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile i-NOTE WIFI 2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S95241[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PIXELDROID HD2 Build/PIXEL_AOKP_JELLY_BEAN) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95242[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; HTC One S Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S95243[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; MK16i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S95244[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; U8860 Build/HuaweiU8860) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S95245[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAN76) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S95246[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S95247[] = "Mozilla/5.0 (Linux; Android 4.1.1; rk30sdk Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S95248[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A800S Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S95249[] = "Mozilla/5.0 (Linux; Android 4.2.2; U9200 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95250[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; U2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S95251[] = "Mozilla/5.0 (Linux; Android 4.0.4; ENERGY i8 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S95252[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-LS670 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95253[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-I8160 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Mobile Safari/537.31 OPR/14.0.1074.54070";
const char S95254[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9001 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95255[] = "Mozilla/5.0 (Linux; Android 4.1.2; Titanium S1 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95256[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A860S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95257[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 slurp";
const char S95258[] = "Mozilla/5.0 (Linux; Android 4.0.4; i9430 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95259[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; Skate Build/MIUI-2013-01-03-Faia) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95260[] = "Mozilla/5.0 (Linux; Android 4.0.4; myTouch_4G_Slide Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95261[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8530) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S95262[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Acro S Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95263[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 2 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S95264[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID8065 Build/ICS.MID8065.X) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S95265[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; M8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S95266[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; LT26i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S95267[] = "Mozilla/5.0 (Linux; Android 4.0.4; A9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95268[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V889M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95269[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S95270[] = "Mozilla/5.0 (Linux; Android 4.0.4; VP10 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S95271[] = "OneBrowser/3.1 (NokiaN70-1/5.0737.3.0.1)";
const char S95272[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; C12 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95273[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; P85(A9D3) Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S95274[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; GT-S5360) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S95275[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lovme X20 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95276[] = "Mozilla/5.0 (Linux; Android 4.0.4; M1062 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S95277[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; AT500 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S95278[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Boreal(FJA-7A-P) Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S95279[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A35 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S95280[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Andi_3.5i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S95281[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-D710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S95282[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Celkon_A75) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S95283[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A888 Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95284[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; ST23i Build/11.0.A.5.5) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S95285[] = "Mozilla/5.0 (Linux; Android 4.0.3; Maxwell_Lite Build/1.0.2-X-19:27) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S95286[] = "Mozilla/5.0 (Linux; Android 2.3.6; SAMSUNG-SGH-I717 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S95287[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:22.0) Gecko/X Thunderbird/22.0";
const char S95288[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Incredimail/1.2.1.4897";
const char S95289[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F100L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95290[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; LEMON_P7_) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S95291[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P936 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95292[] = "Mozilla/5.0 (Linux; Android 4.1.1; A9 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S95293[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8150 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95294[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I200 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S95295[] = "Mozilla/5.0 (Linux; Android 4.0.4; Primo_F1 Build/WALTONPrimo_F1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S95296[] = "OneBrowser/3.5/Lenovo-A288t_TD/S100 Linux/2.6.35 Android/2.3.5 Release/02.29.2012 Browser/AppleWebkit533.1 Mobile Safari/533.1";
const char S95297[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-STYLE Q 5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95298[] = "Mozilla/5.0 (Linux; Android 4.0.4; IRULU-AL101 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S95299[] = "Mozilla/5.0 (Linux; Android 4.0.4; Fly_IQ442 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95300[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOLO X910 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95301[] = "Mozilla/5.0 (Linux; Android 4.0.4; E330 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95302[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT330 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S95303[] = "Mozilla/5.0 (Linux; Android 4.0.3; ZP100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95304[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; Fly_IQ440 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95305[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian72) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S95306[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S95307[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5282 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S95308[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VS840 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S95309[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; ST27i Build/6.2.A.1.100) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S95310[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I747 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S95311[] = "Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US) AppleWebKit/530.7 (KHTML, like Gecko) Chrome/2.0.175.0 Safari/530.7";
const char S95312[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Lenovo A820 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95313[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/5.0; chromeframe/12.0.742.112)";
const char S95314[] = "Mozilla/5.0 (Linux; Android 4.2.1; Lenovo A830 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95315[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S95316[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M870 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S95317[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C5170 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S95318[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; ZTE T790 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95319[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-M830 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S95320[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A27 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S95321[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; iBall Andi4.5q Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S95322[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A288t Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S95323[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx600) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S95324[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E16i Build/2.1.1.A.0.53) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S95325[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s7233e) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S95326[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; HTC Desire X) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S95327[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-7240 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S95328[] = "Mozilla/5.0 (Linux; Android 4.0.4; dL1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95329[] = "Mozilla/5.0 (Linux; Android 4.1.1; Telenor Smart Pro 2 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95330[] = "Mozilla/5.0 (Linux; Android 4.2.1; ultrafone 701HD Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95331[] = "Mozilla/5.0 (Linux; Android 4.1.2; L-05D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95332[] = "Mozilla/5.0 (Linux; Android 4.1.2; AT-AS43D3 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95333[] = "Mozilla/5.0 (Linux; Android 4.1.1; Fusion Bolt Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S95334[] = "Mozilla/5.0 (Linux; Android 4.0.4; F-04E Build/V14R31C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95335[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 97 TITANIUMHD Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S95336[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; MID8048) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S95337[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ZTE V768 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S95338[] = "Mozilla/5.0 (Linux; Android 4.2.2; TouchPad Build/JDQ39E) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S95339[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E980 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S95340[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S95341[] = "Mozilla/5.0 (Linux; Android 4.0.4; A701 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95342[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT28h Build/6.2.B.0.211) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S95343[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9200 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S95344[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-M919 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/jfltetmo";
const char S95345[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9260 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S95346[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I535 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S95347[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; MT27i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S95348[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E612f Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S95349[] = "Mozilla/5.0 (Linux; Android 4.0.3; LG-E610 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S95350[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; TA-FONE A37 HD Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33756";
const char S95351[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID8048 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S95352[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-E400 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S95353[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE75-1) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S95354[] = "Mozilla/5.0 (Linux; Android 4.2.1; ZP810 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S95355[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Titanium_S5) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S95356[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; MT11i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S95357[] = "Mozilla/5.0 (Linux; Android 4.1.2; Studio 5.3s Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S95358[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SM-T211 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S95359[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Version/3.1.2 Safari/525.21";
const char S95360[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S95361[] = "SEC-SGHM300/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S95362[] = "Mozilla/5.0 (Linux; U; Android SMB Cricket Transform; en-us; SPH-M920 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S95363[] = "Mozilla/5.0 (Linux; U; Android Shiftr182 Point-Blank ROM v2.3; en-us; SPH-M920 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S95364[] = "Xda_Stellar/ 240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S95365[] = "SonyEricssonK800c/R8BF Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S95366[] = "SonyEricssonK310a/R4DA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S95367[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; C8600 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S95368[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; C8600 Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S95369[] = "SAMSUNG-SGH-T519/T519UVFG8 Profile/MIDP-2.0 Configuration/CLDC-1.1 NetFront/3.2";
const char S95370[] = "Mozilla/5.0 (iDEN; U; CPU iDEN Android Eclair Creekside; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S95371[] = "LGE-VX5400/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S95372[] = "SAMSUNG-SGH-T109/T109UVHG7 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S95373[] = "Haier EC-6060/Bakrie Telecom/BOLT v2.31";
const char S95374[] = "SAGEM-my700x/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.f.1.161 (GUI)";
const char S95375[] = "SAGEM-my700x/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.f.1.162 (GUI)";
const char S95376[] = "LG/U970/v1.0";
const char S95377[] = "Nokia7610/2.0 (7.0642.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.4.0.65/27/351";
const char S95378[] = "SAMSUNG-GT-B7330/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.2.2.51";
const char S95379[] = "NokiaC6-00/UC Browser7.6.1.82/50/351";
const char S95380[] = "KDDI-SH34 UP.Browser/6.2.0.13.4 (GUI) MMP/2.0";
const char S95381[] = "Bing/1.3.0 CFNetwork/485.10.2 Darwin/10.3.1";
const char S95382[] = "StumbleUpon/2.52 CFNetwork/467.12 Darwin/10.3.1";
const char S95383[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/14.0.835.202)";
const char S95384[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/14.0.835.202)";
const char S95385[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/14.0.835.202) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S95386[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/14.0.835.202) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1";
const char S95387[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.1; chromeframe/14.0.835.186; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)";
const char S95388[] = "Mozilla/5.0 (Windows NT 6.0; chromeframe/14.0.835.186) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1";
const char S95389[] = "Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 4.1; Avant Browser)";
const char S95390[] = "Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 4.1; Avant Browser)";
const char S95391[] = "Mozilla/5.0 (Windows; U; Windows NT 4.10; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10";
const char S95392[] = "MICROMAXQ55";
const char S95393[] = "Mozilla/5.0 (Windows NT 4.10) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S95394[] = "Mozilla/5.0 (Linux; U; Linux Ventana; zh-tw; Transformer Prime TF201 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S95395[] = "Mozilla/5.0 (Linux; U; Linux Ventana; en-us; Transformer Prime TF201 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13";
const char S95396[] = "JUC(Linux;U;Android2.2.2;Zh_cn;HTC Desire;480*800;)UCWEB7.8.0.95/139/355";
const char S95397[] = "JUC(Linux;U;Android2.2;Zh_cn;HTC Desire;480*800;)UCWEB7.8.0.95/139/352";
const char S95398[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Flock/3.0.6.4257 Chrome/5.0.375.1281 Safari/533.4";
const char S95399[] = "MQQBrowser/2.0 (Nokia5230;SymbianOS/9.1 Series60/3.0)";
const char S95400[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6.2 GTB7.1";
const char S95401[] = "gt-s5233a/UC Browser7.6.1.82/69/352";
const char S95402[] = "gt-s5233a/UC Browser7.6.1.82/69/351";
const char S95403[] = "JUC(Linux;U;2.1-update1;Zh_cn;E15i;320*480;)UCWEB7.7.0.85/139/800";
const char S95404[] = "JUC(Linux;U;2.1-update1;En;E15i;320*480;)UCWEB7.7.0.85/139/999";
const char S95405[] = "samsung-gt-s5350/UC Browser7.7.1.88/69/444";
const char S95406[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P509 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95407[] = "nokia6233/UC Browser7.9.0.102/69/352";
const char S95408[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0.1) Gecko/X Postbox/2.5.3";
const char S95409[] = "Opera/9.80 (Windows 95; U; ru) Presto/2.9.168 Version/11.51";
const char S95410[] = "Nexian NX-G509/MTK Release/10.01.2009 Browser/MAUI Profile";
const char S95411[] = "Mozilla/5.0 (Series40; Nokia5130c-2/06.94; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.3.23.12";
const char S95412[] = "Mozilla/5.0 (Windows NT 6.0; rv:9.0.1) Gecko/X Firefox/9.0.1 slurp";
const char S95413[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S95414[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:5.0) Gecko/X Firefox/5.0 SeaMonkey/2.2";
const char S95415[] = "Mozilla/5.0 (Windows NT 5.1; rv:2.1.1) Gecko/X Firefox/4.0.2pre Fennec/4.0.1";
const char S95416[] = "MQQBrowser/34 (iOS 4; U; CPU like Mac OS X; zh-cn)";
const char S95417[] = "MQQBrowser/34 (iOS 4S; U; CPU like Mac OS X; zh-cn)";
const char S95418[] = "GoogleEarth/5.0.11733.9347(Windows;Microsoft Windows XP (Service Pack 3);en-US;kml:2.2;client:Free;type:default)";
const char S95419[] = "GoogleEarth/5.0.11337.1968(Windows;Microsoft Windows XP (Service Pack 3);en-US;kml:2.2;client:Free;type:default)";
const char S95420[] = "acer_E101 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S95421[] = "Mozilla/4.0 (compatible; MSIE 4.5; Windows 98; )";
const char S95422[] = "nokian95/UC Browser8.0.3.107/69/444";
const char S95423[] = "nokian95_8gb/UC Browser8.0.3.107/69/444";
const char S95424[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S95425[] = "Mozilla/5.0 (Series40; Nokia305/03.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S95426[] = "Instagram 3.0.0 (iPod touch; iPhone OS 5.1.1; en_US) AppleWebKit/420+";
const char S95427[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; GT-I8000 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95428[] = "Android/1.0 (cdma_spyder 6.7.2-180_DHD-16_M4-31)";
const char S95429[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; P85(R8A1) Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95430[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; P85(R8A1) Build/GRJ22)";
const char S95431[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; P85(R8A1) Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S95432[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-N7000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95433[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95434[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95435[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95436[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95437[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S95438[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95439[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95440[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95441[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S95442[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SmartTab1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S95443[] = "Nokia6670/2.0 (6.0522.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.9.1.120/27/351/UCWEB";
const char S95444[] = "Nokia300/5.0 (07.44) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia300) UCBrowser8.4.0.159/70/352/UCWEB Mobile";
const char S95445[] = "HTC Streaming Player tmous / 1.0 / htc_ruby / 2.3.4";
const char S95446[] = "HTC Streaming Player tmous / 1.0 / htc_ruby / 4.0.3";
const char S95447[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P690 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S95448[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; LG-LU3100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95449[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST15i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S95450[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST18i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S95451[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/MIUI.es) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S95452[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; fa-fa; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; fa) Presto/2.8.119 Version/11.10";
const char S95453[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95454[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SONY-Q8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95455[] = "Mozilla/5.0 (Linux; Android 4.1.2; A501 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S95456[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S95457[] = "Nokia3500c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia3500c) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S95458[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95459[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95460[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Version/5.1 Safari/7534.48.3";
const char S95461[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95462[] = "MQQBrowser/3.3 (Nokia5233;SymbianOS/9.1 Series60/3.0)";
const char S95463[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9220 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S95464[] = "Mozilla/5.0 (Linux; U; Android Froyo 222 (V10G) by carlostaringa01; es-es; LG-P350 Optimus Me Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S95465[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.2.2.29837 (Firestorm-Release; firestorm skin) Safari/533.3";
const char S95466[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40HX753; CC/GBR; en) Presto/2.10.250 Version/11.60";
const char S95467[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26i Build/6.1.A.2.45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95468[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; Micromax A57 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S95469[] = "Mozilla/5.0 (Linux; Android 4.2.1; XT910 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S95470[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Loox Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S95471[] = "JUC (Linux; U; 4.0.3; zh-cn; KFTT; 800*1280) UCWEB7.9.0.94/139/444";
const char S95472[] = "Mozilla/5.0 (Linux; Android 4.0.4; Wilco Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S95473[] = "Mozilla/5.0 (Linux; Android 4.0.4; eZee'Tab10c Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S95474[] = "Mozilla/5.0 (Linux; Android 4.0.3; Allview2SpeedDuo Build/SV1.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S95475[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC Desire V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95476[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxx368) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S95477[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3653w) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S95478[] = "OneBrowser/3.0 (SAMSUNG-GT-C3312)";
const char S95479[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95480[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; SK17i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S95481[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MediaPad 7 Lite Build/HuaweiMediaPad) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S95482[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030E Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S95483[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build/IMM76D; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/355";
const char S95484[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3100 Build/IML74K; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/352";
const char S95485[] = "Mozilla/5.0 (Linux; U; Android 3.1; fr-fr; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0100, fr";
const char S95486[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.3.3000";
const char S95487[] = "Mozilla/5.0 (Linux; U; Android 3.2; pt-br; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.3.3000";
const char S95488[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6211 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S95489[] = "JUC (Linux; U; 2.3.7; zh-cn; ZTE_U880s; 320*480) UCWEB7.9.3.103/139/800";
const char S95490[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MiPad 4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S95491[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A95 Build/CelkonCelkon_A95) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S95492[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A288t Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95493[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26i Build/6.1.A.2.45) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S95494[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Rhyme S510b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S95495[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.2.2.259";
const char S95496[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S95497[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A888 Duo Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S95498[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT-AS45SE Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S95499[] = "Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 6 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S95500[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-CN; MB525 Build/MIUI) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S95501[] = "Opera/9.64 (Windows NT 4.10; U; zh-cn) Presto/2.1.1";
const char S95502[] = "MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; TCL A996 Build/REL_1.2.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95503[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC T528t Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S95504[] = "Mozilla/5.0 (Linux; Android 4.0.3; F-10D Build/V21R48A) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95505[] = "Mozilla/5.0 (Linux; Android 4.0.4; F-04E Build/V14R31C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95506[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I8160 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S95507[] = "Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A888 Duo Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95508[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S95509[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT894 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95510[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S95511[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T679 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95512[] = "Mozilla/5.0 (Linux; Android 4.0.4; UBISLATE7C+ Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95513[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S95514[] = "Mozilla/5.0 (Linux; Android 4.0.4; U30GT-H Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S95515[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I500 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95516[] = "Mozilla/5.0 (Linux; Android 4.0.4; Gpad_G1 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95517[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB520 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95518[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5302 Build/GINGERBREAD; 240*320) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444";
const char S95519[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6151) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S95520[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Symphony_W10) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S95521[] = "Mozilla/5.0 (Linux; Android 4.0.4; UMI X1 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S95522[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S95523[] = "Mozilla/5.0 (Linux; Android 4.2.1; ThL W7 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95524[] = "Mozilla/5.0 (Linux; Android 4.1.1; Micromax P362 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95525[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717D Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95526[] = "Mozilla/5.0 (X11; Linux x86_64; ST26i-o) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S95527[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Coolpad7295 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S95528[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PC36100 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95529[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; MAGPAD9.1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S95530[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S95531[] = "Mozilla/5.0 (Linux; Android 4.0.4; Thrill 430x Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95532[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A89 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95533[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P700 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S95534[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST26i Build/11.0.A.7.5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S95535[] = "Mozilla/5.0 (Linux; Android 4.0.4; WALKMAN Build/icx1227_E111) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95536[] = "OperaMini/7.0.5.45389 CFNetwork/485.12.7 Darwin/10.4.0";
const char S95537[] = "Mozilla/5.0 (Linux; Android 4.1.1; AllviewSpeedDuo Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S95538[] = "Mozilla/5.0 (Linux; Android 4.0.4; MAXWEST Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S95539[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax_A110 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S95540[] = "SonyEricssonT700/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssont700) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S95541[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; XT530) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S95542[] = "Mozilla/5.0 (Linux; Android 4.0.4; TD-1010 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S95543[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad7295 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S95544[] = "Mozilla/5.0 (Linux; Android 4.1.2; ME865 Build/6.7.2_GC-384) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S95545[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB-730 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S95546[] = "Mozilla/5.0 (Linux; Android 4.0.4; A878 Duo Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S95547[] = "MQQBrowser/3.5 (Nokia5233;SymbianOS/9.1 Series60/3.0)";
const char S95548[] = "Outlook/15.0 (15.0.4505.1002; MSI; x64)";
const char S95549[] = "Mozilla/5.0 (Linux; Android 4.1.1; JXD-P1000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S95550[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; MT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S95551[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100P Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S95552[] = "Mozilla/5.0 (Linux; Android 4.1.1; Limo X2+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95553[] = "Mozilla/5.0 (Linux; Android 4.0.4; Starmobile CRYSTAL Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95554[] = "Mozilla/5.0 (Linux; Android 4.1.1; Optimus_San_Remo Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95555[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZP910 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95556[] = "Mozilla/5.0 (Linux; Android 4.0.4; A12 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95557[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT26i Build/6.2.B.0.200) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S95558[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAC-70031 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S95559[] = "Mozilla/5.0 (Linux; Android 4.1.1; MI 2 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S95560[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P930 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S95561[] = "Mozilla/5.0 (Linux; Android 4.1.2; MI-ONE Plus Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S95562[] = "Lynx/2.8.7rel.1 libwww-FM/2.14 SSL-MM/1.4.1 OpenSSL/1.0.1e";
const char S95563[] = "Mozilla/5.0 (Linux; Android 4.0.4; TECNO Q1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95564[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; Lenovo A298t Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.292 U3/0.8.0 Mobile Safari/534.31";
const char S95565[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NOKIAN95";
const char S95566[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST17i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95567[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5282 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S95568[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-05D Build/9.1.C.0.475) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95569[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Inspire 4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000";
const char S95570[] = "Mozilla/5.0 (Linux; Android 4.0.4; Ctrl_V1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95571[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI G520-5000 Build/HuaweiG520-5000) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S95572[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LRA Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95573[] = "Mozilla/5.0 (Linux; Android 4.2.2; SM-T311 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S95574[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TP7-1000DC light Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S95575[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; MG-555 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95576[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID9742 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444";
const char S95577[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Q800) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S95578[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T699 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S95579[] = "Mozilla/5.0 (Linux; Android 2.3.5; LG-P930 Build/GRJ90) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S95580[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Micromax A116 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S95581[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S95582[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A500) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S95583[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328t Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S95584[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; C5155 Build/IML77) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S95585[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; Micromax A116 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95586[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S95587[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; es-US ) AppleWebKit/540.0 (KHTML like Gecko) Version/6.0 Safari/8900.00";
const char S95588[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Titanium_S1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S95589[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U8950D Build/HuaweiU8950D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S95590[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Incredible S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S95591[] = "Mozilla/5.0 (Linux; Android 4.0.4; MAXWEST Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95592[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S95593[] = "Mozilla/5.0 (Series40; Nokia109/04.12; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S95594[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; TECNO F7 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/33567";
const char S95595[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I337M Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/jfltecan";
const char S95596[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-e2222) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S95597[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Iris 349) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S95598[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; ALCATEL ONE TOUCH 918N Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S95599[] = "NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291";
const char S95600[] = "UCWEB/2.0 (Linux; U; Adr 4.2.2; en-US; GT-I9500) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S95601[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; iris454 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S95602[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC T528t Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S95603[] = "Mozilla/5.0 (compatible; BecomeBot/3.0; MSIE 6.0 compatible; +http://www.become.com/site_owners.html);c2VvLWNyYXdsZXI=";
const char S95604[] = "Mozilla/5.0 (Linux; Android 4.2.2; Vivid 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95605[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT-AS40SE Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95606[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHL21 Build/S5170) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95607[] = "Mozilla/5.0 (Linux; Android 4.2.2; KYY21 Build/100.0.2480) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95608[] = "Mozilla/5.0 (Linux; Android 4.1.1; A8 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95609[] = "Mozilla/5.0 (Linux; Android 4.0.4; N-07D Build/AX) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95610[] = "Mozilla/5.0 (Linux; Android 4.0.4; Andi4.5h Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95611[] = "Mozilla/5.0 (Linux; Android 4.0.3; PAD703 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95612[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7300 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S95613[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6201 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S95614[] = "Mozilla/5.0 (Linux; Android 4.0.4; Ctrl_V1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95615[] = "Mozilla/5.0 (Linux; Android 4.1.1; GEM10313BK Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S95616[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE T82 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95617[] = "Mozilla/5.0 (Linux; Android 4.2.2; SBM206SH Build/S0008) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95618[] = "Mozilla/5.0 (Linux; Android 4.0.3; ME171 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S95619[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-P760 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95620[] = "Mozilla/5.0 (Linux; Android 4.2.1; Panasonic P51 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95621[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; INTEX_CLOUD_X1 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S95622[] = "Mozilla/5.0 (Linux; Android 4.3; HTC6435LVW Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95623[] = "Mozilla/5.0 (Linux; Android 4.3; Amazon Kindle Fire HD Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S95624[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; F10 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95625[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; MID8048 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S95626[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amaze 4G Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95627[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.2; en-gb; GT-I9152 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S95628[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC 802t Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95629[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GenxDroid7) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S95630[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; C2104) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S95631[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8951 Build/HuaweiT8951) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S95632[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Karbonn A2) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S95633[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1650.11 Mobile Safari/537.36";
const char S95634[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-S738C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S95635[] = "Mozilla/5.0 (Linux; Android 4.2.1; Panasonic P51 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95636[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.1; en-in; Micromax A88 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S95637[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Fly IQ4411 Quad Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95638[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonck15i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S95639[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I537 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S95640[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:9.0.1) Gecko/X Firefox/9.0.1 SeaMonkey/2.6.1";
const char S95641[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; A701 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95642[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A62 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S95643[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8190) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S95644[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A45) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S95645[] = "UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; X10i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S95646[] = "Micromax X1i Ultra/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S95647[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-E400 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S95648[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A3+) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S95649[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC_Wildfire_S_A510e Build/GRJ90) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S95650[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; HTC One Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S95651[] = "Mozilla/5.0 (Linux; Android 4.2.1; M6589 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S95652[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A500 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S95653[] = "Mozilla/5.0 (Linux; Android 4.2.2; T3 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S95654[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; NookColor Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.2/encore";
const char S95655[] = "Mozilla/5.0 (Linux; Android 4.1.1; A8 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S95656[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N8000 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S95657[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-I9003 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S95658[] = "Mozilla/5.0 (Linux; Android 4.0.3; PG86100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S95659[] = "Mozilla/5.0 (Linux; Android 4.2.2; Xperia T Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S95660[] = "Mozilla/5.0 (Linux; Android 4.1.1; O+ 8.52 Android Build/Oplus8.52_Android) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S95661[] = "NokiaC2-00/2.0 (03.99) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; NokiaC2-00) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S95662[] = "NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; NokiaX2-00) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S95663[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax A110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S95664[] = "Mozilla/5.0 (Linux; Android 4.3; tr-tr; SAMSUNG SM-N9000Q Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Version/1.5 Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95665[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ604 Build/I.7.1-45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S95666[] = "SAMSUNG-SAMSUNG SGH-X620/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S95667[] = "SAMSUNG-SGH-X620C/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S95668[] = "SAMSUNG-SGH-X620/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S95669[] = "SonyEricssonJ105a/R1GA SEMC/NewsReader/1.0";
const char S95670[] = "SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S95671[] = "NokiaE60-1/3.0 (2.0618.06.05) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S95672[] = "Nokia3555c/2.0 (03.41) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S95673[] = "SonyEricssonW395c/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1, SonyEricssonW395c/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S95674[] = "SonyEricssonW395c/R1DA Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S95675[] = "SEC-SGHE950/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S95676[] = "MOT-L2/0A.53.10R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S95677[] = "SCH-R410 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S95678[] = "Samsung-SPHA920 AU-MIC-A920/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S95679[] = "SAMSUNG-GT-M8800H/1.0 SHP/VPP/R5 NetFront/3.5 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S95680[] = "KWC-S1310-V/1001 UP.Browser/7.2.6.1.840 (GUI) MMP/2.0";
const char S95681[] = "LGE-MX505/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S95682[] = "SAMSUNG-SGH-i900/JHID1";
const char S95683[] = "Huawei/1.0/0HuaweiG3510/WAP2.0/Obigo-Browser/Q03C MMS/Obigo-MMS/1.2";
const char S95684[] = "HTC-2125/1.2 Mozilla/4.0 (compatible; MSIE 5.5; Windows CE; Smartphone; 240x320)";
const char S95685[] = "HTC-2125/1.2 Mozilla/4.0 (compatible; MSIE 5.5; Windows CE; Smartphone; 240x320) (compatible; MSIE 4.01; Windows CE; Smartphone; 320x240)";
const char S95686[] = "Samsung-SPHM620 AU-MIC-M620/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S95687[] = "Samsung-SPHM620W AU-MIC-M620W/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S95688[] = "Huawei/1.0/G6605/Browser/obigo-browser/q03c";
const char S95689[] = "Mot-WX260/1.0 ObigoInternetBrowser/Q03C";
const char S95690[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Motorola-XT502 Build/FRG83) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S95691[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Motorola-XT502 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95692[] = "KDDI-SA3A UP.Browser/6.2.0.13.1.4 (GUI) MMP/2.0";
const char S95693[] = "SAMSUNG-GT-S5750E/1.0 SHP/VPP/R5 Dolfin/2.0 SMM-MMS/1.2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S95694[] = "KDDI-TS3P UP.Browser/6.2_7.2.7.1.K.4.306 (GUI) MMP/2.0";
const char S95695[] = "NOKIA6120c/UC Browser7.8.0.95/28/351";
const char S95696[] = "Safari/6533.19.4 CFNetwork/454.11.5 Darwin/10.5.0 (i386) (iMac7%2C1)";
const char S95697[] = "Safari/6533.18.5 CFNetwork/454.11.5 Darwin/10.5.0 (i386) (MacBookAir3%2C2)";
const char S95698[] = "Safari/6533.21.1 CFNetwork/454.11.5 Darwin/10.5.0 (i386) (MacPro3%2C1)";
const char S95699[] = "Safari/6533.19.4 CFNetwork/454.11.5 Darwin/10.5.0 (i386) (MacBookPro7%2C1)";
const char S95700[] = "Safari/6533.19.4 CFNetwork/454.11.5 Darwin/10.5.0 (i386) (MacBook6%2C1)";
const char S95701[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.13) Gecko/X Epic/1.2 Firefox/3.6.10 GTB7.1";
const char S95702[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-GB; rv:1.9.1.10) Gecko/X Postbox/1.0.1";
const char S95703[] = "Mozilla/5.0 (Windows NT 5.0; rv:2.0.1) Gecko/X Firefox/4.0.1 SeaMonkey/2.1";
const char S95704[] = "samsung-gt-b7722/UC Browser7.9.0.102/69/352";
const char S95705[] = "SonyEricssonT700/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssont700/UC Browser8.0.3.107/69/444";
const char S95706[] = "Mozilla/5.0 (Windows NT 4.0; rv:5.0.1) Gecko/X Firefox/5.0.1";
const char S95707[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-tw; MOT-A1680 Build/DRD35) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S95708[] = "Mozilla/4.0 (jig browser web; 1.0.4; SH06A3)";
const char S95709[] = "Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:11.0) Gecko/X Firefox/11.0.1-x64 PaleMoon/11.0.1-x64";
const char S95710[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Sprint:SCH-i830; PPC; 240x320)";
const char S95711[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Sprint; SCH-i830; PPC; 240x320)";
const char S95712[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.4; en-US; rv:1.9.0.4) Gecko/X Firefox/2.0.0.18";
const char S95713[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.0m";
const char S95714[] = "Windows Phone Search (Windows Phone OS 7.10;DELL;Venue Pro;7.10;7720)";
const char S95715[] = "Windows Phone Search (Windows Phone OS 7.10;DELL;Venue Pro;7.10;8107)";
const char S95716[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i667)";
const char S95717[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; nl-nl) AppleWebKit/532.3+ (KHTML, like Gecko) Version/4.1 Safari/531.9";
const char S95718[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; nb-no) AppleWebKit/533.16 (KHTML, like Gecko) Version/4.1 Safari/533.16";
const char S95719[] = "samsung-gt-s3653w/UC Browser7.9.0.102/69/352";
const char S95720[] = "JUC(Linux; U;2.3.4;zh_cn;Lenovo A68e;320*480;)";
const char S95721[] = "MQQBrowser/35 Mozilla/5.0 (iPhone 4; U; CPU iPhone OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3";
const char S95722[] = "NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; kau; nokiax2-05) UCBrowser8.2.1.144/70/355/UCWEB Mobile";
const char S95723[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIANokia N81)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S95724[] = "NokiaE72-1/UCWEB8.3.0.133/28/800";
const char S95725[] = "Mozilla/5.0 (Java; U; en-us; nokiae72-1) UCBrowser8.3.1.161/70/352/UCWEB Mobile  3gpp-gba";
const char S95726[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Boston Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95727[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Boston Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95728[] = "Mozilla/5.0 (Linux; U; Android 2.3; es-us; Space Livepad 7.2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95729[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; pt-pt; Z71 Build/GRJ22; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95730[] = "MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95731[] = "MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95732[] = "Mozilla/5.0 (Linux; U; Android/2.1.2 ; zh-CN) AppleWebKit/533.1 elite/fzz Safari/533.1 Version/4.0 P160UNA/1.0";
const char S95733[] = "eBayAndroid/2.1.2.15";
const char S95734[] = "Mozilla/5.0 (Linux; Android 4.0.3; Woxter Tablet PC 85CXi Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S95735[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95736[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-U V880 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95737[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Explorer A310e Build/GRJ90)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95738[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S95739[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95740[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95741[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95742[] = "SAMSUNG-GT-N7000/100.40004";
const char S95743[] = "iBrowser/Mini2.8 (Nokia5130c-2/07.95)";
const char S95744[] = "iBrowser/Mini2.8 (SAMSUNG-GT-C3312)";
const char S95745[] = "nokia500/UC Browser7.7.1.88/69/444  3gpp-gba";
const char S95746[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT22i Build/6.1.B.0.544) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S95747[] = "Mozilla/5.0 (X11; OpenBSD amd64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4";
const char S95748[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE61i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile";
const char S95749[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:18.0) Gecko/X Thunderbird/18.0";
const char S95750[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5560) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S95751[] = "UCWEB/2.0(Java; U; MIDP-2.0; fa; samsung-gt-s5560i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S95752[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5560) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S95753[] = "UCWEB/2.0(Java; U; MIDP-2.0; fa; gt-s5560) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S95754[] = "iBrowser/2.7 (NokiaN97;SymbianOS/9.1 Series60/3.0)";
const char S95755[] = "iBrowser/2.7 (Nokia5235;SymbianOS/9.1 Series60/3.0)";
const char S95756[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; LT26i Build/6.1.A.2.45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95757[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; N-02E Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95758[] = "Micromax X268/Q03C MAUI-browser";
const char S95759[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; ZTE-BLADE Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95760[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; MT27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95761[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9103 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S95762[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P350 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S95763[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; SCH-i899 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S95764[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxq55) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S95765[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100G Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S95766[] = "Mozilla/5.0 (Linux; Android 4.0.4; Moii E801 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S95767[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; Microsoft; Virtual)";
const char S95768[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB710 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S95769[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; Micromax A87 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S95770[] = "Mozilla/5.0 (Linux; Android 4.0.4; FreeTAB 9701 HD X1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S95771[] = "Mozilla/5.0 (Linux; Android 4.1.1; POV_TAB-PROTAB25 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S95772[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; micromaxx600) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S95773[] = "Nokia303/5.0 (14.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia303) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S95774[] = "Mozilla/5.0 (Linux; Android 4.1.2; B92 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S95775[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I5510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95776[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; Karbonn_A18 Build/KarbonnA18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S95777[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9.2.28) Gecko/X Thunderbird/3.1.20";
const char S95778[] = "Mozilla/5.0 (Linux; Android 4.0.3; GFIVE Bravo Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S95779[] = "Mozilla/5.0 (Linux; Android 4.0.4; iBuddy Connect Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S95780[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; HTC Desire Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S95781[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Opera/9.80 (Android; Opera Mini/6.5.26969/28.3445; U; en) Presto/2.8.119 Version/11.10";
const char S95782[] = "CSLG32/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S95783[] = "MQQBrowser/3.7/Adr (Linux; U; 4.0.4; zh-cn; MI 1S Build/IMM76D;480*854)";
const char S95784[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95785[] = "Mozilla/5.0 (Linux; Android 4.0.4; TOUCH85 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S95786[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S95787[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sgh-f480) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S95788[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sgh-f480v) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S95789[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; U8180 Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S95790[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5660 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S95791[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; LG-P880 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S95792[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:19.0) Gecko/X Firefox/19.0 slurp";
const char S95793[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HUAWEI U8815 Build/HuaweiU8815) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S95794[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; ZTE-Z990G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S95795[] = "Mozilla/5.0 (Linux; Android 4.1.1; Cynus T2 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S95796[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC T528w Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S95797[] = "Mozilla/5.0 (Linux; Android 4.2.2; A700 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S95798[] = "MQQBrowser/3.7/Adr (Linux; U; 4.0.3; zh-cn; HTC T328w Build/IML74K;480*800)";
const char S95799[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; FreeTAB 9701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S95800[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A288t Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S95801[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F240K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S95802[] = "Mozilla/5.0 (Linux; Android 4.0.4; IMO S88 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95803[] = "Mozilla/5.0 (Linux; Android 4.0.4; U8800 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95804[] = "Mozilla/5.0 (Linux; Android 4.0.4; LGL21 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95805[] = "Mozilla/5.0 (Linux; Android 4.0.4; A111 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S95806[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xperia P Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95807[] = "Mozilla/5.0 (Linux; Android 4.0.3; TAB275 Build/TAB275) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S95808[] = "Mozilla/5.0 (Linux; Android 4.0.4; A9 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S95809[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT-AS40SE Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95810[] = "Mozilla/5.0 (Linux; Android 4.0.4; Titan TV S320 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95811[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T959V Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95812[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT8000 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S95813[] = "Mozilla/5.0 (Linux; Android 4.1.2; Titan TV S320 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95814[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-H9500 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95815[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95816[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; SHV-E110s Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95817[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A15 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95818[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A5+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S95819[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9001 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95820[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S95821[] = "Mozilla/5.0 (Linux; Android 4.0.4; A111 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95822[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A789 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S95823[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGP311 Build/10.1.C.0.344) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S95824[] = "Mozilla/5.0 (Linux; Android 4.1.2; PMP7280C3G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S95825[] = "Mozilla/5.0 (Linux; Android 4.1.1; MOMO8 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S95826[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9100 Build/MIUIv5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S95827[] = "Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6303classic) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S95828[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; XCD_28) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S95829[] = "Mozilla/5.0 (Linux; Android 4.0.4; VP10 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S95830[] = "MQQBrowser/3.3 (E63;SymbianOS/9.1 Series60/3.0)";
const char S95831[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; LG-P970) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S95832[] = "Mozilla/5.0 (Linux; Android 4.1.1; C1505 Build/11.3.A.0.47) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S95833[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S95834[] = "Mozilla/5.0 (Java; U; en-in; sunmicro) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S95835[] = "Mozilla/5.0 (Java; U; en-us; generic) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S95836[] = "OneBrowser/3.1 (SonyEricssonU10i/R7BA084)";
const char S95837[] = "Nokia110/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia110/UC Browser8.0.3.107/69/444";
const char S95838[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx660) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S95839[] = "LG-KM900/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-km900) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S95840[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGT-X41 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S95841[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID1040C Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S95842[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MAPAN MX913 DC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95843[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9070 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S95844[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6352 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S95845[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8951 Build/HuaweiT8951) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S95846[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile i-style Q3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95847[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6313T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95848[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTCEVOV4G Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S95849[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P716 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95850[] = "Mozilla/5.0 (Linux; Android 4.1.1; Starmobile DIAMOND Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95851[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A860L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95852[] = "Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 3 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95853[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530M Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95854[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID SPYDER Build/HDish) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95855[] = "Mozilla/5.0 (Linux; Android 4.0.4; KPN Smart 300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95856[] = "Mozilla/5.0 (Linux; Android 4.1.2; MT27i Build/6.2.A.0.400) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95857[] = "Mozilla/5.0 (Linux; Android 4.1.1; MyPhone A818g Duo Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95858[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE T22 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95859[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASP-4500Z Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95860[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; TECNO_B3) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S95861[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaC7-00";
const char S95862[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-P760 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95863[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Z105 Build/malataZ105) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95864[] = "JUC (Linux; U; 4.0.4; zh-cn; Lenovo A798t; 480*782) UCWEB8.3.1.151/145/33220";
const char S95865[] = "UCWEB/2.0 (Linux; U; Adr 2.2; en-US; GT-I5500) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S95866[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; Evodroid W10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S95867[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE Plus Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S95868[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S95869[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A298t Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S95870[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Aqua flash Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S95871[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTCEVOV4G Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S95872[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; LG-P350) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S95873[] = "Mozilla/5.0 (Linux; Android 4.1.1; OV-SteelCore10+ Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S95874[] = "Nokia114/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia114) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S95875[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; zh-CN; GT-S5660) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile";
const char S95876[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP7100D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S95877[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; MID5301 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S95878[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PIRANHA BUSINESS III TAB 10.1 ZEUS WHITE Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S95879[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:21.0) Gecko/X Firefox/21.0 SeaMonkey/2.18";
const char S95880[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; TECNO_N7) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S95881[] = "UCWEB/2.0 (iOS; U; iPd OS 6_1_3; zh-CN; iPd5,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile";
const char S95882[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-L38C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S95883[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Sensation Z710e Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S95884[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI-ONE Plus Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S95885[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LT26i Build/6.2.B.0.200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S95886[] = "NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac1-02) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S95887[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-I8190 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S95888[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-LG730 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S95889[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Q1000 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S95890[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; BN NookHD+ Build/JDQ39E)";
const char S95891[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9228 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S95892[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A3+) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S95893[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; GT-I9100) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S95894[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; LT18i) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S95895[] = "Mozilla/5.0 (Linux; Android 4.2.2; SHL22 Build/S6290) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95896[] = "UCWEB/2.0 (Linux; U; Adr 2.2; en-US; GT-I5510) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S95897[] = "Mozilla/5.0 (Linux; Android 4.0.4; H9500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95898[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E989 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95899[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F100S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95900[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I897 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95901[] = "Mozilla/5.0 (Linux; Android 4.0.4; M-MP75S23G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S95902[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I8150 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95903[] = "Mozilla/5.0 (Linux; Android 4.0.4; Nexus S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95904[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonwt13i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S95905[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S95906[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.1.0.0 Chrome/17.0.963.38 Safari/535.11";
const char S95907[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; gaba A100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95908[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.3; ARM; Trident/6.0; Touch)";
const char S95909[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; SonyEricssonU8i) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S95910[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9152 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300";
const char S95911[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax P560 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95912[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua_Star) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S95913[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; vivo Xplay Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S95914[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S5312) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S95915[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LG730 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S95916[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw580i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S95917[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P778 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S95918[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9200 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S95919[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Q1000 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S95920[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI 1S Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S95921[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_Desire_C) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S95922[] = "Mozilla/5.0 (Linux; Android 4.2.1; ASUS Tablet P1801-T Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S95923[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; C1504) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S95924[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; Karbonn_A6 Build/MocorDroid4.0.3) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S95925[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5830) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S95926[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5830 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S95927[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A87 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S95928[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI 1S Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S95929[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6312) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S95930[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A62) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S95931[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SCH-I545 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S95932[] = "Mozilla/5.0 (Linux; Android 4.0.4; AT-AS40SE Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S95933[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST18i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S95934[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0000 Build/HuaweiY300-0000) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S95935[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; C1604) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S95936[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAP4055DUO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S95937[] = "Mozilla/5.0 (Linux; Android 4.2.2; Ideapad K1 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S95938[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; TECNO N3 Build/master) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S95939[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; WT19i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S95940[] = "Mozilla/5.0 (Linux; Android 4.0.4; HYUNDAI T10 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S95941[] = "Mozilla/5.0 (Linux; Android 4.0.4; A701 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S95942[] = "Mozilla/5.0 (Linux; Android 4.1.2; Primo-N1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S95943[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5282 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S95944[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S95945[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S95946[] = "MOTORIZR-Z8/46.00.00 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 381) Opera 8.65 [en]";
const char S95947[] = "dopod C720/5.1.342/WAP1.2 Profile/MIDP2.0 Configuration/CLDC1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone)";
const char S95948[] = "DopodC720W/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 320x240)";
const char S95949[] = "LG-TU500 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S95950[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) ASUS-GalaxyII/1.0";
const char S95951[] = "Alcatel-OT-363/010_02 ObigoInternetBrowser/Q03C";
const char S95952[] = "Alcatel-OT-363A/1.0 ObigoInternetBrowser/Q03C";
const char S95953[] = "Alcatel-OT-363/1.0 model-orange ObigoInternetBrowser/Q03C";
const char S95954[] = "Alcatel-OT-363/1.0 ObigoInternetBrowser/Q03C";
const char S95955[] = "Nokia6620/2.0 (2.14.1) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S95956[] = "SonyEricssonW830c/R1KG Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S95957[] = "SonyEricssonW830a/R1KG Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S95958[] = "Zte-tu218_TD/1.0 ThreadX/4.0b Larena/2.40 Release/3.15.2010 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S95959[] = "Zte-tu218_TD/1.0 ThreadX/4.0b Larena/2.40 Release/3.15.2010 Browser/NetFront3.5";
const char S95960[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) acer_P300";
const char S95961[] = "SonyEricssonA8i_TD/1.0 Ophone/2.0 (Linux; Android 2.1) Release/3.2.2010 Browser/WAP 2.0 AppleWebKit/530.17 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S95962[] = "SAMSUNG-SGH-T619/T619UVFG8 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S95963[] = "Mozilla/5.0 (HTC-T9199/1.08.706.1;U;Windows Mobile/6.5;Profile/MIDP-2.0 Configuration/CLDC-1.1;480*800;CTC/2.0) Opera/9.7";
const char S95964[] = "Mozilla/5.0 (HTC-T9199/1.18.1401.8;U;Windows Mobile/6.5;Profile/MIDP-2.0 Configuration/CLDC-1.1;480*800;CTC/2.0) Opera/9.7";
const char S95965[] = "SEC-SGHB220/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S95966[] = "Motorola-VE20 Obigo/Q05A1 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S95967[] = "SonyEricssonW350i";
const char S95968[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) T-Mobile_LEO";
const char S95969[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) KDDI-TS01";
const char S95970[] = "LG-CB630/V10d Obigo/Q05A Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S95971[] = "Samsung-SCHR330 Infraware-Polaris/6.0 MMP/2.0";
const char S95972[] = "ZTE-V8301/MB6801_V1_Z1_VN_F1BPa101 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C";
const char S95973[] = "NexianNX-G331/MTK Release/02.09.2009 Browser/MAUI Profile";
const char S95974[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_Magic Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95975[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_Magic Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95976[] = "Nokia5233/UC Browser7.2.2.51/50/352";
const char S95977[] = "Mozilla/5.0 (Linux; U; Android Eclair; en-us; i-mobile i858 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S95978[] = "Mozilla/5.0 (Linux; U; Android Eclair; th-th; i-mobile i858 Build/ECLAIR) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S95979[] = "Xda_Serra/480x640 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S95980[] = "LG-GB165 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S95981[] = "Mozilla/5.0 (Series40; NokiaX2-01/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.1.21.8";
const char S95982[] = "LGE-GU290V Obigo/Q7.0 MMP/2.0";
const char S95983[] = "Mozilla/5.0 (Linux; U; Android 1.0.0; nl-nl; Tablet PC Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S95984[] = "SEC-SGHC200";
const char S95985[] = "KDDI-TS3N UP.Browser/6.2_7.2.7.1.K.3.352 (GUI) MMP/2.0";
const char S95986[] = "Lava-A9/S016 LMP/LML Release/2010.03.04 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S95987[] = "Vodafone/1.0/LG-KU380/V10a Browser/Obigo-Q05A1/2.16 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/2.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S95988[] = "Mozilla/5.0 (Windows NT 4.10) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1";
const char S95989[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.8.10793.201";
const char S95990[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0) QQBrowser/6.8.10793.201";
const char S95991[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.8.10793.201";
const char S95992[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; SGH-I997 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95993[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-I997 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S95994[] = "Videocon_V1603_SW_V10a/MAUI.09A.W09.52.MP.V4/wap2.0/OBIGO_Q03C/MIDP 1.0";
const char S95995[] = "Videocon_V1611_SW/1.0 MTK/MAUI.09A.W09.52.MP.V3.F15 Release/11.19.2010 Browser/OBIGO_Q03C";
const char S95996[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:11.0) Gecko/X Firefox/11.0 SeaMonkey/2.8";
const char S95997[] = "Mozilla/5.0 Opera/9.5 (KDDI-SN3Q; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00";
const char S95998[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:7.0) Gecko/X Firefox/7.0 SeaMonkey/2.4";
const char S95999[] = "Nokia5300/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia5300/UC Browser7.9.0.102/69/352";
const char S96000[] = "iBrowser/2.6 (Nokia5230;SymbianOS/9.1 Series60/3.0)";
const char S96001[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Flipboard/1.7.2";
const char S96002[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.7.2";
const char S96003[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.7.2";
const char S96004[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.7.1";
const char S96005[] = "JUC (Linux; U; 2.1-update1; zh-cn; XT800; 480*854) UCWEB7.9.4.145/139/800";
const char S96006[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SPH-D700 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96007[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; fr-fr) OrangeBot-Mobile AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A306 Safari/(mobilesearch.support@orange-ftgroup.com)";
const char S96008[] = "MQQBrowser/32 (iOS; U; CPU like Mac OS X; zh-cn)";
const char S96009[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; pl-pl; GT540 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96010[] = "Opera/9.25 (OpenSolaris; U; en)";
const char S96011[] = "Mozilla/5.0 (Windows NT 5.0; rv:10.0.2) Gecko/X Thunderbird/10.0.2";
const char S96012[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; rv:1.9.2) Gecko/X Camino/3.0";
const char S96013[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S96014[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-es; N7000 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96015[] = "Mozilla/5.0(Linux;U;Android3.2;Zh_cn;GT-P7500;800*1232;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S96016[] = "JUC(Linux;U;Android3.2;Zh_cn;GT-P7500;800*1232;)UCWEB7.8.0.95/139/355";
const char S96017[] = "Mozilla/5.0 (Linux; Android 4.0.4; NookColor Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S96018[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-31)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96019[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S96020[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK BNTV250 Build/GINGERBREAD 1.4.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, us";
const char S96021[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96022[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96023[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96024[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Milestone PLUS Build/45.1.28) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96025[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Milestone PLUS Build/45.1.27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96026[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Milestone PLUS Build/45.1.15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96027[] = "JUC(Linux; U;2.3.5;zh_cn;Philips W626;320*480;)UCWEB7.6.0.75/139/32543";
const char S96028[] = "LG-GM730/v10a WM/6.1 IEMobile/6.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S96029[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Ubuntu/8.04 Chromium/11.0.696.68 Chrome/11.0.696.68 Safari/534.24";
const char S96030[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.24 (KHTML, like Gecko) Ubuntu/8.04 Chromium/11.0.696.68 Chrome/11.0.696.68 Safari/534.24";
const char S96031[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; BilliosBot)";
const char S96032[] = "Mozilla/5.0 (Windows NT 6.0; rv:12.0) Gecko/X Firefox/12.0 Kylo/1.1.1";
const char S96033[] = "MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96034[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S96035[] = "Mozilla/5.0 (Java; U; en-us; nokia6630) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S96036[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; WT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96037[] = "Mozilla/5.0 (Linux; Android 4.0.3; POLY PAD 9716HD Build/ICS.g12refM1005TR.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S96038[] = "Mozilla/5.0 (Linux; Android 4.1.2; MB860 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S96039[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S96040[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; FP-001A-XON Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96041[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96042[] = "JUC(Linux;U;Android4.0.3;Zh_cn;HTC Sensation XE with Beats Audio Z715e;540*960;)UCWEB7.8.0.95/139/444";
const char S96043[] = "Mozilla/5.0 (Java; U; en-us; micromax) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S96044[] = "Mozilla/5.0 (Linux; Android 4.0.3; Next8P12 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S96045[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; A75 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96046[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonk790i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S96047[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; gt-s5233w) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S96048[] = "UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; gt-s5233w) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S96049[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; ST18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96050[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonk550i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S96051[] = "HTC_Touch_Viva_T2223 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.6.1.82";
const char S96052[] = "JUC (Linux; U; 4.0.4; zh-cn; MOMO9_plus_III; 800*444) UCWEB1.0.1.113/147/800";
const char S96053[] = "iBrowser/2.7 (NokiaC7-00;SymbianOS/9.1 Series60/3.0)";
const char S96054[] = "Mozilla/5.0 (Java; U; en-us; micromaxq5) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S96055[] = "Mozilla/5.0 (Linux; Android 4.0.4; DJC Touchtab3 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S96056[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; ALCATEL ONE TOUCH 985N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96057[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxq80) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S96058[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P690 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96059[] = "Mozilla/5.0 (Linux; Android 4.0.4; WALKMAN Build/icx1227_E110) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96060[] = "Mozilla/5.0 (Windows NT 6.0; rv:20.0) Gecko/X Thunderbird/20.0";
const char S96061[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96062[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96063[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3113, 39, 1.8.2)";
const char S96064[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P3113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3113, 39, 1.8.2)";
const char S96065[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P5110, 42, 1.9.1)";
const char S96066[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S96067[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S96068[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-P6200 Build/JOP40D)";
const char S96069[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-p6200 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96070[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2a1pre) Gecko/X Minefield/3.2a1pre";
const char S96071[] = "iBrowser/2.5 (NokiaN868MP;SymbianOS/9.3 Series60/3.2)";
const char S96072[] = "MQQBrowser/25 Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE-N788 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96073[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-mobile i-STYLE 4 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96074[] = "MQQBrowser/3.4/Adr (Linux; U; 4.0.4; zh-cn; GT-I9100G Build/IMM76D.ZQLR7;480*800)";
const char S96075[] = "Mozilla/5.0 (Linux; Android 4.1.1-SDK_V1.00_X.X release-keys; Xteam Smartpad 810c Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S96076[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328t Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96077[] = "LG-GW300/V100 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-gw300) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S96078[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; PIRANHA BUSINESS TAB 10.1 ZEUS WHITE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96079[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.2.2.259";
const char S96080[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID7048 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S96081[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96082[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LG-P970 Build/IMM76L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96083[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LG-P765 Build/IMM76L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96084[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; 5860E Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96085[] = "Mozilla/5.0 (Linux; Android 4.0.4; A2 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96086[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE T22 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96087[] = "Mozilla/5.0 (Linux; Android 4.0.4; MEDION P4013 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96088[] = "Mozilla/5.0 (Linux; Android 4.1.2; MB860 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96089[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9103 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96090[] = "Mozilla/5.0 (Linux; Android 4.1.2; Desire S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96091[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOLO_A800 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96092[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC EVO 3D X515m Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S96093[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGP321 Build/10.1.1.A.0.72) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S96094[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9220 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S96095[] = "iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 4.1; en-us; SmartTab1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96096[] = "OneBrowser/3.1 (SAMSUNG GT-C3322)";
const char S96097[] = "MQQBrowser/3.3 (E71;SymbianOS/9.1 Series60/3.0)";
const char S96098[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A780 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96099[] = "JUC (Linux; U; 2.3.9; zh-cn; Celkon_A_85; 320*480) UCWEB7.9.0.94/139/444";
const char S96100[] = "MQQBrowser/3.4 (Nokia5230;SymbianOS/9.1 Series60/3.0)";
const char S96101[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; IRIS_501 Build/LAVAIRIS501) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96102[] = "Mozilla/5.0 (Linux; Android 4.0.4; S120 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96103[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Lenovo P700i Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96104[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo A798t Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S96105[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-C660 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96106[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; Lenovo A288t Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S96107[] = "Mozilla/5.0 (Linux; Android 4.1.2; SBM203SH Build/S0018) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96108[] = "Mozilla/5.0 (Linux; Android 4.0.4; Spice Mi-530 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96109[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D700 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96110[] = "SAMSUNG-GT-B7330/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S96111[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; Kindle Fire Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S96112[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MITO A322 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96113[] = "Mozilla/5.0 (Linux; Android 4.1.1; ASP-4500Z Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96114[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6400L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S96115[] = "Mozilla/5.0 (Linux; Android 4.1.2; P5_Quad Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96116[] = "Mozilla/5.0 (Linux; Android 4.0.3; PMP3470B Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S96117[] = "Mozilla/5.0 (Linux; Android 4.0.4; H9500 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96118[] = "JUC (Linux; U; 4.0.3; zh-cn; HUAWEI C8812; 480*800) UCWEB8.3.1.151/145/33265";
const char S96119[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; Accord711 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96120[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I699 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S96121[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; LG-C660) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S96122[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC One X+ Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96123[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A6 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96124[] = "Opera/9.80 (Windows NT 5.1; Edition ASUS) Presto/2.12.388 Version/12.15";
const char S96125[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; iDj7n Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96126[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96127[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R811 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S96128[] = "Mozilla/5.0 (X11; FreeBSD amd64; rv:20.0) Gecko/X Firefox/20.0 SeaMonkey/2.17";
const char S96129[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S96130[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; E860 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96131[] = "Mozilla/5.0 (Linux; Android 4.0.4; Sen S3 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S96132[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) WB250 UP.Browser/7.2.7.2.559 (GUI)  MMP/2.0";
const char S96133[] = "JUC (Linux; U; 2.3.6; zh-cn; ZTE_N855D; 320*480) UCWEB7.9.3.103/139/33031";
const char S96134[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; WF7C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96135[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-US; XCD35 Build/FRF91) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S96136[] = "iBrowser/2.6 (SonyEricssonU5;SymbianOS/9.1 Series60/3.0)";
const char S96137[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; OPPOX905 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96138[] = "MQQBrowser/Mini2.8 (Nokia2700c-2/07.80)";
const char S96139[] = "Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabA2207A-H Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S96140[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; iBall_Andi3e) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S96141[] = "Mozilla/5.0 (Linux; Android 4.0.4; Novo7Flame Build/X.tm) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S96142[] = "Mozilla/5.0 (Linux; Android 4.0.4; V96A Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S96143[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; zh-CN; Android2.x) U2/1.0.0 UCBrowser/8.8.3.278 U2/1.0.0 Mobile";
const char S96144[] = "Mozilla/5.0 (Linux; Android 4.1.1; C1605 Build/11.3.A.2.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96145[] = "Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook HD+ Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S96146[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S96147[] = "Mozilla/5.0 (Linux; Android 4.1.1; IQ442 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96148[] = "Mozilla/5.0 (Linux; Android 4.1.2; 201F Build/V33R12A) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96149[] = "Mozilla/5.0 (Linux; Android 4.0.4; Novo7Flame Build/X.tm) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S96150[] = "Mozilla/5.0 (Linux; Android 4.0.4; W30(QuadCore) Build/W30) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S96151[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S96152[] = "Mozilla/5.0 (Linux; Android 4.0.4; A11 Build/KarbonnA11) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96153[] = "Mozilla/5.0 (Linux; Android 4.0.4; L-01D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96154[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI H881C Build/HuaweiH881C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96155[] = "Mozilla/5.0 (Linux; Android 2.3.6; SCH-I510 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S96156[] = "Mozilla/5.0 (Linux; Android 4.0.4; Philips W336 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96157[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9500 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S96158[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F100S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96159[] = "Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB A93.2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S96160[] = "Mozilla/5.0 (Linux; Android 4.0.3; PMP5197DULTRA Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S96161[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-E400 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S96162[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID7042 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444";
const char S96163[] = "Mozilla/5.0 (Linux; Android 4.1.2; X21 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96164[] = "Mozilla/5.0 (Linux; Android 4.0.3; M032 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96165[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S96166[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EL72B Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96167[] = "Mozilla/5.0 (Linux; Android 4.0.4; ThL W1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96168[] = "Mozilla/5.0 (Linux; Android 4.1.2; Skyfire2 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96169[] = "Mozilla/5.0 (Linux; Android 4.0.4; AD683G Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S96170[] = "Mozilla/5.0 (Linux; Android 4.1.1; ONE TOUCH TAB 7HD Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S96171[] = "Mozilla/5.0 (Linux; Android 4.0.4; A111 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96172[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E270L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S96173[] = "Mozilla/5.0 (Linux; Android 4.0.4; ThL W1 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96174[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96175[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua_Glow) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S96176[] = "Mozilla/5.0 (Linux; Android 4.0.4; M-MP1010i Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S96177[] = "Mozilla/5.0 (Linux; Android 4.1.2; L-05D Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S96178[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-US; LG-P970 Build/FRG83G) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S96179[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly s Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96180[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717R Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S96181[] = "Mozilla/5.0 (Linux; Android 4.0.4; KFJWI Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/19.77.34.5 Safari/537.22";
const char S96182[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9220 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S96183[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; Monga M1 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96184[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) Nokia500";
const char S96185[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8552 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S96186[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F220K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S96187[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ALCATEL ONE TOUCH 930D Build/1.2.10.4) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S96188[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; vivo Xplay Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S96189[] = "Mozilla/5.0 (Linux; Android 4.1.2; TA-FONE A37 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96190[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; zh-CN; GT-I9308) U2/1.0.0 UCBrowser/9.1.1.309 U2/1.0.0 Mobile";
const char S96191[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; SK17i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S96192[] = "Mozilla/5.0 (Linux; Android 4.0.4; JY-G3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S96193[] = "DoCoMo/2.0 SH901iC(c100;TB;W24H12)";
const char S96194[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:15.0) Gecko/X Firefox/15.0.1";
const char S96195[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (KFJWI, 35, 1.7.0)";
const char S96196[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; Transformer TF101 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96197[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I879 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96198[] = "Mozilla/5.0 (Linux; Android 4.1.2; folio100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S96199[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Engage 7HD Build/Engage7HD.Starmobile.V012.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96200[] = "Mozilla/5.0 (Linux; Android 4.1.1; WM8850-mid Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96201[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A810S Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96202[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Penta_WS802C) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S96203[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; zh-CN; GT-S5830i) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile";
const char S96204[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Lenovo A820t Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.292 U3/0.8.0 Mobile Safari/534.31";
const char S96205[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; zh-CN; GT-I9300) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile";
const char S96206[] = "Mozilla/5.0 (Linux; Android 4.2.1; LT25i Build/9.1.A.1.140) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S96207[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MID8011 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96208[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-I8190L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S96209[] = "Mozilla/5.0 (Linux; Android 4.2.1; Spice Mi-535 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96210[] = "Mozilla/5.0 (Linux; Android 4.0.3; A97i Build/CelkonA97i_Celkon) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96211[] = "Mozilla/5.0 (Linux; Android 4.0.4; M-MP75S23G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S96212[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon";
const char S96213[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; M886 Build/HuaweiM886) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S96214[] = "Mozilla/5.0 (Linux; Android 4.0.3; KFOT Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S96215[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A4+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S96216[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9152 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S96217[] = "NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291";
const char S96218[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A57 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S96219[] = "Nokia6500s-1/2.0 (08.30) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6500s-1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S96220[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S96221[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Spice Mi-435 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S96222[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; C1504) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S96223[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; UBISLATE7C+ Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S96224[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9260 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S96225[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo A789 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S96226[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N8000 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S96227[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S96228[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; LG-P920 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S96229[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SGH-M919 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S96230[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I8268 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S96231[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; TECNO N3) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S96232[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; U20i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S96233[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; X10i Build/3.0.1.G.0.75) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S96234[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A780L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96235[] = "Mozilla/5.0 (Linux; Android 4.0.4; B92 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96236[] = "Mozilla/5.0 (Linux; Android 4.1.2; Amaze 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96237[] = "Mozilla/5.0 (Linux; Android 4.1.1; T-17B Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S96238[] = "Mozilla/5.0 (Linux; Android 4.0.3; OP0118-12 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S96239[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-in; Q800 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S96240[] = "Mozilla/5.0 (Linux; Android 4.3; HTC Sensation Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96241[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MB886 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96242[] = "Mozilla/5.0 (Linux; Android 4.0.4; VSD220 Build/IMM76D.UI23ED12_VSC) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S96243[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-us; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S96244[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; XOLO A700 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S96245[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S96246[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; en-US; Spice_Mi-422 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S96247[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; Lenovo S920_ROW Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33482";
const char S96248[] = "Mozilla/5.0 (Android; Mobile: Galaxy Nexus; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S96249[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S96250[] = "Mozilla/5.0 (Linux; Android 4.3; GT-I9300 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S96251[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E220S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96252[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_4) AppleWebKit/537.13  (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2";
const char S96253[] = "Mozilla/5.0 (Linux; Android 4.0.4; Titan TV S320 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96254[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S96255[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; SK17i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S96256[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAP4040_DUO Build/PrestigioPAP4040DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S96257[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; HUAWEI P6-U06 Build/HuaweiP6-U06) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S96258[] = "JUC (Linux; U; 4.1.2; zh-cn; GT-S5282; 240*320) UCWEB7.9.0.94/139/444";
const char S96259[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; C5170 Build/IML77) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S96260[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.3.2.300";
const char S96261[] = "Mozilla/5.0 (Linux; Android 4.2.2; zh-cn; SCH-P709 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S96262[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax_A110 Build/JRO03C) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S96263[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Intex_Aqua_3.2) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S96264[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N7100 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S96265[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A500) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S96266[] = "UCWEB/2.0 (Linux; U; Adr 2.3.9; en-US; Celkon_A_85) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S96267[] = "Mozilla/5.0 (Linux; Android 4.2.1; ultrafone 701HD Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S96268[] = "Mozilla/5.0 (Linux; Android 4.3; ADR6425LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S96269[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; WT19i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S96270[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST27i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S96271[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOLO X910 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S96272[] = "UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-S5670) U2/1.0.0 UCBrowser/9.3.0.321 Mobile";
const char S96273[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6802) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S96274[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A2) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S96275[] = "Mozilla/5.0 (Linux; Android 4.1.2; SH-04E Build/S6181) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S96276[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-B5330 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S96277[] = "UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-S5282) U2/1.0.0 UCBrowser/9.3.0.321 Mobile";
const char S96278[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LS970 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S96279[] = "Mozilla/5.0 (Linux; Android 4.0.4; IdeaTab_A1107 Build/MR1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S96280[] = "Mozilla/5.0 (Linux; Android 3.1; GT-P7510 Build/HMJ37) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S96281[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; IRIS 455 Build/LAVAIRIS455) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S96282[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S96283[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S96284[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT28at Build/6.1.C.1.111) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S96285[] = "HTC_P4350-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)";
const char S96286[] = "HTCP4350-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)";
const char S96287[] = "SonyEricssonW880i/R8BA Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96288[] = "SAMSUNG-SGH-M628/1.0 Release/2.28.2008 Browser/Openwave6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S96289[] = "SAMSUNG-SGH-M628_CMCC/1.0 Release/2.28.2008 Browser/Openwave6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S96290[] = "SAMSUNG-SGH-T319/T319UVFD4 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S96291[] = "SAMSUNG-SGH-T319R/T319UVFG3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S96292[] = "LG-KP505 Teleca/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96293[] = "SonyEricssonZ250i/R1F003 Browser/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96294[] = "SonyEricssonZ250a/R1C003 Browser/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96295[] = "PANTECH-C810/R01 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S96296[] = "PANTECH-C820/WM28.31 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S96297[] = "SEC-SGHX680/1.0 TSS/2.5";
const char S96298[] = "Samsung-SGH-B320/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S96299[] = "SCH-R300 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S96300[] = "LG/KE990d/v10a Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96301[] = "Alcatel-OT-S319/1.0 model-orange ObigoInternetBrowser/Q03C";
const char S96302[] = "Alcatel-OT-S319/1.0 ObigoInternetBrowser/Q03C";
const char S96303[] = "UP.Browser/5.0/Spice G6550";
const char S96304[] = "Mozilla/5.0 (Series40; NokiaC3-00/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.1.21.8";
const char S96305[] = "NexianNX-W755/MTK Release/01.01.2011 Browser/MAUI Profile/MIDP2.0Configuration/CLDC-1";
const char S96306[] = "SAMSUNG-SGH-E690/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S96307[] = "KWC-laylo/AAR10044 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0";
const char S96308[] = "KWC-M1400/AAZ10043 UP.Browser/7.2.7.2.536 (GUI) MMP/2.0";
const char S96309[] = "KWC-laylo/AAR1004 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0";
const char S96310[] = "KWC-M1400/AAR10042 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0";
const char S96311[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; LU2300 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S96312[] = "DoCoMo/2.0 N07A3(c500;TB;W24H16)";
const char S96313[] = "Nokia5230/UC Browser7.4.0.65/50/351";
const char S96314[] = "SAMSUNG-SGH-J208/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S96315[] = "SAMSUNG-SGH-Z540/1.0 SHP/VPP/R5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S96316[] = "SAMSUNG-SGH-ZV10/1.0 SHP/VPP/R5  NetFront/3.2 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S96317[] = "Nokia2650/1.0 (6.19) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S96318[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Xda Mini S; 240x320)";
const char S96319[] = "Opera/7 Presto";
const char S96320[] = "Mozilla/4.0 (compatible; MSIE 5.23; Mac_PowerPC) Opera 7.54  [en]";
const char S96321[] = "LENOVO-i966/66Mt";
const char S96322[] = "DoCoMo/2.0 F09B(c100;TB;W20H09)";
const char S96323[] = "DoCoMo/2.0 SH09B(c500;TB;W24H16)";
const char S96324[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.2.8853.201";
const char S96325[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; A70CHT Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96326[] = "Devicescape-Agent/4.0.136 (Android - 671a80e2-00db-3dbe-39ec-c247c7ebcb1d) tmobile_wispr1";
const char S96327[] = "Devicescape-Agent/4.0.136 (Android - X-0000-0000-0000-X) tmobile_wispr1";
const char S96328[] = "Devicescape-Agent/4.0.137 (Android - X-0000-0000-0000-X) tmobile_wispr1";
const char S96329[] = "MobileRSSFree/3.4 CFNetwork/485.10.2 Darwin/10.3.1";
const char S96330[] = "MobileRSS/3.4 CFNetwork/485.10.2 Darwin/10.3.1";
const char S96331[] = "RSSRadio/1.35 CFNetwork/485.2 Darwin/10.3.1";
const char S96332[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Sensation 4G Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75 Mobile Safari/535.7";
const char S96333[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC Sensation Z710e Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S96334[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.4; en-US; rv:1.9.1.6) Gecko/X FireDownload/1.2pre Firefox/3.5.6 Wyzo/3.5.6.1 FireTorrent/1.1pre";
const char S96335[] = "JUC(Linux;U;2.3.4;Zh_cn;GT-S5830;320*480;)UCWEB7.8.1.96/139/800";
const char S96336[] = "nokian73-1/UC Browser7.8.0.95/69/352";
const char S96337[] = "Mozilla/5.0 (X11; FreeBSD i386; rv:2.0) Gecko/X Firefox/4.0";
const char S96338[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.2.13) Gecko/X Postbox/2.1.4";
const char S96339[] = "Mozilla/5.0 (Windows NT 6.0; rv:10.0.2) Gecko/X Firefox/10.0.2 slurp";
const char S96340[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Iron/5.0.381.0 Chrome/5.0.381 Safari/533.4";
const char S96341[] = "Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; Panasonic UT-PB1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96342[] = "Opera/9.80 (X11; FreeBSD 9.0-RELEASE i386; U; en) Presto/2.10.229 Version/11.60";
const char S96343[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S96344[] = "Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHW-M430W Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S96345[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; iDx10 3G Build/ICS.b02ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96346[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96347[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96348[] = "Mozilla/5.0 (Linux; Android 4.0.4; N-07D Build/AX) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S96349[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Incredible 2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96350[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Incredible 2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96351[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:0.9.4.1) Gecko/X Netscape6/6.2.3";
const char S96352[] = "JUC(Linux;U;2.3.5;Zh_cn;ThL V7;480*800;)UCWEB7.8.1.96/139/800";
const char S96353[] = "JUC(Linux;U;2.1-update1;Zh_cn;OMAP_SS;480*800;)UCWEB7.8.1.96/139/999";
const char S96354[] = "Mozilla/5.0 (Linux; Android 4.1.1; Droid Incredible Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S96355[] = "Mozilla/5.0 (compatible; Konqueror/2.1.2; X11)";
const char S96356[] = "Coolpad-W702/BrewMP1.0.3/Qualcom6270/ObigoQ7/MIDP-2.1Configuration/CLDC-1.1";
const char S96357[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s3370) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S96358[] = "JUC(Linux;U;Android2.3.6;Zh_cn;A818 Duo;320*480;)UCWEB7.8.0.95/139/444";
const char S96359[] = "Mozilla/5.0 (Linux; Android 4.0.4; MOMO11 bird Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S96360[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; GT-B7510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S96361[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.0 Mobile Safari/535.19 Silk-Accelerated=true";
const char S96362[] = "Nokia311/5.0 (03.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.1.161/69/355";
const char S96363[] = "MQQBrowser/35 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3";
const char S96364[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.2.2.29837 (Firestorm-Release; firestorm skin) Safari/533.3";
const char S96365[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96366[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; Galaxy Nexus Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S96367[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/7534.48.3 TeamStream HD (iPad1,1 232 1.6.1)";
const char S96368[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; MTS-SP101 Build/HuaweiC8511) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S96369[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I927 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S96370[] = "Mozilla/5.0 (Linux; Android 4.0.3; POLY PAD 9708HD Build/ICS.g12refM1005TR.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S96371[] = "BlackBerry9000/5.2.0.89 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96372[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonw350i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S96373[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-S6012 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S96374[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonk770i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S96375[] = "Mozilla/5.0 (Linux; Android 4.1.1; WEXLER-TAB-7T Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S96376[] = "MicromaxX275/MstarBrowser1.0/MIDP2.0Configuration/CLDC1.1";
const char S96377[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:10.0.2) Gecko/X Thunderbird/10.0.2";
const char S96378[] = "Mozilla/5.0 (Linux; Android 4.0.3; eGlide XL PRO II Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S96379[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A95 Build/CelkonCelkon_A95) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96380[] = "Mozilla/5.0 (Linux; Android 4.0.4; X403 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S96381[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC VLE_U Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S96382[] = "JUC (Linux; U; 4.0.4; zh-cn; GT-I9308; 720*1280) UCWEB8.7.4.225/145/800";
const char S96383[] = "LG-GM360/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-gm360) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S96384[] = "SonyEricssonW995/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonw995) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S96385[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:19.0) Gecko/X Firefox/19.0 slurp";
const char S96386[] = "Opera/9.80 (Windows NT 5.1; Edition ASUS) Presto/2.12.388 Version/12.14";
const char S96387[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST70104-1 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96388[] = "YahooMobileMail/1.0 (Android Mail; 1.4.6) (GT-P7510;samsung;GT-P7510;4.0.4/IMM76D)";
const char S96389[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6810 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96390[] = "Instagram 3.2.0 Android (13/3.2; 160dpi; 600x976; samsung; GT-P6200; GT-P6200; smdkc210; en_US)";
const char S96391[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P7511 Build/IMM76L; CyanogenMod-Ganbarou V1.00 GT) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96392[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259";
const char S96393[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259";
const char S96394[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P6200 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S96395[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P3113 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96396[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; A8T Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96397[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P970 Build/JZO54K)";
const char S96398[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; ST27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96399[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S96400[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; vodafone_555_blue) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S96401[] = "Mozilla/5.0 (Linux; Android 4.2.1; SGH-I927 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S96402[] = "Mozilla/5.0 (Linux; Android 4.0.4; Spice_Mi-495 Build/SpiceSpice_Mi-495) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S96403[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U795 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S96404[] = "Nokia5300/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5300) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S96405[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; Penta IS701C Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96406[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96407[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MT27i Build/6.1.1.B.1.54) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96408[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT-AS45IPS Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96409[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P935 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96410[] = "Mozilla/5.0 (Linux; Android 4.0.4; Gpad_G1 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96411[] = "Mozilla/5.0 (Linux; Android 4.0.4; IRIS_501 Build/LAVAIRIS501) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96412[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Cloudfone ICE 2Gs Build/FSV06) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96413[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN700W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96414[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Y1 Build/HCL ME Tablet Y1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96415[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sgh-f480i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S96416[] = "Mozilla/5.0 (Java; U; en-us; nokia6680) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/405/UCWEB Mobile";
const char S96417[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-E730 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96418[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.2.2.259";
const char S96419[] = "Mozilla/5.0 (Linux; Android 4.1.2; P-02E Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96420[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; Micromax A30 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96421[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; Nexus 4 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S96422[] = "Mozilla/5.0 (Linux; Android 4.0.4; QW TB-9718D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S96423[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 97 TITANIUMHD Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S96424[] = "Mozilla/5.0 (Linux; Android 4.1.2; JY-G2 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96425[] = "Mozilla/5.0 (Linux; Android 4.0.4; Turkcell Maxi Plus 5 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96426[] = "Mozilla/5.0 (Linux; Android 4.0.4; A9 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96427[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.5) Gecko/X Firefox/3.6.5pre Fennec/1.1";
const char S96428[] = "Mozilla/5.0 (Windows 95) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2";
const char S96429[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S880 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96430[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; 932i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96431[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Barnes & Noble Nook HD+ Build/JZO54K)";
const char S96432[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE U790 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S96433[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAN91) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S96434[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; LG-MS840 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96435[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/ZV5.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S96436[] = "Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB A93.2 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S96437[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-S5830i Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Mobile Safari/537.31 OPR/14.0.1074.54070";
const char S96438[] = "Mozilla/5.0 (Linux; Android 4.0.3; A511 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S96439[] = "Mozilla/5.0 (Linux; Android 4.0.3; TAB466EUK Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S96440[] = "Mozilla/5.0 (Linux; Android 4.0.4; Philips W336 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96441[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 slurp";
const char S96442[] = "Mozilla/5.0 (Linux; Android 4.0.4; M-MP1010i Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S96443[] = "Mozilla/5.0 (Linux; Android 4.0.4; KHPHN05ANDA Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96444[] = "VG1515_Videocon/1.0 Qualcom/6240 Release/3.10.2011 Browser/Obigo Q7 Profile/MIDP-2.0 Configuration/CLDC1.1";
const char S96445[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; U30GT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96446[] = "JUC (Linux; U; 4.0.4; zh-cn; GT-I9300; 480*800) UCWEB8.2.2.135/145/800";
const char S96447[] = "Mozilla/5.0 (Linux; Android 4.0.4; ThL W5 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96448[] = "Mozilla/5.0 (Linux; Android 4.0.3; Maxwell_Lite Build/1.0.2-X-19:27) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S96449[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F100S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96450[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SCH-I400 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96451[] = "JUC (Linux; U; 2.2.1; zh-cn; Ideos; 240*320) UCWEB7.9.0.94/139/444";
const char S96452[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung gt-e2222) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile,SAMSUNG-GT-E2222/E2222JVKJ3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S96453[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TOUCH85 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96454[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S96455[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P705 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96456[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:20.0) Gecko/X Firefox/20.0 slurp";
const char S96457[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; AD683G) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S96458[] = "Mozilla/5.0 (Linux; Android 4.0.4; F7HD Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S96459[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; m865 Build/Ultimate CM7; TITAN V7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96460[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT-I9000) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S96461[] = "Mozilla/5.0 (Linux; Android 4.0.4; GN800 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96462[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAP5000DUO Build/PrestigioPAP5000DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96463[] = "Mozilla/5.0 (Linux; Android 2.3.6; SAMSUNG-SGH-I927 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S96464[] = "samsung-gt-s3850/UC Browser7.6.1.82/69/352";
const char S96465[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID9042 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S96466[] = "OneBrowser/3.1 (Nokia5310XpressMusic/10)";
const char S96467[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-E400 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96468[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Y1 Build/HCL ME Tablet Y1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96469[] = "Mozilla/5.0 (Linux; Android 4.0.4; UBISLATE7C+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S96470[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; SGH-T959) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S96471[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; Vodafone_858) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S96472[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TM-7024 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96473[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Digma iDj7 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96474[] = "Mozilla/5.0 (Linux; Android 4.0.4; 754TPC Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S96475[] = "Mozilla/5.0 (Linux; Android 4.0.4; A5Classic Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96476[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromax) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S96477[] = "Mozilla/5.0 (Linux; Android 4.0.4; Andi4.5h Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96478[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOLO_A800 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96479[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.53) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S96480[] = "Mozilla/5.0 (Linux; Android 4.1.1; R8113 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96481[] = "Mozilla/5.0 (X11; OpenBSD amd64; rv:21.0) Gecko/X Firefox/21.0";
const char S96482[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Sensation XE Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96483[] = "Mozilla/5.0 (Linux; Android 4.0.4; 201M Build/7.7.1Q-164_SMJ-165) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96484[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAP4040_DUO Build/PrestigioPAP4040DUO) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S96485[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L300 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S96486[] = "Mozilla/5.0 (Linux; Android 4.1.2; Q800 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S96487[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xolo_X500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96488[] = "Mozilla/5.0 (Linux; Android 4.0.4; BNTV600 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S96489[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; MTS-SP101) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S96490[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; iris502) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S96491[] = "Mozilla/5.0 (Linux; Android 4.0.4; Engage 7HD Build/Engage7HD.Starmobile.V012.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S96492[] = "Mozilla/5.0 (Linux; Android 4.0.4; Aqua flash Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96493[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-S5570I Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S96494[] = "Mozilla/5.0 (Linux; Android 4.1.1; ZTE V807 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96495[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-US; GT-I9000 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S96496[] = "Mozilla/5.0 (X11; U; Linux mips64; en-US; rv:1.8.1.14) Gecko/X Mandriva/2.0.0.14-3.3mdv2008.1 (2008.1) Firefox/2.0.0.14";
const char S96497[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus One Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96498[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Micromax_A72 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S96499[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; TAB1050 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96500[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax_P600) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S96501[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; One XL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S96502[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A5 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S96503[] = "SAMSUNG-GT-E2220/E2220XXLA2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S96504[] = "HS-U950/1.0 Linux/3.0 Android/4.0 Browser/AppleWebKit534.30 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S96505[] = "Lemon B489_Maui Browser/3.83; Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96506[] = "Mozilla/5.0 (Linux; Android 4.1.1; MTV_volt Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S96507[] = "Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook HD Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S96508[] = "JUC (Linux; U; 4.1.2; zh-cn; ST26i; 480*854) UCWEB7.9.0.94/139/444";
const char S96509[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i667;7.10;8862)";
const char S96510[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6400L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S96511[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Elcano Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96512[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P999 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S96513[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9105P Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S96514[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6352 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S96515[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; Sen S2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96516[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U795 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S96517[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 OverDrive Media Console/2.6.5";
const char S96518[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9505 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S96519[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9268 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S96520[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S96521[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U705T Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S96522[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax P560 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S96523[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I9003) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S96524[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo A798t Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S96525[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; AURUS III Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S96526[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6812) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S96527[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A820L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96528[] = "Mozilla/5.0 (Linux; Android 4.0.4; A9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96529[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-01D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S96530[] = "Mozilla/5.0 (Linux; Android 4.0.4; IMO S88 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96531[] = "Mozilla/5.0 (Linux; Android 4.2.1; mobiistar touch LAI 504 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96532[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-p6800 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S96533[] = "Mozilla/5.0 (Linux; Android 4.0.4; VIVO 4.3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96534[] = "Mozilla/5.0 (Linux; Android 4.2.1; UMI X1S Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96535[] = "Mozilla/5.0 (Linux; Android 4.2.2; Cyclone Voyager 2 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S96536[] = "Mozilla/5.0 (Linux; Android 4.0.4; Thrill 430x Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96537[] = "Mozilla/5.0 (Linux; Android 4.1.2; FPT V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96538[] = "Mozilla/5.0 (Linux; Android 4.1.1; Celkon A119 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96539[] = "Mozilla/5.0 (Linux; Android 4.0.4; Legend2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96540[] = "Mozilla/5.0 (Linux; Android 4.1.2; 201F Build/V35R13E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96541[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC One Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb";
const char S96542[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:14.0) Gecko/X Firefox/14.0.1";
const char S96543[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; U8665 Build/HuaweiU8665; CyanogenMod 7.2 Unofficial) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96544[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; LG-P880 Build/JDQ39E)";
const char S96545[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; SamsungI8910) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S96546[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0000 Build/HuaweiY300-0000) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96547[] = "Mozilla/5.0 (Linux; Android 4.0.4; Quantum 7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S96548[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S5312) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S96549[] = "Mozilla/5.0 (Linux; Android 4.3; SAMSUNG-SGH-I717 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96550[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Intex_Aqua_Marvel) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S96551[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; s8003) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S96552[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Celkon A118 Build/CelkonA118) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96553[] = "Googlebot/2.1 (+http://www.googlebot.com/bot.html; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 3.5.30729; .NET CLR 3.0.3;c2VvLWNyYXdsZXI=";
const char S96554[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; A75) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S96555[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx368) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S96556[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9190 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.20.1364.172 Mobile Safari/537.22";
const char S96557[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; s5230) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S96558[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; HTC Ruby Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S96559[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.20.1364.172 Mobile Safari/537.22";
const char S96560[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S96561[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; HTC One Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S96562[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I537 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S96563[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9505 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S96564[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E615 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S96565[] = "Mozilla/5.0 (Linux; Android 4.3; GT-I9000 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S96566[] = "Mozilla/5.0 (iPad; U; CPU OS 6_1_3 like Mac OS X; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.10080IT Mobile";
const char S96567[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E410 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96568[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; Lenovo P780_ROW Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33482";
const char S96569[] = "Instagram 4.1.2 Android (16/4.1.2; 320dpi; 720x1280; samsung/Verizon; SCH-I535; d2vzw; qcom; en_US)";
const char S96570[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S5312) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S96571[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; LT18i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S96572[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaX7-00) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S96573[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; SPH-M930BST Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S96574[] = "UCWEB/2.0 (Linux; U; Adr 4.2.2; en-US; GT-I9082) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S96575[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Raider X710e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S96576[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT22i Build/6.1.1.B.1.54) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S96577[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ607 Build/7.7.1-122-4-FLEM-WIFI-LA) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S96578[] = "UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-I9100) U2/1.0.0 UCBrowser/9.3.0.321 Mobile";
const char S96579[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A90S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S96580[] = "Mozilla/5.0 (Linux; Android 4.0.4; S22 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S96581[] = "UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; Micromax A116) U2/1.0.0 UCBrowser/9.3.0.321 Mobile";
const char S96582[] = "Mozilla/5.0 (Linux; Android 4.0.4; PNDPP48GP Build/ICS.PNDPP48GP.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S96583[] = "Mozilla/5.0 (Linux; Android 4.0.4; LNV-Lenovo A600e Build/ICS) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96584[] = "Mozilla/5.0 (Linux; Android 4.0.3; ADR910L Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S96585[] = "Mozilla/5.0 (Linux; Android 4.2.1; MyPhone A919i Duo Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S96586[] = "Mozilla/5.0 (Linux; Android 4.0.4; w30hd(QuadCore) Build/W30hd) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S96587[] = "Mozilla/5.0 (Linux; Android 4.1.1; Vodafone 875 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S96588[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N5110 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S96589[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT28h Build/6.2.B.0.211) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S96590[] = "Nokia2700c-2/2.0 (09.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2700c-2/UC Browser7.6.1.82/69/352";
const char S96591[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2690/UC Browser7.6.1.82/69/352";
const char S96592[] = "SAMSUNG-SGH-L310/L310XEHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S96593[] = "BlackBerry8820/4.5.0.182 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102 UP.Browser/5.0.3.3";
const char S96594[] = "BlackBerry8100/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100 UP.Browser/5.0.3.3";
const char S96595[] = "HTC_Touch_HD_T8282 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S96596[] = "LG-ME970d MIC/1.1.14 MIDP-2.0/CLDC-1.1";
const char S96597[] = "LG-GT810H Netfront/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S96598[] = "SonyEricssonK508c/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96599[] = "SonyEricssonK508i/R2AA SEMC-Browser/4.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96600[] = "SonyEricssonK508i/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96601[] = "SAMSUNG-SGH-X650/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S96602[] = "SAMSUNG-GT-E1130/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S96603[] = "SAMSUNG-GT-E1130B/E1130BXXJD1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S96604[] = "Samsung-SPHM610 AU-MIC-M610/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96605[] = "Videocon_1405_SwL110C_03A9_V1_0_B_Wap Browser";
const char S96606[] = "NexianNX-G730/MTK Release/10.01.2009 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1";
const char S96607[] = "LG-GM210/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S96608[] = "SAMSUNG-SGH-E500/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S96609[] = "Nokia5800 XpressMusic/UC Browser7.2.2.51/50/352";
const char S96610[] = "LG/KU385/v1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96611[] = "SAMSUNG-M3510/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S96612[] = "Samsung-SCHR561 Infraware-Polaris/6.0 MMP/2.0";
const char S96613[] = "LG/U300/v1.0";
const char S96614[] = "Xda_Ignito/480x640 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S96615[] = "K451/1.0 MTK/MAUI.08A.W08.28.MP.V2 Release/7.30.2009 Browser/OBIGO-Q03C";
const char S96616[] = "TIANYU-KTOUCH/H877/MIDP2.0/CLDC1.1/Screen-320X240";
const char S96617[] = "TIANYU-KTOUCH/E62/MIDP2.0/CLDC1.1/Screen-240X400";
const char S96618[] = "Q-mobile M56";
const char S96619[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; NOKIA; Lumia 800)";
const char S96620[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Snap_S521; Windows Phone 6.5)";
const char S96621[] = "sonyericssonw580i/UC Browser7.9.0.102/70/352";
const char S96622[] = "sonyericssonw580i/UC Browser7.9.0.102/69/352";
const char S96623[] = "Mozilla/5.0 Opera/9.5 (KDDI-TS3R; BREW; Opera Mobi; U; ja) Presto/2.2.1";
const char S96624[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.0.9) Gecko/X Firefox/3.0.9 Wyzo/3.0.3";
const char S96625[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1) Opera 7.54  [en]";
const char S96626[] = "Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; de-de; MZ601 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S96627[] = "Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; en-gb; MZ601 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S96628[] = "nokia5250/UC Browser7.9.0.102/69/352  3gpp-gba";
const char S96629[] = "Mozilla/5.0 (X11; U; Linux i686; en-GB; rv:1.9.2.24) Gecko/X Ubuntu/10.04 (lucid) Firefox/2.0.0.6";
const char S96630[] = "MQQBrowser/2.4 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S96631[] = "MQQBrowser/2.4 Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Safari/7534.48.3";
const char S96632[] = "MQQBrowser/2.7 (Nokia5230;SymbianOS/9.1 Series60/3.0)";
const char S96633[] = "Nokia5228/UC Browser7.9.0.102/50/355/UCWEB";
const char S96634[] = "sonyericssonw580i/UC Browser8.0.3.107/69/444";
const char S96635[] = "sonyericssonw580i/UC Browser8.0.3.107/70/352";
const char S96636[] = "Mozilla/5.0 (S60V3; U; en-us; NokiaE51)/UC Browser8.4.0.159/28/352/UCWEB Mobile";
const char S96637[] = "Mozilla/5.0 (S60V3; U; en-in; NokiaE52-1)/UC Browser8.4.0.159/28/352/UCWEB Mobile";
const char S96638[] = "HTC_Touch_Diamond2_T5353 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB7.9.0.94/31/800";
const char S96639[] = "Mozilla/5.0 (Series40; Nokia202/20.28; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S96640[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Comodo_Dragon/21.0.2.0";
const char S96641[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Comodo_Dragon/21.0.2.0";
const char S96642[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; fj_vortex Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96643[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; Ainol Novo8 Advanced Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96644[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Ainol Novo8 Advanced Build/v1012_30_809_D5_CHDJ_A38_v1012.60-C_CH@X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96645[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; AM7-A1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96646[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; GT-I5801 Build/ECLAIR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S96647[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; Karbonn_A1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S96648[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T679 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96649[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T679 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96650[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.0.2) Gecko/X Netscape/7.01";
const char S96651[] = "Mozilla/5.0 (Windows NT 5.1; rv:5.0) Gecko/X Firefox/5.0 CometBrowser/5.0";
const char S96652[] = "Mozilla/5.0 (X11; U; Linux; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.2.2.6613 Safari/532.4";
const char S96653[] = "Mozilla/5.0 (Java; U; en-us; sonyericssonk800i) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S96654[] = "DoCoMo/2.0 P906i(c100;TB;W24H15) AppEngine-Google; (+http://code.google.com/appengine; appid: s~google.com:mobile-checker)";
const char S96655[] = "KDDI-CA3A UP.Browser/6.2.0.13.2 (GUI) MMP/2.0 AppEngine-Google; (+http://code.google.com/appengine; appid: s~google.com:mobile-checker)";
const char S96656[] = "Mozilla/5.0 (Java; U; en-us; s5230) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S96657[] = "SoftBank/1.0/920SH/SHJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1 AppEngine-Google; (+http://code.google.com/appengine; appid: s~google.com:mobile-checker)";
const char S96658[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA5630d-1)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S96659[] = "iBrowser/for Blackberry2.0.0 (BlackBerry8520/5.0.0.592)";
const char S96660[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax P250(Funbook) Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S96661[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96662[] = "Mozilla/5.0 (Linux; Android 4.0.3; Woxter Tablet PC 100CX Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S96663[] = "MQQBrowser/2.8 (NokiaC5-05;SymbianOS/9.1 Series60/3.0)";
const char S96664[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; Nexus S Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S96665[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-b5310) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S96666[] = "Mozilla/5.0 (Java; U; en-us; micromax) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S96667[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s3370) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S96668[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE50)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S96669[] = "Mozilla/5.0 (Linux; Android 4.0.3; PG86100 Build/MIUI) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S96670[] = "Mozilla/5.0 (Linux; Android 4.0.4; N-08D Build/AX) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S96671[] = "Opera/9.80 (BREW; Opera Mini/4.3.4/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S96672[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows 2000)";
const char S96673[] = "Nokia205/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia205) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S96674[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MOMO15 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96675[] = "Netscape 4.8 (Win XP): Mozilla/4.8 (Windows NT 5.1; U)";
const char S96676[] = "Nokia3120classic/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia3120classic) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S96677[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 7 Build/JOP40D; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96678[] = "LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-gs500) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S96679[] = "Mozilla/5.0 (Linux; Android 4.0.3; HWA580 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S96680[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; en-in; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96681[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; V9C Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96682[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s8500) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S96683[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A25 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S96684[] = "Mozilla/5.0 (Linux; Android 4.0.4; Turkcell Maxi Plus 5 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96685[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;HD7 T9292;7.10;8862)";
const char S96686[] = "Android/1.0 (GT-P7300 HTJ85B)";
const char S96687[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-gb; GT-P7310 Build/HRI83) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S96688[] = "YahooMobileMessenger/1.0 (Android Messenger; 1.8.3) (GT-P1000; samsung; GT-P1000; 2.3.3/GINGERBREAD)";
const char S96689[] = "Mozilla/5.0 (Linux; U; Android 3.2; en_us ; GT-P6200 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S96690[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-US; GT-P6200 Build/HTJ85B) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S96691[] = "Instagram 3.0.5 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7500; GT-P7500; p3; en_US)";
const char S96692[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.15/274,1.9.15.274,2012-12-21 15:38, +0800";
const char S96693[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7300B Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0700, th";
const char S96694[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; fr-fr; GT-P5110 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +0100, fr";
const char S96695[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-ca; GT-P5110 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, -0800, ca";
const char S96696[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; GT-P5110 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0100, fr";
const char S96697[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; GT-P5110 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.27/381,1.9.27.381,2013-03-21 11:39, +0100, fr";
const char S96698[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-P3100 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31";
const char S96699[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-P3100 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S96700[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328d) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96701[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE Plus Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S96702[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A11 Build/KarbonnA11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96703[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; A90 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96704[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; IM-A760S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96705[] = "HD_mini_T5555 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.6.1.82";
const char S96706[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I897 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96707[] = "OneBrowser/3.0 (Nokia305/03.60)";
const char S96708[] = "JUC (Linux; U; 2.3.4; zh-cn; R800i; 480*854) UCWEB7.9.0.94/139/444";
const char S96709[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MT15i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96710[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Vision Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96711[] = "Mozilla/5.0 (Linux; Android 4.1.2; NexusHD2 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96712[] = "Mozilla/5.0 (Linux; Android 4.2.2; Ascend G300 Build/JOP40C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96713[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:21.0) Gecko/X Thunderbird/21.0";
const char S96714[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.7.5) Gecko/X Netscape/8.1.3";
const char S96715[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.3) Gecko/X Ubuntu/8.04 Firefox/4.0";
const char S96716[] = "NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/70/444";
const char S96717[] = "Mozilla/5.0 (Windows NT 4.10) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12";
const char S96718[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (KFOT, 35, 1.7.0)";
const char S96719[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonw580i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S96720[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; GT-I9000 Build/ECLAIR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96721[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:20.0) Gecko/X Firefox/20.0 slurp";
const char S96722[] = "Mozilla/5.0 (Linux; Android 4.1.2; A953 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96723[] = "Mozilla/5.0 (Linux; Android 4.0.4; A2 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96724[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax P500(Funbook) Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S96725[] = "Mozilla/5.0 (Linux; Android 4.0.4; W30(QuadCore) Build/W30) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S96726[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/MIUIv5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96727[] = "Mozilla/5.0 (Linux; Android 4.0.4; 754TPC Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S96728[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 5 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96729[] = "Mozilla/5.0 (Linux; Android 4.0.4; fnac 4.5 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96730[] = "Mozilla/5.0 (Linux; Android 4.1.1; GOCLEVER TAB M723G Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S96731[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; ST25i Build/6.1.1.B.1.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96732[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL32EX650; CC/VNM; vi) Presto/2.10.250 Version/11.60";
const char S96733[] = "Nokia5610d-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5610d-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S96734[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; Karbonn_A1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96735[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S96736[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-P930 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96737[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2; ARM; Trident/4.0; Touch; .NET4.0E; .NET4.0C; Tablet PC 2.0)";
const char S96738[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S96739[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq55) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S96740[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328d Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S96741[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A798t Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S96742[] = "Mozilla/5.0 (Linux; Android 4.0.4; S22 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96743[] = "OneBrowser/3.1 (NokiaE5-00/101.003/sw_platform=S60;sw_platform_version=3.2;java_build_version=1.0.5)";
const char S96744[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; WM8850-mid Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S96745[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.2.2.259";
const char S96746[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT918 Build/2_26C_2009) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S96747[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) iCab/4.9 Safari/533.16";
const char S96748[] = "JUC(Linux;U;Android4.0.4;Zh_cn;HTC Incredible S;480*800;)UCWEB7.8.0.95/139/444";
const char S96749[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB307 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S96750[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I957 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96751[] = "JUC (Linux; U; 4.0.4; zh-cn; HUAWEI_U8825D; 480*800) UCWEB7.9.0.94/139/444";
const char S96752[] = "Nokia114/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia114) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S96753[] = "Mozilla/5.0 (Linux; Android 4.0.4; MP969 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S96754[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-N8000) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S96755[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; HTC_One_X) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S96756[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Streak 7 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96757[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; Spice_Mi-320) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S96758[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Q-Smart_S1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S96759[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9003 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96760[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C5155 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S96761[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M380W Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S96762[] = "Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76B) AppleWebKit/[WEBKIT_VERSION] (KHTML, like Gecko) Chrome/[CHROME_VERSION] Mobile Safari/[WEBKIT_VERSION] pss-webkit-request";
const char S96763[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP7170B3G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S96764[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S96765[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S96766[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; A90S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S96767[] = "Mozilla/5.0 (Linux; Android 4.0.3; ThinkPad Tablet Build/ThinkPadTablet_A400_03) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S96768[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; N861 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S96769[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-mobile i-STYLE 6 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96770[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5570C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S96771[] = "Mozilla/5.0 (Linux; Android 4.1.2; ThL W5 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96772[] = "Mozilla/5.0 (Linux; Android 2.3.6; LG-E400 Build/GRK39F) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S96773[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; OfficeLiveConnector.1.4; .NET CLR 3.0.30729; OfficeLivePatch.1.3; FDM) chromeframe/4.0";
const char S96774[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9400 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96775[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A820L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96776[] = "Mozilla/5.0 (Linux; Android 4.0.4; N-08D Build/AX) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S96777[] = "Mozilla/5.0 (Linux; Android 4.1.2; ME865 Build/6.7.2_GC-384) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96778[] = "Mozilla/5.0 (Linux; Android 4.0.4; GLADIATOR Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96779[] = "Mozilla/5.0 (Linux; Android 4.1.1; Vodafone 975 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96780[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P970 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96781[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Sensation Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S96782[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; C1604 Build/11.3.A.2.1) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S96783[] = "Mozilla/5.0 (Linux; Android 2.3.6; SCH-I405 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S96784[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LG-MS770 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S96785[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; A97i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S96786[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Videocon A15 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S96787[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U807 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S96788[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233w) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S96789[] = "UCWEB/2.0 (Java; U; MIDP-2.0; vi; gt-s5233w) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S96790[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TCL A510 Build/TCL_A510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96791[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PIRANHA ULTRA TAB 10.1\" Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96792[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-S7572 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96793[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; zh-CN; GT-I9003) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile";
const char S96794[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; V9A) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S96795[] = "IUC(U;iOS 6.1.3;Zh-cn;320*480;)/UCWEB8.3.1.151/42/997";
const char S96796[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Lemon_P9) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S96797[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax_P360) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S96798[] = "Mozilla/5.0 (Linux; Android 4.0.4; HS-10DTB2 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S96799[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.27/385,1.9.27.385,2013-03-31 15:31, +0000, us";
const char S96800[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Incredible 2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96801[] = "Mozilla/5.0 (Linux; Android 4.0.4; S22 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96802[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-MS910 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S96803[] = "Mozilla/5.0 (Linux; Android 4.2.2; Novo7 Venus Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S96804[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S96805[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LNV-Lenovo A790e Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S96806[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-Optimus Build/VDTv6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96807[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6312 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S96808[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R970 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S96809[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.1; en-in; C1504 Build/11.3.A.0.47) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S96810[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; C1604) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S96811[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHL21 Build/S5170) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S96812[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; MT27i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S96813[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9050 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S96814[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx78) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S96815[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; X907 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S96816[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Intex Aqua Marvel) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S96817[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A5Classic Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S96818[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI MT1-U06 Build/HuaweiMT1-U06) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S96819[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S7560 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S96820[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Karbonn A2 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S96821[] = "Mozilla/5.0 (Linux; Android 2.3.3; GT-I9100 Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60015";
const char S96822[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A111 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S96823[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx368) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S96824[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; A50 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S96825[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua Sx) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S96826[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S96827[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.4; en-in; WT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S96828[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; SGH-T989 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S96829[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; C5170 Build/IML77) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S96830[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-E450 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S96831[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP7100D3G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S96832[] = "Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291";
const char S96833[] = "Mozilla/5.0 (Linux; Android 4.1.1; Optimus_San_Remo Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96834[] = "Mozilla/5.0 (Linux; Android 4.2.1; ZP800H Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96835[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9103 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96836[] = "Mozilla/5.0 (Linux; Android 4.0.4; Fly_IQ442 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96837[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9506 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96838[] = "Mozilla/5.0 (Linux; Android 4.1.1; O+ 8.52 Android Build/Oplus8.52_Android) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96839[] = "Mozilla/5.0 (Linux; Android 4.0.4; IRIS_501 Build/LAVAIRIS501) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96840[] = "Mozilla/5.0 (Linux; Android 4.1.2; IQ451 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96841[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAC-97032 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S96842[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; Karbonn A2+ Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S96843[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us";
const char S96844[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I605 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us";
const char S96845[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; SPH-D710 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S96846[] = "Mozilla/5.0 (WinNT4.0) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2";
const char S96847[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; iBall_Slide_3G7271) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S96848[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);";
const char S96849[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Z Build/JZO54K; CyanogenMod-CM10 base (X unofficial build by quarx, compiled by sevenrock)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96850[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A800S Build/HOANG MINH H?U_VEGAVIETTEAM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96851[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; R813T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96852[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S96853[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8262) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S96854[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax A91 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96855[] = "Mozilla/5.0 (Linux; Android 4.1.2; ALCATEL ONE TOUCH 8000A Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S96856[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A2) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S96857[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S96858[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0530, in";
const char S96859[] = "sonyericssons312/UC Browser8.0.3.107/69/444";
const char S96860[] = "Mozilla/5.0 (X11; Linux x86_64; HUAWEI Y300-0151 Build/HuaweiY300-0151) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24 TT/4.1.1";
const char S96861[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Micromax A72 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.3.2.300";
const char S96862[] = "Googlebot/2.1 (+http://www.googlebot.com/bot.html; MSIE 8.0; Windows NT 5.1; Trident/4.0; FunWebProducts; GTB6.5; .NET CLR 1.1.4322; .NET CLR 2.0.5072;c2VvLWNyYXdsZXI=";
const char S96863[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-e2222) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S96864[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I739 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S96865[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Mobile Safari/535.19 Silk-Accelerated=false";
const char S96866[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LG-LU6200 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S96867[] = "Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A390_ROW Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S96868[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-gb; GT-I9000 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96869[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I800 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96870[] = "Mozilla/5.0 (Linux; Android 4.1.1; BS471 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S96871[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:23.0) Gecko/X Firefox/23.0 slurp";
const char S96872[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Rikomagic MK802IIIS Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96873[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; MOVE Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96874[] = "Mozilla/5.0 (Series40; Nokia2010/11.95; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S96875[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac6-00) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba";
const char S96876[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT28h Build/6.2.B.0.211) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S96877[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MT25i) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S96878[] = "LG-A390/V100 Obigo/Q05A MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S96879[] = "UCWEB/2.0 (Linux; U; Adr 4.2.2; en-US; GT-I9082) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S96880[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; LG-E467f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96881[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN85) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S96882[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8552) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S96883[] = "Mozilla/5.0 (Linux; Android 4.2.2; KYY21 Build/205.0.0d00) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S96884[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LT18i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S96885[] = "Mozilla/5.0 (Linux; Android 4.2.2; A501 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S96886[] = "Mozilla/5.0 (Linux; Android 4.2.2; Cyclone Voyager 2 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S96887[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LG-MS770 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S96888[] = "Mozilla/5.0 (Linux; Android 4.0.4; Thrill 430x Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S96889[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Incredible S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S96890[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S96891[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.1.0.64021 Mobile/10A523 Safari/7534.48.3";
const char S96892[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.1.0.64021 Mobile/10A8426 Safari/7534.48.3";
const char S96893[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.1.0.64021 Mobile/10A8500 Safari/7534.48.3";
const char S96894[] = "Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.1.0.64021 Mobile/10A550 Safari/7534.48.3";
const char S96895[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.1.1.64324 Mobile/10A403 Safari/7534.48.3";
const char S96896[] = "Windows Phone Search (Windows Phone OS 8.0;NOKIA;Lumia 822;8.0;10327)";
const char S96897[] = "Mozilla/5.0 (Linux; Android 4.0.4; O+_8.9_Android Build/OplusOplus_A1000) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S96898[] = "Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S96899[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7102 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.5 Mobile Safari/533.1";
const char S96900[] = "Mozilla/5.0 (Linux; Android 4.1.2; ME371MG Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S96901[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-H Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S96902[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-L710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S96903[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-M950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S96904[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E971 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S96905[] = "MOT-E790/0E.30.34R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96906[] = "MOT-E1 iTunes/0E.30.42R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96907[] = "MOT-E1 - RokR/00.00.00 MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96908[] = "MOT-E1 ROKR iTunes/0E.30.48R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96909[] = "MOT-E1 Walkman Phone/0E.30.48R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96910[] = "MOT-ROKR_E1/0E.30.48R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96911[] = "A3100 Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/16108; U; en)";
const char S96912[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; *HTC/X05HT*) Opera 9.50";
const char S96913[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; SHARP/WS020SH; PPC; 480x800) Opera Mobi; Opera 9.5 [ja]";
const char S96914[] = "SonyEricssonC902/R3EG SEMC/NewsReader/1.0";
const char S96915[] = "SAMSUNG-GT-E2550/1.0 RTK-E/1.0 DF/1.0 Release/23.04.2009 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzUzMDkzMDQ2NTEwMjE4 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S96916[] = "SonyEricssonK790c/R8BA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96917[] = "SonyEricssonK790/R2B Release/Feb-07-2007 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96918[] = "SonyEricssonW700c/R1DB Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96919[] = "Spice M-940 Gold/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1, Spice M-940 Gold/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S96920[] = "Spice M-940 Gold/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S96921[] = "Nokia5132_CMCC/2.0 (07.91) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S96922[] = "LG-TU550 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96923[] = "SAMSUNG-SGH-T409/T409UVGE3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S96924[] = "Samsung-SPHZ400 AU-MIC-Z400/2.0 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S96925[] = "LGE-AX260/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S96926[] = "NexianNX-G920/MTK Release/01.01.2010 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1";
const char S96927[] = "SAMSUNG-SGH-A766/A766UXIA4 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S96928[] = "Mozilla/5.0 (Series40; NokiaX2-00/08.17; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.1.21.8";
const char S96929[] = "Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2730c-1/UC Browser7.8.0.95/70/401";
const char S96930[] = "SAMSUNG-GT-C6112/C6112DDJB2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S96931[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Xda Orbit; 240x320)";
const char S96932[] = "ZTE-F165/1.0 ACS-NF/3.2 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96933[] = "NexianNX-G235/MTK Release/10.01.2010 Browser/MAUI profile";
const char S96934[] = "NEC-N6206/1.0 Release/06.15.2006 Browser/CMS2.0.0 Profile/MIDP-2.0";
const char S96935[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; Taylor)";
const char S96936[] = "i-mobileIE3250";
const char S96937[] = "Samsung-SPHM510 AU-MIC-M510/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S96938[] = "LG-KM380 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S96939[] = "Videocon_V1688(Twist)_WAP2.0_MIDP2.0_Configration/CLDC1.1";
const char S96940[] = "Nokia5530/UC Browser7.8.0.95/50/355";
const char S96941[] = "NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-02/UC Browser7.8.0.95/70/355";
const char S96942[] = "SonyEricssonT100/R101";
const char S96943[] = "KDDI-KC3I UP.Browser/6.2.0.14.1.2 (GUI) MMP/2.0";
const char S96944[] = "j2me/UC Browser7.9.0.102/69/352";
const char S96945[] = "maui/UC Browser7.9.0.102/69/352 /AJSC/1.5";
const char S96946[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; sdk Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S96947[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; google_sdk Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S96948[] = "Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S96949[] = "Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S96950[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SPH-D710 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96951[] = "Mozilla/5.0(Linux;U;Android2.3.5;Zh_cn;HTC Incredible S;480*800;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S96952[] = "JUC(Linux;U;Android2.3.3;Zh_cn;HTC Incredible S;480*800;)UCWEB7.8.0.95/139/351";
const char S96953[] = "JUC(Linux;U;Android2.3.5;Zh_cn;HTC Incredible S;480*800;)UCWEB7.8.0.95/139/352";
const char S96954[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/ITL41F) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7";
const char S96955[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM30C) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S96956[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/HLK75F) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7";
const char S96957[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/IML77) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S96958[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM76) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7";
const char S96959[] = "Mozilla/5.0 (X11; U; Linux x86_64; C) AppleWebKit/533.3 (KHTML, like Gecko) PhantomJS/1.2.0 Safari/533.3";
const char S96960[] = "browseripad/23286 CFNetwork/467.12 Darwin/10.3.1";
const char S96961[] = "Perfect%20Browser-iPad/2.91 CFNetwork/467.12 Darwin/10.3.1";
const char S96962[] = "BlackBerry8900/4.2.0.127 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/159/444";
const char S96963[] = "YL-COOLPADN900/03.15 POLARIS/5.30 CTC/1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)";
const char S96964[] = "YL-COOLPADN900/03.15 POLARIS/5.30 CTC/1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)";
const char S96965[] = "MQQBrowser/2.0 (NokiaC7-00;SymbianOS/9.1 Series60/3.0)";
const char S96966[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:10.0.2) Gecko/X Firefox/10.0.2 SeaMonkey/2.7.2";
const char S96967[] = "Opera/9.80 (Windows NT 5.1; Opera Tablet/11648; U; en) Presto/2.7.81 Version/11.00";
const char S96968[] = "LENOVO-P780/176A";
const char S96969[] = "lg-gu220/UC Browser7.9.0.102/69/352";
const char S96970[] = "sonyericssonk770i/UC Browser8.0.3.107/69/444";
const char S96971[] = "sonyericssonk770i/UC Browser8.0.3.107/69/352";
const char S96972[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL32CX523; CC/GBR; en) Presto/2.7.61 Version/11.00";
const char S96973[] = "Mozilla/5.0 (Windows NT 6.0; rv:9.0.1) Gecko/X Firefox/9.0.1 Epic/9.0.1";
const char S96974[] = "Bing%20for%20iPad/1.1.2 CFNetwork/485.12.7 Darwin/10.4.0";
const char S96975[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.7) Gecko/X Epic/1.1 Firefox/3.6.7";
const char S96976[] = "Mozilla/5.0 (S60V5; U; en-in; NokiaN97 mini)/UC Browser8.4.0.159/50/352/UCWEB Mobile";
const char S96977[] = "NokiaN97 mini/UCWEB8.4.0.150/50/999";
const char S96978[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S96979[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959 Build/IMM76I; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S96980[] = "Mozilla/5.0 (X11; SunOS sun4u; rv:7.0) Gecko/X Firefox/7.0";
const char S96981[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:7.0) Gecko/X Firefox/7.0 SeaMonkey/2.4";
const char S96982[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46HX823; CC/GBR; en) Presto/2.7.61 Version/11.00";
const char S96983[] = "sgh-f480i/UC Browser7.9.0.102/69/352";
const char S96984[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HT_PAD1051 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96985[] = "JUC(Linux;U;Android2.2.2;Zh_cn;Karbonn_A1;240*320;)UCWEB7.8.0.95/139/444";
const char S96986[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Evo 4G Build/MIUI) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S96987[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96988[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03H; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S96989[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ST18i Build/4.0.A.2.368; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96990[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; GT-I5503 Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S96991[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; GT-I5503 Build/ERE27) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S96992[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96993[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LS855 Build/CTMod 3.68)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96994[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; iDx10 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S96995[] = "JUC (Linux; U; 2.2.2; zh-cn; Lenovo_A390e; 320*480)";
const char S96996[] = "Mozilla/5.0 (Linux; Android 4.0.3; AN7G3 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S96997[] = "Mozilla/5.0 (Windows; U; WinNT4.0; en-US; rv:1.8.1.12) Gecko/X Firefox/2.0.0.12 Navigator/9.0.0.6";
const char S96998[] = "Opera/9.80 (X11; FreeBSD 9.0-RELEASE-p3 amd64; U; en) Presto/2.10.289 Version/12.02";
const char S96999[] = "JUC(Linux; U;2.3.5;zh_cn;HTC S510b;480*800;)UCWEB7.6.0.75/139/32500";
const char S97000[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE90)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S97001[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, gb";
const char S97002[] = "Opera/9.80 (Windows NT 5.0; U; Edition Next; de) Presto/2.11.310 Version/12.50";
const char S97003[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MTP-710 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97004[] = "Dolphin/6.2 CFNetwork/609 Darwin/13.0.0";
const char S97005[] = "SAMSUNG-SGH-T679/100.20306";
const char S97006[] = "Mozilla/5.0 (Java; U; en-us; sonyericssonu5i) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S97007[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97008[] = "Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/69/444";
const char S97009[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT862 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97010[] = "Mozilla/5.0 (Linux; Android 4.0.4; MiniPro Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97011[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/(Windows; U; Windows NT 5.1; en-US) AppleWebKit/23.411; U; en) Presto/2.5.25 Version/10.54";
const char S97012[] = "Mozilla/5.0 (Linux; Android 4.0.3; TOUCHMATE Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S97013[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97014[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; A73 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97015[] = "Mozilla/5.0 (compatible; Konqueror/2.2.2; FreeBSD)";
const char S97016[] = "AppleCoreMedia/1.0.0. (AAS-2.2.2; U; Windows NT 6.1)";
const char S97017[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s3650) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S97018[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonk750i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S97019[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; U20i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S97020[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; A1_07 Build/GINGERBREAD; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97021[] = "iBrowser/2.7 (NokiaN97mini;SymbianOS/9.1 Series60/3.0)";
const char S97022[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Sensation XE Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97023[] = "MQQBrowser/3.7/Adr (Linux; U; 4.0.4; zh-cn; GT-I9220 Build/IMM76D.ZCLPR;800*1280)";
const char S97024[] = "samsung-gt-s5380k/UC Browser7.6.1.82/69/351";
const char S97025[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC_T328w; 480*800) UCWEB7.9.4.145/139/800";
const char S97026[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; TOOKY A110 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97027[] = "Mozilla/5.0 (Linux; Android 4.0.4; SONY-Q8 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97028[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; U20i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S97029[] = "UCWEB/2.0 (Java; U; MIDP-2.0; id; tcl-c616) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S97030[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; tcl-c616) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S97031[] = "Mozilla/5.0 (Linux; Android 4.0.4; F-05E Build/V13R36C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S97032[] = "Mozilla/5.0 (Linux; Android 4.0.3; F-05D Build/V08R31C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S97033[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-I997 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S97034[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;7 Mozart T8698;7.10;8862)";
const char S97035[] = "Android/1.0 (GT-P1000 FROYO)";
const char S97036[] = "Android/1.0 (GT-P1000 IMM76L)";
const char S97037[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6200 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97038[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6800 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97039[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6800 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97040[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-P6800 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.2.1-65 (Android 4.0.4; Samsung GT-P6800 Build/cm_p6800-userdebug 4.0.4 IMM76L eng.locerra.X.X test-keys)";
const char S97041[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97042[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.3; th-th; GT-P1000T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97043[] = "Android/4.0.4; samsung GT-P7510; 2.3.2";
const char S97044[] = "Rdio.Net; Android/2.5; Android/4.0.4; DM=samsung; DN=GT-P7510; DC=samsung; DF=samsung/GT-P7510/GT-P7510:4.0.4/IMM76D/VLLH3:user/release-keys; DH=p3";
const char S97045[] = "Instagram 3.3.3 Android (13/3.2; 160dpi; 600x976; samsung; GT-P6200; GT-P6200; smdkc210; en_GB)";
const char S97046[] = "Instagram 3.3.3 Android (15/4.0.4; 160dpi; 800x1232; samsung; GT-P6800; GT-P6800; smdk4210; en_US)";
const char S97047[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000";
const char S97048[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.0.6) Gecko/X Ubuntu/8.10 (intrepid) Minefield/3.0 MEGAUPLOAD 2.0";
const char S97049[] = "iBrowser/2.5 (NokiaE65;SymbianOS/9.1 Series60/3.0)";
const char S97050[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6500D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S97051[] = "Mozilla/5.0 (Linux; Android 4.1.1; M-MP875S2 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S97052[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; LG-P700 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S97053[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI-ONE C1 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S97054[] = "Mozilla/5.0 (Linux; Android 4.0.4; SH530U Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S97055[] = "Mozilla/5.0 (Linux; Android 4.0.4; tmn smart a15 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S97056[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:2.0.1) Gecko/X Postbox/2.5.3";
const char S97057[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V889D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S97058[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S97059[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; Lenovo A60+ Build/LenovoLePhone) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S97060[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9050 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S97061[] = "Mozilla/5.0 (Linux; Android 4.0.4; A898 Duo Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S97062[] = "Mozilla/5.0 (Linux; Android 4.0.4; A898 Duo Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97063[] = "Mozilla/5.0 (Linux; Android 4.0.4; PC906 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S97064[] = "Mozilla/5.0 (Linux; Android 4.0.4; shooteru Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97065[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE T22 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97066[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:11.0) Gecko/X Thunderbird/11.0.1";
const char S97067[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MSI Enjoy 10 Plus Build/1.3ST) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97068[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; TM-9737W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97069[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46HX753; CC/GBR; en) Presto/2.10.250 Version/11.60";
const char S97070[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; SAMSUNGSGH-i450) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S97071[] = "Mozilla/5.0 (Linux; Android 4.0.4; Burst S280 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97072[] = "Mozilla/5.0 (Linux; Android 4.0.4; Fly_IQ450 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97073[] = "Mozilla/5.0 (Linux; Android 4.2.1; E2001 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97074[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-T100K Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97075[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-SU640 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97076[] = "JUC (Linux; U; 2.3.6; zh-cn; SHW-M110S; 480*800) UCWEB7.9.0.94/139/444";
const char S97077[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ALCATEL ONE TOUCH 930D Build/1.2.10.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S97078[] = "Mozilla/5.0 (Linux; Android 4.0.4; Oysters T34 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S97079[] = "Mozilla/5.0 (Linux; Android 4.2.2; Optimus G Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S97080[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-CN; HTC Desire Build/FRF91) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S97081[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6201 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S97082[] = "Mozilla/5.0 (Linux; Android 4.0.3; T-909S Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97083[] = "Mozilla/5.0 (Linux; Android 4.0.4; A11 Build/KarbonnA11) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97084[] = "Mozilla/5.0 (Linux; Android 4.1.2; Surfer 8.31 3G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S97085[] = "Mozilla/5.0 (Linux; Android 4.1.1; U30GT-H Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S97086[] = "Mozilla/5.0 (Linux; Android 4.0.4; Spice Mi-530 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S97087[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LG-LU6200 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S97088[] = "Mozilla/5.0 (Linux; Android 4.0.3; Karbonn_A18 Build/KarbonnA18) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S97089[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I699 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S97090[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo P700i Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S97091[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40EX653; CC/GBR; en) Presto/2.10.250 Version/11.60";
const char S97092[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I10-LE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97093[] = "JUC (Linux; U; 2.3.6; zh-cn; Symphony_W10; 320*480) UCWEB7.9.0.94/139/444";
const char S97094[] = "Mozilla/5.0 (Linux; Android 4.1.1; N90DC_S Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S97095[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ADR6425LVW Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97096[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk800i) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S97097[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; SCH-S720C Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S97098[] = "NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac1-02) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S97099[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5753e) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S97100[] = "JUC (Linux; U; 2.3.7; zh-cn; LG-P940; 480*800) UCWEB7.9.0.94/139/444";
const char S97101[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P990 Build/IMM76L)";
const char S97102[] = "SAMSUNG-GT-P3113/100.40101";
const char S97103[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U807 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S97104[] = "Nokia7020/2.0 (08.11) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7020) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S97105[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; HTC_Hero) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S97106[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; P300(Funbook)) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S97107[] = "Mozilla/5.0 (Linux; Android 4.1.2; ADR6425LVW Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97108[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; HTC Wildfire) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S97109[] = "Mozilla/5.0 (Linux; Android 4.0.4; A78 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S97110[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT22i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S97111[] = "Mozilla/5.0 (Linux; Android 2.3.4; LT15i Build/4.0.2.A.0.62) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S97112[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6201 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S97113[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-N7000) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S97114[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97115[] = "Mozilla/5.0 (Linux; Android 4.0.4; SH-09D Build/S4060) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97116[] = "LG-GM730/v10a WM/6.1 IEMobile/6.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S97117[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 97 TITANIUMHD Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S97118[] = "Mozilla/5.0 (Linux; Android 4.0.4; Woxter Tablet PC 97IPS Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S97119[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-sgh-t528g) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S97120[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-c3322) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S97121[] = "Mozilla/5.0 (Linux; Android 4.1.2; ALCATEL ONE TOUCH 8000D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97122[] = "Mozilla/5.0 (Linux; Android 4.1.2; E2001 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97123[] = "Mozilla/5.0 (Linux; Android 4.1.2; GN708W Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97124[] = "Mozilla/5.0 (Linux; Android 3.2.1; G100W Build/HTK55D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S97125[] = "Mozilla/5.0 (Linux; Android 4.0.3; PAP4300 DUO Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S97126[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGP311 Build/10.1.C.0.370) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S97127[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; R801 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S97128[] = "Mozilla/5.0 (Linux; Android 4.0.3; I_7520 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97129[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; TECNO F7 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S97130[] = "Mozilla/5.0 (Linux; Android 4.1.2; B1-710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S97131[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; HTC One X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97132[] = "SonyEricssonC905/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonc905) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S97133[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S97134[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Spice_Mi-435) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S97135[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HTC 802w Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S97136[] = "Mozilla/5.0 (Linux; Android 4.0.3; F-05D Build/V11R40A) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97137[] = "JUC(Linux;U;Android2.3.6;Zh_cn;A1+;320*480;)UCWEB7.8.0.95/139/444";
const char S97138[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad 5210S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97139[] = "Opera/9.80 (X11; FreeBSD 9.1-RELEASE amd64) Presto/2.12.388 Version/12.11";
const char S97140[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Karbonn_A2 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S97141[] = "OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; W100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97142[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lemon P101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97143[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; zh-CN; GT-S5830) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile";
const char S97144[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; LG-P715) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S97145[] = "Mozilla/5.0 (Linux; Android 4.2.1; ZP810 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97146[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;GT-I8350;7.10;8862)";
const char S97147[] = "Mozilla/5.0 (Linux; Android 4.1.1; GSmart Maya M1 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97148[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ZTE U930 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S97149[] = "Mozilla/5.0 (Linux; Android 4.0.4; VL-107L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S97150[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; A27 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S97151[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S97152[] = "Mozilla/5.0 (Linux; Android 4.0.4; WT19a Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97153[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC VLE_U Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S97154[] = "Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB M713G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S97155[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC EVO 3D X515m Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S97156[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI Y300-0000 Build/HuaweiY300-0000) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S97157[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; R815T Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S97158[] = "Mozilla/5.0 (Series40; Nokia2010/11.95; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.36";
const char S97159[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S97160[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-E455 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S97161[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; N320) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S97162[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SPH-L720 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S97163[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; SK17i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S97164[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A11 Build/KarbonnA11) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S97165[] = "Mozilla/5.0 (Linux; Android 4.0.4; Magic Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97166[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6812 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S97167[] = "Mozilla/5.0 (Linux; Android 4.0.4; S22 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97168[] = "Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 3 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97169[] = "Mozilla/5.0 (Linux; Android 4.0.4; L-01E Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97170[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97171[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC6435LVW Build/JDQ39E; Carbon-CARBON-JB-UNOFFICIAL-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97172[] = "Mozilla/5.0 (Linux; Android 4.0.4; N-03E Build/AX) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97173[] = "Mozilla/5.0 (Linux; Android 4.2.2; NookColor Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S97174[] = "Mozilla/5.0 (Linux; Android 4.0.4; A12 Build/RheaTK) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97175[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST17i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97176[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I8150 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97177[] = "Mozilla/5.0 (Linux; Android 4.0.3; IM-A830L Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97178[] = "Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 10 Build/1.1.11-1015 X-10:30) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S97179[] = "Mozilla/5.0 (Linux; Android 4.1.2; Primo-X1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97180[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; LG-VM696 Build/ZV5.GWK74) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S97181[] = "NokiaC5-05/23.5.015 (SymbianOS/9.4; U; Series60/5.0 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Safari/525 3gpp-gba";
const char S97182[] = "Mozilla/5.0 (Linux; Android 4.1.1; ITQ700 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S97183[] = "Mozilla/5.0 (Linux; Android 4.0.4; M-MP860S2 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S97184[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S97185[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-D600 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S97186[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-us; N860 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S97187[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC One X+ Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S97188[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; vi-vn; IM-A820L Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 MoKee OpenSource/MOKEE_4.2.2_A820L_V1/ef65l";
const char S97189[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; PAD720 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97190[] = "Mozilla/5.0 (Linux; Android 4.2.2; Inspire 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97191[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_ChaCha_A810e) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S97192[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI Y300-0000 Build/HuaweiY300-0000) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S97193[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; 8720 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S97194[] = "Mozilla/5.0 (Android; Mobile: SGH-M919; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S97195[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Micromax_A110Q Build/JOP40D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S97196[] = "UCWEB/2.0 (iOS; U; iPh OS 5_0_1; zh-CN; iPh3,1) U2/1.0.0 UCBrowser/9.2.0.312 U2/1.0.0 Mobile";
const char S97197[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Coolpad 5890 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S97198[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo S720 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S97199[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia701) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba";
const char S97200[] = "Mozilla/5.0 (Linux; Android 4.1.2; SM-T210 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S97201[] = "Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-H Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S97202[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI A199 Build/HuaweiA199) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S97203[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:23.0) Gecko/X Firefox/23.0 slurp";
const char S97204[] = "Mozilla/5.0 (Linux; Android 4.1.2; Connect-3G-2.0 Build/HCL) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S97205[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn_A2+ Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S97206[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6812 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S97207[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I927 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97208[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; ALCATEL ONE TOUCH 918N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S97209[] = "Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311";
const char S97210[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I8160) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S97211[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-c3322) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S97212[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; HTC 609d Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97213[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; ZTE-BLADE) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S97214[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E10i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S97215[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN86 8MP) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S97216[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S6012 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S97217[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Q1000 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S97218[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE90) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S97219[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; WT19i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S97220[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5570 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S97221[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI-ONE Plus Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S97222[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; C1604 Build/11.3.A.2.1) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S97223[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; LG-E612 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S97224[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5312 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S97225[] = "Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar_Touch_KEM_452_HDMI Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S97226[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6700s) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S97227[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT30a Build/7.0.A.3.195) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S97228[] = "UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-S6802) U2/1.0.0 UCBrowser/9.3.0.321 Mobile";
const char S97229[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cherry w500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S97230[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-P7510 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.2/p4wifi";
const char S97231[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT26i Build/6.1.A.2.55) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S97232[] = "Mozilla/5.0 (Linux; Android 4.0.4; B92 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S97233[] = "Mozilla/5.0 (Linux; Android 4.1.2; BLU STUDIO 5.3 II Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S97234[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; TouchPad Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.2/tenderloin";
const char S97235[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S97236[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-LS720 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S97237[] = "Mozilla/5.0 (Linux; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S97238[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S97239[] = "LG-TU720/V0.9 Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S97240[] = "BlackBerry8310/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102 UP.Browser/5.0.3.3";
const char S97241[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; GT540R Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S97242[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; LG-GT540R Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 MMS/LG-Android-MMS-V1.0/1.2";
const char S97243[] = "HTC_HD_T8282 Opera/9.7 (Windows NT 5.1; U; en)";
const char S97244[] = "HTC_Touch_HD_T8282 Opera/9.7 (Windows NT 5.1; U; en)";
const char S97245[] = "HTC_Touch_HD_T8282 Opera/9.70 (Windows NT 5.1; U; en)";
const char S97246[] = "SHARP-TQ-GX17/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.3.2.d.1.112 (GUI) MMP/2.0";
const char S97247[] = "SAMSUNG-SGH-E340/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S97248[] = "SAMSUNG-SGH-A717/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S97249[] = "HTC_P3300 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S97250[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC_P3300/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S97251[] = "HTC-8500/1.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8)";
const char S97252[] = "SonyEricssonZ1010/R1K SEMC-Browser/4.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S97253[] = "SonyEricssonZ1010/R1H SEMC-Browser/4.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S97254[] = "SonyEricssonZ1010/R1G SEMC-Browser/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S97255[] = "MOT-W7/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.2 (GUI) MMP/2.0 Push/PO";
const char S97256[] = "KWC-M1000/1.0.07 UP.Browser/6.2.3.9.g.1.110 (GUI) MMP/2.0";
const char S97257[] = "KWC-M1000/1.0.06 UP.Browser/6.2.3.9.g.1.110 (GUI) MMP/2.0";
const char S97258[] = "HTC_S621 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S97259[] = "HTCS621-Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S97260[] = "Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia6650fold/04.18; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S97261[] = "SonyEricssonK320i/R4GC Java/SEMC-Java/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S97262[] = "SAMSUNG-GT-B3310i/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S97263[] = "ZTE-R231/2.0 Profile/MIDP-2.0";
const char S97264[] = "Samsung-SPHM330 Infraware-Polaris/6.1 MMP/2.0";
const char S97265[] = "NokiaN-GageQD/1.0 SymbianOS/6.1 Series60/1.2 Profile/MIDP-1.0 Configuration/CLDC-1.0/UC Browser7.4.0.65/26/351";
const char S97266[] = "SAMSUNG-GT-S7070C/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S97267[] = "KDDI-TS3H UP.Browser/6.2_7.2.7.1.K.1.5.1.120 (GUI) MMP/2.0";
const char S97268[] = "MOT-BLAZE/00.72 UP.Browser/7.2.7.5.678 (GUI) MMP/2.0";
const char S97269[] = "lg-t300";
const char S97270[] = "PT-GF200 CLDC/CLDC-1.0 MIDP/MIDP-1.0";
const char S97271[] = "KDDI-CA38 UP.Browser/6.2.0.13.1.5 (GUI) MMP/2.0";
const char S97272[] = "KDDI-ST33 UP.Browser/6.2.0.13.2 (GUI) MMP/2.0";
const char S97273[] = "Nokia5233/UCWEB8.1.0.104/50/800";
const char S97274[] = "MQQBrowser/2.0 (NokiaE5-00;SymbianOS/9.1 Series60/3.0)";
const char S97275[] = "Mozilla/5.0 (Windows NT 5.0; rv:2.0) Gecko/X Firefox/4.0 CometBird/4.0";
const char S97276[] = "IUC(U;iOS 4.2.1;Zh-cn;320*480;)/UCWEB7.7.1.90/41/997";
const char S97277[] = "IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB7.7.1.90/41/997";
const char S97278[] = "MQQBrowser/2.0 (NokiaC5-03;SymbianOS/9.1 Series60/3.0)";
const char S97279[] = "Mozilla/5.0 (X11; OpenBSD amd64; rv:7.0.1) Gecko/X Firefox/7.0.1";
const char S97280[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:9.0) Gecko/X Thunderbird/9.0.1 OracleBeehiveExtension/1.0.0.4-OracleInternal Lightning/1.1.1";
const char S97281[] = "SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonw20/UC Browser8.0.3.107/69/352";
const char S97282[] = "Reeder/1.5.5 CFNetwork/485.12.7 Darwin/10.4.0";
const char S97283[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:8.0.1) Gecko/X Firefox/8.0.1 SeaMonkey/2.5";
const char S97284[] = "SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonj105i/UC Browser8.0.3.107/69/352";
const char S97285[] = "Facebook/4110.0 CFNetwork/485.10.2 Darwin/10.3.1";
const char S97286[] = "SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonw20/UC Browser7.9.0.102/69/352";
const char S97287[] = "mauie800/UC Browser8.0.3.107/69/444";
const char S97288[] = "Mozilla/5.0 (Windows NT 6.0; rv:7.0) Gecko/X Firefox/7.0 PaleMoon/7.0";
const char S97289[] = "Mozilla/5.0 (X11; SunOS sun4v; rv:5.0) Gecko/X Firefox/5.0";
const char S97290[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 Comodo_Dragon/19.0.3.0";
const char S97291[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 Comodo_Dragon/19.0.3.0";
const char S97292[] = "MQQBrowser/34 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S97293[] = "MQQBrowser/34 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3";
const char S97294[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6.1 FBSMTWB";
const char S97295[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; fi-FI) AppleWebKit/528.16 (KHTML, like Gecko) Version/4.1 Safari/528.16";
const char S97296[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en) AppleWebKit/526.9 (KHTML, like Gecko) Version/4.1 Safari/526.8";
const char S97297[] = "GoogleEarth/5.2.1.1588(Macintosh;Mac OS X (10.6.8);en;kml:2.2;client:Free;type:default)";
const char S97298[] = "nokia701/UC Browser7.9.0.102/69/352  3gpp-gba";
const char S97299[] = "Nokia5610d-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5610d-1/UC Browser7.9.0.102/69/352";
const char S97300[] = "Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:9.1) Gecko/X Firefox/9.1-x64 PaleMoon/9.1-x64";
const char S97301[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC One V Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S97302[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; X10i Build/2.1.A.0.435) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352";
const char S97303[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-tw; MH350 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97304[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; EB-W51GJ Build/2.00a) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97305[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; EB-W51GJ Build/2.01d) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97306[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; zh-tw; MioPad 6 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97307[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; MID1126 Build/CaribbeanSunset_v1.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97308[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MyPad 103 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97309[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; WT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97310[] = "iBrowser/Mini2.8 (Nokia305/03.60)";
const char S97311[] = "Mozilla/5.0 Opera/9.5 (KDDI-TS3V; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00";
const char S97312[] = "Mozilla/5.0 (Linux; Android 4.0.4; Nexus 7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S97313[] = "nokia6120c/UC Browser7.5.1.77/69/352";
const char S97314[] = "SAMSUNG-SGH-i780/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S97315[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.13) Gecko/X Linux Mint/11 (Katya) Firefox/3.6.13 (Swiftfox)";
const char S97316[] = "Mozilla/5.0 Opera/9.5 (KDDI-SH3J; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00";
const char S97317[] = "Mozilla/5.0 (Java; U; en-in; nokian72) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S97318[] = "Mozilla/5.0 (Macintosh; Linux; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S97319[] = "Mozilla/5.0 (Windows NT 6.0; rv:13.0) Gecko/X Firefox/13.0.3.0 IceDragon/13.0.3.0";
const char S97320[] = "NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/70/352";
const char S97321[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; Penta IS701C Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S97322[] = "SAMSUNG-GT-I8350/UCWEB2.5.0.202/47/32079";
const char S97323[] = "iBrowser/2.7 (NokiaC6-01;SymbianOS/9.1 Series60/3.0)";
const char S97324[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA6210 Navigator)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S97325[] = "NokiaC5-03/UC Browser7.0.0.41/50/351";
const char S97326[] = "Mozilla/5.0 (Java; U; en-us; lg-t300) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/405/UCWEB Mobile";
const char S97327[] = "Instagram 3.1.2 (iPad; iPhone OS 5.1.1; en_US) AppleWebKit/420+";
const char S97328[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1b2) Gecko/X Fedora/3.1-0.4.beta2.fc11 Minefield/3.1b2";
const char S97329[] = "JUC(Linux;U;Android2.3.4;Zh_cn;MK16i;480*854;)UCWEB7.8.0.95/139/444";
const char S97330[] = "Mozilla/5.0 (Linux; Android 4.0.4; folio100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S97331[] = "MQQBrowser/3.1 (NokiaC7-00;SymbianOS/9.1 Series60/3.0)";
const char S97332[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.1 Mobile/8C148 Safari/7534.48.3 UCBrowser8.5.0.195/41/355";
const char S97333[] = "Mozilla/5.0 (BB10; Bold) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.0.1337 Mobile Safari/537.10+";
const char S97334[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; A7* Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97335[] = "MQQBrowser/3.1 (NokiaN8-00;SymbianOS/9.1 Series60/3.0)";
const char S97336[] = "JUC (Linux; U; 2.3.6; zh-cn; U8860; 480*854) UCWEB7.9.4.145/139/7675";
const char S97337[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; X500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S97338[] = "Mozilla/5.0 (Linux; Android 4.0.4; GPLUS N930 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97339[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; CELKON CT2 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S97340[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; micromaxq50) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S97341[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Iron/4.0.275.2 Chrome/4.0.275.2 Safari/532.5";
const char S97342[] = "JUC (Linux; U; 4.0.3; zh-cn; PLOYER-MOMO; 800*444) UCWEB1.0.1.113/147/800";
const char S97343[] = "Mozilla/5.0 (Linux; Android 4.2.1; LG-P970 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97344[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/6534.57.2 Sleipnir/4.0.1";
const char S97345[] = "Mozilla/5.0 (Linux; Android 4.1.1; OmegaS200 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97346[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-in; MTABneo Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97347[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; PIRANHA BUSINESS TAB 7\" Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97348[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Micromax P350 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97349[] = "Mozilla/5.0 (Windows NT 6.0; rv:19.0) Gecko/X Thunderbird/19.0";
const char S97350[] = "MQQBrowser/2.7 (Nokia5530;SymbianOS/9.1 Series60/3.0)";
const char S97351[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; EVOTAB FUN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97352[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; MID7012 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S97353[] = "Mozilla/5.0 (Linux; Android 4.0.4; Android T3i Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97354[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; OPPOX907 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97355[] = "Mozilla/5.0 (Linux; Android 4.0.4; N-07D Build/AX) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S97356[] = "Android/3.1; samsung GT-P7500; 2.3.2";
const char S97357[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6200 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S97358[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97359[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97360[] = "JUC(Linux;U;Android4.0.4;Zh_cn;Micromax A110;480*854;)UCWEB7.8.0.95/139/444";
const char S97361[] = "SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; sonyericssonw20i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S97362[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; MB526 Build/4.5.1-134_DFP-231) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S97363[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; E860 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S97364[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; GT-B7510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S97365[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-MS910 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S97366[] = "LG-GW550 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.2.2.51";
const char S97367[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL32EX421; CC/XLA; en) Presto/2.7.61 Version/11.00";
const char S97368[] = "Mozilla/5.0 (Linux; U; Android 3.1; ru-ru; TM-7025 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97369[] = "Nokia2700c-2/2.0 (09.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/70/352";
const char S97370[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-mobile i-style 3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97371[] = "Mozilla/5.0 (X11; FreeBSD amd64; rv:18.0) Gecko/X Firefox/18.0 SeaMonkey/2.15";
const char S97372[] = "Mozilla/5.0 (Linux; Android 4.0.4; RunboX5-W Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97373[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-N8000 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S97374[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; XOLO_A800 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S97375[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A95 Build/CelkonCelkon_A95) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S97376[] = "Mozilla/5.0 (X11; OpenBSD amd64; rv:19.0) Gecko/X Firefox/19.0";
const char S97377[] = "Mozilla/5.0 (Linux; Android 4.0.4; WALKMAN Build/icx1227_E110) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97378[] = "Mozilla/5.0 (Linux; Android 4.0.6; A9550 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97379[] = "Mozilla/5.0 (Linux; Android 4.0.4; SO-02D Build/6.1.F.0.106) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97380[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI 1S Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S97381[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT-AS45IPS Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97382[] = "Mozilla/5.0 (Linux; Android 4.0.4; N-03E Build/AX) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97383[] = "Mozilla/5.0 (Linux; Android 4.0.4; IQ444 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97384[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-gx200) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S97385[] = "Mozilla/5.0 (Linux; Android 4.0.4; Titan TV S320 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97386[] = "Mozilla/5.0 (Linux; Android 4.1.2; N-04E Build/AX) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97387[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC T329t Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S97388[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB860 Build/4.5.1A-129_OLY-145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S97389[] = "Mozilla/5.0 (Linux; Android 4.1.2; One V Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97390[] = "Mozilla/5.0 (Linux; Android 4.0.4; L-01D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97391[] = "Mozilla/5.0 (Linux; Android 4.0.4; Thrill 430x Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97392[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A800S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97393[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-01D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S97394[] = "Mozilla/5.0 (Linux; Android 4.1.1; X507S Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97395[] = "Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_E7310 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S97396[] = "Mozilla/5.0 (Linux; Android 4.1.2; OUYA Console Build/JZO54L-OUYA) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97397[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; R801 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S97398[] = "MQQBrowser/3.4 (NokiaC5-03;SymbianOS/9.1 Series60/3.0)";
const char S97399[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T959 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S97400[] = "Mozilla/5.0 (Linux; Android 4.0.4; Quantum 7 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S97401[] = "MQQBrowser/3.5/Adr (Linux; U; 4.0.4; zh-cn; HUAWEI U9508 Build/U9508V100R001CHNC00B018;720*1280)";
const char S97402[] = "Mozilla/5.0 (Linux; Android 4.0.4; M-MP1010i Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S97403[] = "Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:20.0) Gecko/X Firefox/20.0 slurp";
const char S97404[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26ii Build/6.1.A.2.45) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S97405[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo A789 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S97406[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile i-STYLE Q2 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97407[] = "Mozilla/5.0 (Linux; Android 4.0.4; E330 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97408[] = "JUC(Linux;U;2.3.6;En;GT-S5830i;320*480;)UCWEB7.7.0.85/139/999";
const char S97409[] = "HTC-ST7377/1.59.502.3 (67150) Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S97410[] = "JUC (Linux; U; 4.0.4; zh-cn; GT-I9308; 720*1280) UCWEB7.9.4.145/139/800";
const char S97411[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC T328d Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97412[] = "OneBrowser/3.1 (SAMSUNG GT-C3222)";
const char S97413[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S97414[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F100S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S97415[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; C1604 Build/11.1.A.0.68) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S97416[] = "OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A1+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97417[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST27i Build/6.1.1.B.1.10) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S97418[] = "Nokia5230/UC Browser7.0.0.41/50/351";
const char S97419[] = "MQQBrowser/26 (Linux; U; 2.3.6; zh-cn; ZTE V788D Build/UNI_CN_V788D 2.0;320*480)";
const char S97420[] = "Mozilla/5.0 (Linux; Android 4.0.4; W60 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97421[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Q-Smart S20 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S97422[] = "MQQBrowser/2.8/Adr (Linux; U; 2.3.6; zh-cn; HUAWEI U8661 Build/U8661V100R001C17B827;320*480)";
const char S97423[] = "Mozilla/5.0 (Linux; Android 4.0.4; Aurora-II Build/X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S97424[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cloudfone Thrill 430g Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97425[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:5.0) Gecko/X Firefox/5.0 SeaMonkey/2.2";
const char S97426[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Livingstone 2 Build/1.1.7 X-10:33) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S97427[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A15 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S97428[] = "Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/537.9 (KHTML, like Gecko) Chrome/23.0.1260.0 Mobile Safari/537.9";
const char S97429[] = "JUC (Linux; U; 4.1.2; zh-cn; GT-N7000; 800*1280) UCWEB7.9.0.94/139/444";
const char S97430[] = "Mozilla/5.0 (Linux; Android 4.0.4; Flare (S100) Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97431[] = "Mozilla/5.0 (Linux; Android 4.1.2; JY-G2 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97432[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAN93) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S97433[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; Skate Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97434[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Rhyme_S510b) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S97435[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-N8000 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S97436[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; LG-P920) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S97437[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Intex_Aqua_3.2 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S97438[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI T8833 Build/HuaweiT8833) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S97439[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; C8650 Build/HuaweiC8650) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S97440[] = "Mozilla/5.0 (Linux; Android 4.1.1; T-17B Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S97441[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Sensation XE Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97442[] = "Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 3 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97443[] = "Mozilla/5.0 (Linux; Android 4.1.1; CT920 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S97444[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9505 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S97445[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCEVOV4G Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S97446[] = "Mozilla/5.0 (Linux; Android 4.0.3; E350 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S97447[] = "Mozilla/5.0 (Linux; Android 4.2.2; Transformer TF101 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S97448[] = "Mozilla/5.0 (Linux; Android 4.0.3; F-01D Build/V10R32D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S97449[] = "Mozilla/5.0 (Linux; Android 4.0.3; ISW13F Build/V69R51I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97450[] = "Mozilla/5.0 (Linux; Android 4.0.4; WIKO-CINK SLIM Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S97451[] = "Mozilla/5.0 (Linux; Android 4.0.4; Burst S280 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97452[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-01D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S97453[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaN97";
const char S97454[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P6810 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S97455[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.3.2.289";
const char S97456[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97457[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-T100K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97458[] = "Mozilla/5.0 (Linux; Android 4.0.4; Blaze S180 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97459[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROIDX Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97460[] = "Mozilla/5.0 (Linux; Android 4.0.4; TM400 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97461[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-1 Build/HuaweiU8950-1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S97462[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9210T Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S97463[] = "HTC_TouchDual Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S97464[] = "Mozilla/5.0 (Linux; Android 4.1.2; Titan TV S320 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97465[] = "Mozilla/5.0 (Linux; Android 4.2.2; Huawei MediaPad Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S97466[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E615 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S97467[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGP321 Build/10.3.1.A.0.236) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S97468[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.2.2.259";
const char S97469[] = "UCWEB/2.0 (Linux; U; Adr 2.3.3; zh-CN; C8650) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile";
const char S97470[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MID7032 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444";
const char S97471[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Micromax A87 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.3.2.289";
const char S97472[] = "Mozilla/5.0 (Linux; Android 2.3.7; HTC Desire Build/GRI40) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S97473[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328t Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S97474[] = "Mozilla/5.0 (Linux; Android 2.3.6; SAMSUNG-SGH-I997 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S97475[] = "VPNGate/5.0 (Windows NT 6.1; WOW64; rv:22.0) Gecko/X Firefox/2637";
const char S97476[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.6) Gecko/X  Firefox[xSP_2:1e4a1b03d1b6cd8a174a826f76e009f4_259]  X  (.NET CLR 3.5.30729) FBSMTWB";
const char S97477[] = "SAMSUNG-GT-E3217B/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S97478[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Liberty Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97479[] = "OneBrowser/3.1 (NokiaN79-1/32.001/sw_platform=S60;sw_platform_version=3.2;java_build_version=1.0.4)";
const char S97480[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Iron/9.0.600.2 Chrome/9.0.600.2 Safari/534.13";
const char S97481[] = "JUC (Linux; U; 2.3.6; zh-cn; SCH-I699; 480*800) UCWEB7.9.0.94/139/800";
const char S97482[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A798t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97483[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MOVI M3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97484[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; S930 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97485[] = "UCWEB/2.0 (Java; U; MIDP-2.0; es-LA; samsung-gt-m3710) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S97486[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; G1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97487[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; HUAWEI_U8950-1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S97488[] = "Mozilla/5.0 (Linux; Android 4.0.4; D2-711 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S97489[] = "Opera/9.80 (Android 4.2.2; Linux; Opera Mobi/ADR-X; U; en) Presto/2.9.201 Version/11.50";
const char S97490[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; zh-CN; GT-S5660) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile";
const char S97491[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; zh-CN; XT615) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile";
const char S97492[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GOCLEVER TAB R104 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97493[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717D Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97494[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Aqua Sx Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S97495[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S97496[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; V1 Build/HCL+ME+Tablet+V1) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S97497[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9220 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S97498[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A2+ Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S97499[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:22.0) Gecko/X Firefox/22.0 slurp";
const char S97500[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; ST27i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S97501[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; R801 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S97502[] = "Mozilla/5.0 (Linux; Android 4.0.6; A9550 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97503[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-E435 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S97504[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; LT26i Build/6.2.B.0.200) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S97505[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; iris_401e) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S97506[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S97507[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw150i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S97508[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Lenovo A390_ROW Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/352";
const char S97509[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Coolpad 5890 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S97510[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LG-P765 Build/IMM76L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S97511[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; TECNO Q1 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S97512[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R970 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/jflteusc";
const char S97513[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; Android2.x) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S97514[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae65-1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S97515[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE65) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S97516[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax A91) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S97517[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; HTC Ruby Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S97518[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT28h) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S97519[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; A97i Build/CelkonA97i_Celkon) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S97520[] = "Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB A93.2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S97521[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-N7102 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97522[] = "Mozilla/5.0 (Linux; Android 4.0.4; Wilco Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97523[] = "Mozilla/5.0 (Linux; Android 4.1.2; KM-E100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97524[] = "Mozilla/5.0 (Linux; Android 4.0.3; Aiki-5 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97525[] = "Mozilla/5.0 (Linux; Android 4.0.4; R8111 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97526[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S97527[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233w) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S97528[] = "UCWEB/2.0 (Java; U; MIDP-2.0; vi; gt-s5233w) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S97529[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A760S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97530[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E270L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97531[] = "Mozilla/5.0 (Linux; Android 4.3; XT926 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97532[] = "Mozilla/5.0 (Linux; Android 4.3; SGH-I337M Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97533[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; MB860 Build/4.5.1A-129_OLY-145) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S97534[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T959 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97535[] = "facebookexternalhit/1.0 ( http://www.facebook.com/externalhit_uatext.php)";
const char S97536[] = "Opera/8.01 (J2ME/MIDP; Opera Mini/1.0.1520/HiFi; mobile phone; no; U; ssr)";
const char S97537[] = "Mozilla/5.0 (Java; U; en-us; lg-t325) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S97538[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; zh-CN; ZTE_N880E) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile";
const char S97539[] = "Mozilla/5.0 (Linux; Android 4.0.4; T1-D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97540[] = "Mozilla/5.0 (Linux; Android 4.1.2; MT27i Build/6.2.A.0.400) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97541[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S11t Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S97542[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; ZTE U807 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S97543[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; X720 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97544[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T959V Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97545[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E989 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97546[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/10.53 (Windows NT 5.1; U; pl; compatible; Googlebot/20.2497; U; id) Presto/2.5.25;c2VvLWNyYXdsZXI=";
const char S97547[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9001 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/ariesve";
const char S97548[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; Karbonn_A1) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S97549[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P970 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S97550[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Safari/537.36";
const char S97551[] = "Opera/9.80 (Windows NT 4.0; U; ru) Presto/2.10.229 Version/11.64";
const char S97552[] = "SAMSUNG-GT-C3312R Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10";
const char S97553[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S97554[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E970 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S97555[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S97556[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; MID7048 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S97557[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S97558[] = "Mozilla/5.0 (Linux; Android 4.0.3; P300(Funbook) Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S97559[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-79) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S97560[] = "Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311";
const char S97561[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; HTC 802w Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S97562[] = "Mozilla/5.0 (Linux; Android 4.1.1; TANK 4.5 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97563[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V889D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S97564[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SGH-T989 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S97565[] = "Mozilla/5.0 (Linux; Android 4.1.2; ST27i Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S97566[] = "MQQBrowser/2.4 Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3";
const char S97567[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-us; Micromax A27 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S97568[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT22i) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S97569[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Maxwell 2 Build/1.0.0_X-16:07) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97570[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; N861 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S97571[] = "Mozilla/5.0 (Android; Mobile: DROID RAZR HD; rv:20.0) Gecko/20.0 Firefox/20.0";
const char S97572[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; he-il; Aiki-4S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97573[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S97574[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT-I5801) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S97575[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Touchpad Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97576[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-I9105 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S97577[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua_Glow) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S97578[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-c3303i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S97579[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E16i Build/2.1.1.A.0.53) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S97580[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E467f Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97581[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; th-th; i-mobile IQ 6.1 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97582[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-sgh-a877) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S97583[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P5200 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S97584[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6812 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S97585[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6710s) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S97586[] = "Mozilla/5.0 (Linux; Android 4.3; XT910 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S97587[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Karbonn_A10) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S97588[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A11 Build/KarbonnA11) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S97589[] = "Mozilla/5.0 (Linux; Android 4.1.1; Celkon A119 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S97590[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-7240 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S97591[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT27i Build/6.1.1.B.1.54) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S97592[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; MT27i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S97593[] = "Mozilla/5.0 (Linux; Android 4.3; GT-P7500 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S97594[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Titanium S1 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S97595[] = "Mozilla/5.0 (Linux; Android 4.2.2; SBM206SH Build/S0008) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S97596[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-B5330 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S97597[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST25i Build/6.1.1.B.1.10) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S97598[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S97599[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-US; GT-P1000 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S97600[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I717 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97601[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5670) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S97602[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID1065 Build/ICS.MID1065.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S97603[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Aqua.Active) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S97604[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia700) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S97605[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE T790 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S97606[] = "Mozilla/5.0 (Linux; Android 4.0.4; Nexus S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S97607[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9001 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S97608[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-S6012 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S97609[] = "Mozilla/5.0 (Linux; Android 4.1.1; O+ 8.6 Android Build/Oplus8.6_Android) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S97610[] = "Nokia112/2.0 (03.32) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia112) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S97611[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-I739 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S97612[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax A91 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S97613[] = "Mozilla/5.0 (Linux; Android 4.2.2; Sensation XL Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S97614[] = "Mozilla/5.0 (Linux; Android 4.0.3; Aiki-5 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S97615[] = "Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/11A465 Safari/8536.25";
const char S97616[] = "NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; NokiaC2-03) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S97617[] = "Mozilla/5.0 (Linux; Android 4.2.1; IdeaTab S6000-F Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S97618[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E430 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S97619[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S97620[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S97621[] = "Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0151 Build/HuaweiY300-0151) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S97622[] = "Mozilla/5.0 (Linux; Android 4.1.1; E270BSA Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S97623[] = "SAMSUNG-SGH-X497/X497UCEG3 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S97624[] = "Mozilla/5.0 (compatible;MSIE 6.0;Linux MOTOROKR Z6W-orange) MOT-Z6w/R6635_G_81.03.2BR Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony 1.0";
const char S97625[] = "Mozilla/5.0 (compatible;MSIE 6.0;Linux MOTOROKR Z6W-orange) MOT-Z6w/R6635_G_81.01.61R Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony 1.0";
const char S97626[] = "MOT-MOTOROKR-Z6W/Mozilla/5.0 (compatible;MSIE 6.0;Linux MOTOROKR Z6W)/R6635_G_81.xx.yyI Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony 1.0";
const char S97627[] = "MOT-Z6w/R6635_G_81.01.64R Mozilla/5.0 (compatible; MSIE 6.0; Linux; MOTOROKR Z6W) Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony/1.0";
const char S97628[] = "CORAL 400/WAP2.0";
const char S97629[] = "MOT-ZN200/1.0 Release/09.01.2008 Browser/CMCS1.0 Software/0.34D Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S97630[] = "LG-KP135 Obigo/WAP2.0";
const char S97631[] = "SAMSUNG-GT-C3060/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S97632[] = "LG-KF390 Teleca/Q05A-3.12 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S97633[] = "SEC-SGHU300/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S97634[] = "MOT-W360/1.0 Release/03.06.2008 Browser/CMCS1.0 Software/1.180 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S97635[] = "SonyEricssonC510/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S97636[] = "SonyEricssonC510a/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S97637[] = "Sonim XP3300";
const char S97638[] = "HTCP3300-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)";
const char S97639[] = "HTC_P3300-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)";
const char S97640[] = "HTC P3300/1.35.621.1 Mozilla/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)";
const char S97641[] = "NexianNX-W750/MTK Release/01.01.2010 Browser/MAUI Profile/MIDP2.0 Configuration/CLCD-1";
const char S97642[] = "HTC_S620 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S97643[] = "LG9700/1.1 TELECA/Q7.0-1.3 MMP/2.0";
const char S97644[] = "LG9700/1.0 TELECA/Q7.0-1.3 MMP/2.0";
const char S97645[] = "BlackBerry7290/4.0.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S97646[] = "MQQBrowser/20 Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Lenovo A60 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97647[] = "SAMSUNG-GT-E2120/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S97648[] = "SEC-SGHE720C/1.0 TSS/2.5";
const char S97649[] = "SEC-SGHE720/1.0 TSS/2.5";
const char S97650[] = "ZTE-T165i/1.0 ACS-NF/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S97651[] = "SAMSUNG-GT-M7600B/1.0 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S97652[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; Xda Exec; 640x480)";
const char S97653[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC;JASJAR;640x480; PPC; Xda Exec; 640x480)";
const char S97654[] = "Nokia2692_CMCC/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S97655[] = "SAMSUNG-GT-B5722C/1.0 RTK-E/1.0 DF/1.0 Release/10.14.2009 Browser/NetFront3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S97656[] = "SFR115/g9qA/EFS-SFR-R9SM-P109A17V1.0.4/R221 Obigo/Q03C WAP/2.0";
const char S97657[] = "Dopod818Pro/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; Dopod818 Pro)";
const char S97658[] = "Dopod 818 Pro Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; Dopod 818 Pro)";
const char S97659[] = "Mot-WX280/1.0 ObigoInternetBrowser/Q03C";
const char S97660[] = "SCH-F309/ UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S97661[] = "NOKIAE5-00/UC Browser7.4.0.65/28/351";
const char S97662[] = "Mozilla/4.0 (compatible; MSIE 4.0; SmartPhone; Symbian OS/1.1.0) NetFront/3.1";
const char S97663[] = "KDDI-KC3S UP.Browser/6.2.0.15.1.1 (GUI) MMP/2.0";
const char S97664[] = "TIANYU-KTOUCH/B2010/Screen-128X128";
const char S97665[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.0.3.0 Chrome/17.0.963.38 Safari/535.11";
const char S97666[] = "SonyEricssonU1/UC Browser8.0.4.121/50/444/UCWEB";
const char S97667[] = "BlackBerry9650/4.4.0.560 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/159/352";
const char S97668[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Slider SL101 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S97669[] = "Nokia3120classic/2.0 (07.16) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia3120classic/UC Browser7.9.0.102/69/352";
const char S97670[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.140 Safari/535.19 Comodo_Dragon/18.0.3.0";
const char S97671[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.140 Safari/535.19 Comodo_Dragon/18.0.3.0";
const char S97672[] = "NokiaC6-01/UC Browser7.6.1.82/50/351";
const char S97673[] = "nokiac6-01/UC Browser7.6.1.82/69/351  3gpp-gba";
const char S97674[] = "MacOutlook/14.13.0.X (Intel Mac OS X 10.6.8)";
const char S97675[] = "HTC_P4350 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S97676[] = "Nokia6500s-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6500s-1/UC Browser8.0.3.107/69/352";
const char S97677[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S97678[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75 Mobile Safari/535.7";
const char S97679[] = "MQQBrowser/30 (iOS; U; CPU like Mac OS X; zh-cn)";
const char S97680[] = "samsung-gt-s5253/UC Browser7.9.0.102/69/444";
const char S97681[] = "Opera/9.23 (Nintendo Wii; U; ; 1038-58; Wii Internet Channel/1.0; en)";
const char S97682[] = "Mozilla/5.0 (Linux; U; Android 3.2; tr-tr; chagall Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S97683[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.11.12388.201";
const char S97684[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/4.0; QQDownload 677; qdesk 2.3.1186.202; EmbeddedWB 14.52 from: http://www.bsalsa.com/ EmbeddedWB 14.52; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Alexa Toolbar) QQBrowser/6.11.12388.201";
const char S97685[] = "Mozilla/5.0 (Windows NT 5.0; rv:7.0.1) Gecko/X Thunderbird/7.0.1";
const char S97686[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I500 Build/MIUI ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97687[] = "JUC(Linux;U;2.3.3;Zh_cn;HTC A510c;320*480;)UCWEB7.6.0.75/139/32125";
const char S97688[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; tegav2 Build/HLK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S97689[] = "Mozilla/5.0 (Linux; U; Android 3.2; hd-us; tegav2 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S97690[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:9.0.1) Gecko/X Firefox/9.0.1";
const char S97691[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT894 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97692[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 Sleipnir/3.5.1";
const char S97693[] = "Mozilla/5.0 (Linux; Android 4.0.4; Nexus One Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97694[] = "Mozilla/5.0 (Android; Linux armv7l; rv:2.0) Gecko/X Firefox/4.0 Fennec/4.0";
const char S97695[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; in-id; G2 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97696[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SK17i Build/4.0.2.A.0.42; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97697[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97698[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; A73 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S97699[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; CoolPad8013 Build/2.2.060.P1.X.8013) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S97700[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; CoolPad8013 Build/2.2.062.P1.X.8013) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S97701[] = "Mozilla/5.0 (X11; OpenBSD amd64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S97702[] = "Mozilla/5.0 (X11; OpenBSD i386) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7";
const char S97703[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; Nexus 7 Build/JRO03D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S97704[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; Nexus 7 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S97705[] = "Mozilla/5.0 (Windows; U; WinNT4.0; en-US; rv:1.4) Gecko/X Netscape/7.1 (ax)";
const char S97706[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.7.5) Gecko/X Netscape/8.0.1";
const char S97707[] = "nokian73-1/UC Browser7.5.1.77/69/352";
const char S97708[] = "nokiae52-1/UC Browser7.9.0.102/69/352  3gpp-gba";
const char S97709[] = "MQQBrowser/26 (Linux; U; 2.3.5; zh-cn; Lenovo A65 Build/A65_S218_X;320*480)";
const char S97710[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; de-AT; rv:1.9.1.8) Gecko/X Firefox/3.6.6";
const char S97711[] = "JUC(Linux; U;2.3.4;zh_cn;S8600;480*800;)UCWEB7.6.0.75/139/7713";
const char S97712[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97713[] = "MQQBrowser/3.1 (NokiaC5-03;SymbianOS/9.1 Series60/3.0)";
const char S97714[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S97715[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax P250(Funbook) Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S97716[] = "JUC(Linux;U;Android2.3.4;Zh_cn;A75;320*480;)UCWEB7.8.0.95/139/444";
const char S97717[] = "sonyericssons312/UC Browser7.9.0.102/69/352";
const char S97718[] = "SAMSUNG-SAMSUNG-SGH-I777/100.20306";
const char S97719[] = "MQQBrowser/2.8 (Nokia5250;SymbianOS/9.1 Series60/3.0)";
const char S97720[] = "Mozilla/5.0 (Linux; Android 4.0.3; Polaroid Tablet Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S97721[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; M3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97722[] = "Instagram 3.1.2 (iPhone; iPhone OS 5.0.1; en_US) AppleWebKit/420+";
const char S97723[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S97724[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-b7722) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S97725[] = "Nokia6500s-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia6500s-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S97726[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL55HX729; CC/USA; en) Presto/2.7.61 Version/11.00";
const char S97727[] = "samsung-gt-s5380k/UC Browser7.7.1.88/69/444";
const char S97728[] = "IUC(U;iOS 6.0.1;Zh-cn;320*480;)/UCWEB8.3.1.151/42/997";
const char S97729[] = "Mozilla/5.0 (Windows NT 6.0; rv:18.0) Gecko/X Thunderbird/18.0";
const char S97730[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; GT-B7510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S97731[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/6.0.2 Safari/7536.26.17 Sleipnir/3.9";
const char S97732[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; N860 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S97733[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.6 Mobile Safari/535.19 Silk-Accelerated=false";
const char S97734[] = "Mozilla-Firefox5.0 AppEngine-Google; (+http://code.google.com/appengine; appid: s~zhoulixia86)";
const char S97735[] = "OneBrowser/3.0 (NokiaC2-00/03.45)";
const char S97736[] = "MQQBrowser/2.8/Adr (Linux; U; 2.3.6; zh-cn; ZTE V889D Build/UNI_CN_V889D 2.2;480*800)";
const char S97737[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; en) Presto/2.9.168 Version/11.52";
const char S97738[] = "Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB M713G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S97739[] = "NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; kau; nokiax2-05) UCBrowser8.4.0.159/70/352/UCWEB Mobile";
const char S97740[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97741[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax P275 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S97742[] = "SAMSUNG-SGH-T769/100.20306";
const char S97743[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHL21 Build/S2140) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S97744[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; GT-P7500 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S97745[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S97746[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/R13.4-rc2) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S97747[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500R Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S97748[] = "YahooMobileMail/1.0 (Android Mail; 1.4.5) (GT-P1000N;samsung;GT-P1000N;2.2.1/FROYO)";
const char S97749[] = "YahooMobileMail/1.0 (Android Mail; 1.4.6) (GT-P1000;samsung;GT-P1000;4.1.1/JRO03R)";
const char S97750[] = "WhatsApp/2.8.9245 Android/2.2 Device/samsung-GT-P1000";
const char S97751[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-ch; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P5110, 36, 1.7.1)";
const char S97752[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P5110, 42, 1.9.1)";
const char S97753[] = "Instagram 3.3.2 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P3100; espressorf; espresso; en_GB)";
const char S97754[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-CN; GT-P1000 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S97755[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-CN; GT-P1000 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S97756[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.8.1.6) Gecko/X Firefox/2.0.0.6 Flock/0.9.0.2";
const char S97757[] = "Mozilla/5.0 (Linux; Android 4.0.4; Consul 7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S97758[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S97759[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssons312) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S97760[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A7+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S97761[] = "Mozilla/5.0 (Java; U; en-us; nokiac6-00) UCBrowser8.3.1.161/70/465/UCWEB Mobile  3gpp-gba";
const char S97762[] = "Mozilla/5.0 (Linux; Android 4.0.4; TM-4504 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97763[] = "Mozilla/5.0 (Linux; Android 4.0.4; R97B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S97764[] = "Opera/9.80 (Windows 98) Presto/2.12.388 Version/12.10";
const char S97765[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97766[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-N7000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97767[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE-N880E Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97768[] = "SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonw20) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S97769[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ZTE U930 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S97770[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; GT-I5801 Build/ECLAIR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S97771[] = "Mozilla/5.0 (Linux; Android 4.0.3; GEM10312BK Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S97772[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 1 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97773[] = "Mozilla/5.0 (iPad; U; CPU OS 5_1_1 like Mac OS X; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile";
const char S97774[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, +0100, gb";
const char S97775[] = "NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-05/UC Browser8.0.3.107/69/444";
const char S97776[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/7534.48.3 TeamStream (iPod2,1 232 1.6.2)";
const char S97777[] = "Mozilla/5.0 (Linux; Android 4.1.2; G2S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97778[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S97779[] = "Mozilla/5.0 (Linux; Android 4.0.4; Blaze S180 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S97780[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire Build/CM10) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S97781[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A850K Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97782[] = "Mozilla/5.0 (Linux; Android 4.0.4; N-07D Build/AX) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97783[] = "Mozilla/5.0 (Linux; Android 4.0.4; GN800 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97784[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARNOVA 90G3 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S97785[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT25c Build/9.0.E.1.80) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97786[] = "Mozilla/5.0 (Linux; Android 4.1.1; I7100 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97787[] = "Mozilla/5.0 (Linux; Android 4.1.2; E2001 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97788[] = "Mozilla/5.0 (Linux; Android 4.0.4; FJL21 Build/V35R45B) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97789[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us ; PD_Novel Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S97790[] = "Mozilla/5.0 (Linux; Android 4.1.1; M1 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97791[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Incredible S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S97792[] = "Mozilla/5.0 (Linux; Android 4.1.1; Xelio 7 pro Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S97793[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZP810 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97794[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-S6102 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Mobile Safari/537.31 OPR/14.0.1074.54070";
const char S97795[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97796[] = "MQQBrowser/3.7/ZTEU795_TD/1.0 Linux/2.6.39 Android/4.0 Release/6.10.2012 Browser/AppleWebKit534.30";
const char S97797[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; MT15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S97798[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-CN; GT-S5830 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S97799[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S97800[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S97801[] = "Mozilla/5.0 (Linux; Android 4.0.4; CAL21 Build/AX) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97802[] = "JUC (Linux; U; 2.3.4; zh-cn; HTC_S610d; 480*800) UCWEB7.9.4.145/139/32449";
const char S97803[] = "Mozilla/5.0 (Linux; Android 4.0.3; M032 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S97804[] = "Mozilla/5.0 (Linux; Android 4.2.1; ZP800H Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97805[] = "LG-CT810/V10x IEMobile/7.11 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S97806[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.3.4641 Chrome/7.0.517.450 Safari/534.7";
const char S97807[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE N880E Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S97808[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; A97i Build/CelkonA97i_Celkon) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S97809[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.19/284,1.9.19.284,2013-01-12 05:21, +0530, in";
const char S97810[] = "UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; HTC_Desire) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S97811[] = "SonyEricssonW890i/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw890i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S97812[] = "Mozilla/5.0 (Linux; Android 4.0.4; N-08D Build/AX) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S97813[] = "MQQBrowser/421 Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3";
const char S97814[] = "SAMSUNG-GT-B6520/1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S97815[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GSmart G1362 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97816[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; eZee'Tab704 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97817[] = "MQQBrowser/3.0/Adr (Linux; U; 4.0.4; zh-cn; GN700W Build/GN700W_T16;480*800)";
const char S97818[] = "JUC (Linux; U; 2.3.6; zh-cn; A7+; 320*480) UCWEB7.9.0.94/139/444";
const char S97819[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; tr-tr; SmartTab10 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97820[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F120S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97821[] = "Mozilla/5.0 (Linux; Android 4.0.3; TM-9741 Build/V1.1.ICECREAMSANDWICH.rus.teXet.X.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S97822[] = "Mozilla/5.0 (Linux; Android 4.0.4; Q-Smart S20 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S97823[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; T8830Pro Build/HuaweiT8830Pro) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S97824[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S97825[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A818_Duo) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S97826[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-CN; MT15i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S97827[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-T100K Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S97828[] = "Mozilla/5.0 (Linux; Android 4.1.1; M030 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97829[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HTC Sensation XL with Beats Audio X315e Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S97830[] = "OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S97831[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A298t Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S97832[] = "Mozilla/5.0 (Linux; Android 4.1.1; X507 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S97833[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S97834[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S97835[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9000 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S97836[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:21.0) Gecko/X Firefox/21.0 slurp";
const char S97837[] = "Mozilla/5.0 (Linux; Android 4.0.3; TAB466EUK Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S97838[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE BLADE III Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S97839[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; GT-P3113 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S97840[] = "Mozilla/5.0 (Linux; Android 4.0.4; Oysters T34 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S97841[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAP4044DUO Build/PrestigioPAP4044DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97842[] = "Mozilla/5.0 (Linux; Android 4.0.4; F7HD Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S97843[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; OV-BaseCore7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97844[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; LG-P970) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S97845[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; LT22i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S97846[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6810) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S97847[] = "Mozilla/5.0 (Linux; Android 4.1.1; O+ 8.6 Android Build/Oplus8.6_Android) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97848[] = "Mozilla/5.0 (Linux; Android 4.0.4; MIDC901 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97849[] = "Mozilla/5.0 (Linux; Android 4.0.3; A90 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97850[] = "Mozilla/5.0 (Linux; Android 4.2.2; M9 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S97851[] = "Mozilla/5.0 (Linux; Android 4.2.1; i-mobile IQ X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97852[] = "Mozilla/5.0 (Linux; Android 4.1.1; Orange Kivo Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97853[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I8190 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97854[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-V909 Build/HMJ37; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97855[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3 TeamStream (iPhone3,1 293 2.2.2)";
const char S97856[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PIRANHA ULTRA II TAB 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97857[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HTC Rhyme S510b Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S97858[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN91) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S97859[] = "Mozilla/5.0 (Linux; Android 2.3.7; LT22i Build/6.0.B.1.564) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S97860[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; LT18i) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S97861[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC 802t Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97862[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; HTC One X) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S97863[] = "Mozilla/5.0 (Linux; Android 4.1.1; IRIS 455 Build/LAVAIRIS455) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97864[] = "Mozilla/5.0 (Linux; Android 4.1.2; New Andromax-i Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97865[] = "Mozilla/5.0 (Linux; Android 4.1.2; Andromax U Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S97866[] = "Mozilla/5.0 (DTV) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.1.5 AQUOSBrowser/1.0 (AS00DTV;V;0001;0001)AQUOS-AS/2.0 LC-70LE950X";
const char S97867[] = "Mozilla/5.0 (DTV) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.5 AQUOSBrowser/1.0 (AS00DTV;V;0001;0001) AQUOS-AS/2.0 LC-80LE840X";
const char S97868[] = "Mozilla/5.0 (DTV) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.4 AQUOSBrowser/1.0 (JP00DTV;B;0001;0001) InettvBrowser/2.2 (08001F;DTV10VSFC3;0001;0001)";
const char S97869[] = "SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; fa; sonyericssonw20i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile";
const char S97870[] = "Mozilla/5.0 (Linux; Android 4.0.4; M2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S97871[] = "OneBrowser/3.1 (SAMSUNG GT-C3510)";
const char S97872[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax A110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S97873[] = "Nokia205/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia205) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S97874[] = "NetFront/4.2 (BREW 5.0.1.2; U; en-us; SONIM; NetFront/4.2/AMB) Sprint XP3410 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S97875[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MITO A15 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97876[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; Lenovo P770 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/352";
const char S97877[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SAMSUNG-SGH-I727, 42, 1.9.1)";
const char S97878[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Incredible S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S97879[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN80) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S97880[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6312 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S97881[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-B5512) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S97882[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Celkon*A86) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S97883[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Gpad_G1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S97884[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S5303 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S97885[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Y3 Build/HCL) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S97886[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian8-00) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile  3gpp-gba";
const char S97887[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XOLO A700 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S97888[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; de-de; Huawei MediaPad Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97889[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; iris454) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S97890[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; iris454 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S97891[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; GT-I5801 Build/ECLAIR) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S97892[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-S7572 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S97893[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-MS870 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S97894[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-CN; GT-I9008L Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S97895[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S97896[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328d Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S97897[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:24.0) Gecko/X Firefox/24.0";
const char S97898[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8833 Build/HuaweiT8833) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S97899[] = "Mozilla/5.0 (Linux; Android 2.3.4; myTouch_4G_Slide Build/GRJ22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S97900[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Event Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S97901[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID8042 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S97902[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Rhyme S510b Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S97903[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S97904[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9050 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S97905[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Y3 Build/HCL) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S97906[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; vivo X1 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S97907[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MK16i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S97908[] = "Mozilla/5.0 (Linux; Android 4.2.1; i-mobile IQ 5.3 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97909[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; WT19i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S97910[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LG-E615 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S97911[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; LG-E612) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S97912[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-I9103 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S97913[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; VT75C Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S97914[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; LT29i Build/9.1.B.1.67) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S97915[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; IRIS_501) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S97916[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; A54 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S97917[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; SCH-i929 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S97918[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-S7572 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S97919[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-US; GT-P6200 Build/HTJ85B) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S97920[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-B5512 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S97921[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M031 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S97922[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S97923[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; Micromax A60) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S97924[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-P715 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S97925[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT25c Build/9.0.E.1.80) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97926[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-T100K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97927[] = "Mozilla/5.0 (Linux; Android 4.2.2; MB865 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97928[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Iris_349 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S97929[] = "Mozilla/5.0 (Linux; Android 4.0.4; i9430 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97930[] = "Mozilla/5.0 (Linux; Android 4.1.1; Orange Zali Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97931[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 slurp";
const char S97932[] = "Mozilla/5.0 (Windows; U; WinNT4.0; en-US; rv:0.9.4.1) Gecko/X Netscape6/6.2.2";
const char S97933[] = "Mozilla/5.0 (Linux; Android 4.0.4; MIDC801 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S97934[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A89 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S97935[] = "IUC(U;iOS 6.1.3;Zh-cn;320*480;)/UCWEB8.2.1.132/42/997";
const char S97936[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 XiaoMi/MiuiBrowser/1.0";
const char S97937[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; GOCLEVER HYBRID Build/GOCLEVER HYBRID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S97938[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cloudfone Excite 500d Build/FSV08) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97939[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; SCH-i929 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S97940[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A9+ Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300";
const char S97941[] = "Mozilla/5.0 (Linux; Android 4.1.1; myPad P4 Lite Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S97942[] = "Mozilla/5.0 (Linux; Android 4.0.4; iris454 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97943[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-CN; R815T Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S97944[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire 600 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97945[] = "NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291";
const char S97946[] = "Mozilla/5.0 (Linux; Android 2.3.5; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60015";
const char S97947[] = "MQQBrowser/3.6 (Nokia5233;SymbianOS/9.1 Series60/3.0)";
const char S97948[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_One_V) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S97949[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-D700 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97950[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; HTC Desire X) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S97951[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Droid RAZR MAXX HD Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97952[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9268 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S97953[] = "Mozilla/5.0 (Linux; Android 4.1.1; Arc Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S97954[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9400 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S97955[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PantechP8010 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S97956[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S97957[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-in; A52 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S97958[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; LG-LS970 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S97959[] = "Mozilla/5.0 (Linux; Android 4.1.2; Spice Mi-515 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S97960[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328d Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S97961[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HTC 802w Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S97962[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-E435 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S97963[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E980 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S97964[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M032 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S97965[] = "Mozilla/5.0 (Linux; Android 4.1.1; ZTE V807 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S97966[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Orbit TAB Phone 9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97967[] = "UCWEB/2.0 (Linux; U; Adr 4.0.1; en-US; AMAZE_S200) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S97968[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-in; Micromax A116 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300";
const char S97969[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_Desire_V) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S97970[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI-ONE C1 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S97971[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9128 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S97972[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T769 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S97973[] = "Nokia110/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311";
const char S97974[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-P705 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S97975[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Karbonn A26 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S97976[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; SOWA_S6600 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S97977[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9502 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S97978[] = "Mozilla/5.0 (Linux; Android 4.0.4; TM500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S97979[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-in; Micromax A87 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S97980[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-in; Micromax A115 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S97981[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-us; LG-L38C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S97982[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; HTC Vision Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S97983[] = "Mozilla/5.0 (Linux; Android 4.2.1; iDxD10 3G Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S97984[] = "UCWEB/2.0 (Linux; U; Adr 2.2; en-US; GT-P1000) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S97985[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALLVIEW_TX1_Quasar Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S97986[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT926 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/xt926";
const char S97987[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Karbonn_A4_) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S97988[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA5630d-1) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S97989[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk770i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S97990[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Spice_Mi-502) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S97991[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S97992[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E15i Build/2.1.1.A.0.6) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S97993[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Micromax_A110Q Build/JOP40D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S97994[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; Nokia6220c-1) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S97995[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; TECNO_N7 Build/TECNOTECNO_N7) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S97996[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Sensation Z710e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S97997[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A30 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S97998[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SGH-M919 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S97999[] = "Mozilla/5.0 (Linux; Android 4.2.1; mobiistar touch LAI 504 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S98000[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; Micromax A76 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S98001[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; A111) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S98002[] = "Mozilla/5.0 (Linux; Android 4.0.4; SC-01D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S98003[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T869 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S98004[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; XOLO A700 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S98005[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; LS670 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S98006[] = "Mozilla/5.0 (Linux; Android 4.1.2; Primo-N1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98007[] = "Mozilla/5.0 (Linux; Android 4.1.1; NovoCrystal Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S98008[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A65) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S98009[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; N320) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S98010[] = "UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; A210) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S98011[] = "Mozilla/5.0 (Linux; Android 4.0.4; GN800 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S98012[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S98013[] = "UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Micromax_A116) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S98014[] = "Mozilla/5.0 (Linux; Android 4.0.4; TECNO_N7 Build/TECNOTECNO_N7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S98015[] = "Mozilla/5.0 (Linux; Android 4.0.4; WALKMAN Build/icx1227_E111) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S98016[] = "Mozilla/5.0 (Linux; Android 4.1.2; L-04E Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S98017[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I897 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98018[] = "Mozilla/5.0 (Linux; Android 4.0.3; M729 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S98019[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V70 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S98020[] = "Mozilla/5.0 (Linux; Android 4.2.1; B6000 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S98021[] = "Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 97 TITANIUMHD Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S98022[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S98023[] = "Mozilla/5.0 (Linux; Android 4.1.1; Micromax A110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S98024[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S98025[] = "UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-I8552) U2/1.0.0 UCBrowser/9.3.0.321 Mobile";
const char S98026[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 UCBrowser/9.3.0.326 Mobile";
const char S98027[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S98028[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26w Build/6.2.B.1.96) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S98029[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC One V Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S98030[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire C) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S98031[] = "Mozilla/5.0 (Linux; Android 4.2.1; ME173X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S98032[] = "Mozilla/5.0 (Linux; Android 4.1.2; C525c Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S98033[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S98034[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S98035[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8010 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S98036[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S98037[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S98038[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5215 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S98039[] = "SAMSUNG-SGH-X495/X495UVEL1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S98040[] = "SEC-SGHE770/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S98041[] = "SAMSUNG-SGH-J770/J770AHHI2 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S98042[] = "Philips X116";
const char S98043[] = "Alcatel-OT-303A/1.0 ObigoInternetBrowser/Q03C";
const char S98044[] = "Alcatel-OT-303/1.0 ObigoInternetBrowser/Q03C";
const char S98045[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) i-mate 8502; 240x320";
const char S98046[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) i-mate 8502/ 240x320";
const char S98047[] = "SEC-SGHE530/1.0 TSS/2.5";
const char S98048[] = "SEC-SGHE530C/1.0 TSS/2.5";
const char S98049[] = "LG-KP275 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S98050[] = "SonyEricssonJ10i2/R7CA";
const char S98051[] = "SonyEricssonJ10i/R7CA Browser/NetFront Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2";
const char S98052[] = "SAMSUNG-SGH-Z720/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S98053[] = "SAMSUNG-SGH-Z720-Vodafone/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S98054[] = "SonyEricssonF500i/R2E SEMC-Browser/4.0 Profile/MIDP-1.0 MIDP-2.0 Configuration/CLDC-1.1";
const char S98055[] = "SonyEricssonF500i/R2AA SEMC-Browser/4.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S98056[] = "SonyEricssonF500i/R2N SEMC-Browser/4.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S98057[] = "SonyEricssonF500i/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S98058[] = "SAMSUNG-SGH-X670/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S98059[] = "LG-KU580-Orange/v10a Browser/Obigo-Q05A1/2.16 MMS/LG-MMS-V1.0/1.2 Java/LGVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S98060[] = "LG-KU580 Browser/Obigo-Q05A1/2.16 MMS/LG-MMS-V1.0/1.2 Java/LGVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Ext-V-Profile/VSCL-2.0.0";
const char S98061[] = "Vodafone/1.0/LG-KU580/V10a Browser/Obigo-Q05A1/2.16 MMS/LG-MMS-V1.0/1.2 Java/LGVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Ext-V-Profile/VSCL-2.0.0";
const char S98062[] = "LG/KU580/V10a Browser/Obigo-Q05A1/2.16 MMS/LG-MMS-V1.0/1.2 Java/LGVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Ext-V-Profile/VSCL-2.0.0";
const char S98063[] = "acer_S200 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5)";
const char S98064[] = "acer_S200 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5.3.5)";
const char S98065[] = "Mozilla/SMB3(Z105)/Samsung";
const char S98066[] = "SEC-SGHB500/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S98067[] = "Spice M5750";
const char S98068[] = "Mozilla/4.0 (BREW 3.1.5; U; en-us; Sanyo; NetFront/3.5.1/AMB) PL3810";
const char S98069[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Vodafone/1.0/HTC_v1415/1.18.161.3";
const char S98070[] = "Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303iclassic/UC Browser7.7.1.88/69/444";
const char S98071[] = "NokiaX2-00/5.0 (04.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-00/UC Browser7.8.0.95/70/355";
const char S98072[] = "SAMSUNG-GT-C3222/1.0 RTK-E/1.0 DF/1.0 Release/11.12.2010 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S98073[] = "Fly_E135/MAUIBrowser/Configuration/WAP2.0";
const char S98074[] = "SAMSUNG-GT-S3930C_CMCC/1.0 RTK-E/1.0 SHP/VPP/R5 Release/17.08.2008 Browser/NetFront3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S98075[] = "Mozilla/5.0 (Linux; U; Android 1.6; in-id; Nexian NX-A890 Build/DONUT) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S98076[] = "NokiaX6-00/UC Browser7.4.0.65/50/355";
const char S98077[] = "SAMSUNG-SGH-C165/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S98078[] = "SEC-SGHA501 SHP/CDR-2.2 NetFront/3.2 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S98079[] = "SAMSUNG-SGH-E380/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S98080[] = "LG-KF311/V10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S98081[] = "MOT-VU204/00.62 UP.Browser/6.2.3.4.c.1.128 (GUI) MMP/2.0";
const char S98082[] = "NokiaC5-03/UC Browser7.6.1.82/50/351";
const char S98083[] = "Nokia5235/UC Browser7.7.1.88/50/444";
const char S98084[] = "Nokia7610/2.0 (6.0522.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.8.0.95/27/351";
const char S98085[] = "KDDI-ST34 UP.Browser/6.2.0.13.2 (GUI) MMP/2.0";
const char S98086[] = "ZTE-C78/1.0 SMIT-Browser/2.0.0";
const char S98087[] = "Flipboard/1.5.3 CFNetwork/467.12 Darwin/10.3.1";
const char S98088[] = "SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonj108i/UC Browser7.7.1.88/69/444";
const char S98089[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Flock/3.0.6.4257 Chrome/5.0.375.1281 Safari/533.4";
const char S98090[] = "Mozilla/5.0 (Linux; U; Android 1.6; zh-cn; 3GW100 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Lenovo-3GW100/1.0.0";
const char S98091[] = "Mozilla/5.0 (Linux; U; Android 1.6; zh-cn; qsd8250_surf Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Lenovo-3GW100/1.0.0";
const char S98092[] = "Mozilla/5.0 (Linux; U; Android 1.6; zh-cn; Lenovo 3GW100 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S98093[] = "MobileSafari/525.20 CFNetwork/339.3 Darwin/9.4.1";
const char S98094[] = "gt-s5233a/UC Browser7.5.1.77/69/352";
const char S98095[] = "gt-s5233a/UC Browser7.5.1.77/69/351";
const char S98096[] = "MOT-A-B2/02.01 UP.Browser/7.2.7.1.c.1.102 (GUI) MMP/2.0 Push/PO";
const char S98097[] = "Opera/9.52 (Windows ME; U; en)";
const char S98098[] = "Opera/9.50 (Windows ME; U; en)";
const char S98099[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US; rv:1.8.1) Gecko/X Camino/1.1a1";
const char S98100[] = "NOKIAE61i/UC Browser7.9.0.102/28/355/UCWEB";
const char S98101[] = "NOKIAE61/UC Browser7.9.0.102/28/355/UCWEB";
const char S98102[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-I777 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98103[] = "Dolphin http client/8.2.0(62) (Android)";
const char S98104[] = "Dolphin http client/8.2.1(141) (Android)";
const char S98105[] = "Dolphin http client/8.2.0(139) (Android)";
const char S98106[] = "Dolphin http client/8.2.2(144) (Android)";
const char S98107[] = "Nokia5230/UCWEB8.1.0.104/50/800";
const char S98108[] = "samsung-gt-s8500/UC Browser8.0.3.107/69/352";
const char S98109[] = "Mozilla/5.0 (Windows; U; Windows NT 7.1; ru; rv:1.9.3.15) Gecko/X Firefox/6.0";
const char S98110[] = "Mozilla/4.0 (compatible; MSIE 6.0; ; Linux armv5tejl; U) Opera 8.02 [en_US] Maemo browser 0.4.34 N770/SU-18";
const char S98111[] = "Mozilla/5.0 (Windows NT 5.1; rv:7.0.1) Gecko/X Thunderbird/7.0.1 OracleBeehiveExtension/1.0.0.4-OracleInternal Lightning/1.0b7";
const char S98112[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.30 (KHTML, like Gecko) Ubuntu/11.10 Chromium/12.0.742.112 Chrome/12.0.742.112 Safari/534.30";
const char S98113[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.30 (KHTML, like Gecko) Ubuntu/11.10 Chromium/12.0.742.91 Chrome/12.0.742.91 Safari/534.30";
const char S98114[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.7) Gecko/X Epic/ Firefox/";
const char S98115[] = "Mozilla/5.0 Opera/9.5 (KDDI-SN3P; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00";
const char S98116[] = "SonyEricssonW910i/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonw910i/UC Browser7.7.1.88/69/444";
const char S98117[] = "Mozilla/5.0 (Macintosh OS) Gecko/X Firefox/8.0.0";
const char S98118[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Galaxy Mini (SGH-T499) Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98119[] = "iBrowser/Mini2.7 (NokiaX2-01/08.63)";
const char S98120[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Transformer Prime TF201 Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S98121[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I500 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98122[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S98123[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5670 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98124[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; ALCATEL ONE TOUCH 918N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98125[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98126[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98127[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, gb";
const char S98128[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:34, us";
const char S98129[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC One X Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98130[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S98131[] = "SAMSUNG-SAMSUNG-SGH-I997/100.202";
const char S98132[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:14.0) Gecko/X Firefox/14.0.3.0 IceDragon/14.0.3.0";
const char S98133[] = "WebProcess/7534.48.3 CFNetwork/520.0.13 Darwin/11.0.0 (x86_64) (MacBookPro8%2C1)";
const char S98134[] = "Mozilla/5.0 (Java; U; en-us; lg-lg900g) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S98135[] = "lg-t310/UC Browser7.9.0.102/69/352";
const char S98136[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_3; en-us) AppleWebKit/534.53.11 (KHTML, like Gecko) FluidApp/1.2 Safari/534.53.11";
const char S98137[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-us) AppleWebKit/534.57.7 (KHTML, like Gecko) FluidApp/1.2 Safari/534.57.7";
const char S98138[] = "SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonw20i) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S98139[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT15i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S98140[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S98141[] = "NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/70/352";
const char S98142[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P970 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S98143[] = "nokia5250/UC Browser8.0.3.107/69/444  3gpp-gba";
const char S98144[] = "480x800 LGE VS750 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65";
const char S98145[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; md-us; disgo8100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98146[] = "Mozilla/5.0 (Linux; Android 4.0.3; eZee'Tab702 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S98147[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; QQBrowser/7.0.4350.400)";
const char S98148[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S98149[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC Incredible S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S98150[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-gu220) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S98151[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; SPH-L710 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S98152[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonw880i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S98153[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S98154[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxx660) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S98155[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; ZTE-Z990G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S98156[] = "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.0.1337 Mobile Safari/537.10+";
const char S98157[] = "Mozilla/5.0 (Java; U; en-us; nokian95_8gb) UCBrowser8.3.0.154/69/352/UCWEB Mobile";
const char S98158[] = "GoogleEarth/6.0.3.2197(Macintosh;Mac OS X (10.8.2);en;kml:2.2;client:Free;type:default)";
const char S98159[] = "JUC (Linux; U; 4.0.4; zh-cn; GT-I9300; 720*1280) UCWEB8.7.4.225/145/800";
const char S98160[] = "Nokia7020/2.0 (08.11) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia7020) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S98161[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko, M5WebKit/1.0.1, M5HybridSDK/1.0.0) Version/5.1.0 Mobile/6.0.1 Mammoth/5.1.0";
const char S98162[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5560) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S98163[] = "UCWEB/2.0 (Java; U; MIDP-2.0; fa; samsung-gt-s5560i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S98164[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03O) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S98165[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 Build/6.7.2-180_DR4-16_M2-37) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S98166[] = "MQQBrowser/40 Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3";
const char S98167[] = "320x240; VZW; UTStar-XV6175.1; Windows Mobile 6.5 Standard; (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S98168[] = "Mozilla/5.0 (Java; U; es-la; alcatel-ot-807a) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S98169[] = "Mozilla/5.0 (Linux; Android 4.0.3; iDxD7 3G Build/ICS) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S98170[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A7+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S98171[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonk810i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S98172[] = "WhatsApp/2.7.9876 Android/3.2 Device/samsung-GT-P7500";
const char S98173[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7500 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S98174[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7500R Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S98175[] = "Instagram 3.2.0 Android (8/2.2; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; GT-P1000; en_GB)";
const char S98176[] = "Instagram 3.2.0 Android (10/2.3.3; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; gt-p1000; en_US)";
const char S98177[] = "Instagram 3.2.0 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P3100; espressorf; espresso; en_GB)";
const char S98178[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7500 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S98179[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7500R Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S98180[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7500 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S98181[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-P3100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S98182[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-P3100 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S98183[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6210 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.53005";
const char S98184[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6210 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S98185[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; TAB307 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S98186[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A789 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98187[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; Micromax A87 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S98188[] = "JUC (Linux; U; 4.0.4; zh-cn; A100; 480*782) UCWEB7.9.0.94/139/355";
const char S98189[] = "Mozilla/5.0 (Linux; Android 4.0.4; SmartPad870 Build/E8HD-T364-1280x768) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S98190[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax P250(Funbook) Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S98191[] = "MQQBrowser/25 (Linux; U; 2.3.5; zh-cn; LNV-Lenovo A790e Build/Lenovo A790e_S021_X;480*800)";
const char S98192[] = "JUC (Linux; U; 4.0.3; zh-cn; HTC T328d; 480*800) UCWEB8.1.1.122/145/33021";
const char S98193[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A52 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98194[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Intex Aqua 3.2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98195[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_C8810 Build/C8810V100R001C92B862) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98196[] = "MQQBrowser/3.7/Adr (Linux; U; 4.0.4; zh-cn; GT-I9300 Build/IMM76D.I9300ZNBLH3;720*1280)";
const char S98197[] = "Mozilla/5.0 (Java; U; en-us; lg-c310) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S98198[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; A73 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S98199[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S98200[] = "Mozilla/5.0 (Linux; Android 4.0.4; T-02D Build/V19R50B) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S98201[] = "Mozilla/5.0 (Linux; Android 4.0.4; UBISLATE7C+ Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S98202[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; HTC Ruby Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S98203[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S98204[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I500 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S98205[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P765 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S98206[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S98207[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3 UCBrowser8.5.0.195/42/355";
const char S98208[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40HX757; CC/COL; es-ES) Presto/2.10.250 Version/11.60";
const char S98209[] = "LG-GW305/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-gw305) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S98210[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S98211[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I5801 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S98212[] = "Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Kem 432 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S98213[] = "Mozilla/5.0 (Linux; Android 4.0.4; Philips W336 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S98214[] = "Mozilla/5.0 (Linux; Android 4.1.2; MI-ONE Plus Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S98215[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-CN; HTC EVO 3D X515m Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S98216[] = "Mozilla/5.0 (Linux; Android 4.0.4; Smartphone Android by SFR STARADDICT II Build/IceCreamSandwich) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98217[] = "Mozilla/5.0 (Linux; Android 4.0.4; N-08D Build/AX) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S98218[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonwt13i) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S98219[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonwt13i) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S98220[] = "Mozilla/5.0 (Linux; Android 4.1.1; eZee Tab805 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S98221[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; LG-E975w Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98222[] = "JUC (Linux; U; 2.3.5; zh-cn; HTC_S510b; 480*800) UCWEB7.9.4.145/139/32500";
const char S98223[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID9742 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S98224[] = "Mozilla/5.0 (Linux; Android 4.0.4; TM400 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S98225[] = "Nokia206/2.0 (03.59) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia206) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S98226[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; EVO Build/JRO03C; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98227[] = "Mozilla/5.0 (Linux; Android 4.0.4; PadFone 2 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98228[] = "MQQBrowser/2.8/Adr (Linux; U; 2.3.5; zh-cn; Lenovo A780 Build/Lenovo_A780_S026_X;480*800)";
const char S98229[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI U8661 Build/HuaweiU8661) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98230[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  SonyEricssonP990i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S98231[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T329t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98232[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx285) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S98233[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S98234[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE R22 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98235[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-SU760 Build/ICS) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98236[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; SHV-E120L Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S98237[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S98238[] = "JUC (Linux; U; 2.2.2; zh-cn; U8500; 320*480) UCWEB7.9.0.94/139/444";
const char S98239[] = "JUC (Linux; U; 2.2; zh-cn; GT-I5500; 240*320) UCWEB7.9.0.94/139/444";
const char S98240[] = "JUC (Linux; U; 4.0.4; zh-cn; A15; 480*800) UCWEB7.9.0.94/139/444";
const char S98241[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HUAWEI Y 220T Build/HuaweiY220T) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S98242[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xperia S Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S98243[] = "Mozilla/5.0 (Linux; Android 4.0.4; SmartPad7503G Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S98244[] = "Mozilla/5.0 (Linux; Android 4.0.3; Karbonn_A18 Build/KarbonnA18) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98245[] = "Mozilla/5.0 (Linux; Android 4.1.2; A919i Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98246[] = "SuperBot/4.7.0.72 (Win32)";
const char S98247[] = "SAMSUNG-GT-B7330/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S98248[] = "Mozilla/5.0 (Java; U; en-us; nokian72) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S98249[] = "oneX+ Linux/2.6.35 Android/4.0.1 Release/07.20.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1";
const char S98250[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98251[] = "Nokia306/2.0 (03.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia306/UC Browser8.0.3.107/69/444";
const char S98252[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL22EX553; CC/GBR; en) Presto/2.10.250 Version/11.60";
const char S98253[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT862 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S98254[] = "OneBrowser/3.1 (SAMSUNG-GT-B7722/B7722DDKI1)";
const char S98255[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; A1+ Build/MocorDroid2.3.5; 320*480) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444";
const char S98256[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6102E Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98257[] = "OneBrowser/3.1 (NokiaC5-00/071.005/sw_platform=S60;sw_platform_version=3.2;java_build_version=1.0.5)";
const char S98258[] = "UCWEB/2.0(Java; U; MIDP-2.0; es-la; samsung-gt-m3710) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S98259[] = "UCWEB/2.0(Java; U; MIDP-2.0; vi; q-mobile) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S98260[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; A3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98261[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8951 Build/HuaweiT8951) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98262[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A298t Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S98263[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9050 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S98264[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S98265[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ADR6400L Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S98266[] = "Mozilla/5.0 (Linux; Android 4.0.4; FPT Tablet III Build/ARES_00.01.61.H1127) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S98267[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; CELKON CT2 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S98268[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_916) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S98269[] = "Mozilla/5.0 (Linux; Android 4.0.4; Miracle Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S98270[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; SCH-I779 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S98271[] = "verification|card identification|cvn|security code|cvv code|cvc XiaoMi/MiuiBrowser/1.0";
const char S98272[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; X907 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S98273[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A3 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S98274[] = "Instagram 3.1.2 (iPhone; iPhone OS 6.1.3; en_US) AppleWebKit/420+";
const char S98275[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAC-70031 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S98276[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (HTC Glacier, 42, 1.9.1)";
const char S98277[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8833 Build/HuaweiT8833) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S98278[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC Vision Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98279[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; zh-CN; MI 2) U2/1.0.0 UCBrowser/9.0.0.288 U2/1.0.0 Mobile";
const char S98280[] = "Mozilla/5.0 (Linux; Android 4.0.4; Fly_IQ450 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S98281[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xperia S Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98282[] = "LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gs500) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S98283[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MB865 Build/6.7.2_EDEM-18) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S98284[] = "Mozilla/5.0 (Linux; Android 4.1.1; I7100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98285[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB192 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98286[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE-T U880 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S98287[] = "Mozilla/5.0 (Linux; Android 4.0.4; LIFETAB_S9512 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S98288[] = "Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S98289[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID7036 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444";
const char S98290[] = "Mozilla/5.0 (Linux; Android 4.0.4; KM-S200 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98291[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP7280C3G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S98292[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-US; Karbonn_A1 Build/FRF91) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S98293[] = "Mozilla/5.0 (Linux; Android 4.0.4; tmn smart a15 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98294[] = "Mozilla/5.0 (Linux; Android 4.0.4; SH-06D Build/S9030) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98295[] = "Mozilla/5.0 (Linux; Android 4.0.3; URBANO PROGRESSO Build/011.0.3100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98296[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S98297[] = "Mozilla/5.0 (Linux; Android 4.0.4; SH-01E Build/S5270) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98298[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S98299[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Aquaris Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S98300[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE T82 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98301[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P765 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S98302[] = "Mozilla/5.0 (Linux; Android 4.1.1; T-17P Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S98303[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T959 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98304[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax P600 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98305[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT25c Build/9.0.E.1.80) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98306[] = "Mozilla/5.0 (Linux; Android 4.1.1; XOLO_A800 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98307[] = "Mozilla/5.0 (Linux; Android 4.0.3; TAB275 Build/TAB275) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S98308[] = "Mozilla/5.0 (Linux; Android 4.0.4; S120 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98309[] = "Mozilla/5.0 (Linux; Android 4.0.4; TA-M72CBT Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S98310[] = "Mozilla/5.0 (Linux; Android 4.0.4; R8111 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98311[] = "Mozilla/5.0 (Linux; Android 4.1.1; GEM10313BK Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S98312[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033M Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98313[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S98314[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax P500(Funbook) Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S98315[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian70-1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S98316[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian70) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S98317[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T679 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98318[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5235) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba";
const char S98319[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; A54 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S98320[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaN77) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S98321[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROIDX Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S98322[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC Explorer A310e Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98323[] = "Mozilla/5.0 (Linux; Android 4.0.3; I_7520 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98324[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA3250) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S98325[] = "Mozilla/5.0 (Linux; Android 4.0; Swipe_Fablet_F1 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98326[] = "Mozilla/5.0 (Linux; Android 4.0.4; MegaFon_SP-AI Build/1.2.10.4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98327[] = "Mozilla/5.0 (Linux; Android 4.1.2; Coolpad7295 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98328[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI T8833 Build/HuaweiT8833) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S98329[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonwt13i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S98330[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S98331[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC pyramid Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98332[] = "nokiae66-1/UC Browser8.0.3.107/69/444";
const char S98333[] = "TCL_GENESEE_E708_TD/1.0 Android/2.2 Release/01.25.2011 Browser/WAP2.0 AppleWebKit/533.1";
const char S98334[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100G Build/IMM76D; 480*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/32092";
const char S98335[] = "OneBrowser/3.1 (Nokia306/03.63)";
const char S98336[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Celkon CT 9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98337[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Active Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98338[] = "MQQBrowser/3.0/Adr (Linux; U; 4.0.4; zh-cn; HUAWEI U8950D Build/U8950DV100R001C17B939;540*960)";
const char S98339[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.4 Safari/534.34";
const char S98340[] = "JUC (Linux; U; 4.1.2; zh-cn; L003; 480*800) UCWEB7.9.0.94/139/800";
const char S98341[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S98342[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PIRANHA Ultra Tab Zeus White 10.1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S98343[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; LG-P920) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S98344[] = "Mozilla/5.0 (Series40; Nokia5330-1d/06.88; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49";
const char S98345[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; HUAWEI_U8825D) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S98346[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; V8403) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S98347[] = "Mozilla/5.0 (Linux; Android 4.0.4; O+_8.5_Android Build/OplusOplus_A51) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S98348[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; AE910 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98349[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; FreeTAB 9701 IPS Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S98350[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SAMSUNG-SGH-I717, 42, 1.9.1)";
const char S98351[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9260 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S98352[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Wynncom_G1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S98353[] = "JUC (Linux; U; 4.1.2; zh-cn; GT-N8000; 800*1232) UCWEB7.9.0.94/139/444";
const char S98354[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZTE V987 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98355[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Ctrl_V1) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S98356[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC 802w Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98357[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-P5100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S98358[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN76) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S98359[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8190 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S98360[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; SAMSUNGSGH-i450) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S98361[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaE52-1";
const char S98362[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S98363[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC 802d Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98364[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C6603 Build/10.1.A.1.434) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S98365[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S98366[] = "Mozilla/5.0 (Linux; Android 4.0.4; HS-F1 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98367[] = "Mozilla/5.0 (Linux; Android 2.3.5; HTC Explorer A310e Build/GRJ90) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S98368[] = "Mozilla/5.0 (Linux; Android 2.3.5; SGH-I727R Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S98369[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE T790 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98370[] = "Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_S9714 Build/JRO03R) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S98371[] = "Mozilla/5.0 (Linux; Android 4.1.1; IdeaTabA2109A Build/JRO03R) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S98372[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC 802w Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S98373[] = "Mozilla/5.0 (Linux; Android 4.0.4; i9430 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98374[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; TECNO D5 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S98375[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9220 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S98376[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; C2104) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S98377[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HTC X920e Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S98378[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A765e Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S98379[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; Lenovo P770 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S98380[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I8262D Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S98381[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HTC T528w Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S98382[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; PantechP9070 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S98383[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LG-P970 Build/IMM76L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S98384[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HTC T528t Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S98385[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire 600 dual sim Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98386[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; IdeaTab A2107A-H Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S98387[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291";
const char S98388[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-US; GT-I5510 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S98389[] = "Mozilla/5.0 (Linux; Android 4.0.4; A9+ Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S98390[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq80) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S98391[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST25i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S98392[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; A75) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S98393[] = "NokiaX2-01/5.0 (08.71) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291";
const char S98394[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; SGH-T989 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S98395[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6730c) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S98396[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Vision Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S98397[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M440S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S98398[] = "Mozilla/5.0 (Linux; Android 4.2.2; A501 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S98399[] = "Mozilla/5.0 (Linux; Android 4.0.4; Avea inTouch 2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S98400[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V889M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S98401[] = "Mozilla/5.0 (Linux; Android 4.0.3; Newman N2 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S98402[] = "Mozilla/5.0 (Linux; Android 4.2.2; Sensation XL Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S98403[] = "Mozilla/5.0 (Linux; Android 4.2.1; SKY HD9500 PLUS Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S98404[] = "Mozilla/5.0 (Linux; Android 4.2.2; Huawei MediaPad Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S98405[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; LT22i) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S98406[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S98407[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8190) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S98408[] = "Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-55EX630; CC/AUS; en) Presto/2.10.250 Version/11.60";
const char S98409[] = "Opera/9.80 (Linux armv7l; U; ; en) Presto/2.10.250 Version/11.60 Sony Bravia/8.011 (KDL-55EX630, , en, US)";
const char S98410[] = "OV-SteelCore10+ Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24";
const char S98411[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-P930 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98412[] = "MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S11t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98413[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; M1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S98414[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.6 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/537.6 Midori/0.5";
const char S98415[] = "Mozilla/5.0 (Linux; Android 4.2.1; Micromax A116 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98416[] = "Mozilla/5.0 (Linux; Android 4.2.1; iris458Q Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98417[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LGL55C Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S98418[] = "Mozilla/5.0 (Linux; Android 4.1.2; Surfer 8.31 3G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S98419[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; Titanium S1 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S98420[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-in; C5302 Build/12.0.A.1.211) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S98421[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S98422[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Cloud Z5 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S98423[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9103 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S98424[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8190N Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S98425[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST15i) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S98426[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN91) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S98427[] = "Mozilla/5.0 (Linux; Android 4.2.1; AVEO X6 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S98428[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t300) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S98429[] = "Mozilla/5.0 (Windows 98) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2";
const char S98430[] = "Opera/9.80 (Android 4.0.3; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.7.81 Version/11.00";
const char S98431[] = "Mozilla/5.0 (Linux; Android 4.1.1; Symphony Xplorer W35 Build/UNKNOWN) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S98432[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TBD753 Build/Zeki.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S98433[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; ru-ru; novo9-Spark Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S98434[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 UCBrowser/2.3.2.300";
const char S98435[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; XOLO A1000 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S98436[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; L3 Build/TECNOL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/352";
const char S98437[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5580C Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S98438[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT26i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S98439[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; N860 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S98440[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; MID9742) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S98441[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; PIONEER_P1 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S98442[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; Micromax A91 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S98443[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Aqua.Active) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S98444[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; LG-P769 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S98445[] = "Mozilla/5.0 (Linux; Android 4.0.3; A90 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S98446[] = "Mozilla/5.0 (Linux; Android 4.1.2; A111 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S98447[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I8262D Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S98448[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9192 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S98449[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo A798t Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S98450[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lava-a10) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S98451[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI T8950 Build/HuaweiT8950) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S98452[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; X10i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S98453[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6810 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S98454[] = "Mozilla/5.0 (Linux; Android 4.0.4; Avea inTouch 2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S98455[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; GT-7240R Build/GT72XX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S98456[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A87 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S98457[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S98458[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Spice Mi-450 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S98459[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; in-id; E1-B Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S98460[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Aqua_Star Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S98461[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; Karbonn A26 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33756";
const char S98462[] = "Mozilla/5.0 (Linux; Android 4.2.2; Ideapad K1 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S98463[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST70104-1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S98464[] = "Nokia210/2.0 (04.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia210) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S98465[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; ALCATEL_ONE_TOUCH_918N Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S98466[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Incredible S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S98467[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9220 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S98468[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; V9C) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S98469[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S98470[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9228 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S98471[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328t Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S98472[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Titanium_S5 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S98473[] = "Mozilla/5.0 (Linux; Android 4.0.4; KM-E100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S98474[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A89 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S98475[] = "UCWEB/2.0 (Linux; U; Adr 4.2.2; en-US; GT-I9505) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S98476[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua_Star) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S98477[] = "Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_E7310 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S98478[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E410 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S98479[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAX5-01) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S98480[] = "Mozilla/5.0 (Linux; Android 4.0.3; TAB466EUK Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S98481[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; LG-E400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S98482[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae7-00) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile  3gpp-gba";
const char S98483[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; it-it; starlight Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98484[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8262 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S98485[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire 600 dual sim Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.20.1364.172 Mobile Safari/537.22";
const char S98486[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua_Sx) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S98487[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; GT-I9100) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S98488[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; ST15i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S98489[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-E455 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S98490[] = "Mozilla/5.0 (Linux; Android 4.1.1; CT920 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S98491[] = "Mozilla/5.0 (Linux; Android 4.2.1; BLU DASH 4.5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S98492[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-7240R Build/GT72XX) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S98493[] = "Mozilla/5.0 (Linux; Android 4.2.2; SH-08E Build/S8220) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S98494[] = "Mozilla/5.0 (Linux; Android 4.2.1; SKY_NANO Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S98495[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC Explorer A310e) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S98496[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9220 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S98497[] = "Mozilla/5.0 (Linux; Android 4.0.4; H9500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S98498[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP7170B3GCE Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S98499[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI-ONE Plus Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S98500[] = "Mozilla/5.0 (Linux; Android 4.2.1; ZP800H Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S98501[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S98502[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5303 Build/12.0.A.1.257) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S98503[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/11A465 Safari/7534.48.3";
const char S98504[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S98505[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-LS860 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S98506[] = "Mozilla/5.0 (Linux; Android 4.0.4; MZ601 Build/I.7.1-45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S98507[] = "Mozilla/5.0 (Linux; Android 4.2.1; Micromax A110Q Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S98508[] = "Mozilla/5.0 (Linux; Android 4.1.2; PantechP9090 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S98509[] = "Mozilla/5.0 (Linux; Android 4.1.2; USCC-C6721 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S98510[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) MOT-Q9/02.00.01R";
const char S98511[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) MOT-Q9/01.09.34I";
const char S98512[] = "SIE-EF81/39 UP.Browser/7.0.0.1.181 (GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.1.3.h (GUI) MMP/2.0";
const char S98513[] = "SIE-EF81/48 UP.Browser/7.0.0.1.181 (GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.1.3.h (GUI) MMP/2.0";
const char S98514[] = "SIE-EF81/58 UP.Browser/7.0.0.1.181 (GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.1.k.10 (GUI) MMP/2.0";
const char S98515[] = "HTCS620-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 320x240)";
const char S98516[] = "SAMSUNG-SGH-U708/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1/*MzU1MjYyMDExMTk4OTk4";
const char S98517[] = "HTC_P4550 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S98518[] = "HTC_TyTN_II Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S98519[] = "SonyEricssonP800/R101 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S98520[] = "SCH-F699/ Embider/5.30 (GUI; compatible)";
const char S98521[] = "SAMSUNG-SGH-I616/1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S98522[] = "MOT-R4541/00.72 UP.Browser/7.2.6.1.841 (GUI) MMP/2.0";
const char S98523[] = "LGE-MX380/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S98524[] = "Mozilla/5.0 (Linux; U; Android 2.2; ru-ru; MTC 1055 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98525[] = "Mozilla/5.0 (Linux; U; Android 2.2; ru-ru; Beeline E400 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98526[] = "MOT-E680i/E680I_G_0D.C5.ABP Mozilla/4.0 (compatible; MSIE 6.0; Motorola; 1320) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 7.50 [zh-tw]";
const char S98527[] = "MOT-E680i/E680I_G_0D.C5.A4P Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola E680i; 1036) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 7.50 [en]";
const char S98528[] = "MOT-E680i/E680I_G_0D.C5.A3P Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola E680i; 1030) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 7.50 [en]";
const char S98529[] = "MOT-E680i/E680I_G_0D.C5.A6P Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola E680i; 1115) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 7.50 [en]";
const char S98530[] = "MOT-E680i/E680I_G_0D.C5.AAP Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola E680i; 1229) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 7.50 [en]";
const char S98531[] = "MOT-E680i/E680I_G_0D.C3.A8P Mozilla/4.0 (compatible; MSIE 6.0; Motorola; 1030) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 7.50 [zh-tw]";
const char S98532[] = "Nokia6310i/1.0 (4.80) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S98533[] = "LENOVO-A330/(2009.06.05) S007/WAP2.0";
const char S98534[] = "LENOVO-A330/(2009.08.26) S009/WAP2.0";
const char S98535[] = "LENOVO-A330/(2009.08.26) S012/WAP2.0";
const char S98536[] = "SAMSUNG-GT-S5500H/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1/*MA==";
const char S98537[] = "SAMSUNG-GT-S5500/ SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S98538[] = "ZTE-F252/1.0 ACS-NF/3.2 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S98539[] = "SonyEricssonK790i/R8BF Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S98540[] = "dopodT8388/1.0 WindowsMobile/6.5 CEOS/5.2 release/5.0 IE/6.0 WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1 4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S98541[] = "dopodT8388/1.0 WindowsMobile/6.5 CEOS/5.2 release/5.0 IE/6.0 WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S98542[] = "SAMSUNG-SGH-Z248/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S98543[] = "Spice QT-57/(2010.06.22) S005/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S98544[] = "NokiaN97/UC Browser7.6.1.82/50/352";
const char S98545[] = "NokiaC1-01/2.0 (04.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac1-01/UC Browser7.6.1.82/69/352";
const char S98546[] = "Nokia5230/UCWEB7.8.0.87/50/999";
const char S98547[] = "NokiaE66/UC Browser7.7.1.88/28/351";
const char S98548[] = "Vodafone/1.0/SamsungSGHX680V/X680BUFG4 Browser/NF/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S98549[] = "Samsung-SPHM530 AU-OBIGO/Q04C1-1.22 MMP/2.0";
const char S98550[] = "Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 SonyEricssonP100/01; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525";
const char S98551[] = "DoCoMo/2.0 F08C(c100;TB;W20H09)";
const char S98552[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/1A542a Safari/419.3";
const char S98553[] = "HOSIN_M100/SW1.0.0/WAP2.0";
const char S98554[] = "nokia6233/UC Browser7.7.1.88/69/444";
const char S98555[] = "Opera/7.23 (Windows NT 6.0; U) [zh-cn]";
const char S98556[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.8.3";
const char S98557[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.8.4";
const char S98558[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.8.1";
const char S98559[] = "micromax/UC Browser7.9.0.102/69/352";
const char S98560[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.9) Gecko/X Linux Mint/7 (Gloria) Shiretoko/3.5.7";
const char S98561[] = "Mozilla/4.0 (compatible; ipMonitor 10.5)";
const char S98562[] = "IUC(U;iOS 4.3.1;Zh-cn;320*480;)/UCWEB7.6.0.75/41/997";
const char S98563[] = "IUC(U;iOS 4.2.1;Zh-cn;320*480;)/UCWEB7.6.0.75/41/997";
const char S98564[] = "NokiaE7-00/UC Browser7.9.0.102/50/355/UCWEB";
const char S98565[] = "SonyEricssonC510/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonc510/UC Browser8.0.3.107/69/444";
const char S98566[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.152 Safari/535.19 Comodo_Dragon/18.1.2.0";
const char S98567[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.152 Safari/535.19 Comodo_Dragon/18.1.2.0";
const char S98568[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/ Sleipnir/1.9.1m";
const char S98569[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3 Sleipnir/1.9.2m";
const char S98570[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/1.9.2m";
const char S98571[] = "Mozilla/5.0 (Java; U; en-us; nexiannx-g868t) UCBrowser8.2.0.132/69/444/UCWEB Mobile";
const char S98572[] = "Mozilla/5.0 (Java; U; en-in; spice) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S98573[] = "Mozilla/5.0 (Windows NT 5.1; rv:6.0) Gecko/X Firefox/6.0 slurp";
const char S98574[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.2) Gecko/X Ubuntu/9.25 (jaunty) Firefox/3.8";
const char S98575[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Iron/6.0.475.1 Chrome/6.0.475.1 Safari/534.3";
const char S98576[] = "LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  lg-gd510/UC Browser8.0.3.107/69/352";
const char S98577[] = "MacOutlook/14.14.0.X (Intel Mac OS X 10.5.8)";
const char S98578[] = "MQQBrowser/2.0 (NokiaE72-1;SymbianOS/9.1 Series60/3.0)";
const char S98579[] = "GoogleEarth/5.2.1.1588(Macintosh;Mac OS X (10.5.8);en;kml:2.2;client:Free;type:default)";
const char S98580[] = "Mozilla/4.0 (jig browser web; 1.0.4; 830P)";
const char S98581[] = "Mozilla/5.0 (X11; SunOS sun4u; rv:6.0) Gecko/X Firefox/6.0";
const char S98582[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) Opera 7.50 [en]";
const char S98583[] = "Mozilla/5.0 (Windows NT 5.0; U) Opera 7.50";
const char S98584[] = "Mozilla/5.0 (Windows NT 6.0; rv:13.0) Gecko/X Firefox/13.0.1 slurp";
const char S98585[] = "Dolphin http client/8.6.1(159) (Android)";
const char S98586[] = "Dolphin http client/8.6.0(156) (Android)";
const char S98587[] = "Mozilla/5.0 (Windows NT 6.0; rv:6.0.2) Gecko/X Firefox/6.0.2 slurp";
const char S98588[] = "spice/UC Browser7.9.0.102/69/352";
const char S98589[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-tw; MioPad 6 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98590[] = "Mozilla/5.0 (Series40; Nokia7310/09.40; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S98591[] = "Mozilla/5.0 (Linux; U; Android unknown; fr-fr; A70HB Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S98592[] = "Mozilla/5.0 (Linux; U; Android 2.2.1.1832; th-th; i-mobile i810 Build/s1832) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98593[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; th-th; i-mobile i810 Build/s1834) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98594[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; HTC Tattoo Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98595[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98596[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98597[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98598[] = "MDN/1.0 CFNetwork/485.13.9 Darwin/10.7.0";
const char S98599[] = "JUC (Linux; U; 2.2.2; zh-cn; GT-I9008L; 480*800) UCWEB7.9.4.145/139/800";
const char S98600[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5260) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S98601[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.7.2) Gecko/X Netscape/7.2 (ax;nscd1)";
const char S98602[] = "nokiae65-1/UC Browser7.7.1.88/69/444";
const char S98603[] = "Mozilla/5.0 (Linux; Android 4.0.3; GEM8112 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S98604[] = "NOKIA-Lumia 800/UCWEB2.5.0.202/47/32079";
const char S98605[] = "iBrowser/2.4 (E63-1;SymbianOS/9.1 Series60/3.0)";
const char S98606[] = "Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.215 Safari/535.1";
const char S98607[] = "Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.215 Safari/535.1";
const char S98608[] = "Mozilla/5.0 (X11; U; SunOS i86pc; de; rv:1.9.1b3) Gecko/X Firefox/3.1b3";
const char S98609[] = "HTC_P3470 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S98610[] = "Mozilla/5.0 (Linux; Android 4.0.3; PMP3270B Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S98611[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; Galaxy Nexus Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S98612[] = "Mozilla/5.0 (Java; U; en-in; lg-t300) UCBrowser8.3.0.154/69/405/UCWEB Mobile";
const char S98613[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9308 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98614[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.0.16) Gecko/X Firefox/3.0.16 Flock/2.5.6";
const char S98615[] = "Mozilla/5.0 (Java; U; en-us; lava-a10) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S98616[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAX5-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile";
const char S98617[] = "JUC(Linux;U;Android2.3.6;Zh_cn;GT-B5512;240*320;)UCWEB7.8.0.95/139/444";
const char S98618[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S98619[] = "Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU8i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/352/UCWEB Mobile";
const char S98620[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonw150i) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S98621[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonw150i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S98622[] = "UCWEB/2.0(Java; U; MIDP-2.0; fa; sonyericssonw150i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S98623[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/6.5.27431/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S98624[] = "Mozilla/5.0 (Windows NT 6.2; rv:5.0) Gecko/X Firefox/5.0 CometBird/5.0";
const char S98625[] = "Mozilla/5.0 (Windows NT 5.0; rv:13.0) Gecko/X Firefox/13.0 SeaMonkey/2.10";
const char S98626[] = "Skype/4.0.0.1325 CFNetwork/548.1.4 Darwin/11.0.0";
const char S98627[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S98628[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Spice_Mi-425 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98629[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3 TeamStream HD (iPad2,1 284 1.7)";
const char S98630[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ST15i Build/4.0.2.A.0.42; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98631[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I5503 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S98632[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; mot-ex223) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S98633[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MK16i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S98634[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3370) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S98635[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; myTouch_4G_Slide Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98636[] = "Mozilla/5.0 (Linux; Android 4.1.2; SC-03D Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S98637[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; Nexus S Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S98638[] = "Mozilla/5.0 (Android; Linux armv7l;rv:5.0) Gecko/X Firefox/5.0 Fennec/5.0";
const char S98639[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98640[] = "Instagram 1.1.1 Android (8/2.2; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; GT-P1000; en_GB)";
const char S98641[] = "Instagram 1.1.6 Android (10/2.3.3; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; gt-p1000; en_US)";
const char S98642[] = "Instagram 3.2.0 Android (15/4.0.3; 160dpi; 800x1232; samsung; GT-P5100; espresso10rf; espresso10; en_US)";
const char S98643[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; GT-P3100 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31";
const char S98644[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P3100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S98645[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S98646[] = "Mozilla/5.0(Linux; U; Android 2.3.4; zh-CN; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.0.212 U3/0.8.0 Mobile Safari/534.31";
const char S98647[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31";
const char S98648[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S98649[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; GT-P1000T Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.1.258 U3/0.8.0 Mobile Safari/534.31";
const char S98650[] = "PPC; 240x320; HTC_P3300/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S98651[] = "JUC(Linux;U;2.3.4;En;LT18i;480*854;)UCWEB7.7.0.85/139/999";
const char S98652[] = "JUC(Linux;U;Android4.0.4;Zh_cn;ST18i;480*854;)UCWEB7.8.0.95/139/444";
const char S98653[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98654[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U1 Build/HCL ME Tablet U1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98655[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; A100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S98656[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24";
const char S98657[] = "Mozilla/5.0 (Linux; Android 4.0.4; Novo7Fire Build/X.qm) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S98658[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i929 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98659[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9308 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S98660[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S98661[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-I8190 Build/JRO03H) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S98662[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-T U880 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S98663[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; PG86100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S98664[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:19.0) Gecko/X Firefox/19.0 slurp";
const char S98665[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; P300(Funbook) Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S98666[] = "JUC (Linux; U; 2.2; zh-cn; generic; 480*800) UCWEB7.9.0.94/139/444";
const char S98667[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5350) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S98668[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Evo V 4G Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98669[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lemon) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S98670[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MK16i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S98671[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9003 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S98672[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5503 Build/10.1.1.A.1.210) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S98673[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5503 Build/10.1.1.A.1.191) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S98674[] = "Mozilla/5.0 (Linux; Android 4.1.1; PAP5000TDUO Build/PrestigioPAP5000TDUO) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S98675[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile";
const char S98676[] = "Mozilla/5.0 (Linux; Android 4.0.4; Alpha GTR Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98677[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xperia ion Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98678[] = "Mozilla/5.0 (Linux; Android 4.1.2; KM-E100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98679[] = "Mozilla/5.0 (Linux; Android 4.0.4; Inspire 4G Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98680[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M440S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98681[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98682[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; I-Buddy 3 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S98683[] = "Mozilla/5.0 (Linux; Android 4.0.4; Novo7Flame Build/X.tm) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S98684[] = "Mozilla/5.0 (Linux; Android 4.0.4; VIVO 4.3 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98685[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax P500(Funbook) Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S98686[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mot-ex223) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S98687[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC_Amaze_4G Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S98688[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; ZTE U790 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S98689[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; SCH-i929 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S98690[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-LU6200 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98691[] = "Mozilla/5.0 (Linux; Android 4.2.2; Inspire 4G Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98692[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; PIONEER Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S98693[] = "Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Kem 432 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98694[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gu220) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S98695[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-L38C Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S98696[] = "YahooMobileMessenger/1.0 (Android Mail; 1.4.5) (inc; HTC; ADR6300; 2.3.4/GRJ22)";
const char S98697[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; shooteru Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98698[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LNV-Lenovo A790e Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98699[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E10i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S98700[] = "Mozilla/5.0 (Linux; Android 4.0.4; Andi4.5h Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S98701[] = "Mozilla/5.0 (Linux; Android 4.1.2; iris502 Build/LAVAIRIS502) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98702[] = "Mozilla/5.0 (Java; U; en-us; sgh-f480i) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S98703[] = "Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia305/UC Browser7.6.1.82/69/351";
const char S98704[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; htc_jewel Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98705[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6802 Build/GINGERBREAD; 320*480) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444";
const char S98706[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.3.5 Safari/534.34";
const char S98707[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E10i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S98708[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-S5660 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98709[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-SU880 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S98710[] = "Mozilla/5.0 (Java; U; en-us; nokia6680) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S98711[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:5.0) Gecko/X Thunderbird/5.0";
const char S98712[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOR801 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98713[] = "Mozilla/5.0 (Linux; Android 4.0.3; Evodroid W10 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S98714[] = "Nokia5330-1d/5.0 (06.88) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5330-1d) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S98715[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID7042 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S98716[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-S6012 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S98717[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; VS920 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98718[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; Micromax A60 Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S98719[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; ONDA MID Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S98720[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAE55-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S98721[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5330) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S98722[] = "TBD753B Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S98723[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; HTC_Wildfire) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S98724[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S98725[] = "Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB M713G Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S98726[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; TECNO_B3 Build/TECNO_B3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/33567";
const char S98727[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; WT19a) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S98728[] = "Mozilla/5.0 (Linux; Android 4.1.2; UMI X1 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S98729[] = "Mozilla/5.0 (Linux; Android 4.2.2; Xperia S Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S98730[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; T-Mobile_myTouch_3G_Slide) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S98731[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; X909 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S98732[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI T8950 Build/HuaweiT8950) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S98733[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A820L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S98734[] = "Mozilla/5.0 (Linux; U; Android 4.2.1 Jelly Bean; es-es; SonyEricssonMT11i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98735[] = "Mozilla/5.0 (Linux; Android 4.1.2; GN708W Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98736[] = "Mozilla/5.0 (Linux; Android 4.0.3; P300(Funbook) Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S98737[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT30a Build/7.0.A.3.195) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98738[] = "Mozilla/5.0 (Linux; Android 4.0.4; Spice_Mi-495 Build/SpiceSpice_Mi-495) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98739[] = "Mozilla/5.0 (Linux; Android 4.0.4; V96A Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S98740[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MyTouch Slide 4G Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98741[] = "Mozilla/5.0 (Linux; Android 4.0.4; Blaze S180 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98742[] = "Mozilla/5.0 (Linux; Android 4.0.3; RMD-1035 Build/ICS.g12refM1005RU.X) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S98743[] = "Mozilla/5.0 (Linux; Android 4.0.4; Magic Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S98744[] = "Mozilla/5.0 (Linux; Android 4.0.4; RunboX5-W Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98745[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Mobile Safari/535.19 Silk-Accelerated=false";
const char S98746[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A8 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S98747[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:21.0) Gecko/X Firefox/21.0 slurp";
const char S98748[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-N8010 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S98749[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I605 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S98750[] = "Mozilla/5.0 (Linux; Android 4.0.4; shooteru Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98751[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; ALCATEL_ONE_TOUCH_918N Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S98752[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Spice_Mi-355) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S98753[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S98754[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F160L Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98755[] = "Mozilla/5.0 (Linux; Android 4.0.4; P-06D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98756[] = "Mozilla/5.0 (Linux; Android 4.1.2; SBM200SH Build/S1002) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98757[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E970 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S98758[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Lenovo K860i Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S98759[] = "OneBrowser/3.1 (GT-S5233A/S523AJPJG3)";
const char S98760[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI MT1-U06 Build/HuaweiMT1-U06) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S98761[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST26i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S98762[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S98763[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-SU870 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98764[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F120K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98765[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHV-E170K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98766[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT26ii Build/6.2.B.0.200) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S98767[] = "Mozilla/5.0 (Linux; Android 4.1.2; folio100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S98768[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAB-1030 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S98769[] = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus S 4G Build/JRO03R) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S98770[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-b7722) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S98771[] = "Mozilla/5.0 (Linux; Android 4.0.4; CROSS A7S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98772[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; A953 Build/JZO54K)";
const char S98773[] = "JUC (Linux; U; 2.3.6; zh-cn; U8185; 240*320) UCWEB7.9.0.94/139/444";
const char S98774[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; MAGPAD9.7 Pro Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S98775[] = "Mozilla/5.0 (Linux; Android 4.0.4; Y3 Build/HCL) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S98776[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; zh-CN; E15i) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile";
const char S98777[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-VM670 Build/JDQ39; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98778[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:21.0) Gecko/X Firefox/21.0 TenFourFox/7450";
const char S98779[] = "Mozilla/5.0 (Linux; Android 2.3.3; LG-P500 Build/GRI40) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S98780[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9210T Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98781[] = "Mozilla/5.0 (Linux; Android 2.3.4; LG-MS910 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S98782[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZTE V987 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98783[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; HTC Vision Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S98784[] = "Mozilla/5.0 (Linux; Android 2.3.3; HTC Vision Build/GRI40) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453";
const char S98785[] = "Mozilla/5.0 (Linux; Android 4.0.4; A898 Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98786[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S6312 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S98787[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; LG-P500 Build/FRG83) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S98788[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26i Build/6.1.A.2.45) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S98789[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P3105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S98790[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t300) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S98791[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; IQ440 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S98792[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.7.12) Gecko/X Thunderbird/1.0.7";
const char S98793[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL32EX520; CC/XGA; en) Presto/2.7.61 Version/11.00";
const char S98794[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; U9500 Build/HuaweiU9500) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98795[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98796[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; S8800 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98797[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; Micromax_A70) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S98798[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Lenovo S720i Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98799[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Skate Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98800[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; zh-CN; GT-S5570) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile";
const char S98801[] = "Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB A93.2 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S98802[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6400L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (ADR6400L, 42, 1.9.1)";
const char S98803[] = "Mozilla/5.0 (Linux; Android 4.0.4; fnac 4.5 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S98804[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I777 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S98805[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; IM-A770K Build/->Wangsai_008; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S98806[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N8010 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);";
const char S98807[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; XOLO_A800 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S98808[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-CN; ZTE-T U880 Build/FRG83G) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31";
const char S98809[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Videocon_A20) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S98810[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaN97 mini";
const char S98811[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:7.0.1) Gecko/X Postbox/3.0.8";
const char S98812[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; Lenovo A298t Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S98813[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9228 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S98814[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; MT27i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S98815[] = "Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar_Touch_KEM_452_HDMI Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98816[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A11 Build/KarbonnA11) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S98817[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A798t Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S98818[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; ALCATEL ONE TOUCH 5020D Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S98819[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; T8830Pro Build/HuaweiT8830Pro) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S98820[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; R807 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S98821[] = "Mozilla/5.0 (Linux; Android 4.0.4; M870 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S98822[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9268 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S98823[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; W60 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S98824[] = "Mozilla/5.0 (Linux; Android 2.3.5; YP-G70 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S98825[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S98826[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE N880E Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S98827[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HTC 802w Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S98828[] = "Mozilla/5.0 (Linux; Android 4.0.4; Ctrl_V3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98829[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT26ii Build/6.2.B.0.200) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S98830[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LG-MS910 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S98831[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:22.0) Gecko/X Firefox/22.0 slurp";
const char S98832[] = "UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Micromax A110Q) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S98833[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Desire S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S98834[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Lenovo A830 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S98835[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ALCATEL ONE TOUCH 992D Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S98836[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk810i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S98837[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI A199 Build/HuaweiA199) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S98838[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S98839[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9228 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S98840[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S98841[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI G525-U00 Build/HuaweiG525-U00) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S98842[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI-ONE C1 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S98843[] = "SonyEricssonU10i/R7BA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonu10i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S98844[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-E612 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S98845[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; SCH-S720C Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S98846[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A44) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S98847[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-L38C Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S98848[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I777 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S98849[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; LG-E400) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S98850[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8190) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S98851[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw350i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S98852[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-c3303i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S98853[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; HTC Desire X) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S98854[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MT11i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S98855[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9200 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S98856[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E10i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S98857[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F100S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S98858[] = "Mozilla/5.0 (Linux; Android 4.2.2; VS920 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S98859[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A3+ Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S98860[] = "Mozilla/5.0 (Linux; Android 4.1.2; SH631W Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S98861[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax P600 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S98862[] = "Mozilla/5.0 (Linux; Android 4.2.2; M785 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S98863[] = "Mozilla/5.0 (Linux; Android 4.0.3; URBANO PROGRESSO Build/011.0.3100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S98864[] = "Mozilla/5.0 (Linux; Android 4.2.1; GSmart Maya M1 v2 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S98865[] = "Mozilla/5.0 (Linux; U; Android 4.1; zh-tw; ThL W6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98866[] = "Mozilla/5.0 (Linux; U; Android 3.1; es-es; LG-P920 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98867[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-; aigoPad M80E Build/E8HD-ELITE-1280x768) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S98868[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; UNUSUAL Vortex Pocket Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98869[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Next8P12) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S98870[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-B5330 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S98871[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) Nokia5530";
const char S98872[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; ST18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S98873[] = "Mozilla/5.0 (Linux; Android 4.0.3; Karbonn_A18 Build/KarbonnA18) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98874[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-US; GT-I5801 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S98875[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Q1000 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S98876[] = "SAMSUNG-GT-S5292 Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10";
const char S98877[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; iBall Slide 3G7271 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S98878[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; CONNECT 2G Build/HCL ME Tablet CONNECT 2G) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S98879[] = "Mozilla/5.0 (Linux; Android 4.2.1; Aqua I-5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98880[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; zh-CN; MI 1S) U2/1.0.0 UCBrowser/9.1.1.309 U2/1.0.0 Mobile";
const char S98881[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax A68 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S98882[] = "UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Micromax A116) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S98883[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Celkon A 200 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S98884[] = "NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291";
const char S98885[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax P360 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S98886[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; S100 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S98887[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Q800) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S98888[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S98889[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L720 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/jfltespr";
const char S98890[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; C1504 Build/11.3.A.0.47) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S98891[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N8010 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb";
const char S98892[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGP312 Build/10.1.C.0.370) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S98893[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; Xperia Ray Build/JZO54K)";
const char S98894[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-T679 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98895[] = "OneBrowser/3.1 (SonyEricssonW810i/R4EA031)";
const char S98896[] = "Nokia114/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia114/UC Browser8.0.3.107/69/444";
const char S98897[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98898[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Next7C12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98899[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; LG-E425) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S98900[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon";
const char S98901[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ZTE-Z990G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S98902[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9228 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S98903[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S98904[] = "Mozilla/5.0 (Linux; Android 4.1.2; Lenovo A706_ROW Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98905[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; vivo Y11 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S98906[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A7 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S98907[] = "Mozilla/5.0 (Linux; Android 4.0.4; iris454 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S98908[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9500 Build/JDQ39) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S98909[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Sensation Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S98910[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I905 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S98911[] = "Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 510;7.10;8779)";
const char S98912[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8190N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98913[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A500 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.289";
const char S98914[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HUAWEI U8815 Build/HuaweiU8815C02B952) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S98915[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-S7560M Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S98916[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Barnes & Noble Nook Tablet Build/JDQ39E; LiquidSmooth-Liquid-JB-v2.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98917[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);";
const char S98918[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; SCH-S738C Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S98919[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; A90 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98920[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; A111 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300";
const char S98921[] = "Mozilla/5.0 (Linux; Android 4.1.2; 201F Build/V35R13E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98922[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N8000 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S98923[] = "Mozilla/5.0 (Linux; Android 4.0.4; BNTV400 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S98924[] = "Mozilla/5.0 (Linux; Android 4.1.1; Vodafone 875 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S98925[] = "Mozilla/5.0 (Linux; Android 4.2.1; ZP980 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S98926[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax A68 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S98927[] = "JUC (Linux; U; 4.1.1; zh-cn; HUAWEI C8813; 480*854) UCWEB8.5.1.179/145/33523";
const char S98928[] = "Mozilla/5.0 (Linux; Android 4.0; Swipe_Fablet_F1 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S98929[] = "Mozilla/5.0 (Linux; Android 4.1.1; IQ430 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98930[] = "Mozilla/5.0 (Linux; Android 2.3.4; R800i Build/4.0.2.A.0.42) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S98931[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; T720 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98932[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.289";
const char S98933[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S98934[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-in; MT11i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S98935[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Spice_Mi-502) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S98936[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; TECNO_D3) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S98937[] = "Mozilla/5.0 (Linux; Android 4.0.4; Aqua flash Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S98938[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A798t Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S98939[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; Xbox; Xbox One)";
const char S98940[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; ST15i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S98941[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P715 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S98942[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S98943[] = "Nokia7310/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7310) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S98944[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LT15i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S98945[] = "SonyEricssonW995/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw995) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S98946[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-m5650) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S98947[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-B5512 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S98948[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-E612 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S98949[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; L3 Build/TECNOL3) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S98950[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Karbonn_A4_) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S98951[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I8190 Build/JRO03H.I8190XXALJL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S98952[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S98953[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9500 Build/JDQ39) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S98954[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; HUAWEI_Y300-0100) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S98955[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Aqua I-5 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S98956[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P990 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S98957[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S98958[] = "Mozilla/5.0 (Linux; Android 4.0.4; U30GT-H Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S98959[] = "Mozilla/5.0 (Linux; Android 4.2.2; V812 Core4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S98960[] = "Mozilla/5.0 (Linux; Android 4.0.4; dL1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S98961[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC One S Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S98962[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-L38C Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S98963[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Nexus S Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S98964[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; HTC Glacier Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S98965[] = "Mozilla/5.0 (Java; U; en-us; samsung gt-c3222) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S98966[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko, M5WebKit/1.0.2, M5HybridSDK/1.0.0) Version/5.1.1 Mobile/7.0 Mammoth/5.1.1";
const char S98967[] = "Mozilla/5.0 (Linux; Android 4.0.4; CloudPad 700d Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S98968[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.1; en-in; A210 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S98969[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT907 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/xt907";
const char S98970[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; LG-E612) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S98971[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC One X Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S98972[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A4+) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S98973[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac6-01) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba";
const char S98974[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIANokia N81) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S98975[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST25i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S98976[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; SK17i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S98977[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5312 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S98978[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I727 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98979[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_ChaCha_A810e) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S98980[] = "[FBAN/FB4A;FBAV/3.6.1;FBBV/X;FBDM/{density=2.0,width=720,height=1184};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/DROID RAZR HD;FBSV/4.1.2;FBCA/armeabi-v7a:armeabi;]";
const char S98981[] = "UCWEB/2.0 (Linux; U; Adr 2.3.8; en-US; Andi3.5KKe) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S98982[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI T8951 Build/HuaweiT8951) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S98983[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-I959 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S98984[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-SU870 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S98985[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; LG-P705 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S98986[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E170K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S98987[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; iris504Q Build/iris504Q) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S98988[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-M919 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S98989[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S6312 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S98990[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8190N Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S98991[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S98992[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC One X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S98993[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.5 Mobile Safari/533.1";
const char S98994[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E988 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S98995[] = "Nokia201/2.0 (11.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia201) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S98996[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S98997[] = "Mozilla/5.0 (Linux; Android 4.1.2; CINK FIVE Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S98998[] = "Mozilla/5.0 (Linux; Android 4.1.1; Orange Daytona Build/HuaweiG510-0200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S98999[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8260 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S99000[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S99001[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515m Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S99002[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; HTC Tattoo Build/FLAN) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S99003[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; HTC Tattoo Build/ERD35B) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S99004[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; HTC Tattoo Build/ERD72) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S99005[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; HTC Tattoo Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S99006[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HTC Tattoo Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S99007[] = "SAMSUNG-SGH-F110/F110XAHB3 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S99008[] = "LG-GC900/V10a Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S99009[] = "Mozilla/5.0 (Vodafone/1.0/LG-GC900/V10b Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S99010[] = "Mozilla/5.0 (Vodafone/1.0/LG-GC900/V10c Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S99011[] = "Mozilla/5.0 (Linux; U; Android WildPuzzleROM v8.0.7 froyo 2.2; fr-fr; HTC Wildfire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99012[] = "Mozilla/5.0 (Linux; U; Android WildPuzzleROM v8.0.11 FRF91 2.2; en-gb; HTC Wildfire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99013[] = "Mozilla/5.0 (Linux; U; Android WildPuzzleROM v8.0.10 FRF91 2.2; ru-ru; HTC Wildfire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99014[] = "Mozilla/5.0 (Linux; U; Android SUnAaBh.v1.1; en-gb; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99015[] = "KWC-E2000/1013 UP.Browser/7.2.6.1.567 (GUI) MMP/2.0";
const char S99016[] = "KWC-E2000/1022 UP.Browser/7.2.6.1.537 (GUI) MMP/2.0";
const char S99017[] = "KWC-E2000/10086 UP.Browser/7.2.6.1.537 (GUI) MMP/2.0";
const char S99018[] = "KWC-E2000/1017 UP.Browser/7.2.6.1.537 (GUI) MMP/2.0";
const char S99019[] = "KWC-E2000/1016 UP.Browser/7.2.6.1.537 (GUI) MMP/2.0";
const char S99020[] = "SEC-SGHC300/1.0";
const char S99021[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; MDA compact/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1)";
const char S99022[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; Tactile_internet-orange-LS Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S99023[] = "SEC-SGHB320/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S99024[] = "SEC-SGHM120/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S99025[] = "LG-KG200j/V01 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1 LG-KG200j/V01 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S99026[] = "Nexian NX-T909/MTK Release/10.01.2009 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1";
const char S99027[] = "MOT-V980/80.2E.27I MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S99028[] = "MOT-V980M/83.28.38. MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S99029[] = "SEC-SGHB220B/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S99030[] = "BlackBerry8703e/4.5.0.148 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/104";
const char S99031[] = "SonyEricssonK310i";
const char S99032[] = "Mozilla/5.0 (Linux; U; Android SUnAaBh.v1.1; en-gb; HTC_Wildfire_A3333 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99033[] = "Mozilla/5.0 (Linux; U; Android WildPuzzleClearROM qmod02 base_1.37.405.1; ru-ru; HTC_Wildfire_A3333 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S99034[] = "Mozilla/5.0 (Linux; U; Android WildPuzzleROM WWE_1.25.405.1; en-gb; HTC_Wildfire_A3333 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S99035[] = "HTC_Hermes Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S99036[] = "PPC; 240x320; HTC_TyTN/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S99037[] = "MOT-V220ENS/0B.D1.32R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S99038[] = "MOT-V220/0B.D1.09R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S99039[] = "SonyEricssonK510i/R4EA Java/SEMC-Java/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S99040[] = "KWC-K127/1204 UP.Browser/6.2.3.9.g.1.110 (GUI) MMP/2.0";
const char S99041[] = "KWC-K127/11031 UP.Browser/6.2.3.9.g.1.110 (GUI) MMP/2.0";
const char S99042[] = "KWC-K127/1002 UP.Browser/6.2.3.9.g.1.110 (GUI) MMP/2.0";
const char S99043[] = "LG/U8110/v2.0";
const char S99044[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HTC-S710d/1.30.1401.3) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99045[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HTC-S710d/1.43.1401.5) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99046[] = "ZTE-R90/1.0 ACS-NF/3.4 QTV2.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S99047[] = "SAMSUNG-SGH-M140L/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S99048[] = "LG-LX400 TELECA-LX400/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S99049[] = "Videocon_V1435/WAP2.0 Profile/MT6253/QCIF";
const char S99050[] = "Videocon_V1435/WAP2.0 Profile/MT6253/QCIF, Videocon_V1435/WAP2.0 Profile/MT6253/QCIF";
const char S99051[] = "SAMSUNG-SGH-U908/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S99052[] = "Nokia6630/1.0 (5.03.08) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.4.0.65/27/351";
const char S99053[] = "SAMSUNG-GT-C6112/C6112DDJB2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S99054[] = "SAMSUNG-SGH-i900/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.6.1.82";
const char S99055[] = "HTC_P3400i-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.6.1.82";
const char S99056[] = "NokiaC5-03/SymbianOS/9.1 Series60/3.0 3gpp-gba";
const char S99057[] = "BlackBerry7100/4.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S99058[] = "Mozilla/4.1 (compatible; MSIE 5.0; Symbian OS; Series 60;424) Opera 6.10 [en]";
const char S99059[] = "K444/1.0 MTK/MAUI.08A.W08.28.MP.V2 Release/7.30.2009 Browser/OBIGO-Q03C";
const char S99060[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322) Sleipnir/2.21";
const char S99061[] = "gt-s5233w/UC Browser7.7.1.88/69/444";
const char S99062[] = "SamsungI8910";
const char S99063[] = "Mozilla/5.0 (Linux; U; Android 1.6; zh-cn; MOT-A1680 Build/DRD35) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S99064[] = "Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.7.2) Gecko/X Netscape/7.2 (ax)";
const char S99065[] = "Mozilla/5.0 (Windows; U; Windows NT 4.0; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.38 Safari/532.0";
const char S99066[] = "Mozilla/4.0 (jig browser web; 1.0.4; 934SH)";
const char S99067[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.18pre) Gecko/X Ubuntu/8.04 (hardy) Namoroka/3.6.18pre";
const char S99068[] = "JUC(Linux;U;Android2.1-update1;Zh_cn;GT-I5801;240*400;)UCWEB7.8.0.95/140/444";
const char S99069[] = "Mozilla/5.0 (X11; Linux i686; rv:7.0) Gecko/X Thunderbird/7.0";
const char S99070[] = "JUC(Linux;U;2.3.3;Zh_cn;HTC Wildfire S A510e;320*480;)UCWEB7.8.1.96/139/800";
const char S99071[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.7.10246.201";
const char S99072[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.7.10246.201";
const char S99073[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; myTouch_4G_Slide Build/GRJ22; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99074[] = "Mozilla/5.0 (Windows NT 5.1; rv:2.2a1pre) Gecko/X Firefox/4.2a1pre";
const char S99075[] = "SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonj20i/UC Browser8.0.3.107/69/444";
const char S99076[] = "JUC(Linux;U;2.3.3;Zh_cn;HTC Desire S;480*800;)UCWEB7.8.1.96/139/800";
const char S99077[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB8.2.0.116/31/999";
const char S99078[] = "Mozilla/5.0 (Windows NT 5.0; rv:8.0) Gecko/X Thunderbird/8.0";
const char S99079[] = "Opera/9.80 (Linux armv7l; Opera Mobi/10; U; en) Presto/2.7.81 Version/11.00";
const char S99080[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X; U; en) Opera 8.51";
const char S99081[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:7.0.1) Gecko/X Thunderbird/7.0.1 Lightning/1.0b7";
const char S99082[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Acer; M310)";
const char S99083[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.8.1.8) Gecko/X Firefox/2.0.0.8 Flock/1.0";
const char S99084[] = "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:6.0.2) Gecko/X Firefox/6.0.2 PaleMoon/6.0.2";
const char S99085[] = "nokiae72-1/UC Browser7.9.0.102/69/352  3gpp-gba";
const char S99086[] = "MQQBrowser/3.0 (Nokia5230;SymbianOS/9.1 Series60/3.0)";
const char S99087[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s8500) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S99088[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/21.0.1180.89)";
const char S99089[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; chromeframe/21.0.1180.89)";
const char S99090[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; i-mobile I-Note Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99091[] = "Opera/9.80 (Android 3.2; Linux; Opera Tablet/ADR-X; U; en) Presto/2.7.81 Version/11.00";
const char S99092[] = "Opera/9.80 (Android 3.2.1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.7.81 Version/11.00";
const char S99093[] = "Mozilla/5.0 (Linux; U; Android 2.3; zh-cn; HTC A6390 Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S99094[] = "Mozilla/5.0 (Linux; U; Android 2.3; zh-cn; HTC A6390 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99095[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; ME171 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S99096[] = "Mozilla/5.0 (Linux; Android 4.0.4; DROID2 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S99097[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; Micromax A50 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99098[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.8.1.24) Gecko/X Thunderbird/2.0.0.24";
const char S99099[] = "nokiae65-1/UC Browser8.0.3.107/69/444";
const char S99100[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; chrome://navigator/locale/navigator.properties; rv:1.8.1.21) Gecko/X SecondLife/1.23.5.X (Second Life Release; default skin)";
const char S99101[] = "SAMSUNG-SGH-i200/1.0 (compatible; MSIE 4.01; Windows CE; Smartphone)/UC Browser7.2.2.51";
const char S99102[] = "iBrowser/2.4 (NokiaNXX;SymbianOS/9.1 Series60/3.0)";
const char S99103[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99104[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99105[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.27 (KHTML, like Gecko) Ubuntu/9.10 Chromium/12.0.711.0 Chrome/12.0.711.0 Safari/534.27";
const char S99106[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.30 (KHTML, like Gecko) Ubuntu/9.10 Chromium/12.0.742.53 Chrome/12.0.742.53 Safari/534.30";
const char S99107[] = "Mozilla/5.0 (Windows NT 6.0; rv:7.0.1) Gecko/X Yoono/1.8.16";
const char S99108[] = "NokiaC5-00/UCWEB8.5.0.163/28/800";
const char S99109[] = "NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/70/352";
const char S99110[] = "Mozilla/5.0 (Symbian3; U; en-us; NokiaX7-00)/UC Browser8.5.0.183/51/352/UCWEB Mobile";
const char S99111[] = "MQQBrowser/3.5/Adr (Linux; U; 4.0.4; zh-cn; GT-I9300 Build/IMM76D.I9300ZNBLH3;720*1280)";
const char S99112[] = "MQQBrowser/35 Mozilla/5.0 (iPhone 3GS; U; CPU iPhone OS 4_1 like Mac OS X; zh-cn) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B117 Safari/7534.48.3";
const char S99113[] = "Mozilla/5.0 (X11; U; Linux i686; de; rv:1.9.0.3) Gecko/X Ubuntu/8.04 Firefox/12.0";
const char S99114[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC Incredible S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S99115[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S99116[] = "nokian97-1/UC Browser7.9.0.102/69/352";
const char S99117[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46EX723; CC/GBR; en) Presto/2.7.61 Version/11.00";
const char S99118[] = "samsung-gt-s8600/UC Browser7.9.0.102/69/352";
const char S99119[] = "JUC(Linux;U;Android4.0.3;Zh_cn;HTC One V;480*800;)UCWEB7.8.0.95/139/444";
const char S99120[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.0 Mobile Safari/535.19 Silk-Accelerated=false";
const char S99121[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko, M5WebKit/1.0.1, M5HybridSDK/1.0.0) Version/5.0.4 Mobile/6.0.1 Mammoth/5.0.4";
const char S99122[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s5260) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S99123[] = "MQQBrowser/2.7 (NokiaC7-00;SymbianOS/9.1 Series60/3.0)";
const char S99124[] = "Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia3120classic) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S99125[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S99126[] = "iBrowser/Mini2.7 (NokiaC2-00/03.45)";
const char S99127[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; gt-s5603) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S99128[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3445; U; en) Presto/2.8.119 Version/11.10";
const char S99129[] = "Mozilla/5.0 (Linux; Android 4.0.4; JT-B1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S99130[] = "NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-05/UC Browser7.9.0.102/69/352";
const char S99131[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s8600) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S99132[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S 4G Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99133[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.53) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S99134[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5560i) UCBrowser8.2.1.144/69/352/UCWEB Mobile";
const char S99135[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.3.4.X (Second Life Release; default skin) Safari/533.3";
const char S99136[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LT15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S99137[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT862 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99138[] = "Mozilla/5.0 (Java; U; en-us; gt-s5230) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/405/UCWEB Mobile";
const char S99139[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; MT15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S99140[] = "NokiaC2-06/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac2-06) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S99141[] = "Android/1.0 (GT-P7310 IMM76D)";
const char S99142[] = "JUC(Linux;U;Android3.1;Zh_cn;GT-P7300;800*1280;)UCWEB7.8.0.95/139/355";
const char S99143[] = "Mozilla/5.0(Linux;U;Android3.1;Zh_cn;GT-P7300;800*1280;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S99144[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7100 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S99145[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; GT-P7100 Build/IMM76L; CyanogenMod-Rs Supremacy v2.0 CM9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S99146[] = "YahooMobile/1.0 (im; 1.8.3.13903); (Linux; U; Android 3.2; GT-P6200 Build/HONEYCOMB_MR2);";
const char S99147[] = "Android/1.0 (GT-P1010 FROYO)";
const char S99148[] = "Android/1.0 (GT-P6210 HTJ85B)";
const char S99149[] = "YahooMobile/1.0 (mail; 1.4.5.12736); (Linux; U; Android 2.2; GT-P1000 Build/FROYO);";
const char S99150[] = "YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 2.2; GT-P1000R Build/FROYO);";
const char S99151[] = "YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 2.2; GT-P1000M Build/FROYO);";
const char S99152[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3100, 36, 1.7.1)";
const char S99153[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3100, 36, 1.7.1)";
const char S99154[] = "Instagram 3.3.3 Android (15/4.0.3; 160dpi; 800x1232; samsung; GT-P5100; espresso10rf; espresso10; en_GB)";
const char S99155[] = "Instagram 3.3.3 Android (8/2.2; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; GT-P1000; en_GB)";
const char S99156[] = "Instagram 3.3.3 Android (10/2.3.3; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; gt-p1000; en_GB)";
const char S99157[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P6800 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S99158[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P6210 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S99159[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P6210 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S99160[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; en) Presto/2.9.168 Version/11.50";
const char S99161[] = "SonyEricssonU1UCBrowser/8.7.1.234/50/444/UCWEB";
const char S99162[] = "Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/MIUI) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S99163[] = "Mozilla/5.0 (Java; U; Id; tcl-c616) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S99164[] = "Mozilla/5.0 (Linux; Android 4.0.3; novo7_Aurora Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S99165[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SK17i Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99166[] = "Mozilla/5.0 (Java; U; en-us; lg-c105) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S99167[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V788D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99168[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; mot-ex128) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S99169[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; LG-E405 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S99170[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; XT862 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99171[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SAMSUNG-SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S99172[] = "MQQBrowser/20 (Linux; U; 2.3.4; zh-cn; HTC A810e Build/GRJ22;480*320)";
const char S99173[] = "JUC(Linux;U;Android2.2.1;Zh_cn;T301;600*800;)UCWEB7.8.0.95/139/444";
const char S99174[] = "Mozilla/5.0 (Linux; Android 4.0.3; iDxD8 3G Build/ICS) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S99175[] = "OneBrowser/3.1 (SAMSUNG GT-E2222)";
const char S99176[] = "Mozilla/5.0 (Linux; Android 4.2.1; Inspire 4G Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S99177[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I5510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S99178[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation Z710e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S99179[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S99180[] = "Mozilla/5.0 (Linux; Android 4.0.4; R8111 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S99181[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/MIUI) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S99182[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.15) Gecko/X Fedora/3.0.10-1.fc12 Thunderbird/3.0.10";
const char S99183[] = "Nokia5300/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5300) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S99184[] = "JUC (Linux; U; 2.3.5; zh-cn; MI-ONE Plus; 480*854) UCWEB8.7.4.225/145/800";
const char S99185[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S99186[] = "Mozilla/5.0 (Linux; Android 4.0.3; ME171 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S99187[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE Plus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99188[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MT27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99189[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/ Safari/533.3";
const char S99190[] = "JUC (Linux; U; 2.3.6; zh-cn; U8655-1; 320*480) UCWEB7.9.0.94/139/444";
const char S99191[] = "LG-KM900/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-km900) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S99192[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9228 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S99193[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S99194[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S99195[] = "NokiaC3-01/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac3-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S99196[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN80) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S99197[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5753e) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S99198[] = "Mozilla/5.0 (Linux; Android 4.0.4; AllviewAX2Frenzy Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S99199[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S99200[] = "HTC_P3470 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S99201[] = "MQQBrowser/26 (Linux; U; 2.3.6; zh-cn; HUAWEI C8810 Build/C8810V100R001C92B862;480*800)";
const char S99202[] = "sonyericssonck13i/UC Browser8.0.3.107/69/444";
const char S99203[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ru-ru; U8860 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99204[] = "SAMSUNG-GT-C3520/C3520JPKK3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S99205[] = "Mozilla/5.0 (Series40; Nokia2690/09.70; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.5.16.11";
const char S99206[] = "Mozilla/5.0 (Java; U; en-us; micromaxx600) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S99207[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:16.0) Gecko/X Firefox/16.0 slurp";
const char S99208[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Micromax A87 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99209[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; Karbonn_A18 Build/KarbonnA18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S99210[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-S5570 Build/FROYO; 240*320) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444";
const char S99211[] = "OneBrowser/3.0 (Nokia7210Supernova/07.2)";
const char S99212[] = "JUC(Linux;U;Android2.3.6;Zh_cn;GT-S6500D;320*480;)UCWEB7.8.0.95/139/351";
const char S99213[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I5801 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S99214[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-P350 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99215[] = "Nokia5300/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5300) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S99216[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT910 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99217[] = "IUC(U;iOS 6.0.1;Zh-cn;320*480;)/UCWEB8.2.1.132/42/997";
const char S99218[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.8.1.24) Gecko/X Thunderbird/2.0.0.24";
const char S99219[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia3110c) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S99220[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A760S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S99221[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lava-a9) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S99222[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; AD683G Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S99223[] = "JUC (Linux; U; 4.0.4; zh-cn; W60; 480*800) UCWEB7.9.0.94/139/444";
const char S99224[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S99225[] = "NokiaC5-00/UCWEB8.8.0.212/28/999";
const char S99226[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Galaxy Gio Build/JELLYBEAN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99227[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; iball Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S99228[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Spice Mi-435 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S99229[] = "Nokia205/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia205) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S99230[] = "Mozilla/5.0 (Linux; Android 4.1.1; AMOI N821 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S99231[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; 7260 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S99232[] = "Mozilla/5.0 (Linux; Android 4.0.4; FPT HD Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S99233[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C6603 Build/10.1.A.1.434) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S99234[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC One X Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S99235[] = "Mozilla/5.0 (Linux; Android 4.0.4; QW TB-9718D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S99236[] = "Mozilla/5.0 (Linux; Android 4.0.4; INFINITY_LOTUS Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S99237[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A800S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S99238[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-CN; ZTE-T U880 Build/FRG83G) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S99239[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Janetter/1.5.1";
const char S99240[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx600) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S99241[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S99242[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; ST27i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S99243[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 1 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S99244[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; ZTE U795 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S99245[] = "Mozilla/5.0 (Linux; Android 4.2.2; XT897 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99246[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; ST26i Build/11.2.A.0.21) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S99247[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6211 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S99248[] = "Mozilla/5.0 (Linux; Android 4.1.1; Xelio 7 pro Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S99249[] = "Mozilla/5.0 (Linux; Android 4.0.4; P-07D Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99250[] = "Mozilla/5.0 (Linux; Android 4.1.2; L-04E Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99251[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6810 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S99252[] = "Mozilla/5.0 (Linux; Android 4.1.1; GOCLEVER Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S99253[] = "GoogleEarth/5.1.3533.1731(Windows;Microsoft Windows Vista (Service Pack 1);en-US;kml:2.2;client:Free;type:default)";
const char S99254[] = "Mozilla/5.0 (Java; U; en-us; samsung gt-c3303) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S99255[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC PH39100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S99256[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZP900H Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99257[] = "Mozilla/5.0 (Linux; Android 4.0.4; MIDC407 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99258[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.0 Safari/537.36 Sleipnir/4.2.9";
const char S99259[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Butterfly Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S99260[] = "Mozilla/5.0 (Linux; Android 4.0.3; TAB-PROTAB2-IPS9 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S99261[] = "Mozilla/5.0 (Linux; Android 4.1.1; M030 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99262[] = "Mozilla/5.0 (Linux; Android 4.0.4; VS950 4G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S99263[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; A25 Build/GRK39F; 240*320) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444";
const char S99264[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Celkon A900 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99265[] = "OneBrowser/Mini1.1 (Android; Linux; Opera Mobi/27; U; en) Presto/2.4.18 Version/10.00";
const char S99266[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.2.2.259";
const char S99267[] = "Mozilla/5.0 (Linux; Android 4.0.4; F-04E Build/V14R31C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S99268[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (DROID4, 42, 1.9.1)";
const char S99269[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Spice Mi-425 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S99270[] = "Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Kem 432 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99271[] = "Mozilla/5.0 (Linux; Android 4.0.9; X21 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99272[] = "Mozilla/5.0 (Linux; Android 4.1.1; C1505 Build/11.3.A.0.47) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S99273[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC Desire C Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S99274[] = "Mozilla/5.0 (Linux; Android 4.0.4; SH530U Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99275[] = "Mozilla/5.0 (Linux; Android 4.1.1; MyPhone A888 Duo Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99276[] = "Mozilla/5.0 (Linux; Android 4.1.2; S100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99277[] = "Mozilla/5.0 (Linux; Android 4.2.2; F-06E Build/V19R47F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99278[] = "Mozilla/5.0 (Linux; Android 4.2.2; pl-pl; SAMSUNG GT-S7275 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19";
const char S99279[] = "nokian97-1/UC Browser8.0.3.107/69/444";
const char S99280[] = "BlackBerry9650/4.4.0.560 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S99281[] = "SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-in; sonyericssonj108i) UCBrowser8.2.0.132/69/452/UCWEB Mobile";
const char S99282[] = "JUC (Linux; U; 2.3.5; zh-cn; Lenovo_A288t; 320*480)";
const char S99283[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; I8 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99284[] = "OneBrowser/3.1 (Nokia5610d-1/10.00)";
const char S99285[] = "OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Fly F350 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99286[] = "Mozilla/5.0 (Java; U; en-us; micromaxq7) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S99287[] = "MQQBrowser/33 Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3";
const char S99288[] = "Mozilla/5.0 (Java; U; fa; samsung-gt-s5560i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile";
const char S99289[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A366t Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99290[] = "Mozilla/5.0 (Linux; Android 4.0.4; MID1060 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S99291[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; V9C Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S99292[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9105P Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99293[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:19.0) Gecko/X Firefox/19.0 SeaMonkey/2.16";
const char S99294[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; ST17i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S99295[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; U8655-1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S99296[] = "LG-GU285/v10a Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lggu280) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S99297[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Celkon_A_200) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S99298[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; tablet Fnac 7 Build/2.0.0 X-10:24) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S99299[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; ZTE V889S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99300[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; R811 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S99301[] = "Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Kem 402 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S99302[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LG-MS770) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S99303[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (MB865, 42, 1.9.1)";
const char S99304[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (SGH-T679, 42, 1.9.1)";
const char S99305[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S99306[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.6.5/X (like Mobile/8C148 Safari/6533.18.5)";
const char S99307[] = "Mozilla/5.0 (Linux; Android 4.0.4; SmartPad7503G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S99308[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax_P250(Funbook)) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S99309[] = "MQQBrowser/33 Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3";
const char S99310[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S99311[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26ii Build/6.1.A.2.45) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S99312[] = "samsung-mini7100/1.0 Linux/2.6.35.7 Android/2.3.5 Release/03.25.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile";
const char S99313[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq50) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S99314[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; TA-FONE_A34) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S99315[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; MT15i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S99316[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S99317[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9508 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99318[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN93) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S99319[] = "Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB A104.2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S99320[] = "Mozilla/5.0 (Linux; Android 4.0.4; Alpha GTR Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99321[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P925g Build/ICS) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99322[] = "Mozilla/5.0 (Linux; Android 4.2.1; MyPhone A919i Duo Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99323[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Iris 349 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S99324[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; U705T Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S99325[] = "Mozilla/5.0 (Linux; Android 2.3.5; N860 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S99326[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC T329t Build/JRO03H) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S99327[] = "Mozilla/5.0 (Linux; Android 4.1.1; Aqua Wonder Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99328[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC One X Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S99329[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; Lenovo S899t Build/S-2-20) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S99330[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A45 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S99331[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HTC T329t Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S99332[] = "Mozilla/5.0 (Linux; Android 4.1.1; Connect-2G-2.0 Build/HCL ME Tablet V2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S99333[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; V9C Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S99334[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo P700i Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S99335[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; IRIS 455) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S99336[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Z7C Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S99337[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; HTC Desire Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S99338[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; T-Mobile myTouch 3G Slide Build/FRG83D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S99339[] = "SonyEricssonG502/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssong502) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S99340[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5830 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S99341[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; HTC One Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S99342[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S99343[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; R800i Build/4.0.2.A.0.42) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S99344[] = "Mozilla/5.0 (Linux; Android 4.1.1; IQ442 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S99345[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-SU870 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S99346[] = "Mozilla/5.0 (Linux; Android 4.0.4; Fly_IQ450 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S99347[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A850S Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S99348[] = "Mozilla/5.0 (Linux; Android 4.0.4; SH-09D Build/S4060) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S99349[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; SGH-T989 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S99350[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5603) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S99351[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.1 Chrome/13.0.800.1 Safari/535.1";
const char S99352[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99353[] = "OneBrowser/3.1 (NokiaX6-00/40.0.002/sw_platform=S60;sw_platform_version=5.0;java_build_version=1.4.43)";
const char S99354[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; AN10DG3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S99355[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; LNV-Lenovo A790e Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S99356[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Spice Mi-355 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S99357[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-S5303 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/405";
const char S99358[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Lenovo A60 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S99359[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; Micromax A51 Build/REL_2.3.12) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S99360[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S99361[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; N350 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S99362[] = "Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Kem 462 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99363[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S99364[] = "Mozilla/5.0 (Linux; Android 4.0.4; Spice Mi-530 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99365[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; Lenovo S880 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S99366[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; P300(Funbook) Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S99367[] = "UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Q700) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S99368[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ALCATEL ONE TOUCH 930D Build/1.2.10.4) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S99369[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; IRIS_501 Build/LAVAIRIS501) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S99370[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; TECNO D3 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S99371[] = "SonyEricssonW910i/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw910i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S99372[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; ZTE-BLADE) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S99373[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; LT18i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S99374[] = "[FBAN/FB4A;FBAV/3.4;FBBV/X;FBDM/{density=1.0,width=320,height=480};FBLC/en_US;FBCR/MetroPCS;FBPN/com.facebook.katana;FBDV/LG-MS770;FBSV/4.0.4;]";
const char S99375[] = "Mozilla/5.0 (Linux; Android 4.0.4; VSD220 Build/IMM76D.UI23ED12_VSC) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S99376[] = "Mozilla/5.0 (Linux; Android 4.0.4; SGH-T959V Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S99377[] = "Mozilla/5.0 (Linux; Android 4.0.4; C725 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S99378[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Titanium_S1) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S99379[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus 7 Build/JWR66V) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S99380[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST25i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S99381[] = "Mozilla/5.0 (Linux; Android 4.0.4; F-05E Build/V17R40A) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S99382[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; A97i Build/CelkonA97i_Celkon) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S99383[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; AD683G Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S99384[] = "Mozilla/5.0 (Linux; Android 4.1.2; XOLO_X1000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S99385[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A90S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S99386[] = "[FBAN/FB4A-AZ;FBAV/2.0;FBDM/{density=1.5,width=1280,height=800};FBLC/en_US;FBCR/;FBPN/com.facebook.katana;FBDV/KFTT;FBSV/4.0.3;]";
const char S99387[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; A500 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99388[] = "UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; MID7012) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S99389[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GN305 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99390[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; Full AOSP on Noontec Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S99391[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; TAB7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S99392[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; EX'NOTE ? Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99393[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; MID8042) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S99394[] = "Mozilla/5.0 (X11; OpenBSD amd64) AppleWebKit/537.32 (KHTML, like Gecko) Chromium/25.0.1349.2 Chrome/25.0.1349.2 Safari/537.32";
const char S99395[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S6012 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S99396[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HTC 802t Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S99397[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-US; XCD35 Build/FRF91) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S99398[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-gb; LG-E615 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S99399[] = "Nokia202/2.0 (20.52) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291";
const char S99400[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-E435 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S99401[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9508 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S99402[] = "Mozilla/5.0 (Linux; Android 4.2.1; SKY_NANO Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S99403[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire U dual sim) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S99404[] = "Mozilla/5.0 (Linux; Android 4.2.2; 202F Build/V17R47F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S99405[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Celkon A107 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S99406[] = "Mozilla/5.0 (Linux; Android 3.1; AT100 Build/HMJ37) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S99407[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; PantechP9070 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S99408[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
const char S99409[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P506 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99410[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia J Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99411[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A356 Build/LenovoLePhone) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99412[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S99413[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S5570 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99414[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; X501_USA_Cricket Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S99415[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S880 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S99416[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-in; A54 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S99417[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; N400) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S99418[] = "Mozilla/5.0 (Linux; Android 4.2.2; M8 pro Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S99419[] = "Mozilla/5.0 (Linux; Android 4.1.2; Gpad_G2 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S99420[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.2; en-gb; HTC One Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S99421[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)";
const char S99422[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9192 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.20.1364.172 Mobile Safari/537.22";
const char S99423[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9082 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.20.1364.172 Mobile Safari/537.22";
const char S99424[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry Unknown; en) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.206 Mobile Safari/534.11+";
const char S99425[] = "OneBrowser/for Blackberry3.0.0 (BlackBerry9800/6.0.0.246)";
const char S99426[] = "OneBrowser/3.1 (SonyEricssonWT13i/R3AF010)";
const char S99427[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; LGL35G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S99428[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 XiaoMi/MiuiBrowser/1.0";
const char S99429[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; SPH-I800; SPRINT)";
const char S99430[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9260 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99431[] = "Nokia206/2.0 (03.58) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia206) UCBrowser8.3.1.161/70/352/UCWEB Mobile";
const char S99432[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N861 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S99433[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E988 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S99434[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; VT75C) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S99435[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.3.2.289";
const char S99436[] = "UCWEB/2.0 (Linux; U; Adr 4.2.2; zh-CN; GT-I9500) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile";
const char S99437[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LS670 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S99438[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S99439[] = "JUC (Linux; U; 4.2.2; zh-cn; GT-I9500; 1080*1920) UCWEB7.9.0.94/139/444";
const char S99440[] = "Mozilla/5.0 (Linux; Android 4.2.1; Q700 Build/XOLOQ700) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S99441[] = "MobileSafari/8536.25 CFNetwork/609.1.4 Darwin/12.4.0";
const char S99442[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon";
const char S99443[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; iris405 Build/LAVAIRIS405) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S99444[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; N400) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S99445[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LGL86C Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S99446[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; TECNO Q1 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/352";
const char S99447[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; hu-hu; DPS Dream 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S99448[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; T-Mobile myTouch Q Build/HuaweiU8730) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S99449[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-us; A5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S99450[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-I8190N Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S99451[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8262 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S99452[] = "Mozilla/5.0 (Linux; Android 4.2.1; SKY_NANO Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S99453[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; TA-FONE A39 HD Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33756";
const char S99454[] = "Mozilla/5.0 (Linux; Android 4.0.4; V96A Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S99455[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; A111 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S99456[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; AMAZE S250 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S99457[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; G928) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S99458[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R811 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S99459[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE C1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S99460[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; A111) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S99461[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus S Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/crespo";
const char S99462[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; INTEX CLOUD X1) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S99463[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHL21 Build/S2140) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S99464[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Tab 7.9 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S99465[] = "Instagram 4.1.2 (iPhone4,1; iPhone OS 6_1_3; en_US; en) AppleWebKit/420+";
const char S99466[] = "Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Kem 462 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S99467[] = "Mozilla/5.0 (Linux; U; Android 4.3; zh-cn; Nexus 4 Build/JWR66Y) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S99468[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I897 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/captivatemtd";
const char S99469[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE T82 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S99470[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I577 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99471[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5282 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S99472[] = "Mozilla/5.0 (Linux; Android 4.0.4; R8111 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S99473[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Titanium S1) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S99474[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; LG-MS870 Build/IMM76L) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S99475[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E470f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S99476[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L720 Build/JWR66V.S014) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99477[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9100 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S99478[] = "LG-GW525 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gw525 teleca) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S99479[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; GT-I9200 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S99480[] = "Mozilla/5.0 (Linux; Android 4.0.4; V790 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S99481[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mot-ex115) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S99482[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; HTC_Wildfire Build/FRG83D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S99483[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I777 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99484[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; TECNO_N3) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S99485[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.2.2; id-id; Vandroid Build/S5-V1.0-2013.07.18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99486[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC One X Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S99487[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Karbonn_A2 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S99488[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5660) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S99489[] = "Mozilla/5.0 (Linux;u;Android 2.3.7;zh-cn;) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Mobile Safari/533.1 (compatible; +http://www.baidu.com/search/spider.html)DAPPER-HOST-IP:108.162.199.56";
const char S99490[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A52) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S99491[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A25 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S99492[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL ONE TOUCH 985N) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S99493[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; LG-E612) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S99494[] = "Mozilla/5.0 (Linux; U; Android 4.3; zh-CN; Galaxy Nexus Build/JWR66Y) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S99495[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; HTC 802t Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S99496[] = "Nokia110/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia110) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S99497[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT28h Build/6.2.B.0.211) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S99498[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; zh-CN; MI 2) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S99499[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9506 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S99500[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC EVO 3D X515m Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S99501[] = "Mozilla/5.0 (Linux; Android 4.0.3; AN7HG3 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S99502[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A89) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S99503[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Aqua flash Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S99504[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S99505[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; C1504) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S99506[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9220 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S99507[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Karbonn A2+) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S99508[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E230K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S99509[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; iris502 Build/LAVAIRIS502) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S99510[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; SPH-D710 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S99511[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; IdeaTab A2107A-H Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S99512[] = "Mozilla/5.0 (Linux; Android 4.2.1; N7889 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S99513[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A820L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S99514[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M032 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S99515[] = "Mozilla/5.0 (Linux; Android 4.1.2; Connect-3G-2.0 Build/HCL) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S99516[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9100) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile";
const char S99517[] = "Mozilla/5.0 (Linux; Android 4.0.4; BNTV600 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S99518[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9502 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.5 Mobile Safari/533.1";
const char S99519[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST18i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S99520[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E973 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S99521[] = "Mozilla/5.0 (Linux; Android 4.1.1; MI 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S99522[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S99523[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5302 Build/12.0.A.2.254) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S99524[] = "LG-KP220/V01 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1 LG-KP220/V01 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S99525[] = "Mozilla/5.0 (Linux; U; Android 1.5; zh-cn; ME501 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S99526[] = "BlackBerry8100/4.5.0.174 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/142 UP.Browser/5.0.3.3";
const char S99527[] = "HTC_P6500 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S99528[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC P6500; OpVer 27.110.1.612";
const char S99529[] = "SAGEM-VF526/1.0/E1N_HF MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.410 (GUI)";
const char S99530[] = "SAGEM-VF526/1.0/EVN_HQ MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.416 (GUI)";
const char S99531[] = "SAGEM-VF526/1.0/EVN_HP MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.416 (GUI)";
const char S99532[] = "SAGEM-VF526/1.0/EVN_HL MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.412 (GUI)";
const char S99533[] = "SAGEM-VF526/1.0/EVN_HF MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.414 (GUI)";
const char S99534[] = "SAGEM-VF526/1.0/EVN_HH MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.407 (GUI)";
const char S99535[] = "MOT-MOTORAZRmaxxV3/98.30.80R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12093102) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S99536[] = "MOT-MOTORAZRmaxxV3/99.41.0BR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163234) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S99537[] = "MOT-MOTORAZRmaxxV3/98.30.72AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12003079) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S99538[] = "MOT-MOTORAZRmaxxV3/96.80.51R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11073071) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S99539[] = "MOT-MOTORAZRmaxxV3/96.A0.0CR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11073121) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S99540[] = "MOT-MOTORAZRmaxxV3/99.41.08R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163225) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S99541[] = "SAMSUNG-SGH-P260-ORANGE/P260BVGG5 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S99542[] = "SAMSUNG-SGH-P260/P260PKGK1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S99543[] = "SAMSUNG-M8800L/1.0 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.9 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S99544[] = "LG-GB160 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S99545[] = "SAMSUNG-SGH-E760/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S99546[] = "SAMSUNG-SGH-F275L/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S99547[] = "Fly_Q400/WapBrowserprofile/WIFIprofile/MIDP2.0Configuration/CLDC1.1";
const char S99548[] = "TNM-V70/1.0 Release/11.22.2008 Browser/Obigo2.0";
const char S99549[] = "SAMSUNG-GT-C3630C_CMCC/1.0 RTK-E/1.0 SHP/VPP-R5 Release/11.17.2009 Browser/NetFront3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1 SMM-MMS/1.2.0";
const char S99550[] = "SAMSUNG-SGH-F406/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S99551[] = "SAMSUNG-GT-B3410/1.0 RTK-E/1.0 DF/1.0 Release/22.12.2009 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S99552[] = "SAMSUNG-GT-B3410/1.0 RTK-E/1.0 DF/1.0 Release/22.12.2009 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S99553[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 240x320; MOT-Q-UMTS/APVER) Opera 8.65 [en]";
const char S99554[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 240x320; MOT-Q-UMTS/01.07.05R) Opera 8.65 [en]";
const char S99555[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 240x320; MOT-Q-UMTS/01.01.03R) Opera 8.65 [en]";
const char S99556[] = "SAMSUNG-E2510/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S99557[] = "HTCT9188_TD/1.0 WindowsMobile/6.5 CEOS/5.2 release/5.3 Opera/9.7 WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (WindowsMobile; PPC; Opera Mobi/35267; U; zh-CN; Presto/2.1.1)";
const char S99558[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTCT9188_TD/1.0 WindowsMobile/6.5 CEOS/5.2 release/5.3 Opera/9.7 WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1; Windows Phone 6.5.3.5)";
const char S99559[] = "SEC-SGHC140/1.0";
const char S99560[] = "Zte-tu236_TD/1.0 ThreadX/4.0b Larena/2.40 Release/4.15.2010 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S99561[] = "LGE-LG300/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S99562[] = "SAMSUNG-SGH-E330/1.0 UP.Browser/6.2.2.6 (GUI) MMP/1.0";
const char S99563[] = "SPH-M910";
const char S99564[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.2.2.51/69/352";
const char S99565[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.2.2.51/50/352";
const char S99566[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.2.2.51/27/352";
const char S99567[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.2.2.51/28/352";
const char S99568[] = "ZTE-R235/1.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S99569[] = "ZTE-F101/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S99570[] = "SCH-U550/1.0 NetFront/3.0.22.2.8 (GUI) MMP/2.0";
const char S99571[] = "Mozilla/5.0 (Linux; U; Android Cricket Eclair 2.1; en-us; ADR6200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S99572[] = "Mozilla/5.0 (Linux; U; Android Cricket Eclair 2.1; en-us; ADR6200 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S99573[] = "Nokia5310XpressMusic/2.0 (05.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5310xpressmusic/UC Browser7.7.1.88/69/444";
const char S99574[] = "NokiaC7-00/UC Browser7.8.0.95/50/355";
const char S99575[] = "Nokia6300";
const char S99576[] = "SEC-SGHX510/1.0";
const char S99577[] = "KDDI-KC3N UP.Browser/6.2.0.15.1.1 (GUI) MMP/2.0";
const char S99578[] = "NEXIAN NX-G911";
const char S99579[] = "KDDI-KC3V UP.Browser/6.2.0.15.1.1 (GUI) MMP/2.0";
const char S99580[] = "SonyEricssonC510/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonc510/UC Browser7.9.0.102/69/352";
const char S99581[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) QQBrowser/6.7.10246.201";
const char S99582[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.7.10246.201";
const char S99583[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0) QQBrowser/6.7.10246.201";
const char S99584[] = "HTC_P3490 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S99585[] = "Safari/6533.18.5 CFNetwork/454.9.7 Darwin/10.4.1 (i386) (MacBookPro7%2C1)";
const char S99586[] = "Safari/6533.18.5 CFNetwork/454.9.7 Darwin/10.4.1 (i386) (MacBook7%2C1)";
const char S99587[] = "IUC(U;iOS 4.1;Zh-cn;320*480;)/UCWEB7.5.0.66/41/997";
const char S99588[] = "IUC(U;iOS 4.2.1;Zh-cn;320*480;)/UCWEB7.5.0.66/41/997";
const char S99589[] = "IUC(U;iOS 3.1.3;Zh-cn;320*480;)/UCWEB7.5.0.66/39/800";
const char S99590[] = "Mozilla/5.0 (compatible; Konqueror/4.3) KHTML/4.3.5 (like Gecko) SUSE";
const char S99591[] = "Spice S7000/(X)S006/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S99592[] = "KDDI-SN3D UP.Browser/6.2_7.2.7.1.K.1.5.1.120 (GUI) MMP/2.0";
const char S99593[] = "Mozilla/5.0 (Windows NT 6.1; rv:2.1.1) Gecko/X Firefox/4.0.2pre Fennec/4.0.1";
const char S99594[] = "JUC(Linux;U;2.3.3;Zh_cn;Desire HD;480*800;)UCWEB7.8.1.96/139/800";
const char S99595[] = "samsung-gt-s3653w/UC Browser7.7.1.88/69/444";
const char S99596[] = "samsung-gt-s7233e/UC Browser7.9.0.102/69/352";
const char S99597[] = "NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac1-02/UC Browser7.9.0.102/69/352";
const char S99598[] = "LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  lg-gd510/UC Browser7.9.0.102/69/352";
const char S99599[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:7.0.1) Gecko/X Yoono/1.8.16";
const char S99600[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:8.0) Gecko/X Thunderbird/8.0 Lightning/1.0";
const char S99601[] = "MQQBrowser/28 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3";
const char S99602[] = "MQQBrowser/28 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3";
const char S99603[] = "sonyericssonk550i/UC Browser8.0.3.107/69/444";
const char S99604[] = "sonyericssonk550i/UC Browser8.0.3.107/70/352";
const char S99605[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.2.139/145/355";
const char S99606[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.2.139/145/355";
const char S99607[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S99608[] = "Dolphin http client/8.0.0(132) (Android)";
const char S99609[] = "Dolphin http client/8.0.1(133) (Android)";
const char S99610[] = "Dolphin http client/8.0.0 beta(131) (Android)";
const char S99611[] = "Mozilla/5.0 Opera/9.5 (KDDI-TS3S; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00";
const char S99612[] = "Mozilla/5.0 (Series40; Nokia6300/07.21; Profile/MIDP-2.0 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.4.0.34.8";
const char S99613[] = "Safari/7534.52.7 CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (iMac12%2C2)";
const char S99614[] = "Safari/7534.52.7 CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookPro5%2C4)";
const char S99615[] = "Safari/7534.52.7 CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookAir4%2C2)";
const char S99616[] = "Safari/7534.52.7 CFNetwork/520.2.5 Darwin/11.2.0 (i386) (MacBook2%2C1)";
const char S99617[] = "Safari/7534.52.7 CFNetwork/520.2.5 Darwin/11.2.0 (i386) (MacPro3%2C1)";
const char S99618[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID3 Build/5.5.1_84_D3G-55; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99619[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.1.19) Gecko/X SeaMonkey/2.0.14";
const char S99620[] = "micromaxq5/UC Browser7.9.0.102/69/352";
const char S99621[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6.1";
const char S99622[] = "Mozilla/5.0 (X11; U; CrOS i686 0.0.0; en-US) AppleWebKit/533.1 (KHTML, like Gecko) Chrome/5.0.321.0 Safari/533.1";
const char S99623[] = "MQQBrowser/2.4 (NokiaE5-00;SymbianOS/9.1 Series60/3.0)";
const char S99624[] = "GoogleEarth/6.0.3.2197(Macintosh;Mac OS X (10.5.8);en;kml:2.2;client:Free;type:default)";
const char S99625[] = "Mozilla/5.0 Opera/9.5 (KDDI-SH3K; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00";
const char S99626[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; T8788)";
const char S99627[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Surround)";
const char S99628[] = "Mozilla/5.0 (X11; U; Linux i686; pl-PL; rv:1.9.0.2) Gecko/X Ubuntu/9.25 (jaunty) Firefox/3.5";
const char S99629[] = "Mozilla/5.0 (X11; Linux i686; rv:5.0) Gecko/X Thunderbird/5.0";
const char S99630[] = "Mozilla/5.0 (Series40; Nokia5610d-1/10.00; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S99631[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:9.0) Gecko/X Firefox/9.0 TenFourFox/G3";
const char S99632[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 3GW100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99633[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; MID7C Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99634[] = "Mozilla/5.0 (Linux; Android 4.0.3; GEM7012 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S99635[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Evo Shift Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99636[] = "Mozilla/5.0 (Linux; Android 4.0.3; MID7108SC Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S99637[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; IdeaTab S2007A-D Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S99638[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.22) Gecko/X SeaMonkey/1.1.17 (Ubuntu-1.1.17+nobinonly-0ubuntu0.8.04.1)";
const char S99639[] = "JUC(Linux; U;2.2.2;zh_cn;Coolpad 5820;320*480;)UCWEB7.6.0.75/139/32521";
const char S99640[] = "Mozilla/4.0 (compatible; MSIE 4.5; Windows NT 5.1; .NET CLR 2.0.40607)";
const char S99641[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s3370) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S99642[] = "sonyericssonw395/UC Browser8.0.3.107/69/444";
const char S99643[] = "sgh-f480i/UC Browser8.0.3.107/69/444";
const char S99644[] = "nokiax7-00/UC Browser8.0.3.107/69/444  3gpp-gba";
const char S99645[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL55NX720; CC/USA; en) Presto/2.7.61 Version/11.00";
const char S99646[] = "MediaGo/2.2.0.223 (Windows NT 5.1) PlayStationStore/4.9.4.14625 PlaybackEngine/1.92.165.06140";
const char S99647[] = "MediaGo/2.2.0.223 (Windows NT 5.1) PlayStationStore/4.9.4.14625 PlaybackEngine/1.92.161.06140";
const char S99648[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.1b3pre) Gecko/X Shiretoko/3.1b3pre";
const char S99649[] = "MQQBrowser/2.7 (NokiaC6-00;SymbianOS/9.1 Series60/3.0)";
const char S99650[] = "gt-s5233s/UC Browser7.5.1.77/69/351";
const char S99651[] = "NokiaX3-00/5.0 (11.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax3-00/UC Browser7.7.1.88/69/444";
const char S99652[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.1b2) Gecko/X Firefox/3.1b2";
const char S99653[] = "LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  lg-gs500/UC Browser7.9.0.102/69/352";
const char S99654[] = "MQQBrowser/25 (Linux; U; 2.3.5; zh-cn; 5860 Build/2.3.025.P1.X.5860;480*800)";
const char S99655[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LS670 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99656[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5560i) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S99657[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:14.0) Gecko/X Firefox/14.0.1 slurp";
const char S99658[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; SK17i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99659[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; SCH-R720 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S99660[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I8530 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S99661[] = "Mozilla/5.0 (Java; U; en-in; micromaxq5) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S99662[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SPH-M820-BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S99663[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46CX520; CC/XGA; en) Presto/2.7.61 Version/11.00";
const char S99664[] = "Mozilla/5.0 (Java; U; en-in; samsung-gt-s3370) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S99665[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s8600) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S99666[] = "Mozilla/5.0 (Java; U; en-us; sonyericssonk530i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile";
const char S99667[] = "Mozilla/5.0 (Java; U; en-in; lg-t325) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S99668[] = "Mozilla/5.0 (Java; U; en-in; micromax) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S99669[] = "iBrowser/2.7 (Nokia5228;SymbianOS/9.1 Series60/3.0)";
const char S99670[] = "MQQBrowser/Mini2.8 (Nokia5130c-2/07.97)";
const char S99671[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.0 Chrome/13.0.800.0 Safari/535.1";
const char S99672[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; XCD35 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S99673[] = "dopod S900_CMCC/52.1941/WAP2.0 Profile/MID2.0 Configuration/CLDC1.1 (compatible;MSIE 4.01;Windows CE;PPC)";
const char S99674[] = "Mozilla/5.0 (Java; U; en-in; micromaxx600) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S99675[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s7233e) UCBrowser8.3.0.154/69/352/UCWEB Mobile";
const char S99676[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10";
const char S99677[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) rekonq/1.80 Safari/534.34";
const char S99678[] = "Mozilla/5.0 (Linux; Android 4.0.4; N-02E Build/AX) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S99679[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN93) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile";
const char S99680[] = "nokian8-00/UC Browser7.9.0.102/70/352  3gpp-gba";
const char S99681[] = "NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac1-02) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile";
const char S99682[] = "Mozilla/5.0 (Java; U; en-in; micromaxx600) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S99683[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; GT-I8320 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99684[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (DROID BIONIC, 36, 1.7.1)";
const char S99685[] = "Mozilla/5.0 (Linux; Android 4.2.1; Evo 3D CDMA Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S99686[] = "Nokia112/2.0 (03.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia112/UC Browser7.9.0.102/69/352";
const char S99687[] = "Mozilla/5.0 (Linux; Android 4.2.1; Ascend G300 Build/JOP40C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S99688[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99689[] = "SonyEricssonG502/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssong502) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S99690[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL32EX427; CC/COL; en) Presto/2.7.61 Version/11.00";
const char S99691[] = "Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:15.0) Gecko/X Firefox/15.0.1 TenFourFox/7450";
const char S99692[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Phoenix Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S99693[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;7 Trophy T8686;7.10;8862)";
const char S99694[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;7 Trophy;7.10;8862)";
const char S99695[] = "Android/1.0 (GT-P6200 HTJ85B)";
const char S99696[] = "Android/1.0 (GT-P6800 HTJ85B)";
const char S99697[] = "Android OS/12 samsung_GT-P7500";
const char S99698[] = "com.june.guessthemovie/1.234 (unknown,Android OS 12,samsung GT-P7500R-GT-P7500R,Scale/1.0000)";
const char S99699[] = "com.kakao.talk/Android Version/2.6.2 Model/GT-P7500M MNC/X";
const char S99700[] = "YahooMobileMessenger/1.0 (Android Messenger; 1.8.3) (espressowifi; samsung; GT-P3113; 4.1.1/JRO03C)";
const char S99701[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P5100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S99702[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P5100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S99703[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ph; GT-P5113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P5113, 34, 1.6.0)";
const char S99704[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P5113, 38, 1.8.1)";
const char S99705[] = "Instagram 3.2.0 Android (13/3.2; 160dpi; 800x1232; samsung; GT-P6800; GT-P6800; smdkc210; en_US)";
const char S99706[] = "Logos/2.2.10 (Linux; U; Android 4.2.1; es-US; GT-P7300 Build/cm_p5-userdebug 4.2.1 JOP40D eng.christian.X.X test-keys)";
const char S99707[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; tr-tr; GT-P7300 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S99708[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7300 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S99709[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P7300 Build/JDQ39)";
const char S99710[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6810 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S99711[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P6810 Build/JRO03R) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S99712[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P6210 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S99713[] = "MQQBrowser/2.8 (Nokia5530;SymbianOS/9.1 Series60/3.0)";
const char S99714[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99715[] = "Nokia7610Supernova/2.0 (06.51) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia7610supernova) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S99716[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssons500i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S99717[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; micromaxx78) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S99718[] = "Mozilla/5.0 (Linux; Android 4.1.2; A953 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S99719[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; iBuddy Connect Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S99720[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P999 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S99721[] = "Mozilla/5.0 (Linux; Android 4.0.4; L-01D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S99722[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99723[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; MK16i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S99724[] = "NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac1-02) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S99725[] = "SonyEricssonC510/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonc510) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S99726[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P920 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S99727[] = "Mozilla/5.0 (Linux; Android 4.0.3; TAB-PROTAB2-IPS9 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S99728[] = "Mozilla/5.0 (Linux; Android 4.0.4; IS12M Build/6.7.5-50_MR1-11) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S99729[] = "Mozilla/5.0 (Linux; Android 4.0.3; AT-AS43D2 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S99730[] = "Mozilla/5.0 (Linux; Android 4.1.2; F-02E Build/V16R46A) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S99731[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, -0400, us";
const char S99732[] = "Mozilla/5.0 (Linux; Android 4.0.4; MAXWEST Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S99733[] = "Mozilla/5.0 (Linux; Android 4.0.4; Xperia S Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S99734[] = "Opera/9.80 (Windows NT 4.0; U; en) Presto/2.2.15 Version/10.10";
const char S99735[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A360 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99736[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; MT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99737[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; micromaxq7) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S99738[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung gt-c3222) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile, SAMSUNG-GT-C3222/C3222DDJL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S99739[] = "Mozilla/5.0 (Linux; Android 4.0.4; ThL W1 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S99740[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Vision Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S99741[] = "Mozilla/5.0 (Linux; Android 4.1.2; Jiayu G3 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S99742[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9103 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S99743[] = "SonyEricssonC905/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; fa; sonyericssonc905) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S99744[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A770K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S99745[] = "Mozilla/5.0 (Linux; Android 4.1.2; IQ451 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S99746[] = "sonyericssonwt13i/UC Browser7.6.1.82/69/351";
const char S99747[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Rhyme S510b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99748[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8833 Build/HuaweiT8833) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S99749[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I777 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S99750[] = "Mozilla/5.0 (Linux; Android 4.1.1; Smartfren Tab 7 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S99751[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t300) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S99752[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t300) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S99753[] = "Opera/9.80 (Windows NT 5.0; U; en) Presto/2.2.15 Version/10.20";
const char S99754[] = "Spice M5454";
const char S99755[] = "samsung-gt-s3850/UC Browser7.8.0.95/69/352";
const char S99756[] = "MQQBrowser/2.6 (E66-1;SymbianOS/9.2 Series60/3.1)";
const char S99757[] = "Mozilla/5.0 (Linux; U; Android-4.0.3; en-us; Galaxy Nexus Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75 Mobile Safari/535.7";
const char S99758[] = "Opera/9.80 (Windows 98; U; en) Presto/2.10.289 Version/12.00";
const char S99759[] = "Nokia5330-1d/5.0 (06.88) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5330-1d) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S99760[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA6110) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile";
const char S99761[] = "Nokia7310/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia7310) UCBrowser8.3.1.161/69/355/UCWEB Mobile";
const char S99762[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC Desire C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99763[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; GT540 Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S99764[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-N8000 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S99765[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; LG-E405 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99766[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC Desire X Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S99767[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2; en-in; XCD35 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99768[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE50) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile";
const char S99769[] = "Nokia5610d-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5610d-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S99770[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5350) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S99771[] = "MQQBrowser/2.4 Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3";
const char S99772[] = "OneBrowser/3.1 (SAMSUNG-GT-B5310/B5310DDJD2)";
const char S99773[] = "JUC (Linux; U; 2.3.5; zh-cn; A56; 320*480) UCWEB7.9.0.94/139/444";
const char S99774[] = "JUC (Linux; U; 4.0.4; zh-cn; MI 1S; 480*854) UCWEB8.5.1.179/145/32504";
const char S99775[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaX7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile";
const char S99776[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99777[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9050 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99778[] = "Mozilla/5.0 (Java; U; en-us; lg-t310i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S99779[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8020 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99780[] = "MQQBrowser/3.3 (Nokia5230;SymbianOS/9.1 Series60/3.0)";
const char S99781[] = "Mozilla/5.0 (Linux; Android 4.0.4; 1688-0329-V9 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S99782[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11 slurp";
const char S99783[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S99784[] = "Mozilla/5.0 (Linux; Android 4.0.4; Celkon A 200 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S99785[] = "Mozilla/5.0 (S60V3; U; en-us; Nokia5730S-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile";
const char S99786[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S99787[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U705T Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S99788[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S99789[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID8048 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S99790[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; VM670 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000";
const char S99791[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; LG-VM696 Build/ZV5.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S99792[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; XCD35 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S99793[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; HUAWEI-U8850 Build/HuaweiU8850) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S99794[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; Spice Mi-425 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S99795[] = "Mozilla/5.0 (Linux; Android 4.0.4; Thunder S220 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S99796[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; MB860 Build/4.5.1A-129_OLY-145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S99797[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax A68 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S99798[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; R805 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S99799[] = "Mozilla/5.0 (Linux; Android 4.0.6; i9220 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S99800[] = "Mozilla/5.0 (Linux; Android 4.0.4; Woxter Tablet PC 97IPS Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S99801[] = "Mozilla/5.0 (Linux; Android 4.1.1; Starmobile FLIRT Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S99802[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-CN; GT-I9008L Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S99803[] = "MQQBrowser/42 Mozilla/5.0 (iPhone 4; U; CPU iPhone OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3 tencent(securitypay)";
const char S99804[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7000 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S99805[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; R801 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S99806[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F120K Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S99807[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; Lenovo A520 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S99808[] = "Mozilla/5.0 (Linux; Android 4.0.3; PMP3270B Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S99809[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-S5360 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S99810[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LT22i Build/6.1.1.B.1.54) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S99811[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax P600 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S99812[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; Lenovo A750 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S99813[] = "Mozilla/5.0 (Linux; Android 4.0.4; SPH-D700 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S99814[] = "Mozilla/5.0 (Linux; Android 4.0.4; tmn smart a15 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S99815[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC One S Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S99816[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P999 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S99817[] = "OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99818[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; LT29i Build/9.1.B.0.411) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S99819[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; HTC Salsa C510e Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S99820[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000";
const char S99821[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT30p Build/7.0.A.3.195) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S99822[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; LT26i Build/6.1.A.2.55) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S99823[] = "Mozilla/5.0 (Linux; Android 4.0.4; AZ210A Build/AZ210A_ICS_02.27I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S99824[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:21.0) Gecko/X Firefox/21.0 slurp";
const char S99825[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:22.0) Gecko/X Thunderbird/22.0";
const char S99826[] = "Mozilla/5.0 (Linux; Android 4.0.4; JY-G3 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S99827[] = "Mozilla/5.0 (Linux; Android 4.0.4; A8 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S99828[] = "Mozilla/5.0 (Linux; Android 4.1.2; Xperia Mini Pro Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S99829[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq6) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S99830[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5235) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba";
const char S99831[] = "Mozilla/5.0 (Linux; Android 4.1.2; 201M Build/9.8.2Q-34_SMJ-102) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S99832[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S99833[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S5310 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99834[] = "Mozilla/5.0 (Linux; Android 4.2.2; Novo7Fire Build/JDQ39B) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S99835[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A780L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99836[] = "Mozilla/5.0 (Linux; Android 4.0.4; ThL W5 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99837[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Aqua Star Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S99838[] = "Mozilla/5.0 (Linux; Android 4.2.1; ZTE V987 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99839[] = "JUC(Linux; U;2.3.4;zh_cn;Lenovo A68e;320*480;)UCWEB7.6.0.75/139/32396";
const char S99840[] = "SonyEricssonT700/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssont700) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S99841[] = "UCWEB/2.0(Symbian; U; S60 V3; en-US; Nokia 6210s) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S99842[] = "Mozilla/5.0 (Linux; Android 4.0.3; Xtreamer Joyz Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S99843[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S99844[] = "Mozilla/5.0 (Linux; Android 4.1.1; IRIS_501 Build/LAVAIRIS501) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99845[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGP321 Build/10.1.1.A.1.253) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S99846[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-P700 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S99847[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710a Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S99848[] = "Mozilla/5.0 (Linux; Android 4.2.1; Discovery Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99849[] = "Palm850/v0100 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S99850[] = "samsung-gt-c6712/UC Browser7.6.1.82/69/351";
const char S99851[] = "iBrowser/Mini2.8 (NokiaC2-01/11.10)";
const char S99852[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i809 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99853[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; HTC HD2 Build/IMM76I; CyanogenMod-9.0.0 RC0/AOKP/Desire C lib) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99854[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A7+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99855[] = "NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.5.0.185/83/352";
const char S99856[] = "JUC(Linux;U;Android4.1.1;Zh_cn;GT-I9300;720*1280;)UCWEB7.8.0.95/139/444";
const char S99857[] = "OneBrowser/3.1 (SAMSUNG-GT-C3752)";
const char S99858[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A520 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99859[] = "JUC (Linux; U; 2.3.4; zh-cn; WT19a; 320*480) UCWEB7.9.0.94/139/444";
const char S99860[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT-I5500) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S99861[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Status) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S99862[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; SGH-T959V Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S99863[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U701T Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S99864[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S99865[] = "Mozilla/5.0 (Linux; Android 4.0.4; i9430 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S99866[] = "Mozilla/5.0 (Linux; Android 4.0.4; Telenor_One_Touch_S Build/IceCreamSandwich) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S99867[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; Lenovo A366t Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S99868[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHW-M440S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S99869[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A830L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S99870[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (MB855, 42, 1.9.1)";
const char S99871[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ZTE_V791) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S99872[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; A10) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S99873[] = "OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99874[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M032 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S99875[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Incredible S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S99876[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A5) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S99877[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A91 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S99878[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8190N Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S99879[] = "Mozilla/5.0 (Linux; Android 2.3.7; 5860E Build/GWK74) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S99880[] = "SonyEricssonC510/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonc510) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S99881[] = "Mozilla/5.0 (Linux; Android 2.3.4; SK17i Build/4.0.2.A.0.62) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453";
const char S99882[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S99883[] = "Mozilla/5.0 (Linux; Android 4.0.4; PantechP8010 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S99884[] = "Mozilla/5.0 (Linux; Android 4.0.3; iball Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99885[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A780 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S99886[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; GT-S5360) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S99887[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Cloud Z5 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S99888[] = "Mozilla/5.0 (Linux; Android 4.0.4; U2 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S99889[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F100S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99890[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Micromax P362) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S99891[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-US; ZTE-BLADE Build/FRF91) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S99892[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S99893[] = "Mozilla/5.0 (Linux; Android 4.1.1; HTC Sensation XE with Beats Audio Z715e Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99894[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9128V Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S99895[] = "Mozilla/5.0 (Linux; Android 4.2.1; ALCATEL ONE TOUCH 8008X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99896[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A12) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S99897[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; vivo Xplay Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S99898[] = "Mozilla/5.0 (Linux; Android 4.0.4; Ctrl_V1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S99899[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC T329t Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S99900[] = "SonyEricssonC902/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonc902) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S99901[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-B5330) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S99902[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAP5000DUO Build/PrestigioPAP5000DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S99903[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP5570C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S99904[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S99905[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S99906[] = "SAMSUNG-GT-E2121B/E2121BXXKC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S99907[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOU701 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99908[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Karbonn A2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99909[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; LT22i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S99910[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  SonyEricssonP990i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S99911[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; SGH-T679) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S99912[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; ZTE N880E Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S99913[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; TM-7043XD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S99914[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328t Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S99915[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax P360 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S99916[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Karbonn A2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S99917[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A3) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S99918[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9300 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S99919[] = "Mozilla/5.0 (Linux; Android 4.0.4; Cloudfone Thrill 430d Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99920[] = "Mozilla/5.0 (Linux; Android 2.3.6; SGH-T679 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S99921[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HTC T528d Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S99922[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; Karbonn_A18 Build/KarbonnA18) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S99923[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Mobile Safari/535.19 Silk-Accelerated=false";
const char S99924[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A2) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S99925[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A62 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S99926[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST27i Build/6.1.1.B.1.10) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S99927[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; Lenovo A750 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S99928[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT-I5503) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S99929[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; GT-B7510) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S99930[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; A90 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S99931[] = "MQQBrowser/3.4/Adr (Linux; U; 4.2.2; zh-cn; GT-I9500 Build/JDQ39.I9500ZCUBMG1;1080*1920)";
const char S99932[] = "Mozilla/5.0 (Linux; Android 4.0.4; U8800 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S99933[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-P930 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S99934[] = "Mozilla/5.0 (Linux; U; Android 4.4; en-us ; SPHS on Hsdroid Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33896";
const char S99935[] = "OneBrowser/3.1 (SonyEricssonW20i/R7DA062)";
const char S99936[] = "Opera/9.21 (Windows NT 6.2; U; en)";
const char S99937[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia5235) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile  3gpp-gba";
const char S99938[] = "Mozilla/5.0 (Linux; Android 4.1.2; MI-ONE C1 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S99939[] = "OneBrowser/3.1 (Nokia3120classic/10.00)";
const char S99940[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Ctrl_V3) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S99941[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.1; id-id; T2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99942[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC One V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S99943[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I879 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S99944[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GN800 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S99945[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S5303 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S99946[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Spice Mi-502 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S99947[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I727R Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S99948[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i929 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S99949[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-us; Micromax A65 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S99950[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; A12 Build/RheaTK) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S99951[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; TECNO F7 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/352";
const char S99952[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; IM-A760S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S99953[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; ME865 Build/6.7.2_GC-180-EDS-20) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S99954[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; C1604) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S99955[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Desire Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S99956[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; AD683G Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S99957[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus S Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S99958[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo P700i Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S99959[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E16i) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S99960[] = "Mozilla/5.0 (Linux; Android 4.0.4; Polaris Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S99961[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; Lenovo S880 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S99962[] = "Mozilla/5.0 (Linux; Android 3.2; Dell Streak 7 Build/HTJ85B) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541";
const char S99963[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S99964[] = "SAMSUNG-GT-E2232/1.0 RTK-E/1.0 MMP/1.0 Release/02.11.2011 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S99965[] = "Mozilla/5.0 (Linux; Android 4.0.3; Micromax P350 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S99966[] = "MQQBrowser/28 Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3";
const char S99967[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; vivo S11 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99968[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S99969[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HUAWEI Y300-0000 Build/HuaweiY300-0000) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S99970[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax P560 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S99971[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-MS870 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99972[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC One X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.2/552,2.0.2.552,2013-05-22 23:05, +0100, gb";
const char S99973[] = "Mozilla/5.0 (Linux; Android 4.0.4; Magic Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S99974[] = "Mozilla/5.0 (Linux; Android 4.2.2; P-03E Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S99975[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9128 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S99976[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6280) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile TRUSTED/1.0";
const char S99977[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I8160) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S99978[] = "Mozilla/5.0 (Linux; Android 4.0; Swipe_Fablet_F1 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S99979[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; IRIS355) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S99980[] = "UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; GT-I9000) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S99981[] = "Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S99982[] = "MQQBrowser/44 Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3";
const char S99983[] = "SAMSUNG-GT-S3778/1.0 RTK-E/1.0 DF3G/1.0 Release/04.04.2011 Browser/NetFront4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S99984[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; PTT1026 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99985[] = "OneBrowser/3.1 (LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S99986[] = "OneBrowser/3.1 (Nokia303/14.60)";
const char S99987[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99988[] = "TOUCH87 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3";
const char S99989[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; A54 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99990[] = "MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; JX0189 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S99991[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; PB-S3-77 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99992[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax7-00) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba";
const char S99993[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9100G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99994[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I8258 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S99995[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S99996[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; HTC_Ruby) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S99997[] = "UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; LG-P705) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S99998[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; WT19i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S99999[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M040 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S100000[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-E450 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S100001[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-us; Karbonn A2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S100002[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MT27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S100003[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-us; LG-E739 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S100004[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; Micromax P350 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S100005[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.3; en-us; Karbonn A6 Build/MocorDroid4.0.3) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S100006[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Incredible 2 Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100007[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; iBall Slide 3G 7334 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S100008[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A90S) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S100009[] = "NokiaC3-00/5.0 (08.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291";
const char S100010[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; MT25i Build/4.1.B.0.631) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300";
const char S100011[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/5.0 (compatible; Googlebot/20.2497; U; en) Presto/2.5.25;c2VvLWNyYXdsZXI=";
const char S100012[] = "Mozilla/5.0 (Windows;) HealthSTAT Crawler 1.0 For problems contact: crawler@healthline.com;c2VvLWNyYXdsZXI=";
const char S100013[] = "Googlebot/2.1 (http://www.googlebot.com/bot.html; MSIE; Windows; Trident/4.0; FunWebProducts; GTB6.5; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.;c2VvLWNyYXdsZXI=";
const char S100014[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.5; aggregator:Spinn3r (Spinn3r 3.5 / Crawler); http://spinn3r.com/robot) Gecko/X;c2VvLWNyYXdsZXI=";
const char S100015[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_4010D) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S100016[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; N320 Build/LAVALAVA_N320) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S100017[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S100018[] = "Mozilla/5.0 (Linux; Android 4.0.3; E350 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S100019[] = "UCWEB/2.0 (Linux; U; Adr 4.2.2; en-US; Cloud_X3) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S100020[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; X10i Build/3.0.1.G.0.75) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S100021[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; TS7 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100022[] = "Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S100023[] = "Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311";
const char S100024[] = "Instagram 4.1.2 Android (10/2.3.5; 240dpi; 480x800; HTC/sprint; PC36100; supersonic; supersonic; en_US)";
const char S100025[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 992D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S100026[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5302 Build/12.0.A.2.245) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S100027[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC_Desire_C Build/IML74K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S100028[] = "Mozilla/5.0 (Linux; Android 4.3; GT-I9300 Build/JSS15J) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S100029[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; R815T Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S100030[] = "Mozilla/5.0 (Linux; Android 4.0.4; F-04E Build/V14R31C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100031[] = "Mozilla/5.0 (Linux; Android 4.0.4; 201M Build/7.7.1Q-164_SMJ-165) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100032[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A800S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100033[] = "UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-I9300) U2/1.0.0 UCBrowser/9.3.0.321 Mobile";
const char S100034[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-R830 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:38, us";
const char S100035[] = "Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Lai 502 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S100036[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Xperia Neo V Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100037[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; SAMSUNG-SGH-I997) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S100038[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ELF-II Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S100039[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-I9100 Build/IML74K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S100040[] = "Mozilla/5.0 (Linux; Android 4.1.1; A9 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S100041[] = "Mozilla/5.0 (X11; U; Darwin Power Macintosh; en-US; rv:1.8.0.12) Gecko/X Firefox/1.5.0.12 Fink Community Edition";
const char S100042[] = "Nokia310/2.0 (07.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia310) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S100043[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer Prime TF201 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);";
const char S100044[] = "Mozilla/5.0 (Linux; Android 4.1.1; SOWA_S6600 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S100045[] = "Nokia206/2.0 (03.58) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia206) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/84/352/UCWEB Mobile";
const char S100046[] = "Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabA2207A-H Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S100047[] = "iBrowser/2.0,Lava-Discover132/2.0";
const char S100048[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; iris405) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S100049[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; GSmart Maya M1 v2 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100050[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT26i Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S100051[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; C5155 Build/IML77) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S100052[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; A10 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S100053[] = "Mozilla/5.0 (Linux; Android 4.1.2; LT28h Build/6.2.B.0.211) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S100054[] = "LinkedInBlackBerry/10.1.4 (1) Q5 SQR100-2/10.1.0.4633 (Research In Motion;RIM BlackBerry Device;Q5;SQR100-2;1)";
const char S100055[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; LG-P990 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/p990";
const char S100056[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5830 Build/FROYO) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S100057[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R813T Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S100058[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST18i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S100059[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; TECNO_P3 Build/master) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S100060[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9128 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S100061[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; zh-CN; MI 1S) U2/1.0.0 UCBrowser/9.2.4.329 U2/1.0.0 Mobile";
const char S100062[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC ChaCha A810e Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S100063[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5236) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile";
const char S100064[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; MT11i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S100065[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A26 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S100066[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A7*) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S100067[] = "Mozilla/5.0 (Windows NT 5.1; rv:24.0) Gecko/X Firefox/24.0 slurp";
const char S100068[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Q800) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S100069[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; SCH-I589) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S100070[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; KFTT Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S100071[] = "Mozilla/5.0 (Linux; Android 4.2.1; IQ450_Quattro Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100072[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LG-LU6200 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100073[] = "Mozilla/5.0 (Linux; Android 4.2.2; Vivid 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100074[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F240 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100075[] = "Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 7 Build/1.0.3 X-12:51) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S100076[] = "Mozilla/5.0 (Linux; Android 4.3; SCH-R530M Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100077[] = "Mozilla/5.0 (Linux; Android 4.0.4; NEO_N002 Build/NeoNEO_N002) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100078[] = "Mozilla/5.0 (Linux; Android 4.0.3; PAD703 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100079[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; XT912 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100080[] = "Mozilla/5.0 (Linux; Android 4.2.1; UMI X1S Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100081[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A840S Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100082[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST15i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100083[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Aqua Wonder Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100084[] = "UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-I8262) U2/1.0.0 UCBrowser/9.3.0.321 Mobile";
const char S100085[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-F100S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100086[] = "Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI U9510E Build/HuaweiU9510E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100087[] = "Mozilla/5.0 (Linux; Android 4.1.2; A27 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100088[] = "Mozilla/5.0 (Linux; Android 4.1.2; SHV-E270L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100089[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A850L Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100090[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE Grand Era Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100091[] = "Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S100092[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo A390_ROW Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100093[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; ST15i Build/4.0.2.A.0.84) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100094[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Spice Mi-435 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100095[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S100096[] = "Mozilla/5.0 (Linux; Android 4.0.4; Fly_IQ442 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100097[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X; zh-CN) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 UCBrowser/9.1.0.287 Mobile";
const char S100098[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LVW Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S100099[] = "Nokia206/2.0 (03.59) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia206) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S100100[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S100101[] = "Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S100102[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S100103[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9205 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780";
const char S100104[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S100105[] = "Mozilla/4.0 SonyEricssonW910iv/R1EA SEMC/NewsReader/1.0";
const char S100106[] = "SonyEricssonW910i/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S100107[] = "TIANYU  B832";
const char S100108[] = "TIANYU-KTOUCH/B832/MIDP2.0/CLDC1.1/Screen-240x320";
const char S100109[] = "Mozilla/5.0 (Linux; U; Android RCMagicv2.1_v6.0; es-es; HTC Magic Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S100110[] = "LG-GD900/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S100111[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SonyEricssonX1i/R3AA Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S100112[] = "X1i Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S100113[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch_HD/2.0; Windows Phone 6.5)";
const char S100114[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch_HD_T8282; Windows Phone 6.5)";
const char S100115[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch_HD_T8282; Windows Phone 6.5.3.5)";
const char S100116[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Touch_HD_T8282; Windows Phone 6.5)";
const char S100117[] = "MOT-C650/0B.D2.23R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S100118[] = "Alcatel-OT-C701A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S100119[] = "Alcatel-OT-C701/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S100120[] = "ZTE-G-N290-BASE-P107A2V1.0.0 Obigo/Q03";
const char S100121[] = "Nokia6600/1.0 (5.53.0) SymbianOS/7.0s Series60/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.4.0.65/27/351";
const char S100122[] = "LG-KP15xQ Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S100123[] = "CoolPadF600_TD/CMCC ThreadX/1.0 LA2KPH/SW2.0 Release/2010.08.10 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S100124[] = "SAMSUNG-SGH-A517/A517UCGG5 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S100125[] = "LG-GS190 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S100126[] = "Nokia6202c_CMCC/2.0 (04.51) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S100127[] = "Samsung-SCHR100 Infraware-Polaris/6.1 MMP/2.0";
const char S100128[] = "SAMSUNG-SGH-T636/UEHI1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S100129[] = "LGE-PLS225/1.0 UP.Browser/6.2.3.7.e.1.101 (GUI) MMP/2.0";
const char S100130[] = "Mozilla/5.0 (Series40; Nokia7230/10.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.6";
const char S100131[] = "Mozilla/5.0 (Linux; U; Android Cricket Froyo 2.2; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100132[] = "SoftBank/1.0/820SC/SCJ002 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S100133[] = "Mozilla/4.08 (820SC;SoftBank) NetFront/3.3";
const char S100134[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; in-id; NX-A892 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100135[] = "Mozilla/5.0 (Symbian/3; Series60/5.3 Nokia600/111.010.1009; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.4.1.12 Mobile Safari/533.4 3gpp-gba";
const char S100136[] = "C5600/001.00 ACS-NetFront/3.2";
const char S100137[] = "ZTE-F188/1.0 ACS-NF/3.4 QTV2.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S100138[] = "ZTE-F188-CSL/1.0 ACS-NF/3.4 QTV2.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S100139[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-tw; Vibo-A688 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S100140[] = "SAMSUNG-S3600/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S100141[] = "BlackBerry7250/4.1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105";
const char S100142[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG/X01SC";
const char S100143[] = "LG-GB175/V01 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S100144[] = "Mot-WX288/1.0 ObigoInternetBrowser/Q03C";
const char S100145[] = "Nokia3660/1.0 (5.08) SymbianOS/6.1 Series60/1.2 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S100146[] = "Nokia3660/1.0 (4.57) SymbianOS/6.1 Series60/0.9 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S100147[] = "Mozilla/4.0 (MobilePhone SCP-8400/US/1.0) NetFront/3.3 MMP/2.0";
const char S100148[] = "NokiaC5-00/UC Browser7.4.0.65/28/351";
const char S100149[] = "SAMSUNG-SGH-i900/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65";
const char S100150[] = "HTC_Touch2_T3333 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65";
const char S100151[] = "HTC_HD2_T8585 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.6.1.82";
const char S100152[] = "HTC_Touch2_T3333 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.6.1.82";
const char S100153[] = "Nokia6600/1.0 (5.53.0) SymbianOS/7.0s Series60/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.7.1.88/27/444";
const char S100154[] = "SEC-SGHC130/1.0";
const char S100155[] = "BlackBerry7250/4.0.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S100156[] = "DoCoMo/2.0 P900i(c100;TB;W24H11)";
const char S100157[] = "COCOON/R01 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S100158[] = "TIANYU-KTOUCH/H777/MIDP2.0/CLDC1.1/Screen-320X240";
const char S100159[] = "Mac OS X/10.7.2 (11C74); ExchangeWebServices/2.0 (123); Mail/5.1 (1251)";
const char S100160[] = "SonyEricssonU8i";
const char S100161[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Touchpad Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100162[] = "MQQBrowser/2.0 (NokiaNXX;SymbianOS/9.1 Series60/3.0)";
const char S100163[] = "BlackBerry9000/4.0.0.248 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/159/444";
const char S100164[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:2.0.1) Gecko/X Postbox/2.5.3";
const char S100165[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; U8220 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100166[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; U8220/Pulse Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100167[] = "PPC; 240x320; HTC_P3450/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65";
const char S100168[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1) Opera 7.50  [en]";
const char S100169[] = "Mozilla/5.0 (Windows; U; Windows NT 2.0; en-US; rv:1.8.1.6) Gecko/X Firefox/2.0.0.6";
const char S100170[] = "Mozilla/5.0 (Windows; U; WinNT4.0; pl; rv:1.8.1.20) Gecko/X Firefox/2.0.0.20";
const char S100171[] = "nokian73-1/UC Browser7.6.1.82/69/352";
const char S100172[] = "MacOutlook/14.13.0.X (Intel Mac OS X 10.5.8)";
const char S100173[] = "samsung-gt-s3850/UCWEB8.0.3.99/70/350";
const char S100174[] = "SonyEricssonC902/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonc902/UC Browser7.9.0.102/69/352";
const char S100175[] = "MacOutlook/14.13.0.X (Intel Mac OS X 10.7.2)";
const char S100176[] = "SonyEricssonT715/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssont715/UC Browser8.0.3.107/69/444";
const char S100177[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; mwp6985)";
const char S100178[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:9.0.1) Gecko/X Firefox/9.0.1 CometBrowser/9.0.1";
const char S100179[] = "AppleCoreMedia/1.0.0.8A293 (iPod; U; CPU OS 4_0 like Mac OS X; en_us)";
const char S100180[] = "AppleCoreMedia/1.0.0.8A400 (iPod; U; CPU OS 4_0_2 like Mac OS X; en_us)";
const char S100181[] = "samsung-gt-s3850/UC Browser7.9.0.102/69/444";
const char S100182[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6400L Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100183[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; http://device.sprintpcs.com/HTC/APA9292KT/X.rdf; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16";
const char S100184[] = "Mozilla/5.0 (SaferSurf rv:9.0) Gecko/X Firefox/9.0";
const char S100185[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.19pre) Gecko/X Prism zdesktop/7.1.4";
const char S100186[] = "samsung-gt-s5263/UC Browser7.8.0.95/69/351";
const char S100187[] = "MQQBrowser/33 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3";
const char S100188[] = "MQQBrowser/33 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3";
const char S100189[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S100190[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; LG; LG-C900)";
const char S100191[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; LG; LG-C900k)";
const char S100192[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; LG; LG-C900B)";
const char S100193[] = "Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.7.5) Gecko/X Netscape/8.1.3";
const char S100194[] = "Nokia5200/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia5200/UC Browser7.9.0.102/69/352";
const char S100195[] = "JUC(Linux;U;Android2.2.2;Zh_cn;LG-P970;480*800;)UCWEB7.8.0.95/139/351";
const char S100196[] = "JUC(Linux;U;2.3.4;Zh_cn;HTC Z710e;540*960;)UCWEB7.6.0.75/139/32035";
const char S100197[] = "Mozilla/5.0 (Windows NT 5.1; rv:5.0) Gecko/X Firefox/5.0 Kylo/1.0.1.76141";
const char S100198[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:7.0) Gecko/X Thunderbird/7.0";
const char S100199[] = "nokiae7-00/UC Browser7.9.0.102/69/352  3gpp-gba";
const char S100200[] = "Mozilla/5.0 Opera/9.5 (KDDI-SN3T; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00";
const char S100201[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; ZTE Roamer Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100202[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; P300(Funbook) Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S100203[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; ST15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S100204[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Vantium Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100205[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-B5512 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100206[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy5 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100207[] = "MQQBrowser/3.5/ZTE-TU880_TD/1.0 Linux/2.6.32 Android/2.2 Release/5.25.2011 Browser/AppleWebKit533.1";
const char S100208[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100209[] = "Opera/9.80 (X11; Linux i686; U; Linux Mint; en) Presto/2.2.15 Version/10.00";
const char S100210[] = "samsung-gt-s3653/UC Browser7.8.0.95/69/444";
const char S100211[] = "LG-GW300/V100 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1  lg-gw300/UC Browser7.9.0.102/69/352";
const char S100212[] = "Mozilla/5.0 (Windows NT 5.1; rv:7.0.1) Gecko/X Firefox/7.0.1 CometBrowser/7.0.1";
const char S100213[] = "Mozilla/5.0 (830SH;SoftBank;SNX) NetFront/3.4";
const char S100214[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:8.0.1) Gecko/X Firefox/8.0.1 SeaMonkey/2.5";
const char S100215[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA6210 Navigator)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S100216[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s5260) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S100217[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100218[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.0.9) Gecko/X Firefox/3.0.9 Wyzo/3.0.3";
const char S100219[] = "Mozilla/5.0 (Windows NT 6.0; rv:2.0.1) Gecko/X Firefox/4.0.1 slurp";
const char S100220[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus 7 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S100221[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Nexus 7 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100222[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Nexus 7 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S100223[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Nexus 7 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100224[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.0.0.0 Chrome/12.0.742.91 Safari/534.30";
const char S100225[] = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:8.0) Gecko/X Thunderbird/8.0 OracleBeehiveExtension/1.0.0.4-OracleInternal Lightning/1.0";
const char S100226[] = "Nokia5300/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia5300) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S100227[] = "samsung-gt-s5620/UC Browser7.4.0.65/69/351";
const char S100228[] = "samsung-gt-s3653/UC Browser7.5.1.77/69/352";
const char S100229[] = "nokiac7-00/UC Browser7.5.1.77/69/351  3gpp-gba";
const char S100230[] = "curl/7.7.2 (powerpc-apple-darwin6.0) libcurl 7.7.2 (OpenSSL 0.9.6b)";
const char S100231[] = "samsung-gt-s5620/UCWEB8.0.3.99/70/350";
const char S100232[] = "Mozilla/5.0 (Series40; NokiaX3-02/05.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.4.0.34.8";
const char S100233[] = "Mozilla/5.0 (Series40; NokiaC2-06/06.51; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15";
const char S100234[] = "nokian70/UC Browser7.9.0.102/69/352";
const char S100235[] = "nokian70-1/UC Browser7.9.0.102/70/352";
const char S100236[] = "LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  lg-kp500 teleca/UC Browser8.0.3.107/69/444";
const char S100237[] = "NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac1-02/UC Browser8.0.3.107/69/444";
const char S100238[] = "Mozilla/5.0 (Java; U; en-in; micromaxq5) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S100239[] = "Mozilla/5.0 (S60V3; U; en-us; Nokia6220c-1)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S100240[] = "SonyEricssonU10i/R7BA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonu10i) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S100241[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V889D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100242[] = "Opera/9.80 (BREW; Opera Mini/5.1/27.2239; U; en) Presto/2.8.119 400X240 LG VN270";
const char S100243[] = "SonyEricssonW995/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonw995/UC Browser7.9.0.102/69/352";
const char S100244[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA6110)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S100245[] = "Mozilla/5.0 (Java; U; en-us; lg-t300) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S100246[] = "SonyEricssonU10i/R7BA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonu10i) UCBrowser8.3.0.154/69/352/UCWEB Mobile";
const char S100247[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; MTS-SP100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S100248[] = "MQQBrowser/2.0 Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S100249[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N8010 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0000, gb";
const char S100250[] = "Mozilla/5.0 (Windows; U; Win95; de; rv:1.8.0.12) Gecko/X Firefox/1.5.0.12";
const char S100251[] = "nokia5800d-1/UC Browser7.5.1.77/69/352";
const char S100252[] = "Nokia302/5.0 (14.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia302/UC Browser7.9.0.102/69/352";
const char S100253[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOR801 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100254[] = "Mozilla/5.0 (Java; U; en-us; micromaxx78) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S100255[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; de-de; AN7G2DTE Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100256[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:2.0) Gecko/X Firefox/4.0 Kylo/1.0.1.76141";
const char S100257[] = "JUC (Linux; U; 2.2.2; zh-cn; Lenovo_A390e; 320*480) UCWEB7.9.3.103/139/32590";
const char S100258[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03O) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75 Mobile Safari/535.7";
const char S100259[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC EVO 3D X515m Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S100260[] = "Nokia303/5.0 (13.47) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia303) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S100261[] = "Netscape 4.8 (Windows Vista): Mozilla/4.8 (Windows NT 6.0; U)";
const char S100262[] = "Opera/9.80 (Windows NT 6.1)";
const char S100263[] = "samsung-gt-s8600/UC Browser8.0.3.107/69/444";
const char S100264[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A300 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100265[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100266[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S100267[] = "SAMSUNG-SGH-I617/1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 320x240)/UC Browser7.2.2.51";
const char S100268[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth Pro/6.0.2.2074 Safari/532.4";
const char S100269[] = "Nokia5610d-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia5610d-1) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S100270[] = "Mozilla/5.0 (Windows NT 4.0; rv:10.0.2) Gecko/X Firefox/10.0.2 SeaMonkey/2.7.2";
const char S100271[] = "User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/3.1; IEMobile/7.0)";
const char S100272[] = "Mozilla/5.0 (Java; U; en-us; micromaxq7) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S100273[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8500) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S100274[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5260) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S100275[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;OMNIA7;7.10;8862)";
const char S100276[] = "Pandora/null Android/3.1 GT-P7510";
const char S100277[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P5100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.2.139/145/355";
const char S100278[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P5100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S100279[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-P5100 Build/IML74K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S100280[] = "WhatsApp/2.8.5023 Android/2.3.3 Device/samsung-GT-P1000";
const char S100281[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.2.2.259";
const char S100282[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P7500 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S100283[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P6201 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S100284[] = "Mozilla/5.0 (X11; Linux mips64; rv:2.0.1) Gecko/X Firefox/4.0.1";
const char S100285[] = "Nokia6300/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia6300) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S100286[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6352 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100287[] = "MQQBrowser/3.1 (NokiaC6-01;SymbianOS/9.1 Series60/3.0)";
const char S100288[] = "Mozilla/5.0 (Linux; Android 4.0.3; ZTE-Mimosa X Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S100289[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S100290[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; Karbonn_A18 Build/KarbonnA18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S100291[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; XT907 Build/9.8.1Q_27-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S100292[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-N7108 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S100293[] = "Mozilla/5.0 (Java; U; en-in; jblend) UCBrowser8.3.0.154/69/444/UCWEB Mobile/HS-T39_TD/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0 Config/CLDC-1.1,/HS-T39_TD/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0 Config/CLDC-1.1";
const char S100294[] = "JUC (Linux; U; 4.0.4; zh-cn; U30GT-H; 1280*752) UCWEB1.0.1.113/147/800";
const char S100295[] = "Mozilla/5.0 (Linux; Android 4.0.4; eZee'Tab802 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S100296[] = "JUC (Linux; U; 2.3.6; zh-cn; Aqua_S; 320*480) UCWEB7.9.0.94/139/444";
const char S100297[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E200 Safari/7534.48.3 TeamStream (iPhone3,3 232 1.6.2)";
const char S100298[] = "SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonw20i) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S100299[] = "Mozilla/5.0 (Java; U; en-us; UCBrowser8.4.0.159/69/352/UCWEB Mobile  SonyEricssonP990i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S100300[] = "iBrowser/Mini2.8 (SAMSUNG-GT-S5222)";
const char S100301[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; A73 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S100302[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100303[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT26i Build/6.1.A.2.45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000";
const char S100304[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9210T Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S100305[] = "MQQBrowser/Mini2.8 (Nokia305/03.60)";
const char S100306[] = "Mozilla/5.0 (Linux; Android 4.0.4; shooteru Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S100307[] = "Mozilla/5.0 (Linux; Android 4.0.4; SCH-L710 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S100308[] = "MQQBrowser/2.8 (NokiaN97;SymbianOS/9.1 Series60/3.0)";
const char S100309[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT22i Build/6.1.B.0.544) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100310[] = "sonyericssonu5i/UC Browser7.5.1.77/69/352";
const char S100311[] = "SAMSUNG-SGH-i710/MSIE4.01/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88";
const char S100312[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A750 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100313[] = "Mozilla/5.0 (Linux; Android 4.0.4; SBM106SH Build/S0020) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S100314[] = "[FBAN/FB4A;FBAV/2.3;FBBV/X;FBDM/{density=1.5,width=480,height=800};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/PC36100;FBSV/2.3.5;]";
const char S100315[] = "samsung-gt-s3653/UC Browser7.2.2.51/69/352";
const char S100316[] = "Nokia5530/UC Browser7.0.0.41/50/351";
const char S100317[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.6) Gecko/X Palemoon/3.5.6.1";
const char S100318[] = "KDDI-TS3J UP.Browser/6.2_7.2.7.1.K.2.234 (GUI) MMP/2.0";
const char S100319[] = "SonyEricssonG502/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssong502/UC Browser7.9.0.102/69/352";
const char S100320[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MotoA953 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100321[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) iCab/4.7 Safari/525.27.1";
const char S100322[] = "LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-gs500) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S100323[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; GT-S5830i Build/ICE CREAM SANDWICH) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100324[] = "Flipboard/1.7.2 CFNetwork/609 Darwin/13.0.0";
const char S100325[] = "iBrowser/Mini2.8 (Nokia311/03.81)";
const char S100326[] = "Mozilla/5.0 (Java; U; en-us; micromaxq50) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S100327[] = "MQQBrowser/3.2 (NokiaC5-03;SymbianOS/9.1 Series60/3.0)";
const char S100328[] = "Mozilla/5.0 (Java; U; en-us; gt-s5233w) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile";
const char S100329[] = "SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; fa; sonyericssonw20i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S100330[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MK16i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S100331[] = "OneBrowser/3.0 (NokiaC2-01/11.10)";
const char S100332[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; LG-P920 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S100333[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-V909 Build/HMJ37) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S100334[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.1 Safari/7534.51.22 Sleipnir/3.8.3";
const char S100335[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GN100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100336[] = "JUC (Linux; U; 2.2.2; zh-cn; MTS-SP101; 240*320) UCWEB7.9.0.94/139/444";
const char S100337[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOR807 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100338[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; YP-G70 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100339[] = "Mozilla/5.0 (Linux; Android 4.2.1; HTC Vision Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S100340[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:17.0) Gecko/X Firefox/17.0 slurp";
const char S100341[] = "Mozilla/5.0 (Linux; Android 4.0.4; EasyPad 971 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S100342[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia5238) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile";
const char S100343[] = "SonyEricssonW910i/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonw910i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S100344[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A1+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S100345[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID7036 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S100346[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; HTC One X+ Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S100347[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; PC36100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S100348[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17 slurp";
const char S100349[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; Intex Aqua 3.2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S100350[] = "Mozilla/5.0 (Linux; Android 4.0.4; D709 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S100351[] = "UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; X10a) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S100352[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I777 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (SAMSUNG-SGH-I777, 40, 1.8.5)";
const char S100353[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S100354[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; iBall Slide 3G 7334 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S100355[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SCH-I589 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S100356[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Chacha A810 Build/X-; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100357[] = "Mozilla/5.0 (Linux; Android 4.1.2; SO-03E Build/10.1.E.0.244) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S100358[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I8245 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100359[] = "Mozilla/5.0 (Linux; Android 4.0.4; N-03E Build/AX) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S100360[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZP900H Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S100361[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE T81 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S100362[] = "Mozilla/5.0 (Linux; Android 4.0.4; L-05D Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S100363[] = "Mozilla/5.0 (Linux; Android 4.0.3; RMD-1035 Build/ICS.g12refM1005RU.X) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S100364[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N8013 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, -0400, us";
const char S100365[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua_Trendy) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S100366[] = "Mozilla/5.0 (Linux; Android 4.0.4; A11 Build/KarbonnA11) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S100367[] = "Mozilla/5.0 (Linux; Android 4.1.2; Jiayu G3 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S100368[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; Lenovo S899t Build/S-2-20) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S100369[] = "Mozilla/5.0 (Linux; Android 4.0.4; Spice Mi-500 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S100370[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328d Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S100371[] = "Mozilla/5.0 (Linux; Android 4.0.4; i-mobile i-style Q3 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S100372[] = "Mozilla/5.0 (Linux; Android 4.1.1; AMOI N821 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S100373[] = "Mozilla/5.0 (Linux; Android 4.1.2; AT-AS43D3 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S100374[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A770K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S100375[] = "Instagram 3.5.3 Android (10/2.3.5; 240dpi; 480x800; HTC/sprint; PC36100; supersonic; supersonic; en_US)";
const char S100376[] = "NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac1-02) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S100377[] = "Mozilla/5.0 (Linux; Android 4.0.4; PMP7170B3G Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S100378[] = "Mozilla/5.0 (Linux; Android 4.0.4; SBM106SH Build/S0020) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S100379[] = "Instagram 3.5.1 (iPod4,1; iPhone OS 6_1_3; en_US; en) AppleWebKit/420+";
const char S100380[] = "SAMSUNG-SGH-T528g/T528UDKJ2 Dolfin/1.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S100381[] = "Mozilla/5.0 (Linux; Android 4.1.2; A953 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S100382[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S100383[] = "Opera/8.51 (FreeBSD 5.1; U; en)";
const char S100384[] = "Mozilla/5.0 (Linux; Android 4.1.2; KM-E100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S100385[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-I415 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S100386[] = "Mozilla/5.0 (Linux; Android 4.0.3; ASTROPLUS Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S100387[] = "Mozilla/5.0 (Linux; Android 4.2.2; Sensation XL Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S100388[] = "Mozilla/5.0 (Linux; Android 4.0.3; AN10DG3 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S100389[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; MAGPAD9.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S100390[] = "Mozilla/5.0 (Linux; Android 4.0.4; Benten A668 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S100391[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S100392[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.2.2.259";
const char S100393[] = "Mozilla/5.0 (Linux; Android 4.1.1; M030 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S100394[] = "Mozilla/5.0 (Series40; Nokia3030/14.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.48";
const char S100395[] = "Mozilla/5.0 (Linux; Android 4.0.4; Telenor_One_Touch_S Build/IceCreamSandwich) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S100396[] = "Mozilla/5.0 (Linux; Android 4.0.4; IQ444 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S100397[] = "Mozilla/5.0 (Linux; Android 4.1.2; SCH-L710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S100398[] = "Mozilla/5.0 (Linux; Android 4.1.1; Cynus T1 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S100399[] = "Mozilla/5.0 (Linux; Android 4.0.4; GEM10312BK Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S100400[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.10) Gecko/X Postbox/1.0.1";
const char S100401[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; VOTO_W7200A Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100402[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.5.0.185/83/352";
const char S100403[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; Discovery Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100404[] = "NokiaC2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-05) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile";
const char S100405[] = "OneBrowser/3.1 (NokiaC2-02/06.96)";
const char S100406[] = "OneBrowser/3.1 (NokiaE63-1/0.21.010)";
const char S100407[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.2.7) Gecko/X Epic/1.1 Firefox/3.6.7";
const char S100408[] = "MQQBrowser/3.7/Adr (Linux; U; 2.3.5; zh-cn; Lenovo A288t Build/MocorDroid2.3.5.A288t_S161_X;320*480)";
const char S100409[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100410[] = "Nokia5200/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5200) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile";
const char S100411[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E10i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S100412[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; TECNO N3 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S100413[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; SAMSUNG-SGH-I897 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S100414[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.2.2.259";
const char S100415[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; X909 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S100416[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MT15i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S100417[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-lu6200 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S100418[] = "Mozilla/5.0 (Linux; Android 4.0.4; Burst S280 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S100419[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-c3322) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S100420[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_903) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S100421[] = "Mozilla/5.0 (Linux; Android 4.0.4; Telenor_One_Touch_S Build/IceCreamSandwich) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S100422[] = "Mozilla/5.0 (Linux; Android 4.0.4; Aqua flash Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S100423[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A89 Build/CelkonCelkon_A89) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S100424[] = "OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; Micromax A116 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100425[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S100426[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Micromax A65 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.3.2.289";
const char S100427[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R811 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S100428[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk530i) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S100429[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA5500) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile";
const char S100430[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; C6602) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S100431[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A760S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S100432[] = "Mozilla/5.0 (Linux; Android 4.1.1; Celkon A119 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S100433[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE R22 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S100434[] = "Mozilla/5.0 (Linux; Android 4.1.2; Samsung GT-S9081 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S100435[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A7+) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S100436[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; V9C Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S100437[] = "Mozilla/5.0 (Linux; Android 4.1.1; rk30sdk Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201";
const char S100438[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S100439[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; ADR6350 Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S100440[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A111) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S100441[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; SAMSUNG-SGH-I997 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S100442[] = "Mozilla/5.0 (Linux; Android 4.0.3; PMP3470B Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S100443[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LG-P999 Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S100444[] = "Mozilla/5.0 (Linux; Android 4.1.1; A7 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S100445[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; SCH-I959 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31";
const char S100446[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I829 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S100447[] = "Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 992D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S100448[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-LS970 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S100449[] = "Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I547 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S100450[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; MT11i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S100451[] = "Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabS2109A-F Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.60140";
const char S100452[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9100 Build/JDQ39E) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S100453[] = "Mozilla/5.0 (Linux; Android 4.0.4; GEM10312BK Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S100454[] = "Mozilla/5.0 (Linux; Android 4.0.3; SBM104SH Build/S0034) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S100455[] = "Mozilla/5.0 (Linux; Android 4.3; One S Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S100456[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3650) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S100457[] = "Mozilla/5.0 (Linux; Android 4.3; LG-E975 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S100458[] = "samsung-gt-s5380k/UC Browser7.8.0.95/69/351";
const char S100459[] = "JUC (Linux; U; 2.3.4; zh-cn; LG-C660; 240*320) UCWEB7.9.0.94/139/444";
const char S100460[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P505 Build/JZO54K; CyanogenMod-10 (Build 6/RC 2)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100461[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA6710s) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile";
const char S100462[] = "Mozilla/5.0 (Linux; Android 4.0.4; IQ444 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S100463[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T528t Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S100464[] = "LG-CU720/V1.0l Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-cu720) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S100465[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A87 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S100466[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MB865 Build/6.7.2_EDEM-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S100467[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V889D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S100468[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-CN; ZTE U880E Build/GWK74) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S100469[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; ZTE V889D Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S100470[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (SCH-I510, 42, 1.9.1)";
const char S100471[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S100472[] = "Mozilla/5.0 (Linux; Android 4.1.1; AVA757 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S100473[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT15i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S100474[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-CN; MB525 Build/MIUI) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31";
const char S100475[] = "Dolphin/6.3 CFNetwork/548.1.4 Darwin/11.0.0";
const char S100476[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; A56 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S100477[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; LG-E405) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S100478[] = "Nokia2700c-2/2.0 (09.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.0.0.260";
const char S100479[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; Karbonn_A18 Build/KarbonnA18) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S100480[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; MTS-SP100) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S100481[] = "Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.2.1 (like Version/10A523 Mobile/6.0.1 Safari/8536.25)";
const char S100482[] = "Mozilla/5.0 (Linux; Android 2.3.4; ST18i Build/4.0.2.A.0.62) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768";
const char S100483[] = "Mozilla/5.0 (Linux; Android 4.0.4; T-02D Build/V19R50B) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S100484[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R813T Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S100485[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire U dual sim Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S100486[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Karbonn A2+) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S100487[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A35 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S100488[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; X909T Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S100489[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ZTE N9120 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S100490[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I8160) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S100491[] = "Mozilla/5.0 (Linux; Android 4.1.2; NookColor Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S100492[] = "Mozilla/5.0 (Linux; Android 4.1.1; AT-AS40SE Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S100493[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (Nexus 7, 52, 1.9.9.2)";
const char S100494[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LGL35G Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31";
const char S100495[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us";
const char S100496[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I537 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0500, us";
const char S100497[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SM-T210R Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us";
const char S100498[] = "Mozilla/5.0 (Linux; Android 4.0.3; S2005A-H Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S100499[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Z7H Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S100500[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Spice_Mi-425) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S100501[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC T528t Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31";
const char S100502[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9505 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.27/385,1.9.27.385,2013-03-31 15:31, +0000, gb";
const char S100503[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S100504[] = "Mozilla/5.0 (Linux; Android 4.1.1; I7100 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S100505[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; N350) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S100506[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk790i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S100507[] = "Mozilla/5.0 (Linux; Android 4.0.4; A2 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36";
const char S100508[] = "Mozilla/5.0 (Linux; Android 4.3; SCH-I605 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.30 Mobile Safari/537.36";
const char S100509[] = "MicromaxX291/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1";
const char S100510[] = "Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia305/UC Browser7.7.1.88/69/444";
const char S100511[] = "Mozilla/5.0 (Linux; Android 4.0.4; MOMO9 plus III Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S100512[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.1; en-in; LG-P500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100513[] = "OneBrowser/for Blackberry3.0.0 (BlackBerry9630/5.0.0.975)";
const char S100514[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Desire S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S100515[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R813T Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S100516[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST25i Build/6.0.B.3.184) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S100517[] = "Instagram 3.4.2 (iPad2,1; iPhone OS 5.0.1; en_US; en) AppleWebKit/420+";
const char S100518[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (SPH-D700, 42, 1.9.1)";
const char S100519[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T769 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (SGH-T769, 42, 1.9.1)";
const char S100520[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; zh-CN; GT-I9003) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile";
const char S100521[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; StarTrail III Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100522[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; U9200 Build/HuaweiU9200) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S100523[] = "Mozilla/5.0 (Linux; Android 4.1.2; PTL21 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S100524[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; Lenovo S880 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S100525[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; MT27i Build/6.1.1.B.1.54) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S100526[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; ST27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S100527[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.3; en-in; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S100528[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Spice Mi-515 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S100529[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GenxDroid7 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S100530[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; IRIS355 Build/IRIS355) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S100531[] = "Mozilla/5.0 (Linux; Android 4.1.1; Genio Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S100532[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; MT11i) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S100533[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-S7560M Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, ca";
const char S100534[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI MT1-U06 Build/HuaweiMT1-U06) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S100535[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S100536[] = "AppleCoreMedia/1.0.0. (AAS-2.2.2; U; Windows NT 6.0)";
const char S100537[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562 Build/IMM76I; 480*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444";
const char S100538[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BE 150 3G Build/ICS.f08refem79v2.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S100539[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 7019 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100540[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8530 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100541[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia701) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile  3gpp-gba";
const char S100542[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TOUCH87 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S100543[] = "Mozilla/5.0 (Linux; Android 4.1.1; S8800 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S100544[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Lenovo S560 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S100545[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Lenovo A830 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S100546[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-N7000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S100547[] = "StumbleUpon/2.52 CFNetwork/609.1.4 Darwin/13.0.0";
const char S100548[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A760S/770K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100549[] = "Mozilla/5.0 (Linux; Android 4.0.4; STM805ND Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S100550[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; U705T Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S100551[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; R807 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S100552[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; R817 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S100553[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-S7572 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S100554[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-CN; HTC Sensation XE with Beats Audio Z715e Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S100555[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3110 Build/JDQ39; Carbon-CARBON-JB-EXP-08APR2013) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S100556[] = "Mozilla/5.0 (Linux; Android 4.2.1; mobiistar touch LAI 504 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S100557[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ONIDA_i099) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S100558[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo A800 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S100559[] = "UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; ST18i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S100560[] = "Mozilla/5.0 (Linux; Android 4.1.1; Micromax P362 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S100561[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; th-th; Airphone_A501 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100562[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HWA580 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100563[] = "OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100564[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; zh-CN; MI 1S) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile";
const char S100565[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-SU760 Build/ICS) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S100566[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Titan Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100567[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9192 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S100568[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5670) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S100569[] = "Mozilla/5.0 (Linux; Android 4.2.1; Lenovo A3000-H Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S100570[] = "Mozilla/5.0 (Linux; Android 4.2.1; Karbonn S2 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S100571[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; SPH-M830 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S100572[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-CN; MB525 Build/MIUI) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S100573[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-gb; GT-S5301 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S100574[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M030 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S100575[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-in; LT22i Build/6.2.A.1.100) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S100576[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A25 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S100577[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5670 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S100578[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT897 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/xt897c";
const char S100579[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6810 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S100580[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Iris 349 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S100581[] = "UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; ST27i) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S100582[] = "LG-GX500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gx500) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S100583[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-E975 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/e975";
const char S100584[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-T769 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S100585[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; U8800Pro Build/HuaweiU8800Pro) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S100586[] = "UCWEB/2.0 (Windows; U; wds 8.0; en-US; HTC;  8X by HTC) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile";
const char S100587[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5312 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S100588[] = "Mozilla/5.0 (Linux; Android 4.0.4; bq Elcano Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S100589[] = "Mozilla/5.0 (Linux; Android 4.0.4; XOLO_A800 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S100590[] = "Mozilla/5.0 (Linux; Android 4.1.2; IQ446 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.20.1364.172 Mobile Safari/537.22";
const char S100591[] = "LG-GX500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gx500) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S100592[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5800d-1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba";
const char S100593[] = "Mozilla/5.0 (Linux; Android 4.0.4; ZTE V70 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S100594[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9082 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300";
const char S100595[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I879 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S100596[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC One X+ Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S100597[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.20.1364.172 Mobile Safari/537.22";
const char S100598[] = "Mozilla/5.0 (Linux; Android 4.1.2; AT-AS43D3 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100599[] = "SAMSUNG-SCH-M715/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S100600[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:13.0) Gecko/X Firefox/13.0.1";
const char S100601[] = "Mozilla/5.0 (Linux; U; Android 1.5.1.12-RT-X.X; zh-cn; K-Touch W650 Build/AliyunOs-2012) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S100602[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Android Build/GRK39F; 320*480) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/33081";
const char S100603[] = "JUC (Linux; U; 2.3.6; zh-cn; Andi107; 240*320) UCWEB7.9.0.94/139/444";
const char S100604[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-SU640 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S100605[] = "OneBrowser/3.1 (SAMSUNG-GT-S5263/S5263DDKA5)";
const char S100606[] = "OneBrowser/3.1 (SonyEricssonK770i/R8BC004)";
const char S100607[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100608[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A298t Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100609[] = "Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.8.0.12) Gecko/X Firefox/1.5.0.12";
const char S100610[] = "Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI C8950D Build/HuaweiC8950D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S100611[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)";
const char S100612[] = "OneBrowser/3.1 (NokiaC6-00/42.0.004/sw_platform=S60;sw_platform_version=5.0;java_build_version=1.4.48)";
const char S100613[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Desire S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0";
const char S100614[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-I9268 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S100615[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; Lenovo A690 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S100616[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; iBall_Andi4.5q) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S100617[] = "JUC (Linux; U; 4.1.2; zh-cn; GT-S6312; 320*480) UCWEB7.9.0.94/139/444";
const char S100618[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; A73 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S100619[] = "JUC (Linux; U; 4.1.1; zh-cn; Micromax_A110; 480*854) UCWEB7.9.0.94/139/444";
const char S100620[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; NookColor Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S100621[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S100622[] = "Mozilla/5.0 (Linux; U; Android 4.1.0; en-np; XT701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100623[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; Dream_D1 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33886";
const char S100624[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-MS695 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon";
const char S100625[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Celkon_A107) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S100626[] = "Mozilla/5.0 (Linux; Android 4.1.2; NexusHD2 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S100627[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Micromax A115) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S100628[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Ctrl_V3 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S100629[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; V9A Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.3.2.300";
const char S100630[] = "MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; vivo Xplay Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100631[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; T8620 Build/HuaweiT8620) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1";
const char S100632[] = "UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Micromax_A116) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile";
const char S100633[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; V901 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S100634[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; N400 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S100635[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; A111 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S100636[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6812) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile";
const char S100637[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; R801 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31";
const char S100638[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; zte-f970) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S100639[] = "OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-in; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S100640[] = "Mozilla/5.0 (Series40; NokiaX2-05/08.30; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62";
const char S100641[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; TECNO_P3 Build/master) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.276 Mobile";
const char S100642[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; LT22i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile";
const char S100643[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-P765 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31";
const char S100644[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST18i) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S100645[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N8000) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S100646[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I9003) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile";
const char S100647[] = "UCWEB/2.0 (Symbian; U; S60 V3; en-US; Nokia5730S-1) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile";
const char S100648[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; N861 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S100649[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-E425 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S100650[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Aqua.Active Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile";
const char S100651[] = "UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Aqua_Wonder) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S100652[] = "Mozilla/5.0 (Linux; Android 4.2.2; ARNOVA 101 G4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36";
const char S100653[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-N7000) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile";
const char S100654[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I337M Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/jfltecan";
const char S100655[] = "Mozilla/5.0 (Series40; Nokia2730c-1/10.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6";
const char S100656[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S100657[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; E5 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31";
const char S100658[] = "SonyEricssonU10i/R7BA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonu10i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S100659[] = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; EVO 4G LTE Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100660[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1";
const char S100661[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; LG-P350) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile";
const char S100662[] = "Instagram 4.1.2 Android (10/2.3.6; 240dpi; 480x800; samsung; SGH-T989; SGH-T989; qcom; en_US)";
const char S100663[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9050 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S100664[] = "UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; TECNO_D5) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S100665[] = "Nokia5330-1d/5.0 (06.88) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5330-1d) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S100666[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A65 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S100667[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk810i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S100668[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8552 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S100669[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S100670[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-I869 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31";
const char S100671[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541";
const char S100672[] = "UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; GT-I9000) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S100673[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; ALCATEL_ONE_TOUCH_985N Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S100674[] = "UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; Karbonn_A1) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S100675[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; ADR6350 Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S100676[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5670 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S100677[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-B5330) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S100678[] = "UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5830) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S100679[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.4.27) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile";
const char S100680[] = "UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua Star) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S100681[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9192 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31";
const char S100682[] = "UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; GT-I9001) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile";
const char S100683[] = "Mozilla/5.0 (Linux; Android 4.0.3; M-270 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100684[] = "Mozilla/5.0 (Linux; Android 4.3; Amazon Kindle Fire Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S100685[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6812) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile";
const char S100686[] = "Mozilla/5.0 (Linux; Android 4.2.2; 202F Build/V17R47F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100687[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; HUAWEI P6-U06 Build/HuaweiP6-U06) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100688[] = "Mozilla/5.0 (Linux; Android 4.2.1; IQ4410 Quad Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100689[] = "Mozilla/5.0 (Linux; Android 4.0.4; NexusHD2 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100690[] = "Mozilla/5.0 (Linux; Android 4.0.4; N10 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S100691[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; N400 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100692[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo A789 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100693[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI A199 Build/HuaweiA199) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100694[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; Micromax A76 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100695[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; HTC Desire SV Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100696[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A10 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100697[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7300 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36";
const char S100698[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5830 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100699[] = "Mozilla/5.0 (Linux; Android 4.1.2; SBM200SH Build/S1004) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100700[] = "Mozilla/5.0 (Linux; Android 4.2.1; SKY HD9500 Plus Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100701[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; SK17i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100702[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A8 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100703[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A1+ Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100704[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Gpad_G2 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100705[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-P760 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36";
const char S100706[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; AURUS III Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100707[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; ST18i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100708[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI C8815 Build/HuaweiC8815) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100709[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A111 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100710[] = "Mozilla/5.0 (iPad; U; CPU OS 7_0 like Mac OS X; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.10080IT Mobile";
const char S100711[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-I9103 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31";
const char S100712[] = "Tweetbot/28500 CFNetwork/672.0.2 Darwin/14.0.0";
const char S100713[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTL22 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S100714[] = "UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5230) U2/1.0.0 UCBrowser/9.2.0.336 U2/1.0.0 Mobile";
const char S100715[] = "Mozilla/5.0 (Linux; Android 4.0.4; MT11i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S100716[] = "Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Safari/7534.48.3 Sleipnir/3.0m";
const char S100717[] = "Mozilla/5.0 (Linux; Android 4.2.2; SM-T315 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S100718[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST25i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36";
const char S100719[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-N5120 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36";
const char S100720[] = "SAMSUNG-SGH-E258/1.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzUzNDUyMDExMjkxNzky UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S100721[] = "SonyEricssonU10i/R7BA";
const char S100722[] = "SEC-SGHU100/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S100723[] = "MOT-W231/0.0.45 UP.Browser/6.3.0.6.c.23 (GUI) MMP/2.0";
const char S100724[] = "HTC_P5500 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S100725[] = "SEC-SGHE870/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S100726[] = "SAMSUNG-SGH-i908/1.0 Opera 9.5";
const char S100727[] = "SAMSUNG-SGH-P850/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S100728[] = "SEC-SGHP300/1.0 TSS/2.5";
const char S100729[] = "Mozilla/4.0 (MobilePhone MM-7500/US/1.0) NetFront/3.1 MMP/2.0";
const char S100730[] = "LG/U8180/v1.0";
const char S100731[] = "HTC_P3470 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S100732[] = "MOTO-W760r/Mozilla/4.0 (compatible;MSIE 6.0;Linux W760r)/R63712_U_71.xx.yyI Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony 1.0";
const char S100733[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; HTC-A315c/1.13.1401.0) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100734[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-tw; HTCA315c_TD/1.0 Android/2.2 release/1.1.2011 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100735[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; HTC Bee Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S100736[] = "SCH-U430-PARROT/1.0 AU-MIC/2.0 (GUI) MMP/2.0";
const char S100737[] = "SCH-U430/1.0 AU-MIC/2.0 (GUI) MMP/2.0";
const char S100738[] = "Mozilla/5.0 (Linux; U; Android FlyX v0.1 Final; en-us; DROIDX Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola DROIDX";
const char S100739[] = "Mozilla/5.0 (Linux; U; Android ICS 1.12.30 B3; en-us; DROIDX Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100740[] = "MOT-EX122/07.2E.00R  Obligo/WAP2.0 MIDP-2.0 Configuration/CLDC-1.1";
const char S100741[] = "SEC-SGHX520/1.0";
const char S100742[] = "SEC-SGHX520X/1.0";
const char S100743[] = "BlackBerry7290/4.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S100744[] = "LGE-VX8610/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S100745[] = "NexianNX-NXT901U/MTK Release/11-20-2010 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1";
const char S100746[] = "NexianNX-G901/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S100747[] = "Alcatel-OT-508A/1.0 ObigoInternetBrowser/Q03C";
const char S100748[] = "Nokia5530/UC Browser7.6.1.82/50/352";
const char S100749[] = "NokiaE72-1/UC Browser7.6.1.82/28/351";
const char S100750[] = "Nokia5250/UC Browser7.7.1.88/50/444";
const char S100751[] = "BlackBerry7100/4.0.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S100752[] = "BlackBerry7230/4.0.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S100753[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320; HTC_P6300/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S100754[] = "Spice D88";
const char S100755[] = "Spice D88/WAP2.0";
const char S100756[] = "KDDI-KC3H UP.Browser/6.2.0.14.1 (GUI) MMP/2.0";
const char S100757[] = "KDDI-MA34 UP.Browser/6.2.0.13.2 (GUI) MMP/2.0";
const char S100758[] = "TIANYU-KTOUCH/V388/MIDP2.0/CLDC1.1/Screen-240X320";
const char S100759[] = "Mozilla/5.0 (compatible; Konqueror/3.4; FreeBSD) KHTML/3.4.3 (like Gecko)";
const char S100760[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; X10i Build/FreeX10-beta4_zdzihu) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100761[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; X10i Build/CMNI15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100762[] = "Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; X10i Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100763[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; X10i Build/FreeX10-beta3_zdzihu) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100764[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; X10i Build/CM 6.1.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100765[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; TAB220 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100766[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Iron/5.0.381.0 Chrome/5.0.381 Safari/533.4";
const char S100767[] = "SEC-SGHE900/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/8.01 (J2ME/MIDP; Opera Mini/2.0.4509/1378; nl; U; ssr)";
const char S100768[] = "Opera/9.80 (Series 60; Opera Mini/7.29482/27.1573; U; en) Presto/2.8.119 Version/11.10";
const char S100769[] = "j2me/UC Browser7.4.0.65/69/351";
const char S100770[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Wyzo/3.5.6.1";
const char S100771[] = "SH0037D Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S100772[] = "Mozilla/5.0 (compatible; Teleca Q7; Brew 3.1.5; U; en) 240X400 LGE-MX700/1.0";
const char S100773[] = "NokiaX2-01/5.0 (08.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Opera/9.60 (J2ME/MIDP;Opera Mini/4.2.13337Mod.by.Dante./503; U; en)Presto/2.2.0";
const char S100774[] = "Nokia6500s-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6500s-1/UC Browser7.9.0.102/69/352";
const char S100775[] = "samsung-gt-s5263/UCWEB8.0.3.99/70/350";
const char S100776[] = "samsung-gt-c6712/UC Browser7.9.0.102/69/444";
const char S100777[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19 Comodo_Dragon/18.3.0.0";
const char S100778[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19 Comodo_Dragon/18.3.0.0";
const char S100779[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; disgo6000 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100780[] = "Mozilla/5.0 (PlayStation Vita 1.51) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2";
const char S100781[] = "Mozilla/5.0 Opera/9.5 (KDDI-CA3F; BREW; Opera Mobi; U; ja) Presto/2.2.1";
const char S100782[] = "MQQBrowser/3.0 (Nokia5233;SymbianOS/9.1 Series60/3.0)";
const char S100783[] = "Mozilla/5.0 Opera/9.5 (KDDI-CA3H; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00";
const char S100784[] = "Mozilla/5.0 (Linux; U; Android 1.0.15; fr-fr; A80HF Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S100785[] = "Mozilla/4.0 (jig browser web; 1.0.4; 911SH)";
const char S100786[] = "samsung-gt-s5253/UCWEB8.0.3.99/70/350";
const char S100787[] = "SonyEricssonK850i/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonk850i/UC Browser8.0.3.107/69/444";
const char S100788[] = "QQBrowser/16 (Linux; U; 2.2.2; zh-cn; ZTE ZTE-C N880S Build/N880SV1.0.0B03) Mobile/0070";
const char S100789[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; 760TPC Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100790[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11 Comodo_Dragon/17.5.2.0";
const char S100791[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11 Comodo_Dragon/17.5.2.0";
const char S100792[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I997 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100793[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Z71 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100794[] = "MQQBrowser/21 Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad 5820 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100795[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; SH8118U Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S100796[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; SH8118U Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S100797[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100798[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; ST18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S100799[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100800[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2; en-au; GT-P1000T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100801[] = "Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24";
const char S100802[] = "SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-in; sonyericssonw20) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S100803[] = "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 4.0) Opera 5.11 [de]";
const char S100804[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; MT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100805[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E15i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S100806[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-D710 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100807[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.24 (KHTML, like Gecko) Ubuntu/9.10 Chromium/11.0.696.57 Chrome/11.0.696.57 Safari/534.24";
const char S100808[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.4; en-US; rv:1.9.1b3) Gecko/X Firefox/3.1b3";
const char S100809[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.13) Gecko/X Epic/1.2 Firefox/3.6.10 GTB7.1";
const char S100810[] = "SonyEricssonU10i/R7BA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonu10i/UC Browser7.7.1.88/69/444";
const char S100811[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:5.0) Gecko/X Thunderbird/5.0 Lightning/1.0b4";
const char S100812[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:2.0.1) Gecko/X Postbox/2.5.2";
const char S100813[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:5.0) Gecko/X Thunderbird/5.0 Lightning/1.0b5";
const char S100814[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:6.0.1) Gecko/X Thunderbird/6.0.1";
const char S100815[] = "nokian96-1/UC Browser7.9.0.102/69/352";
const char S100816[] = "micromaxx600/UC Browser7.9.0.102/69/352";
const char S100817[] = "Opera/9.80 (Windows 98; U; en) Presto/2.10.229 Version/11.60";
const char S100818[] = "JUC (Linux; U; 2.3.6; zh-cn; GT-S5820; 320*480) UCWEB7.9.3.103/139/800";
const char S100819[] = "Mozilla/5.0 (Series40; Nokia302/14.26; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5";
const char S100820[] = "Mozilla/5.0 (Java; U; en-in; lg-gu220) UCBrowser8.2.1.144/69/405/UCWEB Mobile";
const char S100821[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/12.10 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19";
const char S100822[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/12.10 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19";
const char S100823[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328t Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100824[] = "sonyericssonw395/UC Browser7.9.0.102/69/352";
const char S100825[] = "Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/70/352";
const char S100826[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; XT530 Build/V4.530) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S100827[] = "Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/70/352";
const char S100828[] = "LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  lg-kp500 teleca/UC Browser7.9.0.102/70/352";
const char S100829[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S100830[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokian72) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S100831[] = "SAMSUNG-GT-I9000/100.20303";
const char S100832[] = "LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  lg-gs290/UC Browser8.0.3.107/69/444";
const char S100833[] = "Mozilla/5.0 (Java; U; en-in; lg-gu220) UCBrowser8.3.0.154/69/444/UCWEB Mobile";
const char S100834[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; HTC ONE 4G+ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100835[] = "Mozilla/5.0 (Linux; Android 4.0.4; ADR6350 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S100836[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax P500(Funbook) Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S100837[] = "Nokia311/5.0 (03.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/69/444";
const char S100838[] = "MQQBrowser/Mini2.8 (Nokia2690/10.10)";
const char S100839[] = "Mozilla/5.0 (Linux; Android 4.0.3; M031 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S100840[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/1.6.1.1691 (Phoenix Viewer Release; Firebird skin) Safari/533.3";
const char S100841[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; USCCADR6285US Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100842[] = "JUC(Linux;U;Android2.3.6;Zh_cn;A52;240*320;)UCWEB7.8.0.95/139/444";
const char S100843[] = "T-Mobile Dash (compatible; MSIE 4.01; Windows CE; Smartphone; 320x240)/UC Browser7.2.2.51";
const char S100844[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1;en-us;LenovoTablet Build/1.0) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S100845[] = "Nokia5610d-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5610d-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S100846[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; cs-cz; GOCLEVER TAB I71 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100847[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;TITAN X310e;7.10;8862)";
const char S100848[] = "Mozilla/5.0 (Linux; Android 4.0.4; GSmart Rio R1 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S100849[] = "Mozilla/5.0 (Linux; Android 4.0.4; Rio R1 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S100850[] = "JUC(Linux;U;Android3.2;Zh_cn;GT-P7300;800*1232;)UCWEB7.8.0.95/139/355";
const char S100851[] = "Mozilla/5.0(Linux;U;Android3.2;Zh_cn;GT-P7300;800*1232;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S100852[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-P1000R Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1000R, 36, 1.7.1)";
const char S100853[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1000, 39, 1.8.2)";
const char S100854[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-P1000M Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1000M, 39, 1.8.2)";
const char S100855[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.2.139/145/355";
const char S100856[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P3100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S100857[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-P3100 Build/IML74K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S100858[] = "YahooMobileMessenger/1.0 (Android Messenger; 1.8.3) (espressorf; samsung; GT-P3100; 4.0.4/IMM76D)";
const char S100859[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S100860[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S100861[] = "Instagram 3.4.4 Android (16/4.1.1; 160dpi; 800x1232; samsung; GT-P5113; espresso10wifi; espresso10; en_US)";
const char S100862[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.20/286,1.9.20.286,2013-01-25 13:38, +0200, za";
const char S100863[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P6800 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S100864[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-c310) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S100865[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-s365) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S100866[] = "Android/1.0 (SPH-D710 IMM76I)";
const char S100867[] = "Nokia114/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia114) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S100868[] = "alcatel-ot-807a/UC Browser7.0.0.41/69/4255";
const char S100869[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5230) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S100870[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-s5230) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S100871[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5230) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S100872[] = "iBrowser/2.7 (Nokia603;SymbianOS/9.1 Series60/3.0)";
const char S100873[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U8812D Build/HuaweiU8812D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100874[] = "nokian79-1/UC Browser7.5.1.77/69/352";
const char S100875[] = "samsung-gt-s3850/UC Browser7.2.2.51/69/352";
const char S100876[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL55HX820; CC/AUS; en) Presto/2.7.61 Version/11.00";
const char S100877[] = "NokiaC6-00/UC Browser7.0.0.41/50/351";
const char S100878[] = "MQQBrowser/2.6 (NokiaN73(08);SymbianOS/9.1 Series60/3.0)";
const char S100879[] = "MQQBrowser/2.8 (SonyEricssonU5;SymbianOS/9.1 Series60/3.0)";
const char S100880[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s8500) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S100881[] = "iBrowser/2.7 (Nokia5250;SymbianOS/9.1 Series60/3.0)";
const char S100882[] = "LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-in; lg-gs500) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S100883[] = "Mozilla/5.0 (Java; U; en-us; nokia701) UCBrowser8.4.0.159/69/352/UCWEB Mobile  3gpp-gba";
const char S100884[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5670 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100885[] = "MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT22i Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100886[] = "Mozilla/5.0 (Linux; Android 4.0.3; M9 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S100887[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-E730 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S100888[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xperia X10 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100889[] = "SAMSUNG-GT-N8013/100.40004";
const char S100890[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; GT-B7510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100891[] = "MQQBrowser/31 Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3";
const char S100892[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-E510 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S100893[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s7233e) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile";
const char S100894[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; E16i Build/2.1.1.A.0.53; 320*480) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444";
const char S100895[] = "Mozilla/5.0 (Linux; Android 4.0.4; HTC_Pyramid Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S100896[] = "Mozilla/5.0 (S60V5; U; en-us; SamsungI8910) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/444/UCWEB Mobile";
const char S100897[] = "Mozilla/5.0 (Linux; Android 4.1.2; ADR6300 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S100898[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100899[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; iball Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100900[] = "JUC (Linux; U; 2.3.4; zh-cn; ST17i; 320*480) UCWEB7.9.0.94/139/352";
const char S100901[] = "Mozilla/5.0 (Java; U; en-us; jblend) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile/HS-T39_TD/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0 Config/CLDC-1.1";
const char S100902[] = "JUC (Linux; U; 3.1; zh-cn; GT-P7300; 800*1280) UCWEB7.9.0.94/139/444";
const char S100903[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; ST26i Build/11.0.A.3.28) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100904[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S100905[] = "JUC (Linux; U; 2.3.6; zh-cn; Andi3n; 320*480) UCWEB7.9.0.94/139/444";
const char S100906[] = "Instagram 3.2.0 (iPhone3,1; iPhone OS 5.0.1; en_US) AppleWebKit/420+";
const char S100907[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxx266) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S100908[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; HTC One X Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S100909[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; Micromax A60 Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S100910[] = "Mozilla/5.0 (Linux; U; Android 4.1.9; zh-tw; GT-N7005 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S100911[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Karbonn A9+ Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S100912[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P690 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S100913[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT26ii Build/6.1.A.2.45) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S100914[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; LG-P920 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S100915[] = "Nokia5330-1d/5.0 (06.88) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia5330-1d) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S100916[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE C1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S100917[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTCEVODesign4G Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S100918[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC One S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S100919[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; GT540 Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S100920[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Flipboard/1.9.8";
const char S100921[] = "Mozilla/5.0 (Linux; Android 4.0.4; myTouch_4G_Slide Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S100922[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9003 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S100923[] = "Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Kem 452 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S100924[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC One X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, +0100, gb";
const char S100925[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-CN; ST18i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S100926[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9210T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S100927[] = "Mozilla/5.0 (Linux; Android 4.0.4; Celkon A 200 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S100928[] = "Mozilla/5.0 (Linux; Android 4.0.3; A97i Build/CelkonA97i_Celkon) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S100929[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC One X Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S100930[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-LS970 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S100931[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; Lenovo P700 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S100932[] = "Mozilla/5.0 (Linux; Android 4.0.4; A12 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S100933[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; ZTE-Z990G Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31";
const char S100934[] = "Mozilla/5.0 (Linux; Android 4.0.4; P5-Mini Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S100935[] = "Mozilla/5.0 (Linux; Android 4.2.2; LG-E975 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S100936[] = "Mozilla/5.0 (Linux; Android 4.2.1; M6589 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S100937[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1";
const char S100938[] = "iBrowser/2.6 (NokiaC5-05;SymbianOS/9.1 Series60/3.0)";
const char S100939[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8530) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S100940[] = "NOKIAE5-00/UC Browser7.2.2.51/28/351";
const char S100941[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonck15i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S100942[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC EVO 3D X515m Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S100943[] = "Nokia2700c-2/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291";
const char S100944[] = "BlackBerry8110/4.5.0.174 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/303 UP.Browser/5.0.3.3";
const char S100945[] = "HTC_Touch_Diamond2_T5362 Opera/9.50 (Windows NT 5.1; U; en) V2.31.841.6 (64646)";
const char S100946[] = "SAMSUNG-SGH-D510/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S100947[] = "HTC_Touch2_T3335 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S100948[] = "LG-KG300/V01 Obigo/WAP2.0 LG-KG300/V01 Obigo/WAP2.0";
const char S100949[] = "LG-KG300j/V01 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1 LG-KG300j/V01 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S100950[] = "LG-C2500 MIC/WAP2.0 MIDP-2.0/CLDC-1.0 LG-C2500 MIC/WAP2.0 MIDP-2.0/CLDC-1.0";
const char S100951[] = "SAMSUNG-GT-M8910H/1.0 SHP/VPP/R5 Mozilla/5.0 (rv:1.3) NetFront/3.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S100952[] = "Nokia2332c_CMCC/2.0 (08.20) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S100953[] = "LG-KP235-orange/V10b Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S100954[] = "LG-KP235 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S100955[] = "SAMSUNG-SGH-C280/C280BNIE1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S100956[] = "LG-KP202-V10a Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S100957[] = "LG-KP202-V10b Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S100958[] = "LG-KP202 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S100959[] = "SAMSUNG-GT-C6112C/1.0 RTK-E/1.0 DF/1.0 Release/08.18.2009 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S100960[] = "SAMSUNG-SGH-X460/1.0 UP.Browser/6.1.0.6 (GUI) MMP/1.0";
const char S100961[] = "LG-LG6300/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S100962[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) ASUS-P527/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S100963[] = "SonyEricssonC905/R1FA SEMC/NewsReader/1.0";
const char S100964[] = "Motorola-C290-boost Obigo/Q04C1-1.9 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S100965[] = "Motorola-C290 Obigo/Q04C1-1.9 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S100966[] = "BenQ-C30/1.0/WAP2.0/MIDP2.0/CLDC1.1";
const char S100967[] = "SAMSUNG-SGH-A551/1.0 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S100968[] = "ZTE-G X760";
const char S100969[] = "ZTE-G i766";
const char S100970[] = "BlackBerry7100/4.1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100";
const char S100971[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; MDA compact/3.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S100972[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9810; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/6.1.0.50 Mobile Safari/534.11+";
const char S100973[] = "ZTE-R7/1.0 ACS-NF/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S100974[] = "Nokia7610/2.0 (7.0642.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.2.2.51/27/401";
const char S100975[] = "LG-KF900/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S100976[] = "SAMSUNG-SGH-A836/A836UXHK5 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S100977[] = "SAMSUNG-M3510/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S100978[] = "Nokia5235/UC Browser7.4.0.65/50/351";
const char S100979[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Acer_P400 1.023h.00";
const char S100980[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Acer_P400 1.023f.00";
const char S100981[] = "SAMSUNG-GT-S5630C_CMCC/1.0 RTK-E/1.0 SHP/VPP/R5 Release/17.08.2008 Browser/NetFront3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S100982[] = "NOKIA-3806 UP.Browser/6.3.0.8 (GUI) MMP/2.0";
const char S100983[] = "SEC-SGHC170/1.0";
const char S100984[] = "SAMSUNG-SGH-A436/ Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S100985[] = "NokiaN8-00/UC Browser7.4.0.65/50/352";
const char S100986[] = "BenQ-E61/1.00/WAP2.0 UP.Browser/6.3.0.4.c.1.102 (GUI) MMP/2.0";
const char S100987[] = "KDDI-KC3D UP.Browser/6.2.0.13.2 (GUI) MMP/2.0";
const char S100988[] = "KDDI-SN3A UP.Browser/6.2.0.13.1.5 (GUI) MMP/2.0";
const char S100989[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.7.1";
const char S100990[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.7.1";
const char S100991[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.7.2";
const char S100992[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.7.2";
const char S100993[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 Flipboard/1.7.2";
const char S100994[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 Flipboard/1.7.2";
const char S100995[] = "JUC(Linux;U;2.2;Zh_cn;HTC Desire;480*800;)UCWEB7.6.0.75/139/800";
const char S100996[] = "JUC(Linux;U;2.2;Zh_cn;HTC Desire;480*800;)UCWEB7.6.0.75/139/999";
const char S100997[] = "samsung-gt-s5260/UC Browser7.7.1.88/69/444";
const char S100998[] = "Mozilla/5.0 (Windows NT 5.1; rv:6.0.2) Gecko/X Thunderbird/6.0.2 OracleBeehiveExtension/1.0.0.4-OracleInternal Lightning/1.0b5";
const char S100999[] = "Flipboard/0.7.17/34,1.8.1,2012-05-01 00:51";
const char S101000[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC PH39100 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S101001[] = "1PasswordThumbs/1 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookPro8%2C1)";
const char S101002[] = "1PasswordThumbs/1 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookAir4%2C2)";
const char S101003[] = "1PasswordThumbs/1 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (iMac12%2C2)";
const char S101004[] = "1PasswordThumbs/1 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBook5%2C1)";
const char S101005[] = "1PasswordThumbs/1 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacPro3%2C1)";
const char S101006[] = "SonyEricssonG502/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssong502/UC Browser8.0.3.107/69/444";
const char S101007[] = "samsung-gt-s3650/UC Browser7.9.0.102/69/352";
const char S101008[] = "JUC (Linux; U; 2.2.1; zh-cn; SCH-i909; 480*800) UCWEB7.9.3.103/139/800";
const char S101009[] = "MQQBrowser/2.9 (Nokia5230;SymbianOS/9.1 Series60/3.0)";
const char S101010[] = "sonyericssonk530i/UC Browser8.0.3.107/69/444";
const char S101011[] = "sonyericssonk530i/UC Browser8.0.3.107/70/352";
const char S101012[] = "MQQBrowser/2.9 (Nokia5233;SymbianOS/9.1 Series60/3.0)";
const char S101013[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8b5) Gecko/X Fedora/1.5-0.5.0.beta2 Firefox/1.4.1";
const char S101014[] = "Mozilla/5.0 (Windows; U; Windows NT 4.10; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.41 Safari/534.7";
const char S101015[] = "Opera/9.80 (J2ME/MIDP; Opera Mini/7.28870/27.1813; U; en) Presto/2.8.119 Version/11.10";
const char S101016[] = "Opera/9.50 (J2ME/MIDP; Opera Mini/7.29952/27.1659; U; en)";
const char S101017[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:13.0) Gecko/X Firefox/13.0.1 SeaMonkey/2.10.1";
const char S101018[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; DELL; Venue Pro)";
const char S101019[] = "samsung-gt-s7233e";
const char S101020[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Lenovo A60 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101021[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT711 Build/RSTCU10160) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S101022[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT711 Build/RSTCU10160) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S101023[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; es-es; GT-P7300 Build/HRI83) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S101024[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; zh-cn; N12 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101025[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-us; MA440 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S101026[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HUAWEI-M835 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101027[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HW-HUAWEI_C8650/C8650V100R001C92B825; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S101028[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8650+ Build/V100R001C92B861) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101029[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101030[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101031[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SN10T1 Build/IML77) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S101032[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S101033[] = "MQQBrowser/25 Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-E860 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101034[] = "Safari/6531.9 CFNetwork/454.4 Darwin/10.0.0 (i386) (MacBookPro5%2C5)";
const char S101035[] = "Mozilla/5.0 Opera/9.5 (KDDI-SN3R; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00";
const char S101036[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.53) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S101037[] = "nokiae66-1/UC Browser7.9.0.102/69/352";
const char S101038[] = "Mozilla/5.0 (X11; U; Linux i686; en; rv:1.9.1.9) Gecko/X Fedora/2.26.3-9.fc11 Epiphany/2.22 Firefox/3.5";
const char S101039[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.14) Gecko/X Firefox/2.0.0.14 Flock/1.1.2";
const char S101040[] = "gt-s5233a/UC Browser7.7.1.88/69/352";
const char S101041[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:5.0) Gecko/X Thunderbird/5.0 Lightning/1.0b5";
const char S101042[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46EX720; CC/XGA; en) Presto/2.7.61 Version/11.00";
const char S101043[] = "Mozilla/5.0 (X11; FreeBSD amd64; rv:7.0) Gecko/X Firefox/7.0";
const char S101044[] = "Mozilla/5.0 (Windows NT 6.0; rv:5.0) Gecko/X Firefox/5.0 Kylo/1.0.1.76141";
const char S101045[] = "SonyEricssonC905/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonc905/UC Browser7.9.0.102/69/352";
const char S101046[] = "samsung-gt-m5650/UC Browser7.9.0.102/69/352";
const char S101047[] = "lg-t325/UC Browser7.9.0.102/69/352";
const char S101048[] = "Nokia7610Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7610supernova/UC Browser8.0.3.107/69/352";
const char S101049[] = "SonyEricssonW995/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonw995/UC Browser8.0.3.107/69/444";
const char S101050[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN78)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S101051[] = "Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:9.2) Gecko/X Firefox/9.2-x64 PaleMoon/9.2-x64";
const char S101052[] = "NOKIA5630d-1/UC Browser8.0.4.121/28/444/UCWEB";
const char S101053[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EVO Build/IML74K; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S101054[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s5350) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S101055[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3 UCBrowser8.3.0.140/42/355";
const char S101056[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A52 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101057[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-b5310) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S101058[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.2.0.1563 Safari/535.11";
const char S101059[] = "MQQBrowser/Mini2.8 (NokiaX2-00/08.25)";
const char S101060[] = "Mozilla/5.0 (Linux; Android 4.1.2; VS870 4G Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S101061[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; s5230) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S101062[] = "Mozilla/5.0 (Linux; U; Android 2.2; es-us; GT-P1000L Build/FROYO)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101063[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101064[] = "Mozilla/5.0 (Linux; U; Android 2.2; fa-fa; GT-P1000 Build/FROYO)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101065[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P5100, 34, 1.6.0)";
const char S101066[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P5100, 36, 1.7.1)";
const char S101067[] = "Instagram 3.2.0 Android (16/4.1.1; 160dpi; 800x1232; samsung; GT-P7300; p5; p3; en_US)";
const char S101068[] = "Instagram 3.2.0 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7300; GT-P7300; p3; en_US)";
const char S101069[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P3110 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7";
const char S101070[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; GT-P3110 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7";
const char S101071[] = "YahooMobileMessenger/1.0 (Android Messenger; 1.8.3) (GT-P6800; samsung; GT-P6800; 3.2/HTJ85B)";
const char S101072[] = "Windowshop/5.50.1010 (Android/4.0.4/GT-P3113)";
const char S101073[] = "Instagram 3.4.1 Android (13/3.2; 160dpi; 800x1232; samsung; GT-P7510; GT-P7510; p3; en_US)";
const char S101074[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259";
const char S101075[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-P1000 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S101076[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P1000 Build/JRO03R) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S101077[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAX5-01)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S101078[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101079[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw150i) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S101080[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw150i) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S101081[] = "nokian70-1/UC Browser7.6.1.82/69/352";
const char S101082[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2) Gecko/X Firefox/3.6 Kylo/0.6.1.70394";
const char S101083[] = "samsung-gt-s5560i";
const char S101084[] = "nokia6680/UC Browser8.0.3.107/69/444";
const char S101085[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en-US; rv:1.9.2.16pre) Gecko/X Prism zdesktop/7.1.4";
const char S101086[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-C_N880S/N880SV1.0.0B04; 480*800; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101087[] = "Mozilla/5.0 (Java; U; en-us; sonyericssonw150i) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S101088[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-E739 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S101089[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAE61i)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S101090[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; LG-P350 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101091[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; U8860 Build/HuaweiU8860) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S101092[] = "Mozilla/5.0 (Linux; Android 4.0.4; MC7403A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S101093[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.1 Chrome/15.0.900.2 Safari/535.2";
const char S101094[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC Vision Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S101095[] = "Nokia5130c-2/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/69/444";
const char S101096[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; R800i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S101097[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Micromax_A87 Build/GINGERBREAD; 480*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444";
const char S101098[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; Spice Mi-425 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S101099[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA5700) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile";
const char S101100[] = "IUC(U;iOS 6.0.1;Zh-cn;320*480;)/UCWEB8.5.0.163/42/997";
const char S101101[] = "MQQBrowser/3.7/Adr (Linux; U; 4.0.3; zh-cn; HUAWEI C8812 Build/C8812V100R001C92B926;480*800)";
const char S101102[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssons312) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S101103[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101104[] = "Mozilla/5.0 (Linux; Android 4.0.3; F-08D Build/V05R11A) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S101105[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  SonyEricssonW960i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101106[] = "SonyEricssonC903/R1GA Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonc903) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S101107[] = "JUC (Linux; U; 2.3.5; zh-cn; GT-N7000; 800*1280) UCWEB8.7.4.225/145/800";
const char S101108[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S101109[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MT15i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S101110[] = "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:18.0) Gecko/X Firefox/18.0 slurp";
const char S101111[] = "Mozilla/5.0 (X11; Linux x86_64; rv:19.0) Gecko/X Thunderbird/19.0";
const char S101112[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 OverDrive Media Console/2.6.4";
const char S101113[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LG-P769 Build/IMM76L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S101114[] = "Mozilla/5.0 (Linux; Android 4.0.4; SHT21 Build/S2150) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S101115[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; Orange San Francisco Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S101116[] = "Mozilla/5.0 (Linux; Android 4.0.4; FJL21 Build/V31R42A) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S101117[] = "Mozilla/5.0 (Linux; Android 4.0.4; L-06D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S101118[] = "Mozilla/5.0 (Linux; Android 4.0.4; F-05E Build/V13R36C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S101119[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:20.0) Gecko/X Thunderbird/20.0";
const char S101120[] = "Mozilla/5.0 (Linux; Android 4.0.4; PAP5000DUO Build/PrestigioPAP5000DUO) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S101121[] = "Mozilla/5.0 (Linux; Android 4.0.4; TAC-70031 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S101122[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx266) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S101123[] = "Mozilla/5.0 (Linux; Android 4.0.4; LG-P936 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S101124[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Sensation XE Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S101125[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-c105) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S101126[] = "Mozilla/5.0 (Linux; Android 4.2.2; Sensation XL Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S101127[] = "Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.3.2m";
const char S101128[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-T679 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S101129[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A860K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S101130[] = "Mozilla/5.0 (Linux; Android 4.2.2; skz_tenderloin Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S101131[] = "Nokia2700c-2/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.5.0.185/83/352";
const char S101132[] = "Mozilla/5.0 (Windows Phone 8.0; Trident/7.0; rv:11.0; IEMobile/11.0; ARM; Touch; HTC; HTCPO881; X) like Gecko";
const char S101133[] = "Mozilla/5.0 (Windows Phone 8.0; ARM; Trident/7.0; Touch; rv:11.0; IEMobile/11.0; HTC; HTCPO881; Chameleon) like Gecko";
const char S101134[] = "BlackBerry8320/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100 UP.Browser/5.0.3.3";
const char S101135[] = "Mozilla/5.0 (Vodafone/1.0/LG-GW520/V10e Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S101136[] = "Mozilla/5.0 (Vodafone/1.0/LG-GW520/V10c Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S101137[] = "LG/KB770/v10c Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S101138[] = "Mozilla/4.0 (Vodafone/1.0/LG-KB770/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)";
const char S101139[] = "LG-KB770/v10d Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S101140[] = "SonyEricssonK310c/R4EA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101141[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101142[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-gb; GT-P1000 Build/ERD62) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S101143[] = "Mozilla/5.0 (Linux; U; Android 2.1; pt-br; GT-P1000L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101144[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-tw; Camangi-FM600 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101145[] = "Vodafone/SonyEricssonV800/R1S025 Browser/SEMC-Browser/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101146[] = "HTC_Diamond Opera/9.70 (Windows NT 5.1; U; en)";
const char S101147[] = "HTC_Touch_Pro2_T7373 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S101148[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) ASUS-P526/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101149[] = "SAMSUNG-GT-B3410/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101150[] = "MOT-Zacubi iTunes/v3.1 MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101151[] = "MOT-V365/08.C8.37R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101152[] = "SAMSUNG-SGH-D500C/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S101153[] = "Nokia6810/2.0 (4.24) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S101154[] = "MOT-ROKR E2/R564_G_12.00.38P Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola ROKR E2; 781) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.50 [en]";
const char S101155[] = "SAMSUNG-SGH-J758/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S101156[] = "HUAWEI-C6100/001.00 ACS-Netfront/3.2";
const char S101157[] = "SPH-M540/1.0 AU-MIC/2.0 (GUI) MMP/2.0";
const char S101158[] = "SPH-M540-parrot/1.0 AU-MIC/2.0 (GUI) MMP/2.0";
const char S101159[] = "Mozilla/5.0 (SCH-F339/F339DD06;U;REX/3.0;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*320;CTC/2.0) POLARIS/6.15.WEB";
const char S101160[] = "SonyEricssonW850iv/R1ED SEMC/NewsReader/1.0";
const char S101161[] = "SonyEricssonW850i/R1KG Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101162[] = "SAMSUNG-SGH-i780/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S101163[] = "SAMSUNG-GT-S7230E/1.0 SHP/VPP/R5 Dolfin/2.0 SMM-MMS/1.2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S101164[] = "Motorola-K1m-boost Obigo/Q04C1 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101165[] = "Motorola-K1m Obigo/Q04C1 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101166[] = "Nokia7070_CMCC/2.0 (06.82) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S101167[] = "SEC-SGHJ610/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101168[] = "Nokia6275i/2.0 (BL200V0200.nep) UP.Browser/6.2.3.8 MMP/2.0";
const char S101169[] = "ZTE Bluebelt II/1.0.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S101170[] = "Lenovo-E268/VN_S105 LMP/LML Release/2010.10.25 Browser/Obigo_Q03c";
const char S101171[] = "Lenovo-E268/CMCC_S100 LMP/LML Release/2009.10.12 Browser/Obigo_Q03c";
const char S101172[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) i-mate 8502; 240x320";
const char S101173[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i455/UMHA3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S101174[] = "SAMSUNG-SGH-E256/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S101175[] = "Nexian NX-G733/MTK Release/10.01.2009 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1";
const char S101176[] = "NokiaN70-1/5.0737.3.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.0.0.41/27/352";
const char S101177[] = "Nokia5230/UC Browser7.2.2.51/50/352";
const char S101178[] = "SAMSUNG-GT-S3600C/1.0 RTK-E/1.0 DF/1.0 Release/07.01.2008 Browser/NetFront3.4  Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101179[] = "SAMSUNG-SGH-C180/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S101180[] = "LG-T320/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S101181[] = "Nokia5230/SymbianOS/9.1 Series60/3.0";
const char S101182[] = "HTC-8100/1.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320)";
const char S101183[] = "LG/U8130/v1.0";
const char S101184[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) i-mate 6150/ 480x640";
const char S101185[] = "Nokia5530/UC Browser7.4.0.65/50/351";
const char S101186[] = "HTC_Touch_Viva_T2223 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65";
const char S101187[] = "Nokia5235/UC Browser7.6.1.82/50/351";
const char S101188[] = "NokiaE66/UC Browser7.6.1.82/28/351";
const char S101189[] = "NokiaN97 mini/UC Browser7.7.1.88/50/444";
const char S101190[] = "SAMSUNG-GT-B5722/B5722DDKA1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S101191[] = "NokiaE52-1/UC Browser7.7.1.88/28/444";
const char S101192[] = "KDDI-MA35 UP.Browser/6.2_7.2.7.1.K.2.225 (GUI) MMP/2.0";
const char S101193[] = "KDDI-KC3Q UP.Browser/6.2.0.15.1.1 (GUI) MMP/2.0";
const char S101194[] = "DoCoMo/2.0 F10B(c500;TB;W24H16)";
const char S101195[] = "NOKIA6120c/UC Browser7.4.0.65/28/352";
const char S101196[] = "Flipboard/1.5.1 CFNetwork/485.13.9 Darwin/11.0.0";
const char S101197[] = "Nokia6681/1.0 (2.30.0) Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; baiduspider; +http://www.baidu.com/search/spider.html)";
const char S101198[] = "Mozilla/5.0 (QtEmbedded; U; Linux; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Kobo eReader Safari/532.4";
const char S101199[] = "Mozilla/5.0 (QtEmbedded; U; Linux; C -) AppleWebKit/532.4 (KHTML, like Gecko) Kobo eReader Safari/532.4";
const char S101200[] = "MQQBrowser/2.0 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3";
const char S101201[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; md-us; M009F Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101202[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.13) Gecko/X Fedora/3.5.15-1.fc12 Firefox/3.6.13 GTB7.1";
const char S101203[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Ubuntu/9.10 Chromium/10.0.647.0 Chrome/10.0.647.0 Safari/534.16";
const char S101204[] = "MQQBrowser/2.0 (NokiaX6-00;SymbianOS/9.1 Series60/3.0)";
const char S101205[] = "MQQBrowser/2.0 (Nokia5800XpressMusic;SymbianOS/9.1 Series60/3.0)";
const char S101206[] = "MQQBrowser/2.0 (Nokia5800wXpressMusic;SymbianOS/9.1 Series60/3.0)";
const char S101207[] = "Nokia5230/UCWEB8.3.0.133/50/999";
const char S101208[] = "SAGEM-my511X-orange/1.0/ MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.393 (GUI)";
const char S101209[] = "Nokia202/2.0 (20.28) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia202) UCBrowser8.2.1.144/70/355/UCWEB Mobile";
const char S101210[] = "MQQBrowser/31 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3";
const char S101211[] = "MQQBrowser/31 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3";
const char S101212[] = "Opera/6.11 (FreeBSD 4.7-RELEASE i386; U) [en]";
const char S101213[] = "acer_E101 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5)";
const char S101214[] = "Mozilla/5.0 (Macintosh; U; X 10_6_6; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.127 Safari/534.16";
const char S101215[] = "MQQBrowser/2.0 (MAUI WAP Browser)";
const char S101216[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; GT-I6500U Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S101217[] = "JUC(Linux;U;Android2.3.4;Zh_cn;HTC Sensation Z710e;540*960;)UCWEB7.8.0.95/139/352";
const char S101218[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9300 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S101219[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; hu-hu; xPAD-70 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S101220[] = "MQQBrowser/2.5.1 Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/7534.48.3";
const char S101221[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; ST15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101222[] = "iBrowser/Mini2.8 (SAMSUNG-GT-S5253/S5253DDKJ2)";
const char S101223[] = "Mail936 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac8%2C1)";
const char S101224[] = "NokiaE7-00/UC Browser8.0.4.121/50/352/UCWEB";
const char S101225[] = "MQQBrowser/32 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3";
const char S101226[] = "Mozilla/5.0 (Linux; Android 4.0.3; T708 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S101227[] = "NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/70/355";
const char S101228[] = "sonyericssonw350i/UC Browser7.9.0.102/69/352";
const char S101229[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/70/444";
const char S101230[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI U8661 Build/HuaweiU8661) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101231[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s5333) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S101232[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s3650) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S101233[] = "JUC(Linux;U;Android2.3.5;Zh_cn;A56;320*480;)UCWEB7.8.0.95/139/444";
const char S101234[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Wildfire S A510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101235[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian72) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S101236[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S101237[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; GT-P1000 Build/IMM76L) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7";
const char S101238[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000T Build/IMM76L) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7";
const char S101239[] = "YahooMobile/1.0 (mail; 1.4.4.12415); (Linux; U; Android 3.1; GT-P7500 Build/HONEYCOMB_MR1);";
const char S101240[] = "YahooMobile/1.0 (mail; 1.3.8.9816); (Linux; U; Android 3.1; GT-P7500R Build/HONEYCOMB_MR1);";
const char S101241[] = "YahooMobile/1.0 (mail; 2.0.4.15173); (Linux; U; Android 3.1; GT-P7500M Build/HONEYCOMB_MR1);";
const char S101242[] = "YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 3.1; GT-P7500D Build/HONEYCOMB_MR1);";
const char S101243[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6800 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S101244[] = "Instagram 3.0.5 Android (13/3.2; 160dpi; 600x976; samsung; GT-P6200; GT-P6200; smdkc210; en_GB)";
const char S101245[] = "YahooMobileMail/1.0 (Android Mail; 1.4.5) (espressowifi;samsung;GT-P3110;4.0.3/IML74K)";
const char S101246[] = "Android/4.0.4; samsung GT-P7500; 2.3.2";
const char S101247[] = "Rdio.Net; Android/2.5; Android/4.0.4; DM=samsung; DN=GT-P7500R; DC=samsung; DF=samsung/GT-P7500R/GT-P7500R:4.0.4/IMM76D/RUXLH2:user/release-keys; DH=p3";
const char S101248[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7500 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S101249[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6810 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0000, gb";
const char S101250[] = "Dalvik/1.6.0 (Linux; U; Android Jelly Beast 4.2; GT-P3100 Build/JZO54K)";
const char S101251[] = "Dalvik/1.6.0 (Linux; U; Android Jelly Bean 4.1.1; GT-P3100 Build/JRO03C)";
const char S101252[] = "Dalvik/1.6.0 (Linux; U; Android Jelly Bean 4.1.2; GT-P3100 Build/JZO54K)";
const char S101253[] = "nApps (Android OS 4.0.4; GT-P3100; NaverSearch; 4.3.0)";
const char S101254[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7310 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7310, 35, 1.7.0)";
const char S101255[] = "WhatsApp/2.9.296 Android/4.0.3 Device/samsung-GT-P5100";
const char S101256[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S101257[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S101258[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S101259[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S101260[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S101261[] = "Mozilla/5.0 (Linux; Android 4.0.3; Ultimate10-Android4.0 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S101262[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-s310) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S101263[] = "Mozilla/5.0 (S60V3; U; en-us; Nokia5730S-1)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S101264[] = "One X Linux/2.6.35 Android/4.0.3 Release/06.08.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1";
const char S101265[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; u8650 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S101266[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE C1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101267[] = "JUC (Linux; U; 2.3.5; zh-cn; MI-ONE Plus; 480*854) UCWEB8.1.1.122/145/32504";
const char S101268[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P505 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S101269[] = "JUC(Linux;U;2.3.4;Zh_cn;ST18i;480*854;)UCWEB7.6.0.75/139/32037";
const char S101270[] = "JUC(Linux;U;Android4.0.3;Zh_cn;HTC Desire C;320*480;)UCWEB7.8.0.95/139/444";
const char S101271[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Wildfire S Build/MIUI; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101272[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; OPPOX907 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S101273[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID SPYDER Build/IMM76D; CyanogenMod-BLACK WIDOW ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S101274[] = "Instagram 3.0.0 (iPhone; iPhone OS 4.2.1; en_US) AppleWebKit/420+";
const char S101275[] = "Mozilla/5.0 (Java; U; en-us; AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile SonyEricssonP1i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101276[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S101277[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5603) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S101278[] = "HTC_Touch_Cruise_T4242 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S101279[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC_P3650)";
const char S101280[] = "HTC_P3650 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S101281[] = "MOT-V551/08.18.16R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S101282[] = "MOT-V551J/08.18.40R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S101283[] = "SAMSUNG-SGH-U808/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S101284[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 640x480; HTC_X7500/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101285[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 640x480; Ameo 16GB/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101286[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 640x480; Ameo 16GB/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101287[] = "SAMSUNG-SGH-E640/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S101288[] = "MOT-Z6m/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0";
const char S101289[] = "Motorola-Z6m UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0";
const char S101290[] = "SAMSUNG-SGH-T309/T309UVEI1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S101291[] = "LG/U310/v1.0";
const char S101292[] = "SAMSUNG-SGH-E420/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S101293[] = "SEC-SGHD800/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101294[] = "Alcatel-OT-S621/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S101295[] = "Alcatel-OT-S621/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C, Alcatel-OT-S621/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S101296[] = "Alcatel-OT-S621A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S101297[] = "LG-KM710 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S101298[] = "SonyEricssonK850i/R1FA SEMC/NewsReader/1.0";
const char S101299[] = "LG-KF240 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S101300[] = "PANTECH-C630/IUUS10272008 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101301[] = "PANTECH-C630/IUUSX Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101302[] = "MOT-C975/82.21.91I MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101303[] = "SonyEricssonW810i/R1BC Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101304[] = "SAMSUNG-SGH-L600/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S101305[] = "Spice_M6400/WapBrowserConfiguration2.0/MIDP2.0/CLDC1.1";
const char S101306[] = "MOT-W233/2.0.60 UP.Browser/6.3.0.6.c.33 (GUI) MMP/2.0";
const char S101307[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-tw; Vibo-A699 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S101308[] = "Mozilla/5.0(Linux;U;Android1.6;Zh_cn;X10i;480*854;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355";
const char S101309[] = "JUC(Linux;U;Android1.6;Zh_cn;X10i;480*854;)UCWEB7.8.0.95/139/444";
const char S101310[] = "LG/KU950/v1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101311[] = "SAMSUNG-GT-B7320/1.0 MIDP/2.1 CLDC/1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S101312[] = "ZTE silverbelt/1.0.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S101313[] = "HTCS621-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 320x240)";
const char S101314[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonSO-01B Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101315[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SO-01B Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101316[] = "HTC-S620/5.2.968/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone)/UC Browser7.2.2.51";
const char S101317[] = "HTC_S620 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone)/UC Browser7.2.2.51";
const char S101318[] = "AIKO 70W/WAP2.0";
const char S101319[] = "Nokia2865/2.0 (FL190V0200.nep) UP.Browser/6.2.3.8 MMP/2.0";
const char S101320[] = "SAMSUNG-SGH-P250-ORANGE/P250BVHI2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S101321[] = "ZTE-E850(ChinaUnicom)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101322[] = "Huawei/1.0/0HuaweiG6610/SW03 Browser/Obigo-Browser/Q03C MMS/Obigo-MMS/1.2";
const char S101323[] = "LG-GD310_CMCC Release/9.25.2009 Browser/Teleca-Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101324[] = "ZTE-F156/1.0 ACS-NF/3.2 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101325[] = "LG-KE500 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S101326[] = "NokiaC6-00/UC Browser7.4.0.65/50/351";
const char S101327[] = "NokiaE52-1/UC Browser7.6.1.82/28/352";
const char S101328[] = "NokiaE66/UC Browser7.8.0.95/28/358";
const char S101329[] = "Nokia5310XpressMusic/2.0 (08.32) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5310xpressmusic/UC Browser7.9.0.102/69/352";
const char S101330[] = "Vodafone/1.0/HTC_Mercury/1.20.161.6/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; MIDP-2.0 Configuration/CLDC-1.1; PPC; 240x320)";
const char S101331[] = "Nokia6110";
const char S101332[] = "KDDI-HI3C UP.Browser/6.2.0.13.2 (GUI) MMP/2.0";
const char S101333[] = "SoftBank/1.0/812SHs/SHJ001 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101334[] = "SoftBank/1.0/812SH/SHJ002 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101335[] = "SoftBank/1.0/821SH/SHJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101336[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_X960";
const char S101337[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PG86100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101338[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PG86100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101339[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-gb; i-mobile IE 6010 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S101340[] = "Maven/1.9 CFNetwork/548.1.4 Darwin/11.0.0";
const char S101341[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; lt-lt; MID7108 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101342[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.1.6) Gecko/X FireDownload/2.0.1 Firefox/3.0.9 Wyzo/3.0.3 FireTorrent/2.0.1";
const char S101343[] = "Mozilla/5.0 (Windows NT 5.0; rv:5.0) Gecko/X Thunderbird/5.0";
const char S101344[] = "MobileRSS/3.4 CFNetwork/485.12.7 Darwin/10.4.0";
const char S101345[] = "FreeRSSReader/1.9.5 CFNetwork/485.12.7 Darwin/10.4.0";
const char S101346[] = "Reeder/2.5.4 CFNetwork/548.1.4 Darwin/11.0.0";
const char S101347[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S101348[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Mecha Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101349[] = "sonyericssonk750i/UC Browser8.0.3.107/69/444";
const char S101350[] = "sonyericssonk750i/UC Browser8.0.3.107/69/352";
const char S101351[] = "MQQBrowser/27 (iOS; U; CPU like Mac OS X; zh-cn)";
const char S101352[] = "MQQBrowser/261 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405";
const char S101353[] = "Mozilla/5.0 (Linux; U; Android 4.0; en-us; SGH-T839 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101354[] = "Opera/9.80 (X11; OpenBSD i386; U; en) Presto/2.2.15 Version/10.11";
const char S101355[] = "JUC(Linux;U;2.2.1;Zh_cn;GT-S5830;320*480;)UCWEB7.7.0.85/139/800";
const char S101356[] = "JUC(Linux;U;Android2.2;Zh_cn;GT-I5801;240*400;)UCWEB7.8.0.95/140/444";
const char S101357[] = "sonyericssonk810i/UC Browser7.9.0.102/69/352";
const char S101358[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-T989 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S101359[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; GT-I5503 Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S101360[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-ru; GSmart-G1305 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S101361[] = "Mozilla/5.0 (Linux; U; Android 2.2.5; en-us; M3 media box board Build/DTV) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101362[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; KIRA_N9000 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101363[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; pl-pl; Android for A701 Board Build/A701) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101364[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-7000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101365[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; YP-G70 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S101366[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT-7000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101367[] = "JUC(Linux;U;Android2.2.2;Zh_cn;ZTE-BLADE;480*800;)UCWEB7.8.0.95/139/444";
const char S101368[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.6 (KHTML, like Gecko) Ubuntu/9.10 Chromium/7.0.507.0 Chrome/7.0.507.0 Safari/534.6";
const char S101369[] = "Mozilla/4.0 (compatible; MSIE 6.0; Googlebot/2.1; +http://www.google.com/bot.html)";
const char S101370[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL22EX320; CC/GBR; en) Presto/2.7.61 Version/11.00";
const char S101371[] = "Mozilla/5.0 (S60V3; U; en-in; NOKIAN82)/UC Browser8.4.0.159/28/352/UCWEB Mobile";
const char S101372[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN85)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S101373[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101374[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Incredible S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S101375[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V788D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101376[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_C8810 Build/C8810V100R001C92B862) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101377[] = "LG-E500/V100 Obigo/Q7.3 MMS/LG-MMS-V1.1/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S101378[] = "MQQBrowser/3.5/Adr (Linux; U; 2.3.5; zh-cn; HTC Incredible S Build/GRJ90;480*800)";
const char S101379[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; LG-P350 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101380[] = "MQQBrowser/30 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S101381[] = "Mozilla/5.0 (Linux; Android 4.1.1; PC36100 Build/black bean AOKP) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S101382[] = "JUC (Linux; U; 2.3.5; zh-cn; DOOV_D700; 320*480) UCWEB7.9.3.103/139/32669";
const char S101383[] = "Spice M5262";
const char S101384[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; A56 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S101385[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-t310i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S101386[] = "YahooMobile/1.0 (mail; 1.4.5.12736); (Linux; U; Android 2.3.3; GT-P1000 Build/GINGERBREAD_MR1);";
const char S101387[] = "YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 2.3.3; GT-P1000R Build/GINGERBREAD_MR1);";
const char S101388[] = "YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 2.3.3; GT-P1000M Build/GINGERBREAD_MR1);";
const char S101389[] = "Instagram 3.0.5 Android (10/2.3.3; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; gt-p1000; en_GB)";
const char S101390[] = "Instagram 3.0.5 Android (8/2.2; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; GT-P1000; en_GB)";
const char S101391[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S101392[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S101393[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7500R, 36, 1.7.1)";
const char S101394[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7500, 36, 1.7.1)";
const char S101395[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7500D, 40, 1.8.5)";
const char S101396[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; GT-P6800 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S101397[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-P7510 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.16/276,1.9.16.276,2012-12-21 16:37, +0000, gb";
const char S101398[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7510 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, -0500, us";
const char S101399[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7510 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, -0500, us";
const char S101400[] = "WhatsApp/2.9.369 Android/4.0.4 Device/samsung-GT-P3100";
const char S101401[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-CN; GT-P6200 Build/HTJ85B) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S101402[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P5110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S101403[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P5110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S101404[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S101405[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P3110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S101406[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7500 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S101407[] = "Opera/9.00 (Windows ME; U; en)";
const char S101408[] = "JUC (Linux; U; 2.3.6; zh-cn; ZTE_V889D; 480*800) UCWEB7.9.4.145/139/800";
const char S101409[] = "MQQBrowser/Mini2.8 (Nokia311/03.81)";
const char S101410[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonck13i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S101411[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101412[] = "Opera/9.80 (Linux mips; U; CE-HTML/1.0 NETTV/3.0.1; PHILIPS-AVM-2012; en) Presto/2.9.167 Version/11.50";
const char S101413[] = "NokiaX2-02/2.0 (11.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/70/355";
const char S101414[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I5801 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S101415[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Karbonn_A18 Build/KarbonnA18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S101416[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S101417[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3653w) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S101418[] = "Nokia7310/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7310) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S101419[] = "Instagram 3.0.1 (iPod touch; iPhone OS 6.0; en_US) AppleWebKit/420+";
const char S101420[] = "SonyEricssonW705/R1GA Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S101421[] = "SonyEricssonC903/R1GA Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S101422[] = "SonyEricssonT715/R1HA Browser/NetFront Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.5";
const char S101423[] = "SonyEricssonT715a/R1GA SEMC/NewsReader/1.0";
const char S101424[] = "BlackBerry8120/4.3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100 UP.Browser/5.0.3.3";
const char S101425[] = "SonyEricssonP1i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101426[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) ASUSP550/1.0";
const char S101427[] = "SAMSUNG-GT-S3653/S365MDDIJ1";
const char S101428[] = "samsung-gt-s3653";
const char S101429[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) O2 Xda Zinc";
const char S101430[] = "SAMSUNG-SGH-T209/T209UVFD3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S101431[] = "SAMSUNG-SGH-T209R/T209UVFG1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S101432[] = "SAMSUNG-GT-B7610/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S101433[] = "SAGEM-myC5-2v/1.0 UP.Browser/6.2.3.3.g.2.107 (GUI) MMP/2.0";
const char S101434[] = "SAGEM-myC5-2v/1.0 UP.Browser/6.2.3.3.g.2.106 (GUI) MMP/2.0";
const char S101435[] = "SAGEM-myC5-2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.6.d.5 (GUI) MMP/1.0";
const char S101436[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; O2 Xda Atom; PPC; 240x320)";
const char S101437[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SAMSUNG-GT-i8000V; Windows Phone 6.5)";
const char S101438[] = "Mozilla/4.0 SAMSUNG-GT-i8000H/1.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5)";
const char S101439[] = "LG-TU915 Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101440[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC i-mate JAMA; 240x320; PPC)";
const char S101441[] = "SAMSUNG-GT-C3300K/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S101442[] = "LG-KM330 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S101443[] = "MOT-V3am/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0";
const char S101444[] = "Motorola_V3c UP.Browser/6.2.3.4.c.1.109 (GUI) MMP/2.0";
const char S101445[] = "SAMSUNG-SGH-E890/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S101446[] = "Mozilla/5.0 (compatible; MSIE 6.0; Polaris 6.15.web; Brew 3.1.5; U; en) LG-LG235/1.0";
const char S101447[] = "SonyEricssonC901/R1GA SEMC/NewsReader/1.0";
const char S101448[] = "LG-KF245 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S101449[] = "UTS-7126M/1.0 UP.Browser/6.3.0.7 (GUI) MMP/2.0";
const char S101450[] = "HPiPAQrw6800/1.0/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)";
const char S101451[] = "LG-GB210 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S101452[] = "NokiaX5-00_TD/CMCC SymbianOS/9.3 Series60/3.2 Release/11.1026.019; Mozilla/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML,like Gecko) Safari/413";
const char S101453[] = "LG-GB170/V01 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S101454[] = "SAMSUNG-SGH-F266/ Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S101455[] = "SAMSUNG-GT-S3310C/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S101456[] = "SAMSUNG-SGH-V820L/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S101457[] = "SonyEricssonW150/R2AD034 TelecaBrowser/Q07C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101458[] = "SAMSUNG-GT-C3730C_CMCC/1.0 RTK-E/1.0 SHP/VPP-R5 Release/8.17.2009 Browser/NetFront3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1 SMM-MMS/1.2.0";
const char S101459[] = "ZTE-U E880/Obigo Q03c/MIDP-2.0 Configuration/CDLC-1.1";
const char S101460[] = "SAMSUNG-SGH-i908E_CMCC/1.0 Release/03.07 Browser/IE6 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S101461[] = "SCH-M519/ POLARIS/6.15 (GUI; compatible; UP.Browser)";
const char S101462[] = "Mozilla/5.0 (SCH-W589/W589DK26;U;REX/3.0;BREW/3.1.5;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*320;CTC/2.0) POLARIS/6.15.WEB";
const char S101463[] = "LG/KU800/v1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101464[] = "SAMSUNG-SGH-J700/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S101465[] = "SoftBank/1.0/920SC/SCJ002 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101466[] = "Mozilla/4.08 (920SC;SoftBank) NetFront/3.3";
const char S101467[] = "Dopod838Pro/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; Dopod838Pro)";
const char S101468[] = "NokiaE72-1/UC Browser7.4.0.65/28/355";
const char S101469[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) Toshiba G910_G920";
const char S101470[] = "LGE-MX8500/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S101471[] = "LG-GD310/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S101472[] = "LG-KG130 Obigo/WAP2.0";
const char S101473[] = "SAGEM-VF540/1.0/EVN_MK MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.1.576 (GUI)";
const char S101474[] = "Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5130c-2/UC Browser7.6.1.82/70/351";
const char S101475[] = "Nokia6681/2.0 (6.09.00) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.4.0.65/27/351";
const char S101476[] = "HTC_Touch_HD_T8282 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65";
const char S101477[] = "Nokia5230/UCWEB7.6.0.75/50/800";
const char S101478[] = "Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5130c-2/UC Browser7.7.1.88/70/351";
const char S101479[] = "sonyericssonk800i/UC Browser7.7.1.88/69/444";
const char S101480[] = "sonyericssonk800i/UC Browser7.7.1.88/69/352";
const char S101481[] = "Nokia6630/1.0 (5.03.08) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/27/351";
const char S101482[] = "NokiaN72/S024 LMP/2010.06.09 Release/Obigo2.0 Profile/MIDP-2.0 Configuaration/CLDC-1.1";
const char S101483[] = "SonyEricssonT200/R101";
const char S101484[] = "DoCoMo/2.0 SH903iTV(c100;TB;W24H16)";
const char S101485[] = "KDDI-TS3E UP.Browser/6.2.0.13.1.5 (GUI) MMP/2.0";
const char S101486[] = "KDDI-TS3G UP.Browser/6.2.0.13.2 (GUI) MMP/2.0";
const char S101487[] = "TIANYU-KTOUCH/A7726/MIDP2.0/CLDC1.1/Screen-240x320";
const char S101488[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.0 Chrome/13.0.800.0 Safari/535.1";
const char S101489[] = "NokiaE72-1/UCWEB8.1.0.104/28/800";
const char S101490[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.8.1";
const char S101491[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Flipboard/1.8.1";
const char S101492[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Flipboard/1.8.2";
const char S101493[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.8.4";
const char S101494[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.8.4";
const char S101495[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Flipboard/1.8.4";
const char S101496[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.8.3";
const char S101497[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.8.3";
const char S101498[] = "JUC(Linux;U;2.2;Zh_cn;bravo;480*800;)UCWEB7.4.1.61/139/800";
const char S101499[] = "Mozilla/4.0 (compatible; Opera 4.0)";
const char S101500[] = "HTC_P3400i-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65";
const char S101501[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) WS027SH";
const char S101502[] = "ASUS-GalaxyMini/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95";
const char S101503[] = "MQQBrowser/26 (U; CPU like Mac OS X; zh-cn)";
const char S101504[] = "Dolphin http client/8.1.0 beta2(135) (Android)";
const char S101505[] = "Dolphin http client/8.1.0(136) (Android)";
const char S101506[] = "Dolphin http client/8.1.1(137) (Android)";
const char S101507[] = "Dolphin http client/8.1.0 beta(133) (Android)";
const char S101508[] = "MQQBrowser/Mini2.2 (NokiaX2-01/08.63)";
const char S101509[] = "MediaGo/2.1.0.392 (Windows NT 6.1; WOW64) PlayStationStore/4.7.14.14146 PlaybackEngine/1.88.106.12050";
const char S101510[] = "MediaGo/2.2.0.223 (Windows NT 6.1; WOW64) PlayStationStore/4.9.4.14625 PlaybackEngine/1.92.168.06150";
const char S101511[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; xPAD-70 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101512[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; Micromax A60 Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S101513[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.5.9225.201";
const char S101514[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.7.10246.201";
const char S101515[] = "Mozilla/4.0 (compatible; MSIE 5.0; UNIX) Opera 6.11 [en]";
const char S101516[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5 Comodo_Dragon/19.1.0.0";
const char S101517[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5 Comodo_Dragon/19.1.0.0";
const char S101518[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; T9295)";
const char S101519[] = "JUC(Linux;U;2.2;Zh_cn;HTC Desire;480*800;)UCWEB7.5.0.66/139/800";
const char S101520[] = "JUC(Linux;U;2.2;Zh_cn;Desire HD;480*800;)UCWEB7.6.0.75/139/800";
const char S101521[] = "Mozilla/4.0 (jig browser web; 1.0.4; SN3O)";
const char S101522[] = "Mozilla/5.0 (compatible; Konqueror/4.6) KHTML/4.6.0 (like Gecko) SUSE";
const char S101523[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; Micromax A60 Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S101524[] = "iBrowser/Mini2.7 (Nokia2690/10.10)";
const char S101525[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 Sleipnir/3.5.2";
const char S101526[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; T8 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101527[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Milestone Build/SHOLS_U2_05.26.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S101528[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; N860 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S101529[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S101530[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S101531[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; ZTE-BLADE Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101532[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I897 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S101533[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; MTS-SP100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S101534[] = "Android/1.0 (TF101 IML74K)";
const char S101535[] = "JUC(Linux; U;2.3.4;zh_cn;HTC S710d;480*800;)UCWEB7.6.0.75/139/7701";
const char S101536[] = "JUC(Linux;U;2.3.5;Zh_cn;GT-I9100;480*800;)UCWEB7.8.1.96/139/800";
const char S101537[] = "Mozilla/5.0 (S60V5; U; en-us; SamsungI8910)/UC Browser8.5.0.183/50/352/UCWEB Mobile";
const char S101538[] = "Opera/9.80 (Windows NT 4.0; U; en) Presto/2.6.30 Version/10.63";
const char S101539[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-US) AppleWebKit/530.10 (KHTML, like Gecko) Chrome/2.0.181 Safari/530.10";
const char S101540[] = "Mozilla/5.0 (S60V3; U; en-in; NOKIA5320)/UC Browser8.4.0.159/28/352/UCWEB Mobile";
const char S101541[] = "Mozilla/5.0 (S60V5; U; en-us; Nokia603)/UC Browser8.5.0.183/50/352/UCWEB Mobile";
const char S101542[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us ; PD_Novel Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/405";
const char S101543[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL32EX721; CC/ESP; en) Presto/2.7.61 Version/11.00";
const char S101544[] = "Mozilla/5.0 (S60V3; U; kau; NokiaE75-1)/UC Browser8.4.0.159/28/352/UCWEB Mobile";
const char S101545[] = "Reeder/1010.79.00 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro5%2C5)";
const char S101546[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA5630d-1)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S101547[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Y2 Build/HCL ME Tablet Y2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S101548[] = "Mozilla/5.0 (Linux; Android 4.0.3; N90 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S101549[] = "MQQBrowser/2.5 (E63-1;SymbianOS/9.2 Series60/3.1)";
const char S101550[] = "NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac1-02) UCBrowser8.3.0.154/70/444/UCWEB Mobile";
const char S101551[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/5.1 Mobile/8B117 Safari/7534.48.3 UCBrowser8.4.0.159/41/355";
const char S101552[] = "UCWEB/2.0 (Java; U; MIDP-2.0; vi; samsung-gt-s5753e) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S101553[] = "Android OS/12 samsung_GT-P7510";
const char S101554[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.2.139/145/355";
const char S101555[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-P1000 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S101556[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7300 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S101557[] = "Instagram 3.0.5 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P3100; espressorf; espresso; en_GB)";
const char S101558[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7510, 40, 1.8.5)";
const char S101559[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P5110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S101560[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P5110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S101561[] = "Android/4.0.3; samsung GT-P5113; 2.3.1";
const char S101562[] = "eHarmony-Android/2.1.7 (GT-P5113; Android OS 4.0.4; en_US; id null)";
const char S101563[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P1000 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:34, es";
const char S101564[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT-P1000L Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/292,1.9.23.292,2013-02-06 08:41, -0300";
const char S101565[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P1000 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/285,1.9.19.285,2013-01-24 11:19, +0800, china, cn";
const char S101566[] = "Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P6210 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, -0500, ca";
const char S101567[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1000, 36, 1.7.1)";
const char S101568[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1000, 36, 1.7.1)";
const char S101569[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; GT-P1000L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1000L, 39, 1.8.2)";
const char S101570[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-P1000M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1000M, 39, 1.8.2)";
const char S101571[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; el-gr; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1000, 39, 1.8.2)";
const char S101572[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1000, 40, 1.8.5)";
const char S101573[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000N Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1000N, 41, 1.9.0)";
const char S101574[] = "Android/4.0.3; samsung GT-P5100; 2.3.2";
const char S101575[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S101576[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P6810 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S101577[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6800 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S101578[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P7510 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S101579[] = "theScore/3.1.1 Android-OS/3.1 (Android; Tablet, GT-P7500D)";
const char S101580[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S880 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S101581[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S101582[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; folio100 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S101583[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I897 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S101584[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOU701 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101585[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; Desire HD Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S101586[] = "JUC (Linux; U; 2.3.7; zh-cn; Blade; 480*800) UCWEB7.9.4.145/139/800";
const char S101587[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-P100 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S101588[] = "JUC (Linux; U; 2.3.6; zh-cn; MB526; 480*854) UCWEB7.9.4.145/139/800";
const char S101589[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; ST18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101590[] = "SAMSUNG-GT-P5113/100.40004";
const char S101591[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOR805 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101592[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101593[] = "Mozilla/5.0 (Linux; Android 4.0.3; DIT1010 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S101594[] = "iBrowser/Mini2.8 (SAMSUNG-GT-S5620/S5620DDKC1)";
const char S101595[] = "MQQBrowser/36 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3";
const char S101596[] = "Nokia2700c-2/2.0 (09.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/70/355";
const char S101597[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-lg800g) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S101598[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-S7275 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S101599[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-S7275 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S101600[] = "Nokia3500c/2.0 (05.51) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3500c) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S101601[] = "Nokia5610d-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5610d-1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S101602[] = "MOT-EM325/1.0 Release/12.20.2008 Browser/CMCS1.0 Software/1.300 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101603[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320; HTC_P3300/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101604[] = "BlackBerry8830/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105 UP.Browser/5.0.3.3";
const char S101605[] = "SAMSUNG-SGH-I907/UCJD1 Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S101606[] = "FarEasTone/1.1/SharpWXT923/SHS001 Mozilla/5.00 (compatible) Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101607[] = "FarEasTone/1.0/SharpWXT92/SHS001 Mozilla/4.08 (compatible) Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101608[] = "SonyEricssonT700/R3EA SEMC/NewsReader/1.0";
const char S101609[] = "SIE-EL71/31 UP.Browser/7.1.0.e.24(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.e.24 (GUI) MMP/2.0";
const char S101610[] = "SIE-EL71/41 UP.Browser/7.1.0.k.6(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.k.6 (GUI) MMP/2.0";
const char S101611[] = "SIE-EL71/41 UP.Browser/7.1.0.k.6(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101612[] = "SIE-EL71/12 UP.Browser/7.1.0.e.18(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.e.18 (GUI) MMP/2.0";
const char S101613[] = "SIE-EL71/22 UP.Browser/7.1.0.e.23(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.e.23 (GUI) MMP/2.0";
const char S101614[] = "Nexian NX-G505/MTK Release/10.01.2009 Browser/MAUI Profile/No Java";
const char S101615[] = "NokiaC5-01_TD/CMCC SymbianOS/9.3 Series60/3.2 Release/20.104603; Mozilla/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML,like Gecko) Safari/413";
const char S101616[] = "SonyEricssonW908c/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S101617[] = "Indie/Indie_B07A102 Obigo/Q03C";
const char S101618[] = "Indie/Indie_SF_B07A101 Obigo/Q03C";
const char S101619[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-AURA/R6639_G_81.20.5FR_A BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony 1.0";
const char S101620[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-AURA/R6639_G_81.21.06R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony %bv";
const char S101621[] = "Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-AURA/R6639_G_81.20.75R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony 1.0";
const char S101622[] = "Samsung-SPHM300 AU-MIC-M300/2.0 MMP/2.0";
const char S101623[] = "Samsung-SPHM300 AU-MIC/2.0 MMP/2.0";
const char S101624[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) O2 Xda Atom Life";
const char S101625[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) MWG Atom Life";
const char S101626[] = "SAMSUNG-SGH-X480/1.0";
const char S101627[] = "SAMSUNG-SGH-X480C/1.0";
const char S101628[] = "SonyEricssonK300c/R2BA SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101629[] = "Panasonic-VS6/1.0/RP1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101630[] = "LGE-VX8800/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S101631[] = "Huawei/1.0/G5730/Browser/obigo-browser/q03c";
const char S101632[] = "HTC_P4350 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S101633[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320;MWg Atom V";
const char S101634[] = "SCH-U810/1.0 NetFront/3.0.22.1.14 (GUI) MMP/2.0";
const char S101635[] = "BlackBerry8130/4.3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/126 UP.Browser/5.0.3.3";
const char S101636[] = "MOT-V975/82.31.56I MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101637[] = "PPC; 240x320; HTC_P3450/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S101638[] = "China Unicom/1.1/SH0902C/SHS001 Mozilla/5.00 (compatible) Browser/NF/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101639[] = "LG-KF757-Orange/v10d Browser/Obigo-Q05A/3.12MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S101640[] = "SEC-SGHL320/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101641[] = "Nokia6300/2.0 (07.00) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6300/UC Browser7.8.0.95/70/401";
const char S101642[] = "Huawei/1.0/U1000/B000 Browser/Obigo-Browser";
const char S101643[] = "MOT-W450/0.1.72.K1/12.24.2008 Browser/UPB6.3 Software/13.119I";
const char S101644[] = "NokiaC1-01/2.0 (03.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac1-01/UC Browser7.8.0.95/70/401";
const char S101645[] = "Zte-tu208_TD/1.0 ThreadX/1708ULC Larena/2.00 Release/9.25.2010 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101646[] = "LG-TU330/V10a Teleca/Q7.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101647[] = "Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Motorola A1000;708) Opera 7.50 [en]./A1000./1.0";
const char S101648[] = "Videocon_V1535/WAP2.0 Profile/MT6253/QCIF";
const char S101649[] = "LENOVO-A310/(2009.04.17) S010/WAP2.0";
const char S101650[] = "Nokia5730S-1";
const char S101651[] = "Mozilla/5.0 (Linux; U; Android 1.6; zh-tw; TaiwanMobileT1 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S101652[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; eee_701 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S101653[] = "LG-GM630 Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S101654[] = "i-mobileS281";
const char S101655[] = "SAGEM-my750X/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101656[] = "W9229ObigolnternetBrowser/QO3C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101657[] = "DoCoMo/2.0 SO903i(c100;TB;W24H18)";
const char S101658[] = "SAMSUNG-M2710/M2710DDIF4 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S101659[] = "SAMSUNG-SGH-J700L/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S101660[] = "LGE-VX9400/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S101661[] = "LGE-VX7100/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S101662[] = "SonyEricssonJ10i2/R7BA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2";
const char S101663[] = "SHARP-TQ-GX30/1.0 Profile/MIDP-1.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.6.c.1.104 (GUI) MMP/1.0";
const char S101664[] = "SHARP-TQ-GX30i/1.0 Profile/MIDP-1.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.6.c.1.104 (GUI) MMP/1.0";
const char S101665[] = "NokiaN97/UC Browser7.4.0.65/50/351";
const char S101666[] = "Nokia6680/1.0 (5.04.07) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.4.0.65/27/352";
const char S101667[] = "NokiaX2-00/5.0 (04.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-00/UC Browser7.6.1.82/69/352";
const char S101668[] = "Nokia2700c-2/2.0 (09.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2700c-2/UC Browser7.7.1.88/70/352";
const char S101669[] = "NOKIA5320/UC Browser7.7.1.88/28/444";
const char S101670[] = "Nokia5235/UC Browser7.8.0.95/50/355";
const char S101671[] = "Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_6) AppleWebKit (KHTML, like Gecko) Chrome/1";
const char S101672[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/1.0 Safari/533.3";
const char S101673[] = "Apple iPhone v1.1.4 CoreMedia v1.0.0.4A102";
const char S101674[] = "Apple iPhone v1.1.5 CoreMedia v1.0.0.4B1";
const char S101675[] = "Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Nokia 6600/5.53.0; 1657) Opera 8.60 [en]";
const char S101676[] = "Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Nokia 6600/5.53.0; 9399) Opera 8.65 [en]";
const char S101677[] = "KDDI-PT34 UP.Browser/6.2.0.14.3.1 (GUI) MMP/2.0";
const char S101678[] = "Spice S580";
const char S101679[] = "NexianNX-T780/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101680[] = "samsung-gt-s3850";
const char S101681[] = "SAMSUNG-GT-S3850/S3850ZCKE1";
const char S101682[] = "sonyericssonk530i/UC Browser7.7.1.88/69/444";
const char S101683[] = "Android/1.0 (SGH-I897 FROYO)";
const char S101684[] = "iBrowser/26 Spice Mi350n Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; huaqin73_gb Build/ALPS.GB.FDD2.MP.V2.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101685[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; PC7251 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101686[] = "MQQBrowser/2.6 (Nokia5233;SymbianOS/9.1 Series60/3.0)";
const char S101687[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; ASUS; E600)";
const char S101688[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.7) Gecko/X Ubuntu/10.04 (lucid) Firefox/3.5.7";
const char S101689[] = "Mozilla/5.0 (OS/2; Warp 4.5; rv:2.0) Gecko/X Firefox/4.0";
const char S101690[] = "JUC(Linux;U;2.2;Zh_cn;HTC Desire;480*800;)UCWEB7.8.1.96/139/800";
const char S101691[] = "JUC(Linux;U;2.2;Zh_cn;HTC Desire;480*800;)UCWEB7.8.1.96/139/999";
const char S101692[] = "NokiaC2-00/2.0 (03.42) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-00/UC Browser7.7.1.88/69/444";
const char S101693[] = "Mozilla/5.0 (Windows; U; Windows CE 5.2; en-US; rv:1.9.2a1pre) Gecko/X Fennec/1.0a1";
const char S101694[] = "NOKIA5320/UC Browser7.9.0.102/28/355/UCWEB";
const char S101695[] = "UC Browser8.0.4.121/28/351/UCWEB";
const char S101696[] = "UC Browser8.0.4.121/28/352/UCWEB";
const char S101697[] = "UC Browser8.0.4.121/50/352/UCWEB";
const char S101698[] = "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.4.1.0 Chrome/17.0.963.56 Safari/535.11";
const char S101699[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.4.1.0 Chrome/17.0.963.56 Safari/535.11";
const char S101700[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.7.2";
const char S101701[] = "Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.7.2";
const char S101702[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.2.0.1563 Safari/535.11";
const char S101703[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.2.0.1563 Safari/535.11";
const char S101704[] = "Nexian NX-T910/MTK Release/10.01.2009 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1";
const char S101705[] = "NOKIAN73/UC Browser7.0.0.41/28/352";
const char S101706[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S101707[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TeamDRH ICS for GTablet Build/IML77) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S101708[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HW-HUAWEI_C8650/C8650V100R001C92B825; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S101709[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8650+ Build/V100R001C92B861) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101710[] = "iBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101711[] = "Tweetbot/24000 CFNetwork/548.1.4 Darwin/11.0.0";
const char S101712[] = "Tweetbot/24203 CFNetwork/548.1.4 Darwin/11.0.0";
const char S101713[] = "Tweetbot/24300 CFNetwork/548.1.4 Darwin/11.0.0";
const char S101714[] = "HTC_Snap_S523/V2.03.841.1 (65854) Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5)";
const char S101715[] = "Opera/9.50 (Windows NT 5.1; MDA_compact_IV/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1; U; en)";
const char S101716[] = "Mozilla/5.0 (X11; OpenBSD i386) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30";
const char S101717[] = "Mozilla/5.0 (X11; OpenBSD amd64) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30";
const char S101718[] = "NOKIAE61i/UC Browser8.0.4.121/28/352/UCWEB";
const char S101719[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Dell Streak 7 Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S101720[] = "contype chromeframe/21.0.1180.60";
const char S101721[] = "contype chromeframe/21.0.1180.89";
const char S101722[] = "contype chromeframe/21.0.1180.83";
const char S101723[] = "Mozilla/5.0 (Linux; U; Android 1.6; th-th; i-mobile 3G 8500 Build/DRD35) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S101724[] = "Opera/9.80 (Android 1.6; Linux; Opera Mobi/ADR-X; U; pl) Presto/2.9.201 Version/11.50";
const char S101725[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Touchpad Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S101726[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT720 Build/GWK74; Bravo2.3_V1.0-brianlee) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101727[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101728[] = "Mozilla/5.0 (Linux; Android 4.0.4; MotoA953 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S101729[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101730[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A780 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101731[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; U8180 Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S101732[] = "JUC (Linux; U; 2.2; zh-cn; GT-I5508; 240*320) UCWEB7.9.3.103/140/800";
const char S101733[] = "Mozilla/5.0 (Linux; Android 4.1.1; SGH-I535 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S101734[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I535 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S101735[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46HX729; CC/USA; en) Presto/2.7.61 Version/11.00";
const char S101736[] = "MQQBrowser/2.9 (NokiaC5-03;SymbianOS/9.1 Series60/3.0)";
const char S101737[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101738[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101739[] = "NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/70/352";
const char S101740[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PC36100 Build/black bean AOKP) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S101741[] = "Mozilla/5.0 (Linux; Android 4.0.3; TAB-PROTAB2.4XL Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S101742[] = "Mozilla/5.0 (Linux; Android 4.0.4; iberry AUXUS AX03G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S101743[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gu220) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S101744[] = "iBrowser/Mini2.8 (Nokia5233/51.1.002/sw_platform=S60;sw_platform_version=5.0;java_build_version=1.4.43)";
const char S101745[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P3100 Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S101746[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Mobile Maxthon";
const char S101747[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3100 Build/IMM76D)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S101748[] = "YahooMobileMessenger/1.0 (Android Mail; 1.4.6) (GT-P7510; samsung; GT-P7510; 4.0.4/IMM76D)";
const char S101749[] = "YahooMobile/1.0 (mail; 1.4.5.12736); (Linux; U; Android 3.2; GT-P7500 Build/HONEYCOMB_MR2);";
const char S101750[] = "YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 3.2; GT-P7500R Build/HONEYCOMB_MR2);";
const char S101751[] = "MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S101752[] = "MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 4.0.3; en-za; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S101753[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5100 Build/IML74K; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444";
const char S101754[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5100 Build/IMM76D; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/352";
const char S101755[] = "Instagram 1.1.8 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P3100; espressorf; espresso; en_US)";
const char S101756[] = "YahooMobileMessenger/1.0 (Android Messenger; 1.8.3) (espresso10rf; samsung; GT-P5100; 4.0.3/IML74K)";
const char S101757[] = "Android OS/16 samsung_GT-P5113";
const char S101758[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S101759[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P6210, 39, 1.8.2)";
const char S101760[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S101761[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P3110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S101762[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7501 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +0100, de";
const char S101763[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S101764[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S101765[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P7300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0100, fr";
const char S101766[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S101767[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S101768[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S101769[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-P1000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S101770[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-P1000 Build/GFAN_ROM_X) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S101771[] = "theScore/3.1.0 Android-OS/4.0.4 (Android; Tablet, GT-P7510)";
const char S101772[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P5113 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S101773[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I8262D Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.282 U3/0.8.0 Mobile Safari/534.31";
const char S101774[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S101775[] = "MOT-EX232/1.0 REXL4/1.0 BrewMP/1.0.3 Release/6.15.2011 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101776[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ru-ru; U8220/U8230 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101777[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Xperia Play Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S101778[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; GT-I9001 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101779[] = "Mozilla/5.0 (Linux; Android 4.0.3; MP959 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S101780[] = "Mozilla/5.0 (Java; U; en-us; sonyericssonw580i) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S101781[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE-C_N760/N760V1.0.0B04; 320*480; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101782[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_7260 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101783[] = "MQQBrowser/3.5/Adr (Linux; U; 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.62;480*854)";
const char S101784[] = "SonyEricssonG502/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; sonyericssong502) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile";
const char S101785[] = "Instagram 3.0.0 (iPhone; iPhone OS 5.0.1; en_US) AppleWebKit/420+";
const char S101786[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-t310i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S101787[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian73-1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S101788[] = "Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.80 (KHTML, like Gecko) Version/7.1 Mobile/11A4388 Safari/8536.25";
const char S101789[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-tw; CHT8000 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S101790[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Samsung-SPHI325 320x240";
const char S101791[] = "SAMSUNG-SGH-D528/NetFront 3.2/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101792[] = "Mozilla/5.0 (SCH-F839/F839DH11;U;REX/3.0;BREW/3.1.5;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*400;CTC/2.0) POLARIS/6.15.WEB";
const char S101793[] = "MOT-V180/0B.D2.2ER MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S101794[] = "MOT-V180ENS/0B.D1.32R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S101795[] = "MOT-EM28/0.0.62 UP.Browser/6.3.0.6.c.28 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101796[] = "ZTE-F951-Orange/1.0 ACS-NF/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101797[] = "Nokia3100b/1.0 (05.01) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S101798[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Vodafone/1.0/SamsungSGHI620V/I620BUGG3";
const char S101799[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) SEC-SGHI620/1.0";
const char S101800[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Vodafone/1.0/SamsungSGHI620V/I620BUGFA";
const char S101801[] = "LG-KF700Q Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S101802[] = "LG/U8380/v1.0";
const char S101803[] = "SEC-SGHX630/1.0 TSS/2.5";
const char S101804[] = "SEC-SGHD407/1.0 TSS/2.5";
const char S101805[] = "HTC_Touch_Pro2_T7373 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S101806[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Sprint T7380";
const char S101807[] = "LG-KG220 MIC/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S101808[] = "Mozilla/5.0 (ZTE-E_N72/N72V1.0.0B02;U;Windows Mobile/6.1;Profile/MIDP-2.0 Configuration/CLDC-1.1;320*240;CTC/2.0) IE/6.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S101809[] = "SAMSUNG-SGH-P200/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S101810[] = "SAMSUNG-SGH-P200-ORANGE/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S101811[] = "LG/U370/v1.0";
const char S101812[] = "SAMSUNG-SGH-X576/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S101813[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; Archos5 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S101814[] = "SHARP-TQ-GX25/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.3.3.d.1.109 (GUI) MMP/2.0";
const char S101815[] = "DoCoMo/2.0 N902iS(c100;TB;W24H12)";
const char S101816[] = "SAMSUNG-GT-S3650H/1.0 SHP/VPP/R5 NetFront/3.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S101817[] = "SCH-A950 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S101818[] = "SAMSUNG-SGH-X490/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S101819[] = "New York/R3.0 NF-Browser/3.3 Openwave-MMS/4.5 IA-Java/1.0 Profile/MIDP2.0/CLDC1.1";
const char S101820[] = "SAMSUNG-GT-C3212/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S101821[] = "HTC_HD2_T8585 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65";
const char S101822[] = "LG-KF301/V01 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S101823[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Incredible Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101824[] = "PPC; 240x320; HTC_P3450/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.6.1.82";
const char S101825[] = "Nokia6681/2.0 (6.09.00) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/27/351";
const char S101826[] = "Vodafone/1.0/SamsungSGHM300V/M300BUGG2/Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S101827[] = "Vodafone/1.0/SamsungSGHM300V/M300JBGF3/Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S101828[] = "SoftBank/1.0/922SH/SHJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101829[] = "Mozilla/5.0 (922SH;SoftBank) NetFront/3.4";
const char S101830[] = "LGE-LG280 AU-OBIGO/Q05A1-3.12 MMP/2.0";
const char S101831[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; Neonode N2; 176x220)";
const char S101832[] = "HTC_P3490 Opera/9.50 (Windows NT 5.1; U; en)";
const char S101833[] = "KDDI-KC3U UP.Browser/6.2.0.15.2 (GUI) MMP/2.0";
const char S101834[] = "Mozilla/5.0 (X11; U; FreeBSD i386; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S101835[] = "Mozilla/5.0 (X11; U; FreeBSD amd64; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S101836[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Iron/9.0.600.2 Chrome/9.0.600.2 Safari/534.13";
const char S101837[] = "Opera/9.80 (Windows NT 5.1; Opera Mobi/14316; U; en) Presto/2.7.81 Version/11.00";
const char S101838[] = "NokiaC2-03/2.0 (06.51) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-03/UC Browser7.8.0.95/70/355";
const char S101839[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.6.9632.201";
const char S101840[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) QQBrowser/6.6.9632.201";
const char S101841[] = "RSSOwl/2.0.5.X (Windows; U; en)";
const char S101842[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.7) Gecko/X Epic/ Firefox/";
const char S101843[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.0 Safari/533.3";
const char S101844[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.8pre) Gecko/X Prism zdesktop/2.0";
const char S101845[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.0.1) Gecko/X Prism (zdesktop/1.0.4)";
const char S101846[] = "JUC(Linux;U;2.2.1;Zh_cn;MB525;480*854;)UCWEB7.8.1.96/139/800";
const char S101847[] = "JUC(Linux;U;2.2.2;Zh_cn;ME525;480*854;)UCWEB7.8.1.96/139/800";
const char S101848[] = "Nokia6500s-1/2.0 (06.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6500s-1/UC Browser7.7.1.88/69/444";
const char S101849[] = "NOKIA5700/UC Browser8.0.4.121/28/444/UCWEB";
const char S101850[] = "Opera/9.80 (Windows NT 6.2; Opera Tablet/11648; U; en) Presto/2.7.81 Version/11.00";
const char S101851[] = "Opera/9.10 (S60; SymbOS; Opera Tablet/9174; U; en-us) Presto/2.6.71 Version/9.5";
const char S101852[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; ASK 105SP Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S101853[] = "Opera/9.80 (Android; Linux; Opera Mobi/23731; U; en) Presto/2.9.201 Version/11.50";
const char S101854[] = "NokiaE63/UCWEB8.1.0.104/28/999";
const char S101855[] = "NOKIA-Lumia 710/UCWEB2.0.0.149/47/32079";
const char S101856[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S101857[] = "GmailPush/2.0.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S101858[] = "iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101859[] = "JUC(Linux;U;Android1.6;Zh_cn;E10i;240*320;)UCWEB7.8.0.95/139/355";
const char S101860[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; AM7-A1 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101861[] = "Mozilla/5.0 (Linux; U; Android SDK 8 Froyo (Bionic Build V0.5); en-us; Ally Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101862[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; bq Verne Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101863[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; bq DaVinci Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101864[] = "MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; H701 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101865[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC One V Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S101866[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR MAXX 4G Build/6.5.1-167_DHD-14_M2-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101867[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR MAXX Build/6.5.1-167_DHD-14_M2-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101868[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; DROID RAZR MAXX Build/6.5.1-167_DHD-14_M3-8) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S101869[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; edenTAB ET-701 Build/Gingerbread.GRJ40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101870[] = "LG-T310/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S101871[] = "sonyericssonk530i/UC Browser7.9.0.102/69/352";
const char S101872[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 TeamStream (iPod4,1 215 1.6.1)";
const char S101873[] = "Opera/9.80 (Mac OS X; Opera Mobi/35779; U; en) Presto/2.10.254 Version/12.00";
const char S101874[] = "iBrowser/Mini2.8 (SAMSUNG GT-C3303)";
const char S101875[] = "NOKIA-Lumia 710/UCWEB2.4.0.188/47/32079";
const char S101876[] = "1PasswordThumbs/1 CFNetwork/596.1 Darwin/12.1.0 (x86_64) (MacBookAir4%2C2)";
const char S101877[] = "1PasswordThumbs/1 CFNetwork/596.1 Darwin/12.1.0 (x86_64) (MacBookPro8%2C2)";
const char S101878[] = "1PasswordThumbs/1 CFNetwork/596.1 Darwin/12.1.0 (x86_64) (iMac12%2C2)";
const char S101879[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03R; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:34, us";
const char S101880[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-sg; Nexus 7 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, sg";
const char S101881[] = "iBrowser/Mini2.7 (Nokia200/11.56)";
const char S101882[] = "Mozilla/5.0 (S60V5; U; en-in; Nokia701)/UC Browser8.4.0.159/50/352/UCWEB Mobile";
const char S101883[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC EVO 3D X515m Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S101884[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10A403 Safari/7534.48.3 UCBrowser8.4.0.159/42/355";
const char S101885[] = "Instagram 3.0.0 (iPad; iPhone OS 5.1.1; en_US) AppleWebKit/420+";
const char S101886[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA6710s)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S101887[] = "SonyEricssonC510/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; sonyericssonc510) UCBrowser8.4.0.159/70/352/UCWEB Mobile";
const char S101888[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/1.6.1.1691 (Phoenix Viewer Release Mesh; Firebird skin) Safari/533.3";
const char S101889[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; SX-SP715A Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S101890[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6210 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S101891[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7510, 42, 1.9.1)";
const char S101892[] = "YahooMobileMail/1.0 (Android Mail; 1.4.6) (espresso10wifi;samsung;GT-P5113;4.0.4/IMM76D)";
const char S101893[] = "Instagram 1.1.8 Android (15/4.0.3; 160dpi; 800x1232; samsung; GT-P5100; espresso10rf; espresso10; en_GB)";
const char S101894[] = "Instagram 1.1.5 Android (13/3.2; 160dpi; 800x1232; samsung; GT-P6800; GT-P6800; smdkc210; en_GB)";
const char S101895[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6800 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S101896[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6200 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S101897[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P6200 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S101898[] = "Instagram 3.3.2 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7300; GT-P7300; p3; en_US)";
const char S101899[] = "Instagram 3.3.3 Android (16/4.1.1; 160dpi; 800x1232; samsung; GT-P7300; p5; p3; en_US)";
const char S101900[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-CN; GT-P6800 Build/HTJ85B) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.1.258 U3/0.8.0 Mobile Safari/534.31";
const char S101901[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P5100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S101902[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S101903[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S101904[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-P1010 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000";
const char S101905[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S101906[] = "theScore/3.1.2 Android-OS/2.2 (Android; Phone, GT-P1000R)";
const char S101907[] = "UCWEB/2.0 (Java; U; MIDP-2.0; fa; sonyericssonu5i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S101908[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonu5i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S101909[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233a) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S101910[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233a) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S101911[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HW-HUAWEI_C8500S/C8500SV100R001C92B627SP01; 240*320; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S101912[] = "iBrowser/Mini2.8 (SAMSUNG-GT-E2652)";
const char S101913[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8530) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S101914[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 55.8.03.4; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101915[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 57.3.13.5; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101916[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 64.4.08.7; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101917[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 55.8.03.4; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH) ; tr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101918[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 67.2.73.6; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101919[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 57.0.96.2; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH) ; de) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101920[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 69.6.43.6; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101921[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 57.0.96.2; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101922[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 67.2.73.6; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; en) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101923[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 56.2.00.0; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101924[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 82.3.59.9; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101925[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 86.5.68.7; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; en) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101926[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 74.8.30.4; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101927[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.24.5; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101928[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.21.6; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; ) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101929[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.24.5; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; ) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101930[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.27.14; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101931[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.24.5; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; en) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101932[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.27.14; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; ro) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101933[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.27.19; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101934[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.27.13; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101935[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.21.6; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101936[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.27.19; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; de) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101937[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.27.14; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; en) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101938[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.21.6; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; pl) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101939[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.28.14; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101940[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.24.5; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; ro) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101941[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.24.5; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101942[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.24.5; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; ) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101943[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.21.6; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; ) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101944[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.27.13; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101945[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.24.5; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; en) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101946[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.27.13; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; pl) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101947[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.21.6; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; pl) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101948[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.21.6; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101949[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.24.5; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; pl) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101950[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.27.14; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101951[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.27.19; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101952[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.28.14; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S101953[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-s365) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S101954[] = "SAMSUNG-S5050/S5050XEIF1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101955[] = "Nokia8801/2.0 (03.78) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101956[] = "Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5130c-2/UC Browser7.5.1.77/69/352";
const char S101957[] = "HTCP3600-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)";
const char S101958[] = "HTC_P3600-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)";
const char S101959[] = "BenQ-S88/1.00/WAP2.0/MIDP2.0/CLDC1.0 UP.Browser/6.3.0.4.c.1.102 (GUI) MMP/2.0";
const char S101960[] = "HTC_Touch_Diamond2_T5362/V2.31.841.6 (64646) Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S101961[] = "HTC Streaming Player htc_wwe / 1.0 / htc_legend / 2.2";
const char S101962[] = "HTC Streaming Player vodafone_au / 1.0 / htc_legend / 2.2";
const char S101963[] = "HTC Streaming Player vodafone_uk / 1.0 / htc_legend / 2.2";
const char S101964[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1";
const char S101965[] = "Vodafone/SamsungSGHE860V/E860JBEH4/Browser/NF/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101966[] = "Vodafone/SamsungSGHE860V/E860JBFA2/Browser/NF/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101967[] = "Vodafone/SamsungSGHE860V/E860AEFA2/Browser/NF/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101968[] = "Vodafone/SamsungSGHE860V/E860AEEH6/Browser/NF/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101969[] = "Vodafone/SamsungSGHE860V/E860BUEH6/Browser/NF/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101970[] = "LG7100-parrot/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S101971[] = "LG7100/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S101972[] = "Xda_Zest_MSIE/480x640 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S101973[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SP; 240x320; HTC_S710/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101974[] = "HTC_S710 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S101975[] = "Zte-tu232_TD/1.0 ThreadX/4.0b Larena/2.40 Release/4.15.2010 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101976[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) SAMSUNG-SGH-i601/WM534";
const char S101977[] = "SAMSUNG-GT-M3310/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101978[] = "LGE-LG231 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S101979[] = "Mozilla/4.0 (MobilePhone PM-8200/US/1.0) NetFront/3.1 MMP/2.0";
const char S101980[] = "SAMSUNG-SGH-I607/UCHA1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S101981[] = "LGE-VX8100/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S101982[] = "SAMSUNG-SGH-J708i/1.0 RTK-E/1.0 DF/1.0 Release/12.18.2008 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzUyNDUwMDM4NDQzMTY1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S101983[] = "SAMSUNG-SGH-J708i_CMCC/1.0 RTK-E/1.0 DF/1.0 Release/12.18.2008 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1/*VPGf5U4AAOpQ8Z/lTADq UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S101984[] = "SAMSUNG-GT-S3601C_CMCC/1.0 RTK-E/1.0 DF/1.0 Release/01.09.2009 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101985[] = "SAMSUNG-SGH-J706/ Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S101986[] = "SonyEricssonS500c/R8BE Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101987[] = "Nokia6276/2.0 (BL200V0300.nep) UP.Browser/6.2.3.8 MMP/2.0";
const char S101988[] = "Nokia3555/2.0 (03.46) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S101989[] = "NexianNX-G780/MTK Release/01.01.2011 Browser/MAUI Profile/MIDP2.0Configuration/CLDC-1";
const char S101990[] = "NexianNXG798/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S101991[] = "LG-LX160 AU-MIC-LX160/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S101992[] = "Mozilla/5.0 (SCH-W799/W799DC01;U;REX/3.0;BREW/3.1.5;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*400;CTC/2.0) POLARIS/6.2.WEB";
const char S101993[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) Vodafone/1.0/HTC_Mercury/2.04.161.2";
const char S101994[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) Vodafone/1.0/HTC_Mercury/2.18.161.4";
const char S101995[] = "WELLCOM-W189 WAP Browser";
const char S101996[] = "WELLCOM-W189_WAP_Browser";
const char S101997[] = "SHARP-TQ-GX15/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.3.2.d.1.110 (GUI) MMP/2.0";
const char S101998[] = "SHARP-TQ-GX15/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.3.2.d.1.112 (GUI) MMP/2.0";
const char S101999[] = "LG/KF700/v2.0";
const char S102000[] = "Fly_E140_India/WAPBrowser2.0Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102001[] = "Fly_E140/WAPBrowser2.0Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102002[] = "Mozilla/5.0 (Symbian/3; Series60/5.2 Nokia701/111.010.0107; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.4.1.3 Mobile Safari/533.4 3gpp-gba";
const char S102003[] = "LGE-MX830/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S102004[] = "SAMSUNG-SGH-D900i/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S102005[] = "Nokia6630/1.0 (6.03.40) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.2.2.51/27/352";
const char S102006[] = "SAMSUNG-GT-S7070C/1.0 SHP/VPP/R5 Dolfin/1.0 SMM-MMS/1.2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S102007[] = "SAMSUNG-SGH-T419/T419UVHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 NetFront/3.2";
const char S102008[] = "SHARP SH6017D / Obigo Browser Q03C";
const char S102009[] = "SAMSUNG-SGH-ZV50-Vodafone/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S102010[] = "NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-01/UC Browser7.7.1.88/70/444";
const char S102011[] = "Nokia5802_CMCC/21.0.025.6 (SymbianOS/9.4; U; Series60/5.0 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S102012[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) ASUS-P527/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102013[] = "Sanyo-SCP6600CA NetFront/3.1 MMP/2.0 Mozilla/4.0";
const char S102014[] = "LG-GU295g/V10i Teleca/Q7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S102015[] = "LG-KP550-Bouygues/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S102016[] = "MOT-W315/1.0 UP.Browser/6.2.2.6.n.1.101 (GUI) MMP/2.0";
const char S102017[] = "SonyEricssonZ300i/R3M/TelecaBrowser/4.08";
const char S102018[] = "Nokia6708/2.0 (V1.0.0) SymbianOS/7.0 UIQ/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S102019[] = "SAMSUNG-GT-S5560C/1.0 SHP/R5 Release/02.01.2009 Browser/NetFront3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S102020[] = "Lenovo-P580/CMCC_S017 BMP/F2 Release/2008.11.18 Browser/TelecaQ03C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102021[] = "MOT-A780/R52_G_0D.50.ADP Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola A780; 935) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 7.50 [en]";
const char S102022[] = "MOT-A780/R52_G_0D.50.AEP Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola A780; 1027) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 7.50 [en]";
const char S102023[] = "MOT-A780/R52_G_0D.43.A3P Mozilla/4.0 (compatible; MSIE 6.0; Motorola; 850) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 7.50 [zh-tw]";
const char S102024[] = "SAMSUNG-GT-i8000/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65";
const char S102025[] = "Nokia7610/2.0 (7.0642.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.6.1.82/27/354";
const char S102026[] = "BenQ-SIEMENS - E52/1.00/WAP2.0/MIDP2.0/CLDC1.0 UP.Browser/6.3.0.4.c.1.102 (GUI) MMP/2.0";
const char S102027[] = "KDDI-HI3A UP.Browser/6.2.0.13.1.4 (GUI) MMP/2.0";
const char S102028[] = "PPC; 240x320; HTC_P3350/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S102029[] = "Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Nokia 6670/6.0522.0; 9399) Opera 8.65 [en]";
const char S102030[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; sdk Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S102031[] = "SonyEricssonU5/UC Browser7.7.1.88/50/444";
const char S102032[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nook Tablet Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102033[] = "NOKIA6120c/UC Browser7.2.2.51/28/351";
const char S102034[] = "Mozilla/5.0 Opera/9.5 (KDDI-SH3D; BREW; Opera Mobi; U; ja) Presto/2.2.1";
const char S102035[] = "Mozilla/5.0 (OS/2; U; Warp 4.5; en-US; rv:1.8.1.23) Gecko/X SeaMonkey/1.1.18";
const char S102036[] = "samsung-s5230/UC Browser7.7.1.88/69/444";
const char S102037[] = "gt-s5230/UC Browser7.7.1.88/69/444";
const char S102038[] = "JUC(Linux;U;2.3.3;Zh_cn;HTC Wildfire S A510e;320*480;)UCWEB7.7.0.85/139/800";
const char S102039[] = "Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia3120classic/UC Browser7.8.0.95/69/352";
const char S102040[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/3.0.16)";
const char S102041[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/3.0.17)";
const char S102042[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Sleipnir/3.0.17)";
const char S102043[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3 Sleipnir/1.9.2m";
const char S102044[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/1.9.2m";
const char S102045[] = "Opera/9.80 (iPhone; Opera Mini/6.13548/25.729; U; ru) Presto/2.5.25 Version/10.54";
const char S102046[] = "NokiaC6-01/UC Browser7.9.0.102/50/351/UCWEB";
const char S102047[] = "NokiaE71/UCWEB8.1.0.104/28/999";
const char S102048[] = "Opera/9.80 (Windows NT 5.1; Opera Mobi/23731; U; en) Presto/2.9.201 Version/11.50";
const char S102049[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-uk; Elonex_765ET Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102050[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i917R)";
const char S102051[] = "NOKIAN95/UC Browser7.4.0.65/28/352";
const char S102052[] = "NOKIAN95 8GB/UC Browser7.4.0.65/28/352";
const char S102053[] = "JUC(Linux;U;2.3.3;Zh_cn;HTC Incredible S;480*800;)UCWEB7.8.1.96/139/800";
const char S102054[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Dream/Sapphire Build/FRG83; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102055[] = "MQQBrowser/2.2 Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/7534.48.3";
const char S102056[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MT11i Build/4.1.B.0.431) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S102057[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; SCH-R720 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S102058[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102059[] = "Mozilla/5.0 (Windows; U; Win64 (AMD64); en-US; Firefox";
const char S102060[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en_us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102061[] = "Mozilla/5.0 (Linux; U; Android 3.2; en_us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102062[] = "Mozilla/5.0 (Linux; U; Android 1.6; zh-cn; 3GW101 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Lenovo-3GW101/1.0.0";
const char S102063[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; MOT-XT806/QLCG_M2_02.11.8; 480*854; CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S102064[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; MOT-XT806/QLCG_M2_03.10.4; 480*854; CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S102065[] = "Opera/9.80 (Android 2.1-update1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.8.149 Version/11.10";
const char S102066[] = "JUC(Linux;U;Android2.2.1;Zh_cn;Spice Mi-270;240*320;)UCWEB7.8.0.95/140/444";
const char S102067[] = "Opera/9.80 (Android 2.3.5; Linux; Opera Tablet/ADR-X; U; en) Presto/2.7.81 Version/11.00";
const char S102068[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; FZ-A1A Build/HTK55) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S102069[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I5801 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S102070[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Incredible S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102071[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Incredible S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102072[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S102073[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-D700 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S102074[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102075[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-P1000 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102076[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; TAB260 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102077[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S102078[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I777 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S102079[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN85)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S102080[] = "Safari/8536.25 CFNetwork/596.1 Darwin/12.1.0 (x86_64) (MacBookAir4%2C2)";
const char S102081[] = "Safari/8536.25 CFNetwork/596.1 Darwin/12.1.0 (x86_64) (MacBookPro8%2C1)";
const char S102082[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3 Sleipnir/2.0m";
const char S102083[] = "SAMSUNG_GT-I7680_TD/1.0 OPhone/2.0 Release/2.1.2010 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 AppleWebKit/528.5+";
const char S102084[] = "JUC(Linux; U;2.3.3;zh_cn;Lenovo A60;320*480;)";
const char S102085[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonwt13i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S102086[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s3653w) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S102087[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102088[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5333) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S102089[] = "MQQBrowser/2.9 (NokiaC7-00;SymbianOS/9.1 Series60/3.0)";
const char S102090[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-t325) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S102091[] = "Mozilla/5.0 (Linux; Android 4.1.2; C2105 Build/15.0.A.1.31) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S102092[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC first Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S102093[] = "Mozilla/5.0 (Java; U; en-us; sonyericssonk770i) UCBrowser8.4.0.159/69/352/UCWEB Mobile";
const char S102094[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t325) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S102095[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-P1000 Build/GINGERBREAD; 600*1024) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/352";
const char S102096[] = "GetJarSDK/X.00 uk.co.exelentia.wikipedia/50 (X)  Android/2.2 (samsung; GT-P1000T; GT-P1000T)";
const char S102097[] = "WhatsApp/2.6.4974 Android/2.2 Device/samsung-GT-P1000";
const char S102098[] = "GetJarSDK/X.00 com.nevosoft.mysteryville/2 (X)  Android/3.2 (samsung; GT-P6200; GT-P6200)";
const char S102099[] = "com.eurosport.news.android/3.1.1/Android/3.2/samsung-GT-P6200";
const char S102100[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S102101[] = "GetJarSDK/X.00 com.nevosoft.mysteryville2.free/5 (X)  Android/3.2 (samsung; GT-P7500; GT-P7500)";
const char S102102[] = "Android/3.2; samsung GT-P7500; 2.3.2";
const char S102103[] = "Rdio.Net; Android/2.5.1; Android/3.2; DM=samsung; DN=GT-P7500R; DC=samsung; DF=samsung/GT-P7500R/GT-P7500R:3.2/HTJ85B/UXKK2:user/release-keys; DH=p3";
const char S102104[] = "YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 4.0.4; GT-P7510 Build/Unknown);";
const char S102105[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S102106[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S102107[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3110, 34, 1.6.0)";
const char S102108[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-ph; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3110, 38, 1.8.1)";
const char S102109[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, nl";
const char S102110[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/6.5.26969/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S102111[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/6.5.26969/28.3392; U; en) Presto/2.8.119 Version/11.10";
const char S102112[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S102113[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S102114[] = "JUC(Linux;U;Android4.0.4;Zh_cn;GT-P6200;600*976;)UCWEB7.8.0.95/139/444";
const char S102115[] = "Mozilla/5.0(Linux;U;Android4.0.4;Zh_cn;GT-P6200;600*976;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S102116[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-P3110 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.11/226,1.9.11.226,2012-12-04 16:21, +0000";
const char S102117[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; GT-P3110 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/292,1.9.23.292,2013-02-06 08:41, +0100, it";
const char S102118[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-P3110 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0100, de";
const char S102119[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3110, 40, 1.8.5)";
const char S102120[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; nl-be; GT-P7500 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0100, be";
const char S102121[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; nl-nl; GT-P7500 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0100, nl";
const char S102122[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6800 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S102123[] = "Nokia206/2.0 (03.59) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia206) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S102124[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6500D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352";
const char S102125[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5503 Build/10.1.1.A.1.253) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S102126[] = "MQQBrowser/3.5/Adr (Linux; U; 4.0.3; zh-cn; HTC T328w Build/IML74K;480*800)";
const char S102127[] = "Nokia6085/2.0 (06.00) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6085) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S102128[] = "MQQBrowser/2.7 (Nokia5250;SymbianOS/9.1 Series60/3.0)";
const char S102129[] = "JUC(Linux;U;Android4.0.3;Zh_cn;HTC Sensation Z710e;540*960;)UCWEB7.8.0.95/139/352";
const char S102130[] = "MQQBrowser/3.1 (E63;SymbianOS/9.1 Series60/3.0)";
const char S102131[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102132[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT22i Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102133[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian72) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S102134[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 58.0.65.5; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S102135[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 67.2.73.6; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S102136[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 64.4.08.7; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S102137[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 58.0.65.5; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH) ; it) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S102138[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 74.8.30.4; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S102139[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 58.0.65.5; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH) ; sr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S102140[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 82.3.59.9; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S102141[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 86.5.68.7; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; de) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S102142[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 86.5.68.7; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S102143[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 58.0.65.5; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH) ; pt) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S102144[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 74.8.30.4; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; sr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S102145[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 58.7.54.2; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S102146[] = "Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-46EX641; CC/USA; en) Presto/2.10.250 Version/11.60";
const char S102147[] = "Nokia-1606 UP.Browser/6.3.0.8.c.1.102 (GUI) MMP/2.0";
const char S102148[] = "MOT-L71/AAUG2128AA 08.02.06R/10.21.2005 MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/1.0";
const char S102149[] = "MOT-W5/08.00.05R MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/1.0";
const char S102150[] = "SonyEricssonW302c/R1BB Browser/OpenWave/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102151[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; GIGABYTE-MS800";
const char S102152[] = "Mozilla/5.0 (Linux; U; Android 1.6; 2.1-update1; en-us; apanda-A60 Build/ERE27; apanda-A60 Build/DONUT) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S102153[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; O2 Xda Atom Exec; PPC; 240x320)";
const char S102154[] = "MOTO-Z10/50.00.00 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 335) Opera 8.65 [en]";
const char S102155[] = "Lenovo-S700/CMCC_S101 LMP/LML Release/2008.12.24 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S102156[] = "Lenovo-S700/S300 LMP/LML Release/2009.06.11 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S102157[] = "Zte-tu722_TD/1.0 ThreadX/4.0b Larena/2.40 Release/4.15.2010 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102158[] = "SAMSUNG-GT-B5702C_CMCC/1.0 RTK-E/1.0 DF/1.0 Release/01.14.2009 Browser/NetFront3.4  Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102159[] = "Alcatel-TH3/1.0 UP.Browser/6.2.ALCATEL MMP/1.0";
const char S102160[] = "SonyEricssonZ600/R601 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S102161[] = "SCH-U520 UP.Browser/6.2.3.8.f.1.100 (GUI) MMP/2.0";
const char S102162[] = "SAMSUNG-SGH-i740/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S102163[] = "Alcatel-OT-v770/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S102164[] = "SAMSUNG-SGH-C425/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S102165[] = "SEC-SGHF200/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102166[] = "Motorola-ic902 Obigo/Q04C1 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102167[] = "SAGEM-myV-55/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.6.d.3.100 (GUI) MMP/1.0";
const char S102168[] = "LGE-AX300/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S102169[] = "LGE-AX500/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S102170[] = "LG-KF701/v10b Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102171[] = "Nokia5208_CMCC/2.0 (06.20) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102172[] = "Nokia5208/2.0 (06.20) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102173[] = "Samsung-SCHU740 AU-OBIGO/Q04C1-1.22 MMP/2.0";
const char S102174[] = "SAGEM-my411C/1.0/ MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.386 (GUI)";
const char S102175[] = "SAGEM-my411Cv/1.0/ MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.363 (GUI)";
const char S102176[] = "HTC_Touch_Pro_T7272 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S102177[] = "SAMSUNG-S3600H/1.0 RTK-E/1.0 DF/1.0 Release/07.01.2008 Browser/NetFront3.4  Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102178[] = "BlackBerry8800/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/179 UP.Browser/5.0.3.3";
const char S102179[] = "SAGEM-myV-56/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.3.3.g.2.102 (GUI) MMP/2.0";
const char S102180[] = "LG-KU311/v10d Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S102181[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) DopodD810";
const char S102182[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-T849 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102183[] = "NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-01/UC Browser7.8.0.95/70/355";
const char S102184[] = "SonyEricssonS302c/R1BC Browser/OpenWave/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102185[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SC-01B)";
const char S102186[] = "HPiPAQGlisten/1.00_1.00 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC;320x240";
const char S102187[] = "MOT-W418G/07.2E.00 Obigo/Q03C MMP/2.0";
const char S102188[] = "LG-MG280d Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S102189[] = "LG-MG280c Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S102190[] = "LG-MG280 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S102191[] = "SAMSUNG-GT-C3212/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S102192[] = "NexianNX-G238/MTK Release/22.11.2007 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1";
const char S102193[] = "NokiaN73/UC Browser7.0.0.41/28/351";
const char S102194[] = "Nokia6600/1.0 (5.53.0) SymbianOS/7.0s Series60/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.2.2.51/27/352";
const char S102195[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Mozilla/5.0 (HTC-T9199/1.08.706.1;U;Windows Mobile/6.5;Profile/MIDP-2.0 Configuration/CLDC-1.1;480*800;CTC/2.0)MSIE/6.0; Windows Phone 6.5.3.5)";
const char S102196[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Mozilla/5.0 (HTC-T9199/1.18.1401.8;U;Windows Mobile/6.5;Profile/MIDP-2.0 Configuration/CLDC-1.1;480*800;CTC/2.0)MSIE/6.0; Windows Phone 6.5.3.5)";
const char S102197[] = "SAMSUNG-SGH-F488E/1.0 RTK-E/1.0 DF/1.0 Release/08.03.2007 Browser/NetFront3.4  Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102198[] = "SAGEM-my501X/1.0/ MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.f.1.160 (GUI)";
const char S102199[] = "SEC-SGHC210/1.0";
const char S102200[] = "SAMSUNG-SGH-P180-ORANGE/P180BVGK7 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S102201[] = "PG-C300/R03 MIC/1.1.14 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102202[] = "PG-C300/R01 MIC/1.1.14 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102203[] = "SAMSUNG-GT-B7732/1.0 SHP/VPP/R5 Dolfin/1.5 SMM-MMS/1.2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102204[] = "SAMSUNG-SGH-P270-ORANGE/6_HJ1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S102205[] = "NokiaE52-1/UC Browser7.4.0.65/28/351";
const char S102206[] = "HTC_Touch2_T3320 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65";
const char S102207[] = "NokiaE63/UCWEB7.6.0.75/28/800";
const char S102208[] = "NOKIAN79/UC Browser7.6.1.82/28/351";
const char S102209[] = "sonyericssonw580i/UC Browser7.7.1.88/69/444";
const char S102210[] = "NokiaN97 mini/UC Browser7.8.0.95/50/355";
const char S102211[] = "Vodafone/1.0/SamsungSGHX660V/X660BUEK4/Browser/TSS/2.5 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S102212[] = "Vodafone/1.0/SamsungSGHX660V/X660JBEK2/Browser/TSS/2.5 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S102213[] = "Mozilla/4.08 (SmartPhone; Symbian OS-Series60/1.03) NetFront/3.2";
const char S102214[] = "V908/MIDP2.0/CLDC1.1/Screen-240x320";
const char S102215[] = "SCH-R500/1.0 AU-MIC/2.0 (GUI) MMP/2.0";
const char S102216[] = "SAMSUNGSGH-i450";
const char S102217[] = "DoCoMo/2.0 L01A(c100;TB;W24H17)";
const char S102218[] = "Mozilla/5.0 (Windows; U; Windows CE 5.2; en-US; rv:1.9.2a1pre) Gecko/20090513 Fennec/1.0a1";
const char S102219[] = "KDDI-KC3Y UP.Browser/6.2.0.17.1.1 (GUI) MMP/2.0";
const char S102220[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; XT532 Build/V4.37B)";
const char S102221[] = "lg-t300/UC Browser7.9.0.102/69/352";
const char S102222[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.8pre) Gecko/X Prism zdesktop/2.0";
const char S102223[] = "MQQBrowser/Mini2.4 (SAMSUNG-GT-C3312)";
const char S102224[] = "iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102225[] = "Mozilla/5.0 (Windows; U; Windows NT 4.0; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3";
const char S102226[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; chrome://navigator/locale/navigator.properties; rv:1.8.1.21) Gecko/X SecondLife/1.23.5.X (Second Life Release; default skin)";
const char S102227[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.7) Gecko/X Epic/ Firefox/";
const char S102228[] = "KDDI-SA3C UP.Browser/6.2_7.2.7.1.K.1.5.1.120 (GUI) MMP/2.0";
const char S102229[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.2) Gecko/X Firefox/3.6 Kylo/0.8.4.74873";
const char S102230[] = "Palm680/RC1 (iPhone; U; CPU iPhone OS 2_2_1 like Mac OS X; zh-cn)/UCWEB7.4.1.68/41/997";
const char S102231[] = "Mozilla/4.0 (jig browser web; 1.0.4; 913SH)";
const char S102232[] = "samsung-gt-s3370/UC Browser7.6.1.82/69/351";
const char S102233[] = "Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5310xpressmusic/UC Browser7.6.1.82/69/352";
const char S102234[] = "NokiaC3-01/5.0 (06.05) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac3-01/UC Browser7.7.1.88/69/444";
const char S102235[] = "MQQBrowser/2.0 (Nokia6120c(B20.01);SymbianOS/9.1 Series60/3.0)";
const char S102236[] = "MQQBrowser/2.0 (Nokia6120c(B20.03);SymbianOS/9.1 Series60/3.0)";
const char S102237[] = "Mozilla/5.0 (Windows NT 6.1; rv:9.0.1) Gecko/X SeaMonkey/2.6.1";
const char S102238[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-GB) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.2.8.X (Second Life Release; default skin) Safari/533.3";
const char S102239[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A500 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S102240[] = "Dolphin http client/7.6.0(127) (Android)";
const char S102241[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S102242[] = "Opera/9.80 (Windows NT 6.1; Opera Tablet/23731; U; en) Presto/2.9.201 Version/11.50";
const char S102243[] = "Opera/9.80 (iPhone; Opera Mini/5.19802/25.697; U; ru) Presto/2.5.25 Version/10.54";
const char S102244[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S102245[] = "NOKIA-Lumia 710/UCWEB2.1.0.155/47/32079";
const char S102246[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10/Nutch-1.2";
const char S102247[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) FluidApp/1.0 Safari/533.21.1";
const char S102248[] = "Mozilla/4.0 (jig browser web; 1.0.4; 931SC)";
const char S102249[] = "Mozilla/5.0 Opera/9.5 (KDDI-SH3F; BREW; Opera Mobi; U; ja) Presto/2.2.1";
const char S102250[] = "JUC(Linux;U;Android2.1-update1;Zh_cn;XCD 28;240*320;)UCWEB7.8.0.95/140/444";
const char S102251[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; LG-KH5200 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102252[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC_Desire_S Build/GRI40; 480*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/800";
const char S102253[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; CUBE U9GT 2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102254[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; TAB-PROTAB2XXL Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102255[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102256[] = "MQQBrowser/3.3/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102257[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102258[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2.1; en-in; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102259[] = "Opera/9.80 (Android android4.0; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.10.254 Version/12.00";
const char S102260[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; Spice Mi-270 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S102261[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC One X Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S102262[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I727 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S102263[] = "NOKIA-Lumia 800/UCWEB2.4.0.188/47/32079";
const char S102264[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko)  QtWeb Internet Browser/3.5 http://www.QtWeb.net";
const char S102265[] = "Nokia7230/5.0 (06.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia7230) UCBrowser8.2.1.144/70/355/UCWEB Mobile";
const char S102266[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; Karbonn_A1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102267[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN76)/UC Browser8.2.0.132/28/355/UCWEB Mobile";
const char S102268[] = "Mozilla/5.0 (Linux; Android 4.1.1; T-Mobile myTouch 4G Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S102269[] = "Mozilla/5.0 (Linux; Android 4.0.4; XT894 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S102270[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5350) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S102271[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s3370) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S102272[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-c105) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S102273[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i917R;7.10;8862)";
const char S102274[] = "Android OS/10 samsung_GT-P1000";
const char S102275[] = "Mozilla/5.0 (Linux; U; Android HDVS_zz_2.2.1; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102276[] = "bitartist.marksix/3.2 (Linux; Android VER:8,samsung,GT-P1000)";
const char S102277[] = "com.kakao.talk/Android Version/2.2.2 Model/GT-P1000R";
const char S102278[] = "Mozilla/5.0 (Linux; U; Android 3.2; it-it; GT-P7500 Build/HTJ85B)Maxthon AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S102279[] = "iBrowser/2.8 Mozilla/5.0 (Linux; U; Android 2.2; en-za; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102280[] = "YahooMobileMessenger/1.0 (Android Mail; 1.4.6) (espresso10wifi; samsung; GT-P5113; 4.0.4/IMM76D)";
const char S102281[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S102282[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6200L Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S102283[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6200 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S102284[] = "TwitterAndroid/1.0.5 (109) GT-P7500/13 (samsung;GT-P7500)";
const char S102285[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444";
const char S102286[] = "Instagram 3.1.0 Android (13/3.2; 160dpi; 600x976; samsung; GT-P6200; GT-P6200; smdkc210; en_GB)";
const char S102287[] = "Instagram 3.1.0 Android (13/3.2; 160dpi; 800x1232; samsung; GT-P7500; GT-P7500; p3; en_GB)";
const char S102288[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P6800, 42, 1.9.1)";
const char S102289[] = "Instagram 1.0.5 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7500; GT-P7500; p3; en_US)";
const char S102290[] = "Layar/7.0.2 Android/3.2 (samsung GT-P7510)";
const char S102291[] = "Rdio.Net; Android/2.5.1; Android/3.2; DM=samsung; DN=GT-P7510; DC=samsung; DF=samsung/GT-P7510/GT-P7510:3.2/HTJ85B/VLKL2:user/release-keys; DH=p3";
const char S102292[] = "Instagram 1.1.1 Android (12/3.1; 160dpi; 1280x800; samsung; GT-P7300; GT-P7300; p3; en_US)";
const char S102293[] = "YahooMobile/1.0 (Android Sportacular; 70-3.0) (Android; GT-P5113; Android OS/4.0.4)";
const char S102294[] = "YahooMobile/1.0 (Android Sportacular; 76-3.0.6) (Android; GT-P5113; Android OS/4.1.1)";
const char S102295[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7300 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S102296[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7300 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S102297[] = "Instagram 3.4.0 Android (16/4.1.1; 160dpi; 800x1232; samsung; GT-P5110; espresso10wifi; espresso10; en_GB)";
const char S102298[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P6200 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S102299[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S102300[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3113 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S102301[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P6210 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S102302[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; GT-P5100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S102303[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7300 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S102304[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 OverDrive Media Console/2.6.4";
const char S102305[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259";
const char S102306[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S102307[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P5100 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315";
const char S102308[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P5100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315";
const char S102309[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5600) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S102310[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gx300) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S102311[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40EX725; CC/DEU; en) Presto/2.7.61 Version/11.00";
const char S102312[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; M9 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102313[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S102314[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-b5310) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S102315[] = "Nokia111/2.0 (03.32) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia111) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S102316[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LM6200-TA; 04.10.08; 0xX;); LG NetCast.TV-2012 0";
const char S102317[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LM6200-UE; 04.20.18; 0xX;); LG NetCast.TV-2012 0";
const char S102318[] = "Mozilla/5.0 (X11; U; Linux i686; en-us; rv:2.0) Gecko/X Fedora/16-dev Firefox/4.0";
const char S102319[] = "Mozilla/5.0 (Windows Phone 8.0; ARM; Trident/7.0; Touch; rv:11.0; IEMobile/11.0; NOKIA; Lumia 928) like Gecko";
const char S102320[] = "Mozilla/5.0 (Linux; Android 2.3.5; GT-S5360 Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S102321[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-S5360 Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S102322[] = "Opera/9.80 (Android; Opera Mini/18.1/30.3558; U; en) Presto/2.8.119 Version/11.10";
const char S102323[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-s310) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S102324[] = "Nokia5200/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5200) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S102325[] = "Nokia6500s-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6500s-1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S102326[] = "Nokia6265/2.0 (HL190V0600.nep) UP.Browser/6.2.3.8 MMP/2.0";
const char S102327[] = "Nokia6265i/2.0 (HL100V1000.nep) UP.Browser/6.2.3.8 MMP/2.0";
const char S102328[] = "SonyEricssonT303a/R2DD002 TelecaBrowser/Q04C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102329[] = "HTC_Touch_3G_T3232 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S102330[] = "Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola ???; 2647) MOT-???/R533_11.12.09Pi_NTA Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [vi]";
const char S102331[] = "MOT-MOTOKRZRmaxxK3/99.31.04R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12153168) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S102332[] = "MOT-E680G/1.0 E680G_G_0D.C5.AEP/04.09.2006 Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola E680G; 1483) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [zh-cn]";
const char S102333[] = "SAMSUNG-SPH-M8400/(null)ID4 (compatible; MSIE 6.0; Windows CE; PPC) Opera 9.5";
const char S102334[] = "Nokia6800/1.0 (3.14) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S102335[] = "SAMSUNG-SGH-G508E/G508EZCIG2 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S102336[] = "Xda_Atmos; 240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S102337[] = "MOT-V557/08.28.40R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102338[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) ASUSM530w/1.0";
const char S102339[] = "SAMSUNG-GT-S7120U/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S102340[] = "LG-CG225 UP.Browser/6.2.3 (GUI) MMP/1.0";
const char S102341[] = "SAGEM-myX-4/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.6.d.5.100 (GUI) MMP/1.0";
const char S102342[] = "SAGEM-myX-4/1.2 UP.Browser/6.2.2.6.d.5.100 (GUI) MMP/1.0";
const char S102343[] = "HTC_MAX Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S102344[] = "HTC MAX 4G Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S102345[] = "SEC-SGHX530/1.0";
const char S102346[] = "SAMSUNG-SGH-G608/1.0 RTK-E/1.0 DF/1.0 Release/06.22.2007 Browser/NetFront3.4  Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102347[] = "LG-KM500 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S102348[] = "LG-CG180/V1.0b UP.Browser/6.2.3 (GUI) MMP/1.0";
const char S102349[] = "Mozilla/5.0 (Linux; U; Android 1.6; zh-tw; FET-T1 Build/DONUT) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S102350[] = "LGE-LG830/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S102351[] = "SAMSUNG-SGH-F408/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S102352[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2690/UC Browser7.8.0.95/70/351";
const char S102353[] = "SonyEricssonD750i/R1N Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102354[] = "SAMSUNG-S7350/S7350BOIH2 SHP/VPP/R5 Opera/9.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S102355[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SC-01B";
const char S102356[] = "LG-GT370 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S102357[] = "ZTE-C_S180/VE_S180_NBTL25FV1.0.0B15 SMIT-BROWSER/V2.0.0 CTC/1.0";
const char S102358[] = "Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola A780; 781) MOT-A780/R532L4_G_11.40.0FR Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [es-la]";
const char S102359[] = "Fly_DS150/MAUIBrowser/Configuration/WAP2.0, Fly_DS150/MAUIBrowser/Configuration/WAP2.0";
const char S102360[] = "Fly_DS150/MAUIBrowser/Configuration/WAP2.0";
const char S102361[] = "SAMSUNG-SGH-T119/T119UVHL2 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102362[] = "NokiaE63/UC Browser7.0.0.41/28/352";
const char S102363[] = "Mozilla/5.0 (Linux; U; Android 1.6; ko-kr; LU2300 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S102364[] = "i-mobile Hitz 2205/2.0 (03.15) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102365[] = "SonyEricssonW20i/R7DA";
const char S102366[] = "SAGEM-my411V/1.0/ MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.317 (GUI)";
const char S102367[] = "SAGEM-my411V/1.0/ MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.326 (GUI)";
const char S102368[] = "LG/KF750/v1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S102369[] = "NokiaN97/UC Browser7.2.2.51/50/352";
const char S102370[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) ASUS-GalaxyII/1.0";
const char S102371[] = "MOT-C257/0.1.31 UP.Browser/6.2.3.9.c.6 (GUI) MMP/2.0";
const char S102372[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 640x480; Ameo/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1) Opera 8.65 [de]";
const char S102373[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 640x480; Ameo/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1) Opera 8.65 [en]";
const char S102374[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 640x480; Ameo/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1) Opera 8.65 [en]";
const char S102375[] = "SIE-MC60/07 Profile/MIDP-1.0 Configuration/CLDC-1.0 UP.Browser/6.1.0.7.3 (GUI) MMP/1.0";
const char S102376[] = "SIE-MC60/10 Profile/MIDP-1.0 Configuration/CLDC-1.0 UP.Browser/6.1.0.7.3 (GUI) MMP/1.0";
const char S102377[] = "SHARP-TQ-GX29/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.3.3.h.1.100 (GUI) MMP/2.0";
const char S102378[] = "LG-C1100 MIC/WAP2.0 MIDP-2.0/CLDC-1.0";
const char S102379[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; Vibo-A699 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102380[] = "NokiaE66/UC Browser7.4.0.65/28/351";
const char S102381[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) SP; 320x240; HTC_S630/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102382[] = "Huawei/1.0/T520/B000 Browser/obigo-browser/q04a";
const char S102383[] = "LG-KG320 MIC/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S102384[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; MUCHTEL-A2 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102385[] = "SIE-A31/07 Profile/MIDP-1.0 Configuration/CLDC-1.0 UP.Browser/6.1.0.7.3.102 (GUI) MMP/1.0";
const char S102386[] = "SIE-A31/09 Profile/MIDP-1.0 Configuration/CLDC-1.0 UP.Browser/6.1.0.7.3.102 (GUI) MMP/1.0";
const char S102387[] = "SIE-A31a/03 Profile/MIDP-1.0 Configuration/CLDC-1.0 UP.Browser/6.1.0.7.3.102 (GUI) MMP/1.0";
const char S102388[] = "CDM8950/T05_0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S102389[] = "LG-KC780 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S102390[] = "SAGEM-my501C/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.f.1.160 (GUI)";
const char S102391[] = "LG-KG800/V10c Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S102392[] = "NOKIAN79/UC Browser7.4.0.65/28/351";
const char S102393[] = "Mozilla/5.0 (Linux; U; Android 1.0.0; nl-nl; TP-07D8 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S102394[] = "Nokia5250/SymbianOS/9.1 Series60/3.0";
const char S102395[] = "NOKIAN79/UC Browser7.8.0.95/28/351";
const char S102396[] = "Nokia5250/UC Browser7.8.0.95/50/355";
const char S102397[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) ASUSM930/1.0";
const char S102398[] = "DopodC800/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)";
const char S102399[] = "SAGEM-my401C/1.0/ MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.258 (GUI)";
const char S102400[] = "LGE-MX8700/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S102401[] = "KDDI-PT33 UP.Browser/6.2.0.13.2 (GUI) MMP/2.0";
const char S102402[] = "KDDI-KC3G UP.Browser/6.2.0.14.1 (GUI) MMP/2.0";
const char S102403[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 640x480; Ameo/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102404[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) Xda Argon; 240x320";
const char S102405[] = "Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Nokia 6630/5.03.08; 9399) Opera 8.65 [en]";
const char S102406[] = "KDDI-KC3T UP.Browser/6.2.0.15.1.1 (GUI) MMP/2.0";
const char S102407[] = "LG-GU282 Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S102408[] = "BIRD S701_GPRS/1.0 OSE/V1.6.0 ULC/2.0 Release/04.21.2008 Browser/JB4.3";
const char S102409[] = "KyoceraC4700/Obigo/Q7 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S102410[] = "KDDI-KC3P UP.Browser/6.2.0.15.2 (GUI) MMP/2.0";
const char S102411[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.5.1";
const char S102412[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 Flipboard/1.5.1";
const char S102413[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.5.1";
const char S102414[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.5.3";
const char S102415[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.5";
const char S102416[] = "SonyEricssonU1/UC Browser7.7.1.88/50/444";
const char S102417[] = "Flipboard/1.6.1 CFNetwork/485.13.9 Darwin/11.0.0";
const char S102418[] = "samsung-gt-s3653/UC Browser8.0.3.107/70/352";
const char S102419[] = "NOKIAE65/UC Browser7.9.0.102/28/355/UCWEB";
const char S102420[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.8";
const char S102421[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.8.2";
const char S102422[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.8.4";
const char S102423[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Flipboard/1.8.4";
const char S102424[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/ Sleipnir/1.8.1m";
const char S102425[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/ Sleipnir/1.8m";
const char S102426[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/ Sleipnir/1.8.2m";
const char S102427[] = "Dolphin http client/7.5.0(124) (Android)";
const char S102428[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; ja-JP; rv:1.4) Gecko/X Netscape/7.1 (ax)";
const char S102429[] = "Safari/7534.48.3 CFNetwork/520.0.13 Darwin/11.0.0 (i386) (MacBook2%2C1)";
const char S102430[] = "Safari/7534.48.3 CFNetwork/520.0.13 Darwin/11.0.0 (x86_64) (MacBookPro8%2C2)";
const char S102431[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/18.0.1025.162)";
const char S102432[] = "samsung-gt-b5310/UC Browser7.5.1.77/69/351";
const char S102433[] = "Nokia3110c/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia3110c/UC Browser7.6.1.82/69/352";
const char S102434[] = "samsung-gt-b7722/UC Browser7.6.1.82/69/351";
const char S102435[] = "JUC(Linux;U;2.2;Zh_cn;HTC Desire;480*800;)UCWEB7.7.0.85/139/800";
const char S102436[] = "JUC(Linux;U;2.2;Zh_cn;HTC Desire;480*800;)UCWEB7.7.0.85/139/999";
const char S102437[] = "samsung-gt-s5263/UC Browser7.6.1.82/69/351";
const char S102438[] = "SAMSUNG-GT-I8000U/1.0 Software/WM6.5 Release/01.24.2009 Profile/MIDP-2.1 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)";
const char S102439[] = "SonyEricssonG502/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssong502/UC Browser7.7.1.88/69/444";
const char S102440[] = "Nokia3500c/2.0 (06.60) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia3500c/UC Browser7.7.1.88/69/444";
const char S102441[] = "SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonw20i/UC Browser7.7.1.88/69/444";
const char S102442[] = "gt-s5233s/UC Browser7.8.0.95/69/401";
const char S102443[] = "MQQBrowser/2.0 (E71-1;SymbianOS/9.1 Series60/3.0)";
const char S102444[] = "sonyericssonw150i/UC Browser7.9.0.102/69/352";
const char S102445[] = "micromaxx510/UC Browser7.9.0.102/69/352";
const char S102446[] = "samsung-gt-c6712/UCWEB8.0.3.99/70/350";
const char S102447[] = "NokiaC2-06/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-06/UC Browser7.9.0.102/70/355";
const char S102448[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102449[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T989 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S102450[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T989 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S102451[] = "iBrowser/2.8 Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102452[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5670 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S102453[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]";
const char S102454[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]";
const char S102455[] = "Mail/1257 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookPro8%2C2)";
const char S102456[] = "Mail/1257 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (iMac12%2C2)";
const char S102457[] = "Nokia5800 XpressMusic/UCWEB8.1.0.104/50/999";
const char S102458[] = "Nokia5800w XpressMusic/UCWEB8.1.0.104/50/800";
const char S102459[] = "sonyericssons500i/UC Browser7.9.0.102/69/352";
const char S102460[] = "sonyericssonk750i/UC Browser7.9.0.102/69/352";
const char S102461[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) FluidApp Version/1.4 Safari/534.56.5";
const char S102462[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0) chromeframe/9.0.597.98";
const char S102463[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13 chromeframe/9.0.597.98";
const char S102464[] = "Vodafone/1.0/SamsungSGHM310V/M310BUHF2 Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S102465[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I5510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S102466[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S102467[] = "1PasswordThumbs/1 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacBookAir4%2C2)";
const char S102468[] = "1PasswordThumbs/1 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacBookPro8%2C1)";
const char S102469[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Sleipnir/3.6.2";
const char S102470[] = "JUC(Linux;U;Android2.2;Zh_cn;Desire HD;480*800;)UCWEB7.8.0.95/139/352";
const char S102471[] = "JUC(Linux;U;Android1.6;Zh_cn;E15i;320*480;)UCWEB7.8.0.95/139/355";
const char S102472[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102473[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB611 Build/BGN_2.3.5; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102474[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PTT1026 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102475[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SCH-I800 Build/GRJ22; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102476[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102477[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S102478[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; LG-P500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S102479[] = "JUC(Linux;U;Android2.2.2;Zh_cn;U8180;240*320;)UCWEB7.8.0.95/139/444";
const char S102480[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; U8160 Build/GRK39F)";
const char S102481[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102482[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E10i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S102483[] = "MQQBrowser/3.3/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Lenovo A60 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102484[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; Liquid Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102485[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.32 (KHTML, like Gecko) Ubuntu/8.04 Chromium/13.0.751.0 Chrome/13.0.751.0 Safari/534.32";
const char S102486[] = "iBrowser/Mini2.7 (Nokia2700c-2/09.98)";
const char S102487[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3 Sleipnir/2.0m";
const char S102488[] = "Mozilla/5.0 (X11; SunOS i86pc) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30";
const char S102489[] = "JUC (Linux; U; 2.2.2; zh-cn; X903; 480*800) UCWEB7.9.3.103/139/7711";
const char S102490[] = "iBrowser/2.4 (NokiaE72-1;SymbianOS/9.1 Series60/3.0)";
const char S102491[] = "MQQBrowser/2.8 (E63;SymbianOS/9.1 Series60/3.0)";
const char S102492[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s3650) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S102493[] = "UCWEB/2.0(Java; U; MIDP-2.0; vi; samsung-gt-s5753e) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S102494[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s8500) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S102495[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.3; en-in; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102496[] = "Nokia306/2.0 (03.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia306) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S102497[] = "Windows Phone Search (Windows Phone OS 7.10;LG;LG-E900h;7.10;8862)";
const char S102498[] = "YahooMobile/1.0 (mail; 1.4.2.11841); (Linux; U; Android 3.2; GT-P6800 Build/HONEYCOMB_MR2);";
const char S102499[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; fa-fa; GT-P1013 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102500[] = "Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-P1013 Build/FROYO)";
const char S102501[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S102502[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S102503[] = "YahooMobileMail/1.0 (Android Mail; 1.4.5) (espressowifi;samsung;GT-P3113;4.1.1/JRO03C)";
const char S102504[] = "Instagram 3.0.5 Android (13/3.2; 160dpi; 800x1232; samsung; GT-P6800; GT-P6800; smdkc210; en_GB)";
const char S102505[] = "Instagram 3.1.0 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P3100; espressorf; espresso; en_GB)";
const char S102506[] = "YahooMobileMessenger/1.0 (Android Mail; 1.4.5) (GT-P7300; samsung; GT-P7300; 3.2/HTJ85B)";
const char S102507[] = "nApps (Android OS 4.0.4; GT-P3113; NaverSearch; 4.1.0)";
const char S102508[] = "Dalvik/1.6.0 (Linux; U; Android Jelly Bean 4.1.2; GT-P3113 Build/JZO54K)";
const char S102509[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]";
const char S102510[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]";
const char S102511[] = "Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13 Opera/9.80 (Android; Opera Mini/6.0.25270/28.3126; U; fr) Presto/2.8.119 Version/11.10";
const char S102512[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Opera/9.80 (Android; Opera Mini/6.0.25270/28.3445; U; en) Presto/2.8.119 Version/11.10";
const char S102513[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P3113 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355";
const char S102514[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3113 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S102515[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-P1000 Build/JZO54K; CyanogenMod-Sapir) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.10/221,1.9.10.221,2012-11-19 13:37, fr";
const char S102516[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; GT-P1000 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0100, nl";
const char S102517[] = "JUC(Linux;U;Android4.0.4;Zh_cn;GT-P6800;800*1232;)UCWEB7.8.0.95/139/444";
const char S102518[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P7510 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/405";
const char S102519[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6800 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S102520[] = "WhatsApp/2.9.369 Android/2.2 Device/samsung-GT-P1000";
const char S102521[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; sv-se; GT-P3100 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +0100, se";
const char S102522[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; pt-br; GT-P3100 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, -0200, br";
const char S102523[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3100 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0530, in";
const char S102524[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S102525[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S102526[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S102527[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3100B Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S102528[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P3100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S102529[] = "theScore/3.1.3 Android-OS/4.1.1 (Android; Phone, GT-P3113)";
const char S102530[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; zte-p107a20(propus)) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S102531[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S102532[] = "Opera/5.02 (Windows NT 5.0; U; [en])";
const char S102533[] = "iBrowser/Mini2.7 (SAMSUNG-GT-C3312)";
const char S102534[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t325) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S102535[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t325) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S102536[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian72) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S102537[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3650) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S102538[] = "Mozilla/5.0 (DTV) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.5 AQUOSBrowser/1.0 (AS00DTV;V;0001;0001) AQUOS-AS/2.0 LC-60LE640X";
const char S102539[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-S7275 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S102540[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-b5310) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S102541[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5333) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S102542[] = "Dalvik/1.4.0 (Linux; U; Android 4.4; SPHS on Hsdroid Build/MocorDroid2.3.5)";
const char S102543[] = "Mozilla/5.0 (Linux; U; Android 4.4; en-us; SPHS on Hsdroid Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102544[] = "com.google.android.youtube/4.5.17(Linux; U; Android 4.4; en_AU; SPHS on Hsdroid Build/MocorDroid2.3.5) gzip";
const char S102545[] = "Mozilla/5.0 (Windows Phone 8.0; Trident/7.0; rv:11.0; IEMobile/11.0; ARM; Touch; HTC; HTC6990LVW) like Gecko";
const char S102546[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Opera/9.5) HTC-MP6950 BM";
const char S102547[] = "PPC; 240x320; HTC_P4351/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S102548[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_M900";
const char S102549[] = "SAMSUNG-SGH-T809/T809UVEJ9 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S102550[] = "SonyEricssonW960i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102551[] = "RSSFeeds/R1A SonyEricssonP990i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102552[] = "Mozilla/5.0 (Linux; U; Android 1.5; de-at; RBM2 Build/CRB17) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S102553[] = "SAMSUNG-GT-I8000U/1.0 Software/WM6.5 Release/01.24.2009 Profile/MIDP-2.1 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S102554[] = "Mozilla/5.0 (Linux; U; Android 1.6; zh-cn; dopod A3288 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S102555[] = "Mozilla/5.0 (Linux; U; Android 1.6; zh-cn; dopod A3288 Build/DRC79) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S102556[] = "SAMSUNG-GT-I8000U/1.0 Software/WM6.5 Release/01.24.2009 Profile/MIDP-2.1 Configuration/CLDC-1.1 Opera 9.5 (Windows CE; PPC; Opera Mobi; U; zh-cn)";
const char S102557[] = "SonyEricssonW890i/R1FA SEMC/NewsReader/1.0";
const char S102558[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTCA3380_TD/1.0 Android/2.2 release/9.1.2010 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102559[] = "SIE-CX70/50 UP.Browser/7.0.2.2.d.3(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102560[] = "SIE-CX70/56 UP.Browser/7.0.2.2.d.4(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102561[] = "SIE-CX70/43 UP.Browser/7.0.2.2.d.1.100(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102562[] = "Samsung-SPHM220 Polaris/5.0 MMP/2.0";
const char S102563[] = "MOT-V750/1.0 UP.Browser/7.2.6.1.731 (GUI) MMP/2.0";
const char S102564[] = "Alcatel-TH4/1.0 UP.Browser/6.2.ALCATEL MMP/1.0";
const char S102565[] = "SAMSUNG-SGH-P920/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S102566[] = "SonyEricssonZ500a/R1A SEMC-Browser/4.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102567[] = "SonyEricssonZ500a/R4A SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102568[] = "LG/U450/v1.0";
const char S102569[] = "SEC-SGHI320/1.0 jBrowser/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102570[] = "SEC-SGHD807/1.0 TSS/2.5";
const char S102571[] = "MOT-R901_/00.62 UP.Browser/6.2.3.4.c.1.112 (GUI) MMP/2.0";
const char S102572[] = "SAMSUNG-GT-M8910B/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S102573[] = "LG5500-parrot/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S102574[] = "LG5500/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S102575[] = "LG-KF755/v10a Browser/Obigo-Q05A/3.11 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S102576[] = "SAMSUNG-SGH-i600/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S102577[] = "HTC_Touch_Pro_T7272 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S102578[] = "Mozilla/5.0 (Linux; U; Android Ultimate Dream 5.0; en-us; AMS Dream 5.0 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S102579[] = "LG-LG6400/LG6400V00 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S102580[] = "ZTE-F852/1.0 ACS-NF/3.2 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102581[] = "Nokia5100/1.0 (5.20) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S102582[] = "MOT-EM25/1.0 Release/11.01.2008 Browser/CMCS1.0 Software/0.34K Profile/MIDP-2.0 Configuretion/CLDC-1.1";
const char S102583[] = "Alcatel-OT-S520/1.0 ObigoInternetBrowser/Q03C";
const char S102584[] = "Alcatel-OT-S520A/1.0 ObigoInternetBrowser/Q03C";
const char S102585[] = "SEC-SGHE310/1.0";
const char S102586[] = "NexianNX-G755/SPR Release/01.03.2011 Browser/Profile/MIDP2.0 Configuration/CLDC-1";
const char S102587[] = "SonyEricssonC905c/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3";
const char S102588[] = "SEC-SGHX210/1.0";
const char S102589[] = "BenQ-T51/1.0/WAP2.0/MIDP2.0/CLDC1.1 UP.Browser/6.3.0.4.c.1.105 (GUI) MMP/2.0";
const char S102590[] = "SonyEricssonW300i";
const char S102591[] = "NokiaN72/5.0819.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.2.2.51/27/352";
const char S102592[] = "NokiaX6-00/UC Browser7.2.2.51/50/352";
const char S102593[] = "SAMSUNG-M3200/M3200JCHJ1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S102594[] = "NokiaE63/UCWEB7.4.0.57/28/800";
const char S102595[] = "SonyEricssonK300a/R2AT SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102596[] = "SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S102597[] = "Samsung-SPHM560 Access-NetFront/3.5.1 MMP/2.0";
const char S102598[] = "SAMSUNG-SGH-E330N/1.0 UP.Browser/6.2.2.6 (GUI) MMP/1.0";
const char S102599[] = "NEC-e373/001.00 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1 Qtv/1.00";
const char S102600[] = "NEC-e373/001.00 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Qtv/1.00";
const char S102601[] = "LGE-VX8700/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S102602[] = "CoolPadF608/CMCC ThreadX/1.0 ArenaA2KPH/SW2.0 Release/2009.10.01 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102603[] = "Lenovo-P80/CMCC_S101 LMP/XM Release/2008.08.26 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S102604[] = "SAGEM-myC5-3/1.0 UP.Browser/6.2.3.3.g.2.107 (GUI) MMP/2.0";
const char S102605[] = "Alcatel-OT-S320/1.0 ObigoInternetBrowser/Q03C";
const char S102606[] = "Alcatel-OT-S320/1.0 model-orange ObigoInternetBrowser/Q03C";
const char S102607[] = "Nokia5233/UCWEB7.6.0.75/50/800";
const char S102608[] = "NokiaN-GageQD/1.0 SymbianOS/6.1 Series60/1.2 Profile/MIDP-1.0 Configuration/CLDC-1.0/UC Browser7.6.1.82/26/401";
const char S102609[] = "lg-gb270";
const char S102610[] = "NokiaN97/UC Browser7.8.0.95/50/351";
const char S102611[] = "LGE-LG600 POLARIS/6.01 MMP/2.0";
const char S102612[] = "DopodD600/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)";
const char S102613[] = "dopod D600_CMCC/5.1.422/WAP1.2 Profile/MIDP2.0 Configuration/CLDC1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)";
const char S102614[] = "MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102615[] = "MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.2; th-th; GT-P1000T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102616[] = "SEC-SGHD357, TSS/2.5, Rev 1.4";
const char S102617[] = "SEC-SGHX686/1.0 TSS/2.5";
const char S102618[] = "SEC-SGHD307, TSS/2.5, REV0.6";
const char S102619[] = "SCH-S259/ UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S102620[] = "SCH-S259 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S102621[] = "Samsung-SPHM610 AU-OBIGO/Q04C1-1.17 MMP/2.0";
const char S102622[] = "SoftBank/1.0/920SH/SHJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102623[] = "SoftBank/1.0/921SH/SHJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102624[] = "SoftBank/1.0/820SH/SHJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102625[] = "Mozilla/4.08 (820SH;SoftBank) NetFront/3.4";
const char S102626[] = "KDDI-TS3T UP.Browser/6.2_7.2.7.1.K.4.306 (GUI) MMP/2.0";
const char S102627[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.7.1";
const char S102628[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.7.2";
const char S102629[] = "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Flipboard/1.7.2";
const char S102630[] = "MQQBrowser/27 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405";
const char S102631[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; ZTE-C_V9E/FRY_V9EV1.0.0B02; 240*400; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102632[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102633[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102634[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; fr-ca; XT720 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102635[] = "Android/3.1-EAS-1.2";
const char S102636[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B500 Flipboard/1.5.1";
const char S102637[] = "Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B500 Flipboard/1.5.3";
const char S102638[] = "MQQBrowser/Mini2.5 (SAMSUNG-GT-C3312)";
const char S102639[] = "Mozilla/5.0 Opera/9.5 (KDDI-SH3G; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00";
const char S102640[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-us) AppleWebKit/531+ (KHTML, like Gecko) Version/5.0 Safari/531.2+ Midori/0.3";
const char S102641[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.4; es-es; WORDM902R Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S102642[] = "NokiaX7-00/UC Browser7.7.1.88/50/444";
const char S102643[] = "MQQBrowser/2.0 (NokiaN73(08);SymbianOS/9.1 Series60/3.0)";
const char S102644[] = "MQQBrowser/2.0 (NokiaN73(12);SymbianOS/9.1 Series60/3.0)";
const char S102645[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB860 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S102646[] = "NokiaC5-03/UCWEB8.1.0.104/50/800";
const char S102647[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-N7000 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102648[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC Velocity 4G Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S102649[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Transformer TF101 Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S102650[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S102651[] = "GoogleEarth/5.2.1.1588(Windows;Microsoft Windows (6.0.6002.2);en-US;kml:2.2;client:Free;type:default)";
const char S102652[] = "GoogleEarth/5.2.1.1588(Windows;Microsoft Windows (6.0.6002.2);en;kml:2.2;client:Free;type:default)";
const char S102653[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; USCCHTC-PC93100; USC-US)";
const char S102654[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; 7 Pro T7576)";
const char S102655[] = "Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; T2 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S102656[] = "JUC(Linux;U;2.2.1;Zh_cn;GT-I9000;480*800;)UCWEB7.7.0.85/139/800";
const char S102657[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; iView_1010NB Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S102658[] = "MQQBrowser/2.0 (NokiaE52-1;SymbianOS/9.1 Series60/3.0)";
const char S102659[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC_Desire Build/FRF91; 480*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/800";
const char S102660[] = "MQQBrowser/2.0 (Nokia5530;SymbianOS/9.1 Series60/3.0)";
const char S102661[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; A953 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102662[] = "Nokia303/5.0 (13.44) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia303/UC Browser7.9.0.102/70/355";
const char S102663[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LT15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S102664[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; XCD35 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S102665[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; iDx7 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102666[] = "JUC(Linux;U;Android2.3.4;Zh_cn;HTC EVO 3D X515m;540*960;)UCWEB7.8.0.95/139/355";
const char S102667[] = "Mozilla/5.0 (Linux; U; Android 2.2; pl-pl; TAB_I70 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102668[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; LG-P705 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S102669[] = "Android/1.0 (SGH-T959V FROYO)";
const char S102670[] = "MQQBrowser/25 Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i889 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102671[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P970 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S102672[] = "MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME525+ Build/4.5.3-109_DPP-11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102673[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102674[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) QQBrowser/6.3.8908.201";
const char S102675[] = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.3.8908.201";
const char S102676[] = "Mozilla/5.0 (S60V5; U; en-us; NokiaE7-00)/UC Browser8.5.0.183/50/352/UCWEB Mobile";
const char S102677[] = "NokiaC2-06/2.0 (07.48) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac2-06) UCBrowser8.4.0.159/70/352/UCWEB Mobile";
const char S102678[] = "Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40NX723; CC/GBR; en) Presto/2.7.61 Version/11.00";
const char S102679[] = "MQQBrowser/261 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1";
const char S102680[] = "MQQBrowser/261 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2";
const char S102681[] = "iBrowser/2.4 (E71-1;SymbianOS/9.1 Series60/3.0)";
const char S102682[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3 Sleipnir/1.9.2m";
const char S102683[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIA5700)/UC Browser8.5.0.183/28/352/UCWEB Mobile";
const char S102684[] = "iBrowser/Mini2.8 (NokiaC2-03/06.96)";
const char S102685[] = "Nokia112/2.0 (03.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia112) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S102686[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s5333) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S102687[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s8530) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S102688[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s5350) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S102689[] = "YahooMobile/1.0 (mail; 1.3.10.10337); (Linux; U; Android 3.2; GT-P7510 Build/HONEYCOMB_MR2);";
const char S102690[] = "MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 3.2; vi-vn; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S102691[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7510 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S102692[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.2.1; en-ie; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102693[] = "Instagram 3.0.5 Android (15/4.0.3; 160dpi; 800x1232; samsung; GT-P5100; espresso10rf; espresso10; en_GB)";
const char S102694[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P7500 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S102695[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6800 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/352";
const char S102696[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fa-fa; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (X11; Linux zvav; U; fa) Presto/2.8.119 Version/11.10";
const char S102697[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10";
const char S102698[] = "YahooMobile/1.0 (Android Live Scores; 2.1.3) (Android; GT-P7510; Android OS/3.1)";
const char S102699[] = "YahooMobile/1.0 (Android Sportacular; 71-3.0.1) (Android; GT-P7510; Android OS/4.0.4)";
const char S102700[] = "YahooMobile/1.0 (Android Sportacular; 77-3.0.7) (Android; GT-P7510; Android OS/3.2)";
const char S102701[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P1000 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355";
const char S102702[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P1000 Build/IMM76L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S102703[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S102704[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5113 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7";
const char S102705[] = "Instagram 3.4.0 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P3110; espressowifi; espresso; en_GB)";
const char S102706[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P6211 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0100, de";
const char S102707[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7310 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +1300, nz";
const char S102708[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259";
const char S102709[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259";
const char S102710[] = "Orca-Android/2.2.5-release OS/4.1.1 Model/GT-P3113 VersionCode/X Product/Messenger Carrier/null Manufacturer/samsung Brand/samsung";
const char S102711[] = "YahooMobile/1.0 (Android Sportacular; 76-3.0.6) (Android; GT-P3113; Android OS/4.1.1)";
const char S102712[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13 Maxthon/4.0.3.3000";
const char S102713[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.3.3000";
const char S102714[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P6800 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S102715[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7300 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S102716[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.0.181/145/355";
const char S102717[] = "UCWEB/2.0 (Java; U; MIDP-2.0; vi; samsung-gt-m5650) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S102718[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-b7722) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S102719[] = "NokiaC3-01/5.0 (06.05) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac3-01) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S102720[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-sgh-a877) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S102721[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian70-1) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S102722[] = "MQQBrowser/3.0 (NokiaC5-03;SymbianOS/9.1 Series60/3.0)";
const char S102723[] = "Mozilla/5.0 (Linux; Android 4.0.3; AN10BG3 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S102724[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.3; en-in; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102725[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-U8 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102726[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5560) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S102727[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5333) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S102728[] = "Opera/9.80 (Linux armv6l; U; en) Presto/2.8.115 Version/11.10 AQUOS-AS/1.0 LC-40LE835X";
const char S102729[] = "Mozilla/5.0 (Linux; Android 2.3.5; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S102730[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t310i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S102731[] = "SonyEricssonU100i/R1BA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.0";
const char S102732[] = "VertuAscentTi/2.0 (500.10) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S102733[] = "VertuAscentTi/2.0 (501.01) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S102734[] = "VertuAscentTi/2.0 (500.03) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S102735[] = "VertuAscentTi/2.0 (501.02) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S102736[] = "dopodA6288/1.0 Android/1.5/1.04 release/5.17.2009 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102737[] = "acer_E100 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5)";
const char S102738[] = "NokiaN-GageQD/1.0 SymbianOS/6.1 Series60/0.9 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S102739[] = "NokiaN-Gage/1.0 (4.03) SymbianOS/6.1 Series60/0.9 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S102740[] = "SAMSUNG-S3030/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S102741[] = "LENOVO-i919/66R";
const char S102742[] = "Nokia2700c-2/2.0 (09.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2700c-2/UC Browser7.4.0.65/70/352";
const char S102743[] = "SAMSUNG-SGH-i608/1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S102744[] = "MOT-C390/0B.A0.0FR MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S102745[] = "ZTE-F152/1.0 ACS-NF/3.2 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102746[] = "ZTE-C79/1.0 SMIT-Browser/2.0.0";
const char S102747[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) O2 Xda Atom Life";
const char S102748[] = "Lenovo-TD36t/S100 LMP/LML Release/2010.08.25 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S102749[] = "LG-KE800 MIC/1.1.14 MIDP-2.0/CLDC-1.1";
const char S102750[] = "portalmmm/2.0 K550im(c100;TB)";
const char S102751[] = "LG-KM385 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S102752[] = "SIE-E71/39 UP.Browser/7.1.0.k.5(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.k.5 (GUI) MMP/2.0";
const char S102753[] = "SIE-E71/37 UP.Browser/7.1.0.k.4(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.k.4 (GUI) MMP/2.0";
const char S102754[] = "BlackBerry8820/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100 UP.Browser/5.0.3.3";
const char S102755[] = "LG-KU730 Obigo/WAP2.0 MIDP 2.0/CLDC 1.1";
const char S102756[] = "SAMSUNG-SGH-X100/PEARL UP.Browser/6.1.0.6 (GUI) MMP/1.0";
const char S102757[] = "SAMSUNG-SGH-X550/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S102758[] = "Videocon_V1430_V1_0_3_Wap browser ver 2.0";
const char S102759[] = "HPiPAQhw692x/1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x240)";
const char S102760[] = "MOT-PARS_/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0";
const char S102761[] = "SEC-SGHE600";
const char S102762[] = "SEC-SGHE600C";
const char S102763[] = "HTC_P4350 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S102764[] = "LG-ME770 MIC/1.1.14 MIDP-2.0/CLDC-1.1";
const char S102765[] = "LG-ME770c MIC/1.1.14 MIDP-2.0/CLDC-1.1";
const char S102766[] = "SonyEricssonM608c/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 513) Opera 8.65 [en]";
const char S102767[] = "SonyEricssonM608c/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 306) Opera 8.60 [en]";
const char S102768[] = "MTC535/REF-RU-MTS-P103D2V1.0.1/2.0 Profile/MIDP-2.0";
const char S102769[] = "SAMSUNG-SGH-A746/UXHD2 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S102770[] = "SAMSUNG-SGH-J638/J638ZTHF2 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S102771[] = "SAMSUNG-SGH-G808/1.0 SHP/VPP/R5 NetFront/3.4 Qtv/5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S102772[] = "garmin-asus-Nuvifone-M20/1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S102773[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) garmin-asus-Nuvifone-M20/1.0 Mozilla/4.0";
const char S102774[] = "SAMSUNG-SGH-S366/ Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S102775[] = "LG-T300/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S102776[] = "LG/L600V/v1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102777[] = "NexianNX-G272/MTK Release/01.01.2011 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1";
const char S102778[] = "Nokia7230/5.0 (06.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7230/UC Browser7.8.0.95/70/351";
const char S102779[] = "HTC-3100/1.2 Mozilla/4.0 (compatible; MSIE 5.5; Windows CE; Smartphone; 240x320)";
const char S102780[] = "DopodD810/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)";
const char S102781[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; MB860 Build/4.5.91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102782[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2690/UC Browser7.7.1.88/70/351";
const char S102783[] = "Nokia7230/5.0 (06.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7230/UC Browser7.7.1.88/70/351";
const char S102784[] = "Xda_trion; 240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S102785[] = "SAMSUNG-SGH-i780/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S102786[] = "Lenovo-i325wg/S120 LMP/LML Release/2010.08.16 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S102787[] = "Lenovo-i325wg/S118 LMP/LML Release/2010.07.12 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S102788[] = "dopodF3188 Mozilla/5.0 (like Gecko) Obigo/Q7";
const char S102789[] = "SAMSUNG-SGH-A706/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S102790[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LS700 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102791[] = "BenQ-EF71/1.00/WAP2.0/MIDP2.0/CLDC1.0 UP.Browser/6.3.0.7.c.1.105 (GUI) MMP/2.0";
const char S102792[] = "BenQ-EF71/1.00/WAP2.0/MIDP2.0/CLDC1.0 UP.Browser/6.3.0.7.c.1.103 (GUI) MMP/2.0";
const char S102793[] = "Mozilla/5.0 (Linux; U; Android 1.6; zh-cn; dopodA8188_TD/1.0 Android/Dount release/12.3.2009 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/DRD20) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S102794[] = "SAMSUNG-SGH-F250/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S102795[] = "R830/2.0 Profile/MIDP-2.0";
const char S102796[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; Vibo A699 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102797[] = "MOT-E1000/80.3F.36. MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102798[] = "SAMSUNG-SGH-A227/A227UCHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S102799[] = "SonyEricssonX2a/R3AA Browser/Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S102800[] = "SAMSUNG-GT-I6330C_CMCC/1.0 RTK-E/1.0 SHP/VPP-R5 Release/5.25.2009 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1 SMM-MMS/1.2.0";
const char S102801[] = "WellcomW3112/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102802[] = "SonyEricssonZ530i/R6DA Java/SEMC-Java/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102803[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) 320x320; VZW; UTStar-SMT5800; Window Mobile 6.0 Standard;";
const char S102804[] = "SAMSUNG-SGH-U900L/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102805[] = "Lenovo-P650WG/S100 LMP/LML Release/2010.02.22 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S102806[] = "HTC_P5530 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S102807[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) MP6950SP";
const char S102808[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; A14 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S102809[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-tw; A14 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S102810[] = "Nokia 2605 UP.Browser/6.2.3.9.j.1.107-20081119 (GUI) MMP/2.0";
const char S102811[] = "ZTE-F850/1.0 ACS-NF/3.2 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102812[] = "LG-GT810H Netfront/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102813[] = "LGE-VX5300/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S102814[] = "NokiaE71/UCWEB7.6.0.75/28/800";
const char S102815[] = "NokiaC6-00/SymbianOS/9.1 Series60/3.0";
const char S102816[] = "NOKIAE65/UC Browser7.6.1.82/28/352";
const char S102817[] = "SamsungI8510";
const char S102818[] = "BenQ-C36/1.0/WAP2.0/MIDP2.0/CLDC1.1";
const char S102819[] = "Nokia5233/UCWEB7.8.0.87/50/800";
const char S102820[] = "NokiaC6-00/UCWEB7.8.0.87/50/999";
const char S102821[] = "Nokia7310/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7310/UC Browser7.9.0.102/70/355";
const char S102822[] = "BlackBerry7730/4.0.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102823[] = "SEC-SGHE790/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102824[] = "SEC-SGHB510/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S102825[] = "Mozilla/5.0 (SH906i;FOMA;like Gecko)";
const char S102826[] = "HTC_P3701 Opera/9.50 (Windows NT 5.1; U; es-ES)";
const char S102827[] = "MyPhoneQ22Duo/MTK Release/V1 Nov-20-2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S102828[] = "MyPhoneQ22DUO/MTK Release/V4BSP18 NOV-9-2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S102829[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SAMSUNG_B7610/1.0; Windows Phone 6.5)";
const char S102830[] = "MQQBrowser/2.0 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3";
const char S102831[] = "MQQBrowser/2.0 Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3";
const char S102832[] = "Android/1.0 (ST18i 4.0.2.A.0.42)";
const char S102833[] = "Mozilla/4.0 (compatible; MSIE 6.0; ; ) Opera 9.7";
const char S102834[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729) chromeframe/7.0.517.44";
const char S102835[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0) chromeframe/7.0.517.43";
const char S102836[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus S Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75 Mobile Safari/535.7";
const char S102837[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75 Mobile Safari/535.7";
const char S102838[] = "Mozilla/5.0 (Linux; U; Android Itfunz2.1_Eclipse_Terminator_II; zh-cn; Milestone Build/SHOLS_U2_02.36.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S102839[] = "Mozilla/5.0 (Linux; U; Android Itfunz2.1_Eclipse_Terminator_II_Final; zh-cn; Milestone Build/SHOLS_U2_02.36.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S102840[] = "Mozilla/4.0 (compatible; MSIE 5.01; Windows 2000)";
const char S102841[] = "Mozilla/4.0 (compatible; MSIE 5.00; Windows 2000)";
const char S102842[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.127 Safari/533.4";
const char S102843[] = "NOKIANokia N81/UC Browser7.9.0.102/28/355/UCWEB";
const char S102844[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]";
const char S102845[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]";
const char S102846[] = "samsung-gt-s5253/UC Browser8.0.3.107/70/352";
const char S102847[] = "1PasswordThumbs/1 CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookPro6%2C2)";
const char S102848[] = "1PasswordThumbs/1 CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookAir3%2C2)";
const char S102849[] = "MQQBrowser/2.0 (NokiaC6-00;SymbianOS/9.1 Series60/3.0)";
const char S102850[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; U20i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S102851[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S102852[] = "MQQBrowser/Mini2.5 (NokiaC1-01/04.40)";
const char S102853[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/20.0.1132.47)";
const char S102854[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; chromeframe/20.0.1132.57)";
const char S102855[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; T7575)";
const char S102856[] = "Palm680/RC1 (iPhone; U; CPU iPhone OS 2_2_1 like Mac OS X; zh-cn)/UCWEB7.2.0.46/39/997";
const char S102857[] = "Opera/9.15";
const char S102858[] = "Opera/9.12";
const char S102859[] = "Mozilla/5.0 (Windows; U; Windows NT 4.10; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13";
const char S102860[] = "sonyericssonk550i/UC Browser7.7.1.88/69/444";
const char S102861[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S102862[] = "sonyericssonk810i/UC Browser8.0.3.107/69/444";
const char S102863[] = "Mozilla/5.0 (S60V5; U; en-in; Nokia603)/UC Browser8.4.0.159/50/352/UCWEB Mobile";
const char S102864[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Pyramid Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S102865[] = "NokiaC5-03/UCWEB8.3.0.133/50/800";
const char S102866[] = "Safari(U;iOS 5.1.1;Zh-cn;320*480;)/UCWEB8.4.1.169/42/997";
const char S102867[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S102868[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102869[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7500 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S102870[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5670 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102871[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102872[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P690 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S102873[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S102874[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102875[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Lenovo A60 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102876[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i929 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102877[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; GT-B7510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S102878[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102879[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; MK16i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S102880[] = "MQQBrowser/35 Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S102881[] = "YahooMobileMessenger/1.0 (Android Mail; 1.4.5) (grouper; asus; Nexus 7; 4.1.1/JRO03D)";
const char S102882[] = "YahooMobileMessenger/1.0 (Android Mail; 1.4.5) (grouper; asus; Nexus 7; 4.1.2/JZO54K)";
const char S102883[] = "YahooMobileMessenger/1.0 (Android Messenger; 1.8.2) (grouper; asus; Nexus 7; 4.1.2/JZO54K)";
const char S102884[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B117,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3";
const char S102885[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/530.1 (KHTML, like Gecko) Chrome/2.0.166.1 Safari/530.1";
const char S102886[] = "MQQBrowser/2.6 (Nokia5800XpressMusic;SymbianOS/9.1 Series60/3.0)";
const char S102887[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.1.0.0 Chrome/12.0.742.91 Safari/534.30";
const char S102888[] = "Opera/9.80 (Windows 98; U; en) Presto/2.10.229 Version/11.62";
const char S102889[] = "UCWEB/2.0(Java; U; MIDP-2.0; id; tcl-c616) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S102890[] = "UCWEB/2.0(Java; U; MIDP-2.0; id; tcl-c616) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S102891[] = "Mozilla/5.0 (Java; U; en-us; sonyericssonk800i) UCBrowser8.2.1.144/70/352/UCWEB Mobile";
const char S102892[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-c310) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S102893[] = "Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia3120classic) UCBrowser8.4.0.159/70/352/UCWEB Mobile";
const char S102894[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-lg900g) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S102895[] = "Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i917;7.10;8862)";
const char S102896[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT-I9200 Build/4.5.1A-1_SUN-198_7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S102897[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P6210 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S102898[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S102899[] = "Mozilla/5.0 (Linux; U; Android 2.1; pt-br; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S102900[] = "TwitterAndroid/1.0.5 (109) GT-P3100/15 (samsung;espressorf)";
const char S102901[] = "TwitterAndroid/2.0.1 (122) GT-P7500/12 (samsung;GT-P7500;samsung;GT-P7500;)";
const char S102902[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S102903[] = "Android/1.0 (GT-P7501 HTJ85B)";
const char S102904[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7560 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S102905[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P7501 Build/IMM76L; CyanogenMod-Ganbarou V0.11 GT) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S102906[] = "AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; GT-P6810 Build/JRO03L)";
const char S102907[] = "Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P6810 Build/JZO54K)";
const char S102908[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; GT-P6810 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S102909[] = "MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S102910[] = "MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S102911[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S102912[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S102913[] = "Instagram 3.0.5 Android (13/3.2; 160dpi; 800x1232; samsung; GT-P7300; GT-P7300; p3; en_GB)";
const char S102914[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1010, 40, 1.8.5)";
const char S102915[] = "Rdio.Net; Android/2.4.4; Android/4.0.4; DM=samsung; DN=GT-P7500R; DC=samsung; DF=samsung/GT-P7500R/GT-P7500R:4.0.4/IMM76D/RUXLH2:user/release-keys; DH=p3";
const char S102916[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P5110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S102917[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P5110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S102918[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P7500 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0200, za";
const char S102919[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P7500 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355";
const char S102920[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P7500M Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S102921[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P2000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S102922[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S102923[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S102924[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S102925[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S102926[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000 Build/MIUI) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S102927[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-P5100 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0000, gb";
const char S102928[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; GT-P5100 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.14/271,1.9.14.271,2012-12-21 21:40, +0300, by";
const char S102929[] = "Mozilla/5.0 (Linux; U; Android 3.1; fr-fr; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.4.1000";
const char S102930[] = "Nokia5220XpressMusic/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5220xpressmusic) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S102931[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3 UCBrowser8.3.1.173/41/355";
const char S102932[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5260) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S102933[] = "Opera/9.80 (Linux armv6l; U; en) Presto/2.8.115 Version/11.10 AQUOS-AS/1.0 LC-46LE835X";
const char S102934[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL933; 55.8.03.4; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S102935[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL933; 64.4.08.7; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S102936[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL933; 67.2.73.6; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S102937[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL933; 64.4.08.7; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; pl) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S102938[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL933; 67.8.18.2; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; en) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S102939[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL933; 74.8.30.4; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S102940[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL933; 72.0.77.6; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S102941[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL933; 86.5.68.7; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S102942[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gx300) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S102943[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae71-1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S102944[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146 Safari/8536.25";
const char S102945[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_1_6 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A403 Safari/8536.25";
const char S102946[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_1 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11D4 Safari/8536.25";
const char S102947[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B142 Safari/8536.25";
const char S102948[] = "Nokia3110c/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311";
const char S102949[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-I8160P Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S102950[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-I8160 Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S102951[] = "Dolphin http client/10.2.0(106) (Android)";
const char S102952[] = "Dolphin http client/10.2.1(107) (Android)";
const char S102953[] = "Dolphin http client/10.2.2(108) (Android)";
const char S102954[] = "Dolphin http client/10.2.6(112) (Android)";
const char S102955[] = "Dolphin http client/10.2.7(113) (Android)";
const char S102956[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; SAMSUNG-GT-S3850) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S102957[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) HTC-MP6950 BM";
const char S102958[] = "Nokia2700c-2/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2700c-2/UC Browser7.5.1.77/70/351";
const char S102959[] = "GIONEE-d002/SW1.0.0/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102960[] = "MOT-K1v/08.03.08R MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/1.0";
const char S102961[] = "BlackBerry8300/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/179 UP.Browser/5.0.3.3";
const char S102962[] = "Mozilla/5.0 (Linux; U; Android 1.5; zh-tw; CHT8000 Build/CRB17) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S102963[] = "MOT-V1100/98.20.33BR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12043064) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S102964[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC P6500; OpVer 27.110.1.612) Opera 8.65 [en]";
const char S102965[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC_P6500/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1) Opera 8.65 [en]";
const char S102966[] = "LG-CU405/V1.0h Obigo/Q04C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102967[] = "KWC-K325/1.1.00 UP.Browser/6.2.3.9.g.1.107 (GUI) MMP/2.0";
const char S102968[] = "KWC-K325/1.0.04 UP.Browser/6.2.3.9.g.1.107 (GUI) MMP/2.0";
const char S102969[] = "SAMSUNG-SGHC260/C260MBAGE6 Browser/Rainbow/3.0";
const char S102970[] = "MOT-v3i/08.B4.38R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102971[] = "HTC_P5520 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S102972[] = "ZTE-A261/P103B9V1.0.0/WAP2.0 Profile";
const char S102973[] = "MOT-V191/0A.52.47R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102974[] = "SAMSUNG-SGH-E700A/BSI UP.Browser/6.1.0.6 (GUI) MMP/1.0";
const char S102975[] = "SAMSUNG-SGH-E700/BSI UP.Browser/6.1.0.6 (GUI) MMP/1.0";
const char S102976[] = "M-6460";
const char S102977[] = "SAMSUNG-SGH-C166/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S102978[] = "SEC-SGHX500/1.0 TSS/2.5";
const char S102979[] = "MOT-V197/0A.63.12R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102980[] = "LG-KM501 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S102981[] = "SAMSUNG-SGH-i718/MSIE4.01/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8)";
const char S102982[] = "SAMSUNG-SGH-J630/J630JHHD4 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S102983[] = "SAMSUNG-SGH-L770S/1.0 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S102984[] = "LG/U8550/v1.0";
const char S102985[] = "Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Nokia N70/5.0737.3.0.1; 7111) Opera 8.50 [en]";
const char S102986[] = "SAMSUNG-SGH-A516/ Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S102987[] = "SonyEricssonU100i/R7BA Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S102988[] = "YL-COOLPADS100/06.10.S100 INFRAWARE/2.8 CTC/1.0";
const char S102989[] = "Samsung-SPHM500 AU-OBIGO/Q04C1-1.17_PRE_1 MMP/2.0";
const char S102990[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) MWg Zinc II";
const char S102991[] = "SAMSUNG-SGH-J165L/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S102992[] = "SAMSUNG-SGH-B110L/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S102993[] = "SEC-SGHI620/1.0 Browser/Opera/8.65 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S102994[] = "MicroMax-X113";
const char S102995[] = "Alcatel-OT-V670A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S102996[] = "Alcatel-OT-V670/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S102997[] = "SAMSUNG-GT-C6712/C6712ZCKE2";
const char S102998[] = "samsung-gt-c6712";
const char S102999[] = "WellcoM W929 WAP Browser";
const char S103000[] = "BenQ-T60/1.0/WAP2.0/MIDP2.0/CLDC1.1";
const char S103001[] = "MOT-8720_/00.62 UP.Browser/6.2.3.4.c.1.104 (GUI) MMP/2.0";
const char S103002[] = "SAGEM-my400V/2.0/ MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.f.1.150 (GUI)";
const char S103003[] = "SAGEM-my400V/2.0/ MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.f.1.159 (GUI)";
const char S103004[] = "Bird.M19/LO,901/WAP2.0/MIDP-2.0/CLDC-1.1 Browser/UP.Browser/7.1.0.f.1.144 (GUI)";
const char S103005[] = "Bird.M19/LU,901/WAP2.0/MIDP-2.0/CLDC-1.1 Browser/UP.Browser/7.1.0.f.1.144 (GUI)";
const char S103006[] = "Bird.M19/L1,901/WAP2.0/MIDP-2.0/CLDC-1.1 Browser/UP.Browser/7.1.0.f.1.144 (GUI)";
const char S103007[] = "i-mobileS382/SW10.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103008[] = "PPC; 240x320; HTC_P4351/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S103009[] = "SAMSUNG-GT-S6700C/1.0 SHP/R5 Release/02.01.2009 Browser/NetFront3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S103010[] = "SAMSUNG-GT-S5680_TD/1.0 RTK-E/1.0 SHP/VPP-R5 Release/3.18.2010 Browser/Dolfin1.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 SMM-MMS/1.2.0";
const char S103011[] = "SAMSUNG-GT-S3710/1.0 RTK-E/1.0 SHP/VPP/R5 Release/07.09.2010 Browser/NetFront3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S103012[] = "Mozilla/5.0 (Linux; U; Android 1.6; zh-tw; Vibo-A688 Build/DONUT) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S103013[] = "SEC-SGHI300/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 WAP/1.0";
const char S103014[] = "Nokia6510/1.0 (04.12)";
const char S103015[] = "SonyEricssonK700i";
const char S103016[] = "Nokia5530/UC Browser7.2.2.51/50/352";
const char S103017[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i458B/ZTHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S103018[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i458B/ Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S103019[] = "LG-KU800/v10a Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S103020[] = "MOT-W213/0.0.45 UP.Browser/6.3.0.6.c.17 (GUI) MMP/2.0";
const char S103021[] = "SonyEricssonZ550i/R6CA Java/SEMC-Java/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103022[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; MDA_Vario_V/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103023[] = "Alcatel-OT-I650/1.0 ObigoInternetBrowser/Q03C";
const char S103024[] = "HTC_Touch_Viva_T2223 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S103025[] = "Alcatel-BH4/1.0 UP.Browser/6.2.ALCATEL MMP/1.0";
const char S103026[] = "SonyEricssonJ220i/R5N TelecaBrowser/4.08";
const char S103027[] = "SAMSUNG-SGH-Z360/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S103028[] = "LG-GD550/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S103029[] = "BenQ-T33/1.0/WAP2.0/MIDP2.0/CLDC1.1 UP.Browser/6.3.0.4.c.1.105 (GUI) MMP/2.0";
const char S103030[] = "SFR114/g8eA/FES-FR-SFR-P103D1V1.0.1/R220/2.0 Profile/MIDP-2.0";
const char S103031[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Vodafone/1.0/HTC_Mercury/3.54.162.4";
const char S103032[] = "imobileGM5220";
const char S103033[] = "Nokia-2228 UP.Browser/6.3.0.8.c.1.104 (GUI) MMP/2.0";
const char S103034[] = "LCTW660 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103035[] = "SAMSUNG-SGH-E780/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S103036[] = "Nokia-2705/1.0 UP.Browser/6.2.3.9.j.1.107-20090216 (GUI) MMP/2.0";
const char S103037[] = "MOT-QA30";
const char S103038[] = "SAMSUNG-GT-M8800C/1.0 SHP/VPP/R5 NetFront/3.5 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S103039[] = "LG-LX570 AU-MIC-LX570/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103040[] = "SonyEricssonS710a/R1A SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103041[] = "Nokia5230/UCWEB7.5.0.66/50/800";
const char S103042[] = "Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2730c-1/UC Browser7.7.1.88/70/401";
const char S103043[] = "Nokia5800w XpressMusic/UCWEB7.8.0.87/50/999";
const char S103044[] = "Nokia5800 XpressMusic/UCWEB7.8.0.87/50/800";
const char S103045[] = "SAMSUNG-SGH-D980/D980JVIB2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S103046[] = "SP; 240x320; HTC_S710/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; Smartphone; 240x320)/UC Browser7.2.2.51";
const char S103047[] = "NokiaE52-1/UC Browser7.8.0.95/28/351";
const char S103048[] = "NOKIA6710s/UC Browser7.6.1.82/28/352";
const char S103049[] = "nokia6233/UC Browser7.8.0.95/69/401";
const char S103050[] = "BlackBerry7230/4.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103051[] = "NokiaN70";
const char S103052[] = "NokiaN70-1";
const char S103053[] = "MOT-MLNBT/00.62 UP.Browser/6.2.3.4.e.1.100 (GUI) MMP/2.0";
const char S103054[] = "LG-LX140/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S103055[] = "Vodafone/1.0/SamsungSGHJ600V/J600BUGH1 Browser/NF/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103056[] = "Vodafone/1.0/SamsungSGHJ600V/J600DUHC1 Browser/NF/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103057[] = "WELLCOM/W920/MIDP2.0/CLDC1.1/Screen-240x320";
const char S103058[] = "KDDI-TS21 UP.Browser/6.0.2.273 (GUI) MMP/1.1";
const char S103059[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0) AddSugarSpiderBot www.idealobserver.com";
const char S103060[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.6.9632.201";
const char S103061[] = "sonyericssonk550i/UC Browser7.9.0.102/69/352";
const char S103062[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.8.10898.201";
const char S103063[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-I727 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103064[] = "HS-T89_TD/1.0 Android/2.2 Release/09.26.2011 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103065[] = "Dolphin http client/7.4.1(122) (Android)";
const char S103066[] = "Dolphin http client/7.4.0 Beta(119) (Android)";
const char S103067[] = "Dolphin http client/7.4.0(120) (Android)";
const char S103068[] = "MQQBrowser/Mini2.2 (Nokia2690/10.10)";
const char S103069[] = "SAMSUNG-GT-C3312";
const char S103070[] = "Mozilla/5.0 (Linux; U; Android 3.0; us-us; LogicPD Zoom2 Build/HONEYCOMB) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S103071[] = "Cherry Mobile-eclipse2;Mozilla/4.0 (compatible; MSIE 6.0; PPC;240x400)";
const char S103072[] = "Opera/9.80 (Macintosh; Intel Mac OS X 10.7.3; U; en) Presto/2.8.131 Version/11.11";
const char S103073[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]";
const char S103074[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]";
const char S103075[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]";
const char S103076[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]";
const char S103077[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Ubuntu/12.04 Chromium/15.0.874.120 Chrome/15.0.874.120 Safari/535.2";
const char S103078[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.2 (KHTML, like Gecko) Ubuntu/12.04 Chromium/15.0.874.120 Chrome/15.0.874.120 Safari/535.2";
const char S103079[] = "JUC (Linux; U; 2.2.2; zh-cn; ZTE-T_U880; 480*800) UCWEB7.9.3.103/139/800";
const char S103080[] = "MQQBrowser/2.0 (Nokia5250;SymbianOS/9.1 Series60/3.0)";
const char S103081[] = "NokiaN97 mini/UCWEB8.1.0.104/50/999";
const char S103082[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A500 Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S103083[] = "MQQBrowser/2.3 Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/7534.48.3";
const char S103084[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-N7000 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S103085[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5; WS027SH)";
const char S103086[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; Acer; Allegro)";
const char S103087[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; Acer; Allegro; bouygues)";
const char S103088[] = "JUC(Linux;U;2.3.3;Zh_cn;C8650;320*480;)UCWEB7.7.0.85/139/800";
const char S103089[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; X500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S103090[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT894 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103091[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; XT894 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103092[] = "NOKIA-Lumia 800/UCWEB2.3.0.182/47/32079";
const char S103093[] = "NOKIA-Lumia 710/UCWEB2.3.0.182/47/32079";
const char S103094[] = "NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/70/352";
const char S103095[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB200 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103096[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; streak7 Build/HMJ19) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S103097[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; VX610A Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103098[] = "JUC(Linux;U;2.3.4;Zh_cn;Android;480*800;)UCWEB7.8.1.96/139/800";
const char S103099[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103100[] = "JUC(Linux;U;Android4.0.3;Zh_cn;HTC One X;720*1184;)UCWEB7.8.0.95/139/444";
const char S103101[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Incredible S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103102[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Incredible S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103103[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ME722 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103104[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103105[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S103106[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S 4G Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103107[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S 4G Build/IML74K; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103108[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; HTC Desire Build/FRG83G)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103109[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Desire Build/FRF91)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103110[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I5500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S103111[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7000 Build/JRO03H; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103112[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7000 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103113[] = "JUC(Linux;U;Android2.3.4;Zh_cn;LG-P690;320*480;)UCWEB7.8.0.95/139/444";
const char S103114[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S103115[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103116[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P698 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S103117[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MT15i Build/4.0.2.A.0.42; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103118[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/1.0.0.8 Chrome/18.0.966.0 Safari/535.12";
const char S103119[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/1.0.0.8 Chrome/18.0.966.0 Safari/535.12";
const char S103120[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3 Sleipnir/2.0m";
const char S103121[] = "Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU1)/UC Browser8.4.0.159/50/352/UCWEB Mobile";
const char S103122[] = "YahooMobileMail/1.0 (Android Mail; 1.4.5) (grouper;asus;Nexus 7;4.1.1/JRO03D)";
const char S103123[] = "YahooMobileMail/1.0 (Android Mail; 1.4.6) (grouper;asus;Nexus 7;4.1.2/JZO54K)";
const char S103124[] = "YahooMobileMail/1.0 (Android Mail; 1.4.5) (grouper;asus;Nexus 7;4.1.2/JZO54K)";
const char S103125[] = "Mozilla/5.0 (X11; U; FreeBSD i386; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7";
const char S103126[] = "MQQBrowser/33 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/7534.48.3";
const char S103127[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I5801 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S103128[] = "NOKIA3250/UC Browser8.0.4.121/28/444/UCWEB";
const char S103129[] = "Nokia5200/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5200) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S103130[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (HUAWEI-M931, 42, 1.9.1)";
const char S103131[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LAVA iris 351 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103132[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7500 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S103133[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7300 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S103134[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2; in-id; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103135[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37)Maxthon AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S103136[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-; GT-P7500 Build/HMJ37)Maxthon AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S103137[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S103138[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6800 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S103139[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-US; GT-P6800 Build/HTJ85B) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S103140[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S103141[] = "Mozilla/5.0 (Linux; U; Android 2.1; id-id; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S103142[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103143[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103144[] = "TwitterAndroid/2.0.1 (122) GT-P6800/13 (samsung;GT-P6800;samsung;GT-P6800;)";
const char S103145[] = "Instagram 1.0.6 Android (13/3.2; 160dpi; 600x976; samsung; GT-P6200; GT-P6200; smdkc210; en_US)";
const char S103146[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-P1010 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.7/210,1.9.7.210,2012-10-25 19:53, gb";
const char S103147[] = "Rdio.Net; Android/2.2.10; Android/4.0.4; DM=samsung; DN=GT-P5113; DC=samsung; DF=samsung/espresso10wifibby/espresso10wifi:4.0.4/IMM76D/P5113UEBLH3:user/release-keys; DH=espresso10";
const char S103148[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P3113 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S103149[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; GT-P6800 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S103150[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P6800 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103151[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 3.2; id-id; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S103152[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 3.2; id-id; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S103153[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S103154[] = "com.eurosport.news.android/3.1.1/Android/3.2/samsung-GT-P6800";
const char S103155[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3030; U; en) Presto/2.8.119 Version/11.10";
const char S103156[] = "Instagram 3.1.0 Android (15/4.0.3; 160dpi; 800x1232; samsung; GT-P5100; espresso10rf; espresso10; en_GB)";
const char S103157[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.6/148,1.9.6.148,2012-09-11 08:01, gb";
const char S103158[] = "MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S103159[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S103160[] = "Android/1.0 (GT-P7320 MASTER)";
const char S103161[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7500, 36, 1.7.1)";
const char S103162[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500D Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7500D, 36, 1.7.1)";
const char S103163[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500M Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7500M, 41, 1.9.0)";
const char S103164[] = "Android/4.0.4; samsung GT-P6800; 2.3.2";
const char S103165[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P3110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S103166[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P5113 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355";
const char S103167[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103168[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103169[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103170[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P6200, 36, 1.7.1)";
const char S103171[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P6200L, 39, 1.8.2)";
const char S103172[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S103173[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S103174[] = "Instagram 3.4.0 Android (8/2.2.1; 240dpi; 600x1024; samsung; GT-P1010; GT-P1010; p1lite; en_US)";
const char S103175[] = "Instagram 3.4.2 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P6210; GT-P6210; smdk4210; en_US)";
const char S103176[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7310 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0000, gb";
const char S103177[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7310 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13 Flipboard/1.9.19/285,1.9.19.285,2013-01-24 11:21, +0800, china, cn";
const char S103178[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3113 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S103179[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3113 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S103180[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-P7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0100, be";
const char S103181[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P5100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S103182[] = "theScore/3.1.0 Android-OS/3.2 (Android; Phone, GT-P6210)";
const char S103183[] = "theScore/3.1.1 Android-OS/2.2.1 (Android; Phone, GT-P1010)";
const char S103184[] = "theScore/3.1.1 Android-OS/4.1.1 (Android; Tablet, GT-P5113)";
const char S103185[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; GT-P5100 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31";
const char S103186[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P5100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31";
const char S103187[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7510 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S103188[] = "theScore/3.2.1 Android-OS/4.0.4 (Android; Tablet, GT-P7510)";
const char S103189[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7510 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S103190[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; gt-s5560) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S103191[] = "UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; samsung-gt-s5560i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S103192[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-ks360 teleca) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S103193[] = "JUC (Linux; U; 4.1.2; zh-cn; GT-I8262D; 480*800) UCWEB7.9.4.145/139/800";
const char S103194[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S103195[] = "MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S103196[] = "MQQBrowser/3.0 (NokiaN8-00;SymbianOS/9.1 Series60/3.0)";
const char S103197[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3650) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S103198[] = "SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; sonyericssonw20) UCBrowser8.3.0.154/70/444/UCWEB Mobile";
const char S103199[] = "MQQBrowser/3.5/Adr (Linux; U; 2.3.3; zh-cn; HTC Desire S Build/GRI40;480*800)";
const char S103200[] = "Nokia6500s-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia6500s-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S103201[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; zte-p80aiii-f160) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S103202[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t310) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S103203[] = "UCWEB/2.0 (Java; U; MIDP-2.0; es-LA; lg-t310) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S103204[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6630) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S103205[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; s5230) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S103206[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-ks360 teleca) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S103207[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t325) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S103208[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3370) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S103209[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3653w) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S103210[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 55LM6700-TA; 04.10.10; 0xX;); LG NetCast.TV-2012";
const char S103211[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+PORTAL_KEY+SCREEN+TUNER; LGE; SP820; 08.00.65; 0xX;); LG NetCast.TV-2012 0";
const char S103212[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (ZmEufAdvOJ6+jpzS29ugr8sbtxJwbNEwTvj+UjrNdn8=) Chromium/26.0.1410.63 Chrome/26.0.1410.63 Safari/535.2";
const char S103213[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (BiJcJ/NnWWNRCJbpE1IDAUARApQI13bBpXjUKIJ3NX0=) Chromium/26.0.1410.63 Chrome/26.0.1410.63 Safari/535.2";
const char S103214[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (2+5mbYbRy06RbV9AqKfDxK1OxQTSP/3ogwQnfKgRpe8=) Chromium/26.0.1410.63 Chrome/26.0.1410.63 Safari/535.2";
const char S103215[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (HRfVdPGb2DXDoQv7j0jiZ7kumjupqYEZuuv+uUjYaXY=) Chromium/26.0.1410.63 Chrome/26.0.1410.63 Safari/535.2";
const char S103216[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (XPokjOYxFHsUdRJ3O2EuYjHwfjohEnE9z01PffnbHEI=) Chromium/26.0.1410.63 Chrome/26.0.1410.63 Safari/535.2";
const char S103217[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gu220) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S103218[] = "Mozilla/5.0 (Windows Phone 8.0; Trident/7.0; rv:11.0; IEMobile/11.0; ARM; Touch; HTC; Windows Phone 8X by HTC) like Gecko";
const char S103219[] = "Mozilla/5.0 (Windows Phone 8.0; Trident/7.0; rv:11.0; IEMobile/11.0; ARM; Touch; HTC; PM23300) like Gecko";
const char S103220[] = "Mozilla/5.0 (Windows Phone 8.0; ARM; Trident/7.0; Touch; rv:11.0; IEMobile/11.0; HTC; PM23300) like Gecko";
const char S103221[] = "Mozilla/5.0 (Windows Phone 8.0; ARM; Trident/7.0; Touch; rv:11.0; IEMobile/11.0; HTC; Windows Phone 8X by HTC) like Gecko";
const char S103222[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-c310) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S103223[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; SAMSUNG-GT-S5620) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S103224[] = "Nokia114/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia114) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S103225[] = "Nokia5130c-2/2.0 (06.93) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5130c-2/UC Browser7.4.0.65/70/351";
const char S103226[] = "SAMSUNG-SGH-Z238/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1/*MzUzMjEyMDEwMDQ4ODI5";
const char S103227[] = "SAMSUNG-SGH-E908/NetFront 3.2/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzUyMDAzMDExNzEyNjE3";
const char S103228[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; apanda-A60 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S103229[] = "MOT-V177 UP.Browser/6.2.3.9.c.6 (GUI) MMP/2.0";
const char S103230[] = "MOT-V177/0.1.66 UP.Browser/6.2.3.9.c.9 (GUI) MMP/2.0";
const char S103231[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; O2 Xda Zinc; PPC; 240x320)";
const char S103232[] = "SAMSUNG-SGH-Z510/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103233[] = "SAMSUNG-M3200/M3200JCHJ1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S103234[] = "samsung-gt-s5620";
const char S103235[] = "SAMSUNG-GT-S5620/S5620DDJE1";
const char S103236[] = "SAMSUNG-SGH-X496/X496UZEI1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S103237[] = "FLY-SX210/Profile/MIDP.2.0Configuration/CLDC.1.0";
const char S103238[] = "FLY-SX210/Profile/MIDP.2.0Configuration/CLDC.1.0, FLY-SX210/Profile/MIDP.2.0Configuration/CLDC.1.0";
const char S103239[] = "ASUS-GalaxyII/1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; ASUS-GalaxyII)";
const char S103240[] = "BlackBerry8830/4.5.0.138 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105 UP.Browser/5.0.3.3";
const char S103241[] = "SAMSUNG-S9402/S9402XEIA2 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103242[] = "BlackBerry8110/4.3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102 UP.Browser/5.0.3.3";
const char S103243[] = "W3319 WAP Browser";
const char S103244[] = "Palm750/v0005 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S103245[] = "Alcatel-OT-565A/1.0 ObigoInternetBrowser/Q03C";
const char S103246[] = "MOT-RAZRV3xR/86.8A.10R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 2089) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]";
const char S103247[] = "MOT-RAZRV3xR/86.43.15P BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 2121) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [es]";
const char S103248[] = "SPICE-C6600/3.00 IAC/V1.0 WESTLAKE/0.1.8";
const char S103249[] = "LG-GT810H Netfront/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S103250[] = "Mozilla/5.0 (SCH-W609/W609DJ13;U;REX/3.0;BREW/3.1.5;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*400;CTC/2.0) POLARIS/6.2.WEB";
const char S103251[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) O2 Xda Atom Life";
const char S103252[] = "Haier-T10C/1.0 iPanel/2.0 WAP2.0 (compatible; UP.Browser/6.2.2.4; UPG1; UP/4.0; Embedded)";
const char S103253[] = "LG_TB200_CMCC A2000pH/2.32.00 Release/9.14.2009 Browser/Teleca/Q7.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103254[] = "SoftBank/1.0/920P/PJP23 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103255[] = "SonyEricssonW902plus/R3DB Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.2";
const char S103256[] = "SAMSUNG-SGH-A821/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S103257[] = "SAMSUNG-SGH-E747/ Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S103258[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2690/UC Browser7.2.2.51/70/352";
const char S103259[] = "T-Mobile Dash Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 320x240; Smartphone)";
const char S103260[] = "LG/KB620/v1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S103261[] = "MOT-W385m/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0";
const char S103262[] = "SCH-R400 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S103263[] = "LG/KT525/v1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S103264[] = "Spice/1.0/QT-68/Obigo Browser/Q03C Browser/Q03C MMS/1.0/MIDP-2.0 Configuration/CLDC-1.1";
const char S103265[] = "MOT-W403/0.1.27 UP.Browser/6.3.0.6.c.43 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103266[] = "LG-B2050 MIC/WAP2.0 MIDP-2.0/CLDC-1.0";
const char S103267[] = "Motorola-ic602 Obigo/Q04C1 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103268[] = "SCH-A930 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S103269[] = "SonyEricssonT230/R101";
const char S103270[] = "SAGEM-my401X/1.0/ MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.g.2.107 (GUI) MMP/2.0";
const char S103271[] = "Samsung-SCHR610 AU-OBIGO/Q04C1-1.22 MMP/2.0";
const char S103272[] = "LG-TU750/V10a Teleca/Q7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S103273[] = "SonyEricssonZ555a/R10CA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103274[] = "LG-KG110 V100 AU/4.10 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S103275[] = "HTC Streaming Player us_cellular_wwe / 1.0 / htc_lexikon / 2.2";
const char S103276[] = "HTC Streaming Player verizon_wwe / 1.0 / htc_lexikon / 2.2";
const char S103277[] = "SAMSUNG-GT-B7300/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S103278[] = "Nokia5232/CMCC (SymbianOS/9.4; Series60/5.0; Release/12.1.091; Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Safari/525";
const char S103279[] = "Mozilla/5.0(Linux;U;Android2.2;Zh_cn;XCD35;480*800;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355";
const char S103280[] = "JUC(Linux;U;Android2.2;Zh_cn;XCD35;480*800;)UCWEB7.8.0.95/139/355";
const char S103281[] = "SAMSUNG-GT-S5253/S5253DDKC1";
const char S103282[] = "samsung-gt-s5253";
const char S103283[] = "PANTECH-C810/R01 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S103284[] = "Telkomsel F188/1.0 ACS-NF/3.3 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103285[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9860; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/6.1.0.60 Mobile Safari/534.11+";
const char S103286[] = "LG-MG230d Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S103287[] = "LG-MG230 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S103288[] = "Lenovo-i909/S155 Release/2008.08.20 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S103289[] = "Lenovo-i909/S141 Release/2007.11.28 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S103290[] = "Nokia6108/1.0 (05.30) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S103291[] = "BenQ-EF61/1.00/WAP2.0/MIDP2.0/CLDC1.0 UP.Browser/6.3.0.4.c.1.102 (GUI) MMP/2.0";
const char S103292[] = "SAMSUNG-GT-B7320/1.0 MIDP/2.1 CLDC/1.1 Mozilla/4.0";
const char S103293[] = "SAMSUNG-GT-I6320C_CMCC/1.0 RTK-E/1.0 SHP/VPP/R5 Release/17.08.2008 Browser/NetFront3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S103294[] = "SAMSUNG-GT-I8510T/1.0 SymbianOS/9.3 Series60/3.2 Release/5.30.2008 Browser/OSS3.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S103295[] = "SHARP/1.0/SH9010C/SHC001 Mozilla/4.08 (compatible) Browser/NF/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103296[] = "HTC_S743 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S103297[] = "Lenovo-X1m/S102 LMP/LML Release/2008.11.21 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S103298[] = "WidgetManager; SAMSUNG-GT-I8320-Vodafone;AppleWebKit/528.5+(X11;U;Linux;en;i8320BUJC1)";
const char S103299[] = "Mozilla/5.0 (TY-E379/212810_7350_V0101;U;MTK/08B;Profile/MIDP-2.0 Configuration/CLDC-1.1;240X320;CTC/2.0) Obigo/Q05A";
const char S103300[] = "Mozilla/5.0 (TY-E379/212210_6480_V0701;U;MTK/08B;Profile/MIDP-2.0 Configuration/CLDC-1.1;240*320;CTC/2.0) Obigo/Q05A";
const char S103301[] = "T6/1.0 ACS-NF/3.4 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103302[] = "Mozilla/5.0 (SCH-F619/F619DG26;U;REX/3.0;BREW/3.1.5;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*320;CTC/2.0) POLARIS/6.15.WEB";
const char S103303[] = "Panasonic-VS2/1.0/RP1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103304[] = "MOT-JACQU/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0";
const char S103305[] = "Alcatel-MISS-SIXTY-MSX-10/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A";
const char S103306[] = "LG-S5200 MIC/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S103307[] = "Mot-WX265/1.0 ObigoInternetBrowser/Q03C";
const char S103308[] = "Alcatel-OT-315M/1.0 ObigoInternetBrowser/Q03C";
const char S103309[] = "Nokia3230/2.0 (5.0717.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.4.0.65/27/351";
const char S103310[] = "HTC_Touch_Diamond2_T5353 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65";
const char S103311[] = "Nokia5800 XpressMusic/UCWEB7.6.0.75/50/800";
const char S103312[] = "Mozilla/5.0 (SymbianOS/9.1; U; en-us) AppleWebKit/413 (KHTML, Like Gecko) Safari/413 Nokia/X01NK";
const char S103313[] = "Nokia N96/UC Browser7.7.1.88/28/444";
const char S103314[] = "NokiaN97 mini/UC Browser7.6.1.82/50/351";
const char S103315[] = "SEC-SGHC250/1.0";
const char S103316[] = "NOKIAE61/UC Browser7.7.1.88/28/352";
const char S103317[] = "NOKIAE61i/UC Browser7.7.2.91/28/355";
const char S103318[] = "sonyericssonk810i/UC Browser7.7.1.88/69/444";
const char S103319[] = "NOKIA5700/UC Browser7.7.1.88/28/444";
const char S103320[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) /Kyocera-E4000";
const char S103321[] = "HTC Streaming Player cingular_us / 1.0 / htc_liberty / 2.2.2";
const char S103322[] = "HTC_P3300 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S103323[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) PPC; 240x320; HTC_P3300/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103324[] = "SAMSUNG-SGH-E376/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S103325[] = "SonyEricssonT280i";
const char S103326[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) Xda_trion; 240x320";
const char S103327[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE;O2 Xda Flame; PPC; 480x640 )";
const char S103328[] = "SAMSUNG-SGH-X566/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S103329[] = "SoftBank/1.0/912T/TJ001 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103330[] = "SoftBank/1.0/921T/TJ001 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103331[] = "KDDI-PT35 UP.Browser/6.2.0.15.1.1 (GUI) MMP/2.0";
const char S103332[] = "PANTECH-C510/R01 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103333[] = "KDDI-KC3K UP.Browser/6.2.0.14.1 (GUI) MMP/2.0";
const char S103334[] = "Opera/8.01 (J2ME/MIDP; Opera Mini/3.9/1724; en; U; ssr)";
const char S103335[] = "Mozilla/5.0 (en-US) AppleWebKit/534.51 (KHTML, like Gecko) Chrome/12.0.742 Safari/534.51";
const char S103336[] = "Chrome/12";
const char S103337[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.5.9225.201";
const char S103338[] = "Mozilla/5.0 (Linux; U; Android 2.2.2;en-us; ZTE-C_R750+/FRY_INT_R750PLUSV1.0.0B02; 320*480; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1)";
const char S103339[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-N7000 Build/GINGERBREAD; 800*1280) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/800";
const char S103340[] = "Flipboard/1.7.2 CFNetwork/485.12.7 Darwin/10.4.0";
const char S103341[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.7 (KHTML, like Gecko) Ubuntu/12.04 Chromium/16.0.912.77 Chrome/16.0.912.77 Safari/535.7";
const char S103342[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.7 (KHTML, like Gecko) Ubuntu/12.04 Chromium/16.0.912.77 Chrome/16.0.912.77 Safari/535.7";
const char S103343[] = "Opera/9.80 (iPad; Opera Mini/7.1.32694/27.1407; U; en) Presto/2.8.119 Version/11.10";
const char S103344[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Kyros MID7022 8K Build/GRK39F; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103345[] = "JUC (Linux; U; 2.3.5; zh-cn; MEIZU_MX; 640*960) UCWEB7.9.3.103/139/800";
const char S103346[] = "Nokia801T_TD/CMCC (SymbianOS/3; Release/027.005; Series60/5.2 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko)NokiaBrowser/7.3.1.37 Mobile Safari/533.4 3gpp-gba";
const char S103347[] = "MQQBrowser/2.8 Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103348[] = "NOKIA6120c/UC Browser7.0.0.41/28/352";
const char S103349[] = "Opera/9.80 (Windows NT 6.0; U; en) Presto/2.6.31 Version/10.70";
const char S103350[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB7.2.0.46/31/800";
const char S103351[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Iron/4.0.275.2 Chrome/4.0.275.2 Safari/532.5";
const char S103352[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8A400 Safari/6533.18.5";
const char S103353[] = "NokiaC7-00/SymbianOS/9.1 Series60/3.0";
const char S103354[] = "Nokia5230/UCWEB7.2.2.51/50/800";
const char S103355[] = "Opera/9.80 (X11; FreeBSD 8.2-STABLE i386; U; en) Presto/2.9.168 Version/11.50";
const char S103356[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; FujitsuToshibaMobileCommun; IS12T; KDDI)";
const char S103357[] = "HTC_Touch_Diamond2_T5353 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB8.1.0.104/31/800";
const char S103358[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en_us ; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S103359[] = "Dolphin/1.4 (+http://otsing.delfi.ee/?c=crawler)";
const char S103360[] = "Dolphin http client/1.0(1) (Android)";
const char S103361[] = "Dolphin http client/1.0.6(162) (Android)";
const char S103362[] = "sonyericssonk770i/UC Browser7.7.1.88/69/444";
const char S103363[] = "Mail/1251 CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookPro8%2C2)";
const char S103364[] = "NokiaX7-00/UC Browser7.9.0.102/50/355/UCWEB";
const char S103365[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; I897 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103366[] = "Safari5530.19 CFNetwork/438.14 Darwin/9.8.0 (i386) (MacBook4%2C1)";
const char S103367[] = "NokiaC6-00/UCWEB8.1.0.104/50/800";
const char S103368[] = "GoogleEarth/6.2.0001.6400(iPhoneOS (iPhone4S);Mac OS X (5.1.1);en;kml:2.2;client:Free;type:default)";
const char S103369[] = "GoogleEarth/6.2.0001.6400(iPhoneOS (iPad2);Mac OS X (5.1.1);en;kml:2.2;client:Free;type:default)";
const char S103370[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; HTC Desire Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103371[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Desire Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103372[] = "MQQBrowser/Mini2.6 (NokiaC1-01/04.40)";
const char S103373[] = "Mozilla/5.0 (S60V5; U; en-in; Nokia700)/UC Browser8.4.0.159/50/352/UCWEB Mobile";
const char S103374[] = "NOKIA-Lumia 800/UCWEB2.1.0.155/47/32079";
const char S103375[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-N7000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, gb";
const char S103376[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, sg";
const char S103377[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; GW910)";
const char S103378[] = "NOKIA5630d-1/UC Browser7.9.0.102/28/355/UCWEB";
const char S103379[] = "MQQBrowser/Mini2.5 (NokiaX2-01/08.63)";
const char S103380[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; hd-us; A43 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S103381[] = "JUC(Linux;U;Android2.3.7;Zh_cn;E15i;320*480;)UCWEB7.8.0.95/139/352";
const char S103382[] = "MQQBrowser/25 Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103383[] = "Mozilla/5.0 (Android; Linux armv7l; rv:9.0) Gecko/ Firefox/9.0 Fennec/9.0";
const char S103384[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ZTE-C_N760/N760V1.0.0B03; 320*480; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103385[] = "Mozilla/5.0 (Linux; U; Android 2.3.9; en-us; AT&T HD7 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103386[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103387[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; pt-pt; Racer Build/GRJ22; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103388[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-N7000 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S103389[] = "MQQBrowser/3.3/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103390[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-U V880 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103391[] = "MQQBrowser/3.3/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103392[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S103393[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-E510 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S103394[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P698 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S103395[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MB525 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103396[] = "MQQBrowser/3.3/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8650+ Build/V100R001C92B861) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103397[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I5510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S103398[] = "MQQBrowser/3.1/ZTE-TU880_TD/1.0 Linux/2.6.32 Android/2.2 Release/5.25.2011 Browser/AppleWebKit533.1";
const char S103399[] = "YahooMobileMessenger/1.0 (Android Mail; 1.4.4) (glacier; HTC; HTC Glacier; 2.3.4/GRJ22)";
const char S103400[] = "YahooMobileMessenger/1.0 (Android Messenger; 1.6.0) (glacier; HTC; HTC Glacier; 2.3.4/GRJ22)";
const char S103401[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103402[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC One X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103403[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103404[] = "JUC (Linux; U; 2.2; zh-cn; HTC_Aria_A6380; 320*480) UCWEB7.9.3.103/139/800";
const char S103405[] = "MQQBrowser/27 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2";
const char S103406[] = "Facebook 4110.0 (iPod touch; iPhone OS 4.3.5; en_US)";
const char S103407[] = "YahooMobile/1.0 (Android Sportacular; 66-2.2.4) (Android; Nexus 7; Android OS/4.1.1)";
const char S103408[] = "YahooMobile/1.0 (Android Sportacular; 67-2.2.5) (Android; Nexus 7; Android OS/4.1.2)";
const char S103409[] = "JUC (Linux; U; 2.1-update1; zh-cn; ZTE-C_N600; 240*320) UCWEB7.9.3.103/140/32160";
const char S103410[] = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Ubuntu/8.04 Chromium/8.0.552.215 Chrome/8.0.552.215 Safari/534.10";
const char S103411[] = "NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiax3-02) UCBrowser8.2.1.144/70/355/UCWEB Mobile";
const char S103412[] = "iBrowser/Mini2.7 (NokiaX2-02/11.57)";
const char S103413[] = "Nokia300/5.0 (07.03) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia300) UCBrowser8.2.1.144/70/355/UCWEB Mobile";
const char S103414[] = "Opera/9.80 (X11; FreeBSD 9.0-RELEASE i386; U; en) Presto/2.10.289 Version/12.00";
const char S103415[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-t325) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S103416[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LT15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S103417[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-c105) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S103418[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Incredible 2 Build/JRO03H; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103419[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-s365) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S103420[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s8500) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S103421[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.9; GT-I9200 Build/IML74K)";
const char S103422[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; GT-I9200 Build/4.1.A.0.562) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103423[] = "Mozilla/5.0 (Linux; U; Android 4.0.9; vi-vn; GT-I9200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103424[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-I9200 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.25/301,1.9.25.301,2013-02-22 16:02, +0000, pl";
const char S103425[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9200 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.26/377,1.9.26.377,2013-03-16 17:10, +0100, pl";
const char S103426[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9152 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103427[] = "YahooMobile/1.0 (mail; 1.3.8.9816); (Linux; U; Android 3.1; GT-P7510 Build/HONEYCOMB_MR1);";
const char S103428[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6800 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S103429[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7500 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S103430[] = "YahooMobileMail/1.0 (Android Mail; 1.4.5) (GT-P6210;samsung;GT-P6210;3.2/HTJ85B)";
const char S103431[] = "YahooMobileMail/1.0 (Android Mail; 1.4.6) (GT-P1010;samsung;GT-P1010;2.2.1/FROYO)";
const char S103432[] = "MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S103433[] = "MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200L Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S103434[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S103435[] = "YahooMobileMessenger/1.0 (Android Mail; 1.4.5) (espresso10wifi; samsung; GT-P5110; 4.0.4/IMM76D)";
const char S103436[] = "JUC(Linux;U;Android4.1.1;Zh_cn;GT-P3100;600*976;)UCWEB7.8.0.95/139/351";
const char S103437[] = "Mozilla/5.0(Linux;U;Android4.1.1;Zh_cn;GT-P3100;600*976;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/444";
const char S103438[] = "JUC(Linux;U;Android4.1.2;Zh_cn;GT-P3100;600*976;)UCWEB7.8.0.95/139/352";
const char S103439[] = "YahooMobile/1.0 (Android Sportacular; 69-2.3) (Android; GT-P7500R; Android OS/3.1)";
const char S103440[] = "YahooMobile/1.0 (Android Sportacular; 69-2.3) (Android; GT-P7500R; Android OS/4.0.4)";
const char S103441[] = "YahooMobile/1.0 (Android Sportacular; 76-3.0.6) (Android; GT-P7500; Android OS/3.2)";
const char S103442[] = "YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 4.0.4; GT-P7500 Build/Unknown);";
const char S103443[] = "YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 4.0.4; GT-P7500R Build/Unknown);";
const char S103444[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-P1000 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355";
const char S103445[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P1000 Build/JRO03R) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S103446[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7500 Build/HTJ85B; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/352";
const char S103447[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3100, 40, 1.8.5)";
const char S103448[] = "Instagram 3.3.3 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P3113; espressowifi; espresso; en_CA)";
const char S103449[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-ca; GT-P7500R Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7500R, 34, 1.6.0)";
const char S103450[] = "Mozilla/5.0 (Linux; U; Android 3.2; es-us; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7500, 39, 1.8.2)";
const char S103451[] = "Android/4.0.4; samsung GT-P3100; 2.3.1";
const char S103452[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P6200 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.16/276,1.9.16.276,2012-12-21 16:37, +0100, be";
const char S103453[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7510 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0200, ch";
const char S103454[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GT-P7510 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0100, fr";
const char S103455[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P5110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S103456[] = "WhatsApp/2.9.369 Android/3.2 Device/samsung-GT-P6200";
const char S103457[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P6810 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0800, au";
const char S103458[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P6810 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S103459[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7310 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.4.1000";
const char S103460[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P5110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S103461[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000";
const char S103462[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P3110 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.53463";
const char S103463[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P6200 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.1.258 U3/0.8.0 Mobile Safari/534.31";
const char S103464[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; GT-P3100 Build/JRO03C) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S103465[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P6200 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.49 Safari/537.31";
const char S103466[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 OverDrive Media Console/2.6.4";
const char S103467[] = "UCWEB/2.0 (Java; U; MIDP-2.0; vi; opera) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S103468[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-ks360 teleca) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S103469[] = "Nokia6085/2.0 (06.00) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6085) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S103470[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-c310) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S103471[] = "UCWEB/2.0(Java; U; MIDP-2.0; es-la; lg-t310) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S103472[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-t310) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S103473[] = "UCWEB/2.0(Java; U; MIDP-2.0; fa; lg-t310) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S103474[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5603) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S103475[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5603) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S103476[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9295 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/391,1.9.28.391,2013-04-07 20:11, +0000, pl";
const char S103477[] = "JUC (Linux; U; 2.3.5; zh-cn; MI-ONE Plus; 480*854) UCWEB8.5.1.179/145/800";
const char S103478[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-R720 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103479[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5753e) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S103480[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-c105) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S103481[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-S7270 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S103482[] = "AndroidDownloadManager/4.2.2 (Linux; U; Android 4.2.2; GT-S7272 Build/JDQ39)";
const char S103483[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-S7272 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103484[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sgh-f480) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S103485[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-s310) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S103486[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LM6200-UE; 03.21.12; 0xX;); LG NetCast.TV-2012";
const char S103487[] = "Mozilla/5.0 (DirectFB; Linux i686) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+PORTAL_KEY+SCREEN+TUNER; LGE; 47LM9600-NC; 09.00.00; 0xX;); LG NetCast.TV-2012";
const char S103488[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LM6700-UA; 04.31.43; 0xX;); LG NetCast.TV-2012";
const char S103489[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LM6700-TA; 03.12.04; 0xX;); LG NetCast.TV-2012 0";
const char S103490[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL963; 64.4.08.7; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S103491[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL963; 74.8.30.4; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; de) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S103492[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL963; 74.8.30.4; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S103493[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL963; 82.3.59.9; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S103494[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL963; 64.6.47.6; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; ru) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S103495[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL963; 87.7.55.0; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; ru) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S103496[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_L4333; 7.1.33.0.1.1; a5; ) ; ToshibaTP/2.0.0 (+DRM) ; en) AppleWebKit/537.4 (KHTML, like Gecko) TOSHIBA-DTV (DTV_L4333; 7.1.33.0.1.1; 2013A; EU)";
const char S103497[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_L4333; 7.1.37.16.01.1; a5; ) ; ToshibaTP/2.0.0 (+DRM) ; fr) AppleWebKit/537.4 (KHTML, like Gecko) TOSHIBA-DTV (DTV_L4333; 7.1.37.16.01.1; 2013A; EU)";
const char S103498[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_L4333; 7.1.45.08.01.1; a5; ) ; ToshibaTP/2.0.0 (+DRM) ; fr) AppleWebKit/537.4 (KHTML, like Gecko) TOSHIBA-DTV (DTV_L4333; 7.1.45.08.01.1; 2013A; EU)";
const char S103499[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_L4333; 7.1.45.22.01.1; a5; ) ; ToshibaTP/2.0.0 (+DRM) ; fr) AppleWebKit/537.4 (KHTML, like Gecko) TOSHIBA-DTV (DTV_L4333; 7.1.45.22.01.1; 2013A; EU)";
const char S103500[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_L4333; 7.1.56.12.01.1; a5; ) ; ToshibaTP/2.0.0 (+DRM) ; fr) AppleWebKit/537.4 (KHTML, like Gecko) TOSHIBA-DTV (DTV_L4333; 7.1.56.12.01.1; 2013A; EU)";
const char S103501[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_L4333; 7.1.68.00.01.1; a5; ) ; ToshibaTP/2.0.0 (+DRM) ; fr) AppleWebKit/537.4 (KHTML, like Gecko) TOSHIBA-DTV (DTV_L4333; 7.1.68.00.01.1; 2013A; EU)";
const char S103502[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8500) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S103503[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t325) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S103504[] = "Mozilla/5.0 (Windows Phone 8.0; Trident/7.0; rv:11.0; IEMobile/11.0; ARM; Touch; NOKIA; Lumia 822) like Gecko";
const char S103505[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/23.0.1271.95 Chrome/23.0.1271.95 Safari/537.6+ Ubuntu/13.10 (3.6.1-2ubuntu3) Epiphany/3.6.1";
const char S103506[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/23.0.1271.95 Chrome/23.0.1271.95 Safari/537.6+ Ubuntu/13.10 (3.6.1-2ubuntu3) Epiphany/3.6.1";
const char S103507[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lemon t99) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S103508[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; SAMSUNG-GT-S3653) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S103509[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Coast/1.0.1.62563 Mobile/9A405 Safari/7534.48.3";
const char S103510[] = "HTC_HD2_T8585 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S103511[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) HTC_HD2_T8585/480x800";
const char S103512[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) PPC; 240x320; HTC_P3600i/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103513[] = "Nokia2690/2.0 (09.70) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.4.0.65/70/351";
const char S103514[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2690/UC Browser7.4.0.65/70/351";
const char S103515[] = "LG/GD880 Browser/AppleWebKit/531 Widget/LGMW/3.0 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S103516[] = "PPC; 240x320; HTC_WIZARD/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S103517[] = "PPC; 240x320; HTC_WIZARD/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S103518[] = "Nokia 3310";
const char S103519[] = "HTC-ST7377/2.10.502.4 (93577) Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S103520[] = "HTC_P3400i-Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S103521[] = "MOT-V550/0E.65.25R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S103522[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-tw; HTC Tattoo Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103523[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-tw; HTC Tattoo Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103524[] = "Panasonic-VS3/1.0/RP1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103525[] = "Mozilla/5.0 (Linux; U; Android BlueSparks-V5; fr-fr; SonyEricssonX10i Build/2.1.A.0.435) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S103526[] = "Mozilla/5.0 (Linux; U; Android 2.1.1; en-gb; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103527[] = "MOT-V620/0E.65.25R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S103528[] = "SAMSUNG-SGH-X460C/1.0 UP.Browser/6.1.0.6 (GUI) MMP/1.0";
const char S103529[] = "SAMSUNG-SGH-E630C/1.0 UP.Browser/6.2.2.6 (GUI) MMP/1.0";
const char S103530[] = "SAMSUNG-SGH-E630/1.0 UP.Browser/6.2.2.6 (GUI) MMP/1.0";
const char S103531[] = "SEC-SGHD720 SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S103532[] = "SEC-SGHD720/1.0 SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S103533[] = "HTIL-V70/1.0 Release/11.05.2008 Browser/Obigo2.0";
const char S103534[] = "SonyEricssonW902/R3EG SEMC/NewsReader/1.0";
const char S103535[] = "LG-KG120 MIC/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S103536[] = "LG-GS200 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S103537[] = "Huawei-U526/1.0 BREW/3.1 Browser/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103538[] = "SAGEM-myX-7/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.5.d.3 (GUI) MMP/1.0";
const char S103539[] = "LG-MG295 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S103540[] = "LG/KE990c/v10a Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103541[] = "SAMSUNG-SGH-X600/K3 UP.Browser/6.1.0.6 (GUI) MMP/1.0";
const char S103542[] = "SEC-SGHP510/1.0 UP.Browser/6.2.2.6 (GUI) MMP/1.0";
const char S103543[] = "SEC-SGHP510 UP.Browser/6.2.2.6 (GUI) MMP/1.0";
const char S103544[] = "HTC_P4350 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S103545[] = "LGE-AX830/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S103546[] = "LG/KU450/v10a Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103547[] = "LGE-LG800 AU-OBIGO/Q04C1-1.22  MMP/2.0";
const char S103548[] = "BlackBerry8100/4.2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/120 UP.Browser/5.0.3.3";
const char S103549[] = "BenQ-E81/48 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.1.341 (GUI) MMP/2.0 Push/PO";
const char S103550[] = "BenQ-E81/52 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.1.421 (GUI) MMP/2.0 Push/PO";
const char S103551[] = "ZTE-F103/1.0 ACS-NF/3.4 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103552[] = "LG-KU970/v1.0 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S103553[] = "SonyEricssonW380i/R10CA SEMC/NewsReader/1.0";
const char S103554[] = "LG-TU575/V0.9 Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103555[] = "SAMSUNG-SGH-A226/ Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S103556[] = "Motorola-V9m Obigo/Q04C1 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103557[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i520/XEGH1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S103558[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i520V/BUGH1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S103559[] = "TS705/1.0";
const char S103560[] = "BenQ-S81/1.00/WAP2.0/MIDP2.0/CLDC1.1 UP.Browser/6.3.0.3.c.4 (GUI) MMP/2.0";
const char S103561[] = "LG-KE820 MIC/1.1.14 MIDP-2.0/CLDC-1.1";
const char S103562[] = "Nokia3300/1.0 (5.20) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S103563[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; T-Mobile_Espresso Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103564[] = "MQQBrowser/21 Mozilla/5.0 (HW-HUAWEI_C8500/C8500V100R001C92B234;U;Android/2.1;240*320;CTC/2.0) AppleWebKit/530.17 Mobile Safari/530.17";
const char S103565[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) ASUS-P526/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103566[] = "KWC-K24/1.0.06 UP.Browser/6.2.3.9.g.1.109 (GUI) MMP/2.0";
const char S103567[] = "KWC-K24/1.0.07 UP.Browser/6.2.3.9.g.1.109 (GUI) MMP/2.0";
const char S103568[] = "MOT-A3000_CMCC/1.0 Release/09.09.2008 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/WM6.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S103569[] = "HTC_Snap Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S103570[] = "Mozilla/4.0 SonyEricssonW595v/R3EH SEMC/NewsReader/1.0";
const char S103571[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9100 Build/GINGERBREAD; 480*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/800";
const char S103572[] = "Nokia6200/1.0 (4.22) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S103573[] = "LG/KP330/v1.0";
const char S103574[] = "SAGEM-my301X/1.0/ MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.g.2.106 (GUI) MMP/2.0";
const char S103575[] = "NokiaE71/UC Browser7.0.0.41/28/352";
const char S103576[] = "SonyEricssonW550i/R4GC Java/SEMC-Java/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103577[] = "SEC-SGHC100G/1.0 UP.Browser/5.0.5.1 (GUI)";
const char S103578[] = "SonyEricssonW395/R1BA010 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S103579[] = "SAMSUNG-SGH-i900/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.2.2.51";
const char S103580[] = "Nokia3510/1.0 (5.02)";
const char S103581[] = "Nokia8910i/1.0 (05.31) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S103582[] = "Mozilla/5.0 (Linux; U; Android WildPuzzleROM_2.1 Eclair v1.0.1; en-gb; HTC_Wildfire_A3333 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S103583[] = "Nokia8800a/2.0 (07.30) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S103584[] = "SAMSUNG-SGH-i600ORANGE/BVGD2 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S103585[] = "SHARP/1.1/SH9020C/SHC001 Mozilla/5.00 (compatible) Browser/NF/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103586[] = "SAMSUNG-GT-S5200C/1.0 RTK-E/1.0 DF/1.0 Release/02.01.2009 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103587[] = "Spice S950 NF-Browser/3.3 Openwave-MMS/4.5 IA-Java/1.0 Profile/MIDP2.0/CLDC1.1";
const char S103588[] = "Nokia5230/UCWEB7.4.0.57/50/800";
const char S103589[] = "MOT-C380/0B.D1.09R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S103590[] = "WidgetManager; SAMSUNG-GT-I6410-Vodafone;AppleWebKit/528.5+(X11;U;Linux;en;i6410BUJC1)";
const char S103591[] = "HTC_P3400i-Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S103592[] = "NokiaC7-00/UC Browser7.4.0.65/50/351";
const char S103593[] = "Xda_Zest_Opera/480x640 Opera/8.65 (Windows NT 5.1; U; en)";
const char S103594[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) HPiPAQ610/1.0 (PPC; 240x320)";
const char S103595[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) VDA-GPS/R1 Mozilla/4.0/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S103596[] = "SGH-F310/1.0 NetFront/3.3";
const char S103597[] = "LG-F2400 MIC/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S103598[] = "SoftBank/1.0/931SC/SCJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103599[] = "SIE-CL75/01 UP.Browser/6.2.3.9.d.1 (GUI) MMP/2.0";
const char S103600[] = "SIE-CL75/01 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.9.d.1 (GUI) MMP/2.0";
const char S103601[] = "Mozilla/5.0 (SCH-S239/S239DH25;U;REX/3.0;BREW/3.1.5;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*320;CTC/2.0) POLARIS/6.15.WEB";
const char S103602[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; O2 Xda Stealth; PPC; 240x320)";
const char S103603[] = "SEC-SGHD410";
const char S103604[] = "SonyEricssonT250a/R5CA005 TelecaBrowser/1.1.14.20";
const char S103605[] = "Emegeton Enzo/1.0 TMSS-Browser/1.0.0 Profile/MIDP/2.0-compatible Configuration/CLDC-1.1";
const char S103606[] = "MOT-V361/08.B7.AER MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103607[] = "gt-s5233s";
const char S103608[] = "LG/KF390Q/v1.0";
const char S103609[] = "Bleu.450x/LV,A01/WAP2.0/MIDP-2.0/CLDC-1.1 Browser/UP.Browser/7.1.0.f.1.160 (GUI)";
const char S103610[] = "SAMSUNG-SGH-A801/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S103611[] = "E71i/MITO838 Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103612[] = "LGE-LG570 AU-OBIGO/Q04C1-1.27 MMP/2.0";
const char S103613[] = "LGE-LG570V AU-OBIGO/Q04C1-1.27 MMP/2.0";
const char S103614[] = "LG-KG200/V01 Obigo/WAP2.0 LG-KG200/V01 Obigo/WAP2.0";
const char S103615[] = "SAMSUNG-SGH-E800/1.0 UP.Browser/6.2.2.6 (GUI) MMP/1.0";
const char S103616[] = "SAMSUNG-SGH-M610/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S103617[] = "NokiaN97 mini/UC Browser7.4.0.65/50/351";
const char S103618[] = "MOT-A1600_CMCC/1.0 LinuxOS/2.4.20 Release/6.16.2008 Browser/Opera8.00 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R542_G_11.60.61P";
const char S103619[] = "MOT-A1600/1.0 LinuxOS/2.4.20 Release/1.28.2008 Browser/Opera8.00 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R542_G_11.62.01R";
const char S103620[] = "SAMSUNG-SGH-i908E_CMCC/1.0 Release/03.07 Browser/IE6 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)";
const char S103621[] = "MOT-V171 UP.Browser/6.2.2.7 (GUI) MMP/1.0";
const char S103622[] = "SEC-SGHX120";
const char S103623[] = "NOKIA5320/UC Browser7.8.0.95/28/351";
const char S103624[] = "NokiaN-GageQD/1.0 SymbianOS/6.1 Series60/1.2 Profile/MIDP-1.0 Configuration/CLDC-1.0/UC Browser7.8.0.95/26/351";
const char S103625[] = "NokiaE7-00/UC Browser7.8.0.95/50/355";
const char S103626[] = "NokiaC6-01/UC Browser7.8.0.95/50/351";
const char S103627[] = "Nokia6680/1.0 (4.04.07) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/27/352";
const char S103628[] = "nokia6288/UC Browser7.8.0.95/69/401";
const char S103629[] = "Nokia5800XpressMusic/SymbianOS/9.1 Series60/3.0";
const char S103630[] = "Mozilla/5.0 (Linux; U; Android 1.6; en-ca; LG GW620R Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S103631[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 640x480; Ameo/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103632[] = "SAMSUNG-GT-E2330/1.0 RTK-E/1.0 MMP/1.0 Release/12.16.2010 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S103633[] = "Mozilla/4.0 (compatible; MSIE 5.5; Symbian OS-Series60) NetFront/3.2";
const char S103634[] = "Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS-Series60) NetFront/3.2";
const char S103635[] = "Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Nokia 3230/5.0604.0; 9399) Opera 8.65 [en]";
const char S103636[] = "Mozilla/5.0 (iPod; U; CPU iPhone OS 2_0 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko)";
const char S103637[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) SAMSUNG-SGH-i601/WM534";
const char S103638[] = "ZTE-F158/1.0 ACS-NF/3.2 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103639[] = "SAMSUNG-SGH-C426/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S103640[] = "SoftBank/1.0/810T/TJ001 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103641[] = "SoftBank/1.0/814T/TJ001 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103642[] = "SoftBank/1.0/815T/TJ001 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103643[] = "Spice S710";
const char S103644[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Alltel HTC PPC6800";
const char S103645[] = "Spice S5330";
const char S103646[] = "Spice S5420";
const char S103647[] = "LG-PLS150 AU-MIC-PLS150/1.0 MMP/2.0 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S103648[] = "Opera/9.80 (Windows NT 6.1; Opera Mobi/7457; U; en) Presto/2.5.28 Version/10.1";
const char S103649[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; ja-jp; L-06C Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S103650[] = "samsung-gt-s3653/UC Browser7.9.0.102/69/352";
const char S103651[] = "SonyEricssonW830i/R1KG Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103652[] = "SonyEricssonK850i/R2AG SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103653[] = "sonyericssonw350i/UC Browser7.7.1.88/69/444";
const char S103654[] = "SonyEricssonU5/UC Browser7.8.0.95/50/351";
const char S103655[] = "Reeder/1010.59.00 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookAir4%2C2)";
const char S103656[] = "Reeder/1010.59.00 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookPro8%2C1)";
const char S103657[] = "Reeder/1010.69.00 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (iMac11%2C1)";
const char S103658[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T959 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S103659[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S103660[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S103661[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P7510 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S103662[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7";
const char S103663[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.8.3";
const char S103664[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.8.4";
const char S103665[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.8.4";
const char S103666[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.8.3";
const char S103667[] = "MQQBrowser/Mini2.2 (SAMSUNG-GT-C3312)";
const char S103668[] = "MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103669[] = "JUC(Linux;U;2.3.3;Zh_cn;HTC Sensation Z710e;540*960;)UCWEB7.8.1.96/139/800";
const char S103670[] = "MQQBrowser/2.0 (Nokia5235;SymbianOS/9.1 Series60/3.0)";
const char S103671[] = "Nokia N96/UC Browser7.9.0.102/28/355/UCWEB";
const char S103672[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_2; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.3.4641 Chrome/7.0.517.450 Safari/534.7";
const char S103673[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.3.4641 Chrome/7.0.517.450 Safari/534.7";
const char S103674[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/1.9.4 Safari/533.1";
const char S103675[] = "Reeder/1010.59.00 CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookPro8%2C2)";
const char S103676[] = "Opera/9.80 (BlackBerry; Opera Mini/6.24209/27.1366; U; en) Presto/2.8.119 Version/11.10";
const char S103677[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]";
const char S103678[] = "Dolphin http client/8.3.0(65) (Android)";
const char S103679[] = "Instagram 2.5.0 (iPod touch; iPhone OS 5.0.1; en_US)";
const char S103680[] = "NokiaX6-00/UCWEB8.1.0.104/50/999";
const char S103681[] = "Nokia5530/UCWEB8.1.0.104/50/800";
const char S103682[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) Version/5.1 Safari/7534.51.22 Sleipnir/3.4";
const char S103683[] = "MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103684[] = "Mozilla/5.0 (X11) KHTML/4.7.4 (like Gecko) Konqueror/4.7";
const char S103685[] = "Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3 Sleipnir/1.9.2m";
const char S103686[] = "NokiaE66/UCWEB8.1.0.104/28/800";
const char S103687[] = "MQQBrowser/2.1 Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/7534.48.3";
const char S103688[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; X500 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103689[] = "gt-s5233a/UC Browser7.8.0.95/69/351";
const char S103690[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.3.8908.201";
const char S103691[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.5.0 Safari/534.34";
const char S103692[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; XV6975; Windows Phone 6.5.3.5)";
const char S103693[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; ZTE; Tania)";
const char S103694[] = "Windows Phone Search (Windows Phone OS 7.10;FujitsuToshibaMobileCommun;IS12T;7.10;8107)";
const char S103695[] = "JUC (Linux; U; 2.2; zh-cn; MotoA953; 480*854) UCWEB7.9.3.103/139/800";
const char S103696[] = "NokiaN8-00/UCWEB8.1.0.104/51/800";
const char S103697[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-US) AppleWebKit/534.12 (KHTML, like Gecko) Puffin/1.3.4232M Mobile Safari/534.12";
const char S103698[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103699[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Nexus S Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103700[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Nexus S Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103701[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, gb";
const char S103702[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.8.4/105,1.8.4.105,2012-06-26 17:19, in";
const char S103703[] = "NOKIA-Lumia 900/UCWEB2.3.0.182/47/32079";
const char S103704[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; HTC Vision Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S103705[] = "Mozilla/5.0 (Linux; U; Android Cricket Eclair 2.1-update1; en-us; SAMSUNG Intercept SPH-M910 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S103706[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; SmartQT10 Build/MID) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103707[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; th-th; A3 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103708[] = "MQQBrowser/2.9/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HW-HUAWEI_C8650/C8650V100R001C92B825; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S103709[] = "MQQBrowser/2.7/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HW-HUAWEI_C8650/C8650V100R001C92B825; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S103710[] = "Mozilla/5.0 (Linux; U; Android 3.0UI; en-us; MAXQ X7 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103711[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S103712[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; SK17i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S103713[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; XCD35 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S103714[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103715[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT26i Build/6.1.A.0.452) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S103716[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SPH-M910 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103717[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103718[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT18i Build/4.1.B.0.431) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S103719[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I727 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103720[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S103721[] = "iBrowser/2.8 Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.53) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S103722[] = "iBrowser/Mini2.7 (Nokia5130c-2/07.97)";
const char S103723[] = "Mozilla/5.0 (S60V3; U; en-us; NOKIAN78)/UC Browser8.4.0.159/28/352/UCWEB Mobile";
const char S103724[] = "MQQBrowser/Mini2.7 (NokiaX2-01/08.63)";
const char S103725[] = "Mozilla/5.0 (X11; SunOS i86pc) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.810.0 Safari/535.1";
const char S103726[] = "NokiaC2-06/2.0 (07.48) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac2-06) UCBrowser8.2.1.144/70/355/UCWEB Mobile";
const char S103727[] = "Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Ubuntu/9.10 Chromium/8.0.558.0 Chrome/8.0.558.0 Safari/534.10";
const char S103728[] = "NOKIA-Lumia 900/UCWEB2.4.0.188/47/32079";
const char S103729[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I5500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103730[] = "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.5.1 (development) Safari/534.34";
const char S103731[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-s310) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S103732[] = "Nokia5220XpressMusic/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5220xpressmusic) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S103733[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonw150i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S103734[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokian72) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S103735[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-c105) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S103736[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8600) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S103737[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B)Maxthon AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S103738[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6800 Build/HTJ85B)Maxthon AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S103739[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-ph; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Mobile Maxthon Mobile Maxthon";
const char S103740[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S103741[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S103742[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-gb; GT-P7500 Build/ERD62) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103743[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 MobileSafari/534.13";
const char S103744[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.8.4/105,1.8.4.105,2012-06-26 17:19, gb";
const char S103745[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, nl";
const char S103746[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.8.4/102,1.8.4.102,2012-06-23 14:36, china, au";
const char S103747[] = "MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S103748[] = "MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S103749[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S103750[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S103751[] = "YahooMobileMail/1.0 (Android Mail; 1.4.5) (espresso10wifi;samsung;GT-P5110;4.0.4/IMM76D)";
const char S103752[] = "Instagram 3.2.0 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P3113; espressowifi; espresso; en_US)";
const char S103753[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S103754[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P3110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.0.181/145/352";
const char S103755[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S103756[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S103757[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S103758[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [UsableNet Lift Mobile]";
const char S103759[] = "Instagram 3.1.0 Android (13/3.2; 160dpi; 800x1232; samsung; GT-P6800; GT-P6800; smdkc210; en_GB)";
const char S103760[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10";
const char S103761[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10";
const char S103762[] = "Instagram 3.2.0 Android (16/4.1.1; 160dpi; 800x1232; samsung; GT-P5113; espresso10wifi; espresso10; en_US)";
const char S103763[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]";
const char S103764[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]";
const char S103765[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7300 Build/HMJ37; 800*1280) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/352";
const char S103766[] = "Instagram 3.2.0 Android (15/4.0.3; 160dpi; 600x976; samsung; GT-P3110; espressowifi; espresso; en_GB)";
const char S103767[] = "JUC(Linux;U;Android4.0.4;Zh_cn;GT-P7310;800*1232;)UCWEB7.8.0.95/139/352";
const char S103768[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P6210 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355";
const char S103769[] = "WhatsApp/2.9.369 Android/4.0.4 Device/samsung-GT-P6800";
const char S103770[] = "WhatsApp/2.9.369 Android/3.2 Device/samsung-GT-P7300";
const char S103771[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S103772[] = "MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S103773[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S103774[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S103775[] = "MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S103776[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S103777[] = "Mozilla/5.0(Linux; U; Android 4.0.4; zh-CN; GT-P3110 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.0.212 U3/0.8.0 Mobile Safari/534.31";
const char S103778[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; GT-P3110 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S103779[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S103780[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P5100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S103781[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P7501 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S103782[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7501 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S103783[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-P1000 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0000, ie";
const char S103784[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; GT-P1000 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0100, fr";
const char S103785[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259";
const char S103786[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P6810 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.87 Safari/537.22";
const char S103787[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P6810 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22";
const char S103788[] = "Mozilla/5.0 (Linux; U; Android 3.1; es-es; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.3.3000";
const char S103789[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13 Maxthon/4.0.3.3000";
const char S103790[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-P5100 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S103791[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7511 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22";
const char S103792[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-P1010 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S103793[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 OverDrive Media Console/2.6.4";
const char S103794[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P6210 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S103795[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6210 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S103796[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-lg800g) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S103797[] = "UCWEB/2.0 (Java; U; MIDP-2.0; ru; samsung-gt-s5250) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S103798[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-a350) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S103799[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.19/284,1.9.19.284,2013-01-17 18:10, +0800, china, cn";
const char S103800[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262D Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S103801[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-lg900g) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S103802[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-b5310) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S103803[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5600) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S103804[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233w) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S103805[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233w) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile";
const char S103806[] = "UCWEB/2.0 (Java; U; MIDP-2.0; vi; gt-s5233w) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S103807[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s7070) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S103808[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gt350) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S103809[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-US780 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S103810[] = "iBrowser/Mini2.7 (NokiaX2-00/08.25)";
const char S103811[] = "UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; opera) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S103812[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; 11a h700tecno) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S103813[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5350) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S103814[] = "Mozilla/5.0 (DTV) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.5 AQUOSBrowser/1.0 (AS00DTV;V;0001;0001) AQUOS-AS/2.0 LC-46LE840X";
const char S103815[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-S7275 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36";
const char S103816[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gu220) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S103817[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5560) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S103818[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LM7600-TA; 03.10.07; 0xX;); LG NetCast.TV-2012";
const char S103819[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL838; 55.4.02.0; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S103820[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL838; 48.4.72.8; t12; ) ; ToshibaTP/1.1.0 () ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S103821[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL838; 55.4.02.0; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM) ; pl) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S103822[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL838; 73.0.51.8; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S103823[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL838; 85.3.88.6; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S103824[] = "Mozilla/5.0 (Linux; U; Android 4.4.5; en-us; C5170 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103825[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_1_6 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 [FBAN/FBIOS;FBAV/5.5;FBBV/X;FBDV/iPhone3,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/7.1.6;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBOP/0]";
const char S103826[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 [FBAN/FBIOS;FBAV/6.2;FBBV/X;FBDV/iPhone5,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/6.1.4;FBSS/2; FBCR/VinaPhone;FBID/phone;FBLC/en_US;FBOP/1]";
const char S103827[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11D43 [FBAN/FBIOS;FBAV/6.4;FBBV/X;FBDV/iPhone6,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/7.1;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBOP/1]";
const char S103828[] = "Mozilla/5.0 (Linux; Android 4.3; GT-I9505G Build/JWR66V.S014) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1612.0 Mobile Safari/537.36";
const char S103829[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-s365) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S103830[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-S5302 Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S103831[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; SAMSUNG-GT-C6712) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S103832[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S103833[] = "SAMSUNG-SCH-M490/(null)IC4 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S103834[] = "Mozilla/4.0(MTS/1.0/MTC547/VF_RU_RU_P107A4V1.2.1/Obigo Q03C/MIDP-2.0 Configuration/CLDC-1.1)";
const char S103835[] = "LG-CT810/V10x IEMobile/7.11 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S103836[] = "Mozilla/5.0 (Linux; U; Android SleekSense-Rc2-2.2; en-us; USCCADR6275US Carrier ID 45 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103837[] = "PPC; 240x320; HTC_P3450/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S103838[] = "(Opera) Vodafone/1.0/HPiPAQDataMessenger/1.00.00 Browser/Opera/9.5 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/15142; U; en)";
const char S103839[] = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ja-jp; 001HT Build/FRF91) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17";
const char S103840[] = "MOT-V547/08.18.40R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S103841[] = "Nokia6788i/CMCC SymbianOS/9.3 Series60/3.2 Release/19.013.314.1; Mozilla/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML,like Gecko) Safari/413";
const char S103842[] = "LG-KP233/V10a Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S103843[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 LGKT615/v10A; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S103844[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 LGKT615/v10C; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S103845[] = "VIDEOCON_V1616/WAP2.0 Profile/MT6253/QVGA";
const char S103846[] = "SAMSUNG-SGH-D508/1.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzU2MDk1MDA1MjY5NTE5 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S103847[] = "SAMSUNG-SGH-U808E/1.0 RTK-E/1.0 DF/1.0 Release/04.23.2008 Browser/NetFront3.4  Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103848[] = "LG-G262 MIC/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S103849[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTCA3360_TD/1.0 Android/2.2 release/10.1.2010 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103850[] = "LG-C1150 MIC/WAP2.0 MIDP-2.0/CLDC-1.0";
const char S103851[] = "LG10000/1.0 TELECA/Q05A1-3.8 MMP/2.0";
const char S103852[] = "SonyEricssonT290i/R101";
const char S103853[] = "Alcatel-OT-S853/1.0 UP.Browser/7.1 (GUI) MMP/2.0";
const char S103854[] = "LG-B2150 MIC/WAP2.0 MIDP-2.0/CLDC-1.0";
const char S103855[] = "SANYO-S750/2.130 UP.Browser/7.0.2.1.sn.1.222 (GUI) MMP/2.0";
const char S103856[] = "SANYO-S750/1.810 UP.Browser/7.0.2.1.sn.1.159 (GUI) MMP/2.0";
const char S103857[] = "SAMSUNG-SGH-E236/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S103858[] = "SEC-SGHX506/1.0 TSS/2.5";
const char S103859[] = "SAMSUNG-SGH-X300/1.0";
const char S103860[] = "HTC MAX 4G Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S103861[] = "SEC-SGHZX20 SHP/CDR-2.2 NetFront/3.2 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S103862[] = "SAMSUNG-GT-S8000T/1.0 SHP/R5 NetFront/3.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S103863[] = "Nokia6650/2.0 (05.40) Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S103864[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) LG-KS20/V10a Browser/IEMobile/7.6 MMS/LG-MMS-WINCE-V1.0/1.2 Java/LGVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103865[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/LG-KS20/V10b Browser/IEMobile/7.6 MMS/LG-MMS-WINCE-V1.0/1.2 Java/LGVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103866[] = "HTC_Polaris Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S103867[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) 1.0/HTC_Polaris";
const char S103868[] = "Opera 9.5(Windows NT;  U; en) SAMSUNG-GT-B7300/1.0";
const char S103869[] = "SEC-SGHX507, TSS/2.5, Rev 1.4";
const char S103870[] = "SEC-SGHX507/1.0 TSS/2.5";
const char S103871[] = "Mozilla/5.0 (HUAWEI/S6510V100R001C1X;U;REX/1.0;Opera Mini/4.2;BREW/3.1.5;Profile/MIDP-2.0 Configuration/CLDC-1.1;320*240;CTC/2.0) NetFront/3.5";
const char S103872[] = "Mozilla/5.0 (Linux; U; Android 1.5; pt-br; LG GW620F Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S103873[] = "MOT-V555/08.18.40R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S103874[] = "LG-KP200 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S103875[] = "SAMSUNG-M3510C/1.0 RTK-E/1.0 DF/1.0 Release/09.01.2008 Browser/NetFront3.4  Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103876[] = "SAMSUNG-E1210M/E1210MRYIC2 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S103877[] = "NokiaC2-00/2.0 (03.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-00/UC Browser7.8.0.95/70/401";
const char S103878[] = "SonyEricssonK818c/R6BC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103879[] = "SoftBank/1.0/910SH/SHJ002 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103880[] = "Mozilla/4.08 (910SH;SoftBank) NetFront/3.3";
const char S103881[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; HTC-P5310 BM ) Opera 8.65 [en]";
const char S103882[] = "F230/1.0 ACS-NF/3.2 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103883[] = "MOT-MOTOROKR-U9/Mozilla/4.0 (compatible;MSIE 6.0;Linux MOTOROKR U9)/R6632_G_81.xx.yyI Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony 1.0";
const char S103884[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ME861 Build/OLYKC_U4_1.15.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103885[] = "LGE-VX9800/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S103886[] = "SAMSUNG-SGH-C416 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S103887[] = "SAMSUNG-GT-S5263/S5263DDKE1";
const char S103888[] = "samsung-gt-s5263";
const char S103889[] = "SAMSUNG-SGH-C161/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S103890[] = "NokiaC3-03/5.0 (p) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+";
const char S103891[] = "BlackBerry8600/5.0.0.564 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/1";
const char S103892[] = "Mozilla/5.0 (SCH-S569/S569DF09;U;REX/3.0;BREW/3.1.5;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*320;CTC/2.0) POLARIS/6.15.WEB";
const char S103893[] = "LG8600/1.1 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S103894[] = "LG8600/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S103895[] = "HUAWEI-M318/001.00 ACS-NetFront/3.2";
const char S103896[] = "SEC-SGHI321N/1.0 jBrowser/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103897[] = "Lenovo-S710/S100 LMP/LML Release/2010.05.22 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S103898[] = "SonyEricssonX5/R1DA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.0";
const char S103899[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) ASUSM930/1.0";
const char S103900[] = "samsungsph-m540";
const char S103901[] = "SonyEricssonJ20i/R7BA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2";
const char S103902[] = "SPH-M800/1.0 AU-MIC/2.0 (GUI) MMP/2.0";
const char S103903[] = "SonyEricssonK600i";
const char S103904[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) O2 Xda Atom Life";
const char S103905[] = "Mozilla/5.0 (SCH-W709/1.0;U;REX/3.0;BREW/3.1.5;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*400;CTC/2.0) POLARIS/6.15.WEB";
const char S103906[] = "WinWAP-X/3.0 (Neonode N2; Smartphone; WCE; 176x220)";
const char S103907[] = "SAMSUNG-SGH-i728_CMCC_U1/1.0 Release/03.07 Browser/IE6 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S103908[] = "KDDI-AM01 UP.Browser/ KJ/ Mozilla/4.0 (REX; U; en-us; Sanyo; SCP-6750/US; NetFront/3.4/AMB)";
const char S103909[] = "LG-KP230-V10a Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S103910[] = "ZTE-F260/1.0 ACS-NF/3.2 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103911[] = "SEC-SGHX800/1.0 NetFront/3.2";
const char S103912[] = "LGE-VX5200/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S103913[] = "SonyEricssonJ210i/R2L/TelecaBrowser/4.08";
const char S103914[] = "SIE-AF51/12 Profile/MIDP-1.0 Configuration/CLDC-1.0 UP.Browser/6.1.0.7.3.102 (GUI) MMP/1.0";
const char S103915[] = "MOT-V1050/83.38.49I MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103916[] = "NokiaE71/UCWEB7.4.0.57/28/800";
const char S103917[] = "Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Motorola M1000;817c) Opera 7.50 [en]./M1000./1.0";
const char S103918[] = "Mozilla/5.0 (Linux; U; Android FroydVillain 1.6 (2.2.1); en-gb; HTC Hero Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103919[] = "Nokia6670/2.0 (6.0522.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.4.0.65/27/351";
const char S103920[] = "NokiaE72-1/UCWEB7.6.0.75/28/800";
const char S103921[] = "NokiaX6-00/UCWEB7.6.0.75/50/800";
const char S103922[] = "Nokia5530/UCWEB7.6.0.75/50/800";
const char S103923[] = "Nokia6600/1.0 (5.53.0) SymbianOS/7.0s Series60/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.6.1.82/27/351";
const char S103924[] = "SEC-SGHX430";
const char S103925[] = "NokiaE63/UCWEB7.8.0.87/28/999";
const char S103926[] = "NokiaE71/UCWEB7.8.0.87/28/999";
const char S103927[] = "sonyericssonk750i/UC Browser7.7.1.88/69/444";
const char S103928[] = "Nokia6600/1.0 (5.53.0) SymbianOS/7.0s Series60/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.8.0.95/27/351";
const char S103929[] = "NOKIAE61i/UC Browser7.8.0.95/28/355";
const char S103930[] = "HTC_S310-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 176x220)";
const char S103931[] = "LG-ME850d MIC/1.1.14 MIDP-2.0/CLDC-1.1";
const char S103932[] = "MOT-ME860/1.0 Mozilla/5.0 Android/2.2.2 Release/6.30.2011 Browser/AppleWebKit/533.1";
const char S103933[] = "MQQBrowser/25 Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103934[] = "Alcatel-OT-C630/1.0 ObigoInternetBrowser/Q03C";
const char S103935[] = "LG-G1800 MIC/WAP2.0 MIDP-2.0/CLDC-1.0";
const char S103936[] = "Panasonic-X800/1.0 SymbianOS/7.0 Series60/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S103937[] = "Panasonic-X701/1.0 SymbianOS/7.0 Series60/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S103938[] = "DoCoMo/2.0 P700i(c100;TB;W30H15)";
const char S103939[] = "portalmmm/2.0 Z320i(c100;TB)";
const char S103940[] = "portalmmm/2.0 S401i(c10;TB)";
const char S103941[] = "Mozilla/4.0 (compatible; MSIE 5.0; Symbian OS; SonyEricsson P900; 306) Opera 6.30 [en]";
const char S103942[] = "SAMSUNG-SGH-I617/1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S103943[] = "Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Nokia E61/0633.09.04; 9730) Opera 8.65 [en]";
const char S103944[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) ASUS1210/MR2_V3.5.0A";
const char S103945[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; MDA III/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1)";
const char S103946[] = "SoftBank/1.0/913SH/SHJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103947[] = "LG-LX260 EMBIDER-LX260/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103948[] = "SoftBank/1.0/821P/PJP10 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103949[] = "SoftBank/1.0/822SH/SHJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S103950[] = "Mozilla/5.0 (Linux; U; Android 1.5; ja-jp; HT-03A Build/CDB72) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S103951[] = "SAMSUNG-SGH-M130L/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S103952[] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) Speedy Spider (http://www.entireweb.com/about/search_tech/speedy_spider/)";
const char S103953[] = "SonyEricssonU5/UC Browser7.9.0.102/50/354/UCWEB";
const char S103954[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1 QQBrowser/6.9.11153.201";
const char S103955[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SPH-D710 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103956[] = "JUC(Linux;U;Android2.3.4;Zh_cn;ST17i;320*480;)UCWEB7.8.0.95/139/352";
const char S103957[] = "JUC(Linux;U;Android2.2.1;Zh_cn;Ideos;240*320;)UCWEB7.8.0.95/139/352";
const char S103958[] = "YahooMobileMail/1.0 (Android Mail; 1.3.8) (lgl55c;LGE;LGL55C;2.3.4/GRJ22)";
const char S103959[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; cs-cz; PMP3084BA Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103960[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.0.3.0 Chrome/17.0.963.38 Safari/535.11";
const char S103961[] = "Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 ME Build/1.6.0_229-MZ606) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1";
const char S103962[] = "Nokia5233/UCWEB7.4.0.57/50/800";
const char S103963[] = "Dolphin HD 4.1 (iPad; iPhone OS 5.1; en_US)";
const char S103964[] = "Dolphin HD 4.1 (iPad; iPhone OS 5.1.1; en_US)";
const char S103965[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; HTC One X Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S103966[] = "Nokia N96/UC Browser7.6.1.82/28/351";
const char S103967[] = "Nokia5250/UC Browser7.6.1.82/50/352";
const char S103968[] = "Mac OS X/10.7.1 (11B2118); ExchangeWebServices/2.0 (123); Mail/5.0 (1244.2)";
const char S103969[] = "Mac OS X/10.7.1 (11B26); ExchangeWebServices/2.0 (123); Mail/5.0 (1244.2)";
const char S103970[] = "Flipboard/1.8.1 CFNetwork/485.12.7 Darwin/10.4.0";
const char S103971[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Xperia Play Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103972[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xperia Play Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S103973[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]";
const char S103974[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 5_1 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]";
const char S103975[] = "Mozilla/5.0 (PlayStation Vita 1.52) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2";
const char S103976[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6400L Build/GRJ22)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103977[] = "Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 5.1; zh-cn) Opera 8.65";
const char S103978[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/19.0.1084.52)";
const char S103979[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; chromeframe/19.0.1084.52)";
const char S103980[] = "Facebook 4110.0 (iPhone; iPhone OS 4.1; en_US)";
const char S103981[] = "Opera/9.26 (Windows; U; pl)";
const char S103982[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; FujitsuToshibaMobileCommun; IS12T; KDDI)";
const char S103983[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Schubert)";
const char S103984[] = "Opera/9.80 (Windows Mobile; WCE; Opera Mobi/7457; U; en) Presto/2.5.28 Version/10.1";
const char S103985[] = "RealPlayer%20Downloader/1569 CFNetwork/454.11.12 Darwin/10.7.0 (i386) (MacBookPro7%2C1)";
const char S103986[] = "NOKIANokia N81/UC Browser7.7.1.88/28/444";
const char S103987[] = "BlackBerry9530/4.2.0.128 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/161/352";
const char S103988[] = "NokiaC5-00/UCWEB8.1.0.104/28/800";
const char S103989[] = "NokiaC5-00/UCWEB8.2.0.116/28/800";
const char S103990[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Kindle Fire Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.8.4/111,1.8.4.111,2012-07-09 19:03, us";
const char S103991[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P690 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S103992[] = "MQQBrowser/2.6 (Nokia5230;SymbianOS/9.1 Series60/3.0)";
const char S103993[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC One X Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S103994[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; es-es; bq Verne Plus Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S103995[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; HTC_P515E Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103996[] = "MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103997[] = "Mozilla/5.0 (Linux; U; Android 2.3; en-us; MAXQ X7 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S103998[] = "MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HW-HUAWEI_C8650/C8650V100R001C92B825; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1";
const char S103999[] = "MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT531 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104000[] = "JUC(Linux;U;Android1.6;Zh_cn;Android Dev Phone 1;320*480;)UCWEB7.8.0.95/139/444";
const char S104001[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104002[] = "JUC(Linux;U;Android2.3.4;Zh_cn;HTC Sensation XE with Beats Audio Z715e;540*960;)UCWEB7.8.0.95/139/352";
const char S104003[] = "YahooMobileMessenger/1.0 (Android Mail; 1.4.2) (ace; HTC; Desire HD; 2.3.3/GRI40)";
const char S104004[] = "iBrowser/2.8 Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104005[] = "MQQBrowser/3.3/Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-U V880 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104006[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; Spice Mi-425 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S104007[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S104008[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NookTablet Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S104009[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; LT22i Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S104010[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; huaqin73_gb Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S104011[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; MT27i Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S104012[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S104013[] = "MQQBrowser/33 Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3";
const char S104014[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Nexus 7 Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (Nexus 7, 34, 1.6.0)";
const char S104015[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (Nexus 7, 34, 1.6.0)";
const char S104016[] = "Rdio.Net; Android/2.2.8; Android/4.1.1; DM=asus; DN=Nexus 7; DC=google; DF=google/nakasi/grouper:4.1.1/JRO03D/X:user/release-keys; DH=grouper";
const char S104017[] = "Rdio.Net; Android/2.2.10; Android/4.1.2; DM=asus; DN=Nexus 7; DC=google; DF=google/nakasi/grouper:4.1.2/JZO54K/X:user/release-keys; DH=grouper";
const char S104018[] = "Mozilla/5.0 (Linux; U; Android 4.1; en-us; Nexus 7 Build/JRN84D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, us";
const char S104019[] = "Opera/9.80 (X11; FreeBSD 9.0-RELEASE i386; U; en) Presto/2.10.229 Version/11.61";
const char S104020[] = "MQQBrowser/2.8 (E71;SymbianOS/9.1 Series60/3.0)";
const char S104021[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-gx300) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S104022[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9105 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S104023[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9105 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S104024[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5230w) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S104025[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s5753e) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S104026[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-a350) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S104027[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000";
const char S104028[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;T9295;7.10;8862)";
const char S104029[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LAVA iris 351 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.2.2.259";
const char S104030[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; LAVA iris 351 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S104031[] = "JUC(Linux;U;Android2.2.1;Zh_cn;GT-P1010;600*1024;)UCWEB7.8.0.95/139/355";
const char S104032[] = "MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104033[] = "MQQBrowser/2.7/Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104034[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104035[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104036[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104037[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7503 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S104038[] = "Android/1.0 (GT-P6810 IMM76D)";
const char S104039[] = "YahooMobile/1.0 (mail; 1.3.9.10087); (Linux; U; Android 3.2; GT-P6210 Build/HONEYCOMB_MR2);";
const char S104040[] = "YahooMobileMail/1.0 (Android Mail; 1.4.5) (GT-P7310;samsung;GT-P7310;3.2/HTJ85B)";
const char S104041[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5100 Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104042[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P5100 Build/IMM76D)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104043[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S104044[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S104045[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104046[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104047[] = "MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S104048[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S104049[] = "YahooMobileMail/1.0 (Android Mail; 1.4.5) (GT-P6201;samsung;GT-P6201;3.2/HTJ85B)";
const char S104050[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 3.2; hu-hu; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S104051[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7300 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/405";
const char S104052[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7300B Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S104053[] = "Instagram 3.2.0 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P6210; GT-P6210; smdk4210; en_US)";
const char S104054[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P1000 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S104055[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P1000 Build/IMM76L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S104056[] = "Android/3.2; samsung GT-P7310; 2.3.1";
const char S104057[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104058[] = "Instagram 3.1.0 Android (10/2.3.3; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; gt-p1000; en_GB)";
const char S104059[] = "Instagram 3.1.0 Android (8/2.2; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; GT-P1000; en_GB)";
const char S104060[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P3113 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S104061[] = "TwitterAndroid/1.0.5 (109) GT-P6200/13 (samsung;GT-P6200)";
const char S104062[] = "TwitterAndroid/2.0.1 (122) GT-P1000N/8 (samsung;GT-P1000N;samsung;GT-P1000N;)";
const char S104063[] = "Instagram 3.1.0 Android (12/3.1; 160dpi; 1280x800; samsung; GT-P7510; GT-P7510; p3; en_US)";
const char S104064[] = "GoogleEarth/7.0.0001.8239(Android;Android (GT-P3113-espressowifi-user-4.0.4);en-US;kml:2.2;client:Free;type:default)";
const char S104065[] = "TwitterAndroid/2.0.1 (122) GT-P6200/13 (samsung;GT-P6200;samsung;GT-P6200;)";
const char S104066[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7310 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7310, 36, 1.7.1)";
const char S104067[] = "OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-za; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104068[] = "TwitterAndroid/2.0.1 (122) GT-P5100/15 (samsung;espresso10rf;samsung;espresso10rfxx;)";
const char S104069[] = "Android OS/3.2 samsung_GT-P6210";
const char S104070[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S104071[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S104072[] = "Instagram 3.0.5 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7510; GT-P7510; p3; en_US)";
const char S104073[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104074[] = "Instagram 1.0.5 Android (13/3.2; 160dpi; 800x1232; samsung; GT-P6800; GT-P6800; smdkc210; en_GB)";
const char S104075[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S104076[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S104077[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7310 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7310, 38, 1.8.1)";
const char S104078[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0100, si";
const char S104079[] = "Rdio.Net; Android/2.4.2; Android/4.0.4; DM=samsung; DN=GT-P7510; DC=samsung; DF=samsung/GT-P7510/GT-P7510:4.0.4/IMM76D/UELPL:user/release-keys; DH=p3";
const char S104080[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; GT-P7310 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7310, 38, 1.8.1)";
const char S104081[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7310 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7310, 40, 1.8.5)";
const char S104082[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104083[] = "Instagram 3.3.2 Android (15/4.0.3; 160dpi; 600x976; samsung; GT-P3110; espressowifi; espresso; en_US)";
const char S104084[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]";
const char S104085[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]";
const char S104086[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P5100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S104087[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P3113 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S104088[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 UCBrowser/2.2.2.259";
const char S104089[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 UCBrowser/2.2.2.259";
const char S104090[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104091[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 MobileSafari/534.13";
const char S104092[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S104093[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P6210 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +0000, gb";
const char S104094[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S104095[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S104096[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7300 Build/HTJ85B; 320*493) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444";
const char S104097[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P5113 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S104098[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P5100, 39, 1.8.2)";
const char S104099[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-P3110 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31";
const char S104100[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P6800, 39, 1.8.2)";
const char S104101[] = "Evernote Android/AndroidMarket_45x_243.14540 (en_CA); Android/4.1.1; GT-P5113/16;";
const char S104102[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259";
const char S104103[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-P7501 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.3.3000";
const char S104104[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259";
const char S104105[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6800 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S104106[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-P6800 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S104107[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S104108[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P7510 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S104109[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P7500 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31";
const char S104110[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P7310 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S104111[] = "theScore/3.2.1 Android-OS/4.1.1 (Android; Tablet, GT-P5113)";
const char S104112[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-P5110 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31";
const char S104113[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-P5110 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S104114[] = "theScore/3.2.0 Android-OS/2.2 (Android; Phone, GT-P1000R)";
const char S104115[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315";
const char S104116[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s7233e) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S104117[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-p520) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S104118[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-US780 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S104119[] = "SAMSUNG-GT-I8350/UCWEB2.3.0.182/47/32079";
const char S104120[] = "UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; sonyericssonu5i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S104121[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s8600) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S104122[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-sgh-u900) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S104123[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8500) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S104124[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-a230) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S104125[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-c300) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S104126[] = "UCWEB/2.0 (Java; U; MIDP-2.0; Pt-BR; lg-c300) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S104127[] = "Nokia3500c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3500c) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S104128[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mot-ex115) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S104129[] = "Mozilla/5.0 (DTV) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.5 AQUOSBrowser/1.0 (AS00DTV;V;0001;0001) AQUOS-AS/2.0 LC-60LE840X";
const char S104130[] = "Opera/9.80 (Linux armv6l; U; en) Presto/2.8.115 Version/11.10 AQUOS-AS/1.0 LC-70LE735X";
const char S104131[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly s Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S104132[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-S7270 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140";
const char S104133[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-S7275 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S104134[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-c310) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S104135[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 55LM6200-UE; 04.02.85; 0xX;); LG NetCast.TV-2012 0";
const char S104136[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 55LM7600-TA; 04.10.08; 0xX;); LG NetCast.TV-2012";
const char S104137[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+SCREEN+TUNER; LGE; GLOBAL-PLAT4; 03.17.00; 0xX;); LG NetCast.TV-2012";
const char S104138[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; DLOBAL-PLAT4; 04.34.05; 0xX;); LG NetCast.TV-2012 0";
const char S104139[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8600) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S104140[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-ks360 teleca) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S104141[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5350) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S104142[] = "Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11D4 Safari/8536.25";
const char S104143[] = "Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11D8a Safari/8536.25";
const char S104144[] = "Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.49.1 (KHTML, like Gecko) Version/6.0 Mobile/11D17 Safari/8536.25";
const char S104145[] = "Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11D10 Safari/8536.25";
const char S104146[] = "Mozilla/5.0 (iPhone; CPU OS 7_1 like Mac OS X) AppleWebKit/537.80 (KHTML, like Gecko) Version/7.1 Mobile/11A4388 Safari/8536.25";
const char S104147[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-S7270 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S104148[] = "Mozilla/5.0 (Linux; Android 4.0.4; TECNO_N7 Build/TECNOTECNO_N7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S104149[] = "UCWEB/2.0(Java; U; MIDP-2.0; vi; samsung-gt-b3410w) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S104150[] = "Nokia6300/2.0 (05.00) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6300/UC Browser7.4.0.65/69/352";
const char S104151[] = "SAMSUNG-SPH-M7350/(null)ID4 (compatible; MSIE 6.0; Windows CE; PPC) Opera 9.5";
const char S104152[] = "Nokia7210Supernova/2.0 (06.51) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7210supernova/UC Browser7.4.0.65/70/351";
const char S104153[] = "SAMSUNG-SGH-F338/1.0 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1/*MzU5MDQyMDEwMzkwNjcx";
const char S104154[] = "SPICE-S600/WAP Browser, SPICE-S600/WAP Browser";
const char S104155[] = "SPICE-S600/WAP Browser";
const char S104156[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2690/UC Browser7.5.1.77/70/351";
const char S104157[] = "SAMSUNG-SGH-I907/UCJD1 Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S104158[] = "Mozilla/5.0 (Linux; U; Android SenseHero 2.1; en-us; A6277 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 Sprint APA6277KT";
const char S104159[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) 320x240; VZW; UTStar-XV6175.1; Windows Mobile 6.5 Standard;";
const char S104160[] = "SAMSUNG-GT-C3510/C3510JAIL1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S104161[] = "Haier-1200P/1.0 iPanel/2.0 WAP2.0 (compatible; UP.Browser/6.2.2.4; UPG1; UP/4.0; Embedded)";
const char S104162[] = "SAMSUNG-GT-S3100/S3100XXIH4 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S104163[] = "LENOVO-P709/S100 NXP5210/DV7 Release/4.25.2007 Obigo/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S104164[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_TOUCH_DIAMOND2/2.0; Windows Phone 6.5)";
const char S104165[] = "TCL-e300/w05.44.08/WAP-2.0/MIDP-2.0 Configuration/CLDC-1.0";
const char S104166[] = "SAMSUNG-SGH-Z720M-Vodafone/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S104167[] = "MOT-V195/0A.63.12R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104168[] = "SonyEricssonW950i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104169[] = "HTC_Touch_HD2_T8585 Opera/10.00 (Windows NT 5.1; U; en)";
const char S104170[] = "VertuAscentTi-C/2.0 (501.02) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S104171[] = "VertuAscentTi-C/2.0 (501.01) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S104172[] = "SEC-SGHP207/a/, TSS/2.5, Rev1.1h";
const char S104173[] = "SEC-SGHP207, TSS/2.5, Rev1.1e";
const char S104174[] = "Mozilla/5.0 (Linux; U; Android 1.6; zh-cn; dopodA6388_TD/1.0 Android/Dount release/2.26.2010 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/DRD20) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S104175[] = "SoftBank/1.0/820P/PJP10 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104176[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) HPiPAQ610/1.0 (PPC; 240x320)";
const char S104177[] = "HPiPAQ610/1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC)";
const char S104178[] = "SHARP/1.1/SH1810C/SHC001 Mozilla/5.00 (compatible) Browser/NF/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104179[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 240x320; HTC_P3600/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104180[] = "Lenovo-E218/CMCC_S101 LMP/LML Release/2009.05.20";
const char S104181[] = "SAMSUNG-SGH-X608/1.0*MzUyNjIzMDAwMDA1MDEw UP.Browser/6.1.0.6 (GUI) MMP/1.0";
const char S104182[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P929 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S104183[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P929 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S104184[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) HTC-P5310 BM";
const char S104185[] = "Fly-E300/Profile/MIDP.2.0Configuration/CLDC.1.0, Fly-E300/Profile/MIDP.2.0Configuration/CLDC.1.0";
const char S104186[] = "Alcatel-Mandarina-Duck/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S104187[] = "YuLong-Coolpad2938/1.0 iPanel/2.0 WAP2.0";
const char S104188[] = "Motorola-K1m UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0";
const char S104189[] = "LGE-RD9100/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S104190[] = "LG-C3300 MIC/WAP2.0 MIDP-2.0/CLDC-1.0";
const char S104191[] = "Alcatel-OT-C707/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S104192[] = "Xda_Venn-Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S104193[] = "SIE-CX75/13 UP.Browser/7.0.2.2.d.7(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104194[] = "SIE-CX75/25 UP.Browser/7.0.2.2.d.7(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104195[] = "SIE-CX75/10 UP.Browser/7.0.2.2.d.5(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104196[] = "LGE-LG385 AU-OBIGO/Q04C1-1.27 MMP/2.0";
const char S104197[] = "ZTE-F153/1.0 ACS-NF/3.2 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104198[] = "SAMSUNG-SGH-L288/1.0 SHP/VPP1.0 Release/12.15.2006 Browser/R5.NetFront3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1 SMM-MMS/1.2.0";
const char S104199[] = "SAMSUNG-SGH-E480/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S104200[] = "LGE-LG380/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S104201[] = "LG380/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S104202[] = "NexianNX-G933/MTK Release/01.01.2010 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1";
const char S104203[] = "SoftBank/1.0/705P/PJP10 Browser/Teleca-Browser/3.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104204[] = "SoftBank/1.0/708SC/SCJ001 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104205[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) ASUSP550/1.0";
const char S104206[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) ASUS-P835/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104207[] = "dopod T3238_CMCC/5.2.20257/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S104208[] = "SAMSUNG-SGH-L878E/1.0 SymbianOS/9.3 Series60/3.2 Release/6.25.2008 Browser/OSS3.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S104209[] = "SonyEricssonZ710i/R1JC SEMC/NewsReader/1.0";
const char S104210[] = "SEC-SGHC500/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S104211[] = "LG-MG220 UP.Browser/6.2.3 (GUI) MMP/1.0";
const char S104212[] = "Alcatel-Crystal/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S104213[] = "SoftBank/1.0/001P/PJP10 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104214[] = "LG-KE590 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S104215[] = "SonyEricssonZ610i/R1JG SEMC/NewsReader/1.0";
const char S104216[] = "SAMSUNG-SCH-M715/(null)ID4 (compatible; MSIE 6.0; Windows CE; PPC) Opera 9.5";
const char S104217[] = "HTC_P3650 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) O2MTouchC Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6;HTC_P5500; tmn PortalBliss 1.0)";
const char S104218[] = "SAMSUNG-GT-S3653/S3653JVIH2 SHP/VPP/R5 Jasmine/0.8 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S104219[] = "Mozilla/5.0 (Linux; U; Android Special; en-gb; Desire_A8181 Build/Jaguaralani Gamer's Edition) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104220[] = "Mozilla/5.0 (Linux; U; Android PDAMobiz 2.2 II; en-us; Desire_A8181 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104221[] = "SCH-R610 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S104222[] = "SoftBank/1.0/911SH/SHJ002 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104223[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MB525 Build/3.4.2-117; 480*854) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.1.100/139/800";
const char S104224[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB525 Build/3.4.2-177; 480*854) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.1.100/139/800";
const char S104225[] = "SAMSUNG-GT-B6520/1.0 Mozilla/4.0 Release/8.11.2010 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104226[] = "PPC; 240x320; HTC_TyTN/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Dopod838Pro";
const char S104227[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) DopodM700";
const char S104228[] = "DopodM700/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)";
const char S104229[] = "samsung-gt-s5333";
const char S104230[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T759 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104231[] = "SoftBank/1.0/911T/TJ002 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104232[] = "WellcomW3110/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104233[] = "Alcatel-OT-216/1.0 ObigoInternetBrowser/Q03C";
const char S104234[] = "SAMSUNG-SGH-L811/L811JVHL1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S104235[] = "LGE/U8210/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104236[] = "SoftBank/1.0/805SC/SCJ001 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104237[] = "Alcatel-OT-C700A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S104238[] = "Nokia7610/2.0 (7.0642.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.0.0.41/27/352";
const char S104239[] = "Nokia6788/CMCC SymbianOS/9.3 Series60/3.2 Release/11.018.314.1; Mozilla/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML,like Gecko) Safari/413";
const char S104240[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) LG-MS20 IEMobile/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S104241[] = "LG-KF305 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S104242[] = "dopod P660_CMCC/5.2.1941/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S104243[] = "SonyEricssonZ555i/R10CA Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104244[] = "HTCT9188_TD/1.0 WindowsMobile/6.5 CEOS/5.2 release/5.3 Opera/9.7 WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S104245[] = "ZTE-F188(Smart Sandbox)/1.0 ACS-NF/3.3 QTV2.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104246[] = "SAMSUNG-SGH-L400-Vodafone/L400BUGJ2 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S104247[] = "LG-KU310/v10d Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S104248[] = "AKAI4411/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104249[] = "SAMSUNG-SCH-i220a (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.2.2.51";
const char S104250[] = "Nokia6255/1.0 (N190V0700.nep) UP.Browser/6.2.3.2.h.1.104 (GUI) MMP/2.0";
const char S104251[] = "SAMSUNG-GT-B5310";
const char S104252[] = "MOT-A-A6/00.02 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0 Push/PO";
const char S104253[] = "Nokia6310/1.0 (05.01)";
const char S104254[] = "Lenovo-TD30t/S100 LMP/LML Release/2009.10.15 Profile/MIDP2.1 Configuration/CLDC1.1";
const char S104255[] = "SAMSUNG-GT-B7702_TD/1.0 RTK-E/1.0 SHP/VPP-R5 Release/03.18.2010 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1 SMM-MMS/1.2.0";
const char S104256[] = "SAMSUNG-SGH-L168-CMCC/Polaris 5.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104257[] = "Panasonic-MX7/1.0/RP1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104258[] = "Panasonic-X400P/RP1 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S104259[] = "Panasonic-SA7/1.0/RP1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104260[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; O2 Graphite; Smartphone; 240x320)";
const char S104261[] = "SAMSUNG-SGH-A811/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S104262[] = "MOT-A1890/GE00.08.02P INFRAWARE/5.01 CTC/1.0";
const char S104263[] = "SAMSUNG-SGH-T609/T609UVFC8 Profile/MIDP-2.0 Configuration/CLDC-1.1 TSS/2.5";
const char S104264[] = "Alcatel-MissSixty/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S104265[] = "SAMSUNG-GT-E3210/E3210DDKB6 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0";
const char S104266[] = "SAMSUNG-SGH-P220-ORANGE/P220BVGK1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S104267[] = "SAMSUNG-SGH-T336/ Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S104268[] = "SAMSUNG-C5110-ORANGE/BVIF4 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S104269[] = "Mozilla/4.0 (compatible;MSIE 6.0;Linux MOTOZINE ZN5)MOTOZINE ZN5/R6637_G_81.03.0AR Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony 1.0";
const char S104270[] = "SAMSUNG-GT-C5530_TD/1.0 RTK-E/1.0 SHP/VPP-R5 Release/03.18.2010 Browser/Dolfin1.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 Nextreaming SMM-MMS/1.2.0";
const char S104271[] = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.5.1.77/70/351";
const char S104272[] = "NokiaE72-1/UC Browser7.0.0.41/28/352";
const char S104273[] = "NokiaC5-03/UC Browser7.4.0.65/50/355";
const char S104274[] = "NokiaN73/UCWEB7.6.0.75/28/401";
const char S104275[] = "NokiaX6-00/SymbianOS/9.1 Series60/3.0";
const char S104276[] = "SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1 sonyericssonj105i/UC Browser7.6.1.82/70/351";
const char S104277[] = "SEC-SGHC230/1.0";
const char S104278[] = "NokiaE72-1/UCWEB7.8.0.87/28/800";
const char S104279[] = "NOKIA6210 Navigator/UC Browser7.7.1.88/28/352";
const char S104280[] = "Nokia5530/UCWEB7.8.0.87/50/999";
const char S104281[] = "Nokia6220c-1/UC Browser7.8.0.95/28/352";
const char S104282[] = "sonyericssonk800i/UC Browser7.8.0.95/70/352";
const char S104283[] = "LG-MU500 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104284[] = "LG-CP150/V10n Teleca/WAP2.0";
const char S104285[] = "Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303iclassic/UC Browser7.8.0.95/70/351";
const char S104286[] = "Palm Treo Pro/ ver 1.01/ Windows CE;IE Mobile 7.11 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S104287[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; MB525 Build/JRDNEM_U3_2.21.0; 480*854) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/800";
const char S104288[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104289[] = "BlackBerry8300/4.5.0.182 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102 UP.Browser/5.0.3.3";
const char S104290[] = "BlackBerry5810/3.6.0";
const char S104291[] = "SAMSUNG-SGH-C406/1.0";
const char S104292[] = "SAMSUNG-SGH-X486/1.0";
const char S104293[] = "Alcatel-BG3/1.0 UP.Browser/5.0.3.1.2";
const char S104294[] = "SonyEricssonS700c/R3B SEMC-Browser/4.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104295[] = "SonyEricssonT123i/R1DA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S104296[] = "SonyEricssonV600i";
const char S104297[] = "SonyEricssonZ200/R101";
const char S104298[] = "LG-C2000 UP.Browser/6.2.3 (GUI) MMP/1.0";
const char S104299[] = "LG-KE970";
const char S104300[] = "LG-S5000 MIC/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S104301[] = "SHARP-TQ-GX10i/1.1 Profile/MIDP-1.0 Configuration/CLDC-1.0 UP.Browser/6.1.0.6.1.d.2 (GUI) MMP/1.0";
const char S104302[] = "PG-C150/R01 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104303[] = "Vodafone/1.0/SamsungSGHE570V/E570BUGA3 Browser/NF/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104304[] = "Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Xda cosmo; 320x240)";
const char S104305[] = "Xda_nova; 240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S104306[] = "KDDI-CA37 UP.Browser/6.2.0.12.1.3 (GUI) MMP/2.0";
const char S104307[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC_P3600i/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104308[] = "dopod S900_CMCC/5.2.1941/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S104309[] = "LG-KG120 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S104310[] = "BIRD.F501/1.00 Nucleus RTOS/V1.11.19 MTK6227/05C_V24 Release/11.21.2006 Browser/Teleca Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S104311[] = "Spice S650/KAA736_0.90.626 Nucleus/1.0 MTK/6223 Release/06.26.2007 Browser/Teleca_obigo Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S104312[] = "SoftBank/1.0/816SH/SHJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104313[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Vodafone/1.0/HTC_VPACompactIV/4.14.162.2";
const char S104314[] = "LENOVO-V800 LENOVO-V800";
const char S104315[] = "Mozilla/4.1 (compatible; MSIE 5.0; Symbian OS; Nokia 3650;452) Opera 6.20  [en]";
const char S104316[] = "dopod T2222_CMCC/5.2.19578/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S104317[] = "LG-KG810 UP.Browser/6.2.3 (GUI) MMP/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104318[] = "WELLCOM_W819 WAP Browser";
const char S104319[] = "PANTECH-EUROPA-U4000-orange/1.0 Obigo/Q04C MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S104320[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; A100 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S104321[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; HTC/Shooter_U/1.01.161.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104322[] = "Flipboard/1.5.1 CFNetwork/485.12.7 Darwin/10.4.0";
const char S104323[] = "Chrome/0";
const char S104324[] = "Mozilla/5.0 (SaferSurf) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.2.149.30 Safari/525.13";
const char S104325[] = "sonyericssonw810i/UC Browser7.0.0.41/69/352";
const char S104326[] = "sonyericssonk810i/UC Browser7.8.0.95/70/351";
const char S104327[] = "Opera/9.80 (Windows NT 6.0; Opera Mobi/23731; U; en) Presto/2.9.201 Version/11.50";
const char S104328[] = "Opera/9.80 (Windows NT 6.0; Opera Mobi/14316; U; en) Presto/2.7.81 Version/11.00";
const char S104329[] = "BlackBerry9860/5.0.0.496 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/161/350";
const char S104330[] = "BlackBerry9000/5.2.0.93 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/159/351";
const char S104331[] = "Mozilla/5.0 (Linux; U; Android 2.3.9; zh-tw; HTC T9199 For Vodafone Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104332[] = "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/ Sleipnir/1.8.3m";
const char S104333[] = "samsung-sgh-a877";
const char S104334[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/2.2.0 Safari/533.1";
const char S104335[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/2.2.0 Safari/533.1";
const char S104336[] = "NokiaE63/UCWEB7.3.1.56/28/999";
const char S104337[] = "NokiaN97/UCWEB7.6.0.75/50/999";
const char S104338[] = "JUC(Linux;U;2.2;Zh_cn;HTC Legend;320*480;)UCWEB7.6.0.75/139/800";
const char S104339[] = "NokiaE66/UCWEB7.8.0.87/28/800";
const char S104340[] = "lg-lg800g";
const char S104341[] = "NOKIA6110/UC Browser7.9.0.102/28/355/UCWEB";
const char S104342[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; Xperia Arc Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S104343[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xperia Arc Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S104344[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Xperia Arc Build/MIUIv4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S104345[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xperia Arc Build/MIUIv4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S104346[] = "Mozilla/5.0 (DirectFB; U; Linux 35230; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.1.4(+mouse+3D+SCREEN+TUNER; LGE; 42LW5700-TA; 04.03.10; 0xX;); LG NetCast.TV-2011";
const char S104347[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]";
const char S104348[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]";
const char S104349[] = "Reeder/1.4 CFNetwork/485.12.7 Darwin/10.4.0";
const char S104350[] = "DoCoMo/2.0 P900i(c100;TB;W24H11)(compatible; ichiro/mobile goo;+http://help.goo.ne.jp/door/crawler.html)";
const char S104351[] = "MQQBrowser/Mini2.5 (Nokia200/10.58)";
const char S104352[] = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1";
const char S104353[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/2.1.0 Safari/533.1";
const char S104354[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MOTWX435KT Build/FROYO)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104355[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC One S Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S104356[] = "NOKIA Lumia 900/2175.1601.8773.12141";
const char S104357[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, in";
const char S104358[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Ultimate)";
const char S104359[] = "NOKIANokia N81/UC Browser7.8.0.95/28/355";
const char S104360[] = "MQQBrowser/2.0 (NokiaE65;SymbianOS/9.1 Series60/3.0)";
const char S104361[] = "NOKIAE5-00/UCWEB8.1.0.104/28/800";
const char S104362[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1 QQBrowser/6.10.11509.201";
const char S104363[] = "Mozilla/5.0 (SaferSurf) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.21 Safari/536.11";
const char S104364[] = "Mozilla/5.0 (Java; U; en-us; sonyericssonk770i) UCBrowser8.3.0.154/69/352/UCWEB Mobile";
const char S104365[] = "Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; AT100 Build/HTK55D)Maxthon AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S104366[] = "MQQBrowser/2.4 Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/7534.48.3";
const char S104367[] = "MQQBrowser/Mini2.6 (NokiaX2-01/08.63)";
const char S104368[] = "Mozilla/5.0 (SaferSurf) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1165.0 Safari/537.1";
const char S104369[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) Version/5.1 Safari/7534.51.22 Sleipnir/3.6";
const char S104370[] = "Mozilla/5.0 (PLAYSTATION 3 4.23) AppleWebKit/531.22.8 (KHTML, like Gecko)";
const char S104371[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC_Incredible_S Build/GRI40; 480*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/800";
const char S104372[] = "Mozilla/5.0 (Android; Linux armv7l; rv:8.0) Gecko/ Firefox/8.0 Fennec/8.0";
const char S104373[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB860 Build/MIUI-##VERSION##-OLYMPUS-EN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104374[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; MT810 Build/ECLAIR) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530";
const char S104375[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; nl-nl; TAB411 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104376[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; XT530 Build/V4.530) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S104377[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.53) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S104378[] = "JUC(Linux;U;Android2.3.5;Zh_cn;HTC Sensation XL with Beats Audio X315e;480*800;)UCWEB7.8.0.95/139/351";
const char S104379[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P970 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S104380[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; LT26i Build/6.0.A.3.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S104381[] = "MQQBrowser/2.9/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104382[] = "MQQBrowser/3.3/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Incredible S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104383[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P970 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S104384[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5660 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S104385[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I777 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104386[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/2.1.0 Safari/533.1";
const char S104387[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; ZTE-BLADE Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S104388[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S104389[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, us";
const char S104390[] = "NokiaN97 mini/UCWEB8.3.0.133/50/999";
const char S104391[] = "GoogleEarth/7.0.0001.8177(Android;Android (Nexus 7-grouper-user-4.1.2);ja-JP;kml:2.2;client:Free;type:default)";
const char S104392[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Nexus 7 Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104393[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; Nexus 7 Build/JRN84D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104394[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)/Nutch-1.0";
const char S104395[] = "NokiaE66/UCWEB7.6.0.75/28/999";
const char S104396[] = "MQQBrowser/Mini2.7 (NokiaC1-01/04.40)";
const char S104397[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; 6532.9) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5";
const char S104398[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-gu220) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S104399[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s8600) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S104400[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; sgh-f480) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S104401[] = "OneBrowser/3.0 (Nokia205/03.18)";
const char S104402[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s7250d) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S104403[] = "OneBrowser/3.0 (SAMSUNG-GT-C3312R)";
const char S104404[] = "NokiaC3-01/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac3-01) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S104405[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-m5650) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S104406[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5302 Build/12.0.A.1.222) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S104407[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8552 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S104408[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;T7575;7.10;8862)";
const char S104409[] = "Windows Phone Search (Windows Phone OS 7.10;LG;LG-C900B;7.10;8862)";
const char S104410[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-I9200 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S104411[] = "Mozilla/5.0 (Linux; Android 4.0.9; GT-I9200 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S104412[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7100 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S104413[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2; zh-tw; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104414[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104415[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104416[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S104417[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104418[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104419[] = "MQQBrowser/3.3/Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104420[] = "MQQBrowser/3.3/Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104421[] = "MQQBrowser/3.3/Mozilla/5.0 (Linux; U; Android 2.2; en-au; GT-P1000T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104422[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [operamini] [UsableNet Lift Mobile]";
const char S104423[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37; 800*1280) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.4.145/139/800";
const char S104424[] = "Android OS/13 samsung_GT-P6800";
const char S104425[] = "Mozilla/1.0 (Linux; Java; android; Android SDK 15; samsung-GT-P6800; Mobile Safari;BOCHK 4.3.4) NetUtil/2.0 HTTPCore/1.0 SS/800x1232";
const char S104426[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104427[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.3; ar-ae; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104428[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104429[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 4.0.3; ar-ae; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104430[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104431[] = "Pandora/1.7.1 Android/4.0.4 GT-P6210";
const char S104432[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6200 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S104433[] = "Mozilla/5.0 (Linux; U; Android 3.0; en-gb; GT-P3113 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104434[] = "MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S104435[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S104436[] = "WhatsApp/2.7.8402 Android/4.0.3 Device/samsung-GT-P3100";
const char S104437[] = "YahooMobileMessenger/1.0 (Android Mail; 1.4.6) (espressowifi; samsung; GT-P3110; 4.0.4/IMM76D)";
const char S104438[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-P1010 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S104439[] = "WhatsApp/2.7.4705 Android/2.3.3 Device/samsung-GT-P1000";
const char S104440[] = "TwitterAndroid/1.0.5 (109) GT-P6800/15 (samsung;GT-P6800)";
const char S104441[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.7/210,1.9.7.210,2012-10-25 19:53, it";
const char S104442[] = "YahooMobileMessenger/1.0 (Android Mail; 1.4.6) (GT-P6210; samsung; GT-P6210; 4.0.4/IMM76D)";
const char S104443[] = "Instagram 3.2.0 Android (13/3.2; 160dpi; 800x1232; samsung; GT-P7510; GT-P7510; p3; en_US)";
const char S104444[] = "Android OS/4.0.3 samsung_GT-P5110";
const char S104445[] = "GoogleEarth/7.0.0000.7386(Android;Android (GT-P7510-GT-P7510-user-3.2);nl-NL;kml:2.2;client:Free;type:default)";
const char S104446[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104447[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104448[] = "Instagram 1.0.6 Android (15/4.0.3; 160dpi; 800x1232; samsung; GT-P5100; espresso10rf; espresso10; en_GB)";
const char S104449[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6200 Build/IMM76D; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444";
const char S104450[] = "Android OS/12 samsung_GT-P7300";
const char S104451[] = "MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6200 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S104452[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6200 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S104453[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-P1010 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S104454[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6201 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.22/288,1.9.22.288,2013-01-31 13:09, +0000, gb";
const char S104455[] = "Instagram 1.1.8 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7510; GT-P7510; p3; en_US)";
const char S104456[] = "YahooMobile/1.0 (mail; 1.4.5.12736); (Linux; U; Android 4.0.4; GT-P6800 Build/Unknown);";
const char S104457[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7510 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S104458[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104459[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104460[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-au; GT-P7320T Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.16/276,1.9.16.276,2012-12-21 16:37, +1100, au";
const char S104461[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7320 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +0800, sg";
const char S104462[] = "Instagram 3.4.0 Android (8/2.2; 240dpi; 600x1024; samsung; SGH-T849; SGH-T849; GT-P1000; en_US)";
const char S104463[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/999";
const char S104464[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3110 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.16/276,1.9.16.276,2012-12-21 16:37, +0000, gb";
const char S104465[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; GT-P7500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0800, tw";
const char S104466[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-sg; GT-P7500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.20/286,1.9.20.286,2013-01-25 13:38, +0800, sg";
const char S104467[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7300 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.16/276,1.9.16.276,2012-12-21 16:37, +0000, gh";
const char S104468[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3113 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3113, 38, 1.8.1)";
const char S104469[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3113 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (GT-P3113, 40, 1.8.5)";
const char S104470[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7300, 39, 1.8.2)";
const char S104471[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P7300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S104472[] = "iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 2.2; fa-fa; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104473[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]";
const char S104474[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]";
const char S104475[] = "GoogleEarth/7.0.0002.8421(Android;Android (GT-P5100-espresso10rf-user-4.0.3);it-IT;kml:2.2;client:Free;type:default)";
const char S104476[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6210 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P6210, 39, 1.8.2)";
const char S104477[] = "Mozilla/5.0(Linux; U; Android 3.2; zh-CN; GT-P6210 Build/HTJ85B) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.0.212 U3/0.8.0 Mobile Safari/534.31";
const char S104478[] = "Android/1.0 (GT-P6201 HTJ85B)";
const char S104479[] = "GoogleEarth/7.0.0001.8177(Android;Android (GT-P5113-espresso10wifi-user-4.1.1);en-CA;kml:2.2;client:Free;type:default)";
const char S104480[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P3108 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.1.258 U3/0.8.0 Mobile Safari/534.31";
const char S104481[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S104482[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104483[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7100, 39, 1.8.2)";
const char S104484[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7300, 39, 1.8.2)";
const char S104485[] = "Evernote Android/AndroidMarket_45x_243.14540 (en_CA); Android/4.1.1; GT-P3113/16;";
const char S104486[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S104487[] = "TuneIn Radio/5.2 (Android 8; GT-P1000M; Java)";
const char S104488[] = "TuneIn Radio/5.2 (Android 8; GT-P1000; Java)";
const char S104489[] = "GoogleEarth/7.0.0003.8527(Android;Android (GT-P5110-espresso10wifi-user-4.1.1);en-GB;kml:2.2;client:Free;type:default)";
const char S104490[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P6200, 39, 1.8.2)";
const char S104491[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259";
const char S104492[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P1000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.2.2.259";
const char S104493[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S104494[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P3110 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S104495[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S104496[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S104497[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6810 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 MobileSafari/534.13 Maxthon/4.0.3.3000";
const char S104498[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P1000 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S104499[] = "Mozilla/5.0 (Linux; U; Android 3.2; ru-ru; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [operamini] [UsableNet Lift Mobile]";
const char S104500[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P7511 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0100, de";
const char S104501[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3110 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3110, 40, 1.8.5)";
const char S104502[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3110 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3110, 40, 1.8.5)";
const char S104503[] = "theScore/3.1.0 Android-OS/3.2 (Android; Phone, GT-P6200)";
const char S104504[] = "Mozilla/5.0 (Linux; U; Android 3.2; ru-ru; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.3.3000";
const char S104505[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P5110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S104506[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P5110 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S104507[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [operamini] [UsableNet Lift Mobile]";
const char S104508[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; GT-P3100 Build/JZO54K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7";
const char S104509[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; da-dk; GT-P6800 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0700, dk";
const char S104510[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-P6800 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.27/381,1.9.27.381,2013-03-21 11:39, +0000, gb";
const char S104511[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7501 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22";
const char S104512[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7310 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13 UCBrowser/2.2.2.259";
const char S104513[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S104514[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-CN; GT-P7300 Build/HTJ85B) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31";
const char S104515[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 OverDrive Media Console/2.6.4";
const char S104516[] = "theScore/3.2.1 Android-OS/4.0.4 (Android; Phone, GT-P3113)";
const char S104517[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259";
const char S104518[] = "Mozilla/5.0 (Linux; Android 4.0.3; GT-P5110 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.53463";
const char S104519[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259";
const char S104520[] = "MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S104521[] = "MQQBrowser/3.6/Adr (Linux; U; 4.1.2; zh-cn; GT-I8262D Build/JZO54K.I8262DZCAMA9;480*800)";
const char S104522[] = "MQQBrowser/3.6/Adr (Linux; U; 4.1.2; zh-cn; GT-I8262D Build/JZO54K.I8262DZCAMB2;480*800)";
const char S104523[] = "Mozilla/5.0 (Linux; Android 4.1.2; C5502 Build/10.1.1.A.1.226) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S104524[] = "Mozilla/5.0 (Linux; Android 4.1.2; OUYA Console Build/JZO54K-OUYA) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31";
const char S104525[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lemon t99) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S104526[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5260) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S104527[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-sgh-a877) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S104528[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9295 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.1/549,2.0.1.549,2013-05-15 23:10, +0200, pl";
const char S104529[] = "Mozilla/5.0 (Java; U; en-us; samsung-gt-s8500) UCBrowser8.2.1.144/70/352/UCWEB Mobile";
const char S104530[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; s5230) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S104531[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s7230e) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S104532[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6680) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S104533[] = "Opera/9.80 (Linux armv6l; U; en) Presto/2.8.115 Version/11.10 AQUOS-AS/1.0 LC-40LE835M";
const char S104534[] = "Opera/9.80 (Linux armv6l; U; en) Presto/2.8.115 Version/11.10 AQUOS-AS/1.0 LC-70LE735M";
const char S104535[] = "Opera/9.80 (Linux armv6l; U; en) Presto/2.8.115 Version/11.10 AQUOS-AS/1.0 LC-60LE835X";
const char S104536[] = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:26.0) Gecko/X Firefox/26.0 slurp";
const char S104537[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5250) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S104538[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8500) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S104539[] = "Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LA740S-ZB; 03.20.04; 0xX;); LG NetCast.TV-2013 /03.20.04 (LG, 47LA740S-ZB, wireless)";
const char S104540[] = "Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; GLOBAL-PLAT4; 03.20.00; 0xX;); LG NetCast.TV-2013 /03.20.00 (LG, GLOBAL-PLAT4, wireless)";
const char S104541[] = "Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LA6210-TD; 03.10.61; 0xX;); LG NetCast.TV-2013 /03.10.61 (LG, 47LA6210-TD, wireless)";
const char S104542[] = "Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LA8600-NA; 03.20.23; 0xX;); LG NetCast.TV-2013 /03.20.23 (LG, 47LA8600-NA, wireless)";
const char S104543[] = "Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LA6200-TA; 03.20.09; 0xX;); LG NetCast.TV-2013 /03.20.09 (LG, 42LA6200-TA, wireless)";
const char S104544[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 65LM6200-TA; 04.00.28; 0xX;); LG NetCast.TV-2012";
const char S104545[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 55LM6400-UA; 04.12.23; 0xX;); LG NetCast.TV-2012";
const char S104546[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LM660S-ZA; 04.10.23; 0xX;); LG NetCast.TV-2012";
const char S104547[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+SCREEN+TUNER; LGE; 42LS5700-TB; 03.09.38; 0xX;); LG NetCast.TV-2012";
const char S104548[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LM6700-DA; 04.10.11; 0xX;); LG NetCast.TV-2012";
const char S104549[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LM6700-TA; 04.10.10; 0xX;); LG NetCast.TV-2012";
const char S104550[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_WL968; 7.0.27.3; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S104551[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_WL968; 7.0.27.8; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; pl) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S104552[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_WL968; 7.0.27.13; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S104553[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6120c) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S104554[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-78) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1612.0 Mobile Safari/537.36";
const char S104555[] = "Mozilla/5.0 (Linux; Android 4.3; Nexus 10 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1612.0 Safari/537.36";
const char S104556[] = "Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.55 Mobile Safari/537.36 OPR/16.0.1212.62222";
const char S104557[] = "Mozilla/5.0 (Linux; Android 4.2; Nexus 4 Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63417";
const char S104558[] = "Mozilla/5.0 (Linux; Android 4.0.4; Q-Smart S20 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S104559[] = "Mozilla/5.0 (Linux; Android 4.1.2; B15 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S104560[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-S5570 Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S104561[] = "Mozilla/5.0 (Linux; Android 2.3.5; YP-G70 Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S104562[] = "Mozilla/5.0 (Linux; Android 4.0.4; IM-A760S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S104563[] = "Opera/9.80 (Series 60; Opera Mini/17.3.2013/28.3692; U; en) Presto/2.8.119 Version/11.10";
const char S104564[] = "HTC_P5520 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S104565[] = "HTC-8500/1.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S104566[] = "SAMSUNG-SGH-i718plus_CMCC/1.0 Release/03.07 Browser/IE6 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S104567[] = "Mozilla/5.0 (Linux; U; Android ChocolateEclair 1.0.1; en-gb; HTC Hero Build/CE1.0.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S104568[] = "BIRD_D730/V1.0.0/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 BIRD_D730/V1.0.0/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S104569[] = "BIRD_D730/V1.0.0/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S104570[] = "SAMSUNG-SGH-P858/1.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzU3ODk0MDAwMDgyNzMw UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S104571[] = "PPC; 240x320; HTC_WIZARD/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S104572[] = "SAMSUNG-SGH-D838/NetFront 3.2/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzUyNjY2MDEwMTk0NTU1";
const char S104573[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) O2 Xda Zinc";
const char S104574[] = "Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2730c-1/UC Browser7.2.2.51/70/352";
const char S104575[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8; DopodU1000) Opera 8.65 [en]";
const char S104576[] = "MOT-MOTOVE66_CMCC/1.0 LinuxOS/2.6.10 Release/11.28.2008 Browser/OSS1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R6743_G_74.31.00RPL";
const char S104577[] = "Nokia2700c-2/2.0 (07.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2700c-2/UC Browser7.2.2.51/70/352";
const char S104578[] = "BenQ-E53/1.0 ACS-NF/3.3 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104579[] = "Fly_DS185/WAP2.0Configuration/Browser";
const char S104580[] = "HTC_S620 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S104581[] = "SAMSUNG-SGH-F308/NetFront 3.2/WAP2.0/*MzU0MjExMDEwMjkxMzM5";
const char S104582[] = "SonyEricssonK858c/R1DA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S104583[] = "SAMSUNG-SGH-D808/NetFront 3.2/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzU3ODEyMDA3MjI5MjEx";
const char S104584[] = "BlackBerry9650x/5.0.0.549 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/1";
const char S104585[] = "MOT-MOTOROKRE8_CMCC/1.0 LinuxOS/2.6.10 Release/11.16.2007 Browser/OSS1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R6718_G_71.10.13R_C";
const char S104586[] = "SAMSUNG-SGH-Z368/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1/*MzU0MDQ1MDEwMDk0NjY5";
const char S104587[] = "SAMSUNG-SGH-U708E/1.0 RTK-E/1.0 DF/1.0 Release/02.05.2008 Browser/NetFront3.4  Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104588[] = "LG8550/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S104589[] = "SonyEricssonT818/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3";
const char S104590[] = "SAMSUNG-SGH-P240/P240XAHB4 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S104591[] = "NOKIA-7208/1.0 UP.Browser/6.3.0.7.g.1.100-20Mar09 (GUI) MMP/2.0";
const char S104592[] = "Vodafone/Sharp902SH/SHG001/1.0 Browser/UP.Browser/7.0.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Ext-J-Profile/JSCL-1.2.2 Ext-V-Profile/VSCL-2.0.0";
const char S104593[] = "Vodafone/Sharp902SH/SHG002/1.0 Browser/UP.Browser/7.0.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Ext-J-Profile/JSCL-1.2.2 Ext-V-Profile/VSCL-2.0.0";
const char S104594[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.9) PPC; 240x320; i-mate K-JAM PPC/1.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104595[] = "HTC_S630 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S104596[] = "SHARP-TQ-GX27/1.0 Profile/MIDP-1.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.4.163 (GUI) MMP/1.0";
const char S104597[] = "SAMSUNG-SGH-E378/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S104598[] = "Panasonic-SC3/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.1.0.7.8 (GUI) MMP/1.0";
const char S104599[] = "MOT-VU30/00.62 UP.Browser/6.2.3.4.c.1.124 (GUI) MMP/2.0";
const char S104600[] = "SonyEricssonMODEL/R1JG Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104601[] = "SAMSUNG-SGH-E848/1.0 Release/2.21.2007 Browser/NetFront3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzU2NTQzMDE3ODM4Njgw";
const char S104602[] = "Panasonic-VS7/1.0/RP1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104603[] = "MOT-E680/R51_G_0F.38.A4R MIB/2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104604[] = "Alcatel-OT-280/1.0 model-orange ObigoInternetBrowser/Q03C";
const char S104605[] = "Alcatel-OT-280/1.0 ObigoInternetBrowser/Q03C";
const char S104606[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) LG-MS25 IEMobile/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S104607[] = "NexianNX-G988/MTK Release/10.01.2009 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1";
const char S104608[] = "Huawei/1.0/T566/B000 Browser/obigo-browser/q04a";
const char S104609[] = "SCH-M569/ UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S104610[] = "LGE/U8150/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S104611[] = "Motorola-ic502 Obigo/Q04C1-1.9 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104612[] = "Alcatel-OT-C550/1.0 ObigoInternetBrowser/Q03C";
const char S104613[] = "SAMSUNG-GT-S6888/1.0 SHP/VPP/R5 Dolfin/2.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S104614[] = "LG-B2100 MIC/WAP2.0 MIDP-2.0/CLDC-1.0";
const char S104615[] = "Mozilla/4.0 SonyEricssonC902v/R3EE SEMC/NewsReader/1.0";
const char S104616[] = "ZTE F860/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104617[] = "PICASSO-ZF908/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104618[] = "NokiaC3-00/5.0 (07.20) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac3-00/UC Browser7.5.1.77/70/401";
const char S104619[] = "FarEasTone/1.1/SharpWXT930/SHS001 Mozilla/5.00 (compatible) Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104620[] = "ASUS1210/V4.2.4 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 240x320; ASUS1210) Opera 8.65";
const char S104621[] = "dopod T8288_CMCC/5.2.19965/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S104622[] = "WellcoM_W930";
const char S104623[] = "LG-M4410 MIC/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S104624[] = "LG-KX500/1.0 UP.Browser/6.2.3.8 CTC/1.0";
const char S104625[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320; MDA Vario/1.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104626[] = "SAMSUNG-SGH-F118/1.0 RTK-E/1.0 DF/1.0 Release/06.22.2007 Browser/NetFront3.4  Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104627[] = "LG-U8290 MIC/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S104628[] = "LG-ME970c MIC/1.1.14 MIDP-2.0/CLDC-1.1";
const char S104629[] = "Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5310xpressmusic/UC Browser7.8.0.95/70/351";
const char S104630[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) O2 Xda Atom";
const char S104631[] = "SonyEricssonK700/R7DA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2";
const char S104632[] = "SAMSUNG-SGH-E768/1.0 RTK-E/1.0 DF/1.0 Release/11.27.2006 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzU1Njc1MDEwMDg4NDI1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S104633[] = "Debussy/WAP1.2 Profile/MIDP2.0 Configuration/CLDC1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 176x220; HPiPAQ510/1.0)";
const char S104634[] = "SAMSUNG-SGH-i908E_CMCC/1.0 Release/03.07 Browser/IE6 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104635[] = "HTC_Snap Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S104636[] = "SAMSUNG-GT-S7520U/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S104637[] = "SAMSUNG-GT-B7300C_CMCC/1.0 Opera 9.5";
const char S104638[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; SCH-I905 4G Build/HONEYCOMB) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S104639[] = "BlackBerry7130e/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105";
const char S104640[] = "SonyEricssonP908/R101 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S104641[] = "SoftBank/1.0/840P/PJP10 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104642[] = "SAMSUNG-GT-I8180C_CMCC/1.0 WindowsCE/5.0 WindowsMobile/6.5 Release/09.04.2009 Browser/IE6 Profile/MIDP-2.1 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S104643[] = "SCH-U540/1.0 AU-MIC/2.0 (GUI) MMP/2.0";
const char S104644[] = "HTC_P3701/1.37.621.2 Mozilla/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S104645[] = "HTC_P3702 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S104646[] = "SUN F188/1.0 ACS-NF/3.3 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104647[] = "Nokia6165i/2.0 (AZ100V1800.nep) UP.Browser/6.2.3.8 MMP/2.0";
const char S104648[] = "Mozilla/5.0 (SCH-W289/W289EC01;U;REX/3.0;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*320;CTC/2.0) POLARIS/6.2.WEB";
const char S104649[] = "Alcatel-Mandarina-Duck-MOON/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S104650[] = "Sanyo-SCP7000CA NetFront/3.1 MMP/2.0 Mozilla/4.0";
const char S104651[] = "Fly-B400/Profile/MIDP.2.0Configuration/CLDC.1.0";
const char S104652[] = "SAMSUNG-SGH-E635/E635UVEF3 UP.Browser/6.2.2.6 (GUI) MMP/1.0";
const char S104653[] = "SAMSUNG-SGH-A177/A177UCIC3 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S104654[] = "WELLCOM-W981_WAP_Browser";
const char S104655[] = "SonyEricssonW900i/R5BC Java/SEMC-Java/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104656[] = "Mozilla/5.0 (Series40; NokiaX2-00/04.21; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.3.23.12";
const char S104657[] = "SoftBank/1.0/709SC/SCJ001 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104658[] = "MOT-GATW2/00.62 UP.Browser/6.2.3.4.c.1.114 (GUI) MMP/2.0";
const char S104659[] = "Fly_DS100/WAP2.0Configuration/Browser";
const char S104660[] = "ASUS-J208/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ASUS-J208/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104661[] = "HTC_S730 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S104662[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Sprint Treo850e";
const char S104663[] = "FLY-SL600/BSI AU.Browser/2.0 QO3C1 MMP/1.0";
const char S104664[] = "SAMSUNG-GT-I8510H/1.0 SymbianOS/9.3 Series60/3.2 Release/5.30.2008 Browser/OSS3.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S104665[] = "SAMSUNG-SGH-E700/BSI Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S104666[] = "SHARP SH7110C / Obigo Browser 2.0";
const char S104667[] = "LG-C2600 MIC/WAP2.0 MIDP-2.0/CLDC-1.0 LG-C2600 MIC/WAP2.0 MIDP-2.0/CLDC-1.0";
const char S104668[] = "Nokia6235/1.0 (S190V0200.nep) UP.Browser/6.2.3.2 MMP/2.0";
const char S104669[] = "Nokia2608/2.0 UP.Browser/6.3.0.7.g.1.100-27Oct08 (GUI) MMP/2.0";
const char S104670[] = "SAMSUNG-SGH-D988/1.0 RTK-E/1.0 DF/1.0 Release/08.17.2007 Browser/Openwave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzU0NDAxMDIxNTQ2Njg2 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S104671[] = "SonyEricssonZ558i/R4FA Java/SEMC-Java/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104672[] = "NOKIA5320/UC Browser7.4.0.65/28/352";
const char S104673[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) DopodU1000";
const char S104674[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; vi-vn; HTC_Sensation_S710e Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104675[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC) Opera 8.65 [en] Toshiba G910_G920";
const char S104676[] = "Mozilla/5.0 (SCH-W319/W319DG29;U;REX/3.0;BREW/3.1.5;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*320;CTC/2.0) POLARIS/6.2.WEB";
const char S104677[] = "SAMSUNG-S8300C/1.0 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S104678[] = "Alcatel-OT-S218/1.0 ObigoInternetBrowser/Q03C";
const char S104679[] = "Alcatel-OT-383A/1.0 ObigoInternetBrowser/Q03C";
const char S104680[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 9900; en) AppleWebKit/534.11+ (KHTML, like Gecko) Version/6.1.0.73 Mobile Safari/534.11+";
const char S104681[] = "MOT-V635/08.48.24R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S104682[] = "LG-U8500-V10 MIC/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S104683[] = "MOT-VE20/1.0 UP.Browser/7.2.6.1.731 (GUI) MMP/2.0";
const char S104684[] = "SamsungSGH-ZV30/1.0 SHP/VPP/R5 NetFront/3.2 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S104685[] = "HTC-2125/1.2 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 176x220)";
const char S104686[] = "Nokia2855/2.0 (VR100V0600.nep) UP.Browser/6.2.3.8 MMP/2.0";
const char S104687[] = "SEC-SGHP710/1.0 TSS/2.5";
const char S104688[] = "SoftBank/1.0/002P/PJP10 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104689[] = "Karbonn-K400/0.1 NXP/1.0 Release/08.10.2009 Browser/mBrowser4.0 Profile/MIDP-1.0 Configration/CLDC-1.0";
const char S104690[] = "SharpWXT91/SHS001/1.0 Browser/UP.Browser/7.0.2.1a.f.1.101 (GUI) Profile/MIDP-2.0 Configuration/CLDC-1.1 Ext-J-Profile/JSCL-1.2.2 Ext-V-Profile/VSCL-2.0.0";
const char S104691[] = "Fly-HT2_i08 ObigoInternetBrowser/QO3C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104692[] = "Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7210supernova/UC Browser7.6.1.82/70/351";
const char S104693[] = "HTC_P3700 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S104694[] = "Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2730c-1/UC Browser7.5.1.77/70/351";
const char S104695[] = "Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.5.1.77/70/352";
const char S104696[] = "NOKIAE61i/UC Browser7.4.0.65/28/352";
const char S104697[] = "SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1 sonyericssonj105i/UC Browser7.4.0.65/70/351";
const char S104698[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-tw; Vibo-A600 Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104699[] = "NokiaN97mini/SymbianOS/9.1 Series60/3.0";
const char S104700[] = "NokiaN72/5.0819.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UCWEB7.6.0.75/27/401";
const char S104701[] = "NokiaC6-00/UCWEB7.6.0.75/50/401";
const char S104702[] = "Lenovo-P612/2009.08.12 P612_S112_090812/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104703[] = "BlackBerry8320/4.3.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104704[] = "BenQ-SIEMENS - C31/1.00/WAP2.0/MIDP2.0/CLDC1.0 UP.Browser/6.3.0.4.c.1.102 (GUI) MMP/2.0";
const char S104705[] = "Nokia6630/1.0 (5.03.08) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.6.1.82/27/351";
const char S104706[] = "sonyericssonk800i/UC Browser7.6.1.82/69/352";
const char S104707[] = "Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7210supernova/UC Browser7.7.1.88/70/351";
const char S104708[] = "NokiaE72/UCWEB7.7.0.81/28/999";
const char S104709[] = "MOT-C155 UP.Browser/6.2.2.7 (GUI) MMP/1.0";
const char S104710[] = "SEC-SGHE710";
const char S104711[] = "SEC-SGHE315-E315UVDH1-TSS1.01";
const char S104712[] = "SEC-SGHE315-E315UVEC3-TSS1.01";
const char S104713[] = "NOKIA6700s/UC Browser7.7.1.88/28/444";
const char S104714[] = "NokiaN97 mini/UCWEB7.8.0.87/50/800";
const char S104715[] = "lg-t310";
const char S104716[] = "Nokia7230/5.0 (06.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7230/UC Browser7.6.1.82/70/351";
const char S104717[] = "NokiaC2-01/5.0 (10.50) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-01/UC Browser7.6.1.82/70/351";
const char S104718[] = "NokiaX2-01/5.0 (07.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-01/UC Browser7.2.2.51/70/351";
const char S104719[] = "Xda_trion; 240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)";
const char S104720[] = "MQQBrowser/25 Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104721[] = "SonyEricssonWT13i/R3AD013 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104722[] = "SCH-A650 UP.Browser/6.2.2.3.d.2.100 (GUI) MMP/2.0";
const char S104723[] = "SonyEricssonCK15i/R3AC020 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104724[] = "MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; U20i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S104725[] = "AUDIOVOX-SMT5600/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 176x220)";
const char S104726[] = "AUDIOVOX-SMT5600/1.2 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 176x220)";
const char S104727[] = "Nokia7650/1.0 SymbianOS/6.1 Series60/0.9 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S104728[] = "Nokia N70";
const char S104729[] = "SAMSUNG-SGH-E356/1.0";
const char S104730[] = "SEC-SGHD730 SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S104731[] = "SEC-SGHE620/1.0 TSS/2.5";
const char S104732[] = "SonyEricssonK300i";
const char S104733[] = "SonyEricssonK500i";
const char S104734[] = "SonyEricssonK750c";
const char S104735[] = "SonyEricssonZ310i";
const char S104736[] = "LG-G4015 UP.Browser/6.2.2 (GUI) MMP/1.0";
const char S104737[] = "LG-MG800 MIC/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S104738[] = "LG-U8500 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104739[] = "KDDI-CA23 UP.Browser/6.2.0.5.136 (GUI) MMP/2.0";
const char S104740[] = "PG-1410/R01 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104741[] = "portalmmm/2.0 Z650i(c100;TB)";
const char S104742[] = "portalmmm/2.0 S400i(c10;TB)";
const char S104743[] = "portalmmm/2.0 S501i(c20;TB)";
const char S104744[] = "PG-1310V/R01 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104745[] = "Vodafone/1.0/HTC_wizard/2.21.3.102/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)";
const char S104746[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.9) MOT-Q9/01.02.23R";
const char S104747[] = "SAMSUNG-SGH-i718plus_CMCC/1.0 Release/03.07 Browser/IE6 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)";
const char S104748[] = "O2Jet";
const char S104749[] = "SCH-W339/ Embider/5.10.TI (GUI)";
const char S104750[] = "SoftBank/1.0/810P/PJP10 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104751[] = "KONKA_M526_CMCC/KCEM526.CMCC.2.01.704 MTK/6225 Release/7.4.2007 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104752[] = "SoftBank/1.0/822P/PJP10 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104753[] = "SoftBank/1.0/815SH/SHJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104754[] = "SoftBank/1.0/814SH/SHJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104755[] = "SCH-A645/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S104756[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Dopod838Pro";
const char S104757[] = "Spice S820";
const char S104758[] = "LG-KC560 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S104759[] = "WELLCOM_W398";
const char S104760[] = "Samsung-SPHA860 AU-MIC/2.0 MMP/2.0";
const char S104761[] = "Vodafone/1.0/SamsungSGHB520V/B520BUHI1 Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S104762[] = "SAMSUNG-SGH-F265L/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S104763[] = "Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0; en-GB; Nokia N97 RM-505) AppleWebKit/532.4 (KHTML, like Gecko) Qt/4.6.3 Mobile Safari/532.4";
const char S104764[] = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.6.9632.201";
const char S104765[] = "SonyEricssonG705/R1BA Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S104766[] = "SonyEricssonW100i/R1CA003 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S104767[] = "SonyEricssonU5/UC Browser7.6.1.82/50/352";
const char S104768[] = "Opera/9.80 (Windows NT 5.1; Opera Mobi/7457; U; en) Presto/2.5.28 Version/10.1";
const char S104769[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Flipboard/1.5.1";
const char S104770[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-us; Kobo Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S104771[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104772[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Desire HD Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S104773[] = "YahooMobile/1.0 (mail; 1.3.9.10087); (Linux; U; Android 2.3.4; GT-I9100 Build/GINGERBREAD_MR1);";
const char S104774[] = "YahooMobile/1.0 (mail; 1.3.8.9816); (Linux; U; Android 2.3.3; GT-I9100M Build/GINGERBREAD_MR1);";
const char S104775[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104776[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; ko-kr; MB861 Build/4.5.91; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104777[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 719)";
const char S104778[] = "HTC_Snap_S521 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone)";
const char S104779[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_P4550; Windows Phone 6.5)";
const char S104780[] = "JUC (Linux; U; 2.3.5; zh-cn; MEIZU_MX; 640*960) UCWEB7.8.0.95/139/355";
const char S104781[] = "MQQBrowser/Mini2.5 (SAMSUNG-GT-S5222)";
const char S104782[] = "MQQBrowser/2.7 Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104783[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104784[] = "NokiaN97 mini/UCWEB7.6.0.75/50/800";
const char S104785[] = "Opera/9.80 (Linux i686; Opera Tablet/86; U; en) Presto/2.7.81 Version/11.00";
const char S104786[] = "Opera/9.80 (And. 2.2.1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.7.81 Version/11.00";
const char S104787[] = "NokiaC5-03/UCWEB7.8.0.87/50/800";
const char S104788[] = "iBrowser/Mini2.4 (NokiaC1-01/04.40)";
const char S104789[] = "Dolphin HD 3.5 (iPad; iPhone OS 5.0.1; en_US)";
const char S104790[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]";
const char S104791[] = "Mozilla/5.0 (iPod touch; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]";
const char S104792[] = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) [operamini] [UsableNet Lift Mobile]";
const char S104793[] = "Dolphin/4.1 CFNetwork/485.12.7 Darwin/10.4.0";
const char S104794[] = "Dolphin HD 4.1 (iPad; iPhone OS 4.2.1; en_US)";
const char S104795[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Evo 3D Build/GRJ22; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104796[] = "Puffin/3337 CFNetwork/485.12.7 Darwin/10.4.0";
const char S104797[] = "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Flipboard/1.7.2";
const char S104798[] = "NokiaN97/UCWEB8.1.0.104/50/999";
const char S104799[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR Build/6.5.1-167_DHD-14_M3-8)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104800[] = "NokiaX6-00/UCWEB8.3.0.133/50/800";
const char S104801[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9000 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S104802[] = "Instacast/2.0.1 (like iTunes/10.1.2)";
const char S104803[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, gb";
const char S104804[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; Microsoft; XDeviceEmulator)";
const char S104805[] = "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; Taylor)";
const char S104806[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; MWP6885)";
const char S104807[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; ZTE; Windows Phone - Internet 7; SFR)";
const char S104808[] = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/1.5.2.1185 (Phoenix Viewer Release; Firebird skin) Safari/533.3";
const char S104809[] = "lg-lg900g/UC Browser7.9.0.102/69/352";
const char S104810[] = "Nokia5250/UCWEB8.1.0.104/50/999";
const char S104811[] = "SonyEricssonU8i/UC Browser8.0.4.121/50/352/UCWEB";
const char S104812[] = "NOKIA-Lumia 900/UCWEB2.2.0.157/47/32079";
const char S104813[] = "Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.0m";
const char S104814[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; WT19i Build/4.1.B.0.431) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S104815[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.53.11 (KHTML, like Gecko) Version/5.1 Safari/7534.51.22 Sleipnir/3.3";
const char S104816[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC One S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, gb";
const char S104817[] = "Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; A853 Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104818[] = "Mozilla/5.0 (Linux; U; Android 1.0.0; es-es; bq Verne Plus Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S104819[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; X10 TripNMiUI-IRIS Build/NI17) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104820[] = "JUC(Linux;U;Android2.1-update1;Zh_cn;GT-I5700;320*480;)UCWEB7.8.0.95/139/355";
const char S104821[] = "JUC(Linux;U;Android2.2;Zh_cn;HTC Legend;320*480;)UCWEB7.8.0.95/139/355";
const char S104822[] = "MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.53) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S104823[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Touchpad Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104824[] = "Dalvik/1.1.0 (Linux; U; Android 2.1; PD_Novel Build/ERD79)";
const char S104825[] = "JUC(Linux;U;Android2.3.4;Zh_cn;SCH-R720;320*480;)UCWEB7.8.0.95/139/444";
const char S104826[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en_gb) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/1.9.4 Safari/533.1";
const char S104827[] = "Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; T959 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104828[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-T959V Build/FROYO)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104829[] = "2.3 SAMSUNG-SGH-I807D---Mozilla/5.0 (Linux; U; Android 2.3; xx-xx; SAMSUNG-SGH-I807D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104830[] = "iBrowser/2.8 Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104831[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2; en-in; LG-P500 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104832[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; VM670 Build/FRG83G)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104833[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; Spice Mi280 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S104834[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo P70 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104835[] = "Mozilla/5.0 (Linux; U; Android RGUI 2.3.23; en-us; GT-I9100 Build/RGUI MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104836[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PG86100 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7";
const char S104837[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC EVO 3D X515m Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104838[] = "NOKIA-Lumia 800/UCWEB2.0.0.149/47/32079";
const char S104839[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.25 (KHTML, like Gecko) Version/5.1 Safari/7534.51.22 Sleipnir/3.6";
const char S104840[] = "Reeder/2.3 CFNetwork/485.12.7 Darwin/10.4.0";
const char S104841[] = "MQQBrowser/2.0 (NokiaN97mini;SymbianOS/9.1 Series60/3.0)";
const char S104842[] = "Instagram 3.2.0 Android (16/4.1.2; 213dpi; 800x1205; asus/google; Nexus 7; grouper; grouper; en_US)";
const char S104843[] = "TransperaSDK v3.3.0.262 : google : Nexus 7 : Android OS : 4.1.1";
const char S104844[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus 7 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104845[] = "sonyericssons500i/UC Browser8.0.3.107/70/352";
const char S104846[] = "Opera/9.80 (Android; Linux; Opera Tablet/23731; U; en) Presto/2.9.201 Version/11.50";
const char S104847[] = "SAMSUNG-GT-I8350/UCWEB2.1.0.155/47/32079";
const char S104848[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s7233e) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S104849[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6630) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S104850[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; s8003) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S104851[] = "SAMSUNG-GT-S5222R";
const char S104852[] = "OneBrowser/3.0 (SAMSUNG-GT-S5222R)";
const char S104853[] = "Evernote Android/AndroidMarket_43x_58 (en_CA); Android/3.1; HTC-P715a/12;";
const char S104854[] = "Android/3.1; HTC HTC-P715a; 2.3.0";
const char S104855[] = "SAMSUNG-GT-S3802 [operamini] [UsableNet Lift Mobile]";
const char S104856[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; SX-SP715A Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7";
const char S104857[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; BOHP P5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 baiduboxapp/042_1.7.3_diordna_008_084/spla_01_6.3.2_5P+PHOB/Xa/544F1608FBFB8335CXA5342AA69%7CX/1";
const char S104858[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GN205 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) FlyFlow/2.4 Version/4.0 Mobile Safari/533.1 baidubrowser/042_1.8.4.2_diordna_008_084/EENOIG_01_4.3.2_502NG/Xj/8DDC5302B7B201F86B1C73BXFF%7CX/1";
const char S104859[] = "Mozilla/5.0 (Java; U; en-us; sonyericssonk770i) UCBrowser8.2.1.144/69/444/UCWEB Mobile";
const char S104860[] = "Nokia3500c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia3500c) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S104861[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; s5230) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S104862[] = "Windows Phone Search (Windows Phone OS 7.10;HTC;T8788;7.10;8862)";
const char S104863[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-I9200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S104864[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9152 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S104865[] = "MQQBrowser/21 Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104866[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6810 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, au";
const char S104867[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7100 Build/RS_Supremacy_AOKP v0.2 - IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S104868[] = "JUC(Linux;U;Android3.2;Zh_cn;GT-P7510;800*1232;)UCWEB7.8.0.95/139/351";
const char S104869[] = "iBrowser/2.8 Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S104870[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 3.2; zh-tw; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 MobileSafari/534.13";
const char S104871[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7510 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S104872[] = "iBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104873[] = "iBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104874[] = "Mozilla/5.0 (Linux; U; Android 3.2; ru-ru; GT-P7510 Build/HTJ85B)Maxthon AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S104875[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S104876[] = "YahooMobile/1.0 (mail; 1.4.5.12736); (Linux; U; Android 3.2; GT-P7310 Build/HONEYCOMB_MR2);";
const char S104877[] = "MQQBrowser/2.8 Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104878[] = "iBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104879[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S104880[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104881[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104882[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104883[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P5113 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S104884[] = "JUC(Linux;U;Android4.0.4;Zh_cn;GT-P7500;800*1232;)UCWEB7.8.0.95/139/444";
const char S104885[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-P7300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104886[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7310 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S104887[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P6210 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S104888[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352";
const char S104889[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7310 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, se";
const char S104890[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; GT-P7310 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0100, nl";
const char S104891[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S104892[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104893[] = "YahooMobile/1.0 (mail; 1.4.5.12736); (Linux; U; Android 4.0.4; GT-P6200 Build/Unknown);";
const char S104894[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104895[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-P7500 Build/GINGERBREAD)";
const char S104896[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; GT-P7500 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S104897[] = "Android OS/12 samsung_GT-P7310";
const char S104898[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104899[] = "TuneIn Radio/6.6 (Android 15; GT-P5100; Java)";
const char S104900[] = "Android/1.0 (GT-P7511 HTJ85B)";
const char S104901[] = "MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S104902[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S104903[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6810 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S104904[] = "MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104905[] = "YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 4.0.4; GT-P7310 Build/Unknown);";
const char S104906[] = "Instagram 1.0.6 Android (12/3.1; 160dpi; 1280x800; samsung; GT-P7300; GT-P7300; p3; en_GB)";
const char S104907[] = "MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S104908[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [operamini] [UsableNet Lift Mobile]";
const char S104909[] = "Mozilla/5.0 (Linux; Android 3.2; GT-P7500 Build/unknown) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S104910[] = "Instagram 1.0.6 Android (8/2.2; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; GT-P1000; en_GB)";
const char S104911[] = "Instagram 1.0.6 Android (10/2.3.3; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; gt-p1000n; en_GB)";
const char S104912[] = "WhatsApp/2.7.8604 Android/3.1 Device/samsung-GT-P7300";
const char S104913[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7510 Build/HMJ37; 800*1280) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/352";
const char S104914[] = "WhatsApp/2.7.5752 Android/2.2 Device/samsung-GT-P1000";
const char S104915[] = "YahooMobileMessenger/1.0 (Android Mail; 1.4.6) (GT-P1000; samsung; GT-P1000; 2.2.1/FROYO)";
const char S104916[] = "YahooMobileMessenger/1.0 (Android Messenger; 1.8.3) (GT-P1000; samsung; GT-P1000; 4.0.4/IMM76L)";
const char S104917[] = "Instagram 3.0.5 Android (15/4.0.4; 160dpi; 1024x552; samsung; GT-P3113; espressowifi; espresso; en_CA)";
const char S104918[] = "WhatsApp/2.7.8863 Android/3.2 Device/samsung-GT-P6200";
const char S104919[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104920[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104921[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7510 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.10/220,1.9.10.220,2008-02-29 09:33, vn";
const char S104922[] = "Instagram 1.0.6 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P3100; espressorf; espresso; en_GB)";
const char S104923[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6800 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S104924[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6810 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P6810, 36, 1.7.1)";
const char S104925[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P1000 Build/JZO54K; CyanogenMod-Sapir) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P1000, 36, 1.7.1)";
const char S104926[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-au; GT-P7320T Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, au";
const char S104927[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, id";
const char S104928[] = "Android OS/13 samsung_GT-P6200";
const char S104929[] = "Instagram 3.3.1 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7510; GT-P7510; p3; en_US)";
const char S104930[] = "TuneIn Radio/6.7 (Android 8; GT-P1000; Java)";
const char S104931[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7500 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S104932[] = "Instagram 3.3.3 Android (15/4.0.4; 160dpi; 800x1232; samsung; GT-P5113; espresso10wifi; espresso10; en_CA)";
const char S104933[] = "Evernote Android/master_90 (ko_KR); Android/3.1; GT-P7300/12;";
const char S104934[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P5100 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.14/271,1.9.14.271,2012-12-20 13:41, +0300, by";
const char S104935[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P1000, 36, 1.7.1)";
const char S104936[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104937[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104938[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7501 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S104939[] = "Android/3.2; samsung GT-P7300; 2.3.2";
const char S104940[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [operamini] [UsableNet Lift Mobile]";
const char S104941[] = "JUC(Linux;U;Android3.2;Zh_cn;GT-P7501;800*1232;)UCWEB7.8.0.95/139/352";
const char S104942[] = "Mozilla/5.0(Linux;U;Android3.2;Zh_cn;GT-P7501;800*1232;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352";
const char S104943[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fa-fa; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; fa) Presto/2.8.119 Version/11.10";
const char S104944[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6210 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355";
const char S104945[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; GT-P1000 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104946[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; fa-fa; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; fa) Presto/2.8.119 Version/11.10";
const char S104947[] = "TuneIn Radio/6.6 (Android 13; GT-P7500; Java)";
const char S104948[] = "TuneIn Radio/6.6 (Android 13; GT-P7500R; Java)";
const char S104949[] = "WhatsApp/2.8.5610 Android/3.0.1 Device/samsung-GT-P7100";
const char S104950[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3110 Build/IMM76D; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444";
const char S104951[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S104952[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build/IMM76D; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.4.145/139/800";
const char S104953[] = "MQQBrowser/2.8 Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S104954[] = "Orca-Android/1.5.005 OS/3.1 Model/GT-P7500";
const char S104955[] = "JUC(Linux;U;Android4.2.1;Zh_cn;GT-P3100;600*976;)UCWEB7.8.0.95/139/444";
const char S104956[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6200 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S104957[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S104958[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-P7500 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S104959[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-ca; GT-P7500 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S104960[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-P1000 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S104961[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-P1000 Build/MIUI-XJ) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S104962[] = "WhatsApp/2.9.1587 Android/2.3.3 Device/samsung-GT-P1000";
const char S104963[] = "MQQBrowser/2.8 Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S104964[] = "Mozilla/5.0 (Linux; U; Android 3.2; es-es; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.3.3000";
const char S104965[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259";
const char S104966[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P3110 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31";
const char S104967[] = "Mozilla/5.0 (Linux; U; Android 3.2; ru-ru; GT-P7300 Build/HTJ85B)Maxthon AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S104968[] = "Instagram 3.4.2 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7310; GT-P7310; p3; en_US)";
const char S104969[] = "Evernote Android/AndroidMarket_45x_243.14540 (en_US); Android/3.2; GT-P6210/13;";
const char S104970[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 UCBrowser/2.2.2.259";
const char S104971[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13 UCBrowser/2.2.2.259";
const char S104972[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; sk-sk; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]";
const char S104973[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S104974[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200L Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S104975[] = "JUC(Linux;U;Android4.1.1;Zh_cn;GT-P5100;800*1232;)UCWEB7.8.0.95/139/351";
const char S104976[] = "Mozilla/5.0(Linux;U;Android4.1.1;Zh_cn;GT-P5100;800*1232;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/351";
const char S104977[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S104978[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S104979[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P6200 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0100, pl";
const char S104980[] = "Android_CellLocation_2.0.X_GgPoOkKfHjObFgFeIpBlDcAgCfKpDkAaInDiObLoHcLeKdLfBcDpIjFcIbNgHaBfNiCoEeIdHcGmFgHjDdOeNbMiLmNiAkKl@GO%E5%A4%A9%E6%B0%94EX_3.52;X;X;samsung;GT-P6200;13";
const char S104981[] = "JUC(Linux;U;Android4.1.1;Zh_cn;GT-P1000;600*1024;)UCWEB7.8.0.95/139/444";
const char S104982[] = "theScore/3.0.3 Android-OS/4.1.1 (Android; Phone, GT-P3113)";
const char S104983[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-P6211 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0100, de";
const char S104984[] = "Orca-Android/2.2.4-release OS/4.0.4 Model/GT-P3100 VersionCode/97447 Product/Messenger Carrier/MY_MAXIS Manufacturer/samsung Brand/samsung";
const char S104985[] = "Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P6210 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.3.3000";
const char S104986[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; zh-CN; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31";
const char S104987[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31";
const char S104988[] = "Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; GT-P7510 Build/HRI83) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7510, 39, 1.8.2)";
const char S104989[] = "Mozilla/5.0 (Linux; U; Android 3.2; fa-fa; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.3.3000";
const char S104990[] = "GoogleEarth/7.0.0001.8177(Android;Android (GT-P3110-espressowifi-user-4.1.1);en-GB;kml:2.2;client:Free;type:default)";
const char S104991[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3110 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S104992[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3110 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S104993[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P1000 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104994[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-CN; GT-P7510 Build/HTJ85B) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.1.258 U3/0.8.0 Mobile Safari/534.31";
const char S104995[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P6810 Build/JRO03R) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.28 Safari/537.31";
const char S104996[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-CN; GT-P7500 Build/HMJ37) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.1.258 U3/0.8.0 Mobile Safari/534.31";
const char S104997[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259";
const char S104998[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S104999[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7510 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.0.181/145/352";
const char S105000[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P5110 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.1.258 U3/0.8.0 Mobile Safari/534.31";
const char S105001[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-CN; GT-P6201 Build/HTJ85B) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31";
const char S105002[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P3100 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315";
const char S105003[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S105004[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S105005[] = "theScore/3.2.1 Android-OS/3.2 (Android; Phone, GT-P6210)";
const char S105006[] = "theScore/3.2.2 Android-OS/2.2.1 (Android; Phone, GT-P1010)";
const char S105007[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 [UsableNet Lift Mobile]";
const char S105008[] = "Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P6810 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0100, fr";
const char S105009[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.1; id-id; GT-P5100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105010[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3113 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S105011[] = "Orca-Android/2.2.5-release OS/4.1.1 Model/GT-P5100 VersionCode/X Product/Messenger Carrier/MY_MAXIS Manufacturer/samsung Brand/samsung";
const char S105012[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-gb; GT-P3113 Build/ERD62) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S105013[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105014[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105015[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3110 Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.27/383,1.9.27.383,2013-03-25 21:34, +0800, china, cn";
const char S105016[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 OverDrive Media Console/2.6.4";
const char S105017[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7310 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S105018[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105019[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105020[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7510 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S105021[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; GT-P1000 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444";
const char S105022[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-P6200 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S105023[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7511 Build/JDQ39; CyanogenMod-Ganbarou V3.04 GT-P7511) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105024[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; de-at; GT-P7511 Build/JOP40G; CyanogenMod-Ganbarou V3.02 GT-P7511) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105025[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; GT-P5113 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S105026[] = "Mozilla/5.0 (Java; U; en-us; lg-t310i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S105027[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s5260) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S105028[] = "Nokia5200/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia5200) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S105029[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; s8003) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S105030[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5230w) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S105031[] = "Mozilla/5.0 (Linux; Android 4.2.2; C5503 Build/10.3.A.0.175) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22";
const char S105032[] = "Mozilla/5.0 (Linux; Android 4.1.2; SOL22 Build/10.2.F.3.15) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S105033[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5333) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile";
const char S105034[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-gt350) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S105035[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; mot-ex115) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S105036[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s7230e) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S105037[] = "Mozilla/5.0 (Java; U; en-in; lg-t300) UCBrowser8.4.0.159/69/444/UCWEB Mobile";
const char S105038[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s7070) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S105039[] = "UCWEB/2.0 (Java; U; MIDP-2.0; vi; q-mobile) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S105040[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; zte-p80aiii-f160) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile";
const char S105041[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3370) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S105042[] = "Opera/9.80 (Linux armv6l; U; en) Presto/2.8.115 Version/11.10 AQUOS-AS/1.0 LC-52LE835X";
const char S105043[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; s8003) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S105044[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5230w) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S105045[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8600) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S105046[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5753e) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S105047[] = "Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 50LA6210-TD; 03.20.09; 0xX;); LG NetCast.TV-2013 /03.20.09 (LG, 50LA6210-TD, wireless)";
const char S105048[] = "Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 55LA6210-TD; 03.20.15; 0xX;); LG NetCast.TV-2013 /03.20.15 (LG, 55LA6210-TD, wireless)";
const char S105049[] = "Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 55LA8600-TA; 03.05.56; 0xX;); LG NetCast.TV-2013 /03.05.56 (LG, 55LA8600-TA, wireless)";
const char S105050[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 50PM4710-TD; 04.00.09; 0xX;); LG NetCast.TV-2012 0";
const char S105051[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 55LM660S-ZA; 03.03.31; 0xX;); LG NetCast.TV-2012";
const char S105052[] = "Mozilla/5.0 (DirectFB; U; Linux 35230; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.1.4(+3D+SCREEN+TUNER; LGE; 47LW5700-TA; 04.01.19; 0xX;); LG NetCast.TV-2011";
const char S105053[] = "Mozilla/5.0 (DirectFB; U; Linux mips; en) AppleWebKit/531.2+ (KHTML, like Gecko, Safari/531.2+); LG Browser/4.0.9(+mouse+3D+SCREEN+TUNER; LGE;42LE7500-ZA; 03.05.04; 0xX;); LG NetCast.TV-2011";
const char S105054[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LM640S-ZA; 04.01.73; 0xX;); LG NetCast.TV-2012 0";
const char S105055[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 55LM620S-ZE; 04.20.23; 0xX;); LG NetCast.TV-2012";
const char S105056[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LM6600-TA; 04.10.09; 0xX;); LG NetCast.TV-2012";
const char S105057[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LM7600-DA; 04.00.15; 0xX;); LG NetCast.TV-2012";
const char S105058[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LM6410-TB; 04.20.11; 0xX;); LG NetCast.TV-2012";
const char S105059[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LM640S-ZA; 04.40.23; 0xX;); LG NetCast.TV-2012";
const char S105060[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 84LM9600-TA; 04.20.74; 0xX;); LG NetCast.TV-2012";
const char S105061[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+SCREEN+TUNER; LGE; 32LS570T-ZB; 04.10.23; 0xX;); LG NetCast.TV-2012";
const char S105062[] = "Opera/9.80 (Linux mips; Opera TV Store/4234; U; es-ES) Presto/2.9.167 Version/11.50 Model/Sony-KDL-46HX755 SonyCEBrowser/1.0 (KDL-46HX755; CTV/PKG1.306BRB; CHL)";
const char S105063[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (sTgbfDapDmqYrcInk12ne+0od4IFAmY5UFkEXIq+Vlg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2";
const char S105064[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X) AppleWebKit/907.0 (KHTML, like Gecko) Chrome/33.0.1.0.8321.0.15.0 Safari/907.0";
const char S105065[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8600) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S105066[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5230w) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S105067[] = "Mozilla/5.0 (Linux; Android 4.3; HTC One Build/JWR66Y.H1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1616.0 Mobile Safari/537.36";
const char S105068[] = "Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1612.0 Mobile Safari/537.36";
const char S105069[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1612.0 Mobile Safari/537.36";
const char S105070[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.32 (KHTML, like Gecko) Chromium/25.0.1349.2 Chrome/25.0.1349.2 Safari/537.32 Ubuntu/13.10 (3.6.1-2ubuntu3) Epiphany/3.6.1";
const char S105071[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.32 (KHTML, like Gecko) Chromium/25.0.1349.2 Chrome/25.0.1349.2 Safari/537.32 Ubuntu/13.10 (3.9.2-0ubuntu1~saucy1) Epiphany/3.9.2";
const char S105072[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian95) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S105073[] = "Mozilla/5.0 (Linux; Android 4.0.4; ST25i Build/6.1.1.B.1.89) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S105074[] = "Mozilla/5.0 (Linux; Android 4.1.1; Lenovo P770 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S105075[] = "Mozilla/5.0 (Linux; Android 4.0.4; MOBHIG0080 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S105076[] = "Mozilla/5.0 (Linux; Android 4.0.4; Q80 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63537";
const char S105077[] = "Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S105078[] = "Mozilla/5.0 (Linux; Android 4.1.2; LG-F180 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S105079[] = "Mozilla/5.0 (Linux; Android 2.3.6; GT-S6102 Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S105080[] = "Mozilla/5.0 (X11; Linux i686; rv:6.0) Gecko/X Firefox/29.0";
const char S105081[] = "Mozilla/5.066 (X11; I; Linux x86-64+Win64; cs; rv:1.9.1.14) Gecko/X Firefox/29.0.0.99";
const char S105082[] = "Mozilla/5.0 (PlayStation Vita 3.00) AppleWebKit/536.26 (KHTML, like Gecko) Silk/3.2";
const char S105083[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3230) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S105084[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-78) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/32.0.1651.2 Mobile Safari/537.36";
const char S105085[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Coast/1.1.0.00000 Mobile/11A4449d Safari/7534.48.3";
const char S105086[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Coast/1.1.0.64172 Mobile/11A465 Safari/7534.48.3";
const char S105087[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae65-1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S105088[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; SAMSUNG-GT-S5263) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S105089[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; GT-S5233S) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S105090[] = "Nokia6300/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6300/UC Browser7.5.1.77/69/401";
const char S105091[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) O2 Xda Atom";
const char S105092[] = "Nokia2700c-2/2.0 (07.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2700c-2/UC Browser7.0.0.41/70/352";
const char S105093[] = "Mozilla/5.0 (Series40/3.0FP1; Nokia5130c-2/07.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/0.8.3";
const char S105094[] = "Nokia3110c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia3110c/UC Browser7.4.0.65/70/351";
const char S105095[] = "MOT-A3000_CMCC/1.0 Release/09.09.2008 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/WM6.1 Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/16747; U; en)";
const char S105096[] = "Nokia6303iclassic/5.0 (07.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303iclassic/UC Browser7.6.1.82/70/351";
const char S105097[] = "BlackBerry8703e/4.5.0.148 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/104 UP.Browser/5.0.3.3";
const char S105098[] = "HTC_Touch_Diamond2_T5353 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S105099[] = "Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5310xpressmusic/UC Browser7.4.0.65/70/351";
const char S105100[] = "SAMSUNG-SGH-CC03/1.0 RTK-E/1.0 SYSOL/1.0 Release/09.01.2007 Browser/OpenWave6.2.3.3.c.1.101/*MzU5MDk5MDEwNjgyNzU0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S105101[] = "MOT-SLVR(RED)/08.C4.35R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105102[] = "Mozilla/5.0 (Linux; U; Android RCMix_v2.0; en-gb; HTC_HERO Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S105103[] = "SAMSUNG-SGH-F258/1.0 RTK-E/1.0 DF/1.0 Release/07.04.2007 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzUyNTkxMDIwODE1MzMw UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S105104[] = "samsung-gt-s8500";
const char S105105[] = "SonyEricssonW958c/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 306) Opera 8.60 [en]";
const char S105106[] = "SAMSUNG-SGH-E878/NetFront 3.2/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzU5MTU2MDA4MDQwMzY5";
const char S105107[] = "Nokia6303classic/2.0 (09.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303classic/UC Browser7.5.1.77/70/351";
const char S105108[] = "Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303classic/UC Browser7.4.0.65/70/351";
const char S105109[] = "HTC-8900/1.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S105110[] = "SAMSUNG-SGH-D908i/1.0 RTK-E/1.0 DF/1.0 Release/11.27.2006 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzU1NjU0MDEwMTkyNDEy UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S105111[] = "MOT-K3/99.41.04R MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S105112[] = "Nokia2730c-1/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2730c-1/UC Browser7.4.0.65/70/351";
const char S105113[] = "lg-x330";
const char S105114[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) O2 Xda Atom";
const char S105115[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) HP iPAQ rw6828";
const char S105116[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) O2 Xda Atom Exec";
const char S105117[] = "nokia6280/UC Browser7.5.1.77/69/401";
const char S105118[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) 480x800 SAMSUNG SCH-i920 PPC";
const char S105119[] = "SAMSUNG_GT_i8000 Opera/9.50 (Windows NT 5.1; U; en)";
const char S105120[] = "NokiaX2-01/5.0 (07.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-01/UC Browser7.4.0.65/70/351";
const char S105121[] = "NokiaC2-01/5.0 (10.50) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-01/UC Browser7.2.2.51/70/352";
const char S105122[] = "SAMSUNG-SGH-i908/1.0 Release/03.07 Browser/IE6 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105123[] = "Q-Mobile-F220/Profile/MIDP.2.0Configuration/CLDC.1.0";
const char S105124[] = "SAMSUNG-SGH-E578-CMCC/NetFront 3.2/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzUyOTk3MDExOTY1MDEw";
const char S105125[] = "WELLCOM-W388_WAP_Browser";
const char S105126[] = "SAMSUNG-SGH-F278/1.0 RTK-E/1.0 DF/1.0 Release/07.04.2007 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzU0NDIwMDIwMTYxNTk0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S105127[] = "SCH-W239/1.0 POLARIS/5.30.WAP CTC/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S105128[] = "MOT-Q9W/01.11.21I Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S105129[] = "SonyEricssonW888c/R8BA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105130[] = "LG/KF690/v10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S105131[] = "SAMSUNG-SGH-E898/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0 Untrusted/1.0";
const char S105132[] = "Nokia2865i/2.0 (FL190V0200.nep) UP.Browser/6.2.3.8 MMP/2.0";
const char S105133[] = "HTC-S411-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 240x320)";
const char S105134[] = "DopodS301/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 240x320)";
const char S105135[] = "dopod S700/5.2.19538/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S105136[] = "SoftBank/1.0/930P/PJP22 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105137[] = "Panasonic-A210/1.0 UP.Browser/6.1.0.7.8 (GUI) MMP/1.0";
const char S105138[] = "FLY-SX390/2.0 (03.15) Profile/MIDP-2.0 Configuration/CLDC-1.1, FLY-SX390/2.0 (03.15) Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105139[] = "nokia6288/UC Browser7.2.2.51/69/401";
const char S105140[] = "SAMSUNG-SGH-i718plus_CMCC/1.0 Release/03.07 Browser/IE6 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S105141[] = "LG-ME770d MIC/1.1.14 MIDP-2.0/CLDC-1.1";
const char S105142[] = "SAMSUNG-GT-C5510U/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S105143[] = "LG-F1200 UP.Browser/6.2.3 (GUI) MMP/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105144[] = "SoftBank/1.0/830SC/SCJ001 Browser/NetFront/3.3";
const char S105145[] = "SAMSUNG-SGH-X600A/K3 UP.Browser/6.1.0.6 (GUI) MMP/1.0";
const char S105146[] = "Philips292 / Obigo Browser 2.0";
const char S105147[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) DopodD818c";
const char S105148[] = "Bird.M29/LV,A01/WAP2.0/MIDP-2.0/CLDC-1.1 Browser/UP.Browser/7.1.0.f.1.160 (GUI)";
const char S105149[] = "LG-C3400 MIC/WAP2.0 MIDP-2.0/CLDC-1.0";
const char S105150[] = "SonyEricssonT610/R2L SEMC-Browser/4.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105151[] = "SonyEricssonZ710c/R1EF Release/Sep-08-2006 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105152[] = "Alcatel-OT-C717/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S105153[] = "SonyEricssonC702a/R3EA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.3";
const char S105154[] = "SEC-SGHD347/1.0 TSS/2.5";
const char S105155[] = "Alcatel-OT-C552/1.0 UP.Browser/7.1 (GUI) MMP/2.0";
const char S105156[] = "Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 LG KS10/v10A; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413";
const char S105157[] = "HTC_P4350 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S105158[] = "SHARP-TQ-GX20/1.0 Profile/MIDP-1.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.2.107 (GUI) MMP/1.0";
const char S105159[] = "BlackBerry8703e/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/126 UP.Browser/5.0.3.3";
const char S105160[] = "PHILIPS9A9W ObigoInternetBrowser/QO3C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105161[] = "SAMSUNG-SGH-J708/1.0 RTK-E/1.0 DF/1.0 Release/02.05.2008 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzUzMzg1MDIwNDgzMzMz UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S105162[] = "Mozilla/5.0 (dopod-T5399/1.21.706.12;U;Windows Mobile/6.5;Profile/MIDP-2.0 Configuration/CLDC-1.1;480*640;CTC/2.0) MSIE/6.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S105163[] = "gt-s5233w";
const char S105164[] = "W891 ObigoInternetBrowser/QO3C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105165[] = "Mozilla/4.0 (compatible; MSIE 5.0; Series60/3.0 Nokia6630/4.06.0 Profile/MIDP-2.0 Configuration/CLDC-1.1)";
const char S105166[] = "MOT-MileStone/00.00.00 MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105167[] = "MOT-ZN5/FSRM319_81_00_7FR_V3 Mozilla/4.0 (compatible; OSS/1.0; Linux MOTOZINE ZN5) Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony 1.0";
const char S105168[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 480x800; GSmart-S1200";
const char S105169[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; GSmart-MS802";
const char S105170[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) LG-KS20/V10a Browser/IEMobile/7.6 MMS/LG-MMS-WINCE-V1.0/1.2 Java/LGVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105171[] = "SonyEricssonW995a/R1EA";
const char S105172[] = "wellcom w3331";
const char S105173[] = "BlackBerry8700/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/120 UP.Browser/5.0.3.3";
const char S105174[] = "SAGEM-my302X/1.0/ MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.g.2.106 (GUI) MMP/2.0";
const char S105175[] = "HTC-P4600/%ROM_VERSION% Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S105176[] = "Lenovo-E228/CMCC BMP/F2 Release/2008.09.10 Browser/TelecaQ03C";
const char S105177[] = "Utec-M699 UP.Browser/7.2.6.1.591 (GUI) MMP/2.0";
const char S105178[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8500/1.0; U; Bada/1.0; vi-vn) AppleWebKit/533.1 (KHTML";
const char S105179[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) HP iPAQ rw6828";
const char S105180[] = "Mozilla/5.0 (Linux; U; Android Master Rom 1.1.4; en-us; SGH-T959 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S105181[] = "SAMSUNG-SGH-U608/1.0 Release/1.24.2007 Browser/NetFront3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105182[] = "SonyEricssonK111i/R1EA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S105183[] = "NEC-e238/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.3.2.g.1.106 (GUI) MMP/2.0";
const char S105184[] = "Samsung-SGH-i408_CMCC/1.0 SymbianOS/9.2 Series60/3.1 Release/5.25.2007 Browser/OSS3.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S105185[] = "LG-KG280 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S105186[] = "SAMSUNG-GT-M8910U/1.0 SHP/VPP/R5 Release/26.08.2009 NetFront/3.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1";
const char S105187[] = "CLARO-X930/X931_V1_Z8_ES_A1EF101 Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105188[] = "Lenovo-P82/S103 LMP/LML Release/2009.05.19 Profile/MIDP2.0 Configuration/CLDC1.1";
const char S105189[] = "SAMSUNG-SGH-L708E_CMCC/1.0 RTK-E/1.0 DF/1.0 Release/08.17.2007 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzU1Mjc5MDIzODMzMjIy UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S105190[] = "LG-KG245/MIC/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S105191[] = "Samsung-SPHA640 AU-MIC-A640/2.0 MMP/2.0";
const char S105192[] = "LG-U8210/V100 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S105193[] = "SEC-SGHE810";
const char S105194[] = "Nokia6300/2.0 (07.00) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6300/UC Browser7.7.1.88/70/444";
const char S105195[] = "Mozilla/5.0 (Series40; Nokia5130c-2/07.95; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.2.26.6";
const char S105196[] = "BlackBerry8700/4.1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100 UP.Browser/5.0.3.3";
const char S105197[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A555 Build/1.16.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 320X480 motorola A555";
const char S105198[] = "SonyEricssonK618i/R1KG SEMC/NewsReader/1.0";
const char S105199[] = "Mozilla/5.0 (Series40; NokiaX3-00/08.50; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.2.26.6";
const char S105200[] = "Mozilla/5.0 (Series40; Nokia2700c-2/09.95; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.2.26.6";
const char S105201[] = "MOT-MOTOV9/A0.05.20R_03 BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003348) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [zh]";
const char S105202[] = "SAMSUNG-GT-I8000U/1.0 Software/WM6.5 Release/01.24.2009 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S105203[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC i-mate 8502; 240x320; PPC i-mate 8502/ 240x320; PPC i-mate 8502; 240x320; PPC)";
const char S105204[] = "SonyEricssonV630iv/R1CE SEMC/NewsReader/1.0";
const char S105205[] = "SonyEricssonZ520i/R3F";
const char S105206[] = "MOT-W375/1.0 Release/03.30.2007 Browser/UPB6.3 Software/0.0.80";
const char S105207[] = "Mozilla/5.0 (Linux; U; Android VENROM_DL25_RC3_2.2.1; en-us; SCH-R880 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105208[] = "Mozilla/5.0(Linux;U;Android2.2;Zh_cn;SAMSUNG-SGH-I897;480*800;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355";
const char S105209[] = "Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; Asus A10 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17; A10-V5.2.4-user-20101118";
const char S105210[] = "HD_mini_T5555 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S105211[] = "BlackBerry8700/4.5.0.124 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100 UP.Browser/5.0.3.3";
const char S105212[] = "Opera/9.70 (Windows NT 6.1; U; en-gb) Presto/2.2.15";
const char S105213[] = "Palm750/v0101 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S105214[] = "HTC-3100/1.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8)";
const char S105215[] = "Mozilla/5.0 (Linux; U; Android Custom 2.3.3 by Snegovik; ru-ru; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2";
const char S105216[] = "MQQBrowser/21 Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME722 Build/MLS2GC_2.6.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105217[] = "NokiaC1-01/2.0 (04.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac1-01/UC Browser7.4.0.65/70/351";
const char S105218[] = "MOT-V186/08.30.10R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S105219[] = "Mozilla/5.0 (Linux; U; Android 2.1.2; de-de; LG-P990 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105220[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) ASUS-P525/1.0";
const char S105221[] = "Mozilla/5.0 (Linux; U; Android X.666 v1.1; en-us; X10a Build/2.1.A.0.435) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S105222[] = "SAMSUNG-SGH-X668/TSS 2.5.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S105223[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) o2 Xda comet";
const char S105224[] = "PPC; 240x320; HTC_TyTN/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Dopod838Pro";
const char S105225[] = "SCH-M609/M609CH01 POLARIS/5.30.WAP CTC/1.0";
const char S105226[] = "Mozilla/5.0 (SCH-M519/M519CJ26;U;REX/3.0;BREW/3.1.5;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*320;CTC/2.0) POLARIS/6.15.WEB";
const char S105227[] = "Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2690/UC Browser7.0.0.41/70/352";
const char S105228[] = "Fly_DS210 ObigoInternetBrowser/QO3C Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105229[] = "SonyEricssonK608i/R2BA";
const char S105230[] = "LG-KG77_CMCC/08.03.14 Infineon/SGold2 Release/29.12.2006 Browser/MIC1.1.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105231[] = "LG-CT810/V10x IEMobile/7.11 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S105232[] = "MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105233[] = "HTC-P4600/1.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)";
const char S105234[] = "HUAWEI G6610";
const char S105235[] = "BlackBerryUnknown/5.0.0.526 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/11";
const char S105236[] = "SAMSUNG-E2210C/1.0 RTK-E/1.0 DF/1.0 Release/14.01.2009 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzUzNDY2MDMxMTQ0ODgw UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S105237[] = "Alcatel-OT-V570A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S105238[] = "SAMSUNG-SGH-E100/T2 UP.Browser/6.1.0.6 (GUI) MMP/1.0";
const char S105239[] = "SCH-U410T/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S105240[] = "LG-KX266/1.0 UP.Browser/6.2.3.8 CTC/1.0";
const char S105241[] = "SAMSUNG-S5320/S5320XXIK1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S105242[] = "SAMSUNG-SGH-G818E/1.0 SymbianOS/9.2 Series60/3.1 Release/1.15.2008 Browser/OSS3.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S105243[] = "Alcatel-OT-S920/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q04A";
const char S105244[] = "SCH-F299/F299DE04 POLARIS/5.30.WAP CTC/1.0";
const char S105245[] = "LG-MX100 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S105246[] = "Mozilla/5.0 (Linux; U; Android 1.5; en-; Geeksphone-One) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S105247[] = "Mozilla/5.0 (SCH-S559/S559DI05;U;REX/3.0;BREW/3.1.5;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*320;CTC/2.0) POLARIS/6.15.WEB";
const char S105248[] = "SAGEM-myX-8/1.0 UP.Browser/6.2.3.3.g.1 (GUI) MMP/2.0";
const char S105249[] = "SAMSUNG-SGH-Z310/1.0 SHP/VPP/R5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1";
const char S105250[] = "SonyEricssonZ770i/R1FA SEMC/NewsReader/1.0";
const char S105251[] = "LG-GD300s_CMCC A2000pH/2.30.10 Release/20.4.2009 Browser/Teleca/Q7.1 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105252[] = "LGE-MX300/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0";
const char S105253[] = "SonyEricssonW600c/R7B Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105254[] = "ZTE-W N61/1.0.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S105255[] = "NokiaN-GageQD/1.0 SymbianOS/6.1 Series60/1.2 Profile/MIDP-1.0 Configuration/CLDC-1.0/UC Browser7.0.0.41/26/352";
const char S105256[] = "SAMSUNG-S3500";
const char S105257[] = "Lenovo-P609/2008.12.16 R002_QCY_LANG_C_V0.1.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105258[] = "SAMSUNG-SGH-D820/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S105259[] = "SonyEricssonS312";
const char S105260[] = "NOKIA5320/UC Browser7.2.2.51/28/352";
const char S105261[] = "Mozilla/4.0 SonyEricssonW715v/R1GA SEMC/NewsReader/1.0";
const char S105262[] = "SIE-S75/40 UP.Browser/7.1.0.e.20(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.e.20 (GUI) MMP/2.0";
const char S105263[] = "SAMSUNG-SGH-D900/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S105264[] = "SAMSUNG-SGH-G600/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S105265[] = "SonyEricssonTM717/R1FC SEMC/NewsReader/1.0";
const char S105266[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) O2 Xda Zinc";
const char S105267[] = "RSSFeeds/R1A SonyEricssonM600i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105268[] = "Mozilla/5.0 (Linux; U; Android Ultimate EB28 v3.0.1; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105269[] = "SAMSUNG-B2100/B2100JBID1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0";
const char S105270[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) O2 Xda Atom Exec";
const char S105271[] = "WELLCOM-W980 WAP Browser";
const char S105272[] = "Bird.S689/K03,MN/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.6.d.3.100 (GUI) MMP/1.0";
const char S105273[] = "LG-KE858 MIC/1.1.14 MIDP-2.0/CLDC-1.1";
const char S105274[] = "Fly-V130/Profile/MIDP.2.0Configuration/CLDC.1.0";
const char S105275[] = "SAMSUNG-SGH-E778/NetFront 3.2/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105276[] = "SAMSUNG-C3310C_CMCC/1.0 RTK-E/1.0 DF/1.0 Release/11.05.2008 Browser/NetFront3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105277[] = "SAMSUNG-GT-S3030C/1.0 Release/8.25.2008 Browser/Openwave6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S105278[] = "SAMSUNG-SGH-U908E/1.0 RTK-E/1.0 DF/1.0 Release/08.03.2007 Browser/NetFront3.4  Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105279[] = "ONDA OTV1000-P107A2V1.0.0 Obigo/Q03";
const char S105280[] = "SonyEricssonGeneric A1/R8BF SEMC/NewsReader/1.0";
const char S105281[] = "Mozilla/4.0 SonyEricssonW890iv/R1FA SEMC/NewsReader/1.0";
const char S105282[] = "Nokia6680/1.0 (5.04.07) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.2.2.51/27/401";
const char S105283[] = "Nokia6235i/1.0 (S190V0200.nep) UP.Browser/6.2.3.2 MMP/2.0";
const char S105284[] = "SEC-SGHE610/1.0";
const char S105285[] = "DPD-S900C/5U-DIYPDA???? OPERA/9.5 CTC/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S105286[] = "LG-GM730/v10a WM/6.1 IEMobile/6.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105287[] = "NokiaC1-01/2.0 (04.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac1-01/UC Browser7.2.2.51/70/352";
const char S105288[] = "Mozilla/5.0 (Linux;U; Android 2.1-update1; zh-cn; ZTE-V9e Build/V9eV1.0.0B02;800*480;CTC/2.0) AppleWebkit/530.17(KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S105289[] = "Motorola_ES405B/191 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)";
const char S105290[] = "Mozilla/5.0 (Linux; U; Android Eclair BitSnacker.com v2.5; sv-se; SonyEricssonX10i Build/2.1.A.0.435) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17";
const char S105291[] = "Mozilla/5.0 (Linux; U; Android 2.1.1; en-us; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105292[] = "SAMSUNG-SGH-C278/1.0 RTK-E/1.0 DF/1.0 Release/17.08.2007 Browser/OpenWave6.2.3.3.c.1.101/*MzU0MDQ1MDMwMTk4MjUw UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S105293[] = "LG-MG185 UP.Browser/6.2.3 (GUI) MMP/1.0";
const char S105294[] = "LG-CG300 UP.Browser/6.2.3 (GUI) MMP/1.0";
const char S105295[] = "SonyEricssonJ210i/R101";
const char S105296[] = "SAMSUNG-SCH-F309/1.0 POLARIS/5.30.WAP CTC/1.0";
const char S105297[] = "Motorola-V9m UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0";
const char S105298[] = "SAGEM-myX-6/1.0 UP.Browser/6.1.0.6.1.c.4 (GUI) MMP/1.0";
const char S105299[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) i-mate 810-F/ 320x240";
const char S105300[] = "Alcatel-ELLE-N3/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C";
const char S105301[] = "Huawei/1.0/H11HW/B000 Browser/Obigo-Browser/Q04A";
const char S105302[] = "MOT-V545/0E.66.04R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S105303[] = "Nokia 6206 UP.Browser/6.2.3.9.j.1.106 (GUI) MMP/2.0";
const char S105304[] = "LG-KG296 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1";
const char S105305[] = "SEC-SGHX810/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105306[] = "SAMSUNG-SCH-W699/1.0 POLARIS/5.30.WAP CTC/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S105307[] = "PG-1400/R01 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105308[] = "SCH-W629 Infraware/5.10.CU (GUI)";
const char S105309[] = "PG-8000/R01 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105310[] = "SCH-U620/1.0 NetFront/3.0.22.2.3 (GUI) MMP/2.0";
const char S105311[] = "LG-F2300 MIC/WAP2.0 MIDP-2.0/CLDC-1.0";
const char S105312[] = "Nokia 7205 UP.Browser/6.2.3.9.j.1.106 (GUI) MMP/2.0";
const char S105313[] = "SIE-M6V/50 UP.Browser/7.0.2.2.d.3(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105314[] = "Nokia3110c/2.0 (06.02) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia3110c/UC Browser7.0.0.41/70/352";
const char S105315[] = "SAGEM-my700x/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.1.300 (GUI)";
const char S105316[] = "Nokia5800 XpressMusic/UCWEB7.5.0.66/50/999";
const char S105317[] = "NOKIA5700/UC Browser7.4.0.65/28/401";
const char S105318[] = "NokiaN73/UCWEB7.5.0.66/28/999";
const char S105319[] = "Nokia8910/1.0 (04.02)";
const char S105320[] = "NOKIA6110/UC Browser7.4.0.65/28/352";
const char S105321[] = "NokiaC5-00/UCWEB7.6.0.75/28/401";
const char S105322[] = "samsung-gt-b7722";
const char S105323[] = "LG-KG90N MIC/1.1.14 MIDP-2.0/CLDC-1.1";
const char S105324[] = "lg-c100";
const char S105325[] = "Nokia5250/UC Browser7.4.0.65/50/355";
const char S105326[] = "Nokia5238/SymbianOS/9.1 Series60/3.0";
const char S105327[] = "Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8530/1.0; U; Bada/1.2; vi-vn) AppleWebKit/533.1 (KHTML";
const char S105328[] = "TwitterAndroid/2.0.1 (122) GT-I9000/8 (samsung;GT-I9000;samsung;GT-I9000;)";
const char S105329[] = "lg-gu220";
const char S105330[] = "Moto-Morrison/1.0";
const char S105331[] = "Nokia 5320/UCWEB7.6.0.75/28/800";
const char S105332[] = "samsung-gt-s3370";
const char S105333[] = "NOKIAE61i/UC Browser7.6.1.82/28/352";
const char S105334[] = "NOKIA5700/UC Browser7.6.1.82/28/352";
const char S105335[] = "SamsungE708 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S105336[] = "SAMSUNG-SGH-S500/SHARK UP.Browser/5.0.5.1 (GUI)";
const char S105337[] = "NokiaE52-1/UCWEB7.8.0.87/28/999";
const char S105338[] = "HTC_Touch_HD_T8282 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB7.8.0.87/31/999";
const char S105339[] = "LG-KG70 MIC/1.1.14 MIDP-2.0/CLDC-1.1";
const char S105340[] = "samsung-gt-s5253/UC Browser7.7.1.88/69/444";
const char S105341[] = "SonyEricssonU1/UC Browser7.8.0.95/50/351";
const char S105342[] = "Nokia N96/UC Browser7.8.0.95/28/352";
const char S105343[] = "Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303classic/UC Browser7.8.0.95/70/351";
const char S105344[] = "SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1 sonyericssonw20/UC Browser7.7.1.88/70/444";
const char S105345[] = "Nokia6303iclassic/5.0 (07.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303iclassic/UC Browser7.2.2.51/70/351";
const char S105346[] = "gt-s5233s/UC Browser7.4.0.65/70/351";
const char S105347[] = "Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7210supernova/UC Browser7.8.0.95/70/355";
const char S105348[] = "SAMSUNG-SGH-F218/1.0 Release/7.25.2007 Browser/NetFront3.4/*MzU4MjIwMDEwMTYwOTAz";
const char S105349[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-I987 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105350[] = "HTC_Touch_Diamond2_T5353 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)";
const char S105351[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105352[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 1.5; zh-cn; ME600 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1";
const char S105353[] = "Smartone/SharpSX813/SHS001/1.0 Browser/UP.Browser/7.0.2.1a.1.c.1.103 (GUI) Profile/MIDP-2.0 Configuration/CLDC-1.1 Ext-J-Profile/JSCL-1.2.2 Ext-V-Profile/VSCL-2.0.0";
const char S105354[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Nexus S 4G Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105355[] = "HPiPAQhw6900/1.0/Mozilla/4.0";
const char S105356[] = "Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; MotoA953 Build/MILS2_U6_2.2.16; 480*854) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/800";
const char S105357[] = "AUDIOVOX-SMT5600";
const char S105358[] = "BlackBerry7520/4.0.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105359[] = "BlackBerry7280/4.0.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105360[] = "Nokia3600/1.0 SymbianOS/6.1 Series60/0.9 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S105361[] = "Nokia7900/2.0 (nd3.30_p) Profile/MIDP-2.1 Configuration/CLDC-1.1";
const char S105362[] = "NokiaE65-1";
const char S105363[] = "SIE-SP65/47 UP.Browser/7.0.2.2.d.1(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105364[] = "SCH-A850 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S105365[] = "SAMSUNG-SGH-C400/1.0";
const char S105366[] = "SCH-W579 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S105367[] = "SAMSUNG-SGH-X636/1.0";
const char S105368[] = "SCH-N330 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S105369[] = "SAMSUNG-SGH-E750/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0";
const char S105370[] = "SAMSUNG-SGH-E820/1.0 UP.Browser/6.2.2.6 (GUI) MMP/1.0";
const char S105371[] = "SAMSUNG-SGH-X481/1.0";
const char S105372[] = "MOT-C975";
const char S105373[] = "Motorola-E365 UP.Browser/6.1.0.7.3 (GUI) MMP/1.0";
const char S105374[] = "SAGEM-my202C/Orange1.0 UP.Browser/5.0.5.6 (GUI)";
const char S105375[] = "SAGEM-my300X/1.0/ MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.g.2.106 (GUI) MMP/2.0";
const char S105376[] = "SAGEM-myX-5-2T/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.6.d.4.102 (GUI) MMP/1.0";
const char S105377[] = "SAGEM-myW-7/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105378[] = "SCH-A970 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S105379[] = "SonyEricssonW700c";
const char S105380[] = "LG-C1300 UP.Browser/6.2.2 (GUI) MMP/1.0";
const char S105381[] = "LG-C3100 AU/4.10 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S105382[] = "LG-G1600 AU/4.10 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S105383[] = "LG-G1610 AU/4.10 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S105384[] = "LG-G7100 AU/4.10 Profile/MIDP-1.0 Configuration/CLDC-1.0";
const char S105385[] = "NEC-e353/001.00 ACS-NF/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Qtv/1.0";
const char S105386[] = "PHILIPS859 ObigoInternetBrowser/2.0";
const char S105387[] = "BenQ-S80/1.00/WAP2.0/MIDP2.0/CLDC1.1 UP.Browser/6.3.0.1.119 (GUI) MMP/2.0";
const char S105388[] = "PANTECH G300";
const char S105389[] = "PT-G700";
const char S105390[] = "NEC-e121/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.3.9.d.1 (GUI) MMP/2.0";
const char S105391[] = "PG-3500/R01 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105392[] = "PG-C120/R01 MIC/1.1.14 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105393[] = "O2-X2i/6.1.0.7 UP.Browser/6.1.0.7.8.c.1.103 (GUI) MMP/1.0";
const char S105394[] = "portalmmm/2.0 SG344i(c10;TB)";
const char S105395[] = "portalmmm/2.0 S410i(c10;TB)";
const char S105396[] = "Mozilla/4.0 (compatible; MSIE 5.0; Symbian OS; SonyEricsson P900; 316) Opera 6.31 [en]";
const char S105397[] = "Mozilla/4.0 (compatible; MSIE 5.0; Symbian OS; SonyEricsson P910; 318) Opera 6.31 [en]";
const char S105398[] = "SAMSUNG-SGH-I520";
const char S105399[] = "SAMSUNG-SGH-I750";
const char S105400[] = "FLY-2040i/BSI AU.Browser/2.0 QO3C1 MMP/1.0";
const char S105401[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.0) NetFront/3.3";
const char S105402[] = "Vodafone/1.0/SamsungSGHX510V/X510BUFK3 Browser/Nokia/3.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.0";
const char S105403[] = "T600; V3.02.E8 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) PPC; 480x640; GIGABYTE-g-Smart";
const char S105404[] = "HTC-8900/1.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S105405[] = "HTC_TyTN_II Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S105406[] = "Xda_nova; 240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)";
const char S105407[] = "SCH-R510 UP.Browser/6.2.3.8 (GUI) MMP/2.0";
const char S105408[] = "Mozilla/5.0 (itouch; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1A538a Safari/419.3";
const char S105409[] = "Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux armv6l; U) Opera 8.5 [en_US] Maemo browser 0.7.6 RX-34_2007SE_2.2006.51-6";
const char S105410[] = "SonyEricssonP700i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 513) Opera 8.65 [en]";
const char S105411[] = "BlackBerry9100/4.6.0.31 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100";
const char S105412[] = "LGE-LG830 AU-OBIGO/Q05A1-3.8 MMP/2.0";
const char S105413[] = "MOT-Q8_CMCC/1.0 Release/01.10.2007 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/WM6.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S105414[] = "KDDI-CA35 UP.Browser/6.2.0.11.1.2 (GUI) MMP/2.0";
const char S105415[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Xda Graphite/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105416[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) HTCS620";
const char S105417[] = "Vodafone/1.0/HTC_VPACompactIV/5.4.172.1/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; MIDP-2.0 Configuration/CLDC-1.1; PPC; 240x320)";
const char S105418[] = "LG-B2100 Obigo/WAP2.0 MIDP-2.0/CLDC-1.0";
const char S105419[] = "LGE-LG200/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S105420[] = "Samsung-SPHM620 AU-OBIGO/Q04C1-1.22 MMP/2.0";
const char S105421[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) HTC/X03HT";
const char S105422[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) 320x240; VZW; Motorola-A4500; Windows Mobile 6.1 Standard";
const char S105423[] = "SoftBank/1.0/706SC/SCJ001 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105424[] = "LGE-MX240/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S105425[] = "SAMSUNG-SGH-i688_CMCC/1.0 Release/03.07 Browser/IE6 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)";
const char S105426[] = "KDDI-SA39 UP.Browser/6.2.0.12.1.4 (GUI) MMP/2.0";
const char S105427[] = "KDDI-CA34 UP.Browser/6.2.0.10.3.3 (GUI) MMP/2.0";
const char S105428[] = "Qtek8020 (Mozilla\\4.0 compatible; MSIE 4.01; Windows CE; Smartphone; 176x220)";
const char S105429[] = "SCH-A915 UP.Browser/6.2.3.2 (GUI) MMP/2.0";
const char S105430[] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) HTC-SMT5800";
const char S105431[] = "Samsung-SPHA640 AU-MIC/1.1.12.0 MMP/2.0";
const char S105432[] = "Mozilla/5.0 (Linux; U; Android 1.5; GT-I7500) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S105433[] = "SonyEricssonA195i/R1AA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.0";
const char S105434[] = "SAGEM-my721X/1.0/EAN_ET MIDP-2.0 Configuration/CLDC-1.L Browser/UP.Browser/7.2.7.1.352 (GUI)";
const char S105435[] = "Mozilla/5.0 (Linux; U; Android 1.5; pl-pl; Era G1 Build/CRC1) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-20090728)";
const char S105436[] = "SAGEM-my202/1.0 UP.Browser/6.1.0.6.1.c.5 (GUI) MMP/1.0";
const char S105437[] = "Spice S705";
const char S105438[] = "Mozilla/5.0 (Linux; U; Android 1.5; zh-cn; dopod A6288 Build/CUPCAKE) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S105439[] = "Mozilla/5.0 (Linux; U; Android 1.5; zh-cn; MT710 Build/CUPCAKE) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2";
const char S105440[] = "Mozilla/5.0 (Series40; NokiaX3-02/05.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.0.9.10";
const char S105441[] = "Opera/9.5 (Microsoft Windows; PPC; 640x480; HTC_X7500/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mobi/2745; U; en)";
const char S105442[] = "mauie800";
const char S105443[] = "samsung-gt-s3650";
const char S105444[] = "Mozilla/5.0 (compatible; Konqueror/3.5; Darwin 8.10.0; X11; Power Macintosh; en_US)KHTML/3.5.6 (like Gecko)";
const char S105445[] = "SonyEricssonW710i/R1JC Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S105446[] = "SonyEricssonC510/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1 sonyericssonc510/UC Browser7.7.1.88/70/351";
const char S105447[] = "sonyericssonk530i/UC Browser7.8.0.95/70/351";
const char S105448[] = "SonyEricssonW910i/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1 sonyericssonw910i/UC Browser7.9.0.102/70/352";
const char S105449[] = "Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.6.1";
const char S105450[] = "Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 VertuConstellationQuest/051.719.2; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.2 3gpp-gba";
const char S105451[] = "Mozilla/5.0 (compatible; en-US; RGAnalytics/1.0; )";
const char S105452[] = "MQQBrowser/2.9/Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105453[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Desire Z Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S105454[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; sv-se; ST17i Build/4.0.2.A.0.42; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105455[] = "MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT319 Build/V4.37B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105456[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I5801 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105457[] = "Dalvik/1.4.0 (Linux; U; Android 2.3.7; U8500 Build/GRK39F)";
const char S105458[] = "YahooMobileMessenger/1.0 (Android Mail; 1.3.9) (GT-I9100; samsung; GT-I9100; 2.3.4/GINGERBREAD)";
const char S105459[] = "MQQBrowser/2.9/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105460[] = "MQQBrowser/25 Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HUAWEI T8300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105461[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; U8120 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105462[] = "TuneIn Radio/5.2 (Android 10; SGH-I727R; Java)";
const char S105463[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/1.9.1 Safari/533.1";
const char S105464[] = "Logos/0.9.3 (Linux; U; Android 2.2.1; en_US; PRS-T1 Build/FRG83)";
const char S105465[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Sensation 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105466[] = "samsung-gt-s8530";
const char S105467[] = "Mozilla/5.0 (BlackBerry; U; BlackBerry 8520; en-US) AppleWebKit/534.1+ (KHTML, like Gecko) Version/6.0.0.141 Mobile Safari/534.1+";
const char S105468[] = "WebProcess/7534.48.3 CFNetwork/520.0.13 Darwin/11.1.0 (x86_64) (MacBookPro8%2C2)";
const char S105469[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Kyros MID7016 GB 8K Build/GRK39F; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105470[] = "JUC(Linux;U;Android2.3.3;Zh_cn;MID7010;480*800;)UCWEB7.8.0.95/139/444";
const char S105471[] = "JUC (Linux; U; 2.3.5; zh-cn; MEIZU MX; 640*960) UCWEB8.2.3.141/145/999";
const char S105472[] = "JUC (Linux; U; 2.3.5; zh-cn; MEIZU MX; 640*960) UCWEB8.3.2.156/145/800";
const char S105473[] = "iBrowser/Mini2.4 (SAMSUNG-GT-C3312)";
const char S105474[] = "Nokia5800 XpressMusic/UCWEB7.4.0.57/50/999";
const char S105475[] = "NokiaN97 mini/UCWEB7.5.0.66/50/999";
const char S105476[] = "NokiaE66/UCWEB7.5.0.66/28/800";
const char S105477[] = "NokiaN97/UCWEB7.8.0.87/50/999";
const char S105478[] = "NokiaX3-00/5.0 (08.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax3-00/UC Browser7.8.0.95/70/351";
const char S105479[] = "samsung-gt-s8500/UC Browser7.9.0.102/69/352";
const char S105480[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Sensation Z710e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Mobile Maxthon";
const char S105481[] = "Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.1.8 Safari/534.34";
const char S105482[] = "Dolphin/4.0.1 CFNetwork/548.1.4 Darwin/11.0.0";
const char S105483[] = "contype chromeframe/18.0.1025.162";
const char S105484[] = "Dolphin 4.0.1 (iPhone; iPhone OS 5.1; en_US)";
const char S105485[] = "Opera/9.80 (Windows NT 5.1; Opera Tablet/23731; U; en) Presto/2.9.201 Version/11.50";
const char S105486[] = "Opera/9.80 (Windows NT 6.0; Opera Tablet/23731; U; en) Presto/2.9.201 Version/11.50";
const char S105487[] = "Opera/9.80 (Windows NT 6.0; Opera Tablet/11648; U; en) Presto/2.7.81 Version/11.00";
const char S105488[] = "Mozilla/5.0 (CrOS; iPhone) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1071.0 Mobile Safari/536.3";
const char S105489[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xperia Ray Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S105490[] = "Mozilla/5.0 (DirectFB; U; Linux i686; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.1.4(+mouse+3D+SCREEN+TUNER; LGE; GLOBAL-PLAT3; 00.03.01; 0xX;); LG NetCast.TV-2011";
const char S105491[] = "Mozilla/5.0 (DirectFB; U; Linux 35230; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.1.4(+SCREEN+TUNER; LGE; GLOBAL-PLAT3; 04.02.28; 0xX;); LG NetCast.TV-2011";
const char S105492[] = "Mozilla Compatible/5.0 (iPhone/iPad; Mxit Mobi/5.9/1.8.5.168; U;) Opera Mini/3.1";
const char S105493[] = "Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]";
const char S105494[] = "Opera/9.80 (iPhone; Opera Mini/7.2.37083/27.1667; U; es) Presto/2.8.119 Version/11.10";
const char S105495[] = "Dolphin/4.1 CFNetwork/548.0.4 Darwin/11.0.0";
const char S105496[] = "Dolphin/3.5 CFNetwork/485.13.9 Darwin/11.0.0";
const char S105497[] = "Dolphin 3.5.1 (iPod touch; iPhone OS 5.0.1; en_US)";
const char S105498[] = "Dolphin 4.0.1 (iPhone; iPhone OS 5.0.1; en_US)";
const char S105499[] = "Dolphin/1.0.1 CFNetwork/485.13.9 Darwin/11.0.0";
const char S105500[] = "Dolphin HD 4.0 (iPad; iPhone OS 4.3.3; en_US)";
const char S105501[] = "Dolphin/5.0 CFNetwork/548.1.4 Darwin/11.0.0";
const char S105502[] = "Dolphin 4.0.1 (iPhone; iPhone OS 4.3.5; fi_FI)";
const char S105503[] = "Dolphin HD 5.0 (iPad; iPhone OS 5.0.1; en_US@calendar=buddhist)";
const char S105504[] = "Dolphin 3.5.1 (iPhone; iPhone OS 4.3.3; en_US)";
const char S105505[] = "Dolphin HD 5.1 (iPad; iPhone OS 5.0.1; zh_TW)";
const char S105506[] = "Dolphin 3.5.1 (iPhone; iPhone OS 5.1.1; en_US)";
const char S105507[] = "samsung-gt-s5380k";
const char S105508[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I717 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105509[] = "Mozilla/5.0 (PlayStation Vita 1.65) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2";
const char S105510[] = "Knott's 1.0 rv:54 (iPod touch; iPhone OS 4.1; en_US)";
const char S105511[] = "NOKIA Lumia 800/1750.0805.8773.12220";
const char S105512[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-74)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105513[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC One X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, gb";
const char S105514[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Ultimate)";
const char S105515[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Mazaa)";
const char S105516[] = "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; P4301)";
const char S105517[] = "Opera/9.80 (X11; Linux i686; U; Linux Mint; en) Presto/2.8.131 Version/11.11";
const char S105518[] = "Safari/7534.48.3 CFNetwork/520.0.13 Darwin/11.1.0 (x86_64) (MacBookPro3%2C1)";
const char S105519[] = "Nokia 5320/UCWEB8.1.0.104/28/800";
const char S105520[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.7.1";
const char S105521[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST18i Build/4.1.B.0.431) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S105522[] = "contype chromeframe/20.0.1132.57";
const char S105523[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/ Sleipnir/1.8.3m";
const char S105524[] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1 Safari/7534.51.22 Sleipnir/3.6";
const char S105525[] = "JUC(Linux;U;Android1.6;Zh_cn;X10a;480*854;)UCWEB7.8.0.95/139/355";
const char S105526[] = "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105527[] = "JUC(Linux;U;Android2.3.5;Zh_cn;GT-I9103;480*800;)UCWEB7.8.0.95/139/351";
const char S105528[] = "Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I5801 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S105529[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sony Tablet S Build/TISU0077)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105530[] = "Mozilla/5.0 (Linux; U; Android 2.3.3; en_us ; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S105531[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-I9000 Build/IMM76I; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S105532[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC One V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, gb";
const char S105533[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB865 Build/5.5.1-175_EDMR1.25)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105534[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S105535[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; HTC Wildfire Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S105536[] = "Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; Dell Streak Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S105537[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.6/135,1.9.6.135,2012-08-19 13:37, gb";
const char S105538[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Evo Shift Build/GRI40; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S105539[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; SGH-T959V Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S105540[] = "Mozilla/5.0 (Linux; Android 4.0.4; Incredible Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S105541[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I777 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S105542[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; Micromax_A70 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S105543[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; P4301)";
const char S105544[] = "Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; Acer; Allegro)";
const char S105545[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; Amazon Kindle Fire Build/JRO03L; CyanogenMod-10)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S105546[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Amazon Kindle Fire Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:34, in";
const char S105547[] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1 slurp";
const char S105548[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Ainovo Flame (Nexus 7) Build/JRO03D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S105549[] = "Logos/2.0.9 (Linux; U; Android 4.1.1; en-GB; Nexus 7 Build/aokp_grouper-userdebug 4.1.1 JRO03H eng.root.X.X test-keys)";
const char S105550[] = "JUC (Linux; U; 4.1.1; zh-cn; Nexus 7; 800*1205) UCWEB8.7.0.187/145/999";
const char S105551[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-us; Nexus 7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105552[] = "Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/58.0.1025.166  Safari/535.19";
const char S105553[] = "MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Nexus 7 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S105554[] = "Mozilla/5.0 (Linux; Android; Nexus 7 Build/JRO03D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Safari/535.19";
const char S105555[] = "1PasswordThumbs1 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac7%2C1)";
const char S105556[] = "Opera/9.80 (Macintosh; Intel Mac OS X; Opera Mobi/14316; U; en) Presto/2.7.81 Version/11.00";
const char S105557[] = "nokia6630/UC Browser7.9.0.102/69/352";
const char S105558[] = "NokiaC6-01/UCWEB8.1.0.104/51/800";
const char S105559[] = "MQQBrowser/2.4 (E63-1;SymbianOS/9.1 Series60/3.0)";
const char S105560[] = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.4.1 Safari/534.34";
const char S105561[] = "Mozilla/5.0 (Java; U; en-us; lg-gu220) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile";
const char S105562[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; s5230) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S105563[] = "Nokia205/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia205) UCBrowser8.3.1.161/70/465/UCWEB Mobile";
const char S105564[] = "Mozilla/5.0 (Java; U; en-us; lg-lg900g) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/405/UCWEB Mobile";
const char S105565[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia3230) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S105566[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s7233e) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S105567[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTCEVOV4G Build/unknown) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S105568[] = "Mozilla/5.0 (Linux; Android 4.1.2; ZTE V988 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S105569[] = "Mozilla/5.0 (Linux; Android 4.1.2; PadFone Infinity Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S105570[] = "SAMSUNG-GT-S5292 SAMSUNG-GT-S5292";
const char S105571[] = "SAMSUNG-GT-S3802 Opera/9.80 (J2ME/MIDP; Opera Mini/7.0.32562/28.3692; U; en) Presto/2.8.119 Version/11.10";
const char S105572[] = "Android OS/3.1 HTC_HTC-P715a";
const char S105573[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SX-SP715A Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105574[] = "Rdio.Net; Android/2.4.9; Android/3.1; DM=HTC; DN=HTC-P715a; DC=rogers_wwe; DF=rogers_wwe/htc_puccinilte/puccinilte:3.1/HMJ15/X.4:user/release-keys; DH=verdilte";
const char S105575[] = "Mozilla/5.0 (Linux; U; Android 2.3.4; en_us ; SX-SP715A Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S105576[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-ca; HTC-P715a Build/HMJ15) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.16/276,1.9.16.276,2012-12-21 16:37, -0500, ca";
const char S105577[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; SX-SP715A Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355";
const char S105578[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; zte-p107a20(propus)) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S105579[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6680) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S105580[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S105581[] = "MQQBrowser/3.7/Adr (Linux; U; 4.1.2; zh-cn; Lenovo S920 Build/S820_S101_X;720*1280)";
const char S105582[] = "Nokia5300/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia5300) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S105583[] = "Windows Phone Search (Windows Phone OS 7.10;LG;LG-E906;7.10;8862)";
const char S105584[] = "Windows Phone Search (Windows Phone OS 7.10;ZTE;Tania;7.10;8862)";
const char S105585[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; LAVA iris 351 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/405";
const char S105586[] = "iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LAVA iris 351 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105587[] = "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; LAVA iris 351 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/405";
const char S105588[] = "JUC(Linux;U;Android2.3.5;Zh_cn;LAVA iris 351;320*480;)UCWEB7.8.0.95/139/444";
const char S105589[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; GT-I9200 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S105590[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-I9200 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/352";
const char S105591[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9152 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +0100, pl";
const char S105592[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; Maya M1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S105593[] = "MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105594[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, id";
const char S105595[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7500 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.2.139/145/355";
const char S105596[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7510 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355";
const char S105597[] = "TuneIn Radio/5.1 (Android 10; GT-P1000; Java)";
const char S105598[] = "Mozilla/5.0 (Linux; U; Android 2.1; es-us; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105599[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7300 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S105600[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 MobileSafari/534.13 DolphinHDCN/7.0.0";
const char S105601[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6210 Build/HTJ85B)Maxthon AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105602[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P6810 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, fr";
const char S105603[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7501 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355";
const char S105604[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S105605[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105606[] = "JUC(Linux;U;Android3.2;Zh_cn;GT-P7310;800*1232;)UCWEB7.8.0.95/139/352";
const char S105607[] = "YahooMobile/1.0 (mail; 1.4.5.12736); (Linux; U; Android 3.1; GT-P7300 Build/HONEYCOMB_MR1);";
const char S105608[] = "Mozilla/5.0 (Linux; U; Android 2.1; pt-br; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105609[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105610[] = "TuneIn Radio/4.1 (Android 10; GT-P1000; Java)";
const char S105611[] = "Instagram 3.0.5 Android (15/4.0.3; 160dpi; 600x976; samsung; GT-P3110; espressowifi; espresso; en_GB)";
const char S105612[] = "GoogleEarth/7.0.0000.7386(Android;Android (GT-P7500-GT-P7500-user-3.1);it-IT;kml:2.2;client:Free;type:default)";
const char S105613[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; GT-P6200 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7";
const char S105614[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105615[] = "MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GT-P6810 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1";
const char S105616[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105617[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7510 Build/HTJ85B; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.4.145/139/800";
const char S105618[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; GT-P6800 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, br";
const char S105619[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P3108 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S105620[] = "Mozilla/5.0 (Linux; U; Android 3.2; ar-ae; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, sa";
const char S105621[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105622[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105623[] = "Mozilla/5.0 (Linux; Android 10.2.1.5; GT-P5100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S105624[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; ms-my; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105625[] = "TuneIn Radio/6.5 (Android 15; GT-P5100; Java)";
const char S105626[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P7310 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S105627[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; GT-P7310 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, cz";
const char S105628[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6210 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105629[] = "YahooMobile/1.0 (mail; 1.4.5.12736); (Linux; U; Android 3.2; GT-P7300 Build/HONEYCOMB_MR2);";
const char S105630[] = "GoogleEarth/6.2.0001.6363(Android;Android (GT-P7510-GT-P7510-user-4.0.4);en-US;kml:2.2;client:Free;type:default)";
const char S105631[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105632[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6810 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7";
const char S105633[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.8.4/104,1.8.4.104,2012-06-25 20:49, de";
const char S105634[] = "MQQBrowser/2.8 Mozilla/5.0 (Linux; U; Android 4.0.3; ar-ae; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105635[] = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2a1pre) Gecko/X Firefox/3.8a1pre AppEngine CirruxCache CoralWebPrxMozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30,gzip(gfe) (via docs.google.com/viewer)";
const char S105636[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; GT-P7500 Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105637[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7310 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S105638[] = "Instagram 3.2.0 Android (8/2.2; 240dpi; 600x1024; samsung; SGH-T849; SGH-T849; GT-P1000; en_US)";
const char S105639[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105640[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76D)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105641[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, zm";
const char S105642[] = "MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105643[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P7300 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31";
const char S105644[] = "MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105645[] = "MQQBrowser/20 Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105646[] = "MQQBrowser/2.7 Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105647[] = "Instagram 3.2.0 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7310; GT-P7310; p3; en_US)";
const char S105648[] = "MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105649[] = "Android_CellLocation_2.0.X_KjDdHjNlNlNfLpObCmOoGkJcIjMbDcIi@%E4%B8%96%E7%BA%AA%E4%BD%B3%E7%BC%98_3.11;X;X;samsung;GT-P3100;15";
const char S105650[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P3110 Build/IMM76D)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105651[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105652[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P7510 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S105653[] = "TuneIn Radio/6.3 (Android 12; GT-P7500M; Java)";
const char S105654[] = "Rdio.Net; Android/2.2.11; Android/4.0.4; DM=samsung; DN=GT-P7500R; DC=samsung; DF=samsung/GT-P7500R/GT-P7500R:4.0.4/IMM76D/RUXLH2:user/release-keys; DH=p3";
const char S105655[] = "MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105656[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6201 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105657[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105658[] = "MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S105659[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-gb; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105660[] = "YahooMobileMessenger/1.0 (Android Mail; 1.4.6) (GT-P1010; samsung; GT-P1010; 2.3.6/GINGERBREAD)";
const char S105661[] = "MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105662[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7501 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105663[] = "MQQBrowser/25 Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105664[] = "JUC(Linux;U;Android4.0.3;Zh_cn;GT-P5110;800*1232;)UCWEB7.8.0.95/139/444";
const char S105665[] = "TuneIn Radio/6.7 (Android 15; GT-P5113; Java)";
const char S105666[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P3200 Build/Smartfren.androtab.v25.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105667[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, gb";
const char S105668[] = "TuneIn Radio/6.6 (Android 15; GT-P7500R; Java)";
const char S105669[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P1000 Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105670[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6810 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S105671[] = "iBrowser/2.8 Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105672[] = "TuneIn Radio/6.7 (Android 15; GT-P7510; Java)";
const char S105673[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57";
const char S105674[] = "TuneIn Radio Pro/6.6 (Android 10; GT-P1000M; Java)";
const char S105675[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105676[] = "Rdio.Net; Android/2.1.4; Android/3.2; DM=samsung; DN=GT-P7500R; DC=samsung; DF=samsung/GT-P7500R/GT-P7500R:3.2/HTJ85B/UXKK2:user/release-keys; DH=p3";
const char S105677[] = "TuneIn Radio/6.7 (Android 15; GT-P3113; Java)";
const char S105678[] = "Logos/2.2.7 (Linux; U; Android 4.1.2; en-NZ; GT-P7510 Build/aokp_p4wifi-userdebug 4.1.2 JZO54K eng.trip.X.X test-keys)";
const char S105679[] = "Instagram 3.1.0 Android (15/4.0.4; 160dpi; 800x1232; samsung; GT-P5113; espresso10wifi; espresso10; en_CA)";
const char S105680[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105681[] = "Rdio.Net; Android/2.4.2; Android/2.2; DM=samsung; DN=GT-P1000M; DC=samsung; DF=samsung/GT-P1000M/GT-P1000M/GT-P1000M:2.2/FROYO/UGJJ7:user/release-keys; DH=GT-P1000";
const char S105682[] = "YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 4.0.4; GT-P6810 Build/Unknown);";
const char S105683[] = "TuneIn Radio Pro/6.3 (Android 15; GT-P7510; Java)";
const char S105684[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6201 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355";
const char S105685[] = "TuneIn Radio Pro/6.7 (Android 15; GT-P3113; Java)";
const char S105686[] = "JUC(Linux;U;Android4.0.4;Zh_cn;GT-P7510;800*1232;)UCWEB7.8.0.95/139/351";
const char S105687[] = "Orca-Android/2.2.2-release OS/3.2 Model/GT-P6210 VersionCode/88371 Product/Messenger Carrier/null Manufacturer/samsung Brand/samsung";
const char S105688[] = "Instagram 3.3.2 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7310; GT-P7310; p3; en_GB)";
const char S105689[] = "TuneIn Radio/6.6 (Android 15; GT-P3113; Java)";
const char S105690[] = "TuneIn Radio/5.2 (Android 12; GT-P7500R; Java)";
const char S105691[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6810 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355";
const char S105692[] = "JUC(Linux;U;Android4.0.4;Zh_cn;GT-P7300;800*1232;)UCWEB7.8.0.95/139/355";
const char S105693[] = "TuneIn Radio/6.7 (Android 13; GT-P7510; Java)";
const char S105694[] = "Orca-Android/2.1.2-release OS/3.2 Model/GT-P6800 VersionCode/78596 Product/Messenger Carrier/MY_MAXIS";
const char S105695[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10";
const char S105696[] = "MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 3.2; fa-fa; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105697[] = "Instagram 3.1.0 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P3113; espressowifi; espresso; en_CA)";
const char S105698[] = "TuneIn Radio/6.3 (Android 15; GT-P5113; Java)";
const char S105699[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7310 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13";
const char S105700[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3113 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3113, 38, 1.8.1)";
const char S105701[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]";
const char S105702[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-09 19:03, us";
const char S105703[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 3.2; en-au; GT-P6810 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105704[] = "Mozilla/5.0 (Linux; U; Android 2.1; en-gb; GT-P7310 Build/ERD62) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S105705[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7500 Build/IMM76D; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/355";
const char S105706[] = "TuneIn Radio Pro/6.6 (Android 15; GT-P5113; Java)";
const char S105707[] = "Rdio.Net; Android/2.4.4; Android/4.0.4; DM=samsung; DN=GT-P5113; DC=samsung; DF=samsung/espresso10wifibby/espresso10wifi:4.0.4/IMM76D/P5113UEBLH3:user/release-keys; DH=espresso10";
const char S105708[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P3110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355";
const char S105709[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GT-P3110 Build/JRO03C) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7";
const char S105710[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P5113 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105711[] = "Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P2000 Build/IMM76D)";
const char S105712[] = "TuneIn Radio/6.5 (Android 15; GT-P3113; Java)";
const char S105713[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P1000 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105714[] = "Android_CellLocation_2.0.com.qihoo360.mobilesafe;null;null;samsung;GT-P7310;15";
const char S105715[] = "Evernote Android/AndroidMarket_442_189 (en_US); Android/4.0.4; GT-P6810/15;";
const char S105716[] = "Instagram 3.3.0 Android (15/4.0.4; 160dpi; 800x1232; samsung; GT-P7320; GT-P7320; qcom; en_US)";
const char S105717[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105718[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0000, gb";
const char S105719[] = "iBrowser/3.0/Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105720[] = "iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105721[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7501 Build/IMM76L; CyanogenMod-Ganbarou V1.00 GT) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +0100, de";
const char S105722[] = "eHarmony-Android/2.1.8 (GT-P3113; Android OS 4.1.1; en_CA; id null)";
const char S105723[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105724[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; GT-P5110 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0100, nl";
const char S105725[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105726[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105727[] = "TuneIn Radio/6.7 (Android 16; GT-P3113; Java)";
const char S105728[] = "TuneIn Radio Pro/6.8 (Android 16; GT-P3113; Java)";
const char S105729[] = "TuneIn Radio/6.7 (Android 16; GT-P5113; Java)";
const char S105730[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7500 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S105731[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-tw; GT-P7510 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S105732[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; GT-P1000 Build/MIUI-XJ) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S105733[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-tw; GT-P7300 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S105734[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105735[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105736[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]";
const char S105737[] = "Orca-Android/2.2.5-release OS/4.1.1 Model/GT-P3110 VersionCode/X Product/Messenger Carrier/null Manufacturer/samsung Brand/samsung";
const char S105738[] = "TuneIn Radio/6.1 (Android 13; GT-P7510; Java)";
const char S105739[] = "TuneIn Radio/6.3 (Android 13; GT-P7500R; Java)";
const char S105740[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7501 Build/JOP40D; CyanogenMod-Ganbarou V3.00 GT-P7501) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105741[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105742[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P7500 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444";
const char S105743[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; GT-P3100 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355";
const char S105744[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-P1010 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, es";
const char S105745[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; GT-P5100 Build/JRO03C) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S105746[] = "MQQBrowser/3.3/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105747[] = "Rdio.Net; Android/2.4.9; Android/3.1; DM=samsung; DN=GT-P7500R; DC=samsung; DF=samsung/GT-P7500R/GT-P7500R:3.1/HMJ37/UXKH4:user/release-keys; DH=p3";
const char S105748[] = "Orca-Android/2.2.1-release OS/3.1 Model/GT-P7300 VersionCode/87403 Product/Messenger Carrier/MY_MAXIS Manufacturer/samsung Brand/samsung";
const char S105749[] = "com.kakao.talk/Android Version/2.5.0 Model/GT-P1000R MNC/X";
const char S105750[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P5110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1";
const char S105751[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7500 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7500, 39, 1.8.2)";
const char S105752[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105753[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1010 Build/FROYO; 600*1024) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/352";
const char S105754[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259";
const char S105755[] = "YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 4.0.4; GT-P1000 Build/Unknown);";
const char S105756[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P6800 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31";
const char S105757[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GT-P7310 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.20/286,1.9.20.286,2013-01-25 13:38, +0800, tw";
const char S105758[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105759[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105760[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7500 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S105761[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-P1000 Build/MIUI-XJ) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000";
const char S105762[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6201 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105763[] = "TuneIn Radio Pro/6.1 (Android 15; GT-P7510; Java)";
const char S105764[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; GT-P7500 Build/JOP40G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S105765[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; GT-P1000 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]";
const char S105766[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3110 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105767[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S105768[] = "TuneIn Radio Pro/5.2 (Android 15; GT-P7510; Java)";
const char S105769[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; pl-pl; GT-P7300 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0100, pl";
const char S105770[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P3100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31";
const char S105771[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; uk-ua; GT-P6200 Build/IMM76D)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105772[] = "Mozilla/5.0 (Linux; U; Android 2.3.1; en-us ; GT-P7510 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S105773[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; GT-P5100 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/352";
const char S105774[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-P1000 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444";
const char S105775[] = "com.eurosport.news.android/3.1.3/Android/4.0.3/samsung-GT-P3110";
const char S105776[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105777[] = "Rdio.Net; Android/2.4.10; Android/3.2; DM=samsung; DN=GT-P7500R; DC=samsung; DF=samsung/GT-P7500R/GT-P7500R:3.2/HTJ85B/UXKK2:user/release-keys; DH=p3";
const char S105778[] = "Instagram 3.0.5 Android (15/4.0.4; 160dpi; 800x1232; samsung; GT-P5113; espresso10wifi; espresso10; en_CA)";
const char S105779[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 3.2; fa-fa; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105780[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P5113, 40, 1.8.5)";
const char S105781[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3113 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S105782[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105783[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6211 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S105784[] = "MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 MobileSafari/534.13";
const char S105785[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105786[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3113 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, -0600, us";
const char S105787[] = "JUC(Linux;U;Android3.1;Zh_cn;GT-P7510;1280*800;)UCWEB7.8.0.95/139/351";
const char S105788[] = "GoogleEarth/7.0.0002.8421(Android;Android (GT-P3100-espressorf-user-4.0.3);it-IT;kml:2.2;client:Free;type:default)";
const char S105789[] = "Mozilla/5.0 (Linux; U; Android 3.1; es-es; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.3.3000";
const char S105790[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105791[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; ru-ru; GT-P7100 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S105792[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P1000 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0700, th";
const char S105793[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P1000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S105794[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7300 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S105795[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, fr";
const char S105796[] = "Instagram 1.0.6 Android (13/3.2; 160dpi; 800x1232; samsung; GT-P7510; GT-P7510; p3; en_US)";
const char S105797[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259";
const char S105798[] = "MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 3.2; ru-ru; GT-P7310 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105799[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-P7501 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.20/286,1.9.20.286,2013-01-25 13:38, +0100, de";
const char S105800[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7510 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S105801[] = "theScore/3.1.2 Android-OS/4.0.4 (Android; Tablet, GT-P6800)";
const char S105802[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7510 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S105803[] = "Mozilla/5.0 (Linux; U; Android 3.1; en-US; GT-P7510 Build/HMJ37) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S105804[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-P5110 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259";
const char S105805[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259";
const char S105806[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; ar-eg; GT-P6800 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S105807[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7500 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315";
const char S105808[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; ru-ru; GT-P6210 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0000, ru";
const char S105809[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7510 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7510, 40, 1.8.5)";
const char S105810[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S105811[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-nz; GT-P7310 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S105812[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3110 Build/JRO03C; 1024*552) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444";
const char S105813[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105814[] = "YahooMobile/1.0 (mail; 1.3.9.10087); (Linux; U; Android 2.2.1; GT-P1010 Build/FROYO);";
const char S105815[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13 UCBrowser/2.2.2.259";
const char S105816[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-CN; GT-P7500 Build/HMJ37) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31";
const char S105817[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3100 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105818[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5110 Build/IMM76D; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444";
const char S105819[] = "TuneIn Radio/4.0 (Android 8; GT-P1000; Java)";
const char S105820[] = "Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-P1010 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1010, 40, 1.8.5)";
const char S105821[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P6201 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000";
const char S105822[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/800";
const char S105823[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-au; GT-P7320T Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7320T, 40, 1.8.5)";
const char S105824[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P5110 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31";
const char S105825[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.1.232";
const char S105826[] = "iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2; fa-fa; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105827[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P5113 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315";
const char S105828[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-P7300 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315";
const char S105829[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P1000 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105830[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259";
const char S105831[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7310 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S105832[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S105833[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7500 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S105834[] = "MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 MobileSafari/534.13";
const char S105835[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P2000 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22";
const char S105836[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 OverDrive Media Console/2.6.4";
const char S105837[] = "iBrowser/2.8 Mozilla/5.0 (Linux; U; Android 3.2; fa-fa; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105838[] = "theScore/3.2.1 Android-OS/3.1 (Android; Tablet, GT-P7500M)";
const char S105839[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6210 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 OverDrive Media Console/2.6.4";
const char S105840[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-us ; GT-P3110 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S105841[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-P3110 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31";
const char S105842[] = "MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105843[] = "MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 4.0.3; ar-ae; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105844[] = "TuneIn Radio Pro/6.6 (Android 16; GT-P3113; Java)";
const char S105845[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6810 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S105846[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P7300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S105847[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7510 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7510, 40, 1.8.5)";
const char S105848[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-P6810 Build/JOP40D)";
const char S105849[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-P7501 Build/JDQ39; CyanogenMod-Ganbarou V3.03 GT-P7501) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0100, de";
const char S105850[] = "Mozilla/5.0 (Linux; U; Android 3.2; ru-ru; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 UCBrowser/2.2.2.259";
const char S105851[] = "MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105852[] = "MQQBrowser/2.9/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 MobileSafari/534.13";
const char S105853[] = "Mozilla/5.0 (Linux; Android 4.1.1; GT-P5100 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.53005";
const char S105854[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-CN; GT-P7310 Build/HMJ37) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S105855[] = "TuneIn Radio/6.6 (Android 16; GT-P3113; Java)";
const char S105856[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P1000 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315";
const char S105857[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-CN; GT-P7510 Build/HMJ37) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S105858[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6210 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S105859[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6201 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.27/381,1.9.27.381,2013-03-21 11:39, +0100, de";
const char S105860[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7310 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S105861[] = "TuneIn Radio/6.1 (Android 16; GT-P3113; Java)";
const char S105862[] = "MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13";
const char S105863[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-P6800 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S105864[] = "Mozilla/5.0 (Android; (OS 4.0.4) (IM 28,684MB) (EM 28,684MB) (Build IMM76D.VLLH3) (Model GT-P7510)  (Brand samsung)) WW_Mobile_Android/2.2.15.509";
const char S105865[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-P3100 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S105866[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-P3110 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S105867[] = "Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7310 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355";
const char S105868[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-P7510 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.53005";
const char S105869[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P5110 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105870[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-P5113 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31";
const char S105871[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; GT-P6200L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]";
const char S105872[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; GT-P7300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7300, 42, 1.9.1)";
const char S105873[] = "iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
const char S105874[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; GT-P5100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]";
const char S105875[] = "Mozilla/5.0 (Linux; U; Android 2.2.1; zh-CN; GT-P1010 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31";
const char S105876[] = "Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT-P1000 Build/MIUI-XJ) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Mobile Maxthon";
const char S105877[] = "Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7310 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13 Maxthon/4.0.4.1000";
const char S105878[] = "Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P7100 Build/JDQ39)";
const char S105879[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P6810 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Safari/537.31";
const char S105880[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105881[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6800 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.27/381,1.9.27.381,2013-03-21 11:39, +0800, hk";
const char S105882[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-P3113 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31";
const char S105883[] = "Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 UCBrowser/2.3.1.257";
const char S105884[] = "Mozilla/5.0 (Linux; Android 3.2; GT-P6800 Build/HTJ85B) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315";
const char S105885[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7320 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.27/383,1.9.27.383,2013-03-25 21:34, +0800, china, cn";
const char S105886[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3100 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105887[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6810 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 OverDrive Media Console/2.6.4";
const char S105888[] = "Orca-Android/2.2.3-release OS/3.2 Model/GT-P6200 VersionCode/94668 Product/Messenger Carrier/MY_MAXIS Manufacturer/samsung Brand/samsung";
const char S105889[] = "Mozilla/5.0 (Linux; U; Android 4.2.1; en-ca; GT-P6210 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105890[] = "Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-P5100 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31";
const char S105891[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-P1000 Build/IMM76L) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile";
const char S105892[] = "OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30";
const char S105893[] = "Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-P7511 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.27/381,1.9.27.381,2013-03-21 11:39, +0100";
const char S105894[] = "Mozilla/5.0 (Linux; Android 4.2.1; GT-P6210 Build/JOP40G) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Safari/537.31";
const char S105895[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-P1000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +1100, au";
const char S105896[] = "Mozilla/5.0 (Linux; Android 3.1; GT-P7300 Build/HMJ37) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.53463";
const char S105897[] = "Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 OverDrive Media Console/2.6.4";
const char S105898[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; GT-P7300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000";
const char S105899[] = "Mozilla/5.0 (Linux; Android 3.2; GT-P7300 Build/HTJ85B) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.53005";
const char S105900[] = "MQQBrowser/2.9/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 MobileSafari/534.13";
const char S105901[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6630) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S105902[] = "Mozilla/5.0 (Linux; Android 4.0.4; Micromax A110 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S105903[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I8262D Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31";
const char S105904[] = "UCWEB/2.0 (Linux; U; Adr 4.1.2; zh-CN; GT-I8262D) U2/1.0.0 UCBrowser/8.8.1.258 U2/1.0.0 Mobile";
const char S105905[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-I8262 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.27/381,1.9.27.381,2013-03-21 11:39, +0000, sn";
const char S105906[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-p520) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile";
const char S105907[] = "Mozilla/5.0 (Linux; Android 4.2.2; C5503 Build/10.3.A.0.317) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.68 Mobile Safari/537.36";
const char S105908[] = "Mozilla/5.0 (Linux; Android 4.2.2; C5503 Build/10.3.A.0.232) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
const char S105909[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; C5502 Build/10.1.1.A.1.164) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1";
const char S105910[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; C5502 Build/10.1.1.A.1.273) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1";
const char S105911[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.2.2.259";
const char S105912[] = "Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GFIVE Fararee (A78) Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S105913[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3230) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S105914[] = "Mozilla/5.0 (Linux; Android 4.1.2; WL-101GQC Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36";
const char S105915[] = "Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Amaze 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30";
const char S105916[] = "Mozilla/5.0 (Linux; Android 4.0.4; NC10+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19";
const char S105917[] = "UCWEB/2.0(Java; U; MIDP-2.0; en-US; gt-s5230w) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S105918[] = "Mozilla/5.0 (Linux; Android 4.2.2; C6806 Build/14.1.B.1.277) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31";
const char S105919[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sgh-f480i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S105920[] = "Mozilla/5.0 (DTV) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.5 AQUOSBrowser/1.0 (AS00DTV;V;0001;0001) AQUOS-AS/2.0 LC-52LE840X";
const char S105921[] = "Mozilla/5.0 (DTV) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.5 AQUOSBrowser/1.0 (AS00DTV;V;0001;0001) AQUOS-AS/2.0 LC-46LX840H";
const char S105922[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-tw; HTC Butterfly s Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (HTC Butterfly s, 50, 1.9.9)";
const char S105923[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly s Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201";
const char S105924[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; zh-tw; HTC Butterfly s Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon";
const char S105925[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-S7270 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.1.1364.172 Mobile Safari/537.22";
const char S105926[] = "Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; GT-S7275 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.2/552,2.0.2.552,2013-05-22 23:05, +0200, fr";
const char S105927[] = "Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:26.0) Gecko/X Firefox/26.0 slurp";
const char S105928[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; zte nova) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile";
const char S105929[] = "Nokia5300/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5300) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile";
const char S105930[] = "Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I527 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36";
const char S105931[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s7233e) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S105932[] = "Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LA690S-ZB; 03.10.74; 0xX;); LG NetCast.TV-2013 /03.10.74 (LG, 42LA690S-ZB, wireless)";
const char S105933[] = "Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LA740S-ZB; 03.10.81; 0xX;); LG NetCast.TV-2013 /03.10.81 (LG, 42LA740S-ZB, wireless)";
const char S105934[] = "Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 32LN570V-ZA; 02.06.01; 0xX;); LG NetCast.TV-2013";
const char S105935[] = "Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LA660S-ZA; 03.10.74; 0xX;); LG NetCast.TV-2013 /03.10.74 (LG, 47LA660S-ZA, wireless)";
const char S105936[] = "Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 55LA6200-UA; 03.07.24; 0xX;); LG NetCast.TV-2013 /03.07.24 (LG, 55LA6200-UA, wireless)";
const char S105937[] = "Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LA667S-ZB; 03.10.81; 0xX;); LG NetCast.TV-2013 /03.10.81 (LG, 42LA667S-ZB, wireless)";
const char S105938[] = "Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LA790V-ZA; 03.05.02; 0xX;); LG NetCast.TV-2013 /03.05.02 (LG, 47LA790V-ZA, wired)";
const char S105939[] = "Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LA860V-ZA; 03.05.49; 0xX;); LG NetCast.TV-2013 /03.05.49 (LG, 42LA860V-ZA, wireless)";
const char S105940[] = "Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LA6210-TD; 03.10.34; 0xX;); LG NetCast.TV-2013 /03.10.34 (LG, 42LA6210-TD, wireless)";
const char S105941[] = "Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+SCREEN+TUNER; LGE; 32LN571B-TE; 03.10.65; 0xX;); LG NetCast.TV-2013 /03.10.65 (LG, 32LN571B-TE, wireless)";
const char S105942[] = "Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LA6200-TA; 03.20.09; 0xX;); LG NetCast.TV-2013 /03.20.09 (LG, 47LA6200-TA, wired)";
const char S105943[] = "Mozilla/5.0 (DirectFB; U; Linux 35230; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.1.4(+mouse+PORTAL_KEY+SCREEN+TUNER; LGE; 37LV375S-ZC; 12.00.00; 0xX;); LG NetCast.TV-2012";
const char S105944[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LM960V-ZB; 04.10.26; 0xX;); LG NetCast.TV-2012 0";
const char S105945[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+SCREEN+TUNER; LGE; 55LS5700-TB; 04.00.28; 0xX;); LG NetCast.TV-2012";
const char S105946[] = "Mozilla/5.0 (DirectFB; U; Linux 35230; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.1.4(+mouse+SCREEN+TUNER; LGE; 55LV3730-TD; 04.03.10; 0xX;); LG NetCast.TV-2011";
const char S105947[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+SCREEN+TUNER; LGE; 42LS570T-ZB; 04.10.23; 0xX;); LG NetCast.TV-2012 0";
const char S105948[] = "Mozilla/5.0 (DirectFB; U; Linux 35230; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.1.4(+mouse+PORTAL_KEY+SCREEN+TUNER; LGE; 37LV5500-ZC; 06.01.04; 0xX;); LG NetCast.TV-2011";
const char S105949[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LM6710-TB; 04.00.29; 0xX;); LG NetCast.TV-2012";
const char S105950[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 55LM9600-TA; 04.20.12; 0xX;); LG NetCast.TV-2012 0";
const char S105951[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LM6410-TB; 04.20.11; 0xX;); LG NetCast.TV-2012 0";
const char S105952[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 32LM620T-ZE; 04.40.23; 0xX;); LG NetCast.TV-2012";
const char S105953[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+SCREEN+TUNER; LGE; 47LS5700-TB; 04.30.11; 0xX;); LG NetCast.TV-2012";
const char S105954[] = "Mozilla/5.0 (DirectFB; U; Linux 35230; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.1.4(+mouse+3D+SCREEN+TUNER; LGE; 50PZ950-TA; 05.01.01; 0xX;); LG NetCast.TV-2011";
const char S105955[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 72LM9500-TA; 04.30.11; 0xX;); LG NetCast.TV-2012 0";
const char S105956[] = "Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LM669S-ZC; 04.40.27; 0xX;); LG NetCast.TV-2012 0";
const char S105957[] = "Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL968; 7.0.27.3; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)";
const char S105958[] = "Opera/9.80 (Linux mips; Opera TV Store/4502; U; fr) Presto/2.10.250 Version/11.60 Model/Sony-KDL-55HX855 SonyCEBrowser/1.0 (KDL-55HX855; CTV/PKG1.700EUA; FRA)";
const char S105959[] = "Opera/9.80 (Linux mips; Opera TV Store/4510; U; de) Presto/2.10.250 Version/11.60 Model/Sony-KDL-46HX758 SonyCEBrowser/1.0 (KDL-46HX758; CTV/PKG1.910EUA; DEU)";
const char S105960[] = "Opera/9.80 (Linux mips; Opera TV Store/4510; U; el) Presto/2.10.250 Version/11.60 Model/Sony-KDL-46HX850 SonyCEBrowser/1.0 (KDL-46HX850; CTV/PKG1.910EUA; GRC)";
const char S105961[] = "Opera/9.80 (Linux mips; U; Model/Sony-KDL-40HX75A SonyCEBrowser/1.0 (KDL-40HX75A; CTV/1.0; TWN; en); en) Presto/2.10.250 Version/11.60";
const char S105962[] = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.13+ (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2 Fedora/18 Epiphany/3.6.1";
const char S105963[] = "Nokia6085/2.0 (06.00) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia6085) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile";
const char S105964[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6680) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S105965[] = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 7_1_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/9B206 Safari/6531.22.7";
const char S105966[] = "Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Mobile/11D11 Twitter for iPhone";
const char S105967[] = "Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11D41";
const char S105968[] = "Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-79) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1612.0 Mobile Safari/537.36";
const char S105969[] = "Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1622.0 Mobile Safari/537.36";
const char S105970[] = "Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1612.0 Mobile Safari/537.36";
const char S105971[] = "Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1612.0 Mobile Safari/537.36";
const char S105972[] = "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/31.0.1610.0 Mobile/12E55 Safari/8536.25";
const char S105973[] = "Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/31.0.1610.0 Mobile/12E55 Safari/8536.25";
const char S105974[] = "Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/31.0.1617.0 Mobile/12E55 Safari/8536.25";
const char S105975[] = "Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/31.0.1617.610 Mobile/11A4449d Safari/8536.25";
const char S105976[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae66-1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S105977[] = "Nokia5300/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5300) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S105978[] = "Mozilla/5.0 (Linux; Android 4.0.3; MediaPad 7 Lite Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63537";
const char S105979[] = "Mozilla/5.0 (Linux; Android 2.3.4; GT-S5830 Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S105980[] = "Mozilla/5.0 (Linux; Android 4.2.1; Micromax A116 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S105981[] = "Mozilla/5.0 (Linux; Android 4.2.2; HTL22 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S105982[] = "Mozilla/5.0 (Linux; Android 4.1.2; A511 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63537";
const char S105983[] = "Mozilla/5.0 (Linux; Android 2.3.6; XT303 Build/07.17.04R_S) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S105984[] = "Mozilla/5.0 (Linux; Android 4.2.1; ZTE Grand X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S105985[] = "Mozilla/5.0 (Linux; Android 4.1.2; XT919 Build/2_290_2008) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S105986[] = "Mozilla/5.0 (Linux; Android 2.3.4; LG-C660h Build/GRJ22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S105987[] = "Mozilla/5.0 (Linux; Android 4.0.4; LT15i Build/Xperia Ultimate HDT 4.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S105988[] = "Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63537";
const char S105989[] = "Mozilla/5.0 (Linux; Android 4.0.4; A8 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S105990[] = "Mozilla/5.0 (Linux; Android 4.1.2; IM-A850L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S105991[] = "Mozilla/5.0 (Linux; Android 4.2.2; SPH-L710 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537";
const char S105992[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5600) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S105993[] = "Mozilla/5.0 (X11; Linux x86_32; rv:28.0) Gecko/X Firefox/28.0";
const char S105994[] = "Mozilla/5.0 (Windows; U; AltWindows; ru) Gecko/X Firefox/28.0";
const char S105995[] = "Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.30 Mobile Safari/537.36 OPR/17.0.1254.63751";
const char S105996[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sgh-f480) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S105997[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; SAMSUNG-GT-S5253) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S105998[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6630) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S105999[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mot-ex223) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile";
const char S106000[] = "UCWEB/2.0 (Java; U; MIDP-2.0; en-US; SAMSUNG-GT-S5380K) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S106001[] = "Nokia3110c/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia3110c) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile";
const char S106002[] = "Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780";
const char S106003[] = "Nokia6300/2.0 (05.00) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6300/UC Browser7.2.2.51/69/401";
const char S106004[] = "Mozilla/5.0 (Series40; Nokia6300/07.00; Profile/MIDP-2.0 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.3.23.12";
const char S106005[] = "880SH/1.103 Browser/VF-NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S106006[] = "Nokia6300/2.0 (05.50) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6300/UC Browser7.0.0.41/69/352";
const char S106007[] = " SonyEricssonZ310iv/R1DB Java/SEMC-Java/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1";
const char S106008[] = "Opera/8 Presto";
const char S106009[] = "Opera/9 Presto";
const char S106010[] = "Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_1) AppleWebKit (KHTML, like Gecko) Chrome/10";
const char S106011[] = "Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_1) AppleWebKit (KHTML, like Gecko) Chrome/9";
const char S106012[] = "Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_1) AppleWebKit (KHTML, like Gecko) Chrome/11";
const char S106013[] = "Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_1) AppleWebKit (KHTML, like Gecko) Chrome/12";
const char S106014[] = "Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_1) AppleWebKit (KHTML, like Gecko) Chrome/13";
const char S106015[] = "Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_1) AppleWebKit (KHTML, like Gecko) Chrome/14";
const char S106016[] = "Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_1) AppleWebKit (KHTML, like Gecko) Chrome/15";
const char S106017[] = "Mozilla/5.0 (Windows; U; Windows NT 6.0;) AppleWebkit (KHTML, like Gecko) Version/2 Safari";
const char S106018[] = "Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_1) AppleWebKit (KHTML, like Gecko) Chrome/8";
const char S106019[] = "Chrome/3";
const char S106020[] = "Chrome/2";
const char S106021[] = "Chrome/6";
const char S106022[] = "Chrome/7";
const char S106023[] = "Chrome/13";
const char S106024[] = "Chrome/14";
const char S106025[] = "Chrome/15";
const char S106026[] = "Nokia6820/2.0 (4.83) Profile/MIDP-1.0 Configuration/CLDC-1.0 (compatible; Googlebot-Mobile/2.1; +http://www.google.com/bot.html)";
const char S106027[] = "Mozilla/5.0 (DirectFB; U; Linux 35230; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.1.4(+SCREEN+TUNER; LGE; 47LV3730-TD; 04.01.09; 0xX;); LG NetCast.TV-2011";
const char S106028[] = "Mozilla/5.0 (DirectFB; U; Linux 35230; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.1.4(+mouse+SCREEN+TUNER; LGE; 42LV3730-TD; 04.03.10; 0xX;); LG NetCast.TV-2011";
const char S106029[] = "Mozilla/5.0 (DirectFB; U; Linux mips; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.0.10(+mouse+PORTAL_KEY+SCREEN+TUNER; LGE; 42LE5500-ZA; 03.21.01; 0xX;); LG NetCast.TV-2010";
const char S106030[] = "Mozilla/5.0 (DirectFB; U; Linux 35230; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.1.4(+mouse+3D+PORTAL_KEY+SCREEN+TUNER; LGE; 42LW5500-ZE; 05.11.08; 0xX;); LG NetCast.TV-2011";
const char S106031[] = "Mozilla/5.0 (DirectFB; U; Linux mips; en) AppleWebKit/528.5+ (KHTML, like Gecko, Safari/528.5+); LG Browser/3.0.13(+mouse+3D+SCREEN+TUNER; LGE; 42LE7500-ZA; 03.05.04; 0xX;); LG Netcast.Media-2011 (avkb/pcx";
const char S106032[] = "Mozilla/5.0 (DirectFB; U; Linux mips; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.0.10(+SCREEN+TUNER; LGE; 42LD650-ZC; 03.20.21; 0xX;); LG NetCast.TV-2010";
const char S106033[] = "Dolphin 3.5.1 (iPhone; iPhone";
const char S106034[] = "Dolphin/3.0 CFNetwork/548.0.3 Darwin/11.0.0";
const char S106035[] = "Mozilla/5.0 (Java; U; vi; fpt-b730) UCBrowser8.3.0.154/69/352/UCWEB Mobile";
const char S106036[] = "fpt-b730/UC Browser7.9.0.102/69/352";
const char S106037[] = "Mozilla/5.0 (Java; U; en-us; fpt-b730) UCBrowser8.2.0.132/69/452/UCWEB Mobile";
const char S106038[] = "Mozilla/5.0 (Java; U; vi; fpt-b730) UCBrowser8.2.1.144/69/352/UCWEB Mobile";
const char S106039[] = "fpt-b730/UC Browser7.8.0.95/69/401";
const char S106040[] = "Mozilla/5.0 (Mobile; rv:14.0) Gecko/14.0 Firefox/14.0";
const char S106041[] = "(?i)bot|spider|crawler|slurp";
const char S106042[] = "(?i)bot|spider|crawler|slurp|[^\\s]+(?=bot|spider|crawler|slurp)[^\\s]+";
const char S106043[] = ".+";
const char S106044[] = "W3C_Validator";
const char S106045[] = "^Opera/";
const char S106046[] = "Windows NT 6\\.0";
const char S106047[] = "Windows NT 6\\.1";
const char S106048[] = "Opera Mini|Opera Mobi";
const char S106049[] = "MIDP|CLDC";
const char S106050[] = "(?<=Opera/)[\\d+.]+";
const char S106051[] = "(?<=Version/)\\d+\\.\\d+";
const char S106052[] = "(?<=\\()[^;]+";
const char S106053[] = "(?<=Opera/\\d\\.\\d \\()[^;\\)]+";
const char S106054[] = "Android( |)2\\.2\\.1";
const char S106055[] = "(?<=;)([^;\\)]+)(?=Build/)";
const char S106056[] = "(?<=\\w\\w(_|-)\\w\\w;)[^;\\)]+";
const char S106057[] = "(?<=;\\sAndroid)[\\d\\s][^;\\)]+";
const char S106058[] = "(?<=UCWEB)\\d+\\.\\d+|(?<=UCWEB/)\\d+\\.\\d+";
const char S106059[] = "(?<=UC Browser)\\d+\\.\\d+";
const char S106060[] = "(?<=UP\\.Browser)[\\d\\.]+";
const char S106061[] = "(?<=Opera/)\\d+\\.\\d+|(?<=Opera )\\d+\\.\\d+";
const char S106062[] = "(?<=Opera Mini/)\\d+\\.\\d+";
const char S106063[] = "(?<=Opera Mobi/)\\d+\\.\\d+";
const char S106064[] = "(?<=Ninesky-android-mobile/)\\d+\\.\\d+";
const char S106065[] = "(?<=CrMo/)\\d+";
const char S106066[] = "(?<=Chrome/)\\d+";
const char S106067[] = "(?<=AppleWebKit/)[\\d\\.\\+]+";
const char S106068[] = "(?<=Mobile Safari/)[\\d\\.]+";
const char S106069[] = "(?<=Flipboard/)\\d+\\.\\d+";
const char S106070[] = "MQQBrowser/(\\d+\\.\\d|\\d+ )";
const char S106071[] = "Maxthon";
const char S106072[] = "UCBrowser/\\d+\\.\\d";
const char S106073[] = "iBrowser/\\d+\\.\\d";
const char S106074[] = "TeamStream";
const char S106075[] = "Maxthon/\\d+\\.\\d";
const char S106076[] = "Ninesky/\\d+\\.\\d";
const char S106077[] = "OPR/\\d+\\.\\d+";
const char S106078[] = "Android( |)2\\.2\\.2";
const char S106079[] = "MQQBrowser/\\d+\\.\\d";
const char S106080[] = "Android( |)2\\.2\\.3";
const char S106081[] = "Android( |)2\\.3\\.1";
const char S106082[] = "Android( |)2\\.3\\.2";
const char S106083[] = "Android( |)2\\.3\\.3";
const char S106084[] = "(?<=UCBrowser)\\d+\\.\\d+";
const char S106085[] = "(?<=UCBrowser/)\\d+\\.\\d+";
const char S106086[] = "^(One|i)Browser/(\\d+|\\d+\\.\\d)";
const char S106087[] = "Mobile Maxthon";
const char S106088[] = "\\)Maxthon|Maxthon$";
const char S106089[] = "Ninesky/\\d\\.\\d";
const char S106090[] = "^YahooMobile/\\d+\\.\\d";
const char S106091[] = "Android( |)2\\.3\\.4";
const char S106092[] = "\\)Maxthon";
const char S106093[] = "Cyanogen";
const char S106094[] = "Maxthon$";
const char S106095[] = "(One|i)Browser/\\d\\.\\d";
const char S106096[] = "Android( |)2\\.3\\.5";
const char S106097[] = "OneBrowser/\\d+\\.\\d";
const char S106098[] = "Maxthon(/\\d+\\.\\d+|$)";
const char S106099[] = "UCBrowser/\\d+\\.\\d+";
const char S106100[] = "^iBrowser/(\\d+\\.\\d|\\d+ )";
const char S106101[] = "Android( |)2\\.3\\.6";
const char S106102[] = "OneBrowser/\\d\\.\\d";
const char S106103[] = "Android( |)2\\.3\\.7";
const char S106104[] = "(?<=CyanogenMod-)\\d";
const char S106105[] = "(?<=CyanogenMod)\\d";
const char S106106[] = "CyanogenMod";
const char S106107[] = "CM7";
const char S106108[] = "MQQBrowser/\\d\\.\\d";
const char S106109[] = "Chrome/[0-9][^\\d]";
const char S106110[] = "(?<=Chrome/)\\d+\\.\\d+";
const char S106111[] = "(?<=\\(|;|,)[^\\);]+";
const char S106112[] = "(?<=RockMelt/)\\d+\\.\\d+";
const char S106113[] = "Ubuntu/\\d+\\.\\d+";
const char S106114[] = "Sleipnir/\\d+\\.\\d+";
const char S106115[] = "Nutch-\\d+\\.\\d+";
const char S106116[] = "Comodo_Dragon/\\d+\\.\\d+";
const char S106117[] = "Iron/\\d+\\.\\d";
const char S106118[] = "GoogleTV";
const char S106119[] = "chromeframe/\\d+\\.\\d";
const char S106120[] = "Flock/\\d+\\.\\d";
const char S106121[] = "Chrome/1[0-9]";
const char S106122[] = "(?<=^).+(?=Mozilla)";
const char S106123[] = "Puffin/\\d+\\.\\d";
const char S106124[] = "(?<=QQBrowser/)\\d+\\.\\d+";
const char S106125[] = "AvantBrowser/Tri-Core";
const char S106126[] = "(?<=Midori/)\\d\\.\\d";
const char S106127[] = "Chrome/2[0-9]";
const char S106128[] = "Ubuntu Chromium";
const char S106129[] = "Chrome/3[0-9]";
const char S106130[] = "Linux Mint";
const char S106131[] = "Ubuntu;";
const char S106132[] = "ASUS\\)";
const char S106133[] = "Mac OS X \\d+(\\.|_)\\d";
const char S106134[] = "Windows NT \\d\\.\\d";
const char S106135[] = "(?<=KDL)[^;]+";
const char S106136[] = "SonyEricsson";
const char S106137[] = "(?<=SonyEricsson)[^/]+";
const char S106138[] = "(?<=NetFront/)[\\d\\.]+";
const char S106139[] = "(?<=Opera )[^\\s]+";
const char S106140[] = "(?<=SEMC-Browser/)[^\\s]+";
const char S106141[] = "(?<=TelecaBrowser/)[^\\s]+";
const char S106142[] = "(?<=SymbianOS/)[^;]+ ";
const char S106143[] = "(?<=Series60/)[^\\s]+";
const char S106144[] = "(?<=U(C|P)\\.Browser/)\\d+\\.\\d+";
const char S106145[] = "(?<=UCBrowser)(|/)\\d+\\.\\d+";
const char S106146[] = "UCWEB/\\d\\.\\d";
const char S106147[] = "Silk-Accelerated";
const char S106148[] = "(?<=Silk/)\\d+\\.\\d+";
const char S106149[] = "(?<=Accelerated=)(true|false)";
const char S106150[] = "Android \\d|Android\\d";
const char S106151[] = "GT-\\w\\d+";
const char S106152[] = "2\\.2/FROYO";
const char S106153[] = "theScore/\\d\\.\\d";
const char S106154[] = "^Instagram \\d\\.\\d";
const char S106155[] = "^WhatsApp/2\\.7";
const char S106156[] = "(FROYO|GINGERBREAD)\\)$";
const char S106157[] = "Android 1|Android1";
const char S106158[] = "Android 2\\.0|Android2\\.0";
const char S106159[] = "Android 2\\.1|Android2\\.1";
const char S106160[] = "Android 2\\.2|Android2\\.2";
const char S106161[] = "Maxthon/\\d+\\.\\d|Maxthon";
const char S106162[] = "^iBrowser/\\d+";
const char S106163[] = "Android 2\\.3|Android2\\.3";
const char S106164[] = "Android 2\\.4|Android2\\.4";
const char S106165[] = "Android 3|Android3";
const char S106166[] = "Flipboard/\\d+\\.\\d";
const char S106167[] = "\\[operamini\\]";
const char S106168[] = "^(One|i)Browser/(\\d+\\.\\d|\\d+ )";
const char S106169[] = "(?<=OverDrive Media Console/)\\d\\.\\d";
const char S106170[] = "Android( |)4\\.0";
const char S106171[] = "Android( |)4\\.0\\.\\d+";
const char S106172[] = "Chrome/\\d+";
const char S106173[] = "Android 4\\.0\\.1|Android4\\.0\\.1";
const char S106174[] = "(?<=^).+(?=Linux)";
const char S106175[] = "Android 4\\.0\\.2|Android4\\.0\\.2";
const char S106176[] = "Android 4\\.0\\.3|Android4\\.0\\.3";
const char S106177[] = "Maxthon(/| \\()\\d+\\.\\d";
const char S106178[] = "^(One|i)Browser/\\d+\\.\\d";
const char S106179[] = "Android 4\\.0\\.4|Android4\\.0\\.4";
const char S106180[] = "(One|i)Browser/(\\d+\\.\\d|\\d+ )";
const char S106181[] = "Maxthon \\(\\d\\.\\d";
const char S106182[] = "PARANOIDANDROID";
const char S106183[] = "Android 4\\.0\\.(5|6|8|9)";
const char S106184[] = "Android 4\\.1|Android4\\.1";
const char S106185[] = "Android( |)4\\.1\\.\\d+";
const char S106186[] = "Android 4\\.1\\.1|Android4\\.1\\.1";
const char S106187[] = "Maxthon$|\\)Maxthon";
const char S106188[] = "^OneBrowser/\\d+\\.\\d";
const char S106189[] = "MiuiBrowser/\\d\\.\\d";
const char S106190[] = "Android 4\\.1\\.2|Android4\\.1\\.2";
const char S106191[] = "(?<=^OneBrowser/)\\d\\.\\d";
const char S106192[] = "Android 4\\.2|Android4\\.2";
const char S106193[] = "Android( |)4\\.2\\.\\d+";
const char S106194[] = "Android 4\\.2\\.1|Android4\\.2\\.1";
const char S106195[] = "Maxthon(/\\d+\\.\\d|$)";
const char S106196[] = "Android 4\\.2\\.2|Android4\\.2\\.2";
const char S106197[] = "Maxthon \\(\\d+\\.\\d";
const char S106198[] = "CARBON-JB";
const char S106199[] = "Liqiud-JB";
const char S106200[] = "\\((iPhone|iPad|iPod)";
const char S106201[] = "Flipboard";
const char S106202[] = "Nokia|nokia";
const char S106203[] = "webOS|hpwOS";
const char S106204[] = "(?<=\\()iPhone|iPad|iPod";
const char S106205[] = "(?<=Version/)[\\d\\.]+";
const char S106206[] = "(?<=OS )[\\d(_|\\.)\\d]+";
const char S106207[] = "(?<=Sleipnir/)[\\d\\.]+";
const char S106208[] = "(?<=AppleWebKit/)[\\d\\.]+";
const char S106209[] = "(?<=Mobile/)[\\d\\w]+";
const char S106210[] = "(?<=Safari/)[\\d\\.]+";
const char S106211[] = "(?<=Dolphin )[\\d\\.]+";
const char S106212[] = "(?<=Dolphin HD )[\\d\\.]+";
const char S106213[] = "Twitter";
const char S106214[] = "Miu Ptt";
const char S106215[] = "Incredimail/\\d+\\.\\d";
const char S106216[] = "PhantomJS/\\d+\\.\\d";
const char S106217[] = "UP\\.Browser/(\\d+\\.\\d|\\d+ )";
const char S106218[] = "BOLT";
const char S106219[] = "mac";
const char S106220[] = "win";
const char S106221[] = "(?<=BOLT/)[\\d\\.]+";
const char S106222[] = "(F|f)ire(F|f)o(X|x)|(I|i)ceweasel|BonEcho";
const char S106223[] = "\\(Macintosh;|\\(Win|\\(X1\\d;|WOW64";
const char S106224[] = "Fennec|Linux Mint|Thunderbird";
const char S106225[] = "(?<=X11;)\\w+";
const char S106226[] = "(?<=Firefox)/\\d+\\.\\d+";
const char S106227[] = "(?<=Iceweasel)/\\d+\\.\\d+";
const char S106228[] = "(?<=Mozilla/)\\d\\.\\d";
const char S106229[] = "(?<=Mozilla/\\d\\.\\d \\()[^;\\)]+";
const char S106230[] = "Windows NT \\d\\.\\d(?=;)|Mac OS X [^\\s]+(?=;)|Ubuntu|X11|SunOS|OpenBSD|FreeBSD";
const char S106231[] = "Fedora/\\d+\\.\\d+";
const char S106232[] = "Nutch";
const char S106233[] = "Debian-\\d\\.\\d";
const char S106234[] = "\\w(U|u)buntu/\\d+\\.\\d+";
const char S106235[] = "(?<=\\((U|u)buntu-)[^\\)]+";
const char S106236[] = "(?<=Firefox)[^\\d]+";
const char S106237[] = "(?<=Win)[^;]+";
const char S106238[] = "Fedora/";
const char S106239[] = "Pale(M|m)oon/\\d+\\.\\d";
const char S106240[] = "(E|e)pic/\\d+\\.\\d|Epic/";
const char S106241[] = "Comet(Bird|Browser)/\\d+\\.\\d";
const char S106242[] = "Kylo/\\d+\\.\\d";
const char S106243[] = "Wyzo/\\d+\\.\\d";
const char S106244[] = "Camino/\\d+\\.\\d";
const char S106245[] = "Minefield/\\d+\\.\\d";
const char S106246[] = "Navigator/\\d+\\.\\d";
const char S106247[] = "IceDragon/\\d+\\.\\d";
const char S106248[] = "Opera \\d+\\.\\d\\d$";
const char S106249[] = "lolifox/\\d\\.\\d";
const char S106250[] = "Avant TriCore";
const char S106251[] = "Midori/(|\\\")\\d+(|\\\")\\.(|\\\")\\d(|\\\")";
const char S106252[] = "SeaMonkey/\\d\\.\\d";
const char S106253[] = "firefox/\\d\\.\\d";
const char S106254[] = "BonEcho/\\d\\.\\d";
const char S106255[] = "Maemo Browser \\d\\.\\d";
const char S106256[] = "Linux mips";
const char S106257[] = "Firefox/";
const char S106258[] = "Fennec/|\\(Mobile; rv";
const char S106259[] = "(?<=Mozilla/)[\\d\\.]+";
const char S106260[] = "(?<=Fennec/)[\\d\\.]+";
const char S106261[] = "Fennec";
const char S106262[] = "(?<=\\()[^\\)]+";
const char S106263[] = "(?<=Gecko/)[\\d]+";
const char S106264[] = "(?<=Firefox/)[\\d\\.]+";
const char S106265[] = "MSIE";
const char S106266[] = "compatible";
const char S106267[] = "Windows XP";
const char S106268[] = "Windows NT";
const char S106269[] = "Windows ME";
const char S106270[] = "Win32";
const char S106271[] = "Windows [\\d\\.]+";
const char S106272[] = "Windows NT [\\d\\.]+";
const char S106273[] = "Mac_PowerPC";
const char S106274[] = "PPC Mac OS";
const char S106275[] = "Mobile";
const char S106276[] = "Windows CE";
const char S106277[] = "Phone";
const char S106278[] = "UC(\\.| ||)Browser";
const char S106279[] = "Opera";
const char S106280[] = "(?i)LG-|LG_|SC-|Toshiba|XV\\d+|Nokia|LGE|PalmOS|POLARIS|PalmSource|Nutch|NetFront|Samsung|HTC";
const char S106281[] = "(?<=MSIE\\s)[\\d\\.]+";
const char S106282[] = "(?<=Outlook )[\\d\\.]+";
const char S106283[] = "(?<=Windows)[\\s\\w\\d\\.]+";
const char S106284[] = "Windows NT \\d\\.\\d(?=;)|Mac OS X [^\\s]+(?=;)|Ubuntu|X11";
const char S106285[] = "chromeframe";
const char S106286[] = "Media Center PC";
const char S106287[] = "Zune";
const char S106288[] = "ARM";
const char S106289[] = "Touch";
const char S106290[] = "WOW64;";
const char S106291[] = "(?<=heritrix/)[\\d\\.]+";
const char S106292[] = "Maxthon|MAXTHON";
const char S106293[] = "SlimBrowser/\\d+\\.\\d";
const char S106294[] = "Avant(| )Browser";
const char S106295[] = "Xbox";
const char S106296[] = "Midori/\\d+\\.\\d";
const char S106297[] = "(?<=Maxthon)(/| )\\d+\\.\\d";
const char S106298[] = "(?<=MAXTHON) \\d\\.\\d";
const char S106299[] = "Win(95|32|98|NT4\\.0|64|3\\.\\d)";
const char S106300[] = "PPC";
const char S106301[] = "Tablet PC";
const char S106302[] = "SV1";
const char S106303[] = "(?<=UCWEB)\\d\\.\\d";
const char S106304[] = "Nokia";
const char S106305[] = "Series40";
const char S106306[] = "(?<=Nokia)[^/]+";
const char S106307[] = "(?<=Series40)[^;]+";
const char S106308[] = "(?<=S40OviBrowser/)\\d+\\.\\d+\\.\\d+";
const char S106309[] = "iBrowser/Mini\\d+\\.\\d";
const char S106310[] = "Series60";
const char S106311[] = "(?<=Series60/)[\\w\\d\\.]+";
const char S106312[] = "MQQBrowser/Mini\\d\\.\\d";
const char S106313[] = "^Mozilla";
const char S106314[] = "Safari";
const char S106315[] = "Windows|Macintosh|X11";
const char S106316[] = "Chrome/";
const char S106317[] = "(?<=Safari/)\\d+\\.\\d+";
const char S106318[] = "Windows NT \\d\\.\\d(?=;)|Mac OS X [^\\s]+(?=;)|Ubuntu( |(/|)\\d+\\.\\d+)|X11";
const char S106319[] = "(?<=Version/)\\d\\.\\d";
const char S106320[] = "(?<=Mac OS X 10_\\d_\\d;)[^;\\)]+";
const char S106321[] = "Mac OS X 10_\\d";
const char S106322[] = "(?<=PhantomJS/)\\d\\.\\d";
const char S106323[] = "SecondLife/\\d+\\.\\d";
const char S106324[] = "Google Earth/\\d+\\.\\d";
const char S106325[] = "rekonq(/\\d+\\.\\d| )";
const char S106326[] = "Skyfire/\\d+\\.\\d";
const char S106327[] = "Sleipnir/\\d+\\.\\d";
const char S106328[] = "QupZilla/\\d+\\.\\d";
const char S106329[] = "konqueror/\\d+\\.\\d";
const char S106330[] = "Google Earth Pro/\\d+\\.\\d";
const char S106331[] = "OmniWeb/.+\\.\\d+";
const char S106332[] = "Qt/\\d\\.\\d";
const char S106333[] = "kded/\\d\\.\\d";
const char S106334[] = "Midori/\\d\\.\\d";
const char S106335[] = "iCab/\\d\\.\\d";
const char S106336[] = "(?<=Fluid/)\\d\\.\\d";
const char S106337[] = "(?<=Iron/)\\d\\.\\d";
const char S106338[] = "(?<=Stainless/)\\d\\.\\d";
const char S106339[] = "Skyfire/";
const char S106340[] = "(?<=Skyfire/)[\\d\\.]+";
const char S106341[] = "Windows Phone OS";
const char S106342[] = "^WordPress/";
const char S106343[] = "(?<=http://)\\w+";
const char S106344[] = "Thunderbird";
const char S106345[] = "\\(Macintosh;|\\(Windows|\\(X11;";
const char S106346[] = "Firefox|Iceweasel|Fennec|Linux Mint|Ubuntu";
const char S106347[] = "(?<=Thunderbird/)\\d+\\.\\d+";
const char S106348[] = "(?<=(Fedora)/)\\d+\\.\\d+";
const char S106349[] = "Darwin";
const char S106350[] = "CFNetwork";
const char S106351[] = "^[^/]+";
const char S106352[] = "(i)?Mac([^\\d]+)?";
const char S106353[] = "(?<=Darwin/)\\d+\\.\\d+";
const char S106354[] = "^Dolphin/\\d+\\.\\d";
const char S106355[] = "Android 4\\.3|Android4\\.3";
const char S106356[] = "Android 4\\.4|Android4\\.4";
const char S106357[] = "(?i)alcatel";
const char S106358[] = "(?<=Alcatel-)[^/\\s]+";
const char S106359[] = "^Amoi";
const char S106360[] = "^Apple";
const char S106361[] = "CoreMedia";
const char S106362[] = "^AUDIOVOX";
const char S106363[] = "BenQ";
const char S106364[] = "BENQ";
const char S106365[] = "^BIRD";
const char S106366[] = "Black[B|b]erry";
const char S106367[] = "^Mozilla/";
const char S106368[] = "BlackBerry";
const char S106369[] = "Version/\\d+";
const char S106370[] = "(?<=Black[B|b]erry)[^/]+";
const char S106371[] = "(?<=Black[B|b]erry\\w{4}/)\\d+\\.\\d+\\.\\d+|(?<=Black[B|b]erry\\w{5}/)\\d+\\.\\d+\\.\\d+";
const char S106372[] = "(?<=UP.Browser/)\\d+\\.\\d+";
const char S106373[] = "UC Browser\\d+\\.\\d+";
const char S106374[] = "(?<=BlackBerry)[^;]+";
const char S106375[] = "(?<=Version/)\\d+\\.\\d+\\.\\d+";
const char S106376[] = "^DoCoMo";
const char S106377[] = "(?<=DoCoMo/)\\d+\\.\\d+";
const char S106378[] = "(?<=DoCoMo/\\d\\.\\d[\\s|/])[\\w\\d]+";
const char S106379[] = "^Fly|^FLY";
const char S106380[] = "^HD2|HTC";
const char S106381[] = "(?<=\\sHTC[\\s|_])[^;\\s]+";
const char S106382[] = "(?i)Huawei";
const char S106383[] = "^KDDI";
const char S106384[] = "KDDI-\\w+";
const char S106385[] = "^kyocera|^QC-|^KWC-";
const char S106386[] = "^LCT";
const char S106387[] = "^Lenovo";
const char S106388[] = "(?<=^Lenovo[-_])[\\w\\d]+";
const char S106389[] = "^LG/";
const char S106390[] = "(?<=^LG/)[\\w\\d]+";
const char S106391[] = "(?<=^LG(-|/))[^\\s/]+";
const char S106392[] = "(?<=NovarraVision)\\d\\.\\d";
const char S106393[] = "(?<=Browser/Teleca-)[^\\s]+";
const char S106394[] = "(?<=UCBrowser)\\d\\.\\d";
const char S106395[] = "(?<=Browser/Obigo-)[^/]+/[\\d\\.]+";
const char S106396[] = "(?<=Opera Mini/)[^\\);]+";
const char S106397[] = "^LG-";
const char S106398[] = "(?<=^LG-)[\\w\\d]+";
const char S106399[] = "UC( |)Browser(|/)\\d+\\.\\d";
const char S106400[] = "^LGE-";
const char S106401[] = "(?<=^LGE-)[\\w\\d]+";
const char S106402[] = "(?<=^LGE-)[^\\s/]+";
const char S106403[] = "^MOT-";
const char S106404[] = "(?<=^MOT-)[^/]+";
const char S106405[] = "(?<=UP\\.Browser/)[\\d\\.]+";
const char S106406[] = "\\([^\\)]+\\)";
const char S106407[] = "^NEC|KGT";
const char S106408[] = "^Nexian|^NEXIAN";
const char S106409[] = "(?i)nokia";
const char S106410[] = "Series(40|60)";
const char S106411[] = "(?i)(?<=Nokia)[^/\\)]+";
const char S106412[] = "(?<=S40OviBrowser/)\\d+\\.\\d+.\\d+";
const char S106413[] = "^iBrowser/Mini\\d+\\.\\d+";
const char S106414[] = "^OneBrowser/\\d+\\.\\d+";
const char S106415[] = "^MQQBrowser/Mini\\d+\\.\\d+";
const char S106416[] = "Palm";
const char S106417[] = "Palm(S|s)ource";
const char S106418[] = "^Panasonic";
const char S106419[] = "^Pantech|^PT-|^PANTECH|^PG-";
const char S106420[] = "^Philips|^PHILIPS";
const char S106421[] = "^Qtek";
const char S106422[] = "iPhone|iPod|iPad|Nokia|BlackBerry|Android|Skyfire";
const char S106423[] = "(?<=Mozilla/\\d\\.\\d \\()[^)]+";
const char S106424[] = "Phantomjs/\\d\\.\\d";
const char S106425[] = "LG Browser/\\d\\.\\d";
const char S106426[] = "(?<=LGE;)[^;]+";
const char S106427[] = "^Sagem|^SAGEM";
const char S106428[] = "(?i)SAMSUNG|SEC-|SCH-";
const char S106429[] = "Series60/";
const char S106430[] = "Samsung/";
const char S106431[] = "SAMSUNG-GT-";
const char S106432[] = "(?i)(?<=SAMSUNG)-?[^/]+";
const char S106433[] = "(?i)(?<=SEC|SCH)-?[^/]+";
const char S106434[] = "(?<=Bada/)\\d+\\.\\d+";
const char S106435[] = "(?<=Dolfin/)\\d+\\.\\d+";
const char S106436[] = "(?<=Jasmine/)\\d+\\.\\d+";
const char S106437[] = "UC(\\.|)Browser/\\d+\\.\\d+";
const char S106438[] = "(?<=Windows)\\w+;";
const char S106439[] = "Android \\d\\.\\d";
const char S106440[] = "UCWEB\\d\\.\\d";
const char S106441[] = "(?<=SAMSUNG-GT-)[^/-]+";
const char S106442[] = "(?<=UC\\.Browser/)\\d+\\.\\d+";
const char S106443[] = "(?<=MSIEMobile )\\d+\\.\\d+";
const char S106444[] = "(?<=Samsung/)[^/]+";
const char S106445[] = "(?<=Series60/)[\\d\\.]+";
const char S106446[] = "(?<=SymbianOS/)[\\d\\.]+";
const char S106447[] = "^Sanyo|^SANYO";
const char S106448[] = "Sanyo";
const char S106449[] = "SCP";
const char S106450[] = "^Sharp|^SHARP";
const char S106451[] = "^SIE-";
const char S106452[] = "^SoftBank|^Vodafone|^J-PHONE";
const char S106453[] = "(SoftBank|Vodafone|J-PHONE)\\/\\d\\.\\d\\/\\w+";
const char S106454[] = "Spice";
const char S106455[] = "^TIANYU-KTOUCH";
const char S106456[] = "(?<=TIANYU-KTOUCH/)[^/]+";
const char S106457[] = "^Vodafone";
const char S106458[] = "webOS";
const char S106459[] = "HTC";
const char S106460[] = "(?<=\\(|;|\\))[^;\\)]+";
const char S106461[] = "^ZTE";
const char S106462[] = "(?i)Webkit";
const char S106463[] = "Chrome|Safari|Android|Nokia|BlackBerry";
const char S106464[] = "(?<=\\(KHTML, like Gecko\\))[^\\(\\)]+";
const char S106465[] = "Fennec|Linux Mint|Firefox|IceWeasel|Thunderbird|WebKit|BonEcho";
const char S106466[] = "Windows NT \\d\\.\\d(?=;)|Mac OS X [^\\s]+(?=;)|Ubuntu|X11|Linux i686";
const char S106467[] = "Win\\d(\\d|x)";
const char S106468[] = "Fedora";
const char S106469[] = "(?<=StumbleUpon/)\\d\\.\\d";
const char S106470[] = "Postbox/\\d+\\.\\d";
const char S106471[] = "Minefield/\\d\\.\\d";
const char S106472[] = "Netscape(\\d|)/\\d\\.\\d";
const char S106473[] = "lolifox/\\d+\\.\\d";
const char S106474[] = "Icedove/\\d+\\.\\d";
const char S106475[] = "zdesktop/\\d+\\.\\d";
const char S106476[] = "Abrowser/\\d+\\.\\d";
const char S106477[] = "Mnenhy/\\d+\\.\\d";
const char S106478[] = "Konqueror/\\d+\\.\\d";
const char S106479[] = "Ubuntu/\\d+\\.\\d";
const char S106480[] = "Palemoon/\\d+\\.\\d";
const char S106481[] = "SeaMonkey(/\\d\\.\\d+|$)";
const char S106482[] = "GranParadiso/\\d\\.\\d";
const char S106483[] = "K9Browser/\\d+\\.\\d";
const char S106484[] = "iTunes";
const char S106485[] = "iPod/\\d\\.\\d";
const char S106486[] = "(?<=Opera/)[\\d+.]+|(?<=Opera )[\\d\\.]+";
const char S106487[] = "(?<=Presto/)\\d+\\.\\d+";
const char S106488[] = "Opera Mini";
const char S106489[] = "(?<=Opera Mini/att/)\\d(/|\\.)\\d";
const char S106490[] = "^[^;]+";
const char S106491[] = "(?<=Opera Mini/)\\d(/|\\.)\\d";
const char S106492[] = "(?<= \\d\\d\\dX\\d\\d\\d).+";
const char S106493[] = "(?<=Windows NT )\\d\\.\\d";
const char S106494[] = "Opera Mini//\\d+";
const char S106495[] = "Version/\\d+\\.\\d+";
const char S106496[] = "Opera Mobi";
const char S106497[] = "^portalmmm";
const char S106498[] = "Outlook";
const char S106499[] = "(?<=Microsoft Outlook )[\\d\\.]+";
const char S106500[] = "(?<=Outlook 20\\d\\d)[\\d\\.]+";
const char S106501[] = "Profile/MIDP-\\d\\.\\d Configuration/CLDC-\\d\\.\\d";
const char S106502[] = ".+(?=Profile/MIDP-\\d\\.\\d Configuration/CLDC-\\d\\.\\d)";
const char S106503[] = "WAP|MIDP|CLDC|Teleca-|Screen-|mobile";
const char S106504[] = "^iOS/.+dataaccessd";
const char S106505[] = "^Podcasts";
const char S106506[] = "^Tiny Tiny RSS";
const char S106507[] = "^Amazon CloudFront";
const char S106508[] = "^Wada\\.vn";
const char S106509[] = "^FeedDemon";
const char S106510[] = "^python-requests";
const char S106511[] = "^RSSInclude";
const char S106512[] = "^curl";
const char S106513[] = "^Pulp/";
const char S106514[] = "Win98";
const char S106515[] = "AppEngine";
const char S106516[] = "^[^\\(]+";
const char S106517[] = "darwin";
const char S106518[] = "^(I|J)UC";
const char S106519[] = "^(J|I)UC";
const char S106520[] = "UCWEB\\d+\\.\\d";
const char S106521[] = "(?<=(Z|z)h(-|_)cn;)[^;]+";
const char S106522[] = "Linux;\\sU;\\s\\d\\.\\d";
const char S106523[] = "iOS \\d\\.\\d";
const char S106524[] = "^server-bag";
const char S106525[] = "iPhone OS,\\d\\.\\d";
const char S106526[] = ",iPhone|,iPod|,iPad";
const char S106527[] = "^-1";
const char S106528[] = "^MicroMax|^Micromax";
const char S106529[] = "^Micromax.+$";
const char S106530[] = "jig browser";
const char S106531[] = "1.0.4; .+\\)$";
const char S106532[] = "Konqueror";
const char S106533[] = "Konqueror/\\d\\.\\d";
const char S106534[] = "Ubuntu";
const char S106535[] = "heritrix";
const char S106536[] = "heritrix/\\d\\.\\d";
const char S106537[] = "Java/\\d\\.\\d";
const char S106538[] = "(?<=Java/)\\d\\.\\d";
const char S106539[] = "Windows .+\\)";
const char S106540[] = "Max OS X \\d+\\.\\d";
const char S106541[] = "UCWEB";
const char S106542[] = "(?<=UCBrowser)(|/)\\d\\.\\d";
const char S106543[] = "(?<=\\w\\w-\\w\\w;)[^\\)]+";
const char S106544[] = "^MacOutlook";
const char S106545[] = "(?<=MacOutlook/)[\\d\\.]+";
const char S106546[] = "(?<=Mac OS X )[\\d\\.]+";
const char S106547[] = "\\w+";


const Profile P5477 = {"5477", NULL};	/*Profile ID - 5477*/
const Value V4861_279_57148 = {S0, NULL};	/*Value = False*/
const Property P4861_279 = {S1, 0, &V4861_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V4861_282_57153 = {S2, NULL};	/*Value = True*/
const Property P4861_282 = {S3, 0, &V4861_282_57153, &P4861_279};	/*Property = BlobBuilder*/
const Value V4861_350_57444 = {S2, NULL};	/*Value = True*/
const Property P4861_350 = {S4, 0, &V4861_350_57444, &P4861_282};	/*Property = Canvas*/
const Value V4861_283_57159 = {S2, NULL};	/*Value = True*/
const Property P4861_283 = {S5, 0, &V4861_283_57159, &P4861_350};	/*Property = CssBackground*/
const Value V4861_284_57163 = {S0, NULL};	/*Value = False*/
const Property P4861_284 = {S6, 0, &V4861_284_57163, &P4861_283};	/*Property = CssBorderImage*/
const Value V4861_285_57166 = {S0, NULL};	/*Value = False*/
const Property P4861_285 = {S7, 0, &V4861_285_57166, &P4861_284};	/*Property = CssCanvas*/
const Value V4861_286_57168 = {S2, NULL};	/*Value = True*/
const Property P4861_286 = {S8, 0, &V4861_286_57168, &P4861_285};	/*Property = CssColor*/
const Value V4861_237_55784 = {S2, NULL};	/*Value = True*/
const Property P4861_237 = {S9, 0, &V4861_237_55784, &P4861_286};	/*Property = CssColumn*/
const Value V4861_287_57172 = {S0, NULL};	/*Value = False*/
const Property P4861_287 = {S10, 0, &V4861_287_57172, &P4861_237};	/*Property = CssFlexbox*/
const Value V4861_288_57174 = {S2, NULL};	/*Value = True*/
const Property P4861_288 = {S11, 0, &V4861_288_57174, &P4861_287};	/*Property = CssFont*/
const Value V4861_289_57759 = {S0, NULL};	/*Value = False*/
const Property P4861_289 = {S12, 0, &V4861_289_57759, &P4861_288};	/*Property = CssImages*/
const Value V4861_291_57180 = {S2, NULL};	/*Value = True*/
const Property P4861_291 = {S13, 0, &V4861_291_57180, &P4861_289};	/*Property = CssMediaQueries*/
const Value V4861_292_57183 = {S2, NULL};	/*Value = True*/
const Property P4861_292 = {S14, 0, &V4861_292_57183, &P4861_291};	/*Property = CssMinMax*/
const Value V4861_293_57187 = {S0, NULL};	/*Value = False*/
const Property P4861_293 = {S15, 0, &V4861_293_57187, &P4861_292};	/*Property = CssOverflow*/
const Value V4861_294_57189 = {S2, NULL};	/*Value = True*/
const Property P4861_294 = {S16, 0, &V4861_294_57189, &P4861_293};	/*Property = CssPosition*/
const Value V4861_295_57192 = {S2, NULL};	/*Value = True*/
const Property P4861_295 = {S17, 0, &V4861_295_57192, &P4861_294};	/*Property = CssText*/
const Value V4861_297_57195 = {S2, NULL};	/*Value = True*/
const Property P4861_297 = {S18, 0, &V4861_297_57195, &P4861_295};	/*Property = CssTransforms*/
const Value V4861_241_55799 = {S2, NULL};	/*Value = True*/
const Property P4861_241 = {S19, 0, &V4861_241_55799, &P4861_297};	/*Property = CssTransitions*/
const Value V4861_298_57206 = {S2, NULL};	/*Value = True*/
const Property P4861_298 = {S20, 0, &V4861_298_57206, &P4861_241};	/*Property = CssUI*/
const Value V4861_300_57216 = {S0, NULL};	/*Value = False*/
const Property P4861_300 = {S21, 0, &V4861_300_57216, &P4861_298};	/*Property = DataSet*/
const Value V4861_301_57218 = {S2, NULL};	/*Value = True*/
const Property P4861_301 = {S22, 0, &V4861_301_57218, &P4861_300};	/*Property = DataUrl*/
const Value V4861_302_57222 = {S0, NULL};	/*Value = False*/
const Property P4861_302 = {S23, 0, &V4861_302_57222, &P4861_301};	/*Property = DeviceOrientation*/
const Value V4861_304_57227 = {S2, NULL};	/*Value = True*/
const Property P4861_304 = {S24, 0, &V4861_304_57227, &P4861_302};	/*Property = FileReader*/
const Value V4861_305_57231 = {S0, NULL};	/*Value = False*/
const Property P4861_305 = {S25, 0, &V4861_305_57231, &P4861_304};	/*Property = FileSaver*/
const Value V4861_306_57237 = {S0, NULL};	/*Value = False*/
const Property P4861_306 = {S26, 0, &V4861_306_57237, &P4861_305};	/*Property = FileWriter*/
const Value V4861_307_57239 = {S2, NULL};	/*Value = True*/
const Property P4861_307 = {S27, 0, &V4861_307_57239, &P4861_306};	/*Property = FormData*/
const Value V4861_309_57246 = {S0, NULL};	/*Value = False*/
const Property P4861_309 = {S28, 0, &V4861_309_57246, &P4861_307};	/*Property = Fullscreen*/
const Value V4861_310_57248 = {S2, NULL};	/*Value = True*/
const Property P4861_310 = {S29, 0, &V4861_310_57248, &P4861_309};	/*Property = GeoLocation*/
const Value V4861_312_57254 = {S2, NULL};	/*Value = True*/
const Property P4861_312 = {S30, 0, &V4861_312_57254, &P4861_310};	/*Property = History*/
const Value V4861_314_57261 = {S0, NULL};	/*Value = False*/
const Property P4861_314 = {S31, 0, &V4861_314_57261, &P4861_312};	/*Property = Html5*/
const Value V4861_313_57258 = {S0, NULL};	/*Value = False*/
const Property P4861_313 = {S32, 0, &V4861_313_57258, &P4861_314};	/*Property = Html-Media-Capture*/
const Value V4861_315_57264 = {S0, NULL};	/*Value = False*/
const Property P4861_315 = {S33, 0, &V4861_315_57264, &P4861_313};	/*Property = Iframe*/
const Value V4861_316_57266 = {S2, NULL};	/*Value = True*/
const Property P4861_316 = {S34, 0, &V4861_316_57266, &P4861_315};	/*Property = IndexedDB*/
const Value V4861_317_57269 = {S2, NULL};	/*Value = True*/
const Property P4861_317 = {S35, 0, &V4861_317_57269, &P4861_316};	/*Property = Json*/
const Value V4861_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P4861_15 = {S37, 0, &V4861_15_28, &P4861_317};	/*Property = LayoutEngine*/
const Value V4861_344_57272 = {S2, NULL};	/*Value = True*/
const Property P4861_344 = {S38, 0, &V4861_344_57272, &P4861_15};	/*Property = Masking*/
const Value V4861_320_57290 = {S2, NULL};	/*Value = True*/
const Property P4861_320 = {S39, 0, &V4861_320_57290, &P4861_344};	/*Property = PostMessage*/
const Value V4861_321_57293 = {S2, NULL};	/*Value = True*/
const Property P4861_321 = {S40, 0, &V4861_321_57293, &P4861_320};	/*Property = Progress*/
const Value V4861_322_57296 = {S2, NULL};	/*Value = True*/
const Property P4861_322 = {S41, 0, &V4861_322_57296, &P4861_321};	/*Property = Prompts*/
const Value V4861_323_57299 = {S2, NULL};	/*Value = True*/
const Property P4861_323 = {S42, 0, &V4861_323_57299, &P4861_322};	/*Property = Selector*/
const Value V4861_324_57306 = {S0, NULL};	/*Value = False*/
const Property P4861_324 = {S43, 0, &V4861_324_57306, &P4861_323};	/*Property = Svg*/
const Value V4861_327_57315 = {S0, NULL};	/*Value = False*/
const Property P4861_327 = {S44, 0, &V4861_327_57315, &P4861_324};	/*Property = TouchEvents*/
const Value V4861_328_57318 = {S0, NULL};	/*Value = False*/
const Property P4861_328 = {S45, 0, &V4861_328_57318, &P4861_327};	/*Property = Track*/
const Value V4861_329_57323 = {S2, NULL};	/*Value = True*/
const Property P4861_329 = {S46, 0, &V4861_329_57323, &P4861_328};	/*Property = Video*/
const Value V4861_330_57326 = {S2, NULL};	/*Value = True*/
const Property P4861_330 = {S47, 0, &V4861_330_57326, &P4861_329};	/*Property = Viewport*/
const Value V4861_332_57338 = {S2, NULL};	/*Value = True*/
const Property P4861_332 = {S48, 0, &V4861_332_57338, &P4861_330};	/*Property = WebWorkers*/
const Value V4861_333_57342 = {S0, NULL};	/*Value = False*/
const Property P4861_333 = {S49, 0, &V4861_333_57342, &P4861_332};	/*Property = Xhr2*/
const Profile P4861 = {"4861", &P4861_333};	/*Profile ID - 4861*/
const Value V12496_1_1 = {S2, NULL};	/*Value = True*/
const Property P12496_1 = {S50, 0, &V12496_1_1, NULL};	/*Property = IsMobile*/
const Value V12496_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12496_5 = {S52, 0, &V12496_5_1227, &P12496_1};	/*Property = ScreenPixelsHeight*/
const Value V12496_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P12496_4 = {S54, 0, &V12496_4_1943, &P12496_5};	/*Property = ScreenPixelsWidth*/
const Profile P12496 = {"12496", &P12496_4};	/*Profile ID - 12496*/
const Profile P18092 = {"18092", NULL};	/*Profile ID - 18092*/
const Value V3508_279_57148 = {S0, NULL};	/*Value = False*/
const Property P3508_279 = {S1, 0, &V3508_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V3508_282_57154 = {S0, NULL};	/*Value = False*/
const Property P3508_282 = {S3, 0, &V3508_282_57154, &P3508_279};	/*Property = BlobBuilder*/
const Value V3508_350_57444 = {S2, NULL};	/*Value = True*/
const Property P3508_350 = {S4, 0, &V3508_350_57444, &P3508_282};	/*Property = Canvas*/
const Value V3508_283_57159 = {S2, NULL};	/*Value = True*/
const Property P3508_283 = {S5, 0, &V3508_283_57159, &P3508_350};	/*Property = CssBackground*/
const Value V3508_284_57163 = {S0, NULL};	/*Value = False*/
const Property P3508_284 = {S6, 0, &V3508_284_57163, &P3508_283};	/*Property = CssBorderImage*/
const Value V3508_285_57166 = {S0, NULL};	/*Value = False*/
const Property P3508_285 = {S7, 0, &V3508_285_57166, &P3508_284};	/*Property = CssCanvas*/
const Value V3508_286_57168 = {S2, NULL};	/*Value = True*/
const Property P3508_286 = {S8, 0, &V3508_286_57168, &P3508_285};	/*Property = CssColor*/
const Value V3508_237_55784 = {S2, NULL};	/*Value = True*/
const Property P3508_237 = {S9, 0, &V3508_237_55784, &P3508_286};	/*Property = CssColumn*/
const Value V3508_287_57172 = {S0, NULL};	/*Value = False*/
const Property P3508_287 = {S10, 0, &V3508_287_57172, &P3508_237};	/*Property = CssFlexbox*/
const Value V3508_288_57175 = {S0, NULL};	/*Value = False*/
const Property P3508_288 = {S11, 0, &V3508_288_57175, &P3508_287};	/*Property = CssFont*/
const Value V3508_289_57759 = {S0, NULL};	/*Value = False*/
const Property P3508_289 = {S12, 0, &V3508_289_57759, &P3508_288};	/*Property = CssImages*/
const Value V3508_291_57181 = {S0, NULL};	/*Value = False*/
const Property P3508_291 = {S13, 0, &V3508_291_57181, &P3508_289};	/*Property = CssMediaQueries*/
const Value V3508_292_57183 = {S2, NULL};	/*Value = True*/
const Property P3508_292 = {S14, 0, &V3508_292_57183, &P3508_291};	/*Property = CssMinMax*/
const Value V3508_293_57187 = {S0, NULL};	/*Value = False*/
const Property P3508_293 = {S15, 0, &V3508_293_57187, &P3508_292};	/*Property = CssOverflow*/
const Value V3508_294_57190 = {S0, NULL};	/*Value = False*/
const Property P3508_294 = {S16, 0, &V3508_294_57190, &P3508_293};	/*Property = CssPosition*/
const Value V3508_295_57192 = {S2, NULL};	/*Value = True*/
const Property P3508_295 = {S17, 0, &V3508_295_57192, &P3508_294};	/*Property = CssText*/
const Value V3508_297_57195 = {S2, NULL};	/*Value = True*/
const Property P3508_297 = {S18, 0, &V3508_297_57195, &P3508_295};	/*Property = CssTransforms*/
const Value V3508_241_55799 = {S2, NULL};	/*Value = True*/
const Property P3508_241 = {S19, 0, &V3508_241_55799, &P3508_297};	/*Property = CssTransitions*/
const Value V3508_298_57206 = {S2, NULL};	/*Value = True*/
const Property P3508_298 = {S20, 0, &V3508_298_57206, &P3508_241};	/*Property = CssUI*/
const Value V3508_300_57216 = {S0, NULL};	/*Value = False*/
const Property P3508_300 = {S21, 0, &V3508_300_57216, &P3508_298};	/*Property = DataSet*/
const Value V3508_301_57218 = {S2, NULL};	/*Value = True*/
const Property P3508_301 = {S22, 0, &V3508_301_57218, &P3508_300};	/*Property = DataUrl*/
const Value V3508_302_57222 = {S0, NULL};	/*Value = False*/
const Property P3508_302 = {S23, 0, &V3508_302_57222, &P3508_301};	/*Property = DeviceOrientation*/
const Value V3508_304_57228 = {S0, NULL};	/*Value = False*/
const Property P3508_304 = {S24, 0, &V3508_304_57228, &P3508_302};	/*Property = FileReader*/
const Value V3508_305_57231 = {S0, NULL};	/*Value = False*/
const Property P3508_305 = {S25, 0, &V3508_305_57231, &P3508_304};	/*Property = FileSaver*/
const Value V3508_306_57237 = {S0, NULL};	/*Value = False*/
const Property P3508_306 = {S26, 0, &V3508_306_57237, &P3508_305};	/*Property = FileWriter*/
const Value V3508_307_57240 = {S0, NULL};	/*Value = False*/
const Property P3508_307 = {S27, 0, &V3508_307_57240, &P3508_306};	/*Property = FormData*/
const Value V3508_309_57246 = {S0, NULL};	/*Value = False*/
const Property P3508_309 = {S28, 0, &V3508_309_57246, &P3508_307};	/*Property = Fullscreen*/
const Value V3508_310_57248 = {S2, NULL};	/*Value = True*/
const Property P3508_310 = {S29, 0, &V3508_310_57248, &P3508_309};	/*Property = GeoLocation*/
const Value V3508_312_57255 = {S0, NULL};	/*Value = False*/
const Property P3508_312 = {S30, 0, &V3508_312_57255, &P3508_310};	/*Property = History*/
const Value V3508_314_57261 = {S0, NULL};	/*Value = False*/
const Property P3508_314 = {S31, 0, &V3508_314_57261, &P3508_312};	/*Property = Html5*/
const Value V3508_313_57258 = {S0, NULL};	/*Value = False*/
const Property P3508_313 = {S32, 0, &V3508_313_57258, &P3508_314};	/*Property = Html-Media-Capture*/
const Value V3508_315_57264 = {S0, NULL};	/*Value = False*/
const Property P3508_315 = {S33, 0, &V3508_315_57264, &P3508_313};	/*Property = Iframe*/
const Value V3508_316_57267 = {S0, NULL};	/*Value = False*/
const Property P3508_316 = {S34, 0, &V3508_316_57267, &P3508_315};	/*Property = IndexedDB*/
const Value V3508_317_57269 = {S2, NULL};	/*Value = True*/
const Property P3508_317 = {S35, 0, &V3508_317_57269, &P3508_316};	/*Property = Json*/
const Value V3508_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3508_15 = {S37, 0, &V3508_15_28, &P3508_317};	/*Property = LayoutEngine*/
const Value V3508_344_57272 = {S2, NULL};	/*Value = True*/
const Property P3508_344 = {S38, 0, &V3508_344_57272, &P3508_15};	/*Property = Masking*/
const Value V3508_320_57290 = {S2, NULL};	/*Value = True*/
const Property P3508_320 = {S39, 0, &V3508_320_57290, &P3508_344};	/*Property = PostMessage*/
const Value V3508_321_57293 = {S2, NULL};	/*Value = True*/
const Property P3508_321 = {S40, 0, &V3508_321_57293, &P3508_320};	/*Property = Progress*/
const Value V3508_322_57296 = {S2, NULL};	/*Value = True*/
const Property P3508_322 = {S41, 0, &V3508_322_57296, &P3508_321};	/*Property = Prompts*/
const Value V3508_323_57299 = {S2, NULL};	/*Value = True*/
const Property P3508_323 = {S42, 0, &V3508_323_57299, &P3508_322};	/*Property = Selector*/
const Value V3508_324_57306 = {S0, NULL};	/*Value = False*/
const Property P3508_324 = {S43, 0, &V3508_324_57306, &P3508_323};	/*Property = Svg*/
const Value V3508_327_57315 = {S0, NULL};	/*Value = False*/
const Property P3508_327 = {S44, 0, &V3508_327_57315, &P3508_324};	/*Property = TouchEvents*/
const Value V3508_328_57318 = {S0, NULL};	/*Value = False*/
const Property P3508_328 = {S45, 0, &V3508_328_57318, &P3508_327};	/*Property = Track*/
const Value V3508_329_57323 = {S2, NULL};	/*Value = True*/
const Property P3508_329 = {S46, 0, &V3508_329_57323, &P3508_328};	/*Property = Video*/
const Value V3508_330_57326 = {S2, NULL};	/*Value = True*/
const Property P3508_330 = {S47, 0, &V3508_330_57326, &P3508_329};	/*Property = Viewport*/
const Value V3508_332_57339 = {S0, NULL};	/*Value = False*/
const Property P3508_332 = {S48, 0, &V3508_332_57339, &P3508_330};	/*Property = WebWorkers*/
const Value V3508_333_57342 = {S0, NULL};	/*Value = False*/
const Property P3508_333 = {S49, 0, &V3508_333_57342, &P3508_332};	/*Property = Xhr2*/
const Profile P3508 = {"3508", &P3508_333};	/*Profile ID - 3508*/
const Profile P5096 = {"5096", NULL};	/*Profile ID - 5096*/
const Profile P5267 = {"5267", NULL};	/*Profile ID - 5267*/
const Value V3834_279_57148 = {S0, NULL};	/*Value = False*/
const Property P3834_279 = {S1, 0, &V3834_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V3834_282_57154 = {S0, NULL};	/*Value = False*/
const Property P3834_282 = {S3, 0, &V3834_282_57154, &P3834_279};	/*Property = BlobBuilder*/
const Value V3834_350_57444 = {S2, NULL};	/*Value = True*/
const Property P3834_350 = {S4, 0, &V3834_350_57444, &P3834_282};	/*Property = Canvas*/
const Value V3834_283_57159 = {S2, NULL};	/*Value = True*/
const Property P3834_283 = {S5, 0, &V3834_283_57159, &P3834_350};	/*Property = CssBackground*/
const Value V3834_284_57163 = {S0, NULL};	/*Value = False*/
const Property P3834_284 = {S6, 0, &V3834_284_57163, &P3834_283};	/*Property = CssBorderImage*/
const Value V3834_285_57166 = {S0, NULL};	/*Value = False*/
const Property P3834_285 = {S7, 0, &V3834_285_57166, &P3834_284};	/*Property = CssCanvas*/
const Value V3834_286_57168 = {S2, NULL};	/*Value = True*/
const Property P3834_286 = {S8, 0, &V3834_286_57168, &P3834_285};	/*Property = CssColor*/
const Value V3834_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3834_237 = {S9, 0, &V3834_237_55786, &P3834_286};	/*Property = CssColumn*/
const Value V3834_287_57172 = {S0, NULL};	/*Value = False*/
const Property P3834_287 = {S10, 0, &V3834_287_57172, &P3834_237};	/*Property = CssFlexbox*/
const Value V3834_288_57175 = {S0, NULL};	/*Value = False*/
const Property P3834_288 = {S11, 0, &V3834_288_57175, &P3834_287};	/*Property = CssFont*/
const Value V3834_289_57759 = {S0, NULL};	/*Value = False*/
const Property P3834_289 = {S12, 0, &V3834_289_57759, &P3834_288};	/*Property = CssImages*/
const Value V3834_291_57180 = {S2, NULL};	/*Value = True*/
const Property P3834_291 = {S13, 0, &V3834_291_57180, &P3834_289};	/*Property = CssMediaQueries*/
const Value V3834_292_57183 = {S2, NULL};	/*Value = True*/
const Property P3834_292 = {S14, 0, &V3834_292_57183, &P3834_291};	/*Property = CssMinMax*/
const Value V3834_293_57187 = {S0, NULL};	/*Value = False*/
const Property P3834_293 = {S15, 0, &V3834_293_57187, &P3834_292};	/*Property = CssOverflow*/
const Value V3834_294_57190 = {S0, NULL};	/*Value = False*/
const Property P3834_294 = {S16, 0, &V3834_294_57190, &P3834_293};	/*Property = CssPosition*/
const Value V3834_295_57192 = {S2, NULL};	/*Value = True*/
const Property P3834_295 = {S17, 0, &V3834_295_57192, &P3834_294};	/*Property = CssText*/
const Value V3834_297_57195 = {S2, NULL};	/*Value = True*/
const Property P3834_297 = {S18, 0, &V3834_297_57195, &P3834_295};	/*Property = CssTransforms*/
const Value V3834_241_55799 = {S2, NULL};	/*Value = True*/
const Property P3834_241 = {S19, 0, &V3834_241_55799, &P3834_297};	/*Property = CssTransitions*/
const Value V3834_298_57206 = {S2, NULL};	/*Value = True*/
const Property P3834_298 = {S20, 0, &V3834_298_57206, &P3834_241};	/*Property = CssUI*/
const Value V3834_300_57216 = {S0, NULL};	/*Value = False*/
const Property P3834_300 = {S21, 0, &V3834_300_57216, &P3834_298};	/*Property = DataSet*/
const Value V3834_301_57218 = {S2, NULL};	/*Value = True*/
const Property P3834_301 = {S22, 0, &V3834_301_57218, &P3834_300};	/*Property = DataUrl*/
const Value V3834_302_57221 = {S2, NULL};	/*Value = True*/
const Property P3834_302 = {S23, 0, &V3834_302_57221, &P3834_301};	/*Property = DeviceOrientation*/
const Value V3834_304_57228 = {S0, NULL};	/*Value = False*/
const Property P3834_304 = {S24, 0, &V3834_304_57228, &P3834_302};	/*Property = FileReader*/
const Value V3834_305_57231 = {S0, NULL};	/*Value = False*/
const Property P3834_305 = {S25, 0, &V3834_305_57231, &P3834_304};	/*Property = FileSaver*/
const Value V3834_306_57237 = {S0, NULL};	/*Value = False*/
const Property P3834_306 = {S26, 0, &V3834_306_57237, &P3834_305};	/*Property = FileWriter*/
const Value V3834_307_57239 = {S2, NULL};	/*Value = True*/
const Property P3834_307 = {S27, 0, &V3834_307_57239, &P3834_306};	/*Property = FormData*/
const Value V3834_309_57246 = {S0, NULL};	/*Value = False*/
const Property P3834_309 = {S28, 0, &V3834_309_57246, &P3834_307};	/*Property = Fullscreen*/
const Value V3834_310_57248 = {S2, NULL};	/*Value = True*/
const Property P3834_310 = {S29, 0, &V3834_310_57248, &P3834_309};	/*Property = GeoLocation*/
const Value V3834_312_57255 = {S0, NULL};	/*Value = False*/
const Property P3834_312 = {S30, 0, &V3834_312_57255, &P3834_310};	/*Property = History*/
const Value V3834_314_57261 = {S0, NULL};	/*Value = False*/
const Property P3834_314 = {S31, 0, &V3834_314_57261, &P3834_312};	/*Property = Html5*/
const Value V3834_313_57258 = {S0, NULL};	/*Value = False*/
const Property P3834_313 = {S32, 0, &V3834_313_57258, &P3834_314};	/*Property = Html-Media-Capture*/
const Value V3834_315_57264 = {S0, NULL};	/*Value = False*/
const Property P3834_315 = {S33, 0, &V3834_315_57264, &P3834_313};	/*Property = Iframe*/
const Value V3834_316_57267 = {S0, NULL};	/*Value = False*/
const Property P3834_316 = {S34, 0, &V3834_316_57267, &P3834_315};	/*Property = IndexedDB*/
const Value V3834_317_57269 = {S2, NULL};	/*Value = True*/
const Property P3834_317 = {S35, 0, &V3834_317_57269, &P3834_316};	/*Property = Json*/
const Value V3834_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3834_15 = {S37, 0, &V3834_15_28, &P3834_317};	/*Property = LayoutEngine*/
const Value V3834_344_57272 = {S2, NULL};	/*Value = True*/
const Property P3834_344 = {S38, 0, &V3834_344_57272, &P3834_15};	/*Property = Masking*/
const Value V3834_320_57290 = {S2, NULL};	/*Value = True*/
const Property P3834_320 = {S39, 0, &V3834_320_57290, &P3834_344};	/*Property = PostMessage*/
const Value V3834_321_57293 = {S2, NULL};	/*Value = True*/
const Property P3834_321 = {S40, 0, &V3834_321_57293, &P3834_320};	/*Property = Progress*/
const Value V3834_322_57296 = {S2, NULL};	/*Value = True*/
const Property P3834_322 = {S41, 0, &V3834_322_57296, &P3834_321};	/*Property = Prompts*/
const Value V3834_323_57299 = {S2, NULL};	/*Value = True*/
const Property P3834_323 = {S42, 0, &V3834_323_57299, &P3834_322};	/*Property = Selector*/
const Value V3834_324_57306 = {S0, NULL};	/*Value = False*/
const Property P3834_324 = {S43, 0, &V3834_324_57306, &P3834_323};	/*Property = Svg*/
const Value V3834_327_57315 = {S0, NULL};	/*Value = False*/
const Property P3834_327 = {S44, 0, &V3834_327_57315, &P3834_324};	/*Property = TouchEvents*/
const Value V3834_328_57318 = {S0, NULL};	/*Value = False*/
const Property P3834_328 = {S45, 0, &V3834_328_57318, &P3834_327};	/*Property = Track*/
const Value V3834_329_57323 = {S2, NULL};	/*Value = True*/
const Property P3834_329 = {S46, 0, &V3834_329_57323, &P3834_328};	/*Property = Video*/
const Value V3834_330_57326 = {S2, NULL};	/*Value = True*/
const Property P3834_330 = {S47, 0, &V3834_330_57326, &P3834_329};	/*Property = Viewport*/
const Value V3834_332_57339 = {S0, NULL};	/*Value = False*/
const Property P3834_332 = {S48, 0, &V3834_332_57339, &P3834_330};	/*Property = WebWorkers*/
const Value V3834_333_57342 = {S0, NULL};	/*Value = False*/
const Property P3834_333 = {S49, 0, &V3834_333_57342, &P3834_332};	/*Property = Xhr2*/
const Profile P3834 = {"3834", &P3834_333};	/*Profile ID - 3834*/
const Value V13553_1_1 = {S2, NULL};	/*Value = True*/
const Property P13553_1 = {S50, 0, &V13553_1_1, NULL};	/*Property = IsMobile*/
const Value V13553_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13553_5 = {S52, 0, &V13553_5_1227, &P13553_1};	/*Property = ScreenPixelsHeight*/
const Value V13553_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13553_4 = {S54, 0, &V13553_4_1229, &P13553_5};	/*Property = ScreenPixelsWidth*/
const Profile P13553 = {"13553", &P13553_4};	/*Profile ID - 13553*/
const Value V14232_1_1 = {S2, NULL};	/*Value = True*/
const Property P14232_1 = {S50, 0, &V14232_1_1, NULL};	/*Property = IsMobile*/
const Value V14232_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14232_5 = {S52, 0, &V14232_5_1222, &P14232_1};	/*Property = ScreenPixelsHeight*/
const Value V14232_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14232_4 = {S54, 0, &V14232_4_1224, &P14232_5};	/*Property = ScreenPixelsWidth*/
const Profile P14232 = {"14232", &P14232_4};	/*Profile ID - 14232*/
const Profile P17483 = {"17483", NULL};	/*Profile ID - 17483*/
const Value V17602_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_279 = {S1, 0, &V17602_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17602_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_282 = {S3, 0, &V17602_282_57155, &P17602_279};	/*Property = BlobBuilder*/
const Value V17602_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_350 = {S4, 0, &V17602_350_57446, &P17602_282};	/*Property = Canvas*/
const Value V17602_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_283 = {S5, 0, &V17602_283_57161, &P17602_350};	/*Property = CssBackground*/
const Value V17602_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_284 = {S6, 0, &V17602_284_57164, &P17602_283};	/*Property = CssBorderImage*/
const Value V17602_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_285 = {S7, 0, &V17602_285_57167, &P17602_284};	/*Property = CssCanvas*/
const Value V17602_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_286 = {S8, 0, &V17602_286_57170, &P17602_285};	/*Property = CssColor*/
const Value V17602_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_237 = {S9, 0, &V17602_237_55786, &P17602_286};	/*Property = CssColumn*/
const Value V17602_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_287 = {S10, 0, &V17602_287_57173, &P17602_237};	/*Property = CssFlexbox*/
const Value V17602_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_288 = {S11, 0, &V17602_288_57176, &P17602_287};	/*Property = CssFont*/
const Value V17602_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_289 = {S12, 0, &V17602_289_86899, &P17602_288};	/*Property = CssImages*/
const Value V17602_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_291 = {S13, 0, &V17602_291_57182, &P17602_289};	/*Property = CssMediaQueries*/
const Value V17602_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_292 = {S14, 0, &V17602_292_57185, &P17602_291};	/*Property = CssMinMax*/
const Value V17602_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_293 = {S15, 0, &V17602_293_57188, &P17602_292};	/*Property = CssOverflow*/
const Value V17602_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_294 = {S16, 0, &V17602_294_57191, &P17602_293};	/*Property = CssPosition*/
const Value V17602_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_295 = {S17, 0, &V17602_295_57194, &P17602_294};	/*Property = CssText*/
const Value V17602_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_297 = {S18, 0, &V17602_297_57197, &P17602_295};	/*Property = CssTransforms*/
const Value V17602_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_241 = {S19, 0, &V17602_241_55801, &P17602_297};	/*Property = CssTransitions*/
const Value V17602_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_298 = {S20, 0, &V17602_298_57208, &P17602_241};	/*Property = CssUI*/
const Value V17602_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_300 = {S21, 0, &V17602_300_57217, &P17602_298};	/*Property = DataSet*/
const Value V17602_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_301 = {S22, 0, &V17602_301_57220, &P17602_300};	/*Property = DataUrl*/
const Value V17602_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_302 = {S23, 0, &V17602_302_57223, &P17602_301};	/*Property = DeviceOrientation*/
const Value V17602_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_304 = {S24, 0, &V17602_304_57229, &P17602_302};	/*Property = FileReader*/
const Value V17602_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_305 = {S25, 0, &V17602_305_57232, &P17602_304};	/*Property = FileSaver*/
const Value V17602_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_306 = {S26, 0, &V17602_306_57238, &P17602_305};	/*Property = FileWriter*/
const Value V17602_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_307 = {S27, 0, &V17602_307_57241, &P17602_306};	/*Property = FormData*/
const Value V17602_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_309 = {S28, 0, &V17602_309_57247, &P17602_307};	/*Property = Fullscreen*/
const Value V17602_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_310 = {S29, 0, &V17602_310_57250, &P17602_309};	/*Property = GeoLocation*/
const Value V17602_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_312 = {S30, 0, &V17602_312_57256, &P17602_310};	/*Property = History*/
const Value V17602_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_314 = {S31, 0, &V17602_314_57262, &P17602_312};	/*Property = Html5*/
const Value V17602_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_313 = {S32, 0, &V17602_313_57259, &P17602_314};	/*Property = Html-Media-Capture*/
const Value V17602_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_315 = {S33, 0, &V17602_315_57265, &P17602_313};	/*Property = Iframe*/
const Value V17602_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_316 = {S34, 0, &V17602_316_57268, &P17602_315};	/*Property = IndexedDB*/
const Value V17602_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_317 = {S35, 0, &V17602_317_57271, &P17602_316};	/*Property = Json*/
const Value V17602_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17602_15 = {S37, 0, &V17602_15_28, &P17602_317};	/*Property = LayoutEngine*/
const Value V17602_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_344 = {S38, 0, &V17602_344_57274, &P17602_15};	/*Property = Masking*/
const Value V17602_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_320 = {S39, 0, &V17602_320_57292, &P17602_344};	/*Property = PostMessage*/
const Value V17602_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_321 = {S40, 0, &V17602_321_57295, &P17602_320};	/*Property = Progress*/
const Value V17602_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_322 = {S41, 0, &V17602_322_57298, &P17602_321};	/*Property = Prompts*/
const Value V17602_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_323 = {S42, 0, &V17602_323_57301, &P17602_322};	/*Property = Selector*/
const Value V17602_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_324 = {S43, 0, &V17602_324_57307, &P17602_323};	/*Property = Svg*/
const Value V17602_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_327 = {S44, 0, &V17602_327_57316, &P17602_324};	/*Property = TouchEvents*/
const Value V17602_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_328 = {S45, 0, &V17602_328_57319, &P17602_327};	/*Property = Track*/
const Value V17602_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_329 = {S46, 0, &V17602_329_57325, &P17602_328};	/*Property = Video*/
const Value V17602_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_330 = {S47, 0, &V17602_330_57328, &P17602_329};	/*Property = Viewport*/
const Value V17602_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_332 = {S48, 0, &V17602_332_57340, &P17602_330};	/*Property = WebWorkers*/
const Value V17602_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17602_333 = {S49, 0, &V17602_333_57343, &P17602_332};	/*Property = Xhr2*/
const Profile P17602 = {"17602", &P17602_333};	/*Profile ID - 17602*/
const Value V3410_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_279 = {S1, 0, &V3410_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3410_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_282 = {S3, 0, &V3410_282_57155, &P3410_279};	/*Property = BlobBuilder*/
const Value V3410_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_350 = {S4, 0, &V3410_350_57446, &P3410_282};	/*Property = Canvas*/
const Value V3410_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_283 = {S5, 0, &V3410_283_57161, &P3410_350};	/*Property = CssBackground*/
const Value V3410_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_284 = {S6, 0, &V3410_284_57164, &P3410_283};	/*Property = CssBorderImage*/
const Value V3410_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_285 = {S7, 0, &V3410_285_57167, &P3410_284};	/*Property = CssCanvas*/
const Value V3410_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_286 = {S8, 0, &V3410_286_57170, &P3410_285};	/*Property = CssColor*/
const Value V3410_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_237 = {S9, 0, &V3410_237_55786, &P3410_286};	/*Property = CssColumn*/
const Value V3410_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_287 = {S10, 0, &V3410_287_57173, &P3410_237};	/*Property = CssFlexbox*/
const Value V3410_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_288 = {S11, 0, &V3410_288_57176, &P3410_287};	/*Property = CssFont*/
const Value V3410_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_289 = {S12, 0, &V3410_289_86899, &P3410_288};	/*Property = CssImages*/
const Value V3410_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_291 = {S13, 0, &V3410_291_57182, &P3410_289};	/*Property = CssMediaQueries*/
const Value V3410_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_292 = {S14, 0, &V3410_292_57185, &P3410_291};	/*Property = CssMinMax*/
const Value V3410_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_293 = {S15, 0, &V3410_293_57188, &P3410_292};	/*Property = CssOverflow*/
const Value V3410_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_294 = {S16, 0, &V3410_294_57191, &P3410_293};	/*Property = CssPosition*/
const Value V3410_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_295 = {S17, 0, &V3410_295_57194, &P3410_294};	/*Property = CssText*/
const Value V3410_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_297 = {S18, 0, &V3410_297_57197, &P3410_295};	/*Property = CssTransforms*/
const Value V3410_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_241 = {S19, 0, &V3410_241_55801, &P3410_297};	/*Property = CssTransitions*/
const Value V3410_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_298 = {S20, 0, &V3410_298_57208, &P3410_241};	/*Property = CssUI*/
const Value V3410_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_300 = {S21, 0, &V3410_300_57217, &P3410_298};	/*Property = DataSet*/
const Value V3410_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_301 = {S22, 0, &V3410_301_57220, &P3410_300};	/*Property = DataUrl*/
const Value V3410_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_302 = {S23, 0, &V3410_302_57223, &P3410_301};	/*Property = DeviceOrientation*/
const Value V3410_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_304 = {S24, 0, &V3410_304_57229, &P3410_302};	/*Property = FileReader*/
const Value V3410_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_305 = {S25, 0, &V3410_305_57232, &P3410_304};	/*Property = FileSaver*/
const Value V3410_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_306 = {S26, 0, &V3410_306_57238, &P3410_305};	/*Property = FileWriter*/
const Value V3410_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_307 = {S27, 0, &V3410_307_57241, &P3410_306};	/*Property = FormData*/
const Value V3410_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_309 = {S28, 0, &V3410_309_57247, &P3410_307};	/*Property = Fullscreen*/
const Value V3410_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_310 = {S29, 0, &V3410_310_57250, &P3410_309};	/*Property = GeoLocation*/
const Value V3410_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_312 = {S30, 0, &V3410_312_57256, &P3410_310};	/*Property = History*/
const Value V3410_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_314 = {S31, 0, &V3410_314_57262, &P3410_312};	/*Property = Html5*/
const Value V3410_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_313 = {S32, 0, &V3410_313_57259, &P3410_314};	/*Property = Html-Media-Capture*/
const Value V3410_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_315 = {S33, 0, &V3410_315_57265, &P3410_313};	/*Property = Iframe*/
const Value V3410_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_316 = {S34, 0, &V3410_316_57268, &P3410_315};	/*Property = IndexedDB*/
const Value V3410_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_317 = {S35, 0, &V3410_317_57271, &P3410_316};	/*Property = Json*/
const Value V3410_15_43264 = {S59, NULL};	/*Value = Mango*/
const Property P3410_15 = {S37, 0, &V3410_15_43264, &P3410_317};	/*Property = LayoutEngine*/
const Value V3410_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_344 = {S38, 0, &V3410_344_57274, &P3410_15};	/*Property = Masking*/
const Value V3410_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_320 = {S39, 0, &V3410_320_57292, &P3410_344};	/*Property = PostMessage*/
const Value V3410_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_321 = {S40, 0, &V3410_321_57295, &P3410_320};	/*Property = Progress*/
const Value V3410_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_322 = {S41, 0, &V3410_322_57298, &P3410_321};	/*Property = Prompts*/
const Value V3410_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_323 = {S42, 0, &V3410_323_57301, &P3410_322};	/*Property = Selector*/
const Value V3410_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_324 = {S43, 0, &V3410_324_57307, &P3410_323};	/*Property = Svg*/
const Value V3410_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_327 = {S44, 0, &V3410_327_57316, &P3410_324};	/*Property = TouchEvents*/
const Value V3410_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_328 = {S45, 0, &V3410_328_57319, &P3410_327};	/*Property = Track*/
const Value V3410_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_329 = {S46, 0, &V3410_329_57325, &P3410_328};	/*Property = Video*/
const Value V3410_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_330 = {S47, 0, &V3410_330_57328, &P3410_329};	/*Property = Viewport*/
const Value V3410_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_332 = {S48, 0, &V3410_332_57340, &P3410_330};	/*Property = WebWorkers*/
const Value V3410_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3410_333 = {S49, 0, &V3410_333_57343, &P3410_332};	/*Property = Xhr2*/
const Profile P3410 = {"3410", &P3410_333};	/*Profile ID - 3410*/
const Profile P5026 = {"5026", NULL};	/*Profile ID - 5026*/
const Value V14450_1_1 = {S2, NULL};	/*Value = True*/
const Property P14450_1 = {S50, 0, &V14450_1_1, NULL};	/*Property = IsMobile*/
const Value V14450_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14450_5 = {S52, 0, &V14450_5_1380, &P14450_1};	/*Property = ScreenPixelsHeight*/
const Value V14450_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14450_4 = {S54, 0, &V14450_4_1279, &P14450_5};	/*Property = ScreenPixelsWidth*/
const Profile P14450 = {"14450", &P14450_4};	/*Profile ID - 14450*/
const Value V3441_279_57148 = {S0, NULL};	/*Value = False*/
const Property P3441_279 = {S1, 0, &V3441_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V3441_282_57153 = {S2, NULL};	/*Value = True*/
const Property P3441_282 = {S3, 0, &V3441_282_57153, &P3441_279};	/*Property = BlobBuilder*/
const Value V3441_350_57444 = {S2, NULL};	/*Value = True*/
const Property P3441_350 = {S4, 0, &V3441_350_57444, &P3441_282};	/*Property = Canvas*/
const Value V3441_283_57159 = {S2, NULL};	/*Value = True*/
const Property P3441_283 = {S5, 0, &V3441_283_57159, &P3441_350};	/*Property = CssBackground*/
const Value V3441_284_57163 = {S0, NULL};	/*Value = False*/
const Property P3441_284 = {S6, 0, &V3441_284_57163, &P3441_283};	/*Property = CssBorderImage*/
const Value V3441_285_57166 = {S0, NULL};	/*Value = False*/
const Property P3441_285 = {S7, 0, &V3441_285_57166, &P3441_284};	/*Property = CssCanvas*/
const Value V3441_286_57168 = {S2, NULL};	/*Value = True*/
const Property P3441_286 = {S8, 0, &V3441_286_57168, &P3441_285};	/*Property = CssColor*/
const Value V3441_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3441_237 = {S9, 0, &V3441_237_55786, &P3441_286};	/*Property = CssColumn*/
const Value V3441_287_57172 = {S0, NULL};	/*Value = False*/
const Property P3441_287 = {S10, 0, &V3441_287_57172, &P3441_237};	/*Property = CssFlexbox*/
const Value V3441_288_57174 = {S2, NULL};	/*Value = True*/
const Property P3441_288 = {S11, 0, &V3441_288_57174, &P3441_287};	/*Property = CssFont*/
const Value V3441_289_57759 = {S0, NULL};	/*Value = False*/
const Property P3441_289 = {S12, 0, &V3441_289_57759, &P3441_288};	/*Property = CssImages*/
const Value V3441_291_57180 = {S2, NULL};	/*Value = True*/
const Property P3441_291 = {S13, 0, &V3441_291_57180, &P3441_289};	/*Property = CssMediaQueries*/
const Value V3441_292_57183 = {S2, NULL};	/*Value = True*/
const Property P3441_292 = {S14, 0, &V3441_292_57183, &P3441_291};	/*Property = CssMinMax*/
const Value V3441_293_57187 = {S0, NULL};	/*Value = False*/
const Property P3441_293 = {S15, 0, &V3441_293_57187, &P3441_292};	/*Property = CssOverflow*/
const Value V3441_294_57189 = {S2, NULL};	/*Value = True*/
const Property P3441_294 = {S16, 0, &V3441_294_57189, &P3441_293};	/*Property = CssPosition*/
const Value V3441_295_57192 = {S2, NULL};	/*Value = True*/
const Property P3441_295 = {S17, 0, &V3441_295_57192, &P3441_294};	/*Property = CssText*/
const Value V3441_297_57195 = {S2, NULL};	/*Value = True*/
const Property P3441_297 = {S18, 0, &V3441_297_57195, &P3441_295};	/*Property = CssTransforms*/
const Value V3441_241_55799 = {S2, NULL};	/*Value = True*/
const Property P3441_241 = {S19, 0, &V3441_241_55799, &P3441_297};	/*Property = CssTransitions*/
const Value V3441_298_57206 = {S2, NULL};	/*Value = True*/
const Property P3441_298 = {S20, 0, &V3441_298_57206, &P3441_241};	/*Property = CssUI*/
const Value V3441_300_57216 = {S0, NULL};	/*Value = False*/
const Property P3441_300 = {S21, 0, &V3441_300_57216, &P3441_298};	/*Property = DataSet*/
const Value V3441_301_57218 = {S2, NULL};	/*Value = True*/
const Property P3441_301 = {S22, 0, &V3441_301_57218, &P3441_300};	/*Property = DataUrl*/
const Value V3441_302_57222 = {S0, NULL};	/*Value = False*/
const Property P3441_302 = {S23, 0, &V3441_302_57222, &P3441_301};	/*Property = DeviceOrientation*/
const Value V3441_304_57227 = {S2, NULL};	/*Value = True*/
const Property P3441_304 = {S24, 0, &V3441_304_57227, &P3441_302};	/*Property = FileReader*/
const Value V3441_305_57231 = {S0, NULL};	/*Value = False*/
const Property P3441_305 = {S25, 0, &V3441_305_57231, &P3441_304};	/*Property = FileSaver*/
const Value V3441_306_57237 = {S0, NULL};	/*Value = False*/
const Property P3441_306 = {S26, 0, &V3441_306_57237, &P3441_305};	/*Property = FileWriter*/
const Value V3441_307_57239 = {S2, NULL};	/*Value = True*/
const Property P3441_307 = {S27, 0, &V3441_307_57239, &P3441_306};	/*Property = FormData*/
const Value V3441_309_57246 = {S0, NULL};	/*Value = False*/
const Property P3441_309 = {S28, 0, &V3441_309_57246, &P3441_307};	/*Property = Fullscreen*/
const Value V3441_310_57248 = {S2, NULL};	/*Value = True*/
const Property P3441_310 = {S29, 0, &V3441_310_57248, &P3441_309};	/*Property = GeoLocation*/
const Value V3441_312_57254 = {S2, NULL};	/*Value = True*/
const Property P3441_312 = {S30, 0, &V3441_312_57254, &P3441_310};	/*Property = History*/
const Value V3441_314_57261 = {S0, NULL};	/*Value = False*/
const Property P3441_314 = {S31, 0, &V3441_314_57261, &P3441_312};	/*Property = Html5*/
const Value V3441_313_57258 = {S0, NULL};	/*Value = False*/
const Property P3441_313 = {S32, 0, &V3441_313_57258, &P3441_314};	/*Property = Html-Media-Capture*/
const Value V3441_315_57264 = {S0, NULL};	/*Value = False*/
const Property P3441_315 = {S33, 0, &V3441_315_57264, &P3441_313};	/*Property = Iframe*/
const Value V3441_316_57266 = {S2, NULL};	/*Value = True*/
const Property P3441_316 = {S34, 0, &V3441_316_57266, &P3441_315};	/*Property = IndexedDB*/
const Value V3441_317_57269 = {S2, NULL};	/*Value = True*/
const Property P3441_317 = {S35, 0, &V3441_317_57269, &P3441_316};	/*Property = Json*/
const Value V3441_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3441_15 = {S37, 0, &V3441_15_28, &P3441_317};	/*Property = LayoutEngine*/
const Value V3441_344_57272 = {S2, NULL};	/*Value = True*/
const Property P3441_344 = {S38, 0, &V3441_344_57272, &P3441_15};	/*Property = Masking*/
const Value V3441_320_57290 = {S2, NULL};	/*Value = True*/
const Property P3441_320 = {S39, 0, &V3441_320_57290, &P3441_344};	/*Property = PostMessage*/
const Value V3441_321_57293 = {S2, NULL};	/*Value = True*/
const Property P3441_321 = {S40, 0, &V3441_321_57293, &P3441_320};	/*Property = Progress*/
const Value V3441_322_57296 = {S2, NULL};	/*Value = True*/
const Property P3441_322 = {S41, 0, &V3441_322_57296, &P3441_321};	/*Property = Prompts*/
const Value V3441_323_57299 = {S2, NULL};	/*Value = True*/
const Property P3441_323 = {S42, 0, &V3441_323_57299, &P3441_322};	/*Property = Selector*/
const Value V3441_324_57306 = {S0, NULL};	/*Value = False*/
const Property P3441_324 = {S43, 0, &V3441_324_57306, &P3441_323};	/*Property = Svg*/
const Value V3441_327_57315 = {S0, NULL};	/*Value = False*/
const Property P3441_327 = {S44, 0, &V3441_327_57315, &P3441_324};	/*Property = TouchEvents*/
const Value V3441_328_57318 = {S0, NULL};	/*Value = False*/
const Property P3441_328 = {S45, 0, &V3441_328_57318, &P3441_327};	/*Property = Track*/
const Value V3441_329_57323 = {S2, NULL};	/*Value = True*/
const Property P3441_329 = {S46, 0, &V3441_329_57323, &P3441_328};	/*Property = Video*/
const Value V3441_330_57326 = {S2, NULL};	/*Value = True*/
const Property P3441_330 = {S47, 0, &V3441_330_57326, &P3441_329};	/*Property = Viewport*/
const Value V3441_332_57338 = {S2, NULL};	/*Value = True*/
const Property P3441_332 = {S48, 0, &V3441_332_57338, &P3441_330};	/*Property = WebWorkers*/
const Value V3441_333_57342 = {S0, NULL};	/*Value = False*/
const Property P3441_333 = {S49, 0, &V3441_333_57342, &P3441_332};	/*Property = Xhr2*/
const Profile P3441 = {"3441", &P3441_333};	/*Profile ID - 3441*/
const Profile P5062 = {"5062", NULL};	/*Profile ID - 5062*/
const Value V14202_1_1 = {S2, NULL};	/*Value = True*/
const Property P14202_1 = {S50, 0, &V14202_1_1, NULL};	/*Property = IsMobile*/
const Value V14202_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14202_5 = {S52, 0, &V14202_5_1277, &P14202_1};	/*Property = ScreenPixelsHeight*/
const Value V14202_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14202_4 = {S54, 0, &V14202_4_1279, &P14202_5};	/*Property = ScreenPixelsWidth*/
const Profile P14202 = {"14202", &P14202_4};	/*Profile ID - 14202*/
const Profile P5448 = {"5448", NULL};	/*Profile ID - 5448*/
const Value V4808_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_279 = {S1, 0, &V4808_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4808_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_282 = {S3, 0, &V4808_282_57155, &P4808_279};	/*Property = BlobBuilder*/
const Value V4808_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_350 = {S4, 0, &V4808_350_57446, &P4808_282};	/*Property = Canvas*/
const Value V4808_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_283 = {S5, 0, &V4808_283_57161, &P4808_350};	/*Property = CssBackground*/
const Value V4808_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_284 = {S6, 0, &V4808_284_57164, &P4808_283};	/*Property = CssBorderImage*/
const Value V4808_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_285 = {S7, 0, &V4808_285_57167, &P4808_284};	/*Property = CssCanvas*/
const Value V4808_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_286 = {S8, 0, &V4808_286_57170, &P4808_285};	/*Property = CssColor*/
const Value V4808_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_237 = {S9, 0, &V4808_237_55786, &P4808_286};	/*Property = CssColumn*/
const Value V4808_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_287 = {S10, 0, &V4808_287_57173, &P4808_237};	/*Property = CssFlexbox*/
const Value V4808_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_288 = {S11, 0, &V4808_288_57176, &P4808_287};	/*Property = CssFont*/
const Value V4808_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_289 = {S12, 0, &V4808_289_86899, &P4808_288};	/*Property = CssImages*/
const Value V4808_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_291 = {S13, 0, &V4808_291_57182, &P4808_289};	/*Property = CssMediaQueries*/
const Value V4808_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_292 = {S14, 0, &V4808_292_57185, &P4808_291};	/*Property = CssMinMax*/
const Value V4808_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_293 = {S15, 0, &V4808_293_57188, &P4808_292};	/*Property = CssOverflow*/
const Value V4808_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_294 = {S16, 0, &V4808_294_57191, &P4808_293};	/*Property = CssPosition*/
const Value V4808_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_295 = {S17, 0, &V4808_295_57194, &P4808_294};	/*Property = CssText*/
const Value V4808_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_297 = {S18, 0, &V4808_297_57197, &P4808_295};	/*Property = CssTransforms*/
const Value V4808_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_241 = {S19, 0, &V4808_241_55801, &P4808_297};	/*Property = CssTransitions*/
const Value V4808_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_298 = {S20, 0, &V4808_298_57208, &P4808_241};	/*Property = CssUI*/
const Value V4808_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_300 = {S21, 0, &V4808_300_57217, &P4808_298};	/*Property = DataSet*/
const Value V4808_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_301 = {S22, 0, &V4808_301_57220, &P4808_300};	/*Property = DataUrl*/
const Value V4808_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_302 = {S23, 0, &V4808_302_57223, &P4808_301};	/*Property = DeviceOrientation*/
const Value V4808_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_304 = {S24, 0, &V4808_304_57229, &P4808_302};	/*Property = FileReader*/
const Value V4808_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_305 = {S25, 0, &V4808_305_57232, &P4808_304};	/*Property = FileSaver*/
const Value V4808_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_306 = {S26, 0, &V4808_306_57238, &P4808_305};	/*Property = FileWriter*/
const Value V4808_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_307 = {S27, 0, &V4808_307_57241, &P4808_306};	/*Property = FormData*/
const Value V4808_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_309 = {S28, 0, &V4808_309_57247, &P4808_307};	/*Property = Fullscreen*/
const Value V4808_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_310 = {S29, 0, &V4808_310_57250, &P4808_309};	/*Property = GeoLocation*/
const Value V4808_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_312 = {S30, 0, &V4808_312_57256, &P4808_310};	/*Property = History*/
const Value V4808_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_314 = {S31, 0, &V4808_314_57262, &P4808_312};	/*Property = Html5*/
const Value V4808_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_313 = {S32, 0, &V4808_313_57259, &P4808_314};	/*Property = Html-Media-Capture*/
const Value V4808_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_315 = {S33, 0, &V4808_315_57265, &P4808_313};	/*Property = Iframe*/
const Value V4808_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_316 = {S34, 0, &V4808_316_57268, &P4808_315};	/*Property = IndexedDB*/
const Value V4808_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_317 = {S35, 0, &V4808_317_57271, &P4808_316};	/*Property = Json*/
const Value V4808_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P4808_15 = {S37, 0, &V4808_15_30, &P4808_317};	/*Property = LayoutEngine*/
const Value V4808_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_344 = {S38, 0, &V4808_344_57274, &P4808_15};	/*Property = Masking*/
const Value V4808_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_320 = {S39, 0, &V4808_320_57292, &P4808_344};	/*Property = PostMessage*/
const Value V4808_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_321 = {S40, 0, &V4808_321_57295, &P4808_320};	/*Property = Progress*/
const Value V4808_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_322 = {S41, 0, &V4808_322_57298, &P4808_321};	/*Property = Prompts*/
const Value V4808_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_323 = {S42, 0, &V4808_323_57301, &P4808_322};	/*Property = Selector*/
const Value V4808_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_324 = {S43, 0, &V4808_324_57307, &P4808_323};	/*Property = Svg*/
const Value V4808_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_327 = {S44, 0, &V4808_327_57316, &P4808_324};	/*Property = TouchEvents*/
const Value V4808_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_328 = {S45, 0, &V4808_328_57319, &P4808_327};	/*Property = Track*/
const Value V4808_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_329 = {S46, 0, &V4808_329_57325, &P4808_328};	/*Property = Video*/
const Value V4808_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_330 = {S47, 0, &V4808_330_57328, &P4808_329};	/*Property = Viewport*/
const Value V4808_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_332 = {S48, 0, &V4808_332_57340, &P4808_330};	/*Property = WebWorkers*/
const Value V4808_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4808_333 = {S49, 0, &V4808_333_57343, &P4808_332};	/*Property = Xhr2*/
const Profile P4808 = {"4808", &P4808_333};	/*Profile ID - 4808*/
const Value V13999_1_1 = {S2, NULL};	/*Value = True*/
const Property P13999_1 = {S50, 0, &V13999_1_1, NULL};	/*Property = IsMobile*/
const Value V13999_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13999_5 = {S52, 0, &V13999_5_1227, &P13999_1};	/*Property = ScreenPixelsHeight*/
const Value V13999_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13999_4 = {S54, 0, &V13999_4_1229, &P13999_5};	/*Property = ScreenPixelsWidth*/
const Profile P13999 = {"13999", &P13999_4};	/*Profile ID - 13999*/
const Profile P5222 = {"5222", NULL};	/*Profile ID - 5222*/
const Value V17183_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_279 = {S1, 0, &V17183_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17183_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_282 = {S3, 0, &V17183_282_57155, &P17183_279};	/*Property = BlobBuilder*/
const Value V17183_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_350 = {S4, 0, &V17183_350_57446, &P17183_282};	/*Property = Canvas*/
const Value V17183_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_283 = {S5, 0, &V17183_283_57161, &P17183_350};	/*Property = CssBackground*/
const Value V17183_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_284 = {S6, 0, &V17183_284_57164, &P17183_283};	/*Property = CssBorderImage*/
const Value V17183_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_285 = {S7, 0, &V17183_285_57167, &P17183_284};	/*Property = CssCanvas*/
const Value V17183_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_286 = {S8, 0, &V17183_286_57170, &P17183_285};	/*Property = CssColor*/
const Value V17183_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_237 = {S9, 0, &V17183_237_55786, &P17183_286};	/*Property = CssColumn*/
const Value V17183_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_287 = {S10, 0, &V17183_287_57173, &P17183_237};	/*Property = CssFlexbox*/
const Value V17183_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_288 = {S11, 0, &V17183_288_57176, &P17183_287};	/*Property = CssFont*/
const Value V17183_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_289 = {S12, 0, &V17183_289_86899, &P17183_288};	/*Property = CssImages*/
const Value V17183_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_291 = {S13, 0, &V17183_291_57182, &P17183_289};	/*Property = CssMediaQueries*/
const Value V17183_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_292 = {S14, 0, &V17183_292_57185, &P17183_291};	/*Property = CssMinMax*/
const Value V17183_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_293 = {S15, 0, &V17183_293_57188, &P17183_292};	/*Property = CssOverflow*/
const Value V17183_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_294 = {S16, 0, &V17183_294_57191, &P17183_293};	/*Property = CssPosition*/
const Value V17183_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_295 = {S17, 0, &V17183_295_57194, &P17183_294};	/*Property = CssText*/
const Value V17183_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_297 = {S18, 0, &V17183_297_57197, &P17183_295};	/*Property = CssTransforms*/
const Value V17183_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_241 = {S19, 0, &V17183_241_55801, &P17183_297};	/*Property = CssTransitions*/
const Value V17183_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_298 = {S20, 0, &V17183_298_57208, &P17183_241};	/*Property = CssUI*/
const Value V17183_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_300 = {S21, 0, &V17183_300_57217, &P17183_298};	/*Property = DataSet*/
const Value V17183_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_301 = {S22, 0, &V17183_301_57220, &P17183_300};	/*Property = DataUrl*/
const Value V17183_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_302 = {S23, 0, &V17183_302_57223, &P17183_301};	/*Property = DeviceOrientation*/
const Value V17183_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_304 = {S24, 0, &V17183_304_57229, &P17183_302};	/*Property = FileReader*/
const Value V17183_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_305 = {S25, 0, &V17183_305_57232, &P17183_304};	/*Property = FileSaver*/
const Value V17183_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_306 = {S26, 0, &V17183_306_57238, &P17183_305};	/*Property = FileWriter*/
const Value V17183_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_307 = {S27, 0, &V17183_307_57241, &P17183_306};	/*Property = FormData*/
const Value V17183_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_309 = {S28, 0, &V17183_309_57247, &P17183_307};	/*Property = Fullscreen*/
const Value V17183_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_310 = {S29, 0, &V17183_310_57250, &P17183_309};	/*Property = GeoLocation*/
const Value V17183_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_312 = {S30, 0, &V17183_312_57256, &P17183_310};	/*Property = History*/
const Value V17183_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_314 = {S31, 0, &V17183_314_57262, &P17183_312};	/*Property = Html5*/
const Value V17183_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_313 = {S32, 0, &V17183_313_57259, &P17183_314};	/*Property = Html-Media-Capture*/
const Value V17183_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_315 = {S33, 0, &V17183_315_57265, &P17183_313};	/*Property = Iframe*/
const Value V17183_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_316 = {S34, 0, &V17183_316_57268, &P17183_315};	/*Property = IndexedDB*/
const Value V17183_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_317 = {S35, 0, &V17183_317_57271, &P17183_316};	/*Property = Json*/
const Value V17183_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P17183_15 = {S37, 0, &V17183_15_30, &P17183_317};	/*Property = LayoutEngine*/
const Value V17183_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_344 = {S38, 0, &V17183_344_57274, &P17183_15};	/*Property = Masking*/
const Value V17183_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_320 = {S39, 0, &V17183_320_57292, &P17183_344};	/*Property = PostMessage*/
const Value V17183_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_321 = {S40, 0, &V17183_321_57295, &P17183_320};	/*Property = Progress*/
const Value V17183_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_322 = {S41, 0, &V17183_322_57298, &P17183_321};	/*Property = Prompts*/
const Value V17183_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_323 = {S42, 0, &V17183_323_57301, &P17183_322};	/*Property = Selector*/
const Value V17183_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_324 = {S43, 0, &V17183_324_57307, &P17183_323};	/*Property = Svg*/
const Value V17183_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_327 = {S44, 0, &V17183_327_57316, &P17183_324};	/*Property = TouchEvents*/
const Value V17183_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_328 = {S45, 0, &V17183_328_57319, &P17183_327};	/*Property = Track*/
const Value V17183_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_329 = {S46, 0, &V17183_329_57325, &P17183_328};	/*Property = Video*/
const Value V17183_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_330 = {S47, 0, &V17183_330_57328, &P17183_329};	/*Property = Viewport*/
const Value V17183_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_332 = {S48, 0, &V17183_332_57340, &P17183_330};	/*Property = WebWorkers*/
const Value V17183_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17183_333 = {S49, 0, &V17183_333_57343, &P17183_332};	/*Property = Xhr2*/
const Profile P17183 = {"17183", &P17183_333};	/*Profile ID - 17183*/
const Value V4030_279_57148 = {S0, NULL};	/*Value = False*/
const Property P4030_279 = {S1, 0, &V4030_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V4030_282_57154 = {S0, NULL};	/*Value = False*/
const Property P4030_282 = {S3, 0, &V4030_282_57154, &P4030_279};	/*Property = BlobBuilder*/
const Value V4030_350_57444 = {S2, NULL};	/*Value = True*/
const Property P4030_350 = {S4, 0, &V4030_350_57444, &P4030_282};	/*Property = Canvas*/
const Value V4030_283_57159 = {S2, NULL};	/*Value = True*/
const Property P4030_283 = {S5, 0, &V4030_283_57159, &P4030_350};	/*Property = CssBackground*/
const Value V4030_284_57163 = {S0, NULL};	/*Value = False*/
const Property P4030_284 = {S6, 0, &V4030_284_57163, &P4030_283};	/*Property = CssBorderImage*/
const Value V4030_285_57166 = {S0, NULL};	/*Value = False*/
const Property P4030_285 = {S7, 0, &V4030_285_57166, &P4030_284};	/*Property = CssCanvas*/
const Value V4030_286_57168 = {S2, NULL};	/*Value = True*/
const Property P4030_286 = {S8, 0, &V4030_286_57168, &P4030_285};	/*Property = CssColor*/
const Value V4030_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4030_237 = {S9, 0, &V4030_237_55786, &P4030_286};	/*Property = CssColumn*/
const Value V4030_287_57172 = {S0, NULL};	/*Value = False*/
const Property P4030_287 = {S10, 0, &V4030_287_57172, &P4030_237};	/*Property = CssFlexbox*/
const Value V4030_288_57174 = {S2, NULL};	/*Value = True*/
const Property P4030_288 = {S11, 0, &V4030_288_57174, &P4030_287};	/*Property = CssFont*/
const Value V4030_289_57759 = {S0, NULL};	/*Value = False*/
const Property P4030_289 = {S12, 0, &V4030_289_57759, &P4030_288};	/*Property = CssImages*/
const Value V4030_291_57181 = {S0, NULL};	/*Value = False*/
const Property P4030_291 = {S13, 0, &V4030_291_57181, &P4030_289};	/*Property = CssMediaQueries*/
const Value V4030_292_57183 = {S2, NULL};	/*Value = True*/
const Property P4030_292 = {S14, 0, &V4030_292_57183, &P4030_291};	/*Property = CssMinMax*/
const Value V4030_293_57187 = {S0, NULL};	/*Value = False*/
const Property P4030_293 = {S15, 0, &V4030_293_57187, &P4030_292};	/*Property = CssOverflow*/
const Value V4030_294_57190 = {S0, NULL};	/*Value = False*/
const Property P4030_294 = {S16, 0, &V4030_294_57190, &P4030_293};	/*Property = CssPosition*/
const Value V4030_295_57192 = {S2, NULL};	/*Value = True*/
const Property P4030_295 = {S17, 0, &V4030_295_57192, &P4030_294};	/*Property = CssText*/
const Value V4030_297_57195 = {S2, NULL};	/*Value = True*/
const Property P4030_297 = {S18, 0, &V4030_297_57195, &P4030_295};	/*Property = CssTransforms*/
const Value V4030_241_55799 = {S2, NULL};	/*Value = True*/
const Property P4030_241 = {S19, 0, &V4030_241_55799, &P4030_297};	/*Property = CssTransitions*/
const Value V4030_298_57206 = {S2, NULL};	/*Value = True*/
const Property P4030_298 = {S20, 0, &V4030_298_57206, &P4030_241};	/*Property = CssUI*/
const Value V4030_300_57216 = {S0, NULL};	/*Value = False*/
const Property P4030_300 = {S21, 0, &V4030_300_57216, &P4030_298};	/*Property = DataSet*/
const Value V4030_301_57218 = {S2, NULL};	/*Value = True*/
const Property P4030_301 = {S22, 0, &V4030_301_57218, &P4030_300};	/*Property = DataUrl*/
const Value V4030_302_57221 = {S2, NULL};	/*Value = True*/
const Property P4030_302 = {S23, 0, &V4030_302_57221, &P4030_301};	/*Property = DeviceOrientation*/
const Value V4030_304_57228 = {S0, NULL};	/*Value = False*/
const Property P4030_304 = {S24, 0, &V4030_304_57228, &P4030_302};	/*Property = FileReader*/
const Value V4030_305_57231 = {S0, NULL};	/*Value = False*/
const Property P4030_305 = {S25, 0, &V4030_305_57231, &P4030_304};	/*Property = FileSaver*/
const Value V4030_306_57237 = {S0, NULL};	/*Value = False*/
const Property P4030_306 = {S26, 0, &V4030_306_57237, &P4030_305};	/*Property = FileWriter*/
const Value V4030_307_57239 = {S2, NULL};	/*Value = True*/
const Property P4030_307 = {S27, 0, &V4030_307_57239, &P4030_306};	/*Property = FormData*/
const Value V4030_309_57246 = {S0, NULL};	/*Value = False*/
const Property P4030_309 = {S28, 0, &V4030_309_57246, &P4030_307};	/*Property = Fullscreen*/
const Value V4030_310_57248 = {S2, NULL};	/*Value = True*/
const Property P4030_310 = {S29, 0, &V4030_310_57248, &P4030_309};	/*Property = GeoLocation*/
const Value V4030_312_57255 = {S0, NULL};	/*Value = False*/
const Property P4030_312 = {S30, 0, &V4030_312_57255, &P4030_310};	/*Property = History*/
const Value V4030_314_57261 = {S0, NULL};	/*Value = False*/
const Property P4030_314 = {S31, 0, &V4030_314_57261, &P4030_312};	/*Property = Html5*/
const Value V4030_313_57258 = {S0, NULL};	/*Value = False*/
const Property P4030_313 = {S32, 0, &V4030_313_57258, &P4030_314};	/*Property = Html-Media-Capture*/
const Value V4030_315_57264 = {S0, NULL};	/*Value = False*/
const Property P4030_315 = {S33, 0, &V4030_315_57264, &P4030_313};	/*Property = Iframe*/
const Value V4030_316_57267 = {S0, NULL};	/*Value = False*/
const Property P4030_316 = {S34, 0, &V4030_316_57267, &P4030_315};	/*Property = IndexedDB*/
const Value V4030_317_57269 = {S2, NULL};	/*Value = True*/
const Property P4030_317 = {S35, 0, &V4030_317_57269, &P4030_316};	/*Property = Json*/
const Value V4030_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P4030_15 = {S37, 0, &V4030_15_28, &P4030_317};	/*Property = LayoutEngine*/
const Value V4030_344_57272 = {S2, NULL};	/*Value = True*/
const Property P4030_344 = {S38, 0, &V4030_344_57272, &P4030_15};	/*Property = Masking*/
const Value V4030_320_57290 = {S2, NULL};	/*Value = True*/
const Property P4030_320 = {S39, 0, &V4030_320_57290, &P4030_344};	/*Property = PostMessage*/
const Value V4030_321_57293 = {S2, NULL};	/*Value = True*/
const Property P4030_321 = {S40, 0, &V4030_321_57293, &P4030_320};	/*Property = Progress*/
const Value V4030_322_57296 = {S2, NULL};	/*Value = True*/
const Property P4030_322 = {S41, 0, &V4030_322_57296, &P4030_321};	/*Property = Prompts*/
const Value V4030_323_57299 = {S2, NULL};	/*Value = True*/
const Property P4030_323 = {S42, 0, &V4030_323_57299, &P4030_322};	/*Property = Selector*/
const Value V4030_324_57306 = {S0, NULL};	/*Value = False*/
const Property P4030_324 = {S43, 0, &V4030_324_57306, &P4030_323};	/*Property = Svg*/
const Value V4030_327_57315 = {S0, NULL};	/*Value = False*/
const Property P4030_327 = {S44, 0, &V4030_327_57315, &P4030_324};	/*Property = TouchEvents*/
const Value V4030_328_57318 = {S0, NULL};	/*Value = False*/
const Property P4030_328 = {S45, 0, &V4030_328_57318, &P4030_327};	/*Property = Track*/
const Value V4030_329_57323 = {S2, NULL};	/*Value = True*/
const Property P4030_329 = {S46, 0, &V4030_329_57323, &P4030_328};	/*Property = Video*/
const Value V4030_330_57326 = {S2, NULL};	/*Value = True*/
const Property P4030_330 = {S47, 0, &V4030_330_57326, &P4030_329};	/*Property = Viewport*/
const Value V4030_332_57339 = {S0, NULL};	/*Value = False*/
const Property P4030_332 = {S48, 0, &V4030_332_57339, &P4030_330};	/*Property = WebWorkers*/
const Value V4030_333_57342 = {S0, NULL};	/*Value = False*/
const Property P4030_333 = {S49, 0, &V4030_333_57342, &P4030_332};	/*Property = Xhr2*/
const Profile P4030 = {"4030", &P4030_333};	/*Profile ID - 4030*/
const Profile P5204 = {"5204", NULL};	/*Profile ID - 5204*/
const Value V16503_1_1 = {S2, NULL};	/*Value = True*/
const Property P16503_1 = {S50, 0, &V16503_1_1, NULL};	/*Property = IsMobile*/
const Value V16503_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16503_5 = {S52, 0, &V16503_5_1293, &P16503_1};	/*Property = ScreenPixelsHeight*/
const Value V16503_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16503_4 = {S54, 0, &V16503_4_1296, &P16503_5};	/*Property = ScreenPixelsWidth*/
const Profile P16503 = {"16503", &P16503_4};	/*Profile ID - 16503*/
const Value V3690_279_57148 = {S0, NULL};	/*Value = False*/
const Property P3690_279 = {S1, 0, &V3690_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V3690_282_57154 = {S0, NULL};	/*Value = False*/
const Property P3690_282 = {S3, 0, &V3690_282_57154, &P3690_279};	/*Property = BlobBuilder*/
const Value V3690_350_57444 = {S2, NULL};	/*Value = True*/
const Property P3690_350 = {S4, 0, &V3690_350_57444, &P3690_282};	/*Property = Canvas*/
const Value V3690_283_57159 = {S2, NULL};	/*Value = True*/
const Property P3690_283 = {S5, 0, &V3690_283_57159, &P3690_350};	/*Property = CssBackground*/
const Value V3690_284_57163 = {S0, NULL};	/*Value = False*/
const Property P3690_284 = {S6, 0, &V3690_284_57163, &P3690_283};	/*Property = CssBorderImage*/
const Value V3690_285_57166 = {S0, NULL};	/*Value = False*/
const Property P3690_285 = {S7, 0, &V3690_285_57166, &P3690_284};	/*Property = CssCanvas*/
const Value V3690_286_57168 = {S2, NULL};	/*Value = True*/
const Property P3690_286 = {S8, 0, &V3690_286_57168, &P3690_285};	/*Property = CssColor*/
const Value V3690_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3690_237 = {S9, 0, &V3690_237_55786, &P3690_286};	/*Property = CssColumn*/
const Value V3690_287_57172 = {S0, NULL};	/*Value = False*/
const Property P3690_287 = {S10, 0, &V3690_287_57172, &P3690_237};	/*Property = CssFlexbox*/
const Value V3690_288_57175 = {S0, NULL};	/*Value = False*/
const Property P3690_288 = {S11, 0, &V3690_288_57175, &P3690_287};	/*Property = CssFont*/
const Value V3690_289_57759 = {S0, NULL};	/*Value = False*/
const Property P3690_289 = {S12, 0, &V3690_289_57759, &P3690_288};	/*Property = CssImages*/
const Value V3690_291_57181 = {S0, NULL};	/*Value = False*/
const Property P3690_291 = {S13, 0, &V3690_291_57181, &P3690_289};	/*Property = CssMediaQueries*/
const Value V3690_292_57183 = {S2, NULL};	/*Value = True*/
const Property P3690_292 = {S14, 0, &V3690_292_57183, &P3690_291};	/*Property = CssMinMax*/
const Value V3690_293_57187 = {S0, NULL};	/*Value = False*/
const Property P3690_293 = {S15, 0, &V3690_293_57187, &P3690_292};	/*Property = CssOverflow*/
const Value V3690_294_57190 = {S0, NULL};	/*Value = False*/
const Property P3690_294 = {S16, 0, &V3690_294_57190, &P3690_293};	/*Property = CssPosition*/
const Value V3690_295_57192 = {S2, NULL};	/*Value = True*/
const Property P3690_295 = {S17, 0, &V3690_295_57192, &P3690_294};	/*Property = CssText*/
const Value V3690_297_57195 = {S2, NULL};	/*Value = True*/
const Property P3690_297 = {S18, 0, &V3690_297_57195, &P3690_295};	/*Property = CssTransforms*/
const Value V3690_241_55799 = {S2, NULL};	/*Value = True*/
const Property P3690_241 = {S19, 0, &V3690_241_55799, &P3690_297};	/*Property = CssTransitions*/
const Value V3690_298_57206 = {S2, NULL};	/*Value = True*/
const Property P3690_298 = {S20, 0, &V3690_298_57206, &P3690_241};	/*Property = CssUI*/
const Value V3690_300_57216 = {S0, NULL};	/*Value = False*/
const Property P3690_300 = {S21, 0, &V3690_300_57216, &P3690_298};	/*Property = DataSet*/
const Value V3690_301_57218 = {S2, NULL};	/*Value = True*/
const Property P3690_301 = {S22, 0, &V3690_301_57218, &P3690_300};	/*Property = DataUrl*/
const Value V3690_302_57222 = {S0, NULL};	/*Value = False*/
const Property P3690_302 = {S23, 0, &V3690_302_57222, &P3690_301};	/*Property = DeviceOrientation*/
const Value V3690_304_57228 = {S0, NULL};	/*Value = False*/
const Property P3690_304 = {S24, 0, &V3690_304_57228, &P3690_302};	/*Property = FileReader*/
const Value V3690_305_57231 = {S0, NULL};	/*Value = False*/
const Property P3690_305 = {S25, 0, &V3690_305_57231, &P3690_304};	/*Property = FileSaver*/
const Value V3690_306_57237 = {S0, NULL};	/*Value = False*/
const Property P3690_306 = {S26, 0, &V3690_306_57237, &P3690_305};	/*Property = FileWriter*/
const Value V3690_307_57240 = {S0, NULL};	/*Value = False*/
const Property P3690_307 = {S27, 0, &V3690_307_57240, &P3690_306};	/*Property = FormData*/
const Value V3690_309_57246 = {S0, NULL};	/*Value = False*/
const Property P3690_309 = {S28, 0, &V3690_309_57246, &P3690_307};	/*Property = Fullscreen*/
const Value V3690_310_57249 = {S0, NULL};	/*Value = False*/
const Property P3690_310 = {S29, 0, &V3690_310_57249, &P3690_309};	/*Property = GeoLocation*/
const Value V3690_312_57255 = {S0, NULL};	/*Value = False*/
const Property P3690_312 = {S30, 0, &V3690_312_57255, &P3690_310};	/*Property = History*/
const Value V3690_314_57261 = {S0, NULL};	/*Value = False*/
const Property P3690_314 = {S31, 0, &V3690_314_57261, &P3690_312};	/*Property = Html5*/
const Value V3690_313_57258 = {S0, NULL};	/*Value = False*/
const Property P3690_313 = {S32, 0, &V3690_313_57258, &P3690_314};	/*Property = Html-Media-Capture*/
const Value V3690_315_57264 = {S0, NULL};	/*Value = False*/
const Property P3690_315 = {S33, 0, &V3690_315_57264, &P3690_313};	/*Property = Iframe*/
const Value V3690_316_57267 = {S0, NULL};	/*Value = False*/
const Property P3690_316 = {S34, 0, &V3690_316_57267, &P3690_315};	/*Property = IndexedDB*/
const Value V3690_317_57269 = {S2, NULL};	/*Value = True*/
const Property P3690_317 = {S35, 0, &V3690_317_57269, &P3690_316};	/*Property = Json*/
const Value V3690_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3690_15 = {S37, 0, &V3690_15_28, &P3690_317};	/*Property = LayoutEngine*/
const Value V3690_344_57272 = {S2, NULL};	/*Value = True*/
const Property P3690_344 = {S38, 0, &V3690_344_57272, &P3690_15};	/*Property = Masking*/
const Value V3690_320_57290 = {S2, NULL};	/*Value = True*/
const Property P3690_320 = {S39, 0, &V3690_320_57290, &P3690_344};	/*Property = PostMessage*/
const Value V3690_321_57293 = {S2, NULL};	/*Value = True*/
const Property P3690_321 = {S40, 0, &V3690_321_57293, &P3690_320};	/*Property = Progress*/
const Value V3690_322_57296 = {S2, NULL};	/*Value = True*/
const Property P3690_322 = {S41, 0, &V3690_322_57296, &P3690_321};	/*Property = Prompts*/
const Value V3690_323_57299 = {S2, NULL};	/*Value = True*/
const Property P3690_323 = {S42, 0, &V3690_323_57299, &P3690_322};	/*Property = Selector*/
const Value V3690_324_57306 = {S0, NULL};	/*Value = False*/
const Property P3690_324 = {S43, 0, &V3690_324_57306, &P3690_323};	/*Property = Svg*/
const Value V3690_327_57315 = {S0, NULL};	/*Value = False*/
const Property P3690_327 = {S44, 0, &V3690_327_57315, &P3690_324};	/*Property = TouchEvents*/
const Value V3690_328_57318 = {S0, NULL};	/*Value = False*/
const Property P3690_328 = {S45, 0, &V3690_328_57318, &P3690_327};	/*Property = Track*/
const Value V3690_329_57324 = {S0, NULL};	/*Value = False*/
const Property P3690_329 = {S46, 0, &V3690_329_57324, &P3690_328};	/*Property = Video*/
const Value V3690_330_57326 = {S2, NULL};	/*Value = True*/
const Property P3690_330 = {S47, 0, &V3690_330_57326, &P3690_329};	/*Property = Viewport*/
const Value V3690_332_57339 = {S0, NULL};	/*Value = False*/
const Property P3690_332 = {S48, 0, &V3690_332_57339, &P3690_330};	/*Property = WebWorkers*/
const Value V3690_333_57342 = {S0, NULL};	/*Value = False*/
const Property P3690_333 = {S49, 0, &V3690_333_57342, &P3690_332};	/*Property = Xhr2*/
const Profile P3690 = {"3690", &P3690_333};	/*Profile ID - 3690*/
const Profile P5129 = {"5129", NULL};	/*Profile ID - 5129*/
const Value V14188_1_1 = {S2, NULL};	/*Value = True*/
const Property P14188_1 = {S50, 0, &V14188_1_1, NULL};	/*Property = IsMobile*/
const Value V14188_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P14188_5 = {S52, 0, &V14188_5_1275, &P14188_1};	/*Property = ScreenPixelsHeight*/
const Value V14188_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P14188_4 = {S54, 0, &V14188_4_1278, &P14188_5};	/*Property = ScreenPixelsWidth*/
const Profile P14188 = {"14188", &P14188_4};	/*Profile ID - 14188*/
const Value V4617_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_279 = {S1, 0, &V4617_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4617_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_282 = {S3, 0, &V4617_282_57155, &P4617_279};	/*Property = BlobBuilder*/
const Value V4617_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_350 = {S4, 0, &V4617_350_57446, &P4617_282};	/*Property = Canvas*/
const Value V4617_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_283 = {S5, 0, &V4617_283_57161, &P4617_350};	/*Property = CssBackground*/
const Value V4617_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_284 = {S6, 0, &V4617_284_57164, &P4617_283};	/*Property = CssBorderImage*/
const Value V4617_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_285 = {S7, 0, &V4617_285_57167, &P4617_284};	/*Property = CssCanvas*/
const Value V4617_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_286 = {S8, 0, &V4617_286_57170, &P4617_285};	/*Property = CssColor*/
const Value V4617_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_237 = {S9, 0, &V4617_237_55786, &P4617_286};	/*Property = CssColumn*/
const Value V4617_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_287 = {S10, 0, &V4617_287_57173, &P4617_237};	/*Property = CssFlexbox*/
const Value V4617_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_288 = {S11, 0, &V4617_288_57176, &P4617_287};	/*Property = CssFont*/
const Value V4617_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_289 = {S12, 0, &V4617_289_86899, &P4617_288};	/*Property = CssImages*/
const Value V4617_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_291 = {S13, 0, &V4617_291_57182, &P4617_289};	/*Property = CssMediaQueries*/
const Value V4617_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_292 = {S14, 0, &V4617_292_57185, &P4617_291};	/*Property = CssMinMax*/
const Value V4617_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_293 = {S15, 0, &V4617_293_57188, &P4617_292};	/*Property = CssOverflow*/
const Value V4617_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_294 = {S16, 0, &V4617_294_57191, &P4617_293};	/*Property = CssPosition*/
const Value V4617_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_295 = {S17, 0, &V4617_295_57194, &P4617_294};	/*Property = CssText*/
const Value V4617_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_297 = {S18, 0, &V4617_297_57197, &P4617_295};	/*Property = CssTransforms*/
const Value V4617_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_241 = {S19, 0, &V4617_241_55801, &P4617_297};	/*Property = CssTransitions*/
const Value V4617_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_298 = {S20, 0, &V4617_298_57208, &P4617_241};	/*Property = CssUI*/
const Value V4617_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_300 = {S21, 0, &V4617_300_57217, &P4617_298};	/*Property = DataSet*/
const Value V4617_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_301 = {S22, 0, &V4617_301_57220, &P4617_300};	/*Property = DataUrl*/
const Value V4617_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_302 = {S23, 0, &V4617_302_57223, &P4617_301};	/*Property = DeviceOrientation*/
const Value V4617_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_304 = {S24, 0, &V4617_304_57229, &P4617_302};	/*Property = FileReader*/
const Value V4617_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_305 = {S25, 0, &V4617_305_57232, &P4617_304};	/*Property = FileSaver*/
const Value V4617_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_306 = {S26, 0, &V4617_306_57238, &P4617_305};	/*Property = FileWriter*/
const Value V4617_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_307 = {S27, 0, &V4617_307_57241, &P4617_306};	/*Property = FormData*/
const Value V4617_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_309 = {S28, 0, &V4617_309_57247, &P4617_307};	/*Property = Fullscreen*/
const Value V4617_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_310 = {S29, 0, &V4617_310_57250, &P4617_309};	/*Property = GeoLocation*/
const Value V4617_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_312 = {S30, 0, &V4617_312_57256, &P4617_310};	/*Property = History*/
const Value V4617_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_314 = {S31, 0, &V4617_314_57262, &P4617_312};	/*Property = Html5*/
const Value V4617_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_313 = {S32, 0, &V4617_313_57259, &P4617_314};	/*Property = Html-Media-Capture*/
const Value V4617_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_315 = {S33, 0, &V4617_315_57265, &P4617_313};	/*Property = Iframe*/
const Value V4617_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_316 = {S34, 0, &V4617_316_57268, &P4617_315};	/*Property = IndexedDB*/
const Value V4617_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_317 = {S35, 0, &V4617_317_57271, &P4617_316};	/*Property = Json*/
const Value V4617_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P4617_15 = {S37, 0, &V4617_15_28, &P4617_317};	/*Property = LayoutEngine*/
const Value V4617_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_344 = {S38, 0, &V4617_344_57274, &P4617_15};	/*Property = Masking*/
const Value V4617_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_320 = {S39, 0, &V4617_320_57292, &P4617_344};	/*Property = PostMessage*/
const Value V4617_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_321 = {S40, 0, &V4617_321_57295, &P4617_320};	/*Property = Progress*/
const Value V4617_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_322 = {S41, 0, &V4617_322_57298, &P4617_321};	/*Property = Prompts*/
const Value V4617_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_323 = {S42, 0, &V4617_323_57301, &P4617_322};	/*Property = Selector*/
const Value V4617_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_324 = {S43, 0, &V4617_324_57307, &P4617_323};	/*Property = Svg*/
const Value V4617_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_327 = {S44, 0, &V4617_327_57316, &P4617_324};	/*Property = TouchEvents*/
const Value V4617_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_328 = {S45, 0, &V4617_328_57319, &P4617_327};	/*Property = Track*/
const Value V4617_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_329 = {S46, 0, &V4617_329_57325, &P4617_328};	/*Property = Video*/
const Value V4617_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_330 = {S47, 0, &V4617_330_57328, &P4617_329};	/*Property = Viewport*/
const Value V4617_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_332 = {S48, 0, &V4617_332_57340, &P4617_330};	/*Property = WebWorkers*/
const Value V4617_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4617_333 = {S49, 0, &V4617_333_57343, &P4617_332};	/*Property = Xhr2*/
const Profile P4617 = {"4617", &P4617_333};	/*Profile ID - 4617*/
const Profile P5768 = {"5768", NULL};	/*Profile ID - 5768*/
const Value V17475_1_1 = {S2, NULL};	/*Value = True*/
const Property P17475_1 = {S50, 0, &V17475_1_1, NULL};	/*Property = IsMobile*/
const Value V17475_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17475_5 = {S52, 0, &V17475_5_1222, &P17475_1};	/*Property = ScreenPixelsHeight*/
const Value V17475_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17475_4 = {S54, 0, &V17475_4_1224, &P17475_5};	/*Property = ScreenPixelsWidth*/
const Profile P17475 = {"17475", &P17475_4};	/*Profile ID - 17475*/
const Value V12096_1_1 = {S2, NULL};	/*Value = True*/
const Property P12096_1 = {S50, 0, &V12096_1_1, NULL};	/*Property = IsMobile*/
const Value V12096_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12096_5 = {S52, 0, &V12096_5_1222, &P12096_1};	/*Property = ScreenPixelsHeight*/
const Value V12096_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12096_4 = {S54, 0, &V12096_4_1224, &P12096_5};	/*Property = ScreenPixelsWidth*/
const Profile P12096 = {"12096", &P12096_4};	/*Profile ID - 12096*/
const Value V17173_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_279 = {S1, 0, &V17173_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17173_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_282 = {S3, 0, &V17173_282_57155, &P17173_279};	/*Property = BlobBuilder*/
const Value V17173_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_350 = {S4, 0, &V17173_350_57446, &P17173_282};	/*Property = Canvas*/
const Value V17173_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_283 = {S5, 0, &V17173_283_57161, &P17173_350};	/*Property = CssBackground*/
const Value V17173_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_284 = {S6, 0, &V17173_284_57164, &P17173_283};	/*Property = CssBorderImage*/
const Value V17173_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_285 = {S7, 0, &V17173_285_57167, &P17173_284};	/*Property = CssCanvas*/
const Value V17173_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_286 = {S8, 0, &V17173_286_57170, &P17173_285};	/*Property = CssColor*/
const Value V17173_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_237 = {S9, 0, &V17173_237_55786, &P17173_286};	/*Property = CssColumn*/
const Value V17173_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_287 = {S10, 0, &V17173_287_57173, &P17173_237};	/*Property = CssFlexbox*/
const Value V17173_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_288 = {S11, 0, &V17173_288_57176, &P17173_287};	/*Property = CssFont*/
const Value V17173_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_289 = {S12, 0, &V17173_289_86899, &P17173_288};	/*Property = CssImages*/
const Value V17173_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_291 = {S13, 0, &V17173_291_57182, &P17173_289};	/*Property = CssMediaQueries*/
const Value V17173_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_292 = {S14, 0, &V17173_292_57185, &P17173_291};	/*Property = CssMinMax*/
const Value V17173_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_293 = {S15, 0, &V17173_293_57188, &P17173_292};	/*Property = CssOverflow*/
const Value V17173_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_294 = {S16, 0, &V17173_294_57191, &P17173_293};	/*Property = CssPosition*/
const Value V17173_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_295 = {S17, 0, &V17173_295_57194, &P17173_294};	/*Property = CssText*/
const Value V17173_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_297 = {S18, 0, &V17173_297_57197, &P17173_295};	/*Property = CssTransforms*/
const Value V17173_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_241 = {S19, 0, &V17173_241_55801, &P17173_297};	/*Property = CssTransitions*/
const Value V17173_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_298 = {S20, 0, &V17173_298_57208, &P17173_241};	/*Property = CssUI*/
const Value V17173_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_300 = {S21, 0, &V17173_300_57217, &P17173_298};	/*Property = DataSet*/
const Value V17173_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_301 = {S22, 0, &V17173_301_57220, &P17173_300};	/*Property = DataUrl*/
const Value V17173_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_302 = {S23, 0, &V17173_302_57223, &P17173_301};	/*Property = DeviceOrientation*/
const Value V17173_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_304 = {S24, 0, &V17173_304_57229, &P17173_302};	/*Property = FileReader*/
const Value V17173_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_305 = {S25, 0, &V17173_305_57232, &P17173_304};	/*Property = FileSaver*/
const Value V17173_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_306 = {S26, 0, &V17173_306_57238, &P17173_305};	/*Property = FileWriter*/
const Value V17173_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_307 = {S27, 0, &V17173_307_57241, &P17173_306};	/*Property = FormData*/
const Value V17173_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_309 = {S28, 0, &V17173_309_57247, &P17173_307};	/*Property = Fullscreen*/
const Value V17173_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_310 = {S29, 0, &V17173_310_57250, &P17173_309};	/*Property = GeoLocation*/
const Value V17173_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_312 = {S30, 0, &V17173_312_57256, &P17173_310};	/*Property = History*/
const Value V17173_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_314 = {S31, 0, &V17173_314_57262, &P17173_312};	/*Property = Html5*/
const Value V17173_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_313 = {S32, 0, &V17173_313_57259, &P17173_314};	/*Property = Html-Media-Capture*/
const Value V17173_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_315 = {S33, 0, &V17173_315_57265, &P17173_313};	/*Property = Iframe*/
const Value V17173_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_316 = {S34, 0, &V17173_316_57268, &P17173_315};	/*Property = IndexedDB*/
const Value V17173_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_317 = {S35, 0, &V17173_317_57271, &P17173_316};	/*Property = Json*/
const Value V17173_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P17173_15 = {S37, 0, &V17173_15_30, &P17173_317};	/*Property = LayoutEngine*/
const Value V17173_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_344 = {S38, 0, &V17173_344_57274, &P17173_15};	/*Property = Masking*/
const Value V17173_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_320 = {S39, 0, &V17173_320_57292, &P17173_344};	/*Property = PostMessage*/
const Value V17173_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_321 = {S40, 0, &V17173_321_57295, &P17173_320};	/*Property = Progress*/
const Value V17173_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_322 = {S41, 0, &V17173_322_57298, &P17173_321};	/*Property = Prompts*/
const Value V17173_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_323 = {S42, 0, &V17173_323_57301, &P17173_322};	/*Property = Selector*/
const Value V17173_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_324 = {S43, 0, &V17173_324_57307, &P17173_323};	/*Property = Svg*/
const Value V17173_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_327 = {S44, 0, &V17173_327_57316, &P17173_324};	/*Property = TouchEvents*/
const Value V17173_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_328 = {S45, 0, &V17173_328_57319, &P17173_327};	/*Property = Track*/
const Value V17173_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_329 = {S46, 0, &V17173_329_57325, &P17173_328};	/*Property = Video*/
const Value V17173_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_330 = {S47, 0, &V17173_330_57328, &P17173_329};	/*Property = Viewport*/
const Value V17173_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_332 = {S48, 0, &V17173_332_57340, &P17173_330};	/*Property = WebWorkers*/
const Value V17173_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17173_333 = {S49, 0, &V17173_333_57343, &P17173_332};	/*Property = Xhr2*/
const Profile P17173 = {"17173", &P17173_333};	/*Profile ID - 17173*/
const Value V4223_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_279 = {S1, 0, &V4223_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4223_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_282 = {S3, 0, &V4223_282_57155, &P4223_279};	/*Property = BlobBuilder*/
const Value V4223_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_350 = {S4, 0, &V4223_350_57446, &P4223_282};	/*Property = Canvas*/
const Value V4223_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_283 = {S5, 0, &V4223_283_57161, &P4223_350};	/*Property = CssBackground*/
const Value V4223_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_284 = {S6, 0, &V4223_284_57164, &P4223_283};	/*Property = CssBorderImage*/
const Value V4223_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_285 = {S7, 0, &V4223_285_57167, &P4223_284};	/*Property = CssCanvas*/
const Value V4223_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_286 = {S8, 0, &V4223_286_57170, &P4223_285};	/*Property = CssColor*/
const Value V4223_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_237 = {S9, 0, &V4223_237_55786, &P4223_286};	/*Property = CssColumn*/
const Value V4223_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_287 = {S10, 0, &V4223_287_57173, &P4223_237};	/*Property = CssFlexbox*/
const Value V4223_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_288 = {S11, 0, &V4223_288_57176, &P4223_287};	/*Property = CssFont*/
const Value V4223_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_289 = {S12, 0, &V4223_289_86899, &P4223_288};	/*Property = CssImages*/
const Value V4223_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_291 = {S13, 0, &V4223_291_57182, &P4223_289};	/*Property = CssMediaQueries*/
const Value V4223_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_292 = {S14, 0, &V4223_292_57185, &P4223_291};	/*Property = CssMinMax*/
const Value V4223_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_293 = {S15, 0, &V4223_293_57188, &P4223_292};	/*Property = CssOverflow*/
const Value V4223_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_294 = {S16, 0, &V4223_294_57191, &P4223_293};	/*Property = CssPosition*/
const Value V4223_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_295 = {S17, 0, &V4223_295_57194, &P4223_294};	/*Property = CssText*/
const Value V4223_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_297 = {S18, 0, &V4223_297_57197, &P4223_295};	/*Property = CssTransforms*/
const Value V4223_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_241 = {S19, 0, &V4223_241_55801, &P4223_297};	/*Property = CssTransitions*/
const Value V4223_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_298 = {S20, 0, &V4223_298_57208, &P4223_241};	/*Property = CssUI*/
const Value V4223_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_300 = {S21, 0, &V4223_300_57217, &P4223_298};	/*Property = DataSet*/
const Value V4223_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_301 = {S22, 0, &V4223_301_57220, &P4223_300};	/*Property = DataUrl*/
const Value V4223_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_302 = {S23, 0, &V4223_302_57223, &P4223_301};	/*Property = DeviceOrientation*/
const Value V4223_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_304 = {S24, 0, &V4223_304_57229, &P4223_302};	/*Property = FileReader*/
const Value V4223_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_305 = {S25, 0, &V4223_305_57232, &P4223_304};	/*Property = FileSaver*/
const Value V4223_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_306 = {S26, 0, &V4223_306_57238, &P4223_305};	/*Property = FileWriter*/
const Value V4223_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_307 = {S27, 0, &V4223_307_57241, &P4223_306};	/*Property = FormData*/
const Value V4223_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_309 = {S28, 0, &V4223_309_57247, &P4223_307};	/*Property = Fullscreen*/
const Value V4223_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_310 = {S29, 0, &V4223_310_57250, &P4223_309};	/*Property = GeoLocation*/
const Value V4223_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_312 = {S30, 0, &V4223_312_57256, &P4223_310};	/*Property = History*/
const Value V4223_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_314 = {S31, 0, &V4223_314_57262, &P4223_312};	/*Property = Html5*/
const Value V4223_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_313 = {S32, 0, &V4223_313_57259, &P4223_314};	/*Property = Html-Media-Capture*/
const Value V4223_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_315 = {S33, 0, &V4223_315_57265, &P4223_313};	/*Property = Iframe*/
const Value V4223_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_316 = {S34, 0, &V4223_316_57268, &P4223_315};	/*Property = IndexedDB*/
const Value V4223_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_317 = {S35, 0, &V4223_317_57271, &P4223_316};	/*Property = Json*/
const Value V4223_15_43263 = {S65, NULL};	/*Value = Netfront*/
const Property P4223_15 = {S37, 0, &V4223_15_43263, &P4223_317};	/*Property = LayoutEngine*/
const Value V4223_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_344 = {S38, 0, &V4223_344_57274, &P4223_15};	/*Property = Masking*/
const Value V4223_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_320 = {S39, 0, &V4223_320_57292, &P4223_344};	/*Property = PostMessage*/
const Value V4223_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_321 = {S40, 0, &V4223_321_57295, &P4223_320};	/*Property = Progress*/
const Value V4223_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_322 = {S41, 0, &V4223_322_57298, &P4223_321};	/*Property = Prompts*/
const Value V4223_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_323 = {S42, 0, &V4223_323_57301, &P4223_322};	/*Property = Selector*/
const Value V4223_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_324 = {S43, 0, &V4223_324_57307, &P4223_323};	/*Property = Svg*/
const Value V4223_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_327 = {S44, 0, &V4223_327_57316, &P4223_324};	/*Property = TouchEvents*/
const Value V4223_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_328 = {S45, 0, &V4223_328_57319, &P4223_327};	/*Property = Track*/
const Value V4223_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_329 = {S46, 0, &V4223_329_57325, &P4223_328};	/*Property = Video*/
const Value V4223_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_330 = {S47, 0, &V4223_330_57328, &P4223_329};	/*Property = Viewport*/
const Value V4223_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_332 = {S48, 0, &V4223_332_57340, &P4223_330};	/*Property = WebWorkers*/
const Value V4223_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4223_333 = {S49, 0, &V4223_333_57343, &P4223_332};	/*Property = Xhr2*/
const Profile P4223 = {"4223", &P4223_333};	/*Profile ID - 4223*/
const Value V12431_1_1 = {S2, NULL};	/*Value = True*/
const Property P12431_1 = {S50, 0, &V12431_1_1, NULL};	/*Property = IsMobile*/
const Value V12431_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12431_5 = {S52, 0, &V12431_5_1380, &P12431_1};	/*Property = ScreenPixelsHeight*/
const Value V12431_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12431_4 = {S54, 0, &V12431_4_1279, &P12431_5};	/*Property = ScreenPixelsWidth*/
const Profile P12431 = {"12431", &P12431_4};	/*Profile ID - 12431*/
const Profile P17484 = {"17484", NULL};	/*Profile ID - 17484*/
const Value V3625_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_279 = {S1, 0, &V3625_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3625_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_282 = {S3, 0, &V3625_282_57155, &P3625_279};	/*Property = BlobBuilder*/
const Value V3625_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_350 = {S4, 0, &V3625_350_57446, &P3625_282};	/*Property = Canvas*/
const Value V3625_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_283 = {S5, 0, &V3625_283_57161, &P3625_350};	/*Property = CssBackground*/
const Value V3625_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_284 = {S6, 0, &V3625_284_57164, &P3625_283};	/*Property = CssBorderImage*/
const Value V3625_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_285 = {S7, 0, &V3625_285_57167, &P3625_284};	/*Property = CssCanvas*/
const Value V3625_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_286 = {S8, 0, &V3625_286_57170, &P3625_285};	/*Property = CssColor*/
const Value V3625_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_237 = {S9, 0, &V3625_237_55786, &P3625_286};	/*Property = CssColumn*/
const Value V3625_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_287 = {S10, 0, &V3625_287_57173, &P3625_237};	/*Property = CssFlexbox*/
const Value V3625_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_288 = {S11, 0, &V3625_288_57176, &P3625_287};	/*Property = CssFont*/
const Value V3625_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_289 = {S12, 0, &V3625_289_86899, &P3625_288};	/*Property = CssImages*/
const Value V3625_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_291 = {S13, 0, &V3625_291_57182, &P3625_289};	/*Property = CssMediaQueries*/
const Value V3625_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_292 = {S14, 0, &V3625_292_57185, &P3625_291};	/*Property = CssMinMax*/
const Value V3625_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_293 = {S15, 0, &V3625_293_57188, &P3625_292};	/*Property = CssOverflow*/
const Value V3625_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_294 = {S16, 0, &V3625_294_57191, &P3625_293};	/*Property = CssPosition*/
const Value V3625_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_295 = {S17, 0, &V3625_295_57194, &P3625_294};	/*Property = CssText*/
const Value V3625_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_297 = {S18, 0, &V3625_297_57197, &P3625_295};	/*Property = CssTransforms*/
const Value V3625_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_241 = {S19, 0, &V3625_241_55801, &P3625_297};	/*Property = CssTransitions*/
const Value V3625_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_298 = {S20, 0, &V3625_298_57208, &P3625_241};	/*Property = CssUI*/
const Value V3625_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_300 = {S21, 0, &V3625_300_57217, &P3625_298};	/*Property = DataSet*/
const Value V3625_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_301 = {S22, 0, &V3625_301_57220, &P3625_300};	/*Property = DataUrl*/
const Value V3625_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_302 = {S23, 0, &V3625_302_57223, &P3625_301};	/*Property = DeviceOrientation*/
const Value V3625_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_304 = {S24, 0, &V3625_304_57229, &P3625_302};	/*Property = FileReader*/
const Value V3625_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_305 = {S25, 0, &V3625_305_57232, &P3625_304};	/*Property = FileSaver*/
const Value V3625_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_306 = {S26, 0, &V3625_306_57238, &P3625_305};	/*Property = FileWriter*/
const Value V3625_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_307 = {S27, 0, &V3625_307_57241, &P3625_306};	/*Property = FormData*/
const Value V3625_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_309 = {S28, 0, &V3625_309_57247, &P3625_307};	/*Property = Fullscreen*/
const Value V3625_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_310 = {S29, 0, &V3625_310_57250, &P3625_309};	/*Property = GeoLocation*/
const Value V3625_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_312 = {S30, 0, &V3625_312_57256, &P3625_310};	/*Property = History*/
const Value V3625_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_314 = {S31, 0, &V3625_314_57262, &P3625_312};	/*Property = Html5*/
const Value V3625_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_313 = {S32, 0, &V3625_313_57259, &P3625_314};	/*Property = Html-Media-Capture*/
const Value V3625_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_315 = {S33, 0, &V3625_315_57265, &P3625_313};	/*Property = Iframe*/
const Value V3625_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_316 = {S34, 0, &V3625_316_57268, &P3625_315};	/*Property = IndexedDB*/
const Value V3625_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_317 = {S35, 0, &V3625_317_57271, &P3625_316};	/*Property = Json*/
const Value V3625_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_15 = {S37, 0, &V3625_15_72, &P3625_317};	/*Property = LayoutEngine*/
const Value V3625_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_344 = {S38, 0, &V3625_344_57274, &P3625_15};	/*Property = Masking*/
const Value V3625_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_320 = {S39, 0, &V3625_320_57292, &P3625_344};	/*Property = PostMessage*/
const Value V3625_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_321 = {S40, 0, &V3625_321_57295, &P3625_320};	/*Property = Progress*/
const Value V3625_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_322 = {S41, 0, &V3625_322_57298, &P3625_321};	/*Property = Prompts*/
const Value V3625_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_323 = {S42, 0, &V3625_323_57301, &P3625_322};	/*Property = Selector*/
const Value V3625_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_324 = {S43, 0, &V3625_324_57307, &P3625_323};	/*Property = Svg*/
const Value V3625_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_327 = {S44, 0, &V3625_327_57316, &P3625_324};	/*Property = TouchEvents*/
const Value V3625_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_328 = {S45, 0, &V3625_328_57319, &P3625_327};	/*Property = Track*/
const Value V3625_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_329 = {S46, 0, &V3625_329_57325, &P3625_328};	/*Property = Video*/
const Value V3625_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_330 = {S47, 0, &V3625_330_57328, &P3625_329};	/*Property = Viewport*/
const Value V3625_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_332 = {S48, 0, &V3625_332_57340, &P3625_330};	/*Property = WebWorkers*/
const Value V3625_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3625_333 = {S49, 0, &V3625_333_57343, &P3625_332};	/*Property = Xhr2*/
const Profile P3625 = {"3625", &P3625_333};	/*Profile ID - 3625*/
const Value V17561_1_1 = {S2, NULL};	/*Value = True*/
const Property P17561_1 = {S50, 0, &V17561_1_1, NULL};	/*Property = IsMobile*/
const Value V17561_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P17561_5 = {S52, 0, &V17561_5_1380, &P17561_1};	/*Property = ScreenPixelsHeight*/
const Value V17561_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17561_4 = {S54, 0, &V17561_4_1279, &P17561_5};	/*Property = ScreenPixelsWidth*/
const Profile P17561 = {"17561", &P17561_4};	/*Profile ID - 17561*/
const Value V3430_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_279 = {S1, 0, &V3430_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3430_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_282 = {S3, 0, &V3430_282_57155, &P3430_279};	/*Property = BlobBuilder*/
const Value V3430_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_350 = {S4, 0, &V3430_350_57446, &P3430_282};	/*Property = Canvas*/
const Value V3430_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_283 = {S5, 0, &V3430_283_57161, &P3430_350};	/*Property = CssBackground*/
const Value V3430_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_284 = {S6, 0, &V3430_284_57164, &P3430_283};	/*Property = CssBorderImage*/
const Value V3430_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_285 = {S7, 0, &V3430_285_57167, &P3430_284};	/*Property = CssCanvas*/
const Value V3430_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_286 = {S8, 0, &V3430_286_57170, &P3430_285};	/*Property = CssColor*/
const Value V3430_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_237 = {S9, 0, &V3430_237_55786, &P3430_286};	/*Property = CssColumn*/
const Value V3430_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_287 = {S10, 0, &V3430_287_57173, &P3430_237};	/*Property = CssFlexbox*/
const Value V3430_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_288 = {S11, 0, &V3430_288_57176, &P3430_287};	/*Property = CssFont*/
const Value V3430_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_289 = {S12, 0, &V3430_289_86899, &P3430_288};	/*Property = CssImages*/
const Value V3430_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_291 = {S13, 0, &V3430_291_57182, &P3430_289};	/*Property = CssMediaQueries*/
const Value V3430_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_292 = {S14, 0, &V3430_292_57185, &P3430_291};	/*Property = CssMinMax*/
const Value V3430_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_293 = {S15, 0, &V3430_293_57188, &P3430_292};	/*Property = CssOverflow*/
const Value V3430_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_294 = {S16, 0, &V3430_294_57191, &P3430_293};	/*Property = CssPosition*/
const Value V3430_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_295 = {S17, 0, &V3430_295_57194, &P3430_294};	/*Property = CssText*/
const Value V3430_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_297 = {S18, 0, &V3430_297_57197, &P3430_295};	/*Property = CssTransforms*/
const Value V3430_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_241 = {S19, 0, &V3430_241_55801, &P3430_297};	/*Property = CssTransitions*/
const Value V3430_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_298 = {S20, 0, &V3430_298_57208, &P3430_241};	/*Property = CssUI*/
const Value V3430_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_300 = {S21, 0, &V3430_300_57217, &P3430_298};	/*Property = DataSet*/
const Value V3430_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_301 = {S22, 0, &V3430_301_57220, &P3430_300};	/*Property = DataUrl*/
const Value V3430_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_302 = {S23, 0, &V3430_302_57223, &P3430_301};	/*Property = DeviceOrientation*/
const Value V3430_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_304 = {S24, 0, &V3430_304_57229, &P3430_302};	/*Property = FileReader*/
const Value V3430_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_305 = {S25, 0, &V3430_305_57232, &P3430_304};	/*Property = FileSaver*/
const Value V3430_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_306 = {S26, 0, &V3430_306_57238, &P3430_305};	/*Property = FileWriter*/
const Value V3430_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_307 = {S27, 0, &V3430_307_57241, &P3430_306};	/*Property = FormData*/
const Value V3430_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_309 = {S28, 0, &V3430_309_57247, &P3430_307};	/*Property = Fullscreen*/
const Value V3430_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_310 = {S29, 0, &V3430_310_57250, &P3430_309};	/*Property = GeoLocation*/
const Value V3430_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_312 = {S30, 0, &V3430_312_57256, &P3430_310};	/*Property = History*/
const Value V3430_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_314 = {S31, 0, &V3430_314_57262, &P3430_312};	/*Property = Html5*/
const Value V3430_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_313 = {S32, 0, &V3430_313_57259, &P3430_314};	/*Property = Html-Media-Capture*/
const Value V3430_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_315 = {S33, 0, &V3430_315_57265, &P3430_313};	/*Property = Iframe*/
const Value V3430_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_316 = {S34, 0, &V3430_316_57268, &P3430_315};	/*Property = IndexedDB*/
const Value V3430_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_317 = {S35, 0, &V3430_317_57271, &P3430_316};	/*Property = Json*/
const Value V3430_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P3430_15 = {S37, 0, &V3430_15_31, &P3430_317};	/*Property = LayoutEngine*/
const Value V3430_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_344 = {S38, 0, &V3430_344_57274, &P3430_15};	/*Property = Masking*/
const Value V3430_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_320 = {S39, 0, &V3430_320_57292, &P3430_344};	/*Property = PostMessage*/
const Value V3430_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_321 = {S40, 0, &V3430_321_57295, &P3430_320};	/*Property = Progress*/
const Value V3430_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_322 = {S41, 0, &V3430_322_57298, &P3430_321};	/*Property = Prompts*/
const Value V3430_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_323 = {S42, 0, &V3430_323_57301, &P3430_322};	/*Property = Selector*/
const Value V3430_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_324 = {S43, 0, &V3430_324_57307, &P3430_323};	/*Property = Svg*/
const Value V3430_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_327 = {S44, 0, &V3430_327_57316, &P3430_324};	/*Property = TouchEvents*/
const Value V3430_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_328 = {S45, 0, &V3430_328_57319, &P3430_327};	/*Property = Track*/
const Value V3430_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_329 = {S46, 0, &V3430_329_57325, &P3430_328};	/*Property = Video*/
const Value V3430_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_330 = {S47, 0, &V3430_330_57328, &P3430_329};	/*Property = Viewport*/
const Value V3430_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_332 = {S48, 0, &V3430_332_57340, &P3430_330};	/*Property = WebWorkers*/
const Value V3430_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3430_333 = {S49, 0, &V3430_333_57343, &P3430_332};	/*Property = Xhr2*/
const Profile P3430 = {"3430", &P3430_333};	/*Profile ID - 3430*/
const Value V13265_1_1 = {S2, NULL};	/*Value = True*/
const Property P13265_1 = {S50, 0, &V13265_1_1, NULL};	/*Property = IsMobile*/
const Value V13265_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13265_5 = {S52, 0, &V13265_5_1304, &P13265_1};	/*Property = ScreenPixelsHeight*/
const Value V13265_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13265_4 = {S54, 0, &V13265_4_1224, &P13265_5};	/*Property = ScreenPixelsWidth*/
const Profile P13265 = {"13265", &P13265_4};	/*Profile ID - 13265*/
const Value V13490_1_1 = {S2, NULL};	/*Value = True*/
const Property P13490_1 = {S50, 0, &V13490_1_1, NULL};	/*Property = IsMobile*/
const Value V13490_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13490_5 = {S52, 0, &V13490_5_1222, &P13490_1};	/*Property = ScreenPixelsHeight*/
const Value V13490_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13490_4 = {S54, 0, &V13490_4_1224, &P13490_5};	/*Property = ScreenPixelsWidth*/
const Profile P13490 = {"13490", &P13490_4};	/*Profile ID - 13490*/
const Value V22270_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_279 = {S1, 0, &V22270_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22270_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_282 = {S3, 0, &V22270_282_57155, &P22270_279};	/*Property = BlobBuilder*/
const Value V22270_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_350 = {S4, 0, &V22270_350_57446, &P22270_282};	/*Property = Canvas*/
const Value V22270_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_283 = {S5, 0, &V22270_283_57161, &P22270_350};	/*Property = CssBackground*/
const Value V22270_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_284 = {S6, 0, &V22270_284_57164, &P22270_283};	/*Property = CssBorderImage*/
const Value V22270_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_285 = {S7, 0, &V22270_285_57167, &P22270_284};	/*Property = CssCanvas*/
const Value V22270_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_286 = {S8, 0, &V22270_286_57170, &P22270_285};	/*Property = CssColor*/
const Value V22270_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_237 = {S9, 0, &V22270_237_55786, &P22270_286};	/*Property = CssColumn*/
const Value V22270_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_287 = {S10, 0, &V22270_287_57173, &P22270_237};	/*Property = CssFlexbox*/
const Value V22270_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_288 = {S11, 0, &V22270_288_57176, &P22270_287};	/*Property = CssFont*/
const Value V22270_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_289 = {S12, 0, &V22270_289_86899, &P22270_288};	/*Property = CssImages*/
const Value V22270_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_291 = {S13, 0, &V22270_291_57182, &P22270_289};	/*Property = CssMediaQueries*/
const Value V22270_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_292 = {S14, 0, &V22270_292_57185, &P22270_291};	/*Property = CssMinMax*/
const Value V22270_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_293 = {S15, 0, &V22270_293_57188, &P22270_292};	/*Property = CssOverflow*/
const Value V22270_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_294 = {S16, 0, &V22270_294_57191, &P22270_293};	/*Property = CssPosition*/
const Value V22270_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_295 = {S17, 0, &V22270_295_57194, &P22270_294};	/*Property = CssText*/
const Value V22270_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_297 = {S18, 0, &V22270_297_57197, &P22270_295};	/*Property = CssTransforms*/
const Value V22270_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_241 = {S19, 0, &V22270_241_55801, &P22270_297};	/*Property = CssTransitions*/
const Value V22270_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_298 = {S20, 0, &V22270_298_57208, &P22270_241};	/*Property = CssUI*/
const Value V22270_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_300 = {S21, 0, &V22270_300_57217, &P22270_298};	/*Property = DataSet*/
const Value V22270_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_301 = {S22, 0, &V22270_301_57220, &P22270_300};	/*Property = DataUrl*/
const Value V22270_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_302 = {S23, 0, &V22270_302_57223, &P22270_301};	/*Property = DeviceOrientation*/
const Value V22270_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_304 = {S24, 0, &V22270_304_57229, &P22270_302};	/*Property = FileReader*/
const Value V22270_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_305 = {S25, 0, &V22270_305_57232, &P22270_304};	/*Property = FileSaver*/
const Value V22270_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_306 = {S26, 0, &V22270_306_57238, &P22270_305};	/*Property = FileWriter*/
const Value V22270_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_307 = {S27, 0, &V22270_307_57241, &P22270_306};	/*Property = FormData*/
const Value V22270_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_309 = {S28, 0, &V22270_309_57247, &P22270_307};	/*Property = Fullscreen*/
const Value V22270_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_310 = {S29, 0, &V22270_310_57250, &P22270_309};	/*Property = GeoLocation*/
const Value V22270_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_312 = {S30, 0, &V22270_312_57256, &P22270_310};	/*Property = History*/
const Value V22270_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_314 = {S31, 0, &V22270_314_57262, &P22270_312};	/*Property = Html5*/
const Value V22270_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_313 = {S32, 0, &V22270_313_57259, &P22270_314};	/*Property = Html-Media-Capture*/
const Value V22270_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_315 = {S33, 0, &V22270_315_57265, &P22270_313};	/*Property = Iframe*/
const Value V22270_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_316 = {S34, 0, &V22270_316_57268, &P22270_315};	/*Property = IndexedDB*/
const Value V22270_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_317 = {S35, 0, &V22270_317_57271, &P22270_316};	/*Property = Json*/
const Value V22270_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P22270_15 = {S37, 0, &V22270_15_30, &P22270_317};	/*Property = LayoutEngine*/
const Value V22270_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_344 = {S38, 0, &V22270_344_57274, &P22270_15};	/*Property = Masking*/
const Value V22270_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_320 = {S39, 0, &V22270_320_57292, &P22270_344};	/*Property = PostMessage*/
const Value V22270_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_321 = {S40, 0, &V22270_321_57295, &P22270_320};	/*Property = Progress*/
const Value V22270_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_322 = {S41, 0, &V22270_322_57298, &P22270_321};	/*Property = Prompts*/
const Value V22270_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_323 = {S42, 0, &V22270_323_57301, &P22270_322};	/*Property = Selector*/
const Value V22270_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_324 = {S43, 0, &V22270_324_57307, &P22270_323};	/*Property = Svg*/
const Value V22270_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_327 = {S44, 0, &V22270_327_57316, &P22270_324};	/*Property = TouchEvents*/
const Value V22270_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_328 = {S45, 0, &V22270_328_57319, &P22270_327};	/*Property = Track*/
const Value V22270_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_329 = {S46, 0, &V22270_329_57325, &P22270_328};	/*Property = Video*/
const Value V22270_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_330 = {S47, 0, &V22270_330_57328, &P22270_329};	/*Property = Viewport*/
const Value V22270_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_332 = {S48, 0, &V22270_332_57340, &P22270_330};	/*Property = WebWorkers*/
const Value V22270_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22270_333 = {S49, 0, &V22270_333_57343, &P22270_332};	/*Property = Xhr2*/
const Profile P22270 = {"22270", &P22270_333};	/*Profile ID - 22270*/
const Value V20254_1_1 = {S2, NULL};	/*Value = True*/
const Property P20254_1 = {S50, 0, &V20254_1_1, NULL};	/*Property = IsMobile*/
const Value V20254_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P20254_5 = {S52, 0, &V20254_5_1275, &P20254_1};	/*Property = ScreenPixelsHeight*/
const Value V20254_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20254_4 = {S54, 0, &V20254_4_1229, &P20254_5};	/*Property = ScreenPixelsWidth*/
const Profile P20254 = {"20254", &P20254_4};	/*Profile ID - 20254*/
const Value V3976_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_279 = {S1, 0, &V3976_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3976_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_282 = {S3, 0, &V3976_282_57155, &P3976_279};	/*Property = BlobBuilder*/
const Value V3976_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_350 = {S4, 0, &V3976_350_57446, &P3976_282};	/*Property = Canvas*/
const Value V3976_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_283 = {S5, 0, &V3976_283_57161, &P3976_350};	/*Property = CssBackground*/
const Value V3976_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_284 = {S6, 0, &V3976_284_57164, &P3976_283};	/*Property = CssBorderImage*/
const Value V3976_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_285 = {S7, 0, &V3976_285_57167, &P3976_284};	/*Property = CssCanvas*/
const Value V3976_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_286 = {S8, 0, &V3976_286_57170, &P3976_285};	/*Property = CssColor*/
const Value V3976_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_237 = {S9, 0, &V3976_237_55786, &P3976_286};	/*Property = CssColumn*/
const Value V3976_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_287 = {S10, 0, &V3976_287_57173, &P3976_237};	/*Property = CssFlexbox*/
const Value V3976_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_288 = {S11, 0, &V3976_288_57176, &P3976_287};	/*Property = CssFont*/
const Value V3976_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_289 = {S12, 0, &V3976_289_86899, &P3976_288};	/*Property = CssImages*/
const Value V3976_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_291 = {S13, 0, &V3976_291_57182, &P3976_289};	/*Property = CssMediaQueries*/
const Value V3976_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_292 = {S14, 0, &V3976_292_57185, &P3976_291};	/*Property = CssMinMax*/
const Value V3976_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_293 = {S15, 0, &V3976_293_57188, &P3976_292};	/*Property = CssOverflow*/
const Value V3976_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_294 = {S16, 0, &V3976_294_57191, &P3976_293};	/*Property = CssPosition*/
const Value V3976_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_295 = {S17, 0, &V3976_295_57194, &P3976_294};	/*Property = CssText*/
const Value V3976_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_297 = {S18, 0, &V3976_297_57197, &P3976_295};	/*Property = CssTransforms*/
const Value V3976_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_241 = {S19, 0, &V3976_241_55801, &P3976_297};	/*Property = CssTransitions*/
const Value V3976_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_298 = {S20, 0, &V3976_298_57208, &P3976_241};	/*Property = CssUI*/
const Value V3976_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_300 = {S21, 0, &V3976_300_57217, &P3976_298};	/*Property = DataSet*/
const Value V3976_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_301 = {S22, 0, &V3976_301_57220, &P3976_300};	/*Property = DataUrl*/
const Value V3976_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_302 = {S23, 0, &V3976_302_57223, &P3976_301};	/*Property = DeviceOrientation*/
const Value V3976_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_304 = {S24, 0, &V3976_304_57229, &P3976_302};	/*Property = FileReader*/
const Value V3976_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_305 = {S25, 0, &V3976_305_57232, &P3976_304};	/*Property = FileSaver*/
const Value V3976_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_306 = {S26, 0, &V3976_306_57238, &P3976_305};	/*Property = FileWriter*/
const Value V3976_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_307 = {S27, 0, &V3976_307_57241, &P3976_306};	/*Property = FormData*/
const Value V3976_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_309 = {S28, 0, &V3976_309_57247, &P3976_307};	/*Property = Fullscreen*/
const Value V3976_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_310 = {S29, 0, &V3976_310_57250, &P3976_309};	/*Property = GeoLocation*/
const Value V3976_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_312 = {S30, 0, &V3976_312_57256, &P3976_310};	/*Property = History*/
const Value V3976_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_314 = {S31, 0, &V3976_314_57262, &P3976_312};	/*Property = Html5*/
const Value V3976_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_313 = {S32, 0, &V3976_313_57259, &P3976_314};	/*Property = Html-Media-Capture*/
const Value V3976_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_315 = {S33, 0, &V3976_315_57265, &P3976_313};	/*Property = Iframe*/
const Value V3976_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_316 = {S34, 0, &V3976_316_57268, &P3976_315};	/*Property = IndexedDB*/
const Value V3976_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_317 = {S35, 0, &V3976_317_57271, &P3976_316};	/*Property = Json*/
const Value V3976_15_43263 = {S65, NULL};	/*Value = Netfront*/
const Property P3976_15 = {S37, 0, &V3976_15_43263, &P3976_317};	/*Property = LayoutEngine*/
const Value V3976_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_344 = {S38, 0, &V3976_344_57274, &P3976_15};	/*Property = Masking*/
const Value V3976_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_320 = {S39, 0, &V3976_320_57292, &P3976_344};	/*Property = PostMessage*/
const Value V3976_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_321 = {S40, 0, &V3976_321_57295, &P3976_320};	/*Property = Progress*/
const Value V3976_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_322 = {S41, 0, &V3976_322_57298, &P3976_321};	/*Property = Prompts*/
const Value V3976_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_323 = {S42, 0, &V3976_323_57301, &P3976_322};	/*Property = Selector*/
const Value V3976_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_324 = {S43, 0, &V3976_324_57307, &P3976_323};	/*Property = Svg*/
const Value V3976_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_327 = {S44, 0, &V3976_327_57316, &P3976_324};	/*Property = TouchEvents*/
const Value V3976_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_328 = {S45, 0, &V3976_328_57319, &P3976_327};	/*Property = Track*/
const Value V3976_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_329 = {S46, 0, &V3976_329_57325, &P3976_328};	/*Property = Video*/
const Value V3976_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_330 = {S47, 0, &V3976_330_57328, &P3976_329};	/*Property = Viewport*/
const Value V3976_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_332 = {S48, 0, &V3976_332_57340, &P3976_330};	/*Property = WebWorkers*/
const Value V3976_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3976_333 = {S49, 0, &V3976_333_57343, &P3976_332};	/*Property = Xhr2*/
const Profile P3976 = {"3976", &P3976_333};	/*Profile ID - 3976*/
const Value V13371_1_1 = {S2, NULL};	/*Value = True*/
const Property P13371_1 = {S50, 0, &V13371_1_1, NULL};	/*Property = IsMobile*/
const Value V13371_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13371_5 = {S52, 0, &V13371_5_1222, &P13371_1};	/*Property = ScreenPixelsHeight*/
const Value V13371_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13371_4 = {S54, 0, &V13371_4_1224, &P13371_5};	/*Property = ScreenPixelsWidth*/
const Profile P13371 = {"13371", &P13371_4};	/*Profile ID - 13371*/
const Profile P17485 = {"17485", NULL};	/*Profile ID - 17485*/
const Value V3487_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_279 = {S1, 0, &V3487_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3487_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_282 = {S3, 0, &V3487_282_57155, &P3487_279};	/*Property = BlobBuilder*/
const Value V3487_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_350 = {S4, 0, &V3487_350_57446, &P3487_282};	/*Property = Canvas*/
const Value V3487_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_283 = {S5, 0, &V3487_283_57161, &P3487_350};	/*Property = CssBackground*/
const Value V3487_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_284 = {S6, 0, &V3487_284_57164, &P3487_283};	/*Property = CssBorderImage*/
const Value V3487_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_285 = {S7, 0, &V3487_285_57167, &P3487_284};	/*Property = CssCanvas*/
const Value V3487_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_286 = {S8, 0, &V3487_286_57170, &P3487_285};	/*Property = CssColor*/
const Value V3487_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_237 = {S9, 0, &V3487_237_55786, &P3487_286};	/*Property = CssColumn*/
const Value V3487_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_287 = {S10, 0, &V3487_287_57173, &P3487_237};	/*Property = CssFlexbox*/
const Value V3487_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_288 = {S11, 0, &V3487_288_57176, &P3487_287};	/*Property = CssFont*/
const Value V3487_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_289 = {S12, 0, &V3487_289_86899, &P3487_288};	/*Property = CssImages*/
const Value V3487_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_291 = {S13, 0, &V3487_291_57182, &P3487_289};	/*Property = CssMediaQueries*/
const Value V3487_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_292 = {S14, 0, &V3487_292_57185, &P3487_291};	/*Property = CssMinMax*/
const Value V3487_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_293 = {S15, 0, &V3487_293_57188, &P3487_292};	/*Property = CssOverflow*/
const Value V3487_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_294 = {S16, 0, &V3487_294_57191, &P3487_293};	/*Property = CssPosition*/
const Value V3487_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_295 = {S17, 0, &V3487_295_57194, &P3487_294};	/*Property = CssText*/
const Value V3487_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_297 = {S18, 0, &V3487_297_57197, &P3487_295};	/*Property = CssTransforms*/
const Value V3487_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_241 = {S19, 0, &V3487_241_55801, &P3487_297};	/*Property = CssTransitions*/
const Value V3487_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_298 = {S20, 0, &V3487_298_57208, &P3487_241};	/*Property = CssUI*/
const Value V3487_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_300 = {S21, 0, &V3487_300_57217, &P3487_298};	/*Property = DataSet*/
const Value V3487_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_301 = {S22, 0, &V3487_301_57220, &P3487_300};	/*Property = DataUrl*/
const Value V3487_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_302 = {S23, 0, &V3487_302_57223, &P3487_301};	/*Property = DeviceOrientation*/
const Value V3487_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_304 = {S24, 0, &V3487_304_57229, &P3487_302};	/*Property = FileReader*/
const Value V3487_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_305 = {S25, 0, &V3487_305_57232, &P3487_304};	/*Property = FileSaver*/
const Value V3487_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_306 = {S26, 0, &V3487_306_57238, &P3487_305};	/*Property = FileWriter*/
const Value V3487_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_307 = {S27, 0, &V3487_307_57241, &P3487_306};	/*Property = FormData*/
const Value V3487_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_309 = {S28, 0, &V3487_309_57247, &P3487_307};	/*Property = Fullscreen*/
const Value V3487_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_310 = {S29, 0, &V3487_310_57250, &P3487_309};	/*Property = GeoLocation*/
const Value V3487_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_312 = {S30, 0, &V3487_312_57256, &P3487_310};	/*Property = History*/
const Value V3487_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_314 = {S31, 0, &V3487_314_57262, &P3487_312};	/*Property = Html5*/
const Value V3487_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_313 = {S32, 0, &V3487_313_57259, &P3487_314};	/*Property = Html-Media-Capture*/
const Value V3487_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_315 = {S33, 0, &V3487_315_57265, &P3487_313};	/*Property = Iframe*/
const Value V3487_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_316 = {S34, 0, &V3487_316_57268, &P3487_315};	/*Property = IndexedDB*/
const Value V3487_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_317 = {S35, 0, &V3487_317_57271, &P3487_316};	/*Property = Json*/
const Value V3487_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_15 = {S37, 0, &V3487_15_72, &P3487_317};	/*Property = LayoutEngine*/
const Value V3487_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_344 = {S38, 0, &V3487_344_57274, &P3487_15};	/*Property = Masking*/
const Value V3487_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_320 = {S39, 0, &V3487_320_57292, &P3487_344};	/*Property = PostMessage*/
const Value V3487_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_321 = {S40, 0, &V3487_321_57295, &P3487_320};	/*Property = Progress*/
const Value V3487_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_322 = {S41, 0, &V3487_322_57298, &P3487_321};	/*Property = Prompts*/
const Value V3487_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_323 = {S42, 0, &V3487_323_57301, &P3487_322};	/*Property = Selector*/
const Value V3487_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_324 = {S43, 0, &V3487_324_57307, &P3487_323};	/*Property = Svg*/
const Value V3487_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_327 = {S44, 0, &V3487_327_57316, &P3487_324};	/*Property = TouchEvents*/
const Value V3487_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_328 = {S45, 0, &V3487_328_57319, &P3487_327};	/*Property = Track*/
const Value V3487_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_329 = {S46, 0, &V3487_329_57325, &P3487_328};	/*Property = Video*/
const Value V3487_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_330 = {S47, 0, &V3487_330_57328, &P3487_329};	/*Property = Viewport*/
const Value V3487_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_332 = {S48, 0, &V3487_332_57340, &P3487_330};	/*Property = WebWorkers*/
const Value V3487_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3487_333 = {S49, 0, &V3487_333_57343, &P3487_332};	/*Property = Xhr2*/
const Profile P3487 = {"3487", &P3487_333};	/*Profile ID - 3487*/
const Value V12472_1_1 = {S2, NULL};	/*Value = True*/
const Property P12472_1 = {S50, 0, &V12472_1_1, NULL};	/*Property = IsMobile*/
const Value V12472_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12472_5 = {S52, 0, &V12472_5_1233, &P12472_1};	/*Property = ScreenPixelsHeight*/
const Value V12472_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12472_4 = {S54, 0, &V12472_4_1235, &P12472_5};	/*Property = ScreenPixelsWidth*/
const Profile P12472 = {"12472", &P12472_4};	/*Profile ID - 12472*/
const Profile P17486 = {"17486", NULL};	/*Profile ID - 17486*/
const Value V3605_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_279 = {S1, 0, &V3605_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3605_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_282 = {S3, 0, &V3605_282_57155, &P3605_279};	/*Property = BlobBuilder*/
const Value V3605_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_350 = {S4, 0, &V3605_350_57446, &P3605_282};	/*Property = Canvas*/
const Value V3605_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_283 = {S5, 0, &V3605_283_57161, &P3605_350};	/*Property = CssBackground*/
const Value V3605_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_284 = {S6, 0, &V3605_284_57164, &P3605_283};	/*Property = CssBorderImage*/
const Value V3605_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_285 = {S7, 0, &V3605_285_57167, &P3605_284};	/*Property = CssCanvas*/
const Value V3605_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_286 = {S8, 0, &V3605_286_57170, &P3605_285};	/*Property = CssColor*/
const Value V3605_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_237 = {S9, 0, &V3605_237_55786, &P3605_286};	/*Property = CssColumn*/
const Value V3605_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_287 = {S10, 0, &V3605_287_57173, &P3605_237};	/*Property = CssFlexbox*/
const Value V3605_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_288 = {S11, 0, &V3605_288_57176, &P3605_287};	/*Property = CssFont*/
const Value V3605_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_289 = {S12, 0, &V3605_289_86899, &P3605_288};	/*Property = CssImages*/
const Value V3605_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_291 = {S13, 0, &V3605_291_57182, &P3605_289};	/*Property = CssMediaQueries*/
const Value V3605_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_292 = {S14, 0, &V3605_292_57185, &P3605_291};	/*Property = CssMinMax*/
const Value V3605_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_293 = {S15, 0, &V3605_293_57188, &P3605_292};	/*Property = CssOverflow*/
const Value V3605_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_294 = {S16, 0, &V3605_294_57191, &P3605_293};	/*Property = CssPosition*/
const Value V3605_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_295 = {S17, 0, &V3605_295_57194, &P3605_294};	/*Property = CssText*/
const Value V3605_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_297 = {S18, 0, &V3605_297_57197, &P3605_295};	/*Property = CssTransforms*/
const Value V3605_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_241 = {S19, 0, &V3605_241_55801, &P3605_297};	/*Property = CssTransitions*/
const Value V3605_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_298 = {S20, 0, &V3605_298_57208, &P3605_241};	/*Property = CssUI*/
const Value V3605_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_300 = {S21, 0, &V3605_300_57217, &P3605_298};	/*Property = DataSet*/
const Value V3605_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_301 = {S22, 0, &V3605_301_57220, &P3605_300};	/*Property = DataUrl*/
const Value V3605_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_302 = {S23, 0, &V3605_302_57223, &P3605_301};	/*Property = DeviceOrientation*/
const Value V3605_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_304 = {S24, 0, &V3605_304_57229, &P3605_302};	/*Property = FileReader*/
const Value V3605_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_305 = {S25, 0, &V3605_305_57232, &P3605_304};	/*Property = FileSaver*/
const Value V3605_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_306 = {S26, 0, &V3605_306_57238, &P3605_305};	/*Property = FileWriter*/
const Value V3605_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_307 = {S27, 0, &V3605_307_57241, &P3605_306};	/*Property = FormData*/
const Value V3605_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_309 = {S28, 0, &V3605_309_57247, &P3605_307};	/*Property = Fullscreen*/
const Value V3605_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_310 = {S29, 0, &V3605_310_57250, &P3605_309};	/*Property = GeoLocation*/
const Value V3605_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_312 = {S30, 0, &V3605_312_57256, &P3605_310};	/*Property = History*/
const Value V3605_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_314 = {S31, 0, &V3605_314_57262, &P3605_312};	/*Property = Html5*/
const Value V3605_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_313 = {S32, 0, &V3605_313_57259, &P3605_314};	/*Property = Html-Media-Capture*/
const Value V3605_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_315 = {S33, 0, &V3605_315_57265, &P3605_313};	/*Property = Iframe*/
const Value V3605_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_316 = {S34, 0, &V3605_316_57268, &P3605_315};	/*Property = IndexedDB*/
const Value V3605_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_317 = {S35, 0, &V3605_317_57271, &P3605_316};	/*Property = Json*/
const Value V3605_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3605_15 = {S37, 0, &V3605_15_28, &P3605_317};	/*Property = LayoutEngine*/
const Value V3605_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_344 = {S38, 0, &V3605_344_57274, &P3605_15};	/*Property = Masking*/
const Value V3605_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_320 = {S39, 0, &V3605_320_57292, &P3605_344};	/*Property = PostMessage*/
const Value V3605_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_321 = {S40, 0, &V3605_321_57295, &P3605_320};	/*Property = Progress*/
const Value V3605_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_322 = {S41, 0, &V3605_322_57298, &P3605_321};	/*Property = Prompts*/
const Value V3605_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_323 = {S42, 0, &V3605_323_57301, &P3605_322};	/*Property = Selector*/
const Value V3605_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_324 = {S43, 0, &V3605_324_57307, &P3605_323};	/*Property = Svg*/
const Value V3605_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_327 = {S44, 0, &V3605_327_57316, &P3605_324};	/*Property = TouchEvents*/
const Value V3605_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_328 = {S45, 0, &V3605_328_57319, &P3605_327};	/*Property = Track*/
const Value V3605_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_329 = {S46, 0, &V3605_329_57325, &P3605_328};	/*Property = Video*/
const Value V3605_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_330 = {S47, 0, &V3605_330_57328, &P3605_329};	/*Property = Viewport*/
const Value V3605_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_332 = {S48, 0, &V3605_332_57340, &P3605_330};	/*Property = WebWorkers*/
const Value V3605_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3605_333 = {S49, 0, &V3605_333_57343, &P3605_332};	/*Property = Xhr2*/
const Profile P3605 = {"3605", &P3605_333};	/*Profile ID - 3605*/
const Profile P5926 = {"5926", NULL};	/*Profile ID - 5926*/
const Value V13270_1_1 = {S2, NULL};	/*Value = True*/
const Property P13270_1 = {S50, 0, &V13270_1_1, NULL};	/*Property = IsMobile*/
const Value V13270_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13270_5 = {S52, 0, &V13270_5_1294, &P13270_1};	/*Property = ScreenPixelsHeight*/
const Value V13270_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13270_4 = {S54, 0, &V13270_4_1296, &P13270_5};	/*Property = ScreenPixelsWidth*/
const Profile P13270 = {"13270", &P13270_4};	/*Profile ID - 13270*/
const Value V3439_279_57148 = {S0, NULL};	/*Value = False*/
const Property P3439_279 = {S1, 0, &V3439_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V3439_282_57154 = {S0, NULL};	/*Value = False*/
const Property P3439_282 = {S3, 0, &V3439_282_57154, &P3439_279};	/*Property = BlobBuilder*/
const Value V3439_350_57444 = {S2, NULL};	/*Value = True*/
const Property P3439_350 = {S4, 0, &V3439_350_57444, &P3439_282};	/*Property = Canvas*/
const Value V3439_283_57159 = {S2, NULL};	/*Value = True*/
const Property P3439_283 = {S5, 0, &V3439_283_57159, &P3439_350};	/*Property = CssBackground*/
const Value V3439_284_57163 = {S0, NULL};	/*Value = False*/
const Property P3439_284 = {S6, 0, &V3439_284_57163, &P3439_283};	/*Property = CssBorderImage*/
const Value V3439_285_57166 = {S0, NULL};	/*Value = False*/
const Property P3439_285 = {S7, 0, &V3439_285_57166, &P3439_284};	/*Property = CssCanvas*/
const Value V3439_286_57168 = {S2, NULL};	/*Value = True*/
const Property P3439_286 = {S8, 0, &V3439_286_57168, &P3439_285};	/*Property = CssColor*/
const Value V3439_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3439_237 = {S9, 0, &V3439_237_55786, &P3439_286};	/*Property = CssColumn*/
const Value V3439_287_57172 = {S0, NULL};	/*Value = False*/
const Property P3439_287 = {S10, 0, &V3439_287_57172, &P3439_237};	/*Property = CssFlexbox*/
const Value V3439_288_57175 = {S0, NULL};	/*Value = False*/
const Property P3439_288 = {S11, 0, &V3439_288_57175, &P3439_287};	/*Property = CssFont*/
const Value V3439_289_57759 = {S0, NULL};	/*Value = False*/
const Property P3439_289 = {S12, 0, &V3439_289_57759, &P3439_288};	/*Property = CssImages*/
const Value V3439_291_57181 = {S0, NULL};	/*Value = False*/
const Property P3439_291 = {S13, 0, &V3439_291_57181, &P3439_289};	/*Property = CssMediaQueries*/
const Value V3439_292_57183 = {S2, NULL};	/*Value = True*/
const Property P3439_292 = {S14, 0, &V3439_292_57183, &P3439_291};	/*Property = CssMinMax*/
const Value V3439_293_57187 = {S0, NULL};	/*Value = False*/
const Property P3439_293 = {S15, 0, &V3439_293_57187, &P3439_292};	/*Property = CssOverflow*/
const Value V3439_294_57190 = {S0, NULL};	/*Value = False*/
const Property P3439_294 = {S16, 0, &V3439_294_57190, &P3439_293};	/*Property = CssPosition*/
const Value V3439_295_57192 = {S2, NULL};	/*Value = True*/
const Property P3439_295 = {S17, 0, &V3439_295_57192, &P3439_294};	/*Property = CssText*/
const Value V3439_297_57195 = {S2, NULL};	/*Value = True*/
const Property P3439_297 = {S18, 0, &V3439_297_57195, &P3439_295};	/*Property = CssTransforms*/
const Value V3439_241_55799 = {S2, NULL};	/*Value = True*/
const Property P3439_241 = {S19, 0, &V3439_241_55799, &P3439_297};	/*Property = CssTransitions*/
const Value V3439_298_57206 = {S2, NULL};	/*Value = True*/
const Property P3439_298 = {S20, 0, &V3439_298_57206, &P3439_241};	/*Property = CssUI*/
const Value V3439_300_57216 = {S0, NULL};	/*Value = False*/
const Property P3439_300 = {S21, 0, &V3439_300_57216, &P3439_298};	/*Property = DataSet*/
const Value V3439_301_57218 = {S2, NULL};	/*Value = True*/
const Property P3439_301 = {S22, 0, &V3439_301_57218, &P3439_300};	/*Property = DataUrl*/
const Value V3439_302_57222 = {S0, NULL};	/*Value = False*/
const Property P3439_302 = {S23, 0, &V3439_302_57222, &P3439_301};	/*Property = DeviceOrientation*/
const Value V3439_304_57228 = {S0, NULL};	/*Value = False*/
const Property P3439_304 = {S24, 0, &V3439_304_57228, &P3439_302};	/*Property = FileReader*/
const Value V3439_305_57231 = {S0, NULL};	/*Value = False*/
const Property P3439_305 = {S25, 0, &V3439_305_57231, &P3439_304};	/*Property = FileSaver*/
const Value V3439_306_57237 = {S0, NULL};	/*Value = False*/
const Property P3439_306 = {S26, 0, &V3439_306_57237, &P3439_305};	/*Property = FileWriter*/
const Value V3439_307_57240 = {S0, NULL};	/*Value = False*/
const Property P3439_307 = {S27, 0, &V3439_307_57240, &P3439_306};	/*Property = FormData*/
const Value V3439_309_57246 = {S0, NULL};	/*Value = False*/
const Property P3439_309 = {S28, 0, &V3439_309_57246, &P3439_307};	/*Property = Fullscreen*/
const Value V3439_310_57249 = {S0, NULL};	/*Value = False*/
const Property P3439_310 = {S29, 0, &V3439_310_57249, &P3439_309};	/*Property = GeoLocation*/
const Value V3439_312_57255 = {S0, NULL};	/*Value = False*/
const Property P3439_312 = {S30, 0, &V3439_312_57255, &P3439_310};	/*Property = History*/
const Value V3439_314_57261 = {S0, NULL};	/*Value = False*/
const Property P3439_314 = {S31, 0, &V3439_314_57261, &P3439_312};	/*Property = Html5*/
const Value V3439_313_57258 = {S0, NULL};	/*Value = False*/
const Property P3439_313 = {S32, 0, &V3439_313_57258, &P3439_314};	/*Property = Html-Media-Capture*/
const Value V3439_315_57264 = {S0, NULL};	/*Value = False*/
const Property P3439_315 = {S33, 0, &V3439_315_57264, &P3439_313};	/*Property = Iframe*/
const Value V3439_316_57267 = {S0, NULL};	/*Value = False*/
const Property P3439_316 = {S34, 0, &V3439_316_57267, &P3439_315};	/*Property = IndexedDB*/
const Value V3439_317_57269 = {S2, NULL};	/*Value = True*/
const Property P3439_317 = {S35, 0, &V3439_317_57269, &P3439_316};	/*Property = Json*/
const Value V3439_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3439_15 = {S37, 0, &V3439_15_28, &P3439_317};	/*Property = LayoutEngine*/
const Value V3439_344_57272 = {S2, NULL};	/*Value = True*/
const Property P3439_344 = {S38, 0, &V3439_344_57272, &P3439_15};	/*Property = Masking*/
const Value V3439_320_57290 = {S2, NULL};	/*Value = True*/
const Property P3439_320 = {S39, 0, &V3439_320_57290, &P3439_344};	/*Property = PostMessage*/
const Value V3439_321_57293 = {S2, NULL};	/*Value = True*/
const Property P3439_321 = {S40, 0, &V3439_321_57293, &P3439_320};	/*Property = Progress*/
const Value V3439_322_57296 = {S2, NULL};	/*Value = True*/
const Property P3439_322 = {S41, 0, &V3439_322_57296, &P3439_321};	/*Property = Prompts*/
const Value V3439_323_57299 = {S2, NULL};	/*Value = True*/
const Property P3439_323 = {S42, 0, &V3439_323_57299, &P3439_322};	/*Property = Selector*/
const Value V3439_324_57306 = {S0, NULL};	/*Value = False*/
const Property P3439_324 = {S43, 0, &V3439_324_57306, &P3439_323};	/*Property = Svg*/
const Value V3439_327_57315 = {S0, NULL};	/*Value = False*/
const Property P3439_327 = {S44, 0, &V3439_327_57315, &P3439_324};	/*Property = TouchEvents*/
const Value V3439_328_57318 = {S0, NULL};	/*Value = False*/
const Property P3439_328 = {S45, 0, &V3439_328_57318, &P3439_327};	/*Property = Track*/
const Value V3439_329_57324 = {S0, NULL};	/*Value = False*/
const Property P3439_329 = {S46, 0, &V3439_329_57324, &P3439_328};	/*Property = Video*/
const Value V3439_330_57326 = {S2, NULL};	/*Value = True*/
const Property P3439_330 = {S47, 0, &V3439_330_57326, &P3439_329};	/*Property = Viewport*/
const Value V3439_332_57339 = {S0, NULL};	/*Value = False*/
const Property P3439_332 = {S48, 0, &V3439_332_57339, &P3439_330};	/*Property = WebWorkers*/
const Value V3439_333_57342 = {S0, NULL};	/*Value = False*/
const Property P3439_333 = {S49, 0, &V3439_333_57342, &P3439_332};	/*Property = Xhr2*/
const Profile P3439 = {"3439", &P3439_333};	/*Profile ID - 3439*/
const Profile P5890 = {"5890", NULL};	/*Profile ID - 5890*/
const Value V13554_1_1 = {S2, NULL};	/*Value = True*/
const Property P13554_1 = {S50, 0, &V13554_1_1, NULL};	/*Property = IsMobile*/
const Value V13554_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13554_5 = {S52, 0, &V13554_5_1222, &P13554_1};	/*Property = ScreenPixelsHeight*/
const Value V13554_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13554_4 = {S54, 0, &V13554_4_1224, &P13554_5};	/*Property = ScreenPixelsWidth*/
const Profile P13554 = {"13554", &P13554_4};	/*Profile ID - 13554*/
const Value V13272_1_1 = {S2, NULL};	/*Value = True*/
const Property P13272_1 = {S50, 0, &V13272_1_1, NULL};	/*Property = IsMobile*/
const Value V13272_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13272_5 = {S52, 0, &V13272_5_1222, &P13272_1};	/*Property = ScreenPixelsHeight*/
const Value V13272_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13272_4 = {S54, 0, &V13272_4_1224, &P13272_5};	/*Property = ScreenPixelsWidth*/
const Profile P13272 = {"13272", &P13272_4};	/*Profile ID - 13272*/
const Profile P5843 = {"5843", NULL};	/*Profile ID - 5843*/
const Value V13914_1_1 = {S2, NULL};	/*Value = True*/
const Property P13914_1 = {S50, 0, &V13914_1_1, NULL};	/*Property = IsMobile*/
const Value V13914_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13914_5 = {S52, 0, &V13914_5_1222, &P13914_1};	/*Property = ScreenPixelsHeight*/
const Value V13914_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13914_4 = {S54, 0, &V13914_4_1224, &P13914_5};	/*Property = ScreenPixelsWidth*/
const Profile P13914 = {"13914", &P13914_4};	/*Profile ID - 13914*/
const Value V13273_1_1 = {S2, NULL};	/*Value = True*/
const Property P13273_1 = {S50, 0, &V13273_1_1, NULL};	/*Property = IsMobile*/
const Value V13273_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13273_5 = {S52, 0, &V13273_5_1222, &P13273_1};	/*Property = ScreenPixelsHeight*/
const Value V13273_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13273_4 = {S54, 0, &V13273_4_1224, &P13273_5};	/*Property = ScreenPixelsWidth*/
const Profile P13273 = {"13273", &P13273_4};	/*Profile ID - 13273*/
const Value V3402_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_279 = {S1, 0, &V3402_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3402_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_282 = {S3, 0, &V3402_282_57155, &P3402_279};	/*Property = BlobBuilder*/
const Value V3402_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_350 = {S4, 0, &V3402_350_57446, &P3402_282};	/*Property = Canvas*/
const Value V3402_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_283 = {S5, 0, &V3402_283_57161, &P3402_350};	/*Property = CssBackground*/
const Value V3402_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_284 = {S6, 0, &V3402_284_57164, &P3402_283};	/*Property = CssBorderImage*/
const Value V3402_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_285 = {S7, 0, &V3402_285_57167, &P3402_284};	/*Property = CssCanvas*/
const Value V3402_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_286 = {S8, 0, &V3402_286_57170, &P3402_285};	/*Property = CssColor*/
const Value V3402_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_237 = {S9, 0, &V3402_237_55786, &P3402_286};	/*Property = CssColumn*/
const Value V3402_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_287 = {S10, 0, &V3402_287_57173, &P3402_237};	/*Property = CssFlexbox*/
const Value V3402_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_288 = {S11, 0, &V3402_288_57176, &P3402_287};	/*Property = CssFont*/
const Value V3402_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_289 = {S12, 0, &V3402_289_86899, &P3402_288};	/*Property = CssImages*/
const Value V3402_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_291 = {S13, 0, &V3402_291_57182, &P3402_289};	/*Property = CssMediaQueries*/
const Value V3402_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_292 = {S14, 0, &V3402_292_57185, &P3402_291};	/*Property = CssMinMax*/
const Value V3402_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_293 = {S15, 0, &V3402_293_57188, &P3402_292};	/*Property = CssOverflow*/
const Value V3402_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_294 = {S16, 0, &V3402_294_57191, &P3402_293};	/*Property = CssPosition*/
const Value V3402_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_295 = {S17, 0, &V3402_295_57194, &P3402_294};	/*Property = CssText*/
const Value V3402_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_297 = {S18, 0, &V3402_297_57197, &P3402_295};	/*Property = CssTransforms*/
const Value V3402_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_241 = {S19, 0, &V3402_241_55801, &P3402_297};	/*Property = CssTransitions*/
const Value V3402_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_298 = {S20, 0, &V3402_298_57208, &P3402_241};	/*Property = CssUI*/
const Value V3402_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_300 = {S21, 0, &V3402_300_57217, &P3402_298};	/*Property = DataSet*/
const Value V3402_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_301 = {S22, 0, &V3402_301_57220, &P3402_300};	/*Property = DataUrl*/
const Value V3402_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_302 = {S23, 0, &V3402_302_57223, &P3402_301};	/*Property = DeviceOrientation*/
const Value V3402_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_304 = {S24, 0, &V3402_304_57229, &P3402_302};	/*Property = FileReader*/
const Value V3402_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_305 = {S25, 0, &V3402_305_57232, &P3402_304};	/*Property = FileSaver*/
const Value V3402_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_306 = {S26, 0, &V3402_306_57238, &P3402_305};	/*Property = FileWriter*/
const Value V3402_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_307 = {S27, 0, &V3402_307_57241, &P3402_306};	/*Property = FormData*/
const Value V3402_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_309 = {S28, 0, &V3402_309_57247, &P3402_307};	/*Property = Fullscreen*/
const Value V3402_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_310 = {S29, 0, &V3402_310_57250, &P3402_309};	/*Property = GeoLocation*/
const Value V3402_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_312 = {S30, 0, &V3402_312_57256, &P3402_310};	/*Property = History*/
const Value V3402_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_314 = {S31, 0, &V3402_314_57262, &P3402_312};	/*Property = Html5*/
const Value V3402_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_313 = {S32, 0, &V3402_313_57259, &P3402_314};	/*Property = Html-Media-Capture*/
const Value V3402_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_315 = {S33, 0, &V3402_315_57265, &P3402_313};	/*Property = Iframe*/
const Value V3402_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_316 = {S34, 0, &V3402_316_57268, &P3402_315};	/*Property = IndexedDB*/
const Value V3402_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_317 = {S35, 0, &V3402_317_57271, &P3402_316};	/*Property = Json*/
const Value V3402_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_15 = {S37, 0, &V3402_15_72, &P3402_317};	/*Property = LayoutEngine*/
const Value V3402_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_344 = {S38, 0, &V3402_344_57274, &P3402_15};	/*Property = Masking*/
const Value V3402_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_320 = {S39, 0, &V3402_320_57292, &P3402_344};	/*Property = PostMessage*/
const Value V3402_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_321 = {S40, 0, &V3402_321_57295, &P3402_320};	/*Property = Progress*/
const Value V3402_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_322 = {S41, 0, &V3402_322_57298, &P3402_321};	/*Property = Prompts*/
const Value V3402_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_323 = {S42, 0, &V3402_323_57301, &P3402_322};	/*Property = Selector*/
const Value V3402_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_324 = {S43, 0, &V3402_324_57307, &P3402_323};	/*Property = Svg*/
const Value V3402_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_327 = {S44, 0, &V3402_327_57316, &P3402_324};	/*Property = TouchEvents*/
const Value V3402_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_328 = {S45, 0, &V3402_328_57319, &P3402_327};	/*Property = Track*/
const Value V3402_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_329 = {S46, 0, &V3402_329_57325, &P3402_328};	/*Property = Video*/
const Value V3402_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_330 = {S47, 0, &V3402_330_57328, &P3402_329};	/*Property = Viewport*/
const Value V3402_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_332 = {S48, 0, &V3402_332_57340, &P3402_330};	/*Property = WebWorkers*/
const Value V3402_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3402_333 = {S49, 0, &V3402_333_57343, &P3402_332};	/*Property = Xhr2*/
const Profile P3402 = {"3402", &P3402_333};	/*Profile ID - 3402*/
const Profile P6712 = {"6712", NULL};	/*Profile ID - 6712*/
const Value V13274_1_1 = {S2, NULL};	/*Value = True*/
const Property P13274_1 = {S50, 0, &V13274_1_1, NULL};	/*Property = IsMobile*/
const Value V13274_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13274_5 = {S52, 0, &V13274_5_1293, &P13274_1};	/*Property = ScreenPixelsHeight*/
const Value V13274_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13274_4 = {S54, 0, &V13274_4_1296, &P13274_5};	/*Property = ScreenPixelsWidth*/
const Profile P13274 = {"13274", &P13274_4};	/*Profile ID - 13274*/
const Value V16103_1_1 = {S2, NULL};	/*Value = True*/
const Property P16103_1 = {S50, 0, &V16103_1_1, NULL};	/*Property = IsMobile*/
const Value V16103_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P16103_5 = {S52, 0, &V16103_5_1269, &P16103_1};	/*Property = ScreenPixelsHeight*/
const Value V16103_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16103_4 = {S54, 0, &V16103_4_1229, &P16103_5};	/*Property = ScreenPixelsWidth*/
const Profile P16103 = {"16103", &P16103_4};	/*Profile ID - 16103*/
const Value V4715_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_279 = {S1, 0, &V4715_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4715_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_282 = {S3, 0, &V4715_282_57155, &P4715_279};	/*Property = BlobBuilder*/
const Value V4715_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_350 = {S4, 0, &V4715_350_57446, &P4715_282};	/*Property = Canvas*/
const Value V4715_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_283 = {S5, 0, &V4715_283_57161, &P4715_350};	/*Property = CssBackground*/
const Value V4715_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_284 = {S6, 0, &V4715_284_57164, &P4715_283};	/*Property = CssBorderImage*/
const Value V4715_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_285 = {S7, 0, &V4715_285_57167, &P4715_284};	/*Property = CssCanvas*/
const Value V4715_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_286 = {S8, 0, &V4715_286_57170, &P4715_285};	/*Property = CssColor*/
const Value V4715_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_237 = {S9, 0, &V4715_237_55786, &P4715_286};	/*Property = CssColumn*/
const Value V4715_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_287 = {S10, 0, &V4715_287_57173, &P4715_237};	/*Property = CssFlexbox*/
const Value V4715_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_288 = {S11, 0, &V4715_288_57176, &P4715_287};	/*Property = CssFont*/
const Value V4715_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_289 = {S12, 0, &V4715_289_86899, &P4715_288};	/*Property = CssImages*/
const Value V4715_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_291 = {S13, 0, &V4715_291_57182, &P4715_289};	/*Property = CssMediaQueries*/
const Value V4715_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_292 = {S14, 0, &V4715_292_57185, &P4715_291};	/*Property = CssMinMax*/
const Value V4715_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_293 = {S15, 0, &V4715_293_57188, &P4715_292};	/*Property = CssOverflow*/
const Value V4715_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_294 = {S16, 0, &V4715_294_57191, &P4715_293};	/*Property = CssPosition*/
const Value V4715_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_295 = {S17, 0, &V4715_295_57194, &P4715_294};	/*Property = CssText*/
const Value V4715_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_297 = {S18, 0, &V4715_297_57197, &P4715_295};	/*Property = CssTransforms*/
const Value V4715_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_241 = {S19, 0, &V4715_241_55801, &P4715_297};	/*Property = CssTransitions*/
const Value V4715_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_298 = {S20, 0, &V4715_298_57208, &P4715_241};	/*Property = CssUI*/
const Value V4715_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_300 = {S21, 0, &V4715_300_57217, &P4715_298};	/*Property = DataSet*/
const Value V4715_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_301 = {S22, 0, &V4715_301_57220, &P4715_300};	/*Property = DataUrl*/
const Value V4715_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_302 = {S23, 0, &V4715_302_57223, &P4715_301};	/*Property = DeviceOrientation*/
const Value V4715_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_304 = {S24, 0, &V4715_304_57229, &P4715_302};	/*Property = FileReader*/
const Value V4715_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_305 = {S25, 0, &V4715_305_57232, &P4715_304};	/*Property = FileSaver*/
const Value V4715_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_306 = {S26, 0, &V4715_306_57238, &P4715_305};	/*Property = FileWriter*/
const Value V4715_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_307 = {S27, 0, &V4715_307_57241, &P4715_306};	/*Property = FormData*/
const Value V4715_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_309 = {S28, 0, &V4715_309_57247, &P4715_307};	/*Property = Fullscreen*/
const Value V4715_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_310 = {S29, 0, &V4715_310_57250, &P4715_309};	/*Property = GeoLocation*/
const Value V4715_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_312 = {S30, 0, &V4715_312_57256, &P4715_310};	/*Property = History*/
const Value V4715_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_314 = {S31, 0, &V4715_314_57262, &P4715_312};	/*Property = Html5*/
const Value V4715_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_313 = {S32, 0, &V4715_313_57259, &P4715_314};	/*Property = Html-Media-Capture*/
const Value V4715_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_315 = {S33, 0, &V4715_315_57265, &P4715_313};	/*Property = Iframe*/
const Value V4715_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_316 = {S34, 0, &V4715_316_57268, &P4715_315};	/*Property = IndexedDB*/
const Value V4715_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_317 = {S35, 0, &V4715_317_57271, &P4715_316};	/*Property = Json*/
const Value V4715_15_90838 = {S70, NULL};	/*Value = U3*/
const Property P4715_15 = {S37, 0, &V4715_15_90838, &P4715_317};	/*Property = LayoutEngine*/
const Value V4715_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_344 = {S38, 0, &V4715_344_57274, &P4715_15};	/*Property = Masking*/
const Value V4715_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_320 = {S39, 0, &V4715_320_57292, &P4715_344};	/*Property = PostMessage*/
const Value V4715_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_321 = {S40, 0, &V4715_321_57295, &P4715_320};	/*Property = Progress*/
const Value V4715_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_322 = {S41, 0, &V4715_322_57298, &P4715_321};	/*Property = Prompts*/
const Value V4715_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_323 = {S42, 0, &V4715_323_57301, &P4715_322};	/*Property = Selector*/
const Value V4715_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_324 = {S43, 0, &V4715_324_57307, &P4715_323};	/*Property = Svg*/
const Value V4715_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_327 = {S44, 0, &V4715_327_57316, &P4715_324};	/*Property = TouchEvents*/
const Value V4715_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_328 = {S45, 0, &V4715_328_57319, &P4715_327};	/*Property = Track*/
const Value V4715_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_329 = {S46, 0, &V4715_329_57325, &P4715_328};	/*Property = Video*/
const Value V4715_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_330 = {S47, 0, &V4715_330_57328, &P4715_329};	/*Property = Viewport*/
const Value V4715_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_332 = {S48, 0, &V4715_332_57340, &P4715_330};	/*Property = WebWorkers*/
const Value V4715_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4715_333 = {S49, 0, &V4715_333_57343, &P4715_332};	/*Property = Xhr2*/
const Profile P4715 = {"4715", &P4715_333};	/*Profile ID - 4715*/
const Value V4737_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_279 = {S1, 0, &V4737_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4737_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_282 = {S3, 0, &V4737_282_57155, &P4737_279};	/*Property = BlobBuilder*/
const Value V4737_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_350 = {S4, 0, &V4737_350_57446, &P4737_282};	/*Property = Canvas*/
const Value V4737_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_283 = {S5, 0, &V4737_283_57161, &P4737_350};	/*Property = CssBackground*/
const Value V4737_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_284 = {S6, 0, &V4737_284_57164, &P4737_283};	/*Property = CssBorderImage*/
const Value V4737_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_285 = {S7, 0, &V4737_285_57167, &P4737_284};	/*Property = CssCanvas*/
const Value V4737_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_286 = {S8, 0, &V4737_286_57170, &P4737_285};	/*Property = CssColor*/
const Value V4737_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_237 = {S9, 0, &V4737_237_55786, &P4737_286};	/*Property = CssColumn*/
const Value V4737_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_287 = {S10, 0, &V4737_287_57173, &P4737_237};	/*Property = CssFlexbox*/
const Value V4737_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_288 = {S11, 0, &V4737_288_57176, &P4737_287};	/*Property = CssFont*/
const Value V4737_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_289 = {S12, 0, &V4737_289_86899, &P4737_288};	/*Property = CssImages*/
const Value V4737_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_291 = {S13, 0, &V4737_291_57182, &P4737_289};	/*Property = CssMediaQueries*/
const Value V4737_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_292 = {S14, 0, &V4737_292_57185, &P4737_291};	/*Property = CssMinMax*/
const Value V4737_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_293 = {S15, 0, &V4737_293_57188, &P4737_292};	/*Property = CssOverflow*/
const Value V4737_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_294 = {S16, 0, &V4737_294_57191, &P4737_293};	/*Property = CssPosition*/
const Value V4737_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_295 = {S17, 0, &V4737_295_57194, &P4737_294};	/*Property = CssText*/
const Value V4737_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_297 = {S18, 0, &V4737_297_57197, &P4737_295};	/*Property = CssTransforms*/
const Value V4737_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_241 = {S19, 0, &V4737_241_55801, &P4737_297};	/*Property = CssTransitions*/
const Value V4737_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_298 = {S20, 0, &V4737_298_57208, &P4737_241};	/*Property = CssUI*/
const Value V4737_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_300 = {S21, 0, &V4737_300_57217, &P4737_298};	/*Property = DataSet*/
const Value V4737_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_301 = {S22, 0, &V4737_301_57220, &P4737_300};	/*Property = DataUrl*/
const Value V4737_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_302 = {S23, 0, &V4737_302_57223, &P4737_301};	/*Property = DeviceOrientation*/
const Value V4737_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_304 = {S24, 0, &V4737_304_57229, &P4737_302};	/*Property = FileReader*/
const Value V4737_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_305 = {S25, 0, &V4737_305_57232, &P4737_304};	/*Property = FileSaver*/
const Value V4737_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_306 = {S26, 0, &V4737_306_57238, &P4737_305};	/*Property = FileWriter*/
const Value V4737_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_307 = {S27, 0, &V4737_307_57241, &P4737_306};	/*Property = FormData*/
const Value V4737_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_309 = {S28, 0, &V4737_309_57247, &P4737_307};	/*Property = Fullscreen*/
const Value V4737_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_310 = {S29, 0, &V4737_310_57250, &P4737_309};	/*Property = GeoLocation*/
const Value V4737_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_312 = {S30, 0, &V4737_312_57256, &P4737_310};	/*Property = History*/
const Value V4737_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_314 = {S31, 0, &V4737_314_57262, &P4737_312};	/*Property = Html5*/
const Value V4737_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_313 = {S32, 0, &V4737_313_57259, &P4737_314};	/*Property = Html-Media-Capture*/
const Value V4737_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_315 = {S33, 0, &V4737_315_57265, &P4737_313};	/*Property = Iframe*/
const Value V4737_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_316 = {S34, 0, &V4737_316_57268, &P4737_315};	/*Property = IndexedDB*/
const Value V4737_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_317 = {S35, 0, &V4737_317_57271, &P4737_316};	/*Property = Json*/
const Value V4737_15_90838 = {S70, NULL};	/*Value = U3*/
const Property P4737_15 = {S37, 0, &V4737_15_90838, &P4737_317};	/*Property = LayoutEngine*/
const Value V4737_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_344 = {S38, 0, &V4737_344_57274, &P4737_15};	/*Property = Masking*/
const Value V4737_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_320 = {S39, 0, &V4737_320_57292, &P4737_344};	/*Property = PostMessage*/
const Value V4737_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_321 = {S40, 0, &V4737_321_57295, &P4737_320};	/*Property = Progress*/
const Value V4737_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_322 = {S41, 0, &V4737_322_57298, &P4737_321};	/*Property = Prompts*/
const Value V4737_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_323 = {S42, 0, &V4737_323_57301, &P4737_322};	/*Property = Selector*/
const Value V4737_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_324 = {S43, 0, &V4737_324_57307, &P4737_323};	/*Property = Svg*/
const Value V4737_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_327 = {S44, 0, &V4737_327_57316, &P4737_324};	/*Property = TouchEvents*/
const Value V4737_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_328 = {S45, 0, &V4737_328_57319, &P4737_327};	/*Property = Track*/
const Value V4737_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_329 = {S46, 0, &V4737_329_57325, &P4737_328};	/*Property = Video*/
const Value V4737_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_330 = {S47, 0, &V4737_330_57328, &P4737_329};	/*Property = Viewport*/
const Value V4737_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_332 = {S48, 0, &V4737_332_57340, &P4737_330};	/*Property = WebWorkers*/
const Value V4737_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4737_333 = {S49, 0, &V4737_333_57343, &P4737_332};	/*Property = Xhr2*/
const Profile P4737 = {"4737", &P4737_333};	/*Profile ID - 4737*/
const Value V4742_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_279 = {S1, 0, &V4742_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4742_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_282 = {S3, 0, &V4742_282_57155, &P4742_279};	/*Property = BlobBuilder*/
const Value V4742_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_350 = {S4, 0, &V4742_350_57446, &P4742_282};	/*Property = Canvas*/
const Value V4742_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_283 = {S5, 0, &V4742_283_57161, &P4742_350};	/*Property = CssBackground*/
const Value V4742_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_284 = {S6, 0, &V4742_284_57164, &P4742_283};	/*Property = CssBorderImage*/
const Value V4742_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_285 = {S7, 0, &V4742_285_57167, &P4742_284};	/*Property = CssCanvas*/
const Value V4742_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_286 = {S8, 0, &V4742_286_57170, &P4742_285};	/*Property = CssColor*/
const Value V4742_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_237 = {S9, 0, &V4742_237_55786, &P4742_286};	/*Property = CssColumn*/
const Value V4742_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_287 = {S10, 0, &V4742_287_57173, &P4742_237};	/*Property = CssFlexbox*/
const Value V4742_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_288 = {S11, 0, &V4742_288_57176, &P4742_287};	/*Property = CssFont*/
const Value V4742_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_289 = {S12, 0, &V4742_289_86899, &P4742_288};	/*Property = CssImages*/
const Value V4742_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_291 = {S13, 0, &V4742_291_57182, &P4742_289};	/*Property = CssMediaQueries*/
const Value V4742_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_292 = {S14, 0, &V4742_292_57185, &P4742_291};	/*Property = CssMinMax*/
const Value V4742_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_293 = {S15, 0, &V4742_293_57188, &P4742_292};	/*Property = CssOverflow*/
const Value V4742_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_294 = {S16, 0, &V4742_294_57191, &P4742_293};	/*Property = CssPosition*/
const Value V4742_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_295 = {S17, 0, &V4742_295_57194, &P4742_294};	/*Property = CssText*/
const Value V4742_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_297 = {S18, 0, &V4742_297_57197, &P4742_295};	/*Property = CssTransforms*/
const Value V4742_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_241 = {S19, 0, &V4742_241_55801, &P4742_297};	/*Property = CssTransitions*/
const Value V4742_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_298 = {S20, 0, &V4742_298_57208, &P4742_241};	/*Property = CssUI*/
const Value V4742_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_300 = {S21, 0, &V4742_300_57217, &P4742_298};	/*Property = DataSet*/
const Value V4742_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_301 = {S22, 0, &V4742_301_57220, &P4742_300};	/*Property = DataUrl*/
const Value V4742_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_302 = {S23, 0, &V4742_302_57223, &P4742_301};	/*Property = DeviceOrientation*/
const Value V4742_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_304 = {S24, 0, &V4742_304_57229, &P4742_302};	/*Property = FileReader*/
const Value V4742_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_305 = {S25, 0, &V4742_305_57232, &P4742_304};	/*Property = FileSaver*/
const Value V4742_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_306 = {S26, 0, &V4742_306_57238, &P4742_305};	/*Property = FileWriter*/
const Value V4742_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_307 = {S27, 0, &V4742_307_57241, &P4742_306};	/*Property = FormData*/
const Value V4742_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_309 = {S28, 0, &V4742_309_57247, &P4742_307};	/*Property = Fullscreen*/
const Value V4742_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_310 = {S29, 0, &V4742_310_57250, &P4742_309};	/*Property = GeoLocation*/
const Value V4742_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_312 = {S30, 0, &V4742_312_57256, &P4742_310};	/*Property = History*/
const Value V4742_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_314 = {S31, 0, &V4742_314_57262, &P4742_312};	/*Property = Html5*/
const Value V4742_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_313 = {S32, 0, &V4742_313_57259, &P4742_314};	/*Property = Html-Media-Capture*/
const Value V4742_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_315 = {S33, 0, &V4742_315_57265, &P4742_313};	/*Property = Iframe*/
const Value V4742_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_316 = {S34, 0, &V4742_316_57268, &P4742_315};	/*Property = IndexedDB*/
const Value V4742_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_317 = {S35, 0, &V4742_317_57271, &P4742_316};	/*Property = Json*/
const Value V4742_15_90838 = {S70, NULL};	/*Value = U3*/
const Property P4742_15 = {S37, 0, &V4742_15_90838, &P4742_317};	/*Property = LayoutEngine*/
const Value V4742_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_344 = {S38, 0, &V4742_344_57274, &P4742_15};	/*Property = Masking*/
const Value V4742_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_320 = {S39, 0, &V4742_320_57292, &P4742_344};	/*Property = PostMessage*/
const Value V4742_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_321 = {S40, 0, &V4742_321_57295, &P4742_320};	/*Property = Progress*/
const Value V4742_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_322 = {S41, 0, &V4742_322_57298, &P4742_321};	/*Property = Prompts*/
const Value V4742_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_323 = {S42, 0, &V4742_323_57301, &P4742_322};	/*Property = Selector*/
const Value V4742_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_324 = {S43, 0, &V4742_324_57307, &P4742_323};	/*Property = Svg*/
const Value V4742_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_327 = {S44, 0, &V4742_327_57316, &P4742_324};	/*Property = TouchEvents*/
const Value V4742_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_328 = {S45, 0, &V4742_328_57319, &P4742_327};	/*Property = Track*/
const Value V4742_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_329 = {S46, 0, &V4742_329_57325, &P4742_328};	/*Property = Video*/
const Value V4742_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_330 = {S47, 0, &V4742_330_57328, &P4742_329};	/*Property = Viewport*/
const Value V4742_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_332 = {S48, 0, &V4742_332_57340, &P4742_330};	/*Property = WebWorkers*/
const Value V4742_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4742_333 = {S49, 0, &V4742_333_57343, &P4742_332};	/*Property = Xhr2*/
const Profile P4742 = {"4742", &P4742_333};	/*Profile ID - 4742*/
const Value V13399_1_1 = {S2, NULL};	/*Value = True*/
const Property P13399_1 = {S50, 0, &V13399_1_1, NULL};	/*Property = IsMobile*/
const Value V13399_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13399_5 = {S52, 0, &V13399_5_1293, &P13399_1};	/*Property = ScreenPixelsHeight*/
const Value V13399_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13399_4 = {S54, 0, &V13399_4_1296, &P13399_5};	/*Property = ScreenPixelsWidth*/
const Profile P13399 = {"13399", &P13399_4};	/*Profile ID - 13399*/
const Value V15127_1_1 = {S2, NULL};	/*Value = True*/
const Property P15127_1 = {S50, 0, &V15127_1_1, NULL};	/*Property = IsMobile*/
const Value V15127_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15127_5 = {S52, 0, &V15127_5_1233, &P15127_1};	/*Property = ScreenPixelsHeight*/
const Value V15127_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15127_4 = {S54, 0, &V15127_4_1235, &P15127_5};	/*Property = ScreenPixelsWidth*/
const Profile P15127 = {"15127", &P15127_4};	/*Profile ID - 15127*/
const Value V11795_1_1 = {S2, NULL};	/*Value = True*/
const Property P11795_1 = {S50, 0, &V11795_1_1, NULL};	/*Property = IsMobile*/
const Value V11795_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11795_5 = {S52, 0, &V11795_5_1293, &P11795_1};	/*Property = ScreenPixelsHeight*/
const Value V11795_4_24031 = {S71, NULL};	/*Value = 120*/
const Property P11795_4 = {S54, 0, &V11795_4_24031, &P11795_5};	/*Property = ScreenPixelsWidth*/
const Profile P11795 = {"11795", &P11795_4};	/*Profile ID - 11795*/
const Value V11798_1_1 = {S2, NULL};	/*Value = True*/
const Property P11798_1 = {S50, 0, &V11798_1_1, NULL};	/*Property = IsMobile*/
const Value V11798_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11798_5 = {S52, 0, &V11798_5_1233, &P11798_1};	/*Property = ScreenPixelsHeight*/
const Value V11798_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11798_4 = {S54, 0, &V11798_4_1235, &P11798_5};	/*Property = ScreenPixelsWidth*/
const Profile P11798 = {"11798", &P11798_4};	/*Profile ID - 11798*/
const Value V13451_1_1 = {S2, NULL};	/*Value = True*/
const Property P13451_1 = {S50, 0, &V13451_1_1, NULL};	/*Property = IsMobile*/
const Value V13451_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13451_5 = {S52, 0, &V13451_5_1222, &P13451_1};	/*Property = ScreenPixelsHeight*/
const Value V13451_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13451_4 = {S54, 0, &V13451_4_1224, &P13451_5};	/*Property = ScreenPixelsWidth*/
const Profile P13451 = {"13451", &P13451_4};	/*Profile ID - 13451*/
const Value V12375_1_1 = {S2, NULL};	/*Value = True*/
const Property P12375_1 = {S50, 0, &V12375_1_1, NULL};	/*Property = IsMobile*/
const Value V12375_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12375_5 = {S52, 0, &V12375_5_1222, &P12375_1};	/*Property = ScreenPixelsHeight*/
const Value V12375_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12375_4 = {S54, 0, &V12375_4_1224, &P12375_5};	/*Property = ScreenPixelsWidth*/
const Profile P12375 = {"12375", &P12375_4};	/*Profile ID - 12375*/
const Value V12544_1_1 = {S2, NULL};	/*Value = True*/
const Property P12544_1 = {S50, 0, &V12544_1_1, NULL};	/*Property = IsMobile*/
const Value V12544_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12544_5 = {S52, 0, &V12544_5_1222, &P12544_1};	/*Property = ScreenPixelsHeight*/
const Value V12544_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12544_4 = {S54, 0, &V12544_4_1224, &P12544_5};	/*Property = ScreenPixelsWidth*/
const Profile P12544 = {"12544", &P12544_4};	/*Profile ID - 12544*/
const Value V20449_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_279 = {S1, 0, &V20449_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20449_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_282 = {S3, 0, &V20449_282_57155, &P20449_279};	/*Property = BlobBuilder*/
const Value V20449_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_350 = {S4, 0, &V20449_350_57446, &P20449_282};	/*Property = Canvas*/
const Value V20449_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_283 = {S5, 0, &V20449_283_57161, &P20449_350};	/*Property = CssBackground*/
const Value V20449_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_284 = {S6, 0, &V20449_284_57164, &P20449_283};	/*Property = CssBorderImage*/
const Value V20449_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_285 = {S7, 0, &V20449_285_57167, &P20449_284};	/*Property = CssCanvas*/
const Value V20449_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_286 = {S8, 0, &V20449_286_57170, &P20449_285};	/*Property = CssColor*/
const Value V20449_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_237 = {S9, 0, &V20449_237_55786, &P20449_286};	/*Property = CssColumn*/
const Value V20449_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_287 = {S10, 0, &V20449_287_57173, &P20449_237};	/*Property = CssFlexbox*/
const Value V20449_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_288 = {S11, 0, &V20449_288_57176, &P20449_287};	/*Property = CssFont*/
const Value V20449_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_289 = {S12, 0, &V20449_289_86899, &P20449_288};	/*Property = CssImages*/
const Value V20449_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_291 = {S13, 0, &V20449_291_57182, &P20449_289};	/*Property = CssMediaQueries*/
const Value V20449_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_292 = {S14, 0, &V20449_292_57185, &P20449_291};	/*Property = CssMinMax*/
const Value V20449_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_293 = {S15, 0, &V20449_293_57188, &P20449_292};	/*Property = CssOverflow*/
const Value V20449_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_294 = {S16, 0, &V20449_294_57191, &P20449_293};	/*Property = CssPosition*/
const Value V20449_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_295 = {S17, 0, &V20449_295_57194, &P20449_294};	/*Property = CssText*/
const Value V20449_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_297 = {S18, 0, &V20449_297_57197, &P20449_295};	/*Property = CssTransforms*/
const Value V20449_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_241 = {S19, 0, &V20449_241_55801, &P20449_297};	/*Property = CssTransitions*/
const Value V20449_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_298 = {S20, 0, &V20449_298_57208, &P20449_241};	/*Property = CssUI*/
const Value V20449_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_300 = {S21, 0, &V20449_300_57217, &P20449_298};	/*Property = DataSet*/
const Value V20449_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_301 = {S22, 0, &V20449_301_57220, &P20449_300};	/*Property = DataUrl*/
const Value V20449_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_302 = {S23, 0, &V20449_302_57223, &P20449_301};	/*Property = DeviceOrientation*/
const Value V20449_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_304 = {S24, 0, &V20449_304_57229, &P20449_302};	/*Property = FileReader*/
const Value V20449_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_305 = {S25, 0, &V20449_305_57232, &P20449_304};	/*Property = FileSaver*/
const Value V20449_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_306 = {S26, 0, &V20449_306_57238, &P20449_305};	/*Property = FileWriter*/
const Value V20449_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_307 = {S27, 0, &V20449_307_57241, &P20449_306};	/*Property = FormData*/
const Value V20449_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_309 = {S28, 0, &V20449_309_57247, &P20449_307};	/*Property = Fullscreen*/
const Value V20449_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_310 = {S29, 0, &V20449_310_57250, &P20449_309};	/*Property = GeoLocation*/
const Value V20449_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_312 = {S30, 0, &V20449_312_57256, &P20449_310};	/*Property = History*/
const Value V20449_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_314 = {S31, 0, &V20449_314_57262, &P20449_312};	/*Property = Html5*/
const Value V20449_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_313 = {S32, 0, &V20449_313_57259, &P20449_314};	/*Property = Html-Media-Capture*/
const Value V20449_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_315 = {S33, 0, &V20449_315_57265, &P20449_313};	/*Property = Iframe*/
const Value V20449_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_316 = {S34, 0, &V20449_316_57268, &P20449_315};	/*Property = IndexedDB*/
const Value V20449_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_317 = {S35, 0, &V20449_317_57271, &P20449_316};	/*Property = Json*/
const Value V20449_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P20449_15 = {S37, 0, &V20449_15_31, &P20449_317};	/*Property = LayoutEngine*/
const Value V20449_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_344 = {S38, 0, &V20449_344_57274, &P20449_15};	/*Property = Masking*/
const Value V20449_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_320 = {S39, 0, &V20449_320_57292, &P20449_344};	/*Property = PostMessage*/
const Value V20449_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_321 = {S40, 0, &V20449_321_57295, &P20449_320};	/*Property = Progress*/
const Value V20449_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_322 = {S41, 0, &V20449_322_57298, &P20449_321};	/*Property = Prompts*/
const Value V20449_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_323 = {S42, 0, &V20449_323_57301, &P20449_322};	/*Property = Selector*/
const Value V20449_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_324 = {S43, 0, &V20449_324_57307, &P20449_323};	/*Property = Svg*/
const Value V20449_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_327 = {S44, 0, &V20449_327_57316, &P20449_324};	/*Property = TouchEvents*/
const Value V20449_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_328 = {S45, 0, &V20449_328_57319, &P20449_327};	/*Property = Track*/
const Value V20449_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_329 = {S46, 0, &V20449_329_57325, &P20449_328};	/*Property = Video*/
const Value V20449_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_330 = {S47, 0, &V20449_330_57328, &P20449_329};	/*Property = Viewport*/
const Value V20449_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_332 = {S48, 0, &V20449_332_57340, &P20449_330};	/*Property = WebWorkers*/
const Value V20449_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20449_333 = {S49, 0, &V20449_333_57343, &P20449_332};	/*Property = Xhr2*/
const Profile P20449 = {"20449", &P20449_333};	/*Profile ID - 20449*/
const Value V4037_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_279 = {S1, 0, &V4037_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4037_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_282 = {S3, 0, &V4037_282_57155, &P4037_279};	/*Property = BlobBuilder*/
const Value V4037_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_350 = {S4, 0, &V4037_350_57446, &P4037_282};	/*Property = Canvas*/
const Value V4037_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_283 = {S5, 0, &V4037_283_57161, &P4037_350};	/*Property = CssBackground*/
const Value V4037_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_284 = {S6, 0, &V4037_284_57164, &P4037_283};	/*Property = CssBorderImage*/
const Value V4037_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_285 = {S7, 0, &V4037_285_57167, &P4037_284};	/*Property = CssCanvas*/
const Value V4037_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_286 = {S8, 0, &V4037_286_57170, &P4037_285};	/*Property = CssColor*/
const Value V4037_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_237 = {S9, 0, &V4037_237_55786, &P4037_286};	/*Property = CssColumn*/
const Value V4037_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_287 = {S10, 0, &V4037_287_57173, &P4037_237};	/*Property = CssFlexbox*/
const Value V4037_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_288 = {S11, 0, &V4037_288_57176, &P4037_287};	/*Property = CssFont*/
const Value V4037_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_289 = {S12, 0, &V4037_289_86899, &P4037_288};	/*Property = CssImages*/
const Value V4037_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_291 = {S13, 0, &V4037_291_57182, &P4037_289};	/*Property = CssMediaQueries*/
const Value V4037_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_292 = {S14, 0, &V4037_292_57185, &P4037_291};	/*Property = CssMinMax*/
const Value V4037_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_293 = {S15, 0, &V4037_293_57188, &P4037_292};	/*Property = CssOverflow*/
const Value V4037_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_294 = {S16, 0, &V4037_294_57191, &P4037_293};	/*Property = CssPosition*/
const Value V4037_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_295 = {S17, 0, &V4037_295_57194, &P4037_294};	/*Property = CssText*/
const Value V4037_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_297 = {S18, 0, &V4037_297_57197, &P4037_295};	/*Property = CssTransforms*/
const Value V4037_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_241 = {S19, 0, &V4037_241_55801, &P4037_297};	/*Property = CssTransitions*/
const Value V4037_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_298 = {S20, 0, &V4037_298_57208, &P4037_241};	/*Property = CssUI*/
const Value V4037_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_300 = {S21, 0, &V4037_300_57217, &P4037_298};	/*Property = DataSet*/
const Value V4037_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_301 = {S22, 0, &V4037_301_57220, &P4037_300};	/*Property = DataUrl*/
const Value V4037_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_302 = {S23, 0, &V4037_302_57223, &P4037_301};	/*Property = DeviceOrientation*/
const Value V4037_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_304 = {S24, 0, &V4037_304_57229, &P4037_302};	/*Property = FileReader*/
const Value V4037_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_305 = {S25, 0, &V4037_305_57232, &P4037_304};	/*Property = FileSaver*/
const Value V4037_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_306 = {S26, 0, &V4037_306_57238, &P4037_305};	/*Property = FileWriter*/
const Value V4037_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_307 = {S27, 0, &V4037_307_57241, &P4037_306};	/*Property = FormData*/
const Value V4037_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_309 = {S28, 0, &V4037_309_57247, &P4037_307};	/*Property = Fullscreen*/
const Value V4037_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_310 = {S29, 0, &V4037_310_57250, &P4037_309};	/*Property = GeoLocation*/
const Value V4037_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_312 = {S30, 0, &V4037_312_57256, &P4037_310};	/*Property = History*/
const Value V4037_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_314 = {S31, 0, &V4037_314_57262, &P4037_312};	/*Property = Html5*/
const Value V4037_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_313 = {S32, 0, &V4037_313_57259, &P4037_314};	/*Property = Html-Media-Capture*/
const Value V4037_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_315 = {S33, 0, &V4037_315_57265, &P4037_313};	/*Property = Iframe*/
const Value V4037_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_316 = {S34, 0, &V4037_316_57268, &P4037_315};	/*Property = IndexedDB*/
const Value V4037_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_317 = {S35, 0, &V4037_317_57271, &P4037_316};	/*Property = Json*/
const Value V4037_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P4037_15 = {S37, 0, &V4037_15_28, &P4037_317};	/*Property = LayoutEngine*/
const Value V4037_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_344 = {S38, 0, &V4037_344_57274, &P4037_15};	/*Property = Masking*/
const Value V4037_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_320 = {S39, 0, &V4037_320_57292, &P4037_344};	/*Property = PostMessage*/
const Value V4037_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_321 = {S40, 0, &V4037_321_57295, &P4037_320};	/*Property = Progress*/
const Value V4037_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_322 = {S41, 0, &V4037_322_57298, &P4037_321};	/*Property = Prompts*/
const Value V4037_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_323 = {S42, 0, &V4037_323_57301, &P4037_322};	/*Property = Selector*/
const Value V4037_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_324 = {S43, 0, &V4037_324_57307, &P4037_323};	/*Property = Svg*/
const Value V4037_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_327 = {S44, 0, &V4037_327_57316, &P4037_324};	/*Property = TouchEvents*/
const Value V4037_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_328 = {S45, 0, &V4037_328_57319, &P4037_327};	/*Property = Track*/
const Value V4037_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_329 = {S46, 0, &V4037_329_57325, &P4037_328};	/*Property = Video*/
const Value V4037_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_330 = {S47, 0, &V4037_330_57328, &P4037_329};	/*Property = Viewport*/
const Value V4037_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_332 = {S48, 0, &V4037_332_57340, &P4037_330};	/*Property = WebWorkers*/
const Value V4037_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4037_333 = {S49, 0, &V4037_333_57343, &P4037_332};	/*Property = Xhr2*/
const Profile P4037 = {"4037", &P4037_333};	/*Profile ID - 4037*/
const Value V16012_1_1 = {S2, NULL};	/*Value = True*/
const Property P16012_1 = {S50, 0, &V16012_1_1, NULL};	/*Property = IsMobile*/
const Value V16012_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16012_5 = {S52, 0, &V16012_5_1222, &P16012_1};	/*Property = ScreenPixelsHeight*/
const Value V16012_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16012_4 = {S54, 0, &V16012_4_1224, &P16012_5};	/*Property = ScreenPixelsWidth*/
const Profile P16012 = {"16012", &P16012_4};	/*Profile ID - 16012*/
const Value V11805_1_1 = {S2, NULL};	/*Value = True*/
const Property P11805_1 = {S50, 0, &V11805_1_1, NULL};	/*Property = IsMobile*/
const Value V11805_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11805_5 = {S52, 0, &V11805_5_1293, &P11805_1};	/*Property = ScreenPixelsHeight*/
const Value V11805_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11805_4 = {S54, 0, &V11805_4_1296, &P11805_5};	/*Property = ScreenPixelsWidth*/
const Profile P11805 = {"11805", &P11805_4};	/*Profile ID - 11805*/
const Value V4836_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_279 = {S1, 0, &V4836_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4836_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_282 = {S3, 0, &V4836_282_57155, &P4836_279};	/*Property = BlobBuilder*/
const Value V4836_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_350 = {S4, 0, &V4836_350_57446, &P4836_282};	/*Property = Canvas*/
const Value V4836_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_283 = {S5, 0, &V4836_283_57161, &P4836_350};	/*Property = CssBackground*/
const Value V4836_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_284 = {S6, 0, &V4836_284_57164, &P4836_283};	/*Property = CssBorderImage*/
const Value V4836_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_285 = {S7, 0, &V4836_285_57167, &P4836_284};	/*Property = CssCanvas*/
const Value V4836_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_286 = {S8, 0, &V4836_286_57170, &P4836_285};	/*Property = CssColor*/
const Value V4836_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_237 = {S9, 0, &V4836_237_55786, &P4836_286};	/*Property = CssColumn*/
const Value V4836_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_287 = {S10, 0, &V4836_287_57173, &P4836_237};	/*Property = CssFlexbox*/
const Value V4836_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_288 = {S11, 0, &V4836_288_57176, &P4836_287};	/*Property = CssFont*/
const Value V4836_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_289 = {S12, 0, &V4836_289_86899, &P4836_288};	/*Property = CssImages*/
const Value V4836_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_291 = {S13, 0, &V4836_291_57182, &P4836_289};	/*Property = CssMediaQueries*/
const Value V4836_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_292 = {S14, 0, &V4836_292_57185, &P4836_291};	/*Property = CssMinMax*/
const Value V4836_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_293 = {S15, 0, &V4836_293_57188, &P4836_292};	/*Property = CssOverflow*/
const Value V4836_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_294 = {S16, 0, &V4836_294_57191, &P4836_293};	/*Property = CssPosition*/
const Value V4836_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_295 = {S17, 0, &V4836_295_57194, &P4836_294};	/*Property = CssText*/
const Value V4836_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_297 = {S18, 0, &V4836_297_57197, &P4836_295};	/*Property = CssTransforms*/
const Value V4836_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_241 = {S19, 0, &V4836_241_55801, &P4836_297};	/*Property = CssTransitions*/
const Value V4836_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_298 = {S20, 0, &V4836_298_57208, &P4836_241};	/*Property = CssUI*/
const Value V4836_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_300 = {S21, 0, &V4836_300_57217, &P4836_298};	/*Property = DataSet*/
const Value V4836_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_301 = {S22, 0, &V4836_301_57220, &P4836_300};	/*Property = DataUrl*/
const Value V4836_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_302 = {S23, 0, &V4836_302_57223, &P4836_301};	/*Property = DeviceOrientation*/
const Value V4836_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_304 = {S24, 0, &V4836_304_57229, &P4836_302};	/*Property = FileReader*/
const Value V4836_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_305 = {S25, 0, &V4836_305_57232, &P4836_304};	/*Property = FileSaver*/
const Value V4836_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_306 = {S26, 0, &V4836_306_57238, &P4836_305};	/*Property = FileWriter*/
const Value V4836_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_307 = {S27, 0, &V4836_307_57241, &P4836_306};	/*Property = FormData*/
const Value V4836_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_309 = {S28, 0, &V4836_309_57247, &P4836_307};	/*Property = Fullscreen*/
const Value V4836_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_310 = {S29, 0, &V4836_310_57250, &P4836_309};	/*Property = GeoLocation*/
const Value V4836_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_312 = {S30, 0, &V4836_312_57256, &P4836_310};	/*Property = History*/
const Value V4836_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_314 = {S31, 0, &V4836_314_57262, &P4836_312};	/*Property = Html5*/
const Value V4836_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_313 = {S32, 0, &V4836_313_57259, &P4836_314};	/*Property = Html-Media-Capture*/
const Value V4836_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_315 = {S33, 0, &V4836_315_57265, &P4836_313};	/*Property = Iframe*/
const Value V4836_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_316 = {S34, 0, &V4836_316_57268, &P4836_315};	/*Property = IndexedDB*/
const Value V4836_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_317 = {S35, 0, &V4836_317_57271, &P4836_316};	/*Property = Json*/
const Value V4836_15_44266 = {S72, NULL};	/*Value = MotoWebkit*/
const Property P4836_15 = {S37, 0, &V4836_15_44266, &P4836_317};	/*Property = LayoutEngine*/
const Value V4836_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_344 = {S38, 0, &V4836_344_57274, &P4836_15};	/*Property = Masking*/
const Value V4836_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_320 = {S39, 0, &V4836_320_57292, &P4836_344};	/*Property = PostMessage*/
const Value V4836_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_321 = {S40, 0, &V4836_321_57295, &P4836_320};	/*Property = Progress*/
const Value V4836_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_322 = {S41, 0, &V4836_322_57298, &P4836_321};	/*Property = Prompts*/
const Value V4836_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_323 = {S42, 0, &V4836_323_57301, &P4836_322};	/*Property = Selector*/
const Value V4836_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_324 = {S43, 0, &V4836_324_57307, &P4836_323};	/*Property = Svg*/
const Value V4836_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_327 = {S44, 0, &V4836_327_57316, &P4836_324};	/*Property = TouchEvents*/
const Value V4836_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_328 = {S45, 0, &V4836_328_57319, &P4836_327};	/*Property = Track*/
const Value V4836_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_329 = {S46, 0, &V4836_329_57325, &P4836_328};	/*Property = Video*/
const Value V4836_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_330 = {S47, 0, &V4836_330_57328, &P4836_329};	/*Property = Viewport*/
const Value V4836_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_332 = {S48, 0, &V4836_332_57340, &P4836_330};	/*Property = WebWorkers*/
const Value V4836_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4836_333 = {S49, 0, &V4836_333_57343, &P4836_332};	/*Property = Xhr2*/
const Profile P4836 = {"4836", &P4836_333};	/*Profile ID - 4836*/
const Profile P5704 = {"5704", NULL};	/*Profile ID - 5704*/
const Value V11806_1_1 = {S2, NULL};	/*Value = True*/
const Property P11806_1 = {S50, 0, &V11806_1_1, NULL};	/*Property = IsMobile*/
const Value V11806_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11806_5 = {S52, 0, &V11806_5_1233, &P11806_1};	/*Property = ScreenPixelsHeight*/
const Value V11806_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11806_4 = {S54, 0, &V11806_4_1235, &P11806_5};	/*Property = ScreenPixelsWidth*/
const Profile P11806 = {"11806", &P11806_4};	/*Profile ID - 11806*/
const Profile P17017 = {"17017", NULL};	/*Profile ID - 17017*/
const Value V18088_1_1 = {S2, NULL};	/*Value = True*/
const Property P18088_1 = {S50, 0, &V18088_1_1, NULL};	/*Property = IsMobile*/
const Value V18088_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P18088_5 = {S52, 0, &V18088_5_37, &P18088_1};	/*Property = ScreenPixelsHeight*/
const Value V18088_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P18088_4 = {S54, 0, &V18088_4_36, &P18088_5};	/*Property = ScreenPixelsWidth*/
const Profile P18088 = {"18088", &P18088_4};	/*Profile ID - 18088*/
const Value V17368_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_279 = {S1, 0, &V17368_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17368_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_282 = {S3, 0, &V17368_282_57155, &P17368_279};	/*Property = BlobBuilder*/
const Value V17368_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_350 = {S4, 0, &V17368_350_57446, &P17368_282};	/*Property = Canvas*/
const Value V17368_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_283 = {S5, 0, &V17368_283_57161, &P17368_350};	/*Property = CssBackground*/
const Value V17368_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_284 = {S6, 0, &V17368_284_57164, &P17368_283};	/*Property = CssBorderImage*/
const Value V17368_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_285 = {S7, 0, &V17368_285_57167, &P17368_284};	/*Property = CssCanvas*/
const Value V17368_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_286 = {S8, 0, &V17368_286_57170, &P17368_285};	/*Property = CssColor*/
const Value V17368_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_237 = {S9, 0, &V17368_237_55786, &P17368_286};	/*Property = CssColumn*/
const Value V17368_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_287 = {S10, 0, &V17368_287_57173, &P17368_237};	/*Property = CssFlexbox*/
const Value V17368_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_288 = {S11, 0, &V17368_288_57176, &P17368_287};	/*Property = CssFont*/
const Value V17368_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_289 = {S12, 0, &V17368_289_86899, &P17368_288};	/*Property = CssImages*/
const Value V17368_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_291 = {S13, 0, &V17368_291_57182, &P17368_289};	/*Property = CssMediaQueries*/
const Value V17368_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_292 = {S14, 0, &V17368_292_57185, &P17368_291};	/*Property = CssMinMax*/
const Value V17368_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_293 = {S15, 0, &V17368_293_57188, &P17368_292};	/*Property = CssOverflow*/
const Value V17368_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_294 = {S16, 0, &V17368_294_57191, &P17368_293};	/*Property = CssPosition*/
const Value V17368_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_295 = {S17, 0, &V17368_295_57194, &P17368_294};	/*Property = CssText*/
const Value V17368_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_297 = {S18, 0, &V17368_297_57197, &P17368_295};	/*Property = CssTransforms*/
const Value V17368_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_241 = {S19, 0, &V17368_241_55801, &P17368_297};	/*Property = CssTransitions*/
const Value V17368_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_298 = {S20, 0, &V17368_298_57208, &P17368_241};	/*Property = CssUI*/
const Value V17368_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_300 = {S21, 0, &V17368_300_57217, &P17368_298};	/*Property = DataSet*/
const Value V17368_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_301 = {S22, 0, &V17368_301_57220, &P17368_300};	/*Property = DataUrl*/
const Value V17368_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_302 = {S23, 0, &V17368_302_57223, &P17368_301};	/*Property = DeviceOrientation*/
const Value V17368_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_304 = {S24, 0, &V17368_304_57229, &P17368_302};	/*Property = FileReader*/
const Value V17368_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_305 = {S25, 0, &V17368_305_57232, &P17368_304};	/*Property = FileSaver*/
const Value V17368_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_306 = {S26, 0, &V17368_306_57238, &P17368_305};	/*Property = FileWriter*/
const Value V17368_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_307 = {S27, 0, &V17368_307_57241, &P17368_306};	/*Property = FormData*/
const Value V17368_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_309 = {S28, 0, &V17368_309_57247, &P17368_307};	/*Property = Fullscreen*/
const Value V17368_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_310 = {S29, 0, &V17368_310_57250, &P17368_309};	/*Property = GeoLocation*/
const Value V17368_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_312 = {S30, 0, &V17368_312_57256, &P17368_310};	/*Property = History*/
const Value V17368_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_314 = {S31, 0, &V17368_314_57262, &P17368_312};	/*Property = Html5*/
const Value V17368_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_313 = {S32, 0, &V17368_313_57259, &P17368_314};	/*Property = Html-Media-Capture*/
const Value V17368_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_315 = {S33, 0, &V17368_315_57265, &P17368_313};	/*Property = Iframe*/
const Value V17368_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_316 = {S34, 0, &V17368_316_57268, &P17368_315};	/*Property = IndexedDB*/
const Value V17368_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_317 = {S35, 0, &V17368_317_57271, &P17368_316};	/*Property = Json*/
const Value V17368_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P17368_15 = {S37, 0, &V17368_15_31, &P17368_317};	/*Property = LayoutEngine*/
const Value V17368_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_344 = {S38, 0, &V17368_344_57274, &P17368_15};	/*Property = Masking*/
const Value V17368_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_320 = {S39, 0, &V17368_320_57292, &P17368_344};	/*Property = PostMessage*/
const Value V17368_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_321 = {S40, 0, &V17368_321_57295, &P17368_320};	/*Property = Progress*/
const Value V17368_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_322 = {S41, 0, &V17368_322_57298, &P17368_321};	/*Property = Prompts*/
const Value V17368_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_323 = {S42, 0, &V17368_323_57301, &P17368_322};	/*Property = Selector*/
const Value V17368_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_324 = {S43, 0, &V17368_324_57307, &P17368_323};	/*Property = Svg*/
const Value V17368_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_327 = {S44, 0, &V17368_327_57316, &P17368_324};	/*Property = TouchEvents*/
const Value V17368_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_328 = {S45, 0, &V17368_328_57319, &P17368_327};	/*Property = Track*/
const Value V17368_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_329 = {S46, 0, &V17368_329_57325, &P17368_328};	/*Property = Video*/
const Value V17368_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_330 = {S47, 0, &V17368_330_57328, &P17368_329};	/*Property = Viewport*/
const Value V17368_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_332 = {S48, 0, &V17368_332_57340, &P17368_330};	/*Property = WebWorkers*/
const Value V17368_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17368_333 = {S49, 0, &V17368_333_57343, &P17368_332};	/*Property = Xhr2*/
const Profile P17368 = {"17368", &P17368_333};	/*Profile ID - 17368*/
const Value V3780_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_279 = {S1, 0, &V3780_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3780_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_282 = {S3, 0, &V3780_282_57155, &P3780_279};	/*Property = BlobBuilder*/
const Value V3780_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_350 = {S4, 0, &V3780_350_57446, &P3780_282};	/*Property = Canvas*/
const Value V3780_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_283 = {S5, 0, &V3780_283_57161, &P3780_350};	/*Property = CssBackground*/
const Value V3780_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_284 = {S6, 0, &V3780_284_57164, &P3780_283};	/*Property = CssBorderImage*/
const Value V3780_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_285 = {S7, 0, &V3780_285_57167, &P3780_284};	/*Property = CssCanvas*/
const Value V3780_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_286 = {S8, 0, &V3780_286_57170, &P3780_285};	/*Property = CssColor*/
const Value V3780_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_237 = {S9, 0, &V3780_237_55786, &P3780_286};	/*Property = CssColumn*/
const Value V3780_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_287 = {S10, 0, &V3780_287_57173, &P3780_237};	/*Property = CssFlexbox*/
const Value V3780_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_288 = {S11, 0, &V3780_288_57176, &P3780_287};	/*Property = CssFont*/
const Value V3780_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_289 = {S12, 0, &V3780_289_86899, &P3780_288};	/*Property = CssImages*/
const Value V3780_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_291 = {S13, 0, &V3780_291_57182, &P3780_289};	/*Property = CssMediaQueries*/
const Value V3780_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_292 = {S14, 0, &V3780_292_57185, &P3780_291};	/*Property = CssMinMax*/
const Value V3780_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_293 = {S15, 0, &V3780_293_57188, &P3780_292};	/*Property = CssOverflow*/
const Value V3780_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_294 = {S16, 0, &V3780_294_57191, &P3780_293};	/*Property = CssPosition*/
const Value V3780_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_295 = {S17, 0, &V3780_295_57194, &P3780_294};	/*Property = CssText*/
const Value V3780_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_297 = {S18, 0, &V3780_297_57197, &P3780_295};	/*Property = CssTransforms*/
const Value V3780_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_241 = {S19, 0, &V3780_241_55801, &P3780_297};	/*Property = CssTransitions*/
const Value V3780_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_298 = {S20, 0, &V3780_298_57208, &P3780_241};	/*Property = CssUI*/
const Value V3780_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_300 = {S21, 0, &V3780_300_57217, &P3780_298};	/*Property = DataSet*/
const Value V3780_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_301 = {S22, 0, &V3780_301_57220, &P3780_300};	/*Property = DataUrl*/
const Value V3780_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_302 = {S23, 0, &V3780_302_57223, &P3780_301};	/*Property = DeviceOrientation*/
const Value V3780_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_304 = {S24, 0, &V3780_304_57229, &P3780_302};	/*Property = FileReader*/
const Value V3780_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_305 = {S25, 0, &V3780_305_57232, &P3780_304};	/*Property = FileSaver*/
const Value V3780_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_306 = {S26, 0, &V3780_306_57238, &P3780_305};	/*Property = FileWriter*/
const Value V3780_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_307 = {S27, 0, &V3780_307_57241, &P3780_306};	/*Property = FormData*/
const Value V3780_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_309 = {S28, 0, &V3780_309_57247, &P3780_307};	/*Property = Fullscreen*/
const Value V3780_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_310 = {S29, 0, &V3780_310_57250, &P3780_309};	/*Property = GeoLocation*/
const Value V3780_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_312 = {S30, 0, &V3780_312_57256, &P3780_310};	/*Property = History*/
const Value V3780_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_314 = {S31, 0, &V3780_314_57262, &P3780_312};	/*Property = Html5*/
const Value V3780_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_313 = {S32, 0, &V3780_313_57259, &P3780_314};	/*Property = Html-Media-Capture*/
const Value V3780_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_315 = {S33, 0, &V3780_315_57265, &P3780_313};	/*Property = Iframe*/
const Value V3780_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_316 = {S34, 0, &V3780_316_57268, &P3780_315};	/*Property = IndexedDB*/
const Value V3780_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_317 = {S35, 0, &V3780_317_57271, &P3780_316};	/*Property = Json*/
const Value V3780_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3780_15 = {S37, 0, &V3780_15_28, &P3780_317};	/*Property = LayoutEngine*/
const Value V3780_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_344 = {S38, 0, &V3780_344_57274, &P3780_15};	/*Property = Masking*/
const Value V3780_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_320 = {S39, 0, &V3780_320_57292, &P3780_344};	/*Property = PostMessage*/
const Value V3780_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_321 = {S40, 0, &V3780_321_57295, &P3780_320};	/*Property = Progress*/
const Value V3780_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_322 = {S41, 0, &V3780_322_57298, &P3780_321};	/*Property = Prompts*/
const Value V3780_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_323 = {S42, 0, &V3780_323_57301, &P3780_322};	/*Property = Selector*/
const Value V3780_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_324 = {S43, 0, &V3780_324_57307, &P3780_323};	/*Property = Svg*/
const Value V3780_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_327 = {S44, 0, &V3780_327_57316, &P3780_324};	/*Property = TouchEvents*/
const Value V3780_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_328 = {S45, 0, &V3780_328_57319, &P3780_327};	/*Property = Track*/
const Value V3780_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_329 = {S46, 0, &V3780_329_57325, &P3780_328};	/*Property = Video*/
const Value V3780_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_330 = {S47, 0, &V3780_330_57328, &P3780_329};	/*Property = Viewport*/
const Value V3780_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_332 = {S48, 0, &V3780_332_57340, &P3780_330};	/*Property = WebWorkers*/
const Value V3780_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3780_333 = {S49, 0, &V3780_333_57343, &P3780_332};	/*Property = Xhr2*/
const Profile P3780 = {"3780", &P3780_333};	/*Profile ID - 3780*/
const Profile P6909 = {"6909", NULL};	/*Profile ID - 6909*/
const Value V14253_1_1 = {S2, NULL};	/*Value = True*/
const Property P14253_1 = {S50, 0, &V14253_1_1, NULL};	/*Property = IsMobile*/
const Value V14253_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14253_5 = {S52, 0, &V14253_5_1294, &P14253_1};	/*Property = ScreenPixelsHeight*/
const Value V14253_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14253_4 = {S54, 0, &V14253_4_1296, &P14253_5};	/*Property = ScreenPixelsWidth*/
const Profile P14253 = {"14253", &P14253_4};	/*Profile ID - 14253*/
const Value V3507_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_279 = {S1, 0, &V3507_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3507_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_282 = {S3, 0, &V3507_282_57155, &P3507_279};	/*Property = BlobBuilder*/
const Value V3507_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_350 = {S4, 0, &V3507_350_57446, &P3507_282};	/*Property = Canvas*/
const Value V3507_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_283 = {S5, 0, &V3507_283_57161, &P3507_350};	/*Property = CssBackground*/
const Value V3507_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_284 = {S6, 0, &V3507_284_57164, &P3507_283};	/*Property = CssBorderImage*/
const Value V3507_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_285 = {S7, 0, &V3507_285_57167, &P3507_284};	/*Property = CssCanvas*/
const Value V3507_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_286 = {S8, 0, &V3507_286_57170, &P3507_285};	/*Property = CssColor*/
const Value V3507_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_237 = {S9, 0, &V3507_237_55786, &P3507_286};	/*Property = CssColumn*/
const Value V3507_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_287 = {S10, 0, &V3507_287_57173, &P3507_237};	/*Property = CssFlexbox*/
const Value V3507_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_288 = {S11, 0, &V3507_288_57176, &P3507_287};	/*Property = CssFont*/
const Value V3507_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_289 = {S12, 0, &V3507_289_86899, &P3507_288};	/*Property = CssImages*/
const Value V3507_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_291 = {S13, 0, &V3507_291_57182, &P3507_289};	/*Property = CssMediaQueries*/
const Value V3507_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_292 = {S14, 0, &V3507_292_57185, &P3507_291};	/*Property = CssMinMax*/
const Value V3507_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_293 = {S15, 0, &V3507_293_57188, &P3507_292};	/*Property = CssOverflow*/
const Value V3507_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_294 = {S16, 0, &V3507_294_57191, &P3507_293};	/*Property = CssPosition*/
const Value V3507_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_295 = {S17, 0, &V3507_295_57194, &P3507_294};	/*Property = CssText*/
const Value V3507_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_297 = {S18, 0, &V3507_297_57197, &P3507_295};	/*Property = CssTransforms*/
const Value V3507_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_241 = {S19, 0, &V3507_241_55801, &P3507_297};	/*Property = CssTransitions*/
const Value V3507_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_298 = {S20, 0, &V3507_298_57208, &P3507_241};	/*Property = CssUI*/
const Value V3507_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_300 = {S21, 0, &V3507_300_57217, &P3507_298};	/*Property = DataSet*/
const Value V3507_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_301 = {S22, 0, &V3507_301_57220, &P3507_300};	/*Property = DataUrl*/
const Value V3507_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_302 = {S23, 0, &V3507_302_57223, &P3507_301};	/*Property = DeviceOrientation*/
const Value V3507_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_304 = {S24, 0, &V3507_304_57229, &P3507_302};	/*Property = FileReader*/
const Value V3507_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_305 = {S25, 0, &V3507_305_57232, &P3507_304};	/*Property = FileSaver*/
const Value V3507_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_306 = {S26, 0, &V3507_306_57238, &P3507_305};	/*Property = FileWriter*/
const Value V3507_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_307 = {S27, 0, &V3507_307_57241, &P3507_306};	/*Property = FormData*/
const Value V3507_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_309 = {S28, 0, &V3507_309_57247, &P3507_307};	/*Property = Fullscreen*/
const Value V3507_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_310 = {S29, 0, &V3507_310_57250, &P3507_309};	/*Property = GeoLocation*/
const Value V3507_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_312 = {S30, 0, &V3507_312_57256, &P3507_310};	/*Property = History*/
const Value V3507_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_314 = {S31, 0, &V3507_314_57262, &P3507_312};	/*Property = Html5*/
const Value V3507_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_313 = {S32, 0, &V3507_313_57259, &P3507_314};	/*Property = Html-Media-Capture*/
const Value V3507_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_315 = {S33, 0, &V3507_315_57265, &P3507_313};	/*Property = Iframe*/
const Value V3507_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_316 = {S34, 0, &V3507_316_57268, &P3507_315};	/*Property = IndexedDB*/
const Value V3507_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_317 = {S35, 0, &V3507_317_57271, &P3507_316};	/*Property = Json*/
const Value V3507_15_43264 = {S59, NULL};	/*Value = Mango*/
const Property P3507_15 = {S37, 0, &V3507_15_43264, &P3507_317};	/*Property = LayoutEngine*/
const Value V3507_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_344 = {S38, 0, &V3507_344_57274, &P3507_15};	/*Property = Masking*/
const Value V3507_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_320 = {S39, 0, &V3507_320_57292, &P3507_344};	/*Property = PostMessage*/
const Value V3507_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_321 = {S40, 0, &V3507_321_57295, &P3507_320};	/*Property = Progress*/
const Value V3507_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_322 = {S41, 0, &V3507_322_57298, &P3507_321};	/*Property = Prompts*/
const Value V3507_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_323 = {S42, 0, &V3507_323_57301, &P3507_322};	/*Property = Selector*/
const Value V3507_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_324 = {S43, 0, &V3507_324_57307, &P3507_323};	/*Property = Svg*/
const Value V3507_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_327 = {S44, 0, &V3507_327_57316, &P3507_324};	/*Property = TouchEvents*/
const Value V3507_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_328 = {S45, 0, &V3507_328_57319, &P3507_327};	/*Property = Track*/
const Value V3507_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_329 = {S46, 0, &V3507_329_57325, &P3507_328};	/*Property = Video*/
const Value V3507_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_330 = {S47, 0, &V3507_330_57328, &P3507_329};	/*Property = Viewport*/
const Value V3507_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_332 = {S48, 0, &V3507_332_57340, &P3507_330};	/*Property = WebWorkers*/
const Value V3507_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3507_333 = {S49, 0, &V3507_333_57343, &P3507_332};	/*Property = Xhr2*/
const Profile P3507 = {"3507", &P3507_333};	/*Profile ID - 3507*/
const Profile P7116 = {"7116", NULL};	/*Profile ID - 7116*/
const Value V11639_1_1 = {S2, NULL};	/*Value = True*/
const Property P11639_1 = {S50, 0, &V11639_1_1, NULL};	/*Property = IsMobile*/
const Value V11639_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11639_5 = {S52, 0, &V11639_5_1222, &P11639_1};	/*Property = ScreenPixelsHeight*/
const Value V11639_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P11639_4 = {S54, 0, &V11639_4_1229, &P11639_5};	/*Property = ScreenPixelsWidth*/
const Profile P11639 = {"11639", &P11639_4};	/*Profile ID - 11639*/
const Value V11808_1_1 = {S2, NULL};	/*Value = True*/
const Property P11808_1 = {S50, 0, &V11808_1_1, NULL};	/*Property = IsMobile*/
const Value V11808_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11808_5 = {S52, 0, &V11808_5_1222, &P11808_1};	/*Property = ScreenPixelsHeight*/
const Value V11808_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11808_4 = {S54, 0, &V11808_4_1224, &P11808_5};	/*Property = ScreenPixelsWidth*/
const Profile P11808 = {"11808", &P11808_4};	/*Profile ID - 11808*/
const Value V12044_1_1 = {S2, NULL};	/*Value = True*/
const Property P12044_1 = {S50, 0, &V12044_1_1, NULL};	/*Property = IsMobile*/
const Value V12044_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12044_5 = {S52, 0, &V12044_5_1380, &P12044_1};	/*Property = ScreenPixelsHeight*/
const Value V12044_4_4287 = {S73, NULL};	/*Value = 432*/
const Property P12044_4 = {S54, 0, &V12044_4_4287, &P12044_5};	/*Property = ScreenPixelsWidth*/
const Profile P12044 = {"12044", &P12044_4};	/*Profile ID - 12044*/
const Value V11813_1_1 = {S2, NULL};	/*Value = True*/
const Property P11813_1 = {S50, 0, &V11813_1_1, NULL};	/*Property = IsMobile*/
const Value V11813_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11813_5 = {S52, 0, &V11813_5_1222, &P11813_1};	/*Property = ScreenPixelsHeight*/
const Value V11813_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11813_4 = {S54, 0, &V11813_4_1224, &P11813_5};	/*Property = ScreenPixelsWidth*/
const Profile P11813 = {"11813", &P11813_4};	/*Profile ID - 11813*/
const Value V11889_1_1 = {S2, NULL};	/*Value = True*/
const Property P11889_1 = {S50, 0, &V11889_1_1, NULL};	/*Property = IsMobile*/
const Value V11889_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P11889_5 = {S52, 0, &V11889_5_1280, &P11889_1};	/*Property = ScreenPixelsHeight*/
const Value V11889_4_1368 = {S74, NULL};	/*Value = 208*/
const Property P11889_4 = {S54, 0, &V11889_4_1368, &P11889_5};	/*Property = ScreenPixelsWidth*/
const Profile P11889 = {"11889", &P11889_4};	/*Profile ID - 11889*/
const Profile P5521 = {"5521", NULL};	/*Profile ID - 5521*/
const Value V17457_1_2 = {S0, NULL};	/*Value = False*/
const Property P17457_1 = {S50, 0, &V17457_1_2, NULL};	/*Property = IsMobile*/
const Value V17457_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P17457_5 = {S52, 0, &V17457_5_37, &P17457_1};	/*Property = ScreenPixelsHeight*/
const Value V17457_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P17457_4 = {S54, 0, &V17457_4_36, &P17457_5};	/*Property = ScreenPixelsWidth*/
const Profile P17457 = {"17457", &P17457_4};	/*Profile ID - 17457*/
const Value V11812_1_1 = {S2, NULL};	/*Value = True*/
const Property P11812_1 = {S50, 0, &V11812_1_1, NULL};	/*Property = IsMobile*/
const Value V11812_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11812_5 = {S52, 0, &V11812_5_1222, &P11812_1};	/*Property = ScreenPixelsHeight*/
const Value V11812_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11812_4 = {S54, 0, &V11812_4_1224, &P11812_5};	/*Property = ScreenPixelsWidth*/
const Profile P11812 = {"11812", &P11812_4};	/*Profile ID - 11812*/
const Value V11815_1_1 = {S2, NULL};	/*Value = True*/
const Property P11815_1 = {S50, 0, &V11815_1_1, NULL};	/*Property = IsMobile*/
const Value V11815_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11815_5 = {S52, 0, &V11815_5_1222, &P11815_1};	/*Property = ScreenPixelsHeight*/
const Value V11815_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11815_4 = {S54, 0, &V11815_4_1224, &P11815_5};	/*Property = ScreenPixelsWidth*/
const Profile P11815 = {"11815", &P11815_4};	/*Profile ID - 11815*/
const Profile P5338 = {"5338", NULL};	/*Profile ID - 5338*/
const Value V16338_1_1 = {S2, NULL};	/*Value = True*/
const Property P16338_1 = {S50, 0, &V16338_1_1, NULL};	/*Property = IsMobile*/
const Value V16338_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16338_5 = {S52, 0, &V16338_5_1233, &P16338_1};	/*Property = ScreenPixelsHeight*/
const Value V16338_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16338_4 = {S54, 0, &V16338_4_1235, &P16338_5};	/*Property = ScreenPixelsWidth*/
const Profile P16338 = {"16338", &P16338_4};	/*Profile ID - 16338*/
const Value V11705_1_1 = {S2, NULL};	/*Value = True*/
const Property P11705_1 = {S50, 0, &V11705_1_1, NULL};	/*Property = IsMobile*/
const Value V11705_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11705_5 = {S52, 0, &V11705_5_1233, &P11705_1};	/*Property = ScreenPixelsHeight*/
const Value V11705_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11705_4 = {S54, 0, &V11705_4_1235, &P11705_5};	/*Property = ScreenPixelsWidth*/
const Profile P11705 = {"11705", &P11705_4};	/*Profile ID - 11705*/
const Value V11816_1_1 = {S2, NULL};	/*Value = True*/
const Property P11816_1 = {S50, 0, &V11816_1_1, NULL};	/*Property = IsMobile*/
const Value V11816_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11816_5 = {S52, 0, &V11816_5_1233, &P11816_1};	/*Property = ScreenPixelsHeight*/
const Value V11816_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11816_4 = {S54, 0, &V11816_4_1235, &P11816_5};	/*Property = ScreenPixelsWidth*/
const Profile P11816 = {"11816", &P11816_4};	/*Profile ID - 11816*/
const Value V11817_1_1 = {S2, NULL};	/*Value = True*/
const Property P11817_1 = {S50, 0, &V11817_1_1, NULL};	/*Property = IsMobile*/
const Value V11817_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11817_5 = {S52, 0, &V11817_5_1233, &P11817_1};	/*Property = ScreenPixelsHeight*/
const Value V11817_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11817_4 = {S54, 0, &V11817_4_1235, &P11817_5};	/*Property = ScreenPixelsWidth*/
const Profile P11817 = {"11817", &P11817_4};	/*Profile ID - 11817*/
const Value V11818_1_1 = {S2, NULL};	/*Value = True*/
const Property P11818_1 = {S50, 0, &V11818_1_1, NULL};	/*Property = IsMobile*/
const Value V11818_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11818_5 = {S52, 0, &V11818_5_1293, &P11818_1};	/*Property = ScreenPixelsHeight*/
const Value V11818_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11818_4 = {S54, 0, &V11818_4_1296, &P11818_5};	/*Property = ScreenPixelsWidth*/
const Profile P11818 = {"11818", &P11818_4};	/*Profile ID - 11818*/
const Value V11820_1_1 = {S2, NULL};	/*Value = True*/
const Property P11820_1 = {S50, 0, &V11820_1_1, NULL};	/*Property = IsMobile*/
const Value V11820_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11820_5 = {S52, 0, &V11820_5_1293, &P11820_1};	/*Property = ScreenPixelsHeight*/
const Value V11820_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11820_4 = {S54, 0, &V11820_4_1296, &P11820_5};	/*Property = ScreenPixelsWidth*/
const Profile P11820 = {"11820", &P11820_4};	/*Profile ID - 11820*/
const Value V4722_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_279 = {S1, 0, &V4722_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4722_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_282 = {S3, 0, &V4722_282_57155, &P4722_279};	/*Property = BlobBuilder*/
const Value V4722_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_350 = {S4, 0, &V4722_350_57446, &P4722_282};	/*Property = Canvas*/
const Value V4722_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_283 = {S5, 0, &V4722_283_57161, &P4722_350};	/*Property = CssBackground*/
const Value V4722_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_284 = {S6, 0, &V4722_284_57164, &P4722_283};	/*Property = CssBorderImage*/
const Value V4722_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_285 = {S7, 0, &V4722_285_57167, &P4722_284};	/*Property = CssCanvas*/
const Value V4722_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_286 = {S8, 0, &V4722_286_57170, &P4722_285};	/*Property = CssColor*/
const Value V4722_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_237 = {S9, 0, &V4722_237_55786, &P4722_286};	/*Property = CssColumn*/
const Value V4722_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_287 = {S10, 0, &V4722_287_57173, &P4722_237};	/*Property = CssFlexbox*/
const Value V4722_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_288 = {S11, 0, &V4722_288_57176, &P4722_287};	/*Property = CssFont*/
const Value V4722_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_289 = {S12, 0, &V4722_289_86899, &P4722_288};	/*Property = CssImages*/
const Value V4722_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_291 = {S13, 0, &V4722_291_57182, &P4722_289};	/*Property = CssMediaQueries*/
const Value V4722_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_292 = {S14, 0, &V4722_292_57185, &P4722_291};	/*Property = CssMinMax*/
const Value V4722_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_293 = {S15, 0, &V4722_293_57188, &P4722_292};	/*Property = CssOverflow*/
const Value V4722_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_294 = {S16, 0, &V4722_294_57191, &P4722_293};	/*Property = CssPosition*/
const Value V4722_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_295 = {S17, 0, &V4722_295_57194, &P4722_294};	/*Property = CssText*/
const Value V4722_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_297 = {S18, 0, &V4722_297_57197, &P4722_295};	/*Property = CssTransforms*/
const Value V4722_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_241 = {S19, 0, &V4722_241_55801, &P4722_297};	/*Property = CssTransitions*/
const Value V4722_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_298 = {S20, 0, &V4722_298_57208, &P4722_241};	/*Property = CssUI*/
const Value V4722_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_300 = {S21, 0, &V4722_300_57217, &P4722_298};	/*Property = DataSet*/
const Value V4722_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_301 = {S22, 0, &V4722_301_57220, &P4722_300};	/*Property = DataUrl*/
const Value V4722_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_302 = {S23, 0, &V4722_302_57223, &P4722_301};	/*Property = DeviceOrientation*/
const Value V4722_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_304 = {S24, 0, &V4722_304_57229, &P4722_302};	/*Property = FileReader*/
const Value V4722_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_305 = {S25, 0, &V4722_305_57232, &P4722_304};	/*Property = FileSaver*/
const Value V4722_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_306 = {S26, 0, &V4722_306_57238, &P4722_305};	/*Property = FileWriter*/
const Value V4722_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_307 = {S27, 0, &V4722_307_57241, &P4722_306};	/*Property = FormData*/
const Value V4722_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_309 = {S28, 0, &V4722_309_57247, &P4722_307};	/*Property = Fullscreen*/
const Value V4722_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_310 = {S29, 0, &V4722_310_57250, &P4722_309};	/*Property = GeoLocation*/
const Value V4722_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_312 = {S30, 0, &V4722_312_57256, &P4722_310};	/*Property = History*/
const Value V4722_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_314 = {S31, 0, &V4722_314_57262, &P4722_312};	/*Property = Html5*/
const Value V4722_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_313 = {S32, 0, &V4722_313_57259, &P4722_314};	/*Property = Html-Media-Capture*/
const Value V4722_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_315 = {S33, 0, &V4722_315_57265, &P4722_313};	/*Property = Iframe*/
const Value V4722_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_316 = {S34, 0, &V4722_316_57268, &P4722_315};	/*Property = IndexedDB*/
const Value V4722_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_317 = {S35, 0, &V4722_317_57271, &P4722_316};	/*Property = Json*/
const Value V4722_15_90838 = {S70, NULL};	/*Value = U3*/
const Property P4722_15 = {S37, 0, &V4722_15_90838, &P4722_317};	/*Property = LayoutEngine*/
const Value V4722_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_344 = {S38, 0, &V4722_344_57274, &P4722_15};	/*Property = Masking*/
const Value V4722_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_320 = {S39, 0, &V4722_320_57292, &P4722_344};	/*Property = PostMessage*/
const Value V4722_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_321 = {S40, 0, &V4722_321_57295, &P4722_320};	/*Property = Progress*/
const Value V4722_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_322 = {S41, 0, &V4722_322_57298, &P4722_321};	/*Property = Prompts*/
const Value V4722_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_323 = {S42, 0, &V4722_323_57301, &P4722_322};	/*Property = Selector*/
const Value V4722_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_324 = {S43, 0, &V4722_324_57307, &P4722_323};	/*Property = Svg*/
const Value V4722_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_327 = {S44, 0, &V4722_327_57316, &P4722_324};	/*Property = TouchEvents*/
const Value V4722_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_328 = {S45, 0, &V4722_328_57319, &P4722_327};	/*Property = Track*/
const Value V4722_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_329 = {S46, 0, &V4722_329_57325, &P4722_328};	/*Property = Video*/
const Value V4722_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_330 = {S47, 0, &V4722_330_57328, &P4722_329};	/*Property = Viewport*/
const Value V4722_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_332 = {S48, 0, &V4722_332_57340, &P4722_330};	/*Property = WebWorkers*/
const Value V4722_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4722_333 = {S49, 0, &V4722_333_57343, &P4722_332};	/*Property = Xhr2*/
const Profile P4722 = {"4722", &P4722_333};	/*Profile ID - 4722*/
const Value V14862_1_1 = {S2, NULL};	/*Value = True*/
const Property P14862_1 = {S50, 0, &V14862_1_1, NULL};	/*Property = IsMobile*/
const Value V14862_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14862_5 = {S52, 0, &V14862_5_1222, &P14862_1};	/*Property = ScreenPixelsHeight*/
const Value V14862_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14862_4 = {S54, 0, &V14862_4_1224, &P14862_5};	/*Property = ScreenPixelsWidth*/
const Profile P14862 = {"14862", &P14862_4};	/*Profile ID - 14862*/
const Value V4759_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_279 = {S1, 0, &V4759_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4759_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_282 = {S3, 0, &V4759_282_57155, &P4759_279};	/*Property = BlobBuilder*/
const Value V4759_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_350 = {S4, 0, &V4759_350_57446, &P4759_282};	/*Property = Canvas*/
const Value V4759_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_283 = {S5, 0, &V4759_283_57161, &P4759_350};	/*Property = CssBackground*/
const Value V4759_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_284 = {S6, 0, &V4759_284_57164, &P4759_283};	/*Property = CssBorderImage*/
const Value V4759_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_285 = {S7, 0, &V4759_285_57167, &P4759_284};	/*Property = CssCanvas*/
const Value V4759_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_286 = {S8, 0, &V4759_286_57170, &P4759_285};	/*Property = CssColor*/
const Value V4759_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_237 = {S9, 0, &V4759_237_55786, &P4759_286};	/*Property = CssColumn*/
const Value V4759_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_287 = {S10, 0, &V4759_287_57173, &P4759_237};	/*Property = CssFlexbox*/
const Value V4759_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_288 = {S11, 0, &V4759_288_57176, &P4759_287};	/*Property = CssFont*/
const Value V4759_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_289 = {S12, 0, &V4759_289_86899, &P4759_288};	/*Property = CssImages*/
const Value V4759_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_291 = {S13, 0, &V4759_291_57182, &P4759_289};	/*Property = CssMediaQueries*/
const Value V4759_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_292 = {S14, 0, &V4759_292_57185, &P4759_291};	/*Property = CssMinMax*/
const Value V4759_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_293 = {S15, 0, &V4759_293_57188, &P4759_292};	/*Property = CssOverflow*/
const Value V4759_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_294 = {S16, 0, &V4759_294_57191, &P4759_293};	/*Property = CssPosition*/
const Value V4759_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_295 = {S17, 0, &V4759_295_57194, &P4759_294};	/*Property = CssText*/
const Value V4759_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_297 = {S18, 0, &V4759_297_57197, &P4759_295};	/*Property = CssTransforms*/
const Value V4759_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_241 = {S19, 0, &V4759_241_55801, &P4759_297};	/*Property = CssTransitions*/
const Value V4759_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_298 = {S20, 0, &V4759_298_57208, &P4759_241};	/*Property = CssUI*/
const Value V4759_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_300 = {S21, 0, &V4759_300_57217, &P4759_298};	/*Property = DataSet*/
const Value V4759_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_301 = {S22, 0, &V4759_301_57220, &P4759_300};	/*Property = DataUrl*/
const Value V4759_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_302 = {S23, 0, &V4759_302_57223, &P4759_301};	/*Property = DeviceOrientation*/
const Value V4759_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_304 = {S24, 0, &V4759_304_57229, &P4759_302};	/*Property = FileReader*/
const Value V4759_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_305 = {S25, 0, &V4759_305_57232, &P4759_304};	/*Property = FileSaver*/
const Value V4759_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_306 = {S26, 0, &V4759_306_57238, &P4759_305};	/*Property = FileWriter*/
const Value V4759_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_307 = {S27, 0, &V4759_307_57241, &P4759_306};	/*Property = FormData*/
const Value V4759_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_309 = {S28, 0, &V4759_309_57247, &P4759_307};	/*Property = Fullscreen*/
const Value V4759_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_310 = {S29, 0, &V4759_310_57250, &P4759_309};	/*Property = GeoLocation*/
const Value V4759_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_312 = {S30, 0, &V4759_312_57256, &P4759_310};	/*Property = History*/
const Value V4759_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_314 = {S31, 0, &V4759_314_57262, &P4759_312};	/*Property = Html5*/
const Value V4759_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_313 = {S32, 0, &V4759_313_57259, &P4759_314};	/*Property = Html-Media-Capture*/
const Value V4759_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_315 = {S33, 0, &V4759_315_57265, &P4759_313};	/*Property = Iframe*/
const Value V4759_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_316 = {S34, 0, &V4759_316_57268, &P4759_315};	/*Property = IndexedDB*/
const Value V4759_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_317 = {S35, 0, &V4759_317_57271, &P4759_316};	/*Property = Json*/
const Value V4759_15_90838 = {S70, NULL};	/*Value = U3*/
const Property P4759_15 = {S37, 0, &V4759_15_90838, &P4759_317};	/*Property = LayoutEngine*/
const Value V4759_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_344 = {S38, 0, &V4759_344_57274, &P4759_15};	/*Property = Masking*/
const Value V4759_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_320 = {S39, 0, &V4759_320_57292, &P4759_344};	/*Property = PostMessage*/
const Value V4759_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_321 = {S40, 0, &V4759_321_57295, &P4759_320};	/*Property = Progress*/
const Value V4759_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_322 = {S41, 0, &V4759_322_57298, &P4759_321};	/*Property = Prompts*/
const Value V4759_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_323 = {S42, 0, &V4759_323_57301, &P4759_322};	/*Property = Selector*/
const Value V4759_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_324 = {S43, 0, &V4759_324_57307, &P4759_323};	/*Property = Svg*/
const Value V4759_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_327 = {S44, 0, &V4759_327_57316, &P4759_324};	/*Property = TouchEvents*/
const Value V4759_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_328 = {S45, 0, &V4759_328_57319, &P4759_327};	/*Property = Track*/
const Value V4759_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_329 = {S46, 0, &V4759_329_57325, &P4759_328};	/*Property = Video*/
const Value V4759_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_330 = {S47, 0, &V4759_330_57328, &P4759_329};	/*Property = Viewport*/
const Value V4759_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_332 = {S48, 0, &V4759_332_57340, &P4759_330};	/*Property = WebWorkers*/
const Value V4759_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4759_333 = {S49, 0, &V4759_333_57343, &P4759_332};	/*Property = Xhr2*/
const Profile P4759 = {"4759", &P4759_333};	/*Profile ID - 4759*/
const Value V14807_1_1 = {S2, NULL};	/*Value = True*/
const Property P14807_1 = {S50, 0, &V14807_1_1, NULL};	/*Property = IsMobile*/
const Value V14807_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14807_5 = {S52, 0, &V14807_5_1233, &P14807_1};	/*Property = ScreenPixelsHeight*/
const Value V14807_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14807_4 = {S54, 0, &V14807_4_1235, &P14807_5};	/*Property = ScreenPixelsWidth*/
const Profile P14807 = {"14807", &P14807_4};	/*Profile ID - 14807*/
const Value V11827_1_1 = {S2, NULL};	/*Value = True*/
const Property P11827_1 = {S50, 0, &V11827_1_1, NULL};	/*Property = IsMobile*/
const Value V11827_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11827_5 = {S52, 0, &V11827_5_1222, &P11827_1};	/*Property = ScreenPixelsHeight*/
const Value V11827_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11827_4 = {S54, 0, &V11827_4_1224, &P11827_5};	/*Property = ScreenPixelsWidth*/
const Profile P11827 = {"11827", &P11827_4};	/*Profile ID - 11827*/
const Value V11830_1_1 = {S2, NULL};	/*Value = True*/
const Property P11830_1 = {S50, 0, &V11830_1_1, NULL};	/*Property = IsMobile*/
const Value V11830_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P11830_5 = {S52, 0, &V11830_5_1275, &P11830_1};	/*Property = ScreenPixelsHeight*/
const Value V11830_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P11830_4 = {S54, 0, &V11830_4_1278, &P11830_5};	/*Property = ScreenPixelsWidth*/
const Profile P11830 = {"11830", &P11830_4};	/*Profile ID - 11830*/
const Value V11628_1_1 = {S2, NULL};	/*Value = True*/
const Property P11628_1 = {S50, 0, &V11628_1_1, NULL};	/*Property = IsMobile*/
const Value V11628_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11628_5 = {S52, 0, &V11628_5_1293, &P11628_1};	/*Property = ScreenPixelsHeight*/
const Value V11628_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11628_4 = {S54, 0, &V11628_4_1296, &P11628_5};	/*Property = ScreenPixelsWidth*/
const Profile P11628 = {"11628", &P11628_4};	/*Profile ID - 11628*/
const Value V3772_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_279 = {S1, 0, &V3772_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3772_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_282 = {S3, 0, &V3772_282_57155, &P3772_279};	/*Property = BlobBuilder*/
const Value V3772_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_350 = {S4, 0, &V3772_350_57446, &P3772_282};	/*Property = Canvas*/
const Value V3772_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_283 = {S5, 0, &V3772_283_57161, &P3772_350};	/*Property = CssBackground*/
const Value V3772_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_284 = {S6, 0, &V3772_284_57164, &P3772_283};	/*Property = CssBorderImage*/
const Value V3772_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_285 = {S7, 0, &V3772_285_57167, &P3772_284};	/*Property = CssCanvas*/
const Value V3772_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_286 = {S8, 0, &V3772_286_57170, &P3772_285};	/*Property = CssColor*/
const Value V3772_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_237 = {S9, 0, &V3772_237_55786, &P3772_286};	/*Property = CssColumn*/
const Value V3772_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_287 = {S10, 0, &V3772_287_57173, &P3772_237};	/*Property = CssFlexbox*/
const Value V3772_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_288 = {S11, 0, &V3772_288_57176, &P3772_287};	/*Property = CssFont*/
const Value V3772_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_289 = {S12, 0, &V3772_289_86899, &P3772_288};	/*Property = CssImages*/
const Value V3772_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_291 = {S13, 0, &V3772_291_57182, &P3772_289};	/*Property = CssMediaQueries*/
const Value V3772_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_292 = {S14, 0, &V3772_292_57185, &P3772_291};	/*Property = CssMinMax*/
const Value V3772_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_293 = {S15, 0, &V3772_293_57188, &P3772_292};	/*Property = CssOverflow*/
const Value V3772_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_294 = {S16, 0, &V3772_294_57191, &P3772_293};	/*Property = CssPosition*/
const Value V3772_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_295 = {S17, 0, &V3772_295_57194, &P3772_294};	/*Property = CssText*/
const Value V3772_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_297 = {S18, 0, &V3772_297_57197, &P3772_295};	/*Property = CssTransforms*/
const Value V3772_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_241 = {S19, 0, &V3772_241_55801, &P3772_297};	/*Property = CssTransitions*/
const Value V3772_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_298 = {S20, 0, &V3772_298_57208, &P3772_241};	/*Property = CssUI*/
const Value V3772_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_300 = {S21, 0, &V3772_300_57217, &P3772_298};	/*Property = DataSet*/
const Value V3772_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_301 = {S22, 0, &V3772_301_57220, &P3772_300};	/*Property = DataUrl*/
const Value V3772_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_302 = {S23, 0, &V3772_302_57223, &P3772_301};	/*Property = DeviceOrientation*/
const Value V3772_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_304 = {S24, 0, &V3772_304_57229, &P3772_302};	/*Property = FileReader*/
const Value V3772_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_305 = {S25, 0, &V3772_305_57232, &P3772_304};	/*Property = FileSaver*/
const Value V3772_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_306 = {S26, 0, &V3772_306_57238, &P3772_305};	/*Property = FileWriter*/
const Value V3772_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_307 = {S27, 0, &V3772_307_57241, &P3772_306};	/*Property = FormData*/
const Value V3772_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_309 = {S28, 0, &V3772_309_57247, &P3772_307};	/*Property = Fullscreen*/
const Value V3772_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_310 = {S29, 0, &V3772_310_57250, &P3772_309};	/*Property = GeoLocation*/
const Value V3772_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_312 = {S30, 0, &V3772_312_57256, &P3772_310};	/*Property = History*/
const Value V3772_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_314 = {S31, 0, &V3772_314_57262, &P3772_312};	/*Property = Html5*/
const Value V3772_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_313 = {S32, 0, &V3772_313_57259, &P3772_314};	/*Property = Html-Media-Capture*/
const Value V3772_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_315 = {S33, 0, &V3772_315_57265, &P3772_313};	/*Property = Iframe*/
const Value V3772_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_316 = {S34, 0, &V3772_316_57268, &P3772_315};	/*Property = IndexedDB*/
const Value V3772_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_317 = {S35, 0, &V3772_317_57271, &P3772_316};	/*Property = Json*/
const Value V3772_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P3772_15 = {S37, 0, &V3772_15_31, &P3772_317};	/*Property = LayoutEngine*/
const Value V3772_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_344 = {S38, 0, &V3772_344_57274, &P3772_15};	/*Property = Masking*/
const Value V3772_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_320 = {S39, 0, &V3772_320_57292, &P3772_344};	/*Property = PostMessage*/
const Value V3772_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_321 = {S40, 0, &V3772_321_57295, &P3772_320};	/*Property = Progress*/
const Value V3772_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_322 = {S41, 0, &V3772_322_57298, &P3772_321};	/*Property = Prompts*/
const Value V3772_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_323 = {S42, 0, &V3772_323_57301, &P3772_322};	/*Property = Selector*/
const Value V3772_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_324 = {S43, 0, &V3772_324_57307, &P3772_323};	/*Property = Svg*/
const Value V3772_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_327 = {S44, 0, &V3772_327_57316, &P3772_324};	/*Property = TouchEvents*/
const Value V3772_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_328 = {S45, 0, &V3772_328_57319, &P3772_327};	/*Property = Track*/
const Value V3772_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_329 = {S46, 0, &V3772_329_57325, &P3772_328};	/*Property = Video*/
const Value V3772_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_330 = {S47, 0, &V3772_330_57328, &P3772_329};	/*Property = Viewport*/
const Value V3772_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_332 = {S48, 0, &V3772_332_57340, &P3772_330};	/*Property = WebWorkers*/
const Value V3772_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3772_333 = {S49, 0, &V3772_333_57343, &P3772_332};	/*Property = Xhr2*/
const Profile P3772 = {"3772", &P3772_333};	/*Profile ID - 3772*/
const Value V11833_1_1 = {S2, NULL};	/*Value = True*/
const Property P11833_1 = {S50, 0, &V11833_1_1, NULL};	/*Property = IsMobile*/
const Value V11833_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11833_5 = {S52, 0, &V11833_5_1222, &P11833_1};	/*Property = ScreenPixelsHeight*/
const Value V11833_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11833_4 = {S54, 0, &V11833_4_1224, &P11833_5};	/*Property = ScreenPixelsWidth*/
const Profile P11833 = {"11833", &P11833_4};	/*Profile ID - 11833*/
const Profile P16000 = {"16000", NULL};	/*Profile ID - 16000*/
const Value V11834_1_1 = {S2, NULL};	/*Value = True*/
const Property P11834_1 = {S50, 0, &V11834_1_1, NULL};	/*Property = IsMobile*/
const Value V11834_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11834_5 = {S52, 0, &V11834_5_1222, &P11834_1};	/*Property = ScreenPixelsHeight*/
const Value V11834_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11834_4 = {S54, 0, &V11834_4_1224, &P11834_5};	/*Property = ScreenPixelsWidth*/
const Profile P11834 = {"11834", &P11834_4};	/*Profile ID - 11834*/
const Value V11696_1_1 = {S2, NULL};	/*Value = True*/
const Property P11696_1 = {S50, 0, &V11696_1_1, NULL};	/*Property = IsMobile*/
const Value V11696_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P11696_5 = {S52, 0, &V11696_5_1380, &P11696_1};	/*Property = ScreenPixelsHeight*/
const Value V11696_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P11696_4 = {S54, 0, &V11696_4_1279, &P11696_5};	/*Property = ScreenPixelsWidth*/
const Profile P11696 = {"11696", &P11696_4};	/*Profile ID - 11696*/
const Value V11836_1_1 = {S2, NULL};	/*Value = True*/
const Property P11836_1 = {S50, 0, &V11836_1_1, NULL};	/*Property = IsMobile*/
const Value V11836_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11836_5 = {S52, 0, &V11836_5_1222, &P11836_1};	/*Property = ScreenPixelsHeight*/
const Value V11836_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11836_4 = {S54, 0, &V11836_4_1224, &P11836_5};	/*Property = ScreenPixelsWidth*/
const Profile P11836 = {"11836", &P11836_4};	/*Profile ID - 11836*/
const Value V11838_1_1 = {S2, NULL};	/*Value = True*/
const Property P11838_1 = {S50, 0, &V11838_1_1, NULL};	/*Property = IsMobile*/
const Value V11838_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11838_5 = {S52, 0, &V11838_5_1222, &P11838_1};	/*Property = ScreenPixelsHeight*/
const Value V11838_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11838_4 = {S54, 0, &V11838_4_1224, &P11838_5};	/*Property = ScreenPixelsWidth*/
const Profile P11838 = {"11838", &P11838_4};	/*Profile ID - 11838*/
const Value V20417_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_279 = {S1, 0, &V20417_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20417_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_282 = {S3, 0, &V20417_282_57155, &P20417_279};	/*Property = BlobBuilder*/
const Value V20417_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_350 = {S4, 0, &V20417_350_57446, &P20417_282};	/*Property = Canvas*/
const Value V20417_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_283 = {S5, 0, &V20417_283_57161, &P20417_350};	/*Property = CssBackground*/
const Value V20417_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_284 = {S6, 0, &V20417_284_57164, &P20417_283};	/*Property = CssBorderImage*/
const Value V20417_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_285 = {S7, 0, &V20417_285_57167, &P20417_284};	/*Property = CssCanvas*/
const Value V20417_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_286 = {S8, 0, &V20417_286_57170, &P20417_285};	/*Property = CssColor*/
const Value V20417_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_237 = {S9, 0, &V20417_237_55786, &P20417_286};	/*Property = CssColumn*/
const Value V20417_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_287 = {S10, 0, &V20417_287_57173, &P20417_237};	/*Property = CssFlexbox*/
const Value V20417_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_288 = {S11, 0, &V20417_288_57176, &P20417_287};	/*Property = CssFont*/
const Value V20417_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_289 = {S12, 0, &V20417_289_86899, &P20417_288};	/*Property = CssImages*/
const Value V20417_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_291 = {S13, 0, &V20417_291_57182, &P20417_289};	/*Property = CssMediaQueries*/
const Value V20417_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_292 = {S14, 0, &V20417_292_57185, &P20417_291};	/*Property = CssMinMax*/
const Value V20417_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_293 = {S15, 0, &V20417_293_57188, &P20417_292};	/*Property = CssOverflow*/
const Value V20417_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_294 = {S16, 0, &V20417_294_57191, &P20417_293};	/*Property = CssPosition*/
const Value V20417_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_295 = {S17, 0, &V20417_295_57194, &P20417_294};	/*Property = CssText*/
const Value V20417_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_297 = {S18, 0, &V20417_297_57197, &P20417_295};	/*Property = CssTransforms*/
const Value V20417_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_241 = {S19, 0, &V20417_241_55801, &P20417_297};	/*Property = CssTransitions*/
const Value V20417_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_298 = {S20, 0, &V20417_298_57208, &P20417_241};	/*Property = CssUI*/
const Value V20417_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_300 = {S21, 0, &V20417_300_57217, &P20417_298};	/*Property = DataSet*/
const Value V20417_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_301 = {S22, 0, &V20417_301_57220, &P20417_300};	/*Property = DataUrl*/
const Value V20417_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_302 = {S23, 0, &V20417_302_57223, &P20417_301};	/*Property = DeviceOrientation*/
const Value V20417_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_304 = {S24, 0, &V20417_304_57229, &P20417_302};	/*Property = FileReader*/
const Value V20417_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_305 = {S25, 0, &V20417_305_57232, &P20417_304};	/*Property = FileSaver*/
const Value V20417_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_306 = {S26, 0, &V20417_306_57238, &P20417_305};	/*Property = FileWriter*/
const Value V20417_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_307 = {S27, 0, &V20417_307_57241, &P20417_306};	/*Property = FormData*/
const Value V20417_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_309 = {S28, 0, &V20417_309_57247, &P20417_307};	/*Property = Fullscreen*/
const Value V20417_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_310 = {S29, 0, &V20417_310_57250, &P20417_309};	/*Property = GeoLocation*/
const Value V20417_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_312 = {S30, 0, &V20417_312_57256, &P20417_310};	/*Property = History*/
const Value V20417_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_314 = {S31, 0, &V20417_314_57262, &P20417_312};	/*Property = Html5*/
const Value V20417_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_313 = {S32, 0, &V20417_313_57259, &P20417_314};	/*Property = Html-Media-Capture*/
const Value V20417_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_315 = {S33, 0, &V20417_315_57265, &P20417_313};	/*Property = Iframe*/
const Value V20417_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_316 = {S34, 0, &V20417_316_57268, &P20417_315};	/*Property = IndexedDB*/
const Value V20417_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_317 = {S35, 0, &V20417_317_57271, &P20417_316};	/*Property = Json*/
const Value V20417_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P20417_15 = {S37, 0, &V20417_15_31, &P20417_317};	/*Property = LayoutEngine*/
const Value V20417_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_344 = {S38, 0, &V20417_344_57274, &P20417_15};	/*Property = Masking*/
const Value V20417_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_320 = {S39, 0, &V20417_320_57292, &P20417_344};	/*Property = PostMessage*/
const Value V20417_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_321 = {S40, 0, &V20417_321_57295, &P20417_320};	/*Property = Progress*/
const Value V20417_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_322 = {S41, 0, &V20417_322_57298, &P20417_321};	/*Property = Prompts*/
const Value V20417_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_323 = {S42, 0, &V20417_323_57301, &P20417_322};	/*Property = Selector*/
const Value V20417_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_324 = {S43, 0, &V20417_324_57307, &P20417_323};	/*Property = Svg*/
const Value V20417_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_327 = {S44, 0, &V20417_327_57316, &P20417_324};	/*Property = TouchEvents*/
const Value V20417_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_328 = {S45, 0, &V20417_328_57319, &P20417_327};	/*Property = Track*/
const Value V20417_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_329 = {S46, 0, &V20417_329_57325, &P20417_328};	/*Property = Video*/
const Value V20417_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_330 = {S47, 0, &V20417_330_57328, &P20417_329};	/*Property = Viewport*/
const Value V20417_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_332 = {S48, 0, &V20417_332_57340, &P20417_330};	/*Property = WebWorkers*/
const Value V20417_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20417_333 = {S49, 0, &V20417_333_57343, &P20417_332};	/*Property = Xhr2*/
const Profile P20417 = {"20417", &P20417_333};	/*Profile ID - 20417*/
const Value V4924_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_279 = {S1, 0, &V4924_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4924_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_282 = {S3, 0, &V4924_282_57155, &P4924_279};	/*Property = BlobBuilder*/
const Value V4924_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_350 = {S4, 0, &V4924_350_57446, &P4924_282};	/*Property = Canvas*/
const Value V4924_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_283 = {S5, 0, &V4924_283_57161, &P4924_350};	/*Property = CssBackground*/
const Value V4924_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_284 = {S6, 0, &V4924_284_57164, &P4924_283};	/*Property = CssBorderImage*/
const Value V4924_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_285 = {S7, 0, &V4924_285_57167, &P4924_284};	/*Property = CssCanvas*/
const Value V4924_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_286 = {S8, 0, &V4924_286_57170, &P4924_285};	/*Property = CssColor*/
const Value V4924_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_237 = {S9, 0, &V4924_237_55786, &P4924_286};	/*Property = CssColumn*/
const Value V4924_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_287 = {S10, 0, &V4924_287_57173, &P4924_237};	/*Property = CssFlexbox*/
const Value V4924_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_288 = {S11, 0, &V4924_288_57176, &P4924_287};	/*Property = CssFont*/
const Value V4924_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_289 = {S12, 0, &V4924_289_86899, &P4924_288};	/*Property = CssImages*/
const Value V4924_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_291 = {S13, 0, &V4924_291_57182, &P4924_289};	/*Property = CssMediaQueries*/
const Value V4924_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_292 = {S14, 0, &V4924_292_57185, &P4924_291};	/*Property = CssMinMax*/
const Value V4924_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_293 = {S15, 0, &V4924_293_57188, &P4924_292};	/*Property = CssOverflow*/
const Value V4924_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_294 = {S16, 0, &V4924_294_57191, &P4924_293};	/*Property = CssPosition*/
const Value V4924_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_295 = {S17, 0, &V4924_295_57194, &P4924_294};	/*Property = CssText*/
const Value V4924_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_297 = {S18, 0, &V4924_297_57197, &P4924_295};	/*Property = CssTransforms*/
const Value V4924_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_241 = {S19, 0, &V4924_241_55801, &P4924_297};	/*Property = CssTransitions*/
const Value V4924_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_298 = {S20, 0, &V4924_298_57208, &P4924_241};	/*Property = CssUI*/
const Value V4924_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_300 = {S21, 0, &V4924_300_57217, &P4924_298};	/*Property = DataSet*/
const Value V4924_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_301 = {S22, 0, &V4924_301_57220, &P4924_300};	/*Property = DataUrl*/
const Value V4924_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_302 = {S23, 0, &V4924_302_57223, &P4924_301};	/*Property = DeviceOrientation*/
const Value V4924_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_304 = {S24, 0, &V4924_304_57229, &P4924_302};	/*Property = FileReader*/
const Value V4924_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_305 = {S25, 0, &V4924_305_57232, &P4924_304};	/*Property = FileSaver*/
const Value V4924_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_306 = {S26, 0, &V4924_306_57238, &P4924_305};	/*Property = FileWriter*/
const Value V4924_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_307 = {S27, 0, &V4924_307_57241, &P4924_306};	/*Property = FormData*/
const Value V4924_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_309 = {S28, 0, &V4924_309_57247, &P4924_307};	/*Property = Fullscreen*/
const Value V4924_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_310 = {S29, 0, &V4924_310_57250, &P4924_309};	/*Property = GeoLocation*/
const Value V4924_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_312 = {S30, 0, &V4924_312_57256, &P4924_310};	/*Property = History*/
const Value V4924_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_314 = {S31, 0, &V4924_314_57262, &P4924_312};	/*Property = Html5*/
const Value V4924_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_313 = {S32, 0, &V4924_313_57259, &P4924_314};	/*Property = Html-Media-Capture*/
const Value V4924_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_315 = {S33, 0, &V4924_315_57265, &P4924_313};	/*Property = Iframe*/
const Value V4924_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_316 = {S34, 0, &V4924_316_57268, &P4924_315};	/*Property = IndexedDB*/
const Value V4924_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_317 = {S35, 0, &V4924_317_57271, &P4924_316};	/*Property = Json*/
const Value V4924_15_43263 = {S65, NULL};	/*Value = Netfront*/
const Property P4924_15 = {S37, 0, &V4924_15_43263, &P4924_317};	/*Property = LayoutEngine*/
const Value V4924_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_344 = {S38, 0, &V4924_344_57274, &P4924_15};	/*Property = Masking*/
const Value V4924_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_320 = {S39, 0, &V4924_320_57292, &P4924_344};	/*Property = PostMessage*/
const Value V4924_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_321 = {S40, 0, &V4924_321_57295, &P4924_320};	/*Property = Progress*/
const Value V4924_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_322 = {S41, 0, &V4924_322_57298, &P4924_321};	/*Property = Prompts*/
const Value V4924_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_323 = {S42, 0, &V4924_323_57301, &P4924_322};	/*Property = Selector*/
const Value V4924_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_324 = {S43, 0, &V4924_324_57307, &P4924_323};	/*Property = Svg*/
const Value V4924_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_327 = {S44, 0, &V4924_327_57316, &P4924_324};	/*Property = TouchEvents*/
const Value V4924_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_328 = {S45, 0, &V4924_328_57319, &P4924_327};	/*Property = Track*/
const Value V4924_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_329 = {S46, 0, &V4924_329_57325, &P4924_328};	/*Property = Video*/
const Value V4924_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_330 = {S47, 0, &V4924_330_57328, &P4924_329};	/*Property = Viewport*/
const Value V4924_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_332 = {S48, 0, &V4924_332_57340, &P4924_330};	/*Property = WebWorkers*/
const Value V4924_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4924_333 = {S49, 0, &V4924_333_57343, &P4924_332};	/*Property = Xhr2*/
const Profile P4924 = {"4924", &P4924_333};	/*Profile ID - 4924*/
const Value V11839_1_1 = {S2, NULL};	/*Value = True*/
const Property P11839_1 = {S50, 0, &V11839_1_1, NULL};	/*Property = IsMobile*/
const Value V11839_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11839_5 = {S52, 0, &V11839_5_1233, &P11839_1};	/*Property = ScreenPixelsHeight*/
const Value V11839_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11839_4 = {S54, 0, &V11839_4_1235, &P11839_5};	/*Property = ScreenPixelsWidth*/
const Profile P11839 = {"11839", &P11839_4};	/*Profile ID - 11839*/
const Profile P19954 = {"19954", NULL};	/*Profile ID - 19954*/
const Value V11840_1_1 = {S2, NULL};	/*Value = True*/
const Property P11840_1 = {S50, 0, &V11840_1_1, NULL};	/*Property = IsMobile*/
const Value V11840_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11840_5 = {S52, 0, &V11840_5_1222, &P11840_1};	/*Property = ScreenPixelsHeight*/
const Value V11840_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11840_4 = {S54, 0, &V11840_4_1224, &P11840_5};	/*Property = ScreenPixelsWidth*/
const Profile P11840 = {"11840", &P11840_4};	/*Profile ID - 11840*/
const Value V3601_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_279 = {S1, 0, &V3601_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3601_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_282 = {S3, 0, &V3601_282_57155, &P3601_279};	/*Property = BlobBuilder*/
const Value V3601_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_350 = {S4, 0, &V3601_350_57446, &P3601_282};	/*Property = Canvas*/
const Value V3601_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_283 = {S5, 0, &V3601_283_57161, &P3601_350};	/*Property = CssBackground*/
const Value V3601_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_284 = {S6, 0, &V3601_284_57164, &P3601_283};	/*Property = CssBorderImage*/
const Value V3601_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_285 = {S7, 0, &V3601_285_57167, &P3601_284};	/*Property = CssCanvas*/
const Value V3601_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_286 = {S8, 0, &V3601_286_57170, &P3601_285};	/*Property = CssColor*/
const Value V3601_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_237 = {S9, 0, &V3601_237_55786, &P3601_286};	/*Property = CssColumn*/
const Value V3601_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_287 = {S10, 0, &V3601_287_57173, &P3601_237};	/*Property = CssFlexbox*/
const Value V3601_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_288 = {S11, 0, &V3601_288_57176, &P3601_287};	/*Property = CssFont*/
const Value V3601_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_289 = {S12, 0, &V3601_289_86899, &P3601_288};	/*Property = CssImages*/
const Value V3601_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_291 = {S13, 0, &V3601_291_57182, &P3601_289};	/*Property = CssMediaQueries*/
const Value V3601_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_292 = {S14, 0, &V3601_292_57185, &P3601_291};	/*Property = CssMinMax*/
const Value V3601_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_293 = {S15, 0, &V3601_293_57188, &P3601_292};	/*Property = CssOverflow*/
const Value V3601_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_294 = {S16, 0, &V3601_294_57191, &P3601_293};	/*Property = CssPosition*/
const Value V3601_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_295 = {S17, 0, &V3601_295_57194, &P3601_294};	/*Property = CssText*/
const Value V3601_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_297 = {S18, 0, &V3601_297_57197, &P3601_295};	/*Property = CssTransforms*/
const Value V3601_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_241 = {S19, 0, &V3601_241_55801, &P3601_297};	/*Property = CssTransitions*/
const Value V3601_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_298 = {S20, 0, &V3601_298_57208, &P3601_241};	/*Property = CssUI*/
const Value V3601_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_300 = {S21, 0, &V3601_300_57217, &P3601_298};	/*Property = DataSet*/
const Value V3601_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_301 = {S22, 0, &V3601_301_57220, &P3601_300};	/*Property = DataUrl*/
const Value V3601_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_302 = {S23, 0, &V3601_302_57223, &P3601_301};	/*Property = DeviceOrientation*/
const Value V3601_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_304 = {S24, 0, &V3601_304_57229, &P3601_302};	/*Property = FileReader*/
const Value V3601_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_305 = {S25, 0, &V3601_305_57232, &P3601_304};	/*Property = FileSaver*/
const Value V3601_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_306 = {S26, 0, &V3601_306_57238, &P3601_305};	/*Property = FileWriter*/
const Value V3601_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_307 = {S27, 0, &V3601_307_57241, &P3601_306};	/*Property = FormData*/
const Value V3601_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_309 = {S28, 0, &V3601_309_57247, &P3601_307};	/*Property = Fullscreen*/
const Value V3601_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_310 = {S29, 0, &V3601_310_57250, &P3601_309};	/*Property = GeoLocation*/
const Value V3601_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_312 = {S30, 0, &V3601_312_57256, &P3601_310};	/*Property = History*/
const Value V3601_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_314 = {S31, 0, &V3601_314_57262, &P3601_312};	/*Property = Html5*/
const Value V3601_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_313 = {S32, 0, &V3601_313_57259, &P3601_314};	/*Property = Html-Media-Capture*/
const Value V3601_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_315 = {S33, 0, &V3601_315_57265, &P3601_313};	/*Property = Iframe*/
const Value V3601_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_316 = {S34, 0, &V3601_316_57268, &P3601_315};	/*Property = IndexedDB*/
const Value V3601_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_317 = {S35, 0, &V3601_317_57271, &P3601_316};	/*Property = Json*/
const Value V3601_15_43263 = {S65, NULL};	/*Value = Netfront*/
const Property P3601_15 = {S37, 0, &V3601_15_43263, &P3601_317};	/*Property = LayoutEngine*/
const Value V3601_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_344 = {S38, 0, &V3601_344_57274, &P3601_15};	/*Property = Masking*/
const Value V3601_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_320 = {S39, 0, &V3601_320_57292, &P3601_344};	/*Property = PostMessage*/
const Value V3601_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_321 = {S40, 0, &V3601_321_57295, &P3601_320};	/*Property = Progress*/
const Value V3601_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_322 = {S41, 0, &V3601_322_57298, &P3601_321};	/*Property = Prompts*/
const Value V3601_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_323 = {S42, 0, &V3601_323_57301, &P3601_322};	/*Property = Selector*/
const Value V3601_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_324 = {S43, 0, &V3601_324_57307, &P3601_323};	/*Property = Svg*/
const Value V3601_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_327 = {S44, 0, &V3601_327_57316, &P3601_324};	/*Property = TouchEvents*/
const Value V3601_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_328 = {S45, 0, &V3601_328_57319, &P3601_327};	/*Property = Track*/
const Value V3601_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_329 = {S46, 0, &V3601_329_57325, &P3601_328};	/*Property = Video*/
const Value V3601_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_330 = {S47, 0, &V3601_330_57328, &P3601_329};	/*Property = Viewport*/
const Value V3601_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_332 = {S48, 0, &V3601_332_57340, &P3601_330};	/*Property = WebWorkers*/
const Value V3601_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3601_333 = {S49, 0, &V3601_333_57343, &P3601_332};	/*Property = Xhr2*/
const Profile P3601 = {"3601", &P3601_333};	/*Profile ID - 3601*/
const Value V13220_1_1 = {S2, NULL};	/*Value = True*/
const Property P13220_1 = {S50, 0, &V13220_1_1, NULL};	/*Property = IsMobile*/
const Value V13220_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13220_5 = {S52, 0, &V13220_5_1222, &P13220_1};	/*Property = ScreenPixelsHeight*/
const Value V13220_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13220_4 = {S54, 0, &V13220_4_1224, &P13220_5};	/*Property = ScreenPixelsWidth*/
const Profile P13220 = {"13220", &P13220_4};	/*Profile ID - 13220*/
const Value V14708_1_1 = {S2, NULL};	/*Value = True*/
const Property P14708_1 = {S50, 0, &V14708_1_1, NULL};	/*Property = IsMobile*/
const Value V14708_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14708_5 = {S52, 0, &V14708_5_1222, &P14708_1};	/*Property = ScreenPixelsHeight*/
const Value V14708_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14708_4 = {S54, 0, &V14708_4_1224, &P14708_5};	/*Property = ScreenPixelsWidth*/
const Profile P14708 = {"14708", &P14708_4};	/*Profile ID - 14708*/
const Value V14437_1_1 = {S2, NULL};	/*Value = True*/
const Property P14437_1 = {S50, 0, &V14437_1_1, NULL};	/*Property = IsMobile*/
const Value V14437_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14437_5 = {S52, 0, &V14437_5_1293, &P14437_1};	/*Property = ScreenPixelsHeight*/
const Value V14437_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14437_4 = {S54, 0, &V14437_4_1296, &P14437_5};	/*Property = ScreenPixelsWidth*/
const Profile P14437 = {"14437", &P14437_4};	/*Profile ID - 14437*/
const Value V11841_1_1 = {S2, NULL};	/*Value = True*/
const Property P11841_1 = {S50, 0, &V11841_1_1, NULL};	/*Property = IsMobile*/
const Value V11841_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11841_5 = {S52, 0, &V11841_5_1222, &P11841_1};	/*Property = ScreenPixelsHeight*/
const Value V11841_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11841_4 = {S54, 0, &V11841_4_1224, &P11841_5};	/*Property = ScreenPixelsWidth*/
const Profile P11841 = {"11841", &P11841_4};	/*Profile ID - 11841*/
const Value V11849_1_1 = {S2, NULL};	/*Value = True*/
const Property P11849_1 = {S50, 0, &V11849_1_1, NULL};	/*Property = IsMobile*/
const Value V11849_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11849_5 = {S52, 0, &V11849_5_1222, &P11849_1};	/*Property = ScreenPixelsHeight*/
const Value V11849_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11849_4 = {S54, 0, &V11849_4_1224, &P11849_5};	/*Property = ScreenPixelsWidth*/
const Profile P11849 = {"11849", &P11849_4};	/*Profile ID - 11849*/
const Value V11843_1_1 = {S2, NULL};	/*Value = True*/
const Property P11843_1 = {S50, 0, &V11843_1_1, NULL};	/*Property = IsMobile*/
const Value V11843_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11843_5 = {S52, 0, &V11843_5_1222, &P11843_1};	/*Property = ScreenPixelsHeight*/
const Value V11843_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11843_4 = {S54, 0, &V11843_4_1224, &P11843_5};	/*Property = ScreenPixelsWidth*/
const Profile P11843 = {"11843", &P11843_4};	/*Profile ID - 11843*/
const Value V13902_1_1 = {S2, NULL};	/*Value = True*/
const Property P13902_1 = {S50, 0, &V13902_1_1, NULL};	/*Property = IsMobile*/
const Value V13902_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13902_5 = {S52, 0, &V13902_5_1293, &P13902_1};	/*Property = ScreenPixelsHeight*/
const Value V13902_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13902_4 = {S54, 0, &V13902_4_1296, &P13902_5};	/*Property = ScreenPixelsWidth*/
const Profile P13902 = {"13902", &P13902_4};	/*Profile ID - 13902*/
const Value V14909_1_1 = {S2, NULL};	/*Value = True*/
const Property P14909_1 = {S50, 0, &V14909_1_1, NULL};	/*Property = IsMobile*/
const Value V14909_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14909_5 = {S52, 0, &V14909_5_1222, &P14909_1};	/*Property = ScreenPixelsHeight*/
const Value V14909_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14909_4 = {S54, 0, &V14909_4_1224, &P14909_5};	/*Property = ScreenPixelsWidth*/
const Profile P14909 = {"14909", &P14909_4};	/*Profile ID - 14909*/
const Value V14790_1_1 = {S2, NULL};	/*Value = True*/
const Property P14790_1 = {S50, 0, &V14790_1_1, NULL};	/*Property = IsMobile*/
const Value V14790_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14790_5 = {S52, 0, &V14790_5_1222, &P14790_1};	/*Property = ScreenPixelsHeight*/
const Value V14790_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14790_4 = {S54, 0, &V14790_4_1224, &P14790_5};	/*Property = ScreenPixelsWidth*/
const Profile P14790 = {"14790", &P14790_4};	/*Profile ID - 14790*/
const Value V11929_1_1 = {S2, NULL};	/*Value = True*/
const Property P11929_1 = {S50, 0, &V11929_1_1, NULL};	/*Property = IsMobile*/
const Value V11929_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P11929_5 = {S52, 0, &V11929_5_1275, &P11929_1};	/*Property = ScreenPixelsHeight*/
const Value V11929_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P11929_4 = {S54, 0, &V11929_4_1278, &P11929_5};	/*Property = ScreenPixelsWidth*/
const Profile P11929 = {"11929", &P11929_4};	/*Profile ID - 11929*/
const Value V11729_1_1 = {S2, NULL};	/*Value = True*/
const Property P11729_1 = {S50, 0, &V11729_1_1, NULL};	/*Property = IsMobile*/
const Value V11729_5_31826 = {S75, NULL};	/*Value = 260*/
const Property P11729_5 = {S52, 0, &V11729_5_31826, &P11729_1};	/*Property = ScreenPixelsHeight*/
const Value V11729_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11729_4 = {S54, 0, &V11729_4_1224, &P11729_5};	/*Property = ScreenPixelsWidth*/
const Profile P11729 = {"11729", &P11729_4};	/*Profile ID - 11729*/
const Value V11851_1_1 = {S2, NULL};	/*Value = True*/
const Property P11851_1 = {S50, 0, &V11851_1_1, NULL};	/*Property = IsMobile*/
const Value V11851_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11851_5 = {S52, 0, &V11851_5_1222, &P11851_1};	/*Property = ScreenPixelsHeight*/
const Value V11851_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11851_4 = {S54, 0, &V11851_4_1224, &P11851_5};	/*Property = ScreenPixelsWidth*/
const Profile P11851 = {"11851", &P11851_4};	/*Profile ID - 11851*/
const Value V22271_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_279 = {S1, 0, &V22271_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22271_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_282 = {S3, 0, &V22271_282_57155, &P22271_279};	/*Property = BlobBuilder*/
const Value V22271_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_350 = {S4, 0, &V22271_350_57446, &P22271_282};	/*Property = Canvas*/
const Value V22271_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_283 = {S5, 0, &V22271_283_57161, &P22271_350};	/*Property = CssBackground*/
const Value V22271_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_284 = {S6, 0, &V22271_284_57164, &P22271_283};	/*Property = CssBorderImage*/
const Value V22271_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_285 = {S7, 0, &V22271_285_57167, &P22271_284};	/*Property = CssCanvas*/
const Value V22271_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_286 = {S8, 0, &V22271_286_57170, &P22271_285};	/*Property = CssColor*/
const Value V22271_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_237 = {S9, 0, &V22271_237_55786, &P22271_286};	/*Property = CssColumn*/
const Value V22271_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_287 = {S10, 0, &V22271_287_57173, &P22271_237};	/*Property = CssFlexbox*/
const Value V22271_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_288 = {S11, 0, &V22271_288_57176, &P22271_287};	/*Property = CssFont*/
const Value V22271_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_289 = {S12, 0, &V22271_289_86899, &P22271_288};	/*Property = CssImages*/
const Value V22271_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_291 = {S13, 0, &V22271_291_57182, &P22271_289};	/*Property = CssMediaQueries*/
const Value V22271_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_292 = {S14, 0, &V22271_292_57185, &P22271_291};	/*Property = CssMinMax*/
const Value V22271_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_293 = {S15, 0, &V22271_293_57188, &P22271_292};	/*Property = CssOverflow*/
const Value V22271_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_294 = {S16, 0, &V22271_294_57191, &P22271_293};	/*Property = CssPosition*/
const Value V22271_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_295 = {S17, 0, &V22271_295_57194, &P22271_294};	/*Property = CssText*/
const Value V22271_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_297 = {S18, 0, &V22271_297_57197, &P22271_295};	/*Property = CssTransforms*/
const Value V22271_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_241 = {S19, 0, &V22271_241_55801, &P22271_297};	/*Property = CssTransitions*/
const Value V22271_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_298 = {S20, 0, &V22271_298_57208, &P22271_241};	/*Property = CssUI*/
const Value V22271_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_300 = {S21, 0, &V22271_300_57217, &P22271_298};	/*Property = DataSet*/
const Value V22271_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_301 = {S22, 0, &V22271_301_57220, &P22271_300};	/*Property = DataUrl*/
const Value V22271_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_302 = {S23, 0, &V22271_302_57223, &P22271_301};	/*Property = DeviceOrientation*/
const Value V22271_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_304 = {S24, 0, &V22271_304_57229, &P22271_302};	/*Property = FileReader*/
const Value V22271_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_305 = {S25, 0, &V22271_305_57232, &P22271_304};	/*Property = FileSaver*/
const Value V22271_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_306 = {S26, 0, &V22271_306_57238, &P22271_305};	/*Property = FileWriter*/
const Value V22271_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_307 = {S27, 0, &V22271_307_57241, &P22271_306};	/*Property = FormData*/
const Value V22271_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_309 = {S28, 0, &V22271_309_57247, &P22271_307};	/*Property = Fullscreen*/
const Value V22271_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_310 = {S29, 0, &V22271_310_57250, &P22271_309};	/*Property = GeoLocation*/
const Value V22271_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_312 = {S30, 0, &V22271_312_57256, &P22271_310};	/*Property = History*/
const Value V22271_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_314 = {S31, 0, &V22271_314_57262, &P22271_312};	/*Property = Html5*/
const Value V22271_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_313 = {S32, 0, &V22271_313_57259, &P22271_314};	/*Property = Html-Media-Capture*/
const Value V22271_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_315 = {S33, 0, &V22271_315_57265, &P22271_313};	/*Property = Iframe*/
const Value V22271_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_316 = {S34, 0, &V22271_316_57268, &P22271_315};	/*Property = IndexedDB*/
const Value V22271_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_317 = {S35, 0, &V22271_317_57271, &P22271_316};	/*Property = Json*/
const Value V22271_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P22271_15 = {S37, 0, &V22271_15_30, &P22271_317};	/*Property = LayoutEngine*/
const Value V22271_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_344 = {S38, 0, &V22271_344_57274, &P22271_15};	/*Property = Masking*/
const Value V22271_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_320 = {S39, 0, &V22271_320_57292, &P22271_344};	/*Property = PostMessage*/
const Value V22271_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_321 = {S40, 0, &V22271_321_57295, &P22271_320};	/*Property = Progress*/
const Value V22271_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_322 = {S41, 0, &V22271_322_57298, &P22271_321};	/*Property = Prompts*/
const Value V22271_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_323 = {S42, 0, &V22271_323_57301, &P22271_322};	/*Property = Selector*/
const Value V22271_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_324 = {S43, 0, &V22271_324_57307, &P22271_323};	/*Property = Svg*/
const Value V22271_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_327 = {S44, 0, &V22271_327_57316, &P22271_324};	/*Property = TouchEvents*/
const Value V22271_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_328 = {S45, 0, &V22271_328_57319, &P22271_327};	/*Property = Track*/
const Value V22271_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_329 = {S46, 0, &V22271_329_57325, &P22271_328};	/*Property = Video*/
const Value V22271_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_330 = {S47, 0, &V22271_330_57328, &P22271_329};	/*Property = Viewport*/
const Value V22271_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_332 = {S48, 0, &V22271_332_57340, &P22271_330};	/*Property = WebWorkers*/
const Value V22271_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22271_333 = {S49, 0, &V22271_333_57343, &P22271_332};	/*Property = Xhr2*/
const Profile P22271 = {"22271", &P22271_333};	/*Profile ID - 22271*/
const Value V17489_1_1 = {S2, NULL};	/*Value = True*/
const Property P17489_1 = {S50, 0, &V17489_1_1, NULL};	/*Property = IsMobile*/
const Value V17489_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P17489_5 = {S52, 0, &V17489_5_1380, &P17489_1};	/*Property = ScreenPixelsHeight*/
const Value V17489_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17489_4 = {S54, 0, &V17489_4_1279, &P17489_5};	/*Property = ScreenPixelsWidth*/
const Profile P17489 = {"17489", &P17489_4};	/*Profile ID - 17489*/
const Value V11854_1_1 = {S2, NULL};	/*Value = True*/
const Property P11854_1 = {S50, 0, &V11854_1_1, NULL};	/*Property = IsMobile*/
const Value V11854_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P11854_5 = {S52, 0, &V11854_5_1380, &P11854_1};	/*Property = ScreenPixelsHeight*/
const Value V11854_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P11854_4 = {S54, 0, &V11854_4_1279, &P11854_5};	/*Property = ScreenPixelsWidth*/
const Profile P11854 = {"11854", &P11854_4};	/*Profile ID - 11854*/
const Value V11857_1_1 = {S2, NULL};	/*Value = True*/
const Property P11857_1 = {S50, 0, &V11857_1_1, NULL};	/*Property = IsMobile*/
const Value V11857_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P11857_5 = {S52, 0, &V11857_5_1294, &P11857_1};	/*Property = ScreenPixelsHeight*/
const Value V11857_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11857_4 = {S54, 0, &V11857_4_1296, &P11857_5};	/*Property = ScreenPixelsWidth*/
const Profile P11857 = {"11857", &P11857_4};	/*Profile ID - 11857*/
const Value V13748_1_1 = {S2, NULL};	/*Value = True*/
const Property P13748_1 = {S50, 0, &V13748_1_1, NULL};	/*Property = IsMobile*/
const Value V13748_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13748_5 = {S52, 0, &V13748_5_1304, &P13748_1};	/*Property = ScreenPixelsHeight*/
const Value V13748_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13748_4 = {S54, 0, &V13748_4_1224, &P13748_5};	/*Property = ScreenPixelsWidth*/
const Profile P13748 = {"13748", &P13748_4};	/*Profile ID - 13748*/
const Value V4626_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_279 = {S1, 0, &V4626_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4626_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_282 = {S3, 0, &V4626_282_57155, &P4626_279};	/*Property = BlobBuilder*/
const Value V4626_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_350 = {S4, 0, &V4626_350_57446, &P4626_282};	/*Property = Canvas*/
const Value V4626_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_283 = {S5, 0, &V4626_283_57161, &P4626_350};	/*Property = CssBackground*/
const Value V4626_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_284 = {S6, 0, &V4626_284_57164, &P4626_283};	/*Property = CssBorderImage*/
const Value V4626_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_285 = {S7, 0, &V4626_285_57167, &P4626_284};	/*Property = CssCanvas*/
const Value V4626_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_286 = {S8, 0, &V4626_286_57170, &P4626_285};	/*Property = CssColor*/
const Value V4626_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_237 = {S9, 0, &V4626_237_55786, &P4626_286};	/*Property = CssColumn*/
const Value V4626_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_287 = {S10, 0, &V4626_287_57173, &P4626_237};	/*Property = CssFlexbox*/
const Value V4626_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_288 = {S11, 0, &V4626_288_57176, &P4626_287};	/*Property = CssFont*/
const Value V4626_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_289 = {S12, 0, &V4626_289_86899, &P4626_288};	/*Property = CssImages*/
const Value V4626_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_291 = {S13, 0, &V4626_291_57182, &P4626_289};	/*Property = CssMediaQueries*/
const Value V4626_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_292 = {S14, 0, &V4626_292_57185, &P4626_291};	/*Property = CssMinMax*/
const Value V4626_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_293 = {S15, 0, &V4626_293_57188, &P4626_292};	/*Property = CssOverflow*/
const Value V4626_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_294 = {S16, 0, &V4626_294_57191, &P4626_293};	/*Property = CssPosition*/
const Value V4626_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_295 = {S17, 0, &V4626_295_57194, &P4626_294};	/*Property = CssText*/
const Value V4626_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_297 = {S18, 0, &V4626_297_57197, &P4626_295};	/*Property = CssTransforms*/
const Value V4626_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_241 = {S19, 0, &V4626_241_55801, &P4626_297};	/*Property = CssTransitions*/
const Value V4626_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_298 = {S20, 0, &V4626_298_57208, &P4626_241};	/*Property = CssUI*/
const Value V4626_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_300 = {S21, 0, &V4626_300_57217, &P4626_298};	/*Property = DataSet*/
const Value V4626_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_301 = {S22, 0, &V4626_301_57220, &P4626_300};	/*Property = DataUrl*/
const Value V4626_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_302 = {S23, 0, &V4626_302_57223, &P4626_301};	/*Property = DeviceOrientation*/
const Value V4626_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_304 = {S24, 0, &V4626_304_57229, &P4626_302};	/*Property = FileReader*/
const Value V4626_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_305 = {S25, 0, &V4626_305_57232, &P4626_304};	/*Property = FileSaver*/
const Value V4626_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_306 = {S26, 0, &V4626_306_57238, &P4626_305};	/*Property = FileWriter*/
const Value V4626_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_307 = {S27, 0, &V4626_307_57241, &P4626_306};	/*Property = FormData*/
const Value V4626_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_309 = {S28, 0, &V4626_309_57247, &P4626_307};	/*Property = Fullscreen*/
const Value V4626_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_310 = {S29, 0, &V4626_310_57250, &P4626_309};	/*Property = GeoLocation*/
const Value V4626_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_312 = {S30, 0, &V4626_312_57256, &P4626_310};	/*Property = History*/
const Value V4626_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_314 = {S31, 0, &V4626_314_57262, &P4626_312};	/*Property = Html5*/
const Value V4626_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_313 = {S32, 0, &V4626_313_57259, &P4626_314};	/*Property = Html-Media-Capture*/
const Value V4626_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_315 = {S33, 0, &V4626_315_57265, &P4626_313};	/*Property = Iframe*/
const Value V4626_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_316 = {S34, 0, &V4626_316_57268, &P4626_315};	/*Property = IndexedDB*/
const Value V4626_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_317 = {S35, 0, &V4626_317_57271, &P4626_316};	/*Property = Json*/
const Value V4626_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P4626_15 = {S37, 0, &V4626_15_31, &P4626_317};	/*Property = LayoutEngine*/
const Value V4626_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_344 = {S38, 0, &V4626_344_57274, &P4626_15};	/*Property = Masking*/
const Value V4626_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_320 = {S39, 0, &V4626_320_57292, &P4626_344};	/*Property = PostMessage*/
const Value V4626_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_321 = {S40, 0, &V4626_321_57295, &P4626_320};	/*Property = Progress*/
const Value V4626_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_322 = {S41, 0, &V4626_322_57298, &P4626_321};	/*Property = Prompts*/
const Value V4626_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_323 = {S42, 0, &V4626_323_57301, &P4626_322};	/*Property = Selector*/
const Value V4626_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_324 = {S43, 0, &V4626_324_57307, &P4626_323};	/*Property = Svg*/
const Value V4626_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_327 = {S44, 0, &V4626_327_57316, &P4626_324};	/*Property = TouchEvents*/
const Value V4626_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_328 = {S45, 0, &V4626_328_57319, &P4626_327};	/*Property = Track*/
const Value V4626_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_329 = {S46, 0, &V4626_329_57325, &P4626_328};	/*Property = Video*/
const Value V4626_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_330 = {S47, 0, &V4626_330_57328, &P4626_329};	/*Property = Viewport*/
const Value V4626_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_332 = {S48, 0, &V4626_332_57340, &P4626_330};	/*Property = WebWorkers*/
const Value V4626_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4626_333 = {S49, 0, &V4626_333_57343, &P4626_332};	/*Property = Xhr2*/
const Profile P4626 = {"4626", &P4626_333};	/*Profile ID - 4626*/
const Value V11869_1_1 = {S2, NULL};	/*Value = True*/
const Property P11869_1 = {S50, 0, &V11869_1_1, NULL};	/*Property = IsMobile*/
const Value V11869_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11869_5 = {S52, 0, &V11869_5_1222, &P11869_1};	/*Property = ScreenPixelsHeight*/
const Value V11869_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11869_4 = {S54, 0, &V11869_4_1224, &P11869_5};	/*Property = ScreenPixelsWidth*/
const Profile P11869 = {"11869", &P11869_4};	/*Profile ID - 11869*/
const Value V18182_1_1 = {S2, NULL};	/*Value = True*/
const Property P18182_1 = {S50, 0, &V18182_1_1, NULL};	/*Property = IsMobile*/
const Value V18182_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P18182_5 = {S52, 0, &V18182_5_37, &P18182_1};	/*Property = ScreenPixelsHeight*/
const Value V18182_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P18182_4 = {S54, 0, &V18182_4_36, &P18182_5};	/*Property = ScreenPixelsWidth*/
const Profile P18182 = {"18182", &P18182_4};	/*Profile ID - 18182*/
const Value V15210_1_1 = {S2, NULL};	/*Value = True*/
const Property P15210_1 = {S50, 0, &V15210_1_1, NULL};	/*Property = IsMobile*/
const Value V15210_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15210_5 = {S52, 0, &V15210_5_1227, &P15210_1};	/*Property = ScreenPixelsHeight*/
const Value V15210_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15210_4 = {S54, 0, &V15210_4_1229, &P15210_5};	/*Property = ScreenPixelsWidth*/
const Profile P15210 = {"15210", &P15210_4};	/*Profile ID - 15210*/
const Value V17612_1_1 = {S2, NULL};	/*Value = True*/
const Property P17612_1 = {S50, 0, &V17612_1_1, NULL};	/*Property = IsMobile*/
const Value V17612_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17612_5 = {S52, 0, &V17612_5_1222, &P17612_1};	/*Property = ScreenPixelsHeight*/
const Value V17612_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17612_4 = {S54, 0, &V17612_4_1224, &P17612_5};	/*Property = ScreenPixelsWidth*/
const Profile P17612 = {"17612", &P17612_4};	/*Profile ID - 17612*/
const Value V20111_1_2 = {S0, NULL};	/*Value = False*/
const Property P20111_1 = {S50, 0, &V20111_1_2, NULL};	/*Property = IsMobile*/
const Value V20111_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20111_5 = {S52, 0, &V20111_5_37, &P20111_1};	/*Property = ScreenPixelsHeight*/
const Value V20111_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20111_4 = {S54, 0, &V20111_4_36, &P20111_5};	/*Property = ScreenPixelsWidth*/
const Profile P20111 = {"20111", &P20111_4};	/*Profile ID - 20111*/
const Value V11876_1_1 = {S2, NULL};	/*Value = True*/
const Property P11876_1 = {S50, 0, &V11876_1_1, NULL};	/*Property = IsMobile*/
const Value V11876_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11876_5 = {S52, 0, &V11876_5_1222, &P11876_1};	/*Property = ScreenPixelsHeight*/
const Value V11876_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11876_4 = {S54, 0, &V11876_4_1224, &P11876_5};	/*Property = ScreenPixelsWidth*/
const Profile P11876 = {"11876", &P11876_4};	/*Profile ID - 11876*/
const Value V11877_1_1 = {S2, NULL};	/*Value = True*/
const Property P11877_1 = {S50, 0, &V11877_1_1, NULL};	/*Property = IsMobile*/
const Value V11877_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11877_5 = {S52, 0, &V11877_5_1233, &P11877_1};	/*Property = ScreenPixelsHeight*/
const Value V11877_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11877_4 = {S54, 0, &V11877_4_1235, &P11877_5};	/*Property = ScreenPixelsWidth*/
const Profile P11877 = {"11877", &P11877_4};	/*Profile ID - 11877*/
const Value V11878_1_1 = {S2, NULL};	/*Value = True*/
const Property P11878_1 = {S50, 0, &V11878_1_1, NULL};	/*Property = IsMobile*/
const Value V11878_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11878_5 = {S52, 0, &V11878_5_1222, &P11878_1};	/*Property = ScreenPixelsHeight*/
const Value V11878_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11878_4 = {S54, 0, &V11878_4_1224, &P11878_5};	/*Property = ScreenPixelsWidth*/
const Profile P11878 = {"11878", &P11878_4};	/*Profile ID - 11878*/
const Value V14977_1_1 = {S2, NULL};	/*Value = True*/
const Property P14977_1 = {S50, 0, &V14977_1_1, NULL};	/*Property = IsMobile*/
const Value V14977_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14977_5 = {S52, 0, &V14977_5_1222, &P14977_1};	/*Property = ScreenPixelsHeight*/
const Value V14977_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14977_4 = {S54, 0, &V14977_4_1224, &P14977_5};	/*Property = ScreenPixelsWidth*/
const Profile P14977 = {"14977", &P14977_4};	/*Profile ID - 14977*/
const Value V4093_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_279 = {S1, 0, &V4093_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4093_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_282 = {S3, 0, &V4093_282_57155, &P4093_279};	/*Property = BlobBuilder*/
const Value V4093_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_350 = {S4, 0, &V4093_350_57446, &P4093_282};	/*Property = Canvas*/
const Value V4093_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_283 = {S5, 0, &V4093_283_57161, &P4093_350};	/*Property = CssBackground*/
const Value V4093_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_284 = {S6, 0, &V4093_284_57164, &P4093_283};	/*Property = CssBorderImage*/
const Value V4093_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_285 = {S7, 0, &V4093_285_57167, &P4093_284};	/*Property = CssCanvas*/
const Value V4093_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_286 = {S8, 0, &V4093_286_57170, &P4093_285};	/*Property = CssColor*/
const Value V4093_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_237 = {S9, 0, &V4093_237_55786, &P4093_286};	/*Property = CssColumn*/
const Value V4093_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_287 = {S10, 0, &V4093_287_57173, &P4093_237};	/*Property = CssFlexbox*/
const Value V4093_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_288 = {S11, 0, &V4093_288_57176, &P4093_287};	/*Property = CssFont*/
const Value V4093_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_289 = {S12, 0, &V4093_289_86899, &P4093_288};	/*Property = CssImages*/
const Value V4093_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_291 = {S13, 0, &V4093_291_57182, &P4093_289};	/*Property = CssMediaQueries*/
const Value V4093_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_292 = {S14, 0, &V4093_292_57185, &P4093_291};	/*Property = CssMinMax*/
const Value V4093_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_293 = {S15, 0, &V4093_293_57188, &P4093_292};	/*Property = CssOverflow*/
const Value V4093_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_294 = {S16, 0, &V4093_294_57191, &P4093_293};	/*Property = CssPosition*/
const Value V4093_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_295 = {S17, 0, &V4093_295_57194, &P4093_294};	/*Property = CssText*/
const Value V4093_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_297 = {S18, 0, &V4093_297_57197, &P4093_295};	/*Property = CssTransforms*/
const Value V4093_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_241 = {S19, 0, &V4093_241_55801, &P4093_297};	/*Property = CssTransitions*/
const Value V4093_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_298 = {S20, 0, &V4093_298_57208, &P4093_241};	/*Property = CssUI*/
const Value V4093_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_300 = {S21, 0, &V4093_300_57217, &P4093_298};	/*Property = DataSet*/
const Value V4093_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_301 = {S22, 0, &V4093_301_57220, &P4093_300};	/*Property = DataUrl*/
const Value V4093_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_302 = {S23, 0, &V4093_302_57223, &P4093_301};	/*Property = DeviceOrientation*/
const Value V4093_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_304 = {S24, 0, &V4093_304_57229, &P4093_302};	/*Property = FileReader*/
const Value V4093_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_305 = {S25, 0, &V4093_305_57232, &P4093_304};	/*Property = FileSaver*/
const Value V4093_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_306 = {S26, 0, &V4093_306_57238, &P4093_305};	/*Property = FileWriter*/
const Value V4093_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_307 = {S27, 0, &V4093_307_57241, &P4093_306};	/*Property = FormData*/
const Value V4093_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_309 = {S28, 0, &V4093_309_57247, &P4093_307};	/*Property = Fullscreen*/
const Value V4093_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_310 = {S29, 0, &V4093_310_57250, &P4093_309};	/*Property = GeoLocation*/
const Value V4093_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_312 = {S30, 0, &V4093_312_57256, &P4093_310};	/*Property = History*/
const Value V4093_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_314 = {S31, 0, &V4093_314_57262, &P4093_312};	/*Property = Html5*/
const Value V4093_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_313 = {S32, 0, &V4093_313_57259, &P4093_314};	/*Property = Html-Media-Capture*/
const Value V4093_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_315 = {S33, 0, &V4093_315_57265, &P4093_313};	/*Property = Iframe*/
const Value V4093_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_316 = {S34, 0, &V4093_316_57268, &P4093_315};	/*Property = IndexedDB*/
const Value V4093_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_317 = {S35, 0, &V4093_317_57271, &P4093_316};	/*Property = Json*/
const Value V4093_15_43264 = {S59, NULL};	/*Value = Mango*/
const Property P4093_15 = {S37, 0, &V4093_15_43264, &P4093_317};	/*Property = LayoutEngine*/
const Value V4093_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_344 = {S38, 0, &V4093_344_57274, &P4093_15};	/*Property = Masking*/
const Value V4093_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_320 = {S39, 0, &V4093_320_57292, &P4093_344};	/*Property = PostMessage*/
const Value V4093_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_321 = {S40, 0, &V4093_321_57295, &P4093_320};	/*Property = Progress*/
const Value V4093_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_322 = {S41, 0, &V4093_322_57298, &P4093_321};	/*Property = Prompts*/
const Value V4093_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_323 = {S42, 0, &V4093_323_57301, &P4093_322};	/*Property = Selector*/
const Value V4093_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_324 = {S43, 0, &V4093_324_57307, &P4093_323};	/*Property = Svg*/
const Value V4093_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_327 = {S44, 0, &V4093_327_57316, &P4093_324};	/*Property = TouchEvents*/
const Value V4093_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_328 = {S45, 0, &V4093_328_57319, &P4093_327};	/*Property = Track*/
const Value V4093_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_329 = {S46, 0, &V4093_329_57325, &P4093_328};	/*Property = Video*/
const Value V4093_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_330 = {S47, 0, &V4093_330_57328, &P4093_329};	/*Property = Viewport*/
const Value V4093_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_332 = {S48, 0, &V4093_332_57340, &P4093_330};	/*Property = WebWorkers*/
const Value V4093_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4093_333 = {S49, 0, &V4093_333_57343, &P4093_332};	/*Property = Xhr2*/
const Profile P4093 = {"4093", &P4093_333};	/*Profile ID - 4093*/
const Profile P5826 = {"5826", NULL};	/*Profile ID - 5826*/
const Value V12054_1_1 = {S2, NULL};	/*Value = True*/
const Property P12054_1 = {S50, 0, &V12054_1_1, NULL};	/*Property = IsMobile*/
const Value V12054_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12054_5 = {S52, 0, &V12054_5_1277, &P12054_1};	/*Property = ScreenPixelsHeight*/
const Value V12054_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P12054_4 = {S54, 0, &V12054_4_1278, &P12054_5};	/*Property = ScreenPixelsWidth*/
const Profile P12054 = {"12054", &P12054_4};	/*Profile ID - 12054*/
const Value V11881_1_1 = {S2, NULL};	/*Value = True*/
const Property P11881_1 = {S50, 0, &V11881_1_1, NULL};	/*Property = IsMobile*/
const Value V11881_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11881_5 = {S52, 0, &V11881_5_1293, &P11881_1};	/*Property = ScreenPixelsHeight*/
const Value V11881_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11881_4 = {S54, 0, &V11881_4_1296, &P11881_5};	/*Property = ScreenPixelsWidth*/
const Profile P11881 = {"11881", &P11881_4};	/*Profile ID - 11881*/
const Value V17671_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_279 = {S1, 0, &V17671_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17671_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_282 = {S3, 0, &V17671_282_57155, &P17671_279};	/*Property = BlobBuilder*/
const Value V17671_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_350 = {S4, 0, &V17671_350_57446, &P17671_282};	/*Property = Canvas*/
const Value V17671_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_283 = {S5, 0, &V17671_283_57161, &P17671_350};	/*Property = CssBackground*/
const Value V17671_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_284 = {S6, 0, &V17671_284_57164, &P17671_283};	/*Property = CssBorderImage*/
const Value V17671_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_285 = {S7, 0, &V17671_285_57167, &P17671_284};	/*Property = CssCanvas*/
const Value V17671_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_286 = {S8, 0, &V17671_286_57170, &P17671_285};	/*Property = CssColor*/
const Value V17671_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_237 = {S9, 0, &V17671_237_55786, &P17671_286};	/*Property = CssColumn*/
const Value V17671_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_287 = {S10, 0, &V17671_287_57173, &P17671_237};	/*Property = CssFlexbox*/
const Value V17671_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_288 = {S11, 0, &V17671_288_57176, &P17671_287};	/*Property = CssFont*/
const Value V17671_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_289 = {S12, 0, &V17671_289_86899, &P17671_288};	/*Property = CssImages*/
const Value V17671_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_291 = {S13, 0, &V17671_291_57182, &P17671_289};	/*Property = CssMediaQueries*/
const Value V17671_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_292 = {S14, 0, &V17671_292_57185, &P17671_291};	/*Property = CssMinMax*/
const Value V17671_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_293 = {S15, 0, &V17671_293_57188, &P17671_292};	/*Property = CssOverflow*/
const Value V17671_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_294 = {S16, 0, &V17671_294_57191, &P17671_293};	/*Property = CssPosition*/
const Value V17671_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_295 = {S17, 0, &V17671_295_57194, &P17671_294};	/*Property = CssText*/
const Value V17671_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_297 = {S18, 0, &V17671_297_57197, &P17671_295};	/*Property = CssTransforms*/
const Value V17671_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_241 = {S19, 0, &V17671_241_55801, &P17671_297};	/*Property = CssTransitions*/
const Value V17671_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_298 = {S20, 0, &V17671_298_57208, &P17671_241};	/*Property = CssUI*/
const Value V17671_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_300 = {S21, 0, &V17671_300_57217, &P17671_298};	/*Property = DataSet*/
const Value V17671_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_301 = {S22, 0, &V17671_301_57220, &P17671_300};	/*Property = DataUrl*/
const Value V17671_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_302 = {S23, 0, &V17671_302_57223, &P17671_301};	/*Property = DeviceOrientation*/
const Value V17671_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_304 = {S24, 0, &V17671_304_57229, &P17671_302};	/*Property = FileReader*/
const Value V17671_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_305 = {S25, 0, &V17671_305_57232, &P17671_304};	/*Property = FileSaver*/
const Value V17671_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_306 = {S26, 0, &V17671_306_57238, &P17671_305};	/*Property = FileWriter*/
const Value V17671_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_307 = {S27, 0, &V17671_307_57241, &P17671_306};	/*Property = FormData*/
const Value V17671_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_309 = {S28, 0, &V17671_309_57247, &P17671_307};	/*Property = Fullscreen*/
const Value V17671_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_310 = {S29, 0, &V17671_310_57250, &P17671_309};	/*Property = GeoLocation*/
const Value V17671_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_312 = {S30, 0, &V17671_312_57256, &P17671_310};	/*Property = History*/
const Value V17671_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_314 = {S31, 0, &V17671_314_57262, &P17671_312};	/*Property = Html5*/
const Value V17671_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_313 = {S32, 0, &V17671_313_57259, &P17671_314};	/*Property = Html-Media-Capture*/
const Value V17671_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_315 = {S33, 0, &V17671_315_57265, &P17671_313};	/*Property = Iframe*/
const Value V17671_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_316 = {S34, 0, &V17671_316_57268, &P17671_315};	/*Property = IndexedDB*/
const Value V17671_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_317 = {S35, 0, &V17671_317_57271, &P17671_316};	/*Property = Json*/
const Value V17671_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_15 = {S37, 0, &V17671_15_72, &P17671_317};	/*Property = LayoutEngine*/
const Value V17671_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_344 = {S38, 0, &V17671_344_57274, &P17671_15};	/*Property = Masking*/
const Value V17671_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_320 = {S39, 0, &V17671_320_57292, &P17671_344};	/*Property = PostMessage*/
const Value V17671_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_321 = {S40, 0, &V17671_321_57295, &P17671_320};	/*Property = Progress*/
const Value V17671_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_322 = {S41, 0, &V17671_322_57298, &P17671_321};	/*Property = Prompts*/
const Value V17671_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_323 = {S42, 0, &V17671_323_57301, &P17671_322};	/*Property = Selector*/
const Value V17671_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_324 = {S43, 0, &V17671_324_57307, &P17671_323};	/*Property = Svg*/
const Value V17671_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_327 = {S44, 0, &V17671_327_57316, &P17671_324};	/*Property = TouchEvents*/
const Value V17671_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_328 = {S45, 0, &V17671_328_57319, &P17671_327};	/*Property = Track*/
const Value V17671_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_329 = {S46, 0, &V17671_329_57325, &P17671_328};	/*Property = Video*/
const Value V17671_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_330 = {S47, 0, &V17671_330_57328, &P17671_329};	/*Property = Viewport*/
const Value V17671_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_332 = {S48, 0, &V17671_332_57340, &P17671_330};	/*Property = WebWorkers*/
const Value V17671_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17671_333 = {S49, 0, &V17671_333_57343, &P17671_332};	/*Property = Xhr2*/
const Profile P17671 = {"17671", &P17671_333};	/*Profile ID - 17671*/
const Value V11883_1_1 = {S2, NULL};	/*Value = True*/
const Property P11883_1 = {S50, 0, &V11883_1_1, NULL};	/*Property = IsMobile*/
const Value V11883_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11883_5 = {S52, 0, &V11883_5_1293, &P11883_1};	/*Property = ScreenPixelsHeight*/
const Value V11883_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11883_4 = {S54, 0, &V11883_4_1296, &P11883_5};	/*Property = ScreenPixelsWidth*/
const Profile P11883 = {"11883", &P11883_4};	/*Profile ID - 11883*/
const Value V15364_1_2 = {S0, NULL};	/*Value = False*/
const Property P15364_1 = {S50, 0, &V15364_1_2, NULL};	/*Property = IsMobile*/
const Value V15364_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P15364_5 = {S52, 0, &V15364_5_37, &P15364_1};	/*Property = ScreenPixelsHeight*/
const Value V15364_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P15364_4 = {S54, 0, &V15364_4_36, &P15364_5};	/*Property = ScreenPixelsWidth*/
const Profile P15364 = {"15364", &P15364_4};	/*Profile ID - 15364*/
const Value V3680_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_279 = {S1, 0, &V3680_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3680_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_282 = {S3, 0, &V3680_282_57155, &P3680_279};	/*Property = BlobBuilder*/
const Value V3680_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_350 = {S4, 0, &V3680_350_57446, &P3680_282};	/*Property = Canvas*/
const Value V3680_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_283 = {S5, 0, &V3680_283_57161, &P3680_350};	/*Property = CssBackground*/
const Value V3680_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_284 = {S6, 0, &V3680_284_57164, &P3680_283};	/*Property = CssBorderImage*/
const Value V3680_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_285 = {S7, 0, &V3680_285_57167, &P3680_284};	/*Property = CssCanvas*/
const Value V3680_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_286 = {S8, 0, &V3680_286_57170, &P3680_285};	/*Property = CssColor*/
const Value V3680_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_237 = {S9, 0, &V3680_237_55786, &P3680_286};	/*Property = CssColumn*/
const Value V3680_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_287 = {S10, 0, &V3680_287_57173, &P3680_237};	/*Property = CssFlexbox*/
const Value V3680_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_288 = {S11, 0, &V3680_288_57176, &P3680_287};	/*Property = CssFont*/
const Value V3680_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_289 = {S12, 0, &V3680_289_86899, &P3680_288};	/*Property = CssImages*/
const Value V3680_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_291 = {S13, 0, &V3680_291_57182, &P3680_289};	/*Property = CssMediaQueries*/
const Value V3680_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_292 = {S14, 0, &V3680_292_57185, &P3680_291};	/*Property = CssMinMax*/
const Value V3680_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_293 = {S15, 0, &V3680_293_57188, &P3680_292};	/*Property = CssOverflow*/
const Value V3680_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_294 = {S16, 0, &V3680_294_57191, &P3680_293};	/*Property = CssPosition*/
const Value V3680_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_295 = {S17, 0, &V3680_295_57194, &P3680_294};	/*Property = CssText*/
const Value V3680_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_297 = {S18, 0, &V3680_297_57197, &P3680_295};	/*Property = CssTransforms*/
const Value V3680_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_241 = {S19, 0, &V3680_241_55801, &P3680_297};	/*Property = CssTransitions*/
const Value V3680_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_298 = {S20, 0, &V3680_298_57208, &P3680_241};	/*Property = CssUI*/
const Value V3680_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_300 = {S21, 0, &V3680_300_57217, &P3680_298};	/*Property = DataSet*/
const Value V3680_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_301 = {S22, 0, &V3680_301_57220, &P3680_300};	/*Property = DataUrl*/
const Value V3680_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_302 = {S23, 0, &V3680_302_57223, &P3680_301};	/*Property = DeviceOrientation*/
const Value V3680_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_304 = {S24, 0, &V3680_304_57229, &P3680_302};	/*Property = FileReader*/
const Value V3680_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_305 = {S25, 0, &V3680_305_57232, &P3680_304};	/*Property = FileSaver*/
const Value V3680_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_306 = {S26, 0, &V3680_306_57238, &P3680_305};	/*Property = FileWriter*/
const Value V3680_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_307 = {S27, 0, &V3680_307_57241, &P3680_306};	/*Property = FormData*/
const Value V3680_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_309 = {S28, 0, &V3680_309_57247, &P3680_307};	/*Property = Fullscreen*/
const Value V3680_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_310 = {S29, 0, &V3680_310_57250, &P3680_309};	/*Property = GeoLocation*/
const Value V3680_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_312 = {S30, 0, &V3680_312_57256, &P3680_310};	/*Property = History*/
const Value V3680_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_314 = {S31, 0, &V3680_314_57262, &P3680_312};	/*Property = Html5*/
const Value V3680_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_313 = {S32, 0, &V3680_313_57259, &P3680_314};	/*Property = Html-Media-Capture*/
const Value V3680_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_315 = {S33, 0, &V3680_315_57265, &P3680_313};	/*Property = Iframe*/
const Value V3680_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_316 = {S34, 0, &V3680_316_57268, &P3680_315};	/*Property = IndexedDB*/
const Value V3680_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_317 = {S35, 0, &V3680_317_57271, &P3680_316};	/*Property = Json*/
const Value V3680_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_15 = {S37, 0, &V3680_15_72, &P3680_317};	/*Property = LayoutEngine*/
const Value V3680_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_344 = {S38, 0, &V3680_344_57274, &P3680_15};	/*Property = Masking*/
const Value V3680_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_320 = {S39, 0, &V3680_320_57292, &P3680_344};	/*Property = PostMessage*/
const Value V3680_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_321 = {S40, 0, &V3680_321_57295, &P3680_320};	/*Property = Progress*/
const Value V3680_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_322 = {S41, 0, &V3680_322_57298, &P3680_321};	/*Property = Prompts*/
const Value V3680_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_323 = {S42, 0, &V3680_323_57301, &P3680_322};	/*Property = Selector*/
const Value V3680_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_324 = {S43, 0, &V3680_324_57307, &P3680_323};	/*Property = Svg*/
const Value V3680_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_327 = {S44, 0, &V3680_327_57316, &P3680_324};	/*Property = TouchEvents*/
const Value V3680_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_328 = {S45, 0, &V3680_328_57319, &P3680_327};	/*Property = Track*/
const Value V3680_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_329 = {S46, 0, &V3680_329_57325, &P3680_328};	/*Property = Video*/
const Value V3680_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_330 = {S47, 0, &V3680_330_57328, &P3680_329};	/*Property = Viewport*/
const Value V3680_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_332 = {S48, 0, &V3680_332_57340, &P3680_330};	/*Property = WebWorkers*/
const Value V3680_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3680_333 = {S49, 0, &V3680_333_57343, &P3680_332};	/*Property = Xhr2*/
const Profile P3680 = {"3680", &P3680_333};	/*Profile ID - 3680*/
const Profile P5530 = {"5530", NULL};	/*Profile ID - 5530*/
const Value V11885_1_1 = {S2, NULL};	/*Value = True*/
const Property P11885_1 = {S50, 0, &V11885_1_1, NULL};	/*Property = IsMobile*/
const Value V11885_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11885_5 = {S52, 0, &V11885_5_1233, &P11885_1};	/*Property = ScreenPixelsHeight*/
const Value V11885_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11885_4 = {S54, 0, &V11885_4_1235, &P11885_5};	/*Property = ScreenPixelsWidth*/
const Profile P11885 = {"11885", &P11885_4};	/*Profile ID - 11885*/
const Value V11886_1_1 = {S2, NULL};	/*Value = True*/
const Property P11886_1 = {S50, 0, &V11886_1_1, NULL};	/*Property = IsMobile*/
const Value V11886_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P11886_5 = {S52, 0, &V11886_5_1380, &P11886_1};	/*Property = ScreenPixelsHeight*/
const Value V11886_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P11886_4 = {S54, 0, &V11886_4_1279, &P11886_5};	/*Property = ScreenPixelsWidth*/
const Profile P11886 = {"11886", &P11886_4};	/*Profile ID - 11886*/
const Value V11888_1_1 = {S2, NULL};	/*Value = True*/
const Property P11888_1 = {S50, 0, &V11888_1_1, NULL};	/*Property = IsMobile*/
const Value V11888_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11888_5 = {S52, 0, &V11888_5_1293, &P11888_1};	/*Property = ScreenPixelsHeight*/
const Value V11888_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11888_4 = {S54, 0, &V11888_4_1296, &P11888_5};	/*Property = ScreenPixelsWidth*/
const Profile P11888 = {"11888", &P11888_4};	/*Profile ID - 11888*/
const Value V14716_1_1 = {S2, NULL};	/*Value = True*/
const Property P14716_1 = {S50, 0, &V14716_1_1, NULL};	/*Property = IsMobile*/
const Value V14716_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P14716_5 = {S52, 0, &V14716_5_1280, &P14716_1};	/*Property = ScreenPixelsHeight*/
const Value V14716_4_1368 = {S74, NULL};	/*Value = 208*/
const Property P14716_4 = {S54, 0, &V14716_4_1368, &P14716_5};	/*Property = ScreenPixelsWidth*/
const Profile P14716 = {"14716", &P14716_4};	/*Profile ID - 14716*/
const Value V11890_1_1 = {S2, NULL};	/*Value = True*/
const Property P11890_1 = {S50, 0, &V11890_1_1, NULL};	/*Property = IsMobile*/
const Value V11890_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11890_5 = {S52, 0, &V11890_5_1222, &P11890_1};	/*Property = ScreenPixelsHeight*/
const Value V11890_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11890_4 = {S54, 0, &V11890_4_1224, &P11890_5};	/*Property = ScreenPixelsWidth*/
const Profile P11890 = {"11890", &P11890_4};	/*Profile ID - 11890*/
const Value V11891_1_1 = {S2, NULL};	/*Value = True*/
const Property P11891_1 = {S50, 0, &V11891_1_1, NULL};	/*Property = IsMobile*/
const Value V11891_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11891_5 = {S52, 0, &V11891_5_1293, &P11891_1};	/*Property = ScreenPixelsHeight*/
const Value V11891_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11891_4 = {S54, 0, &V11891_4_1296, &P11891_5};	/*Property = ScreenPixelsWidth*/
const Profile P11891 = {"11891", &P11891_4};	/*Profile ID - 11891*/
const Value V14247_1_1 = {S2, NULL};	/*Value = True*/
const Property P14247_1 = {S50, 0, &V14247_1_1, NULL};	/*Property = IsMobile*/
const Value V14247_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14247_5 = {S52, 0, &V14247_5_1293, &P14247_1};	/*Property = ScreenPixelsHeight*/
const Value V14247_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14247_4 = {S54, 0, &V14247_4_1296, &P14247_5};	/*Property = ScreenPixelsWidth*/
const Profile P14247 = {"14247", &P14247_4};	/*Profile ID - 14247*/
const Value V13226_1_1 = {S2, NULL};	/*Value = True*/
const Property P13226_1 = {S50, 0, &V13226_1_1, NULL};	/*Property = IsMobile*/
const Value V13226_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13226_5 = {S52, 0, &V13226_5_1294, &P13226_1};	/*Property = ScreenPixelsHeight*/
const Value V13226_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13226_4 = {S54, 0, &V13226_4_1296, &P13226_5};	/*Property = ScreenPixelsWidth*/
const Profile P13226 = {"13226", &P13226_4};	/*Profile ID - 13226*/
const Value V11894_1_1 = {S2, NULL};	/*Value = True*/
const Property P11894_1 = {S50, 0, &V11894_1_1, NULL};	/*Property = IsMobile*/
const Value V11894_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11894_5 = {S52, 0, &V11894_5_1293, &P11894_1};	/*Property = ScreenPixelsHeight*/
const Value V11894_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11894_4 = {S54, 0, &V11894_4_1296, &P11894_5};	/*Property = ScreenPixelsWidth*/
const Profile P11894 = {"11894", &P11894_4};	/*Profile ID - 11894*/
const Value V3756_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_279 = {S1, 0, &V3756_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3756_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_282 = {S3, 0, &V3756_282_57155, &P3756_279};	/*Property = BlobBuilder*/
const Value V3756_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_350 = {S4, 0, &V3756_350_57446, &P3756_282};	/*Property = Canvas*/
const Value V3756_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_283 = {S5, 0, &V3756_283_57161, &P3756_350};	/*Property = CssBackground*/
const Value V3756_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_284 = {S6, 0, &V3756_284_57164, &P3756_283};	/*Property = CssBorderImage*/
const Value V3756_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_285 = {S7, 0, &V3756_285_57167, &P3756_284};	/*Property = CssCanvas*/
const Value V3756_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_286 = {S8, 0, &V3756_286_57170, &P3756_285};	/*Property = CssColor*/
const Value V3756_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_237 = {S9, 0, &V3756_237_55786, &P3756_286};	/*Property = CssColumn*/
const Value V3756_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_287 = {S10, 0, &V3756_287_57173, &P3756_237};	/*Property = CssFlexbox*/
const Value V3756_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_288 = {S11, 0, &V3756_288_57176, &P3756_287};	/*Property = CssFont*/
const Value V3756_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_289 = {S12, 0, &V3756_289_86899, &P3756_288};	/*Property = CssImages*/
const Value V3756_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_291 = {S13, 0, &V3756_291_57182, &P3756_289};	/*Property = CssMediaQueries*/
const Value V3756_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_292 = {S14, 0, &V3756_292_57185, &P3756_291};	/*Property = CssMinMax*/
const Value V3756_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_293 = {S15, 0, &V3756_293_57188, &P3756_292};	/*Property = CssOverflow*/
const Value V3756_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_294 = {S16, 0, &V3756_294_57191, &P3756_293};	/*Property = CssPosition*/
const Value V3756_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_295 = {S17, 0, &V3756_295_57194, &P3756_294};	/*Property = CssText*/
const Value V3756_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_297 = {S18, 0, &V3756_297_57197, &P3756_295};	/*Property = CssTransforms*/
const Value V3756_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_241 = {S19, 0, &V3756_241_55801, &P3756_297};	/*Property = CssTransitions*/
const Value V3756_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_298 = {S20, 0, &V3756_298_57208, &P3756_241};	/*Property = CssUI*/
const Value V3756_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_300 = {S21, 0, &V3756_300_57217, &P3756_298};	/*Property = DataSet*/
const Value V3756_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_301 = {S22, 0, &V3756_301_57220, &P3756_300};	/*Property = DataUrl*/
const Value V3756_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_302 = {S23, 0, &V3756_302_57223, &P3756_301};	/*Property = DeviceOrientation*/
const Value V3756_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_304 = {S24, 0, &V3756_304_57229, &P3756_302};	/*Property = FileReader*/
const Value V3756_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_305 = {S25, 0, &V3756_305_57232, &P3756_304};	/*Property = FileSaver*/
const Value V3756_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_306 = {S26, 0, &V3756_306_57238, &P3756_305};	/*Property = FileWriter*/
const Value V3756_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_307 = {S27, 0, &V3756_307_57241, &P3756_306};	/*Property = FormData*/
const Value V3756_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_309 = {S28, 0, &V3756_309_57247, &P3756_307};	/*Property = Fullscreen*/
const Value V3756_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_310 = {S29, 0, &V3756_310_57250, &P3756_309};	/*Property = GeoLocation*/
const Value V3756_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_312 = {S30, 0, &V3756_312_57256, &P3756_310};	/*Property = History*/
const Value V3756_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_314 = {S31, 0, &V3756_314_57262, &P3756_312};	/*Property = Html5*/
const Value V3756_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_313 = {S32, 0, &V3756_313_57259, &P3756_314};	/*Property = Html-Media-Capture*/
const Value V3756_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_315 = {S33, 0, &V3756_315_57265, &P3756_313};	/*Property = Iframe*/
const Value V3756_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_316 = {S34, 0, &V3756_316_57268, &P3756_315};	/*Property = IndexedDB*/
const Value V3756_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_317 = {S35, 0, &V3756_317_57271, &P3756_316};	/*Property = Json*/
const Value V3756_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3756_15 = {S37, 0, &V3756_15_28, &P3756_317};	/*Property = LayoutEngine*/
const Value V3756_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_344 = {S38, 0, &V3756_344_57274, &P3756_15};	/*Property = Masking*/
const Value V3756_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_320 = {S39, 0, &V3756_320_57292, &P3756_344};	/*Property = PostMessage*/
const Value V3756_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_321 = {S40, 0, &V3756_321_57295, &P3756_320};	/*Property = Progress*/
const Value V3756_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_322 = {S41, 0, &V3756_322_57298, &P3756_321};	/*Property = Prompts*/
const Value V3756_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_323 = {S42, 0, &V3756_323_57301, &P3756_322};	/*Property = Selector*/
const Value V3756_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_324 = {S43, 0, &V3756_324_57307, &P3756_323};	/*Property = Svg*/
const Value V3756_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_327 = {S44, 0, &V3756_327_57316, &P3756_324};	/*Property = TouchEvents*/
const Value V3756_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_328 = {S45, 0, &V3756_328_57319, &P3756_327};	/*Property = Track*/
const Value V3756_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_329 = {S46, 0, &V3756_329_57325, &P3756_328};	/*Property = Video*/
const Value V3756_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_330 = {S47, 0, &V3756_330_57328, &P3756_329};	/*Property = Viewport*/
const Value V3756_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_332 = {S48, 0, &V3756_332_57340, &P3756_330};	/*Property = WebWorkers*/
const Value V3756_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3756_333 = {S49, 0, &V3756_333_57343, &P3756_332};	/*Property = Xhr2*/
const Profile P3756 = {"3756", &P3756_333};	/*Profile ID - 3756*/
const Profile P5776 = {"5776", NULL};	/*Profile ID - 5776*/
const Value V11689_1_1 = {S2, NULL};	/*Value = True*/
const Property P11689_1 = {S50, 0, &V11689_1_1, NULL};	/*Property = IsMobile*/
const Value V11689_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P11689_5 = {S52, 0, &V11689_5_1280, &P11689_1};	/*Property = ScreenPixelsHeight*/
const Value V11689_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11689_4 = {S54, 0, &V11689_4_1235, &P11689_5};	/*Property = ScreenPixelsWidth*/
const Profile P11689 = {"11689", &P11689_4};	/*Profile ID - 11689*/
const Value V14931_1_1 = {S2, NULL};	/*Value = True*/
const Property P14931_1 = {S50, 0, &V14931_1_1, NULL};	/*Property = IsMobile*/
const Value V14931_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14931_5 = {S52, 0, &V14931_5_1222, &P14931_1};	/*Property = ScreenPixelsHeight*/
const Value V14931_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14931_4 = {S54, 0, &V14931_4_1224, &P14931_5};	/*Property = ScreenPixelsWidth*/
const Profile P14931 = {"14931", &P14931_4};	/*Profile ID - 14931*/
const Value V17595_1_2 = {S0, NULL};	/*Value = False*/
const Property P17595_1 = {S50, 0, &V17595_1_2, NULL};	/*Property = IsMobile*/
const Value V17595_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P17595_5 = {S52, 0, &V17595_5_37, &P17595_1};	/*Property = ScreenPixelsHeight*/
const Value V17595_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P17595_4 = {S54, 0, &V17595_4_36, &P17595_5};	/*Property = ScreenPixelsWidth*/
const Profile P17595 = {"17595", &P17595_4};	/*Profile ID - 17595*/
const Profile P17295 = {"17295", NULL};	/*Profile ID - 17295*/
const Value V17318_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17318_279 = {S1, 0, &V17318_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17318_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17318_282 = {S3, 0, &V17318_282_57154, &P17318_279};	/*Property = BlobBuilder*/
const Value V17318_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17318_350 = {S4, 0, &V17318_350_57444, &P17318_282};	/*Property = Canvas*/
const Value V17318_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17318_283 = {S5, 0, &V17318_283_57159, &P17318_350};	/*Property = CssBackground*/
const Value V17318_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17318_284 = {S6, 0, &V17318_284_57163, &P17318_283};	/*Property = CssBorderImage*/
const Value V17318_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17318_285 = {S7, 0, &V17318_285_57166, &P17318_284};	/*Property = CssCanvas*/
const Value V17318_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17318_286 = {S8, 0, &V17318_286_57168, &P17318_285};	/*Property = CssColor*/
const Value V17318_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17318_237 = {S9, 0, &V17318_237_55786, &P17318_286};	/*Property = CssColumn*/
const Value V17318_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17318_287 = {S10, 0, &V17318_287_57172, &P17318_237};	/*Property = CssFlexbox*/
const Value V17318_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17318_288 = {S11, 0, &V17318_288_57175, &P17318_287};	/*Property = CssFont*/
const Value V17318_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17318_289 = {S12, 0, &V17318_289_57759, &P17318_288};	/*Property = CssImages*/
const Value V17318_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17318_291 = {S13, 0, &V17318_291_57181, &P17318_289};	/*Property = CssMediaQueries*/
const Value V17318_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17318_292 = {S14, 0, &V17318_292_57183, &P17318_291};	/*Property = CssMinMax*/
const Value V17318_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17318_293 = {S15, 0, &V17318_293_57187, &P17318_292};	/*Property = CssOverflow*/
const Value V17318_294_57190 = {S0, NULL};	/*Value = False*/
const Property P17318_294 = {S16, 0, &V17318_294_57190, &P17318_293};	/*Property = CssPosition*/
const Value V17318_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17318_295 = {S17, 0, &V17318_295_57192, &P17318_294};	/*Property = CssText*/
const Value V17318_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17318_297 = {S18, 0, &V17318_297_57195, &P17318_295};	/*Property = CssTransforms*/
const Value V17318_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17318_241 = {S19, 0, &V17318_241_55799, &P17318_297};	/*Property = CssTransitions*/
const Value V17318_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17318_298 = {S20, 0, &V17318_298_57206, &P17318_241};	/*Property = CssUI*/
const Value V17318_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17318_300 = {S21, 0, &V17318_300_57216, &P17318_298};	/*Property = DataSet*/
const Value V17318_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17318_301 = {S22, 0, &V17318_301_57218, &P17318_300};	/*Property = DataUrl*/
const Value V17318_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17318_302 = {S23, 0, &V17318_302_57222, &P17318_301};	/*Property = DeviceOrientation*/
const Value V17318_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17318_304 = {S24, 0, &V17318_304_57228, &P17318_302};	/*Property = FileReader*/
const Value V17318_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17318_305 = {S25, 0, &V17318_305_57231, &P17318_304};	/*Property = FileSaver*/
const Value V17318_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17318_306 = {S26, 0, &V17318_306_57237, &P17318_305};	/*Property = FileWriter*/
const Value V17318_307_57240 = {S0, NULL};	/*Value = False*/
const Property P17318_307 = {S27, 0, &V17318_307_57240, &P17318_306};	/*Property = FormData*/
const Value V17318_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17318_309 = {S28, 0, &V17318_309_57246, &P17318_307};	/*Property = Fullscreen*/
const Value V17318_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17318_310 = {S29, 0, &V17318_310_57248, &P17318_309};	/*Property = GeoLocation*/
const Value V17318_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17318_312 = {S30, 0, &V17318_312_57255, &P17318_310};	/*Property = History*/
const Value V17318_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17318_314 = {S31, 0, &V17318_314_57261, &P17318_312};	/*Property = Html5*/
const Value V17318_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17318_313 = {S32, 0, &V17318_313_57258, &P17318_314};	/*Property = Html-Media-Capture*/
const Value V17318_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17318_315 = {S33, 0, &V17318_315_57264, &P17318_313};	/*Property = Iframe*/
const Value V17318_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17318_316 = {S34, 0, &V17318_316_57267, &P17318_315};	/*Property = IndexedDB*/
const Value V17318_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17318_317 = {S35, 0, &V17318_317_57269, &P17318_316};	/*Property = Json*/
const Value V17318_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17318_15 = {S37, 0, &V17318_15_28, &P17318_317};	/*Property = LayoutEngine*/
const Value V17318_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17318_344 = {S38, 0, &V17318_344_57272, &P17318_15};	/*Property = Masking*/
const Value V17318_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17318_320 = {S39, 0, &V17318_320_57290, &P17318_344};	/*Property = PostMessage*/
const Value V17318_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17318_321 = {S40, 0, &V17318_321_57293, &P17318_320};	/*Property = Progress*/
const Value V17318_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17318_322 = {S41, 0, &V17318_322_57296, &P17318_321};	/*Property = Prompts*/
const Value V17318_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17318_323 = {S42, 0, &V17318_323_57299, &P17318_322};	/*Property = Selector*/
const Value V17318_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17318_324 = {S43, 0, &V17318_324_57306, &P17318_323};	/*Property = Svg*/
const Value V17318_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17318_327 = {S44, 0, &V17318_327_57315, &P17318_324};	/*Property = TouchEvents*/
const Value V17318_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17318_328 = {S45, 0, &V17318_328_57318, &P17318_327};	/*Property = Track*/
const Value V17318_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17318_329 = {S46, 0, &V17318_329_57323, &P17318_328};	/*Property = Video*/
const Value V17318_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17318_330 = {S47, 0, &V17318_330_57326, &P17318_329};	/*Property = Viewport*/
const Value V17318_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17318_332 = {S48, 0, &V17318_332_57339, &P17318_330};	/*Property = WebWorkers*/
const Value V17318_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17318_333 = {S49, 0, &V17318_333_57342, &P17318_332};	/*Property = Xhr2*/
const Profile P17318 = {"17318", &P17318_333};	/*Profile ID - 17318*/
const Value V15366_1_1 = {S2, NULL};	/*Value = True*/
const Property P15366_1 = {S50, 0, &V15366_1_1, NULL};	/*Property = IsMobile*/
const Value V15366_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15366_5 = {S52, 0, &V15366_5_1222, &P15366_1};	/*Property = ScreenPixelsHeight*/
const Value V15366_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15366_4 = {S54, 0, &V15366_4_1224, &P15366_5};	/*Property = ScreenPixelsWidth*/
const Profile P15366 = {"15366", &P15366_4};	/*Profile ID - 15366*/
const Value V16045_1_1 = {S2, NULL};	/*Value = True*/
const Property P16045_1 = {S50, 0, &V16045_1_1, NULL};	/*Property = IsMobile*/
const Value V16045_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16045_5 = {S52, 0, &V16045_5_1227, &P16045_1};	/*Property = ScreenPixelsHeight*/
const Value V16045_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16045_4 = {S54, 0, &V16045_4_1229, &P16045_5};	/*Property = ScreenPixelsWidth*/
const Profile P16045 = {"16045", &P16045_4};	/*Profile ID - 16045*/
const Value V11900_1_1 = {S2, NULL};	/*Value = True*/
const Property P11900_1 = {S50, 0, &V11900_1_1, NULL};	/*Property = IsMobile*/
const Value V11900_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11900_5 = {S52, 0, &V11900_5_1233, &P11900_1};	/*Property = ScreenPixelsHeight*/
const Value V11900_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11900_4 = {S54, 0, &V11900_4_1235, &P11900_5};	/*Property = ScreenPixelsWidth*/
const Profile P11900 = {"11900", &P11900_4};	/*Profile ID - 11900*/
const Value V13485_1_1 = {S2, NULL};	/*Value = True*/
const Property P13485_1 = {S50, 0, &V13485_1_1, NULL};	/*Property = IsMobile*/
const Value V13485_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13485_5 = {S52, 0, &V13485_5_1380, &P13485_1};	/*Property = ScreenPixelsHeight*/
const Value V13485_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13485_4 = {S54, 0, &V13485_4_1279, &P13485_5};	/*Property = ScreenPixelsWidth*/
const Profile P13485 = {"13485", &P13485_4};	/*Profile ID - 13485*/
const Value V16074_1_1 = {S2, NULL};	/*Value = True*/
const Property P16074_1 = {S50, 0, &V16074_1_1, NULL};	/*Property = IsMobile*/
const Value V16074_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16074_5 = {S52, 0, &V16074_5_1293, &P16074_1};	/*Property = ScreenPixelsHeight*/
const Value V16074_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16074_4 = {S54, 0, &V16074_4_1296, &P16074_5};	/*Property = ScreenPixelsWidth*/
const Profile P16074 = {"16074", &P16074_4};	/*Profile ID - 16074*/
const Value V11625_1_1 = {S2, NULL};	/*Value = True*/
const Property P11625_1 = {S50, 0, &V11625_1_1, NULL};	/*Property = IsMobile*/
const Value V11625_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11625_5 = {S52, 0, &V11625_5_1222, &P11625_1};	/*Property = ScreenPixelsHeight*/
const Value V11625_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11625_4 = {S54, 0, &V11625_4_1224, &P11625_5};	/*Property = ScreenPixelsWidth*/
const Profile P11625 = {"11625", &P11625_4};	/*Profile ID - 11625*/
const Value V13186_1_1 = {S2, NULL};	/*Value = True*/
const Property P13186_1 = {S50, 0, &V13186_1_1, NULL};	/*Property = IsMobile*/
const Value V13186_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13186_5 = {S52, 0, &V13186_5_1222, &P13186_1};	/*Property = ScreenPixelsHeight*/
const Value V13186_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13186_4 = {S54, 0, &V13186_4_1224, &P13186_5};	/*Property = ScreenPixelsWidth*/
const Profile P13186 = {"13186", &P13186_4};	/*Profile ID - 13186*/
const Value V11627_1_1 = {S2, NULL};	/*Value = True*/
const Property P11627_1 = {S50, 0, &V11627_1_1, NULL};	/*Property = IsMobile*/
const Value V11627_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11627_5 = {S52, 0, &V11627_5_1222, &P11627_1};	/*Property = ScreenPixelsHeight*/
const Value V11627_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11627_4 = {S54, 0, &V11627_4_1224, &P11627_5};	/*Property = ScreenPixelsWidth*/
const Profile P11627 = {"11627", &P11627_4};	/*Profile ID - 11627*/
const Value V12309_1_1 = {S2, NULL};	/*Value = True*/
const Property P12309_1 = {S50, 0, &V12309_1_1, NULL};	/*Property = IsMobile*/
const Value V12309_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12309_5 = {S52, 0, &V12309_5_1293, &P12309_1};	/*Property = ScreenPixelsHeight*/
const Value V12309_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12309_4 = {S54, 0, &V12309_4_1296, &P12309_5};	/*Property = ScreenPixelsWidth*/
const Profile P12309 = {"12309", &P12309_4};	/*Profile ID - 12309*/
const Value V4810_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_279 = {S1, 0, &V4810_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4810_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_282 = {S3, 0, &V4810_282_57155, &P4810_279};	/*Property = BlobBuilder*/
const Value V4810_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_350 = {S4, 0, &V4810_350_57446, &P4810_282};	/*Property = Canvas*/
const Value V4810_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_283 = {S5, 0, &V4810_283_57161, &P4810_350};	/*Property = CssBackground*/
const Value V4810_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_284 = {S6, 0, &V4810_284_57164, &P4810_283};	/*Property = CssBorderImage*/
const Value V4810_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_285 = {S7, 0, &V4810_285_57167, &P4810_284};	/*Property = CssCanvas*/
const Value V4810_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_286 = {S8, 0, &V4810_286_57170, &P4810_285};	/*Property = CssColor*/
const Value V4810_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_237 = {S9, 0, &V4810_237_55786, &P4810_286};	/*Property = CssColumn*/
const Value V4810_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_287 = {S10, 0, &V4810_287_57173, &P4810_237};	/*Property = CssFlexbox*/
const Value V4810_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_288 = {S11, 0, &V4810_288_57176, &P4810_287};	/*Property = CssFont*/
const Value V4810_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_289 = {S12, 0, &V4810_289_86899, &P4810_288};	/*Property = CssImages*/
const Value V4810_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_291 = {S13, 0, &V4810_291_57182, &P4810_289};	/*Property = CssMediaQueries*/
const Value V4810_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_292 = {S14, 0, &V4810_292_57185, &P4810_291};	/*Property = CssMinMax*/
const Value V4810_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_293 = {S15, 0, &V4810_293_57188, &P4810_292};	/*Property = CssOverflow*/
const Value V4810_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_294 = {S16, 0, &V4810_294_57191, &P4810_293};	/*Property = CssPosition*/
const Value V4810_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_295 = {S17, 0, &V4810_295_57194, &P4810_294};	/*Property = CssText*/
const Value V4810_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_297 = {S18, 0, &V4810_297_57197, &P4810_295};	/*Property = CssTransforms*/
const Value V4810_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_241 = {S19, 0, &V4810_241_55801, &P4810_297};	/*Property = CssTransitions*/
const Value V4810_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_298 = {S20, 0, &V4810_298_57208, &P4810_241};	/*Property = CssUI*/
const Value V4810_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_300 = {S21, 0, &V4810_300_57217, &P4810_298};	/*Property = DataSet*/
const Value V4810_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_301 = {S22, 0, &V4810_301_57220, &P4810_300};	/*Property = DataUrl*/
const Value V4810_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_302 = {S23, 0, &V4810_302_57223, &P4810_301};	/*Property = DeviceOrientation*/
const Value V4810_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_304 = {S24, 0, &V4810_304_57229, &P4810_302};	/*Property = FileReader*/
const Value V4810_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_305 = {S25, 0, &V4810_305_57232, &P4810_304};	/*Property = FileSaver*/
const Value V4810_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_306 = {S26, 0, &V4810_306_57238, &P4810_305};	/*Property = FileWriter*/
const Value V4810_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_307 = {S27, 0, &V4810_307_57241, &P4810_306};	/*Property = FormData*/
const Value V4810_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_309 = {S28, 0, &V4810_309_57247, &P4810_307};	/*Property = Fullscreen*/
const Value V4810_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_310 = {S29, 0, &V4810_310_57250, &P4810_309};	/*Property = GeoLocation*/
const Value V4810_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_312 = {S30, 0, &V4810_312_57256, &P4810_310};	/*Property = History*/
const Value V4810_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_314 = {S31, 0, &V4810_314_57262, &P4810_312};	/*Property = Html5*/
const Value V4810_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_313 = {S32, 0, &V4810_313_57259, &P4810_314};	/*Property = Html-Media-Capture*/
const Value V4810_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_315 = {S33, 0, &V4810_315_57265, &P4810_313};	/*Property = Iframe*/
const Value V4810_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_316 = {S34, 0, &V4810_316_57268, &P4810_315};	/*Property = IndexedDB*/
const Value V4810_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_317 = {S35, 0, &V4810_317_57271, &P4810_316};	/*Property = Json*/
const Value V4810_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P4810_15 = {S37, 0, &V4810_15_31, &P4810_317};	/*Property = LayoutEngine*/
const Value V4810_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_344 = {S38, 0, &V4810_344_57274, &P4810_15};	/*Property = Masking*/
const Value V4810_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_320 = {S39, 0, &V4810_320_57292, &P4810_344};	/*Property = PostMessage*/
const Value V4810_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_321 = {S40, 0, &V4810_321_57295, &P4810_320};	/*Property = Progress*/
const Value V4810_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_322 = {S41, 0, &V4810_322_57298, &P4810_321};	/*Property = Prompts*/
const Value V4810_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_323 = {S42, 0, &V4810_323_57301, &P4810_322};	/*Property = Selector*/
const Value V4810_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_324 = {S43, 0, &V4810_324_57307, &P4810_323};	/*Property = Svg*/
const Value V4810_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_327 = {S44, 0, &V4810_327_57316, &P4810_324};	/*Property = TouchEvents*/
const Value V4810_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_328 = {S45, 0, &V4810_328_57319, &P4810_327};	/*Property = Track*/
const Value V4810_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_329 = {S46, 0, &V4810_329_57325, &P4810_328};	/*Property = Video*/
const Value V4810_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_330 = {S47, 0, &V4810_330_57328, &P4810_329};	/*Property = Viewport*/
const Value V4810_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_332 = {S48, 0, &V4810_332_57340, &P4810_330};	/*Property = WebWorkers*/
const Value V4810_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4810_333 = {S49, 0, &V4810_333_57343, &P4810_332};	/*Property = Xhr2*/
const Profile P4810 = {"4810", &P4810_333};	/*Profile ID - 4810*/
const Profile P7127 = {"7127", NULL};	/*Profile ID - 7127*/
const Profile P17285 = {"17285", NULL};	/*Profile ID - 17285*/
const Value V17193_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17193_279 = {S1, 0, &V17193_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17193_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17193_282 = {S3, 0, &V17193_282_57154, &P17193_279};	/*Property = BlobBuilder*/
const Value V17193_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17193_350 = {S4, 0, &V17193_350_57444, &P17193_282};	/*Property = Canvas*/
const Value V17193_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17193_283 = {S5, 0, &V17193_283_57159, &P17193_350};	/*Property = CssBackground*/
const Value V17193_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17193_284 = {S6, 0, &V17193_284_57163, &P17193_283};	/*Property = CssBorderImage*/
const Value V17193_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17193_285 = {S7, 0, &V17193_285_57166, &P17193_284};	/*Property = CssCanvas*/
const Value V17193_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17193_286 = {S8, 0, &V17193_286_57168, &P17193_285};	/*Property = CssColor*/
const Value V17193_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17193_237 = {S9, 0, &V17193_237_55786, &P17193_286};	/*Property = CssColumn*/
const Value V17193_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17193_287 = {S10, 0, &V17193_287_57172, &P17193_237};	/*Property = CssFlexbox*/
const Value V17193_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17193_288 = {S11, 0, &V17193_288_57175, &P17193_287};	/*Property = CssFont*/
const Value V17193_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17193_289 = {S12, 0, &V17193_289_57759, &P17193_288};	/*Property = CssImages*/
const Value V17193_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17193_291 = {S13, 0, &V17193_291_57181, &P17193_289};	/*Property = CssMediaQueries*/
const Value V17193_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17193_292 = {S14, 0, &V17193_292_57183, &P17193_291};	/*Property = CssMinMax*/
const Value V17193_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17193_293 = {S15, 0, &V17193_293_57187, &P17193_292};	/*Property = CssOverflow*/
const Value V17193_294_57190 = {S0, NULL};	/*Value = False*/
const Property P17193_294 = {S16, 0, &V17193_294_57190, &P17193_293};	/*Property = CssPosition*/
const Value V17193_295_57193 = {S0, NULL};	/*Value = False*/
const Property P17193_295 = {S17, 0, &V17193_295_57193, &P17193_294};	/*Property = CssText*/
const Value V17193_297_57196 = {S0, NULL};	/*Value = False*/
const Property P17193_297 = {S18, 0, &V17193_297_57196, &P17193_295};	/*Property = CssTransforms*/
const Value V17193_241_55800 = {S0, NULL};	/*Value = False*/
const Property P17193_241 = {S19, 0, &V17193_241_55800, &P17193_297};	/*Property = CssTransitions*/
const Value V17193_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17193_298 = {S20, 0, &V17193_298_57206, &P17193_241};	/*Property = CssUI*/
const Value V17193_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17193_300 = {S21, 0, &V17193_300_57216, &P17193_298};	/*Property = DataSet*/
const Value V17193_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17193_301 = {S22, 0, &V17193_301_57218, &P17193_300};	/*Property = DataUrl*/
const Value V17193_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17193_302 = {S23, 0, &V17193_302_57222, &P17193_301};	/*Property = DeviceOrientation*/
const Value V17193_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17193_304 = {S24, 0, &V17193_304_57228, &P17193_302};	/*Property = FileReader*/
const Value V17193_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17193_305 = {S25, 0, &V17193_305_57231, &P17193_304};	/*Property = FileSaver*/
const Value V17193_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17193_306 = {S26, 0, &V17193_306_57237, &P17193_305};	/*Property = FileWriter*/
const Value V17193_307_57240 = {S0, NULL};	/*Value = False*/
const Property P17193_307 = {S27, 0, &V17193_307_57240, &P17193_306};	/*Property = FormData*/
const Value V17193_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17193_309 = {S28, 0, &V17193_309_57246, &P17193_307};	/*Property = Fullscreen*/
const Value V17193_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17193_310 = {S29, 0, &V17193_310_57248, &P17193_309};	/*Property = GeoLocation*/
const Value V17193_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17193_312 = {S30, 0, &V17193_312_57255, &P17193_310};	/*Property = History*/
const Value V17193_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17193_314 = {S31, 0, &V17193_314_57261, &P17193_312};	/*Property = Html5*/
const Value V17193_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17193_313 = {S32, 0, &V17193_313_57258, &P17193_314};	/*Property = Html-Media-Capture*/
const Value V17193_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17193_315 = {S33, 0, &V17193_315_57264, &P17193_313};	/*Property = Iframe*/
const Value V17193_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17193_316 = {S34, 0, &V17193_316_57267, &P17193_315};	/*Property = IndexedDB*/
const Value V17193_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17193_317 = {S35, 0, &V17193_317_57269, &P17193_316};	/*Property = Json*/
const Value V17193_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P17193_15 = {S37, 0, &V17193_15_30, &P17193_317};	/*Property = LayoutEngine*/
const Value V17193_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17193_344 = {S38, 0, &V17193_344_57272, &P17193_15};	/*Property = Masking*/
const Value V17193_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17193_320 = {S39, 0, &V17193_320_57290, &P17193_344};	/*Property = PostMessage*/
const Value V17193_321_57294 = {S0, NULL};	/*Value = False*/
const Property P17193_321 = {S40, 0, &V17193_321_57294, &P17193_320};	/*Property = Progress*/
const Value V17193_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17193_322 = {S41, 0, &V17193_322_57296, &P17193_321};	/*Property = Prompts*/
const Value V17193_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17193_323 = {S42, 0, &V17193_323_57299, &P17193_322};	/*Property = Selector*/
const Value V17193_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17193_324 = {S43, 0, &V17193_324_57306, &P17193_323};	/*Property = Svg*/
const Value V17193_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17193_327 = {S44, 0, &V17193_327_57315, &P17193_324};	/*Property = TouchEvents*/
const Value V17193_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17193_328 = {S45, 0, &V17193_328_57318, &P17193_327};	/*Property = Track*/
const Value V17193_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17193_329 = {S46, 0, &V17193_329_57323, &P17193_328};	/*Property = Video*/
const Value V17193_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17193_330 = {S47, 0, &V17193_330_57326, &P17193_329};	/*Property = Viewport*/
const Value V17193_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17193_332 = {S48, 0, &V17193_332_57339, &P17193_330};	/*Property = WebWorkers*/
const Value V17193_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17193_333 = {S49, 0, &V17193_333_57342, &P17193_332};	/*Property = Xhr2*/
const Profile P17193 = {"17193", &P17193_333};	/*Profile ID - 17193*/
const Value V3409_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_279 = {S1, 0, &V3409_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3409_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_282 = {S3, 0, &V3409_282_57155, &P3409_279};	/*Property = BlobBuilder*/
const Value V3409_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_350 = {S4, 0, &V3409_350_57446, &P3409_282};	/*Property = Canvas*/
const Value V3409_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_283 = {S5, 0, &V3409_283_57161, &P3409_350};	/*Property = CssBackground*/
const Value V3409_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_284 = {S6, 0, &V3409_284_57164, &P3409_283};	/*Property = CssBorderImage*/
const Value V3409_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_285 = {S7, 0, &V3409_285_57167, &P3409_284};	/*Property = CssCanvas*/
const Value V3409_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_286 = {S8, 0, &V3409_286_57170, &P3409_285};	/*Property = CssColor*/
const Value V3409_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_237 = {S9, 0, &V3409_237_55786, &P3409_286};	/*Property = CssColumn*/
const Value V3409_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_287 = {S10, 0, &V3409_287_57173, &P3409_237};	/*Property = CssFlexbox*/
const Value V3409_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_288 = {S11, 0, &V3409_288_57176, &P3409_287};	/*Property = CssFont*/
const Value V3409_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_289 = {S12, 0, &V3409_289_86899, &P3409_288};	/*Property = CssImages*/
const Value V3409_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_291 = {S13, 0, &V3409_291_57182, &P3409_289};	/*Property = CssMediaQueries*/
const Value V3409_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_292 = {S14, 0, &V3409_292_57185, &P3409_291};	/*Property = CssMinMax*/
const Value V3409_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_293 = {S15, 0, &V3409_293_57188, &P3409_292};	/*Property = CssOverflow*/
const Value V3409_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_294 = {S16, 0, &V3409_294_57191, &P3409_293};	/*Property = CssPosition*/
const Value V3409_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_295 = {S17, 0, &V3409_295_57194, &P3409_294};	/*Property = CssText*/
const Value V3409_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_297 = {S18, 0, &V3409_297_57197, &P3409_295};	/*Property = CssTransforms*/
const Value V3409_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_241 = {S19, 0, &V3409_241_55801, &P3409_297};	/*Property = CssTransitions*/
const Value V3409_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_298 = {S20, 0, &V3409_298_57208, &P3409_241};	/*Property = CssUI*/
const Value V3409_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_300 = {S21, 0, &V3409_300_57217, &P3409_298};	/*Property = DataSet*/
const Value V3409_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_301 = {S22, 0, &V3409_301_57220, &P3409_300};	/*Property = DataUrl*/
const Value V3409_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_302 = {S23, 0, &V3409_302_57223, &P3409_301};	/*Property = DeviceOrientation*/
const Value V3409_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_304 = {S24, 0, &V3409_304_57229, &P3409_302};	/*Property = FileReader*/
const Value V3409_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_305 = {S25, 0, &V3409_305_57232, &P3409_304};	/*Property = FileSaver*/
const Value V3409_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_306 = {S26, 0, &V3409_306_57238, &P3409_305};	/*Property = FileWriter*/
const Value V3409_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_307 = {S27, 0, &V3409_307_57241, &P3409_306};	/*Property = FormData*/
const Value V3409_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_309 = {S28, 0, &V3409_309_57247, &P3409_307};	/*Property = Fullscreen*/
const Value V3409_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_310 = {S29, 0, &V3409_310_57250, &P3409_309};	/*Property = GeoLocation*/
const Value V3409_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_312 = {S30, 0, &V3409_312_57256, &P3409_310};	/*Property = History*/
const Value V3409_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_314 = {S31, 0, &V3409_314_57262, &P3409_312};	/*Property = Html5*/
const Value V3409_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_313 = {S32, 0, &V3409_313_57259, &P3409_314};	/*Property = Html-Media-Capture*/
const Value V3409_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_315 = {S33, 0, &V3409_315_57265, &P3409_313};	/*Property = Iframe*/
const Value V3409_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_316 = {S34, 0, &V3409_316_57268, &P3409_315};	/*Property = IndexedDB*/
const Value V3409_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_317 = {S35, 0, &V3409_317_57271, &P3409_316};	/*Property = Json*/
const Value V3409_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P3409_15 = {S37, 0, &V3409_15_29, &P3409_317};	/*Property = LayoutEngine*/
const Value V3409_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_344 = {S38, 0, &V3409_344_57274, &P3409_15};	/*Property = Masking*/
const Value V3409_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_320 = {S39, 0, &V3409_320_57292, &P3409_344};	/*Property = PostMessage*/
const Value V3409_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_321 = {S40, 0, &V3409_321_57295, &P3409_320};	/*Property = Progress*/
const Value V3409_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_322 = {S41, 0, &V3409_322_57298, &P3409_321};	/*Property = Prompts*/
const Value V3409_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_323 = {S42, 0, &V3409_323_57301, &P3409_322};	/*Property = Selector*/
const Value V3409_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_324 = {S43, 0, &V3409_324_57307, &P3409_323};	/*Property = Svg*/
const Value V3409_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_327 = {S44, 0, &V3409_327_57316, &P3409_324};	/*Property = TouchEvents*/
const Value V3409_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_328 = {S45, 0, &V3409_328_57319, &P3409_327};	/*Property = Track*/
const Value V3409_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_329 = {S46, 0, &V3409_329_57325, &P3409_328};	/*Property = Video*/
const Value V3409_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_330 = {S47, 0, &V3409_330_57328, &P3409_329};	/*Property = Viewport*/
const Value V3409_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_332 = {S48, 0, &V3409_332_57340, &P3409_330};	/*Property = WebWorkers*/
const Value V3409_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3409_333 = {S49, 0, &V3409_333_57343, &P3409_332};	/*Property = Xhr2*/
const Profile P3409 = {"3409", &P3409_333};	/*Profile ID - 3409*/
const Profile P5690 = {"5690", NULL};	/*Profile ID - 5690*/
const Value V3418_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_279 = {S1, 0, &V3418_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3418_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_282 = {S3, 0, &V3418_282_57155, &P3418_279};	/*Property = BlobBuilder*/
const Value V3418_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_350 = {S4, 0, &V3418_350_57446, &P3418_282};	/*Property = Canvas*/
const Value V3418_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_283 = {S5, 0, &V3418_283_57161, &P3418_350};	/*Property = CssBackground*/
const Value V3418_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_284 = {S6, 0, &V3418_284_57164, &P3418_283};	/*Property = CssBorderImage*/
const Value V3418_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_285 = {S7, 0, &V3418_285_57167, &P3418_284};	/*Property = CssCanvas*/
const Value V3418_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_286 = {S8, 0, &V3418_286_57170, &P3418_285};	/*Property = CssColor*/
const Value V3418_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_237 = {S9, 0, &V3418_237_55786, &P3418_286};	/*Property = CssColumn*/
const Value V3418_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_287 = {S10, 0, &V3418_287_57173, &P3418_237};	/*Property = CssFlexbox*/
const Value V3418_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_288 = {S11, 0, &V3418_288_57176, &P3418_287};	/*Property = CssFont*/
const Value V3418_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_289 = {S12, 0, &V3418_289_86899, &P3418_288};	/*Property = CssImages*/
const Value V3418_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_291 = {S13, 0, &V3418_291_57182, &P3418_289};	/*Property = CssMediaQueries*/
const Value V3418_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_292 = {S14, 0, &V3418_292_57185, &P3418_291};	/*Property = CssMinMax*/
const Value V3418_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_293 = {S15, 0, &V3418_293_57188, &P3418_292};	/*Property = CssOverflow*/
const Value V3418_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_294 = {S16, 0, &V3418_294_57191, &P3418_293};	/*Property = CssPosition*/
const Value V3418_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_295 = {S17, 0, &V3418_295_57194, &P3418_294};	/*Property = CssText*/
const Value V3418_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_297 = {S18, 0, &V3418_297_57197, &P3418_295};	/*Property = CssTransforms*/
const Value V3418_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_241 = {S19, 0, &V3418_241_55801, &P3418_297};	/*Property = CssTransitions*/
const Value V3418_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_298 = {S20, 0, &V3418_298_57208, &P3418_241};	/*Property = CssUI*/
const Value V3418_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_300 = {S21, 0, &V3418_300_57217, &P3418_298};	/*Property = DataSet*/
const Value V3418_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_301 = {S22, 0, &V3418_301_57220, &P3418_300};	/*Property = DataUrl*/
const Value V3418_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_302 = {S23, 0, &V3418_302_57223, &P3418_301};	/*Property = DeviceOrientation*/
const Value V3418_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_304 = {S24, 0, &V3418_304_57229, &P3418_302};	/*Property = FileReader*/
const Value V3418_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_305 = {S25, 0, &V3418_305_57232, &P3418_304};	/*Property = FileSaver*/
const Value V3418_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_306 = {S26, 0, &V3418_306_57238, &P3418_305};	/*Property = FileWriter*/
const Value V3418_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_307 = {S27, 0, &V3418_307_57241, &P3418_306};	/*Property = FormData*/
const Value V3418_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_309 = {S28, 0, &V3418_309_57247, &P3418_307};	/*Property = Fullscreen*/
const Value V3418_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_310 = {S29, 0, &V3418_310_57250, &P3418_309};	/*Property = GeoLocation*/
const Value V3418_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_312 = {S30, 0, &V3418_312_57256, &P3418_310};	/*Property = History*/
const Value V3418_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_314 = {S31, 0, &V3418_314_57262, &P3418_312};	/*Property = Html5*/
const Value V3418_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_313 = {S32, 0, &V3418_313_57259, &P3418_314};	/*Property = Html-Media-Capture*/
const Value V3418_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_315 = {S33, 0, &V3418_315_57265, &P3418_313};	/*Property = Iframe*/
const Value V3418_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_316 = {S34, 0, &V3418_316_57268, &P3418_315};	/*Property = IndexedDB*/
const Value V3418_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_317 = {S35, 0, &V3418_317_57271, &P3418_316};	/*Property = Json*/
const Value V3418_15_43264 = {S59, NULL};	/*Value = Mango*/
const Property P3418_15 = {S37, 0, &V3418_15_43264, &P3418_317};	/*Property = LayoutEngine*/
const Value V3418_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_344 = {S38, 0, &V3418_344_57274, &P3418_15};	/*Property = Masking*/
const Value V3418_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_320 = {S39, 0, &V3418_320_57292, &P3418_344};	/*Property = PostMessage*/
const Value V3418_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_321 = {S40, 0, &V3418_321_57295, &P3418_320};	/*Property = Progress*/
const Value V3418_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_322 = {S41, 0, &V3418_322_57298, &P3418_321};	/*Property = Prompts*/
const Value V3418_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_323 = {S42, 0, &V3418_323_57301, &P3418_322};	/*Property = Selector*/
const Value V3418_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_324 = {S43, 0, &V3418_324_57307, &P3418_323};	/*Property = Svg*/
const Value V3418_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_327 = {S44, 0, &V3418_327_57316, &P3418_324};	/*Property = TouchEvents*/
const Value V3418_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_328 = {S45, 0, &V3418_328_57319, &P3418_327};	/*Property = Track*/
const Value V3418_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_329 = {S46, 0, &V3418_329_57325, &P3418_328};	/*Property = Video*/
const Value V3418_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_330 = {S47, 0, &V3418_330_57328, &P3418_329};	/*Property = Viewport*/
const Value V3418_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_332 = {S48, 0, &V3418_332_57340, &P3418_330};	/*Property = WebWorkers*/
const Value V3418_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3418_333 = {S49, 0, &V3418_333_57343, &P3418_332};	/*Property = Xhr2*/
const Profile P3418 = {"3418", &P3418_333};	/*Profile ID - 3418*/
const Profile P6217 = {"6217", NULL};	/*Profile ID - 6217*/
const Value V12141_1_1 = {S2, NULL};	/*Value = True*/
const Property P12141_1 = {S50, 0, &V12141_1_1, NULL};	/*Property = IsMobile*/
const Value V12141_5_1796 = {S64, NULL};	/*Value = 360*/
const Property P12141_5 = {S52, 0, &V12141_5_1796, &P12141_1};	/*Property = ScreenPixelsHeight*/
const Value V12141_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12141_4 = {S54, 0, &V12141_4_1229, &P12141_5};	/*Property = ScreenPixelsWidth*/
const Profile P12141 = {"12141", &P12141_4};	/*Profile ID - 12141*/
const Value V4948_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_279 = {S1, 0, &V4948_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4948_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_282 = {S3, 0, &V4948_282_57155, &P4948_279};	/*Property = BlobBuilder*/
const Value V4948_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_350 = {S4, 0, &V4948_350_57446, &P4948_282};	/*Property = Canvas*/
const Value V4948_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_283 = {S5, 0, &V4948_283_57161, &P4948_350};	/*Property = CssBackground*/
const Value V4948_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_284 = {S6, 0, &V4948_284_57164, &P4948_283};	/*Property = CssBorderImage*/
const Value V4948_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_285 = {S7, 0, &V4948_285_57167, &P4948_284};	/*Property = CssCanvas*/
const Value V4948_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_286 = {S8, 0, &V4948_286_57170, &P4948_285};	/*Property = CssColor*/
const Value V4948_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_237 = {S9, 0, &V4948_237_55786, &P4948_286};	/*Property = CssColumn*/
const Value V4948_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_287 = {S10, 0, &V4948_287_57173, &P4948_237};	/*Property = CssFlexbox*/
const Value V4948_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_288 = {S11, 0, &V4948_288_57176, &P4948_287};	/*Property = CssFont*/
const Value V4948_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_289 = {S12, 0, &V4948_289_86899, &P4948_288};	/*Property = CssImages*/
const Value V4948_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_291 = {S13, 0, &V4948_291_57182, &P4948_289};	/*Property = CssMediaQueries*/
const Value V4948_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_292 = {S14, 0, &V4948_292_57185, &P4948_291};	/*Property = CssMinMax*/
const Value V4948_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_293 = {S15, 0, &V4948_293_57188, &P4948_292};	/*Property = CssOverflow*/
const Value V4948_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_294 = {S16, 0, &V4948_294_57191, &P4948_293};	/*Property = CssPosition*/
const Value V4948_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_295 = {S17, 0, &V4948_295_57194, &P4948_294};	/*Property = CssText*/
const Value V4948_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_297 = {S18, 0, &V4948_297_57197, &P4948_295};	/*Property = CssTransforms*/
const Value V4948_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_241 = {S19, 0, &V4948_241_55801, &P4948_297};	/*Property = CssTransitions*/
const Value V4948_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_298 = {S20, 0, &V4948_298_57208, &P4948_241};	/*Property = CssUI*/
const Value V4948_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_300 = {S21, 0, &V4948_300_57217, &P4948_298};	/*Property = DataSet*/
const Value V4948_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_301 = {S22, 0, &V4948_301_57220, &P4948_300};	/*Property = DataUrl*/
const Value V4948_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_302 = {S23, 0, &V4948_302_57223, &P4948_301};	/*Property = DeviceOrientation*/
const Value V4948_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_304 = {S24, 0, &V4948_304_57229, &P4948_302};	/*Property = FileReader*/
const Value V4948_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_305 = {S25, 0, &V4948_305_57232, &P4948_304};	/*Property = FileSaver*/
const Value V4948_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_306 = {S26, 0, &V4948_306_57238, &P4948_305};	/*Property = FileWriter*/
const Value V4948_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_307 = {S27, 0, &V4948_307_57241, &P4948_306};	/*Property = FormData*/
const Value V4948_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_309 = {S28, 0, &V4948_309_57247, &P4948_307};	/*Property = Fullscreen*/
const Value V4948_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_310 = {S29, 0, &V4948_310_57250, &P4948_309};	/*Property = GeoLocation*/
const Value V4948_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_312 = {S30, 0, &V4948_312_57256, &P4948_310};	/*Property = History*/
const Value V4948_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_314 = {S31, 0, &V4948_314_57262, &P4948_312};	/*Property = Html5*/
const Value V4948_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_313 = {S32, 0, &V4948_313_57259, &P4948_314};	/*Property = Html-Media-Capture*/
const Value V4948_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_315 = {S33, 0, &V4948_315_57265, &P4948_313};	/*Property = Iframe*/
const Value V4948_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_316 = {S34, 0, &V4948_316_57268, &P4948_315};	/*Property = IndexedDB*/
const Value V4948_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_317 = {S35, 0, &V4948_317_57271, &P4948_316};	/*Property = Json*/
const Value V4948_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_15 = {S37, 0, &V4948_15_72, &P4948_317};	/*Property = LayoutEngine*/
const Value V4948_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_344 = {S38, 0, &V4948_344_57274, &P4948_15};	/*Property = Masking*/
const Value V4948_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_320 = {S39, 0, &V4948_320_57292, &P4948_344};	/*Property = PostMessage*/
const Value V4948_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_321 = {S40, 0, &V4948_321_57295, &P4948_320};	/*Property = Progress*/
const Value V4948_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_322 = {S41, 0, &V4948_322_57298, &P4948_321};	/*Property = Prompts*/
const Value V4948_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_323 = {S42, 0, &V4948_323_57301, &P4948_322};	/*Property = Selector*/
const Value V4948_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_324 = {S43, 0, &V4948_324_57307, &P4948_323};	/*Property = Svg*/
const Value V4948_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_327 = {S44, 0, &V4948_327_57316, &P4948_324};	/*Property = TouchEvents*/
const Value V4948_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_328 = {S45, 0, &V4948_328_57319, &P4948_327};	/*Property = Track*/
const Value V4948_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_329 = {S46, 0, &V4948_329_57325, &P4948_328};	/*Property = Video*/
const Value V4948_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_330 = {S47, 0, &V4948_330_57328, &P4948_329};	/*Property = Viewport*/
const Value V4948_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_332 = {S48, 0, &V4948_332_57340, &P4948_330};	/*Property = WebWorkers*/
const Value V4948_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4948_333 = {S49, 0, &V4948_333_57343, &P4948_332};	/*Property = Xhr2*/
const Profile P4948 = {"4948", &P4948_333};	/*Profile ID - 4948*/
const Value V11631_1_1 = {S2, NULL};	/*Value = True*/
const Property P11631_1 = {S50, 0, &V11631_1_1, NULL};	/*Property = IsMobile*/
const Value V11631_5_31826 = {S75, NULL};	/*Value = 260*/
const Property P11631_5 = {S52, 0, &V11631_5_31826, &P11631_1};	/*Property = ScreenPixelsHeight*/
const Value V11631_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11631_4 = {S54, 0, &V11631_4_1224, &P11631_5};	/*Property = ScreenPixelsWidth*/
const Profile P11631 = {"11631", &P11631_4};	/*Profile ID - 11631*/
const Value V3412_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_279 = {S1, 0, &V3412_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3412_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_282 = {S3, 0, &V3412_282_57155, &P3412_279};	/*Property = BlobBuilder*/
const Value V3412_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_350 = {S4, 0, &V3412_350_57446, &P3412_282};	/*Property = Canvas*/
const Value V3412_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_283 = {S5, 0, &V3412_283_57161, &P3412_350};	/*Property = CssBackground*/
const Value V3412_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_284 = {S6, 0, &V3412_284_57164, &P3412_283};	/*Property = CssBorderImage*/
const Value V3412_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_285 = {S7, 0, &V3412_285_57167, &P3412_284};	/*Property = CssCanvas*/
const Value V3412_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_286 = {S8, 0, &V3412_286_57170, &P3412_285};	/*Property = CssColor*/
const Value V3412_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_237 = {S9, 0, &V3412_237_55786, &P3412_286};	/*Property = CssColumn*/
const Value V3412_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_287 = {S10, 0, &V3412_287_57173, &P3412_237};	/*Property = CssFlexbox*/
const Value V3412_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_288 = {S11, 0, &V3412_288_57176, &P3412_287};	/*Property = CssFont*/
const Value V3412_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_289 = {S12, 0, &V3412_289_86899, &P3412_288};	/*Property = CssImages*/
const Value V3412_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_291 = {S13, 0, &V3412_291_57182, &P3412_289};	/*Property = CssMediaQueries*/
const Value V3412_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_292 = {S14, 0, &V3412_292_57185, &P3412_291};	/*Property = CssMinMax*/
const Value V3412_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_293 = {S15, 0, &V3412_293_57188, &P3412_292};	/*Property = CssOverflow*/
const Value V3412_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_294 = {S16, 0, &V3412_294_57191, &P3412_293};	/*Property = CssPosition*/
const Value V3412_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_295 = {S17, 0, &V3412_295_57194, &P3412_294};	/*Property = CssText*/
const Value V3412_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_297 = {S18, 0, &V3412_297_57197, &P3412_295};	/*Property = CssTransforms*/
const Value V3412_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_241 = {S19, 0, &V3412_241_55801, &P3412_297};	/*Property = CssTransitions*/
const Value V3412_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_298 = {S20, 0, &V3412_298_57208, &P3412_241};	/*Property = CssUI*/
const Value V3412_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_300 = {S21, 0, &V3412_300_57217, &P3412_298};	/*Property = DataSet*/
const Value V3412_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_301 = {S22, 0, &V3412_301_57220, &P3412_300};	/*Property = DataUrl*/
const Value V3412_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_302 = {S23, 0, &V3412_302_57223, &P3412_301};	/*Property = DeviceOrientation*/
const Value V3412_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_304 = {S24, 0, &V3412_304_57229, &P3412_302};	/*Property = FileReader*/
const Value V3412_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_305 = {S25, 0, &V3412_305_57232, &P3412_304};	/*Property = FileSaver*/
const Value V3412_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_306 = {S26, 0, &V3412_306_57238, &P3412_305};	/*Property = FileWriter*/
const Value V3412_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_307 = {S27, 0, &V3412_307_57241, &P3412_306};	/*Property = FormData*/
const Value V3412_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_309 = {S28, 0, &V3412_309_57247, &P3412_307};	/*Property = Fullscreen*/
const Value V3412_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_310 = {S29, 0, &V3412_310_57250, &P3412_309};	/*Property = GeoLocation*/
const Value V3412_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_312 = {S30, 0, &V3412_312_57256, &P3412_310};	/*Property = History*/
const Value V3412_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_314 = {S31, 0, &V3412_314_57262, &P3412_312};	/*Property = Html5*/
const Value V3412_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_313 = {S32, 0, &V3412_313_57259, &P3412_314};	/*Property = Html-Media-Capture*/
const Value V3412_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_315 = {S33, 0, &V3412_315_57265, &P3412_313};	/*Property = Iframe*/
const Value V3412_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_316 = {S34, 0, &V3412_316_57268, &P3412_315};	/*Property = IndexedDB*/
const Value V3412_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_317 = {S35, 0, &V3412_317_57271, &P3412_316};	/*Property = Json*/
const Value V3412_15_43264 = {S59, NULL};	/*Value = Mango*/
const Property P3412_15 = {S37, 0, &V3412_15_43264, &P3412_317};	/*Property = LayoutEngine*/
const Value V3412_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_344 = {S38, 0, &V3412_344_57274, &P3412_15};	/*Property = Masking*/
const Value V3412_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_320 = {S39, 0, &V3412_320_57292, &P3412_344};	/*Property = PostMessage*/
const Value V3412_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_321 = {S40, 0, &V3412_321_57295, &P3412_320};	/*Property = Progress*/
const Value V3412_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_322 = {S41, 0, &V3412_322_57298, &P3412_321};	/*Property = Prompts*/
const Value V3412_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_323 = {S42, 0, &V3412_323_57301, &P3412_322};	/*Property = Selector*/
const Value V3412_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_324 = {S43, 0, &V3412_324_57307, &P3412_323};	/*Property = Svg*/
const Value V3412_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_327 = {S44, 0, &V3412_327_57316, &P3412_324};	/*Property = TouchEvents*/
const Value V3412_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_328 = {S45, 0, &V3412_328_57319, &P3412_327};	/*Property = Track*/
const Value V3412_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_329 = {S46, 0, &V3412_329_57325, &P3412_328};	/*Property = Video*/
const Value V3412_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_330 = {S47, 0, &V3412_330_57328, &P3412_329};	/*Property = Viewport*/
const Value V3412_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_332 = {S48, 0, &V3412_332_57340, &P3412_330};	/*Property = WebWorkers*/
const Value V3412_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3412_333 = {S49, 0, &V3412_333_57343, &P3412_332};	/*Property = Xhr2*/
const Profile P3412 = {"3412", &P3412_333};	/*Profile ID - 3412*/
const Profile P5027 = {"5027", NULL};	/*Profile ID - 5027*/
const Value V11633_1_1 = {S2, NULL};	/*Value = True*/
const Property P11633_1 = {S50, 0, &V11633_1_1, NULL};	/*Property = IsMobile*/
const Value V11633_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11633_5 = {S52, 0, &V11633_5_1222, &P11633_1};	/*Property = ScreenPixelsHeight*/
const Value V11633_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11633_4 = {S54, 0, &V11633_4_1224, &P11633_5};	/*Property = ScreenPixelsWidth*/
const Profile P11633 = {"11633", &P11633_4};	/*Profile ID - 11633*/
const Profile P5035 = {"5035", NULL};	/*Profile ID - 5035*/
const Value V17470_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17470_279 = {S1, 0, &V17470_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17470_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17470_282 = {S3, 0, &V17470_282_57154, &P17470_279};	/*Property = BlobBuilder*/
const Value V17470_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17470_350 = {S4, 0, &V17470_350_57444, &P17470_282};	/*Property = Canvas*/
const Value V17470_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17470_283 = {S5, 0, &V17470_283_57159, &P17470_350};	/*Property = CssBackground*/
const Value V17470_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17470_284 = {S6, 0, &V17470_284_57163, &P17470_283};	/*Property = CssBorderImage*/
const Value V17470_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17470_285 = {S7, 0, &V17470_285_57166, &P17470_284};	/*Property = CssCanvas*/
const Value V17470_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17470_286 = {S8, 0, &V17470_286_57168, &P17470_285};	/*Property = CssColor*/
const Value V17470_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17470_237 = {S9, 0, &V17470_237_55786, &P17470_286};	/*Property = CssColumn*/
const Value V17470_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17470_287 = {S10, 0, &V17470_287_57172, &P17470_237};	/*Property = CssFlexbox*/
const Value V17470_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17470_288 = {S11, 0, &V17470_288_57175, &P17470_287};	/*Property = CssFont*/
const Value V17470_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17470_289 = {S12, 0, &V17470_289_57759, &P17470_288};	/*Property = CssImages*/
const Value V17470_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17470_291 = {S13, 0, &V17470_291_57181, &P17470_289};	/*Property = CssMediaQueries*/
const Value V17470_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17470_292 = {S14, 0, &V17470_292_57183, &P17470_291};	/*Property = CssMinMax*/
const Value V17470_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17470_293 = {S15, 0, &V17470_293_57187, &P17470_292};	/*Property = CssOverflow*/
const Value V17470_294_57190 = {S0, NULL};	/*Value = False*/
const Property P17470_294 = {S16, 0, &V17470_294_57190, &P17470_293};	/*Property = CssPosition*/
const Value V17470_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17470_295 = {S17, 0, &V17470_295_57192, &P17470_294};	/*Property = CssText*/
const Value V17470_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17470_297 = {S18, 0, &V17470_297_57195, &P17470_295};	/*Property = CssTransforms*/
const Value V17470_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17470_241 = {S19, 0, &V17470_241_55799, &P17470_297};	/*Property = CssTransitions*/
const Value V17470_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17470_298 = {S20, 0, &V17470_298_57206, &P17470_241};	/*Property = CssUI*/
const Value V17470_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17470_300 = {S21, 0, &V17470_300_57216, &P17470_298};	/*Property = DataSet*/
const Value V17470_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17470_301 = {S22, 0, &V17470_301_57218, &P17470_300};	/*Property = DataUrl*/
const Value V17470_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17470_302 = {S23, 0, &V17470_302_57222, &P17470_301};	/*Property = DeviceOrientation*/
const Value V17470_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17470_304 = {S24, 0, &V17470_304_57228, &P17470_302};	/*Property = FileReader*/
const Value V17470_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17470_305 = {S25, 0, &V17470_305_57231, &P17470_304};	/*Property = FileSaver*/
const Value V17470_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17470_306 = {S26, 0, &V17470_306_57237, &P17470_305};	/*Property = FileWriter*/
const Value V17470_307_57240 = {S0, NULL};	/*Value = False*/
const Property P17470_307 = {S27, 0, &V17470_307_57240, &P17470_306};	/*Property = FormData*/
const Value V17470_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17470_309 = {S28, 0, &V17470_309_57246, &P17470_307};	/*Property = Fullscreen*/
const Value V17470_310_57249 = {S0, NULL};	/*Value = False*/
const Property P17470_310 = {S29, 0, &V17470_310_57249, &P17470_309};	/*Property = GeoLocation*/
const Value V17470_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17470_312 = {S30, 0, &V17470_312_57255, &P17470_310};	/*Property = History*/
const Value V17470_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17470_314 = {S31, 0, &V17470_314_57261, &P17470_312};	/*Property = Html5*/
const Value V17470_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17470_313 = {S32, 0, &V17470_313_57258, &P17470_314};	/*Property = Html-Media-Capture*/
const Value V17470_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17470_315 = {S33, 0, &V17470_315_57264, &P17470_313};	/*Property = Iframe*/
const Value V17470_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17470_316 = {S34, 0, &V17470_316_57267, &P17470_315};	/*Property = IndexedDB*/
const Value V17470_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17470_317 = {S35, 0, &V17470_317_57269, &P17470_316};	/*Property = Json*/
const Value V17470_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17470_15 = {S37, 0, &V17470_15_28, &P17470_317};	/*Property = LayoutEngine*/
const Value V17470_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17470_344 = {S38, 0, &V17470_344_57272, &P17470_15};	/*Property = Masking*/
const Value V17470_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17470_320 = {S39, 0, &V17470_320_57290, &P17470_344};	/*Property = PostMessage*/
const Value V17470_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17470_321 = {S40, 0, &V17470_321_57293, &P17470_320};	/*Property = Progress*/
const Value V17470_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17470_322 = {S41, 0, &V17470_322_57296, &P17470_321};	/*Property = Prompts*/
const Value V17470_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17470_323 = {S42, 0, &V17470_323_57299, &P17470_322};	/*Property = Selector*/
const Value V17470_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17470_324 = {S43, 0, &V17470_324_57306, &P17470_323};	/*Property = Svg*/
const Value V17470_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17470_327 = {S44, 0, &V17470_327_57315, &P17470_324};	/*Property = TouchEvents*/
const Value V17470_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17470_328 = {S45, 0, &V17470_328_57318, &P17470_327};	/*Property = Track*/
const Value V17470_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17470_329 = {S46, 0, &V17470_329_57323, &P17470_328};	/*Property = Video*/
const Value V17470_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17470_330 = {S47, 0, &V17470_330_57326, &P17470_329};	/*Property = Viewport*/
const Value V17470_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17470_332 = {S48, 0, &V17470_332_57339, &P17470_330};	/*Property = WebWorkers*/
const Value V17470_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17470_333 = {S49, 0, &V17470_333_57342, &P17470_332};	/*Property = Xhr2*/
const Profile P17470 = {"17470", &P17470_333};	/*Profile ID - 17470*/
const Value V19968_1_1 = {S2, NULL};	/*Value = True*/
const Property P19968_1 = {S50, 0, &V19968_1_1, NULL};	/*Property = IsMobile*/
const Value V19968_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P19968_5 = {S52, 0, &V19968_5_1222, &P19968_1};	/*Property = ScreenPixelsHeight*/
const Value V19968_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P19968_4 = {S54, 0, &V19968_4_1224, &P19968_5};	/*Property = ScreenPixelsWidth*/
const Profile P19968 = {"19968", &P19968_4};	/*Profile ID - 19968*/
const Value V13163_1_1 = {S2, NULL};	/*Value = True*/
const Property P13163_1 = {S50, 0, &V13163_1_1, NULL};	/*Property = IsMobile*/
const Value V13163_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P13163_5 = {S52, 0, &V13163_5_1275, &P13163_1};	/*Property = ScreenPixelsHeight*/
const Value V13163_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P13163_4 = {S54, 0, &V13163_4_1278, &P13163_5};	/*Property = ScreenPixelsWidth*/
const Profile P13163 = {"13163", &P13163_4};	/*Profile ID - 13163*/
const Value V3600_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_279 = {S1, 0, &V3600_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3600_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_282 = {S3, 0, &V3600_282_57155, &P3600_279};	/*Property = BlobBuilder*/
const Value V3600_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_350 = {S4, 0, &V3600_350_57446, &P3600_282};	/*Property = Canvas*/
const Value V3600_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_283 = {S5, 0, &V3600_283_57161, &P3600_350};	/*Property = CssBackground*/
const Value V3600_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_284 = {S6, 0, &V3600_284_57164, &P3600_283};	/*Property = CssBorderImage*/
const Value V3600_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_285 = {S7, 0, &V3600_285_57167, &P3600_284};	/*Property = CssCanvas*/
const Value V3600_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_286 = {S8, 0, &V3600_286_57170, &P3600_285};	/*Property = CssColor*/
const Value V3600_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_237 = {S9, 0, &V3600_237_55786, &P3600_286};	/*Property = CssColumn*/
const Value V3600_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_287 = {S10, 0, &V3600_287_57173, &P3600_237};	/*Property = CssFlexbox*/
const Value V3600_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_288 = {S11, 0, &V3600_288_57176, &P3600_287};	/*Property = CssFont*/
const Value V3600_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_289 = {S12, 0, &V3600_289_86899, &P3600_288};	/*Property = CssImages*/
const Value V3600_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_291 = {S13, 0, &V3600_291_57182, &P3600_289};	/*Property = CssMediaQueries*/
const Value V3600_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_292 = {S14, 0, &V3600_292_57185, &P3600_291};	/*Property = CssMinMax*/
const Value V3600_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_293 = {S15, 0, &V3600_293_57188, &P3600_292};	/*Property = CssOverflow*/
const Value V3600_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_294 = {S16, 0, &V3600_294_57191, &P3600_293};	/*Property = CssPosition*/
const Value V3600_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_295 = {S17, 0, &V3600_295_57194, &P3600_294};	/*Property = CssText*/
const Value V3600_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_297 = {S18, 0, &V3600_297_57197, &P3600_295};	/*Property = CssTransforms*/
const Value V3600_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_241 = {S19, 0, &V3600_241_55801, &P3600_297};	/*Property = CssTransitions*/
const Value V3600_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_298 = {S20, 0, &V3600_298_57208, &P3600_241};	/*Property = CssUI*/
const Value V3600_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_300 = {S21, 0, &V3600_300_57217, &P3600_298};	/*Property = DataSet*/
const Value V3600_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_301 = {S22, 0, &V3600_301_57220, &P3600_300};	/*Property = DataUrl*/
const Value V3600_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_302 = {S23, 0, &V3600_302_57223, &P3600_301};	/*Property = DeviceOrientation*/
const Value V3600_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_304 = {S24, 0, &V3600_304_57229, &P3600_302};	/*Property = FileReader*/
const Value V3600_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_305 = {S25, 0, &V3600_305_57232, &P3600_304};	/*Property = FileSaver*/
const Value V3600_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_306 = {S26, 0, &V3600_306_57238, &P3600_305};	/*Property = FileWriter*/
const Value V3600_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_307 = {S27, 0, &V3600_307_57241, &P3600_306};	/*Property = FormData*/
const Value V3600_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_309 = {S28, 0, &V3600_309_57247, &P3600_307};	/*Property = Fullscreen*/
const Value V3600_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_310 = {S29, 0, &V3600_310_57250, &P3600_309};	/*Property = GeoLocation*/
const Value V3600_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_312 = {S30, 0, &V3600_312_57256, &P3600_310};	/*Property = History*/
const Value V3600_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_314 = {S31, 0, &V3600_314_57262, &P3600_312};	/*Property = Html5*/
const Value V3600_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_313 = {S32, 0, &V3600_313_57259, &P3600_314};	/*Property = Html-Media-Capture*/
const Value V3600_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_315 = {S33, 0, &V3600_315_57265, &P3600_313};	/*Property = Iframe*/
const Value V3600_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_316 = {S34, 0, &V3600_316_57268, &P3600_315};	/*Property = IndexedDB*/
const Value V3600_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_317 = {S35, 0, &V3600_317_57271, &P3600_316};	/*Property = Json*/
const Value V3600_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_15 = {S37, 0, &V3600_15_72, &P3600_317};	/*Property = LayoutEngine*/
const Value V3600_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_344 = {S38, 0, &V3600_344_57274, &P3600_15};	/*Property = Masking*/
const Value V3600_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_320 = {S39, 0, &V3600_320_57292, &P3600_344};	/*Property = PostMessage*/
const Value V3600_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_321 = {S40, 0, &V3600_321_57295, &P3600_320};	/*Property = Progress*/
const Value V3600_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_322 = {S41, 0, &V3600_322_57298, &P3600_321};	/*Property = Prompts*/
const Value V3600_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_323 = {S42, 0, &V3600_323_57301, &P3600_322};	/*Property = Selector*/
const Value V3600_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_324 = {S43, 0, &V3600_324_57307, &P3600_323};	/*Property = Svg*/
const Value V3600_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_327 = {S44, 0, &V3600_327_57316, &P3600_324};	/*Property = TouchEvents*/
const Value V3600_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_328 = {S45, 0, &V3600_328_57319, &P3600_327};	/*Property = Track*/
const Value V3600_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_329 = {S46, 0, &V3600_329_57325, &P3600_328};	/*Property = Video*/
const Value V3600_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_330 = {S47, 0, &V3600_330_57328, &P3600_329};	/*Property = Viewport*/
const Value V3600_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_332 = {S48, 0, &V3600_332_57340, &P3600_330};	/*Property = WebWorkers*/
const Value V3600_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3600_333 = {S49, 0, &V3600_333_57343, &P3600_332};	/*Property = Xhr2*/
const Profile P3600 = {"3600", &P3600_333};	/*Profile ID - 3600*/
const Value V13216_1_1 = {S2, NULL};	/*Value = True*/
const Property P13216_1 = {S50, 0, &V13216_1_1, NULL};	/*Property = IsMobile*/
const Value V13216_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13216_5 = {S52, 0, &V13216_5_1233, &P13216_1};	/*Property = ScreenPixelsHeight*/
const Value V13216_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13216_4 = {S54, 0, &V13216_4_1235, &P13216_5};	/*Property = ScreenPixelsWidth*/
const Profile P13216 = {"13216", &P13216_4};	/*Profile ID - 13216*/
const Value V11636_1_1 = {S2, NULL};	/*Value = True*/
const Property P11636_1 = {S50, 0, &V11636_1_1, NULL};	/*Property = IsMobile*/
const Value V11636_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11636_5 = {S52, 0, &V11636_5_1222, &P11636_1};	/*Property = ScreenPixelsHeight*/
const Value V11636_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11636_4 = {S54, 0, &V11636_4_1224, &P11636_5};	/*Property = ScreenPixelsWidth*/
const Profile P11636 = {"11636", &P11636_4};	/*Profile ID - 11636*/
const Value V14234_1_1 = {S2, NULL};	/*Value = True*/
const Property P14234_1 = {S50, 0, &V14234_1_1, NULL};	/*Property = IsMobile*/
const Value V14234_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14234_5 = {S52, 0, &V14234_5_1294, &P14234_1};	/*Property = ScreenPixelsHeight*/
const Value V14234_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14234_4 = {S54, 0, &V14234_4_1296, &P14234_5};	/*Property = ScreenPixelsWidth*/
const Profile P14234 = {"14234", &P14234_4};	/*Profile ID - 14234*/
const Value V12301_1_1 = {S2, NULL};	/*Value = True*/
const Property P12301_1 = {S50, 0, &V12301_1_1, NULL};	/*Property = IsMobile*/
const Value V12301_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12301_5 = {S52, 0, &V12301_5_1233, &P12301_1};	/*Property = ScreenPixelsHeight*/
const Value V12301_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12301_4 = {S54, 0, &V12301_4_1235, &P12301_5};	/*Property = ScreenPixelsWidth*/
const Profile P12301 = {"12301", &P12301_4};	/*Profile ID - 12301*/
const Value V11903_1_1 = {S2, NULL};	/*Value = True*/
const Property P11903_1 = {S50, 0, &V11903_1_1, NULL};	/*Property = IsMobile*/
const Value V11903_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11903_5 = {S52, 0, &V11903_5_1293, &P11903_1};	/*Property = ScreenPixelsHeight*/
const Value V11903_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11903_4 = {S54, 0, &V11903_4_1296, &P11903_5};	/*Property = ScreenPixelsWidth*/
const Profile P11903 = {"11903", &P11903_4};	/*Profile ID - 11903*/
const Value V17170_1_1 = {S2, NULL};	/*Value = True*/
const Property P17170_1 = {S50, 0, &V17170_1_1, NULL};	/*Property = IsMobile*/
const Value V17170_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17170_5 = {S52, 0, &V17170_5_1293, &P17170_1};	/*Property = ScreenPixelsHeight*/
const Value V17170_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17170_4 = {S54, 0, &V17170_4_1296, &P17170_5};	/*Property = ScreenPixelsWidth*/
const Profile P17170 = {"17170", &P17170_4};	/*Profile ID - 17170*/
const Profile P19950 = {"19950", NULL};	/*Profile ID - 19950*/
const Value V11905_1_1 = {S2, NULL};	/*Value = True*/
const Property P11905_1 = {S50, 0, &V11905_1_1, NULL};	/*Property = IsMobile*/
const Value V11905_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11905_5 = {S52, 0, &V11905_5_1222, &P11905_1};	/*Property = ScreenPixelsHeight*/
const Value V11905_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11905_4 = {S54, 0, &V11905_4_1224, &P11905_5};	/*Property = ScreenPixelsWidth*/
const Profile P11905 = {"11905", &P11905_4};	/*Profile ID - 11905*/
const Profile P5033 = {"5033", NULL};	/*Profile ID - 5033*/
const Value V14880_1_1 = {S2, NULL};	/*Value = True*/
const Property P14880_1 = {S50, 0, &V14880_1_1, NULL};	/*Property = IsMobile*/
const Value V14880_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14880_5 = {S52, 0, &V14880_5_1380, &P14880_1};	/*Property = ScreenPixelsHeight*/
const Value V14880_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14880_4 = {S54, 0, &V14880_4_1279, &P14880_5};	/*Property = ScreenPixelsWidth*/
const Profile P14880 = {"14880", &P14880_4};	/*Profile ID - 14880*/
const Value V3417_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_279 = {S1, 0, &V3417_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3417_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_282 = {S3, 0, &V3417_282_57155, &P3417_279};	/*Property = BlobBuilder*/
const Value V3417_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_350 = {S4, 0, &V3417_350_57446, &P3417_282};	/*Property = Canvas*/
const Value V3417_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_283 = {S5, 0, &V3417_283_57161, &P3417_350};	/*Property = CssBackground*/
const Value V3417_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_284 = {S6, 0, &V3417_284_57164, &P3417_283};	/*Property = CssBorderImage*/
const Value V3417_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_285 = {S7, 0, &V3417_285_57167, &P3417_284};	/*Property = CssCanvas*/
const Value V3417_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_286 = {S8, 0, &V3417_286_57170, &P3417_285};	/*Property = CssColor*/
const Value V3417_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_237 = {S9, 0, &V3417_237_55786, &P3417_286};	/*Property = CssColumn*/
const Value V3417_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_287 = {S10, 0, &V3417_287_57173, &P3417_237};	/*Property = CssFlexbox*/
const Value V3417_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_288 = {S11, 0, &V3417_288_57176, &P3417_287};	/*Property = CssFont*/
const Value V3417_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_289 = {S12, 0, &V3417_289_86899, &P3417_288};	/*Property = CssImages*/
const Value V3417_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_291 = {S13, 0, &V3417_291_57182, &P3417_289};	/*Property = CssMediaQueries*/
const Value V3417_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_292 = {S14, 0, &V3417_292_57185, &P3417_291};	/*Property = CssMinMax*/
const Value V3417_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_293 = {S15, 0, &V3417_293_57188, &P3417_292};	/*Property = CssOverflow*/
const Value V3417_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_294 = {S16, 0, &V3417_294_57191, &P3417_293};	/*Property = CssPosition*/
const Value V3417_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_295 = {S17, 0, &V3417_295_57194, &P3417_294};	/*Property = CssText*/
const Value V3417_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_297 = {S18, 0, &V3417_297_57197, &P3417_295};	/*Property = CssTransforms*/
const Value V3417_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_241 = {S19, 0, &V3417_241_55801, &P3417_297};	/*Property = CssTransitions*/
const Value V3417_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_298 = {S20, 0, &V3417_298_57208, &P3417_241};	/*Property = CssUI*/
const Value V3417_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_300 = {S21, 0, &V3417_300_57217, &P3417_298};	/*Property = DataSet*/
const Value V3417_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_301 = {S22, 0, &V3417_301_57220, &P3417_300};	/*Property = DataUrl*/
const Value V3417_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_302 = {S23, 0, &V3417_302_57223, &P3417_301};	/*Property = DeviceOrientation*/
const Value V3417_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_304 = {S24, 0, &V3417_304_57229, &P3417_302};	/*Property = FileReader*/
const Value V3417_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_305 = {S25, 0, &V3417_305_57232, &P3417_304};	/*Property = FileSaver*/
const Value V3417_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_306 = {S26, 0, &V3417_306_57238, &P3417_305};	/*Property = FileWriter*/
const Value V3417_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_307 = {S27, 0, &V3417_307_57241, &P3417_306};	/*Property = FormData*/
const Value V3417_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_309 = {S28, 0, &V3417_309_57247, &P3417_307};	/*Property = Fullscreen*/
const Value V3417_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_310 = {S29, 0, &V3417_310_57250, &P3417_309};	/*Property = GeoLocation*/
const Value V3417_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_312 = {S30, 0, &V3417_312_57256, &P3417_310};	/*Property = History*/
const Value V3417_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_314 = {S31, 0, &V3417_314_57262, &P3417_312};	/*Property = Html5*/
const Value V3417_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_313 = {S32, 0, &V3417_313_57259, &P3417_314};	/*Property = Html-Media-Capture*/
const Value V3417_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_315 = {S33, 0, &V3417_315_57265, &P3417_313};	/*Property = Iframe*/
const Value V3417_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_316 = {S34, 0, &V3417_316_57268, &P3417_315};	/*Property = IndexedDB*/
const Value V3417_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_317 = {S35, 0, &V3417_317_57271, &P3417_316};	/*Property = Json*/
const Value V3417_15_43264 = {S59, NULL};	/*Value = Mango*/
const Property P3417_15 = {S37, 0, &V3417_15_43264, &P3417_317};	/*Property = LayoutEngine*/
const Value V3417_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_344 = {S38, 0, &V3417_344_57274, &P3417_15};	/*Property = Masking*/
const Value V3417_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_320 = {S39, 0, &V3417_320_57292, &P3417_344};	/*Property = PostMessage*/
const Value V3417_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_321 = {S40, 0, &V3417_321_57295, &P3417_320};	/*Property = Progress*/
const Value V3417_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_322 = {S41, 0, &V3417_322_57298, &P3417_321};	/*Property = Prompts*/
const Value V3417_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_323 = {S42, 0, &V3417_323_57301, &P3417_322};	/*Property = Selector*/
const Value V3417_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_324 = {S43, 0, &V3417_324_57307, &P3417_323};	/*Property = Svg*/
const Value V3417_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_327 = {S44, 0, &V3417_327_57316, &P3417_324};	/*Property = TouchEvents*/
const Value V3417_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_328 = {S45, 0, &V3417_328_57319, &P3417_327};	/*Property = Track*/
const Value V3417_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_329 = {S46, 0, &V3417_329_57325, &P3417_328};	/*Property = Video*/
const Value V3417_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_330 = {S47, 0, &V3417_330_57328, &P3417_329};	/*Property = Viewport*/
const Value V3417_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_332 = {S48, 0, &V3417_332_57340, &P3417_330};	/*Property = WebWorkers*/
const Value V3417_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3417_333 = {S49, 0, &V3417_333_57343, &P3417_332};	/*Property = Xhr2*/
const Profile P3417 = {"3417", &P3417_333};	/*Profile ID - 3417*/
const Value V11907_1_1 = {S2, NULL};	/*Value = True*/
const Property P11907_1 = {S50, 0, &V11907_1_1, NULL};	/*Property = IsMobile*/
const Value V11907_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11907_5 = {S52, 0, &V11907_5_1222, &P11907_1};	/*Property = ScreenPixelsHeight*/
const Value V11907_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11907_4 = {S54, 0, &V11907_4_1224, &P11907_5};	/*Property = ScreenPixelsWidth*/
const Profile P11907 = {"11907", &P11907_4};	/*Profile ID - 11907*/
const Value V3502_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_279 = {S1, 0, &V3502_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3502_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_282 = {S3, 0, &V3502_282_57155, &P3502_279};	/*Property = BlobBuilder*/
const Value V3502_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_350 = {S4, 0, &V3502_350_57446, &P3502_282};	/*Property = Canvas*/
const Value V3502_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_283 = {S5, 0, &V3502_283_57161, &P3502_350};	/*Property = CssBackground*/
const Value V3502_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_284 = {S6, 0, &V3502_284_57164, &P3502_283};	/*Property = CssBorderImage*/
const Value V3502_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_285 = {S7, 0, &V3502_285_57167, &P3502_284};	/*Property = CssCanvas*/
const Value V3502_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_286 = {S8, 0, &V3502_286_57170, &P3502_285};	/*Property = CssColor*/
const Value V3502_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_237 = {S9, 0, &V3502_237_55786, &P3502_286};	/*Property = CssColumn*/
const Value V3502_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_287 = {S10, 0, &V3502_287_57173, &P3502_237};	/*Property = CssFlexbox*/
const Value V3502_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_288 = {S11, 0, &V3502_288_57176, &P3502_287};	/*Property = CssFont*/
const Value V3502_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_289 = {S12, 0, &V3502_289_86899, &P3502_288};	/*Property = CssImages*/
const Value V3502_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_291 = {S13, 0, &V3502_291_57182, &P3502_289};	/*Property = CssMediaQueries*/
const Value V3502_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_292 = {S14, 0, &V3502_292_57185, &P3502_291};	/*Property = CssMinMax*/
const Value V3502_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_293 = {S15, 0, &V3502_293_57188, &P3502_292};	/*Property = CssOverflow*/
const Value V3502_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_294 = {S16, 0, &V3502_294_57191, &P3502_293};	/*Property = CssPosition*/
const Value V3502_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_295 = {S17, 0, &V3502_295_57194, &P3502_294};	/*Property = CssText*/
const Value V3502_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_297 = {S18, 0, &V3502_297_57197, &P3502_295};	/*Property = CssTransforms*/
const Value V3502_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_241 = {S19, 0, &V3502_241_55801, &P3502_297};	/*Property = CssTransitions*/
const Value V3502_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_298 = {S20, 0, &V3502_298_57208, &P3502_241};	/*Property = CssUI*/
const Value V3502_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_300 = {S21, 0, &V3502_300_57217, &P3502_298};	/*Property = DataSet*/
const Value V3502_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_301 = {S22, 0, &V3502_301_57220, &P3502_300};	/*Property = DataUrl*/
const Value V3502_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_302 = {S23, 0, &V3502_302_57223, &P3502_301};	/*Property = DeviceOrientation*/
const Value V3502_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_304 = {S24, 0, &V3502_304_57229, &P3502_302};	/*Property = FileReader*/
const Value V3502_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_305 = {S25, 0, &V3502_305_57232, &P3502_304};	/*Property = FileSaver*/
const Value V3502_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_306 = {S26, 0, &V3502_306_57238, &P3502_305};	/*Property = FileWriter*/
const Value V3502_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_307 = {S27, 0, &V3502_307_57241, &P3502_306};	/*Property = FormData*/
const Value V3502_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_309 = {S28, 0, &V3502_309_57247, &P3502_307};	/*Property = Fullscreen*/
const Value V3502_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_310 = {S29, 0, &V3502_310_57250, &P3502_309};	/*Property = GeoLocation*/
const Value V3502_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_312 = {S30, 0, &V3502_312_57256, &P3502_310};	/*Property = History*/
const Value V3502_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_314 = {S31, 0, &V3502_314_57262, &P3502_312};	/*Property = Html5*/
const Value V3502_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_313 = {S32, 0, &V3502_313_57259, &P3502_314};	/*Property = Html-Media-Capture*/
const Value V3502_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_315 = {S33, 0, &V3502_315_57265, &P3502_313};	/*Property = Iframe*/
const Value V3502_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_316 = {S34, 0, &V3502_316_57268, &P3502_315};	/*Property = IndexedDB*/
const Value V3502_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_317 = {S35, 0, &V3502_317_57271, &P3502_316};	/*Property = Json*/
const Value V3502_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3502_15 = {S37, 0, &V3502_15_28, &P3502_317};	/*Property = LayoutEngine*/
const Value V3502_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_344 = {S38, 0, &V3502_344_57274, &P3502_15};	/*Property = Masking*/
const Value V3502_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_320 = {S39, 0, &V3502_320_57292, &P3502_344};	/*Property = PostMessage*/
const Value V3502_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_321 = {S40, 0, &V3502_321_57295, &P3502_320};	/*Property = Progress*/
const Value V3502_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_322 = {S41, 0, &V3502_322_57298, &P3502_321};	/*Property = Prompts*/
const Value V3502_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_323 = {S42, 0, &V3502_323_57301, &P3502_322};	/*Property = Selector*/
const Value V3502_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_324 = {S43, 0, &V3502_324_57307, &P3502_323};	/*Property = Svg*/
const Value V3502_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_327 = {S44, 0, &V3502_327_57316, &P3502_324};	/*Property = TouchEvents*/
const Value V3502_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_328 = {S45, 0, &V3502_328_57319, &P3502_327};	/*Property = Track*/
const Value V3502_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_329 = {S46, 0, &V3502_329_57325, &P3502_328};	/*Property = Video*/
const Value V3502_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_330 = {S47, 0, &V3502_330_57328, &P3502_329};	/*Property = Viewport*/
const Value V3502_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_332 = {S48, 0, &V3502_332_57340, &P3502_330};	/*Property = WebWorkers*/
const Value V3502_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3502_333 = {S49, 0, &V3502_333_57343, &P3502_332};	/*Property = Xhr2*/
const Profile P3502 = {"3502", &P3502_333};	/*Profile ID - 3502*/
const Value V15197_1_1 = {S2, NULL};	/*Value = True*/
const Property P15197_1 = {S50, 0, &V15197_1_1, NULL};	/*Property = IsMobile*/
const Value V15197_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15197_5 = {S52, 0, &V15197_5_1222, &P15197_1};	/*Property = ScreenPixelsHeight*/
const Value V15197_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15197_4 = {S54, 0, &V15197_4_1224, &P15197_5};	/*Property = ScreenPixelsWidth*/
const Profile P15197 = {"15197", &P15197_4};	/*Profile ID - 15197*/
const Value V11914_1_1 = {S2, NULL};	/*Value = True*/
const Property P11914_1 = {S50, 0, &V11914_1_1, NULL};	/*Property = IsMobile*/
const Value V11914_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11914_5 = {S52, 0, &V11914_5_1293, &P11914_1};	/*Property = ScreenPixelsHeight*/
const Value V11914_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11914_4 = {S54, 0, &V11914_4_1296, &P11914_5};	/*Property = ScreenPixelsWidth*/
const Profile P11914 = {"11914", &P11914_4};	/*Profile ID - 11914*/
const Value V11880_1_1 = {S2, NULL};	/*Value = True*/
const Property P11880_1 = {S50, 0, &V11880_1_1, NULL};	/*Property = IsMobile*/
const Value V11880_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11880_5 = {S52, 0, &V11880_5_1222, &P11880_1};	/*Property = ScreenPixelsHeight*/
const Value V11880_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11880_4 = {S54, 0, &V11880_4_1224, &P11880_5};	/*Property = ScreenPixelsWidth*/
const Profile P11880 = {"11880", &P11880_4};	/*Profile ID - 11880*/
const Value V15200_1_1 = {S2, NULL};	/*Value = True*/
const Property P15200_1 = {S50, 0, &V15200_1_1, NULL};	/*Property = IsMobile*/
const Value V15200_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15200_5 = {S52, 0, &V15200_5_1222, &P15200_1};	/*Property = ScreenPixelsHeight*/
const Value V15200_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15200_4 = {S54, 0, &V15200_4_1224, &P15200_5};	/*Property = ScreenPixelsWidth*/
const Profile P15200 = {"15200", &P15200_4};	/*Profile ID - 15200*/
const Value V17202_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17202_279 = {S1, 0, &V17202_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17202_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17202_282 = {S3, 0, &V17202_282_57154, &P17202_279};	/*Property = BlobBuilder*/
const Value V17202_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17202_350 = {S4, 0, &V17202_350_57444, &P17202_282};	/*Property = Canvas*/
const Value V17202_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17202_283 = {S5, 0, &V17202_283_57159, &P17202_350};	/*Property = CssBackground*/
const Value V17202_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17202_284 = {S6, 0, &V17202_284_57163, &P17202_283};	/*Property = CssBorderImage*/
const Value V17202_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17202_285 = {S7, 0, &V17202_285_57166, &P17202_284};	/*Property = CssCanvas*/
const Value V17202_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17202_286 = {S8, 0, &V17202_286_57168, &P17202_285};	/*Property = CssColor*/
const Value V17202_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17202_237 = {S9, 0, &V17202_237_55784, &P17202_286};	/*Property = CssColumn*/
const Value V17202_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17202_287 = {S10, 0, &V17202_287_57172, &P17202_237};	/*Property = CssFlexbox*/
const Value V17202_288_57174 = {S2, NULL};	/*Value = True*/
const Property P17202_288 = {S11, 0, &V17202_288_57174, &P17202_287};	/*Property = CssFont*/
const Value V17202_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17202_289 = {S12, 0, &V17202_289_57759, &P17202_288};	/*Property = CssImages*/
const Value V17202_291_57180 = {S2, NULL};	/*Value = True*/
const Property P17202_291 = {S13, 0, &V17202_291_57180, &P17202_289};	/*Property = CssMediaQueries*/
const Value V17202_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17202_292 = {S14, 0, &V17202_292_57183, &P17202_291};	/*Property = CssMinMax*/
const Value V17202_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17202_293 = {S15, 0, &V17202_293_57187, &P17202_292};	/*Property = CssOverflow*/
const Value V17202_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17202_294 = {S16, 0, &V17202_294_57189, &P17202_293};	/*Property = CssPosition*/
const Value V17202_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17202_295 = {S17, 0, &V17202_295_57192, &P17202_294};	/*Property = CssText*/
const Value V17202_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17202_297 = {S18, 0, &V17202_297_57195, &P17202_295};	/*Property = CssTransforms*/
const Value V17202_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17202_241 = {S19, 0, &V17202_241_55799, &P17202_297};	/*Property = CssTransitions*/
const Value V17202_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17202_298 = {S20, 0, &V17202_298_57206, &P17202_241};	/*Property = CssUI*/
const Value V17202_300_57215 = {S2, NULL};	/*Value = True*/
const Property P17202_300 = {S21, 0, &V17202_300_57215, &P17202_298};	/*Property = DataSet*/
const Value V17202_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17202_301 = {S22, 0, &V17202_301_57218, &P17202_300};	/*Property = DataUrl*/
const Value V17202_302_57221 = {S2, NULL};	/*Value = True*/
const Property P17202_302 = {S23, 0, &V17202_302_57221, &P17202_301};	/*Property = DeviceOrientation*/
const Value V17202_304_57227 = {S2, NULL};	/*Value = True*/
const Property P17202_304 = {S24, 0, &V17202_304_57227, &P17202_302};	/*Property = FileReader*/
const Value V17202_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17202_305 = {S25, 0, &V17202_305_57231, &P17202_304};	/*Property = FileSaver*/
const Value V17202_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17202_306 = {S26, 0, &V17202_306_57237, &P17202_305};	/*Property = FileWriter*/
const Value V17202_307_57239 = {S2, NULL};	/*Value = True*/
const Property P17202_307 = {S27, 0, &V17202_307_57239, &P17202_306};	/*Property = FormData*/
const Value V17202_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17202_309 = {S28, 0, &V17202_309_57246, &P17202_307};	/*Property = Fullscreen*/
const Value V17202_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17202_310 = {S29, 0, &V17202_310_57248, &P17202_309};	/*Property = GeoLocation*/
const Value V17202_312_57254 = {S2, NULL};	/*Value = True*/
const Property P17202_312 = {S30, 0, &V17202_312_57254, &P17202_310};	/*Property = History*/
const Value V17202_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17202_314 = {S31, 0, &V17202_314_57261, &P17202_312};	/*Property = Html5*/
const Value V17202_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17202_313 = {S32, 0, &V17202_313_57258, &P17202_314};	/*Property = Html-Media-Capture*/
const Value V17202_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17202_315 = {S33, 0, &V17202_315_57264, &P17202_313};	/*Property = Iframe*/
const Value V17202_316_57266 = {S2, NULL};	/*Value = True*/
const Property P17202_316 = {S34, 0, &V17202_316_57266, &P17202_315};	/*Property = IndexedDB*/
const Value V17202_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17202_317 = {S35, 0, &V17202_317_57269, &P17202_316};	/*Property = Json*/
const Value V17202_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P17202_15 = {S37, 0, &V17202_15_29, &P17202_317};	/*Property = LayoutEngine*/
const Value V17202_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17202_344 = {S38, 0, &V17202_344_57272, &P17202_15};	/*Property = Masking*/
const Value V17202_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17202_320 = {S39, 0, &V17202_320_57290, &P17202_344};	/*Property = PostMessage*/
const Value V17202_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17202_321 = {S40, 0, &V17202_321_57293, &P17202_320};	/*Property = Progress*/
const Value V17202_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17202_322 = {S41, 0, &V17202_322_57296, &P17202_321};	/*Property = Prompts*/
const Value V17202_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17202_323 = {S42, 0, &V17202_323_57299, &P17202_322};	/*Property = Selector*/
const Value V17202_324_57305 = {S2, NULL};	/*Value = True*/
const Property P17202_324 = {S43, 0, &V17202_324_57305, &P17202_323};	/*Property = Svg*/
const Value V17202_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17202_327 = {S44, 0, &V17202_327_57315, &P17202_324};	/*Property = TouchEvents*/
const Value V17202_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17202_328 = {S45, 0, &V17202_328_57318, &P17202_327};	/*Property = Track*/
const Value V17202_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17202_329 = {S46, 0, &V17202_329_57323, &P17202_328};	/*Property = Video*/
const Value V17202_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17202_330 = {S47, 0, &V17202_330_57326, &P17202_329};	/*Property = Viewport*/
const Value V17202_332_57338 = {S2, NULL};	/*Value = True*/
const Property P17202_332 = {S48, 0, &V17202_332_57338, &P17202_330};	/*Property = WebWorkers*/
const Value V17202_333_57341 = {S2, NULL};	/*Value = True*/
const Property P17202_333 = {S49, 0, &V17202_333_57341, &P17202_332};	/*Property = Xhr2*/
const Profile P17202 = {"17202", &P17202_333};	/*Profile ID - 17202*/
const Value V16943_1_1 = {S2, NULL};	/*Value = True*/
const Property P16943_1 = {S50, 0, &V16943_1_1, NULL};	/*Property = IsMobile*/
const Value V16943_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16943_5 = {S52, 0, &V16943_5_1233, &P16943_1};	/*Property = ScreenPixelsHeight*/
const Value V16943_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16943_4 = {S54, 0, &V16943_4_1235, &P16943_5};	/*Property = ScreenPixelsWidth*/
const Profile P16943 = {"16943", &P16943_4};	/*Profile ID - 16943*/
const Value V11919_1_1 = {S2, NULL};	/*Value = True*/
const Property P11919_1 = {S50, 0, &V11919_1_1, NULL};	/*Property = IsMobile*/
const Value V11919_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11919_5 = {S52, 0, &V11919_5_1222, &P11919_1};	/*Property = ScreenPixelsHeight*/
const Value V11919_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11919_4 = {S54, 0, &V11919_4_1224, &P11919_5};	/*Property = ScreenPixelsWidth*/
const Profile P11919 = {"11919", &P11919_4};	/*Profile ID - 11919*/
const Value V4248_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_279 = {S1, 0, &V4248_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4248_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_282 = {S3, 0, &V4248_282_57155, &P4248_279};	/*Property = BlobBuilder*/
const Value V4248_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_350 = {S4, 0, &V4248_350_57446, &P4248_282};	/*Property = Canvas*/
const Value V4248_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_283 = {S5, 0, &V4248_283_57161, &P4248_350};	/*Property = CssBackground*/
const Value V4248_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_284 = {S6, 0, &V4248_284_57164, &P4248_283};	/*Property = CssBorderImage*/
const Value V4248_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_285 = {S7, 0, &V4248_285_57167, &P4248_284};	/*Property = CssCanvas*/
const Value V4248_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_286 = {S8, 0, &V4248_286_57170, &P4248_285};	/*Property = CssColor*/
const Value V4248_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_237 = {S9, 0, &V4248_237_55786, &P4248_286};	/*Property = CssColumn*/
const Value V4248_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_287 = {S10, 0, &V4248_287_57173, &P4248_237};	/*Property = CssFlexbox*/
const Value V4248_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_288 = {S11, 0, &V4248_288_57176, &P4248_287};	/*Property = CssFont*/
const Value V4248_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_289 = {S12, 0, &V4248_289_86899, &P4248_288};	/*Property = CssImages*/
const Value V4248_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_291 = {S13, 0, &V4248_291_57182, &P4248_289};	/*Property = CssMediaQueries*/
const Value V4248_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_292 = {S14, 0, &V4248_292_57185, &P4248_291};	/*Property = CssMinMax*/
const Value V4248_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_293 = {S15, 0, &V4248_293_57188, &P4248_292};	/*Property = CssOverflow*/
const Value V4248_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_294 = {S16, 0, &V4248_294_57191, &P4248_293};	/*Property = CssPosition*/
const Value V4248_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_295 = {S17, 0, &V4248_295_57194, &P4248_294};	/*Property = CssText*/
const Value V4248_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_297 = {S18, 0, &V4248_297_57197, &P4248_295};	/*Property = CssTransforms*/
const Value V4248_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_241 = {S19, 0, &V4248_241_55801, &P4248_297};	/*Property = CssTransitions*/
const Value V4248_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_298 = {S20, 0, &V4248_298_57208, &P4248_241};	/*Property = CssUI*/
const Value V4248_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_300 = {S21, 0, &V4248_300_57217, &P4248_298};	/*Property = DataSet*/
const Value V4248_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_301 = {S22, 0, &V4248_301_57220, &P4248_300};	/*Property = DataUrl*/
const Value V4248_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_302 = {S23, 0, &V4248_302_57223, &P4248_301};	/*Property = DeviceOrientation*/
const Value V4248_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_304 = {S24, 0, &V4248_304_57229, &P4248_302};	/*Property = FileReader*/
const Value V4248_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_305 = {S25, 0, &V4248_305_57232, &P4248_304};	/*Property = FileSaver*/
const Value V4248_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_306 = {S26, 0, &V4248_306_57238, &P4248_305};	/*Property = FileWriter*/
const Value V4248_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_307 = {S27, 0, &V4248_307_57241, &P4248_306};	/*Property = FormData*/
const Value V4248_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_309 = {S28, 0, &V4248_309_57247, &P4248_307};	/*Property = Fullscreen*/
const Value V4248_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_310 = {S29, 0, &V4248_310_57250, &P4248_309};	/*Property = GeoLocation*/
const Value V4248_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_312 = {S30, 0, &V4248_312_57256, &P4248_310};	/*Property = History*/
const Value V4248_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_314 = {S31, 0, &V4248_314_57262, &P4248_312};	/*Property = Html5*/
const Value V4248_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_313 = {S32, 0, &V4248_313_57259, &P4248_314};	/*Property = Html-Media-Capture*/
const Value V4248_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_315 = {S33, 0, &V4248_315_57265, &P4248_313};	/*Property = Iframe*/
const Value V4248_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_316 = {S34, 0, &V4248_316_57268, &P4248_315};	/*Property = IndexedDB*/
const Value V4248_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_317 = {S35, 0, &V4248_317_57271, &P4248_316};	/*Property = Json*/
const Value V4248_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P4248_15 = {S37, 0, &V4248_15_28, &P4248_317};	/*Property = LayoutEngine*/
const Value V4248_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_344 = {S38, 0, &V4248_344_57274, &P4248_15};	/*Property = Masking*/
const Value V4248_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_320 = {S39, 0, &V4248_320_57292, &P4248_344};	/*Property = PostMessage*/
const Value V4248_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_321 = {S40, 0, &V4248_321_57295, &P4248_320};	/*Property = Progress*/
const Value V4248_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_322 = {S41, 0, &V4248_322_57298, &P4248_321};	/*Property = Prompts*/
const Value V4248_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_323 = {S42, 0, &V4248_323_57301, &P4248_322};	/*Property = Selector*/
const Value V4248_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_324 = {S43, 0, &V4248_324_57307, &P4248_323};	/*Property = Svg*/
const Value V4248_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_327 = {S44, 0, &V4248_327_57316, &P4248_324};	/*Property = TouchEvents*/
const Value V4248_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_328 = {S45, 0, &V4248_328_57319, &P4248_327};	/*Property = Track*/
const Value V4248_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_329 = {S46, 0, &V4248_329_57325, &P4248_328};	/*Property = Video*/
const Value V4248_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_330 = {S47, 0, &V4248_330_57328, &P4248_329};	/*Property = Viewport*/
const Value V4248_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_332 = {S48, 0, &V4248_332_57340, &P4248_330};	/*Property = WebWorkers*/
const Value V4248_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4248_333 = {S49, 0, &V4248_333_57343, &P4248_332};	/*Property = Xhr2*/
const Profile P4248 = {"4248", &P4248_333};	/*Profile ID - 4248*/
const Value V15198_1_1 = {S2, NULL};	/*Value = True*/
const Property P15198_1 = {S50, 0, &V15198_1_1, NULL};	/*Property = IsMobile*/
const Value V15198_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15198_5 = {S52, 0, &V15198_5_1222, &P15198_1};	/*Property = ScreenPixelsHeight*/
const Value V15198_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15198_4 = {S54, 0, &V15198_4_1224, &P15198_5};	/*Property = ScreenPixelsWidth*/
const Profile P15198 = {"15198", &P15198_4};	/*Profile ID - 15198*/
const Value V14763_1_1 = {S2, NULL};	/*Value = True*/
const Property P14763_1 = {S50, 0, &V14763_1_1, NULL};	/*Property = IsMobile*/
const Value V14763_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P14763_5 = {S52, 0, &V14763_5_1280, &P14763_1};	/*Property = ScreenPixelsHeight*/
const Value V14763_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14763_4 = {S54, 0, &V14763_4_1235, &P14763_5};	/*Property = ScreenPixelsWidth*/
const Profile P14763 = {"14763", &P14763_4};	/*Profile ID - 14763*/
const Value V12138_1_1 = {S2, NULL};	/*Value = True*/
const Property P12138_1 = {S50, 0, &V12138_1_1, NULL};	/*Property = IsMobile*/
const Value V12138_5_31826 = {S75, NULL};	/*Value = 260*/
const Property P12138_5 = {S52, 0, &V12138_5_31826, &P12138_1};	/*Property = ScreenPixelsHeight*/
const Value V12138_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12138_4 = {S54, 0, &V12138_4_1224, &P12138_5};	/*Property = ScreenPixelsWidth*/
const Profile P12138 = {"12138", &P12138_4};	/*Profile ID - 12138*/
const Value V11642_1_1 = {S2, NULL};	/*Value = True*/
const Property P11642_1 = {S50, 0, &V11642_1_1, NULL};	/*Property = IsMobile*/
const Value V11642_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11642_5 = {S52, 0, &V11642_5_1222, &P11642_1};	/*Property = ScreenPixelsHeight*/
const Value V11642_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11642_4 = {S54, 0, &V11642_4_1224, &P11642_5};	/*Property = ScreenPixelsWidth*/
const Profile P11642 = {"11642", &P11642_4};	/*Profile ID - 11642*/
const Value V11643_1_1 = {S2, NULL};	/*Value = True*/
const Property P11643_1 = {S50, 0, &V11643_1_1, NULL};	/*Property = IsMobile*/
const Value V11643_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11643_5 = {S52, 0, &V11643_5_1222, &P11643_1};	/*Property = ScreenPixelsHeight*/
const Value V11643_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11643_4 = {S54, 0, &V11643_4_1224, &P11643_5};	/*Property = ScreenPixelsWidth*/
const Profile P11643 = {"11643", &P11643_4};	/*Profile ID - 11643*/
const Value V11644_1_1 = {S2, NULL};	/*Value = True*/
const Property P11644_1 = {S50, 0, &V11644_1_1, NULL};	/*Property = IsMobile*/
const Value V11644_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11644_5 = {S52, 0, &V11644_5_1293, &P11644_1};	/*Property = ScreenPixelsHeight*/
const Value V11644_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11644_4 = {S54, 0, &V11644_4_1296, &P11644_5};	/*Property = ScreenPixelsWidth*/
const Profile P11644 = {"11644", &P11644_4};	/*Profile ID - 11644*/
const Value V13977_1_1 = {S2, NULL};	/*Value = True*/
const Property P13977_1 = {S50, 0, &V13977_1_1, NULL};	/*Property = IsMobile*/
const Value V13977_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13977_5 = {S52, 0, &V13977_5_1222, &P13977_1};	/*Property = ScreenPixelsHeight*/
const Value V13977_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13977_4 = {S54, 0, &V13977_4_1224, &P13977_5};	/*Property = ScreenPixelsWidth*/
const Profile P13977 = {"13977", &P13977_4};	/*Profile ID - 13977*/
const Value V11645_1_1 = {S2, NULL};	/*Value = True*/
const Property P11645_1 = {S50, 0, &V11645_1_1, NULL};	/*Property = IsMobile*/
const Value V11645_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11645_5 = {S52, 0, &V11645_5_1293, &P11645_1};	/*Property = ScreenPixelsHeight*/
const Value V11645_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11645_4 = {S54, 0, &V11645_4_1296, &P11645_5};	/*Property = ScreenPixelsWidth*/
const Profile P11645 = {"11645", &P11645_4};	/*Profile ID - 11645*/
const Value V11646_1_1 = {S2, NULL};	/*Value = True*/
const Property P11646_1 = {S50, 0, &V11646_1_1, NULL};	/*Property = IsMobile*/
const Value V11646_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11646_5 = {S52, 0, &V11646_5_1222, &P11646_1};	/*Property = ScreenPixelsHeight*/
const Value V11646_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11646_4 = {S54, 0, &V11646_4_1224, &P11646_5};	/*Property = ScreenPixelsWidth*/
const Profile P11646 = {"11646", &P11646_4};	/*Profile ID - 11646*/
const Value V20126_1_1 = {S2, NULL};	/*Value = True*/
const Property P20126_1 = {S50, 0, &V20126_1_1, NULL};	/*Property = IsMobile*/
const Value V20126_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20126_5 = {S52, 0, &V20126_5_37, &P20126_1};	/*Property = ScreenPixelsHeight*/
const Value V20126_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20126_4 = {S54, 0, &V20126_4_36, &P20126_5};	/*Property = ScreenPixelsWidth*/
const Profile P20126 = {"20126", &P20126_4};	/*Profile ID - 20126*/
const Value V13179_1_1 = {S2, NULL};	/*Value = True*/
const Property P13179_1 = {S50, 0, &V13179_1_1, NULL};	/*Property = IsMobile*/
const Value V13179_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13179_5 = {S52, 0, &V13179_5_1233, &P13179_1};	/*Property = ScreenPixelsHeight*/
const Value V13179_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13179_4 = {S54, 0, &V13179_4_1235, &P13179_5};	/*Property = ScreenPixelsWidth*/
const Profile P13179 = {"13179", &P13179_4};	/*Profile ID - 13179*/
const Value V12056_1_1 = {S2, NULL};	/*Value = True*/
const Property P12056_1 = {S50, 0, &V12056_1_1, NULL};	/*Property = IsMobile*/
const Value V12056_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12056_5 = {S52, 0, &V12056_5_1222, &P12056_1};	/*Property = ScreenPixelsHeight*/
const Value V12056_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12056_4 = {S54, 0, &V12056_4_1224, &P12056_5};	/*Property = ScreenPixelsWidth*/
const Profile P12056 = {"12056", &P12056_4};	/*Profile ID - 12056*/
const Value V11649_1_1 = {S2, NULL};	/*Value = True*/
const Property P11649_1 = {S50, 0, &V11649_1_1, NULL};	/*Property = IsMobile*/
const Value V11649_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P11649_5 = {S52, 0, &V11649_5_1294, &P11649_1};	/*Property = ScreenPixelsHeight*/
const Value V11649_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11649_4 = {S54, 0, &V11649_4_1296, &P11649_5};	/*Property = ScreenPixelsWidth*/
const Profile P11649 = {"11649", &P11649_4};	/*Profile ID - 11649*/
const Value V13867_1_1 = {S2, NULL};	/*Value = True*/
const Property P13867_1 = {S50, 0, &V13867_1_1, NULL};	/*Property = IsMobile*/
const Value V13867_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13867_5 = {S52, 0, &V13867_5_1222, &P13867_1};	/*Property = ScreenPixelsHeight*/
const Value V13867_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13867_4 = {S54, 0, &V13867_4_1224, &P13867_5};	/*Property = ScreenPixelsWidth*/
const Profile P13867 = {"13867", &P13867_4};	/*Profile ID - 13867*/
const Value V16902_1_1 = {S2, NULL};	/*Value = True*/
const Property P16902_1 = {S50, 0, &V16902_1_1, NULL};	/*Property = IsMobile*/
const Value V16902_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16902_5 = {S52, 0, &V16902_5_1222, &P16902_1};	/*Property = ScreenPixelsHeight*/
const Value V16902_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16902_4 = {S54, 0, &V16902_4_1224, &P16902_5};	/*Property = ScreenPixelsWidth*/
const Profile P16902 = {"16902", &P16902_4};	/*Profile ID - 16902*/
const Value V11730_1_1 = {S2, NULL};	/*Value = True*/
const Property P11730_1 = {S50, 0, &V11730_1_1, NULL};	/*Property = IsMobile*/
const Value V11730_5_1796 = {S64, NULL};	/*Value = 360*/
const Property P11730_5 = {S52, 0, &V11730_5_1796, &P11730_1};	/*Property = ScreenPixelsHeight*/
const Value V11730_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P11730_4 = {S54, 0, &V11730_4_1229, &P11730_5};	/*Property = ScreenPixelsWidth*/
const Profile P11730 = {"11730", &P11730_4};	/*Profile ID - 11730*/
const Value V13206_1_1 = {S2, NULL};	/*Value = True*/
const Property P13206_1 = {S50, 0, &V13206_1_1, NULL};	/*Property = IsMobile*/
const Value V13206_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13206_5 = {S52, 0, &V13206_5_1233, &P13206_1};	/*Property = ScreenPixelsHeight*/
const Value V13206_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13206_4 = {S54, 0, &V13206_4_1235, &P13206_5};	/*Property = ScreenPixelsWidth*/
const Profile P13206 = {"13206", &P13206_4};	/*Profile ID - 13206*/
const Value V12089_1_1 = {S2, NULL};	/*Value = True*/
const Property P12089_1 = {S50, 0, &V12089_1_1, NULL};	/*Property = IsMobile*/
const Value V12089_5_1512 = {S77, NULL};	/*Value = 416*/
const Property P12089_5 = {S52, 0, &V12089_5_1512, &P12089_1};	/*Property = ScreenPixelsHeight*/
const Value V12089_4_1513 = {S78, NULL};	/*Value = 352*/
const Property P12089_4 = {S54, 0, &V12089_4_1513, &P12089_5};	/*Property = ScreenPixelsWidth*/
const Profile P12089 = {"12089", &P12089_4};	/*Profile ID - 12089*/
const Value V11912_1_1 = {S2, NULL};	/*Value = True*/
const Property P11912_1 = {S50, 0, &V11912_1_1, NULL};	/*Property = IsMobile*/
const Value V11912_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11912_5 = {S52, 0, &V11912_5_1233, &P11912_1};	/*Property = ScreenPixelsHeight*/
const Value V11912_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11912_4 = {S54, 0, &V11912_4_1235, &P11912_5};	/*Property = ScreenPixelsWidth*/
const Profile P11912 = {"11912", &P11912_4};	/*Profile ID - 11912*/
const Value V12205_1_1 = {S2, NULL};	/*Value = True*/
const Property P12205_1 = {S50, 0, &V12205_1_1, NULL};	/*Property = IsMobile*/
const Value V12205_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12205_5 = {S52, 0, &V12205_5_1222, &P12205_1};	/*Property = ScreenPixelsHeight*/
const Value V12205_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12205_4 = {S54, 0, &V12205_4_1224, &P12205_5};	/*Property = ScreenPixelsWidth*/
const Profile P12205 = {"12205", &P12205_4};	/*Profile ID - 12205*/
const Value V11651_1_1 = {S2, NULL};	/*Value = True*/
const Property P11651_1 = {S50, 0, &V11651_1_1, NULL};	/*Property = IsMobile*/
const Value V11651_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11651_5 = {S52, 0, &V11651_5_1293, &P11651_1};	/*Property = ScreenPixelsHeight*/
const Value V11651_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11651_4 = {S54, 0, &V11651_4_1296, &P11651_5};	/*Property = ScreenPixelsWidth*/
const Profile P11651 = {"11651", &P11651_4};	/*Profile ID - 11651*/
const Value V17425_1_1 = {S2, NULL};	/*Value = True*/
const Property P17425_1 = {S50, 0, &V17425_1_1, NULL};	/*Property = IsMobile*/
const Value V17425_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17425_5 = {S52, 0, &V17425_5_1222, &P17425_1};	/*Property = ScreenPixelsHeight*/
const Value V17425_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17425_4 = {S54, 0, &V17425_4_1224, &P17425_5};	/*Property = ScreenPixelsWidth*/
const Profile P17425 = {"17425", &P17425_4};	/*Profile ID - 17425*/
const Value V12175_1_1 = {S2, NULL};	/*Value = True*/
const Property P12175_1 = {S50, 0, &V12175_1_1, NULL};	/*Property = IsMobile*/
const Value V12175_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12175_5 = {S52, 0, &V12175_5_1304, &P12175_1};	/*Property = ScreenPixelsHeight*/
const Value V12175_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12175_4 = {S54, 0, &V12175_4_1224, &P12175_5};	/*Property = ScreenPixelsWidth*/
const Profile P12175 = {"12175", &P12175_4};	/*Profile ID - 12175*/
const Value V17542_1_1 = {S2, NULL};	/*Value = True*/
const Property P17542_1 = {S50, 0, &V17542_1_1, NULL};	/*Property = IsMobile*/
const Value V17542_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17542_5 = {S52, 0, &V17542_5_1293, &P17542_1};	/*Property = ScreenPixelsHeight*/
const Value V17542_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17542_4 = {S54, 0, &V17542_4_1296, &P17542_5};	/*Property = ScreenPixelsWidth*/
const Profile P17542 = {"17542", &P17542_4};	/*Profile ID - 17542*/
const Value V11924_1_1 = {S2, NULL};	/*Value = True*/
const Property P11924_1 = {S50, 0, &V11924_1_1, NULL};	/*Property = IsMobile*/
const Value V11924_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11924_5 = {S52, 0, &V11924_5_1222, &P11924_1};	/*Property = ScreenPixelsHeight*/
const Value V11924_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11924_4 = {S54, 0, &V11924_4_1224, &P11924_5};	/*Property = ScreenPixelsWidth*/
const Profile P11924 = {"11924", &P11924_4};	/*Profile ID - 11924*/
const Value V4893_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_279 = {S1, 0, &V4893_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4893_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_282 = {S3, 0, &V4893_282_57155, &P4893_279};	/*Property = BlobBuilder*/
const Value V4893_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_350 = {S4, 0, &V4893_350_57446, &P4893_282};	/*Property = Canvas*/
const Value V4893_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_283 = {S5, 0, &V4893_283_57161, &P4893_350};	/*Property = CssBackground*/
const Value V4893_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_284 = {S6, 0, &V4893_284_57164, &P4893_283};	/*Property = CssBorderImage*/
const Value V4893_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_285 = {S7, 0, &V4893_285_57167, &P4893_284};	/*Property = CssCanvas*/
const Value V4893_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_286 = {S8, 0, &V4893_286_57170, &P4893_285};	/*Property = CssColor*/
const Value V4893_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_237 = {S9, 0, &V4893_237_55786, &P4893_286};	/*Property = CssColumn*/
const Value V4893_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_287 = {S10, 0, &V4893_287_57173, &P4893_237};	/*Property = CssFlexbox*/
const Value V4893_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_288 = {S11, 0, &V4893_288_57176, &P4893_287};	/*Property = CssFont*/
const Value V4893_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_289 = {S12, 0, &V4893_289_86899, &P4893_288};	/*Property = CssImages*/
const Value V4893_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_291 = {S13, 0, &V4893_291_57182, &P4893_289};	/*Property = CssMediaQueries*/
const Value V4893_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_292 = {S14, 0, &V4893_292_57185, &P4893_291};	/*Property = CssMinMax*/
const Value V4893_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_293 = {S15, 0, &V4893_293_57188, &P4893_292};	/*Property = CssOverflow*/
const Value V4893_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_294 = {S16, 0, &V4893_294_57191, &P4893_293};	/*Property = CssPosition*/
const Value V4893_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_295 = {S17, 0, &V4893_295_57194, &P4893_294};	/*Property = CssText*/
const Value V4893_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_297 = {S18, 0, &V4893_297_57197, &P4893_295};	/*Property = CssTransforms*/
const Value V4893_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_241 = {S19, 0, &V4893_241_55801, &P4893_297};	/*Property = CssTransitions*/
const Value V4893_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_298 = {S20, 0, &V4893_298_57208, &P4893_241};	/*Property = CssUI*/
const Value V4893_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_300 = {S21, 0, &V4893_300_57217, &P4893_298};	/*Property = DataSet*/
const Value V4893_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_301 = {S22, 0, &V4893_301_57220, &P4893_300};	/*Property = DataUrl*/
const Value V4893_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_302 = {S23, 0, &V4893_302_57223, &P4893_301};	/*Property = DeviceOrientation*/
const Value V4893_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_304 = {S24, 0, &V4893_304_57229, &P4893_302};	/*Property = FileReader*/
const Value V4893_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_305 = {S25, 0, &V4893_305_57232, &P4893_304};	/*Property = FileSaver*/
const Value V4893_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_306 = {S26, 0, &V4893_306_57238, &P4893_305};	/*Property = FileWriter*/
const Value V4893_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_307 = {S27, 0, &V4893_307_57241, &P4893_306};	/*Property = FormData*/
const Value V4893_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_309 = {S28, 0, &V4893_309_57247, &P4893_307};	/*Property = Fullscreen*/
const Value V4893_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_310 = {S29, 0, &V4893_310_57250, &P4893_309};	/*Property = GeoLocation*/
const Value V4893_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_312 = {S30, 0, &V4893_312_57256, &P4893_310};	/*Property = History*/
const Value V4893_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_314 = {S31, 0, &V4893_314_57262, &P4893_312};	/*Property = Html5*/
const Value V4893_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_313 = {S32, 0, &V4893_313_57259, &P4893_314};	/*Property = Html-Media-Capture*/
const Value V4893_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_315 = {S33, 0, &V4893_315_57265, &P4893_313};	/*Property = Iframe*/
const Value V4893_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_316 = {S34, 0, &V4893_316_57268, &P4893_315};	/*Property = IndexedDB*/
const Value V4893_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_317 = {S35, 0, &V4893_317_57271, &P4893_316};	/*Property = Json*/
const Value V4893_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_15 = {S37, 0, &V4893_15_72, &P4893_317};	/*Property = LayoutEngine*/
const Value V4893_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_344 = {S38, 0, &V4893_344_57274, &P4893_15};	/*Property = Masking*/
const Value V4893_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_320 = {S39, 0, &V4893_320_57292, &P4893_344};	/*Property = PostMessage*/
const Value V4893_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_321 = {S40, 0, &V4893_321_57295, &P4893_320};	/*Property = Progress*/
const Value V4893_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_322 = {S41, 0, &V4893_322_57298, &P4893_321};	/*Property = Prompts*/
const Value V4893_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_323 = {S42, 0, &V4893_323_57301, &P4893_322};	/*Property = Selector*/
const Value V4893_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_324 = {S43, 0, &V4893_324_57307, &P4893_323};	/*Property = Svg*/
const Value V4893_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_327 = {S44, 0, &V4893_327_57316, &P4893_324};	/*Property = TouchEvents*/
const Value V4893_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_328 = {S45, 0, &V4893_328_57319, &P4893_327};	/*Property = Track*/
const Value V4893_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_329 = {S46, 0, &V4893_329_57325, &P4893_328};	/*Property = Video*/
const Value V4893_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_330 = {S47, 0, &V4893_330_57328, &P4893_329};	/*Property = Viewport*/
const Value V4893_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_332 = {S48, 0, &V4893_332_57340, &P4893_330};	/*Property = WebWorkers*/
const Value V4893_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4893_333 = {S49, 0, &V4893_333_57343, &P4893_332};	/*Property = Xhr2*/
const Profile P4893 = {"4893", &P4893_333};	/*Profile ID - 4893*/
const Value V11927_1_1 = {S2, NULL};	/*Value = True*/
const Property P11927_1 = {S50, 0, &V11927_1_1, NULL};	/*Property = IsMobile*/
const Value V11927_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11927_5 = {S52, 0, &V11927_5_1222, &P11927_1};	/*Property = ScreenPixelsHeight*/
const Value V11927_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11927_4 = {S54, 0, &V11927_4_1224, &P11927_5};	/*Property = ScreenPixelsWidth*/
const Profile P11927 = {"11927", &P11927_4};	/*Profile ID - 11927*/
const Value V11918_1_1 = {S2, NULL};	/*Value = True*/
const Property P11918_1 = {S50, 0, &V11918_1_1, NULL};	/*Property = IsMobile*/
const Value V11918_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P11918_5 = {S52, 0, &V11918_5_1380, &P11918_1};	/*Property = ScreenPixelsHeight*/
const Value V11918_4_46004 = {S79, NULL};	/*Value = 440*/
const Property P11918_4 = {S54, 0, &V11918_4_46004, &P11918_5};	/*Property = ScreenPixelsWidth*/
const Profile P11918 = {"11918", &P11918_4};	/*Profile ID - 11918*/
const Value V11933_1_1 = {S2, NULL};	/*Value = True*/
const Property P11933_1 = {S50, 0, &V11933_1_1, NULL};	/*Property = IsMobile*/
const Value V11933_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11933_5 = {S52, 0, &V11933_5_1293, &P11933_1};	/*Property = ScreenPixelsHeight*/
const Value V11933_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11933_4 = {S54, 0, &V11933_4_1296, &P11933_5};	/*Property = ScreenPixelsWidth*/
const Profile P11933 = {"11933", &P11933_4};	/*Profile ID - 11933*/
const Value V11934_1_1 = {S2, NULL};	/*Value = True*/
const Property P11934_1 = {S50, 0, &V11934_1_1, NULL};	/*Property = IsMobile*/
const Value V11934_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11934_5 = {S52, 0, &V11934_5_1293, &P11934_1};	/*Property = ScreenPixelsHeight*/
const Value V11934_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11934_4 = {S54, 0, &V11934_4_1296, &P11934_5};	/*Property = ScreenPixelsWidth*/
const Profile P11934 = {"11934", &P11934_4};	/*Profile ID - 11934*/
const Value V11938_1_1 = {S2, NULL};	/*Value = True*/
const Property P11938_1 = {S50, 0, &V11938_1_1, NULL};	/*Property = IsMobile*/
const Value V11938_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P11938_5 = {S52, 0, &V11938_5_1380, &P11938_1};	/*Property = ScreenPixelsHeight*/
const Value V11938_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P11938_4 = {S54, 0, &V11938_4_1279, &P11938_5};	/*Property = ScreenPixelsWidth*/
const Profile P11938 = {"11938", &P11938_4};	/*Profile ID - 11938*/
const Value V11939_1_1 = {S2, NULL};	/*Value = True*/
const Property P11939_1 = {S50, 0, &V11939_1_1, NULL};	/*Property = IsMobile*/
const Value V11939_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11939_5 = {S52, 0, &V11939_5_1233, &P11939_1};	/*Property = ScreenPixelsHeight*/
const Value V11939_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11939_4 = {S54, 0, &V11939_4_1235, &P11939_5};	/*Property = ScreenPixelsWidth*/
const Profile P11939 = {"11939", &P11939_4};	/*Profile ID - 11939*/
const Value V11943_1_1 = {S2, NULL};	/*Value = True*/
const Property P11943_1 = {S50, 0, &V11943_1_1, NULL};	/*Property = IsMobile*/
const Value V11943_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11943_5 = {S52, 0, &V11943_5_1222, &P11943_1};	/*Property = ScreenPixelsHeight*/
const Value V11943_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11943_4 = {S54, 0, &V11943_4_1224, &P11943_5};	/*Property = ScreenPixelsWidth*/
const Profile P11943 = {"11943", &P11943_4};	/*Profile ID - 11943*/
const Value V11944_1_1 = {S2, NULL};	/*Value = True*/
const Property P11944_1 = {S50, 0, &V11944_1_1, NULL};	/*Property = IsMobile*/
const Value V11944_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11944_5 = {S52, 0, &V11944_5_1222, &P11944_1};	/*Property = ScreenPixelsHeight*/
const Value V11944_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11944_4 = {S54, 0, &V11944_4_1224, &P11944_5};	/*Property = ScreenPixelsWidth*/
const Profile P11944 = {"11944", &P11944_4};	/*Profile ID - 11944*/
const Value V3636_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_279 = {S1, 0, &V3636_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3636_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_282 = {S3, 0, &V3636_282_57155, &P3636_279};	/*Property = BlobBuilder*/
const Value V3636_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_350 = {S4, 0, &V3636_350_57446, &P3636_282};	/*Property = Canvas*/
const Value V3636_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_283 = {S5, 0, &V3636_283_57161, &P3636_350};	/*Property = CssBackground*/
const Value V3636_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_284 = {S6, 0, &V3636_284_57164, &P3636_283};	/*Property = CssBorderImage*/
const Value V3636_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_285 = {S7, 0, &V3636_285_57167, &P3636_284};	/*Property = CssCanvas*/
const Value V3636_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_286 = {S8, 0, &V3636_286_57170, &P3636_285};	/*Property = CssColor*/
const Value V3636_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_237 = {S9, 0, &V3636_237_55786, &P3636_286};	/*Property = CssColumn*/
const Value V3636_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_287 = {S10, 0, &V3636_287_57173, &P3636_237};	/*Property = CssFlexbox*/
const Value V3636_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_288 = {S11, 0, &V3636_288_57176, &P3636_287};	/*Property = CssFont*/
const Value V3636_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_289 = {S12, 0, &V3636_289_86899, &P3636_288};	/*Property = CssImages*/
const Value V3636_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_291 = {S13, 0, &V3636_291_57182, &P3636_289};	/*Property = CssMediaQueries*/
const Value V3636_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_292 = {S14, 0, &V3636_292_57185, &P3636_291};	/*Property = CssMinMax*/
const Value V3636_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_293 = {S15, 0, &V3636_293_57188, &P3636_292};	/*Property = CssOverflow*/
const Value V3636_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_294 = {S16, 0, &V3636_294_57191, &P3636_293};	/*Property = CssPosition*/
const Value V3636_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_295 = {S17, 0, &V3636_295_57194, &P3636_294};	/*Property = CssText*/
const Value V3636_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_297 = {S18, 0, &V3636_297_57197, &P3636_295};	/*Property = CssTransforms*/
const Value V3636_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_241 = {S19, 0, &V3636_241_55801, &P3636_297};	/*Property = CssTransitions*/
const Value V3636_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_298 = {S20, 0, &V3636_298_57208, &P3636_241};	/*Property = CssUI*/
const Value V3636_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_300 = {S21, 0, &V3636_300_57217, &P3636_298};	/*Property = DataSet*/
const Value V3636_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_301 = {S22, 0, &V3636_301_57220, &P3636_300};	/*Property = DataUrl*/
const Value V3636_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_302 = {S23, 0, &V3636_302_57223, &P3636_301};	/*Property = DeviceOrientation*/
const Value V3636_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_304 = {S24, 0, &V3636_304_57229, &P3636_302};	/*Property = FileReader*/
const Value V3636_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_305 = {S25, 0, &V3636_305_57232, &P3636_304};	/*Property = FileSaver*/
const Value V3636_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_306 = {S26, 0, &V3636_306_57238, &P3636_305};	/*Property = FileWriter*/
const Value V3636_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_307 = {S27, 0, &V3636_307_57241, &P3636_306};	/*Property = FormData*/
const Value V3636_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_309 = {S28, 0, &V3636_309_57247, &P3636_307};	/*Property = Fullscreen*/
const Value V3636_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_310 = {S29, 0, &V3636_310_57250, &P3636_309};	/*Property = GeoLocation*/
const Value V3636_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_312 = {S30, 0, &V3636_312_57256, &P3636_310};	/*Property = History*/
const Value V3636_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_314 = {S31, 0, &V3636_314_57262, &P3636_312};	/*Property = Html5*/
const Value V3636_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_313 = {S32, 0, &V3636_313_57259, &P3636_314};	/*Property = Html-Media-Capture*/
const Value V3636_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_315 = {S33, 0, &V3636_315_57265, &P3636_313};	/*Property = Iframe*/
const Value V3636_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_316 = {S34, 0, &V3636_316_57268, &P3636_315};	/*Property = IndexedDB*/
const Value V3636_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_317 = {S35, 0, &V3636_317_57271, &P3636_316};	/*Property = Json*/
const Value V3636_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3636_15 = {S37, 0, &V3636_15_28, &P3636_317};	/*Property = LayoutEngine*/
const Value V3636_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_344 = {S38, 0, &V3636_344_57274, &P3636_15};	/*Property = Masking*/
const Value V3636_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_320 = {S39, 0, &V3636_320_57292, &P3636_344};	/*Property = PostMessage*/
const Value V3636_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_321 = {S40, 0, &V3636_321_57295, &P3636_320};	/*Property = Progress*/
const Value V3636_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_322 = {S41, 0, &V3636_322_57298, &P3636_321};	/*Property = Prompts*/
const Value V3636_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_323 = {S42, 0, &V3636_323_57301, &P3636_322};	/*Property = Selector*/
const Value V3636_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_324 = {S43, 0, &V3636_324_57307, &P3636_323};	/*Property = Svg*/
const Value V3636_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_327 = {S44, 0, &V3636_327_57316, &P3636_324};	/*Property = TouchEvents*/
const Value V3636_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_328 = {S45, 0, &V3636_328_57319, &P3636_327};	/*Property = Track*/
const Value V3636_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_329 = {S46, 0, &V3636_329_57325, &P3636_328};	/*Property = Video*/
const Value V3636_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_330 = {S47, 0, &V3636_330_57328, &P3636_329};	/*Property = Viewport*/
const Value V3636_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_332 = {S48, 0, &V3636_332_57340, &P3636_330};	/*Property = WebWorkers*/
const Value V3636_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3636_333 = {S49, 0, &V3636_333_57343, &P3636_332};	/*Property = Xhr2*/
const Profile P3636 = {"3636", &P3636_333};	/*Profile ID - 3636*/
const Value V11945_1_1 = {S2, NULL};	/*Value = True*/
const Property P11945_1 = {S50, 0, &V11945_1_1, NULL};	/*Property = IsMobile*/
const Value V11945_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P11945_5 = {S52, 0, &V11945_5_1304, &P11945_1};	/*Property = ScreenPixelsHeight*/
const Value V11945_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11945_4 = {S54, 0, &V11945_4_1224, &P11945_5};	/*Property = ScreenPixelsWidth*/
const Profile P11945 = {"11945", &P11945_4};	/*Profile ID - 11945*/
const Value V13183_1_1 = {S2, NULL};	/*Value = True*/
const Property P13183_1 = {S50, 0, &V13183_1_1, NULL};	/*Property = IsMobile*/
const Value V13183_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13183_5 = {S52, 0, &V13183_5_1222, &P13183_1};	/*Property = ScreenPixelsHeight*/
const Value V13183_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13183_4 = {S54, 0, &V13183_4_1224, &P13183_5};	/*Property = ScreenPixelsWidth*/
const Profile P13183 = {"13183", &P13183_4};	/*Profile ID - 13183*/
const Value V11948_1_1 = {S2, NULL};	/*Value = True*/
const Property P11948_1 = {S50, 0, &V11948_1_1, NULL};	/*Property = IsMobile*/
const Value V11948_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11948_5 = {S52, 0, &V11948_5_1233, &P11948_1};	/*Property = ScreenPixelsHeight*/
const Value V11948_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11948_4 = {S54, 0, &V11948_4_1235, &P11948_5};	/*Property = ScreenPixelsWidth*/
const Profile P11948 = {"11948", &P11948_4};	/*Profile ID - 11948*/
const Value V11950_1_1 = {S2, NULL};	/*Value = True*/
const Property P11950_1 = {S50, 0, &V11950_1_1, NULL};	/*Property = IsMobile*/
const Value V11950_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P11950_5 = {S52, 0, &V11950_5_1269, &P11950_1};	/*Property = ScreenPixelsHeight*/
const Value V11950_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P11950_4 = {S54, 0, &V11950_4_1229, &P11950_5};	/*Property = ScreenPixelsWidth*/
const Profile P11950 = {"11950", &P11950_4};	/*Profile ID - 11950*/
const Value V14845_1_1 = {S2, NULL};	/*Value = True*/
const Property P14845_1 = {S50, 0, &V14845_1_1, NULL};	/*Property = IsMobile*/
const Value V14845_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14845_5 = {S52, 0, &V14845_5_1227, &P14845_1};	/*Property = ScreenPixelsHeight*/
const Value V14845_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14845_4 = {S54, 0, &V14845_4_1229, &P14845_5};	/*Property = ScreenPixelsWidth*/
const Profile P14845 = {"14845", &P14845_4};	/*Profile ID - 14845*/
const Value V13800_1_1 = {S2, NULL};	/*Value = True*/
const Property P13800_1 = {S50, 0, &V13800_1_1, NULL};	/*Property = IsMobile*/
const Value V13800_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13800_5 = {S52, 0, &V13800_5_1222, &P13800_1};	/*Property = ScreenPixelsHeight*/
const Value V13800_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13800_4 = {S54, 0, &V13800_4_1224, &P13800_5};	/*Property = ScreenPixelsWidth*/
const Profile P13800 = {"13800", &P13800_4};	/*Profile ID - 13800*/
const Value V11685_1_1 = {S2, NULL};	/*Value = True*/
const Property P11685_1 = {S50, 0, &V11685_1_1, NULL};	/*Property = IsMobile*/
const Value V11685_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11685_5 = {S52, 0, &V11685_5_1222, &P11685_1};	/*Property = ScreenPixelsHeight*/
const Value V11685_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11685_4 = {S54, 0, &V11685_4_1224, &P11685_5};	/*Property = ScreenPixelsWidth*/
const Profile P11685 = {"11685", &P11685_4};	/*Profile ID - 11685*/
const Value V14732_1_1 = {S2, NULL};	/*Value = True*/
const Property P14732_1 = {S50, 0, &V14732_1_1, NULL};	/*Property = IsMobile*/
const Value V14732_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14732_5 = {S52, 0, &V14732_5_1222, &P14732_1};	/*Property = ScreenPixelsHeight*/
const Value V14732_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14732_4 = {S54, 0, &V14732_4_1279, &P14732_5};	/*Property = ScreenPixelsWidth*/
const Profile P14732 = {"14732", &P14732_4};	/*Profile ID - 14732*/
const Value V11953_1_1 = {S2, NULL};	/*Value = True*/
const Property P11953_1 = {S50, 0, &V11953_1_1, NULL};	/*Property = IsMobile*/
const Value V11953_5_4286 = {S73, NULL};	/*Value = 432*/
const Property P11953_5 = {S52, 0, &V11953_5_4286, &P11953_1};	/*Property = ScreenPixelsHeight*/
const Value V11953_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11953_4 = {S54, 0, &V11953_4_1224, &P11953_5};	/*Property = ScreenPixelsWidth*/
const Profile P11953 = {"11953", &P11953_4};	/*Profile ID - 11953*/
const Value V14478_1_1 = {S2, NULL};	/*Value = True*/
const Property P14478_1 = {S50, 0, &V14478_1_1, NULL};	/*Property = IsMobile*/
const Value V14478_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14478_5 = {S52, 0, &V14478_5_1380, &P14478_1};	/*Property = ScreenPixelsHeight*/
const Value V14478_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14478_4 = {S54, 0, &V14478_4_1279, &P14478_5};	/*Property = ScreenPixelsWidth*/
const Profile P14478 = {"14478", &P14478_4};	/*Profile ID - 14478*/
const Value V12064_1_1 = {S2, NULL};	/*Value = True*/
const Property P12064_1 = {S50, 0, &V12064_1_1, NULL};	/*Property = IsMobile*/
const Value V12064_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12064_5 = {S52, 0, &V12064_5_1233, &P12064_1};	/*Property = ScreenPixelsHeight*/
const Value V12064_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12064_4 = {S54, 0, &V12064_4_1235, &P12064_5};	/*Property = ScreenPixelsWidth*/
const Profile P12064 = {"12064", &P12064_4};	/*Profile ID - 12064*/
const Value V12487_1_1 = {S2, NULL};	/*Value = True*/
const Property P12487_1 = {S50, 0, &V12487_1_1, NULL};	/*Property = IsMobile*/
const Value V12487_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12487_5 = {S52, 0, &V12487_5_1222, &P12487_1};	/*Property = ScreenPixelsHeight*/
const Value V12487_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12487_4 = {S54, 0, &V12487_4_1224, &P12487_5};	/*Property = ScreenPixelsWidth*/
const Profile P12487 = {"12487", &P12487_4};	/*Profile ID - 12487*/
const Value V11959_1_1 = {S2, NULL};	/*Value = True*/
const Property P11959_1 = {S50, 0, &V11959_1_1, NULL};	/*Property = IsMobile*/
const Value V11959_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11959_5 = {S52, 0, &V11959_5_1222, &P11959_1};	/*Property = ScreenPixelsHeight*/
const Value V11959_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11959_4 = {S54, 0, &V11959_4_1224, &P11959_5};	/*Property = ScreenPixelsWidth*/
const Profile P11959 = {"11959", &P11959_4};	/*Profile ID - 11959*/
const Value V11964_1_1 = {S2, NULL};	/*Value = True*/
const Property P11964_1 = {S50, 0, &V11964_1_1, NULL};	/*Property = IsMobile*/
const Value V11964_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11964_5 = {S52, 0, &V11964_5_1222, &P11964_1};	/*Property = ScreenPixelsHeight*/
const Value V11964_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11964_4 = {S54, 0, &V11964_4_1224, &P11964_5};	/*Property = ScreenPixelsWidth*/
const Profile P11964 = {"11964", &P11964_4};	/*Profile ID - 11964*/
const Value V13103_1_1 = {S2, NULL};	/*Value = True*/
const Property P13103_1 = {S50, 0, &V13103_1_1, NULL};	/*Property = IsMobile*/
const Value V13103_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13103_5 = {S52, 0, &V13103_5_1233, &P13103_1};	/*Property = ScreenPixelsHeight*/
const Value V13103_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13103_4 = {S54, 0, &V13103_4_1235, &P13103_5};	/*Property = ScreenPixelsWidth*/
const Profile P13103 = {"13103", &P13103_4};	/*Profile ID - 13103*/
const Value V17424_1_1 = {S2, NULL};	/*Value = True*/
const Property P17424_1 = {S50, 0, &V17424_1_1, NULL};	/*Property = IsMobile*/
const Value V17424_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17424_5 = {S52, 0, &V17424_5_1222, &P17424_1};	/*Property = ScreenPixelsHeight*/
const Value V17424_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17424_4 = {S54, 0, &V17424_4_1224, &P17424_5};	/*Property = ScreenPixelsWidth*/
const Profile P17424 = {"17424", &P17424_4};	/*Profile ID - 17424*/
const Value V11965_1_1 = {S2, NULL};	/*Value = True*/
const Property P11965_1 = {S50, 0, &V11965_1_1, NULL};	/*Property = IsMobile*/
const Value V11965_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P11965_5 = {S52, 0, &V11965_5_1380, &P11965_1};	/*Property = ScreenPixelsHeight*/
const Value V11965_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P11965_4 = {S54, 0, &V11965_4_1279, &P11965_5};	/*Property = ScreenPixelsWidth*/
const Profile P11965 = {"11965", &P11965_4};	/*Profile ID - 11965*/
const Value V15792_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_279 = {S1, 0, &V15792_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V15792_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_282 = {S3, 0, &V15792_282_57155, &P15792_279};	/*Property = BlobBuilder*/
const Value V15792_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_350 = {S4, 0, &V15792_350_57446, &P15792_282};	/*Property = Canvas*/
const Value V15792_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_283 = {S5, 0, &V15792_283_57161, &P15792_350};	/*Property = CssBackground*/
const Value V15792_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_284 = {S6, 0, &V15792_284_57164, &P15792_283};	/*Property = CssBorderImage*/
const Value V15792_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_285 = {S7, 0, &V15792_285_57167, &P15792_284};	/*Property = CssCanvas*/
const Value V15792_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_286 = {S8, 0, &V15792_286_57170, &P15792_285};	/*Property = CssColor*/
const Value V15792_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_237 = {S9, 0, &V15792_237_55786, &P15792_286};	/*Property = CssColumn*/
const Value V15792_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_287 = {S10, 0, &V15792_287_57173, &P15792_237};	/*Property = CssFlexbox*/
const Value V15792_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_288 = {S11, 0, &V15792_288_57176, &P15792_287};	/*Property = CssFont*/
const Value V15792_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_289 = {S12, 0, &V15792_289_86899, &P15792_288};	/*Property = CssImages*/
const Value V15792_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_291 = {S13, 0, &V15792_291_57182, &P15792_289};	/*Property = CssMediaQueries*/
const Value V15792_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_292 = {S14, 0, &V15792_292_57185, &P15792_291};	/*Property = CssMinMax*/
const Value V15792_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_293 = {S15, 0, &V15792_293_57188, &P15792_292};	/*Property = CssOverflow*/
const Value V15792_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_294 = {S16, 0, &V15792_294_57191, &P15792_293};	/*Property = CssPosition*/
const Value V15792_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_295 = {S17, 0, &V15792_295_57194, &P15792_294};	/*Property = CssText*/
const Value V15792_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_297 = {S18, 0, &V15792_297_57197, &P15792_295};	/*Property = CssTransforms*/
const Value V15792_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_241 = {S19, 0, &V15792_241_55801, &P15792_297};	/*Property = CssTransitions*/
const Value V15792_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_298 = {S20, 0, &V15792_298_57208, &P15792_241};	/*Property = CssUI*/
const Value V15792_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_300 = {S21, 0, &V15792_300_57217, &P15792_298};	/*Property = DataSet*/
const Value V15792_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_301 = {S22, 0, &V15792_301_57220, &P15792_300};	/*Property = DataUrl*/
const Value V15792_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_302 = {S23, 0, &V15792_302_57223, &P15792_301};	/*Property = DeviceOrientation*/
const Value V15792_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_304 = {S24, 0, &V15792_304_57229, &P15792_302};	/*Property = FileReader*/
const Value V15792_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_305 = {S25, 0, &V15792_305_57232, &P15792_304};	/*Property = FileSaver*/
const Value V15792_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_306 = {S26, 0, &V15792_306_57238, &P15792_305};	/*Property = FileWriter*/
const Value V15792_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_307 = {S27, 0, &V15792_307_57241, &P15792_306};	/*Property = FormData*/
const Value V15792_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_309 = {S28, 0, &V15792_309_57247, &P15792_307};	/*Property = Fullscreen*/
const Value V15792_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_310 = {S29, 0, &V15792_310_57250, &P15792_309};	/*Property = GeoLocation*/
const Value V15792_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_312 = {S30, 0, &V15792_312_57256, &P15792_310};	/*Property = History*/
const Value V15792_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_314 = {S31, 0, &V15792_314_57262, &P15792_312};	/*Property = Html5*/
const Value V15792_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_313 = {S32, 0, &V15792_313_57259, &P15792_314};	/*Property = Html-Media-Capture*/
const Value V15792_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_315 = {S33, 0, &V15792_315_57265, &P15792_313};	/*Property = Iframe*/
const Value V15792_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_316 = {S34, 0, &V15792_316_57268, &P15792_315};	/*Property = IndexedDB*/
const Value V15792_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_317 = {S35, 0, &V15792_317_57271, &P15792_316};	/*Property = Json*/
const Value V15792_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P15792_15 = {S37, 0, &V15792_15_30, &P15792_317};	/*Property = LayoutEngine*/
const Value V15792_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_344 = {S38, 0, &V15792_344_57274, &P15792_15};	/*Property = Masking*/
const Value V15792_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_320 = {S39, 0, &V15792_320_57292, &P15792_344};	/*Property = PostMessage*/
const Value V15792_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_321 = {S40, 0, &V15792_321_57295, &P15792_320};	/*Property = Progress*/
const Value V15792_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_322 = {S41, 0, &V15792_322_57298, &P15792_321};	/*Property = Prompts*/
const Value V15792_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_323 = {S42, 0, &V15792_323_57301, &P15792_322};	/*Property = Selector*/
const Value V15792_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_324 = {S43, 0, &V15792_324_57307, &P15792_323};	/*Property = Svg*/
const Value V15792_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_327 = {S44, 0, &V15792_327_57316, &P15792_324};	/*Property = TouchEvents*/
const Value V15792_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_328 = {S45, 0, &V15792_328_57319, &P15792_327};	/*Property = Track*/
const Value V15792_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_329 = {S46, 0, &V15792_329_57325, &P15792_328};	/*Property = Video*/
const Value V15792_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_330 = {S47, 0, &V15792_330_57328, &P15792_329};	/*Property = Viewport*/
const Value V15792_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_332 = {S48, 0, &V15792_332_57340, &P15792_330};	/*Property = WebWorkers*/
const Value V15792_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P15792_333 = {S49, 0, &V15792_333_57343, &P15792_332};	/*Property = Xhr2*/
const Profile P15792 = {"15792", &P15792_333};	/*Profile ID - 15792*/
const Value V16054_1_1 = {S2, NULL};	/*Value = True*/
const Property P16054_1 = {S50, 0, &V16054_1_1, NULL};	/*Property = IsMobile*/
const Value V16054_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16054_5 = {S52, 0, &V16054_5_1222, &P16054_1};	/*Property = ScreenPixelsHeight*/
const Value V16054_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16054_4 = {S54, 0, &V16054_4_1224, &P16054_5};	/*Property = ScreenPixelsWidth*/
const Profile P16054 = {"16054", &P16054_4};	/*Profile ID - 16054*/
const Value V22272_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_279 = {S1, 0, &V22272_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22272_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_282 = {S3, 0, &V22272_282_57155, &P22272_279};	/*Property = BlobBuilder*/
const Value V22272_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_350 = {S4, 0, &V22272_350_57446, &P22272_282};	/*Property = Canvas*/
const Value V22272_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_283 = {S5, 0, &V22272_283_57161, &P22272_350};	/*Property = CssBackground*/
const Value V22272_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_284 = {S6, 0, &V22272_284_57164, &P22272_283};	/*Property = CssBorderImage*/
const Value V22272_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_285 = {S7, 0, &V22272_285_57167, &P22272_284};	/*Property = CssCanvas*/
const Value V22272_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_286 = {S8, 0, &V22272_286_57170, &P22272_285};	/*Property = CssColor*/
const Value V22272_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_237 = {S9, 0, &V22272_237_55786, &P22272_286};	/*Property = CssColumn*/
const Value V22272_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_287 = {S10, 0, &V22272_287_57173, &P22272_237};	/*Property = CssFlexbox*/
const Value V22272_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_288 = {S11, 0, &V22272_288_57176, &P22272_287};	/*Property = CssFont*/
const Value V22272_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_289 = {S12, 0, &V22272_289_86899, &P22272_288};	/*Property = CssImages*/
const Value V22272_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_291 = {S13, 0, &V22272_291_57182, &P22272_289};	/*Property = CssMediaQueries*/
const Value V22272_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_292 = {S14, 0, &V22272_292_57185, &P22272_291};	/*Property = CssMinMax*/
const Value V22272_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_293 = {S15, 0, &V22272_293_57188, &P22272_292};	/*Property = CssOverflow*/
const Value V22272_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_294 = {S16, 0, &V22272_294_57191, &P22272_293};	/*Property = CssPosition*/
const Value V22272_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_295 = {S17, 0, &V22272_295_57194, &P22272_294};	/*Property = CssText*/
const Value V22272_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_297 = {S18, 0, &V22272_297_57197, &P22272_295};	/*Property = CssTransforms*/
const Value V22272_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_241 = {S19, 0, &V22272_241_55801, &P22272_297};	/*Property = CssTransitions*/
const Value V22272_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_298 = {S20, 0, &V22272_298_57208, &P22272_241};	/*Property = CssUI*/
const Value V22272_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_300 = {S21, 0, &V22272_300_57217, &P22272_298};	/*Property = DataSet*/
const Value V22272_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_301 = {S22, 0, &V22272_301_57220, &P22272_300};	/*Property = DataUrl*/
const Value V22272_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_302 = {S23, 0, &V22272_302_57223, &P22272_301};	/*Property = DeviceOrientation*/
const Value V22272_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_304 = {S24, 0, &V22272_304_57229, &P22272_302};	/*Property = FileReader*/
const Value V22272_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_305 = {S25, 0, &V22272_305_57232, &P22272_304};	/*Property = FileSaver*/
const Value V22272_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_306 = {S26, 0, &V22272_306_57238, &P22272_305};	/*Property = FileWriter*/
const Value V22272_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_307 = {S27, 0, &V22272_307_57241, &P22272_306};	/*Property = FormData*/
const Value V22272_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_309 = {S28, 0, &V22272_309_57247, &P22272_307};	/*Property = Fullscreen*/
const Value V22272_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_310 = {S29, 0, &V22272_310_57250, &P22272_309};	/*Property = GeoLocation*/
const Value V22272_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_312 = {S30, 0, &V22272_312_57256, &P22272_310};	/*Property = History*/
const Value V22272_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_314 = {S31, 0, &V22272_314_57262, &P22272_312};	/*Property = Html5*/
const Value V22272_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_313 = {S32, 0, &V22272_313_57259, &P22272_314};	/*Property = Html-Media-Capture*/
const Value V22272_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_315 = {S33, 0, &V22272_315_57265, &P22272_313};	/*Property = Iframe*/
const Value V22272_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_316 = {S34, 0, &V22272_316_57268, &P22272_315};	/*Property = IndexedDB*/
const Value V22272_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_317 = {S35, 0, &V22272_317_57271, &P22272_316};	/*Property = Json*/
const Value V22272_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P22272_15 = {S37, 0, &V22272_15_30, &P22272_317};	/*Property = LayoutEngine*/
const Value V22272_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_344 = {S38, 0, &V22272_344_57274, &P22272_15};	/*Property = Masking*/
const Value V22272_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_320 = {S39, 0, &V22272_320_57292, &P22272_344};	/*Property = PostMessage*/
const Value V22272_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_321 = {S40, 0, &V22272_321_57295, &P22272_320};	/*Property = Progress*/
const Value V22272_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_322 = {S41, 0, &V22272_322_57298, &P22272_321};	/*Property = Prompts*/
const Value V22272_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_323 = {S42, 0, &V22272_323_57301, &P22272_322};	/*Property = Selector*/
const Value V22272_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_324 = {S43, 0, &V22272_324_57307, &P22272_323};	/*Property = Svg*/
const Value V22272_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_327 = {S44, 0, &V22272_327_57316, &P22272_324};	/*Property = TouchEvents*/
const Value V22272_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_328 = {S45, 0, &V22272_328_57319, &P22272_327};	/*Property = Track*/
const Value V22272_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_329 = {S46, 0, &V22272_329_57325, &P22272_328};	/*Property = Video*/
const Value V22272_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_330 = {S47, 0, &V22272_330_57328, &P22272_329};	/*Property = Viewport*/
const Value V22272_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_332 = {S48, 0, &V22272_332_57340, &P22272_330};	/*Property = WebWorkers*/
const Value V22272_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22272_333 = {S49, 0, &V22272_333_57343, &P22272_332};	/*Property = Xhr2*/
const Profile P22272 = {"22272", &P22272_333};	/*Profile ID - 22272*/
const Value V4578_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_279 = {S1, 0, &V4578_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4578_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_282 = {S3, 0, &V4578_282_57155, &P4578_279};	/*Property = BlobBuilder*/
const Value V4578_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_350 = {S4, 0, &V4578_350_57446, &P4578_282};	/*Property = Canvas*/
const Value V4578_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_283 = {S5, 0, &V4578_283_57161, &P4578_350};	/*Property = CssBackground*/
const Value V4578_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_284 = {S6, 0, &V4578_284_57164, &P4578_283};	/*Property = CssBorderImage*/
const Value V4578_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_285 = {S7, 0, &V4578_285_57167, &P4578_284};	/*Property = CssCanvas*/
const Value V4578_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_286 = {S8, 0, &V4578_286_57170, &P4578_285};	/*Property = CssColor*/
const Value V4578_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_237 = {S9, 0, &V4578_237_55786, &P4578_286};	/*Property = CssColumn*/
const Value V4578_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_287 = {S10, 0, &V4578_287_57173, &P4578_237};	/*Property = CssFlexbox*/
const Value V4578_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_288 = {S11, 0, &V4578_288_57176, &P4578_287};	/*Property = CssFont*/
const Value V4578_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_289 = {S12, 0, &V4578_289_86899, &P4578_288};	/*Property = CssImages*/
const Value V4578_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_291 = {S13, 0, &V4578_291_57182, &P4578_289};	/*Property = CssMediaQueries*/
const Value V4578_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_292 = {S14, 0, &V4578_292_57185, &P4578_291};	/*Property = CssMinMax*/
const Value V4578_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_293 = {S15, 0, &V4578_293_57188, &P4578_292};	/*Property = CssOverflow*/
const Value V4578_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_294 = {S16, 0, &V4578_294_57191, &P4578_293};	/*Property = CssPosition*/
const Value V4578_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_295 = {S17, 0, &V4578_295_57194, &P4578_294};	/*Property = CssText*/
const Value V4578_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_297 = {S18, 0, &V4578_297_57197, &P4578_295};	/*Property = CssTransforms*/
const Value V4578_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_241 = {S19, 0, &V4578_241_55801, &P4578_297};	/*Property = CssTransitions*/
const Value V4578_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_298 = {S20, 0, &V4578_298_57208, &P4578_241};	/*Property = CssUI*/
const Value V4578_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_300 = {S21, 0, &V4578_300_57217, &P4578_298};	/*Property = DataSet*/
const Value V4578_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_301 = {S22, 0, &V4578_301_57220, &P4578_300};	/*Property = DataUrl*/
const Value V4578_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_302 = {S23, 0, &V4578_302_57223, &P4578_301};	/*Property = DeviceOrientation*/
const Value V4578_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_304 = {S24, 0, &V4578_304_57229, &P4578_302};	/*Property = FileReader*/
const Value V4578_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_305 = {S25, 0, &V4578_305_57232, &P4578_304};	/*Property = FileSaver*/
const Value V4578_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_306 = {S26, 0, &V4578_306_57238, &P4578_305};	/*Property = FileWriter*/
const Value V4578_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_307 = {S27, 0, &V4578_307_57241, &P4578_306};	/*Property = FormData*/
const Value V4578_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_309 = {S28, 0, &V4578_309_57247, &P4578_307};	/*Property = Fullscreen*/
const Value V4578_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_310 = {S29, 0, &V4578_310_57250, &P4578_309};	/*Property = GeoLocation*/
const Value V4578_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_312 = {S30, 0, &V4578_312_57256, &P4578_310};	/*Property = History*/
const Value V4578_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_314 = {S31, 0, &V4578_314_57262, &P4578_312};	/*Property = Html5*/
const Value V4578_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_313 = {S32, 0, &V4578_313_57259, &P4578_314};	/*Property = Html-Media-Capture*/
const Value V4578_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_315 = {S33, 0, &V4578_315_57265, &P4578_313};	/*Property = Iframe*/
const Value V4578_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_316 = {S34, 0, &V4578_316_57268, &P4578_315};	/*Property = IndexedDB*/
const Value V4578_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_317 = {S35, 0, &V4578_317_57271, &P4578_316};	/*Property = Json*/
const Value V4578_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_15 = {S37, 0, &V4578_15_72, &P4578_317};	/*Property = LayoutEngine*/
const Value V4578_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_344 = {S38, 0, &V4578_344_57274, &P4578_15};	/*Property = Masking*/
const Value V4578_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_320 = {S39, 0, &V4578_320_57292, &P4578_344};	/*Property = PostMessage*/
const Value V4578_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_321 = {S40, 0, &V4578_321_57295, &P4578_320};	/*Property = Progress*/
const Value V4578_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_322 = {S41, 0, &V4578_322_57298, &P4578_321};	/*Property = Prompts*/
const Value V4578_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_323 = {S42, 0, &V4578_323_57301, &P4578_322};	/*Property = Selector*/
const Value V4578_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_324 = {S43, 0, &V4578_324_57307, &P4578_323};	/*Property = Svg*/
const Value V4578_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_327 = {S44, 0, &V4578_327_57316, &P4578_324};	/*Property = TouchEvents*/
const Value V4578_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_328 = {S45, 0, &V4578_328_57319, &P4578_327};	/*Property = Track*/
const Value V4578_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_329 = {S46, 0, &V4578_329_57325, &P4578_328};	/*Property = Video*/
const Value V4578_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_330 = {S47, 0, &V4578_330_57328, &P4578_329};	/*Property = Viewport*/
const Value V4578_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_332 = {S48, 0, &V4578_332_57340, &P4578_330};	/*Property = WebWorkers*/
const Value V4578_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4578_333 = {S49, 0, &V4578_333_57343, &P4578_332};	/*Property = Xhr2*/
const Profile P4578 = {"4578", &P4578_333};	/*Profile ID - 4578*/
const Value V11970_1_1 = {S2, NULL};	/*Value = True*/
const Property P11970_1 = {S50, 0, &V11970_1_1, NULL};	/*Property = IsMobile*/
const Value V11970_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11970_5 = {S52, 0, &V11970_5_1222, &P11970_1};	/*Property = ScreenPixelsHeight*/
const Value V11970_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11970_4 = {S54, 0, &V11970_4_1224, &P11970_5};	/*Property = ScreenPixelsWidth*/
const Profile P11970 = {"11970", &P11970_4};	/*Profile ID - 11970*/
const Value V4203_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_279 = {S1, 0, &V4203_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4203_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_282 = {S3, 0, &V4203_282_57155, &P4203_279};	/*Property = BlobBuilder*/
const Value V4203_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_350 = {S4, 0, &V4203_350_57446, &P4203_282};	/*Property = Canvas*/
const Value V4203_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_283 = {S5, 0, &V4203_283_57161, &P4203_350};	/*Property = CssBackground*/
const Value V4203_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_284 = {S6, 0, &V4203_284_57164, &P4203_283};	/*Property = CssBorderImage*/
const Value V4203_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_285 = {S7, 0, &V4203_285_57167, &P4203_284};	/*Property = CssCanvas*/
const Value V4203_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_286 = {S8, 0, &V4203_286_57170, &P4203_285};	/*Property = CssColor*/
const Value V4203_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_237 = {S9, 0, &V4203_237_55786, &P4203_286};	/*Property = CssColumn*/
const Value V4203_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_287 = {S10, 0, &V4203_287_57173, &P4203_237};	/*Property = CssFlexbox*/
const Value V4203_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_288 = {S11, 0, &V4203_288_57176, &P4203_287};	/*Property = CssFont*/
const Value V4203_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_289 = {S12, 0, &V4203_289_86899, &P4203_288};	/*Property = CssImages*/
const Value V4203_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_291 = {S13, 0, &V4203_291_57182, &P4203_289};	/*Property = CssMediaQueries*/
const Value V4203_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_292 = {S14, 0, &V4203_292_57185, &P4203_291};	/*Property = CssMinMax*/
const Value V4203_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_293 = {S15, 0, &V4203_293_57188, &P4203_292};	/*Property = CssOverflow*/
const Value V4203_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_294 = {S16, 0, &V4203_294_57191, &P4203_293};	/*Property = CssPosition*/
const Value V4203_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_295 = {S17, 0, &V4203_295_57194, &P4203_294};	/*Property = CssText*/
const Value V4203_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_297 = {S18, 0, &V4203_297_57197, &P4203_295};	/*Property = CssTransforms*/
const Value V4203_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_241 = {S19, 0, &V4203_241_55801, &P4203_297};	/*Property = CssTransitions*/
const Value V4203_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_298 = {S20, 0, &V4203_298_57208, &P4203_241};	/*Property = CssUI*/
const Value V4203_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_300 = {S21, 0, &V4203_300_57217, &P4203_298};	/*Property = DataSet*/
const Value V4203_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_301 = {S22, 0, &V4203_301_57220, &P4203_300};	/*Property = DataUrl*/
const Value V4203_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_302 = {S23, 0, &V4203_302_57223, &P4203_301};	/*Property = DeviceOrientation*/
const Value V4203_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_304 = {S24, 0, &V4203_304_57229, &P4203_302};	/*Property = FileReader*/
const Value V4203_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_305 = {S25, 0, &V4203_305_57232, &P4203_304};	/*Property = FileSaver*/
const Value V4203_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_306 = {S26, 0, &V4203_306_57238, &P4203_305};	/*Property = FileWriter*/
const Value V4203_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_307 = {S27, 0, &V4203_307_57241, &P4203_306};	/*Property = FormData*/
const Value V4203_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_309 = {S28, 0, &V4203_309_57247, &P4203_307};	/*Property = Fullscreen*/
const Value V4203_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_310 = {S29, 0, &V4203_310_57250, &P4203_309};	/*Property = GeoLocation*/
const Value V4203_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_312 = {S30, 0, &V4203_312_57256, &P4203_310};	/*Property = History*/
const Value V4203_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_314 = {S31, 0, &V4203_314_57262, &P4203_312};	/*Property = Html5*/
const Value V4203_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_313 = {S32, 0, &V4203_313_57259, &P4203_314};	/*Property = Html-Media-Capture*/
const Value V4203_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_315 = {S33, 0, &V4203_315_57265, &P4203_313};	/*Property = Iframe*/
const Value V4203_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_316 = {S34, 0, &V4203_316_57268, &P4203_315};	/*Property = IndexedDB*/
const Value V4203_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_317 = {S35, 0, &V4203_317_57271, &P4203_316};	/*Property = Json*/
const Value V4203_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_15 = {S37, 0, &V4203_15_72, &P4203_317};	/*Property = LayoutEngine*/
const Value V4203_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_344 = {S38, 0, &V4203_344_57274, &P4203_15};	/*Property = Masking*/
const Value V4203_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_320 = {S39, 0, &V4203_320_57292, &P4203_344};	/*Property = PostMessage*/
const Value V4203_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_321 = {S40, 0, &V4203_321_57295, &P4203_320};	/*Property = Progress*/
const Value V4203_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_322 = {S41, 0, &V4203_322_57298, &P4203_321};	/*Property = Prompts*/
const Value V4203_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_323 = {S42, 0, &V4203_323_57301, &P4203_322};	/*Property = Selector*/
const Value V4203_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_324 = {S43, 0, &V4203_324_57307, &P4203_323};	/*Property = Svg*/
const Value V4203_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_327 = {S44, 0, &V4203_327_57316, &P4203_324};	/*Property = TouchEvents*/
const Value V4203_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_328 = {S45, 0, &V4203_328_57319, &P4203_327};	/*Property = Track*/
const Value V4203_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_329 = {S46, 0, &V4203_329_57325, &P4203_328};	/*Property = Video*/
const Value V4203_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_330 = {S47, 0, &V4203_330_57328, &P4203_329};	/*Property = Viewport*/
const Value V4203_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_332 = {S48, 0, &V4203_332_57340, &P4203_330};	/*Property = WebWorkers*/
const Value V4203_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4203_333 = {S49, 0, &V4203_333_57343, &P4203_332};	/*Property = Xhr2*/
const Profile P4203 = {"4203", &P4203_333};	/*Profile ID - 4203*/
const Value V11971_1_1 = {S2, NULL};	/*Value = True*/
const Property P11971_1 = {S50, 0, &V11971_1_1, NULL};	/*Property = IsMobile*/
const Value V11971_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11971_5 = {S52, 0, &V11971_5_1293, &P11971_1};	/*Property = ScreenPixelsHeight*/
const Value V11971_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11971_4 = {S54, 0, &V11971_4_1296, &P11971_5};	/*Property = ScreenPixelsWidth*/
const Profile P11971 = {"11971", &P11971_4};	/*Profile ID - 11971*/
const Value V14643_1_1 = {S2, NULL};	/*Value = True*/
const Property P14643_1 = {S50, 0, &V14643_1_1, NULL};	/*Property = IsMobile*/
const Value V14643_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14643_5 = {S52, 0, &V14643_5_1227, &P14643_1};	/*Property = ScreenPixelsHeight*/
const Value V14643_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14643_4 = {S54, 0, &V14643_4_1229, &P14643_5};	/*Property = ScreenPixelsWidth*/
const Profile P14643 = {"14643", &P14643_4};	/*Profile ID - 14643*/
const Value V18336_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_279 = {S1, 0, &V18336_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18336_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_282 = {S3, 0, &V18336_282_57155, &P18336_279};	/*Property = BlobBuilder*/
const Value V18336_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_350 = {S4, 0, &V18336_350_57446, &P18336_282};	/*Property = Canvas*/
const Value V18336_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_283 = {S5, 0, &V18336_283_57161, &P18336_350};	/*Property = CssBackground*/
const Value V18336_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_284 = {S6, 0, &V18336_284_57164, &P18336_283};	/*Property = CssBorderImage*/
const Value V18336_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_285 = {S7, 0, &V18336_285_57167, &P18336_284};	/*Property = CssCanvas*/
const Value V18336_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_286 = {S8, 0, &V18336_286_57170, &P18336_285};	/*Property = CssColor*/
const Value V18336_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_237 = {S9, 0, &V18336_237_55786, &P18336_286};	/*Property = CssColumn*/
const Value V18336_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_287 = {S10, 0, &V18336_287_57173, &P18336_237};	/*Property = CssFlexbox*/
const Value V18336_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_288 = {S11, 0, &V18336_288_57176, &P18336_287};	/*Property = CssFont*/
const Value V18336_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_289 = {S12, 0, &V18336_289_86899, &P18336_288};	/*Property = CssImages*/
const Value V18336_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_291 = {S13, 0, &V18336_291_57182, &P18336_289};	/*Property = CssMediaQueries*/
const Value V18336_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_292 = {S14, 0, &V18336_292_57185, &P18336_291};	/*Property = CssMinMax*/
const Value V18336_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_293 = {S15, 0, &V18336_293_57188, &P18336_292};	/*Property = CssOverflow*/
const Value V18336_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_294 = {S16, 0, &V18336_294_57191, &P18336_293};	/*Property = CssPosition*/
const Value V18336_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_295 = {S17, 0, &V18336_295_57194, &P18336_294};	/*Property = CssText*/
const Value V18336_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_297 = {S18, 0, &V18336_297_57197, &P18336_295};	/*Property = CssTransforms*/
const Value V18336_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_241 = {S19, 0, &V18336_241_55801, &P18336_297};	/*Property = CssTransitions*/
const Value V18336_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_298 = {S20, 0, &V18336_298_57208, &P18336_241};	/*Property = CssUI*/
const Value V18336_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_300 = {S21, 0, &V18336_300_57217, &P18336_298};	/*Property = DataSet*/
const Value V18336_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_301 = {S22, 0, &V18336_301_57220, &P18336_300};	/*Property = DataUrl*/
const Value V18336_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_302 = {S23, 0, &V18336_302_57223, &P18336_301};	/*Property = DeviceOrientation*/
const Value V18336_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_304 = {S24, 0, &V18336_304_57229, &P18336_302};	/*Property = FileReader*/
const Value V18336_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_305 = {S25, 0, &V18336_305_57232, &P18336_304};	/*Property = FileSaver*/
const Value V18336_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_306 = {S26, 0, &V18336_306_57238, &P18336_305};	/*Property = FileWriter*/
const Value V18336_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_307 = {S27, 0, &V18336_307_57241, &P18336_306};	/*Property = FormData*/
const Value V18336_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_309 = {S28, 0, &V18336_309_57247, &P18336_307};	/*Property = Fullscreen*/
const Value V18336_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_310 = {S29, 0, &V18336_310_57250, &P18336_309};	/*Property = GeoLocation*/
const Value V18336_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_312 = {S30, 0, &V18336_312_57256, &P18336_310};	/*Property = History*/
const Value V18336_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_314 = {S31, 0, &V18336_314_57262, &P18336_312};	/*Property = Html5*/
const Value V18336_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_313 = {S32, 0, &V18336_313_57259, &P18336_314};	/*Property = Html-Media-Capture*/
const Value V18336_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_315 = {S33, 0, &V18336_315_57265, &P18336_313};	/*Property = Iframe*/
const Value V18336_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_316 = {S34, 0, &V18336_316_57268, &P18336_315};	/*Property = IndexedDB*/
const Value V18336_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_317 = {S35, 0, &V18336_317_57271, &P18336_316};	/*Property = Json*/
const Value V18336_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18336_15 = {S37, 0, &V18336_15_31, &P18336_317};	/*Property = LayoutEngine*/
const Value V18336_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_344 = {S38, 0, &V18336_344_57274, &P18336_15};	/*Property = Masking*/
const Value V18336_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_320 = {S39, 0, &V18336_320_57292, &P18336_344};	/*Property = PostMessage*/
const Value V18336_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_321 = {S40, 0, &V18336_321_57295, &P18336_320};	/*Property = Progress*/
const Value V18336_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_322 = {S41, 0, &V18336_322_57298, &P18336_321};	/*Property = Prompts*/
const Value V18336_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_323 = {S42, 0, &V18336_323_57301, &P18336_322};	/*Property = Selector*/
const Value V18336_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_324 = {S43, 0, &V18336_324_57307, &P18336_323};	/*Property = Svg*/
const Value V18336_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_327 = {S44, 0, &V18336_327_57316, &P18336_324};	/*Property = TouchEvents*/
const Value V18336_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_328 = {S45, 0, &V18336_328_57319, &P18336_327};	/*Property = Track*/
const Value V18336_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_329 = {S46, 0, &V18336_329_57325, &P18336_328};	/*Property = Video*/
const Value V18336_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_330 = {S47, 0, &V18336_330_57328, &P18336_329};	/*Property = Viewport*/
const Value V18336_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_332 = {S48, 0, &V18336_332_57340, &P18336_330};	/*Property = WebWorkers*/
const Value V18336_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18336_333 = {S49, 0, &V18336_333_57343, &P18336_332};	/*Property = Xhr2*/
const Profile P18336 = {"18336", &P18336_333};	/*Profile ID - 18336*/
const Value V11972_1_1 = {S2, NULL};	/*Value = True*/
const Property P11972_1 = {S50, 0, &V11972_1_1, NULL};	/*Property = IsMobile*/
const Value V11972_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11972_5 = {S52, 0, &V11972_5_1222, &P11972_1};	/*Property = ScreenPixelsHeight*/
const Value V11972_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11972_4 = {S54, 0, &V11972_4_1224, &P11972_5};	/*Property = ScreenPixelsWidth*/
const Profile P11972 = {"11972", &P11972_4};	/*Profile ID - 11972*/
const Value V11973_1_1 = {S2, NULL};	/*Value = True*/
const Property P11973_1 = {S50, 0, &V11973_1_1, NULL};	/*Property = IsMobile*/
const Value V11973_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11973_5 = {S52, 0, &V11973_5_1233, &P11973_1};	/*Property = ScreenPixelsHeight*/
const Value V11973_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11973_4 = {S54, 0, &V11973_4_1235, &P11973_5};	/*Property = ScreenPixelsWidth*/
const Profile P11973 = {"11973", &P11973_4};	/*Profile ID - 11973*/
const Value V3728_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_279 = {S1, 0, &V3728_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3728_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_282 = {S3, 0, &V3728_282_57155, &P3728_279};	/*Property = BlobBuilder*/
const Value V3728_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_350 = {S4, 0, &V3728_350_57446, &P3728_282};	/*Property = Canvas*/
const Value V3728_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_283 = {S5, 0, &V3728_283_57161, &P3728_350};	/*Property = CssBackground*/
const Value V3728_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_284 = {S6, 0, &V3728_284_57164, &P3728_283};	/*Property = CssBorderImage*/
const Value V3728_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_285 = {S7, 0, &V3728_285_57167, &P3728_284};	/*Property = CssCanvas*/
const Value V3728_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_286 = {S8, 0, &V3728_286_57170, &P3728_285};	/*Property = CssColor*/
const Value V3728_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_237 = {S9, 0, &V3728_237_55786, &P3728_286};	/*Property = CssColumn*/
const Value V3728_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_287 = {S10, 0, &V3728_287_57173, &P3728_237};	/*Property = CssFlexbox*/
const Value V3728_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_288 = {S11, 0, &V3728_288_57176, &P3728_287};	/*Property = CssFont*/
const Value V3728_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_289 = {S12, 0, &V3728_289_86899, &P3728_288};	/*Property = CssImages*/
const Value V3728_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_291 = {S13, 0, &V3728_291_57182, &P3728_289};	/*Property = CssMediaQueries*/
const Value V3728_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_292 = {S14, 0, &V3728_292_57185, &P3728_291};	/*Property = CssMinMax*/
const Value V3728_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_293 = {S15, 0, &V3728_293_57188, &P3728_292};	/*Property = CssOverflow*/
const Value V3728_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_294 = {S16, 0, &V3728_294_57191, &P3728_293};	/*Property = CssPosition*/
const Value V3728_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_295 = {S17, 0, &V3728_295_57194, &P3728_294};	/*Property = CssText*/
const Value V3728_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_297 = {S18, 0, &V3728_297_57197, &P3728_295};	/*Property = CssTransforms*/
const Value V3728_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_241 = {S19, 0, &V3728_241_55801, &P3728_297};	/*Property = CssTransitions*/
const Value V3728_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_298 = {S20, 0, &V3728_298_57208, &P3728_241};	/*Property = CssUI*/
const Value V3728_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_300 = {S21, 0, &V3728_300_57217, &P3728_298};	/*Property = DataSet*/
const Value V3728_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_301 = {S22, 0, &V3728_301_57220, &P3728_300};	/*Property = DataUrl*/
const Value V3728_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_302 = {S23, 0, &V3728_302_57223, &P3728_301};	/*Property = DeviceOrientation*/
const Value V3728_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_304 = {S24, 0, &V3728_304_57229, &P3728_302};	/*Property = FileReader*/
const Value V3728_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_305 = {S25, 0, &V3728_305_57232, &P3728_304};	/*Property = FileSaver*/
const Value V3728_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_306 = {S26, 0, &V3728_306_57238, &P3728_305};	/*Property = FileWriter*/
const Value V3728_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_307 = {S27, 0, &V3728_307_57241, &P3728_306};	/*Property = FormData*/
const Value V3728_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_309 = {S28, 0, &V3728_309_57247, &P3728_307};	/*Property = Fullscreen*/
const Value V3728_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_310 = {S29, 0, &V3728_310_57250, &P3728_309};	/*Property = GeoLocation*/
const Value V3728_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_312 = {S30, 0, &V3728_312_57256, &P3728_310};	/*Property = History*/
const Value V3728_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_314 = {S31, 0, &V3728_314_57262, &P3728_312};	/*Property = Html5*/
const Value V3728_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_313 = {S32, 0, &V3728_313_57259, &P3728_314};	/*Property = Html-Media-Capture*/
const Value V3728_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_315 = {S33, 0, &V3728_315_57265, &P3728_313};	/*Property = Iframe*/
const Value V3728_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_316 = {S34, 0, &V3728_316_57268, &P3728_315};	/*Property = IndexedDB*/
const Value V3728_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_317 = {S35, 0, &V3728_317_57271, &P3728_316};	/*Property = Json*/
const Value V3728_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3728_15 = {S37, 0, &V3728_15_28, &P3728_317};	/*Property = LayoutEngine*/
const Value V3728_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_344 = {S38, 0, &V3728_344_57274, &P3728_15};	/*Property = Masking*/
const Value V3728_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_320 = {S39, 0, &V3728_320_57292, &P3728_344};	/*Property = PostMessage*/
const Value V3728_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_321 = {S40, 0, &V3728_321_57295, &P3728_320};	/*Property = Progress*/
const Value V3728_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_322 = {S41, 0, &V3728_322_57298, &P3728_321};	/*Property = Prompts*/
const Value V3728_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_323 = {S42, 0, &V3728_323_57301, &P3728_322};	/*Property = Selector*/
const Value V3728_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_324 = {S43, 0, &V3728_324_57307, &P3728_323};	/*Property = Svg*/
const Value V3728_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_327 = {S44, 0, &V3728_327_57316, &P3728_324};	/*Property = TouchEvents*/
const Value V3728_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_328 = {S45, 0, &V3728_328_57319, &P3728_327};	/*Property = Track*/
const Value V3728_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_329 = {S46, 0, &V3728_329_57325, &P3728_328};	/*Property = Video*/
const Value V3728_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_330 = {S47, 0, &V3728_330_57328, &P3728_329};	/*Property = Viewport*/
const Value V3728_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_332 = {S48, 0, &V3728_332_57340, &P3728_330};	/*Property = WebWorkers*/
const Value V3728_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3728_333 = {S49, 0, &V3728_333_57343, &P3728_332};	/*Property = Xhr2*/
const Profile P3728 = {"3728", &P3728_333};	/*Profile ID - 3728*/
const Value V11977_1_1 = {S2, NULL};	/*Value = True*/
const Property P11977_1 = {S50, 0, &V11977_1_1, NULL};	/*Property = IsMobile*/
const Value V11977_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P11977_5 = {S52, 0, &V11977_5_1304, &P11977_1};	/*Property = ScreenPixelsHeight*/
const Value V11977_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11977_4 = {S54, 0, &V11977_4_1224, &P11977_5};	/*Property = ScreenPixelsWidth*/
const Profile P11977 = {"11977", &P11977_4};	/*Profile ID - 11977*/
const Value V11958_1_1 = {S2, NULL};	/*Value = True*/
const Property P11958_1 = {S50, 0, &V11958_1_1, NULL};	/*Property = IsMobile*/
const Value V11958_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P11958_5 = {S52, 0, &V11958_5_1380, &P11958_1};	/*Property = ScreenPixelsHeight*/
const Value V11958_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11958_4 = {S54, 0, &V11958_4_1224, &P11958_5};	/*Property = ScreenPixelsWidth*/
const Profile P11958 = {"11958", &P11958_4};	/*Profile ID - 11958*/
const Value V11978_1_1 = {S2, NULL};	/*Value = True*/
const Property P11978_1 = {S50, 0, &V11978_1_1, NULL};	/*Property = IsMobile*/
const Value V11978_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11978_5 = {S52, 0, &V11978_5_1222, &P11978_1};	/*Property = ScreenPixelsHeight*/
const Value V11978_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11978_4 = {S54, 0, &V11978_4_1224, &P11978_5};	/*Property = ScreenPixelsWidth*/
const Profile P11978 = {"11978", &P11978_4};	/*Profile ID - 11978*/
const Value V14493_1_1 = {S2, NULL};	/*Value = True*/
const Property P14493_1 = {S50, 0, &V14493_1_1, NULL};	/*Property = IsMobile*/
const Value V14493_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14493_5 = {S52, 0, &V14493_5_1293, &P14493_1};	/*Property = ScreenPixelsHeight*/
const Value V14493_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14493_4 = {S54, 0, &V14493_4_1296, &P14493_5};	/*Property = ScreenPixelsWidth*/
const Profile P14493 = {"14493", &P14493_4};	/*Profile ID - 14493*/
const Value V13392_1_1 = {S2, NULL};	/*Value = True*/
const Property P13392_1 = {S50, 0, &V13392_1_1, NULL};	/*Property = IsMobile*/
const Value V13392_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13392_5 = {S52, 0, &V13392_5_1222, &P13392_1};	/*Property = ScreenPixelsHeight*/
const Value V13392_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13392_4 = {S54, 0, &V13392_4_1224, &P13392_5};	/*Property = ScreenPixelsWidth*/
const Profile P13392 = {"13392", &P13392_4};	/*Profile ID - 13392*/
const Value V11980_1_1 = {S2, NULL};	/*Value = True*/
const Property P11980_1 = {S50, 0, &V11980_1_1, NULL};	/*Property = IsMobile*/
const Value V11980_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11980_5 = {S52, 0, &V11980_5_1222, &P11980_1};	/*Property = ScreenPixelsHeight*/
const Value V11980_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11980_4 = {S54, 0, &V11980_4_1224, &P11980_5};	/*Property = ScreenPixelsWidth*/
const Profile P11980 = {"11980", &P11980_4};	/*Profile ID - 11980*/
const Value V11983_1_1 = {S2, NULL};	/*Value = True*/
const Property P11983_1 = {S50, 0, &V11983_1_1, NULL};	/*Property = IsMobile*/
const Value V11983_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11983_5 = {S52, 0, &V11983_5_1233, &P11983_1};	/*Property = ScreenPixelsHeight*/
const Value V11983_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11983_4 = {S54, 0, &V11983_4_1235, &P11983_5};	/*Property = ScreenPixelsWidth*/
const Profile P11983 = {"11983", &P11983_4};	/*Profile ID - 11983*/
const Value V12351_1_1 = {S2, NULL};	/*Value = True*/
const Property P12351_1 = {S50, 0, &V12351_1_1, NULL};	/*Property = IsMobile*/
const Value V12351_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12351_5 = {S52, 0, &V12351_5_1222, &P12351_1};	/*Property = ScreenPixelsHeight*/
const Value V12351_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12351_4 = {S54, 0, &V12351_4_1224, &P12351_5};	/*Property = ScreenPixelsWidth*/
const Profile P12351 = {"12351", &P12351_4};	/*Profile ID - 12351*/
const Value V12400_1_1 = {S2, NULL};	/*Value = True*/
const Property P12400_1 = {S50, 0, &V12400_1_1, NULL};	/*Property = IsMobile*/
const Value V12400_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12400_5 = {S52, 0, &V12400_5_1222, &P12400_1};	/*Property = ScreenPixelsHeight*/
const Value V12400_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12400_4 = {S54, 0, &V12400_4_1224, &P12400_5};	/*Property = ScreenPixelsWidth*/
const Profile P12400 = {"12400", &P12400_4};	/*Profile ID - 12400*/
const Value V3525_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_279 = {S1, 0, &V3525_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3525_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_282 = {S3, 0, &V3525_282_57155, &P3525_279};	/*Property = BlobBuilder*/
const Value V3525_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_350 = {S4, 0, &V3525_350_57446, &P3525_282};	/*Property = Canvas*/
const Value V3525_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_283 = {S5, 0, &V3525_283_57161, &P3525_350};	/*Property = CssBackground*/
const Value V3525_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_284 = {S6, 0, &V3525_284_57164, &P3525_283};	/*Property = CssBorderImage*/
const Value V3525_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_285 = {S7, 0, &V3525_285_57167, &P3525_284};	/*Property = CssCanvas*/
const Value V3525_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_286 = {S8, 0, &V3525_286_57170, &P3525_285};	/*Property = CssColor*/
const Value V3525_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_237 = {S9, 0, &V3525_237_55786, &P3525_286};	/*Property = CssColumn*/
const Value V3525_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_287 = {S10, 0, &V3525_287_57173, &P3525_237};	/*Property = CssFlexbox*/
const Value V3525_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_288 = {S11, 0, &V3525_288_57176, &P3525_287};	/*Property = CssFont*/
const Value V3525_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_289 = {S12, 0, &V3525_289_86899, &P3525_288};	/*Property = CssImages*/
const Value V3525_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_291 = {S13, 0, &V3525_291_57182, &P3525_289};	/*Property = CssMediaQueries*/
const Value V3525_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_292 = {S14, 0, &V3525_292_57185, &P3525_291};	/*Property = CssMinMax*/
const Value V3525_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_293 = {S15, 0, &V3525_293_57188, &P3525_292};	/*Property = CssOverflow*/
const Value V3525_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_294 = {S16, 0, &V3525_294_57191, &P3525_293};	/*Property = CssPosition*/
const Value V3525_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_295 = {S17, 0, &V3525_295_57194, &P3525_294};	/*Property = CssText*/
const Value V3525_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_297 = {S18, 0, &V3525_297_57197, &P3525_295};	/*Property = CssTransforms*/
const Value V3525_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_241 = {S19, 0, &V3525_241_55801, &P3525_297};	/*Property = CssTransitions*/
const Value V3525_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_298 = {S20, 0, &V3525_298_57208, &P3525_241};	/*Property = CssUI*/
const Value V3525_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_300 = {S21, 0, &V3525_300_57217, &P3525_298};	/*Property = DataSet*/
const Value V3525_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_301 = {S22, 0, &V3525_301_57220, &P3525_300};	/*Property = DataUrl*/
const Value V3525_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_302 = {S23, 0, &V3525_302_57223, &P3525_301};	/*Property = DeviceOrientation*/
const Value V3525_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_304 = {S24, 0, &V3525_304_57229, &P3525_302};	/*Property = FileReader*/
const Value V3525_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_305 = {S25, 0, &V3525_305_57232, &P3525_304};	/*Property = FileSaver*/
const Value V3525_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_306 = {S26, 0, &V3525_306_57238, &P3525_305};	/*Property = FileWriter*/
const Value V3525_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_307 = {S27, 0, &V3525_307_57241, &P3525_306};	/*Property = FormData*/
const Value V3525_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_309 = {S28, 0, &V3525_309_57247, &P3525_307};	/*Property = Fullscreen*/
const Value V3525_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_310 = {S29, 0, &V3525_310_57250, &P3525_309};	/*Property = GeoLocation*/
const Value V3525_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_312 = {S30, 0, &V3525_312_57256, &P3525_310};	/*Property = History*/
const Value V3525_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_314 = {S31, 0, &V3525_314_57262, &P3525_312};	/*Property = Html5*/
const Value V3525_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_313 = {S32, 0, &V3525_313_57259, &P3525_314};	/*Property = Html-Media-Capture*/
const Value V3525_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_315 = {S33, 0, &V3525_315_57265, &P3525_313};	/*Property = Iframe*/
const Value V3525_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_316 = {S34, 0, &V3525_316_57268, &P3525_315};	/*Property = IndexedDB*/
const Value V3525_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_317 = {S35, 0, &V3525_317_57271, &P3525_316};	/*Property = Json*/
const Value V3525_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3525_15 = {S37, 0, &V3525_15_28, &P3525_317};	/*Property = LayoutEngine*/
const Value V3525_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_344 = {S38, 0, &V3525_344_57274, &P3525_15};	/*Property = Masking*/
const Value V3525_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_320 = {S39, 0, &V3525_320_57292, &P3525_344};	/*Property = PostMessage*/
const Value V3525_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_321 = {S40, 0, &V3525_321_57295, &P3525_320};	/*Property = Progress*/
const Value V3525_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_322 = {S41, 0, &V3525_322_57298, &P3525_321};	/*Property = Prompts*/
const Value V3525_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_323 = {S42, 0, &V3525_323_57301, &P3525_322};	/*Property = Selector*/
const Value V3525_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_324 = {S43, 0, &V3525_324_57307, &P3525_323};	/*Property = Svg*/
const Value V3525_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_327 = {S44, 0, &V3525_327_57316, &P3525_324};	/*Property = TouchEvents*/
const Value V3525_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_328 = {S45, 0, &V3525_328_57319, &P3525_327};	/*Property = Track*/
const Value V3525_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_329 = {S46, 0, &V3525_329_57325, &P3525_328};	/*Property = Video*/
const Value V3525_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_330 = {S47, 0, &V3525_330_57328, &P3525_329};	/*Property = Viewport*/
const Value V3525_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_332 = {S48, 0, &V3525_332_57340, &P3525_330};	/*Property = WebWorkers*/
const Value V3525_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3525_333 = {S49, 0, &V3525_333_57343, &P3525_332};	/*Property = Xhr2*/
const Profile P3525 = {"3525", &P3525_333};	/*Profile ID - 3525*/
const Profile P6757 = {"6757", NULL};	/*Profile ID - 6757*/
const Value V12519_1_1 = {S2, NULL};	/*Value = True*/
const Property P12519_1 = {S50, 0, &V12519_1_1, NULL};	/*Property = IsMobile*/
const Value V12519_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12519_5 = {S52, 0, &V12519_5_1277, &P12519_1};	/*Property = ScreenPixelsHeight*/
const Value V12519_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12519_4 = {S54, 0, &V12519_4_1279, &P12519_5};	/*Property = ScreenPixelsWidth*/
const Profile P12519 = {"12519", &P12519_4};	/*Profile ID - 12519*/
const Value V17660_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_279 = {S1, 0, &V17660_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17660_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_282 = {S3, 0, &V17660_282_57155, &P17660_279};	/*Property = BlobBuilder*/
const Value V17660_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_350 = {S4, 0, &V17660_350_57446, &P17660_282};	/*Property = Canvas*/
const Value V17660_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_283 = {S5, 0, &V17660_283_57161, &P17660_350};	/*Property = CssBackground*/
const Value V17660_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_284 = {S6, 0, &V17660_284_57164, &P17660_283};	/*Property = CssBorderImage*/
const Value V17660_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_285 = {S7, 0, &V17660_285_57167, &P17660_284};	/*Property = CssCanvas*/
const Value V17660_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_286 = {S8, 0, &V17660_286_57170, &P17660_285};	/*Property = CssColor*/
const Value V17660_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_237 = {S9, 0, &V17660_237_55786, &P17660_286};	/*Property = CssColumn*/
const Value V17660_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_287 = {S10, 0, &V17660_287_57173, &P17660_237};	/*Property = CssFlexbox*/
const Value V17660_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_288 = {S11, 0, &V17660_288_57176, &P17660_287};	/*Property = CssFont*/
const Value V17660_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_289 = {S12, 0, &V17660_289_86899, &P17660_288};	/*Property = CssImages*/
const Value V17660_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_291 = {S13, 0, &V17660_291_57182, &P17660_289};	/*Property = CssMediaQueries*/
const Value V17660_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_292 = {S14, 0, &V17660_292_57185, &P17660_291};	/*Property = CssMinMax*/
const Value V17660_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_293 = {S15, 0, &V17660_293_57188, &P17660_292};	/*Property = CssOverflow*/
const Value V17660_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_294 = {S16, 0, &V17660_294_57191, &P17660_293};	/*Property = CssPosition*/
const Value V17660_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_295 = {S17, 0, &V17660_295_57194, &P17660_294};	/*Property = CssText*/
const Value V17660_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_297 = {S18, 0, &V17660_297_57197, &P17660_295};	/*Property = CssTransforms*/
const Value V17660_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_241 = {S19, 0, &V17660_241_55801, &P17660_297};	/*Property = CssTransitions*/
const Value V17660_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_298 = {S20, 0, &V17660_298_57208, &P17660_241};	/*Property = CssUI*/
const Value V17660_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_300 = {S21, 0, &V17660_300_57217, &P17660_298};	/*Property = DataSet*/
const Value V17660_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_301 = {S22, 0, &V17660_301_57220, &P17660_300};	/*Property = DataUrl*/
const Value V17660_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_302 = {S23, 0, &V17660_302_57223, &P17660_301};	/*Property = DeviceOrientation*/
const Value V17660_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_304 = {S24, 0, &V17660_304_57229, &P17660_302};	/*Property = FileReader*/
const Value V17660_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_305 = {S25, 0, &V17660_305_57232, &P17660_304};	/*Property = FileSaver*/
const Value V17660_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_306 = {S26, 0, &V17660_306_57238, &P17660_305};	/*Property = FileWriter*/
const Value V17660_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_307 = {S27, 0, &V17660_307_57241, &P17660_306};	/*Property = FormData*/
const Value V17660_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_309 = {S28, 0, &V17660_309_57247, &P17660_307};	/*Property = Fullscreen*/
const Value V17660_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_310 = {S29, 0, &V17660_310_57250, &P17660_309};	/*Property = GeoLocation*/
const Value V17660_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_312 = {S30, 0, &V17660_312_57256, &P17660_310};	/*Property = History*/
const Value V17660_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_314 = {S31, 0, &V17660_314_57262, &P17660_312};	/*Property = Html5*/
const Value V17660_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_313 = {S32, 0, &V17660_313_57259, &P17660_314};	/*Property = Html-Media-Capture*/
const Value V17660_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_315 = {S33, 0, &V17660_315_57265, &P17660_313};	/*Property = Iframe*/
const Value V17660_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_316 = {S34, 0, &V17660_316_57268, &P17660_315};	/*Property = IndexedDB*/
const Value V17660_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_317 = {S35, 0, &V17660_317_57271, &P17660_316};	/*Property = Json*/
const Value V17660_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_15 = {S37, 0, &V17660_15_72, &P17660_317};	/*Property = LayoutEngine*/
const Value V17660_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_344 = {S38, 0, &V17660_344_57274, &P17660_15};	/*Property = Masking*/
const Value V17660_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_320 = {S39, 0, &V17660_320_57292, &P17660_344};	/*Property = PostMessage*/
const Value V17660_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_321 = {S40, 0, &V17660_321_57295, &P17660_320};	/*Property = Progress*/
const Value V17660_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_322 = {S41, 0, &V17660_322_57298, &P17660_321};	/*Property = Prompts*/
const Value V17660_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_323 = {S42, 0, &V17660_323_57301, &P17660_322};	/*Property = Selector*/
const Value V17660_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_324 = {S43, 0, &V17660_324_57307, &P17660_323};	/*Property = Svg*/
const Value V17660_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_327 = {S44, 0, &V17660_327_57316, &P17660_324};	/*Property = TouchEvents*/
const Value V17660_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_328 = {S45, 0, &V17660_328_57319, &P17660_327};	/*Property = Track*/
const Value V17660_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_329 = {S46, 0, &V17660_329_57325, &P17660_328};	/*Property = Video*/
const Value V17660_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_330 = {S47, 0, &V17660_330_57328, &P17660_329};	/*Property = Viewport*/
const Value V17660_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_332 = {S48, 0, &V17660_332_57340, &P17660_330};	/*Property = WebWorkers*/
const Value V17660_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17660_333 = {S49, 0, &V17660_333_57343, &P17660_332};	/*Property = Xhr2*/
const Profile P17660 = {"17660", &P17660_333};	/*Profile ID - 17660*/
const Value V17006_1_1 = {S2, NULL};	/*Value = True*/
const Property P17006_1 = {S50, 0, &V17006_1_1, NULL};	/*Property = IsMobile*/
const Value V17006_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P17006_5 = {S52, 0, &V17006_5_1294, &P17006_1};	/*Property = ScreenPixelsHeight*/
const Value V17006_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17006_4 = {S54, 0, &V17006_4_1296, &P17006_5};	/*Property = ScreenPixelsWidth*/
const Profile P17006 = {"17006", &P17006_4};	/*Profile ID - 17006*/
const Profile P22835 = {"22835", NULL};	/*Profile ID - 22835*/
const Profile P17779 = {"17779", NULL};	/*Profile ID - 17779*/
const Value V17078_1_1 = {S2, NULL};	/*Value = True*/
const Property P17078_1 = {S50, 0, &V17078_1_1, NULL};	/*Property = IsMobile*/
const Value V17078_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17078_5 = {S52, 0, &V17078_5_1293, &P17078_1};	/*Property = ScreenPixelsHeight*/
const Value V17078_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17078_4 = {S54, 0, &V17078_4_1296, &P17078_5};	/*Property = ScreenPixelsWidth*/
const Profile P17078 = {"17078", &P17078_4};	/*Profile ID - 17078*/
const Value V13854_1_1 = {S2, NULL};	/*Value = True*/
const Property P13854_1 = {S50, 0, &V13854_1_1, NULL};	/*Property = IsMobile*/
const Value V13854_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13854_5 = {S52, 0, &V13854_5_1233, &P13854_1};	/*Property = ScreenPixelsHeight*/
const Value V13854_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13854_4 = {S54, 0, &V13854_4_1235, &P13854_5};	/*Property = ScreenPixelsWidth*/
const Profile P13854 = {"13854", &P13854_4};	/*Profile ID - 13854*/
const Value V15298_1_1 = {S2, NULL};	/*Value = True*/
const Property P15298_1 = {S50, 0, &V15298_1_1, NULL};	/*Property = IsMobile*/
const Value V15298_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15298_5 = {S52, 0, &V15298_5_1293, &P15298_1};	/*Property = ScreenPixelsHeight*/
const Value V15298_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15298_4 = {S54, 0, &V15298_4_1296, &P15298_5};	/*Property = ScreenPixelsWidth*/
const Profile P15298 = {"15298", &P15298_4};	/*Profile ID - 15298*/
const Profile P5810 = {"5810", NULL};	/*Profile ID - 5810*/
const Value V14612_1_1 = {S2, NULL};	/*Value = True*/
const Property P14612_1 = {S50, 0, &V14612_1_1, NULL};	/*Property = IsMobile*/
const Value V14612_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14612_5 = {S52, 0, &V14612_5_1233, &P14612_1};	/*Property = ScreenPixelsHeight*/
const Value V14612_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14612_4 = {S54, 0, &V14612_4_1235, &P14612_5};	/*Property = ScreenPixelsWidth*/
const Profile P14612 = {"14612", &P14612_4};	/*Profile ID - 14612*/
const Value V17261_1_1 = {S2, NULL};	/*Value = True*/
const Property P17261_1 = {S50, 0, &V17261_1_1, NULL};	/*Property = IsMobile*/
const Value V17261_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17261_5 = {S52, 0, &V17261_5_1277, &P17261_1};	/*Property = ScreenPixelsHeight*/
const Value V17261_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17261_4 = {S54, 0, &V17261_4_1279, &P17261_5};	/*Property = ScreenPixelsWidth*/
const Profile P17261 = {"17261", &P17261_4};	/*Profile ID - 17261*/
const Value V14633_1_1 = {S2, NULL};	/*Value = True*/
const Property P14633_1 = {S50, 0, &V14633_1_1, NULL};	/*Property = IsMobile*/
const Value V14633_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14633_5 = {S52, 0, &V14633_5_1304, &P14633_1};	/*Property = ScreenPixelsHeight*/
const Value V14633_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14633_4 = {S54, 0, &V14633_4_1224, &P14633_5};	/*Property = ScreenPixelsWidth*/
const Profile P14633 = {"14633", &P14633_4};	/*Profile ID - 14633*/
const Value V11997_1_1 = {S2, NULL};	/*Value = True*/
const Property P11997_1 = {S50, 0, &V11997_1_1, NULL};	/*Property = IsMobile*/
const Value V11997_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11997_5 = {S52, 0, &V11997_5_1293, &P11997_1};	/*Property = ScreenPixelsHeight*/
const Value V11997_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11997_4 = {S54, 0, &V11997_4_1296, &P11997_5};	/*Property = ScreenPixelsWidth*/
const Profile P11997 = {"11997", &P11997_4};	/*Profile ID - 11997*/
const Value V13373_1_1 = {S2, NULL};	/*Value = True*/
const Property P13373_1 = {S50, 0, &V13373_1_1, NULL};	/*Property = IsMobile*/
const Value V13373_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13373_5 = {S52, 0, &V13373_5_1294, &P13373_1};	/*Property = ScreenPixelsHeight*/
const Value V13373_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13373_4 = {S54, 0, &V13373_4_1296, &P13373_5};	/*Property = ScreenPixelsWidth*/
const Profile P13373 = {"13373", &P13373_4};	/*Profile ID - 13373*/
const Value V11999_1_1 = {S2, NULL};	/*Value = True*/
const Property P11999_1 = {S50, 0, &V11999_1_1, NULL};	/*Property = IsMobile*/
const Value V11999_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11999_5 = {S52, 0, &V11999_5_1293, &P11999_1};	/*Property = ScreenPixelsHeight*/
const Value V11999_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11999_4 = {S54, 0, &V11999_4_1296, &P11999_5};	/*Property = ScreenPixelsWidth*/
const Profile P11999 = {"11999", &P11999_4};	/*Profile ID - 11999*/
const Value V12005_1_1 = {S2, NULL};	/*Value = True*/
const Property P12005_1 = {S50, 0, &V12005_1_1, NULL};	/*Property = IsMobile*/
const Value V12005_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12005_5 = {S52, 0, &V12005_5_1222, &P12005_1};	/*Property = ScreenPixelsHeight*/
const Value V12005_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P12005_4 = {S54, 0, &V12005_4_8554, &P12005_5};	/*Property = ScreenPixelsWidth*/
const Profile P12005 = {"12005", &P12005_4};	/*Profile ID - 12005*/
const Value V11680_1_1 = {S2, NULL};	/*Value = True*/
const Property P11680_1 = {S50, 0, &V11680_1_1, NULL};	/*Property = IsMobile*/
const Value V11680_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P11680_5 = {S52, 0, &V11680_5_1227, &P11680_1};	/*Property = ScreenPixelsHeight*/
const Value V11680_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P11680_4 = {S54, 0, &V11680_4_1229, &P11680_5};	/*Property = ScreenPixelsWidth*/
const Profile P11680 = {"11680", &P11680_4};	/*Profile ID - 11680*/
const Value V12165_1_1 = {S2, NULL};	/*Value = True*/
const Property P12165_1 = {S50, 0, &V12165_1_1, NULL};	/*Property = IsMobile*/
const Value V12165_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12165_5 = {S52, 0, &V12165_5_1304, &P12165_1};	/*Property = ScreenPixelsHeight*/
const Value V12165_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12165_4 = {S54, 0, &V12165_4_1224, &P12165_5};	/*Property = ScreenPixelsWidth*/
const Profile P12165 = {"12165", &P12165_4};	/*Profile ID - 12165*/
const Value V15257_1_1 = {S2, NULL};	/*Value = True*/
const Property P15257_1 = {S50, 0, &V15257_1_1, NULL};	/*Property = IsMobile*/
const Value V15257_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15257_5 = {S52, 0, &V15257_5_1380, &P15257_1};	/*Property = ScreenPixelsHeight*/
const Value V15257_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15257_4 = {S54, 0, &V15257_4_1279, &P15257_5};	/*Property = ScreenPixelsWidth*/
const Profile P15257 = {"15257", &P15257_4};	/*Profile ID - 15257*/
const Profile P6011 = {"6011", NULL};	/*Profile ID - 6011*/
const Value V14533_1_1 = {S2, NULL};	/*Value = True*/
const Property P14533_1 = {S50, 0, &V14533_1_1, NULL};	/*Property = IsMobile*/
const Value V14533_5_1796 = {S64, NULL};	/*Value = 360*/
const Property P14533_5 = {S52, 0, &V14533_5_1796, &P14533_1};	/*Property = ScreenPixelsHeight*/
const Value V14533_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14533_4 = {S54, 0, &V14533_4_1229, &P14533_5};	/*Property = ScreenPixelsWidth*/
const Profile P14533 = {"14533", &P14533_4};	/*Profile ID - 14533*/
const Value V12018_1_1 = {S2, NULL};	/*Value = True*/
const Property P12018_1 = {S50, 0, &V12018_1_1, NULL};	/*Property = IsMobile*/
const Value V12018_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12018_5 = {S52, 0, &V12018_5_1222, &P12018_1};	/*Property = ScreenPixelsHeight*/
const Value V12018_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12018_4 = {S54, 0, &V12018_4_1224, &P12018_5};	/*Property = ScreenPixelsWidth*/
const Profile P12018 = {"12018", &P12018_4};	/*Profile ID - 12018*/
const Value V12024_1_1 = {S2, NULL};	/*Value = True*/
const Property P12024_1 = {S50, 0, &V12024_1_1, NULL};	/*Property = IsMobile*/
const Value V12024_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12024_5 = {S52, 0, &V12024_5_1222, &P12024_1};	/*Property = ScreenPixelsHeight*/
const Value V12024_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12024_4 = {S54, 0, &V12024_4_1224, &P12024_5};	/*Property = ScreenPixelsWidth*/
const Profile P12024 = {"12024", &P12024_4};	/*Profile ID - 12024*/
const Value V12025_1_1 = {S2, NULL};	/*Value = True*/
const Property P12025_1 = {S50, 0, &V12025_1_1, NULL};	/*Property = IsMobile*/
const Value V12025_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12025_5 = {S52, 0, &V12025_5_1222, &P12025_1};	/*Property = ScreenPixelsHeight*/
const Value V12025_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12025_4 = {S54, 0, &V12025_4_1224, &P12025_5};	/*Property = ScreenPixelsWidth*/
const Profile P12025 = {"12025", &P12025_4};	/*Profile ID - 12025*/
const Value V16213_1_1 = {S2, NULL};	/*Value = True*/
const Property P16213_1 = {S50, 0, &V16213_1_1, NULL};	/*Property = IsMobile*/
const Value V16213_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16213_5 = {S52, 0, &V16213_5_1222, &P16213_1};	/*Property = ScreenPixelsHeight*/
const Value V16213_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16213_4 = {S54, 0, &V16213_4_1224, &P16213_5};	/*Property = ScreenPixelsWidth*/
const Profile P16213 = {"16213", &P16213_4};	/*Profile ID - 16213*/
const Value V12558_1_1 = {S2, NULL};	/*Value = True*/
const Property P12558_1 = {S50, 0, &V12558_1_1, NULL};	/*Property = IsMobile*/
const Value V12558_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12558_5 = {S52, 0, &V12558_5_1222, &P12558_1};	/*Property = ScreenPixelsHeight*/
const Value V12558_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12558_4 = {S54, 0, &V12558_4_1224, &P12558_5};	/*Property = ScreenPixelsWidth*/
const Profile P12558 = {"12558", &P12558_4};	/*Profile ID - 12558*/
const Value V16177_1_1 = {S2, NULL};	/*Value = True*/
const Property P16177_1 = {S50, 0, &V16177_1_1, NULL};	/*Property = IsMobile*/
const Value V16177_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16177_5 = {S52, 0, &V16177_5_1227, &P16177_1};	/*Property = ScreenPixelsHeight*/
const Value V16177_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16177_4 = {S54, 0, &V16177_4_1229, &P16177_5};	/*Property = ScreenPixelsWidth*/
const Profile P16177 = {"16177", &P16177_4};	/*Profile ID - 16177*/
const Value V12023_1_1 = {S2, NULL};	/*Value = True*/
const Property P12023_1 = {S50, 0, &V12023_1_1, NULL};	/*Property = IsMobile*/
const Value V12023_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12023_5 = {S52, 0, &V12023_5_1227, &P12023_1};	/*Property = ScreenPixelsHeight*/
const Value V12023_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12023_4 = {S54, 0, &V12023_4_1229, &P12023_5};	/*Property = ScreenPixelsWidth*/
const Profile P12023 = {"12023", &P12023_4};	/*Profile ID - 12023*/
const Value V11941_1_1 = {S2, NULL};	/*Value = True*/
const Property P11941_1 = {S50, 0, &V11941_1_1, NULL};	/*Property = IsMobile*/
const Value V11941_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11941_5 = {S52, 0, &V11941_5_1222, &P11941_1};	/*Property = ScreenPixelsHeight*/
const Value V11941_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11941_4 = {S54, 0, &V11941_4_1224, &P11941_5};	/*Property = ScreenPixelsWidth*/
const Profile P11941 = {"11941", &P11941_4};	/*Profile ID - 11941*/
const Value V12029_1_1 = {S2, NULL};	/*Value = True*/
const Property P12029_1 = {S50, 0, &V12029_1_1, NULL};	/*Property = IsMobile*/
const Value V12029_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12029_5 = {S52, 0, &V12029_5_1222, &P12029_1};	/*Property = ScreenPixelsHeight*/
const Value V12029_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12029_4 = {S54, 0, &V12029_4_1224, &P12029_5};	/*Property = ScreenPixelsWidth*/
const Profile P12029 = {"12029", &P12029_4};	/*Profile ID - 12029*/
const Value V12034_1_1 = {S2, NULL};	/*Value = True*/
const Property P12034_1 = {S50, 0, &V12034_1_1, NULL};	/*Property = IsMobile*/
const Value V12034_5_31826 = {S75, NULL};	/*Value = 260*/
const Property P12034_5 = {S52, 0, &V12034_5_31826, &P12034_1};	/*Property = ScreenPixelsHeight*/
const Value V12034_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12034_4 = {S54, 0, &V12034_4_1224, &P12034_5};	/*Property = ScreenPixelsWidth*/
const Profile P12034 = {"12034", &P12034_4};	/*Profile ID - 12034*/
const Value V12035_1_1 = {S2, NULL};	/*Value = True*/
const Property P12035_1 = {S50, 0, &V12035_1_1, NULL};	/*Property = IsMobile*/
const Value V12035_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12035_5 = {S52, 0, &V12035_5_1293, &P12035_1};	/*Property = ScreenPixelsHeight*/
const Value V12035_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12035_4 = {S54, 0, &V12035_4_1296, &P12035_5};	/*Property = ScreenPixelsWidth*/
const Profile P12035 = {"12035", &P12035_4};	/*Profile ID - 12035*/
const Value V11984_1_1 = {S2, NULL};	/*Value = True*/
const Property P11984_1 = {S50, 0, &V11984_1_1, NULL};	/*Property = IsMobile*/
const Value V11984_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11984_5 = {S52, 0, &V11984_5_1222, &P11984_1};	/*Property = ScreenPixelsHeight*/
const Value V11984_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11984_4 = {S54, 0, &V11984_4_1224, &P11984_5};	/*Property = ScreenPixelsWidth*/
const Profile P11984 = {"11984", &P11984_4};	/*Profile ID - 11984*/
const Value V12039_1_1 = {S2, NULL};	/*Value = True*/
const Property P12039_1 = {S50, 0, &V12039_1_1, NULL};	/*Property = IsMobile*/
const Value V12039_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12039_5 = {S52, 0, &V12039_5_1222, &P12039_1};	/*Property = ScreenPixelsHeight*/
const Value V12039_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12039_4 = {S54, 0, &V12039_4_1224, &P12039_5};	/*Property = ScreenPixelsWidth*/
const Profile P12039 = {"12039", &P12039_4};	/*Profile ID - 12039*/
const Value V11728_1_1 = {S2, NULL};	/*Value = True*/
const Property P11728_1 = {S50, 0, &V11728_1_1, NULL};	/*Property = IsMobile*/
const Value V11728_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P11728_5 = {S52, 0, &V11728_5_1380, &P11728_1};	/*Property = ScreenPixelsHeight*/
const Value V11728_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P11728_4 = {S54, 0, &V11728_4_1279, &P11728_5};	/*Property = ScreenPixelsWidth*/
const Profile P11728 = {"11728", &P11728_4};	/*Profile ID - 11728*/
const Value V12040_1_1 = {S2, NULL};	/*Value = True*/
const Property P12040_1 = {S50, 0, &V12040_1_1, NULL};	/*Property = IsMobile*/
const Value V12040_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12040_5 = {S52, 0, &V12040_5_1380, &P12040_1};	/*Property = ScreenPixelsHeight*/
const Value V12040_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12040_4 = {S54, 0, &V12040_4_1279, &P12040_5};	/*Property = ScreenPixelsWidth*/
const Profile P12040 = {"12040", &P12040_4};	/*Profile ID - 12040*/
const Value V11653_1_1 = {S2, NULL};	/*Value = True*/
const Property P11653_1 = {S50, 0, &V11653_1_1, NULL};	/*Property = IsMobile*/
const Value V11653_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11653_5 = {S52, 0, &V11653_5_1222, &P11653_1};	/*Property = ScreenPixelsHeight*/
const Value V11653_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11653_4 = {S54, 0, &V11653_4_1224, &P11653_5};	/*Property = ScreenPixelsWidth*/
const Profile P11653 = {"11653", &P11653_4};	/*Profile ID - 11653*/
const Value V12059_1_1 = {S2, NULL};	/*Value = True*/
const Property P12059_1 = {S50, 0, &V12059_1_1, NULL};	/*Property = IsMobile*/
const Value V12059_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12059_5 = {S52, 0, &V12059_5_1293, &P12059_1};	/*Property = ScreenPixelsHeight*/
const Value V12059_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12059_4 = {S54, 0, &V12059_4_1296, &P12059_5};	/*Property = ScreenPixelsWidth*/
const Profile P12059 = {"12059", &P12059_4};	/*Profile ID - 12059*/
const Value V12210_1_1 = {S2, NULL};	/*Value = True*/
const Property P12210_1 = {S50, 0, &V12210_1_1, NULL};	/*Property = IsMobile*/
const Value V12210_5_31826 = {S75, NULL};	/*Value = 260*/
const Property P12210_5 = {S52, 0, &V12210_5_31826, &P12210_1};	/*Property = ScreenPixelsHeight*/
const Value V12210_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12210_4 = {S54, 0, &V12210_4_1224, &P12210_5};	/*Property = ScreenPixelsWidth*/
const Profile P12210 = {"12210", &P12210_4};	/*Profile ID - 12210*/
const Value V13921_1_1 = {S2, NULL};	/*Value = True*/
const Property P13921_1 = {S50, 0, &V13921_1_1, NULL};	/*Property = IsMobile*/
const Value V13921_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13921_5 = {S52, 0, &V13921_5_1233, &P13921_1};	/*Property = ScreenPixelsHeight*/
const Value V13921_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13921_4 = {S54, 0, &V13921_4_1235, &P13921_5};	/*Property = ScreenPixelsWidth*/
const Profile P13921 = {"13921", &P13921_4};	/*Profile ID - 13921*/
const Value V11707_1_1 = {S2, NULL};	/*Value = True*/
const Property P11707_1 = {S50, 0, &V11707_1_1, NULL};	/*Property = IsMobile*/
const Value V11707_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11707_5 = {S52, 0, &V11707_5_1293, &P11707_1};	/*Property = ScreenPixelsHeight*/
const Value V11707_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11707_4 = {S54, 0, &V11707_4_1296, &P11707_5};	/*Property = ScreenPixelsWidth*/
const Profile P11707 = {"11707", &P11707_4};	/*Profile ID - 11707*/
const Value V12042_1_1 = {S2, NULL};	/*Value = True*/
const Property P12042_1 = {S50, 0, &V12042_1_1, NULL};	/*Property = IsMobile*/
const Value V12042_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12042_5 = {S52, 0, &V12042_5_1233, &P12042_1};	/*Property = ScreenPixelsHeight*/
const Value V12042_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12042_4 = {S54, 0, &V12042_4_1235, &P12042_5};	/*Property = ScreenPixelsWidth*/
const Profile P12042 = {"12042", &P12042_4};	/*Profile ID - 12042*/
const Value V14896_1_1 = {S2, NULL};	/*Value = True*/
const Property P14896_1 = {S50, 0, &V14896_1_1, NULL};	/*Property = IsMobile*/
const Value V14896_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14896_5 = {S52, 0, &V14896_5_1294, &P14896_1};	/*Property = ScreenPixelsHeight*/
const Value V14896_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14896_4 = {S54, 0, &V14896_4_1296, &P14896_5};	/*Property = ScreenPixelsWidth*/
const Profile P14896 = {"14896", &P14896_4};	/*Profile ID - 14896*/
const Value V12045_1_1 = {S2, NULL};	/*Value = True*/
const Property P12045_1 = {S50, 0, &V12045_1_1, NULL};	/*Property = IsMobile*/
const Value V12045_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12045_5 = {S52, 0, &V12045_5_1222, &P12045_1};	/*Property = ScreenPixelsHeight*/
const Value V12045_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12045_4 = {S54, 0, &V12045_4_1224, &P12045_5};	/*Property = ScreenPixelsWidth*/
const Profile P12045 = {"12045", &P12045_4};	/*Profile ID - 12045*/
const Value V16108_1_1 = {S2, NULL};	/*Value = True*/
const Property P16108_1 = {S50, 0, &V16108_1_1, NULL};	/*Property = IsMobile*/
const Value V16108_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16108_5 = {S52, 0, &V16108_5_1222, &P16108_1};	/*Property = ScreenPixelsHeight*/
const Value V16108_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16108_4 = {S54, 0, &V16108_4_1224, &P16108_5};	/*Property = ScreenPixelsWidth*/
const Profile P16108 = {"16108", &P16108_4};	/*Profile ID - 16108*/
const Value V13241_1_1 = {S2, NULL};	/*Value = True*/
const Property P13241_1 = {S50, 0, &V13241_1_1, NULL};	/*Property = IsMobile*/
const Value V13241_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13241_5 = {S52, 0, &V13241_5_1380, &P13241_1};	/*Property = ScreenPixelsHeight*/
const Value V13241_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13241_4 = {S54, 0, &V13241_4_1279, &P13241_5};	/*Property = ScreenPixelsWidth*/
const Profile P13241 = {"13241", &P13241_4};	/*Profile ID - 13241*/
const Value V11870_1_1 = {S2, NULL};	/*Value = True*/
const Property P11870_1 = {S50, 0, &V11870_1_1, NULL};	/*Property = IsMobile*/
const Value V11870_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11870_5 = {S52, 0, &V11870_5_1222, &P11870_1};	/*Property = ScreenPixelsHeight*/
const Value V11870_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11870_4 = {S54, 0, &V11870_4_1224, &P11870_5};	/*Property = ScreenPixelsWidth*/
const Profile P11870 = {"11870", &P11870_4};	/*Profile ID - 11870*/
const Value V12048_1_1 = {S2, NULL};	/*Value = True*/
const Property P12048_1 = {S50, 0, &V12048_1_1, NULL};	/*Property = IsMobile*/
const Value V12048_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12048_5 = {S52, 0, &V12048_5_1380, &P12048_1};	/*Property = ScreenPixelsHeight*/
const Value V12048_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12048_4 = {S54, 0, &V12048_4_1279, &P12048_5};	/*Property = ScreenPixelsWidth*/
const Profile P12048 = {"12048", &P12048_4};	/*Profile ID - 12048*/
const Value V17925_1_1 = {S2, NULL};	/*Value = True*/
const Property P17925_1 = {S50, 0, &V17925_1_1, NULL};	/*Property = IsMobile*/
const Value V17925_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P17925_5 = {S52, 0, &V17925_5_1380, &P17925_1};	/*Property = ScreenPixelsHeight*/
const Value V17925_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17925_4 = {S54, 0, &V17925_4_1279, &P17925_5};	/*Property = ScreenPixelsWidth*/
const Profile P17925 = {"17925", &P17925_4};	/*Profile ID - 17925*/
const Value V12049_1_1 = {S2, NULL};	/*Value = True*/
const Property P12049_1 = {S50, 0, &V12049_1_1, NULL};	/*Property = IsMobile*/
const Value V12049_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12049_5 = {S52, 0, &V12049_5_1293, &P12049_1};	/*Property = ScreenPixelsHeight*/
const Value V12049_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12049_4 = {S54, 0, &V12049_4_1296, &P12049_5};	/*Property = ScreenPixelsWidth*/
const Profile P12049 = {"12049", &P12049_4};	/*Profile ID - 12049*/
const Value V14782_1_1 = {S2, NULL};	/*Value = True*/
const Property P14782_1 = {S50, 0, &V14782_1_1, NULL};	/*Property = IsMobile*/
const Value V14782_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14782_5 = {S52, 0, &V14782_5_1380, &P14782_1};	/*Property = ScreenPixelsHeight*/
const Value V14782_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14782_4 = {S54, 0, &V14782_4_1279, &P14782_5};	/*Property = ScreenPixelsWidth*/
const Profile P14782 = {"14782", &P14782_4};	/*Profile ID - 14782*/
const Value V13770_1_1 = {S2, NULL};	/*Value = True*/
const Property P13770_1 = {S50, 0, &V13770_1_1, NULL};	/*Property = IsMobile*/
const Value V13770_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13770_5 = {S52, 0, &V13770_5_1222, &P13770_1};	/*Property = ScreenPixelsHeight*/
const Value V13770_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13770_4 = {S54, 0, &V13770_4_1224, &P13770_5};	/*Property = ScreenPixelsWidth*/
const Profile P13770 = {"13770", &P13770_4};	/*Profile ID - 13770*/
const Profile P17895 = {"17895", NULL};	/*Profile ID - 17895*/
const Value V17896_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_279 = {S1, 0, &V17896_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17896_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_282 = {S3, 0, &V17896_282_57155, &P17896_279};	/*Property = BlobBuilder*/
const Value V17896_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_350 = {S4, 0, &V17896_350_57446, &P17896_282};	/*Property = Canvas*/
const Value V17896_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_283 = {S5, 0, &V17896_283_57161, &P17896_350};	/*Property = CssBackground*/
const Value V17896_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_284 = {S6, 0, &V17896_284_57164, &P17896_283};	/*Property = CssBorderImage*/
const Value V17896_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_285 = {S7, 0, &V17896_285_57167, &P17896_284};	/*Property = CssCanvas*/
const Value V17896_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_286 = {S8, 0, &V17896_286_57170, &P17896_285};	/*Property = CssColor*/
const Value V17896_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_237 = {S9, 0, &V17896_237_55786, &P17896_286};	/*Property = CssColumn*/
const Value V17896_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_287 = {S10, 0, &V17896_287_57173, &P17896_237};	/*Property = CssFlexbox*/
const Value V17896_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_288 = {S11, 0, &V17896_288_57176, &P17896_287};	/*Property = CssFont*/
const Value V17896_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_289 = {S12, 0, &V17896_289_86899, &P17896_288};	/*Property = CssImages*/
const Value V17896_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_291 = {S13, 0, &V17896_291_57182, &P17896_289};	/*Property = CssMediaQueries*/
const Value V17896_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_292 = {S14, 0, &V17896_292_57185, &P17896_291};	/*Property = CssMinMax*/
const Value V17896_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_293 = {S15, 0, &V17896_293_57188, &P17896_292};	/*Property = CssOverflow*/
const Value V17896_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_294 = {S16, 0, &V17896_294_57191, &P17896_293};	/*Property = CssPosition*/
const Value V17896_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_295 = {S17, 0, &V17896_295_57194, &P17896_294};	/*Property = CssText*/
const Value V17896_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_297 = {S18, 0, &V17896_297_57197, &P17896_295};	/*Property = CssTransforms*/
const Value V17896_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_241 = {S19, 0, &V17896_241_55801, &P17896_297};	/*Property = CssTransitions*/
const Value V17896_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_298 = {S20, 0, &V17896_298_57208, &P17896_241};	/*Property = CssUI*/
const Value V17896_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_300 = {S21, 0, &V17896_300_57217, &P17896_298};	/*Property = DataSet*/
const Value V17896_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_301 = {S22, 0, &V17896_301_57220, &P17896_300};	/*Property = DataUrl*/
const Value V17896_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_302 = {S23, 0, &V17896_302_57223, &P17896_301};	/*Property = DeviceOrientation*/
const Value V17896_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_304 = {S24, 0, &V17896_304_57229, &P17896_302};	/*Property = FileReader*/
const Value V17896_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_305 = {S25, 0, &V17896_305_57232, &P17896_304};	/*Property = FileSaver*/
const Value V17896_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_306 = {S26, 0, &V17896_306_57238, &P17896_305};	/*Property = FileWriter*/
const Value V17896_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_307 = {S27, 0, &V17896_307_57241, &P17896_306};	/*Property = FormData*/
const Value V17896_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_309 = {S28, 0, &V17896_309_57247, &P17896_307};	/*Property = Fullscreen*/
const Value V17896_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_310 = {S29, 0, &V17896_310_57250, &P17896_309};	/*Property = GeoLocation*/
const Value V17896_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_312 = {S30, 0, &V17896_312_57256, &P17896_310};	/*Property = History*/
const Value V17896_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_314 = {S31, 0, &V17896_314_57262, &P17896_312};	/*Property = Html5*/
const Value V17896_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_313 = {S32, 0, &V17896_313_57259, &P17896_314};	/*Property = Html-Media-Capture*/
const Value V17896_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_315 = {S33, 0, &V17896_315_57265, &P17896_313};	/*Property = Iframe*/
const Value V17896_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_316 = {S34, 0, &V17896_316_57268, &P17896_315};	/*Property = IndexedDB*/
const Value V17896_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_317 = {S35, 0, &V17896_317_57271, &P17896_316};	/*Property = Json*/
const Value V17896_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_15 = {S37, 0, &V17896_15_72, &P17896_317};	/*Property = LayoutEngine*/
const Value V17896_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_344 = {S38, 0, &V17896_344_57274, &P17896_15};	/*Property = Masking*/
const Value V17896_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_320 = {S39, 0, &V17896_320_57292, &P17896_344};	/*Property = PostMessage*/
const Value V17896_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_321 = {S40, 0, &V17896_321_57295, &P17896_320};	/*Property = Progress*/
const Value V17896_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_322 = {S41, 0, &V17896_322_57298, &P17896_321};	/*Property = Prompts*/
const Value V17896_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_323 = {S42, 0, &V17896_323_57301, &P17896_322};	/*Property = Selector*/
const Value V17896_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_324 = {S43, 0, &V17896_324_57307, &P17896_323};	/*Property = Svg*/
const Value V17896_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_327 = {S44, 0, &V17896_327_57316, &P17896_324};	/*Property = TouchEvents*/
const Value V17896_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_328 = {S45, 0, &V17896_328_57319, &P17896_327};	/*Property = Track*/
const Value V17896_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_329 = {S46, 0, &V17896_329_57325, &P17896_328};	/*Property = Video*/
const Value V17896_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_330 = {S47, 0, &V17896_330_57328, &P17896_329};	/*Property = Viewport*/
const Value V17896_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_332 = {S48, 0, &V17896_332_57340, &P17896_330};	/*Property = WebWorkers*/
const Value V17896_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17896_333 = {S49, 0, &V17896_333_57343, &P17896_332};	/*Property = Xhr2*/
const Profile P17896 = {"17896", &P17896_333};	/*Profile ID - 17896*/
const Value V17897_1_1 = {S2, NULL};	/*Value = True*/
const Property P17897_1 = {S50, 0, &V17897_1_1, NULL};	/*Property = IsMobile*/
const Value V17897_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17897_5 = {S52, 0, &V17897_5_1222, &P17897_1};	/*Property = ScreenPixelsHeight*/
const Value V17897_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17897_4 = {S54, 0, &V17897_4_1224, &P17897_5};	/*Property = ScreenPixelsWidth*/
const Profile P17897 = {"17897", &P17897_4};	/*Profile ID - 17897*/
const Value V16003_1_1 = {S2, NULL};	/*Value = True*/
const Property P16003_1 = {S50, 0, &V16003_1_1, NULL};	/*Property = IsMobile*/
const Value V16003_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16003_5 = {S52, 0, &V16003_5_1222, &P16003_1};	/*Property = ScreenPixelsHeight*/
const Value V16003_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16003_4 = {S54, 0, &V16003_4_1224, &P16003_5};	/*Property = ScreenPixelsWidth*/
const Profile P16003 = {"16003", &P16003_4};	/*Profile ID - 16003*/
const Value V15518_1_1 = {S2, NULL};	/*Value = True*/
const Property P15518_1 = {S50, 0, &V15518_1_1, NULL};	/*Property = IsMobile*/
const Value V15518_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15518_5 = {S52, 0, &V15518_5_1304, &P15518_1};	/*Property = ScreenPixelsHeight*/
const Value V15518_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15518_4 = {S54, 0, &V15518_4_1224, &P15518_5};	/*Property = ScreenPixelsWidth*/
const Profile P15518 = {"15518", &P15518_4};	/*Profile ID - 15518*/
const Value V14784_1_1 = {S2, NULL};	/*Value = True*/
const Property P14784_1 = {S50, 0, &V14784_1_1, NULL};	/*Property = IsMobile*/
const Value V14784_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14784_5 = {S52, 0, &V14784_5_1277, &P14784_1};	/*Property = ScreenPixelsHeight*/
const Value V14784_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14784_4 = {S54, 0, &V14784_4_1279, &P14784_5};	/*Property = ScreenPixelsWidth*/
const Profile P14784 = {"14784", &P14784_4};	/*Profile ID - 14784*/
const Profile P17979 = {"17979", NULL};	/*Profile ID - 17979*/
const Value V20428_279_57147 = {S2, NULL};	/*Value = True*/
const Property P20428_279 = {S1, 0, &V20428_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V20428_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20428_282 = {S3, 0, &V20428_282_57154, &P20428_279};	/*Property = BlobBuilder*/
const Value V20428_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20428_350 = {S4, 0, &V20428_350_57444, &P20428_282};	/*Property = Canvas*/
const Value V20428_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20428_283 = {S5, 0, &V20428_283_57159, &P20428_350};	/*Property = CssBackground*/
const Value V20428_284_57162 = {S2, NULL};	/*Value = True*/
const Property P20428_284 = {S6, 0, &V20428_284_57162, &P20428_283};	/*Property = CssBorderImage*/
const Value V20428_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20428_285 = {S7, 0, &V20428_285_57166, &P20428_284};	/*Property = CssCanvas*/
const Value V20428_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20428_286 = {S8, 0, &V20428_286_57168, &P20428_285};	/*Property = CssColor*/
const Value V20428_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20428_237 = {S9, 0, &V20428_237_55786, &P20428_286};	/*Property = CssColumn*/
const Value V20428_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20428_287 = {S10, 0, &V20428_287_57172, &P20428_237};	/*Property = CssFlexbox*/
const Value V20428_288_57174 = {S2, NULL};	/*Value = True*/
const Property P20428_288 = {S11, 0, &V20428_288_57174, &P20428_287};	/*Property = CssFont*/
const Value V20428_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20428_289 = {S12, 0, &V20428_289_57759, &P20428_288};	/*Property = CssImages*/
const Value V20428_291_57180 = {S2, NULL};	/*Value = True*/
const Property P20428_291 = {S13, 0, &V20428_291_57180, &P20428_289};	/*Property = CssMediaQueries*/
const Value V20428_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20428_292 = {S14, 0, &V20428_292_57183, &P20428_291};	/*Property = CssMinMax*/
const Value V20428_293_57186 = {S2, NULL};	/*Value = True*/
const Property P20428_293 = {S15, 0, &V20428_293_57186, &P20428_292};	/*Property = CssOverflow*/
const Value V20428_294_57189 = {S2, NULL};	/*Value = True*/
const Property P20428_294 = {S16, 0, &V20428_294_57189, &P20428_293};	/*Property = CssPosition*/
const Value V20428_295_57192 = {S2, NULL};	/*Value = True*/
const Property P20428_295 = {S17, 0, &V20428_295_57192, &P20428_294};	/*Property = CssText*/
const Value V20428_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20428_297 = {S18, 0, &V20428_297_57195, &P20428_295};	/*Property = CssTransforms*/
const Value V20428_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20428_241 = {S19, 0, &V20428_241_55799, &P20428_297};	/*Property = CssTransitions*/
const Value V20428_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20428_298 = {S20, 0, &V20428_298_57206, &P20428_241};	/*Property = CssUI*/
const Value V20428_300_57215 = {S2, NULL};	/*Value = True*/
const Property P20428_300 = {S21, 0, &V20428_300_57215, &P20428_298};	/*Property = DataSet*/
const Value V20428_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20428_301 = {S22, 0, &V20428_301_57218, &P20428_300};	/*Property = DataUrl*/
const Value V20428_302_57221 = {S2, NULL};	/*Value = True*/
const Property P20428_302 = {S23, 0, &V20428_302_57221, &P20428_301};	/*Property = DeviceOrientation*/
const Value V20428_304_57227 = {S2, NULL};	/*Value = True*/
const Property P20428_304 = {S24, 0, &V20428_304_57227, &P20428_302};	/*Property = FileReader*/
const Value V20428_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20428_305 = {S25, 0, &V20428_305_57231, &P20428_304};	/*Property = FileSaver*/
const Value V20428_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20428_306 = {S26, 0, &V20428_306_57237, &P20428_305};	/*Property = FileWriter*/
const Value V20428_307_57239 = {S2, NULL};	/*Value = True*/
const Property P20428_307 = {S27, 0, &V20428_307_57239, &P20428_306};	/*Property = FormData*/
const Value V20428_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20428_309 = {S28, 0, &V20428_309_57246, &P20428_307};	/*Property = Fullscreen*/
const Value V20428_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20428_310 = {S29, 0, &V20428_310_57248, &P20428_309};	/*Property = GeoLocation*/
const Value V20428_312_57254 = {S2, NULL};	/*Value = True*/
const Property P20428_312 = {S30, 0, &V20428_312_57254, &P20428_310};	/*Property = History*/
const Value V20428_314_57260 = {S2, NULL};	/*Value = True*/
const Property P20428_314 = {S31, 0, &V20428_314_57260, &P20428_312};	/*Property = Html5*/
const Value V20428_313_57257 = {S2, NULL};	/*Value = True*/
const Property P20428_313 = {S32, 0, &V20428_313_57257, &P20428_314};	/*Property = Html-Media-Capture*/
const Value V20428_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20428_315 = {S33, 0, &V20428_315_57264, &P20428_313};	/*Property = Iframe*/
const Value V20428_316_57266 = {S2, NULL};	/*Value = True*/
const Property P20428_316 = {S34, 0, &V20428_316_57266, &P20428_315};	/*Property = IndexedDB*/
const Value V20428_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20428_317 = {S35, 0, &V20428_317_57269, &P20428_316};	/*Property = Json*/
const Value V20428_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P20428_15 = {S37, 0, &V20428_15_28, &P20428_317};	/*Property = LayoutEngine*/
const Value V20428_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20428_344 = {S38, 0, &V20428_344_57272, &P20428_15};	/*Property = Masking*/
const Value V20428_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20428_320 = {S39, 0, &V20428_320_57290, &P20428_344};	/*Property = PostMessage*/
const Value V20428_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20428_321 = {S40, 0, &V20428_321_57293, &P20428_320};	/*Property = Progress*/
const Value V20428_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20428_322 = {S41, 0, &V20428_322_57296, &P20428_321};	/*Property = Prompts*/
const Value V20428_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20428_323 = {S42, 0, &V20428_323_57299, &P20428_322};	/*Property = Selector*/
const Value V20428_324_57305 = {S2, NULL};	/*Value = True*/
const Property P20428_324 = {S43, 0, &V20428_324_57305, &P20428_323};	/*Property = Svg*/
const Value V20428_327_57314 = {S2, NULL};	/*Value = True*/
const Property P20428_327 = {S44, 0, &V20428_327_57314, &P20428_324};	/*Property = TouchEvents*/
const Value V20428_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20428_328 = {S45, 0, &V20428_328_57318, &P20428_327};	/*Property = Track*/
const Value V20428_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20428_329 = {S46, 0, &V20428_329_57323, &P20428_328};	/*Property = Video*/
const Value V20428_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20428_330 = {S47, 0, &V20428_330_57326, &P20428_329};	/*Property = Viewport*/
const Value V20428_332_57338 = {S2, NULL};	/*Value = True*/
const Property P20428_332 = {S48, 0, &V20428_332_57338, &P20428_330};	/*Property = WebWorkers*/
const Value V20428_333_57341 = {S2, NULL};	/*Value = True*/
const Property P20428_333 = {S49, 0, &V20428_333_57341, &P20428_332};	/*Property = Xhr2*/
const Profile P20428 = {"20428", &P20428_333};	/*Profile ID - 20428*/
const Value V15365_1_1 = {S2, NULL};	/*Value = True*/
const Property P15365_1 = {S50, 0, &V15365_1_1, NULL};	/*Property = IsMobile*/
const Value V15365_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15365_5 = {S52, 0, &V15365_5_1277, &P15365_1};	/*Property = ScreenPixelsHeight*/
const Value V15365_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15365_4 = {S54, 0, &V15365_4_1279, &P15365_5};	/*Property = ScreenPixelsWidth*/
const Profile P15365 = {"15365", &P15365_4};	/*Profile ID - 15365*/
const Value V3553_279_57147 = {S2, NULL};	/*Value = True*/
const Property P3553_279 = {S1, 0, &V3553_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V3553_282_57153 = {S2, NULL};	/*Value = True*/
const Property P3553_282 = {S3, 0, &V3553_282_57153, &P3553_279};	/*Property = BlobBuilder*/
const Value V3553_350_57444 = {S2, NULL};	/*Value = True*/
const Property P3553_350 = {S4, 0, &V3553_350_57444, &P3553_282};	/*Property = Canvas*/
const Value V3553_283_57159 = {S2, NULL};	/*Value = True*/
const Property P3553_283 = {S5, 0, &V3553_283_57159, &P3553_350};	/*Property = CssBackground*/
const Value V3553_284_57162 = {S2, NULL};	/*Value = True*/
const Property P3553_284 = {S6, 0, &V3553_284_57162, &P3553_283};	/*Property = CssBorderImage*/
const Value V3553_285_57166 = {S0, NULL};	/*Value = False*/
const Property P3553_285 = {S7, 0, &V3553_285_57166, &P3553_284};	/*Property = CssCanvas*/
const Value V3553_286_57168 = {S2, NULL};	/*Value = True*/
const Property P3553_286 = {S8, 0, &V3553_286_57168, &P3553_285};	/*Property = CssColor*/
const Value V3553_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3553_237 = {S9, 0, &V3553_237_55786, &P3553_286};	/*Property = CssColumn*/
const Value V3553_287_57172 = {S0, NULL};	/*Value = False*/
const Property P3553_287 = {S10, 0, &V3553_287_57172, &P3553_237};	/*Property = CssFlexbox*/
const Value V3553_288_57175 = {S0, NULL};	/*Value = False*/
const Property P3553_288 = {S11, 0, &V3553_288_57175, &P3553_287};	/*Property = CssFont*/
const Value V3553_289_57759 = {S0, NULL};	/*Value = False*/
const Property P3553_289 = {S12, 0, &V3553_289_57759, &P3553_288};	/*Property = CssImages*/
const Value V3553_291_57181 = {S0, NULL};	/*Value = False*/
const Property P3553_291 = {S13, 0, &V3553_291_57181, &P3553_289};	/*Property = CssMediaQueries*/
const Value V3553_292_57183 = {S2, NULL};	/*Value = True*/
const Property P3553_292 = {S14, 0, &V3553_292_57183, &P3553_291};	/*Property = CssMinMax*/
const Value V3553_293_57187 = {S0, NULL};	/*Value = False*/
const Property P3553_293 = {S15, 0, &V3553_293_57187, &P3553_292};	/*Property = CssOverflow*/
const Value V3553_294_57189 = {S2, NULL};	/*Value = True*/
const Property P3553_294 = {S16, 0, &V3553_294_57189, &P3553_293};	/*Property = CssPosition*/
const Value V3553_295_57192 = {S2, NULL};	/*Value = True*/
const Property P3553_295 = {S17, 0, &V3553_295_57192, &P3553_294};	/*Property = CssText*/
const Value V3553_297_57195 = {S2, NULL};	/*Value = True*/
const Property P3553_297 = {S18, 0, &V3553_297_57195, &P3553_295};	/*Property = CssTransforms*/
const Value V3553_241_55799 = {S2, NULL};	/*Value = True*/
const Property P3553_241 = {S19, 0, &V3553_241_55799, &P3553_297};	/*Property = CssTransitions*/
const Value V3553_298_57206 = {S2, NULL};	/*Value = True*/
const Property P3553_298 = {S20, 0, &V3553_298_57206, &P3553_241};	/*Property = CssUI*/
const Value V3553_300_57215 = {S2, NULL};	/*Value = True*/
const Property P3553_300 = {S21, 0, &V3553_300_57215, &P3553_298};	/*Property = DataSet*/
const Value V3553_301_57218 = {S2, NULL};	/*Value = True*/
const Property P3553_301 = {S22, 0, &V3553_301_57218, &P3553_300};	/*Property = DataUrl*/
const Value V3553_302_57221 = {S2, NULL};	/*Value = True*/
const Property P3553_302 = {S23, 0, &V3553_302_57221, &P3553_301};	/*Property = DeviceOrientation*/
const Value V3553_304_57227 = {S2, NULL};	/*Value = True*/
const Property P3553_304 = {S24, 0, &V3553_304_57227, &P3553_302};	/*Property = FileReader*/
const Value V3553_305_57231 = {S0, NULL};	/*Value = False*/
const Property P3553_305 = {S25, 0, &V3553_305_57231, &P3553_304};	/*Property = FileSaver*/
const Value V3553_306_57237 = {S0, NULL};	/*Value = False*/
const Property P3553_306 = {S26, 0, &V3553_306_57237, &P3553_305};	/*Property = FileWriter*/
const Value V3553_307_57239 = {S2, NULL};	/*Value = True*/
const Property P3553_307 = {S27, 0, &V3553_307_57239, &P3553_306};	/*Property = FormData*/
const Value V3553_309_57245 = {S2, NULL};	/*Value = True*/
const Property P3553_309 = {S28, 0, &V3553_309_57245, &P3553_307};	/*Property = Fullscreen*/
const Value V3553_310_57248 = {S2, NULL};	/*Value = True*/
const Property P3553_310 = {S29, 0, &V3553_310_57248, &P3553_309};	/*Property = GeoLocation*/
const Value V3553_312_57254 = {S2, NULL};	/*Value = True*/
const Property P3553_312 = {S30, 0, &V3553_312_57254, &P3553_310};	/*Property = History*/
const Value V3553_314_57261 = {S0, NULL};	/*Value = False*/
const Property P3553_314 = {S31, 0, &V3553_314_57261, &P3553_312};	/*Property = Html5*/
const Value V3553_313_57258 = {S0, NULL};	/*Value = False*/
const Property P3553_313 = {S32, 0, &V3553_313_57258, &P3553_314};	/*Property = Html-Media-Capture*/
const Value V3553_315_57264 = {S0, NULL};	/*Value = False*/
const Property P3553_315 = {S33, 0, &V3553_315_57264, &P3553_313};	/*Property = Iframe*/
const Value V3553_316_57266 = {S2, NULL};	/*Value = True*/
const Property P3553_316 = {S34, 0, &V3553_316_57266, &P3553_315};	/*Property = IndexedDB*/
const Value V3553_317_57269 = {S2, NULL};	/*Value = True*/
const Property P3553_317 = {S35, 0, &V3553_317_57269, &P3553_316};	/*Property = Json*/
const Value V3553_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3553_15 = {S37, 0, &V3553_15_28, &P3553_317};	/*Property = LayoutEngine*/
const Value V3553_344_57272 = {S2, NULL};	/*Value = True*/
const Property P3553_344 = {S38, 0, &V3553_344_57272, &P3553_15};	/*Property = Masking*/
const Value V3553_320_57290 = {S2, NULL};	/*Value = True*/
const Property P3553_320 = {S39, 0, &V3553_320_57290, &P3553_344};	/*Property = PostMessage*/
const Value V3553_321_57293 = {S2, NULL};	/*Value = True*/
const Property P3553_321 = {S40, 0, &V3553_321_57293, &P3553_320};	/*Property = Progress*/
const Value V3553_322_57296 = {S2, NULL};	/*Value = True*/
const Property P3553_322 = {S41, 0, &V3553_322_57296, &P3553_321};	/*Property = Prompts*/
const Value V3553_323_57299 = {S2, NULL};	/*Value = True*/
const Property P3553_323 = {S42, 0, &V3553_323_57299, &P3553_322};	/*Property = Selector*/
const Value V3553_324_57305 = {S2, NULL};	/*Value = True*/
const Property P3553_324 = {S43, 0, &V3553_324_57305, &P3553_323};	/*Property = Svg*/
const Value V3553_327_57315 = {S0, NULL};	/*Value = False*/
const Property P3553_327 = {S44, 0, &V3553_327_57315, &P3553_324};	/*Property = TouchEvents*/
const Value V3553_328_57318 = {S0, NULL};	/*Value = False*/
const Property P3553_328 = {S45, 0, &V3553_328_57318, &P3553_327};	/*Property = Track*/
const Value V3553_329_57323 = {S2, NULL};	/*Value = True*/
const Property P3553_329 = {S46, 0, &V3553_329_57323, &P3553_328};	/*Property = Video*/
const Value V3553_330_57326 = {S2, NULL};	/*Value = True*/
const Property P3553_330 = {S47, 0, &V3553_330_57326, &P3553_329};	/*Property = Viewport*/
const Value V3553_332_57339 = {S0, NULL};	/*Value = False*/
const Property P3553_332 = {S48, 0, &V3553_332_57339, &P3553_330};	/*Property = WebWorkers*/
const Value V3553_333_57341 = {S2, NULL};	/*Value = True*/
const Property P3553_333 = {S49, 0, &V3553_333_57341, &P3553_332};	/*Property = Xhr2*/
const Profile P3553 = {"3553", &P3553_333};	/*Profile ID - 3553*/
const Profile P7305 = {"7305", NULL};	/*Profile ID - 7305*/
const Value V17912_1_1 = {S2, NULL};	/*Value = True*/
const Property P17912_1 = {S50, 0, &V17912_1_1, NULL};	/*Property = IsMobile*/
const Value V17912_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17912_5 = {S52, 0, &V17912_5_1277, &P17912_1};	/*Property = ScreenPixelsHeight*/
const Value V17912_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17912_4 = {S54, 0, &V17912_4_1279, &P17912_5};	/*Property = ScreenPixelsWidth*/
const Profile P17912 = {"17912", &P17912_4};	/*Profile ID - 17912*/
const Value V4848_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_279 = {S1, 0, &V4848_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4848_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_282 = {S3, 0, &V4848_282_57155, &P4848_279};	/*Property = BlobBuilder*/
const Value V4848_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_350 = {S4, 0, &V4848_350_57446, &P4848_282};	/*Property = Canvas*/
const Value V4848_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_283 = {S5, 0, &V4848_283_57161, &P4848_350};	/*Property = CssBackground*/
const Value V4848_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_284 = {S6, 0, &V4848_284_57164, &P4848_283};	/*Property = CssBorderImage*/
const Value V4848_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_285 = {S7, 0, &V4848_285_57167, &P4848_284};	/*Property = CssCanvas*/
const Value V4848_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_286 = {S8, 0, &V4848_286_57170, &P4848_285};	/*Property = CssColor*/
const Value V4848_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_237 = {S9, 0, &V4848_237_55786, &P4848_286};	/*Property = CssColumn*/
const Value V4848_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_287 = {S10, 0, &V4848_287_57173, &P4848_237};	/*Property = CssFlexbox*/
const Value V4848_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_288 = {S11, 0, &V4848_288_57176, &P4848_287};	/*Property = CssFont*/
const Value V4848_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_289 = {S12, 0, &V4848_289_86899, &P4848_288};	/*Property = CssImages*/
const Value V4848_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_291 = {S13, 0, &V4848_291_57182, &P4848_289};	/*Property = CssMediaQueries*/
const Value V4848_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_292 = {S14, 0, &V4848_292_57185, &P4848_291};	/*Property = CssMinMax*/
const Value V4848_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_293 = {S15, 0, &V4848_293_57188, &P4848_292};	/*Property = CssOverflow*/
const Value V4848_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_294 = {S16, 0, &V4848_294_57191, &P4848_293};	/*Property = CssPosition*/
const Value V4848_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_295 = {S17, 0, &V4848_295_57194, &P4848_294};	/*Property = CssText*/
const Value V4848_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_297 = {S18, 0, &V4848_297_57197, &P4848_295};	/*Property = CssTransforms*/
const Value V4848_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_241 = {S19, 0, &V4848_241_55801, &P4848_297};	/*Property = CssTransitions*/
const Value V4848_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_298 = {S20, 0, &V4848_298_57208, &P4848_241};	/*Property = CssUI*/
const Value V4848_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_300 = {S21, 0, &V4848_300_57217, &P4848_298};	/*Property = DataSet*/
const Value V4848_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_301 = {S22, 0, &V4848_301_57220, &P4848_300};	/*Property = DataUrl*/
const Value V4848_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_302 = {S23, 0, &V4848_302_57223, &P4848_301};	/*Property = DeviceOrientation*/
const Value V4848_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_304 = {S24, 0, &V4848_304_57229, &P4848_302};	/*Property = FileReader*/
const Value V4848_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_305 = {S25, 0, &V4848_305_57232, &P4848_304};	/*Property = FileSaver*/
const Value V4848_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_306 = {S26, 0, &V4848_306_57238, &P4848_305};	/*Property = FileWriter*/
const Value V4848_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_307 = {S27, 0, &V4848_307_57241, &P4848_306};	/*Property = FormData*/
const Value V4848_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_309 = {S28, 0, &V4848_309_57247, &P4848_307};	/*Property = Fullscreen*/
const Value V4848_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_310 = {S29, 0, &V4848_310_57250, &P4848_309};	/*Property = GeoLocation*/
const Value V4848_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_312 = {S30, 0, &V4848_312_57256, &P4848_310};	/*Property = History*/
const Value V4848_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_314 = {S31, 0, &V4848_314_57262, &P4848_312};	/*Property = Html5*/
const Value V4848_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_313 = {S32, 0, &V4848_313_57259, &P4848_314};	/*Property = Html-Media-Capture*/
const Value V4848_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_315 = {S33, 0, &V4848_315_57265, &P4848_313};	/*Property = Iframe*/
const Value V4848_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_316 = {S34, 0, &V4848_316_57268, &P4848_315};	/*Property = IndexedDB*/
const Value V4848_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_317 = {S35, 0, &V4848_317_57271, &P4848_316};	/*Property = Json*/
const Value V4848_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P4848_15 = {S37, 0, &V4848_15_28, &P4848_317};	/*Property = LayoutEngine*/
const Value V4848_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_344 = {S38, 0, &V4848_344_57274, &P4848_15};	/*Property = Masking*/
const Value V4848_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_320 = {S39, 0, &V4848_320_57292, &P4848_344};	/*Property = PostMessage*/
const Value V4848_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_321 = {S40, 0, &V4848_321_57295, &P4848_320};	/*Property = Progress*/
const Value V4848_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_322 = {S41, 0, &V4848_322_57298, &P4848_321};	/*Property = Prompts*/
const Value V4848_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_323 = {S42, 0, &V4848_323_57301, &P4848_322};	/*Property = Selector*/
const Value V4848_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_324 = {S43, 0, &V4848_324_57307, &P4848_323};	/*Property = Svg*/
const Value V4848_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_327 = {S44, 0, &V4848_327_57316, &P4848_324};	/*Property = TouchEvents*/
const Value V4848_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_328 = {S45, 0, &V4848_328_57319, &P4848_327};	/*Property = Track*/
const Value V4848_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_329 = {S46, 0, &V4848_329_57325, &P4848_328};	/*Property = Video*/
const Value V4848_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_330 = {S47, 0, &V4848_330_57328, &P4848_329};	/*Property = Viewport*/
const Value V4848_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_332 = {S48, 0, &V4848_332_57340, &P4848_330};	/*Property = WebWorkers*/
const Value V4848_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4848_333 = {S49, 0, &V4848_333_57343, &P4848_332};	/*Property = Xhr2*/
const Profile P4848 = {"4848", &P4848_333};	/*Profile ID - 4848*/
const Value V11656_1_1 = {S2, NULL};	/*Value = True*/
const Property P11656_1 = {S50, 0, &V11656_1_1, NULL};	/*Property = IsMobile*/
const Value V11656_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P11656_5 = {S52, 0, &V11656_5_1304, &P11656_1};	/*Property = ScreenPixelsHeight*/
const Value V11656_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11656_4 = {S54, 0, &V11656_4_1224, &P11656_5};	/*Property = ScreenPixelsWidth*/
const Profile P11656 = {"11656", &P11656_4};	/*Profile ID - 11656*/
const Value V11658_1_1 = {S2, NULL};	/*Value = True*/
const Property P11658_1 = {S50, 0, &V11658_1_1, NULL};	/*Property = IsMobile*/
const Value V11658_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P11658_5 = {S52, 0, &V11658_5_1280, &P11658_1};	/*Property = ScreenPixelsHeight*/
const Value V11658_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11658_4 = {S54, 0, &V11658_4_1235, &P11658_5};	/*Property = ScreenPixelsWidth*/
const Profile P11658 = {"11658", &P11658_4};	/*Profile ID - 11658*/
const Value V3606_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_279 = {S1, 0, &V3606_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3606_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_282 = {S3, 0, &V3606_282_57155, &P3606_279};	/*Property = BlobBuilder*/
const Value V3606_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_350 = {S4, 0, &V3606_350_57446, &P3606_282};	/*Property = Canvas*/
const Value V3606_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_283 = {S5, 0, &V3606_283_57161, &P3606_350};	/*Property = CssBackground*/
const Value V3606_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_284 = {S6, 0, &V3606_284_57164, &P3606_283};	/*Property = CssBorderImage*/
const Value V3606_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_285 = {S7, 0, &V3606_285_57167, &P3606_284};	/*Property = CssCanvas*/
const Value V3606_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_286 = {S8, 0, &V3606_286_57170, &P3606_285};	/*Property = CssColor*/
const Value V3606_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_237 = {S9, 0, &V3606_237_55786, &P3606_286};	/*Property = CssColumn*/
const Value V3606_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_287 = {S10, 0, &V3606_287_57173, &P3606_237};	/*Property = CssFlexbox*/
const Value V3606_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_288 = {S11, 0, &V3606_288_57176, &P3606_287};	/*Property = CssFont*/
const Value V3606_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_289 = {S12, 0, &V3606_289_86899, &P3606_288};	/*Property = CssImages*/
const Value V3606_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_291 = {S13, 0, &V3606_291_57182, &P3606_289};	/*Property = CssMediaQueries*/
const Value V3606_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_292 = {S14, 0, &V3606_292_57185, &P3606_291};	/*Property = CssMinMax*/
const Value V3606_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_293 = {S15, 0, &V3606_293_57188, &P3606_292};	/*Property = CssOverflow*/
const Value V3606_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_294 = {S16, 0, &V3606_294_57191, &P3606_293};	/*Property = CssPosition*/
const Value V3606_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_295 = {S17, 0, &V3606_295_57194, &P3606_294};	/*Property = CssText*/
const Value V3606_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_297 = {S18, 0, &V3606_297_57197, &P3606_295};	/*Property = CssTransforms*/
const Value V3606_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_241 = {S19, 0, &V3606_241_55801, &P3606_297};	/*Property = CssTransitions*/
const Value V3606_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_298 = {S20, 0, &V3606_298_57208, &P3606_241};	/*Property = CssUI*/
const Value V3606_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_300 = {S21, 0, &V3606_300_57217, &P3606_298};	/*Property = DataSet*/
const Value V3606_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_301 = {S22, 0, &V3606_301_57220, &P3606_300};	/*Property = DataUrl*/
const Value V3606_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_302 = {S23, 0, &V3606_302_57223, &P3606_301};	/*Property = DeviceOrientation*/
const Value V3606_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_304 = {S24, 0, &V3606_304_57229, &P3606_302};	/*Property = FileReader*/
const Value V3606_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_305 = {S25, 0, &V3606_305_57232, &P3606_304};	/*Property = FileSaver*/
const Value V3606_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_306 = {S26, 0, &V3606_306_57238, &P3606_305};	/*Property = FileWriter*/
const Value V3606_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_307 = {S27, 0, &V3606_307_57241, &P3606_306};	/*Property = FormData*/
const Value V3606_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_309 = {S28, 0, &V3606_309_57247, &P3606_307};	/*Property = Fullscreen*/
const Value V3606_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_310 = {S29, 0, &V3606_310_57250, &P3606_309};	/*Property = GeoLocation*/
const Value V3606_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_312 = {S30, 0, &V3606_312_57256, &P3606_310};	/*Property = History*/
const Value V3606_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_314 = {S31, 0, &V3606_314_57262, &P3606_312};	/*Property = Html5*/
const Value V3606_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_313 = {S32, 0, &V3606_313_57259, &P3606_314};	/*Property = Html-Media-Capture*/
const Value V3606_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_315 = {S33, 0, &V3606_315_57265, &P3606_313};	/*Property = Iframe*/
const Value V3606_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_316 = {S34, 0, &V3606_316_57268, &P3606_315};	/*Property = IndexedDB*/
const Value V3606_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_317 = {S35, 0, &V3606_317_57271, &P3606_316};	/*Property = Json*/
const Value V3606_15_43264 = {S59, NULL};	/*Value = Mango*/
const Property P3606_15 = {S37, 0, &V3606_15_43264, &P3606_317};	/*Property = LayoutEngine*/
const Value V3606_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_344 = {S38, 0, &V3606_344_57274, &P3606_15};	/*Property = Masking*/
const Value V3606_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_320 = {S39, 0, &V3606_320_57292, &P3606_344};	/*Property = PostMessage*/
const Value V3606_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_321 = {S40, 0, &V3606_321_57295, &P3606_320};	/*Property = Progress*/
const Value V3606_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_322 = {S41, 0, &V3606_322_57298, &P3606_321};	/*Property = Prompts*/
const Value V3606_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_323 = {S42, 0, &V3606_323_57301, &P3606_322};	/*Property = Selector*/
const Value V3606_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_324 = {S43, 0, &V3606_324_57307, &P3606_323};	/*Property = Svg*/
const Value V3606_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_327 = {S44, 0, &V3606_327_57316, &P3606_324};	/*Property = TouchEvents*/
const Value V3606_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_328 = {S45, 0, &V3606_328_57319, &P3606_327};	/*Property = Track*/
const Value V3606_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_329 = {S46, 0, &V3606_329_57325, &P3606_328};	/*Property = Video*/
const Value V3606_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_330 = {S47, 0, &V3606_330_57328, &P3606_329};	/*Property = Viewport*/
const Value V3606_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_332 = {S48, 0, &V3606_332_57340, &P3606_330};	/*Property = WebWorkers*/
const Value V3606_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3606_333 = {S49, 0, &V3606_333_57343, &P3606_332};	/*Property = Xhr2*/
const Profile P3606 = {"3606", &P3606_333};	/*Profile ID - 3606*/
const Profile P7063 = {"7063", NULL};	/*Profile ID - 7063*/
const Value V12186_1_1 = {S2, NULL};	/*Value = True*/
const Property P12186_1 = {S50, 0, &V12186_1_1, NULL};	/*Property = IsMobile*/
const Value V12186_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12186_5 = {S52, 0, &V12186_5_1233, &P12186_1};	/*Property = ScreenPixelsHeight*/
const Value V12186_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12186_4 = {S54, 0, &V12186_4_1235, &P12186_5};	/*Property = ScreenPixelsWidth*/
const Profile P12186 = {"12186", &P12186_4};	/*Profile ID - 12186*/
const Value V13277_1_1 = {S2, NULL};	/*Value = True*/
const Property P13277_1 = {S50, 0, &V13277_1_1, NULL};	/*Property = IsMobile*/
const Value V13277_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13277_5 = {S52, 0, &V13277_5_1222, &P13277_1};	/*Property = ScreenPixelsHeight*/
const Value V13277_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13277_4 = {S54, 0, &V13277_4_1224, &P13277_5};	/*Property = ScreenPixelsWidth*/
const Profile P13277 = {"13277", &P13277_4};	/*Profile ID - 13277*/
const Value V18007_1_1 = {S2, NULL};	/*Value = True*/
const Property P18007_1 = {S50, 0, &V18007_1_1, NULL};	/*Property = IsMobile*/
const Value V18007_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18007_5 = {S52, 0, &V18007_5_1277, &P18007_1};	/*Property = ScreenPixelsHeight*/
const Value V18007_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18007_4 = {S54, 0, &V18007_4_1279, &P18007_5};	/*Property = ScreenPixelsWidth*/
const Profile P18007 = {"18007", &P18007_4};	/*Profile ID - 18007*/
const Value V12323_1_1 = {S2, NULL};	/*Value = True*/
const Property P12323_1 = {S50, 0, &V12323_1_1, NULL};	/*Property = IsMobile*/
const Value V12323_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12323_5 = {S52, 0, &V12323_5_1277, &P12323_1};	/*Property = ScreenPixelsHeight*/
const Value V12323_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12323_4 = {S54, 0, &V12323_4_1279, &P12323_5};	/*Property = ScreenPixelsWidth*/
const Profile P12323 = {"12323", &P12323_4};	/*Profile ID - 12323*/
const Value V12189_1_1 = {S2, NULL};	/*Value = True*/
const Property P12189_1 = {S50, 0, &V12189_1_1, NULL};	/*Property = IsMobile*/
const Value V12189_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12189_5 = {S52, 0, &V12189_5_1277, &P12189_1};	/*Property = ScreenPixelsHeight*/
const Value V12189_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12189_4 = {S54, 0, &V12189_4_1279, &P12189_5};	/*Property = ScreenPixelsWidth*/
const Profile P12189 = {"12189", &P12189_4};	/*Profile ID - 12189*/
const Value V11660_1_1 = {S2, NULL};	/*Value = True*/
const Property P11660_1 = {S50, 0, &V11660_1_1, NULL};	/*Property = IsMobile*/
const Value V11660_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11660_5 = {S52, 0, &V11660_5_1222, &P11660_1};	/*Property = ScreenPixelsHeight*/
const Value V11660_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11660_4 = {S54, 0, &V11660_4_1224, &P11660_5};	/*Property = ScreenPixelsWidth*/
const Profile P11660 = {"11660", &P11660_4};	/*Profile ID - 11660*/
const Value V3978_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_279 = {S1, 0, &V3978_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3978_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_282 = {S3, 0, &V3978_282_57155, &P3978_279};	/*Property = BlobBuilder*/
const Value V3978_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_350 = {S4, 0, &V3978_350_57446, &P3978_282};	/*Property = Canvas*/
const Value V3978_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_283 = {S5, 0, &V3978_283_57161, &P3978_350};	/*Property = CssBackground*/
const Value V3978_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_284 = {S6, 0, &V3978_284_57164, &P3978_283};	/*Property = CssBorderImage*/
const Value V3978_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_285 = {S7, 0, &V3978_285_57167, &P3978_284};	/*Property = CssCanvas*/
const Value V3978_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_286 = {S8, 0, &V3978_286_57170, &P3978_285};	/*Property = CssColor*/
const Value V3978_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_237 = {S9, 0, &V3978_237_55786, &P3978_286};	/*Property = CssColumn*/
const Value V3978_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_287 = {S10, 0, &V3978_287_57173, &P3978_237};	/*Property = CssFlexbox*/
const Value V3978_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_288 = {S11, 0, &V3978_288_57176, &P3978_287};	/*Property = CssFont*/
const Value V3978_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_289 = {S12, 0, &V3978_289_86899, &P3978_288};	/*Property = CssImages*/
const Value V3978_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_291 = {S13, 0, &V3978_291_57182, &P3978_289};	/*Property = CssMediaQueries*/
const Value V3978_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_292 = {S14, 0, &V3978_292_57185, &P3978_291};	/*Property = CssMinMax*/
const Value V3978_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_293 = {S15, 0, &V3978_293_57188, &P3978_292};	/*Property = CssOverflow*/
const Value V3978_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_294 = {S16, 0, &V3978_294_57191, &P3978_293};	/*Property = CssPosition*/
const Value V3978_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_295 = {S17, 0, &V3978_295_57194, &P3978_294};	/*Property = CssText*/
const Value V3978_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_297 = {S18, 0, &V3978_297_57197, &P3978_295};	/*Property = CssTransforms*/
const Value V3978_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_241 = {S19, 0, &V3978_241_55801, &P3978_297};	/*Property = CssTransitions*/
const Value V3978_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_298 = {S20, 0, &V3978_298_57208, &P3978_241};	/*Property = CssUI*/
const Value V3978_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_300 = {S21, 0, &V3978_300_57217, &P3978_298};	/*Property = DataSet*/
const Value V3978_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_301 = {S22, 0, &V3978_301_57220, &P3978_300};	/*Property = DataUrl*/
const Value V3978_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_302 = {S23, 0, &V3978_302_57223, &P3978_301};	/*Property = DeviceOrientation*/
const Value V3978_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_304 = {S24, 0, &V3978_304_57229, &P3978_302};	/*Property = FileReader*/
const Value V3978_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_305 = {S25, 0, &V3978_305_57232, &P3978_304};	/*Property = FileSaver*/
const Value V3978_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_306 = {S26, 0, &V3978_306_57238, &P3978_305};	/*Property = FileWriter*/
const Value V3978_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_307 = {S27, 0, &V3978_307_57241, &P3978_306};	/*Property = FormData*/
const Value V3978_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_309 = {S28, 0, &V3978_309_57247, &P3978_307};	/*Property = Fullscreen*/
const Value V3978_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_310 = {S29, 0, &V3978_310_57250, &P3978_309};	/*Property = GeoLocation*/
const Value V3978_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_312 = {S30, 0, &V3978_312_57256, &P3978_310};	/*Property = History*/
const Value V3978_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_314 = {S31, 0, &V3978_314_57262, &P3978_312};	/*Property = Html5*/
const Value V3978_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_313 = {S32, 0, &V3978_313_57259, &P3978_314};	/*Property = Html-Media-Capture*/
const Value V3978_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_315 = {S33, 0, &V3978_315_57265, &P3978_313};	/*Property = Iframe*/
const Value V3978_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_316 = {S34, 0, &V3978_316_57268, &P3978_315};	/*Property = IndexedDB*/
const Value V3978_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_317 = {S35, 0, &V3978_317_57271, &P3978_316};	/*Property = Json*/
const Value V3978_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3978_15 = {S37, 0, &V3978_15_28, &P3978_317};	/*Property = LayoutEngine*/
const Value V3978_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_344 = {S38, 0, &V3978_344_57274, &P3978_15};	/*Property = Masking*/
const Value V3978_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_320 = {S39, 0, &V3978_320_57292, &P3978_344};	/*Property = PostMessage*/
const Value V3978_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_321 = {S40, 0, &V3978_321_57295, &P3978_320};	/*Property = Progress*/
const Value V3978_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_322 = {S41, 0, &V3978_322_57298, &P3978_321};	/*Property = Prompts*/
const Value V3978_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_323 = {S42, 0, &V3978_323_57301, &P3978_322};	/*Property = Selector*/
const Value V3978_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_324 = {S43, 0, &V3978_324_57307, &P3978_323};	/*Property = Svg*/
const Value V3978_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_327 = {S44, 0, &V3978_327_57316, &P3978_324};	/*Property = TouchEvents*/
const Value V3978_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_328 = {S45, 0, &V3978_328_57319, &P3978_327};	/*Property = Track*/
const Value V3978_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_329 = {S46, 0, &V3978_329_57325, &P3978_328};	/*Property = Video*/
const Value V3978_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_330 = {S47, 0, &V3978_330_57328, &P3978_329};	/*Property = Viewport*/
const Value V3978_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_332 = {S48, 0, &V3978_332_57340, &P3978_330};	/*Property = WebWorkers*/
const Value V3978_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3978_333 = {S49, 0, &V3978_333_57343, &P3978_332};	/*Property = Xhr2*/
const Profile P3978 = {"3978", &P3978_333};	/*Profile ID - 3978*/
const Profile P5795 = {"5795", NULL};	/*Profile ID - 5795*/
const Value V11661_1_1 = {S2, NULL};	/*Value = True*/
const Property P11661_1 = {S50, 0, &V11661_1_1, NULL};	/*Property = IsMobile*/
const Value V11661_5_37454 = {S80, NULL};	/*Value = 200*/
const Property P11661_5 = {S52, 0, &V11661_5_37454, &P11661_1};	/*Property = ScreenPixelsHeight*/
const Value V11661_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P11661_4 = {S54, 0, &V11661_4_8554, &P11661_5};	/*Property = ScreenPixelsWidth*/
const Profile P11661 = {"11661", &P11661_4};	/*Profile ID - 11661*/
const Value V11662_1_1 = {S2, NULL};	/*Value = True*/
const Property P11662_1 = {S50, 0, &V11662_1_1, NULL};	/*Property = IsMobile*/
const Value V11662_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11662_5 = {S52, 0, &V11662_5_1233, &P11662_1};	/*Property = ScreenPixelsHeight*/
const Value V11662_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11662_4 = {S54, 0, &V11662_4_1235, &P11662_5};	/*Property = ScreenPixelsWidth*/
const Profile P11662 = {"11662", &P11662_4};	/*Profile ID - 11662*/
const Value V15758_1_1 = {S2, NULL};	/*Value = True*/
const Property P15758_1 = {S50, 0, &V15758_1_1, NULL};	/*Property = IsMobile*/
const Value V15758_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15758_5 = {S52, 0, &V15758_5_1233, &P15758_1};	/*Property = ScreenPixelsHeight*/
const Value V15758_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15758_4 = {S54, 0, &V15758_4_1235, &P15758_5};	/*Property = ScreenPixelsWidth*/
const Profile P15758 = {"15758", &P15758_4};	/*Profile ID - 15758*/
const Value V17992_1_1 = {S2, NULL};	/*Value = True*/
const Property P17992_1 = {S50, 0, &V17992_1_1, NULL};	/*Property = IsMobile*/
const Value V17992_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17992_5 = {S52, 0, &V17992_5_1222, &P17992_1};	/*Property = ScreenPixelsHeight*/
const Value V17992_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17992_4 = {S54, 0, &V17992_4_1224, &P17992_5};	/*Property = ScreenPixelsWidth*/
const Profile P17992 = {"17992", &P17992_4};	/*Profile ID - 17992*/
const Value V17148_1_1 = {S2, NULL};	/*Value = True*/
const Property P17148_1 = {S50, 0, &V17148_1_1, NULL};	/*Property = IsMobile*/
const Value V17148_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17148_5 = {S52, 0, &V17148_5_1233, &P17148_1};	/*Property = ScreenPixelsHeight*/
const Value V17148_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17148_4 = {S54, 0, &V17148_4_1235, &P17148_5};	/*Property = ScreenPixelsWidth*/
const Profile P17148 = {"17148", &P17148_4};	/*Profile ID - 17148*/
const Value V13278_1_1 = {S2, NULL};	/*Value = True*/
const Property P13278_1 = {S50, 0, &V13278_1_1, NULL};	/*Property = IsMobile*/
const Value V13278_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13278_5 = {S52, 0, &V13278_5_1222, &P13278_1};	/*Property = ScreenPixelsHeight*/
const Value V13278_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13278_4 = {S54, 0, &V13278_4_1224, &P13278_5};	/*Property = ScreenPixelsWidth*/
const Profile P13278 = {"13278", &P13278_4};	/*Profile ID - 13278*/
const Value V13279_1_1 = {S2, NULL};	/*Value = True*/
const Property P13279_1 = {S50, 0, &V13279_1_1, NULL};	/*Property = IsMobile*/
const Value V13279_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13279_5 = {S52, 0, &V13279_5_1222, &P13279_1};	/*Property = ScreenPixelsHeight*/
const Value V13279_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13279_4 = {S54, 0, &V13279_4_1224, &P13279_5};	/*Property = ScreenPixelsWidth*/
const Profile P13279 = {"13279", &P13279_4};	/*Profile ID - 13279*/
const Value V13280_1_1 = {S2, NULL};	/*Value = True*/
const Property P13280_1 = {S50, 0, &V13280_1_1, NULL};	/*Property = IsMobile*/
const Value V13280_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13280_5 = {S52, 0, &V13280_5_1233, &P13280_1};	/*Property = ScreenPixelsHeight*/
const Value V13280_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13280_4 = {S54, 0, &V13280_4_1235, &P13280_5};	/*Property = ScreenPixelsWidth*/
const Profile P13280 = {"13280", &P13280_4};	/*Profile ID - 13280*/
const Value V13281_1_1 = {S2, NULL};	/*Value = True*/
const Property P13281_1 = {S50, 0, &V13281_1_1, NULL};	/*Property = IsMobile*/
const Value V13281_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13281_5 = {S52, 0, &V13281_5_1293, &P13281_1};	/*Property = ScreenPixelsHeight*/
const Value V13281_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13281_4 = {S54, 0, &V13281_4_1296, &P13281_5};	/*Property = ScreenPixelsWidth*/
const Profile P13281 = {"13281", &P13281_4};	/*Profile ID - 13281*/
const Value V13644_1_1 = {S2, NULL};	/*Value = True*/
const Property P13644_1 = {S50, 0, &V13644_1_1, NULL};	/*Property = IsMobile*/
const Value V13644_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13644_5 = {S52, 0, &V13644_5_1227, &P13644_1};	/*Property = ScreenPixelsHeight*/
const Value V13644_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13644_4 = {S54, 0, &V13644_4_1229, &P13644_5};	/*Property = ScreenPixelsWidth*/
const Profile P13644 = {"13644", &P13644_4};	/*Profile ID - 13644*/
const Profile P5208 = {"5208", NULL};	/*Profile ID - 5208*/
const Value V13283_1_1 = {S2, NULL};	/*Value = True*/
const Property P13283_1 = {S50, 0, &V13283_1_1, NULL};	/*Property = IsMobile*/
const Value V13283_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13283_5 = {S52, 0, &V13283_5_1380, &P13283_1};	/*Property = ScreenPixelsHeight*/
const Value V13283_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13283_4 = {S54, 0, &V13283_4_1279, &P13283_5};	/*Property = ScreenPixelsWidth*/
const Profile P13283 = {"13283", &P13283_4};	/*Profile ID - 13283*/
const Value V14074_1_1 = {S2, NULL};	/*Value = True*/
const Property P14074_1 = {S50, 0, &V14074_1_1, NULL};	/*Property = IsMobile*/
const Value V14074_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14074_5 = {S52, 0, &V14074_5_1222, &P14074_1};	/*Property = ScreenPixelsHeight*/
const Value V14074_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14074_4 = {S54, 0, &V14074_4_1279, &P14074_5};	/*Property = ScreenPixelsWidth*/
const Profile P14074 = {"14074", &P14074_4};	/*Profile ID - 14074*/
const Value V14888_1_1 = {S2, NULL};	/*Value = True*/
const Property P14888_1 = {S50, 0, &V14888_1_1, NULL};	/*Property = IsMobile*/
const Value V14888_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14888_5 = {S52, 0, &V14888_5_1277, &P14888_1};	/*Property = ScreenPixelsHeight*/
const Value V14888_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P14888_4 = {S54, 0, &V14888_4_1278, &P14888_5};	/*Property = ScreenPixelsWidth*/
const Profile P14888 = {"14888", &P14888_4};	/*Profile ID - 14888*/
const Value V12181_1_1 = {S2, NULL};	/*Value = True*/
const Property P12181_1 = {S50, 0, &V12181_1_1, NULL};	/*Property = IsMobile*/
const Value V12181_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12181_5 = {S52, 0, &V12181_5_1277, &P12181_1};	/*Property = ScreenPixelsHeight*/
const Value V12181_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P12181_4 = {S54, 0, &V12181_4_1278, &P12181_5};	/*Property = ScreenPixelsWidth*/
const Profile P12181 = {"12181", &P12181_4};	/*Profile ID - 12181*/
const Value V12135_1_1 = {S2, NULL};	/*Value = True*/
const Property P12135_1 = {S50, 0, &V12135_1_1, NULL};	/*Property = IsMobile*/
const Value V12135_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12135_5 = {S52, 0, &V12135_5_1304, &P12135_1};	/*Property = ScreenPixelsHeight*/
const Value V12135_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12135_4 = {S54, 0, &V12135_4_1224, &P12135_5};	/*Property = ScreenPixelsWidth*/
const Profile P12135 = {"12135", &P12135_4};	/*Profile ID - 12135*/
const Value V13374_1_1 = {S2, NULL};	/*Value = True*/
const Property P13374_1 = {S50, 0, &V13374_1_1, NULL};	/*Property = IsMobile*/
const Value V13374_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P13374_5 = {S52, 0, &V13374_5_1275, &P13374_1};	/*Property = ScreenPixelsHeight*/
const Value V13374_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13374_4 = {S54, 0, &V13374_4_1229, &P13374_5};	/*Property = ScreenPixelsWidth*/
const Profile P13374 = {"13374", &P13374_4};	/*Profile ID - 13374*/
const Value V17714_1_1 = {S2, NULL};	/*Value = True*/
const Property P17714_1 = {S50, 0, &V17714_1_1, NULL};	/*Property = IsMobile*/
const Value V17714_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17714_5 = {S52, 0, &V17714_5_1227, &P17714_1};	/*Property = ScreenPixelsHeight*/
const Value V17714_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17714_4 = {S54, 0, &V17714_4_1229, &P17714_5};	/*Property = ScreenPixelsWidth*/
const Profile P17714 = {"17714", &P17714_4};	/*Profile ID - 17714*/
const Value V14753_1_1 = {S2, NULL};	/*Value = True*/
const Property P14753_1 = {S50, 0, &V14753_1_1, NULL};	/*Property = IsMobile*/
const Value V14753_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14753_5 = {S52, 0, &V14753_5_1222, &P14753_1};	/*Property = ScreenPixelsHeight*/
const Value V14753_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14753_4 = {S54, 0, &V14753_4_1224, &P14753_5};	/*Property = ScreenPixelsWidth*/
const Profile P14753 = {"14753", &P14753_4};	/*Profile ID - 14753*/
const Value V11667_1_1 = {S2, NULL};	/*Value = True*/
const Property P11667_1 = {S50, 0, &V11667_1_1, NULL};	/*Property = IsMobile*/
const Value V11667_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11667_5 = {S52, 0, &V11667_5_1293, &P11667_1};	/*Property = ScreenPixelsHeight*/
const Value V11667_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11667_4 = {S54, 0, &V11667_4_1296, &P11667_5};	/*Property = ScreenPixelsWidth*/
const Profile P11667 = {"11667", &P11667_4};	/*Profile ID - 11667*/
const Value V16072_1_1 = {S2, NULL};	/*Value = True*/
const Property P16072_1 = {S50, 0, &V16072_1_1, NULL};	/*Property = IsMobile*/
const Value V16072_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16072_5 = {S52, 0, &V16072_5_1222, &P16072_1};	/*Property = ScreenPixelsHeight*/
const Value V16072_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16072_4 = {S54, 0, &V16072_4_1224, &P16072_5};	/*Property = ScreenPixelsWidth*/
const Profile P16072 = {"16072", &P16072_4};	/*Profile ID - 16072*/
const Value V11669_1_1 = {S2, NULL};	/*Value = True*/
const Property P11669_1 = {S50, 0, &V11669_1_1, NULL};	/*Property = IsMobile*/
const Value V11669_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P11669_5 = {S52, 0, &V11669_5_1280, &P11669_1};	/*Property = ScreenPixelsHeight*/
const Value V11669_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11669_4 = {S54, 0, &V11669_4_1235, &P11669_5};	/*Property = ScreenPixelsWidth*/
const Profile P11669 = {"11669", &P11669_4};	/*Profile ID - 11669*/
const Value V14438_1_1 = {S2, NULL};	/*Value = True*/
const Property P14438_1 = {S50, 0, &V14438_1_1, NULL};	/*Property = IsMobile*/
const Value V14438_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14438_5 = {S52, 0, &V14438_5_1222, &P14438_1};	/*Property = ScreenPixelsHeight*/
const Value V14438_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14438_4 = {S54, 0, &V14438_4_1224, &P14438_5};	/*Property = ScreenPixelsWidth*/
const Profile P14438 = {"14438", &P14438_4};	/*Profile ID - 14438*/
const Value V14358_1_1 = {S2, NULL};	/*Value = True*/
const Property P14358_1 = {S50, 0, &V14358_1_1, NULL};	/*Property = IsMobile*/
const Value V14358_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14358_5 = {S52, 0, &V14358_5_1380, &P14358_1};	/*Property = ScreenPixelsHeight*/
const Value V14358_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14358_4 = {S54, 0, &V14358_4_1279, &P14358_5};	/*Property = ScreenPixelsWidth*/
const Profile P14358 = {"14358", &P14358_4};	/*Profile ID - 14358*/
const Value V14454_1_1 = {S2, NULL};	/*Value = True*/
const Property P14454_1 = {S50, 0, &V14454_1_1, NULL};	/*Property = IsMobile*/
const Value V14454_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14454_5 = {S52, 0, &V14454_5_1222, &P14454_1};	/*Property = ScreenPixelsHeight*/
const Value V14454_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14454_4 = {S54, 0, &V14454_4_1224, &P14454_5};	/*Property = ScreenPixelsWidth*/
const Profile P14454 = {"14454", &P14454_4};	/*Profile ID - 14454*/
const Value V11694_1_1 = {S2, NULL};	/*Value = True*/
const Property P11694_1 = {S50, 0, &V11694_1_1, NULL};	/*Property = IsMobile*/
const Value V11694_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11694_5 = {S52, 0, &V11694_5_1222, &P11694_1};	/*Property = ScreenPixelsHeight*/
const Value V11694_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11694_4 = {S54, 0, &V11694_4_1224, &P11694_5};	/*Property = ScreenPixelsWidth*/
const Profile P11694 = {"11694", &P11694_4};	/*Profile ID - 11694*/
const Value V12125_1_1 = {S2, NULL};	/*Value = True*/
const Property P12125_1 = {S50, 0, &V12125_1_1, NULL};	/*Property = IsMobile*/
const Value V12125_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12125_5 = {S52, 0, &V12125_5_1222, &P12125_1};	/*Property = ScreenPixelsHeight*/
const Value V12125_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12125_4 = {S54, 0, &V12125_4_1224, &P12125_5};	/*Property = ScreenPixelsWidth*/
const Profile P12125 = {"12125", &P12125_4};	/*Profile ID - 12125*/
const Value V12139_1_1 = {S2, NULL};	/*Value = True*/
const Property P12139_1 = {S50, 0, &V12139_1_1, NULL};	/*Property = IsMobile*/
const Value V12139_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12139_5 = {S52, 0, &V12139_5_1222, &P12139_1};	/*Property = ScreenPixelsHeight*/
const Value V12139_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12139_4 = {S54, 0, &V12139_4_1224, &P12139_5};	/*Property = ScreenPixelsWidth*/
const Profile P12139 = {"12139", &P12139_4};	/*Profile ID - 12139*/
const Value V12439_1_1 = {S2, NULL};	/*Value = True*/
const Property P12439_1 = {S50, 0, &V12439_1_1, NULL};	/*Property = IsMobile*/
const Value V12439_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12439_5 = {S52, 0, &V12439_5_1222, &P12439_1};	/*Property = ScreenPixelsHeight*/
const Value V12439_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12439_4 = {S54, 0, &V12439_4_1224, &P12439_5};	/*Property = ScreenPixelsWidth*/
const Profile P12439 = {"12439", &P12439_4};	/*Profile ID - 12439*/
const Value V13298_1_1 = {S2, NULL};	/*Value = True*/
const Property P13298_1 = {S50, 0, &V13298_1_1, NULL};	/*Property = IsMobile*/
const Value V13298_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13298_5 = {S52, 0, &V13298_5_1293, &P13298_1};	/*Property = ScreenPixelsHeight*/
const Value V13298_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13298_4 = {S54, 0, &V13298_4_1296, &P13298_5};	/*Property = ScreenPixelsWidth*/
const Profile P13298 = {"13298", &P13298_4};	/*Profile ID - 13298*/
const Value V13297_1_1 = {S2, NULL};	/*Value = True*/
const Property P13297_1 = {S50, 0, &V13297_1_1, NULL};	/*Property = IsMobile*/
const Value V13297_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13297_5 = {S52, 0, &V13297_5_1227, &P13297_1};	/*Property = ScreenPixelsHeight*/
const Value V13297_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13297_4 = {S54, 0, &V13297_4_1229, &P13297_5};	/*Property = ScreenPixelsWidth*/
const Profile P13297 = {"13297", &P13297_4};	/*Profile ID - 13297*/
const Value V13300_1_1 = {S2, NULL};	/*Value = True*/
const Property P13300_1 = {S50, 0, &V13300_1_1, NULL};	/*Property = IsMobile*/
const Value V13300_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13300_5 = {S52, 0, &V13300_5_1233, &P13300_1};	/*Property = ScreenPixelsHeight*/
const Value V13300_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13300_4 = {S54, 0, &V13300_4_1235, &P13300_5};	/*Property = ScreenPixelsWidth*/
const Profile P13300 = {"13300", &P13300_4};	/*Profile ID - 13300*/
const Value V13301_1_1 = {S2, NULL};	/*Value = True*/
const Property P13301_1 = {S50, 0, &V13301_1_1, NULL};	/*Property = IsMobile*/
const Value V13301_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13301_5 = {S52, 0, &V13301_5_1233, &P13301_1};	/*Property = ScreenPixelsHeight*/
const Value V13301_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13301_4 = {S54, 0, &V13301_4_1235, &P13301_5};	/*Property = ScreenPixelsWidth*/
const Profile P13301 = {"13301", &P13301_4};	/*Profile ID - 13301*/
const Value V3519_279_57148 = {S0, NULL};	/*Value = False*/
const Property P3519_279 = {S1, 0, &V3519_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V3519_282_57154 = {S0, NULL};	/*Value = False*/
const Property P3519_282 = {S3, 0, &V3519_282_57154, &P3519_279};	/*Property = BlobBuilder*/
const Value V3519_350_57444 = {S2, NULL};	/*Value = True*/
const Property P3519_350 = {S4, 0, &V3519_350_57444, &P3519_282};	/*Property = Canvas*/
const Value V3519_283_57159 = {S2, NULL};	/*Value = True*/
const Property P3519_283 = {S5, 0, &V3519_283_57159, &P3519_350};	/*Property = CssBackground*/
const Value V3519_284_57163 = {S0, NULL};	/*Value = False*/
const Property P3519_284 = {S6, 0, &V3519_284_57163, &P3519_283};	/*Property = CssBorderImage*/
const Value V3519_285_57166 = {S0, NULL};	/*Value = False*/
const Property P3519_285 = {S7, 0, &V3519_285_57166, &P3519_284};	/*Property = CssCanvas*/
const Value V3519_286_57168 = {S2, NULL};	/*Value = True*/
const Property P3519_286 = {S8, 0, &V3519_286_57168, &P3519_285};	/*Property = CssColor*/
const Value V3519_237_55785 = {S0, NULL};	/*Value = False*/
const Property P3519_237 = {S9, 0, &V3519_237_55785, &P3519_286};	/*Property = CssColumn*/
const Value V3519_287_57172 = {S0, NULL};	/*Value = False*/
const Property P3519_287 = {S10, 0, &V3519_287_57172, &P3519_237};	/*Property = CssFlexbox*/
const Value V3519_288_57175 = {S0, NULL};	/*Value = False*/
const Property P3519_288 = {S11, 0, &V3519_288_57175, &P3519_287};	/*Property = CssFont*/
const Value V3519_289_57759 = {S0, NULL};	/*Value = False*/
const Property P3519_289 = {S12, 0, &V3519_289_57759, &P3519_288};	/*Property = CssImages*/
const Value V3519_291_57181 = {S0, NULL};	/*Value = False*/
const Property P3519_291 = {S13, 0, &V3519_291_57181, &P3519_289};	/*Property = CssMediaQueries*/
const Value V3519_292_57183 = {S2, NULL};	/*Value = True*/
const Property P3519_292 = {S14, 0, &V3519_292_57183, &P3519_291};	/*Property = CssMinMax*/
const Value V3519_293_57187 = {S0, NULL};	/*Value = False*/
const Property P3519_293 = {S15, 0, &V3519_293_57187, &P3519_292};	/*Property = CssOverflow*/
const Value V3519_294_57189 = {S2, NULL};	/*Value = True*/
const Property P3519_294 = {S16, 0, &V3519_294_57189, &P3519_293};	/*Property = CssPosition*/
const Value V3519_295_57192 = {S2, NULL};	/*Value = True*/
const Property P3519_295 = {S17, 0, &V3519_295_57192, &P3519_294};	/*Property = CssText*/
const Value V3519_297_57195 = {S2, NULL};	/*Value = True*/
const Property P3519_297 = {S18, 0, &V3519_297_57195, &P3519_295};	/*Property = CssTransforms*/
const Value V3519_241_55799 = {S2, NULL};	/*Value = True*/
const Property P3519_241 = {S19, 0, &V3519_241_55799, &P3519_297};	/*Property = CssTransitions*/
const Value V3519_298_57206 = {S2, NULL};	/*Value = True*/
const Property P3519_298 = {S20, 0, &V3519_298_57206, &P3519_241};	/*Property = CssUI*/
const Value V3519_300_57216 = {S0, NULL};	/*Value = False*/
const Property P3519_300 = {S21, 0, &V3519_300_57216, &P3519_298};	/*Property = DataSet*/
const Value V3519_301_57218 = {S2, NULL};	/*Value = True*/
const Property P3519_301 = {S22, 0, &V3519_301_57218, &P3519_300};	/*Property = DataUrl*/
const Value V3519_302_57222 = {S0, NULL};	/*Value = False*/
const Property P3519_302 = {S23, 0, &V3519_302_57222, &P3519_301};	/*Property = DeviceOrientation*/
const Value V3519_304_57228 = {S0, NULL};	/*Value = False*/
const Property P3519_304 = {S24, 0, &V3519_304_57228, &P3519_302};	/*Property = FileReader*/
const Value V3519_305_57231 = {S0, NULL};	/*Value = False*/
const Property P3519_305 = {S25, 0, &V3519_305_57231, &P3519_304};	/*Property = FileSaver*/
const Value V3519_306_57237 = {S0, NULL};	/*Value = False*/
const Property P3519_306 = {S26, 0, &V3519_306_57237, &P3519_305};	/*Property = FileWriter*/
const Value V3519_307_57240 = {S0, NULL};	/*Value = False*/
const Property P3519_307 = {S27, 0, &V3519_307_57240, &P3519_306};	/*Property = FormData*/
const Value V3519_309_57246 = {S0, NULL};	/*Value = False*/
const Property P3519_309 = {S28, 0, &V3519_309_57246, &P3519_307};	/*Property = Fullscreen*/
const Value V3519_310_57248 = {S2, NULL};	/*Value = True*/
const Property P3519_310 = {S29, 0, &V3519_310_57248, &P3519_309};	/*Property = GeoLocation*/
const Value V3519_312_57255 = {S0, NULL};	/*Value = False*/
const Property P3519_312 = {S30, 0, &V3519_312_57255, &P3519_310};	/*Property = History*/
const Value V3519_314_57261 = {S0, NULL};	/*Value = False*/
const Property P3519_314 = {S31, 0, &V3519_314_57261, &P3519_312};	/*Property = Html5*/
const Value V3519_313_57258 = {S0, NULL};	/*Value = False*/
const Property P3519_313 = {S32, 0, &V3519_313_57258, &P3519_314};	/*Property = Html-Media-Capture*/
const Value V3519_315_57264 = {S0, NULL};	/*Value = False*/
const Property P3519_315 = {S33, 0, &V3519_315_57264, &P3519_313};	/*Property = Iframe*/
const Value V3519_316_57267 = {S0, NULL};	/*Value = False*/
const Property P3519_316 = {S34, 0, &V3519_316_57267, &P3519_315};	/*Property = IndexedDB*/
const Value V3519_317_57269 = {S2, NULL};	/*Value = True*/
const Property P3519_317 = {S35, 0, &V3519_317_57269, &P3519_316};	/*Property = Json*/
const Value V3519_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3519_15 = {S37, 0, &V3519_15_28, &P3519_317};	/*Property = LayoutEngine*/
const Value V3519_344_57272 = {S2, NULL};	/*Value = True*/
const Property P3519_344 = {S38, 0, &V3519_344_57272, &P3519_15};	/*Property = Masking*/
const Value V3519_320_57290 = {S2, NULL};	/*Value = True*/
const Property P3519_320 = {S39, 0, &V3519_320_57290, &P3519_344};	/*Property = PostMessage*/
const Value V3519_321_57293 = {S2, NULL};	/*Value = True*/
const Property P3519_321 = {S40, 0, &V3519_321_57293, &P3519_320};	/*Property = Progress*/
const Value V3519_322_57296 = {S2, NULL};	/*Value = True*/
const Property P3519_322 = {S41, 0, &V3519_322_57296, &P3519_321};	/*Property = Prompts*/
const Value V3519_323_57299 = {S2, NULL};	/*Value = True*/
const Property P3519_323 = {S42, 0, &V3519_323_57299, &P3519_322};	/*Property = Selector*/
const Value V3519_324_57306 = {S0, NULL};	/*Value = False*/
const Property P3519_324 = {S43, 0, &V3519_324_57306, &P3519_323};	/*Property = Svg*/
const Value V3519_327_57315 = {S0, NULL};	/*Value = False*/
const Property P3519_327 = {S44, 0, &V3519_327_57315, &P3519_324};	/*Property = TouchEvents*/
const Value V3519_328_57318 = {S0, NULL};	/*Value = False*/
const Property P3519_328 = {S45, 0, &V3519_328_57318, &P3519_327};	/*Property = Track*/
const Value V3519_329_57323 = {S2, NULL};	/*Value = True*/
const Property P3519_329 = {S46, 0, &V3519_329_57323, &P3519_328};	/*Property = Video*/
const Value V3519_330_57326 = {S2, NULL};	/*Value = True*/
const Property P3519_330 = {S47, 0, &V3519_330_57326, &P3519_329};	/*Property = Viewport*/
const Value V3519_332_57339 = {S0, NULL};	/*Value = False*/
const Property P3519_332 = {S48, 0, &V3519_332_57339, &P3519_330};	/*Property = WebWorkers*/
const Value V3519_333_57342 = {S0, NULL};	/*Value = False*/
const Property P3519_333 = {S49, 0, &V3519_333_57342, &P3519_332};	/*Property = Xhr2*/
const Profile P3519 = {"3519", &P3519_333};	/*Profile ID - 3519*/
const Profile P5156 = {"5156", NULL};	/*Profile ID - 5156*/
const Value V14523_1_1 = {S2, NULL};	/*Value = True*/
const Property P14523_1 = {S50, 0, &V14523_1_1, NULL};	/*Property = IsMobile*/
const Value V14523_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14523_5 = {S52, 0, &V14523_5_1277, &P14523_1};	/*Property = ScreenPixelsHeight*/
const Value V14523_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P14523_4 = {S54, 0, &V14523_4_1278, &P14523_5};	/*Property = ScreenPixelsWidth*/
const Profile P14523 = {"14523", &P14523_4};	/*Profile ID - 14523*/
const Value V12226_1_1 = {S2, NULL};	/*Value = True*/
const Property P12226_1 = {S50, 0, &V12226_1_1, NULL};	/*Property = IsMobile*/
const Value V12226_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12226_5 = {S52, 0, &V12226_5_1227, &P12226_1};	/*Property = ScreenPixelsHeight*/
const Value V12226_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12226_4 = {S54, 0, &V12226_4_1229, &P12226_5};	/*Property = ScreenPixelsWidth*/
const Profile P12226 = {"12226", &P12226_4};	/*Profile ID - 12226*/
const Value V11747_1_1 = {S2, NULL};	/*Value = True*/
const Property P11747_1 = {S50, 0, &V11747_1_1, NULL};	/*Property = IsMobile*/
const Value V11747_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P11747_5 = {S52, 0, &V11747_5_1227, &P11747_1};	/*Property = ScreenPixelsHeight*/
const Value V11747_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P11747_4 = {S54, 0, &V11747_4_1229, &P11747_5};	/*Property = ScreenPixelsWidth*/
const Profile P11747 = {"11747", &P11747_4};	/*Profile ID - 11747*/
const Value V12108_1_1 = {S2, NULL};	/*Value = True*/
const Property P12108_1 = {S50, 0, &V12108_1_1, NULL};	/*Property = IsMobile*/
const Value V12108_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12108_5 = {S52, 0, &V12108_5_1222, &P12108_1};	/*Property = ScreenPixelsHeight*/
const Value V12108_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12108_4 = {S54, 0, &V12108_4_1224, &P12108_5};	/*Property = ScreenPixelsWidth*/
const Profile P12108 = {"12108", &P12108_4};	/*Profile ID - 12108*/
const Value V13305_1_1 = {S2, NULL};	/*Value = True*/
const Property P13305_1 = {S50, 0, &V13305_1_1, NULL};	/*Property = IsMobile*/
const Value V13305_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13305_5 = {S52, 0, &V13305_5_1293, &P13305_1};	/*Property = ScreenPixelsHeight*/
const Value V13305_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13305_4 = {S54, 0, &V13305_4_1296, &P13305_5};	/*Property = ScreenPixelsWidth*/
const Profile P13305 = {"13305", &P13305_4};	/*Profile ID - 13305*/
const Value V13307_1_1 = {S2, NULL};	/*Value = True*/
const Property P13307_1 = {S50, 0, &V13307_1_1, NULL};	/*Property = IsMobile*/
const Value V13307_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13307_5 = {S52, 0, &V13307_5_1233, &P13307_1};	/*Property = ScreenPixelsHeight*/
const Value V13307_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13307_4 = {S54, 0, &V13307_4_1235, &P13307_5};	/*Property = ScreenPixelsWidth*/
const Profile P13307 = {"13307", &P13307_4};	/*Profile ID - 13307*/
const Value V12638_1_1 = {S2, NULL};	/*Value = True*/
const Property P12638_1 = {S50, 0, &V12638_1_1, NULL};	/*Property = IsMobile*/
const Value V12638_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12638_5 = {S52, 0, &V12638_5_1222, &P12638_1};	/*Property = ScreenPixelsHeight*/
const Value V12638_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12638_4 = {S54, 0, &V12638_4_1279, &P12638_5};	/*Property = ScreenPixelsWidth*/
const Profile P12638 = {"12638", &P12638_4};	/*Profile ID - 12638*/
const Value V13308_1_1 = {S2, NULL};	/*Value = True*/
const Property P13308_1 = {S50, 0, &V13308_1_1, NULL};	/*Property = IsMobile*/
const Value V13308_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13308_5 = {S52, 0, &V13308_5_1222, &P13308_1};	/*Property = ScreenPixelsHeight*/
const Value V13308_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13308_4 = {S54, 0, &V13308_4_1224, &P13308_5};	/*Property = ScreenPixelsWidth*/
const Profile P13308 = {"13308", &P13308_4};	/*Profile ID - 13308*/
const Value V13979_1_1 = {S2, NULL};	/*Value = True*/
const Property P13979_1 = {S50, 0, &V13979_1_1, NULL};	/*Property = IsMobile*/
const Value V13979_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13979_5 = {S52, 0, &V13979_5_1222, &P13979_1};	/*Property = ScreenPixelsHeight*/
const Value V13979_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13979_4 = {S54, 0, &V13979_4_1224, &P13979_5};	/*Property = ScreenPixelsWidth*/
const Profile P13979 = {"13979", &P13979_4};	/*Profile ID - 13979*/
const Value V11684_1_1 = {S2, NULL};	/*Value = True*/
const Property P11684_1 = {S50, 0, &V11684_1_1, NULL};	/*Property = IsMobile*/
const Value V11684_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P11684_5 = {S52, 0, &V11684_5_1275, &P11684_1};	/*Property = ScreenPixelsHeight*/
const Value V11684_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P11684_4 = {S54, 0, &V11684_4_1278, &P11684_5};	/*Property = ScreenPixelsWidth*/
const Profile P11684 = {"11684", &P11684_4};	/*Profile ID - 11684*/
const Value V3459_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_279 = {S1, 0, &V3459_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3459_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_282 = {S3, 0, &V3459_282_57155, &P3459_279};	/*Property = BlobBuilder*/
const Value V3459_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_350 = {S4, 0, &V3459_350_57446, &P3459_282};	/*Property = Canvas*/
const Value V3459_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_283 = {S5, 0, &V3459_283_57161, &P3459_350};	/*Property = CssBackground*/
const Value V3459_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_284 = {S6, 0, &V3459_284_57164, &P3459_283};	/*Property = CssBorderImage*/
const Value V3459_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_285 = {S7, 0, &V3459_285_57167, &P3459_284};	/*Property = CssCanvas*/
const Value V3459_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_286 = {S8, 0, &V3459_286_57170, &P3459_285};	/*Property = CssColor*/
const Value V3459_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_237 = {S9, 0, &V3459_237_55786, &P3459_286};	/*Property = CssColumn*/
const Value V3459_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_287 = {S10, 0, &V3459_287_57173, &P3459_237};	/*Property = CssFlexbox*/
const Value V3459_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_288 = {S11, 0, &V3459_288_57176, &P3459_287};	/*Property = CssFont*/
const Value V3459_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_289 = {S12, 0, &V3459_289_86899, &P3459_288};	/*Property = CssImages*/
const Value V3459_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_291 = {S13, 0, &V3459_291_57182, &P3459_289};	/*Property = CssMediaQueries*/
const Value V3459_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_292 = {S14, 0, &V3459_292_57185, &P3459_291};	/*Property = CssMinMax*/
const Value V3459_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_293 = {S15, 0, &V3459_293_57188, &P3459_292};	/*Property = CssOverflow*/
const Value V3459_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_294 = {S16, 0, &V3459_294_57191, &P3459_293};	/*Property = CssPosition*/
const Value V3459_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_295 = {S17, 0, &V3459_295_57194, &P3459_294};	/*Property = CssText*/
const Value V3459_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_297 = {S18, 0, &V3459_297_57197, &P3459_295};	/*Property = CssTransforms*/
const Value V3459_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_241 = {S19, 0, &V3459_241_55801, &P3459_297};	/*Property = CssTransitions*/
const Value V3459_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_298 = {S20, 0, &V3459_298_57208, &P3459_241};	/*Property = CssUI*/
const Value V3459_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_300 = {S21, 0, &V3459_300_57217, &P3459_298};	/*Property = DataSet*/
const Value V3459_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_301 = {S22, 0, &V3459_301_57220, &P3459_300};	/*Property = DataUrl*/
const Value V3459_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_302 = {S23, 0, &V3459_302_57223, &P3459_301};	/*Property = DeviceOrientation*/
const Value V3459_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_304 = {S24, 0, &V3459_304_57229, &P3459_302};	/*Property = FileReader*/
const Value V3459_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_305 = {S25, 0, &V3459_305_57232, &P3459_304};	/*Property = FileSaver*/
const Value V3459_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_306 = {S26, 0, &V3459_306_57238, &P3459_305};	/*Property = FileWriter*/
const Value V3459_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_307 = {S27, 0, &V3459_307_57241, &P3459_306};	/*Property = FormData*/
const Value V3459_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_309 = {S28, 0, &V3459_309_57247, &P3459_307};	/*Property = Fullscreen*/
const Value V3459_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_310 = {S29, 0, &V3459_310_57250, &P3459_309};	/*Property = GeoLocation*/
const Value V3459_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_312 = {S30, 0, &V3459_312_57256, &P3459_310};	/*Property = History*/
const Value V3459_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_314 = {S31, 0, &V3459_314_57262, &P3459_312};	/*Property = Html5*/
const Value V3459_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_313 = {S32, 0, &V3459_313_57259, &P3459_314};	/*Property = Html-Media-Capture*/
const Value V3459_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_315 = {S33, 0, &V3459_315_57265, &P3459_313};	/*Property = Iframe*/
const Value V3459_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_316 = {S34, 0, &V3459_316_57268, &P3459_315};	/*Property = IndexedDB*/
const Value V3459_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_317 = {S35, 0, &V3459_317_57271, &P3459_316};	/*Property = Json*/
const Value V3459_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3459_15 = {S37, 0, &V3459_15_28, &P3459_317};	/*Property = LayoutEngine*/
const Value V3459_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_344 = {S38, 0, &V3459_344_57274, &P3459_15};	/*Property = Masking*/
const Value V3459_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_320 = {S39, 0, &V3459_320_57292, &P3459_344};	/*Property = PostMessage*/
const Value V3459_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_321 = {S40, 0, &V3459_321_57295, &P3459_320};	/*Property = Progress*/
const Value V3459_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_322 = {S41, 0, &V3459_322_57298, &P3459_321};	/*Property = Prompts*/
const Value V3459_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_323 = {S42, 0, &V3459_323_57301, &P3459_322};	/*Property = Selector*/
const Value V3459_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_324 = {S43, 0, &V3459_324_57307, &P3459_323};	/*Property = Svg*/
const Value V3459_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_327 = {S44, 0, &V3459_327_57316, &P3459_324};	/*Property = TouchEvents*/
const Value V3459_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_328 = {S45, 0, &V3459_328_57319, &P3459_327};	/*Property = Track*/
const Value V3459_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_329 = {S46, 0, &V3459_329_57325, &P3459_328};	/*Property = Video*/
const Value V3459_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_330 = {S47, 0, &V3459_330_57328, &P3459_329};	/*Property = Viewport*/
const Value V3459_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_332 = {S48, 0, &V3459_332_57340, &P3459_330};	/*Property = WebWorkers*/
const Value V3459_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3459_333 = {S49, 0, &V3459_333_57343, &P3459_332};	/*Property = Xhr2*/
const Profile P3459 = {"3459", &P3459_333};	/*Profile ID - 3459*/
const Value V13310_1_1 = {S2, NULL};	/*Value = True*/
const Property P13310_1 = {S50, 0, &V13310_1_1, NULL};	/*Property = IsMobile*/
const Value V13310_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13310_5 = {S52, 0, &V13310_5_1304, &P13310_1};	/*Property = ScreenPixelsHeight*/
const Value V13310_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13310_4 = {S54, 0, &V13310_4_1224, &P13310_5};	/*Property = ScreenPixelsWidth*/
const Profile P13310 = {"13310", &P13310_4};	/*Profile ID - 13310*/
const Value V15435_1_1 = {S2, NULL};	/*Value = True*/
const Property P15435_1 = {S50, 0, &V15435_1_1, NULL};	/*Property = IsMobile*/
const Value V15435_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15435_5 = {S52, 0, &V15435_5_1222, &P15435_1};	/*Property = ScreenPixelsHeight*/
const Value V15435_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15435_4 = {S54, 0, &V15435_4_1224, &P15435_5};	/*Property = ScreenPixelsWidth*/
const Profile P15435 = {"15435", &P15435_4};	/*Profile ID - 15435*/
const Value V12068_1_1 = {S2, NULL};	/*Value = True*/
const Property P12068_1 = {S50, 0, &V12068_1_1, NULL};	/*Property = IsMobile*/
const Value V12068_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12068_5 = {S52, 0, &V12068_5_1293, &P12068_1};	/*Property = ScreenPixelsHeight*/
const Value V12068_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12068_4 = {S54, 0, &V12068_4_1296, &P12068_5};	/*Property = ScreenPixelsWidth*/
const Profile P12068 = {"12068", &P12068_4};	/*Profile ID - 12068*/
const Value V15216_1_1 = {S2, NULL};	/*Value = True*/
const Property P15216_1 = {S50, 0, &V15216_1_1, NULL};	/*Property = IsMobile*/
const Value V15216_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15216_5 = {S52, 0, &V15216_5_1380, &P15216_1};	/*Property = ScreenPixelsHeight*/
const Value V15216_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15216_4 = {S54, 0, &V15216_4_1279, &P15216_5};	/*Property = ScreenPixelsWidth*/
const Profile P15216 = {"15216", &P15216_4};	/*Profile ID - 15216*/
const Value V14997_1_1 = {S2, NULL};	/*Value = True*/
const Property P14997_1 = {S50, 0, &V14997_1_1, NULL};	/*Property = IsMobile*/
const Value V14997_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14997_5 = {S52, 0, &V14997_5_1227, &P14997_1};	/*Property = ScreenPixelsHeight*/
const Value V14997_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14997_4 = {S54, 0, &V14997_4_1229, &P14997_5};	/*Property = ScreenPixelsWidth*/
const Profile P14997 = {"14997", &P14997_4};	/*Profile ID - 14997*/
const Profile P5629 = {"5629", NULL};	/*Profile ID - 5629*/
const Value V15760_279_57148 = {S0, NULL};	/*Value = False*/
const Property P15760_279 = {S1, 0, &V15760_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V15760_282_57154 = {S0, NULL};	/*Value = False*/
const Property P15760_282 = {S3, 0, &V15760_282_57154, &P15760_279};	/*Property = BlobBuilder*/
const Value V15760_350_57444 = {S2, NULL};	/*Value = True*/
const Property P15760_350 = {S4, 0, &V15760_350_57444, &P15760_282};	/*Property = Canvas*/
const Value V15760_283_57159 = {S2, NULL};	/*Value = True*/
const Property P15760_283 = {S5, 0, &V15760_283_57159, &P15760_350};	/*Property = CssBackground*/
const Value V15760_284_57163 = {S0, NULL};	/*Value = False*/
const Property P15760_284 = {S6, 0, &V15760_284_57163, &P15760_283};	/*Property = CssBorderImage*/
const Value V15760_285_57166 = {S0, NULL};	/*Value = False*/
const Property P15760_285 = {S7, 0, &V15760_285_57166, &P15760_284};	/*Property = CssCanvas*/
const Value V15760_286_57168 = {S2, NULL};	/*Value = True*/
const Property P15760_286 = {S8, 0, &V15760_286_57168, &P15760_285};	/*Property = CssColor*/
const Value V15760_237_55784 = {S2, NULL};	/*Value = True*/
const Property P15760_237 = {S9, 0, &V15760_237_55784, &P15760_286};	/*Property = CssColumn*/
const Value V15760_287_57172 = {S0, NULL};	/*Value = False*/
const Property P15760_287 = {S10, 0, &V15760_287_57172, &P15760_237};	/*Property = CssFlexbox*/
const Value V15760_288_57175 = {S0, NULL};	/*Value = False*/
const Property P15760_288 = {S11, 0, &V15760_288_57175, &P15760_287};	/*Property = CssFont*/
const Value V15760_289_57759 = {S0, NULL};	/*Value = False*/
const Property P15760_289 = {S12, 0, &V15760_289_57759, &P15760_288};	/*Property = CssImages*/
const Value V15760_291_57181 = {S0, NULL};	/*Value = False*/
const Property P15760_291 = {S13, 0, &V15760_291_57181, &P15760_289};	/*Property = CssMediaQueries*/
const Value V15760_292_57183 = {S2, NULL};	/*Value = True*/
const Property P15760_292 = {S14, 0, &V15760_292_57183, &P15760_291};	/*Property = CssMinMax*/
const Value V15760_293_57187 = {S0, NULL};	/*Value = False*/
const Property P15760_293 = {S15, 0, &V15760_293_57187, &P15760_292};	/*Property = CssOverflow*/
const Value V15760_294_57190 = {S0, NULL};	/*Value = False*/
const Property P15760_294 = {S16, 0, &V15760_294_57190, &P15760_293};	/*Property = CssPosition*/
const Value V15760_295_57192 = {S2, NULL};	/*Value = True*/
const Property P15760_295 = {S17, 0, &V15760_295_57192, &P15760_294};	/*Property = CssText*/
const Value V15760_297_57195 = {S2, NULL};	/*Value = True*/
const Property P15760_297 = {S18, 0, &V15760_297_57195, &P15760_295};	/*Property = CssTransforms*/
const Value V15760_241_55799 = {S2, NULL};	/*Value = True*/
const Property P15760_241 = {S19, 0, &V15760_241_55799, &P15760_297};	/*Property = CssTransitions*/
const Value V15760_298_57206 = {S2, NULL};	/*Value = True*/
const Property P15760_298 = {S20, 0, &V15760_298_57206, &P15760_241};	/*Property = CssUI*/
const Value V15760_300_57216 = {S0, NULL};	/*Value = False*/
const Property P15760_300 = {S21, 0, &V15760_300_57216, &P15760_298};	/*Property = DataSet*/
const Value V15760_301_57218 = {S2, NULL};	/*Value = True*/
const Property P15760_301 = {S22, 0, &V15760_301_57218, &P15760_300};	/*Property = DataUrl*/
const Value V15760_302_57222 = {S0, NULL};	/*Value = False*/
const Property P15760_302 = {S23, 0, &V15760_302_57222, &P15760_301};	/*Property = DeviceOrientation*/
const Value V15760_304_57228 = {S0, NULL};	/*Value = False*/
const Property P15760_304 = {S24, 0, &V15760_304_57228, &P15760_302};	/*Property = FileReader*/
const Value V15760_305_57231 = {S0, NULL};	/*Value = False*/
const Property P15760_305 = {S25, 0, &V15760_305_57231, &P15760_304};	/*Property = FileSaver*/
const Value V15760_306_57237 = {S0, NULL};	/*Value = False*/
const Property P15760_306 = {S26, 0, &V15760_306_57237, &P15760_305};	/*Property = FileWriter*/
const Value V15760_307_57240 = {S0, NULL};	/*Value = False*/
const Property P15760_307 = {S27, 0, &V15760_307_57240, &P15760_306};	/*Property = FormData*/
const Value V15760_309_57246 = {S0, NULL};	/*Value = False*/
const Property P15760_309 = {S28, 0, &V15760_309_57246, &P15760_307};	/*Property = Fullscreen*/
const Value V15760_310_57248 = {S2, NULL};	/*Value = True*/
const Property P15760_310 = {S29, 0, &V15760_310_57248, &P15760_309};	/*Property = GeoLocation*/
const Value V15760_312_57255 = {S0, NULL};	/*Value = False*/
const Property P15760_312 = {S30, 0, &V15760_312_57255, &P15760_310};	/*Property = History*/
const Value V15760_314_57261 = {S0, NULL};	/*Value = False*/
const Property P15760_314 = {S31, 0, &V15760_314_57261, &P15760_312};	/*Property = Html5*/
const Value V15760_313_57258 = {S0, NULL};	/*Value = False*/
const Property P15760_313 = {S32, 0, &V15760_313_57258, &P15760_314};	/*Property = Html-Media-Capture*/
const Value V15760_315_57264 = {S0, NULL};	/*Value = False*/
const Property P15760_315 = {S33, 0, &V15760_315_57264, &P15760_313};	/*Property = Iframe*/
const Value V15760_316_57267 = {S0, NULL};	/*Value = False*/
const Property P15760_316 = {S34, 0, &V15760_316_57267, &P15760_315};	/*Property = IndexedDB*/
const Value V15760_317_57269 = {S2, NULL};	/*Value = True*/
const Property P15760_317 = {S35, 0, &V15760_317_57269, &P15760_316};	/*Property = Json*/
const Value V15760_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P15760_15 = {S37, 0, &V15760_15_28, &P15760_317};	/*Property = LayoutEngine*/
const Value V15760_344_57272 = {S2, NULL};	/*Value = True*/
const Property P15760_344 = {S38, 0, &V15760_344_57272, &P15760_15};	/*Property = Masking*/
const Value V15760_320_57290 = {S2, NULL};	/*Value = True*/
const Property P15760_320 = {S39, 0, &V15760_320_57290, &P15760_344};	/*Property = PostMessage*/
const Value V15760_321_57293 = {S2, NULL};	/*Value = True*/
const Property P15760_321 = {S40, 0, &V15760_321_57293, &P15760_320};	/*Property = Progress*/
const Value V15760_322_57296 = {S2, NULL};	/*Value = True*/
const Property P15760_322 = {S41, 0, &V15760_322_57296, &P15760_321};	/*Property = Prompts*/
const Value V15760_323_57299 = {S2, NULL};	/*Value = True*/
const Property P15760_323 = {S42, 0, &V15760_323_57299, &P15760_322};	/*Property = Selector*/
const Value V15760_324_57306 = {S0, NULL};	/*Value = False*/
const Property P15760_324 = {S43, 0, &V15760_324_57306, &P15760_323};	/*Property = Svg*/
const Value V15760_327_57315 = {S0, NULL};	/*Value = False*/
const Property P15760_327 = {S44, 0, &V15760_327_57315, &P15760_324};	/*Property = TouchEvents*/
const Value V15760_328_57318 = {S0, NULL};	/*Value = False*/
const Property P15760_328 = {S45, 0, &V15760_328_57318, &P15760_327};	/*Property = Track*/
const Value V15760_329_57323 = {S2, NULL};	/*Value = True*/
const Property P15760_329 = {S46, 0, &V15760_329_57323, &P15760_328};	/*Property = Video*/
const Value V15760_330_57326 = {S2, NULL};	/*Value = True*/
const Property P15760_330 = {S47, 0, &V15760_330_57326, &P15760_329};	/*Property = Viewport*/
const Value V15760_332_57339 = {S0, NULL};	/*Value = False*/
const Property P15760_332 = {S48, 0, &V15760_332_57339, &P15760_330};	/*Property = WebWorkers*/
const Value V15760_333_57342 = {S0, NULL};	/*Value = False*/
const Property P15760_333 = {S49, 0, &V15760_333_57342, &P15760_332};	/*Property = Xhr2*/
const Profile P15760 = {"15760", &P15760_333};	/*Profile ID - 15760*/
const Value V12274_1_1 = {S2, NULL};	/*Value = True*/
const Property P12274_1 = {S50, 0, &V12274_1_1, NULL};	/*Property = IsMobile*/
const Value V12274_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12274_5 = {S52, 0, &V12274_5_1304, &P12274_1};	/*Property = ScreenPixelsHeight*/
const Value V12274_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12274_4 = {S54, 0, &V12274_4_1224, &P12274_5};	/*Property = ScreenPixelsWidth*/
const Profile P12274 = {"12274", &P12274_4};	/*Profile ID - 12274*/
const Value V17467_1_1 = {S2, NULL};	/*Value = True*/
const Property P17467_1 = {S50, 0, &V17467_1_1, NULL};	/*Property = IsMobile*/
const Value V17467_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17467_5 = {S52, 0, &V17467_5_1227, &P17467_1};	/*Property = ScreenPixelsHeight*/
const Value V17467_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17467_4 = {S54, 0, &V17467_4_1229, &P17467_5};	/*Property = ScreenPixelsWidth*/
const Profile P17467 = {"17467", &P17467_4};	/*Profile ID - 17467*/
const Value V12077_1_1 = {S2, NULL};	/*Value = True*/
const Property P12077_1 = {S50, 0, &V12077_1_1, NULL};	/*Property = IsMobile*/
const Value V12077_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12077_5 = {S52, 0, &V12077_5_1233, &P12077_1};	/*Property = ScreenPixelsHeight*/
const Value V12077_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12077_4 = {S54, 0, &V12077_4_1235, &P12077_5};	/*Property = ScreenPixelsWidth*/
const Profile P12077 = {"12077", &P12077_4};	/*Profile ID - 12077*/
const Value V12080_1_1 = {S2, NULL};	/*Value = True*/
const Property P12080_1 = {S50, 0, &V12080_1_1, NULL};	/*Property = IsMobile*/
const Value V12080_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12080_5 = {S52, 0, &V12080_5_1222, &P12080_1};	/*Property = ScreenPixelsHeight*/
const Value V12080_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12080_4 = {S54, 0, &V12080_4_1224, &P12080_5};	/*Property = ScreenPixelsWidth*/
const Profile P12080 = {"12080", &P12080_4};	/*Profile ID - 12080*/
const Value V11744_1_1 = {S2, NULL};	/*Value = True*/
const Property P11744_1 = {S50, 0, &V11744_1_1, NULL};	/*Property = IsMobile*/
const Value V11744_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P11744_5 = {S52, 0, &V11744_5_1380, &P11744_1};	/*Property = ScreenPixelsHeight*/
const Value V11744_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P11744_4 = {S54, 0, &V11744_4_1279, &P11744_5};	/*Property = ScreenPixelsWidth*/
const Profile P11744 = {"11744", &P11744_4};	/*Profile ID - 11744*/
const Value V12082_1_1 = {S2, NULL};	/*Value = True*/
const Property P12082_1 = {S50, 0, &V12082_1_1, NULL};	/*Property = IsMobile*/
const Value V12082_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P12082_5 = {S52, 0, &V12082_5_1269, &P12082_1};	/*Property = ScreenPixelsHeight*/
const Value V12082_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12082_4 = {S54, 0, &V12082_4_1229, &P12082_5};	/*Property = ScreenPixelsWidth*/
const Profile P12082 = {"12082", &P12082_4};	/*Profile ID - 12082*/
const Value V17163_1_1 = {S2, NULL};	/*Value = True*/
const Property P17163_1 = {S50, 0, &V17163_1_1, NULL};	/*Property = IsMobile*/
const Value V17163_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17163_5 = {S52, 0, &V17163_5_1222, &P17163_1};	/*Property = ScreenPixelsHeight*/
const Value V17163_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17163_4 = {S54, 0, &V17163_4_1224, &P17163_5};	/*Property = ScreenPixelsWidth*/
const Profile P17163 = {"17163", &P17163_4};	/*Profile ID - 17163*/
const Value V11724_1_1 = {S2, NULL};	/*Value = True*/
const Property P11724_1 = {S50, 0, &V11724_1_1, NULL};	/*Property = IsMobile*/
const Value V11724_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P11724_5 = {S52, 0, &V11724_5_1380, &P11724_1};	/*Property = ScreenPixelsHeight*/
const Value V11724_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P11724_4 = {S54, 0, &V11724_4_1279, &P11724_5};	/*Property = ScreenPixelsWidth*/
const Profile P11724 = {"11724", &P11724_4};	/*Profile ID - 11724*/
const Value V14664_1_1 = {S2, NULL};	/*Value = True*/
const Property P14664_1 = {S50, 0, &V14664_1_1, NULL};	/*Property = IsMobile*/
const Value V14664_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14664_5 = {S52, 0, &V14664_5_1293, &P14664_1};	/*Property = ScreenPixelsHeight*/
const Value V14664_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14664_4 = {S54, 0, &V14664_4_1296, &P14664_5};	/*Property = ScreenPixelsWidth*/
const Profile P14664 = {"14664", &P14664_4};	/*Profile ID - 14664*/
const Value V15661_1_1 = {S2, NULL};	/*Value = True*/
const Property P15661_1 = {S50, 0, &V15661_1_1, NULL};	/*Property = IsMobile*/
const Value V15661_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15661_5 = {S52, 0, &V15661_5_1293, &P15661_1};	/*Property = ScreenPixelsHeight*/
const Value V15661_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15661_4 = {S54, 0, &V15661_4_1296, &P15661_5};	/*Property = ScreenPixelsWidth*/
const Profile P15661 = {"15661", &P15661_4};	/*Profile ID - 15661*/
const Value V12088_1_1 = {S2, NULL};	/*Value = True*/
const Property P12088_1 = {S50, 0, &V12088_1_1, NULL};	/*Property = IsMobile*/
const Value V12088_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P12088_5 = {S52, 0, &V12088_5_1275, &P12088_1};	/*Property = ScreenPixelsHeight*/
const Value V12088_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P12088_4 = {S54, 0, &V12088_4_1278, &P12088_5};	/*Property = ScreenPixelsWidth*/
const Profile P12088 = {"12088", &P12088_4};	/*Profile ID - 12088*/
const Value V11734_1_1 = {S2, NULL};	/*Value = True*/
const Property P11734_1 = {S50, 0, &V11734_1_1, NULL};	/*Property = IsMobile*/
const Value V11734_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11734_5 = {S52, 0, &V11734_5_1222, &P11734_1};	/*Property = ScreenPixelsHeight*/
const Value V11734_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11734_4 = {S54, 0, &V11734_4_1224, &P11734_5};	/*Property = ScreenPixelsWidth*/
const Profile P11734 = {"11734", &P11734_4};	/*Profile ID - 11734*/
const Value V12095_1_1 = {S2, NULL};	/*Value = True*/
const Property P12095_1 = {S50, 0, &V12095_1_1, NULL};	/*Property = IsMobile*/
const Value V12095_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12095_5 = {S52, 0, &V12095_5_1222, &P12095_1};	/*Property = ScreenPixelsHeight*/
const Value V12095_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12095_4 = {S54, 0, &V12095_4_1224, &P12095_5};	/*Property = ScreenPixelsWidth*/
const Profile P12095 = {"12095", &P12095_4};	/*Profile ID - 12095*/
const Value V12097_1_1 = {S2, NULL};	/*Value = True*/
const Property P12097_1 = {S50, 0, &V12097_1_1, NULL};	/*Property = IsMobile*/
const Value V12097_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12097_5 = {S52, 0, &V12097_5_1222, &P12097_1};	/*Property = ScreenPixelsHeight*/
const Value V12097_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12097_4 = {S54, 0, &V12097_4_1224, &P12097_5};	/*Property = ScreenPixelsWidth*/
const Profile P12097 = {"12097", &P12097_4};	/*Profile ID - 12097*/
const Value V16169_1_1 = {S2, NULL};	/*Value = True*/
const Property P16169_1 = {S50, 0, &V16169_1_1, NULL};	/*Property = IsMobile*/
const Value V16169_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16169_5 = {S52, 0, &V16169_5_1222, &P16169_1};	/*Property = ScreenPixelsHeight*/
const Value V16169_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16169_4 = {S54, 0, &V16169_4_1224, &P16169_5};	/*Property = ScreenPixelsWidth*/
const Profile P16169 = {"16169", &P16169_4};	/*Profile ID - 16169*/
const Value V12099_1_1 = {S2, NULL};	/*Value = True*/
const Property P12099_1 = {S50, 0, &V12099_1_1, NULL};	/*Property = IsMobile*/
const Value V12099_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12099_5 = {S52, 0, &V12099_5_1233, &P12099_1};	/*Property = ScreenPixelsHeight*/
const Value V12099_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12099_4 = {S54, 0, &V12099_4_1235, &P12099_5};	/*Property = ScreenPixelsWidth*/
const Profile P12099 = {"12099", &P12099_4};	/*Profile ID - 12099*/
const Value V12101_1_1 = {S2, NULL};	/*Value = True*/
const Property P12101_1 = {S50, 0, &V12101_1_1, NULL};	/*Property = IsMobile*/
const Value V12101_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12101_5 = {S52, 0, &V12101_5_1380, &P12101_1};	/*Property = ScreenPixelsHeight*/
const Value V12101_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12101_4 = {S54, 0, &V12101_4_1279, &P12101_5};	/*Property = ScreenPixelsWidth*/
const Profile P12101 = {"12101", &P12101_4};	/*Profile ID - 12101*/
const Value V17431_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_279 = {S1, 0, &V17431_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17431_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_282 = {S3, 0, &V17431_282_57155, &P17431_279};	/*Property = BlobBuilder*/
const Value V17431_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_350 = {S4, 0, &V17431_350_57446, &P17431_282};	/*Property = Canvas*/
const Value V17431_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_283 = {S5, 0, &V17431_283_57161, &P17431_350};	/*Property = CssBackground*/
const Value V17431_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_284 = {S6, 0, &V17431_284_57164, &P17431_283};	/*Property = CssBorderImage*/
const Value V17431_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_285 = {S7, 0, &V17431_285_57167, &P17431_284};	/*Property = CssCanvas*/
const Value V17431_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_286 = {S8, 0, &V17431_286_57170, &P17431_285};	/*Property = CssColor*/
const Value V17431_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_237 = {S9, 0, &V17431_237_55786, &P17431_286};	/*Property = CssColumn*/
const Value V17431_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_287 = {S10, 0, &V17431_287_57173, &P17431_237};	/*Property = CssFlexbox*/
const Value V17431_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_288 = {S11, 0, &V17431_288_57176, &P17431_287};	/*Property = CssFont*/
const Value V17431_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_289 = {S12, 0, &V17431_289_86899, &P17431_288};	/*Property = CssImages*/
const Value V17431_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_291 = {S13, 0, &V17431_291_57182, &P17431_289};	/*Property = CssMediaQueries*/
const Value V17431_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_292 = {S14, 0, &V17431_292_57185, &P17431_291};	/*Property = CssMinMax*/
const Value V17431_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_293 = {S15, 0, &V17431_293_57188, &P17431_292};	/*Property = CssOverflow*/
const Value V17431_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_294 = {S16, 0, &V17431_294_57191, &P17431_293};	/*Property = CssPosition*/
const Value V17431_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_295 = {S17, 0, &V17431_295_57194, &P17431_294};	/*Property = CssText*/
const Value V17431_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_297 = {S18, 0, &V17431_297_57197, &P17431_295};	/*Property = CssTransforms*/
const Value V17431_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_241 = {S19, 0, &V17431_241_55801, &P17431_297};	/*Property = CssTransitions*/
const Value V17431_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_298 = {S20, 0, &V17431_298_57208, &P17431_241};	/*Property = CssUI*/
const Value V17431_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_300 = {S21, 0, &V17431_300_57217, &P17431_298};	/*Property = DataSet*/
const Value V17431_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_301 = {S22, 0, &V17431_301_57220, &P17431_300};	/*Property = DataUrl*/
const Value V17431_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_302 = {S23, 0, &V17431_302_57223, &P17431_301};	/*Property = DeviceOrientation*/
const Value V17431_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_304 = {S24, 0, &V17431_304_57229, &P17431_302};	/*Property = FileReader*/
const Value V17431_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_305 = {S25, 0, &V17431_305_57232, &P17431_304};	/*Property = FileSaver*/
const Value V17431_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_306 = {S26, 0, &V17431_306_57238, &P17431_305};	/*Property = FileWriter*/
const Value V17431_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_307 = {S27, 0, &V17431_307_57241, &P17431_306};	/*Property = FormData*/
const Value V17431_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_309 = {S28, 0, &V17431_309_57247, &P17431_307};	/*Property = Fullscreen*/
const Value V17431_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_310 = {S29, 0, &V17431_310_57250, &P17431_309};	/*Property = GeoLocation*/
const Value V17431_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_312 = {S30, 0, &V17431_312_57256, &P17431_310};	/*Property = History*/
const Value V17431_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_314 = {S31, 0, &V17431_314_57262, &P17431_312};	/*Property = Html5*/
const Value V17431_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_313 = {S32, 0, &V17431_313_57259, &P17431_314};	/*Property = Html-Media-Capture*/
const Value V17431_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_315 = {S33, 0, &V17431_315_57265, &P17431_313};	/*Property = Iframe*/
const Value V17431_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_316 = {S34, 0, &V17431_316_57268, &P17431_315};	/*Property = IndexedDB*/
const Value V17431_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_317 = {S35, 0, &V17431_317_57271, &P17431_316};	/*Property = Json*/
const Value V17431_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P17431_15 = {S37, 0, &V17431_15_29, &P17431_317};	/*Property = LayoutEngine*/
const Value V17431_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_344 = {S38, 0, &V17431_344_57274, &P17431_15};	/*Property = Masking*/
const Value V17431_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_320 = {S39, 0, &V17431_320_57292, &P17431_344};	/*Property = PostMessage*/
const Value V17431_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_321 = {S40, 0, &V17431_321_57295, &P17431_320};	/*Property = Progress*/
const Value V17431_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_322 = {S41, 0, &V17431_322_57298, &P17431_321};	/*Property = Prompts*/
const Value V17431_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_323 = {S42, 0, &V17431_323_57301, &P17431_322};	/*Property = Selector*/
const Value V17431_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_324 = {S43, 0, &V17431_324_57307, &P17431_323};	/*Property = Svg*/
const Value V17431_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_327 = {S44, 0, &V17431_327_57316, &P17431_324};	/*Property = TouchEvents*/
const Value V17431_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_328 = {S45, 0, &V17431_328_57319, &P17431_327};	/*Property = Track*/
const Value V17431_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_329 = {S46, 0, &V17431_329_57325, &P17431_328};	/*Property = Video*/
const Value V17431_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_330 = {S47, 0, &V17431_330_57328, &P17431_329};	/*Property = Viewport*/
const Value V17431_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_332 = {S48, 0, &V17431_332_57340, &P17431_330};	/*Property = WebWorkers*/
const Value V17431_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17431_333 = {S49, 0, &V17431_333_57343, &P17431_332};	/*Property = Xhr2*/
const Profile P17431 = {"17431", &P17431_333};	/*Profile ID - 17431*/
const Value V12110_1_1 = {S2, NULL};	/*Value = True*/
const Property P12110_1 = {S50, 0, &V12110_1_1, NULL};	/*Property = IsMobile*/
const Value V12110_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12110_5 = {S52, 0, &V12110_5_1277, &P12110_1};	/*Property = ScreenPixelsHeight*/
const Value V12110_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12110_4 = {S54, 0, &V12110_4_1279, &P12110_5};	/*Property = ScreenPixelsWidth*/
const Profile P12110 = {"12110", &P12110_4};	/*Profile ID - 12110*/
const Value V14682_1_1 = {S2, NULL};	/*Value = True*/
const Property P14682_1 = {S50, 0, &V14682_1_1, NULL};	/*Property = IsMobile*/
const Value V14682_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14682_5 = {S52, 0, &V14682_5_1222, &P14682_1};	/*Property = ScreenPixelsHeight*/
const Value V14682_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14682_4 = {S54, 0, &V14682_4_1224, &P14682_5};	/*Property = ScreenPixelsWidth*/
const Profile P14682 = {"14682", &P14682_4};	/*Profile ID - 14682*/
const Value V18407_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_279 = {S1, 0, &V18407_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18407_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_282 = {S3, 0, &V18407_282_57155, &P18407_279};	/*Property = BlobBuilder*/
const Value V18407_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_350 = {S4, 0, &V18407_350_57446, &P18407_282};	/*Property = Canvas*/
const Value V18407_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_283 = {S5, 0, &V18407_283_57161, &P18407_350};	/*Property = CssBackground*/
const Value V18407_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_284 = {S6, 0, &V18407_284_57164, &P18407_283};	/*Property = CssBorderImage*/
const Value V18407_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_285 = {S7, 0, &V18407_285_57167, &P18407_284};	/*Property = CssCanvas*/
const Value V18407_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_286 = {S8, 0, &V18407_286_57170, &P18407_285};	/*Property = CssColor*/
const Value V18407_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_237 = {S9, 0, &V18407_237_55786, &P18407_286};	/*Property = CssColumn*/
const Value V18407_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_287 = {S10, 0, &V18407_287_57173, &P18407_237};	/*Property = CssFlexbox*/
const Value V18407_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_288 = {S11, 0, &V18407_288_57176, &P18407_287};	/*Property = CssFont*/
const Value V18407_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_289 = {S12, 0, &V18407_289_86899, &P18407_288};	/*Property = CssImages*/
const Value V18407_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_291 = {S13, 0, &V18407_291_57182, &P18407_289};	/*Property = CssMediaQueries*/
const Value V18407_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_292 = {S14, 0, &V18407_292_57185, &P18407_291};	/*Property = CssMinMax*/
const Value V18407_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_293 = {S15, 0, &V18407_293_57188, &P18407_292};	/*Property = CssOverflow*/
const Value V18407_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_294 = {S16, 0, &V18407_294_57191, &P18407_293};	/*Property = CssPosition*/
const Value V18407_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_295 = {S17, 0, &V18407_295_57194, &P18407_294};	/*Property = CssText*/
const Value V18407_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_297 = {S18, 0, &V18407_297_57197, &P18407_295};	/*Property = CssTransforms*/
const Value V18407_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_241 = {S19, 0, &V18407_241_55801, &P18407_297};	/*Property = CssTransitions*/
const Value V18407_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_298 = {S20, 0, &V18407_298_57208, &P18407_241};	/*Property = CssUI*/
const Value V18407_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_300 = {S21, 0, &V18407_300_57217, &P18407_298};	/*Property = DataSet*/
const Value V18407_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_301 = {S22, 0, &V18407_301_57220, &P18407_300};	/*Property = DataUrl*/
const Value V18407_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_302 = {S23, 0, &V18407_302_57223, &P18407_301};	/*Property = DeviceOrientation*/
const Value V18407_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_304 = {S24, 0, &V18407_304_57229, &P18407_302};	/*Property = FileReader*/
const Value V18407_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_305 = {S25, 0, &V18407_305_57232, &P18407_304};	/*Property = FileSaver*/
const Value V18407_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_306 = {S26, 0, &V18407_306_57238, &P18407_305};	/*Property = FileWriter*/
const Value V18407_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_307 = {S27, 0, &V18407_307_57241, &P18407_306};	/*Property = FormData*/
const Value V18407_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_309 = {S28, 0, &V18407_309_57247, &P18407_307};	/*Property = Fullscreen*/
const Value V18407_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_310 = {S29, 0, &V18407_310_57250, &P18407_309};	/*Property = GeoLocation*/
const Value V18407_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_312 = {S30, 0, &V18407_312_57256, &P18407_310};	/*Property = History*/
const Value V18407_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_314 = {S31, 0, &V18407_314_57262, &P18407_312};	/*Property = Html5*/
const Value V18407_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_313 = {S32, 0, &V18407_313_57259, &P18407_314};	/*Property = Html-Media-Capture*/
const Value V18407_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_315 = {S33, 0, &V18407_315_57265, &P18407_313};	/*Property = Iframe*/
const Value V18407_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_316 = {S34, 0, &V18407_316_57268, &P18407_315};	/*Property = IndexedDB*/
const Value V18407_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_317 = {S35, 0, &V18407_317_57271, &P18407_316};	/*Property = Json*/
const Value V18407_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18407_15 = {S37, 0, &V18407_15_31, &P18407_317};	/*Property = LayoutEngine*/
const Value V18407_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_344 = {S38, 0, &V18407_344_57274, &P18407_15};	/*Property = Masking*/
const Value V18407_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_320 = {S39, 0, &V18407_320_57292, &P18407_344};	/*Property = PostMessage*/
const Value V18407_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_321 = {S40, 0, &V18407_321_57295, &P18407_320};	/*Property = Progress*/
const Value V18407_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_322 = {S41, 0, &V18407_322_57298, &P18407_321};	/*Property = Prompts*/
const Value V18407_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_323 = {S42, 0, &V18407_323_57301, &P18407_322};	/*Property = Selector*/
const Value V18407_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_324 = {S43, 0, &V18407_324_57307, &P18407_323};	/*Property = Svg*/
const Value V18407_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_327 = {S44, 0, &V18407_327_57316, &P18407_324};	/*Property = TouchEvents*/
const Value V18407_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_328 = {S45, 0, &V18407_328_57319, &P18407_327};	/*Property = Track*/
const Value V18407_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_329 = {S46, 0, &V18407_329_57325, &P18407_328};	/*Property = Video*/
const Value V18407_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_330 = {S47, 0, &V18407_330_57328, &P18407_329};	/*Property = Viewport*/
const Value V18407_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_332 = {S48, 0, &V18407_332_57340, &P18407_330};	/*Property = WebWorkers*/
const Value V18407_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18407_333 = {S49, 0, &V18407_333_57343, &P18407_332};	/*Property = Xhr2*/
const Profile P18407 = {"18407", &P18407_333};	/*Profile ID - 18407*/
const Value V12137_1_1 = {S2, NULL};	/*Value = True*/
const Property P12137_1 = {S50, 0, &V12137_1_1, NULL};	/*Property = IsMobile*/
const Value V12137_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12137_5 = {S52, 0, &V12137_5_1304, &P12137_1};	/*Property = ScreenPixelsHeight*/
const Value V12137_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12137_4 = {S54, 0, &V12137_4_1224, &P12137_5};	/*Property = ScreenPixelsWidth*/
const Profile P12137 = {"12137", &P12137_4};	/*Profile ID - 12137*/
const Profile P7133 = {"7133", NULL};	/*Profile ID - 7133*/
const Value V17203_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17203_279 = {S1, 0, &V17203_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17203_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17203_282 = {S3, 0, &V17203_282_57154, &P17203_279};	/*Property = BlobBuilder*/
const Value V17203_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17203_350 = {S4, 0, &V17203_350_57444, &P17203_282};	/*Property = Canvas*/
const Value V17203_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17203_283 = {S5, 0, &V17203_283_57159, &P17203_350};	/*Property = CssBackground*/
const Value V17203_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17203_284 = {S6, 0, &V17203_284_57163, &P17203_283};	/*Property = CssBorderImage*/
const Value V17203_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17203_285 = {S7, 0, &V17203_285_57166, &P17203_284};	/*Property = CssCanvas*/
const Value V17203_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17203_286 = {S8, 0, &V17203_286_57168, &P17203_285};	/*Property = CssColor*/
const Value V17203_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17203_237 = {S9, 0, &V17203_237_55784, &P17203_286};	/*Property = CssColumn*/
const Value V17203_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17203_287 = {S10, 0, &V17203_287_57172, &P17203_237};	/*Property = CssFlexbox*/
const Value V17203_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17203_288 = {S11, 0, &V17203_288_57175, &P17203_287};	/*Property = CssFont*/
const Value V17203_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17203_289 = {S12, 0, &V17203_289_57759, &P17203_288};	/*Property = CssImages*/
const Value V17203_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17203_291 = {S13, 0, &V17203_291_57181, &P17203_289};	/*Property = CssMediaQueries*/
const Value V17203_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17203_292 = {S14, 0, &V17203_292_57183, &P17203_291};	/*Property = CssMinMax*/
const Value V17203_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17203_293 = {S15, 0, &V17203_293_57187, &P17203_292};	/*Property = CssOverflow*/
const Value V17203_294_57190 = {S0, NULL};	/*Value = False*/
const Property P17203_294 = {S16, 0, &V17203_294_57190, &P17203_293};	/*Property = CssPosition*/
const Value V17203_295_57193 = {S0, NULL};	/*Value = False*/
const Property P17203_295 = {S17, 0, &V17203_295_57193, &P17203_294};	/*Property = CssText*/
const Value V17203_297_57196 = {S0, NULL};	/*Value = False*/
const Property P17203_297 = {S18, 0, &V17203_297_57196, &P17203_295};	/*Property = CssTransforms*/
const Value V17203_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17203_241 = {S19, 0, &V17203_241_55799, &P17203_297};	/*Property = CssTransitions*/
const Value V17203_298_57207 = {S0, NULL};	/*Value = False*/
const Property P17203_298 = {S20, 0, &V17203_298_57207, &P17203_241};	/*Property = CssUI*/
const Value V17203_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17203_300 = {S21, 0, &V17203_300_57216, &P17203_298};	/*Property = DataSet*/
const Value V17203_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17203_301 = {S22, 0, &V17203_301_57218, &P17203_300};	/*Property = DataUrl*/
const Value V17203_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17203_302 = {S23, 0, &V17203_302_57222, &P17203_301};	/*Property = DeviceOrientation*/
const Value V17203_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17203_304 = {S24, 0, &V17203_304_57228, &P17203_302};	/*Property = FileReader*/
const Value V17203_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17203_305 = {S25, 0, &V17203_305_57231, &P17203_304};	/*Property = FileSaver*/
const Value V17203_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17203_306 = {S26, 0, &V17203_306_57237, &P17203_305};	/*Property = FileWriter*/
const Value V17203_307_57240 = {S0, NULL};	/*Value = False*/
const Property P17203_307 = {S27, 0, &V17203_307_57240, &P17203_306};	/*Property = FormData*/
const Value V17203_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17203_309 = {S28, 0, &V17203_309_57246, &P17203_307};	/*Property = Fullscreen*/
const Value V17203_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17203_310 = {S29, 0, &V17203_310_57248, &P17203_309};	/*Property = GeoLocation*/
const Value V17203_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17203_312 = {S30, 0, &V17203_312_57255, &P17203_310};	/*Property = History*/
const Value V17203_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17203_314 = {S31, 0, &V17203_314_57261, &P17203_312};	/*Property = Html5*/
const Value V17203_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17203_313 = {S32, 0, &V17203_313_57258, &P17203_314};	/*Property = Html-Media-Capture*/
const Value V17203_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17203_315 = {S33, 0, &V17203_315_57264, &P17203_313};	/*Property = Iframe*/
const Value V17203_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17203_316 = {S34, 0, &V17203_316_57267, &P17203_315};	/*Property = IndexedDB*/
const Value V17203_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17203_317 = {S35, 0, &V17203_317_57269, &P17203_316};	/*Property = Json*/
const Value V17203_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P17203_15 = {S37, 0, &V17203_15_29, &P17203_317};	/*Property = LayoutEngine*/
const Value V17203_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17203_344 = {S38, 0, &V17203_344_57272, &P17203_15};	/*Property = Masking*/
const Value V17203_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17203_320 = {S39, 0, &V17203_320_57290, &P17203_344};	/*Property = PostMessage*/
const Value V17203_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17203_321 = {S40, 0, &V17203_321_57293, &P17203_320};	/*Property = Progress*/
const Value V17203_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17203_322 = {S41, 0, &V17203_322_57296, &P17203_321};	/*Property = Prompts*/
const Value V17203_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17203_323 = {S42, 0, &V17203_323_57299, &P17203_322};	/*Property = Selector*/
const Value V17203_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17203_324 = {S43, 0, &V17203_324_57306, &P17203_323};	/*Property = Svg*/
const Value V17203_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17203_327 = {S44, 0, &V17203_327_57315, &P17203_324};	/*Property = TouchEvents*/
const Value V17203_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17203_328 = {S45, 0, &V17203_328_57318, &P17203_327};	/*Property = Track*/
const Value V17203_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17203_329 = {S46, 0, &V17203_329_57323, &P17203_328};	/*Property = Video*/
const Value V17203_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17203_330 = {S47, 0, &V17203_330_57326, &P17203_329};	/*Property = Viewport*/
const Value V17203_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17203_332 = {S48, 0, &V17203_332_57339, &P17203_330};	/*Property = WebWorkers*/
const Value V17203_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17203_333 = {S49, 0, &V17203_333_57342, &P17203_332};	/*Property = Xhr2*/
const Profile P17203 = {"17203", &P17203_333};	/*Profile ID - 17203*/
const Value V17558_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17558_279 = {S1, 0, &V17558_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17558_282_57153 = {S2, NULL};	/*Value = True*/
const Property P17558_282 = {S3, 0, &V17558_282_57153, &P17558_279};	/*Property = BlobBuilder*/
const Value V17558_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17558_350 = {S4, 0, &V17558_350_57444, &P17558_282};	/*Property = Canvas*/
const Value V17558_283_57160 = {S0, NULL};	/*Value = False*/
const Property P17558_283 = {S5, 0, &V17558_283_57160, &P17558_350};	/*Property = CssBackground*/
const Value V17558_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17558_284 = {S6, 0, &V17558_284_57163, &P17558_283};	/*Property = CssBorderImage*/
const Value V17558_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17558_285 = {S7, 0, &V17558_285_57166, &P17558_284};	/*Property = CssCanvas*/
const Value V17558_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17558_286 = {S8, 0, &V17558_286_57168, &P17558_285};	/*Property = CssColor*/
const Value V17558_237_55785 = {S0, NULL};	/*Value = False*/
const Property P17558_237 = {S9, 0, &V17558_237_55785, &P17558_286};	/*Property = CssColumn*/
const Value V17558_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17558_287 = {S10, 0, &V17558_287_57172, &P17558_237};	/*Property = CssFlexbox*/
const Value V17558_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17558_288 = {S11, 0, &V17558_288_57175, &P17558_287};	/*Property = CssFont*/
const Value V17558_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17558_289 = {S12, 0, &V17558_289_86899, &P17558_288};	/*Property = CssImages*/
const Value V17558_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17558_291 = {S13, 0, &V17558_291_57181, &P17558_289};	/*Property = CssMediaQueries*/
const Value V17558_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17558_292 = {S14, 0, &V17558_292_57183, &P17558_291};	/*Property = CssMinMax*/
const Value V17558_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17558_293 = {S15, 0, &V17558_293_57187, &P17558_292};	/*Property = CssOverflow*/
const Value V17558_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17558_294 = {S16, 0, &V17558_294_57189, &P17558_293};	/*Property = CssPosition*/
const Value V17558_295_57193 = {S0, NULL};	/*Value = False*/
const Property P17558_295 = {S17, 0, &V17558_295_57193, &P17558_294};	/*Property = CssText*/
const Value V17558_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17558_297 = {S18, 0, &V17558_297_57195, &P17558_295};	/*Property = CssTransforms*/
const Value V17558_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17558_241 = {S19, 0, &V17558_241_55799, &P17558_297};	/*Property = CssTransitions*/
const Value V17558_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17558_298 = {S20, 0, &V17558_298_57206, &P17558_241};	/*Property = CssUI*/
const Value V17558_300_57215 = {S2, NULL};	/*Value = True*/
const Property P17558_300 = {S21, 0, &V17558_300_57215, &P17558_298};	/*Property = DataSet*/
const Value V17558_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17558_301 = {S22, 0, &V17558_301_57218, &P17558_300};	/*Property = DataUrl*/
const Value V17558_302_57221 = {S2, NULL};	/*Value = True*/
const Property P17558_302 = {S23, 0, &V17558_302_57221, &P17558_301};	/*Property = DeviceOrientation*/
const Value V17558_304_57227 = {S2, NULL};	/*Value = True*/
const Property P17558_304 = {S24, 0, &V17558_304_57227, &P17558_302};	/*Property = FileReader*/
const Value V17558_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17558_305 = {S25, 0, &V17558_305_57231, &P17558_304};	/*Property = FileSaver*/
const Value V17558_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17558_306 = {S26, 0, &V17558_306_57237, &P17558_305};	/*Property = FileWriter*/
const Value V17558_307_57239 = {S2, NULL};	/*Value = True*/
const Property P17558_307 = {S27, 0, &V17558_307_57239, &P17558_306};	/*Property = FormData*/
const Value V17558_309_57245 = {S2, NULL};	/*Value = True*/
const Property P17558_309 = {S28, 0, &V17558_309_57245, &P17558_307};	/*Property = Fullscreen*/
const Value V17558_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17558_310 = {S29, 0, &V17558_310_57248, &P17558_309};	/*Property = GeoLocation*/
const Value V17558_312_57254 = {S2, NULL};	/*Value = True*/
const Property P17558_312 = {S30, 0, &V17558_312_57254, &P17558_310};	/*Property = History*/
const Value V17558_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17558_314 = {S31, 0, &V17558_314_57261, &P17558_312};	/*Property = Html5*/
const Value V17558_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17558_313 = {S32, 0, &V17558_313_57258, &P17558_314};	/*Property = Html-Media-Capture*/
const Value V17558_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17558_315 = {S33, 0, &V17558_315_57264, &P17558_313};	/*Property = Iframe*/
const Value V17558_316_57266 = {S2, NULL};	/*Value = True*/
const Property P17558_316 = {S34, 0, &V17558_316_57266, &P17558_315};	/*Property = IndexedDB*/
const Value V17558_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17558_317 = {S35, 0, &V17558_317_57269, &P17558_316};	/*Property = Json*/
const Value V17558_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P17558_15 = {S37, 0, &V17558_15_29, &P17558_317};	/*Property = LayoutEngine*/
const Value V17558_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17558_344 = {S38, 0, &V17558_344_57272, &P17558_15};	/*Property = Masking*/
const Value V17558_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17558_320 = {S39, 0, &V17558_320_57290, &P17558_344};	/*Property = PostMessage*/
const Value V17558_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17558_321 = {S40, 0, &V17558_321_57293, &P17558_320};	/*Property = Progress*/
const Value V17558_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17558_322 = {S41, 0, &V17558_322_57296, &P17558_321};	/*Property = Prompts*/
const Value V17558_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17558_323 = {S42, 0, &V17558_323_57299, &P17558_322};	/*Property = Selector*/
const Value V17558_324_57305 = {S2, NULL};	/*Value = True*/
const Property P17558_324 = {S43, 0, &V17558_324_57305, &P17558_323};	/*Property = Svg*/
const Value V17558_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17558_327 = {S44, 0, &V17558_327_57315, &P17558_324};	/*Property = TouchEvents*/
const Value V17558_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17558_328 = {S45, 0, &V17558_328_57318, &P17558_327};	/*Property = Track*/
const Value V17558_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17558_329 = {S46, 0, &V17558_329_57323, &P17558_328};	/*Property = Video*/
const Value V17558_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17558_330 = {S47, 0, &V17558_330_57326, &P17558_329};	/*Property = Viewport*/
const Value V17558_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17558_332 = {S48, 0, &V17558_332_57339, &P17558_330};	/*Property = WebWorkers*/
const Value V17558_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17558_333 = {S49, 0, &V17558_333_57342, &P17558_332};	/*Property = Xhr2*/
const Profile P17558 = {"17558", &P17558_333};	/*Profile ID - 17558*/
const Value V13442_1_1 = {S2, NULL};	/*Value = True*/
const Property P13442_1 = {S50, 0, &V13442_1_1, NULL};	/*Property = IsMobile*/
const Value V13442_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13442_5 = {S52, 0, &V13442_5_1222, &P13442_1};	/*Property = ScreenPixelsHeight*/
const Value V13442_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13442_4 = {S54, 0, &V13442_4_1224, &P13442_5};	/*Property = ScreenPixelsWidth*/
const Profile P13442 = {"13442", &P13442_4};	/*Profile ID - 13442*/
const Value V14971_1_1 = {S2, NULL};	/*Value = True*/
const Property P14971_1 = {S50, 0, &V14971_1_1, NULL};	/*Property = IsMobile*/
const Value V14971_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14971_5 = {S52, 0, &V14971_5_1222, &P14971_1};	/*Property = ScreenPixelsHeight*/
const Value V14971_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14971_4 = {S54, 0, &V14971_4_1224, &P14971_5};	/*Property = ScreenPixelsWidth*/
const Profile P14971 = {"14971", &P14971_4};	/*Profile ID - 14971*/
const Value V12174_1_1 = {S2, NULL};	/*Value = True*/
const Property P12174_1 = {S50, 0, &V12174_1_1, NULL};	/*Property = IsMobile*/
const Value V12174_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12174_5 = {S52, 0, &V12174_5_1222, &P12174_1};	/*Property = ScreenPixelsHeight*/
const Value V12174_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12174_4 = {S54, 0, &V12174_4_1224, &P12174_5};	/*Property = ScreenPixelsWidth*/
const Profile P12174 = {"12174", &P12174_4};	/*Profile ID - 12174*/
const Value V12126_1_1 = {S2, NULL};	/*Value = True*/
const Property P12126_1 = {S50, 0, &V12126_1_1, NULL};	/*Property = IsMobile*/
const Value V12126_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12126_5 = {S52, 0, &V12126_5_1222, &P12126_1};	/*Property = ScreenPixelsHeight*/
const Value V12126_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12126_4 = {S54, 0, &V12126_4_1224, &P12126_5};	/*Property = ScreenPixelsWidth*/
const Profile P12126 = {"12126", &P12126_4};	/*Profile ID - 12126*/
const Value V15478_1_1 = {S2, NULL};	/*Value = True*/
const Property P15478_1 = {S50, 0, &V15478_1_1, NULL};	/*Property = IsMobile*/
const Value V15478_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15478_5 = {S52, 0, &V15478_5_1222, &P15478_1};	/*Property = ScreenPixelsHeight*/
const Value V15478_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15478_4 = {S54, 0, &V15478_4_1224, &P15478_5};	/*Property = ScreenPixelsWidth*/
const Profile P15478 = {"15478", &P15478_4};	/*Profile ID - 15478*/
const Value V11702_1_1 = {S2, NULL};	/*Value = True*/
const Property P11702_1 = {S50, 0, &V11702_1_1, NULL};	/*Property = IsMobile*/
const Value V11702_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P11702_5 = {S52, 0, &V11702_5_1277, &P11702_1};	/*Property = ScreenPixelsHeight*/
const Value V11702_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P11702_4 = {S54, 0, &V11702_4_1279, &P11702_5};	/*Property = ScreenPixelsWidth*/
const Profile P11702 = {"11702", &P11702_4};	/*Profile ID - 11702*/
const Value V3534_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_279 = {S1, 0, &V3534_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3534_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_282 = {S3, 0, &V3534_282_57155, &P3534_279};	/*Property = BlobBuilder*/
const Value V3534_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_350 = {S4, 0, &V3534_350_57446, &P3534_282};	/*Property = Canvas*/
const Value V3534_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_283 = {S5, 0, &V3534_283_57161, &P3534_350};	/*Property = CssBackground*/
const Value V3534_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_284 = {S6, 0, &V3534_284_57164, &P3534_283};	/*Property = CssBorderImage*/
const Value V3534_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_285 = {S7, 0, &V3534_285_57167, &P3534_284};	/*Property = CssCanvas*/
const Value V3534_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_286 = {S8, 0, &V3534_286_57170, &P3534_285};	/*Property = CssColor*/
const Value V3534_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_237 = {S9, 0, &V3534_237_55786, &P3534_286};	/*Property = CssColumn*/
const Value V3534_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_287 = {S10, 0, &V3534_287_57173, &P3534_237};	/*Property = CssFlexbox*/
const Value V3534_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_288 = {S11, 0, &V3534_288_57176, &P3534_287};	/*Property = CssFont*/
const Value V3534_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_289 = {S12, 0, &V3534_289_86899, &P3534_288};	/*Property = CssImages*/
const Value V3534_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_291 = {S13, 0, &V3534_291_57182, &P3534_289};	/*Property = CssMediaQueries*/
const Value V3534_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_292 = {S14, 0, &V3534_292_57185, &P3534_291};	/*Property = CssMinMax*/
const Value V3534_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_293 = {S15, 0, &V3534_293_57188, &P3534_292};	/*Property = CssOverflow*/
const Value V3534_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_294 = {S16, 0, &V3534_294_57191, &P3534_293};	/*Property = CssPosition*/
const Value V3534_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_295 = {S17, 0, &V3534_295_57194, &P3534_294};	/*Property = CssText*/
const Value V3534_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_297 = {S18, 0, &V3534_297_57197, &P3534_295};	/*Property = CssTransforms*/
const Value V3534_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_241 = {S19, 0, &V3534_241_55801, &P3534_297};	/*Property = CssTransitions*/
const Value V3534_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_298 = {S20, 0, &V3534_298_57208, &P3534_241};	/*Property = CssUI*/
const Value V3534_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_300 = {S21, 0, &V3534_300_57217, &P3534_298};	/*Property = DataSet*/
const Value V3534_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_301 = {S22, 0, &V3534_301_57220, &P3534_300};	/*Property = DataUrl*/
const Value V3534_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_302 = {S23, 0, &V3534_302_57223, &P3534_301};	/*Property = DeviceOrientation*/
const Value V3534_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_304 = {S24, 0, &V3534_304_57229, &P3534_302};	/*Property = FileReader*/
const Value V3534_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_305 = {S25, 0, &V3534_305_57232, &P3534_304};	/*Property = FileSaver*/
const Value V3534_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_306 = {S26, 0, &V3534_306_57238, &P3534_305};	/*Property = FileWriter*/
const Value V3534_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_307 = {S27, 0, &V3534_307_57241, &P3534_306};	/*Property = FormData*/
const Value V3534_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_309 = {S28, 0, &V3534_309_57247, &P3534_307};	/*Property = Fullscreen*/
const Value V3534_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_310 = {S29, 0, &V3534_310_57250, &P3534_309};	/*Property = GeoLocation*/
const Value V3534_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_312 = {S30, 0, &V3534_312_57256, &P3534_310};	/*Property = History*/
const Value V3534_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_314 = {S31, 0, &V3534_314_57262, &P3534_312};	/*Property = Html5*/
const Value V3534_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_313 = {S32, 0, &V3534_313_57259, &P3534_314};	/*Property = Html-Media-Capture*/
const Value V3534_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_315 = {S33, 0, &V3534_315_57265, &P3534_313};	/*Property = Iframe*/
const Value V3534_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_316 = {S34, 0, &V3534_316_57268, &P3534_315};	/*Property = IndexedDB*/
const Value V3534_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_317 = {S35, 0, &V3534_317_57271, &P3534_316};	/*Property = Json*/
const Value V3534_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3534_15 = {S37, 0, &V3534_15_28, &P3534_317};	/*Property = LayoutEngine*/
const Value V3534_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_344 = {S38, 0, &V3534_344_57274, &P3534_15};	/*Property = Masking*/
const Value V3534_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_320 = {S39, 0, &V3534_320_57292, &P3534_344};	/*Property = PostMessage*/
const Value V3534_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_321 = {S40, 0, &V3534_321_57295, &P3534_320};	/*Property = Progress*/
const Value V3534_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_322 = {S41, 0, &V3534_322_57298, &P3534_321};	/*Property = Prompts*/
const Value V3534_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_323 = {S42, 0, &V3534_323_57301, &P3534_322};	/*Property = Selector*/
const Value V3534_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_324 = {S43, 0, &V3534_324_57307, &P3534_323};	/*Property = Svg*/
const Value V3534_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_327 = {S44, 0, &V3534_327_57316, &P3534_324};	/*Property = TouchEvents*/
const Value V3534_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_328 = {S45, 0, &V3534_328_57319, &P3534_327};	/*Property = Track*/
const Value V3534_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_329 = {S46, 0, &V3534_329_57325, &P3534_328};	/*Property = Video*/
const Value V3534_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_330 = {S47, 0, &V3534_330_57328, &P3534_329};	/*Property = Viewport*/
const Value V3534_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_332 = {S48, 0, &V3534_332_57340, &P3534_330};	/*Property = WebWorkers*/
const Value V3534_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3534_333 = {S49, 0, &V3534_333_57343, &P3534_332};	/*Property = Xhr2*/
const Profile P3534 = {"3534", &P3534_333};	/*Profile ID - 3534*/
const Value V14414_1_1 = {S2, NULL};	/*Value = True*/
const Property P14414_1 = {S50, 0, &V14414_1_1, NULL};	/*Property = IsMobile*/
const Value V14414_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14414_5 = {S52, 0, &V14414_5_1222, &P14414_1};	/*Property = ScreenPixelsHeight*/
const Value V14414_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14414_4 = {S54, 0, &V14414_4_1224, &P14414_5};	/*Property = ScreenPixelsWidth*/
const Profile P14414 = {"14414", &P14414_4};	/*Profile ID - 14414*/
const Value V12130_1_1 = {S2, NULL};	/*Value = True*/
const Property P12130_1 = {S50, 0, &V12130_1_1, NULL};	/*Property = IsMobile*/
const Value V12130_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12130_5 = {S52, 0, &V12130_5_1233, &P12130_1};	/*Property = ScreenPixelsHeight*/
const Value V12130_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12130_4 = {S54, 0, &V12130_4_1235, &P12130_5};	/*Property = ScreenPixelsWidth*/
const Profile P12130 = {"12130", &P12130_4};	/*Profile ID - 12130*/
const Value V12129_1_1 = {S2, NULL};	/*Value = True*/
const Property P12129_1 = {S50, 0, &V12129_1_1, NULL};	/*Property = IsMobile*/
const Value V12129_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P12129_5 = {S52, 0, &V12129_5_1280, &P12129_1};	/*Property = ScreenPixelsHeight*/
const Value V12129_4_1368 = {S74, NULL};	/*Value = 208*/
const Property P12129_4 = {S54, 0, &V12129_4_1368, &P12129_5};	/*Property = ScreenPixelsWidth*/
const Profile P12129 = {"12129", &P12129_4};	/*Profile ID - 12129*/
const Value V12132_1_1 = {S2, NULL};	/*Value = True*/
const Property P12132_1 = {S50, 0, &V12132_1_1, NULL};	/*Property = IsMobile*/
const Value V12132_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12132_5 = {S52, 0, &V12132_5_1222, &P12132_1};	/*Property = ScreenPixelsHeight*/
const Value V12132_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12132_4 = {S54, 0, &V12132_4_1224, &P12132_5};	/*Property = ScreenPixelsWidth*/
const Profile P12132 = {"12132", &P12132_4};	/*Profile ID - 12132*/
const Value V14432_1_1 = {S2, NULL};	/*Value = True*/
const Property P14432_1 = {S50, 0, &V14432_1_1, NULL};	/*Property = IsMobile*/
const Value V14432_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14432_5 = {S52, 0, &V14432_5_1277, &P14432_1};	/*Property = ScreenPixelsHeight*/
const Value V14432_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14432_4 = {S54, 0, &V14432_4_1224, &P14432_5};	/*Property = ScreenPixelsWidth*/
const Profile P14432 = {"14432", &P14432_4};	/*Profile ID - 14432*/
const Value V12140_1_1 = {S2, NULL};	/*Value = True*/
const Property P12140_1 = {S50, 0, &V12140_1_1, NULL};	/*Property = IsMobile*/
const Value V12140_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12140_5 = {S52, 0, &V12140_5_1277, &P12140_1};	/*Property = ScreenPixelsHeight*/
const Value V12140_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P12140_4 = {S54, 0, &V12140_4_1278, &P12140_5};	/*Property = ScreenPixelsWidth*/
const Profile P12140 = {"12140", &P12140_4};	/*Profile ID - 12140*/
const Value V12143_1_1 = {S2, NULL};	/*Value = True*/
const Property P12143_1 = {S50, 0, &V12143_1_1, NULL};	/*Property = IsMobile*/
const Value V12143_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12143_5 = {S52, 0, &V12143_5_1222, &P12143_1};	/*Property = ScreenPixelsHeight*/
const Value V12143_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12143_4 = {S54, 0, &V12143_4_1224, &P12143_5};	/*Property = ScreenPixelsWidth*/
const Profile P12143 = {"12143", &P12143_4};	/*Profile ID - 12143*/
const Value V12142_1_1 = {S2, NULL};	/*Value = True*/
const Property P12142_1 = {S50, 0, &V12142_1_1, NULL};	/*Property = IsMobile*/
const Value V12142_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P12142_5 = {S52, 0, &V12142_5_1275, &P12142_1};	/*Property = ScreenPixelsHeight*/
const Value V12142_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P12142_4 = {S54, 0, &V12142_4_1278, &P12142_5};	/*Property = ScreenPixelsWidth*/
const Profile P12142 = {"12142", &P12142_4};	/*Profile ID - 12142*/
const Value V13484_1_1 = {S2, NULL};	/*Value = True*/
const Property P13484_1 = {S50, 0, &V13484_1_1, NULL};	/*Property = IsMobile*/
const Value V13484_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13484_5 = {S52, 0, &V13484_5_1293, &P13484_1};	/*Property = ScreenPixelsHeight*/
const Value V13484_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13484_4 = {S54, 0, &V13484_4_1296, &P13484_5};	/*Property = ScreenPixelsWidth*/
const Profile P13484 = {"13484", &P13484_4};	/*Profile ID - 13484*/
const Value V15722_1_1 = {S2, NULL};	/*Value = True*/
const Property P15722_1 = {S50, 0, &V15722_1_1, NULL};	/*Property = IsMobile*/
const Value V15722_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15722_5 = {S52, 0, &V15722_5_1380, &P15722_1};	/*Property = ScreenPixelsHeight*/
const Value V15722_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15722_4 = {S54, 0, &V15722_4_1279, &P15722_5};	/*Property = ScreenPixelsWidth*/
const Profile P15722 = {"15722", &P15722_4};	/*Profile ID - 15722*/
const Value V14709_1_1 = {S2, NULL};	/*Value = True*/
const Property P14709_1 = {S50, 0, &V14709_1_1, NULL};	/*Property = IsMobile*/
const Value V14709_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14709_5 = {S52, 0, &V14709_5_1304, &P14709_1};	/*Property = ScreenPixelsHeight*/
const Value V14709_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14709_4 = {S54, 0, &V14709_4_1224, &P14709_5};	/*Property = ScreenPixelsWidth*/
const Profile P14709 = {"14709", &P14709_4};	/*Profile ID - 14709*/
const Value V17058_1_1 = {S2, NULL};	/*Value = True*/
const Property P17058_1 = {S50, 0, &V17058_1_1, NULL};	/*Property = IsMobile*/
const Value V17058_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17058_5 = {S52, 0, &V17058_5_1222, &P17058_1};	/*Property = ScreenPixelsHeight*/
const Value V17058_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17058_4 = {S54, 0, &V17058_4_1279, &P17058_5};	/*Property = ScreenPixelsWidth*/
const Profile P17058 = {"17058", &P17058_4};	/*Profile ID - 17058*/
const Value V13617_1_1 = {S2, NULL};	/*Value = True*/
const Property P13617_1 = {S50, 0, &V13617_1_1, NULL};	/*Property = IsMobile*/
const Value V13617_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13617_5 = {S52, 0, &V13617_5_1222, &P13617_1};	/*Property = ScreenPixelsHeight*/
const Value V13617_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13617_4 = {S54, 0, &V13617_4_1224, &P13617_5};	/*Property = ScreenPixelsWidth*/
const Profile P13617 = {"13617", &P13617_4};	/*Profile ID - 13617*/
const Value V3807_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_279 = {S1, 0, &V3807_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3807_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_282 = {S3, 0, &V3807_282_57155, &P3807_279};	/*Property = BlobBuilder*/
const Value V3807_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_350 = {S4, 0, &V3807_350_57446, &P3807_282};	/*Property = Canvas*/
const Value V3807_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_283 = {S5, 0, &V3807_283_57161, &P3807_350};	/*Property = CssBackground*/
const Value V3807_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_284 = {S6, 0, &V3807_284_57164, &P3807_283};	/*Property = CssBorderImage*/
const Value V3807_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_285 = {S7, 0, &V3807_285_57167, &P3807_284};	/*Property = CssCanvas*/
const Value V3807_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_286 = {S8, 0, &V3807_286_57170, &P3807_285};	/*Property = CssColor*/
const Value V3807_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_237 = {S9, 0, &V3807_237_55786, &P3807_286};	/*Property = CssColumn*/
const Value V3807_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_287 = {S10, 0, &V3807_287_57173, &P3807_237};	/*Property = CssFlexbox*/
const Value V3807_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_288 = {S11, 0, &V3807_288_57176, &P3807_287};	/*Property = CssFont*/
const Value V3807_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_289 = {S12, 0, &V3807_289_86899, &P3807_288};	/*Property = CssImages*/
const Value V3807_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_291 = {S13, 0, &V3807_291_57182, &P3807_289};	/*Property = CssMediaQueries*/
const Value V3807_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_292 = {S14, 0, &V3807_292_57185, &P3807_291};	/*Property = CssMinMax*/
const Value V3807_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_293 = {S15, 0, &V3807_293_57188, &P3807_292};	/*Property = CssOverflow*/
const Value V3807_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_294 = {S16, 0, &V3807_294_57191, &P3807_293};	/*Property = CssPosition*/
const Value V3807_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_295 = {S17, 0, &V3807_295_57194, &P3807_294};	/*Property = CssText*/
const Value V3807_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_297 = {S18, 0, &V3807_297_57197, &P3807_295};	/*Property = CssTransforms*/
const Value V3807_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_241 = {S19, 0, &V3807_241_55801, &P3807_297};	/*Property = CssTransitions*/
const Value V3807_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_298 = {S20, 0, &V3807_298_57208, &P3807_241};	/*Property = CssUI*/
const Value V3807_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_300 = {S21, 0, &V3807_300_57217, &P3807_298};	/*Property = DataSet*/
const Value V3807_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_301 = {S22, 0, &V3807_301_57220, &P3807_300};	/*Property = DataUrl*/
const Value V3807_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_302 = {S23, 0, &V3807_302_57223, &P3807_301};	/*Property = DeviceOrientation*/
const Value V3807_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_304 = {S24, 0, &V3807_304_57229, &P3807_302};	/*Property = FileReader*/
const Value V3807_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_305 = {S25, 0, &V3807_305_57232, &P3807_304};	/*Property = FileSaver*/
const Value V3807_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_306 = {S26, 0, &V3807_306_57238, &P3807_305};	/*Property = FileWriter*/
const Value V3807_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_307 = {S27, 0, &V3807_307_57241, &P3807_306};	/*Property = FormData*/
const Value V3807_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_309 = {S28, 0, &V3807_309_57247, &P3807_307};	/*Property = Fullscreen*/
const Value V3807_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_310 = {S29, 0, &V3807_310_57250, &P3807_309};	/*Property = GeoLocation*/
const Value V3807_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_312 = {S30, 0, &V3807_312_57256, &P3807_310};	/*Property = History*/
const Value V3807_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_314 = {S31, 0, &V3807_314_57262, &P3807_312};	/*Property = Html5*/
const Value V3807_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_313 = {S32, 0, &V3807_313_57259, &P3807_314};	/*Property = Html-Media-Capture*/
const Value V3807_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_315 = {S33, 0, &V3807_315_57265, &P3807_313};	/*Property = Iframe*/
const Value V3807_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_316 = {S34, 0, &V3807_316_57268, &P3807_315};	/*Property = IndexedDB*/
const Value V3807_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_317 = {S35, 0, &V3807_317_57271, &P3807_316};	/*Property = Json*/
const Value V3807_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3807_15 = {S37, 0, &V3807_15_28, &P3807_317};	/*Property = LayoutEngine*/
const Value V3807_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_344 = {S38, 0, &V3807_344_57274, &P3807_15};	/*Property = Masking*/
const Value V3807_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_320 = {S39, 0, &V3807_320_57292, &P3807_344};	/*Property = PostMessage*/
const Value V3807_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_321 = {S40, 0, &V3807_321_57295, &P3807_320};	/*Property = Progress*/
const Value V3807_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_322 = {S41, 0, &V3807_322_57298, &P3807_321};	/*Property = Prompts*/
const Value V3807_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_323 = {S42, 0, &V3807_323_57301, &P3807_322};	/*Property = Selector*/
const Value V3807_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_324 = {S43, 0, &V3807_324_57307, &P3807_323};	/*Property = Svg*/
const Value V3807_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_327 = {S44, 0, &V3807_327_57316, &P3807_324};	/*Property = TouchEvents*/
const Value V3807_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_328 = {S45, 0, &V3807_328_57319, &P3807_327};	/*Property = Track*/
const Value V3807_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_329 = {S46, 0, &V3807_329_57325, &P3807_328};	/*Property = Video*/
const Value V3807_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_330 = {S47, 0, &V3807_330_57328, &P3807_329};	/*Property = Viewport*/
const Value V3807_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_332 = {S48, 0, &V3807_332_57340, &P3807_330};	/*Property = WebWorkers*/
const Value V3807_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3807_333 = {S49, 0, &V3807_333_57343, &P3807_332};	/*Property = Xhr2*/
const Profile P3807 = {"3807", &P3807_333};	/*Profile ID - 3807*/
const Value V13316_1_1 = {S2, NULL};	/*Value = True*/
const Property P13316_1 = {S50, 0, &V13316_1_1, NULL};	/*Property = IsMobile*/
const Value V13316_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13316_5 = {S52, 0, &V13316_5_1304, &P13316_1};	/*Property = ScreenPixelsHeight*/
const Value V13316_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13316_4 = {S54, 0, &V13316_4_1224, &P13316_5};	/*Property = ScreenPixelsWidth*/
const Profile P13316 = {"13316", &P13316_4};	/*Profile ID - 13316*/
const Value V18122_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_279 = {S1, 0, &V18122_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18122_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_282 = {S3, 0, &V18122_282_57155, &P18122_279};	/*Property = BlobBuilder*/
const Value V18122_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_350 = {S4, 0, &V18122_350_57446, &P18122_282};	/*Property = Canvas*/
const Value V18122_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_283 = {S5, 0, &V18122_283_57161, &P18122_350};	/*Property = CssBackground*/
const Value V18122_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_284 = {S6, 0, &V18122_284_57164, &P18122_283};	/*Property = CssBorderImage*/
const Value V18122_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_285 = {S7, 0, &V18122_285_57167, &P18122_284};	/*Property = CssCanvas*/
const Value V18122_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_286 = {S8, 0, &V18122_286_57170, &P18122_285};	/*Property = CssColor*/
const Value V18122_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_237 = {S9, 0, &V18122_237_55786, &P18122_286};	/*Property = CssColumn*/
const Value V18122_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_287 = {S10, 0, &V18122_287_57173, &P18122_237};	/*Property = CssFlexbox*/
const Value V18122_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_288 = {S11, 0, &V18122_288_57176, &P18122_287};	/*Property = CssFont*/
const Value V18122_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_289 = {S12, 0, &V18122_289_86899, &P18122_288};	/*Property = CssImages*/
const Value V18122_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_291 = {S13, 0, &V18122_291_57182, &P18122_289};	/*Property = CssMediaQueries*/
const Value V18122_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_292 = {S14, 0, &V18122_292_57185, &P18122_291};	/*Property = CssMinMax*/
const Value V18122_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_293 = {S15, 0, &V18122_293_57188, &P18122_292};	/*Property = CssOverflow*/
const Value V18122_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_294 = {S16, 0, &V18122_294_57191, &P18122_293};	/*Property = CssPosition*/
const Value V18122_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_295 = {S17, 0, &V18122_295_57194, &P18122_294};	/*Property = CssText*/
const Value V18122_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_297 = {S18, 0, &V18122_297_57197, &P18122_295};	/*Property = CssTransforms*/
const Value V18122_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_241 = {S19, 0, &V18122_241_55801, &P18122_297};	/*Property = CssTransitions*/
const Value V18122_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_298 = {S20, 0, &V18122_298_57208, &P18122_241};	/*Property = CssUI*/
const Value V18122_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_300 = {S21, 0, &V18122_300_57217, &P18122_298};	/*Property = DataSet*/
const Value V18122_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_301 = {S22, 0, &V18122_301_57220, &P18122_300};	/*Property = DataUrl*/
const Value V18122_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_302 = {S23, 0, &V18122_302_57223, &P18122_301};	/*Property = DeviceOrientation*/
const Value V18122_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_304 = {S24, 0, &V18122_304_57229, &P18122_302};	/*Property = FileReader*/
const Value V18122_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_305 = {S25, 0, &V18122_305_57232, &P18122_304};	/*Property = FileSaver*/
const Value V18122_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_306 = {S26, 0, &V18122_306_57238, &P18122_305};	/*Property = FileWriter*/
const Value V18122_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_307 = {S27, 0, &V18122_307_57241, &P18122_306};	/*Property = FormData*/
const Value V18122_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_309 = {S28, 0, &V18122_309_57247, &P18122_307};	/*Property = Fullscreen*/
const Value V18122_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_310 = {S29, 0, &V18122_310_57250, &P18122_309};	/*Property = GeoLocation*/
const Value V18122_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_312 = {S30, 0, &V18122_312_57256, &P18122_310};	/*Property = History*/
const Value V18122_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_314 = {S31, 0, &V18122_314_57262, &P18122_312};	/*Property = Html5*/
const Value V18122_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_313 = {S32, 0, &V18122_313_57259, &P18122_314};	/*Property = Html-Media-Capture*/
const Value V18122_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_315 = {S33, 0, &V18122_315_57265, &P18122_313};	/*Property = Iframe*/
const Value V18122_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_316 = {S34, 0, &V18122_316_57268, &P18122_315};	/*Property = IndexedDB*/
const Value V18122_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_317 = {S35, 0, &V18122_317_57271, &P18122_316};	/*Property = Json*/
const Value V18122_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18122_15 = {S37, 0, &V18122_15_31, &P18122_317};	/*Property = LayoutEngine*/
const Value V18122_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_344 = {S38, 0, &V18122_344_57274, &P18122_15};	/*Property = Masking*/
const Value V18122_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_320 = {S39, 0, &V18122_320_57292, &P18122_344};	/*Property = PostMessage*/
const Value V18122_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_321 = {S40, 0, &V18122_321_57295, &P18122_320};	/*Property = Progress*/
const Value V18122_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_322 = {S41, 0, &V18122_322_57298, &P18122_321};	/*Property = Prompts*/
const Value V18122_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_323 = {S42, 0, &V18122_323_57301, &P18122_322};	/*Property = Selector*/
const Value V18122_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_324 = {S43, 0, &V18122_324_57307, &P18122_323};	/*Property = Svg*/
const Value V18122_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_327 = {S44, 0, &V18122_327_57316, &P18122_324};	/*Property = TouchEvents*/
const Value V18122_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_328 = {S45, 0, &V18122_328_57319, &P18122_327};	/*Property = Track*/
const Value V18122_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_329 = {S46, 0, &V18122_329_57325, &P18122_328};	/*Property = Video*/
const Value V18122_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_330 = {S47, 0, &V18122_330_57328, &P18122_329};	/*Property = Viewport*/
const Value V18122_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_332 = {S48, 0, &V18122_332_57340, &P18122_330};	/*Property = WebWorkers*/
const Value V18122_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18122_333 = {S49, 0, &V18122_333_57343, &P18122_332};	/*Property = Xhr2*/
const Profile P18122 = {"18122", &P18122_333};	/*Profile ID - 18122*/
const Value V13317_1_1 = {S2, NULL};	/*Value = True*/
const Property P13317_1 = {S50, 0, &V13317_1_1, NULL};	/*Property = IsMobile*/
const Value V13317_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13317_5 = {S52, 0, &V13317_5_1222, &P13317_1};	/*Property = ScreenPixelsHeight*/
const Value V13317_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13317_4 = {S54, 0, &V13317_4_1224, &P13317_5};	/*Property = ScreenPixelsWidth*/
const Profile P13317 = {"13317", &P13317_4};	/*Profile ID - 13317*/
const Value V13315_1_1 = {S2, NULL};	/*Value = True*/
const Property P13315_1 = {S50, 0, &V13315_1_1, NULL};	/*Property = IsMobile*/
const Value V13315_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13315_5 = {S52, 0, &V13315_5_1222, &P13315_1};	/*Property = ScreenPixelsHeight*/
const Value V13315_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13315_4 = {S54, 0, &V13315_4_1224, &P13315_5};	/*Property = ScreenPixelsWidth*/
const Profile P13315 = {"13315", &P13315_4};	/*Profile ID - 13315*/
const Value V12399_1_1 = {S2, NULL};	/*Value = True*/
const Property P12399_1 = {S50, 0, &V12399_1_1, NULL};	/*Property = IsMobile*/
const Value V12399_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P12399_5 = {S52, 0, &V12399_5_1275, &P12399_1};	/*Property = ScreenPixelsHeight*/
const Value V12399_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P12399_4 = {S54, 0, &V12399_4_1278, &P12399_5};	/*Property = ScreenPixelsWidth*/
const Profile P12399 = {"12399", &P12399_4};	/*Profile ID - 12399*/
const Value V12152_1_1 = {S2, NULL};	/*Value = True*/
const Property P12152_1 = {S50, 0, &V12152_1_1, NULL};	/*Property = IsMobile*/
const Value V12152_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12152_5 = {S52, 0, &V12152_5_1294, &P12152_1};	/*Property = ScreenPixelsHeight*/
const Value V12152_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12152_4 = {S54, 0, &V12152_4_1296, &P12152_5};	/*Property = ScreenPixelsWidth*/
const Profile P12152 = {"12152", &P12152_4};	/*Profile ID - 12152*/
const Value V12148_1_1 = {S2, NULL};	/*Value = True*/
const Property P12148_1 = {S50, 0, &V12148_1_1, NULL};	/*Property = IsMobile*/
const Value V12148_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12148_5 = {S52, 0, &V12148_5_1222, &P12148_1};	/*Property = ScreenPixelsHeight*/
const Value V12148_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12148_4 = {S54, 0, &V12148_4_1224, &P12148_5};	/*Property = ScreenPixelsWidth*/
const Profile P12148 = {"12148", &P12148_4};	/*Profile ID - 12148*/
const Value V14743_1_1 = {S2, NULL};	/*Value = True*/
const Property P14743_1 = {S50, 0, &V14743_1_1, NULL};	/*Property = IsMobile*/
const Value V14743_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14743_5 = {S52, 0, &V14743_5_1222, &P14743_1};	/*Property = ScreenPixelsHeight*/
const Value V14743_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14743_4 = {S54, 0, &V14743_4_1224, &P14743_5};	/*Property = ScreenPixelsWidth*/
const Profile P14743 = {"14743", &P14743_4};	/*Profile ID - 14743*/
const Value V13319_1_1 = {S2, NULL};	/*Value = True*/
const Property P13319_1 = {S50, 0, &V13319_1_1, NULL};	/*Property = IsMobile*/
const Value V13319_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13319_5 = {S52, 0, &V13319_5_1304, &P13319_1};	/*Property = ScreenPixelsHeight*/
const Value V13319_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13319_4 = {S54, 0, &V13319_4_1224, &P13319_5};	/*Property = ScreenPixelsWidth*/
const Profile P13319 = {"13319", &P13319_4};	/*Profile ID - 13319*/
const Profile P22839 = {"22839", NULL};	/*Profile ID - 22839*/
const Value V12164_1_1 = {S2, NULL};	/*Value = True*/
const Property P12164_1 = {S50, 0, &V12164_1_1, NULL};	/*Property = IsMobile*/
const Value V12164_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12164_5 = {S52, 0, &V12164_5_1380, &P12164_1};	/*Property = ScreenPixelsHeight*/
const Value V12164_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12164_4 = {S54, 0, &V12164_4_1279, &P12164_5};	/*Property = ScreenPixelsWidth*/
const Profile P12164 = {"12164", &P12164_4};	/*Profile ID - 12164*/
const Value V12273_1_1 = {S2, NULL};	/*Value = True*/
const Property P12273_1 = {S50, 0, &V12273_1_1, NULL};	/*Property = IsMobile*/
const Value V12273_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12273_5 = {S52, 0, &V12273_5_1222, &P12273_1};	/*Property = ScreenPixelsHeight*/
const Value V12273_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12273_4 = {S54, 0, &V12273_4_1224, &P12273_5};	/*Property = ScreenPixelsWidth*/
const Profile P12273 = {"12273", &P12273_4};	/*Profile ID - 12273*/
const Value V12161_1_1 = {S2, NULL};	/*Value = True*/
const Property P12161_1 = {S50, 0, &V12161_1_1, NULL};	/*Property = IsMobile*/
const Value V12161_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12161_5 = {S52, 0, &V12161_5_1233, &P12161_1};	/*Property = ScreenPixelsHeight*/
const Value V12161_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12161_4 = {S54, 0, &V12161_4_1235, &P12161_5};	/*Property = ScreenPixelsWidth*/
const Profile P12161 = {"12161", &P12161_4};	/*Profile ID - 12161*/
const Value V12172_1_1 = {S2, NULL};	/*Value = True*/
const Property P12172_1 = {S50, 0, &V12172_1_1, NULL};	/*Property = IsMobile*/
const Value V12172_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12172_5 = {S52, 0, &V12172_5_1233, &P12172_1};	/*Property = ScreenPixelsHeight*/
const Value V12172_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12172_4 = {S54, 0, &V12172_4_1235, &P12172_5};	/*Property = ScreenPixelsWidth*/
const Profile P12172 = {"12172", &P12172_4};	/*Profile ID - 12172*/
const Value V12173_1_1 = {S2, NULL};	/*Value = True*/
const Property P12173_1 = {S50, 0, &V12173_1_1, NULL};	/*Property = IsMobile*/
const Value V12173_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12173_5 = {S52, 0, &V12173_5_1380, &P12173_1};	/*Property = ScreenPixelsHeight*/
const Value V12173_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P12173_4 = {S54, 0, &V12173_4_1943, &P12173_5};	/*Property = ScreenPixelsWidth*/
const Profile P12173 = {"12173", &P12173_4};	/*Profile ID - 12173*/
const Value V12178_1_1 = {S2, NULL};	/*Value = True*/
const Property P12178_1 = {S50, 0, &V12178_1_1, NULL};	/*Property = IsMobile*/
const Value V12178_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12178_5 = {S52, 0, &V12178_5_1380, &P12178_1};	/*Property = ScreenPixelsHeight*/
const Value V12178_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12178_4 = {S54, 0, &V12178_4_1279, &P12178_5};	/*Property = ScreenPixelsWidth*/
const Profile P12178 = {"12178", &P12178_4};	/*Profile ID - 12178*/
const Value V12182_1_1 = {S2, NULL};	/*Value = True*/
const Property P12182_1 = {S50, 0, &V12182_1_1, NULL};	/*Property = IsMobile*/
const Value V12182_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12182_5 = {S52, 0, &V12182_5_1293, &P12182_1};	/*Property = ScreenPixelsHeight*/
const Value V12182_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12182_4 = {S54, 0, &V12182_4_1296, &P12182_5};	/*Property = ScreenPixelsWidth*/
const Profile P12182 = {"12182", &P12182_4};	/*Profile ID - 12182*/
const Value V14603_1_1 = {S2, NULL};	/*Value = True*/
const Property P14603_1 = {S50, 0, &V14603_1_1, NULL};	/*Property = IsMobile*/
const Value V14603_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14603_5 = {S52, 0, &V14603_5_1293, &P14603_1};	/*Property = ScreenPixelsHeight*/
const Value V14603_4_24031 = {S71, NULL};	/*Value = 120*/
const Property P14603_4 = {S54, 0, &V14603_4_24031, &P14603_5};	/*Property = ScreenPixelsWidth*/
const Profile P14603 = {"14603", &P14603_4};	/*Profile ID - 14603*/
const Value V11785_1_1 = {S2, NULL};	/*Value = True*/
const Property P11785_1 = {S50, 0, &V11785_1_1, NULL};	/*Property = IsMobile*/
const Value V11785_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11785_5 = {S52, 0, &V11785_5_1222, &P11785_1};	/*Property = ScreenPixelsHeight*/
const Value V11785_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11785_4 = {S54, 0, &V11785_4_1224, &P11785_5};	/*Property = ScreenPixelsWidth*/
const Profile P11785 = {"11785", &P11785_4};	/*Profile ID - 11785*/
const Value V12192_1_1 = {S2, NULL};	/*Value = True*/
const Property P12192_1 = {S50, 0, &V12192_1_1, NULL};	/*Property = IsMobile*/
const Value V12192_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12192_5 = {S52, 0, &V12192_5_1227, &P12192_1};	/*Property = ScreenPixelsHeight*/
const Value V12192_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12192_4 = {S54, 0, &V12192_4_1229, &P12192_5};	/*Property = ScreenPixelsWidth*/
const Profile P12192 = {"12192", &P12192_4};	/*Profile ID - 12192*/
const Value V12193_1_1 = {S2, NULL};	/*Value = True*/
const Property P12193_1 = {S50, 0, &V12193_1_1, NULL};	/*Property = IsMobile*/
const Value V12193_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12193_5 = {S52, 0, &V12193_5_1222, &P12193_1};	/*Property = ScreenPixelsHeight*/
const Value V12193_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12193_4 = {S54, 0, &V12193_4_1224, &P12193_5};	/*Property = ScreenPixelsWidth*/
const Profile P12193 = {"12193", &P12193_4};	/*Profile ID - 12193*/
const Value V14305_1_1 = {S2, NULL};	/*Value = True*/
const Property P14305_1 = {S50, 0, &V14305_1_1, NULL};	/*Property = IsMobile*/
const Value V14305_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14305_5 = {S52, 0, &V14305_5_1222, &P14305_1};	/*Property = ScreenPixelsHeight*/
const Value V14305_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14305_4 = {S54, 0, &V14305_4_1224, &P14305_5};	/*Property = ScreenPixelsWidth*/
const Profile P14305 = {"14305", &P14305_4};	/*Profile ID - 14305*/
const Value V12194_1_1 = {S2, NULL};	/*Value = True*/
const Property P12194_1 = {S50, 0, &V12194_1_1, NULL};	/*Property = IsMobile*/
const Value V12194_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12194_5 = {S52, 0, &V12194_5_1222, &P12194_1};	/*Property = ScreenPixelsHeight*/
const Value V12194_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12194_4 = {S54, 0, &V12194_4_1224, &P12194_5};	/*Property = ScreenPixelsWidth*/
const Profile P12194 = {"12194", &P12194_4};	/*Profile ID - 12194*/
const Value V14392_1_1 = {S2, NULL};	/*Value = True*/
const Property P14392_1 = {S50, 0, &V14392_1_1, NULL};	/*Property = IsMobile*/
const Value V14392_5_1512 = {S77, NULL};	/*Value = 416*/
const Property P14392_5 = {S52, 0, &V14392_5_1512, &P14392_1};	/*Property = ScreenPixelsHeight*/
const Value V14392_4_1513 = {S78, NULL};	/*Value = 352*/
const Property P14392_4 = {S54, 0, &V14392_4_1513, &P14392_5};	/*Property = ScreenPixelsWidth*/
const Profile P14392 = {"14392", &P14392_4};	/*Profile ID - 14392*/
const Value V15081_1_1 = {S2, NULL};	/*Value = True*/
const Property P15081_1 = {S50, 0, &V15081_1_1, NULL};	/*Property = IsMobile*/
const Value V15081_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15081_5 = {S52, 0, &V15081_5_1304, &P15081_1};	/*Property = ScreenPixelsHeight*/
const Value V15081_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15081_4 = {S54, 0, &V15081_4_1224, &P15081_5};	/*Property = ScreenPixelsWidth*/
const Profile P15081 = {"15081", &P15081_4};	/*Profile ID - 15081*/
const Value V14218_1_1 = {S2, NULL};	/*Value = True*/
const Property P14218_1 = {S50, 0, &V14218_1_1, NULL};	/*Property = IsMobile*/
const Value V14218_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14218_5 = {S52, 0, &V14218_5_1222, &P14218_1};	/*Property = ScreenPixelsHeight*/
const Value V14218_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14218_4 = {S54, 0, &V14218_4_1224, &P14218_5};	/*Property = ScreenPixelsWidth*/
const Profile P14218 = {"14218", &P14218_4};	/*Profile ID - 14218*/
const Value V12026_1_1 = {S2, NULL};	/*Value = True*/
const Property P12026_1 = {S50, 0, &V12026_1_1, NULL};	/*Property = IsMobile*/
const Value V12026_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12026_5 = {S52, 0, &V12026_5_1222, &P12026_1};	/*Property = ScreenPixelsHeight*/
const Value V12026_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12026_4 = {S54, 0, &V12026_4_1224, &P12026_5};	/*Property = ScreenPixelsWidth*/
const Profile P12026 = {"12026", &P12026_4};	/*Profile ID - 12026*/
const Value V5001_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_279 = {S1, 0, &V5001_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V5001_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_282 = {S3, 0, &V5001_282_57155, &P5001_279};	/*Property = BlobBuilder*/
const Value V5001_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_350 = {S4, 0, &V5001_350_57446, &P5001_282};	/*Property = Canvas*/
const Value V5001_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_283 = {S5, 0, &V5001_283_57161, &P5001_350};	/*Property = CssBackground*/
const Value V5001_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_284 = {S6, 0, &V5001_284_57164, &P5001_283};	/*Property = CssBorderImage*/
const Value V5001_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_285 = {S7, 0, &V5001_285_57167, &P5001_284};	/*Property = CssCanvas*/
const Value V5001_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_286 = {S8, 0, &V5001_286_57170, &P5001_285};	/*Property = CssColor*/
const Value V5001_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_237 = {S9, 0, &V5001_237_55786, &P5001_286};	/*Property = CssColumn*/
const Value V5001_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_287 = {S10, 0, &V5001_287_57173, &P5001_237};	/*Property = CssFlexbox*/
const Value V5001_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_288 = {S11, 0, &V5001_288_57176, &P5001_287};	/*Property = CssFont*/
const Value V5001_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_289 = {S12, 0, &V5001_289_86899, &P5001_288};	/*Property = CssImages*/
const Value V5001_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_291 = {S13, 0, &V5001_291_57182, &P5001_289};	/*Property = CssMediaQueries*/
const Value V5001_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_292 = {S14, 0, &V5001_292_57185, &P5001_291};	/*Property = CssMinMax*/
const Value V5001_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_293 = {S15, 0, &V5001_293_57188, &P5001_292};	/*Property = CssOverflow*/
const Value V5001_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_294 = {S16, 0, &V5001_294_57191, &P5001_293};	/*Property = CssPosition*/
const Value V5001_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_295 = {S17, 0, &V5001_295_57194, &P5001_294};	/*Property = CssText*/
const Value V5001_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_297 = {S18, 0, &V5001_297_57197, &P5001_295};	/*Property = CssTransforms*/
const Value V5001_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_241 = {S19, 0, &V5001_241_55801, &P5001_297};	/*Property = CssTransitions*/
const Value V5001_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_298 = {S20, 0, &V5001_298_57208, &P5001_241};	/*Property = CssUI*/
const Value V5001_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_300 = {S21, 0, &V5001_300_57217, &P5001_298};	/*Property = DataSet*/
const Value V5001_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_301 = {S22, 0, &V5001_301_57220, &P5001_300};	/*Property = DataUrl*/
const Value V5001_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_302 = {S23, 0, &V5001_302_57223, &P5001_301};	/*Property = DeviceOrientation*/
const Value V5001_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_304 = {S24, 0, &V5001_304_57229, &P5001_302};	/*Property = FileReader*/
const Value V5001_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_305 = {S25, 0, &V5001_305_57232, &P5001_304};	/*Property = FileSaver*/
const Value V5001_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_306 = {S26, 0, &V5001_306_57238, &P5001_305};	/*Property = FileWriter*/
const Value V5001_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_307 = {S27, 0, &V5001_307_57241, &P5001_306};	/*Property = FormData*/
const Value V5001_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_309 = {S28, 0, &V5001_309_57247, &P5001_307};	/*Property = Fullscreen*/
const Value V5001_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_310 = {S29, 0, &V5001_310_57250, &P5001_309};	/*Property = GeoLocation*/
const Value V5001_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_312 = {S30, 0, &V5001_312_57256, &P5001_310};	/*Property = History*/
const Value V5001_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_314 = {S31, 0, &V5001_314_57262, &P5001_312};	/*Property = Html5*/
const Value V5001_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_313 = {S32, 0, &V5001_313_57259, &P5001_314};	/*Property = Html-Media-Capture*/
const Value V5001_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_315 = {S33, 0, &V5001_315_57265, &P5001_313};	/*Property = Iframe*/
const Value V5001_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_316 = {S34, 0, &V5001_316_57268, &P5001_315};	/*Property = IndexedDB*/
const Value V5001_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_317 = {S35, 0, &V5001_317_57271, &P5001_316};	/*Property = Json*/
const Value V5001_15_43263 = {S65, NULL};	/*Value = Netfront*/
const Property P5001_15 = {S37, 0, &V5001_15_43263, &P5001_317};	/*Property = LayoutEngine*/
const Value V5001_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_344 = {S38, 0, &V5001_344_57274, &P5001_15};	/*Property = Masking*/
const Value V5001_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_320 = {S39, 0, &V5001_320_57292, &P5001_344};	/*Property = PostMessage*/
const Value V5001_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_321 = {S40, 0, &V5001_321_57295, &P5001_320};	/*Property = Progress*/
const Value V5001_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_322 = {S41, 0, &V5001_322_57298, &P5001_321};	/*Property = Prompts*/
const Value V5001_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_323 = {S42, 0, &V5001_323_57301, &P5001_322};	/*Property = Selector*/
const Value V5001_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_324 = {S43, 0, &V5001_324_57307, &P5001_323};	/*Property = Svg*/
const Value V5001_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_327 = {S44, 0, &V5001_327_57316, &P5001_324};	/*Property = TouchEvents*/
const Value V5001_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_328 = {S45, 0, &V5001_328_57319, &P5001_327};	/*Property = Track*/
const Value V5001_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_329 = {S46, 0, &V5001_329_57325, &P5001_328};	/*Property = Video*/
const Value V5001_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_330 = {S47, 0, &V5001_330_57328, &P5001_329};	/*Property = Viewport*/
const Value V5001_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_332 = {S48, 0, &V5001_332_57340, &P5001_330};	/*Property = WebWorkers*/
const Value V5001_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P5001_333 = {S49, 0, &V5001_333_57343, &P5001_332};	/*Property = Xhr2*/
const Profile P5001 = {"5001", &P5001_333};	/*Profile ID - 5001*/
const Value V12713_1_1 = {S2, NULL};	/*Value = True*/
const Property P12713_1 = {S50, 0, &V12713_1_1, NULL};	/*Property = IsMobile*/
const Value V12713_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12713_5 = {S52, 0, &V12713_5_1222, &P12713_1};	/*Property = ScreenPixelsHeight*/
const Value V12713_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12713_4 = {S54, 0, &V12713_4_1224, &P12713_5};	/*Property = ScreenPixelsWidth*/
const Profile P12713 = {"12713", &P12713_4};	/*Profile ID - 12713*/
const Value V13322_1_1 = {S2, NULL};	/*Value = True*/
const Property P13322_1 = {S50, 0, &V13322_1_1, NULL};	/*Property = IsMobile*/
const Value V13322_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13322_5 = {S52, 0, &V13322_5_1380, &P13322_1};	/*Property = ScreenPixelsHeight*/
const Value V13322_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13322_4 = {S54, 0, &V13322_4_1279, &P13322_5};	/*Property = ScreenPixelsWidth*/
const Profile P13322 = {"13322", &P13322_4};	/*Profile ID - 13322*/
const Value V12277_1_1 = {S2, NULL};	/*Value = True*/
const Property P12277_1 = {S50, 0, &V12277_1_1, NULL};	/*Property = IsMobile*/
const Value V12277_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12277_5 = {S52, 0, &V12277_5_1380, &P12277_1};	/*Property = ScreenPixelsHeight*/
const Value V12277_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12277_4 = {S54, 0, &V12277_4_1279, &P12277_5};	/*Property = ScreenPixelsWidth*/
const Profile P12277 = {"12277", &P12277_4};	/*Profile ID - 12277*/
const Value V14877_1_1 = {S2, NULL};	/*Value = True*/
const Property P14877_1 = {S50, 0, &V14877_1_1, NULL};	/*Property = IsMobile*/
const Value V14877_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14877_5 = {S52, 0, &V14877_5_1227, &P14877_1};	/*Property = ScreenPixelsHeight*/
const Value V14877_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14877_4 = {S54, 0, &V14877_4_1229, &P14877_5};	/*Property = ScreenPixelsWidth*/
const Profile P14877 = {"14877", &P14877_4};	/*Profile ID - 14877*/
const Value V17245_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17245_279 = {S1, 0, &V17245_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17245_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17245_282 = {S3, 0, &V17245_282_57154, &P17245_279};	/*Property = BlobBuilder*/
const Value V17245_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17245_350 = {S4, 0, &V17245_350_57444, &P17245_282};	/*Property = Canvas*/
const Value V17245_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17245_283 = {S5, 0, &V17245_283_57159, &P17245_350};	/*Property = CssBackground*/
const Value V17245_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17245_284 = {S6, 0, &V17245_284_57163, &P17245_283};	/*Property = CssBorderImage*/
const Value V17245_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17245_285 = {S7, 0, &V17245_285_57166, &P17245_284};	/*Property = CssCanvas*/
const Value V17245_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17245_286 = {S8, 0, &V17245_286_57168, &P17245_285};	/*Property = CssColor*/
const Value V17245_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17245_237 = {S9, 0, &V17245_237_55784, &P17245_286};	/*Property = CssColumn*/
const Value V17245_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17245_287 = {S10, 0, &V17245_287_57172, &P17245_237};	/*Property = CssFlexbox*/
const Value V17245_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17245_288 = {S11, 0, &V17245_288_57175, &P17245_287};	/*Property = CssFont*/
const Value V17245_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17245_289 = {S12, 0, &V17245_289_57759, &P17245_288};	/*Property = CssImages*/
const Value V17245_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17245_291 = {S13, 0, &V17245_291_57181, &P17245_289};	/*Property = CssMediaQueries*/
const Value V17245_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17245_292 = {S14, 0, &V17245_292_57183, &P17245_291};	/*Property = CssMinMax*/
const Value V17245_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17245_293 = {S15, 0, &V17245_293_57187, &P17245_292};	/*Property = CssOverflow*/
const Value V17245_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17245_294 = {S16, 0, &V17245_294_57189, &P17245_293};	/*Property = CssPosition*/
const Value V17245_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17245_295 = {S17, 0, &V17245_295_57192, &P17245_294};	/*Property = CssText*/
const Value V17245_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17245_297 = {S18, 0, &V17245_297_57195, &P17245_295};	/*Property = CssTransforms*/
const Value V17245_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17245_241 = {S19, 0, &V17245_241_55799, &P17245_297};	/*Property = CssTransitions*/
const Value V17245_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17245_298 = {S20, 0, &V17245_298_57206, &P17245_241};	/*Property = CssUI*/
const Value V17245_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17245_300 = {S21, 0, &V17245_300_57216, &P17245_298};	/*Property = DataSet*/
const Value V17245_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17245_301 = {S22, 0, &V17245_301_57218, &P17245_300};	/*Property = DataUrl*/
const Value V17245_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17245_302 = {S23, 0, &V17245_302_57222, &P17245_301};	/*Property = DeviceOrientation*/
const Value V17245_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17245_304 = {S24, 0, &V17245_304_57228, &P17245_302};	/*Property = FileReader*/
const Value V17245_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17245_305 = {S25, 0, &V17245_305_57231, &P17245_304};	/*Property = FileSaver*/
const Value V17245_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17245_306 = {S26, 0, &V17245_306_57237, &P17245_305};	/*Property = FileWriter*/
const Value V17245_307_57240 = {S0, NULL};	/*Value = False*/
const Property P17245_307 = {S27, 0, &V17245_307_57240, &P17245_306};	/*Property = FormData*/
const Value V17245_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17245_309 = {S28, 0, &V17245_309_57246, &P17245_307};	/*Property = Fullscreen*/
const Value V17245_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17245_310 = {S29, 0, &V17245_310_57248, &P17245_309};	/*Property = GeoLocation*/
const Value V17245_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17245_312 = {S30, 0, &V17245_312_57255, &P17245_310};	/*Property = History*/
const Value V17245_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17245_314 = {S31, 0, &V17245_314_57261, &P17245_312};	/*Property = Html5*/
const Value V17245_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17245_313 = {S32, 0, &V17245_313_57258, &P17245_314};	/*Property = Html-Media-Capture*/
const Value V17245_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17245_315 = {S33, 0, &V17245_315_57264, &P17245_313};	/*Property = Iframe*/
const Value V17245_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17245_316 = {S34, 0, &V17245_316_57267, &P17245_315};	/*Property = IndexedDB*/
const Value V17245_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17245_317 = {S35, 0, &V17245_317_57269, &P17245_316};	/*Property = Json*/
const Value V17245_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17245_15 = {S37, 0, &V17245_15_28, &P17245_317};	/*Property = LayoutEngine*/
const Value V17245_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17245_344 = {S38, 0, &V17245_344_57272, &P17245_15};	/*Property = Masking*/
const Value V17245_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17245_320 = {S39, 0, &V17245_320_57290, &P17245_344};	/*Property = PostMessage*/
const Value V17245_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17245_321 = {S40, 0, &V17245_321_57293, &P17245_320};	/*Property = Progress*/
const Value V17245_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17245_322 = {S41, 0, &V17245_322_57296, &P17245_321};	/*Property = Prompts*/
const Value V17245_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17245_323 = {S42, 0, &V17245_323_57299, &P17245_322};	/*Property = Selector*/
const Value V17245_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17245_324 = {S43, 0, &V17245_324_57306, &P17245_323};	/*Property = Svg*/
const Value V17245_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17245_327 = {S44, 0, &V17245_327_57315, &P17245_324};	/*Property = TouchEvents*/
const Value V17245_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17245_328 = {S45, 0, &V17245_328_57318, &P17245_327};	/*Property = Track*/
const Value V17245_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17245_329 = {S46, 0, &V17245_329_57323, &P17245_328};	/*Property = Video*/
const Value V17245_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17245_330 = {S47, 0, &V17245_330_57326, &P17245_329};	/*Property = Viewport*/
const Value V17245_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17245_332 = {S48, 0, &V17245_332_57339, &P17245_330};	/*Property = WebWorkers*/
const Value V17245_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17245_333 = {S49, 0, &V17245_333_57342, &P17245_332};	/*Property = Xhr2*/
const Profile P17245 = {"17245", &P17245_333};	/*Profile ID - 17245*/
const Value V17190_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17190_279 = {S1, 0, &V17190_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17190_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17190_282 = {S3, 0, &V17190_282_57154, &P17190_279};	/*Property = BlobBuilder*/
const Value V17190_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17190_350 = {S4, 0, &V17190_350_57444, &P17190_282};	/*Property = Canvas*/
const Value V17190_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17190_283 = {S5, 0, &V17190_283_57159, &P17190_350};	/*Property = CssBackground*/
const Value V17190_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17190_284 = {S6, 0, &V17190_284_57163, &P17190_283};	/*Property = CssBorderImage*/
const Value V17190_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17190_285 = {S7, 0, &V17190_285_57166, &P17190_284};	/*Property = CssCanvas*/
const Value V17190_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17190_286 = {S8, 0, &V17190_286_57168, &P17190_285};	/*Property = CssColor*/
const Value V17190_237_55785 = {S0, NULL};	/*Value = False*/
const Property P17190_237 = {S9, 0, &V17190_237_55785, &P17190_286};	/*Property = CssColumn*/
const Value V17190_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17190_287 = {S10, 0, &V17190_287_57172, &P17190_237};	/*Property = CssFlexbox*/
const Value V17190_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17190_288 = {S11, 0, &V17190_288_57175, &P17190_287};	/*Property = CssFont*/
const Value V17190_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17190_289 = {S12, 0, &V17190_289_57759, &P17190_288};	/*Property = CssImages*/
const Value V17190_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17190_291 = {S13, 0, &V17190_291_57181, &P17190_289};	/*Property = CssMediaQueries*/
const Value V17190_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17190_292 = {S14, 0, &V17190_292_57183, &P17190_291};	/*Property = CssMinMax*/
const Value V17190_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17190_293 = {S15, 0, &V17190_293_57187, &P17190_292};	/*Property = CssOverflow*/
const Value V17190_294_57190 = {S0, NULL};	/*Value = False*/
const Property P17190_294 = {S16, 0, &V17190_294_57190, &P17190_293};	/*Property = CssPosition*/
const Value V17190_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17190_295 = {S17, 0, &V17190_295_57192, &P17190_294};	/*Property = CssText*/
const Value V17190_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17190_297 = {S18, 0, &V17190_297_57195, &P17190_295};	/*Property = CssTransforms*/
const Value V17190_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17190_241 = {S19, 0, &V17190_241_55799, &P17190_297};	/*Property = CssTransitions*/
const Value V17190_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17190_298 = {S20, 0, &V17190_298_57206, &P17190_241};	/*Property = CssUI*/
const Value V17190_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17190_300 = {S21, 0, &V17190_300_57216, &P17190_298};	/*Property = DataSet*/
const Value V17190_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17190_301 = {S22, 0, &V17190_301_57218, &P17190_300};	/*Property = DataUrl*/
const Value V17190_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17190_302 = {S23, 0, &V17190_302_57222, &P17190_301};	/*Property = DeviceOrientation*/
const Value V17190_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17190_304 = {S24, 0, &V17190_304_57228, &P17190_302};	/*Property = FileReader*/
const Value V17190_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17190_305 = {S25, 0, &V17190_305_57231, &P17190_304};	/*Property = FileSaver*/
const Value V17190_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17190_306 = {S26, 0, &V17190_306_57237, &P17190_305};	/*Property = FileWriter*/
const Value V17190_307_57240 = {S0, NULL};	/*Value = False*/
const Property P17190_307 = {S27, 0, &V17190_307_57240, &P17190_306};	/*Property = FormData*/
const Value V17190_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17190_309 = {S28, 0, &V17190_309_57246, &P17190_307};	/*Property = Fullscreen*/
const Value V17190_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17190_310 = {S29, 0, &V17190_310_57248, &P17190_309};	/*Property = GeoLocation*/
const Value V17190_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17190_312 = {S30, 0, &V17190_312_57255, &P17190_310};	/*Property = History*/
const Value V17190_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17190_314 = {S31, 0, &V17190_314_57261, &P17190_312};	/*Property = Html5*/
const Value V17190_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17190_313 = {S32, 0, &V17190_313_57258, &P17190_314};	/*Property = Html-Media-Capture*/
const Value V17190_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17190_315 = {S33, 0, &V17190_315_57264, &P17190_313};	/*Property = Iframe*/
const Value V17190_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17190_316 = {S34, 0, &V17190_316_57267, &P17190_315};	/*Property = IndexedDB*/
const Value V17190_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17190_317 = {S35, 0, &V17190_317_57269, &P17190_316};	/*Property = Json*/
const Value V17190_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P17190_15 = {S37, 0, &V17190_15_30, &P17190_317};	/*Property = LayoutEngine*/
const Value V17190_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17190_344 = {S38, 0, &V17190_344_57272, &P17190_15};	/*Property = Masking*/
const Value V17190_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17190_320 = {S39, 0, &V17190_320_57290, &P17190_344};	/*Property = PostMessage*/
const Value V17190_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17190_321 = {S40, 0, &V17190_321_57293, &P17190_320};	/*Property = Progress*/
const Value V17190_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17190_322 = {S41, 0, &V17190_322_57296, &P17190_321};	/*Property = Prompts*/
const Value V17190_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17190_323 = {S42, 0, &V17190_323_57299, &P17190_322};	/*Property = Selector*/
const Value V17190_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17190_324 = {S43, 0, &V17190_324_57306, &P17190_323};	/*Property = Svg*/
const Value V17190_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17190_327 = {S44, 0, &V17190_327_57315, &P17190_324};	/*Property = TouchEvents*/
const Value V17190_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17190_328 = {S45, 0, &V17190_328_57318, &P17190_327};	/*Property = Track*/
const Value V17190_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17190_329 = {S46, 0, &V17190_329_57323, &P17190_328};	/*Property = Video*/
const Value V17190_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17190_330 = {S47, 0, &V17190_330_57326, &P17190_329};	/*Property = Viewport*/
const Value V17190_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17190_332 = {S48, 0, &V17190_332_57339, &P17190_330};	/*Property = WebWorkers*/
const Value V17190_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17190_333 = {S49, 0, &V17190_333_57342, &P17190_332};	/*Property = Xhr2*/
const Profile P17190 = {"17190", &P17190_333};	/*Profile ID - 17190*/
const Value V3804_279_57148 = {S0, NULL};	/*Value = False*/
const Property P3804_279 = {S1, 0, &V3804_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V3804_282_57154 = {S0, NULL};	/*Value = False*/
const Property P3804_282 = {S3, 0, &V3804_282_57154, &P3804_279};	/*Property = BlobBuilder*/
const Value V3804_350_57444 = {S2, NULL};	/*Value = True*/
const Property P3804_350 = {S4, 0, &V3804_350_57444, &P3804_282};	/*Property = Canvas*/
const Value V3804_283_57159 = {S2, NULL};	/*Value = True*/
const Property P3804_283 = {S5, 0, &V3804_283_57159, &P3804_350};	/*Property = CssBackground*/
const Value V3804_284_57163 = {S0, NULL};	/*Value = False*/
const Property P3804_284 = {S6, 0, &V3804_284_57163, &P3804_283};	/*Property = CssBorderImage*/
const Value V3804_285_57166 = {S0, NULL};	/*Value = False*/
const Property P3804_285 = {S7, 0, &V3804_285_57166, &P3804_284};	/*Property = CssCanvas*/
const Value V3804_286_57168 = {S2, NULL};	/*Value = True*/
const Property P3804_286 = {S8, 0, &V3804_286_57168, &P3804_285};	/*Property = CssColor*/
const Value V3804_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3804_237 = {S9, 0, &V3804_237_55786, &P3804_286};	/*Property = CssColumn*/
const Value V3804_287_57172 = {S0, NULL};	/*Value = False*/
const Property P3804_287 = {S10, 0, &V3804_287_57172, &P3804_237};	/*Property = CssFlexbox*/
const Value V3804_288_57175 = {S0, NULL};	/*Value = False*/
const Property P3804_288 = {S11, 0, &V3804_288_57175, &P3804_287};	/*Property = CssFont*/
const Value V3804_289_57759 = {S0, NULL};	/*Value = False*/
const Property P3804_289 = {S12, 0, &V3804_289_57759, &P3804_288};	/*Property = CssImages*/
const Value V3804_291_57181 = {S0, NULL};	/*Value = False*/
const Property P3804_291 = {S13, 0, &V3804_291_57181, &P3804_289};	/*Property = CssMediaQueries*/
const Value V3804_292_57183 = {S2, NULL};	/*Value = True*/
const Property P3804_292 = {S14, 0, &V3804_292_57183, &P3804_291};	/*Property = CssMinMax*/
const Value V3804_293_57187 = {S0, NULL};	/*Value = False*/
const Property P3804_293 = {S15, 0, &V3804_293_57187, &P3804_292};	/*Property = CssOverflow*/
const Value V3804_294_57190 = {S0, NULL};	/*Value = False*/
const Property P3804_294 = {S16, 0, &V3804_294_57190, &P3804_293};	/*Property = CssPosition*/
const Value V3804_295_57192 = {S2, NULL};	/*Value = True*/
const Property P3804_295 = {S17, 0, &V3804_295_57192, &P3804_294};	/*Property = CssText*/
const Value V3804_297_57195 = {S2, NULL};	/*Value = True*/
const Property P3804_297 = {S18, 0, &V3804_297_57195, &P3804_295};	/*Property = CssTransforms*/
const Value V3804_241_55799 = {S2, NULL};	/*Value = True*/
const Property P3804_241 = {S19, 0, &V3804_241_55799, &P3804_297};	/*Property = CssTransitions*/
const Value V3804_298_57206 = {S2, NULL};	/*Value = True*/
const Property P3804_298 = {S20, 0, &V3804_298_57206, &P3804_241};	/*Property = CssUI*/
const Value V3804_300_57216 = {S0, NULL};	/*Value = False*/
const Property P3804_300 = {S21, 0, &V3804_300_57216, &P3804_298};	/*Property = DataSet*/
const Value V3804_301_57218 = {S2, NULL};	/*Value = True*/
const Property P3804_301 = {S22, 0, &V3804_301_57218, &P3804_300};	/*Property = DataUrl*/
const Value V3804_302_57222 = {S0, NULL};	/*Value = False*/
const Property P3804_302 = {S23, 0, &V3804_302_57222, &P3804_301};	/*Property = DeviceOrientation*/
const Value V3804_304_57228 = {S0, NULL};	/*Value = False*/
const Property P3804_304 = {S24, 0, &V3804_304_57228, &P3804_302};	/*Property = FileReader*/
const Value V3804_305_57231 = {S0, NULL};	/*Value = False*/
const Property P3804_305 = {S25, 0, &V3804_305_57231, &P3804_304};	/*Property = FileSaver*/
const Value V3804_306_57237 = {S0, NULL};	/*Value = False*/
const Property P3804_306 = {S26, 0, &V3804_306_57237, &P3804_305};	/*Property = FileWriter*/
const Value V3804_307_57240 = {S0, NULL};	/*Value = False*/
const Property P3804_307 = {S27, 0, &V3804_307_57240, &P3804_306};	/*Property = FormData*/
const Value V3804_309_57246 = {S0, NULL};	/*Value = False*/
const Property P3804_309 = {S28, 0, &V3804_309_57246, &P3804_307};	/*Property = Fullscreen*/
const Value V3804_310_57248 = {S2, NULL};	/*Value = True*/
const Property P3804_310 = {S29, 0, &V3804_310_57248, &P3804_309};	/*Property = GeoLocation*/
const Value V3804_312_57255 = {S0, NULL};	/*Value = False*/
const Property P3804_312 = {S30, 0, &V3804_312_57255, &P3804_310};	/*Property = History*/
const Value V3804_314_57261 = {S0, NULL};	/*Value = False*/
const Property P3804_314 = {S31, 0, &V3804_314_57261, &P3804_312};	/*Property = Html5*/
const Value V3804_313_57258 = {S0, NULL};	/*Value = False*/
const Property P3804_313 = {S32, 0, &V3804_313_57258, &P3804_314};	/*Property = Html-Media-Capture*/
const Value V3804_315_57264 = {S0, NULL};	/*Value = False*/
const Property P3804_315 = {S33, 0, &V3804_315_57264, &P3804_313};	/*Property = Iframe*/
const Value V3804_316_57267 = {S0, NULL};	/*Value = False*/
const Property P3804_316 = {S34, 0, &V3804_316_57267, &P3804_315};	/*Property = IndexedDB*/
const Value V3804_317_57269 = {S2, NULL};	/*Value = True*/
const Property P3804_317 = {S35, 0, &V3804_317_57269, &P3804_316};	/*Property = Json*/
const Value V3804_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3804_15 = {S37, 0, &V3804_15_28, &P3804_317};	/*Property = LayoutEngine*/
const Value V3804_344_57272 = {S2, NULL};	/*Value = True*/
const Property P3804_344 = {S38, 0, &V3804_344_57272, &P3804_15};	/*Property = Masking*/
const Value V3804_320_57290 = {S2, NULL};	/*Value = True*/
const Property P3804_320 = {S39, 0, &V3804_320_57290, &P3804_344};	/*Property = PostMessage*/
const Value V3804_321_57293 = {S2, NULL};	/*Value = True*/
const Property P3804_321 = {S40, 0, &V3804_321_57293, &P3804_320};	/*Property = Progress*/
const Value V3804_322_57296 = {S2, NULL};	/*Value = True*/
const Property P3804_322 = {S41, 0, &V3804_322_57296, &P3804_321};	/*Property = Prompts*/
const Value V3804_323_57299 = {S2, NULL};	/*Value = True*/
const Property P3804_323 = {S42, 0, &V3804_323_57299, &P3804_322};	/*Property = Selector*/
const Value V3804_324_57306 = {S0, NULL};	/*Value = False*/
const Property P3804_324 = {S43, 0, &V3804_324_57306, &P3804_323};	/*Property = Svg*/
const Value V3804_327_57315 = {S0, NULL};	/*Value = False*/
const Property P3804_327 = {S44, 0, &V3804_327_57315, &P3804_324};	/*Property = TouchEvents*/
const Value V3804_328_57318 = {S0, NULL};	/*Value = False*/
const Property P3804_328 = {S45, 0, &V3804_328_57318, &P3804_327};	/*Property = Track*/
const Value V3804_329_57323 = {S2, NULL};	/*Value = True*/
const Property P3804_329 = {S46, 0, &V3804_329_57323, &P3804_328};	/*Property = Video*/
const Value V3804_330_57326 = {S2, NULL};	/*Value = True*/
const Property P3804_330 = {S47, 0, &V3804_330_57326, &P3804_329};	/*Property = Viewport*/
const Value V3804_332_57339 = {S0, NULL};	/*Value = False*/
const Property P3804_332 = {S48, 0, &V3804_332_57339, &P3804_330};	/*Property = WebWorkers*/
const Value V3804_333_57342 = {S0, NULL};	/*Value = False*/
const Property P3804_333 = {S49, 0, &V3804_333_57342, &P3804_332};	/*Property = Xhr2*/
const Profile P3804 = {"3804", &P3804_333};	/*Profile ID - 3804*/
const Profile P5055 = {"5055", NULL};	/*Profile ID - 5055*/
const Value V13324_1_1 = {S2, NULL};	/*Value = True*/
const Property P13324_1 = {S50, 0, &V13324_1_1, NULL};	/*Property = IsMobile*/
const Value V13324_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P13324_5 = {S52, 0, &V13324_5_1275, &P13324_1};	/*Property = ScreenPixelsHeight*/
const Value V13324_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P13324_4 = {S54, 0, &V13324_4_1278, &P13324_5};	/*Property = ScreenPixelsWidth*/
const Profile P13324 = {"13324", &P13324_4};	/*Profile ID - 13324*/
const Value V13428_1_1 = {S2, NULL};	/*Value = True*/
const Property P13428_1 = {S50, 0, &V13428_1_1, NULL};	/*Property = IsMobile*/
const Value V13428_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13428_5 = {S52, 0, &V13428_5_1233, &P13428_1};	/*Property = ScreenPixelsHeight*/
const Value V13428_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13428_4 = {S54, 0, &V13428_4_1235, &P13428_5};	/*Property = ScreenPixelsWidth*/
const Profile P13428 = {"13428", &P13428_4};	/*Profile ID - 13428*/
const Value V11754_1_1 = {S2, NULL};	/*Value = True*/
const Property P11754_1 = {S50, 0, &V11754_1_1, NULL};	/*Property = IsMobile*/
const Value V11754_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11754_5 = {S52, 0, &V11754_5_1222, &P11754_1};	/*Property = ScreenPixelsHeight*/
const Value V11754_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11754_4 = {S54, 0, &V11754_4_1224, &P11754_5};	/*Property = ScreenPixelsWidth*/
const Profile P11754 = {"11754", &P11754_4};	/*Profile ID - 11754*/
const Value V12360_1_1 = {S2, NULL};	/*Value = True*/
const Property P12360_1 = {S50, 0, &V12360_1_1, NULL};	/*Property = IsMobile*/
const Value V12360_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12360_5 = {S52, 0, &V12360_5_1222, &P12360_1};	/*Property = ScreenPixelsHeight*/
const Value V12360_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12360_4 = {S54, 0, &V12360_4_1224, &P12360_5};	/*Property = ScreenPixelsWidth*/
const Profile P12360 = {"12360", &P12360_4};	/*Profile ID - 12360*/
const Value V12530_1_1 = {S2, NULL};	/*Value = True*/
const Property P12530_1 = {S50, 0, &V12530_1_1, NULL};	/*Property = IsMobile*/
const Value V12530_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12530_5 = {S52, 0, &V12530_5_1222, &P12530_1};	/*Property = ScreenPixelsHeight*/
const Value V12530_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12530_4 = {S54, 0, &V12530_4_1224, &P12530_5};	/*Property = ScreenPixelsWidth*/
const Profile P12530 = {"12530", &P12530_4};	/*Profile ID - 12530*/
const Value V13329_1_1 = {S2, NULL};	/*Value = True*/
const Property P13329_1 = {S50, 0, &V13329_1_1, NULL};	/*Property = IsMobile*/
const Value V13329_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13329_5 = {S52, 0, &V13329_5_1222, &P13329_1};	/*Property = ScreenPixelsHeight*/
const Value V13329_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13329_4 = {S54, 0, &V13329_4_1224, &P13329_5};	/*Property = ScreenPixelsWidth*/
const Profile P13329 = {"13329", &P13329_4};	/*Profile ID - 13329*/
const Value V17282_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_279 = {S1, 0, &V17282_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17282_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_282 = {S3, 0, &V17282_282_57155, &P17282_279};	/*Property = BlobBuilder*/
const Value V17282_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_350 = {S4, 0, &V17282_350_57446, &P17282_282};	/*Property = Canvas*/
const Value V17282_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_283 = {S5, 0, &V17282_283_57161, &P17282_350};	/*Property = CssBackground*/
const Value V17282_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_284 = {S6, 0, &V17282_284_57164, &P17282_283};	/*Property = CssBorderImage*/
const Value V17282_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_285 = {S7, 0, &V17282_285_57167, &P17282_284};	/*Property = CssCanvas*/
const Value V17282_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_286 = {S8, 0, &V17282_286_57170, &P17282_285};	/*Property = CssColor*/
const Value V17282_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_237 = {S9, 0, &V17282_237_55786, &P17282_286};	/*Property = CssColumn*/
const Value V17282_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_287 = {S10, 0, &V17282_287_57173, &P17282_237};	/*Property = CssFlexbox*/
const Value V17282_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_288 = {S11, 0, &V17282_288_57176, &P17282_287};	/*Property = CssFont*/
const Value V17282_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_289 = {S12, 0, &V17282_289_86899, &P17282_288};	/*Property = CssImages*/
const Value V17282_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_291 = {S13, 0, &V17282_291_57182, &P17282_289};	/*Property = CssMediaQueries*/
const Value V17282_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_292 = {S14, 0, &V17282_292_57185, &P17282_291};	/*Property = CssMinMax*/
const Value V17282_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_293 = {S15, 0, &V17282_293_57188, &P17282_292};	/*Property = CssOverflow*/
const Value V17282_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_294 = {S16, 0, &V17282_294_57191, &P17282_293};	/*Property = CssPosition*/
const Value V17282_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_295 = {S17, 0, &V17282_295_57194, &P17282_294};	/*Property = CssText*/
const Value V17282_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_297 = {S18, 0, &V17282_297_57197, &P17282_295};	/*Property = CssTransforms*/
const Value V17282_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_241 = {S19, 0, &V17282_241_55801, &P17282_297};	/*Property = CssTransitions*/
const Value V17282_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_298 = {S20, 0, &V17282_298_57208, &P17282_241};	/*Property = CssUI*/
const Value V17282_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_300 = {S21, 0, &V17282_300_57217, &P17282_298};	/*Property = DataSet*/
const Value V17282_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_301 = {S22, 0, &V17282_301_57220, &P17282_300};	/*Property = DataUrl*/
const Value V17282_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_302 = {S23, 0, &V17282_302_57223, &P17282_301};	/*Property = DeviceOrientation*/
const Value V17282_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_304 = {S24, 0, &V17282_304_57229, &P17282_302};	/*Property = FileReader*/
const Value V17282_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_305 = {S25, 0, &V17282_305_57232, &P17282_304};	/*Property = FileSaver*/
const Value V17282_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_306 = {S26, 0, &V17282_306_57238, &P17282_305};	/*Property = FileWriter*/
const Value V17282_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_307 = {S27, 0, &V17282_307_57241, &P17282_306};	/*Property = FormData*/
const Value V17282_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_309 = {S28, 0, &V17282_309_57247, &P17282_307};	/*Property = Fullscreen*/
const Value V17282_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_310 = {S29, 0, &V17282_310_57250, &P17282_309};	/*Property = GeoLocation*/
const Value V17282_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_312 = {S30, 0, &V17282_312_57256, &P17282_310};	/*Property = History*/
const Value V17282_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_314 = {S31, 0, &V17282_314_57262, &P17282_312};	/*Property = Html5*/
const Value V17282_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_313 = {S32, 0, &V17282_313_57259, &P17282_314};	/*Property = Html-Media-Capture*/
const Value V17282_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_315 = {S33, 0, &V17282_315_57265, &P17282_313};	/*Property = Iframe*/
const Value V17282_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_316 = {S34, 0, &V17282_316_57268, &P17282_315};	/*Property = IndexedDB*/
const Value V17282_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_317 = {S35, 0, &V17282_317_57271, &P17282_316};	/*Property = Json*/
const Value V17282_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P17282_15 = {S37, 0, &V17282_15_29, &P17282_317};	/*Property = LayoutEngine*/
const Value V17282_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_344 = {S38, 0, &V17282_344_57274, &P17282_15};	/*Property = Masking*/
const Value V17282_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_320 = {S39, 0, &V17282_320_57292, &P17282_344};	/*Property = PostMessage*/
const Value V17282_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_321 = {S40, 0, &V17282_321_57295, &P17282_320};	/*Property = Progress*/
const Value V17282_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_322 = {S41, 0, &V17282_322_57298, &P17282_321};	/*Property = Prompts*/
const Value V17282_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_323 = {S42, 0, &V17282_323_57301, &P17282_322};	/*Property = Selector*/
const Value V17282_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_324 = {S43, 0, &V17282_324_57307, &P17282_323};	/*Property = Svg*/
const Value V17282_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_327 = {S44, 0, &V17282_327_57316, &P17282_324};	/*Property = TouchEvents*/
const Value V17282_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_328 = {S45, 0, &V17282_328_57319, &P17282_327};	/*Property = Track*/
const Value V17282_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_329 = {S46, 0, &V17282_329_57325, &P17282_328};	/*Property = Video*/
const Value V17282_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_330 = {S47, 0, &V17282_330_57328, &P17282_329};	/*Property = Viewport*/
const Value V17282_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_332 = {S48, 0, &V17282_332_57340, &P17282_330};	/*Property = WebWorkers*/
const Value V17282_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17282_333 = {S49, 0, &V17282_333_57343, &P17282_332};	/*Property = Xhr2*/
const Profile P17282 = {"17282", &P17282_333};	/*Profile ID - 17282*/
const Value V13331_1_1 = {S2, NULL};	/*Value = True*/
const Property P13331_1 = {S50, 0, &V13331_1_1, NULL};	/*Property = IsMobile*/
const Value V13331_5_39944 = {S81, NULL};	/*Value = 275*/
const Property P13331_5 = {S52, 0, &V13331_5_39944, &P13331_1};	/*Property = ScreenPixelsHeight*/
const Value V13331_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13331_4 = {S54, 0, &V13331_4_1224, &P13331_5};	/*Property = ScreenPixelsWidth*/
const Profile P13331 = {"13331", &P13331_4};	/*Profile ID - 13331*/
const Value V13334_1_1 = {S2, NULL};	/*Value = True*/
const Property P13334_1 = {S50, 0, &V13334_1_1, NULL};	/*Property = IsMobile*/
const Value V13334_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13334_5 = {S52, 0, &V13334_5_1227, &P13334_1};	/*Property = ScreenPixelsHeight*/
const Value V13334_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13334_4 = {S54, 0, &V13334_4_1229, &P13334_5};	/*Property = ScreenPixelsWidth*/
const Profile P13334 = {"13334", &P13334_4};	/*Profile ID - 13334*/
const Value V14675_1_1 = {S2, NULL};	/*Value = True*/
const Property P14675_1 = {S50, 0, &V14675_1_1, NULL};	/*Property = IsMobile*/
const Value V14675_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14675_5 = {S52, 0, &V14675_5_1222, &P14675_1};	/*Property = ScreenPixelsHeight*/
const Value V14675_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14675_4 = {S54, 0, &V14675_4_1224, &P14675_5};	/*Property = ScreenPixelsWidth*/
const Profile P14675 = {"14675", &P14675_4};	/*Profile ID - 14675*/
const Value V14546_1_1 = {S2, NULL};	/*Value = True*/
const Property P14546_1 = {S50, 0, &V14546_1_1, NULL};	/*Property = IsMobile*/
const Value V14546_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14546_5 = {S52, 0, &V14546_5_1222, &P14546_1};	/*Property = ScreenPixelsHeight*/
const Value V14546_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14546_4 = {S54, 0, &V14546_4_1224, &P14546_5};	/*Property = ScreenPixelsWidth*/
const Profile P14546 = {"14546", &P14546_4};	/*Profile ID - 14546*/
const Value V14916_1_1 = {S2, NULL};	/*Value = True*/
const Property P14916_1 = {S50, 0, &V14916_1_1, NULL};	/*Property = IsMobile*/
const Value V14916_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14916_5 = {S52, 0, &V14916_5_1222, &P14916_1};	/*Property = ScreenPixelsHeight*/
const Value V14916_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14916_4 = {S54, 0, &V14916_4_1224, &P14916_5};	/*Property = ScreenPixelsWidth*/
const Profile P14916 = {"14916", &P14916_4};	/*Profile ID - 14916*/
const Value V17606_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_279 = {S1, 0, &V17606_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17606_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_282 = {S3, 0, &V17606_282_57155, &P17606_279};	/*Property = BlobBuilder*/
const Value V17606_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_350 = {S4, 0, &V17606_350_57446, &P17606_282};	/*Property = Canvas*/
const Value V17606_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_283 = {S5, 0, &V17606_283_57161, &P17606_350};	/*Property = CssBackground*/
const Value V17606_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_284 = {S6, 0, &V17606_284_57164, &P17606_283};	/*Property = CssBorderImage*/
const Value V17606_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_285 = {S7, 0, &V17606_285_57167, &P17606_284};	/*Property = CssCanvas*/
const Value V17606_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_286 = {S8, 0, &V17606_286_57170, &P17606_285};	/*Property = CssColor*/
const Value V17606_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_237 = {S9, 0, &V17606_237_55786, &P17606_286};	/*Property = CssColumn*/
const Value V17606_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_287 = {S10, 0, &V17606_287_57173, &P17606_237};	/*Property = CssFlexbox*/
const Value V17606_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_288 = {S11, 0, &V17606_288_57176, &P17606_287};	/*Property = CssFont*/
const Value V17606_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_289 = {S12, 0, &V17606_289_86899, &P17606_288};	/*Property = CssImages*/
const Value V17606_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_291 = {S13, 0, &V17606_291_57182, &P17606_289};	/*Property = CssMediaQueries*/
const Value V17606_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_292 = {S14, 0, &V17606_292_57185, &P17606_291};	/*Property = CssMinMax*/
const Value V17606_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_293 = {S15, 0, &V17606_293_57188, &P17606_292};	/*Property = CssOverflow*/
const Value V17606_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_294 = {S16, 0, &V17606_294_57191, &P17606_293};	/*Property = CssPosition*/
const Value V17606_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_295 = {S17, 0, &V17606_295_57194, &P17606_294};	/*Property = CssText*/
const Value V17606_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_297 = {S18, 0, &V17606_297_57197, &P17606_295};	/*Property = CssTransforms*/
const Value V17606_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_241 = {S19, 0, &V17606_241_55801, &P17606_297};	/*Property = CssTransitions*/
const Value V17606_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_298 = {S20, 0, &V17606_298_57208, &P17606_241};	/*Property = CssUI*/
const Value V17606_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_300 = {S21, 0, &V17606_300_57217, &P17606_298};	/*Property = DataSet*/
const Value V17606_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_301 = {S22, 0, &V17606_301_57220, &P17606_300};	/*Property = DataUrl*/
const Value V17606_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_302 = {S23, 0, &V17606_302_57223, &P17606_301};	/*Property = DeviceOrientation*/
const Value V17606_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_304 = {S24, 0, &V17606_304_57229, &P17606_302};	/*Property = FileReader*/
const Value V17606_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_305 = {S25, 0, &V17606_305_57232, &P17606_304};	/*Property = FileSaver*/
const Value V17606_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_306 = {S26, 0, &V17606_306_57238, &P17606_305};	/*Property = FileWriter*/
const Value V17606_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_307 = {S27, 0, &V17606_307_57241, &P17606_306};	/*Property = FormData*/
const Value V17606_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_309 = {S28, 0, &V17606_309_57247, &P17606_307};	/*Property = Fullscreen*/
const Value V17606_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_310 = {S29, 0, &V17606_310_57250, &P17606_309};	/*Property = GeoLocation*/
const Value V17606_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_312 = {S30, 0, &V17606_312_57256, &P17606_310};	/*Property = History*/
const Value V17606_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_314 = {S31, 0, &V17606_314_57262, &P17606_312};	/*Property = Html5*/
const Value V17606_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_313 = {S32, 0, &V17606_313_57259, &P17606_314};	/*Property = Html-Media-Capture*/
const Value V17606_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_315 = {S33, 0, &V17606_315_57265, &P17606_313};	/*Property = Iframe*/
const Value V17606_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_316 = {S34, 0, &V17606_316_57268, &P17606_315};	/*Property = IndexedDB*/
const Value V17606_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_317 = {S35, 0, &V17606_317_57271, &P17606_316};	/*Property = Json*/
const Value V17606_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_15 = {S37, 0, &V17606_15_72, &P17606_317};	/*Property = LayoutEngine*/
const Value V17606_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_344 = {S38, 0, &V17606_344_57274, &P17606_15};	/*Property = Masking*/
const Value V17606_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_320 = {S39, 0, &V17606_320_57292, &P17606_344};	/*Property = PostMessage*/
const Value V17606_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_321 = {S40, 0, &V17606_321_57295, &P17606_320};	/*Property = Progress*/
const Value V17606_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_322 = {S41, 0, &V17606_322_57298, &P17606_321};	/*Property = Prompts*/
const Value V17606_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_323 = {S42, 0, &V17606_323_57301, &P17606_322};	/*Property = Selector*/
const Value V17606_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_324 = {S43, 0, &V17606_324_57307, &P17606_323};	/*Property = Svg*/
const Value V17606_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_327 = {S44, 0, &V17606_327_57316, &P17606_324};	/*Property = TouchEvents*/
const Value V17606_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_328 = {S45, 0, &V17606_328_57319, &P17606_327};	/*Property = Track*/
const Value V17606_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_329 = {S46, 0, &V17606_329_57325, &P17606_328};	/*Property = Video*/
const Value V17606_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_330 = {S47, 0, &V17606_330_57328, &P17606_329};	/*Property = Viewport*/
const Value V17606_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_332 = {S48, 0, &V17606_332_57340, &P17606_330};	/*Property = WebWorkers*/
const Value V17606_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17606_333 = {S49, 0, &V17606_333_57343, &P17606_332};	/*Property = Xhr2*/
const Profile P17606 = {"17606", &P17606_333};	/*Profile ID - 17606*/
const Value V13337_1_1 = {S2, NULL};	/*Value = True*/
const Property P13337_1 = {S50, 0, &V13337_1_1, NULL};	/*Property = IsMobile*/
const Value V13337_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13337_5 = {S52, 0, &V13337_5_1222, &P13337_1};	/*Property = ScreenPixelsHeight*/
const Value V13337_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13337_4 = {S54, 0, &V13337_4_1224, &P13337_5};	/*Property = ScreenPixelsWidth*/
const Profile P13337 = {"13337", &P13337_4};	/*Profile ID - 13337*/
const Value V15545_1_1 = {S2, NULL};	/*Value = True*/
const Property P15545_1 = {S50, 0, &V15545_1_1, NULL};	/*Property = IsMobile*/
const Value V15545_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15545_5 = {S52, 0, &V15545_5_1222, &P15545_1};	/*Property = ScreenPixelsHeight*/
const Value V15545_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15545_4 = {S54, 0, &V15545_4_1279, &P15545_5};	/*Property = ScreenPixelsWidth*/
const Profile P15545 = {"15545", &P15545_4};	/*Profile ID - 15545*/
const Value V13340_1_1 = {S2, NULL};	/*Value = True*/
const Property P13340_1 = {S50, 0, &V13340_1_1, NULL};	/*Property = IsMobile*/
const Value V13340_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13340_5 = {S52, 0, &V13340_5_1227, &P13340_1};	/*Property = ScreenPixelsHeight*/
const Value V13340_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13340_4 = {S54, 0, &V13340_4_1229, &P13340_5};	/*Property = ScreenPixelsWidth*/
const Profile P13340 = {"13340", &P13340_4};	/*Profile ID - 13340*/
const Value V13341_1_1 = {S2, NULL};	/*Value = True*/
const Property P13341_1 = {S50, 0, &V13341_1_1, NULL};	/*Property = IsMobile*/
const Value V13341_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13341_5 = {S52, 0, &V13341_5_1304, &P13341_1};	/*Property = ScreenPixelsHeight*/
const Value V13341_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P13341_4 = {S54, 0, &V13341_4_1278, &P13341_5};	/*Property = ScreenPixelsWidth*/
const Profile P13341 = {"13341", &P13341_4};	/*Profile ID - 13341*/
const Value V15526_1_1 = {S2, NULL};	/*Value = True*/
const Property P15526_1 = {S50, 0, &V15526_1_1, NULL};	/*Property = IsMobile*/
const Value V15526_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15526_5 = {S52, 0, &V15526_5_1277, &P15526_1};	/*Property = ScreenPixelsHeight*/
const Value V15526_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15526_4 = {S54, 0, &V15526_4_1279, &P15526_5};	/*Property = ScreenPixelsWidth*/
const Profile P15526 = {"15526", &P15526_4};	/*Profile ID - 15526*/
const Value V12220_1_1 = {S2, NULL};	/*Value = True*/
const Property P12220_1 = {S50, 0, &V12220_1_1, NULL};	/*Property = IsMobile*/
const Value V12220_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12220_5 = {S52, 0, &V12220_5_1380, &P12220_1};	/*Property = ScreenPixelsHeight*/
const Value V12220_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12220_4 = {S54, 0, &V12220_4_1279, &P12220_5};	/*Property = ScreenPixelsWidth*/
const Profile P12220 = {"12220", &P12220_4};	/*Profile ID - 12220*/
const Value V13338_1_1 = {S2, NULL};	/*Value = True*/
const Property P13338_1 = {S50, 0, &V13338_1_1, NULL};	/*Property = IsMobile*/
const Value V13338_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13338_5 = {S52, 0, &V13338_5_1227, &P13338_1};	/*Property = ScreenPixelsHeight*/
const Value V13338_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13338_4 = {S54, 0, &V13338_4_1229, &P13338_5};	/*Property = ScreenPixelsWidth*/
const Profile P13338 = {"13338", &P13338_4};	/*Profile ID - 13338*/
const Value V13343_1_1 = {S2, NULL};	/*Value = True*/
const Property P13343_1 = {S50, 0, &V13343_1_1, NULL};	/*Property = IsMobile*/
const Value V13343_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13343_5 = {S52, 0, &V13343_5_1233, &P13343_1};	/*Property = ScreenPixelsHeight*/
const Value V13343_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13343_4 = {S54, 0, &V13343_4_1235, &P13343_5};	/*Property = ScreenPixelsWidth*/
const Profile P13343 = {"13343", &P13343_4};	/*Profile ID - 13343*/
const Value V3499_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_279 = {S1, 0, &V3499_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3499_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_282 = {S3, 0, &V3499_282_57155, &P3499_279};	/*Property = BlobBuilder*/
const Value V3499_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_350 = {S4, 0, &V3499_350_57446, &P3499_282};	/*Property = Canvas*/
const Value V3499_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_283 = {S5, 0, &V3499_283_57161, &P3499_350};	/*Property = CssBackground*/
const Value V3499_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_284 = {S6, 0, &V3499_284_57164, &P3499_283};	/*Property = CssBorderImage*/
const Value V3499_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_285 = {S7, 0, &V3499_285_57167, &P3499_284};	/*Property = CssCanvas*/
const Value V3499_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_286 = {S8, 0, &V3499_286_57170, &P3499_285};	/*Property = CssColor*/
const Value V3499_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_237 = {S9, 0, &V3499_237_55786, &P3499_286};	/*Property = CssColumn*/
const Value V3499_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_287 = {S10, 0, &V3499_287_57173, &P3499_237};	/*Property = CssFlexbox*/
const Value V3499_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_288 = {S11, 0, &V3499_288_57176, &P3499_287};	/*Property = CssFont*/
const Value V3499_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_289 = {S12, 0, &V3499_289_86899, &P3499_288};	/*Property = CssImages*/
const Value V3499_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_291 = {S13, 0, &V3499_291_57182, &P3499_289};	/*Property = CssMediaQueries*/
const Value V3499_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_292 = {S14, 0, &V3499_292_57185, &P3499_291};	/*Property = CssMinMax*/
const Value V3499_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_293 = {S15, 0, &V3499_293_57188, &P3499_292};	/*Property = CssOverflow*/
const Value V3499_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_294 = {S16, 0, &V3499_294_57191, &P3499_293};	/*Property = CssPosition*/
const Value V3499_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_295 = {S17, 0, &V3499_295_57194, &P3499_294};	/*Property = CssText*/
const Value V3499_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_297 = {S18, 0, &V3499_297_57197, &P3499_295};	/*Property = CssTransforms*/
const Value V3499_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_241 = {S19, 0, &V3499_241_55801, &P3499_297};	/*Property = CssTransitions*/
const Value V3499_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_298 = {S20, 0, &V3499_298_57208, &P3499_241};	/*Property = CssUI*/
const Value V3499_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_300 = {S21, 0, &V3499_300_57217, &P3499_298};	/*Property = DataSet*/
const Value V3499_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_301 = {S22, 0, &V3499_301_57220, &P3499_300};	/*Property = DataUrl*/
const Value V3499_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_302 = {S23, 0, &V3499_302_57223, &P3499_301};	/*Property = DeviceOrientation*/
const Value V3499_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_304 = {S24, 0, &V3499_304_57229, &P3499_302};	/*Property = FileReader*/
const Value V3499_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_305 = {S25, 0, &V3499_305_57232, &P3499_304};	/*Property = FileSaver*/
const Value V3499_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_306 = {S26, 0, &V3499_306_57238, &P3499_305};	/*Property = FileWriter*/
const Value V3499_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_307 = {S27, 0, &V3499_307_57241, &P3499_306};	/*Property = FormData*/
const Value V3499_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_309 = {S28, 0, &V3499_309_57247, &P3499_307};	/*Property = Fullscreen*/
const Value V3499_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_310 = {S29, 0, &V3499_310_57250, &P3499_309};	/*Property = GeoLocation*/
const Value V3499_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_312 = {S30, 0, &V3499_312_57256, &P3499_310};	/*Property = History*/
const Value V3499_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_314 = {S31, 0, &V3499_314_57262, &P3499_312};	/*Property = Html5*/
const Value V3499_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_313 = {S32, 0, &V3499_313_57259, &P3499_314};	/*Property = Html-Media-Capture*/
const Value V3499_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_315 = {S33, 0, &V3499_315_57265, &P3499_313};	/*Property = Iframe*/
const Value V3499_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_316 = {S34, 0, &V3499_316_57268, &P3499_315};	/*Property = IndexedDB*/
const Value V3499_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_317 = {S35, 0, &V3499_317_57271, &P3499_316};	/*Property = Json*/
const Value V3499_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3499_15 = {S37, 0, &V3499_15_28, &P3499_317};	/*Property = LayoutEngine*/
const Value V3499_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_344 = {S38, 0, &V3499_344_57274, &P3499_15};	/*Property = Masking*/
const Value V3499_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_320 = {S39, 0, &V3499_320_57292, &P3499_344};	/*Property = PostMessage*/
const Value V3499_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_321 = {S40, 0, &V3499_321_57295, &P3499_320};	/*Property = Progress*/
const Value V3499_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_322 = {S41, 0, &V3499_322_57298, &P3499_321};	/*Property = Prompts*/
const Value V3499_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_323 = {S42, 0, &V3499_323_57301, &P3499_322};	/*Property = Selector*/
const Value V3499_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_324 = {S43, 0, &V3499_324_57307, &P3499_323};	/*Property = Svg*/
const Value V3499_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_327 = {S44, 0, &V3499_327_57316, &P3499_324};	/*Property = TouchEvents*/
const Value V3499_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_328 = {S45, 0, &V3499_328_57319, &P3499_327};	/*Property = Track*/
const Value V3499_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_329 = {S46, 0, &V3499_329_57325, &P3499_328};	/*Property = Video*/
const Value V3499_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_330 = {S47, 0, &V3499_330_57328, &P3499_329};	/*Property = Viewport*/
const Value V3499_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_332 = {S48, 0, &V3499_332_57340, &P3499_330};	/*Property = WebWorkers*/
const Value V3499_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3499_333 = {S49, 0, &V3499_333_57343, &P3499_332};	/*Property = Xhr2*/
const Profile P3499 = {"3499", &P3499_333};	/*Profile ID - 3499*/
const Value V13344_1_1 = {S2, NULL};	/*Value = True*/
const Property P13344_1 = {S50, 0, &V13344_1_1, NULL};	/*Property = IsMobile*/
const Value V13344_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13344_5 = {S52, 0, &V13344_5_1222, &P13344_1};	/*Property = ScreenPixelsHeight*/
const Value V13344_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13344_4 = {S54, 0, &V13344_4_1224, &P13344_5};	/*Property = ScreenPixelsWidth*/
const Profile P13344 = {"13344", &P13344_4};	/*Profile ID - 13344*/
const Value V17215_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_279 = {S1, 0, &V17215_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17215_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_282 = {S3, 0, &V17215_282_57155, &P17215_279};	/*Property = BlobBuilder*/
const Value V17215_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_350 = {S4, 0, &V17215_350_57446, &P17215_282};	/*Property = Canvas*/
const Value V17215_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_283 = {S5, 0, &V17215_283_57161, &P17215_350};	/*Property = CssBackground*/
const Value V17215_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_284 = {S6, 0, &V17215_284_57164, &P17215_283};	/*Property = CssBorderImage*/
const Value V17215_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_285 = {S7, 0, &V17215_285_57167, &P17215_284};	/*Property = CssCanvas*/
const Value V17215_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_286 = {S8, 0, &V17215_286_57170, &P17215_285};	/*Property = CssColor*/
const Value V17215_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_237 = {S9, 0, &V17215_237_55786, &P17215_286};	/*Property = CssColumn*/
const Value V17215_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_287 = {S10, 0, &V17215_287_57173, &P17215_237};	/*Property = CssFlexbox*/
const Value V17215_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_288 = {S11, 0, &V17215_288_57176, &P17215_287};	/*Property = CssFont*/
const Value V17215_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_289 = {S12, 0, &V17215_289_86899, &P17215_288};	/*Property = CssImages*/
const Value V17215_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_291 = {S13, 0, &V17215_291_57182, &P17215_289};	/*Property = CssMediaQueries*/
const Value V17215_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_292 = {S14, 0, &V17215_292_57185, &P17215_291};	/*Property = CssMinMax*/
const Value V17215_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_293 = {S15, 0, &V17215_293_57188, &P17215_292};	/*Property = CssOverflow*/
const Value V17215_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_294 = {S16, 0, &V17215_294_57191, &P17215_293};	/*Property = CssPosition*/
const Value V17215_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_295 = {S17, 0, &V17215_295_57194, &P17215_294};	/*Property = CssText*/
const Value V17215_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_297 = {S18, 0, &V17215_297_57197, &P17215_295};	/*Property = CssTransforms*/
const Value V17215_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_241 = {S19, 0, &V17215_241_55801, &P17215_297};	/*Property = CssTransitions*/
const Value V17215_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_298 = {S20, 0, &V17215_298_57208, &P17215_241};	/*Property = CssUI*/
const Value V17215_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_300 = {S21, 0, &V17215_300_57217, &P17215_298};	/*Property = DataSet*/
const Value V17215_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_301 = {S22, 0, &V17215_301_57220, &P17215_300};	/*Property = DataUrl*/
const Value V17215_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_302 = {S23, 0, &V17215_302_57223, &P17215_301};	/*Property = DeviceOrientation*/
const Value V17215_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_304 = {S24, 0, &V17215_304_57229, &P17215_302};	/*Property = FileReader*/
const Value V17215_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_305 = {S25, 0, &V17215_305_57232, &P17215_304};	/*Property = FileSaver*/
const Value V17215_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_306 = {S26, 0, &V17215_306_57238, &P17215_305};	/*Property = FileWriter*/
const Value V17215_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_307 = {S27, 0, &V17215_307_57241, &P17215_306};	/*Property = FormData*/
const Value V17215_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_309 = {S28, 0, &V17215_309_57247, &P17215_307};	/*Property = Fullscreen*/
const Value V17215_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_310 = {S29, 0, &V17215_310_57250, &P17215_309};	/*Property = GeoLocation*/
const Value V17215_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_312 = {S30, 0, &V17215_312_57256, &P17215_310};	/*Property = History*/
const Value V17215_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_314 = {S31, 0, &V17215_314_57262, &P17215_312};	/*Property = Html5*/
const Value V17215_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_313 = {S32, 0, &V17215_313_57259, &P17215_314};	/*Property = Html-Media-Capture*/
const Value V17215_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_315 = {S33, 0, &V17215_315_57265, &P17215_313};	/*Property = Iframe*/
const Value V17215_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_316 = {S34, 0, &V17215_316_57268, &P17215_315};	/*Property = IndexedDB*/
const Value V17215_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_317 = {S35, 0, &V17215_317_57271, &P17215_316};	/*Property = Json*/
const Value V17215_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P17215_15 = {S37, 0, &V17215_15_31, &P17215_317};	/*Property = LayoutEngine*/
const Value V17215_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_344 = {S38, 0, &V17215_344_57274, &P17215_15};	/*Property = Masking*/
const Value V17215_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_320 = {S39, 0, &V17215_320_57292, &P17215_344};	/*Property = PostMessage*/
const Value V17215_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_321 = {S40, 0, &V17215_321_57295, &P17215_320};	/*Property = Progress*/
const Value V17215_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_322 = {S41, 0, &V17215_322_57298, &P17215_321};	/*Property = Prompts*/
const Value V17215_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_323 = {S42, 0, &V17215_323_57301, &P17215_322};	/*Property = Selector*/
const Value V17215_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_324 = {S43, 0, &V17215_324_57307, &P17215_323};	/*Property = Svg*/
const Value V17215_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_327 = {S44, 0, &V17215_327_57316, &P17215_324};	/*Property = TouchEvents*/
const Value V17215_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_328 = {S45, 0, &V17215_328_57319, &P17215_327};	/*Property = Track*/
const Value V17215_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_329 = {S46, 0, &V17215_329_57325, &P17215_328};	/*Property = Video*/
const Value V17215_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_330 = {S47, 0, &V17215_330_57328, &P17215_329};	/*Property = Viewport*/
const Value V17215_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_332 = {S48, 0, &V17215_332_57340, &P17215_330};	/*Property = WebWorkers*/
const Value V17215_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17215_333 = {S49, 0, &V17215_333_57343, &P17215_332};	/*Property = Xhr2*/
const Profile P17215 = {"17215", &P17215_333};	/*Profile ID - 17215*/
const Value V13345_1_1 = {S2, NULL};	/*Value = True*/
const Property P13345_1 = {S50, 0, &V13345_1_1, NULL};	/*Property = IsMobile*/
const Value V13345_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13345_5 = {S52, 0, &V13345_5_1233, &P13345_1};	/*Property = ScreenPixelsHeight*/
const Value V13345_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13345_4 = {S54, 0, &V13345_4_1235, &P13345_5};	/*Property = ScreenPixelsWidth*/
const Profile P13345 = {"13345", &P13345_4};	/*Profile ID - 13345*/
const Value V14864_1_1 = {S2, NULL};	/*Value = True*/
const Property P14864_1 = {S50, 0, &V14864_1_1, NULL};	/*Property = IsMobile*/
const Value V14864_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14864_5 = {S52, 0, &V14864_5_1227, &P14864_1};	/*Property = ScreenPixelsHeight*/
const Value V14864_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14864_4 = {S54, 0, &V14864_4_1229, &P14864_5};	/*Property = ScreenPixelsWidth*/
const Profile P14864 = {"14864", &P14864_4};	/*Profile ID - 14864*/
const Value V13348_1_1 = {S2, NULL};	/*Value = True*/
const Property P13348_1 = {S50, 0, &V13348_1_1, NULL};	/*Property = IsMobile*/
const Value V13348_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13348_5 = {S52, 0, &V13348_5_1227, &P13348_1};	/*Property = ScreenPixelsHeight*/
const Value V13348_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13348_4 = {S54, 0, &V13348_4_1229, &P13348_5};	/*Property = ScreenPixelsWidth*/
const Profile P13348 = {"13348", &P13348_4};	/*Profile ID - 13348*/
const Value V14519_1_1 = {S2, NULL};	/*Value = True*/
const Property P14519_1 = {S50, 0, &V14519_1_1, NULL};	/*Property = IsMobile*/
const Value V14519_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P14519_5 = {S52, 0, &V14519_5_1275, &P14519_1};	/*Property = ScreenPixelsHeight*/
const Value V14519_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P14519_4 = {S54, 0, &V14519_4_1278, &P14519_5};	/*Property = ScreenPixelsWidth*/
const Profile P14519 = {"14519", &P14519_4};	/*Profile ID - 14519*/
const Value V13349_1_1 = {S2, NULL};	/*Value = True*/
const Property P13349_1 = {S50, 0, &V13349_1_1, NULL};	/*Property = IsMobile*/
const Value V13349_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13349_5 = {S52, 0, &V13349_5_1277, &P13349_1};	/*Property = ScreenPixelsHeight*/
const Value V13349_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13349_4 = {S54, 0, &V13349_4_1279, &P13349_5};	/*Property = ScreenPixelsWidth*/
const Profile P13349 = {"13349", &P13349_4};	/*Profile ID - 13349*/
const Value V12453_1_1 = {S2, NULL};	/*Value = True*/
const Property P12453_1 = {S50, 0, &V12453_1_1, NULL};	/*Property = IsMobile*/
const Value V12453_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12453_5 = {S52, 0, &V12453_5_1277, &P12453_1};	/*Property = ScreenPixelsHeight*/
const Value V12453_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12453_4 = {S54, 0, &V12453_4_1279, &P12453_5};	/*Property = ScreenPixelsWidth*/
const Profile P12453 = {"12453", &P12453_4};	/*Profile ID - 12453*/
const Value V14384_1_1 = {S2, NULL};	/*Value = True*/
const Property P14384_1 = {S50, 0, &V14384_1_1, NULL};	/*Property = IsMobile*/
const Value V14384_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14384_5 = {S52, 0, &V14384_5_1277, &P14384_1};	/*Property = ScreenPixelsHeight*/
const Value V14384_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P14384_4 = {S54, 0, &V14384_4_1278, &P14384_5};	/*Property = ScreenPixelsWidth*/
const Profile P14384 = {"14384", &P14384_4};	/*Profile ID - 14384*/
const Value V13815_1_1 = {S2, NULL};	/*Value = True*/
const Property P13815_1 = {S50, 0, &V13815_1_1, NULL};	/*Property = IsMobile*/
const Value V13815_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13815_5 = {S52, 0, &V13815_5_1222, &P13815_1};	/*Property = ScreenPixelsHeight*/
const Value V13815_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13815_4 = {S54, 0, &V13815_4_1224, &P13815_5};	/*Property = ScreenPixelsWidth*/
const Profile P13815 = {"13815", &P13815_4};	/*Profile ID - 13815*/
const Value V17617_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_279 = {S1, 0, &V17617_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17617_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_282 = {S3, 0, &V17617_282_57155, &P17617_279};	/*Property = BlobBuilder*/
const Value V17617_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_350 = {S4, 0, &V17617_350_57446, &P17617_282};	/*Property = Canvas*/
const Value V17617_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_283 = {S5, 0, &V17617_283_57161, &P17617_350};	/*Property = CssBackground*/
const Value V17617_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_284 = {S6, 0, &V17617_284_57164, &P17617_283};	/*Property = CssBorderImage*/
const Value V17617_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_285 = {S7, 0, &V17617_285_57167, &P17617_284};	/*Property = CssCanvas*/
const Value V17617_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_286 = {S8, 0, &V17617_286_57170, &P17617_285};	/*Property = CssColor*/
const Value V17617_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_237 = {S9, 0, &V17617_237_55786, &P17617_286};	/*Property = CssColumn*/
const Value V17617_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_287 = {S10, 0, &V17617_287_57173, &P17617_237};	/*Property = CssFlexbox*/
const Value V17617_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_288 = {S11, 0, &V17617_288_57176, &P17617_287};	/*Property = CssFont*/
const Value V17617_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_289 = {S12, 0, &V17617_289_86899, &P17617_288};	/*Property = CssImages*/
const Value V17617_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_291 = {S13, 0, &V17617_291_57182, &P17617_289};	/*Property = CssMediaQueries*/
const Value V17617_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_292 = {S14, 0, &V17617_292_57185, &P17617_291};	/*Property = CssMinMax*/
const Value V17617_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_293 = {S15, 0, &V17617_293_57188, &P17617_292};	/*Property = CssOverflow*/
const Value V17617_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_294 = {S16, 0, &V17617_294_57191, &P17617_293};	/*Property = CssPosition*/
const Value V17617_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_295 = {S17, 0, &V17617_295_57194, &P17617_294};	/*Property = CssText*/
const Value V17617_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_297 = {S18, 0, &V17617_297_57197, &P17617_295};	/*Property = CssTransforms*/
const Value V17617_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_241 = {S19, 0, &V17617_241_55801, &P17617_297};	/*Property = CssTransitions*/
const Value V17617_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_298 = {S20, 0, &V17617_298_57208, &P17617_241};	/*Property = CssUI*/
const Value V17617_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_300 = {S21, 0, &V17617_300_57217, &P17617_298};	/*Property = DataSet*/
const Value V17617_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_301 = {S22, 0, &V17617_301_57220, &P17617_300};	/*Property = DataUrl*/
const Value V17617_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_302 = {S23, 0, &V17617_302_57223, &P17617_301};	/*Property = DeviceOrientation*/
const Value V17617_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_304 = {S24, 0, &V17617_304_57229, &P17617_302};	/*Property = FileReader*/
const Value V17617_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_305 = {S25, 0, &V17617_305_57232, &P17617_304};	/*Property = FileSaver*/
const Value V17617_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_306 = {S26, 0, &V17617_306_57238, &P17617_305};	/*Property = FileWriter*/
const Value V17617_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_307 = {S27, 0, &V17617_307_57241, &P17617_306};	/*Property = FormData*/
const Value V17617_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_309 = {S28, 0, &V17617_309_57247, &P17617_307};	/*Property = Fullscreen*/
const Value V17617_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_310 = {S29, 0, &V17617_310_57250, &P17617_309};	/*Property = GeoLocation*/
const Value V17617_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_312 = {S30, 0, &V17617_312_57256, &P17617_310};	/*Property = History*/
const Value V17617_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_314 = {S31, 0, &V17617_314_57262, &P17617_312};	/*Property = Html5*/
const Value V17617_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_313 = {S32, 0, &V17617_313_57259, &P17617_314};	/*Property = Html-Media-Capture*/
const Value V17617_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_315 = {S33, 0, &V17617_315_57265, &P17617_313};	/*Property = Iframe*/
const Value V17617_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_316 = {S34, 0, &V17617_316_57268, &P17617_315};	/*Property = IndexedDB*/
const Value V17617_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_317 = {S35, 0, &V17617_317_57271, &P17617_316};	/*Property = Json*/
const Value V17617_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_15 = {S37, 0, &V17617_15_72, &P17617_317};	/*Property = LayoutEngine*/
const Value V17617_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_344 = {S38, 0, &V17617_344_57274, &P17617_15};	/*Property = Masking*/
const Value V17617_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_320 = {S39, 0, &V17617_320_57292, &P17617_344};	/*Property = PostMessage*/
const Value V17617_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_321 = {S40, 0, &V17617_321_57295, &P17617_320};	/*Property = Progress*/
const Value V17617_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_322 = {S41, 0, &V17617_322_57298, &P17617_321};	/*Property = Prompts*/
const Value V17617_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_323 = {S42, 0, &V17617_323_57301, &P17617_322};	/*Property = Selector*/
const Value V17617_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_324 = {S43, 0, &V17617_324_57307, &P17617_323};	/*Property = Svg*/
const Value V17617_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_327 = {S44, 0, &V17617_327_57316, &P17617_324};	/*Property = TouchEvents*/
const Value V17617_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_328 = {S45, 0, &V17617_328_57319, &P17617_327};	/*Property = Track*/
const Value V17617_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_329 = {S46, 0, &V17617_329_57325, &P17617_328};	/*Property = Video*/
const Value V17617_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_330 = {S47, 0, &V17617_330_57328, &P17617_329};	/*Property = Viewport*/
const Value V17617_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_332 = {S48, 0, &V17617_332_57340, &P17617_330};	/*Property = WebWorkers*/
const Value V17617_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17617_333 = {S49, 0, &V17617_333_57343, &P17617_332};	/*Property = Xhr2*/
const Profile P17617 = {"17617", &P17617_333};	/*Profile ID - 17617*/
const Value V13501_1_1 = {S2, NULL};	/*Value = True*/
const Property P13501_1 = {S50, 0, &V13501_1_1, NULL};	/*Property = IsMobile*/
const Value V13501_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13501_5 = {S52, 0, &V13501_5_1222, &P13501_1};	/*Property = ScreenPixelsHeight*/
const Value V13501_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13501_4 = {S54, 0, &V13501_4_1224, &P13501_5};	/*Property = ScreenPixelsWidth*/
const Profile P13501 = {"13501", &P13501_4};	/*Profile ID - 13501*/
const Value V14225_1_1 = {S2, NULL};	/*Value = True*/
const Property P14225_1 = {S50, 0, &V14225_1_1, NULL};	/*Property = IsMobile*/
const Value V14225_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14225_5 = {S52, 0, &V14225_5_1227, &P14225_1};	/*Property = ScreenPixelsHeight*/
const Value V14225_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14225_4 = {S54, 0, &V14225_4_1229, &P14225_5};	/*Property = ScreenPixelsWidth*/
const Profile P14225 = {"14225", &P14225_4};	/*Profile ID - 14225*/
const Value V12216_1_1 = {S2, NULL};	/*Value = True*/
const Property P12216_1 = {S50, 0, &V12216_1_1, NULL};	/*Property = IsMobile*/
const Value V12216_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12216_5 = {S52, 0, &V12216_5_1227, &P12216_1};	/*Property = ScreenPixelsHeight*/
const Value V12216_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12216_4 = {S54, 0, &V12216_4_1229, &P12216_5};	/*Property = ScreenPixelsWidth*/
const Profile P12216 = {"12216", &P12216_4};	/*Profile ID - 12216*/
const Value V15759_1_1 = {S2, NULL};	/*Value = True*/
const Property P15759_1 = {S50, 0, &V15759_1_1, NULL};	/*Property = IsMobile*/
const Value V15759_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15759_5 = {S52, 0, &V15759_5_1227, &P15759_1};	/*Property = ScreenPixelsHeight*/
const Value V15759_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15759_4 = {S54, 0, &V15759_4_1229, &P15759_5};	/*Property = ScreenPixelsWidth*/
const Profile P15759 = {"15759", &P15759_4};	/*Profile ID - 15759*/
const Value V14824_1_1 = {S2, NULL};	/*Value = True*/
const Property P14824_1 = {S50, 0, &V14824_1_1, NULL};	/*Property = IsMobile*/
const Value V14824_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14824_5 = {S52, 0, &V14824_5_1222, &P14824_1};	/*Property = ScreenPixelsHeight*/
const Value V14824_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14824_4 = {S54, 0, &V14824_4_1224, &P14824_5};	/*Property = ScreenPixelsWidth*/
const Profile P14824 = {"14824", &P14824_4};	/*Profile ID - 14824*/
const Value V17694_1_1 = {S2, NULL};	/*Value = True*/
const Property P17694_1 = {S50, 0, &V17694_1_1, NULL};	/*Property = IsMobile*/
const Value V17694_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17694_5 = {S52, 0, &V17694_5_1233, &P17694_1};	/*Property = ScreenPixelsHeight*/
const Value V17694_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17694_4 = {S54, 0, &V17694_4_1235, &P17694_5};	/*Property = ScreenPixelsWidth*/
const Profile P17694 = {"17694", &P17694_4};	/*Profile ID - 17694*/
const Value V11932_1_1 = {S2, NULL};	/*Value = True*/
const Property P11932_1 = {S50, 0, &V11932_1_1, NULL};	/*Property = IsMobile*/
const Value V11932_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11932_5 = {S52, 0, &V11932_5_1222, &P11932_1};	/*Property = ScreenPixelsHeight*/
const Value V11932_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11932_4 = {S54, 0, &V11932_4_1224, &P11932_5};	/*Property = ScreenPixelsWidth*/
const Profile P11932 = {"11932", &P11932_4};	/*Profile ID - 11932*/
const Value V12224_1_1 = {S2, NULL};	/*Value = True*/
const Property P12224_1 = {S50, 0, &V12224_1_1, NULL};	/*Property = IsMobile*/
const Value V12224_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12224_5 = {S52, 0, &V12224_5_1222, &P12224_1};	/*Property = ScreenPixelsHeight*/
const Value V12224_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12224_4 = {S54, 0, &V12224_4_1224, &P12224_5};	/*Property = ScreenPixelsWidth*/
const Profile P12224 = {"12224", &P12224_4};	/*Profile ID - 12224*/
const Value V4086_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_279 = {S1, 0, &V4086_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4086_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_282 = {S3, 0, &V4086_282_57155, &P4086_279};	/*Property = BlobBuilder*/
const Value V4086_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_350 = {S4, 0, &V4086_350_57446, &P4086_282};	/*Property = Canvas*/
const Value V4086_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_283 = {S5, 0, &V4086_283_57161, &P4086_350};	/*Property = CssBackground*/
const Value V4086_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_284 = {S6, 0, &V4086_284_57164, &P4086_283};	/*Property = CssBorderImage*/
const Value V4086_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_285 = {S7, 0, &V4086_285_57167, &P4086_284};	/*Property = CssCanvas*/
const Value V4086_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_286 = {S8, 0, &V4086_286_57170, &P4086_285};	/*Property = CssColor*/
const Value V4086_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_237 = {S9, 0, &V4086_237_55786, &P4086_286};	/*Property = CssColumn*/
const Value V4086_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_287 = {S10, 0, &V4086_287_57173, &P4086_237};	/*Property = CssFlexbox*/
const Value V4086_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_288 = {S11, 0, &V4086_288_57176, &P4086_287};	/*Property = CssFont*/
const Value V4086_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_289 = {S12, 0, &V4086_289_86899, &P4086_288};	/*Property = CssImages*/
const Value V4086_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_291 = {S13, 0, &V4086_291_57182, &P4086_289};	/*Property = CssMediaQueries*/
const Value V4086_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_292 = {S14, 0, &V4086_292_57185, &P4086_291};	/*Property = CssMinMax*/
const Value V4086_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_293 = {S15, 0, &V4086_293_57188, &P4086_292};	/*Property = CssOverflow*/
const Value V4086_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_294 = {S16, 0, &V4086_294_57191, &P4086_293};	/*Property = CssPosition*/
const Value V4086_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_295 = {S17, 0, &V4086_295_57194, &P4086_294};	/*Property = CssText*/
const Value V4086_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_297 = {S18, 0, &V4086_297_57197, &P4086_295};	/*Property = CssTransforms*/
const Value V4086_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_241 = {S19, 0, &V4086_241_55801, &P4086_297};	/*Property = CssTransitions*/
const Value V4086_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_298 = {S20, 0, &V4086_298_57208, &P4086_241};	/*Property = CssUI*/
const Value V4086_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_300 = {S21, 0, &V4086_300_57217, &P4086_298};	/*Property = DataSet*/
const Value V4086_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_301 = {S22, 0, &V4086_301_57220, &P4086_300};	/*Property = DataUrl*/
const Value V4086_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_302 = {S23, 0, &V4086_302_57223, &P4086_301};	/*Property = DeviceOrientation*/
const Value V4086_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_304 = {S24, 0, &V4086_304_57229, &P4086_302};	/*Property = FileReader*/
const Value V4086_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_305 = {S25, 0, &V4086_305_57232, &P4086_304};	/*Property = FileSaver*/
const Value V4086_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_306 = {S26, 0, &V4086_306_57238, &P4086_305};	/*Property = FileWriter*/
const Value V4086_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_307 = {S27, 0, &V4086_307_57241, &P4086_306};	/*Property = FormData*/
const Value V4086_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_309 = {S28, 0, &V4086_309_57247, &P4086_307};	/*Property = Fullscreen*/
const Value V4086_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_310 = {S29, 0, &V4086_310_57250, &P4086_309};	/*Property = GeoLocation*/
const Value V4086_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_312 = {S30, 0, &V4086_312_57256, &P4086_310};	/*Property = History*/
const Value V4086_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_314 = {S31, 0, &V4086_314_57262, &P4086_312};	/*Property = Html5*/
const Value V4086_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_313 = {S32, 0, &V4086_313_57259, &P4086_314};	/*Property = Html-Media-Capture*/
const Value V4086_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_315 = {S33, 0, &V4086_315_57265, &P4086_313};	/*Property = Iframe*/
const Value V4086_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_316 = {S34, 0, &V4086_316_57268, &P4086_315};	/*Property = IndexedDB*/
const Value V4086_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_317 = {S35, 0, &V4086_317_57271, &P4086_316};	/*Property = Json*/
const Value V4086_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_15 = {S37, 0, &V4086_15_72, &P4086_317};	/*Property = LayoutEngine*/
const Value V4086_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_344 = {S38, 0, &V4086_344_57274, &P4086_15};	/*Property = Masking*/
const Value V4086_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_320 = {S39, 0, &V4086_320_57292, &P4086_344};	/*Property = PostMessage*/
const Value V4086_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_321 = {S40, 0, &V4086_321_57295, &P4086_320};	/*Property = Progress*/
const Value V4086_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_322 = {S41, 0, &V4086_322_57298, &P4086_321};	/*Property = Prompts*/
const Value V4086_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_323 = {S42, 0, &V4086_323_57301, &P4086_322};	/*Property = Selector*/
const Value V4086_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_324 = {S43, 0, &V4086_324_57307, &P4086_323};	/*Property = Svg*/
const Value V4086_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_327 = {S44, 0, &V4086_327_57316, &P4086_324};	/*Property = TouchEvents*/
const Value V4086_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_328 = {S45, 0, &V4086_328_57319, &P4086_327};	/*Property = Track*/
const Value V4086_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_329 = {S46, 0, &V4086_329_57325, &P4086_328};	/*Property = Video*/
const Value V4086_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_330 = {S47, 0, &V4086_330_57328, &P4086_329};	/*Property = Viewport*/
const Value V4086_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_332 = {S48, 0, &V4086_332_57340, &P4086_330};	/*Property = WebWorkers*/
const Value V4086_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4086_333 = {S49, 0, &V4086_333_57343, &P4086_332};	/*Property = Xhr2*/
const Profile P4086 = {"4086", &P4086_333};	/*Profile ID - 4086*/
const Value V11996_1_1 = {S2, NULL};	/*Value = True*/
const Property P11996_1 = {S50, 0, &V11996_1_1, NULL};	/*Property = IsMobile*/
const Value V11996_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P11996_5 = {S52, 0, &V11996_5_1304, &P11996_1};	/*Property = ScreenPixelsHeight*/
const Value V11996_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11996_4 = {S54, 0, &V11996_4_1224, &P11996_5};	/*Property = ScreenPixelsWidth*/
const Profile P11996 = {"11996", &P11996_4};	/*Profile ID - 11996*/
const Value V13427_1_1 = {S2, NULL};	/*Value = True*/
const Property P13427_1 = {S50, 0, &V13427_1_1, NULL};	/*Property = IsMobile*/
const Value V13427_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13427_5 = {S52, 0, &V13427_5_1222, &P13427_1};	/*Property = ScreenPixelsHeight*/
const Value V13427_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13427_4 = {S54, 0, &V13427_4_1224, &P13427_5};	/*Property = ScreenPixelsWidth*/
const Profile P13427 = {"13427", &P13427_4};	/*Profile ID - 13427*/
const Value V12236_1_1 = {S2, NULL};	/*Value = True*/
const Property P12236_1 = {S50, 0, &V12236_1_1, NULL};	/*Property = IsMobile*/
const Value V12236_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12236_5 = {S52, 0, &V12236_5_1222, &P12236_1};	/*Property = ScreenPixelsHeight*/
const Value V12236_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12236_4 = {S54, 0, &V12236_4_1224, &P12236_5};	/*Property = ScreenPixelsWidth*/
const Profile P12236 = {"12236", &P12236_4};	/*Profile ID - 12236*/
const Value V12248_1_1 = {S2, NULL};	/*Value = True*/
const Property P12248_1 = {S50, 0, &V12248_1_1, NULL};	/*Property = IsMobile*/
const Value V12248_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12248_5 = {S52, 0, &V12248_5_1293, &P12248_1};	/*Property = ScreenPixelsHeight*/
const Value V12248_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12248_4 = {S54, 0, &V12248_4_1296, &P12248_5};	/*Property = ScreenPixelsWidth*/
const Profile P12248 = {"12248", &P12248_4};	/*Profile ID - 12248*/
const Value V12243_1_1 = {S2, NULL};	/*Value = True*/
const Property P12243_1 = {S50, 0, &V12243_1_1, NULL};	/*Property = IsMobile*/
const Value V12243_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12243_5 = {S52, 0, &V12243_5_1380, &P12243_1};	/*Property = ScreenPixelsHeight*/
const Value V12243_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12243_4 = {S54, 0, &V12243_4_1279, &P12243_5};	/*Property = ScreenPixelsWidth*/
const Profile P12243 = {"12243", &P12243_4};	/*Profile ID - 12243*/
const Value V14107_1_1 = {S2, NULL};	/*Value = True*/
const Property P14107_1 = {S50, 0, &V14107_1_1, NULL};	/*Property = IsMobile*/
const Value V14107_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14107_5 = {S52, 0, &V14107_5_1222, &P14107_1};	/*Property = ScreenPixelsHeight*/
const Value V14107_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14107_4 = {S54, 0, &V14107_4_1224, &P14107_5};	/*Property = ScreenPixelsWidth*/
const Profile P14107 = {"14107", &P14107_4};	/*Profile ID - 14107*/
const Value V13639_1_1 = {S2, NULL};	/*Value = True*/
const Property P13639_1 = {S50, 0, &V13639_1_1, NULL};	/*Property = IsMobile*/
const Value V13639_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13639_5 = {S52, 0, &V13639_5_1293, &P13639_1};	/*Property = ScreenPixelsHeight*/
const Value V13639_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13639_4 = {S54, 0, &V13639_4_1296, &P13639_5};	/*Property = ScreenPixelsWidth*/
const Profile P13639 = {"13639", &P13639_4};	/*Profile ID - 13639*/
const Value V12249_1_1 = {S2, NULL};	/*Value = True*/
const Property P12249_1 = {S50, 0, &V12249_1_1, NULL};	/*Property = IsMobile*/
const Value V12249_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12249_5 = {S52, 0, &V12249_5_1222, &P12249_1};	/*Property = ScreenPixelsHeight*/
const Value V12249_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12249_4 = {S54, 0, &V12249_4_1224, &P12249_5};	/*Property = ScreenPixelsWidth*/
const Profile P12249 = {"12249", &P12249_4};	/*Profile ID - 12249*/
const Value V15377_1_1 = {S2, NULL};	/*Value = True*/
const Property P15377_1 = {S50, 0, &V15377_1_1, NULL};	/*Property = IsMobile*/
const Value V15377_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15377_5 = {S52, 0, &V15377_5_1380, &P15377_1};	/*Property = ScreenPixelsHeight*/
const Value V15377_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15377_4 = {S54, 0, &V15377_4_1279, &P15377_5};	/*Property = ScreenPixelsWidth*/
const Profile P15377 = {"15377", &P15377_4};	/*Profile ID - 15377*/
const Value V14839_1_1 = {S2, NULL};	/*Value = True*/
const Property P14839_1 = {S50, 0, &V14839_1_1, NULL};	/*Property = IsMobile*/
const Value V14839_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14839_5 = {S52, 0, &V14839_5_1222, &P14839_1};	/*Property = ScreenPixelsHeight*/
const Value V14839_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14839_4 = {S54, 0, &V14839_4_1224, &P14839_5};	/*Property = ScreenPixelsWidth*/
const Profile P14839 = {"14839", &P14839_4};	/*Profile ID - 14839*/
const Value V12251_1_1 = {S2, NULL};	/*Value = True*/
const Property P12251_1 = {S50, 0, &V12251_1_1, NULL};	/*Property = IsMobile*/
const Value V12251_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12251_5 = {S52, 0, &V12251_5_1222, &P12251_1};	/*Property = ScreenPixelsHeight*/
const Value V12251_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12251_4 = {S54, 0, &V12251_4_1224, &P12251_5};	/*Property = ScreenPixelsWidth*/
const Profile P12251 = {"12251", &P12251_4};	/*Profile ID - 12251*/
const Value V12256_1_1 = {S2, NULL};	/*Value = True*/
const Property P12256_1 = {S50, 0, &V12256_1_1, NULL};	/*Property = IsMobile*/
const Value V12256_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12256_5 = {S52, 0, &V12256_5_1233, &P12256_1};	/*Property = ScreenPixelsHeight*/
const Value V12256_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12256_4 = {S54, 0, &V12256_4_1235, &P12256_5};	/*Property = ScreenPixelsWidth*/
const Profile P12256 = {"12256", &P12256_4};	/*Profile ID - 12256*/
const Value V15134_1_1 = {S2, NULL};	/*Value = True*/
const Property P15134_1 = {S50, 0, &V15134_1_1, NULL};	/*Property = IsMobile*/
const Value V15134_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15134_5 = {S52, 0, &V15134_5_1227, &P15134_1};	/*Property = ScreenPixelsHeight*/
const Value V15134_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15134_4 = {S54, 0, &V15134_4_1229, &P15134_5};	/*Property = ScreenPixelsWidth*/
const Profile P15134 = {"15134", &P15134_4};	/*Profile ID - 15134*/
const Value V12258_1_1 = {S2, NULL};	/*Value = True*/
const Property P12258_1 = {S50, 0, &V12258_1_1, NULL};	/*Property = IsMobile*/
const Value V12258_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12258_5 = {S52, 0, &V12258_5_1304, &P12258_1};	/*Property = ScreenPixelsHeight*/
const Value V12258_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12258_4 = {S54, 0, &V12258_4_1224, &P12258_5};	/*Property = ScreenPixelsWidth*/
const Profile P12258 = {"12258", &P12258_4};	/*Profile ID - 12258*/
const Value V12259_1_1 = {S2, NULL};	/*Value = True*/
const Property P12259_1 = {S50, 0, &V12259_1_1, NULL};	/*Property = IsMobile*/
const Value V12259_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12259_5 = {S52, 0, &V12259_5_1222, &P12259_1};	/*Property = ScreenPixelsHeight*/
const Value V12259_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12259_4 = {S54, 0, &V12259_4_1224, &P12259_5};	/*Property = ScreenPixelsWidth*/
const Profile P12259 = {"12259", &P12259_4};	/*Profile ID - 12259*/
const Value V13370_1_1 = {S2, NULL};	/*Value = True*/
const Property P13370_1 = {S50, 0, &V13370_1_1, NULL};	/*Property = IsMobile*/
const Value V13370_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13370_5 = {S52, 0, &V13370_5_1222, &P13370_1};	/*Property = ScreenPixelsHeight*/
const Value V13370_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13370_4 = {S54, 0, &V13370_4_1224, &P13370_5};	/*Property = ScreenPixelsWidth*/
const Profile P13370 = {"13370", &P13370_4};	/*Profile ID - 13370*/
const Value V12262_1_1 = {S2, NULL};	/*Value = True*/
const Property P12262_1 = {S50, 0, &V12262_1_1, NULL};	/*Property = IsMobile*/
const Value V12262_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12262_5 = {S52, 0, &V12262_5_1304, &P12262_1};	/*Property = ScreenPixelsHeight*/
const Value V12262_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12262_4 = {S54, 0, &V12262_4_1224, &P12262_5};	/*Property = ScreenPixelsWidth*/
const Profile P12262 = {"12262", &P12262_4};	/*Profile ID - 12262*/
const Value V12263_1_1 = {S2, NULL};	/*Value = True*/
const Property P12263_1 = {S50, 0, &V12263_1_1, NULL};	/*Property = IsMobile*/
const Value V12263_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12263_5 = {S52, 0, &V12263_5_1277, &P12263_1};	/*Property = ScreenPixelsHeight*/
const Value V12263_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12263_4 = {S54, 0, &V12263_4_1279, &P12263_5};	/*Property = ScreenPixelsWidth*/
const Profile P12263 = {"12263", &P12263_4};	/*Profile ID - 12263*/
const Value V12379_1_1 = {S2, NULL};	/*Value = True*/
const Property P12379_1 = {S50, 0, &V12379_1_1, NULL};	/*Property = IsMobile*/
const Value V12379_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12379_5 = {S52, 0, &V12379_5_1277, &P12379_1};	/*Property = ScreenPixelsHeight*/
const Value V12379_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12379_4 = {S54, 0, &V12379_4_1279, &P12379_5};	/*Property = ScreenPixelsWidth*/
const Profile P12379 = {"12379", &P12379_4};	/*Profile ID - 12379*/
const Value V12265_1_1 = {S2, NULL};	/*Value = True*/
const Property P12265_1 = {S50, 0, &V12265_1_1, NULL};	/*Property = IsMobile*/
const Value V12265_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12265_5 = {S52, 0, &V12265_5_1233, &P12265_1};	/*Property = ScreenPixelsHeight*/
const Value V12265_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12265_4 = {S54, 0, &V12265_4_1235, &P12265_5};	/*Property = ScreenPixelsWidth*/
const Profile P12265 = {"12265", &P12265_4};	/*Profile ID - 12265*/
const Value V14288_1_1 = {S2, NULL};	/*Value = True*/
const Property P14288_1 = {S50, 0, &V14288_1_1, NULL};	/*Property = IsMobile*/
const Value V14288_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14288_5 = {S52, 0, &V14288_5_1293, &P14288_1};	/*Property = ScreenPixelsHeight*/
const Value V14288_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14288_4 = {S54, 0, &V14288_4_1296, &P14288_5};	/*Property = ScreenPixelsWidth*/
const Profile P14288 = {"14288", &P14288_4};	/*Profile ID - 14288*/
const Value V14162_1_1 = {S2, NULL};	/*Value = True*/
const Property P14162_1 = {S50, 0, &V14162_1_1, NULL};	/*Property = IsMobile*/
const Value V14162_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14162_5 = {S52, 0, &V14162_5_1222, &P14162_1};	/*Property = ScreenPixelsHeight*/
const Value V14162_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14162_4 = {S54, 0, &V14162_4_1224, &P14162_5};	/*Property = ScreenPixelsWidth*/
const Profile P14162 = {"14162", &P14162_4};	/*Profile ID - 14162*/
const Value V15206_1_1 = {S2, NULL};	/*Value = True*/
const Property P15206_1 = {S50, 0, &V15206_1_1, NULL};	/*Property = IsMobile*/
const Value V15206_5_4340 = {S78, NULL};	/*Value = 352*/
const Property P15206_5 = {S52, 0, &V15206_5_4340, &P15206_1};	/*Property = ScreenPixelsHeight*/
const Value V15206_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P15206_4 = {S54, 0, &V15206_4_2336, &P15206_5};	/*Property = ScreenPixelsWidth*/
const Profile P15206 = {"15206", &P15206_4};	/*Profile ID - 15206*/
const Value V13378_1_1 = {S2, NULL};	/*Value = True*/
const Property P13378_1 = {S50, 0, &V13378_1_1, NULL};	/*Property = IsMobile*/
const Value V13378_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13378_5 = {S52, 0, &V13378_5_1293, &P13378_1};	/*Property = ScreenPixelsHeight*/
const Value V13378_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13378_4 = {S54, 0, &V13378_4_1296, &P13378_5};	/*Property = ScreenPixelsWidth*/
const Profile P13378 = {"13378", &P13378_4};	/*Profile ID - 13378*/
const Value V15425_1_1 = {S2, NULL};	/*Value = True*/
const Property P15425_1 = {S50, 0, &V15425_1_1, NULL};	/*Property = IsMobile*/
const Value V15425_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15425_5 = {S52, 0, &V15425_5_1304, &P15425_1};	/*Property = ScreenPixelsHeight*/
const Value V15425_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15425_4 = {S54, 0, &V15425_4_1224, &P15425_5};	/*Property = ScreenPixelsWidth*/
const Profile P15425 = {"15425", &P15425_4};	/*Profile ID - 15425*/
const Value V12280_1_1 = {S2, NULL};	/*Value = True*/
const Property P12280_1 = {S50, 0, &V12280_1_1, NULL};	/*Property = IsMobile*/
const Value V12280_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P12280_5 = {S52, 0, &V12280_5_1403, &P12280_1};	/*Property = ScreenPixelsHeight*/
const Value V12280_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P12280_4 = {S54, 0, &V12280_4_8554, &P12280_5};	/*Property = ScreenPixelsWidth*/
const Profile P12280 = {"12280", &P12280_4};	/*Profile ID - 12280*/
const Profile P17090 = {"17090", NULL};	/*Profile ID - 17090*/
const Value V17138_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17138_279 = {S1, 0, &V17138_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17138_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17138_282 = {S3, 0, &V17138_282_57154, &P17138_279};	/*Property = BlobBuilder*/
const Value V17138_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17138_350 = {S4, 0, &V17138_350_57444, &P17138_282};	/*Property = Canvas*/
const Value V17138_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17138_283 = {S5, 0, &V17138_283_57159, &P17138_350};	/*Property = CssBackground*/
const Value V17138_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17138_284 = {S6, 0, &V17138_284_57163, &P17138_283};	/*Property = CssBorderImage*/
const Value V17138_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17138_285 = {S7, 0, &V17138_285_57166, &P17138_284};	/*Property = CssCanvas*/
const Value V17138_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17138_286 = {S8, 0, &V17138_286_57168, &P17138_285};	/*Property = CssColor*/
const Value V17138_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17138_237 = {S9, 0, &V17138_237_55784, &P17138_286};	/*Property = CssColumn*/
const Value V17138_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17138_287 = {S10, 0, &V17138_287_57172, &P17138_237};	/*Property = CssFlexbox*/
const Value V17138_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17138_288 = {S11, 0, &V17138_288_57175, &P17138_287};	/*Property = CssFont*/
const Value V17138_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17138_289 = {S12, 0, &V17138_289_57759, &P17138_288};	/*Property = CssImages*/
const Value V17138_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17138_291 = {S13, 0, &V17138_291_57181, &P17138_289};	/*Property = CssMediaQueries*/
const Value V17138_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17138_292 = {S14, 0, &V17138_292_57183, &P17138_291};	/*Property = CssMinMax*/
const Value V17138_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17138_293 = {S15, 0, &V17138_293_57187, &P17138_292};	/*Property = CssOverflow*/
const Value V17138_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17138_294 = {S16, 0, &V17138_294_57189, &P17138_293};	/*Property = CssPosition*/
const Value V17138_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17138_295 = {S17, 0, &V17138_295_57192, &P17138_294};	/*Property = CssText*/
const Value V17138_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17138_297 = {S18, 0, &V17138_297_57195, &P17138_295};	/*Property = CssTransforms*/
const Value V17138_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17138_241 = {S19, 0, &V17138_241_55799, &P17138_297};	/*Property = CssTransitions*/
const Value V17138_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17138_298 = {S20, 0, &V17138_298_57206, &P17138_241};	/*Property = CssUI*/
const Value V17138_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17138_300 = {S21, 0, &V17138_300_57216, &P17138_298};	/*Property = DataSet*/
const Value V17138_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17138_301 = {S22, 0, &V17138_301_57218, &P17138_300};	/*Property = DataUrl*/
const Value V17138_302_57221 = {S2, NULL};	/*Value = True*/
const Property P17138_302 = {S23, 0, &V17138_302_57221, &P17138_301};	/*Property = DeviceOrientation*/
const Value V17138_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17138_304 = {S24, 0, &V17138_304_57228, &P17138_302};	/*Property = FileReader*/
const Value V17138_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17138_305 = {S25, 0, &V17138_305_57231, &P17138_304};	/*Property = FileSaver*/
const Value V17138_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17138_306 = {S26, 0, &V17138_306_57237, &P17138_305};	/*Property = FileWriter*/
const Value V17138_307_57240 = {S0, NULL};	/*Value = False*/
const Property P17138_307 = {S27, 0, &V17138_307_57240, &P17138_306};	/*Property = FormData*/
const Value V17138_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17138_309 = {S28, 0, &V17138_309_57246, &P17138_307};	/*Property = Fullscreen*/
const Value V17138_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17138_310 = {S29, 0, &V17138_310_57248, &P17138_309};	/*Property = GeoLocation*/
const Value V17138_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17138_312 = {S30, 0, &V17138_312_57255, &P17138_310};	/*Property = History*/
const Value V17138_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17138_314 = {S31, 0, &V17138_314_57261, &P17138_312};	/*Property = Html5*/
const Value V17138_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17138_313 = {S32, 0, &V17138_313_57258, &P17138_314};	/*Property = Html-Media-Capture*/
const Value V17138_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17138_315 = {S33, 0, &V17138_315_57264, &P17138_313};	/*Property = Iframe*/
const Value V17138_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17138_316 = {S34, 0, &V17138_316_57267, &P17138_315};	/*Property = IndexedDB*/
const Value V17138_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17138_317 = {S35, 0, &V17138_317_57269, &P17138_316};	/*Property = Json*/
const Value V17138_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17138_15 = {S37, 0, &V17138_15_28, &P17138_317};	/*Property = LayoutEngine*/
const Value V17138_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17138_344 = {S38, 0, &V17138_344_57272, &P17138_15};	/*Property = Masking*/
const Value V17138_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17138_320 = {S39, 0, &V17138_320_57290, &P17138_344};	/*Property = PostMessage*/
const Value V17138_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17138_321 = {S40, 0, &V17138_321_57293, &P17138_320};	/*Property = Progress*/
const Value V17138_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17138_322 = {S41, 0, &V17138_322_57296, &P17138_321};	/*Property = Prompts*/
const Value V17138_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17138_323 = {S42, 0, &V17138_323_57299, &P17138_322};	/*Property = Selector*/
const Value V17138_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17138_324 = {S43, 0, &V17138_324_57306, &P17138_323};	/*Property = Svg*/
const Value V17138_327_57314 = {S2, NULL};	/*Value = True*/
const Property P17138_327 = {S44, 0, &V17138_327_57314, &P17138_324};	/*Property = TouchEvents*/
const Value V17138_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17138_328 = {S45, 0, &V17138_328_57318, &P17138_327};	/*Property = Track*/
const Value V17138_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17138_329 = {S46, 0, &V17138_329_57323, &P17138_328};	/*Property = Video*/
const Value V17138_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17138_330 = {S47, 0, &V17138_330_57326, &P17138_329};	/*Property = Viewport*/
const Value V17138_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17138_332 = {S48, 0, &V17138_332_57339, &P17138_330};	/*Property = WebWorkers*/
const Value V17138_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17138_333 = {S49, 0, &V17138_333_57342, &P17138_332};	/*Property = Xhr2*/
const Profile P17138 = {"17138", &P17138_333};	/*Profile ID - 17138*/
const Profile P17145 = {"17145", NULL};	/*Profile ID - 17145*/
const Value V17596_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17596_279 = {S1, 0, &V17596_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17596_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17596_282 = {S3, 0, &V17596_282_57154, &P17596_279};	/*Property = BlobBuilder*/
const Value V17596_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17596_350 = {S4, 0, &V17596_350_57444, &P17596_282};	/*Property = Canvas*/
const Value V17596_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17596_283 = {S5, 0, &V17596_283_57159, &P17596_350};	/*Property = CssBackground*/
const Value V17596_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17596_284 = {S6, 0, &V17596_284_57163, &P17596_283};	/*Property = CssBorderImage*/
const Value V17596_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17596_285 = {S7, 0, &V17596_285_57166, &P17596_284};	/*Property = CssCanvas*/
const Value V17596_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17596_286 = {S8, 0, &V17596_286_57168, &P17596_285};	/*Property = CssColor*/
const Value V17596_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17596_237 = {S9, 0, &V17596_237_55786, &P17596_286};	/*Property = CssColumn*/
const Value V17596_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17596_287 = {S10, 0, &V17596_287_57172, &P17596_237};	/*Property = CssFlexbox*/
const Value V17596_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17596_288 = {S11, 0, &V17596_288_57175, &P17596_287};	/*Property = CssFont*/
const Value V17596_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17596_289 = {S12, 0, &V17596_289_57759, &P17596_288};	/*Property = CssImages*/
const Value V17596_291_57180 = {S2, NULL};	/*Value = True*/
const Property P17596_291 = {S13, 0, &V17596_291_57180, &P17596_289};	/*Property = CssMediaQueries*/
const Value V17596_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17596_292 = {S14, 0, &V17596_292_57183, &P17596_291};	/*Property = CssMinMax*/
const Value V17596_293_57186 = {S2, NULL};	/*Value = True*/
const Property P17596_293 = {S15, 0, &V17596_293_57186, &P17596_292};	/*Property = CssOverflow*/
const Value V17596_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17596_294 = {S16, 0, &V17596_294_57189, &P17596_293};	/*Property = CssPosition*/
const Value V17596_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17596_295 = {S17, 0, &V17596_295_57192, &P17596_294};	/*Property = CssText*/
const Value V17596_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17596_297 = {S18, 0, &V17596_297_57195, &P17596_295};	/*Property = CssTransforms*/
const Value V17596_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17596_241 = {S19, 0, &V17596_241_55799, &P17596_297};	/*Property = CssTransitions*/
const Value V17596_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17596_298 = {S20, 0, &V17596_298_57206, &P17596_241};	/*Property = CssUI*/
const Value V17596_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17596_300 = {S21, 0, &V17596_300_57216, &P17596_298};	/*Property = DataSet*/
const Value V17596_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17596_301 = {S22, 0, &V17596_301_57218, &P17596_300};	/*Property = DataUrl*/
const Value V17596_302_57221 = {S2, NULL};	/*Value = True*/
const Property P17596_302 = {S23, 0, &V17596_302_57221, &P17596_301};	/*Property = DeviceOrientation*/
const Value V17596_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17596_304 = {S24, 0, &V17596_304_57228, &P17596_302};	/*Property = FileReader*/
const Value V17596_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17596_305 = {S25, 0, &V17596_305_57231, &P17596_304};	/*Property = FileSaver*/
const Value V17596_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17596_306 = {S26, 0, &V17596_306_57237, &P17596_305};	/*Property = FileWriter*/
const Value V17596_307_57239 = {S2, NULL};	/*Value = True*/
const Property P17596_307 = {S27, 0, &V17596_307_57239, &P17596_306};	/*Property = FormData*/
const Value V17596_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17596_309 = {S28, 0, &V17596_309_57246, &P17596_307};	/*Property = Fullscreen*/
const Value V17596_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17596_310 = {S29, 0, &V17596_310_57248, &P17596_309};	/*Property = GeoLocation*/
const Value V17596_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17596_312 = {S30, 0, &V17596_312_57255, &P17596_310};	/*Property = History*/
const Value V17596_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17596_314 = {S31, 0, &V17596_314_57261, &P17596_312};	/*Property = Html5*/
const Value V17596_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17596_313 = {S32, 0, &V17596_313_57258, &P17596_314};	/*Property = Html-Media-Capture*/
const Value V17596_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17596_315 = {S33, 0, &V17596_315_57264, &P17596_313};	/*Property = Iframe*/
const Value V17596_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17596_316 = {S34, 0, &V17596_316_57267, &P17596_315};	/*Property = IndexedDB*/
const Value V17596_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17596_317 = {S35, 0, &V17596_317_57269, &P17596_316};	/*Property = Json*/
const Value V17596_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17596_15 = {S37, 0, &V17596_15_28, &P17596_317};	/*Property = LayoutEngine*/
const Value V17596_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17596_344 = {S38, 0, &V17596_344_57272, &P17596_15};	/*Property = Masking*/
const Value V17596_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17596_320 = {S39, 0, &V17596_320_57290, &P17596_344};	/*Property = PostMessage*/
const Value V17596_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17596_321 = {S40, 0, &V17596_321_57293, &P17596_320};	/*Property = Progress*/
const Value V17596_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17596_322 = {S41, 0, &V17596_322_57296, &P17596_321};	/*Property = Prompts*/
const Value V17596_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17596_323 = {S42, 0, &V17596_323_57299, &P17596_322};	/*Property = Selector*/
const Value V17596_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17596_324 = {S43, 0, &V17596_324_57306, &P17596_323};	/*Property = Svg*/
const Value V17596_327_57314 = {S2, NULL};	/*Value = True*/
const Property P17596_327 = {S44, 0, &V17596_327_57314, &P17596_324};	/*Property = TouchEvents*/
const Value V17596_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17596_328 = {S45, 0, &V17596_328_57318, &P17596_327};	/*Property = Track*/
const Value V17596_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17596_329 = {S46, 0, &V17596_329_57323, &P17596_328};	/*Property = Video*/
const Value V17596_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17596_330 = {S47, 0, &V17596_330_57326, &P17596_329};	/*Property = Viewport*/
const Value V17596_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17596_332 = {S48, 0, &V17596_332_57339, &P17596_330};	/*Property = WebWorkers*/
const Value V17596_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17596_333 = {S49, 0, &V17596_333_57342, &P17596_332};	/*Property = Xhr2*/
const Profile P17596 = {"17596", &P17596_333};	/*Profile ID - 17596*/
const Value V22701_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22701_279 = {S1, 0, &V22701_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22701_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22701_282 = {S3, 0, &V22701_282_57154, &P22701_279};	/*Property = BlobBuilder*/
const Value V22701_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22701_350 = {S4, 0, &V22701_350_57444, &P22701_282};	/*Property = Canvas*/
const Value V22701_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22701_283 = {S5, 0, &V22701_283_57159, &P22701_350};	/*Property = CssBackground*/
const Value V22701_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22701_284 = {S6, 0, &V22701_284_57162, &P22701_283};	/*Property = CssBorderImage*/
const Value V22701_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22701_285 = {S7, 0, &V22701_285_57166, &P22701_284};	/*Property = CssCanvas*/
const Value V22701_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22701_286 = {S8, 0, &V22701_286_57168, &P22701_285};	/*Property = CssColor*/
const Value V22701_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22701_237 = {S9, 0, &V22701_237_55786, &P22701_286};	/*Property = CssColumn*/
const Value V22701_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22701_287 = {S10, 0, &V22701_287_57172, &P22701_237};	/*Property = CssFlexbox*/
const Value V22701_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22701_288 = {S11, 0, &V22701_288_57174, &P22701_287};	/*Property = CssFont*/
const Value V22701_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22701_289 = {S12, 0, &V22701_289_57759, &P22701_288};	/*Property = CssImages*/
const Value V22701_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22701_291 = {S13, 0, &V22701_291_57180, &P22701_289};	/*Property = CssMediaQueries*/
const Value V22701_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22701_292 = {S14, 0, &V22701_292_57183, &P22701_291};	/*Property = CssMinMax*/
const Value V22701_293_57186 = {S2, NULL};	/*Value = True*/
const Property P22701_293 = {S15, 0, &V22701_293_57186, &P22701_292};	/*Property = CssOverflow*/
const Value V22701_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22701_294 = {S16, 0, &V22701_294_57189, &P22701_293};	/*Property = CssPosition*/
const Value V22701_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22701_295 = {S17, 0, &V22701_295_57192, &P22701_294};	/*Property = CssText*/
const Value V22701_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22701_297 = {S18, 0, &V22701_297_57195, &P22701_295};	/*Property = CssTransforms*/
const Value V22701_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22701_241 = {S19, 0, &V22701_241_55799, &P22701_297};	/*Property = CssTransitions*/
const Value V22701_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22701_298 = {S20, 0, &V22701_298_57206, &P22701_241};	/*Property = CssUI*/
const Value V22701_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22701_300 = {S21, 0, &V22701_300_57215, &P22701_298};	/*Property = DataSet*/
const Value V22701_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22701_301 = {S22, 0, &V22701_301_57218, &P22701_300};	/*Property = DataUrl*/
const Value V22701_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22701_302 = {S23, 0, &V22701_302_57221, &P22701_301};	/*Property = DeviceOrientation*/
const Value V22701_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22701_304 = {S24, 0, &V22701_304_57227, &P22701_302};	/*Property = FileReader*/
const Value V22701_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22701_305 = {S25, 0, &V22701_305_57231, &P22701_304};	/*Property = FileSaver*/
const Value V22701_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22701_306 = {S26, 0, &V22701_306_57237, &P22701_305};	/*Property = FileWriter*/
const Value V22701_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22701_307 = {S27, 0, &V22701_307_57239, &P22701_306};	/*Property = FormData*/
const Value V22701_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22701_309 = {S28, 0, &V22701_309_57246, &P22701_307};	/*Property = Fullscreen*/
const Value V22701_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22701_310 = {S29, 0, &V22701_310_57248, &P22701_309};	/*Property = GeoLocation*/
const Value V22701_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22701_312 = {S30, 0, &V22701_312_57254, &P22701_310};	/*Property = History*/
const Value V22701_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22701_314 = {S31, 0, &V22701_314_57260, &P22701_312};	/*Property = Html5*/
const Value V22701_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22701_313 = {S32, 0, &V22701_313_57258, &P22701_314};	/*Property = Html-Media-Capture*/
const Value V22701_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22701_315 = {S33, 0, &V22701_315_57264, &P22701_313};	/*Property = Iframe*/
const Value V22701_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22701_316 = {S34, 0, &V22701_316_57267, &P22701_315};	/*Property = IndexedDB*/
const Value V22701_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22701_317 = {S35, 0, &V22701_317_57269, &P22701_316};	/*Property = Json*/
const Value V22701_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22701_15 = {S37, 0, &V22701_15_72, &P22701_317};	/*Property = LayoutEngine*/
const Value V22701_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22701_344 = {S38, 0, &V22701_344_57272, &P22701_15};	/*Property = Masking*/
const Value V22701_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22701_320 = {S39, 0, &V22701_320_57290, &P22701_344};	/*Property = PostMessage*/
const Value V22701_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22701_321 = {S40, 0, &V22701_321_57293, &P22701_320};	/*Property = Progress*/
const Value V22701_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22701_322 = {S41, 0, &V22701_322_57296, &P22701_321};	/*Property = Prompts*/
const Value V22701_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22701_323 = {S42, 0, &V22701_323_57299, &P22701_322};	/*Property = Selector*/
const Value V22701_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22701_324 = {S43, 0, &V22701_324_57305, &P22701_323};	/*Property = Svg*/
const Value V22701_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22701_327 = {S44, 0, &V22701_327_57314, &P22701_324};	/*Property = TouchEvents*/
const Value V22701_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22701_328 = {S45, 0, &V22701_328_57318, &P22701_327};	/*Property = Track*/
const Value V22701_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22701_329 = {S46, 0, &V22701_329_57323, &P22701_328};	/*Property = Video*/
const Value V22701_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22701_330 = {S47, 0, &V22701_330_57326, &P22701_329};	/*Property = Viewport*/
const Value V22701_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22701_332 = {S48, 0, &V22701_332_57338, &P22701_330};	/*Property = WebWorkers*/
const Value V22701_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22701_333 = {S49, 0, &V22701_333_57342, &P22701_332};	/*Property = Xhr2*/
const Profile P22701 = {"22701", &P22701_333};	/*Profile ID - 22701*/
const Value V17346_1_1 = {S2, NULL};	/*Value = True*/
const Property P17346_1 = {S50, 0, &V17346_1_1, NULL};	/*Property = IsMobile*/
const Value V17346_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P17346_5 = {S52, 0, &V17346_5_1403, &P17346_1};	/*Property = ScreenPixelsHeight*/
const Value V17346_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P17346_4 = {S54, 0, &V17346_4_8554, &P17346_5};	/*Property = ScreenPixelsWidth*/
const Profile P17346 = {"17346", &P17346_4};	/*Profile ID - 17346*/
const Value V18215_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_279 = {S1, 0, &V18215_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18215_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_282 = {S3, 0, &V18215_282_57155, &P18215_279};	/*Property = BlobBuilder*/
const Value V18215_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_350 = {S4, 0, &V18215_350_57446, &P18215_282};	/*Property = Canvas*/
const Value V18215_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_283 = {S5, 0, &V18215_283_57161, &P18215_350};	/*Property = CssBackground*/
const Value V18215_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_284 = {S6, 0, &V18215_284_57164, &P18215_283};	/*Property = CssBorderImage*/
const Value V18215_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_285 = {S7, 0, &V18215_285_57167, &P18215_284};	/*Property = CssCanvas*/
const Value V18215_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_286 = {S8, 0, &V18215_286_57170, &P18215_285};	/*Property = CssColor*/
const Value V18215_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_237 = {S9, 0, &V18215_237_55786, &P18215_286};	/*Property = CssColumn*/
const Value V18215_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_287 = {S10, 0, &V18215_287_57173, &P18215_237};	/*Property = CssFlexbox*/
const Value V18215_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_288 = {S11, 0, &V18215_288_57176, &P18215_287};	/*Property = CssFont*/
const Value V18215_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_289 = {S12, 0, &V18215_289_86899, &P18215_288};	/*Property = CssImages*/
const Value V18215_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_291 = {S13, 0, &V18215_291_57182, &P18215_289};	/*Property = CssMediaQueries*/
const Value V18215_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_292 = {S14, 0, &V18215_292_57185, &P18215_291};	/*Property = CssMinMax*/
const Value V18215_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_293 = {S15, 0, &V18215_293_57188, &P18215_292};	/*Property = CssOverflow*/
const Value V18215_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_294 = {S16, 0, &V18215_294_57191, &P18215_293};	/*Property = CssPosition*/
const Value V18215_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_295 = {S17, 0, &V18215_295_57194, &P18215_294};	/*Property = CssText*/
const Value V18215_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_297 = {S18, 0, &V18215_297_57197, &P18215_295};	/*Property = CssTransforms*/
const Value V18215_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_241 = {S19, 0, &V18215_241_55801, &P18215_297};	/*Property = CssTransitions*/
const Value V18215_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_298 = {S20, 0, &V18215_298_57208, &P18215_241};	/*Property = CssUI*/
const Value V18215_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_300 = {S21, 0, &V18215_300_57217, &P18215_298};	/*Property = DataSet*/
const Value V18215_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_301 = {S22, 0, &V18215_301_57220, &P18215_300};	/*Property = DataUrl*/
const Value V18215_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_302 = {S23, 0, &V18215_302_57223, &P18215_301};	/*Property = DeviceOrientation*/
const Value V18215_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_304 = {S24, 0, &V18215_304_57229, &P18215_302};	/*Property = FileReader*/
const Value V18215_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_305 = {S25, 0, &V18215_305_57232, &P18215_304};	/*Property = FileSaver*/
const Value V18215_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_306 = {S26, 0, &V18215_306_57238, &P18215_305};	/*Property = FileWriter*/
const Value V18215_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_307 = {S27, 0, &V18215_307_57241, &P18215_306};	/*Property = FormData*/
const Value V18215_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_309 = {S28, 0, &V18215_309_57247, &P18215_307};	/*Property = Fullscreen*/
const Value V18215_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_310 = {S29, 0, &V18215_310_57250, &P18215_309};	/*Property = GeoLocation*/
const Value V18215_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_312 = {S30, 0, &V18215_312_57256, &P18215_310};	/*Property = History*/
const Value V18215_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_314 = {S31, 0, &V18215_314_57262, &P18215_312};	/*Property = Html5*/
const Value V18215_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_313 = {S32, 0, &V18215_313_57259, &P18215_314};	/*Property = Html-Media-Capture*/
const Value V18215_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_315 = {S33, 0, &V18215_315_57265, &P18215_313};	/*Property = Iframe*/
const Value V18215_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_316 = {S34, 0, &V18215_316_57268, &P18215_315};	/*Property = IndexedDB*/
const Value V18215_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_317 = {S35, 0, &V18215_317_57271, &P18215_316};	/*Property = Json*/
const Value V18215_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_15 = {S37, 0, &V18215_15_72, &P18215_317};	/*Property = LayoutEngine*/
const Value V18215_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_344 = {S38, 0, &V18215_344_57274, &P18215_15};	/*Property = Masking*/
const Value V18215_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_320 = {S39, 0, &V18215_320_57292, &P18215_344};	/*Property = PostMessage*/
const Value V18215_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_321 = {S40, 0, &V18215_321_57295, &P18215_320};	/*Property = Progress*/
const Value V18215_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_322 = {S41, 0, &V18215_322_57298, &P18215_321};	/*Property = Prompts*/
const Value V18215_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_323 = {S42, 0, &V18215_323_57301, &P18215_322};	/*Property = Selector*/
const Value V18215_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_324 = {S43, 0, &V18215_324_57307, &P18215_323};	/*Property = Svg*/
const Value V18215_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_327 = {S44, 0, &V18215_327_57316, &P18215_324};	/*Property = TouchEvents*/
const Value V18215_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_328 = {S45, 0, &V18215_328_57319, &P18215_327};	/*Property = Track*/
const Value V18215_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_329 = {S46, 0, &V18215_329_57325, &P18215_328};	/*Property = Video*/
const Value V18215_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_330 = {S47, 0, &V18215_330_57328, &P18215_329};	/*Property = Viewport*/
const Value V18215_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_332 = {S48, 0, &V18215_332_57340, &P18215_330};	/*Property = WebWorkers*/
const Value V18215_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18215_333 = {S49, 0, &V18215_333_57343, &P18215_332};	/*Property = Xhr2*/
const Profile P18215 = {"18215", &P18215_333};	/*Profile ID - 18215*/
const Value V4201_279_57148 = {S0, NULL};	/*Value = False*/
const Property P4201_279 = {S1, 0, &V4201_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V4201_282_57154 = {S0, NULL};	/*Value = False*/
const Property P4201_282 = {S3, 0, &V4201_282_57154, &P4201_279};	/*Property = BlobBuilder*/
const Value V4201_350_57444 = {S2, NULL};	/*Value = True*/
const Property P4201_350 = {S4, 0, &V4201_350_57444, &P4201_282};	/*Property = Canvas*/
const Value V4201_283_57159 = {S2, NULL};	/*Value = True*/
const Property P4201_283 = {S5, 0, &V4201_283_57159, &P4201_350};	/*Property = CssBackground*/
const Value V4201_284_57163 = {S0, NULL};	/*Value = False*/
const Property P4201_284 = {S6, 0, &V4201_284_57163, &P4201_283};	/*Property = CssBorderImage*/
const Value V4201_285_57166 = {S0, NULL};	/*Value = False*/
const Property P4201_285 = {S7, 0, &V4201_285_57166, &P4201_284};	/*Property = CssCanvas*/
const Value V4201_286_57168 = {S2, NULL};	/*Value = True*/
const Property P4201_286 = {S8, 0, &V4201_286_57168, &P4201_285};	/*Property = CssColor*/
const Value V4201_237_55784 = {S2, NULL};	/*Value = True*/
const Property P4201_237 = {S9, 0, &V4201_237_55784, &P4201_286};	/*Property = CssColumn*/
const Value V4201_287_57172 = {S0, NULL};	/*Value = False*/
const Property P4201_287 = {S10, 0, &V4201_287_57172, &P4201_237};	/*Property = CssFlexbox*/
const Value V4201_288_57175 = {S0, NULL};	/*Value = False*/
const Property P4201_288 = {S11, 0, &V4201_288_57175, &P4201_287};	/*Property = CssFont*/
const Value V4201_289_57759 = {S0, NULL};	/*Value = False*/
const Property P4201_289 = {S12, 0, &V4201_289_57759, &P4201_288};	/*Property = CssImages*/
const Value V4201_291_57181 = {S0, NULL};	/*Value = False*/
const Property P4201_291 = {S13, 0, &V4201_291_57181, &P4201_289};	/*Property = CssMediaQueries*/
const Value V4201_292_57183 = {S2, NULL};	/*Value = True*/
const Property P4201_292 = {S14, 0, &V4201_292_57183, &P4201_291};	/*Property = CssMinMax*/
const Value V4201_293_57187 = {S0, NULL};	/*Value = False*/
const Property P4201_293 = {S15, 0, &V4201_293_57187, &P4201_292};	/*Property = CssOverflow*/
const Value V4201_294_57189 = {S2, NULL};	/*Value = True*/
const Property P4201_294 = {S16, 0, &V4201_294_57189, &P4201_293};	/*Property = CssPosition*/
const Value V4201_295_57192 = {S2, NULL};	/*Value = True*/
const Property P4201_295 = {S17, 0, &V4201_295_57192, &P4201_294};	/*Property = CssText*/
const Value V4201_297_57195 = {S2, NULL};	/*Value = True*/
const Property P4201_297 = {S18, 0, &V4201_297_57195, &P4201_295};	/*Property = CssTransforms*/
const Value V4201_241_55799 = {S2, NULL};	/*Value = True*/
const Property P4201_241 = {S19, 0, &V4201_241_55799, &P4201_297};	/*Property = CssTransitions*/
const Value V4201_298_57206 = {S2, NULL};	/*Value = True*/
const Property P4201_298 = {S20, 0, &V4201_298_57206, &P4201_241};	/*Property = CssUI*/
const Value V4201_300_57216 = {S0, NULL};	/*Value = False*/
const Property P4201_300 = {S21, 0, &V4201_300_57216, &P4201_298};	/*Property = DataSet*/
const Value V4201_301_57218 = {S2, NULL};	/*Value = True*/
const Property P4201_301 = {S22, 0, &V4201_301_57218, &P4201_300};	/*Property = DataUrl*/
const Value V4201_302_57222 = {S0, NULL};	/*Value = False*/
const Property P4201_302 = {S23, 0, &V4201_302_57222, &P4201_301};	/*Property = DeviceOrientation*/
const Value V4201_304_57228 = {S0, NULL};	/*Value = False*/
const Property P4201_304 = {S24, 0, &V4201_304_57228, &P4201_302};	/*Property = FileReader*/
const Value V4201_305_57231 = {S0, NULL};	/*Value = False*/
const Property P4201_305 = {S25, 0, &V4201_305_57231, &P4201_304};	/*Property = FileSaver*/
const Value V4201_306_57237 = {S0, NULL};	/*Value = False*/
const Property P4201_306 = {S26, 0, &V4201_306_57237, &P4201_305};	/*Property = FileWriter*/
const Value V4201_307_57240 = {S0, NULL};	/*Value = False*/
const Property P4201_307 = {S27, 0, &V4201_307_57240, &P4201_306};	/*Property = FormData*/
const Value V4201_309_57246 = {S0, NULL};	/*Value = False*/
const Property P4201_309 = {S28, 0, &V4201_309_57246, &P4201_307};	/*Property = Fullscreen*/
const Value V4201_310_57248 = {S2, NULL};	/*Value = True*/
const Property P4201_310 = {S29, 0, &V4201_310_57248, &P4201_309};	/*Property = GeoLocation*/
const Value V4201_312_57255 = {S0, NULL};	/*Value = False*/
const Property P4201_312 = {S30, 0, &V4201_312_57255, &P4201_310};	/*Property = History*/
const Value V4201_314_57261 = {S0, NULL};	/*Value = False*/
const Property P4201_314 = {S31, 0, &V4201_314_57261, &P4201_312};	/*Property = Html5*/
const Value V4201_313_57258 = {S0, NULL};	/*Value = False*/
const Property P4201_313 = {S32, 0, &V4201_313_57258, &P4201_314};	/*Property = Html-Media-Capture*/
const Value V4201_315_57264 = {S0, NULL};	/*Value = False*/
const Property P4201_315 = {S33, 0, &V4201_315_57264, &P4201_313};	/*Property = Iframe*/
const Value V4201_316_57267 = {S0, NULL};	/*Value = False*/
const Property P4201_316 = {S34, 0, &V4201_316_57267, &P4201_315};	/*Property = IndexedDB*/
const Value V4201_317_57269 = {S2, NULL};	/*Value = True*/
const Property P4201_317 = {S35, 0, &V4201_317_57269, &P4201_316};	/*Property = Json*/
const Value V4201_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P4201_15 = {S37, 0, &V4201_15_28, &P4201_317};	/*Property = LayoutEngine*/
const Value V4201_344_57272 = {S2, NULL};	/*Value = True*/
const Property P4201_344 = {S38, 0, &V4201_344_57272, &P4201_15};	/*Property = Masking*/
const Value V4201_320_57290 = {S2, NULL};	/*Value = True*/
const Property P4201_320 = {S39, 0, &V4201_320_57290, &P4201_344};	/*Property = PostMessage*/
const Value V4201_321_57293 = {S2, NULL};	/*Value = True*/
const Property P4201_321 = {S40, 0, &V4201_321_57293, &P4201_320};	/*Property = Progress*/
const Value V4201_322_57296 = {S2, NULL};	/*Value = True*/
const Property P4201_322 = {S41, 0, &V4201_322_57296, &P4201_321};	/*Property = Prompts*/
const Value V4201_323_57299 = {S2, NULL};	/*Value = True*/
const Property P4201_323 = {S42, 0, &V4201_323_57299, &P4201_322};	/*Property = Selector*/
const Value V4201_324_57306 = {S0, NULL};	/*Value = False*/
const Property P4201_324 = {S43, 0, &V4201_324_57306, &P4201_323};	/*Property = Svg*/
const Value V4201_327_57315 = {S0, NULL};	/*Value = False*/
const Property P4201_327 = {S44, 0, &V4201_327_57315, &P4201_324};	/*Property = TouchEvents*/
const Value V4201_328_57318 = {S0, NULL};	/*Value = False*/
const Property P4201_328 = {S45, 0, &V4201_328_57318, &P4201_327};	/*Property = Track*/
const Value V4201_329_57323 = {S2, NULL};	/*Value = True*/
const Property P4201_329 = {S46, 0, &V4201_329_57323, &P4201_328};	/*Property = Video*/
const Value V4201_330_57326 = {S2, NULL};	/*Value = True*/
const Property P4201_330 = {S47, 0, &V4201_330_57326, &P4201_329};	/*Property = Viewport*/
const Value V4201_332_57339 = {S0, NULL};	/*Value = False*/
const Property P4201_332 = {S48, 0, &V4201_332_57339, &P4201_330};	/*Property = WebWorkers*/
const Value V4201_333_57342 = {S0, NULL};	/*Value = False*/
const Property P4201_333 = {S49, 0, &V4201_333_57342, &P4201_332};	/*Property = Xhr2*/
const Profile P4201 = {"4201", &P4201_333};	/*Profile ID - 4201*/
const Value V12376_1_1 = {S2, NULL};	/*Value = True*/
const Property P12376_1 = {S50, 0, &V12376_1_1, NULL};	/*Property = IsMobile*/
const Value V12376_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12376_5 = {S52, 0, &V12376_5_1293, &P12376_1};	/*Property = ScreenPixelsHeight*/
const Value V12376_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12376_4 = {S54, 0, &V12376_4_1296, &P12376_5};	/*Property = ScreenPixelsWidth*/
const Profile P12376 = {"12376", &P12376_4};	/*Profile ID - 12376*/
const Value V12287_1_1 = {S2, NULL};	/*Value = True*/
const Property P12287_1 = {S50, 0, &V12287_1_1, NULL};	/*Property = IsMobile*/
const Value V12287_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12287_5 = {S52, 0, &V12287_5_1222, &P12287_1};	/*Property = ScreenPixelsHeight*/
const Value V12287_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12287_4 = {S54, 0, &V12287_4_1224, &P12287_5};	/*Property = ScreenPixelsWidth*/
const Profile P12287 = {"12287", &P12287_4};	/*Profile ID - 12287*/
const Value V12051_1_1 = {S2, NULL};	/*Value = True*/
const Property P12051_1 = {S50, 0, &V12051_1_1, NULL};	/*Property = IsMobile*/
const Value V12051_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12051_5 = {S52, 0, &V12051_5_1222, &P12051_1};	/*Property = ScreenPixelsHeight*/
const Value V12051_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12051_4 = {S54, 0, &V12051_4_1224, &P12051_5};	/*Property = ScreenPixelsWidth*/
const Profile P12051 = {"12051", &P12051_4};	/*Profile ID - 12051*/
const Value V12289_1_1 = {S2, NULL};	/*Value = True*/
const Property P12289_1 = {S50, 0, &V12289_1_1, NULL};	/*Property = IsMobile*/
const Value V12289_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12289_5 = {S52, 0, &V12289_5_1293, &P12289_1};	/*Property = ScreenPixelsHeight*/
const Value V12289_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12289_4 = {S54, 0, &V12289_4_1296, &P12289_5};	/*Property = ScreenPixelsWidth*/
const Profile P12289 = {"12289", &P12289_4};	/*Profile ID - 12289*/
const Value V12290_1_1 = {S2, NULL};	/*Value = True*/
const Property P12290_1 = {S50, 0, &V12290_1_1, NULL};	/*Property = IsMobile*/
const Value V12290_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P12290_5 = {S52, 0, &V12290_5_1269, &P12290_1};	/*Property = ScreenPixelsHeight*/
const Value V12290_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12290_4 = {S54, 0, &V12290_4_1229, &P12290_5};	/*Property = ScreenPixelsWidth*/
const Profile P12290 = {"12290", &P12290_4};	/*Profile ID - 12290*/
const Value V12292_1_1 = {S2, NULL};	/*Value = True*/
const Property P12292_1 = {S50, 0, &V12292_1_1, NULL};	/*Property = IsMobile*/
const Value V12292_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12292_5 = {S52, 0, &V12292_5_1222, &P12292_1};	/*Property = ScreenPixelsHeight*/
const Value V12292_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12292_4 = {S54, 0, &V12292_4_1224, &P12292_5};	/*Property = ScreenPixelsWidth*/
const Profile P12292 = {"12292", &P12292_4};	/*Profile ID - 12292*/
const Profile P6173 = {"6173", NULL};	/*Profile ID - 6173*/
const Value V12299_1_1 = {S2, NULL};	/*Value = True*/
const Property P12299_1 = {S50, 0, &V12299_1_1, NULL};	/*Property = IsMobile*/
const Value V12299_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12299_5 = {S52, 0, &V12299_5_1293, &P12299_1};	/*Property = ScreenPixelsHeight*/
const Value V12299_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12299_4 = {S54, 0, &V12299_4_1296, &P12299_5};	/*Property = ScreenPixelsWidth*/
const Profile P12299 = {"12299", &P12299_4};	/*Profile ID - 12299*/
const Value V12144_1_1 = {S2, NULL};	/*Value = True*/
const Property P12144_1 = {S50, 0, &V12144_1_1, NULL};	/*Property = IsMobile*/
const Value V12144_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12144_5 = {S52, 0, &V12144_5_1277, &P12144_1};	/*Property = ScreenPixelsHeight*/
const Value V12144_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12144_4 = {S54, 0, &V12144_4_1279, &P12144_5};	/*Property = ScreenPixelsWidth*/
const Profile P12144 = {"12144", &P12144_4};	/*Profile ID - 12144*/
const Value V12303_1_1 = {S2, NULL};	/*Value = True*/
const Property P12303_1 = {S50, 0, &V12303_1_1, NULL};	/*Property = IsMobile*/
const Value V12303_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12303_5 = {S52, 0, &V12303_5_1222, &P12303_1};	/*Property = ScreenPixelsHeight*/
const Value V12303_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12303_4 = {S54, 0, &V12303_4_1224, &P12303_5};	/*Property = ScreenPixelsWidth*/
const Profile P12303 = {"12303", &P12303_4};	/*Profile ID - 12303*/
const Value V12310_1_1 = {S2, NULL};	/*Value = True*/
const Property P12310_1 = {S50, 0, &V12310_1_1, NULL};	/*Property = IsMobile*/
const Value V12310_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12310_5 = {S52, 0, &V12310_5_1222, &P12310_1};	/*Property = ScreenPixelsHeight*/
const Value V12310_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12310_4 = {S54, 0, &V12310_4_1224, &P12310_5};	/*Property = ScreenPixelsWidth*/
const Profile P12310 = {"12310", &P12310_4};	/*Profile ID - 12310*/
const Value V12313_1_1 = {S2, NULL};	/*Value = True*/
const Property P12313_1 = {S50, 0, &V12313_1_1, NULL};	/*Property = IsMobile*/
const Value V12313_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12313_5 = {S52, 0, &V12313_5_1294, &P12313_1};	/*Property = ScreenPixelsHeight*/
const Value V12313_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12313_4 = {S54, 0, &V12313_4_1296, &P12313_5};	/*Property = ScreenPixelsWidth*/
const Profile P12313 = {"12313", &P12313_4};	/*Profile ID - 12313*/
const Value V12311_1_1 = {S2, NULL};	/*Value = True*/
const Property P12311_1 = {S50, 0, &V12311_1_1, NULL};	/*Property = IsMobile*/
const Value V12311_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12311_5 = {S52, 0, &V12311_5_1277, &P12311_1};	/*Property = ScreenPixelsHeight*/
const Value V12311_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12311_4 = {S54, 0, &V12311_4_1279, &P12311_5};	/*Property = ScreenPixelsWidth*/
const Profile P12311 = {"12311", &P12311_4};	/*Profile ID - 12311*/
const Value V12316_1_1 = {S2, NULL};	/*Value = True*/
const Property P12316_1 = {S50, 0, &V12316_1_1, NULL};	/*Property = IsMobile*/
const Value V12316_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12316_5 = {S52, 0, &V12316_5_1277, &P12316_1};	/*Property = ScreenPixelsHeight*/
const Value V12316_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12316_4 = {S54, 0, &V12316_4_1279, &P12316_5};	/*Property = ScreenPixelsWidth*/
const Profile P12316 = {"12316", &P12316_4};	/*Profile ID - 12316*/
const Value V13844_1_1 = {S2, NULL};	/*Value = True*/
const Property P13844_1 = {S50, 0, &V13844_1_1, NULL};	/*Property = IsMobile*/
const Value V13844_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13844_5 = {S52, 0, &V13844_5_1233, &P13844_1};	/*Property = ScreenPixelsHeight*/
const Value V13844_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13844_4 = {S54, 0, &V13844_4_1235, &P13844_5};	/*Property = ScreenPixelsWidth*/
const Profile P13844 = {"13844", &P13844_4};	/*Profile ID - 13844*/
const Value V14842_1_1 = {S2, NULL};	/*Value = True*/
const Property P14842_1 = {S50, 0, &V14842_1_1, NULL};	/*Property = IsMobile*/
const Value V14842_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14842_5 = {S52, 0, &V14842_5_1222, &P14842_1};	/*Property = ScreenPixelsHeight*/
const Value V14842_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14842_4 = {S54, 0, &V14842_4_1224, &P14842_5};	/*Property = ScreenPixelsWidth*/
const Profile P14842 = {"14842", &P14842_4};	/*Profile ID - 14842*/
const Value V12325_1_1 = {S2, NULL};	/*Value = True*/
const Property P12325_1 = {S50, 0, &V12325_1_1, NULL};	/*Property = IsMobile*/
const Value V12325_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12325_5 = {S52, 0, &V12325_5_1380, &P12325_1};	/*Property = ScreenPixelsHeight*/
const Value V12325_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12325_4 = {S54, 0, &V12325_4_1279, &P12325_5};	/*Property = ScreenPixelsWidth*/
const Profile P12325 = {"12325", &P12325_4};	/*Profile ID - 12325*/
const Value V14222_1_1 = {S2, NULL};	/*Value = True*/
const Property P14222_1 = {S50, 0, &V14222_1_1, NULL};	/*Property = IsMobile*/
const Value V14222_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14222_5 = {S52, 0, &V14222_5_1222, &P14222_1};	/*Property = ScreenPixelsHeight*/
const Value V14222_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14222_4 = {S54, 0, &V14222_4_1224, &P14222_5};	/*Property = ScreenPixelsWidth*/
const Profile P14222 = {"14222", &P14222_4};	/*Profile ID - 14222*/
const Value V14972_1_1 = {S2, NULL};	/*Value = True*/
const Property P14972_1 = {S50, 0, &V14972_1_1, NULL};	/*Property = IsMobile*/
const Value V14972_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14972_5 = {S52, 0, &V14972_5_1222, &P14972_1};	/*Property = ScreenPixelsHeight*/
const Value V14972_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14972_4 = {S54, 0, &V14972_4_1224, &P14972_5};	/*Property = ScreenPixelsWidth*/
const Profile P14972 = {"14972", &P14972_4};	/*Profile ID - 14972*/
const Value V12338_1_1 = {S2, NULL};	/*Value = True*/
const Property P12338_1 = {S50, 0, &V12338_1_1, NULL};	/*Property = IsMobile*/
const Value V12338_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12338_5 = {S52, 0, &V12338_5_1277, &P12338_1};	/*Property = ScreenPixelsHeight*/
const Value V12338_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12338_4 = {S54, 0, &V12338_4_1279, &P12338_5};	/*Property = ScreenPixelsWidth*/
const Profile P12338 = {"12338", &P12338_4};	/*Profile ID - 12338*/
const Value V16745_1_1 = {S2, NULL};	/*Value = True*/
const Property P16745_1 = {S50, 0, &V16745_1_1, NULL};	/*Property = IsMobile*/
const Value V16745_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16745_5 = {S52, 0, &V16745_5_1277, &P16745_1};	/*Property = ScreenPixelsHeight*/
const Value V16745_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16745_4 = {S54, 0, &V16745_4_1279, &P16745_5};	/*Property = ScreenPixelsWidth*/
const Profile P16745 = {"16745", &P16745_4};	/*Profile ID - 16745*/
const Value V12342_1_1 = {S2, NULL};	/*Value = True*/
const Property P12342_1 = {S50, 0, &V12342_1_1, NULL};	/*Property = IsMobile*/
const Value V12342_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12342_5 = {S52, 0, &V12342_5_1222, &P12342_1};	/*Property = ScreenPixelsHeight*/
const Value V12342_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12342_4 = {S54, 0, &V12342_4_1224, &P12342_5};	/*Property = ScreenPixelsWidth*/
const Profile P12342 = {"12342", &P12342_4};	/*Profile ID - 12342*/
const Value V12332_1_1 = {S2, NULL};	/*Value = True*/
const Property P12332_1 = {S50, 0, &V12332_1_1, NULL};	/*Property = IsMobile*/
const Value V12332_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12332_5 = {S52, 0, &V12332_5_1293, &P12332_1};	/*Property = ScreenPixelsHeight*/
const Value V12332_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12332_4 = {S54, 0, &V12332_4_1296, &P12332_5};	/*Property = ScreenPixelsWidth*/
const Profile P12332 = {"12332", &P12332_4};	/*Profile ID - 12332*/
const Value V12346_1_1 = {S2, NULL};	/*Value = True*/
const Property P12346_1 = {S50, 0, &V12346_1_1, NULL};	/*Property = IsMobile*/
const Value V12346_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12346_5 = {S52, 0, &V12346_5_1227, &P12346_1};	/*Property = ScreenPixelsHeight*/
const Value V12346_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12346_4 = {S54, 0, &V12346_4_1229, &P12346_5};	/*Property = ScreenPixelsWidth*/
const Profile P12346 = {"12346", &P12346_4};	/*Profile ID - 12346*/
const Value V12347_1_1 = {S2, NULL};	/*Value = True*/
const Property P12347_1 = {S50, 0, &V12347_1_1, NULL};	/*Property = IsMobile*/
const Value V12347_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12347_5 = {S52, 0, &V12347_5_1222, &P12347_1};	/*Property = ScreenPixelsHeight*/
const Value V12347_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12347_4 = {S54, 0, &V12347_4_1224, &P12347_5};	/*Property = ScreenPixelsWidth*/
const Profile P12347 = {"12347", &P12347_4};	/*Profile ID - 12347*/
const Value V13535_1_1 = {S2, NULL};	/*Value = True*/
const Property P13535_1 = {S50, 0, &V13535_1_1, NULL};	/*Property = IsMobile*/
const Value V13535_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13535_5 = {S52, 0, &V13535_5_1222, &P13535_1};	/*Property = ScreenPixelsHeight*/
const Value V13535_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13535_4 = {S54, 0, &V13535_4_1224, &P13535_5};	/*Property = ScreenPixelsWidth*/
const Profile P13535 = {"13535", &P13535_4};	/*Profile ID - 13535*/
const Value V14559_1_1 = {S2, NULL};	/*Value = True*/
const Property P14559_1 = {S50, 0, &V14559_1_1, NULL};	/*Property = IsMobile*/
const Value V14559_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14559_5 = {S52, 0, &V14559_5_1293, &P14559_1};	/*Property = ScreenPixelsHeight*/
const Value V14559_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14559_4 = {S54, 0, &V14559_4_1296, &P14559_5};	/*Property = ScreenPixelsWidth*/
const Profile P14559 = {"14559", &P14559_4};	/*Profile ID - 14559*/
const Value V12369_1_1 = {S2, NULL};	/*Value = True*/
const Property P12369_1 = {S50, 0, &V12369_1_1, NULL};	/*Property = IsMobile*/
const Value V12369_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12369_5 = {S52, 0, &V12369_5_1222, &P12369_1};	/*Property = ScreenPixelsHeight*/
const Value V12369_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12369_4 = {S54, 0, &V12369_4_1224, &P12369_5};	/*Property = ScreenPixelsWidth*/
const Profile P12369 = {"12369", &P12369_4};	/*Profile ID - 12369*/
const Value V12371_1_1 = {S2, NULL};	/*Value = True*/
const Property P12371_1 = {S50, 0, &V12371_1_1, NULL};	/*Property = IsMobile*/
const Value V12371_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P12371_5 = {S52, 0, &V12371_5_1269, &P12371_1};	/*Property = ScreenPixelsHeight*/
const Value V12371_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12371_4 = {S54, 0, &V12371_4_1229, &P12371_5};	/*Property = ScreenPixelsWidth*/
const Profile P12371 = {"12371", &P12371_4};	/*Profile ID - 12371*/
const Value V12380_1_1 = {S2, NULL};	/*Value = True*/
const Property P12380_1 = {S50, 0, &V12380_1_1, NULL};	/*Property = IsMobile*/
const Value V12380_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12380_5 = {S52, 0, &V12380_5_1293, &P12380_1};	/*Property = ScreenPixelsHeight*/
const Value V12380_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12380_4 = {S54, 0, &V12380_4_1296, &P12380_5};	/*Property = ScreenPixelsWidth*/
const Profile P12380 = {"12380", &P12380_4};	/*Profile ID - 12380*/
const Value V12377_1_1 = {S2, NULL};	/*Value = True*/
const Property P12377_1 = {S50, 0, &V12377_1_1, NULL};	/*Property = IsMobile*/
const Value V12377_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12377_5 = {S52, 0, &V12377_5_1222, &P12377_1};	/*Property = ScreenPixelsHeight*/
const Value V12377_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12377_4 = {S54, 0, &V12377_4_1224, &P12377_5};	/*Property = ScreenPixelsWidth*/
const Profile P12377 = {"12377", &P12377_4};	/*Profile ID - 12377*/
const Value V12391_1_1 = {S2, NULL};	/*Value = True*/
const Property P12391_1 = {S50, 0, &V12391_1_1, NULL};	/*Property = IsMobile*/
const Value V12391_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12391_5 = {S52, 0, &V12391_5_1294, &P12391_1};	/*Property = ScreenPixelsHeight*/
const Value V12391_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12391_4 = {S54, 0, &V12391_4_1296, &P12391_5};	/*Property = ScreenPixelsWidth*/
const Profile P12391 = {"12391", &P12391_4};	/*Profile ID - 12391*/
const Value V14258_1_1 = {S2, NULL};	/*Value = True*/
const Property P14258_1 = {S50, 0, &V14258_1_1, NULL};	/*Property = IsMobile*/
const Value V14258_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14258_5 = {S52, 0, &V14258_5_1293, &P14258_1};	/*Property = ScreenPixelsHeight*/
const Value V14258_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14258_4 = {S54, 0, &V14258_4_1296, &P14258_5};	/*Property = ScreenPixelsWidth*/
const Profile P14258 = {"14258", &P14258_4};	/*Profile ID - 14258*/
const Value V14873_1_1 = {S2, NULL};	/*Value = True*/
const Property P14873_1 = {S50, 0, &V14873_1_1, NULL};	/*Property = IsMobile*/
const Value V14873_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14873_5 = {S52, 0, &V14873_5_1277, &P14873_1};	/*Property = ScreenPixelsHeight*/
const Value V14873_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14873_4 = {S54, 0, &V14873_4_1279, &P14873_5};	/*Property = ScreenPixelsWidth*/
const Profile P14873 = {"14873", &P14873_4};	/*Profile ID - 14873*/
const Value V14635_1_1 = {S2, NULL};	/*Value = True*/
const Property P14635_1 = {S50, 0, &V14635_1_1, NULL};	/*Property = IsMobile*/
const Value V14635_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14635_5 = {S52, 0, &V14635_5_1380, &P14635_1};	/*Property = ScreenPixelsHeight*/
const Value V14635_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14635_4 = {S54, 0, &V14635_4_1279, &P14635_5};	/*Property = ScreenPixelsWidth*/
const Profile P14635 = {"14635", &P14635_4};	/*Profile ID - 14635*/
const Value V12206_1_1 = {S2, NULL};	/*Value = True*/
const Property P12206_1 = {S50, 0, &V12206_1_1, NULL};	/*Property = IsMobile*/
const Value V12206_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12206_5 = {S52, 0, &V12206_5_1304, &P12206_1};	/*Property = ScreenPixelsHeight*/
const Value V12206_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12206_4 = {S54, 0, &V12206_4_1224, &P12206_5};	/*Property = ScreenPixelsWidth*/
const Profile P12206 = {"12206", &P12206_4};	/*Profile ID - 12206*/
const Value V14244_1_1 = {S2, NULL};	/*Value = True*/
const Property P14244_1 = {S50, 0, &V14244_1_1, NULL};	/*Property = IsMobile*/
const Value V14244_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14244_5 = {S52, 0, &V14244_5_1222, &P14244_1};	/*Property = ScreenPixelsHeight*/
const Value V14244_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14244_4 = {S54, 0, &V14244_4_1224, &P14244_5};	/*Property = ScreenPixelsWidth*/
const Profile P14244 = {"14244", &P14244_4};	/*Profile ID - 14244*/
const Value V13783_1_1 = {S2, NULL};	/*Value = True*/
const Property P13783_1 = {S50, 0, &V13783_1_1, NULL};	/*Property = IsMobile*/
const Value V13783_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13783_5 = {S52, 0, &V13783_5_1304, &P13783_1};	/*Property = ScreenPixelsHeight*/
const Value V13783_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P13783_4 = {S54, 0, &V13783_4_1278, &P13783_5};	/*Property = ScreenPixelsWidth*/
const Profile P13783 = {"13783", &P13783_4};	/*Profile ID - 13783*/
const Profile P5520 = {"5520", NULL};	/*Profile ID - 5520*/
const Value V3797_279_57148 = {S0, NULL};	/*Value = False*/
const Property P3797_279 = {S1, 0, &V3797_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V3797_282_57154 = {S0, NULL};	/*Value = False*/
const Property P3797_282 = {S3, 0, &V3797_282_57154, &P3797_279};	/*Property = BlobBuilder*/
const Value V3797_350_57444 = {S2, NULL};	/*Value = True*/
const Property P3797_350 = {S4, 0, &V3797_350_57444, &P3797_282};	/*Property = Canvas*/
const Value V3797_283_57159 = {S2, NULL};	/*Value = True*/
const Property P3797_283 = {S5, 0, &V3797_283_57159, &P3797_350};	/*Property = CssBackground*/
const Value V3797_284_57163 = {S0, NULL};	/*Value = False*/
const Property P3797_284 = {S6, 0, &V3797_284_57163, &P3797_283};	/*Property = CssBorderImage*/
const Value V3797_285_57166 = {S0, NULL};	/*Value = False*/
const Property P3797_285 = {S7, 0, &V3797_285_57166, &P3797_284};	/*Property = CssCanvas*/
const Value V3797_286_57168 = {S2, NULL};	/*Value = True*/
const Property P3797_286 = {S8, 0, &V3797_286_57168, &P3797_285};	/*Property = CssColor*/
const Value V3797_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3797_237 = {S9, 0, &V3797_237_55786, &P3797_286};	/*Property = CssColumn*/
const Value V3797_287_57172 = {S0, NULL};	/*Value = False*/
const Property P3797_287 = {S10, 0, &V3797_287_57172, &P3797_237};	/*Property = CssFlexbox*/
const Value V3797_288_57175 = {S0, NULL};	/*Value = False*/
const Property P3797_288 = {S11, 0, &V3797_288_57175, &P3797_287};	/*Property = CssFont*/
const Value V3797_289_57759 = {S0, NULL};	/*Value = False*/
const Property P3797_289 = {S12, 0, &V3797_289_57759, &P3797_288};	/*Property = CssImages*/
const Value V3797_291_57181 = {S0, NULL};	/*Value = False*/
const Property P3797_291 = {S13, 0, &V3797_291_57181, &P3797_289};	/*Property = CssMediaQueries*/
const Value V3797_292_57183 = {S2, NULL};	/*Value = True*/
const Property P3797_292 = {S14, 0, &V3797_292_57183, &P3797_291};	/*Property = CssMinMax*/
const Value V3797_293_57187 = {S0, NULL};	/*Value = False*/
const Property P3797_293 = {S15, 0, &V3797_293_57187, &P3797_292};	/*Property = CssOverflow*/
const Value V3797_294_57190 = {S0, NULL};	/*Value = False*/
const Property P3797_294 = {S16, 0, &V3797_294_57190, &P3797_293};	/*Property = CssPosition*/
const Value V3797_295_57192 = {S2, NULL};	/*Value = True*/
const Property P3797_295 = {S17, 0, &V3797_295_57192, &P3797_294};	/*Property = CssText*/
const Value V3797_297_57195 = {S2, NULL};	/*Value = True*/
const Property P3797_297 = {S18, 0, &V3797_297_57195, &P3797_295};	/*Property = CssTransforms*/
const Value V3797_241_55799 = {S2, NULL};	/*Value = True*/
const Property P3797_241 = {S19, 0, &V3797_241_55799, &P3797_297};	/*Property = CssTransitions*/
const Value V3797_298_57206 = {S2, NULL};	/*Value = True*/
const Property P3797_298 = {S20, 0, &V3797_298_57206, &P3797_241};	/*Property = CssUI*/
const Value V3797_300_57216 = {S0, NULL};	/*Value = False*/
const Property P3797_300 = {S21, 0, &V3797_300_57216, &P3797_298};	/*Property = DataSet*/
const Value V3797_301_57218 = {S2, NULL};	/*Value = True*/
const Property P3797_301 = {S22, 0, &V3797_301_57218, &P3797_300};	/*Property = DataUrl*/
const Value V3797_302_57222 = {S0, NULL};	/*Value = False*/
const Property P3797_302 = {S23, 0, &V3797_302_57222, &P3797_301};	/*Property = DeviceOrientation*/
const Value V3797_304_57228 = {S0, NULL};	/*Value = False*/
const Property P3797_304 = {S24, 0, &V3797_304_57228, &P3797_302};	/*Property = FileReader*/
const Value V3797_305_57231 = {S0, NULL};	/*Value = False*/
const Property P3797_305 = {S25, 0, &V3797_305_57231, &P3797_304};	/*Property = FileSaver*/
const Value V3797_306_57237 = {S0, NULL};	/*Value = False*/
const Property P3797_306 = {S26, 0, &V3797_306_57237, &P3797_305};	/*Property = FileWriter*/
const Value V3797_307_57240 = {S0, NULL};	/*Value = False*/
const Property P3797_307 = {S27, 0, &V3797_307_57240, &P3797_306};	/*Property = FormData*/
const Value V3797_309_57246 = {S0, NULL};	/*Value = False*/
const Property P3797_309 = {S28, 0, &V3797_309_57246, &P3797_307};	/*Property = Fullscreen*/
const Value V3797_310_57248 = {S2, NULL};	/*Value = True*/
const Property P3797_310 = {S29, 0, &V3797_310_57248, &P3797_309};	/*Property = GeoLocation*/
const Value V3797_312_57255 = {S0, NULL};	/*Value = False*/
const Property P3797_312 = {S30, 0, &V3797_312_57255, &P3797_310};	/*Property = History*/
const Value V3797_314_57261 = {S0, NULL};	/*Value = False*/
const Property P3797_314 = {S31, 0, &V3797_314_57261, &P3797_312};	/*Property = Html5*/
const Value V3797_313_57258 = {S0, NULL};	/*Value = False*/
const Property P3797_313 = {S32, 0, &V3797_313_57258, &P3797_314};	/*Property = Html-Media-Capture*/
const Value V3797_315_57264 = {S0, NULL};	/*Value = False*/
const Property P3797_315 = {S33, 0, &V3797_315_57264, &P3797_313};	/*Property = Iframe*/
const Value V3797_316_57267 = {S0, NULL};	/*Value = False*/
const Property P3797_316 = {S34, 0, &V3797_316_57267, &P3797_315};	/*Property = IndexedDB*/
const Value V3797_317_57269 = {S2, NULL};	/*Value = True*/
const Property P3797_317 = {S35, 0, &V3797_317_57269, &P3797_316};	/*Property = Json*/
const Value V3797_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3797_15 = {S37, 0, &V3797_15_28, &P3797_317};	/*Property = LayoutEngine*/
const Value V3797_344_57272 = {S2, NULL};	/*Value = True*/
const Property P3797_344 = {S38, 0, &V3797_344_57272, &P3797_15};	/*Property = Masking*/
const Value V3797_320_57290 = {S2, NULL};	/*Value = True*/
const Property P3797_320 = {S39, 0, &V3797_320_57290, &P3797_344};	/*Property = PostMessage*/
const Value V3797_321_57293 = {S2, NULL};	/*Value = True*/
const Property P3797_321 = {S40, 0, &V3797_321_57293, &P3797_320};	/*Property = Progress*/
const Value V3797_322_57296 = {S2, NULL};	/*Value = True*/
const Property P3797_322 = {S41, 0, &V3797_322_57296, &P3797_321};	/*Property = Prompts*/
const Value V3797_323_57299 = {S2, NULL};	/*Value = True*/
const Property P3797_323 = {S42, 0, &V3797_323_57299, &P3797_322};	/*Property = Selector*/
const Value V3797_324_57306 = {S0, NULL};	/*Value = False*/
const Property P3797_324 = {S43, 0, &V3797_324_57306, &P3797_323};	/*Property = Svg*/
const Value V3797_327_57315 = {S0, NULL};	/*Value = False*/
const Property P3797_327 = {S44, 0, &V3797_327_57315, &P3797_324};	/*Property = TouchEvents*/
const Value V3797_328_57318 = {S0, NULL};	/*Value = False*/
const Property P3797_328 = {S45, 0, &V3797_328_57318, &P3797_327};	/*Property = Track*/
const Value V3797_329_57324 = {S0, NULL};	/*Value = False*/
const Property P3797_329 = {S46, 0, &V3797_329_57324, &P3797_328};	/*Property = Video*/
const Value V3797_330_57326 = {S2, NULL};	/*Value = True*/
const Property P3797_330 = {S47, 0, &V3797_330_57326, &P3797_329};	/*Property = Viewport*/
const Value V3797_332_57339 = {S0, NULL};	/*Value = False*/
const Property P3797_332 = {S48, 0, &V3797_332_57339, &P3797_330};	/*Property = WebWorkers*/
const Value V3797_333_57342 = {S0, NULL};	/*Value = False*/
const Property P3797_333 = {S49, 0, &V3797_333_57342, &P3797_332};	/*Property = Xhr2*/
const Profile P3797 = {"3797", &P3797_333};	/*Profile ID - 3797*/
const Value V11726_1_1 = {S2, NULL};	/*Value = True*/
const Property P11726_1 = {S50, 0, &V11726_1_1, NULL};	/*Property = IsMobile*/
const Value V11726_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P11726_5 = {S52, 0, &V11726_5_1380, &P11726_1};	/*Property = ScreenPixelsHeight*/
const Value V11726_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P11726_4 = {S54, 0, &V11726_4_1279, &P11726_5};	/*Property = ScreenPixelsWidth*/
const Profile P11726 = {"11726", &P11726_4};	/*Profile ID - 11726*/
const Value V12412_1_1 = {S2, NULL};	/*Value = True*/
const Property P12412_1 = {S50, 0, &V12412_1_1, NULL};	/*Property = IsMobile*/
const Value V12412_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12412_5 = {S52, 0, &V12412_5_1222, &P12412_1};	/*Property = ScreenPixelsHeight*/
const Value V12412_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12412_4 = {S54, 0, &V12412_4_1224, &P12412_5};	/*Property = ScreenPixelsWidth*/
const Profile P12412 = {"12412", &P12412_4};	/*Profile ID - 12412*/
const Value V14583_1_1 = {S2, NULL};	/*Value = True*/
const Property P14583_1 = {S50, 0, &V14583_1_1, NULL};	/*Property = IsMobile*/
const Value V14583_5_1512 = {S77, NULL};	/*Value = 416*/
const Property P14583_5 = {S52, 0, &V14583_5_1512, &P14583_1};	/*Property = ScreenPixelsHeight*/
const Value V14583_4_1513 = {S78, NULL};	/*Value = 352*/
const Property P14583_4 = {S54, 0, &V14583_4_1513, &P14583_5};	/*Property = ScreenPixelsWidth*/
const Profile P14583 = {"14583", &P14583_4};	/*Profile ID - 14583*/
const Value V13668_1_1 = {S2, NULL};	/*Value = True*/
const Property P13668_1 = {S50, 0, &V13668_1_1, NULL};	/*Property = IsMobile*/
const Value V13668_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13668_5 = {S52, 0, &V13668_5_1222, &P13668_1};	/*Property = ScreenPixelsHeight*/
const Value V13668_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13668_4 = {S54, 0, &V13668_4_1229, &P13668_5};	/*Property = ScreenPixelsWidth*/
const Profile P13668 = {"13668", &P13668_4};	/*Profile ID - 13668*/
const Value V12421_1_1 = {S2, NULL};	/*Value = True*/
const Property P12421_1 = {S50, 0, &V12421_1_1, NULL};	/*Property = IsMobile*/
const Value V12421_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12421_5 = {S52, 0, &V12421_5_1293, &P12421_1};	/*Property = ScreenPixelsHeight*/
const Value V12421_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12421_4 = {S54, 0, &V12421_4_1296, &P12421_5};	/*Property = ScreenPixelsWidth*/
const Profile P12421 = {"12421", &P12421_4};	/*Profile ID - 12421*/
const Profile P18110 = {"18110", NULL};	/*Profile ID - 18110*/
const Value V18337_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_279 = {S1, 0, &V18337_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18337_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_282 = {S3, 0, &V18337_282_57155, &P18337_279};	/*Property = BlobBuilder*/
const Value V18337_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_350 = {S4, 0, &V18337_350_57446, &P18337_282};	/*Property = Canvas*/
const Value V18337_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_283 = {S5, 0, &V18337_283_57161, &P18337_350};	/*Property = CssBackground*/
const Value V18337_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_284 = {S6, 0, &V18337_284_57164, &P18337_283};	/*Property = CssBorderImage*/
const Value V18337_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_285 = {S7, 0, &V18337_285_57167, &P18337_284};	/*Property = CssCanvas*/
const Value V18337_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_286 = {S8, 0, &V18337_286_57170, &P18337_285};	/*Property = CssColor*/
const Value V18337_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_237 = {S9, 0, &V18337_237_55786, &P18337_286};	/*Property = CssColumn*/
const Value V18337_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_287 = {S10, 0, &V18337_287_57173, &P18337_237};	/*Property = CssFlexbox*/
const Value V18337_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_288 = {S11, 0, &V18337_288_57176, &P18337_287};	/*Property = CssFont*/
const Value V18337_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_289 = {S12, 0, &V18337_289_86899, &P18337_288};	/*Property = CssImages*/
const Value V18337_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_291 = {S13, 0, &V18337_291_57182, &P18337_289};	/*Property = CssMediaQueries*/
const Value V18337_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_292 = {S14, 0, &V18337_292_57185, &P18337_291};	/*Property = CssMinMax*/
const Value V18337_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_293 = {S15, 0, &V18337_293_57188, &P18337_292};	/*Property = CssOverflow*/
const Value V18337_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_294 = {S16, 0, &V18337_294_57191, &P18337_293};	/*Property = CssPosition*/
const Value V18337_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_295 = {S17, 0, &V18337_295_57194, &P18337_294};	/*Property = CssText*/
const Value V18337_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_297 = {S18, 0, &V18337_297_57197, &P18337_295};	/*Property = CssTransforms*/
const Value V18337_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_241 = {S19, 0, &V18337_241_55801, &P18337_297};	/*Property = CssTransitions*/
const Value V18337_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_298 = {S20, 0, &V18337_298_57208, &P18337_241};	/*Property = CssUI*/
const Value V18337_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_300 = {S21, 0, &V18337_300_57217, &P18337_298};	/*Property = DataSet*/
const Value V18337_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_301 = {S22, 0, &V18337_301_57220, &P18337_300};	/*Property = DataUrl*/
const Value V18337_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_302 = {S23, 0, &V18337_302_57223, &P18337_301};	/*Property = DeviceOrientation*/
const Value V18337_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_304 = {S24, 0, &V18337_304_57229, &P18337_302};	/*Property = FileReader*/
const Value V18337_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_305 = {S25, 0, &V18337_305_57232, &P18337_304};	/*Property = FileSaver*/
const Value V18337_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_306 = {S26, 0, &V18337_306_57238, &P18337_305};	/*Property = FileWriter*/
const Value V18337_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_307 = {S27, 0, &V18337_307_57241, &P18337_306};	/*Property = FormData*/
const Value V18337_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_309 = {S28, 0, &V18337_309_57247, &P18337_307};	/*Property = Fullscreen*/
const Value V18337_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_310 = {S29, 0, &V18337_310_57250, &P18337_309};	/*Property = GeoLocation*/
const Value V18337_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_312 = {S30, 0, &V18337_312_57256, &P18337_310};	/*Property = History*/
const Value V18337_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_314 = {S31, 0, &V18337_314_57262, &P18337_312};	/*Property = Html5*/
const Value V18337_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_313 = {S32, 0, &V18337_313_57259, &P18337_314};	/*Property = Html-Media-Capture*/
const Value V18337_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_315 = {S33, 0, &V18337_315_57265, &P18337_313};	/*Property = Iframe*/
const Value V18337_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_316 = {S34, 0, &V18337_316_57268, &P18337_315};	/*Property = IndexedDB*/
const Value V18337_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_317 = {S35, 0, &V18337_317_57271, &P18337_316};	/*Property = Json*/
const Value V18337_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18337_15 = {S37, 0, &V18337_15_31, &P18337_317};	/*Property = LayoutEngine*/
const Value V18337_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_344 = {S38, 0, &V18337_344_57274, &P18337_15};	/*Property = Masking*/
const Value V18337_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_320 = {S39, 0, &V18337_320_57292, &P18337_344};	/*Property = PostMessage*/
const Value V18337_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_321 = {S40, 0, &V18337_321_57295, &P18337_320};	/*Property = Progress*/
const Value V18337_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_322 = {S41, 0, &V18337_322_57298, &P18337_321};	/*Property = Prompts*/
const Value V18337_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_323 = {S42, 0, &V18337_323_57301, &P18337_322};	/*Property = Selector*/
const Value V18337_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_324 = {S43, 0, &V18337_324_57307, &P18337_323};	/*Property = Svg*/
const Value V18337_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_327 = {S44, 0, &V18337_327_57316, &P18337_324};	/*Property = TouchEvents*/
const Value V18337_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_328 = {S45, 0, &V18337_328_57319, &P18337_327};	/*Property = Track*/
const Value V18337_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_329 = {S46, 0, &V18337_329_57325, &P18337_328};	/*Property = Video*/
const Value V18337_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_330 = {S47, 0, &V18337_330_57328, &P18337_329};	/*Property = Viewport*/
const Value V18337_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_332 = {S48, 0, &V18337_332_57340, &P18337_330};	/*Property = WebWorkers*/
const Value V18337_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18337_333 = {S49, 0, &V18337_333_57343, &P18337_332};	/*Property = Xhr2*/
const Profile P18337 = {"18337", &P18337_333};	/*Profile ID - 18337*/
const Value V12176_1_1 = {S2, NULL};	/*Value = True*/
const Property P12176_1 = {S50, 0, &V12176_1_1, NULL};	/*Property = IsMobile*/
const Value V12176_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12176_5 = {S52, 0, &V12176_5_1222, &P12176_1};	/*Property = ScreenPixelsHeight*/
const Value V12176_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12176_4 = {S54, 0, &V12176_4_1224, &P12176_5};	/*Property = ScreenPixelsWidth*/
const Profile P12176 = {"12176", &P12176_4};	/*Profile ID - 12176*/
const Value V13740_1_1 = {S2, NULL};	/*Value = True*/
const Property P13740_1 = {S50, 0, &V13740_1_1, NULL};	/*Property = IsMobile*/
const Value V13740_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13740_5 = {S52, 0, &V13740_5_1222, &P13740_1};	/*Property = ScreenPixelsHeight*/
const Value V13740_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13740_4 = {S54, 0, &V13740_4_1224, &P13740_5};	/*Property = ScreenPixelsWidth*/
const Profile P13740 = {"13740", &P13740_4};	/*Profile ID - 13740*/
const Value V12436_1_1 = {S2, NULL};	/*Value = True*/
const Property P12436_1 = {S50, 0, &V12436_1_1, NULL};	/*Property = IsMobile*/
const Value V12436_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12436_5 = {S52, 0, &V12436_5_1294, &P12436_1};	/*Property = ScreenPixelsHeight*/
const Value V12436_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12436_4 = {S54, 0, &V12436_4_1296, &P12436_5};	/*Property = ScreenPixelsWidth*/
const Profile P12436 = {"12436", &P12436_4};	/*Profile ID - 12436*/
const Value V12004_1_1 = {S2, NULL};	/*Value = True*/
const Property P12004_1 = {S50, 0, &V12004_1_1, NULL};	/*Property = IsMobile*/
const Value V12004_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12004_5 = {S52, 0, &V12004_5_1222, &P12004_1};	/*Property = ScreenPixelsHeight*/
const Value V12004_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12004_4 = {S54, 0, &V12004_4_1224, &P12004_5};	/*Property = ScreenPixelsWidth*/
const Profile P12004 = {"12004", &P12004_4};	/*Profile ID - 12004*/
const Value V12441_1_1 = {S2, NULL};	/*Value = True*/
const Property P12441_1 = {S50, 0, &V12441_1_1, NULL};	/*Property = IsMobile*/
const Value V12441_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12441_5 = {S52, 0, &V12441_5_1293, &P12441_1};	/*Property = ScreenPixelsHeight*/
const Value V12441_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12441_4 = {S54, 0, &V12441_4_1296, &P12441_5};	/*Property = ScreenPixelsWidth*/
const Profile P12441 = {"12441", &P12441_4};	/*Profile ID - 12441*/
const Value V12444_1_1 = {S2, NULL};	/*Value = True*/
const Property P12444_1 = {S50, 0, &V12444_1_1, NULL};	/*Property = IsMobile*/
const Value V12444_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12444_5 = {S52, 0, &V12444_5_1222, &P12444_1};	/*Property = ScreenPixelsHeight*/
const Value V12444_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12444_4 = {S54, 0, &V12444_4_1224, &P12444_5};	/*Property = ScreenPixelsWidth*/
const Profile P12444 = {"12444", &P12444_4};	/*Profile ID - 12444*/
const Value V14895_1_1 = {S2, NULL};	/*Value = True*/
const Property P14895_1 = {S50, 0, &V14895_1_1, NULL};	/*Property = IsMobile*/
const Value V14895_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14895_5 = {S52, 0, &V14895_5_1222, &P14895_1};	/*Property = ScreenPixelsHeight*/
const Value V14895_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14895_4 = {S54, 0, &V14895_4_1224, &P14895_5};	/*Property = ScreenPixelsWidth*/
const Profile P14895 = {"14895", &P14895_4};	/*Profile ID - 14895*/
const Profile P18120 = {"18120", NULL};	/*Profile ID - 18120*/
const Value V12446_1_1 = {S2, NULL};	/*Value = True*/
const Property P12446_1 = {S50, 0, &V12446_1_1, NULL};	/*Property = IsMobile*/
const Value V12446_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12446_5 = {S52, 0, &V12446_5_1304, &P12446_1};	/*Property = ScreenPixelsHeight*/
const Value V12446_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12446_4 = {S54, 0, &V12446_4_1224, &P12446_5};	/*Property = ScreenPixelsWidth*/
const Profile P12446 = {"12446", &P12446_4};	/*Profile ID - 12446*/
const Value V12449_1_1 = {S2, NULL};	/*Value = True*/
const Property P12449_1 = {S50, 0, &V12449_1_1, NULL};	/*Property = IsMobile*/
const Value V12449_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12449_5 = {S52, 0, &V12449_5_1380, &P12449_1};	/*Property = ScreenPixelsHeight*/
const Value V12449_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12449_4 = {S54, 0, &V12449_4_1279, &P12449_5};	/*Property = ScreenPixelsWidth*/
const Profile P12449 = {"12449", &P12449_4};	/*Profile ID - 12449*/
const Value V12450_1_1 = {S2, NULL};	/*Value = True*/
const Property P12450_1 = {S50, 0, &V12450_1_1, NULL};	/*Property = IsMobile*/
const Value V12450_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12450_5 = {S52, 0, &V12450_5_1380, &P12450_1};	/*Property = ScreenPixelsHeight*/
const Value V12450_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12450_4 = {S54, 0, &V12450_4_1279, &P12450_5};	/*Property = ScreenPixelsWidth*/
const Profile P12450 = {"12450", &P12450_4};	/*Profile ID - 12450*/
const Value V13746_1_1 = {S2, NULL};	/*Value = True*/
const Property P13746_1 = {S50, 0, &V13746_1_1, NULL};	/*Property = IsMobile*/
const Value V13746_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13746_5 = {S52, 0, &V13746_5_1277, &P13746_1};	/*Property = ScreenPixelsHeight*/
const Value V13746_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13746_4 = {S54, 0, &V13746_4_1279, &P13746_5};	/*Property = ScreenPixelsWidth*/
const Profile P13746 = {"13746", &P13746_4};	/*Profile ID - 13746*/
const Value V12452_1_1 = {S2, NULL};	/*Value = True*/
const Property P12452_1 = {S50, 0, &V12452_1_1, NULL};	/*Property = IsMobile*/
const Value V12452_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12452_5 = {S52, 0, &V12452_5_1222, &P12452_1};	/*Property = ScreenPixelsHeight*/
const Value V12452_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12452_4 = {S54, 0, &V12452_4_1224, &P12452_5};	/*Property = ScreenPixelsWidth*/
const Profile P12452 = {"12452", &P12452_4};	/*Profile ID - 12452*/
const Value V12455_1_1 = {S2, NULL};	/*Value = True*/
const Property P12455_1 = {S50, 0, &V12455_1_1, NULL};	/*Property = IsMobile*/
const Value V12455_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12455_5 = {S52, 0, &V12455_5_1222, &P12455_1};	/*Property = ScreenPixelsHeight*/
const Value V12455_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12455_4 = {S54, 0, &V12455_4_1224, &P12455_5};	/*Property = ScreenPixelsWidth*/
const Profile P12455 = {"12455", &P12455_4};	/*Profile ID - 12455*/
const Value V12456_1_1 = {S2, NULL};	/*Value = True*/
const Property P12456_1 = {S50, 0, &V12456_1_1, NULL};	/*Property = IsMobile*/
const Value V12456_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12456_5 = {S52, 0, &V12456_5_1380, &P12456_1};	/*Property = ScreenPixelsHeight*/
const Value V12456_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12456_4 = {S54, 0, &V12456_4_1279, &P12456_5};	/*Property = ScreenPixelsWidth*/
const Profile P12456 = {"12456", &P12456_4};	/*Profile ID - 12456*/
const Value V12457_1_1 = {S2, NULL};	/*Value = True*/
const Property P12457_1 = {S50, 0, &V12457_1_1, NULL};	/*Property = IsMobile*/
const Value V12457_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12457_5 = {S52, 0, &V12457_5_1293, &P12457_1};	/*Property = ScreenPixelsHeight*/
const Value V12457_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12457_4 = {S54, 0, &V12457_4_1296, &P12457_5};	/*Property = ScreenPixelsWidth*/
const Profile P12457 = {"12457", &P12457_4};	/*Profile ID - 12457*/
const Profile P17605 = {"17605", NULL};	/*Profile ID - 17605*/
const Value V12458_1_1 = {S2, NULL};	/*Value = True*/
const Property P12458_1 = {S50, 0, &V12458_1_1, NULL};	/*Property = IsMobile*/
const Value V12458_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12458_5 = {S52, 0, &V12458_5_1304, &P12458_1};	/*Property = ScreenPixelsHeight*/
const Value V12458_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12458_4 = {S54, 0, &V12458_4_1224, &P12458_5};	/*Property = ScreenPixelsWidth*/
const Profile P12458 = {"12458", &P12458_4};	/*Profile ID - 12458*/
const Value V12331_1_1 = {S2, NULL};	/*Value = True*/
const Property P12331_1 = {S50, 0, &V12331_1_1, NULL};	/*Property = IsMobile*/
const Value V12331_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12331_5 = {S52, 0, &V12331_5_1380, &P12331_1};	/*Property = ScreenPixelsHeight*/
const Value V12331_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12331_4 = {S54, 0, &V12331_4_1279, &P12331_5};	/*Property = ScreenPixelsWidth*/
const Profile P12331 = {"12331", &P12331_4};	/*Profile ID - 12331*/
const Value V12460_1_1 = {S2, NULL};	/*Value = True*/
const Property P12460_1 = {S50, 0, &V12460_1_1, NULL};	/*Property = IsMobile*/
const Value V12460_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12460_5 = {S52, 0, &V12460_5_1277, &P12460_1};	/*Property = ScreenPixelsHeight*/
const Value V12460_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12460_4 = {S54, 0, &V12460_4_1279, &P12460_5};	/*Property = ScreenPixelsWidth*/
const Profile P12460 = {"12460", &P12460_4};	/*Profile ID - 12460*/
const Value V12464_1_1 = {S2, NULL};	/*Value = True*/
const Property P12464_1 = {S50, 0, &V12464_1_1, NULL};	/*Property = IsMobile*/
const Value V12464_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12464_5 = {S52, 0, &V12464_5_1222, &P12464_1};	/*Property = ScreenPixelsHeight*/
const Value V12464_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12464_4 = {S54, 0, &V12464_4_1224, &P12464_5};	/*Property = ScreenPixelsWidth*/
const Profile P12464 = {"12464", &P12464_4};	/*Profile ID - 12464*/
const Value V12467_1_1 = {S2, NULL};	/*Value = True*/
const Property P12467_1 = {S50, 0, &V12467_1_1, NULL};	/*Property = IsMobile*/
const Value V12467_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12467_5 = {S52, 0, &V12467_5_1233, &P12467_1};	/*Property = ScreenPixelsHeight*/
const Value V12467_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12467_4 = {S54, 0, &V12467_4_1235, &P12467_5};	/*Property = ScreenPixelsWidth*/
const Profile P12467 = {"12467", &P12467_4};	/*Profile ID - 12467*/
const Value V12473_1_1 = {S2, NULL};	/*Value = True*/
const Property P12473_1 = {S50, 0, &V12473_1_1, NULL};	/*Property = IsMobile*/
const Value V12473_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12473_5 = {S52, 0, &V12473_5_1222, &P12473_1};	/*Property = ScreenPixelsHeight*/
const Value V12473_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12473_4 = {S54, 0, &V12473_4_1224, &P12473_5};	/*Property = ScreenPixelsWidth*/
const Profile P12473 = {"12473", &P12473_4};	/*Profile ID - 12473*/
const Value V12260_1_1 = {S2, NULL};	/*Value = True*/
const Property P12260_1 = {S50, 0, &V12260_1_1, NULL};	/*Property = IsMobile*/
const Value V12260_5_1796 = {S64, NULL};	/*Value = 360*/
const Property P12260_5 = {S52, 0, &V12260_5_1796, &P12260_1};	/*Property = ScreenPixelsHeight*/
const Value V12260_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12260_4 = {S54, 0, &V12260_4_1224, &P12260_5};	/*Property = ScreenPixelsWidth*/
const Profile P12260 = {"12260", &P12260_4};	/*Profile ID - 12260*/
const Value V12478_1_1 = {S2, NULL};	/*Value = True*/
const Property P12478_1 = {S50, 0, &V12478_1_1, NULL};	/*Property = IsMobile*/
const Value V12478_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12478_5 = {S52, 0, &V12478_5_1277, &P12478_1};	/*Property = ScreenPixelsHeight*/
const Value V12478_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12478_4 = {S54, 0, &V12478_4_1279, &P12478_5};	/*Property = ScreenPixelsWidth*/
const Profile P12478 = {"12478", &P12478_4};	/*Profile ID - 12478*/
const Value V14496_1_1 = {S2, NULL};	/*Value = True*/
const Property P14496_1 = {S50, 0, &V14496_1_1, NULL};	/*Property = IsMobile*/
const Value V14496_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14496_5 = {S52, 0, &V14496_5_1222, &P14496_1};	/*Property = ScreenPixelsHeight*/
const Value V14496_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14496_4 = {S54, 0, &V14496_4_1224, &P14496_5};	/*Property = ScreenPixelsWidth*/
const Profile P14496 = {"14496", &P14496_4};	/*Profile ID - 14496*/
const Value V12482_1_1 = {S2, NULL};	/*Value = True*/
const Property P12482_1 = {S50, 0, &V12482_1_1, NULL};	/*Property = IsMobile*/
const Value V12482_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12482_5 = {S52, 0, &V12482_5_1227, &P12482_1};	/*Property = ScreenPixelsHeight*/
const Value V12482_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12482_4 = {S54, 0, &V12482_4_1229, &P12482_5};	/*Property = ScreenPixelsWidth*/
const Profile P12482 = {"12482", &P12482_4};	/*Profile ID - 12482*/
const Value V18325_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18325_279 = {S1, 0, &V18325_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18325_282_57153 = {S2, NULL};	/*Value = True*/
const Property P18325_282 = {S3, 0, &V18325_282_57153, &P18325_279};	/*Property = BlobBuilder*/
const Value V18325_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18325_350 = {S4, 0, &V18325_350_57444, &P18325_282};	/*Property = Canvas*/
const Value V18325_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18325_283 = {S5, 0, &V18325_283_57159, &P18325_350};	/*Property = CssBackground*/
const Value V18325_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18325_284 = {S6, 0, &V18325_284_57163, &P18325_283};	/*Property = CssBorderImage*/
const Value V18325_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18325_285 = {S7, 0, &V18325_285_57166, &P18325_284};	/*Property = CssCanvas*/
const Value V18325_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18325_286 = {S8, 0, &V18325_286_57168, &P18325_285};	/*Property = CssColor*/
const Value V18325_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18325_237 = {S9, 0, &V18325_237_55786, &P18325_286};	/*Property = CssColumn*/
const Value V18325_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18325_287 = {S10, 0, &V18325_287_57172, &P18325_237};	/*Property = CssFlexbox*/
const Value V18325_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18325_288 = {S11, 0, &V18325_288_57175, &P18325_287};	/*Property = CssFont*/
const Value V18325_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18325_289 = {S12, 0, &V18325_289_57759, &P18325_288};	/*Property = CssImages*/
const Value V18325_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18325_291 = {S13, 0, &V18325_291_57180, &P18325_289};	/*Property = CssMediaQueries*/
const Value V18325_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18325_292 = {S14, 0, &V18325_292_57183, &P18325_291};	/*Property = CssMinMax*/
const Value V18325_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18325_293 = {S15, 0, &V18325_293_57187, &P18325_292};	/*Property = CssOverflow*/
const Value V18325_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18325_294 = {S16, 0, &V18325_294_57190, &P18325_293};	/*Property = CssPosition*/
const Value V18325_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18325_295 = {S17, 0, &V18325_295_57192, &P18325_294};	/*Property = CssText*/
const Value V18325_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18325_297 = {S18, 0, &V18325_297_57195, &P18325_295};	/*Property = CssTransforms*/
const Value V18325_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18325_241 = {S19, 0, &V18325_241_55799, &P18325_297};	/*Property = CssTransitions*/
const Value V18325_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18325_298 = {S20, 0, &V18325_298_57206, &P18325_241};	/*Property = CssUI*/
const Value V18325_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18325_300 = {S21, 0, &V18325_300_57216, &P18325_298};	/*Property = DataSet*/
const Value V18325_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18325_301 = {S22, 0, &V18325_301_57218, &P18325_300};	/*Property = DataUrl*/
const Value V18325_302_57221 = {S2, NULL};	/*Value = True*/
const Property P18325_302 = {S23, 0, &V18325_302_57221, &P18325_301};	/*Property = DeviceOrientation*/
const Value V18325_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18325_304 = {S24, 0, &V18325_304_57227, &P18325_302};	/*Property = FileReader*/
const Value V18325_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18325_305 = {S25, 0, &V18325_305_57231, &P18325_304};	/*Property = FileSaver*/
const Value V18325_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18325_306 = {S26, 0, &V18325_306_57237, &P18325_305};	/*Property = FileWriter*/
const Value V18325_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18325_307 = {S27, 0, &V18325_307_57239, &P18325_306};	/*Property = FormData*/
const Value V18325_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18325_309 = {S28, 0, &V18325_309_57246, &P18325_307};	/*Property = Fullscreen*/
const Value V18325_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18325_310 = {S29, 0, &V18325_310_57248, &P18325_309};	/*Property = GeoLocation*/
const Value V18325_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18325_312 = {S30, 0, &V18325_312_57255, &P18325_310};	/*Property = History*/
const Value V18325_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18325_314 = {S31, 0, &V18325_314_57261, &P18325_312};	/*Property = Html5*/
const Value V18325_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18325_313 = {S32, 0, &V18325_313_57258, &P18325_314};	/*Property = Html-Media-Capture*/
const Value V18325_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18325_315 = {S33, 0, &V18325_315_57264, &P18325_313};	/*Property = Iframe*/
const Value V18325_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18325_316 = {S34, 0, &V18325_316_57266, &P18325_315};	/*Property = IndexedDB*/
const Value V18325_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18325_317 = {S35, 0, &V18325_317_57269, &P18325_316};	/*Property = Json*/
const Value V18325_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18325_15 = {S37, 0, &V18325_15_28, &P18325_317};	/*Property = LayoutEngine*/
const Value V18325_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18325_344 = {S38, 0, &V18325_344_57272, &P18325_15};	/*Property = Masking*/
const Value V18325_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18325_320 = {S39, 0, &V18325_320_57290, &P18325_344};	/*Property = PostMessage*/
const Value V18325_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18325_321 = {S40, 0, &V18325_321_57293, &P18325_320};	/*Property = Progress*/
const Value V18325_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18325_322 = {S41, 0, &V18325_322_57296, &P18325_321};	/*Property = Prompts*/
const Value V18325_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18325_323 = {S42, 0, &V18325_323_57299, &P18325_322};	/*Property = Selector*/
const Value V18325_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18325_324 = {S43, 0, &V18325_324_57306, &P18325_323};	/*Property = Svg*/
const Value V18325_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18325_327 = {S44, 0, &V18325_327_57315, &P18325_324};	/*Property = TouchEvents*/
const Value V18325_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18325_328 = {S45, 0, &V18325_328_57318, &P18325_327};	/*Property = Track*/
const Value V18325_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18325_329 = {S46, 0, &V18325_329_57323, &P18325_328};	/*Property = Video*/
const Value V18325_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18325_330 = {S47, 0, &V18325_330_57326, &P18325_329};	/*Property = Viewport*/
const Value V18325_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18325_332 = {S48, 0, &V18325_332_57339, &P18325_330};	/*Property = WebWorkers*/
const Value V18325_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18325_333 = {S49, 0, &V18325_333_57342, &P18325_332};	/*Property = Xhr2*/
const Profile P18325 = {"18325", &P18325_333};	/*Profile ID - 18325*/
const Value V17316_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17316_279 = {S1, 0, &V17316_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17316_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17316_282 = {S3, 0, &V17316_282_57154, &P17316_279};	/*Property = BlobBuilder*/
const Value V17316_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17316_350 = {S4, 0, &V17316_350_57444, &P17316_282};	/*Property = Canvas*/
const Value V17316_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17316_283 = {S5, 0, &V17316_283_57159, &P17316_350};	/*Property = CssBackground*/
const Value V17316_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17316_284 = {S6, 0, &V17316_284_57163, &P17316_283};	/*Property = CssBorderImage*/
const Value V17316_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17316_285 = {S7, 0, &V17316_285_57166, &P17316_284};	/*Property = CssCanvas*/
const Value V17316_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17316_286 = {S8, 0, &V17316_286_57168, &P17316_285};	/*Property = CssColor*/
const Value V17316_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17316_237 = {S9, 0, &V17316_237_55784, &P17316_286};	/*Property = CssColumn*/
const Value V17316_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17316_287 = {S10, 0, &V17316_287_57172, &P17316_237};	/*Property = CssFlexbox*/
const Value V17316_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17316_288 = {S11, 0, &V17316_288_57175, &P17316_287};	/*Property = CssFont*/
const Value V17316_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17316_289 = {S12, 0, &V17316_289_57759, &P17316_288};	/*Property = CssImages*/
const Value V17316_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17316_291 = {S13, 0, &V17316_291_57181, &P17316_289};	/*Property = CssMediaQueries*/
const Value V17316_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17316_292 = {S14, 0, &V17316_292_57183, &P17316_291};	/*Property = CssMinMax*/
const Value V17316_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17316_293 = {S15, 0, &V17316_293_57187, &P17316_292};	/*Property = CssOverflow*/
const Value V17316_294_57190 = {S0, NULL};	/*Value = False*/
const Property P17316_294 = {S16, 0, &V17316_294_57190, &P17316_293};	/*Property = CssPosition*/
const Value V17316_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17316_295 = {S17, 0, &V17316_295_57192, &P17316_294};	/*Property = CssText*/
const Value V17316_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17316_297 = {S18, 0, &V17316_297_57195, &P17316_295};	/*Property = CssTransforms*/
const Value V17316_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17316_241 = {S19, 0, &V17316_241_55799, &P17316_297};	/*Property = CssTransitions*/
const Value V17316_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17316_298 = {S20, 0, &V17316_298_57206, &P17316_241};	/*Property = CssUI*/
const Value V17316_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17316_300 = {S21, 0, &V17316_300_57216, &P17316_298};	/*Property = DataSet*/
const Value V17316_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17316_301 = {S22, 0, &V17316_301_57218, &P17316_300};	/*Property = DataUrl*/
const Value V17316_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17316_302 = {S23, 0, &V17316_302_57222, &P17316_301};	/*Property = DeviceOrientation*/
const Value V17316_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17316_304 = {S24, 0, &V17316_304_57228, &P17316_302};	/*Property = FileReader*/
const Value V17316_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17316_305 = {S25, 0, &V17316_305_57231, &P17316_304};	/*Property = FileSaver*/
const Value V17316_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17316_306 = {S26, 0, &V17316_306_57237, &P17316_305};	/*Property = FileWriter*/
const Value V17316_307_57240 = {S0, NULL};	/*Value = False*/
const Property P17316_307 = {S27, 0, &V17316_307_57240, &P17316_306};	/*Property = FormData*/
const Value V17316_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17316_309 = {S28, 0, &V17316_309_57246, &P17316_307};	/*Property = Fullscreen*/
const Value V17316_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17316_310 = {S29, 0, &V17316_310_57248, &P17316_309};	/*Property = GeoLocation*/
const Value V17316_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17316_312 = {S30, 0, &V17316_312_57255, &P17316_310};	/*Property = History*/
const Value V17316_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17316_314 = {S31, 0, &V17316_314_57261, &P17316_312};	/*Property = Html5*/
const Value V17316_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17316_313 = {S32, 0, &V17316_313_57258, &P17316_314};	/*Property = Html-Media-Capture*/
const Value V17316_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17316_315 = {S33, 0, &V17316_315_57264, &P17316_313};	/*Property = Iframe*/
const Value V17316_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17316_316 = {S34, 0, &V17316_316_57267, &P17316_315};	/*Property = IndexedDB*/
const Value V17316_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17316_317 = {S35, 0, &V17316_317_57269, &P17316_316};	/*Property = Json*/
const Value V17316_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17316_15 = {S37, 0, &V17316_15_28, &P17316_317};	/*Property = LayoutEngine*/
const Value V17316_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17316_344 = {S38, 0, &V17316_344_57272, &P17316_15};	/*Property = Masking*/
const Value V17316_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17316_320 = {S39, 0, &V17316_320_57290, &P17316_344};	/*Property = PostMessage*/
const Value V17316_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17316_321 = {S40, 0, &V17316_321_57293, &P17316_320};	/*Property = Progress*/
const Value V17316_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17316_322 = {S41, 0, &V17316_322_57296, &P17316_321};	/*Property = Prompts*/
const Value V17316_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17316_323 = {S42, 0, &V17316_323_57299, &P17316_322};	/*Property = Selector*/
const Value V17316_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17316_324 = {S43, 0, &V17316_324_57306, &P17316_323};	/*Property = Svg*/
const Value V17316_327_57314 = {S2, NULL};	/*Value = True*/
const Property P17316_327 = {S44, 0, &V17316_327_57314, &P17316_324};	/*Property = TouchEvents*/
const Value V17316_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17316_328 = {S45, 0, &V17316_328_57318, &P17316_327};	/*Property = Track*/
const Value V17316_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17316_329 = {S46, 0, &V17316_329_57323, &P17316_328};	/*Property = Video*/
const Value V17316_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17316_330 = {S47, 0, &V17316_330_57326, &P17316_329};	/*Property = Viewport*/
const Value V17316_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17316_332 = {S48, 0, &V17316_332_57339, &P17316_330};	/*Property = WebWorkers*/
const Value V17316_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17316_333 = {S49, 0, &V17316_333_57342, &P17316_332};	/*Property = Xhr2*/
const Profile P17316 = {"17316", &P17316_333};	/*Profile ID - 17316*/
const Value V12484_1_1 = {S2, NULL};	/*Value = True*/
const Property P12484_1 = {S50, 0, &V12484_1_1, NULL};	/*Property = IsMobile*/
const Value V12484_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12484_5 = {S52, 0, &V12484_5_1222, &P12484_1};	/*Property = ScreenPixelsHeight*/
const Value V12484_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12484_4 = {S54, 0, &V12484_4_1224, &P12484_5};	/*Property = ScreenPixelsWidth*/
const Profile P12484 = {"12484", &P12484_4};	/*Profile ID - 12484*/
const Value V16176_1_1 = {S2, NULL};	/*Value = True*/
const Property P16176_1 = {S50, 0, &V16176_1_1, NULL};	/*Property = IsMobile*/
const Value V16176_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16176_5 = {S52, 0, &V16176_5_1222, &P16176_1};	/*Property = ScreenPixelsHeight*/
const Value V16176_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16176_4 = {S54, 0, &V16176_4_1224, &P16176_5};	/*Property = ScreenPixelsWidth*/
const Profile P16176 = {"16176", &P16176_4};	/*Profile ID - 16176*/
const Value V23010_1_1 = {S2, NULL};	/*Value = True*/
const Property P23010_1 = {S50, 0, &V23010_1_1, NULL};	/*Property = IsMobile*/
const Value V23010_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23010_5 = {S52, 0, &V23010_5_1222, &P23010_1};	/*Property = ScreenPixelsHeight*/
const Value V23010_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23010_4 = {S54, 0, &V23010_4_1224, &P23010_5};	/*Property = ScreenPixelsWidth*/
const Profile P23010 = {"23010", &P23010_4};	/*Profile ID - 23010*/
const Value V15373_1_1 = {S2, NULL};	/*Value = True*/
const Property P15373_1 = {S50, 0, &V15373_1_1, NULL};	/*Property = IsMobile*/
const Value V15373_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P15373_5 = {S52, 0, &V15373_5_1275, &P15373_1};	/*Property = ScreenPixelsHeight*/
const Value V15373_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P15373_4 = {S54, 0, &V15373_4_1278, &P15373_5};	/*Property = ScreenPixelsWidth*/
const Profile P15373 = {"15373", &P15373_4};	/*Profile ID - 15373*/
const Value V11679_1_1 = {S2, NULL};	/*Value = True*/
const Property P11679_1 = {S50, 0, &V11679_1_1, NULL};	/*Property = IsMobile*/
const Value V11679_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P11679_5 = {S52, 0, &V11679_5_1280, &P11679_1};	/*Property = ScreenPixelsHeight*/
const Value V11679_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11679_4 = {S54, 0, &V11679_4_1235, &P11679_5};	/*Property = ScreenPixelsWidth*/
const Profile P11679 = {"11679", &P11679_4};	/*Profile ID - 11679*/
const Value V11681_1_1 = {S2, NULL};	/*Value = True*/
const Property P11681_1 = {S50, 0, &V11681_1_1, NULL};	/*Property = IsMobile*/
const Value V11681_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11681_5 = {S52, 0, &V11681_5_1293, &P11681_1};	/*Property = ScreenPixelsHeight*/
const Value V11681_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11681_4 = {S54, 0, &V11681_4_1296, &P11681_5};	/*Property = ScreenPixelsWidth*/
const Profile P11681 = {"11681", &P11681_4};	/*Profile ID - 11681*/
const Value V11683_1_1 = {S2, NULL};	/*Value = True*/
const Property P11683_1 = {S50, 0, &V11683_1_1, NULL};	/*Property = IsMobile*/
const Value V11683_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11683_5 = {S52, 0, &V11683_5_1222, &P11683_1};	/*Property = ScreenPixelsHeight*/
const Value V11683_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11683_4 = {S54, 0, &V11683_4_1224, &P11683_5};	/*Property = ScreenPixelsWidth*/
const Profile P11683 = {"11683", &P11683_4};	/*Profile ID - 11683*/
const Value V14705_1_1 = {S2, NULL};	/*Value = True*/
const Property P14705_1 = {S50, 0, &V14705_1_1, NULL};	/*Property = IsMobile*/
const Value V14705_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14705_5 = {S52, 0, &V14705_5_1222, &P14705_1};	/*Property = ScreenPixelsHeight*/
const Value V14705_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14705_4 = {S54, 0, &V14705_4_1224, &P14705_5};	/*Property = ScreenPixelsWidth*/
const Profile P14705 = {"14705", &P14705_4};	/*Profile ID - 14705*/
const Value V11692_1_1 = {S2, NULL};	/*Value = True*/
const Property P11692_1 = {S50, 0, &V11692_1_1, NULL};	/*Property = IsMobile*/
const Value V11692_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P11692_5 = {S52, 0, &V11692_5_1280, &P11692_1};	/*Property = ScreenPixelsHeight*/
const Value V11692_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11692_4 = {S54, 0, &V11692_4_1235, &P11692_5};	/*Property = ScreenPixelsWidth*/
const Profile P11692 = {"11692", &P11692_4};	/*Profile ID - 11692*/
const Value V11698_1_1 = {S2, NULL};	/*Value = True*/
const Property P11698_1 = {S50, 0, &V11698_1_1, NULL};	/*Property = IsMobile*/
const Value V11698_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11698_5 = {S52, 0, &V11698_5_1222, &P11698_1};	/*Property = ScreenPixelsHeight*/
const Value V11698_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11698_4 = {S54, 0, &V11698_4_1224, &P11698_5};	/*Property = ScreenPixelsWidth*/
const Profile P11698 = {"11698", &P11698_4};	/*Profile ID - 11698*/
const Value V11699_1_1 = {S2, NULL};	/*Value = True*/
const Property P11699_1 = {S50, 0, &V11699_1_1, NULL};	/*Property = IsMobile*/
const Value V11699_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11699_5 = {S52, 0, &V11699_5_1293, &P11699_1};	/*Property = ScreenPixelsHeight*/
const Value V11699_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11699_4 = {S54, 0, &V11699_4_1296, &P11699_5};	/*Property = ScreenPixelsWidth*/
const Profile P11699 = {"11699", &P11699_4};	/*Profile ID - 11699*/
const Value V11635_1_1 = {S2, NULL};	/*Value = True*/
const Property P11635_1 = {S50, 0, &V11635_1_1, NULL};	/*Property = IsMobile*/
const Value V11635_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P11635_5 = {S52, 0, &V11635_5_1294, &P11635_1};	/*Property = ScreenPixelsHeight*/
const Value V11635_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11635_4 = {S54, 0, &V11635_4_1296, &P11635_5};	/*Property = ScreenPixelsWidth*/
const Profile P11635 = {"11635", &P11635_4};	/*Profile ID - 11635*/
const Value V14286_1_1 = {S2, NULL};	/*Value = True*/
const Property P14286_1 = {S50, 0, &V14286_1_1, NULL};	/*Property = IsMobile*/
const Value V14286_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14286_5 = {S52, 0, &V14286_5_1227, &P14286_1};	/*Property = ScreenPixelsHeight*/
const Value V14286_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14286_4 = {S54, 0, &V14286_4_1229, &P14286_5};	/*Property = ScreenPixelsWidth*/
const Profile P14286 = {"14286", &P14286_4};	/*Profile ID - 14286*/
const Value V11700_1_1 = {S2, NULL};	/*Value = True*/
const Property P11700_1 = {S50, 0, &V11700_1_1, NULL};	/*Property = IsMobile*/
const Value V11700_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11700_5 = {S52, 0, &V11700_5_1233, &P11700_1};	/*Property = ScreenPixelsHeight*/
const Value V11700_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11700_4 = {S54, 0, &V11700_4_1235, &P11700_5};	/*Property = ScreenPixelsWidth*/
const Profile P11700 = {"11700", &P11700_4};	/*Profile ID - 11700*/
const Value V14604_1_1 = {S2, NULL};	/*Value = True*/
const Property P14604_1 = {S50, 0, &V14604_1_1, NULL};	/*Property = IsMobile*/
const Value V14604_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14604_5 = {S52, 0, &V14604_5_1277, &P14604_1};	/*Property = ScreenPixelsHeight*/
const Value V14604_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14604_4 = {S54, 0, &V14604_4_1279, &P14604_5};	/*Property = ScreenPixelsWidth*/
const Profile P14604 = {"14604", &P14604_4};	/*Profile ID - 14604*/
const Value V12488_1_1 = {S2, NULL};	/*Value = True*/
const Property P12488_1 = {S50, 0, &V12488_1_1, NULL};	/*Property = IsMobile*/
const Value V12488_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P12488_5 = {S52, 0, &V12488_5_1333, &P12488_1};	/*Property = ScreenPixelsHeight*/
const Value V12488_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P12488_4 = {S54, 0, &V12488_4_1335, &P12488_5};	/*Property = ScreenPixelsWidth*/
const Profile P12488 = {"12488", &P12488_4};	/*Profile ID - 12488*/
const Value V12490_1_1 = {S2, NULL};	/*Value = True*/
const Property P12490_1 = {S50, 0, &V12490_1_1, NULL};	/*Property = IsMobile*/
const Value V12490_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12490_5 = {S52, 0, &V12490_5_1233, &P12490_1};	/*Property = ScreenPixelsHeight*/
const Value V12490_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12490_4 = {S54, 0, &V12490_4_1235, &P12490_5};	/*Property = ScreenPixelsWidth*/
const Profile P12490 = {"12490", &P12490_4};	/*Profile ID - 12490*/
const Value V12495_1_1 = {S2, NULL};	/*Value = True*/
const Property P12495_1 = {S50, 0, &V12495_1_1, NULL};	/*Property = IsMobile*/
const Value V12495_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12495_5 = {S52, 0, &V12495_5_1222, &P12495_1};	/*Property = ScreenPixelsHeight*/
const Value V12495_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12495_4 = {S54, 0, &V12495_4_1224, &P12495_5};	/*Property = ScreenPixelsWidth*/
const Profile P12495 = {"12495", &P12495_4};	/*Profile ID - 12495*/
const Value V12497_1_1 = {S2, NULL};	/*Value = True*/
const Property P12497_1 = {S50, 0, &V12497_1_1, NULL};	/*Property = IsMobile*/
const Value V12497_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12497_5 = {S52, 0, &V12497_5_1233, &P12497_1};	/*Property = ScreenPixelsHeight*/
const Value V12497_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12497_4 = {S54, 0, &V12497_4_1235, &P12497_5};	/*Property = ScreenPixelsWidth*/
const Profile P12497 = {"12497", &P12497_4};	/*Profile ID - 12497*/
const Value V14669_1_1 = {S2, NULL};	/*Value = True*/
const Property P14669_1 = {S50, 0, &V14669_1_1, NULL};	/*Property = IsMobile*/
const Value V14669_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14669_5 = {S52, 0, &V14669_5_1222, &P14669_1};	/*Property = ScreenPixelsHeight*/
const Value V14669_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14669_4 = {S54, 0, &V14669_4_1224, &P14669_5};	/*Property = ScreenPixelsWidth*/
const Profile P14669 = {"14669", &P14669_4};	/*Profile ID - 14669*/
const Value V14830_1_1 = {S2, NULL};	/*Value = True*/
const Property P14830_1 = {S50, 0, &V14830_1_1, NULL};	/*Property = IsMobile*/
const Value V14830_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14830_5 = {S52, 0, &V14830_5_1380, &P14830_1};	/*Property = ScreenPixelsHeight*/
const Value V14830_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14830_4 = {S54, 0, &V14830_4_1279, &P14830_5};	/*Property = ScreenPixelsWidth*/
const Profile P14830 = {"14830", &P14830_4};	/*Profile ID - 14830*/
const Value V12580_1_1 = {S2, NULL};	/*Value = True*/
const Property P12580_1 = {S50, 0, &V12580_1_1, NULL};	/*Property = IsMobile*/
const Value V12580_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12580_5 = {S52, 0, &V12580_5_1294, &P12580_1};	/*Property = ScreenPixelsHeight*/
const Value V12580_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12580_4 = {S54, 0, &V12580_4_1296, &P12580_5};	/*Property = ScreenPixelsWidth*/
const Profile P12580 = {"12580", &P12580_4};	/*Profile ID - 12580*/
const Value V13552_1_1 = {S2, NULL};	/*Value = True*/
const Property P13552_1 = {S50, 0, &V13552_1_1, NULL};	/*Property = IsMobile*/
const Value V13552_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P13552_5 = {S52, 0, &V13552_5_1275, &P13552_1};	/*Property = ScreenPixelsHeight*/
const Value V13552_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P13552_4 = {S54, 0, &V13552_4_1278, &P13552_5};	/*Property = ScreenPixelsWidth*/
const Profile P13552 = {"13552", &P13552_4};	/*Profile ID - 13552*/
const Value V13706_1_1 = {S2, NULL};	/*Value = True*/
const Property P13706_1 = {S50, 0, &V13706_1_1, NULL};	/*Property = IsMobile*/
const Value V13706_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P13706_5 = {S52, 0, &V13706_5_1275, &P13706_1};	/*Property = ScreenPixelsHeight*/
const Value V13706_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P13706_4 = {S54, 0, &V13706_4_1278, &P13706_5};	/*Property = ScreenPixelsWidth*/
const Profile P13706 = {"13706", &P13706_4};	/*Profile ID - 13706*/
const Value V14453_1_1 = {S2, NULL};	/*Value = True*/
const Property P14453_1 = {S50, 0, &V14453_1_1, NULL};	/*Property = IsMobile*/
const Value V14453_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14453_5 = {S52, 0, &V14453_5_1222, &P14453_1};	/*Property = ScreenPixelsHeight*/
const Value V14453_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14453_4 = {S54, 0, &V14453_4_1224, &P14453_5};	/*Property = ScreenPixelsWidth*/
const Profile P14453 = {"14453", &P14453_4};	/*Profile ID - 14453*/
const Value V17968_1_1 = {S2, NULL};	/*Value = True*/
const Property P17968_1 = {S50, 0, &V17968_1_1, NULL};	/*Property = IsMobile*/
const Value V17968_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P17968_5 = {S52, 0, &V17968_5_37, &P17968_1};	/*Property = ScreenPixelsHeight*/
const Value V17968_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P17968_4 = {S54, 0, &V17968_4_36, &P17968_5};	/*Property = ScreenPixelsWidth*/
const Profile P17968 = {"17968", &P17968_4};	/*Profile ID - 17968*/
const Value V11703_1_1 = {S2, NULL};	/*Value = True*/
const Property P11703_1 = {S50, 0, &V11703_1_1, NULL};	/*Property = IsMobile*/
const Value V11703_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11703_5 = {S52, 0, &V11703_5_1293, &P11703_1};	/*Property = ScreenPixelsHeight*/
const Value V11703_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11703_4 = {S54, 0, &V11703_4_1296, &P11703_5};	/*Property = ScreenPixelsWidth*/
const Profile P11703 = {"11703", &P11703_4};	/*Profile ID - 11703*/
const Value V14372_1_1 = {S2, NULL};	/*Value = True*/
const Property P14372_1 = {S50, 0, &V14372_1_1, NULL};	/*Property = IsMobile*/
const Value V14372_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14372_5 = {S52, 0, &V14372_5_1304, &P14372_1};	/*Property = ScreenPixelsHeight*/
const Value V14372_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14372_4 = {S54, 0, &V14372_4_1224, &P14372_5};	/*Property = ScreenPixelsWidth*/
const Profile P14372 = {"14372", &P14372_4};	/*Profile ID - 14372*/
const Value V11797_1_1 = {S2, NULL};	/*Value = True*/
const Property P11797_1 = {S50, 0, &V11797_1_1, NULL};	/*Property = IsMobile*/
const Value V11797_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11797_5 = {S52, 0, &V11797_5_1222, &P11797_1};	/*Property = ScreenPixelsHeight*/
const Value V11797_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11797_4 = {S54, 0, &V11797_4_1224, &P11797_5};	/*Property = ScreenPixelsWidth*/
const Profile P11797 = {"11797", &P11797_4};	/*Profile ID - 11797*/
const Value V13634_1_1 = {S2, NULL};	/*Value = True*/
const Property P13634_1 = {S50, 0, &V13634_1_1, NULL};	/*Property = IsMobile*/
const Value V13634_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13634_5 = {S52, 0, &V13634_5_1304, &P13634_1};	/*Property = ScreenPixelsHeight*/
const Value V13634_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13634_4 = {S54, 0, &V13634_4_1224, &P13634_5};	/*Property = ScreenPixelsWidth*/
const Profile P13634 = {"13634", &P13634_4};	/*Profile ID - 13634*/
const Value V14433_1_1 = {S2, NULL};	/*Value = True*/
const Property P14433_1 = {S50, 0, &V14433_1_1, NULL};	/*Property = IsMobile*/
const Value V14433_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14433_5 = {S52, 0, &V14433_5_1227, &P14433_1};	/*Property = ScreenPixelsHeight*/
const Value V14433_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14433_4 = {S54, 0, &V14433_4_1229, &P14433_5};	/*Property = ScreenPixelsWidth*/
const Profile P14433 = {"14433", &P14433_4};	/*Profile ID - 14433*/
const Value V13333_1_1 = {S2, NULL};	/*Value = True*/
const Property P13333_1 = {S50, 0, &V13333_1_1, NULL};	/*Property = IsMobile*/
const Value V13333_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13333_5 = {S52, 0, &V13333_5_1380, &P13333_1};	/*Property = ScreenPixelsHeight*/
const Value V13333_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13333_4 = {S54, 0, &V13333_4_1279, &P13333_5};	/*Property = ScreenPixelsWidth*/
const Profile P13333 = {"13333", &P13333_4};	/*Profile ID - 13333*/
const Value V13352_1_1 = {S2, NULL};	/*Value = True*/
const Property P13352_1 = {S50, 0, &V13352_1_1, NULL};	/*Property = IsMobile*/
const Value V13352_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13352_5 = {S52, 0, &V13352_5_1293, &P13352_1};	/*Property = ScreenPixelsHeight*/
const Value V13352_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13352_4 = {S54, 0, &V13352_4_1296, &P13352_5};	/*Property = ScreenPixelsWidth*/
const Profile P13352 = {"13352", &P13352_4};	/*Profile ID - 13352*/
const Value V13361_1_1 = {S2, NULL};	/*Value = True*/
const Property P13361_1 = {S50, 0, &V13361_1_1, NULL};	/*Property = IsMobile*/
const Value V13361_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13361_5 = {S52, 0, &V13361_5_1222, &P13361_1};	/*Property = ScreenPixelsHeight*/
const Value V13361_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13361_4 = {S54, 0, &V13361_4_1224, &P13361_5};	/*Property = ScreenPixelsWidth*/
const Profile P13361 = {"13361", &P13361_4};	/*Profile ID - 13361*/
const Value V13363_1_1 = {S2, NULL};	/*Value = True*/
const Property P13363_1 = {S50, 0, &V13363_1_1, NULL};	/*Property = IsMobile*/
const Value V13363_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13363_5 = {S52, 0, &V13363_5_1277, &P13363_1};	/*Property = ScreenPixelsHeight*/
const Value V13363_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13363_4 = {S54, 0, &V13363_4_1279, &P13363_5};	/*Property = ScreenPixelsWidth*/
const Profile P13363 = {"13363", &P13363_4};	/*Profile ID - 13363*/
const Value V17944_1_1 = {S2, NULL};	/*Value = True*/
const Property P17944_1 = {S50, 0, &V17944_1_1, NULL};	/*Property = IsMobile*/
const Value V17944_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17944_5 = {S52, 0, &V17944_5_1227, &P17944_1};	/*Property = ScreenPixelsHeight*/
const Value V17944_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17944_4 = {S54, 0, &V17944_4_1229, &P17944_5};	/*Property = ScreenPixelsWidth*/
const Profile P17944 = {"17944", &P17944_4};	/*Profile ID - 17944*/
const Value V13367_1_1 = {S2, NULL};	/*Value = True*/
const Property P13367_1 = {S50, 0, &V13367_1_1, NULL};	/*Property = IsMobile*/
const Value V13367_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13367_5 = {S52, 0, &V13367_5_1222, &P13367_1};	/*Property = ScreenPixelsHeight*/
const Value V13367_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13367_4 = {S54, 0, &V13367_4_1224, &P13367_5};	/*Property = ScreenPixelsWidth*/
const Profile P13367 = {"13367", &P13367_4};	/*Profile ID - 13367*/
const Value V13381_1_1 = {S2, NULL};	/*Value = True*/
const Property P13381_1 = {S50, 0, &V13381_1_1, NULL};	/*Property = IsMobile*/
const Value V13381_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13381_5 = {S52, 0, &V13381_5_1222, &P13381_1};	/*Property = ScreenPixelsHeight*/
const Value V13381_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13381_4 = {S54, 0, &V13381_4_1224, &P13381_5};	/*Property = ScreenPixelsWidth*/
const Profile P13381 = {"13381", &P13381_4};	/*Profile ID - 13381*/
const Value V13384_1_1 = {S2, NULL};	/*Value = True*/
const Property P13384_1 = {S50, 0, &V13384_1_1, NULL};	/*Property = IsMobile*/
const Value V13384_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13384_5 = {S52, 0, &V13384_5_1222, &P13384_1};	/*Property = ScreenPixelsHeight*/
const Value V13384_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13384_4 = {S54, 0, &V13384_4_1224, &P13384_5};	/*Property = ScreenPixelsWidth*/
const Profile P13384 = {"13384", &P13384_4};	/*Profile ID - 13384*/
const Value V13386_1_1 = {S2, NULL};	/*Value = True*/
const Property P13386_1 = {S50, 0, &V13386_1_1, NULL};	/*Property = IsMobile*/
const Value V13386_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13386_5 = {S52, 0, &V13386_5_1304, &P13386_1};	/*Property = ScreenPixelsHeight*/
const Value V13386_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13386_4 = {S54, 0, &V13386_4_1224, &P13386_5};	/*Property = ScreenPixelsWidth*/
const Profile P13386 = {"13386", &P13386_4};	/*Profile ID - 13386*/
const Value V11742_1_1 = {S2, NULL};	/*Value = True*/
const Property P11742_1 = {S50, 0, &V11742_1_1, NULL};	/*Property = IsMobile*/
const Value V11742_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11742_5 = {S52, 0, &V11742_5_1233, &P11742_1};	/*Property = ScreenPixelsHeight*/
const Value V11742_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11742_4 = {S54, 0, &V11742_4_1235, &P11742_5};	/*Property = ScreenPixelsWidth*/
const Profile P11742 = {"11742", &P11742_4};	/*Profile ID - 11742*/
const Value V14042_1_1 = {S2, NULL};	/*Value = True*/
const Property P14042_1 = {S50, 0, &V14042_1_1, NULL};	/*Property = IsMobile*/
const Value V14042_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14042_5 = {S52, 0, &V14042_5_1227, &P14042_1};	/*Property = ScreenPixelsHeight*/
const Value V14042_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14042_4 = {S54, 0, &V14042_4_1229, &P14042_5};	/*Property = ScreenPixelsWidth*/
const Profile P14042 = {"14042", &P14042_4};	/*Profile ID - 14042*/
const Value V15158_1_1 = {S2, NULL};	/*Value = True*/
const Property P15158_1 = {S50, 0, &V15158_1_1, NULL};	/*Property = IsMobile*/
const Value V15158_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15158_5 = {S52, 0, &V15158_5_1222, &P15158_1};	/*Property = ScreenPixelsHeight*/
const Value V15158_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15158_4 = {S54, 0, &V15158_4_1224, &P15158_5};	/*Property = ScreenPixelsWidth*/
const Profile P15158 = {"15158", &P15158_4};	/*Profile ID - 15158*/
const Value V16584_1_1 = {S2, NULL};	/*Value = True*/
const Property P16584_1 = {S50, 0, &V16584_1_1, NULL};	/*Property = IsMobile*/
const Value V16584_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16584_5 = {S52, 0, &V16584_5_1227, &P16584_1};	/*Property = ScreenPixelsHeight*/
const Value V16584_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16584_4 = {S54, 0, &V16584_4_1229, &P16584_5};	/*Property = ScreenPixelsWidth*/
const Profile P16584 = {"16584", &P16584_4};	/*Profile ID - 16584*/
const Value V16383_1_1 = {S2, NULL};	/*Value = True*/
const Property P16383_1 = {S50, 0, &V16383_1_1, NULL};	/*Property = IsMobile*/
const Value V16383_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16383_5 = {S52, 0, &V16383_5_1222, &P16383_1};	/*Property = ScreenPixelsHeight*/
const Value V16383_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16383_4 = {S54, 0, &V16383_4_1224, &P16383_5};	/*Property = ScreenPixelsWidth*/
const Profile P16383 = {"16383", &P16383_4};	/*Profile ID - 16383*/
const Value V12154_1_1 = {S2, NULL};	/*Value = True*/
const Property P12154_1 = {S50, 0, &V12154_1_1, NULL};	/*Property = IsMobile*/
const Value V12154_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12154_5 = {S52, 0, &V12154_5_1233, &P12154_1};	/*Property = ScreenPixelsHeight*/
const Value V12154_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12154_4 = {S54, 0, &V12154_4_1235, &P12154_5};	/*Property = ScreenPixelsWidth*/
const Profile P12154 = {"12154", &P12154_4};	/*Profile ID - 12154*/
const Value V12643_1_1 = {S2, NULL};	/*Value = True*/
const Property P12643_1 = {S50, 0, &V12643_1_1, NULL};	/*Property = IsMobile*/
const Value V12643_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12643_5 = {S52, 0, &V12643_5_1293, &P12643_1};	/*Property = ScreenPixelsHeight*/
const Value V12643_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12643_4 = {S54, 0, &V12643_4_1296, &P12643_5};	/*Property = ScreenPixelsWidth*/
const Profile P12643 = {"12643", &P12643_4};	/*Profile ID - 12643*/
const Value V13408_1_1 = {S2, NULL};	/*Value = True*/
const Property P13408_1 = {S50, 0, &V13408_1_1, NULL};	/*Property = IsMobile*/
const Value V13408_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13408_5 = {S52, 0, &V13408_5_1304, &P13408_1};	/*Property = ScreenPixelsHeight*/
const Value V13408_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13408_4 = {S54, 0, &V13408_4_1224, &P13408_5};	/*Property = ScreenPixelsWidth*/
const Profile P13408 = {"13408", &P13408_4};	/*Profile ID - 13408*/
const Value V13407_1_1 = {S2, NULL};	/*Value = True*/
const Property P13407_1 = {S50, 0, &V13407_1_1, NULL};	/*Property = IsMobile*/
const Value V13407_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13407_5 = {S52, 0, &V13407_5_1222, &P13407_1};	/*Property = ScreenPixelsHeight*/
const Value V13407_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13407_4 = {S54, 0, &V13407_4_1224, &P13407_5};	/*Property = ScreenPixelsWidth*/
const Profile P13407 = {"13407", &P13407_4};	/*Profile ID - 13407*/
const Value V13406_1_1 = {S2, NULL};	/*Value = True*/
const Property P13406_1 = {S50, 0, &V13406_1_1, NULL};	/*Property = IsMobile*/
const Value V13406_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13406_5 = {S52, 0, &V13406_5_1294, &P13406_1};	/*Property = ScreenPixelsHeight*/
const Value V13406_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13406_4 = {S54, 0, &V13406_4_1296, &P13406_5};	/*Property = ScreenPixelsWidth*/
const Profile P13406 = {"13406", &P13406_4};	/*Profile ID - 13406*/
const Value V13591_1_1 = {S2, NULL};	/*Value = True*/
const Property P13591_1 = {S50, 0, &V13591_1_1, NULL};	/*Property = IsMobile*/
const Value V13591_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13591_5 = {S52, 0, &V13591_5_1227, &P13591_1};	/*Property = ScreenPixelsHeight*/
const Value V13591_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13591_4 = {S54, 0, &V13591_4_1229, &P13591_5};	/*Property = ScreenPixelsWidth*/
const Profile P13591 = {"13591", &P13591_4};	/*Profile ID - 13591*/
const Value V13413_1_1 = {S2, NULL};	/*Value = True*/
const Property P13413_1 = {S50, 0, &V13413_1_1, NULL};	/*Property = IsMobile*/
const Value V13413_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13413_5 = {S52, 0, &V13413_5_1227, &P13413_1};	/*Property = ScreenPixelsHeight*/
const Value V13413_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13413_4 = {S54, 0, &V13413_4_1229, &P13413_5};	/*Property = ScreenPixelsWidth*/
const Profile P13413 = {"13413", &P13413_4};	/*Profile ID - 13413*/
const Value V13409_1_1 = {S2, NULL};	/*Value = True*/
const Property P13409_1 = {S50, 0, &V13409_1_1, NULL};	/*Property = IsMobile*/
const Value V13409_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13409_5 = {S52, 0, &V13409_5_1222, &P13409_1};	/*Property = ScreenPixelsHeight*/
const Value V13409_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13409_4 = {S54, 0, &V13409_4_1224, &P13409_5};	/*Property = ScreenPixelsWidth*/
const Profile P13409 = {"13409", &P13409_4};	/*Profile ID - 13409*/
const Value V11765_1_1 = {S2, NULL};	/*Value = True*/
const Property P11765_1 = {S50, 0, &V11765_1_1, NULL};	/*Property = IsMobile*/
const Value V11765_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P11765_5 = {S52, 0, &V11765_5_1304, &P11765_1};	/*Property = ScreenPixelsHeight*/
const Value V11765_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11765_4 = {S54, 0, &V11765_4_1224, &P11765_5};	/*Property = ScreenPixelsWidth*/
const Profile P11765 = {"11765", &P11765_4};	/*Profile ID - 11765*/
const Value V11708_1_1 = {S2, NULL};	/*Value = True*/
const Property P11708_1 = {S50, 0, &V11708_1_1, NULL};	/*Property = IsMobile*/
const Value V11708_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11708_5 = {S52, 0, &V11708_5_1233, &P11708_1};	/*Property = ScreenPixelsHeight*/
const Value V11708_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11708_4 = {S54, 0, &V11708_4_1235, &P11708_5};	/*Property = ScreenPixelsWidth*/
const Profile P11708 = {"11708", &P11708_4};	/*Profile ID - 11708*/
const Value V11709_1_1 = {S2, NULL};	/*Value = True*/
const Property P11709_1 = {S50, 0, &V11709_1_1, NULL};	/*Property = IsMobile*/
const Value V11709_5_37454 = {S80, NULL};	/*Value = 200*/
const Property P11709_5 = {S52, 0, &V11709_5_37454, &P11709_1};	/*Property = ScreenPixelsHeight*/
const Value V11709_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P11709_4 = {S54, 0, &V11709_4_8554, &P11709_5};	/*Property = ScreenPixelsWidth*/
const Profile P11709 = {"11709", &P11709_4};	/*Profile ID - 11709*/
const Value V11710_1_1 = {S2, NULL};	/*Value = True*/
const Property P11710_1 = {S50, 0, &V11710_1_1, NULL};	/*Property = IsMobile*/
const Value V11710_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11710_5 = {S52, 0, &V11710_5_1293, &P11710_1};	/*Property = ScreenPixelsHeight*/
const Value V11710_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11710_4 = {S54, 0, &V11710_4_1296, &P11710_5};	/*Property = ScreenPixelsWidth*/
const Profile P11710 = {"11710", &P11710_4};	/*Profile ID - 11710*/
const Value V12061_1_1 = {S2, NULL};	/*Value = True*/
const Property P12061_1 = {S50, 0, &V12061_1_1, NULL};	/*Property = IsMobile*/
const Value V12061_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12061_5 = {S52, 0, &V12061_5_1222, &P12061_1};	/*Property = ScreenPixelsHeight*/
const Value V12061_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12061_4 = {S54, 0, &V12061_4_1224, &P12061_5};	/*Property = ScreenPixelsWidth*/
const Profile P12061 = {"12061", &P12061_4};	/*Profile ID - 12061*/
const Value V13418_1_1 = {S2, NULL};	/*Value = True*/
const Property P13418_1 = {S50, 0, &V13418_1_1, NULL};	/*Property = IsMobile*/
const Value V13418_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13418_5 = {S52, 0, &V13418_5_1304, &P13418_1};	/*Property = ScreenPixelsHeight*/
const Value V13418_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13418_4 = {S54, 0, &V13418_4_1224, &P13418_5};	/*Property = ScreenPixelsWidth*/
const Profile P13418 = {"13418", &P13418_4};	/*Profile ID - 13418*/
const Value V13505_1_1 = {S2, NULL};	/*Value = True*/
const Property P13505_1 = {S50, 0, &V13505_1_1, NULL};	/*Property = IsMobile*/
const Value V13505_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P13505_5 = {S52, 0, &V13505_5_1269, &P13505_1};	/*Property = ScreenPixelsHeight*/
const Value V13505_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13505_4 = {S54, 0, &V13505_4_1229, &P13505_5};	/*Property = ScreenPixelsWidth*/
const Profile P13505 = {"13505", &P13505_4};	/*Profile ID - 13505*/
const Value V13420_1_1 = {S2, NULL};	/*Value = True*/
const Property P13420_1 = {S50, 0, &V13420_1_1, NULL};	/*Property = IsMobile*/
const Value V13420_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13420_5 = {S52, 0, &V13420_5_1380, &P13420_1};	/*Property = ScreenPixelsHeight*/
const Value V13420_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13420_4 = {S54, 0, &V13420_4_1279, &P13420_5};	/*Property = ScreenPixelsWidth*/
const Profile P13420 = {"13420", &P13420_4};	/*Profile ID - 13420*/
const Value V11712_1_1 = {S2, NULL};	/*Value = True*/
const Property P11712_1 = {S50, 0, &V11712_1_1, NULL};	/*Property = IsMobile*/
const Value V11712_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P11712_5 = {S52, 0, &V11712_5_1294, &P11712_1};	/*Property = ScreenPixelsHeight*/
const Value V11712_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11712_4 = {S54, 0, &V11712_4_1296, &P11712_5};	/*Property = ScreenPixelsWidth*/
const Profile P11712 = {"11712", &P11712_4};	/*Profile ID - 11712*/
const Value V11713_1_1 = {S2, NULL};	/*Value = True*/
const Property P11713_1 = {S50, 0, &V11713_1_1, NULL};	/*Property = IsMobile*/
const Value V11713_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11713_5 = {S52, 0, &V11713_5_1222, &P11713_1};	/*Property = ScreenPixelsHeight*/
const Value V11713_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11713_4 = {S54, 0, &V11713_4_1224, &P11713_5};	/*Property = ScreenPixelsWidth*/
const Profile P11713 = {"11713", &P11713_4};	/*Profile ID - 11713*/
const Value V17718_1_1 = {S2, NULL};	/*Value = True*/
const Property P17718_1 = {S50, 0, &V17718_1_1, NULL};	/*Property = IsMobile*/
const Value V17718_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17718_5 = {S52, 0, &V17718_5_1269, &P17718_1};	/*Property = ScreenPixelsHeight*/
const Value V17718_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17718_4 = {S54, 0, &V17718_4_1229, &P17718_5};	/*Property = ScreenPixelsWidth*/
const Profile P17718 = {"17718", &P17718_4};	/*Profile ID - 17718*/
const Value V13998_1_1 = {S2, NULL};	/*Value = True*/
const Property P13998_1 = {S50, 0, &V13998_1_1, NULL};	/*Property = IsMobile*/
const Value V13998_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13998_5 = {S52, 0, &V13998_5_1222, &P13998_1};	/*Property = ScreenPixelsHeight*/
const Value V13998_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13998_4 = {S54, 0, &V13998_4_1224, &P13998_5};	/*Property = ScreenPixelsWidth*/
const Profile P13998 = {"13998", &P13998_4};	/*Profile ID - 13998*/
const Value V16381_1_1 = {S2, NULL};	/*Value = True*/
const Property P16381_1 = {S50, 0, &V16381_1_1, NULL};	/*Property = IsMobile*/
const Value V16381_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16381_5 = {S52, 0, &V16381_5_1222, &P16381_1};	/*Property = ScreenPixelsHeight*/
const Value V16381_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16381_4 = {S54, 0, &V16381_4_1224, &P16381_5};	/*Property = ScreenPixelsWidth*/
const Profile P16381 = {"16381", &P16381_4};	/*Profile ID - 16381*/
const Value V11716_1_1 = {S2, NULL};	/*Value = True*/
const Property P11716_1 = {S50, 0, &V11716_1_1, NULL};	/*Property = IsMobile*/
const Value V11716_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11716_5 = {S52, 0, &V11716_5_1222, &P11716_1};	/*Property = ScreenPixelsHeight*/
const Value V11716_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11716_4 = {S54, 0, &V11716_4_1224, &P11716_5};	/*Property = ScreenPixelsWidth*/
const Profile P11716 = {"11716", &P11716_4};	/*Profile ID - 11716*/
const Value V22303_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_279 = {S1, 0, &V22303_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22303_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_282 = {S3, 0, &V22303_282_57155, &P22303_279};	/*Property = BlobBuilder*/
const Value V22303_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_350 = {S4, 0, &V22303_350_57446, &P22303_282};	/*Property = Canvas*/
const Value V22303_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_283 = {S5, 0, &V22303_283_57161, &P22303_350};	/*Property = CssBackground*/
const Value V22303_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_284 = {S6, 0, &V22303_284_57164, &P22303_283};	/*Property = CssBorderImage*/
const Value V22303_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_285 = {S7, 0, &V22303_285_57167, &P22303_284};	/*Property = CssCanvas*/
const Value V22303_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_286 = {S8, 0, &V22303_286_57170, &P22303_285};	/*Property = CssColor*/
const Value V22303_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_237 = {S9, 0, &V22303_237_55786, &P22303_286};	/*Property = CssColumn*/
const Value V22303_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_287 = {S10, 0, &V22303_287_57173, &P22303_237};	/*Property = CssFlexbox*/
const Value V22303_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_288 = {S11, 0, &V22303_288_57176, &P22303_287};	/*Property = CssFont*/
const Value V22303_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_289 = {S12, 0, &V22303_289_86899, &P22303_288};	/*Property = CssImages*/
const Value V22303_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_291 = {S13, 0, &V22303_291_57182, &P22303_289};	/*Property = CssMediaQueries*/
const Value V22303_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_292 = {S14, 0, &V22303_292_57185, &P22303_291};	/*Property = CssMinMax*/
const Value V22303_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_293 = {S15, 0, &V22303_293_57188, &P22303_292};	/*Property = CssOverflow*/
const Value V22303_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_294 = {S16, 0, &V22303_294_57191, &P22303_293};	/*Property = CssPosition*/
const Value V22303_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_295 = {S17, 0, &V22303_295_57194, &P22303_294};	/*Property = CssText*/
const Value V22303_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_297 = {S18, 0, &V22303_297_57197, &P22303_295};	/*Property = CssTransforms*/
const Value V22303_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_241 = {S19, 0, &V22303_241_55801, &P22303_297};	/*Property = CssTransitions*/
const Value V22303_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_298 = {S20, 0, &V22303_298_57208, &P22303_241};	/*Property = CssUI*/
const Value V22303_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_300 = {S21, 0, &V22303_300_57217, &P22303_298};	/*Property = DataSet*/
const Value V22303_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_301 = {S22, 0, &V22303_301_57220, &P22303_300};	/*Property = DataUrl*/
const Value V22303_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_302 = {S23, 0, &V22303_302_57223, &P22303_301};	/*Property = DeviceOrientation*/
const Value V22303_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_304 = {S24, 0, &V22303_304_57229, &P22303_302};	/*Property = FileReader*/
const Value V22303_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_305 = {S25, 0, &V22303_305_57232, &P22303_304};	/*Property = FileSaver*/
const Value V22303_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_306 = {S26, 0, &V22303_306_57238, &P22303_305};	/*Property = FileWriter*/
const Value V22303_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_307 = {S27, 0, &V22303_307_57241, &P22303_306};	/*Property = FormData*/
const Value V22303_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_309 = {S28, 0, &V22303_309_57247, &P22303_307};	/*Property = Fullscreen*/
const Value V22303_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_310 = {S29, 0, &V22303_310_57250, &P22303_309};	/*Property = GeoLocation*/
const Value V22303_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_312 = {S30, 0, &V22303_312_57256, &P22303_310};	/*Property = History*/
const Value V22303_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_314 = {S31, 0, &V22303_314_57262, &P22303_312};	/*Property = Html5*/
const Value V22303_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_313 = {S32, 0, &V22303_313_57259, &P22303_314};	/*Property = Html-Media-Capture*/
const Value V22303_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_315 = {S33, 0, &V22303_315_57265, &P22303_313};	/*Property = Iframe*/
const Value V22303_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_316 = {S34, 0, &V22303_316_57268, &P22303_315};	/*Property = IndexedDB*/
const Value V22303_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_317 = {S35, 0, &V22303_317_57271, &P22303_316};	/*Property = Json*/
const Value V22303_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_15 = {S37, 0, &V22303_15_72, &P22303_317};	/*Property = LayoutEngine*/
const Value V22303_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_344 = {S38, 0, &V22303_344_57274, &P22303_15};	/*Property = Masking*/
const Value V22303_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_320 = {S39, 0, &V22303_320_57292, &P22303_344};	/*Property = PostMessage*/
const Value V22303_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_321 = {S40, 0, &V22303_321_57295, &P22303_320};	/*Property = Progress*/
const Value V22303_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_322 = {S41, 0, &V22303_322_57298, &P22303_321};	/*Property = Prompts*/
const Value V22303_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_323 = {S42, 0, &V22303_323_57301, &P22303_322};	/*Property = Selector*/
const Value V22303_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_324 = {S43, 0, &V22303_324_57307, &P22303_323};	/*Property = Svg*/
const Value V22303_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_327 = {S44, 0, &V22303_327_57316, &P22303_324};	/*Property = TouchEvents*/
const Value V22303_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_328 = {S45, 0, &V22303_328_57319, &P22303_327};	/*Property = Track*/
const Value V22303_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_329 = {S46, 0, &V22303_329_57325, &P22303_328};	/*Property = Video*/
const Value V22303_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_330 = {S47, 0, &V22303_330_57328, &P22303_329};	/*Property = Viewport*/
const Value V22303_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_332 = {S48, 0, &V22303_332_57340, &P22303_330};	/*Property = WebWorkers*/
const Value V22303_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22303_333 = {S49, 0, &V22303_333_57343, &P22303_332};	/*Property = Xhr2*/
const Profile P22303 = {"22303", &P22303_333};	/*Profile ID - 22303*/
const Value V11717_1_1 = {S2, NULL};	/*Value = True*/
const Property P11717_1 = {S50, 0, &V11717_1_1, NULL};	/*Property = IsMobile*/
const Value V11717_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P11717_5 = {S52, 0, &V11717_5_1277, &P11717_1};	/*Property = ScreenPixelsHeight*/
const Value V11717_4_37618 = {S83, NULL};	/*Value = 272*/
const Property P11717_4 = {S54, 0, &V11717_4_37618, &P11717_5};	/*Property = ScreenPixelsWidth*/
const Profile P11717 = {"11717", &P11717_4};	/*Profile ID - 11717*/
const Value V14741_1_1 = {S2, NULL};	/*Value = True*/
const Property P14741_1 = {S50, 0, &V14741_1_1, NULL};	/*Property = IsMobile*/
const Value V14741_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14741_5 = {S52, 0, &V14741_5_1293, &P14741_1};	/*Property = ScreenPixelsHeight*/
const Value V14741_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14741_4 = {S54, 0, &V14741_4_1296, &P14741_5};	/*Property = ScreenPixelsWidth*/
const Profile P14741 = {"14741", &P14741_4};	/*Profile ID - 14741*/
const Value V13445_1_1 = {S2, NULL};	/*Value = True*/
const Property P13445_1 = {S50, 0, &V13445_1_1, NULL};	/*Property = IsMobile*/
const Value V13445_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13445_5 = {S52, 0, &V13445_5_1222, &P13445_1};	/*Property = ScreenPixelsHeight*/
const Value V13445_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13445_4 = {S54, 0, &V13445_4_1224, &P13445_5};	/*Property = ScreenPixelsWidth*/
const Profile P13445 = {"13445", &P13445_4};	/*Profile ID - 13445*/
const Value V3581_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_279 = {S1, 0, &V3581_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3581_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_282 = {S3, 0, &V3581_282_57155, &P3581_279};	/*Property = BlobBuilder*/
const Value V3581_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_350 = {S4, 0, &V3581_350_57446, &P3581_282};	/*Property = Canvas*/
const Value V3581_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_283 = {S5, 0, &V3581_283_57161, &P3581_350};	/*Property = CssBackground*/
const Value V3581_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_284 = {S6, 0, &V3581_284_57164, &P3581_283};	/*Property = CssBorderImage*/
const Value V3581_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_285 = {S7, 0, &V3581_285_57167, &P3581_284};	/*Property = CssCanvas*/
const Value V3581_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_286 = {S8, 0, &V3581_286_57170, &P3581_285};	/*Property = CssColor*/
const Value V3581_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_237 = {S9, 0, &V3581_237_55786, &P3581_286};	/*Property = CssColumn*/
const Value V3581_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_287 = {S10, 0, &V3581_287_57173, &P3581_237};	/*Property = CssFlexbox*/
const Value V3581_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_288 = {S11, 0, &V3581_288_57176, &P3581_287};	/*Property = CssFont*/
const Value V3581_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_289 = {S12, 0, &V3581_289_86899, &P3581_288};	/*Property = CssImages*/
const Value V3581_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_291 = {S13, 0, &V3581_291_57182, &P3581_289};	/*Property = CssMediaQueries*/
const Value V3581_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_292 = {S14, 0, &V3581_292_57185, &P3581_291};	/*Property = CssMinMax*/
const Value V3581_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_293 = {S15, 0, &V3581_293_57188, &P3581_292};	/*Property = CssOverflow*/
const Value V3581_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_294 = {S16, 0, &V3581_294_57191, &P3581_293};	/*Property = CssPosition*/
const Value V3581_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_295 = {S17, 0, &V3581_295_57194, &P3581_294};	/*Property = CssText*/
const Value V3581_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_297 = {S18, 0, &V3581_297_57197, &P3581_295};	/*Property = CssTransforms*/
const Value V3581_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_241 = {S19, 0, &V3581_241_55801, &P3581_297};	/*Property = CssTransitions*/
const Value V3581_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_298 = {S20, 0, &V3581_298_57208, &P3581_241};	/*Property = CssUI*/
const Value V3581_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_300 = {S21, 0, &V3581_300_57217, &P3581_298};	/*Property = DataSet*/
const Value V3581_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_301 = {S22, 0, &V3581_301_57220, &P3581_300};	/*Property = DataUrl*/
const Value V3581_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_302 = {S23, 0, &V3581_302_57223, &P3581_301};	/*Property = DeviceOrientation*/
const Value V3581_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_304 = {S24, 0, &V3581_304_57229, &P3581_302};	/*Property = FileReader*/
const Value V3581_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_305 = {S25, 0, &V3581_305_57232, &P3581_304};	/*Property = FileSaver*/
const Value V3581_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_306 = {S26, 0, &V3581_306_57238, &P3581_305};	/*Property = FileWriter*/
const Value V3581_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_307 = {S27, 0, &V3581_307_57241, &P3581_306};	/*Property = FormData*/
const Value V3581_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_309 = {S28, 0, &V3581_309_57247, &P3581_307};	/*Property = Fullscreen*/
const Value V3581_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_310 = {S29, 0, &V3581_310_57250, &P3581_309};	/*Property = GeoLocation*/
const Value V3581_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_312 = {S30, 0, &V3581_312_57256, &P3581_310};	/*Property = History*/
const Value V3581_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_314 = {S31, 0, &V3581_314_57262, &P3581_312};	/*Property = Html5*/
const Value V3581_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_313 = {S32, 0, &V3581_313_57259, &P3581_314};	/*Property = Html-Media-Capture*/
const Value V3581_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_315 = {S33, 0, &V3581_315_57265, &P3581_313};	/*Property = Iframe*/
const Value V3581_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_316 = {S34, 0, &V3581_316_57268, &P3581_315};	/*Property = IndexedDB*/
const Value V3581_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_317 = {S35, 0, &V3581_317_57271, &P3581_316};	/*Property = Json*/
const Value V3581_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_15 = {S37, 0, &V3581_15_72, &P3581_317};	/*Property = LayoutEngine*/
const Value V3581_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_344 = {S38, 0, &V3581_344_57274, &P3581_15};	/*Property = Masking*/
const Value V3581_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_320 = {S39, 0, &V3581_320_57292, &P3581_344};	/*Property = PostMessage*/
const Value V3581_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_321 = {S40, 0, &V3581_321_57295, &P3581_320};	/*Property = Progress*/
const Value V3581_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_322 = {S41, 0, &V3581_322_57298, &P3581_321};	/*Property = Prompts*/
const Value V3581_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_323 = {S42, 0, &V3581_323_57301, &P3581_322};	/*Property = Selector*/
const Value V3581_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_324 = {S43, 0, &V3581_324_57307, &P3581_323};	/*Property = Svg*/
const Value V3581_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_327 = {S44, 0, &V3581_327_57316, &P3581_324};	/*Property = TouchEvents*/
const Value V3581_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_328 = {S45, 0, &V3581_328_57319, &P3581_327};	/*Property = Track*/
const Value V3581_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_329 = {S46, 0, &V3581_329_57325, &P3581_328};	/*Property = Video*/
const Value V3581_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_330 = {S47, 0, &V3581_330_57328, &P3581_329};	/*Property = Viewport*/
const Value V3581_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_332 = {S48, 0, &V3581_332_57340, &P3581_330};	/*Property = WebWorkers*/
const Value V3581_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3581_333 = {S49, 0, &V3581_333_57343, &P3581_332};	/*Property = Xhr2*/
const Profile P3581 = {"3581", &P3581_333};	/*Profile ID - 3581*/
const Value V11723_1_1 = {S2, NULL};	/*Value = True*/
const Property P11723_1 = {S50, 0, &V11723_1_1, NULL};	/*Property = IsMobile*/
const Value V11723_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11723_5 = {S52, 0, &V11723_5_1222, &P11723_1};	/*Property = ScreenPixelsHeight*/
const Value V11723_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11723_4 = {S54, 0, &V11723_4_1224, &P11723_5};	/*Property = ScreenPixelsWidth*/
const Profile P11723 = {"11723", &P11723_4};	/*Profile ID - 11723*/
const Value V17191_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17191_279 = {S1, 0, &V17191_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17191_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17191_282 = {S3, 0, &V17191_282_57154, &P17191_279};	/*Property = BlobBuilder*/
const Value V17191_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17191_350 = {S4, 0, &V17191_350_57444, &P17191_282};	/*Property = Canvas*/
const Value V17191_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17191_283 = {S5, 0, &V17191_283_57159, &P17191_350};	/*Property = CssBackground*/
const Value V17191_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17191_284 = {S6, 0, &V17191_284_57163, &P17191_283};	/*Property = CssBorderImage*/
const Value V17191_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17191_285 = {S7, 0, &V17191_285_57166, &P17191_284};	/*Property = CssCanvas*/
const Value V17191_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17191_286 = {S8, 0, &V17191_286_57168, &P17191_285};	/*Property = CssColor*/
const Value V17191_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17191_237 = {S9, 0, &V17191_237_55786, &P17191_286};	/*Property = CssColumn*/
const Value V17191_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17191_287 = {S10, 0, &V17191_287_57172, &P17191_237};	/*Property = CssFlexbox*/
const Value V17191_288_57174 = {S2, NULL};	/*Value = True*/
const Property P17191_288 = {S11, 0, &V17191_288_57174, &P17191_287};	/*Property = CssFont*/
const Value V17191_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17191_289 = {S12, 0, &V17191_289_57759, &P17191_288};	/*Property = CssImages*/
const Value V17191_291_57180 = {S2, NULL};	/*Value = True*/
const Property P17191_291 = {S13, 0, &V17191_291_57180, &P17191_289};	/*Property = CssMediaQueries*/
const Value V17191_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17191_292 = {S14, 0, &V17191_292_57183, &P17191_291};	/*Property = CssMinMax*/
const Value V17191_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17191_293 = {S15, 0, &V17191_293_57187, &P17191_292};	/*Property = CssOverflow*/
const Value V17191_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17191_294 = {S16, 0, &V17191_294_57189, &P17191_293};	/*Property = CssPosition*/
const Value V17191_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17191_295 = {S17, 0, &V17191_295_57192, &P17191_294};	/*Property = CssText*/
const Value V17191_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17191_297 = {S18, 0, &V17191_297_57195, &P17191_295};	/*Property = CssTransforms*/
const Value V17191_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17191_241 = {S19, 0, &V17191_241_55799, &P17191_297};	/*Property = CssTransitions*/
const Value V17191_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17191_298 = {S20, 0, &V17191_298_57206, &P17191_241};	/*Property = CssUI*/
const Value V17191_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17191_300 = {S21, 0, &V17191_300_57216, &P17191_298};	/*Property = DataSet*/
const Value V17191_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17191_301 = {S22, 0, &V17191_301_57218, &P17191_300};	/*Property = DataUrl*/
const Value V17191_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17191_302 = {S23, 0, &V17191_302_57222, &P17191_301};	/*Property = DeviceOrientation*/
const Value V17191_304_57227 = {S2, NULL};	/*Value = True*/
const Property P17191_304 = {S24, 0, &V17191_304_57227, &P17191_302};	/*Property = FileReader*/
const Value V17191_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17191_305 = {S25, 0, &V17191_305_57231, &P17191_304};	/*Property = FileSaver*/
const Value V17191_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17191_306 = {S26, 0, &V17191_306_57237, &P17191_305};	/*Property = FileWriter*/
const Value V17191_307_57239 = {S2, NULL};	/*Value = True*/
const Property P17191_307 = {S27, 0, &V17191_307_57239, &P17191_306};	/*Property = FormData*/
const Value V17191_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17191_309 = {S28, 0, &V17191_309_57246, &P17191_307};	/*Property = Fullscreen*/
const Value V17191_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17191_310 = {S29, 0, &V17191_310_57248, &P17191_309};	/*Property = GeoLocation*/
const Value V17191_312_57254 = {S2, NULL};	/*Value = True*/
const Property P17191_312 = {S30, 0, &V17191_312_57254, &P17191_310};	/*Property = History*/
const Value V17191_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17191_314 = {S31, 0, &V17191_314_57261, &P17191_312};	/*Property = Html5*/
const Value V17191_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17191_313 = {S32, 0, &V17191_313_57258, &P17191_314};	/*Property = Html-Media-Capture*/
const Value V17191_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17191_315 = {S33, 0, &V17191_315_57264, &P17191_313};	/*Property = Iframe*/
const Value V17191_316_57266 = {S2, NULL};	/*Value = True*/
const Property P17191_316 = {S34, 0, &V17191_316_57266, &P17191_315};	/*Property = IndexedDB*/
const Value V17191_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17191_317 = {S35, 0, &V17191_317_57269, &P17191_316};	/*Property = Json*/
const Value V17191_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P17191_15 = {S37, 0, &V17191_15_30, &P17191_317};	/*Property = LayoutEngine*/
const Value V17191_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17191_344 = {S38, 0, &V17191_344_57272, &P17191_15};	/*Property = Masking*/
const Value V17191_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17191_320 = {S39, 0, &V17191_320_57290, &P17191_344};	/*Property = PostMessage*/
const Value V17191_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17191_321 = {S40, 0, &V17191_321_57293, &P17191_320};	/*Property = Progress*/
const Value V17191_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17191_322 = {S41, 0, &V17191_322_57296, &P17191_321};	/*Property = Prompts*/
const Value V17191_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17191_323 = {S42, 0, &V17191_323_57299, &P17191_322};	/*Property = Selector*/
const Value V17191_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17191_324 = {S43, 0, &V17191_324_57306, &P17191_323};	/*Property = Svg*/
const Value V17191_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17191_327 = {S44, 0, &V17191_327_57315, &P17191_324};	/*Property = TouchEvents*/
const Value V17191_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17191_328 = {S45, 0, &V17191_328_57318, &P17191_327};	/*Property = Track*/
const Value V17191_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17191_329 = {S46, 0, &V17191_329_57323, &P17191_328};	/*Property = Video*/
const Value V17191_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17191_330 = {S47, 0, &V17191_330_57326, &P17191_329};	/*Property = Viewport*/
const Value V17191_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17191_332 = {S48, 0, &V17191_332_57339, &P17191_330};	/*Property = WebWorkers*/
const Value V17191_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17191_333 = {S49, 0, &V17191_333_57342, &P17191_332};	/*Property = Xhr2*/
const Profile P17191 = {"17191", &P17191_333};	/*Profile ID - 17191*/
const Value V13437_1_1 = {S2, NULL};	/*Value = True*/
const Property P13437_1 = {S50, 0, &V13437_1_1, NULL};	/*Property = IsMobile*/
const Value V13437_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13437_5 = {S52, 0, &V13437_5_1277, &P13437_1};	/*Property = ScreenPixelsHeight*/
const Value V13437_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13437_4 = {S54, 0, &V13437_4_1279, &P13437_5};	/*Property = ScreenPixelsWidth*/
const Profile P13437 = {"13437", &P13437_4};	/*Profile ID - 13437*/
const Value V14929_1_1 = {S2, NULL};	/*Value = True*/
const Property P14929_1 = {S50, 0, &V14929_1_1, NULL};	/*Property = IsMobile*/
const Value V14929_5_1796 = {S64, NULL};	/*Value = 360*/
const Property P14929_5 = {S52, 0, &V14929_5_1796, &P14929_1};	/*Property = ScreenPixelsHeight*/
const Value V14929_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14929_4 = {S54, 0, &V14929_4_1229, &P14929_5};	/*Property = ScreenPixelsWidth*/
const Profile P14929 = {"14929", &P14929_4};	/*Profile ID - 14929*/
const Value V13439_1_1 = {S2, NULL};	/*Value = True*/
const Property P13439_1 = {S50, 0, &V13439_1_1, NULL};	/*Property = IsMobile*/
const Value V13439_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13439_5 = {S52, 0, &V13439_5_1293, &P13439_1};	/*Property = ScreenPixelsHeight*/
const Value V13439_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13439_4 = {S54, 0, &V13439_4_1296, &P13439_5};	/*Property = ScreenPixelsWidth*/
const Profile P13439 = {"13439", &P13439_4};	/*Profile ID - 13439*/
const Value V13440_1_1 = {S2, NULL};	/*Value = True*/
const Property P13440_1 = {S50, 0, &V13440_1_1, NULL};	/*Property = IsMobile*/
const Value V13440_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13440_5 = {S52, 0, &V13440_5_1233, &P13440_1};	/*Property = ScreenPixelsHeight*/
const Value V13440_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13440_4 = {S54, 0, &V13440_4_1235, &P13440_5};	/*Property = ScreenPixelsWidth*/
const Profile P13440 = {"13440", &P13440_4};	/*Profile ID - 13440*/
const Value V13443_1_1 = {S2, NULL};	/*Value = True*/
const Property P13443_1 = {S50, 0, &V13443_1_1, NULL};	/*Property = IsMobile*/
const Value V13443_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P13443_5 = {S52, 0, &V13443_5_1275, &P13443_1};	/*Property = ScreenPixelsHeight*/
const Value V13443_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P13443_4 = {S54, 0, &V13443_4_1278, &P13443_5};	/*Property = ScreenPixelsWidth*/
const Profile P13443 = {"13443", &P13443_4};	/*Profile ID - 13443*/
const Value V14714_1_1 = {S2, NULL};	/*Value = True*/
const Property P14714_1 = {S50, 0, &V14714_1_1, NULL};	/*Property = IsMobile*/
const Value V14714_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14714_5 = {S52, 0, &V14714_5_1222, &P14714_1};	/*Property = ScreenPixelsHeight*/
const Value V14714_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14714_4 = {S54, 0, &V14714_4_1224, &P14714_5};	/*Property = ScreenPixelsWidth*/
const Profile P14714 = {"14714", &P14714_4};	/*Profile ID - 14714*/
const Value V14534_1_1 = {S2, NULL};	/*Value = True*/
const Property P14534_1 = {S50, 0, &V14534_1_1, NULL};	/*Property = IsMobile*/
const Value V14534_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14534_5 = {S52, 0, &V14534_5_1304, &P14534_1};	/*Property = ScreenPixelsHeight*/
const Value V14534_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14534_4 = {S54, 0, &V14534_4_1224, &P14534_5};	/*Property = ScreenPixelsWidth*/
const Profile P14534 = {"14534", &P14534_4};	/*Profile ID - 14534*/
const Value V14531_1_1 = {S2, NULL};	/*Value = True*/
const Property P14531_1 = {S50, 0, &V14531_1_1, NULL};	/*Property = IsMobile*/
const Value V14531_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14531_5 = {S52, 0, &V14531_5_1233, &P14531_1};	/*Property = ScreenPixelsHeight*/
const Value V14531_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14531_4 = {S54, 0, &V14531_4_1235, &P14531_5};	/*Property = ScreenPixelsWidth*/
const Profile P14531 = {"14531", &P14531_4};	/*Profile ID - 14531*/
const Value V13452_1_1 = {S2, NULL};	/*Value = True*/
const Property P13452_1 = {S50, 0, &V13452_1_1, NULL};	/*Property = IsMobile*/
const Value V13452_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P13452_5 = {S52, 0, &V13452_5_1832, &P13452_1};	/*Property = ScreenPixelsHeight*/
const Value V13452_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P13452_4 = {S54, 0, &V13452_4_5645, &P13452_5};	/*Property = ScreenPixelsWidth*/
const Profile P13452 = {"13452", &P13452_4};	/*Profile ID - 13452*/
const Value V13454_1_1 = {S2, NULL};	/*Value = True*/
const Property P13454_1 = {S50, 0, &V13454_1_1, NULL};	/*Property = IsMobile*/
const Value V13454_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13454_5 = {S52, 0, &V13454_5_1293, &P13454_1};	/*Property = ScreenPixelsHeight*/
const Value V13454_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13454_4 = {S54, 0, &V13454_4_1296, &P13454_5};	/*Property = ScreenPixelsWidth*/
const Profile P13454 = {"13454", &P13454_4};	/*Profile ID - 13454*/
const Value V14872_1_1 = {S2, NULL};	/*Value = True*/
const Property P14872_1 = {S50, 0, &V14872_1_1, NULL};	/*Property = IsMobile*/
const Value V14872_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14872_5 = {S52, 0, &V14872_5_1227, &P14872_1};	/*Property = ScreenPixelsHeight*/
const Value V14872_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14872_4 = {S54, 0, &V14872_4_1229, &P14872_5};	/*Property = ScreenPixelsWidth*/
const Profile P14872 = {"14872", &P14872_4};	/*Profile ID - 14872*/
const Value V13457_1_1 = {S2, NULL};	/*Value = True*/
const Property P13457_1 = {S50, 0, &V13457_1_1, NULL};	/*Property = IsMobile*/
const Value V13457_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13457_5 = {S52, 0, &V13457_5_1222, &P13457_1};	/*Property = ScreenPixelsHeight*/
const Value V13457_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13457_4 = {S54, 0, &V13457_4_1224, &P13457_5};	/*Property = ScreenPixelsWidth*/
const Profile P13457 = {"13457", &P13457_4};	/*Profile ID - 13457*/
const Value V16948_1_1 = {S2, NULL};	/*Value = True*/
const Property P16948_1 = {S50, 0, &V16948_1_1, NULL};	/*Property = IsMobile*/
const Value V16948_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P16948_5 = {S52, 0, &V16948_5_1380, &P16948_1};	/*Property = ScreenPixelsHeight*/
const Value V16948_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16948_4 = {S54, 0, &V16948_4_1279, &P16948_5};	/*Property = ScreenPixelsWidth*/
const Profile P16948 = {"16948", &P16948_4};	/*Profile ID - 16948*/
const Value V14199_1_1 = {S2, NULL};	/*Value = True*/
const Property P14199_1 = {S50, 0, &V14199_1_1, NULL};	/*Property = IsMobile*/
const Value V14199_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14199_5 = {S52, 0, &V14199_5_1293, &P14199_1};	/*Property = ScreenPixelsHeight*/
const Value V14199_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14199_4 = {S54, 0, &V14199_4_1296, &P14199_5};	/*Property = ScreenPixelsWidth*/
const Profile P14199 = {"14199", &P14199_4};	/*Profile ID - 14199*/
const Value V14980_1_1 = {S2, NULL};	/*Value = True*/
const Property P14980_1 = {S50, 0, &V14980_1_1, NULL};	/*Property = IsMobile*/
const Value V14980_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14980_5 = {S52, 0, &V14980_5_1304, &P14980_1};	/*Property = ScreenPixelsHeight*/
const Value V14980_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14980_4 = {S54, 0, &V14980_4_1224, &P14980_5};	/*Property = ScreenPixelsWidth*/
const Profile P14980 = {"14980", &P14980_4};	/*Profile ID - 14980*/
const Value V17330_1_1 = {S2, NULL};	/*Value = True*/
const Property P17330_1 = {S50, 0, &V17330_1_1, NULL};	/*Property = IsMobile*/
const Value V17330_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17330_5 = {S52, 0, &V17330_5_1227, &P17330_1};	/*Property = ScreenPixelsHeight*/
const Value V17330_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17330_4 = {S54, 0, &V17330_4_1229, &P17330_5};	/*Property = ScreenPixelsWidth*/
const Profile P17330 = {"17330", &P17330_4};	/*Profile ID - 17330*/
const Value V13684_1_1 = {S2, NULL};	/*Value = True*/
const Property P13684_1 = {S50, 0, &V13684_1_1, NULL};	/*Property = IsMobile*/
const Value V13684_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13684_5 = {S52, 0, &V13684_5_1227, &P13684_1};	/*Property = ScreenPixelsHeight*/
const Value V13684_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13684_4 = {S54, 0, &V13684_4_1229, &P13684_5};	/*Property = ScreenPixelsWidth*/
const Profile P13684 = {"13684", &P13684_4};	/*Profile ID - 13684*/
const Value V12420_1_1 = {S2, NULL};	/*Value = True*/
const Property P12420_1 = {S50, 0, &V12420_1_1, NULL};	/*Property = IsMobile*/
const Value V12420_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12420_5 = {S52, 0, &V12420_5_1380, &P12420_1};	/*Property = ScreenPixelsHeight*/
const Value V12420_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12420_4 = {S54, 0, &V12420_4_1279, &P12420_5};	/*Property = ScreenPixelsWidth*/
const Profile P12420 = {"12420", &P12420_4};	/*Profile ID - 12420*/
const Profile P7472 = {"7472", NULL};	/*Profile ID - 7472*/
const Value V17214_1_1 = {S2, NULL};	/*Value = True*/
const Property P17214_1 = {S50, 0, &V17214_1_1, NULL};	/*Property = IsMobile*/
const Value V17214_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17214_5 = {S52, 0, &V17214_5_1277, &P17214_1};	/*Property = ScreenPixelsHeight*/
const Value V17214_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P17214_4 = {S54, 0, &V17214_4_2336, &P17214_5};	/*Property = ScreenPixelsWidth*/
const Profile P17214 = {"17214", &P17214_4};	/*Profile ID - 17214*/
const Value V17725_1_1 = {S2, NULL};	/*Value = True*/
const Property P17725_1 = {S50, 0, &V17725_1_1, NULL};	/*Property = IsMobile*/
const Value V17725_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17725_5 = {S52, 0, &V17725_5_1277, &P17725_1};	/*Property = ScreenPixelsHeight*/
const Value V17725_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P17725_4 = {S54, 0, &V17725_4_2336, &P17725_5};	/*Property = ScreenPixelsWidth*/
const Profile P17725 = {"17725", &P17725_4};	/*Profile ID - 17725*/
const Value V16815_1_1 = {S2, NULL};	/*Value = True*/
const Property P16815_1 = {S50, 0, &V16815_1_1, NULL};	/*Property = IsMobile*/
const Value V16815_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16815_5 = {S52, 0, &V16815_5_1227, &P16815_1};	/*Property = ScreenPixelsHeight*/
const Value V16815_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16815_4 = {S54, 0, &V16815_4_1229, &P16815_5};	/*Property = ScreenPixelsWidth*/
const Profile P16815 = {"16815", &P16815_4};	/*Profile ID - 16815*/
const Value V16862_1_1 = {S2, NULL};	/*Value = True*/
const Property P16862_1 = {S50, 0, &V16862_1_1, NULL};	/*Property = IsMobile*/
const Value V16862_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16862_5 = {S52, 0, &V16862_5_1233, &P16862_1};	/*Property = ScreenPixelsHeight*/
const Value V16862_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16862_4 = {S54, 0, &V16862_4_1235, &P16862_5};	/*Property = ScreenPixelsWidth*/
const Profile P16862 = {"16862", &P16862_4};	/*Profile ID - 16862*/
const Value V12247_1_1 = {S2, NULL};	/*Value = True*/
const Property P12247_1 = {S50, 0, &V12247_1_1, NULL};	/*Property = IsMobile*/
const Value V12247_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12247_5 = {S52, 0, &V12247_5_1293, &P12247_1};	/*Property = ScreenPixelsHeight*/
const Value V12247_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12247_4 = {S54, 0, &V12247_4_1296, &P12247_5};	/*Property = ScreenPixelsWidth*/
const Profile P12247 = {"12247", &P12247_4};	/*Profile ID - 12247*/
const Value V14475_1_1 = {S2, NULL};	/*Value = True*/
const Property P14475_1 = {S50, 0, &V14475_1_1, NULL};	/*Property = IsMobile*/
const Value V14475_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14475_5 = {S52, 0, &V14475_5_1233, &P14475_1};	/*Property = ScreenPixelsHeight*/
const Value V14475_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14475_4 = {S54, 0, &V14475_4_1235, &P14475_5};	/*Property = ScreenPixelsWidth*/
const Profile P14475 = {"14475", &P14475_4};	/*Profile ID - 14475*/
const Value V13487_1_1 = {S2, NULL};	/*Value = True*/
const Property P13487_1 = {S50, 0, &V13487_1_1, NULL};	/*Property = IsMobile*/
const Value V13487_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13487_5 = {S52, 0, &V13487_5_1222, &P13487_1};	/*Property = ScreenPixelsHeight*/
const Value V13487_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13487_4 = {S54, 0, &V13487_4_1224, &P13487_5};	/*Property = ScreenPixelsWidth*/
const Profile P13487 = {"13487", &P13487_4};	/*Profile ID - 13487*/
const Value V14837_1_1 = {S2, NULL};	/*Value = True*/
const Property P14837_1 = {S50, 0, &V14837_1_1, NULL};	/*Property = IsMobile*/
const Value V14837_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14837_5 = {S52, 0, &V14837_5_1304, &P14837_1};	/*Property = ScreenPixelsHeight*/
const Value V14837_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14837_4 = {S54, 0, &V14837_4_1224, &P14837_5};	/*Property = ScreenPixelsWidth*/
const Profile P14837 = {"14837", &P14837_4};	/*Profile ID - 14837*/
const Value V16036_1_1 = {S2, NULL};	/*Value = True*/
const Property P16036_1 = {S50, 0, &V16036_1_1, NULL};	/*Property = IsMobile*/
const Value V16036_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16036_5 = {S52, 0, &V16036_5_1227, &P16036_1};	/*Property = ScreenPixelsHeight*/
const Value V16036_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16036_4 = {S54, 0, &V16036_4_1229, &P16036_5};	/*Property = ScreenPixelsWidth*/
const Profile P16036 = {"16036", &P16036_4};	/*Profile ID - 16036*/
const Value V4536_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_279 = {S1, 0, &V4536_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4536_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_282 = {S3, 0, &V4536_282_57155, &P4536_279};	/*Property = BlobBuilder*/
const Value V4536_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_350 = {S4, 0, &V4536_350_57446, &P4536_282};	/*Property = Canvas*/
const Value V4536_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_283 = {S5, 0, &V4536_283_57161, &P4536_350};	/*Property = CssBackground*/
const Value V4536_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_284 = {S6, 0, &V4536_284_57164, &P4536_283};	/*Property = CssBorderImage*/
const Value V4536_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_285 = {S7, 0, &V4536_285_57167, &P4536_284};	/*Property = CssCanvas*/
const Value V4536_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_286 = {S8, 0, &V4536_286_57170, &P4536_285};	/*Property = CssColor*/
const Value V4536_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_237 = {S9, 0, &V4536_237_55786, &P4536_286};	/*Property = CssColumn*/
const Value V4536_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_287 = {S10, 0, &V4536_287_57173, &P4536_237};	/*Property = CssFlexbox*/
const Value V4536_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_288 = {S11, 0, &V4536_288_57176, &P4536_287};	/*Property = CssFont*/
const Value V4536_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_289 = {S12, 0, &V4536_289_86899, &P4536_288};	/*Property = CssImages*/
const Value V4536_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_291 = {S13, 0, &V4536_291_57182, &P4536_289};	/*Property = CssMediaQueries*/
const Value V4536_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_292 = {S14, 0, &V4536_292_57185, &P4536_291};	/*Property = CssMinMax*/
const Value V4536_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_293 = {S15, 0, &V4536_293_57188, &P4536_292};	/*Property = CssOverflow*/
const Value V4536_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_294 = {S16, 0, &V4536_294_57191, &P4536_293};	/*Property = CssPosition*/
const Value V4536_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_295 = {S17, 0, &V4536_295_57194, &P4536_294};	/*Property = CssText*/
const Value V4536_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_297 = {S18, 0, &V4536_297_57197, &P4536_295};	/*Property = CssTransforms*/
const Value V4536_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_241 = {S19, 0, &V4536_241_55801, &P4536_297};	/*Property = CssTransitions*/
const Value V4536_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_298 = {S20, 0, &V4536_298_57208, &P4536_241};	/*Property = CssUI*/
const Value V4536_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_300 = {S21, 0, &V4536_300_57217, &P4536_298};	/*Property = DataSet*/
const Value V4536_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_301 = {S22, 0, &V4536_301_57220, &P4536_300};	/*Property = DataUrl*/
const Value V4536_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_302 = {S23, 0, &V4536_302_57223, &P4536_301};	/*Property = DeviceOrientation*/
const Value V4536_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_304 = {S24, 0, &V4536_304_57229, &P4536_302};	/*Property = FileReader*/
const Value V4536_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_305 = {S25, 0, &V4536_305_57232, &P4536_304};	/*Property = FileSaver*/
const Value V4536_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_306 = {S26, 0, &V4536_306_57238, &P4536_305};	/*Property = FileWriter*/
const Value V4536_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_307 = {S27, 0, &V4536_307_57241, &P4536_306};	/*Property = FormData*/
const Value V4536_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_309 = {S28, 0, &V4536_309_57247, &P4536_307};	/*Property = Fullscreen*/
const Value V4536_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_310 = {S29, 0, &V4536_310_57250, &P4536_309};	/*Property = GeoLocation*/
const Value V4536_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_312 = {S30, 0, &V4536_312_57256, &P4536_310};	/*Property = History*/
const Value V4536_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_314 = {S31, 0, &V4536_314_57262, &P4536_312};	/*Property = Html5*/
const Value V4536_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_313 = {S32, 0, &V4536_313_57259, &P4536_314};	/*Property = Html-Media-Capture*/
const Value V4536_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_315 = {S33, 0, &V4536_315_57265, &P4536_313};	/*Property = Iframe*/
const Value V4536_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_316 = {S34, 0, &V4536_316_57268, &P4536_315};	/*Property = IndexedDB*/
const Value V4536_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_317 = {S35, 0, &V4536_317_57271, &P4536_316};	/*Property = Json*/
const Value V4536_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P4536_15 = {S37, 0, &V4536_15_31, &P4536_317};	/*Property = LayoutEngine*/
const Value V4536_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_344 = {S38, 0, &V4536_344_57274, &P4536_15};	/*Property = Masking*/
const Value V4536_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_320 = {S39, 0, &V4536_320_57292, &P4536_344};	/*Property = PostMessage*/
const Value V4536_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_321 = {S40, 0, &V4536_321_57295, &P4536_320};	/*Property = Progress*/
const Value V4536_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_322 = {S41, 0, &V4536_322_57298, &P4536_321};	/*Property = Prompts*/
const Value V4536_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_323 = {S42, 0, &V4536_323_57301, &P4536_322};	/*Property = Selector*/
const Value V4536_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_324 = {S43, 0, &V4536_324_57307, &P4536_323};	/*Property = Svg*/
const Value V4536_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_327 = {S44, 0, &V4536_327_57316, &P4536_324};	/*Property = TouchEvents*/
const Value V4536_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_328 = {S45, 0, &V4536_328_57319, &P4536_327};	/*Property = Track*/
const Value V4536_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_329 = {S46, 0, &V4536_329_57325, &P4536_328};	/*Property = Video*/
const Value V4536_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_330 = {S47, 0, &V4536_330_57328, &P4536_329};	/*Property = Viewport*/
const Value V4536_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_332 = {S48, 0, &V4536_332_57340, &P4536_330};	/*Property = WebWorkers*/
const Value V4536_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4536_333 = {S49, 0, &V4536_333_57343, &P4536_332};	/*Property = Xhr2*/
const Profile P4536 = {"4536", &P4536_333};	/*Profile ID - 4536*/
const Value V13498_1_1 = {S2, NULL};	/*Value = True*/
const Property P13498_1 = {S50, 0, &V13498_1_1, NULL};	/*Property = IsMobile*/
const Value V13498_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13498_5 = {S52, 0, &V13498_5_1222, &P13498_1};	/*Property = ScreenPixelsHeight*/
const Value V13498_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13498_4 = {S54, 0, &V13498_4_1279, &P13498_5};	/*Property = ScreenPixelsWidth*/
const Profile P13498 = {"13498", &P13498_4};	/*Profile ID - 13498*/
const Value V13502_1_1 = {S2, NULL};	/*Value = True*/
const Property P13502_1 = {S50, 0, &V13502_1_1, NULL};	/*Property = IsMobile*/
const Value V13502_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13502_5 = {S52, 0, &V13502_5_1222, &P13502_1};	/*Property = ScreenPixelsHeight*/
const Value V13502_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13502_4 = {S54, 0, &V13502_4_1224, &P13502_5};	/*Property = ScreenPixelsWidth*/
const Profile P13502 = {"13502", &P13502_4};	/*Profile ID - 13502*/
const Value V13503_1_1 = {S2, NULL};	/*Value = True*/
const Property P13503_1 = {S50, 0, &V13503_1_1, NULL};	/*Property = IsMobile*/
const Value V13503_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13503_5 = {S52, 0, &V13503_5_1227, &P13503_1};	/*Property = ScreenPixelsHeight*/
const Value V13503_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13503_4 = {S54, 0, &V13503_4_1229, &P13503_5};	/*Property = ScreenPixelsWidth*/
const Profile P13503 = {"13503", &P13503_4};	/*Profile ID - 13503*/
const Value V13350_1_1 = {S2, NULL};	/*Value = True*/
const Property P13350_1 = {S50, 0, &V13350_1_1, NULL};	/*Property = IsMobile*/
const Value V13350_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13350_5 = {S52, 0, &V13350_5_1233, &P13350_1};	/*Property = ScreenPixelsHeight*/
const Value V13350_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13350_4 = {S54, 0, &V13350_4_1296, &P13350_5};	/*Property = ScreenPixelsWidth*/
const Profile P13350 = {"13350", &P13350_4};	/*Profile ID - 13350*/
const Value V13351_1_1 = {S2, NULL};	/*Value = True*/
const Property P13351_1 = {S50, 0, &V13351_1_1, NULL};	/*Property = IsMobile*/
const Value V13351_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13351_5 = {S52, 0, &V13351_5_1222, &P13351_1};	/*Property = ScreenPixelsHeight*/
const Value V13351_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13351_4 = {S54, 0, &V13351_4_1224, &P13351_5};	/*Property = ScreenPixelsWidth*/
const Profile P13351 = {"13351", &P13351_4};	/*Profile ID - 13351*/
const Value V17984_1_1 = {S2, NULL};	/*Value = True*/
const Property P17984_1 = {S50, 0, &V17984_1_1, NULL};	/*Property = IsMobile*/
const Value V17984_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17984_5 = {S52, 0, &V17984_5_1293, &P17984_1};	/*Property = ScreenPixelsHeight*/
const Value V17984_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17984_4 = {S54, 0, &V17984_4_1296, &P17984_5};	/*Property = ScreenPixelsWidth*/
const Profile P17984 = {"17984", &P17984_4};	/*Profile ID - 17984*/
const Value V13353_1_1 = {S2, NULL};	/*Value = True*/
const Property P13353_1 = {S50, 0, &V13353_1_1, NULL};	/*Property = IsMobile*/
const Value V13353_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13353_5 = {S52, 0, &V13353_5_1293, &P13353_1};	/*Property = ScreenPixelsHeight*/
const Value V13353_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13353_4 = {S54, 0, &V13353_4_1296, &P13353_5};	/*Property = ScreenPixelsWidth*/
const Profile P13353 = {"13353", &P13353_4};	/*Profile ID - 13353*/
const Value V13354_1_1 = {S2, NULL};	/*Value = True*/
const Property P13354_1 = {S50, 0, &V13354_1_1, NULL};	/*Property = IsMobile*/
const Value V13354_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13354_5 = {S52, 0, &V13354_5_1233, &P13354_1};	/*Property = ScreenPixelsHeight*/
const Value V13354_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13354_4 = {S54, 0, &V13354_4_1235, &P13354_5};	/*Property = ScreenPixelsWidth*/
const Profile P13354 = {"13354", &P13354_4};	/*Profile ID - 13354*/
const Value V12501_1_1 = {S2, NULL};	/*Value = True*/
const Property P12501_1 = {S50, 0, &V12501_1_1, NULL};	/*Property = IsMobile*/
const Value V12501_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12501_5 = {S52, 0, &V12501_5_1222, &P12501_1};	/*Property = ScreenPixelsHeight*/
const Value V12501_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12501_4 = {S54, 0, &V12501_4_1224, &P12501_5};	/*Property = ScreenPixelsWidth*/
const Profile P12501 = {"12501", &P12501_4};	/*Profile ID - 12501*/
const Value V11804_1_1 = {S2, NULL};	/*Value = True*/
const Property P11804_1 = {S50, 0, &V11804_1_1, NULL};	/*Property = IsMobile*/
const Value V11804_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P11804_5 = {S52, 0, &V11804_5_1304, &P11804_1};	/*Property = ScreenPixelsHeight*/
const Value V11804_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11804_4 = {S54, 0, &V11804_4_1224, &P11804_5};	/*Property = ScreenPixelsWidth*/
const Profile P11804 = {"11804", &P11804_4};	/*Profile ID - 11804*/
const Value V13358_1_1 = {S2, NULL};	/*Value = True*/
const Property P13358_1 = {S50, 0, &V13358_1_1, NULL};	/*Property = IsMobile*/
const Value V13358_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13358_5 = {S52, 0, &V13358_5_1293, &P13358_1};	/*Property = ScreenPixelsHeight*/
const Value V13358_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13358_4 = {S54, 0, &V13358_4_1296, &P13358_5};	/*Property = ScreenPixelsWidth*/
const Profile P13358 = {"13358", &P13358_4};	/*Profile ID - 13358*/
const Value V13359_1_1 = {S2, NULL};	/*Value = True*/
const Property P13359_1 = {S50, 0, &V13359_1_1, NULL};	/*Property = IsMobile*/
const Value V13359_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13359_5 = {S52, 0, &V13359_5_1222, &P13359_1};	/*Property = ScreenPixelsHeight*/
const Value V13359_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13359_4 = {S54, 0, &V13359_4_1224, &P13359_5};	/*Property = ScreenPixelsWidth*/
const Profile P13359 = {"13359", &P13359_4};	/*Profile ID - 13359*/
const Value V16676_1_1 = {S2, NULL};	/*Value = True*/
const Property P16676_1 = {S50, 0, &V16676_1_1, NULL};	/*Property = IsMobile*/
const Value V16676_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P16676_5 = {S52, 0, &V16676_5_1304, &P16676_1};	/*Property = ScreenPixelsHeight*/
const Value V16676_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16676_4 = {S54, 0, &V16676_4_1224, &P16676_5};	/*Property = ScreenPixelsWidth*/
const Profile P16676 = {"16676", &P16676_4};	/*Profile ID - 16676*/
const Value V13362_1_1 = {S2, NULL};	/*Value = True*/
const Property P13362_1 = {S50, 0, &V13362_1_1, NULL};	/*Property = IsMobile*/
const Value V13362_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13362_5 = {S52, 0, &V13362_5_1222, &P13362_1};	/*Property = ScreenPixelsHeight*/
const Value V13362_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13362_4 = {S54, 0, &V13362_4_1224, &P13362_5};	/*Property = ScreenPixelsWidth*/
const Profile P13362 = {"13362", &P13362_4};	/*Profile ID - 13362*/
const Value V13155_1_1 = {S2, NULL};	/*Value = True*/
const Property P13155_1 = {S50, 0, &V13155_1_1, NULL};	/*Property = IsMobile*/
const Value V13155_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13155_5 = {S52, 0, &V13155_5_1380, &P13155_1};	/*Property = ScreenPixelsHeight*/
const Value V13155_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13155_4 = {S54, 0, &V13155_4_1279, &P13155_5};	/*Property = ScreenPixelsWidth*/
const Profile P13155 = {"13155", &P13155_4};	/*Profile ID - 13155*/
const Value V13364_1_1 = {S2, NULL};	/*Value = True*/
const Property P13364_1 = {S50, 0, &V13364_1_1, NULL};	/*Property = IsMobile*/
const Value V13364_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13364_5 = {S52, 0, &V13364_5_1222, &P13364_1};	/*Property = ScreenPixelsHeight*/
const Value V13364_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13364_4 = {S54, 0, &V13364_4_1224, &P13364_5};	/*Property = ScreenPixelsWidth*/
const Profile P13364 = {"13364", &P13364_4};	/*Profile ID - 13364*/
const Value V11768_1_1 = {S2, NULL};	/*Value = True*/
const Property P11768_1 = {S50, 0, &V11768_1_1, NULL};	/*Property = IsMobile*/
const Value V11768_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11768_5 = {S52, 0, &V11768_5_1233, &P11768_1};	/*Property = ScreenPixelsHeight*/
const Value V11768_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11768_4 = {S54, 0, &V11768_4_1235, &P11768_5};	/*Property = ScreenPixelsWidth*/
const Profile P11768 = {"11768", &P11768_4};	/*Profile ID - 11768*/
const Value V13368_1_1 = {S2, NULL};	/*Value = True*/
const Property P13368_1 = {S50, 0, &V13368_1_1, NULL};	/*Property = IsMobile*/
const Value V13368_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13368_5 = {S52, 0, &V13368_5_1222, &P13368_1};	/*Property = ScreenPixelsHeight*/
const Value V13368_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13368_4 = {S54, 0, &V13368_4_1224, &P13368_5};	/*Property = ScreenPixelsWidth*/
const Profile P13368 = {"13368", &P13368_4};	/*Profile ID - 13368*/
const Value V13369_1_1 = {S2, NULL};	/*Value = True*/
const Property P13369_1 = {S50, 0, &V13369_1_1, NULL};	/*Property = IsMobile*/
const Value V13369_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13369_5 = {S52, 0, &V13369_5_1222, &P13369_1};	/*Property = ScreenPixelsHeight*/
const Value V13369_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13369_4 = {S54, 0, &V13369_4_1224, &P13369_5};	/*Property = ScreenPixelsWidth*/
const Profile P13369 = {"13369", &P13369_4};	/*Profile ID - 13369*/
const Value V13380_1_1 = {S2, NULL};	/*Value = True*/
const Property P13380_1 = {S50, 0, &V13380_1_1, NULL};	/*Property = IsMobile*/
const Value V13380_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13380_5 = {S52, 0, &V13380_5_1222, &P13380_1};	/*Property = ScreenPixelsHeight*/
const Value V13380_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13380_4 = {S54, 0, &V13380_4_1224, &P13380_5};	/*Property = ScreenPixelsWidth*/
const Profile P13380 = {"13380", &P13380_4};	/*Profile ID - 13380*/
const Value V13382_1_1 = {S2, NULL};	/*Value = True*/
const Property P13382_1 = {S50, 0, &V13382_1_1, NULL};	/*Property = IsMobile*/
const Value V13382_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13382_5 = {S52, 0, &V13382_5_1222, &P13382_1};	/*Property = ScreenPixelsHeight*/
const Value V13382_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13382_4 = {S54, 0, &V13382_4_1224, &P13382_5};	/*Property = ScreenPixelsWidth*/
const Profile P13382 = {"13382", &P13382_4};	/*Profile ID - 13382*/
const Value V13385_1_1 = {S2, NULL};	/*Value = True*/
const Property P13385_1 = {S50, 0, &V13385_1_1, NULL};	/*Property = IsMobile*/
const Value V13385_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13385_5 = {S52, 0, &V13385_5_1222, &P13385_1};	/*Property = ScreenPixelsHeight*/
const Value V13385_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13385_4 = {S54, 0, &V13385_4_1224, &P13385_5};	/*Property = ScreenPixelsWidth*/
const Profile P13385 = {"13385", &P13385_4};	/*Profile ID - 13385*/
const Value V13387_1_1 = {S2, NULL};	/*Value = True*/
const Property P13387_1 = {S50, 0, &V13387_1_1, NULL};	/*Property = IsMobile*/
const Value V13387_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13387_5 = {S52, 0, &V13387_5_1293, &P13387_1};	/*Property = ScreenPixelsHeight*/
const Value V13387_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13387_4 = {S54, 0, &V13387_4_1296, &P13387_5};	/*Property = ScreenPixelsWidth*/
const Profile P13387 = {"13387", &P13387_4};	/*Profile ID - 13387*/
const Value V15209_1_1 = {S2, NULL};	/*Value = True*/
const Property P15209_1 = {S50, 0, &V15209_1_1, NULL};	/*Property = IsMobile*/
const Value V15209_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15209_5 = {S52, 0, &V15209_5_1222, &P15209_1};	/*Property = ScreenPixelsHeight*/
const Value V15209_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15209_4 = {S54, 0, &V15209_4_1224, &P15209_5};	/*Property = ScreenPixelsWidth*/
const Profile P15209 = {"15209", &P15209_4};	/*Profile ID - 15209*/
const Value V4732_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_279 = {S1, 0, &V4732_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4732_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_282 = {S3, 0, &V4732_282_57155, &P4732_279};	/*Property = BlobBuilder*/
const Value V4732_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_350 = {S4, 0, &V4732_350_57446, &P4732_282};	/*Property = Canvas*/
const Value V4732_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_283 = {S5, 0, &V4732_283_57161, &P4732_350};	/*Property = CssBackground*/
const Value V4732_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_284 = {S6, 0, &V4732_284_57164, &P4732_283};	/*Property = CssBorderImage*/
const Value V4732_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_285 = {S7, 0, &V4732_285_57167, &P4732_284};	/*Property = CssCanvas*/
const Value V4732_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_286 = {S8, 0, &V4732_286_57170, &P4732_285};	/*Property = CssColor*/
const Value V4732_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_237 = {S9, 0, &V4732_237_55786, &P4732_286};	/*Property = CssColumn*/
const Value V4732_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_287 = {S10, 0, &V4732_287_57173, &P4732_237};	/*Property = CssFlexbox*/
const Value V4732_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_288 = {S11, 0, &V4732_288_57176, &P4732_287};	/*Property = CssFont*/
const Value V4732_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_289 = {S12, 0, &V4732_289_86899, &P4732_288};	/*Property = CssImages*/
const Value V4732_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_291 = {S13, 0, &V4732_291_57182, &P4732_289};	/*Property = CssMediaQueries*/
const Value V4732_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_292 = {S14, 0, &V4732_292_57185, &P4732_291};	/*Property = CssMinMax*/
const Value V4732_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_293 = {S15, 0, &V4732_293_57188, &P4732_292};	/*Property = CssOverflow*/
const Value V4732_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_294 = {S16, 0, &V4732_294_57191, &P4732_293};	/*Property = CssPosition*/
const Value V4732_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_295 = {S17, 0, &V4732_295_57194, &P4732_294};	/*Property = CssText*/
const Value V4732_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_297 = {S18, 0, &V4732_297_57197, &P4732_295};	/*Property = CssTransforms*/
const Value V4732_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_241 = {S19, 0, &V4732_241_55801, &P4732_297};	/*Property = CssTransitions*/
const Value V4732_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_298 = {S20, 0, &V4732_298_57208, &P4732_241};	/*Property = CssUI*/
const Value V4732_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_300 = {S21, 0, &V4732_300_57217, &P4732_298};	/*Property = DataSet*/
const Value V4732_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_301 = {S22, 0, &V4732_301_57220, &P4732_300};	/*Property = DataUrl*/
const Value V4732_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_302 = {S23, 0, &V4732_302_57223, &P4732_301};	/*Property = DeviceOrientation*/
const Value V4732_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_304 = {S24, 0, &V4732_304_57229, &P4732_302};	/*Property = FileReader*/
const Value V4732_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_305 = {S25, 0, &V4732_305_57232, &P4732_304};	/*Property = FileSaver*/
const Value V4732_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_306 = {S26, 0, &V4732_306_57238, &P4732_305};	/*Property = FileWriter*/
const Value V4732_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_307 = {S27, 0, &V4732_307_57241, &P4732_306};	/*Property = FormData*/
const Value V4732_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_309 = {S28, 0, &V4732_309_57247, &P4732_307};	/*Property = Fullscreen*/
const Value V4732_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_310 = {S29, 0, &V4732_310_57250, &P4732_309};	/*Property = GeoLocation*/
const Value V4732_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_312 = {S30, 0, &V4732_312_57256, &P4732_310};	/*Property = History*/
const Value V4732_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_314 = {S31, 0, &V4732_314_57262, &P4732_312};	/*Property = Html5*/
const Value V4732_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_313 = {S32, 0, &V4732_313_57259, &P4732_314};	/*Property = Html-Media-Capture*/
const Value V4732_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_315 = {S33, 0, &V4732_315_57265, &P4732_313};	/*Property = Iframe*/
const Value V4732_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_316 = {S34, 0, &V4732_316_57268, &P4732_315};	/*Property = IndexedDB*/
const Value V4732_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_317 = {S35, 0, &V4732_317_57271, &P4732_316};	/*Property = Json*/
const Value V4732_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P4732_15 = {S37, 0, &V4732_15_31, &P4732_317};	/*Property = LayoutEngine*/
const Value V4732_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_344 = {S38, 0, &V4732_344_57274, &P4732_15};	/*Property = Masking*/
const Value V4732_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_320 = {S39, 0, &V4732_320_57292, &P4732_344};	/*Property = PostMessage*/
const Value V4732_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_321 = {S40, 0, &V4732_321_57295, &P4732_320};	/*Property = Progress*/
const Value V4732_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_322 = {S41, 0, &V4732_322_57298, &P4732_321};	/*Property = Prompts*/
const Value V4732_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_323 = {S42, 0, &V4732_323_57301, &P4732_322};	/*Property = Selector*/
const Value V4732_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_324 = {S43, 0, &V4732_324_57307, &P4732_323};	/*Property = Svg*/
const Value V4732_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_327 = {S44, 0, &V4732_327_57316, &P4732_324};	/*Property = TouchEvents*/
const Value V4732_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_328 = {S45, 0, &V4732_328_57319, &P4732_327};	/*Property = Track*/
const Value V4732_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_329 = {S46, 0, &V4732_329_57325, &P4732_328};	/*Property = Video*/
const Value V4732_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_330 = {S47, 0, &V4732_330_57328, &P4732_329};	/*Property = Viewport*/
const Value V4732_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_332 = {S48, 0, &V4732_332_57340, &P4732_330};	/*Property = WebWorkers*/
const Value V4732_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4732_333 = {S49, 0, &V4732_333_57343, &P4732_332};	/*Property = Xhr2*/
const Profile P4732 = {"4732", &P4732_333};	/*Profile ID - 4732*/
const Value V11814_1_1 = {S2, NULL};	/*Value = True*/
const Property P11814_1 = {S50, 0, &V11814_1_1, NULL};	/*Property = IsMobile*/
const Value V11814_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P11814_5 = {S52, 0, &V11814_5_1333, &P11814_1};	/*Property = ScreenPixelsHeight*/
const Value V11814_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P11814_4 = {S54, 0, &V11814_4_1335, &P11814_5};	/*Property = ScreenPixelsWidth*/
const Profile P11814 = {"11814", &P11814_4};	/*Profile ID - 11814*/
const Value V13388_1_1 = {S2, NULL};	/*Value = True*/
const Property P13388_1 = {S50, 0, &V13388_1_1, NULL};	/*Property = IsMobile*/
const Value V13388_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13388_5 = {S52, 0, &V13388_5_1293, &P13388_1};	/*Property = ScreenPixelsHeight*/
const Value V13388_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13388_4 = {S54, 0, &V13388_4_1296, &P13388_5};	/*Property = ScreenPixelsWidth*/
const Profile P13388 = {"13388", &P13388_4};	/*Profile ID - 13388*/
const Value V13391_1_1 = {S2, NULL};	/*Value = True*/
const Property P13391_1 = {S50, 0, &V13391_1_1, NULL};	/*Property = IsMobile*/
const Value V13391_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13391_5 = {S52, 0, &V13391_5_1222, &P13391_1};	/*Property = ScreenPixelsHeight*/
const Value V13391_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13391_4 = {S54, 0, &V13391_4_1224, &P13391_5};	/*Property = ScreenPixelsWidth*/
const Profile P13391 = {"13391", &P13391_4};	/*Profile ID - 13391*/
const Value V13393_1_1 = {S2, NULL};	/*Value = True*/
const Property P13393_1 = {S50, 0, &V13393_1_1, NULL};	/*Property = IsMobile*/
const Value V13393_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13393_5 = {S52, 0, &V13393_5_1294, &P13393_1};	/*Property = ScreenPixelsHeight*/
const Value V13393_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13393_4 = {S54, 0, &V13393_4_1296, &P13393_5};	/*Property = ScreenPixelsWidth*/
const Profile P13393 = {"13393", &P13393_4};	/*Profile ID - 13393*/
const Value V13401_1_1 = {S2, NULL};	/*Value = True*/
const Property P13401_1 = {S50, 0, &V13401_1_1, NULL};	/*Property = IsMobile*/
const Value V13401_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13401_5 = {S52, 0, &V13401_5_1222, &P13401_1};	/*Property = ScreenPixelsHeight*/
const Value V13401_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13401_4 = {S54, 0, &V13401_4_1224, &P13401_5};	/*Property = ScreenPixelsWidth*/
const Profile P13401 = {"13401", &P13401_4};	/*Profile ID - 13401*/
const Value V13402_1_1 = {S2, NULL};	/*Value = True*/
const Property P13402_1 = {S50, 0, &V13402_1_1, NULL};	/*Property = IsMobile*/
const Value V13402_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13402_5 = {S52, 0, &V13402_5_1304, &P13402_1};	/*Property = ScreenPixelsHeight*/
const Value V13402_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13402_4 = {S54, 0, &V13402_4_1224, &P13402_5};	/*Property = ScreenPixelsWidth*/
const Profile P13402 = {"13402", &P13402_4};	/*Profile ID - 13402*/
const Value V13403_1_1 = {S2, NULL};	/*Value = True*/
const Property P13403_1 = {S50, 0, &V13403_1_1, NULL};	/*Property = IsMobile*/
const Value V13403_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13403_5 = {S52, 0, &V13403_5_1233, &P13403_1};	/*Property = ScreenPixelsHeight*/
const Value V13403_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13403_4 = {S54, 0, &V13403_4_1235, &P13403_5};	/*Property = ScreenPixelsWidth*/
const Profile P13403 = {"13403", &P13403_4};	/*Profile ID - 13403*/
const Value V13404_1_1 = {S2, NULL};	/*Value = True*/
const Property P13404_1 = {S50, 0, &V13404_1_1, NULL};	/*Property = IsMobile*/
const Value V13404_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13404_5 = {S52, 0, &V13404_5_1233, &P13404_1};	/*Property = ScreenPixelsHeight*/
const Value V13404_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13404_4 = {S54, 0, &V13404_4_1235, &P13404_5};	/*Property = ScreenPixelsWidth*/
const Profile P13404 = {"13404", &P13404_4};	/*Profile ID - 13404*/
const Value V16153_1_1 = {S2, NULL};	/*Value = True*/
const Property P16153_1 = {S50, 0, &V16153_1_1, NULL};	/*Property = IsMobile*/
const Value V16153_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16153_5 = {S52, 0, &V16153_5_1222, &P16153_1};	/*Property = ScreenPixelsHeight*/
const Value V16153_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16153_4 = {S54, 0, &V16153_4_1224, &P16153_5};	/*Property = ScreenPixelsWidth*/
const Profile P16153 = {"16153", &P16153_4};	/*Profile ID - 16153*/
const Value V13405_1_1 = {S2, NULL};	/*Value = True*/
const Property P13405_1 = {S50, 0, &V13405_1_1, NULL};	/*Property = IsMobile*/
const Value V13405_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13405_5 = {S52, 0, &V13405_5_1293, &P13405_1};	/*Property = ScreenPixelsHeight*/
const Value V13405_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13405_4 = {S54, 0, &V13405_4_1296, &P13405_5};	/*Property = ScreenPixelsWidth*/
const Profile P13405 = {"13405", &P13405_4};	/*Profile ID - 13405*/
const Value V13410_1_1 = {S2, NULL};	/*Value = True*/
const Property P13410_1 = {S50, 0, &V13410_1_1, NULL};	/*Property = IsMobile*/
const Value V13410_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13410_5 = {S52, 0, &V13410_5_1293, &P13410_1};	/*Property = ScreenPixelsHeight*/
const Value V13410_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13410_4 = {S54, 0, &V13410_4_1296, &P13410_5};	/*Property = ScreenPixelsWidth*/
const Profile P13410 = {"13410", &P13410_4};	/*Profile ID - 13410*/
const Value V13412_1_1 = {S2, NULL};	/*Value = True*/
const Property P13412_1 = {S50, 0, &V13412_1_1, NULL};	/*Property = IsMobile*/
const Value V13412_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13412_5 = {S52, 0, &V13412_5_1222, &P13412_1};	/*Property = ScreenPixelsHeight*/
const Value V13412_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13412_4 = {S54, 0, &V13412_4_1224, &P13412_5};	/*Property = ScreenPixelsWidth*/
const Profile P13412 = {"13412", &P13412_4};	/*Profile ID - 13412*/
const Value V16957_1_1 = {S2, NULL};	/*Value = True*/
const Property P16957_1 = {S50, 0, &V16957_1_1, NULL};	/*Property = IsMobile*/
const Value V16957_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16957_5 = {S52, 0, &V16957_5_1227, &P16957_1};	/*Property = ScreenPixelsHeight*/
const Value V16957_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16957_4 = {S54, 0, &V16957_4_1229, &P16957_5};	/*Property = ScreenPixelsWidth*/
const Profile P16957 = {"16957", &P16957_4};	/*Profile ID - 16957*/
const Value V12516_1_1 = {S2, NULL};	/*Value = True*/
const Property P12516_1 = {S50, 0, &V12516_1_1, NULL};	/*Property = IsMobile*/
const Value V12516_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12516_5 = {S52, 0, &V12516_5_1227, &P12516_1};	/*Property = ScreenPixelsHeight*/
const Value V12516_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12516_4 = {S54, 0, &V12516_4_1229, &P12516_5};	/*Property = ScreenPixelsWidth*/
const Profile P12516 = {"12516", &P12516_4};	/*Profile ID - 12516*/
const Value V13419_1_1 = {S2, NULL};	/*Value = True*/
const Property P13419_1 = {S50, 0, &V13419_1_1, NULL};	/*Property = IsMobile*/
const Value V13419_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13419_5 = {S52, 0, &V13419_5_1293, &P13419_1};	/*Property = ScreenPixelsHeight*/
const Value V13419_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13419_4 = {S54, 0, &V13419_4_1296, &P13419_5};	/*Property = ScreenPixelsWidth*/
const Profile P13419 = {"13419", &P13419_4};	/*Profile ID - 13419*/
const Value V18095_1_1 = {S2, NULL};	/*Value = True*/
const Property P18095_1 = {S50, 0, &V18095_1_1, NULL};	/*Property = IsMobile*/
const Value V18095_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P18095_5 = {S52, 0, &V18095_5_37, &P18095_1};	/*Property = ScreenPixelsHeight*/
const Value V18095_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P18095_4 = {S54, 0, &V18095_4_36, &P18095_5};	/*Property = ScreenPixelsWidth*/
const Profile P18095 = {"18095", &P18095_4};	/*Profile ID - 18095*/
const Profile P18094 = {"18094", NULL};	/*Profile ID - 18094*/
const Value V18373_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_279 = {S1, 0, &V18373_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18373_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_282 = {S3, 0, &V18373_282_57155, &P18373_279};	/*Property = BlobBuilder*/
const Value V18373_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_350 = {S4, 0, &V18373_350_57446, &P18373_282};	/*Property = Canvas*/
const Value V18373_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_283 = {S5, 0, &V18373_283_57161, &P18373_350};	/*Property = CssBackground*/
const Value V18373_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_284 = {S6, 0, &V18373_284_57164, &P18373_283};	/*Property = CssBorderImage*/
const Value V18373_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_285 = {S7, 0, &V18373_285_57167, &P18373_284};	/*Property = CssCanvas*/
const Value V18373_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_286 = {S8, 0, &V18373_286_57170, &P18373_285};	/*Property = CssColor*/
const Value V18373_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_237 = {S9, 0, &V18373_237_55786, &P18373_286};	/*Property = CssColumn*/
const Value V18373_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_287 = {S10, 0, &V18373_287_57173, &P18373_237};	/*Property = CssFlexbox*/
const Value V18373_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_288 = {S11, 0, &V18373_288_57176, &P18373_287};	/*Property = CssFont*/
const Value V18373_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_289 = {S12, 0, &V18373_289_86899, &P18373_288};	/*Property = CssImages*/
const Value V18373_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_291 = {S13, 0, &V18373_291_57182, &P18373_289};	/*Property = CssMediaQueries*/
const Value V18373_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_292 = {S14, 0, &V18373_292_57185, &P18373_291};	/*Property = CssMinMax*/
const Value V18373_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_293 = {S15, 0, &V18373_293_57188, &P18373_292};	/*Property = CssOverflow*/
const Value V18373_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_294 = {S16, 0, &V18373_294_57191, &P18373_293};	/*Property = CssPosition*/
const Value V18373_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_295 = {S17, 0, &V18373_295_57194, &P18373_294};	/*Property = CssText*/
const Value V18373_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_297 = {S18, 0, &V18373_297_57197, &P18373_295};	/*Property = CssTransforms*/
const Value V18373_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_241 = {S19, 0, &V18373_241_55801, &P18373_297};	/*Property = CssTransitions*/
const Value V18373_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_298 = {S20, 0, &V18373_298_57208, &P18373_241};	/*Property = CssUI*/
const Value V18373_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_300 = {S21, 0, &V18373_300_57217, &P18373_298};	/*Property = DataSet*/
const Value V18373_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_301 = {S22, 0, &V18373_301_57220, &P18373_300};	/*Property = DataUrl*/
const Value V18373_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_302 = {S23, 0, &V18373_302_57223, &P18373_301};	/*Property = DeviceOrientation*/
const Value V18373_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_304 = {S24, 0, &V18373_304_57229, &P18373_302};	/*Property = FileReader*/
const Value V18373_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_305 = {S25, 0, &V18373_305_57232, &P18373_304};	/*Property = FileSaver*/
const Value V18373_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_306 = {S26, 0, &V18373_306_57238, &P18373_305};	/*Property = FileWriter*/
const Value V18373_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_307 = {S27, 0, &V18373_307_57241, &P18373_306};	/*Property = FormData*/
const Value V18373_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_309 = {S28, 0, &V18373_309_57247, &P18373_307};	/*Property = Fullscreen*/
const Value V18373_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_310 = {S29, 0, &V18373_310_57250, &P18373_309};	/*Property = GeoLocation*/
const Value V18373_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_312 = {S30, 0, &V18373_312_57256, &P18373_310};	/*Property = History*/
const Value V18373_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_314 = {S31, 0, &V18373_314_57262, &P18373_312};	/*Property = Html5*/
const Value V18373_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_313 = {S32, 0, &V18373_313_57259, &P18373_314};	/*Property = Html-Media-Capture*/
const Value V18373_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_315 = {S33, 0, &V18373_315_57265, &P18373_313};	/*Property = Iframe*/
const Value V18373_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_316 = {S34, 0, &V18373_316_57268, &P18373_315};	/*Property = IndexedDB*/
const Value V18373_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_317 = {S35, 0, &V18373_317_57271, &P18373_316};	/*Property = Json*/
const Value V18373_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18373_15 = {S37, 0, &V18373_15_31, &P18373_317};	/*Property = LayoutEngine*/
const Value V18373_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_344 = {S38, 0, &V18373_344_57274, &P18373_15};	/*Property = Masking*/
const Value V18373_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_320 = {S39, 0, &V18373_320_57292, &P18373_344};	/*Property = PostMessage*/
const Value V18373_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_321 = {S40, 0, &V18373_321_57295, &P18373_320};	/*Property = Progress*/
const Value V18373_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_322 = {S41, 0, &V18373_322_57298, &P18373_321};	/*Property = Prompts*/
const Value V18373_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_323 = {S42, 0, &V18373_323_57301, &P18373_322};	/*Property = Selector*/
const Value V18373_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_324 = {S43, 0, &V18373_324_57307, &P18373_323};	/*Property = Svg*/
const Value V18373_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_327 = {S44, 0, &V18373_327_57316, &P18373_324};	/*Property = TouchEvents*/
const Value V18373_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_328 = {S45, 0, &V18373_328_57319, &P18373_327};	/*Property = Track*/
const Value V18373_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_329 = {S46, 0, &V18373_329_57325, &P18373_328};	/*Property = Video*/
const Value V18373_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_330 = {S47, 0, &V18373_330_57328, &P18373_329};	/*Property = Viewport*/
const Value V18373_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_332 = {S48, 0, &V18373_332_57340, &P18373_330};	/*Property = WebWorkers*/
const Value V18373_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18373_333 = {S49, 0, &V18373_333_57343, &P18373_332};	/*Property = Xhr2*/
const Profile P18373 = {"18373", &P18373_333};	/*Profile ID - 18373*/
const Value V17951_1_1 = {S2, NULL};	/*Value = True*/
const Property P17951_1 = {S50, 0, &V17951_1_1, NULL};	/*Property = IsMobile*/
const Value V17951_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17951_5 = {S52, 0, &V17951_5_1269, &P17951_1};	/*Property = ScreenPixelsHeight*/
const Value V17951_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17951_4 = {S54, 0, &V17951_4_1229, &P17951_5};	/*Property = ScreenPixelsWidth*/
const Profile P17951 = {"17951", &P17951_4};	/*Profile ID - 17951*/
const Value V13421_1_1 = {S2, NULL};	/*Value = True*/
const Property P13421_1 = {S50, 0, &V13421_1_1, NULL};	/*Property = IsMobile*/
const Value V13421_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13421_5 = {S52, 0, &V13421_5_1222, &P13421_1};	/*Property = ScreenPixelsHeight*/
const Value V13421_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13421_4 = {S54, 0, &V13421_4_1224, &P13421_5};	/*Property = ScreenPixelsWidth*/
const Profile P13421 = {"13421", &P13421_4};	/*Profile ID - 13421*/
const Value V13953_1_1 = {S2, NULL};	/*Value = True*/
const Property P13953_1 = {S50, 0, &V13953_1_1, NULL};	/*Property = IsMobile*/
const Value V13953_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13953_5 = {S52, 0, &V13953_5_1222, &P13953_1};	/*Property = ScreenPixelsHeight*/
const Value V13953_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13953_4 = {S54, 0, &V13953_4_1224, &P13953_5};	/*Property = ScreenPixelsWidth*/
const Profile P13953 = {"13953", &P13953_4};	/*Profile ID - 13953*/
const Value V13425_1_1 = {S2, NULL};	/*Value = True*/
const Property P13425_1 = {S50, 0, &V13425_1_1, NULL};	/*Property = IsMobile*/
const Value V13425_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13425_5 = {S52, 0, &V13425_5_1380, &P13425_1};	/*Property = ScreenPixelsHeight*/
const Value V13425_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13425_4 = {S54, 0, &V13425_4_1279, &P13425_5};	/*Property = ScreenPixelsWidth*/
const Profile P13425 = {"13425", &P13425_4};	/*Profile ID - 13425*/
const Profile P17603 = {"17603", NULL};	/*Profile ID - 17603*/
const Value V13438_1_1 = {S2, NULL};	/*Value = True*/
const Property P13438_1 = {S50, 0, &V13438_1_1, NULL};	/*Property = IsMobile*/
const Value V13438_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13438_5 = {S52, 0, &V13438_5_1222, &P13438_1};	/*Property = ScreenPixelsHeight*/
const Value V13438_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13438_4 = {S54, 0, &V13438_4_1224, &P13438_5};	/*Property = ScreenPixelsWidth*/
const Profile P13438 = {"13438", &P13438_4};	/*Profile ID - 13438*/
const Value V13398_1_1 = {S2, NULL};	/*Value = True*/
const Property P13398_1 = {S50, 0, &V13398_1_1, NULL};	/*Property = IsMobile*/
const Value V13398_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13398_5 = {S52, 0, &V13398_5_1227, &P13398_1};	/*Property = ScreenPixelsHeight*/
const Value V13398_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13398_4 = {S54, 0, &V13398_4_1229, &P13398_5};	/*Property = ScreenPixelsWidth*/
const Profile P13398 = {"13398", &P13398_4};	/*Profile ID - 13398*/
const Value V13446_1_1 = {S2, NULL};	/*Value = True*/
const Property P13446_1 = {S50, 0, &V13446_1_1, NULL};	/*Property = IsMobile*/
const Value V13446_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P13446_5 = {S52, 0, &V13446_5_1333, &P13446_1};	/*Property = ScreenPixelsHeight*/
const Value V13446_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P13446_4 = {S54, 0, &V13446_4_1335, &P13446_5};	/*Property = ScreenPixelsWidth*/
const Profile P13446 = {"13446", &P13446_4};	/*Profile ID - 13446*/
const Value V14345_1_1 = {S2, NULL};	/*Value = True*/
const Property P14345_1 = {S50, 0, &V14345_1_1, NULL};	/*Property = IsMobile*/
const Value V14345_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P14345_5 = {S52, 0, &V14345_5_1280, &P14345_1};	/*Property = ScreenPixelsHeight*/
const Value V14345_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14345_4 = {S54, 0, &V14345_4_1235, &P14345_5};	/*Property = ScreenPixelsWidth*/
const Profile P14345 = {"14345", &P14345_4};	/*Profile ID - 14345*/
const Value V19971_1_1 = {S2, NULL};	/*Value = True*/
const Property P19971_1 = {S50, 0, &V19971_1_1, NULL};	/*Property = IsMobile*/
const Value V19971_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P19971_5 = {S52, 0, &V19971_5_1222, &P19971_1};	/*Property = ScreenPixelsHeight*/
const Value V19971_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P19971_4 = {S54, 0, &V19971_4_1224, &P19971_5};	/*Property = ScreenPixelsWidth*/
const Profile P19971 = {"19971", &P19971_4};	/*Profile ID - 19971*/
const Value V16627_1_1 = {S2, NULL};	/*Value = True*/
const Property P16627_1 = {S50, 0, &V16627_1_1, NULL};	/*Property = IsMobile*/
const Value V16627_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16627_5 = {S52, 0, &V16627_5_1233, &P16627_1};	/*Property = ScreenPixelsHeight*/
const Value V16627_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16627_4 = {S54, 0, &V16627_4_1235, &P16627_5};	/*Property = ScreenPixelsWidth*/
const Profile P16627 = {"16627", &P16627_4};	/*Profile ID - 16627*/
const Value V16161_1_1 = {S2, NULL};	/*Value = True*/
const Property P16161_1 = {S50, 0, &V16161_1_1, NULL};	/*Property = IsMobile*/
const Value V16161_5_37452 = {S86, NULL};	/*Value = 48*/
const Property P16161_5 = {S52, 0, &V16161_5_37452, &P16161_1};	/*Property = ScreenPixelsHeight*/
const Value V16161_4_34872 = {S87, NULL};	/*Value = 84*/
const Property P16161_4 = {S54, 0, &V16161_4_34872, &P16161_5};	/*Property = ScreenPixelsWidth*/
const Profile P16161 = {"16161", &P16161_4};	/*Profile ID - 16161*/
const Value V18107_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18107_279 = {S1, 0, &V18107_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18107_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18107_282 = {S3, 0, &V18107_282_57154, &P18107_279};	/*Property = BlobBuilder*/
const Value V18107_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18107_350 = {S4, 0, &V18107_350_57444, &P18107_282};	/*Property = Canvas*/
const Value V18107_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18107_283 = {S5, 0, &V18107_283_57159, &P18107_350};	/*Property = CssBackground*/
const Value V18107_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18107_284 = {S6, 0, &V18107_284_57163, &P18107_283};	/*Property = CssBorderImage*/
const Value V18107_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18107_285 = {S7, 0, &V18107_285_57166, &P18107_284};	/*Property = CssCanvas*/
const Value V18107_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18107_286 = {S8, 0, &V18107_286_57168, &P18107_285};	/*Property = CssColor*/
const Value V18107_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18107_237 = {S9, 0, &V18107_237_55786, &P18107_286};	/*Property = CssColumn*/
const Value V18107_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18107_287 = {S10, 0, &V18107_287_57172, &P18107_237};	/*Property = CssFlexbox*/
const Value V18107_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18107_288 = {S11, 0, &V18107_288_57175, &P18107_287};	/*Property = CssFont*/
const Value V18107_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18107_289 = {S12, 0, &V18107_289_57759, &P18107_288};	/*Property = CssImages*/
const Value V18107_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18107_291 = {S13, 0, &V18107_291_57181, &P18107_289};	/*Property = CssMediaQueries*/
const Value V18107_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18107_292 = {S14, 0, &V18107_292_57183, &P18107_291};	/*Property = CssMinMax*/
const Value V18107_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18107_293 = {S15, 0, &V18107_293_57187, &P18107_292};	/*Property = CssOverflow*/
const Value V18107_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18107_294 = {S16, 0, &V18107_294_57190, &P18107_293};	/*Property = CssPosition*/
const Value V18107_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18107_295 = {S17, 0, &V18107_295_57193, &P18107_294};	/*Property = CssText*/
const Value V18107_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18107_297 = {S18, 0, &V18107_297_57195, &P18107_295};	/*Property = CssTransforms*/
const Value V18107_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18107_241 = {S19, 0, &V18107_241_55799, &P18107_297};	/*Property = CssTransitions*/
const Value V18107_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18107_298 = {S20, 0, &V18107_298_57206, &P18107_241};	/*Property = CssUI*/
const Value V18107_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18107_300 = {S21, 0, &V18107_300_57216, &P18107_298};	/*Property = DataSet*/
const Value V18107_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18107_301 = {S22, 0, &V18107_301_57218, &P18107_300};	/*Property = DataUrl*/
const Value V18107_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18107_302 = {S23, 0, &V18107_302_57222, &P18107_301};	/*Property = DeviceOrientation*/
const Value V18107_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18107_304 = {S24, 0, &V18107_304_57228, &P18107_302};	/*Property = FileReader*/
const Value V18107_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18107_305 = {S25, 0, &V18107_305_57231, &P18107_304};	/*Property = FileSaver*/
const Value V18107_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18107_306 = {S26, 0, &V18107_306_57237, &P18107_305};	/*Property = FileWriter*/
const Value V18107_307_57240 = {S0, NULL};	/*Value = False*/
const Property P18107_307 = {S27, 0, &V18107_307_57240, &P18107_306};	/*Property = FormData*/
const Value V18107_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18107_309 = {S28, 0, &V18107_309_57246, &P18107_307};	/*Property = Fullscreen*/
const Value V18107_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18107_310 = {S29, 0, &V18107_310_57248, &P18107_309};	/*Property = GeoLocation*/
const Value V18107_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18107_312 = {S30, 0, &V18107_312_57255, &P18107_310};	/*Property = History*/
const Value V18107_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18107_314 = {S31, 0, &V18107_314_57261, &P18107_312};	/*Property = Html5*/
const Value V18107_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18107_313 = {S32, 0, &V18107_313_57258, &P18107_314};	/*Property = Html-Media-Capture*/
const Value V18107_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18107_315 = {S33, 0, &V18107_315_57264, &P18107_313};	/*Property = Iframe*/
const Value V18107_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18107_316 = {S34, 0, &V18107_316_57267, &P18107_315};	/*Property = IndexedDB*/
const Value V18107_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18107_317 = {S35, 0, &V18107_317_57269, &P18107_316};	/*Property = Json*/
const Value V18107_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18107_15 = {S37, 0, &V18107_15_29, &P18107_317};	/*Property = LayoutEngine*/
const Value V18107_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18107_344 = {S38, 0, &V18107_344_57272, &P18107_15};	/*Property = Masking*/
const Value V18107_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18107_320 = {S39, 0, &V18107_320_57290, &P18107_344};	/*Property = PostMessage*/
const Value V18107_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18107_321 = {S40, 0, &V18107_321_57293, &P18107_320};	/*Property = Progress*/
const Value V18107_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18107_322 = {S41, 0, &V18107_322_57296, &P18107_321};	/*Property = Prompts*/
const Value V18107_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18107_323 = {S42, 0, &V18107_323_57299, &P18107_322};	/*Property = Selector*/
const Value V18107_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18107_324 = {S43, 0, &V18107_324_57306, &P18107_323};	/*Property = Svg*/
const Value V18107_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18107_327 = {S44, 0, &V18107_327_57315, &P18107_324};	/*Property = TouchEvents*/
const Value V18107_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18107_328 = {S45, 0, &V18107_328_57318, &P18107_327};	/*Property = Track*/
const Value V18107_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18107_329 = {S46, 0, &V18107_329_57323, &P18107_328};	/*Property = Video*/
const Value V18107_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18107_330 = {S47, 0, &V18107_330_57326, &P18107_329};	/*Property = Viewport*/
const Value V18107_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18107_332 = {S48, 0, &V18107_332_57339, &P18107_330};	/*Property = WebWorkers*/
const Value V18107_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18107_333 = {S49, 0, &V18107_333_57342, &P18107_332};	/*Property = Xhr2*/
const Profile P18107 = {"18107", &P18107_333};	/*Profile ID - 18107*/
const Profile P18588 = {"18588", NULL};	/*Profile ID - 18588*/
const Value V3504_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_279 = {S1, 0, &V3504_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3504_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_282 = {S3, 0, &V3504_282_57155, &P3504_279};	/*Property = BlobBuilder*/
const Value V3504_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_350 = {S4, 0, &V3504_350_57446, &P3504_282};	/*Property = Canvas*/
const Value V3504_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_283 = {S5, 0, &V3504_283_57161, &P3504_350};	/*Property = CssBackground*/
const Value V3504_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_284 = {S6, 0, &V3504_284_57164, &P3504_283};	/*Property = CssBorderImage*/
const Value V3504_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_285 = {S7, 0, &V3504_285_57167, &P3504_284};	/*Property = CssCanvas*/
const Value V3504_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_286 = {S8, 0, &V3504_286_57170, &P3504_285};	/*Property = CssColor*/
const Value V3504_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_237 = {S9, 0, &V3504_237_55786, &P3504_286};	/*Property = CssColumn*/
const Value V3504_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_287 = {S10, 0, &V3504_287_57173, &P3504_237};	/*Property = CssFlexbox*/
const Value V3504_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_288 = {S11, 0, &V3504_288_57176, &P3504_287};	/*Property = CssFont*/
const Value V3504_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_289 = {S12, 0, &V3504_289_86899, &P3504_288};	/*Property = CssImages*/
const Value V3504_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_291 = {S13, 0, &V3504_291_57182, &P3504_289};	/*Property = CssMediaQueries*/
const Value V3504_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_292 = {S14, 0, &V3504_292_57185, &P3504_291};	/*Property = CssMinMax*/
const Value V3504_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_293 = {S15, 0, &V3504_293_57188, &P3504_292};	/*Property = CssOverflow*/
const Value V3504_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_294 = {S16, 0, &V3504_294_57191, &P3504_293};	/*Property = CssPosition*/
const Value V3504_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_295 = {S17, 0, &V3504_295_57194, &P3504_294};	/*Property = CssText*/
const Value V3504_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_297 = {S18, 0, &V3504_297_57197, &P3504_295};	/*Property = CssTransforms*/
const Value V3504_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_241 = {S19, 0, &V3504_241_55801, &P3504_297};	/*Property = CssTransitions*/
const Value V3504_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_298 = {S20, 0, &V3504_298_57208, &P3504_241};	/*Property = CssUI*/
const Value V3504_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_300 = {S21, 0, &V3504_300_57217, &P3504_298};	/*Property = DataSet*/
const Value V3504_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_301 = {S22, 0, &V3504_301_57220, &P3504_300};	/*Property = DataUrl*/
const Value V3504_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_302 = {S23, 0, &V3504_302_57223, &P3504_301};	/*Property = DeviceOrientation*/
const Value V3504_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_304 = {S24, 0, &V3504_304_57229, &P3504_302};	/*Property = FileReader*/
const Value V3504_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_305 = {S25, 0, &V3504_305_57232, &P3504_304};	/*Property = FileSaver*/
const Value V3504_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_306 = {S26, 0, &V3504_306_57238, &P3504_305};	/*Property = FileWriter*/
const Value V3504_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_307 = {S27, 0, &V3504_307_57241, &P3504_306};	/*Property = FormData*/
const Value V3504_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_309 = {S28, 0, &V3504_309_57247, &P3504_307};	/*Property = Fullscreen*/
const Value V3504_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_310 = {S29, 0, &V3504_310_57250, &P3504_309};	/*Property = GeoLocation*/
const Value V3504_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_312 = {S30, 0, &V3504_312_57256, &P3504_310};	/*Property = History*/
const Value V3504_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_314 = {S31, 0, &V3504_314_57262, &P3504_312};	/*Property = Html5*/
const Value V3504_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_313 = {S32, 0, &V3504_313_57259, &P3504_314};	/*Property = Html-Media-Capture*/
const Value V3504_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_315 = {S33, 0, &V3504_315_57265, &P3504_313};	/*Property = Iframe*/
const Value V3504_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_316 = {S34, 0, &V3504_316_57268, &P3504_315};	/*Property = IndexedDB*/
const Value V3504_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_317 = {S35, 0, &V3504_317_57271, &P3504_316};	/*Property = Json*/
const Value V3504_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3504_15 = {S37, 0, &V3504_15_28, &P3504_317};	/*Property = LayoutEngine*/
const Value V3504_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_344 = {S38, 0, &V3504_344_57274, &P3504_15};	/*Property = Masking*/
const Value V3504_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_320 = {S39, 0, &V3504_320_57292, &P3504_344};	/*Property = PostMessage*/
const Value V3504_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_321 = {S40, 0, &V3504_321_57295, &P3504_320};	/*Property = Progress*/
const Value V3504_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_322 = {S41, 0, &V3504_322_57298, &P3504_321};	/*Property = Prompts*/
const Value V3504_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_323 = {S42, 0, &V3504_323_57301, &P3504_322};	/*Property = Selector*/
const Value V3504_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_324 = {S43, 0, &V3504_324_57307, &P3504_323};	/*Property = Svg*/
const Value V3504_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_327 = {S44, 0, &V3504_327_57316, &P3504_324};	/*Property = TouchEvents*/
const Value V3504_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_328 = {S45, 0, &V3504_328_57319, &P3504_327};	/*Property = Track*/
const Value V3504_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_329 = {S46, 0, &V3504_329_57325, &P3504_328};	/*Property = Video*/
const Value V3504_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_330 = {S47, 0, &V3504_330_57328, &P3504_329};	/*Property = Viewport*/
const Value V3504_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_332 = {S48, 0, &V3504_332_57340, &P3504_330};	/*Property = WebWorkers*/
const Value V3504_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3504_333 = {S49, 0, &V3504_333_57343, &P3504_332};	/*Property = Xhr2*/
const Profile P3504 = {"3504", &P3504_333};	/*Profile ID - 3504*/
const Value V17799_1_1 = {S2, NULL};	/*Value = True*/
const Property P17799_1 = {S50, 0, &V17799_1_1, NULL};	/*Property = IsMobile*/
const Value V17799_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17799_5 = {S52, 0, &V17799_5_1233, &P17799_1};	/*Property = ScreenPixelsHeight*/
const Value V17799_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17799_4 = {S54, 0, &V17799_4_1235, &P17799_5};	/*Property = ScreenPixelsWidth*/
const Profile P17799 = {"17799", &P17799_4};	/*Profile ID - 17799*/
const Profile P17811 = {"17811", NULL};	/*Profile ID - 17811*/
const Value V11766_1_1 = {S2, NULL};	/*Value = True*/
const Property P11766_1 = {S50, 0, &V11766_1_1, NULL};	/*Property = IsMobile*/
const Value V11766_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P11766_5 = {S52, 0, &V11766_5_1280, &P11766_1};	/*Property = ScreenPixelsHeight*/
const Value V11766_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11766_4 = {S54, 0, &V11766_4_1235, &P11766_5};	/*Property = ScreenPixelsWidth*/
const Profile P11766 = {"11766", &P11766_4};	/*Profile ID - 11766*/
const Value V4902_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_279 = {S1, 0, &V4902_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4902_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_282 = {S3, 0, &V4902_282_57155, &P4902_279};	/*Property = BlobBuilder*/
const Value V4902_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_350 = {S4, 0, &V4902_350_57446, &P4902_282};	/*Property = Canvas*/
const Value V4902_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_283 = {S5, 0, &V4902_283_57161, &P4902_350};	/*Property = CssBackground*/
const Value V4902_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_284 = {S6, 0, &V4902_284_57164, &P4902_283};	/*Property = CssBorderImage*/
const Value V4902_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_285 = {S7, 0, &V4902_285_57167, &P4902_284};	/*Property = CssCanvas*/
const Value V4902_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_286 = {S8, 0, &V4902_286_57170, &P4902_285};	/*Property = CssColor*/
const Value V4902_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_237 = {S9, 0, &V4902_237_55786, &P4902_286};	/*Property = CssColumn*/
const Value V4902_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_287 = {S10, 0, &V4902_287_57173, &P4902_237};	/*Property = CssFlexbox*/
const Value V4902_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_288 = {S11, 0, &V4902_288_57176, &P4902_287};	/*Property = CssFont*/
const Value V4902_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_289 = {S12, 0, &V4902_289_86899, &P4902_288};	/*Property = CssImages*/
const Value V4902_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_291 = {S13, 0, &V4902_291_57182, &P4902_289};	/*Property = CssMediaQueries*/
const Value V4902_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_292 = {S14, 0, &V4902_292_57185, &P4902_291};	/*Property = CssMinMax*/
const Value V4902_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_293 = {S15, 0, &V4902_293_57188, &P4902_292};	/*Property = CssOverflow*/
const Value V4902_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_294 = {S16, 0, &V4902_294_57191, &P4902_293};	/*Property = CssPosition*/
const Value V4902_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_295 = {S17, 0, &V4902_295_57194, &P4902_294};	/*Property = CssText*/
const Value V4902_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_297 = {S18, 0, &V4902_297_57197, &P4902_295};	/*Property = CssTransforms*/
const Value V4902_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_241 = {S19, 0, &V4902_241_55801, &P4902_297};	/*Property = CssTransitions*/
const Value V4902_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_298 = {S20, 0, &V4902_298_57208, &P4902_241};	/*Property = CssUI*/
const Value V4902_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_300 = {S21, 0, &V4902_300_57217, &P4902_298};	/*Property = DataSet*/
const Value V4902_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_301 = {S22, 0, &V4902_301_57220, &P4902_300};	/*Property = DataUrl*/
const Value V4902_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_302 = {S23, 0, &V4902_302_57223, &P4902_301};	/*Property = DeviceOrientation*/
const Value V4902_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_304 = {S24, 0, &V4902_304_57229, &P4902_302};	/*Property = FileReader*/
const Value V4902_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_305 = {S25, 0, &V4902_305_57232, &P4902_304};	/*Property = FileSaver*/
const Value V4902_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_306 = {S26, 0, &V4902_306_57238, &P4902_305};	/*Property = FileWriter*/
const Value V4902_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_307 = {S27, 0, &V4902_307_57241, &P4902_306};	/*Property = FormData*/
const Value V4902_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_309 = {S28, 0, &V4902_309_57247, &P4902_307};	/*Property = Fullscreen*/
const Value V4902_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_310 = {S29, 0, &V4902_310_57250, &P4902_309};	/*Property = GeoLocation*/
const Value V4902_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_312 = {S30, 0, &V4902_312_57256, &P4902_310};	/*Property = History*/
const Value V4902_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_314 = {S31, 0, &V4902_314_57262, &P4902_312};	/*Property = Html5*/
const Value V4902_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_313 = {S32, 0, &V4902_313_57259, &P4902_314};	/*Property = Html-Media-Capture*/
const Value V4902_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_315 = {S33, 0, &V4902_315_57265, &P4902_313};	/*Property = Iframe*/
const Value V4902_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_316 = {S34, 0, &V4902_316_57268, &P4902_315};	/*Property = IndexedDB*/
const Value V4902_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_317 = {S35, 0, &V4902_317_57271, &P4902_316};	/*Property = Json*/
const Value V4902_15_43265 = {S88, NULL};	/*Value = Lumi*/
const Property P4902_15 = {S37, 0, &V4902_15_43265, &P4902_317};	/*Property = LayoutEngine*/
const Value V4902_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_344 = {S38, 0, &V4902_344_57274, &P4902_15};	/*Property = Masking*/
const Value V4902_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_320 = {S39, 0, &V4902_320_57292, &P4902_344};	/*Property = PostMessage*/
const Value V4902_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_321 = {S40, 0, &V4902_321_57295, &P4902_320};	/*Property = Progress*/
const Value V4902_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_322 = {S41, 0, &V4902_322_57298, &P4902_321};	/*Property = Prompts*/
const Value V4902_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_323 = {S42, 0, &V4902_323_57301, &P4902_322};	/*Property = Selector*/
const Value V4902_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_324 = {S43, 0, &V4902_324_57307, &P4902_323};	/*Property = Svg*/
const Value V4902_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_327 = {S44, 0, &V4902_327_57316, &P4902_324};	/*Property = TouchEvents*/
const Value V4902_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_328 = {S45, 0, &V4902_328_57319, &P4902_327};	/*Property = Track*/
const Value V4902_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_329 = {S46, 0, &V4902_329_57325, &P4902_328};	/*Property = Video*/
const Value V4902_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_330 = {S47, 0, &V4902_330_57328, &P4902_329};	/*Property = Viewport*/
const Value V4902_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_332 = {S48, 0, &V4902_332_57340, &P4902_330};	/*Property = WebWorkers*/
const Value V4902_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4902_333 = {S49, 0, &V4902_333_57343, &P4902_332};	/*Property = Xhr2*/
const Profile P4902 = {"4902", &P4902_333};	/*Profile ID - 4902*/
const Value V15278_1_1 = {S2, NULL};	/*Value = True*/
const Property P15278_1 = {S50, 0, &V15278_1_1, NULL};	/*Property = IsMobile*/
const Value V15278_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15278_5 = {S52, 0, &V15278_5_1293, &P15278_1};	/*Property = ScreenPixelsHeight*/
const Value V15278_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15278_4 = {S54, 0, &V15278_4_1296, &P15278_5};	/*Property = ScreenPixelsWidth*/
const Profile P15278 = {"15278", &P15278_4};	/*Profile ID - 15278*/
const Value V13459_1_1 = {S2, NULL};	/*Value = True*/
const Property P13459_1 = {S50, 0, &V13459_1_1, NULL};	/*Property = IsMobile*/
const Value V13459_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13459_5 = {S52, 0, &V13459_5_1222, &P13459_1};	/*Property = ScreenPixelsHeight*/
const Value V13459_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13459_4 = {S54, 0, &V13459_4_1224, &P13459_5};	/*Property = ScreenPixelsWidth*/
const Profile P13459 = {"13459", &P13459_4};	/*Profile ID - 13459*/
const Value V12465_1_1 = {S2, NULL};	/*Value = True*/
const Property P12465_1 = {S50, 0, &V12465_1_1, NULL};	/*Property = IsMobile*/
const Value V12465_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12465_5 = {S52, 0, &V12465_5_1222, &P12465_1};	/*Property = ScreenPixelsHeight*/
const Value V12465_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12465_4 = {S54, 0, &V12465_4_1224, &P12465_5};	/*Property = ScreenPixelsWidth*/
const Profile P12465 = {"12465", &P12465_4};	/*Profile ID - 12465*/
const Value V13558_1_1 = {S2, NULL};	/*Value = True*/
const Property P13558_1 = {S50, 0, &V13558_1_1, NULL};	/*Property = IsMobile*/
const Value V13558_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13558_5 = {S52, 0, &V13558_5_1293, &P13558_1};	/*Property = ScreenPixelsHeight*/
const Value V13558_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13558_4 = {S54, 0, &V13558_4_1224, &P13558_5};	/*Property = ScreenPixelsWidth*/
const Profile P13558 = {"13558", &P13558_4};	/*Profile ID - 13558*/
const Value V13462_1_1 = {S2, NULL};	/*Value = True*/
const Property P13462_1 = {S50, 0, &V13462_1_1, NULL};	/*Property = IsMobile*/
const Value V13462_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13462_5 = {S52, 0, &V13462_5_1233, &P13462_1};	/*Property = ScreenPixelsHeight*/
const Value V13462_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13462_4 = {S54, 0, &V13462_4_1235, &P13462_5};	/*Property = ScreenPixelsWidth*/
const Profile P13462 = {"13462", &P13462_4};	/*Profile ID - 13462*/
const Profile P6590 = {"6590", NULL};	/*Profile ID - 6590*/
const Value V13453_1_1 = {S2, NULL};	/*Value = True*/
const Property P13453_1 = {S50, 0, &V13453_1_1, NULL};	/*Property = IsMobile*/
const Value V13453_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13453_5 = {S52, 0, &V13453_5_1222, &P13453_1};	/*Property = ScreenPixelsHeight*/
const Value V13453_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13453_4 = {S54, 0, &V13453_4_1224, &P13453_5};	/*Property = ScreenPixelsWidth*/
const Profile P13453 = {"13453", &P13453_4};	/*Profile ID - 13453*/
const Value V11957_1_1 = {S2, NULL};	/*Value = True*/
const Property P11957_1 = {S50, 0, &V11957_1_1, NULL};	/*Property = IsMobile*/
const Value V11957_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11957_5 = {S52, 0, &V11957_5_1222, &P11957_1};	/*Property = ScreenPixelsHeight*/
const Value V11957_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11957_4 = {S54, 0, &V11957_4_1224, &P11957_5};	/*Property = ScreenPixelsWidth*/
const Profile P11957 = {"11957", &P11957_4};	/*Profile ID - 11957*/
const Value V12191_1_1 = {S2, NULL};	/*Value = True*/
const Property P12191_1 = {S50, 0, &V12191_1_1, NULL};	/*Property = IsMobile*/
const Value V12191_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12191_5 = {S52, 0, &V12191_5_1227, &P12191_1};	/*Property = ScreenPixelsHeight*/
const Value V12191_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12191_4 = {S54, 0, &V12191_4_1229, &P12191_5};	/*Property = ScreenPixelsWidth*/
const Profile P12191 = {"12191", &P12191_4};	/*Profile ID - 12191*/
const Value V16444_1_1 = {S2, NULL};	/*Value = True*/
const Property P16444_1 = {S50, 0, &V16444_1_1, NULL};	/*Property = IsMobile*/
const Value V16444_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16444_5 = {S52, 0, &V16444_5_1227, &P16444_1};	/*Property = ScreenPixelsHeight*/
const Value V16444_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16444_4 = {S54, 0, &V16444_4_1229, &P16444_5};	/*Property = ScreenPixelsWidth*/
const Profile P16444 = {"16444", &P16444_4};	/*Profile ID - 16444*/
const Value V15163_1_1 = {S2, NULL};	/*Value = True*/
const Property P15163_1 = {S50, 0, &V15163_1_1, NULL};	/*Property = IsMobile*/
const Value V15163_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15163_5 = {S52, 0, &V15163_5_1222, &P15163_1};	/*Property = ScreenPixelsHeight*/
const Value V15163_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15163_4 = {S54, 0, &V15163_4_1224, &P15163_5};	/*Property = ScreenPixelsWidth*/
const Profile P15163 = {"15163", &P15163_4};	/*Profile ID - 15163*/
const Value V12134_1_1 = {S2, NULL};	/*Value = True*/
const Property P12134_1 = {S50, 0, &V12134_1_1, NULL};	/*Property = IsMobile*/
const Value V12134_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P12134_5 = {S52, 0, &V12134_5_1269, &P12134_1};	/*Property = ScreenPixelsHeight*/
const Value V12134_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12134_4 = {S54, 0, &V12134_4_1229, &P12134_5};	/*Property = ScreenPixelsWidth*/
const Profile P12134 = {"12134", &P12134_4};	/*Profile ID - 12134*/
const Value V13468_1_1 = {S2, NULL};	/*Value = True*/
const Property P13468_1 = {S50, 0, &V13468_1_1, NULL};	/*Property = IsMobile*/
const Value V13468_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13468_5 = {S52, 0, &V13468_5_1233, &P13468_1};	/*Property = ScreenPixelsHeight*/
const Value V13468_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13468_4 = {S54, 0, &V13468_4_1235, &P13468_5};	/*Property = ScreenPixelsWidth*/
const Profile P13468 = {"13468", &P13468_4};	/*Profile ID - 13468*/
const Value V13469_1_1 = {S2, NULL};	/*Value = True*/
const Property P13469_1 = {S50, 0, &V13469_1_1, NULL};	/*Property = IsMobile*/
const Value V13469_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13469_5 = {S52, 0, &V13469_5_1293, &P13469_1};	/*Property = ScreenPixelsHeight*/
const Value V13469_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13469_4 = {S54, 0, &V13469_4_1296, &P13469_5};	/*Property = ScreenPixelsWidth*/
const Profile P13469 = {"13469", &P13469_4};	/*Profile ID - 13469*/
const Value V18046_1_1 = {S2, NULL};	/*Value = True*/
const Property P18046_1 = {S50, 0, &V18046_1_1, NULL};	/*Property = IsMobile*/
const Value V18046_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P18046_5 = {S52, 0, &V18046_5_1832, &P18046_1};	/*Property = ScreenPixelsHeight*/
const Value V18046_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P18046_4 = {S54, 0, &V18046_4_5645, &P18046_5};	/*Property = ScreenPixelsWidth*/
const Profile P18046 = {"18046", &P18046_4};	/*Profile ID - 18046*/
const Value V17730_1_1 = {S2, NULL};	/*Value = True*/
const Property P17730_1 = {S50, 0, &V17730_1_1, NULL};	/*Property = IsMobile*/
const Value V17730_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17730_5 = {S52, 0, &V17730_5_1277, &P17730_1};	/*Property = ScreenPixelsHeight*/
const Value V17730_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P17730_4 = {S54, 0, &V17730_4_2336, &P17730_5};	/*Property = ScreenPixelsWidth*/
const Profile P17730 = {"17730", &P17730_4};	/*Profile ID - 17730*/
const Value V17732_1_1 = {S2, NULL};	/*Value = True*/
const Property P17732_1 = {S50, 0, &V17732_1_1, NULL};	/*Property = IsMobile*/
const Value V17732_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17732_5 = {S52, 0, &V17732_5_1304, &P17732_1};	/*Property = ScreenPixelsHeight*/
const Value V17732_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P17732_4 = {S54, 0, &V17732_4_2336, &P17732_5};	/*Property = ScreenPixelsWidth*/
const Profile P17732 = {"17732", &P17732_4};	/*Profile ID - 17732*/
const Value V17735_1_1 = {S2, NULL};	/*Value = True*/
const Property P17735_1 = {S50, 0, &V17735_1_1, NULL};	/*Property = IsMobile*/
const Value V17735_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17735_5 = {S52, 0, &V17735_5_1277, &P17735_1};	/*Property = ScreenPixelsHeight*/
const Value V17735_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P17735_4 = {S54, 0, &V17735_4_2336, &P17735_5};	/*Property = ScreenPixelsWidth*/
const Profile P17735 = {"17735", &P17735_4};	/*Profile ID - 17735*/
const Value V13472_1_1 = {S2, NULL};	/*Value = True*/
const Property P13472_1 = {S50, 0, &V13472_1_1, NULL};	/*Property = IsMobile*/
const Value V13472_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P13472_5 = {S52, 0, &V13472_5_1832, &P13472_1};	/*Property = ScreenPixelsHeight*/
const Value V13472_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P13472_4 = {S54, 0, &V13472_4_2336, &P13472_5};	/*Property = ScreenPixelsWidth*/
const Profile P13472 = {"13472", &P13472_4};	/*Profile ID - 13472*/
const Value V13474_1_1 = {S2, NULL};	/*Value = True*/
const Property P13474_1 = {S50, 0, &V13474_1_1, NULL};	/*Property = IsMobile*/
const Value V13474_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13474_5 = {S52, 0, &V13474_5_1380, &P13474_1};	/*Property = ScreenPixelsHeight*/
const Value V13474_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13474_4 = {S54, 0, &V13474_4_1279, &P13474_5};	/*Property = ScreenPixelsWidth*/
const Profile P13474 = {"13474", &P13474_4};	/*Profile ID - 13474*/
const Value V13475_1_1 = {S2, NULL};	/*Value = True*/
const Property P13475_1 = {S50, 0, &V13475_1_1, NULL};	/*Property = IsMobile*/
const Value V13475_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13475_5 = {S52, 0, &V13475_5_1293, &P13475_1};	/*Property = ScreenPixelsHeight*/
const Value V13475_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13475_4 = {S54, 0, &V13475_4_1296, &P13475_5};	/*Property = ScreenPixelsWidth*/
const Profile P13475 = {"13475", &P13475_4};	/*Profile ID - 13475*/
const Value V13479_1_1 = {S2, NULL};	/*Value = True*/
const Property P13479_1 = {S50, 0, &V13479_1_1, NULL};	/*Property = IsMobile*/
const Value V13479_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P13479_5 = {S52, 0, &V13479_5_1333, &P13479_1};	/*Property = ScreenPixelsHeight*/
const Value V13479_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P13479_4 = {S54, 0, &V13479_4_1335, &P13479_5};	/*Property = ScreenPixelsWidth*/
const Profile P13479 = {"13479", &P13479_4};	/*Profile ID - 13479*/
const Value V13481_1_1 = {S2, NULL};	/*Value = True*/
const Property P13481_1 = {S50, 0, &V13481_1_1, NULL};	/*Property = IsMobile*/
const Value V13481_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13481_5 = {S52, 0, &V13481_5_1233, &P13481_1};	/*Property = ScreenPixelsHeight*/
const Value V13481_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13481_4 = {S54, 0, &V13481_4_1235, &P13481_5};	/*Property = ScreenPixelsWidth*/
const Profile P13481 = {"13481", &P13481_4};	/*Profile ID - 13481*/
const Value V13482_1_1 = {S2, NULL};	/*Value = True*/
const Property P13482_1 = {S50, 0, &V13482_1_1, NULL};	/*Property = IsMobile*/
const Value V13482_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13482_5 = {S52, 0, &V13482_5_1227, &P13482_1};	/*Property = ScreenPixelsHeight*/
const Value V13482_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13482_4 = {S54, 0, &V13482_4_1229, &P13482_5};	/*Property = ScreenPixelsWidth*/
const Profile P13482 = {"13482", &P13482_4};	/*Profile ID - 13482*/
const Value V13483_1_1 = {S2, NULL};	/*Value = True*/
const Property P13483_1 = {S50, 0, &V13483_1_1, NULL};	/*Property = IsMobile*/
const Value V13483_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13483_5 = {S52, 0, &V13483_5_1380, &P13483_1};	/*Property = ScreenPixelsHeight*/
const Value V13483_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13483_4 = {S54, 0, &V13483_4_1224, &P13483_5};	/*Property = ScreenPixelsWidth*/
const Profile P13483 = {"13483", &P13483_4};	/*Profile ID - 13483*/
const Value V13486_1_1 = {S2, NULL};	/*Value = True*/
const Property P13486_1 = {S50, 0, &V13486_1_1, NULL};	/*Property = IsMobile*/
const Value V13486_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13486_5 = {S52, 0, &V13486_5_1293, &P13486_1};	/*Property = ScreenPixelsHeight*/
const Value V13486_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13486_4 = {S54, 0, &V13486_4_1296, &P13486_5};	/*Property = ScreenPixelsWidth*/
const Profile P13486 = {"13486", &P13486_4};	/*Profile ID - 13486*/
const Value V13488_1_1 = {S2, NULL};	/*Value = True*/
const Property P13488_1 = {S50, 0, &V13488_1_1, NULL};	/*Property = IsMobile*/
const Value V13488_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13488_5 = {S52, 0, &V13488_5_1380, &P13488_1};	/*Property = ScreenPixelsHeight*/
const Value V13488_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13488_4 = {S54, 0, &V13488_4_1279, &P13488_5};	/*Property = ScreenPixelsWidth*/
const Profile P13488 = {"13488", &P13488_4};	/*Profile ID - 13488*/
const Profile P6476 = {"6476", NULL};	/*Profile ID - 6476*/
const Value V13489_1_1 = {S2, NULL};	/*Value = True*/
const Property P13489_1 = {S50, 0, &V13489_1_1, NULL};	/*Property = IsMobile*/
const Value V13489_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13489_5 = {S52, 0, &V13489_5_1222, &P13489_1};	/*Property = ScreenPixelsHeight*/
const Value V13489_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13489_4 = {S54, 0, &V13489_4_1224, &P13489_5};	/*Property = ScreenPixelsWidth*/
const Profile P13489 = {"13489", &P13489_4};	/*Profile ID - 13489*/
const Value V13494_1_1 = {S2, NULL};	/*Value = True*/
const Property P13494_1 = {S50, 0, &V13494_1_1, NULL};	/*Property = IsMobile*/
const Value V13494_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13494_5 = {S52, 0, &V13494_5_1380, &P13494_1};	/*Property = ScreenPixelsHeight*/
const Value V13494_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13494_4 = {S54, 0, &V13494_4_1279, &P13494_5};	/*Property = ScreenPixelsWidth*/
const Profile P13494 = {"13494", &P13494_4};	/*Profile ID - 13494*/
const Value V17248_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17248_279 = {S1, 0, &V17248_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17248_282_57153 = {S2, NULL};	/*Value = True*/
const Property P17248_282 = {S3, 0, &V17248_282_57153, &P17248_279};	/*Property = BlobBuilder*/
const Value V17248_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17248_350 = {S4, 0, &V17248_350_57444, &P17248_282};	/*Property = Canvas*/
const Value V17248_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17248_283 = {S5, 0, &V17248_283_57159, &P17248_350};	/*Property = CssBackground*/
const Value V17248_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17248_284 = {S6, 0, &V17248_284_57163, &P17248_283};	/*Property = CssBorderImage*/
const Value V17248_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17248_285 = {S7, 0, &V17248_285_57166, &P17248_284};	/*Property = CssCanvas*/
const Value V17248_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17248_286 = {S8, 0, &V17248_286_57168, &P17248_285};	/*Property = CssColor*/
const Value V17248_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17248_237 = {S9, 0, &V17248_237_55786, &P17248_286};	/*Property = CssColumn*/
const Value V17248_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17248_287 = {S10, 0, &V17248_287_57172, &P17248_237};	/*Property = CssFlexbox*/
const Value V17248_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17248_288 = {S11, 0, &V17248_288_57175, &P17248_287};	/*Property = CssFont*/
const Value V17248_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17248_289 = {S12, 0, &V17248_289_57759, &P17248_288};	/*Property = CssImages*/
const Value V17248_291_57180 = {S2, NULL};	/*Value = True*/
const Property P17248_291 = {S13, 0, &V17248_291_57180, &P17248_289};	/*Property = CssMediaQueries*/
const Value V17248_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17248_292 = {S14, 0, &V17248_292_57183, &P17248_291};	/*Property = CssMinMax*/
const Value V17248_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17248_293 = {S15, 0, &V17248_293_57187, &P17248_292};	/*Property = CssOverflow*/
const Value V17248_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17248_294 = {S16, 0, &V17248_294_57189, &P17248_293};	/*Property = CssPosition*/
const Value V17248_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17248_295 = {S17, 0, &V17248_295_57192, &P17248_294};	/*Property = CssText*/
const Value V17248_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17248_297 = {S18, 0, &V17248_297_57195, &P17248_295};	/*Property = CssTransforms*/
const Value V17248_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17248_241 = {S19, 0, &V17248_241_55799, &P17248_297};	/*Property = CssTransitions*/
const Value V17248_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17248_298 = {S20, 0, &V17248_298_57206, &P17248_241};	/*Property = CssUI*/
const Value V17248_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17248_300 = {S21, 0, &V17248_300_57216, &P17248_298};	/*Property = DataSet*/
const Value V17248_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17248_301 = {S22, 0, &V17248_301_57218, &P17248_300};	/*Property = DataUrl*/
const Value V17248_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17248_302 = {S23, 0, &V17248_302_57222, &P17248_301};	/*Property = DeviceOrientation*/
const Value V17248_304_57227 = {S2, NULL};	/*Value = True*/
const Property P17248_304 = {S24, 0, &V17248_304_57227, &P17248_302};	/*Property = FileReader*/
const Value V17248_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17248_305 = {S25, 0, &V17248_305_57231, &P17248_304};	/*Property = FileSaver*/
const Value V17248_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17248_306 = {S26, 0, &V17248_306_57237, &P17248_305};	/*Property = FileWriter*/
const Value V17248_307_57239 = {S2, NULL};	/*Value = True*/
const Property P17248_307 = {S27, 0, &V17248_307_57239, &P17248_306};	/*Property = FormData*/
const Value V17248_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17248_309 = {S28, 0, &V17248_309_57246, &P17248_307};	/*Property = Fullscreen*/
const Value V17248_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17248_310 = {S29, 0, &V17248_310_57248, &P17248_309};	/*Property = GeoLocation*/
const Value V17248_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17248_312 = {S30, 0, &V17248_312_57255, &P17248_310};	/*Property = History*/
const Value V17248_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17248_314 = {S31, 0, &V17248_314_57261, &P17248_312};	/*Property = Html5*/
const Value V17248_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17248_313 = {S32, 0, &V17248_313_57258, &P17248_314};	/*Property = Html-Media-Capture*/
const Value V17248_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17248_315 = {S33, 0, &V17248_315_57264, &P17248_313};	/*Property = Iframe*/
const Value V17248_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17248_316 = {S34, 0, &V17248_316_57267, &P17248_315};	/*Property = IndexedDB*/
const Value V17248_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17248_317 = {S35, 0, &V17248_317_57269, &P17248_316};	/*Property = Json*/
const Value V17248_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17248_15 = {S37, 0, &V17248_15_28, &P17248_317};	/*Property = LayoutEngine*/
const Value V17248_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17248_344 = {S38, 0, &V17248_344_57272, &P17248_15};	/*Property = Masking*/
const Value V17248_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17248_320 = {S39, 0, &V17248_320_57290, &P17248_344};	/*Property = PostMessage*/
const Value V17248_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17248_321 = {S40, 0, &V17248_321_57293, &P17248_320};	/*Property = Progress*/
const Value V17248_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17248_322 = {S41, 0, &V17248_322_57296, &P17248_321};	/*Property = Prompts*/
const Value V17248_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17248_323 = {S42, 0, &V17248_323_57299, &P17248_322};	/*Property = Selector*/
const Value V17248_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17248_324 = {S43, 0, &V17248_324_57306, &P17248_323};	/*Property = Svg*/
const Value V17248_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17248_327 = {S44, 0, &V17248_327_57315, &P17248_324};	/*Property = TouchEvents*/
const Value V17248_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17248_328 = {S45, 0, &V17248_328_57318, &P17248_327};	/*Property = Track*/
const Value V17248_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17248_329 = {S46, 0, &V17248_329_57323, &P17248_328};	/*Property = Video*/
const Value V17248_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17248_330 = {S47, 0, &V17248_330_57326, &P17248_329};	/*Property = Viewport*/
const Value V17248_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17248_332 = {S48, 0, &V17248_332_57339, &P17248_330};	/*Property = WebWorkers*/
const Value V17248_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17248_333 = {S49, 0, &V17248_333_57342, &P17248_332};	/*Property = Xhr2*/
const Profile P17248 = {"17248", &P17248_333};	/*Profile ID - 17248*/
const Value V13496_1_1 = {S2, NULL};	/*Value = True*/
const Property P13496_1 = {S50, 0, &V13496_1_1, NULL};	/*Property = IsMobile*/
const Value V13496_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13496_5 = {S52, 0, &V13496_5_1380, &P13496_1};	/*Property = ScreenPixelsHeight*/
const Value V13496_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13496_4 = {S54, 0, &V13496_4_1279, &P13496_5};	/*Property = ScreenPixelsWidth*/
const Profile P13496 = {"13496", &P13496_4};	/*Profile ID - 13496*/
const Value V12528_1_1 = {S2, NULL};	/*Value = True*/
const Property P12528_1 = {S50, 0, &V12528_1_1, NULL};	/*Property = IsMobile*/
const Value V12528_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12528_5 = {S52, 0, &V12528_5_1222, &P12528_1};	/*Property = ScreenPixelsHeight*/
const Value V12528_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12528_4 = {S54, 0, &V12528_4_1224, &P12528_5};	/*Property = ScreenPixelsWidth*/
const Profile P12528 = {"12528", &P12528_4};	/*Profile ID - 12528*/
const Value V14332_1_1 = {S2, NULL};	/*Value = True*/
const Property P14332_1 = {S50, 0, &V14332_1_1, NULL};	/*Property = IsMobile*/
const Value V14332_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14332_5 = {S52, 0, &V14332_5_1233, &P14332_1};	/*Property = ScreenPixelsHeight*/
const Value V14332_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14332_4 = {S54, 0, &V14332_4_1235, &P14332_5};	/*Property = ScreenPixelsWidth*/
const Profile P14332 = {"14332", &P14332_4};	/*Profile ID - 14332*/
const Value V13511_1_1 = {S2, NULL};	/*Value = True*/
const Property P13511_1 = {S50, 0, &V13511_1_1, NULL};	/*Property = IsMobile*/
const Value V13511_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13511_5 = {S52, 0, &V13511_5_1380, &P13511_1};	/*Property = ScreenPixelsHeight*/
const Value V13511_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13511_4 = {S54, 0, &V13511_4_1279, &P13511_5};	/*Property = ScreenPixelsWidth*/
const Profile P13511 = {"13511", &P13511_4};	/*Profile ID - 13511*/
const Value V13517_1_1 = {S2, NULL};	/*Value = True*/
const Property P13517_1 = {S50, 0, &V13517_1_1, NULL};	/*Property = IsMobile*/
const Value V13517_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13517_5 = {S52, 0, &V13517_5_1277, &P13517_1};	/*Property = ScreenPixelsHeight*/
const Value V13517_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13517_4 = {S54, 0, &V13517_4_1279, &P13517_5};	/*Property = ScreenPixelsWidth*/
const Profile P13517 = {"13517", &P13517_4};	/*Profile ID - 13517*/
const Value V13526_1_1 = {S2, NULL};	/*Value = True*/
const Property P13526_1 = {S50, 0, &V13526_1_1, NULL};	/*Property = IsMobile*/
const Value V13526_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13526_5 = {S52, 0, &V13526_5_1227, &P13526_1};	/*Property = ScreenPixelsHeight*/
const Value V13526_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13526_4 = {S54, 0, &V13526_4_1229, &P13526_5};	/*Property = ScreenPixelsWidth*/
const Profile P13526 = {"13526", &P13526_4};	/*Profile ID - 13526*/
const Value V15539_1_1 = {S2, NULL};	/*Value = True*/
const Property P15539_1 = {S50, 0, &V15539_1_1, NULL};	/*Property = IsMobile*/
const Value V15539_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15539_5 = {S52, 0, &V15539_5_1227, &P15539_1};	/*Property = ScreenPixelsHeight*/
const Value V15539_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15539_4 = {S54, 0, &V15539_4_1229, &P15539_5};	/*Property = ScreenPixelsWidth*/
const Profile P15539 = {"15539", &P15539_4};	/*Profile ID - 15539*/
const Value V13531_1_1 = {S2, NULL};	/*Value = True*/
const Property P13531_1 = {S50, 0, &V13531_1_1, NULL};	/*Property = IsMobile*/
const Value V13531_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13531_5 = {S52, 0, &V13531_5_1222, &P13531_1};	/*Property = ScreenPixelsHeight*/
const Value V13531_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13531_4 = {S54, 0, &V13531_4_1224, &P13531_5};	/*Property = ScreenPixelsWidth*/
const Profile P13531 = {"13531", &P13531_4};	/*Profile ID - 13531*/
const Value V13560_1_1 = {S2, NULL};	/*Value = True*/
const Property P13560_1 = {S50, 0, &V13560_1_1, NULL};	/*Property = IsMobile*/
const Value V13560_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13560_5 = {S52, 0, &V13560_5_1222, &P13560_1};	/*Property = ScreenPixelsHeight*/
const Value V13560_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13560_4 = {S54, 0, &V13560_4_1224, &P13560_5};	/*Property = ScreenPixelsWidth*/
const Profile P13560 = {"13560", &P13560_4};	/*Profile ID - 13560*/
const Value V14512_1_1 = {S2, NULL};	/*Value = True*/
const Property P14512_1 = {S50, 0, &V14512_1_1, NULL};	/*Property = IsMobile*/
const Value V14512_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14512_5 = {S52, 0, &V14512_5_1233, &P14512_1};	/*Property = ScreenPixelsHeight*/
const Value V14512_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14512_4 = {S54, 0, &V14512_4_1235, &P14512_5};	/*Property = ScreenPixelsWidth*/
const Profile P14512 = {"14512", &P14512_4};	/*Profile ID - 14512*/
const Value V13543_1_1 = {S2, NULL};	/*Value = True*/
const Property P13543_1 = {S50, 0, &V13543_1_1, NULL};	/*Property = IsMobile*/
const Value V13543_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13543_5 = {S52, 0, &V13543_5_1222, &P13543_1};	/*Property = ScreenPixelsHeight*/
const Value V13543_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13543_4 = {S54, 0, &V13543_4_1224, &P13543_5};	/*Property = ScreenPixelsWidth*/
const Profile P13543 = {"13543", &P13543_4};	/*Profile ID - 13543*/
const Value V13545_1_1 = {S2, NULL};	/*Value = True*/
const Property P13545_1 = {S50, 0, &V13545_1_1, NULL};	/*Property = IsMobile*/
const Value V13545_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13545_5 = {S52, 0, &V13545_5_1222, &P13545_1};	/*Property = ScreenPixelsHeight*/
const Value V13545_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13545_4 = {S54, 0, &V13545_4_1224, &P13545_5};	/*Property = ScreenPixelsWidth*/
const Profile P13545 = {"13545", &P13545_4};	/*Profile ID - 13545*/
const Value V14983_1_1 = {S2, NULL};	/*Value = True*/
const Property P14983_1 = {S50, 0, &V14983_1_1, NULL};	/*Property = IsMobile*/
const Value V14983_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P14983_5 = {S52, 0, &V14983_5_1275, &P14983_1};	/*Property = ScreenPixelsHeight*/
const Value V14983_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P14983_4 = {S54, 0, &V14983_4_1278, &P14983_5};	/*Property = ScreenPixelsWidth*/
const Profile P14983 = {"14983", &P14983_4};	/*Profile ID - 14983*/
const Value V13548_1_1 = {S2, NULL};	/*Value = True*/
const Property P13548_1 = {S50, 0, &V13548_1_1, NULL};	/*Property = IsMobile*/
const Value V13548_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13548_5 = {S52, 0, &V13548_5_1222, &P13548_1};	/*Property = ScreenPixelsHeight*/
const Value V13548_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13548_4 = {S54, 0, &V13548_4_1224, &P13548_5};	/*Property = ScreenPixelsWidth*/
const Profile P13548 = {"13548", &P13548_4};	/*Profile ID - 13548*/
const Value V13550_1_1 = {S2, NULL};	/*Value = True*/
const Property P13550_1 = {S50, 0, &V13550_1_1, NULL};	/*Property = IsMobile*/
const Value V13550_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13550_5 = {S52, 0, &V13550_5_1233, &P13550_1};	/*Property = ScreenPixelsHeight*/
const Value V13550_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13550_4 = {S54, 0, &V13550_4_1235, &P13550_5};	/*Property = ScreenPixelsWidth*/
const Profile P13550 = {"13550", &P13550_4};	/*Profile ID - 13550*/
const Value V14467_1_1 = {S2, NULL};	/*Value = True*/
const Property P14467_1 = {S50, 0, &V14467_1_1, NULL};	/*Property = IsMobile*/
const Value V14467_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14467_5 = {S52, 0, &V14467_5_1277, &P14467_1};	/*Property = ScreenPixelsHeight*/
const Value V14467_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14467_4 = {S54, 0, &V14467_4_1279, &P14467_5};	/*Property = ScreenPixelsWidth*/
const Profile P14467 = {"14467", &P14467_4};	/*Profile ID - 14467*/
const Value V12209_1_1 = {S2, NULL};	/*Value = True*/
const Property P12209_1 = {S50, 0, &V12209_1_1, NULL};	/*Property = IsMobile*/
const Value V12209_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12209_5 = {S52, 0, &V12209_5_1222, &P12209_1};	/*Property = ScreenPixelsHeight*/
const Value V12209_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12209_4 = {S54, 0, &V12209_4_1224, &P12209_5};	/*Property = ScreenPixelsWidth*/
const Profile P12209 = {"12209", &P12209_4};	/*Profile ID - 12209*/
const Value V13713_1_1 = {S2, NULL};	/*Value = True*/
const Property P13713_1 = {S50, 0, &V13713_1_1, NULL};	/*Property = IsMobile*/
const Value V13713_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13713_5 = {S52, 0, &V13713_5_1222, &P13713_1};	/*Property = ScreenPixelsHeight*/
const Value V13713_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13713_4 = {S54, 0, &V13713_4_1224, &P13713_5};	/*Property = ScreenPixelsWidth*/
const Profile P13713 = {"13713", &P13713_4};	/*Profile ID - 13713*/
const Value V13296_1_1 = {S2, NULL};	/*Value = True*/
const Property P13296_1 = {S50, 0, &V13296_1_1, NULL};	/*Property = IsMobile*/
const Value V13296_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13296_5 = {S52, 0, &V13296_5_1227, &P13296_1};	/*Property = ScreenPixelsHeight*/
const Value V13296_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13296_4 = {S54, 0, &V13296_4_1229, &P13296_5};	/*Property = ScreenPixelsWidth*/
const Profile P13296 = {"13296", &P13296_4};	/*Profile ID - 13296*/
const Value V17945_1_1 = {S2, NULL};	/*Value = True*/
const Property P17945_1 = {S50, 0, &V17945_1_1, NULL};	/*Property = IsMobile*/
const Value V17945_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17945_5 = {S52, 0, &V17945_5_1227, &P17945_1};	/*Property = ScreenPixelsHeight*/
const Value V17945_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17945_4 = {S54, 0, &V17945_4_1229, &P17945_5};	/*Property = ScreenPixelsWidth*/
const Profile P17945 = {"17945", &P17945_4};	/*Profile ID - 17945*/
const Value V14984_1_1 = {S2, NULL};	/*Value = True*/
const Property P14984_1 = {S50, 0, &V14984_1_1, NULL};	/*Property = IsMobile*/
const Value V14984_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P14984_5 = {S52, 0, &V14984_5_1275, &P14984_1};	/*Property = ScreenPixelsHeight*/
const Value V14984_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P14984_4 = {S54, 0, &V14984_4_1278, &P14984_5};	/*Property = ScreenPixelsWidth*/
const Profile P14984 = {"14984", &P14984_4};	/*Profile ID - 14984*/
const Value V13563_1_1 = {S2, NULL};	/*Value = True*/
const Property P13563_1 = {S50, 0, &V13563_1_1, NULL};	/*Property = IsMobile*/
const Value V13563_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13563_5 = {S52, 0, &V13563_5_1233, &P13563_1};	/*Property = ScreenPixelsHeight*/
const Value V13563_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13563_4 = {S54, 0, &V13563_4_1235, &P13563_5};	/*Property = ScreenPixelsWidth*/
const Profile P13563 = {"13563", &P13563_4};	/*Profile ID - 13563*/
const Value V13540_1_1 = {S2, NULL};	/*Value = True*/
const Property P13540_1 = {S50, 0, &V13540_1_1, NULL};	/*Property = IsMobile*/
const Value V13540_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13540_5 = {S52, 0, &V13540_5_1293, &P13540_1};	/*Property = ScreenPixelsHeight*/
const Value V13540_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13540_4 = {S54, 0, &V13540_4_1296, &P13540_5};	/*Property = ScreenPixelsWidth*/
const Profile P13540 = {"13540", &P13540_4};	/*Profile ID - 13540*/
const Value V11792_1_1 = {S2, NULL};	/*Value = True*/
const Property P11792_1 = {S50, 0, &V11792_1_1, NULL};	/*Property = IsMobile*/
const Value V11792_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P11792_5 = {S52, 0, &V11792_5_1294, &P11792_1};	/*Property = ScreenPixelsHeight*/
const Value V11792_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11792_4 = {S54, 0, &V11792_4_1296, &P11792_5};	/*Property = ScreenPixelsWidth*/
const Profile P11792 = {"11792", &P11792_4};	/*Profile ID - 11792*/
const Value V12159_1_1 = {S2, NULL};	/*Value = True*/
const Property P12159_1 = {S50, 0, &V12159_1_1, NULL};	/*Property = IsMobile*/
const Value V12159_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12159_5 = {S52, 0, &V12159_5_1222, &P12159_1};	/*Property = ScreenPixelsHeight*/
const Value V12159_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12159_4 = {S54, 0, &V12159_4_1224, &P12159_5};	/*Property = ScreenPixelsWidth*/
const Profile P12159 = {"12159", &P12159_4};	/*Profile ID - 12159*/
const Value V14587_1_1 = {S2, NULL};	/*Value = True*/
const Property P14587_1 = {S50, 0, &V14587_1_1, NULL};	/*Property = IsMobile*/
const Value V14587_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14587_5 = {S52, 0, &V14587_5_1294, &P14587_1};	/*Property = ScreenPixelsHeight*/
const Value V14587_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14587_4 = {S54, 0, &V14587_4_1296, &P14587_5};	/*Property = ScreenPixelsWidth*/
const Profile P14587 = {"14587", &P14587_4};	/*Profile ID - 14587*/
const Value V13578_1_1 = {S2, NULL};	/*Value = True*/
const Property P13578_1 = {S50, 0, &V13578_1_1, NULL};	/*Property = IsMobile*/
const Value V13578_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13578_5 = {S52, 0, &V13578_5_1222, &P13578_1};	/*Property = ScreenPixelsHeight*/
const Value V13578_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13578_4 = {S54, 0, &V13578_4_1224, &P13578_5};	/*Property = ScreenPixelsWidth*/
const Profile P13578 = {"13578", &P13578_4};	/*Profile ID - 13578*/
const Value V12970_1_1 = {S2, NULL};	/*Value = True*/
const Property P12970_1 = {S50, 0, &V12970_1_1, NULL};	/*Property = IsMobile*/
const Value V12970_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12970_5 = {S52, 0, &V12970_5_1227, &P12970_1};	/*Property = ScreenPixelsHeight*/
const Value V12970_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12970_4 = {S54, 0, &V12970_4_1229, &P12970_5};	/*Property = ScreenPixelsWidth*/
const Profile P12970 = {"12970", &P12970_4};	/*Profile ID - 12970*/
const Value V12511_1_1 = {S2, NULL};	/*Value = True*/
const Property P12511_1 = {S50, 0, &V12511_1_1, NULL};	/*Property = IsMobile*/
const Value V12511_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12511_5 = {S52, 0, &V12511_5_1222, &P12511_1};	/*Property = ScreenPixelsHeight*/
const Value V12511_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12511_4 = {S54, 0, &V12511_4_1224, &P12511_5};	/*Property = ScreenPixelsWidth*/
const Profile P12511 = {"12511", &P12511_4};	/*Profile ID - 12511*/
const Value V14553_1_1 = {S2, NULL};	/*Value = True*/
const Property P14553_1 = {S50, 0, &V14553_1_1, NULL};	/*Property = IsMobile*/
const Value V14553_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14553_5 = {S52, 0, &V14553_5_1233, &P14553_1};	/*Property = ScreenPixelsHeight*/
const Value V14553_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14553_4 = {S54, 0, &V14553_4_1235, &P14553_5};	/*Property = ScreenPixelsWidth*/
const Profile P14553 = {"14553", &P14553_4};	/*Profile ID - 14553*/
const Value V15234_1_1 = {S2, NULL};	/*Value = True*/
const Property P15234_1 = {S50, 0, &V15234_1_1, NULL};	/*Property = IsMobile*/
const Value V15234_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15234_5 = {S52, 0, &V15234_5_1380, &P15234_1};	/*Property = ScreenPixelsHeight*/
const Value V15234_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15234_4 = {S54, 0, &V15234_4_1279, &P15234_5};	/*Property = ScreenPixelsWidth*/
const Profile P15234 = {"15234", &P15234_4};	/*Profile ID - 15234*/
const Value V16331_1_1 = {S2, NULL};	/*Value = True*/
const Property P16331_1 = {S50, 0, &V16331_1_1, NULL};	/*Property = IsMobile*/
const Value V16331_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16331_5 = {S52, 0, &V16331_5_1277, &P16331_1};	/*Property = ScreenPixelsHeight*/
const Value V16331_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16331_4 = {S54, 0, &V16331_4_1279, &P16331_5};	/*Property = ScreenPixelsWidth*/
const Profile P16331 = {"16331", &P16331_4};	/*Profile ID - 16331*/
const Value V13587_1_1 = {S2, NULL};	/*Value = True*/
const Property P13587_1 = {S50, 0, &V13587_1_1, NULL};	/*Property = IsMobile*/
const Value V13587_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13587_5 = {S52, 0, &V13587_5_1233, &P13587_1};	/*Property = ScreenPixelsHeight*/
const Value V13587_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13587_4 = {S54, 0, &V13587_4_1235, &P13587_5};	/*Property = ScreenPixelsWidth*/
const Profile P13587 = {"13587", &P13587_4};	/*Profile ID - 13587*/
const Value V13721_1_1 = {S2, NULL};	/*Value = True*/
const Property P13721_1 = {S50, 0, &V13721_1_1, NULL};	/*Property = IsMobile*/
const Value V13721_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13721_5 = {S52, 0, &V13721_5_1222, &P13721_1};	/*Property = ScreenPixelsHeight*/
const Value V13721_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13721_4 = {S54, 0, &V13721_4_1224, &P13721_5};	/*Property = ScreenPixelsWidth*/
const Profile P13721 = {"13721", &P13721_4};	/*Profile ID - 13721*/
const Value V21193_1_1 = {S2, NULL};	/*Value = True*/
const Property P21193_1 = {S50, 0, &V21193_1_1, NULL};	/*Property = IsMobile*/
const Value V21193_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21193_5 = {S52, 0, &V21193_5_1227, &P21193_1};	/*Property = ScreenPixelsHeight*/
const Value V21193_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21193_4 = {S54, 0, &V21193_4_1229, &P21193_5};	/*Property = ScreenPixelsWidth*/
const Profile P21193 = {"21193", &P21193_4};	/*Profile ID - 21193*/
const Value V13585_1_1 = {S2, NULL};	/*Value = True*/
const Property P13585_1 = {S50, 0, &V13585_1_1, NULL};	/*Property = IsMobile*/
const Value V13585_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13585_5 = {S52, 0, &V13585_5_1222, &P13585_1};	/*Property = ScreenPixelsHeight*/
const Value V13585_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13585_4 = {S54, 0, &V13585_4_1224, &P13585_5};	/*Property = ScreenPixelsWidth*/
const Profile P13585 = {"13585", &P13585_4};	/*Profile ID - 13585*/
const Value V11962_1_1 = {S2, NULL};	/*Value = True*/
const Property P11962_1 = {S50, 0, &V11962_1_1, NULL};	/*Property = IsMobile*/
const Value V11962_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11962_5 = {S52, 0, &V11962_5_1222, &P11962_1};	/*Property = ScreenPixelsHeight*/
const Value V11962_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11962_4 = {S54, 0, &V11962_4_1224, &P11962_5};	/*Property = ScreenPixelsWidth*/
const Profile P11962 = {"11962", &P11962_4};	/*Profile ID - 11962*/
const Value V4112_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_279 = {S1, 0, &V4112_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4112_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_282 = {S3, 0, &V4112_282_57155, &P4112_279};	/*Property = BlobBuilder*/
const Value V4112_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_350 = {S4, 0, &V4112_350_57446, &P4112_282};	/*Property = Canvas*/
const Value V4112_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_283 = {S5, 0, &V4112_283_57161, &P4112_350};	/*Property = CssBackground*/
const Value V4112_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_284 = {S6, 0, &V4112_284_57164, &P4112_283};	/*Property = CssBorderImage*/
const Value V4112_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_285 = {S7, 0, &V4112_285_57167, &P4112_284};	/*Property = CssCanvas*/
const Value V4112_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_286 = {S8, 0, &V4112_286_57170, &P4112_285};	/*Property = CssColor*/
const Value V4112_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_237 = {S9, 0, &V4112_237_55786, &P4112_286};	/*Property = CssColumn*/
const Value V4112_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_287 = {S10, 0, &V4112_287_57173, &P4112_237};	/*Property = CssFlexbox*/
const Value V4112_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_288 = {S11, 0, &V4112_288_57176, &P4112_287};	/*Property = CssFont*/
const Value V4112_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_289 = {S12, 0, &V4112_289_86899, &P4112_288};	/*Property = CssImages*/
const Value V4112_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_291 = {S13, 0, &V4112_291_57182, &P4112_289};	/*Property = CssMediaQueries*/
const Value V4112_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_292 = {S14, 0, &V4112_292_57185, &P4112_291};	/*Property = CssMinMax*/
const Value V4112_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_293 = {S15, 0, &V4112_293_57188, &P4112_292};	/*Property = CssOverflow*/
const Value V4112_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_294 = {S16, 0, &V4112_294_57191, &P4112_293};	/*Property = CssPosition*/
const Value V4112_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_295 = {S17, 0, &V4112_295_57194, &P4112_294};	/*Property = CssText*/
const Value V4112_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_297 = {S18, 0, &V4112_297_57197, &P4112_295};	/*Property = CssTransforms*/
const Value V4112_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_241 = {S19, 0, &V4112_241_55801, &P4112_297};	/*Property = CssTransitions*/
const Value V4112_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_298 = {S20, 0, &V4112_298_57208, &P4112_241};	/*Property = CssUI*/
const Value V4112_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_300 = {S21, 0, &V4112_300_57217, &P4112_298};	/*Property = DataSet*/
const Value V4112_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_301 = {S22, 0, &V4112_301_57220, &P4112_300};	/*Property = DataUrl*/
const Value V4112_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_302 = {S23, 0, &V4112_302_57223, &P4112_301};	/*Property = DeviceOrientation*/
const Value V4112_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_304 = {S24, 0, &V4112_304_57229, &P4112_302};	/*Property = FileReader*/
const Value V4112_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_305 = {S25, 0, &V4112_305_57232, &P4112_304};	/*Property = FileSaver*/
const Value V4112_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_306 = {S26, 0, &V4112_306_57238, &P4112_305};	/*Property = FileWriter*/
const Value V4112_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_307 = {S27, 0, &V4112_307_57241, &P4112_306};	/*Property = FormData*/
const Value V4112_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_309 = {S28, 0, &V4112_309_57247, &P4112_307};	/*Property = Fullscreen*/
const Value V4112_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_310 = {S29, 0, &V4112_310_57250, &P4112_309};	/*Property = GeoLocation*/
const Value V4112_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_312 = {S30, 0, &V4112_312_57256, &P4112_310};	/*Property = History*/
const Value V4112_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_314 = {S31, 0, &V4112_314_57262, &P4112_312};	/*Property = Html5*/
const Value V4112_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_313 = {S32, 0, &V4112_313_57259, &P4112_314};	/*Property = Html-Media-Capture*/
const Value V4112_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_315 = {S33, 0, &V4112_315_57265, &P4112_313};	/*Property = Iframe*/
const Value V4112_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_316 = {S34, 0, &V4112_316_57268, &P4112_315};	/*Property = IndexedDB*/
const Value V4112_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_317 = {S35, 0, &V4112_317_57271, &P4112_316};	/*Property = Json*/
const Value V4112_15_44266 = {S72, NULL};	/*Value = MotoWebkit*/
const Property P4112_15 = {S37, 0, &V4112_15_44266, &P4112_317};	/*Property = LayoutEngine*/
const Value V4112_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_344 = {S38, 0, &V4112_344_57274, &P4112_15};	/*Property = Masking*/
const Value V4112_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_320 = {S39, 0, &V4112_320_57292, &P4112_344};	/*Property = PostMessage*/
const Value V4112_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_321 = {S40, 0, &V4112_321_57295, &P4112_320};	/*Property = Progress*/
const Value V4112_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_322 = {S41, 0, &V4112_322_57298, &P4112_321};	/*Property = Prompts*/
const Value V4112_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_323 = {S42, 0, &V4112_323_57301, &P4112_322};	/*Property = Selector*/
const Value V4112_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_324 = {S43, 0, &V4112_324_57307, &P4112_323};	/*Property = Svg*/
const Value V4112_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_327 = {S44, 0, &V4112_327_57316, &P4112_324};	/*Property = TouchEvents*/
const Value V4112_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_328 = {S45, 0, &V4112_328_57319, &P4112_327};	/*Property = Track*/
const Value V4112_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_329 = {S46, 0, &V4112_329_57325, &P4112_328};	/*Property = Video*/
const Value V4112_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_330 = {S47, 0, &V4112_330_57328, &P4112_329};	/*Property = Viewport*/
const Value V4112_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_332 = {S48, 0, &V4112_332_57340, &P4112_330};	/*Property = WebWorkers*/
const Value V4112_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4112_333 = {S49, 0, &V4112_333_57343, &P4112_332};	/*Property = Xhr2*/
const Profile P4112 = {"4112", &P4112_333};	/*Profile ID - 4112*/
const Value V12067_1_1 = {S2, NULL};	/*Value = True*/
const Property P12067_1 = {S50, 0, &V12067_1_1, NULL};	/*Property = IsMobile*/
const Value V12067_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12067_5 = {S52, 0, &V12067_5_1222, &P12067_1};	/*Property = ScreenPixelsHeight*/
const Value V12067_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12067_4 = {S54, 0, &V12067_4_1224, &P12067_5};	/*Property = ScreenPixelsWidth*/
const Profile P12067 = {"12067", &P12067_4};	/*Profile ID - 12067*/
const Value V17283_1_1 = {S2, NULL};	/*Value = True*/
const Property P17283_1 = {S50, 0, &V17283_1_1, NULL};	/*Property = IsMobile*/
const Value V17283_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P17283_5 = {S52, 0, &V17283_5_1403, &P17283_1};	/*Property = ScreenPixelsHeight*/
const Value V17283_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P17283_4 = {S54, 0, &V17283_4_1404, &P17283_5};	/*Property = ScreenPixelsWidth*/
const Profile P17283 = {"17283", &P17283_4};	/*Profile ID - 17283*/
const Value V13600_1_1 = {S2, NULL};	/*Value = True*/
const Property P13600_1 = {S50, 0, &V13600_1_1, NULL};	/*Property = IsMobile*/
const Value V13600_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13600_5 = {S52, 0, &V13600_5_1222, &P13600_1};	/*Property = ScreenPixelsHeight*/
const Value V13600_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13600_4 = {S54, 0, &V13600_4_1224, &P13600_5};	/*Property = ScreenPixelsWidth*/
const Profile P13600 = {"13600", &P13600_4};	/*Profile ID - 13600*/
const Value V11819_1_1 = {S2, NULL};	/*Value = True*/
const Property P11819_1 = {S50, 0, &V11819_1_1, NULL};	/*Property = IsMobile*/
const Value V11819_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P11819_5 = {S52, 0, &V11819_5_1294, &P11819_1};	/*Property = ScreenPixelsHeight*/
const Value V11819_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11819_4 = {S54, 0, &V11819_4_1296, &P11819_5};	/*Property = ScreenPixelsWidth*/
const Profile P11819 = {"11819", &P11819_4};	/*Profile ID - 11819*/
const Value V13603_1_1 = {S2, NULL};	/*Value = True*/
const Property P13603_1 = {S50, 0, &V13603_1_1, NULL};	/*Property = IsMobile*/
const Value V13603_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13603_5 = {S52, 0, &V13603_5_1222, &P13603_1};	/*Property = ScreenPixelsHeight*/
const Value V13603_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13603_4 = {S54, 0, &V13603_4_1224, &P13603_5};	/*Property = ScreenPixelsWidth*/
const Profile P13603 = {"13603", &P13603_4};	/*Profile ID - 13603*/
const Value V15293_1_1 = {S2, NULL};	/*Value = True*/
const Property P15293_1 = {S50, 0, &V15293_1_1, NULL};	/*Property = IsMobile*/
const Value V15293_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15293_5 = {S52, 0, &V15293_5_1227, &P15293_1};	/*Property = ScreenPixelsHeight*/
const Value V15293_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15293_4 = {S54, 0, &V15293_4_1229, &P15293_5};	/*Property = ScreenPixelsWidth*/
const Profile P15293 = {"15293", &P15293_4};	/*Profile ID - 15293*/
const Value V17953_1_1 = {S2, NULL};	/*Value = True*/
const Property P17953_1 = {S50, 0, &V17953_1_1, NULL};	/*Property = IsMobile*/
const Value V17953_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17953_5 = {S52, 0, &V17953_5_1277, &P17953_1};	/*Property = ScreenPixelsHeight*/
const Value V17953_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17953_4 = {S54, 0, &V17953_4_1279, &P17953_5};	/*Property = ScreenPixelsWidth*/
const Profile P17953 = {"17953", &P17953_4};	/*Profile ID - 17953*/
const Value V13607_1_1 = {S2, NULL};	/*Value = True*/
const Property P13607_1 = {S50, 0, &V13607_1_1, NULL};	/*Property = IsMobile*/
const Value V13607_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13607_5 = {S52, 0, &V13607_5_1233, &P13607_1};	/*Property = ScreenPixelsHeight*/
const Value V13607_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13607_4 = {S54, 0, &V13607_4_1235, &P13607_5};	/*Property = ScreenPixelsWidth*/
const Profile P13607 = {"13607", &P13607_4};	/*Profile ID - 13607*/
const Value V12493_1_1 = {S2, NULL};	/*Value = True*/
const Property P12493_1 = {S50, 0, &V12493_1_1, NULL};	/*Property = IsMobile*/
const Value V12493_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12493_5 = {S52, 0, &V12493_5_1277, &P12493_1};	/*Property = ScreenPixelsHeight*/
const Value V12493_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12493_4 = {S54, 0, &V12493_4_1279, &P12493_5};	/*Property = ScreenPixelsWidth*/
const Profile P12493 = {"12493", &P12493_4};	/*Profile ID - 12493*/
const Value V13608_1_1 = {S2, NULL};	/*Value = True*/
const Property P13608_1 = {S50, 0, &V13608_1_1, NULL};	/*Property = IsMobile*/
const Value V13608_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13608_5 = {S52, 0, &V13608_5_1233, &P13608_1};	/*Property = ScreenPixelsHeight*/
const Value V13608_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13608_4 = {S54, 0, &V13608_4_1235, &P13608_5};	/*Property = ScreenPixelsWidth*/
const Profile P13608 = {"13608", &P13608_4};	/*Profile ID - 13608*/
const Value V14967_1_1 = {S2, NULL};	/*Value = True*/
const Property P14967_1 = {S50, 0, &V14967_1_1, NULL};	/*Property = IsMobile*/
const Value V14967_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14967_5 = {S52, 0, &V14967_5_1277, &P14967_1};	/*Property = ScreenPixelsHeight*/
const Value V14967_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14967_4 = {S54, 0, &V14967_4_1279, &P14967_5};	/*Property = ScreenPixelsWidth*/
const Profile P14967 = {"14967", &P14967_4};	/*Profile ID - 14967*/
const Value V14085_1_1 = {S2, NULL};	/*Value = True*/
const Property P14085_1 = {S50, 0, &V14085_1_1, NULL};	/*Property = IsMobile*/
const Value V14085_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14085_5 = {S52, 0, &V14085_5_1277, &P14085_1};	/*Property = ScreenPixelsHeight*/
const Value V14085_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14085_4 = {S54, 0, &V14085_4_1279, &P14085_5};	/*Property = ScreenPixelsWidth*/
const Profile P14085 = {"14085", &P14085_4};	/*Profile ID - 14085*/
const Value V13613_1_1 = {S2, NULL};	/*Value = True*/
const Property P13613_1 = {S50, 0, &V13613_1_1, NULL};	/*Property = IsMobile*/
const Value V13613_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13613_5 = {S52, 0, &V13613_5_1222, &P13613_1};	/*Property = ScreenPixelsHeight*/
const Value V13613_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13613_4 = {S54, 0, &V13613_4_1224, &P13613_5};	/*Property = ScreenPixelsWidth*/
const Profile P13613 = {"13613", &P13613_4};	/*Profile ID - 13613*/
const Value V13623_1_1 = {S2, NULL};	/*Value = True*/
const Property P13623_1 = {S50, 0, &V13623_1_1, NULL};	/*Property = IsMobile*/
const Value V13623_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13623_5 = {S52, 0, &V13623_5_1380, &P13623_1};	/*Property = ScreenPixelsHeight*/
const Value V13623_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13623_4 = {S54, 0, &V13623_4_1279, &P13623_5};	/*Property = ScreenPixelsWidth*/
const Profile P13623 = {"13623", &P13623_4};	/*Profile ID - 13623*/
const Value V13629_1_1 = {S2, NULL};	/*Value = True*/
const Property P13629_1 = {S50, 0, &V13629_1_1, NULL};	/*Property = IsMobile*/
const Value V13629_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13629_5 = {S52, 0, &V13629_5_1293, &P13629_1};	/*Property = ScreenPixelsHeight*/
const Value V13629_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13629_4 = {S54, 0, &V13629_4_1296, &P13629_5};	/*Property = ScreenPixelsWidth*/
const Profile P13629 = {"13629", &P13629_4};	/*Profile ID - 13629*/
const Value V13633_1_1 = {S2, NULL};	/*Value = True*/
const Property P13633_1 = {S50, 0, &V13633_1_1, NULL};	/*Property = IsMobile*/
const Value V13633_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P13633_5 = {S52, 0, &V13633_5_1269, &P13633_1};	/*Property = ScreenPixelsHeight*/
const Value V13633_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13633_4 = {S54, 0, &V13633_4_1229, &P13633_5};	/*Property = ScreenPixelsWidth*/
const Profile P13633 = {"13633", &P13633_4};	/*Profile ID - 13633*/
const Value V21270_1_1 = {S2, NULL};	/*Value = True*/
const Property P21270_1 = {S50, 0, &V21270_1_1, NULL};	/*Property = IsMobile*/
const Value V21270_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21270_5 = {S52, 0, &V21270_5_1269, &P21270_1};	/*Property = ScreenPixelsHeight*/
const Value V21270_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21270_4 = {S54, 0, &V21270_4_1229, &P21270_5};	/*Property = ScreenPixelsWidth*/
const Profile P21270 = {"21270", &P21270_4};	/*Profile ID - 21270*/
const Value V14949_1_1 = {S2, NULL};	/*Value = True*/
const Property P14949_1 = {S50, 0, &V14949_1_1, NULL};	/*Property = IsMobile*/
const Value V14949_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14949_5 = {S52, 0, &V14949_5_1222, &P14949_1};	/*Property = ScreenPixelsHeight*/
const Value V14949_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14949_4 = {S54, 0, &V14949_4_1224, &P14949_5};	/*Property = ScreenPixelsWidth*/
const Profile P14949 = {"14949", &P14949_4};	/*Profile ID - 14949*/
const Value V13647_1_1 = {S2, NULL};	/*Value = True*/
const Property P13647_1 = {S50, 0, &V13647_1_1, NULL};	/*Property = IsMobile*/
const Value V13647_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13647_5 = {S52, 0, &V13647_5_1233, &P13647_1};	/*Property = ScreenPixelsHeight*/
const Value V13647_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13647_4 = {S54, 0, &V13647_4_1235, &P13647_5};	/*Property = ScreenPixelsWidth*/
const Profile P13647 = {"13647", &P13647_4};	/*Profile ID - 13647*/
const Value V13648_1_1 = {S2, NULL};	/*Value = True*/
const Property P13648_1 = {S50, 0, &V13648_1_1, NULL};	/*Property = IsMobile*/
const Value V13648_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P13648_5 = {S52, 0, &V13648_5_1275, &P13648_1};	/*Property = ScreenPixelsHeight*/
const Value V13648_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P13648_4 = {S54, 0, &V13648_4_1278, &P13648_5};	/*Property = ScreenPixelsWidth*/
const Profile P13648 = {"13648", &P13648_4};	/*Profile ID - 13648*/
const Value V11727_1_1 = {S2, NULL};	/*Value = True*/
const Property P11727_1 = {S50, 0, &V11727_1_1, NULL};	/*Property = IsMobile*/
const Value V11727_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11727_5 = {S52, 0, &V11727_5_1222, &P11727_1};	/*Property = ScreenPixelsHeight*/
const Value V11727_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11727_4 = {S54, 0, &V11727_4_1224, &P11727_5};	/*Property = ScreenPixelsWidth*/
const Profile P11727 = {"11727", &P11727_4};	/*Profile ID - 11727*/
const Value V13659_1_1 = {S2, NULL};	/*Value = True*/
const Property P13659_1 = {S50, 0, &V13659_1_1, NULL};	/*Property = IsMobile*/
const Value V13659_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13659_5 = {S52, 0, &V13659_5_1277, &P13659_1};	/*Property = ScreenPixelsHeight*/
const Value V13659_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13659_4 = {S54, 0, &V13659_4_1279, &P13659_5};	/*Property = ScreenPixelsWidth*/
const Profile P13659 = {"13659", &P13659_4};	/*Profile ID - 13659*/
const Value V15745_1_1 = {S2, NULL};	/*Value = True*/
const Property P15745_1 = {S50, 0, &V15745_1_1, NULL};	/*Property = IsMobile*/
const Value V15745_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15745_5 = {S52, 0, &V15745_5_1304, &P15745_1};	/*Property = ScreenPixelsHeight*/
const Value V15745_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P15745_4 = {S54, 0, &V15745_4_1278, &P15745_5};	/*Property = ScreenPixelsWidth*/
const Profile P15745 = {"15745", &P15745_4};	/*Profile ID - 15745*/
const Value V14182_1_1 = {S2, NULL};	/*Value = True*/
const Property P14182_1 = {S50, 0, &V14182_1_1, NULL};	/*Property = IsMobile*/
const Value V14182_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14182_5 = {S52, 0, &V14182_5_1304, &P14182_1};	/*Property = ScreenPixelsHeight*/
const Value V14182_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14182_4 = {S54, 0, &V14182_4_1224, &P14182_5};	/*Property = ScreenPixelsWidth*/
const Profile P14182 = {"14182", &P14182_4};	/*Profile ID - 14182*/
const Value V12588_1_1 = {S2, NULL};	/*Value = True*/
const Property P12588_1 = {S50, 0, &V12588_1_1, NULL};	/*Property = IsMobile*/
const Value V12588_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P12588_5 = {S52, 0, &V12588_5_1275, &P12588_1};	/*Property = ScreenPixelsHeight*/
const Value V12588_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P12588_4 = {S54, 0, &V12588_4_1278, &P12588_5};	/*Property = ScreenPixelsWidth*/
const Profile P12588 = {"12588", &P12588_4};	/*Profile ID - 12588*/
const Value V11731_1_1 = {S2, NULL};	/*Value = True*/
const Property P11731_1 = {S50, 0, &V11731_1_1, NULL};	/*Property = IsMobile*/
const Value V11731_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11731_5 = {S52, 0, &V11731_5_1293, &P11731_1};	/*Property = ScreenPixelsHeight*/
const Value V11731_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11731_4 = {S54, 0, &V11731_4_1296, &P11731_5};	/*Property = ScreenPixelsWidth*/
const Profile P11731 = {"11731", &P11731_4};	/*Profile ID - 11731*/
const Value V16083_1_1 = {S2, NULL};	/*Value = True*/
const Property P16083_1 = {S50, 0, &V16083_1_1, NULL};	/*Property = IsMobile*/
const Value V16083_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P16083_5 = {S52, 0, &V16083_5_1380, &P16083_1};	/*Property = ScreenPixelsHeight*/
const Value V16083_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16083_4 = {S54, 0, &V16083_4_1279, &P16083_5};	/*Property = ScreenPixelsWidth*/
const Profile P16083 = {"16083", &P16083_4};	/*Profile ID - 16083*/
const Value V11732_1_1 = {S2, NULL};	/*Value = True*/
const Property P11732_1 = {S50, 0, &V11732_1_1, NULL};	/*Property = IsMobile*/
const Value V11732_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P11732_5 = {S52, 0, &V11732_5_1380, &P11732_1};	/*Property = ScreenPixelsHeight*/
const Value V11732_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P11732_4 = {S54, 0, &V11732_4_1279, &P11732_5};	/*Property = ScreenPixelsWidth*/
const Profile P11732 = {"11732", &P11732_4};	/*Profile ID - 11732*/
const Value V14557_1_1 = {S2, NULL};	/*Value = True*/
const Property P14557_1 = {S50, 0, &V14557_1_1, NULL};	/*Property = IsMobile*/
const Value V14557_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14557_5 = {S52, 0, &V14557_5_1222, &P14557_1};	/*Property = ScreenPixelsHeight*/
const Value V14557_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14557_4 = {S54, 0, &V14557_4_1224, &P14557_5};	/*Property = ScreenPixelsWidth*/
const Profile P14557 = {"14557", &P14557_4};	/*Profile ID - 14557*/
const Value V3990_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_279 = {S1, 0, &V3990_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3990_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_282 = {S3, 0, &V3990_282_57155, &P3990_279};	/*Property = BlobBuilder*/
const Value V3990_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_350 = {S4, 0, &V3990_350_57446, &P3990_282};	/*Property = Canvas*/
const Value V3990_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_283 = {S5, 0, &V3990_283_57161, &P3990_350};	/*Property = CssBackground*/
const Value V3990_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_284 = {S6, 0, &V3990_284_57164, &P3990_283};	/*Property = CssBorderImage*/
const Value V3990_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_285 = {S7, 0, &V3990_285_57167, &P3990_284};	/*Property = CssCanvas*/
const Value V3990_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_286 = {S8, 0, &V3990_286_57170, &P3990_285};	/*Property = CssColor*/
const Value V3990_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_237 = {S9, 0, &V3990_237_55786, &P3990_286};	/*Property = CssColumn*/
const Value V3990_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_287 = {S10, 0, &V3990_287_57173, &P3990_237};	/*Property = CssFlexbox*/
const Value V3990_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_288 = {S11, 0, &V3990_288_57176, &P3990_287};	/*Property = CssFont*/
const Value V3990_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_289 = {S12, 0, &V3990_289_86899, &P3990_288};	/*Property = CssImages*/
const Value V3990_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_291 = {S13, 0, &V3990_291_57182, &P3990_289};	/*Property = CssMediaQueries*/
const Value V3990_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_292 = {S14, 0, &V3990_292_57185, &P3990_291};	/*Property = CssMinMax*/
const Value V3990_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_293 = {S15, 0, &V3990_293_57188, &P3990_292};	/*Property = CssOverflow*/
const Value V3990_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_294 = {S16, 0, &V3990_294_57191, &P3990_293};	/*Property = CssPosition*/
const Value V3990_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_295 = {S17, 0, &V3990_295_57194, &P3990_294};	/*Property = CssText*/
const Value V3990_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_297 = {S18, 0, &V3990_297_57197, &P3990_295};	/*Property = CssTransforms*/
const Value V3990_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_241 = {S19, 0, &V3990_241_55801, &P3990_297};	/*Property = CssTransitions*/
const Value V3990_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_298 = {S20, 0, &V3990_298_57208, &P3990_241};	/*Property = CssUI*/
const Value V3990_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_300 = {S21, 0, &V3990_300_57217, &P3990_298};	/*Property = DataSet*/
const Value V3990_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_301 = {S22, 0, &V3990_301_57220, &P3990_300};	/*Property = DataUrl*/
const Value V3990_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_302 = {S23, 0, &V3990_302_57223, &P3990_301};	/*Property = DeviceOrientation*/
const Value V3990_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_304 = {S24, 0, &V3990_304_57229, &P3990_302};	/*Property = FileReader*/
const Value V3990_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_305 = {S25, 0, &V3990_305_57232, &P3990_304};	/*Property = FileSaver*/
const Value V3990_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_306 = {S26, 0, &V3990_306_57238, &P3990_305};	/*Property = FileWriter*/
const Value V3990_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_307 = {S27, 0, &V3990_307_57241, &P3990_306};	/*Property = FormData*/
const Value V3990_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_309 = {S28, 0, &V3990_309_57247, &P3990_307};	/*Property = Fullscreen*/
const Value V3990_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_310 = {S29, 0, &V3990_310_57250, &P3990_309};	/*Property = GeoLocation*/
const Value V3990_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_312 = {S30, 0, &V3990_312_57256, &P3990_310};	/*Property = History*/
const Value V3990_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_314 = {S31, 0, &V3990_314_57262, &P3990_312};	/*Property = Html5*/
const Value V3990_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_313 = {S32, 0, &V3990_313_57259, &P3990_314};	/*Property = Html-Media-Capture*/
const Value V3990_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_315 = {S33, 0, &V3990_315_57265, &P3990_313};	/*Property = Iframe*/
const Value V3990_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_316 = {S34, 0, &V3990_316_57268, &P3990_315};	/*Property = IndexedDB*/
const Value V3990_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_317 = {S35, 0, &V3990_317_57271, &P3990_316};	/*Property = Json*/
const Value V3990_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_15 = {S37, 0, &V3990_15_72, &P3990_317};	/*Property = LayoutEngine*/
const Value V3990_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_344 = {S38, 0, &V3990_344_57274, &P3990_15};	/*Property = Masking*/
const Value V3990_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_320 = {S39, 0, &V3990_320_57292, &P3990_344};	/*Property = PostMessage*/
const Value V3990_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_321 = {S40, 0, &V3990_321_57295, &P3990_320};	/*Property = Progress*/
const Value V3990_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_322 = {S41, 0, &V3990_322_57298, &P3990_321};	/*Property = Prompts*/
const Value V3990_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_323 = {S42, 0, &V3990_323_57301, &P3990_322};	/*Property = Selector*/
const Value V3990_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_324 = {S43, 0, &V3990_324_57307, &P3990_323};	/*Property = Svg*/
const Value V3990_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_327 = {S44, 0, &V3990_327_57316, &P3990_324};	/*Property = TouchEvents*/
const Value V3990_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_328 = {S45, 0, &V3990_328_57319, &P3990_327};	/*Property = Track*/
const Value V3990_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_329 = {S46, 0, &V3990_329_57325, &P3990_328};	/*Property = Video*/
const Value V3990_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_330 = {S47, 0, &V3990_330_57328, &P3990_329};	/*Property = Viewport*/
const Value V3990_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_332 = {S48, 0, &V3990_332_57340, &P3990_330};	/*Property = WebWorkers*/
const Value V3990_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3990_333 = {S49, 0, &V3990_333_57343, &P3990_332};	/*Property = Xhr2*/
const Profile P3990 = {"3990", &P3990_333};	/*Profile ID - 3990*/
const Value V11733_1_1 = {S2, NULL};	/*Value = True*/
const Property P11733_1 = {S50, 0, &V11733_1_1, NULL};	/*Property = IsMobile*/
const Value V11733_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11733_5 = {S52, 0, &V11733_5_1233, &P11733_1};	/*Property = ScreenPixelsHeight*/
const Value V11733_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11733_4 = {S54, 0, &V11733_4_1235, &P11733_5};	/*Property = ScreenPixelsWidth*/
const Profile P11733 = {"11733", &P11733_4};	/*Profile ID - 11733*/
const Value V16210_1_1 = {S2, NULL};	/*Value = True*/
const Property P16210_1 = {S50, 0, &V16210_1_1, NULL};	/*Property = IsMobile*/
const Value V16210_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16210_5 = {S52, 0, &V16210_5_1222, &P16210_1};	/*Property = ScreenPixelsHeight*/
const Value V16210_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16210_4 = {S54, 0, &V16210_4_1224, &P16210_5};	/*Property = ScreenPixelsWidth*/
const Profile P16210 = {"16210", &P16210_4};	/*Profile ID - 16210*/
const Value V11736_1_1 = {S2, NULL};	/*Value = True*/
const Property P11736_1 = {S50, 0, &V11736_1_1, NULL};	/*Property = IsMobile*/
const Value V11736_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P11736_5 = {S52, 0, &V11736_5_1294, &P11736_1};	/*Property = ScreenPixelsHeight*/
const Value V11736_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11736_4 = {S54, 0, &V11736_4_1296, &P11736_5};	/*Property = ScreenPixelsWidth*/
const Profile P11736 = {"11736", &P11736_4};	/*Profile ID - 11736*/
const Value V14254_1_1 = {S2, NULL};	/*Value = True*/
const Property P14254_1 = {S50, 0, &V14254_1_1, NULL};	/*Property = IsMobile*/
const Value V14254_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14254_5 = {S52, 0, &V14254_5_1294, &P14254_1};	/*Property = ScreenPixelsHeight*/
const Value V14254_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14254_4 = {S54, 0, &V14254_4_1296, &P14254_5};	/*Property = ScreenPixelsWidth*/
const Profile P14254 = {"14254", &P14254_4};	/*Profile ID - 14254*/
const Value V14698_1_1 = {S2, NULL};	/*Value = True*/
const Property P14698_1 = {S50, 0, &V14698_1_1, NULL};	/*Property = IsMobile*/
const Value V14698_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14698_5 = {S52, 0, &V14698_5_1227, &P14698_1};	/*Property = ScreenPixelsHeight*/
const Value V14698_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14698_4 = {S54, 0, &V14698_4_1229, &P14698_5};	/*Property = ScreenPixelsWidth*/
const Profile P14698 = {"14698", &P14698_4};	/*Profile ID - 14698*/
const Value V11949_1_1 = {S2, NULL};	/*Value = True*/
const Property P11949_1 = {S50, 0, &V11949_1_1, NULL};	/*Property = IsMobile*/
const Value V11949_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11949_5 = {S52, 0, &V11949_5_1222, &P11949_1};	/*Property = ScreenPixelsHeight*/
const Value V11949_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11949_4 = {S54, 0, &V11949_4_1224, &P11949_5};	/*Property = ScreenPixelsWidth*/
const Profile P11949 = {"11949", &P11949_4};	/*Profile ID - 11949*/
const Value V11745_1_1 = {S2, NULL};	/*Value = True*/
const Property P11745_1 = {S50, 0, &V11745_1_1, NULL};	/*Property = IsMobile*/
const Value V11745_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11745_5 = {S52, 0, &V11745_5_1222, &P11745_1};	/*Property = ScreenPixelsHeight*/
const Value V11745_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11745_4 = {S54, 0, &V11745_4_1224, &P11745_5};	/*Property = ScreenPixelsWidth*/
const Profile P11745 = {"11745", &P11745_4};	/*Profile ID - 11745*/
const Value V17622_1_1 = {S2, NULL};	/*Value = True*/
const Property P17622_1 = {S50, 0, &V17622_1_1, NULL};	/*Property = IsMobile*/
const Value V17622_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17622_5 = {S52, 0, &V17622_5_1277, &P17622_1};	/*Property = ScreenPixelsHeight*/
const Value V17622_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17622_4 = {S54, 0, &V17622_4_1279, &P17622_5};	/*Property = ScreenPixelsWidth*/
const Profile P17622 = {"17622", &P17622_4};	/*Profile ID - 17622*/
const Value V13201_1_1 = {S2, NULL};	/*Value = True*/
const Property P13201_1 = {S50, 0, &V13201_1_1, NULL};	/*Property = IsMobile*/
const Value V13201_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13201_5 = {S52, 0, &V13201_5_1277, &P13201_1};	/*Property = ScreenPixelsHeight*/
const Value V13201_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13201_4 = {S54, 0, &V13201_4_1279, &P13201_5};	/*Property = ScreenPixelsWidth*/
const Profile P13201 = {"13201", &P13201_4};	/*Profile ID - 13201*/
const Value V16392_1_1 = {S2, NULL};	/*Value = True*/
const Property P16392_1 = {S50, 0, &V16392_1_1, NULL};	/*Property = IsMobile*/
const Value V16392_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16392_5 = {S52, 0, &V16392_5_1222, &P16392_1};	/*Property = ScreenPixelsHeight*/
const Value V16392_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16392_4 = {S54, 0, &V16392_4_1224, &P16392_5};	/*Property = ScreenPixelsWidth*/
const Profile P16392 = {"16392", &P16392_4};	/*Profile ID - 16392*/
const Value V16218_1_1 = {S2, NULL};	/*Value = True*/
const Property P16218_1 = {S50, 0, &V16218_1_1, NULL};	/*Property = IsMobile*/
const Value V16218_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P16218_5 = {S52, 0, &V16218_5_1280, &P16218_1};	/*Property = ScreenPixelsHeight*/
const Value V16218_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16218_4 = {S54, 0, &V16218_4_1235, &P16218_5};	/*Property = ScreenPixelsWidth*/
const Profile P16218 = {"16218", &P16218_4};	/*Profile ID - 16218*/
const Value V13504_1_1 = {S2, NULL};	/*Value = True*/
const Property P13504_1 = {S50, 0, &V13504_1_1, NULL};	/*Property = IsMobile*/
const Value V13504_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13504_5 = {S52, 0, &V13504_5_1293, &P13504_1};	/*Property = ScreenPixelsHeight*/
const Value V13504_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13504_4 = {S54, 0, &V13504_4_1296, &P13504_5};	/*Property = ScreenPixelsWidth*/
const Profile P13504 = {"13504", &P13504_4};	/*Profile ID - 13504*/
const Value V17051_1_1 = {S2, NULL};	/*Value = True*/
const Property P17051_1 = {S50, 0, &V17051_1_1, NULL};	/*Property = IsMobile*/
const Value V17051_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17051_5 = {S52, 0, &V17051_5_1222, &P17051_1};	/*Property = ScreenPixelsHeight*/
const Value V17051_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17051_4 = {S54, 0, &V17051_4_1224, &P17051_5};	/*Property = ScreenPixelsWidth*/
const Profile P17051 = {"17051", &P17051_4};	/*Profile ID - 17051*/
const Value V13669_1_1 = {S2, NULL};	/*Value = True*/
const Property P13669_1 = {S50, 0, &V13669_1_1, NULL};	/*Property = IsMobile*/
const Value V13669_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13669_5 = {S52, 0, &V13669_5_1277, &P13669_1};	/*Property = ScreenPixelsHeight*/
const Value V13669_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13669_4 = {S54, 0, &V13669_4_1279, &P13669_5};	/*Property = ScreenPixelsWidth*/
const Profile P13669 = {"13669", &P13669_4};	/*Profile ID - 13669*/
const Value V13671_1_1 = {S2, NULL};	/*Value = True*/
const Property P13671_1 = {S50, 0, &V13671_1_1, NULL};	/*Property = IsMobile*/
const Value V13671_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13671_5 = {S52, 0, &V13671_5_1222, &P13671_1};	/*Property = ScreenPixelsHeight*/
const Value V13671_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13671_4 = {S54, 0, &V13671_4_1224, &P13671_5};	/*Property = ScreenPixelsWidth*/
const Profile P13671 = {"13671", &P13671_4};	/*Profile ID - 13671*/
const Value V13677_1_1 = {S2, NULL};	/*Value = True*/
const Property P13677_1 = {S50, 0, &V13677_1_1, NULL};	/*Property = IsMobile*/
const Value V13677_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13677_5 = {S52, 0, &V13677_5_1304, &P13677_1};	/*Property = ScreenPixelsHeight*/
const Value V13677_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13677_4 = {S54, 0, &V13677_4_1224, &P13677_5};	/*Property = ScreenPixelsWidth*/
const Profile P13677 = {"13677", &P13677_4};	/*Profile ID - 13677*/
const Value V13691_1_1 = {S2, NULL};	/*Value = True*/
const Property P13691_1 = {S50, 0, &V13691_1_1, NULL};	/*Property = IsMobile*/
const Value V13691_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13691_5 = {S52, 0, &V13691_5_1277, &P13691_1};	/*Property = ScreenPixelsHeight*/
const Value V13691_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13691_4 = {S54, 0, &V13691_4_1279, &P13691_5};	/*Property = ScreenPixelsWidth*/
const Profile P13691 = {"13691", &P13691_4};	/*Profile ID - 13691*/
const Value V17931_1_1 = {S2, NULL};	/*Value = True*/
const Property P17931_1 = {S50, 0, &V17931_1_1, NULL};	/*Property = IsMobile*/
const Value V17931_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P17931_5 = {S52, 0, &V17931_5_1458, &P17931_1};	/*Property = ScreenPixelsHeight*/
const Value V17931_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P17931_4 = {S54, 0, &V17931_4_1460, &P17931_5};	/*Property = ScreenPixelsWidth*/
const Profile P17931 = {"17931", &P17931_4};	/*Profile ID - 17931*/
const Value V17241_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17241_279 = {S1, 0, &V17241_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17241_282_57153 = {S2, NULL};	/*Value = True*/
const Property P17241_282 = {S3, 0, &V17241_282_57153, &P17241_279};	/*Property = BlobBuilder*/
const Value V17241_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17241_350 = {S4, 0, &V17241_350_57444, &P17241_282};	/*Property = Canvas*/
const Value V17241_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17241_283 = {S5, 0, &V17241_283_57159, &P17241_350};	/*Property = CssBackground*/
const Value V17241_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17241_284 = {S6, 0, &V17241_284_57163, &P17241_283};	/*Property = CssBorderImage*/
const Value V17241_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17241_285 = {S7, 0, &V17241_285_57166, &P17241_284};	/*Property = CssCanvas*/
const Value V17241_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17241_286 = {S8, 0, &V17241_286_57168, &P17241_285};	/*Property = CssColor*/
const Value V17241_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17241_237 = {S9, 0, &V17241_237_55786, &P17241_286};	/*Property = CssColumn*/
const Value V17241_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17241_287 = {S10, 0, &V17241_287_57172, &P17241_237};	/*Property = CssFlexbox*/
const Value V17241_288_57174 = {S2, NULL};	/*Value = True*/
const Property P17241_288 = {S11, 0, &V17241_288_57174, &P17241_287};	/*Property = CssFont*/
const Value V17241_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17241_289 = {S12, 0, &V17241_289_57759, &P17241_288};	/*Property = CssImages*/
const Value V17241_291_57180 = {S2, NULL};	/*Value = True*/
const Property P17241_291 = {S13, 0, &V17241_291_57180, &P17241_289};	/*Property = CssMediaQueries*/
const Value V17241_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17241_292 = {S14, 0, &V17241_292_57183, &P17241_291};	/*Property = CssMinMax*/
const Value V17241_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17241_293 = {S15, 0, &V17241_293_57187, &P17241_292};	/*Property = CssOverflow*/
const Value V17241_294_57190 = {S0, NULL};	/*Value = False*/
const Property P17241_294 = {S16, 0, &V17241_294_57190, &P17241_293};	/*Property = CssPosition*/
const Value V17241_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17241_295 = {S17, 0, &V17241_295_57192, &P17241_294};	/*Property = CssText*/
const Value V17241_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17241_297 = {S18, 0, &V17241_297_57195, &P17241_295};	/*Property = CssTransforms*/
const Value V17241_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17241_241 = {S19, 0, &V17241_241_55799, &P17241_297};	/*Property = CssTransitions*/
const Value V17241_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17241_298 = {S20, 0, &V17241_298_57206, &P17241_241};	/*Property = CssUI*/
const Value V17241_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17241_300 = {S21, 0, &V17241_300_57216, &P17241_298};	/*Property = DataSet*/
const Value V17241_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17241_301 = {S22, 0, &V17241_301_57218, &P17241_300};	/*Property = DataUrl*/
const Value V17241_302_57221 = {S2, NULL};	/*Value = True*/
const Property P17241_302 = {S23, 0, &V17241_302_57221, &P17241_301};	/*Property = DeviceOrientation*/
const Value V17241_304_57227 = {S2, NULL};	/*Value = True*/
const Property P17241_304 = {S24, 0, &V17241_304_57227, &P17241_302};	/*Property = FileReader*/
const Value V17241_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17241_305 = {S25, 0, &V17241_305_57231, &P17241_304};	/*Property = FileSaver*/
const Value V17241_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17241_306 = {S26, 0, &V17241_306_57237, &P17241_305};	/*Property = FileWriter*/
const Value V17241_307_57239 = {S2, NULL};	/*Value = True*/
const Property P17241_307 = {S27, 0, &V17241_307_57239, &P17241_306};	/*Property = FormData*/
const Value V17241_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17241_309 = {S28, 0, &V17241_309_57246, &P17241_307};	/*Property = Fullscreen*/
const Value V17241_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17241_310 = {S29, 0, &V17241_310_57248, &P17241_309};	/*Property = GeoLocation*/
const Value V17241_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17241_312 = {S30, 0, &V17241_312_57255, &P17241_310};	/*Property = History*/
const Value V17241_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17241_314 = {S31, 0, &V17241_314_57261, &P17241_312};	/*Property = Html5*/
const Value V17241_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17241_313 = {S32, 0, &V17241_313_57258, &P17241_314};	/*Property = Html-Media-Capture*/
const Value V17241_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17241_315 = {S33, 0, &V17241_315_57264, &P17241_313};	/*Property = Iframe*/
const Value V17241_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17241_316 = {S34, 0, &V17241_316_57267, &P17241_315};	/*Property = IndexedDB*/
const Value V17241_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17241_317 = {S35, 0, &V17241_317_57269, &P17241_316};	/*Property = Json*/
const Value V17241_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17241_15 = {S37, 0, &V17241_15_28, &P17241_317};	/*Property = LayoutEngine*/
const Value V17241_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17241_344 = {S38, 0, &V17241_344_57272, &P17241_15};	/*Property = Masking*/
const Value V17241_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17241_320 = {S39, 0, &V17241_320_57290, &P17241_344};	/*Property = PostMessage*/
const Value V17241_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17241_321 = {S40, 0, &V17241_321_57293, &P17241_320};	/*Property = Progress*/
const Value V17241_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17241_322 = {S41, 0, &V17241_322_57296, &P17241_321};	/*Property = Prompts*/
const Value V17241_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17241_323 = {S42, 0, &V17241_323_57299, &P17241_322};	/*Property = Selector*/
const Value V17241_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17241_324 = {S43, 0, &V17241_324_57306, &P17241_323};	/*Property = Svg*/
const Value V17241_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17241_327 = {S44, 0, &V17241_327_57315, &P17241_324};	/*Property = TouchEvents*/
const Value V17241_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17241_328 = {S45, 0, &V17241_328_57318, &P17241_327};	/*Property = Track*/
const Value V17241_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17241_329 = {S46, 0, &V17241_329_57323, &P17241_328};	/*Property = Video*/
const Value V17241_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17241_330 = {S47, 0, &V17241_330_57326, &P17241_329};	/*Property = Viewport*/
const Value V17241_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17241_332 = {S48, 0, &V17241_332_57339, &P17241_330};	/*Property = WebWorkers*/
const Value V17241_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17241_333 = {S49, 0, &V17241_333_57342, &P17241_332};	/*Property = Xhr2*/
const Profile P17241 = {"17241", &P17241_333};	/*Profile ID - 17241*/
const Value V17559_1_1 = {S2, NULL};	/*Value = True*/
const Property P17559_1 = {S50, 0, &V17559_1_1, NULL};	/*Property = IsMobile*/
const Value V17559_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17559_5 = {S52, 0, &V17559_5_1304, &P17559_1};	/*Property = ScreenPixelsHeight*/
const Value V17559_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17559_4 = {S54, 0, &V17559_4_1224, &P17559_5};	/*Property = ScreenPixelsWidth*/
const Profile P17559 = {"17559", &P17559_4};	/*Profile ID - 17559*/
const Value V13704_1_1 = {S2, NULL};	/*Value = True*/
const Property P13704_1 = {S50, 0, &V13704_1_1, NULL};	/*Property = IsMobile*/
const Value V13704_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13704_5 = {S52, 0, &V13704_5_1304, &P13704_1};	/*Property = ScreenPixelsHeight*/
const Value V13704_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13704_4 = {S54, 0, &V13704_4_1224, &P13704_5};	/*Property = ScreenPixelsWidth*/
const Profile P13704 = {"13704", &P13704_4};	/*Profile ID - 13704*/
const Value V13941_1_1 = {S2, NULL};	/*Value = True*/
const Property P13941_1 = {S50, 0, &V13941_1_1, NULL};	/*Property = IsMobile*/
const Value V13941_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13941_5 = {S52, 0, &V13941_5_1227, &P13941_1};	/*Property = ScreenPixelsHeight*/
const Value V13941_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13941_4 = {S54, 0, &V13941_4_1229, &P13941_5};	/*Property = ScreenPixelsWidth*/
const Profile P13941 = {"13941", &P13941_4};	/*Profile ID - 13941*/
const Value V13711_1_1 = {S2, NULL};	/*Value = True*/
const Property P13711_1 = {S50, 0, &V13711_1_1, NULL};	/*Property = IsMobile*/
const Value V13711_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13711_5 = {S52, 0, &V13711_5_1293, &P13711_1};	/*Property = ScreenPixelsHeight*/
const Value V13711_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13711_4 = {S54, 0, &V13711_4_1296, &P13711_5};	/*Property = ScreenPixelsWidth*/
const Profile P13711 = {"13711", &P13711_4};	/*Profile ID - 13711*/
const Value V14011_1_1 = {S2, NULL};	/*Value = True*/
const Property P14011_1 = {S50, 0, &V14011_1_1, NULL};	/*Property = IsMobile*/
const Value V14011_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14011_5 = {S52, 0, &V14011_5_1277, &P14011_1};	/*Property = ScreenPixelsHeight*/
const Value V14011_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14011_4 = {S54, 0, &V14011_4_1279, &P14011_5};	/*Property = ScreenPixelsWidth*/
const Profile P14011 = {"14011", &P14011_4};	/*Profile ID - 14011*/
const Value V14748_1_1 = {S2, NULL};	/*Value = True*/
const Property P14748_1 = {S50, 0, &V14748_1_1, NULL};	/*Property = IsMobile*/
const Value V14748_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14748_5 = {S52, 0, &V14748_5_1380, &P14748_1};	/*Property = ScreenPixelsHeight*/
const Value V14748_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14748_4 = {S54, 0, &V14748_4_1279, &P14748_5};	/*Property = ScreenPixelsWidth*/
const Profile P14748 = {"14748", &P14748_4};	/*Profile ID - 14748*/
const Value V14961_1_1 = {S2, NULL};	/*Value = True*/
const Property P14961_1 = {S50, 0, &V14961_1_1, NULL};	/*Property = IsMobile*/
const Value V14961_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14961_5 = {S52, 0, &V14961_5_1222, &P14961_1};	/*Property = ScreenPixelsHeight*/
const Value V14961_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14961_4 = {S54, 0, &V14961_4_1224, &P14961_5};	/*Property = ScreenPixelsWidth*/
const Profile P14961 = {"14961", &P14961_4};	/*Profile ID - 14961*/
const Value V13993_1_1 = {S2, NULL};	/*Value = True*/
const Property P13993_1 = {S50, 0, &V13993_1_1, NULL};	/*Property = IsMobile*/
const Value V13993_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13993_5 = {S52, 0, &V13993_5_1380, &P13993_1};	/*Property = ScreenPixelsHeight*/
const Value V13993_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13993_4 = {S54, 0, &V13993_4_1279, &P13993_5};	/*Property = ScreenPixelsWidth*/
const Profile P13993 = {"13993", &P13993_4};	/*Profile ID - 13993*/
const Value V14353_1_1 = {S2, NULL};	/*Value = True*/
const Property P14353_1 = {S50, 0, &V14353_1_1, NULL};	/*Property = IsMobile*/
const Value V14353_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14353_5 = {S52, 0, &V14353_5_1304, &P14353_1};	/*Property = ScreenPixelsHeight*/
const Value V14353_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14353_4 = {S54, 0, &V14353_4_1224, &P14353_5};	/*Property = ScreenPixelsWidth*/
const Profile P14353 = {"14353", &P14353_4};	/*Profile ID - 14353*/
const Value V13730_1_1 = {S2, NULL};	/*Value = True*/
const Property P13730_1 = {S50, 0, &V13730_1_1, NULL};	/*Property = IsMobile*/
const Value V13730_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13730_5 = {S52, 0, &V13730_5_1222, &P13730_1};	/*Property = ScreenPixelsHeight*/
const Value V13730_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13730_4 = {S54, 0, &V13730_4_1224, &P13730_5};	/*Property = ScreenPixelsWidth*/
const Profile P13730 = {"13730", &P13730_4};	/*Profile ID - 13730*/
const Value V13749_1_1 = {S2, NULL};	/*Value = True*/
const Property P13749_1 = {S50, 0, &V13749_1_1, NULL};	/*Property = IsMobile*/
const Value V13749_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13749_5 = {S52, 0, &V13749_5_1304, &P13749_1};	/*Property = ScreenPixelsHeight*/
const Value V13749_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13749_4 = {S54, 0, &V13749_4_1224, &P13749_5};	/*Property = ScreenPixelsWidth*/
const Profile P13749 = {"13749", &P13749_4};	/*Profile ID - 13749*/
const Value V13750_1_1 = {S2, NULL};	/*Value = True*/
const Property P13750_1 = {S50, 0, &V13750_1_1, NULL};	/*Property = IsMobile*/
const Value V13750_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13750_5 = {S52, 0, &V13750_5_1233, &P13750_1};	/*Property = ScreenPixelsHeight*/
const Value V13750_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13750_4 = {S54, 0, &V13750_4_1235, &P13750_5};	/*Property = ScreenPixelsWidth*/
const Profile P13750 = {"13750", &P13750_4};	/*Profile ID - 13750*/
const Value V13755_1_1 = {S2, NULL};	/*Value = True*/
const Property P13755_1 = {S50, 0, &V13755_1_1, NULL};	/*Property = IsMobile*/
const Value V13755_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13755_5 = {S52, 0, &V13755_5_1222, &P13755_1};	/*Property = ScreenPixelsHeight*/
const Value V13755_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13755_4 = {S54, 0, &V13755_4_1224, &P13755_5};	/*Property = ScreenPixelsWidth*/
const Profile P13755 = {"13755", &P13755_4};	/*Profile ID - 13755*/
const Value V13757_1_1 = {S2, NULL};	/*Value = True*/
const Property P13757_1 = {S50, 0, &V13757_1_1, NULL};	/*Property = IsMobile*/
const Value V13757_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13757_5 = {S52, 0, &V13757_5_1293, &P13757_1};	/*Property = ScreenPixelsHeight*/
const Value V13757_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13757_4 = {S54, 0, &V13757_4_1296, &P13757_5};	/*Property = ScreenPixelsWidth*/
const Profile P13757 = {"13757", &P13757_4};	/*Profile ID - 13757*/
const Value V13760_1_1 = {S2, NULL};	/*Value = True*/
const Property P13760_1 = {S50, 0, &V13760_1_1, NULL};	/*Property = IsMobile*/
const Value V13760_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13760_5 = {S52, 0, &V13760_5_1233, &P13760_1};	/*Property = ScreenPixelsHeight*/
const Value V13760_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13760_4 = {S54, 0, &V13760_4_1235, &P13760_5};	/*Property = ScreenPixelsWidth*/
const Profile P13760 = {"13760", &P13760_4};	/*Profile ID - 13760*/
const Value V13762_1_1 = {S2, NULL};	/*Value = True*/
const Property P13762_1 = {S50, 0, &V13762_1_1, NULL};	/*Property = IsMobile*/
const Value V13762_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13762_5 = {S52, 0, &V13762_5_1380, &P13762_1};	/*Property = ScreenPixelsHeight*/
const Value V13762_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13762_4 = {S54, 0, &V13762_4_1279, &P13762_5};	/*Property = ScreenPixelsWidth*/
const Profile P13762 = {"13762", &P13762_4};	/*Profile ID - 13762*/
const Value V13904_1_1 = {S2, NULL};	/*Value = True*/
const Property P13904_1 = {S50, 0, &V13904_1_1, NULL};	/*Property = IsMobile*/
const Value V13904_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13904_5 = {S52, 0, &V13904_5_1293, &P13904_1};	/*Property = ScreenPixelsHeight*/
const Value V13904_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13904_4 = {S54, 0, &V13904_4_1296, &P13904_5};	/*Property = ScreenPixelsWidth*/
const Profile P13904 = {"13904", &P13904_4};	/*Profile ID - 13904*/
const Value V13761_1_1 = {S2, NULL};	/*Value = True*/
const Property P13761_1 = {S50, 0, &V13761_1_1, NULL};	/*Property = IsMobile*/
const Value V13761_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13761_5 = {S52, 0, &V13761_5_1222, &P13761_1};	/*Property = ScreenPixelsHeight*/
const Value V13761_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13761_4 = {S54, 0, &V13761_4_1224, &P13761_5};	/*Property = ScreenPixelsWidth*/
const Profile P13761 = {"13761", &P13761_4};	/*Profile ID - 13761*/
const Value V13733_1_1 = {S2, NULL};	/*Value = True*/
const Property P13733_1 = {S50, 0, &V13733_1_1, NULL};	/*Property = IsMobile*/
const Value V13733_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13733_5 = {S52, 0, &V13733_5_1233, &P13733_1};	/*Property = ScreenPixelsHeight*/
const Value V13733_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13733_4 = {S54, 0, &V13733_4_1235, &P13733_5};	/*Property = ScreenPixelsWidth*/
const Profile P13733 = {"13733", &P13733_4};	/*Profile ID - 13733*/
const Value V18005_1_1 = {S2, NULL};	/*Value = True*/
const Property P18005_1 = {S50, 0, &V18005_1_1, NULL};	/*Property = IsMobile*/
const Value V18005_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18005_5 = {S52, 0, &V18005_5_1222, &P18005_1};	/*Property = ScreenPixelsHeight*/
const Value V18005_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18005_4 = {S54, 0, &V18005_4_1224, &P18005_5};	/*Property = ScreenPixelsWidth*/
const Profile P18005 = {"18005", &P18005_4};	/*Profile ID - 18005*/
const Value V17670_1_1 = {S2, NULL};	/*Value = True*/
const Property P17670_1 = {S50, 0, &V17670_1_1, NULL};	/*Property = IsMobile*/
const Value V17670_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17670_5 = {S52, 0, &V17670_5_1222, &P17670_1};	/*Property = ScreenPixelsHeight*/
const Value V17670_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17670_4 = {S54, 0, &V17670_4_1224, &P17670_5};	/*Property = ScreenPixelsWidth*/
const Profile P17670 = {"17670", &P17670_4};	/*Profile ID - 17670*/
const Value V13765_1_1 = {S2, NULL};	/*Value = True*/
const Property P13765_1 = {S50, 0, &V13765_1_1, NULL};	/*Property = IsMobile*/
const Value V13765_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13765_5 = {S52, 0, &V13765_5_1380, &P13765_1};	/*Property = ScreenPixelsHeight*/
const Value V13765_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13765_4 = {S54, 0, &V13765_4_1279, &P13765_5};	/*Property = ScreenPixelsWidth*/
const Profile P13765 = {"13765", &P13765_4};	/*Profile ID - 13765*/
const Value V15944_1_1 = {S2, NULL};	/*Value = True*/
const Property P15944_1 = {S50, 0, &V15944_1_1, NULL};	/*Property = IsMobile*/
const Value V15944_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15944_5 = {S52, 0, &V15944_5_1222, &P15944_1};	/*Property = ScreenPixelsHeight*/
const Value V15944_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15944_4 = {S54, 0, &V15944_4_1224, &P15944_5};	/*Property = ScreenPixelsWidth*/
const Profile P15944 = {"15944", &P15944_4};	/*Profile ID - 15944*/
const Value V13775_1_1 = {S2, NULL};	/*Value = True*/
const Property P13775_1 = {S50, 0, &V13775_1_1, NULL};	/*Property = IsMobile*/
const Value V13775_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P13775_5 = {S52, 0, &V13775_5_1269, &P13775_1};	/*Property = ScreenPixelsHeight*/
const Value V13775_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13775_4 = {S54, 0, &V13775_4_1229, &P13775_5};	/*Property = ScreenPixelsWidth*/
const Profile P13775 = {"13775", &P13775_4};	/*Profile ID - 13775*/
const Value V13776_1_1 = {S2, NULL};	/*Value = True*/
const Property P13776_1 = {S50, 0, &V13776_1_1, NULL};	/*Property = IsMobile*/
const Value V13776_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13776_5 = {S52, 0, &V13776_5_1380, &P13776_1};	/*Property = ScreenPixelsHeight*/
const Value V13776_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13776_4 = {S54, 0, &V13776_4_1279, &P13776_5};	/*Property = ScreenPixelsWidth*/
const Profile P13776 = {"13776", &P13776_4};	/*Profile ID - 13776*/
const Value V15207_1_1 = {S2, NULL};	/*Value = True*/
const Property P15207_1 = {S50, 0, &V15207_1_1, NULL};	/*Property = IsMobile*/
const Value V15207_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15207_5 = {S52, 0, &V15207_5_1222, &P15207_1};	/*Property = ScreenPixelsHeight*/
const Value V15207_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15207_4 = {S54, 0, &V15207_4_1224, &P15207_5};	/*Property = ScreenPixelsWidth*/
const Profile P15207 = {"15207", &P15207_4};	/*Profile ID - 15207*/
const Value V13782_1_1 = {S2, NULL};	/*Value = True*/
const Property P13782_1 = {S50, 0, &V13782_1_1, NULL};	/*Property = IsMobile*/
const Value V13782_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13782_5 = {S52, 0, &V13782_5_1222, &P13782_1};	/*Property = ScreenPixelsHeight*/
const Value V13782_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13782_4 = {S54, 0, &V13782_4_1224, &P13782_5};	/*Property = ScreenPixelsWidth*/
const Profile P13782 = {"13782", &P13782_4};	/*Profile ID - 13782*/
const Value V13784_1_1 = {S2, NULL};	/*Value = True*/
const Property P13784_1 = {S50, 0, &V13784_1_1, NULL};	/*Property = IsMobile*/
const Value V13784_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13784_5 = {S52, 0, &V13784_5_1222, &P13784_1};	/*Property = ScreenPixelsHeight*/
const Value V13784_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13784_4 = {S54, 0, &V13784_4_1224, &P13784_5};	/*Property = ScreenPixelsWidth*/
const Profile P13784 = {"13784", &P13784_4};	/*Profile ID - 13784*/
const Value V14482_1_1 = {S2, NULL};	/*Value = True*/
const Property P14482_1 = {S50, 0, &V14482_1_1, NULL};	/*Property = IsMobile*/
const Value V14482_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14482_5 = {S52, 0, &V14482_5_1222, &P14482_1};	/*Property = ScreenPixelsHeight*/
const Value V14482_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14482_4 = {S54, 0, &V14482_4_1224, &P14482_5};	/*Property = ScreenPixelsWidth*/
const Profile P14482 = {"14482", &P14482_4};	/*Profile ID - 14482*/
const Value V12414_1_1 = {S2, NULL};	/*Value = True*/
const Property P12414_1 = {S50, 0, &V12414_1_1, NULL};	/*Property = IsMobile*/
const Value V12414_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12414_5 = {S52, 0, &V12414_5_1222, &P12414_1};	/*Property = ScreenPixelsHeight*/
const Value V12414_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12414_4 = {S54, 0, &V12414_4_1224, &P12414_5};	/*Property = ScreenPixelsWidth*/
const Profile P12414 = {"12414", &P12414_4};	/*Profile ID - 12414*/
const Value V13787_1_1 = {S2, NULL};	/*Value = True*/
const Property P13787_1 = {S50, 0, &V13787_1_1, NULL};	/*Property = IsMobile*/
const Value V13787_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P13787_5 = {S52, 0, &V13787_5_1458, &P13787_1};	/*Property = ScreenPixelsHeight*/
const Value V13787_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P13787_4 = {S54, 0, &V13787_4_1460, &P13787_5};	/*Property = ScreenPixelsWidth*/
const Profile P13787 = {"13787", &P13787_4};	/*Profile ID - 13787*/
const Value V14912_1_1 = {S2, NULL};	/*Value = True*/
const Property P14912_1 = {S50, 0, &V14912_1_1, NULL};	/*Property = IsMobile*/
const Value V14912_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P14912_5 = {S52, 0, &V14912_5_1333, &P14912_1};	/*Property = ScreenPixelsHeight*/
const Value V14912_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P14912_4 = {S54, 0, &V14912_4_1335, &P14912_5};	/*Property = ScreenPixelsWidth*/
const Profile P14912 = {"14912", &P14912_4};	/*Profile ID - 14912*/
const Value V12849_1_1 = {S2, NULL};	/*Value = True*/
const Property P12849_1 = {S50, 0, &V12849_1_1, NULL};	/*Property = IsMobile*/
const Value V12849_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12849_5 = {S52, 0, &V12849_5_1304, &P12849_1};	/*Property = ScreenPixelsHeight*/
const Value V12849_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12849_4 = {S54, 0, &V12849_4_1224, &P12849_5};	/*Property = ScreenPixelsWidth*/
const Profile P12849 = {"12849", &P12849_4};	/*Profile ID - 12849*/
const Value V11750_1_1 = {S2, NULL};	/*Value = True*/
const Property P11750_1 = {S50, 0, &V11750_1_1, NULL};	/*Property = IsMobile*/
const Value V11750_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11750_5 = {S52, 0, &V11750_5_1222, &P11750_1};	/*Property = ScreenPixelsHeight*/
const Value V11750_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11750_4 = {S54, 0, &V11750_4_1224, &P11750_5};	/*Property = ScreenPixelsWidth*/
const Profile P11750 = {"11750", &P11750_4};	/*Profile ID - 11750*/
const Value V11751_1_1 = {S2, NULL};	/*Value = True*/
const Property P11751_1 = {S50, 0, &V11751_1_1, NULL};	/*Property = IsMobile*/
const Value V11751_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P11751_5 = {S52, 0, &V11751_5_1304, &P11751_1};	/*Property = ScreenPixelsHeight*/
const Value V11751_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11751_4 = {S54, 0, &V11751_4_1224, &P11751_5};	/*Property = ScreenPixelsWidth*/
const Profile P11751 = {"11751", &P11751_4};	/*Profile ID - 11751*/
const Value V14607_1_1 = {S2, NULL};	/*Value = True*/
const Property P14607_1 = {S50, 0, &V14607_1_1, NULL};	/*Property = IsMobile*/
const Value V14607_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14607_5 = {S52, 0, &V14607_5_1304, &P14607_1};	/*Property = ScreenPixelsHeight*/
const Value V14607_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14607_4 = {S54, 0, &V14607_4_1224, &P14607_5};	/*Property = ScreenPixelsWidth*/
const Profile P14607 = {"14607", &P14607_4};	/*Profile ID - 14607*/
const Value V14283_1_1 = {S2, NULL};	/*Value = True*/
const Property P14283_1 = {S50, 0, &V14283_1_1, NULL};	/*Property = IsMobile*/
const Value V14283_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P14283_5 = {S52, 0, &V14283_5_1280, &P14283_1};	/*Property = ScreenPixelsHeight*/
const Value V14283_4_1368 = {S74, NULL};	/*Value = 208*/
const Property P14283_4 = {S54, 0, &V14283_4_1368, &P14283_5};	/*Property = ScreenPixelsWidth*/
const Profile P14283 = {"14283", &P14283_4};	/*Profile ID - 14283*/
const Value V11755_1_1 = {S2, NULL};	/*Value = True*/
const Property P11755_1 = {S50, 0, &V11755_1_1, NULL};	/*Property = IsMobile*/
const Value V11755_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P11755_5 = {S52, 0, &V11755_5_1280, &P11755_1};	/*Property = ScreenPixelsHeight*/
const Value V11755_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11755_4 = {S54, 0, &V11755_4_1235, &P11755_5};	/*Property = ScreenPixelsWidth*/
const Profile P11755 = {"11755", &P11755_4};	/*Profile ID - 11755*/
const Value V11758_1_1 = {S2, NULL};	/*Value = True*/
const Property P11758_1 = {S50, 0, &V11758_1_1, NULL};	/*Property = IsMobile*/
const Value V11758_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P11758_5 = {S52, 0, &V11758_5_1294, &P11758_1};	/*Property = ScreenPixelsHeight*/
const Value V11758_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11758_4 = {S54, 0, &V11758_4_1296, &P11758_5};	/*Property = ScreenPixelsWidth*/
const Profile P11758 = {"11758", &P11758_4};	/*Profile ID - 11758*/
const Value V11760_1_1 = {S2, NULL};	/*Value = True*/
const Property P11760_1 = {S50, 0, &V11760_1_1, NULL};	/*Property = IsMobile*/
const Value V11760_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P11760_5 = {S52, 0, &V11760_5_1304, &P11760_1};	/*Property = ScreenPixelsHeight*/
const Value V11760_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11760_4 = {S54, 0, &V11760_4_1224, &P11760_5};	/*Property = ScreenPixelsWidth*/
const Profile P11760 = {"11760", &P11760_4};	/*Profile ID - 11760*/
const Value V11761_1_1 = {S2, NULL};	/*Value = True*/
const Property P11761_1 = {S50, 0, &V11761_1_1, NULL};	/*Property = IsMobile*/
const Value V11761_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11761_5 = {S52, 0, &V11761_5_1222, &P11761_1};	/*Property = ScreenPixelsHeight*/
const Value V11761_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11761_4 = {S54, 0, &V11761_4_1224, &P11761_5};	/*Property = ScreenPixelsWidth*/
const Profile P11761 = {"11761", &P11761_4};	/*Profile ID - 11761*/
const Value V11762_1_1 = {S2, NULL};	/*Value = True*/
const Property P11762_1 = {S50, 0, &V11762_1_1, NULL};	/*Property = IsMobile*/
const Value V11762_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11762_5 = {S52, 0, &V11762_5_1233, &P11762_1};	/*Property = ScreenPixelsHeight*/
const Value V11762_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11762_4 = {S54, 0, &V11762_4_1235, &P11762_5};	/*Property = ScreenPixelsWidth*/
const Profile P11762 = {"11762", &P11762_4};	/*Profile ID - 11762*/
const Value V11874_1_1 = {S2, NULL};	/*Value = True*/
const Property P11874_1 = {S50, 0, &V11874_1_1, NULL};	/*Property = IsMobile*/
const Value V11874_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11874_5 = {S52, 0, &V11874_5_1222, &P11874_1};	/*Property = ScreenPixelsHeight*/
const Value V11874_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11874_4 = {S54, 0, &V11874_4_1224, &P11874_5};	/*Property = ScreenPixelsWidth*/
const Profile P11874 = {"11874", &P11874_4};	/*Profile ID - 11874*/
const Value V17810_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_279 = {S1, 0, &V17810_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17810_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_282 = {S3, 0, &V17810_282_57155, &P17810_279};	/*Property = BlobBuilder*/
const Value V17810_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_350 = {S4, 0, &V17810_350_57446, &P17810_282};	/*Property = Canvas*/
const Value V17810_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_283 = {S5, 0, &V17810_283_57161, &P17810_350};	/*Property = CssBackground*/
const Value V17810_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_284 = {S6, 0, &V17810_284_57164, &P17810_283};	/*Property = CssBorderImage*/
const Value V17810_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_285 = {S7, 0, &V17810_285_57167, &P17810_284};	/*Property = CssCanvas*/
const Value V17810_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_286 = {S8, 0, &V17810_286_57170, &P17810_285};	/*Property = CssColor*/
const Value V17810_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_237 = {S9, 0, &V17810_237_55786, &P17810_286};	/*Property = CssColumn*/
const Value V17810_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_287 = {S10, 0, &V17810_287_57173, &P17810_237};	/*Property = CssFlexbox*/
const Value V17810_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_288 = {S11, 0, &V17810_288_57176, &P17810_287};	/*Property = CssFont*/
const Value V17810_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_289 = {S12, 0, &V17810_289_86899, &P17810_288};	/*Property = CssImages*/
const Value V17810_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_291 = {S13, 0, &V17810_291_57182, &P17810_289};	/*Property = CssMediaQueries*/
const Value V17810_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_292 = {S14, 0, &V17810_292_57185, &P17810_291};	/*Property = CssMinMax*/
const Value V17810_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_293 = {S15, 0, &V17810_293_57188, &P17810_292};	/*Property = CssOverflow*/
const Value V17810_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_294 = {S16, 0, &V17810_294_57191, &P17810_293};	/*Property = CssPosition*/
const Value V17810_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_295 = {S17, 0, &V17810_295_57194, &P17810_294};	/*Property = CssText*/
const Value V17810_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_297 = {S18, 0, &V17810_297_57197, &P17810_295};	/*Property = CssTransforms*/
const Value V17810_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_241 = {S19, 0, &V17810_241_55801, &P17810_297};	/*Property = CssTransitions*/
const Value V17810_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_298 = {S20, 0, &V17810_298_57208, &P17810_241};	/*Property = CssUI*/
const Value V17810_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_300 = {S21, 0, &V17810_300_57217, &P17810_298};	/*Property = DataSet*/
const Value V17810_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_301 = {S22, 0, &V17810_301_57220, &P17810_300};	/*Property = DataUrl*/
const Value V17810_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_302 = {S23, 0, &V17810_302_57223, &P17810_301};	/*Property = DeviceOrientation*/
const Value V17810_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_304 = {S24, 0, &V17810_304_57229, &P17810_302};	/*Property = FileReader*/
const Value V17810_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_305 = {S25, 0, &V17810_305_57232, &P17810_304};	/*Property = FileSaver*/
const Value V17810_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_306 = {S26, 0, &V17810_306_57238, &P17810_305};	/*Property = FileWriter*/
const Value V17810_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_307 = {S27, 0, &V17810_307_57241, &P17810_306};	/*Property = FormData*/
const Value V17810_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_309 = {S28, 0, &V17810_309_57247, &P17810_307};	/*Property = Fullscreen*/
const Value V17810_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_310 = {S29, 0, &V17810_310_57250, &P17810_309};	/*Property = GeoLocation*/
const Value V17810_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_312 = {S30, 0, &V17810_312_57256, &P17810_310};	/*Property = History*/
const Value V17810_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_314 = {S31, 0, &V17810_314_57262, &P17810_312};	/*Property = Html5*/
const Value V17810_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_313 = {S32, 0, &V17810_313_57259, &P17810_314};	/*Property = Html-Media-Capture*/
const Value V17810_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_315 = {S33, 0, &V17810_315_57265, &P17810_313};	/*Property = Iframe*/
const Value V17810_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_316 = {S34, 0, &V17810_316_57268, &P17810_315};	/*Property = IndexedDB*/
const Value V17810_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_317 = {S35, 0, &V17810_317_57271, &P17810_316};	/*Property = Json*/
const Value V17810_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_15 = {S37, 0, &V17810_15_72, &P17810_317};	/*Property = LayoutEngine*/
const Value V17810_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_344 = {S38, 0, &V17810_344_57274, &P17810_15};	/*Property = Masking*/
const Value V17810_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_320 = {S39, 0, &V17810_320_57292, &P17810_344};	/*Property = PostMessage*/
const Value V17810_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_321 = {S40, 0, &V17810_321_57295, &P17810_320};	/*Property = Progress*/
const Value V17810_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_322 = {S41, 0, &V17810_322_57298, &P17810_321};	/*Property = Prompts*/
const Value V17810_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_323 = {S42, 0, &V17810_323_57301, &P17810_322};	/*Property = Selector*/
const Value V17810_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_324 = {S43, 0, &V17810_324_57307, &P17810_323};	/*Property = Svg*/
const Value V17810_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_327 = {S44, 0, &V17810_327_57316, &P17810_324};	/*Property = TouchEvents*/
const Value V17810_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_328 = {S45, 0, &V17810_328_57319, &P17810_327};	/*Property = Track*/
const Value V17810_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_329 = {S46, 0, &V17810_329_57325, &P17810_328};	/*Property = Video*/
const Value V17810_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_330 = {S47, 0, &V17810_330_57328, &P17810_329};	/*Property = Viewport*/
const Value V17810_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_332 = {S48, 0, &V17810_332_57340, &P17810_330};	/*Property = WebWorkers*/
const Value V17810_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17810_333 = {S49, 0, &V17810_333_57343, &P17810_332};	/*Property = Xhr2*/
const Profile P17810 = {"17810", &P17810_333};	/*Profile ID - 17810*/
const Value V13507_1_1 = {S2, NULL};	/*Value = True*/
const Property P13507_1 = {S50, 0, &V13507_1_1, NULL};	/*Property = IsMobile*/
const Value V13507_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13507_5 = {S52, 0, &V13507_5_1222, &P13507_1};	/*Property = ScreenPixelsHeight*/
const Value V13507_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13507_4 = {S54, 0, &V13507_4_1224, &P13507_5};	/*Property = ScreenPixelsWidth*/
const Profile P13507 = {"13507", &P13507_4};	/*Profile ID - 13507*/
const Value V17827_1_1 = {S2, NULL};	/*Value = True*/
const Property P17827_1 = {S50, 0, &V17827_1_1, NULL};	/*Property = IsMobile*/
const Value V17827_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17827_5 = {S52, 0, &V17827_5_1222, &P17827_1};	/*Property = ScreenPixelsHeight*/
const Value V17827_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17827_4 = {S54, 0, &V17827_4_1224, &P17827_5};	/*Property = ScreenPixelsWidth*/
const Profile P17827 = {"17827", &P17827_4};	/*Profile ID - 17827*/
const Value V13509_1_1 = {S2, NULL};	/*Value = True*/
const Property P13509_1 = {S50, 0, &V13509_1_1, NULL};	/*Property = IsMobile*/
const Value V13509_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13509_5 = {S52, 0, &V13509_5_1227, &P13509_1};	/*Property = ScreenPixelsHeight*/
const Value V13509_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13509_4 = {S54, 0, &V13509_4_1229, &P13509_5};	/*Property = ScreenPixelsWidth*/
const Profile P13509 = {"13509", &P13509_4};	/*Profile ID - 13509*/
const Value V11810_1_1 = {S2, NULL};	/*Value = True*/
const Property P11810_1 = {S50, 0, &V11810_1_1, NULL};	/*Property = IsMobile*/
const Value V11810_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P11810_5 = {S52, 0, &V11810_5_1294, &P11810_1};	/*Property = ScreenPixelsHeight*/
const Value V11810_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11810_4 = {S54, 0, &V11810_4_1296, &P11810_5};	/*Property = ScreenPixelsWidth*/
const Profile P11810 = {"11810", &P11810_4};	/*Profile ID - 11810*/
const Profile P5282 = {"5282", NULL};	/*Profile ID - 5282*/
const Value V3579_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_279 = {S1, 0, &V3579_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3579_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_282 = {S3, 0, &V3579_282_57155, &P3579_279};	/*Property = BlobBuilder*/
const Value V3579_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_350 = {S4, 0, &V3579_350_57446, &P3579_282};	/*Property = Canvas*/
const Value V3579_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_283 = {S5, 0, &V3579_283_57161, &P3579_350};	/*Property = CssBackground*/
const Value V3579_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_284 = {S6, 0, &V3579_284_57164, &P3579_283};	/*Property = CssBorderImage*/
const Value V3579_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_285 = {S7, 0, &V3579_285_57167, &P3579_284};	/*Property = CssCanvas*/
const Value V3579_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_286 = {S8, 0, &V3579_286_57170, &P3579_285};	/*Property = CssColor*/
const Value V3579_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_237 = {S9, 0, &V3579_237_55786, &P3579_286};	/*Property = CssColumn*/
const Value V3579_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_287 = {S10, 0, &V3579_287_57173, &P3579_237};	/*Property = CssFlexbox*/
const Value V3579_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_288 = {S11, 0, &V3579_288_57176, &P3579_287};	/*Property = CssFont*/
const Value V3579_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_289 = {S12, 0, &V3579_289_86899, &P3579_288};	/*Property = CssImages*/
const Value V3579_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_291 = {S13, 0, &V3579_291_57182, &P3579_289};	/*Property = CssMediaQueries*/
const Value V3579_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_292 = {S14, 0, &V3579_292_57185, &P3579_291};	/*Property = CssMinMax*/
const Value V3579_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_293 = {S15, 0, &V3579_293_57188, &P3579_292};	/*Property = CssOverflow*/
const Value V3579_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_294 = {S16, 0, &V3579_294_57191, &P3579_293};	/*Property = CssPosition*/
const Value V3579_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_295 = {S17, 0, &V3579_295_57194, &P3579_294};	/*Property = CssText*/
const Value V3579_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_297 = {S18, 0, &V3579_297_57197, &P3579_295};	/*Property = CssTransforms*/
const Value V3579_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_241 = {S19, 0, &V3579_241_55801, &P3579_297};	/*Property = CssTransitions*/
const Value V3579_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_298 = {S20, 0, &V3579_298_57208, &P3579_241};	/*Property = CssUI*/
const Value V3579_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_300 = {S21, 0, &V3579_300_57217, &P3579_298};	/*Property = DataSet*/
const Value V3579_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_301 = {S22, 0, &V3579_301_57220, &P3579_300};	/*Property = DataUrl*/
const Value V3579_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_302 = {S23, 0, &V3579_302_57223, &P3579_301};	/*Property = DeviceOrientation*/
const Value V3579_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_304 = {S24, 0, &V3579_304_57229, &P3579_302};	/*Property = FileReader*/
const Value V3579_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_305 = {S25, 0, &V3579_305_57232, &P3579_304};	/*Property = FileSaver*/
const Value V3579_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_306 = {S26, 0, &V3579_306_57238, &P3579_305};	/*Property = FileWriter*/
const Value V3579_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_307 = {S27, 0, &V3579_307_57241, &P3579_306};	/*Property = FormData*/
const Value V3579_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_309 = {S28, 0, &V3579_309_57247, &P3579_307};	/*Property = Fullscreen*/
const Value V3579_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_310 = {S29, 0, &V3579_310_57250, &P3579_309};	/*Property = GeoLocation*/
const Value V3579_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_312 = {S30, 0, &V3579_312_57256, &P3579_310};	/*Property = History*/
const Value V3579_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_314 = {S31, 0, &V3579_314_57262, &P3579_312};	/*Property = Html5*/
const Value V3579_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_313 = {S32, 0, &V3579_313_57259, &P3579_314};	/*Property = Html-Media-Capture*/
const Value V3579_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_315 = {S33, 0, &V3579_315_57265, &P3579_313};	/*Property = Iframe*/
const Value V3579_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_316 = {S34, 0, &V3579_316_57268, &P3579_315};	/*Property = IndexedDB*/
const Value V3579_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_317 = {S35, 0, &V3579_317_57271, &P3579_316};	/*Property = Json*/
const Value V3579_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_15 = {S37, 0, &V3579_15_72, &P3579_317};	/*Property = LayoutEngine*/
const Value V3579_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_344 = {S38, 0, &V3579_344_57274, &P3579_15};	/*Property = Masking*/
const Value V3579_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_320 = {S39, 0, &V3579_320_57292, &P3579_344};	/*Property = PostMessage*/
const Value V3579_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_321 = {S40, 0, &V3579_321_57295, &P3579_320};	/*Property = Progress*/
const Value V3579_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_322 = {S41, 0, &V3579_322_57298, &P3579_321};	/*Property = Prompts*/
const Value V3579_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_323 = {S42, 0, &V3579_323_57301, &P3579_322};	/*Property = Selector*/
const Value V3579_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_324 = {S43, 0, &V3579_324_57307, &P3579_323};	/*Property = Svg*/
const Value V3579_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_327 = {S44, 0, &V3579_327_57316, &P3579_324};	/*Property = TouchEvents*/
const Value V3579_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_328 = {S45, 0, &V3579_328_57319, &P3579_327};	/*Property = Track*/
const Value V3579_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_329 = {S46, 0, &V3579_329_57325, &P3579_328};	/*Property = Video*/
const Value V3579_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_330 = {S47, 0, &V3579_330_57328, &P3579_329};	/*Property = Viewport*/
const Value V3579_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_332 = {S48, 0, &V3579_332_57340, &P3579_330};	/*Property = WebWorkers*/
const Value V3579_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3579_333 = {S49, 0, &V3579_333_57343, &P3579_332};	/*Property = Xhr2*/
const Profile P3579 = {"3579", &P3579_333};	/*Profile ID - 3579*/
const Value V13510_1_1 = {S2, NULL};	/*Value = True*/
const Property P13510_1 = {S50, 0, &V13510_1_1, NULL};	/*Property = IsMobile*/
const Value V13510_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13510_5 = {S52, 0, &V13510_5_1222, &P13510_1};	/*Property = ScreenPixelsHeight*/
const Value V13510_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13510_4 = {S54, 0, &V13510_4_1224, &P13510_5};	/*Property = ScreenPixelsWidth*/
const Profile P13510 = {"13510", &P13510_4};	/*Profile ID - 13510*/
const Value V11763_1_1 = {S2, NULL};	/*Value = True*/
const Property P11763_1 = {S50, 0, &V11763_1_1, NULL};	/*Property = IsMobile*/
const Value V11763_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11763_5 = {S52, 0, &V11763_5_1222, &P11763_1};	/*Property = ScreenPixelsHeight*/
const Value V11763_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11763_4 = {S54, 0, &V11763_4_1224, &P11763_5};	/*Property = ScreenPixelsWidth*/
const Profile P11763 = {"11763", &P11763_4};	/*Profile ID - 11763*/
const Value V11769_1_1 = {S2, NULL};	/*Value = True*/
const Property P11769_1 = {S50, 0, &V11769_1_1, NULL};	/*Property = IsMobile*/
const Value V11769_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P11769_5 = {S52, 0, &V11769_5_1333, &P11769_1};	/*Property = ScreenPixelsHeight*/
const Value V11769_4_1368 = {S74, NULL};	/*Value = 208*/
const Property P11769_4 = {S54, 0, &V11769_4_1368, &P11769_5};	/*Property = ScreenPixelsWidth*/
const Profile P11769 = {"11769", &P11769_4};	/*Profile ID - 11769*/
const Value V11770_1_1 = {S2, NULL};	/*Value = True*/
const Property P11770_1 = {S50, 0, &V11770_1_1, NULL};	/*Property = IsMobile*/
const Value V11770_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P11770_5 = {S52, 0, &V11770_5_1294, &P11770_1};	/*Property = ScreenPixelsHeight*/
const Value V11770_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11770_4 = {S54, 0, &V11770_4_1296, &P11770_5};	/*Property = ScreenPixelsWidth*/
const Profile P11770 = {"11770", &P11770_4};	/*Profile ID - 11770*/
const Value V13557_1_1 = {S2, NULL};	/*Value = True*/
const Property P13557_1 = {S50, 0, &V13557_1_1, NULL};	/*Property = IsMobile*/
const Value V13557_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13557_5 = {S52, 0, &V13557_5_1293, &P13557_1};	/*Property = ScreenPixelsHeight*/
const Value V13557_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13557_4 = {S54, 0, &V13557_4_1296, &P13557_5};	/*Property = ScreenPixelsWidth*/
const Profile P13557 = {"13557", &P13557_4};	/*Profile ID - 13557*/
const Value V12727_1_1 = {S2, NULL};	/*Value = True*/
const Property P12727_1 = {S50, 0, &V12727_1_1, NULL};	/*Property = IsMobile*/
const Value V12727_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12727_5 = {S52, 0, &V12727_5_1293, &P12727_1};	/*Property = ScreenPixelsHeight*/
const Value V12727_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12727_4 = {S54, 0, &V12727_4_1296, &P12727_5};	/*Property = ScreenPixelsWidth*/
const Profile P12727 = {"12727", &P12727_4};	/*Profile ID - 12727*/
const Value V11772_1_1 = {S2, NULL};	/*Value = True*/
const Property P11772_1 = {S50, 0, &V11772_1_1, NULL};	/*Property = IsMobile*/
const Value V11772_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11772_5 = {S52, 0, &V11772_5_1222, &P11772_1};	/*Property = ScreenPixelsHeight*/
const Value V11772_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11772_4 = {S54, 0, &V11772_4_1224, &P11772_5};	/*Property = ScreenPixelsWidth*/
const Profile P11772 = {"11772", &P11772_4};	/*Profile ID - 11772*/
const Value V11775_1_1 = {S2, NULL};	/*Value = True*/
const Property P11775_1 = {S50, 0, &V11775_1_1, NULL};	/*Property = IsMobile*/
const Value V11775_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11775_5 = {S52, 0, &V11775_5_1233, &P11775_1};	/*Property = ScreenPixelsHeight*/
const Value V11775_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11775_4 = {S54, 0, &V11775_4_1235, &P11775_5};	/*Property = ScreenPixelsWidth*/
const Profile P11775 = {"11775", &P11775_4};	/*Profile ID - 11775*/
const Value V14848_1_1 = {S2, NULL};	/*Value = True*/
const Property P14848_1 = {S50, 0, &V14848_1_1, NULL};	/*Property = IsMobile*/
const Value V14848_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14848_5 = {S52, 0, &V14848_5_1222, &P14848_1};	/*Property = ScreenPixelsHeight*/
const Value V14848_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14848_4 = {S54, 0, &V14848_4_1224, &P14848_5};	/*Property = ScreenPixelsWidth*/
const Profile P14848 = {"14848", &P14848_4};	/*Profile ID - 14848*/
const Value V11781_1_1 = {S2, NULL};	/*Value = True*/
const Property P11781_1 = {S50, 0, &V11781_1_1, NULL};	/*Property = IsMobile*/
const Value V11781_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11781_5 = {S52, 0, &V11781_5_1293, &P11781_1};	/*Property = ScreenPixelsHeight*/
const Value V11781_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11781_4 = {S54, 0, &V11781_4_1296, &P11781_5};	/*Property = ScreenPixelsWidth*/
const Profile P11781 = {"11781", &P11781_4};	/*Profile ID - 11781*/
const Value V11782_1_1 = {S2, NULL};	/*Value = True*/
const Property P11782_1 = {S50, 0, &V11782_1_1, NULL};	/*Property = IsMobile*/
const Value V11782_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11782_5 = {S52, 0, &V11782_5_1233, &P11782_1};	/*Property = ScreenPixelsHeight*/
const Value V11782_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11782_4 = {S54, 0, &V11782_4_1235, &P11782_5};	/*Property = ScreenPixelsWidth*/
const Profile P11782 = {"11782", &P11782_4};	/*Profile ID - 11782*/
const Value V14460_1_1 = {S2, NULL};	/*Value = True*/
const Property P14460_1 = {S50, 0, &V14460_1_1, NULL};	/*Property = IsMobile*/
const Value V14460_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14460_5 = {S52, 0, &V14460_5_1293, &P14460_1};	/*Property = ScreenPixelsHeight*/
const Value V14460_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14460_4 = {S54, 0, &V14460_4_1296, &P14460_5};	/*Property = ScreenPixelsWidth*/
const Profile P14460 = {"14460", &P14460_4};	/*Profile ID - 14460*/
const Value V11783_1_1 = {S2, NULL};	/*Value = True*/
const Property P11783_1 = {S50, 0, &V11783_1_1, NULL};	/*Property = IsMobile*/
const Value V11783_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11783_5 = {S52, 0, &V11783_5_1293, &P11783_1};	/*Property = ScreenPixelsHeight*/
const Value V11783_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11783_4 = {S54, 0, &V11783_4_1296, &P11783_5};	/*Property = ScreenPixelsWidth*/
const Profile P11783 = {"11783", &P11783_4};	/*Profile ID - 11783*/
const Value V11784_1_1 = {S2, NULL};	/*Value = True*/
const Property P11784_1 = {S50, 0, &V11784_1_1, NULL};	/*Property = IsMobile*/
const Value V11784_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11784_5 = {S52, 0, &V11784_5_1222, &P11784_1};	/*Property = ScreenPixelsHeight*/
const Value V11784_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11784_4 = {S54, 0, &V11784_4_1224, &P11784_5};	/*Property = ScreenPixelsWidth*/
const Profile P11784 = {"11784", &P11784_4};	/*Profile ID - 11784*/
const Value V13197_1_1 = {S2, NULL};	/*Value = True*/
const Property P13197_1 = {S50, 0, &V13197_1_1, NULL};	/*Property = IsMobile*/
const Value V13197_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13197_5 = {S52, 0, &V13197_5_1222, &P13197_1};	/*Property = ScreenPixelsHeight*/
const Value V13197_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13197_4 = {S54, 0, &V13197_4_1224, &P13197_5};	/*Property = ScreenPixelsWidth*/
const Profile P13197 = {"13197", &P13197_4};	/*Profile ID - 13197*/
const Value V11787_1_1 = {S2, NULL};	/*Value = True*/
const Property P11787_1 = {S50, 0, &V11787_1_1, NULL};	/*Property = IsMobile*/
const Value V11787_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11787_5 = {S52, 0, &V11787_5_1222, &P11787_1};	/*Property = ScreenPixelsHeight*/
const Value V11787_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11787_4 = {S54, 0, &V11787_4_1224, &P11787_5};	/*Property = ScreenPixelsWidth*/
const Profile P11787 = {"11787", &P11787_4};	/*Profile ID - 11787*/
const Value V12009_1_1 = {S2, NULL};	/*Value = True*/
const Property P12009_1 = {S50, 0, &V12009_1_1, NULL};	/*Property = IsMobile*/
const Value V12009_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12009_5 = {S52, 0, &V12009_5_1222, &P12009_1};	/*Property = ScreenPixelsHeight*/
const Value V12009_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12009_4 = {S54, 0, &V12009_4_1224, &P12009_5};	/*Property = ScreenPixelsWidth*/
const Profile P12009 = {"12009", &P12009_4};	/*Profile ID - 12009*/
const Value V11788_1_1 = {S2, NULL};	/*Value = True*/
const Property P11788_1 = {S50, 0, &V11788_1_1, NULL};	/*Property = IsMobile*/
const Value V11788_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11788_5 = {S52, 0, &V11788_5_1293, &P11788_1};	/*Property = ScreenPixelsHeight*/
const Value V11788_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11788_4 = {S54, 0, &V11788_4_1296, &P11788_5};	/*Property = ScreenPixelsWidth*/
const Profile P11788 = {"11788", &P11788_4};	/*Profile ID - 11788*/
const Value V11790_1_1 = {S2, NULL};	/*Value = True*/
const Property P11790_1 = {S50, 0, &V11790_1_1, NULL};	/*Property = IsMobile*/
const Value V11790_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11790_5 = {S52, 0, &V11790_5_1222, &P11790_1};	/*Property = ScreenPixelsHeight*/
const Value V11790_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11790_4 = {S54, 0, &V11790_4_1224, &P11790_5};	/*Property = ScreenPixelsWidth*/
const Profile P11790 = {"11790", &P11790_4};	/*Profile ID - 11790*/
const Value V11791_1_1 = {S2, NULL};	/*Value = True*/
const Property P11791_1 = {S50, 0, &V11791_1_1, NULL};	/*Property = IsMobile*/
const Value V11791_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11791_5 = {S52, 0, &V11791_5_1222, &P11791_1};	/*Property = ScreenPixelsHeight*/
const Value V11791_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11791_4 = {S54, 0, &V11791_4_1224, &P11791_5};	/*Property = ScreenPixelsWidth*/
const Profile P11791 = {"11791", &P11791_4};	/*Profile ID - 11791*/
const Value V11793_1_1 = {S2, NULL};	/*Value = True*/
const Property P11793_1 = {S50, 0, &V11793_1_1, NULL};	/*Property = IsMobile*/
const Value V11793_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11793_5 = {S52, 0, &V11793_5_1293, &P11793_1};	/*Property = ScreenPixelsHeight*/
const Value V11793_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11793_4 = {S54, 0, &V11793_4_1296, &P11793_5};	/*Property = ScreenPixelsWidth*/
const Profile P11793 = {"11793", &P11793_4};	/*Profile ID - 11793*/
const Value V11868_1_1 = {S2, NULL};	/*Value = True*/
const Property P11868_1 = {S50, 0, &V11868_1_1, NULL};	/*Property = IsMobile*/
const Value V11868_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11868_5 = {S52, 0, &V11868_5_1222, &P11868_1};	/*Property = ScreenPixelsHeight*/
const Value V11868_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11868_4 = {S54, 0, &V11868_4_1224, &P11868_5};	/*Property = ScreenPixelsWidth*/
const Profile P11868 = {"11868", &P11868_4};	/*Profile ID - 11868*/
const Value V11811_1_1 = {S2, NULL};	/*Value = True*/
const Property P11811_1 = {S50, 0, &V11811_1_1, NULL};	/*Property = IsMobile*/
const Value V11811_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P11811_5 = {S52, 0, &V11811_5_1304, &P11811_1};	/*Property = ScreenPixelsHeight*/
const Value V11811_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11811_4 = {S54, 0, &V11811_4_1224, &P11811_5};	/*Property = ScreenPixelsWidth*/
const Profile P11811 = {"11811", &P11811_4};	/*Profile ID - 11811*/
const Value V13796_1_1 = {S2, NULL};	/*Value = True*/
const Property P13796_1 = {S50, 0, &V13796_1_1, NULL};	/*Property = IsMobile*/
const Value V13796_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13796_5 = {S52, 0, &V13796_5_1222, &P13796_1};	/*Property = ScreenPixelsHeight*/
const Value V13796_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13796_4 = {S54, 0, &V13796_4_1224, &P13796_5};	/*Property = ScreenPixelsWidth*/
const Profile P13796 = {"13796", &P13796_4};	/*Profile ID - 13796*/
const Value V13798_1_1 = {S2, NULL};	/*Value = True*/
const Property P13798_1 = {S50, 0, &V13798_1_1, NULL};	/*Property = IsMobile*/
const Value V13798_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13798_5 = {S52, 0, &V13798_5_1277, &P13798_1};	/*Property = ScreenPixelsHeight*/
const Value V13798_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P13798_4 = {S54, 0, &V13798_4_8554, &P13798_5};	/*Property = ScreenPixelsWidth*/
const Profile P13798 = {"13798", &P13798_4};	/*Profile ID - 13798*/
const Value V17923_1_1 = {S2, NULL};	/*Value = True*/
const Property P17923_1 = {S50, 0, &V17923_1_1, NULL};	/*Property = IsMobile*/
const Value V17923_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17923_5 = {S52, 0, &V17923_5_1277, &P17923_1};	/*Property = ScreenPixelsHeight*/
const Value V17923_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P17923_4 = {S54, 0, &V17923_4_8554, &P17923_5};	/*Property = ScreenPixelsWidth*/
const Profile P17923 = {"17923", &P17923_4};	/*Profile ID - 17923*/
const Value V11947_1_1 = {S2, NULL};	/*Value = True*/
const Property P11947_1 = {S50, 0, &V11947_1_1, NULL};	/*Property = IsMobile*/
const Value V11947_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11947_5 = {S52, 0, &V11947_5_1233, &P11947_1};	/*Property = ScreenPixelsHeight*/
const Value V11947_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11947_4 = {S54, 0, &V11947_4_1235, &P11947_5};	/*Property = ScreenPixelsWidth*/
const Profile P11947 = {"11947", &P11947_4};	/*Profile ID - 11947*/
const Value V13804_1_1 = {S2, NULL};	/*Value = True*/
const Property P13804_1 = {S50, 0, &V13804_1_1, NULL};	/*Property = IsMobile*/
const Value V13804_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13804_5 = {S52, 0, &V13804_5_1293, &P13804_1};	/*Property = ScreenPixelsHeight*/
const Value V13804_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13804_4 = {S54, 0, &V13804_4_1296, &P13804_5};	/*Property = ScreenPixelsWidth*/
const Profile P13804 = {"13804", &P13804_4};	/*Profile ID - 13804*/
const Value V12990_1_1 = {S2, NULL};	/*Value = True*/
const Property P12990_1 = {S50, 0, &V12990_1_1, NULL};	/*Property = IsMobile*/
const Value V12990_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12990_5 = {S52, 0, &V12990_5_1222, &P12990_1};	/*Property = ScreenPixelsHeight*/
const Value V12990_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12990_4 = {S54, 0, &V12990_4_1224, &P12990_5};	/*Property = ScreenPixelsWidth*/
const Profile P12990 = {"12990", &P12990_4};	/*Profile ID - 12990*/
const Value V13808_1_1 = {S2, NULL};	/*Value = True*/
const Property P13808_1 = {S50, 0, &V13808_1_1, NULL};	/*Property = IsMobile*/
const Value V13808_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13808_5 = {S52, 0, &V13808_5_1233, &P13808_1};	/*Property = ScreenPixelsHeight*/
const Value V13808_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13808_4 = {S54, 0, &V13808_4_1235, &P13808_5};	/*Property = ScreenPixelsWidth*/
const Profile P13808 = {"13808", &P13808_4};	/*Profile ID - 13808*/
const Value V13806_1_1 = {S2, NULL};	/*Value = True*/
const Property P13806_1 = {S50, 0, &V13806_1_1, NULL};	/*Property = IsMobile*/
const Value V13806_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13806_5 = {S52, 0, &V13806_5_1222, &P13806_1};	/*Property = ScreenPixelsHeight*/
const Value V13806_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13806_4 = {S54, 0, &V13806_4_1224, &P13806_5};	/*Property = ScreenPixelsWidth*/
const Profile P13806 = {"13806", &P13806_4};	/*Profile ID - 13806*/
const Value V11796_1_1 = {S2, NULL};	/*Value = True*/
const Property P11796_1 = {S50, 0, &V11796_1_1, NULL};	/*Property = IsMobile*/
const Value V11796_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11796_5 = {S52, 0, &V11796_5_1293, &P11796_1};	/*Property = ScreenPixelsHeight*/
const Value V11796_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11796_4 = {S54, 0, &V11796_4_1296, &P11796_5};	/*Property = ScreenPixelsWidth*/
const Profile P11796 = {"11796", &P11796_4};	/*Profile ID - 11796*/
const Value V13817_1_1 = {S2, NULL};	/*Value = True*/
const Property P13817_1 = {S50, 0, &V13817_1_1, NULL};	/*Property = IsMobile*/
const Value V13817_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13817_5 = {S52, 0, &V13817_5_1233, &P13817_1};	/*Property = ScreenPixelsHeight*/
const Value V13817_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13817_4 = {S54, 0, &V13817_4_1235, &P13817_5};	/*Property = ScreenPixelsWidth*/
const Profile P13817 = {"13817", &P13817_4};	/*Profile ID - 13817*/
const Value V14256_1_1 = {S2, NULL};	/*Value = True*/
const Property P14256_1 = {S50, 0, &V14256_1_1, NULL};	/*Property = IsMobile*/
const Value V14256_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14256_5 = {S52, 0, &V14256_5_1293, &P14256_1};	/*Property = ScreenPixelsHeight*/
const Value V14256_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14256_4 = {S54, 0, &V14256_4_1296, &P14256_5};	/*Property = ScreenPixelsWidth*/
const Profile P14256 = {"14256", &P14256_4};	/*Profile ID - 14256*/
const Value V11864_1_1 = {S2, NULL};	/*Value = True*/
const Property P11864_1 = {S50, 0, &V11864_1_1, NULL};	/*Property = IsMobile*/
const Value V11864_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11864_5 = {S52, 0, &V11864_5_1293, &P11864_1};	/*Property = ScreenPixelsHeight*/
const Value V11864_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11864_4 = {S54, 0, &V11864_4_1296, &P11864_5};	/*Property = ScreenPixelsWidth*/
const Profile P11864 = {"11864", &P11864_4};	/*Profile ID - 11864*/
const Value V13825_1_1 = {S2, NULL};	/*Value = True*/
const Property P13825_1 = {S50, 0, &V13825_1_1, NULL};	/*Property = IsMobile*/
const Value V13825_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13825_5 = {S52, 0, &V13825_5_1293, &P13825_1};	/*Property = ScreenPixelsHeight*/
const Value V13825_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13825_4 = {S54, 0, &V13825_4_1296, &P13825_5};	/*Property = ScreenPixelsWidth*/
const Profile P13825 = {"13825", &P13825_4};	/*Profile ID - 13825*/
const Value V13828_1_1 = {S2, NULL};	/*Value = True*/
const Property P13828_1 = {S50, 0, &V13828_1_1, NULL};	/*Property = IsMobile*/
const Value V13828_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13828_5 = {S52, 0, &V13828_5_1222, &P13828_1};	/*Property = ScreenPixelsHeight*/
const Value V13828_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13828_4 = {S54, 0, &V13828_4_1224, &P13828_5};	/*Property = ScreenPixelsWidth*/
const Profile P13828 = {"13828", &P13828_4};	/*Profile ID - 13828*/
const Value V14545_1_1 = {S2, NULL};	/*Value = True*/
const Property P14545_1 = {S50, 0, &V14545_1_1, NULL};	/*Property = IsMobile*/
const Value V14545_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14545_5 = {S52, 0, &V14545_5_1380, &P14545_1};	/*Property = ScreenPixelsHeight*/
const Value V14545_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14545_4 = {S54, 0, &V14545_4_1279, &P14545_5};	/*Property = ScreenPixelsWidth*/
const Profile P14545 = {"14545", &P14545_4};	/*Profile ID - 14545*/
const Value V14137_1_1 = {S2, NULL};	/*Value = True*/
const Property P14137_1 = {S50, 0, &V14137_1_1, NULL};	/*Property = IsMobile*/
const Value V14137_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14137_5 = {S52, 0, &V14137_5_1222, &P14137_1};	/*Property = ScreenPixelsHeight*/
const Value V14137_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14137_4 = {S54, 0, &V14137_4_1224, &P14137_5};	/*Property = ScreenPixelsWidth*/
const Profile P14137 = {"14137", &P14137_4};	/*Profile ID - 14137*/
const Value V13832_1_1 = {S2, NULL};	/*Value = True*/
const Property P13832_1 = {S50, 0, &V13832_1_1, NULL};	/*Property = IsMobile*/
const Value V13832_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13832_5 = {S52, 0, &V13832_5_1304, &P13832_1};	/*Property = ScreenPixelsHeight*/
const Value V13832_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13832_4 = {S54, 0, &V13832_4_1224, &P13832_5};	/*Property = ScreenPixelsWidth*/
const Profile P13832 = {"13832", &P13832_4};	/*Profile ID - 13832*/
const Value V13835_1_1 = {S2, NULL};	/*Value = True*/
const Property P13835_1 = {S50, 0, &V13835_1_1, NULL};	/*Property = IsMobile*/
const Value V13835_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13835_5 = {S52, 0, &V13835_5_1233, &P13835_1};	/*Property = ScreenPixelsHeight*/
const Value V13835_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13835_4 = {S54, 0, &V13835_4_1235, &P13835_5};	/*Property = ScreenPixelsWidth*/
const Profile P13835 = {"13835", &P13835_4};	/*Profile ID - 13835*/
const Value V4615_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_279 = {S1, 0, &V4615_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4615_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_282 = {S3, 0, &V4615_282_57155, &P4615_279};	/*Property = BlobBuilder*/
const Value V4615_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_350 = {S4, 0, &V4615_350_57446, &P4615_282};	/*Property = Canvas*/
const Value V4615_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_283 = {S5, 0, &V4615_283_57161, &P4615_350};	/*Property = CssBackground*/
const Value V4615_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_284 = {S6, 0, &V4615_284_57164, &P4615_283};	/*Property = CssBorderImage*/
const Value V4615_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_285 = {S7, 0, &V4615_285_57167, &P4615_284};	/*Property = CssCanvas*/
const Value V4615_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_286 = {S8, 0, &V4615_286_57170, &P4615_285};	/*Property = CssColor*/
const Value V4615_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_237 = {S9, 0, &V4615_237_55786, &P4615_286};	/*Property = CssColumn*/
const Value V4615_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_287 = {S10, 0, &V4615_287_57173, &P4615_237};	/*Property = CssFlexbox*/
const Value V4615_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_288 = {S11, 0, &V4615_288_57176, &P4615_287};	/*Property = CssFont*/
const Value V4615_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_289 = {S12, 0, &V4615_289_86899, &P4615_288};	/*Property = CssImages*/
const Value V4615_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_291 = {S13, 0, &V4615_291_57182, &P4615_289};	/*Property = CssMediaQueries*/
const Value V4615_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_292 = {S14, 0, &V4615_292_57185, &P4615_291};	/*Property = CssMinMax*/
const Value V4615_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_293 = {S15, 0, &V4615_293_57188, &P4615_292};	/*Property = CssOverflow*/
const Value V4615_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_294 = {S16, 0, &V4615_294_57191, &P4615_293};	/*Property = CssPosition*/
const Value V4615_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_295 = {S17, 0, &V4615_295_57194, &P4615_294};	/*Property = CssText*/
const Value V4615_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_297 = {S18, 0, &V4615_297_57197, &P4615_295};	/*Property = CssTransforms*/
const Value V4615_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_241 = {S19, 0, &V4615_241_55801, &P4615_297};	/*Property = CssTransitions*/
const Value V4615_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_298 = {S20, 0, &V4615_298_57208, &P4615_241};	/*Property = CssUI*/
const Value V4615_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_300 = {S21, 0, &V4615_300_57217, &P4615_298};	/*Property = DataSet*/
const Value V4615_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_301 = {S22, 0, &V4615_301_57220, &P4615_300};	/*Property = DataUrl*/
const Value V4615_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_302 = {S23, 0, &V4615_302_57223, &P4615_301};	/*Property = DeviceOrientation*/
const Value V4615_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_304 = {S24, 0, &V4615_304_57229, &P4615_302};	/*Property = FileReader*/
const Value V4615_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_305 = {S25, 0, &V4615_305_57232, &P4615_304};	/*Property = FileSaver*/
const Value V4615_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_306 = {S26, 0, &V4615_306_57238, &P4615_305};	/*Property = FileWriter*/
const Value V4615_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_307 = {S27, 0, &V4615_307_57241, &P4615_306};	/*Property = FormData*/
const Value V4615_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_309 = {S28, 0, &V4615_309_57247, &P4615_307};	/*Property = Fullscreen*/
const Value V4615_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_310 = {S29, 0, &V4615_310_57250, &P4615_309};	/*Property = GeoLocation*/
const Value V4615_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_312 = {S30, 0, &V4615_312_57256, &P4615_310};	/*Property = History*/
const Value V4615_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_314 = {S31, 0, &V4615_314_57262, &P4615_312};	/*Property = Html5*/
const Value V4615_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_313 = {S32, 0, &V4615_313_57259, &P4615_314};	/*Property = Html-Media-Capture*/
const Value V4615_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_315 = {S33, 0, &V4615_315_57265, &P4615_313};	/*Property = Iframe*/
const Value V4615_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_316 = {S34, 0, &V4615_316_57268, &P4615_315};	/*Property = IndexedDB*/
const Value V4615_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_317 = {S35, 0, &V4615_317_57271, &P4615_316};	/*Property = Json*/
const Value V4615_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P4615_15 = {S37, 0, &V4615_15_28, &P4615_317};	/*Property = LayoutEngine*/
const Value V4615_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_344 = {S38, 0, &V4615_344_57274, &P4615_15};	/*Property = Masking*/
const Value V4615_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_320 = {S39, 0, &V4615_320_57292, &P4615_344};	/*Property = PostMessage*/
const Value V4615_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_321 = {S40, 0, &V4615_321_57295, &P4615_320};	/*Property = Progress*/
const Value V4615_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_322 = {S41, 0, &V4615_322_57298, &P4615_321};	/*Property = Prompts*/
const Value V4615_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_323 = {S42, 0, &V4615_323_57301, &P4615_322};	/*Property = Selector*/
const Value V4615_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_324 = {S43, 0, &V4615_324_57307, &P4615_323};	/*Property = Svg*/
const Value V4615_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_327 = {S44, 0, &V4615_327_57316, &P4615_324};	/*Property = TouchEvents*/
const Value V4615_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_328 = {S45, 0, &V4615_328_57319, &P4615_327};	/*Property = Track*/
const Value V4615_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_329 = {S46, 0, &V4615_329_57325, &P4615_328};	/*Property = Video*/
const Value V4615_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_330 = {S47, 0, &V4615_330_57328, &P4615_329};	/*Property = Viewport*/
const Value V4615_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_332 = {S48, 0, &V4615_332_57340, &P4615_330};	/*Property = WebWorkers*/
const Value V4615_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4615_333 = {S49, 0, &V4615_333_57343, &P4615_332};	/*Property = Xhr2*/
const Profile P4615 = {"4615", &P4615_333};	/*Profile ID - 4615*/
const Value V14495_1_1 = {S2, NULL};	/*Value = True*/
const Property P14495_1 = {S50, 0, &V14495_1_1, NULL};	/*Property = IsMobile*/
const Value V14495_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P14495_5 = {S52, 0, &V14495_5_1280, &P14495_1};	/*Property = ScreenPixelsHeight*/
const Value V14495_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14495_4 = {S54, 0, &V14495_4_1235, &P14495_5};	/*Property = ScreenPixelsWidth*/
const Profile P14495 = {"14495", &P14495_4};	/*Profile ID - 14495*/
const Value V13839_1_1 = {S2, NULL};	/*Value = True*/
const Property P13839_1 = {S50, 0, &V13839_1_1, NULL};	/*Property = IsMobile*/
const Value V13839_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13839_5 = {S52, 0, &V13839_5_1222, &P13839_1};	/*Property = ScreenPixelsHeight*/
const Value V13839_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13839_4 = {S54, 0, &V13839_4_1224, &P13839_5};	/*Property = ScreenPixelsWidth*/
const Profile P13839 = {"13839", &P13839_4};	/*Profile ID - 13839*/
const Value V14831_1_1 = {S2, NULL};	/*Value = True*/
const Property P14831_1 = {S50, 0, &V14831_1_1, NULL};	/*Property = IsMobile*/
const Value V14831_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14831_5 = {S52, 0, &V14831_5_1304, &P14831_1};	/*Property = ScreenPixelsHeight*/
const Value V14831_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14831_4 = {S54, 0, &V14831_4_1224, &P14831_5};	/*Property = ScreenPixelsWidth*/
const Profile P14831 = {"14831", &P14831_4};	/*Profile ID - 14831*/
const Value V13856_1_1 = {S2, NULL};	/*Value = True*/
const Property P13856_1 = {S50, 0, &V13856_1_1, NULL};	/*Property = IsMobile*/
const Value V13856_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13856_5 = {S52, 0, &V13856_5_1304, &P13856_1};	/*Property = ScreenPixelsHeight*/
const Value V13856_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13856_4 = {S54, 0, &V13856_4_1224, &P13856_5};	/*Property = ScreenPixelsWidth*/
const Profile P13856 = {"13856", &P13856_4};	/*Profile ID - 13856*/
const Value V14851_1_1 = {S2, NULL};	/*Value = True*/
const Property P14851_1 = {S50, 0, &V14851_1_1, NULL};	/*Property = IsMobile*/
const Value V14851_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14851_5 = {S52, 0, &V14851_5_1304, &P14851_1};	/*Property = ScreenPixelsHeight*/
const Value V14851_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14851_4 = {S54, 0, &V14851_4_1224, &P14851_5};	/*Property = ScreenPixelsWidth*/
const Profile P14851 = {"14851", &P14851_4};	/*Profile ID - 14851*/
const Value V11801_1_1 = {S2, NULL};	/*Value = True*/
const Property P11801_1 = {S50, 0, &V11801_1_1, NULL};	/*Property = IsMobile*/
const Value V11801_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11801_5 = {S52, 0, &V11801_5_1222, &P11801_1};	/*Property = ScreenPixelsHeight*/
const Value V11801_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11801_4 = {S54, 0, &V11801_4_1224, &P11801_5};	/*Property = ScreenPixelsWidth*/
const Profile P11801 = {"11801", &P11801_4};	/*Profile ID - 11801*/
const Value V11802_1_1 = {S2, NULL};	/*Value = True*/
const Property P11802_1 = {S50, 0, &V11802_1_1, NULL};	/*Property = IsMobile*/
const Value V11802_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11802_5 = {S52, 0, &V11802_5_1293, &P11802_1};	/*Property = ScreenPixelsHeight*/
const Value V11802_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11802_4 = {S54, 0, &V11802_4_1296, &P11802_5};	/*Property = ScreenPixelsWidth*/
const Profile P11802 = {"11802", &P11802_4};	/*Profile ID - 11802*/
const Value V11803_1_1 = {S2, NULL};	/*Value = True*/
const Property P11803_1 = {S50, 0, &V11803_1_1, NULL};	/*Property = IsMobile*/
const Value V11803_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11803_5 = {S52, 0, &V11803_5_1293, &P11803_1};	/*Property = ScreenPixelsHeight*/
const Value V11803_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11803_4 = {S54, 0, &V11803_4_1296, &P11803_5};	/*Property = ScreenPixelsWidth*/
const Profile P11803 = {"11803", &P11803_4};	/*Profile ID - 11803*/
const Value V13512_1_1 = {S2, NULL};	/*Value = True*/
const Property P13512_1 = {S50, 0, &V13512_1_1, NULL};	/*Property = IsMobile*/
const Value V13512_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13512_5 = {S52, 0, &V13512_5_1293, &P13512_1};	/*Property = ScreenPixelsHeight*/
const Value V13512_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13512_4 = {S54, 0, &V13512_4_1296, &P13512_5};	/*Property = ScreenPixelsWidth*/
const Profile P13512 = {"13512", &P13512_4};	/*Profile ID - 13512*/
const Value V13513_1_1 = {S2, NULL};	/*Value = True*/
const Property P13513_1 = {S50, 0, &V13513_1_1, NULL};	/*Property = IsMobile*/
const Value V13513_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13513_5 = {S52, 0, &V13513_5_1293, &P13513_1};	/*Property = ScreenPixelsHeight*/
const Value V13513_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13513_4 = {S54, 0, &V13513_4_1296, &P13513_5};	/*Property = ScreenPixelsWidth*/
const Profile P13513 = {"13513", &P13513_4};	/*Profile ID - 13513*/
const Value V13514_1_1 = {S2, NULL};	/*Value = True*/
const Property P13514_1 = {S50, 0, &V13514_1_1, NULL};	/*Property = IsMobile*/
const Value V13514_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13514_5 = {S52, 0, &V13514_5_1222, &P13514_1};	/*Property = ScreenPixelsHeight*/
const Value V13514_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13514_4 = {S54, 0, &V13514_4_1224, &P13514_5};	/*Property = ScreenPixelsWidth*/
const Profile P13514 = {"13514", &P13514_4};	/*Profile ID - 13514*/
const Value V4521_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_279 = {S1, 0, &V4521_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4521_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_282 = {S3, 0, &V4521_282_57155, &P4521_279};	/*Property = BlobBuilder*/
const Value V4521_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_350 = {S4, 0, &V4521_350_57446, &P4521_282};	/*Property = Canvas*/
const Value V4521_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_283 = {S5, 0, &V4521_283_57161, &P4521_350};	/*Property = CssBackground*/
const Value V4521_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_284 = {S6, 0, &V4521_284_57164, &P4521_283};	/*Property = CssBorderImage*/
const Value V4521_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_285 = {S7, 0, &V4521_285_57167, &P4521_284};	/*Property = CssCanvas*/
const Value V4521_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_286 = {S8, 0, &V4521_286_57170, &P4521_285};	/*Property = CssColor*/
const Value V4521_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_237 = {S9, 0, &V4521_237_55786, &P4521_286};	/*Property = CssColumn*/
const Value V4521_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_287 = {S10, 0, &V4521_287_57173, &P4521_237};	/*Property = CssFlexbox*/
const Value V4521_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_288 = {S11, 0, &V4521_288_57176, &P4521_287};	/*Property = CssFont*/
const Value V4521_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_289 = {S12, 0, &V4521_289_86899, &P4521_288};	/*Property = CssImages*/
const Value V4521_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_291 = {S13, 0, &V4521_291_57182, &P4521_289};	/*Property = CssMediaQueries*/
const Value V4521_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_292 = {S14, 0, &V4521_292_57185, &P4521_291};	/*Property = CssMinMax*/
const Value V4521_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_293 = {S15, 0, &V4521_293_57188, &P4521_292};	/*Property = CssOverflow*/
const Value V4521_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_294 = {S16, 0, &V4521_294_57191, &P4521_293};	/*Property = CssPosition*/
const Value V4521_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_295 = {S17, 0, &V4521_295_57194, &P4521_294};	/*Property = CssText*/
const Value V4521_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_297 = {S18, 0, &V4521_297_57197, &P4521_295};	/*Property = CssTransforms*/
const Value V4521_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_241 = {S19, 0, &V4521_241_55801, &P4521_297};	/*Property = CssTransitions*/
const Value V4521_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_298 = {S20, 0, &V4521_298_57208, &P4521_241};	/*Property = CssUI*/
const Value V4521_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_300 = {S21, 0, &V4521_300_57217, &P4521_298};	/*Property = DataSet*/
const Value V4521_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_301 = {S22, 0, &V4521_301_57220, &P4521_300};	/*Property = DataUrl*/
const Value V4521_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_302 = {S23, 0, &V4521_302_57223, &P4521_301};	/*Property = DeviceOrientation*/
const Value V4521_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_304 = {S24, 0, &V4521_304_57229, &P4521_302};	/*Property = FileReader*/
const Value V4521_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_305 = {S25, 0, &V4521_305_57232, &P4521_304};	/*Property = FileSaver*/
const Value V4521_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_306 = {S26, 0, &V4521_306_57238, &P4521_305};	/*Property = FileWriter*/
const Value V4521_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_307 = {S27, 0, &V4521_307_57241, &P4521_306};	/*Property = FormData*/
const Value V4521_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_309 = {S28, 0, &V4521_309_57247, &P4521_307};	/*Property = Fullscreen*/
const Value V4521_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_310 = {S29, 0, &V4521_310_57250, &P4521_309};	/*Property = GeoLocation*/
const Value V4521_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_312 = {S30, 0, &V4521_312_57256, &P4521_310};	/*Property = History*/
const Value V4521_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_314 = {S31, 0, &V4521_314_57262, &P4521_312};	/*Property = Html5*/
const Value V4521_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_313 = {S32, 0, &V4521_313_57259, &P4521_314};	/*Property = Html-Media-Capture*/
const Value V4521_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_315 = {S33, 0, &V4521_315_57265, &P4521_313};	/*Property = Iframe*/
const Value V4521_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_316 = {S34, 0, &V4521_316_57268, &P4521_315};	/*Property = IndexedDB*/
const Value V4521_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_317 = {S35, 0, &V4521_317_57271, &P4521_316};	/*Property = Json*/
const Value V4521_15_43265 = {S88, NULL};	/*Value = Lumi*/
const Property P4521_15 = {S37, 0, &V4521_15_43265, &P4521_317};	/*Property = LayoutEngine*/
const Value V4521_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_344 = {S38, 0, &V4521_344_57274, &P4521_15};	/*Property = Masking*/
const Value V4521_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_320 = {S39, 0, &V4521_320_57292, &P4521_344};	/*Property = PostMessage*/
const Value V4521_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_321 = {S40, 0, &V4521_321_57295, &P4521_320};	/*Property = Progress*/
const Value V4521_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_322 = {S41, 0, &V4521_322_57298, &P4521_321};	/*Property = Prompts*/
const Value V4521_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_323 = {S42, 0, &V4521_323_57301, &P4521_322};	/*Property = Selector*/
const Value V4521_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_324 = {S43, 0, &V4521_324_57307, &P4521_323};	/*Property = Svg*/
const Value V4521_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_327 = {S44, 0, &V4521_327_57316, &P4521_324};	/*Property = TouchEvents*/
const Value V4521_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_328 = {S45, 0, &V4521_328_57319, &P4521_327};	/*Property = Track*/
const Value V4521_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_329 = {S46, 0, &V4521_329_57325, &P4521_328};	/*Property = Video*/
const Value V4521_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_330 = {S47, 0, &V4521_330_57328, &P4521_329};	/*Property = Viewport*/
const Value V4521_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_332 = {S48, 0, &V4521_332_57340, &P4521_330};	/*Property = WebWorkers*/
const Value V4521_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4521_333 = {S49, 0, &V4521_333_57343, &P4521_332};	/*Property = Xhr2*/
const Profile P4521 = {"4521", &P4521_333};	/*Profile ID - 4521*/
const Value V13516_1_1 = {S2, NULL};	/*Value = True*/
const Property P13516_1 = {S50, 0, &V13516_1_1, NULL};	/*Property = IsMobile*/
const Value V13516_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13516_5 = {S52, 0, &V13516_5_1304, &P13516_1};	/*Property = ScreenPixelsHeight*/
const Value V13516_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13516_4 = {S54, 0, &V13516_4_1224, &P13516_5};	/*Property = ScreenPixelsWidth*/
const Profile P13516 = {"13516", &P13516_4};	/*Profile ID - 13516*/
const Value V13518_1_1 = {S2, NULL};	/*Value = True*/
const Property P13518_1 = {S50, 0, &V13518_1_1, NULL};	/*Property = IsMobile*/
const Value V13518_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13518_5 = {S52, 0, &V13518_5_1233, &P13518_1};	/*Property = ScreenPixelsHeight*/
const Value V13518_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13518_4 = {S54, 0, &V13518_4_1235, &P13518_5};	/*Property = ScreenPixelsWidth*/
const Profile P13518 = {"13518", &P13518_4};	/*Profile ID - 13518*/
const Value V13522_1_1 = {S2, NULL};	/*Value = True*/
const Property P13522_1 = {S50, 0, &V13522_1_1, NULL};	/*Property = IsMobile*/
const Value V13522_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13522_5 = {S52, 0, &V13522_5_1222, &P13522_1};	/*Property = ScreenPixelsHeight*/
const Value V13522_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13522_4 = {S54, 0, &V13522_4_1224, &P13522_5};	/*Property = ScreenPixelsWidth*/
const Profile P13522 = {"13522", &P13522_4};	/*Profile ID - 13522*/
const Value V13332_1_1 = {S2, NULL};	/*Value = True*/
const Property P13332_1 = {S50, 0, &V13332_1_1, NULL};	/*Property = IsMobile*/
const Value V13332_5_4286 = {S73, NULL};	/*Value = 432*/
const Property P13332_5 = {S52, 0, &V13332_5_4286, &P13332_1};	/*Property = ScreenPixelsHeight*/
const Value V13332_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13332_4 = {S54, 0, &V13332_4_1224, &P13332_5};	/*Property = ScreenPixelsWidth*/
const Profile P13332 = {"13332", &P13332_4};	/*Profile ID - 13332*/
const Value V13528_1_1 = {S2, NULL};	/*Value = True*/
const Property P13528_1 = {S50, 0, &V13528_1_1, NULL};	/*Property = IsMobile*/
const Value V13528_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13528_5 = {S52, 0, &V13528_5_1222, &P13528_1};	/*Property = ScreenPixelsHeight*/
const Value V13528_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13528_4 = {S54, 0, &V13528_4_1224, &P13528_5};	/*Property = ScreenPixelsWidth*/
const Profile P13528 = {"13528", &P13528_4};	/*Profile ID - 13528*/
const Value V13529_1_1 = {S2, NULL};	/*Value = True*/
const Property P13529_1 = {S50, 0, &V13529_1_1, NULL};	/*Property = IsMobile*/
const Value V13529_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13529_5 = {S52, 0, &V13529_5_1293, &P13529_1};	/*Property = ScreenPixelsHeight*/
const Value V13529_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13529_4 = {S54, 0, &V13529_4_1296, &P13529_5};	/*Property = ScreenPixelsWidth*/
const Profile P13529 = {"13529", &P13529_4};	/*Profile ID - 13529*/
const Value V11858_1_1 = {S2, NULL};	/*Value = True*/
const Property P11858_1 = {S50, 0, &V11858_1_1, NULL};	/*Property = IsMobile*/
const Value V11858_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11858_5 = {S52, 0, &V11858_5_1233, &P11858_1};	/*Property = ScreenPixelsHeight*/
const Value V11858_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11858_4 = {S54, 0, &V11858_4_1235, &P11858_5};	/*Property = ScreenPixelsWidth*/
const Profile P11858 = {"11858", &P11858_4};	/*Profile ID - 11858*/
const Value V12100_1_1 = {S2, NULL};	/*Value = True*/
const Property P12100_1 = {S50, 0, &V12100_1_1, NULL};	/*Property = IsMobile*/
const Value V12100_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12100_5 = {S52, 0, &V12100_5_1304, &P12100_1};	/*Property = ScreenPixelsHeight*/
const Value V12100_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12100_4 = {S54, 0, &V12100_4_1224, &P12100_5};	/*Property = ScreenPixelsWidth*/
const Profile P12100 = {"12100", &P12100_4};	/*Profile ID - 12100*/
const Value V12012_1_1 = {S2, NULL};	/*Value = True*/
const Property P12012_1 = {S50, 0, &V12012_1_1, NULL};	/*Property = IsMobile*/
const Value V12012_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12012_5 = {S52, 0, &V12012_5_1222, &P12012_1};	/*Property = ScreenPixelsHeight*/
const Value V12012_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12012_4 = {S54, 0, &V12012_4_1224, &P12012_5};	/*Property = ScreenPixelsWidth*/
const Profile P12012 = {"12012", &P12012_4};	/*Profile ID - 12012*/
const Value V14592_1_1 = {S2, NULL};	/*Value = True*/
const Property P14592_1 = {S50, 0, &V14592_1_1, NULL};	/*Property = IsMobile*/
const Value V14592_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14592_5 = {S52, 0, &V14592_5_1222, &P14592_1};	/*Property = ScreenPixelsHeight*/
const Value V14592_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14592_4 = {S54, 0, &V14592_4_1224, &P14592_5};	/*Property = ScreenPixelsWidth*/
const Profile P14592 = {"14592", &P14592_4};	/*Profile ID - 14592*/
const Value V13532_1_1 = {S2, NULL};	/*Value = True*/
const Property P13532_1 = {S50, 0, &V13532_1_1, NULL};	/*Property = IsMobile*/
const Value V13532_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13532_5 = {S52, 0, &V13532_5_1294, &P13532_1};	/*Property = ScreenPixelsHeight*/
const Value V13532_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13532_4 = {S54, 0, &V13532_4_1296, &P13532_5};	/*Property = ScreenPixelsWidth*/
const Profile P13532 = {"13532", &P13532_4};	/*Profile ID - 13532*/
const Value V15998_1_1 = {S2, NULL};	/*Value = True*/
const Property P15998_1 = {S50, 0, &V15998_1_1, NULL};	/*Property = IsMobile*/
const Value V15998_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15998_5 = {S52, 0, &V15998_5_1222, &P15998_1};	/*Property = ScreenPixelsHeight*/
const Value V15998_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15998_4 = {S54, 0, &V15998_4_1224, &P15998_5};	/*Property = ScreenPixelsWidth*/
const Profile P15998 = {"15998", &P15998_4};	/*Profile ID - 15998*/
const Value V14569_1_1 = {S2, NULL};	/*Value = True*/
const Property P14569_1 = {S50, 0, &V14569_1_1, NULL};	/*Property = IsMobile*/
const Value V14569_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14569_5 = {S52, 0, &V14569_5_1222, &P14569_1};	/*Property = ScreenPixelsHeight*/
const Value V14569_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14569_4 = {S54, 0, &V14569_4_1224, &P14569_5};	/*Property = ScreenPixelsWidth*/
const Profile P14569 = {"14569", &P14569_4};	/*Profile ID - 14569*/
const Value V11826_1_1 = {S2, NULL};	/*Value = True*/
const Property P11826_1 = {S50, 0, &V11826_1_1, NULL};	/*Property = IsMobile*/
const Value V11826_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P11826_5 = {S52, 0, &V11826_5_1380, &P11826_1};	/*Property = ScreenPixelsHeight*/
const Value V11826_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P11826_4 = {S54, 0, &V11826_4_1279, &P11826_5};	/*Property = ScreenPixelsWidth*/
const Profile P11826 = {"11826", &P11826_4};	/*Profile ID - 11826*/
const Value V11828_1_1 = {S2, NULL};	/*Value = True*/
const Property P11828_1 = {S50, 0, &V11828_1_1, NULL};	/*Property = IsMobile*/
const Value V11828_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11828_5 = {S52, 0, &V11828_5_1222, &P11828_1};	/*Property = ScreenPixelsHeight*/
const Value V11828_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11828_4 = {S54, 0, &V11828_4_1224, &P11828_5};	/*Property = ScreenPixelsWidth*/
const Profile P11828 = {"11828", &P11828_4};	/*Profile ID - 11828*/
const Value V11829_1_1 = {S2, NULL};	/*Value = True*/
const Property P11829_1 = {S50, 0, &V11829_1_1, NULL};	/*Property = IsMobile*/
const Value V11829_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11829_5 = {S52, 0, &V11829_5_1222, &P11829_1};	/*Property = ScreenPixelsHeight*/
const Value V11829_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11829_4 = {S54, 0, &V11829_4_1224, &P11829_5};	/*Property = ScreenPixelsWidth*/
const Profile P11829 = {"11829", &P11829_4};	/*Profile ID - 11829*/
const Value V11837_1_1 = {S2, NULL};	/*Value = True*/
const Property P11837_1 = {S50, 0, &V11837_1_1, NULL};	/*Property = IsMobile*/
const Value V11837_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P11837_5 = {S52, 0, &V11837_5_1294, &P11837_1};	/*Property = ScreenPixelsHeight*/
const Value V11837_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11837_4 = {S54, 0, &V11837_4_1296, &P11837_5};	/*Property = ScreenPixelsWidth*/
const Profile P11837 = {"11837", &P11837_4};	/*Profile ID - 11837*/
const Value V13534_1_1 = {S2, NULL};	/*Value = True*/
const Property P13534_1 = {S50, 0, &V13534_1_1, NULL};	/*Property = IsMobile*/
const Value V13534_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13534_5 = {S52, 0, &V13534_5_1233, &P13534_1};	/*Property = ScreenPixelsHeight*/
const Value V13534_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13534_4 = {S54, 0, &V13534_4_1235, &P13534_5};	/*Property = ScreenPixelsWidth*/
const Profile P13534 = {"13534", &P13534_4};	/*Profile ID - 13534*/
const Value V16098_1_1 = {S2, NULL};	/*Value = True*/
const Property P16098_1 = {S50, 0, &V16098_1_1, NULL};	/*Property = IsMobile*/
const Value V16098_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P16098_5 = {S52, 0, &V16098_5_1304, &P16098_1};	/*Property = ScreenPixelsHeight*/
const Value V16098_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16098_4 = {S54, 0, &V16098_4_1224, &P16098_5};	/*Property = ScreenPixelsWidth*/
const Profile P16098 = {"16098", &P16098_4};	/*Profile ID - 16098*/
const Value V11844_1_1 = {S2, NULL};	/*Value = True*/
const Property P11844_1 = {S50, 0, &V11844_1_1, NULL};	/*Property = IsMobile*/
const Value V11844_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11844_5 = {S52, 0, &V11844_5_1233, &P11844_1};	/*Property = ScreenPixelsHeight*/
const Value V11844_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11844_4 = {S54, 0, &V11844_4_1235, &P11844_5};	/*Property = ScreenPixelsWidth*/
const Profile P11844 = {"11844", &P11844_4};	/*Profile ID - 11844*/
const Value V13536_1_1 = {S2, NULL};	/*Value = True*/
const Property P13536_1 = {S50, 0, &V13536_1_1, NULL};	/*Property = IsMobile*/
const Value V13536_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13536_5 = {S52, 0, &V13536_5_1222, &P13536_1};	/*Property = ScreenPixelsHeight*/
const Value V13536_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13536_4 = {S54, 0, &V13536_4_1224, &P13536_5};	/*Property = ScreenPixelsWidth*/
const Profile P13536 = {"13536", &P13536_4};	/*Profile ID - 13536*/
const Value V13537_1_1 = {S2, NULL};	/*Value = True*/
const Property P13537_1 = {S50, 0, &V13537_1_1, NULL};	/*Property = IsMobile*/
const Value V13537_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13537_5 = {S52, 0, &V13537_5_1304, &P13537_1};	/*Property = ScreenPixelsHeight*/
const Value V13537_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13537_4 = {S54, 0, &V13537_4_1224, &P13537_5};	/*Property = ScreenPixelsWidth*/
const Profile P13537 = {"13537", &P13537_4};	/*Profile ID - 13537*/
const Value V13539_1_1 = {S2, NULL};	/*Value = True*/
const Property P13539_1 = {S50, 0, &V13539_1_1, NULL};	/*Property = IsMobile*/
const Value V13539_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13539_5 = {S52, 0, &V13539_5_1233, &P13539_1};	/*Property = ScreenPixelsHeight*/
const Value V13539_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13539_4 = {S54, 0, &V13539_4_1235, &P13539_5};	/*Property = ScreenPixelsWidth*/
const Profile P13539 = {"13539", &P13539_4};	/*Profile ID - 13539*/
const Value V14564_1_1 = {S2, NULL};	/*Value = True*/
const Property P14564_1 = {S50, 0, &V14564_1_1, NULL};	/*Property = IsMobile*/
const Value V14564_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14564_5 = {S52, 0, &V14564_5_1222, &P14564_1};	/*Property = ScreenPixelsHeight*/
const Value V14564_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14564_4 = {S54, 0, &V14564_4_1224, &P14564_5};	/*Property = ScreenPixelsWidth*/
const Profile P14564 = {"14564", &P14564_4};	/*Profile ID - 14564*/
const Value V11850_1_1 = {S2, NULL};	/*Value = True*/
const Property P11850_1 = {S50, 0, &V11850_1_1, NULL};	/*Property = IsMobile*/
const Value V11850_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P11850_5 = {S52, 0, &V11850_5_1294, &P11850_1};	/*Property = ScreenPixelsHeight*/
const Value V11850_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11850_4 = {S54, 0, &V11850_4_1296, &P11850_5};	/*Property = ScreenPixelsWidth*/
const Profile P11850 = {"11850", &P11850_4};	/*Profile ID - 11850*/
const Value V11859_1_1 = {S2, NULL};	/*Value = True*/
const Property P11859_1 = {S50, 0, &V11859_1_1, NULL};	/*Property = IsMobile*/
const Value V11859_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11859_5 = {S52, 0, &V11859_5_1222, &P11859_1};	/*Property = ScreenPixelsHeight*/
const Value V11859_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11859_4 = {S54, 0, &V11859_4_1224, &P11859_5};	/*Property = ScreenPixelsWidth*/
const Profile P11859 = {"11859", &P11859_4};	/*Profile ID - 11859*/
const Value V11860_1_1 = {S2, NULL};	/*Value = True*/
const Property P11860_1 = {S50, 0, &V11860_1_1, NULL};	/*Property = IsMobile*/
const Value V11860_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P11860_5 = {S52, 0, &V11860_5_1280, &P11860_1};	/*Property = ScreenPixelsHeight*/
const Value V11860_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11860_4 = {S54, 0, &V11860_4_1235, &P11860_5};	/*Property = ScreenPixelsWidth*/
const Profile P11860 = {"11860", &P11860_4};	/*Profile ID - 11860*/
const Value V15468_1_1 = {S2, NULL};	/*Value = True*/
const Property P15468_1 = {S50, 0, &V15468_1_1, NULL};	/*Property = IsMobile*/
const Value V15468_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15468_5 = {S52, 0, &V15468_5_1233, &P15468_1};	/*Property = ScreenPixelsHeight*/
const Value V15468_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15468_4 = {S54, 0, &V15468_4_1235, &P15468_5};	/*Property = ScreenPixelsWidth*/
const Profile P15468 = {"15468", &P15468_4};	/*Profile ID - 15468*/
const Value V11861_1_1 = {S2, NULL};	/*Value = True*/
const Property P11861_1 = {S50, 0, &V11861_1_1, NULL};	/*Property = IsMobile*/
const Value V11861_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P11861_5 = {S52, 0, &V11861_5_1280, &P11861_1};	/*Property = ScreenPixelsHeight*/
const Value V11861_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11861_4 = {S54, 0, &V11861_4_1235, &P11861_5};	/*Property = ScreenPixelsWidth*/
const Profile P11861 = {"11861", &P11861_4};	/*Profile ID - 11861*/
const Value V13542_1_1 = {S2, NULL};	/*Value = True*/
const Property P13542_1 = {S50, 0, &V13542_1_1, NULL};	/*Property = IsMobile*/
const Value V13542_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13542_5 = {S52, 0, &V13542_5_1222, &P13542_1};	/*Property = ScreenPixelsHeight*/
const Value V13542_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13542_4 = {S54, 0, &V13542_4_1224, &P13542_5};	/*Property = ScreenPixelsWidth*/
const Profile P13542 = {"13542", &P13542_4};	/*Profile ID - 13542*/
const Value V13544_1_1 = {S2, NULL};	/*Value = True*/
const Property P13544_1 = {S50, 0, &V13544_1_1, NULL};	/*Property = IsMobile*/
const Value V13544_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13544_5 = {S52, 0, &V13544_5_1294, &P13544_1};	/*Property = ScreenPixelsHeight*/
const Value V13544_4_33019 = {S61, NULL};	/*Value = 160*/
const Property P13544_4 = {S54, 0, &V13544_4_33019, &P13544_5};	/*Property = ScreenPixelsWidth*/
const Profile P13544 = {"13544", &P13544_4};	/*Profile ID - 13544*/
const Value V11847_1_1 = {S2, NULL};	/*Value = True*/
const Property P11847_1 = {S50, 0, &V11847_1_1, NULL};	/*Property = IsMobile*/
const Value V11847_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11847_5 = {S52, 0, &V11847_5_1233, &P11847_1};	/*Property = ScreenPixelsHeight*/
const Value V11847_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11847_4 = {S54, 0, &V11847_4_1235, &P11847_5};	/*Property = ScreenPixelsWidth*/
const Profile P11847 = {"11847", &P11847_4};	/*Profile ID - 11847*/
const Value V13547_1_1 = {S2, NULL};	/*Value = True*/
const Property P13547_1 = {S50, 0, &V13547_1_1, NULL};	/*Property = IsMobile*/
const Value V13547_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13547_5 = {S52, 0, &V13547_5_1233, &P13547_1};	/*Property = ScreenPixelsHeight*/
const Value V13547_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13547_4 = {S54, 0, &V13547_4_1235, &P13547_5};	/*Property = ScreenPixelsWidth*/
const Profile P13547 = {"13547", &P13547_4};	/*Profile ID - 13547*/
const Value V13546_1_1 = {S2, NULL};	/*Value = True*/
const Property P13546_1 = {S50, 0, &V13546_1_1, NULL};	/*Property = IsMobile*/
const Value V13546_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13546_5 = {S52, 0, &V13546_5_1293, &P13546_1};	/*Property = ScreenPixelsHeight*/
const Value V13546_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13546_4 = {S54, 0, &V13546_4_1296, &P13546_5};	/*Property = ScreenPixelsWidth*/
const Profile P13546 = {"13546", &P13546_4};	/*Profile ID - 13546*/
const Value V17260_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_279 = {S1, 0, &V17260_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17260_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_282 = {S3, 0, &V17260_282_57155, &P17260_279};	/*Property = BlobBuilder*/
const Value V17260_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_350 = {S4, 0, &V17260_350_57446, &P17260_282};	/*Property = Canvas*/
const Value V17260_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_283 = {S5, 0, &V17260_283_57161, &P17260_350};	/*Property = CssBackground*/
const Value V17260_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_284 = {S6, 0, &V17260_284_57164, &P17260_283};	/*Property = CssBorderImage*/
const Value V17260_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_285 = {S7, 0, &V17260_285_57167, &P17260_284};	/*Property = CssCanvas*/
const Value V17260_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_286 = {S8, 0, &V17260_286_57170, &P17260_285};	/*Property = CssColor*/
const Value V17260_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_237 = {S9, 0, &V17260_237_55786, &P17260_286};	/*Property = CssColumn*/
const Value V17260_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_287 = {S10, 0, &V17260_287_57173, &P17260_237};	/*Property = CssFlexbox*/
const Value V17260_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_288 = {S11, 0, &V17260_288_57176, &P17260_287};	/*Property = CssFont*/
const Value V17260_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_289 = {S12, 0, &V17260_289_86899, &P17260_288};	/*Property = CssImages*/
const Value V17260_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_291 = {S13, 0, &V17260_291_57182, &P17260_289};	/*Property = CssMediaQueries*/
const Value V17260_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_292 = {S14, 0, &V17260_292_57185, &P17260_291};	/*Property = CssMinMax*/
const Value V17260_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_293 = {S15, 0, &V17260_293_57188, &P17260_292};	/*Property = CssOverflow*/
const Value V17260_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_294 = {S16, 0, &V17260_294_57191, &P17260_293};	/*Property = CssPosition*/
const Value V17260_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_295 = {S17, 0, &V17260_295_57194, &P17260_294};	/*Property = CssText*/
const Value V17260_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_297 = {S18, 0, &V17260_297_57197, &P17260_295};	/*Property = CssTransforms*/
const Value V17260_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_241 = {S19, 0, &V17260_241_55801, &P17260_297};	/*Property = CssTransitions*/
const Value V17260_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_298 = {S20, 0, &V17260_298_57208, &P17260_241};	/*Property = CssUI*/
const Value V17260_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_300 = {S21, 0, &V17260_300_57217, &P17260_298};	/*Property = DataSet*/
const Value V17260_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_301 = {S22, 0, &V17260_301_57220, &P17260_300};	/*Property = DataUrl*/
const Value V17260_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_302 = {S23, 0, &V17260_302_57223, &P17260_301};	/*Property = DeviceOrientation*/
const Value V17260_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_304 = {S24, 0, &V17260_304_57229, &P17260_302};	/*Property = FileReader*/
const Value V17260_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_305 = {S25, 0, &V17260_305_57232, &P17260_304};	/*Property = FileSaver*/
const Value V17260_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_306 = {S26, 0, &V17260_306_57238, &P17260_305};	/*Property = FileWriter*/
const Value V17260_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_307 = {S27, 0, &V17260_307_57241, &P17260_306};	/*Property = FormData*/
const Value V17260_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_309 = {S28, 0, &V17260_309_57247, &P17260_307};	/*Property = Fullscreen*/
const Value V17260_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_310 = {S29, 0, &V17260_310_57250, &P17260_309};	/*Property = GeoLocation*/
const Value V17260_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_312 = {S30, 0, &V17260_312_57256, &P17260_310};	/*Property = History*/
const Value V17260_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_314 = {S31, 0, &V17260_314_57262, &P17260_312};	/*Property = Html5*/
const Value V17260_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_313 = {S32, 0, &V17260_313_57259, &P17260_314};	/*Property = Html-Media-Capture*/
const Value V17260_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_315 = {S33, 0, &V17260_315_57265, &P17260_313};	/*Property = Iframe*/
const Value V17260_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_316 = {S34, 0, &V17260_316_57268, &P17260_315};	/*Property = IndexedDB*/
const Value V17260_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_317 = {S35, 0, &V17260_317_57271, &P17260_316};	/*Property = Json*/
const Value V17260_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P17260_15 = {S37, 0, &V17260_15_29, &P17260_317};	/*Property = LayoutEngine*/
const Value V17260_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_344 = {S38, 0, &V17260_344_57274, &P17260_15};	/*Property = Masking*/
const Value V17260_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_320 = {S39, 0, &V17260_320_57292, &P17260_344};	/*Property = PostMessage*/
const Value V17260_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_321 = {S40, 0, &V17260_321_57295, &P17260_320};	/*Property = Progress*/
const Value V17260_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_322 = {S41, 0, &V17260_322_57298, &P17260_321};	/*Property = Prompts*/
const Value V17260_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_323 = {S42, 0, &V17260_323_57301, &P17260_322};	/*Property = Selector*/
const Value V17260_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_324 = {S43, 0, &V17260_324_57307, &P17260_323};	/*Property = Svg*/
const Value V17260_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_327 = {S44, 0, &V17260_327_57316, &P17260_324};	/*Property = TouchEvents*/
const Value V17260_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_328 = {S45, 0, &V17260_328_57319, &P17260_327};	/*Property = Track*/
const Value V17260_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_329 = {S46, 0, &V17260_329_57325, &P17260_328};	/*Property = Video*/
const Value V17260_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_330 = {S47, 0, &V17260_330_57328, &P17260_329};	/*Property = Viewport*/
const Value V17260_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_332 = {S48, 0, &V17260_332_57340, &P17260_330};	/*Property = WebWorkers*/
const Value V17260_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17260_333 = {S49, 0, &V17260_333_57343, &P17260_332};	/*Property = Xhr2*/
const Profile P17260 = {"17260", &P17260_333};	/*Profile ID - 17260*/
const Value V11863_1_1 = {S2, NULL};	/*Value = True*/
const Property P11863_1 = {S50, 0, &V11863_1_1, NULL};	/*Property = IsMobile*/
const Value V11863_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11863_5 = {S52, 0, &V11863_5_1293, &P11863_1};	/*Property = ScreenPixelsHeight*/
const Value V11863_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11863_4 = {S54, 0, &V11863_4_1296, &P11863_5};	/*Property = ScreenPixelsWidth*/
const Profile P11863 = {"11863", &P11863_4};	/*Profile ID - 11863*/
const Value V14457_1_1 = {S2, NULL};	/*Value = True*/
const Property P14457_1 = {S50, 0, &V14457_1_1, NULL};	/*Property = IsMobile*/
const Value V14457_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14457_5 = {S52, 0, &V14457_5_1222, &P14457_1};	/*Property = ScreenPixelsHeight*/
const Value V14457_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14457_4 = {S54, 0, &V14457_4_1224, &P14457_5};	/*Property = ScreenPixelsWidth*/
const Profile P14457 = {"14457", &P14457_4};	/*Profile ID - 14457*/
const Value V11865_1_1 = {S2, NULL};	/*Value = True*/
const Property P11865_1 = {S50, 0, &V11865_1_1, NULL};	/*Property = IsMobile*/
const Value V11865_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P11865_5 = {S52, 0, &V11865_5_1294, &P11865_1};	/*Property = ScreenPixelsHeight*/
const Value V11865_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11865_4 = {S54, 0, &V11865_4_1296, &P11865_5};	/*Property = ScreenPixelsWidth*/
const Profile P11865 = {"11865", &P11865_4};	/*Profile ID - 11865*/
const Value V11866_1_1 = {S2, NULL};	/*Value = True*/
const Property P11866_1 = {S50, 0, &V11866_1_1, NULL};	/*Property = IsMobile*/
const Value V11866_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11866_5 = {S52, 0, &V11866_5_1222, &P11866_1};	/*Property = ScreenPixelsHeight*/
const Value V11866_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11866_4 = {S54, 0, &V11866_4_1224, &P11866_5};	/*Property = ScreenPixelsWidth*/
const Profile P11866 = {"11866", &P11866_4};	/*Profile ID - 11866*/
const Value V11867_1_1 = {S2, NULL};	/*Value = True*/
const Property P11867_1 = {S50, 0, &V11867_1_1, NULL};	/*Property = IsMobile*/
const Value V11867_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11867_5 = {S52, 0, &V11867_5_1293, &P11867_1};	/*Property = ScreenPixelsHeight*/
const Value V11867_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11867_4 = {S54, 0, &V11867_4_1296, &P11867_5};	/*Property = ScreenPixelsWidth*/
const Profile P11867 = {"11867", &P11867_4};	/*Profile ID - 11867*/
const Value V13858_1_1 = {S2, NULL};	/*Value = True*/
const Property P13858_1 = {S50, 0, &V13858_1_1, NULL};	/*Property = IsMobile*/
const Value V13858_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13858_5 = {S52, 0, &V13858_5_1277, &P13858_1};	/*Property = ScreenPixelsHeight*/
const Value V13858_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13858_4 = {S54, 0, &V13858_4_1279, &P13858_5};	/*Property = ScreenPixelsWidth*/
const Profile P13858 = {"13858", &P13858_4};	/*Profile ID - 13858*/
const Value V11825_1_1 = {S2, NULL};	/*Value = True*/
const Property P11825_1 = {S50, 0, &V11825_1_1, NULL};	/*Property = IsMobile*/
const Value V11825_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11825_5 = {S52, 0, &V11825_5_1233, &P11825_1};	/*Property = ScreenPixelsHeight*/
const Value V11825_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11825_4 = {S54, 0, &V11825_4_1235, &P11825_5};	/*Property = ScreenPixelsWidth*/
const Profile P11825 = {"11825", &P11825_4};	/*Profile ID - 11825*/
const Value V11871_1_1 = {S2, NULL};	/*Value = True*/
const Property P11871_1 = {S50, 0, &V11871_1_1, NULL};	/*Property = IsMobile*/
const Value V11871_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11871_5 = {S52, 0, &V11871_5_1293, &P11871_1};	/*Property = ScreenPixelsHeight*/
const Value V11871_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11871_4 = {S54, 0, &V11871_4_1296, &P11871_5};	/*Property = ScreenPixelsWidth*/
const Profile P11871 = {"11871", &P11871_4};	/*Profile ID - 11871*/
const Profile P5049 = {"5049", NULL};	/*Profile ID - 5049*/
const Value V16225_1_1 = {S2, NULL};	/*Value = True*/
const Property P16225_1 = {S50, 0, &V16225_1_1, NULL};	/*Property = IsMobile*/
const Value V16225_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16225_5 = {S52, 0, &V16225_5_1227, &P16225_1};	/*Property = ScreenPixelsHeight*/
const Value V16225_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16225_4 = {S54, 0, &V16225_4_1229, &P16225_5};	/*Property = ScreenPixelsWidth*/
const Profile P16225 = {"16225", &P16225_4};	/*Profile ID - 16225*/
const Value V11906_1_1 = {S2, NULL};	/*Value = True*/
const Property P11906_1 = {S50, 0, &V11906_1_1, NULL};	/*Property = IsMobile*/
const Value V11906_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P11906_5 = {S52, 0, &V11906_5_1227, &P11906_1};	/*Property = ScreenPixelsHeight*/
const Value V11906_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P11906_4 = {S54, 0, &V11906_4_1229, &P11906_5};	/*Property = ScreenPixelsWidth*/
const Profile P11906 = {"11906", &P11906_4};	/*Profile ID - 11906*/
const Value V14341_1_1 = {S2, NULL};	/*Value = True*/
const Property P14341_1 = {S50, 0, &V14341_1_1, NULL};	/*Property = IsMobile*/
const Value V14341_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14341_5 = {S52, 0, &V14341_5_1222, &P14341_1};	/*Property = ScreenPixelsHeight*/
const Value V14341_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14341_4 = {S54, 0, &V14341_4_1224, &P14341_5};	/*Property = ScreenPixelsWidth*/
const Profile P14341 = {"14341", &P14341_4};	/*Profile ID - 14341*/
const Value V17621_1_1 = {S2, NULL};	/*Value = True*/
const Property P17621_1 = {S50, 0, &V17621_1_1, NULL};	/*Property = IsMobile*/
const Value V17621_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17621_5 = {S52, 0, &V17621_5_1227, &P17621_1};	/*Property = ScreenPixelsHeight*/
const Value V17621_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17621_4 = {S54, 0, &V17621_4_1229, &P17621_5};	/*Property = ScreenPixelsWidth*/
const Profile P17621 = {"17621", &P17621_4};	/*Profile ID - 17621*/
const Value V16860_1_1 = {S2, NULL};	/*Value = True*/
const Property P16860_1 = {S50, 0, &V16860_1_1, NULL};	/*Property = IsMobile*/
const Value V16860_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16860_5 = {S52, 0, &V16860_5_1227, &P16860_1};	/*Property = ScreenPixelsHeight*/
const Value V16860_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16860_4 = {S54, 0, &V16860_4_1229, &P16860_5};	/*Property = ScreenPixelsWidth*/
const Profile P16860 = {"16860", &P16860_4};	/*Profile ID - 16860*/
const Value V13045_1_1 = {S2, NULL};	/*Value = True*/
const Property P13045_1 = {S50, 0, &V13045_1_1, NULL};	/*Property = IsMobile*/
const Value V13045_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13045_5 = {S52, 0, &V13045_5_1233, &P13045_1};	/*Property = ScreenPixelsHeight*/
const Value V13045_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13045_4 = {S54, 0, &V13045_4_1235, &P13045_5};	/*Property = ScreenPixelsWidth*/
const Profile P13045 = {"13045", &P13045_4};	/*Profile ID - 13045*/
const Value V12594_1_1 = {S2, NULL};	/*Value = True*/
const Property P12594_1 = {S50, 0, &V12594_1_1, NULL};	/*Property = IsMobile*/
const Value V12594_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12594_5 = {S52, 0, &V12594_5_1304, &P12594_1};	/*Property = ScreenPixelsHeight*/
const Value V12594_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12594_4 = {S54, 0, &V12594_4_1224, &P12594_5};	/*Property = ScreenPixelsWidth*/
const Profile P12594 = {"12594", &P12594_4};	/*Profile ID - 12594*/
const Value V13875_1_1 = {S2, NULL};	/*Value = True*/
const Property P13875_1 = {S50, 0, &V13875_1_1, NULL};	/*Property = IsMobile*/
const Value V13875_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13875_5 = {S52, 0, &V13875_5_1304, &P13875_1};	/*Property = ScreenPixelsHeight*/
const Value V13875_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13875_4 = {S54, 0, &V13875_4_1224, &P13875_5};	/*Property = ScreenPixelsWidth*/
const Profile P13875 = {"13875", &P13875_4};	/*Profile ID - 13875*/
const Value V17250_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17250_279 = {S1, 0, &V17250_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17250_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17250_282 = {S3, 0, &V17250_282_57154, &P17250_279};	/*Property = BlobBuilder*/
const Value V17250_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17250_350 = {S4, 0, &V17250_350_57444, &P17250_282};	/*Property = Canvas*/
const Value V17250_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17250_283 = {S5, 0, &V17250_283_57159, &P17250_350};	/*Property = CssBackground*/
const Value V17250_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17250_284 = {S6, 0, &V17250_284_57163, &P17250_283};	/*Property = CssBorderImage*/
const Value V17250_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17250_285 = {S7, 0, &V17250_285_57166, &P17250_284};	/*Property = CssCanvas*/
const Value V17250_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17250_286 = {S8, 0, &V17250_286_57168, &P17250_285};	/*Property = CssColor*/
const Value V17250_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17250_237 = {S9, 0, &V17250_237_55784, &P17250_286};	/*Property = CssColumn*/
const Value V17250_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17250_287 = {S10, 0, &V17250_287_57172, &P17250_237};	/*Property = CssFlexbox*/
const Value V17250_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17250_288 = {S11, 0, &V17250_288_57175, &P17250_287};	/*Property = CssFont*/
const Value V17250_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17250_289 = {S12, 0, &V17250_289_57759, &P17250_288};	/*Property = CssImages*/
const Value V17250_291_57180 = {S2, NULL};	/*Value = True*/
const Property P17250_291 = {S13, 0, &V17250_291_57180, &P17250_289};	/*Property = CssMediaQueries*/
const Value V17250_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17250_292 = {S14, 0, &V17250_292_57183, &P17250_291};	/*Property = CssMinMax*/
const Value V17250_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17250_293 = {S15, 0, &V17250_293_57187, &P17250_292};	/*Property = CssOverflow*/
const Value V17250_294_57190 = {S0, NULL};	/*Value = False*/
const Property P17250_294 = {S16, 0, &V17250_294_57190, &P17250_293};	/*Property = CssPosition*/
const Value V17250_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17250_295 = {S17, 0, &V17250_295_57192, &P17250_294};	/*Property = CssText*/
const Value V17250_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17250_297 = {S18, 0, &V17250_297_57195, &P17250_295};	/*Property = CssTransforms*/
const Value V17250_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17250_241 = {S19, 0, &V17250_241_55799, &P17250_297};	/*Property = CssTransitions*/
const Value V17250_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17250_298 = {S20, 0, &V17250_298_57206, &P17250_241};	/*Property = CssUI*/
const Value V17250_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17250_300 = {S21, 0, &V17250_300_57216, &P17250_298};	/*Property = DataSet*/
const Value V17250_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17250_301 = {S22, 0, &V17250_301_57218, &P17250_300};	/*Property = DataUrl*/
const Value V17250_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17250_302 = {S23, 0, &V17250_302_57222, &P17250_301};	/*Property = DeviceOrientation*/
const Value V17250_304_57227 = {S2, NULL};	/*Value = True*/
const Property P17250_304 = {S24, 0, &V17250_304_57227, &P17250_302};	/*Property = FileReader*/
const Value V17250_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17250_305 = {S25, 0, &V17250_305_57231, &P17250_304};	/*Property = FileSaver*/
const Value V17250_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17250_306 = {S26, 0, &V17250_306_57237, &P17250_305};	/*Property = FileWriter*/
const Value V17250_307_57239 = {S2, NULL};	/*Value = True*/
const Property P17250_307 = {S27, 0, &V17250_307_57239, &P17250_306};	/*Property = FormData*/
const Value V17250_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17250_309 = {S28, 0, &V17250_309_57246, &P17250_307};	/*Property = Fullscreen*/
const Value V17250_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17250_310 = {S29, 0, &V17250_310_57248, &P17250_309};	/*Property = GeoLocation*/
const Value V17250_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17250_312 = {S30, 0, &V17250_312_57255, &P17250_310};	/*Property = History*/
const Value V17250_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17250_314 = {S31, 0, &V17250_314_57261, &P17250_312};	/*Property = Html5*/
const Value V17250_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17250_313 = {S32, 0, &V17250_313_57258, &P17250_314};	/*Property = Html-Media-Capture*/
const Value V17250_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17250_315 = {S33, 0, &V17250_315_57264, &P17250_313};	/*Property = Iframe*/
const Value V17250_316_57266 = {S2, NULL};	/*Value = True*/
const Property P17250_316 = {S34, 0, &V17250_316_57266, &P17250_315};	/*Property = IndexedDB*/
const Value V17250_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17250_317 = {S35, 0, &V17250_317_57269, &P17250_316};	/*Property = Json*/
const Value V17250_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17250_15 = {S37, 0, &V17250_15_28, &P17250_317};	/*Property = LayoutEngine*/
const Value V17250_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17250_344 = {S38, 0, &V17250_344_57272, &P17250_15};	/*Property = Masking*/
const Value V17250_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17250_320 = {S39, 0, &V17250_320_57290, &P17250_344};	/*Property = PostMessage*/
const Value V17250_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17250_321 = {S40, 0, &V17250_321_57293, &P17250_320};	/*Property = Progress*/
const Value V17250_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17250_322 = {S41, 0, &V17250_322_57296, &P17250_321};	/*Property = Prompts*/
const Value V17250_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17250_323 = {S42, 0, &V17250_323_57299, &P17250_322};	/*Property = Selector*/
const Value V17250_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17250_324 = {S43, 0, &V17250_324_57306, &P17250_323};	/*Property = Svg*/
const Value V17250_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17250_327 = {S44, 0, &V17250_327_57315, &P17250_324};	/*Property = TouchEvents*/
const Value V17250_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17250_328 = {S45, 0, &V17250_328_57318, &P17250_327};	/*Property = Track*/
const Value V17250_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17250_329 = {S46, 0, &V17250_329_57323, &P17250_328};	/*Property = Video*/
const Value V17250_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17250_330 = {S47, 0, &V17250_330_57326, &P17250_329};	/*Property = Viewport*/
const Value V17250_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17250_332 = {S48, 0, &V17250_332_57339, &P17250_330};	/*Property = WebWorkers*/
const Value V17250_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17250_333 = {S49, 0, &V17250_333_57342, &P17250_332};	/*Property = Xhr2*/
const Profile P17250 = {"17250", &P17250_333};	/*Profile ID - 17250*/
const Profile P17296 = {"17296", NULL};	/*Profile ID - 17296*/
const Value V14334_1_1 = {S2, NULL};	/*Value = True*/
const Property P14334_1 = {S50, 0, &V14334_1_1, NULL};	/*Property = IsMobile*/
const Value V14334_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14334_5 = {S52, 0, &V14334_5_1304, &P14334_1};	/*Property = ScreenPixelsHeight*/
const Value V14334_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14334_4 = {S54, 0, &V14334_4_1224, &P14334_5};	/*Property = ScreenPixelsWidth*/
const Profile P14334 = {"14334", &P14334_4};	/*Profile ID - 14334*/
const Value V13882_1_1 = {S2, NULL};	/*Value = True*/
const Property P13882_1 = {S50, 0, &V13882_1_1, NULL};	/*Property = IsMobile*/
const Value V13882_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13882_5 = {S52, 0, &V13882_5_1227, &P13882_1};	/*Property = ScreenPixelsHeight*/
const Value V13882_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13882_4 = {S54, 0, &V13882_4_1229, &P13882_5};	/*Property = ScreenPixelsWidth*/
const Profile P13882 = {"13882", &P13882_4};	/*Profile ID - 13882*/
const Value V14033_1_1 = {S2, NULL};	/*Value = True*/
const Property P14033_1 = {S50, 0, &V14033_1_1, NULL};	/*Property = IsMobile*/
const Value V14033_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P14033_5 = {S52, 0, &V14033_5_1275, &P14033_1};	/*Property = ScreenPixelsHeight*/
const Value V14033_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P14033_4 = {S54, 0, &V14033_4_1278, &P14033_5};	/*Property = ScreenPixelsWidth*/
const Profile P14033 = {"14033", &P14033_4};	/*Profile ID - 14033*/
const Value V14966_1_1 = {S2, NULL};	/*Value = True*/
const Property P14966_1 = {S50, 0, &V14966_1_1, NULL};	/*Property = IsMobile*/
const Value V14966_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P14966_5 = {S52, 0, &V14966_5_1275, &P14966_1};	/*Property = ScreenPixelsHeight*/
const Value V14966_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14966_4 = {S54, 0, &V14966_4_1229, &P14966_5};	/*Property = ScreenPixelsWidth*/
const Profile P14966 = {"14966", &P14966_4};	/*Profile ID - 14966*/
const Value V13889_1_1 = {S2, NULL};	/*Value = True*/
const Property P13889_1 = {S50, 0, &V13889_1_1, NULL};	/*Property = IsMobile*/
const Value V13889_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13889_5 = {S52, 0, &V13889_5_1277, &P13889_1};	/*Property = ScreenPixelsHeight*/
const Value V13889_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13889_4 = {S54, 0, &V13889_4_1279, &P13889_5};	/*Property = ScreenPixelsWidth*/
const Profile P13889 = {"13889", &P13889_4};	/*Profile ID - 13889*/
const Value V15345_1_1 = {S2, NULL};	/*Value = True*/
const Property P15345_1 = {S50, 0, &V15345_1_1, NULL};	/*Property = IsMobile*/
const Value V15345_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15345_5 = {S52, 0, &V15345_5_1277, &P15345_1};	/*Property = ScreenPixelsHeight*/
const Value V15345_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15345_4 = {S54, 0, &V15345_4_1279, &P15345_5};	/*Property = ScreenPixelsWidth*/
const Profile P15345 = {"15345", &P15345_4};	/*Profile ID - 15345*/
const Value V11821_1_1 = {S2, NULL};	/*Value = True*/
const Property P11821_1 = {S50, 0, &V11821_1_1, NULL};	/*Property = IsMobile*/
const Value V11821_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11821_5 = {S52, 0, &V11821_5_1293, &P11821_1};	/*Property = ScreenPixelsHeight*/
const Value V11821_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11821_4 = {S54, 0, &V11821_4_1296, &P11821_5};	/*Property = ScreenPixelsWidth*/
const Profile P11821 = {"11821", &P11821_4};	/*Profile ID - 11821*/
const Value V14386_1_1 = {S2, NULL};	/*Value = True*/
const Property P14386_1 = {S50, 0, &V14386_1_1, NULL};	/*Property = IsMobile*/
const Value V14386_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14386_5 = {S52, 0, &V14386_5_1294, &P14386_1};	/*Property = ScreenPixelsHeight*/
const Value V14386_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14386_4 = {S54, 0, &V14386_4_1296, &P14386_5};	/*Property = ScreenPixelsWidth*/
const Profile P14386 = {"14386", &P14386_4};	/*Profile ID - 14386*/
const Value V14806_1_1 = {S2, NULL};	/*Value = True*/
const Property P14806_1 = {S50, 0, &V14806_1_1, NULL};	/*Property = IsMobile*/
const Value V14806_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14806_5 = {S52, 0, &V14806_5_1380, &P14806_1};	/*Property = ScreenPixelsHeight*/
const Value V14806_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14806_4 = {S54, 0, &V14806_4_1279, &P14806_5};	/*Property = ScreenPixelsWidth*/
const Profile P14806 = {"14806", &P14806_4};	/*Profile ID - 14806*/
const Value V14826_1_1 = {S2, NULL};	/*Value = True*/
const Property P14826_1 = {S50, 0, &V14826_1_1, NULL};	/*Property = IsMobile*/
const Value V14826_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14826_5 = {S52, 0, &V14826_5_1293, &P14826_1};	/*Property = ScreenPixelsHeight*/
const Value V14826_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14826_4 = {S54, 0, &V14826_4_1296, &P14826_5};	/*Property = ScreenPixelsWidth*/
const Profile P14826 = {"14826", &P14826_4};	/*Profile ID - 14826*/
const Value V13894_1_1 = {S2, NULL};	/*Value = True*/
const Property P13894_1 = {S50, 0, &V13894_1_1, NULL};	/*Property = IsMobile*/
const Value V13894_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13894_5 = {S52, 0, &V13894_5_1380, &P13894_1};	/*Property = ScreenPixelsHeight*/
const Value V13894_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13894_4 = {S54, 0, &V13894_4_1279, &P13894_5};	/*Property = ScreenPixelsWidth*/
const Profile P13894 = {"13894", &P13894_4};	/*Profile ID - 13894*/
const Value V13900_1_1 = {S2, NULL};	/*Value = True*/
const Property P13900_1 = {S50, 0, &V13900_1_1, NULL};	/*Property = IsMobile*/
const Value V13900_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13900_5 = {S52, 0, &V13900_5_1294, &P13900_1};	/*Property = ScreenPixelsHeight*/
const Value V13900_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13900_4 = {S54, 0, &V13900_4_1296, &P13900_5};	/*Property = ScreenPixelsWidth*/
const Profile P13900 = {"13900", &P13900_4};	/*Profile ID - 13900*/
const Value V17108_1_1 = {S2, NULL};	/*Value = True*/
const Property P17108_1 = {S50, 0, &V17108_1_1, NULL};	/*Property = IsMobile*/
const Value V17108_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17108_5 = {S52, 0, &V17108_5_1227, &P17108_1};	/*Property = ScreenPixelsHeight*/
const Value V17108_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17108_4 = {S54, 0, &V17108_4_1229, &P17108_5};	/*Property = ScreenPixelsWidth*/
const Profile P17108 = {"17108", &P17108_4};	/*Profile ID - 17108*/
const Value V11884_1_1 = {S2, NULL};	/*Value = True*/
const Property P11884_1 = {S50, 0, &V11884_1_1, NULL};	/*Property = IsMobile*/
const Value V11884_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P11884_5 = {S52, 0, &V11884_5_1294, &P11884_1};	/*Property = ScreenPixelsHeight*/
const Value V11884_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11884_4 = {S54, 0, &V11884_4_1296, &P11884_5};	/*Property = ScreenPixelsWidth*/
const Profile P11884 = {"11884", &P11884_4};	/*Profile ID - 11884*/
const Value V13907_1_1 = {S2, NULL};	/*Value = True*/
const Property P13907_1 = {S50, 0, &V13907_1_1, NULL};	/*Property = IsMobile*/
const Value V13907_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13907_5 = {S52, 0, &V13907_5_1380, &P13907_1};	/*Property = ScreenPixelsHeight*/
const Value V13907_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P13907_4 = {S54, 0, &V13907_4_1943, &P13907_5};	/*Property = ScreenPixelsWidth*/
const Profile P13907 = {"13907", &P13907_4};	/*Profile ID - 13907*/
const Value V13909_1_1 = {S2, NULL};	/*Value = True*/
const Property P13909_1 = {S50, 0, &V13909_1_1, NULL};	/*Property = IsMobile*/
const Value V13909_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13909_5 = {S52, 0, &V13909_5_1233, &P13909_1};	/*Property = ScreenPixelsHeight*/
const Value V13909_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13909_4 = {S54, 0, &V13909_4_1235, &P13909_5};	/*Property = ScreenPixelsWidth*/
const Profile P13909 = {"13909", &P13909_4};	/*Profile ID - 13909*/
const Value V13915_1_1 = {S2, NULL};	/*Value = True*/
const Property P13915_1 = {S50, 0, &V13915_1_1, NULL};	/*Property = IsMobile*/
const Value V13915_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13915_5 = {S52, 0, &V13915_5_1233, &P13915_1};	/*Property = ScreenPixelsHeight*/
const Value V13915_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13915_4 = {S54, 0, &V13915_4_1235, &P13915_5};	/*Property = ScreenPixelsWidth*/
const Profile P13915 = {"13915", &P13915_4};	/*Profile ID - 13915*/
const Value V13916_1_1 = {S2, NULL};	/*Value = True*/
const Property P13916_1 = {S50, 0, &V13916_1_1, NULL};	/*Property = IsMobile*/
const Value V13916_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13916_5 = {S52, 0, &V13916_5_1293, &P13916_1};	/*Property = ScreenPixelsHeight*/
const Value V13916_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13916_4 = {S54, 0, &V13916_4_1296, &P13916_5};	/*Property = ScreenPixelsWidth*/
const Profile P13916 = {"13916", &P13916_4};	/*Profile ID - 13916*/
const Value V13918_1_1 = {S2, NULL};	/*Value = True*/
const Property P13918_1 = {S50, 0, &V13918_1_1, NULL};	/*Property = IsMobile*/
const Value V13918_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13918_5 = {S52, 0, &V13918_5_1222, &P13918_1};	/*Property = ScreenPixelsHeight*/
const Value V13918_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13918_4 = {S54, 0, &V13918_4_1224, &P13918_5};	/*Property = ScreenPixelsWidth*/
const Profile P13918 = {"13918", &P13918_4};	/*Profile ID - 13918*/
const Value V13885_1_1 = {S2, NULL};	/*Value = True*/
const Property P13885_1 = {S50, 0, &V13885_1_1, NULL};	/*Property = IsMobile*/
const Value V13885_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13885_5 = {S52, 0, &V13885_5_1222, &P13885_1};	/*Property = ScreenPixelsHeight*/
const Value V13885_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13885_4 = {S54, 0, &V13885_4_1224, &P13885_5};	/*Property = ScreenPixelsWidth*/
const Profile P13885 = {"13885", &P13885_4};	/*Profile ID - 13885*/
const Value V14985_1_1 = {S2, NULL};	/*Value = True*/
const Property P14985_1 = {S50, 0, &V14985_1_1, NULL};	/*Property = IsMobile*/
const Value V14985_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14985_5 = {S52, 0, &V14985_5_1227, &P14985_1};	/*Property = ScreenPixelsHeight*/
const Value V14985_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14985_4 = {S54, 0, &V14985_4_1229, &P14985_5};	/*Property = ScreenPixelsWidth*/
const Profile P14985 = {"14985", &P14985_4};	/*Profile ID - 14985*/
const Value V13913_1_1 = {S2, NULL};	/*Value = True*/
const Property P13913_1 = {S50, 0, &V13913_1_1, NULL};	/*Property = IsMobile*/
const Value V13913_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13913_5 = {S52, 0, &V13913_5_1233, &P13913_1};	/*Property = ScreenPixelsHeight*/
const Value V13913_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13913_4 = {S54, 0, &V13913_4_1235, &P13913_5};	/*Property = ScreenPixelsWidth*/
const Profile P13913 = {"13913", &P13913_4};	/*Profile ID - 13913*/
const Value V13931_1_1 = {S2, NULL};	/*Value = True*/
const Property P13931_1 = {S50, 0, &V13931_1_1, NULL};	/*Property = IsMobile*/
const Value V13931_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13931_5 = {S52, 0, &V13931_5_1222, &P13931_1};	/*Property = ScreenPixelsHeight*/
const Value V13931_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13931_4 = {S54, 0, &V13931_4_1224, &P13931_5};	/*Property = ScreenPixelsWidth*/
const Profile P13931 = {"13931", &P13931_4};	/*Profile ID - 13931*/
const Value V13937_1_1 = {S2, NULL};	/*Value = True*/
const Property P13937_1 = {S50, 0, &V13937_1_1, NULL};	/*Property = IsMobile*/
const Value V13937_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13937_5 = {S52, 0, &V13937_5_1294, &P13937_1};	/*Property = ScreenPixelsHeight*/
const Value V13937_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13937_4 = {S54, 0, &V13937_4_1296, &P13937_5};	/*Property = ScreenPixelsWidth*/
const Profile P13937 = {"13937", &P13937_4};	/*Profile ID - 13937*/
const Value V16553_1_1 = {S2, NULL};	/*Value = True*/
const Property P16553_1 = {S50, 0, &V16553_1_1, NULL};	/*Property = IsMobile*/
const Value V16553_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16553_5 = {S52, 0, &V16553_5_1227, &P16553_1};	/*Property = ScreenPixelsHeight*/
const Value V16553_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16553_4 = {S54, 0, &V16553_4_1229, &P16553_5};	/*Property = ScreenPixelsWidth*/
const Profile P16553 = {"16553", &P16553_4};	/*Profile ID - 16553*/
const Value V13945_1_1 = {S2, NULL};	/*Value = True*/
const Property P13945_1 = {S50, 0, &V13945_1_1, NULL};	/*Property = IsMobile*/
const Value V13945_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13945_5 = {S52, 0, &V13945_5_1222, &P13945_1};	/*Property = ScreenPixelsHeight*/
const Value V13945_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13945_4 = {S54, 0, &V13945_4_1224, &P13945_5};	/*Property = ScreenPixelsWidth*/
const Profile P13945 = {"13945", &P13945_4};	/*Profile ID - 13945*/
const Value V14540_1_1 = {S2, NULL};	/*Value = True*/
const Property P14540_1 = {S50, 0, &V14540_1_1, NULL};	/*Property = IsMobile*/
const Value V14540_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14540_5 = {S52, 0, &V14540_5_1380, &P14540_1};	/*Property = ScreenPixelsHeight*/
const Value V14540_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14540_4 = {S54, 0, &V14540_4_1279, &P14540_5};	/*Property = ScreenPixelsWidth*/
const Profile P14540 = {"14540", &P14540_4};	/*Profile ID - 14540*/
const Value V14711_1_1 = {S2, NULL};	/*Value = True*/
const Property P14711_1 = {S50, 0, &V14711_1_1, NULL};	/*Property = IsMobile*/
const Value V14711_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14711_5 = {S52, 0, &V14711_5_1222, &P14711_1};	/*Property = ScreenPixelsHeight*/
const Value V14711_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14711_4 = {S54, 0, &V14711_4_1224, &P14711_5};	/*Property = ScreenPixelsWidth*/
const Profile P14711 = {"14711", &P14711_4};	/*Profile ID - 14711*/
const Value V11985_1_1 = {S2, NULL};	/*Value = True*/
const Property P11985_1 = {S50, 0, &V11985_1_1, NULL};	/*Property = IsMobile*/
const Value V11985_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P11985_5 = {S52, 0, &V11985_5_1227, &P11985_1};	/*Property = ScreenPixelsHeight*/
const Value V11985_4_4350 = {S90, NULL};	/*Value = 345*/
const Property P11985_4 = {S54, 0, &V11985_4_4350, &P11985_5};	/*Property = ScreenPixelsWidth*/
const Profile P11985 = {"11985", &P11985_4};	/*Profile ID - 11985*/
const Value V11715_1_1 = {S2, NULL};	/*Value = True*/
const Property P11715_1 = {S50, 0, &V11715_1_1, NULL};	/*Property = IsMobile*/
const Value V11715_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11715_5 = {S52, 0, &V11715_5_1233, &P11715_1};	/*Property = ScreenPixelsHeight*/
const Value V11715_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11715_4 = {S54, 0, &V11715_4_1235, &P11715_5};	/*Property = ScreenPixelsWidth*/
const Profile P11715 = {"11715", &P11715_4};	/*Profile ID - 11715*/
const Value V15118_1_1 = {S2, NULL};	/*Value = True*/
const Property P15118_1 = {S50, 0, &V15118_1_1, NULL};	/*Property = IsMobile*/
const Value V15118_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P15118_5 = {S52, 0, &V15118_5_1275, &P15118_1};	/*Property = ScreenPixelsHeight*/
const Value V15118_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P15118_4 = {S54, 0, &V15118_4_1278, &P15118_5};	/*Property = ScreenPixelsWidth*/
const Profile P15118 = {"15118", &P15118_4};	/*Profile ID - 15118*/
const Value V17461_1_2 = {S0, NULL};	/*Value = False*/
const Property P17461_1 = {S50, 0, &V17461_1_2, NULL};	/*Property = IsMobile*/
const Value V17461_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P17461_5 = {S52, 0, &V17461_5_37, &P17461_1};	/*Property = ScreenPixelsHeight*/
const Value V17461_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P17461_4 = {S54, 0, &V17461_4_36, &P17461_5};	/*Property = ScreenPixelsWidth*/
const Profile P17461 = {"17461", &P17461_4};	/*Profile ID - 17461*/
const Value V4392_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_279 = {S1, 0, &V4392_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4392_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_282 = {S3, 0, &V4392_282_57155, &P4392_279};	/*Property = BlobBuilder*/
const Value V4392_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_350 = {S4, 0, &V4392_350_57446, &P4392_282};	/*Property = Canvas*/
const Value V4392_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_283 = {S5, 0, &V4392_283_57161, &P4392_350};	/*Property = CssBackground*/
const Value V4392_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_284 = {S6, 0, &V4392_284_57164, &P4392_283};	/*Property = CssBorderImage*/
const Value V4392_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_285 = {S7, 0, &V4392_285_57167, &P4392_284};	/*Property = CssCanvas*/
const Value V4392_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_286 = {S8, 0, &V4392_286_57170, &P4392_285};	/*Property = CssColor*/
const Value V4392_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_237 = {S9, 0, &V4392_237_55786, &P4392_286};	/*Property = CssColumn*/
const Value V4392_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_287 = {S10, 0, &V4392_287_57173, &P4392_237};	/*Property = CssFlexbox*/
const Value V4392_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_288 = {S11, 0, &V4392_288_57176, &P4392_287};	/*Property = CssFont*/
const Value V4392_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_289 = {S12, 0, &V4392_289_86899, &P4392_288};	/*Property = CssImages*/
const Value V4392_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_291 = {S13, 0, &V4392_291_57182, &P4392_289};	/*Property = CssMediaQueries*/
const Value V4392_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_292 = {S14, 0, &V4392_292_57185, &P4392_291};	/*Property = CssMinMax*/
const Value V4392_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_293 = {S15, 0, &V4392_293_57188, &P4392_292};	/*Property = CssOverflow*/
const Value V4392_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_294 = {S16, 0, &V4392_294_57191, &P4392_293};	/*Property = CssPosition*/
const Value V4392_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_295 = {S17, 0, &V4392_295_57194, &P4392_294};	/*Property = CssText*/
const Value V4392_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_297 = {S18, 0, &V4392_297_57197, &P4392_295};	/*Property = CssTransforms*/
const Value V4392_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_241 = {S19, 0, &V4392_241_55801, &P4392_297};	/*Property = CssTransitions*/
const Value V4392_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_298 = {S20, 0, &V4392_298_57208, &P4392_241};	/*Property = CssUI*/
const Value V4392_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_300 = {S21, 0, &V4392_300_57217, &P4392_298};	/*Property = DataSet*/
const Value V4392_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_301 = {S22, 0, &V4392_301_57220, &P4392_300};	/*Property = DataUrl*/
const Value V4392_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_302 = {S23, 0, &V4392_302_57223, &P4392_301};	/*Property = DeviceOrientation*/
const Value V4392_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_304 = {S24, 0, &V4392_304_57229, &P4392_302};	/*Property = FileReader*/
const Value V4392_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_305 = {S25, 0, &V4392_305_57232, &P4392_304};	/*Property = FileSaver*/
const Value V4392_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_306 = {S26, 0, &V4392_306_57238, &P4392_305};	/*Property = FileWriter*/
const Value V4392_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_307 = {S27, 0, &V4392_307_57241, &P4392_306};	/*Property = FormData*/
const Value V4392_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_309 = {S28, 0, &V4392_309_57247, &P4392_307};	/*Property = Fullscreen*/
const Value V4392_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_310 = {S29, 0, &V4392_310_57250, &P4392_309};	/*Property = GeoLocation*/
const Value V4392_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_312 = {S30, 0, &V4392_312_57256, &P4392_310};	/*Property = History*/
const Value V4392_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_314 = {S31, 0, &V4392_314_57262, &P4392_312};	/*Property = Html5*/
const Value V4392_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_313 = {S32, 0, &V4392_313_57259, &P4392_314};	/*Property = Html-Media-Capture*/
const Value V4392_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_315 = {S33, 0, &V4392_315_57265, &P4392_313};	/*Property = Iframe*/
const Value V4392_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_316 = {S34, 0, &V4392_316_57268, &P4392_315};	/*Property = IndexedDB*/
const Value V4392_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_317 = {S35, 0, &V4392_317_57271, &P4392_316};	/*Property = Json*/
const Value V4392_15_43263 = {S65, NULL};	/*Value = Netfront*/
const Property P4392_15 = {S37, 0, &V4392_15_43263, &P4392_317};	/*Property = LayoutEngine*/
const Value V4392_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_344 = {S38, 0, &V4392_344_57274, &P4392_15};	/*Property = Masking*/
const Value V4392_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_320 = {S39, 0, &V4392_320_57292, &P4392_344};	/*Property = PostMessage*/
const Value V4392_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_321 = {S40, 0, &V4392_321_57295, &P4392_320};	/*Property = Progress*/
const Value V4392_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_322 = {S41, 0, &V4392_322_57298, &P4392_321};	/*Property = Prompts*/
const Value V4392_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_323 = {S42, 0, &V4392_323_57301, &P4392_322};	/*Property = Selector*/
const Value V4392_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_324 = {S43, 0, &V4392_324_57307, &P4392_323};	/*Property = Svg*/
const Value V4392_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_327 = {S44, 0, &V4392_327_57316, &P4392_324};	/*Property = TouchEvents*/
const Value V4392_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_328 = {S45, 0, &V4392_328_57319, &P4392_327};	/*Property = Track*/
const Value V4392_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_329 = {S46, 0, &V4392_329_57325, &P4392_328};	/*Property = Video*/
const Value V4392_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_330 = {S47, 0, &V4392_330_57328, &P4392_329};	/*Property = Viewport*/
const Value V4392_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_332 = {S48, 0, &V4392_332_57340, &P4392_330};	/*Property = WebWorkers*/
const Value V4392_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4392_333 = {S49, 0, &V4392_333_57343, &P4392_332};	/*Property = Xhr2*/
const Profile P4392 = {"4392", &P4392_333};	/*Profile ID - 4392*/
const Value V14970_1_1 = {S2, NULL};	/*Value = True*/
const Property P14970_1 = {S50, 0, &V14970_1_1, NULL};	/*Property = IsMobile*/
const Value V14970_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14970_5 = {S52, 0, &V14970_5_1293, &P14970_1};	/*Property = ScreenPixelsHeight*/
const Value V14970_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14970_4 = {S54, 0, &V14970_4_1296, &P14970_5};	/*Property = ScreenPixelsWidth*/
const Profile P14970 = {"14970", &P14970_4};	/*Profile ID - 14970*/
const Value V20110_1_2 = {S0, NULL};	/*Value = False*/
const Property P20110_1 = {S50, 0, &V20110_1_2, NULL};	/*Property = IsMobile*/
const Value V20110_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20110_5 = {S52, 0, &V20110_5_37, &P20110_1};	/*Property = ScreenPixelsHeight*/
const Value V20110_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20110_4 = {S54, 0, &V20110_4_36, &P20110_5};	/*Property = ScreenPixelsWidth*/
const Profile P20110 = {"20110", &P20110_4};	/*Profile ID - 20110*/
const Value V13959_1_1 = {S2, NULL};	/*Value = True*/
const Property P13959_1 = {S50, 0, &V13959_1_1, NULL};	/*Property = IsMobile*/
const Value V13959_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P13959_5 = {S52, 0, &V13959_5_1280, &P13959_1};	/*Property = ScreenPixelsHeight*/
const Value V13959_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13959_4 = {S54, 0, &V13959_4_1235, &P13959_5};	/*Property = ScreenPixelsWidth*/
const Profile P13959 = {"13959", &P13959_4};	/*Profile ID - 13959*/
const Value V13960_1_1 = {S2, NULL};	/*Value = True*/
const Property P13960_1 = {S50, 0, &V13960_1_1, NULL};	/*Property = IsMobile*/
const Value V13960_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13960_5 = {S52, 0, &V13960_5_1227, &P13960_1};	/*Property = ScreenPixelsHeight*/
const Value V13960_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13960_4 = {S54, 0, &V13960_4_1229, &P13960_5};	/*Property = ScreenPixelsWidth*/
const Profile P13960 = {"13960", &P13960_4};	/*Profile ID - 13960*/
const Value V15160_1_1 = {S2, NULL};	/*Value = True*/
const Property P15160_1 = {S50, 0, &V15160_1_1, NULL};	/*Property = IsMobile*/
const Value V15160_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15160_5 = {S52, 0, &V15160_5_1277, &P15160_1};	/*Property = ScreenPixelsHeight*/
const Value V15160_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P15160_4 = {S54, 0, &V15160_4_2336, &P15160_5};	/*Property = ScreenPixelsWidth*/
const Profile P15160 = {"15160", &P15160_4};	/*Profile ID - 15160*/
const Value V13962_1_1 = {S2, NULL};	/*Value = True*/
const Property P13962_1 = {S50, 0, &V13962_1_1, NULL};	/*Property = IsMobile*/
const Value V13962_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13962_5 = {S52, 0, &V13962_5_1293, &P13962_1};	/*Property = ScreenPixelsHeight*/
const Value V13962_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13962_4 = {S54, 0, &V13962_4_1296, &P13962_5};	/*Property = ScreenPixelsWidth*/
const Profile P13962 = {"13962", &P13962_4};	/*Profile ID - 13962*/
const Value V13964_1_1 = {S2, NULL};	/*Value = True*/
const Property P13964_1 = {S50, 0, &V13964_1_1, NULL};	/*Property = IsMobile*/
const Value V13964_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13964_5 = {S52, 0, &V13964_5_1304, &P13964_1};	/*Property = ScreenPixelsHeight*/
const Value V13964_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13964_4 = {S54, 0, &V13964_4_1224, &P13964_5};	/*Property = ScreenPixelsWidth*/
const Profile P13964 = {"13964", &P13964_4};	/*Profile ID - 13964*/
const Value V11893_1_1 = {S2, NULL};	/*Value = True*/
const Property P11893_1 = {S50, 0, &V11893_1_1, NULL};	/*Property = IsMobile*/
const Value V11893_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P11893_5 = {S52, 0, &V11893_5_1304, &P11893_1};	/*Property = ScreenPixelsHeight*/
const Value V11893_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11893_4 = {S54, 0, &V11893_4_1224, &P11893_5};	/*Property = ScreenPixelsWidth*/
const Profile P11893 = {"11893", &P11893_4};	/*Profile ID - 11893*/
const Value V11897_1_1 = {S2, NULL};	/*Value = True*/
const Property P11897_1 = {S50, 0, &V11897_1_1, NULL};	/*Property = IsMobile*/
const Value V11897_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P11897_5 = {S52, 0, &V11897_5_1304, &P11897_1};	/*Property = ScreenPixelsHeight*/
const Value V11897_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11897_4 = {S54, 0, &V11897_4_1224, &P11897_5};	/*Property = ScreenPixelsWidth*/
const Profile P11897 = {"11897", &P11897_4};	/*Profile ID - 11897*/
const Value V17769_1_1 = {S2, NULL};	/*Value = True*/
const Property P17769_1 = {S50, 0, &V17769_1_1, NULL};	/*Property = IsMobile*/
const Value V17769_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17769_5 = {S52, 0, &V17769_5_1222, &P17769_1};	/*Property = ScreenPixelsHeight*/
const Value V17769_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17769_4 = {S54, 0, &V17769_4_1224, &P17769_5};	/*Property = ScreenPixelsWidth*/
const Profile P17769 = {"17769", &P17769_4};	/*Profile ID - 17769*/
const Value V11904_1_1 = {S2, NULL};	/*Value = True*/
const Property P11904_1 = {S50, 0, &V11904_1_1, NULL};	/*Property = IsMobile*/
const Value V11904_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11904_5 = {S52, 0, &V11904_5_1233, &P11904_1};	/*Property = ScreenPixelsHeight*/
const Value V11904_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11904_4 = {S54, 0, &V11904_4_1235, &P11904_5};	/*Property = ScreenPixelsWidth*/
const Profile P11904 = {"11904", &P11904_4};	/*Profile ID - 11904*/
const Value V4222_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_279 = {S1, 0, &V4222_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4222_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_282 = {S3, 0, &V4222_282_57155, &P4222_279};	/*Property = BlobBuilder*/
const Value V4222_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_350 = {S4, 0, &V4222_350_57446, &P4222_282};	/*Property = Canvas*/
const Value V4222_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_283 = {S5, 0, &V4222_283_57161, &P4222_350};	/*Property = CssBackground*/
const Value V4222_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_284 = {S6, 0, &V4222_284_57164, &P4222_283};	/*Property = CssBorderImage*/
const Value V4222_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_285 = {S7, 0, &V4222_285_57167, &P4222_284};	/*Property = CssCanvas*/
const Value V4222_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_286 = {S8, 0, &V4222_286_57170, &P4222_285};	/*Property = CssColor*/
const Value V4222_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_237 = {S9, 0, &V4222_237_55786, &P4222_286};	/*Property = CssColumn*/
const Value V4222_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_287 = {S10, 0, &V4222_287_57173, &P4222_237};	/*Property = CssFlexbox*/
const Value V4222_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_288 = {S11, 0, &V4222_288_57176, &P4222_287};	/*Property = CssFont*/
const Value V4222_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_289 = {S12, 0, &V4222_289_86899, &P4222_288};	/*Property = CssImages*/
const Value V4222_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_291 = {S13, 0, &V4222_291_57182, &P4222_289};	/*Property = CssMediaQueries*/
const Value V4222_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_292 = {S14, 0, &V4222_292_57185, &P4222_291};	/*Property = CssMinMax*/
const Value V4222_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_293 = {S15, 0, &V4222_293_57188, &P4222_292};	/*Property = CssOverflow*/
const Value V4222_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_294 = {S16, 0, &V4222_294_57191, &P4222_293};	/*Property = CssPosition*/
const Value V4222_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_295 = {S17, 0, &V4222_295_57194, &P4222_294};	/*Property = CssText*/
const Value V4222_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_297 = {S18, 0, &V4222_297_57197, &P4222_295};	/*Property = CssTransforms*/
const Value V4222_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_241 = {S19, 0, &V4222_241_55801, &P4222_297};	/*Property = CssTransitions*/
const Value V4222_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_298 = {S20, 0, &V4222_298_57208, &P4222_241};	/*Property = CssUI*/
const Value V4222_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_300 = {S21, 0, &V4222_300_57217, &P4222_298};	/*Property = DataSet*/
const Value V4222_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_301 = {S22, 0, &V4222_301_57220, &P4222_300};	/*Property = DataUrl*/
const Value V4222_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_302 = {S23, 0, &V4222_302_57223, &P4222_301};	/*Property = DeviceOrientation*/
const Value V4222_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_304 = {S24, 0, &V4222_304_57229, &P4222_302};	/*Property = FileReader*/
const Value V4222_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_305 = {S25, 0, &V4222_305_57232, &P4222_304};	/*Property = FileSaver*/
const Value V4222_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_306 = {S26, 0, &V4222_306_57238, &P4222_305};	/*Property = FileWriter*/
const Value V4222_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_307 = {S27, 0, &V4222_307_57241, &P4222_306};	/*Property = FormData*/
const Value V4222_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_309 = {S28, 0, &V4222_309_57247, &P4222_307};	/*Property = Fullscreen*/
const Value V4222_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_310 = {S29, 0, &V4222_310_57250, &P4222_309};	/*Property = GeoLocation*/
const Value V4222_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_312 = {S30, 0, &V4222_312_57256, &P4222_310};	/*Property = History*/
const Value V4222_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_314 = {S31, 0, &V4222_314_57262, &P4222_312};	/*Property = Html5*/
const Value V4222_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_313 = {S32, 0, &V4222_313_57259, &P4222_314};	/*Property = Html-Media-Capture*/
const Value V4222_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_315 = {S33, 0, &V4222_315_57265, &P4222_313};	/*Property = Iframe*/
const Value V4222_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_316 = {S34, 0, &V4222_316_57268, &P4222_315};	/*Property = IndexedDB*/
const Value V4222_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_317 = {S35, 0, &V4222_317_57271, &P4222_316};	/*Property = Json*/
const Value V4222_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_15 = {S37, 0, &V4222_15_72, &P4222_317};	/*Property = LayoutEngine*/
const Value V4222_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_344 = {S38, 0, &V4222_344_57274, &P4222_15};	/*Property = Masking*/
const Value V4222_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_320 = {S39, 0, &V4222_320_57292, &P4222_344};	/*Property = PostMessage*/
const Value V4222_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_321 = {S40, 0, &V4222_321_57295, &P4222_320};	/*Property = Progress*/
const Value V4222_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_322 = {S41, 0, &V4222_322_57298, &P4222_321};	/*Property = Prompts*/
const Value V4222_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_323 = {S42, 0, &V4222_323_57301, &P4222_322};	/*Property = Selector*/
const Value V4222_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_324 = {S43, 0, &V4222_324_57307, &P4222_323};	/*Property = Svg*/
const Value V4222_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_327 = {S44, 0, &V4222_327_57316, &P4222_324};	/*Property = TouchEvents*/
const Value V4222_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_328 = {S45, 0, &V4222_328_57319, &P4222_327};	/*Property = Track*/
const Value V4222_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_329 = {S46, 0, &V4222_329_57325, &P4222_328};	/*Property = Video*/
const Value V4222_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_330 = {S47, 0, &V4222_330_57328, &P4222_329};	/*Property = Viewport*/
const Value V4222_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_332 = {S48, 0, &V4222_332_57340, &P4222_330};	/*Property = WebWorkers*/
const Value V4222_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4222_333 = {S49, 0, &V4222_333_57343, &P4222_332};	/*Property = Xhr2*/
const Profile P4222 = {"4222", &P4222_333};	/*Profile ID - 4222*/
const Value V11908_1_1 = {S2, NULL};	/*Value = True*/
const Property P11908_1 = {S50, 0, &V11908_1_1, NULL};	/*Property = IsMobile*/
const Value V11908_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P11908_5 = {S52, 0, &V11908_5_1304, &P11908_1};	/*Property = ScreenPixelsHeight*/
const Value V11908_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11908_4 = {S54, 0, &V11908_4_1224, &P11908_5};	/*Property = ScreenPixelsWidth*/
const Profile P11908 = {"11908", &P11908_4};	/*Profile ID - 11908*/
const Value V14340_1_1 = {S2, NULL};	/*Value = True*/
const Property P14340_1 = {S50, 0, &V14340_1_1, NULL};	/*Property = IsMobile*/
const Value V14340_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14340_5 = {S52, 0, &V14340_5_1293, &P14340_1};	/*Property = ScreenPixelsHeight*/
const Value V14340_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14340_4 = {S54, 0, &V14340_4_1296, &P14340_5};	/*Property = ScreenPixelsWidth*/
const Profile P14340 = {"14340", &P14340_4};	/*Profile ID - 14340*/
const Value V11915_1_1 = {S2, NULL};	/*Value = True*/
const Property P11915_1 = {S50, 0, &V11915_1_1, NULL};	/*Property = IsMobile*/
const Value V11915_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11915_5 = {S52, 0, &V11915_5_1233, &P11915_1};	/*Property = ScreenPixelsHeight*/
const Value V11915_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11915_4 = {S54, 0, &V11915_4_1235, &P11915_5};	/*Property = ScreenPixelsWidth*/
const Profile P11915 = {"11915", &P11915_4};	/*Profile ID - 11915*/
const Value V11917_1_1 = {S2, NULL};	/*Value = True*/
const Property P11917_1 = {S50, 0, &V11917_1_1, NULL};	/*Property = IsMobile*/
const Value V11917_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11917_5 = {S52, 0, &V11917_5_1222, &P11917_1};	/*Property = ScreenPixelsHeight*/
const Value V11917_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11917_4 = {S54, 0, &V11917_4_1224, &P11917_5};	/*Property = ScreenPixelsWidth*/
const Profile P11917 = {"11917", &P11917_4};	/*Profile ID - 11917*/
const Value V11916_1_1 = {S2, NULL};	/*Value = True*/
const Property P11916_1 = {S50, 0, &V11916_1_1, NULL};	/*Property = IsMobile*/
const Value V11916_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P11916_5 = {S52, 0, &V11916_5_1294, &P11916_1};	/*Property = ScreenPixelsHeight*/
const Value V11916_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11916_4 = {S54, 0, &V11916_4_1296, &P11916_5};	/*Property = ScreenPixelsWidth*/
const Profile P11916 = {"11916", &P11916_4};	/*Profile ID - 11916*/
const Value V16882_1_1 = {S2, NULL};	/*Value = True*/
const Property P16882_1 = {S50, 0, &V16882_1_1, NULL};	/*Property = IsMobile*/
const Value V16882_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16882_5 = {S52, 0, &V16882_5_1222, &P16882_1};	/*Property = ScreenPixelsHeight*/
const Value V16882_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16882_4 = {S54, 0, &V16882_4_1224, &P16882_5};	/*Property = ScreenPixelsWidth*/
const Profile P16882 = {"16882", &P16882_4};	/*Profile ID - 16882*/
const Value V13965_1_1 = {S2, NULL};	/*Value = True*/
const Property P13965_1 = {S50, 0, &V13965_1_1, NULL};	/*Property = IsMobile*/
const Value V13965_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13965_5 = {S52, 0, &V13965_5_1293, &P13965_1};	/*Property = ScreenPixelsHeight*/
const Value V13965_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13965_4 = {S54, 0, &V13965_4_1296, &P13965_5};	/*Property = ScreenPixelsWidth*/
const Profile P13965 = {"13965", &P13965_4};	/*Profile ID - 13965*/
const Value V13974_1_1 = {S2, NULL};	/*Value = True*/
const Property P13974_1 = {S50, 0, &V13974_1_1, NULL};	/*Property = IsMobile*/
const Value V13974_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13974_5 = {S52, 0, &V13974_5_1222, &P13974_1};	/*Property = ScreenPixelsHeight*/
const Value V13974_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13974_4 = {S54, 0, &V13974_4_1224, &P13974_5};	/*Property = ScreenPixelsWidth*/
const Profile P13974 = {"13974", &P13974_4};	/*Profile ID - 13974*/
const Value V15447_1_1 = {S2, NULL};	/*Value = True*/
const Property P15447_1 = {S50, 0, &V15447_1_1, NULL};	/*Property = IsMobile*/
const Value V15447_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15447_5 = {S52, 0, &V15447_5_1227, &P15447_1};	/*Property = ScreenPixelsHeight*/
const Value V15447_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15447_4 = {S54, 0, &V15447_4_1229, &P15447_5};	/*Property = ScreenPixelsWidth*/
const Profile P15447 = {"15447", &P15447_4};	/*Profile ID - 15447*/
const Value V12864_1_1 = {S2, NULL};	/*Value = True*/
const Property P12864_1 = {S50, 0, &V12864_1_1, NULL};	/*Property = IsMobile*/
const Value V12864_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12864_5 = {S52, 0, &V12864_5_1222, &P12864_1};	/*Property = ScreenPixelsHeight*/
const Value V12864_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12864_4 = {S54, 0, &V12864_4_1224, &P12864_5};	/*Property = ScreenPixelsWidth*/
const Profile P12864 = {"12864", &P12864_4};	/*Profile ID - 12864*/
const Value V16905_1_1 = {S2, NULL};	/*Value = True*/
const Property P16905_1 = {S50, 0, &V16905_1_1, NULL};	/*Property = IsMobile*/
const Value V16905_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16905_5 = {S52, 0, &V16905_5_1222, &P16905_1};	/*Property = ScreenPixelsHeight*/
const Value V16905_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16905_4 = {S54, 0, &V16905_4_1224, &P16905_5};	/*Property = ScreenPixelsWidth*/
const Profile P16905 = {"16905", &P16905_4};	/*Profile ID - 16905*/
const Value V13987_1_1 = {S2, NULL};	/*Value = True*/
const Property P13987_1 = {S50, 0, &V13987_1_1, NULL};	/*Property = IsMobile*/
const Value V13987_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P13987_5 = {S52, 0, &V13987_5_1333, &P13987_1};	/*Property = ScreenPixelsHeight*/
const Value V13987_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P13987_4 = {S54, 0, &V13987_4_1335, &P13987_5};	/*Property = ScreenPixelsWidth*/
const Profile P13987 = {"13987", &P13987_4};	/*Profile ID - 13987*/
const Value V13991_1_1 = {S2, NULL};	/*Value = True*/
const Property P13991_1 = {S50, 0, &V13991_1_1, NULL};	/*Property = IsMobile*/
const Value V13991_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13991_5 = {S52, 0, &V13991_5_1227, &P13991_1};	/*Property = ScreenPixelsHeight*/
const Value V13991_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13991_4 = {S54, 0, &V13991_4_1229, &P13991_5};	/*Property = ScreenPixelsWidth*/
const Profile P13991 = {"13991", &P13991_4};	/*Profile ID - 13991*/
const Value V15679_1_1 = {S2, NULL};	/*Value = True*/
const Property P15679_1 = {S50, 0, &V15679_1_1, NULL};	/*Property = IsMobile*/
const Value V15679_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15679_5 = {S52, 0, &V15679_5_1227, &P15679_1};	/*Property = ScreenPixelsHeight*/
const Value V15679_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15679_4 = {S54, 0, &V15679_4_1229, &P15679_5};	/*Property = ScreenPixelsWidth*/
const Profile P15679 = {"15679", &P15679_4};	/*Profile ID - 15679*/
const Value V15517_1_1 = {S2, NULL};	/*Value = True*/
const Property P15517_1 = {S50, 0, &V15517_1_1, NULL};	/*Property = IsMobile*/
const Value V15517_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P15517_5 = {S52, 0, &V15517_5_1269, &P15517_1};	/*Property = ScreenPixelsHeight*/
const Value V15517_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15517_4 = {S54, 0, &V15517_4_1229, &P15517_5};	/*Property = ScreenPixelsWidth*/
const Profile P15517 = {"15517", &P15517_4};	/*Profile ID - 15517*/
const Value V12041_1_1 = {S2, NULL};	/*Value = True*/
const Property P12041_1 = {S50, 0, &V12041_1_1, NULL};	/*Property = IsMobile*/
const Value V12041_5_37453 = {S91, NULL};	/*Value = 65*/
const Property P12041_5 = {S52, 0, &V12041_5_37453, &P12041_1};	/*Property = ScreenPixelsHeight*/
const Value V12041_4_31956 = {S92, NULL};	/*Value = 96*/
const Property P12041_4 = {S54, 0, &V12041_4_31956, &P12041_5};	/*Property = ScreenPixelsWidth*/
const Profile P12041 = {"12041", &P12041_4};	/*Profile ID - 12041*/
const Value V14693_1_1 = {S2, NULL};	/*Value = True*/
const Property P14693_1 = {S50, 0, &V14693_1_1, NULL};	/*Property = IsMobile*/
const Value V14693_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14693_5 = {S52, 0, &V14693_5_1293, &P14693_1};	/*Property = ScreenPixelsHeight*/
const Value V14693_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14693_4 = {S54, 0, &V14693_4_1296, &P14693_5};	/*Property = ScreenPixelsWidth*/
const Profile P14693 = {"14693", &P14693_4};	/*Profile ID - 14693*/
const Value V17935_1_1 = {S2, NULL};	/*Value = True*/
const Property P17935_1 = {S50, 0, &V17935_1_1, NULL};	/*Property = IsMobile*/
const Value V17935_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17935_5 = {S52, 0, &V17935_5_1227, &P17935_1};	/*Property = ScreenPixelsHeight*/
const Value V17935_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17935_4 = {S54, 0, &V17935_4_1229, &P17935_5};	/*Property = ScreenPixelsWidth*/
const Profile P17935 = {"17935", &P17935_4};	/*Profile ID - 17935*/
const Value V15995_1_1 = {S2, NULL};	/*Value = True*/
const Property P15995_1 = {S50, 0, &V15995_1_1, NULL};	/*Property = IsMobile*/
const Value V15995_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15995_5 = {S52, 0, &V15995_5_1227, &P15995_1};	/*Property = ScreenPixelsHeight*/
const Value V15995_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15995_4 = {S54, 0, &V15995_4_1229, &P15995_5};	/*Property = ScreenPixelsWidth*/
const Profile P15995 = {"15995", &P15995_4};	/*Profile ID - 15995*/
const Value V14014_1_1 = {S2, NULL};	/*Value = True*/
const Property P14014_1 = {S50, 0, &V14014_1_1, NULL};	/*Property = IsMobile*/
const Value V14014_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14014_5 = {S52, 0, &V14014_5_1227, &P14014_1};	/*Property = ScreenPixelsHeight*/
const Value V14014_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14014_4 = {S54, 0, &V14014_4_1229, &P14014_5};	/*Property = ScreenPixelsWidth*/
const Profile P14014 = {"14014", &P14014_4};	/*Profile ID - 14014*/
const Value V14006_1_1 = {S2, NULL};	/*Value = True*/
const Property P14006_1 = {S50, 0, &V14006_1_1, NULL};	/*Property = IsMobile*/
const Value V14006_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14006_5 = {S52, 0, &V14006_5_1294, &P14006_1};	/*Property = ScreenPixelsHeight*/
const Value V14006_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14006_4 = {S54, 0, &V14006_4_1296, &P14006_5};	/*Property = ScreenPixelsWidth*/
const Profile P14006 = {"14006", &P14006_4};	/*Profile ID - 14006*/
const Value V14017_1_1 = {S2, NULL};	/*Value = True*/
const Property P14017_1 = {S50, 0, &V14017_1_1, NULL};	/*Property = IsMobile*/
const Value V14017_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14017_5 = {S52, 0, &V14017_5_1227, &P14017_1};	/*Property = ScreenPixelsHeight*/
const Value V14017_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14017_4 = {S54, 0, &V14017_4_1229, &P14017_5};	/*Property = ScreenPixelsWidth*/
const Profile P14017 = {"14017", &P14017_4};	/*Profile ID - 14017*/
const Value V15223_1_1 = {S2, NULL};	/*Value = True*/
const Property P15223_1 = {S50, 0, &V15223_1_1, NULL};	/*Property = IsMobile*/
const Value V15223_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15223_5 = {S52, 0, &V15223_5_1227, &P15223_1};	/*Property = ScreenPixelsHeight*/
const Value V15223_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15223_4 = {S54, 0, &V15223_4_1229, &P15223_5};	/*Property = ScreenPixelsWidth*/
const Profile P15223 = {"15223", &P15223_4};	/*Profile ID - 15223*/
const Value V14799_1_1 = {S2, NULL};	/*Value = True*/
const Property P14799_1 = {S50, 0, &V14799_1_1, NULL};	/*Property = IsMobile*/
const Value V14799_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14799_5 = {S52, 0, &V14799_5_1222, &P14799_1};	/*Property = ScreenPixelsHeight*/
const Value V14799_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14799_4 = {S54, 0, &V14799_4_1224, &P14799_5};	/*Property = ScreenPixelsWidth*/
const Profile P14799 = {"14799", &P14799_4};	/*Profile ID - 14799*/
const Value V14020_1_1 = {S2, NULL};	/*Value = True*/
const Property P14020_1 = {S50, 0, &V14020_1_1, NULL};	/*Property = IsMobile*/
const Value V14020_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14020_5 = {S52, 0, &V14020_5_1222, &P14020_1};	/*Property = ScreenPixelsHeight*/
const Value V14020_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14020_4 = {S54, 0, &V14020_4_1224, &P14020_5};	/*Property = ScreenPixelsWidth*/
const Profile P14020 = {"14020", &P14020_4};	/*Profile ID - 14020*/
const Value V14025_1_1 = {S2, NULL};	/*Value = True*/
const Property P14025_1 = {S50, 0, &V14025_1_1, NULL};	/*Property = IsMobile*/
const Value V14025_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14025_5 = {S52, 0, &V14025_5_1277, &P14025_1};	/*Property = ScreenPixelsHeight*/
const Value V14025_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14025_4 = {S54, 0, &V14025_4_1279, &P14025_5};	/*Property = ScreenPixelsWidth*/
const Profile P14025 = {"14025", &P14025_4};	/*Profile ID - 14025*/
const Value V14029_1_1 = {S2, NULL};	/*Value = True*/
const Property P14029_1 = {S50, 0, &V14029_1_1, NULL};	/*Property = IsMobile*/
const Value V14029_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14029_5 = {S52, 0, &V14029_5_1304, &P14029_1};	/*Property = ScreenPixelsHeight*/
const Value V14029_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14029_4 = {S54, 0, &V14029_4_1224, &P14029_5};	/*Property = ScreenPixelsWidth*/
const Profile P14029 = {"14029", &P14029_4};	/*Profile ID - 14029*/
const Value V14031_1_1 = {S2, NULL};	/*Value = True*/
const Property P14031_1 = {S50, 0, &V14031_1_1, NULL};	/*Property = IsMobile*/
const Value V14031_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14031_5 = {S52, 0, &V14031_5_1222, &P14031_1};	/*Property = ScreenPixelsHeight*/
const Value V14031_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14031_4 = {S54, 0, &V14031_4_1224, &P14031_5};	/*Property = ScreenPixelsWidth*/
const Profile P14031 = {"14031", &P14031_4};	/*Profile ID - 14031*/
const Value V14993_1_1 = {S2, NULL};	/*Value = True*/
const Property P14993_1 = {S50, 0, &V14993_1_1, NULL};	/*Property = IsMobile*/
const Value V14993_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P14993_5 = {S52, 0, &V14993_5_1269, &P14993_1};	/*Property = ScreenPixelsHeight*/
const Value V14993_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14993_4 = {S54, 0, &V14993_4_1229, &P14993_5};	/*Property = ScreenPixelsWidth*/
const Profile P14993 = {"14993", &P14993_4};	/*Profile ID - 14993*/
const Value V14978_1_1 = {S2, NULL};	/*Value = True*/
const Property P14978_1 = {S50, 0, &V14978_1_1, NULL};	/*Property = IsMobile*/
const Value V14978_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14978_5 = {S52, 0, &V14978_5_1277, &P14978_1};	/*Property = ScreenPixelsHeight*/
const Value V14978_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P14978_4 = {S54, 0, &V14978_4_1278, &P14978_5};	/*Property = ScreenPixelsWidth*/
const Profile P14978 = {"14978", &P14978_4};	/*Profile ID - 14978*/
const Value V14036_1_1 = {S2, NULL};	/*Value = True*/
const Property P14036_1 = {S50, 0, &V14036_1_1, NULL};	/*Property = IsMobile*/
const Value V14036_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14036_5 = {S52, 0, &V14036_5_1304, &P14036_1};	/*Property = ScreenPixelsHeight*/
const Value V14036_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14036_4 = {S54, 0, &V14036_4_1224, &P14036_5};	/*Property = ScreenPixelsWidth*/
const Profile P14036 = {"14036", &P14036_4};	/*Profile ID - 14036*/
const Value V14039_1_1 = {S2, NULL};	/*Value = True*/
const Property P14039_1 = {S50, 0, &V14039_1_1, NULL};	/*Property = IsMobile*/
const Value V14039_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14039_5 = {S52, 0, &V14039_5_1304, &P14039_1};	/*Property = ScreenPixelsHeight*/
const Value V14039_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14039_4 = {S54, 0, &V14039_4_1224, &P14039_5};	/*Property = ScreenPixelsWidth*/
const Profile P14039 = {"14039", &P14039_4};	/*Profile ID - 14039*/
const Value V13042_1_1 = {S2, NULL};	/*Value = True*/
const Property P13042_1 = {S50, 0, &V13042_1_1, NULL};	/*Property = IsMobile*/
const Value V13042_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13042_5 = {S52, 0, &V13042_5_1227, &P13042_1};	/*Property = ScreenPixelsHeight*/
const Value V13042_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13042_4 = {S54, 0, &V13042_4_1229, &P13042_5};	/*Property = ScreenPixelsWidth*/
const Profile P13042 = {"13042", &P13042_4};	/*Profile ID - 13042*/
const Value V14046_1_1 = {S2, NULL};	/*Value = True*/
const Property P14046_1 = {S50, 0, &V14046_1_1, NULL};	/*Property = IsMobile*/
const Value V14046_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14046_5 = {S52, 0, &V14046_5_1222, &P14046_1};	/*Property = ScreenPixelsHeight*/
const Value V14046_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14046_4 = {S54, 0, &V14046_4_1224, &P14046_5};	/*Property = ScreenPixelsWidth*/
const Profile P14046 = {"14046", &P14046_4};	/*Profile ID - 14046*/
const Value V14057_1_1 = {S2, NULL};	/*Value = True*/
const Property P14057_1 = {S50, 0, &V14057_1_1, NULL};	/*Property = IsMobile*/
const Value V14057_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14057_5 = {S52, 0, &V14057_5_1293, &P14057_1};	/*Property = ScreenPixelsHeight*/
const Value V14057_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14057_4 = {S54, 0, &V14057_4_1296, &P14057_5};	/*Property = ScreenPixelsWidth*/
const Profile P14057 = {"14057", &P14057_4};	/*Profile ID - 14057*/
const Value V14058_1_1 = {S2, NULL};	/*Value = True*/
const Property P14058_1 = {S50, 0, &V14058_1_1, NULL};	/*Property = IsMobile*/
const Value V14058_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14058_5 = {S52, 0, &V14058_5_1222, &P14058_1};	/*Property = ScreenPixelsHeight*/
const Value V14058_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14058_4 = {S54, 0, &V14058_4_1224, &P14058_5};	/*Property = ScreenPixelsWidth*/
const Profile P14058 = {"14058", &P14058_4};	/*Profile ID - 14058*/
const Value V14071_1_1 = {S2, NULL};	/*Value = True*/
const Property P14071_1 = {S50, 0, &V14071_1_1, NULL};	/*Property = IsMobile*/
const Value V14071_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P14071_5 = {S52, 0, &V14071_5_1275, &P14071_1};	/*Property = ScreenPixelsHeight*/
const Value V14071_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P14071_4 = {S54, 0, &V14071_4_1278, &P14071_5};	/*Property = ScreenPixelsWidth*/
const Profile P14071 = {"14071", &P14071_4};	/*Profile ID - 14071*/
const Value V14078_1_1 = {S2, NULL};	/*Value = True*/
const Property P14078_1 = {S50, 0, &V14078_1_1, NULL};	/*Property = IsMobile*/
const Value V14078_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14078_5 = {S52, 0, &V14078_5_1222, &P14078_1};	/*Property = ScreenPixelsHeight*/
const Value V14078_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14078_4 = {S54, 0, &V14078_4_1224, &P14078_5};	/*Property = ScreenPixelsWidth*/
const Profile P14078 = {"14078", &P14078_4};	/*Profile ID - 14078*/
const Profile P19945 = {"19945", NULL};	/*Profile ID - 19945*/
const Value V23918_1_1 = {S2, NULL};	/*Value = True*/
const Property P23918_1 = {S50, 0, &V23918_1_1, NULL};	/*Property = IsMobile*/
const Value V23918_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23918_5 = {S52, 0, &V23918_5_1269, &P23918_1};	/*Property = ScreenPixelsHeight*/
const Value V23918_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23918_4 = {S54, 0, &V23918_4_1229, &P23918_5};	/*Property = ScreenPixelsWidth*/
const Profile P23918 = {"23918", &P23918_4};	/*Profile ID - 23918*/
const Profile P5284 = {"5284", NULL};	/*Profile ID - 5284*/
const Value V14086_1_1 = {S2, NULL};	/*Value = True*/
const Property P14086_1 = {S50, 0, &V14086_1_1, NULL};	/*Property = IsMobile*/
const Value V14086_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14086_5 = {S52, 0, &V14086_5_1304, &P14086_1};	/*Property = ScreenPixelsHeight*/
const Value V14086_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14086_4 = {S54, 0, &V14086_4_1224, &P14086_5};	/*Property = ScreenPixelsWidth*/
const Profile P14086 = {"14086", &P14086_4};	/*Profile ID - 14086*/
const Value V13100_1_1 = {S2, NULL};	/*Value = True*/
const Property P13100_1 = {S50, 0, &V13100_1_1, NULL};	/*Property = IsMobile*/
const Value V13100_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13100_5 = {S52, 0, &V13100_5_1222, &P13100_1};	/*Property = ScreenPixelsHeight*/
const Value V13100_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13100_4 = {S54, 0, &V13100_4_1224, &P13100_5};	/*Property = ScreenPixelsWidth*/
const Profile P13100 = {"13100", &P13100_4};	/*Profile ID - 13100*/
const Value V14184_1_1 = {S2, NULL};	/*Value = True*/
const Property P14184_1 = {S50, 0, &V14184_1_1, NULL};	/*Property = IsMobile*/
const Value V14184_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14184_5 = {S52, 0, &V14184_5_1233, &P14184_1};	/*Property = ScreenPixelsHeight*/
const Value V14184_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14184_4 = {S54, 0, &V14184_4_1235, &P14184_5};	/*Property = ScreenPixelsWidth*/
const Profile P14184 = {"14184", &P14184_4};	/*Profile ID - 14184*/
const Value V14089_1_1 = {S2, NULL};	/*Value = True*/
const Property P14089_1 = {S50, 0, &V14089_1_1, NULL};	/*Property = IsMobile*/
const Value V14089_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14089_5 = {S52, 0, &V14089_5_1233, &P14089_1};	/*Property = ScreenPixelsHeight*/
const Value V14089_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14089_4 = {S54, 0, &V14089_4_1235, &P14089_5};	/*Property = ScreenPixelsWidth*/
const Profile P14089 = {"14089", &P14089_4};	/*Profile ID - 14089*/
const Value V14060_1_1 = {S2, NULL};	/*Value = True*/
const Property P14060_1 = {S50, 0, &V14060_1_1, NULL};	/*Property = IsMobile*/
const Value V14060_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14060_5 = {S52, 0, &V14060_5_1222, &P14060_1};	/*Property = ScreenPixelsHeight*/
const Value V14060_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14060_4 = {S54, 0, &V14060_4_1224, &P14060_5};	/*Property = ScreenPixelsWidth*/
const Profile P14060 = {"14060", &P14060_4};	/*Profile ID - 14060*/
const Value V14090_1_1 = {S2, NULL};	/*Value = True*/
const Property P14090_1 = {S50, 0, &V14090_1_1, NULL};	/*Property = IsMobile*/
const Value V14090_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14090_5 = {S52, 0, &V14090_5_1304, &P14090_1};	/*Property = ScreenPixelsHeight*/
const Value V14090_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14090_4 = {S54, 0, &V14090_4_1224, &P14090_5};	/*Property = ScreenPixelsWidth*/
const Profile P14090 = {"14090", &P14090_4};	/*Profile ID - 14090*/
const Value V4528_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_279 = {S1, 0, &V4528_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4528_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_282 = {S3, 0, &V4528_282_57155, &P4528_279};	/*Property = BlobBuilder*/
const Value V4528_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_350 = {S4, 0, &V4528_350_57446, &P4528_282};	/*Property = Canvas*/
const Value V4528_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_283 = {S5, 0, &V4528_283_57161, &P4528_350};	/*Property = CssBackground*/
const Value V4528_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_284 = {S6, 0, &V4528_284_57164, &P4528_283};	/*Property = CssBorderImage*/
const Value V4528_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_285 = {S7, 0, &V4528_285_57167, &P4528_284};	/*Property = CssCanvas*/
const Value V4528_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_286 = {S8, 0, &V4528_286_57170, &P4528_285};	/*Property = CssColor*/
const Value V4528_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_237 = {S9, 0, &V4528_237_55786, &P4528_286};	/*Property = CssColumn*/
const Value V4528_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_287 = {S10, 0, &V4528_287_57173, &P4528_237};	/*Property = CssFlexbox*/
const Value V4528_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_288 = {S11, 0, &V4528_288_57176, &P4528_287};	/*Property = CssFont*/
const Value V4528_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_289 = {S12, 0, &V4528_289_86899, &P4528_288};	/*Property = CssImages*/
const Value V4528_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_291 = {S13, 0, &V4528_291_57182, &P4528_289};	/*Property = CssMediaQueries*/
const Value V4528_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_292 = {S14, 0, &V4528_292_57185, &P4528_291};	/*Property = CssMinMax*/
const Value V4528_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_293 = {S15, 0, &V4528_293_57188, &P4528_292};	/*Property = CssOverflow*/
const Value V4528_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_294 = {S16, 0, &V4528_294_57191, &P4528_293};	/*Property = CssPosition*/
const Value V4528_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_295 = {S17, 0, &V4528_295_57194, &P4528_294};	/*Property = CssText*/
const Value V4528_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_297 = {S18, 0, &V4528_297_57197, &P4528_295};	/*Property = CssTransforms*/
const Value V4528_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_241 = {S19, 0, &V4528_241_55801, &P4528_297};	/*Property = CssTransitions*/
const Value V4528_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_298 = {S20, 0, &V4528_298_57208, &P4528_241};	/*Property = CssUI*/
const Value V4528_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_300 = {S21, 0, &V4528_300_57217, &P4528_298};	/*Property = DataSet*/
const Value V4528_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_301 = {S22, 0, &V4528_301_57220, &P4528_300};	/*Property = DataUrl*/
const Value V4528_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_302 = {S23, 0, &V4528_302_57223, &P4528_301};	/*Property = DeviceOrientation*/
const Value V4528_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_304 = {S24, 0, &V4528_304_57229, &P4528_302};	/*Property = FileReader*/
const Value V4528_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_305 = {S25, 0, &V4528_305_57232, &P4528_304};	/*Property = FileSaver*/
const Value V4528_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_306 = {S26, 0, &V4528_306_57238, &P4528_305};	/*Property = FileWriter*/
const Value V4528_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_307 = {S27, 0, &V4528_307_57241, &P4528_306};	/*Property = FormData*/
const Value V4528_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_309 = {S28, 0, &V4528_309_57247, &P4528_307};	/*Property = Fullscreen*/
const Value V4528_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_310 = {S29, 0, &V4528_310_57250, &P4528_309};	/*Property = GeoLocation*/
const Value V4528_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_312 = {S30, 0, &V4528_312_57256, &P4528_310};	/*Property = History*/
const Value V4528_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_314 = {S31, 0, &V4528_314_57262, &P4528_312};	/*Property = Html5*/
const Value V4528_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_313 = {S32, 0, &V4528_313_57259, &P4528_314};	/*Property = Html-Media-Capture*/
const Value V4528_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_315 = {S33, 0, &V4528_315_57265, &P4528_313};	/*Property = Iframe*/
const Value V4528_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_316 = {S34, 0, &V4528_316_57268, &P4528_315};	/*Property = IndexedDB*/
const Value V4528_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_317 = {S35, 0, &V4528_317_57271, &P4528_316};	/*Property = Json*/
const Value V4528_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_15 = {S37, 0, &V4528_15_72, &P4528_317};	/*Property = LayoutEngine*/
const Value V4528_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_344 = {S38, 0, &V4528_344_57274, &P4528_15};	/*Property = Masking*/
const Value V4528_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_320 = {S39, 0, &V4528_320_57292, &P4528_344};	/*Property = PostMessage*/
const Value V4528_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_321 = {S40, 0, &V4528_321_57295, &P4528_320};	/*Property = Progress*/
const Value V4528_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_322 = {S41, 0, &V4528_322_57298, &P4528_321};	/*Property = Prompts*/
const Value V4528_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_323 = {S42, 0, &V4528_323_57301, &P4528_322};	/*Property = Selector*/
const Value V4528_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_324 = {S43, 0, &V4528_324_57307, &P4528_323};	/*Property = Svg*/
const Value V4528_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_327 = {S44, 0, &V4528_327_57316, &P4528_324};	/*Property = TouchEvents*/
const Value V4528_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_328 = {S45, 0, &V4528_328_57319, &P4528_327};	/*Property = Track*/
const Value V4528_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_329 = {S46, 0, &V4528_329_57325, &P4528_328};	/*Property = Video*/
const Value V4528_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_330 = {S47, 0, &V4528_330_57328, &P4528_329};	/*Property = Viewport*/
const Value V4528_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_332 = {S48, 0, &V4528_332_57340, &P4528_330};	/*Property = WebWorkers*/
const Value V4528_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4528_333 = {S49, 0, &V4528_333_57343, &P4528_332};	/*Property = Xhr2*/
const Profile P4528 = {"4528", &P4528_333};	/*Profile ID - 4528*/
const Value V12986_1_1 = {S2, NULL};	/*Value = True*/
const Property P12986_1 = {S50, 0, &V12986_1_1, NULL};	/*Property = IsMobile*/
const Value V12986_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12986_5 = {S52, 0, &V12986_5_1380, &P12986_1};	/*Property = ScreenPixelsHeight*/
const Value V12986_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12986_4 = {S54, 0, &V12986_4_1279, &P12986_5};	/*Property = ScreenPixelsWidth*/
const Profile P12986 = {"12986", &P12986_4};	/*Profile ID - 12986*/
const Value V12711_1_1 = {S2, NULL};	/*Value = True*/
const Property P12711_1 = {S50, 0, &V12711_1_1, NULL};	/*Property = IsMobile*/
const Value V12711_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P12711_5 = {S52, 0, &V12711_5_1275, &P12711_1};	/*Property = ScreenPixelsHeight*/
const Value V12711_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12711_4 = {S54, 0, &V12711_4_1229, &P12711_5};	/*Property = ScreenPixelsWidth*/
const Profile P12711 = {"12711", &P12711_4};	/*Profile ID - 12711*/
const Value V14111_1_1 = {S2, NULL};	/*Value = True*/
const Property P14111_1 = {S50, 0, &V14111_1_1, NULL};	/*Property = IsMobile*/
const Value V14111_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P14111_5 = {S52, 0, &V14111_5_1269, &P14111_1};	/*Property = ScreenPixelsHeight*/
const Value V14111_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14111_4 = {S54, 0, &V14111_4_1229, &P14111_5};	/*Property = ScreenPixelsWidth*/
const Profile P14111 = {"14111", &P14111_4};	/*Profile ID - 14111*/
const Value V14516_1_1 = {S2, NULL};	/*Value = True*/
const Property P14516_1 = {S50, 0, &V14516_1_1, NULL};	/*Property = IsMobile*/
const Value V14516_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14516_5 = {S52, 0, &V14516_5_1222, &P14516_1};	/*Property = ScreenPixelsHeight*/
const Value V14516_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14516_4 = {S54, 0, &V14516_4_1224, &P14516_5};	/*Property = ScreenPixelsWidth*/
const Profile P14516 = {"14516", &P14516_4};	/*Profile ID - 14516*/
const Value V14115_1_1 = {S2, NULL};	/*Value = True*/
const Property P14115_1 = {S50, 0, &V14115_1_1, NULL};	/*Property = IsMobile*/
const Value V14115_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14115_5 = {S52, 0, &V14115_5_1380, &P14115_1};	/*Property = ScreenPixelsHeight*/
const Value V14115_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14115_4 = {S54, 0, &V14115_4_1279, &P14115_5};	/*Property = ScreenPixelsWidth*/
const Profile P14115 = {"14115", &P14115_4};	/*Profile ID - 14115*/
const Value V15177_1_1 = {S2, NULL};	/*Value = True*/
const Property P15177_1 = {S50, 0, &V15177_1_1, NULL};	/*Property = IsMobile*/
const Value V15177_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15177_5 = {S52, 0, &V15177_5_1222, &P15177_1};	/*Property = ScreenPixelsHeight*/
const Value V15177_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15177_4 = {S54, 0, &V15177_4_1224, &P15177_5};	/*Property = ScreenPixelsWidth*/
const Profile P15177 = {"15177", &P15177_4};	/*Profile ID - 15177*/
const Value V14998_1_1 = {S2, NULL};	/*Value = True*/
const Property P14998_1 = {S50, 0, &V14998_1_1, NULL};	/*Property = IsMobile*/
const Value V14998_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14998_5 = {S52, 0, &V14998_5_1380, &P14998_1};	/*Property = ScreenPixelsHeight*/
const Value V14998_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14998_4 = {S54, 0, &V14998_4_1279, &P14998_5};	/*Property = ScreenPixelsWidth*/
const Profile P14998 = {"14998", &P14998_4};	/*Profile ID - 14998*/
const Value V14117_1_1 = {S2, NULL};	/*Value = True*/
const Property P14117_1 = {S50, 0, &V14117_1_1, NULL};	/*Property = IsMobile*/
const Value V14117_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14117_5 = {S52, 0, &V14117_5_1277, &P14117_1};	/*Property = ScreenPixelsHeight*/
const Value V14117_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14117_4 = {S54, 0, &V14117_4_1279, &P14117_5};	/*Property = ScreenPixelsWidth*/
const Profile P14117 = {"14117", &P14117_4};	/*Profile ID - 14117*/
const Value V14969_1_1 = {S2, NULL};	/*Value = True*/
const Property P14969_1 = {S50, 0, &V14969_1_1, NULL};	/*Property = IsMobile*/
const Value V14969_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14969_5 = {S52, 0, &V14969_5_1227, &P14969_1};	/*Property = ScreenPixelsHeight*/
const Value V14969_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14969_4 = {S54, 0, &V14969_4_1229, &P14969_5};	/*Property = ScreenPixelsWidth*/
const Profile P14969 = {"14969", &P14969_4};	/*Profile ID - 14969*/
const Value V17930_1_1 = {S2, NULL};	/*Value = True*/
const Property P17930_1 = {S50, 0, &V17930_1_1, NULL};	/*Property = IsMobile*/
const Value V17930_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17930_5 = {S52, 0, &V17930_5_1304, &P17930_1};	/*Property = ScreenPixelsHeight*/
const Value V17930_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17930_4 = {S54, 0, &V17930_4_1224, &P17930_5};	/*Property = ScreenPixelsWidth*/
const Profile P17930 = {"17930", &P17930_4};	/*Profile ID - 17930*/
const Value V14123_1_1 = {S2, NULL};	/*Value = True*/
const Property P14123_1 = {S50, 0, &V14123_1_1, NULL};	/*Property = IsMobile*/
const Value V14123_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14123_5 = {S52, 0, &V14123_5_1227, &P14123_1};	/*Property = ScreenPixelsHeight*/
const Value V14123_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14123_4 = {S54, 0, &V14123_4_1229, &P14123_5};	/*Property = ScreenPixelsWidth*/
const Profile P14123 = {"14123", &P14123_4};	/*Profile ID - 14123*/
const Value V14999_1_1 = {S2, NULL};	/*Value = True*/
const Property P14999_1 = {S50, 0, &V14999_1_1, NULL};	/*Property = IsMobile*/
const Value V14999_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14999_5 = {S52, 0, &V14999_5_1233, &P14999_1};	/*Property = ScreenPixelsHeight*/
const Value V14999_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14999_4 = {S54, 0, &V14999_4_1235, &P14999_5};	/*Property = ScreenPixelsWidth*/
const Profile P14999 = {"14999", &P14999_4};	/*Profile ID - 14999*/
const Value V15928_1_1 = {S2, NULL};	/*Value = True*/
const Property P15928_1 = {S50, 0, &V15928_1_1, NULL};	/*Property = IsMobile*/
const Value V15928_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15928_5 = {S52, 0, &V15928_5_1222, &P15928_1};	/*Property = ScreenPixelsHeight*/
const Value V15928_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15928_4 = {S54, 0, &V15928_4_1224, &P15928_5};	/*Property = ScreenPixelsWidth*/
const Profile P15928 = {"15928", &P15928_4};	/*Profile ID - 15928*/
const Value V14161_1_1 = {S2, NULL};	/*Value = True*/
const Property P14161_1 = {S50, 0, &V14161_1_1, NULL};	/*Property = IsMobile*/
const Value V14161_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14161_5 = {S52, 0, &V14161_5_1277, &P14161_1};	/*Property = ScreenPixelsHeight*/
const Value V14161_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14161_4 = {S54, 0, &V14161_4_1279, &P14161_5};	/*Property = ScreenPixelsWidth*/
const Profile P14161 = {"14161", &P14161_4};	/*Profile ID - 14161*/
const Value V18589_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18589_279 = {S1, 0, &V18589_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18589_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18589_282 = {S3, 0, &V18589_282_57154, &P18589_279};	/*Property = BlobBuilder*/
const Value V18589_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18589_350 = {S4, 0, &V18589_350_57444, &P18589_282};	/*Property = Canvas*/
const Value V18589_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18589_283 = {S5, 0, &V18589_283_57159, &P18589_350};	/*Property = CssBackground*/
const Value V18589_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18589_284 = {S6, 0, &V18589_284_57164, &P18589_283};	/*Property = CssBorderImage*/
const Value V18589_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18589_285 = {S7, 0, &V18589_285_57166, &P18589_284};	/*Property = CssCanvas*/
const Value V18589_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18589_286 = {S8, 0, &V18589_286_57168, &P18589_285};	/*Property = CssColor*/
const Value V18589_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18589_237 = {S9, 0, &V18589_237_55786, &P18589_286};	/*Property = CssColumn*/
const Value V18589_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18589_287 = {S10, 0, &V18589_287_57172, &P18589_237};	/*Property = CssFlexbox*/
const Value V18589_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18589_288 = {S11, 0, &V18589_288_57176, &P18589_287};	/*Property = CssFont*/
const Value V18589_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18589_289 = {S12, 0, &V18589_289_57759, &P18589_288};	/*Property = CssImages*/
const Value V18589_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18589_291 = {S13, 0, &V18589_291_57182, &P18589_289};	/*Property = CssMediaQueries*/
const Value V18589_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18589_292 = {S14, 0, &V18589_292_57183, &P18589_291};	/*Property = CssMinMax*/
const Value V18589_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18589_293 = {S15, 0, &V18589_293_57188, &P18589_292};	/*Property = CssOverflow*/
const Value V18589_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18589_294 = {S16, 0, &V18589_294_57191, &P18589_293};	/*Property = CssPosition*/
const Value V18589_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18589_295 = {S17, 0, &V18589_295_57192, &P18589_294};	/*Property = CssText*/
const Value V18589_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18589_297 = {S18, 0, &V18589_297_57195, &P18589_295};	/*Property = CssTransforms*/
const Value V18589_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18589_241 = {S19, 0, &V18589_241_55799, &P18589_297};	/*Property = CssTransitions*/
const Value V18589_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18589_298 = {S20, 0, &V18589_298_57206, &P18589_241};	/*Property = CssUI*/
const Value V18589_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18589_300 = {S21, 0, &V18589_300_57217, &P18589_298};	/*Property = DataSet*/
const Value V18589_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18589_301 = {S22, 0, &V18589_301_57218, &P18589_300};	/*Property = DataUrl*/
const Value V18589_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18589_302 = {S23, 0, &V18589_302_57223, &P18589_301};	/*Property = DeviceOrientation*/
const Value V18589_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18589_304 = {S24, 0, &V18589_304_57229, &P18589_302};	/*Property = FileReader*/
const Value V18589_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18589_305 = {S25, 0, &V18589_305_57231, &P18589_304};	/*Property = FileSaver*/
const Value V18589_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18589_306 = {S26, 0, &V18589_306_57237, &P18589_305};	/*Property = FileWriter*/
const Value V18589_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18589_307 = {S27, 0, &V18589_307_57241, &P18589_306};	/*Property = FormData*/
const Value V18589_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18589_309 = {S28, 0, &V18589_309_57246, &P18589_307};	/*Property = Fullscreen*/
const Value V18589_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18589_310 = {S29, 0, &V18589_310_57248, &P18589_309};	/*Property = GeoLocation*/
const Value V18589_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18589_312 = {S30, 0, &V18589_312_57256, &P18589_310};	/*Property = History*/
const Value V18589_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18589_314 = {S31, 0, &V18589_314_57262, &P18589_312};	/*Property = Html5*/
const Value V18589_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18589_313 = {S32, 0, &V18589_313_57259, &P18589_314};	/*Property = Html-Media-Capture*/
const Value V18589_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18589_315 = {S33, 0, &V18589_315_57264, &P18589_313};	/*Property = Iframe*/
const Value V18589_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18589_316 = {S34, 0, &V18589_316_57268, &P18589_315};	/*Property = IndexedDB*/
const Value V18589_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18589_317 = {S35, 0, &V18589_317_57269, &P18589_316};	/*Property = Json*/
const Value V18589_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P18589_15 = {S37, 0, &V18589_15_72, &P18589_317};	/*Property = LayoutEngine*/
const Value V18589_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18589_344 = {S38, 0, &V18589_344_57272, &P18589_15};	/*Property = Masking*/
const Value V18589_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18589_320 = {S39, 0, &V18589_320_57290, &P18589_344};	/*Property = PostMessage*/
const Value V18589_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18589_321 = {S40, 0, &V18589_321_57293, &P18589_320};	/*Property = Progress*/
const Value V18589_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18589_322 = {S41, 0, &V18589_322_57296, &P18589_321};	/*Property = Prompts*/
const Value V18589_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18589_323 = {S42, 0, &V18589_323_57299, &P18589_322};	/*Property = Selector*/
const Value V18589_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18589_324 = {S43, 0, &V18589_324_57307, &P18589_323};	/*Property = Svg*/
const Value V18589_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18589_327 = {S44, 0, &V18589_327_57316, &P18589_324};	/*Property = TouchEvents*/
const Value V18589_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18589_328 = {S45, 0, &V18589_328_57318, &P18589_327};	/*Property = Track*/
const Value V18589_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18589_329 = {S46, 0, &V18589_329_57323, &P18589_328};	/*Property = Video*/
const Value V18589_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18589_330 = {S47, 0, &V18589_330_57326, &P18589_329};	/*Property = Viewport*/
const Value V18589_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18589_332 = {S48, 0, &V18589_332_57340, &P18589_330};	/*Property = WebWorkers*/
const Value V18589_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18589_333 = {S49, 0, &V18589_333_57343, &P18589_332};	/*Property = Xhr2*/
const Profile P18589 = {"18589", &P18589_333};	/*Profile ID - 18589*/
const Value V4571_279_57148 = {S0, NULL};	/*Value = False*/
const Property P4571_279 = {S1, 0, &V4571_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V4571_282_57154 = {S0, NULL};	/*Value = False*/
const Property P4571_282 = {S3, 0, &V4571_282_57154, &P4571_279};	/*Property = BlobBuilder*/
const Value V4571_350_57444 = {S2, NULL};	/*Value = True*/
const Property P4571_350 = {S4, 0, &V4571_350_57444, &P4571_282};	/*Property = Canvas*/
const Value V4571_283_57159 = {S2, NULL};	/*Value = True*/
const Property P4571_283 = {S5, 0, &V4571_283_57159, &P4571_350};	/*Property = CssBackground*/
const Value V4571_284_57163 = {S0, NULL};	/*Value = False*/
const Property P4571_284 = {S6, 0, &V4571_284_57163, &P4571_283};	/*Property = CssBorderImage*/
const Value V4571_285_57166 = {S0, NULL};	/*Value = False*/
const Property P4571_285 = {S7, 0, &V4571_285_57166, &P4571_284};	/*Property = CssCanvas*/
const Value V4571_286_57168 = {S2, NULL};	/*Value = True*/
const Property P4571_286 = {S8, 0, &V4571_286_57168, &P4571_285};	/*Property = CssColor*/
const Value V4571_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4571_237 = {S9, 0, &V4571_237_55786, &P4571_286};	/*Property = CssColumn*/
const Value V4571_287_57172 = {S0, NULL};	/*Value = False*/
const Property P4571_287 = {S10, 0, &V4571_287_57172, &P4571_237};	/*Property = CssFlexbox*/
const Value V4571_288_57175 = {S0, NULL};	/*Value = False*/
const Property P4571_288 = {S11, 0, &V4571_288_57175, &P4571_287};	/*Property = CssFont*/
const Value V4571_289_57759 = {S0, NULL};	/*Value = False*/
const Property P4571_289 = {S12, 0, &V4571_289_57759, &P4571_288};	/*Property = CssImages*/
const Value V4571_291_57180 = {S2, NULL};	/*Value = True*/
const Property P4571_291 = {S13, 0, &V4571_291_57180, &P4571_289};	/*Property = CssMediaQueries*/
const Value V4571_292_57183 = {S2, NULL};	/*Value = True*/
const Property P4571_292 = {S14, 0, &V4571_292_57183, &P4571_291};	/*Property = CssMinMax*/
const Value V4571_293_57186 = {S2, NULL};	/*Value = True*/
const Property P4571_293 = {S15, 0, &V4571_293_57186, &P4571_292};	/*Property = CssOverflow*/
const Value V4571_294_57189 = {S2, NULL};	/*Value = True*/
const Property P4571_294 = {S16, 0, &V4571_294_57189, &P4571_293};	/*Property = CssPosition*/
const Value V4571_295_57192 = {S2, NULL};	/*Value = True*/
const Property P4571_295 = {S17, 0, &V4571_295_57192, &P4571_294};	/*Property = CssText*/
const Value V4571_297_57195 = {S2, NULL};	/*Value = True*/
const Property P4571_297 = {S18, 0, &V4571_297_57195, &P4571_295};	/*Property = CssTransforms*/
const Value V4571_241_55799 = {S2, NULL};	/*Value = True*/
const Property P4571_241 = {S19, 0, &V4571_241_55799, &P4571_297};	/*Property = CssTransitions*/
const Value V4571_298_57206 = {S2, NULL};	/*Value = True*/
const Property P4571_298 = {S20, 0, &V4571_298_57206, &P4571_241};	/*Property = CssUI*/
const Value V4571_300_57216 = {S0, NULL};	/*Value = False*/
const Property P4571_300 = {S21, 0, &V4571_300_57216, &P4571_298};	/*Property = DataSet*/
const Value V4571_301_57218 = {S2, NULL};	/*Value = True*/
const Property P4571_301 = {S22, 0, &V4571_301_57218, &P4571_300};	/*Property = DataUrl*/
const Value V4571_302_57221 = {S2, NULL};	/*Value = True*/
const Property P4571_302 = {S23, 0, &V4571_302_57221, &P4571_301};	/*Property = DeviceOrientation*/
const Value V4571_304_57228 = {S0, NULL};	/*Value = False*/
const Property P4571_304 = {S24, 0, &V4571_304_57228, &P4571_302};	/*Property = FileReader*/
const Value V4571_305_57231 = {S0, NULL};	/*Value = False*/
const Property P4571_305 = {S25, 0, &V4571_305_57231, &P4571_304};	/*Property = FileSaver*/
const Value V4571_306_57237 = {S0, NULL};	/*Value = False*/
const Property P4571_306 = {S26, 0, &V4571_306_57237, &P4571_305};	/*Property = FileWriter*/
const Value V4571_307_57239 = {S2, NULL};	/*Value = True*/
const Property P4571_307 = {S27, 0, &V4571_307_57239, &P4571_306};	/*Property = FormData*/
const Value V4571_309_57246 = {S0, NULL};	/*Value = False*/
const Property P4571_309 = {S28, 0, &V4571_309_57246, &P4571_307};	/*Property = Fullscreen*/
const Value V4571_310_57248 = {S2, NULL};	/*Value = True*/
const Property P4571_310 = {S29, 0, &V4571_310_57248, &P4571_309};	/*Property = GeoLocation*/
const Value V4571_312_57255 = {S0, NULL};	/*Value = False*/
const Property P4571_312 = {S30, 0, &V4571_312_57255, &P4571_310};	/*Property = History*/
const Value V4571_314_57261 = {S0, NULL};	/*Value = False*/
const Property P4571_314 = {S31, 0, &V4571_314_57261, &P4571_312};	/*Property = Html5*/
const Value V4571_313_57258 = {S0, NULL};	/*Value = False*/
const Property P4571_313 = {S32, 0, &V4571_313_57258, &P4571_314};	/*Property = Html-Media-Capture*/
const Value V4571_315_57264 = {S0, NULL};	/*Value = False*/
const Property P4571_315 = {S33, 0, &V4571_315_57264, &P4571_313};	/*Property = Iframe*/
const Value V4571_316_57267 = {S0, NULL};	/*Value = False*/
const Property P4571_316 = {S34, 0, &V4571_316_57267, &P4571_315};	/*Property = IndexedDB*/
const Value V4571_317_57269 = {S2, NULL};	/*Value = True*/
const Property P4571_317 = {S35, 0, &V4571_317_57269, &P4571_316};	/*Property = Json*/
const Value V4571_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P4571_15 = {S37, 0, &V4571_15_28, &P4571_317};	/*Property = LayoutEngine*/
const Value V4571_344_57272 = {S2, NULL};	/*Value = True*/
const Property P4571_344 = {S38, 0, &V4571_344_57272, &P4571_15};	/*Property = Masking*/
const Value V4571_320_57290 = {S2, NULL};	/*Value = True*/
const Property P4571_320 = {S39, 0, &V4571_320_57290, &P4571_344};	/*Property = PostMessage*/
const Value V4571_321_57293 = {S2, NULL};	/*Value = True*/
const Property P4571_321 = {S40, 0, &V4571_321_57293, &P4571_320};	/*Property = Progress*/
const Value V4571_322_57296 = {S2, NULL};	/*Value = True*/
const Property P4571_322 = {S41, 0, &V4571_322_57296, &P4571_321};	/*Property = Prompts*/
const Value V4571_323_57299 = {S2, NULL};	/*Value = True*/
const Property P4571_323 = {S42, 0, &V4571_323_57299, &P4571_322};	/*Property = Selector*/
const Value V4571_324_57306 = {S0, NULL};	/*Value = False*/
const Property P4571_324 = {S43, 0, &V4571_324_57306, &P4571_323};	/*Property = Svg*/
const Value V4571_327_57314 = {S2, NULL};	/*Value = True*/
const Property P4571_327 = {S44, 0, &V4571_327_57314, &P4571_324};	/*Property = TouchEvents*/
const Value V4571_328_57318 = {S0, NULL};	/*Value = False*/
const Property P4571_328 = {S45, 0, &V4571_328_57318, &P4571_327};	/*Property = Track*/
const Value V4571_329_57323 = {S2, NULL};	/*Value = True*/
const Property P4571_329 = {S46, 0, &V4571_329_57323, &P4571_328};	/*Property = Video*/
const Value V4571_330_57326 = {S2, NULL};	/*Value = True*/
const Property P4571_330 = {S47, 0, &V4571_330_57326, &P4571_329};	/*Property = Viewport*/
const Value V4571_332_57339 = {S0, NULL};	/*Value = False*/
const Property P4571_332 = {S48, 0, &V4571_332_57339, &P4571_330};	/*Property = WebWorkers*/
const Value V4571_333_57342 = {S0, NULL};	/*Value = False*/
const Property P4571_333 = {S49, 0, &V4571_333_57342, &P4571_332};	/*Property = Xhr2*/
const Profile P4571 = {"4571", &P4571_333};	/*Profile ID - 4571*/
const Profile P17129 = {"17129", NULL};	/*Profile ID - 17129*/
const Value V15001_1_1 = {S2, NULL};	/*Value = True*/
const Property P15001_1 = {S50, 0, &V15001_1_1, NULL};	/*Property = IsMobile*/
const Value V15001_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15001_5 = {S52, 0, &V15001_5_1222, &P15001_1};	/*Property = ScreenPixelsHeight*/
const Value V15001_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15001_4 = {S54, 0, &V15001_4_1224, &P15001_5};	/*Property = ScreenPixelsWidth*/
const Profile P15001 = {"15001", &P15001_4};	/*Profile ID - 15001*/
const Value V15341_1_1 = {S2, NULL};	/*Value = True*/
const Property P15341_1 = {S50, 0, &V15341_1_1, NULL};	/*Property = IsMobile*/
const Value V15341_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15341_5 = {S52, 0, &V15341_5_1293, &P15341_1};	/*Property = ScreenPixelsHeight*/
const Value V15341_4_37572 = {S93, NULL};	/*Value = 132*/
const Property P15341_4 = {S54, 0, &V15341_4_37572, &P15341_5};	/*Property = ScreenPixelsWidth*/
const Profile P15341 = {"15341", &P15341_4};	/*Profile ID - 15341*/
const Value V14146_1_1 = {S2, NULL};	/*Value = True*/
const Property P14146_1 = {S50, 0, &V14146_1_1, NULL};	/*Property = IsMobile*/
const Value V14146_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14146_5 = {S52, 0, &V14146_5_1222, &P14146_1};	/*Property = ScreenPixelsHeight*/
const Value V14146_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14146_4 = {S54, 0, &V14146_4_1224, &P14146_5};	/*Property = ScreenPixelsWidth*/
const Profile P14146 = {"14146", &P14146_4};	/*Profile ID - 14146*/
const Value V13891_1_1 = {S2, NULL};	/*Value = True*/
const Property P13891_1 = {S50, 0, &V13891_1_1, NULL};	/*Property = IsMobile*/
const Value V13891_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13891_5 = {S52, 0, &V13891_5_1233, &P13891_1};	/*Property = ScreenPixelsHeight*/
const Value V13891_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13891_4 = {S54, 0, &V13891_4_1235, &P13891_5};	/*Property = ScreenPixelsWidth*/
const Profile P13891 = {"13891", &P13891_4};	/*Profile ID - 13891*/
const Value V12308_1_1 = {S2, NULL};	/*Value = True*/
const Property P12308_1 = {S50, 0, &V12308_1_1, NULL};	/*Property = IsMobile*/
const Value V12308_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12308_5 = {S52, 0, &V12308_5_1222, &P12308_1};	/*Property = ScreenPixelsHeight*/
const Value V12308_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12308_4 = {S54, 0, &V12308_4_1224, &P12308_5};	/*Property = ScreenPixelsWidth*/
const Profile P12308 = {"12308", &P12308_4};	/*Profile ID - 12308*/
const Value V13372_1_1 = {S2, NULL};	/*Value = True*/
const Property P13372_1 = {S50, 0, &V13372_1_1, NULL};	/*Property = IsMobile*/
const Value V13372_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13372_5 = {S52, 0, &V13372_5_1222, &P13372_1};	/*Property = ScreenPixelsHeight*/
const Value V13372_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13372_4 = {S54, 0, &V13372_4_1224, &P13372_5};	/*Property = ScreenPixelsWidth*/
const Profile P13372 = {"13372", &P13372_4};	/*Profile ID - 13372*/
const Value V14158_1_1 = {S2, NULL};	/*Value = True*/
const Property P14158_1 = {S50, 0, &V14158_1_1, NULL};	/*Property = IsMobile*/
const Value V14158_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14158_5 = {S52, 0, &V14158_5_1233, &P14158_1};	/*Property = ScreenPixelsHeight*/
const Value V14158_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14158_4 = {S54, 0, &V14158_4_1235, &P14158_5};	/*Property = ScreenPixelsWidth*/
const Profile P14158 = {"14158", &P14158_4};	/*Profile ID - 14158*/
const Value V12032_1_1 = {S2, NULL};	/*Value = True*/
const Property P12032_1 = {S50, 0, &V12032_1_1, NULL};	/*Property = IsMobile*/
const Value V12032_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12032_5 = {S52, 0, &V12032_5_1277, &P12032_1};	/*Property = ScreenPixelsHeight*/
const Value V12032_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12032_4 = {S54, 0, &V12032_4_1279, &P12032_5};	/*Property = ScreenPixelsWidth*/
const Profile P12032 = {"12032", &P12032_4};	/*Profile ID - 12032*/
const Value V3816_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_279 = {S1, 0, &V3816_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3816_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_282 = {S3, 0, &V3816_282_57155, &P3816_279};	/*Property = BlobBuilder*/
const Value V3816_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_350 = {S4, 0, &V3816_350_57446, &P3816_282};	/*Property = Canvas*/
const Value V3816_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_283 = {S5, 0, &V3816_283_57161, &P3816_350};	/*Property = CssBackground*/
const Value V3816_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_284 = {S6, 0, &V3816_284_57164, &P3816_283};	/*Property = CssBorderImage*/
const Value V3816_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_285 = {S7, 0, &V3816_285_57167, &P3816_284};	/*Property = CssCanvas*/
const Value V3816_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_286 = {S8, 0, &V3816_286_57170, &P3816_285};	/*Property = CssColor*/
const Value V3816_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_237 = {S9, 0, &V3816_237_55786, &P3816_286};	/*Property = CssColumn*/
const Value V3816_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_287 = {S10, 0, &V3816_287_57173, &P3816_237};	/*Property = CssFlexbox*/
const Value V3816_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_288 = {S11, 0, &V3816_288_57176, &P3816_287};	/*Property = CssFont*/
const Value V3816_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_289 = {S12, 0, &V3816_289_86899, &P3816_288};	/*Property = CssImages*/
const Value V3816_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_291 = {S13, 0, &V3816_291_57182, &P3816_289};	/*Property = CssMediaQueries*/
const Value V3816_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_292 = {S14, 0, &V3816_292_57185, &P3816_291};	/*Property = CssMinMax*/
const Value V3816_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_293 = {S15, 0, &V3816_293_57188, &P3816_292};	/*Property = CssOverflow*/
const Value V3816_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_294 = {S16, 0, &V3816_294_57191, &P3816_293};	/*Property = CssPosition*/
const Value V3816_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_295 = {S17, 0, &V3816_295_57194, &P3816_294};	/*Property = CssText*/
const Value V3816_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_297 = {S18, 0, &V3816_297_57197, &P3816_295};	/*Property = CssTransforms*/
const Value V3816_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_241 = {S19, 0, &V3816_241_55801, &P3816_297};	/*Property = CssTransitions*/
const Value V3816_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_298 = {S20, 0, &V3816_298_57208, &P3816_241};	/*Property = CssUI*/
const Value V3816_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_300 = {S21, 0, &V3816_300_57217, &P3816_298};	/*Property = DataSet*/
const Value V3816_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_301 = {S22, 0, &V3816_301_57220, &P3816_300};	/*Property = DataUrl*/
const Value V3816_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_302 = {S23, 0, &V3816_302_57223, &P3816_301};	/*Property = DeviceOrientation*/
const Value V3816_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_304 = {S24, 0, &V3816_304_57229, &P3816_302};	/*Property = FileReader*/
const Value V3816_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_305 = {S25, 0, &V3816_305_57232, &P3816_304};	/*Property = FileSaver*/
const Value V3816_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_306 = {S26, 0, &V3816_306_57238, &P3816_305};	/*Property = FileWriter*/
const Value V3816_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_307 = {S27, 0, &V3816_307_57241, &P3816_306};	/*Property = FormData*/
const Value V3816_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_309 = {S28, 0, &V3816_309_57247, &P3816_307};	/*Property = Fullscreen*/
const Value V3816_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_310 = {S29, 0, &V3816_310_57250, &P3816_309};	/*Property = GeoLocation*/
const Value V3816_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_312 = {S30, 0, &V3816_312_57256, &P3816_310};	/*Property = History*/
const Value V3816_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_314 = {S31, 0, &V3816_314_57262, &P3816_312};	/*Property = Html5*/
const Value V3816_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_313 = {S32, 0, &V3816_313_57259, &P3816_314};	/*Property = Html-Media-Capture*/
const Value V3816_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_315 = {S33, 0, &V3816_315_57265, &P3816_313};	/*Property = Iframe*/
const Value V3816_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_316 = {S34, 0, &V3816_316_57268, &P3816_315};	/*Property = IndexedDB*/
const Value V3816_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_317 = {S35, 0, &V3816_317_57271, &P3816_316};	/*Property = Json*/
const Value V3816_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_15 = {S37, 0, &V3816_15_72, &P3816_317};	/*Property = LayoutEngine*/
const Value V3816_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_344 = {S38, 0, &V3816_344_57274, &P3816_15};	/*Property = Masking*/
const Value V3816_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_320 = {S39, 0, &V3816_320_57292, &P3816_344};	/*Property = PostMessage*/
const Value V3816_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_321 = {S40, 0, &V3816_321_57295, &P3816_320};	/*Property = Progress*/
const Value V3816_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_322 = {S41, 0, &V3816_322_57298, &P3816_321};	/*Property = Prompts*/
const Value V3816_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_323 = {S42, 0, &V3816_323_57301, &P3816_322};	/*Property = Selector*/
const Value V3816_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_324 = {S43, 0, &V3816_324_57307, &P3816_323};	/*Property = Svg*/
const Value V3816_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_327 = {S44, 0, &V3816_327_57316, &P3816_324};	/*Property = TouchEvents*/
const Value V3816_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_328 = {S45, 0, &V3816_328_57319, &P3816_327};	/*Property = Track*/
const Value V3816_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_329 = {S46, 0, &V3816_329_57325, &P3816_328};	/*Property = Video*/
const Value V3816_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_330 = {S47, 0, &V3816_330_57328, &P3816_329};	/*Property = Viewport*/
const Value V3816_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_332 = {S48, 0, &V3816_332_57340, &P3816_330};	/*Property = WebWorkers*/
const Value V3816_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3816_333 = {S49, 0, &V3816_333_57343, &P3816_332};	/*Property = Xhr2*/
const Profile P3816 = {"3816", &P3816_333};	/*Profile ID - 3816*/
const Value V11923_1_1 = {S2, NULL};	/*Value = True*/
const Property P11923_1 = {S50, 0, &V11923_1_1, NULL};	/*Property = IsMobile*/
const Value V11923_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11923_5 = {S52, 0, &V11923_5_1233, &P11923_1};	/*Property = ScreenPixelsHeight*/
const Value V11923_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11923_4 = {S54, 0, &V11923_4_1235, &P11923_5};	/*Property = ScreenPixelsWidth*/
const Profile P11923 = {"11923", &P11923_4};	/*Profile ID - 11923*/
const Value V11928_1_1 = {S2, NULL};	/*Value = True*/
const Property P11928_1 = {S50, 0, &V11928_1_1, NULL};	/*Property = IsMobile*/
const Value V11928_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P11928_5 = {S52, 0, &V11928_5_1294, &P11928_1};	/*Property = ScreenPixelsHeight*/
const Value V11928_4_33019 = {S61, NULL};	/*Value = 160*/
const Property P11928_4 = {S54, 0, &V11928_4_33019, &P11928_5};	/*Property = ScreenPixelsWidth*/
const Profile P11928 = {"11928", &P11928_4};	/*Profile ID - 11928*/
const Value V11935_1_1 = {S2, NULL};	/*Value = True*/
const Property P11935_1 = {S50, 0, &V11935_1_1, NULL};	/*Property = IsMobile*/
const Value V11935_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P11935_5 = {S52, 0, &V11935_5_1304, &P11935_1};	/*Property = ScreenPixelsHeight*/
const Value V11935_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11935_4 = {S54, 0, &V11935_4_1224, &P11935_5};	/*Property = ScreenPixelsWidth*/
const Profile P11935 = {"11935", &P11935_4};	/*Profile ID - 11935*/
const Value V11936_1_1 = {S2, NULL};	/*Value = True*/
const Property P11936_1 = {S50, 0, &V11936_1_1, NULL};	/*Property = IsMobile*/
const Value V11936_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P11936_5 = {S52, 0, &V11936_5_1294, &P11936_1};	/*Property = ScreenPixelsHeight*/
const Value V11936_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11936_4 = {S54, 0, &V11936_4_1296, &P11936_5};	/*Property = ScreenPixelsWidth*/
const Profile P11936 = {"11936", &P11936_4};	/*Profile ID - 11936*/
const Value V11937_1_1 = {S2, NULL};	/*Value = True*/
const Property P11937_1 = {S50, 0, &V11937_1_1, NULL};	/*Property = IsMobile*/
const Value V11937_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P11937_5 = {S52, 0, &V11937_5_1304, &P11937_1};	/*Property = ScreenPixelsHeight*/
const Value V11937_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11937_4 = {S54, 0, &V11937_4_1224, &P11937_5};	/*Property = ScreenPixelsWidth*/
const Profile P11937 = {"11937", &P11937_4};	/*Profile ID - 11937*/
const Value V11940_1_1 = {S2, NULL};	/*Value = True*/
const Property P11940_1 = {S50, 0, &V11940_1_1, NULL};	/*Property = IsMobile*/
const Value V11940_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P11940_5 = {S52, 0, &V11940_5_1333, &P11940_1};	/*Property = ScreenPixelsHeight*/
const Value V11940_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P11940_4 = {S54, 0, &V11940_4_1335, &P11940_5};	/*Property = ScreenPixelsWidth*/
const Profile P11940 = {"11940", &P11940_4};	/*Profile ID - 11940*/
const Value V15068_1_1 = {S2, NULL};	/*Value = True*/
const Property P15068_1 = {S50, 0, &V15068_1_1, NULL};	/*Property = IsMobile*/
const Value V15068_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15068_5 = {S52, 0, &V15068_5_1294, &P15068_1};	/*Property = ScreenPixelsHeight*/
const Value V15068_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15068_4 = {S54, 0, &V15068_4_1296, &P15068_5};	/*Property = ScreenPixelsWidth*/
const Profile P15068 = {"15068", &P15068_4};	/*Profile ID - 15068*/
const Value V11951_1_1 = {S2, NULL};	/*Value = True*/
const Property P11951_1 = {S50, 0, &V11951_1_1, NULL};	/*Property = IsMobile*/
const Value V11951_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11951_5 = {S52, 0, &V11951_5_1233, &P11951_1};	/*Property = ScreenPixelsHeight*/
const Value V11951_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11951_4 = {S54, 0, &V11951_4_1235, &P11951_5};	/*Property = ScreenPixelsWidth*/
const Profile P11951 = {"11951", &P11951_4};	/*Profile ID - 11951*/
const Value V16182_1_1 = {S2, NULL};	/*Value = True*/
const Property P16182_1 = {S50, 0, &V16182_1_1, NULL};	/*Property = IsMobile*/
const Value V16182_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16182_5 = {S52, 0, &V16182_5_1233, &P16182_1};	/*Property = ScreenPixelsHeight*/
const Value V16182_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16182_4 = {S54, 0, &V16182_4_1235, &P16182_5};	/*Property = ScreenPixelsWidth*/
const Profile P16182 = {"16182", &P16182_4};	/*Profile ID - 16182*/
const Value V13240_1_1 = {S2, NULL};	/*Value = True*/
const Property P13240_1 = {S50, 0, &V13240_1_1, NULL};	/*Property = IsMobile*/
const Value V13240_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13240_5 = {S52, 0, &V13240_5_1222, &P13240_1};	/*Property = ScreenPixelsHeight*/
const Value V13240_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13240_4 = {S54, 0, &V13240_4_1224, &P13240_5};	/*Property = ScreenPixelsWidth*/
const Profile P13240 = {"13240", &P13240_4};	/*Profile ID - 13240*/
const Value V12000_1_1 = {S2, NULL};	/*Value = True*/
const Property P12000_1 = {S50, 0, &V12000_1_1, NULL};	/*Property = IsMobile*/
const Value V12000_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12000_5 = {S52, 0, &V12000_5_1222, &P12000_1};	/*Property = ScreenPixelsHeight*/
const Value V12000_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12000_4 = {S54, 0, &V12000_4_1224, &P12000_5};	/*Property = ScreenPixelsWidth*/
const Profile P12000 = {"12000", &P12000_4};	/*Profile ID - 12000*/
const Value V11956_1_1 = {S2, NULL};	/*Value = True*/
const Property P11956_1 = {S50, 0, &V11956_1_1, NULL};	/*Property = IsMobile*/
const Value V11956_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11956_5 = {S52, 0, &V11956_5_1222, &P11956_1};	/*Property = ScreenPixelsHeight*/
const Value V11956_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11956_4 = {S54, 0, &V11956_4_1224, &P11956_5};	/*Property = ScreenPixelsWidth*/
const Profile P11956 = {"11956", &P11956_4};	/*Profile ID - 11956*/
const Value V14163_1_1 = {S2, NULL};	/*Value = True*/
const Property P14163_1 = {S50, 0, &V14163_1_1, NULL};	/*Property = IsMobile*/
const Value V14163_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14163_5 = {S52, 0, &V14163_5_1233, &P14163_1};	/*Property = ScreenPixelsHeight*/
const Value V14163_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14163_4 = {S54, 0, &V14163_4_1235, &P14163_5};	/*Property = ScreenPixelsWidth*/
const Profile P14163 = {"14163", &P14163_4};	/*Profile ID - 14163*/
const Value V14588_1_1 = {S2, NULL};	/*Value = True*/
const Property P14588_1 = {S50, 0, &V14588_1_1, NULL};	/*Property = IsMobile*/
const Value V14588_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14588_5 = {S52, 0, &V14588_5_1222, &P14588_1};	/*Property = ScreenPixelsHeight*/
const Value V14588_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14588_4 = {S54, 0, &V14588_4_1224, &P14588_5};	/*Property = ScreenPixelsWidth*/
const Profile P14588 = {"14588", &P14588_4};	/*Profile ID - 14588*/
const Value V14168_1_1 = {S2, NULL};	/*Value = True*/
const Property P14168_1 = {S50, 0, &V14168_1_1, NULL};	/*Property = IsMobile*/
const Value V14168_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14168_5 = {S52, 0, &V14168_5_1222, &P14168_1};	/*Property = ScreenPixelsHeight*/
const Value V14168_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14168_4 = {S54, 0, &V14168_4_1224, &P14168_5};	/*Property = ScreenPixelsWidth*/
const Profile P14168 = {"14168", &P14168_4};	/*Profile ID - 14168*/
const Value V14176_1_1 = {S2, NULL};	/*Value = True*/
const Property P14176_1 = {S50, 0, &V14176_1_1, NULL};	/*Property = IsMobile*/
const Value V14176_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14176_5 = {S52, 0, &V14176_5_1222, &P14176_1};	/*Property = ScreenPixelsHeight*/
const Value V14176_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14176_4 = {S54, 0, &V14176_4_1224, &P14176_5};	/*Property = ScreenPixelsWidth*/
const Profile P14176 = {"14176", &P14176_4};	/*Profile ID - 14176*/
const Value V14180_1_1 = {S2, NULL};	/*Value = True*/
const Property P14180_1 = {S50, 0, &V14180_1_1, NULL};	/*Property = IsMobile*/
const Value V14180_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14180_5 = {S52, 0, &V14180_5_1227, &P14180_1};	/*Property = ScreenPixelsHeight*/
const Value V14180_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14180_4 = {S54, 0, &V14180_4_1229, &P14180_5};	/*Property = ScreenPixelsWidth*/
const Profile P14180 = {"14180", &P14180_4};	/*Profile ID - 14180*/
const Value V12896_1_1 = {S2, NULL};	/*Value = True*/
const Property P12896_1 = {S50, 0, &V12896_1_1, NULL};	/*Property = IsMobile*/
const Value V12896_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12896_5 = {S52, 0, &V12896_5_1227, &P12896_1};	/*Property = ScreenPixelsHeight*/
const Value V12896_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12896_4 = {S54, 0, &V12896_4_1229, &P12896_5};	/*Property = ScreenPixelsWidth*/
const Profile P12896 = {"12896", &P12896_4};	/*Profile ID - 12896*/
const Value V14186_1_1 = {S2, NULL};	/*Value = True*/
const Property P14186_1 = {S50, 0, &V14186_1_1, NULL};	/*Property = IsMobile*/
const Value V14186_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14186_5 = {S52, 0, &V14186_5_1277, &P14186_1};	/*Property = ScreenPixelsHeight*/
const Value V14186_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14186_4 = {S54, 0, &V14186_4_1279, &P14186_5};	/*Property = ScreenPixelsWidth*/
const Profile P14186 = {"14186", &P14186_4};	/*Profile ID - 14186*/
const Value V12255_1_1 = {S2, NULL};	/*Value = True*/
const Property P12255_1 = {S50, 0, &V12255_1_1, NULL};	/*Property = IsMobile*/
const Value V12255_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12255_5 = {S52, 0, &V12255_5_1293, &P12255_1};	/*Property = ScreenPixelsHeight*/
const Value V12255_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12255_4 = {S54, 0, &V12255_4_1296, &P12255_5};	/*Property = ScreenPixelsWidth*/
const Profile P12255 = {"12255", &P12255_4};	/*Profile ID - 12255*/
const Value V14189_1_1 = {S2, NULL};	/*Value = True*/
const Property P14189_1 = {S50, 0, &V14189_1_1, NULL};	/*Property = IsMobile*/
const Value V14189_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14189_5 = {S52, 0, &V14189_5_1233, &P14189_1};	/*Property = ScreenPixelsHeight*/
const Value V14189_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14189_4 = {S54, 0, &V14189_4_1235, &P14189_5};	/*Property = ScreenPixelsWidth*/
const Profile P14189 = {"14189", &P14189_4};	/*Profile ID - 14189*/
const Value V12435_1_1 = {S2, NULL};	/*Value = True*/
const Property P12435_1 = {S50, 0, &V12435_1_1, NULL};	/*Property = IsMobile*/
const Value V12435_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12435_5 = {S52, 0, &V12435_5_1222, &P12435_1};	/*Property = ScreenPixelsHeight*/
const Value V12435_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12435_4 = {S54, 0, &V12435_4_1224, &P12435_5};	/*Property = ScreenPixelsWidth*/
const Profile P12435 = {"12435", &P12435_4};	/*Profile ID - 12435*/
const Value V15006_1_1 = {S2, NULL};	/*Value = True*/
const Property P15006_1 = {S50, 0, &V15006_1_1, NULL};	/*Property = IsMobile*/
const Value V15006_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15006_5 = {S52, 0, &V15006_5_1227, &P15006_1};	/*Property = ScreenPixelsHeight*/
const Value V15006_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15006_4 = {S54, 0, &V15006_4_1229, &P15006_5};	/*Property = ScreenPixelsWidth*/
const Profile P15006 = {"15006", &P15006_4};	/*Profile ID - 15006*/
const Value V13724_1_1 = {S2, NULL};	/*Value = True*/
const Property P13724_1 = {S50, 0, &V13724_1_1, NULL};	/*Property = IsMobile*/
const Value V13724_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13724_5 = {S52, 0, &V13724_5_1227, &P13724_1};	/*Property = ScreenPixelsHeight*/
const Value V13724_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13724_4 = {S54, 0, &V13724_4_1229, &P13724_5};	/*Property = ScreenPixelsWidth*/
const Profile P13724 = {"13724", &P13724_4};	/*Profile ID - 13724*/
const Value V15649_1_1 = {S2, NULL};	/*Value = True*/
const Property P15649_1 = {S50, 0, &V15649_1_1, NULL};	/*Property = IsMobile*/
const Value V15649_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15649_5 = {S52, 0, &V15649_5_1222, &P15649_1};	/*Property = ScreenPixelsHeight*/
const Value V15649_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15649_4 = {S54, 0, &V15649_4_1224, &P15649_5};	/*Property = ScreenPixelsWidth*/
const Profile P15649 = {"15649", &P15649_4};	/*Profile ID - 15649*/
const Value V15007_1_1 = {S2, NULL};	/*Value = True*/
const Property P15007_1 = {S50, 0, &V15007_1_1, NULL};	/*Property = IsMobile*/
const Value V15007_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15007_5 = {S52, 0, &V15007_5_1222, &P15007_1};	/*Property = ScreenPixelsHeight*/
const Value V15007_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15007_4 = {S54, 0, &V15007_4_1224, &P15007_5};	/*Property = ScreenPixelsWidth*/
const Profile P15007 = {"15007", &P15007_4};	/*Profile ID - 15007*/
const Value V14195_1_1 = {S2, NULL};	/*Value = True*/
const Property P14195_1 = {S50, 0, &V14195_1_1, NULL};	/*Property = IsMobile*/
const Value V14195_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14195_5 = {S52, 0, &V14195_5_1222, &P14195_1};	/*Property = ScreenPixelsHeight*/
const Value V14195_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14195_4 = {S54, 0, &V14195_4_1224, &P14195_5};	/*Property = ScreenPixelsWidth*/
const Profile P14195 = {"14195", &P14195_4};	/*Profile ID - 14195*/
const Value V19976_1_1 = {S2, NULL};	/*Value = True*/
const Property P19976_1 = {S50, 0, &V19976_1_1, NULL};	/*Property = IsMobile*/
const Value V19976_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P19976_5 = {S52, 0, &V19976_5_1275, &P19976_1};	/*Property = ScreenPixelsHeight*/
const Value V19976_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19976_4 = {S54, 0, &V19976_4_1229, &P19976_5};	/*Property = ScreenPixelsWidth*/
const Profile P19976 = {"19976", &P19976_4};	/*Profile ID - 19976*/
const Value V14207_1_1 = {S2, NULL};	/*Value = True*/
const Property P14207_1 = {S50, 0, &V14207_1_1, NULL};	/*Property = IsMobile*/
const Value V14207_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14207_5 = {S52, 0, &V14207_5_1227, &P14207_1};	/*Property = ScreenPixelsHeight*/
const Value V14207_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14207_4 = {S54, 0, &V14207_4_1229, &P14207_5};	/*Property = ScreenPixelsWidth*/
const Profile P14207 = {"14207", &P14207_4};	/*Profile ID - 14207*/
const Profile P22832 = {"22832", NULL};	/*Profile ID - 22832*/
const Value V20279_1_1 = {S2, NULL};	/*Value = True*/
const Property P20279_1 = {S50, 0, &V20279_1_1, NULL};	/*Property = IsMobile*/
const Value V20279_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20279_5 = {S52, 0, &V20279_5_1222, &P20279_1};	/*Property = ScreenPixelsHeight*/
const Value V20279_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20279_4 = {S54, 0, &V20279_4_1224, &P20279_5};	/*Property = ScreenPixelsWidth*/
const Profile P20279 = {"20279", &P20279_4};	/*Profile ID - 20279*/
const Value V13685_1_1 = {S2, NULL};	/*Value = True*/
const Property P13685_1 = {S50, 0, &V13685_1_1, NULL};	/*Property = IsMobile*/
const Value V13685_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13685_5 = {S52, 0, &V13685_5_1277, &P13685_1};	/*Property = ScreenPixelsHeight*/
const Value V13685_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13685_4 = {S54, 0, &V13685_4_1279, &P13685_5};	/*Property = ScreenPixelsWidth*/
const Profile P13685 = {"13685", &P13685_4};	/*Profile ID - 13685*/
const Value V4841_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_279 = {S1, 0, &V4841_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4841_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_282 = {S3, 0, &V4841_282_57155, &P4841_279};	/*Property = BlobBuilder*/
const Value V4841_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_350 = {S4, 0, &V4841_350_57446, &P4841_282};	/*Property = Canvas*/
const Value V4841_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_283 = {S5, 0, &V4841_283_57161, &P4841_350};	/*Property = CssBackground*/
const Value V4841_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_284 = {S6, 0, &V4841_284_57164, &P4841_283};	/*Property = CssBorderImage*/
const Value V4841_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_285 = {S7, 0, &V4841_285_57167, &P4841_284};	/*Property = CssCanvas*/
const Value V4841_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_286 = {S8, 0, &V4841_286_57170, &P4841_285};	/*Property = CssColor*/
const Value V4841_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_237 = {S9, 0, &V4841_237_55786, &P4841_286};	/*Property = CssColumn*/
const Value V4841_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_287 = {S10, 0, &V4841_287_57173, &P4841_237};	/*Property = CssFlexbox*/
const Value V4841_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_288 = {S11, 0, &V4841_288_57176, &P4841_287};	/*Property = CssFont*/
const Value V4841_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_289 = {S12, 0, &V4841_289_86899, &P4841_288};	/*Property = CssImages*/
const Value V4841_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_291 = {S13, 0, &V4841_291_57182, &P4841_289};	/*Property = CssMediaQueries*/
const Value V4841_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_292 = {S14, 0, &V4841_292_57185, &P4841_291};	/*Property = CssMinMax*/
const Value V4841_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_293 = {S15, 0, &V4841_293_57188, &P4841_292};	/*Property = CssOverflow*/
const Value V4841_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_294 = {S16, 0, &V4841_294_57191, &P4841_293};	/*Property = CssPosition*/
const Value V4841_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_295 = {S17, 0, &V4841_295_57194, &P4841_294};	/*Property = CssText*/
const Value V4841_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_297 = {S18, 0, &V4841_297_57197, &P4841_295};	/*Property = CssTransforms*/
const Value V4841_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_241 = {S19, 0, &V4841_241_55801, &P4841_297};	/*Property = CssTransitions*/
const Value V4841_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_298 = {S20, 0, &V4841_298_57208, &P4841_241};	/*Property = CssUI*/
const Value V4841_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_300 = {S21, 0, &V4841_300_57217, &P4841_298};	/*Property = DataSet*/
const Value V4841_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_301 = {S22, 0, &V4841_301_57220, &P4841_300};	/*Property = DataUrl*/
const Value V4841_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_302 = {S23, 0, &V4841_302_57223, &P4841_301};	/*Property = DeviceOrientation*/
const Value V4841_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_304 = {S24, 0, &V4841_304_57229, &P4841_302};	/*Property = FileReader*/
const Value V4841_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_305 = {S25, 0, &V4841_305_57232, &P4841_304};	/*Property = FileSaver*/
const Value V4841_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_306 = {S26, 0, &V4841_306_57238, &P4841_305};	/*Property = FileWriter*/
const Value V4841_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_307 = {S27, 0, &V4841_307_57241, &P4841_306};	/*Property = FormData*/
const Value V4841_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_309 = {S28, 0, &V4841_309_57247, &P4841_307};	/*Property = Fullscreen*/
const Value V4841_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_310 = {S29, 0, &V4841_310_57250, &P4841_309};	/*Property = GeoLocation*/
const Value V4841_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_312 = {S30, 0, &V4841_312_57256, &P4841_310};	/*Property = History*/
const Value V4841_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_314 = {S31, 0, &V4841_314_57262, &P4841_312};	/*Property = Html5*/
const Value V4841_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_313 = {S32, 0, &V4841_313_57259, &P4841_314};	/*Property = Html-Media-Capture*/
const Value V4841_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_315 = {S33, 0, &V4841_315_57265, &P4841_313};	/*Property = Iframe*/
const Value V4841_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_316 = {S34, 0, &V4841_316_57268, &P4841_315};	/*Property = IndexedDB*/
const Value V4841_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_317 = {S35, 0, &V4841_317_57271, &P4841_316};	/*Property = Json*/
const Value V4841_15_43265 = {S88, NULL};	/*Value = Lumi*/
const Property P4841_15 = {S37, 0, &V4841_15_43265, &P4841_317};	/*Property = LayoutEngine*/
const Value V4841_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_344 = {S38, 0, &V4841_344_57274, &P4841_15};	/*Property = Masking*/
const Value V4841_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_320 = {S39, 0, &V4841_320_57292, &P4841_344};	/*Property = PostMessage*/
const Value V4841_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_321 = {S40, 0, &V4841_321_57295, &P4841_320};	/*Property = Progress*/
const Value V4841_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_322 = {S41, 0, &V4841_322_57298, &P4841_321};	/*Property = Prompts*/
const Value V4841_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_323 = {S42, 0, &V4841_323_57301, &P4841_322};	/*Property = Selector*/
const Value V4841_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_324 = {S43, 0, &V4841_324_57307, &P4841_323};	/*Property = Svg*/
const Value V4841_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_327 = {S44, 0, &V4841_327_57316, &P4841_324};	/*Property = TouchEvents*/
const Value V4841_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_328 = {S45, 0, &V4841_328_57319, &P4841_327};	/*Property = Track*/
const Value V4841_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_329 = {S46, 0, &V4841_329_57325, &P4841_328};	/*Property = Video*/
const Value V4841_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_330 = {S47, 0, &V4841_330_57328, &P4841_329};	/*Property = Viewport*/
const Value V4841_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_332 = {S48, 0, &V4841_332_57340, &P4841_330};	/*Property = WebWorkers*/
const Value V4841_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4841_333 = {S49, 0, &V4841_333_57343, &P4841_332};	/*Property = Xhr2*/
const Profile P4841 = {"4841", &P4841_333};	/*Profile ID - 4841*/
const Value V15010_1_1 = {S2, NULL};	/*Value = True*/
const Property P15010_1 = {S50, 0, &V15010_1_1, NULL};	/*Property = IsMobile*/
const Value V15010_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15010_5 = {S52, 0, &V15010_5_1222, &P15010_1};	/*Property = ScreenPixelsHeight*/
const Value V15010_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15010_4 = {S54, 0, &V15010_4_1224, &P15010_5};	/*Property = ScreenPixelsWidth*/
const Profile P15010 = {"15010", &P15010_4};	/*Profile ID - 15010*/
const Value V17613_1_1 = {S2, NULL};	/*Value = True*/
const Property P17613_1 = {S50, 0, &V17613_1_1, NULL};	/*Property = IsMobile*/
const Value V17613_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P17613_5 = {S52, 0, &V17613_5_1294, &P17613_1};	/*Property = ScreenPixelsHeight*/
const Value V17613_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17613_4 = {S54, 0, &V17613_4_1296, &P17613_5};	/*Property = ScreenPixelsWidth*/
const Profile P17613 = {"17613", &P17613_4};	/*Profile ID - 17613*/
const Value V14230_1_1 = {S2, NULL};	/*Value = True*/
const Property P14230_1 = {S50, 0, &V14230_1_1, NULL};	/*Property = IsMobile*/
const Value V14230_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14230_5 = {S52, 0, &V14230_5_1293, &P14230_1};	/*Property = ScreenPixelsHeight*/
const Value V14230_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14230_4 = {S54, 0, &V14230_4_1296, &P14230_5};	/*Property = ScreenPixelsWidth*/
const Profile P14230 = {"14230", &P14230_4};	/*Profile ID - 14230*/
const Value V15011_1_1 = {S2, NULL};	/*Value = True*/
const Property P15011_1 = {S50, 0, &V15011_1_1, NULL};	/*Property = IsMobile*/
const Value V15011_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15011_5 = {S52, 0, &V15011_5_1277, &P15011_1};	/*Property = ScreenPixelsHeight*/
const Value V15011_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15011_4 = {S54, 0, &V15011_4_1279, &P15011_5};	/*Property = ScreenPixelsWidth*/
const Profile P15011 = {"15011", &P15011_4};	/*Profile ID - 15011*/
const Value V15014_1_1 = {S2, NULL};	/*Value = True*/
const Property P15014_1 = {S50, 0, &V15014_1_1, NULL};	/*Property = IsMobile*/
const Value V15014_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15014_5 = {S52, 0, &V15014_5_1222, &P15014_1};	/*Property = ScreenPixelsHeight*/
const Value V15014_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15014_4 = {S54, 0, &V15014_4_1224, &P15014_5};	/*Property = ScreenPixelsWidth*/
const Profile P15014 = {"15014", &P15014_4};	/*Profile ID - 15014*/
const Value V15015_1_1 = {S2, NULL};	/*Value = True*/
const Property P15015_1 = {S50, 0, &V15015_1_1, NULL};	/*Property = IsMobile*/
const Value V15015_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15015_5 = {S52, 0, &V15015_5_1293, &P15015_1};	/*Property = ScreenPixelsHeight*/
const Value V15015_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15015_4 = {S54, 0, &V15015_4_1296, &P15015_5};	/*Property = ScreenPixelsWidth*/
const Profile P15015 = {"15015", &P15015_4};	/*Profile ID - 15015*/
const Value V15016_1_1 = {S2, NULL};	/*Value = True*/
const Property P15016_1 = {S50, 0, &V15016_1_1, NULL};	/*Property = IsMobile*/
const Value V15016_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15016_5 = {S52, 0, &V15016_5_1293, &P15016_1};	/*Property = ScreenPixelsHeight*/
const Value V15016_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15016_4 = {S54, 0, &V15016_4_1296, &P15016_5};	/*Property = ScreenPixelsWidth*/
const Profile P15016 = {"15016", &P15016_4};	/*Profile ID - 15016*/
const Value V15018_1_1 = {S2, NULL};	/*Value = True*/
const Property P15018_1 = {S50, 0, &V15018_1_1, NULL};	/*Property = IsMobile*/
const Value V15018_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15018_5 = {S52, 0, &V15018_5_1293, &P15018_1};	/*Property = ScreenPixelsHeight*/
const Value V15018_4_24031 = {S71, NULL};	/*Value = 120*/
const Property P15018_4 = {S54, 0, &V15018_4_24031, &P15018_5};	/*Property = ScreenPixelsWidth*/
const Profile P15018 = {"15018", &P15018_4};	/*Profile ID - 15018*/
const Profile P19944 = {"19944", NULL};	/*Profile ID - 19944*/
const Value V15017_1_1 = {S2, NULL};	/*Value = True*/
const Property P15017_1 = {S50, 0, &V15017_1_1, NULL};	/*Property = IsMobile*/
const Value V15017_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15017_5 = {S52, 0, &V15017_5_1227, &P15017_1};	/*Property = ScreenPixelsHeight*/
const Value V15017_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15017_4 = {S54, 0, &V15017_4_1229, &P15017_5};	/*Property = ScreenPixelsWidth*/
const Profile P15017 = {"15017", &P15017_4};	/*Profile ID - 15017*/
const Value V11991_1_1 = {S2, NULL};	/*Value = True*/
const Property P11991_1 = {S50, 0, &V11991_1_1, NULL};	/*Property = IsMobile*/
const Value V11991_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P11991_5 = {S52, 0, &V11991_5_1304, &P11991_1};	/*Property = ScreenPixelsHeight*/
const Value V11991_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11991_4 = {S54, 0, &V11991_4_1224, &P11991_5};	/*Property = ScreenPixelsWidth*/
const Profile P11991 = {"11991", &P11991_4};	/*Profile ID - 11991*/
const Value V15021_1_1 = {S2, NULL};	/*Value = True*/
const Property P15021_1 = {S50, 0, &V15021_1_1, NULL};	/*Property = IsMobile*/
const Value V15021_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15021_5 = {S52, 0, &V15021_5_1293, &P15021_1};	/*Property = ScreenPixelsHeight*/
const Value V15021_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15021_4 = {S54, 0, &V15021_4_1296, &P15021_5};	/*Property = ScreenPixelsWidth*/
const Profile P15021 = {"15021", &P15021_4};	/*Profile ID - 15021*/
const Value V15022_1_1 = {S2, NULL};	/*Value = True*/
const Property P15022_1 = {S50, 0, &V15022_1_1, NULL};	/*Property = IsMobile*/
const Value V15022_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15022_5 = {S52, 0, &V15022_5_1294, &P15022_1};	/*Property = ScreenPixelsHeight*/
const Value V15022_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15022_4 = {S54, 0, &V15022_4_1296, &P15022_5};	/*Property = ScreenPixelsWidth*/
const Profile P15022 = {"15022", &P15022_4};	/*Profile ID - 15022*/
const Value V3866_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_279 = {S1, 0, &V3866_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3866_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_282 = {S3, 0, &V3866_282_57155, &P3866_279};	/*Property = BlobBuilder*/
const Value V3866_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_350 = {S4, 0, &V3866_350_57446, &P3866_282};	/*Property = Canvas*/
const Value V3866_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_283 = {S5, 0, &V3866_283_57161, &P3866_350};	/*Property = CssBackground*/
const Value V3866_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_284 = {S6, 0, &V3866_284_57164, &P3866_283};	/*Property = CssBorderImage*/
const Value V3866_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_285 = {S7, 0, &V3866_285_57167, &P3866_284};	/*Property = CssCanvas*/
const Value V3866_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_286 = {S8, 0, &V3866_286_57170, &P3866_285};	/*Property = CssColor*/
const Value V3866_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_237 = {S9, 0, &V3866_237_55786, &P3866_286};	/*Property = CssColumn*/
const Value V3866_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_287 = {S10, 0, &V3866_287_57173, &P3866_237};	/*Property = CssFlexbox*/
const Value V3866_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_288 = {S11, 0, &V3866_288_57176, &P3866_287};	/*Property = CssFont*/
const Value V3866_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_289 = {S12, 0, &V3866_289_86899, &P3866_288};	/*Property = CssImages*/
const Value V3866_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_291 = {S13, 0, &V3866_291_57182, &P3866_289};	/*Property = CssMediaQueries*/
const Value V3866_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_292 = {S14, 0, &V3866_292_57185, &P3866_291};	/*Property = CssMinMax*/
const Value V3866_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_293 = {S15, 0, &V3866_293_57188, &P3866_292};	/*Property = CssOverflow*/
const Value V3866_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_294 = {S16, 0, &V3866_294_57191, &P3866_293};	/*Property = CssPosition*/
const Value V3866_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_295 = {S17, 0, &V3866_295_57194, &P3866_294};	/*Property = CssText*/
const Value V3866_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_297 = {S18, 0, &V3866_297_57197, &P3866_295};	/*Property = CssTransforms*/
const Value V3866_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_241 = {S19, 0, &V3866_241_55801, &P3866_297};	/*Property = CssTransitions*/
const Value V3866_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_298 = {S20, 0, &V3866_298_57208, &P3866_241};	/*Property = CssUI*/
const Value V3866_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_300 = {S21, 0, &V3866_300_57217, &P3866_298};	/*Property = DataSet*/
const Value V3866_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_301 = {S22, 0, &V3866_301_57220, &P3866_300};	/*Property = DataUrl*/
const Value V3866_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_302 = {S23, 0, &V3866_302_57223, &P3866_301};	/*Property = DeviceOrientation*/
const Value V3866_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_304 = {S24, 0, &V3866_304_57229, &P3866_302};	/*Property = FileReader*/
const Value V3866_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_305 = {S25, 0, &V3866_305_57232, &P3866_304};	/*Property = FileSaver*/
const Value V3866_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_306 = {S26, 0, &V3866_306_57238, &P3866_305};	/*Property = FileWriter*/
const Value V3866_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_307 = {S27, 0, &V3866_307_57241, &P3866_306};	/*Property = FormData*/
const Value V3866_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_309 = {S28, 0, &V3866_309_57247, &P3866_307};	/*Property = Fullscreen*/
const Value V3866_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_310 = {S29, 0, &V3866_310_57250, &P3866_309};	/*Property = GeoLocation*/
const Value V3866_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_312 = {S30, 0, &V3866_312_57256, &P3866_310};	/*Property = History*/
const Value V3866_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_314 = {S31, 0, &V3866_314_57262, &P3866_312};	/*Property = Html5*/
const Value V3866_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_313 = {S32, 0, &V3866_313_57259, &P3866_314};	/*Property = Html-Media-Capture*/
const Value V3866_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_315 = {S33, 0, &V3866_315_57265, &P3866_313};	/*Property = Iframe*/
const Value V3866_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_316 = {S34, 0, &V3866_316_57268, &P3866_315};	/*Property = IndexedDB*/
const Value V3866_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_317 = {S35, 0, &V3866_317_57271, &P3866_316};	/*Property = Json*/
const Value V3866_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3866_15 = {S37, 0, &V3866_15_28, &P3866_317};	/*Property = LayoutEngine*/
const Value V3866_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_344 = {S38, 0, &V3866_344_57274, &P3866_15};	/*Property = Masking*/
const Value V3866_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_320 = {S39, 0, &V3866_320_57292, &P3866_344};	/*Property = PostMessage*/
const Value V3866_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_321 = {S40, 0, &V3866_321_57295, &P3866_320};	/*Property = Progress*/
const Value V3866_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_322 = {S41, 0, &V3866_322_57298, &P3866_321};	/*Property = Prompts*/
const Value V3866_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_323 = {S42, 0, &V3866_323_57301, &P3866_322};	/*Property = Selector*/
const Value V3866_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_324 = {S43, 0, &V3866_324_57307, &P3866_323};	/*Property = Svg*/
const Value V3866_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_327 = {S44, 0, &V3866_327_57316, &P3866_324};	/*Property = TouchEvents*/
const Value V3866_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_328 = {S45, 0, &V3866_328_57319, &P3866_327};	/*Property = Track*/
const Value V3866_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_329 = {S46, 0, &V3866_329_57325, &P3866_328};	/*Property = Video*/
const Value V3866_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_330 = {S47, 0, &V3866_330_57328, &P3866_329};	/*Property = Viewport*/
const Value V3866_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_332 = {S48, 0, &V3866_332_57340, &P3866_330};	/*Property = WebWorkers*/
const Value V3866_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3866_333 = {S49, 0, &V3866_333_57343, &P3866_332};	/*Property = Xhr2*/
const Profile P3866 = {"3866", &P3866_333};	/*Profile ID - 3866*/
const Value V15023_1_1 = {S2, NULL};	/*Value = True*/
const Property P15023_1 = {S50, 0, &V15023_1_1, NULL};	/*Property = IsMobile*/
const Value V15023_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15023_5 = {S52, 0, &V15023_5_1222, &P15023_1};	/*Property = ScreenPixelsHeight*/
const Value V15023_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15023_4 = {S54, 0, &V15023_4_1224, &P15023_5};	/*Property = ScreenPixelsWidth*/
const Profile P15023 = {"15023", &P15023_4};	/*Profile ID - 15023*/
const Value V15024_1_1 = {S2, NULL};	/*Value = True*/
const Property P15024_1 = {S50, 0, &V15024_1_1, NULL};	/*Property = IsMobile*/
const Value V15024_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15024_5 = {S52, 0, &V15024_5_1304, &P15024_1};	/*Property = ScreenPixelsHeight*/
const Value V15024_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15024_4 = {S54, 0, &V15024_4_1224, &P15024_5};	/*Property = ScreenPixelsWidth*/
const Profile P15024 = {"15024", &P15024_4};	/*Profile ID - 15024*/
const Value V15025_1_1 = {S2, NULL};	/*Value = True*/
const Property P15025_1 = {S50, 0, &V15025_1_1, NULL};	/*Property = IsMobile*/
const Value V15025_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15025_5 = {S52, 0, &V15025_5_1293, &P15025_1};	/*Property = ScreenPixelsHeight*/
const Value V15025_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15025_4 = {S54, 0, &V15025_4_1296, &P15025_5};	/*Property = ScreenPixelsWidth*/
const Profile P15025 = {"15025", &P15025_4};	/*Profile ID - 15025*/
const Value V15028_1_1 = {S2, NULL};	/*Value = True*/
const Property P15028_1 = {S50, 0, &V15028_1_1, NULL};	/*Property = IsMobile*/
const Value V15028_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15028_5 = {S52, 0, &V15028_5_1277, &P15028_1};	/*Property = ScreenPixelsHeight*/
const Value V15028_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15028_4 = {S54, 0, &V15028_4_1279, &P15028_5};	/*Property = ScreenPixelsWidth*/
const Profile P15028 = {"15028", &P15028_4};	/*Profile ID - 15028*/
const Value V13653_1_1 = {S2, NULL};	/*Value = True*/
const Property P13653_1 = {S50, 0, &V13653_1_1, NULL};	/*Property = IsMobile*/
const Value V13653_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13653_5 = {S52, 0, &V13653_5_1380, &P13653_1};	/*Property = ScreenPixelsHeight*/
const Value V13653_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13653_4 = {S54, 0, &V13653_4_1279, &P13653_5};	/*Property = ScreenPixelsWidth*/
const Profile P13653 = {"13653", &P13653_4};	/*Profile ID - 13653*/
const Value V14233_1_1 = {S2, NULL};	/*Value = True*/
const Property P14233_1 = {S50, 0, &V14233_1_1, NULL};	/*Property = IsMobile*/
const Value V14233_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14233_5 = {S52, 0, &V14233_5_1294, &P14233_1};	/*Property = ScreenPixelsHeight*/
const Value V14233_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14233_4 = {S54, 0, &V14233_4_1296, &P14233_5};	/*Property = ScreenPixelsWidth*/
const Profile P14233 = {"14233", &P14233_4};	/*Profile ID - 14233*/
const Value V15030_1_1 = {S2, NULL};	/*Value = True*/
const Property P15030_1 = {S50, 0, &V15030_1_1, NULL};	/*Property = IsMobile*/
const Value V15030_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15030_5 = {S52, 0, &V15030_5_1304, &P15030_1};	/*Property = ScreenPixelsHeight*/
const Value V15030_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15030_4 = {S54, 0, &V15030_4_1224, &P15030_5};	/*Property = ScreenPixelsWidth*/
const Profile P15030 = {"15030", &P15030_4};	/*Profile ID - 15030*/
const Value V14566_1_1 = {S2, NULL};	/*Value = True*/
const Property P14566_1 = {S50, 0, &V14566_1_1, NULL};	/*Property = IsMobile*/
const Value V14566_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14566_5 = {S52, 0, &V14566_5_1293, &P14566_1};	/*Property = ScreenPixelsHeight*/
const Value V14566_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14566_4 = {S54, 0, &V14566_4_1296, &P14566_5};	/*Property = ScreenPixelsWidth*/
const Profile P14566 = {"14566", &P14566_4};	/*Profile ID - 14566*/
const Value V15035_1_1 = {S2, NULL};	/*Value = True*/
const Property P15035_1 = {S50, 0, &V15035_1_1, NULL};	/*Property = IsMobile*/
const Value V15035_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15035_5 = {S52, 0, &V15035_5_1304, &P15035_1};	/*Property = ScreenPixelsHeight*/
const Value V15035_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15035_4 = {S54, 0, &V15035_4_1224, &P15035_5};	/*Property = ScreenPixelsWidth*/
const Profile P15035 = {"15035", &P15035_4};	/*Profile ID - 15035*/
const Value V17510_1_1 = {S2, NULL};	/*Value = True*/
const Property P17510_1 = {S50, 0, &V17510_1_1, NULL};	/*Property = IsMobile*/
const Value V17510_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17510_5 = {S52, 0, &V17510_5_1304, &P17510_1};	/*Property = ScreenPixelsHeight*/
const Value V17510_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17510_4 = {S54, 0, &V17510_4_1224, &P17510_5};	/*Property = ScreenPixelsWidth*/
const Profile P17510 = {"17510", &P17510_4};	/*Profile ID - 17510*/
const Profile P17662 = {"17662", NULL};	/*Profile ID - 17662*/
const Value V14243_1_1 = {S2, NULL};	/*Value = True*/
const Property P14243_1 = {S50, 0, &V14243_1_1, NULL};	/*Property = IsMobile*/
const Value V14243_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14243_5 = {S52, 0, &V14243_5_1294, &P14243_1};	/*Property = ScreenPixelsHeight*/
const Value V14243_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14243_4 = {S54, 0, &V14243_4_1296, &P14243_5};	/*Property = ScreenPixelsWidth*/
const Profile P14243 = {"14243", &P14243_4};	/*Profile ID - 14243*/
const Value V17463_1_1 = {S2, NULL};	/*Value = True*/
const Property P17463_1 = {S50, 0, &V17463_1_1, NULL};	/*Property = IsMobile*/
const Value V17463_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17463_5 = {S52, 0, &V17463_5_1233, &P17463_1};	/*Property = ScreenPixelsHeight*/
const Value V17463_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17463_4 = {S54, 0, &V17463_4_1235, &P17463_5};	/*Property = ScreenPixelsWidth*/
const Profile P17463 = {"17463", &P17463_4};	/*Profile ID - 17463*/
const Value V15639_1_1 = {S2, NULL};	/*Value = True*/
const Property P15639_1 = {S50, 0, &V15639_1_1, NULL};	/*Property = IsMobile*/
const Value V15639_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15639_5 = {S52, 0, &V15639_5_1222, &P15639_1};	/*Property = ScreenPixelsHeight*/
const Value V15639_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15639_4 = {S54, 0, &V15639_4_1224, &P15639_5};	/*Property = ScreenPixelsWidth*/
const Profile P15639 = {"15639", &P15639_4};	/*Profile ID - 15639*/
const Value V15039_1_1 = {S2, NULL};	/*Value = True*/
const Property P15039_1 = {S50, 0, &V15039_1_1, NULL};	/*Property = IsMobile*/
const Value V15039_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15039_5 = {S52, 0, &V15039_5_1293, &P15039_1};	/*Property = ScreenPixelsHeight*/
const Value V15039_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15039_4 = {S54, 0, &V15039_4_1296, &P15039_5};	/*Property = ScreenPixelsWidth*/
const Profile P15039 = {"15039", &P15039_4};	/*Profile ID - 15039*/
const Value V15311_1_1 = {S2, NULL};	/*Value = True*/
const Property P15311_1 = {S50, 0, &V15311_1_1, NULL};	/*Property = IsMobile*/
const Value V15311_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15311_5 = {S52, 0, &V15311_5_1233, &P15311_1};	/*Property = ScreenPixelsHeight*/
const Value V15311_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15311_4 = {S54, 0, &V15311_4_1235, &P15311_5};	/*Property = ScreenPixelsWidth*/
const Profile P15311 = {"15311", &P15311_4};	/*Profile ID - 15311*/
const Value V15040_1_1 = {S2, NULL};	/*Value = True*/
const Property P15040_1 = {S50, 0, &V15040_1_1, NULL};	/*Property = IsMobile*/
const Value V15040_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15040_5 = {S52, 0, &V15040_5_1233, &P15040_1};	/*Property = ScreenPixelsHeight*/
const Value V15040_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15040_4 = {S54, 0, &V15040_4_1235, &P15040_5};	/*Property = ScreenPixelsWidth*/
const Profile P15040 = {"15040", &P15040_4};	/*Profile ID - 15040*/
const Value V16602_1_1 = {S2, NULL};	/*Value = True*/
const Property P16602_1 = {S50, 0, &V16602_1_1, NULL};	/*Property = IsMobile*/
const Value V16602_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16602_5 = {S52, 0, &V16602_5_1222, &P16602_1};	/*Property = ScreenPixelsHeight*/
const Value V16602_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16602_4 = {S54, 0, &V16602_4_1224, &P16602_5};	/*Property = ScreenPixelsWidth*/
const Profile P16602 = {"16602", &P16602_4};	/*Profile ID - 16602*/
const Value V12383_1_1 = {S2, NULL};	/*Value = True*/
const Property P12383_1 = {S50, 0, &V12383_1_1, NULL};	/*Property = IsMobile*/
const Value V12383_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12383_5 = {S52, 0, &V12383_5_1233, &P12383_1};	/*Property = ScreenPixelsHeight*/
const Value V12383_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12383_4 = {S54, 0, &V12383_4_1235, &P12383_5};	/*Property = ScreenPixelsWidth*/
const Profile P12383 = {"12383", &P12383_4};	/*Profile ID - 12383*/
const Value V13640_1_1 = {S2, NULL};	/*Value = True*/
const Property P13640_1 = {S50, 0, &V13640_1_1, NULL};	/*Property = IsMobile*/
const Value V13640_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13640_5 = {S52, 0, &V13640_5_1222, &P13640_1};	/*Property = ScreenPixelsHeight*/
const Value V13640_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13640_4 = {S54, 0, &V13640_4_1224, &P13640_5};	/*Property = ScreenPixelsWidth*/
const Profile P13640 = {"13640", &P13640_4};	/*Profile ID - 13640*/
const Value V17473_1_1 = {S2, NULL};	/*Value = True*/
const Property P17473_1 = {S50, 0, &V17473_1_1, NULL};	/*Property = IsMobile*/
const Value V17473_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17473_5 = {S52, 0, &V17473_5_1222, &P17473_1};	/*Property = ScreenPixelsHeight*/
const Value V17473_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17473_4 = {S54, 0, &V17473_4_1224, &P17473_5};	/*Property = ScreenPixelsWidth*/
const Profile P17473 = {"17473", &P17473_4};	/*Profile ID - 17473*/
const Value V14174_1_1 = {S2, NULL};	/*Value = True*/
const Property P14174_1 = {S50, 0, &V14174_1_1, NULL};	/*Property = IsMobile*/
const Value V14174_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14174_5 = {S52, 0, &V14174_5_1293, &P14174_1};	/*Property = ScreenPixelsHeight*/
const Value V14174_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14174_4 = {S54, 0, &V14174_4_1296, &P14174_5};	/*Property = ScreenPixelsWidth*/
const Profile P14174 = {"14174", &P14174_4};	/*Profile ID - 14174*/
const Value V17319_1_1 = {S2, NULL};	/*Value = True*/
const Property P17319_1 = {S50, 0, &V17319_1_1, NULL};	/*Property = IsMobile*/
const Value V17319_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17319_5 = {S52, 0, &V17319_5_1293, &P17319_1};	/*Property = ScreenPixelsHeight*/
const Value V17319_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17319_4 = {S54, 0, &V17319_4_1296, &P17319_5};	/*Property = ScreenPixelsWidth*/
const Profile P17319 = {"17319", &P17319_4};	/*Profile ID - 17319*/
const Value V13892_1_1 = {S2, NULL};	/*Value = True*/
const Property P13892_1 = {S50, 0, &V13892_1_1, NULL};	/*Property = IsMobile*/
const Value V13892_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13892_5 = {S52, 0, &V13892_5_1222, &P13892_1};	/*Property = ScreenPixelsHeight*/
const Value V13892_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13892_4 = {S54, 0, &V13892_4_1224, &P13892_5};	/*Property = ScreenPixelsWidth*/
const Profile P13892 = {"13892", &P13892_4};	/*Profile ID - 13892*/
const Profile P5214 = {"5214", NULL};	/*Profile ID - 5214*/
const Value V15047_1_1 = {S2, NULL};	/*Value = True*/
const Property P15047_1 = {S50, 0, &V15047_1_1, NULL};	/*Property = IsMobile*/
const Value V15047_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15047_5 = {S52, 0, &V15047_5_1277, &P15047_1};	/*Property = ScreenPixelsHeight*/
const Value V15047_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15047_4 = {S54, 0, &V15047_4_1279, &P15047_5};	/*Property = ScreenPixelsWidth*/
const Profile P15047 = {"15047", &P15047_4};	/*Profile ID - 15047*/
const Value V15048_1_1 = {S2, NULL};	/*Value = True*/
const Property P15048_1 = {S50, 0, &V15048_1_1, NULL};	/*Property = IsMobile*/
const Value V15048_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15048_5 = {S52, 0, &V15048_5_1293, &P15048_1};	/*Property = ScreenPixelsHeight*/
const Value V15048_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15048_4 = {S54, 0, &V15048_4_1296, &P15048_5};	/*Property = ScreenPixelsWidth*/
const Profile P15048 = {"15048", &P15048_4};	/*Profile ID - 15048*/
const Value V15050_1_1 = {S2, NULL};	/*Value = True*/
const Property P15050_1 = {S50, 0, &V15050_1_1, NULL};	/*Property = IsMobile*/
const Value V15050_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15050_5 = {S52, 0, &V15050_5_1380, &P15050_1};	/*Property = ScreenPixelsHeight*/
const Value V15050_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15050_4 = {S54, 0, &V15050_4_1279, &P15050_5};	/*Property = ScreenPixelsWidth*/
const Profile P15050 = {"15050", &P15050_4};	/*Profile ID - 15050*/
const Value V13791_1_1 = {S2, NULL};	/*Value = True*/
const Property P13791_1 = {S50, 0, &V13791_1_1, NULL};	/*Property = IsMobile*/
const Value V13791_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13791_5 = {S52, 0, &V13791_5_1233, &P13791_1};	/*Property = ScreenPixelsHeight*/
const Value V13791_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13791_4 = {S54, 0, &V13791_4_1235, &P13791_5};	/*Property = ScreenPixelsWidth*/
const Profile P13791 = {"13791", &P13791_4};	/*Profile ID - 13791*/
const Value V15055_1_1 = {S2, NULL};	/*Value = True*/
const Property P15055_1 = {S50, 0, &V15055_1_1, NULL};	/*Property = IsMobile*/
const Value V15055_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15055_5 = {S52, 0, &V15055_5_1222, &P15055_1};	/*Property = ScreenPixelsHeight*/
const Value V15055_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15055_4 = {S54, 0, &V15055_4_1224, &P15055_5};	/*Property = ScreenPixelsWidth*/
const Profile P15055 = {"15055", &P15055_4};	/*Profile ID - 15055*/
const Value V15089_1_1 = {S2, NULL};	/*Value = True*/
const Property P15089_1 = {S50, 0, &V15089_1_1, NULL};	/*Property = IsMobile*/
const Value V15089_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15089_5 = {S52, 0, &V15089_5_1380, &P15089_1};	/*Property = ScreenPixelsHeight*/
const Value V15089_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15089_4 = {S54, 0, &V15089_4_1279, &P15089_5};	/*Property = ScreenPixelsWidth*/
const Profile P15089 = {"15089", &P15089_4};	/*Profile ID - 15089*/
const Value V13794_1_1 = {S2, NULL};	/*Value = True*/
const Property P13794_1 = {S50, 0, &V13794_1_1, NULL};	/*Property = IsMobile*/
const Value V13794_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13794_5 = {S52, 0, &V13794_5_1293, &P13794_1};	/*Property = ScreenPixelsHeight*/
const Value V13794_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13794_4 = {S54, 0, &V13794_4_1296, &P13794_5};	/*Property = ScreenPixelsWidth*/
const Profile P13794 = {"13794", &P13794_4};	/*Profile ID - 13794*/
const Value V15057_1_1 = {S2, NULL};	/*Value = True*/
const Property P15057_1 = {S50, 0, &V15057_1_1, NULL};	/*Property = IsMobile*/
const Value V15057_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15057_5 = {S52, 0, &V15057_5_1222, &P15057_1};	/*Property = ScreenPixelsHeight*/
const Value V15057_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15057_4 = {S54, 0, &V15057_4_1224, &P15057_5};	/*Property = ScreenPixelsWidth*/
const Profile P15057 = {"15057", &P15057_4};	/*Profile ID - 15057*/
const Value V15288_1_1 = {S2, NULL};	/*Value = True*/
const Property P15288_1 = {S50, 0, &V15288_1_1, NULL};	/*Property = IsMobile*/
const Value V15288_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15288_5 = {S52, 0, &V15288_5_1233, &P15288_1};	/*Property = ScreenPixelsHeight*/
const Value V15288_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15288_4 = {S54, 0, &V15288_4_1235, &P15288_5};	/*Property = ScreenPixelsWidth*/
const Profile P15288 = {"15288", &P15288_4};	/*Profile ID - 15288*/
const Value V15059_1_1 = {S2, NULL};	/*Value = True*/
const Property P15059_1 = {S50, 0, &V15059_1_1, NULL};	/*Property = IsMobile*/
const Value V15059_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15059_5 = {S52, 0, &V15059_5_1293, &P15059_1};	/*Property = ScreenPixelsHeight*/
const Value V15059_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15059_4 = {S54, 0, &V15059_4_1296, &P15059_5};	/*Property = ScreenPixelsWidth*/
const Profile P15059 = {"15059", &P15059_4};	/*Profile ID - 15059*/
const Value V14246_1_1 = {S2, NULL};	/*Value = True*/
const Property P14246_1 = {S50, 0, &V14246_1_1, NULL};	/*Property = IsMobile*/
const Value V14246_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14246_5 = {S52, 0, &V14246_5_1222, &P14246_1};	/*Property = ScreenPixelsHeight*/
const Value V14246_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14246_4 = {S54, 0, &V14246_4_1224, &P14246_5};	/*Property = ScreenPixelsWidth*/
const Profile P14246 = {"14246", &P14246_4};	/*Profile ID - 14246*/
const Value V4755_279_57148 = {S0, NULL};	/*Value = False*/
const Property P4755_279 = {S1, 0, &V4755_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V4755_282_57153 = {S2, NULL};	/*Value = True*/
const Property P4755_282 = {S3, 0, &V4755_282_57153, &P4755_279};	/*Property = BlobBuilder*/
const Value V4755_350_57444 = {S2, NULL};	/*Value = True*/
const Property P4755_350 = {S4, 0, &V4755_350_57444, &P4755_282};	/*Property = Canvas*/
const Value V4755_283_57159 = {S2, NULL};	/*Value = True*/
const Property P4755_283 = {S5, 0, &V4755_283_57159, &P4755_350};	/*Property = CssBackground*/
const Value V4755_284_57163 = {S0, NULL};	/*Value = False*/
const Property P4755_284 = {S6, 0, &V4755_284_57163, &P4755_283};	/*Property = CssBorderImage*/
const Value V4755_285_57166 = {S0, NULL};	/*Value = False*/
const Property P4755_285 = {S7, 0, &V4755_285_57166, &P4755_284};	/*Property = CssCanvas*/
const Value V4755_286_57168 = {S2, NULL};	/*Value = True*/
const Property P4755_286 = {S8, 0, &V4755_286_57168, &P4755_285};	/*Property = CssColor*/
const Value V4755_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4755_237 = {S9, 0, &V4755_237_55786, &P4755_286};	/*Property = CssColumn*/
const Value V4755_287_57172 = {S0, NULL};	/*Value = False*/
const Property P4755_287 = {S10, 0, &V4755_287_57172, &P4755_237};	/*Property = CssFlexbox*/
const Value V4755_288_57175 = {S0, NULL};	/*Value = False*/
const Property P4755_288 = {S11, 0, &V4755_288_57175, &P4755_287};	/*Property = CssFont*/
const Value V4755_289_57759 = {S0, NULL};	/*Value = False*/
const Property P4755_289 = {S12, 0, &V4755_289_57759, &P4755_288};	/*Property = CssImages*/
const Value V4755_291_57180 = {S2, NULL};	/*Value = True*/
const Property P4755_291 = {S13, 0, &V4755_291_57180, &P4755_289};	/*Property = CssMediaQueries*/
const Value V4755_292_57183 = {S2, NULL};	/*Value = True*/
const Property P4755_292 = {S14, 0, &V4755_292_57183, &P4755_291};	/*Property = CssMinMax*/
const Value V4755_293_57187 = {S0, NULL};	/*Value = False*/
const Property P4755_293 = {S15, 0, &V4755_293_57187, &P4755_292};	/*Property = CssOverflow*/
const Value V4755_294_57190 = {S0, NULL};	/*Value = False*/
const Property P4755_294 = {S16, 0, &V4755_294_57190, &P4755_293};	/*Property = CssPosition*/
const Value V4755_295_57192 = {S2, NULL};	/*Value = True*/
const Property P4755_295 = {S17, 0, &V4755_295_57192, &P4755_294};	/*Property = CssText*/
const Value V4755_297_57195 = {S2, NULL};	/*Value = True*/
const Property P4755_297 = {S18, 0, &V4755_297_57195, &P4755_295};	/*Property = CssTransforms*/
const Value V4755_241_55799 = {S2, NULL};	/*Value = True*/
const Property P4755_241 = {S19, 0, &V4755_241_55799, &P4755_297};	/*Property = CssTransitions*/
const Value V4755_298_57206 = {S2, NULL};	/*Value = True*/
const Property P4755_298 = {S20, 0, &V4755_298_57206, &P4755_241};	/*Property = CssUI*/
const Value V4755_300_57216 = {S0, NULL};	/*Value = False*/
const Property P4755_300 = {S21, 0, &V4755_300_57216, &P4755_298};	/*Property = DataSet*/
const Value V4755_301_57218 = {S2, NULL};	/*Value = True*/
const Property P4755_301 = {S22, 0, &V4755_301_57218, &P4755_300};	/*Property = DataUrl*/
const Value V4755_302_57221 = {S2, NULL};	/*Value = True*/
const Property P4755_302 = {S23, 0, &V4755_302_57221, &P4755_301};	/*Property = DeviceOrientation*/
const Value V4755_304_57227 = {S2, NULL};	/*Value = True*/
const Property P4755_304 = {S24, 0, &V4755_304_57227, &P4755_302};	/*Property = FileReader*/
const Value V4755_305_57231 = {S0, NULL};	/*Value = False*/
const Property P4755_305 = {S25, 0, &V4755_305_57231, &P4755_304};	/*Property = FileSaver*/
const Value V4755_306_57237 = {S0, NULL};	/*Value = False*/
const Property P4755_306 = {S26, 0, &V4755_306_57237, &P4755_305};	/*Property = FileWriter*/
const Value V4755_307_57239 = {S2, NULL};	/*Value = True*/
const Property P4755_307 = {S27, 0, &V4755_307_57239, &P4755_306};	/*Property = FormData*/
const Value V4755_309_57246 = {S0, NULL};	/*Value = False*/
const Property P4755_309 = {S28, 0, &V4755_309_57246, &P4755_307};	/*Property = Fullscreen*/
const Value V4755_310_57248 = {S2, NULL};	/*Value = True*/
const Property P4755_310 = {S29, 0, &V4755_310_57248, &P4755_309};	/*Property = GeoLocation*/
const Value V4755_312_57255 = {S0, NULL};	/*Value = False*/
const Property P4755_312 = {S30, 0, &V4755_312_57255, &P4755_310};	/*Property = History*/
const Value V4755_314_57261 = {S0, NULL};	/*Value = False*/
const Property P4755_314 = {S31, 0, &V4755_314_57261, &P4755_312};	/*Property = Html5*/
const Value V4755_313_57258 = {S0, NULL};	/*Value = False*/
const Property P4755_313 = {S32, 0, &V4755_313_57258, &P4755_314};	/*Property = Html-Media-Capture*/
const Value V4755_315_57264 = {S0, NULL};	/*Value = False*/
const Property P4755_315 = {S33, 0, &V4755_315_57264, &P4755_313};	/*Property = Iframe*/
const Value V4755_316_57267 = {S0, NULL};	/*Value = False*/
const Property P4755_316 = {S34, 0, &V4755_316_57267, &P4755_315};	/*Property = IndexedDB*/
const Value V4755_317_57269 = {S2, NULL};	/*Value = True*/
const Property P4755_317 = {S35, 0, &V4755_317_57269, &P4755_316};	/*Property = Json*/
const Value V4755_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P4755_15 = {S37, 0, &V4755_15_28, &P4755_317};	/*Property = LayoutEngine*/
const Value V4755_344_57272 = {S2, NULL};	/*Value = True*/
const Property P4755_344 = {S38, 0, &V4755_344_57272, &P4755_15};	/*Property = Masking*/
const Value V4755_320_57290 = {S2, NULL};	/*Value = True*/
const Property P4755_320 = {S39, 0, &V4755_320_57290, &P4755_344};	/*Property = PostMessage*/
const Value V4755_321_57293 = {S2, NULL};	/*Value = True*/
const Property P4755_321 = {S40, 0, &V4755_321_57293, &P4755_320};	/*Property = Progress*/
const Value V4755_322_57296 = {S2, NULL};	/*Value = True*/
const Property P4755_322 = {S41, 0, &V4755_322_57296, &P4755_321};	/*Property = Prompts*/
const Value V4755_323_57299 = {S2, NULL};	/*Value = True*/
const Property P4755_323 = {S42, 0, &V4755_323_57299, &P4755_322};	/*Property = Selector*/
const Value V4755_324_57306 = {S0, NULL};	/*Value = False*/
const Property P4755_324 = {S43, 0, &V4755_324_57306, &P4755_323};	/*Property = Svg*/
const Value V4755_327_57314 = {S2, NULL};	/*Value = True*/
const Property P4755_327 = {S44, 0, &V4755_327_57314, &P4755_324};	/*Property = TouchEvents*/
const Value V4755_328_57318 = {S0, NULL};	/*Value = False*/
const Property P4755_328 = {S45, 0, &V4755_328_57318, &P4755_327};	/*Property = Track*/
const Value V4755_329_57323 = {S2, NULL};	/*Value = True*/
const Property P4755_329 = {S46, 0, &V4755_329_57323, &P4755_328};	/*Property = Video*/
const Value V4755_330_57326 = {S2, NULL};	/*Value = True*/
const Property P4755_330 = {S47, 0, &V4755_330_57326, &P4755_329};	/*Property = Viewport*/
const Value V4755_332_57339 = {S0, NULL};	/*Value = False*/
const Property P4755_332 = {S48, 0, &V4755_332_57339, &P4755_330};	/*Property = WebWorkers*/
const Value V4755_333_57342 = {S0, NULL};	/*Value = False*/
const Property P4755_333 = {S49, 0, &V4755_333_57342, &P4755_332};	/*Property = Xhr2*/
const Profile P4755 = {"4755", &P4755_333};	/*Profile ID - 4755*/
const Profile P6545 = {"6545", NULL};	/*Profile ID - 6545*/
const Value V17686_1_1 = {S2, NULL};	/*Value = True*/
const Property P17686_1 = {S50, 0, &V17686_1_1, NULL};	/*Property = IsMobile*/
const Value V17686_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P17686_5 = {S52, 0, &V17686_5_9981, &P17686_1};	/*Property = ScreenPixelsHeight*/
const Value V17686_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P17686_4 = {S54, 0, &V17686_4_2336, &P17686_5};	/*Property = ScreenPixelsWidth*/
const Profile P17686 = {"17686", &P17686_4};	/*Profile ID - 17686*/
const Value V15060_1_1 = {S2, NULL};	/*Value = True*/
const Property P15060_1 = {S50, 0, &V15060_1_1, NULL};	/*Property = IsMobile*/
const Value V15060_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15060_5 = {S52, 0, &V15060_5_1277, &P15060_1};	/*Property = ScreenPixelsHeight*/
const Value V15060_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15060_4 = {S54, 0, &V15060_4_1279, &P15060_5};	/*Property = ScreenPixelsWidth*/
const Profile P15060 = {"15060", &P15060_4};	/*Profile ID - 15060*/
const Value V12924_1_1 = {S2, NULL};	/*Value = True*/
const Property P12924_1 = {S50, 0, &V12924_1_1, NULL};	/*Property = IsMobile*/
const Value V12924_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12924_5 = {S52, 0, &V12924_5_1233, &P12924_1};	/*Property = ScreenPixelsHeight*/
const Value V12924_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12924_4 = {S54, 0, &V12924_4_1235, &P12924_5};	/*Property = ScreenPixelsWidth*/
const Profile P12924 = {"12924", &P12924_4};	/*Profile ID - 12924*/
const Value V15062_1_1 = {S2, NULL};	/*Value = True*/
const Property P15062_1 = {S50, 0, &V15062_1_1, NULL};	/*Property = IsMobile*/
const Value V15062_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15062_5 = {S52, 0, &V15062_5_1233, &P15062_1};	/*Property = ScreenPixelsHeight*/
const Value V15062_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15062_4 = {S54, 0, &V15062_4_1235, &P15062_5};	/*Property = ScreenPixelsWidth*/
const Profile P15062 = {"15062", &P15062_4};	/*Profile ID - 15062*/
const Value V15063_1_1 = {S2, NULL};	/*Value = True*/
const Property P15063_1 = {S50, 0, &V15063_1_1, NULL};	/*Property = IsMobile*/
const Value V15063_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15063_5 = {S52, 0, &V15063_5_1222, &P15063_1};	/*Property = ScreenPixelsHeight*/
const Value V15063_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15063_4 = {S54, 0, &V15063_4_1224, &P15063_5};	/*Property = ScreenPixelsWidth*/
const Profile P15063 = {"15063", &P15063_4};	/*Profile ID - 15063*/
const Value V15067_1_1 = {S2, NULL};	/*Value = True*/
const Property P15067_1 = {S50, 0, &V15067_1_1, NULL};	/*Property = IsMobile*/
const Value V15067_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15067_5 = {S52, 0, &V15067_5_1222, &P15067_1};	/*Property = ScreenPixelsHeight*/
const Value V15067_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15067_4 = {S54, 0, &V15067_4_1224, &P15067_5};	/*Property = ScreenPixelsWidth*/
const Profile P15067 = {"15067", &P15067_4};	/*Profile ID - 15067*/
const Value V16967_1_1 = {S2, NULL};	/*Value = True*/
const Property P16967_1 = {S50, 0, &V16967_1_1, NULL};	/*Property = IsMobile*/
const Value V16967_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P16967_5 = {S52, 0, &V16967_5_1275, &P16967_1};	/*Property = ScreenPixelsHeight*/
const Value V16967_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P16967_4 = {S54, 0, &V16967_4_1278, &P16967_5};	/*Property = ScreenPixelsWidth*/
const Profile P16967 = {"16967", &P16967_4};	/*Profile ID - 16967*/
const Value V15069_1_1 = {S2, NULL};	/*Value = True*/
const Property P15069_1 = {S50, 0, &V15069_1_1, NULL};	/*Property = IsMobile*/
const Value V15069_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P15069_5 = {S52, 0, &V15069_5_1269, &P15069_1};	/*Property = ScreenPixelsHeight*/
const Value V15069_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15069_4 = {S54, 0, &V15069_4_1229, &P15069_5};	/*Property = ScreenPixelsWidth*/
const Profile P15069 = {"15069", &P15069_4};	/*Profile ID - 15069*/
const Value V15070_1_1 = {S2, NULL};	/*Value = True*/
const Property P15070_1 = {S50, 0, &V15070_1_1, NULL};	/*Property = IsMobile*/
const Value V15070_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15070_5 = {S52, 0, &V15070_5_1293, &P15070_1};	/*Property = ScreenPixelsHeight*/
const Value V15070_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15070_4 = {S54, 0, &V15070_4_1296, &P15070_5};	/*Property = ScreenPixelsWidth*/
const Profile P15070 = {"15070", &P15070_4};	/*Profile ID - 15070*/
const Value V15071_1_1 = {S2, NULL};	/*Value = True*/
const Property P15071_1 = {S50, 0, &V15071_1_1, NULL};	/*Property = IsMobile*/
const Value V15071_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15071_5 = {S52, 0, &V15071_5_1222, &P15071_1};	/*Property = ScreenPixelsHeight*/
const Value V15071_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15071_4 = {S54, 0, &V15071_4_1224, &P15071_5};	/*Property = ScreenPixelsWidth*/
const Profile P15071 = {"15071", &P15071_4};	/*Profile ID - 15071*/
const Value V15072_1_1 = {S2, NULL};	/*Value = True*/
const Property P15072_1 = {S50, 0, &V15072_1_1, NULL};	/*Property = IsMobile*/
const Value V15072_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15072_5 = {S52, 0, &V15072_5_1277, &P15072_1};	/*Property = ScreenPixelsHeight*/
const Value V15072_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15072_4 = {S54, 0, &V15072_4_1279, &P15072_5};	/*Property = ScreenPixelsWidth*/
const Profile P15072 = {"15072", &P15072_4};	/*Profile ID - 15072*/
const Value V15074_1_1 = {S2, NULL};	/*Value = True*/
const Property P15074_1 = {S50, 0, &V15074_1_1, NULL};	/*Property = IsMobile*/
const Value V15074_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15074_5 = {S52, 0, &V15074_5_1293, &P15074_1};	/*Property = ScreenPixelsHeight*/
const Value V15074_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15074_4 = {S54, 0, &V15074_4_1296, &P15074_5};	/*Property = ScreenPixelsWidth*/
const Profile P15074 = {"15074", &P15074_4};	/*Profile ID - 15074*/
const Value V15077_1_1 = {S2, NULL};	/*Value = True*/
const Property P15077_1 = {S50, 0, &V15077_1_1, NULL};	/*Property = IsMobile*/
const Value V15077_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15077_5 = {S52, 0, &V15077_5_1222, &P15077_1};	/*Property = ScreenPixelsHeight*/
const Value V15077_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15077_4 = {S54, 0, &V15077_4_1224, &P15077_5};	/*Property = ScreenPixelsWidth*/
const Profile P15077 = {"15077", &P15077_4};	/*Profile ID - 15077*/
const Value V15078_1_1 = {S2, NULL};	/*Value = True*/
const Property P15078_1 = {S50, 0, &V15078_1_1, NULL};	/*Property = IsMobile*/
const Value V15078_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15078_5 = {S52, 0, &V15078_5_1293, &P15078_1};	/*Property = ScreenPixelsHeight*/
const Value V15078_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15078_4 = {S54, 0, &V15078_4_1296, &P15078_5};	/*Property = ScreenPixelsWidth*/
const Profile P15078 = {"15078", &P15078_4};	/*Profile ID - 15078*/
const Value V15080_1_1 = {S2, NULL};	/*Value = True*/
const Property P15080_1 = {S50, 0, &V15080_1_1, NULL};	/*Property = IsMobile*/
const Value V15080_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15080_5 = {S52, 0, &V15080_5_1293, &P15080_1};	/*Property = ScreenPixelsHeight*/
const Value V15080_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15080_4 = {S54, 0, &V15080_4_1296, &P15080_5};	/*Property = ScreenPixelsWidth*/
const Profile P15080 = {"15080", &P15080_4};	/*Profile ID - 15080*/
const Value V15079_1_1 = {S2, NULL};	/*Value = True*/
const Property P15079_1 = {S50, 0, &V15079_1_1, NULL};	/*Property = IsMobile*/
const Value V15079_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P15079_5 = {S52, 0, &V15079_5_1269, &P15079_1};	/*Property = ScreenPixelsHeight*/
const Value V15079_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15079_4 = {S54, 0, &V15079_4_1229, &P15079_5};	/*Property = ScreenPixelsWidth*/
const Profile P15079 = {"15079", &P15079_4};	/*Profile ID - 15079*/
const Value V15741_1_1 = {S2, NULL};	/*Value = True*/
const Property P15741_1 = {S50, 0, &V15741_1_1, NULL};	/*Property = IsMobile*/
const Value V15741_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15741_5 = {S52, 0, &V15741_5_1293, &P15741_1};	/*Property = ScreenPixelsHeight*/
const Value V15741_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15741_4 = {S54, 0, &V15741_4_1296, &P15741_5};	/*Property = ScreenPixelsWidth*/
const Profile P15741 = {"15741", &P15741_4};	/*Profile ID - 15741*/
const Value V15083_1_1 = {S2, NULL};	/*Value = True*/
const Property P15083_1 = {S50, 0, &V15083_1_1, NULL};	/*Property = IsMobile*/
const Value V15083_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15083_5 = {S52, 0, &V15083_5_1222, &P15083_1};	/*Property = ScreenPixelsHeight*/
const Value V15083_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15083_4 = {S54, 0, &V15083_4_1224, &P15083_5};	/*Property = ScreenPixelsWidth*/
const Profile P15083 = {"15083", &P15083_4};	/*Profile ID - 15083*/
const Value V13899_1_1 = {S2, NULL};	/*Value = True*/
const Property P13899_1 = {S50, 0, &V13899_1_1, NULL};	/*Property = IsMobile*/
const Value V13899_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13899_5 = {S52, 0, &V13899_5_1293, &P13899_1};	/*Property = ScreenPixelsHeight*/
const Value V13899_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13899_4 = {S54, 0, &V13899_4_1296, &P13899_5};	/*Property = ScreenPixelsWidth*/
const Profile P13899 = {"13899", &P13899_4};	/*Profile ID - 13899*/
const Value V14360_1_1 = {S2, NULL};	/*Value = True*/
const Property P14360_1 = {S50, 0, &V14360_1_1, NULL};	/*Property = IsMobile*/
const Value V14360_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14360_5 = {S52, 0, &V14360_5_1233, &P14360_1};	/*Property = ScreenPixelsHeight*/
const Value V14360_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14360_4 = {S54, 0, &V14360_4_1235, &P14360_5};	/*Property = ScreenPixelsWidth*/
const Profile P14360 = {"14360", &P14360_4};	/*Profile ID - 14360*/
const Value V15084_1_1 = {S2, NULL};	/*Value = True*/
const Property P15084_1 = {S50, 0, &V15084_1_1, NULL};	/*Property = IsMobile*/
const Value V15084_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15084_5 = {S52, 0, &V15084_5_1293, &P15084_1};	/*Property = ScreenPixelsHeight*/
const Value V15084_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15084_4 = {S54, 0, &V15084_4_1296, &P15084_5};	/*Property = ScreenPixelsWidth*/
const Profile P15084 = {"15084", &P15084_4};	/*Profile ID - 15084*/
const Value V15085_1_1 = {S2, NULL};	/*Value = True*/
const Property P15085_1 = {S50, 0, &V15085_1_1, NULL};	/*Property = IsMobile*/
const Value V15085_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15085_5 = {S52, 0, &V15085_5_1222, &P15085_1};	/*Property = ScreenPixelsHeight*/
const Value V15085_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15085_4 = {S54, 0, &V15085_4_1224, &P15085_5};	/*Property = ScreenPixelsWidth*/
const Profile P15085 = {"15085", &P15085_4};	/*Profile ID - 15085*/
const Value V13929_1_1 = {S2, NULL};	/*Value = True*/
const Property P13929_1 = {S50, 0, &V13929_1_1, NULL};	/*Property = IsMobile*/
const Value V13929_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13929_5 = {S52, 0, &V13929_5_1380, &P13929_1};	/*Property = ScreenPixelsHeight*/
const Value V13929_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13929_4 = {S54, 0, &V13929_4_1279, &P13929_5};	/*Property = ScreenPixelsWidth*/
const Profile P13929 = {"13929", &P13929_4};	/*Profile ID - 13929*/
const Value V15086_1_1 = {S2, NULL};	/*Value = True*/
const Property P15086_1 = {S50, 0, &V15086_1_1, NULL};	/*Property = IsMobile*/
const Value V15086_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15086_5 = {S52, 0, &V15086_5_1380, &P15086_1};	/*Property = ScreenPixelsHeight*/
const Value V15086_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15086_4 = {S54, 0, &V15086_4_1279, &P15086_5};	/*Property = ScreenPixelsWidth*/
const Profile P15086 = {"15086", &P15086_4};	/*Profile ID - 15086*/
const Value V17610_1_1 = {S2, NULL};	/*Value = True*/
const Property P17610_1 = {S50, 0, &V17610_1_1, NULL};	/*Property = IsMobile*/
const Value V17610_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17610_5 = {S52, 0, &V17610_5_1222, &P17610_1};	/*Property = ScreenPixelsHeight*/
const Value V17610_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17610_4 = {S54, 0, &V17610_4_1224, &P17610_5};	/*Property = ScreenPixelsWidth*/
const Profile P17610 = {"17610", &P17610_4};	/*Profile ID - 17610*/
const Value V20385_1_2 = {S0, NULL};	/*Value = False*/
const Property P20385_1 = {S50, 0, &V20385_1_2, NULL};	/*Property = IsMobile*/
const Value V20385_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20385_5 = {S52, 0, &V20385_5_37, &P20385_1};	/*Property = ScreenPixelsHeight*/
const Value V20385_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20385_4 = {S54, 0, &V20385_4_36, &P20385_5};	/*Property = ScreenPixelsWidth*/
const Profile P20385 = {"20385", &P20385_4};	/*Profile ID - 20385*/
const Value V20398_1_2 = {S0, NULL};	/*Value = False*/
const Property P20398_1 = {S50, 0, &V20398_1_2, NULL};	/*Property = IsMobile*/
const Value V20398_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20398_5 = {S52, 0, &V20398_5_37, &P20398_1};	/*Property = ScreenPixelsHeight*/
const Value V20398_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20398_4 = {S54, 0, &V20398_4_36, &P20398_5};	/*Property = ScreenPixelsWidth*/
const Profile P20398 = {"20398", &P20398_4};	/*Profile ID - 20398*/
const Value V20395_1_2 = {S0, NULL};	/*Value = False*/
const Property P20395_1 = {S50, 0, &V20395_1_2, NULL};	/*Property = IsMobile*/
const Value V20395_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20395_5 = {S52, 0, &V20395_5_37, &P20395_1};	/*Property = ScreenPixelsHeight*/
const Value V20395_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20395_4 = {S54, 0, &V20395_4_36, &P20395_5};	/*Property = ScreenPixelsWidth*/
const Profile P20395 = {"20395", &P20395_4};	/*Profile ID - 20395*/
const Value V12568_1_1 = {S2, NULL};	/*Value = True*/
const Property P12568_1 = {S50, 0, &V12568_1_1, NULL};	/*Property = IsMobile*/
const Value V12568_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12568_5 = {S52, 0, &V12568_5_1380, &P12568_1};	/*Property = ScreenPixelsHeight*/
const Value V12568_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12568_4 = {S54, 0, &V12568_4_1279, &P12568_5};	/*Property = ScreenPixelsWidth*/
const Profile P12568 = {"12568", &P12568_4};	/*Profile ID - 12568*/
const Value V15091_1_1 = {S2, NULL};	/*Value = True*/
const Property P15091_1 = {S50, 0, &V15091_1_1, NULL};	/*Property = IsMobile*/
const Value V15091_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15091_5 = {S52, 0, &V15091_5_1233, &P15091_1};	/*Property = ScreenPixelsHeight*/
const Value V15091_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15091_4 = {S54, 0, &V15091_4_1235, &P15091_5};	/*Property = ScreenPixelsWidth*/
const Profile P15091 = {"15091", &P15091_4};	/*Profile ID - 15091*/
const Value V15092_1_1 = {S2, NULL};	/*Value = True*/
const Property P15092_1 = {S50, 0, &V15092_1_1, NULL};	/*Property = IsMobile*/
const Value V15092_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15092_5 = {S52, 0, &V15092_5_1277, &P15092_1};	/*Property = ScreenPixelsHeight*/
const Value V15092_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15092_4 = {S54, 0, &V15092_4_1224, &P15092_5};	/*Property = ScreenPixelsWidth*/
const Profile P15092 = {"15092", &P15092_4};	/*Profile ID - 15092*/
const Value V13549_1_1 = {S2, NULL};	/*Value = True*/
const Property P13549_1 = {S50, 0, &V13549_1_1, NULL};	/*Property = IsMobile*/
const Value V13549_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13549_5 = {S52, 0, &V13549_5_1233, &P13549_1};	/*Property = ScreenPixelsHeight*/
const Value V13549_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13549_4 = {S54, 0, &V13549_4_1235, &P13549_5};	/*Property = ScreenPixelsWidth*/
const Profile P13549 = {"13549", &P13549_4};	/*Profile ID - 13549*/
const Value V15095_1_1 = {S2, NULL};	/*Value = True*/
const Property P15095_1 = {S50, 0, &V15095_1_1, NULL};	/*Property = IsMobile*/
const Value V15095_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15095_5 = {S52, 0, &V15095_5_1233, &P15095_1};	/*Property = ScreenPixelsHeight*/
const Value V15095_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15095_4 = {S54, 0, &V15095_4_1235, &P15095_5};	/*Property = ScreenPixelsWidth*/
const Profile P15095 = {"15095", &P15095_4};	/*Profile ID - 15095*/
const Value V11902_1_1 = {S2, NULL};	/*Value = True*/
const Property P11902_1 = {S50, 0, &V11902_1_1, NULL};	/*Property = IsMobile*/
const Value V11902_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11902_5 = {S52, 0, &V11902_5_1222, &P11902_1};	/*Property = ScreenPixelsHeight*/
const Value V11902_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11902_4 = {S54, 0, &V11902_4_1224, &P11902_5};	/*Property = ScreenPixelsWidth*/
const Profile P11902 = {"11902", &P11902_4};	/*Profile ID - 11902*/
const Value V14451_1_1 = {S2, NULL};	/*Value = True*/
const Property P14451_1 = {S50, 0, &V14451_1_1, NULL};	/*Property = IsMobile*/
const Value V14451_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14451_5 = {S52, 0, &V14451_5_1222, &P14451_1};	/*Property = ScreenPixelsHeight*/
const Value V14451_4_1368 = {S74, NULL};	/*Value = 208*/
const Property P14451_4 = {S54, 0, &V14451_4_1368, &P14451_5};	/*Property = ScreenPixelsWidth*/
const Profile P14451 = {"14451", &P14451_4};	/*Profile ID - 14451*/
const Value V13744_1_1 = {S2, NULL};	/*Value = True*/
const Property P13744_1 = {S50, 0, &V13744_1_1, NULL};	/*Property = IsMobile*/
const Value V13744_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13744_5 = {S52, 0, &V13744_5_1222, &P13744_1};	/*Property = ScreenPixelsHeight*/
const Value V13744_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13744_4 = {S54, 0, &V13744_4_1224, &P13744_5};	/*Property = ScreenPixelsWidth*/
const Profile P13744 = {"13744", &P13744_4};	/*Profile ID - 13744*/
const Value V15099_1_1 = {S2, NULL};	/*Value = True*/
const Property P15099_1 = {S50, 0, &V15099_1_1, NULL};	/*Property = IsMobile*/
const Value V15099_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15099_5 = {S52, 0, &V15099_5_1293, &P15099_1};	/*Property = ScreenPixelsHeight*/
const Value V15099_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15099_4 = {S54, 0, &V15099_4_1296, &P15099_5};	/*Property = ScreenPixelsWidth*/
const Profile P15099 = {"15099", &P15099_4};	/*Profile ID - 15099*/
const Value V13559_1_1 = {S2, NULL};	/*Value = True*/
const Property P13559_1 = {S50, 0, &V13559_1_1, NULL};	/*Property = IsMobile*/
const Value V13559_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13559_5 = {S52, 0, &V13559_5_1222, &P13559_1};	/*Property = ScreenPixelsHeight*/
const Value V13559_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13559_4 = {S54, 0, &V13559_4_1224, &P13559_5};	/*Property = ScreenPixelsWidth*/
const Profile P13559 = {"13559", &P13559_4};	/*Profile ID - 13559*/
const Value V14135_1_1 = {S2, NULL};	/*Value = True*/
const Property P14135_1 = {S50, 0, &V14135_1_1, NULL};	/*Property = IsMobile*/
const Value V14135_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14135_5 = {S52, 0, &V14135_5_1294, &P14135_1};	/*Property = ScreenPixelsHeight*/
const Value V14135_4_33019 = {S61, NULL};	/*Value = 160*/
const Property P14135_4 = {S54, 0, &V14135_4_33019, &P14135_5};	/*Property = ScreenPixelsWidth*/
const Profile P14135 = {"14135", &P14135_4};	/*Profile ID - 14135*/
const Value V14455_1_1 = {S2, NULL};	/*Value = True*/
const Property P14455_1 = {S50, 0, &V14455_1_1, NULL};	/*Property = IsMobile*/
const Value V14455_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14455_5 = {S52, 0, &V14455_5_1222, &P14455_1};	/*Property = ScreenPixelsHeight*/
const Value V14455_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14455_4 = {S54, 0, &V14455_4_1224, &P14455_5};	/*Property = ScreenPixelsWidth*/
const Profile P14455 = {"14455", &P14455_4};	/*Profile ID - 14455*/
const Value V4970_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_279 = {S1, 0, &V4970_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4970_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_282 = {S3, 0, &V4970_282_57155, &P4970_279};	/*Property = BlobBuilder*/
const Value V4970_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_350 = {S4, 0, &V4970_350_57446, &P4970_282};	/*Property = Canvas*/
const Value V4970_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_283 = {S5, 0, &V4970_283_57161, &P4970_350};	/*Property = CssBackground*/
const Value V4970_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_284 = {S6, 0, &V4970_284_57164, &P4970_283};	/*Property = CssBorderImage*/
const Value V4970_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_285 = {S7, 0, &V4970_285_57167, &P4970_284};	/*Property = CssCanvas*/
const Value V4970_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_286 = {S8, 0, &V4970_286_57170, &P4970_285};	/*Property = CssColor*/
const Value V4970_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_237 = {S9, 0, &V4970_237_55786, &P4970_286};	/*Property = CssColumn*/
const Value V4970_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_287 = {S10, 0, &V4970_287_57173, &P4970_237};	/*Property = CssFlexbox*/
const Value V4970_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_288 = {S11, 0, &V4970_288_57176, &P4970_287};	/*Property = CssFont*/
const Value V4970_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_289 = {S12, 0, &V4970_289_86899, &P4970_288};	/*Property = CssImages*/
const Value V4970_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_291 = {S13, 0, &V4970_291_57182, &P4970_289};	/*Property = CssMediaQueries*/
const Value V4970_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_292 = {S14, 0, &V4970_292_57185, &P4970_291};	/*Property = CssMinMax*/
const Value V4970_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_293 = {S15, 0, &V4970_293_57188, &P4970_292};	/*Property = CssOverflow*/
const Value V4970_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_294 = {S16, 0, &V4970_294_57191, &P4970_293};	/*Property = CssPosition*/
const Value V4970_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_295 = {S17, 0, &V4970_295_57194, &P4970_294};	/*Property = CssText*/
const Value V4970_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_297 = {S18, 0, &V4970_297_57197, &P4970_295};	/*Property = CssTransforms*/
const Value V4970_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_241 = {S19, 0, &V4970_241_55801, &P4970_297};	/*Property = CssTransitions*/
const Value V4970_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_298 = {S20, 0, &V4970_298_57208, &P4970_241};	/*Property = CssUI*/
const Value V4970_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_300 = {S21, 0, &V4970_300_57217, &P4970_298};	/*Property = DataSet*/
const Value V4970_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_301 = {S22, 0, &V4970_301_57220, &P4970_300};	/*Property = DataUrl*/
const Value V4970_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_302 = {S23, 0, &V4970_302_57223, &P4970_301};	/*Property = DeviceOrientation*/
const Value V4970_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_304 = {S24, 0, &V4970_304_57229, &P4970_302};	/*Property = FileReader*/
const Value V4970_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_305 = {S25, 0, &V4970_305_57232, &P4970_304};	/*Property = FileSaver*/
const Value V4970_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_306 = {S26, 0, &V4970_306_57238, &P4970_305};	/*Property = FileWriter*/
const Value V4970_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_307 = {S27, 0, &V4970_307_57241, &P4970_306};	/*Property = FormData*/
const Value V4970_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_309 = {S28, 0, &V4970_309_57247, &P4970_307};	/*Property = Fullscreen*/
const Value V4970_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_310 = {S29, 0, &V4970_310_57250, &P4970_309};	/*Property = GeoLocation*/
const Value V4970_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_312 = {S30, 0, &V4970_312_57256, &P4970_310};	/*Property = History*/
const Value V4970_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_314 = {S31, 0, &V4970_314_57262, &P4970_312};	/*Property = Html5*/
const Value V4970_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_313 = {S32, 0, &V4970_313_57259, &P4970_314};	/*Property = Html-Media-Capture*/
const Value V4970_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_315 = {S33, 0, &V4970_315_57265, &P4970_313};	/*Property = Iframe*/
const Value V4970_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_316 = {S34, 0, &V4970_316_57268, &P4970_315};	/*Property = IndexedDB*/
const Value V4970_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_317 = {S35, 0, &V4970_317_57271, &P4970_316};	/*Property = Json*/
const Value V4970_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P4970_15 = {S37, 0, &V4970_15_28, &P4970_317};	/*Property = LayoutEngine*/
const Value V4970_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_344 = {S38, 0, &V4970_344_57274, &P4970_15};	/*Property = Masking*/
const Value V4970_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_320 = {S39, 0, &V4970_320_57292, &P4970_344};	/*Property = PostMessage*/
const Value V4970_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_321 = {S40, 0, &V4970_321_57295, &P4970_320};	/*Property = Progress*/
const Value V4970_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_322 = {S41, 0, &V4970_322_57298, &P4970_321};	/*Property = Prompts*/
const Value V4970_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_323 = {S42, 0, &V4970_323_57301, &P4970_322};	/*Property = Selector*/
const Value V4970_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_324 = {S43, 0, &V4970_324_57307, &P4970_323};	/*Property = Svg*/
const Value V4970_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_327 = {S44, 0, &V4970_327_57316, &P4970_324};	/*Property = TouchEvents*/
const Value V4970_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_328 = {S45, 0, &V4970_328_57319, &P4970_327};	/*Property = Track*/
const Value V4970_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_329 = {S46, 0, &V4970_329_57325, &P4970_328};	/*Property = Video*/
const Value V4970_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_330 = {S47, 0, &V4970_330_57328, &P4970_329};	/*Property = Viewport*/
const Value V4970_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_332 = {S48, 0, &V4970_332_57340, &P4970_330};	/*Property = WebWorkers*/
const Value V4970_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4970_333 = {S49, 0, &V4970_333_57343, &P4970_332};	/*Property = Xhr2*/
const Profile P4970 = {"4970", &P4970_333};	/*Profile ID - 4970*/
const Value V12112_1_1 = {S2, NULL};	/*Value = True*/
const Property P12112_1 = {S50, 0, &V12112_1_1, NULL};	/*Property = IsMobile*/
const Value V12112_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12112_5 = {S52, 0, &V12112_5_1233, &P12112_1};	/*Property = ScreenPixelsHeight*/
const Value V12112_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12112_4 = {S54, 0, &V12112_4_1235, &P12112_5};	/*Property = ScreenPixelsWidth*/
const Profile P12112 = {"12112", &P12112_4};	/*Profile ID - 12112*/
const Value V20125_1_2 = {S0, NULL};	/*Value = False*/
const Property P20125_1 = {S50, 0, &V20125_1_2, NULL};	/*Property = IsMobile*/
const Value V20125_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20125_5 = {S52, 0, &V20125_5_37, &P20125_1};	/*Property = ScreenPixelsHeight*/
const Value V20125_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20125_4 = {S54, 0, &V20125_4_36, &P20125_5};	/*Property = ScreenPixelsWidth*/
const Profile P20125 = {"20125", &P20125_4};	/*Profile ID - 20125*/
const Value V13712_1_1 = {S2, NULL};	/*Value = True*/
const Property P13712_1 = {S50, 0, &V13712_1_1, NULL};	/*Property = IsMobile*/
const Value V13712_5_1796 = {S64, NULL};	/*Value = 360*/
const Property P13712_5 = {S52, 0, &V13712_5_1796, &P13712_1};	/*Property = ScreenPixelsHeight*/
const Value V13712_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13712_4 = {S54, 0, &V13712_4_1224, &P13712_5};	/*Property = ScreenPixelsWidth*/
const Profile P13712 = {"13712", &P13712_4};	/*Profile ID - 13712*/
const Value V15104_1_1 = {S2, NULL};	/*Value = True*/
const Property P15104_1 = {S50, 0, &V15104_1_1, NULL};	/*Property = IsMobile*/
const Value V15104_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15104_5 = {S52, 0, &V15104_5_1233, &P15104_1};	/*Property = ScreenPixelsHeight*/
const Value V15104_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15104_4 = {S54, 0, &V15104_4_1235, &P15104_5};	/*Property = ScreenPixelsWidth*/
const Profile P15104 = {"15104", &P15104_4};	/*Profile ID - 15104*/
const Profile P7205 = {"7205", NULL};	/*Profile ID - 7205*/
const Value V14320_1_1 = {S2, NULL};	/*Value = True*/
const Property P14320_1 = {S50, 0, &V14320_1_1, NULL};	/*Property = IsMobile*/
const Value V14320_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14320_5 = {S52, 0, &V14320_5_1293, &P14320_1};	/*Property = ScreenPixelsHeight*/
const Value V14320_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14320_4 = {S54, 0, &V14320_4_1296, &P14320_5};	/*Property = ScreenPixelsWidth*/
const Profile P14320 = {"14320", &P14320_4};	/*Profile ID - 14320*/
const Value V12221_1_1 = {S2, NULL};	/*Value = True*/
const Property P12221_1 = {S50, 0, &V12221_1_1, NULL};	/*Property = IsMobile*/
const Value V12221_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12221_5 = {S52, 0, &V12221_5_1222, &P12221_1};	/*Property = ScreenPixelsHeight*/
const Value V12221_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12221_4 = {S54, 0, &V12221_4_1224, &P12221_5};	/*Property = ScreenPixelsWidth*/
const Profile P12221 = {"12221", &P12221_4};	/*Profile ID - 12221*/
const Value V15109_1_1 = {S2, NULL};	/*Value = True*/
const Property P15109_1 = {S50, 0, &V15109_1_1, NULL};	/*Property = IsMobile*/
const Value V15109_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15109_5 = {S52, 0, &V15109_5_1380, &P15109_1};	/*Property = ScreenPixelsHeight*/
const Value V15109_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15109_4 = {S54, 0, &V15109_4_1279, &P15109_5};	/*Property = ScreenPixelsWidth*/
const Profile P15109 = {"15109", &P15109_4};	/*Profile ID - 15109*/
const Value V14037_1_1 = {S2, NULL};	/*Value = True*/
const Property P14037_1 = {S50, 0, &V14037_1_1, NULL};	/*Property = IsMobile*/
const Value V14037_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14037_5 = {S52, 0, &V14037_5_1294, &P14037_1};	/*Property = ScreenPixelsHeight*/
const Value V14037_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14037_4 = {S54, 0, &V14037_4_1296, &P14037_5};	/*Property = ScreenPixelsWidth*/
const Profile P14037 = {"14037", &P14037_4};	/*Profile ID - 14037*/
const Value V15112_1_1 = {S2, NULL};	/*Value = True*/
const Property P15112_1 = {S50, 0, &V15112_1_1, NULL};	/*Property = IsMobile*/
const Value V15112_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15112_5 = {S52, 0, &V15112_5_1222, &P15112_1};	/*Property = ScreenPixelsHeight*/
const Value V15112_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15112_4 = {S54, 0, &V15112_4_1224, &P15112_5};	/*Property = ScreenPixelsWidth*/
const Profile P15112 = {"15112", &P15112_4};	/*Profile ID - 15112*/
const Value V15116_1_1 = {S2, NULL};	/*Value = True*/
const Property P15116_1 = {S50, 0, &V15116_1_1, NULL};	/*Property = IsMobile*/
const Value V15116_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15116_5 = {S52, 0, &V15116_5_1233, &P15116_1};	/*Property = ScreenPixelsHeight*/
const Value V15116_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15116_4 = {S54, 0, &V15116_4_1235, &P15116_5};	/*Property = ScreenPixelsWidth*/
const Profile P15116 = {"15116", &P15116_4};	/*Profile ID - 15116*/
const Value V12384_1_1 = {S2, NULL};	/*Value = True*/
const Property P12384_1 = {S50, 0, &V12384_1_1, NULL};	/*Property = IsMobile*/
const Value V12384_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12384_5 = {S52, 0, &V12384_5_1293, &P12384_1};	/*Property = ScreenPixelsHeight*/
const Value V12384_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12384_4 = {S54, 0, &V12384_4_1296, &P12384_5};	/*Property = ScreenPixelsWidth*/
const Profile P12384 = {"12384", &P12384_4};	/*Profile ID - 12384*/
const Value V14075_1_1 = {S2, NULL};	/*Value = True*/
const Property P14075_1 = {S50, 0, &V14075_1_1, NULL};	/*Property = IsMobile*/
const Value V14075_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14075_5 = {S52, 0, &V14075_5_1277, &P14075_1};	/*Property = ScreenPixelsHeight*/
const Value V14075_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P14075_4 = {S54, 0, &V14075_4_8554, &P14075_5};	/*Property = ScreenPixelsWidth*/
const Profile P14075 = {"14075", &P14075_4};	/*Profile ID - 14075*/
const Value V15122_1_1 = {S2, NULL};	/*Value = True*/
const Property P15122_1 = {S50, 0, &V15122_1_1, NULL};	/*Property = IsMobile*/
const Value V15122_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15122_5 = {S52, 0, &V15122_5_1304, &P15122_1};	/*Property = ScreenPixelsHeight*/
const Value V15122_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15122_4 = {S54, 0, &V15122_4_1224, &P15122_5};	/*Property = ScreenPixelsWidth*/
const Profile P15122 = {"15122", &P15122_4};	/*Profile ID - 15122*/
const Value V15136_1_1 = {S2, NULL};	/*Value = True*/
const Property P15136_1 = {S50, 0, &V15136_1_1, NULL};	/*Property = IsMobile*/
const Value V15136_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15136_5 = {S52, 0, &V15136_5_1222, &P15136_1};	/*Property = ScreenPixelsHeight*/
const Value V15136_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15136_4 = {S54, 0, &V15136_4_1224, &P15136_5};	/*Property = ScreenPixelsWidth*/
const Profile P15136 = {"15136", &P15136_4};	/*Profile ID - 15136*/
const Value V15126_1_1 = {S2, NULL};	/*Value = True*/
const Property P15126_1 = {S50, 0, &V15126_1_1, NULL};	/*Property = IsMobile*/
const Value V15126_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15126_5 = {S52, 0, &V15126_5_1293, &P15126_1};	/*Property = ScreenPixelsHeight*/
const Value V15126_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15126_4 = {S54, 0, &V15126_4_1296, &P15126_5};	/*Property = ScreenPixelsWidth*/
const Profile P15126 = {"15126", &P15126_4};	/*Profile ID - 15126*/
const Value V14270_1_1 = {S2, NULL};	/*Value = True*/
const Property P14270_1 = {S50, 0, &V14270_1_1, NULL};	/*Property = IsMobile*/
const Value V14270_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14270_5 = {S52, 0, &V14270_5_1233, &P14270_1};	/*Property = ScreenPixelsHeight*/
const Value V14270_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14270_4 = {S54, 0, &V14270_4_1235, &P14270_5};	/*Property = ScreenPixelsWidth*/
const Profile P14270 = {"14270", &P14270_4};	/*Profile ID - 14270*/
const Value V13719_1_1 = {S2, NULL};	/*Value = True*/
const Property P13719_1 = {S50, 0, &V13719_1_1, NULL};	/*Property = IsMobile*/
const Value V13719_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13719_5 = {S52, 0, &V13719_5_1294, &P13719_1};	/*Property = ScreenPixelsHeight*/
const Value V13719_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13719_4 = {S54, 0, &V13719_4_1296, &P13719_5};	/*Property = ScreenPixelsWidth*/
const Profile P13719 = {"13719", &P13719_4};	/*Profile ID - 13719*/
const Value V13655_1_1 = {S2, NULL};	/*Value = True*/
const Property P13655_1 = {S50, 0, &V13655_1_1, NULL};	/*Property = IsMobile*/
const Value V13655_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13655_5 = {S52, 0, &V13655_5_1233, &P13655_1};	/*Property = ScreenPixelsHeight*/
const Value V13655_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13655_4 = {S54, 0, &V13655_4_1235, &P13655_5};	/*Property = ScreenPixelsWidth*/
const Profile P13655 = {"13655", &P13655_4};	/*Profile ID - 13655*/
const Value V15102_1_1 = {S2, NULL};	/*Value = True*/
const Property P15102_1 = {S50, 0, &V15102_1_1, NULL};	/*Property = IsMobile*/
const Value V15102_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P15102_5 = {S52, 0, &V15102_5_9981, &P15102_1};	/*Property = ScreenPixelsHeight*/
const Value V15102_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P15102_4 = {S54, 0, &V15102_4_2336, &P15102_5};	/*Property = ScreenPixelsWidth*/
const Profile P15102 = {"15102", &P15102_4};	/*Profile ID - 15102*/
const Value V15131_1_1 = {S2, NULL};	/*Value = True*/
const Property P15131_1 = {S50, 0, &V15131_1_1, NULL};	/*Property = IsMobile*/
const Value V15131_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15131_5 = {S52, 0, &V15131_5_1380, &P15131_1};	/*Property = ScreenPixelsHeight*/
const Value V15131_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15131_4 = {S54, 0, &V15131_4_1279, &P15131_5};	/*Property = ScreenPixelsWidth*/
const Profile P15131 = {"15131", &P15131_4};	/*Profile ID - 15131*/
const Value V15132_1_1 = {S2, NULL};	/*Value = True*/
const Property P15132_1 = {S50, 0, &V15132_1_1, NULL};	/*Property = IsMobile*/
const Value V15132_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15132_5 = {S52, 0, &V15132_5_1293, &P15132_1};	/*Property = ScreenPixelsHeight*/
const Value V15132_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15132_4 = {S54, 0, &V15132_4_1296, &P15132_5};	/*Property = ScreenPixelsWidth*/
const Profile P15132 = {"15132", &P15132_4};	/*Profile ID - 15132*/
const Value V23919_1_1 = {S2, NULL};	/*Value = True*/
const Property P23919_1 = {S50, 0, &V23919_1_1, NULL};	/*Property = IsMobile*/
const Value V23919_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23919_5 = {S52, 0, &V23919_5_1269, &P23919_1};	/*Property = ScreenPixelsHeight*/
const Value V23919_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23919_4 = {S54, 0, &V23919_4_1229, &P23919_5};	/*Property = ScreenPixelsWidth*/
const Profile P23919 = {"23919", &P23919_4};	/*Profile ID - 23919*/
const Value V23922_1_1 = {S2, NULL};	/*Value = True*/
const Property P23922_1 = {S50, 0, &V23922_1_1, NULL};	/*Property = IsMobile*/
const Value V23922_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23922_5 = {S52, 0, &V23922_5_1269, &P23922_1};	/*Property = ScreenPixelsHeight*/
const Value V23922_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23922_4 = {S54, 0, &V23922_4_1229, &P23922_5};	/*Property = ScreenPixelsWidth*/
const Profile P23922 = {"23922", &P23922_4};	/*Profile ID - 23922*/
const Value V15133_1_1 = {S2, NULL};	/*Value = True*/
const Property P15133_1 = {S50, 0, &V15133_1_1, NULL};	/*Property = IsMobile*/
const Value V15133_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P15133_5 = {S52, 0, &V15133_5_1269, &P15133_1};	/*Property = ScreenPixelsHeight*/
const Value V15133_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15133_4 = {S54, 0, &V15133_4_1229, &P15133_5};	/*Property = ScreenPixelsWidth*/
const Profile P15133 = {"15133", &P15133_4};	/*Profile ID - 15133*/
const Value V13610_1_1 = {S2, NULL};	/*Value = True*/
const Property P13610_1 = {S50, 0, &V13610_1_1, NULL};	/*Property = IsMobile*/
const Value V13610_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13610_5 = {S52, 0, &V13610_5_1304, &P13610_1};	/*Property = ScreenPixelsHeight*/
const Value V13610_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13610_4 = {S54, 0, &V13610_4_1224, &P13610_5};	/*Property = ScreenPixelsWidth*/
const Profile P13610 = {"13610", &P13610_4};	/*Profile ID - 13610*/
const Value V13866_1_1 = {S2, NULL};	/*Value = True*/
const Property P13866_1 = {S50, 0, &V13866_1_1, NULL};	/*Property = IsMobile*/
const Value V13866_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13866_5 = {S52, 0, &V13866_5_1222, &P13866_1};	/*Property = ScreenPixelsHeight*/
const Value V13866_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13866_4 = {S54, 0, &V13866_4_1224, &P13866_5};	/*Property = ScreenPixelsWidth*/
const Profile P13866 = {"13866", &P13866_4};	/*Profile ID - 13866*/
const Value V12425_1_1 = {S2, NULL};	/*Value = True*/
const Property P12425_1 = {S50, 0, &V12425_1_1, NULL};	/*Property = IsMobile*/
const Value V12425_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12425_5 = {S52, 0, &V12425_5_1233, &P12425_1};	/*Property = ScreenPixelsHeight*/
const Value V12425_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12425_4 = {S54, 0, &V12425_4_1235, &P12425_5};	/*Property = ScreenPixelsWidth*/
const Profile P12425 = {"12425", &P12425_4};	/*Profile ID - 12425*/
const Value V15137_1_1 = {S2, NULL};	/*Value = True*/
const Property P15137_1 = {S50, 0, &V15137_1_1, NULL};	/*Property = IsMobile*/
const Value V15137_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15137_5 = {S52, 0, &V15137_5_1222, &P15137_1};	/*Property = ScreenPixelsHeight*/
const Value V15137_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15137_4 = {S54, 0, &V15137_4_1224, &P15137_5};	/*Property = ScreenPixelsWidth*/
const Profile P15137 = {"15137", &P15137_4};	/*Profile ID - 15137*/
const Value V15620_1_1 = {S2, NULL};	/*Value = True*/
const Property P15620_1 = {S50, 0, &V15620_1_1, NULL};	/*Property = IsMobile*/
const Value V15620_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15620_5 = {S52, 0, &V15620_5_1293, &P15620_1};	/*Property = ScreenPixelsHeight*/
const Value V15620_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15620_4 = {S54, 0, &V15620_4_1296, &P15620_5};	/*Property = ScreenPixelsWidth*/
const Profile P15620 = {"15620", &P15620_4};	/*Profile ID - 15620*/
const Value V13568_1_1 = {S2, NULL};	/*Value = True*/
const Property P13568_1 = {S50, 0, &V13568_1_1, NULL};	/*Property = IsMobile*/
const Value V13568_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13568_5 = {S52, 0, &V13568_5_1293, &P13568_1};	/*Property = ScreenPixelsHeight*/
const Value V13568_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13568_4 = {S54, 0, &V13568_4_1296, &P13568_5};	/*Property = ScreenPixelsWidth*/
const Profile P13568 = {"13568", &P13568_4};	/*Profile ID - 13568*/
const Value V12212_1_1 = {S2, NULL};	/*Value = True*/
const Property P12212_1 = {S50, 0, &V12212_1_1, NULL};	/*Property = IsMobile*/
const Value V12212_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12212_5 = {S52, 0, &V12212_5_1233, &P12212_1};	/*Property = ScreenPixelsHeight*/
const Value V12212_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12212_4 = {S54, 0, &V12212_4_1235, &P12212_5};	/*Property = ScreenPixelsWidth*/
const Profile P12212 = {"12212", &P12212_4};	/*Profile ID - 12212*/
const Value V16092_1_1 = {S2, NULL};	/*Value = True*/
const Property P16092_1 = {S50, 0, &V16092_1_1, NULL};	/*Property = IsMobile*/
const Value V16092_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P16092_5 = {S52, 0, &V16092_5_1403, &P16092_1};	/*Property = ScreenPixelsHeight*/
const Value V16092_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P16092_4 = {S54, 0, &V16092_4_1404, &P16092_5};	/*Property = ScreenPixelsWidth*/
const Profile P16092 = {"16092", &P16092_4};	/*Profile ID - 16092*/
const Value V15140_1_1 = {S2, NULL};	/*Value = True*/
const Property P15140_1 = {S50, 0, &V15140_1_1, NULL};	/*Property = IsMobile*/
const Value V15140_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15140_5 = {S52, 0, &V15140_5_1304, &P15140_1};	/*Property = ScreenPixelsHeight*/
const Value V15140_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15140_4 = {S54, 0, &V15140_4_1224, &P15140_5};	/*Property = ScreenPixelsWidth*/
const Profile P15140 = {"15140", &P15140_4};	/*Profile ID - 15140*/
const Value V15142_1_1 = {S2, NULL};	/*Value = True*/
const Property P15142_1 = {S50, 0, &V15142_1_1, NULL};	/*Property = IsMobile*/
const Value V15142_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15142_5 = {S52, 0, &V15142_5_1380, &P15142_1};	/*Property = ScreenPixelsHeight*/
const Value V15142_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15142_4 = {S54, 0, &V15142_4_1279, &P15142_5};	/*Property = ScreenPixelsWidth*/
const Profile P15142 = {"15142", &P15142_4};	/*Profile ID - 15142*/
const Value V15143_1_1 = {S2, NULL};	/*Value = True*/
const Property P15143_1 = {S50, 0, &V15143_1_1, NULL};	/*Property = IsMobile*/
const Value V15143_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15143_5 = {S52, 0, &V15143_5_1293, &P15143_1};	/*Property = ScreenPixelsHeight*/
const Value V15143_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15143_4 = {S54, 0, &V15143_4_1296, &P15143_5};	/*Property = ScreenPixelsWidth*/
const Profile P15143 = {"15143", &P15143_4};	/*Profile ID - 15143*/
const Value V15145_1_1 = {S2, NULL};	/*Value = True*/
const Property P15145_1 = {S50, 0, &V15145_1_1, NULL};	/*Property = IsMobile*/
const Value V15145_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15145_5 = {S52, 0, &V15145_5_1380, &P15145_1};	/*Property = ScreenPixelsHeight*/
const Value V15145_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15145_4 = {S54, 0, &V15145_4_1279, &P15145_5};	/*Property = ScreenPixelsWidth*/
const Profile P15145 = {"15145", &P15145_4};	/*Profile ID - 15145*/
const Value V15146_1_1 = {S2, NULL};	/*Value = True*/
const Property P15146_1 = {S50, 0, &V15146_1_1, NULL};	/*Property = IsMobile*/
const Value V15146_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15146_5 = {S52, 0, &V15146_5_1233, &P15146_1};	/*Property = ScreenPixelsHeight*/
const Value V15146_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15146_4 = {S54, 0, &V15146_4_1235, &P15146_5};	/*Property = ScreenPixelsWidth*/
const Profile P15146 = {"15146", &P15146_4};	/*Profile ID - 15146*/
const Value V11961_1_1 = {S2, NULL};	/*Value = True*/
const Property P11961_1 = {S50, 0, &V11961_1_1, NULL};	/*Property = IsMobile*/
const Value V11961_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11961_5 = {S52, 0, &V11961_5_1233, &P11961_1};	/*Property = ScreenPixelsHeight*/
const Value V11961_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11961_4 = {S54, 0, &V11961_4_1235, &P11961_5};	/*Property = ScreenPixelsWidth*/
const Profile P11961 = {"11961", &P11961_4};	/*Profile ID - 11961*/
const Value V11963_1_1 = {S2, NULL};	/*Value = True*/
const Property P11963_1 = {S50, 0, &V11963_1_1, NULL};	/*Property = IsMobile*/
const Value V11963_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11963_5 = {S52, 0, &V11963_5_1293, &P11963_1};	/*Property = ScreenPixelsHeight*/
const Value V11963_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11963_4 = {S54, 0, &V11963_4_1296, &P11963_5};	/*Property = ScreenPixelsWidth*/
const Profile P11963 = {"11963", &P11963_4};	/*Profile ID - 11963*/
const Value V11992_1_1 = {S2, NULL};	/*Value = True*/
const Property P11992_1 = {S50, 0, &V11992_1_1, NULL};	/*Property = IsMobile*/
const Value V11992_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11992_5 = {S52, 0, &V11992_5_1293, &P11992_1};	/*Property = ScreenPixelsHeight*/
const Value V11992_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11992_4 = {S54, 0, &V11992_4_1296, &P11992_5};	/*Property = ScreenPixelsWidth*/
const Profile P11992 = {"11992", &P11992_4};	/*Profile ID - 11992*/
const Value V11966_1_1 = {S2, NULL};	/*Value = True*/
const Property P11966_1 = {S50, 0, &V11966_1_1, NULL};	/*Property = IsMobile*/
const Value V11966_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11966_5 = {S52, 0, &V11966_5_1222, &P11966_1};	/*Property = ScreenPixelsHeight*/
const Value V11966_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11966_4 = {S54, 0, &V11966_4_1224, &P11966_5};	/*Property = ScreenPixelsWidth*/
const Profile P11966 = {"11966", &P11966_4};	/*Profile ID - 11966*/
const Value V17918_1_1 = {S2, NULL};	/*Value = True*/
const Property P17918_1 = {S50, 0, &V17918_1_1, NULL};	/*Property = IsMobile*/
const Value V17918_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17918_5 = {S52, 0, &V17918_5_1222, &P17918_1};	/*Property = ScreenPixelsHeight*/
const Value V17918_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17918_4 = {S54, 0, &V17918_4_1224, &P17918_5};	/*Property = ScreenPixelsWidth*/
const Profile P17918 = {"17918", &P17918_4};	/*Profile ID - 17918*/
const Value V11967_1_1 = {S2, NULL};	/*Value = True*/
const Property P11967_1 = {S50, 0, &V11967_1_1, NULL};	/*Property = IsMobile*/
const Value V11967_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11967_5 = {S52, 0, &V11967_5_1293, &P11967_1};	/*Property = ScreenPixelsHeight*/
const Value V11967_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11967_4 = {S54, 0, &V11967_4_1296, &P11967_5};	/*Property = ScreenPixelsWidth*/
const Profile P11967 = {"11967", &P11967_4};	/*Profile ID - 11967*/
const Value V14280_1_1 = {S2, NULL};	/*Value = True*/
const Property P14280_1 = {S50, 0, &V14280_1_1, NULL};	/*Property = IsMobile*/
const Value V14280_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14280_5 = {S52, 0, &V14280_5_1222, &P14280_1};	/*Property = ScreenPixelsHeight*/
const Value V14280_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14280_4 = {S54, 0, &V14280_4_1224, &P14280_5};	/*Property = ScreenPixelsWidth*/
const Profile P14280 = {"14280", &P14280_4};	/*Profile ID - 14280*/
const Value V17773_1_1 = {S2, NULL};	/*Value = True*/
const Property P17773_1 = {S50, 0, &V17773_1_1, NULL};	/*Property = IsMobile*/
const Value V17773_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17773_5 = {S52, 0, &V17773_5_1222, &P17773_1};	/*Property = ScreenPixelsHeight*/
const Value V17773_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17773_4 = {S54, 0, &V17773_4_1224, &P17773_5};	/*Property = ScreenPixelsWidth*/
const Profile P17773 = {"17773", &P17773_4};	/*Profile ID - 17773*/
const Value V15150_1_1 = {S2, NULL};	/*Value = True*/
const Property P15150_1 = {S50, 0, &V15150_1_1, NULL};	/*Property = IsMobile*/
const Value V15150_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15150_5 = {S52, 0, &V15150_5_1222, &P15150_1};	/*Property = ScreenPixelsHeight*/
const Value V15150_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15150_4 = {S54, 0, &V15150_4_1224, &P15150_5};	/*Property = ScreenPixelsWidth*/
const Profile P15150 = {"15150", &P15150_4};	/*Profile ID - 15150*/
const Value V16100_1_1 = {S2, NULL};	/*Value = True*/
const Property P16100_1 = {S50, 0, &V16100_1_1, NULL};	/*Property = IsMobile*/
const Value V16100_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16100_5 = {S52, 0, &V16100_5_1293, &P16100_1};	/*Property = ScreenPixelsHeight*/
const Value V16100_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16100_4 = {S54, 0, &V16100_4_1296, &P16100_5};	/*Property = ScreenPixelsWidth*/
const Profile P16100 = {"16100", &P16100_4};	/*Profile ID - 16100*/
const Value V4901_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_279 = {S1, 0, &V4901_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4901_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_282 = {S3, 0, &V4901_282_57155, &P4901_279};	/*Property = BlobBuilder*/
const Value V4901_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_350 = {S4, 0, &V4901_350_57446, &P4901_282};	/*Property = Canvas*/
const Value V4901_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_283 = {S5, 0, &V4901_283_57161, &P4901_350};	/*Property = CssBackground*/
const Value V4901_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_284 = {S6, 0, &V4901_284_57164, &P4901_283};	/*Property = CssBorderImage*/
const Value V4901_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_285 = {S7, 0, &V4901_285_57167, &P4901_284};	/*Property = CssCanvas*/
const Value V4901_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_286 = {S8, 0, &V4901_286_57170, &P4901_285};	/*Property = CssColor*/
const Value V4901_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_237 = {S9, 0, &V4901_237_55786, &P4901_286};	/*Property = CssColumn*/
const Value V4901_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_287 = {S10, 0, &V4901_287_57173, &P4901_237};	/*Property = CssFlexbox*/
const Value V4901_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_288 = {S11, 0, &V4901_288_57176, &P4901_287};	/*Property = CssFont*/
const Value V4901_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_289 = {S12, 0, &V4901_289_86899, &P4901_288};	/*Property = CssImages*/
const Value V4901_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_291 = {S13, 0, &V4901_291_57182, &P4901_289};	/*Property = CssMediaQueries*/
const Value V4901_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_292 = {S14, 0, &V4901_292_57185, &P4901_291};	/*Property = CssMinMax*/
const Value V4901_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_293 = {S15, 0, &V4901_293_57188, &P4901_292};	/*Property = CssOverflow*/
const Value V4901_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_294 = {S16, 0, &V4901_294_57191, &P4901_293};	/*Property = CssPosition*/
const Value V4901_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_295 = {S17, 0, &V4901_295_57194, &P4901_294};	/*Property = CssText*/
const Value V4901_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_297 = {S18, 0, &V4901_297_57197, &P4901_295};	/*Property = CssTransforms*/
const Value V4901_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_241 = {S19, 0, &V4901_241_55801, &P4901_297};	/*Property = CssTransitions*/
const Value V4901_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_298 = {S20, 0, &V4901_298_57208, &P4901_241};	/*Property = CssUI*/
const Value V4901_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_300 = {S21, 0, &V4901_300_57217, &P4901_298};	/*Property = DataSet*/
const Value V4901_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_301 = {S22, 0, &V4901_301_57220, &P4901_300};	/*Property = DataUrl*/
const Value V4901_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_302 = {S23, 0, &V4901_302_57223, &P4901_301};	/*Property = DeviceOrientation*/
const Value V4901_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_304 = {S24, 0, &V4901_304_57229, &P4901_302};	/*Property = FileReader*/
const Value V4901_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_305 = {S25, 0, &V4901_305_57232, &P4901_304};	/*Property = FileSaver*/
const Value V4901_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_306 = {S26, 0, &V4901_306_57238, &P4901_305};	/*Property = FileWriter*/
const Value V4901_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_307 = {S27, 0, &V4901_307_57241, &P4901_306};	/*Property = FormData*/
const Value V4901_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_309 = {S28, 0, &V4901_309_57247, &P4901_307};	/*Property = Fullscreen*/
const Value V4901_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_310 = {S29, 0, &V4901_310_57250, &P4901_309};	/*Property = GeoLocation*/
const Value V4901_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_312 = {S30, 0, &V4901_312_57256, &P4901_310};	/*Property = History*/
const Value V4901_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_314 = {S31, 0, &V4901_314_57262, &P4901_312};	/*Property = Html5*/
const Value V4901_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_313 = {S32, 0, &V4901_313_57259, &P4901_314};	/*Property = Html-Media-Capture*/
const Value V4901_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_315 = {S33, 0, &V4901_315_57265, &P4901_313};	/*Property = Iframe*/
const Value V4901_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_316 = {S34, 0, &V4901_316_57268, &P4901_315};	/*Property = IndexedDB*/
const Value V4901_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_317 = {S35, 0, &V4901_317_57271, &P4901_316};	/*Property = Json*/
const Value V4901_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_15 = {S37, 0, &V4901_15_72, &P4901_317};	/*Property = LayoutEngine*/
const Value V4901_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_344 = {S38, 0, &V4901_344_57274, &P4901_15};	/*Property = Masking*/
const Value V4901_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_320 = {S39, 0, &V4901_320_57292, &P4901_344};	/*Property = PostMessage*/
const Value V4901_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_321 = {S40, 0, &V4901_321_57295, &P4901_320};	/*Property = Progress*/
const Value V4901_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_322 = {S41, 0, &V4901_322_57298, &P4901_321};	/*Property = Prompts*/
const Value V4901_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_323 = {S42, 0, &V4901_323_57301, &P4901_322};	/*Property = Selector*/
const Value V4901_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_324 = {S43, 0, &V4901_324_57307, &P4901_323};	/*Property = Svg*/
const Value V4901_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_327 = {S44, 0, &V4901_327_57316, &P4901_324};	/*Property = TouchEvents*/
const Value V4901_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_328 = {S45, 0, &V4901_328_57319, &P4901_327};	/*Property = Track*/
const Value V4901_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_329 = {S46, 0, &V4901_329_57325, &P4901_328};	/*Property = Video*/
const Value V4901_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_330 = {S47, 0, &V4901_330_57328, &P4901_329};	/*Property = Viewport*/
const Value V4901_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_332 = {S48, 0, &V4901_332_57340, &P4901_330};	/*Property = WebWorkers*/
const Value V4901_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4901_333 = {S49, 0, &V4901_333_57343, &P4901_332};	/*Property = Xhr2*/
const Profile P4901 = {"4901", &P4901_333};	/*Profile ID - 4901*/
const Value V14136_1_1 = {S2, NULL};	/*Value = True*/
const Property P14136_1 = {S50, 0, &V14136_1_1, NULL};	/*Property = IsMobile*/
const Value V14136_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14136_5 = {S52, 0, &V14136_5_1293, &P14136_1};	/*Property = ScreenPixelsHeight*/
const Value V14136_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14136_4 = {S54, 0, &V14136_4_1296, &P14136_5};	/*Property = ScreenPixelsWidth*/
const Profile P14136 = {"14136", &P14136_4};	/*Profile ID - 14136*/
const Value V13820_1_1 = {S2, NULL};	/*Value = True*/
const Property P13820_1 = {S50, 0, &V13820_1_1, NULL};	/*Property = IsMobile*/
const Value V13820_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13820_5 = {S52, 0, &V13820_5_1293, &P13820_1};	/*Property = ScreenPixelsHeight*/
const Value V13820_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13820_4 = {S54, 0, &V13820_4_1296, &P13820_5};	/*Property = ScreenPixelsWidth*/
const Profile P13820 = {"13820", &P13820_4};	/*Profile ID - 13820*/
const Value V14282_1_1 = {S2, NULL};	/*Value = True*/
const Property P14282_1 = {S50, 0, &V14282_1_1, NULL};	/*Property = IsMobile*/
const Value V14282_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14282_5 = {S52, 0, &V14282_5_1294, &P14282_1};	/*Property = ScreenPixelsHeight*/
const Value V14282_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14282_4 = {S54, 0, &V14282_4_1296, &P14282_5};	/*Property = ScreenPixelsWidth*/
const Profile P14282 = {"14282", &P14282_4};	/*Profile ID - 14282*/
const Value V12833_1_1 = {S2, NULL};	/*Value = True*/
const Property P12833_1 = {S50, 0, &V12833_1_1, NULL};	/*Property = IsMobile*/
const Value V12833_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12833_5 = {S52, 0, &V12833_5_1380, &P12833_1};	/*Property = ScreenPixelsHeight*/
const Value V12833_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12833_4 = {S54, 0, &V12833_4_1279, &P12833_5};	/*Property = ScreenPixelsWidth*/
const Profile P12833 = {"12833", &P12833_4};	/*Profile ID - 12833*/
const Value V15152_1_1 = {S2, NULL};	/*Value = True*/
const Property P15152_1 = {S50, 0, &V15152_1_1, NULL};	/*Property = IsMobile*/
const Value V15152_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15152_5 = {S52, 0, &V15152_5_1380, &P15152_1};	/*Property = ScreenPixelsHeight*/
const Value V15152_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15152_4 = {S54, 0, &V15152_4_1279, &P15152_5};	/*Property = ScreenPixelsWidth*/
const Profile P15152 = {"15152", &P15152_4};	/*Profile ID - 15152*/
const Value V11969_1_1 = {S2, NULL};	/*Value = True*/
const Property P11969_1 = {S50, 0, &V11969_1_1, NULL};	/*Property = IsMobile*/
const Value V11969_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11969_5 = {S52, 0, &V11969_5_1222, &P11969_1};	/*Property = ScreenPixelsHeight*/
const Value V11969_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11969_4 = {S54, 0, &V11969_4_1224, &P11969_5};	/*Property = ScreenPixelsWidth*/
const Profile P11969 = {"11969", &P11969_4};	/*Profile ID - 11969*/
const Value V14404_1_1 = {S2, NULL};	/*Value = True*/
const Property P14404_1 = {S50, 0, &V14404_1_1, NULL};	/*Property = IsMobile*/
const Value V14404_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14404_5 = {S52, 0, &V14404_5_1304, &P14404_1};	/*Property = ScreenPixelsHeight*/
const Value V14404_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14404_4 = {S54, 0, &V14404_4_1224, &P14404_5};	/*Property = ScreenPixelsWidth*/
const Profile P14404 = {"14404", &P14404_4};	/*Profile ID - 14404*/
const Value V16471_1_1 = {S2, NULL};	/*Value = True*/
const Property P16471_1 = {S50, 0, &V16471_1_1, NULL};	/*Property = IsMobile*/
const Value V16471_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16471_5 = {S52, 0, &V16471_5_1233, &P16471_1};	/*Property = ScreenPixelsHeight*/
const Value V16471_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16471_4 = {S54, 0, &V16471_4_1235, &P16471_5};	/*Property = ScreenPixelsWidth*/
const Profile P16471 = {"16471", &P16471_4};	/*Profile ID - 16471*/
const Value V15153_1_1 = {S2, NULL};	/*Value = True*/
const Property P15153_1 = {S50, 0, &V15153_1_1, NULL};	/*Property = IsMobile*/
const Value V15153_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15153_5 = {S52, 0, &V15153_5_1293, &P15153_1};	/*Property = ScreenPixelsHeight*/
const Value V15153_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15153_4 = {S54, 0, &V15153_4_1296, &P15153_5};	/*Property = ScreenPixelsWidth*/
const Profile P15153 = {"15153", &P15153_4};	/*Profile ID - 15153*/
const Value V14211_1_1 = {S2, NULL};	/*Value = True*/
const Property P14211_1 = {S50, 0, &V14211_1_1, NULL};	/*Property = IsMobile*/
const Value V14211_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P14211_5 = {S52, 0, &V14211_5_1458, &P14211_1};	/*Property = ScreenPixelsHeight*/
const Value V14211_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P14211_4 = {S54, 0, &V14211_4_1460, &P14211_5};	/*Property = ScreenPixelsWidth*/
const Profile P14211 = {"14211", &P14211_4};	/*Profile ID - 14211*/
const Value V12146_1_1 = {S2, NULL};	/*Value = True*/
const Property P12146_1 = {S50, 0, &V12146_1_1, NULL};	/*Property = IsMobile*/
const Value V12146_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12146_5 = {S52, 0, &V12146_5_1233, &P12146_1};	/*Property = ScreenPixelsHeight*/
const Value V12146_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12146_4 = {S54, 0, &V12146_4_1235, &P12146_5};	/*Property = ScreenPixelsWidth*/
const Profile P12146 = {"12146", &P12146_4};	/*Profile ID - 12146*/
const Value V17518_1_1 = {S2, NULL};	/*Value = True*/
const Property P17518_1 = {S50, 0, &V17518_1_1, NULL};	/*Property = IsMobile*/
const Value V17518_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17518_5 = {S52, 0, &V17518_5_1269, &P17518_1};	/*Property = ScreenPixelsHeight*/
const Value V17518_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17518_4 = {S54, 0, &V17518_4_1229, &P17518_5};	/*Property = ScreenPixelsWidth*/
const Profile P17518 = {"17518", &P17518_4};	/*Profile ID - 17518*/
const Value V11974_1_1 = {S2, NULL};	/*Value = True*/
const Property P11974_1 = {S50, 0, &V11974_1_1, NULL};	/*Property = IsMobile*/
const Value V11974_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11974_5 = {S52, 0, &V11974_5_1233, &P11974_1};	/*Property = ScreenPixelsHeight*/
const Value V11974_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11974_4 = {S54, 0, &V11974_4_1235, &P11974_5};	/*Property = ScreenPixelsWidth*/
const Profile P11974 = {"11974", &P11974_4};	/*Profile ID - 11974*/
const Value V11975_1_1 = {S2, NULL};	/*Value = True*/
const Property P11975_1 = {S50, 0, &V11975_1_1, NULL};	/*Property = IsMobile*/
const Value V11975_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11975_5 = {S52, 0, &V11975_5_1222, &P11975_1};	/*Property = ScreenPixelsHeight*/
const Value V11975_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11975_4 = {S54, 0, &V11975_4_1224, &P11975_5};	/*Property = ScreenPixelsWidth*/
const Profile P11975 = {"11975", &P11975_4};	/*Profile ID - 11975*/
const Value V11976_1_1 = {S2, NULL};	/*Value = True*/
const Property P11976_1 = {S50, 0, &V11976_1_1, NULL};	/*Property = IsMobile*/
const Value V11976_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11976_5 = {S52, 0, &V11976_5_1233, &P11976_1};	/*Property = ScreenPixelsHeight*/
const Value V11976_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11976_4 = {S54, 0, &V11976_4_1235, &P11976_5};	/*Property = ScreenPixelsWidth*/
const Profile P11976 = {"11976", &P11976_4};	/*Profile ID - 11976*/
const Value V17111_1_1 = {S2, NULL};	/*Value = True*/
const Property P17111_1 = {S50, 0, &V17111_1_1, NULL};	/*Property = IsMobile*/
const Value V17111_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17111_5 = {S52, 0, &V17111_5_1222, &P17111_1};	/*Property = ScreenPixelsHeight*/
const Value V17111_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17111_4 = {S54, 0, &V17111_4_1224, &P17111_5};	/*Property = ScreenPixelsWidth*/
const Profile P17111 = {"17111", &P17111_4};	/*Profile ID - 17111*/
const Value V12718_1_1 = {S2, NULL};	/*Value = True*/
const Property P12718_1 = {S50, 0, &V12718_1_1, NULL};	/*Property = IsMobile*/
const Value V12718_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12718_5 = {S52, 0, &V12718_5_1277, &P12718_1};	/*Property = ScreenPixelsHeight*/
const Value V12718_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P12718_4 = {S54, 0, &V12718_4_1278, &P12718_5};	/*Property = ScreenPixelsWidth*/
const Profile P12718 = {"12718", &P12718_4};	/*Profile ID - 12718*/
const Value V15167_1_1 = {S2, NULL};	/*Value = True*/
const Property P15167_1 = {S50, 0, &V15167_1_1, NULL};	/*Property = IsMobile*/
const Value V15167_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15167_5 = {S52, 0, &V15167_5_1304, &P15167_1};	/*Property = ScreenPixelsHeight*/
const Value V15167_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15167_4 = {S54, 0, &V15167_4_1224, &P15167_5};	/*Property = ScreenPixelsWidth*/
const Profile P15167 = {"15167", &P15167_4};	/*Profile ID - 15167*/
const Profile P7227 = {"7227", NULL};	/*Profile ID - 7227*/
const Value V17064_1_1 = {S2, NULL};	/*Value = True*/
const Property P17064_1 = {S50, 0, &V17064_1_1, NULL};	/*Property = IsMobile*/
const Value V17064_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17064_5 = {S52, 0, &V17064_5_1222, &P17064_1};	/*Property = ScreenPixelsHeight*/
const Value V17064_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17064_4 = {S54, 0, &V17064_4_1224, &P17064_5};	/*Property = ScreenPixelsWidth*/
const Profile P17064 = {"17064", &P17064_4};	/*Profile ID - 17064*/
const Value V15169_1_1 = {S2, NULL};	/*Value = True*/
const Property P15169_1 = {S50, 0, &V15169_1_1, NULL};	/*Property = IsMobile*/
const Value V15169_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15169_5 = {S52, 0, &V15169_5_1294, &P15169_1};	/*Property = ScreenPixelsHeight*/
const Value V15169_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15169_4 = {S54, 0, &V15169_4_1296, &P15169_5};	/*Property = ScreenPixelsWidth*/
const Profile P15169 = {"15169", &P15169_4};	/*Profile ID - 15169*/
const Value V15170_1_1 = {S2, NULL};	/*Value = True*/
const Property P15170_1 = {S50, 0, &V15170_1_1, NULL};	/*Property = IsMobile*/
const Value V15170_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15170_5 = {S52, 0, &V15170_5_1222, &P15170_1};	/*Property = ScreenPixelsHeight*/
const Value V15170_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15170_4 = {S54, 0, &V15170_4_1224, &P15170_5};	/*Property = ScreenPixelsWidth*/
const Profile P15170 = {"15170", &P15170_4};	/*Profile ID - 15170*/
const Value V14865_1_1 = {S2, NULL};	/*Value = True*/
const Property P14865_1 = {S50, 0, &V14865_1_1, NULL};	/*Property = IsMobile*/
const Value V14865_5_1512 = {S77, NULL};	/*Value = 416*/
const Property P14865_5 = {S52, 0, &V14865_5_1512, &P14865_1};	/*Property = ScreenPixelsHeight*/
const Value V14865_4_1513 = {S78, NULL};	/*Value = 352*/
const Property P14865_4 = {S54, 0, &V14865_4_1513, &P14865_5};	/*Property = ScreenPixelsWidth*/
const Profile P14865 = {"14865", &P14865_4};	/*Profile ID - 14865*/
const Value V11981_1_1 = {S2, NULL};	/*Value = True*/
const Property P11981_1 = {S50, 0, &V11981_1_1, NULL};	/*Property = IsMobile*/
const Value V11981_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11981_5 = {S52, 0, &V11981_5_1222, &P11981_1};	/*Property = ScreenPixelsHeight*/
const Value V11981_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11981_4 = {S54, 0, &V11981_4_1224, &P11981_5};	/*Property = ScreenPixelsWidth*/
const Profile P11981 = {"11981", &P11981_4};	/*Profile ID - 11981*/
const Value V16529_1_1 = {S2, NULL};	/*Value = True*/
const Property P16529_1 = {S50, 0, &V16529_1_1, NULL};	/*Property = IsMobile*/
const Value V16529_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16529_5 = {S52, 0, &V16529_5_1294, &P16529_1};	/*Property = ScreenPixelsHeight*/
const Value V16529_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16529_4 = {S54, 0, &V16529_4_1296, &P16529_5};	/*Property = ScreenPixelsWidth*/
const Profile P16529 = {"16529", &P16529_4};	/*Profile ID - 16529*/
const Value V15173_1_1 = {S2, NULL};	/*Value = True*/
const Property P15173_1 = {S50, 0, &V15173_1_1, NULL};	/*Property = IsMobile*/
const Value V15173_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15173_5 = {S52, 0, &V15173_5_1233, &P15173_1};	/*Property = ScreenPixelsHeight*/
const Value V15173_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15173_4 = {S54, 0, &V15173_4_1235, &P15173_5};	/*Property = ScreenPixelsWidth*/
const Profile P15173 = {"15173", &P15173_4};	/*Profile ID - 15173*/
const Value V15174_1_1 = {S2, NULL};	/*Value = True*/
const Property P15174_1 = {S50, 0, &V15174_1_1, NULL};	/*Property = IsMobile*/
const Value V15174_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15174_5 = {S52, 0, &V15174_5_1222, &P15174_1};	/*Property = ScreenPixelsHeight*/
const Value V15174_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15174_4 = {S54, 0, &V15174_4_1224, &P15174_5};	/*Property = ScreenPixelsWidth*/
const Profile P15174 = {"15174", &P15174_4};	/*Profile ID - 15174*/
const Value V15176_1_1 = {S2, NULL};	/*Value = True*/
const Property P15176_1 = {S50, 0, &V15176_1_1, NULL};	/*Property = IsMobile*/
const Value V15176_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15176_5 = {S52, 0, &V15176_5_1222, &P15176_1};	/*Property = ScreenPixelsHeight*/
const Value V15176_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15176_4 = {S54, 0, &V15176_4_1224, &P15176_5};	/*Property = ScreenPixelsWidth*/
const Profile P15176 = {"15176", &P15176_4};	/*Profile ID - 15176*/
const Value V22371_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_279 = {S1, 0, &V22371_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22371_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_282 = {S3, 0, &V22371_282_57155, &P22371_279};	/*Property = BlobBuilder*/
const Value V22371_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_350 = {S4, 0, &V22371_350_57446, &P22371_282};	/*Property = Canvas*/
const Value V22371_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_283 = {S5, 0, &V22371_283_57161, &P22371_350};	/*Property = CssBackground*/
const Value V22371_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_284 = {S6, 0, &V22371_284_57164, &P22371_283};	/*Property = CssBorderImage*/
const Value V22371_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_285 = {S7, 0, &V22371_285_57167, &P22371_284};	/*Property = CssCanvas*/
const Value V22371_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_286 = {S8, 0, &V22371_286_57170, &P22371_285};	/*Property = CssColor*/
const Value V22371_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_237 = {S9, 0, &V22371_237_55786, &P22371_286};	/*Property = CssColumn*/
const Value V22371_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_287 = {S10, 0, &V22371_287_57173, &P22371_237};	/*Property = CssFlexbox*/
const Value V22371_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_288 = {S11, 0, &V22371_288_57176, &P22371_287};	/*Property = CssFont*/
const Value V22371_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_289 = {S12, 0, &V22371_289_86899, &P22371_288};	/*Property = CssImages*/
const Value V22371_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_291 = {S13, 0, &V22371_291_57182, &P22371_289};	/*Property = CssMediaQueries*/
const Value V22371_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_292 = {S14, 0, &V22371_292_57185, &P22371_291};	/*Property = CssMinMax*/
const Value V22371_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_293 = {S15, 0, &V22371_293_57188, &P22371_292};	/*Property = CssOverflow*/
const Value V22371_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_294 = {S16, 0, &V22371_294_57191, &P22371_293};	/*Property = CssPosition*/
const Value V22371_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_295 = {S17, 0, &V22371_295_57194, &P22371_294};	/*Property = CssText*/
const Value V22371_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_297 = {S18, 0, &V22371_297_57197, &P22371_295};	/*Property = CssTransforms*/
const Value V22371_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_241 = {S19, 0, &V22371_241_55801, &P22371_297};	/*Property = CssTransitions*/
const Value V22371_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_298 = {S20, 0, &V22371_298_57208, &P22371_241};	/*Property = CssUI*/
const Value V22371_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_300 = {S21, 0, &V22371_300_57217, &P22371_298};	/*Property = DataSet*/
const Value V22371_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_301 = {S22, 0, &V22371_301_57220, &P22371_300};	/*Property = DataUrl*/
const Value V22371_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_302 = {S23, 0, &V22371_302_57223, &P22371_301};	/*Property = DeviceOrientation*/
const Value V22371_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_304 = {S24, 0, &V22371_304_57229, &P22371_302};	/*Property = FileReader*/
const Value V22371_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_305 = {S25, 0, &V22371_305_57232, &P22371_304};	/*Property = FileSaver*/
const Value V22371_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_306 = {S26, 0, &V22371_306_57238, &P22371_305};	/*Property = FileWriter*/
const Value V22371_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_307 = {S27, 0, &V22371_307_57241, &P22371_306};	/*Property = FormData*/
const Value V22371_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_309 = {S28, 0, &V22371_309_57247, &P22371_307};	/*Property = Fullscreen*/
const Value V22371_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_310 = {S29, 0, &V22371_310_57250, &P22371_309};	/*Property = GeoLocation*/
const Value V22371_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_312 = {S30, 0, &V22371_312_57256, &P22371_310};	/*Property = History*/
const Value V22371_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_314 = {S31, 0, &V22371_314_57262, &P22371_312};	/*Property = Html5*/
const Value V22371_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_313 = {S32, 0, &V22371_313_57259, &P22371_314};	/*Property = Html-Media-Capture*/
const Value V22371_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_315 = {S33, 0, &V22371_315_57265, &P22371_313};	/*Property = Iframe*/
const Value V22371_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_316 = {S34, 0, &V22371_316_57268, &P22371_315};	/*Property = IndexedDB*/
const Value V22371_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_317 = {S35, 0, &V22371_317_57271, &P22371_316};	/*Property = Json*/
const Value V22371_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P22371_15 = {S37, 0, &V22371_15_30, &P22371_317};	/*Property = LayoutEngine*/
const Value V22371_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_344 = {S38, 0, &V22371_344_57274, &P22371_15};	/*Property = Masking*/
const Value V22371_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_320 = {S39, 0, &V22371_320_57292, &P22371_344};	/*Property = PostMessage*/
const Value V22371_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_321 = {S40, 0, &V22371_321_57295, &P22371_320};	/*Property = Progress*/
const Value V22371_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_322 = {S41, 0, &V22371_322_57298, &P22371_321};	/*Property = Prompts*/
const Value V22371_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_323 = {S42, 0, &V22371_323_57301, &P22371_322};	/*Property = Selector*/
const Value V22371_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_324 = {S43, 0, &V22371_324_57307, &P22371_323};	/*Property = Svg*/
const Value V22371_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_327 = {S44, 0, &V22371_327_57316, &P22371_324};	/*Property = TouchEvents*/
const Value V22371_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_328 = {S45, 0, &V22371_328_57319, &P22371_327};	/*Property = Track*/
const Value V22371_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_329 = {S46, 0, &V22371_329_57325, &P22371_328};	/*Property = Video*/
const Value V22371_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_330 = {S47, 0, &V22371_330_57328, &P22371_329};	/*Property = Viewport*/
const Value V22371_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_332 = {S48, 0, &V22371_332_57340, &P22371_330};	/*Property = WebWorkers*/
const Value V22371_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22371_333 = {S49, 0, &V22371_333_57343, &P22371_332};	/*Property = Xhr2*/
const Profile P22371 = {"22371", &P22371_333};	/*Profile ID - 22371*/
const Value V15178_1_1 = {S2, NULL};	/*Value = True*/
const Property P15178_1 = {S50, 0, &V15178_1_1, NULL};	/*Property = IsMobile*/
const Value V15178_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15178_5 = {S52, 0, &V15178_5_1277, &P15178_1};	/*Property = ScreenPixelsHeight*/
const Value V15178_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15178_4 = {S54, 0, &V15178_4_1279, &P15178_5};	/*Property = ScreenPixelsWidth*/
const Profile P15178 = {"15178", &P15178_4};	/*Profile ID - 15178*/
const Value V13541_1_1 = {S2, NULL};	/*Value = True*/
const Property P13541_1 = {S50, 0, &V13541_1_1, NULL};	/*Property = IsMobile*/
const Value V13541_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13541_5 = {S52, 0, &V13541_5_1233, &P13541_1};	/*Property = ScreenPixelsHeight*/
const Value V13541_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13541_4 = {S54, 0, &V13541_4_1235, &P13541_5};	/*Property = ScreenPixelsWidth*/
const Profile P13541 = {"13541", &P13541_4};	/*Profile ID - 13541*/
const Value V15179_1_1 = {S2, NULL};	/*Value = True*/
const Property P15179_1 = {S50, 0, &V15179_1_1, NULL};	/*Property = IsMobile*/
const Value V15179_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15179_5 = {S52, 0, &V15179_5_1233, &P15179_1};	/*Property = ScreenPixelsHeight*/
const Value V15179_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15179_4 = {S54, 0, &V15179_4_1235, &P15179_5};	/*Property = ScreenPixelsWidth*/
const Profile P15179 = {"15179", &P15179_4};	/*Profile ID - 15179*/
const Value V15180_1_1 = {S2, NULL};	/*Value = True*/
const Property P15180_1 = {S50, 0, &V15180_1_1, NULL};	/*Property = IsMobile*/
const Value V15180_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15180_5 = {S52, 0, &V15180_5_1222, &P15180_1};	/*Property = ScreenPixelsHeight*/
const Value V15180_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15180_4 = {S54, 0, &V15180_4_1224, &P15180_5};	/*Property = ScreenPixelsWidth*/
const Profile P15180 = {"15180", &P15180_4};	/*Profile ID - 15180*/
const Value V15181_1_1 = {S2, NULL};	/*Value = True*/
const Property P15181_1 = {S50, 0, &V15181_1_1, NULL};	/*Property = IsMobile*/
const Value V15181_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15181_5 = {S52, 0, &V15181_5_1293, &P15181_1};	/*Property = ScreenPixelsHeight*/
const Value V15181_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15181_4 = {S54, 0, &V15181_4_1296, &P15181_5};	/*Property = ScreenPixelsWidth*/
const Profile P15181 = {"15181", &P15181_4};	/*Profile ID - 15181*/
const Value V15185_1_1 = {S2, NULL};	/*Value = True*/
const Property P15185_1 = {S50, 0, &V15185_1_1, NULL};	/*Property = IsMobile*/
const Value V15185_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15185_5 = {S52, 0, &V15185_5_1294, &P15185_1};	/*Property = ScreenPixelsHeight*/
const Value V15185_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15185_4 = {S54, 0, &V15185_4_1296, &P15185_5};	/*Property = ScreenPixelsWidth*/
const Profile P15185 = {"15185", &P15185_4};	/*Profile ID - 15185*/
const Value V12795_1_1 = {S2, NULL};	/*Value = True*/
const Property P12795_1 = {S50, 0, &V12795_1_1, NULL};	/*Property = IsMobile*/
const Value V12795_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12795_5 = {S52, 0, &V12795_5_1222, &P12795_1};	/*Property = ScreenPixelsHeight*/
const Value V12795_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12795_4 = {S54, 0, &V12795_4_1224, &P12795_5};	/*Property = ScreenPixelsWidth*/
const Profile P12795 = {"12795", &P12795_4};	/*Profile ID - 12795*/
const Value V15187_1_1 = {S2, NULL};	/*Value = True*/
const Property P15187_1 = {S50, 0, &V15187_1_1, NULL};	/*Property = IsMobile*/
const Value V15187_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15187_5 = {S52, 0, &V15187_5_1222, &P15187_1};	/*Property = ScreenPixelsHeight*/
const Value V15187_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15187_4 = {S54, 0, &V15187_4_1224, &P15187_5};	/*Property = ScreenPixelsWidth*/
const Profile P15187 = {"15187", &P15187_4};	/*Profile ID - 15187*/
const Value V15188_1_1 = {S2, NULL};	/*Value = True*/
const Property P15188_1 = {S50, 0, &V15188_1_1, NULL};	/*Property = IsMobile*/
const Value V15188_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15188_5 = {S52, 0, &V15188_5_1227, &P15188_1};	/*Property = ScreenPixelsHeight*/
const Value V15188_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15188_4 = {S54, 0, &V15188_4_1229, &P15188_5};	/*Property = ScreenPixelsWidth*/
const Profile P15188 = {"15188", &P15188_4};	/*Profile ID - 15188*/
const Value V13893_1_1 = {S2, NULL};	/*Value = True*/
const Property P13893_1 = {S50, 0, &V13893_1_1, NULL};	/*Property = IsMobile*/
const Value V13893_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13893_5 = {S52, 0, &V13893_5_1294, &P13893_1};	/*Property = ScreenPixelsHeight*/
const Value V13893_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13893_4 = {S54, 0, &V13893_4_1296, &P13893_5};	/*Property = ScreenPixelsWidth*/
const Profile P13893 = {"13893", &P13893_4};	/*Profile ID - 13893*/
const Value V15189_1_1 = {S2, NULL};	/*Value = True*/
const Property P15189_1 = {S50, 0, &V15189_1_1, NULL};	/*Property = IsMobile*/
const Value V15189_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15189_5 = {S52, 0, &V15189_5_1233, &P15189_1};	/*Property = ScreenPixelsHeight*/
const Value V15189_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15189_4 = {S54, 0, &V15189_4_1235, &P15189_5};	/*Property = ScreenPixelsWidth*/
const Profile P15189 = {"15189", &P15189_4};	/*Profile ID - 15189*/
const Value V15436_1_1 = {S2, NULL};	/*Value = True*/
const Property P15436_1 = {S50, 0, &V15436_1_1, NULL};	/*Property = IsMobile*/
const Value V15436_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15436_5 = {S52, 0, &V15436_5_1222, &P15436_1};	/*Property = ScreenPixelsHeight*/
const Value V15436_4_1368 = {S74, NULL};	/*Value = 208*/
const Property P15436_4 = {S54, 0, &V15436_4_1368, &P15436_5};	/*Property = ScreenPixelsWidth*/
const Profile P15436 = {"15436", &P15436_4};	/*Profile ID - 15436*/
const Value V15190_1_1 = {S2, NULL};	/*Value = True*/
const Property P15190_1 = {S50, 0, &V15190_1_1, NULL};	/*Property = IsMobile*/
const Value V15190_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15190_5 = {S52, 0, &V15190_5_1293, &P15190_1};	/*Property = ScreenPixelsHeight*/
const Value V15190_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15190_4 = {S54, 0, &V15190_4_1296, &P15190_5};	/*Property = ScreenPixelsWidth*/
const Profile P15190 = {"15190", &P15190_4};	/*Profile ID - 15190*/
const Value V15191_1_1 = {S2, NULL};	/*Value = True*/
const Property P15191_1 = {S50, 0, &V15191_1_1, NULL};	/*Property = IsMobile*/
const Value V15191_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15191_5 = {S52, 0, &V15191_5_1294, &P15191_1};	/*Property = ScreenPixelsHeight*/
const Value V15191_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15191_4 = {S54, 0, &V15191_4_1296, &P15191_5};	/*Property = ScreenPixelsWidth*/
const Profile P15191 = {"15191", &P15191_4};	/*Profile ID - 15191*/
const Value V11987_1_1 = {S2, NULL};	/*Value = True*/
const Property P11987_1 = {S50, 0, &V11987_1_1, NULL};	/*Property = IsMobile*/
const Value V11987_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P11987_5 = {S52, 0, &V11987_5_1304, &P11987_1};	/*Property = ScreenPixelsHeight*/
const Value V11987_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11987_4 = {S54, 0, &V11987_4_1224, &P11987_5};	/*Property = ScreenPixelsWidth*/
const Profile P11987 = {"11987", &P11987_4};	/*Profile ID - 11987*/
const Value V15564_1_1 = {S2, NULL};	/*Value = True*/
const Property P15564_1 = {S50, 0, &V15564_1_1, NULL};	/*Property = IsMobile*/
const Value V15564_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15564_5 = {S52, 0, &V15564_5_1380, &P15564_1};	/*Property = ScreenPixelsHeight*/
const Value V15564_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15564_4 = {S54, 0, &V15564_4_1279, &P15564_5};	/*Property = ScreenPixelsWidth*/
const Profile P15564 = {"15564", &P15564_4};	/*Profile ID - 15564*/
const Value V15265_1_1 = {S2, NULL};	/*Value = True*/
const Property P15265_1 = {S50, 0, &V15265_1_1, NULL};	/*Property = IsMobile*/
const Value V15265_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15265_5 = {S52, 0, &V15265_5_1233, &P15265_1};	/*Property = ScreenPixelsHeight*/
const Value V15265_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15265_4 = {S54, 0, &V15265_4_1235, &P15265_5};	/*Property = ScreenPixelsWidth*/
const Profile P15265 = {"15265", &P15265_4};	/*Profile ID - 15265*/
const Value V15267_1_1 = {S2, NULL};	/*Value = True*/
const Property P15267_1 = {S50, 0, &V15267_1_1, NULL};	/*Property = IsMobile*/
const Value V15267_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15267_5 = {S52, 0, &V15267_5_1294, &P15267_1};	/*Property = ScreenPixelsHeight*/
const Value V15267_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15267_4 = {S54, 0, &V15267_4_1296, &P15267_5};	/*Property = ScreenPixelsWidth*/
const Profile P15267 = {"15267", &P15267_4};	/*Profile ID - 15267*/
const Value V15266_1_1 = {S2, NULL};	/*Value = True*/
const Property P15266_1 = {S50, 0, &V15266_1_1, NULL};	/*Property = IsMobile*/
const Value V15266_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15266_5 = {S52, 0, &V15266_5_1293, &P15266_1};	/*Property = ScreenPixelsHeight*/
const Value V15266_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15266_4 = {S54, 0, &V15266_4_1296, &P15266_5};	/*Property = ScreenPixelsWidth*/
const Profile P15266 = {"15266", &P15266_4};	/*Profile ID - 15266*/
const Value V14376_1_1 = {S2, NULL};	/*Value = True*/
const Property P14376_1 = {S50, 0, &V14376_1_1, NULL};	/*Property = IsMobile*/
const Value V14376_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P14376_5 = {S52, 0, &V14376_5_37, &P14376_1};	/*Property = ScreenPixelsHeight*/
const Value V14376_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P14376_4 = {S54, 0, &V14376_4_36, &P14376_5};	/*Property = ScreenPixelsWidth*/
const Profile P14376 = {"14376", &P14376_4};	/*Profile ID - 14376*/
const Value V15270_1_1 = {S2, NULL};	/*Value = True*/
const Property P15270_1 = {S50, 0, &V15270_1_1, NULL};	/*Property = IsMobile*/
const Value V15270_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15270_5 = {S52, 0, &V15270_5_1293, &P15270_1};	/*Property = ScreenPixelsHeight*/
const Value V15270_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15270_4 = {S54, 0, &V15270_4_1296, &P15270_5};	/*Property = ScreenPixelsWidth*/
const Profile P15270 = {"15270", &P15270_4};	/*Profile ID - 15270*/
const Value V13214_1_1 = {S2, NULL};	/*Value = True*/
const Property P13214_1 = {S50, 0, &V13214_1_1, NULL};	/*Property = IsMobile*/
const Value V13214_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P13214_5 = {S52, 0, &V13214_5_1333, &P13214_1};	/*Property = ScreenPixelsHeight*/
const Value V13214_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P13214_4 = {S54, 0, &V13214_4_1335, &P13214_5};	/*Property = ScreenPixelsWidth*/
const Profile P13214 = {"13214", &P13214_4};	/*Profile ID - 13214*/
const Value V11993_1_1 = {S2, NULL};	/*Value = True*/
const Property P11993_1 = {S50, 0, &V11993_1_1, NULL};	/*Property = IsMobile*/
const Value V11993_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P11993_5 = {S52, 0, &V11993_5_1280, &P11993_1};	/*Property = ScreenPixelsHeight*/
const Value V11993_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11993_4 = {S54, 0, &V11993_4_1235, &P11993_5};	/*Property = ScreenPixelsWidth*/
const Profile P11993 = {"11993", &P11993_4};	/*Profile ID - 11993*/
const Value V11988_1_1 = {S2, NULL};	/*Value = True*/
const Property P11988_1 = {S50, 0, &V11988_1_1, NULL};	/*Property = IsMobile*/
const Value V11988_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11988_5 = {S52, 0, &V11988_5_1222, &P11988_1};	/*Property = ScreenPixelsHeight*/
const Value V11988_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11988_4 = {S54, 0, &V11988_4_1224, &P11988_5};	/*Property = ScreenPixelsWidth*/
const Profile P11988 = {"11988", &P11988_4};	/*Profile ID - 11988*/
const Value V15273_1_1 = {S2, NULL};	/*Value = True*/
const Property P15273_1 = {S50, 0, &V15273_1_1, NULL};	/*Property = IsMobile*/
const Value V15273_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15273_5 = {S52, 0, &V15273_5_1222, &P15273_1};	/*Property = ScreenPixelsHeight*/
const Value V15273_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15273_4 = {S54, 0, &V15273_4_1224, &P15273_5};	/*Property = ScreenPixelsWidth*/
const Profile P15273 = {"15273", &P15273_4};	/*Profile ID - 15273*/
const Value V16738_1_1 = {S2, NULL};	/*Value = True*/
const Property P16738_1 = {S50, 0, &V16738_1_1, NULL};	/*Property = IsMobile*/
const Value V16738_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16738_5 = {S52, 0, &V16738_5_1293, &P16738_1};	/*Property = ScreenPixelsHeight*/
const Value V16738_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16738_4 = {S54, 0, &V16738_4_1296, &P16738_5};	/*Property = ScreenPixelsWidth*/
const Profile P16738 = {"16738", &P16738_4};	/*Profile ID - 16738*/
const Value V15277_1_1 = {S2, NULL};	/*Value = True*/
const Property P15277_1 = {S50, 0, &V15277_1_1, NULL};	/*Property = IsMobile*/
const Value V15277_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15277_5 = {S52, 0, &V15277_5_1222, &P15277_1};	/*Property = ScreenPixelsHeight*/
const Value V15277_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15277_4 = {S54, 0, &V15277_4_1224, &P15277_5};	/*Property = ScreenPixelsWidth*/
const Profile P15277 = {"15277", &P15277_4};	/*Profile ID - 15277*/
const Value V13936_1_1 = {S2, NULL};	/*Value = True*/
const Property P13936_1 = {S50, 0, &V13936_1_1, NULL};	/*Property = IsMobile*/
const Value V13936_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13936_5 = {S52, 0, &V13936_5_1293, &P13936_1};	/*Property = ScreenPixelsHeight*/
const Value V13936_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13936_4 = {S54, 0, &V13936_4_1296, &P13936_5};	/*Property = ScreenPixelsWidth*/
const Profile P13936 = {"13936", &P13936_4};	/*Profile ID - 13936*/
const Value V16980_1_1 = {S2, NULL};	/*Value = True*/
const Property P16980_1 = {S50, 0, &V16980_1_1, NULL};	/*Property = IsMobile*/
const Value V16980_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16980_5 = {S52, 0, &V16980_5_1222, &P16980_1};	/*Property = ScreenPixelsHeight*/
const Value V16980_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16980_4 = {S54, 0, &V16980_4_1279, &P16980_5};	/*Property = ScreenPixelsWidth*/
const Profile P16980 = {"16980", &P16980_4};	/*Profile ID - 16980*/
const Value V15279_1_1 = {S2, NULL};	/*Value = True*/
const Property P15279_1 = {S50, 0, &V15279_1_1, NULL};	/*Property = IsMobile*/
const Value V15279_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15279_5 = {S52, 0, &V15279_5_1233, &P15279_1};	/*Property = ScreenPixelsHeight*/
const Value V15279_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15279_4 = {S54, 0, &V15279_4_1235, &P15279_5};	/*Property = ScreenPixelsWidth*/
const Profile P15279 = {"15279", &P15279_4};	/*Profile ID - 15279*/
const Value V15280_1_1 = {S2, NULL};	/*Value = True*/
const Property P15280_1 = {S50, 0, &V15280_1_1, NULL};	/*Property = IsMobile*/
const Value V15280_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15280_5 = {S52, 0, &V15280_5_1222, &P15280_1};	/*Property = ScreenPixelsHeight*/
const Value V15280_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15280_4 = {S54, 0, &V15280_4_1224, &P15280_5};	/*Property = ScreenPixelsWidth*/
const Profile P15280 = {"15280", &P15280_4};	/*Profile ID - 15280*/
const Value V15282_1_1 = {S2, NULL};	/*Value = True*/
const Property P15282_1 = {S50, 0, &V15282_1_1, NULL};	/*Property = IsMobile*/
const Value V15282_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15282_5 = {S52, 0, &V15282_5_1222, &P15282_1};	/*Property = ScreenPixelsHeight*/
const Value V15282_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15282_4 = {S54, 0, &V15282_4_1224, &P15282_5};	/*Property = ScreenPixelsWidth*/
const Profile P15282 = {"15282", &P15282_4};	/*Profile ID - 15282*/
const Value V15283_1_1 = {S2, NULL};	/*Value = True*/
const Property P15283_1 = {S50, 0, &V15283_1_1, NULL};	/*Property = IsMobile*/
const Value V15283_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15283_5 = {S52, 0, &V15283_5_1294, &P15283_1};	/*Property = ScreenPixelsHeight*/
const Value V15283_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15283_4 = {S54, 0, &V15283_4_1296, &P15283_5};	/*Property = ScreenPixelsWidth*/
const Profile P15283 = {"15283", &P15283_4};	/*Profile ID - 15283*/
const Value V4643_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_279 = {S1, 0, &V4643_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4643_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_282 = {S3, 0, &V4643_282_57155, &P4643_279};	/*Property = BlobBuilder*/
const Value V4643_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_350 = {S4, 0, &V4643_350_57446, &P4643_282};	/*Property = Canvas*/
const Value V4643_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_283 = {S5, 0, &V4643_283_57161, &P4643_350};	/*Property = CssBackground*/
const Value V4643_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_284 = {S6, 0, &V4643_284_57164, &P4643_283};	/*Property = CssBorderImage*/
const Value V4643_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_285 = {S7, 0, &V4643_285_57167, &P4643_284};	/*Property = CssCanvas*/
const Value V4643_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_286 = {S8, 0, &V4643_286_57170, &P4643_285};	/*Property = CssColor*/
const Value V4643_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_237 = {S9, 0, &V4643_237_55786, &P4643_286};	/*Property = CssColumn*/
const Value V4643_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_287 = {S10, 0, &V4643_287_57173, &P4643_237};	/*Property = CssFlexbox*/
const Value V4643_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_288 = {S11, 0, &V4643_288_57176, &P4643_287};	/*Property = CssFont*/
const Value V4643_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_289 = {S12, 0, &V4643_289_86899, &P4643_288};	/*Property = CssImages*/
const Value V4643_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_291 = {S13, 0, &V4643_291_57182, &P4643_289};	/*Property = CssMediaQueries*/
const Value V4643_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_292 = {S14, 0, &V4643_292_57185, &P4643_291};	/*Property = CssMinMax*/
const Value V4643_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_293 = {S15, 0, &V4643_293_57188, &P4643_292};	/*Property = CssOverflow*/
const Value V4643_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_294 = {S16, 0, &V4643_294_57191, &P4643_293};	/*Property = CssPosition*/
const Value V4643_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_295 = {S17, 0, &V4643_295_57194, &P4643_294};	/*Property = CssText*/
const Value V4643_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_297 = {S18, 0, &V4643_297_57197, &P4643_295};	/*Property = CssTransforms*/
const Value V4643_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_241 = {S19, 0, &V4643_241_55801, &P4643_297};	/*Property = CssTransitions*/
const Value V4643_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_298 = {S20, 0, &V4643_298_57208, &P4643_241};	/*Property = CssUI*/
const Value V4643_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_300 = {S21, 0, &V4643_300_57217, &P4643_298};	/*Property = DataSet*/
const Value V4643_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_301 = {S22, 0, &V4643_301_57220, &P4643_300};	/*Property = DataUrl*/
const Value V4643_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_302 = {S23, 0, &V4643_302_57223, &P4643_301};	/*Property = DeviceOrientation*/
const Value V4643_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_304 = {S24, 0, &V4643_304_57229, &P4643_302};	/*Property = FileReader*/
const Value V4643_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_305 = {S25, 0, &V4643_305_57232, &P4643_304};	/*Property = FileSaver*/
const Value V4643_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_306 = {S26, 0, &V4643_306_57238, &P4643_305};	/*Property = FileWriter*/
const Value V4643_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_307 = {S27, 0, &V4643_307_57241, &P4643_306};	/*Property = FormData*/
const Value V4643_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_309 = {S28, 0, &V4643_309_57247, &P4643_307};	/*Property = Fullscreen*/
const Value V4643_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_310 = {S29, 0, &V4643_310_57250, &P4643_309};	/*Property = GeoLocation*/
const Value V4643_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_312 = {S30, 0, &V4643_312_57256, &P4643_310};	/*Property = History*/
const Value V4643_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_314 = {S31, 0, &V4643_314_57262, &P4643_312};	/*Property = Html5*/
const Value V4643_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_313 = {S32, 0, &V4643_313_57259, &P4643_314};	/*Property = Html-Media-Capture*/
const Value V4643_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_315 = {S33, 0, &V4643_315_57265, &P4643_313};	/*Property = Iframe*/
const Value V4643_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_316 = {S34, 0, &V4643_316_57268, &P4643_315};	/*Property = IndexedDB*/
const Value V4643_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_317 = {S35, 0, &V4643_317_57271, &P4643_316};	/*Property = Json*/
const Value V4643_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P4643_15 = {S37, 0, &V4643_15_30, &P4643_317};	/*Property = LayoutEngine*/
const Value V4643_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_344 = {S38, 0, &V4643_344_57274, &P4643_15};	/*Property = Masking*/
const Value V4643_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_320 = {S39, 0, &V4643_320_57292, &P4643_344};	/*Property = PostMessage*/
const Value V4643_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_321 = {S40, 0, &V4643_321_57295, &P4643_320};	/*Property = Progress*/
const Value V4643_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_322 = {S41, 0, &V4643_322_57298, &P4643_321};	/*Property = Prompts*/
const Value V4643_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_323 = {S42, 0, &V4643_323_57301, &P4643_322};	/*Property = Selector*/
const Value V4643_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_324 = {S43, 0, &V4643_324_57307, &P4643_323};	/*Property = Svg*/
const Value V4643_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_327 = {S44, 0, &V4643_327_57316, &P4643_324};	/*Property = TouchEvents*/
const Value V4643_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_328 = {S45, 0, &V4643_328_57319, &P4643_327};	/*Property = Track*/
const Value V4643_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_329 = {S46, 0, &V4643_329_57325, &P4643_328};	/*Property = Video*/
const Value V4643_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_330 = {S47, 0, &V4643_330_57328, &P4643_329};	/*Property = Viewport*/
const Value V4643_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_332 = {S48, 0, &V4643_332_57340, &P4643_330};	/*Property = WebWorkers*/
const Value V4643_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4643_333 = {S49, 0, &V4643_333_57343, &P4643_332};	/*Property = Xhr2*/
const Profile P4643 = {"4643", &P4643_333};	/*Profile ID - 4643*/
const Value V15284_1_1 = {S2, NULL};	/*Value = True*/
const Property P15284_1 = {S50, 0, &V15284_1_1, NULL};	/*Property = IsMobile*/
const Value V15284_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15284_5 = {S52, 0, &V15284_5_1304, &P15284_1};	/*Property = ScreenPixelsHeight*/
const Value V15284_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15284_4 = {S54, 0, &V15284_4_1224, &P15284_5};	/*Property = ScreenPixelsWidth*/
const Profile P15284 = {"15284", &P15284_4};	/*Profile ID - 15284*/
const Value V15285_1_1 = {S2, NULL};	/*Value = True*/
const Property P15285_1 = {S50, 0, &V15285_1_1, NULL};	/*Property = IsMobile*/
const Value V15285_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15285_5 = {S52, 0, &V15285_5_1222, &P15285_1};	/*Property = ScreenPixelsHeight*/
const Value V15285_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15285_4 = {S54, 0, &V15285_4_1224, &P15285_5};	/*Property = ScreenPixelsWidth*/
const Profile P15285 = {"15285", &P15285_4};	/*Profile ID - 15285*/
const Value V15286_1_1 = {S2, NULL};	/*Value = True*/
const Property P15286_1 = {S50, 0, &V15286_1_1, NULL};	/*Property = IsMobile*/
const Value V15286_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15286_5 = {S52, 0, &V15286_5_1222, &P15286_1};	/*Property = ScreenPixelsHeight*/
const Value V15286_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15286_4 = {S54, 0, &V15286_4_1224, &P15286_5};	/*Property = ScreenPixelsWidth*/
const Profile P15286 = {"15286", &P15286_4};	/*Profile ID - 15286*/
const Value V15287_1_1 = {S2, NULL};	/*Value = True*/
const Property P15287_1 = {S50, 0, &V15287_1_1, NULL};	/*Property = IsMobile*/
const Value V15287_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15287_5 = {S52, 0, &V15287_5_1293, &P15287_1};	/*Property = ScreenPixelsHeight*/
const Value V15287_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15287_4 = {S54, 0, &V15287_4_1296, &P15287_5};	/*Property = ScreenPixelsWidth*/
const Profile P15287 = {"15287", &P15287_4};	/*Profile ID - 15287*/
const Value V15289_1_1 = {S2, NULL};	/*Value = True*/
const Property P15289_1 = {S50, 0, &V15289_1_1, NULL};	/*Property = IsMobile*/
const Value V15289_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15289_5 = {S52, 0, &V15289_5_1293, &P15289_1};	/*Property = ScreenPixelsHeight*/
const Value V15289_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15289_4 = {S54, 0, &V15289_4_1296, &P15289_5};	/*Property = ScreenPixelsWidth*/
const Profile P15289 = {"15289", &P15289_4};	/*Profile ID - 15289*/
const Value V15292_1_1 = {S2, NULL};	/*Value = True*/
const Property P15292_1 = {S50, 0, &V15292_1_1, NULL};	/*Property = IsMobile*/
const Value V15292_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15292_5 = {S52, 0, &V15292_5_1233, &P15292_1};	/*Property = ScreenPixelsHeight*/
const Value V15292_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15292_4 = {S54, 0, &V15292_4_1235, &P15292_5};	/*Property = ScreenPixelsWidth*/
const Profile P15292 = {"15292", &P15292_4};	/*Profile ID - 15292*/
const Value V15294_1_1 = {S2, NULL};	/*Value = True*/
const Property P15294_1 = {S50, 0, &V15294_1_1, NULL};	/*Property = IsMobile*/
const Value V15294_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15294_5 = {S52, 0, &V15294_5_1294, &P15294_1};	/*Property = ScreenPixelsHeight*/
const Value V15294_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15294_4 = {S54, 0, &V15294_4_1296, &P15294_5};	/*Property = ScreenPixelsWidth*/
const Profile P15294 = {"15294", &P15294_4};	/*Profile ID - 15294*/
const Value V15295_1_1 = {S2, NULL};	/*Value = True*/
const Property P15295_1 = {S50, 0, &V15295_1_1, NULL};	/*Property = IsMobile*/
const Value V15295_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15295_5 = {S52, 0, &V15295_5_1293, &P15295_1};	/*Property = ScreenPixelsHeight*/
const Value V15295_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15295_4 = {S54, 0, &V15295_4_1296, &P15295_5};	/*Property = ScreenPixelsWidth*/
const Profile P15295 = {"15295", &P15295_4};	/*Profile ID - 15295*/
const Value V15297_1_1 = {S2, NULL};	/*Value = True*/
const Property P15297_1 = {S50, 0, &V15297_1_1, NULL};	/*Property = IsMobile*/
const Value V15297_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P15297_5 = {S52, 0, &V15297_5_1269, &P15297_1};	/*Property = ScreenPixelsHeight*/
const Value V15297_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15297_4 = {S54, 0, &V15297_4_1229, &P15297_5};	/*Property = ScreenPixelsWidth*/
const Profile P15297 = {"15297", &P15297_4};	/*Profile ID - 15297*/
const Value V15274_1_1 = {S2, NULL};	/*Value = True*/
const Property P15274_1 = {S50, 0, &V15274_1_1, NULL};	/*Property = IsMobile*/
const Value V15274_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15274_5 = {S52, 0, &V15274_5_1293, &P15274_1};	/*Property = ScreenPixelsHeight*/
const Value V15274_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15274_4 = {S54, 0, &V15274_4_1296, &P15274_5};	/*Property = ScreenPixelsWidth*/
const Profile P15274 = {"15274", &P15274_4};	/*Profile ID - 15274*/
const Value V15299_1_1 = {S2, NULL};	/*Value = True*/
const Property P15299_1 = {S50, 0, &V15299_1_1, NULL};	/*Property = IsMobile*/
const Value V15299_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15299_5 = {S52, 0, &V15299_5_1380, &P15299_1};	/*Property = ScreenPixelsHeight*/
const Value V15299_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P15299_4 = {S54, 0, &V15299_4_1943, &P15299_5};	/*Property = ScreenPixelsWidth*/
const Profile P15299 = {"15299", &P15299_4};	/*Profile ID - 15299*/
const Value V15301_1_1 = {S2, NULL};	/*Value = True*/
const Property P15301_1 = {S50, 0, &V15301_1_1, NULL};	/*Property = IsMobile*/
const Value V15301_5_37567 = {S95, NULL};	/*Value = 300*/
const Property P15301_5 = {S52, 0, &V15301_5_37567, &P15301_1};	/*Property = ScreenPixelsHeight*/
const Value V15301_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15301_4 = {S54, 0, &V15301_4_1224, &P15301_5};	/*Property = ScreenPixelsWidth*/
const Profile P15301 = {"15301", &P15301_4};	/*Profile ID - 15301*/
const Value V15303_1_1 = {S2, NULL};	/*Value = True*/
const Property P15303_1 = {S50, 0, &V15303_1_1, NULL};	/*Property = IsMobile*/
const Value V15303_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15303_5 = {S52, 0, &V15303_5_1304, &P15303_1};	/*Property = ScreenPixelsHeight*/
const Value V15303_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15303_4 = {S54, 0, &V15303_4_1224, &P15303_5};	/*Property = ScreenPixelsWidth*/
const Profile P15303 = {"15303", &P15303_4};	/*Profile ID - 15303*/
const Value V15304_1_1 = {S2, NULL};	/*Value = True*/
const Property P15304_1 = {S50, 0, &V15304_1_1, NULL};	/*Property = IsMobile*/
const Value V15304_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15304_5 = {S52, 0, &V15304_5_1277, &P15304_1};	/*Property = ScreenPixelsHeight*/
const Value V15304_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15304_4 = {S54, 0, &V15304_4_1224, &P15304_5};	/*Property = ScreenPixelsWidth*/
const Profile P15304 = {"15304", &P15304_4};	/*Profile ID - 15304*/
const Value V15305_1_1 = {S2, NULL};	/*Value = True*/
const Property P15305_1 = {S50, 0, &V15305_1_1, NULL};	/*Property = IsMobile*/
const Value V15305_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15305_5 = {S52, 0, &V15305_5_1222, &P15305_1};	/*Property = ScreenPixelsHeight*/
const Value V15305_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15305_4 = {S54, 0, &V15305_4_1224, &P15305_5};	/*Property = ScreenPixelsWidth*/
const Profile P15305 = {"15305", &P15305_4};	/*Profile ID - 15305*/
const Value V12772_1_1 = {S2, NULL};	/*Value = True*/
const Property P12772_1 = {S50, 0, &V12772_1_1, NULL};	/*Property = IsMobile*/
const Value V12772_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12772_5 = {S52, 0, &V12772_5_1233, &P12772_1};	/*Property = ScreenPixelsHeight*/
const Value V12772_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12772_4 = {S54, 0, &V12772_4_1235, &P12772_5};	/*Property = ScreenPixelsWidth*/
const Profile P12772 = {"12772", &P12772_4};	/*Profile ID - 12772*/
const Value V15306_1_1 = {S2, NULL};	/*Value = True*/
const Property P15306_1 = {S50, 0, &V15306_1_1, NULL};	/*Property = IsMobile*/
const Value V15306_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15306_5 = {S52, 0, &V15306_5_1222, &P15306_1};	/*Property = ScreenPixelsHeight*/
const Value V15306_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15306_4 = {S54, 0, &V15306_4_1224, &P15306_5};	/*Property = ScreenPixelsWidth*/
const Profile P15306 = {"15306", &P15306_4};	/*Profile ID - 15306*/
const Value V13705_1_1 = {S2, NULL};	/*Value = True*/
const Property P13705_1 = {S50, 0, &V13705_1_1, NULL};	/*Property = IsMobile*/
const Value V13705_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P13705_5 = {S52, 0, &V13705_5_1333, &P13705_1};	/*Property = ScreenPixelsHeight*/
const Value V13705_4_37572 = {S93, NULL};	/*Value = 132*/
const Property P13705_4 = {S54, 0, &V13705_4_37572, &P13705_5};	/*Property = ScreenPixelsWidth*/
const Profile P13705 = {"13705", &P13705_4};	/*Profile ID - 13705*/
const Value V15309_1_1 = {S2, NULL};	/*Value = True*/
const Property P15309_1 = {S50, 0, &V15309_1_1, NULL};	/*Property = IsMobile*/
const Value V15309_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15309_5 = {S52, 0, &V15309_5_1293, &P15309_1};	/*Property = ScreenPixelsHeight*/
const Value V15309_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15309_4 = {S54, 0, &V15309_4_1296, &P15309_5};	/*Property = ScreenPixelsWidth*/
const Profile P15309 = {"15309", &P15309_4};	/*Profile ID - 15309*/
const Value V18043_1_1 = {S2, NULL};	/*Value = True*/
const Property P18043_1 = {S50, 0, &V18043_1_1, NULL};	/*Property = IsMobile*/
const Value V18043_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18043_5 = {S52, 0, &V18043_5_1227, &P18043_1};	/*Property = ScreenPixelsHeight*/
const Value V18043_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18043_4 = {S54, 0, &V18043_4_1229, &P18043_5};	/*Property = ScreenPixelsWidth*/
const Profile P18043 = {"18043", &P18043_4};	/*Profile ID - 18043*/
const Value V13294_1_1 = {S2, NULL};	/*Value = True*/
const Property P13294_1 = {S50, 0, &V13294_1_1, NULL};	/*Property = IsMobile*/
const Value V13294_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13294_5 = {S52, 0, &V13294_5_1222, &P13294_1};	/*Property = ScreenPixelsHeight*/
const Value V13294_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13294_4 = {S54, 0, &V13294_4_1224, &P13294_5};	/*Property = ScreenPixelsWidth*/
const Profile P13294 = {"13294", &P13294_4};	/*Profile ID - 13294*/
const Value V20393_1_2 = {S0, NULL};	/*Value = False*/
const Property P20393_1 = {S50, 0, &V20393_1_2, NULL};	/*Property = IsMobile*/
const Value V20393_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20393_5 = {S52, 0, &V20393_5_37, &P20393_1};	/*Property = ScreenPixelsHeight*/
const Value V20393_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20393_4 = {S54, 0, &V20393_4_36, &P20393_5};	/*Property = ScreenPixelsWidth*/
const Profile P20393 = {"20393", &P20393_4};	/*Profile ID - 20393*/
const Value V4381_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_279 = {S1, 0, &V4381_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4381_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_282 = {S3, 0, &V4381_282_57155, &P4381_279};	/*Property = BlobBuilder*/
const Value V4381_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_350 = {S4, 0, &V4381_350_57446, &P4381_282};	/*Property = Canvas*/
const Value V4381_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_283 = {S5, 0, &V4381_283_57161, &P4381_350};	/*Property = CssBackground*/
const Value V4381_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_284 = {S6, 0, &V4381_284_57164, &P4381_283};	/*Property = CssBorderImage*/
const Value V4381_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_285 = {S7, 0, &V4381_285_57167, &P4381_284};	/*Property = CssCanvas*/
const Value V4381_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_286 = {S8, 0, &V4381_286_57170, &P4381_285};	/*Property = CssColor*/
const Value V4381_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_237 = {S9, 0, &V4381_237_55786, &P4381_286};	/*Property = CssColumn*/
const Value V4381_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_287 = {S10, 0, &V4381_287_57173, &P4381_237};	/*Property = CssFlexbox*/
const Value V4381_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_288 = {S11, 0, &V4381_288_57176, &P4381_287};	/*Property = CssFont*/
const Value V4381_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_289 = {S12, 0, &V4381_289_86899, &P4381_288};	/*Property = CssImages*/
const Value V4381_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_291 = {S13, 0, &V4381_291_57182, &P4381_289};	/*Property = CssMediaQueries*/
const Value V4381_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_292 = {S14, 0, &V4381_292_57185, &P4381_291};	/*Property = CssMinMax*/
const Value V4381_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_293 = {S15, 0, &V4381_293_57188, &P4381_292};	/*Property = CssOverflow*/
const Value V4381_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_294 = {S16, 0, &V4381_294_57191, &P4381_293};	/*Property = CssPosition*/
const Value V4381_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_295 = {S17, 0, &V4381_295_57194, &P4381_294};	/*Property = CssText*/
const Value V4381_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_297 = {S18, 0, &V4381_297_57197, &P4381_295};	/*Property = CssTransforms*/
const Value V4381_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_241 = {S19, 0, &V4381_241_55801, &P4381_297};	/*Property = CssTransitions*/
const Value V4381_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_298 = {S20, 0, &V4381_298_57208, &P4381_241};	/*Property = CssUI*/
const Value V4381_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_300 = {S21, 0, &V4381_300_57217, &P4381_298};	/*Property = DataSet*/
const Value V4381_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_301 = {S22, 0, &V4381_301_57220, &P4381_300};	/*Property = DataUrl*/
const Value V4381_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_302 = {S23, 0, &V4381_302_57223, &P4381_301};	/*Property = DeviceOrientation*/
const Value V4381_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_304 = {S24, 0, &V4381_304_57229, &P4381_302};	/*Property = FileReader*/
const Value V4381_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_305 = {S25, 0, &V4381_305_57232, &P4381_304};	/*Property = FileSaver*/
const Value V4381_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_306 = {S26, 0, &V4381_306_57238, &P4381_305};	/*Property = FileWriter*/
const Value V4381_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_307 = {S27, 0, &V4381_307_57241, &P4381_306};	/*Property = FormData*/
const Value V4381_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_309 = {S28, 0, &V4381_309_57247, &P4381_307};	/*Property = Fullscreen*/
const Value V4381_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_310 = {S29, 0, &V4381_310_57250, &P4381_309};	/*Property = GeoLocation*/
const Value V4381_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_312 = {S30, 0, &V4381_312_57256, &P4381_310};	/*Property = History*/
const Value V4381_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_314 = {S31, 0, &V4381_314_57262, &P4381_312};	/*Property = Html5*/
const Value V4381_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_313 = {S32, 0, &V4381_313_57259, &P4381_314};	/*Property = Html-Media-Capture*/
const Value V4381_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_315 = {S33, 0, &V4381_315_57265, &P4381_313};	/*Property = Iframe*/
const Value V4381_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_316 = {S34, 0, &V4381_316_57268, &P4381_315};	/*Property = IndexedDB*/
const Value V4381_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_317 = {S35, 0, &V4381_317_57271, &P4381_316};	/*Property = Json*/
const Value V4381_15_43265 = {S88, NULL};	/*Value = Lumi*/
const Property P4381_15 = {S37, 0, &V4381_15_43265, &P4381_317};	/*Property = LayoutEngine*/
const Value V4381_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_344 = {S38, 0, &V4381_344_57274, &P4381_15};	/*Property = Masking*/
const Value V4381_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_320 = {S39, 0, &V4381_320_57292, &P4381_344};	/*Property = PostMessage*/
const Value V4381_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_321 = {S40, 0, &V4381_321_57295, &P4381_320};	/*Property = Progress*/
const Value V4381_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_322 = {S41, 0, &V4381_322_57298, &P4381_321};	/*Property = Prompts*/
const Value V4381_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_323 = {S42, 0, &V4381_323_57301, &P4381_322};	/*Property = Selector*/
const Value V4381_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_324 = {S43, 0, &V4381_324_57307, &P4381_323};	/*Property = Svg*/
const Value V4381_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_327 = {S44, 0, &V4381_327_57316, &P4381_324};	/*Property = TouchEvents*/
const Value V4381_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_328 = {S45, 0, &V4381_328_57319, &P4381_327};	/*Property = Track*/
const Value V4381_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_329 = {S46, 0, &V4381_329_57325, &P4381_328};	/*Property = Video*/
const Value V4381_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_330 = {S47, 0, &V4381_330_57328, &P4381_329};	/*Property = Viewport*/
const Value V4381_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_332 = {S48, 0, &V4381_332_57340, &P4381_330};	/*Property = WebWorkers*/
const Value V4381_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4381_333 = {S49, 0, &V4381_333_57343, &P4381_332};	/*Property = Xhr2*/
const Profile P4381 = {"4381", &P4381_333};	/*Profile ID - 4381*/
const Value V15314_1_1 = {S2, NULL};	/*Value = True*/
const Property P15314_1 = {S50, 0, &V15314_1_1, NULL};	/*Property = IsMobile*/
const Value V15314_5_31955 = {S92, NULL};	/*Value = 96*/
const Property P15314_5 = {S52, 0, &V15314_5_31955, &P15314_1};	/*Property = ScreenPixelsHeight*/
const Value V15314_4_31956 = {S92, NULL};	/*Value = 96*/
const Property P15314_4 = {S54, 0, &V15314_4_31956, &P15314_5};	/*Property = ScreenPixelsWidth*/
const Profile P15314 = {"15314", &P15314_4};	/*Profile ID - 15314*/
const Value V15316_1_1 = {S2, NULL};	/*Value = True*/
const Property P15316_1 = {S50, 0, &V15316_1_1, NULL};	/*Property = IsMobile*/
const Value V15316_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15316_5 = {S52, 0, &V15316_5_1293, &P15316_1};	/*Property = ScreenPixelsHeight*/
const Value V15316_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15316_4 = {S54, 0, &V15316_4_1296, &P15316_5};	/*Property = ScreenPixelsWidth*/
const Profile P15316 = {"15316", &P15316_4};	/*Profile ID - 15316*/
const Value V15315_1_1 = {S2, NULL};	/*Value = True*/
const Property P15315_1 = {S50, 0, &V15315_1_1, NULL};	/*Property = IsMobile*/
const Value V15315_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15315_5 = {S52, 0, &V15315_5_1222, &P15315_1};	/*Property = ScreenPixelsHeight*/
const Value V15315_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15315_4 = {S54, 0, &V15315_4_1224, &P15315_5};	/*Property = ScreenPixelsWidth*/
const Profile P15315 = {"15315", &P15315_4};	/*Profile ID - 15315*/
const Value V15318_1_1 = {S2, NULL};	/*Value = True*/
const Property P15318_1 = {S50, 0, &V15318_1_1, NULL};	/*Property = IsMobile*/
const Value V15318_5_37465 = {S96, NULL};	/*Value = 80*/
const Property P15318_5 = {S52, 0, &V15318_5_37465, &P15318_1};	/*Property = ScreenPixelsHeight*/
const Value V15318_4_37576 = {S97, NULL};	/*Value = 101*/
const Property P15318_4 = {S54, 0, &V15318_4_37576, &P15318_5};	/*Property = ScreenPixelsWidth*/
const Profile P15318 = {"15318", &P15318_4};	/*Profile ID - 15318*/
const Value V15317_1_1 = {S2, NULL};	/*Value = True*/
const Property P15317_1 = {S50, 0, &V15317_1_1, NULL};	/*Property = IsMobile*/
const Value V15317_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15317_5 = {S52, 0, &V15317_5_1293, &P15317_1};	/*Property = ScreenPixelsHeight*/
const Value V15317_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15317_4 = {S54, 0, &V15317_4_1296, &P15317_5};	/*Property = ScreenPixelsWidth*/
const Profile P15317 = {"15317", &P15317_4};	/*Profile ID - 15317*/
const Value V15319_1_1 = {S2, NULL};	/*Value = True*/
const Property P15319_1 = {S50, 0, &V15319_1_1, NULL};	/*Property = IsMobile*/
const Value V15319_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15319_5 = {S52, 0, &V15319_5_1293, &P15319_1};	/*Property = ScreenPixelsHeight*/
const Value V15319_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15319_4 = {S54, 0, &V15319_4_1296, &P15319_5};	/*Property = ScreenPixelsWidth*/
const Profile P15319 = {"15319", &P15319_4};	/*Profile ID - 15319*/
const Value V14091_1_1 = {S2, NULL};	/*Value = True*/
const Property P14091_1 = {S50, 0, &V14091_1_1, NULL};	/*Property = IsMobile*/
const Value V14091_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14091_5 = {S52, 0, &V14091_5_1222, &P14091_1};	/*Property = ScreenPixelsHeight*/
const Value V14091_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14091_4 = {S54, 0, &V14091_4_1224, &P14091_5};	/*Property = ScreenPixelsWidth*/
const Profile P14091 = {"14091", &P14091_4};	/*Profile ID - 14091*/
const Value V15320_1_1 = {S2, NULL};	/*Value = True*/
const Property P15320_1 = {S50, 0, &V15320_1_1, NULL};	/*Property = IsMobile*/
const Value V15320_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15320_5 = {S52, 0, &V15320_5_1222, &P15320_1};	/*Property = ScreenPixelsHeight*/
const Value V15320_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15320_4 = {S54, 0, &V15320_4_1224, &P15320_5};	/*Property = ScreenPixelsWidth*/
const Profile P15320 = {"15320", &P15320_4};	/*Profile ID - 15320*/
const Value V13564_1_1 = {S2, NULL};	/*Value = True*/
const Property P13564_1 = {S50, 0, &V13564_1_1, NULL};	/*Property = IsMobile*/
const Value V13564_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13564_5 = {S52, 0, &V13564_5_1294, &P13564_1};	/*Property = ScreenPixelsHeight*/
const Value V13564_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13564_4 = {S54, 0, &V13564_4_1296, &P13564_5};	/*Property = ScreenPixelsWidth*/
const Profile P13564 = {"13564", &P13564_4};	/*Profile ID - 13564*/
const Value V15321_1_1 = {S2, NULL};	/*Value = True*/
const Property P15321_1 = {S50, 0, &V15321_1_1, NULL};	/*Property = IsMobile*/
const Value V15321_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15321_5 = {S52, 0, &V15321_5_1233, &P15321_1};	/*Property = ScreenPixelsHeight*/
const Value V15321_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15321_4 = {S54, 0, &V15321_4_1235, &P15321_5};	/*Property = ScreenPixelsWidth*/
const Profile P15321 = {"15321", &P15321_4};	/*Profile ID - 15321*/
const Value V15322_1_1 = {S2, NULL};	/*Value = True*/
const Property P15322_1 = {S50, 0, &V15322_1_1, NULL};	/*Property = IsMobile*/
const Value V15322_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15322_5 = {S52, 0, &V15322_5_1293, &P15322_1};	/*Property = ScreenPixelsHeight*/
const Value V15322_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15322_4 = {S54, 0, &V15322_4_1296, &P15322_5};	/*Property = ScreenPixelsWidth*/
const Profile P15322 = {"15322", &P15322_4};	/*Profile ID - 15322*/
const Value V15323_1_1 = {S2, NULL};	/*Value = True*/
const Property P15323_1 = {S50, 0, &V15323_1_1, NULL};	/*Property = IsMobile*/
const Value V15323_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15323_5 = {S52, 0, &V15323_5_1293, &P15323_1};	/*Property = ScreenPixelsHeight*/
const Value V15323_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15323_4 = {S54, 0, &V15323_4_1296, &P15323_5};	/*Property = ScreenPixelsWidth*/
const Profile P15323 = {"15323", &P15323_4};	/*Profile ID - 15323*/
const Value V15020_1_1 = {S2, NULL};	/*Value = True*/
const Property P15020_1 = {S50, 0, &V15020_1_1, NULL};	/*Property = IsMobile*/
const Value V15020_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15020_5 = {S52, 0, &V15020_5_1222, &P15020_1};	/*Property = ScreenPixelsHeight*/
const Value V15020_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15020_4 = {S54, 0, &V15020_4_1224, &P15020_5};	/*Property = ScreenPixelsWidth*/
const Profile P15020 = {"15020", &P15020_4};	/*Profile ID - 15020*/
const Value V15326_1_1 = {S2, NULL};	/*Value = True*/
const Property P15326_1 = {S50, 0, &V15326_1_1, NULL};	/*Property = IsMobile*/
const Value V15326_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15326_5 = {S52, 0, &V15326_5_1233, &P15326_1};	/*Property = ScreenPixelsHeight*/
const Value V15326_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15326_4 = {S54, 0, &V15326_4_1235, &P15326_5};	/*Property = ScreenPixelsWidth*/
const Profile P15326 = {"15326", &P15326_4};	/*Profile ID - 15326*/
const Value V15324_1_1 = {S2, NULL};	/*Value = True*/
const Property P15324_1 = {S50, 0, &V15324_1_1, NULL};	/*Property = IsMobile*/
const Value V15324_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15324_5 = {S52, 0, &V15324_5_1233, &P15324_1};	/*Property = ScreenPixelsHeight*/
const Value V15324_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15324_4 = {S54, 0, &V15324_4_1235, &P15324_5};	/*Property = ScreenPixelsWidth*/
const Profile P15324 = {"15324", &P15324_4};	/*Profile ID - 15324*/
const Value V16253_1_1 = {S2, NULL};	/*Value = True*/
const Property P16253_1 = {S50, 0, &V16253_1_1, NULL};	/*Property = IsMobile*/
const Value V16253_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16253_5 = {S52, 0, &V16253_5_1222, &P16253_1};	/*Property = ScreenPixelsHeight*/
const Value V16253_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16253_4 = {S54, 0, &V16253_4_1224, &P16253_5};	/*Property = ScreenPixelsWidth*/
const Profile P16253 = {"16253", &P16253_4};	/*Profile ID - 16253*/
const Value V15328_1_1 = {S2, NULL};	/*Value = True*/
const Property P15328_1 = {S50, 0, &V15328_1_1, NULL};	/*Property = IsMobile*/
const Value V15328_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15328_5 = {S52, 0, &V15328_5_1222, &P15328_1};	/*Property = ScreenPixelsHeight*/
const Value V15328_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15328_4 = {S54, 0, &V15328_4_1224, &P15328_5};	/*Property = ScreenPixelsWidth*/
const Profile P15328 = {"15328", &P15328_4};	/*Profile ID - 15328*/
const Value V15330_1_1 = {S2, NULL};	/*Value = True*/
const Property P15330_1 = {S50, 0, &V15330_1_1, NULL};	/*Property = IsMobile*/
const Value V15330_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15330_5 = {S52, 0, &V15330_5_1233, &P15330_1};	/*Property = ScreenPixelsHeight*/
const Value V15330_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15330_4 = {S54, 0, &V15330_4_1235, &P15330_5};	/*Property = ScreenPixelsWidth*/
const Profile P15330 = {"15330", &P15330_4};	/*Profile ID - 15330*/
const Value V15333_1_1 = {S2, NULL};	/*Value = True*/
const Property P15333_1 = {S50, 0, &V15333_1_1, NULL};	/*Property = IsMobile*/
const Value V15333_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15333_5 = {S52, 0, &V15333_5_1222, &P15333_1};	/*Property = ScreenPixelsHeight*/
const Value V15333_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15333_4 = {S54, 0, &V15333_4_1224, &P15333_5};	/*Property = ScreenPixelsWidth*/
const Profile P15333 = {"15333", &P15333_4};	/*Profile ID - 15333*/
const Value V15334_1_1 = {S2, NULL};	/*Value = True*/
const Property P15334_1 = {S50, 0, &V15334_1_1, NULL};	/*Property = IsMobile*/
const Value V15334_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15334_5 = {S52, 0, &V15334_5_1233, &P15334_1};	/*Property = ScreenPixelsHeight*/
const Value V15334_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15334_4 = {S54, 0, &V15334_4_1235, &P15334_5};	/*Property = ScreenPixelsWidth*/
const Profile P15334 = {"15334", &P15334_4};	/*Profile ID - 15334*/
const Value V15336_1_1 = {S2, NULL};	/*Value = True*/
const Property P15336_1 = {S50, 0, &V15336_1_1, NULL};	/*Property = IsMobile*/
const Value V15336_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15336_5 = {S52, 0, &V15336_5_1233, &P15336_1};	/*Property = ScreenPixelsHeight*/
const Value V15336_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15336_4 = {S54, 0, &V15336_4_1235, &P15336_5};	/*Property = ScreenPixelsWidth*/
const Profile P15336 = {"15336", &P15336_4};	/*Profile ID - 15336*/
const Value V15337_1_1 = {S2, NULL};	/*Value = True*/
const Property P15337_1 = {S50, 0, &V15337_1_1, NULL};	/*Property = IsMobile*/
const Value V15337_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15337_5 = {S52, 0, &V15337_5_1233, &P15337_1};	/*Property = ScreenPixelsHeight*/
const Value V15337_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15337_4 = {S54, 0, &V15337_4_1235, &P15337_5};	/*Property = ScreenPixelsWidth*/
const Profile P15337 = {"15337", &P15337_4};	/*Profile ID - 15337*/
const Value V15339_1_1 = {S2, NULL};	/*Value = True*/
const Property P15339_1 = {S50, 0, &V15339_1_1, NULL};	/*Property = IsMobile*/
const Value V15339_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15339_5 = {S52, 0, &V15339_5_1294, &P15339_1};	/*Property = ScreenPixelsHeight*/
const Value V15339_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15339_4 = {S54, 0, &V15339_4_1296, &P15339_5};	/*Property = ScreenPixelsWidth*/
const Profile P15339 = {"15339", &P15339_4};	/*Profile ID - 15339*/
const Value V17932_1_1 = {S2, NULL};	/*Value = True*/
const Property P17932_1 = {S50, 0, &V17932_1_1, NULL};	/*Property = IsMobile*/
const Value V17932_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17932_5 = {S52, 0, &V17932_5_1222, &P17932_1};	/*Property = ScreenPixelsHeight*/
const Value V17932_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17932_4 = {S54, 0, &V17932_4_1224, &P17932_5};	/*Property = ScreenPixelsWidth*/
const Profile P17932 = {"17932", &P17932_4};	/*Profile ID - 17932*/
const Value V11998_1_1 = {S2, NULL};	/*Value = True*/
const Property P11998_1 = {S50, 0, &V11998_1_1, NULL};	/*Property = IsMobile*/
const Value V11998_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11998_5 = {S52, 0, &V11998_5_1233, &P11998_1};	/*Property = ScreenPixelsHeight*/
const Value V11998_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11998_4 = {S54, 0, &V11998_4_1235, &P11998_5};	/*Property = ScreenPixelsWidth*/
const Profile P11998 = {"11998", &P11998_4};	/*Profile ID - 11998*/
const Value V12003_1_1 = {S2, NULL};	/*Value = True*/
const Property P12003_1 = {S50, 0, &V12003_1_1, NULL};	/*Property = IsMobile*/
const Value V12003_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12003_5 = {S52, 0, &V12003_5_1293, &P12003_1};	/*Property = ScreenPixelsHeight*/
const Value V12003_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12003_4 = {S54, 0, &V12003_4_1296, &P12003_5};	/*Property = ScreenPixelsWidth*/
const Profile P12003 = {"12003", &P12003_4};	/*Profile ID - 12003*/
const Value V4977_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_279 = {S1, 0, &V4977_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4977_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_282 = {S3, 0, &V4977_282_57155, &P4977_279};	/*Property = BlobBuilder*/
const Value V4977_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_350 = {S4, 0, &V4977_350_57446, &P4977_282};	/*Property = Canvas*/
const Value V4977_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_283 = {S5, 0, &V4977_283_57161, &P4977_350};	/*Property = CssBackground*/
const Value V4977_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_284 = {S6, 0, &V4977_284_57164, &P4977_283};	/*Property = CssBorderImage*/
const Value V4977_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_285 = {S7, 0, &V4977_285_57167, &P4977_284};	/*Property = CssCanvas*/
const Value V4977_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_286 = {S8, 0, &V4977_286_57170, &P4977_285};	/*Property = CssColor*/
const Value V4977_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_237 = {S9, 0, &V4977_237_55786, &P4977_286};	/*Property = CssColumn*/
const Value V4977_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_287 = {S10, 0, &V4977_287_57173, &P4977_237};	/*Property = CssFlexbox*/
const Value V4977_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_288 = {S11, 0, &V4977_288_57176, &P4977_287};	/*Property = CssFont*/
const Value V4977_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_289 = {S12, 0, &V4977_289_86899, &P4977_288};	/*Property = CssImages*/
const Value V4977_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_291 = {S13, 0, &V4977_291_57182, &P4977_289};	/*Property = CssMediaQueries*/
const Value V4977_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_292 = {S14, 0, &V4977_292_57185, &P4977_291};	/*Property = CssMinMax*/
const Value V4977_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_293 = {S15, 0, &V4977_293_57188, &P4977_292};	/*Property = CssOverflow*/
const Value V4977_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_294 = {S16, 0, &V4977_294_57191, &P4977_293};	/*Property = CssPosition*/
const Value V4977_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_295 = {S17, 0, &V4977_295_57194, &P4977_294};	/*Property = CssText*/
const Value V4977_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_297 = {S18, 0, &V4977_297_57197, &P4977_295};	/*Property = CssTransforms*/
const Value V4977_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_241 = {S19, 0, &V4977_241_55801, &P4977_297};	/*Property = CssTransitions*/
const Value V4977_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_298 = {S20, 0, &V4977_298_57208, &P4977_241};	/*Property = CssUI*/
const Value V4977_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_300 = {S21, 0, &V4977_300_57217, &P4977_298};	/*Property = DataSet*/
const Value V4977_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_301 = {S22, 0, &V4977_301_57220, &P4977_300};	/*Property = DataUrl*/
const Value V4977_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_302 = {S23, 0, &V4977_302_57223, &P4977_301};	/*Property = DeviceOrientation*/
const Value V4977_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_304 = {S24, 0, &V4977_304_57229, &P4977_302};	/*Property = FileReader*/
const Value V4977_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_305 = {S25, 0, &V4977_305_57232, &P4977_304};	/*Property = FileSaver*/
const Value V4977_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_306 = {S26, 0, &V4977_306_57238, &P4977_305};	/*Property = FileWriter*/
const Value V4977_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_307 = {S27, 0, &V4977_307_57241, &P4977_306};	/*Property = FormData*/
const Value V4977_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_309 = {S28, 0, &V4977_309_57247, &P4977_307};	/*Property = Fullscreen*/
const Value V4977_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_310 = {S29, 0, &V4977_310_57250, &P4977_309};	/*Property = GeoLocation*/
const Value V4977_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_312 = {S30, 0, &V4977_312_57256, &P4977_310};	/*Property = History*/
const Value V4977_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_314 = {S31, 0, &V4977_314_57262, &P4977_312};	/*Property = Html5*/
const Value V4977_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_313 = {S32, 0, &V4977_313_57259, &P4977_314};	/*Property = Html-Media-Capture*/
const Value V4977_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_315 = {S33, 0, &V4977_315_57265, &P4977_313};	/*Property = Iframe*/
const Value V4977_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_316 = {S34, 0, &V4977_316_57268, &P4977_315};	/*Property = IndexedDB*/
const Value V4977_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_317 = {S35, 0, &V4977_317_57271, &P4977_316};	/*Property = Json*/
const Value V4977_15_43263 = {S65, NULL};	/*Value = Netfront*/
const Property P4977_15 = {S37, 0, &V4977_15_43263, &P4977_317};	/*Property = LayoutEngine*/
const Value V4977_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_344 = {S38, 0, &V4977_344_57274, &P4977_15};	/*Property = Masking*/
const Value V4977_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_320 = {S39, 0, &V4977_320_57292, &P4977_344};	/*Property = PostMessage*/
const Value V4977_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_321 = {S40, 0, &V4977_321_57295, &P4977_320};	/*Property = Progress*/
const Value V4977_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_322 = {S41, 0, &V4977_322_57298, &P4977_321};	/*Property = Prompts*/
const Value V4977_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_323 = {S42, 0, &V4977_323_57301, &P4977_322};	/*Property = Selector*/
const Value V4977_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_324 = {S43, 0, &V4977_324_57307, &P4977_323};	/*Property = Svg*/
const Value V4977_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_327 = {S44, 0, &V4977_327_57316, &P4977_324};	/*Property = TouchEvents*/
const Value V4977_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_328 = {S45, 0, &V4977_328_57319, &P4977_327};	/*Property = Track*/
const Value V4977_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_329 = {S46, 0, &V4977_329_57325, &P4977_328};	/*Property = Video*/
const Value V4977_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_330 = {S47, 0, &V4977_330_57328, &P4977_329};	/*Property = Viewport*/
const Value V4977_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_332 = {S48, 0, &V4977_332_57340, &P4977_330};	/*Property = WebWorkers*/
const Value V4977_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4977_333 = {S49, 0, &V4977_333_57343, &P4977_332};	/*Property = Xhr2*/
const Profile P4977 = {"4977", &P4977_333};	/*Profile ID - 4977*/
const Value V15342_1_1 = {S2, NULL};	/*Value = True*/
const Property P15342_1 = {S50, 0, &V15342_1_1, NULL};	/*Property = IsMobile*/
const Value V15342_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15342_5 = {S52, 0, &V15342_5_1233, &P15342_1};	/*Property = ScreenPixelsHeight*/
const Value V15342_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15342_4 = {S54, 0, &V15342_4_1235, &P15342_5};	/*Property = ScreenPixelsWidth*/
const Profile P15342 = {"15342", &P15342_4};	/*Profile ID - 15342*/
const Value V15343_1_1 = {S2, NULL};	/*Value = True*/
const Property P15343_1 = {S50, 0, &V15343_1_1, NULL};	/*Property = IsMobile*/
const Value V15343_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15343_5 = {S52, 0, &V15343_5_1222, &P15343_1};	/*Property = ScreenPixelsHeight*/
const Value V15343_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15343_4 = {S54, 0, &V15343_4_1224, &P15343_5};	/*Property = ScreenPixelsWidth*/
const Profile P15343 = {"15343", &P15343_4};	/*Profile ID - 15343*/
const Value V12871_1_1 = {S2, NULL};	/*Value = True*/
const Property P12871_1 = {S50, 0, &V12871_1_1, NULL};	/*Property = IsMobile*/
const Value V12871_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12871_5 = {S52, 0, &V12871_5_1277, &P12871_1};	/*Property = ScreenPixelsHeight*/
const Value V12871_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12871_4 = {S54, 0, &V12871_4_1279, &P12871_5};	/*Property = ScreenPixelsWidth*/
const Profile P12871 = {"12871", &P12871_4};	/*Profile ID - 12871*/
const Value V15347_1_1 = {S2, NULL};	/*Value = True*/
const Property P15347_1 = {S50, 0, &V15347_1_1, NULL};	/*Property = IsMobile*/
const Value V15347_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15347_5 = {S52, 0, &V15347_5_1233, &P15347_1};	/*Property = ScreenPixelsHeight*/
const Value V15347_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15347_4 = {S54, 0, &V15347_4_1235, &P15347_5};	/*Property = ScreenPixelsWidth*/
const Profile P15347 = {"15347", &P15347_4};	/*Profile ID - 15347*/
const Value V15348_1_1 = {S2, NULL};	/*Value = True*/
const Property P15348_1 = {S50, 0, &V15348_1_1, NULL};	/*Property = IsMobile*/
const Value V15348_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15348_5 = {S52, 0, &V15348_5_1222, &P15348_1};	/*Property = ScreenPixelsHeight*/
const Value V15348_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15348_4 = {S54, 0, &V15348_4_1224, &P15348_5};	/*Property = ScreenPixelsWidth*/
const Profile P15348 = {"15348", &P15348_4};	/*Profile ID - 15348*/
const Value V15351_1_1 = {S2, NULL};	/*Value = True*/
const Property P15351_1 = {S50, 0, &V15351_1_1, NULL};	/*Property = IsMobile*/
const Value V15351_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15351_5 = {S52, 0, &V15351_5_1222, &P15351_1};	/*Property = ScreenPixelsHeight*/
const Value V15351_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15351_4 = {S54, 0, &V15351_4_1224, &P15351_5};	/*Property = ScreenPixelsWidth*/
const Profile P15351 = {"15351", &P15351_4};	/*Profile ID - 15351*/
const Profile P17813 = {"17813", NULL};	/*Profile ID - 17813*/
const Value V12013_1_1 = {S2, NULL};	/*Value = True*/
const Property P12013_1 = {S50, 0, &V12013_1_1, NULL};	/*Property = IsMobile*/
const Value V12013_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12013_5 = {S52, 0, &V12013_5_1380, &P12013_1};	/*Property = ScreenPixelsHeight*/
const Value V12013_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12013_4 = {S54, 0, &V12013_4_1279, &P12013_5};	/*Property = ScreenPixelsWidth*/
const Profile P12013 = {"12013", &P12013_4};	/*Profile ID - 12013*/
const Value V14317_1_1 = {S2, NULL};	/*Value = True*/
const Property P14317_1 = {S50, 0, &V14317_1_1, NULL};	/*Property = IsMobile*/
const Value V14317_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14317_5 = {S52, 0, &V14317_5_1222, &P14317_1};	/*Property = ScreenPixelsHeight*/
const Value V14317_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14317_4 = {S54, 0, &V14317_4_1224, &P14317_5};	/*Property = ScreenPixelsWidth*/
const Profile P14317 = {"14317", &P14317_4};	/*Profile ID - 14317*/
const Value V15352_1_1 = {S2, NULL};	/*Value = True*/
const Property P15352_1 = {S50, 0, &V15352_1_1, NULL};	/*Property = IsMobile*/
const Value V15352_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15352_5 = {S52, 0, &V15352_5_1233, &P15352_1};	/*Property = ScreenPixelsHeight*/
const Value V15352_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15352_4 = {S54, 0, &V15352_4_1235, &P15352_5};	/*Property = ScreenPixelsWidth*/
const Profile P15352 = {"15352", &P15352_4};	/*Profile ID - 15352*/
const Value V15353_1_1 = {S2, NULL};	/*Value = True*/
const Property P15353_1 = {S50, 0, &V15353_1_1, NULL};	/*Property = IsMobile*/
const Value V15353_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15353_5 = {S52, 0, &V15353_5_1222, &P15353_1};	/*Property = ScreenPixelsHeight*/
const Value V15353_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15353_4 = {S54, 0, &V15353_4_1224, &P15353_5};	/*Property = ScreenPixelsWidth*/
const Profile P15353 = {"15353", &P15353_4};	/*Profile ID - 15353*/
const Value V15355_1_1 = {S2, NULL};	/*Value = True*/
const Property P15355_1 = {S50, 0, &V15355_1_1, NULL};	/*Property = IsMobile*/
const Value V15355_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15355_5 = {S52, 0, &V15355_5_1294, &P15355_1};	/*Property = ScreenPixelsHeight*/
const Value V15355_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15355_4 = {S54, 0, &V15355_4_1296, &P15355_5};	/*Property = ScreenPixelsWidth*/
const Profile P15355 = {"15355", &P15355_4};	/*Profile ID - 15355*/
const Value V14313_1_1 = {S2, NULL};	/*Value = True*/
const Property P14313_1 = {S50, 0, &V14313_1_1, NULL};	/*Property = IsMobile*/
const Value V14313_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14313_5 = {S52, 0, &V14313_5_1293, &P14313_1};	/*Property = ScreenPixelsHeight*/
const Value V14313_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14313_4 = {S54, 0, &V14313_4_1296, &P14313_5};	/*Property = ScreenPixelsWidth*/
const Profile P14313 = {"14313", &P14313_4};	/*Profile ID - 14313*/
const Value V11911_1_1 = {S2, NULL};	/*Value = True*/
const Property P11911_1 = {S50, 0, &V11911_1_1, NULL};	/*Property = IsMobile*/
const Value V11911_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11911_5 = {S52, 0, &V11911_5_1293, &P11911_1};	/*Property = ScreenPixelsHeight*/
const Value V11911_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11911_4 = {S54, 0, &V11911_4_1296, &P11911_5};	/*Property = ScreenPixelsWidth*/
const Profile P11911 = {"11911", &P11911_4};	/*Profile ID - 11911*/
const Value V12014_1_1 = {S2, NULL};	/*Value = True*/
const Property P12014_1 = {S50, 0, &V12014_1_1, NULL};	/*Property = IsMobile*/
const Value V12014_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12014_5 = {S52, 0, &V12014_5_1233, &P12014_1};	/*Property = ScreenPixelsHeight*/
const Value V12014_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12014_4 = {S54, 0, &V12014_4_1235, &P12014_5};	/*Property = ScreenPixelsWidth*/
const Profile P12014 = {"12014", &P12014_4};	/*Profile ID - 12014*/
const Value V12015_1_1 = {S2, NULL};	/*Value = True*/
const Property P12015_1 = {S50, 0, &V12015_1_1, NULL};	/*Property = IsMobile*/
const Value V12015_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12015_5 = {S52, 0, &V12015_5_1222, &P12015_1};	/*Property = ScreenPixelsHeight*/
const Value V12015_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12015_4 = {S54, 0, &V12015_4_1224, &P12015_5};	/*Property = ScreenPixelsWidth*/
const Profile P12015 = {"12015", &P12015_4};	/*Profile ID - 12015*/
const Value V12016_1_1 = {S2, NULL};	/*Value = True*/
const Property P12016_1 = {S50, 0, &V12016_1_1, NULL};	/*Property = IsMobile*/
const Value V12016_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12016_5 = {S52, 0, &V12016_5_1222, &P12016_1};	/*Property = ScreenPixelsHeight*/
const Value V12016_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12016_4 = {S54, 0, &V12016_4_1224, &P12016_5};	/*Property = ScreenPixelsWidth*/
const Profile P12016 = {"12016", &P12016_4};	/*Profile ID - 12016*/
const Value V12021_1_1 = {S2, NULL};	/*Value = True*/
const Property P12021_1 = {S50, 0, &V12021_1_1, NULL};	/*Property = IsMobile*/
const Value V12021_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12021_5 = {S52, 0, &V12021_5_1222, &P12021_1};	/*Property = ScreenPixelsHeight*/
const Value V12021_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12021_4 = {S54, 0, &V12021_4_1224, &P12021_5};	/*Property = ScreenPixelsWidth*/
const Profile P12021 = {"12021", &P12021_4};	/*Profile ID - 12021*/
const Value V12031_1_1 = {S2, NULL};	/*Value = True*/
const Property P12031_1 = {S50, 0, &V12031_1_1, NULL};	/*Property = IsMobile*/
const Value V12031_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12031_5 = {S52, 0, &V12031_5_1222, &P12031_1};	/*Property = ScreenPixelsHeight*/
const Value V12031_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12031_4 = {S54, 0, &V12031_4_1224, &P12031_5};	/*Property = ScreenPixelsWidth*/
const Profile P12031 = {"12031", &P12031_4};	/*Profile ID - 12031*/
const Value V12548_1_1 = {S2, NULL};	/*Value = True*/
const Property P12548_1 = {S50, 0, &V12548_1_1, NULL};	/*Property = IsMobile*/
const Value V12548_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12548_5 = {S52, 0, &V12548_5_1380, &P12548_1};	/*Property = ScreenPixelsHeight*/
const Value V12548_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12548_4 = {S54, 0, &V12548_4_1279, &P12548_5};	/*Property = ScreenPixelsWidth*/
const Profile P12548 = {"12548", &P12548_4};	/*Profile ID - 12548*/
const Value V15105_1_1 = {S2, NULL};	/*Value = True*/
const Property P15105_1 = {S50, 0, &V15105_1_1, NULL};	/*Property = IsMobile*/
const Value V15105_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P15105_5 = {S52, 0, &V15105_5_1333, &P15105_1};	/*Property = ScreenPixelsHeight*/
const Value V15105_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P15105_4 = {S54, 0, &V15105_4_1335, &P15105_5};	/*Property = ScreenPixelsWidth*/
const Profile P15105 = {"15105", &P15105_4};	/*Profile ID - 15105*/
const Value V18246_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_279 = {S1, 0, &V18246_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18246_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_282 = {S3, 0, &V18246_282_57155, &P18246_279};	/*Property = BlobBuilder*/
const Value V18246_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_350 = {S4, 0, &V18246_350_57446, &P18246_282};	/*Property = Canvas*/
const Value V18246_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_283 = {S5, 0, &V18246_283_57161, &P18246_350};	/*Property = CssBackground*/
const Value V18246_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_284 = {S6, 0, &V18246_284_57164, &P18246_283};	/*Property = CssBorderImage*/
const Value V18246_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_285 = {S7, 0, &V18246_285_57167, &P18246_284};	/*Property = CssCanvas*/
const Value V18246_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_286 = {S8, 0, &V18246_286_57170, &P18246_285};	/*Property = CssColor*/
const Value V18246_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_237 = {S9, 0, &V18246_237_55786, &P18246_286};	/*Property = CssColumn*/
const Value V18246_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_287 = {S10, 0, &V18246_287_57173, &P18246_237};	/*Property = CssFlexbox*/
const Value V18246_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_288 = {S11, 0, &V18246_288_57176, &P18246_287};	/*Property = CssFont*/
const Value V18246_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_289 = {S12, 0, &V18246_289_86899, &P18246_288};	/*Property = CssImages*/
const Value V18246_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_291 = {S13, 0, &V18246_291_57182, &P18246_289};	/*Property = CssMediaQueries*/
const Value V18246_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_292 = {S14, 0, &V18246_292_57185, &P18246_291};	/*Property = CssMinMax*/
const Value V18246_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_293 = {S15, 0, &V18246_293_57188, &P18246_292};	/*Property = CssOverflow*/
const Value V18246_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_294 = {S16, 0, &V18246_294_57191, &P18246_293};	/*Property = CssPosition*/
const Value V18246_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_295 = {S17, 0, &V18246_295_57194, &P18246_294};	/*Property = CssText*/
const Value V18246_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_297 = {S18, 0, &V18246_297_57197, &P18246_295};	/*Property = CssTransforms*/
const Value V18246_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_241 = {S19, 0, &V18246_241_55801, &P18246_297};	/*Property = CssTransitions*/
const Value V18246_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_298 = {S20, 0, &V18246_298_57208, &P18246_241};	/*Property = CssUI*/
const Value V18246_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_300 = {S21, 0, &V18246_300_57217, &P18246_298};	/*Property = DataSet*/
const Value V18246_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_301 = {S22, 0, &V18246_301_57220, &P18246_300};	/*Property = DataUrl*/
const Value V18246_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_302 = {S23, 0, &V18246_302_57223, &P18246_301};	/*Property = DeviceOrientation*/
const Value V18246_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_304 = {S24, 0, &V18246_304_57229, &P18246_302};	/*Property = FileReader*/
const Value V18246_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_305 = {S25, 0, &V18246_305_57232, &P18246_304};	/*Property = FileSaver*/
const Value V18246_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_306 = {S26, 0, &V18246_306_57238, &P18246_305};	/*Property = FileWriter*/
const Value V18246_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_307 = {S27, 0, &V18246_307_57241, &P18246_306};	/*Property = FormData*/
const Value V18246_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_309 = {S28, 0, &V18246_309_57247, &P18246_307};	/*Property = Fullscreen*/
const Value V18246_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_310 = {S29, 0, &V18246_310_57250, &P18246_309};	/*Property = GeoLocation*/
const Value V18246_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_312 = {S30, 0, &V18246_312_57256, &P18246_310};	/*Property = History*/
const Value V18246_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_314 = {S31, 0, &V18246_314_57262, &P18246_312};	/*Property = Html5*/
const Value V18246_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_313 = {S32, 0, &V18246_313_57259, &P18246_314};	/*Property = Html-Media-Capture*/
const Value V18246_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_315 = {S33, 0, &V18246_315_57265, &P18246_313};	/*Property = Iframe*/
const Value V18246_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_316 = {S34, 0, &V18246_316_57268, &P18246_315};	/*Property = IndexedDB*/
const Value V18246_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_317 = {S35, 0, &V18246_317_57271, &P18246_316};	/*Property = Json*/
const Value V18246_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_15 = {S37, 0, &V18246_15_72, &P18246_317};	/*Property = LayoutEngine*/
const Value V18246_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_344 = {S38, 0, &V18246_344_57274, &P18246_15};	/*Property = Masking*/
const Value V18246_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_320 = {S39, 0, &V18246_320_57292, &P18246_344};	/*Property = PostMessage*/
const Value V18246_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_321 = {S40, 0, &V18246_321_57295, &P18246_320};	/*Property = Progress*/
const Value V18246_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_322 = {S41, 0, &V18246_322_57298, &P18246_321};	/*Property = Prompts*/
const Value V18246_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_323 = {S42, 0, &V18246_323_57301, &P18246_322};	/*Property = Selector*/
const Value V18246_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_324 = {S43, 0, &V18246_324_57307, &P18246_323};	/*Property = Svg*/
const Value V18246_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_327 = {S44, 0, &V18246_327_57316, &P18246_324};	/*Property = TouchEvents*/
const Value V18246_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_328 = {S45, 0, &V18246_328_57319, &P18246_327};	/*Property = Track*/
const Value V18246_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_329 = {S46, 0, &V18246_329_57325, &P18246_328};	/*Property = Video*/
const Value V18246_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_330 = {S47, 0, &V18246_330_57328, &P18246_329};	/*Property = Viewport*/
const Value V18246_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_332 = {S48, 0, &V18246_332_57340, &P18246_330};	/*Property = WebWorkers*/
const Value V18246_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18246_333 = {S49, 0, &V18246_333_57343, &P18246_332};	/*Property = Xhr2*/
const Profile P18246 = {"18246", &P18246_333};	/*Profile ID - 18246*/
const Value V14308_1_1 = {S2, NULL};	/*Value = True*/
const Property P14308_1 = {S50, 0, &V14308_1_1, NULL};	/*Property = IsMobile*/
const Value V14308_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14308_5 = {S52, 0, &V14308_5_1222, &P14308_1};	/*Property = ScreenPixelsHeight*/
const Value V14308_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14308_4 = {S54, 0, &V14308_4_1224, &P14308_5};	/*Property = ScreenPixelsWidth*/
const Profile P14308 = {"14308", &P14308_4};	/*Profile ID - 14308*/
const Value V15560_1_1 = {S2, NULL};	/*Value = True*/
const Property P15560_1 = {S50, 0, &V15560_1_1, NULL};	/*Property = IsMobile*/
const Value V15560_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15560_5 = {S52, 0, &V15560_5_1233, &P15560_1};	/*Property = ScreenPixelsHeight*/
const Value V15560_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15560_4 = {S54, 0, &V15560_4_1235, &P15560_5};	/*Property = ScreenPixelsWidth*/
const Profile P15560 = {"15560", &P15560_4};	/*Profile ID - 15560*/
const Value V11672_1_1 = {S2, NULL};	/*Value = True*/
const Property P11672_1 = {S50, 0, &V11672_1_1, NULL};	/*Property = IsMobile*/
const Value V11672_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P11672_5 = {S52, 0, &V11672_5_1222, &P11672_1};	/*Property = ScreenPixelsHeight*/
const Value V11672_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11672_4 = {S54, 0, &V11672_4_1224, &P11672_5};	/*Property = ScreenPixelsWidth*/
const Profile P11672 = {"11672", &P11672_4};	/*Profile ID - 11672*/
const Value V15380_1_1 = {S2, NULL};	/*Value = True*/
const Property P15380_1 = {S50, 0, &V15380_1_1, NULL};	/*Property = IsMobile*/
const Value V15380_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15380_5 = {S52, 0, &V15380_5_1233, &P15380_1};	/*Property = ScreenPixelsHeight*/
const Value V15380_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15380_4 = {S54, 0, &V15380_4_1235, &P15380_5};	/*Property = ScreenPixelsWidth*/
const Profile P15380 = {"15380", &P15380_4};	/*Profile ID - 15380*/
const Value V14527_1_1 = {S2, NULL};	/*Value = True*/
const Property P14527_1 = {S50, 0, &V14527_1_1, NULL};	/*Property = IsMobile*/
const Value V14527_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14527_5 = {S52, 0, &V14527_5_1294, &P14527_1};	/*Property = ScreenPixelsHeight*/
const Value V14527_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14527_4 = {S54, 0, &V14527_4_1296, &P14527_5};	/*Property = ScreenPixelsWidth*/
const Profile P14527 = {"14527", &P14527_4};	/*Profile ID - 14527*/
const Value V15381_1_1 = {S2, NULL};	/*Value = True*/
const Property P15381_1 = {S50, 0, &V15381_1_1, NULL};	/*Property = IsMobile*/
const Value V15381_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15381_5 = {S52, 0, &V15381_5_1293, &P15381_1};	/*Property = ScreenPixelsHeight*/
const Value V15381_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15381_4 = {S54, 0, &V15381_4_1296, &P15381_5};	/*Property = ScreenPixelsWidth*/
const Profile P15381 = {"15381", &P15381_4};	/*Profile ID - 15381*/
const Value V15383_1_1 = {S2, NULL};	/*Value = True*/
const Property P15383_1 = {S50, 0, &V15383_1_1, NULL};	/*Property = IsMobile*/
const Value V15383_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15383_5 = {S52, 0, &V15383_5_1293, &P15383_1};	/*Property = ScreenPixelsHeight*/
const Value V15383_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15383_4 = {S54, 0, &V15383_4_1296, &P15383_5};	/*Property = ScreenPixelsWidth*/
const Profile P15383 = {"15383", &P15383_4};	/*Profile ID - 15383*/
const Value V17104_1_1 = {S2, NULL};	/*Value = True*/
const Property P17104_1 = {S50, 0, &V17104_1_1, NULL};	/*Property = IsMobile*/
const Value V17104_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17104_5 = {S52, 0, &V17104_5_1222, &P17104_1};	/*Property = ScreenPixelsHeight*/
const Value V17104_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17104_4 = {S54, 0, &V17104_4_1224, &P17104_5};	/*Property = ScreenPixelsWidth*/
const Profile P17104 = {"17104", &P17104_4};	/*Profile ID - 17104*/
const Value V15384_1_1 = {S2, NULL};	/*Value = True*/
const Property P15384_1 = {S50, 0, &V15384_1_1, NULL};	/*Property = IsMobile*/
const Value V15384_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P15384_5 = {S52, 0, &V15384_5_1269, &P15384_1};	/*Property = ScreenPixelsHeight*/
const Value V15384_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15384_4 = {S54, 0, &V15384_4_1229, &P15384_5};	/*Property = ScreenPixelsWidth*/
const Profile P15384 = {"15384", &P15384_4};	/*Profile ID - 15384*/
const Profile P6704 = {"6704", NULL};	/*Profile ID - 6704*/
const Value V17387_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_279 = {S1, 0, &V17387_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17387_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_282 = {S3, 0, &V17387_282_57155, &P17387_279};	/*Property = BlobBuilder*/
const Value V17387_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_350 = {S4, 0, &V17387_350_57446, &P17387_282};	/*Property = Canvas*/
const Value V17387_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_283 = {S5, 0, &V17387_283_57161, &P17387_350};	/*Property = CssBackground*/
const Value V17387_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_284 = {S6, 0, &V17387_284_57164, &P17387_283};	/*Property = CssBorderImage*/
const Value V17387_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_285 = {S7, 0, &V17387_285_57167, &P17387_284};	/*Property = CssCanvas*/
const Value V17387_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_286 = {S8, 0, &V17387_286_57170, &P17387_285};	/*Property = CssColor*/
const Value V17387_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_237 = {S9, 0, &V17387_237_55786, &P17387_286};	/*Property = CssColumn*/
const Value V17387_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_287 = {S10, 0, &V17387_287_57173, &P17387_237};	/*Property = CssFlexbox*/
const Value V17387_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_288 = {S11, 0, &V17387_288_57176, &P17387_287};	/*Property = CssFont*/
const Value V17387_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_289 = {S12, 0, &V17387_289_86899, &P17387_288};	/*Property = CssImages*/
const Value V17387_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_291 = {S13, 0, &V17387_291_57182, &P17387_289};	/*Property = CssMediaQueries*/
const Value V17387_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_292 = {S14, 0, &V17387_292_57185, &P17387_291};	/*Property = CssMinMax*/
const Value V17387_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_293 = {S15, 0, &V17387_293_57188, &P17387_292};	/*Property = CssOverflow*/
const Value V17387_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_294 = {S16, 0, &V17387_294_57191, &P17387_293};	/*Property = CssPosition*/
const Value V17387_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_295 = {S17, 0, &V17387_295_57194, &P17387_294};	/*Property = CssText*/
const Value V17387_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_297 = {S18, 0, &V17387_297_57197, &P17387_295};	/*Property = CssTransforms*/
const Value V17387_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_241 = {S19, 0, &V17387_241_55801, &P17387_297};	/*Property = CssTransitions*/
const Value V17387_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_298 = {S20, 0, &V17387_298_57208, &P17387_241};	/*Property = CssUI*/
const Value V17387_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_300 = {S21, 0, &V17387_300_57217, &P17387_298};	/*Property = DataSet*/
const Value V17387_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_301 = {S22, 0, &V17387_301_57220, &P17387_300};	/*Property = DataUrl*/
const Value V17387_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_302 = {S23, 0, &V17387_302_57223, &P17387_301};	/*Property = DeviceOrientation*/
const Value V17387_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_304 = {S24, 0, &V17387_304_57229, &P17387_302};	/*Property = FileReader*/
const Value V17387_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_305 = {S25, 0, &V17387_305_57232, &P17387_304};	/*Property = FileSaver*/
const Value V17387_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_306 = {S26, 0, &V17387_306_57238, &P17387_305};	/*Property = FileWriter*/
const Value V17387_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_307 = {S27, 0, &V17387_307_57241, &P17387_306};	/*Property = FormData*/
const Value V17387_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_309 = {S28, 0, &V17387_309_57247, &P17387_307};	/*Property = Fullscreen*/
const Value V17387_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_310 = {S29, 0, &V17387_310_57250, &P17387_309};	/*Property = GeoLocation*/
const Value V17387_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_312 = {S30, 0, &V17387_312_57256, &P17387_310};	/*Property = History*/
const Value V17387_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_314 = {S31, 0, &V17387_314_57262, &P17387_312};	/*Property = Html5*/
const Value V17387_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_313 = {S32, 0, &V17387_313_57259, &P17387_314};	/*Property = Html-Media-Capture*/
const Value V17387_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_315 = {S33, 0, &V17387_315_57265, &P17387_313};	/*Property = Iframe*/
const Value V17387_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_316 = {S34, 0, &V17387_316_57268, &P17387_315};	/*Property = IndexedDB*/
const Value V17387_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_317 = {S35, 0, &V17387_317_57271, &P17387_316};	/*Property = Json*/
const Value V17387_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P17387_15 = {S37, 0, &V17387_15_30, &P17387_317};	/*Property = LayoutEngine*/
const Value V17387_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_344 = {S38, 0, &V17387_344_57274, &P17387_15};	/*Property = Masking*/
const Value V17387_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_320 = {S39, 0, &V17387_320_57292, &P17387_344};	/*Property = PostMessage*/
const Value V17387_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_321 = {S40, 0, &V17387_321_57295, &P17387_320};	/*Property = Progress*/
const Value V17387_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_322 = {S41, 0, &V17387_322_57298, &P17387_321};	/*Property = Prompts*/
const Value V17387_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_323 = {S42, 0, &V17387_323_57301, &P17387_322};	/*Property = Selector*/
const Value V17387_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_324 = {S43, 0, &V17387_324_57307, &P17387_323};	/*Property = Svg*/
const Value V17387_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_327 = {S44, 0, &V17387_327_57316, &P17387_324};	/*Property = TouchEvents*/
const Value V17387_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_328 = {S45, 0, &V17387_328_57319, &P17387_327};	/*Property = Track*/
const Value V17387_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_329 = {S46, 0, &V17387_329_57325, &P17387_328};	/*Property = Video*/
const Value V17387_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_330 = {S47, 0, &V17387_330_57328, &P17387_329};	/*Property = Viewport*/
const Value V17387_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_332 = {S48, 0, &V17387_332_57340, &P17387_330};	/*Property = WebWorkers*/
const Value V17387_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17387_333 = {S49, 0, &V17387_333_57343, &P17387_332};	/*Property = Xhr2*/
const Profile P17387 = {"17387", &P17387_333};	/*Profile ID - 17387*/
const Value V15389_1_1 = {S2, NULL};	/*Value = True*/
const Property P15389_1 = {S50, 0, &V15389_1_1, NULL};	/*Property = IsMobile*/
const Value V15389_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15389_5 = {S52, 0, &V15389_5_1227, &P15389_1};	/*Property = ScreenPixelsHeight*/
const Value V15389_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15389_4 = {S54, 0, &V15389_4_1229, &P15389_5};	/*Property = ScreenPixelsWidth*/
const Profile P15389 = {"15389", &P15389_4};	/*Profile ID - 15389*/
const Value V15390_1_1 = {S2, NULL};	/*Value = True*/
const Property P15390_1 = {S50, 0, &V15390_1_1, NULL};	/*Property = IsMobile*/
const Value V15390_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15390_5 = {S52, 0, &V15390_5_1227, &P15390_1};	/*Property = ScreenPixelsHeight*/
const Value V15390_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15390_4 = {S54, 0, &V15390_4_1229, &P15390_5};	/*Property = ScreenPixelsWidth*/
const Profile P15390 = {"15390", &P15390_4};	/*Profile ID - 15390*/
const Value V15391_1_1 = {S2, NULL};	/*Value = True*/
const Property P15391_1 = {S50, 0, &V15391_1_1, NULL};	/*Property = IsMobile*/
const Value V15391_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15391_5 = {S52, 0, &V15391_5_1222, &P15391_1};	/*Property = ScreenPixelsHeight*/
const Value V15391_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15391_4 = {S54, 0, &V15391_4_1224, &P15391_5};	/*Property = ScreenPixelsWidth*/
const Profile P15391 = {"15391", &P15391_4};	/*Profile ID - 15391*/
const Value V13855_1_1 = {S2, NULL};	/*Value = True*/
const Property P13855_1 = {S50, 0, &V13855_1_1, NULL};	/*Property = IsMobile*/
const Value V13855_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13855_5 = {S52, 0, &V13855_5_1294, &P13855_1};	/*Property = ScreenPixelsHeight*/
const Value V13855_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13855_4 = {S54, 0, &V13855_4_1296, &P13855_5};	/*Property = ScreenPixelsWidth*/
const Profile P13855 = {"13855", &P13855_4};	/*Profile ID - 13855*/
const Value V15392_1_1 = {S2, NULL};	/*Value = True*/
const Property P15392_1 = {S50, 0, &V15392_1_1, NULL};	/*Property = IsMobile*/
const Value V15392_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15392_5 = {S52, 0, &V15392_5_1233, &P15392_1};	/*Property = ScreenPixelsHeight*/
const Value V15392_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15392_4 = {S54, 0, &V15392_4_1235, &P15392_5};	/*Property = ScreenPixelsWidth*/
const Profile P15392 = {"15392", &P15392_4};	/*Profile ID - 15392*/
const Value V15393_1_1 = {S2, NULL};	/*Value = True*/
const Property P15393_1 = {S50, 0, &V15393_1_1, NULL};	/*Property = IsMobile*/
const Value V15393_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15393_5 = {S52, 0, &V15393_5_1233, &P15393_1};	/*Property = ScreenPixelsHeight*/
const Value V15393_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15393_4 = {S54, 0, &V15393_4_1235, &P15393_5};	/*Property = ScreenPixelsWidth*/
const Profile P15393 = {"15393", &P15393_4};	/*Profile ID - 15393*/
const Value V15394_1_1 = {S2, NULL};	/*Value = True*/
const Property P15394_1 = {S50, 0, &V15394_1_1, NULL};	/*Property = IsMobile*/
const Value V15394_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15394_5 = {S52, 0, &V15394_5_1222, &P15394_1};	/*Property = ScreenPixelsHeight*/
const Value V15394_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15394_4 = {S54, 0, &V15394_4_1224, &P15394_5};	/*Property = ScreenPixelsWidth*/
const Profile P15394 = {"15394", &P15394_4};	/*Profile ID - 15394*/
const Value V15395_1_1 = {S2, NULL};	/*Value = True*/
const Property P15395_1 = {S50, 0, &V15395_1_1, NULL};	/*Property = IsMobile*/
const Value V15395_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15395_5 = {S52, 0, &V15395_5_1294, &P15395_1};	/*Property = ScreenPixelsHeight*/
const Value V15395_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15395_4 = {S54, 0, &V15395_4_1296, &P15395_5};	/*Property = ScreenPixelsWidth*/
const Profile P15395 = {"15395", &P15395_4};	/*Profile ID - 15395*/
const Profile P5043 = {"5043", NULL};	/*Profile ID - 5043*/
const Value V15228_1_1 = {S2, NULL};	/*Value = True*/
const Property P15228_1 = {S50, 0, &V15228_1_1, NULL};	/*Property = IsMobile*/
const Value V15228_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15228_5 = {S52, 0, &V15228_5_1222, &P15228_1};	/*Property = ScreenPixelsHeight*/
const Value V15228_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15228_4 = {S54, 0, &V15228_4_1224, &P15228_5};	/*Property = ScreenPixelsWidth*/
const Profile P15228 = {"15228", &P15228_4};	/*Profile ID - 15228*/
const Value V15398_1_1 = {S2, NULL};	/*Value = True*/
const Property P15398_1 = {S50, 0, &V15398_1_1, NULL};	/*Property = IsMobile*/
const Value V15398_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15398_5 = {S52, 0, &V15398_5_1293, &P15398_1};	/*Property = ScreenPixelsHeight*/
const Value V15398_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15398_4 = {S54, 0, &V15398_4_1296, &P15398_5};	/*Property = ScreenPixelsWidth*/
const Profile P15398 = {"15398", &P15398_4};	/*Profile ID - 15398*/
const Value V15399_1_1 = {S2, NULL};	/*Value = True*/
const Property P15399_1 = {S50, 0, &V15399_1_1, NULL};	/*Property = IsMobile*/
const Value V15399_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15399_5 = {S52, 0, &V15399_5_1233, &P15399_1};	/*Property = ScreenPixelsHeight*/
const Value V15399_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15399_4 = {S54, 0, &V15399_4_1235, &P15399_5};	/*Property = ScreenPixelsWidth*/
const Profile P15399 = {"15399", &P15399_4};	/*Profile ID - 15399*/
const Value V15400_1_1 = {S2, NULL};	/*Value = True*/
const Property P15400_1 = {S50, 0, &V15400_1_1, NULL};	/*Property = IsMobile*/
const Value V15400_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P15400_5 = {S52, 0, &V15400_5_1333, &P15400_1};	/*Property = ScreenPixelsHeight*/
const Value V15400_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P15400_4 = {S54, 0, &V15400_4_1335, &P15400_5};	/*Property = ScreenPixelsWidth*/
const Profile P15400 = {"15400", &P15400_4};	/*Profile ID - 15400*/
const Value V15401_1_1 = {S2, NULL};	/*Value = True*/
const Property P15401_1 = {S50, 0, &V15401_1_1, NULL};	/*Property = IsMobile*/
const Value V15401_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15401_5 = {S52, 0, &V15401_5_1222, &P15401_1};	/*Property = ScreenPixelsHeight*/
const Value V15401_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15401_4 = {S54, 0, &V15401_4_1224, &P15401_5};	/*Property = ScreenPixelsWidth*/
const Profile P15401 = {"15401", &P15401_4};	/*Profile ID - 15401*/
const Value V15403_1_1 = {S2, NULL};	/*Value = True*/
const Property P15403_1 = {S50, 0, &V15403_1_1, NULL};	/*Property = IsMobile*/
const Value V15403_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15403_5 = {S52, 0, &V15403_5_1222, &P15403_1};	/*Property = ScreenPixelsHeight*/
const Value V15403_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15403_4 = {S54, 0, &V15403_4_1224, &P15403_5};	/*Property = ScreenPixelsWidth*/
const Profile P15403 = {"15403", &P15403_4};	/*Profile ID - 15403*/
const Value V15405_1_1 = {S2, NULL};	/*Value = True*/
const Property P15405_1 = {S50, 0, &V15405_1_1, NULL};	/*Property = IsMobile*/
const Value V15405_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15405_5 = {S52, 0, &V15405_5_1222, &P15405_1};	/*Property = ScreenPixelsHeight*/
const Value V15405_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15405_4 = {S54, 0, &V15405_4_1224, &P15405_5};	/*Property = ScreenPixelsWidth*/
const Profile P15405 = {"15405", &P15405_4};	/*Profile ID - 15405*/
const Value V15404_1_1 = {S2, NULL};	/*Value = True*/
const Property P15404_1 = {S50, 0, &V15404_1_1, NULL};	/*Property = IsMobile*/
const Value V15404_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P15404_5 = {S52, 0, &V15404_5_1403, &P15404_1};	/*Property = ScreenPixelsHeight*/
const Value V15404_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P15404_4 = {S54, 0, &V15404_4_1404, &P15404_5};	/*Property = ScreenPixelsWidth*/
const Profile P15404 = {"15404", &P15404_4};	/*Profile ID - 15404*/
const Value V14219_1_1 = {S2, NULL};	/*Value = True*/
const Property P14219_1 = {S50, 0, &V14219_1_1, NULL};	/*Property = IsMobile*/
const Value V14219_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14219_5 = {S52, 0, &V14219_5_1222, &P14219_1};	/*Property = ScreenPixelsHeight*/
const Value V14219_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14219_4 = {S54, 0, &V14219_4_1224, &P14219_5};	/*Property = ScreenPixelsWidth*/
const Profile P14219 = {"14219", &P14219_4};	/*Profile ID - 14219*/
const Value V15396_1_1 = {S2, NULL};	/*Value = True*/
const Property P15396_1 = {S50, 0, &V15396_1_1, NULL};	/*Property = IsMobile*/
const Value V15396_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15396_5 = {S52, 0, &V15396_5_1222, &P15396_1};	/*Property = ScreenPixelsHeight*/
const Value V15396_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15396_4 = {S54, 0, &V15396_4_1224, &P15396_5};	/*Property = ScreenPixelsWidth*/
const Profile P15396 = {"15396", &P15396_4};	/*Profile ID - 15396*/
const Value V15409_1_1 = {S2, NULL};	/*Value = True*/
const Property P15409_1 = {S50, 0, &V15409_1_1, NULL};	/*Property = IsMobile*/
const Value V15409_5_24030 = {S71, NULL};	/*Value = 120*/
const Property P15409_5 = {S52, 0, &V15409_5_24030, &P15409_1};	/*Property = ScreenPixelsHeight*/
const Value V15409_4_24031 = {S71, NULL};	/*Value = 120*/
const Property P15409_4 = {S54, 0, &V15409_4_24031, &P15409_5};	/*Property = ScreenPixelsWidth*/
const Profile P15409 = {"15409", &P15409_4};	/*Profile ID - 15409*/
const Value V15407_1_1 = {S2, NULL};	/*Value = True*/
const Property P15407_1 = {S50, 0, &V15407_1_1, NULL};	/*Property = IsMobile*/
const Value V15407_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15407_5 = {S52, 0, &V15407_5_1222, &P15407_1};	/*Property = ScreenPixelsHeight*/
const Value V15407_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15407_4 = {S54, 0, &V15407_4_1224, &P15407_5};	/*Property = ScreenPixelsWidth*/
const Profile P15407 = {"15407", &P15407_4};	/*Profile ID - 15407*/
const Value V15411_1_1 = {S2, NULL};	/*Value = True*/
const Property P15411_1 = {S50, 0, &V15411_1_1, NULL};	/*Property = IsMobile*/
const Value V15411_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15411_5 = {S52, 0, &V15411_5_1293, &P15411_1};	/*Property = ScreenPixelsHeight*/
const Value V15411_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15411_4 = {S54, 0, &V15411_4_1296, &P15411_5};	/*Property = ScreenPixelsWidth*/
const Profile P15411 = {"15411", &P15411_4};	/*Profile ID - 15411*/
const Value V15412_1_1 = {S2, NULL};	/*Value = True*/
const Property P15412_1 = {S50, 0, &V15412_1_1, NULL};	/*Property = IsMobile*/
const Value V15412_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15412_5 = {S52, 0, &V15412_5_1293, &P15412_1};	/*Property = ScreenPixelsHeight*/
const Value V15412_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15412_4 = {S54, 0, &V15412_4_1296, &P15412_5};	/*Property = ScreenPixelsWidth*/
const Profile P15412 = {"15412", &P15412_4};	/*Profile ID - 15412*/
const Value V15408_1_1 = {S2, NULL};	/*Value = True*/
const Property P15408_1 = {S50, 0, &V15408_1_1, NULL};	/*Property = IsMobile*/
const Value V15408_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15408_5 = {S52, 0, &V15408_5_1294, &P15408_1};	/*Property = ScreenPixelsHeight*/
const Value V15408_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15408_4 = {S54, 0, &V15408_4_1296, &P15408_5};	/*Property = ScreenPixelsWidth*/
const Profile P15408 = {"15408", &P15408_4};	/*Profile ID - 15408*/
const Value V14143_1_1 = {S2, NULL};	/*Value = True*/
const Property P14143_1 = {S50, 0, &V14143_1_1, NULL};	/*Property = IsMobile*/
const Value V14143_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14143_5 = {S52, 0, &V14143_5_1380, &P14143_1};	/*Property = ScreenPixelsHeight*/
const Value V14143_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14143_4 = {S54, 0, &V14143_4_1279, &P14143_5};	/*Property = ScreenPixelsWidth*/
const Profile P14143 = {"14143", &P14143_4};	/*Profile ID - 14143*/
const Value V15410_1_1 = {S2, NULL};	/*Value = True*/
const Property P15410_1 = {S50, 0, &V15410_1_1, NULL};	/*Property = IsMobile*/
const Value V15410_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15410_5 = {S52, 0, &V15410_5_1233, &P15410_1};	/*Property = ScreenPixelsHeight*/
const Value V15410_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15410_4 = {S54, 0, &V15410_4_1235, &P15410_5};	/*Property = ScreenPixelsWidth*/
const Profile P15410 = {"15410", &P15410_4};	/*Profile ID - 15410*/
const Value V13779_1_1 = {S2, NULL};	/*Value = True*/
const Property P13779_1 = {S50, 0, &V13779_1_1, NULL};	/*Property = IsMobile*/
const Value V13779_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13779_5 = {S52, 0, &V13779_5_1293, &P13779_1};	/*Property = ScreenPixelsHeight*/
const Value V13779_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13779_4 = {S54, 0, &V13779_4_1296, &P13779_5};	/*Property = ScreenPixelsWidth*/
const Profile P13779 = {"13779", &P13779_4};	/*Profile ID - 13779*/
const Value V15413_1_1 = {S2, NULL};	/*Value = True*/
const Property P15413_1 = {S50, 0, &V15413_1_1, NULL};	/*Property = IsMobile*/
const Value V15413_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15413_5 = {S52, 0, &V15413_5_1293, &P15413_1};	/*Property = ScreenPixelsHeight*/
const Value V15413_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15413_4 = {S54, 0, &V15413_4_1296, &P15413_5};	/*Property = ScreenPixelsWidth*/
const Profile P15413 = {"15413", &P15413_4};	/*Profile ID - 15413*/
const Value V14673_1_1 = {S2, NULL};	/*Value = True*/
const Property P14673_1 = {S50, 0, &V14673_1_1, NULL};	/*Property = IsMobile*/
const Value V14673_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14673_5 = {S52, 0, &V14673_5_1222, &P14673_1};	/*Property = ScreenPixelsHeight*/
const Value V14673_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14673_4 = {S54, 0, &V14673_4_1224, &P14673_5};	/*Property = ScreenPixelsWidth*/
const Profile P14673 = {"14673", &P14673_4};	/*Profile ID - 14673*/
const Value V15418_1_1 = {S2, NULL};	/*Value = True*/
const Property P15418_1 = {S50, 0, &V15418_1_1, NULL};	/*Property = IsMobile*/
const Value V15418_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15418_5 = {S52, 0, &V15418_5_1293, &P15418_1};	/*Property = ScreenPixelsHeight*/
const Value V15418_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15418_4 = {S54, 0, &V15418_4_1296, &P15418_5};	/*Property = ScreenPixelsWidth*/
const Profile P15418 = {"15418", &P15418_4};	/*Profile ID - 15418*/
const Value V15417_1_1 = {S2, NULL};	/*Value = True*/
const Property P15417_1 = {S50, 0, &V15417_1_1, NULL};	/*Property = IsMobile*/
const Value V15417_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15417_5 = {S52, 0, &V15417_5_1222, &P15417_1};	/*Property = ScreenPixelsHeight*/
const Value V15417_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15417_4 = {S54, 0, &V15417_4_1224, &P15417_5};	/*Property = ScreenPixelsWidth*/
const Profile P15417 = {"15417", &P15417_4};	/*Profile ID - 15417*/
const Value V14919_1_1 = {S2, NULL};	/*Value = True*/
const Property P14919_1 = {S50, 0, &V14919_1_1, NULL};	/*Property = IsMobile*/
const Value V14919_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P14919_5 = {S52, 0, &V14919_5_1403, &P14919_1};	/*Property = ScreenPixelsHeight*/
const Value V14919_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P14919_4 = {S54, 0, &V14919_4_1404, &P14919_5};	/*Property = ScreenPixelsWidth*/
const Profile P14919 = {"14919", &P14919_4};	/*Profile ID - 14919*/
const Value V15201_1_1 = {S2, NULL};	/*Value = True*/
const Property P15201_1 = {S50, 0, &V15201_1_1, NULL};	/*Property = IsMobile*/
const Value V15201_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P15201_5 = {S52, 0, &V15201_5_1403, &P15201_1};	/*Property = ScreenPixelsHeight*/
const Value V15201_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P15201_4 = {S54, 0, &V15201_4_1404, &P15201_5};	/*Property = ScreenPixelsWidth*/
const Profile P15201 = {"15201", &P15201_4};	/*Profile ID - 15201*/
const Value V15575_1_1 = {S2, NULL};	/*Value = True*/
const Property P15575_1 = {S50, 0, &V15575_1_1, NULL};	/*Property = IsMobile*/
const Value V15575_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15575_5 = {S52, 0, &V15575_5_1227, &P15575_1};	/*Property = ScreenPixelsHeight*/
const Value V15575_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15575_4 = {S54, 0, &V15575_4_1229, &P15575_5};	/*Property = ScreenPixelsWidth*/
const Profile P15575 = {"15575", &P15575_4};	/*Profile ID - 15575*/
const Value V15419_1_1 = {S2, NULL};	/*Value = True*/
const Property P15419_1 = {S50, 0, &V15419_1_1, NULL};	/*Property = IsMobile*/
const Value V15419_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15419_5 = {S52, 0, &V15419_5_1222, &P15419_1};	/*Property = ScreenPixelsHeight*/
const Value V15419_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15419_4 = {S54, 0, &V15419_4_1224, &P15419_5};	/*Property = ScreenPixelsWidth*/
const Profile P15419 = {"15419", &P15419_4};	/*Profile ID - 15419*/
const Value V15420_1_1 = {S2, NULL};	/*Value = True*/
const Property P15420_1 = {S50, 0, &V15420_1_1, NULL};	/*Property = IsMobile*/
const Value V15420_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15420_5 = {S52, 0, &V15420_5_1222, &P15420_1};	/*Property = ScreenPixelsHeight*/
const Value V15420_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15420_4 = {S54, 0, &V15420_4_1224, &P15420_5};	/*Property = ScreenPixelsWidth*/
const Profile P15420 = {"15420", &P15420_4};	/*Profile ID - 15420*/
const Value V15426_1_1 = {S2, NULL};	/*Value = True*/
const Property P15426_1 = {S50, 0, &V15426_1_1, NULL};	/*Property = IsMobile*/
const Value V15426_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P15426_5 = {S52, 0, &V15426_5_1269, &P15426_1};	/*Property = ScreenPixelsHeight*/
const Value V15426_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15426_4 = {S54, 0, &V15426_4_1229, &P15426_5};	/*Property = ScreenPixelsWidth*/
const Profile P15426 = {"15426", &P15426_4};	/*Profile ID - 15426*/
const Value V14336_1_1 = {S2, NULL};	/*Value = True*/
const Property P14336_1 = {S50, 0, &V14336_1_1, NULL};	/*Property = IsMobile*/
const Value V14336_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14336_5 = {S52, 0, &V14336_5_1294, &P14336_1};	/*Property = ScreenPixelsHeight*/
const Value V14336_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14336_4 = {S54, 0, &V14336_4_1296, &P14336_5};	/*Property = ScreenPixelsWidth*/
const Profile P14336 = {"14336", &P14336_4};	/*Profile ID - 14336*/
const Value V15428_1_1 = {S2, NULL};	/*Value = True*/
const Property P15428_1 = {S50, 0, &V15428_1_1, NULL};	/*Property = IsMobile*/
const Value V15428_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15428_5 = {S52, 0, &V15428_5_1233, &P15428_1};	/*Property = ScreenPixelsHeight*/
const Value V15428_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15428_4 = {S54, 0, &V15428_4_1235, &P15428_5};	/*Property = ScreenPixelsWidth*/
const Profile P15428 = {"15428", &P15428_4};	/*Profile ID - 15428*/
const Value V15427_1_1 = {S2, NULL};	/*Value = True*/
const Property P15427_1 = {S50, 0, &V15427_1_1, NULL};	/*Property = IsMobile*/
const Value V15427_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15427_5 = {S52, 0, &V15427_5_1222, &P15427_1};	/*Property = ScreenPixelsHeight*/
const Value V15427_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15427_4 = {S54, 0, &V15427_4_1224, &P15427_5};	/*Property = ScreenPixelsWidth*/
const Profile P15427 = {"15427", &P15427_4};	/*Profile ID - 15427*/
const Value V15429_1_1 = {S2, NULL};	/*Value = True*/
const Property P15429_1 = {S50, 0, &V15429_1_1, NULL};	/*Property = IsMobile*/
const Value V15429_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15429_5 = {S52, 0, &V15429_5_1222, &P15429_1};	/*Property = ScreenPixelsHeight*/
const Value V15429_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15429_4 = {S54, 0, &V15429_4_1224, &P15429_5};	/*Property = ScreenPixelsWidth*/
const Profile P15429 = {"15429", &P15429_4};	/*Profile ID - 15429*/
const Value V15184_1_1 = {S2, NULL};	/*Value = True*/
const Property P15184_1 = {S50, 0, &V15184_1_1, NULL};	/*Property = IsMobile*/
const Value V15184_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15184_5 = {S52, 0, &V15184_5_1227, &P15184_1};	/*Property = ScreenPixelsHeight*/
const Value V15184_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15184_4 = {S54, 0, &V15184_4_1229, &P15184_5};	/*Property = ScreenPixelsWidth*/
const Profile P15184 = {"15184", &P15184_4};	/*Profile ID - 15184*/
const Value V14291_1_1 = {S2, NULL};	/*Value = True*/
const Property P14291_1 = {S50, 0, &V14291_1_1, NULL};	/*Property = IsMobile*/
const Value V14291_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14291_5 = {S52, 0, &V14291_5_1293, &P14291_1};	/*Property = ScreenPixelsHeight*/
const Value V14291_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14291_4 = {S54, 0, &V14291_4_1296, &P14291_5};	/*Property = ScreenPixelsWidth*/
const Profile P14291 = {"14291", &P14291_4};	/*Profile ID - 14291*/
const Value V20184_1_1 = {S2, NULL};	/*Value = True*/
const Property P20184_1 = {S50, 0, &V20184_1_1, NULL};	/*Property = IsMobile*/
const Value V20184_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20184_5 = {S52, 0, &V20184_5_1227, &P20184_1};	/*Property = ScreenPixelsHeight*/
const Value V20184_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20184_4 = {S54, 0, &V20184_4_1229, &P20184_5};	/*Property = ScreenPixelsWidth*/
const Profile P20184 = {"20184", &P20184_4};	/*Profile ID - 20184*/
const Value V15432_1_1 = {S2, NULL};	/*Value = True*/
const Property P15432_1 = {S50, 0, &V15432_1_1, NULL};	/*Property = IsMobile*/
const Value V15432_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15432_5 = {S52, 0, &V15432_5_1294, &P15432_1};	/*Property = ScreenPixelsHeight*/
const Value V15432_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15432_4 = {S54, 0, &V15432_4_1296, &P15432_5};	/*Property = ScreenPixelsWidth*/
const Profile P15432 = {"15432", &P15432_4};	/*Profile ID - 15432*/
const Value V15433_1_1 = {S2, NULL};	/*Value = True*/
const Property P15433_1 = {S50, 0, &V15433_1_1, NULL};	/*Property = IsMobile*/
const Value V15433_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15433_5 = {S52, 0, &V15433_5_1293, &P15433_1};	/*Property = ScreenPixelsHeight*/
const Value V15433_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15433_4 = {S54, 0, &V15433_4_1296, &P15433_5};	/*Property = ScreenPixelsWidth*/
const Profile P15433 = {"15433", &P15433_4};	/*Profile ID - 15433*/
const Value V15434_1_1 = {S2, NULL};	/*Value = True*/
const Property P15434_1 = {S50, 0, &V15434_1_1, NULL};	/*Property = IsMobile*/
const Value V15434_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P15434_5 = {S52, 0, &V15434_5_1333, &P15434_1};	/*Property = ScreenPixelsHeight*/
const Value V15434_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P15434_4 = {S54, 0, &V15434_4_1335, &P15434_5};	/*Property = ScreenPixelsWidth*/
const Profile P15434 = {"15434", &P15434_4};	/*Profile ID - 15434*/
const Value V17070_1_1 = {S2, NULL};	/*Value = True*/
const Property P17070_1 = {S50, 0, &V17070_1_1, NULL};	/*Property = IsMobile*/
const Value V17070_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P17070_5 = {S52, 0, &V17070_5_1333, &P17070_1};	/*Property = ScreenPixelsHeight*/
const Value V17070_4_37596 = {S98, NULL};	/*Value = 144*/
const Property P17070_4 = {S54, 0, &V17070_4_37596, &P17070_5};	/*Property = ScreenPixelsWidth*/
const Profile P17070 = {"17070", &P17070_4};	/*Profile ID - 17070*/
const Value V15437_1_1 = {S2, NULL};	/*Value = True*/
const Property P15437_1 = {S50, 0, &V15437_1_1, NULL};	/*Property = IsMobile*/
const Value V15437_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15437_5 = {S52, 0, &V15437_5_1233, &P15437_1};	/*Property = ScreenPixelsHeight*/
const Value V15437_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15437_4 = {S54, 0, &V15437_4_1235, &P15437_5};	/*Property = ScreenPixelsWidth*/
const Profile P15437 = {"15437", &P15437_4};	/*Profile ID - 15437*/
const Value V15438_1_1 = {S2, NULL};	/*Value = True*/
const Property P15438_1 = {S50, 0, &V15438_1_1, NULL};	/*Property = IsMobile*/
const Value V15438_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P15438_5 = {S52, 0, &V15438_5_1403, &P15438_1};	/*Property = ScreenPixelsHeight*/
const Value V15438_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P15438_4 = {S54, 0, &V15438_4_1404, &P15438_5};	/*Property = ScreenPixelsWidth*/
const Profile P15438 = {"15438", &P15438_4};	/*Profile ID - 15438*/
const Value V15439_1_1 = {S2, NULL};	/*Value = True*/
const Property P15439_1 = {S50, 0, &V15439_1_1, NULL};	/*Property = IsMobile*/
const Value V15439_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15439_5 = {S52, 0, &V15439_5_1222, &P15439_1};	/*Property = ScreenPixelsHeight*/
const Value V15439_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15439_4 = {S54, 0, &V15439_4_1224, &P15439_5};	/*Property = ScreenPixelsWidth*/
const Profile P15439 = {"15439", &P15439_4};	/*Profile ID - 15439*/
const Value V15441_1_1 = {S2, NULL};	/*Value = True*/
const Property P15441_1 = {S50, 0, &V15441_1_1, NULL};	/*Property = IsMobile*/
const Value V15441_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15441_5 = {S52, 0, &V15441_5_1293, &P15441_1};	/*Property = ScreenPixelsHeight*/
const Value V15441_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15441_4 = {S54, 0, &V15441_4_1296, &P15441_5};	/*Property = ScreenPixelsWidth*/
const Profile P15441 = {"15441", &P15441_4};	/*Profile ID - 15441*/
const Value V15440_1_1 = {S2, NULL};	/*Value = True*/
const Property P15440_1 = {S50, 0, &V15440_1_1, NULL};	/*Property = IsMobile*/
const Value V15440_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15440_5 = {S52, 0, &V15440_5_1294, &P15440_1};	/*Property = ScreenPixelsHeight*/
const Value V15440_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15440_4 = {S54, 0, &V15440_4_1296, &P15440_5};	/*Property = ScreenPixelsWidth*/
const Profile P15440 = {"15440", &P15440_4};	/*Profile ID - 15440*/
const Value V15444_1_1 = {S2, NULL};	/*Value = True*/
const Property P15444_1 = {S50, 0, &V15444_1_1, NULL};	/*Property = IsMobile*/
const Value V15444_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15444_5 = {S52, 0, &V15444_5_1233, &P15444_1};	/*Property = ScreenPixelsHeight*/
const Value V15444_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15444_4 = {S54, 0, &V15444_4_1235, &P15444_5};	/*Property = ScreenPixelsWidth*/
const Profile P15444 = {"15444", &P15444_4};	/*Profile ID - 15444*/
const Value V15445_1_1 = {S2, NULL};	/*Value = True*/
const Property P15445_1 = {S50, 0, &V15445_1_1, NULL};	/*Property = IsMobile*/
const Value V15445_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15445_5 = {S52, 0, &V15445_5_1293, &P15445_1};	/*Property = ScreenPixelsHeight*/
const Value V15445_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15445_4 = {S54, 0, &V15445_4_1296, &P15445_5};	/*Property = ScreenPixelsWidth*/
const Profile P15445 = {"15445", &P15445_4};	/*Profile ID - 15445*/
const Value V15446_1_1 = {S2, NULL};	/*Value = True*/
const Property P15446_1 = {S50, 0, &V15446_1_1, NULL};	/*Property = IsMobile*/
const Value V15446_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15446_5 = {S52, 0, &V15446_5_1293, &P15446_1};	/*Property = ScreenPixelsHeight*/
const Value V15446_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15446_4 = {S54, 0, &V15446_4_1296, &P15446_5};	/*Property = ScreenPixelsWidth*/
const Profile P15446 = {"15446", &P15446_4};	/*Profile ID - 15446*/
const Value V15448_1_1 = {S2, NULL};	/*Value = True*/
const Property P15448_1 = {S50, 0, &V15448_1_1, NULL};	/*Property = IsMobile*/
const Value V15448_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15448_5 = {S52, 0, &V15448_5_1227, &P15448_1};	/*Property = ScreenPixelsHeight*/
const Value V15448_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15448_4 = {S54, 0, &V15448_4_1229, &P15448_5};	/*Property = ScreenPixelsWidth*/
const Profile P15448 = {"15448", &P15448_4};	/*Profile ID - 15448*/
const Value V15430_1_1 = {S2, NULL};	/*Value = True*/
const Property P15430_1 = {S50, 0, &V15430_1_1, NULL};	/*Property = IsMobile*/
const Value V15430_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15430_5 = {S52, 0, &V15430_5_1233, &P15430_1};	/*Property = ScreenPixelsHeight*/
const Value V15430_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15430_4 = {S54, 0, &V15430_4_1235, &P15430_5};	/*Property = ScreenPixelsWidth*/
const Profile P15430 = {"15430", &P15430_4};	/*Profile ID - 15430*/
const Value V15449_1_1 = {S2, NULL};	/*Value = True*/
const Property P15449_1 = {S50, 0, &V15449_1_1, NULL};	/*Property = IsMobile*/
const Value V15449_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15449_5 = {S52, 0, &V15449_5_1294, &P15449_1};	/*Property = ScreenPixelsHeight*/
const Value V15449_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15449_4 = {S54, 0, &V15449_4_1296, &P15449_5};	/*Property = ScreenPixelsWidth*/
const Profile P15449 = {"15449", &P15449_4};	/*Profile ID - 15449*/
const Value V15451_1_1 = {S2, NULL};	/*Value = True*/
const Property P15451_1 = {S50, 0, &V15451_1_1, NULL};	/*Property = IsMobile*/
const Value V15451_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15451_5 = {S52, 0, &V15451_5_1293, &P15451_1};	/*Property = ScreenPixelsHeight*/
const Value V15451_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15451_4 = {S54, 0, &V15451_4_1296, &P15451_5};	/*Property = ScreenPixelsWidth*/
const Profile P15451 = {"15451", &P15451_4};	/*Profile ID - 15451*/
const Value V15453_1_1 = {S2, NULL};	/*Value = True*/
const Property P15453_1 = {S50, 0, &V15453_1_1, NULL};	/*Property = IsMobile*/
const Value V15453_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15453_5 = {S52, 0, &V15453_5_1222, &P15453_1};	/*Property = ScreenPixelsHeight*/
const Value V15453_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15453_4 = {S54, 0, &V15453_4_1224, &P15453_5};	/*Property = ScreenPixelsWidth*/
const Profile P15453 = {"15453", &P15453_4};	/*Profile ID - 15453*/
const Value V15455_1_1 = {S2, NULL};	/*Value = True*/
const Property P15455_1 = {S50, 0, &V15455_1_1, NULL};	/*Property = IsMobile*/
const Value V15455_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P15455_5 = {S52, 0, &V15455_5_1333, &P15455_1};	/*Property = ScreenPixelsHeight*/
const Value V15455_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P15455_4 = {S54, 0, &V15455_4_1335, &P15455_5};	/*Property = ScreenPixelsWidth*/
const Profile P15455 = {"15455", &P15455_4};	/*Profile ID - 15455*/
const Value V15457_1_1 = {S2, NULL};	/*Value = True*/
const Property P15457_1 = {S50, 0, &V15457_1_1, NULL};	/*Property = IsMobile*/
const Value V15457_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15457_5 = {S52, 0, &V15457_5_1222, &P15457_1};	/*Property = ScreenPixelsHeight*/
const Value V15457_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15457_4 = {S54, 0, &V15457_4_1224, &P15457_5};	/*Property = ScreenPixelsWidth*/
const Profile P15457 = {"15457", &P15457_4};	/*Profile ID - 15457*/
const Value V15454_1_1 = {S2, NULL};	/*Value = True*/
const Property P15454_1 = {S50, 0, &V15454_1_1, NULL};	/*Property = IsMobile*/
const Value V15454_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P15454_5 = {S52, 0, &V15454_5_1333, &P15454_1};	/*Property = ScreenPixelsHeight*/
const Value V15454_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P15454_4 = {S54, 0, &V15454_4_1335, &P15454_5};	/*Property = ScreenPixelsWidth*/
const Profile P15454 = {"15454", &P15454_4};	/*Profile ID - 15454*/
const Value V15458_1_1 = {S2, NULL};	/*Value = True*/
const Property P15458_1 = {S50, 0, &V15458_1_1, NULL};	/*Property = IsMobile*/
const Value V15458_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15458_5 = {S52, 0, &V15458_5_1233, &P15458_1};	/*Property = ScreenPixelsHeight*/
const Value V15458_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15458_4 = {S54, 0, &V15458_4_1235, &P15458_5};	/*Property = ScreenPixelsWidth*/
const Profile P15458 = {"15458", &P15458_4};	/*Profile ID - 15458*/
const Value V15459_1_1 = {S2, NULL};	/*Value = True*/
const Property P15459_1 = {S50, 0, &V15459_1_1, NULL};	/*Property = IsMobile*/
const Value V15459_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15459_5 = {S52, 0, &V15459_5_1222, &P15459_1};	/*Property = ScreenPixelsHeight*/
const Value V15459_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15459_4 = {S54, 0, &V15459_4_1224, &P15459_5};	/*Property = ScreenPixelsWidth*/
const Profile P15459 = {"15459", &P15459_4};	/*Profile ID - 15459*/
const Value V12863_1_1 = {S2, NULL};	/*Value = True*/
const Property P12863_1 = {S50, 0, &V12863_1_1, NULL};	/*Property = IsMobile*/
const Value V12863_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12863_5 = {S52, 0, &V12863_5_1233, &P12863_1};	/*Property = ScreenPixelsHeight*/
const Value V12863_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12863_4 = {S54, 0, &V12863_4_1235, &P12863_5};	/*Property = ScreenPixelsWidth*/
const Profile P12863 = {"12863", &P12863_4};	/*Profile ID - 12863*/
const Value V15461_1_1 = {S2, NULL};	/*Value = True*/
const Property P15461_1 = {S50, 0, &V15461_1_1, NULL};	/*Property = IsMobile*/
const Value V15461_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15461_5 = {S52, 0, &V15461_5_1294, &P15461_1};	/*Property = ScreenPixelsHeight*/
const Value V15461_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15461_4 = {S54, 0, &V15461_4_1296, &P15461_5};	/*Property = ScreenPixelsWidth*/
const Profile P15461 = {"15461", &P15461_4};	/*Profile ID - 15461*/
const Value V15460_1_1 = {S2, NULL};	/*Value = True*/
const Property P15460_1 = {S50, 0, &V15460_1_1, NULL};	/*Property = IsMobile*/
const Value V15460_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15460_5 = {S52, 0, &V15460_5_1293, &P15460_1};	/*Property = ScreenPixelsHeight*/
const Value V15460_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15460_4 = {S54, 0, &V15460_4_1296, &P15460_5};	/*Property = ScreenPixelsWidth*/
const Profile P15460 = {"15460", &P15460_4};	/*Profile ID - 15460*/
const Value V15462_1_1 = {S2, NULL};	/*Value = True*/
const Property P15462_1 = {S50, 0, &V15462_1_1, NULL};	/*Property = IsMobile*/
const Value V15462_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15462_5 = {S52, 0, &V15462_5_1222, &P15462_1};	/*Property = ScreenPixelsHeight*/
const Value V15462_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15462_4 = {S54, 0, &V15462_4_1224, &P15462_5};	/*Property = ScreenPixelsWidth*/
const Profile P15462 = {"15462", &P15462_4};	/*Profile ID - 15462*/
const Value V15464_1_1 = {S2, NULL};	/*Value = True*/
const Property P15464_1 = {S50, 0, &V15464_1_1, NULL};	/*Property = IsMobile*/
const Value V15464_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15464_5 = {S52, 0, &V15464_5_1222, &P15464_1};	/*Property = ScreenPixelsHeight*/
const Value V15464_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15464_4 = {S54, 0, &V15464_4_1224, &P15464_5};	/*Property = ScreenPixelsWidth*/
const Profile P15464 = {"15464", &P15464_4};	/*Profile ID - 15464*/
const Value V15466_1_1 = {S2, NULL};	/*Value = True*/
const Property P15466_1 = {S50, 0, &V15466_1_1, NULL};	/*Property = IsMobile*/
const Value V15466_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15466_5 = {S52, 0, &V15466_5_1293, &P15466_1};	/*Property = ScreenPixelsHeight*/
const Value V15466_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15466_4 = {S54, 0, &V15466_4_1296, &P15466_5};	/*Property = ScreenPixelsWidth*/
const Profile P15466 = {"15466", &P15466_4};	/*Profile ID - 15466*/
const Value V15450_1_1 = {S2, NULL};	/*Value = True*/
const Property P15450_1 = {S50, 0, &V15450_1_1, NULL};	/*Property = IsMobile*/
const Value V15450_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15450_5 = {S52, 0, &V15450_5_1222, &P15450_1};	/*Property = ScreenPixelsHeight*/
const Value V15450_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15450_4 = {S54, 0, &V15450_4_1224, &P15450_5};	/*Property = ScreenPixelsWidth*/
const Profile P15450 = {"15450", &P15450_4};	/*Profile ID - 15450*/
const Value V15467_1_1 = {S2, NULL};	/*Value = True*/
const Property P15467_1 = {S50, 0, &V15467_1_1, NULL};	/*Property = IsMobile*/
const Value V15467_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15467_5 = {S52, 0, &V15467_5_1233, &P15467_1};	/*Property = ScreenPixelsHeight*/
const Value V15467_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15467_4 = {S54, 0, &V15467_4_1235, &P15467_5};	/*Property = ScreenPixelsWidth*/
const Profile P15467 = {"15467", &P15467_4};	/*Profile ID - 15467*/
const Value V15469_1_1 = {S2, NULL};	/*Value = True*/
const Property P15469_1 = {S50, 0, &V15469_1_1, NULL};	/*Property = IsMobile*/
const Value V15469_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15469_5 = {S52, 0, &V15469_5_1233, &P15469_1};	/*Property = ScreenPixelsHeight*/
const Value V15469_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15469_4 = {S54, 0, &V15469_4_1235, &P15469_5};	/*Property = ScreenPixelsWidth*/
const Profile P15469 = {"15469", &P15469_4};	/*Profile ID - 15469*/
const Value V15472_1_1 = {S2, NULL};	/*Value = True*/
const Property P15472_1 = {S50, 0, &V15472_1_1, NULL};	/*Property = IsMobile*/
const Value V15472_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15472_5 = {S52, 0, &V15472_5_1233, &P15472_1};	/*Property = ScreenPixelsHeight*/
const Value V15472_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15472_4 = {S54, 0, &V15472_4_1235, &P15472_5};	/*Property = ScreenPixelsWidth*/
const Profile P15472 = {"15472", &P15472_4};	/*Profile ID - 15472*/
const Value V12252_1_1 = {S2, NULL};	/*Value = True*/
const Property P12252_1 = {S50, 0, &V12252_1_1, NULL};	/*Property = IsMobile*/
const Value V12252_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12252_5 = {S52, 0, &V12252_5_1222, &P12252_1};	/*Property = ScreenPixelsHeight*/
const Value V12252_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12252_4 = {S54, 0, &V12252_4_1224, &P12252_5};	/*Property = ScreenPixelsWidth*/
const Profile P12252 = {"12252", &P12252_4};	/*Profile ID - 12252*/
const Value V15473_1_1 = {S2, NULL};	/*Value = True*/
const Property P15473_1 = {S50, 0, &V15473_1_1, NULL};	/*Property = IsMobile*/
const Value V15473_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15473_5 = {S52, 0, &V15473_5_1233, &P15473_1};	/*Property = ScreenPixelsHeight*/
const Value V15473_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15473_4 = {S54, 0, &V15473_4_1235, &P15473_5};	/*Property = ScreenPixelsWidth*/
const Profile P15473 = {"15473", &P15473_4};	/*Profile ID - 15473*/
const Value V15474_1_1 = {S2, NULL};	/*Value = True*/
const Property P15474_1 = {S50, 0, &V15474_1_1, NULL};	/*Property = IsMobile*/
const Value V15474_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15474_5 = {S52, 0, &V15474_5_1233, &P15474_1};	/*Property = ScreenPixelsHeight*/
const Value V15474_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15474_4 = {S54, 0, &V15474_4_1235, &P15474_5};	/*Property = ScreenPixelsWidth*/
const Profile P15474 = {"15474", &P15474_4};	/*Profile ID - 15474*/
const Value V15475_1_1 = {S2, NULL};	/*Value = True*/
const Property P15475_1 = {S50, 0, &V15475_1_1, NULL};	/*Property = IsMobile*/
const Value V15475_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15475_5 = {S52, 0, &V15475_5_1293, &P15475_1};	/*Property = ScreenPixelsHeight*/
const Value V15475_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15475_4 = {S54, 0, &V15475_4_1296, &P15475_5};	/*Property = ScreenPixelsWidth*/
const Profile P15475 = {"15475", &P15475_4};	/*Profile ID - 15475*/
const Value V15477_1_1 = {S2, NULL};	/*Value = True*/
const Property P15477_1 = {S50, 0, &V15477_1_1, NULL};	/*Property = IsMobile*/
const Value V15477_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15477_5 = {S52, 0, &V15477_5_1294, &P15477_1};	/*Property = ScreenPixelsHeight*/
const Value V15477_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15477_4 = {S54, 0, &V15477_4_1296, &P15477_5};	/*Property = ScreenPixelsWidth*/
const Profile P15477 = {"15477", &P15477_4};	/*Profile ID - 15477*/
const Value V13693_1_1 = {S2, NULL};	/*Value = True*/
const Property P13693_1 = {S50, 0, &V13693_1_1, NULL};	/*Property = IsMobile*/
const Value V13693_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13693_5 = {S52, 0, &V13693_5_1293, &P13693_1};	/*Property = ScreenPixelsHeight*/
const Value V13693_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13693_4 = {S54, 0, &V13693_4_1296, &P13693_5};	/*Property = ScreenPixelsWidth*/
const Profile P13693 = {"13693", &P13693_4};	/*Profile ID - 13693*/
const Value V15479_1_1 = {S2, NULL};	/*Value = True*/
const Property P15479_1 = {S50, 0, &V15479_1_1, NULL};	/*Property = IsMobile*/
const Value V15479_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15479_5 = {S52, 0, &V15479_5_1222, &P15479_1};	/*Property = ScreenPixelsHeight*/
const Value V15479_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15479_4 = {S54, 0, &V15479_4_1224, &P15479_5};	/*Property = ScreenPixelsWidth*/
const Profile P15479 = {"15479", &P15479_4};	/*Profile ID - 15479*/
const Value V14030_1_1 = {S2, NULL};	/*Value = True*/
const Property P14030_1 = {S50, 0, &V14030_1_1, NULL};	/*Property = IsMobile*/
const Value V14030_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14030_5 = {S52, 0, &V14030_5_1227, &P14030_1};	/*Property = ScreenPixelsHeight*/
const Value V14030_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14030_4 = {S54, 0, &V14030_4_1229, &P14030_5};	/*Property = ScreenPixelsWidth*/
const Profile P14030 = {"14030", &P14030_4};	/*Profile ID - 14030*/
const Value V15465_1_1 = {S2, NULL};	/*Value = True*/
const Property P15465_1 = {S50, 0, &V15465_1_1, NULL};	/*Property = IsMobile*/
const Value V15465_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15465_5 = {S52, 0, &V15465_5_1233, &P15465_1};	/*Property = ScreenPixelsHeight*/
const Value V15465_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15465_4 = {S54, 0, &V15465_4_1235, &P15465_5};	/*Property = ScreenPixelsWidth*/
const Profile P15465 = {"15465", &P15465_4};	/*Profile ID - 15465*/
const Value V15483_1_1 = {S2, NULL};	/*Value = True*/
const Property P15483_1 = {S50, 0, &V15483_1_1, NULL};	/*Property = IsMobile*/
const Value V15483_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15483_5 = {S52, 0, &V15483_5_1222, &P15483_1};	/*Property = ScreenPixelsHeight*/
const Value V15483_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15483_4 = {S54, 0, &V15483_4_1229, &P15483_5};	/*Property = ScreenPixelsWidth*/
const Profile P15483 = {"15483", &P15483_4};	/*Profile ID - 15483*/
const Value V15486_1_1 = {S2, NULL};	/*Value = True*/
const Property P15486_1 = {S50, 0, &V15486_1_1, NULL};	/*Property = IsMobile*/
const Value V15486_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15486_5 = {S52, 0, &V15486_5_1233, &P15486_1};	/*Property = ScreenPixelsHeight*/
const Value V15486_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15486_4 = {S54, 0, &V15486_4_1235, &P15486_5};	/*Property = ScreenPixelsWidth*/
const Profile P15486 = {"15486", &P15486_4};	/*Profile ID - 15486*/
const Value V15487_1_1 = {S2, NULL};	/*Value = True*/
const Property P15487_1 = {S50, 0, &V15487_1_1, NULL};	/*Property = IsMobile*/
const Value V15487_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15487_5 = {S52, 0, &V15487_5_1304, &P15487_1};	/*Property = ScreenPixelsHeight*/
const Value V15487_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15487_4 = {S54, 0, &V15487_4_1224, &P15487_5};	/*Property = ScreenPixelsWidth*/
const Profile P15487 = {"15487", &P15487_4};	/*Profile ID - 15487*/
const Value V14342_1_1 = {S2, NULL};	/*Value = True*/
const Property P14342_1 = {S50, 0, &V14342_1_1, NULL};	/*Property = IsMobile*/
const Value V14342_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14342_5 = {S52, 0, &V14342_5_1294, &P14342_1};	/*Property = ScreenPixelsHeight*/
const Value V14342_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14342_4 = {S54, 0, &V14342_4_1296, &P14342_5};	/*Property = ScreenPixelsWidth*/
const Profile P14342 = {"14342", &P14342_4};	/*Profile ID - 14342*/
const Value V14026_1_1 = {S2, NULL};	/*Value = True*/
const Property P14026_1 = {S50, 0, &V14026_1_1, NULL};	/*Property = IsMobile*/
const Value V14026_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P14026_5 = {S52, 0, &V14026_5_1333, &P14026_1};	/*Property = ScreenPixelsHeight*/
const Value V14026_4_37572 = {S93, NULL};	/*Value = 132*/
const Property P14026_4 = {S54, 0, &V14026_4_37572, &P14026_5};	/*Property = ScreenPixelsWidth*/
const Profile P14026 = {"14026", &P14026_4};	/*Profile ID - 14026*/
const Value V12036_1_1 = {S2, NULL};	/*Value = True*/
const Property P12036_1 = {S50, 0, &V12036_1_1, NULL};	/*Property = IsMobile*/
const Value V12036_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12036_5 = {S52, 0, &V12036_5_1222, &P12036_1};	/*Property = ScreenPixelsHeight*/
const Value V12036_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12036_4 = {S54, 0, &V12036_4_1224, &P12036_5};	/*Property = ScreenPixelsWidth*/
const Profile P12036 = {"12036", &P12036_4};	/*Profile ID - 12036*/
const Value V12700_1_1 = {S2, NULL};	/*Value = True*/
const Property P12700_1 = {S50, 0, &V12700_1_1, NULL};	/*Property = IsMobile*/
const Value V12700_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12700_5 = {S52, 0, &V12700_5_1222, &P12700_1};	/*Property = ScreenPixelsHeight*/
const Value V12700_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12700_4 = {S54, 0, &V12700_4_1224, &P12700_5};	/*Property = ScreenPixelsWidth*/
const Profile P12700 = {"12700", &P12700_4};	/*Profile ID - 12700*/
const Value V13949_1_1 = {S2, NULL};	/*Value = True*/
const Property P13949_1 = {S50, 0, &V13949_1_1, NULL};	/*Property = IsMobile*/
const Value V13949_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13949_5 = {S52, 0, &V13949_5_1304, &P13949_1};	/*Property = ScreenPixelsHeight*/
const Value V13949_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13949_4 = {S54, 0, &V13949_4_1224, &P13949_5};	/*Property = ScreenPixelsWidth*/
const Profile P13949 = {"13949", &P13949_4};	/*Profile ID - 13949*/
const Value V15490_1_1 = {S2, NULL};	/*Value = True*/
const Property P15490_1 = {S50, 0, &V15490_1_1, NULL};	/*Property = IsMobile*/
const Value V15490_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15490_5 = {S52, 0, &V15490_5_1293, &P15490_1};	/*Property = ScreenPixelsHeight*/
const Value V15490_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15490_4 = {S54, 0, &V15490_4_1296, &P15490_5};	/*Property = ScreenPixelsWidth*/
const Profile P15490 = {"15490", &P15490_4};	/*Profile ID - 15490*/
const Value V15491_1_1 = {S2, NULL};	/*Value = True*/
const Property P15491_1 = {S50, 0, &V15491_1_1, NULL};	/*Property = IsMobile*/
const Value V15491_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15491_5 = {S52, 0, &V15491_5_1233, &P15491_1};	/*Property = ScreenPixelsHeight*/
const Value V15491_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15491_4 = {S54, 0, &V15491_4_1235, &P15491_5};	/*Property = ScreenPixelsWidth*/
const Profile P15491 = {"15491", &P15491_4};	/*Profile ID - 15491*/
const Value V15492_1_1 = {S2, NULL};	/*Value = True*/
const Property P15492_1 = {S50, 0, &V15492_1_1, NULL};	/*Property = IsMobile*/
const Value V15492_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15492_5 = {S52, 0, &V15492_5_1222, &P15492_1};	/*Property = ScreenPixelsHeight*/
const Value V15492_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15492_4 = {S54, 0, &V15492_4_1224, &P15492_5};	/*Property = ScreenPixelsWidth*/
const Profile P15492 = {"15492", &P15492_4};	/*Profile ID - 15492*/
const Value V15493_1_1 = {S2, NULL};	/*Value = True*/
const Property P15493_1 = {S50, 0, &V15493_1_1, NULL};	/*Property = IsMobile*/
const Value V15493_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15493_5 = {S52, 0, &V15493_5_1233, &P15493_1};	/*Property = ScreenPixelsHeight*/
const Value V15493_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15493_4 = {S54, 0, &V15493_4_1235, &P15493_5};	/*Property = ScreenPixelsWidth*/
const Profile P15493 = {"15493", &P15493_4};	/*Profile ID - 15493*/
const Profile P7336 = {"7336", NULL};	/*Profile ID - 7336*/
const Value V15514_1_1 = {S2, NULL};	/*Value = True*/
const Property P15514_1 = {S50, 0, &V15514_1_1, NULL};	/*Property = IsMobile*/
const Value V15514_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15514_5 = {S52, 0, &V15514_5_1233, &P15514_1};	/*Property = ScreenPixelsHeight*/
const Value V15514_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15514_4 = {S54, 0, &V15514_4_1235, &P15514_5};	/*Property = ScreenPixelsWidth*/
const Profile P15514 = {"15514", &P15514_4};	/*Profile ID - 15514*/
const Value V15494_1_1 = {S2, NULL};	/*Value = True*/
const Property P15494_1 = {S50, 0, &V15494_1_1, NULL};	/*Property = IsMobile*/
const Value V15494_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15494_5 = {S52, 0, &V15494_5_1233, &P15494_1};	/*Property = ScreenPixelsHeight*/
const Value V15494_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15494_4 = {S54, 0, &V15494_4_1235, &P15494_5};	/*Property = ScreenPixelsWidth*/
const Profile P15494 = {"15494", &P15494_4};	/*Profile ID - 15494*/
const Value V17179_1_1 = {S2, NULL};	/*Value = True*/
const Property P17179_1 = {S50, 0, &V17179_1_1, NULL};	/*Property = IsMobile*/
const Value V17179_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17179_5 = {S52, 0, &V17179_5_1233, &P17179_1};	/*Property = ScreenPixelsHeight*/
const Value V17179_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17179_4 = {S54, 0, &V17179_4_1235, &P17179_5};	/*Property = ScreenPixelsWidth*/
const Profile P17179 = {"17179", &P17179_4};	/*Profile ID - 17179*/
const Value V15495_1_1 = {S2, NULL};	/*Value = True*/
const Property P15495_1 = {S50, 0, &V15495_1_1, NULL};	/*Property = IsMobile*/
const Value V15495_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15495_5 = {S52, 0, &V15495_5_1293, &P15495_1};	/*Property = ScreenPixelsHeight*/
const Value V15495_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15495_4 = {S54, 0, &V15495_4_1296, &P15495_5};	/*Property = ScreenPixelsWidth*/
const Profile P15495 = {"15495", &P15495_4};	/*Profile ID - 15495*/
const Value V14185_1_1 = {S2, NULL};	/*Value = True*/
const Property P14185_1 = {S50, 0, &V14185_1_1, NULL};	/*Property = IsMobile*/
const Value V14185_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14185_5 = {S52, 0, &V14185_5_1293, &P14185_1};	/*Property = ScreenPixelsHeight*/
const Value V14185_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14185_4 = {S54, 0, &V14185_4_1296, &P14185_5};	/*Property = ScreenPixelsWidth*/
const Profile P14185 = {"14185", &P14185_4};	/*Profile ID - 14185*/
const Value V15496_1_1 = {S2, NULL};	/*Value = True*/
const Property P15496_1 = {S50, 0, &V15496_1_1, NULL};	/*Property = IsMobile*/
const Value V15496_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15496_5 = {S52, 0, &V15496_5_1222, &P15496_1};	/*Property = ScreenPixelsHeight*/
const Value V15496_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15496_4 = {S54, 0, &V15496_4_1224, &P15496_5};	/*Property = ScreenPixelsWidth*/
const Profile P15496 = {"15496", &P15496_4};	/*Profile ID - 15496*/
const Value V14028_1_1 = {S2, NULL};	/*Value = True*/
const Property P14028_1 = {S50, 0, &V14028_1_1, NULL};	/*Property = IsMobile*/
const Value V14028_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14028_5 = {S52, 0, &V14028_5_1222, &P14028_1};	/*Property = ScreenPixelsHeight*/
const Value V14028_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14028_4 = {S54, 0, &V14028_4_1224, &P14028_5};	/*Property = ScreenPixelsWidth*/
const Profile P14028 = {"14028", &P14028_4};	/*Profile ID - 14028*/
const Value V14504_1_1 = {S2, NULL};	/*Value = True*/
const Property P14504_1 = {S50, 0, &V14504_1_1, NULL};	/*Property = IsMobile*/
const Value V14504_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14504_5 = {S52, 0, &V14504_5_1293, &P14504_1};	/*Property = ScreenPixelsHeight*/
const Value V14504_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14504_4 = {S54, 0, &V14504_4_1296, &P14504_5};	/*Property = ScreenPixelsWidth*/
const Profile P14504 = {"14504", &P14504_4};	/*Profile ID - 14504*/
const Value V15497_1_1 = {S2, NULL};	/*Value = True*/
const Property P15497_1 = {S50, 0, &V15497_1_1, NULL};	/*Property = IsMobile*/
const Value V15497_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15497_5 = {S52, 0, &V15497_5_1293, &P15497_1};	/*Property = ScreenPixelsHeight*/
const Value V15497_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15497_4 = {S54, 0, &V15497_4_1296, &P15497_5};	/*Property = ScreenPixelsWidth*/
const Profile P15497 = {"15497", &P15497_4};	/*Profile ID - 15497*/
const Value V15498_1_1 = {S2, NULL};	/*Value = True*/
const Property P15498_1 = {S50, 0, &V15498_1_1, NULL};	/*Property = IsMobile*/
const Value V15498_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15498_5 = {S52, 0, &V15498_5_1233, &P15498_1};	/*Property = ScreenPixelsHeight*/
const Value V15498_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15498_4 = {S54, 0, &V15498_4_1235, &P15498_5};	/*Property = ScreenPixelsWidth*/
const Profile P15498 = {"15498", &P15498_4};	/*Profile ID - 15498*/
const Value V15499_1_1 = {S2, NULL};	/*Value = True*/
const Property P15499_1 = {S50, 0, &V15499_1_1, NULL};	/*Property = IsMobile*/
const Value V15499_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15499_5 = {S52, 0, &V15499_5_1222, &P15499_1};	/*Property = ScreenPixelsHeight*/
const Value V15499_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15499_4 = {S54, 0, &V15499_4_1224, &P15499_5};	/*Property = ScreenPixelsWidth*/
const Profile P15499 = {"15499", &P15499_4};	/*Profile ID - 15499*/
const Value V15502_1_1 = {S2, NULL};	/*Value = True*/
const Property P15502_1 = {S50, 0, &V15502_1_1, NULL};	/*Property = IsMobile*/
const Value V15502_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15502_5 = {S52, 0, &V15502_5_1233, &P15502_1};	/*Property = ScreenPixelsHeight*/
const Value V15502_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15502_4 = {S54, 0, &V15502_4_1235, &P15502_5};	/*Property = ScreenPixelsWidth*/
const Profile P15502 = {"15502", &P15502_4};	/*Profile ID - 15502*/
const Value V15501_1_1 = {S2, NULL};	/*Value = True*/
const Property P15501_1 = {S50, 0, &V15501_1_1, NULL};	/*Property = IsMobile*/
const Value V15501_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15501_5 = {S52, 0, &V15501_5_1277, &P15501_1};	/*Property = ScreenPixelsHeight*/
const Value V15501_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15501_4 = {S54, 0, &V15501_4_1279, &P15501_5};	/*Property = ScreenPixelsWidth*/
const Profile P15501 = {"15501", &P15501_4};	/*Profile ID - 15501*/
const Value V15503_1_1 = {S2, NULL};	/*Value = True*/
const Property P15503_1 = {S50, 0, &V15503_1_1, NULL};	/*Property = IsMobile*/
const Value V15503_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15503_5 = {S52, 0, &V15503_5_1293, &P15503_1};	/*Property = ScreenPixelsHeight*/
const Value V15503_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15503_4 = {S54, 0, &V15503_4_1296, &P15503_5};	/*Property = ScreenPixelsWidth*/
const Profile P15503 = {"15503", &P15503_4};	/*Profile ID - 15503*/
const Value V14055_1_1 = {S2, NULL};	/*Value = True*/
const Property P14055_1 = {S50, 0, &V14055_1_1, NULL};	/*Property = IsMobile*/
const Value V14055_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14055_5 = {S52, 0, &V14055_5_1293, &P14055_1};	/*Property = ScreenPixelsHeight*/
const Value V14055_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14055_4 = {S54, 0, &V14055_4_1296, &P14055_5};	/*Property = ScreenPixelsWidth*/
const Profile P14055 = {"14055", &P14055_4};	/*Profile ID - 14055*/
const Value V15504_1_1 = {S2, NULL};	/*Value = True*/
const Property P15504_1 = {S50, 0, &V15504_1_1, NULL};	/*Property = IsMobile*/
const Value V15504_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15504_5 = {S52, 0, &V15504_5_1222, &P15504_1};	/*Property = ScreenPixelsHeight*/
const Value V15504_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15504_4 = {S54, 0, &V15504_4_1224, &P15504_5};	/*Property = ScreenPixelsWidth*/
const Profile P15504 = {"15504", &P15504_4};	/*Profile ID - 15504*/
const Value V15505_1_1 = {S2, NULL};	/*Value = True*/
const Property P15505_1 = {S50, 0, &V15505_1_1, NULL};	/*Property = IsMobile*/
const Value V15505_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15505_5 = {S52, 0, &V15505_5_1222, &P15505_1};	/*Property = ScreenPixelsHeight*/
const Value V15505_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15505_4 = {S54, 0, &V15505_4_1224, &P15505_5};	/*Property = ScreenPixelsWidth*/
const Profile P15505 = {"15505", &P15505_4};	/*Profile ID - 15505*/
const Value V15506_1_1 = {S2, NULL};	/*Value = True*/
const Property P15506_1 = {S50, 0, &V15506_1_1, NULL};	/*Property = IsMobile*/
const Value V15506_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15506_5 = {S52, 0, &V15506_5_1222, &P15506_1};	/*Property = ScreenPixelsHeight*/
const Value V15506_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15506_4 = {S54, 0, &V15506_4_1224, &P15506_5};	/*Property = ScreenPixelsWidth*/
const Profile P15506 = {"15506", &P15506_4};	/*Profile ID - 15506*/
const Value V12550_1_1 = {S2, NULL};	/*Value = True*/
const Property P12550_1 = {S50, 0, &V12550_1_1, NULL};	/*Property = IsMobile*/
const Value V12550_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12550_5 = {S52, 0, &V12550_5_1222, &P12550_1};	/*Property = ScreenPixelsHeight*/
const Value V12550_4_1368 = {S74, NULL};	/*Value = 208*/
const Property P12550_4 = {S54, 0, &V12550_4_1368, &P12550_5};	/*Property = ScreenPixelsWidth*/
const Profile P12550 = {"12550", &P12550_4};	/*Profile ID - 12550*/
const Value V17176_1_1 = {S2, NULL};	/*Value = True*/
const Property P17176_1 = {S50, 0, &V17176_1_1, NULL};	/*Property = IsMobile*/
const Value V17176_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17176_5 = {S52, 0, &V17176_5_1293, &P17176_1};	/*Property = ScreenPixelsHeight*/
const Value V17176_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17176_4 = {S54, 0, &V17176_4_1296, &P17176_5};	/*Property = ScreenPixelsWidth*/
const Profile P17176 = {"17176", &P17176_4};	/*Profile ID - 17176*/
const Value V15507_1_1 = {S2, NULL};	/*Value = True*/
const Property P15507_1 = {S50, 0, &V15507_1_1, NULL};	/*Property = IsMobile*/
const Value V15507_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15507_5 = {S52, 0, &V15507_5_1233, &P15507_1};	/*Property = ScreenPixelsHeight*/
const Value V15507_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15507_4 = {S54, 0, &V15507_4_1235, &P15507_5};	/*Property = ScreenPixelsWidth*/
const Profile P15507 = {"15507", &P15507_4};	/*Profile ID - 15507*/
const Value V13667_1_1 = {S2, NULL};	/*Value = True*/
const Property P13667_1 = {S50, 0, &V13667_1_1, NULL};	/*Property = IsMobile*/
const Value V13667_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13667_5 = {S52, 0, &V13667_5_1222, &P13667_1};	/*Property = ScreenPixelsHeight*/
const Value V13667_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13667_4 = {S54, 0, &V13667_4_1224, &P13667_5};	/*Property = ScreenPixelsWidth*/
const Profile P13667 = {"13667", &P13667_4};	/*Profile ID - 13667*/
const Value V15508_1_1 = {S2, NULL};	/*Value = True*/
const Property P15508_1 = {S50, 0, &V15508_1_1, NULL};	/*Property = IsMobile*/
const Value V15508_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15508_5 = {S52, 0, &V15508_5_1222, &P15508_1};	/*Property = ScreenPixelsHeight*/
const Value V15508_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15508_4 = {S54, 0, &V15508_4_1224, &P15508_5};	/*Property = ScreenPixelsWidth*/
const Profile P15508 = {"15508", &P15508_4};	/*Profile ID - 15508*/
const Value V15509_1_1 = {S2, NULL};	/*Value = True*/
const Property P15509_1 = {S50, 0, &V15509_1_1, NULL};	/*Property = IsMobile*/
const Value V15509_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15509_5 = {S52, 0, &V15509_5_1233, &P15509_1};	/*Property = ScreenPixelsHeight*/
const Value V15509_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15509_4 = {S54, 0, &V15509_4_1235, &P15509_5};	/*Property = ScreenPixelsWidth*/
const Profile P15509 = {"15509", &P15509_4};	/*Profile ID - 15509*/
const Value V15511_1_1 = {S2, NULL};	/*Value = True*/
const Property P15511_1 = {S50, 0, &V15511_1_1, NULL};	/*Property = IsMobile*/
const Value V15511_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15511_5 = {S52, 0, &V15511_5_1233, &P15511_1};	/*Property = ScreenPixelsHeight*/
const Value V15511_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15511_4 = {S54, 0, &V15511_4_1235, &P15511_5};	/*Property = ScreenPixelsWidth*/
const Profile P15511 = {"15511", &P15511_4};	/*Profile ID - 15511*/
const Value V15512_1_1 = {S2, NULL};	/*Value = True*/
const Property P15512_1 = {S50, 0, &V15512_1_1, NULL};	/*Property = IsMobile*/
const Value V15512_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15512_5 = {S52, 0, &V15512_5_1380, &P15512_1};	/*Property = ScreenPixelsHeight*/
const Value V15512_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15512_4 = {S54, 0, &V15512_4_1279, &P15512_5};	/*Property = ScreenPixelsWidth*/
const Profile P15512 = {"15512", &P15512_4};	/*Profile ID - 15512*/
const Value V22273_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_279 = {S1, 0, &V22273_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22273_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_282 = {S3, 0, &V22273_282_57155, &P22273_279};	/*Property = BlobBuilder*/
const Value V22273_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_350 = {S4, 0, &V22273_350_57446, &P22273_282};	/*Property = Canvas*/
const Value V22273_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_283 = {S5, 0, &V22273_283_57161, &P22273_350};	/*Property = CssBackground*/
const Value V22273_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_284 = {S6, 0, &V22273_284_57164, &P22273_283};	/*Property = CssBorderImage*/
const Value V22273_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_285 = {S7, 0, &V22273_285_57167, &P22273_284};	/*Property = CssCanvas*/
const Value V22273_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_286 = {S8, 0, &V22273_286_57170, &P22273_285};	/*Property = CssColor*/
const Value V22273_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_237 = {S9, 0, &V22273_237_55786, &P22273_286};	/*Property = CssColumn*/
const Value V22273_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_287 = {S10, 0, &V22273_287_57173, &P22273_237};	/*Property = CssFlexbox*/
const Value V22273_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_288 = {S11, 0, &V22273_288_57176, &P22273_287};	/*Property = CssFont*/
const Value V22273_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_289 = {S12, 0, &V22273_289_86899, &P22273_288};	/*Property = CssImages*/
const Value V22273_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_291 = {S13, 0, &V22273_291_57182, &P22273_289};	/*Property = CssMediaQueries*/
const Value V22273_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_292 = {S14, 0, &V22273_292_57185, &P22273_291};	/*Property = CssMinMax*/
const Value V22273_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_293 = {S15, 0, &V22273_293_57188, &P22273_292};	/*Property = CssOverflow*/
const Value V22273_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_294 = {S16, 0, &V22273_294_57191, &P22273_293};	/*Property = CssPosition*/
const Value V22273_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_295 = {S17, 0, &V22273_295_57194, &P22273_294};	/*Property = CssText*/
const Value V22273_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_297 = {S18, 0, &V22273_297_57197, &P22273_295};	/*Property = CssTransforms*/
const Value V22273_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_241 = {S19, 0, &V22273_241_55801, &P22273_297};	/*Property = CssTransitions*/
const Value V22273_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_298 = {S20, 0, &V22273_298_57208, &P22273_241};	/*Property = CssUI*/
const Value V22273_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_300 = {S21, 0, &V22273_300_57217, &P22273_298};	/*Property = DataSet*/
const Value V22273_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_301 = {S22, 0, &V22273_301_57220, &P22273_300};	/*Property = DataUrl*/
const Value V22273_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_302 = {S23, 0, &V22273_302_57223, &P22273_301};	/*Property = DeviceOrientation*/
const Value V22273_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_304 = {S24, 0, &V22273_304_57229, &P22273_302};	/*Property = FileReader*/
const Value V22273_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_305 = {S25, 0, &V22273_305_57232, &P22273_304};	/*Property = FileSaver*/
const Value V22273_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_306 = {S26, 0, &V22273_306_57238, &P22273_305};	/*Property = FileWriter*/
const Value V22273_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_307 = {S27, 0, &V22273_307_57241, &P22273_306};	/*Property = FormData*/
const Value V22273_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_309 = {S28, 0, &V22273_309_57247, &P22273_307};	/*Property = Fullscreen*/
const Value V22273_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_310 = {S29, 0, &V22273_310_57250, &P22273_309};	/*Property = GeoLocation*/
const Value V22273_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_312 = {S30, 0, &V22273_312_57256, &P22273_310};	/*Property = History*/
const Value V22273_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_314 = {S31, 0, &V22273_314_57262, &P22273_312};	/*Property = Html5*/
const Value V22273_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_313 = {S32, 0, &V22273_313_57259, &P22273_314};	/*Property = Html-Media-Capture*/
const Value V22273_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_315 = {S33, 0, &V22273_315_57265, &P22273_313};	/*Property = Iframe*/
const Value V22273_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_316 = {S34, 0, &V22273_316_57268, &P22273_315};	/*Property = IndexedDB*/
const Value V22273_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_317 = {S35, 0, &V22273_317_57271, &P22273_316};	/*Property = Json*/
const Value V22273_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_15 = {S37, 0, &V22273_15_72, &P22273_317};	/*Property = LayoutEngine*/
const Value V22273_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_344 = {S38, 0, &V22273_344_57274, &P22273_15};	/*Property = Masking*/
const Value V22273_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_320 = {S39, 0, &V22273_320_57292, &P22273_344};	/*Property = PostMessage*/
const Value V22273_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_321 = {S40, 0, &V22273_321_57295, &P22273_320};	/*Property = Progress*/
const Value V22273_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_322 = {S41, 0, &V22273_322_57298, &P22273_321};	/*Property = Prompts*/
const Value V22273_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_323 = {S42, 0, &V22273_323_57301, &P22273_322};	/*Property = Selector*/
const Value V22273_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_324 = {S43, 0, &V22273_324_57307, &P22273_323};	/*Property = Svg*/
const Value V22273_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_327 = {S44, 0, &V22273_327_57316, &P22273_324};	/*Property = TouchEvents*/
const Value V22273_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_328 = {S45, 0, &V22273_328_57319, &P22273_327};	/*Property = Track*/
const Value V22273_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_329 = {S46, 0, &V22273_329_57325, &P22273_328};	/*Property = Video*/
const Value V22273_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_330 = {S47, 0, &V22273_330_57328, &P22273_329};	/*Property = Viewport*/
const Value V22273_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_332 = {S48, 0, &V22273_332_57340, &P22273_330};	/*Property = WebWorkers*/
const Value V22273_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22273_333 = {S49, 0, &V22273_333_57343, &P22273_332};	/*Property = Xhr2*/
const Profile P22273 = {"22273", &P22273_333};	/*Profile ID - 22273*/
const Value V15516_1_1 = {S2, NULL};	/*Value = True*/
const Property P15516_1 = {S50, 0, &V15516_1_1, NULL};	/*Property = IsMobile*/
const Value V15516_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15516_5 = {S52, 0, &V15516_5_1233, &P15516_1};	/*Property = ScreenPixelsHeight*/
const Value V15516_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P15516_4 = {S54, 0, &V15516_4_1335, &P15516_5};	/*Property = ScreenPixelsWidth*/
const Profile P15516 = {"15516", &P15516_4};	/*Profile ID - 15516*/
const Value V15519_1_1 = {S2, NULL};	/*Value = True*/
const Property P15519_1 = {S50, 0, &V15519_1_1, NULL};	/*Property = IsMobile*/
const Value V15519_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15519_5 = {S52, 0, &V15519_5_1293, &P15519_1};	/*Property = ScreenPixelsHeight*/
const Value V15519_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15519_4 = {S54, 0, &V15519_4_1296, &P15519_5};	/*Property = ScreenPixelsWidth*/
const Profile P15519 = {"15519", &P15519_4};	/*Profile ID - 15519*/
const Value V15521_1_1 = {S2, NULL};	/*Value = True*/
const Property P15521_1 = {S50, 0, &V15521_1_1, NULL};	/*Property = IsMobile*/
const Value V15521_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15521_5 = {S52, 0, &V15521_5_1233, &P15521_1};	/*Property = ScreenPixelsHeight*/
const Value V15521_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15521_4 = {S54, 0, &V15521_4_1235, &P15521_5};	/*Property = ScreenPixelsWidth*/
const Profile P15521 = {"15521", &P15521_4};	/*Profile ID - 15521*/
const Value V15520_1_1 = {S2, NULL};	/*Value = True*/
const Property P15520_1 = {S50, 0, &V15520_1_1, NULL};	/*Property = IsMobile*/
const Value V15520_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15520_5 = {S52, 0, &V15520_5_1222, &P15520_1};	/*Property = ScreenPixelsHeight*/
const Value V15520_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15520_4 = {S54, 0, &V15520_4_1224, &P15520_5};	/*Property = ScreenPixelsWidth*/
const Profile P15520 = {"15520", &P15520_4};	/*Profile ID - 15520*/
const Value V13717_1_1 = {S2, NULL};	/*Value = True*/
const Property P13717_1 = {S50, 0, &V13717_1_1, NULL};	/*Property = IsMobile*/
const Value V13717_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13717_5 = {S52, 0, &V13717_5_1294, &P13717_1};	/*Property = ScreenPixelsHeight*/
const Value V13717_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13717_4 = {S54, 0, &V13717_4_1296, &P13717_5};	/*Property = ScreenPixelsWidth*/
const Profile P13717 = {"13717", &P13717_4};	/*Profile ID - 13717*/
const Value V15522_1_1 = {S2, NULL};	/*Value = True*/
const Property P15522_1 = {S50, 0, &V15522_1_1, NULL};	/*Property = IsMobile*/
const Value V15522_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15522_5 = {S52, 0, &V15522_5_1233, &P15522_1};	/*Property = ScreenPixelsHeight*/
const Value V15522_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15522_4 = {S54, 0, &V15522_4_1235, &P15522_5};	/*Property = ScreenPixelsWidth*/
const Profile P15522 = {"15522", &P15522_4};	/*Profile ID - 15522*/
const Value V18098_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_279 = {S1, 0, &V18098_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18098_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_282 = {S3, 0, &V18098_282_57155, &P18098_279};	/*Property = BlobBuilder*/
const Value V18098_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_350 = {S4, 0, &V18098_350_57446, &P18098_282};	/*Property = Canvas*/
const Value V18098_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_283 = {S5, 0, &V18098_283_57161, &P18098_350};	/*Property = CssBackground*/
const Value V18098_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_284 = {S6, 0, &V18098_284_57164, &P18098_283};	/*Property = CssBorderImage*/
const Value V18098_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_285 = {S7, 0, &V18098_285_57167, &P18098_284};	/*Property = CssCanvas*/
const Value V18098_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_286 = {S8, 0, &V18098_286_57170, &P18098_285};	/*Property = CssColor*/
const Value V18098_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_237 = {S9, 0, &V18098_237_55786, &P18098_286};	/*Property = CssColumn*/
const Value V18098_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_287 = {S10, 0, &V18098_287_57173, &P18098_237};	/*Property = CssFlexbox*/
const Value V18098_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_288 = {S11, 0, &V18098_288_57176, &P18098_287};	/*Property = CssFont*/
const Value V18098_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_289 = {S12, 0, &V18098_289_86899, &P18098_288};	/*Property = CssImages*/
const Value V18098_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_291 = {S13, 0, &V18098_291_57182, &P18098_289};	/*Property = CssMediaQueries*/
const Value V18098_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_292 = {S14, 0, &V18098_292_57185, &P18098_291};	/*Property = CssMinMax*/
const Value V18098_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_293 = {S15, 0, &V18098_293_57188, &P18098_292};	/*Property = CssOverflow*/
const Value V18098_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_294 = {S16, 0, &V18098_294_57191, &P18098_293};	/*Property = CssPosition*/
const Value V18098_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_295 = {S17, 0, &V18098_295_57194, &P18098_294};	/*Property = CssText*/
const Value V18098_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_297 = {S18, 0, &V18098_297_57197, &P18098_295};	/*Property = CssTransforms*/
const Value V18098_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_241 = {S19, 0, &V18098_241_55801, &P18098_297};	/*Property = CssTransitions*/
const Value V18098_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_298 = {S20, 0, &V18098_298_57208, &P18098_241};	/*Property = CssUI*/
const Value V18098_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_300 = {S21, 0, &V18098_300_57217, &P18098_298};	/*Property = DataSet*/
const Value V18098_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_301 = {S22, 0, &V18098_301_57220, &P18098_300};	/*Property = DataUrl*/
const Value V18098_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_302 = {S23, 0, &V18098_302_57223, &P18098_301};	/*Property = DeviceOrientation*/
const Value V18098_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_304 = {S24, 0, &V18098_304_57229, &P18098_302};	/*Property = FileReader*/
const Value V18098_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_305 = {S25, 0, &V18098_305_57232, &P18098_304};	/*Property = FileSaver*/
const Value V18098_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_306 = {S26, 0, &V18098_306_57238, &P18098_305};	/*Property = FileWriter*/
const Value V18098_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_307 = {S27, 0, &V18098_307_57241, &P18098_306};	/*Property = FormData*/
const Value V18098_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_309 = {S28, 0, &V18098_309_57247, &P18098_307};	/*Property = Fullscreen*/
const Value V18098_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_310 = {S29, 0, &V18098_310_57250, &P18098_309};	/*Property = GeoLocation*/
const Value V18098_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_312 = {S30, 0, &V18098_312_57256, &P18098_310};	/*Property = History*/
const Value V18098_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_314 = {S31, 0, &V18098_314_57262, &P18098_312};	/*Property = Html5*/
const Value V18098_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_313 = {S32, 0, &V18098_313_57259, &P18098_314};	/*Property = Html-Media-Capture*/
const Value V18098_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_315 = {S33, 0, &V18098_315_57265, &P18098_313};	/*Property = Iframe*/
const Value V18098_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_316 = {S34, 0, &V18098_316_57268, &P18098_315};	/*Property = IndexedDB*/
const Value V18098_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_317 = {S35, 0, &V18098_317_57271, &P18098_316};	/*Property = Json*/
const Value V18098_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18098_15 = {S37, 0, &V18098_15_31, &P18098_317};	/*Property = LayoutEngine*/
const Value V18098_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_344 = {S38, 0, &V18098_344_57274, &P18098_15};	/*Property = Masking*/
const Value V18098_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_320 = {S39, 0, &V18098_320_57292, &P18098_344};	/*Property = PostMessage*/
const Value V18098_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_321 = {S40, 0, &V18098_321_57295, &P18098_320};	/*Property = Progress*/
const Value V18098_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_322 = {S41, 0, &V18098_322_57298, &P18098_321};	/*Property = Prompts*/
const Value V18098_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_323 = {S42, 0, &V18098_323_57301, &P18098_322};	/*Property = Selector*/
const Value V18098_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_324 = {S43, 0, &V18098_324_57307, &P18098_323};	/*Property = Svg*/
const Value V18098_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_327 = {S44, 0, &V18098_327_57316, &P18098_324};	/*Property = TouchEvents*/
const Value V18098_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_328 = {S45, 0, &V18098_328_57319, &P18098_327};	/*Property = Track*/
const Value V18098_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_329 = {S46, 0, &V18098_329_57325, &P18098_328};	/*Property = Video*/
const Value V18098_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_330 = {S47, 0, &V18098_330_57328, &P18098_329};	/*Property = Viewport*/
const Value V18098_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_332 = {S48, 0, &V18098_332_57340, &P18098_330};	/*Property = WebWorkers*/
const Value V18098_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18098_333 = {S49, 0, &V18098_333_57343, &P18098_332};	/*Property = Xhr2*/
const Profile P18098 = {"18098", &P18098_333};	/*Profile ID - 18098*/
const Value V18333_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_279 = {S1, 0, &V18333_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18333_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_282 = {S3, 0, &V18333_282_57155, &P18333_279};	/*Property = BlobBuilder*/
const Value V18333_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_350 = {S4, 0, &V18333_350_57446, &P18333_282};	/*Property = Canvas*/
const Value V18333_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_283 = {S5, 0, &V18333_283_57161, &P18333_350};	/*Property = CssBackground*/
const Value V18333_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_284 = {S6, 0, &V18333_284_57164, &P18333_283};	/*Property = CssBorderImage*/
const Value V18333_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_285 = {S7, 0, &V18333_285_57167, &P18333_284};	/*Property = CssCanvas*/
const Value V18333_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_286 = {S8, 0, &V18333_286_57170, &P18333_285};	/*Property = CssColor*/
const Value V18333_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_237 = {S9, 0, &V18333_237_55786, &P18333_286};	/*Property = CssColumn*/
const Value V18333_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_287 = {S10, 0, &V18333_287_57173, &P18333_237};	/*Property = CssFlexbox*/
const Value V18333_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_288 = {S11, 0, &V18333_288_57176, &P18333_287};	/*Property = CssFont*/
const Value V18333_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_289 = {S12, 0, &V18333_289_86899, &P18333_288};	/*Property = CssImages*/
const Value V18333_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_291 = {S13, 0, &V18333_291_57182, &P18333_289};	/*Property = CssMediaQueries*/
const Value V18333_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_292 = {S14, 0, &V18333_292_57185, &P18333_291};	/*Property = CssMinMax*/
const Value V18333_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_293 = {S15, 0, &V18333_293_57188, &P18333_292};	/*Property = CssOverflow*/
const Value V18333_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_294 = {S16, 0, &V18333_294_57191, &P18333_293};	/*Property = CssPosition*/
const Value V18333_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_295 = {S17, 0, &V18333_295_57194, &P18333_294};	/*Property = CssText*/
const Value V18333_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_297 = {S18, 0, &V18333_297_57197, &P18333_295};	/*Property = CssTransforms*/
const Value V18333_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_241 = {S19, 0, &V18333_241_55801, &P18333_297};	/*Property = CssTransitions*/
const Value V18333_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_298 = {S20, 0, &V18333_298_57208, &P18333_241};	/*Property = CssUI*/
const Value V18333_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_300 = {S21, 0, &V18333_300_57217, &P18333_298};	/*Property = DataSet*/
const Value V18333_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_301 = {S22, 0, &V18333_301_57220, &P18333_300};	/*Property = DataUrl*/
const Value V18333_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_302 = {S23, 0, &V18333_302_57223, &P18333_301};	/*Property = DeviceOrientation*/
const Value V18333_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_304 = {S24, 0, &V18333_304_57229, &P18333_302};	/*Property = FileReader*/
const Value V18333_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_305 = {S25, 0, &V18333_305_57232, &P18333_304};	/*Property = FileSaver*/
const Value V18333_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_306 = {S26, 0, &V18333_306_57238, &P18333_305};	/*Property = FileWriter*/
const Value V18333_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_307 = {S27, 0, &V18333_307_57241, &P18333_306};	/*Property = FormData*/
const Value V18333_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_309 = {S28, 0, &V18333_309_57247, &P18333_307};	/*Property = Fullscreen*/
const Value V18333_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_310 = {S29, 0, &V18333_310_57250, &P18333_309};	/*Property = GeoLocation*/
const Value V18333_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_312 = {S30, 0, &V18333_312_57256, &P18333_310};	/*Property = History*/
const Value V18333_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_314 = {S31, 0, &V18333_314_57262, &P18333_312};	/*Property = Html5*/
const Value V18333_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_313 = {S32, 0, &V18333_313_57259, &P18333_314};	/*Property = Html-Media-Capture*/
const Value V18333_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_315 = {S33, 0, &V18333_315_57265, &P18333_313};	/*Property = Iframe*/
const Value V18333_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_316 = {S34, 0, &V18333_316_57268, &P18333_315};	/*Property = IndexedDB*/
const Value V18333_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_317 = {S35, 0, &V18333_317_57271, &P18333_316};	/*Property = Json*/
const Value V18333_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_15 = {S37, 0, &V18333_15_72, &P18333_317};	/*Property = LayoutEngine*/
const Value V18333_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_344 = {S38, 0, &V18333_344_57274, &P18333_15};	/*Property = Masking*/
const Value V18333_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_320 = {S39, 0, &V18333_320_57292, &P18333_344};	/*Property = PostMessage*/
const Value V18333_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_321 = {S40, 0, &V18333_321_57295, &P18333_320};	/*Property = Progress*/
const Value V18333_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_322 = {S41, 0, &V18333_322_57298, &P18333_321};	/*Property = Prompts*/
const Value V18333_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_323 = {S42, 0, &V18333_323_57301, &P18333_322};	/*Property = Selector*/
const Value V18333_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_324 = {S43, 0, &V18333_324_57307, &P18333_323};	/*Property = Svg*/
const Value V18333_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_327 = {S44, 0, &V18333_327_57316, &P18333_324};	/*Property = TouchEvents*/
const Value V18333_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_328 = {S45, 0, &V18333_328_57319, &P18333_327};	/*Property = Track*/
const Value V18333_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_329 = {S46, 0, &V18333_329_57325, &P18333_328};	/*Property = Video*/
const Value V18333_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_330 = {S47, 0, &V18333_330_57328, &P18333_329};	/*Property = Viewport*/
const Value V18333_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_332 = {S48, 0, &V18333_332_57340, &P18333_330};	/*Property = WebWorkers*/
const Value V18333_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18333_333 = {S49, 0, &V18333_333_57343, &P18333_332};	/*Property = Xhr2*/
const Profile P18333 = {"18333", &P18333_333};	/*Profile ID - 18333*/
const Value V12052_1_1 = {S2, NULL};	/*Value = True*/
const Property P12052_1 = {S50, 0, &V12052_1_1, NULL};	/*Property = IsMobile*/
const Value V12052_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12052_5 = {S52, 0, &V12052_5_1293, &P12052_1};	/*Property = ScreenPixelsHeight*/
const Value V12052_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12052_4 = {S54, 0, &V12052_4_1296, &P12052_5};	/*Property = ScreenPixelsWidth*/
const Profile P12052 = {"12052", &P12052_4};	/*Profile ID - 12052*/
const Value V15525_1_1 = {S2, NULL};	/*Value = True*/
const Property P15525_1 = {S50, 0, &V15525_1_1, NULL};	/*Property = IsMobile*/
const Value V15525_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15525_5 = {S52, 0, &V15525_5_1222, &P15525_1};	/*Property = ScreenPixelsHeight*/
const Value V15525_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15525_4 = {S54, 0, &V15525_4_1224, &P15525_5};	/*Property = ScreenPixelsWidth*/
const Profile P15525 = {"15525", &P15525_4};	/*Profile ID - 15525*/
const Value V15524_1_1 = {S2, NULL};	/*Value = True*/
const Property P15524_1 = {S50, 0, &V15524_1_1, NULL};	/*Property = IsMobile*/
const Value V15524_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15524_5 = {S52, 0, &V15524_5_1233, &P15524_1};	/*Property = ScreenPixelsHeight*/
const Value V15524_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15524_4 = {S54, 0, &V15524_4_1296, &P15524_5};	/*Property = ScreenPixelsWidth*/
const Profile P15524 = {"15524", &P15524_4};	/*Profile ID - 15524*/
const Value V16589_1_1 = {S2, NULL};	/*Value = True*/
const Property P16589_1 = {S50, 0, &V16589_1_1, NULL};	/*Property = IsMobile*/
const Value V16589_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P16589_5 = {S52, 0, &V16589_5_1458, &P16589_1};	/*Property = ScreenPixelsHeight*/
const Value V16589_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16589_4 = {S54, 0, &V16589_4_1229, &P16589_5};	/*Property = ScreenPixelsWidth*/
const Profile P16589 = {"16589", &P16589_4};	/*Profile ID - 16589*/
const Value V15529_1_1 = {S2, NULL};	/*Value = True*/
const Property P15529_1 = {S50, 0, &V15529_1_1, NULL};	/*Property = IsMobile*/
const Value V15529_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15529_5 = {S52, 0, &V15529_5_1293, &P15529_1};	/*Property = ScreenPixelsHeight*/
const Value V15529_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15529_4 = {S54, 0, &V15529_4_1296, &P15529_5};	/*Property = ScreenPixelsWidth*/
const Profile P15529 = {"15529", &P15529_4};	/*Profile ID - 15529*/
const Value V15530_1_1 = {S2, NULL};	/*Value = True*/
const Property P15530_1 = {S50, 0, &V15530_1_1, NULL};	/*Property = IsMobile*/
const Value V15530_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15530_5 = {S52, 0, &V15530_5_1380, &P15530_1};	/*Property = ScreenPixelsHeight*/
const Value V15530_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15530_4 = {S54, 0, &V15530_4_1279, &P15530_5};	/*Property = ScreenPixelsWidth*/
const Profile P15530 = {"15530", &P15530_4};	/*Profile ID - 15530*/
const Value V15531_1_1 = {S2, NULL};	/*Value = True*/
const Property P15531_1 = {S50, 0, &V15531_1_1, NULL};	/*Property = IsMobile*/
const Value V15531_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15531_5 = {S52, 0, &V15531_5_1233, &P15531_1};	/*Property = ScreenPixelsHeight*/
const Value V15531_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15531_4 = {S54, 0, &V15531_4_1235, &P15531_5};	/*Property = ScreenPixelsWidth*/
const Profile P15531 = {"15531", &P15531_4};	/*Profile ID - 15531*/
const Value V15532_1_1 = {S2, NULL};	/*Value = True*/
const Property P15532_1 = {S50, 0, &V15532_1_1, NULL};	/*Property = IsMobile*/
const Value V15532_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15532_5 = {S52, 0, &V15532_5_1222, &P15532_1};	/*Property = ScreenPixelsHeight*/
const Value V15532_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15532_4 = {S54, 0, &V15532_4_1224, &P15532_5};	/*Property = ScreenPixelsWidth*/
const Profile P15532 = {"15532", &P15532_4};	/*Profile ID - 15532*/
const Value V15536_1_1 = {S2, NULL};	/*Value = True*/
const Property P15536_1 = {S50, 0, &V15536_1_1, NULL};	/*Property = IsMobile*/
const Value V15536_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15536_5 = {S52, 0, &V15536_5_1293, &P15536_1};	/*Property = ScreenPixelsHeight*/
const Value V15536_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15536_4 = {S54, 0, &V15536_4_1296, &P15536_5};	/*Property = ScreenPixelsWidth*/
const Profile P15536 = {"15536", &P15536_4};	/*Profile ID - 15536*/
const Value V15537_1_1 = {S2, NULL};	/*Value = True*/
const Property P15537_1 = {S50, 0, &V15537_1_1, NULL};	/*Property = IsMobile*/
const Value V15537_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15537_5 = {S52, 0, &V15537_5_1293, &P15537_1};	/*Property = ScreenPixelsHeight*/
const Value V15537_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15537_4 = {S54, 0, &V15537_4_1296, &P15537_5};	/*Property = ScreenPixelsWidth*/
const Profile P15537 = {"15537", &P15537_4};	/*Profile ID - 15537*/
const Value V15540_1_1 = {S2, NULL};	/*Value = True*/
const Property P15540_1 = {S50, 0, &V15540_1_1, NULL};	/*Property = IsMobile*/
const Value V15540_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15540_5 = {S52, 0, &V15540_5_1380, &P15540_1};	/*Property = ScreenPixelsHeight*/
const Value V15540_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15540_4 = {S54, 0, &V15540_4_1279, &P15540_5};	/*Property = ScreenPixelsWidth*/
const Profile P15540 = {"15540", &P15540_4};	/*Profile ID - 15540*/
const Value V15171_1_1 = {S2, NULL};	/*Value = True*/
const Property P15171_1 = {S50, 0, &V15171_1_1, NULL};	/*Property = IsMobile*/
const Value V15171_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15171_5 = {S52, 0, &V15171_5_1293, &P15171_1};	/*Property = ScreenPixelsHeight*/
const Value V15171_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15171_4 = {S54, 0, &V15171_4_1296, &P15171_5};	/*Property = ScreenPixelsWidth*/
const Profile P15171 = {"15171", &P15171_4};	/*Profile ID - 15171*/
const Value V15542_1_1 = {S2, NULL};	/*Value = True*/
const Property P15542_1 = {S50, 0, &V15542_1_1, NULL};	/*Property = IsMobile*/
const Value V15542_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15542_5 = {S52, 0, &V15542_5_1222, &P15542_1};	/*Property = ScreenPixelsHeight*/
const Value V15542_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15542_4 = {S54, 0, &V15542_4_1224, &P15542_5};	/*Property = ScreenPixelsWidth*/
const Profile P15542 = {"15542", &P15542_4};	/*Profile ID - 15542*/
const Value V12053_1_1 = {S2, NULL};	/*Value = True*/
const Property P12053_1 = {S50, 0, &V12053_1_1, NULL};	/*Property = IsMobile*/
const Value V12053_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12053_5 = {S52, 0, &V12053_5_1293, &P12053_1};	/*Property = ScreenPixelsHeight*/
const Value V12053_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12053_4 = {S54, 0, &V12053_4_1296, &P12053_5};	/*Property = ScreenPixelsWidth*/
const Profile P12053 = {"12053", &P12053_4};	/*Profile ID - 12053*/
const Value V13215_1_1 = {S2, NULL};	/*Value = True*/
const Property P13215_1 = {S50, 0, &V13215_1_1, NULL};	/*Property = IsMobile*/
const Value V13215_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13215_5 = {S52, 0, &V13215_5_1277, &P13215_1};	/*Property = ScreenPixelsHeight*/
const Value V13215_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13215_4 = {S54, 0, &V13215_4_1279, &P13215_5};	/*Property = ScreenPixelsWidth*/
const Profile P13215 = {"13215", &P13215_4};	/*Profile ID - 13215*/
const Value V15543_1_1 = {S2, NULL};	/*Value = True*/
const Property P15543_1 = {S50, 0, &V15543_1_1, NULL};	/*Property = IsMobile*/
const Value V15543_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15543_5 = {S52, 0, &V15543_5_1294, &P15543_1};	/*Property = ScreenPixelsHeight*/
const Value V15543_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15543_4 = {S54, 0, &V15543_4_1296, &P15543_5};	/*Property = ScreenPixelsWidth*/
const Profile P15543 = {"15543", &P15543_4};	/*Profile ID - 15543*/
const Value V4780_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_279 = {S1, 0, &V4780_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4780_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_282 = {S3, 0, &V4780_282_57155, &P4780_279};	/*Property = BlobBuilder*/
const Value V4780_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_350 = {S4, 0, &V4780_350_57446, &P4780_282};	/*Property = Canvas*/
const Value V4780_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_283 = {S5, 0, &V4780_283_57161, &P4780_350};	/*Property = CssBackground*/
const Value V4780_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_284 = {S6, 0, &V4780_284_57164, &P4780_283};	/*Property = CssBorderImage*/
const Value V4780_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_285 = {S7, 0, &V4780_285_57167, &P4780_284};	/*Property = CssCanvas*/
const Value V4780_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_286 = {S8, 0, &V4780_286_57170, &P4780_285};	/*Property = CssColor*/
const Value V4780_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_237 = {S9, 0, &V4780_237_55786, &P4780_286};	/*Property = CssColumn*/
const Value V4780_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_287 = {S10, 0, &V4780_287_57173, &P4780_237};	/*Property = CssFlexbox*/
const Value V4780_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_288 = {S11, 0, &V4780_288_57176, &P4780_287};	/*Property = CssFont*/
const Value V4780_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_289 = {S12, 0, &V4780_289_86899, &P4780_288};	/*Property = CssImages*/
const Value V4780_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_291 = {S13, 0, &V4780_291_57182, &P4780_289};	/*Property = CssMediaQueries*/
const Value V4780_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_292 = {S14, 0, &V4780_292_57185, &P4780_291};	/*Property = CssMinMax*/
const Value V4780_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_293 = {S15, 0, &V4780_293_57188, &P4780_292};	/*Property = CssOverflow*/
const Value V4780_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_294 = {S16, 0, &V4780_294_57191, &P4780_293};	/*Property = CssPosition*/
const Value V4780_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_295 = {S17, 0, &V4780_295_57194, &P4780_294};	/*Property = CssText*/
const Value V4780_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_297 = {S18, 0, &V4780_297_57197, &P4780_295};	/*Property = CssTransforms*/
const Value V4780_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_241 = {S19, 0, &V4780_241_55801, &P4780_297};	/*Property = CssTransitions*/
const Value V4780_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_298 = {S20, 0, &V4780_298_57208, &P4780_241};	/*Property = CssUI*/
const Value V4780_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_300 = {S21, 0, &V4780_300_57217, &P4780_298};	/*Property = DataSet*/
const Value V4780_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_301 = {S22, 0, &V4780_301_57220, &P4780_300};	/*Property = DataUrl*/
const Value V4780_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_302 = {S23, 0, &V4780_302_57223, &P4780_301};	/*Property = DeviceOrientation*/
const Value V4780_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_304 = {S24, 0, &V4780_304_57229, &P4780_302};	/*Property = FileReader*/
const Value V4780_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_305 = {S25, 0, &V4780_305_57232, &P4780_304};	/*Property = FileSaver*/
const Value V4780_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_306 = {S26, 0, &V4780_306_57238, &P4780_305};	/*Property = FileWriter*/
const Value V4780_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_307 = {S27, 0, &V4780_307_57241, &P4780_306};	/*Property = FormData*/
const Value V4780_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_309 = {S28, 0, &V4780_309_57247, &P4780_307};	/*Property = Fullscreen*/
const Value V4780_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_310 = {S29, 0, &V4780_310_57250, &P4780_309};	/*Property = GeoLocation*/
const Value V4780_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_312 = {S30, 0, &V4780_312_57256, &P4780_310};	/*Property = History*/
const Value V4780_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_314 = {S31, 0, &V4780_314_57262, &P4780_312};	/*Property = Html5*/
const Value V4780_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_313 = {S32, 0, &V4780_313_57259, &P4780_314};	/*Property = Html-Media-Capture*/
const Value V4780_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_315 = {S33, 0, &V4780_315_57265, &P4780_313};	/*Property = Iframe*/
const Value V4780_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_316 = {S34, 0, &V4780_316_57268, &P4780_315};	/*Property = IndexedDB*/
const Value V4780_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_317 = {S35, 0, &V4780_317_57271, &P4780_316};	/*Property = Json*/
const Value V4780_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_15 = {S37, 0, &V4780_15_72, &P4780_317};	/*Property = LayoutEngine*/
const Value V4780_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_344 = {S38, 0, &V4780_344_57274, &P4780_15};	/*Property = Masking*/
const Value V4780_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_320 = {S39, 0, &V4780_320_57292, &P4780_344};	/*Property = PostMessage*/
const Value V4780_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_321 = {S40, 0, &V4780_321_57295, &P4780_320};	/*Property = Progress*/
const Value V4780_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_322 = {S41, 0, &V4780_322_57298, &P4780_321};	/*Property = Prompts*/
const Value V4780_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_323 = {S42, 0, &V4780_323_57301, &P4780_322};	/*Property = Selector*/
const Value V4780_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_324 = {S43, 0, &V4780_324_57307, &P4780_323};	/*Property = Svg*/
const Value V4780_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_327 = {S44, 0, &V4780_327_57316, &P4780_324};	/*Property = TouchEvents*/
const Value V4780_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_328 = {S45, 0, &V4780_328_57319, &P4780_327};	/*Property = Track*/
const Value V4780_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_329 = {S46, 0, &V4780_329_57325, &P4780_328};	/*Property = Video*/
const Value V4780_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_330 = {S47, 0, &V4780_330_57328, &P4780_329};	/*Property = Viewport*/
const Value V4780_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_332 = {S48, 0, &V4780_332_57340, &P4780_330};	/*Property = WebWorkers*/
const Value V4780_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4780_333 = {S49, 0, &V4780_333_57343, &P4780_332};	/*Property = Xhr2*/
const Profile P4780 = {"4780", &P4780_333};	/*Profile ID - 4780*/
const Value V15544_1_1 = {S2, NULL};	/*Value = True*/
const Property P15544_1 = {S50, 0, &V15544_1_1, NULL};	/*Property = IsMobile*/
const Value V15544_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15544_5 = {S52, 0, &V15544_5_1380, &P15544_1};	/*Property = ScreenPixelsHeight*/
const Value V15544_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15544_4 = {S54, 0, &V15544_4_1279, &P15544_5};	/*Property = ScreenPixelsWidth*/
const Profile P15544 = {"15544", &P15544_4};	/*Profile ID - 15544*/
const Value V14364_1_1 = {S2, NULL};	/*Value = True*/
const Property P14364_1 = {S50, 0, &V14364_1_1, NULL};	/*Property = IsMobile*/
const Value V14364_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14364_5 = {S52, 0, &V14364_5_1293, &P14364_1};	/*Property = ScreenPixelsHeight*/
const Value V14364_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14364_4 = {S54, 0, &V14364_4_1296, &P14364_5};	/*Property = ScreenPixelsWidth*/
const Profile P14364 = {"14364", &P14364_4};	/*Profile ID - 14364*/
const Value V15546_1_1 = {S2, NULL};	/*Value = True*/
const Property P15546_1 = {S50, 0, &V15546_1_1, NULL};	/*Property = IsMobile*/
const Value V15546_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15546_5 = {S52, 0, &V15546_5_1380, &P15546_1};	/*Property = ScreenPixelsHeight*/
const Value V15546_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15546_4 = {S54, 0, &V15546_4_1279, &P15546_5};	/*Property = ScreenPixelsWidth*/
const Profile P15546 = {"15546", &P15546_4};	/*Profile ID - 15546*/
const Value V15548_1_1 = {S2, NULL};	/*Value = True*/
const Property P15548_1 = {S50, 0, &V15548_1_1, NULL};	/*Property = IsMobile*/
const Value V15548_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15548_5 = {S52, 0, &V15548_5_1233, &P15548_1};	/*Property = ScreenPixelsHeight*/
const Value V15548_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15548_4 = {S54, 0, &V15548_4_1235, &P15548_5};	/*Property = ScreenPixelsWidth*/
const Profile P15548 = {"15548", &P15548_4};	/*Profile ID - 15548*/
const Value V15551_1_1 = {S2, NULL};	/*Value = True*/
const Property P15551_1 = {S50, 0, &V15551_1_1, NULL};	/*Property = IsMobile*/
const Value V15551_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15551_5 = {S52, 0, &V15551_5_1233, &P15551_1};	/*Property = ScreenPixelsHeight*/
const Value V15551_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15551_4 = {S54, 0, &V15551_4_1235, &P15551_5};	/*Property = ScreenPixelsWidth*/
const Profile P15551 = {"15551", &P15551_4};	/*Profile ID - 15551*/
const Value V13816_1_1 = {S2, NULL};	/*Value = True*/
const Property P13816_1 = {S50, 0, &V13816_1_1, NULL};	/*Property = IsMobile*/
const Value V13816_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13816_5 = {S52, 0, &V13816_5_1293, &P13816_1};	/*Property = ScreenPixelsHeight*/
const Value V13816_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13816_4 = {S54, 0, &V13816_4_1296, &P13816_5};	/*Property = ScreenPixelsWidth*/
const Profile P13816 = {"13816", &P13816_4};	/*Profile ID - 13816*/
const Value V14463_1_1 = {S2, NULL};	/*Value = True*/
const Property P14463_1 = {S50, 0, &V14463_1_1, NULL};	/*Property = IsMobile*/
const Value V14463_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14463_5 = {S52, 0, &V14463_5_1293, &P14463_1};	/*Property = ScreenPixelsHeight*/
const Value V14463_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14463_4 = {S54, 0, &V14463_4_1296, &P14463_5};	/*Property = ScreenPixelsWidth*/
const Profile P14463 = {"14463", &P14463_4};	/*Profile ID - 14463*/
const Value V15554_1_1 = {S2, NULL};	/*Value = True*/
const Property P15554_1 = {S50, 0, &V15554_1_1, NULL};	/*Property = IsMobile*/
const Value V15554_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15554_5 = {S52, 0, &V15554_5_1277, &P15554_1};	/*Property = ScreenPixelsHeight*/
const Value V15554_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15554_4 = {S54, 0, &V15554_4_1279, &P15554_5};	/*Property = ScreenPixelsWidth*/
const Profile P15554 = {"15554", &P15554_4};	/*Profile ID - 15554*/
const Value V15555_1_1 = {S2, NULL};	/*Value = True*/
const Property P15555_1 = {S50, 0, &V15555_1_1, NULL};	/*Property = IsMobile*/
const Value V15555_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15555_5 = {S52, 0, &V15555_5_1227, &P15555_1};	/*Property = ScreenPixelsHeight*/
const Value V15555_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15555_4 = {S54, 0, &V15555_4_1229, &P15555_5};	/*Property = ScreenPixelsWidth*/
const Profile P15555 = {"15555", &P15555_4};	/*Profile ID - 15555*/
const Value V20918_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_279 = {S1, 0, &V20918_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20918_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_282 = {S3, 0, &V20918_282_57155, &P20918_279};	/*Property = BlobBuilder*/
const Value V20918_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_350 = {S4, 0, &V20918_350_57446, &P20918_282};	/*Property = Canvas*/
const Value V20918_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_283 = {S5, 0, &V20918_283_57161, &P20918_350};	/*Property = CssBackground*/
const Value V20918_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_284 = {S6, 0, &V20918_284_57164, &P20918_283};	/*Property = CssBorderImage*/
const Value V20918_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_285 = {S7, 0, &V20918_285_57167, &P20918_284};	/*Property = CssCanvas*/
const Value V20918_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_286 = {S8, 0, &V20918_286_57170, &P20918_285};	/*Property = CssColor*/
const Value V20918_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_237 = {S9, 0, &V20918_237_55786, &P20918_286};	/*Property = CssColumn*/
const Value V20918_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_287 = {S10, 0, &V20918_287_57173, &P20918_237};	/*Property = CssFlexbox*/
const Value V20918_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_288 = {S11, 0, &V20918_288_57176, &P20918_287};	/*Property = CssFont*/
const Value V20918_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_289 = {S12, 0, &V20918_289_86899, &P20918_288};	/*Property = CssImages*/
const Value V20918_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_291 = {S13, 0, &V20918_291_57182, &P20918_289};	/*Property = CssMediaQueries*/
const Value V20918_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_292 = {S14, 0, &V20918_292_57185, &P20918_291};	/*Property = CssMinMax*/
const Value V20918_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_293 = {S15, 0, &V20918_293_57188, &P20918_292};	/*Property = CssOverflow*/
const Value V20918_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_294 = {S16, 0, &V20918_294_57191, &P20918_293};	/*Property = CssPosition*/
const Value V20918_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_295 = {S17, 0, &V20918_295_57194, &P20918_294};	/*Property = CssText*/
const Value V20918_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_297 = {S18, 0, &V20918_297_57197, &P20918_295};	/*Property = CssTransforms*/
const Value V20918_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_241 = {S19, 0, &V20918_241_55801, &P20918_297};	/*Property = CssTransitions*/
const Value V20918_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_298 = {S20, 0, &V20918_298_57208, &P20918_241};	/*Property = CssUI*/
const Value V20918_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_300 = {S21, 0, &V20918_300_57217, &P20918_298};	/*Property = DataSet*/
const Value V20918_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_301 = {S22, 0, &V20918_301_57220, &P20918_300};	/*Property = DataUrl*/
const Value V20918_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_302 = {S23, 0, &V20918_302_57223, &P20918_301};	/*Property = DeviceOrientation*/
const Value V20918_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_304 = {S24, 0, &V20918_304_57229, &P20918_302};	/*Property = FileReader*/
const Value V20918_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_305 = {S25, 0, &V20918_305_57232, &P20918_304};	/*Property = FileSaver*/
const Value V20918_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_306 = {S26, 0, &V20918_306_57238, &P20918_305};	/*Property = FileWriter*/
const Value V20918_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_307 = {S27, 0, &V20918_307_57241, &P20918_306};	/*Property = FormData*/
const Value V20918_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_309 = {S28, 0, &V20918_309_57247, &P20918_307};	/*Property = Fullscreen*/
const Value V20918_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_310 = {S29, 0, &V20918_310_57250, &P20918_309};	/*Property = GeoLocation*/
const Value V20918_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_312 = {S30, 0, &V20918_312_57256, &P20918_310};	/*Property = History*/
const Value V20918_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_314 = {S31, 0, &V20918_314_57262, &P20918_312};	/*Property = Html5*/
const Value V20918_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_313 = {S32, 0, &V20918_313_57259, &P20918_314};	/*Property = Html-Media-Capture*/
const Value V20918_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_315 = {S33, 0, &V20918_315_57265, &P20918_313};	/*Property = Iframe*/
const Value V20918_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_316 = {S34, 0, &V20918_316_57268, &P20918_315};	/*Property = IndexedDB*/
const Value V20918_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_317 = {S35, 0, &V20918_317_57271, &P20918_316};	/*Property = Json*/
const Value V20918_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20918_15 = {S37, 0, &V20918_15_29, &P20918_317};	/*Property = LayoutEngine*/
const Value V20918_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_344 = {S38, 0, &V20918_344_57274, &P20918_15};	/*Property = Masking*/
const Value V20918_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_320 = {S39, 0, &V20918_320_57292, &P20918_344};	/*Property = PostMessage*/
const Value V20918_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_321 = {S40, 0, &V20918_321_57295, &P20918_320};	/*Property = Progress*/
const Value V20918_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_322 = {S41, 0, &V20918_322_57298, &P20918_321};	/*Property = Prompts*/
const Value V20918_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_323 = {S42, 0, &V20918_323_57301, &P20918_322};	/*Property = Selector*/
const Value V20918_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_324 = {S43, 0, &V20918_324_57307, &P20918_323};	/*Property = Svg*/
const Value V20918_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_327 = {S44, 0, &V20918_327_57316, &P20918_324};	/*Property = TouchEvents*/
const Value V20918_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_328 = {S45, 0, &V20918_328_57319, &P20918_327};	/*Property = Track*/
const Value V20918_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_329 = {S46, 0, &V20918_329_57325, &P20918_328};	/*Property = Video*/
const Value V20918_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_330 = {S47, 0, &V20918_330_57328, &P20918_329};	/*Property = Viewport*/
const Value V20918_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_332 = {S48, 0, &V20918_332_57340, &P20918_330};	/*Property = WebWorkers*/
const Value V20918_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20918_333 = {S49, 0, &V20918_333_57343, &P20918_332};	/*Property = Xhr2*/
const Profile P20918 = {"20918", &P20918_333};	/*Profile ID - 20918*/
const Value V20085_1_2 = {S0, NULL};	/*Value = False*/
const Property P20085_1 = {S50, 0, &V20085_1_2, NULL};	/*Property = IsMobile*/
const Value V20085_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20085_5 = {S52, 0, &V20085_5_37, &P20085_1};	/*Property = ScreenPixelsHeight*/
const Value V20085_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20085_4 = {S54, 0, &V20085_4_36, &P20085_5};	/*Property = ScreenPixelsWidth*/
const Profile P20085 = {"20085", &P20085_4};	/*Profile ID - 20085*/
const Value V13877_1_1 = {S2, NULL};	/*Value = True*/
const Property P13877_1 = {S50, 0, &V13877_1_1, NULL};	/*Property = IsMobile*/
const Value V13877_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13877_5 = {S52, 0, &V13877_5_1294, &P13877_1};	/*Property = ScreenPixelsHeight*/
const Value V13877_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13877_4 = {S54, 0, &V13877_4_1296, &P13877_5};	/*Property = ScreenPixelsWidth*/
const Profile P13877 = {"13877", &P13877_4};	/*Profile ID - 13877*/
const Value V15559_1_1 = {S2, NULL};	/*Value = True*/
const Property P15559_1 = {S50, 0, &V15559_1_1, NULL};	/*Property = IsMobile*/
const Value V15559_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15559_5 = {S52, 0, &V15559_5_1293, &P15559_1};	/*Property = ScreenPixelsHeight*/
const Value V15559_4_24031 = {S71, NULL};	/*Value = 120*/
const Property P15559_4 = {S54, 0, &V15559_4_24031, &P15559_5};	/*Property = ScreenPixelsWidth*/
const Profile P15559 = {"15559", &P15559_4};	/*Profile ID - 15559*/
const Value V12865_1_1 = {S2, NULL};	/*Value = True*/
const Property P12865_1 = {S50, 0, &V12865_1_1, NULL};	/*Property = IsMobile*/
const Value V12865_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P12865_5 = {S52, 0, &V12865_5_1333, &P12865_1};	/*Property = ScreenPixelsHeight*/
const Value V12865_4_37572 = {S93, NULL};	/*Value = 132*/
const Property P12865_4 = {S54, 0, &V12865_4_37572, &P12865_5};	/*Property = ScreenPixelsWidth*/
const Profile P12865 = {"12865", &P12865_4};	/*Profile ID - 12865*/
const Value V17909_1_1 = {S2, NULL};	/*Value = True*/
const Property P17909_1 = {S50, 0, &V17909_1_1, NULL};	/*Property = IsMobile*/
const Value V17909_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17909_5 = {S52, 0, &V17909_5_1293, &P17909_1};	/*Property = ScreenPixelsHeight*/
const Value V17909_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17909_4 = {S54, 0, &V17909_4_1296, &P17909_5};	/*Property = ScreenPixelsWidth*/
const Profile P17909 = {"17909", &P17909_4};	/*Profile ID - 17909*/
const Value V15563_1_1 = {S2, NULL};	/*Value = True*/
const Property P15563_1 = {S50, 0, &V15563_1_1, NULL};	/*Property = IsMobile*/
const Value V15563_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15563_5 = {S52, 0, &V15563_5_1222, &P15563_1};	/*Property = ScreenPixelsHeight*/
const Value V15563_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15563_4 = {S54, 0, &V15563_4_1224, &P15563_5};	/*Property = ScreenPixelsWidth*/
const Profile P15563 = {"15563", &P15563_4};	/*Profile ID - 15563*/
const Value V15565_1_1 = {S2, NULL};	/*Value = True*/
const Property P15565_1 = {S50, 0, &V15565_1_1, NULL};	/*Property = IsMobile*/
const Value V15565_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15565_5 = {S52, 0, &V15565_5_1222, &P15565_1};	/*Property = ScreenPixelsHeight*/
const Value V15565_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15565_4 = {S54, 0, &V15565_4_1224, &P15565_5};	/*Property = ScreenPixelsWidth*/
const Profile P15565 = {"15565", &P15565_4};	/*Profile ID - 15565*/
const Value V13768_1_1 = {S2, NULL};	/*Value = True*/
const Property P13768_1 = {S50, 0, &V13768_1_1, NULL};	/*Property = IsMobile*/
const Value V13768_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13768_5 = {S52, 0, &V13768_5_1233, &P13768_1};	/*Property = ScreenPixelsHeight*/
const Value V13768_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13768_4 = {S54, 0, &V13768_4_1235, &P13768_5};	/*Property = ScreenPixelsWidth*/
const Profile P13768 = {"13768", &P13768_4};	/*Profile ID - 13768*/
const Value V15566_1_1 = {S2, NULL};	/*Value = True*/
const Property P15566_1 = {S50, 0, &V15566_1_1, NULL};	/*Property = IsMobile*/
const Value V15566_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15566_5 = {S52, 0, &V15566_5_1293, &P15566_1};	/*Property = ScreenPixelsHeight*/
const Value V15566_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15566_4 = {S54, 0, &V15566_4_1296, &P15566_5};	/*Property = ScreenPixelsWidth*/
const Profile P15566 = {"15566", &P15566_4};	/*Profile ID - 15566*/
const Value V15567_1_1 = {S2, NULL};	/*Value = True*/
const Property P15567_1 = {S50, 0, &V15567_1_1, NULL};	/*Property = IsMobile*/
const Value V15567_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15567_5 = {S52, 0, &V15567_5_1222, &P15567_1};	/*Property = ScreenPixelsHeight*/
const Value V15567_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15567_4 = {S54, 0, &V15567_4_1224, &P15567_5};	/*Property = ScreenPixelsWidth*/
const Profile P15567 = {"15567", &P15567_4};	/*Profile ID - 15567*/
const Value V15568_1_1 = {S2, NULL};	/*Value = True*/
const Property P15568_1 = {S50, 0, &V15568_1_1, NULL};	/*Property = IsMobile*/
const Value V15568_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15568_5 = {S52, 0, &V15568_5_1293, &P15568_1};	/*Property = ScreenPixelsHeight*/
const Value V15568_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15568_4 = {S54, 0, &V15568_4_1296, &P15568_5};	/*Property = ScreenPixelsWidth*/
const Profile P15568 = {"15568", &P15568_4};	/*Profile ID - 15568*/
const Value V15569_1_1 = {S2, NULL};	/*Value = True*/
const Property P15569_1 = {S50, 0, &V15569_1_1, NULL};	/*Property = IsMobile*/
const Value V15569_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15569_5 = {S52, 0, &V15569_5_1233, &P15569_1};	/*Property = ScreenPixelsHeight*/
const Value V15569_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15569_4 = {S54, 0, &V15569_4_1235, &P15569_5};	/*Property = ScreenPixelsWidth*/
const Profile P15569 = {"15569", &P15569_4};	/*Profile ID - 15569*/
const Value V19913_1_1 = {S2, NULL};	/*Value = True*/
const Property P19913_1 = {S50, 0, &V19913_1_1, NULL};	/*Property = IsMobile*/
const Value V19913_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P19913_5 = {S52, 0, &V19913_5_1293, &P19913_1};	/*Property = ScreenPixelsHeight*/
const Value V19913_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P19913_4 = {S54, 0, &V19913_4_1296, &P19913_5};	/*Property = ScreenPixelsWidth*/
const Profile P19913 = {"19913", &P19913_4};	/*Profile ID - 19913*/
const Value V15570_1_1 = {S2, NULL};	/*Value = True*/
const Property P15570_1 = {S50, 0, &V15570_1_1, NULL};	/*Property = IsMobile*/
const Value V15570_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15570_5 = {S52, 0, &V15570_5_1293, &P15570_1};	/*Property = ScreenPixelsHeight*/
const Value V15570_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15570_4 = {S54, 0, &V15570_4_1296, &P15570_5};	/*Property = ScreenPixelsWidth*/
const Profile P15570 = {"15570", &P15570_4};	/*Profile ID - 15570*/
const Value V15571_1_1 = {S2, NULL};	/*Value = True*/
const Property P15571_1 = {S50, 0, &V15571_1_1, NULL};	/*Property = IsMobile*/
const Value V15571_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15571_5 = {S52, 0, &V15571_5_1293, &P15571_1};	/*Property = ScreenPixelsHeight*/
const Value V15571_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15571_4 = {S54, 0, &V15571_4_1296, &P15571_5};	/*Property = ScreenPixelsWidth*/
const Profile P15571 = {"15571", &P15571_4};	/*Profile ID - 15571*/
const Value V15573_1_1 = {S2, NULL};	/*Value = True*/
const Property P15573_1 = {S50, 0, &V15573_1_1, NULL};	/*Property = IsMobile*/
const Value V15573_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15573_5 = {S52, 0, &V15573_5_1293, &P15573_1};	/*Property = ScreenPixelsHeight*/
const Value V15573_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15573_4 = {S54, 0, &V15573_4_1296, &P15573_5};	/*Property = ScreenPixelsWidth*/
const Profile P15573 = {"15573", &P15573_4};	/*Profile ID - 15573*/
const Value V15574_1_1 = {S2, NULL};	/*Value = True*/
const Property P15574_1 = {S50, 0, &V15574_1_1, NULL};	/*Property = IsMobile*/
const Value V15574_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15574_5 = {S52, 0, &V15574_5_1293, &P15574_1};	/*Property = ScreenPixelsHeight*/
const Value V15574_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15574_4 = {S54, 0, &V15574_4_1296, &P15574_5};	/*Property = ScreenPixelsWidth*/
const Profile P15574 = {"15574", &P15574_4};	/*Profile ID - 15574*/
const Value V13838_1_1 = {S2, NULL};	/*Value = True*/
const Property P13838_1 = {S50, 0, &V13838_1_1, NULL};	/*Property = IsMobile*/
const Value V13838_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P13838_5 = {S52, 0, &V13838_5_1280, &P13838_1};	/*Property = ScreenPixelsHeight*/
const Value V13838_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13838_4 = {S54, 0, &V13838_4_1235, &P13838_5};	/*Property = ScreenPixelsWidth*/
const Profile P13838 = {"13838", &P13838_4};	/*Profile ID - 13838*/
const Value V15577_1_1 = {S2, NULL};	/*Value = True*/
const Property P15577_1 = {S50, 0, &V15577_1_1, NULL};	/*Property = IsMobile*/
const Value V15577_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15577_5 = {S52, 0, &V15577_5_1304, &P15577_1};	/*Property = ScreenPixelsHeight*/
const Value V15577_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15577_4 = {S54, 0, &V15577_4_1224, &P15577_5};	/*Property = ScreenPixelsWidth*/
const Profile P15577 = {"15577", &P15577_4};	/*Profile ID - 15577*/
const Value V15578_1_1 = {S2, NULL};	/*Value = True*/
const Property P15578_1 = {S50, 0, &V15578_1_1, NULL};	/*Property = IsMobile*/
const Value V15578_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15578_5 = {S52, 0, &V15578_5_1233, &P15578_1};	/*Property = ScreenPixelsHeight*/
const Value V15578_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15578_4 = {S54, 0, &V15578_4_1235, &P15578_5};	/*Property = ScreenPixelsWidth*/
const Profile P15578 = {"15578", &P15578_4};	/*Profile ID - 15578*/
const Value V15580_1_1 = {S2, NULL};	/*Value = True*/
const Property P15580_1 = {S50, 0, &V15580_1_1, NULL};	/*Property = IsMobile*/
const Value V15580_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15580_5 = {S52, 0, &V15580_5_1222, &P15580_1};	/*Property = ScreenPixelsHeight*/
const Value V15580_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15580_4 = {S54, 0, &V15580_4_1224, &P15580_5};	/*Property = ScreenPixelsWidth*/
const Profile P15580 = {"15580", &P15580_4};	/*Profile ID - 15580*/
const Value V14366_1_1 = {S2, NULL};	/*Value = True*/
const Property P14366_1 = {S50, 0, &V14366_1_1, NULL};	/*Property = IsMobile*/
const Value V14366_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14366_5 = {S52, 0, &V14366_5_1294, &P14366_1};	/*Property = ScreenPixelsHeight*/
const Value V14366_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14366_4 = {S54, 0, &V14366_4_1296, &P14366_5};	/*Property = ScreenPixelsWidth*/
const Profile P14366 = {"14366", &P14366_4};	/*Profile ID - 14366*/
const Value V13257_1_1 = {S2, NULL};	/*Value = True*/
const Property P13257_1 = {S50, 0, &V13257_1_1, NULL};	/*Property = IsMobile*/
const Value V13257_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P13257_5 = {S52, 0, &V13257_5_1269, &P13257_1};	/*Property = ScreenPixelsHeight*/
const Value V13257_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13257_4 = {S54, 0, &V13257_4_1229, &P13257_5};	/*Property = ScreenPixelsWidth*/
const Profile P13257 = {"13257", &P13257_4};	/*Profile ID - 13257*/
const Value V15582_1_1 = {S2, NULL};	/*Value = True*/
const Property P15582_1 = {S50, 0, &V15582_1_1, NULL};	/*Property = IsMobile*/
const Value V15582_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15582_5 = {S52, 0, &V15582_5_1304, &P15582_1};	/*Property = ScreenPixelsHeight*/
const Value V15582_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15582_4 = {S54, 0, &V15582_4_1224, &P15582_5};	/*Property = ScreenPixelsWidth*/
const Profile P15582 = {"15582", &P15582_4};	/*Profile ID - 15582*/
const Value V15583_1_1 = {S2, NULL};	/*Value = True*/
const Property P15583_1 = {S50, 0, &V15583_1_1, NULL};	/*Property = IsMobile*/
const Value V15583_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15583_5 = {S52, 0, &V15583_5_1222, &P15583_1};	/*Property = ScreenPixelsHeight*/
const Value V15583_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15583_4 = {S54, 0, &V15583_4_1224, &P15583_5};	/*Property = ScreenPixelsWidth*/
const Profile P15583 = {"15583", &P15583_4};	/*Profile ID - 15583*/
const Profile P7020 = {"7020", NULL};	/*Profile ID - 7020*/
const Value V15584_1_1 = {S2, NULL};	/*Value = True*/
const Property P15584_1 = {S50, 0, &V15584_1_1, NULL};	/*Property = IsMobile*/
const Value V15584_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15584_5 = {S52, 0, &V15584_5_1304, &P15584_1};	/*Property = ScreenPixelsHeight*/
const Value V15584_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15584_4 = {S54, 0, &V15584_4_1224, &P15584_5};	/*Property = ScreenPixelsWidth*/
const Profile P15584 = {"15584", &P15584_4};	/*Profile ID - 15584*/
const Value V12918_1_1 = {S2, NULL};	/*Value = True*/
const Property P12918_1 = {S50, 0, &V12918_1_1, NULL};	/*Property = IsMobile*/
const Value V12918_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12918_5 = {S52, 0, &V12918_5_1227, &P12918_1};	/*Property = ScreenPixelsHeight*/
const Value V12918_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12918_4 = {S54, 0, &V12918_4_1229, &P12918_5};	/*Property = ScreenPixelsWidth*/
const Profile P12918 = {"12918", &P12918_4};	/*Profile ID - 12918*/
const Value V13290_1_1 = {S2, NULL};	/*Value = True*/
const Property P13290_1 = {S50, 0, &V13290_1_1, NULL};	/*Property = IsMobile*/
const Value V13290_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13290_5 = {S52, 0, &V13290_5_1222, &P13290_1};	/*Property = ScreenPixelsHeight*/
const Value V13290_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13290_4 = {S54, 0, &V13290_4_1224, &P13290_5};	/*Property = ScreenPixelsWidth*/
const Profile P13290 = {"13290", &P13290_4};	/*Profile ID - 13290*/
const Value V12879_1_1 = {S2, NULL};	/*Value = True*/
const Property P12879_1 = {S50, 0, &V12879_1_1, NULL};	/*Property = IsMobile*/
const Value V12879_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12879_5 = {S52, 0, &V12879_5_1227, &P12879_1};	/*Property = ScreenPixelsHeight*/
const Value V12879_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P12879_4 = {S54, 0, &V12879_4_1477, &P12879_5};	/*Property = ScreenPixelsWidth*/
const Profile P12879 = {"12879", &P12879_4};	/*Profile ID - 12879*/
const Value V15586_1_1 = {S2, NULL};	/*Value = True*/
const Property P15586_1 = {S50, 0, &V15586_1_1, NULL};	/*Property = IsMobile*/
const Value V15586_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15586_5 = {S52, 0, &V15586_5_1233, &P15586_1};	/*Property = ScreenPixelsHeight*/
const Value V15586_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15586_4 = {S54, 0, &V15586_4_1235, &P15586_5};	/*Property = ScreenPixelsWidth*/
const Profile P15586 = {"15586", &P15586_4};	/*Profile ID - 15586*/
const Value V15588_1_1 = {S2, NULL};	/*Value = True*/
const Property P15588_1 = {S50, 0, &V15588_1_1, NULL};	/*Property = IsMobile*/
const Value V15588_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15588_5 = {S52, 0, &V15588_5_1222, &P15588_1};	/*Property = ScreenPixelsHeight*/
const Value V15588_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15588_4 = {S54, 0, &V15588_4_1224, &P15588_5};	/*Property = ScreenPixelsWidth*/
const Profile P15588 = {"15588", &P15588_4};	/*Profile ID - 15588*/
const Value V15589_1_1 = {S2, NULL};	/*Value = True*/
const Property P15589_1 = {S50, 0, &V15589_1_1, NULL};	/*Property = IsMobile*/
const Value V15589_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15589_5 = {S52, 0, &V15589_5_1380, &P15589_1};	/*Property = ScreenPixelsHeight*/
const Value V15589_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15589_4 = {S54, 0, &V15589_4_1279, &P15589_5};	/*Property = ScreenPixelsWidth*/
const Profile P15589 = {"15589", &P15589_4};	/*Profile ID - 15589*/
const Value V14362_1_1 = {S2, NULL};	/*Value = True*/
const Property P14362_1 = {S50, 0, &V14362_1_1, NULL};	/*Property = IsMobile*/
const Value V14362_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14362_5 = {S52, 0, &V14362_5_1222, &P14362_1};	/*Property = ScreenPixelsHeight*/
const Value V14362_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14362_4 = {S54, 0, &V14362_4_1224, &P14362_5};	/*Property = ScreenPixelsWidth*/
const Profile P14362 = {"14362", &P14362_4};	/*Profile ID - 14362*/
const Value V15591_1_1 = {S2, NULL};	/*Value = True*/
const Property P15591_1 = {S50, 0, &V15591_1_1, NULL};	/*Property = IsMobile*/
const Value V15591_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15591_5 = {S52, 0, &V15591_5_1233, &P15591_1};	/*Property = ScreenPixelsHeight*/
const Value V15591_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15591_4 = {S54, 0, &V15591_4_1235, &P15591_5};	/*Property = ScreenPixelsWidth*/
const Profile P15591 = {"15591", &P15591_4};	/*Profile ID - 15591*/
const Value V15592_1_1 = {S2, NULL};	/*Value = True*/
const Property P15592_1 = {S50, 0, &V15592_1_1, NULL};	/*Property = IsMobile*/
const Value V15592_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15592_5 = {S52, 0, &V15592_5_1277, &P15592_1};	/*Property = ScreenPixelsHeight*/
const Value V15592_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P15592_4 = {S54, 0, &V15592_4_2336, &P15592_5};	/*Property = ScreenPixelsWidth*/
const Profile P15592 = {"15592", &P15592_4};	/*Profile ID - 15592*/
const Value V15593_1_1 = {S2, NULL};	/*Value = True*/
const Property P15593_1 = {S50, 0, &V15593_1_1, NULL};	/*Property = IsMobile*/
const Value V15593_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15593_5 = {S52, 0, &V15593_5_1304, &P15593_1};	/*Property = ScreenPixelsHeight*/
const Value V15593_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15593_4 = {S54, 0, &V15593_4_1224, &P15593_5};	/*Property = ScreenPixelsWidth*/
const Profile P15593 = {"15593", &P15593_4};	/*Profile ID - 15593*/
const Value V13687_1_1 = {S2, NULL};	/*Value = True*/
const Property P13687_1 = {S50, 0, &V13687_1_1, NULL};	/*Property = IsMobile*/
const Value V13687_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13687_5 = {S52, 0, &V13687_5_1380, &P13687_1};	/*Property = ScreenPixelsHeight*/
const Value V13687_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13687_4 = {S54, 0, &V13687_4_1279, &P13687_5};	/*Property = ScreenPixelsWidth*/
const Profile P13687 = {"13687", &P13687_4};	/*Profile ID - 13687*/
const Value V15594_1_1 = {S2, NULL};	/*Value = True*/
const Property P15594_1 = {S50, 0, &V15594_1_1, NULL};	/*Property = IsMobile*/
const Value V15594_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15594_5 = {S52, 0, &V15594_5_1277, &P15594_1};	/*Property = ScreenPixelsHeight*/
const Value V15594_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15594_4 = {S54, 0, &V15594_4_1279, &P15594_5};	/*Property = ScreenPixelsWidth*/
const Profile P15594 = {"15594", &P15594_4};	/*Profile ID - 15594*/
const Value V15597_1_1 = {S2, NULL};	/*Value = True*/
const Property P15597_1 = {S50, 0, &V15597_1_1, NULL};	/*Property = IsMobile*/
const Value V15597_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15597_5 = {S52, 0, &V15597_5_1293, &P15597_1};	/*Property = ScreenPixelsHeight*/
const Value V15597_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15597_4 = {S54, 0, &V15597_4_1296, &P15597_5};	/*Property = ScreenPixelsWidth*/
const Profile P15597 = {"15597", &P15597_4};	/*Profile ID - 15597*/
const Value V15598_1_1 = {S2, NULL};	/*Value = True*/
const Property P15598_1 = {S50, 0, &V15598_1_1, NULL};	/*Property = IsMobile*/
const Value V15598_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15598_5 = {S52, 0, &V15598_5_1293, &P15598_1};	/*Property = ScreenPixelsHeight*/
const Value V15598_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15598_4 = {S54, 0, &V15598_4_1296, &P15598_5};	/*Property = ScreenPixelsWidth*/
const Profile P15598 = {"15598", &P15598_4};	/*Profile ID - 15598*/
const Value V15600_1_1 = {S2, NULL};	/*Value = True*/
const Property P15600_1 = {S50, 0, &V15600_1_1, NULL};	/*Property = IsMobile*/
const Value V15600_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15600_5 = {S52, 0, &V15600_5_1233, &P15600_1};	/*Property = ScreenPixelsHeight*/
const Value V15600_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15600_4 = {S54, 0, &V15600_4_1235, &P15600_5};	/*Property = ScreenPixelsWidth*/
const Profile P15600 = {"15600", &P15600_4};	/*Profile ID - 15600*/
const Value V15601_1_1 = {S2, NULL};	/*Value = True*/
const Property P15601_1 = {S50, 0, &V15601_1_1, NULL};	/*Property = IsMobile*/
const Value V15601_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15601_5 = {S52, 0, &V15601_5_1233, &P15601_1};	/*Property = ScreenPixelsHeight*/
const Value V15601_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15601_4 = {S54, 0, &V15601_4_1235, &P15601_5};	/*Property = ScreenPixelsWidth*/
const Profile P15601 = {"15601", &P15601_4};	/*Profile ID - 15601*/
const Value V15602_1_1 = {S2, NULL};	/*Value = True*/
const Property P15602_1 = {S50, 0, &V15602_1_1, NULL};	/*Property = IsMobile*/
const Value V15602_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15602_5 = {S52, 0, &V15602_5_1227, &P15602_1};	/*Property = ScreenPixelsHeight*/
const Value V15602_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P15602_4 = {S54, 0, &V15602_4_1477, &P15602_5};	/*Property = ScreenPixelsWidth*/
const Profile P15602 = {"15602", &P15602_4};	/*Profile ID - 15602*/
const Value V12065_1_1 = {S2, NULL};	/*Value = True*/
const Property P12065_1 = {S50, 0, &V12065_1_1, NULL};	/*Property = IsMobile*/
const Value V12065_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12065_5 = {S52, 0, &V12065_5_1222, &P12065_1};	/*Property = ScreenPixelsHeight*/
const Value V12065_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12065_4 = {S54, 0, &V12065_4_1224, &P12065_5};	/*Property = ScreenPixelsWidth*/
const Profile P12065 = {"12065", &P12065_4};	/*Profile ID - 12065*/
const Value V15603_1_1 = {S2, NULL};	/*Value = True*/
const Property P15603_1 = {S50, 0, &V15603_1_1, NULL};	/*Property = IsMobile*/
const Value V15603_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15603_5 = {S52, 0, &V15603_5_1233, &P15603_1};	/*Property = ScreenPixelsHeight*/
const Value V15603_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15603_4 = {S54, 0, &V15603_4_1235, &P15603_5};	/*Property = ScreenPixelsWidth*/
const Profile P15603 = {"15603", &P15603_4};	/*Profile ID - 15603*/
const Value V15604_1_1 = {S2, NULL};	/*Value = True*/
const Property P15604_1 = {S50, 0, &V15604_1_1, NULL};	/*Property = IsMobile*/
const Value V15604_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15604_5 = {S52, 0, &V15604_5_1293, &P15604_1};	/*Property = ScreenPixelsHeight*/
const Value V15604_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15604_4 = {S54, 0, &V15604_4_1296, &P15604_5};	/*Property = ScreenPixelsWidth*/
const Profile P15604 = {"15604", &P15604_4};	/*Profile ID - 15604*/
const Value V14524_1_1 = {S2, NULL};	/*Value = True*/
const Property P14524_1 = {S50, 0, &V14524_1_1, NULL};	/*Property = IsMobile*/
const Value V14524_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P14524_5 = {S52, 0, &V14524_5_1333, &P14524_1};	/*Property = ScreenPixelsHeight*/
const Value V14524_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P14524_4 = {S54, 0, &V14524_4_1335, &P14524_5};	/*Property = ScreenPixelsWidth*/
const Profile P14524 = {"14524", &P14524_4};	/*Profile ID - 14524*/
const Value V15605_1_1 = {S2, NULL};	/*Value = True*/
const Property P15605_1 = {S50, 0, &V15605_1_1, NULL};	/*Property = IsMobile*/
const Value V15605_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15605_5 = {S52, 0, &V15605_5_1293, &P15605_1};	/*Property = ScreenPixelsHeight*/
const Value V15605_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15605_4 = {S54, 0, &V15605_4_1296, &P15605_5};	/*Property = ScreenPixelsWidth*/
const Profile P15605 = {"15605", &P15605_4};	/*Profile ID - 15605*/
const Value V14424_1_1 = {S2, NULL};	/*Value = True*/
const Property P14424_1 = {S50, 0, &V14424_1_1, NULL};	/*Property = IsMobile*/
const Value V14424_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14424_5 = {S52, 0, &V14424_5_1293, &P14424_1};	/*Property = ScreenPixelsHeight*/
const Value V14424_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14424_4 = {S54, 0, &V14424_4_1296, &P14424_5};	/*Property = ScreenPixelsWidth*/
const Profile P14424 = {"14424", &P14424_4};	/*Profile ID - 14424*/
const Value V16031_1_1 = {S2, NULL};	/*Value = True*/
const Property P16031_1 = {S50, 0, &V16031_1_1, NULL};	/*Property = IsMobile*/
const Value V16031_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16031_5 = {S52, 0, &V16031_5_1277, &P16031_1};	/*Property = ScreenPixelsHeight*/
const Value V16031_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P16031_4 = {S54, 0, &V16031_4_8554, &P16031_5};	/*Property = ScreenPixelsWidth*/
const Profile P16031 = {"16031", &P16031_4};	/*Profile ID - 16031*/
const Value V15607_1_1 = {S2, NULL};	/*Value = True*/
const Property P15607_1 = {S50, 0, &V15607_1_1, NULL};	/*Property = IsMobile*/
const Value V15607_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15607_5 = {S52, 0, &V15607_5_1222, &P15607_1};	/*Property = ScreenPixelsHeight*/
const Value V15607_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15607_4 = {S54, 0, &V15607_4_1224, &P15607_5};	/*Property = ScreenPixelsWidth*/
const Profile P15607 = {"15607", &P15607_4};	/*Profile ID - 15607*/
const Value V15606_1_1 = {S2, NULL};	/*Value = True*/
const Property P15606_1 = {S50, 0, &V15606_1_1, NULL};	/*Property = IsMobile*/
const Value V15606_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15606_5 = {S52, 0, &V15606_5_1304, &P15606_1};	/*Property = ScreenPixelsHeight*/
const Value V15606_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15606_4 = {S54, 0, &V15606_4_1224, &P15606_5};	/*Property = ScreenPixelsWidth*/
const Profile P15606 = {"15606", &P15606_4};	/*Profile ID - 15606*/
const Value V15608_1_1 = {S2, NULL};	/*Value = True*/
const Property P15608_1 = {S50, 0, &V15608_1_1, NULL};	/*Property = IsMobile*/
const Value V15608_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15608_5 = {S52, 0, &V15608_5_1380, &P15608_1};	/*Property = ScreenPixelsHeight*/
const Value V15608_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15608_4 = {S54, 0, &V15608_4_1279, &P15608_5};	/*Property = ScreenPixelsWidth*/
const Profile P15608 = {"15608", &P15608_4};	/*Profile ID - 15608*/
const Value V15609_1_1 = {S2, NULL};	/*Value = True*/
const Property P15609_1 = {S50, 0, &V15609_1_1, NULL};	/*Property = IsMobile*/
const Value V15609_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15609_5 = {S52, 0, &V15609_5_1233, &P15609_1};	/*Property = ScreenPixelsHeight*/
const Value V15609_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15609_4 = {S54, 0, &V15609_4_1235, &P15609_5};	/*Property = ScreenPixelsWidth*/
const Profile P15609 = {"15609", &P15609_4};	/*Profile ID - 15609*/
const Value V15610_1_1 = {S2, NULL};	/*Value = True*/
const Property P15610_1 = {S50, 0, &V15610_1_1, NULL};	/*Property = IsMobile*/
const Value V15610_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15610_5 = {S52, 0, &V15610_5_1304, &P15610_1};	/*Property = ScreenPixelsHeight*/
const Value V15610_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15610_4 = {S54, 0, &V15610_4_1224, &P15610_5};	/*Property = ScreenPixelsWidth*/
const Profile P15610 = {"15610", &P15610_4};	/*Profile ID - 15610*/
const Value V17423_1_2 = {S0, NULL};	/*Value = False*/
const Property P17423_1 = {S50, 0, &V17423_1_2, NULL};	/*Property = IsMobile*/
const Value V17423_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P17423_5 = {S52, 0, &V17423_5_1294, &P17423_1};	/*Property = ScreenPixelsHeight*/
const Value V17423_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17423_4 = {S54, 0, &V17423_4_1296, &P17423_5};	/*Property = ScreenPixelsWidth*/
const Profile P17423 = {"17423", &P17423_4};	/*Profile ID - 17423*/
const Value V15611_1_1 = {S2, NULL};	/*Value = True*/
const Property P15611_1 = {S50, 0, &V15611_1_1, NULL};	/*Property = IsMobile*/
const Value V15611_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15611_5 = {S52, 0, &V15611_5_1294, &P15611_1};	/*Property = ScreenPixelsHeight*/
const Value V15611_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15611_4 = {S54, 0, &V15611_4_1296, &P15611_5};	/*Property = ScreenPixelsWidth*/
const Profile P15611 = {"15611", &P15611_4};	/*Profile ID - 15611*/
const Value V15612_1_1 = {S2, NULL};	/*Value = True*/
const Property P15612_1 = {S50, 0, &V15612_1_1, NULL};	/*Property = IsMobile*/
const Value V15612_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15612_5 = {S52, 0, &V15612_5_1294, &P15612_1};	/*Property = ScreenPixelsHeight*/
const Value V15612_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15612_4 = {S54, 0, &V15612_4_1296, &P15612_5};	/*Property = ScreenPixelsWidth*/
const Profile P15612 = {"15612", &P15612_4};	/*Profile ID - 15612*/
const Value V12888_1_1 = {S2, NULL};	/*Value = True*/
const Property P12888_1 = {S50, 0, &V12888_1_1, NULL};	/*Property = IsMobile*/
const Value V12888_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12888_5 = {S52, 0, &V12888_5_1277, &P12888_1};	/*Property = ScreenPixelsHeight*/
const Value V12888_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P12888_4 = {S54, 0, &V12888_4_2336, &P12888_5};	/*Property = ScreenPixelsWidth*/
const Profile P12888 = {"12888", &P12888_4};	/*Profile ID - 12888*/
const Value V17560_1_1 = {S2, NULL};	/*Value = True*/
const Property P17560_1 = {S50, 0, &V17560_1_1, NULL};	/*Property = IsMobile*/
const Value V17560_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17560_5 = {S52, 0, &V17560_5_1222, &P17560_1};	/*Property = ScreenPixelsHeight*/
const Value V17560_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17560_4 = {S54, 0, &V17560_4_1224, &P17560_5};	/*Property = ScreenPixelsWidth*/
const Profile P17560 = {"17560", &P17560_4};	/*Profile ID - 17560*/
const Value V14913_1_1 = {S2, NULL};	/*Value = True*/
const Property P14913_1 = {S50, 0, &V14913_1_1, NULL};	/*Property = IsMobile*/
const Value V14913_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14913_5 = {S52, 0, &V14913_5_1277, &P14913_1};	/*Property = ScreenPixelsHeight*/
const Value V14913_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14913_4 = {S54, 0, &V14913_4_1279, &P14913_5};	/*Property = ScreenPixelsWidth*/
const Profile P14913 = {"14913", &P14913_4};	/*Profile ID - 14913*/
const Value V15615_1_1 = {S2, NULL};	/*Value = True*/
const Property P15615_1 = {S50, 0, &V15615_1_1, NULL};	/*Property = IsMobile*/
const Value V15615_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15615_5 = {S52, 0, &V15615_5_1222, &P15615_1};	/*Property = ScreenPixelsHeight*/
const Value V15615_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15615_4 = {S54, 0, &V15615_4_1224, &P15615_5};	/*Property = ScreenPixelsWidth*/
const Profile P15615 = {"15615", &P15615_4};	/*Profile ID - 15615*/
const Value V15616_1_1 = {S2, NULL};	/*Value = True*/
const Property P15616_1 = {S50, 0, &V15616_1_1, NULL};	/*Property = IsMobile*/
const Value V15616_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15616_5 = {S52, 0, &V15616_5_1222, &P15616_1};	/*Property = ScreenPixelsHeight*/
const Value V15616_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15616_4 = {S54, 0, &V15616_4_1224, &P15616_5};	/*Property = ScreenPixelsWidth*/
const Profile P15616 = {"15616", &P15616_4};	/*Profile ID - 15616*/
const Value V17251_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17251_279 = {S1, 0, &V17251_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17251_282_57153 = {S2, NULL};	/*Value = True*/
const Property P17251_282 = {S3, 0, &V17251_282_57153, &P17251_279};	/*Property = BlobBuilder*/
const Value V17251_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17251_350 = {S4, 0, &V17251_350_57444, &P17251_282};	/*Property = Canvas*/
const Value V17251_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17251_283 = {S5, 0, &V17251_283_57159, &P17251_350};	/*Property = CssBackground*/
const Value V17251_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17251_284 = {S6, 0, &V17251_284_57163, &P17251_283};	/*Property = CssBorderImage*/
const Value V17251_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17251_285 = {S7, 0, &V17251_285_57166, &P17251_284};	/*Property = CssCanvas*/
const Value V17251_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17251_286 = {S8, 0, &V17251_286_57168, &P17251_285};	/*Property = CssColor*/
const Value V17251_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17251_237 = {S9, 0, &V17251_237_55784, &P17251_286};	/*Property = CssColumn*/
const Value V17251_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17251_287 = {S10, 0, &V17251_287_57172, &P17251_237};	/*Property = CssFlexbox*/
const Value V17251_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17251_288 = {S11, 0, &V17251_288_57175, &P17251_287};	/*Property = CssFont*/
const Value V17251_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17251_289 = {S12, 0, &V17251_289_57759, &P17251_288};	/*Property = CssImages*/
const Value V17251_291_57180 = {S2, NULL};	/*Value = True*/
const Property P17251_291 = {S13, 0, &V17251_291_57180, &P17251_289};	/*Property = CssMediaQueries*/
const Value V17251_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17251_292 = {S14, 0, &V17251_292_57183, &P17251_291};	/*Property = CssMinMax*/
const Value V17251_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17251_293 = {S15, 0, &V17251_293_57187, &P17251_292};	/*Property = CssOverflow*/
const Value V17251_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17251_294 = {S16, 0, &V17251_294_57189, &P17251_293};	/*Property = CssPosition*/
const Value V17251_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17251_295 = {S17, 0, &V17251_295_57192, &P17251_294};	/*Property = CssText*/
const Value V17251_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17251_297 = {S18, 0, &V17251_297_57195, &P17251_295};	/*Property = CssTransforms*/
const Value V17251_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17251_241 = {S19, 0, &V17251_241_55799, &P17251_297};	/*Property = CssTransitions*/
const Value V17251_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17251_298 = {S20, 0, &V17251_298_57206, &P17251_241};	/*Property = CssUI*/
const Value V17251_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17251_300 = {S21, 0, &V17251_300_57216, &P17251_298};	/*Property = DataSet*/
const Value V17251_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17251_301 = {S22, 0, &V17251_301_57218, &P17251_300};	/*Property = DataUrl*/
const Value V17251_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17251_302 = {S23, 0, &V17251_302_57222, &P17251_301};	/*Property = DeviceOrientation*/
const Value V17251_304_57227 = {S2, NULL};	/*Value = True*/
const Property P17251_304 = {S24, 0, &V17251_304_57227, &P17251_302};	/*Property = FileReader*/
const Value V17251_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17251_305 = {S25, 0, &V17251_305_57231, &P17251_304};	/*Property = FileSaver*/
const Value V17251_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17251_306 = {S26, 0, &V17251_306_57237, &P17251_305};	/*Property = FileWriter*/
const Value V17251_307_57239 = {S2, NULL};	/*Value = True*/
const Property P17251_307 = {S27, 0, &V17251_307_57239, &P17251_306};	/*Property = FormData*/
const Value V17251_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17251_309 = {S28, 0, &V17251_309_57246, &P17251_307};	/*Property = Fullscreen*/
const Value V17251_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17251_310 = {S29, 0, &V17251_310_57248, &P17251_309};	/*Property = GeoLocation*/
const Value V17251_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17251_312 = {S30, 0, &V17251_312_57255, &P17251_310};	/*Property = History*/
const Value V17251_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17251_314 = {S31, 0, &V17251_314_57261, &P17251_312};	/*Property = Html5*/
const Value V17251_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17251_313 = {S32, 0, &V17251_313_57258, &P17251_314};	/*Property = Html-Media-Capture*/
const Value V17251_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17251_315 = {S33, 0, &V17251_315_57264, &P17251_313};	/*Property = Iframe*/
const Value V17251_316_57266 = {S2, NULL};	/*Value = True*/
const Property P17251_316 = {S34, 0, &V17251_316_57266, &P17251_315};	/*Property = IndexedDB*/
const Value V17251_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17251_317 = {S35, 0, &V17251_317_57269, &P17251_316};	/*Property = Json*/
const Value V17251_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17251_15 = {S37, 0, &V17251_15_28, &P17251_317};	/*Property = LayoutEngine*/
const Value V17251_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17251_344 = {S38, 0, &V17251_344_57272, &P17251_15};	/*Property = Masking*/
const Value V17251_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17251_320 = {S39, 0, &V17251_320_57290, &P17251_344};	/*Property = PostMessage*/
const Value V17251_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17251_321 = {S40, 0, &V17251_321_57293, &P17251_320};	/*Property = Progress*/
const Value V17251_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17251_322 = {S41, 0, &V17251_322_57296, &P17251_321};	/*Property = Prompts*/
const Value V17251_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17251_323 = {S42, 0, &V17251_323_57299, &P17251_322};	/*Property = Selector*/
const Value V17251_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17251_324 = {S43, 0, &V17251_324_57306, &P17251_323};	/*Property = Svg*/
const Value V17251_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17251_327 = {S44, 0, &V17251_327_57315, &P17251_324};	/*Property = TouchEvents*/
const Value V17251_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17251_328 = {S45, 0, &V17251_328_57318, &P17251_327};	/*Property = Track*/
const Value V17251_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17251_329 = {S46, 0, &V17251_329_57323, &P17251_328};	/*Property = Video*/
const Value V17251_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17251_330 = {S47, 0, &V17251_330_57326, &P17251_329};	/*Property = Viewport*/
const Value V17251_332_57338 = {S2, NULL};	/*Value = True*/
const Property P17251_332 = {S48, 0, &V17251_332_57338, &P17251_330};	/*Property = WebWorkers*/
const Value V17251_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17251_333 = {S49, 0, &V17251_333_57342, &P17251_332};	/*Property = Xhr2*/
const Profile P17251 = {"17251", &P17251_333};	/*Profile ID - 17251*/
const Value V16598_1_1 = {S2, NULL};	/*Value = True*/
const Property P16598_1 = {S50, 0, &V16598_1_1, NULL};	/*Property = IsMobile*/
const Value V16598_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P16598_5 = {S52, 0, &V16598_5_9981, &P16598_1};	/*Property = ScreenPixelsHeight*/
const Value V16598_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P16598_4 = {S54, 0, &V16598_4_2336, &P16598_5};	/*Property = ScreenPixelsWidth*/
const Profile P16598 = {"16598", &P16598_4};	/*Profile ID - 16598*/
const Value V15617_1_1 = {S2, NULL};	/*Value = True*/
const Property P15617_1 = {S50, 0, &V15617_1_1, NULL};	/*Property = IsMobile*/
const Value V15617_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15617_5 = {S52, 0, &V15617_5_1222, &P15617_1};	/*Property = ScreenPixelsHeight*/
const Value V15617_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15617_4 = {S54, 0, &V15617_4_1224, &P15617_5};	/*Property = ScreenPixelsWidth*/
const Profile P15617 = {"15617", &P15617_4};	/*Profile ID - 15617*/
const Value V15618_1_1 = {S2, NULL};	/*Value = True*/
const Property P15618_1 = {S50, 0, &V15618_1_1, NULL};	/*Property = IsMobile*/
const Value V15618_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15618_5 = {S52, 0, &V15618_5_1233, &P15618_1};	/*Property = ScreenPixelsHeight*/
const Value V15618_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15618_4 = {S54, 0, &V15618_4_1235, &P15618_5};	/*Property = ScreenPixelsWidth*/
const Profile P15618 = {"15618", &P15618_4};	/*Profile ID - 15618*/
const Value V15619_1_1 = {S2, NULL};	/*Value = True*/
const Property P15619_1 = {S50, 0, &V15619_1_1, NULL};	/*Property = IsMobile*/
const Value V15619_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15619_5 = {S52, 0, &V15619_5_1293, &P15619_1};	/*Property = ScreenPixelsHeight*/
const Value V15619_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15619_4 = {S54, 0, &V15619_4_1296, &P15619_5};	/*Property = ScreenPixelsWidth*/
const Profile P15619 = {"15619", &P15619_4};	/*Profile ID - 15619*/
const Value V15621_1_1 = {S2, NULL};	/*Value = True*/
const Property P15621_1 = {S50, 0, &V15621_1_1, NULL};	/*Property = IsMobile*/
const Value V15621_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15621_5 = {S52, 0, &V15621_5_1222, &P15621_1};	/*Property = ScreenPixelsHeight*/
const Value V15621_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15621_4 = {S54, 0, &V15621_4_1224, &P15621_5};	/*Property = ScreenPixelsWidth*/
const Profile P15621 = {"15621", &P15621_4};	/*Profile ID - 15621*/
const Value V14459_1_1 = {S2, NULL};	/*Value = True*/
const Property P14459_1 = {S50, 0, &V14459_1_1, NULL};	/*Property = IsMobile*/
const Value V14459_5_31826 = {S75, NULL};	/*Value = 260*/
const Property P14459_5 = {S52, 0, &V14459_5_31826, &P14459_1};	/*Property = ScreenPixelsHeight*/
const Value V14459_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14459_4 = {S54, 0, &V14459_4_1224, &P14459_5};	/*Property = ScreenPixelsWidth*/
const Profile P14459 = {"14459", &P14459_4};	/*Profile ID - 14459*/
const Value V15623_1_1 = {S2, NULL};	/*Value = True*/
const Property P15623_1 = {S50, 0, &V15623_1_1, NULL};	/*Property = IsMobile*/
const Value V15623_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15623_5 = {S52, 0, &V15623_5_1233, &P15623_1};	/*Property = ScreenPixelsHeight*/
const Value V15623_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15623_4 = {S54, 0, &V15623_4_1235, &P15623_5};	/*Property = ScreenPixelsWidth*/
const Profile P15623 = {"15623", &P15623_4};	/*Profile ID - 15623*/
const Value V15624_1_1 = {S2, NULL};	/*Value = True*/
const Property P15624_1 = {S50, 0, &V15624_1_1, NULL};	/*Property = IsMobile*/
const Value V15624_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15624_5 = {S52, 0, &V15624_5_1222, &P15624_1};	/*Property = ScreenPixelsHeight*/
const Value V15624_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15624_4 = {S54, 0, &V15624_4_1224, &P15624_5};	/*Property = ScreenPixelsWidth*/
const Profile P15624 = {"15624", &P15624_4};	/*Profile ID - 15624*/
const Value V15625_1_1 = {S2, NULL};	/*Value = True*/
const Property P15625_1 = {S50, 0, &V15625_1_1, NULL};	/*Property = IsMobile*/
const Value V15625_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15625_5 = {S52, 0, &V15625_5_1233, &P15625_1};	/*Property = ScreenPixelsHeight*/
const Value V15625_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15625_4 = {S54, 0, &V15625_4_1296, &P15625_5};	/*Property = ScreenPixelsWidth*/
const Profile P15625 = {"15625", &P15625_4};	/*Profile ID - 15625*/
const Value V12411_1_1 = {S2, NULL};	/*Value = True*/
const Property P12411_1 = {S50, 0, &V12411_1_1, NULL};	/*Property = IsMobile*/
const Value V12411_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12411_5 = {S52, 0, &V12411_5_1304, &P12411_1};	/*Property = ScreenPixelsHeight*/
const Value V12411_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12411_4 = {S54, 0, &V12411_4_1224, &P12411_5};	/*Property = ScreenPixelsWidth*/
const Profile P12411 = {"12411", &P12411_4};	/*Profile ID - 12411*/
const Value V15626_1_1 = {S2, NULL};	/*Value = True*/
const Property P15626_1 = {S50, 0, &V15626_1_1, NULL};	/*Property = IsMobile*/
const Value V15626_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15626_5 = {S52, 0, &V15626_5_1233, &P15626_1};	/*Property = ScreenPixelsHeight*/
const Value V15626_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15626_4 = {S54, 0, &V15626_4_1235, &P15626_5};	/*Property = ScreenPixelsWidth*/
const Profile P15626 = {"15626", &P15626_4};	/*Profile ID - 15626*/
const Value V15627_1_1 = {S2, NULL};	/*Value = True*/
const Property P15627_1 = {S50, 0, &V15627_1_1, NULL};	/*Property = IsMobile*/
const Value V15627_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15627_5 = {S52, 0, &V15627_5_1233, &P15627_1};	/*Property = ScreenPixelsHeight*/
const Value V15627_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15627_4 = {S54, 0, &V15627_4_1235, &P15627_5};	/*Property = ScreenPixelsWidth*/
const Profile P15627 = {"15627", &P15627_4};	/*Profile ID - 15627*/
const Value V13670_1_1 = {S2, NULL};	/*Value = True*/
const Property P13670_1 = {S50, 0, &V13670_1_1, NULL};	/*Property = IsMobile*/
const Value V13670_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13670_5 = {S52, 0, &V13670_5_1222, &P13670_1};	/*Property = ScreenPixelsHeight*/
const Value V13670_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13670_4 = {S54, 0, &V13670_4_1224, &P13670_5};	/*Property = ScreenPixelsWidth*/
const Profile P13670 = {"13670", &P13670_4};	/*Profile ID - 13670*/
const Value V12075_1_1 = {S2, NULL};	/*Value = True*/
const Property P12075_1 = {S50, 0, &V12075_1_1, NULL};	/*Property = IsMobile*/
const Value V12075_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12075_5 = {S52, 0, &V12075_5_1380, &P12075_1};	/*Property = ScreenPixelsHeight*/
const Value V12075_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12075_4 = {S54, 0, &V12075_4_1279, &P12075_5};	/*Property = ScreenPixelsWidth*/
const Profile P12075 = {"12075", &P12075_4};	/*Profile ID - 12075*/
const Value V15631_1_1 = {S2, NULL};	/*Value = True*/
const Property P15631_1 = {S50, 0, &V15631_1_1, NULL};	/*Property = IsMobile*/
const Value V15631_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15631_5 = {S52, 0, &V15631_5_1233, &P15631_1};	/*Property = ScreenPixelsHeight*/
const Value V15631_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15631_4 = {S54, 0, &V15631_4_1235, &P15631_5};	/*Property = ScreenPixelsWidth*/
const Profile P15631 = {"15631", &P15631_4};	/*Profile ID - 15631*/
const Value V15632_1_1 = {S2, NULL};	/*Value = True*/
const Property P15632_1 = {S50, 0, &V15632_1_1, NULL};	/*Property = IsMobile*/
const Value V15632_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15632_5 = {S52, 0, &V15632_5_1293, &P15632_1};	/*Property = ScreenPixelsHeight*/
const Value V15632_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15632_4 = {S54, 0, &V15632_4_1296, &P15632_5};	/*Property = ScreenPixelsWidth*/
const Profile P15632 = {"15632", &P15632_4};	/*Profile ID - 15632*/
const Value V15633_1_1 = {S2, NULL};	/*Value = True*/
const Property P15633_1 = {S50, 0, &V15633_1_1, NULL};	/*Property = IsMobile*/
const Value V15633_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15633_5 = {S52, 0, &V15633_5_1293, &P15633_1};	/*Property = ScreenPixelsHeight*/
const Value V15633_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15633_4 = {S54, 0, &V15633_4_1296, &P15633_5};	/*Property = ScreenPixelsWidth*/
const Profile P15633 = {"15633", &P15633_4};	/*Profile ID - 15633*/
const Value V15634_1_1 = {S2, NULL};	/*Value = True*/
const Property P15634_1 = {S50, 0, &V15634_1_1, NULL};	/*Property = IsMobile*/
const Value V15634_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15634_5 = {S52, 0, &V15634_5_1222, &P15634_1};	/*Property = ScreenPixelsHeight*/
const Value V15634_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15634_4 = {S54, 0, &V15634_4_1224, &P15634_5};	/*Property = ScreenPixelsWidth*/
const Profile P15634 = {"15634", &P15634_4};	/*Profile ID - 15634*/
const Profile P22283 = {"22283", NULL};	/*Profile ID - 22283*/
const Value V15635_1_1 = {S2, NULL};	/*Value = True*/
const Property P15635_1 = {S50, 0, &V15635_1_1, NULL};	/*Property = IsMobile*/
const Value V15635_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15635_5 = {S52, 0, &V15635_5_1233, &P15635_1};	/*Property = ScreenPixelsHeight*/
const Value V15635_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15635_4 = {S54, 0, &V15635_4_1235, &P15635_5};	/*Property = ScreenPixelsWidth*/
const Profile P15635 = {"15635", &P15635_4};	/*Profile ID - 15635*/
const Value V3857_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_279 = {S1, 0, &V3857_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3857_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_282 = {S3, 0, &V3857_282_57155, &P3857_279};	/*Property = BlobBuilder*/
const Value V3857_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_350 = {S4, 0, &V3857_350_57446, &P3857_282};	/*Property = Canvas*/
const Value V3857_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_283 = {S5, 0, &V3857_283_57161, &P3857_350};	/*Property = CssBackground*/
const Value V3857_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_284 = {S6, 0, &V3857_284_57164, &P3857_283};	/*Property = CssBorderImage*/
const Value V3857_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_285 = {S7, 0, &V3857_285_57167, &P3857_284};	/*Property = CssCanvas*/
const Value V3857_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_286 = {S8, 0, &V3857_286_57170, &P3857_285};	/*Property = CssColor*/
const Value V3857_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_237 = {S9, 0, &V3857_237_55786, &P3857_286};	/*Property = CssColumn*/
const Value V3857_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_287 = {S10, 0, &V3857_287_57173, &P3857_237};	/*Property = CssFlexbox*/
const Value V3857_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_288 = {S11, 0, &V3857_288_57176, &P3857_287};	/*Property = CssFont*/
const Value V3857_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_289 = {S12, 0, &V3857_289_86899, &P3857_288};	/*Property = CssImages*/
const Value V3857_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_291 = {S13, 0, &V3857_291_57182, &P3857_289};	/*Property = CssMediaQueries*/
const Value V3857_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_292 = {S14, 0, &V3857_292_57185, &P3857_291};	/*Property = CssMinMax*/
const Value V3857_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_293 = {S15, 0, &V3857_293_57188, &P3857_292};	/*Property = CssOverflow*/
const Value V3857_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_294 = {S16, 0, &V3857_294_57191, &P3857_293};	/*Property = CssPosition*/
const Value V3857_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_295 = {S17, 0, &V3857_295_57194, &P3857_294};	/*Property = CssText*/
const Value V3857_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_297 = {S18, 0, &V3857_297_57197, &P3857_295};	/*Property = CssTransforms*/
const Value V3857_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_241 = {S19, 0, &V3857_241_55801, &P3857_297};	/*Property = CssTransitions*/
const Value V3857_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_298 = {S20, 0, &V3857_298_57208, &P3857_241};	/*Property = CssUI*/
const Value V3857_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_300 = {S21, 0, &V3857_300_57217, &P3857_298};	/*Property = DataSet*/
const Value V3857_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_301 = {S22, 0, &V3857_301_57220, &P3857_300};	/*Property = DataUrl*/
const Value V3857_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_302 = {S23, 0, &V3857_302_57223, &P3857_301};	/*Property = DeviceOrientation*/
const Value V3857_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_304 = {S24, 0, &V3857_304_57229, &P3857_302};	/*Property = FileReader*/
const Value V3857_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_305 = {S25, 0, &V3857_305_57232, &P3857_304};	/*Property = FileSaver*/
const Value V3857_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_306 = {S26, 0, &V3857_306_57238, &P3857_305};	/*Property = FileWriter*/
const Value V3857_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_307 = {S27, 0, &V3857_307_57241, &P3857_306};	/*Property = FormData*/
const Value V3857_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_309 = {S28, 0, &V3857_309_57247, &P3857_307};	/*Property = Fullscreen*/
const Value V3857_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_310 = {S29, 0, &V3857_310_57250, &P3857_309};	/*Property = GeoLocation*/
const Value V3857_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_312 = {S30, 0, &V3857_312_57256, &P3857_310};	/*Property = History*/
const Value V3857_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_314 = {S31, 0, &V3857_314_57262, &P3857_312};	/*Property = Html5*/
const Value V3857_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_313 = {S32, 0, &V3857_313_57259, &P3857_314};	/*Property = Html-Media-Capture*/
const Value V3857_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_315 = {S33, 0, &V3857_315_57265, &P3857_313};	/*Property = Iframe*/
const Value V3857_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_316 = {S34, 0, &V3857_316_57268, &P3857_315};	/*Property = IndexedDB*/
const Value V3857_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_317 = {S35, 0, &V3857_317_57271, &P3857_316};	/*Property = Json*/
const Value V3857_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3857_15 = {S37, 0, &V3857_15_28, &P3857_317};	/*Property = LayoutEngine*/
const Value V3857_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_344 = {S38, 0, &V3857_344_57274, &P3857_15};	/*Property = Masking*/
const Value V3857_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_320 = {S39, 0, &V3857_320_57292, &P3857_344};	/*Property = PostMessage*/
const Value V3857_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_321 = {S40, 0, &V3857_321_57295, &P3857_320};	/*Property = Progress*/
const Value V3857_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_322 = {S41, 0, &V3857_322_57298, &P3857_321};	/*Property = Prompts*/
const Value V3857_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_323 = {S42, 0, &V3857_323_57301, &P3857_322};	/*Property = Selector*/
const Value V3857_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_324 = {S43, 0, &V3857_324_57307, &P3857_323};	/*Property = Svg*/
const Value V3857_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_327 = {S44, 0, &V3857_327_57316, &P3857_324};	/*Property = TouchEvents*/
const Value V3857_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_328 = {S45, 0, &V3857_328_57319, &P3857_327};	/*Property = Track*/
const Value V3857_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_329 = {S46, 0, &V3857_329_57325, &P3857_328};	/*Property = Video*/
const Value V3857_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_330 = {S47, 0, &V3857_330_57328, &P3857_329};	/*Property = Viewport*/
const Value V3857_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_332 = {S48, 0, &V3857_332_57340, &P3857_330};	/*Property = WebWorkers*/
const Value V3857_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3857_333 = {S49, 0, &V3857_333_57343, &P3857_332};	/*Property = Xhr2*/
const Profile P3857 = {"3857", &P3857_333};	/*Profile ID - 3857*/
const Value V12637_1_1 = {S2, NULL};	/*Value = True*/
const Property P12637_1 = {S50, 0, &V12637_1_1, NULL};	/*Property = IsMobile*/
const Value V12637_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12637_5 = {S52, 0, &V12637_5_1380, &P12637_1};	/*Property = ScreenPixelsHeight*/
const Value V12637_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12637_4 = {S54, 0, &V12637_4_1279, &P12637_5};	/*Property = ScreenPixelsWidth*/
const Profile P12637 = {"12637", &P12637_4};	/*Profile ID - 12637*/
const Value V15636_1_1 = {S2, NULL};	/*Value = True*/
const Property P15636_1 = {S50, 0, &V15636_1_1, NULL};	/*Property = IsMobile*/
const Value V15636_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15636_5 = {S52, 0, &V15636_5_1233, &P15636_1};	/*Property = ScreenPixelsHeight*/
const Value V15636_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15636_4 = {S54, 0, &V15636_4_1235, &P15636_5};	/*Property = ScreenPixelsWidth*/
const Profile P15636 = {"15636", &P15636_4};	/*Profile ID - 15636*/
const Value V15637_1_1 = {S2, NULL};	/*Value = True*/
const Property P15637_1 = {S50, 0, &V15637_1_1, NULL};	/*Property = IsMobile*/
const Value V15637_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15637_5 = {S52, 0, &V15637_5_1293, &P15637_1};	/*Property = ScreenPixelsHeight*/
const Value V15637_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15637_4 = {S54, 0, &V15637_4_1296, &P15637_5};	/*Property = ScreenPixelsWidth*/
const Profile P15637 = {"15637", &P15637_4};	/*Profile ID - 15637*/
const Value V17083_1_1 = {S2, NULL};	/*Value = True*/
const Property P17083_1 = {S50, 0, &V17083_1_1, NULL};	/*Property = IsMobile*/
const Value V17083_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17083_5 = {S52, 0, &V17083_5_1227, &P17083_1};	/*Property = ScreenPixelsHeight*/
const Value V17083_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17083_4 = {S54, 0, &V17083_4_1229, &P17083_5};	/*Property = ScreenPixelsWidth*/
const Profile P17083 = {"17083", &P17083_4};	/*Profile ID - 17083*/
const Value V15638_1_1 = {S2, NULL};	/*Value = True*/
const Property P15638_1 = {S50, 0, &V15638_1_1, NULL};	/*Property = IsMobile*/
const Value V15638_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15638_5 = {S52, 0, &V15638_5_1294, &P15638_1};	/*Property = ScreenPixelsHeight*/
const Value V15638_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15638_4 = {S54, 0, &V15638_4_1296, &P15638_5};	/*Property = ScreenPixelsWidth*/
const Profile P15638 = {"15638", &P15638_4};	/*Profile ID - 15638*/
const Value V12856_1_1 = {S2, NULL};	/*Value = True*/
const Property P12856_1 = {S50, 0, &V12856_1_1, NULL};	/*Property = IsMobile*/
const Value V12856_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12856_5 = {S52, 0, &V12856_5_1222, &P12856_1};	/*Property = ScreenPixelsHeight*/
const Value V12856_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12856_4 = {S54, 0, &V12856_4_1224, &P12856_5};	/*Property = ScreenPixelsWidth*/
const Profile P12856 = {"12856", &P12856_4};	/*Profile ID - 12856*/
const Value V15640_1_1 = {S2, NULL};	/*Value = True*/
const Property P15640_1 = {S50, 0, &V15640_1_1, NULL};	/*Property = IsMobile*/
const Value V15640_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15640_5 = {S52, 0, &V15640_5_1222, &P15640_1};	/*Property = ScreenPixelsHeight*/
const Value V15640_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15640_4 = {S54, 0, &V15640_4_1224, &P15640_5};	/*Property = ScreenPixelsWidth*/
const Profile P15640 = {"15640", &P15640_4};	/*Profile ID - 15640*/
const Value V12359_1_1 = {S2, NULL};	/*Value = True*/
const Property P12359_1 = {S50, 0, &V12359_1_1, NULL};	/*Property = IsMobile*/
const Value V12359_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12359_5 = {S52, 0, &V12359_5_1222, &P12359_1};	/*Property = ScreenPixelsHeight*/
const Value V12359_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12359_4 = {S54, 0, &V12359_4_1224, &P12359_5};	/*Property = ScreenPixelsWidth*/
const Profile P12359 = {"12359", &P12359_4};	/*Profile ID - 12359*/
const Value V15641_1_1 = {S2, NULL};	/*Value = True*/
const Property P15641_1 = {S50, 0, &V15641_1_1, NULL};	/*Property = IsMobile*/
const Value V15641_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15641_5 = {S52, 0, &V15641_5_1293, &P15641_1};	/*Property = ScreenPixelsHeight*/
const Value V15641_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15641_4 = {S54, 0, &V15641_4_1296, &P15641_5};	/*Property = ScreenPixelsWidth*/
const Profile P15641 = {"15641", &P15641_4};	/*Profile ID - 15641*/
const Value V15239_1_1 = {S2, NULL};	/*Value = True*/
const Property P15239_1 = {S50, 0, &V15239_1_1, NULL};	/*Property = IsMobile*/
const Value V15239_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15239_5 = {S52, 0, &V15239_5_1233, &P15239_1};	/*Property = ScreenPixelsHeight*/
const Value V15239_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15239_4 = {S54, 0, &V15239_4_1235, &P15239_5};	/*Property = ScreenPixelsWidth*/
const Profile P15239 = {"15239", &P15239_4};	/*Profile ID - 15239*/
const Value V15655_1_1 = {S2, NULL};	/*Value = True*/
const Property P15655_1 = {S50, 0, &V15655_1_1, NULL};	/*Property = IsMobile*/
const Value V15655_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15655_5 = {S52, 0, &V15655_5_1222, &P15655_1};	/*Property = ScreenPixelsHeight*/
const Value V15655_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15655_4 = {S54, 0, &V15655_4_1224, &P15655_5};	/*Property = ScreenPixelsWidth*/
const Profile P15655 = {"15655", &P15655_4};	/*Profile ID - 15655*/
const Value V15643_1_1 = {S2, NULL};	/*Value = True*/
const Property P15643_1 = {S50, 0, &V15643_1_1, NULL};	/*Property = IsMobile*/
const Value V15643_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15643_5 = {S52, 0, &V15643_5_1233, &P15643_1};	/*Property = ScreenPixelsHeight*/
const Value V15643_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15643_4 = {S54, 0, &V15643_4_1235, &P15643_5};	/*Property = ScreenPixelsWidth*/
const Profile P15643 = {"15643", &P15643_4};	/*Profile ID - 15643*/
const Value V15644_1_1 = {S2, NULL};	/*Value = True*/
const Property P15644_1 = {S50, 0, &V15644_1_1, NULL};	/*Property = IsMobile*/
const Value V15644_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15644_5 = {S52, 0, &V15644_5_1277, &P15644_1};	/*Property = ScreenPixelsHeight*/
const Value V15644_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15644_4 = {S54, 0, &V15644_4_1279, &P15644_5};	/*Property = ScreenPixelsWidth*/
const Profile P15644 = {"15644", &P15644_4};	/*Profile ID - 15644*/
const Value V15645_1_1 = {S2, NULL};	/*Value = True*/
const Property P15645_1 = {S50, 0, &V15645_1_1, NULL};	/*Property = IsMobile*/
const Value V15645_5_37473 = {S99, NULL};	/*Value = 192*/
const Property P15645_5 = {S52, 0, &V15645_5_37473, &P15645_1};	/*Property = ScreenPixelsHeight*/
const Value V15645_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15645_4 = {S54, 0, &V15645_4_1235, &P15645_5};	/*Property = ScreenPixelsWidth*/
const Profile P15645 = {"15645", &P15645_4};	/*Profile ID - 15645*/
const Value V14427_1_1 = {S2, NULL};	/*Value = True*/
const Property P14427_1 = {S50, 0, &V14427_1_1, NULL};	/*Property = IsMobile*/
const Value V14427_5_4286 = {S73, NULL};	/*Value = 432*/
const Property P14427_5 = {S52, 0, &V14427_5_4286, &P14427_1};	/*Property = ScreenPixelsHeight*/
const Value V14427_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14427_4 = {S54, 0, &V14427_4_1224, &P14427_5};	/*Property = ScreenPixelsWidth*/
const Profile P14427 = {"14427", &P14427_4};	/*Profile ID - 14427*/
const Value V15648_1_1 = {S2, NULL};	/*Value = True*/
const Property P15648_1 = {S50, 0, &V15648_1_1, NULL};	/*Property = IsMobile*/
const Value V15648_5_31955 = {S92, NULL};	/*Value = 96*/
const Property P15648_5 = {S52, 0, &V15648_5_31955, &P15648_1};	/*Property = ScreenPixelsHeight*/
const Value V15648_4_37579 = {S100, NULL};	/*Value = 64*/
const Property P15648_4 = {S54, 0, &V15648_4_37579, &P15648_5};	/*Property = ScreenPixelsWidth*/
const Profile P15648 = {"15648", &P15648_4};	/*Profile ID - 15648*/
const Value V13638_1_1 = {S2, NULL};	/*Value = True*/
const Property P13638_1 = {S50, 0, &V13638_1_1, NULL};	/*Property = IsMobile*/
const Value V13638_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13638_5 = {S52, 0, &V13638_5_1293, &P13638_1};	/*Property = ScreenPixelsHeight*/
const Value V13638_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13638_4 = {S54, 0, &V13638_4_1296, &P13638_5};	/*Property = ScreenPixelsWidth*/
const Profile P13638 = {"13638", &P13638_4};	/*Profile ID - 13638*/
const Value V15651_1_1 = {S2, NULL};	/*Value = True*/
const Property P15651_1 = {S50, 0, &V15651_1_1, NULL};	/*Property = IsMobile*/
const Value V15651_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15651_5 = {S52, 0, &V15651_5_1233, &P15651_1};	/*Property = ScreenPixelsHeight*/
const Value V15651_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15651_4 = {S54, 0, &V15651_4_1235, &P15651_5};	/*Property = ScreenPixelsWidth*/
const Profile P15651 = {"15651", &P15651_4};	/*Profile ID - 15651*/
const Value V15642_1_1 = {S2, NULL};	/*Value = True*/
const Property P15642_1 = {S50, 0, &V15642_1_1, NULL};	/*Property = IsMobile*/
const Value V15642_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15642_5 = {S52, 0, &V15642_5_1293, &P15642_1};	/*Property = ScreenPixelsHeight*/
const Value V15642_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15642_4 = {S54, 0, &V15642_4_1296, &P15642_5};	/*Property = ScreenPixelsWidth*/
const Profile P15642 = {"15642", &P15642_4};	/*Profile ID - 15642*/
const Value V15650_1_1 = {S2, NULL};	/*Value = True*/
const Property P15650_1 = {S50, 0, &V15650_1_1, NULL};	/*Property = IsMobile*/
const Value V15650_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15650_5 = {S52, 0, &V15650_5_1222, &P15650_1};	/*Property = ScreenPixelsHeight*/
const Value V15650_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15650_4 = {S54, 0, &V15650_4_1224, &P15650_5};	/*Property = ScreenPixelsWidth*/
const Profile P15650 = {"15650", &P15650_4};	/*Profile ID - 15650*/
const Value V15653_1_1 = {S2, NULL};	/*Value = True*/
const Property P15653_1 = {S50, 0, &V15653_1_1, NULL};	/*Property = IsMobile*/
const Value V15653_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15653_5 = {S52, 0, &V15653_5_1293, &P15653_1};	/*Property = ScreenPixelsHeight*/
const Value V15653_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15653_4 = {S54, 0, &V15653_4_1296, &P15653_5};	/*Property = ScreenPixelsWidth*/
const Profile P15653 = {"15653", &P15653_4};	/*Profile ID - 15653*/
const Value V14452_1_1 = {S2, NULL};	/*Value = True*/
const Property P14452_1 = {S50, 0, &V14452_1_1, NULL};	/*Property = IsMobile*/
const Value V14452_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14452_5 = {S52, 0, &V14452_5_1233, &P14452_1};	/*Property = ScreenPixelsHeight*/
const Value V14452_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14452_4 = {S54, 0, &V14452_4_1235, &P14452_5};	/*Property = ScreenPixelsWidth*/
const Profile P14452 = {"14452", &P14452_4};	/*Profile ID - 14452*/
const Value V14446_1_1 = {S2, NULL};	/*Value = True*/
const Property P14446_1 = {S50, 0, &V14446_1_1, NULL};	/*Property = IsMobile*/
const Value V14446_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P14446_5 = {S52, 0, &V14446_5_1280, &P14446_1};	/*Property = ScreenPixelsHeight*/
const Value V14446_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14446_4 = {S54, 0, &V14446_4_1235, &P14446_5};	/*Property = ScreenPixelsWidth*/
const Profile P14446 = {"14446", &P14446_4};	/*Profile ID - 14446*/
const Value V15659_1_1 = {S2, NULL};	/*Value = True*/
const Property P15659_1 = {S50, 0, &V15659_1_1, NULL};	/*Property = IsMobile*/
const Value V15659_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15659_5 = {S52, 0, &V15659_5_1380, &P15659_1};	/*Property = ScreenPixelsHeight*/
const Value V15659_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15659_4 = {S54, 0, &V15659_4_1224, &P15659_5};	/*Property = ScreenPixelsWidth*/
const Profile P15659 = {"15659", &P15659_4};	/*Profile ID - 15659*/
const Value V13836_1_1 = {S2, NULL};	/*Value = True*/
const Property P13836_1 = {S50, 0, &V13836_1_1, NULL};	/*Property = IsMobile*/
const Value V13836_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13836_5 = {S52, 0, &V13836_5_1222, &P13836_1};	/*Property = ScreenPixelsHeight*/
const Value V13836_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13836_4 = {S54, 0, &V13836_4_1224, &P13836_5};	/*Property = ScreenPixelsWidth*/
const Profile P13836 = {"13836", &P13836_4};	/*Profile ID - 13836*/
const Value V12312_1_1 = {S2, NULL};	/*Value = True*/
const Property P12312_1 = {S50, 0, &V12312_1_1, NULL};	/*Property = IsMobile*/
const Value V12312_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12312_5 = {S52, 0, &V12312_5_1222, &P12312_1};	/*Property = ScreenPixelsHeight*/
const Value V12312_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12312_4 = {S54, 0, &V12312_4_1224, &P12312_5};	/*Property = ScreenPixelsWidth*/
const Profile P12312 = {"12312", &P12312_4};	/*Profile ID - 12312*/
const Value V15664_1_1 = {S2, NULL};	/*Value = True*/
const Property P15664_1 = {S50, 0, &V15664_1_1, NULL};	/*Property = IsMobile*/
const Value V15664_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15664_5 = {S52, 0, &V15664_5_1293, &P15664_1};	/*Property = ScreenPixelsHeight*/
const Value V15664_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15664_4 = {S54, 0, &V15664_4_1296, &P15664_5};	/*Property = ScreenPixelsWidth*/
const Profile P15664 = {"15664", &P15664_4};	/*Profile ID - 15664*/
const Value V15666_1_1 = {S2, NULL};	/*Value = True*/
const Property P15666_1 = {S50, 0, &V15666_1_1, NULL};	/*Property = IsMobile*/
const Value V15666_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15666_5 = {S52, 0, &V15666_5_1222, &P15666_1};	/*Property = ScreenPixelsHeight*/
const Value V15666_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15666_4 = {S54, 0, &V15666_4_1224, &P15666_5};	/*Property = ScreenPixelsWidth*/
const Profile P15666 = {"15666", &P15666_4};	/*Profile ID - 15666*/
const Value V15667_1_1 = {S2, NULL};	/*Value = True*/
const Property P15667_1 = {S50, 0, &V15667_1_1, NULL};	/*Property = IsMobile*/
const Value V15667_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15667_5 = {S52, 0, &V15667_5_1294, &P15667_1};	/*Property = ScreenPixelsHeight*/
const Value V15667_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15667_4 = {S54, 0, &V15667_4_1296, &P15667_5};	/*Property = ScreenPixelsWidth*/
const Profile P15667 = {"15667", &P15667_4};	/*Profile ID - 15667*/
const Value V15668_1_1 = {S2, NULL};	/*Value = True*/
const Property P15668_1 = {S50, 0, &V15668_1_1, NULL};	/*Property = IsMobile*/
const Value V15668_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15668_5 = {S52, 0, &V15668_5_1222, &P15668_1};	/*Property = ScreenPixelsHeight*/
const Value V15668_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15668_4 = {S54, 0, &V15668_4_1224, &P15668_5};	/*Property = ScreenPixelsWidth*/
const Profile P15668 = {"15668", &P15668_4};	/*Profile ID - 15668*/
const Value V12365_1_1 = {S2, NULL};	/*Value = True*/
const Property P12365_1 = {S50, 0, &V12365_1_1, NULL};	/*Property = IsMobile*/
const Value V12365_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P12365_5 = {S52, 0, &V12365_5_1333, &P12365_1};	/*Property = ScreenPixelsHeight*/
const Value V12365_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P12365_4 = {S54, 0, &V12365_4_1335, &P12365_5};	/*Property = ScreenPixelsWidth*/
const Profile P12365 = {"12365", &P12365_4};	/*Profile ID - 12365*/
const Profile P5298 = {"5298", NULL};	/*Profile ID - 5298*/
const Value V15660_1_1 = {S2, NULL};	/*Value = True*/
const Property P15660_1 = {S50, 0, &V15660_1_1, NULL};	/*Property = IsMobile*/
const Value V15660_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15660_5 = {S52, 0, &V15660_5_1304, &P15660_1};	/*Property = ScreenPixelsHeight*/
const Value V15660_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15660_4 = {S54, 0, &V15660_4_1224, &P15660_5};	/*Property = ScreenPixelsWidth*/
const Profile P15660 = {"15660", &P15660_4};	/*Profile ID - 15660*/
const Value V15669_1_1 = {S2, NULL};	/*Value = True*/
const Property P15669_1 = {S50, 0, &V15669_1_1, NULL};	/*Property = IsMobile*/
const Value V15669_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15669_5 = {S52, 0, &V15669_5_1293, &P15669_1};	/*Property = ScreenPixelsHeight*/
const Value V15669_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15669_4 = {S54, 0, &V15669_4_1296, &P15669_5};	/*Property = ScreenPixelsWidth*/
const Profile P15669 = {"15669", &P15669_4};	/*Profile ID - 15669*/
const Value V15670_1_1 = {S2, NULL};	/*Value = True*/
const Property P15670_1 = {S50, 0, &V15670_1_1, NULL};	/*Property = IsMobile*/
const Value V15670_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15670_5 = {S52, 0, &V15670_5_1222, &P15670_1};	/*Property = ScreenPixelsHeight*/
const Value V15670_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15670_4 = {S54, 0, &V15670_4_1224, &P15670_5};	/*Property = ScreenPixelsWidth*/
const Profile P15670 = {"15670", &P15670_4};	/*Profile ID - 15670*/
const Value V15671_1_1 = {S2, NULL};	/*Value = True*/
const Property P15671_1 = {S50, 0, &V15671_1_1, NULL};	/*Property = IsMobile*/
const Value V15671_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15671_5 = {S52, 0, &V15671_5_1233, &P15671_1};	/*Property = ScreenPixelsHeight*/
const Value V15671_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15671_4 = {S54, 0, &V15671_4_1235, &P15671_5};	/*Property = ScreenPixelsWidth*/
const Profile P15671 = {"15671", &P15671_4};	/*Profile ID - 15671*/
const Value V14010_1_1 = {S2, NULL};	/*Value = True*/
const Property P14010_1 = {S50, 0, &V14010_1_1, NULL};	/*Property = IsMobile*/
const Value V14010_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14010_5 = {S52, 0, &V14010_5_1304, &P14010_1};	/*Property = ScreenPixelsHeight*/
const Value V14010_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14010_4 = {S54, 0, &V14010_4_1224, &P14010_5};	/*Property = ScreenPixelsWidth*/
const Profile P14010 = {"14010", &P14010_4};	/*Profile ID - 14010*/
const Value V14584_1_1 = {S2, NULL};	/*Value = True*/
const Property P14584_1 = {S50, 0, &V14584_1_1, NULL};	/*Property = IsMobile*/
const Value V14584_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14584_5 = {S52, 0, &V14584_5_1227, &P14584_1};	/*Property = ScreenPixelsHeight*/
const Value V14584_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14584_4 = {S54, 0, &V14584_4_1229, &P14584_5};	/*Property = ScreenPixelsWidth*/
const Profile P14584 = {"14584", &P14584_4};	/*Profile ID - 14584*/
const Value V14294_1_1 = {S2, NULL};	/*Value = True*/
const Property P14294_1 = {S50, 0, &V14294_1_1, NULL};	/*Property = IsMobile*/
const Value V14294_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P14294_5 = {S52, 0, &V14294_5_1275, &P14294_1};	/*Property = ScreenPixelsHeight*/
const Value V14294_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P14294_4 = {S54, 0, &V14294_4_1278, &P14294_5};	/*Property = ScreenPixelsWidth*/
const Profile P14294 = {"14294", &P14294_4};	/*Profile ID - 14294*/
const Value V17947_1_1 = {S2, NULL};	/*Value = True*/
const Property P17947_1 = {S50, 0, &V17947_1_1, NULL};	/*Property = IsMobile*/
const Value V17947_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17947_5 = {S52, 0, &V17947_5_1227, &P17947_1};	/*Property = ScreenPixelsHeight*/
const Value V17947_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17947_4 = {S54, 0, &V17947_4_1229, &P17947_5};	/*Property = ScreenPixelsWidth*/
const Profile P17947 = {"17947", &P17947_4};	/*Profile ID - 17947*/
const Value V12091_1_1 = {S2, NULL};	/*Value = True*/
const Property P12091_1 = {S50, 0, &V12091_1_1, NULL};	/*Property = IsMobile*/
const Value V12091_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12091_5 = {S52, 0, &V12091_5_1233, &P12091_1};	/*Property = ScreenPixelsHeight*/
const Value V12091_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12091_4 = {S54, 0, &V12091_4_1235, &P12091_5};	/*Property = ScreenPixelsWidth*/
const Profile P12091 = {"12091", &P12091_4};	/*Profile ID - 12091*/
const Value V12092_1_1 = {S2, NULL};	/*Value = True*/
const Property P12092_1 = {S50, 0, &V12092_1_1, NULL};	/*Property = IsMobile*/
const Value V12092_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12092_5 = {S52, 0, &V12092_5_1277, &P12092_1};	/*Property = ScreenPixelsHeight*/
const Value V12092_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P12092_4 = {S54, 0, &V12092_4_2336, &P12092_5};	/*Property = ScreenPixelsWidth*/
const Profile P12092 = {"12092", &P12092_4};	/*Profile ID - 12092*/
const Value V12094_1_1 = {S2, NULL};	/*Value = True*/
const Property P12094_1 = {S50, 0, &V12094_1_1, NULL};	/*Property = IsMobile*/
const Value V12094_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12094_5 = {S52, 0, &V12094_5_1233, &P12094_1};	/*Property = ScreenPixelsHeight*/
const Value V12094_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12094_4 = {S54, 0, &V12094_4_1235, &P12094_5};	/*Property = ScreenPixelsWidth*/
const Profile P12094 = {"12094", &P12094_4};	/*Profile ID - 12094*/
const Value V12329_1_1 = {S2, NULL};	/*Value = True*/
const Property P12329_1 = {S50, 0, &V12329_1_1, NULL};	/*Property = IsMobile*/
const Value V12329_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12329_5 = {S52, 0, &V12329_5_1222, &P12329_1};	/*Property = ScreenPixelsHeight*/
const Value V12329_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12329_4 = {S54, 0, &V12329_4_1224, &P12329_5};	/*Property = ScreenPixelsWidth*/
const Profile P12329 = {"12329", &P12329_4};	/*Profile ID - 12329*/
const Value V15675_1_1 = {S2, NULL};	/*Value = True*/
const Property P15675_1 = {S50, 0, &V15675_1_1, NULL};	/*Property = IsMobile*/
const Value V15675_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15675_5 = {S52, 0, &V15675_5_1222, &P15675_1};	/*Property = ScreenPixelsHeight*/
const Value V15675_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15675_4 = {S54, 0, &V15675_4_1224, &P15675_5};	/*Property = ScreenPixelsWidth*/
const Profile P15675 = {"15675", &P15675_4};	/*Profile ID - 15675*/
const Value V13896_1_1 = {S2, NULL};	/*Value = True*/
const Property P13896_1 = {S50, 0, &V13896_1_1, NULL};	/*Property = IsMobile*/
const Value V13896_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13896_5 = {S52, 0, &V13896_5_1293, &P13896_1};	/*Property = ScreenPixelsHeight*/
const Value V13896_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13896_4 = {S54, 0, &V13896_4_1296, &P13896_5};	/*Property = ScreenPixelsWidth*/
const Profile P13896 = {"13896", &P13896_4};	/*Profile ID - 13896*/
const Value V15672_1_1 = {S2, NULL};	/*Value = True*/
const Property P15672_1 = {S50, 0, &V15672_1_1, NULL};	/*Property = IsMobile*/
const Value V15672_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15672_5 = {S52, 0, &V15672_5_1233, &P15672_1};	/*Property = ScreenPixelsHeight*/
const Value V15672_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15672_4 = {S54, 0, &V15672_4_1235, &P15672_5};	/*Property = ScreenPixelsWidth*/
const Profile P15672 = {"15672", &P15672_4};	/*Profile ID - 15672*/
const Value V15678_1_1 = {S2, NULL};	/*Value = True*/
const Property P15678_1 = {S50, 0, &V15678_1_1, NULL};	/*Property = IsMobile*/
const Value V15678_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15678_5 = {S52, 0, &V15678_5_1222, &P15678_1};	/*Property = ScreenPixelsHeight*/
const Value V15678_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15678_4 = {S54, 0, &V15678_4_1224, &P15678_5};	/*Property = ScreenPixelsWidth*/
const Profile P15678 = {"15678", &P15678_4};	/*Profile ID - 15678*/
const Value V16971_1_1 = {S2, NULL};	/*Value = True*/
const Property P16971_1 = {S50, 0, &V16971_1_1, NULL};	/*Property = IsMobile*/
const Value V16971_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P16971_5 = {S52, 0, &V16971_5_1304, &P16971_1};	/*Property = ScreenPixelsHeight*/
const Value V16971_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16971_4 = {S54, 0, &V16971_4_1224, &P16971_5};	/*Property = ScreenPixelsWidth*/
const Profile P16971 = {"16971", &P16971_4};	/*Profile ID - 16971*/
const Value V15680_1_1 = {S2, NULL};	/*Value = True*/
const Property P15680_1 = {S50, 0, &V15680_1_1, NULL};	/*Property = IsMobile*/
const Value V15680_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15680_5 = {S52, 0, &V15680_5_1294, &P15680_1};	/*Property = ScreenPixelsHeight*/
const Value V15680_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15680_4 = {S54, 0, &V15680_4_1296, &P15680_5};	/*Property = ScreenPixelsWidth*/
const Profile P15680 = {"15680", &P15680_4};	/*Profile ID - 15680*/
const Value V12098_1_1 = {S2, NULL};	/*Value = True*/
const Property P12098_1 = {S50, 0, &V12098_1_1, NULL};	/*Property = IsMobile*/
const Value V12098_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12098_5 = {S52, 0, &V12098_5_1222, &P12098_1};	/*Property = ScreenPixelsHeight*/
const Value V12098_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12098_4 = {S54, 0, &V12098_4_1224, &P12098_5};	/*Property = ScreenPixelsWidth*/
const Profile P12098 = {"12098", &P12098_4};	/*Profile ID - 12098*/
const Value V17124_1_1 = {S2, NULL};	/*Value = True*/
const Property P17124_1 = {S50, 0, &V17124_1_1, NULL};	/*Property = IsMobile*/
const Value V17124_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17124_5 = {S52, 0, &V17124_5_1222, &P17124_1};	/*Property = ScreenPixelsHeight*/
const Value V17124_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17124_4 = {S54, 0, &V17124_4_1224, &P17124_5};	/*Property = ScreenPixelsWidth*/
const Profile P17124 = {"17124", &P17124_4};	/*Profile ID - 17124*/
const Value V14755_1_1 = {S2, NULL};	/*Value = True*/
const Property P14755_1 = {S50, 0, &V14755_1_1, NULL};	/*Property = IsMobile*/
const Value V14755_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14755_5 = {S52, 0, &V14755_5_1304, &P14755_1};	/*Property = ScreenPixelsHeight*/
const Value V14755_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14755_4 = {S54, 0, &V14755_4_1224, &P14755_5};	/*Property = ScreenPixelsWidth*/
const Profile P14755 = {"14755", &P14755_4};	/*Profile ID - 14755*/
const Value V12654_1_1 = {S2, NULL};	/*Value = True*/
const Property P12654_1 = {S50, 0, &V12654_1_1, NULL};	/*Property = IsMobile*/
const Value V12654_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12654_5 = {S52, 0, &V12654_5_1294, &P12654_1};	/*Property = ScreenPixelsHeight*/
const Value V12654_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12654_4 = {S54, 0, &V12654_4_1296, &P12654_5};	/*Property = ScreenPixelsWidth*/
const Profile P12654 = {"12654", &P12654_4};	/*Profile ID - 12654*/
const Value V15682_1_1 = {S2, NULL};	/*Value = True*/
const Property P15682_1 = {S50, 0, &V15682_1_1, NULL};	/*Property = IsMobile*/
const Value V15682_5_31955 = {S92, NULL};	/*Value = 96*/
const Property P15682_5 = {S52, 0, &V15682_5_31955, &P15682_1};	/*Property = ScreenPixelsHeight*/
const Value V15682_4_31956 = {S92, NULL};	/*Value = 96*/
const Property P15682_4 = {S54, 0, &V15682_4_31956, &P15682_5};	/*Property = ScreenPixelsWidth*/
const Profile P15682 = {"15682", &P15682_4};	/*Profile ID - 15682*/
const Value V12157_1_1 = {S2, NULL};	/*Value = True*/
const Property P12157_1 = {S50, 0, &V12157_1_1, NULL};	/*Property = IsMobile*/
const Value V12157_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12157_5 = {S52, 0, &V12157_5_1222, &P12157_1};	/*Property = ScreenPixelsHeight*/
const Value V12157_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12157_4 = {S54, 0, &V12157_4_1224, &P12157_5};	/*Property = ScreenPixelsWidth*/
const Profile P12157 = {"12157", &P12157_4};	/*Profile ID - 12157*/
const Value V15683_1_1 = {S2, NULL};	/*Value = True*/
const Property P15683_1 = {S50, 0, &V15683_1_1, NULL};	/*Property = IsMobile*/
const Value V15683_5_37465 = {S96, NULL};	/*Value = 80*/
const Property P15683_5 = {S52, 0, &V15683_5_37465, &P15683_1};	/*Property = ScreenPixelsHeight*/
const Value V15683_4_37576 = {S97, NULL};	/*Value = 101*/
const Property P15683_4 = {S54, 0, &V15683_4_37576, &P15683_5};	/*Property = ScreenPixelsWidth*/
const Profile P15683 = {"15683", &P15683_4};	/*Profile ID - 15683*/
const Value V15684_1_1 = {S2, NULL};	/*Value = True*/
const Property P15684_1 = {S50, 0, &V15684_1_1, NULL};	/*Property = IsMobile*/
const Value V15684_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15684_5 = {S52, 0, &V15684_5_1222, &P15684_1};	/*Property = ScreenPixelsHeight*/
const Value V15684_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15684_4 = {S54, 0, &V15684_4_1224, &P15684_5};	/*Property = ScreenPixelsWidth*/
const Profile P15684 = {"15684", &P15684_4};	/*Profile ID - 15684*/
const Value V14515_1_1 = {S2, NULL};	/*Value = True*/
const Property P14515_1 = {S50, 0, &V14515_1_1, NULL};	/*Property = IsMobile*/
const Value V14515_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14515_5 = {S52, 0, &V14515_5_1293, &P14515_1};	/*Property = ScreenPixelsHeight*/
const Value V14515_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14515_4 = {S54, 0, &V14515_4_1296, &P14515_5};	/*Property = ScreenPixelsWidth*/
const Profile P14515 = {"14515", &P14515_4};	/*Profile ID - 14515*/
const Value V15685_1_1 = {S2, NULL};	/*Value = True*/
const Property P15685_1 = {S50, 0, &V15685_1_1, NULL};	/*Property = IsMobile*/
const Value V15685_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15685_5 = {S52, 0, &V15685_5_1222, &P15685_1};	/*Property = ScreenPixelsHeight*/
const Value V15685_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15685_4 = {S54, 0, &V15685_4_1224, &P15685_5};	/*Property = ScreenPixelsWidth*/
const Profile P15685 = {"15685", &P15685_4};	/*Profile ID - 15685*/
const Value V14103_1_1 = {S2, NULL};	/*Value = True*/
const Property P14103_1 = {S50, 0, &V14103_1_1, NULL};	/*Property = IsMobile*/
const Value V14103_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14103_5 = {S52, 0, &V14103_5_1304, &P14103_1};	/*Property = ScreenPixelsHeight*/
const Value V14103_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14103_4 = {S54, 0, &V14103_4_1224, &P14103_5};	/*Property = ScreenPixelsWidth*/
const Profile P14103 = {"14103", &P14103_4};	/*Profile ID - 14103*/
const Value V15687_1_1 = {S2, NULL};	/*Value = True*/
const Property P15687_1 = {S50, 0, &V15687_1_1, NULL};	/*Property = IsMobile*/
const Value V15687_5_24030 = {S71, NULL};	/*Value = 120*/
const Property P15687_5 = {S52, 0, &V15687_5_24030, &P15687_1};	/*Property = ScreenPixelsHeight*/
const Value V15687_4_24031 = {S71, NULL};	/*Value = 120*/
const Property P15687_4 = {S54, 0, &V15687_4_24031, &P15687_5};	/*Property = ScreenPixelsWidth*/
const Profile P15687 = {"15687", &P15687_4};	/*Profile ID - 15687*/
const Value V15688_1_1 = {S2, NULL};	/*Value = True*/
const Property P15688_1 = {S50, 0, &V15688_1_1, NULL};	/*Property = IsMobile*/
const Value V15688_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15688_5 = {S52, 0, &V15688_5_1233, &P15688_1};	/*Property = ScreenPixelsHeight*/
const Value V15688_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15688_4 = {S54, 0, &V15688_4_1235, &P15688_5};	/*Property = ScreenPixelsWidth*/
const Profile P15688 = {"15688", &P15688_4};	/*Profile ID - 15688*/
const Value V12106_1_1 = {S2, NULL};	/*Value = True*/
const Property P12106_1 = {S50, 0, &V12106_1_1, NULL};	/*Property = IsMobile*/
const Value V12106_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12106_5 = {S52, 0, &V12106_5_1233, &P12106_1};	/*Property = ScreenPixelsHeight*/
const Value V12106_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12106_4 = {S54, 0, &V12106_4_1235, &P12106_5};	/*Property = ScreenPixelsWidth*/
const Profile P12106 = {"12106", &P12106_4};	/*Profile ID - 12106*/
const Value V15689_1_1 = {S2, NULL};	/*Value = True*/
const Property P15689_1 = {S50, 0, &V15689_1_1, NULL};	/*Property = IsMobile*/
const Value V15689_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15689_5 = {S52, 0, &V15689_5_1222, &P15689_1};	/*Property = ScreenPixelsHeight*/
const Value V15689_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15689_4 = {S54, 0, &V15689_4_1224, &P15689_5};	/*Property = ScreenPixelsWidth*/
const Profile P15689 = {"15689", &P15689_4};	/*Profile ID - 15689*/
const Value V15690_1_1 = {S2, NULL};	/*Value = True*/
const Property P15690_1 = {S50, 0, &V15690_1_1, NULL};	/*Property = IsMobile*/
const Value V15690_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15690_5 = {S52, 0, &V15690_5_1380, &P15690_1};	/*Property = ScreenPixelsHeight*/
const Value V15690_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P15690_4 = {S54, 0, &V15690_4_1943, &P15690_5};	/*Property = ScreenPixelsWidth*/
const Profile P15690 = {"15690", &P15690_4};	/*Profile ID - 15690*/
const Value V15691_1_1 = {S2, NULL};	/*Value = True*/
const Property P15691_1 = {S50, 0, &V15691_1_1, NULL};	/*Property = IsMobile*/
const Value V15691_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15691_5 = {S52, 0, &V15691_5_1222, &P15691_1};	/*Property = ScreenPixelsHeight*/
const Value V15691_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15691_4 = {S54, 0, &V15691_4_1224, &P15691_5};	/*Property = ScreenPixelsWidth*/
const Profile P15691 = {"15691", &P15691_4};	/*Profile ID - 15691*/
const Value V15693_1_1 = {S2, NULL};	/*Value = True*/
const Property P15693_1 = {S50, 0, &V15693_1_1, NULL};	/*Property = IsMobile*/
const Value V15693_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15693_5 = {S52, 0, &V15693_5_1233, &P15693_1};	/*Property = ScreenPixelsHeight*/
const Value V15693_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15693_4 = {S54, 0, &V15693_4_1235, &P15693_5};	/*Property = ScreenPixelsWidth*/
const Profile P15693 = {"15693", &P15693_4};	/*Profile ID - 15693*/
const Value V15275_1_1 = {S2, NULL};	/*Value = True*/
const Property P15275_1 = {S50, 0, &V15275_1_1, NULL};	/*Property = IsMobile*/
const Value V15275_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15275_5 = {S52, 0, &V15275_5_1222, &P15275_1};	/*Property = ScreenPixelsHeight*/
const Value V15275_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15275_4 = {S54, 0, &V15275_4_1224, &P15275_5};	/*Property = ScreenPixelsWidth*/
const Profile P15275 = {"15275", &P15275_4};	/*Profile ID - 15275*/
const Value V15697_1_1 = {S2, NULL};	/*Value = True*/
const Property P15697_1 = {S50, 0, &V15697_1_1, NULL};	/*Property = IsMobile*/
const Value V15697_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15697_5 = {S52, 0, &V15697_5_1293, &P15697_1};	/*Property = ScreenPixelsHeight*/
const Value V15697_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15697_4 = {S54, 0, &V15697_4_1296, &P15697_5};	/*Property = ScreenPixelsWidth*/
const Profile P15697 = {"15697", &P15697_4};	/*Profile ID - 15697*/
const Value V15698_1_1 = {S2, NULL};	/*Value = True*/
const Property P15698_1 = {S50, 0, &V15698_1_1, NULL};	/*Property = IsMobile*/
const Value V15698_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15698_5 = {S52, 0, &V15698_5_1222, &P15698_1};	/*Property = ScreenPixelsHeight*/
const Value V15698_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15698_4 = {S54, 0, &V15698_4_1224, &P15698_5};	/*Property = ScreenPixelsWidth*/
const Profile P15698 = {"15698", &P15698_4};	/*Profile ID - 15698*/
const Value V12788_1_1 = {S2, NULL};	/*Value = True*/
const Property P12788_1 = {S50, 0, &V12788_1_1, NULL};	/*Property = IsMobile*/
const Value V12788_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P12788_5 = {S52, 0, &V12788_5_1403, &P12788_1};	/*Property = ScreenPixelsHeight*/
const Value V12788_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P12788_4 = {S54, 0, &V12788_4_1404, &P12788_5};	/*Property = ScreenPixelsWidth*/
const Profile P12788 = {"12788", &P12788_4};	/*Profile ID - 12788*/
const Value V12155_1_1 = {S2, NULL};	/*Value = True*/
const Property P12155_1 = {S50, 0, &V12155_1_1, NULL};	/*Property = IsMobile*/
const Value V12155_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12155_5 = {S52, 0, &V12155_5_1233, &P12155_1};	/*Property = ScreenPixelsHeight*/
const Value V12155_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12155_4 = {S54, 0, &V12155_4_1235, &P12155_5};	/*Property = ScreenPixelsWidth*/
const Profile P12155 = {"12155", &P12155_4};	/*Profile ID - 12155*/
const Value V13661_1_1 = {S2, NULL};	/*Value = True*/
const Property P13661_1 = {S50, 0, &V13661_1_1, NULL};	/*Property = IsMobile*/
const Value V13661_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13661_5 = {S52, 0, &V13661_5_1222, &P13661_1};	/*Property = ScreenPixelsHeight*/
const Value V13661_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13661_4 = {S54, 0, &V13661_4_1224, &P13661_5};	/*Property = ScreenPixelsWidth*/
const Profile P13661 = {"13661", &P13661_4};	/*Profile ID - 13661*/
const Value V15701_1_1 = {S2, NULL};	/*Value = True*/
const Property P15701_1 = {S50, 0, &V15701_1_1, NULL};	/*Property = IsMobile*/
const Value V15701_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15701_5 = {S52, 0, &V15701_5_1233, &P15701_1};	/*Property = ScreenPixelsHeight*/
const Value V15701_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15701_4 = {S54, 0, &V15701_4_1235, &P15701_5};	/*Property = ScreenPixelsWidth*/
const Profile P15701 = {"15701", &P15701_4};	/*Profile ID - 15701*/
const Value V13879_1_1 = {S2, NULL};	/*Value = True*/
const Property P13879_1 = {S50, 0, &V13879_1_1, NULL};	/*Property = IsMobile*/
const Value V13879_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13879_5 = {S52, 0, &V13879_5_1222, &P13879_1};	/*Property = ScreenPixelsHeight*/
const Value V13879_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13879_4 = {S54, 0, &V13879_4_1224, &P13879_5};	/*Property = ScreenPixelsWidth*/
const Profile P13879 = {"13879", &P13879_4};	/*Profile ID - 13879*/
const Value V13078_1_1 = {S2, NULL};	/*Value = True*/
const Property P13078_1 = {S50, 0, &V13078_1_1, NULL};	/*Property = IsMobile*/
const Value V13078_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13078_5 = {S52, 0, &V13078_5_1227, &P13078_1};	/*Property = ScreenPixelsHeight*/
const Value V13078_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13078_4 = {S54, 0, &V13078_4_1229, &P13078_5};	/*Property = ScreenPixelsWidth*/
const Profile P13078 = {"13078", &P13078_4};	/*Profile ID - 13078*/
const Value V15703_1_1 = {S2, NULL};	/*Value = True*/
const Property P15703_1 = {S50, 0, &V15703_1_1, NULL};	/*Property = IsMobile*/
const Value V15703_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15703_5 = {S52, 0, &V15703_5_1293, &P15703_1};	/*Property = ScreenPixelsHeight*/
const Value V15703_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15703_4 = {S54, 0, &V15703_4_1296, &P15703_5};	/*Property = ScreenPixelsWidth*/
const Profile P15703 = {"15703", &P15703_4};	/*Profile ID - 15703*/
const Value V15704_1_1 = {S2, NULL};	/*Value = True*/
const Property P15704_1 = {S50, 0, &V15704_1_1, NULL};	/*Property = IsMobile*/
const Value V15704_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15704_5 = {S52, 0, &V15704_5_1233, &P15704_1};	/*Property = ScreenPixelsHeight*/
const Value V15704_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15704_4 = {S54, 0, &V15704_4_1296, &P15704_5};	/*Property = ScreenPixelsWidth*/
const Profile P15704 = {"15704", &P15704_4};	/*Profile ID - 15704*/
const Value V12197_1_1 = {S2, NULL};	/*Value = True*/
const Property P12197_1 = {S50, 0, &V12197_1_1, NULL};	/*Property = IsMobile*/
const Value V12197_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12197_5 = {S52, 0, &V12197_5_1222, &P12197_1};	/*Property = ScreenPixelsHeight*/
const Value V12197_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12197_4 = {S54, 0, &V12197_4_1224, &P12197_5};	/*Property = ScreenPixelsWidth*/
const Profile P12197 = {"12197", &P12197_4};	/*Profile ID - 12197*/
const Value V15706_1_1 = {S2, NULL};	/*Value = True*/
const Property P15706_1 = {S50, 0, &V15706_1_1, NULL};	/*Property = IsMobile*/
const Value V15706_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P15706_5 = {S52, 0, &V15706_5_1333, &P15706_1};	/*Property = ScreenPixelsHeight*/
const Value V15706_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P15706_4 = {S54, 0, &V15706_4_1335, &P15706_5};	/*Property = ScreenPixelsWidth*/
const Profile P15706 = {"15706", &P15706_4};	/*Profile ID - 15706*/
const Value V16518_1_1 = {S2, NULL};	/*Value = True*/
const Property P16518_1 = {S50, 0, &V16518_1_1, NULL};	/*Property = IsMobile*/
const Value V16518_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P16518_5 = {S52, 0, &V16518_5_1333, &P16518_1};	/*Property = ScreenPixelsHeight*/
const Value V16518_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P16518_4 = {S54, 0, &V16518_4_1335, &P16518_5};	/*Property = ScreenPixelsWidth*/
const Profile P16518 = {"16518", &P16518_4};	/*Profile ID - 16518*/
const Value V15708_1_1 = {S2, NULL};	/*Value = True*/
const Property P15708_1 = {S50, 0, &V15708_1_1, NULL};	/*Property = IsMobile*/
const Value V15708_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15708_5 = {S52, 0, &V15708_5_1380, &P15708_1};	/*Property = ScreenPixelsHeight*/
const Value V15708_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P15708_4 = {S54, 0, &V15708_4_1943, &P15708_5};	/*Property = ScreenPixelsWidth*/
const Profile P15708 = {"15708", &P15708_4};	/*Profile ID - 15708*/
const Value V15710_1_1 = {S2, NULL};	/*Value = True*/
const Property P15710_1 = {S50, 0, &V15710_1_1, NULL};	/*Property = IsMobile*/
const Value V15710_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15710_5 = {S52, 0, &V15710_5_1293, &P15710_1};	/*Property = ScreenPixelsHeight*/
const Value V15710_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15710_4 = {S54, 0, &V15710_4_1296, &P15710_5};	/*Property = ScreenPixelsWidth*/
const Profile P15710 = {"15710", &P15710_4};	/*Profile ID - 15710*/
const Value V15711_1_1 = {S2, NULL};	/*Value = True*/
const Property P15711_1 = {S50, 0, &V15711_1_1, NULL};	/*Property = IsMobile*/
const Value V15711_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15711_5 = {S52, 0, &V15711_5_1222, &P15711_1};	/*Property = ScreenPixelsHeight*/
const Value V15711_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15711_4 = {S54, 0, &V15711_4_1224, &P15711_5};	/*Property = ScreenPixelsWidth*/
const Profile P15711 = {"15711", &P15711_4};	/*Profile ID - 15711*/
const Value V15240_1_1 = {S2, NULL};	/*Value = True*/
const Property P15240_1 = {S50, 0, &V15240_1_1, NULL};	/*Property = IsMobile*/
const Value V15240_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15240_5 = {S52, 0, &V15240_5_1293, &P15240_1};	/*Property = ScreenPixelsHeight*/
const Value V15240_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15240_4 = {S54, 0, &V15240_4_1296, &P15240_5};	/*Property = ScreenPixelsWidth*/
const Profile P15240 = {"15240", &P15240_4};	/*Profile ID - 15240*/
const Value V17455_1_2 = {S0, NULL};	/*Value = False*/
const Property P17455_1 = {S50, 0, &V17455_1_2, NULL};	/*Property = IsMobile*/
const Value V17455_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P17455_5 = {S52, 0, &V17455_5_37, &P17455_1};	/*Property = ScreenPixelsHeight*/
const Value V17455_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P17455_4 = {S54, 0, &V17455_4_36, &P17455_5};	/*Property = ScreenPixelsWidth*/
const Profile P17455 = {"17455", &P17455_4};	/*Profile ID - 17455*/
const Value V14465_1_1 = {S2, NULL};	/*Value = True*/
const Property P14465_1 = {S50, 0, &V14465_1_1, NULL};	/*Property = IsMobile*/
const Value V14465_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14465_5 = {S52, 0, &V14465_5_1380, &P14465_1};	/*Property = ScreenPixelsHeight*/
const Value V14465_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14465_4 = {S54, 0, &V14465_4_1279, &P14465_5};	/*Property = ScreenPixelsWidth*/
const Profile P14465 = {"14465", &P14465_4};	/*Profile ID - 14465*/
const Value V15712_1_1 = {S2, NULL};	/*Value = True*/
const Property P15712_1 = {S50, 0, &V15712_1_1, NULL};	/*Property = IsMobile*/
const Value V15712_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15712_5 = {S52, 0, &V15712_5_1233, &P15712_1};	/*Property = ScreenPixelsHeight*/
const Value V15712_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15712_4 = {S54, 0, &V15712_4_1235, &P15712_5};	/*Property = ScreenPixelsWidth*/
const Profile P15712 = {"15712", &P15712_4};	/*Profile ID - 15712*/
const Value V12445_1_1 = {S2, NULL};	/*Value = True*/
const Property P12445_1 = {S50, 0, &V12445_1_1, NULL};	/*Property = IsMobile*/
const Value V12445_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12445_5 = {S52, 0, &V12445_5_1233, &P12445_1};	/*Property = ScreenPixelsHeight*/
const Value V12445_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12445_4 = {S54, 0, &V12445_4_1235, &P12445_5};	/*Property = ScreenPixelsWidth*/
const Profile P12445 = {"12445", &P12445_4};	/*Profile ID - 12445*/
const Value V15714_1_1 = {S2, NULL};	/*Value = True*/
const Property P15714_1 = {S50, 0, &V15714_1_1, NULL};	/*Property = IsMobile*/
const Value V15714_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15714_5 = {S52, 0, &V15714_5_1233, &P15714_1};	/*Property = ScreenPixelsHeight*/
const Value V15714_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15714_4 = {S54, 0, &V15714_4_1235, &P15714_5};	/*Property = ScreenPixelsWidth*/
const Profile P15714 = {"15714", &P15714_4};	/*Profile ID - 15714*/
const Value V16590_1_1 = {S2, NULL};	/*Value = True*/
const Property P16590_1 = {S50, 0, &V16590_1_1, NULL};	/*Property = IsMobile*/
const Value V16590_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16590_5 = {S52, 0, &V16590_5_1294, &P16590_1};	/*Property = ScreenPixelsHeight*/
const Value V16590_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16590_4 = {S54, 0, &V16590_4_1296, &P16590_5};	/*Property = ScreenPixelsWidth*/
const Profile P16590 = {"16590", &P16590_4};	/*Profile ID - 16590*/
const Value V13963_1_1 = {S2, NULL};	/*Value = True*/
const Property P13963_1 = {S50, 0, &V13963_1_1, NULL};	/*Property = IsMobile*/
const Value V13963_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13963_5 = {S52, 0, &V13963_5_1304, &P13963_1};	/*Property = ScreenPixelsHeight*/
const Value V13963_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13963_4 = {S54, 0, &V13963_4_1224, &P13963_5};	/*Property = ScreenPixelsWidth*/
const Profile P13963 = {"13963", &P13963_4};	/*Profile ID - 13963*/
const Value V14183_1_1 = {S2, NULL};	/*Value = True*/
const Property P14183_1 = {S50, 0, &V14183_1_1, NULL};	/*Property = IsMobile*/
const Value V14183_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14183_5 = {S52, 0, &V14183_5_1227, &P14183_1};	/*Property = ScreenPixelsHeight*/
const Value V14183_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14183_4 = {S54, 0, &V14183_4_1229, &P14183_5};	/*Property = ScreenPixelsWidth*/
const Profile P14183 = {"14183", &P14183_4};	/*Profile ID - 14183*/
const Value V15716_1_1 = {S2, NULL};	/*Value = True*/
const Property P15716_1 = {S50, 0, &V15716_1_1, NULL};	/*Property = IsMobile*/
const Value V15716_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15716_5 = {S52, 0, &V15716_5_1304, &P15716_1};	/*Property = ScreenPixelsHeight*/
const Value V15716_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15716_4 = {S54, 0, &V15716_4_1224, &P15716_5};	/*Property = ScreenPixelsWidth*/
const Profile P15716 = {"15716", &P15716_4};	/*Profile ID - 15716*/
const Value V12240_1_1 = {S2, NULL};	/*Value = True*/
const Property P12240_1 = {S50, 0, &V12240_1_1, NULL};	/*Property = IsMobile*/
const Value V12240_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12240_5 = {S52, 0, &V12240_5_1233, &P12240_1};	/*Property = ScreenPixelsHeight*/
const Value V12240_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12240_4 = {S54, 0, &V12240_4_1235, &P12240_5};	/*Property = ScreenPixelsWidth*/
const Profile P12240 = {"12240", &P12240_4};	/*Profile ID - 12240*/
const Value V15718_1_1 = {S2, NULL};	/*Value = True*/
const Property P15718_1 = {S50, 0, &V15718_1_1, NULL};	/*Property = IsMobile*/
const Value V15718_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15718_5 = {S52, 0, &V15718_5_1222, &P15718_1};	/*Property = ScreenPixelsHeight*/
const Value V15718_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15718_4 = {S54, 0, &V15718_4_1224, &P15718_5};	/*Property = ScreenPixelsWidth*/
const Profile P15718 = {"15718", &P15718_4};	/*Profile ID - 15718*/
const Value V15719_1_1 = {S2, NULL};	/*Value = True*/
const Property P15719_1 = {S50, 0, &V15719_1_1, NULL};	/*Property = IsMobile*/
const Value V15719_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P15719_5 = {S52, 0, &V15719_5_1333, &P15719_1};	/*Property = ScreenPixelsHeight*/
const Value V15719_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P15719_4 = {S54, 0, &V15719_4_1335, &P15719_5};	/*Property = ScreenPixelsWidth*/
const Profile P15719 = {"15719", &P15719_4};	/*Profile ID - 15719*/
const Profile P19948 = {"19948", NULL};	/*Profile ID - 19948*/
const Value V15720_1_1 = {S2, NULL};	/*Value = True*/
const Property P15720_1 = {S50, 0, &V15720_1_1, NULL};	/*Property = IsMobile*/
const Value V15720_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15720_5 = {S52, 0, &V15720_5_1380, &P15720_1};	/*Property = ScreenPixelsHeight*/
const Value V15720_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15720_4 = {S54, 0, &V15720_4_1279, &P15720_5};	/*Property = ScreenPixelsWidth*/
const Profile P15720 = {"15720", &P15720_4};	/*Profile ID - 15720*/
const Value V15721_1_1 = {S2, NULL};	/*Value = True*/
const Property P15721_1 = {S50, 0, &V15721_1_1, NULL};	/*Property = IsMobile*/
const Value V15721_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15721_5 = {S52, 0, &V15721_5_1380, &P15721_1};	/*Property = ScreenPixelsHeight*/
const Value V15721_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15721_4 = {S54, 0, &V15721_4_1279, &P15721_5};	/*Property = ScreenPixelsWidth*/
const Profile P15721 = {"15721", &P15721_4};	/*Profile ID - 15721*/
const Value V15724_1_1 = {S2, NULL};	/*Value = True*/
const Property P15724_1 = {S50, 0, &V15724_1_1, NULL};	/*Property = IsMobile*/
const Value V15724_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15724_5 = {S52, 0, &V15724_5_1380, &P15724_1};	/*Property = ScreenPixelsHeight*/
const Value V15724_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15724_4 = {S54, 0, &V15724_4_1279, &P15724_5};	/*Property = ScreenPixelsWidth*/
const Profile P15724 = {"15724", &P15724_4};	/*Profile ID - 15724*/
const Value V15725_1_1 = {S2, NULL};	/*Value = True*/
const Property P15725_1 = {S50, 0, &V15725_1_1, NULL};	/*Property = IsMobile*/
const Value V15725_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15725_5 = {S52, 0, &V15725_5_1233, &P15725_1};	/*Property = ScreenPixelsHeight*/
const Value V15725_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15725_4 = {S54, 0, &V15725_4_1235, &P15725_5};	/*Property = ScreenPixelsWidth*/
const Profile P15725 = {"15725", &P15725_4};	/*Profile ID - 15725*/
const Value V17165_1_1 = {S2, NULL};	/*Value = True*/
const Property P17165_1 = {S50, 0, &V17165_1_1, NULL};	/*Property = IsMobile*/
const Value V17165_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17165_5 = {S52, 0, &V17165_5_1233, &P17165_1};	/*Property = ScreenPixelsHeight*/
const Value V17165_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17165_4 = {S54, 0, &V17165_4_1235, &P17165_5};	/*Property = ScreenPixelsWidth*/
const Profile P17165 = {"17165", &P17165_4};	/*Profile ID - 17165*/
const Value V15726_1_1 = {S2, NULL};	/*Value = True*/
const Property P15726_1 = {S50, 0, &V15726_1_1, NULL};	/*Property = IsMobile*/
const Value V15726_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15726_5 = {S52, 0, &V15726_5_1222, &P15726_1};	/*Property = ScreenPixelsHeight*/
const Value V15726_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15726_4 = {S54, 0, &V15726_4_1224, &P15726_5};	/*Property = ScreenPixelsWidth*/
const Profile P15726 = {"15726", &P15726_4};	/*Profile ID - 15726*/
const Value V15727_1_1 = {S2, NULL};	/*Value = True*/
const Property P15727_1 = {S50, 0, &V15727_1_1, NULL};	/*Property = IsMobile*/
const Value V15727_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15727_5 = {S52, 0, &V15727_5_1277, &P15727_1};	/*Property = ScreenPixelsHeight*/
const Value V15727_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15727_4 = {S54, 0, &V15727_4_1224, &P15727_5};	/*Property = ScreenPixelsWidth*/
const Profile P15727 = {"15727", &P15727_4};	/*Profile ID - 15727*/
const Value V15728_1_1 = {S2, NULL};	/*Value = True*/
const Property P15728_1 = {S50, 0, &V15728_1_1, NULL};	/*Property = IsMobile*/
const Value V15728_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15728_5 = {S52, 0, &V15728_5_1293, &P15728_1};	/*Property = ScreenPixelsHeight*/
const Value V15728_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15728_4 = {S54, 0, &V15728_4_1296, &P15728_5};	/*Property = ScreenPixelsWidth*/
const Profile P15728 = {"15728", &P15728_4};	/*Profile ID - 15728*/
const Value V14444_1_1 = {S2, NULL};	/*Value = True*/
const Property P14444_1 = {S50, 0, &V14444_1_1, NULL};	/*Property = IsMobile*/
const Value V14444_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14444_5 = {S52, 0, &V14444_5_1233, &P14444_1};	/*Property = ScreenPixelsHeight*/
const Value V14444_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14444_4 = {S54, 0, &V14444_4_1235, &P14444_5};	/*Property = ScreenPixelsWidth*/
const Profile P14444 = {"14444", &P14444_4};	/*Profile ID - 14444*/
const Value V14572_1_1 = {S2, NULL};	/*Value = True*/
const Property P14572_1 = {S50, 0, &V14572_1_1, NULL};	/*Property = IsMobile*/
const Value V14572_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14572_5 = {S52, 0, &V14572_5_1380, &P14572_1};	/*Property = ScreenPixelsHeight*/
const Value V14572_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14572_4 = {S54, 0, &V14572_4_1279, &P14572_5};	/*Property = ScreenPixelsWidth*/
const Profile P14572 = {"14572", &P14572_4};	/*Profile ID - 14572*/
const Value V15731_1_1 = {S2, NULL};	/*Value = True*/
const Property P15731_1 = {S50, 0, &V15731_1_1, NULL};	/*Property = IsMobile*/
const Value V15731_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15731_5 = {S52, 0, &V15731_5_1233, &P15731_1};	/*Property = ScreenPixelsHeight*/
const Value V15731_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15731_4 = {S54, 0, &V15731_4_1235, &P15731_5};	/*Property = ScreenPixelsWidth*/
const Profile P15731 = {"15731", &P15731_4};	/*Profile ID - 15731*/
const Value V15732_1_1 = {S2, NULL};	/*Value = True*/
const Property P15732_1 = {S50, 0, &V15732_1_1, NULL};	/*Property = IsMobile*/
const Value V15732_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15732_5 = {S52, 0, &V15732_5_1222, &P15732_1};	/*Property = ScreenPixelsHeight*/
const Value V15732_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15732_4 = {S54, 0, &V15732_4_1224, &P15732_5};	/*Property = ScreenPixelsWidth*/
const Profile P15732 = {"15732", &P15732_4};	/*Profile ID - 15732*/
const Value V15013_1_1 = {S2, NULL};	/*Value = True*/
const Property P15013_1 = {S50, 0, &V15013_1_1, NULL};	/*Property = IsMobile*/
const Value V15013_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15013_5 = {S52, 0, &V15013_5_1233, &P15013_1};	/*Property = ScreenPixelsHeight*/
const Value V15013_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15013_4 = {S54, 0, &V15013_4_1235, &P15013_5};	/*Property = ScreenPixelsWidth*/
const Profile P15013 = {"15013", &P15013_4};	/*Profile ID - 15013*/
const Value V15385_1_1 = {S2, NULL};	/*Value = True*/
const Property P15385_1 = {S50, 0, &V15385_1_1, NULL};	/*Property = IsMobile*/
const Value V15385_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15385_5 = {S52, 0, &V15385_5_1380, &P15385_1};	/*Property = ScreenPixelsHeight*/
const Value V15385_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15385_4 = {S54, 0, &V15385_4_1279, &P15385_5};	/*Property = ScreenPixelsWidth*/
const Profile P15385 = {"15385", &P15385_4};	/*Profile ID - 15385*/
const Value V15734_1_1 = {S2, NULL};	/*Value = True*/
const Property P15734_1 = {S50, 0, &V15734_1_1, NULL};	/*Property = IsMobile*/
const Value V15734_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15734_5 = {S52, 0, &V15734_5_1222, &P15734_1};	/*Property = ScreenPixelsHeight*/
const Value V15734_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15734_4 = {S54, 0, &V15734_4_1224, &P15734_5};	/*Property = ScreenPixelsWidth*/
const Profile P15734 = {"15734", &P15734_4};	/*Profile ID - 15734*/
const Value V15735_1_1 = {S2, NULL};	/*Value = True*/
const Property P15735_1 = {S50, 0, &V15735_1_1, NULL};	/*Property = IsMobile*/
const Value V15735_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15735_5 = {S52, 0, &V15735_5_1222, &P15735_1};	/*Property = ScreenPixelsHeight*/
const Value V15735_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15735_4 = {S54, 0, &V15735_4_1224, &P15735_5};	/*Property = ScreenPixelsWidth*/
const Profile P15735 = {"15735", &P15735_4};	/*Profile ID - 15735*/
const Value V15736_1_1 = {S2, NULL};	/*Value = True*/
const Property P15736_1 = {S50, 0, &V15736_1_1, NULL};	/*Property = IsMobile*/
const Value V15736_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15736_5 = {S52, 0, &V15736_5_1293, &P15736_1};	/*Property = ScreenPixelsHeight*/
const Value V15736_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15736_4 = {S54, 0, &V15736_4_1296, &P15736_5};	/*Property = ScreenPixelsWidth*/
const Profile P15736 = {"15736", &P15736_4};	/*Profile ID - 15736*/
const Value V15581_1_1 = {S2, NULL};	/*Value = True*/
const Property P15581_1 = {S50, 0, &V15581_1_1, NULL};	/*Property = IsMobile*/
const Value V15581_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15581_5 = {S52, 0, &V15581_5_1222, &P15581_1};	/*Property = ScreenPixelsHeight*/
const Value V15581_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15581_4 = {S54, 0, &V15581_4_1224, &P15581_5};	/*Property = ScreenPixelsWidth*/
const Profile P15581 = {"15581", &P15581_4};	/*Profile ID - 15581*/
const Value V15738_1_1 = {S2, NULL};	/*Value = True*/
const Property P15738_1 = {S50, 0, &V15738_1_1, NULL};	/*Property = IsMobile*/
const Value V15738_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15738_5 = {S52, 0, &V15738_5_1293, &P15738_1};	/*Property = ScreenPixelsHeight*/
const Value V15738_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15738_4 = {S54, 0, &V15738_4_1296, &P15738_5};	/*Property = ScreenPixelsWidth*/
const Profile P15738 = {"15738", &P15738_4};	/*Profile ID - 15738*/
const Value V15739_1_1 = {S2, NULL};	/*Value = True*/
const Property P15739_1 = {S50, 0, &V15739_1_1, NULL};	/*Property = IsMobile*/
const Value V15739_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15739_5 = {S52, 0, &V15739_5_1222, &P15739_1};	/*Property = ScreenPixelsHeight*/
const Value V15739_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15739_4 = {S54, 0, &V15739_4_1224, &P15739_5};	/*Property = ScreenPixelsWidth*/
const Profile P15739 = {"15739", &P15739_4};	/*Profile ID - 15739*/
const Value V14359_1_1 = {S2, NULL};	/*Value = True*/
const Property P14359_1 = {S50, 0, &V14359_1_1, NULL};	/*Property = IsMobile*/
const Value V14359_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P14359_5 = {S52, 0, &V14359_5_1275, &P14359_1};	/*Property = ScreenPixelsHeight*/
const Value V14359_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P14359_4 = {S54, 0, &V14359_4_1278, &P14359_5};	/*Property = ScreenPixelsWidth*/
const Profile P14359 = {"14359", &P14359_4};	/*Profile ID - 14359*/
const Value V12170_1_1 = {S2, NULL};	/*Value = True*/
const Property P12170_1 = {S50, 0, &V12170_1_1, NULL};	/*Property = IsMobile*/
const Value V12170_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12170_5 = {S52, 0, &V12170_5_1293, &P12170_1};	/*Property = ScreenPixelsHeight*/
const Value V12170_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12170_4 = {S54, 0, &V12170_4_1296, &P12170_5};	/*Property = ScreenPixelsWidth*/
const Profile P12170 = {"12170", &P12170_4};	/*Profile ID - 12170*/
const Value V15742_1_1 = {S2, NULL};	/*Value = True*/
const Property P15742_1 = {S50, 0, &V15742_1_1, NULL};	/*Property = IsMobile*/
const Value V15742_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15742_5 = {S52, 0, &V15742_5_1233, &P15742_1};	/*Property = ScreenPixelsHeight*/
const Value V15742_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15742_4 = {S54, 0, &V15742_4_1235, &P15742_5};	/*Property = ScreenPixelsWidth*/
const Profile P15742 = {"15742", &P15742_4};	/*Profile ID - 15742*/
const Value V15296_1_1 = {S2, NULL};	/*Value = True*/
const Property P15296_1 = {S50, 0, &V15296_1_1, NULL};	/*Property = IsMobile*/
const Value V15296_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15296_5 = {S52, 0, &V15296_5_1222, &P15296_1};	/*Property = ScreenPixelsHeight*/
const Value V15296_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15296_4 = {S54, 0, &V15296_4_1224, &P15296_5};	/*Property = ScreenPixelsWidth*/
const Profile P15296 = {"15296", &P15296_4};	/*Profile ID - 15296*/
const Value V15743_1_1 = {S2, NULL};	/*Value = True*/
const Property P15743_1 = {S50, 0, &V15743_1_1, NULL};	/*Property = IsMobile*/
const Value V15743_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15743_5 = {S52, 0, &V15743_5_1294, &P15743_1};	/*Property = ScreenPixelsHeight*/
const Value V15743_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15743_4 = {S54, 0, &V15743_4_1296, &P15743_5};	/*Property = ScreenPixelsWidth*/
const Profile P15743 = {"15743", &P15743_4};	/*Profile ID - 15743*/
const Value V19900_1_1 = {S2, NULL};	/*Value = True*/
const Property P19900_1 = {S50, 0, &V19900_1_1, NULL};	/*Property = IsMobile*/
const Value V19900_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P19900_5 = {S52, 0, &V19900_5_1233, &P19900_1};	/*Property = ScreenPixelsHeight*/
const Value V19900_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P19900_4 = {S54, 0, &V19900_4_1235, &P19900_5};	/*Property = ScreenPixelsWidth*/
const Profile P19900 = {"19900", &P19900_4};	/*Profile ID - 19900*/
const Value V15658_1_1 = {S2, NULL};	/*Value = True*/
const Property P15658_1 = {S50, 0, &V15658_1_1, NULL};	/*Property = IsMobile*/
const Value V15658_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15658_5 = {S52, 0, &V15658_5_1222, &P15658_1};	/*Property = ScreenPixelsHeight*/
const Value V15658_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15658_4 = {S54, 0, &V15658_4_1224, &P15658_5};	/*Property = ScreenPixelsWidth*/
const Profile P15658 = {"15658", &P15658_4};	/*Profile ID - 15658*/
const Value V15749_1_1 = {S2, NULL};	/*Value = True*/
const Property P15749_1 = {S50, 0, &V15749_1_1, NULL};	/*Property = IsMobile*/
const Value V15749_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15749_5 = {S52, 0, &V15749_5_1380, &P15749_1};	/*Property = ScreenPixelsHeight*/
const Value V15749_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15749_4 = {S54, 0, &V15749_4_1279, &P15749_5};	/*Property = ScreenPixelsWidth*/
const Profile P15749 = {"15749", &P15749_4};	/*Profile ID - 15749*/
const Value V15750_1_1 = {S2, NULL};	/*Value = True*/
const Property P15750_1 = {S50, 0, &V15750_1_1, NULL};	/*Property = IsMobile*/
const Value V15750_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15750_5 = {S52, 0, &V15750_5_1293, &P15750_1};	/*Property = ScreenPixelsHeight*/
const Value V15750_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15750_4 = {S54, 0, &V15750_4_1296, &P15750_5};	/*Property = ScreenPixelsWidth*/
const Profile P15750 = {"15750", &P15750_4};	/*Profile ID - 15750*/
const Value V15751_1_1 = {S2, NULL};	/*Value = True*/
const Property P15751_1 = {S50, 0, &V15751_1_1, NULL};	/*Property = IsMobile*/
const Value V15751_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15751_5 = {S52, 0, &V15751_5_1222, &P15751_1};	/*Property = ScreenPixelsHeight*/
const Value V15751_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15751_4 = {S54, 0, &V15751_4_1224, &P15751_5};	/*Property = ScreenPixelsWidth*/
const Profile P15751 = {"15751", &P15751_4};	/*Profile ID - 15751*/
const Value V15752_1_1 = {S2, NULL};	/*Value = True*/
const Property P15752_1 = {S50, 0, &V15752_1_1, NULL};	/*Property = IsMobile*/
const Value V15752_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15752_5 = {S52, 0, &V15752_5_1293, &P15752_1};	/*Property = ScreenPixelsHeight*/
const Value V15752_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15752_4 = {S54, 0, &V15752_4_1296, &P15752_5};	/*Property = ScreenPixelsWidth*/
const Profile P15752 = {"15752", &P15752_4};	/*Profile ID - 15752*/
const Value V14417_1_1 = {S2, NULL};	/*Value = True*/
const Property P14417_1 = {S50, 0, &V14417_1_1, NULL};	/*Property = IsMobile*/
const Value V14417_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P14417_5 = {S52, 0, &V14417_5_1275, &P14417_1};	/*Property = ScreenPixelsHeight*/
const Value V14417_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P14417_4 = {S54, 0, &V14417_4_1278, &P14417_5};	/*Property = ScreenPixelsWidth*/
const Profile P14417 = {"14417", &P14417_4};	/*Profile ID - 14417*/
const Value V15744_1_1 = {S2, NULL};	/*Value = True*/
const Property P15744_1 = {S50, 0, &V15744_1_1, NULL};	/*Property = IsMobile*/
const Value V15744_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15744_5 = {S52, 0, &V15744_5_1222, &P15744_1};	/*Property = ScreenPixelsHeight*/
const Value V15744_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15744_4 = {S54, 0, &V15744_4_1224, &P15744_5};	/*Property = ScreenPixelsWidth*/
const Profile P15744 = {"15744", &P15744_4};	/*Profile ID - 15744*/
const Value V15700_1_1 = {S2, NULL};	/*Value = True*/
const Property P15700_1 = {S50, 0, &V15700_1_1, NULL};	/*Property = IsMobile*/
const Value V15700_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15700_5 = {S52, 0, &V15700_5_1233, &P15700_1};	/*Property = ScreenPixelsHeight*/
const Value V15700_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15700_4 = {S54, 0, &V15700_4_1235, &P15700_5};	/*Property = ScreenPixelsWidth*/
const Profile P15700 = {"15700", &P15700_4};	/*Profile ID - 15700*/
const Value V15753_1_1 = {S2, NULL};	/*Value = True*/
const Property P15753_1 = {S50, 0, &V15753_1_1, NULL};	/*Property = IsMobile*/
const Value V15753_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15753_5 = {S52, 0, &V15753_5_1222, &P15753_1};	/*Property = ScreenPixelsHeight*/
const Value V15753_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15753_4 = {S54, 0, &V15753_4_1224, &P15753_5};	/*Property = ScreenPixelsWidth*/
const Profile P15753 = {"15753", &P15753_4};	/*Profile ID - 15753*/
const Value V15754_1_1 = {S2, NULL};	/*Value = True*/
const Property P15754_1 = {S50, 0, &V15754_1_1, NULL};	/*Property = IsMobile*/
const Value V15754_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15754_5 = {S52, 0, &V15754_5_1293, &P15754_1};	/*Property = ScreenPixelsHeight*/
const Value V15754_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15754_4 = {S54, 0, &V15754_4_1296, &P15754_5};	/*Property = ScreenPixelsWidth*/
const Profile P15754 = {"15754", &P15754_4};	/*Profile ID - 15754*/
const Value V15755_1_1 = {S2, NULL};	/*Value = True*/
const Property P15755_1 = {S50, 0, &V15755_1_1, NULL};	/*Property = IsMobile*/
const Value V15755_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15755_5 = {S52, 0, &V15755_5_1233, &P15755_1};	/*Property = ScreenPixelsHeight*/
const Value V15755_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15755_4 = {S54, 0, &V15755_4_1235, &P15755_5};	/*Property = ScreenPixelsWidth*/
const Profile P15755 = {"15755", &P15755_4};	/*Profile ID - 15755*/
const Value V15756_1_1 = {S2, NULL};	/*Value = True*/
const Property P15756_1 = {S50, 0, &V15756_1_1, NULL};	/*Property = IsMobile*/
const Value V15756_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15756_5 = {S52, 0, &V15756_5_1233, &P15756_1};	/*Property = ScreenPixelsHeight*/
const Value V15756_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15756_4 = {S54, 0, &V15756_4_1235, &P15756_5};	/*Property = ScreenPixelsWidth*/
const Profile P15756 = {"15756", &P15756_4};	/*Profile ID - 15756*/
const Value V14415_1_1 = {S2, NULL};	/*Value = True*/
const Property P14415_1 = {S50, 0, &V14415_1_1, NULL};	/*Property = IsMobile*/
const Value V14415_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14415_5 = {S52, 0, &V14415_5_1233, &P14415_1};	/*Property = ScreenPixelsHeight*/
const Value V14415_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14415_4 = {S54, 0, &V14415_4_1235, &P14415_5};	/*Property = ScreenPixelsWidth*/
const Profile P14415 = {"14415", &P14415_4};	/*Profile ID - 14415*/
const Value V15757_1_1 = {S2, NULL};	/*Value = True*/
const Property P15757_1 = {S50, 0, &V15757_1_1, NULL};	/*Property = IsMobile*/
const Value V15757_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15757_5 = {S52, 0, &V15757_5_1227, &P15757_1};	/*Property = ScreenPixelsHeight*/
const Value V15757_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15757_4 = {S54, 0, &V15757_4_1229, &P15757_5};	/*Property = ScreenPixelsWidth*/
const Profile P15757 = {"15757", &P15757_4};	/*Profile ID - 15757*/
const Value V17436_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_279 = {S1, 0, &V17436_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17436_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_282 = {S3, 0, &V17436_282_57155, &P17436_279};	/*Property = BlobBuilder*/
const Value V17436_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_350 = {S4, 0, &V17436_350_57446, &P17436_282};	/*Property = Canvas*/
const Value V17436_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_283 = {S5, 0, &V17436_283_57161, &P17436_350};	/*Property = CssBackground*/
const Value V17436_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_284 = {S6, 0, &V17436_284_57164, &P17436_283};	/*Property = CssBorderImage*/
const Value V17436_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_285 = {S7, 0, &V17436_285_57167, &P17436_284};	/*Property = CssCanvas*/
const Value V17436_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_286 = {S8, 0, &V17436_286_57170, &P17436_285};	/*Property = CssColor*/
const Value V17436_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_237 = {S9, 0, &V17436_237_55786, &P17436_286};	/*Property = CssColumn*/
const Value V17436_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_287 = {S10, 0, &V17436_287_57173, &P17436_237};	/*Property = CssFlexbox*/
const Value V17436_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_288 = {S11, 0, &V17436_288_57176, &P17436_287};	/*Property = CssFont*/
const Value V17436_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_289 = {S12, 0, &V17436_289_86899, &P17436_288};	/*Property = CssImages*/
const Value V17436_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_291 = {S13, 0, &V17436_291_57182, &P17436_289};	/*Property = CssMediaQueries*/
const Value V17436_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_292 = {S14, 0, &V17436_292_57185, &P17436_291};	/*Property = CssMinMax*/
const Value V17436_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_293 = {S15, 0, &V17436_293_57188, &P17436_292};	/*Property = CssOverflow*/
const Value V17436_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_294 = {S16, 0, &V17436_294_57191, &P17436_293};	/*Property = CssPosition*/
const Value V17436_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_295 = {S17, 0, &V17436_295_57194, &P17436_294};	/*Property = CssText*/
const Value V17436_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_297 = {S18, 0, &V17436_297_57197, &P17436_295};	/*Property = CssTransforms*/
const Value V17436_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_241 = {S19, 0, &V17436_241_55801, &P17436_297};	/*Property = CssTransitions*/
const Value V17436_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_298 = {S20, 0, &V17436_298_57208, &P17436_241};	/*Property = CssUI*/
const Value V17436_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_300 = {S21, 0, &V17436_300_57217, &P17436_298};	/*Property = DataSet*/
const Value V17436_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_301 = {S22, 0, &V17436_301_57220, &P17436_300};	/*Property = DataUrl*/
const Value V17436_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_302 = {S23, 0, &V17436_302_57223, &P17436_301};	/*Property = DeviceOrientation*/
const Value V17436_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_304 = {S24, 0, &V17436_304_57229, &P17436_302};	/*Property = FileReader*/
const Value V17436_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_305 = {S25, 0, &V17436_305_57232, &P17436_304};	/*Property = FileSaver*/
const Value V17436_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_306 = {S26, 0, &V17436_306_57238, &P17436_305};	/*Property = FileWriter*/
const Value V17436_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_307 = {S27, 0, &V17436_307_57241, &P17436_306};	/*Property = FormData*/
const Value V17436_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_309 = {S28, 0, &V17436_309_57247, &P17436_307};	/*Property = Fullscreen*/
const Value V17436_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_310 = {S29, 0, &V17436_310_57250, &P17436_309};	/*Property = GeoLocation*/
const Value V17436_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_312 = {S30, 0, &V17436_312_57256, &P17436_310};	/*Property = History*/
const Value V17436_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_314 = {S31, 0, &V17436_314_57262, &P17436_312};	/*Property = Html5*/
const Value V17436_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_313 = {S32, 0, &V17436_313_57259, &P17436_314};	/*Property = Html-Media-Capture*/
const Value V17436_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_315 = {S33, 0, &V17436_315_57265, &P17436_313};	/*Property = Iframe*/
const Value V17436_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_316 = {S34, 0, &V17436_316_57268, &P17436_315};	/*Property = IndexedDB*/
const Value V17436_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_317 = {S35, 0, &V17436_317_57271, &P17436_316};	/*Property = Json*/
const Value V17436_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P17436_15 = {S37, 0, &V17436_15_29, &P17436_317};	/*Property = LayoutEngine*/
const Value V17436_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_344 = {S38, 0, &V17436_344_57274, &P17436_15};	/*Property = Masking*/
const Value V17436_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_320 = {S39, 0, &V17436_320_57292, &P17436_344};	/*Property = PostMessage*/
const Value V17436_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_321 = {S40, 0, &V17436_321_57295, &P17436_320};	/*Property = Progress*/
const Value V17436_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_322 = {S41, 0, &V17436_322_57298, &P17436_321};	/*Property = Prompts*/
const Value V17436_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_323 = {S42, 0, &V17436_323_57301, &P17436_322};	/*Property = Selector*/
const Value V17436_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_324 = {S43, 0, &V17436_324_57307, &P17436_323};	/*Property = Svg*/
const Value V17436_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_327 = {S44, 0, &V17436_327_57316, &P17436_324};	/*Property = TouchEvents*/
const Value V17436_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_328 = {S45, 0, &V17436_328_57319, &P17436_327};	/*Property = Track*/
const Value V17436_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_329 = {S46, 0, &V17436_329_57325, &P17436_328};	/*Property = Video*/
const Value V17436_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_330 = {S47, 0, &V17436_330_57328, &P17436_329};	/*Property = Viewport*/
const Value V17436_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_332 = {S48, 0, &V17436_332_57340, &P17436_330};	/*Property = WebWorkers*/
const Value V17436_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17436_333 = {S49, 0, &V17436_333_57343, &P17436_332};	/*Property = Xhr2*/
const Profile P17436 = {"17436", &P17436_333};	/*Profile ID - 17436*/
const Value V12111_1_1 = {S2, NULL};	/*Value = True*/
const Property P12111_1 = {S50, 0, &V12111_1_1, NULL};	/*Property = IsMobile*/
const Value V12111_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P12111_5 = {S52, 0, &V12111_5_1333, &P12111_1};	/*Property = ScreenPixelsHeight*/
const Value V12111_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P12111_4 = {S54, 0, &V12111_4_1335, &P12111_5};	/*Property = ScreenPixelsWidth*/
const Profile P12111 = {"12111", &P12111_4};	/*Profile ID - 12111*/
const Value V12113_1_1 = {S2, NULL};	/*Value = True*/
const Property P12113_1 = {S50, 0, &V12113_1_1, NULL};	/*Property = IsMobile*/
const Value V12113_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12113_5 = {S52, 0, &V12113_5_1304, &P12113_1};	/*Property = ScreenPixelsHeight*/
const Value V12113_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12113_4 = {S54, 0, &V12113_4_1224, &P12113_5};	/*Property = ScreenPixelsWidth*/
const Profile P12113 = {"12113", &P12113_4};	/*Profile ID - 12113*/
const Value V12121_1_1 = {S2, NULL};	/*Value = True*/
const Property P12121_1 = {S50, 0, &V12121_1_1, NULL};	/*Property = IsMobile*/
const Value V12121_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12121_5 = {S52, 0, &V12121_5_1222, &P12121_1};	/*Property = ScreenPixelsHeight*/
const Value V12121_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12121_4 = {S54, 0, &V12121_4_1224, &P12121_5};	/*Property = ScreenPixelsWidth*/
const Profile P12121 = {"12121", &P12121_4};	/*Profile ID - 12121*/
const Value V12128_1_1 = {S2, NULL};	/*Value = True*/
const Property P12128_1 = {S50, 0, &V12128_1_1, NULL};	/*Property = IsMobile*/
const Value V12128_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12128_5 = {S52, 0, &V12128_5_1294, &P12128_1};	/*Property = ScreenPixelsHeight*/
const Value V12128_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12128_4 = {S54, 0, &V12128_4_1296, &P12128_5};	/*Property = ScreenPixelsWidth*/
const Profile P12128 = {"12128", &P12128_4};	/*Profile ID - 12128*/
const Value V16216_1_1 = {S2, NULL};	/*Value = True*/
const Property P16216_1 = {S50, 0, &V16216_1_1, NULL};	/*Property = IsMobile*/
const Value V16216_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16216_5 = {S52, 0, &V16216_5_1222, &P16216_1};	/*Property = ScreenPixelsHeight*/
const Value V16216_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16216_4 = {S54, 0, &V16216_4_1224, &P16216_5};	/*Property = ScreenPixelsWidth*/
const Profile P16216 = {"16216", &P16216_4};	/*Profile ID - 16216*/
const Value V12317_1_1 = {S2, NULL};	/*Value = True*/
const Property P12317_1 = {S50, 0, &V12317_1_1, NULL};	/*Property = IsMobile*/
const Value V12317_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12317_5 = {S52, 0, &V12317_5_1233, &P12317_1};	/*Property = ScreenPixelsHeight*/
const Value V12317_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12317_4 = {S54, 0, &V12317_4_1235, &P12317_5};	/*Property = ScreenPixelsWidth*/
const Profile P12317 = {"12317", &P12317_4};	/*Profile ID - 12317*/
const Value V12147_1_1 = {S2, NULL};	/*Value = True*/
const Property P12147_1 = {S50, 0, &V12147_1_1, NULL};	/*Property = IsMobile*/
const Value V12147_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12147_5 = {S52, 0, &V12147_5_1233, &P12147_1};	/*Property = ScreenPixelsHeight*/
const Value V12147_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12147_4 = {S54, 0, &V12147_4_1235, &P12147_5};	/*Property = ScreenPixelsWidth*/
const Profile P12147 = {"12147", &P12147_4};	/*Profile ID - 12147*/
const Value V12145_1_1 = {S2, NULL};	/*Value = True*/
const Property P12145_1 = {S50, 0, &V12145_1_1, NULL};	/*Property = IsMobile*/
const Value V12145_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12145_5 = {S52, 0, &V12145_5_1293, &P12145_1};	/*Property = ScreenPixelsHeight*/
const Value V12145_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12145_4 = {S54, 0, &V12145_4_1296, &P12145_5};	/*Property = ScreenPixelsWidth*/
const Profile P12145 = {"12145", &P12145_4};	/*Profile ID - 12145*/
const Value V12149_1_1 = {S2, NULL};	/*Value = True*/
const Property P12149_1 = {S50, 0, &V12149_1_1, NULL};	/*Property = IsMobile*/
const Value V12149_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12149_5 = {S52, 0, &V12149_5_1222, &P12149_1};	/*Property = ScreenPixelsHeight*/
const Value V12149_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12149_4 = {S54, 0, &V12149_4_1224, &P12149_5};	/*Property = ScreenPixelsWidth*/
const Profile P12149 = {"12149", &P12149_4};	/*Profile ID - 12149*/
const Value V12150_1_1 = {S2, NULL};	/*Value = True*/
const Property P12150_1 = {S50, 0, &V12150_1_1, NULL};	/*Property = IsMobile*/
const Value V12150_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12150_5 = {S52, 0, &V12150_5_1222, &P12150_1};	/*Property = ScreenPixelsHeight*/
const Value V12150_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12150_4 = {S54, 0, &V12150_4_1224, &P12150_5};	/*Property = ScreenPixelsWidth*/
const Profile P12150 = {"12150", &P12150_4};	/*Profile ID - 12150*/
const Value V12151_1_1 = {S2, NULL};	/*Value = True*/
const Property P12151_1 = {S50, 0, &V12151_1_1, NULL};	/*Property = IsMobile*/
const Value V12151_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12151_5 = {S52, 0, &V12151_5_1380, &P12151_1};	/*Property = ScreenPixelsHeight*/
const Value V12151_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12151_4 = {S54, 0, &V12151_4_1235, &P12151_5};	/*Property = ScreenPixelsWidth*/
const Profile P12151 = {"12151", &P12151_4};	/*Profile ID - 12151*/
const Value V12156_1_1 = {S2, NULL};	/*Value = True*/
const Property P12156_1 = {S50, 0, &V12156_1_1, NULL};	/*Property = IsMobile*/
const Value V12156_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12156_5 = {S52, 0, &V12156_5_1222, &P12156_1};	/*Property = ScreenPixelsHeight*/
const Value V12156_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12156_4 = {S54, 0, &V12156_4_1224, &P12156_5};	/*Property = ScreenPixelsWidth*/
const Profile P12156 = {"12156", &P12156_4};	/*Profile ID - 12156*/
const Value V13573_1_1 = {S2, NULL};	/*Value = True*/
const Property P13573_1 = {S50, 0, &V13573_1_1, NULL};	/*Property = IsMobile*/
const Value V13573_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13573_5 = {S52, 0, &V13573_5_1233, &P13573_1};	/*Property = ScreenPixelsHeight*/
const Value V13573_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13573_4 = {S54, 0, &V13573_4_1235, &P13573_5};	/*Property = ScreenPixelsWidth*/
const Profile P13573 = {"13573", &P13573_4};	/*Profile ID - 13573*/
const Value V13574_1_1 = {S2, NULL};	/*Value = True*/
const Property P13574_1 = {S50, 0, &V13574_1_1, NULL};	/*Property = IsMobile*/
const Value V13574_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13574_5 = {S52, 0, &V13574_5_1227, &P13574_1};	/*Property = ScreenPixelsHeight*/
const Value V13574_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13574_4 = {S54, 0, &V13574_4_1229, &P13574_5};	/*Property = ScreenPixelsWidth*/
const Profile P13574 = {"13574", &P13574_4};	/*Profile ID - 13574*/
const Value V13580_1_1 = {S2, NULL};	/*Value = True*/
const Property P13580_1 = {S50, 0, &V13580_1_1, NULL};	/*Property = IsMobile*/
const Value V13580_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13580_5 = {S52, 0, &V13580_5_1233, &P13580_1};	/*Property = ScreenPixelsHeight*/
const Value V13580_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13580_4 = {S54, 0, &V13580_4_1235, &P13580_5};	/*Property = ScreenPixelsWidth*/
const Profile P13580 = {"13580", &P13580_4};	/*Profile ID - 13580*/
const Value V13581_1_1 = {S2, NULL};	/*Value = True*/
const Property P13581_1 = {S50, 0, &V13581_1_1, NULL};	/*Property = IsMobile*/
const Value V13581_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13581_5 = {S52, 0, &V13581_5_1233, &P13581_1};	/*Property = ScreenPixelsHeight*/
const Value V13581_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13581_4 = {S54, 0, &V13581_4_1235, &P13581_5};	/*Property = ScreenPixelsWidth*/
const Profile P13581 = {"13581", &P13581_4};	/*Profile ID - 13581*/
const Value V13586_1_1 = {S2, NULL};	/*Value = True*/
const Property P13586_1 = {S50, 0, &V13586_1_1, NULL};	/*Property = IsMobile*/
const Value V13586_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P13586_5 = {S52, 0, &V13586_5_1333, &P13586_1};	/*Property = ScreenPixelsHeight*/
const Value V13586_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P13586_4 = {S54, 0, &V13586_4_1335, &P13586_5};	/*Property = ScreenPixelsWidth*/
const Profile P13586 = {"13586", &P13586_4};	/*Profile ID - 13586*/
const Value V12079_1_1 = {S2, NULL};	/*Value = True*/
const Property P12079_1 = {S50, 0, &V12079_1_1, NULL};	/*Property = IsMobile*/
const Value V12079_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12079_5 = {S52, 0, &V12079_5_1293, &P12079_1};	/*Property = ScreenPixelsHeight*/
const Value V12079_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12079_4 = {S54, 0, &V12079_4_1296, &P12079_5};	/*Property = ScreenPixelsWidth*/
const Profile P12079 = {"12079", &P12079_4};	/*Profile ID - 12079*/
const Value V13588_1_1 = {S2, NULL};	/*Value = True*/
const Property P13588_1 = {S50, 0, &V13588_1_1, NULL};	/*Property = IsMobile*/
const Value V13588_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13588_5 = {S52, 0, &V13588_5_1222, &P13588_1};	/*Property = ScreenPixelsHeight*/
const Value V13588_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13588_4 = {S54, 0, &V13588_4_1224, &P13588_5};	/*Property = ScreenPixelsWidth*/
const Profile P13588 = {"13588", &P13588_4};	/*Profile ID - 13588*/
const Value V13590_1_1 = {S2, NULL};	/*Value = True*/
const Property P13590_1 = {S50, 0, &V13590_1_1, NULL};	/*Property = IsMobile*/
const Value V13590_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13590_5 = {S52, 0, &V13590_5_1277, &P13590_1};	/*Property = ScreenPixelsHeight*/
const Value V13590_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13590_4 = {S54, 0, &V13590_4_1224, &P13590_5};	/*Property = ScreenPixelsWidth*/
const Profile P13590 = {"13590", &P13590_4};	/*Profile ID - 13590*/
const Value V17247_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17247_279 = {S1, 0, &V17247_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17247_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17247_282 = {S3, 0, &V17247_282_57154, &P17247_279};	/*Property = BlobBuilder*/
const Value V17247_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17247_350 = {S4, 0, &V17247_350_57444, &P17247_282};	/*Property = Canvas*/
const Value V17247_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17247_283 = {S5, 0, &V17247_283_57159, &P17247_350};	/*Property = CssBackground*/
const Value V17247_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17247_284 = {S6, 0, &V17247_284_57163, &P17247_283};	/*Property = CssBorderImage*/
const Value V17247_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17247_285 = {S7, 0, &V17247_285_57166, &P17247_284};	/*Property = CssCanvas*/
const Value V17247_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17247_286 = {S8, 0, &V17247_286_57168, &P17247_285};	/*Property = CssColor*/
const Value V17247_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17247_237 = {S9, 0, &V17247_237_55784, &P17247_286};	/*Property = CssColumn*/
const Value V17247_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17247_287 = {S10, 0, &V17247_287_57172, &P17247_237};	/*Property = CssFlexbox*/
const Value V17247_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17247_288 = {S11, 0, &V17247_288_57175, &P17247_287};	/*Property = CssFont*/
const Value V17247_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17247_289 = {S12, 0, &V17247_289_57759, &P17247_288};	/*Property = CssImages*/
const Value V17247_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17247_291 = {S13, 0, &V17247_291_57181, &P17247_289};	/*Property = CssMediaQueries*/
const Value V17247_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17247_292 = {S14, 0, &V17247_292_57183, &P17247_291};	/*Property = CssMinMax*/
const Value V17247_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17247_293 = {S15, 0, &V17247_293_57187, &P17247_292};	/*Property = CssOverflow*/
const Value V17247_294_57190 = {S0, NULL};	/*Value = False*/
const Property P17247_294 = {S16, 0, &V17247_294_57190, &P17247_293};	/*Property = CssPosition*/
const Value V17247_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17247_295 = {S17, 0, &V17247_295_57192, &P17247_294};	/*Property = CssText*/
const Value V17247_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17247_297 = {S18, 0, &V17247_297_57195, &P17247_295};	/*Property = CssTransforms*/
const Value V17247_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17247_241 = {S19, 0, &V17247_241_55799, &P17247_297};	/*Property = CssTransitions*/
const Value V17247_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17247_298 = {S20, 0, &V17247_298_57206, &P17247_241};	/*Property = CssUI*/
const Value V17247_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17247_300 = {S21, 0, &V17247_300_57216, &P17247_298};	/*Property = DataSet*/
const Value V17247_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17247_301 = {S22, 0, &V17247_301_57218, &P17247_300};	/*Property = DataUrl*/
const Value V17247_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17247_302 = {S23, 0, &V17247_302_57222, &P17247_301};	/*Property = DeviceOrientation*/
const Value V17247_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17247_304 = {S24, 0, &V17247_304_57228, &P17247_302};	/*Property = FileReader*/
const Value V17247_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17247_305 = {S25, 0, &V17247_305_57231, &P17247_304};	/*Property = FileSaver*/
const Value V17247_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17247_306 = {S26, 0, &V17247_306_57237, &P17247_305};	/*Property = FileWriter*/
const Value V17247_307_57240 = {S0, NULL};	/*Value = False*/
const Property P17247_307 = {S27, 0, &V17247_307_57240, &P17247_306};	/*Property = FormData*/
const Value V17247_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17247_309 = {S28, 0, &V17247_309_57246, &P17247_307};	/*Property = Fullscreen*/
const Value V17247_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17247_310 = {S29, 0, &V17247_310_57248, &P17247_309};	/*Property = GeoLocation*/
const Value V17247_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17247_312 = {S30, 0, &V17247_312_57255, &P17247_310};	/*Property = History*/
const Value V17247_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17247_314 = {S31, 0, &V17247_314_57261, &P17247_312};	/*Property = Html5*/
const Value V17247_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17247_313 = {S32, 0, &V17247_313_57258, &P17247_314};	/*Property = Html-Media-Capture*/
const Value V17247_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17247_315 = {S33, 0, &V17247_315_57264, &P17247_313};	/*Property = Iframe*/
const Value V17247_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17247_316 = {S34, 0, &V17247_316_57267, &P17247_315};	/*Property = IndexedDB*/
const Value V17247_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17247_317 = {S35, 0, &V17247_317_57269, &P17247_316};	/*Property = Json*/
const Value V17247_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17247_15 = {S37, 0, &V17247_15_28, &P17247_317};	/*Property = LayoutEngine*/
const Value V17247_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17247_344 = {S38, 0, &V17247_344_57272, &P17247_15};	/*Property = Masking*/
const Value V17247_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17247_320 = {S39, 0, &V17247_320_57290, &P17247_344};	/*Property = PostMessage*/
const Value V17247_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17247_321 = {S40, 0, &V17247_321_57293, &P17247_320};	/*Property = Progress*/
const Value V17247_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17247_322 = {S41, 0, &V17247_322_57296, &P17247_321};	/*Property = Prompts*/
const Value V17247_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17247_323 = {S42, 0, &V17247_323_57299, &P17247_322};	/*Property = Selector*/
const Value V17247_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17247_324 = {S43, 0, &V17247_324_57306, &P17247_323};	/*Property = Svg*/
const Value V17247_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17247_327 = {S44, 0, &V17247_327_57315, &P17247_324};	/*Property = TouchEvents*/
const Value V17247_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17247_328 = {S45, 0, &V17247_328_57318, &P17247_327};	/*Property = Track*/
const Value V17247_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17247_329 = {S46, 0, &V17247_329_57323, &P17247_328};	/*Property = Video*/
const Value V17247_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17247_330 = {S47, 0, &V17247_330_57326, &P17247_329};	/*Property = Viewport*/
const Value V17247_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17247_332 = {S48, 0, &V17247_332_57339, &P17247_330};	/*Property = WebWorkers*/
const Value V17247_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17247_333 = {S49, 0, &V17247_333_57342, &P17247_332};	/*Property = Xhr2*/
const Profile P17247 = {"17247", &P17247_333};	/*Profile ID - 17247*/
const Value V12955_1_1 = {S2, NULL};	/*Value = True*/
const Property P12955_1 = {S50, 0, &V12955_1_1, NULL};	/*Property = IsMobile*/
const Value V12955_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12955_5 = {S52, 0, &V12955_5_1227, &P12955_1};	/*Property = ScreenPixelsHeight*/
const Value V12955_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12955_4 = {S54, 0, &V12955_4_1229, &P12955_5};	/*Property = ScreenPixelsWidth*/
const Profile P12955 = {"12955", &P12955_4};	/*Profile ID - 12955*/
const Value V13592_1_1 = {S2, NULL};	/*Value = True*/
const Property P13592_1 = {S50, 0, &V13592_1_1, NULL};	/*Property = IsMobile*/
const Value V13592_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13592_5 = {S52, 0, &V13592_5_1293, &P13592_1};	/*Property = ScreenPixelsHeight*/
const Value V13592_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13592_4 = {S54, 0, &V13592_4_1296, &P13592_5};	/*Property = ScreenPixelsWidth*/
const Profile P13592 = {"13592", &P13592_4};	/*Profile ID - 13592*/
const Value V13593_1_1 = {S2, NULL};	/*Value = True*/
const Property P13593_1 = {S50, 0, &V13593_1_1, NULL};	/*Property = IsMobile*/
const Value V13593_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13593_5 = {S52, 0, &V13593_5_1380, &P13593_1};	/*Property = ScreenPixelsHeight*/
const Value V13593_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13593_4 = {S54, 0, &V13593_4_1279, &P13593_5};	/*Property = ScreenPixelsWidth*/
const Profile P13593 = {"13593", &P13593_4};	/*Profile ID - 13593*/
const Value V13594_1_1 = {S2, NULL};	/*Value = True*/
const Property P13594_1 = {S50, 0, &V13594_1_1, NULL};	/*Property = IsMobile*/
const Value V13594_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13594_5 = {S52, 0, &V13594_5_1304, &P13594_1};	/*Property = ScreenPixelsHeight*/
const Value V13594_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13594_4 = {S54, 0, &V13594_4_1224, &P13594_5};	/*Property = ScreenPixelsWidth*/
const Profile P13594 = {"13594", &P13594_4};	/*Profile ID - 13594*/
const Value V4813_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_279 = {S1, 0, &V4813_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4813_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_282 = {S3, 0, &V4813_282_57155, &P4813_279};	/*Property = BlobBuilder*/
const Value V4813_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_350 = {S4, 0, &V4813_350_57446, &P4813_282};	/*Property = Canvas*/
const Value V4813_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_283 = {S5, 0, &V4813_283_57161, &P4813_350};	/*Property = CssBackground*/
const Value V4813_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_284 = {S6, 0, &V4813_284_57164, &P4813_283};	/*Property = CssBorderImage*/
const Value V4813_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_285 = {S7, 0, &V4813_285_57167, &P4813_284};	/*Property = CssCanvas*/
const Value V4813_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_286 = {S8, 0, &V4813_286_57170, &P4813_285};	/*Property = CssColor*/
const Value V4813_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_237 = {S9, 0, &V4813_237_55786, &P4813_286};	/*Property = CssColumn*/
const Value V4813_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_287 = {S10, 0, &V4813_287_57173, &P4813_237};	/*Property = CssFlexbox*/
const Value V4813_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_288 = {S11, 0, &V4813_288_57176, &P4813_287};	/*Property = CssFont*/
const Value V4813_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_289 = {S12, 0, &V4813_289_86899, &P4813_288};	/*Property = CssImages*/
const Value V4813_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_291 = {S13, 0, &V4813_291_57182, &P4813_289};	/*Property = CssMediaQueries*/
const Value V4813_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_292 = {S14, 0, &V4813_292_57185, &P4813_291};	/*Property = CssMinMax*/
const Value V4813_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_293 = {S15, 0, &V4813_293_57188, &P4813_292};	/*Property = CssOverflow*/
const Value V4813_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_294 = {S16, 0, &V4813_294_57191, &P4813_293};	/*Property = CssPosition*/
const Value V4813_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_295 = {S17, 0, &V4813_295_57194, &P4813_294};	/*Property = CssText*/
const Value V4813_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_297 = {S18, 0, &V4813_297_57197, &P4813_295};	/*Property = CssTransforms*/
const Value V4813_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_241 = {S19, 0, &V4813_241_55801, &P4813_297};	/*Property = CssTransitions*/
const Value V4813_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_298 = {S20, 0, &V4813_298_57208, &P4813_241};	/*Property = CssUI*/
const Value V4813_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_300 = {S21, 0, &V4813_300_57217, &P4813_298};	/*Property = DataSet*/
const Value V4813_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_301 = {S22, 0, &V4813_301_57220, &P4813_300};	/*Property = DataUrl*/
const Value V4813_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_302 = {S23, 0, &V4813_302_57223, &P4813_301};	/*Property = DeviceOrientation*/
const Value V4813_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_304 = {S24, 0, &V4813_304_57229, &P4813_302};	/*Property = FileReader*/
const Value V4813_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_305 = {S25, 0, &V4813_305_57232, &P4813_304};	/*Property = FileSaver*/
const Value V4813_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_306 = {S26, 0, &V4813_306_57238, &P4813_305};	/*Property = FileWriter*/
const Value V4813_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_307 = {S27, 0, &V4813_307_57241, &P4813_306};	/*Property = FormData*/
const Value V4813_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_309 = {S28, 0, &V4813_309_57247, &P4813_307};	/*Property = Fullscreen*/
const Value V4813_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_310 = {S29, 0, &V4813_310_57250, &P4813_309};	/*Property = GeoLocation*/
const Value V4813_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_312 = {S30, 0, &V4813_312_57256, &P4813_310};	/*Property = History*/
const Value V4813_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_314 = {S31, 0, &V4813_314_57262, &P4813_312};	/*Property = Html5*/
const Value V4813_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_313 = {S32, 0, &V4813_313_57259, &P4813_314};	/*Property = Html-Media-Capture*/
const Value V4813_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_315 = {S33, 0, &V4813_315_57265, &P4813_313};	/*Property = Iframe*/
const Value V4813_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_316 = {S34, 0, &V4813_316_57268, &P4813_315};	/*Property = IndexedDB*/
const Value V4813_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_317 = {S35, 0, &V4813_317_57271, &P4813_316};	/*Property = Json*/
const Value V4813_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P4813_15 = {S37, 0, &V4813_15_28, &P4813_317};	/*Property = LayoutEngine*/
const Value V4813_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_344 = {S38, 0, &V4813_344_57274, &P4813_15};	/*Property = Masking*/
const Value V4813_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_320 = {S39, 0, &V4813_320_57292, &P4813_344};	/*Property = PostMessage*/
const Value V4813_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_321 = {S40, 0, &V4813_321_57295, &P4813_320};	/*Property = Progress*/
const Value V4813_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_322 = {S41, 0, &V4813_322_57298, &P4813_321};	/*Property = Prompts*/
const Value V4813_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_323 = {S42, 0, &V4813_323_57301, &P4813_322};	/*Property = Selector*/
const Value V4813_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_324 = {S43, 0, &V4813_324_57307, &P4813_323};	/*Property = Svg*/
const Value V4813_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_327 = {S44, 0, &V4813_327_57316, &P4813_324};	/*Property = TouchEvents*/
const Value V4813_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_328 = {S45, 0, &V4813_328_57319, &P4813_327};	/*Property = Track*/
const Value V4813_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_329 = {S46, 0, &V4813_329_57325, &P4813_328};	/*Property = Video*/
const Value V4813_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_330 = {S47, 0, &V4813_330_57328, &P4813_329};	/*Property = Viewport*/
const Value V4813_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_332 = {S48, 0, &V4813_332_57340, &P4813_330};	/*Property = WebWorkers*/
const Value V4813_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4813_333 = {S49, 0, &V4813_333_57343, &P4813_332};	/*Property = Xhr2*/
const Profile P4813 = {"4813", &P4813_333};	/*Profile ID - 4813*/
const Value V13595_1_1 = {S2, NULL};	/*Value = True*/
const Property P13595_1 = {S50, 0, &V13595_1_1, NULL};	/*Property = IsMobile*/
const Value V13595_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13595_5 = {S52, 0, &V13595_5_1222, &P13595_1};	/*Property = ScreenPixelsHeight*/
const Value V13595_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13595_4 = {S54, 0, &V13595_4_1224, &P13595_5};	/*Property = ScreenPixelsWidth*/
const Profile P13595 = {"13595", &P13595_4};	/*Profile ID - 13595*/
const Value V13597_1_1 = {S2, NULL};	/*Value = True*/
const Property P13597_1 = {S50, 0, &V13597_1_1, NULL};	/*Property = IsMobile*/
const Value V13597_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13597_5 = {S52, 0, &V13597_5_1233, &P13597_1};	/*Property = ScreenPixelsHeight*/
const Value V13597_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13597_4 = {S54, 0, &V13597_4_1235, &P13597_5};	/*Property = ScreenPixelsWidth*/
const Profile P13597 = {"13597", &P13597_4};	/*Profile ID - 13597*/
const Value V13596_1_1 = {S2, NULL};	/*Value = True*/
const Property P13596_1 = {S50, 0, &V13596_1_1, NULL};	/*Property = IsMobile*/
const Value V13596_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P13596_5 = {S52, 0, &V13596_5_1275, &P13596_1};	/*Property = ScreenPixelsHeight*/
const Value V13596_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13596_4 = {S54, 0, &V13596_4_1229, &P13596_5};	/*Property = ScreenPixelsWidth*/
const Profile P13596 = {"13596", &P13596_4};	/*Profile ID - 13596*/
const Value V16789_1_1 = {S2, NULL};	/*Value = True*/
const Property P16789_1 = {S50, 0, &V16789_1_1, NULL};	/*Property = IsMobile*/
const Value V16789_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P16789_5 = {S52, 0, &V16789_5_1380, &P16789_1};	/*Property = ScreenPixelsHeight*/
const Value V16789_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16789_4 = {S54, 0, &V16789_4_1279, &P16789_5};	/*Property = ScreenPixelsWidth*/
const Profile P16789 = {"16789", &P16789_4};	/*Profile ID - 16789*/
const Value V13598_1_1 = {S2, NULL};	/*Value = True*/
const Property P13598_1 = {S50, 0, &V13598_1_1, NULL};	/*Property = IsMobile*/
const Value V13598_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13598_5 = {S52, 0, &V13598_5_1233, &P13598_1};	/*Property = ScreenPixelsHeight*/
const Value V13598_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13598_4 = {S54, 0, &V13598_4_1235, &P13598_5};	/*Property = ScreenPixelsWidth*/
const Profile P13598 = {"13598", &P13598_4};	/*Profile ID - 13598*/
const Value V13599_1_1 = {S2, NULL};	/*Value = True*/
const Property P13599_1 = {S50, 0, &V13599_1_1, NULL};	/*Property = IsMobile*/
const Value V13599_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13599_5 = {S52, 0, &V13599_5_1233, &P13599_1};	/*Property = ScreenPixelsHeight*/
const Value V13599_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13599_4 = {S54, 0, &V13599_4_1235, &P13599_5};	/*Property = ScreenPixelsWidth*/
const Profile P13599 = {"13599", &P13599_4};	/*Profile ID - 13599*/
const Value V13602_1_1 = {S2, NULL};	/*Value = True*/
const Property P13602_1 = {S50, 0, &V13602_1_1, NULL};	/*Property = IsMobile*/
const Value V13602_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13602_5 = {S52, 0, &V13602_5_1304, &P13602_1};	/*Property = ScreenPixelsHeight*/
const Value V13602_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13602_4 = {S54, 0, &V13602_4_1224, &P13602_5};	/*Property = ScreenPixelsWidth*/
const Profile P13602 = {"13602", &P13602_4};	/*Profile ID - 13602*/
const Value V13605_1_1 = {S2, NULL};	/*Value = True*/
const Property P13605_1 = {S50, 0, &V13605_1_1, NULL};	/*Property = IsMobile*/
const Value V13605_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13605_5 = {S52, 0, &V13605_5_1304, &P13605_1};	/*Property = ScreenPixelsHeight*/
const Value V13605_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13605_4 = {S54, 0, &V13605_4_1224, &P13605_5};	/*Property = ScreenPixelsWidth*/
const Profile P13605 = {"13605", &P13605_4};	/*Profile ID - 13605*/
const Value V13611_1_1 = {S2, NULL};	/*Value = True*/
const Property P13611_1 = {S50, 0, &V13611_1_1, NULL};	/*Property = IsMobile*/
const Value V13611_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13611_5 = {S52, 0, &V13611_5_1222, &P13611_1};	/*Property = ScreenPixelsHeight*/
const Value V13611_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13611_4 = {S54, 0, &V13611_4_1224, &P13611_5};	/*Property = ScreenPixelsWidth*/
const Profile P13611 = {"13611", &P13611_4};	/*Profile ID - 13611*/
const Value V13612_1_1 = {S2, NULL};	/*Value = True*/
const Property P13612_1 = {S50, 0, &V13612_1_1, NULL};	/*Property = IsMobile*/
const Value V13612_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13612_5 = {S52, 0, &V13612_5_1233, &P13612_1};	/*Property = ScreenPixelsHeight*/
const Value V13612_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13612_4 = {S54, 0, &V13612_4_1235, &P13612_5};	/*Property = ScreenPixelsWidth*/
const Profile P13612 = {"13612", &P13612_4};	/*Profile ID - 13612*/
const Value V13614_1_1 = {S2, NULL};	/*Value = True*/
const Property P13614_1 = {S50, 0, &V13614_1_1, NULL};	/*Property = IsMobile*/
const Value V13614_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13614_5 = {S52, 0, &V13614_5_1222, &P13614_1};	/*Property = ScreenPixelsHeight*/
const Value V13614_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13614_4 = {S54, 0, &V13614_4_1224, &P13614_5};	/*Property = ScreenPixelsWidth*/
const Profile P13614 = {"13614", &P13614_4};	/*Profile ID - 13614*/
const Value V13615_1_1 = {S2, NULL};	/*Value = True*/
const Property P13615_1 = {S50, 0, &V13615_1_1, NULL};	/*Property = IsMobile*/
const Value V13615_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13615_5 = {S52, 0, &V13615_5_1227, &P13615_1};	/*Property = ScreenPixelsHeight*/
const Value V13615_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13615_4 = {S54, 0, &V13615_4_1229, &P13615_5};	/*Property = ScreenPixelsWidth*/
const Profile P13615 = {"13615", &P13615_4};	/*Profile ID - 13615*/
const Value V13618_1_1 = {S2, NULL};	/*Value = True*/
const Property P13618_1 = {S50, 0, &V13618_1_1, NULL};	/*Property = IsMobile*/
const Value V13618_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13618_5 = {S52, 0, &V13618_5_1294, &P13618_1};	/*Property = ScreenPixelsHeight*/
const Value V13618_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13618_4 = {S54, 0, &V13618_4_1296, &P13618_5};	/*Property = ScreenPixelsWidth*/
const Profile P13618 = {"13618", &P13618_4};	/*Profile ID - 13618*/
const Value V13616_1_1 = {S2, NULL};	/*Value = True*/
const Property P13616_1 = {S50, 0, &V13616_1_1, NULL};	/*Property = IsMobile*/
const Value V13616_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13616_5 = {S52, 0, &V13616_5_1380, &P13616_1};	/*Property = ScreenPixelsHeight*/
const Value V13616_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13616_4 = {S54, 0, &V13616_4_1279, &P13616_5};	/*Property = ScreenPixelsWidth*/
const Profile P13616 = {"13616", &P13616_4};	/*Profile ID - 13616*/
const Value V13620_1_1 = {S2, NULL};	/*Value = True*/
const Property P13620_1 = {S50, 0, &V13620_1_1, NULL};	/*Property = IsMobile*/
const Value V13620_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13620_5 = {S52, 0, &V13620_5_1222, &P13620_1};	/*Property = ScreenPixelsHeight*/
const Value V13620_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13620_4 = {S54, 0, &V13620_4_1224, &P13620_5};	/*Property = ScreenPixelsWidth*/
const Profile P13620 = {"13620", &P13620_4};	/*Profile ID - 13620*/
const Value V13621_1_1 = {S2, NULL};	/*Value = True*/
const Property P13621_1 = {S50, 0, &V13621_1_1, NULL};	/*Property = IsMobile*/
const Value V13621_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13621_5 = {S52, 0, &V13621_5_1233, &P13621_1};	/*Property = ScreenPixelsHeight*/
const Value V13621_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13621_4 = {S54, 0, &V13621_4_1235, &P13621_5};	/*Property = ScreenPixelsWidth*/
const Profile P13621 = {"13621", &P13621_4};	/*Profile ID - 13621*/
const Value V13624_1_1 = {S2, NULL};	/*Value = True*/
const Property P13624_1 = {S50, 0, &V13624_1_1, NULL};	/*Property = IsMobile*/
const Value V13624_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13624_5 = {S52, 0, &V13624_5_1293, &P13624_1};	/*Property = ScreenPixelsHeight*/
const Value V13624_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13624_4 = {S54, 0, &V13624_4_1296, &P13624_5};	/*Property = ScreenPixelsWidth*/
const Profile P13624 = {"13624", &P13624_4};	/*Profile ID - 13624*/
const Value V13626_1_1 = {S2, NULL};	/*Value = True*/
const Property P13626_1 = {S50, 0, &V13626_1_1, NULL};	/*Property = IsMobile*/
const Value V13626_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13626_5 = {S52, 0, &V13626_5_1222, &P13626_1};	/*Property = ScreenPixelsHeight*/
const Value V13626_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13626_4 = {S54, 0, &V13626_4_1224, &P13626_5};	/*Property = ScreenPixelsWidth*/
const Profile P13626 = {"13626", &P13626_4};	/*Profile ID - 13626*/
const Value V13627_1_1 = {S2, NULL};	/*Value = True*/
const Property P13627_1 = {S50, 0, &V13627_1_1, NULL};	/*Property = IsMobile*/
const Value V13627_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13627_5 = {S52, 0, &V13627_5_1304, &P13627_1};	/*Property = ScreenPixelsHeight*/
const Value V13627_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13627_4 = {S54, 0, &V13627_4_1224, &P13627_5};	/*Property = ScreenPixelsWidth*/
const Profile P13627 = {"13627", &P13627_4};	/*Profile ID - 13627*/
const Value V13628_1_1 = {S2, NULL};	/*Value = True*/
const Property P13628_1 = {S50, 0, &V13628_1_1, NULL};	/*Property = IsMobile*/
const Value V13628_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13628_5 = {S52, 0, &V13628_5_1233, &P13628_1};	/*Property = ScreenPixelsHeight*/
const Value V13628_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13628_4 = {S54, 0, &V13628_4_1235, &P13628_5};	/*Property = ScreenPixelsWidth*/
const Profile P13628 = {"13628", &P13628_4};	/*Profile ID - 13628*/
const Value V13555_1_1 = {S2, NULL};	/*Value = True*/
const Property P13555_1 = {S50, 0, &V13555_1_1, NULL};	/*Property = IsMobile*/
const Value V13555_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13555_5 = {S52, 0, &V13555_5_1380, &P13555_1};	/*Property = ScreenPixelsHeight*/
const Value V13555_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13555_4 = {S54, 0, &V13555_4_1279, &P13555_5};	/*Property = ScreenPixelsWidth*/
const Profile P13555 = {"13555", &P13555_4};	/*Profile ID - 13555*/
const Value V13635_1_1 = {S2, NULL};	/*Value = True*/
const Property P13635_1 = {S50, 0, &V13635_1_1, NULL};	/*Property = IsMobile*/
const Value V13635_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13635_5 = {S52, 0, &V13635_5_1293, &P13635_1};	/*Property = ScreenPixelsHeight*/
const Value V13635_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13635_4 = {S54, 0, &V13635_4_1296, &P13635_5};	/*Property = ScreenPixelsWidth*/
const Profile P13635 = {"13635", &P13635_4};	/*Profile ID - 13635*/
const Value V13636_1_1 = {S2, NULL};	/*Value = True*/
const Property P13636_1 = {S50, 0, &V13636_1_1, NULL};	/*Property = IsMobile*/
const Value V13636_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13636_5 = {S52, 0, &V13636_5_1294, &P13636_1};	/*Property = ScreenPixelsHeight*/
const Value V13636_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13636_4 = {S54, 0, &V13636_4_1296, &P13636_5};	/*Property = ScreenPixelsWidth*/
const Profile P13636 = {"13636", &P13636_4};	/*Profile ID - 13636*/
const Value V13637_1_1 = {S2, NULL};	/*Value = True*/
const Property P13637_1 = {S50, 0, &V13637_1_1, NULL};	/*Property = IsMobile*/
const Value V13637_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13637_5 = {S52, 0, &V13637_5_1222, &P13637_1};	/*Property = ScreenPixelsHeight*/
const Value V13637_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13637_4 = {S54, 0, &V13637_4_1224, &P13637_5};	/*Property = ScreenPixelsWidth*/
const Profile P13637 = {"13637", &P13637_4};	/*Profile ID - 13637*/
const Value V4628_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_279 = {S1, 0, &V4628_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4628_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_282 = {S3, 0, &V4628_282_57155, &P4628_279};	/*Property = BlobBuilder*/
const Value V4628_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_350 = {S4, 0, &V4628_350_57446, &P4628_282};	/*Property = Canvas*/
const Value V4628_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_283 = {S5, 0, &V4628_283_57161, &P4628_350};	/*Property = CssBackground*/
const Value V4628_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_284 = {S6, 0, &V4628_284_57164, &P4628_283};	/*Property = CssBorderImage*/
const Value V4628_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_285 = {S7, 0, &V4628_285_57167, &P4628_284};	/*Property = CssCanvas*/
const Value V4628_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_286 = {S8, 0, &V4628_286_57170, &P4628_285};	/*Property = CssColor*/
const Value V4628_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_237 = {S9, 0, &V4628_237_55786, &P4628_286};	/*Property = CssColumn*/
const Value V4628_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_287 = {S10, 0, &V4628_287_57173, &P4628_237};	/*Property = CssFlexbox*/
const Value V4628_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_288 = {S11, 0, &V4628_288_57176, &P4628_287};	/*Property = CssFont*/
const Value V4628_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_289 = {S12, 0, &V4628_289_86899, &P4628_288};	/*Property = CssImages*/
const Value V4628_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_291 = {S13, 0, &V4628_291_57182, &P4628_289};	/*Property = CssMediaQueries*/
const Value V4628_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_292 = {S14, 0, &V4628_292_57185, &P4628_291};	/*Property = CssMinMax*/
const Value V4628_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_293 = {S15, 0, &V4628_293_57188, &P4628_292};	/*Property = CssOverflow*/
const Value V4628_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_294 = {S16, 0, &V4628_294_57191, &P4628_293};	/*Property = CssPosition*/
const Value V4628_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_295 = {S17, 0, &V4628_295_57194, &P4628_294};	/*Property = CssText*/
const Value V4628_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_297 = {S18, 0, &V4628_297_57197, &P4628_295};	/*Property = CssTransforms*/
const Value V4628_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_241 = {S19, 0, &V4628_241_55801, &P4628_297};	/*Property = CssTransitions*/
const Value V4628_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_298 = {S20, 0, &V4628_298_57208, &P4628_241};	/*Property = CssUI*/
const Value V4628_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_300 = {S21, 0, &V4628_300_57217, &P4628_298};	/*Property = DataSet*/
const Value V4628_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_301 = {S22, 0, &V4628_301_57220, &P4628_300};	/*Property = DataUrl*/
const Value V4628_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_302 = {S23, 0, &V4628_302_57223, &P4628_301};	/*Property = DeviceOrientation*/
const Value V4628_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_304 = {S24, 0, &V4628_304_57229, &P4628_302};	/*Property = FileReader*/
const Value V4628_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_305 = {S25, 0, &V4628_305_57232, &P4628_304};	/*Property = FileSaver*/
const Value V4628_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_306 = {S26, 0, &V4628_306_57238, &P4628_305};	/*Property = FileWriter*/
const Value V4628_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_307 = {S27, 0, &V4628_307_57241, &P4628_306};	/*Property = FormData*/
const Value V4628_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_309 = {S28, 0, &V4628_309_57247, &P4628_307};	/*Property = Fullscreen*/
const Value V4628_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_310 = {S29, 0, &V4628_310_57250, &P4628_309};	/*Property = GeoLocation*/
const Value V4628_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_312 = {S30, 0, &V4628_312_57256, &P4628_310};	/*Property = History*/
const Value V4628_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_314 = {S31, 0, &V4628_314_57262, &P4628_312};	/*Property = Html5*/
const Value V4628_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_313 = {S32, 0, &V4628_313_57259, &P4628_314};	/*Property = Html-Media-Capture*/
const Value V4628_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_315 = {S33, 0, &V4628_315_57265, &P4628_313};	/*Property = Iframe*/
const Value V4628_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_316 = {S34, 0, &V4628_316_57268, &P4628_315};	/*Property = IndexedDB*/
const Value V4628_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_317 = {S35, 0, &V4628_317_57271, &P4628_316};	/*Property = Json*/
const Value V4628_15_43265 = {S88, NULL};	/*Value = Lumi*/
const Property P4628_15 = {S37, 0, &V4628_15_43265, &P4628_317};	/*Property = LayoutEngine*/
const Value V4628_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_344 = {S38, 0, &V4628_344_57274, &P4628_15};	/*Property = Masking*/
const Value V4628_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_320 = {S39, 0, &V4628_320_57292, &P4628_344};	/*Property = PostMessage*/
const Value V4628_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_321 = {S40, 0, &V4628_321_57295, &P4628_320};	/*Property = Progress*/
const Value V4628_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_322 = {S41, 0, &V4628_322_57298, &P4628_321};	/*Property = Prompts*/
const Value V4628_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_323 = {S42, 0, &V4628_323_57301, &P4628_322};	/*Property = Selector*/
const Value V4628_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_324 = {S43, 0, &V4628_324_57307, &P4628_323};	/*Property = Svg*/
const Value V4628_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_327 = {S44, 0, &V4628_327_57316, &P4628_324};	/*Property = TouchEvents*/
const Value V4628_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_328 = {S45, 0, &V4628_328_57319, &P4628_327};	/*Property = Track*/
const Value V4628_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_329 = {S46, 0, &V4628_329_57325, &P4628_328};	/*Property = Video*/
const Value V4628_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_330 = {S47, 0, &V4628_330_57328, &P4628_329};	/*Property = Viewport*/
const Value V4628_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_332 = {S48, 0, &V4628_332_57340, &P4628_330};	/*Property = WebWorkers*/
const Value V4628_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4628_333 = {S49, 0, &V4628_333_57343, &P4628_332};	/*Property = Xhr2*/
const Profile P4628 = {"4628", &P4628_333};	/*Profile ID - 4628*/
const Value V13643_1_1 = {S2, NULL};	/*Value = True*/
const Property P13643_1 = {S50, 0, &V13643_1_1, NULL};	/*Property = IsMobile*/
const Value V13643_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13643_5 = {S52, 0, &V13643_5_1233, &P13643_1};	/*Property = ScreenPixelsHeight*/
const Value V13643_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13643_4 = {S54, 0, &V13643_4_1235, &P13643_5};	/*Property = ScreenPixelsWidth*/
const Profile P13643 = {"13643", &P13643_4};	/*Profile ID - 13643*/
const Value V13646_1_1 = {S2, NULL};	/*Value = True*/
const Property P13646_1 = {S50, 0, &V13646_1_1, NULL};	/*Property = IsMobile*/
const Value V13646_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13646_5 = {S52, 0, &V13646_5_1293, &P13646_1};	/*Property = ScreenPixelsHeight*/
const Value V13646_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13646_4 = {S54, 0, &V13646_4_1296, &P13646_5};	/*Property = ScreenPixelsWidth*/
const Profile P13646 = {"13646", &P13646_4};	/*Profile ID - 13646*/
const Value V13649_1_1 = {S2, NULL};	/*Value = True*/
const Property P13649_1 = {S50, 0, &V13649_1_1, NULL};	/*Property = IsMobile*/
const Value V13649_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13649_5 = {S52, 0, &V13649_5_1233, &P13649_1};	/*Property = ScreenPixelsHeight*/
const Value V13649_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13649_4 = {S54, 0, &V13649_4_1235, &P13649_5};	/*Property = ScreenPixelsWidth*/
const Profile P13649 = {"13649", &P13649_4};	/*Profile ID - 13649*/
const Value V13654_1_1 = {S2, NULL};	/*Value = True*/
const Property P13654_1 = {S50, 0, &V13654_1_1, NULL};	/*Property = IsMobile*/
const Value V13654_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13654_5 = {S52, 0, &V13654_5_1222, &P13654_1};	/*Property = ScreenPixelsHeight*/
const Value V13654_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13654_4 = {S54, 0, &V13654_4_1224, &P13654_5};	/*Property = ScreenPixelsWidth*/
const Profile P13654 = {"13654", &P13654_4};	/*Profile ID - 13654*/
const Value V13656_1_1 = {S2, NULL};	/*Value = True*/
const Property P13656_1 = {S50, 0, &V13656_1_1, NULL};	/*Property = IsMobile*/
const Value V13656_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13656_5 = {S52, 0, &V13656_5_1222, &P13656_1};	/*Property = ScreenPixelsHeight*/
const Value V13656_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13656_4 = {S54, 0, &V13656_4_1224, &P13656_5};	/*Property = ScreenPixelsWidth*/
const Profile P13656 = {"13656", &P13656_4};	/*Profile ID - 13656*/
const Value V13658_1_1 = {S2, NULL};	/*Value = True*/
const Property P13658_1 = {S50, 0, &V13658_1_1, NULL};	/*Property = IsMobile*/
const Value V13658_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13658_5 = {S52, 0, &V13658_5_1227, &P13658_1};	/*Property = ScreenPixelsHeight*/
const Value V13658_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13658_4 = {S54, 0, &V13658_4_1229, &P13658_5};	/*Property = ScreenPixelsWidth*/
const Profile P13658 = {"13658", &P13658_4};	/*Profile ID - 13658*/
const Value V12160_1_1 = {S2, NULL};	/*Value = True*/
const Property P12160_1 = {S50, 0, &V12160_1_1, NULL};	/*Property = IsMobile*/
const Value V12160_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12160_5 = {S52, 0, &V12160_5_1227, &P12160_1};	/*Property = ScreenPixelsHeight*/
const Value V12160_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12160_4 = {S54, 0, &V12160_4_1229, &P12160_5};	/*Property = ScreenPixelsWidth*/
const Profile P12160 = {"12160", &P12160_4};	/*Profile ID - 12160*/
const Value V12166_1_1 = {S2, NULL};	/*Value = True*/
const Property P12166_1 = {S50, 0, &V12166_1_1, NULL};	/*Property = IsMobile*/
const Value V12166_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12166_5 = {S52, 0, &V12166_5_1233, &P12166_1};	/*Property = ScreenPixelsHeight*/
const Value V12166_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12166_4 = {S54, 0, &V12166_4_1235, &P12166_5};	/*Property = ScreenPixelsWidth*/
const Profile P12166 = {"12166", &P12166_4};	/*Profile ID - 12166*/
const Value V12168_1_1 = {S2, NULL};	/*Value = True*/
const Property P12168_1 = {S50, 0, &V12168_1_1, NULL};	/*Property = IsMobile*/
const Value V12168_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12168_5 = {S52, 0, &V12168_5_1222, &P12168_1};	/*Property = ScreenPixelsHeight*/
const Value V12168_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12168_4 = {S54, 0, &V12168_4_1224, &P12168_5};	/*Property = ScreenPixelsWidth*/
const Profile P12168 = {"12168", &P12168_4};	/*Profile ID - 12168*/
const Value V16087_1_1 = {S2, NULL};	/*Value = True*/
const Property P16087_1 = {S50, 0, &V16087_1_1, NULL};	/*Property = IsMobile*/
const Value V16087_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16087_5 = {S52, 0, &V16087_5_1227, &P16087_1};	/*Property = ScreenPixelsHeight*/
const Value V16087_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16087_4 = {S54, 0, &V16087_4_1229, &P16087_5};	/*Property = ScreenPixelsWidth*/
const Profile P16087 = {"16087", &P16087_4};	/*Profile ID - 16087*/
const Value V12177_1_1 = {S2, NULL};	/*Value = True*/
const Property P12177_1 = {S50, 0, &V12177_1_1, NULL};	/*Property = IsMobile*/
const Value V12177_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12177_5 = {S52, 0, &V12177_5_1304, &P12177_1};	/*Property = ScreenPixelsHeight*/
const Value V12177_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12177_4 = {S54, 0, &V12177_4_1224, &P12177_5};	/*Property = ScreenPixelsWidth*/
const Profile P12177 = {"12177", &P12177_4};	/*Profile ID - 12177*/
const Value V18028_1_1 = {S2, NULL};	/*Value = True*/
const Property P18028_1 = {S50, 0, &V18028_1_1, NULL};	/*Property = IsMobile*/
const Value V18028_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18028_5 = {S52, 0, &V18028_5_1277, &P18028_1};	/*Property = ScreenPixelsHeight*/
const Value V18028_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18028_4 = {S54, 0, &V18028_4_1279, &P18028_5};	/*Property = ScreenPixelsWidth*/
const Profile P18028 = {"18028", &P18028_4};	/*Profile ID - 18028*/
const Value V12190_1_1 = {S2, NULL};	/*Value = True*/
const Property P12190_1 = {S50, 0, &V12190_1_1, NULL};	/*Property = IsMobile*/
const Value V12190_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12190_5 = {S52, 0, &V12190_5_1233, &P12190_1};	/*Property = ScreenPixelsHeight*/
const Value V12190_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12190_4 = {S54, 0, &V12190_4_1235, &P12190_5};	/*Property = ScreenPixelsWidth*/
const Profile P12190 = {"12190", &P12190_4};	/*Profile ID - 12190*/
const Value V12195_1_1 = {S2, NULL};	/*Value = True*/
const Property P12195_1 = {S50, 0, &V12195_1_1, NULL};	/*Property = IsMobile*/
const Value V12195_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12195_5 = {S52, 0, &V12195_5_1293, &P12195_1};	/*Property = ScreenPixelsHeight*/
const Value V12195_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12195_4 = {S54, 0, &V12195_4_1296, &P12195_5};	/*Property = ScreenPixelsWidth*/
const Profile P12195 = {"12195", &P12195_4};	/*Profile ID - 12195*/
const Value V12199_1_1 = {S2, NULL};	/*Value = True*/
const Property P12199_1 = {S50, 0, &V12199_1_1, NULL};	/*Property = IsMobile*/
const Value V12199_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12199_5 = {S52, 0, &V12199_5_1293, &P12199_1};	/*Property = ScreenPixelsHeight*/
const Value V12199_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12199_4 = {S54, 0, &V12199_4_1296, &P12199_5};	/*Property = ScreenPixelsWidth*/
const Profile P12199 = {"12199", &P12199_4};	/*Profile ID - 12199*/
const Value V12203_1_1 = {S2, NULL};	/*Value = True*/
const Property P12203_1 = {S50, 0, &V12203_1_1, NULL};	/*Property = IsMobile*/
const Value V12203_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12203_5 = {S52, 0, &V12203_5_1222, &P12203_1};	/*Property = ScreenPixelsHeight*/
const Value V12203_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12203_4 = {S54, 0, &V12203_4_1224, &P12203_5};	/*Property = ScreenPixelsWidth*/
const Profile P12203 = {"12203", &P12203_4};	/*Profile ID - 12203*/
const Value V12204_1_1 = {S2, NULL};	/*Value = True*/
const Property P12204_1 = {S50, 0, &V12204_1_1, NULL};	/*Property = IsMobile*/
const Value V12204_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12204_5 = {S52, 0, &V12204_5_1233, &P12204_1};	/*Property = ScreenPixelsHeight*/
const Value V12204_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12204_4 = {S54, 0, &V12204_4_1235, &P12204_5};	/*Property = ScreenPixelsWidth*/
const Profile P12204 = {"12204", &P12204_4};	/*Profile ID - 12204*/
const Value V12201_1_1 = {S2, NULL};	/*Value = True*/
const Property P12201_1 = {S50, 0, &V12201_1_1, NULL};	/*Property = IsMobile*/
const Value V12201_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12201_5 = {S52, 0, &V12201_5_1227, &P12201_1};	/*Property = ScreenPixelsHeight*/
const Value V12201_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12201_4 = {S54, 0, &V12201_4_1229, &P12201_5};	/*Property = ScreenPixelsWidth*/
const Profile P12201 = {"12201", &P12201_4};	/*Profile ID - 12201*/
const Value V12207_1_1 = {S2, NULL};	/*Value = True*/
const Property P12207_1 = {S50, 0, &V12207_1_1, NULL};	/*Property = IsMobile*/
const Value V12207_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12207_5 = {S52, 0, &V12207_5_1222, &P12207_1};	/*Property = ScreenPixelsHeight*/
const Value V12207_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12207_4 = {S54, 0, &V12207_4_1224, &P12207_5};	/*Property = ScreenPixelsWidth*/
const Profile P12207 = {"12207", &P12207_4};	/*Profile ID - 12207*/
const Value V12208_1_1 = {S2, NULL};	/*Value = True*/
const Property P12208_1 = {S50, 0, &V12208_1_1, NULL};	/*Property = IsMobile*/
const Value V12208_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12208_5 = {S52, 0, &V12208_5_1233, &P12208_1};	/*Property = ScreenPixelsHeight*/
const Value V12208_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12208_4 = {S54, 0, &V12208_4_1235, &P12208_5};	/*Property = ScreenPixelsWidth*/
const Profile P12208 = {"12208", &P12208_4};	/*Profile ID - 12208*/
const Value V12211_1_1 = {S2, NULL};	/*Value = True*/
const Property P12211_1 = {S50, 0, &V12211_1_1, NULL};	/*Property = IsMobile*/
const Value V12211_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12211_5 = {S52, 0, &V12211_5_1233, &P12211_1};	/*Property = ScreenPixelsHeight*/
const Value V12211_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12211_4 = {S54, 0, &V12211_4_1235, &P12211_5};	/*Property = ScreenPixelsWidth*/
const Profile P12211 = {"12211", &P12211_4};	/*Profile ID - 12211*/
const Profile P17844 = {"17844", NULL};	/*Profile ID - 17844*/
const Value V15065_1_1 = {S2, NULL};	/*Value = True*/
const Property P15065_1 = {S50, 0, &V15065_1_1, NULL};	/*Property = IsMobile*/
const Value V15065_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15065_5 = {S52, 0, &V15065_5_1277, &P15065_1};	/*Property = ScreenPixelsHeight*/
const Value V15065_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15065_4 = {S54, 0, &V15065_4_1279, &P15065_5};	/*Property = ScreenPixelsWidth*/
const Profile P15065 = {"15065", &P15065_4};	/*Profile ID - 15065*/
const Value V18244_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_279 = {S1, 0, &V18244_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18244_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_282 = {S3, 0, &V18244_282_57155, &P18244_279};	/*Property = BlobBuilder*/
const Value V18244_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_350 = {S4, 0, &V18244_350_57446, &P18244_282};	/*Property = Canvas*/
const Value V18244_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_283 = {S5, 0, &V18244_283_57161, &P18244_350};	/*Property = CssBackground*/
const Value V18244_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_284 = {S6, 0, &V18244_284_57164, &P18244_283};	/*Property = CssBorderImage*/
const Value V18244_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_285 = {S7, 0, &V18244_285_57167, &P18244_284};	/*Property = CssCanvas*/
const Value V18244_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_286 = {S8, 0, &V18244_286_57170, &P18244_285};	/*Property = CssColor*/
const Value V18244_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_237 = {S9, 0, &V18244_237_55786, &P18244_286};	/*Property = CssColumn*/
const Value V18244_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_287 = {S10, 0, &V18244_287_57173, &P18244_237};	/*Property = CssFlexbox*/
const Value V18244_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_288 = {S11, 0, &V18244_288_57176, &P18244_287};	/*Property = CssFont*/
const Value V18244_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_289 = {S12, 0, &V18244_289_86899, &P18244_288};	/*Property = CssImages*/
const Value V18244_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_291 = {S13, 0, &V18244_291_57182, &P18244_289};	/*Property = CssMediaQueries*/
const Value V18244_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_292 = {S14, 0, &V18244_292_57185, &P18244_291};	/*Property = CssMinMax*/
const Value V18244_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_293 = {S15, 0, &V18244_293_57188, &P18244_292};	/*Property = CssOverflow*/
const Value V18244_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_294 = {S16, 0, &V18244_294_57191, &P18244_293};	/*Property = CssPosition*/
const Value V18244_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_295 = {S17, 0, &V18244_295_57194, &P18244_294};	/*Property = CssText*/
const Value V18244_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_297 = {S18, 0, &V18244_297_57197, &P18244_295};	/*Property = CssTransforms*/
const Value V18244_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_241 = {S19, 0, &V18244_241_55801, &P18244_297};	/*Property = CssTransitions*/
const Value V18244_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_298 = {S20, 0, &V18244_298_57208, &P18244_241};	/*Property = CssUI*/
const Value V18244_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_300 = {S21, 0, &V18244_300_57217, &P18244_298};	/*Property = DataSet*/
const Value V18244_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_301 = {S22, 0, &V18244_301_57220, &P18244_300};	/*Property = DataUrl*/
const Value V18244_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_302 = {S23, 0, &V18244_302_57223, &P18244_301};	/*Property = DeviceOrientation*/
const Value V18244_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_304 = {S24, 0, &V18244_304_57229, &P18244_302};	/*Property = FileReader*/
const Value V18244_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_305 = {S25, 0, &V18244_305_57232, &P18244_304};	/*Property = FileSaver*/
const Value V18244_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_306 = {S26, 0, &V18244_306_57238, &P18244_305};	/*Property = FileWriter*/
const Value V18244_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_307 = {S27, 0, &V18244_307_57241, &P18244_306};	/*Property = FormData*/
const Value V18244_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_309 = {S28, 0, &V18244_309_57247, &P18244_307};	/*Property = Fullscreen*/
const Value V18244_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_310 = {S29, 0, &V18244_310_57250, &P18244_309};	/*Property = GeoLocation*/
const Value V18244_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_312 = {S30, 0, &V18244_312_57256, &P18244_310};	/*Property = History*/
const Value V18244_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_314 = {S31, 0, &V18244_314_57262, &P18244_312};	/*Property = Html5*/
const Value V18244_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_313 = {S32, 0, &V18244_313_57259, &P18244_314};	/*Property = Html-Media-Capture*/
const Value V18244_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_315 = {S33, 0, &V18244_315_57265, &P18244_313};	/*Property = Iframe*/
const Value V18244_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_316 = {S34, 0, &V18244_316_57268, &P18244_315};	/*Property = IndexedDB*/
const Value V18244_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_317 = {S35, 0, &V18244_317_57271, &P18244_316};	/*Property = Json*/
const Value V18244_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18244_15 = {S37, 0, &V18244_15_28, &P18244_317};	/*Property = LayoutEngine*/
const Value V18244_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_344 = {S38, 0, &V18244_344_57274, &P18244_15};	/*Property = Masking*/
const Value V18244_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_320 = {S39, 0, &V18244_320_57292, &P18244_344};	/*Property = PostMessage*/
const Value V18244_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_321 = {S40, 0, &V18244_321_57295, &P18244_320};	/*Property = Progress*/
const Value V18244_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_322 = {S41, 0, &V18244_322_57298, &P18244_321};	/*Property = Prompts*/
const Value V18244_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_323 = {S42, 0, &V18244_323_57301, &P18244_322};	/*Property = Selector*/
const Value V18244_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_324 = {S43, 0, &V18244_324_57307, &P18244_323};	/*Property = Svg*/
const Value V18244_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_327 = {S44, 0, &V18244_327_57316, &P18244_324};	/*Property = TouchEvents*/
const Value V18244_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_328 = {S45, 0, &V18244_328_57319, &P18244_327};	/*Property = Track*/
const Value V18244_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_329 = {S46, 0, &V18244_329_57325, &P18244_328};	/*Property = Video*/
const Value V18244_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_330 = {S47, 0, &V18244_330_57328, &P18244_329};	/*Property = Viewport*/
const Value V18244_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_332 = {S48, 0, &V18244_332_57340, &P18244_330};	/*Property = WebWorkers*/
const Value V18244_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18244_333 = {S49, 0, &V18244_333_57343, &P18244_332};	/*Property = Xhr2*/
const Profile P18244 = {"18244", &P18244_333};	/*Profile ID - 18244*/
const Value V12217_1_1 = {S2, NULL};	/*Value = True*/
const Property P12217_1 = {S50, 0, &V12217_1_1, NULL};	/*Property = IsMobile*/
const Value V12217_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12217_5 = {S52, 0, &V12217_5_1233, &P12217_1};	/*Property = ScreenPixelsHeight*/
const Value V12217_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12217_4 = {S54, 0, &V12217_4_1235, &P12217_5};	/*Property = ScreenPixelsWidth*/
const Profile P12217 = {"12217", &P12217_4};	/*Profile ID - 12217*/
const Value V12228_1_1 = {S2, NULL};	/*Value = True*/
const Property P12228_1 = {S50, 0, &V12228_1_1, NULL};	/*Property = IsMobile*/
const Value V12228_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P12228_5 = {S52, 0, &V12228_5_1269, &P12228_1};	/*Property = ScreenPixelsHeight*/
const Value V12228_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12228_4 = {S54, 0, &V12228_4_1229, &P12228_5};	/*Property = ScreenPixelsWidth*/
const Profile P12228 = {"12228", &P12228_4};	/*Profile ID - 12228*/
const Value V3716_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_279 = {S1, 0, &V3716_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3716_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_282 = {S3, 0, &V3716_282_57155, &P3716_279};	/*Property = BlobBuilder*/
const Value V3716_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_350 = {S4, 0, &V3716_350_57446, &P3716_282};	/*Property = Canvas*/
const Value V3716_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_283 = {S5, 0, &V3716_283_57161, &P3716_350};	/*Property = CssBackground*/
const Value V3716_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_284 = {S6, 0, &V3716_284_57164, &P3716_283};	/*Property = CssBorderImage*/
const Value V3716_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_285 = {S7, 0, &V3716_285_57167, &P3716_284};	/*Property = CssCanvas*/
const Value V3716_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_286 = {S8, 0, &V3716_286_57170, &P3716_285};	/*Property = CssColor*/
const Value V3716_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_237 = {S9, 0, &V3716_237_55786, &P3716_286};	/*Property = CssColumn*/
const Value V3716_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_287 = {S10, 0, &V3716_287_57173, &P3716_237};	/*Property = CssFlexbox*/
const Value V3716_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_288 = {S11, 0, &V3716_288_57176, &P3716_287};	/*Property = CssFont*/
const Value V3716_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_289 = {S12, 0, &V3716_289_86899, &P3716_288};	/*Property = CssImages*/
const Value V3716_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_291 = {S13, 0, &V3716_291_57182, &P3716_289};	/*Property = CssMediaQueries*/
const Value V3716_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_292 = {S14, 0, &V3716_292_57185, &P3716_291};	/*Property = CssMinMax*/
const Value V3716_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_293 = {S15, 0, &V3716_293_57188, &P3716_292};	/*Property = CssOverflow*/
const Value V3716_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_294 = {S16, 0, &V3716_294_57191, &P3716_293};	/*Property = CssPosition*/
const Value V3716_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_295 = {S17, 0, &V3716_295_57194, &P3716_294};	/*Property = CssText*/
const Value V3716_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_297 = {S18, 0, &V3716_297_57197, &P3716_295};	/*Property = CssTransforms*/
const Value V3716_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_241 = {S19, 0, &V3716_241_55801, &P3716_297};	/*Property = CssTransitions*/
const Value V3716_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_298 = {S20, 0, &V3716_298_57208, &P3716_241};	/*Property = CssUI*/
const Value V3716_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_300 = {S21, 0, &V3716_300_57217, &P3716_298};	/*Property = DataSet*/
const Value V3716_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_301 = {S22, 0, &V3716_301_57220, &P3716_300};	/*Property = DataUrl*/
const Value V3716_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_302 = {S23, 0, &V3716_302_57223, &P3716_301};	/*Property = DeviceOrientation*/
const Value V3716_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_304 = {S24, 0, &V3716_304_57229, &P3716_302};	/*Property = FileReader*/
const Value V3716_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_305 = {S25, 0, &V3716_305_57232, &P3716_304};	/*Property = FileSaver*/
const Value V3716_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_306 = {S26, 0, &V3716_306_57238, &P3716_305};	/*Property = FileWriter*/
const Value V3716_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_307 = {S27, 0, &V3716_307_57241, &P3716_306};	/*Property = FormData*/
const Value V3716_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_309 = {S28, 0, &V3716_309_57247, &P3716_307};	/*Property = Fullscreen*/
const Value V3716_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_310 = {S29, 0, &V3716_310_57250, &P3716_309};	/*Property = GeoLocation*/
const Value V3716_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_312 = {S30, 0, &V3716_312_57256, &P3716_310};	/*Property = History*/
const Value V3716_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_314 = {S31, 0, &V3716_314_57262, &P3716_312};	/*Property = Html5*/
const Value V3716_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_313 = {S32, 0, &V3716_313_57259, &P3716_314};	/*Property = Html-Media-Capture*/
const Value V3716_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_315 = {S33, 0, &V3716_315_57265, &P3716_313};	/*Property = Iframe*/
const Value V3716_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_316 = {S34, 0, &V3716_316_57268, &P3716_315};	/*Property = IndexedDB*/
const Value V3716_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_317 = {S35, 0, &V3716_317_57271, &P3716_316};	/*Property = Json*/
const Value V3716_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P3716_15 = {S37, 0, &V3716_15_28, &P3716_317};	/*Property = LayoutEngine*/
const Value V3716_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_344 = {S38, 0, &V3716_344_57274, &P3716_15};	/*Property = Masking*/
const Value V3716_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_320 = {S39, 0, &V3716_320_57292, &P3716_344};	/*Property = PostMessage*/
const Value V3716_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_321 = {S40, 0, &V3716_321_57295, &P3716_320};	/*Property = Progress*/
const Value V3716_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_322 = {S41, 0, &V3716_322_57298, &P3716_321};	/*Property = Prompts*/
const Value V3716_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_323 = {S42, 0, &V3716_323_57301, &P3716_322};	/*Property = Selector*/
const Value V3716_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_324 = {S43, 0, &V3716_324_57307, &P3716_323};	/*Property = Svg*/
const Value V3716_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_327 = {S44, 0, &V3716_327_57316, &P3716_324};	/*Property = TouchEvents*/
const Value V3716_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_328 = {S45, 0, &V3716_328_57319, &P3716_327};	/*Property = Track*/
const Value V3716_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_329 = {S46, 0, &V3716_329_57325, &P3716_328};	/*Property = Video*/
const Value V3716_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_330 = {S47, 0, &V3716_330_57328, &P3716_329};	/*Property = Viewport*/
const Value V3716_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_332 = {S48, 0, &V3716_332_57340, &P3716_330};	/*Property = WebWorkers*/
const Value V3716_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3716_333 = {S49, 0, &V3716_333_57343, &P3716_332};	/*Property = Xhr2*/
const Profile P3716 = {"3716", &P3716_333};	/*Profile ID - 3716*/
const Value V12229_1_1 = {S2, NULL};	/*Value = True*/
const Property P12229_1 = {S50, 0, &V12229_1_1, NULL};	/*Property = IsMobile*/
const Value V12229_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12229_5 = {S52, 0, &V12229_5_1222, &P12229_1};	/*Property = ScreenPixelsHeight*/
const Value V12229_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12229_4 = {S54, 0, &V12229_4_1224, &P12229_5};	/*Property = ScreenPixelsWidth*/
const Profile P12229 = {"12229", &P12229_4};	/*Profile ID - 12229*/
const Value V12230_1_1 = {S2, NULL};	/*Value = True*/
const Property P12230_1 = {S50, 0, &V12230_1_1, NULL};	/*Property = IsMobile*/
const Value V12230_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12230_5 = {S52, 0, &V12230_5_1222, &P12230_1};	/*Property = ScreenPixelsHeight*/
const Value V12230_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12230_4 = {S54, 0, &V12230_4_1224, &P12230_5};	/*Property = ScreenPixelsWidth*/
const Profile P12230 = {"12230", &P12230_4};	/*Profile ID - 12230*/
const Value V12231_1_1 = {S2, NULL};	/*Value = True*/
const Property P12231_1 = {S50, 0, &V12231_1_1, NULL};	/*Property = IsMobile*/
const Value V12231_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12231_5 = {S52, 0, &V12231_5_1222, &P12231_1};	/*Property = ScreenPixelsHeight*/
const Value V12231_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12231_4 = {S54, 0, &V12231_4_1224, &P12231_5};	/*Property = ScreenPixelsWidth*/
const Profile P12231 = {"12231", &P12231_4};	/*Profile ID - 12231*/
const Value V12234_1_1 = {S2, NULL};	/*Value = True*/
const Property P12234_1 = {S50, 0, &V12234_1_1, NULL};	/*Property = IsMobile*/
const Value V12234_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P12234_5 = {S52, 0, &V12234_5_1333, &P12234_1};	/*Property = ScreenPixelsHeight*/
const Value V12234_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P12234_4 = {S54, 0, &V12234_4_1335, &P12234_5};	/*Property = ScreenPixelsWidth*/
const Profile P12234 = {"12234", &P12234_4};	/*Profile ID - 12234*/
const Profile P6138 = {"6138", NULL};	/*Profile ID - 6138*/
const Value V4180_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_279 = {S1, 0, &V4180_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4180_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_282 = {S3, 0, &V4180_282_57155, &P4180_279};	/*Property = BlobBuilder*/
const Value V4180_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_350 = {S4, 0, &V4180_350_57446, &P4180_282};	/*Property = Canvas*/
const Value V4180_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_283 = {S5, 0, &V4180_283_57161, &P4180_350};	/*Property = CssBackground*/
const Value V4180_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_284 = {S6, 0, &V4180_284_57164, &P4180_283};	/*Property = CssBorderImage*/
const Value V4180_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_285 = {S7, 0, &V4180_285_57167, &P4180_284};	/*Property = CssCanvas*/
const Value V4180_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_286 = {S8, 0, &V4180_286_57170, &P4180_285};	/*Property = CssColor*/
const Value V4180_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_237 = {S9, 0, &V4180_237_55786, &P4180_286};	/*Property = CssColumn*/
const Value V4180_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_287 = {S10, 0, &V4180_287_57173, &P4180_237};	/*Property = CssFlexbox*/
const Value V4180_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_288 = {S11, 0, &V4180_288_57176, &P4180_287};	/*Property = CssFont*/
const Value V4180_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_289 = {S12, 0, &V4180_289_86899, &P4180_288};	/*Property = CssImages*/
const Value V4180_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_291 = {S13, 0, &V4180_291_57182, &P4180_289};	/*Property = CssMediaQueries*/
const Value V4180_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_292 = {S14, 0, &V4180_292_57185, &P4180_291};	/*Property = CssMinMax*/
const Value V4180_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_293 = {S15, 0, &V4180_293_57188, &P4180_292};	/*Property = CssOverflow*/
const Value V4180_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_294 = {S16, 0, &V4180_294_57191, &P4180_293};	/*Property = CssPosition*/
const Value V4180_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_295 = {S17, 0, &V4180_295_57194, &P4180_294};	/*Property = CssText*/
const Value V4180_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_297 = {S18, 0, &V4180_297_57197, &P4180_295};	/*Property = CssTransforms*/
const Value V4180_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_241 = {S19, 0, &V4180_241_55801, &P4180_297};	/*Property = CssTransitions*/
const Value V4180_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_298 = {S20, 0, &V4180_298_57208, &P4180_241};	/*Property = CssUI*/
const Value V4180_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_300 = {S21, 0, &V4180_300_57217, &P4180_298};	/*Property = DataSet*/
const Value V4180_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_301 = {S22, 0, &V4180_301_57220, &P4180_300};	/*Property = DataUrl*/
const Value V4180_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_302 = {S23, 0, &V4180_302_57223, &P4180_301};	/*Property = DeviceOrientation*/
const Value V4180_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_304 = {S24, 0, &V4180_304_57229, &P4180_302};	/*Property = FileReader*/
const Value V4180_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_305 = {S25, 0, &V4180_305_57232, &P4180_304};	/*Property = FileSaver*/
const Value V4180_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_306 = {S26, 0, &V4180_306_57238, &P4180_305};	/*Property = FileWriter*/
const Value V4180_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_307 = {S27, 0, &V4180_307_57241, &P4180_306};	/*Property = FormData*/
const Value V4180_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_309 = {S28, 0, &V4180_309_57247, &P4180_307};	/*Property = Fullscreen*/
const Value V4180_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_310 = {S29, 0, &V4180_310_57250, &P4180_309};	/*Property = GeoLocation*/
const Value V4180_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_312 = {S30, 0, &V4180_312_57256, &P4180_310};	/*Property = History*/
const Value V4180_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_314 = {S31, 0, &V4180_314_57262, &P4180_312};	/*Property = Html5*/
const Value V4180_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_313 = {S32, 0, &V4180_313_57259, &P4180_314};	/*Property = Html-Media-Capture*/
const Value V4180_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_315 = {S33, 0, &V4180_315_57265, &P4180_313};	/*Property = Iframe*/
const Value V4180_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_316 = {S34, 0, &V4180_316_57268, &P4180_315};	/*Property = IndexedDB*/
const Value V4180_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_317 = {S35, 0, &V4180_317_57271, &P4180_316};	/*Property = Json*/
const Value V4180_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_15 = {S37, 0, &V4180_15_72, &P4180_317};	/*Property = LayoutEngine*/
const Value V4180_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_344 = {S38, 0, &V4180_344_57274, &P4180_15};	/*Property = Masking*/
const Value V4180_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_320 = {S39, 0, &V4180_320_57292, &P4180_344};	/*Property = PostMessage*/
const Value V4180_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_321 = {S40, 0, &V4180_321_57295, &P4180_320};	/*Property = Progress*/
const Value V4180_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_322 = {S41, 0, &V4180_322_57298, &P4180_321};	/*Property = Prompts*/
const Value V4180_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_323 = {S42, 0, &V4180_323_57301, &P4180_322};	/*Property = Selector*/
const Value V4180_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_324 = {S43, 0, &V4180_324_57307, &P4180_323};	/*Property = Svg*/
const Value V4180_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_327 = {S44, 0, &V4180_327_57316, &P4180_324};	/*Property = TouchEvents*/
const Value V4180_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_328 = {S45, 0, &V4180_328_57319, &P4180_327};	/*Property = Track*/
const Value V4180_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_329 = {S46, 0, &V4180_329_57325, &P4180_328};	/*Property = Video*/
const Value V4180_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_330 = {S47, 0, &V4180_330_57328, &P4180_329};	/*Property = Viewport*/
const Value V4180_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_332 = {S48, 0, &V4180_332_57340, &P4180_330};	/*Property = WebWorkers*/
const Value V4180_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4180_333 = {S49, 0, &V4180_333_57343, &P4180_332};	/*Property = Xhr2*/
const Profile P4180 = {"4180", &P4180_333};	/*Profile ID - 4180*/
const Value V12235_1_1 = {S2, NULL};	/*Value = True*/
const Property P12235_1 = {S50, 0, &V12235_1_1, NULL};	/*Property = IsMobile*/
const Value V12235_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12235_5 = {S52, 0, &V12235_5_1277, &P12235_1};	/*Property = ScreenPixelsHeight*/
const Value V12235_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12235_4 = {S54, 0, &V12235_4_1279, &P12235_5};	/*Property = ScreenPixelsWidth*/
const Profile P12235 = {"12235", &P12235_4};	/*Profile ID - 12235*/
const Value V12238_1_1 = {S2, NULL};	/*Value = True*/
const Property P12238_1 = {S50, 0, &V12238_1_1, NULL};	/*Property = IsMobile*/
const Value V12238_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12238_5 = {S52, 0, &V12238_5_1222, &P12238_1};	/*Property = ScreenPixelsHeight*/
const Value V12238_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12238_4 = {S54, 0, &V12238_4_1224, &P12238_5};	/*Property = ScreenPixelsWidth*/
const Profile P12238 = {"12238", &P12238_4};	/*Profile ID - 12238*/
const Value V12239_1_1 = {S2, NULL};	/*Value = True*/
const Property P12239_1 = {S50, 0, &V12239_1_1, NULL};	/*Property = IsMobile*/
const Value V12239_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12239_5 = {S52, 0, &V12239_5_1294, &P12239_1};	/*Property = ScreenPixelsHeight*/
const Value V12239_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12239_4 = {S54, 0, &V12239_4_1296, &P12239_5};	/*Property = ScreenPixelsWidth*/
const Profile P12239 = {"12239", &P12239_4};	/*Profile ID - 12239*/
const Value V12241_1_1 = {S2, NULL};	/*Value = True*/
const Property P12241_1 = {S50, 0, &V12241_1_1, NULL};	/*Property = IsMobile*/
const Value V12241_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12241_5 = {S52, 0, &V12241_5_1293, &P12241_1};	/*Property = ScreenPixelsHeight*/
const Value V12241_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12241_4 = {S54, 0, &V12241_4_1296, &P12241_5};	/*Property = ScreenPixelsWidth*/
const Profile P12241 = {"12241", &P12241_4};	/*Profile ID - 12241*/
const Value V12244_1_1 = {S2, NULL};	/*Value = True*/
const Property P12244_1 = {S50, 0, &V12244_1_1, NULL};	/*Property = IsMobile*/
const Value V12244_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12244_5 = {S52, 0, &V12244_5_1380, &P12244_1};	/*Property = ScreenPixelsHeight*/
const Value V12244_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12244_4 = {S54, 0, &V12244_4_1279, &P12244_5};	/*Property = ScreenPixelsWidth*/
const Profile P12244 = {"12244", &P12244_4};	/*Profile ID - 12244*/
const Value V12242_1_1 = {S2, NULL};	/*Value = True*/
const Property P12242_1 = {S50, 0, &V12242_1_1, NULL};	/*Property = IsMobile*/
const Value V12242_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P12242_5 = {S52, 0, &V12242_5_1275, &P12242_1};	/*Property = ScreenPixelsHeight*/
const Value V12242_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12242_4 = {S54, 0, &V12242_4_1229, &P12242_5};	/*Property = ScreenPixelsWidth*/
const Profile P12242 = {"12242", &P12242_4};	/*Profile ID - 12242*/
const Value V12027_1_1 = {S2, NULL};	/*Value = True*/
const Property P12027_1 = {S50, 0, &V12027_1_1, NULL};	/*Property = IsMobile*/
const Value V12027_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12027_5 = {S52, 0, &V12027_5_1222, &P12027_1};	/*Property = ScreenPixelsHeight*/
const Value V12027_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12027_4 = {S54, 0, &V12027_4_1224, &P12027_5};	/*Property = ScreenPixelsWidth*/
const Profile P12027 = {"12027", &P12027_4};	/*Profile ID - 12027*/
const Value V12257_1_1 = {S2, NULL};	/*Value = True*/
const Property P12257_1 = {S50, 0, &V12257_1_1, NULL};	/*Property = IsMobile*/
const Value V12257_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12257_5 = {S52, 0, &V12257_5_1294, &P12257_1};	/*Property = ScreenPixelsHeight*/
const Value V12257_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12257_4 = {S54, 0, &V12257_4_1296, &P12257_5};	/*Property = ScreenPixelsWidth*/
const Profile P12257 = {"12257", &P12257_4};	/*Profile ID - 12257*/
const Value V12264_1_1 = {S2, NULL};	/*Value = True*/
const Property P12264_1 = {S50, 0, &V12264_1_1, NULL};	/*Property = IsMobile*/
const Value V12264_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12264_5 = {S52, 0, &V12264_5_1380, &P12264_1};	/*Property = ScreenPixelsHeight*/
const Value V12264_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12264_4 = {S54, 0, &V12264_4_1279, &P12264_5};	/*Property = ScreenPixelsWidth*/
const Profile P12264 = {"12264", &P12264_4};	/*Profile ID - 12264*/
const Value V12268_1_1 = {S2, NULL};	/*Value = True*/
const Property P12268_1 = {S50, 0, &V12268_1_1, NULL};	/*Property = IsMobile*/
const Value V12268_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12268_5 = {S52, 0, &V12268_5_1222, &P12268_1};	/*Property = ScreenPixelsHeight*/
const Value V12268_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12268_4 = {S54, 0, &V12268_4_1224, &P12268_5};	/*Property = ScreenPixelsWidth*/
const Profile P12268 = {"12268", &P12268_4};	/*Profile ID - 12268*/
const Value V12269_1_1 = {S2, NULL};	/*Value = True*/
const Property P12269_1 = {S50, 0, &V12269_1_1, NULL};	/*Property = IsMobile*/
const Value V12269_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12269_5 = {S52, 0, &V12269_5_1222, &P12269_1};	/*Property = ScreenPixelsHeight*/
const Value V12269_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12269_4 = {S54, 0, &V12269_4_1224, &P12269_5};	/*Property = ScreenPixelsWidth*/
const Profile P12269 = {"12269", &P12269_4};	/*Profile ID - 12269*/
const Value V12272_1_1 = {S2, NULL};	/*Value = True*/
const Property P12272_1 = {S50, 0, &V12272_1_1, NULL};	/*Property = IsMobile*/
const Value V12272_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12272_5 = {S52, 0, &V12272_5_1227, &P12272_1};	/*Property = ScreenPixelsHeight*/
const Value V12272_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12272_4 = {S54, 0, &V12272_4_1229, &P12272_5};	/*Property = ScreenPixelsWidth*/
const Profile P12272 = {"12272", &P12272_4};	/*Profile ID - 12272*/
const Profile P17137 = {"17137", NULL};	/*Profile ID - 17137*/
const Profile P17131 = {"17131", NULL};	/*Profile ID - 17131*/
const Profile P17084 = {"17084", NULL};	/*Profile ID - 17084*/
const Profile P17144 = {"17144", NULL};	/*Profile ID - 17144*/
const Profile P17350 = {"17350", NULL};	/*Profile ID - 17350*/
const Profile P17134 = {"17134", NULL};	/*Profile ID - 17134*/
const Profile P6274 = {"6274", NULL};	/*Profile ID - 6274*/
const Profile P17141 = {"17141", NULL};	/*Profile ID - 17141*/
const Value V14597_1_1 = {S2, NULL};	/*Value = True*/
const Property P14597_1 = {S50, 0, &V14597_1_1, NULL};	/*Property = IsMobile*/
const Value V14597_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14597_5 = {S52, 0, &V14597_5_1293, &P14597_1};	/*Property = ScreenPixelsHeight*/
const Value V14597_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14597_4 = {S54, 0, &V14597_4_1296, &P14597_5};	/*Property = ScreenPixelsWidth*/
const Profile P14597 = {"14597", &P14597_4};	/*Profile ID - 14597*/
const Value V12286_1_1 = {S2, NULL};	/*Value = True*/
const Property P12286_1 = {S50, 0, &V12286_1_1, NULL};	/*Property = IsMobile*/
const Value V12286_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12286_5 = {S52, 0, &V12286_5_1227, &P12286_1};	/*Property = ScreenPixelsHeight*/
const Value V12286_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12286_4 = {S54, 0, &V12286_4_1229, &P12286_5};	/*Property = ScreenPixelsWidth*/
const Profile P12286 = {"12286", &P12286_4};	/*Profile ID - 12286*/
const Value V13660_1_1 = {S2, NULL};	/*Value = True*/
const Property P13660_1 = {S50, 0, &V13660_1_1, NULL};	/*Property = IsMobile*/
const Value V13660_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13660_5 = {S52, 0, &V13660_5_1222, &P13660_1};	/*Property = ScreenPixelsHeight*/
const Value V13660_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13660_4 = {S54, 0, &V13660_4_1224, &P13660_5};	/*Property = ScreenPixelsWidth*/
const Profile P13660 = {"13660", &P13660_4};	/*Profile ID - 13660*/
const Value V13662_1_1 = {S2, NULL};	/*Value = True*/
const Property P13662_1 = {S50, 0, &V13662_1_1, NULL};	/*Property = IsMobile*/
const Value V13662_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13662_5 = {S52, 0, &V13662_5_1222, &P13662_1};	/*Property = ScreenPixelsHeight*/
const Value V13662_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13662_4 = {S54, 0, &V13662_4_1224, &P13662_5};	/*Property = ScreenPixelsWidth*/
const Profile P13662 = {"13662", &P13662_4};	/*Profile ID - 13662*/
const Value V13663_1_1 = {S2, NULL};	/*Value = True*/
const Property P13663_1 = {S50, 0, &V13663_1_1, NULL};	/*Property = IsMobile*/
const Value V13663_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13663_5 = {S52, 0, &V13663_5_1222, &P13663_1};	/*Property = ScreenPixelsHeight*/
const Value V13663_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13663_4 = {S54, 0, &V13663_4_1224, &P13663_5};	/*Property = ScreenPixelsWidth*/
const Profile P13663 = {"13663", &P13663_4};	/*Profile ID - 13663*/
const Value V12288_1_1 = {S2, NULL};	/*Value = True*/
const Property P12288_1 = {S50, 0, &V12288_1_1, NULL};	/*Property = IsMobile*/
const Value V12288_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12288_5 = {S52, 0, &V12288_5_1293, &P12288_1};	/*Property = ScreenPixelsHeight*/
const Value V12288_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12288_4 = {S54, 0, &V12288_4_1296, &P12288_5};	/*Property = ScreenPixelsWidth*/
const Profile P12288 = {"12288", &P12288_4};	/*Profile ID - 12288*/
const Value V13666_1_1 = {S2, NULL};	/*Value = True*/
const Property P13666_1 = {S50, 0, &V13666_1_1, NULL};	/*Property = IsMobile*/
const Value V13666_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13666_5 = {S52, 0, &V13666_5_1222, &P13666_1};	/*Property = ScreenPixelsHeight*/
const Value V13666_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13666_4 = {S54, 0, &V13666_4_1224, &P13666_5};	/*Property = ScreenPixelsWidth*/
const Profile P13666 = {"13666", &P13666_4};	/*Profile ID - 13666*/
const Value V12294_1_1 = {S2, NULL};	/*Value = True*/
const Property P12294_1 = {S50, 0, &V12294_1_1, NULL};	/*Property = IsMobile*/
const Value V12294_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12294_5 = {S52, 0, &V12294_5_1293, &P12294_1};	/*Property = ScreenPixelsHeight*/
const Value V12294_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12294_4 = {S54, 0, &V12294_4_1296, &P12294_5};	/*Property = ScreenPixelsWidth*/
const Profile P12294 = {"12294", &P12294_4};	/*Profile ID - 12294*/
const Value V17771_1_1 = {S2, NULL};	/*Value = True*/
const Property P17771_1 = {S50, 0, &V17771_1_1, NULL};	/*Property = IsMobile*/
const Value V17771_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17771_5 = {S52, 0, &V17771_5_1222, &P17771_1};	/*Property = ScreenPixelsHeight*/
const Value V17771_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17771_4 = {S54, 0, &V17771_4_1224, &P17771_5};	/*Property = ScreenPixelsWidth*/
const Profile P17771 = {"17771", &P17771_4};	/*Profile ID - 17771*/
const Value V12295_1_1 = {S2, NULL};	/*Value = True*/
const Property P12295_1 = {S50, 0, &V12295_1_1, NULL};	/*Property = IsMobile*/
const Value V12295_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12295_5 = {S52, 0, &V12295_5_1293, &P12295_1};	/*Property = ScreenPixelsHeight*/
const Value V12295_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12295_4 = {S54, 0, &V12295_4_1296, &P12295_5};	/*Property = ScreenPixelsWidth*/
const Profile P12295 = {"12295", &P12295_4};	/*Profile ID - 12295*/
const Value V12291_1_1 = {S2, NULL};	/*Value = True*/
const Property P12291_1 = {S50, 0, &V12291_1_1, NULL};	/*Property = IsMobile*/
const Value V12291_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12291_5 = {S52, 0, &V12291_5_1294, &P12291_1};	/*Property = ScreenPixelsHeight*/
const Value V12291_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12291_4 = {S54, 0, &V12291_4_1296, &P12291_5};	/*Property = ScreenPixelsWidth*/
const Profile P12291 = {"12291", &P12291_4};	/*Profile ID - 12291*/
const Value V12297_1_1 = {S2, NULL};	/*Value = True*/
const Property P12297_1 = {S50, 0, &V12297_1_1, NULL};	/*Property = IsMobile*/
const Value V12297_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12297_5 = {S52, 0, &V12297_5_1380, &P12297_1};	/*Property = ScreenPixelsHeight*/
const Value V12297_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12297_4 = {S54, 0, &V12297_4_1279, &P12297_5};	/*Property = ScreenPixelsWidth*/
const Profile P12297 = {"12297", &P12297_4};	/*Profile ID - 12297*/
const Value V12300_1_1 = {S2, NULL};	/*Value = True*/
const Property P12300_1 = {S50, 0, &V12300_1_1, NULL};	/*Property = IsMobile*/
const Value V12300_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P12300_5 = {S52, 0, &V12300_5_1275, &P12300_1};	/*Property = ScreenPixelsHeight*/
const Value V12300_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12300_4 = {S54, 0, &V12300_4_1229, &P12300_5};	/*Property = ScreenPixelsWidth*/
const Profile P12300 = {"12300", &P12300_4};	/*Profile ID - 12300*/
const Value V12302_1_1 = {S2, NULL};	/*Value = True*/
const Property P12302_1 = {S50, 0, &V12302_1_1, NULL};	/*Property = IsMobile*/
const Value V12302_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12302_5 = {S52, 0, &V12302_5_1233, &P12302_1};	/*Property = ScreenPixelsHeight*/
const Value V12302_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12302_4 = {S54, 0, &V12302_4_1235, &P12302_5};	/*Property = ScreenPixelsWidth*/
const Profile P12302 = {"12302", &P12302_4};	/*Profile ID - 12302*/
const Value V12304_1_1 = {S2, NULL};	/*Value = True*/
const Property P12304_1 = {S50, 0, &V12304_1_1, NULL};	/*Property = IsMobile*/
const Value V12304_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12304_5 = {S52, 0, &V12304_5_1380, &P12304_1};	/*Property = ScreenPixelsHeight*/
const Value V12304_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12304_4 = {S54, 0, &V12304_4_1279, &P12304_5};	/*Property = ScreenPixelsWidth*/
const Profile P12304 = {"12304", &P12304_4};	/*Profile ID - 12304*/
const Value V12306_1_1 = {S2, NULL};	/*Value = True*/
const Property P12306_1 = {S50, 0, &V12306_1_1, NULL};	/*Property = IsMobile*/
const Value V12306_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12306_5 = {S52, 0, &V12306_5_1222, &P12306_1};	/*Property = ScreenPixelsHeight*/
const Value V12306_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12306_4 = {S54, 0, &V12306_4_1224, &P12306_5};	/*Property = ScreenPixelsWidth*/
const Profile P12306 = {"12306", &P12306_4};	/*Profile ID - 12306*/
const Value V12305_1_1 = {S2, NULL};	/*Value = True*/
const Property P12305_1 = {S50, 0, &V12305_1_1, NULL};	/*Property = IsMobile*/
const Value V12305_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12305_5 = {S52, 0, &V12305_5_1227, &P12305_1};	/*Property = ScreenPixelsHeight*/
const Value V12305_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12305_4 = {S54, 0, &V12305_4_1229, &P12305_5};	/*Property = ScreenPixelsWidth*/
const Profile P12305 = {"12305", &P12305_4};	/*Profile ID - 12305*/
const Value V12406_1_1 = {S2, NULL};	/*Value = True*/
const Property P12406_1 = {S50, 0, &V12406_1_1, NULL};	/*Property = IsMobile*/
const Value V12406_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12406_5 = {S52, 0, &V12406_5_1222, &P12406_1};	/*Property = ScreenPixelsHeight*/
const Value V12406_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12406_4 = {S54, 0, &V12406_4_1224, &P12406_5};	/*Property = ScreenPixelsWidth*/
const Profile P12406 = {"12406", &P12406_4};	/*Profile ID - 12406*/
const Value V12315_1_1 = {S2, NULL};	/*Value = True*/
const Property P12315_1 = {S50, 0, &V12315_1_1, NULL};	/*Property = IsMobile*/
const Value V12315_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12315_5 = {S52, 0, &V12315_5_1222, &P12315_1};	/*Property = ScreenPixelsHeight*/
const Value V12315_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12315_4 = {S54, 0, &V12315_4_1224, &P12315_5};	/*Property = ScreenPixelsWidth*/
const Profile P12315 = {"12315", &P12315_4};	/*Profile ID - 12315*/
const Value V12318_1_1 = {S2, NULL};	/*Value = True*/
const Property P12318_1 = {S50, 0, &V12318_1_1, NULL};	/*Property = IsMobile*/
const Value V12318_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12318_5 = {S52, 0, &V12318_5_1293, &P12318_1};	/*Property = ScreenPixelsHeight*/
const Value V12318_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12318_4 = {S54, 0, &V12318_4_1296, &P12318_5};	/*Property = ScreenPixelsWidth*/
const Profile P12318 = {"12318", &P12318_4};	/*Profile ID - 12318*/
const Value V12319_1_1 = {S2, NULL};	/*Value = True*/
const Property P12319_1 = {S50, 0, &V12319_1_1, NULL};	/*Property = IsMobile*/
const Value V12319_5_31826 = {S75, NULL};	/*Value = 260*/
const Property P12319_5 = {S52, 0, &V12319_5_31826, &P12319_1};	/*Property = ScreenPixelsHeight*/
const Value V12319_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12319_4 = {S54, 0, &V12319_4_1224, &P12319_5};	/*Property = ScreenPixelsWidth*/
const Profile P12319 = {"12319", &P12319_4};	/*Profile ID - 12319*/
const Value V12328_1_1 = {S2, NULL};	/*Value = True*/
const Property P12328_1 = {S50, 0, &V12328_1_1, NULL};	/*Property = IsMobile*/
const Value V12328_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12328_5 = {S52, 0, &V12328_5_1293, &P12328_1};	/*Property = ScreenPixelsHeight*/
const Value V12328_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12328_4 = {S54, 0, &V12328_4_1296, &P12328_5};	/*Property = ScreenPixelsWidth*/
const Profile P12328 = {"12328", &P12328_4};	/*Profile ID - 12328*/
const Value V11757_1_1 = {S2, NULL};	/*Value = True*/
const Property P11757_1 = {S50, 0, &V11757_1_1, NULL};	/*Property = IsMobile*/
const Value V11757_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11757_5 = {S52, 0, &V11757_5_1233, &P11757_1};	/*Property = ScreenPixelsHeight*/
const Value V11757_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11757_4 = {S54, 0, &V11757_4_1235, &P11757_5};	/*Property = ScreenPixelsWidth*/
const Profile P11757 = {"11757", &P11757_4};	/*Profile ID - 11757*/
const Value V14062_1_1 = {S2, NULL};	/*Value = True*/
const Property P14062_1 = {S50, 0, &V14062_1_1, NULL};	/*Property = IsMobile*/
const Value V14062_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14062_5 = {S52, 0, &V14062_5_1380, &P14062_1};	/*Property = ScreenPixelsHeight*/
const Value V14062_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14062_4 = {S54, 0, &V14062_4_1279, &P14062_5};	/*Property = ScreenPixelsWidth*/
const Profile P14062 = {"14062", &P14062_4};	/*Profile ID - 14062*/
const Value V22372_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_279 = {S1, 0, &V22372_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22372_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_282 = {S3, 0, &V22372_282_57155, &P22372_279};	/*Property = BlobBuilder*/
const Value V22372_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_350 = {S4, 0, &V22372_350_57446, &P22372_282};	/*Property = Canvas*/
const Value V22372_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_283 = {S5, 0, &V22372_283_57161, &P22372_350};	/*Property = CssBackground*/
const Value V22372_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_284 = {S6, 0, &V22372_284_57164, &P22372_283};	/*Property = CssBorderImage*/
const Value V22372_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_285 = {S7, 0, &V22372_285_57167, &P22372_284};	/*Property = CssCanvas*/
const Value V22372_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_286 = {S8, 0, &V22372_286_57170, &P22372_285};	/*Property = CssColor*/
const Value V22372_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_237 = {S9, 0, &V22372_237_55786, &P22372_286};	/*Property = CssColumn*/
const Value V22372_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_287 = {S10, 0, &V22372_287_57173, &P22372_237};	/*Property = CssFlexbox*/
const Value V22372_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_288 = {S11, 0, &V22372_288_57176, &P22372_287};	/*Property = CssFont*/
const Value V22372_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_289 = {S12, 0, &V22372_289_86899, &P22372_288};	/*Property = CssImages*/
const Value V22372_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_291 = {S13, 0, &V22372_291_57182, &P22372_289};	/*Property = CssMediaQueries*/
const Value V22372_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_292 = {S14, 0, &V22372_292_57185, &P22372_291};	/*Property = CssMinMax*/
const Value V22372_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_293 = {S15, 0, &V22372_293_57188, &P22372_292};	/*Property = CssOverflow*/
const Value V22372_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_294 = {S16, 0, &V22372_294_57191, &P22372_293};	/*Property = CssPosition*/
const Value V22372_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_295 = {S17, 0, &V22372_295_57194, &P22372_294};	/*Property = CssText*/
const Value V22372_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_297 = {S18, 0, &V22372_297_57197, &P22372_295};	/*Property = CssTransforms*/
const Value V22372_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_241 = {S19, 0, &V22372_241_55801, &P22372_297};	/*Property = CssTransitions*/
const Value V22372_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_298 = {S20, 0, &V22372_298_57208, &P22372_241};	/*Property = CssUI*/
const Value V22372_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_300 = {S21, 0, &V22372_300_57217, &P22372_298};	/*Property = DataSet*/
const Value V22372_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_301 = {S22, 0, &V22372_301_57220, &P22372_300};	/*Property = DataUrl*/
const Value V22372_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_302 = {S23, 0, &V22372_302_57223, &P22372_301};	/*Property = DeviceOrientation*/
const Value V22372_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_304 = {S24, 0, &V22372_304_57229, &P22372_302};	/*Property = FileReader*/
const Value V22372_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_305 = {S25, 0, &V22372_305_57232, &P22372_304};	/*Property = FileSaver*/
const Value V22372_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_306 = {S26, 0, &V22372_306_57238, &P22372_305};	/*Property = FileWriter*/
const Value V22372_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_307 = {S27, 0, &V22372_307_57241, &P22372_306};	/*Property = FormData*/
const Value V22372_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_309 = {S28, 0, &V22372_309_57247, &P22372_307};	/*Property = Fullscreen*/
const Value V22372_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_310 = {S29, 0, &V22372_310_57250, &P22372_309};	/*Property = GeoLocation*/
const Value V22372_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_312 = {S30, 0, &V22372_312_57256, &P22372_310};	/*Property = History*/
const Value V22372_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_314 = {S31, 0, &V22372_314_57262, &P22372_312};	/*Property = Html5*/
const Value V22372_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_313 = {S32, 0, &V22372_313_57259, &P22372_314};	/*Property = Html-Media-Capture*/
const Value V22372_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_315 = {S33, 0, &V22372_315_57265, &P22372_313};	/*Property = Iframe*/
const Value V22372_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_316 = {S34, 0, &V22372_316_57268, &P22372_315};	/*Property = IndexedDB*/
const Value V22372_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_317 = {S35, 0, &V22372_317_57271, &P22372_316};	/*Property = Json*/
const Value V22372_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P22372_15 = {S37, 0, &V22372_15_30, &P22372_317};	/*Property = LayoutEngine*/
const Value V22372_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_344 = {S38, 0, &V22372_344_57274, &P22372_15};	/*Property = Masking*/
const Value V22372_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_320 = {S39, 0, &V22372_320_57292, &P22372_344};	/*Property = PostMessage*/
const Value V22372_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_321 = {S40, 0, &V22372_321_57295, &P22372_320};	/*Property = Progress*/
const Value V22372_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_322 = {S41, 0, &V22372_322_57298, &P22372_321};	/*Property = Prompts*/
const Value V22372_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_323 = {S42, 0, &V22372_323_57301, &P22372_322};	/*Property = Selector*/
const Value V22372_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_324 = {S43, 0, &V22372_324_57307, &P22372_323};	/*Property = Svg*/
const Value V22372_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_327 = {S44, 0, &V22372_327_57316, &P22372_324};	/*Property = TouchEvents*/
const Value V22372_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_328 = {S45, 0, &V22372_328_57319, &P22372_327};	/*Property = Track*/
const Value V22372_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_329 = {S46, 0, &V22372_329_57325, &P22372_328};	/*Property = Video*/
const Value V22372_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_330 = {S47, 0, &V22372_330_57328, &P22372_329};	/*Property = Viewport*/
const Value V22372_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_332 = {S48, 0, &V22372_332_57340, &P22372_330};	/*Property = WebWorkers*/
const Value V22372_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22372_333 = {S49, 0, &V22372_333_57343, &P22372_332};	/*Property = Xhr2*/
const Profile P22372 = {"22372", &P22372_333};	/*Profile ID - 22372*/
const Value V23602_279_57148 = {S0, NULL};	/*Value = False*/
const Property P23602_279 = {S1, 0, &V23602_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V23602_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23602_282 = {S3, 0, &V23602_282_57154, &P23602_279};	/*Property = BlobBuilder*/
const Value V23602_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23602_350 = {S4, 0, &V23602_350_57444, &P23602_282};	/*Property = Canvas*/
const Value V23602_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23602_283 = {S5, 0, &V23602_283_57159, &P23602_350};	/*Property = CssBackground*/
const Value V23602_284_57163 = {S0, NULL};	/*Value = False*/
const Property P23602_284 = {S6, 0, &V23602_284_57163, &P23602_283};	/*Property = CssBorderImage*/
const Value V23602_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23602_285 = {S7, 0, &V23602_285_57166, &P23602_284};	/*Property = CssCanvas*/
const Value V23602_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23602_286 = {S8, 0, &V23602_286_57168, &P23602_285};	/*Property = CssColor*/
const Value V23602_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23602_237 = {S9, 0, &V23602_237_55786, &P23602_286};	/*Property = CssColumn*/
const Value V23602_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23602_287 = {S10, 0, &V23602_287_57172, &P23602_237};	/*Property = CssFlexbox*/
const Value V23602_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23602_288 = {S11, 0, &V23602_288_57174, &P23602_287};	/*Property = CssFont*/
const Value V23602_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23602_289 = {S12, 0, &V23602_289_57759, &P23602_288};	/*Property = CssImages*/
const Value V23602_291_57181 = {S0, NULL};	/*Value = False*/
const Property P23602_291 = {S13, 0, &V23602_291_57181, &P23602_289};	/*Property = CssMediaQueries*/
const Value V23602_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23602_292 = {S14, 0, &V23602_292_57183, &P23602_291};	/*Property = CssMinMax*/
const Value V23602_293_57187 = {S0, NULL};	/*Value = False*/
const Property P23602_293 = {S15, 0, &V23602_293_57187, &P23602_292};	/*Property = CssOverflow*/
const Value V23602_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23602_294 = {S16, 0, &V23602_294_57189, &P23602_293};	/*Property = CssPosition*/
const Value V23602_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23602_295 = {S17, 0, &V23602_295_57192, &P23602_294};	/*Property = CssText*/
const Value V23602_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23602_297 = {S18, 0, &V23602_297_57195, &P23602_295};	/*Property = CssTransforms*/
const Value V23602_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23602_241 = {S19, 0, &V23602_241_55799, &P23602_297};	/*Property = CssTransitions*/
const Value V23602_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23602_298 = {S20, 0, &V23602_298_57206, &P23602_241};	/*Property = CssUI*/
const Value V23602_300_57216 = {S0, NULL};	/*Value = False*/
const Property P23602_300 = {S21, 0, &V23602_300_57216, &P23602_298};	/*Property = DataSet*/
const Value V23602_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23602_301 = {S22, 0, &V23602_301_57218, &P23602_300};	/*Property = DataUrl*/
const Value V23602_302_57222 = {S0, NULL};	/*Value = False*/
const Property P23602_302 = {S23, 0, &V23602_302_57222, &P23602_301};	/*Property = DeviceOrientation*/
const Value V23602_304_57228 = {S0, NULL};	/*Value = False*/
const Property P23602_304 = {S24, 0, &V23602_304_57228, &P23602_302};	/*Property = FileReader*/
const Value V23602_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23602_305 = {S25, 0, &V23602_305_57231, &P23602_304};	/*Property = FileSaver*/
const Value V23602_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23602_306 = {S26, 0, &V23602_306_57237, &P23602_305};	/*Property = FileWriter*/
const Value V23602_307_57240 = {S0, NULL};	/*Value = False*/
const Property P23602_307 = {S27, 0, &V23602_307_57240, &P23602_306};	/*Property = FormData*/
const Value V23602_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23602_309 = {S28, 0, &V23602_309_57246, &P23602_307};	/*Property = Fullscreen*/
const Value V23602_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23602_310 = {S29, 0, &V23602_310_57248, &P23602_309};	/*Property = GeoLocation*/
const Value V23602_312_57255 = {S0, NULL};	/*Value = False*/
const Property P23602_312 = {S30, 0, &V23602_312_57255, &P23602_310};	/*Property = History*/
const Value V23602_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23602_314 = {S31, 0, &V23602_314_57261, &P23602_312};	/*Property = Html5*/
const Value V23602_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23602_313 = {S32, 0, &V23602_313_57258, &P23602_314};	/*Property = Html-Media-Capture*/
const Value V23602_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23602_315 = {S33, 0, &V23602_315_57264, &P23602_313};	/*Property = Iframe*/
const Value V23602_316_57267 = {S0, NULL};	/*Value = False*/
const Property P23602_316 = {S34, 0, &V23602_316_57267, &P23602_315};	/*Property = IndexedDB*/
const Value V23602_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23602_317 = {S35, 0, &V23602_317_57269, &P23602_316};	/*Property = Json*/
const Value V23602_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P23602_15 = {S37, 0, &V23602_15_72, &P23602_317};	/*Property = LayoutEngine*/
const Value V23602_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23602_344 = {S38, 0, &V23602_344_57272, &P23602_15};	/*Property = Masking*/
const Value V23602_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23602_320 = {S39, 0, &V23602_320_57290, &P23602_344};	/*Property = PostMessage*/
const Value V23602_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23602_321 = {S40, 0, &V23602_321_57293, &P23602_320};	/*Property = Progress*/
const Value V23602_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23602_322 = {S41, 0, &V23602_322_57296, &P23602_321};	/*Property = Prompts*/
const Value V23602_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23602_323 = {S42, 0, &V23602_323_57299, &P23602_322};	/*Property = Selector*/
const Value V23602_324_57306 = {S0, NULL};	/*Value = False*/
const Property P23602_324 = {S43, 0, &V23602_324_57306, &P23602_323};	/*Property = Svg*/
const Value V23602_327_57315 = {S0, NULL};	/*Value = False*/
const Property P23602_327 = {S44, 0, &V23602_327_57315, &P23602_324};	/*Property = TouchEvents*/
const Value V23602_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23602_328 = {S45, 0, &V23602_328_57318, &P23602_327};	/*Property = Track*/
const Value V23602_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23602_329 = {S46, 0, &V23602_329_57323, &P23602_328};	/*Property = Video*/
const Value V23602_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23602_330 = {S47, 0, &V23602_330_57326, &P23602_329};	/*Property = Viewport*/
const Value V23602_332_57339 = {S0, NULL};	/*Value = False*/
const Property P23602_332 = {S48, 0, &V23602_332_57339, &P23602_330};	/*Property = WebWorkers*/
const Value V23602_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23602_333 = {S49, 0, &V23602_333_57342, &P23602_332};	/*Property = Xhr2*/
const Profile P23602 = {"23602", &P23602_333};	/*Profile ID - 23602*/
const Profile P7480 = {"7480", NULL};	/*Profile ID - 7480*/
const Value V18245_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_279 = {S1, 0, &V18245_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18245_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_282 = {S3, 0, &V18245_282_57155, &P18245_279};	/*Property = BlobBuilder*/
const Value V18245_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_350 = {S4, 0, &V18245_350_57446, &P18245_282};	/*Property = Canvas*/
const Value V18245_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_283 = {S5, 0, &V18245_283_57161, &P18245_350};	/*Property = CssBackground*/
const Value V18245_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_284 = {S6, 0, &V18245_284_57164, &P18245_283};	/*Property = CssBorderImage*/
const Value V18245_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_285 = {S7, 0, &V18245_285_57167, &P18245_284};	/*Property = CssCanvas*/
const Value V18245_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_286 = {S8, 0, &V18245_286_57170, &P18245_285};	/*Property = CssColor*/
const Value V18245_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_237 = {S9, 0, &V18245_237_55786, &P18245_286};	/*Property = CssColumn*/
const Value V18245_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_287 = {S10, 0, &V18245_287_57173, &P18245_237};	/*Property = CssFlexbox*/
const Value V18245_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_288 = {S11, 0, &V18245_288_57176, &P18245_287};	/*Property = CssFont*/
const Value V18245_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_289 = {S12, 0, &V18245_289_86899, &P18245_288};	/*Property = CssImages*/
const Value V18245_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_291 = {S13, 0, &V18245_291_57182, &P18245_289};	/*Property = CssMediaQueries*/
const Value V18245_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_292 = {S14, 0, &V18245_292_57185, &P18245_291};	/*Property = CssMinMax*/
const Value V18245_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_293 = {S15, 0, &V18245_293_57188, &P18245_292};	/*Property = CssOverflow*/
const Value V18245_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_294 = {S16, 0, &V18245_294_57191, &P18245_293};	/*Property = CssPosition*/
const Value V18245_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_295 = {S17, 0, &V18245_295_57194, &P18245_294};	/*Property = CssText*/
const Value V18245_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_297 = {S18, 0, &V18245_297_57197, &P18245_295};	/*Property = CssTransforms*/
const Value V18245_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_241 = {S19, 0, &V18245_241_55801, &P18245_297};	/*Property = CssTransitions*/
const Value V18245_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_298 = {S20, 0, &V18245_298_57208, &P18245_241};	/*Property = CssUI*/
const Value V18245_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_300 = {S21, 0, &V18245_300_57217, &P18245_298};	/*Property = DataSet*/
const Value V18245_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_301 = {S22, 0, &V18245_301_57220, &P18245_300};	/*Property = DataUrl*/
const Value V18245_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_302 = {S23, 0, &V18245_302_57223, &P18245_301};	/*Property = DeviceOrientation*/
const Value V18245_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_304 = {S24, 0, &V18245_304_57229, &P18245_302};	/*Property = FileReader*/
const Value V18245_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_305 = {S25, 0, &V18245_305_57232, &P18245_304};	/*Property = FileSaver*/
const Value V18245_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_306 = {S26, 0, &V18245_306_57238, &P18245_305};	/*Property = FileWriter*/
const Value V18245_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_307 = {S27, 0, &V18245_307_57241, &P18245_306};	/*Property = FormData*/
const Value V18245_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_309 = {S28, 0, &V18245_309_57247, &P18245_307};	/*Property = Fullscreen*/
const Value V18245_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_310 = {S29, 0, &V18245_310_57250, &P18245_309};	/*Property = GeoLocation*/
const Value V18245_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_312 = {S30, 0, &V18245_312_57256, &P18245_310};	/*Property = History*/
const Value V18245_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_314 = {S31, 0, &V18245_314_57262, &P18245_312};	/*Property = Html5*/
const Value V18245_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_313 = {S32, 0, &V18245_313_57259, &P18245_314};	/*Property = Html-Media-Capture*/
const Value V18245_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_315 = {S33, 0, &V18245_315_57265, &P18245_313};	/*Property = Iframe*/
const Value V18245_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_316 = {S34, 0, &V18245_316_57268, &P18245_315};	/*Property = IndexedDB*/
const Value V18245_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_317 = {S35, 0, &V18245_317_57271, &P18245_316};	/*Property = Json*/
const Value V18245_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18245_15 = {S37, 0, &V18245_15_28, &P18245_317};	/*Property = LayoutEngine*/
const Value V18245_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_344 = {S38, 0, &V18245_344_57274, &P18245_15};	/*Property = Masking*/
const Value V18245_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_320 = {S39, 0, &V18245_320_57292, &P18245_344};	/*Property = PostMessage*/
const Value V18245_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_321 = {S40, 0, &V18245_321_57295, &P18245_320};	/*Property = Progress*/
const Value V18245_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_322 = {S41, 0, &V18245_322_57298, &P18245_321};	/*Property = Prompts*/
const Value V18245_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_323 = {S42, 0, &V18245_323_57301, &P18245_322};	/*Property = Selector*/
const Value V18245_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_324 = {S43, 0, &V18245_324_57307, &P18245_323};	/*Property = Svg*/
const Value V18245_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_327 = {S44, 0, &V18245_327_57316, &P18245_324};	/*Property = TouchEvents*/
const Value V18245_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_328 = {S45, 0, &V18245_328_57319, &P18245_327};	/*Property = Track*/
const Value V18245_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_329 = {S46, 0, &V18245_329_57325, &P18245_328};	/*Property = Video*/
const Value V18245_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_330 = {S47, 0, &V18245_330_57328, &P18245_329};	/*Property = Viewport*/
const Value V18245_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_332 = {S48, 0, &V18245_332_57340, &P18245_330};	/*Property = WebWorkers*/
const Value V18245_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18245_333 = {S49, 0, &V18245_333_57343, &P18245_332};	/*Property = Xhr2*/
const Profile P18245 = {"18245", &P18245_333};	/*Profile ID - 18245*/
const Value V18254_1_2 = {S0, NULL};	/*Value = False*/
const Property P18254_1 = {S50, 0, &V18254_1_2, NULL};	/*Property = IsMobile*/
const Value V18254_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P18254_5 = {S52, 0, &V18254_5_37, &P18254_1};	/*Property = ScreenPixelsHeight*/
const Value V18254_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P18254_4 = {S54, 0, &V18254_4_36, &P18254_5};	/*Property = ScreenPixelsWidth*/
const Profile P18254 = {"18254", &P18254_4};	/*Profile ID - 18254*/
const Profile P22924 = {"22924", NULL};	/*Profile ID - 22924*/
const Profile P17294 = {"17294", NULL};	/*Profile ID - 17294*/
const Value V17213_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_279 = {S1, 0, &V17213_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17213_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_282 = {S3, 0, &V17213_282_57155, &P17213_279};	/*Property = BlobBuilder*/
const Value V17213_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_350 = {S4, 0, &V17213_350_57446, &P17213_282};	/*Property = Canvas*/
const Value V17213_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_283 = {S5, 0, &V17213_283_57161, &P17213_350};	/*Property = CssBackground*/
const Value V17213_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_284 = {S6, 0, &V17213_284_57164, &P17213_283};	/*Property = CssBorderImage*/
const Value V17213_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_285 = {S7, 0, &V17213_285_57167, &P17213_284};	/*Property = CssCanvas*/
const Value V17213_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_286 = {S8, 0, &V17213_286_57170, &P17213_285};	/*Property = CssColor*/
const Value V17213_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_237 = {S9, 0, &V17213_237_55786, &P17213_286};	/*Property = CssColumn*/
const Value V17213_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_287 = {S10, 0, &V17213_287_57173, &P17213_237};	/*Property = CssFlexbox*/
const Value V17213_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_288 = {S11, 0, &V17213_288_57176, &P17213_287};	/*Property = CssFont*/
const Value V17213_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_289 = {S12, 0, &V17213_289_86899, &P17213_288};	/*Property = CssImages*/
const Value V17213_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_291 = {S13, 0, &V17213_291_57182, &P17213_289};	/*Property = CssMediaQueries*/
const Value V17213_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_292 = {S14, 0, &V17213_292_57185, &P17213_291};	/*Property = CssMinMax*/
const Value V17213_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_293 = {S15, 0, &V17213_293_57188, &P17213_292};	/*Property = CssOverflow*/
const Value V17213_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_294 = {S16, 0, &V17213_294_57191, &P17213_293};	/*Property = CssPosition*/
const Value V17213_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_295 = {S17, 0, &V17213_295_57194, &P17213_294};	/*Property = CssText*/
const Value V17213_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_297 = {S18, 0, &V17213_297_57197, &P17213_295};	/*Property = CssTransforms*/
const Value V17213_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_241 = {S19, 0, &V17213_241_55801, &P17213_297};	/*Property = CssTransitions*/
const Value V17213_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_298 = {S20, 0, &V17213_298_57208, &P17213_241};	/*Property = CssUI*/
const Value V17213_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_300 = {S21, 0, &V17213_300_57217, &P17213_298};	/*Property = DataSet*/
const Value V17213_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_301 = {S22, 0, &V17213_301_57220, &P17213_300};	/*Property = DataUrl*/
const Value V17213_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_302 = {S23, 0, &V17213_302_57223, &P17213_301};	/*Property = DeviceOrientation*/
const Value V17213_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_304 = {S24, 0, &V17213_304_57229, &P17213_302};	/*Property = FileReader*/
const Value V17213_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_305 = {S25, 0, &V17213_305_57232, &P17213_304};	/*Property = FileSaver*/
const Value V17213_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_306 = {S26, 0, &V17213_306_57238, &P17213_305};	/*Property = FileWriter*/
const Value V17213_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_307 = {S27, 0, &V17213_307_57241, &P17213_306};	/*Property = FormData*/
const Value V17213_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_309 = {S28, 0, &V17213_309_57247, &P17213_307};	/*Property = Fullscreen*/
const Value V17213_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_310 = {S29, 0, &V17213_310_57250, &P17213_309};	/*Property = GeoLocation*/
const Value V17213_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_312 = {S30, 0, &V17213_312_57256, &P17213_310};	/*Property = History*/
const Value V17213_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_314 = {S31, 0, &V17213_314_57262, &P17213_312};	/*Property = Html5*/
const Value V17213_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_313 = {S32, 0, &V17213_313_57259, &P17213_314};	/*Property = Html-Media-Capture*/
const Value V17213_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_315 = {S33, 0, &V17213_315_57265, &P17213_313};	/*Property = Iframe*/
const Value V17213_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_316 = {S34, 0, &V17213_316_57268, &P17213_315};	/*Property = IndexedDB*/
const Value V17213_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_317 = {S35, 0, &V17213_317_57271, &P17213_316};	/*Property = Json*/
const Value V17213_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P17213_15 = {S37, 0, &V17213_15_31, &P17213_317};	/*Property = LayoutEngine*/
const Value V17213_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_344 = {S38, 0, &V17213_344_57274, &P17213_15};	/*Property = Masking*/
const Value V17213_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_320 = {S39, 0, &V17213_320_57292, &P17213_344};	/*Property = PostMessage*/
const Value V17213_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_321 = {S40, 0, &V17213_321_57295, &P17213_320};	/*Property = Progress*/
const Value V17213_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_322 = {S41, 0, &V17213_322_57298, &P17213_321};	/*Property = Prompts*/
const Value V17213_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_323 = {S42, 0, &V17213_323_57301, &P17213_322};	/*Property = Selector*/
const Value V17213_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_324 = {S43, 0, &V17213_324_57307, &P17213_323};	/*Property = Svg*/
const Value V17213_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_327 = {S44, 0, &V17213_327_57316, &P17213_324};	/*Property = TouchEvents*/
const Value V17213_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_328 = {S45, 0, &V17213_328_57319, &P17213_327};	/*Property = Track*/
const Value V17213_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_329 = {S46, 0, &V17213_329_57325, &P17213_328};	/*Property = Video*/
const Value V17213_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_330 = {S47, 0, &V17213_330_57328, &P17213_329};	/*Property = Viewport*/
const Value V17213_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_332 = {S48, 0, &V17213_332_57340, &P17213_330};	/*Property = WebWorkers*/
const Value V17213_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17213_333 = {S49, 0, &V17213_333_57343, &P17213_332};	/*Property = Xhr2*/
const Profile P17213 = {"17213", &P17213_333};	/*Profile ID - 17213*/
const Value V16460_1_1 = {S2, NULL};	/*Value = True*/
const Property P16460_1 = {S50, 0, &V16460_1_1, NULL};	/*Property = IsMobile*/
const Value V16460_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16460_5 = {S52, 0, &V16460_5_1233, &P16460_1};	/*Property = ScreenPixelsHeight*/
const Value V16460_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16460_4 = {S54, 0, &V16460_4_1235, &P16460_5};	/*Property = ScreenPixelsWidth*/
const Profile P16460 = {"16460", &P16460_4};	/*Profile ID - 16460*/
const Value V12334_1_1 = {S2, NULL};	/*Value = True*/
const Property P12334_1 = {S50, 0, &V12334_1_1, NULL};	/*Property = IsMobile*/
const Value V12334_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12334_5 = {S52, 0, &V12334_5_1233, &P12334_1};	/*Property = ScreenPixelsHeight*/
const Value V12334_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12334_4 = {S54, 0, &V12334_4_1235, &P12334_5};	/*Property = ScreenPixelsWidth*/
const Profile P12334 = {"12334", &P12334_4};	/*Profile ID - 12334*/
const Value V12335_1_1 = {S2, NULL};	/*Value = True*/
const Property P12335_1 = {S50, 0, &V12335_1_1, NULL};	/*Property = IsMobile*/
const Value V12335_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12335_5 = {S52, 0, &V12335_5_1293, &P12335_1};	/*Property = ScreenPixelsHeight*/
const Value V12335_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12335_4 = {S54, 0, &V12335_4_1296, &P12335_5};	/*Property = ScreenPixelsWidth*/
const Profile P12335 = {"12335", &P12335_4};	/*Profile ID - 12335*/
const Value V12336_1_1 = {S2, NULL};	/*Value = True*/
const Property P12336_1 = {S50, 0, &V12336_1_1, NULL};	/*Property = IsMobile*/
const Value V12336_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12336_5 = {S52, 0, &V12336_5_1222, &P12336_1};	/*Property = ScreenPixelsHeight*/
const Value V12336_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12336_4 = {S54, 0, &V12336_4_1224, &P12336_5};	/*Property = ScreenPixelsWidth*/
const Profile P12336 = {"12336", &P12336_4};	/*Profile ID - 12336*/
const Value V20116_1_2 = {S0, NULL};	/*Value = False*/
const Property P20116_1 = {S50, 0, &V20116_1_2, NULL};	/*Property = IsMobile*/
const Value V20116_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20116_5 = {S52, 0, &V20116_5_37, &P20116_1};	/*Property = ScreenPixelsHeight*/
const Value V20116_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20116_4 = {S54, 0, &V20116_4_36, &P20116_5};	/*Property = ScreenPixelsWidth*/
const Profile P20116 = {"20116", &P20116_4};	/*Profile ID - 20116*/
const Value V12343_1_1 = {S2, NULL};	/*Value = True*/
const Property P12343_1 = {S50, 0, &V12343_1_1, NULL};	/*Property = IsMobile*/
const Value V12343_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12343_5 = {S52, 0, &V12343_5_1222, &P12343_1};	/*Property = ScreenPixelsHeight*/
const Value V12343_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12343_4 = {S54, 0, &V12343_4_1224, &P12343_5};	/*Property = ScreenPixelsWidth*/
const Profile P12343 = {"12343", &P12343_4};	/*Profile ID - 12343*/
const Value V14497_1_1 = {S2, NULL};	/*Value = True*/
const Property P14497_1 = {S50, 0, &V14497_1_1, NULL};	/*Property = IsMobile*/
const Value V14497_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14497_5 = {S52, 0, &V14497_5_1222, &P14497_1};	/*Property = ScreenPixelsHeight*/
const Value V14497_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14497_4 = {S54, 0, &V14497_4_1224, &P14497_5};	/*Property = ScreenPixelsWidth*/
const Profile P14497 = {"14497", &P14497_4};	/*Profile ID - 14497*/
const Value V12344_1_1 = {S2, NULL};	/*Value = True*/
const Property P12344_1 = {S50, 0, &V12344_1_1, NULL};	/*Property = IsMobile*/
const Value V12344_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12344_5 = {S52, 0, &V12344_5_1222, &P12344_1};	/*Property = ScreenPixelsHeight*/
const Value V12344_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12344_4 = {S54, 0, &V12344_4_1224, &P12344_5};	/*Property = ScreenPixelsWidth*/
const Profile P12344 = {"12344", &P12344_4};	/*Profile ID - 12344*/
const Value V12355_1_1 = {S2, NULL};	/*Value = True*/
const Property P12355_1 = {S50, 0, &V12355_1_1, NULL};	/*Property = IsMobile*/
const Value V12355_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12355_5 = {S52, 0, &V12355_5_1222, &P12355_1};	/*Property = ScreenPixelsHeight*/
const Value V12355_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12355_4 = {S54, 0, &V12355_4_1224, &P12355_5};	/*Property = ScreenPixelsWidth*/
const Profile P12355 = {"12355", &P12355_4};	/*Profile ID - 12355*/
const Value V12356_1_1 = {S2, NULL};	/*Value = True*/
const Property P12356_1 = {S50, 0, &V12356_1_1, NULL};	/*Property = IsMobile*/
const Value V12356_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12356_5 = {S52, 0, &V12356_5_1294, &P12356_1};	/*Property = ScreenPixelsHeight*/
const Value V12356_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12356_4 = {S54, 0, &V12356_4_1296, &P12356_5};	/*Property = ScreenPixelsWidth*/
const Profile P12356 = {"12356", &P12356_4};	/*Profile ID - 12356*/
const Value V17802_1_1 = {S2, NULL};	/*Value = True*/
const Property P17802_1 = {S50, 0, &V17802_1_1, NULL};	/*Property = IsMobile*/
const Value V17802_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17802_5 = {S52, 0, &V17802_5_1222, &P17802_1};	/*Property = ScreenPixelsHeight*/
const Value V17802_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17802_4 = {S54, 0, &V17802_4_1224, &P17802_5};	/*Property = ScreenPixelsWidth*/
const Profile P17802 = {"17802", &P17802_4};	/*Profile ID - 17802*/
const Value V12358_1_1 = {S2, NULL};	/*Value = True*/
const Property P12358_1 = {S50, 0, &V12358_1_1, NULL};	/*Property = IsMobile*/
const Value V12358_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12358_5 = {S52, 0, &V12358_5_1294, &P12358_1};	/*Property = ScreenPixelsHeight*/
const Value V12358_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12358_4 = {S54, 0, &V12358_4_1296, &P12358_5};	/*Property = ScreenPixelsWidth*/
const Profile P12358 = {"12358", &P12358_4};	/*Profile ID - 12358*/
const Value V12363_1_1 = {S2, NULL};	/*Value = True*/
const Property P12363_1 = {S50, 0, &V12363_1_1, NULL};	/*Property = IsMobile*/
const Value V12363_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12363_5 = {S52, 0, &V12363_5_1233, &P12363_1};	/*Property = ScreenPixelsHeight*/
const Value V12363_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12363_4 = {S54, 0, &V12363_4_1235, &P12363_5};	/*Property = ScreenPixelsWidth*/
const Profile P12363 = {"12363", &P12363_4};	/*Profile ID - 12363*/
const Value V12364_1_1 = {S2, NULL};	/*Value = True*/
const Property P12364_1 = {S50, 0, &V12364_1_1, NULL};	/*Property = IsMobile*/
const Value V12364_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12364_5 = {S52, 0, &V12364_5_1233, &P12364_1};	/*Property = ScreenPixelsHeight*/
const Value V12364_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12364_4 = {S54, 0, &V12364_4_1235, &P12364_5};	/*Property = ScreenPixelsWidth*/
const Profile P12364 = {"12364", &P12364_4};	/*Profile ID - 12364*/
const Value V12367_1_1 = {S2, NULL};	/*Value = True*/
const Property P12367_1 = {S50, 0, &V12367_1_1, NULL};	/*Property = IsMobile*/
const Value V12367_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12367_5 = {S52, 0, &V12367_5_1222, &P12367_1};	/*Property = ScreenPixelsHeight*/
const Value V12367_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12367_4 = {S54, 0, &V12367_4_1224, &P12367_5};	/*Property = ScreenPixelsWidth*/
const Profile P12367 = {"12367", &P12367_4};	/*Profile ID - 12367*/
const Value V12366_1_1 = {S2, NULL};	/*Value = True*/
const Property P12366_1 = {S50, 0, &V12366_1_1, NULL};	/*Property = IsMobile*/
const Value V12366_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12366_5 = {S52, 0, &V12366_5_1380, &P12366_1};	/*Property = ScreenPixelsHeight*/
const Value V12366_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12366_4 = {S54, 0, &V12366_4_1279, &P12366_5};	/*Property = ScreenPixelsWidth*/
const Profile P12366 = {"12366", &P12366_4};	/*Profile ID - 12366*/
const Value V12370_1_1 = {S2, NULL};	/*Value = True*/
const Property P12370_1 = {S50, 0, &V12370_1_1, NULL};	/*Property = IsMobile*/
const Value V12370_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12370_5 = {S52, 0, &V12370_5_1222, &P12370_1};	/*Property = ScreenPixelsHeight*/
const Value V12370_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12370_4 = {S54, 0, &V12370_4_1224, &P12370_5};	/*Property = ScreenPixelsWidth*/
const Profile P12370 = {"12370", &P12370_4};	/*Profile ID - 12370*/
const Value V12373_1_1 = {S2, NULL};	/*Value = True*/
const Property P12373_1 = {S50, 0, &V12373_1_1, NULL};	/*Property = IsMobile*/
const Value V12373_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12373_5 = {S52, 0, &V12373_5_1293, &P12373_1};	/*Property = ScreenPixelsHeight*/
const Value V12373_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12373_4 = {S54, 0, &V12373_4_1296, &P12373_5};	/*Property = ScreenPixelsWidth*/
const Profile P12373 = {"12373", &P12373_4};	/*Profile ID - 12373*/
const Value V12374_1_1 = {S2, NULL};	/*Value = True*/
const Property P12374_1 = {S50, 0, &V12374_1_1, NULL};	/*Property = IsMobile*/
const Value V12374_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12374_5 = {S52, 0, &V12374_5_1222, &P12374_1};	/*Property = ScreenPixelsHeight*/
const Value V12374_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12374_4 = {S54, 0, &V12374_4_1224, &P12374_5};	/*Property = ScreenPixelsWidth*/
const Profile P12374 = {"12374", &P12374_4};	/*Profile ID - 12374*/
const Value V12378_1_1 = {S2, NULL};	/*Value = True*/
const Property P12378_1 = {S50, 0, &V12378_1_1, NULL};	/*Property = IsMobile*/
const Value V12378_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12378_5 = {S52, 0, &V12378_5_1293, &P12378_1};	/*Property = ScreenPixelsHeight*/
const Value V12378_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12378_4 = {S54, 0, &V12378_4_1296, &P12378_5};	/*Property = ScreenPixelsWidth*/
const Profile P12378 = {"12378", &P12378_4};	/*Profile ID - 12378*/
const Value V12381_1_1 = {S2, NULL};	/*Value = True*/
const Property P12381_1 = {S50, 0, &V12381_1_1, NULL};	/*Property = IsMobile*/
const Value V12381_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12381_5 = {S52, 0, &V12381_5_1222, &P12381_1};	/*Property = ScreenPixelsHeight*/
const Value V12381_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12381_4 = {S54, 0, &V12381_4_1224, &P12381_5};	/*Property = ScreenPixelsWidth*/
const Profile P12381 = {"12381", &P12381_4};	/*Profile ID - 12381*/
const Value V12382_1_1 = {S2, NULL};	/*Value = True*/
const Property P12382_1 = {S50, 0, &V12382_1_1, NULL};	/*Property = IsMobile*/
const Value V12382_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12382_5 = {S52, 0, &V12382_5_1222, &P12382_1};	/*Property = ScreenPixelsHeight*/
const Value V12382_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12382_4 = {S54, 0, &V12382_4_1224, &P12382_5};	/*Property = ScreenPixelsWidth*/
const Profile P12382 = {"12382", &P12382_4};	/*Profile ID - 12382*/
const Value V12385_1_1 = {S2, NULL};	/*Value = True*/
const Property P12385_1 = {S50, 0, &V12385_1_1, NULL};	/*Property = IsMobile*/
const Value V12385_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12385_5 = {S52, 0, &V12385_5_1233, &P12385_1};	/*Property = ScreenPixelsHeight*/
const Value V12385_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12385_4 = {S54, 0, &V12385_4_1235, &P12385_5};	/*Property = ScreenPixelsWidth*/
const Profile P12385 = {"12385", &P12385_4};	/*Profile ID - 12385*/
const Value V12390_1_1 = {S2, NULL};	/*Value = True*/
const Property P12390_1 = {S50, 0, &V12390_1_1, NULL};	/*Property = IsMobile*/
const Value V12390_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12390_5 = {S52, 0, &V12390_5_1304, &P12390_1};	/*Property = ScreenPixelsHeight*/
const Value V12390_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12390_4 = {S54, 0, &V12390_4_1224, &P12390_5};	/*Property = ScreenPixelsWidth*/
const Profile P12390 = {"12390", &P12390_4};	/*Profile ID - 12390*/
const Value V12393_1_1 = {S2, NULL};	/*Value = True*/
const Property P12393_1 = {S50, 0, &V12393_1_1, NULL};	/*Property = IsMobile*/
const Value V12393_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12393_5 = {S52, 0, &V12393_5_1222, &P12393_1};	/*Property = ScreenPixelsHeight*/
const Value V12393_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12393_4 = {S54, 0, &V12393_4_1224, &P12393_5};	/*Property = ScreenPixelsWidth*/
const Profile P12393 = {"12393", &P12393_4};	/*Profile ID - 12393*/
const Value V12394_1_1 = {S2, NULL};	/*Value = True*/
const Property P12394_1 = {S50, 0, &V12394_1_1, NULL};	/*Property = IsMobile*/
const Value V12394_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12394_5 = {S52, 0, &V12394_5_1222, &P12394_1};	/*Property = ScreenPixelsHeight*/
const Value V12394_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12394_4 = {S54, 0, &V12394_4_1224, &P12394_5};	/*Property = ScreenPixelsWidth*/
const Profile P12394 = {"12394", &P12394_4};	/*Profile ID - 12394*/
const Value V12395_1_1 = {S2, NULL};	/*Value = True*/
const Property P12395_1 = {S50, 0, &V12395_1_1, NULL};	/*Property = IsMobile*/
const Value V12395_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P12395_5 = {S52, 0, &V12395_5_1333, &P12395_1};	/*Property = ScreenPixelsHeight*/
const Value V12395_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P12395_4 = {S54, 0, &V12395_4_1335, &P12395_5};	/*Property = ScreenPixelsWidth*/
const Profile P12395 = {"12395", &P12395_4};	/*Profile ID - 12395*/
const Value V12396_1_1 = {S2, NULL};	/*Value = True*/
const Property P12396_1 = {S50, 0, &V12396_1_1, NULL};	/*Property = IsMobile*/
const Value V12396_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P12396_5 = {S52, 0, &V12396_5_1333, &P12396_1};	/*Property = ScreenPixelsHeight*/
const Value V12396_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P12396_4 = {S54, 0, &V12396_4_1335, &P12396_5};	/*Property = ScreenPixelsWidth*/
const Profile P12396 = {"12396", &P12396_4};	/*Profile ID - 12396*/
const Value V12397_1_1 = {S2, NULL};	/*Value = True*/
const Property P12397_1 = {S50, 0, &V12397_1_1, NULL};	/*Property = IsMobile*/
const Value V12397_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P12397_5 = {S52, 0, &V12397_5_1269, &P12397_1};	/*Property = ScreenPixelsHeight*/
const Value V12397_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12397_4 = {S54, 0, &V12397_4_1229, &P12397_5};	/*Property = ScreenPixelsWidth*/
const Profile P12397 = {"12397", &P12397_4};	/*Profile ID - 12397*/
const Value V12398_1_1 = {S2, NULL};	/*Value = True*/
const Property P12398_1 = {S50, 0, &V12398_1_1, NULL};	/*Property = IsMobile*/
const Value V12398_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12398_5 = {S52, 0, &V12398_5_1233, &P12398_1};	/*Property = ScreenPixelsHeight*/
const Value V12398_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12398_4 = {S54, 0, &V12398_4_1235, &P12398_5};	/*Property = ScreenPixelsWidth*/
const Profile P12398 = {"12398", &P12398_4};	/*Profile ID - 12398*/
const Value V17609_1_1 = {S2, NULL};	/*Value = True*/
const Property P17609_1 = {S50, 0, &V17609_1_1, NULL};	/*Property = IsMobile*/
const Value V17609_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17609_5 = {S52, 0, &V17609_5_1304, &P17609_1};	/*Property = ScreenPixelsHeight*/
const Value V17609_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17609_4 = {S54, 0, &V17609_4_1224, &P17609_5};	/*Property = ScreenPixelsWidth*/
const Profile P17609 = {"17609", &P17609_4};	/*Profile ID - 17609*/
const Value V12404_1_1 = {S2, NULL};	/*Value = True*/
const Property P12404_1 = {S50, 0, &V12404_1_1, NULL};	/*Property = IsMobile*/
const Value V12404_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12404_5 = {S52, 0, &V12404_5_1304, &P12404_1};	/*Property = ScreenPixelsHeight*/
const Value V12404_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12404_4 = {S54, 0, &V12404_4_1224, &P12404_5};	/*Property = ScreenPixelsWidth*/
const Profile P12404 = {"12404", &P12404_4};	/*Profile ID - 12404*/
const Value V12405_1_1 = {S2, NULL};	/*Value = True*/
const Property P12405_1 = {S50, 0, &V12405_1_1, NULL};	/*Property = IsMobile*/
const Value V12405_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12405_5 = {S52, 0, &V12405_5_1277, &P12405_1};	/*Property = ScreenPixelsHeight*/
const Value V12405_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12405_4 = {S54, 0, &V12405_4_1279, &P12405_5};	/*Property = ScreenPixelsWidth*/
const Profile P12405 = {"12405", &P12405_4};	/*Profile ID - 12405*/
const Value V12407_1_1 = {S2, NULL};	/*Value = True*/
const Property P12407_1 = {S50, 0, &V12407_1_1, NULL};	/*Property = IsMobile*/
const Value V12407_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12407_5 = {S52, 0, &V12407_5_1233, &P12407_1};	/*Property = ScreenPixelsHeight*/
const Value V12407_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12407_4 = {S54, 0, &V12407_4_1235, &P12407_5};	/*Property = ScreenPixelsWidth*/
const Profile P12407 = {"12407", &P12407_4};	/*Profile ID - 12407*/
const Value V12409_1_1 = {S2, NULL};	/*Value = True*/
const Property P12409_1 = {S50, 0, &V12409_1_1, NULL};	/*Property = IsMobile*/
const Value V12409_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12409_5 = {S52, 0, &V12409_5_1222, &P12409_1};	/*Property = ScreenPixelsHeight*/
const Value V12409_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12409_4 = {S54, 0, &V12409_4_1224, &P12409_5};	/*Property = ScreenPixelsWidth*/
const Profile P12409 = {"12409", &P12409_4};	/*Profile ID - 12409*/
const Value V12410_1_1 = {S2, NULL};	/*Value = True*/
const Property P12410_1 = {S50, 0, &V12410_1_1, NULL};	/*Property = IsMobile*/
const Value V12410_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12410_5 = {S52, 0, &V12410_5_1293, &P12410_1};	/*Property = ScreenPixelsHeight*/
const Value V12410_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12410_4 = {S54, 0, &V12410_4_1296, &P12410_5};	/*Property = ScreenPixelsWidth*/
const Profile P12410 = {"12410", &P12410_4};	/*Profile ID - 12410*/
const Value V12413_1_1 = {S2, NULL};	/*Value = True*/
const Property P12413_1 = {S50, 0, &V12413_1_1, NULL};	/*Property = IsMobile*/
const Value V12413_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12413_5 = {S52, 0, &V12413_5_1304, &P12413_1};	/*Property = ScreenPixelsHeight*/
const Value V12413_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12413_4 = {S54, 0, &V12413_4_1224, &P12413_5};	/*Property = ScreenPixelsWidth*/
const Profile P12413 = {"12413", &P12413_4};	/*Profile ID - 12413*/
const Value V12417_1_1 = {S2, NULL};	/*Value = True*/
const Property P12417_1 = {S50, 0, &V12417_1_1, NULL};	/*Property = IsMobile*/
const Value V12417_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12417_5 = {S52, 0, &V12417_5_1380, &P12417_1};	/*Property = ScreenPixelsHeight*/
const Value V12417_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12417_4 = {S54, 0, &V12417_4_1279, &P12417_5};	/*Property = ScreenPixelsWidth*/
const Profile P12417 = {"12417", &P12417_4};	/*Profile ID - 12417*/
const Value V12418_1_1 = {S2, NULL};	/*Value = True*/
const Property P12418_1 = {S50, 0, &V12418_1_1, NULL};	/*Property = IsMobile*/
const Value V12418_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12418_5 = {S52, 0, &V12418_5_1233, &P12418_1};	/*Property = ScreenPixelsHeight*/
const Value V12418_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12418_4 = {S54, 0, &V12418_4_1235, &P12418_5};	/*Property = ScreenPixelsWidth*/
const Profile P12418 = {"12418", &P12418_4};	/*Profile ID - 12418*/
const Value V12423_1_1 = {S2, NULL};	/*Value = True*/
const Property P12423_1 = {S50, 0, &V12423_1_1, NULL};	/*Property = IsMobile*/
const Value V12423_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12423_5 = {S52, 0, &V12423_5_1222, &P12423_1};	/*Property = ScreenPixelsHeight*/
const Value V12423_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12423_4 = {S54, 0, &V12423_4_1224, &P12423_5};	/*Property = ScreenPixelsWidth*/
const Profile P12423 = {"12423", &P12423_4};	/*Profile ID - 12423*/
const Value V12427_1_1 = {S2, NULL};	/*Value = True*/
const Property P12427_1 = {S50, 0, &V12427_1_1, NULL};	/*Property = IsMobile*/
const Value V12427_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12427_5 = {S52, 0, &V12427_5_1293, &P12427_1};	/*Property = ScreenPixelsHeight*/
const Value V12427_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12427_4 = {S54, 0, &V12427_4_1296, &P12427_5};	/*Property = ScreenPixelsWidth*/
const Profile P12427 = {"12427", &P12427_4};	/*Profile ID - 12427*/
const Value V12429_1_1 = {S2, NULL};	/*Value = True*/
const Property P12429_1 = {S50, 0, &V12429_1_1, NULL};	/*Property = IsMobile*/
const Value V12429_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12429_5 = {S52, 0, &V12429_5_1222, &P12429_1};	/*Property = ScreenPixelsHeight*/
const Value V12429_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12429_4 = {S54, 0, &V12429_4_1224, &P12429_5};	/*Property = ScreenPixelsWidth*/
const Profile P12429 = {"12429", &P12429_4};	/*Profile ID - 12429*/
const Value V12437_1_1 = {S2, NULL};	/*Value = True*/
const Property P12437_1 = {S50, 0, &V12437_1_1, NULL};	/*Property = IsMobile*/
const Value V12437_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12437_5 = {S52, 0, &V12437_5_1380, &P12437_1};	/*Property = ScreenPixelsHeight*/
const Value V12437_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P12437_4 = {S54, 0, &V12437_4_1943, &P12437_5};	/*Property = ScreenPixelsWidth*/
const Profile P12437 = {"12437", &P12437_4};	/*Profile ID - 12437*/
const Value V12440_1_1 = {S2, NULL};	/*Value = True*/
const Property P12440_1 = {S50, 0, &V12440_1_1, NULL};	/*Property = IsMobile*/
const Value V12440_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12440_5 = {S52, 0, &V12440_5_1222, &P12440_1};	/*Property = ScreenPixelsHeight*/
const Value V12440_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12440_4 = {S54, 0, &V12440_4_1224, &P12440_5};	/*Property = ScreenPixelsWidth*/
const Profile P12440 = {"12440", &P12440_4};	/*Profile ID - 12440*/
const Value V12442_1_1 = {S2, NULL};	/*Value = True*/
const Property P12442_1 = {S50, 0, &V12442_1_1, NULL};	/*Property = IsMobile*/
const Value V12442_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12442_5 = {S52, 0, &V12442_5_1293, &P12442_1};	/*Property = ScreenPixelsHeight*/
const Value V12442_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12442_4 = {S54, 0, &V12442_4_1296, &P12442_5};	/*Property = ScreenPixelsWidth*/
const Profile P12442 = {"12442", &P12442_4};	/*Profile ID - 12442*/
const Value V12443_1_1 = {S2, NULL};	/*Value = True*/
const Property P12443_1 = {S50, 0, &V12443_1_1, NULL};	/*Property = IsMobile*/
const Value V12443_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12443_5 = {S52, 0, &V12443_5_1222, &P12443_1};	/*Property = ScreenPixelsHeight*/
const Value V12443_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12443_4 = {S54, 0, &V12443_4_1224, &P12443_5};	/*Property = ScreenPixelsWidth*/
const Profile P12443 = {"12443", &P12443_4};	/*Profile ID - 12443*/
const Value V4779_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_279 = {S1, 0, &V4779_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4779_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_282 = {S3, 0, &V4779_282_57155, &P4779_279};	/*Property = BlobBuilder*/
const Value V4779_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_350 = {S4, 0, &V4779_350_57446, &P4779_282};	/*Property = Canvas*/
const Value V4779_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_283 = {S5, 0, &V4779_283_57161, &P4779_350};	/*Property = CssBackground*/
const Value V4779_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_284 = {S6, 0, &V4779_284_57164, &P4779_283};	/*Property = CssBorderImage*/
const Value V4779_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_285 = {S7, 0, &V4779_285_57167, &P4779_284};	/*Property = CssCanvas*/
const Value V4779_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_286 = {S8, 0, &V4779_286_57170, &P4779_285};	/*Property = CssColor*/
const Value V4779_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_237 = {S9, 0, &V4779_237_55786, &P4779_286};	/*Property = CssColumn*/
const Value V4779_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_287 = {S10, 0, &V4779_287_57173, &P4779_237};	/*Property = CssFlexbox*/
const Value V4779_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_288 = {S11, 0, &V4779_288_57176, &P4779_287};	/*Property = CssFont*/
const Value V4779_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_289 = {S12, 0, &V4779_289_86899, &P4779_288};	/*Property = CssImages*/
const Value V4779_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_291 = {S13, 0, &V4779_291_57182, &P4779_289};	/*Property = CssMediaQueries*/
const Value V4779_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_292 = {S14, 0, &V4779_292_57185, &P4779_291};	/*Property = CssMinMax*/
const Value V4779_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_293 = {S15, 0, &V4779_293_57188, &P4779_292};	/*Property = CssOverflow*/
const Value V4779_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_294 = {S16, 0, &V4779_294_57191, &P4779_293};	/*Property = CssPosition*/
const Value V4779_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_295 = {S17, 0, &V4779_295_57194, &P4779_294};	/*Property = CssText*/
const Value V4779_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_297 = {S18, 0, &V4779_297_57197, &P4779_295};	/*Property = CssTransforms*/
const Value V4779_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_241 = {S19, 0, &V4779_241_55801, &P4779_297};	/*Property = CssTransitions*/
const Value V4779_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_298 = {S20, 0, &V4779_298_57208, &P4779_241};	/*Property = CssUI*/
const Value V4779_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_300 = {S21, 0, &V4779_300_57217, &P4779_298};	/*Property = DataSet*/
const Value V4779_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_301 = {S22, 0, &V4779_301_57220, &P4779_300};	/*Property = DataUrl*/
const Value V4779_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_302 = {S23, 0, &V4779_302_57223, &P4779_301};	/*Property = DeviceOrientation*/
const Value V4779_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_304 = {S24, 0, &V4779_304_57229, &P4779_302};	/*Property = FileReader*/
const Value V4779_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_305 = {S25, 0, &V4779_305_57232, &P4779_304};	/*Property = FileSaver*/
const Value V4779_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_306 = {S26, 0, &V4779_306_57238, &P4779_305};	/*Property = FileWriter*/
const Value V4779_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_307 = {S27, 0, &V4779_307_57241, &P4779_306};	/*Property = FormData*/
const Value V4779_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_309 = {S28, 0, &V4779_309_57247, &P4779_307};	/*Property = Fullscreen*/
const Value V4779_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_310 = {S29, 0, &V4779_310_57250, &P4779_309};	/*Property = GeoLocation*/
const Value V4779_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_312 = {S30, 0, &V4779_312_57256, &P4779_310};	/*Property = History*/
const Value V4779_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_314 = {S31, 0, &V4779_314_57262, &P4779_312};	/*Property = Html5*/
const Value V4779_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_313 = {S32, 0, &V4779_313_57259, &P4779_314};	/*Property = Html-Media-Capture*/
const Value V4779_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_315 = {S33, 0, &V4779_315_57265, &P4779_313};	/*Property = Iframe*/
const Value V4779_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_316 = {S34, 0, &V4779_316_57268, &P4779_315};	/*Property = IndexedDB*/
const Value V4779_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_317 = {S35, 0, &V4779_317_57271, &P4779_316};	/*Property = Json*/
const Value V4779_15_90838 = {S70, NULL};	/*Value = U3*/
const Property P4779_15 = {S37, 0, &V4779_15_90838, &P4779_317};	/*Property = LayoutEngine*/
const Value V4779_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_344 = {S38, 0, &V4779_344_57274, &P4779_15};	/*Property = Masking*/
const Value V4779_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_320 = {S39, 0, &V4779_320_57292, &P4779_344};	/*Property = PostMessage*/
const Value V4779_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_321 = {S40, 0, &V4779_321_57295, &P4779_320};	/*Property = Progress*/
const Value V4779_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_322 = {S41, 0, &V4779_322_57298, &P4779_321};	/*Property = Prompts*/
const Value V4779_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_323 = {S42, 0, &V4779_323_57301, &P4779_322};	/*Property = Selector*/
const Value V4779_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_324 = {S43, 0, &V4779_324_57307, &P4779_323};	/*Property = Svg*/
const Value V4779_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_327 = {S44, 0, &V4779_327_57316, &P4779_324};	/*Property = TouchEvents*/
const Value V4779_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_328 = {S45, 0, &V4779_328_57319, &P4779_327};	/*Property = Track*/
const Value V4779_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_329 = {S46, 0, &V4779_329_57325, &P4779_328};	/*Property = Video*/
const Value V4779_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_330 = {S47, 0, &V4779_330_57328, &P4779_329};	/*Property = Viewport*/
const Value V4779_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_332 = {S48, 0, &V4779_332_57340, &P4779_330};	/*Property = WebWorkers*/
const Value V4779_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4779_333 = {S49, 0, &V4779_333_57343, &P4779_332};	/*Property = Xhr2*/
const Profile P4779 = {"4779", &P4779_333};	/*Profile ID - 4779*/
const Value V12448_1_1 = {S2, NULL};	/*Value = True*/
const Property P12448_1 = {S50, 0, &V12448_1_1, NULL};	/*Property = IsMobile*/
const Value V12448_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12448_5 = {S52, 0, &V12448_5_1380, &P12448_1};	/*Property = ScreenPixelsHeight*/
const Value V12448_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12448_4 = {S54, 0, &V12448_4_1279, &P12448_5};	/*Property = ScreenPixelsWidth*/
const Profile P12448 = {"12448", &P12448_4};	/*Profile ID - 12448*/
const Value V12454_1_1 = {S2, NULL};	/*Value = True*/
const Property P12454_1 = {S50, 0, &V12454_1_1, NULL};	/*Property = IsMobile*/
const Value V12454_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12454_5 = {S52, 0, &V12454_5_1222, &P12454_1};	/*Property = ScreenPixelsHeight*/
const Value V12454_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12454_4 = {S54, 0, &V12454_4_1224, &P12454_5};	/*Property = ScreenPixelsWidth*/
const Profile P12454 = {"12454", &P12454_4};	/*Profile ID - 12454*/
const Value V12476_1_1 = {S2, NULL};	/*Value = True*/
const Property P12476_1 = {S50, 0, &V12476_1_1, NULL};	/*Property = IsMobile*/
const Value V12476_5_1796 = {S64, NULL};	/*Value = 360*/
const Property P12476_5 = {S52, 0, &V12476_5_1796, &P12476_1};	/*Property = ScreenPixelsHeight*/
const Value V12476_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12476_4 = {S54, 0, &V12476_4_1229, &P12476_5};	/*Property = ScreenPixelsWidth*/
const Profile P12476 = {"12476", &P12476_4};	/*Profile ID - 12476*/
const Value V12470_1_1 = {S2, NULL};	/*Value = True*/
const Property P12470_1 = {S50, 0, &V12470_1_1, NULL};	/*Property = IsMobile*/
const Value V12470_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12470_5 = {S52, 0, &V12470_5_1222, &P12470_1};	/*Property = ScreenPixelsHeight*/
const Value V12470_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12470_4 = {S54, 0, &V12470_4_1224, &P12470_5};	/*Property = ScreenPixelsWidth*/
const Profile P12470 = {"12470", &P12470_4};	/*Profile ID - 12470*/
const Value V14660_1_1 = {S2, NULL};	/*Value = True*/
const Property P14660_1 = {S50, 0, &V14660_1_1, NULL};	/*Property = IsMobile*/
const Value V14660_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14660_5 = {S52, 0, &V14660_5_1222, &P14660_1};	/*Property = ScreenPixelsHeight*/
const Value V14660_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14660_4 = {S54, 0, &V14660_4_1224, &P14660_5};	/*Property = ScreenPixelsWidth*/
const Profile P14660 = {"14660", &P14660_4};	/*Profile ID - 14660*/
const Value V12479_1_1 = {S2, NULL};	/*Value = True*/
const Property P12479_1 = {S50, 0, &V12479_1_1, NULL};	/*Property = IsMobile*/
const Value V12479_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12479_5 = {S52, 0, &V12479_5_1233, &P12479_1};	/*Property = ScreenPixelsHeight*/
const Value V12479_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12479_4 = {S54, 0, &V12479_4_1235, &P12479_5};	/*Property = ScreenPixelsWidth*/
const Profile P12479 = {"12479", &P12479_4};	/*Profile ID - 12479*/
const Value V12471_1_1 = {S2, NULL};	/*Value = True*/
const Property P12471_1 = {S50, 0, &V12471_1_1, NULL};	/*Property = IsMobile*/
const Value V12471_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12471_5 = {S52, 0, &V12471_5_1293, &P12471_1};	/*Property = ScreenPixelsHeight*/
const Value V12471_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12471_4 = {S54, 0, &V12471_4_1296, &P12471_5};	/*Property = ScreenPixelsWidth*/
const Profile P12471 = {"12471", &P12471_4};	/*Profile ID - 12471*/
const Value V17153_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17153_279 = {S1, 0, &V17153_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17153_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17153_282 = {S3, 0, &V17153_282_57154, &P17153_279};	/*Property = BlobBuilder*/
const Value V17153_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17153_350 = {S4, 0, &V17153_350_57444, &P17153_282};	/*Property = Canvas*/
const Value V17153_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17153_283 = {S5, 0, &V17153_283_57159, &P17153_350};	/*Property = CssBackground*/
const Value V17153_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17153_284 = {S6, 0, &V17153_284_57163, &P17153_283};	/*Property = CssBorderImage*/
const Value V17153_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17153_285 = {S7, 0, &V17153_285_57166, &P17153_284};	/*Property = CssCanvas*/
const Value V17153_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17153_286 = {S8, 0, &V17153_286_57168, &P17153_285};	/*Property = CssColor*/
const Value V17153_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17153_237 = {S9, 0, &V17153_237_55786, &P17153_286};	/*Property = CssColumn*/
const Value V17153_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17153_287 = {S10, 0, &V17153_287_57172, &P17153_237};	/*Property = CssFlexbox*/
const Value V17153_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17153_288 = {S11, 0, &V17153_288_57175, &P17153_287};	/*Property = CssFont*/
const Value V17153_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17153_289 = {S12, 0, &V17153_289_57759, &P17153_288};	/*Property = CssImages*/
const Value V17153_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17153_291 = {S13, 0, &V17153_291_57181, &P17153_289};	/*Property = CssMediaQueries*/
const Value V17153_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17153_292 = {S14, 0, &V17153_292_57183, &P17153_291};	/*Property = CssMinMax*/
const Value V17153_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17153_293 = {S15, 0, &V17153_293_57187, &P17153_292};	/*Property = CssOverflow*/
const Value V17153_294_57190 = {S0, NULL};	/*Value = False*/
const Property P17153_294 = {S16, 0, &V17153_294_57190, &P17153_293};	/*Property = CssPosition*/
const Value V17153_295_57193 = {S0, NULL};	/*Value = False*/
const Property P17153_295 = {S17, 0, &V17153_295_57193, &P17153_294};	/*Property = CssText*/
const Value V17153_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17153_297 = {S18, 0, &V17153_297_57195, &P17153_295};	/*Property = CssTransforms*/
const Value V17153_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17153_241 = {S19, 0, &V17153_241_55799, &P17153_297};	/*Property = CssTransitions*/
const Value V17153_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17153_298 = {S20, 0, &V17153_298_57206, &P17153_241};	/*Property = CssUI*/
const Value V17153_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17153_300 = {S21, 0, &V17153_300_57216, &P17153_298};	/*Property = DataSet*/
const Value V17153_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17153_301 = {S22, 0, &V17153_301_57218, &P17153_300};	/*Property = DataUrl*/
const Value V17153_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17153_302 = {S23, 0, &V17153_302_57222, &P17153_301};	/*Property = DeviceOrientation*/
const Value V17153_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17153_304 = {S24, 0, &V17153_304_57228, &P17153_302};	/*Property = FileReader*/
const Value V17153_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17153_305 = {S25, 0, &V17153_305_57231, &P17153_304};	/*Property = FileSaver*/
const Value V17153_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17153_306 = {S26, 0, &V17153_306_57237, &P17153_305};	/*Property = FileWriter*/
const Value V17153_307_57240 = {S0, NULL};	/*Value = False*/
const Property P17153_307 = {S27, 0, &V17153_307_57240, &P17153_306};	/*Property = FormData*/
const Value V17153_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17153_309 = {S28, 0, &V17153_309_57246, &P17153_307};	/*Property = Fullscreen*/
const Value V17153_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17153_310 = {S29, 0, &V17153_310_57248, &P17153_309};	/*Property = GeoLocation*/
const Value V17153_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17153_312 = {S30, 0, &V17153_312_57255, &P17153_310};	/*Property = History*/
const Value V17153_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17153_314 = {S31, 0, &V17153_314_57261, &P17153_312};	/*Property = Html5*/
const Value V17153_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17153_313 = {S32, 0, &V17153_313_57258, &P17153_314};	/*Property = Html-Media-Capture*/
const Value V17153_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17153_315 = {S33, 0, &V17153_315_57264, &P17153_313};	/*Property = Iframe*/
const Value V17153_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17153_316 = {S34, 0, &V17153_316_57267, &P17153_315};	/*Property = IndexedDB*/
const Value V17153_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17153_317 = {S35, 0, &V17153_317_57269, &P17153_316};	/*Property = Json*/
const Value V17153_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17153_15 = {S37, 0, &V17153_15_28, &P17153_317};	/*Property = LayoutEngine*/
const Value V17153_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17153_344 = {S38, 0, &V17153_344_57272, &P17153_15};	/*Property = Masking*/
const Value V17153_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17153_320 = {S39, 0, &V17153_320_57290, &P17153_344};	/*Property = PostMessage*/
const Value V17153_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17153_321 = {S40, 0, &V17153_321_57293, &P17153_320};	/*Property = Progress*/
const Value V17153_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17153_322 = {S41, 0, &V17153_322_57296, &P17153_321};	/*Property = Prompts*/
const Value V17153_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17153_323 = {S42, 0, &V17153_323_57299, &P17153_322};	/*Property = Selector*/
const Value V17153_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17153_324 = {S43, 0, &V17153_324_57306, &P17153_323};	/*Property = Svg*/
const Value V17153_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17153_327 = {S44, 0, &V17153_327_57315, &P17153_324};	/*Property = TouchEvents*/
const Value V17153_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17153_328 = {S45, 0, &V17153_328_57318, &P17153_327};	/*Property = Track*/
const Value V17153_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17153_329 = {S46, 0, &V17153_329_57323, &P17153_328};	/*Property = Video*/
const Value V17153_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17153_330 = {S47, 0, &V17153_330_57326, &P17153_329};	/*Property = Viewport*/
const Value V17153_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17153_332 = {S48, 0, &V17153_332_57339, &P17153_330};	/*Property = WebWorkers*/
const Value V17153_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17153_333 = {S49, 0, &V17153_333_57342, &P17153_332};	/*Property = Xhr2*/
const Profile P17153 = {"17153", &P17153_333};	/*Profile ID - 17153*/
const Value V12481_1_1 = {S2, NULL};	/*Value = True*/
const Property P12481_1 = {S50, 0, &V12481_1_1, NULL};	/*Property = IsMobile*/
const Value V12481_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12481_5 = {S52, 0, &V12481_5_1304, &P12481_1};	/*Property = ScreenPixelsHeight*/
const Value V12481_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12481_4 = {S54, 0, &V12481_4_1224, &P12481_5};	/*Property = ScreenPixelsWidth*/
const Profile P12481 = {"12481", &P12481_4};	/*Profile ID - 12481*/
const Value V12483_1_1 = {S2, NULL};	/*Value = True*/
const Property P12483_1 = {S50, 0, &V12483_1_1, NULL};	/*Property = IsMobile*/
const Value V12483_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12483_5 = {S52, 0, &V12483_5_1222, &P12483_1};	/*Property = ScreenPixelsHeight*/
const Value V12483_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12483_4 = {S54, 0, &V12483_4_1224, &P12483_5};	/*Property = ScreenPixelsWidth*/
const Profile P12483 = {"12483", &P12483_4};	/*Profile ID - 12483*/
const Value V12485_1_1 = {S2, NULL};	/*Value = True*/
const Property P12485_1 = {S50, 0, &V12485_1_1, NULL};	/*Property = IsMobile*/
const Value V12485_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12485_5 = {S52, 0, &V12485_5_1293, &P12485_1};	/*Property = ScreenPixelsHeight*/
const Value V12485_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12485_4 = {S54, 0, &V12485_4_1296, &P12485_5};	/*Property = ScreenPixelsWidth*/
const Profile P12485 = {"12485", &P12485_4};	/*Profile ID - 12485*/
const Value V17166_1_1 = {S2, NULL};	/*Value = True*/
const Property P17166_1 = {S50, 0, &V17166_1_1, NULL};	/*Property = IsMobile*/
const Value V17166_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17166_5 = {S52, 0, &V17166_5_1222, &P17166_1};	/*Property = ScreenPixelsHeight*/
const Value V17166_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17166_4 = {S54, 0, &V17166_4_1224, &P17166_5};	/*Property = ScreenPixelsWidth*/
const Profile P17166 = {"17166", &P17166_4};	/*Profile ID - 17166*/
const Value V12491_1_1 = {S2, NULL};	/*Value = True*/
const Property P12491_1 = {S50, 0, &V12491_1_1, NULL};	/*Property = IsMobile*/
const Value V12491_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12491_5 = {S52, 0, &V12491_5_1293, &P12491_1};	/*Property = ScreenPixelsHeight*/
const Value V12491_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12491_4 = {S54, 0, &V12491_4_1296, &P12491_5};	/*Property = ScreenPixelsWidth*/
const Profile P12491 = {"12491", &P12491_4};	/*Profile ID - 12491*/
const Value V17913_1_1 = {S2, NULL};	/*Value = True*/
const Property P17913_1 = {S50, 0, &V17913_1_1, NULL};	/*Property = IsMobile*/
const Value V17913_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17913_5 = {S52, 0, &V17913_5_1227, &P17913_1};	/*Property = ScreenPixelsHeight*/
const Value V17913_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17913_4 = {S54, 0, &V17913_4_1229, &P17913_5};	/*Property = ScreenPixelsWidth*/
const Profile P17913 = {"17913", &P17913_4};	/*Profile ID - 17913*/
const Value V12498_1_1 = {S2, NULL};	/*Value = True*/
const Property P12498_1 = {S50, 0, &V12498_1_1, NULL};	/*Property = IsMobile*/
const Value V12498_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12498_5 = {S52, 0, &V12498_5_1233, &P12498_1};	/*Property = ScreenPixelsHeight*/
const Value V12498_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12498_4 = {S54, 0, &V12498_4_1235, &P12498_5};	/*Property = ScreenPixelsWidth*/
const Profile P12498 = {"12498", &P12498_4};	/*Profile ID - 12498*/
const Value V12500_1_1 = {S2, NULL};	/*Value = True*/
const Property P12500_1 = {S50, 0, &V12500_1_1, NULL};	/*Property = IsMobile*/
const Value V12500_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12500_5 = {S52, 0, &V12500_5_1222, &P12500_1};	/*Property = ScreenPixelsHeight*/
const Value V12500_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12500_4 = {S54, 0, &V12500_4_1224, &P12500_5};	/*Property = ScreenPixelsWidth*/
const Profile P12500 = {"12500", &P12500_4};	/*Profile ID - 12500*/
const Value V12502_1_1 = {S2, NULL};	/*Value = True*/
const Property P12502_1 = {S50, 0, &V12502_1_1, NULL};	/*Property = IsMobile*/
const Value V12502_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12502_5 = {S52, 0, &V12502_5_1222, &P12502_1};	/*Property = ScreenPixelsHeight*/
const Value V12502_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12502_4 = {S54, 0, &V12502_4_1224, &P12502_5};	/*Property = ScreenPixelsWidth*/
const Profile P12502 = {"12502", &P12502_4};	/*Profile ID - 12502*/
const Value V12509_1_1 = {S2, NULL};	/*Value = True*/
const Property P12509_1 = {S50, 0, &V12509_1_1, NULL};	/*Property = IsMobile*/
const Value V12509_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12509_5 = {S52, 0, &V12509_5_1233, &P12509_1};	/*Property = ScreenPixelsHeight*/
const Value V12509_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12509_4 = {S54, 0, &V12509_4_1235, &P12509_5};	/*Property = ScreenPixelsWidth*/
const Profile P12509 = {"12509", &P12509_4};	/*Profile ID - 12509*/
const Value V12510_1_1 = {S2, NULL};	/*Value = True*/
const Property P12510_1 = {S50, 0, &V12510_1_1, NULL};	/*Property = IsMobile*/
const Value V12510_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12510_5 = {S52, 0, &V12510_5_1294, &P12510_1};	/*Property = ScreenPixelsHeight*/
const Value V12510_4_33019 = {S61, NULL};	/*Value = 160*/
const Property P12510_4 = {S54, 0, &V12510_4_33019, &P12510_5};	/*Property = ScreenPixelsWidth*/
const Profile P12510 = {"12510", &P12510_4};	/*Profile ID - 12510*/
const Value V12512_1_1 = {S2, NULL};	/*Value = True*/
const Property P12512_1 = {S50, 0, &V12512_1_1, NULL};	/*Property = IsMobile*/
const Value V12512_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12512_5 = {S52, 0, &V12512_5_1380, &P12512_1};	/*Property = ScreenPixelsHeight*/
const Value V12512_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12512_4 = {S54, 0, &V12512_4_1279, &P12512_5};	/*Property = ScreenPixelsWidth*/
const Profile P12512 = {"12512", &P12512_4};	/*Profile ID - 12512*/
const Value V12514_1_1 = {S2, NULL};	/*Value = True*/
const Property P12514_1 = {S50, 0, &V12514_1_1, NULL};	/*Property = IsMobile*/
const Value V12514_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12514_5 = {S52, 0, &V12514_5_1222, &P12514_1};	/*Property = ScreenPixelsHeight*/
const Value V12514_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12514_4 = {S54, 0, &V12514_4_1224, &P12514_5};	/*Property = ScreenPixelsWidth*/
const Profile P12514 = {"12514", &P12514_4};	/*Profile ID - 12514*/
const Value V12515_1_1 = {S2, NULL};	/*Value = True*/
const Property P12515_1 = {S50, 0, &V12515_1_1, NULL};	/*Property = IsMobile*/
const Value V12515_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12515_5 = {S52, 0, &V12515_5_1293, &P12515_1};	/*Property = ScreenPixelsHeight*/
const Value V12515_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12515_4 = {S54, 0, &V12515_4_1296, &P12515_5};	/*Property = ScreenPixelsWidth*/
const Profile P12515 = {"12515", &P12515_4};	/*Profile ID - 12515*/
const Value V12517_1_1 = {S2, NULL};	/*Value = True*/
const Property P12517_1 = {S50, 0, &V12517_1_1, NULL};	/*Property = IsMobile*/
const Value V12517_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12517_5 = {S52, 0, &V12517_5_1222, &P12517_1};	/*Property = ScreenPixelsHeight*/
const Value V12517_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12517_4 = {S54, 0, &V12517_4_1224, &P12517_5};	/*Property = ScreenPixelsWidth*/
const Profile P12517 = {"12517", &P12517_4};	/*Profile ID - 12517*/
const Value V12751_1_1 = {S2, NULL};	/*Value = True*/
const Property P12751_1 = {S50, 0, &V12751_1_1, NULL};	/*Property = IsMobile*/
const Value V12751_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12751_5 = {S52, 0, &V12751_5_1294, &P12751_1};	/*Property = ScreenPixelsHeight*/
const Value V12751_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12751_4 = {S54, 0, &V12751_4_1296, &P12751_5};	/*Property = ScreenPixelsWidth*/
const Profile P12751 = {"12751", &P12751_4};	/*Profile ID - 12751*/
const Value V16700_1_1 = {S2, NULL};	/*Value = True*/
const Property P16700_1 = {S50, 0, &V16700_1_1, NULL};	/*Property = IsMobile*/
const Value V16700_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16700_5 = {S52, 0, &V16700_5_1293, &P16700_1};	/*Property = ScreenPixelsHeight*/
const Value V16700_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16700_4 = {S54, 0, &V16700_4_1296, &P16700_5};	/*Property = ScreenPixelsWidth*/
const Profile P16700 = {"16700", &P16700_4};	/*Profile ID - 16700*/
const Value V12752_1_1 = {S2, NULL};	/*Value = True*/
const Property P12752_1 = {S50, 0, &V12752_1_1, NULL};	/*Property = IsMobile*/
const Value V12752_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12752_5 = {S52, 0, &V12752_5_1380, &P12752_1};	/*Property = ScreenPixelsHeight*/
const Value V12752_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12752_4 = {S54, 0, &V12752_4_1279, &P12752_5};	/*Property = ScreenPixelsWidth*/
const Profile P12752 = {"12752", &P12752_4};	/*Profile ID - 12752*/
const Value V12753_1_1 = {S2, NULL};	/*Value = True*/
const Property P12753_1 = {S50, 0, &V12753_1_1, NULL};	/*Property = IsMobile*/
const Value V12753_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12753_5 = {S52, 0, &V12753_5_1233, &P12753_1};	/*Property = ScreenPixelsHeight*/
const Value V12753_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12753_4 = {S54, 0, &V12753_4_1235, &P12753_5};	/*Property = ScreenPixelsWidth*/
const Profile P12753 = {"12753", &P12753_4};	/*Profile ID - 12753*/
const Value V12755_1_1 = {S2, NULL};	/*Value = True*/
const Property P12755_1 = {S50, 0, &V12755_1_1, NULL};	/*Property = IsMobile*/
const Value V12755_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12755_5 = {S52, 0, &V12755_5_1277, &P12755_1};	/*Property = ScreenPixelsHeight*/
const Value V12755_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12755_4 = {S54, 0, &V12755_4_1279, &P12755_5};	/*Property = ScreenPixelsWidth*/
const Profile P12755 = {"12755", &P12755_4};	/*Profile ID - 12755*/
const Value V12756_1_1 = {S2, NULL};	/*Value = True*/
const Property P12756_1 = {S50, 0, &V12756_1_1, NULL};	/*Property = IsMobile*/
const Value V12756_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12756_5 = {S52, 0, &V12756_5_1222, &P12756_1};	/*Property = ScreenPixelsHeight*/
const Value V12756_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12756_4 = {S54, 0, &V12756_4_1224, &P12756_5};	/*Property = ScreenPixelsWidth*/
const Profile P12756 = {"12756", &P12756_4};	/*Profile ID - 12756*/
const Value V12757_1_1 = {S2, NULL};	/*Value = True*/
const Property P12757_1 = {S50, 0, &V12757_1_1, NULL};	/*Property = IsMobile*/
const Value V12757_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12757_5 = {S52, 0, &V12757_5_1233, &P12757_1};	/*Property = ScreenPixelsHeight*/
const Value V12757_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12757_4 = {S54, 0, &V12757_4_1235, &P12757_5};	/*Property = ScreenPixelsWidth*/
const Profile P12757 = {"12757", &P12757_4};	/*Profile ID - 12757*/
const Profile P19949 = {"19949", NULL};	/*Profile ID - 19949*/
const Value V12760_1_1 = {S2, NULL};	/*Value = True*/
const Property P12760_1 = {S50, 0, &V12760_1_1, NULL};	/*Property = IsMobile*/
const Value V12760_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12760_5 = {S52, 0, &V12760_5_1222, &P12760_1};	/*Property = ScreenPixelsHeight*/
const Value V12760_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12760_4 = {S54, 0, &V12760_4_1224, &P12760_5};	/*Property = ScreenPixelsWidth*/
const Profile P12760 = {"12760", &P12760_4};	/*Profile ID - 12760*/
const Value V12761_1_1 = {S2, NULL};	/*Value = True*/
const Property P12761_1 = {S50, 0, &V12761_1_1, NULL};	/*Property = IsMobile*/
const Value V12761_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12761_5 = {S52, 0, &V12761_5_1222, &P12761_1};	/*Property = ScreenPixelsHeight*/
const Value V12761_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12761_4 = {S54, 0, &V12761_4_1224, &P12761_5};	/*Property = ScreenPixelsWidth*/
const Profile P12761 = {"12761", &P12761_4};	/*Profile ID - 12761*/
const Value V12763_1_1 = {S2, NULL};	/*Value = True*/
const Property P12763_1 = {S50, 0, &V12763_1_1, NULL};	/*Property = IsMobile*/
const Value V12763_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12763_5 = {S52, 0, &V12763_5_1380, &P12763_1};	/*Property = ScreenPixelsHeight*/
const Value V12763_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12763_4 = {S54, 0, &V12763_4_1279, &P12763_5};	/*Property = ScreenPixelsWidth*/
const Profile P12763 = {"12763", &P12763_4};	/*Profile ID - 12763*/
const Value V12765_1_1 = {S2, NULL};	/*Value = True*/
const Property P12765_1 = {S50, 0, &V12765_1_1, NULL};	/*Property = IsMobile*/
const Value V12765_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P12765_5 = {S52, 0, &V12765_5_1458, &P12765_1};	/*Property = ScreenPixelsHeight*/
const Value V12765_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P12765_4 = {S54, 0, &V12765_4_1460, &P12765_5};	/*Property = ScreenPixelsWidth*/
const Profile P12765 = {"12765", &P12765_4};	/*Profile ID - 12765*/
const Value V12767_1_1 = {S2, NULL};	/*Value = True*/
const Property P12767_1 = {S50, 0, &V12767_1_1, NULL};	/*Property = IsMobile*/
const Value V12767_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12767_5 = {S52, 0, &V12767_5_1293, &P12767_1};	/*Property = ScreenPixelsHeight*/
const Value V12767_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12767_4 = {S54, 0, &V12767_4_1296, &P12767_5};	/*Property = ScreenPixelsWidth*/
const Profile P12767 = {"12767", &P12767_4};	/*Profile ID - 12767*/
const Value V12768_1_1 = {S2, NULL};	/*Value = True*/
const Property P12768_1 = {S50, 0, &V12768_1_1, NULL};	/*Property = IsMobile*/
const Value V12768_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12768_5 = {S52, 0, &V12768_5_1222, &P12768_1};	/*Property = ScreenPixelsHeight*/
const Value V12768_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12768_4 = {S54, 0, &V12768_4_1224, &P12768_5};	/*Property = ScreenPixelsWidth*/
const Profile P12768 = {"12768", &P12768_4};	/*Profile ID - 12768*/
const Value V12769_1_1 = {S2, NULL};	/*Value = True*/
const Property P12769_1 = {S50, 0, &V12769_1_1, NULL};	/*Property = IsMobile*/
const Value V12769_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12769_5 = {S52, 0, &V12769_5_1277, &P12769_1};	/*Property = ScreenPixelsHeight*/
const Value V12769_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12769_4 = {S54, 0, &V12769_4_1279, &P12769_5};	/*Property = ScreenPixelsWidth*/
const Profile P12769 = {"12769", &P12769_4};	/*Profile ID - 12769*/
const Value V12770_1_1 = {S2, NULL};	/*Value = True*/
const Property P12770_1 = {S50, 0, &V12770_1_1, NULL};	/*Property = IsMobile*/
const Value V12770_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12770_5 = {S52, 0, &V12770_5_1294, &P12770_1};	/*Property = ScreenPixelsHeight*/
const Value V12770_4_33019 = {S61, NULL};	/*Value = 160*/
const Property P12770_4 = {S54, 0, &V12770_4_33019, &P12770_5};	/*Property = ScreenPixelsWidth*/
const Profile P12770 = {"12770", &P12770_4};	/*Profile ID - 12770*/
const Value V16637_1_1 = {S2, NULL};	/*Value = True*/
const Property P16637_1 = {S50, 0, &V16637_1_1, NULL};	/*Property = IsMobile*/
const Value V16637_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16637_5 = {S52, 0, &V16637_5_1294, &P16637_1};	/*Property = ScreenPixelsHeight*/
const Value V16637_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16637_4 = {S54, 0, &V16637_4_1296, &P16637_5};	/*Property = ScreenPixelsWidth*/
const Profile P16637 = {"16637", &P16637_4};	/*Profile ID - 16637*/
const Value V12771_1_1 = {S2, NULL};	/*Value = True*/
const Property P12771_1 = {S50, 0, &V12771_1_1, NULL};	/*Property = IsMobile*/
const Value V12771_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12771_5 = {S52, 0, &V12771_5_1222, &P12771_1};	/*Property = ScreenPixelsHeight*/
const Value V12771_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12771_4 = {S54, 0, &V12771_4_1224, &P12771_5};	/*Property = ScreenPixelsWidth*/
const Profile P12771 = {"12771", &P12771_4};	/*Profile ID - 12771*/
const Value V12773_1_1 = {S2, NULL};	/*Value = True*/
const Property P12773_1 = {S50, 0, &V12773_1_1, NULL};	/*Property = IsMobile*/
const Value V12773_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12773_5 = {S52, 0, &V12773_5_1380, &P12773_1};	/*Property = ScreenPixelsHeight*/
const Value V12773_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12773_4 = {S54, 0, &V12773_4_1279, &P12773_5};	/*Property = ScreenPixelsWidth*/
const Profile P12773 = {"12773", &P12773_4};	/*Profile ID - 12773*/
const Value V12798_1_1 = {S2, NULL};	/*Value = True*/
const Property P12798_1 = {S50, 0, &V12798_1_1, NULL};	/*Property = IsMobile*/
const Value V12798_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P12798_5 = {S52, 0, &V12798_5_1269, &P12798_1};	/*Property = ScreenPixelsHeight*/
const Value V12798_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12798_4 = {S54, 0, &V12798_4_1229, &P12798_5};	/*Property = ScreenPixelsWidth*/
const Profile P12798 = {"12798", &P12798_4};	/*Profile ID - 12798*/
const Value V12775_1_1 = {S2, NULL};	/*Value = True*/
const Property P12775_1 = {S50, 0, &V12775_1_1, NULL};	/*Property = IsMobile*/
const Value V12775_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12775_5 = {S52, 0, &V12775_5_1293, &P12775_1};	/*Property = ScreenPixelsHeight*/
const Value V12775_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12775_4 = {S54, 0, &V12775_4_1296, &P12775_5};	/*Property = ScreenPixelsWidth*/
const Profile P12775 = {"12775", &P12775_4};	/*Profile ID - 12775*/
const Value V12532_1_1 = {S2, NULL};	/*Value = True*/
const Property P12532_1 = {S50, 0, &V12532_1_1, NULL};	/*Property = IsMobile*/
const Value V12532_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P12532_5 = {S52, 0, &V12532_5_1458, &P12532_1};	/*Property = ScreenPixelsHeight*/
const Value V12532_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P12532_4 = {S54, 0, &V12532_4_1460, &P12532_5};	/*Property = ScreenPixelsWidth*/
const Profile P12532 = {"12532", &P12532_4};	/*Profile ID - 12532*/
const Value V12776_1_1 = {S2, NULL};	/*Value = True*/
const Property P12776_1 = {S50, 0, &V12776_1_1, NULL};	/*Property = IsMobile*/
const Value V12776_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12776_5 = {S52, 0, &V12776_5_1222, &P12776_1};	/*Property = ScreenPixelsHeight*/
const Value V12776_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12776_4 = {S54, 0, &V12776_4_1224, &P12776_5};	/*Property = ScreenPixelsWidth*/
const Profile P12776 = {"12776", &P12776_4};	/*Profile ID - 12776*/
const Value V4418_279_57148 = {S0, NULL};	/*Value = False*/
const Property P4418_279 = {S1, 0, &V4418_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V4418_282_57154 = {S0, NULL};	/*Value = False*/
const Property P4418_282 = {S3, 0, &V4418_282_57154, &P4418_279};	/*Property = BlobBuilder*/
const Value V4418_350_57444 = {S2, NULL};	/*Value = True*/
const Property P4418_350 = {S4, 0, &V4418_350_57444, &P4418_282};	/*Property = Canvas*/
const Value V4418_283_57159 = {S2, NULL};	/*Value = True*/
const Property P4418_283 = {S5, 0, &V4418_283_57159, &P4418_350};	/*Property = CssBackground*/
const Value V4418_284_57163 = {S0, NULL};	/*Value = False*/
const Property P4418_284 = {S6, 0, &V4418_284_57163, &P4418_283};	/*Property = CssBorderImage*/
const Value V4418_285_57166 = {S0, NULL};	/*Value = False*/
const Property P4418_285 = {S7, 0, &V4418_285_57166, &P4418_284};	/*Property = CssCanvas*/
const Value V4418_286_57168 = {S2, NULL};	/*Value = True*/
const Property P4418_286 = {S8, 0, &V4418_286_57168, &P4418_285};	/*Property = CssColor*/
const Value V4418_237_55784 = {S2, NULL};	/*Value = True*/
const Property P4418_237 = {S9, 0, &V4418_237_55784, &P4418_286};	/*Property = CssColumn*/
const Value V4418_287_57172 = {S0, NULL};	/*Value = False*/
const Property P4418_287 = {S10, 0, &V4418_287_57172, &P4418_237};	/*Property = CssFlexbox*/
const Value V4418_288_57175 = {S0, NULL};	/*Value = False*/
const Property P4418_288 = {S11, 0, &V4418_288_57175, &P4418_287};	/*Property = CssFont*/
const Value V4418_289_57759 = {S0, NULL};	/*Value = False*/
const Property P4418_289 = {S12, 0, &V4418_289_57759, &P4418_288};	/*Property = CssImages*/
const Value V4418_291_57181 = {S0, NULL};	/*Value = False*/
const Property P4418_291 = {S13, 0, &V4418_291_57181, &P4418_289};	/*Property = CssMediaQueries*/
const Value V4418_292_57183 = {S2, NULL};	/*Value = True*/
const Property P4418_292 = {S14, 0, &V4418_292_57183, &P4418_291};	/*Property = CssMinMax*/
const Value V4418_293_57187 = {S0, NULL};	/*Value = False*/
const Property P4418_293 = {S15, 0, &V4418_293_57187, &P4418_292};	/*Property = CssOverflow*/
const Value V4418_294_57189 = {S2, NULL};	/*Value = True*/
const Property P4418_294 = {S16, 0, &V4418_294_57189, &P4418_293};	/*Property = CssPosition*/
const Value V4418_295_57192 = {S2, NULL};	/*Value = True*/
const Property P4418_295 = {S17, 0, &V4418_295_57192, &P4418_294};	/*Property = CssText*/
const Value V4418_297_57195 = {S2, NULL};	/*Value = True*/
const Property P4418_297 = {S18, 0, &V4418_297_57195, &P4418_295};	/*Property = CssTransforms*/
const Value V4418_241_55799 = {S2, NULL};	/*Value = True*/
const Property P4418_241 = {S19, 0, &V4418_241_55799, &P4418_297};	/*Property = CssTransitions*/
const Value V4418_298_57206 = {S2, NULL};	/*Value = True*/
const Property P4418_298 = {S20, 0, &V4418_298_57206, &P4418_241};	/*Property = CssUI*/
const Value V4418_300_57216 = {S0, NULL};	/*Value = False*/
const Property P4418_300 = {S21, 0, &V4418_300_57216, &P4418_298};	/*Property = DataSet*/
const Value V4418_301_57218 = {S2, NULL};	/*Value = True*/
const Property P4418_301 = {S22, 0, &V4418_301_57218, &P4418_300};	/*Property = DataUrl*/
const Value V4418_302_57222 = {S0, NULL};	/*Value = False*/
const Property P4418_302 = {S23, 0, &V4418_302_57222, &P4418_301};	/*Property = DeviceOrientation*/
const Value V4418_304_57228 = {S0, NULL};	/*Value = False*/
const Property P4418_304 = {S24, 0, &V4418_304_57228, &P4418_302};	/*Property = FileReader*/
const Value V4418_305_57231 = {S0, NULL};	/*Value = False*/
const Property P4418_305 = {S25, 0, &V4418_305_57231, &P4418_304};	/*Property = FileSaver*/
const Value V4418_306_57237 = {S0, NULL};	/*Value = False*/
const Property P4418_306 = {S26, 0, &V4418_306_57237, &P4418_305};	/*Property = FileWriter*/
const Value V4418_307_57239 = {S2, NULL};	/*Value = True*/
const Property P4418_307 = {S27, 0, &V4418_307_57239, &P4418_306};	/*Property = FormData*/
const Value V4418_309_57246 = {S0, NULL};	/*Value = False*/
const Property P4418_309 = {S28, 0, &V4418_309_57246, &P4418_307};	/*Property = Fullscreen*/
const Value V4418_310_57248 = {S2, NULL};	/*Value = True*/
const Property P4418_310 = {S29, 0, &V4418_310_57248, &P4418_309};	/*Property = GeoLocation*/
const Value V4418_312_57255 = {S0, NULL};	/*Value = False*/
const Property P4418_312 = {S30, 0, &V4418_312_57255, &P4418_310};	/*Property = History*/
const Value V4418_314_57261 = {S0, NULL};	/*Value = False*/
const Property P4418_314 = {S31, 0, &V4418_314_57261, &P4418_312};	/*Property = Html5*/
const Value V4418_313_57258 = {S0, NULL};	/*Value = False*/
const Property P4418_313 = {S32, 0, &V4418_313_57258, &P4418_314};	/*Property = Html-Media-Capture*/
const Value V4418_315_57264 = {S0, NULL};	/*Value = False*/
const Property P4418_315 = {S33, 0, &V4418_315_57264, &P4418_313};	/*Property = Iframe*/
const Value V4418_316_57267 = {S0, NULL};	/*Value = False*/
const Property P4418_316 = {S34, 0, &V4418_316_57267, &P4418_315};	/*Property = IndexedDB*/
const Value V4418_317_57269 = {S2, NULL};	/*Value = True*/
const Property P4418_317 = {S35, 0, &V4418_317_57269, &P4418_316};	/*Property = Json*/
const Value V4418_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P4418_15 = {S37, 0, &V4418_15_28, &P4418_317};	/*Property = LayoutEngine*/
const Value V4418_344_57272 = {S2, NULL};	/*Value = True*/
const Property P4418_344 = {S38, 0, &V4418_344_57272, &P4418_15};	/*Property = Masking*/
const Value V4418_320_57290 = {S2, NULL};	/*Value = True*/
const Property P4418_320 = {S39, 0, &V4418_320_57290, &P4418_344};	/*Property = PostMessage*/
const Value V4418_321_57293 = {S2, NULL};	/*Value = True*/
const Property P4418_321 = {S40, 0, &V4418_321_57293, &P4418_320};	/*Property = Progress*/
const Value V4418_322_57296 = {S2, NULL};	/*Value = True*/
const Property P4418_322 = {S41, 0, &V4418_322_57296, &P4418_321};	/*Property = Prompts*/
const Value V4418_323_57299 = {S2, NULL};	/*Value = True*/
const Property P4418_323 = {S42, 0, &V4418_323_57299, &P4418_322};	/*Property = Selector*/
const Value V4418_324_57306 = {S0, NULL};	/*Value = False*/
const Property P4418_324 = {S43, 0, &V4418_324_57306, &P4418_323};	/*Property = Svg*/
const Value V4418_327_57315 = {S0, NULL};	/*Value = False*/
const Property P4418_327 = {S44, 0, &V4418_327_57315, &P4418_324};	/*Property = TouchEvents*/
const Value V4418_328_57318 = {S0, NULL};	/*Value = False*/
const Property P4418_328 = {S45, 0, &V4418_328_57318, &P4418_327};	/*Property = Track*/
const Value V4418_329_57323 = {S2, NULL};	/*Value = True*/
const Property P4418_329 = {S46, 0, &V4418_329_57323, &P4418_328};	/*Property = Video*/
const Value V4418_330_57326 = {S2, NULL};	/*Value = True*/
const Property P4418_330 = {S47, 0, &V4418_330_57326, &P4418_329};	/*Property = Viewport*/
const Value V4418_332_57339 = {S0, NULL};	/*Value = False*/
const Property P4418_332 = {S48, 0, &V4418_332_57339, &P4418_330};	/*Property = WebWorkers*/
const Value V4418_333_57342 = {S0, NULL};	/*Value = False*/
const Property P4418_333 = {S49, 0, &V4418_333_57342, &P4418_332};	/*Property = Xhr2*/
const Profile P4418 = {"4418", &P4418_333};	/*Profile ID - 4418*/
const Profile P6374 = {"6374", NULL};	/*Profile ID - 6374*/
const Value V15232_1_1 = {S2, NULL};	/*Value = True*/
const Property P15232_1 = {S50, 0, &V15232_1_1, NULL};	/*Property = IsMobile*/
const Value V15232_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15232_5 = {S52, 0, &V15232_5_1277, &P15232_1};	/*Property = ScreenPixelsHeight*/
const Value V15232_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P15232_4 = {S54, 0, &V15232_4_8554, &P15232_5};	/*Property = ScreenPixelsWidth*/
const Profile P15232 = {"15232", &P15232_4};	/*Profile ID - 15232*/
const Value V17159_1_1 = {S2, NULL};	/*Value = True*/
const Property P17159_1 = {S50, 0, &V17159_1_1, NULL};	/*Property = IsMobile*/
const Value V17159_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17159_5 = {S52, 0, &V17159_5_1233, &P17159_1};	/*Property = ScreenPixelsHeight*/
const Value V17159_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17159_4 = {S54, 0, &V17159_4_1235, &P17159_5};	/*Property = ScreenPixelsWidth*/
const Profile P17159 = {"17159", &P17159_4};	/*Profile ID - 17159*/
const Value V15354_1_1 = {S2, NULL};	/*Value = True*/
const Property P15354_1 = {S50, 0, &V15354_1_1, NULL};	/*Property = IsMobile*/
const Value V15354_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15354_5 = {S52, 0, &V15354_5_1277, &P15354_1};	/*Property = ScreenPixelsHeight*/
const Value V15354_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15354_4 = {S54, 0, &V15354_4_1279, &P15354_5};	/*Property = ScreenPixelsWidth*/
const Profile P15354 = {"15354", &P15354_4};	/*Profile ID - 15354*/
const Value V4459_279_57148 = {S0, NULL};	/*Value = False*/
const Property P4459_279 = {S1, 0, &V4459_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V4459_282_57154 = {S0, NULL};	/*Value = False*/
const Property P4459_282 = {S3, 0, &V4459_282_57154, &P4459_279};	/*Property = BlobBuilder*/
const Value V4459_350_57444 = {S2, NULL};	/*Value = True*/
const Property P4459_350 = {S4, 0, &V4459_350_57444, &P4459_282};	/*Property = Canvas*/
const Value V4459_283_57159 = {S2, NULL};	/*Value = True*/
const Property P4459_283 = {S5, 0, &V4459_283_57159, &P4459_350};	/*Property = CssBackground*/
const Value V4459_284_57163 = {S0, NULL};	/*Value = False*/
const Property P4459_284 = {S6, 0, &V4459_284_57163, &P4459_283};	/*Property = CssBorderImage*/
const Value V4459_285_57166 = {S0, NULL};	/*Value = False*/
const Property P4459_285 = {S7, 0, &V4459_285_57166, &P4459_284};	/*Property = CssCanvas*/
const Value V4459_286_57168 = {S2, NULL};	/*Value = True*/
const Property P4459_286 = {S8, 0, &V4459_286_57168, &P4459_285};	/*Property = CssColor*/
const Value V4459_237_55784 = {S2, NULL};	/*Value = True*/
const Property P4459_237 = {S9, 0, &V4459_237_55784, &P4459_286};	/*Property = CssColumn*/
const Value V4459_287_57172 = {S0, NULL};	/*Value = False*/
const Property P4459_287 = {S10, 0, &V4459_287_57172, &P4459_237};	/*Property = CssFlexbox*/
const Value V4459_288_57175 = {S0, NULL};	/*Value = False*/
const Property P4459_288 = {S11, 0, &V4459_288_57175, &P4459_287};	/*Property = CssFont*/
const Value V4459_289_57759 = {S0, NULL};	/*Value = False*/
const Property P4459_289 = {S12, 0, &V4459_289_57759, &P4459_288};	/*Property = CssImages*/
const Value V4459_291_57180 = {S2, NULL};	/*Value = True*/
const Property P4459_291 = {S13, 0, &V4459_291_57180, &P4459_289};	/*Property = CssMediaQueries*/
const Value V4459_292_57183 = {S2, NULL};	/*Value = True*/
const Property P4459_292 = {S14, 0, &V4459_292_57183, &P4459_291};	/*Property = CssMinMax*/
const Value V4459_293_57187 = {S0, NULL};	/*Value = False*/
const Property P4459_293 = {S15, 0, &V4459_293_57187, &P4459_292};	/*Property = CssOverflow*/
const Value V4459_294_57190 = {S0, NULL};	/*Value = False*/
const Property P4459_294 = {S16, 0, &V4459_294_57190, &P4459_293};	/*Property = CssPosition*/
const Value V4459_295_57192 = {S2, NULL};	/*Value = True*/
const Property P4459_295 = {S17, 0, &V4459_295_57192, &P4459_294};	/*Property = CssText*/
const Value V4459_297_57195 = {S2, NULL};	/*Value = True*/
const Property P4459_297 = {S18, 0, &V4459_297_57195, &P4459_295};	/*Property = CssTransforms*/
const Value V4459_241_55799 = {S2, NULL};	/*Value = True*/
const Property P4459_241 = {S19, 0, &V4459_241_55799, &P4459_297};	/*Property = CssTransitions*/
const Value V4459_298_57206 = {S2, NULL};	/*Value = True*/
const Property P4459_298 = {S20, 0, &V4459_298_57206, &P4459_241};	/*Property = CssUI*/
const Value V4459_300_57216 = {S0, NULL};	/*Value = False*/
const Property P4459_300 = {S21, 0, &V4459_300_57216, &P4459_298};	/*Property = DataSet*/
const Value V4459_301_57218 = {S2, NULL};	/*Value = True*/
const Property P4459_301 = {S22, 0, &V4459_301_57218, &P4459_300};	/*Property = DataUrl*/
const Value V4459_302_57221 = {S2, NULL};	/*Value = True*/
const Property P4459_302 = {S23, 0, &V4459_302_57221, &P4459_301};	/*Property = DeviceOrientation*/
const Value V4459_304_57228 = {S0, NULL};	/*Value = False*/
const Property P4459_304 = {S24, 0, &V4459_304_57228, &P4459_302};	/*Property = FileReader*/
const Value V4459_305_57231 = {S0, NULL};	/*Value = False*/
const Property P4459_305 = {S25, 0, &V4459_305_57231, &P4459_304};	/*Property = FileSaver*/
const Value V4459_306_57237 = {S0, NULL};	/*Value = False*/
const Property P4459_306 = {S26, 0, &V4459_306_57237, &P4459_305};	/*Property = FileWriter*/
const Value V4459_307_57239 = {S2, NULL};	/*Value = True*/
const Property P4459_307 = {S27, 0, &V4459_307_57239, &P4459_306};	/*Property = FormData*/
const Value V4459_309_57246 = {S0, NULL};	/*Value = False*/
const Property P4459_309 = {S28, 0, &V4459_309_57246, &P4459_307};	/*Property = Fullscreen*/
const Value V4459_310_57248 = {S2, NULL};	/*Value = True*/
const Property P4459_310 = {S29, 0, &V4459_310_57248, &P4459_309};	/*Property = GeoLocation*/
const Value V4459_312_57255 = {S0, NULL};	/*Value = False*/
const Property P4459_312 = {S30, 0, &V4459_312_57255, &P4459_310};	/*Property = History*/
const Value V4459_314_57261 = {S0, NULL};	/*Value = False*/
const Property P4459_314 = {S31, 0, &V4459_314_57261, &P4459_312};	/*Property = Html5*/
const Value V4459_313_57258 = {S0, NULL};	/*Value = False*/
const Property P4459_313 = {S32, 0, &V4459_313_57258, &P4459_314};	/*Property = Html-Media-Capture*/
const Value V4459_315_57264 = {S0, NULL};	/*Value = False*/
const Property P4459_315 = {S33, 0, &V4459_315_57264, &P4459_313};	/*Property = Iframe*/
const Value V4459_316_57267 = {S0, NULL};	/*Value = False*/
const Property P4459_316 = {S34, 0, &V4459_316_57267, &P4459_315};	/*Property = IndexedDB*/
const Value V4459_317_57269 = {S2, NULL};	/*Value = True*/
const Property P4459_317 = {S35, 0, &V4459_317_57269, &P4459_316};	/*Property = Json*/
const Value V4459_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P4459_15 = {S37, 0, &V4459_15_28, &P4459_317};	/*Property = LayoutEngine*/
const Value V4459_344_57272 = {S2, NULL};	/*Value = True*/
const Property P4459_344 = {S38, 0, &V4459_344_57272, &P4459_15};	/*Property = Masking*/
const Value V4459_320_57290 = {S2, NULL};	/*Value = True*/
const Property P4459_320 = {S39, 0, &V4459_320_57290, &P4459_344};	/*Property = PostMessage*/
const Value V4459_321_57293 = {S2, NULL};	/*Value = True*/
const Property P4459_321 = {S40, 0, &V4459_321_57293, &P4459_320};	/*Property = Progress*/
const Value V4459_322_57296 = {S2, NULL};	/*Value = True*/
const Property P4459_322 = {S41, 0, &V4459_322_57296, &P4459_321};	/*Property = Prompts*/
const Value V4459_323_57299 = {S2, NULL};	/*Value = True*/
const Property P4459_323 = {S42, 0, &V4459_323_57299, &P4459_322};	/*Property = Selector*/
const Value V4459_324_57306 = {S0, NULL};	/*Value = False*/
const Property P4459_324 = {S43, 0, &V4459_324_57306, &P4459_323};	/*Property = Svg*/
const Value V4459_327_57315 = {S0, NULL};	/*Value = False*/
const Property P4459_327 = {S44, 0, &V4459_327_57315, &P4459_324};	/*Property = TouchEvents*/
const Value V4459_328_57318 = {S0, NULL};	/*Value = False*/
const Property P4459_328 = {S45, 0, &V4459_328_57318, &P4459_327};	/*Property = Track*/
const Value V4459_329_57323 = {S2, NULL};	/*Value = True*/
const Property P4459_329 = {S46, 0, &V4459_329_57323, &P4459_328};	/*Property = Video*/
const Value V4459_330_57326 = {S2, NULL};	/*Value = True*/
const Property P4459_330 = {S47, 0, &V4459_330_57326, &P4459_329};	/*Property = Viewport*/
const Value V4459_332_57339 = {S0, NULL};	/*Value = False*/
const Property P4459_332 = {S48, 0, &V4459_332_57339, &P4459_330};	/*Property = WebWorkers*/
const Value V4459_333_57342 = {S0, NULL};	/*Value = False*/
const Property P4459_333 = {S49, 0, &V4459_333_57342, &P4459_332};	/*Property = Xhr2*/
const Profile P4459 = {"4459", &P4459_333};	/*Profile ID - 4459*/
const Profile P6546 = {"6546", NULL};	/*Profile ID - 6546*/
const Profile P6437 = {"6437", NULL};	/*Profile ID - 6437*/
const Value V16179_279_57148 = {S0, NULL};	/*Value = False*/
const Property P16179_279 = {S1, 0, &V16179_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V16179_282_57154 = {S0, NULL};	/*Value = False*/
const Property P16179_282 = {S3, 0, &V16179_282_57154, &P16179_279};	/*Property = BlobBuilder*/
const Value V16179_350_57444 = {S2, NULL};	/*Value = True*/
const Property P16179_350 = {S4, 0, &V16179_350_57444, &P16179_282};	/*Property = Canvas*/
const Value V16179_283_57159 = {S2, NULL};	/*Value = True*/
const Property P16179_283 = {S5, 0, &V16179_283_57159, &P16179_350};	/*Property = CssBackground*/
const Value V16179_284_57163 = {S0, NULL};	/*Value = False*/
const Property P16179_284 = {S6, 0, &V16179_284_57163, &P16179_283};	/*Property = CssBorderImage*/
const Value V16179_285_57166 = {S0, NULL};	/*Value = False*/
const Property P16179_285 = {S7, 0, &V16179_285_57166, &P16179_284};	/*Property = CssCanvas*/
const Value V16179_286_57168 = {S2, NULL};	/*Value = True*/
const Property P16179_286 = {S8, 0, &V16179_286_57168, &P16179_285};	/*Property = CssColor*/
const Value V16179_237_55784 = {S2, NULL};	/*Value = True*/
const Property P16179_237 = {S9, 0, &V16179_237_55784, &P16179_286};	/*Property = CssColumn*/
const Value V16179_287_57172 = {S0, NULL};	/*Value = False*/
const Property P16179_287 = {S10, 0, &V16179_287_57172, &P16179_237};	/*Property = CssFlexbox*/
const Value V16179_288_57175 = {S0, NULL};	/*Value = False*/
const Property P16179_288 = {S11, 0, &V16179_288_57175, &P16179_287};	/*Property = CssFont*/
const Value V16179_289_57759 = {S0, NULL};	/*Value = False*/
const Property P16179_289 = {S12, 0, &V16179_289_57759, &P16179_288};	/*Property = CssImages*/
const Value V16179_291_57180 = {S2, NULL};	/*Value = True*/
const Property P16179_291 = {S13, 0, &V16179_291_57180, &P16179_289};	/*Property = CssMediaQueries*/
const Value V16179_292_57183 = {S2, NULL};	/*Value = True*/
const Property P16179_292 = {S14, 0, &V16179_292_57183, &P16179_291};	/*Property = CssMinMax*/
const Value V16179_293_57187 = {S0, NULL};	/*Value = False*/
const Property P16179_293 = {S15, 0, &V16179_293_57187, &P16179_292};	/*Property = CssOverflow*/
const Value V16179_294_57190 = {S0, NULL};	/*Value = False*/
const Property P16179_294 = {S16, 0, &V16179_294_57190, &P16179_293};	/*Property = CssPosition*/
const Value V16179_295_57192 = {S2, NULL};	/*Value = True*/
const Property P16179_295 = {S17, 0, &V16179_295_57192, &P16179_294};	/*Property = CssText*/
const Value V16179_297_57195 = {S2, NULL};	/*Value = True*/
const Property P16179_297 = {S18, 0, &V16179_297_57195, &P16179_295};	/*Property = CssTransforms*/
const Value V16179_241_55799 = {S2, NULL};	/*Value = True*/
const Property P16179_241 = {S19, 0, &V16179_241_55799, &P16179_297};	/*Property = CssTransitions*/
const Value V16179_298_57206 = {S2, NULL};	/*Value = True*/
const Property P16179_298 = {S20, 0, &V16179_298_57206, &P16179_241};	/*Property = CssUI*/
const Value V16179_300_57216 = {S0, NULL};	/*Value = False*/
const Property P16179_300 = {S21, 0, &V16179_300_57216, &P16179_298};	/*Property = DataSet*/
const Value V16179_301_57218 = {S2, NULL};	/*Value = True*/
const Property P16179_301 = {S22, 0, &V16179_301_57218, &P16179_300};	/*Property = DataUrl*/
const Value V16179_302_57221 = {S2, NULL};	/*Value = True*/
const Property P16179_302 = {S23, 0, &V16179_302_57221, &P16179_301};	/*Property = DeviceOrientation*/
const Value V16179_304_57228 = {S0, NULL};	/*Value = False*/
const Property P16179_304 = {S24, 0, &V16179_304_57228, &P16179_302};	/*Property = FileReader*/
const Value V16179_305_57231 = {S0, NULL};	/*Value = False*/
const Property P16179_305 = {S25, 0, &V16179_305_57231, &P16179_304};	/*Property = FileSaver*/
const Value V16179_306_57237 = {S0, NULL};	/*Value = False*/
const Property P16179_306 = {S26, 0, &V16179_306_57237, &P16179_305};	/*Property = FileWriter*/
const Value V16179_307_57239 = {S2, NULL};	/*Value = True*/
const Property P16179_307 = {S27, 0, &V16179_307_57239, &P16179_306};	/*Property = FormData*/
const Value V16179_309_57246 = {S0, NULL};	/*Value = False*/
const Property P16179_309 = {S28, 0, &V16179_309_57246, &P16179_307};	/*Property = Fullscreen*/
const Value V16179_310_57248 = {S2, NULL};	/*Value = True*/
const Property P16179_310 = {S29, 0, &V16179_310_57248, &P16179_309};	/*Property = GeoLocation*/
const Value V16179_312_57255 = {S0, NULL};	/*Value = False*/
const Property P16179_312 = {S30, 0, &V16179_312_57255, &P16179_310};	/*Property = History*/
const Value V16179_314_57261 = {S0, NULL};	/*Value = False*/
const Property P16179_314 = {S31, 0, &V16179_314_57261, &P16179_312};	/*Property = Html5*/
const Value V16179_313_57258 = {S0, NULL};	/*Value = False*/
const Property P16179_313 = {S32, 0, &V16179_313_57258, &P16179_314};	/*Property = Html-Media-Capture*/
const Value V16179_315_57264 = {S0, NULL};	/*Value = False*/
const Property P16179_315 = {S33, 0, &V16179_315_57264, &P16179_313};	/*Property = Iframe*/
const Value V16179_316_57267 = {S0, NULL};	/*Value = False*/
const Property P16179_316 = {S34, 0, &V16179_316_57267, &P16179_315};	/*Property = IndexedDB*/
const Value V16179_317_57269 = {S2, NULL};	/*Value = True*/
const Property P16179_317 = {S35, 0, &V16179_317_57269, &P16179_316};	/*Property = Json*/
const Value V16179_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P16179_15 = {S37, 0, &V16179_15_28, &P16179_317};	/*Property = LayoutEngine*/
const Value V16179_344_57272 = {S2, NULL};	/*Value = True*/
const Property P16179_344 = {S38, 0, &V16179_344_57272, &P16179_15};	/*Property = Masking*/
const Value V16179_320_57290 = {S2, NULL};	/*Value = True*/
const Property P16179_320 = {S39, 0, &V16179_320_57290, &P16179_344};	/*Property = PostMessage*/
const Value V16179_321_57293 = {S2, NULL};	/*Value = True*/
const Property P16179_321 = {S40, 0, &V16179_321_57293, &P16179_320};	/*Property = Progress*/
const Value V16179_322_57296 = {S2, NULL};	/*Value = True*/
const Property P16179_322 = {S41, 0, &V16179_322_57296, &P16179_321};	/*Property = Prompts*/
const Value V16179_323_57299 = {S2, NULL};	/*Value = True*/
const Property P16179_323 = {S42, 0, &V16179_323_57299, &P16179_322};	/*Property = Selector*/
const Value V16179_324_57306 = {S0, NULL};	/*Value = False*/
const Property P16179_324 = {S43, 0, &V16179_324_57306, &P16179_323};	/*Property = Svg*/
const Value V16179_327_57315 = {S0, NULL};	/*Value = False*/
const Property P16179_327 = {S44, 0, &V16179_327_57315, &P16179_324};	/*Property = TouchEvents*/
const Value V16179_328_57318 = {S0, NULL};	/*Value = False*/
const Property P16179_328 = {S45, 0, &V16179_328_57318, &P16179_327};	/*Property = Track*/
const Value V16179_329_57323 = {S2, NULL};	/*Value = True*/
const Property P16179_329 = {S46, 0, &V16179_329_57323, &P16179_328};	/*Property = Video*/
const Value V16179_330_57326 = {S2, NULL};	/*Value = True*/
const Property P16179_330 = {S47, 0, &V16179_330_57326, &P16179_329};	/*Property = Viewport*/
const Value V16179_332_57339 = {S0, NULL};	/*Value = False*/
const Property P16179_332 = {S48, 0, &V16179_332_57339, &P16179_330};	/*Property = WebWorkers*/
const Value V16179_333_57342 = {S0, NULL};	/*Value = False*/
const Property P16179_333 = {S49, 0, &V16179_333_57342, &P16179_332};	/*Property = Xhr2*/
const Profile P16179 = {"16179", &P16179_333};	/*Profile ID - 16179*/
const Profile P16110 = {"16110", NULL};	/*Profile ID - 16110*/
const Value V17207_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17207_279 = {S1, 0, &V17207_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17207_282_57153 = {S2, NULL};	/*Value = True*/
const Property P17207_282 = {S3, 0, &V17207_282_57153, &P17207_279};	/*Property = BlobBuilder*/
const Value V17207_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17207_350 = {S4, 0, &V17207_350_57444, &P17207_282};	/*Property = Canvas*/
const Value V17207_283_57160 = {S0, NULL};	/*Value = False*/
const Property P17207_283 = {S5, 0, &V17207_283_57160, &P17207_350};	/*Property = CssBackground*/
const Value V17207_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17207_284 = {S6, 0, &V17207_284_57163, &P17207_283};	/*Property = CssBorderImage*/
const Value V17207_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17207_285 = {S7, 0, &V17207_285_57166, &P17207_284};	/*Property = CssCanvas*/
const Value V17207_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17207_286 = {S8, 0, &V17207_286_57168, &P17207_285};	/*Property = CssColor*/
const Value V17207_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17207_237 = {S9, 0, &V17207_237_55784, &P17207_286};	/*Property = CssColumn*/
const Value V17207_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17207_287 = {S10, 0, &V17207_287_57172, &P17207_237};	/*Property = CssFlexbox*/
const Value V17207_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17207_288 = {S11, 0, &V17207_288_57175, &P17207_287};	/*Property = CssFont*/
const Value V17207_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17207_289 = {S12, 0, &V17207_289_86899, &P17207_288};	/*Property = CssImages*/
const Value V17207_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17207_291 = {S13, 0, &V17207_291_57181, &P17207_289};	/*Property = CssMediaQueries*/
const Value V17207_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17207_292 = {S14, 0, &V17207_292_57183, &P17207_291};	/*Property = CssMinMax*/
const Value V17207_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17207_293 = {S15, 0, &V17207_293_57187, &P17207_292};	/*Property = CssOverflow*/
const Value V17207_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17207_294 = {S16, 0, &V17207_294_57189, &P17207_293};	/*Property = CssPosition*/
const Value V17207_295_57193 = {S0, NULL};	/*Value = False*/
const Property P17207_295 = {S17, 0, &V17207_295_57193, &P17207_294};	/*Property = CssText*/
const Value V17207_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17207_297 = {S18, 0, &V17207_297_57195, &P17207_295};	/*Property = CssTransforms*/
const Value V17207_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17207_241 = {S19, 0, &V17207_241_55799, &P17207_297};	/*Property = CssTransitions*/
const Value V17207_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17207_298 = {S20, 0, &V17207_298_57206, &P17207_241};	/*Property = CssUI*/
const Value V17207_300_57215 = {S2, NULL};	/*Value = True*/
const Property P17207_300 = {S21, 0, &V17207_300_57215, &P17207_298};	/*Property = DataSet*/
const Value V17207_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17207_301 = {S22, 0, &V17207_301_57218, &P17207_300};	/*Property = DataUrl*/
const Value V17207_302_57221 = {S2, NULL};	/*Value = True*/
const Property P17207_302 = {S23, 0, &V17207_302_57221, &P17207_301};	/*Property = DeviceOrientation*/
const Value V17207_304_57227 = {S2, NULL};	/*Value = True*/
const Property P17207_304 = {S24, 0, &V17207_304_57227, &P17207_302};	/*Property = FileReader*/
const Value V17207_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17207_305 = {S25, 0, &V17207_305_57231, &P17207_304};	/*Property = FileSaver*/
const Value V17207_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17207_306 = {S26, 0, &V17207_306_57237, &P17207_305};	/*Property = FileWriter*/
const Value V17207_307_57239 = {S2, NULL};	/*Value = True*/
const Property P17207_307 = {S27, 0, &V17207_307_57239, &P17207_306};	/*Property = FormData*/
const Value V17207_309_57245 = {S2, NULL};	/*Value = True*/
const Property P17207_309 = {S28, 0, &V17207_309_57245, &P17207_307};	/*Property = Fullscreen*/
const Value V17207_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17207_310 = {S29, 0, &V17207_310_57248, &P17207_309};	/*Property = GeoLocation*/
const Value V17207_312_57254 = {S2, NULL};	/*Value = True*/
const Property P17207_312 = {S30, 0, &V17207_312_57254, &P17207_310};	/*Property = History*/
const Value V17207_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17207_314 = {S31, 0, &V17207_314_57261, &P17207_312};	/*Property = Html5*/
const Value V17207_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17207_313 = {S32, 0, &V17207_313_57258, &P17207_314};	/*Property = Html-Media-Capture*/
const Value V17207_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17207_315 = {S33, 0, &V17207_315_57264, &P17207_313};	/*Property = Iframe*/
const Value V17207_316_57266 = {S2, NULL};	/*Value = True*/
const Property P17207_316 = {S34, 0, &V17207_316_57266, &P17207_315};	/*Property = IndexedDB*/
const Value V17207_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17207_317 = {S35, 0, &V17207_317_57269, &P17207_316};	/*Property = Json*/
const Value V17207_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P17207_15 = {S37, 0, &V17207_15_29, &P17207_317};	/*Property = LayoutEngine*/
const Value V17207_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17207_344 = {S38, 0, &V17207_344_57272, &P17207_15};	/*Property = Masking*/
const Value V17207_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17207_320 = {S39, 0, &V17207_320_57290, &P17207_344};	/*Property = PostMessage*/
const Value V17207_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17207_321 = {S40, 0, &V17207_321_57293, &P17207_320};	/*Property = Progress*/
const Value V17207_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17207_322 = {S41, 0, &V17207_322_57296, &P17207_321};	/*Property = Prompts*/
const Value V17207_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17207_323 = {S42, 0, &V17207_323_57299, &P17207_322};	/*Property = Selector*/
const Value V17207_324_57305 = {S2, NULL};	/*Value = True*/
const Property P17207_324 = {S43, 0, &V17207_324_57305, &P17207_323};	/*Property = Svg*/
const Value V17207_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17207_327 = {S44, 0, &V17207_327_57315, &P17207_324};	/*Property = TouchEvents*/
const Value V17207_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17207_328 = {S45, 0, &V17207_328_57318, &P17207_327};	/*Property = Track*/
const Value V17207_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17207_329 = {S46, 0, &V17207_329_57323, &P17207_328};	/*Property = Video*/
const Value V17207_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17207_330 = {S47, 0, &V17207_330_57326, &P17207_329};	/*Property = Viewport*/
const Value V17207_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17207_332 = {S48, 0, &V17207_332_57339, &P17207_330};	/*Property = WebWorkers*/
const Value V17207_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17207_333 = {S49, 0, &V17207_333_57342, &P17207_332};	/*Property = Xhr2*/
const Profile P17207 = {"17207", &P17207_333};	/*Profile ID - 17207*/
const Value V12780_1_1 = {S2, NULL};	/*Value = True*/
const Property P12780_1 = {S50, 0, &V12780_1_1, NULL};	/*Property = IsMobile*/
const Value V12780_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12780_5 = {S52, 0, &V12780_5_1222, &P12780_1};	/*Property = ScreenPixelsHeight*/
const Value V12780_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12780_4 = {S54, 0, &V12780_4_1224, &P12780_5};	/*Property = ScreenPixelsWidth*/
const Profile P12780 = {"12780", &P12780_4};	/*Profile ID - 12780*/
const Value V12667_1_1 = {S2, NULL};	/*Value = True*/
const Property P12667_1 = {S50, 0, &V12667_1_1, NULL};	/*Property = IsMobile*/
const Value V12667_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12667_5 = {S52, 0, &V12667_5_1293, &P12667_1};	/*Property = ScreenPixelsHeight*/
const Value V12667_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12667_4 = {S54, 0, &V12667_4_1296, &P12667_5};	/*Property = ScreenPixelsWidth*/
const Profile P12667 = {"12667", &P12667_4};	/*Profile ID - 12667*/
const Value V12783_1_1 = {S2, NULL};	/*Value = True*/
const Property P12783_1 = {S50, 0, &V12783_1_1, NULL};	/*Property = IsMobile*/
const Value V12783_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12783_5 = {S52, 0, &V12783_5_1380, &P12783_1};	/*Property = ScreenPixelsHeight*/
const Value V12783_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12783_4 = {S54, 0, &V12783_4_1279, &P12783_5};	/*Property = ScreenPixelsWidth*/
const Profile P12783 = {"12783", &P12783_4};	/*Profile ID - 12783*/
const Value V13923_1_1 = {S2, NULL};	/*Value = True*/
const Property P13923_1 = {S50, 0, &V13923_1_1, NULL};	/*Property = IsMobile*/
const Value V13923_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13923_5 = {S52, 0, &V13923_5_1293, &P13923_1};	/*Property = ScreenPixelsHeight*/
const Value V13923_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13923_4 = {S54, 0, &V13923_4_1296, &P13923_5};	/*Property = ScreenPixelsWidth*/
const Profile P13923 = {"13923", &P13923_4};	/*Profile ID - 13923*/
const Value V12784_1_1 = {S2, NULL};	/*Value = True*/
const Property P12784_1 = {S50, 0, &V12784_1_1, NULL};	/*Property = IsMobile*/
const Value V12784_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12784_5 = {S52, 0, &V12784_5_1227, &P12784_1};	/*Property = ScreenPixelsHeight*/
const Value V12784_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12784_4 = {S54, 0, &V12784_4_1229, &P12784_5};	/*Property = ScreenPixelsWidth*/
const Profile P12784 = {"12784", &P12784_4};	/*Profile ID - 12784*/
const Value V12785_1_1 = {S2, NULL};	/*Value = True*/
const Property P12785_1 = {S50, 0, &V12785_1_1, NULL};	/*Property = IsMobile*/
const Value V12785_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12785_5 = {S52, 0, &V12785_5_1222, &P12785_1};	/*Property = ScreenPixelsHeight*/
const Value V12785_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12785_4 = {S54, 0, &V12785_4_1224, &P12785_5};	/*Property = ScreenPixelsWidth*/
const Profile P12785 = {"12785", &P12785_4};	/*Profile ID - 12785*/
const Value V12786_1_1 = {S2, NULL};	/*Value = True*/
const Property P12786_1 = {S50, 0, &V12786_1_1, NULL};	/*Property = IsMobile*/
const Value V12786_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12786_5 = {S52, 0, &V12786_5_1233, &P12786_1};	/*Property = ScreenPixelsHeight*/
const Value V12786_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12786_4 = {S54, 0, &V12786_4_1235, &P12786_5};	/*Property = ScreenPixelsWidth*/
const Profile P12786 = {"12786", &P12786_4};	/*Profile ID - 12786*/
const Value V17320_1_1 = {S2, NULL};	/*Value = True*/
const Property P17320_1 = {S50, 0, &V17320_1_1, NULL};	/*Property = IsMobile*/
const Value V17320_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17320_5 = {S52, 0, &V17320_5_1277, &P17320_1};	/*Property = ScreenPixelsHeight*/
const Value V17320_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17320_4 = {S54, 0, &V17320_4_1279, &P17320_5};	/*Property = ScreenPixelsWidth*/
const Profile P17320 = {"17320", &P17320_4};	/*Profile ID - 17320*/
const Value V22838_1_1 = {S2, NULL};	/*Value = True*/
const Property P22838_1 = {S50, 0, &V22838_1_1, NULL};	/*Property = IsMobile*/
const Value V22838_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22838_5 = {S52, 0, &V22838_5_1222, &P22838_1};	/*Property = ScreenPixelsHeight*/
const Value V22838_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22838_4 = {S54, 0, &V22838_4_1224, &P22838_5};	/*Property = ScreenPixelsWidth*/
const Profile P22838 = {"22838", &P22838_4};	/*Profile ID - 22838*/
const Value V12789_1_1 = {S2, NULL};	/*Value = True*/
const Property P12789_1 = {S50, 0, &V12789_1_1, NULL};	/*Property = IsMobile*/
const Value V12789_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12789_5 = {S52, 0, &V12789_5_1293, &P12789_1};	/*Property = ScreenPixelsHeight*/
const Value V12789_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12789_4 = {S54, 0, &V12789_4_1296, &P12789_5};	/*Property = ScreenPixelsWidth*/
const Profile P12789 = {"12789", &P12789_4};	/*Profile ID - 12789*/
const Value V14768_1_1 = {S2, NULL};	/*Value = True*/
const Property P14768_1 = {S50, 0, &V14768_1_1, NULL};	/*Property = IsMobile*/
const Value V14768_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14768_5 = {S52, 0, &V14768_5_1222, &P14768_1};	/*Property = ScreenPixelsHeight*/
const Value V14768_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14768_4 = {S54, 0, &V14768_4_1224, &P14768_5};	/*Property = ScreenPixelsWidth*/
const Profile P14768 = {"14768", &P14768_4};	/*Profile ID - 14768*/
const Value V14125_1_1 = {S2, NULL};	/*Value = True*/
const Property P14125_1 = {S50, 0, &V14125_1_1, NULL};	/*Property = IsMobile*/
const Value V14125_5_31826 = {S75, NULL};	/*Value = 260*/
const Property P14125_5 = {S52, 0, &V14125_5_31826, &P14125_1};	/*Property = ScreenPixelsHeight*/
const Value V14125_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14125_4 = {S54, 0, &V14125_4_1224, &P14125_5};	/*Property = ScreenPixelsWidth*/
const Profile P14125 = {"14125", &P14125_4};	/*Profile ID - 14125*/
const Value V12791_1_1 = {S2, NULL};	/*Value = True*/
const Property P12791_1 = {S50, 0, &V12791_1_1, NULL};	/*Property = IsMobile*/
const Value V12791_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12791_5 = {S52, 0, &V12791_5_1222, &P12791_1};	/*Property = ScreenPixelsHeight*/
const Value V12791_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12791_4 = {S54, 0, &V12791_4_1224, &P12791_5};	/*Property = ScreenPixelsWidth*/
const Profile P12791 = {"12791", &P12791_4};	/*Profile ID - 12791*/
const Value V12793_1_1 = {S2, NULL};	/*Value = True*/
const Property P12793_1 = {S50, 0, &V12793_1_1, NULL};	/*Property = IsMobile*/
const Value V12793_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12793_5 = {S52, 0, &V12793_5_1233, &P12793_1};	/*Property = ScreenPixelsHeight*/
const Value V12793_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12793_4 = {S54, 0, &V12793_4_1235, &P12793_5};	/*Property = ScreenPixelsWidth*/
const Profile P12793 = {"12793", &P12793_4};	/*Profile ID - 12793*/
const Value V12794_1_1 = {S2, NULL};	/*Value = True*/
const Property P12794_1 = {S50, 0, &V12794_1_1, NULL};	/*Property = IsMobile*/
const Value V12794_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12794_5 = {S52, 0, &V12794_5_1227, &P12794_1};	/*Property = ScreenPixelsHeight*/
const Value V12794_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P12794_4 = {S54, 0, &V12794_4_1460, &P12794_5};	/*Property = ScreenPixelsWidth*/
const Profile P12794 = {"12794", &P12794_4};	/*Profile ID - 12794*/
const Value V12799_1_1 = {S2, NULL};	/*Value = True*/
const Property P12799_1 = {S50, 0, &V12799_1_1, NULL};	/*Property = IsMobile*/
const Value V12799_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12799_5 = {S52, 0, &V12799_5_1233, &P12799_1};	/*Property = ScreenPixelsHeight*/
const Value V12799_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12799_4 = {S54, 0, &V12799_4_1235, &P12799_5};	/*Property = ScreenPixelsWidth*/
const Profile P12799 = {"12799", &P12799_4};	/*Profile ID - 12799*/
const Value V12797_1_1 = {S2, NULL};	/*Value = True*/
const Property P12797_1 = {S50, 0, &V12797_1_1, NULL};	/*Property = IsMobile*/
const Value V12797_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12797_5 = {S52, 0, &V12797_5_1277, &P12797_1};	/*Property = ScreenPixelsHeight*/
const Value V12797_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12797_4 = {S54, 0, &V12797_4_1279, &P12797_5};	/*Property = ScreenPixelsWidth*/
const Profile P12797 = {"12797", &P12797_4};	/*Profile ID - 12797*/
const Value V15397_1_1 = {S2, NULL};	/*Value = True*/
const Property P15397_1 = {S50, 0, &V15397_1_1, NULL};	/*Property = IsMobile*/
const Value V15397_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P15397_5 = {S52, 0, &V15397_5_1269, &P15397_1};	/*Property = ScreenPixelsHeight*/
const Value V15397_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15397_4 = {S54, 0, &V15397_4_1229, &P15397_5};	/*Property = ScreenPixelsWidth*/
const Profile P15397 = {"15397", &P15397_4};	/*Profile ID - 15397*/
const Value V12801_1_1 = {S2, NULL};	/*Value = True*/
const Property P12801_1 = {S50, 0, &V12801_1_1, NULL};	/*Property = IsMobile*/
const Value V12801_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12801_5 = {S52, 0, &V12801_5_1277, &P12801_1};	/*Property = ScreenPixelsHeight*/
const Value V12801_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12801_4 = {S54, 0, &V12801_4_1279, &P12801_5};	/*Property = ScreenPixelsWidth*/
const Profile P12801 = {"12801", &P12801_4};	/*Profile ID - 12801*/
const Value V12804_1_1 = {S2, NULL};	/*Value = True*/
const Property P12804_1 = {S50, 0, &V12804_1_1, NULL};	/*Property = IsMobile*/
const Value V12804_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12804_5 = {S52, 0, &V12804_5_1222, &P12804_1};	/*Property = ScreenPixelsHeight*/
const Value V12804_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12804_4 = {S54, 0, &V12804_4_1224, &P12804_5};	/*Property = ScreenPixelsWidth*/
const Profile P12804 = {"12804", &P12804_4};	/*Profile ID - 12804*/
const Value V12805_1_1 = {S2, NULL};	/*Value = True*/
const Property P12805_1 = {S50, 0, &V12805_1_1, NULL};	/*Property = IsMobile*/
const Value V12805_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12805_5 = {S52, 0, &V12805_5_1304, &P12805_1};	/*Property = ScreenPixelsHeight*/
const Value V12805_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12805_4 = {S54, 0, &V12805_4_1224, &P12805_5};	/*Property = ScreenPixelsWidth*/
const Profile P12805 = {"12805", &P12805_4};	/*Profile ID - 12805*/
const Value V12809_1_1 = {S2, NULL};	/*Value = True*/
const Property P12809_1 = {S50, 0, &V12809_1_1, NULL};	/*Property = IsMobile*/
const Value V12809_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12809_5 = {S52, 0, &V12809_5_1293, &P12809_1};	/*Property = ScreenPixelsHeight*/
const Value V12809_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12809_4 = {S54, 0, &V12809_4_1296, &P12809_5};	/*Property = ScreenPixelsWidth*/
const Profile P12809 = {"12809", &P12809_4};	/*Profile ID - 12809*/
const Value V12812_1_1 = {S2, NULL};	/*Value = True*/
const Property P12812_1 = {S50, 0, &V12812_1_1, NULL};	/*Property = IsMobile*/
const Value V12812_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12812_5 = {S52, 0, &V12812_5_1294, &P12812_1};	/*Property = ScreenPixelsHeight*/
const Value V12812_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12812_4 = {S54, 0, &V12812_4_1296, &P12812_5};	/*Property = ScreenPixelsWidth*/
const Profile P12812 = {"12812", &P12812_4};	/*Profile ID - 12812*/
const Value V12810_1_1 = {S2, NULL};	/*Value = True*/
const Property P12810_1 = {S50, 0, &V12810_1_1, NULL};	/*Property = IsMobile*/
const Value V12810_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12810_5 = {S52, 0, &V12810_5_1277, &P12810_1};	/*Property = ScreenPixelsHeight*/
const Value V12810_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P12810_4 = {S54, 0, &V12810_4_2336, &P12810_5};	/*Property = ScreenPixelsWidth*/
const Profile P12810 = {"12810", &P12810_4};	/*Profile ID - 12810*/
const Value V12657_1_1 = {S2, NULL};	/*Value = True*/
const Property P12657_1 = {S50, 0, &V12657_1_1, NULL};	/*Property = IsMobile*/
const Value V12657_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P12657_5 = {S52, 0, &V12657_5_1403, &P12657_1};	/*Property = ScreenPixelsHeight*/
const Value V12657_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P12657_4 = {S54, 0, &V12657_4_1404, &P12657_5};	/*Property = ScreenPixelsWidth*/
const Profile P12657 = {"12657", &P12657_4};	/*Profile ID - 12657*/
const Value V16674_1_1 = {S2, NULL};	/*Value = True*/
const Property P16674_1 = {S50, 0, &V16674_1_1, NULL};	/*Property = IsMobile*/
const Value V16674_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P16674_5 = {S52, 0, &V16674_5_1304, &P16674_1};	/*Property = ScreenPixelsHeight*/
const Value V16674_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16674_4 = {S54, 0, &V16674_4_1224, &P16674_5};	/*Property = ScreenPixelsWidth*/
const Profile P16674 = {"16674", &P16674_4};	/*Profile ID - 16674*/
const Value V12814_1_1 = {S2, NULL};	/*Value = True*/
const Property P12814_1 = {S50, 0, &V12814_1_1, NULL};	/*Property = IsMobile*/
const Value V12814_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12814_5 = {S52, 0, &V12814_5_1227, &P12814_1};	/*Property = ScreenPixelsHeight*/
const Value V12814_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12814_4 = {S54, 0, &V12814_4_1229, &P12814_5};	/*Property = ScreenPixelsWidth*/
const Profile P12814 = {"12814", &P12814_4};	/*Profile ID - 12814*/
const Value V12815_1_1 = {S2, NULL};	/*Value = True*/
const Property P12815_1 = {S50, 0, &V12815_1_1, NULL};	/*Property = IsMobile*/
const Value V12815_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12815_5 = {S52, 0, &V12815_5_1233, &P12815_1};	/*Property = ScreenPixelsHeight*/
const Value V12815_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12815_4 = {S54, 0, &V12815_4_1235, &P12815_5};	/*Property = ScreenPixelsWidth*/
const Profile P12815 = {"12815", &P12815_4};	/*Profile ID - 12815*/
const Value V12817_1_1 = {S2, NULL};	/*Value = True*/
const Property P12817_1 = {S50, 0, &V12817_1_1, NULL};	/*Property = IsMobile*/
const Value V12817_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12817_5 = {S52, 0, &V12817_5_1304, &P12817_1};	/*Property = ScreenPixelsHeight*/
const Value V12817_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12817_4 = {S54, 0, &V12817_4_1224, &P12817_5};	/*Property = ScreenPixelsWidth*/
const Profile P12817 = {"12817", &P12817_4};	/*Profile ID - 12817*/
const Value V14574_1_1 = {S2, NULL};	/*Value = True*/
const Property P14574_1 = {S50, 0, &V14574_1_1, NULL};	/*Property = IsMobile*/
const Value V14574_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14574_5 = {S52, 0, &V14574_5_1227, &P14574_1};	/*Property = ScreenPixelsHeight*/
const Value V14574_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14574_4 = {S54, 0, &V14574_4_1229, &P14574_5};	/*Property = ScreenPixelsWidth*/
const Profile P14574 = {"14574", &P14574_4};	/*Profile ID - 14574*/
const Value V12819_1_1 = {S2, NULL};	/*Value = True*/
const Property P12819_1 = {S50, 0, &V12819_1_1, NULL};	/*Property = IsMobile*/
const Value V12819_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12819_5 = {S52, 0, &V12819_5_1304, &P12819_1};	/*Property = ScreenPixelsHeight*/
const Value V12819_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12819_4 = {S54, 0, &V12819_4_1224, &P12819_5};	/*Property = ScreenPixelsWidth*/
const Profile P12819 = {"12819", &P12819_4};	/*Profile ID - 12819*/
const Value V12821_1_1 = {S2, NULL};	/*Value = True*/
const Property P12821_1 = {S50, 0, &V12821_1_1, NULL};	/*Property = IsMobile*/
const Value V12821_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P12821_5 = {S52, 0, &V12821_5_1458, &P12821_1};	/*Property = ScreenPixelsHeight*/
const Value V12821_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P12821_4 = {S54, 0, &V12821_4_1460, &P12821_5};	/*Property = ScreenPixelsWidth*/
const Profile P12821 = {"12821", &P12821_4};	/*Profile ID - 12821*/
const Value V12823_1_1 = {S2, NULL};	/*Value = True*/
const Property P12823_1 = {S50, 0, &V12823_1_1, NULL};	/*Property = IsMobile*/
const Value V12823_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12823_5 = {S52, 0, &V12823_5_1293, &P12823_1};	/*Property = ScreenPixelsHeight*/
const Value V12823_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12823_4 = {S54, 0, &V12823_4_1296, &P12823_5};	/*Property = ScreenPixelsWidth*/
const Profile P12823 = {"12823", &P12823_4};	/*Profile ID - 12823*/
const Value V12822_1_1 = {S2, NULL};	/*Value = True*/
const Property P12822_1 = {S50, 0, &V12822_1_1, NULL};	/*Property = IsMobile*/
const Value V12822_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12822_5 = {S52, 0, &V12822_5_1222, &P12822_1};	/*Property = ScreenPixelsHeight*/
const Value V12822_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12822_4 = {S54, 0, &V12822_4_1224, &P12822_5};	/*Property = ScreenPixelsWidth*/
const Profile P12822 = {"12822", &P12822_4};	/*Profile ID - 12822*/
const Value V12824_1_1 = {S2, NULL};	/*Value = True*/
const Property P12824_1 = {S50, 0, &V12824_1_1, NULL};	/*Property = IsMobile*/
const Value V12824_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12824_5 = {S52, 0, &V12824_5_1222, &P12824_1};	/*Property = ScreenPixelsHeight*/
const Value V12824_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12824_4 = {S54, 0, &V12824_4_1224, &P12824_5};	/*Property = ScreenPixelsWidth*/
const Profile P12824 = {"12824", &P12824_4};	/*Profile ID - 12824*/
const Value V12826_1_1 = {S2, NULL};	/*Value = True*/
const Property P12826_1 = {S50, 0, &V12826_1_1, NULL};	/*Property = IsMobile*/
const Value V12826_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12826_5 = {S52, 0, &V12826_5_1294, &P12826_1};	/*Property = ScreenPixelsHeight*/
const Value V12826_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12826_4 = {S54, 0, &V12826_4_1296, &P12826_5};	/*Property = ScreenPixelsWidth*/
const Profile P12826 = {"12826", &P12826_4};	/*Profile ID - 12826*/
const Value V12827_1_1 = {S2, NULL};	/*Value = True*/
const Property P12827_1 = {S50, 0, &V12827_1_1, NULL};	/*Property = IsMobile*/
const Value V12827_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12827_5 = {S52, 0, &V12827_5_1233, &P12827_1};	/*Property = ScreenPixelsHeight*/
const Value V12827_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12827_4 = {S54, 0, &V12827_4_1235, &P12827_5};	/*Property = ScreenPixelsWidth*/
const Profile P12827 = {"12827", &P12827_4};	/*Profile ID - 12827*/
const Value V12830_1_1 = {S2, NULL};	/*Value = True*/
const Property P12830_1 = {S50, 0, &V12830_1_1, NULL};	/*Property = IsMobile*/
const Value V12830_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12830_5 = {S52, 0, &V12830_5_1380, &P12830_1};	/*Property = ScreenPixelsHeight*/
const Value V12830_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12830_4 = {S54, 0, &V12830_4_1279, &P12830_5};	/*Property = ScreenPixelsWidth*/
const Profile P12830 = {"12830", &P12830_4};	/*Profile ID - 12830*/
const Value V18009_1_1 = {S2, NULL};	/*Value = True*/
const Property P18009_1 = {S50, 0, &V18009_1_1, NULL};	/*Property = IsMobile*/
const Value V18009_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18009_5 = {S52, 0, &V18009_5_1403, &P18009_1};	/*Property = ScreenPixelsHeight*/
const Value V18009_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18009_4 = {S54, 0, &V18009_4_1404, &P18009_5};	/*Property = ScreenPixelsWidth*/
const Profile P18009 = {"18009", &P18009_4};	/*Profile ID - 18009*/
const Value V12836_1_1 = {S2, NULL};	/*Value = True*/
const Property P12836_1 = {S50, 0, &V12836_1_1, NULL};	/*Property = IsMobile*/
const Value V12836_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12836_5 = {S52, 0, &V12836_5_1227, &P12836_1};	/*Property = ScreenPixelsHeight*/
const Value V12836_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12836_4 = {S54, 0, &V12836_4_1229, &P12836_5};	/*Property = ScreenPixelsWidth*/
const Profile P12836 = {"12836", &P12836_4};	/*Profile ID - 12836*/
const Value V12837_1_1 = {S2, NULL};	/*Value = True*/
const Property P12837_1 = {S50, 0, &V12837_1_1, NULL};	/*Property = IsMobile*/
const Value V12837_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12837_5 = {S52, 0, &V12837_5_1233, &P12837_1};	/*Property = ScreenPixelsHeight*/
const Value V12837_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12837_4 = {S54, 0, &V12837_4_1235, &P12837_5};	/*Property = ScreenPixelsWidth*/
const Profile P12837 = {"12837", &P12837_4};	/*Profile ID - 12837*/
const Value V12838_1_1 = {S2, NULL};	/*Value = True*/
const Property P12838_1 = {S50, 0, &V12838_1_1, NULL};	/*Property = IsMobile*/
const Value V12838_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12838_5 = {S52, 0, &V12838_5_1293, &P12838_1};	/*Property = ScreenPixelsHeight*/
const Value V12838_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12838_4 = {S54, 0, &V12838_4_1296, &P12838_5};	/*Property = ScreenPixelsWidth*/
const Profile P12838 = {"12838", &P12838_4};	/*Profile ID - 12838*/
const Value V12842_1_1 = {S2, NULL};	/*Value = True*/
const Property P12842_1 = {S50, 0, &V12842_1_1, NULL};	/*Property = IsMobile*/
const Value V12842_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12842_5 = {S52, 0, &V12842_5_1293, &P12842_1};	/*Property = ScreenPixelsHeight*/
const Value V12842_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12842_4 = {S54, 0, &V12842_4_1296, &P12842_5};	/*Property = ScreenPixelsWidth*/
const Profile P12842 = {"12842", &P12842_4};	/*Profile ID - 12842*/
const Value V12321_1_1 = {S2, NULL};	/*Value = True*/
const Property P12321_1 = {S50, 0, &V12321_1_1, NULL};	/*Property = IsMobile*/
const Value V12321_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12321_5 = {S52, 0, &V12321_5_1380, &P12321_1};	/*Property = ScreenPixelsHeight*/
const Value V12321_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12321_4 = {S54, 0, &V12321_4_1279, &P12321_5};	/*Property = ScreenPixelsWidth*/
const Profile P12321 = {"12321", &P12321_4};	/*Profile ID - 12321*/
const Value V12843_1_1 = {S2, NULL};	/*Value = True*/
const Property P12843_1 = {S50, 0, &V12843_1_1, NULL};	/*Property = IsMobile*/
const Value V12843_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12843_5 = {S52, 0, &V12843_5_1222, &P12843_1};	/*Property = ScreenPixelsHeight*/
const Value V12843_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12843_4 = {S54, 0, &V12843_4_1229, &P12843_5};	/*Property = ScreenPixelsWidth*/
const Profile P12843 = {"12843", &P12843_4};	/*Profile ID - 12843*/
const Value V16105_1_1 = {S2, NULL};	/*Value = True*/
const Property P16105_1 = {S50, 0, &V16105_1_1, NULL};	/*Property = IsMobile*/
const Value V16105_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16105_5 = {S52, 0, &V16105_5_1294, &P16105_1};	/*Property = ScreenPixelsHeight*/
const Value V16105_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16105_4 = {S54, 0, &V16105_4_1296, &P16105_5};	/*Property = ScreenPixelsWidth*/
const Profile P16105 = {"16105", &P16105_4};	/*Profile ID - 16105*/
const Value V13144_1_1 = {S2, NULL};	/*Value = True*/
const Property P13144_1 = {S50, 0, &V13144_1_1, NULL};	/*Property = IsMobile*/
const Value V13144_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13144_5 = {S52, 0, &V13144_5_1227, &P13144_1};	/*Property = ScreenPixelsHeight*/
const Value V13144_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13144_4 = {S54, 0, &V13144_4_1229, &P13144_5};	/*Property = ScreenPixelsWidth*/
const Profile P13144 = {"13144", &P13144_4};	/*Profile ID - 13144*/
const Value V12846_1_1 = {S2, NULL};	/*Value = True*/
const Property P12846_1 = {S50, 0, &V12846_1_1, NULL};	/*Property = IsMobile*/
const Value V12846_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12846_5 = {S52, 0, &V12846_5_1380, &P12846_1};	/*Property = ScreenPixelsHeight*/
const Value V12846_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12846_4 = {S54, 0, &V12846_4_1279, &P12846_5};	/*Property = ScreenPixelsWidth*/
const Profile P12846 = {"12846", &P12846_4};	/*Profile ID - 12846*/
const Value V12847_1_1 = {S2, NULL};	/*Value = True*/
const Property P12847_1 = {S50, 0, &V12847_1_1, NULL};	/*Property = IsMobile*/
const Value V12847_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12847_5 = {S52, 0, &V12847_5_1277, &P12847_1};	/*Property = ScreenPixelsHeight*/
const Value V12847_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12847_4 = {S54, 0, &V12847_4_1279, &P12847_5};	/*Property = ScreenPixelsWidth*/
const Profile P12847 = {"12847", &P12847_4};	/*Profile ID - 12847*/
const Value V12848_1_1 = {S2, NULL};	/*Value = True*/
const Property P12848_1 = {S50, 0, &V12848_1_1, NULL};	/*Property = IsMobile*/
const Value V12848_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P12848_5 = {S52, 0, &V12848_5_1269, &P12848_1};	/*Property = ScreenPixelsHeight*/
const Value V12848_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12848_4 = {S54, 0, &V12848_4_1229, &P12848_5};	/*Property = ScreenPixelsWidth*/
const Profile P12848 = {"12848", &P12848_4};	/*Profile ID - 12848*/
const Value V15865_1_1 = {S2, NULL};	/*Value = True*/
const Property P15865_1 = {S50, 0, &V15865_1_1, NULL};	/*Property = IsMobile*/
const Value V15865_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15865_5 = {S52, 0, &V15865_5_1222, &P15865_1};	/*Property = ScreenPixelsHeight*/
const Value V15865_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15865_4 = {S54, 0, &V15865_4_1229, &P15865_5};	/*Property = ScreenPixelsWidth*/
const Profile P15865 = {"15865", &P15865_4};	/*Profile ID - 15865*/
const Value V12853_1_1 = {S2, NULL};	/*Value = True*/
const Property P12853_1 = {S50, 0, &V12853_1_1, NULL};	/*Property = IsMobile*/
const Value V12853_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12853_5 = {S52, 0, &V12853_5_1222, &P12853_1};	/*Property = ScreenPixelsHeight*/
const Value V12853_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12853_4 = {S54, 0, &V12853_4_1229, &P12853_5};	/*Property = ScreenPixelsWidth*/
const Profile P12853 = {"12853", &P12853_4};	/*Profile ID - 12853*/
const Value V12854_1_1 = {S2, NULL};	/*Value = True*/
const Property P12854_1 = {S50, 0, &V12854_1_1, NULL};	/*Property = IsMobile*/
const Value V12854_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12854_5 = {S52, 0, &V12854_5_1277, &P12854_1};	/*Property = ScreenPixelsHeight*/
const Value V12854_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12854_4 = {S54, 0, &V12854_4_1279, &P12854_5};	/*Property = ScreenPixelsWidth*/
const Profile P12854 = {"12854", &P12854_4};	/*Profile ID - 12854*/
const Value V11895_1_1 = {S2, NULL};	/*Value = True*/
const Property P11895_1 = {S50, 0, &V11895_1_1, NULL};	/*Property = IsMobile*/
const Value V11895_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P11895_5 = {S52, 0, &V11895_5_1294, &P11895_1};	/*Property = ScreenPixelsHeight*/
const Value V11895_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11895_4 = {S54, 0, &V11895_4_1296, &P11895_5};	/*Property = ScreenPixelsWidth*/
const Profile P11895 = {"11895", &P11895_4};	/*Profile ID - 11895*/
const Value V12855_1_1 = {S2, NULL};	/*Value = True*/
const Property P12855_1 = {S50, 0, &V12855_1_1, NULL};	/*Property = IsMobile*/
const Value V12855_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12855_5 = {S52, 0, &V12855_5_1293, &P12855_1};	/*Property = ScreenPixelsHeight*/
const Value V12855_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12855_4 = {S54, 0, &V12855_4_1296, &P12855_5};	/*Property = ScreenPixelsWidth*/
const Profile P12855 = {"12855", &P12855_4};	/*Profile ID - 12855*/
const Value V15857_1_1 = {S2, NULL};	/*Value = True*/
const Property P15857_1 = {S50, 0, &V15857_1_1, NULL};	/*Property = IsMobile*/
const Value V15857_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15857_5 = {S52, 0, &V15857_5_1227, &P15857_1};	/*Property = ScreenPixelsHeight*/
const Value V15857_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15857_4 = {S54, 0, &V15857_4_1229, &P15857_5};	/*Property = ScreenPixelsWidth*/
const Profile P15857 = {"15857", &P15857_4};	/*Profile ID - 15857*/
const Value V4420_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_279 = {S1, 0, &V4420_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4420_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_282 = {S3, 0, &V4420_282_57155, &P4420_279};	/*Property = BlobBuilder*/
const Value V4420_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_350 = {S4, 0, &V4420_350_57446, &P4420_282};	/*Property = Canvas*/
const Value V4420_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_283 = {S5, 0, &V4420_283_57161, &P4420_350};	/*Property = CssBackground*/
const Value V4420_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_284 = {S6, 0, &V4420_284_57164, &P4420_283};	/*Property = CssBorderImage*/
const Value V4420_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_285 = {S7, 0, &V4420_285_57167, &P4420_284};	/*Property = CssCanvas*/
const Value V4420_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_286 = {S8, 0, &V4420_286_57170, &P4420_285};	/*Property = CssColor*/
const Value V4420_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_237 = {S9, 0, &V4420_237_55786, &P4420_286};	/*Property = CssColumn*/
const Value V4420_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_287 = {S10, 0, &V4420_287_57173, &P4420_237};	/*Property = CssFlexbox*/
const Value V4420_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_288 = {S11, 0, &V4420_288_57176, &P4420_287};	/*Property = CssFont*/
const Value V4420_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_289 = {S12, 0, &V4420_289_86899, &P4420_288};	/*Property = CssImages*/
const Value V4420_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_291 = {S13, 0, &V4420_291_57182, &P4420_289};	/*Property = CssMediaQueries*/
const Value V4420_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_292 = {S14, 0, &V4420_292_57185, &P4420_291};	/*Property = CssMinMax*/
const Value V4420_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_293 = {S15, 0, &V4420_293_57188, &P4420_292};	/*Property = CssOverflow*/
const Value V4420_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_294 = {S16, 0, &V4420_294_57191, &P4420_293};	/*Property = CssPosition*/
const Value V4420_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_295 = {S17, 0, &V4420_295_57194, &P4420_294};	/*Property = CssText*/
const Value V4420_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_297 = {S18, 0, &V4420_297_57197, &P4420_295};	/*Property = CssTransforms*/
const Value V4420_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_241 = {S19, 0, &V4420_241_55801, &P4420_297};	/*Property = CssTransitions*/
const Value V4420_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_298 = {S20, 0, &V4420_298_57208, &P4420_241};	/*Property = CssUI*/
const Value V4420_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_300 = {S21, 0, &V4420_300_57217, &P4420_298};	/*Property = DataSet*/
const Value V4420_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_301 = {S22, 0, &V4420_301_57220, &P4420_300};	/*Property = DataUrl*/
const Value V4420_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_302 = {S23, 0, &V4420_302_57223, &P4420_301};	/*Property = DeviceOrientation*/
const Value V4420_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_304 = {S24, 0, &V4420_304_57229, &P4420_302};	/*Property = FileReader*/
const Value V4420_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_305 = {S25, 0, &V4420_305_57232, &P4420_304};	/*Property = FileSaver*/
const Value V4420_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_306 = {S26, 0, &V4420_306_57238, &P4420_305};	/*Property = FileWriter*/
const Value V4420_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_307 = {S27, 0, &V4420_307_57241, &P4420_306};	/*Property = FormData*/
const Value V4420_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_309 = {S28, 0, &V4420_309_57247, &P4420_307};	/*Property = Fullscreen*/
const Value V4420_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_310 = {S29, 0, &V4420_310_57250, &P4420_309};	/*Property = GeoLocation*/
const Value V4420_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_312 = {S30, 0, &V4420_312_57256, &P4420_310};	/*Property = History*/
const Value V4420_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_314 = {S31, 0, &V4420_314_57262, &P4420_312};	/*Property = Html5*/
const Value V4420_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_313 = {S32, 0, &V4420_313_57259, &P4420_314};	/*Property = Html-Media-Capture*/
const Value V4420_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_315 = {S33, 0, &V4420_315_57265, &P4420_313};	/*Property = Iframe*/
const Value V4420_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_316 = {S34, 0, &V4420_316_57268, &P4420_315};	/*Property = IndexedDB*/
const Value V4420_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_317 = {S35, 0, &V4420_317_57271, &P4420_316};	/*Property = Json*/
const Value V4420_15_43264 = {S59, NULL};	/*Value = Mango*/
const Property P4420_15 = {S37, 0, &V4420_15_43264, &P4420_317};	/*Property = LayoutEngine*/
const Value V4420_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_344 = {S38, 0, &V4420_344_57274, &P4420_15};	/*Property = Masking*/
const Value V4420_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_320 = {S39, 0, &V4420_320_57292, &P4420_344};	/*Property = PostMessage*/
const Value V4420_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_321 = {S40, 0, &V4420_321_57295, &P4420_320};	/*Property = Progress*/
const Value V4420_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_322 = {S41, 0, &V4420_322_57298, &P4420_321};	/*Property = Prompts*/
const Value V4420_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_323 = {S42, 0, &V4420_323_57301, &P4420_322};	/*Property = Selector*/
const Value V4420_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_324 = {S43, 0, &V4420_324_57307, &P4420_323};	/*Property = Svg*/
const Value V4420_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_327 = {S44, 0, &V4420_327_57316, &P4420_324};	/*Property = TouchEvents*/
const Value V4420_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_328 = {S45, 0, &V4420_328_57319, &P4420_327};	/*Property = Track*/
const Value V4420_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_329 = {S46, 0, &V4420_329_57325, &P4420_328};	/*Property = Video*/
const Value V4420_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_330 = {S47, 0, &V4420_330_57328, &P4420_329};	/*Property = Viewport*/
const Value V4420_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_332 = {S48, 0, &V4420_332_57340, &P4420_330};	/*Property = WebWorkers*/
const Value V4420_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4420_333 = {S49, 0, &V4420_333_57343, &P4420_332};	/*Property = Xhr2*/
const Profile P4420 = {"4420", &P4420_333};	/*Profile ID - 4420*/
const Profile P6208 = {"6208", NULL};	/*Profile ID - 6208*/
const Value V12860_1_1 = {S2, NULL};	/*Value = True*/
const Property P12860_1 = {S50, 0, &V12860_1_1, NULL};	/*Property = IsMobile*/
const Value V12860_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12860_5 = {S52, 0, &V12860_5_1233, &P12860_1};	/*Property = ScreenPixelsHeight*/
const Value V12860_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12860_4 = {S54, 0, &V12860_4_1235, &P12860_5};	/*Property = ScreenPixelsWidth*/
const Profile P12860 = {"12860", &P12860_4};	/*Profile ID - 12860*/
const Value V16697_1_1 = {S2, NULL};	/*Value = True*/
const Property P16697_1 = {S50, 0, &V16697_1_1, NULL};	/*Property = IsMobile*/
const Value V16697_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16697_5 = {S52, 0, &V16697_5_1294, &P16697_1};	/*Property = ScreenPixelsHeight*/
const Value V16697_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16697_4 = {S54, 0, &V16697_4_1296, &P16697_5};	/*Property = ScreenPixelsWidth*/
const Profile P16697 = {"16697", &P16697_4};	/*Profile ID - 16697*/
const Profile P22784 = {"22784", NULL};	/*Profile ID - 22784*/
const Value V12862_1_1 = {S2, NULL};	/*Value = True*/
const Property P12862_1 = {S50, 0, &V12862_1_1, NULL};	/*Property = IsMobile*/
const Value V12862_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12862_5 = {S52, 0, &V12862_5_1233, &P12862_1};	/*Property = ScreenPixelsHeight*/
const Value V12862_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12862_4 = {S54, 0, &V12862_4_1235, &P12862_5};	/*Property = ScreenPixelsWidth*/
const Profile P12862 = {"12862", &P12862_4};	/*Profile ID - 12862*/
const Value V17795_1_1 = {S2, NULL};	/*Value = True*/
const Property P17795_1 = {S50, 0, &V17795_1_1, NULL};	/*Property = IsMobile*/
const Value V17795_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17795_5 = {S52, 0, &V17795_5_1269, &P17795_1};	/*Property = ScreenPixelsHeight*/
const Value V17795_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17795_4 = {S54, 0, &V17795_4_1229, &P17795_5};	/*Property = ScreenPixelsWidth*/
const Profile P17795 = {"17795", &P17795_4};	/*Profile ID - 17795*/
const Value V12647_1_1 = {S2, NULL};	/*Value = True*/
const Property P12647_1 = {S50, 0, &V12647_1_1, NULL};	/*Property = IsMobile*/
const Value V12647_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12647_5 = {S52, 0, &V12647_5_1304, &P12647_1};	/*Property = ScreenPixelsHeight*/
const Value V12647_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12647_4 = {S54, 0, &V12647_4_1224, &P12647_5};	/*Property = ScreenPixelsWidth*/
const Profile P12647 = {"12647", &P12647_4};	/*Profile ID - 12647*/
const Value V12867_1_1 = {S2, NULL};	/*Value = True*/
const Property P12867_1 = {S50, 0, &V12867_1_1, NULL};	/*Property = IsMobile*/
const Value V12867_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12867_5 = {S52, 0, &V12867_5_1227, &P12867_1};	/*Property = ScreenPixelsHeight*/
const Value V12867_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12867_4 = {S54, 0, &V12867_4_1229, &P12867_5};	/*Property = ScreenPixelsWidth*/
const Profile P12867 = {"12867", &P12867_4};	/*Profile ID - 12867*/
const Value V12866_1_1 = {S2, NULL};	/*Value = True*/
const Property P12866_1 = {S50, 0, &V12866_1_1, NULL};	/*Property = IsMobile*/
const Value V12866_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12866_5 = {S52, 0, &V12866_5_1227, &P12866_1};	/*Property = ScreenPixelsHeight*/
const Value V12866_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12866_4 = {S54, 0, &V12866_4_1229, &P12866_5};	/*Property = ScreenPixelsWidth*/
const Profile P12866 = {"12866", &P12866_4};	/*Profile ID - 12866*/
const Value V12870_1_1 = {S2, NULL};	/*Value = True*/
const Property P12870_1 = {S50, 0, &V12870_1_1, NULL};	/*Property = IsMobile*/
const Value V12870_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12870_5 = {S52, 0, &V12870_5_1233, &P12870_1};	/*Property = ScreenPixelsHeight*/
const Value V12870_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12870_4 = {S54, 0, &V12870_4_1235, &P12870_5};	/*Property = ScreenPixelsWidth*/
const Profile P12870 = {"12870", &P12870_4};	/*Profile ID - 12870*/
const Value V13075_1_1 = {S2, NULL};	/*Value = True*/
const Property P13075_1 = {S50, 0, &V13075_1_1, NULL};	/*Property = IsMobile*/
const Value V13075_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13075_5 = {S52, 0, &V13075_5_1293, &P13075_1};	/*Property = ScreenPixelsHeight*/
const Value V13075_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13075_4 = {S54, 0, &V13075_4_1296, &P13075_5};	/*Property = ScreenPixelsWidth*/
const Profile P13075 = {"13075", &P13075_4};	/*Profile ID - 13075*/
const Value V12877_1_1 = {S2, NULL};	/*Value = True*/
const Property P12877_1 = {S50, 0, &V12877_1_1, NULL};	/*Property = IsMobile*/
const Value V12877_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12877_5 = {S52, 0, &V12877_5_1222, &P12877_1};	/*Property = ScreenPixelsHeight*/
const Value V12877_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12877_4 = {S54, 0, &V12877_4_1224, &P12877_5};	/*Property = ScreenPixelsWidth*/
const Profile P12877 = {"12877", &P12877_4};	/*Profile ID - 12877*/
const Value V12710_1_1 = {S2, NULL};	/*Value = True*/
const Property P12710_1 = {S50, 0, &V12710_1_1, NULL};	/*Property = IsMobile*/
const Value V12710_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12710_5 = {S52, 0, &V12710_5_1277, &P12710_1};	/*Property = ScreenPixelsHeight*/
const Value V12710_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12710_4 = {S54, 0, &V12710_4_1279, &P12710_5};	/*Property = ScreenPixelsWidth*/
const Profile P12710 = {"12710", &P12710_4};	/*Profile ID - 12710*/
const Value V15523_1_1 = {S2, NULL};	/*Value = True*/
const Property P15523_1 = {S50, 0, &V15523_1_1, NULL};	/*Property = IsMobile*/
const Value V15523_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15523_5 = {S52, 0, &V15523_5_1222, &P15523_1};	/*Property = ScreenPixelsHeight*/
const Value V15523_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15523_4 = {S54, 0, &V15523_4_1224, &P15523_5};	/*Property = ScreenPixelsWidth*/
const Profile P15523 = {"15523", &P15523_4};	/*Profile ID - 15523*/
const Value V16610_1_1 = {S2, NULL};	/*Value = True*/
const Property P16610_1 = {S50, 0, &V16610_1_1, NULL};	/*Property = IsMobile*/
const Value V16610_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P16610_5 = {S52, 0, &V16610_5_1304, &P16610_1};	/*Property = ScreenPixelsHeight*/
const Value V16610_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16610_4 = {S54, 0, &V16610_4_1224, &P16610_5};	/*Property = ScreenPixelsWidth*/
const Profile P16610 = {"16610", &P16610_4};	/*Profile ID - 16610*/
const Value V12880_1_1 = {S2, NULL};	/*Value = True*/
const Property P12880_1 = {S50, 0, &V12880_1_1, NULL};	/*Property = IsMobile*/
const Value V12880_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12880_5 = {S52, 0, &V12880_5_1380, &P12880_1};	/*Property = ScreenPixelsHeight*/
const Value V12880_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12880_4 = {S54, 0, &V12880_4_1279, &P12880_5};	/*Property = ScreenPixelsWidth*/
const Profile P12880 = {"12880", &P12880_4};	/*Profile ID - 12880*/
const Value V12881_1_1 = {S2, NULL};	/*Value = True*/
const Property P12881_1 = {S50, 0, &V12881_1_1, NULL};	/*Property = IsMobile*/
const Value V12881_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12881_5 = {S52, 0, &V12881_5_1222, &P12881_1};	/*Property = ScreenPixelsHeight*/
const Value V12881_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12881_4 = {S54, 0, &V12881_4_1224, &P12881_5};	/*Property = ScreenPixelsWidth*/
const Profile P12881 = {"12881", &P12881_4};	/*Profile ID - 12881*/
const Value V12935_1_1 = {S2, NULL};	/*Value = True*/
const Property P12935_1 = {S50, 0, &V12935_1_1, NULL};	/*Property = IsMobile*/
const Value V12935_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12935_5 = {S52, 0, &V12935_5_1277, &P12935_1};	/*Property = ScreenPixelsHeight*/
const Value V12935_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12935_4 = {S54, 0, &V12935_4_1279, &P12935_5};	/*Property = ScreenPixelsWidth*/
const Profile P12935 = {"12935", &P12935_4};	/*Profile ID - 12935*/
const Value V12884_1_1 = {S2, NULL};	/*Value = True*/
const Property P12884_1 = {S50, 0, &V12884_1_1, NULL};	/*Property = IsMobile*/
const Value V12884_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12884_5 = {S52, 0, &V12884_5_1277, &P12884_1};	/*Property = ScreenPixelsHeight*/
const Value V12884_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12884_4 = {S54, 0, &V12884_4_1279, &P12884_5};	/*Property = ScreenPixelsWidth*/
const Profile P12884 = {"12884", &P12884_4};	/*Profile ID - 12884*/
const Value V12885_1_1 = {S2, NULL};	/*Value = True*/
const Property P12885_1 = {S50, 0, &V12885_1_1, NULL};	/*Property = IsMobile*/
const Value V12885_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12885_5 = {S52, 0, &V12885_5_1277, &P12885_1};	/*Property = ScreenPixelsHeight*/
const Value V12885_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12885_4 = {S54, 0, &V12885_4_1279, &P12885_5};	/*Property = ScreenPixelsWidth*/
const Profile P12885 = {"12885", &P12885_4};	/*Profile ID - 12885*/
const Value V12886_1_1 = {S2, NULL};	/*Value = True*/
const Property P12886_1 = {S50, 0, &V12886_1_1, NULL};	/*Property = IsMobile*/
const Value V12886_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12886_5 = {S52, 0, &V12886_5_1233, &P12886_1};	/*Property = ScreenPixelsHeight*/
const Value V12886_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12886_4 = {S54, 0, &V12886_4_1235, &P12886_5};	/*Property = ScreenPixelsWidth*/
const Profile P12886 = {"12886", &P12886_4};	/*Profile ID - 12886*/
const Value V23916_1_1 = {S2, NULL};	/*Value = True*/
const Property P23916_1 = {S50, 0, &V23916_1_1, NULL};	/*Property = IsMobile*/
const Value V23916_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23916_5 = {S52, 0, &V23916_5_1269, &P23916_1};	/*Property = ScreenPixelsHeight*/
const Value V23916_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23916_4 = {S54, 0, &V23916_4_1229, &P23916_5};	/*Property = ScreenPixelsWidth*/
const Profile P23916 = {"23916", &P23916_4};	/*Profile ID - 23916*/
const Value V12622_1_1 = {S2, NULL};	/*Value = True*/
const Property P12622_1 = {S50, 0, &V12622_1_1, NULL};	/*Property = IsMobile*/
const Value V12622_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12622_5 = {S52, 0, &V12622_5_1222, &P12622_1};	/*Property = ScreenPixelsHeight*/
const Value V12622_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12622_4 = {S54, 0, &V12622_4_1224, &P12622_5};	/*Property = ScreenPixelsWidth*/
const Profile P12622 = {"12622", &P12622_4};	/*Profile ID - 12622*/
const Value V15547_1_1 = {S2, NULL};	/*Value = True*/
const Property P15547_1 = {S50, 0, &V15547_1_1, NULL};	/*Property = IsMobile*/
const Value V15547_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15547_5 = {S52, 0, &V15547_5_1304, &P15547_1};	/*Property = ScreenPixelsHeight*/
const Value V15547_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15547_4 = {S54, 0, &V15547_4_1224, &P15547_5};	/*Property = ScreenPixelsWidth*/
const Profile P15547 = {"15547", &P15547_4};	/*Profile ID - 15547*/
const Value V13015_1_1 = {S2, NULL};	/*Value = True*/
const Property P13015_1 = {S50, 0, &V13015_1_1, NULL};	/*Property = IsMobile*/
const Value V13015_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13015_5 = {S52, 0, &V13015_5_1222, &P13015_1};	/*Property = ScreenPixelsHeight*/
const Value V13015_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13015_4 = {S54, 0, &V13015_4_1224, &P13015_5};	/*Property = ScreenPixelsWidth*/
const Profile P13015 = {"13015", &P13015_4};	/*Profile ID - 13015*/
const Value V12889_1_1 = {S2, NULL};	/*Value = True*/
const Property P12889_1 = {S50, 0, &V12889_1_1, NULL};	/*Property = IsMobile*/
const Value V12889_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P12889_5 = {S52, 0, &V12889_5_9981, &P12889_1};	/*Property = ScreenPixelsHeight*/
const Value V12889_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P12889_4 = {S54, 0, &V12889_4_2336, &P12889_5};	/*Property = ScreenPixelsWidth*/
const Profile P12889 = {"12889", &P12889_4};	/*Profile ID - 12889*/
const Value V17324_1_1 = {S2, NULL};	/*Value = True*/
const Property P17324_1 = {S50, 0, &V17324_1_1, NULL};	/*Property = IsMobile*/
const Value V17324_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17324_5 = {S52, 0, &V17324_5_1227, &P17324_1};	/*Property = ScreenPixelsHeight*/
const Value V17324_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17324_4 = {S54, 0, &V17324_4_1229, &P17324_5};	/*Property = ScreenPixelsWidth*/
const Profile P17324 = {"17324", &P17324_4};	/*Profile ID - 17324*/
const Value V13056_1_1 = {S2, NULL};	/*Value = True*/
const Property P13056_1 = {S50, 0, &V13056_1_1, NULL};	/*Property = IsMobile*/
const Value V13056_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13056_5 = {S52, 0, &V13056_5_1233, &P13056_1};	/*Property = ScreenPixelsHeight*/
const Value V13056_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13056_4 = {S54, 0, &V13056_4_1235, &P13056_5};	/*Property = ScreenPixelsWidth*/
const Profile P13056 = {"13056", &P13056_4};	/*Profile ID - 13056*/
const Value V17337_1_1 = {S2, NULL};	/*Value = True*/
const Property P17337_1 = {S50, 0, &V17337_1_1, NULL};	/*Property = IsMobile*/
const Value V17337_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P17337_5 = {S52, 0, &V17337_5_9981, &P17337_1};	/*Property = ScreenPixelsHeight*/
const Value V17337_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P17337_4 = {S54, 0, &V17337_4_2336, &P17337_5};	/*Property = ScreenPixelsWidth*/
const Profile P17337 = {"17337", &P17337_4};	/*Profile ID - 17337*/
const Value V12891_1_1 = {S2, NULL};	/*Value = True*/
const Property P12891_1 = {S50, 0, &V12891_1_1, NULL};	/*Property = IsMobile*/
const Value V12891_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12891_5 = {S52, 0, &V12891_5_1304, &P12891_1};	/*Property = ScreenPixelsHeight*/
const Value V12891_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12891_4 = {S54, 0, &V12891_4_1224, &P12891_5};	/*Property = ScreenPixelsWidth*/
const Profile P12891 = {"12891", &P12891_4};	/*Profile ID - 12891*/
const Value V15874_1_1 = {S2, NULL};	/*Value = True*/
const Property P15874_1 = {S50, 0, &V15874_1_1, NULL};	/*Property = IsMobile*/
const Value V15874_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P15874_5 = {S52, 0, &V15874_5_1269, &P15874_1};	/*Property = ScreenPixelsHeight*/
const Value V15874_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15874_4 = {S54, 0, &V15874_4_1229, &P15874_5};	/*Property = ScreenPixelsWidth*/
const Profile P15874 = {"15874", &P15874_4};	/*Profile ID - 15874*/
const Profile P17322 = {"17322", NULL};	/*Profile ID - 17322*/
const Value V22720_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22720_279 = {S1, 0, &V22720_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22720_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22720_282 = {S3, 0, &V22720_282_57154, &P22720_279};	/*Property = BlobBuilder*/
const Value V22720_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22720_350 = {S4, 0, &V22720_350_57444, &P22720_282};	/*Property = Canvas*/
const Value V22720_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22720_283 = {S5, 0, &V22720_283_57159, &P22720_350};	/*Property = CssBackground*/
const Value V22720_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22720_284 = {S6, 0, &V22720_284_57163, &P22720_283};	/*Property = CssBorderImage*/
const Value V22720_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22720_285 = {S7, 0, &V22720_285_57166, &P22720_284};	/*Property = CssCanvas*/
const Value V22720_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22720_286 = {S8, 0, &V22720_286_57168, &P22720_285};	/*Property = CssColor*/
const Value V22720_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22720_237 = {S9, 0, &V22720_237_55786, &P22720_286};	/*Property = CssColumn*/
const Value V22720_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22720_287 = {S10, 0, &V22720_287_57172, &P22720_237};	/*Property = CssFlexbox*/
const Value V22720_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22720_288 = {S11, 0, &V22720_288_57174, &P22720_287};	/*Property = CssFont*/
const Value V22720_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22720_289 = {S12, 0, &V22720_289_57759, &P22720_288};	/*Property = CssImages*/
const Value V22720_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22720_291 = {S13, 0, &V22720_291_57181, &P22720_289};	/*Property = CssMediaQueries*/
const Value V22720_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22720_292 = {S14, 0, &V22720_292_57183, &P22720_291};	/*Property = CssMinMax*/
const Value V22720_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22720_293 = {S15, 0, &V22720_293_57187, &P22720_292};	/*Property = CssOverflow*/
const Value V22720_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22720_294 = {S16, 0, &V22720_294_57190, &P22720_293};	/*Property = CssPosition*/
const Value V22720_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22720_295 = {S17, 0, &V22720_295_57192, &P22720_294};	/*Property = CssText*/
const Value V22720_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22720_297 = {S18, 0, &V22720_297_57195, &P22720_295};	/*Property = CssTransforms*/
const Value V22720_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22720_241 = {S19, 0, &V22720_241_55799, &P22720_297};	/*Property = CssTransitions*/
const Value V22720_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22720_298 = {S20, 0, &V22720_298_57206, &P22720_241};	/*Property = CssUI*/
const Value V22720_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22720_300 = {S21, 0, &V22720_300_57216, &P22720_298};	/*Property = DataSet*/
const Value V22720_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22720_301 = {S22, 0, &V22720_301_57218, &P22720_300};	/*Property = DataUrl*/
const Value V22720_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22720_302 = {S23, 0, &V22720_302_57221, &P22720_301};	/*Property = DeviceOrientation*/
const Value V22720_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22720_304 = {S24, 0, &V22720_304_57228, &P22720_302};	/*Property = FileReader*/
const Value V22720_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22720_305 = {S25, 0, &V22720_305_57231, &P22720_304};	/*Property = FileSaver*/
const Value V22720_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22720_306 = {S26, 0, &V22720_306_57237, &P22720_305};	/*Property = FileWriter*/
const Value V22720_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22720_307 = {S27, 0, &V22720_307_57239, &P22720_306};	/*Property = FormData*/
const Value V22720_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22720_309 = {S28, 0, &V22720_309_57246, &P22720_307};	/*Property = Fullscreen*/
const Value V22720_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22720_310 = {S29, 0, &V22720_310_57248, &P22720_309};	/*Property = GeoLocation*/
const Value V22720_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22720_312 = {S30, 0, &V22720_312_57255, &P22720_310};	/*Property = History*/
const Value V22720_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22720_314 = {S31, 0, &V22720_314_57261, &P22720_312};	/*Property = Html5*/
const Value V22720_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22720_313 = {S32, 0, &V22720_313_57258, &P22720_314};	/*Property = Html-Media-Capture*/
const Value V22720_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22720_315 = {S33, 0, &V22720_315_57264, &P22720_313};	/*Property = Iframe*/
const Value V22720_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22720_316 = {S34, 0, &V22720_316_57267, &P22720_315};	/*Property = IndexedDB*/
const Value V22720_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22720_317 = {S35, 0, &V22720_317_57269, &P22720_316};	/*Property = Json*/
const Value V22720_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22720_15 = {S37, 0, &V22720_15_72, &P22720_317};	/*Property = LayoutEngine*/
const Value V22720_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22720_344 = {S38, 0, &V22720_344_57272, &P22720_15};	/*Property = Masking*/
const Value V22720_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22720_320 = {S39, 0, &V22720_320_57290, &P22720_344};	/*Property = PostMessage*/
const Value V22720_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22720_321 = {S40, 0, &V22720_321_57293, &P22720_320};	/*Property = Progress*/
const Value V22720_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22720_322 = {S41, 0, &V22720_322_57296, &P22720_321};	/*Property = Prompts*/
const Value V22720_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22720_323 = {S42, 0, &V22720_323_57299, &P22720_322};	/*Property = Selector*/
const Value V22720_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22720_324 = {S43, 0, &V22720_324_57306, &P22720_323};	/*Property = Svg*/
const Value V22720_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22720_327 = {S44, 0, &V22720_327_57315, &P22720_324};	/*Property = TouchEvents*/
const Value V22720_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22720_328 = {S45, 0, &V22720_328_57318, &P22720_327};	/*Property = Track*/
const Value V22720_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22720_329 = {S46, 0, &V22720_329_57323, &P22720_328};	/*Property = Video*/
const Value V22720_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22720_330 = {S47, 0, &V22720_330_57326, &P22720_329};	/*Property = Viewport*/
const Value V22720_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22720_332 = {S48, 0, &V22720_332_57339, &P22720_330};	/*Property = WebWorkers*/
const Value V22720_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22720_333 = {S49, 0, &V22720_333_57342, &P22720_332};	/*Property = Xhr2*/
const Profile P22720 = {"22720", &P22720_333};	/*Profile ID - 22720*/
const Value V12883_1_1 = {S2, NULL};	/*Value = True*/
const Property P12883_1 = {S50, 0, &V12883_1_1, NULL};	/*Property = IsMobile*/
const Value V12883_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12883_5 = {S52, 0, &V12883_5_1293, &P12883_1};	/*Property = ScreenPixelsHeight*/
const Value V12883_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12883_4 = {S54, 0, &V12883_4_1296, &P12883_5};	/*Property = ScreenPixelsWidth*/
const Profile P12883 = {"12883", &P12883_4};	/*Profile ID - 12883*/
const Value V12702_1_1 = {S2, NULL};	/*Value = True*/
const Property P12702_1 = {S50, 0, &V12702_1_1, NULL};	/*Property = IsMobile*/
const Value V12702_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12702_5 = {S52, 0, &V12702_5_1222, &P12702_1};	/*Property = ScreenPixelsHeight*/
const Value V12702_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12702_4 = {S54, 0, &V12702_4_1224, &P12702_5};	/*Property = ScreenPixelsWidth*/
const Profile P12702 = {"12702", &P12702_4};	/*Profile ID - 12702*/
const Value V12894_1_1 = {S2, NULL};	/*Value = True*/
const Property P12894_1 = {S50, 0, &V12894_1_1, NULL};	/*Property = IsMobile*/
const Value V12894_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12894_5 = {S52, 0, &V12894_5_1293, &P12894_1};	/*Property = ScreenPixelsHeight*/
const Value V12894_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12894_4 = {S54, 0, &V12894_4_1296, &P12894_5};	/*Property = ScreenPixelsWidth*/
const Profile P12894 = {"12894", &P12894_4};	/*Profile ID - 12894*/
const Value V16858_1_1 = {S2, NULL};	/*Value = True*/
const Property P16858_1 = {S50, 0, &V16858_1_1, NULL};	/*Property = IsMobile*/
const Value V16858_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16858_5 = {S52, 0, &V16858_5_1222, &P16858_1};	/*Property = ScreenPixelsHeight*/
const Value V16858_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16858_4 = {S54, 0, &V16858_4_1224, &P16858_5};	/*Property = ScreenPixelsWidth*/
const Profile P16858 = {"16858", &P16858_4};	/*Profile ID - 16858*/
const Value V12897_1_1 = {S2, NULL};	/*Value = True*/
const Property P12897_1 = {S50, 0, &V12897_1_1, NULL};	/*Property = IsMobile*/
const Value V12897_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12897_5 = {S52, 0, &V12897_5_1293, &P12897_1};	/*Property = ScreenPixelsHeight*/
const Value V12897_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12897_4 = {S54, 0, &V12897_4_1296, &P12897_5};	/*Property = ScreenPixelsWidth*/
const Profile P12897 = {"12897", &P12897_4};	/*Profile ID - 12897*/
const Value V12898_1_1 = {S2, NULL};	/*Value = True*/
const Property P12898_1 = {S50, 0, &V12898_1_1, NULL};	/*Property = IsMobile*/
const Value V12898_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12898_5 = {S52, 0, &V12898_5_1304, &P12898_1};	/*Property = ScreenPixelsHeight*/
const Value V12898_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12898_4 = {S54, 0, &V12898_4_1224, &P12898_5};	/*Property = ScreenPixelsWidth*/
const Profile P12898 = {"12898", &P12898_4};	/*Profile ID - 12898*/
const Value V12899_1_1 = {S2, NULL};	/*Value = True*/
const Property P12899_1 = {S50, 0, &V12899_1_1, NULL};	/*Property = IsMobile*/
const Value V12899_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12899_5 = {S52, 0, &V12899_5_1293, &P12899_1};	/*Property = ScreenPixelsHeight*/
const Value V12899_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12899_4 = {S54, 0, &V12899_4_1296, &P12899_5};	/*Property = ScreenPixelsWidth*/
const Profile P12899 = {"12899", &P12899_4};	/*Profile ID - 12899*/
const Value V12900_1_1 = {S2, NULL};	/*Value = True*/
const Property P12900_1 = {S50, 0, &V12900_1_1, NULL};	/*Property = IsMobile*/
const Value V12900_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12900_5 = {S52, 0, &V12900_5_1233, &P12900_1};	/*Property = ScreenPixelsHeight*/
const Value V12900_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12900_4 = {S54, 0, &V12900_4_1235, &P12900_5};	/*Property = ScreenPixelsWidth*/
const Profile P12900 = {"12900", &P12900_4};	/*Profile ID - 12900*/
const Value V15875_1_1 = {S2, NULL};	/*Value = True*/
const Property P15875_1 = {S50, 0, &V15875_1_1, NULL};	/*Property = IsMobile*/
const Value V15875_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15875_5 = {S52, 0, &V15875_5_1304, &P15875_1};	/*Property = ScreenPixelsHeight*/
const Value V15875_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15875_4 = {S54, 0, &V15875_4_1224, &P15875_5};	/*Property = ScreenPixelsWidth*/
const Profile P15875 = {"15875", &P15875_4};	/*Profile ID - 15875*/
const Value V12902_1_1 = {S2, NULL};	/*Value = True*/
const Property P12902_1 = {S50, 0, &V12902_1_1, NULL};	/*Property = IsMobile*/
const Value V12902_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12902_5 = {S52, 0, &V12902_5_1233, &P12902_1};	/*Property = ScreenPixelsHeight*/
const Value V12902_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12902_4 = {S54, 0, &V12902_4_1235, &P12902_5};	/*Property = ScreenPixelsWidth*/
const Profile P12902 = {"12902", &P12902_4};	/*Profile ID - 12902*/
const Value V12904_1_1 = {S2, NULL};	/*Value = True*/
const Property P12904_1 = {S50, 0, &V12904_1_1, NULL};	/*Property = IsMobile*/
const Value V12904_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P12904_5 = {S52, 0, &V12904_5_1333, &P12904_1};	/*Property = ScreenPixelsHeight*/
const Value V12904_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P12904_4 = {S54, 0, &V12904_4_1335, &P12904_5};	/*Property = ScreenPixelsWidth*/
const Profile P12904 = {"12904", &P12904_4};	/*Profile ID - 12904*/
const Value V12905_1_1 = {S2, NULL};	/*Value = True*/
const Property P12905_1 = {S50, 0, &V12905_1_1, NULL};	/*Property = IsMobile*/
const Value V12905_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12905_5 = {S52, 0, &V12905_5_1233, &P12905_1};	/*Property = ScreenPixelsHeight*/
const Value V12905_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12905_4 = {S54, 0, &V12905_4_1235, &P12905_5};	/*Property = ScreenPixelsWidth*/
const Profile P12905 = {"12905", &P12905_4};	/*Profile ID - 12905*/
const Value V14910_1_1 = {S2, NULL};	/*Value = True*/
const Property P14910_1 = {S50, 0, &V14910_1_1, NULL};	/*Property = IsMobile*/
const Value V14910_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14910_5 = {S52, 0, &V14910_5_1233, &P14910_1};	/*Property = ScreenPixelsHeight*/
const Value V14910_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14910_4 = {S54, 0, &V14910_4_1235, &P14910_5};	/*Property = ScreenPixelsWidth*/
const Profile P14910 = {"14910", &P14910_4};	/*Profile ID - 14910*/
const Value V16677_1_1 = {S2, NULL};	/*Value = True*/
const Property P16677_1 = {S50, 0, &V16677_1_1, NULL};	/*Property = IsMobile*/
const Value V16677_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16677_5 = {S52, 0, &V16677_5_1294, &P16677_1};	/*Property = ScreenPixelsHeight*/
const Value V16677_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16677_4 = {S54, 0, &V16677_4_1296, &P16677_5};	/*Property = ScreenPixelsWidth*/
const Profile P16677 = {"16677", &P16677_4};	/*Profile ID - 16677*/
const Value V12907_1_1 = {S2, NULL};	/*Value = True*/
const Property P12907_1 = {S50, 0, &V12907_1_1, NULL};	/*Property = IsMobile*/
const Value V12907_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12907_5 = {S52, 0, &V12907_5_1277, &P12907_1};	/*Property = ScreenPixelsHeight*/
const Value V12907_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P12907_4 = {S54, 0, &V12907_4_2336, &P12907_5};	/*Property = ScreenPixelsWidth*/
const Profile P12907 = {"12907", &P12907_4};	/*Profile ID - 12907*/
const Value V12688_1_1 = {S2, NULL};	/*Value = True*/
const Property P12688_1 = {S50, 0, &V12688_1_1, NULL};	/*Property = IsMobile*/
const Value V12688_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12688_5 = {S52, 0, &V12688_5_1222, &P12688_1};	/*Property = ScreenPixelsHeight*/
const Value V12688_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12688_4 = {S54, 0, &V12688_4_1224, &P12688_5};	/*Property = ScreenPixelsWidth*/
const Profile P12688 = {"12688", &P12688_4};	/*Profile ID - 12688*/
const Value V13031_1_1 = {S2, NULL};	/*Value = True*/
const Property P13031_1 = {S50, 0, &V13031_1_1, NULL};	/*Property = IsMobile*/
const Value V13031_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13031_5 = {S52, 0, &V13031_5_1277, &P13031_1};	/*Property = ScreenPixelsHeight*/
const Value V13031_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13031_4 = {S54, 0, &V13031_4_1279, &P13031_5};	/*Property = ScreenPixelsWidth*/
const Profile P13031 = {"13031", &P13031_4};	/*Profile ID - 13031*/
const Value V24569_1_1 = {S2, NULL};	/*Value = True*/
const Property P24569_1 = {S50, 0, &V24569_1_1, NULL};	/*Property = IsMobile*/
const Value V24569_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24569_5 = {S52, 0, &V24569_5_1277, &P24569_1};	/*Property = ScreenPixelsHeight*/
const Value V24569_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24569_4 = {S54, 0, &V24569_4_1279, &P24569_5};	/*Property = ScreenPixelsWidth*/
const Profile P24569 = {"24569", &P24569_4};	/*Profile ID - 24569*/
const Value V12910_1_1 = {S2, NULL};	/*Value = True*/
const Property P12910_1 = {S50, 0, &V12910_1_1, NULL};	/*Property = IsMobile*/
const Value V12910_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12910_5 = {S52, 0, &V12910_5_1294, &P12910_1};	/*Property = ScreenPixelsHeight*/
const Value V12910_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12910_4 = {S54, 0, &V12910_4_1296, &P12910_5};	/*Property = ScreenPixelsWidth*/
const Profile P12910 = {"12910", &P12910_4};	/*Profile ID - 12910*/
const Value V12911_1_1 = {S2, NULL};	/*Value = True*/
const Property P12911_1 = {S50, 0, &V12911_1_1, NULL};	/*Property = IsMobile*/
const Value V12911_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12911_5 = {S52, 0, &V12911_5_1222, &P12911_1};	/*Property = ScreenPixelsHeight*/
const Value V12911_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12911_4 = {S54, 0, &V12911_4_1224, &P12911_5};	/*Property = ScreenPixelsWidth*/
const Profile P12911 = {"12911", &P12911_4};	/*Profile ID - 12911*/
const Value V12912_1_1 = {S2, NULL};	/*Value = True*/
const Property P12912_1 = {S50, 0, &V12912_1_1, NULL};	/*Property = IsMobile*/
const Value V12912_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12912_5 = {S52, 0, &V12912_5_1380, &P12912_1};	/*Property = ScreenPixelsHeight*/
const Value V12912_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12912_4 = {S54, 0, &V12912_4_1279, &P12912_5};	/*Property = ScreenPixelsWidth*/
const Profile P12912 = {"12912", &P12912_4};	/*Profile ID - 12912*/
const Value V12913_1_1 = {S2, NULL};	/*Value = True*/
const Property P12913_1 = {S50, 0, &V12913_1_1, NULL};	/*Property = IsMobile*/
const Value V12913_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12913_5 = {S52, 0, &V12913_5_1233, &P12913_1};	/*Property = ScreenPixelsHeight*/
const Value V12913_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12913_4 = {S54, 0, &V12913_4_1235, &P12913_5};	/*Property = ScreenPixelsWidth*/
const Profile P12913 = {"12913", &P12913_4};	/*Profile ID - 12913*/
const Value V12906_1_1 = {S2, NULL};	/*Value = True*/
const Property P12906_1 = {S50, 0, &V12906_1_1, NULL};	/*Property = IsMobile*/
const Value V12906_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12906_5 = {S52, 0, &V12906_5_1304, &P12906_1};	/*Property = ScreenPixelsHeight*/
const Value V12906_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12906_4 = {S54, 0, &V12906_4_1224, &P12906_5};	/*Property = ScreenPixelsWidth*/
const Profile P12906 = {"12906", &P12906_4};	/*Profile ID - 12906*/
const Value V17433_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_279 = {S1, 0, &V17433_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17433_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_282 = {S3, 0, &V17433_282_57155, &P17433_279};	/*Property = BlobBuilder*/
const Value V17433_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_350 = {S4, 0, &V17433_350_57446, &P17433_282};	/*Property = Canvas*/
const Value V17433_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_283 = {S5, 0, &V17433_283_57161, &P17433_350};	/*Property = CssBackground*/
const Value V17433_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_284 = {S6, 0, &V17433_284_57164, &P17433_283};	/*Property = CssBorderImage*/
const Value V17433_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_285 = {S7, 0, &V17433_285_57167, &P17433_284};	/*Property = CssCanvas*/
const Value V17433_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_286 = {S8, 0, &V17433_286_57170, &P17433_285};	/*Property = CssColor*/
const Value V17433_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_237 = {S9, 0, &V17433_237_55786, &P17433_286};	/*Property = CssColumn*/
const Value V17433_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_287 = {S10, 0, &V17433_287_57173, &P17433_237};	/*Property = CssFlexbox*/
const Value V17433_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_288 = {S11, 0, &V17433_288_57176, &P17433_287};	/*Property = CssFont*/
const Value V17433_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_289 = {S12, 0, &V17433_289_86899, &P17433_288};	/*Property = CssImages*/
const Value V17433_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_291 = {S13, 0, &V17433_291_57182, &P17433_289};	/*Property = CssMediaQueries*/
const Value V17433_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_292 = {S14, 0, &V17433_292_57185, &P17433_291};	/*Property = CssMinMax*/
const Value V17433_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_293 = {S15, 0, &V17433_293_57188, &P17433_292};	/*Property = CssOverflow*/
const Value V17433_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_294 = {S16, 0, &V17433_294_57191, &P17433_293};	/*Property = CssPosition*/
const Value V17433_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_295 = {S17, 0, &V17433_295_57194, &P17433_294};	/*Property = CssText*/
const Value V17433_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_297 = {S18, 0, &V17433_297_57197, &P17433_295};	/*Property = CssTransforms*/
const Value V17433_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_241 = {S19, 0, &V17433_241_55801, &P17433_297};	/*Property = CssTransitions*/
const Value V17433_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_298 = {S20, 0, &V17433_298_57208, &P17433_241};	/*Property = CssUI*/
const Value V17433_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_300 = {S21, 0, &V17433_300_57217, &P17433_298};	/*Property = DataSet*/
const Value V17433_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_301 = {S22, 0, &V17433_301_57220, &P17433_300};	/*Property = DataUrl*/
const Value V17433_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_302 = {S23, 0, &V17433_302_57223, &P17433_301};	/*Property = DeviceOrientation*/
const Value V17433_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_304 = {S24, 0, &V17433_304_57229, &P17433_302};	/*Property = FileReader*/
const Value V17433_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_305 = {S25, 0, &V17433_305_57232, &P17433_304};	/*Property = FileSaver*/
const Value V17433_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_306 = {S26, 0, &V17433_306_57238, &P17433_305};	/*Property = FileWriter*/
const Value V17433_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_307 = {S27, 0, &V17433_307_57241, &P17433_306};	/*Property = FormData*/
const Value V17433_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_309 = {S28, 0, &V17433_309_57247, &P17433_307};	/*Property = Fullscreen*/
const Value V17433_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_310 = {S29, 0, &V17433_310_57250, &P17433_309};	/*Property = GeoLocation*/
const Value V17433_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_312 = {S30, 0, &V17433_312_57256, &P17433_310};	/*Property = History*/
const Value V17433_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_314 = {S31, 0, &V17433_314_57262, &P17433_312};	/*Property = Html5*/
const Value V17433_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_313 = {S32, 0, &V17433_313_57259, &P17433_314};	/*Property = Html-Media-Capture*/
const Value V17433_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_315 = {S33, 0, &V17433_315_57265, &P17433_313};	/*Property = Iframe*/
const Value V17433_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_316 = {S34, 0, &V17433_316_57268, &P17433_315};	/*Property = IndexedDB*/
const Value V17433_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_317 = {S35, 0, &V17433_317_57271, &P17433_316};	/*Property = Json*/
const Value V17433_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P17433_15 = {S37, 0, &V17433_15_29, &P17433_317};	/*Property = LayoutEngine*/
const Value V17433_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_344 = {S38, 0, &V17433_344_57274, &P17433_15};	/*Property = Masking*/
const Value V17433_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_320 = {S39, 0, &V17433_320_57292, &P17433_344};	/*Property = PostMessage*/
const Value V17433_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_321 = {S40, 0, &V17433_321_57295, &P17433_320};	/*Property = Progress*/
const Value V17433_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_322 = {S41, 0, &V17433_322_57298, &P17433_321};	/*Property = Prompts*/
const Value V17433_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_323 = {S42, 0, &V17433_323_57301, &P17433_322};	/*Property = Selector*/
const Value V17433_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_324 = {S43, 0, &V17433_324_57307, &P17433_323};	/*Property = Svg*/
const Value V17433_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_327 = {S44, 0, &V17433_327_57316, &P17433_324};	/*Property = TouchEvents*/
const Value V17433_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_328 = {S45, 0, &V17433_328_57319, &P17433_327};	/*Property = Track*/
const Value V17433_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_329 = {S46, 0, &V17433_329_57325, &P17433_328};	/*Property = Video*/
const Value V17433_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_330 = {S47, 0, &V17433_330_57328, &P17433_329};	/*Property = Viewport*/
const Value V17433_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_332 = {S48, 0, &V17433_332_57340, &P17433_330};	/*Property = WebWorkers*/
const Value V17433_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17433_333 = {S49, 0, &V17433_333_57343, &P17433_332};	/*Property = Xhr2*/
const Profile P17433 = {"17433", &P17433_333};	/*Profile ID - 17433*/
const Value V12915_1_1 = {S2, NULL};	/*Value = True*/
const Property P12915_1 = {S50, 0, &V12915_1_1, NULL};	/*Property = IsMobile*/
const Value V12915_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12915_5 = {S52, 0, &V12915_5_1222, &P12915_1};	/*Property = ScreenPixelsHeight*/
const Value V12915_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12915_4 = {S54, 0, &V12915_4_1224, &P12915_5};	/*Property = ScreenPixelsWidth*/
const Profile P12915 = {"12915", &P12915_4};	/*Profile ID - 12915*/
const Value V12917_1_1 = {S2, NULL};	/*Value = True*/
const Property P12917_1 = {S50, 0, &V12917_1_1, NULL};	/*Property = IsMobile*/
const Value V12917_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12917_5 = {S52, 0, &V12917_5_1293, &P12917_1};	/*Property = ScreenPixelsHeight*/
const Value V12917_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12917_4 = {S54, 0, &V12917_4_1296, &P12917_5};	/*Property = ScreenPixelsWidth*/
const Profile P12917 = {"12917", &P12917_4};	/*Profile ID - 12917*/
const Value V16362_1_1 = {S2, NULL};	/*Value = True*/
const Property P16362_1 = {S50, 0, &V16362_1_1, NULL};	/*Property = IsMobile*/
const Value V16362_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16362_5 = {S52, 0, &V16362_5_1227, &P16362_1};	/*Property = ScreenPixelsHeight*/
const Value V16362_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16362_4 = {S54, 0, &V16362_4_1229, &P16362_5};	/*Property = ScreenPixelsWidth*/
const Profile P16362 = {"16362", &P16362_4};	/*Profile ID - 16362*/
const Value V12920_1_1 = {S2, NULL};	/*Value = True*/
const Property P12920_1 = {S50, 0, &V12920_1_1, NULL};	/*Property = IsMobile*/
const Value V12920_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12920_5 = {S52, 0, &V12920_5_1293, &P12920_1};	/*Property = ScreenPixelsHeight*/
const Value V12920_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12920_4 = {S54, 0, &V12920_4_1296, &P12920_5};	/*Property = ScreenPixelsWidth*/
const Profile P12920 = {"12920", &P12920_4};	/*Profile ID - 12920*/
const Value V12921_1_1 = {S2, NULL};	/*Value = True*/
const Property P12921_1 = {S50, 0, &V12921_1_1, NULL};	/*Property = IsMobile*/
const Value V12921_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12921_5 = {S52, 0, &V12921_5_1222, &P12921_1};	/*Property = ScreenPixelsHeight*/
const Value V12921_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12921_4 = {S54, 0, &V12921_4_1224, &P12921_5};	/*Property = ScreenPixelsWidth*/
const Profile P12921 = {"12921", &P12921_4};	/*Profile ID - 12921*/
const Value V12841_1_1 = {S2, NULL};	/*Value = True*/
const Property P12841_1 = {S50, 0, &V12841_1_1, NULL};	/*Property = IsMobile*/
const Value V12841_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P12841_5 = {S52, 0, &V12841_5_1403, &P12841_1};	/*Property = ScreenPixelsHeight*/
const Value V12841_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P12841_4 = {S54, 0, &V12841_4_1404, &P12841_5};	/*Property = ScreenPixelsWidth*/
const Profile P12841 = {"12841", &P12841_4};	/*Profile ID - 12841*/
const Value V12595_1_1 = {S2, NULL};	/*Value = True*/
const Property P12595_1 = {S50, 0, &V12595_1_1, NULL};	/*Property = IsMobile*/
const Value V12595_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12595_5 = {S52, 0, &V12595_5_1227, &P12595_1};	/*Property = ScreenPixelsHeight*/
const Value V12595_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12595_4 = {S54, 0, &V12595_4_1229, &P12595_5};	/*Property = ScreenPixelsWidth*/
const Profile P12595 = {"12595", &P12595_4};	/*Profile ID - 12595*/
const Value V15921_1_1 = {S2, NULL};	/*Value = True*/
const Property P15921_1 = {S50, 0, &V15921_1_1, NULL};	/*Property = IsMobile*/
const Value V15921_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15921_5 = {S52, 0, &V15921_5_1222, &P15921_1};	/*Property = ScreenPixelsHeight*/
const Value V15921_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15921_4 = {S54, 0, &V15921_4_1224, &P15921_5};	/*Property = ScreenPixelsWidth*/
const Profile P15921 = {"15921", &P15921_4};	/*Profile ID - 15921*/
const Value V12922_1_1 = {S2, NULL};	/*Value = True*/
const Property P12922_1 = {S50, 0, &V12922_1_1, NULL};	/*Property = IsMobile*/
const Value V12922_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12922_5 = {S52, 0, &V12922_5_1293, &P12922_1};	/*Property = ScreenPixelsHeight*/
const Value V12922_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12922_4 = {S54, 0, &V12922_4_1296, &P12922_5};	/*Property = ScreenPixelsWidth*/
const Profile P12922 = {"12922", &P12922_4};	/*Profile ID - 12922*/
const Value V15147_1_1 = {S2, NULL};	/*Value = True*/
const Property P15147_1 = {S50, 0, &V15147_1_1, NULL};	/*Property = IsMobile*/
const Value V15147_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15147_5 = {S52, 0, &V15147_5_1277, &P15147_1};	/*Property = ScreenPixelsHeight*/
const Value V15147_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15147_4 = {S54, 0, &V15147_4_1279, &P15147_5};	/*Property = ScreenPixelsWidth*/
const Profile P15147 = {"15147", &P15147_4};	/*Profile ID - 15147*/
const Value V4777_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_279 = {S1, 0, &V4777_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4777_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_282 = {S3, 0, &V4777_282_57155, &P4777_279};	/*Property = BlobBuilder*/
const Value V4777_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_350 = {S4, 0, &V4777_350_57446, &P4777_282};	/*Property = Canvas*/
const Value V4777_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_283 = {S5, 0, &V4777_283_57161, &P4777_350};	/*Property = CssBackground*/
const Value V4777_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_284 = {S6, 0, &V4777_284_57164, &P4777_283};	/*Property = CssBorderImage*/
const Value V4777_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_285 = {S7, 0, &V4777_285_57167, &P4777_284};	/*Property = CssCanvas*/
const Value V4777_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_286 = {S8, 0, &V4777_286_57170, &P4777_285};	/*Property = CssColor*/
const Value V4777_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_237 = {S9, 0, &V4777_237_55786, &P4777_286};	/*Property = CssColumn*/
const Value V4777_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_287 = {S10, 0, &V4777_287_57173, &P4777_237};	/*Property = CssFlexbox*/
const Value V4777_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_288 = {S11, 0, &V4777_288_57176, &P4777_287};	/*Property = CssFont*/
const Value V4777_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_289 = {S12, 0, &V4777_289_86899, &P4777_288};	/*Property = CssImages*/
const Value V4777_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_291 = {S13, 0, &V4777_291_57182, &P4777_289};	/*Property = CssMediaQueries*/
const Value V4777_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_292 = {S14, 0, &V4777_292_57185, &P4777_291};	/*Property = CssMinMax*/
const Value V4777_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_293 = {S15, 0, &V4777_293_57188, &P4777_292};	/*Property = CssOverflow*/
const Value V4777_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_294 = {S16, 0, &V4777_294_57191, &P4777_293};	/*Property = CssPosition*/
const Value V4777_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_295 = {S17, 0, &V4777_295_57194, &P4777_294};	/*Property = CssText*/
const Value V4777_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_297 = {S18, 0, &V4777_297_57197, &P4777_295};	/*Property = CssTransforms*/
const Value V4777_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_241 = {S19, 0, &V4777_241_55801, &P4777_297};	/*Property = CssTransitions*/
const Value V4777_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_298 = {S20, 0, &V4777_298_57208, &P4777_241};	/*Property = CssUI*/
const Value V4777_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_300 = {S21, 0, &V4777_300_57217, &P4777_298};	/*Property = DataSet*/
const Value V4777_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_301 = {S22, 0, &V4777_301_57220, &P4777_300};	/*Property = DataUrl*/
const Value V4777_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_302 = {S23, 0, &V4777_302_57223, &P4777_301};	/*Property = DeviceOrientation*/
const Value V4777_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_304 = {S24, 0, &V4777_304_57229, &P4777_302};	/*Property = FileReader*/
const Value V4777_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_305 = {S25, 0, &V4777_305_57232, &P4777_304};	/*Property = FileSaver*/
const Value V4777_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_306 = {S26, 0, &V4777_306_57238, &P4777_305};	/*Property = FileWriter*/
const Value V4777_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_307 = {S27, 0, &V4777_307_57241, &P4777_306};	/*Property = FormData*/
const Value V4777_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_309 = {S28, 0, &V4777_309_57247, &P4777_307};	/*Property = Fullscreen*/
const Value V4777_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_310 = {S29, 0, &V4777_310_57250, &P4777_309};	/*Property = GeoLocation*/
const Value V4777_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_312 = {S30, 0, &V4777_312_57256, &P4777_310};	/*Property = History*/
const Value V4777_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_314 = {S31, 0, &V4777_314_57262, &P4777_312};	/*Property = Html5*/
const Value V4777_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_313 = {S32, 0, &V4777_313_57259, &P4777_314};	/*Property = Html-Media-Capture*/
const Value V4777_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_315 = {S33, 0, &V4777_315_57265, &P4777_313};	/*Property = Iframe*/
const Value V4777_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_316 = {S34, 0, &V4777_316_57268, &P4777_315};	/*Property = IndexedDB*/
const Value V4777_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_317 = {S35, 0, &V4777_317_57271, &P4777_316};	/*Property = Json*/
const Value V4777_15_90838 = {S70, NULL};	/*Value = U3*/
const Property P4777_15 = {S37, 0, &V4777_15_90838, &P4777_317};	/*Property = LayoutEngine*/
const Value V4777_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_344 = {S38, 0, &V4777_344_57274, &P4777_15};	/*Property = Masking*/
const Value V4777_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_320 = {S39, 0, &V4777_320_57292, &P4777_344};	/*Property = PostMessage*/
const Value V4777_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_321 = {S40, 0, &V4777_321_57295, &P4777_320};	/*Property = Progress*/
const Value V4777_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_322 = {S41, 0, &V4777_322_57298, &P4777_321};	/*Property = Prompts*/
const Value V4777_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_323 = {S42, 0, &V4777_323_57301, &P4777_322};	/*Property = Selector*/
const Value V4777_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_324 = {S43, 0, &V4777_324_57307, &P4777_323};	/*Property = Svg*/
const Value V4777_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_327 = {S44, 0, &V4777_327_57316, &P4777_324};	/*Property = TouchEvents*/
const Value V4777_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_328 = {S45, 0, &V4777_328_57319, &P4777_327};	/*Property = Track*/
const Value V4777_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_329 = {S46, 0, &V4777_329_57325, &P4777_328};	/*Property = Video*/
const Value V4777_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_330 = {S47, 0, &V4777_330_57328, &P4777_329};	/*Property = Viewport*/
const Value V4777_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_332 = {S48, 0, &V4777_332_57340, &P4777_330};	/*Property = WebWorkers*/
const Value V4777_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4777_333 = {S49, 0, &V4777_333_57343, &P4777_332};	/*Property = Xhr2*/
const Profile P4777 = {"4777", &P4777_333};	/*Profile ID - 4777*/
const Value V17435_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_279 = {S1, 0, &V17435_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17435_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_282 = {S3, 0, &V17435_282_57155, &P17435_279};	/*Property = BlobBuilder*/
const Value V17435_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_350 = {S4, 0, &V17435_350_57446, &P17435_282};	/*Property = Canvas*/
const Value V17435_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_283 = {S5, 0, &V17435_283_57161, &P17435_350};	/*Property = CssBackground*/
const Value V17435_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_284 = {S6, 0, &V17435_284_57164, &P17435_283};	/*Property = CssBorderImage*/
const Value V17435_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_285 = {S7, 0, &V17435_285_57167, &P17435_284};	/*Property = CssCanvas*/
const Value V17435_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_286 = {S8, 0, &V17435_286_57170, &P17435_285};	/*Property = CssColor*/
const Value V17435_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_237 = {S9, 0, &V17435_237_55786, &P17435_286};	/*Property = CssColumn*/
const Value V17435_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_287 = {S10, 0, &V17435_287_57173, &P17435_237};	/*Property = CssFlexbox*/
const Value V17435_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_288 = {S11, 0, &V17435_288_57176, &P17435_287};	/*Property = CssFont*/
const Value V17435_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_289 = {S12, 0, &V17435_289_86899, &P17435_288};	/*Property = CssImages*/
const Value V17435_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_291 = {S13, 0, &V17435_291_57182, &P17435_289};	/*Property = CssMediaQueries*/
const Value V17435_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_292 = {S14, 0, &V17435_292_57185, &P17435_291};	/*Property = CssMinMax*/
const Value V17435_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_293 = {S15, 0, &V17435_293_57188, &P17435_292};	/*Property = CssOverflow*/
const Value V17435_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_294 = {S16, 0, &V17435_294_57191, &P17435_293};	/*Property = CssPosition*/
const Value V17435_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_295 = {S17, 0, &V17435_295_57194, &P17435_294};	/*Property = CssText*/
const Value V17435_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_297 = {S18, 0, &V17435_297_57197, &P17435_295};	/*Property = CssTransforms*/
const Value V17435_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_241 = {S19, 0, &V17435_241_55801, &P17435_297};	/*Property = CssTransitions*/
const Value V17435_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_298 = {S20, 0, &V17435_298_57208, &P17435_241};	/*Property = CssUI*/
const Value V17435_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_300 = {S21, 0, &V17435_300_57217, &P17435_298};	/*Property = DataSet*/
const Value V17435_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_301 = {S22, 0, &V17435_301_57220, &P17435_300};	/*Property = DataUrl*/
const Value V17435_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_302 = {S23, 0, &V17435_302_57223, &P17435_301};	/*Property = DeviceOrientation*/
const Value V17435_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_304 = {S24, 0, &V17435_304_57229, &P17435_302};	/*Property = FileReader*/
const Value V17435_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_305 = {S25, 0, &V17435_305_57232, &P17435_304};	/*Property = FileSaver*/
const Value V17435_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_306 = {S26, 0, &V17435_306_57238, &P17435_305};	/*Property = FileWriter*/
const Value V17435_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_307 = {S27, 0, &V17435_307_57241, &P17435_306};	/*Property = FormData*/
const Value V17435_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_309 = {S28, 0, &V17435_309_57247, &P17435_307};	/*Property = Fullscreen*/
const Value V17435_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_310 = {S29, 0, &V17435_310_57250, &P17435_309};	/*Property = GeoLocation*/
const Value V17435_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_312 = {S30, 0, &V17435_312_57256, &P17435_310};	/*Property = History*/
const Value V17435_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_314 = {S31, 0, &V17435_314_57262, &P17435_312};	/*Property = Html5*/
const Value V17435_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_313 = {S32, 0, &V17435_313_57259, &P17435_314};	/*Property = Html-Media-Capture*/
const Value V17435_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_315 = {S33, 0, &V17435_315_57265, &P17435_313};	/*Property = Iframe*/
const Value V17435_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_316 = {S34, 0, &V17435_316_57268, &P17435_315};	/*Property = IndexedDB*/
const Value V17435_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_317 = {S35, 0, &V17435_317_57271, &P17435_316};	/*Property = Json*/
const Value V17435_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P17435_15 = {S37, 0, &V17435_15_29, &P17435_317};	/*Property = LayoutEngine*/
const Value V17435_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_344 = {S38, 0, &V17435_344_57274, &P17435_15};	/*Property = Masking*/
const Value V17435_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_320 = {S39, 0, &V17435_320_57292, &P17435_344};	/*Property = PostMessage*/
const Value V17435_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_321 = {S40, 0, &V17435_321_57295, &P17435_320};	/*Property = Progress*/
const Value V17435_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_322 = {S41, 0, &V17435_322_57298, &P17435_321};	/*Property = Prompts*/
const Value V17435_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_323 = {S42, 0, &V17435_323_57301, &P17435_322};	/*Property = Selector*/
const Value V17435_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_324 = {S43, 0, &V17435_324_57307, &P17435_323};	/*Property = Svg*/
const Value V17435_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_327 = {S44, 0, &V17435_327_57316, &P17435_324};	/*Property = TouchEvents*/
const Value V17435_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_328 = {S45, 0, &V17435_328_57319, &P17435_327};	/*Property = Track*/
const Value V17435_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_329 = {S46, 0, &V17435_329_57325, &P17435_328};	/*Property = Video*/
const Value V17435_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_330 = {S47, 0, &V17435_330_57328, &P17435_329};	/*Property = Viewport*/
const Value V17435_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_332 = {S48, 0, &V17435_332_57340, &P17435_330};	/*Property = WebWorkers*/
const Value V17435_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17435_333 = {S49, 0, &V17435_333_57343, &P17435_332};	/*Property = Xhr2*/
const Profile P17435 = {"17435", &P17435_333};	/*Profile ID - 17435*/
const Value V13218_1_1 = {S2, NULL};	/*Value = True*/
const Property P13218_1 = {S50, 0, &V13218_1_1, NULL};	/*Property = IsMobile*/
const Value V13218_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13218_5 = {S52, 0, &V13218_5_1277, &P13218_1};	/*Property = ScreenPixelsHeight*/
const Value V13218_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13218_4 = {S54, 0, &V13218_4_1279, &P13218_5};	/*Property = ScreenPixelsWidth*/
const Profile P13218 = {"13218", &P13218_4};	/*Profile ID - 13218*/
const Value V13675_1_1 = {S2, NULL};	/*Value = True*/
const Property P13675_1 = {S50, 0, &V13675_1_1, NULL};	/*Property = IsMobile*/
const Value V13675_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13675_5 = {S52, 0, &V13675_5_1227, &P13675_1};	/*Property = ScreenPixelsHeight*/
const Value V13675_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13675_4 = {S54, 0, &V13675_4_1229, &P13675_5};	/*Property = ScreenPixelsWidth*/
const Profile P13675 = {"13675", &P13675_4};	/*Profile ID - 13675*/
const Value V13674_1_1 = {S2, NULL};	/*Value = True*/
const Property P13674_1 = {S50, 0, &V13674_1_1, NULL};	/*Property = IsMobile*/
const Value V13674_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13674_5 = {S52, 0, &V13674_5_1222, &P13674_1};	/*Property = ScreenPixelsHeight*/
const Value V13674_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13674_4 = {S54, 0, &V13674_4_1224, &P13674_5};	/*Property = ScreenPixelsWidth*/
const Profile P13674 = {"13674", &P13674_4};	/*Profile ID - 13674*/
const Value V13680_1_1 = {S2, NULL};	/*Value = True*/
const Property P13680_1 = {S50, 0, &V13680_1_1, NULL};	/*Property = IsMobile*/
const Value V13680_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13680_5 = {S52, 0, &V13680_5_1233, &P13680_1};	/*Property = ScreenPixelsHeight*/
const Value V13680_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13680_4 = {S54, 0, &V13680_4_1235, &P13680_5};	/*Property = ScreenPixelsWidth*/
const Profile P13680 = {"13680", &P13680_4};	/*Profile ID - 13680*/
const Value V13683_1_1 = {S2, NULL};	/*Value = True*/
const Property P13683_1 = {S50, 0, &V13683_1_1, NULL};	/*Property = IsMobile*/
const Value V13683_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13683_5 = {S52, 0, &V13683_5_1222, &P13683_1};	/*Property = ScreenPixelsHeight*/
const Value V13683_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13683_4 = {S54, 0, &V13683_4_1224, &P13683_5};	/*Property = ScreenPixelsWidth*/
const Profile P13683 = {"13683", &P13683_4};	/*Profile ID - 13683*/
const Value V12118_1_1 = {S2, NULL};	/*Value = True*/
const Property P12118_1 = {S50, 0, &V12118_1_1, NULL};	/*Property = IsMobile*/
const Value V12118_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12118_5 = {S52, 0, &V12118_5_1233, &P12118_1};	/*Property = ScreenPixelsHeight*/
const Value V12118_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12118_4 = {S54, 0, &V12118_4_1235, &P12118_5};	/*Property = ScreenPixelsWidth*/
const Profile P12118 = {"12118", &P12118_4};	/*Profile ID - 12118*/
const Value V24662_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_279 = {S1, 0, &V24662_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24662_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_282 = {S3, 0, &V24662_282_57155, &P24662_279};	/*Property = BlobBuilder*/
const Value V24662_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_350 = {S4, 0, &V24662_350_57446, &P24662_282};	/*Property = Canvas*/
const Value V24662_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_283 = {S5, 0, &V24662_283_57161, &P24662_350};	/*Property = CssBackground*/
const Value V24662_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_284 = {S6, 0, &V24662_284_57164, &P24662_283};	/*Property = CssBorderImage*/
const Value V24662_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_285 = {S7, 0, &V24662_285_57167, &P24662_284};	/*Property = CssCanvas*/
const Value V24662_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_286 = {S8, 0, &V24662_286_57170, &P24662_285};	/*Property = CssColor*/
const Value V24662_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_237 = {S9, 0, &V24662_237_55786, &P24662_286};	/*Property = CssColumn*/
const Value V24662_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_287 = {S10, 0, &V24662_287_57173, &P24662_237};	/*Property = CssFlexbox*/
const Value V24662_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_288 = {S11, 0, &V24662_288_57176, &P24662_287};	/*Property = CssFont*/
const Value V24662_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_289 = {S12, 0, &V24662_289_86899, &P24662_288};	/*Property = CssImages*/
const Value V24662_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_291 = {S13, 0, &V24662_291_57182, &P24662_289};	/*Property = CssMediaQueries*/
const Value V24662_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_292 = {S14, 0, &V24662_292_57185, &P24662_291};	/*Property = CssMinMax*/
const Value V24662_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_293 = {S15, 0, &V24662_293_57188, &P24662_292};	/*Property = CssOverflow*/
const Value V24662_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_294 = {S16, 0, &V24662_294_57191, &P24662_293};	/*Property = CssPosition*/
const Value V24662_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_295 = {S17, 0, &V24662_295_57194, &P24662_294};	/*Property = CssText*/
const Value V24662_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_297 = {S18, 0, &V24662_297_57197, &P24662_295};	/*Property = CssTransforms*/
const Value V24662_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_241 = {S19, 0, &V24662_241_55801, &P24662_297};	/*Property = CssTransitions*/
const Value V24662_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_298 = {S20, 0, &V24662_298_57208, &P24662_241};	/*Property = CssUI*/
const Value V24662_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_300 = {S21, 0, &V24662_300_57217, &P24662_298};	/*Property = DataSet*/
const Value V24662_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_301 = {S22, 0, &V24662_301_57220, &P24662_300};	/*Property = DataUrl*/
const Value V24662_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_302 = {S23, 0, &V24662_302_57223, &P24662_301};	/*Property = DeviceOrientation*/
const Value V24662_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_304 = {S24, 0, &V24662_304_57229, &P24662_302};	/*Property = FileReader*/
const Value V24662_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_305 = {S25, 0, &V24662_305_57232, &P24662_304};	/*Property = FileSaver*/
const Value V24662_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_306 = {S26, 0, &V24662_306_57238, &P24662_305};	/*Property = FileWriter*/
const Value V24662_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_307 = {S27, 0, &V24662_307_57241, &P24662_306};	/*Property = FormData*/
const Value V24662_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_309 = {S28, 0, &V24662_309_57247, &P24662_307};	/*Property = Fullscreen*/
const Value V24662_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_310 = {S29, 0, &V24662_310_57250, &P24662_309};	/*Property = GeoLocation*/
const Value V24662_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_312 = {S30, 0, &V24662_312_57256, &P24662_310};	/*Property = History*/
const Value V24662_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_314 = {S31, 0, &V24662_314_57262, &P24662_312};	/*Property = Html5*/
const Value V24662_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_313 = {S32, 0, &V24662_313_57259, &P24662_314};	/*Property = Html-Media-Capture*/
const Value V24662_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_315 = {S33, 0, &V24662_315_57265, &P24662_313};	/*Property = Iframe*/
const Value V24662_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_316 = {S34, 0, &V24662_316_57268, &P24662_315};	/*Property = IndexedDB*/
const Value V24662_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_317 = {S35, 0, &V24662_317_57271, &P24662_316};	/*Property = Json*/
const Value V24662_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24662_15 = {S37, 0, &V24662_15_90837, &P24662_317};	/*Property = LayoutEngine*/
const Value V24662_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_344 = {S38, 0, &V24662_344_57274, &P24662_15};	/*Property = Masking*/
const Value V24662_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_320 = {S39, 0, &V24662_320_57292, &P24662_344};	/*Property = PostMessage*/
const Value V24662_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_321 = {S40, 0, &V24662_321_57295, &P24662_320};	/*Property = Progress*/
const Value V24662_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_322 = {S41, 0, &V24662_322_57298, &P24662_321};	/*Property = Prompts*/
const Value V24662_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_323 = {S42, 0, &V24662_323_57301, &P24662_322};	/*Property = Selector*/
const Value V24662_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_324 = {S43, 0, &V24662_324_57307, &P24662_323};	/*Property = Svg*/
const Value V24662_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_327 = {S44, 0, &V24662_327_57316, &P24662_324};	/*Property = TouchEvents*/
const Value V24662_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_328 = {S45, 0, &V24662_328_57319, &P24662_327};	/*Property = Track*/
const Value V24662_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_329 = {S46, 0, &V24662_329_57325, &P24662_328};	/*Property = Video*/
const Value V24662_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_330 = {S47, 0, &V24662_330_57328, &P24662_329};	/*Property = Viewport*/
const Value V24662_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_332 = {S48, 0, &V24662_332_57340, &P24662_330};	/*Property = WebWorkers*/
const Value V24662_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24662_333 = {S49, 0, &V24662_333_57343, &P24662_332};	/*Property = Xhr2*/
const Profile P24662 = {"24662", &P24662_333};	/*Profile ID - 24662*/
const Value V14316_1_1 = {S2, NULL};	/*Value = True*/
const Property P14316_1 = {S50, 0, &V14316_1_1, NULL};	/*Property = IsMobile*/
const Value V14316_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14316_5 = {S52, 0, &V14316_5_1294, &P14316_1};	/*Property = ScreenPixelsHeight*/
const Value V14316_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14316_4 = {S54, 0, &V14316_4_1296, &P14316_5};	/*Property = ScreenPixelsWidth*/
const Profile P14316 = {"14316", &P14316_4};	/*Profile ID - 14316*/
const Value V14165_1_1 = {S2, NULL};	/*Value = True*/
const Property P14165_1 = {S50, 0, &V14165_1_1, NULL};	/*Property = IsMobile*/
const Value V14165_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14165_5 = {S52, 0, &V14165_5_1222, &P14165_1};	/*Property = ScreenPixelsHeight*/
const Value V14165_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14165_4 = {S54, 0, &V14165_4_1224, &P14165_5};	/*Property = ScreenPixelsWidth*/
const Profile P14165 = {"14165", &P14165_4};	/*Profile ID - 14165*/
const Value V15809_1_1 = {S2, NULL};	/*Value = True*/
const Property P15809_1 = {S50, 0, &V15809_1_1, NULL};	/*Property = IsMobile*/
const Value V15809_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15809_5 = {S52, 0, &V15809_5_1227, &P15809_1};	/*Property = ScreenPixelsHeight*/
const Value V15809_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15809_4 = {S54, 0, &V15809_4_1229, &P15809_5};	/*Property = ScreenPixelsWidth*/
const Profile P15809 = {"15809", &P15809_4};	/*Profile ID - 15809*/
const Value V14686_1_1 = {S2, NULL};	/*Value = True*/
const Property P14686_1 = {S50, 0, &V14686_1_1, NULL};	/*Property = IsMobile*/
const Value V14686_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14686_5 = {S52, 0, &V14686_5_1380, &P14686_1};	/*Property = ScreenPixelsHeight*/
const Value V14686_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14686_4 = {S54, 0, &V14686_4_1279, &P14686_5};	/*Property = ScreenPixelsWidth*/
const Profile P14686 = {"14686", &P14686_4};	/*Profile ID - 14686*/
const Value V16969_1_1 = {S2, NULL};	/*Value = True*/
const Property P16969_1 = {S50, 0, &V16969_1_1, NULL};	/*Property = IsMobile*/
const Value V16969_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16969_5 = {S52, 0, &V16969_5_1293, &P16969_1};	/*Property = ScreenPixelsHeight*/
const Value V16969_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16969_4 = {S54, 0, &V16969_4_1296, &P16969_5};	/*Property = ScreenPixelsWidth*/
const Profile P16969 = {"16969", &P16969_4};	/*Profile ID - 16969*/
const Profile P17146 = {"17146", NULL};	/*Profile ID - 17146*/
const Value V17776_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17776_279 = {S1, 0, &V17776_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17776_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17776_282 = {S3, 0, &V17776_282_57154, &P17776_279};	/*Property = BlobBuilder*/
const Value V17776_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17776_350 = {S4, 0, &V17776_350_57444, &P17776_282};	/*Property = Canvas*/
const Value V17776_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17776_283 = {S5, 0, &V17776_283_57159, &P17776_350};	/*Property = CssBackground*/
const Value V17776_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17776_284 = {S6, 0, &V17776_284_57163, &P17776_283};	/*Property = CssBorderImage*/
const Value V17776_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17776_285 = {S7, 0, &V17776_285_57166, &P17776_284};	/*Property = CssCanvas*/
const Value V17776_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17776_286 = {S8, 0, &V17776_286_57168, &P17776_285};	/*Property = CssColor*/
const Value V17776_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17776_237 = {S9, 0, &V17776_237_55784, &P17776_286};	/*Property = CssColumn*/
const Value V17776_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17776_287 = {S10, 0, &V17776_287_57172, &P17776_237};	/*Property = CssFlexbox*/
const Value V17776_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17776_288 = {S11, 0, &V17776_288_57175, &P17776_287};	/*Property = CssFont*/
const Value V17776_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17776_289 = {S12, 0, &V17776_289_57759, &P17776_288};	/*Property = CssImages*/
const Value V17776_291_57180 = {S2, NULL};	/*Value = True*/
const Property P17776_291 = {S13, 0, &V17776_291_57180, &P17776_289};	/*Property = CssMediaQueries*/
const Value V17776_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17776_292 = {S14, 0, &V17776_292_57183, &P17776_291};	/*Property = CssMinMax*/
const Value V17776_293_57186 = {S2, NULL};	/*Value = True*/
const Property P17776_293 = {S15, 0, &V17776_293_57186, &P17776_292};	/*Property = CssOverflow*/
const Value V17776_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17776_294 = {S16, 0, &V17776_294_57189, &P17776_293};	/*Property = CssPosition*/
const Value V17776_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17776_295 = {S17, 0, &V17776_295_57192, &P17776_294};	/*Property = CssText*/
const Value V17776_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17776_297 = {S18, 0, &V17776_297_57195, &P17776_295};	/*Property = CssTransforms*/
const Value V17776_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17776_241 = {S19, 0, &V17776_241_55799, &P17776_297};	/*Property = CssTransitions*/
const Value V17776_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17776_298 = {S20, 0, &V17776_298_57206, &P17776_241};	/*Property = CssUI*/
const Value V17776_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17776_300 = {S21, 0, &V17776_300_57216, &P17776_298};	/*Property = DataSet*/
const Value V17776_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17776_301 = {S22, 0, &V17776_301_57218, &P17776_300};	/*Property = DataUrl*/
const Value V17776_302_57221 = {S2, NULL};	/*Value = True*/
const Property P17776_302 = {S23, 0, &V17776_302_57221, &P17776_301};	/*Property = DeviceOrientation*/
const Value V17776_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17776_304 = {S24, 0, &V17776_304_57228, &P17776_302};	/*Property = FileReader*/
const Value V17776_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17776_305 = {S25, 0, &V17776_305_57231, &P17776_304};	/*Property = FileSaver*/
const Value V17776_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17776_306 = {S26, 0, &V17776_306_57237, &P17776_305};	/*Property = FileWriter*/
const Value V17776_307_57239 = {S2, NULL};	/*Value = True*/
const Property P17776_307 = {S27, 0, &V17776_307_57239, &P17776_306};	/*Property = FormData*/
const Value V17776_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17776_309 = {S28, 0, &V17776_309_57246, &P17776_307};	/*Property = Fullscreen*/
const Value V17776_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17776_310 = {S29, 0, &V17776_310_57248, &P17776_309};	/*Property = GeoLocation*/
const Value V17776_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17776_312 = {S30, 0, &V17776_312_57255, &P17776_310};	/*Property = History*/
const Value V17776_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17776_314 = {S31, 0, &V17776_314_57261, &P17776_312};	/*Property = Html5*/
const Value V17776_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17776_313 = {S32, 0, &V17776_313_57258, &P17776_314};	/*Property = Html-Media-Capture*/
const Value V17776_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17776_315 = {S33, 0, &V17776_315_57264, &P17776_313};	/*Property = Iframe*/
const Value V17776_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17776_316 = {S34, 0, &V17776_316_57267, &P17776_315};	/*Property = IndexedDB*/
const Value V17776_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17776_317 = {S35, 0, &V17776_317_57269, &P17776_316};	/*Property = Json*/
const Value V17776_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17776_15 = {S37, 0, &V17776_15_28, &P17776_317};	/*Property = LayoutEngine*/
const Value V17776_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17776_344 = {S38, 0, &V17776_344_57272, &P17776_15};	/*Property = Masking*/
const Value V17776_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17776_320 = {S39, 0, &V17776_320_57290, &P17776_344};	/*Property = PostMessage*/
const Value V17776_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17776_321 = {S40, 0, &V17776_321_57293, &P17776_320};	/*Property = Progress*/
const Value V17776_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17776_322 = {S41, 0, &V17776_322_57296, &P17776_321};	/*Property = Prompts*/
const Value V17776_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17776_323 = {S42, 0, &V17776_323_57299, &P17776_322};	/*Property = Selector*/
const Value V17776_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17776_324 = {S43, 0, &V17776_324_57306, &P17776_323};	/*Property = Svg*/
const Value V17776_327_57314 = {S2, NULL};	/*Value = True*/
const Property P17776_327 = {S44, 0, &V17776_327_57314, &P17776_324};	/*Property = TouchEvents*/
const Value V17776_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17776_328 = {S45, 0, &V17776_328_57318, &P17776_327};	/*Property = Track*/
const Value V17776_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17776_329 = {S46, 0, &V17776_329_57323, &P17776_328};	/*Property = Video*/
const Value V17776_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17776_330 = {S47, 0, &V17776_330_57326, &P17776_329};	/*Property = Viewport*/
const Value V17776_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17776_332 = {S48, 0, &V17776_332_57339, &P17776_330};	/*Property = WebWorkers*/
const Value V17776_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17776_333 = {S49, 0, &V17776_333_57342, &P17776_332};	/*Property = Xhr2*/
const Profile P17776 = {"17776", &P17776_333};	/*Profile ID - 17776*/
const Value V14832_1_1 = {S2, NULL};	/*Value = True*/
const Property P14832_1 = {S50, 0, &V14832_1_1, NULL};	/*Property = IsMobile*/
const Value V14832_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14832_5 = {S52, 0, &V14832_5_1277, &P14832_1};	/*Property = ScreenPixelsHeight*/
const Value V14832_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14832_4 = {S54, 0, &V14832_4_1279, &P14832_5};	/*Property = ScreenPixelsWidth*/
const Profile P14832 = {"14832", &P14832_4};	/*Profile ID - 14832*/
const Value V14266_1_1 = {S2, NULL};	/*Value = True*/
const Property P14266_1 = {S50, 0, &V14266_1_1, NULL};	/*Property = IsMobile*/
const Value V14266_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P14266_5 = {S52, 0, &V14266_5_1333, &P14266_1};	/*Property = ScreenPixelsHeight*/
const Value V14266_4_37596 = {S98, NULL};	/*Value = 144*/
const Property P14266_4 = {S54, 0, &V14266_4_37596, &P14266_5};	/*Property = ScreenPixelsWidth*/
const Profile P14266 = {"14266", &P14266_4};	/*Profile ID - 14266*/
const Value V20518_1_1 = {S2, NULL};	/*Value = True*/
const Property P20518_1 = {S50, 0, &V20518_1_1, NULL};	/*Property = IsMobile*/
const Value V20518_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20518_5 = {S52, 0, &V20518_5_1227, &P20518_1};	/*Property = ScreenPixelsHeight*/
const Value V20518_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20518_4 = {S54, 0, &V20518_4_1229, &P20518_5};	/*Property = ScreenPixelsWidth*/
const Profile P20518 = {"20518", &P20518_4};	/*Profile ID - 20518*/
const Value V17524_1_1 = {S2, NULL};	/*Value = True*/
const Property P17524_1 = {S50, 0, &V17524_1_1, NULL};	/*Property = IsMobile*/
const Value V17524_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17524_5 = {S52, 0, &V17524_5_1304, &P17524_1};	/*Property = ScreenPixelsHeight*/
const Value V17524_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17524_4 = {S54, 0, &V17524_4_1224, &P17524_5};	/*Property = ScreenPixelsWidth*/
const Profile P17524 = {"17524", &P17524_4};	/*Profile ID - 17524*/
const Value V17243_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17243_279 = {S1, 0, &V17243_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17243_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17243_282 = {S3, 0, &V17243_282_57154, &P17243_279};	/*Property = BlobBuilder*/
const Value V17243_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17243_350 = {S4, 0, &V17243_350_57444, &P17243_282};	/*Property = Canvas*/
const Value V17243_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17243_283 = {S5, 0, &V17243_283_57159, &P17243_350};	/*Property = CssBackground*/
const Value V17243_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17243_284 = {S6, 0, &V17243_284_57163, &P17243_283};	/*Property = CssBorderImage*/
const Value V17243_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17243_285 = {S7, 0, &V17243_285_57166, &P17243_284};	/*Property = CssCanvas*/
const Value V17243_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17243_286 = {S8, 0, &V17243_286_57168, &P17243_285};	/*Property = CssColor*/
const Value V17243_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17243_237 = {S9, 0, &V17243_237_55784, &P17243_286};	/*Property = CssColumn*/
const Value V17243_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17243_287 = {S10, 0, &V17243_287_57172, &P17243_237};	/*Property = CssFlexbox*/
const Value V17243_288_57174 = {S2, NULL};	/*Value = True*/
const Property P17243_288 = {S11, 0, &V17243_288_57174, &P17243_287};	/*Property = CssFont*/
const Value V17243_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17243_289 = {S12, 0, &V17243_289_57759, &P17243_288};	/*Property = CssImages*/
const Value V17243_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17243_291 = {S13, 0, &V17243_291_57181, &P17243_289};	/*Property = CssMediaQueries*/
const Value V17243_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17243_292 = {S14, 0, &V17243_292_57183, &P17243_291};	/*Property = CssMinMax*/
const Value V17243_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17243_293 = {S15, 0, &V17243_293_57187, &P17243_292};	/*Property = CssOverflow*/
const Value V17243_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17243_294 = {S16, 0, &V17243_294_57189, &P17243_293};	/*Property = CssPosition*/
const Value V17243_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17243_295 = {S17, 0, &V17243_295_57192, &P17243_294};	/*Property = CssText*/
const Value V17243_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17243_297 = {S18, 0, &V17243_297_57195, &P17243_295};	/*Property = CssTransforms*/
const Value V17243_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17243_241 = {S19, 0, &V17243_241_55799, &P17243_297};	/*Property = CssTransitions*/
const Value V17243_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17243_298 = {S20, 0, &V17243_298_57206, &P17243_241};	/*Property = CssUI*/
const Value V17243_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17243_300 = {S21, 0, &V17243_300_57216, &P17243_298};	/*Property = DataSet*/
const Value V17243_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17243_301 = {S22, 0, &V17243_301_57218, &P17243_300};	/*Property = DataUrl*/
const Value V17243_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17243_302 = {S23, 0, &V17243_302_57222, &P17243_301};	/*Property = DeviceOrientation*/
const Value V17243_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17243_304 = {S24, 0, &V17243_304_57228, &P17243_302};	/*Property = FileReader*/
const Value V17243_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17243_305 = {S25, 0, &V17243_305_57231, &P17243_304};	/*Property = FileSaver*/
const Value V17243_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17243_306 = {S26, 0, &V17243_306_57237, &P17243_305};	/*Property = FileWriter*/
const Value V17243_307_57240 = {S0, NULL};	/*Value = False*/
const Property P17243_307 = {S27, 0, &V17243_307_57240, &P17243_306};	/*Property = FormData*/
const Value V17243_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17243_309 = {S28, 0, &V17243_309_57246, &P17243_307};	/*Property = Fullscreen*/
const Value V17243_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17243_310 = {S29, 0, &V17243_310_57248, &P17243_309};	/*Property = GeoLocation*/
const Value V17243_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17243_312 = {S30, 0, &V17243_312_57255, &P17243_310};	/*Property = History*/
const Value V17243_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17243_314 = {S31, 0, &V17243_314_57261, &P17243_312};	/*Property = Html5*/
const Value V17243_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17243_313 = {S32, 0, &V17243_313_57258, &P17243_314};	/*Property = Html-Media-Capture*/
const Value V17243_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17243_315 = {S33, 0, &V17243_315_57264, &P17243_313};	/*Property = Iframe*/
const Value V17243_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17243_316 = {S34, 0, &V17243_316_57267, &P17243_315};	/*Property = IndexedDB*/
const Value V17243_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17243_317 = {S35, 0, &V17243_317_57269, &P17243_316};	/*Property = Json*/
const Value V17243_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17243_15 = {S37, 0, &V17243_15_28, &P17243_317};	/*Property = LayoutEngine*/
const Value V17243_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17243_344 = {S38, 0, &V17243_344_57272, &P17243_15};	/*Property = Masking*/
const Value V17243_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17243_320 = {S39, 0, &V17243_320_57290, &P17243_344};	/*Property = PostMessage*/
const Value V17243_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17243_321 = {S40, 0, &V17243_321_57293, &P17243_320};	/*Property = Progress*/
const Value V17243_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17243_322 = {S41, 0, &V17243_322_57296, &P17243_321};	/*Property = Prompts*/
const Value V17243_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17243_323 = {S42, 0, &V17243_323_57299, &P17243_322};	/*Property = Selector*/
const Value V17243_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17243_324 = {S43, 0, &V17243_324_57306, &P17243_323};	/*Property = Svg*/
const Value V17243_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17243_327 = {S44, 0, &V17243_327_57315, &P17243_324};	/*Property = TouchEvents*/
const Value V17243_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17243_328 = {S45, 0, &V17243_328_57318, &P17243_327};	/*Property = Track*/
const Value V17243_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17243_329 = {S46, 0, &V17243_329_57323, &P17243_328};	/*Property = Video*/
const Value V17243_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17243_330 = {S47, 0, &V17243_330_57326, &P17243_329};	/*Property = Viewport*/
const Value V17243_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17243_332 = {S48, 0, &V17243_332_57339, &P17243_330};	/*Property = WebWorkers*/
const Value V17243_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17243_333 = {S49, 0, &V17243_333_57342, &P17243_332};	/*Property = Xhr2*/
const Profile P17243 = {"17243", &P17243_333};	/*Profile ID - 17243*/
const Value V13686_1_1 = {S2, NULL};	/*Value = True*/
const Property P13686_1 = {S50, 0, &V13686_1_1, NULL};	/*Property = IsMobile*/
const Value V13686_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13686_5 = {S52, 0, &V13686_5_1227, &P13686_1};	/*Property = ScreenPixelsHeight*/
const Value V13686_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13686_4 = {S54, 0, &V13686_4_1229, &P13686_5};	/*Property = ScreenPixelsWidth*/
const Profile P13686 = {"13686", &P13686_4};	/*Profile ID - 13686*/
const Value V14106_1_1 = {S2, NULL};	/*Value = True*/
const Property P14106_1 = {S50, 0, &V14106_1_1, NULL};	/*Property = IsMobile*/
const Value V14106_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14106_5 = {S52, 0, &V14106_5_1227, &P14106_1};	/*Property = ScreenPixelsHeight*/
const Value V14106_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14106_4 = {S54, 0, &V14106_4_1229, &P14106_5};	/*Property = ScreenPixelsWidth*/
const Profile P14106 = {"14106", &P14106_4};	/*Profile ID - 14106*/
const Value V13689_1_1 = {S2, NULL};	/*Value = True*/
const Property P13689_1 = {S50, 0, &V13689_1_1, NULL};	/*Property = IsMobile*/
const Value V13689_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13689_5 = {S52, 0, &V13689_5_1233, &P13689_1};	/*Property = ScreenPixelsHeight*/
const Value V13689_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13689_4 = {S54, 0, &V13689_4_1235, &P13689_5};	/*Property = ScreenPixelsWidth*/
const Profile P13689 = {"13689", &P13689_4};	/*Profile ID - 13689*/
const Value V13690_1_1 = {S2, NULL};	/*Value = True*/
const Property P13690_1 = {S50, 0, &V13690_1_1, NULL};	/*Property = IsMobile*/
const Value V13690_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13690_5 = {S52, 0, &V13690_5_1233, &P13690_1};	/*Property = ScreenPixelsHeight*/
const Value V13690_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13690_4 = {S54, 0, &V13690_4_1235, &P13690_5};	/*Property = ScreenPixelsWidth*/
const Profile P13690 = {"13690", &P13690_4};	/*Profile ID - 13690*/
const Value V15767_1_1 = {S2, NULL};	/*Value = True*/
const Property P15767_1 = {S50, 0, &V15767_1_1, NULL};	/*Property = IsMobile*/
const Value V15767_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P15767_5 = {S52, 0, &V15767_5_1458, &P15767_1};	/*Property = ScreenPixelsHeight*/
const Value V15767_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P15767_4 = {S54, 0, &V15767_4_37617, &P15767_5};	/*Property = ScreenPixelsWidth*/
const Profile P15767 = {"15767", &P15767_4};	/*Profile ID - 15767*/
const Value V14619_1_1 = {S2, NULL};	/*Value = True*/
const Property P14619_1 = {S50, 0, &V14619_1_1, NULL};	/*Property = IsMobile*/
const Value V14619_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14619_5 = {S52, 0, &V14619_5_1304, &P14619_1};	/*Property = ScreenPixelsHeight*/
const Value V14619_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14619_4 = {S54, 0, &V14619_4_1224, &P14619_5};	/*Property = ScreenPixelsWidth*/
const Profile P14619 = {"14619", &P14619_4};	/*Profile ID - 14619*/
const Value V17317_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17317_279 = {S1, 0, &V17317_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17317_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17317_282 = {S3, 0, &V17317_282_57154, &P17317_279};	/*Property = BlobBuilder*/
const Value V17317_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17317_350 = {S4, 0, &V17317_350_57444, &P17317_282};	/*Property = Canvas*/
const Value V17317_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17317_283 = {S5, 0, &V17317_283_57159, &P17317_350};	/*Property = CssBackground*/
const Value V17317_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17317_284 = {S6, 0, &V17317_284_57163, &P17317_283};	/*Property = CssBorderImage*/
const Value V17317_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17317_285 = {S7, 0, &V17317_285_57166, &P17317_284};	/*Property = CssCanvas*/
const Value V17317_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17317_286 = {S8, 0, &V17317_286_57168, &P17317_285};	/*Property = CssColor*/
const Value V17317_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17317_237 = {S9, 0, &V17317_237_55784, &P17317_286};	/*Property = CssColumn*/
const Value V17317_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17317_287 = {S10, 0, &V17317_287_57172, &P17317_237};	/*Property = CssFlexbox*/
const Value V17317_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17317_288 = {S11, 0, &V17317_288_57175, &P17317_287};	/*Property = CssFont*/
const Value V17317_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17317_289 = {S12, 0, &V17317_289_57759, &P17317_288};	/*Property = CssImages*/
const Value V17317_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17317_291 = {S13, 0, &V17317_291_57181, &P17317_289};	/*Property = CssMediaQueries*/
const Value V17317_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17317_292 = {S14, 0, &V17317_292_57183, &P17317_291};	/*Property = CssMinMax*/
const Value V17317_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17317_293 = {S15, 0, &V17317_293_57187, &P17317_292};	/*Property = CssOverflow*/
const Value V17317_294_57190 = {S0, NULL};	/*Value = False*/
const Property P17317_294 = {S16, 0, &V17317_294_57190, &P17317_293};	/*Property = CssPosition*/
const Value V17317_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17317_295 = {S17, 0, &V17317_295_57192, &P17317_294};	/*Property = CssText*/
const Value V17317_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17317_297 = {S18, 0, &V17317_297_57195, &P17317_295};	/*Property = CssTransforms*/
const Value V17317_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17317_241 = {S19, 0, &V17317_241_55799, &P17317_297};	/*Property = CssTransitions*/
const Value V17317_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17317_298 = {S20, 0, &V17317_298_57206, &P17317_241};	/*Property = CssUI*/
const Value V17317_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17317_300 = {S21, 0, &V17317_300_57216, &P17317_298};	/*Property = DataSet*/
const Value V17317_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17317_301 = {S22, 0, &V17317_301_57218, &P17317_300};	/*Property = DataUrl*/
const Value V17317_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17317_302 = {S23, 0, &V17317_302_57222, &P17317_301};	/*Property = DeviceOrientation*/
const Value V17317_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17317_304 = {S24, 0, &V17317_304_57228, &P17317_302};	/*Property = FileReader*/
const Value V17317_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17317_305 = {S25, 0, &V17317_305_57231, &P17317_304};	/*Property = FileSaver*/
const Value V17317_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17317_306 = {S26, 0, &V17317_306_57237, &P17317_305};	/*Property = FileWriter*/
const Value V17317_307_57240 = {S0, NULL};	/*Value = False*/
const Property P17317_307 = {S27, 0, &V17317_307_57240, &P17317_306};	/*Property = FormData*/
const Value V17317_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17317_309 = {S28, 0, &V17317_309_57246, &P17317_307};	/*Property = Fullscreen*/
const Value V17317_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17317_310 = {S29, 0, &V17317_310_57248, &P17317_309};	/*Property = GeoLocation*/
const Value V17317_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17317_312 = {S30, 0, &V17317_312_57255, &P17317_310};	/*Property = History*/
const Value V17317_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17317_314 = {S31, 0, &V17317_314_57261, &P17317_312};	/*Property = Html5*/
const Value V17317_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17317_313 = {S32, 0, &V17317_313_57258, &P17317_314};	/*Property = Html-Media-Capture*/
const Value V17317_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17317_315 = {S33, 0, &V17317_315_57264, &P17317_313};	/*Property = Iframe*/
const Value V17317_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17317_316 = {S34, 0, &V17317_316_57267, &P17317_315};	/*Property = IndexedDB*/
const Value V17317_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17317_317 = {S35, 0, &V17317_317_57269, &P17317_316};	/*Property = Json*/
const Value V17317_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17317_15 = {S37, 0, &V17317_15_28, &P17317_317};	/*Property = LayoutEngine*/
const Value V17317_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17317_344 = {S38, 0, &V17317_344_57272, &P17317_15};	/*Property = Masking*/
const Value V17317_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17317_320 = {S39, 0, &V17317_320_57290, &P17317_344};	/*Property = PostMessage*/
const Value V17317_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17317_321 = {S40, 0, &V17317_321_57293, &P17317_320};	/*Property = Progress*/
const Value V17317_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17317_322 = {S41, 0, &V17317_322_57296, &P17317_321};	/*Property = Prompts*/
const Value V17317_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17317_323 = {S42, 0, &V17317_323_57299, &P17317_322};	/*Property = Selector*/
const Value V17317_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17317_324 = {S43, 0, &V17317_324_57306, &P17317_323};	/*Property = Svg*/
const Value V17317_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17317_327 = {S44, 0, &V17317_327_57315, &P17317_324};	/*Property = TouchEvents*/
const Value V17317_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17317_328 = {S45, 0, &V17317_328_57318, &P17317_327};	/*Property = Track*/
const Value V17317_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17317_329 = {S46, 0, &V17317_329_57323, &P17317_328};	/*Property = Video*/
const Value V17317_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17317_330 = {S47, 0, &V17317_330_57326, &P17317_329};	/*Property = Viewport*/
const Value V17317_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17317_332 = {S48, 0, &V17317_332_57339, &P17317_330};	/*Property = WebWorkers*/
const Value V17317_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17317_333 = {S49, 0, &V17317_333_57342, &P17317_332};	/*Property = Xhr2*/
const Profile P17317 = {"17317", &P17317_333};	/*Profile ID - 17317*/
const Value V16264_1_1 = {S2, NULL};	/*Value = True*/
const Property P16264_1 = {S50, 0, &V16264_1_1, NULL};	/*Property = IsMobile*/
const Value V16264_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P16264_5 = {S52, 0, &V16264_5_1275, &P16264_1};	/*Property = ScreenPixelsHeight*/
const Value V16264_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P16264_4 = {S54, 0, &V16264_4_1278, &P16264_5};	/*Property = ScreenPixelsWidth*/
const Profile P16264 = {"16264", &P16264_4};	/*Profile ID - 16264*/
const Value V24663_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_279 = {S1, 0, &V24663_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24663_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_282 = {S3, 0, &V24663_282_57155, &P24663_279};	/*Property = BlobBuilder*/
const Value V24663_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_350 = {S4, 0, &V24663_350_57446, &P24663_282};	/*Property = Canvas*/
const Value V24663_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_283 = {S5, 0, &V24663_283_57161, &P24663_350};	/*Property = CssBackground*/
const Value V24663_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_284 = {S6, 0, &V24663_284_57164, &P24663_283};	/*Property = CssBorderImage*/
const Value V24663_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_285 = {S7, 0, &V24663_285_57167, &P24663_284};	/*Property = CssCanvas*/
const Value V24663_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_286 = {S8, 0, &V24663_286_57170, &P24663_285};	/*Property = CssColor*/
const Value V24663_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_237 = {S9, 0, &V24663_237_55786, &P24663_286};	/*Property = CssColumn*/
const Value V24663_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_287 = {S10, 0, &V24663_287_57173, &P24663_237};	/*Property = CssFlexbox*/
const Value V24663_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_288 = {S11, 0, &V24663_288_57176, &P24663_287};	/*Property = CssFont*/
const Value V24663_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_289 = {S12, 0, &V24663_289_86899, &P24663_288};	/*Property = CssImages*/
const Value V24663_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_291 = {S13, 0, &V24663_291_57182, &P24663_289};	/*Property = CssMediaQueries*/
const Value V24663_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_292 = {S14, 0, &V24663_292_57185, &P24663_291};	/*Property = CssMinMax*/
const Value V24663_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_293 = {S15, 0, &V24663_293_57188, &P24663_292};	/*Property = CssOverflow*/
const Value V24663_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_294 = {S16, 0, &V24663_294_57191, &P24663_293};	/*Property = CssPosition*/
const Value V24663_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_295 = {S17, 0, &V24663_295_57194, &P24663_294};	/*Property = CssText*/
const Value V24663_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_297 = {S18, 0, &V24663_297_57197, &P24663_295};	/*Property = CssTransforms*/
const Value V24663_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_241 = {S19, 0, &V24663_241_55801, &P24663_297};	/*Property = CssTransitions*/
const Value V24663_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_298 = {S20, 0, &V24663_298_57208, &P24663_241};	/*Property = CssUI*/
const Value V24663_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_300 = {S21, 0, &V24663_300_57217, &P24663_298};	/*Property = DataSet*/
const Value V24663_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_301 = {S22, 0, &V24663_301_57220, &P24663_300};	/*Property = DataUrl*/
const Value V24663_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_302 = {S23, 0, &V24663_302_57223, &P24663_301};	/*Property = DeviceOrientation*/
const Value V24663_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_304 = {S24, 0, &V24663_304_57229, &P24663_302};	/*Property = FileReader*/
const Value V24663_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_305 = {S25, 0, &V24663_305_57232, &P24663_304};	/*Property = FileSaver*/
const Value V24663_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_306 = {S26, 0, &V24663_306_57238, &P24663_305};	/*Property = FileWriter*/
const Value V24663_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_307 = {S27, 0, &V24663_307_57241, &P24663_306};	/*Property = FormData*/
const Value V24663_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_309 = {S28, 0, &V24663_309_57247, &P24663_307};	/*Property = Fullscreen*/
const Value V24663_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_310 = {S29, 0, &V24663_310_57250, &P24663_309};	/*Property = GeoLocation*/
const Value V24663_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_312 = {S30, 0, &V24663_312_57256, &P24663_310};	/*Property = History*/
const Value V24663_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_314 = {S31, 0, &V24663_314_57262, &P24663_312};	/*Property = Html5*/
const Value V24663_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_313 = {S32, 0, &V24663_313_57259, &P24663_314};	/*Property = Html-Media-Capture*/
const Value V24663_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_315 = {S33, 0, &V24663_315_57265, &P24663_313};	/*Property = Iframe*/
const Value V24663_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_316 = {S34, 0, &V24663_316_57268, &P24663_315};	/*Property = IndexedDB*/
const Value V24663_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_317 = {S35, 0, &V24663_317_57271, &P24663_316};	/*Property = Json*/
const Value V24663_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24663_15 = {S37, 0, &V24663_15_90837, &P24663_317};	/*Property = LayoutEngine*/
const Value V24663_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_344 = {S38, 0, &V24663_344_57274, &P24663_15};	/*Property = Masking*/
const Value V24663_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_320 = {S39, 0, &V24663_320_57292, &P24663_344};	/*Property = PostMessage*/
const Value V24663_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_321 = {S40, 0, &V24663_321_57295, &P24663_320};	/*Property = Progress*/
const Value V24663_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_322 = {S41, 0, &V24663_322_57298, &P24663_321};	/*Property = Prompts*/
const Value V24663_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_323 = {S42, 0, &V24663_323_57301, &P24663_322};	/*Property = Selector*/
const Value V24663_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_324 = {S43, 0, &V24663_324_57307, &P24663_323};	/*Property = Svg*/
const Value V24663_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_327 = {S44, 0, &V24663_327_57316, &P24663_324};	/*Property = TouchEvents*/
const Value V24663_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_328 = {S45, 0, &V24663_328_57319, &P24663_327};	/*Property = Track*/
const Value V24663_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_329 = {S46, 0, &V24663_329_57325, &P24663_328};	/*Property = Video*/
const Value V24663_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_330 = {S47, 0, &V24663_330_57328, &P24663_329};	/*Property = Viewport*/
const Value V24663_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_332 = {S48, 0, &V24663_332_57340, &P24663_330};	/*Property = WebWorkers*/
const Value V24663_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24663_333 = {S49, 0, &V24663_333_57343, &P24663_332};	/*Property = Xhr2*/
const Profile P24663 = {"24663", &P24663_333};	/*Profile ID - 24663*/
const Value V20272_1_2 = {S0, NULL};	/*Value = False*/
const Property P20272_1 = {S50, 0, &V20272_1_2, NULL};	/*Property = IsMobile*/
const Value V20272_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20272_5 = {S52, 0, &V20272_5_37, &P20272_1};	/*Property = ScreenPixelsHeight*/
const Value V20272_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20272_4 = {S54, 0, &V20272_4_36, &P20272_5};	/*Property = ScreenPixelsWidth*/
const Profile P20272 = {"20272", &P20272_4};	/*Profile ID - 20272*/
const Value V17768_1_1 = {S2, NULL};	/*Value = True*/
const Property P17768_1 = {S50, 0, &V17768_1_1, NULL};	/*Property = IsMobile*/
const Value V17768_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17768_5 = {S52, 0, &V17768_5_1222, &P17768_1};	/*Property = ScreenPixelsHeight*/
const Value V17768_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17768_4 = {S54, 0, &V17768_4_1224, &P17768_5};	/*Property = ScreenPixelsWidth*/
const Profile P17768 = {"17768", &P17768_4};	/*Profile ID - 17768*/
const Value V13692_1_1 = {S2, NULL};	/*Value = True*/
const Property P13692_1 = {S50, 0, &V13692_1_1, NULL};	/*Property = IsMobile*/
const Value V13692_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13692_5 = {S52, 0, &V13692_5_1233, &P13692_1};	/*Property = ScreenPixelsHeight*/
const Value V13692_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13692_4 = {S54, 0, &V13692_4_1235, &P13692_5};	/*Property = ScreenPixelsWidth*/
const Profile P13692 = {"13692", &P13692_4};	/*Profile ID - 13692*/
const Value V13694_1_1 = {S2, NULL};	/*Value = True*/
const Property P13694_1 = {S50, 0, &V13694_1_1, NULL};	/*Property = IsMobile*/
const Value V13694_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13694_5 = {S52, 0, &V13694_5_1293, &P13694_1};	/*Property = ScreenPixelsHeight*/
const Value V13694_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13694_4 = {S54, 0, &V13694_4_1296, &P13694_5};	/*Property = ScreenPixelsWidth*/
const Profile P13694 = {"13694", &P13694_4};	/*Profile ID - 13694*/
const Value V13697_1_1 = {S2, NULL};	/*Value = True*/
const Property P13697_1 = {S50, 0, &V13697_1_1, NULL};	/*Property = IsMobile*/
const Value V13697_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13697_5 = {S52, 0, &V13697_5_1222, &P13697_1};	/*Property = ScreenPixelsHeight*/
const Value V13697_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13697_4 = {S54, 0, &V13697_4_1224, &P13697_5};	/*Property = ScreenPixelsWidth*/
const Profile P13697 = {"13697", &P13697_4};	/*Profile ID - 13697*/
const Value V12695_1_1 = {S2, NULL};	/*Value = True*/
const Property P12695_1 = {S50, 0, &V12695_1_1, NULL};	/*Property = IsMobile*/
const Value V12695_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P12695_5 = {S52, 0, &V12695_5_1403, &P12695_1};	/*Property = ScreenPixelsHeight*/
const Value V12695_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P12695_4 = {S54, 0, &V12695_4_1404, &P12695_5};	/*Property = ScreenPixelsWidth*/
const Profile P12695 = {"12695", &P12695_4};	/*Profile ID - 12695*/
const Value V16451_1_1 = {S2, NULL};	/*Value = True*/
const Property P16451_1 = {S50, 0, &V16451_1_1, NULL};	/*Property = IsMobile*/
const Value V16451_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16451_5 = {S52, 0, &V16451_5_1294, &P16451_1};	/*Property = ScreenPixelsHeight*/
const Value V16451_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16451_4 = {S54, 0, &V16451_4_1296, &P16451_5};	/*Property = ScreenPixelsWidth*/
const Profile P16451 = {"16451", &P16451_4};	/*Profile ID - 16451*/
const Value V17497_1_1 = {S2, NULL};	/*Value = True*/
const Property P17497_1 = {S50, 0, &V17497_1_1, NULL};	/*Property = IsMobile*/
const Value V17497_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17497_5 = {S52, 0, &V17497_5_1222, &P17497_1};	/*Property = ScreenPixelsHeight*/
const Value V17497_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17497_4 = {S54, 0, &V17497_4_1224, &P17497_5};	/*Property = ScreenPixelsWidth*/
const Profile P17497 = {"17497", &P17497_4};	/*Profile ID - 17497*/
const Value V20772_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_279 = {S1, 0, &V20772_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20772_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_282 = {S3, 0, &V20772_282_57155, &P20772_279};	/*Property = BlobBuilder*/
const Value V20772_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_350 = {S4, 0, &V20772_350_57446, &P20772_282};	/*Property = Canvas*/
const Value V20772_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_283 = {S5, 0, &V20772_283_57161, &P20772_350};	/*Property = CssBackground*/
const Value V20772_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_284 = {S6, 0, &V20772_284_57164, &P20772_283};	/*Property = CssBorderImage*/
const Value V20772_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_285 = {S7, 0, &V20772_285_57167, &P20772_284};	/*Property = CssCanvas*/
const Value V20772_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_286 = {S8, 0, &V20772_286_57170, &P20772_285};	/*Property = CssColor*/
const Value V20772_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_237 = {S9, 0, &V20772_237_55786, &P20772_286};	/*Property = CssColumn*/
const Value V20772_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_287 = {S10, 0, &V20772_287_57173, &P20772_237};	/*Property = CssFlexbox*/
const Value V20772_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_288 = {S11, 0, &V20772_288_57176, &P20772_287};	/*Property = CssFont*/
const Value V20772_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_289 = {S12, 0, &V20772_289_86899, &P20772_288};	/*Property = CssImages*/
const Value V20772_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_291 = {S13, 0, &V20772_291_57182, &P20772_289};	/*Property = CssMediaQueries*/
const Value V20772_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_292 = {S14, 0, &V20772_292_57185, &P20772_291};	/*Property = CssMinMax*/
const Value V20772_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_293 = {S15, 0, &V20772_293_57188, &P20772_292};	/*Property = CssOverflow*/
const Value V20772_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_294 = {S16, 0, &V20772_294_57191, &P20772_293};	/*Property = CssPosition*/
const Value V20772_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_295 = {S17, 0, &V20772_295_57194, &P20772_294};	/*Property = CssText*/
const Value V20772_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_297 = {S18, 0, &V20772_297_57197, &P20772_295};	/*Property = CssTransforms*/
const Value V20772_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_241 = {S19, 0, &V20772_241_55801, &P20772_297};	/*Property = CssTransitions*/
const Value V20772_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_298 = {S20, 0, &V20772_298_57208, &P20772_241};	/*Property = CssUI*/
const Value V20772_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_300 = {S21, 0, &V20772_300_57217, &P20772_298};	/*Property = DataSet*/
const Value V20772_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_301 = {S22, 0, &V20772_301_57220, &P20772_300};	/*Property = DataUrl*/
const Value V20772_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_302 = {S23, 0, &V20772_302_57223, &P20772_301};	/*Property = DeviceOrientation*/
const Value V20772_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_304 = {S24, 0, &V20772_304_57229, &P20772_302};	/*Property = FileReader*/
const Value V20772_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_305 = {S25, 0, &V20772_305_57232, &P20772_304};	/*Property = FileSaver*/
const Value V20772_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_306 = {S26, 0, &V20772_306_57238, &P20772_305};	/*Property = FileWriter*/
const Value V20772_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_307 = {S27, 0, &V20772_307_57241, &P20772_306};	/*Property = FormData*/
const Value V20772_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_309 = {S28, 0, &V20772_309_57247, &P20772_307};	/*Property = Fullscreen*/
const Value V20772_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_310 = {S29, 0, &V20772_310_57250, &P20772_309};	/*Property = GeoLocation*/
const Value V20772_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_312 = {S30, 0, &V20772_312_57256, &P20772_310};	/*Property = History*/
const Value V20772_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_314 = {S31, 0, &V20772_314_57262, &P20772_312};	/*Property = Html5*/
const Value V20772_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_313 = {S32, 0, &V20772_313_57259, &P20772_314};	/*Property = Html-Media-Capture*/
const Value V20772_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_315 = {S33, 0, &V20772_315_57265, &P20772_313};	/*Property = Iframe*/
const Value V20772_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_316 = {S34, 0, &V20772_316_57268, &P20772_315};	/*Property = IndexedDB*/
const Value V20772_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_317 = {S35, 0, &V20772_317_57271, &P20772_316};	/*Property = Json*/
const Value V20772_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P20772_15 = {S37, 0, &V20772_15_54167, &P20772_317};	/*Property = LayoutEngine*/
const Value V20772_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_344 = {S38, 0, &V20772_344_57274, &P20772_15};	/*Property = Masking*/
const Value V20772_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_320 = {S39, 0, &V20772_320_57292, &P20772_344};	/*Property = PostMessage*/
const Value V20772_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_321 = {S40, 0, &V20772_321_57295, &P20772_320};	/*Property = Progress*/
const Value V20772_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_322 = {S41, 0, &V20772_322_57298, &P20772_321};	/*Property = Prompts*/
const Value V20772_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_323 = {S42, 0, &V20772_323_57301, &P20772_322};	/*Property = Selector*/
const Value V20772_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_324 = {S43, 0, &V20772_324_57307, &P20772_323};	/*Property = Svg*/
const Value V20772_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_327 = {S44, 0, &V20772_327_57316, &P20772_324};	/*Property = TouchEvents*/
const Value V20772_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_328 = {S45, 0, &V20772_328_57319, &P20772_327};	/*Property = Track*/
const Value V20772_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_329 = {S46, 0, &V20772_329_57325, &P20772_328};	/*Property = Video*/
const Value V20772_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_330 = {S47, 0, &V20772_330_57328, &P20772_329};	/*Property = Viewport*/
const Value V20772_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_332 = {S48, 0, &V20772_332_57340, &P20772_330};	/*Property = WebWorkers*/
const Value V20772_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20772_333 = {S49, 0, &V20772_333_57343, &P20772_332};	/*Property = Xhr2*/
const Profile P20772 = {"20772", &P20772_333};	/*Profile ID - 20772*/
const Value V14861_1_1 = {S2, NULL};	/*Value = True*/
const Property P14861_1 = {S50, 0, &V14861_1_1, NULL};	/*Property = IsMobile*/
const Value V14861_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14861_5 = {S52, 0, &V14861_5_1277, &P14861_1};	/*Property = ScreenPixelsHeight*/
const Value V14861_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14861_4 = {S54, 0, &V14861_4_1279, &P14861_5};	/*Property = ScreenPixelsWidth*/
const Profile P14861 = {"14861", &P14861_4};	/*Profile ID - 14861*/
const Value V13861_1_1 = {S2, NULL};	/*Value = True*/
const Property P13861_1 = {S50, 0, &V13861_1_1, NULL};	/*Property = IsMobile*/
const Value V13861_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13861_5 = {S52, 0, &V13861_5_1222, &P13861_1};	/*Property = ScreenPixelsHeight*/
const Value V13861_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13861_4 = {S54, 0, &V13861_4_1224, &P13861_5};	/*Property = ScreenPixelsWidth*/
const Profile P13861 = {"13861", &P13861_4};	/*Profile ID - 13861*/
const Value V13741_1_1 = {S2, NULL};	/*Value = True*/
const Property P13741_1 = {S50, 0, &V13741_1_1, NULL};	/*Property = IsMobile*/
const Value V13741_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13741_5 = {S52, 0, &V13741_5_1293, &P13741_1};	/*Property = ScreenPixelsHeight*/
const Value V13741_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13741_4 = {S54, 0, &V13741_4_1296, &P13741_5};	/*Property = ScreenPixelsWidth*/
const Profile P13741 = {"13741", &P13741_4};	/*Profile ID - 13741*/
const Value V16680_1_1 = {S2, NULL};	/*Value = True*/
const Property P16680_1 = {S50, 0, &V16680_1_1, NULL};	/*Property = IsMobile*/
const Value V16680_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16680_5 = {S52, 0, &V16680_5_1233, &P16680_1};	/*Property = ScreenPixelsHeight*/
const Value V16680_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16680_4 = {S54, 0, &V16680_4_1235, &P16680_5};	/*Property = ScreenPixelsWidth*/
const Profile P16680 = {"16680", &P16680_4};	/*Profile ID - 16680*/
const Value V13758_1_1 = {S2, NULL};	/*Value = True*/
const Property P13758_1 = {S50, 0, &V13758_1_1, NULL};	/*Property = IsMobile*/
const Value V13758_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13758_5 = {S52, 0, &V13758_5_1222, &P13758_1};	/*Property = ScreenPixelsHeight*/
const Value V13758_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13758_4 = {S54, 0, &V13758_4_1224, &P13758_5};	/*Property = ScreenPixelsWidth*/
const Profile P13758 = {"13758", &P13758_4};	/*Profile ID - 13758*/
const Value V13772_1_1 = {S2, NULL};	/*Value = True*/
const Property P13772_1 = {S50, 0, &V13772_1_1, NULL};	/*Property = IsMobile*/
const Value V13772_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13772_5 = {S52, 0, &V13772_5_1222, &P13772_1};	/*Property = ScreenPixelsHeight*/
const Value V13772_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13772_4 = {S54, 0, &V13772_4_1224, &P13772_5};	/*Property = ScreenPixelsWidth*/
const Profile P13772 = {"13772", &P13772_4};	/*Profile ID - 13772*/
const Value V13841_1_1 = {S2, NULL};	/*Value = True*/
const Property P13841_1 = {S50, 0, &V13841_1_1, NULL};	/*Property = IsMobile*/
const Value V13841_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13841_5 = {S52, 0, &V13841_5_1304, &P13841_1};	/*Property = ScreenPixelsHeight*/
const Value V13841_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13841_4 = {S54, 0, &V13841_4_1224, &P13841_5};	/*Property = ScreenPixelsWidth*/
const Profile P13841 = {"13841", &P13841_4};	/*Profile ID - 13841*/
const Value V13971_1_1 = {S2, NULL};	/*Value = True*/
const Property P13971_1 = {S50, 0, &V13971_1_1, NULL};	/*Property = IsMobile*/
const Value V13971_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13971_5 = {S52, 0, &V13971_5_1222, &P13971_1};	/*Property = ScreenPixelsHeight*/
const Value V13971_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13971_4 = {S54, 0, &V13971_4_1224, &P13971_5};	/*Property = ScreenPixelsWidth*/
const Profile P13971 = {"13971", &P13971_4};	/*Profile ID - 13971*/
const Value V13933_1_1 = {S2, NULL};	/*Value = True*/
const Property P13933_1 = {S50, 0, &V13933_1_1, NULL};	/*Property = IsMobile*/
const Value V13933_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13933_5 = {S52, 0, &V13933_5_1293, &P13933_1};	/*Property = ScreenPixelsHeight*/
const Value V13933_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13933_4 = {S54, 0, &V13933_4_1296, &P13933_5};	/*Property = ScreenPixelsWidth*/
const Profile P13933 = {"13933", &P13933_4};	/*Profile ID - 13933*/
const Value V12422_1_1 = {S2, NULL};	/*Value = True*/
const Property P12422_1 = {S50, 0, &V12422_1_1, NULL};	/*Property = IsMobile*/
const Value V12422_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12422_5 = {S52, 0, &V12422_5_1294, &P12422_1};	/*Property = ScreenPixelsHeight*/
const Value V12422_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12422_4 = {S54, 0, &V12422_4_1296, &P12422_5};	/*Property = ScreenPixelsWidth*/
const Profile P12422 = {"12422", &P12422_4};	/*Profile ID - 12422*/
const Value V13948_1_1 = {S2, NULL};	/*Value = True*/
const Property P13948_1 = {S50, 0, &V13948_1_1, NULL};	/*Property = IsMobile*/
const Value V13948_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13948_5 = {S52, 0, &V13948_5_1233, &P13948_1};	/*Property = ScreenPixelsHeight*/
const Value V13948_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13948_4 = {S54, 0, &V13948_4_1235, &P13948_5};	/*Property = ScreenPixelsWidth*/
const Profile P13948 = {"13948", &P13948_4};	/*Profile ID - 13948*/
const Value V13951_1_1 = {S2, NULL};	/*Value = True*/
const Property P13951_1 = {S50, 0, &V13951_1_1, NULL};	/*Property = IsMobile*/
const Value V13951_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13951_5 = {S52, 0, &V13951_5_1293, &P13951_1};	/*Property = ScreenPixelsHeight*/
const Value V13951_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13951_4 = {S54, 0, &V13951_4_1296, &P13951_5};	/*Property = ScreenPixelsWidth*/
const Profile P13951 = {"13951", &P13951_4};	/*Profile ID - 13951*/
const Value V14297_1_1 = {S2, NULL};	/*Value = True*/
const Property P14297_1 = {S50, 0, &V14297_1_1, NULL};	/*Property = IsMobile*/
const Value V14297_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14297_5 = {S52, 0, &V14297_5_1380, &P14297_1};	/*Property = ScreenPixelsHeight*/
const Value V14297_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14297_4 = {S54, 0, &V14297_4_1279, &P14297_5};	/*Property = ScreenPixelsWidth*/
const Profile P14297 = {"14297", &P14297_4};	/*Profile ID - 14297*/
const Value V16439_1_1 = {S2, NULL};	/*Value = True*/
const Property P16439_1 = {S50, 0, &V16439_1_1, NULL};	/*Property = IsMobile*/
const Value V16439_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16439_5 = {S52, 0, &V16439_5_1294, &P16439_1};	/*Property = ScreenPixelsHeight*/
const Value V16439_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16439_4 = {S54, 0, &V16439_4_1296, &P16439_5};	/*Property = ScreenPixelsWidth*/
const Profile P16439 = {"16439", &P16439_4};	/*Profile ID - 16439*/
const Value V21487_1_1 = {S2, NULL};	/*Value = True*/
const Property P21487_1 = {S50, 0, &V21487_1_1, NULL};	/*Property = IsMobile*/
const Value V21487_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21487_5 = {S52, 0, &V21487_5_1227, &P21487_1};	/*Property = ScreenPixelsHeight*/
const Value V21487_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21487_4 = {S54, 0, &V21487_4_1229, &P21487_5};	/*Property = ScreenPixelsWidth*/
const Profile P21487 = {"21487", &P21487_4};	/*Profile ID - 21487*/
const Value V14223_1_1 = {S2, NULL};	/*Value = True*/
const Property P14223_1 = {S50, 0, &V14223_1_1, NULL};	/*Property = IsMobile*/
const Value V14223_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14223_5 = {S52, 0, &V14223_5_1277, &P14223_1};	/*Property = ScreenPixelsHeight*/
const Value V14223_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14223_4 = {S54, 0, &V14223_4_1279, &P14223_5};	/*Property = ScreenPixelsWidth*/
const Profile P14223 = {"14223", &P14223_4};	/*Profile ID - 14223*/
const Value V13860_1_1 = {S2, NULL};	/*Value = True*/
const Property P13860_1 = {S50, 0, &V13860_1_1, NULL};	/*Property = IsMobile*/
const Value V13860_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13860_5 = {S52, 0, &V13860_5_1380, &P13860_1};	/*Property = ScreenPixelsHeight*/
const Value V13860_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13860_4 = {S54, 0, &V13860_4_1279, &P13860_5};	/*Property = ScreenPixelsWidth*/
const Profile P13860 = {"13860", &P13860_4};	/*Profile ID - 13860*/
const Value V14096_1_1 = {S2, NULL};	/*Value = True*/
const Property P14096_1 = {S50, 0, &V14096_1_1, NULL};	/*Property = IsMobile*/
const Value V14096_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14096_5 = {S52, 0, &V14096_5_1233, &P14096_1};	/*Property = ScreenPixelsHeight*/
const Value V14096_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14096_4 = {S54, 0, &V14096_4_1235, &P14096_5};	/*Property = ScreenPixelsWidth*/
const Profile P14096 = {"14096", &P14096_4};	/*Profile ID - 14096*/
const Value V14101_1_1 = {S2, NULL};	/*Value = True*/
const Property P14101_1 = {S50, 0, &V14101_1_1, NULL};	/*Property = IsMobile*/
const Value V14101_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14101_5 = {S52, 0, &V14101_5_1222, &P14101_1};	/*Property = ScreenPixelsHeight*/
const Value V14101_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14101_4 = {S54, 0, &V14101_4_1224, &P14101_5};	/*Property = ScreenPixelsWidth*/
const Profile P14101 = {"14101", &P14101_4};	/*Profile ID - 14101*/
const Value V14154_1_1 = {S2, NULL};	/*Value = True*/
const Property P14154_1 = {S50, 0, &V14154_1_1, NULL};	/*Property = IsMobile*/
const Value V14154_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14154_5 = {S52, 0, &V14154_5_1222, &P14154_1};	/*Property = ScreenPixelsHeight*/
const Value V14154_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14154_4 = {S54, 0, &V14154_4_1224, &P14154_5};	/*Property = ScreenPixelsWidth*/
const Profile P14154 = {"14154", &P14154_4};	/*Profile ID - 14154*/
const Value V14007_1_1 = {S2, NULL};	/*Value = True*/
const Property P14007_1 = {S50, 0, &V14007_1_1, NULL};	/*Property = IsMobile*/
const Value V14007_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14007_5 = {S52, 0, &V14007_5_1380, &P14007_1};	/*Property = ScreenPixelsHeight*/
const Value V14007_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14007_4 = {S54, 0, &V14007_4_1279, &P14007_5};	/*Property = ScreenPixelsWidth*/
const Profile P14007 = {"14007", &P14007_4};	/*Profile ID - 14007*/
const Value V15881_1_1 = {S2, NULL};	/*Value = True*/
const Property P15881_1 = {S50, 0, &V15881_1_1, NULL};	/*Property = IsMobile*/
const Value V15881_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15881_5 = {S52, 0, &V15881_5_1227, &P15881_1};	/*Property = ScreenPixelsHeight*/
const Value V15881_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15881_4 = {S54, 0, &V15881_4_1229, &P15881_5};	/*Property = ScreenPixelsWidth*/
const Profile P15881 = {"15881", &P15881_4};	/*Profile ID - 15881*/
const Value V13872_1_1 = {S2, NULL};	/*Value = True*/
const Property P13872_1 = {S50, 0, &V13872_1_1, NULL};	/*Property = IsMobile*/
const Value V13872_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13872_5 = {S52, 0, &V13872_5_1222, &P13872_1};	/*Property = ScreenPixelsHeight*/
const Value V13872_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13872_4 = {S54, 0, &V13872_4_1224, &P13872_5};	/*Property = ScreenPixelsWidth*/
const Profile P13872 = {"13872", &P13872_4};	/*Profile ID - 13872*/
const Value V16571_1_1 = {S2, NULL};	/*Value = True*/
const Property P16571_1 = {S50, 0, &V16571_1_1, NULL};	/*Property = IsMobile*/
const Value V16571_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16571_5 = {S52, 0, &V16571_5_1277, &P16571_1};	/*Property = ScreenPixelsHeight*/
const Value V16571_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16571_4 = {S54, 0, &V16571_4_1279, &P16571_5};	/*Property = ScreenPixelsWidth*/
const Profile P16571 = {"16571", &P16571_4};	/*Profile ID - 16571*/
const Value V17782_1_1 = {S2, NULL};	/*Value = True*/
const Property P17782_1 = {S50, 0, &V17782_1_1, NULL};	/*Property = IsMobile*/
const Value V17782_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P17782_5 = {S52, 0, &V17782_5_1275, &P17782_1};	/*Property = ScreenPixelsHeight*/
const Value V17782_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17782_4 = {S54, 0, &V17782_4_1229, &P17782_5};	/*Property = ScreenPixelsWidth*/
const Profile P17782 = {"17782", &P17782_4};	/*Profile ID - 17782*/
const Value V17508_1_1 = {S2, NULL};	/*Value = True*/
const Property P17508_1 = {S50, 0, &V17508_1_1, NULL};	/*Property = IsMobile*/
const Value V17508_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17508_5 = {S52, 0, &V17508_5_1222, &P17508_1};	/*Property = ScreenPixelsHeight*/
const Value V17508_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17508_4 = {S54, 0, &V17508_4_1224, &P17508_5};	/*Property = ScreenPixelsWidth*/
const Profile P17508 = {"17508", &P17508_4};	/*Profile ID - 17508*/
const Value V16467_1_1 = {S2, NULL};	/*Value = True*/
const Property P16467_1 = {S50, 0, &V16467_1_1, NULL};	/*Property = IsMobile*/
const Value V16467_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16467_5 = {S52, 0, &V16467_5_1277, &P16467_1};	/*Property = ScreenPixelsHeight*/
const Value V16467_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16467_4 = {S54, 0, &V16467_4_1279, &P16467_5};	/*Property = ScreenPixelsWidth*/
const Profile P16467 = {"16467", &P16467_4};	/*Profile ID - 16467*/
const Value V15051_1_1 = {S2, NULL};	/*Value = True*/
const Property P15051_1 = {S50, 0, &V15051_1_1, NULL};	/*Property = IsMobile*/
const Value V15051_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15051_5 = {S52, 0, &V15051_5_1293, &P15051_1};	/*Property = ScreenPixelsHeight*/
const Value V15051_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15051_4 = {S54, 0, &V15051_4_1296, &P15051_5};	/*Property = ScreenPixelsWidth*/
const Profile P15051 = {"15051", &P15051_4};	/*Profile ID - 15051*/
const Value V14252_1_1 = {S2, NULL};	/*Value = True*/
const Property P14252_1 = {S50, 0, &V14252_1_1, NULL};	/*Property = IsMobile*/
const Value V14252_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14252_5 = {S52, 0, &V14252_5_1277, &P14252_1};	/*Property = ScreenPixelsHeight*/
const Value V14252_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14252_4 = {S54, 0, &V14252_4_1279, &P14252_5};	/*Property = ScreenPixelsWidth*/
const Profile P14252 = {"14252", &P14252_4};	/*Profile ID - 14252*/
const Value V20390_1_2 = {S0, NULL};	/*Value = False*/
const Property P20390_1 = {S50, 0, &V20390_1_2, NULL};	/*Property = IsMobile*/
const Value V20390_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20390_5 = {S52, 0, &V20390_5_37, &P20390_1};	/*Property = ScreenPixelsHeight*/
const Value V20390_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20390_4 = {S54, 0, &V20390_4_36, &P20390_5};	/*Property = ScreenPixelsWidth*/
const Profile P20390 = {"20390", &P20390_4};	/*Profile ID - 20390*/
const Value V15106_1_1 = {S2, NULL};	/*Value = True*/
const Property P15106_1 = {S50, 0, &V15106_1_1, NULL};	/*Property = IsMobile*/
const Value V15106_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15106_5 = {S52, 0, &V15106_5_1277, &P15106_1};	/*Property = ScreenPixelsHeight*/
const Value V15106_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15106_4 = {S54, 0, &V15106_4_1279, &P15106_5};	/*Property = ScreenPixelsWidth*/
const Profile P15106 = {"15106", &P15106_4};	/*Profile ID - 15106*/
const Value V12931_1_1 = {S2, NULL};	/*Value = True*/
const Property P12931_1 = {S50, 0, &V12931_1_1, NULL};	/*Property = IsMobile*/
const Value V12931_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12931_5 = {S52, 0, &V12931_5_1277, &P12931_1};	/*Property = ScreenPixelsHeight*/
const Value V12931_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12931_4 = {S54, 0, &V12931_4_1279, &P12931_5};	/*Property = ScreenPixelsWidth*/
const Profile P12931 = {"12931", &P12931_4};	/*Profile ID - 12931*/
const Value V17488_1_1 = {S2, NULL};	/*Value = True*/
const Property P17488_1 = {S50, 0, &V17488_1_1, NULL};	/*Property = IsMobile*/
const Value V17488_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17488_5 = {S52, 0, &V17488_5_1222, &P17488_1};	/*Property = ScreenPixelsHeight*/
const Value V17488_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17488_4 = {S54, 0, &V17488_4_1224, &P17488_5};	/*Property = ScreenPixelsWidth*/
const Profile P17488 = {"17488", &P17488_4};	/*Profile ID - 17488*/
const Profile P24389 = {"24389", NULL};	/*Profile ID - 24389*/
const Value V15302_1_1 = {S2, NULL};	/*Value = True*/
const Property P15302_1 = {S50, 0, &V15302_1_1, NULL};	/*Property = IsMobile*/
const Value V15302_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15302_5 = {S52, 0, &V15302_5_1304, &P15302_1};	/*Property = ScreenPixelsHeight*/
const Value V15302_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15302_4 = {S54, 0, &V15302_4_1224, &P15302_5};	/*Property = ScreenPixelsWidth*/
const Profile P15302 = {"15302", &P15302_4};	/*Profile ID - 15302*/
const Value V16224_1_1 = {S2, NULL};	/*Value = True*/
const Property P16224_1 = {S50, 0, &V16224_1_1, NULL};	/*Property = IsMobile*/
const Value V16224_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16224_5 = {S52, 0, &V16224_5_1222, &P16224_1};	/*Property = ScreenPixelsHeight*/
const Value V16224_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16224_4 = {S54, 0, &V16224_4_1224, &P16224_5};	/*Property = ScreenPixelsWidth*/
const Profile P16224 = {"16224", &P16224_4};	/*Profile ID - 16224*/
const Value V13004_1_1 = {S2, NULL};	/*Value = True*/
const Property P13004_1 = {S50, 0, &V13004_1_1, NULL};	/*Property = IsMobile*/
const Value V13004_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P13004_5 = {S52, 0, &V13004_5_1269, &P13004_1};	/*Property = ScreenPixelsHeight*/
const Value V13004_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13004_4 = {S54, 0, &V13004_4_1229, &P13004_5};	/*Property = ScreenPixelsWidth*/
const Profile P13004 = {"13004", &P13004_4};	/*Profile ID - 13004*/
const Value V13699_1_1 = {S2, NULL};	/*Value = True*/
const Property P13699_1 = {S50, 0, &V13699_1_1, NULL};	/*Property = IsMobile*/
const Value V13699_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13699_5 = {S52, 0, &V13699_5_1293, &P13699_1};	/*Property = ScreenPixelsHeight*/
const Value V13699_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13699_4 = {S54, 0, &V13699_4_1296, &P13699_5};	/*Property = ScreenPixelsWidth*/
const Profile P13699 = {"13699", &P13699_4};	/*Profile ID - 13699*/
const Value V13700_1_1 = {S2, NULL};	/*Value = True*/
const Property P13700_1 = {S50, 0, &V13700_1_1, NULL};	/*Property = IsMobile*/
const Value V13700_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13700_5 = {S52, 0, &V13700_5_1380, &P13700_1};	/*Property = ScreenPixelsHeight*/
const Value V13700_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13700_4 = {S54, 0, &V13700_4_1279, &P13700_5};	/*Property = ScreenPixelsWidth*/
const Profile P13700 = {"13700", &P13700_4};	/*Profile ID - 13700*/
const Value V13865_1_1 = {S2, NULL};	/*Value = True*/
const Property P13865_1 = {S50, 0, &V13865_1_1, NULL};	/*Property = IsMobile*/
const Value V13865_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13865_5 = {S52, 0, &V13865_5_1294, &P13865_1};	/*Property = ScreenPixelsHeight*/
const Value V13865_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13865_4 = {S54, 0, &V13865_4_1296, &P13865_5};	/*Property = ScreenPixelsWidth*/
const Profile P13865 = {"13865", &P13865_4};	/*Profile ID - 13865*/
const Value V14124_1_1 = {S2, NULL};	/*Value = True*/
const Property P14124_1 = {S50, 0, &V14124_1_1, NULL};	/*Property = IsMobile*/
const Value V14124_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14124_5 = {S52, 0, &V14124_5_1222, &P14124_1};	/*Property = ScreenPixelsHeight*/
const Value V14124_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14124_4 = {S54, 0, &V14124_4_1224, &P14124_5};	/*Property = ScreenPixelsWidth*/
const Profile P14124 = {"14124", &P14124_4};	/*Profile ID - 14124*/
const Profile P5107 = {"5107", NULL};	/*Profile ID - 5107*/
const Value V13707_1_1 = {S2, NULL};	/*Value = True*/
const Property P13707_1 = {S50, 0, &V13707_1_1, NULL};	/*Property = IsMobile*/
const Value V13707_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13707_5 = {S52, 0, &V13707_5_1222, &P13707_1};	/*Property = ScreenPixelsHeight*/
const Value V13707_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13707_4 = {S54, 0, &V13707_4_1279, &P13707_5};	/*Property = ScreenPixelsWidth*/
const Profile P13707 = {"13707", &P13707_4};	/*Profile ID - 13707*/
const Value V13709_1_1 = {S2, NULL};	/*Value = True*/
const Property P13709_1 = {S50, 0, &V13709_1_1, NULL};	/*Property = IsMobile*/
const Value V13709_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13709_5 = {S52, 0, &V13709_5_1233, &P13709_1};	/*Property = ScreenPixelsHeight*/
const Value V13709_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13709_4 = {S54, 0, &V13709_4_1235, &P13709_5};	/*Property = ScreenPixelsWidth*/
const Profile P13709 = {"13709", &P13709_4};	/*Profile ID - 13709*/
const Value V13716_1_1 = {S2, NULL};	/*Value = True*/
const Property P13716_1 = {S50, 0, &V13716_1_1, NULL};	/*Property = IsMobile*/
const Value V13716_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13716_5 = {S52, 0, &V13716_5_1233, &P13716_1};	/*Property = ScreenPixelsHeight*/
const Value V13716_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13716_4 = {S54, 0, &V13716_4_1235, &P13716_5};	/*Property = ScreenPixelsWidth*/
const Profile P13716 = {"13716", &P13716_4};	/*Profile ID - 13716*/
const Value V13715_1_1 = {S2, NULL};	/*Value = True*/
const Property P13715_1 = {S50, 0, &V13715_1_1, NULL};	/*Property = IsMobile*/
const Value V13715_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P13715_5 = {S52, 0, &V13715_5_1832, &P13715_1};	/*Property = ScreenPixelsHeight*/
const Value V13715_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P13715_4 = {S54, 0, &V13715_4_2336, &P13715_5};	/*Property = ScreenPixelsWidth*/
const Profile P13715 = {"13715", &P13715_4};	/*Profile ID - 13715*/
const Value V13714_1_1 = {S2, NULL};	/*Value = True*/
const Property P13714_1 = {S50, 0, &V13714_1_1, NULL};	/*Property = IsMobile*/
const Value V13714_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13714_5 = {S52, 0, &V13714_5_1222, &P13714_1};	/*Property = ScreenPixelsHeight*/
const Value V13714_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13714_4 = {S54, 0, &V13714_4_1224, &P13714_5};	/*Property = ScreenPixelsWidth*/
const Profile P13714 = {"13714", &P13714_4};	/*Profile ID - 13714*/
const Value V12076_1_1 = {S2, NULL};	/*Value = True*/
const Property P12076_1 = {S50, 0, &V12076_1_1, NULL};	/*Property = IsMobile*/
const Value V12076_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P12076_5 = {S52, 0, &V12076_5_1333, &P12076_1};	/*Property = ScreenPixelsHeight*/
const Value V12076_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P12076_4 = {S54, 0, &V12076_4_1335, &P12076_5};	/*Property = ScreenPixelsWidth*/
const Profile P12076 = {"12076", &P12076_4};	/*Profile ID - 12076*/
const Value V13718_1_1 = {S2, NULL};	/*Value = True*/
const Property P13718_1 = {S50, 0, &V13718_1_1, NULL};	/*Property = IsMobile*/
const Value V13718_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13718_5 = {S52, 0, &V13718_5_1233, &P13718_1};	/*Property = ScreenPixelsHeight*/
const Value V13718_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13718_4 = {S54, 0, &V13718_4_1235, &P13718_5};	/*Property = ScreenPixelsWidth*/
const Profile P13718 = {"13718", &P13718_4};	/*Profile ID - 13718*/
const Value V13720_1_1 = {S2, NULL};	/*Value = True*/
const Property P13720_1 = {S50, 0, &V13720_1_1, NULL};	/*Property = IsMobile*/
const Value V13720_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13720_5 = {S52, 0, &V13720_5_1222, &P13720_1};	/*Property = ScreenPixelsHeight*/
const Value V13720_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13720_4 = {S54, 0, &V13720_4_1224, &P13720_5};	/*Property = ScreenPixelsWidth*/
const Profile P13720 = {"13720", &P13720_4};	/*Profile ID - 13720*/
const Value V13723_1_1 = {S2, NULL};	/*Value = True*/
const Property P13723_1 = {S50, 0, &V13723_1_1, NULL};	/*Property = IsMobile*/
const Value V13723_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13723_5 = {S52, 0, &V13723_5_1380, &P13723_1};	/*Property = ScreenPixelsHeight*/
const Value V13723_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P13723_4 = {S54, 0, &V13723_4_1943, &P13723_5};	/*Property = ScreenPixelsWidth*/
const Profile P13723 = {"13723", &P13723_4};	/*Profile ID - 13723*/
const Value V13725_1_1 = {S2, NULL};	/*Value = True*/
const Property P13725_1 = {S50, 0, &V13725_1_1, NULL};	/*Property = IsMobile*/
const Value V13725_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13725_5 = {S52, 0, &V13725_5_1293, &P13725_1};	/*Property = ScreenPixelsHeight*/
const Value V13725_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13725_4 = {S54, 0, &V13725_4_1296, &P13725_5};	/*Property = ScreenPixelsWidth*/
const Profile P13725 = {"13725", &P13725_4};	/*Profile ID - 13725*/
const Value V13727_1_1 = {S2, NULL};	/*Value = True*/
const Property P13727_1 = {S50, 0, &V13727_1_1, NULL};	/*Property = IsMobile*/
const Value V13727_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13727_5 = {S52, 0, &V13727_5_1304, &P13727_1};	/*Property = ScreenPixelsHeight*/
const Value V13727_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13727_4 = {S54, 0, &V13727_4_1224, &P13727_5};	/*Property = ScreenPixelsWidth*/
const Profile P13727 = {"13727", &P13727_4};	/*Profile ID - 13727*/
const Value V13728_1_1 = {S2, NULL};	/*Value = True*/
const Property P13728_1 = {S50, 0, &V13728_1_1, NULL};	/*Property = IsMobile*/
const Value V13728_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13728_5 = {S52, 0, &V13728_5_1222, &P13728_1};	/*Property = ScreenPixelsHeight*/
const Value V13728_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13728_4 = {S54, 0, &V13728_4_1224, &P13728_5};	/*Property = ScreenPixelsWidth*/
const Profile P13728 = {"13728", &P13728_4};	/*Profile ID - 13728*/
const Value V16202_1_1 = {S2, NULL};	/*Value = True*/
const Property P16202_1 = {S50, 0, &V16202_1_1, NULL};	/*Property = IsMobile*/
const Value V16202_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16202_5 = {S52, 0, &V16202_5_1222, &P16202_1};	/*Property = ScreenPixelsHeight*/
const Value V16202_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16202_4 = {S54, 0, &V16202_4_1224, &P16202_5};	/*Property = ScreenPixelsWidth*/
const Profile P16202 = {"16202", &P16202_4};	/*Profile ID - 16202*/
const Value V13734_1_1 = {S2, NULL};	/*Value = True*/
const Property P13734_1 = {S50, 0, &V13734_1_1, NULL};	/*Property = IsMobile*/
const Value V13734_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13734_5 = {S52, 0, &V13734_5_1222, &P13734_1};	/*Property = ScreenPixelsHeight*/
const Value V13734_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13734_4 = {S54, 0, &V13734_4_1224, &P13734_5};	/*Property = ScreenPixelsWidth*/
const Profile P13734 = {"13734", &P13734_4};	/*Profile ID - 13734*/
const Value V13735_1_1 = {S2, NULL};	/*Value = True*/
const Property P13735_1 = {S50, 0, &V13735_1_1, NULL};	/*Property = IsMobile*/
const Value V13735_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13735_5 = {S52, 0, &V13735_5_1222, &P13735_1};	/*Property = ScreenPixelsHeight*/
const Value V13735_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13735_4 = {S54, 0, &V13735_4_1224, &P13735_5};	/*Property = ScreenPixelsWidth*/
const Profile P13735 = {"13735", &P13735_4};	/*Profile ID - 13735*/
const Value V12545_1_1 = {S2, NULL};	/*Value = True*/
const Property P12545_1 = {S50, 0, &V12545_1_1, NULL};	/*Property = IsMobile*/
const Value V12545_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12545_5 = {S52, 0, &V12545_5_1233, &P12545_1};	/*Property = ScreenPixelsHeight*/
const Value V12545_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12545_4 = {S54, 0, &V12545_4_1235, &P12545_5};	/*Property = ScreenPixelsWidth*/
const Profile P12545 = {"12545", &P12545_4};	/*Profile ID - 12545*/
const Value V13737_1_1 = {S2, NULL};	/*Value = True*/
const Property P13737_1 = {S50, 0, &V13737_1_1, NULL};	/*Property = IsMobile*/
const Value V13737_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13737_5 = {S52, 0, &V13737_5_1233, &P13737_1};	/*Property = ScreenPixelsHeight*/
const Value V13737_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13737_4 = {S54, 0, &V13737_4_1235, &P13737_5};	/*Property = ScreenPixelsWidth*/
const Profile P13737 = {"13737", &P13737_4};	/*Profile ID - 13737*/
const Value V13738_1_1 = {S2, NULL};	/*Value = True*/
const Property P13738_1 = {S50, 0, &V13738_1_1, NULL};	/*Property = IsMobile*/
const Value V13738_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13738_5 = {S52, 0, &V13738_5_1222, &P13738_1};	/*Property = ScreenPixelsHeight*/
const Value V13738_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13738_4 = {S54, 0, &V13738_4_1279, &P13738_5};	/*Property = ScreenPixelsWidth*/
const Profile P13738 = {"13738", &P13738_4};	/*Profile ID - 13738*/
const Value V13743_1_1 = {S2, NULL};	/*Value = True*/
const Property P13743_1 = {S50, 0, &V13743_1_1, NULL};	/*Property = IsMobile*/
const Value V13743_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13743_5 = {S52, 0, &V13743_5_1293, &P13743_1};	/*Property = ScreenPixelsHeight*/
const Value V13743_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13743_4 = {S54, 0, &V13743_4_1296, &P13743_5};	/*Property = ScreenPixelsWidth*/
const Profile P13743 = {"13743", &P13743_4};	/*Profile ID - 13743*/
const Value V13742_1_1 = {S2, NULL};	/*Value = True*/
const Property P13742_1 = {S50, 0, &V13742_1_1, NULL};	/*Property = IsMobile*/
const Value V13742_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13742_5 = {S52, 0, &V13742_5_1380, &P13742_1};	/*Property = ScreenPixelsHeight*/
const Value V13742_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13742_4 = {S54, 0, &V13742_4_1279, &P13742_5};	/*Property = ScreenPixelsWidth*/
const Profile P13742 = {"13742", &P13742_4};	/*Profile ID - 13742*/
const Value V13745_1_1 = {S2, NULL};	/*Value = True*/
const Property P13745_1 = {S50, 0, &V13745_1_1, NULL};	/*Property = IsMobile*/
const Value V13745_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13745_5 = {S52, 0, &V13745_5_1222, &P13745_1};	/*Property = ScreenPixelsHeight*/
const Value V13745_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13745_4 = {S54, 0, &V13745_4_1224, &P13745_5};	/*Property = ScreenPixelsWidth*/
const Profile P13745 = {"13745", &P13745_4};	/*Profile ID - 13745*/
const Value V13751_1_1 = {S2, NULL};	/*Value = True*/
const Property P13751_1 = {S50, 0, &V13751_1_1, NULL};	/*Property = IsMobile*/
const Value V13751_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13751_5 = {S52, 0, &V13751_5_1380, &P13751_1};	/*Property = ScreenPixelsHeight*/
const Value V13751_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13751_4 = {S54, 0, &V13751_4_1279, &P13751_5};	/*Property = ScreenPixelsWidth*/
const Profile P13751 = {"13751", &P13751_4};	/*Profile ID - 13751*/
const Value V13753_1_1 = {S2, NULL};	/*Value = True*/
const Property P13753_1 = {S50, 0, &V13753_1_1, NULL};	/*Property = IsMobile*/
const Value V13753_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13753_5 = {S52, 0, &V13753_5_1222, &P13753_1};	/*Property = ScreenPixelsHeight*/
const Value V13753_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13753_4 = {S54, 0, &V13753_4_1224, &P13753_5};	/*Property = ScreenPixelsWidth*/
const Profile P13753 = {"13753", &P13753_4};	/*Profile ID - 13753*/
const Value V13754_1_1 = {S2, NULL};	/*Value = True*/
const Property P13754_1 = {S50, 0, &V13754_1_1, NULL};	/*Property = IsMobile*/
const Value V13754_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13754_5 = {S52, 0, &V13754_5_1222, &P13754_1};	/*Property = ScreenPixelsHeight*/
const Value V13754_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13754_4 = {S54, 0, &V13754_4_1224, &P13754_5};	/*Property = ScreenPixelsWidth*/
const Profile P13754 = {"13754", &P13754_4};	/*Profile ID - 13754*/
const Value V13766_1_1 = {S2, NULL};	/*Value = True*/
const Property P13766_1 = {S50, 0, &V13766_1_1, NULL};	/*Property = IsMobile*/
const Value V13766_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13766_5 = {S52, 0, &V13766_5_1293, &P13766_1};	/*Property = ScreenPixelsHeight*/
const Value V13766_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13766_4 = {S54, 0, &V13766_4_1296, &P13766_5};	/*Property = ScreenPixelsWidth*/
const Profile P13766 = {"13766", &P13766_4};	/*Profile ID - 13766*/
const Value V15599_1_1 = {S2, NULL};	/*Value = True*/
const Property P15599_1 = {S50, 0, &V15599_1_1, NULL};	/*Property = IsMobile*/
const Value V15599_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P15599_5 = {S52, 0, &V15599_5_1333, &P15599_1};	/*Property = ScreenPixelsHeight*/
const Value V15599_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P15599_4 = {S54, 0, &V15599_4_1335, &P15599_5};	/*Property = ScreenPixelsWidth*/
const Profile P15599 = {"15599", &P15599_4};	/*Profile ID - 15599*/
const Value V15674_1_1 = {S2, NULL};	/*Value = True*/
const Property P15674_1 = {S50, 0, &V15674_1_1, NULL};	/*Property = IsMobile*/
const Value V15674_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15674_5 = {S52, 0, &V15674_5_1294, &P15674_1};	/*Property = ScreenPixelsHeight*/
const Value V15674_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15674_4 = {S54, 0, &V15674_4_1296, &P15674_5};	/*Property = ScreenPixelsWidth*/
const Profile P15674 = {"15674", &P15674_4};	/*Profile ID - 15674*/
const Value V13773_1_1 = {S2, NULL};	/*Value = True*/
const Property P13773_1 = {S50, 0, &V13773_1_1, NULL};	/*Property = IsMobile*/
const Value V13773_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13773_5 = {S52, 0, &V13773_5_1233, &P13773_1};	/*Property = ScreenPixelsHeight*/
const Value V13773_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13773_4 = {S54, 0, &V13773_4_1235, &P13773_5};	/*Property = ScreenPixelsWidth*/
const Profile P13773 = {"13773", &P13773_4};	/*Profile ID - 13773*/
const Value V17752_1_1 = {S2, NULL};	/*Value = True*/
const Property P17752_1 = {S50, 0, &V17752_1_1, NULL};	/*Property = IsMobile*/
const Value V17752_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P17752_5 = {S52, 0, &V17752_5_1294, &P17752_1};	/*Property = ScreenPixelsHeight*/
const Value V17752_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17752_4 = {S54, 0, &V17752_4_1296, &P17752_5};	/*Property = ScreenPixelsWidth*/
const Profile P17752 = {"17752", &P17752_4};	/*Profile ID - 17752*/
const Value V13774_1_1 = {S2, NULL};	/*Value = True*/
const Property P13774_1 = {S50, 0, &V13774_1_1, NULL};	/*Property = IsMobile*/
const Value V13774_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13774_5 = {S52, 0, &V13774_5_1222, &P13774_1};	/*Property = ScreenPixelsHeight*/
const Value V13774_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13774_4 = {S54, 0, &V13774_4_1224, &P13774_5};	/*Property = ScreenPixelsWidth*/
const Profile P13774 = {"13774", &P13774_4};	/*Profile ID - 13774*/
const Value V14838_1_1 = {S2, NULL};	/*Value = True*/
const Property P14838_1 = {S50, 0, &V14838_1_1, NULL};	/*Property = IsMobile*/
const Value V14838_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14838_5 = {S52, 0, &V14838_5_1304, &P14838_1};	/*Property = ScreenPixelsHeight*/
const Value V14838_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14838_4 = {S54, 0, &V14838_4_1224, &P14838_5};	/*Property = ScreenPixelsWidth*/
const Profile P14838 = {"14838", &P14838_4};	/*Profile ID - 14838*/
const Value V13778_1_1 = {S2, NULL};	/*Value = True*/
const Property P13778_1 = {S50, 0, &V13778_1_1, NULL};	/*Property = IsMobile*/
const Value V13778_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13778_5 = {S52, 0, &V13778_5_1222, &P13778_1};	/*Property = ScreenPixelsHeight*/
const Value V13778_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13778_4 = {S54, 0, &V13778_4_1224, &P13778_5};	/*Property = ScreenPixelsWidth*/
const Profile P13778 = {"13778", &P13778_4};	/*Profile ID - 13778*/
const Value V13786_1_1 = {S2, NULL};	/*Value = True*/
const Property P13786_1 = {S50, 0, &V13786_1_1, NULL};	/*Property = IsMobile*/
const Value V13786_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13786_5 = {S52, 0, &V13786_5_1304, &P13786_1};	/*Property = ScreenPixelsHeight*/
const Value V13786_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13786_4 = {S54, 0, &V13786_4_1224, &P13786_5};	/*Property = ScreenPixelsWidth*/
const Profile P13786 = {"13786", &P13786_4};	/*Profile ID - 13786*/
const Value V13785_1_1 = {S2, NULL};	/*Value = True*/
const Property P13785_1 = {S50, 0, &V13785_1_1, NULL};	/*Property = IsMobile*/
const Value V13785_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13785_5 = {S52, 0, &V13785_5_1380, &P13785_1};	/*Property = ScreenPixelsHeight*/
const Value V13785_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13785_4 = {S54, 0, &V13785_4_1279, &P13785_5};	/*Property = ScreenPixelsWidth*/
const Profile P13785 = {"13785", &P13785_4};	/*Profile ID - 13785*/
const Value V13790_1_1 = {S2, NULL};	/*Value = True*/
const Property P13790_1 = {S50, 0, &V13790_1_1, NULL};	/*Property = IsMobile*/
const Value V13790_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13790_5 = {S52, 0, &V13790_5_1304, &P13790_1};	/*Property = ScreenPixelsHeight*/
const Value V13790_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13790_4 = {S54, 0, &V13790_4_1224, &P13790_5};	/*Property = ScreenPixelsWidth*/
const Profile P13790 = {"13790", &P13790_4};	/*Profile ID - 13790*/
const Value V13793_1_1 = {S2, NULL};	/*Value = True*/
const Property P13793_1 = {S50, 0, &V13793_1_1, NULL};	/*Property = IsMobile*/
const Value V13793_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13793_5 = {S52, 0, &V13793_5_1233, &P13793_1};	/*Property = ScreenPixelsHeight*/
const Value V13793_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13793_4 = {S54, 0, &V13793_4_1235, &P13793_5};	/*Property = ScreenPixelsWidth*/
const Profile P13793 = {"13793", &P13793_4};	/*Profile ID - 13793*/
const Value V13795_1_1 = {S2, NULL};	/*Value = True*/
const Property P13795_1 = {S50, 0, &V13795_1_1, NULL};	/*Property = IsMobile*/
const Value V13795_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13795_5 = {S52, 0, &V13795_5_1233, &P13795_1};	/*Property = ScreenPixelsHeight*/
const Value V13795_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13795_4 = {S54, 0, &V13795_4_1235, &P13795_5};	/*Property = ScreenPixelsWidth*/
const Profile P13795 = {"13795", &P13795_4};	/*Profile ID - 13795*/
const Value V13797_1_1 = {S2, NULL};	/*Value = True*/
const Property P13797_1 = {S50, 0, &V13797_1_1, NULL};	/*Property = IsMobile*/
const Value V13797_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13797_5 = {S52, 0, &V13797_5_1293, &P13797_1};	/*Property = ScreenPixelsHeight*/
const Value V13797_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13797_4 = {S54, 0, &V13797_4_1296, &P13797_5};	/*Property = ScreenPixelsWidth*/
const Profile P13797 = {"13797", &P13797_4};	/*Profile ID - 13797*/
const Value V13801_1_1 = {S2, NULL};	/*Value = True*/
const Property P13801_1 = {S50, 0, &V13801_1_1, NULL};	/*Property = IsMobile*/
const Value V13801_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13801_5 = {S52, 0, &V13801_5_1222, &P13801_1};	/*Property = ScreenPixelsHeight*/
const Value V13801_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13801_4 = {S54, 0, &V13801_4_1224, &P13801_5};	/*Property = ScreenPixelsWidth*/
const Profile P13801 = {"13801", &P13801_4};	/*Profile ID - 13801*/
const Value V13802_1_1 = {S2, NULL};	/*Value = True*/
const Property P13802_1 = {S50, 0, &V13802_1_1, NULL};	/*Property = IsMobile*/
const Value V13802_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13802_5 = {S52, 0, &V13802_5_1380, &P13802_1};	/*Property = ScreenPixelsHeight*/
const Value V13802_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13802_4 = {S54, 0, &V13802_4_1279, &P13802_5};	/*Property = ScreenPixelsWidth*/
const Profile P13802 = {"13802", &P13802_4};	/*Profile ID - 13802*/
const Value V16704_1_1 = {S2, NULL};	/*Value = True*/
const Property P16704_1 = {S50, 0, &V16704_1_1, NULL};	/*Property = IsMobile*/
const Value V16704_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16704_5 = {S52, 0, &V16704_5_1233, &P16704_1};	/*Property = ScreenPixelsHeight*/
const Value V16704_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16704_4 = {S54, 0, &V16704_4_1235, &P16704_5};	/*Property = ScreenPixelsWidth*/
const Profile P16704 = {"16704", &P16704_4};	/*Profile ID - 16704*/
const Value V13810_1_1 = {S2, NULL};	/*Value = True*/
const Property P13810_1 = {S50, 0, &V13810_1_1, NULL};	/*Property = IsMobile*/
const Value V13810_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13810_5 = {S52, 0, &V13810_5_1304, &P13810_1};	/*Property = ScreenPixelsHeight*/
const Value V13810_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13810_4 = {S54, 0, &V13810_4_1224, &P13810_5};	/*Property = ScreenPixelsWidth*/
const Profile P13810 = {"13810", &P13810_4};	/*Profile ID - 13810*/
const Value V13227_1_1 = {S2, NULL};	/*Value = True*/
const Property P13227_1 = {S50, 0, &V13227_1_1, NULL};	/*Property = IsMobile*/
const Value V13227_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13227_5 = {S52, 0, &V13227_5_1222, &P13227_1};	/*Property = ScreenPixelsHeight*/
const Value V13227_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13227_4 = {S54, 0, &V13227_4_1224, &P13227_5};	/*Property = ScreenPixelsWidth*/
const Profile P13227 = {"13227", &P13227_4};	/*Profile ID - 13227*/
const Value V13819_1_1 = {S2, NULL};	/*Value = True*/
const Property P13819_1 = {S50, 0, &V13819_1_1, NULL};	/*Property = IsMobile*/
const Value V13819_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13819_5 = {S52, 0, &V13819_5_1222, &P13819_1};	/*Property = ScreenPixelsHeight*/
const Value V13819_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13819_4 = {S54, 0, &V13819_4_1224, &P13819_5};	/*Property = ScreenPixelsWidth*/
const Profile P13819 = {"13819", &P13819_4};	/*Profile ID - 13819*/
const Value V13818_1_1 = {S2, NULL};	/*Value = True*/
const Property P13818_1 = {S50, 0, &V13818_1_1, NULL};	/*Property = IsMobile*/
const Value V13818_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13818_5 = {S52, 0, &V13818_5_1304, &P13818_1};	/*Property = ScreenPixelsHeight*/
const Value V13818_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13818_4 = {S54, 0, &V13818_4_1224, &P13818_5};	/*Property = ScreenPixelsWidth*/
const Profile P13818 = {"13818", &P13818_4};	/*Profile ID - 13818*/
const Value V13822_1_1 = {S2, NULL};	/*Value = True*/
const Property P13822_1 = {S50, 0, &V13822_1_1, NULL};	/*Property = IsMobile*/
const Value V13822_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13822_5 = {S52, 0, &V13822_5_1233, &P13822_1};	/*Property = ScreenPixelsHeight*/
const Value V13822_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13822_4 = {S54, 0, &V13822_4_1235, &P13822_5};	/*Property = ScreenPixelsWidth*/
const Profile P13822 = {"13822", &P13822_4};	/*Profile ID - 13822*/
const Value V13824_1_1 = {S2, NULL};	/*Value = True*/
const Property P13824_1 = {S50, 0, &V13824_1_1, NULL};	/*Property = IsMobile*/
const Value V13824_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13824_5 = {S52, 0, &V13824_5_1293, &P13824_1};	/*Property = ScreenPixelsHeight*/
const Value V13824_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13824_4 = {S54, 0, &V13824_4_1224, &P13824_5};	/*Property = ScreenPixelsWidth*/
const Profile P13824 = {"13824", &P13824_4};	/*Profile ID - 13824*/
const Value V13827_1_1 = {S2, NULL};	/*Value = True*/
const Property P13827_1 = {S50, 0, &V13827_1_1, NULL};	/*Property = IsMobile*/
const Value V13827_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13827_5 = {S52, 0, &V13827_5_1222, &P13827_1};	/*Property = ScreenPixelsHeight*/
const Value V13827_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13827_4 = {S54, 0, &V13827_4_1224, &P13827_5};	/*Property = ScreenPixelsWidth*/
const Profile P13827 = {"13827", &P13827_4};	/*Profile ID - 13827*/
const Value V13830_1_1 = {S2, NULL};	/*Value = True*/
const Property P13830_1 = {S50, 0, &V13830_1_1, NULL};	/*Property = IsMobile*/
const Value V13830_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13830_5 = {S52, 0, &V13830_5_1222, &P13830_1};	/*Property = ScreenPixelsHeight*/
const Value V13830_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13830_4 = {S54, 0, &V13830_4_1224, &P13830_5};	/*Property = ScreenPixelsWidth*/
const Profile P13830 = {"13830", &P13830_4};	/*Profile ID - 13830*/
const Value V17803_1_1 = {S2, NULL};	/*Value = True*/
const Property P17803_1 = {S50, 0, &V17803_1_1, NULL};	/*Property = IsMobile*/
const Value V17803_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17803_5 = {S52, 0, &V17803_5_1293, &P17803_1};	/*Property = ScreenPixelsHeight*/
const Value V17803_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17803_4 = {S54, 0, &V17803_4_1296, &P17803_5};	/*Property = ScreenPixelsWidth*/
const Profile P17803 = {"17803", &P17803_4};	/*Profile ID - 17803*/
const Value V17921_1_1 = {S2, NULL};	/*Value = True*/
const Property P17921_1 = {S50, 0, &V17921_1_1, NULL};	/*Property = IsMobile*/
const Value V17921_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17921_5 = {S52, 0, &V17921_5_1222, &P17921_1};	/*Property = ScreenPixelsHeight*/
const Value V17921_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17921_4 = {S54, 0, &V17921_4_1224, &P17921_5};	/*Property = ScreenPixelsWidth*/
const Profile P17921 = {"17921", &P17921_4};	/*Profile ID - 17921*/
const Value V17229_1_1 = {S2, NULL};	/*Value = True*/
const Property P17229_1 = {S50, 0, &V17229_1_1, NULL};	/*Property = IsMobile*/
const Value V17229_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17229_5 = {S52, 0, &V17229_5_1293, &P17229_1};	/*Property = ScreenPixelsHeight*/
const Value V17229_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17229_4 = {S54, 0, &V17229_4_1296, &P17229_5};	/*Property = ScreenPixelsWidth*/
const Profile P17229 = {"17229", &P17229_4};	/*Profile ID - 17229*/
const Value V17983_1_1 = {S2, NULL};	/*Value = True*/
const Property P17983_1 = {S50, 0, &V17983_1_1, NULL};	/*Property = IsMobile*/
const Value V17983_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P17983_5 = {S52, 0, &V17983_5_1294, &P17983_1};	/*Property = ScreenPixelsHeight*/
const Value V17983_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17983_4 = {S54, 0, &V17983_4_1224, &P17983_5};	/*Property = ScreenPixelsWidth*/
const Profile P17983 = {"17983", &P17983_4};	/*Profile ID - 17983*/
const Value V17116_1_1 = {S2, NULL};	/*Value = True*/
const Property P17116_1 = {S50, 0, &V17116_1_1, NULL};	/*Property = IsMobile*/
const Value V17116_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17116_5 = {S52, 0, &V17116_5_1293, &P17116_1};	/*Property = ScreenPixelsHeight*/
const Value V17116_4_24031 = {S71, NULL};	/*Value = 120*/
const Property P17116_4 = {S54, 0, &V17116_4_24031, &P17116_5};	/*Property = ScreenPixelsWidth*/
const Profile P17116 = {"17116", &P17116_4};	/*Profile ID - 17116*/
const Value V12527_1_1 = {S2, NULL};	/*Value = True*/
const Property P12527_1 = {S50, 0, &V12527_1_1, NULL};	/*Property = IsMobile*/
const Value V12527_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12527_5 = {S52, 0, &V12527_5_1233, &P12527_1};	/*Property = ScreenPixelsHeight*/
const Value V12527_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12527_4 = {S54, 0, &V12527_4_1235, &P12527_5};	/*Property = ScreenPixelsWidth*/
const Profile P12527 = {"12527", &P12527_4};	/*Profile ID - 12527*/
const Value V13843_1_1 = {S2, NULL};	/*Value = True*/
const Property P13843_1 = {S50, 0, &V13843_1_1, NULL};	/*Property = IsMobile*/
const Value V13843_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13843_5 = {S52, 0, &V13843_5_1233, &P13843_1};	/*Property = ScreenPixelsHeight*/
const Value V13843_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13843_4 = {S54, 0, &V13843_4_1235, &P13843_5};	/*Property = ScreenPixelsWidth*/
const Profile P13843 = {"13843", &P13843_4};	/*Profile ID - 13843*/
const Value V13845_1_1 = {S2, NULL};	/*Value = True*/
const Property P13845_1 = {S50, 0, &V13845_1_1, NULL};	/*Property = IsMobile*/
const Value V13845_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13845_5 = {S52, 0, &V13845_5_1222, &P13845_1};	/*Property = ScreenPixelsHeight*/
const Value V13845_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13845_4 = {S54, 0, &V13845_4_1224, &P13845_5};	/*Property = ScreenPixelsWidth*/
const Profile P13845 = {"13845", &P13845_4};	/*Profile ID - 13845*/
const Value V13846_1_1 = {S2, NULL};	/*Value = True*/
const Property P13846_1 = {S50, 0, &V13846_1_1, NULL};	/*Property = IsMobile*/
const Value V13846_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13846_5 = {S52, 0, &V13846_5_1304, &P13846_1};	/*Property = ScreenPixelsHeight*/
const Value V13846_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13846_4 = {S54, 0, &V13846_4_1224, &P13846_5};	/*Property = ScreenPixelsWidth*/
const Profile P13846 = {"13846", &P13846_4};	/*Profile ID - 13846*/
const Value V13849_1_1 = {S2, NULL};	/*Value = True*/
const Property P13849_1 = {S50, 0, &V13849_1_1, NULL};	/*Property = IsMobile*/
const Value V13849_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13849_5 = {S52, 0, &V13849_5_1233, &P13849_1};	/*Property = ScreenPixelsHeight*/
const Value V13849_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13849_4 = {S54, 0, &V13849_4_1235, &P13849_5};	/*Property = ScreenPixelsWidth*/
const Profile P13849 = {"13849", &P13849_4};	/*Profile ID - 13849*/
const Value V13848_1_1 = {S2, NULL};	/*Value = True*/
const Property P13848_1 = {S50, 0, &V13848_1_1, NULL};	/*Property = IsMobile*/
const Value V13848_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13848_5 = {S52, 0, &V13848_5_1380, &P13848_1};	/*Property = ScreenPixelsHeight*/
const Value V13848_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13848_4 = {S54, 0, &V13848_4_1279, &P13848_5};	/*Property = ScreenPixelsWidth*/
const Profile P13848 = {"13848", &P13848_4};	/*Profile ID - 13848*/
const Value V13850_1_1 = {S2, NULL};	/*Value = True*/
const Property P13850_1 = {S50, 0, &V13850_1_1, NULL};	/*Property = IsMobile*/
const Value V13850_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P13850_5 = {S52, 0, &V13850_5_1333, &P13850_1};	/*Property = ScreenPixelsHeight*/
const Value V13850_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P13850_4 = {S54, 0, &V13850_4_1335, &P13850_5};	/*Property = ScreenPixelsWidth*/
const Profile P13850 = {"13850", &P13850_4};	/*Profile ID - 13850*/
const Value V14138_1_1 = {S2, NULL};	/*Value = True*/
const Property P14138_1 = {S50, 0, &V14138_1_1, NULL};	/*Property = IsMobile*/
const Value V14138_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14138_5 = {S52, 0, &V14138_5_1233, &P14138_1};	/*Property = ScreenPixelsHeight*/
const Value V14138_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14138_4 = {S54, 0, &V14138_4_1235, &P14138_5};	/*Property = ScreenPixelsWidth*/
const Profile P14138 = {"14138", &P14138_4};	/*Profile ID - 14138*/
const Value V16190_1_1 = {S2, NULL};	/*Value = True*/
const Property P16190_1 = {S50, 0, &V16190_1_1, NULL};	/*Property = IsMobile*/
const Value V16190_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16190_5 = {S52, 0, &V16190_5_1222, &P16190_1};	/*Property = ScreenPixelsHeight*/
const Value V16190_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16190_4 = {S54, 0, &V16190_4_1224, &P16190_5};	/*Property = ScreenPixelsWidth*/
const Profile P16190 = {"16190", &P16190_4};	/*Profile ID - 16190*/
const Value V13859_1_1 = {S2, NULL};	/*Value = True*/
const Property P13859_1 = {S50, 0, &V13859_1_1, NULL};	/*Property = IsMobile*/
const Value V13859_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13859_5 = {S52, 0, &V13859_5_1222, &P13859_1};	/*Property = ScreenPixelsHeight*/
const Value V13859_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13859_4 = {S54, 0, &V13859_4_1224, &P13859_5};	/*Property = ScreenPixelsWidth*/
const Profile P13859 = {"13859", &P13859_4};	/*Profile ID - 13859*/
const Value V13863_1_1 = {S2, NULL};	/*Value = True*/
const Property P13863_1 = {S50, 0, &V13863_1_1, NULL};	/*Property = IsMobile*/
const Value V13863_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13863_5 = {S52, 0, &V13863_5_1233, &P13863_1};	/*Property = ScreenPixelsHeight*/
const Value V13863_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13863_4 = {S54, 0, &V13863_4_1235, &P13863_5};	/*Property = ScreenPixelsWidth*/
const Profile P13863 = {"13863", &P13863_4};	/*Profile ID - 13863*/
const Value V12117_1_1 = {S2, NULL};	/*Value = True*/
const Property P12117_1 = {S50, 0, &V12117_1_1, NULL};	/*Property = IsMobile*/
const Value V12117_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12117_5 = {S52, 0, &V12117_5_1222, &P12117_1};	/*Property = ScreenPixelsHeight*/
const Value V12117_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12117_4 = {S54, 0, &V12117_4_1224, &P12117_5};	/*Property = ScreenPixelsWidth*/
const Profile P12117 = {"12117", &P12117_4};	/*Profile ID - 12117*/
const Value V16336_1_1 = {S2, NULL};	/*Value = True*/
const Property P16336_1 = {S50, 0, &V16336_1_1, NULL};	/*Property = IsMobile*/
const Value V16336_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16336_5 = {S52, 0, &V16336_5_1227, &P16336_1};	/*Property = ScreenPixelsHeight*/
const Value V16336_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16336_4 = {S54, 0, &V16336_4_1229, &P16336_5};	/*Property = ScreenPixelsWidth*/
const Profile P16336 = {"16336", &P16336_4};	/*Profile ID - 16336*/
const Value V16672_1_1 = {S2, NULL};	/*Value = True*/
const Property P16672_1 = {S50, 0, &V16672_1_1, NULL};	/*Property = IsMobile*/
const Value V16672_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16672_5 = {S52, 0, &V16672_5_1233, &P16672_1};	/*Property = ScreenPixelsHeight*/
const Value V16672_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16672_4 = {S54, 0, &V16672_4_1235, &P16672_5};	/*Property = ScreenPixelsWidth*/
const Profile P16672 = {"16672", &P16672_4};	/*Profile ID - 16672*/
const Value V14687_1_1 = {S2, NULL};	/*Value = True*/
const Property P14687_1 = {S50, 0, &V14687_1_1, NULL};	/*Property = IsMobile*/
const Value V14687_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14687_5 = {S52, 0, &V14687_5_1294, &P14687_1};	/*Property = ScreenPixelsHeight*/
const Value V14687_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14687_4 = {S54, 0, &V14687_4_1296, &P14687_5};	/*Property = ScreenPixelsWidth*/
const Profile P14687 = {"14687", &P14687_4};	/*Profile ID - 14687*/
const Value V20775_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_279 = {S1, 0, &V20775_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20775_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_282 = {S3, 0, &V20775_282_57155, &P20775_279};	/*Property = BlobBuilder*/
const Value V20775_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_350 = {S4, 0, &V20775_350_57446, &P20775_282};	/*Property = Canvas*/
const Value V20775_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_283 = {S5, 0, &V20775_283_57161, &P20775_350};	/*Property = CssBackground*/
const Value V20775_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_284 = {S6, 0, &V20775_284_57164, &P20775_283};	/*Property = CssBorderImage*/
const Value V20775_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_285 = {S7, 0, &V20775_285_57167, &P20775_284};	/*Property = CssCanvas*/
const Value V20775_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_286 = {S8, 0, &V20775_286_57170, &P20775_285};	/*Property = CssColor*/
const Value V20775_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_237 = {S9, 0, &V20775_237_55786, &P20775_286};	/*Property = CssColumn*/
const Value V20775_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_287 = {S10, 0, &V20775_287_57173, &P20775_237};	/*Property = CssFlexbox*/
const Value V20775_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_288 = {S11, 0, &V20775_288_57176, &P20775_287};	/*Property = CssFont*/
const Value V20775_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_289 = {S12, 0, &V20775_289_86899, &P20775_288};	/*Property = CssImages*/
const Value V20775_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_291 = {S13, 0, &V20775_291_57182, &P20775_289};	/*Property = CssMediaQueries*/
const Value V20775_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_292 = {S14, 0, &V20775_292_57185, &P20775_291};	/*Property = CssMinMax*/
const Value V20775_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_293 = {S15, 0, &V20775_293_57188, &P20775_292};	/*Property = CssOverflow*/
const Value V20775_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_294 = {S16, 0, &V20775_294_57191, &P20775_293};	/*Property = CssPosition*/
const Value V20775_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_295 = {S17, 0, &V20775_295_57194, &P20775_294};	/*Property = CssText*/
const Value V20775_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_297 = {S18, 0, &V20775_297_57197, &P20775_295};	/*Property = CssTransforms*/
const Value V20775_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_241 = {S19, 0, &V20775_241_55801, &P20775_297};	/*Property = CssTransitions*/
const Value V20775_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_298 = {S20, 0, &V20775_298_57208, &P20775_241};	/*Property = CssUI*/
const Value V20775_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_300 = {S21, 0, &V20775_300_57217, &P20775_298};	/*Property = DataSet*/
const Value V20775_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_301 = {S22, 0, &V20775_301_57220, &P20775_300};	/*Property = DataUrl*/
const Value V20775_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_302 = {S23, 0, &V20775_302_57223, &P20775_301};	/*Property = DeviceOrientation*/
const Value V20775_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_304 = {S24, 0, &V20775_304_57229, &P20775_302};	/*Property = FileReader*/
const Value V20775_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_305 = {S25, 0, &V20775_305_57232, &P20775_304};	/*Property = FileSaver*/
const Value V20775_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_306 = {S26, 0, &V20775_306_57238, &P20775_305};	/*Property = FileWriter*/
const Value V20775_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_307 = {S27, 0, &V20775_307_57241, &P20775_306};	/*Property = FormData*/
const Value V20775_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_309 = {S28, 0, &V20775_309_57247, &P20775_307};	/*Property = Fullscreen*/
const Value V20775_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_310 = {S29, 0, &V20775_310_57250, &P20775_309};	/*Property = GeoLocation*/
const Value V20775_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_312 = {S30, 0, &V20775_312_57256, &P20775_310};	/*Property = History*/
const Value V20775_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_314 = {S31, 0, &V20775_314_57262, &P20775_312};	/*Property = Html5*/
const Value V20775_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_313 = {S32, 0, &V20775_313_57259, &P20775_314};	/*Property = Html-Media-Capture*/
const Value V20775_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_315 = {S33, 0, &V20775_315_57265, &P20775_313};	/*Property = Iframe*/
const Value V20775_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_316 = {S34, 0, &V20775_316_57268, &P20775_315};	/*Property = IndexedDB*/
const Value V20775_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_317 = {S35, 0, &V20775_317_57271, &P20775_316};	/*Property = Json*/
const Value V20775_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_15 = {S37, 0, &V20775_15_72, &P20775_317};	/*Property = LayoutEngine*/
const Value V20775_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_344 = {S38, 0, &V20775_344_57274, &P20775_15};	/*Property = Masking*/
const Value V20775_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_320 = {S39, 0, &V20775_320_57292, &P20775_344};	/*Property = PostMessage*/
const Value V20775_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_321 = {S40, 0, &V20775_321_57295, &P20775_320};	/*Property = Progress*/
const Value V20775_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_322 = {S41, 0, &V20775_322_57298, &P20775_321};	/*Property = Prompts*/
const Value V20775_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_323 = {S42, 0, &V20775_323_57301, &P20775_322};	/*Property = Selector*/
const Value V20775_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_324 = {S43, 0, &V20775_324_57307, &P20775_323};	/*Property = Svg*/
const Value V20775_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_327 = {S44, 0, &V20775_327_57316, &P20775_324};	/*Property = TouchEvents*/
const Value V20775_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_328 = {S45, 0, &V20775_328_57319, &P20775_327};	/*Property = Track*/
const Value V20775_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_329 = {S46, 0, &V20775_329_57325, &P20775_328};	/*Property = Video*/
const Value V20775_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_330 = {S47, 0, &V20775_330_57328, &P20775_329};	/*Property = Viewport*/
const Value V20775_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_332 = {S48, 0, &V20775_332_57340, &P20775_330};	/*Property = WebWorkers*/
const Value V20775_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20775_333 = {S49, 0, &V20775_333_57343, &P20775_332};	/*Property = Xhr2*/
const Profile P20775 = {"20775", &P20775_333};	/*Profile ID - 20775*/
const Value V14333_1_1 = {S2, NULL};	/*Value = True*/
const Property P14333_1 = {S50, 0, &V14333_1_1, NULL};	/*Property = IsMobile*/
const Value V14333_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14333_5 = {S52, 0, &V14333_5_1293, &P14333_1};	/*Property = ScreenPixelsHeight*/
const Value V14333_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14333_4 = {S54, 0, &V14333_4_1296, &P14333_5};	/*Property = ScreenPixelsWidth*/
const Profile P14333 = {"14333", &P14333_4};	/*Profile ID - 14333*/
const Value V17916_1_1 = {S2, NULL};	/*Value = True*/
const Property P17916_1 = {S50, 0, &V17916_1_1, NULL};	/*Property = IsMobile*/
const Value V17916_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17916_5 = {S52, 0, &V17916_5_1222, &P17916_1};	/*Property = ScreenPixelsHeight*/
const Value V17916_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17916_4 = {S54, 0, &V17916_4_1224, &P17916_5};	/*Property = ScreenPixelsWidth*/
const Profile P17916 = {"17916", &P17916_4};	/*Profile ID - 17916*/
const Value V12933_1_1 = {S2, NULL};	/*Value = True*/
const Property P12933_1 = {S50, 0, &V12933_1_1, NULL};	/*Property = IsMobile*/
const Value V12933_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12933_5 = {S52, 0, &V12933_5_1222, &P12933_1};	/*Property = ScreenPixelsHeight*/
const Value V12933_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12933_4 = {S54, 0, &V12933_4_1224, &P12933_5};	/*Property = ScreenPixelsWidth*/
const Profile P12933 = {"12933", &P12933_4};	/*Profile ID - 12933*/
const Value V4453_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_279 = {S1, 0, &V4453_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4453_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_282 = {S3, 0, &V4453_282_57155, &P4453_279};	/*Property = BlobBuilder*/
const Value V4453_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_350 = {S4, 0, &V4453_350_57446, &P4453_282};	/*Property = Canvas*/
const Value V4453_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_283 = {S5, 0, &V4453_283_57161, &P4453_350};	/*Property = CssBackground*/
const Value V4453_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_284 = {S6, 0, &V4453_284_57164, &P4453_283};	/*Property = CssBorderImage*/
const Value V4453_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_285 = {S7, 0, &V4453_285_57167, &P4453_284};	/*Property = CssCanvas*/
const Value V4453_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_286 = {S8, 0, &V4453_286_57170, &P4453_285};	/*Property = CssColor*/
const Value V4453_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_237 = {S9, 0, &V4453_237_55786, &P4453_286};	/*Property = CssColumn*/
const Value V4453_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_287 = {S10, 0, &V4453_287_57173, &P4453_237};	/*Property = CssFlexbox*/
const Value V4453_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_288 = {S11, 0, &V4453_288_57176, &P4453_287};	/*Property = CssFont*/
const Value V4453_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_289 = {S12, 0, &V4453_289_86899, &P4453_288};	/*Property = CssImages*/
const Value V4453_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_291 = {S13, 0, &V4453_291_57182, &P4453_289};	/*Property = CssMediaQueries*/
const Value V4453_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_292 = {S14, 0, &V4453_292_57185, &P4453_291};	/*Property = CssMinMax*/
const Value V4453_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_293 = {S15, 0, &V4453_293_57188, &P4453_292};	/*Property = CssOverflow*/
const Value V4453_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_294 = {S16, 0, &V4453_294_57191, &P4453_293};	/*Property = CssPosition*/
const Value V4453_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_295 = {S17, 0, &V4453_295_57194, &P4453_294};	/*Property = CssText*/
const Value V4453_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_297 = {S18, 0, &V4453_297_57197, &P4453_295};	/*Property = CssTransforms*/
const Value V4453_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_241 = {S19, 0, &V4453_241_55801, &P4453_297};	/*Property = CssTransitions*/
const Value V4453_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_298 = {S20, 0, &V4453_298_57208, &P4453_241};	/*Property = CssUI*/
const Value V4453_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_300 = {S21, 0, &V4453_300_57217, &P4453_298};	/*Property = DataSet*/
const Value V4453_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_301 = {S22, 0, &V4453_301_57220, &P4453_300};	/*Property = DataUrl*/
const Value V4453_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_302 = {S23, 0, &V4453_302_57223, &P4453_301};	/*Property = DeviceOrientation*/
const Value V4453_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_304 = {S24, 0, &V4453_304_57229, &P4453_302};	/*Property = FileReader*/
const Value V4453_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_305 = {S25, 0, &V4453_305_57232, &P4453_304};	/*Property = FileSaver*/
const Value V4453_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_306 = {S26, 0, &V4453_306_57238, &P4453_305};	/*Property = FileWriter*/
const Value V4453_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_307 = {S27, 0, &V4453_307_57241, &P4453_306};	/*Property = FormData*/
const Value V4453_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_309 = {S28, 0, &V4453_309_57247, &P4453_307};	/*Property = Fullscreen*/
const Value V4453_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_310 = {S29, 0, &V4453_310_57250, &P4453_309};	/*Property = GeoLocation*/
const Value V4453_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_312 = {S30, 0, &V4453_312_57256, &P4453_310};	/*Property = History*/
const Value V4453_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_314 = {S31, 0, &V4453_314_57262, &P4453_312};	/*Property = Html5*/
const Value V4453_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_313 = {S32, 0, &V4453_313_57259, &P4453_314};	/*Property = Html-Media-Capture*/
const Value V4453_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_315 = {S33, 0, &V4453_315_57265, &P4453_313};	/*Property = Iframe*/
const Value V4453_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_316 = {S34, 0, &V4453_316_57268, &P4453_315};	/*Property = IndexedDB*/
const Value V4453_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_317 = {S35, 0, &V4453_317_57271, &P4453_316};	/*Property = Json*/
const Value V4453_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P4453_15 = {S37, 0, &V4453_15_28, &P4453_317};	/*Property = LayoutEngine*/
const Value V4453_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_344 = {S38, 0, &V4453_344_57274, &P4453_15};	/*Property = Masking*/
const Value V4453_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_320 = {S39, 0, &V4453_320_57292, &P4453_344};	/*Property = PostMessage*/
const Value V4453_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_321 = {S40, 0, &V4453_321_57295, &P4453_320};	/*Property = Progress*/
const Value V4453_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_322 = {S41, 0, &V4453_322_57298, &P4453_321};	/*Property = Prompts*/
const Value V4453_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_323 = {S42, 0, &V4453_323_57301, &P4453_322};	/*Property = Selector*/
const Value V4453_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_324 = {S43, 0, &V4453_324_57307, &P4453_323};	/*Property = Svg*/
const Value V4453_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_327 = {S44, 0, &V4453_327_57316, &P4453_324};	/*Property = TouchEvents*/
const Value V4453_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_328 = {S45, 0, &V4453_328_57319, &P4453_327};	/*Property = Track*/
const Value V4453_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_329 = {S46, 0, &V4453_329_57325, &P4453_328};	/*Property = Video*/
const Value V4453_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_330 = {S47, 0, &V4453_330_57328, &P4453_329};	/*Property = Viewport*/
const Value V4453_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_332 = {S48, 0, &V4453_332_57340, &P4453_330};	/*Property = WebWorkers*/
const Value V4453_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4453_333 = {S49, 0, &V4453_333_57343, &P4453_332};	/*Property = Xhr2*/
const Profile P4453 = {"4453", &P4453_333};	/*Profile ID - 4453*/
const Profile P6661 = {"6661", NULL};	/*Profile ID - 6661*/
const Value V17287_1_1 = {S2, NULL};	/*Value = True*/
const Property P17287_1 = {S50, 0, &V17287_1_1, NULL};	/*Property = IsMobile*/
const Value V17287_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17287_5 = {S52, 0, &V17287_5_1222, &P17287_1};	/*Property = ScreenPixelsHeight*/
const Value V17287_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17287_4 = {S54, 0, &V17287_4_1224, &P17287_5};	/*Property = ScreenPixelsWidth*/
const Profile P17287 = {"17287", &P17287_4};	/*Profile ID - 17287*/
const Value V15488_1_1 = {S2, NULL};	/*Value = True*/
const Property P15488_1 = {S50, 0, &V15488_1_1, NULL};	/*Property = IsMobile*/
const Value V15488_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P15488_5 = {S52, 0, &V15488_5_1269, &P15488_1};	/*Property = ScreenPixelsHeight*/
const Value V15488_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15488_4 = {S54, 0, &V15488_4_1229, &P15488_5};	/*Property = ScreenPixelsWidth*/
const Profile P15488 = {"15488", &P15488_4};	/*Profile ID - 15488*/
const Value V15489_1_1 = {S2, NULL};	/*Value = True*/
const Property P15489_1 = {S50, 0, &V15489_1_1, NULL};	/*Property = IsMobile*/
const Value V15489_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15489_5 = {S52, 0, &V15489_5_1227, &P15489_1};	/*Property = ScreenPixelsHeight*/
const Value V15489_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15489_4 = {S54, 0, &V15489_4_1229, &P15489_5};	/*Property = ScreenPixelsWidth*/
const Profile P15489 = {"15489", &P15489_4};	/*Profile ID - 15489*/
const Value V16572_1_1 = {S2, NULL};	/*Value = True*/
const Property P16572_1 = {S50, 0, &V16572_1_1, NULL};	/*Property = IsMobile*/
const Value V16572_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16572_5 = {S52, 0, &V16572_5_1233, &P16572_1};	/*Property = ScreenPixelsHeight*/
const Value V16572_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16572_4 = {S54, 0, &V16572_4_1235, &P16572_5};	/*Property = ScreenPixelsWidth*/
const Profile P16572 = {"16572", &P16572_4};	/*Profile ID - 16572*/
const Value V15513_1_1 = {S2, NULL};	/*Value = True*/
const Property P15513_1 = {S50, 0, &V15513_1_1, NULL};	/*Property = IsMobile*/
const Value V15513_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P15513_5 = {S52, 0, &V15513_5_1275, &P15513_1};	/*Property = ScreenPixelsHeight*/
const Value V15513_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P15513_4 = {S54, 0, &V15513_4_1278, &P15513_5};	/*Property = ScreenPixelsWidth*/
const Profile P15513 = {"15513", &P15513_4};	/*Profile ID - 15513*/
const Value V15911_1_1 = {S2, NULL};	/*Value = True*/
const Property P15911_1 = {S50, 0, &V15911_1_1, NULL};	/*Property = IsMobile*/
const Value V15911_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P15911_5 = {S52, 0, &V15911_5_1403, &P15911_1};	/*Property = ScreenPixelsHeight*/
const Value V15911_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P15911_4 = {S54, 0, &V15911_4_1404, &P15911_5};	/*Property = ScreenPixelsWidth*/
const Profile P15911 = {"15911", &P15911_4};	/*Profile ID - 15911*/
const Value V23914_1_1 = {S2, NULL};	/*Value = True*/
const Property P23914_1 = {S50, 0, &V23914_1_1, NULL};	/*Property = IsMobile*/
const Value V23914_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23914_5 = {S52, 0, &V23914_5_1269, &P23914_1};	/*Property = ScreenPixelsHeight*/
const Value V23914_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23914_4 = {S54, 0, &V23914_4_1229, &P23914_5};	/*Property = ScreenPixelsWidth*/
const Profile P23914 = {"23914", &P23914_4};	/*Profile ID - 23914*/
const Value V14911_1_1 = {S2, NULL};	/*Value = True*/
const Property P14911_1 = {S50, 0, &V14911_1_1, NULL};	/*Property = IsMobile*/
const Value V14911_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P14911_5 = {S52, 0, &V14911_5_1269, &P14911_1};	/*Property = ScreenPixelsHeight*/
const Value V14911_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14911_4 = {S54, 0, &V14911_4_1229, &P14911_5};	/*Property = ScreenPixelsWidth*/
const Profile P14911 = {"14911", &P14911_4};	/*Profile ID - 14911*/
const Value V17308_1_1 = {S2, NULL};	/*Value = True*/
const Property P17308_1 = {S50, 0, &V17308_1_1, NULL};	/*Property = IsMobile*/
const Value V17308_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17308_5 = {S52, 0, &V17308_5_1269, &P17308_1};	/*Property = ScreenPixelsHeight*/
const Value V17308_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17308_4 = {S54, 0, &V17308_4_1229, &P17308_5};	/*Property = ScreenPixelsWidth*/
const Profile P17308 = {"17308", &P17308_4};	/*Profile ID - 17308*/
const Value V15585_1_1 = {S2, NULL};	/*Value = True*/
const Property P15585_1 = {S50, 0, &V15585_1_1, NULL};	/*Property = IsMobile*/
const Value V15585_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P15585_5 = {S52, 0, &V15585_5_1403, &P15585_1};	/*Property = ScreenPixelsHeight*/
const Value V15585_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P15585_4 = {S54, 0, &V15585_4_1404, &P15585_5};	/*Property = ScreenPixelsWidth*/
const Profile P15585 = {"15585", &P15585_4};	/*Profile ID - 15585*/
const Value V17281_1_1 = {S2, NULL};	/*Value = True*/
const Property P17281_1 = {S50, 0, &V17281_1_1, NULL};	/*Property = IsMobile*/
const Value V17281_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17281_5 = {S52, 0, &V17281_5_1227, &P17281_1};	/*Property = ScreenPixelsHeight*/
const Value V17281_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P17281_4 = {S54, 0, &V17281_4_1477, &P17281_5};	/*Property = ScreenPixelsWidth*/
const Profile P17281 = {"17281", &P17281_4};	/*Profile ID - 17281*/
const Value V12552_1_1 = {S2, NULL};	/*Value = True*/
const Property P12552_1 = {S50, 0, &V12552_1_1, NULL};	/*Property = IsMobile*/
const Value V12552_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12552_5 = {S52, 0, &V12552_5_1222, &P12552_1};	/*Property = ScreenPixelsHeight*/
const Value V12552_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12552_4 = {S54, 0, &V12552_4_1224, &P12552_5};	/*Property = ScreenPixelsWidth*/
const Profile P12552 = {"12552", &P12552_4};	/*Profile ID - 12552*/
const Value V15880_1_1 = {S2, NULL};	/*Value = True*/
const Property P15880_1 = {S50, 0, &V15880_1_1, NULL};	/*Property = IsMobile*/
const Value V15880_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P15880_5 = {S52, 0, &V15880_5_1403, &P15880_1};	/*Property = ScreenPixelsHeight*/
const Value V15880_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P15880_4 = {S54, 0, &V15880_4_1404, &P15880_5};	/*Property = ScreenPixelsWidth*/
const Profile P15880 = {"15880", &P15880_4};	/*Profile ID - 15880*/
const Value V12562_1_1 = {S2, NULL};	/*Value = True*/
const Property P12562_1 = {S50, 0, &V12562_1_1, NULL};	/*Property = IsMobile*/
const Value V12562_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P12562_5 = {S52, 0, &V12562_5_1269, &P12562_1};	/*Property = ScreenPixelsHeight*/
const Value V12562_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12562_4 = {S54, 0, &V12562_4_1229, &P12562_5};	/*Property = ScreenPixelsWidth*/
const Profile P12562 = {"12562", &P12562_4};	/*Profile ID - 12562*/
const Value V15236_1_1 = {S2, NULL};	/*Value = True*/
const Property P15236_1 = {S50, 0, &V15236_1_1, NULL};	/*Property = IsMobile*/
const Value V15236_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15236_5 = {S52, 0, &V15236_5_1277, &P15236_1};	/*Property = ScreenPixelsHeight*/
const Value V15236_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P15236_4 = {S54, 0, &V15236_4_8554, &P15236_5};	/*Property = ScreenPixelsWidth*/
const Profile P15236 = {"15236", &P15236_4};	/*Profile ID - 15236*/
const Value V15259_1_1 = {S2, NULL};	/*Value = True*/
const Property P15259_1 = {S50, 0, &V15259_1_1, NULL};	/*Property = IsMobile*/
const Value V15259_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15259_5 = {S52, 0, &V15259_5_1277, &P15259_1};	/*Property = ScreenPixelsHeight*/
const Value V15259_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P15259_4 = {S54, 0, &V15259_4_8554, &P15259_5};	/*Property = ScreenPixelsWidth*/
const Profile P15259 = {"15259", &P15259_4};	/*Profile ID - 15259*/
const Value V12731_1_1 = {S2, NULL};	/*Value = True*/
const Property P12731_1 = {S50, 0, &V12731_1_1, NULL};	/*Property = IsMobile*/
const Value V12731_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P12731_5 = {S52, 0, &V12731_5_1403, &P12731_1};	/*Property = ScreenPixelsHeight*/
const Value V12731_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P12731_4 = {S54, 0, &V12731_4_1404, &P12731_5};	/*Property = ScreenPixelsWidth*/
const Profile P12731 = {"12731", &P12731_4};	/*Profile ID - 12731*/
const Value V12581_1_1 = {S2, NULL};	/*Value = True*/
const Property P12581_1 = {S50, 0, &V12581_1_1, NULL};	/*Property = IsMobile*/
const Value V12581_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P12581_5 = {S52, 0, &V12581_5_1275, &P12581_1};	/*Property = ScreenPixelsHeight*/
const Value V12581_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P12581_4 = {S54, 0, &V12581_4_1278, &P12581_5};	/*Property = ScreenPixelsWidth*/
const Profile P12581 = {"12581", &P12581_4};	/*Profile ID - 12581*/
const Value V16991_1_1 = {S2, NULL};	/*Value = True*/
const Property P16991_1 = {S50, 0, &V16991_1_1, NULL};	/*Property = IsMobile*/
const Value V16991_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16991_5 = {S52, 0, &V16991_5_1222, &P16991_1};	/*Property = ScreenPixelsHeight*/
const Value V16991_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16991_4 = {S54, 0, &V16991_4_1224, &P16991_5};	/*Property = ScreenPixelsWidth*/
const Profile P16991 = {"16991", &P16991_4};	/*Profile ID - 16991*/
const Value V12628_1_1 = {S2, NULL};	/*Value = True*/
const Property P12628_1 = {S50, 0, &V12628_1_1, NULL};	/*Property = IsMobile*/
const Value V12628_5_1796 = {S64, NULL};	/*Value = 360*/
const Property P12628_5 = {S52, 0, &V12628_5_1796, &P12628_1};	/*Property = ScreenPixelsHeight*/
const Value V12628_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12628_4 = {S54, 0, &V12628_4_1229, &P12628_5};	/*Property = ScreenPixelsWidth*/
const Profile P12628 = {"12628", &P12628_4};	/*Profile ID - 12628*/
const Value V12601_1_1 = {S2, NULL};	/*Value = True*/
const Property P12601_1 = {S50, 0, &V12601_1_1, NULL};	/*Property = IsMobile*/
const Value V12601_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12601_5 = {S52, 0, &V12601_5_1222, &P12601_1};	/*Property = ScreenPixelsHeight*/
const Value V12601_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12601_4 = {S54, 0, &V12601_4_1224, &P12601_5};	/*Property = ScreenPixelsWidth*/
const Profile P12601 = {"12601", &P12601_4};	/*Profile ID - 12601*/
const Value V12600_1_1 = {S2, NULL};	/*Value = True*/
const Property P12600_1 = {S50, 0, &V12600_1_1, NULL};	/*Property = IsMobile*/
const Value V12600_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12600_5 = {S52, 0, &V12600_5_1222, &P12600_1};	/*Property = ScreenPixelsHeight*/
const Value V12600_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12600_4 = {S54, 0, &V12600_4_1224, &P12600_5};	/*Property = ScreenPixelsWidth*/
const Profile P12600 = {"12600", &P12600_4};	/*Profile ID - 12600*/
const Value V12605_1_1 = {S2, NULL};	/*Value = True*/
const Property P12605_1 = {S50, 0, &V12605_1_1, NULL};	/*Property = IsMobile*/
const Value V12605_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P12605_5 = {S52, 0, &V12605_5_1403, &P12605_1};	/*Property = ScreenPixelsHeight*/
const Value V12605_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P12605_4 = {S54, 0, &V12605_4_1404, &P12605_5};	/*Property = ScreenPixelsWidth*/
const Profile P12605 = {"12605", &P12605_4};	/*Profile ID - 12605*/
const Value V12607_1_1 = {S2, NULL};	/*Value = True*/
const Property P12607_1 = {S50, 0, &V12607_1_1, NULL};	/*Property = IsMobile*/
const Value V12607_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12607_5 = {S52, 0, &V12607_5_1277, &P12607_1};	/*Property = ScreenPixelsHeight*/
const Value V12607_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12607_4 = {S54, 0, &V12607_4_1279, &P12607_5};	/*Property = ScreenPixelsWidth*/
const Profile P12607 = {"12607", &P12607_4};	/*Profile ID - 12607*/
const Value V12610_1_1 = {S2, NULL};	/*Value = True*/
const Property P12610_1 = {S50, 0, &V12610_1_1, NULL};	/*Property = IsMobile*/
const Value V12610_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12610_5 = {S52, 0, &V12610_5_1293, &P12610_1};	/*Property = ScreenPixelsHeight*/
const Value V12610_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12610_4 = {S54, 0, &V12610_4_1296, &P12610_5};	/*Property = ScreenPixelsWidth*/
const Profile P12610 = {"12610", &P12610_4};	/*Profile ID - 12610*/
const Value V15263_1_1 = {S2, NULL};	/*Value = True*/
const Property P15263_1 = {S50, 0, &V15263_1_1, NULL};	/*Property = IsMobile*/
const Value V15263_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15263_5 = {S52, 0, &V15263_5_1227, &P15263_1};	/*Property = ScreenPixelsHeight*/
const Value V15263_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15263_4 = {S54, 0, &V15263_4_1229, &P15263_5};	/*Property = ScreenPixelsWidth*/
const Profile P15263 = {"15263", &P15263_4};	/*Profile ID - 15263*/
const Value V12616_1_1 = {S2, NULL};	/*Value = True*/
const Property P12616_1 = {S50, 0, &V12616_1_1, NULL};	/*Property = IsMobile*/
const Value V12616_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12616_5 = {S52, 0, &V12616_5_1277, &P12616_1};	/*Property = ScreenPixelsHeight*/
const Value V12616_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12616_4 = {S54, 0, &V12616_4_1279, &P12616_5};	/*Property = ScreenPixelsWidth*/
const Profile P12616 = {"12616", &P12616_4};	/*Profile ID - 12616*/
const Value V12634_1_1 = {S2, NULL};	/*Value = True*/
const Property P12634_1 = {S50, 0, &V12634_1_1, NULL};	/*Property = IsMobile*/
const Value V12634_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12634_5 = {S52, 0, &V12634_5_1222, &P12634_1};	/*Property = ScreenPixelsHeight*/
const Value V12634_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12634_4 = {S54, 0, &V12634_4_1224, &P12634_5};	/*Property = ScreenPixelsWidth*/
const Profile P12634 = {"12634", &P12634_4};	/*Profile ID - 12634*/
const Value V12635_1_1 = {S2, NULL};	/*Value = True*/
const Property P12635_1 = {S50, 0, &V12635_1_1, NULL};	/*Property = IsMobile*/
const Value V12635_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12635_5 = {S52, 0, &V12635_5_1222, &P12635_1};	/*Property = ScreenPixelsHeight*/
const Value V12635_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12635_4 = {S54, 0, &V12635_4_1224, &P12635_5};	/*Property = ScreenPixelsWidth*/
const Profile P12635 = {"12635", &P12635_4};	/*Profile ID - 12635*/
const Value V12640_1_1 = {S2, NULL};	/*Value = True*/
const Property P12640_1 = {S50, 0, &V12640_1_1, NULL};	/*Property = IsMobile*/
const Value V12640_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P12640_5 = {S52, 0, &V12640_5_1333, &P12640_1};	/*Property = ScreenPixelsHeight*/
const Value V12640_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P12640_4 = {S54, 0, &V12640_4_1335, &P12640_5};	/*Property = ScreenPixelsWidth*/
const Profile P12640 = {"12640", &P12640_4};	/*Profile ID - 12640*/
const Value V15250_1_1 = {S2, NULL};	/*Value = True*/
const Property P15250_1 = {S50, 0, &V15250_1_1, NULL};	/*Property = IsMobile*/
const Value V15250_5_1796 = {S64, NULL};	/*Value = 360*/
const Property P15250_5 = {S52, 0, &V15250_5_1796, &P15250_1};	/*Property = ScreenPixelsHeight*/
const Value V15250_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15250_4 = {S54, 0, &V15250_4_1229, &P15250_5};	/*Property = ScreenPixelsWidth*/
const Profile P15250 = {"15250", &P15250_4};	/*Profile ID - 15250*/
const Value V12663_1_1 = {S2, NULL};	/*Value = True*/
const Property P12663_1 = {S50, 0, &V12663_1_1, NULL};	/*Property = IsMobile*/
const Value V12663_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12663_5 = {S52, 0, &V12663_5_1227, &P12663_1};	/*Property = ScreenPixelsHeight*/
const Value V12663_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12663_4 = {S54, 0, &V12663_4_1229, &P12663_5};	/*Property = ScreenPixelsWidth*/
const Profile P12663 = {"12663", &P12663_4};	/*Profile ID - 12663*/
const Value V12671_1_1 = {S2, NULL};	/*Value = True*/
const Property P12671_1 = {S50, 0, &V12671_1_1, NULL};	/*Property = IsMobile*/
const Value V12671_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P12671_5 = {S52, 0, &V12671_5_1275, &P12671_1};	/*Property = ScreenPixelsHeight*/
const Value V12671_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P12671_4 = {S54, 0, &V12671_4_1278, &P12671_5};	/*Property = ScreenPixelsWidth*/
const Profile P12671 = {"12671", &P12671_4};	/*Profile ID - 12671*/
const Value V13286_1_1 = {S2, NULL};	/*Value = True*/
const Property P13286_1 = {S50, 0, &V13286_1_1, NULL};	/*Property = IsMobile*/
const Value V13286_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P13286_5 = {S52, 0, &V13286_5_1269, &P13286_1};	/*Property = ScreenPixelsHeight*/
const Value V13286_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13286_4 = {S54, 0, &V13286_4_1229, &P13286_5};	/*Property = ScreenPixelsWidth*/
const Profile P13286 = {"13286", &P13286_4};	/*Profile ID - 13286*/
const Value V20969_1_1 = {S2, NULL};	/*Value = True*/
const Property P20969_1 = {S50, 0, &V20969_1_1, NULL};	/*Property = IsMobile*/
const Value V20969_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20969_5 = {S52, 0, &V20969_5_1403, &P20969_1};	/*Property = ScreenPixelsHeight*/
const Value V20969_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20969_4 = {S54, 0, &V20969_4_1404, &P20969_5};	/*Property = ScreenPixelsWidth*/
const Profile P20969 = {"20969", &P20969_4};	/*Profile ID - 20969*/
const Value V12557_1_1 = {S2, NULL};	/*Value = True*/
const Property P12557_1 = {S50, 0, &V12557_1_1, NULL};	/*Property = IsMobile*/
const Value V12557_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12557_5 = {S52, 0, &V12557_5_1277, &P12557_1};	/*Property = ScreenPixelsHeight*/
const Value V12557_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12557_4 = {S54, 0, &V12557_4_1279, &P12557_5};	/*Property = ScreenPixelsWidth*/
const Profile P12557 = {"12557", &P12557_4};	/*Profile ID - 12557*/
const Value V22233_1_1 = {S2, NULL};	/*Value = True*/
const Property P22233_1 = {S50, 0, &V22233_1_1, NULL};	/*Property = IsMobile*/
const Value V22233_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22233_5 = {S52, 0, &V22233_5_1403, &P22233_1};	/*Property = ScreenPixelsHeight*/
const Value V22233_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22233_4 = {S54, 0, &V22233_4_1404, &P22233_5};	/*Property = ScreenPixelsWidth*/
const Profile P22233 = {"22233", &P22233_4};	/*Profile ID - 22233*/
const Value V16260_1_1 = {S2, NULL};	/*Value = True*/
const Property P16260_1 = {S50, 0, &V16260_1_1, NULL};	/*Property = IsMobile*/
const Value V16260_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16260_5 = {S52, 0, &V16260_5_1222, &P16260_1};	/*Property = ScreenPixelsHeight*/
const Value V16260_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16260_4 = {S54, 0, &V16260_4_1224, &P16260_5};	/*Property = ScreenPixelsWidth*/
const Profile P16260 = {"16260", &P16260_4};	/*Profile ID - 16260*/
const Value V17394_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_279 = {S1, 0, &V17394_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17394_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_282 = {S3, 0, &V17394_282_57155, &P17394_279};	/*Property = BlobBuilder*/
const Value V17394_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_350 = {S4, 0, &V17394_350_57446, &P17394_282};	/*Property = Canvas*/
const Value V17394_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_283 = {S5, 0, &V17394_283_57161, &P17394_350};	/*Property = CssBackground*/
const Value V17394_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_284 = {S6, 0, &V17394_284_57164, &P17394_283};	/*Property = CssBorderImage*/
const Value V17394_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_285 = {S7, 0, &V17394_285_57167, &P17394_284};	/*Property = CssCanvas*/
const Value V17394_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_286 = {S8, 0, &V17394_286_57170, &P17394_285};	/*Property = CssColor*/
const Value V17394_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_237 = {S9, 0, &V17394_237_55786, &P17394_286};	/*Property = CssColumn*/
const Value V17394_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_287 = {S10, 0, &V17394_287_57173, &P17394_237};	/*Property = CssFlexbox*/
const Value V17394_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_288 = {S11, 0, &V17394_288_57176, &P17394_287};	/*Property = CssFont*/
const Value V17394_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_289 = {S12, 0, &V17394_289_86899, &P17394_288};	/*Property = CssImages*/
const Value V17394_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_291 = {S13, 0, &V17394_291_57182, &P17394_289};	/*Property = CssMediaQueries*/
const Value V17394_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_292 = {S14, 0, &V17394_292_57185, &P17394_291};	/*Property = CssMinMax*/
const Value V17394_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_293 = {S15, 0, &V17394_293_57188, &P17394_292};	/*Property = CssOverflow*/
const Value V17394_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_294 = {S16, 0, &V17394_294_57191, &P17394_293};	/*Property = CssPosition*/
const Value V17394_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_295 = {S17, 0, &V17394_295_57194, &P17394_294};	/*Property = CssText*/
const Value V17394_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_297 = {S18, 0, &V17394_297_57197, &P17394_295};	/*Property = CssTransforms*/
const Value V17394_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_241 = {S19, 0, &V17394_241_55801, &P17394_297};	/*Property = CssTransitions*/
const Value V17394_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_298 = {S20, 0, &V17394_298_57208, &P17394_241};	/*Property = CssUI*/
const Value V17394_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_300 = {S21, 0, &V17394_300_57217, &P17394_298};	/*Property = DataSet*/
const Value V17394_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_301 = {S22, 0, &V17394_301_57220, &P17394_300};	/*Property = DataUrl*/
const Value V17394_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_302 = {S23, 0, &V17394_302_57223, &P17394_301};	/*Property = DeviceOrientation*/
const Value V17394_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_304 = {S24, 0, &V17394_304_57229, &P17394_302};	/*Property = FileReader*/
const Value V17394_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_305 = {S25, 0, &V17394_305_57232, &P17394_304};	/*Property = FileSaver*/
const Value V17394_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_306 = {S26, 0, &V17394_306_57238, &P17394_305};	/*Property = FileWriter*/
const Value V17394_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_307 = {S27, 0, &V17394_307_57241, &P17394_306};	/*Property = FormData*/
const Value V17394_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_309 = {S28, 0, &V17394_309_57247, &P17394_307};	/*Property = Fullscreen*/
const Value V17394_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_310 = {S29, 0, &V17394_310_57250, &P17394_309};	/*Property = GeoLocation*/
const Value V17394_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_312 = {S30, 0, &V17394_312_57256, &P17394_310};	/*Property = History*/
const Value V17394_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_314 = {S31, 0, &V17394_314_57262, &P17394_312};	/*Property = Html5*/
const Value V17394_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_313 = {S32, 0, &V17394_313_57259, &P17394_314};	/*Property = Html-Media-Capture*/
const Value V17394_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_315 = {S33, 0, &V17394_315_57265, &P17394_313};	/*Property = Iframe*/
const Value V17394_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_316 = {S34, 0, &V17394_316_57268, &P17394_315};	/*Property = IndexedDB*/
const Value V17394_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_317 = {S35, 0, &V17394_317_57271, &P17394_316};	/*Property = Json*/
const Value V17394_15_43263 = {S65, NULL};	/*Value = Netfront*/
const Property P17394_15 = {S37, 0, &V17394_15_43263, &P17394_317};	/*Property = LayoutEngine*/
const Value V17394_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_344 = {S38, 0, &V17394_344_57274, &P17394_15};	/*Property = Masking*/
const Value V17394_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_320 = {S39, 0, &V17394_320_57292, &P17394_344};	/*Property = PostMessage*/
const Value V17394_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_321 = {S40, 0, &V17394_321_57295, &P17394_320};	/*Property = Progress*/
const Value V17394_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_322 = {S41, 0, &V17394_322_57298, &P17394_321};	/*Property = Prompts*/
const Value V17394_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_323 = {S42, 0, &V17394_323_57301, &P17394_322};	/*Property = Selector*/
const Value V17394_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_324 = {S43, 0, &V17394_324_57307, &P17394_323};	/*Property = Svg*/
const Value V17394_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_327 = {S44, 0, &V17394_327_57316, &P17394_324};	/*Property = TouchEvents*/
const Value V17394_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_328 = {S45, 0, &V17394_328_57319, &P17394_327};	/*Property = Track*/
const Value V17394_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_329 = {S46, 0, &V17394_329_57325, &P17394_328};	/*Property = Video*/
const Value V17394_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_330 = {S47, 0, &V17394_330_57328, &P17394_329};	/*Property = Viewport*/
const Value V17394_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_332 = {S48, 0, &V17394_332_57340, &P17394_330};	/*Property = WebWorkers*/
const Value V17394_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17394_333 = {S49, 0, &V17394_333_57343, &P17394_332};	/*Property = Xhr2*/
const Profile P17394 = {"17394", &P17394_333};	/*Profile ID - 17394*/
const Value V12698_1_1 = {S2, NULL};	/*Value = True*/
const Property P12698_1 = {S50, 0, &V12698_1_1, NULL};	/*Property = IsMobile*/
const Value V12698_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12698_5 = {S52, 0, &V12698_5_1380, &P12698_1};	/*Property = ScreenPixelsHeight*/
const Value V12698_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12698_4 = {S54, 0, &V12698_4_1279, &P12698_5};	/*Property = ScreenPixelsWidth*/
const Profile P12698 = {"12698", &P12698_4};	/*Profile ID - 12698*/
const Value V12706_1_1 = {S2, NULL};	/*Value = True*/
const Property P12706_1 = {S50, 0, &V12706_1_1, NULL};	/*Property = IsMobile*/
const Value V12706_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12706_5 = {S52, 0, &V12706_5_1277, &P12706_1};	/*Property = ScreenPixelsHeight*/
const Value V12706_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12706_4 = {S54, 0, &V12706_4_1279, &P12706_5};	/*Property = ScreenPixelsWidth*/
const Profile P12706 = {"12706", &P12706_4};	/*Profile ID - 12706*/
const Value V12707_1_1 = {S2, NULL};	/*Value = True*/
const Property P12707_1 = {S50, 0, &V12707_1_1, NULL};	/*Property = IsMobile*/
const Value V12707_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P12707_5 = {S52, 0, &V12707_5_1275, &P12707_1};	/*Property = ScreenPixelsHeight*/
const Value V12707_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P12707_4 = {S54, 0, &V12707_4_1278, &P12707_5};	/*Property = ScreenPixelsWidth*/
const Profile P12707 = {"12707", &P12707_4};	/*Profile ID - 12707*/
const Value V12709_1_1 = {S2, NULL};	/*Value = True*/
const Property P12709_1 = {S50, 0, &V12709_1_1, NULL};	/*Property = IsMobile*/
const Value V12709_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P12709_5 = {S52, 0, &V12709_5_1269, &P12709_1};	/*Property = ScreenPixelsHeight*/
const Value V12709_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12709_4 = {S54, 0, &V12709_4_1229, &P12709_5};	/*Property = ScreenPixelsWidth*/
const Profile P12709 = {"12709", &P12709_4};	/*Profile ID - 12709*/
const Value V12725_1_1 = {S2, NULL};	/*Value = True*/
const Property P12725_1 = {S50, 0, &V12725_1_1, NULL};	/*Property = IsMobile*/
const Value V12725_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12725_5 = {S52, 0, &V12725_5_1227, &P12725_1};	/*Property = ScreenPixelsHeight*/
const Value V12725_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12725_4 = {S54, 0, &V12725_4_1229, &P12725_5};	/*Property = ScreenPixelsWidth*/
const Profile P12725 = {"12725", &P12725_4};	/*Profile ID - 12725*/
const Value V18083_1_1 = {S2, NULL};	/*Value = True*/
const Property P18083_1 = {S50, 0, &V18083_1_1, NULL};	/*Property = IsMobile*/
const Value V18083_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18083_5 = {S52, 0, &V18083_5_1227, &P18083_1};	/*Property = ScreenPixelsHeight*/
const Value V18083_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18083_4 = {S54, 0, &V18083_4_1229, &P18083_5};	/*Property = ScreenPixelsWidth*/
const Profile P18083 = {"18083", &P18083_4};	/*Profile ID - 18083*/
const Value V17710_1_1 = {S2, NULL};	/*Value = True*/
const Property P17710_1 = {S50, 0, &V17710_1_1, NULL};	/*Property = IsMobile*/
const Value V17710_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17710_5 = {S52, 0, &V17710_5_1227, &P17710_1};	/*Property = ScreenPixelsHeight*/
const Value V17710_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17710_4 = {S54, 0, &V17710_4_1229, &P17710_5};	/*Property = ScreenPixelsWidth*/
const Profile P17710 = {"17710", &P17710_4};	/*Profile ID - 17710*/
const Value V12749_1_1 = {S2, NULL};	/*Value = True*/
const Property P12749_1 = {S50, 0, &V12749_1_1, NULL};	/*Property = IsMobile*/
const Value V12749_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12749_5 = {S52, 0, &V12749_5_1277, &P12749_1};	/*Property = ScreenPixelsHeight*/
const Value V12749_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12749_4 = {S54, 0, &V12749_4_1279, &P12749_5};	/*Property = ScreenPixelsWidth*/
const Profile P12749 = {"12749", &P12749_4};	/*Profile ID - 12749*/
const Value V21562_1_1 = {S2, NULL};	/*Value = True*/
const Property P21562_1 = {S50, 0, &V21562_1_1, NULL};	/*Property = IsMobile*/
const Value V21562_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21562_5 = {S52, 0, &V21562_5_1277, &P21562_1};	/*Property = ScreenPixelsHeight*/
const Value V21562_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21562_4 = {S54, 0, &V21562_4_1279, &P21562_5};	/*Property = ScreenPixelsWidth*/
const Profile P21562 = {"21562", &P21562_4};	/*Profile ID - 21562*/
const Value V16459_1_1 = {S2, NULL};	/*Value = True*/
const Property P16459_1 = {S50, 0, &V16459_1_1, NULL};	/*Property = IsMobile*/
const Value V16459_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16459_5 = {S52, 0, &V16459_5_1277, &P16459_1};	/*Property = ScreenPixelsHeight*/
const Value V16459_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16459_4 = {S54, 0, &V16459_4_1279, &P16459_5};	/*Property = ScreenPixelsWidth*/
const Profile P16459 = {"16459", &P16459_4};	/*Profile ID - 16459*/
const Value V18115_1_1 = {S2, NULL};	/*Value = True*/
const Property P18115_1 = {S50, 0, &V18115_1_1, NULL};	/*Property = IsMobile*/
const Value V18115_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P18115_5 = {S52, 0, &V18115_5_37, &P18115_1};	/*Property = ScreenPixelsHeight*/
const Value V18115_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P18115_4 = {S54, 0, &V18115_4_36, &P18115_5};	/*Property = ScreenPixelsWidth*/
const Profile P18115 = {"18115", &P18115_4};	/*Profile ID - 18115*/
const Value V22485_1_1 = {S2, NULL};	/*Value = True*/
const Property P22485_1 = {S50, 0, &V22485_1_1, NULL};	/*Property = IsMobile*/
const Value V22485_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P22485_5 = {S52, 0, &V22485_5_1275, &P22485_1};	/*Property = ScreenPixelsHeight*/
const Value V22485_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22485_4 = {S54, 0, &V22485_4_1229, &P22485_5};	/*Property = ScreenPixelsWidth*/
const Profile P22485 = {"22485", &P22485_4};	/*Profile ID - 22485*/
const Value V18604_1_1 = {S2, NULL};	/*Value = True*/
const Property P18604_1 = {S50, 0, &V18604_1_1, NULL};	/*Property = IsMobile*/
const Value V18604_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18604_5 = {S52, 0, &V18604_5_1227, &P18604_1};	/*Property = ScreenPixelsHeight*/
const Value V18604_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18604_4 = {S54, 0, &V18604_4_1229, &P18604_5};	/*Property = ScreenPixelsWidth*/
const Profile P18604 = {"18604", &P18604_4};	/*Profile ID - 18604*/
const Value V12787_1_1 = {S2, NULL};	/*Value = True*/
const Property P12787_1 = {S50, 0, &V12787_1_1, NULL};	/*Property = IsMobile*/
const Value V12787_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12787_5 = {S52, 0, &V12787_5_1222, &P12787_1};	/*Property = ScreenPixelsHeight*/
const Value V12787_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12787_4 = {S54, 0, &V12787_4_1224, &P12787_5};	/*Property = ScreenPixelsWidth*/
const Profile P12787 = {"12787", &P12787_4};	/*Profile ID - 12787*/
const Value V18229_1_1 = {S2, NULL};	/*Value = True*/
const Property P18229_1 = {S50, 0, &V18229_1_1, NULL};	/*Property = IsMobile*/
const Value V18229_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18229_5 = {S52, 0, &V18229_5_1222, &P18229_1};	/*Property = ScreenPixelsHeight*/
const Value V18229_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18229_4 = {S54, 0, &V18229_4_1224, &P18229_5};	/*Property = ScreenPixelsWidth*/
const Profile P18229 = {"18229", &P18229_4};	/*Profile ID - 18229*/
const Value V21015_1_1 = {S2, NULL};	/*Value = True*/
const Property P21015_1 = {S50, 0, &V21015_1_1, NULL};	/*Property = IsMobile*/
const Value V21015_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21015_5 = {S52, 0, &V21015_5_9981, &P21015_1};	/*Property = ScreenPixelsHeight*/
const Value V21015_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21015_4 = {S54, 0, &V21015_4_39098, &P21015_5};	/*Property = ScreenPixelsWidth*/
const Profile P21015 = {"21015", &P21015_4};	/*Profile ID - 21015*/
const Value V12803_1_1 = {S2, NULL};	/*Value = True*/
const Property P12803_1 = {S50, 0, &V12803_1_1, NULL};	/*Property = IsMobile*/
const Value V12803_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12803_5 = {S52, 0, &V12803_5_1227, &P12803_1};	/*Property = ScreenPixelsHeight*/
const Value V12803_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12803_4 = {S54, 0, &V12803_4_1229, &P12803_5};	/*Property = ScreenPixelsWidth*/
const Profile P12803 = {"12803", &P12803_4};	/*Profile ID - 12803*/
const Value V12806_1_1 = {S2, NULL};	/*Value = True*/
const Property P12806_1 = {S50, 0, &V12806_1_1, NULL};	/*Property = IsMobile*/
const Value V12806_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P12806_5 = {S52, 0, &V12806_5_1275, &P12806_1};	/*Property = ScreenPixelsHeight*/
const Value V12806_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P12806_4 = {S54, 0, &V12806_4_1278, &P12806_5};	/*Property = ScreenPixelsWidth*/
const Profile P12806 = {"12806", &P12806_4};	/*Profile ID - 12806*/
const Value V17563_1_1 = {S2, NULL};	/*Value = True*/
const Property P17563_1 = {S50, 0, &V17563_1_1, NULL};	/*Property = IsMobile*/
const Value V17563_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17563_5 = {S52, 0, &V17563_5_1277, &P17563_1};	/*Property = ScreenPixelsHeight*/
const Value V17563_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17563_4 = {S54, 0, &V17563_4_1279, &P17563_5};	/*Property = ScreenPixelsWidth*/
const Profile P17563 = {"17563", &P17563_4};	/*Profile ID - 17563*/
const Profile P22857 = {"22857", NULL};	/*Profile ID - 22857*/
const Value V13035_1_1 = {S2, NULL};	/*Value = True*/
const Property P13035_1 = {S50, 0, &V13035_1_1, NULL};	/*Property = IsMobile*/
const Value V13035_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13035_5 = {S52, 0, &V13035_5_1222, &P13035_1};	/*Property = ScreenPixelsHeight*/
const Value V13035_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13035_4 = {S54, 0, &V13035_4_1224, &P13035_5};	/*Property = ScreenPixelsWidth*/
const Profile P13035 = {"13035", &P13035_4};	/*Profile ID - 13035*/
const Value V12936_1_1 = {S2, NULL};	/*Value = True*/
const Property P12936_1 = {S50, 0, &V12936_1_1, NULL};	/*Property = IsMobile*/
const Value V12936_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12936_5 = {S52, 0, &V12936_5_1304, &P12936_1};	/*Property = ScreenPixelsHeight*/
const Value V12936_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12936_4 = {S54, 0, &V12936_4_1224, &P12936_5};	/*Property = ScreenPixelsWidth*/
const Profile P12936 = {"12936", &P12936_4};	/*Profile ID - 12936*/
const Value V17788_1_1 = {S2, NULL};	/*Value = True*/
const Property P17788_1 = {S50, 0, &V17788_1_1, NULL};	/*Property = IsMobile*/
const Value V17788_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17788_5 = {S52, 0, &V17788_5_1222, &P17788_1};	/*Property = ScreenPixelsHeight*/
const Value V17788_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17788_4 = {S54, 0, &V17788_4_1224, &P17788_5};	/*Property = ScreenPixelsWidth*/
const Profile P17788 = {"17788", &P17788_4};	/*Profile ID - 17788*/
const Value V15974_1_1 = {S2, NULL};	/*Value = True*/
const Property P15974_1 = {S50, 0, &V15974_1_1, NULL};	/*Property = IsMobile*/
const Value V15974_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P15974_5 = {S52, 0, &V15974_5_37, &P15974_1};	/*Property = ScreenPixelsHeight*/
const Value V15974_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P15974_4 = {S54, 0, &V15974_4_36, &P15974_5};	/*Property = ScreenPixelsWidth*/
const Profile P15974 = {"15974", &P15974_4};	/*Profile ID - 15974*/
const Value V12940_1_1 = {S2, NULL};	/*Value = True*/
const Property P12940_1 = {S50, 0, &V12940_1_1, NULL};	/*Property = IsMobile*/
const Value V12940_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12940_5 = {S52, 0, &V12940_5_1222, &P12940_1};	/*Property = ScreenPixelsHeight*/
const Value V12940_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12940_4 = {S54, 0, &V12940_4_1224, &P12940_5};	/*Property = ScreenPixelsWidth*/
const Profile P12940 = {"12940", &P12940_4};	/*Profile ID - 12940*/
const Value V12941_1_1 = {S2, NULL};	/*Value = True*/
const Property P12941_1 = {S50, 0, &V12941_1_1, NULL};	/*Property = IsMobile*/
const Value V12941_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12941_5 = {S52, 0, &V12941_5_1222, &P12941_1};	/*Property = ScreenPixelsHeight*/
const Value V12941_4_1368 = {S74, NULL};	/*Value = 208*/
const Property P12941_4 = {S54, 0, &V12941_4_1368, &P12941_5};	/*Property = ScreenPixelsWidth*/
const Profile P12941 = {"12941", &P12941_4};	/*Profile ID - 12941*/
const Value V12942_1_1 = {S2, NULL};	/*Value = True*/
const Property P12942_1 = {S50, 0, &V12942_1_1, NULL};	/*Property = IsMobile*/
const Value V12942_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12942_5 = {S52, 0, &V12942_5_1227, &P12942_1};	/*Property = ScreenPixelsHeight*/
const Value V12942_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12942_4 = {S54, 0, &V12942_4_1229, &P12942_5};	/*Property = ScreenPixelsWidth*/
const Profile P12942 = {"12942", &P12942_4};	/*Profile ID - 12942*/
const Value V13873_1_1 = {S2, NULL};	/*Value = True*/
const Property P13873_1 = {S50, 0, &V13873_1_1, NULL};	/*Property = IsMobile*/
const Value V13873_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13873_5 = {S52, 0, &V13873_5_1304, &P13873_1};	/*Property = ScreenPixelsHeight*/
const Value V13873_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13873_4 = {S54, 0, &V13873_4_1279, &P13873_5};	/*Property = ScreenPixelsWidth*/
const Profile P13873 = {"13873", &P13873_4};	/*Profile ID - 13873*/
const Value V12730_1_1 = {S2, NULL};	/*Value = True*/
const Property P12730_1 = {S50, 0, &V12730_1_1, NULL};	/*Property = IsMobile*/
const Value V12730_5_37540 = {S105, NULL};	/*Value = 427*/
const Property P12730_5 = {S52, 0, &V12730_5_37540, &P12730_1};	/*Property = ScreenPixelsHeight*/
const Value V12730_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12730_4 = {S54, 0, &V12730_4_1224, &P12730_5};	/*Property = ScreenPixelsWidth*/
const Profile P12730 = {"12730", &P12730_4};	/*Profile ID - 12730*/
const Value V13880_1_1 = {S2, NULL};	/*Value = True*/
const Property P13880_1 = {S50, 0, &V13880_1_1, NULL};	/*Property = IsMobile*/
const Value V13880_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P13880_5 = {S52, 0, &V13880_5_1269, &P13880_1};	/*Property = ScreenPixelsHeight*/
const Value V13880_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13880_4 = {S54, 0, &V13880_4_1229, &P13880_5};	/*Property = ScreenPixelsWidth*/
const Profile P13880 = {"13880", &P13880_4};	/*Profile ID - 13880*/
const Value V13881_1_1 = {S2, NULL};	/*Value = True*/
const Property P13881_1 = {S50, 0, &V13881_1_1, NULL};	/*Property = IsMobile*/
const Value V13881_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13881_5 = {S52, 0, &V13881_5_1233, &P13881_1};	/*Property = ScreenPixelsHeight*/
const Value V13881_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13881_4 = {S54, 0, &V13881_4_1235, &P13881_5};	/*Property = ScreenPixelsWidth*/
const Profile P13881 = {"13881", &P13881_4};	/*Profile ID - 13881*/
const Value V13883_1_1 = {S2, NULL};	/*Value = True*/
const Property P13883_1 = {S50, 0, &V13883_1_1, NULL};	/*Property = IsMobile*/
const Value V13883_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13883_5 = {S52, 0, &V13883_5_1222, &P13883_1};	/*Property = ScreenPixelsHeight*/
const Value V13883_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13883_4 = {S54, 0, &V13883_4_1224, &P13883_5};	/*Property = ScreenPixelsWidth*/
const Profile P13883 = {"13883", &P13883_4};	/*Profile ID - 13883*/
const Profile P22846 = {"22846", NULL};	/*Profile ID - 22846*/
const Value V13888_1_1 = {S2, NULL};	/*Value = True*/
const Property P13888_1 = {S50, 0, &V13888_1_1, NULL};	/*Property = IsMobile*/
const Value V13888_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13888_5 = {S52, 0, &V13888_5_1222, &P13888_1};	/*Property = ScreenPixelsHeight*/
const Value V13888_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13888_4 = {S54, 0, &V13888_4_1224, &P13888_5};	/*Property = ScreenPixelsWidth*/
const Profile P13888 = {"13888", &P13888_4};	/*Profile ID - 13888*/
const Value V13886_1_1 = {S2, NULL};	/*Value = True*/
const Property P13886_1 = {S50, 0, &V13886_1_1, NULL};	/*Property = IsMobile*/
const Value V13886_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13886_5 = {S52, 0, &V13886_5_1222, &P13886_1};	/*Property = ScreenPixelsHeight*/
const Value V13886_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13886_4 = {S54, 0, &V13886_4_1224, &P13886_5};	/*Property = ScreenPixelsWidth*/
const Profile P13886 = {"13886", &P13886_4};	/*Profile ID - 13886*/
const Value V13890_1_1 = {S2, NULL};	/*Value = True*/
const Property P13890_1 = {S50, 0, &V13890_1_1, NULL};	/*Property = IsMobile*/
const Value V13890_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13890_5 = {S52, 0, &V13890_5_1222, &P13890_1};	/*Property = ScreenPixelsHeight*/
const Value V13890_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13890_4 = {S54, 0, &V13890_4_1224, &P13890_5};	/*Property = ScreenPixelsWidth*/
const Profile P13890 = {"13890", &P13890_4};	/*Profile ID - 13890*/
const Value V13897_1_1 = {S2, NULL};	/*Value = True*/
const Property P13897_1 = {S50, 0, &V13897_1_1, NULL};	/*Property = IsMobile*/
const Value V13897_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13897_5 = {S52, 0, &V13897_5_1222, &P13897_1};	/*Property = ScreenPixelsHeight*/
const Value V13897_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13897_4 = {S54, 0, &V13897_4_1224, &P13897_5};	/*Property = ScreenPixelsWidth*/
const Profile P13897 = {"13897", &P13897_4};	/*Profile ID - 13897*/
const Value V14631_1_1 = {S2, NULL};	/*Value = True*/
const Property P14631_1 = {S50, 0, &V14631_1_1, NULL};	/*Property = IsMobile*/
const Value V14631_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14631_5 = {S52, 0, &V14631_5_1304, &P14631_1};	/*Property = ScreenPixelsHeight*/
const Value V14631_4_43771 = {S80, NULL};	/*Value = 200*/
const Property P14631_4 = {S54, 0, &V14631_4_43771, &P14631_5};	/*Property = ScreenPixelsWidth*/
const Profile P14631 = {"14631", &P14631_4};	/*Profile ID - 14631*/
const Value V13898_1_1 = {S2, NULL};	/*Value = True*/
const Property P13898_1 = {S50, 0, &V13898_1_1, NULL};	/*Property = IsMobile*/
const Value V13898_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13898_5 = {S52, 0, &V13898_5_1294, &P13898_1};	/*Property = ScreenPixelsHeight*/
const Value V13898_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13898_4 = {S54, 0, &V13898_4_1296, &P13898_5};	/*Property = ScreenPixelsWidth*/
const Profile P13898 = {"13898", &P13898_4};	/*Profile ID - 13898*/
const Value V13905_1_1 = {S2, NULL};	/*Value = True*/
const Property P13905_1 = {S50, 0, &V13905_1_1, NULL};	/*Property = IsMobile*/
const Value V13905_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13905_5 = {S52, 0, &V13905_5_1304, &P13905_1};	/*Property = ScreenPixelsHeight*/
const Value V13905_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13905_4 = {S54, 0, &V13905_4_1224, &P13905_5};	/*Property = ScreenPixelsWidth*/
const Profile P13905 = {"13905", &P13905_4};	/*Profile ID - 13905*/
const Value V16617_1_1 = {S2, NULL};	/*Value = True*/
const Property P16617_1 = {S50, 0, &V16617_1_1, NULL};	/*Property = IsMobile*/
const Value V16617_5_37488 = {S106, NULL};	/*Value = 256*/
const Property P16617_5 = {S52, 0, &V16617_5_37488, &P16617_1};	/*Property = ScreenPixelsHeight*/
const Value V16617_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16617_4 = {S54, 0, &V16617_4_1279, &P16617_5};	/*Property = ScreenPixelsWidth*/
const Profile P16617 = {"16617", &P16617_4};	/*Profile ID - 16617*/
const Value V13213_1_1 = {S2, NULL};	/*Value = True*/
const Property P13213_1 = {S50, 0, &V13213_1_1, NULL};	/*Property = IsMobile*/
const Value V13213_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13213_5 = {S52, 0, &V13213_5_1222, &P13213_1};	/*Property = ScreenPixelsHeight*/
const Value V13213_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13213_4 = {S54, 0, &V13213_4_1224, &P13213_5};	/*Property = ScreenPixelsWidth*/
const Profile P13213 = {"13213", &P13213_4};	/*Profile ID - 13213*/
const Value V13908_1_1 = {S2, NULL};	/*Value = True*/
const Property P13908_1 = {S50, 0, &V13908_1_1, NULL};	/*Property = IsMobile*/
const Value V13908_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13908_5 = {S52, 0, &V13908_5_1222, &P13908_1};	/*Property = ScreenPixelsHeight*/
const Value V13908_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13908_4 = {S54, 0, &V13908_4_1224, &P13908_5};	/*Property = ScreenPixelsWidth*/
const Profile P13908 = {"13908", &P13908_4};	/*Profile ID - 13908*/
const Value V12779_1_1 = {S2, NULL};	/*Value = True*/
const Property P12779_1 = {S50, 0, &V12779_1_1, NULL};	/*Property = IsMobile*/
const Value V12779_5_1796 = {S64, NULL};	/*Value = 360*/
const Property P12779_5 = {S52, 0, &V12779_5_1796, &P12779_1};	/*Property = ScreenPixelsHeight*/
const Value V12779_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12779_4 = {S54, 0, &V12779_4_1229, &P12779_5};	/*Property = ScreenPixelsWidth*/
const Profile P12779 = {"12779", &P12779_4};	/*Profile ID - 12779*/
const Value V12945_1_1 = {S2, NULL};	/*Value = True*/
const Property P12945_1 = {S50, 0, &V12945_1_1, NULL};	/*Property = IsMobile*/
const Value V12945_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12945_5 = {S52, 0, &V12945_5_1222, &P12945_1};	/*Property = ScreenPixelsHeight*/
const Value V12945_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12945_4 = {S54, 0, &V12945_4_1224, &P12945_5};	/*Property = ScreenPixelsWidth*/
const Profile P12945 = {"12945", &P12945_4};	/*Profile ID - 12945*/
const Value V14426_1_1 = {S2, NULL};	/*Value = True*/
const Property P14426_1 = {S50, 0, &V14426_1_1, NULL};	/*Property = IsMobile*/
const Value V14426_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14426_5 = {S52, 0, &V14426_5_1294, &P14426_1};	/*Property = ScreenPixelsHeight*/
const Value V14426_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14426_4 = {S54, 0, &V14426_4_1296, &P14426_5};	/*Property = ScreenPixelsWidth*/
const Profile P14426 = {"14426", &P14426_4};	/*Profile ID - 14426*/
const Value V12946_1_1 = {S2, NULL};	/*Value = True*/
const Property P12946_1 = {S50, 0, &V12946_1_1, NULL};	/*Property = IsMobile*/
const Value V12946_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12946_5 = {S52, 0, &V12946_5_1293, &P12946_1};	/*Property = ScreenPixelsHeight*/
const Value V12946_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12946_4 = {S54, 0, &V12946_4_1296, &P12946_5};	/*Property = ScreenPixelsWidth*/
const Profile P12946 = {"12946", &P12946_4};	/*Profile ID - 12946*/
const Value V12950_1_1 = {S2, NULL};	/*Value = True*/
const Property P12950_1 = {S50, 0, &V12950_1_1, NULL};	/*Property = IsMobile*/
const Value V12950_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12950_5 = {S52, 0, &V12950_5_1293, &P12950_1};	/*Property = ScreenPixelsHeight*/
const Value V12950_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12950_4 = {S54, 0, &V12950_4_1296, &P12950_5};	/*Property = ScreenPixelsWidth*/
const Profile P12950 = {"12950", &P12950_4};	/*Profile ID - 12950*/
const Value V12951_1_1 = {S2, NULL};	/*Value = True*/
const Property P12951_1 = {S50, 0, &V12951_1_1, NULL};	/*Property = IsMobile*/
const Value V12951_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12951_5 = {S52, 0, &V12951_5_1294, &P12951_1};	/*Property = ScreenPixelsHeight*/
const Value V12951_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12951_4 = {S54, 0, &V12951_4_1296, &P12951_5};	/*Property = ScreenPixelsWidth*/
const Profile P12951 = {"12951", &P12951_4};	/*Profile ID - 12951*/
const Value V12952_1_1 = {S2, NULL};	/*Value = True*/
const Property P12952_1 = {S50, 0, &V12952_1_1, NULL};	/*Property = IsMobile*/
const Value V12952_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12952_5 = {S52, 0, &V12952_5_1293, &P12952_1};	/*Property = ScreenPixelsHeight*/
const Value V12952_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12952_4 = {S54, 0, &V12952_4_1296, &P12952_5};	/*Property = ScreenPixelsWidth*/
const Profile P12952 = {"12952", &P12952_4};	/*Profile ID - 12952*/
const Value V22684_1_1 = {S2, NULL};	/*Value = True*/
const Property P22684_1 = {S50, 0, &V22684_1_1, NULL};	/*Property = IsMobile*/
const Value V22684_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22684_5 = {S52, 0, &V22684_5_1227, &P22684_1};	/*Property = ScreenPixelsHeight*/
const Value V22684_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22684_4 = {S54, 0, &V22684_4_1229, &P22684_5};	/*Property = ScreenPixelsWidth*/
const Profile P22684 = {"22684", &P22684_4};	/*Profile ID - 22684*/
const Value V14272_1_1 = {S2, NULL};	/*Value = True*/
const Property P14272_1 = {S50, 0, &V14272_1_1, NULL};	/*Property = IsMobile*/
const Value V14272_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14272_5 = {S52, 0, &V14272_5_1293, &P14272_1};	/*Property = ScreenPixelsHeight*/
const Value V14272_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14272_4 = {S54, 0, &V14272_4_1296, &P14272_5};	/*Property = ScreenPixelsWidth*/
const Profile P14272 = {"14272", &P14272_4};	/*Profile ID - 14272*/
const Value V20603_1_1 = {S2, NULL};	/*Value = True*/
const Property P20603_1 = {S50, 0, &V20603_1_1, NULL};	/*Property = IsMobile*/
const Value V20603_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20603_5 = {S52, 0, &V20603_5_1277, &P20603_1};	/*Property = ScreenPixelsHeight*/
const Value V20603_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20603_4 = {S54, 0, &V20603_4_1279, &P20603_5};	/*Property = ScreenPixelsWidth*/
const Profile P20603 = {"20603", &P20603_4};	/*Profile ID - 20603*/
const Value V12959_1_1 = {S2, NULL};	/*Value = True*/
const Property P12959_1 = {S50, 0, &V12959_1_1, NULL};	/*Property = IsMobile*/
const Value V12959_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12959_5 = {S52, 0, &V12959_5_1233, &P12959_1};	/*Property = ScreenPixelsHeight*/
const Value V12959_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12959_4 = {S54, 0, &V12959_4_1235, &P12959_5};	/*Property = ScreenPixelsWidth*/
const Profile P12959 = {"12959", &P12959_4};	/*Profile ID - 12959*/
const Value V12958_1_1 = {S2, NULL};	/*Value = True*/
const Property P12958_1 = {S50, 0, &V12958_1_1, NULL};	/*Property = IsMobile*/
const Value V12958_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12958_5 = {S52, 0, &V12958_5_1222, &P12958_1};	/*Property = ScreenPixelsHeight*/
const Value V12958_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12958_4 = {S54, 0, &V12958_4_1224, &P12958_5};	/*Property = ScreenPixelsWidth*/
const Profile P12958 = {"12958", &P12958_4};	/*Profile ID - 12958*/
const Value V12960_1_1 = {S2, NULL};	/*Value = True*/
const Property P12960_1 = {S50, 0, &V12960_1_1, NULL};	/*Property = IsMobile*/
const Value V12960_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12960_5 = {S52, 0, &V12960_5_1304, &P12960_1};	/*Property = ScreenPixelsHeight*/
const Value V12960_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12960_4 = {S54, 0, &V12960_4_1224, &P12960_5};	/*Property = ScreenPixelsWidth*/
const Profile P12960 = {"12960", &P12960_4};	/*Profile ID - 12960*/
const Value V12963_1_1 = {S2, NULL};	/*Value = True*/
const Property P12963_1 = {S50, 0, &V12963_1_1, NULL};	/*Property = IsMobile*/
const Value V12963_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12963_5 = {S52, 0, &V12963_5_1222, &P12963_1};	/*Property = ScreenPixelsHeight*/
const Value V12963_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12963_4 = {S54, 0, &V12963_4_1224, &P12963_5};	/*Property = ScreenPixelsWidth*/
const Profile P12963 = {"12963", &P12963_4};	/*Profile ID - 12963*/
const Value V12962_1_1 = {S2, NULL};	/*Value = True*/
const Property P12962_1 = {S50, 0, &V12962_1_1, NULL};	/*Property = IsMobile*/
const Value V12962_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12962_5 = {S52, 0, &V12962_5_1380, &P12962_1};	/*Property = ScreenPixelsHeight*/
const Value V12962_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12962_4 = {S54, 0, &V12962_4_1279, &P12962_5};	/*Property = ScreenPixelsWidth*/
const Profile P12962 = {"12962", &P12962_4};	/*Profile ID - 12962*/
const Value V12961_1_1 = {S2, NULL};	/*Value = True*/
const Property P12961_1 = {S50, 0, &V12961_1_1, NULL};	/*Property = IsMobile*/
const Value V12961_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12961_5 = {S52, 0, &V12961_5_1277, &P12961_1};	/*Property = ScreenPixelsHeight*/
const Value V12961_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12961_4 = {S54, 0, &V12961_4_1279, &P12961_5};	/*Property = ScreenPixelsWidth*/
const Profile P12961 = {"12961", &P12961_4};	/*Profile ID - 12961*/
const Value V12964_1_1 = {S2, NULL};	/*Value = True*/
const Property P12964_1 = {S50, 0, &V12964_1_1, NULL};	/*Property = IsMobile*/
const Value V12964_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12964_5 = {S52, 0, &V12964_5_1227, &P12964_1};	/*Property = ScreenPixelsHeight*/
const Value V12964_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12964_4 = {S54, 0, &V12964_4_1229, &P12964_5};	/*Property = ScreenPixelsWidth*/
const Profile P12964 = {"12964", &P12964_4};	/*Profile ID - 12964*/
const Value V12965_1_1 = {S2, NULL};	/*Value = True*/
const Property P12965_1 = {S50, 0, &V12965_1_1, NULL};	/*Property = IsMobile*/
const Value V12965_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12965_5 = {S52, 0, &V12965_5_1222, &P12965_1};	/*Property = ScreenPixelsHeight*/
const Value V12965_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12965_4 = {S54, 0, &V12965_4_1224, &P12965_5};	/*Property = ScreenPixelsWidth*/
const Profile P12965 = {"12965", &P12965_4};	/*Profile ID - 12965*/
const Value V12967_1_1 = {S2, NULL};	/*Value = True*/
const Property P12967_1 = {S50, 0, &V12967_1_1, NULL};	/*Property = IsMobile*/
const Value V12967_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12967_5 = {S52, 0, &V12967_5_1222, &P12967_1};	/*Property = ScreenPixelsHeight*/
const Value V12967_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12967_4 = {S54, 0, &V12967_4_1224, &P12967_5};	/*Property = ScreenPixelsWidth*/
const Profile P12967 = {"12967", &P12967_4};	/*Profile ID - 12967*/
const Value V12971_1_1 = {S2, NULL};	/*Value = True*/
const Property P12971_1 = {S50, 0, &V12971_1_1, NULL};	/*Property = IsMobile*/
const Value V12971_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P12971_5 = {S52, 0, &V12971_5_1275, &P12971_1};	/*Property = ScreenPixelsHeight*/
const Value V12971_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12971_4 = {S54, 0, &V12971_4_1229, &P12971_5};	/*Property = ScreenPixelsWidth*/
const Profile P12971 = {"12971", &P12971_4};	/*Profile ID - 12971*/
const Value V12972_1_1 = {S2, NULL};	/*Value = True*/
const Property P12972_1 = {S50, 0, &V12972_1_1, NULL};	/*Property = IsMobile*/
const Value V12972_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12972_5 = {S52, 0, &V12972_5_1227, &P12972_1};	/*Property = ScreenPixelsHeight*/
const Value V12972_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12972_4 = {S54, 0, &V12972_4_1229, &P12972_5};	/*Property = ScreenPixelsWidth*/
const Profile P12972 = {"12972", &P12972_4};	/*Profile ID - 12972*/
const Value V12974_1_1 = {S2, NULL};	/*Value = True*/
const Property P12974_1 = {S50, 0, &V12974_1_1, NULL};	/*Property = IsMobile*/
const Value V12974_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12974_5 = {S52, 0, &V12974_5_1222, &P12974_1};	/*Property = ScreenPixelsHeight*/
const Value V12974_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12974_4 = {S54, 0, &V12974_4_1224, &P12974_5};	/*Property = ScreenPixelsWidth*/
const Profile P12974 = {"12974", &P12974_4};	/*Profile ID - 12974*/
const Value V12976_1_1 = {S2, NULL};	/*Value = True*/
const Property P12976_1 = {S50, 0, &V12976_1_1, NULL};	/*Property = IsMobile*/
const Value V12976_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12976_5 = {S52, 0, &V12976_5_1233, &P12976_1};	/*Property = ScreenPixelsHeight*/
const Value V12976_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12976_4 = {S54, 0, &V12976_4_1235, &P12976_5};	/*Property = ScreenPixelsWidth*/
const Profile P12976 = {"12976", &P12976_4};	/*Profile ID - 12976*/
const Value V12979_1_1 = {S2, NULL};	/*Value = True*/
const Property P12979_1 = {S50, 0, &V12979_1_1, NULL};	/*Property = IsMobile*/
const Value V12979_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12979_5 = {S52, 0, &V12979_5_1222, &P12979_1};	/*Property = ScreenPixelsHeight*/
const Value V12979_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12979_4 = {S54, 0, &V12979_4_1224, &P12979_5};	/*Property = ScreenPixelsWidth*/
const Profile P12979 = {"12979", &P12979_4};	/*Profile ID - 12979*/
const Value V14959_1_1 = {S2, NULL};	/*Value = True*/
const Property P14959_1 = {S50, 0, &V14959_1_1, NULL};	/*Property = IsMobile*/
const Value V14959_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14959_5 = {S52, 0, &V14959_5_1227, &P14959_1};	/*Property = ScreenPixelsHeight*/
const Value V14959_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14959_4 = {S54, 0, &V14959_4_1229, &P14959_5};	/*Property = ScreenPixelsWidth*/
const Profile P14959 = {"14959", &P14959_4};	/*Profile ID - 14959*/
const Value V12980_1_1 = {S2, NULL};	/*Value = True*/
const Property P12980_1 = {S50, 0, &V12980_1_1, NULL};	/*Property = IsMobile*/
const Value V12980_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12980_5 = {S52, 0, &V12980_5_1277, &P12980_1};	/*Property = ScreenPixelsHeight*/
const Value V12980_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12980_4 = {S54, 0, &V12980_4_1279, &P12980_5};	/*Property = ScreenPixelsWidth*/
const Profile P12980 = {"12980", &P12980_4};	/*Profile ID - 12980*/
const Value V16694_1_1 = {S2, NULL};	/*Value = True*/
const Property P16694_1 = {S50, 0, &V16694_1_1, NULL};	/*Property = IsMobile*/
const Value V16694_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16694_5 = {S52, 0, &V16694_5_1293, &P16694_1};	/*Property = ScreenPixelsHeight*/
const Value V16694_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16694_4 = {S54, 0, &V16694_4_1296, &P16694_5};	/*Property = ScreenPixelsWidth*/
const Profile P16694 = {"16694", &P16694_4};	/*Profile ID - 16694*/
const Value V12983_1_1 = {S2, NULL};	/*Value = True*/
const Property P12983_1 = {S50, 0, &V12983_1_1, NULL};	/*Property = IsMobile*/
const Value V12983_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12983_5 = {S52, 0, &V12983_5_1293, &P12983_1};	/*Property = ScreenPixelsHeight*/
const Value V12983_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12983_4 = {S54, 0, &V12983_4_1296, &P12983_5};	/*Property = ScreenPixelsWidth*/
const Profile P12983 = {"12983", &P12983_4};	/*Profile ID - 12983*/
const Value V4812_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_279 = {S1, 0, &V4812_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4812_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_282 = {S3, 0, &V4812_282_57155, &P4812_279};	/*Property = BlobBuilder*/
const Value V4812_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_350 = {S4, 0, &V4812_350_57446, &P4812_282};	/*Property = Canvas*/
const Value V4812_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_283 = {S5, 0, &V4812_283_57161, &P4812_350};	/*Property = CssBackground*/
const Value V4812_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_284 = {S6, 0, &V4812_284_57164, &P4812_283};	/*Property = CssBorderImage*/
const Value V4812_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_285 = {S7, 0, &V4812_285_57167, &P4812_284};	/*Property = CssCanvas*/
const Value V4812_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_286 = {S8, 0, &V4812_286_57170, &P4812_285};	/*Property = CssColor*/
const Value V4812_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_237 = {S9, 0, &V4812_237_55786, &P4812_286};	/*Property = CssColumn*/
const Value V4812_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_287 = {S10, 0, &V4812_287_57173, &P4812_237};	/*Property = CssFlexbox*/
const Value V4812_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_288 = {S11, 0, &V4812_288_57176, &P4812_287};	/*Property = CssFont*/
const Value V4812_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_289 = {S12, 0, &V4812_289_86899, &P4812_288};	/*Property = CssImages*/
const Value V4812_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_291 = {S13, 0, &V4812_291_57182, &P4812_289};	/*Property = CssMediaQueries*/
const Value V4812_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_292 = {S14, 0, &V4812_292_57185, &P4812_291};	/*Property = CssMinMax*/
const Value V4812_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_293 = {S15, 0, &V4812_293_57188, &P4812_292};	/*Property = CssOverflow*/
const Value V4812_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_294 = {S16, 0, &V4812_294_57191, &P4812_293};	/*Property = CssPosition*/
const Value V4812_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_295 = {S17, 0, &V4812_295_57194, &P4812_294};	/*Property = CssText*/
const Value V4812_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_297 = {S18, 0, &V4812_297_57197, &P4812_295};	/*Property = CssTransforms*/
const Value V4812_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_241 = {S19, 0, &V4812_241_55801, &P4812_297};	/*Property = CssTransitions*/
const Value V4812_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_298 = {S20, 0, &V4812_298_57208, &P4812_241};	/*Property = CssUI*/
const Value V4812_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_300 = {S21, 0, &V4812_300_57217, &P4812_298};	/*Property = DataSet*/
const Value V4812_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_301 = {S22, 0, &V4812_301_57220, &P4812_300};	/*Property = DataUrl*/
const Value V4812_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_302 = {S23, 0, &V4812_302_57223, &P4812_301};	/*Property = DeviceOrientation*/
const Value V4812_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_304 = {S24, 0, &V4812_304_57229, &P4812_302};	/*Property = FileReader*/
const Value V4812_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_305 = {S25, 0, &V4812_305_57232, &P4812_304};	/*Property = FileSaver*/
const Value V4812_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_306 = {S26, 0, &V4812_306_57238, &P4812_305};	/*Property = FileWriter*/
const Value V4812_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_307 = {S27, 0, &V4812_307_57241, &P4812_306};	/*Property = FormData*/
const Value V4812_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_309 = {S28, 0, &V4812_309_57247, &P4812_307};	/*Property = Fullscreen*/
const Value V4812_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_310 = {S29, 0, &V4812_310_57250, &P4812_309};	/*Property = GeoLocation*/
const Value V4812_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_312 = {S30, 0, &V4812_312_57256, &P4812_310};	/*Property = History*/
const Value V4812_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_314 = {S31, 0, &V4812_314_57262, &P4812_312};	/*Property = Html5*/
const Value V4812_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_313 = {S32, 0, &V4812_313_57259, &P4812_314};	/*Property = Html-Media-Capture*/
const Value V4812_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_315 = {S33, 0, &V4812_315_57265, &P4812_313};	/*Property = Iframe*/
const Value V4812_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_316 = {S34, 0, &V4812_316_57268, &P4812_315};	/*Property = IndexedDB*/
const Value V4812_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_317 = {S35, 0, &V4812_317_57271, &P4812_316};	/*Property = Json*/
const Value V4812_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P4812_15 = {S37, 0, &V4812_15_31, &P4812_317};	/*Property = LayoutEngine*/
const Value V4812_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_344 = {S38, 0, &V4812_344_57274, &P4812_15};	/*Property = Masking*/
const Value V4812_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_320 = {S39, 0, &V4812_320_57292, &P4812_344};	/*Property = PostMessage*/
const Value V4812_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_321 = {S40, 0, &V4812_321_57295, &P4812_320};	/*Property = Progress*/
const Value V4812_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_322 = {S41, 0, &V4812_322_57298, &P4812_321};	/*Property = Prompts*/
const Value V4812_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_323 = {S42, 0, &V4812_323_57301, &P4812_322};	/*Property = Selector*/
const Value V4812_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_324 = {S43, 0, &V4812_324_57307, &P4812_323};	/*Property = Svg*/
const Value V4812_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_327 = {S44, 0, &V4812_327_57316, &P4812_324};	/*Property = TouchEvents*/
const Value V4812_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_328 = {S45, 0, &V4812_328_57319, &P4812_327};	/*Property = Track*/
const Value V4812_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_329 = {S46, 0, &V4812_329_57325, &P4812_328};	/*Property = Video*/
const Value V4812_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_330 = {S47, 0, &V4812_330_57328, &P4812_329};	/*Property = Viewport*/
const Value V4812_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_332 = {S48, 0, &V4812_332_57340, &P4812_330};	/*Property = WebWorkers*/
const Value V4812_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4812_333 = {S49, 0, &V4812_333_57343, &P4812_332};	/*Property = Xhr2*/
const Profile P4812 = {"4812", &P4812_333};	/*Profile ID - 4812*/
const Value V12715_1_1 = {S2, NULL};	/*Value = True*/
const Property P12715_1 = {S50, 0, &V12715_1_1, NULL};	/*Property = IsMobile*/
const Value V12715_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12715_5 = {S52, 0, &V12715_5_1304, &P12715_1};	/*Property = ScreenPixelsHeight*/
const Value V12715_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12715_4 = {S54, 0, &V12715_4_1224, &P12715_5};	/*Property = ScreenPixelsWidth*/
const Profile P12715 = {"12715", &P12715_4};	/*Profile ID - 12715*/
const Value V12987_1_1 = {S2, NULL};	/*Value = True*/
const Property P12987_1 = {S50, 0, &V12987_1_1, NULL};	/*Property = IsMobile*/
const Value V12987_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12987_5 = {S52, 0, &V12987_5_1277, &P12987_1};	/*Property = ScreenPixelsHeight*/
const Value V12987_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12987_4 = {S54, 0, &V12987_4_1279, &P12987_5};	/*Property = ScreenPixelsWidth*/
const Profile P12987 = {"12987", &P12987_4};	/*Profile ID - 12987*/
const Value V12988_1_1 = {S2, NULL};	/*Value = True*/
const Property P12988_1 = {S50, 0, &V12988_1_1, NULL};	/*Property = IsMobile*/
const Value V12988_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12988_5 = {S52, 0, &V12988_5_1293, &P12988_1};	/*Property = ScreenPixelsHeight*/
const Value V12988_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12988_4 = {S54, 0, &V12988_4_1296, &P12988_5};	/*Property = ScreenPixelsWidth*/
const Profile P12988 = {"12988", &P12988_4};	/*Profile ID - 12988*/
const Value V12989_1_1 = {S2, NULL};	/*Value = True*/
const Property P12989_1 = {S50, 0, &V12989_1_1, NULL};	/*Property = IsMobile*/
const Value V12989_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12989_5 = {S52, 0, &V12989_5_1222, &P12989_1};	/*Property = ScreenPixelsHeight*/
const Value V12989_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12989_4 = {S54, 0, &V12989_4_1224, &P12989_5};	/*Property = ScreenPixelsWidth*/
const Profile P12989 = {"12989", &P12989_4};	/*Profile ID - 12989*/
const Value V12992_1_1 = {S2, NULL};	/*Value = True*/
const Property P12992_1 = {S50, 0, &V12992_1_1, NULL};	/*Property = IsMobile*/
const Value V12992_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12992_5 = {S52, 0, &V12992_5_1222, &P12992_1};	/*Property = ScreenPixelsHeight*/
const Value V12992_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12992_4 = {S54, 0, &V12992_4_1224, &P12992_5};	/*Property = ScreenPixelsWidth*/
const Profile P12992 = {"12992", &P12992_4};	/*Profile ID - 12992*/
const Value V12991_1_1 = {S2, NULL};	/*Value = True*/
const Property P12991_1 = {S50, 0, &V12991_1_1, NULL};	/*Property = IsMobile*/
const Value V12991_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12991_5 = {S52, 0, &V12991_5_1380, &P12991_1};	/*Property = ScreenPixelsHeight*/
const Value V12991_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12991_4 = {S54, 0, &V12991_4_1279, &P12991_5};	/*Property = ScreenPixelsWidth*/
const Profile P12991 = {"12991", &P12991_4};	/*Profile ID - 12991*/
const Value V14209_1_1 = {S2, NULL};	/*Value = True*/
const Property P14209_1 = {S50, 0, &V14209_1_1, NULL};	/*Property = IsMobile*/
const Value V14209_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14209_5 = {S52, 0, &V14209_5_1293, &P14209_1};	/*Property = ScreenPixelsHeight*/
const Value V14209_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14209_4 = {S54, 0, &V14209_4_1296, &P14209_5};	/*Property = ScreenPixelsWidth*/
const Profile P14209 = {"14209", &P14209_4};	/*Profile ID - 14209*/
const Value V12995_1_1 = {S2, NULL};	/*Value = True*/
const Property P12995_1 = {S50, 0, &V12995_1_1, NULL};	/*Property = IsMobile*/
const Value V12995_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12995_5 = {S52, 0, &V12995_5_1222, &P12995_1};	/*Property = ScreenPixelsHeight*/
const Value V12995_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12995_4 = {S54, 0, &V12995_4_1224, &P12995_5};	/*Property = ScreenPixelsWidth*/
const Profile P12995 = {"12995", &P12995_4};	/*Profile ID - 12995*/
const Value V12997_1_1 = {S2, NULL};	/*Value = True*/
const Property P12997_1 = {S50, 0, &V12997_1_1, NULL};	/*Property = IsMobile*/
const Value V12997_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12997_5 = {S52, 0, &V12997_5_1294, &P12997_1};	/*Property = ScreenPixelsHeight*/
const Value V12997_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12997_4 = {S54, 0, &V12997_4_1296, &P12997_5};	/*Property = ScreenPixelsWidth*/
const Profile P12997 = {"12997", &P12997_4};	/*Profile ID - 12997*/
const Value V17709_1_1 = {S2, NULL};	/*Value = True*/
const Property P17709_1 = {S50, 0, &V17709_1_1, NULL};	/*Property = IsMobile*/
const Value V17709_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17709_5 = {S52, 0, &V17709_5_1222, &P17709_1};	/*Property = ScreenPixelsHeight*/
const Value V17709_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17709_4 = {S54, 0, &V17709_4_1279, &P17709_5};	/*Property = ScreenPixelsWidth*/
const Profile P17709 = {"17709", &P17709_4};	/*Profile ID - 17709*/
const Value V13001_1_1 = {S2, NULL};	/*Value = True*/
const Property P13001_1 = {S50, 0, &V13001_1_1, NULL};	/*Property = IsMobile*/
const Value V13001_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13001_5 = {S52, 0, &V13001_5_1222, &P13001_1};	/*Property = ScreenPixelsHeight*/
const Value V13001_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13001_4 = {S54, 0, &V13001_4_1224, &P13001_5};	/*Property = ScreenPixelsWidth*/
const Profile P13001 = {"13001", &P13001_4};	/*Profile ID - 13001*/
const Value V13003_1_1 = {S2, NULL};	/*Value = True*/
const Property P13003_1 = {S50, 0, &V13003_1_1, NULL};	/*Property = IsMobile*/
const Value V13003_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13003_5 = {S52, 0, &V13003_5_1222, &P13003_1};	/*Property = ScreenPixelsHeight*/
const Value V13003_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13003_4 = {S54, 0, &V13003_4_1224, &P13003_5};	/*Property = ScreenPixelsWidth*/
const Profile P13003 = {"13003", &P13003_4};	/*Profile ID - 13003*/
const Value V13000_1_1 = {S2, NULL};	/*Value = True*/
const Property P13000_1 = {S50, 0, &V13000_1_1, NULL};	/*Property = IsMobile*/
const Value V13000_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13000_5 = {S52, 0, &V13000_5_1222, &P13000_1};	/*Property = ScreenPixelsHeight*/
const Value V13000_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13000_4 = {S54, 0, &V13000_4_1224, &P13000_5};	/*Property = ScreenPixelsWidth*/
const Profile P13000 = {"13000", &P13000_4};	/*Profile ID - 13000*/
const Value V13925_1_1 = {S2, NULL};	/*Value = True*/
const Property P13925_1 = {S50, 0, &V13925_1_1, NULL};	/*Property = IsMobile*/
const Value V13925_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13925_5 = {S52, 0, &V13925_5_1222, &P13925_1};	/*Property = ScreenPixelsHeight*/
const Value V13925_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13925_4 = {S54, 0, &V13925_4_1224, &P13925_5};	/*Property = ScreenPixelsWidth*/
const Profile P13925 = {"13925", &P13925_4};	/*Profile ID - 13925*/
const Value V13010_1_1 = {S2, NULL};	/*Value = True*/
const Property P13010_1 = {S50, 0, &V13010_1_1, NULL};	/*Property = IsMobile*/
const Value V13010_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13010_5 = {S52, 0, &V13010_5_1293, &P13010_1};	/*Property = ScreenPixelsHeight*/
const Value V13010_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13010_4 = {S54, 0, &V13010_4_1296, &P13010_5};	/*Property = ScreenPixelsWidth*/
const Profile P13010 = {"13010", &P13010_4};	/*Profile ID - 13010*/
const Value V16930_1_1 = {S2, NULL};	/*Value = True*/
const Property P16930_1 = {S50, 0, &V16930_1_1, NULL};	/*Property = IsMobile*/
const Value V16930_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P16930_5 = {S52, 0, &V16930_5_1380, &P16930_1};	/*Property = ScreenPixelsHeight*/
const Value V16930_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16930_4 = {S54, 0, &V16930_4_1279, &P16930_5};	/*Property = ScreenPixelsWidth*/
const Profile P16930 = {"16930", &P16930_4};	/*Profile ID - 16930*/
const Value V13011_1_1 = {S2, NULL};	/*Value = True*/
const Property P13011_1 = {S50, 0, &V13011_1_1, NULL};	/*Property = IsMobile*/
const Value V13011_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13011_5 = {S52, 0, &V13011_5_1277, &P13011_1};	/*Property = ScreenPixelsHeight*/
const Value V13011_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13011_4 = {S54, 0, &V13011_4_1279, &P13011_5};	/*Property = ScreenPixelsWidth*/
const Profile P13011 = {"13011", &P13011_4};	/*Profile ID - 13011*/
const Value V13016_1_1 = {S2, NULL};	/*Value = True*/
const Property P13016_1 = {S50, 0, &V13016_1_1, NULL};	/*Property = IsMobile*/
const Value V13016_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P13016_5 = {S52, 0, &V13016_5_1333, &P13016_1};	/*Property = ScreenPixelsHeight*/
const Value V13016_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P13016_4 = {S54, 0, &V13016_4_1335, &P13016_5};	/*Property = ScreenPixelsWidth*/
const Profile P13016 = {"13016", &P13016_4};	/*Profile ID - 13016*/
const Value V13017_1_1 = {S2, NULL};	/*Value = True*/
const Property P13017_1 = {S50, 0, &V13017_1_1, NULL};	/*Property = IsMobile*/
const Value V13017_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13017_5 = {S52, 0, &V13017_5_1233, &P13017_1};	/*Property = ScreenPixelsHeight*/
const Value V13017_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13017_4 = {S54, 0, &V13017_4_1235, &P13017_5};	/*Property = ScreenPixelsWidth*/
const Profile P13017 = {"13017", &P13017_4};	/*Profile ID - 13017*/
const Value V16365_1_1 = {S2, NULL};	/*Value = True*/
const Property P16365_1 = {S50, 0, &V16365_1_1, NULL};	/*Property = IsMobile*/
const Value V16365_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P16365_5 = {S52, 0, &V16365_5_1380, &P16365_1};	/*Property = ScreenPixelsHeight*/
const Value V16365_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16365_4 = {S54, 0, &V16365_4_1279, &P16365_5};	/*Property = ScreenPixelsWidth*/
const Profile P16365 = {"16365", &P16365_4};	/*Profile ID - 16365*/
const Value V13020_1_1 = {S2, NULL};	/*Value = True*/
const Property P13020_1 = {S50, 0, &V13020_1_1, NULL};	/*Property = IsMobile*/
const Value V13020_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13020_5 = {S52, 0, &V13020_5_1222, &P13020_1};	/*Property = ScreenPixelsHeight*/
const Value V13020_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13020_4 = {S54, 0, &V13020_4_1224, &P13020_5};	/*Property = ScreenPixelsWidth*/
const Profile P13020 = {"13020", &P13020_4};	/*Profile ID - 13020*/
const Value V13022_1_1 = {S2, NULL};	/*Value = True*/
const Property P13022_1 = {S50, 0, &V13022_1_1, NULL};	/*Property = IsMobile*/
const Value V13022_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13022_5 = {S52, 0, &V13022_5_1233, &P13022_1};	/*Property = ScreenPixelsHeight*/
const Value V13022_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13022_4 = {S54, 0, &V13022_4_1235, &P13022_5};	/*Property = ScreenPixelsWidth*/
const Profile P13022 = {"13022", &P13022_4};	/*Profile ID - 13022*/
const Value V13023_1_1 = {S2, NULL};	/*Value = True*/
const Property P13023_1 = {S50, 0, &V13023_1_1, NULL};	/*Property = IsMobile*/
const Value V13023_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13023_5 = {S52, 0, &V13023_5_1294, &P13023_1};	/*Property = ScreenPixelsHeight*/
const Value V13023_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13023_4 = {S54, 0, &V13023_4_1296, &P13023_5};	/*Property = ScreenPixelsWidth*/
const Profile P13023 = {"13023", &P13023_4};	/*Profile ID - 13023*/
const Value V13024_1_1 = {S2, NULL};	/*Value = True*/
const Property P13024_1 = {S50, 0, &V13024_1_1, NULL};	/*Property = IsMobile*/
const Value V13024_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13024_5 = {S52, 0, &V13024_5_1222, &P13024_1};	/*Property = ScreenPixelsHeight*/
const Value V13024_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13024_4 = {S54, 0, &V13024_4_1224, &P13024_5};	/*Property = ScreenPixelsWidth*/
const Profile P13024 = {"13024", &P13024_4};	/*Profile ID - 13024*/
const Value V13026_1_1 = {S2, NULL};	/*Value = True*/
const Property P13026_1 = {S50, 0, &V13026_1_1, NULL};	/*Property = IsMobile*/
const Value V13026_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13026_5 = {S52, 0, &V13026_5_1277, &P13026_1};	/*Property = ScreenPixelsHeight*/
const Value V13026_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13026_4 = {S54, 0, &V13026_4_1279, &P13026_5};	/*Property = ScreenPixelsWidth*/
const Profile P13026 = {"13026", &P13026_4};	/*Profile ID - 13026*/
const Value V13027_1_1 = {S2, NULL};	/*Value = True*/
const Property P13027_1 = {S50, 0, &V13027_1_1, NULL};	/*Property = IsMobile*/
const Value V13027_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13027_5 = {S52, 0, &V13027_5_1304, &P13027_1};	/*Property = ScreenPixelsHeight*/
const Value V13027_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13027_4 = {S54, 0, &V13027_4_1224, &P13027_5};	/*Property = ScreenPixelsWidth*/
const Profile P13027 = {"13027", &P13027_4};	/*Profile ID - 13027*/
const Value V13028_1_1 = {S2, NULL};	/*Value = True*/
const Property P13028_1 = {S50, 0, &V13028_1_1, NULL};	/*Property = IsMobile*/
const Value V13028_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13028_5 = {S52, 0, &V13028_5_1293, &P13028_1};	/*Property = ScreenPixelsHeight*/
const Value V13028_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13028_4 = {S54, 0, &V13028_4_1296, &P13028_5};	/*Property = ScreenPixelsWidth*/
const Profile P13028 = {"13028", &P13028_4};	/*Profile ID - 13028*/
const Value V13029_1_1 = {S2, NULL};	/*Value = True*/
const Property P13029_1 = {S50, 0, &V13029_1_1, NULL};	/*Property = IsMobile*/
const Value V13029_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13029_5 = {S52, 0, &V13029_5_1222, &P13029_1};	/*Property = ScreenPixelsHeight*/
const Value V13029_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13029_4 = {S54, 0, &V13029_4_1224, &P13029_5};	/*Property = ScreenPixelsWidth*/
const Profile P13029 = {"13029", &P13029_4};	/*Profile ID - 13029*/
const Value V12937_1_1 = {S2, NULL};	/*Value = True*/
const Property P12937_1 = {S50, 0, &V12937_1_1, NULL};	/*Property = IsMobile*/
const Value V12937_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12937_5 = {S52, 0, &V12937_5_1277, &P12937_1};	/*Property = ScreenPixelsHeight*/
const Value V12937_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12937_4 = {S54, 0, &V12937_4_1279, &P12937_5};	/*Property = ScreenPixelsWidth*/
const Profile P12937 = {"12937", &P12937_4};	/*Profile ID - 12937*/
const Value V13030_1_1 = {S2, NULL};	/*Value = True*/
const Property P13030_1 = {S50, 0, &V13030_1_1, NULL};	/*Property = IsMobile*/
const Value V13030_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P13030_5 = {S52, 0, &V13030_5_1275, &P13030_1};	/*Property = ScreenPixelsHeight*/
const Value V13030_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13030_4 = {S54, 0, &V13030_4_1229, &P13030_5};	/*Property = ScreenPixelsWidth*/
const Profile P13030 = {"13030", &P13030_4};	/*Profile ID - 13030*/
const Value V13033_1_1 = {S2, NULL};	/*Value = True*/
const Property P13033_1 = {S50, 0, &V13033_1_1, NULL};	/*Property = IsMobile*/
const Value V13033_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13033_5 = {S52, 0, &V13033_5_1304, &P13033_1};	/*Property = ScreenPixelsHeight*/
const Value V13033_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13033_4 = {S54, 0, &V13033_4_1224, &P13033_5};	/*Property = ScreenPixelsWidth*/
const Profile P13033 = {"13033", &P13033_4};	/*Profile ID - 13033*/
const Value V13034_1_1 = {S2, NULL};	/*Value = True*/
const Property P13034_1 = {S50, 0, &V13034_1_1, NULL};	/*Property = IsMobile*/
const Value V13034_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P13034_5 = {S52, 0, &V13034_5_1403, &P13034_1};	/*Property = ScreenPixelsHeight*/
const Value V13034_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P13034_4 = {S54, 0, &V13034_4_1404, &P13034_5};	/*Property = ScreenPixelsWidth*/
const Profile P13034 = {"13034", &P13034_4};	/*Profile ID - 13034*/
const Value V13039_1_1 = {S2, NULL};	/*Value = True*/
const Property P13039_1 = {S50, 0, &V13039_1_1, NULL};	/*Property = IsMobile*/
const Value V13039_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13039_5 = {S52, 0, &V13039_5_1294, &P13039_1};	/*Property = ScreenPixelsHeight*/
const Value V13039_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13039_4 = {S54, 0, &V13039_4_1296, &P13039_5};	/*Property = ScreenPixelsWidth*/
const Profile P13039 = {"13039", &P13039_4};	/*Profile ID - 13039*/
const Value V13038_1_1 = {S2, NULL};	/*Value = True*/
const Property P13038_1 = {S50, 0, &V13038_1_1, NULL};	/*Property = IsMobile*/
const Value V13038_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13038_5 = {S52, 0, &V13038_5_1380, &P13038_1};	/*Property = ScreenPixelsHeight*/
const Value V13038_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13038_4 = {S54, 0, &V13038_4_1279, &P13038_5};	/*Property = ScreenPixelsWidth*/
const Profile P13038 = {"13038", &P13038_4};	/*Profile ID - 13038*/
const Value V13032_1_1 = {S2, NULL};	/*Value = True*/
const Property P13032_1 = {S50, 0, &V13032_1_1, NULL};	/*Property = IsMobile*/
const Value V13032_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13032_5 = {S52, 0, &V13032_5_1222, &P13032_1};	/*Property = ScreenPixelsHeight*/
const Value V13032_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13032_4 = {S54, 0, &V13032_4_1224, &P13032_5};	/*Property = ScreenPixelsWidth*/
const Profile P13032 = {"13032", &P13032_4};	/*Profile ID - 13032*/
const Value V13043_1_1 = {S2, NULL};	/*Value = True*/
const Property P13043_1 = {S50, 0, &V13043_1_1, NULL};	/*Property = IsMobile*/
const Value V13043_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13043_5 = {S52, 0, &V13043_5_1294, &P13043_1};	/*Property = ScreenPixelsHeight*/
const Value V13043_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13043_4 = {S54, 0, &V13043_4_1296, &P13043_5};	/*Property = ScreenPixelsWidth*/
const Profile P13043 = {"13043", &P13043_4};	/*Profile ID - 13043*/
const Value V13089_1_1 = {S2, NULL};	/*Value = True*/
const Property P13089_1 = {S50, 0, &V13089_1_1, NULL};	/*Property = IsMobile*/
const Value V13089_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13089_5 = {S52, 0, &V13089_5_1222, &P13089_1};	/*Property = ScreenPixelsHeight*/
const Value V13089_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13089_4 = {S54, 0, &V13089_4_1224, &P13089_5};	/*Property = ScreenPixelsWidth*/
const Profile P13089 = {"13089", &P13089_4};	/*Profile ID - 13089*/
const Value V13044_1_1 = {S2, NULL};	/*Value = True*/
const Property P13044_1 = {S50, 0, &V13044_1_1, NULL};	/*Property = IsMobile*/
const Value V13044_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13044_5 = {S52, 0, &V13044_5_1222, &P13044_1};	/*Property = ScreenPixelsHeight*/
const Value V13044_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13044_4 = {S54, 0, &V13044_4_1224, &P13044_5};	/*Property = ScreenPixelsWidth*/
const Profile P13044 = {"13044", &P13044_4};	/*Profile ID - 13044*/
const Value V13047_1_1 = {S2, NULL};	/*Value = True*/
const Property P13047_1 = {S50, 0, &V13047_1_1, NULL};	/*Property = IsMobile*/
const Value V13047_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13047_5 = {S52, 0, &V13047_5_1233, &P13047_1};	/*Property = ScreenPixelsHeight*/
const Value V13047_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13047_4 = {S54, 0, &V13047_4_1235, &P13047_5};	/*Property = ScreenPixelsWidth*/
const Profile P13047 = {"13047", &P13047_4};	/*Profile ID - 13047*/
const Value V13048_1_1 = {S2, NULL};	/*Value = True*/
const Property P13048_1 = {S50, 0, &V13048_1_1, NULL};	/*Property = IsMobile*/
const Value V13048_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13048_5 = {S52, 0, &V13048_5_1304, &P13048_1};	/*Property = ScreenPixelsHeight*/
const Value V13048_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13048_4 = {S54, 0, &V13048_4_1279, &P13048_5};	/*Property = ScreenPixelsWidth*/
const Profile P13048 = {"13048", &P13048_4};	/*Profile ID - 13048*/
const Value V13051_1_1 = {S2, NULL};	/*Value = True*/
const Property P13051_1 = {S50, 0, &V13051_1_1, NULL};	/*Property = IsMobile*/
const Value V13051_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13051_5 = {S52, 0, &V13051_5_1227, &P13051_1};	/*Property = ScreenPixelsHeight*/
const Value V13051_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13051_4 = {S54, 0, &V13051_4_1229, &P13051_5};	/*Property = ScreenPixelsWidth*/
const Profile P13051 = {"13051", &P13051_4};	/*Profile ID - 13051*/
const Value V13054_1_1 = {S2, NULL};	/*Value = True*/
const Property P13054_1 = {S50, 0, &V13054_1_1, NULL};	/*Property = IsMobile*/
const Value V13054_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P13054_5 = {S52, 0, &V13054_5_9981, &P13054_1};	/*Property = ScreenPixelsHeight*/
const Value V13054_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P13054_4 = {S54, 0, &V13054_4_39098, &P13054_5};	/*Property = ScreenPixelsWidth*/
const Profile P13054 = {"13054", &P13054_4};	/*Profile ID - 13054*/
const Value V14951_1_1 = {S2, NULL};	/*Value = True*/
const Property P14951_1 = {S50, 0, &V14951_1_1, NULL};	/*Property = IsMobile*/
const Value V14951_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14951_5 = {S52, 0, &V14951_5_1293, &P14951_1};	/*Property = ScreenPixelsHeight*/
const Value V14951_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14951_4 = {S54, 0, &V14951_4_1296, &P14951_5};	/*Property = ScreenPixelsWidth*/
const Profile P14951 = {"14951", &P14951_4};	/*Profile ID - 14951*/
const Value V13057_1_1 = {S2, NULL};	/*Value = True*/
const Property P13057_1 = {S50, 0, &V13057_1_1, NULL};	/*Property = IsMobile*/
const Value V13057_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13057_5 = {S52, 0, &V13057_5_1380, &P13057_1};	/*Property = ScreenPixelsHeight*/
const Value V13057_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13057_4 = {S54, 0, &V13057_4_1279, &P13057_5};	/*Property = ScreenPixelsWidth*/
const Profile P13057 = {"13057", &P13057_4};	/*Profile ID - 13057*/
const Value V13060_1_1 = {S2, NULL};	/*Value = True*/
const Property P13060_1 = {S50, 0, &V13060_1_1, NULL};	/*Property = IsMobile*/
const Value V13060_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P13060_5 = {S52, 0, &V13060_5_1403, &P13060_1};	/*Property = ScreenPixelsHeight*/
const Value V13060_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P13060_4 = {S54, 0, &V13060_4_1404, &P13060_5};	/*Property = ScreenPixelsWidth*/
const Profile P13060 = {"13060", &P13060_4};	/*Profile ID - 13060*/
const Value V13061_1_1 = {S2, NULL};	/*Value = True*/
const Property P13061_1 = {S50, 0, &V13061_1_1, NULL};	/*Property = IsMobile*/
const Value V13061_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13061_5 = {S52, 0, &V13061_5_1293, &P13061_1};	/*Property = ScreenPixelsHeight*/
const Value V13061_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13061_4 = {S54, 0, &V13061_4_1296, &P13061_5};	/*Property = ScreenPixelsWidth*/
const Profile P13061 = {"13061", &P13061_4};	/*Profile ID - 13061*/
const Value V12825_1_1 = {S2, NULL};	/*Value = True*/
const Property P12825_1 = {S50, 0, &V12825_1_1, NULL};	/*Property = IsMobile*/
const Value V12825_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12825_5 = {S52, 0, &V12825_5_1277, &P12825_1};	/*Property = ScreenPixelsHeight*/
const Value V12825_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P12825_4 = {S54, 0, &V12825_4_1278, &P12825_5};	/*Property = ScreenPixelsWidth*/
const Profile P12825 = {"12825", &P12825_4};	/*Profile ID - 12825*/
const Value V13062_1_1 = {S2, NULL};	/*Value = True*/
const Property P13062_1 = {S50, 0, &V13062_1_1, NULL};	/*Property = IsMobile*/
const Value V13062_5_1796 = {S64, NULL};	/*Value = 360*/
const Property P13062_5 = {S52, 0, &V13062_5_1796, &P13062_1};	/*Property = ScreenPixelsHeight*/
const Value V13062_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13062_4 = {S54, 0, &V13062_4_1229, &P13062_5};	/*Property = ScreenPixelsWidth*/
const Profile P13062 = {"13062", &P13062_4};	/*Profile ID - 13062*/
const Value V13063_1_1 = {S2, NULL};	/*Value = True*/
const Property P13063_1 = {S50, 0, &V13063_1_1, NULL};	/*Property = IsMobile*/
const Value V13063_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13063_5 = {S52, 0, &V13063_5_1222, &P13063_1};	/*Property = ScreenPixelsHeight*/
const Value V13063_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13063_4 = {S54, 0, &V13063_4_1224, &P13063_5};	/*Property = ScreenPixelsWidth*/
const Profile P13063 = {"13063", &P13063_4};	/*Profile ID - 13063*/
const Value V13064_1_1 = {S2, NULL};	/*Value = True*/
const Property P13064_1 = {S50, 0, &V13064_1_1, NULL};	/*Property = IsMobile*/
const Value V13064_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13064_5 = {S52, 0, &V13064_5_1227, &P13064_1};	/*Property = ScreenPixelsHeight*/
const Value V13064_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13064_4 = {S54, 0, &V13064_4_1229, &P13064_5};	/*Property = ScreenPixelsWidth*/
const Profile P13064 = {"13064", &P13064_4};	/*Profile ID - 13064*/
const Value V17546_1_1 = {S2, NULL};	/*Value = True*/
const Property P17546_1 = {S50, 0, &V17546_1_1, NULL};	/*Property = IsMobile*/
const Value V17546_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17546_5 = {S52, 0, &V17546_5_1222, &P17546_1};	/*Property = ScreenPixelsHeight*/
const Value V17546_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17546_4 = {S54, 0, &V17546_4_1224, &P17546_5};	/*Property = ScreenPixelsWidth*/
const Profile P17546 = {"17546", &P17546_4};	/*Profile ID - 17546*/
const Value V18037_1_1 = {S2, NULL};	/*Value = True*/
const Property P18037_1 = {S50, 0, &V18037_1_1, NULL};	/*Property = IsMobile*/
const Value V18037_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18037_5 = {S52, 0, &V18037_5_1227, &P18037_1};	/*Property = ScreenPixelsHeight*/
const Value V18037_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18037_4 = {S54, 0, &V18037_4_1229, &P18037_5};	/*Property = ScreenPixelsWidth*/
const Profile P18037 = {"18037", &P18037_4};	/*Profile ID - 18037*/
const Value V13069_1_1 = {S2, NULL};	/*Value = True*/
const Property P13069_1 = {S50, 0, &V13069_1_1, NULL};	/*Property = IsMobile*/
const Value V13069_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13069_5 = {S52, 0, &V13069_5_1293, &P13069_1};	/*Property = ScreenPixelsHeight*/
const Value V13069_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13069_4 = {S54, 0, &V13069_4_1296, &P13069_5};	/*Property = ScreenPixelsWidth*/
const Profile P13069 = {"13069", &P13069_4};	/*Profile ID - 13069*/
const Value V13071_1_1 = {S2, NULL};	/*Value = True*/
const Property P13071_1 = {S50, 0, &V13071_1_1, NULL};	/*Property = IsMobile*/
const Value V13071_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13071_5 = {S52, 0, &V13071_5_1380, &P13071_1};	/*Property = ScreenPixelsHeight*/
const Value V13071_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13071_4 = {S54, 0, &V13071_4_1279, &P13071_5};	/*Property = ScreenPixelsWidth*/
const Profile P13071 = {"13071", &P13071_4};	/*Profile ID - 13071*/
const Value V13070_1_1 = {S2, NULL};	/*Value = True*/
const Property P13070_1 = {S50, 0, &V13070_1_1, NULL};	/*Property = IsMobile*/
const Value V13070_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13070_5 = {S52, 0, &V13070_5_1227, &P13070_1};	/*Property = ScreenPixelsHeight*/
const Value V13070_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13070_4 = {S54, 0, &V13070_4_1229, &P13070_5};	/*Property = ScreenPixelsWidth*/
const Profile P13070 = {"13070", &P13070_4};	/*Profile ID - 13070*/
const Value V13072_1_1 = {S2, NULL};	/*Value = True*/
const Property P13072_1 = {S50, 0, &V13072_1_1, NULL};	/*Property = IsMobile*/
const Value V13072_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13072_5 = {S52, 0, &V13072_5_1277, &P13072_1};	/*Property = ScreenPixelsHeight*/
const Value V13072_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P13072_4 = {S54, 0, &V13072_4_2336, &P13072_5};	/*Property = ScreenPixelsWidth*/
const Profile P13072 = {"13072", &P13072_4};	/*Profile ID - 13072*/
const Value V13073_1_1 = {S2, NULL};	/*Value = True*/
const Property P13073_1 = {S50, 0, &V13073_1_1, NULL};	/*Property = IsMobile*/
const Value V13073_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13073_5 = {S52, 0, &V13073_5_1222, &P13073_1};	/*Property = ScreenPixelsHeight*/
const Value V13073_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13073_4 = {S54, 0, &V13073_4_1224, &P13073_5};	/*Property = ScreenPixelsWidth*/
const Profile P13073 = {"13073", &P13073_4};	/*Profile ID - 13073*/
const Value V13076_1_1 = {S2, NULL};	/*Value = True*/
const Property P13076_1 = {S50, 0, &V13076_1_1, NULL};	/*Property = IsMobile*/
const Value V13076_5_31826 = {S75, NULL};	/*Value = 260*/
const Property P13076_5 = {S52, 0, &V13076_5_31826, &P13076_1};	/*Property = ScreenPixelsHeight*/
const Value V13076_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13076_4 = {S54, 0, &V13076_4_1224, &P13076_5};	/*Property = ScreenPixelsWidth*/
const Profile P13076 = {"13076", &P13076_4};	/*Profile ID - 13076*/
const Value V13077_1_1 = {S2, NULL};	/*Value = True*/
const Property P13077_1 = {S50, 0, &V13077_1_1, NULL};	/*Property = IsMobile*/
const Value V13077_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13077_5 = {S52, 0, &V13077_5_1277, &P13077_1};	/*Property = ScreenPixelsHeight*/
const Value V13077_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13077_4 = {S54, 0, &V13077_4_1279, &P13077_5};	/*Property = ScreenPixelsWidth*/
const Profile P13077 = {"13077", &P13077_4};	/*Profile ID - 13077*/
const Value V18174_1_1 = {S2, NULL};	/*Value = True*/
const Property P18174_1 = {S50, 0, &V18174_1_1, NULL};	/*Property = IsMobile*/
const Value V18174_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18174_5 = {S52, 0, &V18174_5_9981, &P18174_1};	/*Property = ScreenPixelsHeight*/
const Value V18174_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18174_4 = {S54, 0, &V18174_4_39098, &P18174_5};	/*Property = ScreenPixelsWidth*/
const Profile P18174 = {"18174", &P18174_4};	/*Profile ID - 18174*/
const Value V12572_1_1 = {S2, NULL};	/*Value = True*/
const Property P12572_1 = {S50, 0, &V12572_1_1, NULL};	/*Property = IsMobile*/
const Value V12572_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12572_5 = {S52, 0, &V12572_5_1277, &P12572_1};	/*Property = ScreenPixelsHeight*/
const Value V12572_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12572_4 = {S54, 0, &V12572_4_1279, &P12572_5};	/*Property = ScreenPixelsWidth*/
const Profile P12572 = {"12572", &P12572_4};	/*Profile ID - 12572*/
const Value V13085_1_1 = {S2, NULL};	/*Value = True*/
const Property P13085_1 = {S50, 0, &V13085_1_1, NULL};	/*Property = IsMobile*/
const Value V13085_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13085_5 = {S52, 0, &V13085_5_1233, &P13085_1};	/*Property = ScreenPixelsHeight*/
const Value V13085_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13085_4 = {S54, 0, &V13085_4_1235, &P13085_5};	/*Property = ScreenPixelsWidth*/
const Profile P13085 = {"13085", &P13085_4};	/*Profile ID - 13085*/
const Value V13086_1_1 = {S2, NULL};	/*Value = True*/
const Property P13086_1 = {S50, 0, &V13086_1_1, NULL};	/*Property = IsMobile*/
const Value V13086_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13086_5 = {S52, 0, &V13086_5_1277, &P13086_1};	/*Property = ScreenPixelsHeight*/
const Value V13086_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13086_4 = {S54, 0, &V13086_4_1279, &P13086_5};	/*Property = ScreenPixelsWidth*/
const Profile P13086 = {"13086", &P13086_4};	/*Profile ID - 13086*/
const Value V13088_1_1 = {S2, NULL};	/*Value = True*/
const Property P13088_1 = {S50, 0, &V13088_1_1, NULL};	/*Property = IsMobile*/
const Value V13088_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13088_5 = {S52, 0, &V13088_5_1380, &P13088_1};	/*Property = ScreenPixelsHeight*/
const Value V13088_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13088_4 = {S54, 0, &V13088_4_1279, &P13088_5};	/*Property = ScreenPixelsWidth*/
const Profile P13088 = {"13088", &P13088_4};	/*Profile ID - 13088*/
const Value V13087_1_1 = {S2, NULL};	/*Value = True*/
const Property P13087_1 = {S50, 0, &V13087_1_1, NULL};	/*Property = IsMobile*/
const Value V13087_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13087_5 = {S52, 0, &V13087_5_1222, &P13087_1};	/*Property = ScreenPixelsHeight*/
const Value V13087_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13087_4 = {S54, 0, &V13087_4_1224, &P13087_5};	/*Property = ScreenPixelsWidth*/
const Profile P13087 = {"13087", &P13087_4};	/*Profile ID - 13087*/
const Value V13090_1_1 = {S2, NULL};	/*Value = True*/
const Property P13090_1 = {S50, 0, &V13090_1_1, NULL};	/*Property = IsMobile*/
const Value V13090_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13090_5 = {S52, 0, &V13090_5_1277, &P13090_1};	/*Property = ScreenPixelsHeight*/
const Value V13090_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P13090_4 = {S54, 0, &V13090_4_2336, &P13090_5};	/*Property = ScreenPixelsWidth*/
const Profile P13090 = {"13090", &P13090_4};	/*Profile ID - 13090*/
const Value V13295_1_1 = {S2, NULL};	/*Value = True*/
const Property P13295_1 = {S50, 0, &V13295_1_1, NULL};	/*Property = IsMobile*/
const Value V13295_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13295_5 = {S52, 0, &V13295_5_1222, &P13295_1};	/*Property = ScreenPixelsHeight*/
const Value V13295_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13295_4 = {S54, 0, &V13295_4_1224, &P13295_5};	/*Property = ScreenPixelsWidth*/
const Profile P13295 = {"13295", &P13295_4};	/*Profile ID - 13295*/
const Value V13091_1_1 = {S2, NULL};	/*Value = True*/
const Property P13091_1 = {S50, 0, &V13091_1_1, NULL};	/*Property = IsMobile*/
const Value V13091_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13091_5 = {S52, 0, &V13091_5_1222, &P13091_1};	/*Property = ScreenPixelsHeight*/
const Value V13091_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13091_4 = {S54, 0, &V13091_4_1224, &P13091_5};	/*Property = ScreenPixelsWidth*/
const Profile P13091 = {"13091", &P13091_4};	/*Profile ID - 13091*/
const Value V13092_1_1 = {S2, NULL};	/*Value = True*/
const Property P13092_1 = {S50, 0, &V13092_1_1, NULL};	/*Property = IsMobile*/
const Value V13092_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13092_5 = {S52, 0, &V13092_5_1380, &P13092_1};	/*Property = ScreenPixelsHeight*/
const Value V13092_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13092_4 = {S54, 0, &V13092_4_1279, &P13092_5};	/*Property = ScreenPixelsWidth*/
const Profile P13092 = {"13092", &P13092_4};	/*Profile ID - 13092*/
const Value V13084_1_1 = {S2, NULL};	/*Value = True*/
const Property P13084_1 = {S50, 0, &V13084_1_1, NULL};	/*Property = IsMobile*/
const Value V13084_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13084_5 = {S52, 0, &V13084_5_1293, &P13084_1};	/*Property = ScreenPixelsHeight*/
const Value V13084_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13084_4 = {S54, 0, &V13084_4_1296, &P13084_5};	/*Property = ScreenPixelsWidth*/
const Profile P13084 = {"13084", &P13084_4};	/*Profile ID - 13084*/
const Value V13094_1_1 = {S2, NULL};	/*Value = True*/
const Property P13094_1 = {S50, 0, &V13094_1_1, NULL};	/*Property = IsMobile*/
const Value V13094_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13094_5 = {S52, 0, &V13094_5_1227, &P13094_1};	/*Property = ScreenPixelsHeight*/
const Value V13094_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13094_4 = {S54, 0, &V13094_4_1229, &P13094_5};	/*Property = ScreenPixelsWidth*/
const Profile P13094 = {"13094", &P13094_4};	/*Profile ID - 13094*/
const Value V20771_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_279 = {S1, 0, &V20771_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20771_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_282 = {S3, 0, &V20771_282_57155, &P20771_279};	/*Property = BlobBuilder*/
const Value V20771_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_350 = {S4, 0, &V20771_350_57446, &P20771_282};	/*Property = Canvas*/
const Value V20771_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_283 = {S5, 0, &V20771_283_57161, &P20771_350};	/*Property = CssBackground*/
const Value V20771_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_284 = {S6, 0, &V20771_284_57164, &P20771_283};	/*Property = CssBorderImage*/
const Value V20771_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_285 = {S7, 0, &V20771_285_57167, &P20771_284};	/*Property = CssCanvas*/
const Value V20771_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_286 = {S8, 0, &V20771_286_57170, &P20771_285};	/*Property = CssColor*/
const Value V20771_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_237 = {S9, 0, &V20771_237_55786, &P20771_286};	/*Property = CssColumn*/
const Value V20771_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_287 = {S10, 0, &V20771_287_57173, &P20771_237};	/*Property = CssFlexbox*/
const Value V20771_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_288 = {S11, 0, &V20771_288_57176, &P20771_287};	/*Property = CssFont*/
const Value V20771_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_289 = {S12, 0, &V20771_289_86899, &P20771_288};	/*Property = CssImages*/
const Value V20771_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_291 = {S13, 0, &V20771_291_57182, &P20771_289};	/*Property = CssMediaQueries*/
const Value V20771_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_292 = {S14, 0, &V20771_292_57185, &P20771_291};	/*Property = CssMinMax*/
const Value V20771_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_293 = {S15, 0, &V20771_293_57188, &P20771_292};	/*Property = CssOverflow*/
const Value V20771_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_294 = {S16, 0, &V20771_294_57191, &P20771_293};	/*Property = CssPosition*/
const Value V20771_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_295 = {S17, 0, &V20771_295_57194, &P20771_294};	/*Property = CssText*/
const Value V20771_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_297 = {S18, 0, &V20771_297_57197, &P20771_295};	/*Property = CssTransforms*/
const Value V20771_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_241 = {S19, 0, &V20771_241_55801, &P20771_297};	/*Property = CssTransitions*/
const Value V20771_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_298 = {S20, 0, &V20771_298_57208, &P20771_241};	/*Property = CssUI*/
const Value V20771_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_300 = {S21, 0, &V20771_300_57217, &P20771_298};	/*Property = DataSet*/
const Value V20771_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_301 = {S22, 0, &V20771_301_57220, &P20771_300};	/*Property = DataUrl*/
const Value V20771_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_302 = {S23, 0, &V20771_302_57223, &P20771_301};	/*Property = DeviceOrientation*/
const Value V20771_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_304 = {S24, 0, &V20771_304_57229, &P20771_302};	/*Property = FileReader*/
const Value V20771_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_305 = {S25, 0, &V20771_305_57232, &P20771_304};	/*Property = FileSaver*/
const Value V20771_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_306 = {S26, 0, &V20771_306_57238, &P20771_305};	/*Property = FileWriter*/
const Value V20771_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_307 = {S27, 0, &V20771_307_57241, &P20771_306};	/*Property = FormData*/
const Value V20771_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_309 = {S28, 0, &V20771_309_57247, &P20771_307};	/*Property = Fullscreen*/
const Value V20771_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_310 = {S29, 0, &V20771_310_57250, &P20771_309};	/*Property = GeoLocation*/
const Value V20771_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_312 = {S30, 0, &V20771_312_57256, &P20771_310};	/*Property = History*/
const Value V20771_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_314 = {S31, 0, &V20771_314_57262, &P20771_312};	/*Property = Html5*/
const Value V20771_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_313 = {S32, 0, &V20771_313_57259, &P20771_314};	/*Property = Html-Media-Capture*/
const Value V20771_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_315 = {S33, 0, &V20771_315_57265, &P20771_313};	/*Property = Iframe*/
const Value V20771_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_316 = {S34, 0, &V20771_316_57268, &P20771_315};	/*Property = IndexedDB*/
const Value V20771_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_317 = {S35, 0, &V20771_317_57271, &P20771_316};	/*Property = Json*/
const Value V20771_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P20771_15 = {S37, 0, &V20771_15_54167, &P20771_317};	/*Property = LayoutEngine*/
const Value V20771_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_344 = {S38, 0, &V20771_344_57274, &P20771_15};	/*Property = Masking*/
const Value V20771_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_320 = {S39, 0, &V20771_320_57292, &P20771_344};	/*Property = PostMessage*/
const Value V20771_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_321 = {S40, 0, &V20771_321_57295, &P20771_320};	/*Property = Progress*/
const Value V20771_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_322 = {S41, 0, &V20771_322_57298, &P20771_321};	/*Property = Prompts*/
const Value V20771_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_323 = {S42, 0, &V20771_323_57301, &P20771_322};	/*Property = Selector*/
const Value V20771_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_324 = {S43, 0, &V20771_324_57307, &P20771_323};	/*Property = Svg*/
const Value V20771_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_327 = {S44, 0, &V20771_327_57316, &P20771_324};	/*Property = TouchEvents*/
const Value V20771_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_328 = {S45, 0, &V20771_328_57319, &P20771_327};	/*Property = Track*/
const Value V20771_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_329 = {S46, 0, &V20771_329_57325, &P20771_328};	/*Property = Video*/
const Value V20771_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_330 = {S47, 0, &V20771_330_57328, &P20771_329};	/*Property = Viewport*/
const Value V20771_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_332 = {S48, 0, &V20771_332_57340, &P20771_330};	/*Property = WebWorkers*/
const Value V20771_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20771_333 = {S49, 0, &V20771_333_57343, &P20771_332};	/*Property = Xhr2*/
const Profile P20771 = {"20771", &P20771_333};	/*Profile ID - 20771*/
const Value V13096_1_1 = {S2, NULL};	/*Value = True*/
const Property P13096_1 = {S50, 0, &V13096_1_1, NULL};	/*Property = IsMobile*/
const Value V13096_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13096_5 = {S52, 0, &V13096_5_1222, &P13096_1};	/*Property = ScreenPixelsHeight*/
const Value V13096_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13096_4 = {S54, 0, &V13096_4_1224, &P13096_5};	/*Property = ScreenPixelsWidth*/
const Profile P13096 = {"13096", &P13096_4};	/*Profile ID - 13096*/
const Value V13097_1_1 = {S2, NULL};	/*Value = True*/
const Property P13097_1 = {S50, 0, &V13097_1_1, NULL};	/*Property = IsMobile*/
const Value V13097_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13097_5 = {S52, 0, &V13097_5_1277, &P13097_1};	/*Property = ScreenPixelsHeight*/
const Value V13097_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13097_4 = {S54, 0, &V13097_4_1279, &P13097_5};	/*Property = ScreenPixelsWidth*/
const Profile P13097 = {"13097", &P13097_4};	/*Profile ID - 13097*/
const Value V13093_1_1 = {S2, NULL};	/*Value = True*/
const Property P13093_1 = {S50, 0, &V13093_1_1, NULL};	/*Property = IsMobile*/
const Value V13093_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13093_5 = {S52, 0, &V13093_5_1222, &P13093_1};	/*Property = ScreenPixelsHeight*/
const Value V13093_4_1368 = {S74, NULL};	/*Value = 208*/
const Property P13093_4 = {S54, 0, &V13093_4_1368, &P13093_5};	/*Property = ScreenPixelsWidth*/
const Profile P13093 = {"13093", &P13093_4};	/*Profile ID - 13093*/
const Value V20418_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_279 = {S1, 0, &V20418_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20418_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_282 = {S3, 0, &V20418_282_57155, &P20418_279};	/*Property = BlobBuilder*/
const Value V20418_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_350 = {S4, 0, &V20418_350_57446, &P20418_282};	/*Property = Canvas*/
const Value V20418_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_283 = {S5, 0, &V20418_283_57161, &P20418_350};	/*Property = CssBackground*/
const Value V20418_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_284 = {S6, 0, &V20418_284_57164, &P20418_283};	/*Property = CssBorderImage*/
const Value V20418_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_285 = {S7, 0, &V20418_285_57167, &P20418_284};	/*Property = CssCanvas*/
const Value V20418_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_286 = {S8, 0, &V20418_286_57170, &P20418_285};	/*Property = CssColor*/
const Value V20418_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_237 = {S9, 0, &V20418_237_55786, &P20418_286};	/*Property = CssColumn*/
const Value V20418_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_287 = {S10, 0, &V20418_287_57173, &P20418_237};	/*Property = CssFlexbox*/
const Value V20418_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_288 = {S11, 0, &V20418_288_57176, &P20418_287};	/*Property = CssFont*/
const Value V20418_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_289 = {S12, 0, &V20418_289_86899, &P20418_288};	/*Property = CssImages*/
const Value V20418_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_291 = {S13, 0, &V20418_291_57182, &P20418_289};	/*Property = CssMediaQueries*/
const Value V20418_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_292 = {S14, 0, &V20418_292_57185, &P20418_291};	/*Property = CssMinMax*/
const Value V20418_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_293 = {S15, 0, &V20418_293_57188, &P20418_292};	/*Property = CssOverflow*/
const Value V20418_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_294 = {S16, 0, &V20418_294_57191, &P20418_293};	/*Property = CssPosition*/
const Value V20418_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_295 = {S17, 0, &V20418_295_57194, &P20418_294};	/*Property = CssText*/
const Value V20418_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_297 = {S18, 0, &V20418_297_57197, &P20418_295};	/*Property = CssTransforms*/
const Value V20418_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_241 = {S19, 0, &V20418_241_55801, &P20418_297};	/*Property = CssTransitions*/
const Value V20418_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_298 = {S20, 0, &V20418_298_57208, &P20418_241};	/*Property = CssUI*/
const Value V20418_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_300 = {S21, 0, &V20418_300_57217, &P20418_298};	/*Property = DataSet*/
const Value V20418_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_301 = {S22, 0, &V20418_301_57220, &P20418_300};	/*Property = DataUrl*/
const Value V20418_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_302 = {S23, 0, &V20418_302_57223, &P20418_301};	/*Property = DeviceOrientation*/
const Value V20418_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_304 = {S24, 0, &V20418_304_57229, &P20418_302};	/*Property = FileReader*/
const Value V20418_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_305 = {S25, 0, &V20418_305_57232, &P20418_304};	/*Property = FileSaver*/
const Value V20418_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_306 = {S26, 0, &V20418_306_57238, &P20418_305};	/*Property = FileWriter*/
const Value V20418_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_307 = {S27, 0, &V20418_307_57241, &P20418_306};	/*Property = FormData*/
const Value V20418_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_309 = {S28, 0, &V20418_309_57247, &P20418_307};	/*Property = Fullscreen*/
const Value V20418_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_310 = {S29, 0, &V20418_310_57250, &P20418_309};	/*Property = GeoLocation*/
const Value V20418_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_312 = {S30, 0, &V20418_312_57256, &P20418_310};	/*Property = History*/
const Value V20418_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_314 = {S31, 0, &V20418_314_57262, &P20418_312};	/*Property = Html5*/
const Value V20418_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_313 = {S32, 0, &V20418_313_57259, &P20418_314};	/*Property = Html-Media-Capture*/
const Value V20418_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_315 = {S33, 0, &V20418_315_57265, &P20418_313};	/*Property = Iframe*/
const Value V20418_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_316 = {S34, 0, &V20418_316_57268, &P20418_315};	/*Property = IndexedDB*/
const Value V20418_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_317 = {S35, 0, &V20418_317_57271, &P20418_316};	/*Property = Json*/
const Value V20418_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P20418_15 = {S37, 0, &V20418_15_31, &P20418_317};	/*Property = LayoutEngine*/
const Value V20418_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_344 = {S38, 0, &V20418_344_57274, &P20418_15};	/*Property = Masking*/
const Value V20418_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_320 = {S39, 0, &V20418_320_57292, &P20418_344};	/*Property = PostMessage*/
const Value V20418_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_321 = {S40, 0, &V20418_321_57295, &P20418_320};	/*Property = Progress*/
const Value V20418_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_322 = {S41, 0, &V20418_322_57298, &P20418_321};	/*Property = Prompts*/
const Value V20418_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_323 = {S42, 0, &V20418_323_57301, &P20418_322};	/*Property = Selector*/
const Value V20418_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_324 = {S43, 0, &V20418_324_57307, &P20418_323};	/*Property = Svg*/
const Value V20418_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_327 = {S44, 0, &V20418_327_57316, &P20418_324};	/*Property = TouchEvents*/
const Value V20418_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_328 = {S45, 0, &V20418_328_57319, &P20418_327};	/*Property = Track*/
const Value V20418_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_329 = {S46, 0, &V20418_329_57325, &P20418_328};	/*Property = Video*/
const Value V20418_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_330 = {S47, 0, &V20418_330_57328, &P20418_329};	/*Property = Viewport*/
const Value V20418_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_332 = {S48, 0, &V20418_332_57340, &P20418_330};	/*Property = WebWorkers*/
const Value V20418_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20418_333 = {S49, 0, &V20418_333_57343, &P20418_332};	/*Property = Xhr2*/
const Profile P20418 = {"20418", &P20418_333};	/*Profile ID - 20418*/
const Value V13101_1_1 = {S2, NULL};	/*Value = True*/
const Property P13101_1 = {S50, 0, &V13101_1_1, NULL};	/*Property = IsMobile*/
const Value V13101_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13101_5 = {S52, 0, &V13101_5_1222, &P13101_1};	/*Property = ScreenPixelsHeight*/
const Value V13101_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13101_4 = {S54, 0, &V13101_4_1224, &P13101_5};	/*Property = ScreenPixelsWidth*/
const Profile P13101 = {"13101", &P13101_4};	/*Profile ID - 13101*/
const Value V13102_1_1 = {S2, NULL};	/*Value = True*/
const Property P13102_1 = {S50, 0, &V13102_1_1, NULL};	/*Property = IsMobile*/
const Value V13102_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13102_5 = {S52, 0, &V13102_5_1233, &P13102_1};	/*Property = ScreenPixelsHeight*/
const Value V13102_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13102_4 = {S54, 0, &V13102_4_1235, &P13102_5};	/*Property = ScreenPixelsWidth*/
const Profile P13102 = {"13102", &P13102_4};	/*Profile ID - 13102*/
const Value V23905_1_1 = {S2, NULL};	/*Value = True*/
const Property P23905_1 = {S50, 0, &V23905_1_1, NULL};	/*Property = IsMobile*/
const Value V23905_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23905_5 = {S52, 0, &V23905_5_1227, &P23905_1};	/*Property = ScreenPixelsHeight*/
const Value V23905_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23905_4 = {S54, 0, &V23905_4_1229, &P23905_5};	/*Property = ScreenPixelsWidth*/
const Profile P23905 = {"23905", &P23905_4};	/*Profile ID - 23905*/
const Value V13104_1_1 = {S2, NULL};	/*Value = True*/
const Property P13104_1 = {S50, 0, &V13104_1_1, NULL};	/*Property = IsMobile*/
const Value V13104_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P13104_5 = {S52, 0, &V13104_5_1333, &P13104_1};	/*Property = ScreenPixelsHeight*/
const Value V13104_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P13104_4 = {S54, 0, &V13104_4_1335, &P13104_5};	/*Property = ScreenPixelsWidth*/
const Profile P13104 = {"13104", &P13104_4};	/*Profile ID - 13104*/
const Value V13105_1_1 = {S2, NULL};	/*Value = True*/
const Property P13105_1 = {S50, 0, &V13105_1_1, NULL};	/*Property = IsMobile*/
const Value V13105_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P13105_5 = {S52, 0, &V13105_5_9981, &P13105_1};	/*Property = ScreenPixelsHeight*/
const Value V13105_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P13105_4 = {S54, 0, &V13105_4_2336, &P13105_5};	/*Property = ScreenPixelsWidth*/
const Profile P13105 = {"13105", &P13105_4};	/*Profile ID - 13105*/
const Value V17831_1_1 = {S2, NULL};	/*Value = True*/
const Property P17831_1 = {S50, 0, &V17831_1_1, NULL};	/*Property = IsMobile*/
const Value V17831_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17831_5 = {S52, 0, &V17831_5_1222, &P17831_1};	/*Property = ScreenPixelsHeight*/
const Value V17831_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17831_4 = {S54, 0, &V17831_4_1224, &P17831_5};	/*Property = ScreenPixelsWidth*/
const Profile P17831 = {"17831", &P17831_4};	/*Profile ID - 17831*/
const Value V13107_1_1 = {S2, NULL};	/*Value = True*/
const Property P13107_1 = {S50, 0, &V13107_1_1, NULL};	/*Property = IsMobile*/
const Value V13107_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13107_5 = {S52, 0, &V13107_5_1222, &P13107_1};	/*Property = ScreenPixelsHeight*/
const Value V13107_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13107_4 = {S54, 0, &V13107_4_1224, &P13107_5};	/*Property = ScreenPixelsWidth*/
const Profile P13107 = {"13107", &P13107_4};	/*Profile ID - 13107*/
const Value V13112_1_1 = {S2, NULL};	/*Value = True*/
const Property P13112_1 = {S50, 0, &V13112_1_1, NULL};	/*Property = IsMobile*/
const Value V13112_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13112_5 = {S52, 0, &V13112_5_1222, &P13112_1};	/*Property = ScreenPixelsHeight*/
const Value V13112_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13112_4 = {S54, 0, &V13112_4_1224, &P13112_5};	/*Property = ScreenPixelsWidth*/
const Profile P13112 = {"13112", &P13112_4};	/*Profile ID - 13112*/
const Value V13109_1_1 = {S2, NULL};	/*Value = True*/
const Property P13109_1 = {S50, 0, &V13109_1_1, NULL};	/*Property = IsMobile*/
const Value V13109_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P13109_5 = {S52, 0, &V13109_5_1403, &P13109_1};	/*Property = ScreenPixelsHeight*/
const Value V13109_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P13109_4 = {S54, 0, &V13109_4_1404, &P13109_5};	/*Property = ScreenPixelsWidth*/
const Profile P13109 = {"13109", &P13109_4};	/*Profile ID - 13109*/
const Value V13113_1_1 = {S2, NULL};	/*Value = True*/
const Property P13113_1 = {S50, 0, &V13113_1_1, NULL};	/*Property = IsMobile*/
const Value V13113_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13113_5 = {S52, 0, &V13113_5_1293, &P13113_1};	/*Property = ScreenPixelsHeight*/
const Value V13113_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13113_4 = {S54, 0, &V13113_4_1296, &P13113_5};	/*Property = ScreenPixelsWidth*/
const Profile P13113 = {"13113", &P13113_4};	/*Profile ID - 13113*/
const Value V13115_1_1 = {S2, NULL};	/*Value = True*/
const Property P13115_1 = {S50, 0, &V13115_1_1, NULL};	/*Property = IsMobile*/
const Value V13115_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13115_5 = {S52, 0, &V13115_5_1222, &P13115_1};	/*Property = ScreenPixelsHeight*/
const Value V13115_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13115_4 = {S54, 0, &V13115_4_1224, &P13115_5};	/*Property = ScreenPixelsWidth*/
const Profile P13115 = {"13115", &P13115_4};	/*Profile ID - 13115*/
const Value V13117_1_1 = {S2, NULL};	/*Value = True*/
const Property P13117_1 = {S50, 0, &V13117_1_1, NULL};	/*Property = IsMobile*/
const Value V13117_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13117_5 = {S52, 0, &V13117_5_1233, &P13117_1};	/*Property = ScreenPixelsHeight*/
const Value V13117_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13117_4 = {S54, 0, &V13117_4_1235, &P13117_5};	/*Property = ScreenPixelsWidth*/
const Profile P13117 = {"13117", &P13117_4};	/*Profile ID - 13117*/
const Value V13122_1_1 = {S2, NULL};	/*Value = True*/
const Property P13122_1 = {S50, 0, &V13122_1_1, NULL};	/*Property = IsMobile*/
const Value V13122_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13122_5 = {S52, 0, &V13122_5_1380, &P13122_1};	/*Property = ScreenPixelsHeight*/
const Value V13122_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13122_4 = {S54, 0, &V13122_4_1279, &P13122_5};	/*Property = ScreenPixelsWidth*/
const Profile P13122 = {"13122", &P13122_4};	/*Profile ID - 13122*/
const Value V13123_1_1 = {S2, NULL};	/*Value = True*/
const Property P13123_1 = {S50, 0, &V13123_1_1, NULL};	/*Property = IsMobile*/
const Value V13123_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13123_5 = {S52, 0, &V13123_5_1222, &P13123_1};	/*Property = ScreenPixelsHeight*/
const Value V13123_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13123_4 = {S54, 0, &V13123_4_1224, &P13123_5};	/*Property = ScreenPixelsWidth*/
const Profile P13123 = {"13123", &P13123_4};	/*Profile ID - 13123*/
const Value V13126_1_1 = {S2, NULL};	/*Value = True*/
const Property P13126_1 = {S50, 0, &V13126_1_1, NULL};	/*Property = IsMobile*/
const Value V13126_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P13126_5 = {S52, 0, &V13126_5_37, &P13126_1};	/*Property = ScreenPixelsHeight*/
const Value V13126_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P13126_4 = {S54, 0, &V13126_4_36, &P13126_5};	/*Property = ScreenPixelsWidth*/
const Profile P13126 = {"13126", &P13126_4};	/*Profile ID - 13126*/
const Value V13018_1_1 = {S2, NULL};	/*Value = True*/
const Property P13018_1 = {S50, 0, &V13018_1_1, NULL};	/*Property = IsMobile*/
const Value V13018_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P13018_5 = {S52, 0, &V13018_5_1269, &P13018_1};	/*Property = ScreenPixelsHeight*/
const Value V13018_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13018_4 = {S54, 0, &V13018_4_1229, &P13018_5};	/*Property = ScreenPixelsWidth*/
const Profile P13018 = {"13018", &P13018_4};	/*Profile ID - 13018*/
const Value V13124_1_1 = {S2, NULL};	/*Value = True*/
const Property P13124_1 = {S50, 0, &V13124_1_1, NULL};	/*Property = IsMobile*/
const Value V13124_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13124_5 = {S52, 0, &V13124_5_1227, &P13124_1};	/*Property = ScreenPixelsHeight*/
const Value V13124_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13124_4 = {S54, 0, &V13124_4_1229, &P13124_5};	/*Property = ScreenPixelsWidth*/
const Profile P13124 = {"13124", &P13124_4};	/*Profile ID - 13124*/
const Value V13129_1_1 = {S2, NULL};	/*Value = True*/
const Property P13129_1 = {S50, 0, &V13129_1_1, NULL};	/*Property = IsMobile*/
const Value V13129_5_37567 = {S95, NULL};	/*Value = 300*/
const Property P13129_5 = {S52, 0, &V13129_5_37567, &P13129_1};	/*Property = ScreenPixelsHeight*/
const Value V13129_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13129_4 = {S54, 0, &V13129_4_1224, &P13129_5};	/*Property = ScreenPixelsWidth*/
const Profile P13129 = {"13129", &P13129_4};	/*Profile ID - 13129*/
const Value V17711_1_1 = {S2, NULL};	/*Value = True*/
const Property P17711_1 = {S50, 0, &V17711_1_1, NULL};	/*Property = IsMobile*/
const Value V17711_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17711_5 = {S52, 0, &V17711_5_1277, &P17711_1};	/*Property = ScreenPixelsHeight*/
const Value V17711_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17711_4 = {S54, 0, &V17711_4_1279, &P17711_5};	/*Property = ScreenPixelsWidth*/
const Profile P17711 = {"17711", &P17711_4};	/*Profile ID - 17711*/
const Value V18003_1_1 = {S2, NULL};	/*Value = True*/
const Property P18003_1 = {S50, 0, &V18003_1_1, NULL};	/*Property = IsMobile*/
const Value V18003_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18003_5 = {S52, 0, &V18003_5_1227, &P18003_1};	/*Property = ScreenPixelsHeight*/
const Value V18003_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18003_4 = {S54, 0, &V18003_4_1229, &P18003_5};	/*Property = ScreenPixelsWidth*/
const Profile P18003 = {"18003", &P18003_4};	/*Profile ID - 18003*/
const Value V13133_1_1 = {S2, NULL};	/*Value = True*/
const Property P13133_1 = {S50, 0, &V13133_1_1, NULL};	/*Property = IsMobile*/
const Value V13133_5_24030 = {S71, NULL};	/*Value = 120*/
const Property P13133_5 = {S52, 0, &V13133_5_24030, &P13133_1};	/*Property = ScreenPixelsHeight*/
const Value V13133_4_24031 = {S71, NULL};	/*Value = 120*/
const Property P13133_4 = {S54, 0, &V13133_4_24031, &P13133_5};	/*Property = ScreenPixelsWidth*/
const Profile P13133 = {"13133", &P13133_4};	/*Profile ID - 13133*/
const Value V13137_1_1 = {S2, NULL};	/*Value = True*/
const Property P13137_1 = {S50, 0, &V13137_1_1, NULL};	/*Property = IsMobile*/
const Value V13137_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13137_5 = {S52, 0, &V13137_5_1233, &P13137_1};	/*Property = ScreenPixelsHeight*/
const Value V13137_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13137_4 = {S54, 0, &V13137_4_1235, &P13137_5};	/*Property = ScreenPixelsWidth*/
const Profile P13137 = {"13137", &P13137_4};	/*Profile ID - 13137*/
const Value V13134_1_1 = {S2, NULL};	/*Value = True*/
const Property P13134_1 = {S50, 0, &V13134_1_1, NULL};	/*Property = IsMobile*/
const Value V13134_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13134_5 = {S52, 0, &V13134_5_1380, &P13134_1};	/*Property = ScreenPixelsHeight*/
const Value V13134_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13134_4 = {S54, 0, &V13134_4_1279, &P13134_5};	/*Property = ScreenPixelsWidth*/
const Profile P13134 = {"13134", &P13134_4};	/*Profile ID - 13134*/
const Value V13140_1_1 = {S2, NULL};	/*Value = True*/
const Property P13140_1 = {S50, 0, &V13140_1_1, NULL};	/*Property = IsMobile*/
const Value V13140_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13140_5 = {S52, 0, &V13140_5_1233, &P13140_1};	/*Property = ScreenPixelsHeight*/
const Value V13140_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13140_4 = {S54, 0, &V13140_4_1235, &P13140_5};	/*Property = ScreenPixelsWidth*/
const Profile P13140 = {"13140", &P13140_4};	/*Profile ID - 13140*/
const Value V21561_1_1 = {S2, NULL};	/*Value = True*/
const Property P21561_1 = {S50, 0, &V21561_1_1, NULL};	/*Property = IsMobile*/
const Value V21561_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21561_5 = {S52, 0, &V21561_5_1222, &P21561_1};	/*Property = ScreenPixelsHeight*/
const Value V21561_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21561_4 = {S54, 0, &V21561_4_1224, &P21561_5};	/*Property = ScreenPixelsWidth*/
const Profile P21561 = {"21561", &P21561_4};	/*Profile ID - 21561*/
const Value V13148_1_1 = {S2, NULL};	/*Value = True*/
const Property P13148_1 = {S50, 0, &V13148_1_1, NULL};	/*Property = IsMobile*/
const Value V13148_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13148_5 = {S52, 0, &V13148_5_1277, &P13148_1};	/*Property = ScreenPixelsHeight*/
const Value V13148_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13148_4 = {S54, 0, &V13148_4_1279, &P13148_5};	/*Property = ScreenPixelsWidth*/
const Profile P13148 = {"13148", &P13148_4};	/*Profile ID - 13148*/
const Value V13149_1_1 = {S2, NULL};	/*Value = True*/
const Property P13149_1 = {S50, 0, &V13149_1_1, NULL};	/*Property = IsMobile*/
const Value V13149_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13149_5 = {S52, 0, &V13149_5_1294, &P13149_1};	/*Property = ScreenPixelsHeight*/
const Value V13149_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13149_4 = {S54, 0, &V13149_4_1296, &P13149_5};	/*Property = ScreenPixelsWidth*/
const Profile P13149 = {"13149", &P13149_4};	/*Profile ID - 13149*/
const Value V13150_1_1 = {S2, NULL};	/*Value = True*/
const Property P13150_1 = {S50, 0, &V13150_1_1, NULL};	/*Property = IsMobile*/
const Value V13150_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P13150_5 = {S52, 0, &V13150_5_37, &P13150_1};	/*Property = ScreenPixelsHeight*/
const Value V13150_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P13150_4 = {S54, 0, &V13150_4_36, &P13150_5};	/*Property = ScreenPixelsWidth*/
const Profile P13150 = {"13150", &P13150_4};	/*Profile ID - 13150*/
const Value V13153_1_1 = {S2, NULL};	/*Value = True*/
const Property P13153_1 = {S50, 0, &V13153_1_1, NULL};	/*Property = IsMobile*/
const Value V13153_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P13153_5 = {S52, 0, &V13153_5_1333, &P13153_1};	/*Property = ScreenPixelsHeight*/
const Value V13153_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P13153_4 = {S54, 0, &V13153_4_1335, &P13153_5};	/*Property = ScreenPixelsWidth*/
const Profile P13153 = {"13153", &P13153_4};	/*Profile ID - 13153*/
const Value V13151_1_1 = {S2, NULL};	/*Value = True*/
const Property P13151_1 = {S50, 0, &V13151_1_1, NULL};	/*Property = IsMobile*/
const Value V13151_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13151_5 = {S52, 0, &V13151_5_1233, &P13151_1};	/*Property = ScreenPixelsHeight*/
const Value V13151_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13151_4 = {S54, 0, &V13151_4_1235, &P13151_5};	/*Property = ScreenPixelsWidth*/
const Profile P13151 = {"13151", &P13151_4};	/*Profile ID - 13151*/
const Value V13152_1_1 = {S2, NULL};	/*Value = True*/
const Property P13152_1 = {S50, 0, &V13152_1_1, NULL};	/*Property = IsMobile*/
const Value V13152_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13152_5 = {S52, 0, &V13152_5_1294, &P13152_1};	/*Property = ScreenPixelsHeight*/
const Value V13152_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13152_4 = {S54, 0, &V13152_4_1296, &P13152_5};	/*Property = ScreenPixelsWidth*/
const Profile P13152 = {"13152", &P13152_4};	/*Profile ID - 13152*/
const Value V13154_1_1 = {S2, NULL};	/*Value = True*/
const Property P13154_1 = {S50, 0, &V13154_1_1, NULL};	/*Property = IsMobile*/
const Value V13154_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13154_5 = {S52, 0, &V13154_5_1222, &P13154_1};	/*Property = ScreenPixelsHeight*/
const Value V13154_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13154_4 = {S54, 0, &V13154_4_1224, &P13154_5};	/*Property = ScreenPixelsWidth*/
const Profile P13154 = {"13154", &P13154_4};	/*Profile ID - 13154*/
const Value V13157_1_1 = {S2, NULL};	/*Value = True*/
const Property P13157_1 = {S50, 0, &V13157_1_1, NULL};	/*Property = IsMobile*/
const Value V13157_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P13157_5 = {S52, 0, &V13157_5_1403, &P13157_1};	/*Property = ScreenPixelsHeight*/
const Value V13157_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P13157_4 = {S54, 0, &V13157_4_1404, &P13157_5};	/*Property = ScreenPixelsWidth*/
const Profile P13157 = {"13157", &P13157_4};	/*Profile ID - 13157*/
const Value V17234_1_1 = {S2, NULL};	/*Value = True*/
const Property P17234_1 = {S50, 0, &V17234_1_1, NULL};	/*Property = IsMobile*/
const Value V17234_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17234_5 = {S52, 0, &V17234_5_1277, &P17234_1};	/*Property = ScreenPixelsHeight*/
const Value V17234_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17234_4 = {S54, 0, &V17234_4_1279, &P17234_5};	/*Property = ScreenPixelsWidth*/
const Profile P17234 = {"17234", &P17234_4};	/*Profile ID - 17234*/
const Value V4551_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_279 = {S1, 0, &V4551_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4551_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_282 = {S3, 0, &V4551_282_57155, &P4551_279};	/*Property = BlobBuilder*/
const Value V4551_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_350 = {S4, 0, &V4551_350_57446, &P4551_282};	/*Property = Canvas*/
const Value V4551_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_283 = {S5, 0, &V4551_283_57161, &P4551_350};	/*Property = CssBackground*/
const Value V4551_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_284 = {S6, 0, &V4551_284_57164, &P4551_283};	/*Property = CssBorderImage*/
const Value V4551_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_285 = {S7, 0, &V4551_285_57167, &P4551_284};	/*Property = CssCanvas*/
const Value V4551_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_286 = {S8, 0, &V4551_286_57170, &P4551_285};	/*Property = CssColor*/
const Value V4551_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_237 = {S9, 0, &V4551_237_55786, &P4551_286};	/*Property = CssColumn*/
const Value V4551_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_287 = {S10, 0, &V4551_287_57173, &P4551_237};	/*Property = CssFlexbox*/
const Value V4551_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_288 = {S11, 0, &V4551_288_57176, &P4551_287};	/*Property = CssFont*/
const Value V4551_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_289 = {S12, 0, &V4551_289_86899, &P4551_288};	/*Property = CssImages*/
const Value V4551_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_291 = {S13, 0, &V4551_291_57182, &P4551_289};	/*Property = CssMediaQueries*/
const Value V4551_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_292 = {S14, 0, &V4551_292_57185, &P4551_291};	/*Property = CssMinMax*/
const Value V4551_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_293 = {S15, 0, &V4551_293_57188, &P4551_292};	/*Property = CssOverflow*/
const Value V4551_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_294 = {S16, 0, &V4551_294_57191, &P4551_293};	/*Property = CssPosition*/
const Value V4551_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_295 = {S17, 0, &V4551_295_57194, &P4551_294};	/*Property = CssText*/
const Value V4551_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_297 = {S18, 0, &V4551_297_57197, &P4551_295};	/*Property = CssTransforms*/
const Value V4551_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_241 = {S19, 0, &V4551_241_55801, &P4551_297};	/*Property = CssTransitions*/
const Value V4551_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_298 = {S20, 0, &V4551_298_57208, &P4551_241};	/*Property = CssUI*/
const Value V4551_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_300 = {S21, 0, &V4551_300_57217, &P4551_298};	/*Property = DataSet*/
const Value V4551_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_301 = {S22, 0, &V4551_301_57220, &P4551_300};	/*Property = DataUrl*/
const Value V4551_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_302 = {S23, 0, &V4551_302_57223, &P4551_301};	/*Property = DeviceOrientation*/
const Value V4551_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_304 = {S24, 0, &V4551_304_57229, &P4551_302};	/*Property = FileReader*/
const Value V4551_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_305 = {S25, 0, &V4551_305_57232, &P4551_304};	/*Property = FileSaver*/
const Value V4551_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_306 = {S26, 0, &V4551_306_57238, &P4551_305};	/*Property = FileWriter*/
const Value V4551_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_307 = {S27, 0, &V4551_307_57241, &P4551_306};	/*Property = FormData*/
const Value V4551_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_309 = {S28, 0, &V4551_309_57247, &P4551_307};	/*Property = Fullscreen*/
const Value V4551_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_310 = {S29, 0, &V4551_310_57250, &P4551_309};	/*Property = GeoLocation*/
const Value V4551_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_312 = {S30, 0, &V4551_312_57256, &P4551_310};	/*Property = History*/
const Value V4551_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_314 = {S31, 0, &V4551_314_57262, &P4551_312};	/*Property = Html5*/
const Value V4551_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_313 = {S32, 0, &V4551_313_57259, &P4551_314};	/*Property = Html-Media-Capture*/
const Value V4551_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_315 = {S33, 0, &V4551_315_57265, &P4551_313};	/*Property = Iframe*/
const Value V4551_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_316 = {S34, 0, &V4551_316_57268, &P4551_315};	/*Property = IndexedDB*/
const Value V4551_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_317 = {S35, 0, &V4551_317_57271, &P4551_316};	/*Property = Json*/
const Value V4551_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P4551_15 = {S37, 0, &V4551_15_30, &P4551_317};	/*Property = LayoutEngine*/
const Value V4551_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_344 = {S38, 0, &V4551_344_57274, &P4551_15};	/*Property = Masking*/
const Value V4551_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_320 = {S39, 0, &V4551_320_57292, &P4551_344};	/*Property = PostMessage*/
const Value V4551_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_321 = {S40, 0, &V4551_321_57295, &P4551_320};	/*Property = Progress*/
const Value V4551_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_322 = {S41, 0, &V4551_322_57298, &P4551_321};	/*Property = Prompts*/
const Value V4551_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_323 = {S42, 0, &V4551_323_57301, &P4551_322};	/*Property = Selector*/
const Value V4551_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_324 = {S43, 0, &V4551_324_57307, &P4551_323};	/*Property = Svg*/
const Value V4551_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_327 = {S44, 0, &V4551_327_57316, &P4551_324};	/*Property = TouchEvents*/
const Value V4551_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_328 = {S45, 0, &V4551_328_57319, &P4551_327};	/*Property = Track*/
const Value V4551_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_329 = {S46, 0, &V4551_329_57325, &P4551_328};	/*Property = Video*/
const Value V4551_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_330 = {S47, 0, &V4551_330_57328, &P4551_329};	/*Property = Viewport*/
const Value V4551_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_332 = {S48, 0, &V4551_332_57340, &P4551_330};	/*Property = WebWorkers*/
const Value V4551_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4551_333 = {S49, 0, &V4551_333_57343, &P4551_332};	/*Property = Xhr2*/
const Profile P4551 = {"4551", &P4551_333};	/*Profile ID - 4551*/
const Profile P17188 = {"17188", NULL};	/*Profile ID - 17188*/
const Value V13189_1_1 = {S2, NULL};	/*Value = True*/
const Property P13189_1 = {S50, 0, &V13189_1_1, NULL};	/*Property = IsMobile*/
const Value V13189_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13189_5 = {S52, 0, &V13189_5_1277, &P13189_1};	/*Property = ScreenPixelsHeight*/
const Value V13189_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13189_4 = {S54, 0, &V13189_4_1279, &P13189_5};	/*Property = ScreenPixelsWidth*/
const Profile P13189 = {"13189", &P13189_4};	/*Profile ID - 13189*/
const Value V13233_1_1 = {S2, NULL};	/*Value = True*/
const Property P13233_1 = {S50, 0, &V13233_1_1, NULL};	/*Property = IsMobile*/
const Value V13233_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13233_5 = {S52, 0, &V13233_5_1277, &P13233_1};	/*Property = ScreenPixelsHeight*/
const Value V13233_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13233_4 = {S54, 0, &V13233_4_1279, &P13233_5};	/*Property = ScreenPixelsWidth*/
const Profile P13233 = {"13233", &P13233_4};	/*Profile ID - 13233*/
const Value V13161_1_1 = {S2, NULL};	/*Value = True*/
const Property P13161_1 = {S50, 0, &V13161_1_1, NULL};	/*Property = IsMobile*/
const Value V13161_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13161_5 = {S52, 0, &V13161_5_1222, &P13161_1};	/*Property = ScreenPixelsHeight*/
const Value V13161_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13161_4 = {S54, 0, &V13161_4_1224, &P13161_5};	/*Property = ScreenPixelsWidth*/
const Profile P13161 = {"13161", &P13161_4};	/*Profile ID - 13161*/
const Value V17192_1_1 = {S2, NULL};	/*Value = True*/
const Property P17192_1 = {S50, 0, &V17192_1_1, NULL};	/*Property = IsMobile*/
const Value V17192_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P17192_5 = {S52, 0, &V17192_5_1458, &P17192_1};	/*Property = ScreenPixelsHeight*/
const Value V17192_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P17192_4 = {S54, 0, &V17192_4_1460, &P17192_5};	/*Property = ScreenPixelsWidth*/
const Profile P17192 = {"17192", &P17192_4};	/*Profile ID - 17192*/
const Value V13166_1_1 = {S2, NULL};	/*Value = True*/
const Property P13166_1 = {S50, 0, &V13166_1_1, NULL};	/*Property = IsMobile*/
const Value V13166_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13166_5 = {S52, 0, &V13166_5_1222, &P13166_1};	/*Property = ScreenPixelsHeight*/
const Value V13166_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13166_4 = {S54, 0, &V13166_4_1224, &P13166_5};	/*Property = ScreenPixelsWidth*/
const Profile P13166 = {"13166", &P13166_4};	/*Profile ID - 13166*/
const Value V13167_1_1 = {S2, NULL};	/*Value = True*/
const Property P13167_1 = {S50, 0, &V13167_1_1, NULL};	/*Property = IsMobile*/
const Value V13167_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13167_5 = {S52, 0, &V13167_5_1293, &P13167_1};	/*Property = ScreenPixelsHeight*/
const Value V13167_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13167_4 = {S54, 0, &V13167_4_1296, &P13167_5};	/*Property = ScreenPixelsWidth*/
const Profile P13167 = {"13167", &P13167_4};	/*Profile ID - 13167*/
const Value V21556_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21556_279 = {S1, 0, &V21556_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21556_282_57153 = {S2, NULL};	/*Value = True*/
const Property P21556_282 = {S3, 0, &V21556_282_57153, &P21556_279};	/*Property = BlobBuilder*/
const Value V21556_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21556_350 = {S4, 0, &V21556_350_57444, &P21556_282};	/*Property = Canvas*/
const Value V21556_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21556_283 = {S5, 0, &V21556_283_57159, &P21556_350};	/*Property = CssBackground*/
const Value V21556_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21556_284 = {S6, 0, &V21556_284_57163, &P21556_283};	/*Property = CssBorderImage*/
const Value V21556_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21556_285 = {S7, 0, &V21556_285_57166, &P21556_284};	/*Property = CssCanvas*/
const Value V21556_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21556_286 = {S8, 0, &V21556_286_57168, &P21556_285};	/*Property = CssColor*/
const Value V21556_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21556_237 = {S9, 0, &V21556_237_55786, &P21556_286};	/*Property = CssColumn*/
const Value V21556_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21556_287 = {S10, 0, &V21556_287_57172, &P21556_237};	/*Property = CssFlexbox*/
const Value V21556_288_57174 = {S2, NULL};	/*Value = True*/
const Property P21556_288 = {S11, 0, &V21556_288_57174, &P21556_287};	/*Property = CssFont*/
const Value V21556_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21556_289 = {S12, 0, &V21556_289_57759, &P21556_288};	/*Property = CssImages*/
const Value V21556_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21556_291 = {S13, 0, &V21556_291_57180, &P21556_289};	/*Property = CssMediaQueries*/
const Value V21556_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21556_292 = {S14, 0, &V21556_292_57183, &P21556_291};	/*Property = CssMinMax*/
const Value V21556_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21556_293 = {S15, 0, &V21556_293_57187, &P21556_292};	/*Property = CssOverflow*/
const Value V21556_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21556_294 = {S16, 0, &V21556_294_57189, &P21556_293};	/*Property = CssPosition*/
const Value V21556_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21556_295 = {S17, 0, &V21556_295_57192, &P21556_294};	/*Property = CssText*/
const Value V21556_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21556_297 = {S18, 0, &V21556_297_57195, &P21556_295};	/*Property = CssTransforms*/
const Value V21556_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21556_241 = {S19, 0, &V21556_241_55799, &P21556_297};	/*Property = CssTransitions*/
const Value V21556_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21556_298 = {S20, 0, &V21556_298_57206, &P21556_241};	/*Property = CssUI*/
const Value V21556_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21556_300 = {S21, 0, &V21556_300_57216, &P21556_298};	/*Property = DataSet*/
const Value V21556_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21556_301 = {S22, 0, &V21556_301_57218, &P21556_300};	/*Property = DataUrl*/
const Value V21556_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21556_302 = {S23, 0, &V21556_302_57222, &P21556_301};	/*Property = DeviceOrientation*/
const Value V21556_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21556_304 = {S24, 0, &V21556_304_57227, &P21556_302};	/*Property = FileReader*/
const Value V21556_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21556_305 = {S25, 0, &V21556_305_57231, &P21556_304};	/*Property = FileSaver*/
const Value V21556_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21556_306 = {S26, 0, &V21556_306_57237, &P21556_305};	/*Property = FileWriter*/
const Value V21556_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21556_307 = {S27, 0, &V21556_307_57239, &P21556_306};	/*Property = FormData*/
const Value V21556_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21556_309 = {S28, 0, &V21556_309_57246, &P21556_307};	/*Property = Fullscreen*/
const Value V21556_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21556_310 = {S29, 0, &V21556_310_57248, &P21556_309};	/*Property = GeoLocation*/
const Value V21556_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21556_312 = {S30, 0, &V21556_312_57254, &P21556_310};	/*Property = History*/
const Value V21556_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21556_314 = {S31, 0, &V21556_314_57261, &P21556_312};	/*Property = Html5*/
const Value V21556_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21556_313 = {S32, 0, &V21556_313_57258, &P21556_314};	/*Property = Html-Media-Capture*/
const Value V21556_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21556_315 = {S33, 0, &V21556_315_57264, &P21556_313};	/*Property = Iframe*/
const Value V21556_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21556_316 = {S34, 0, &V21556_316_57266, &P21556_315};	/*Property = IndexedDB*/
const Value V21556_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21556_317 = {S35, 0, &V21556_317_57269, &P21556_316};	/*Property = Json*/
const Value V21556_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21556_15 = {S37, 0, &V21556_15_28, &P21556_317};	/*Property = LayoutEngine*/
const Value V21556_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21556_344 = {S38, 0, &V21556_344_57272, &P21556_15};	/*Property = Masking*/
const Value V21556_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21556_320 = {S39, 0, &V21556_320_57290, &P21556_344};	/*Property = PostMessage*/
const Value V21556_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21556_321 = {S40, 0, &V21556_321_57293, &P21556_320};	/*Property = Progress*/
const Value V21556_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21556_322 = {S41, 0, &V21556_322_57296, &P21556_321};	/*Property = Prompts*/
const Value V21556_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21556_323 = {S42, 0, &V21556_323_57299, &P21556_322};	/*Property = Selector*/
const Value V21556_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21556_324 = {S43, 0, &V21556_324_57306, &P21556_323};	/*Property = Svg*/
const Value V21556_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21556_327 = {S44, 0, &V21556_327_57315, &P21556_324};	/*Property = TouchEvents*/
const Value V21556_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21556_328 = {S45, 0, &V21556_328_57318, &P21556_327};	/*Property = Track*/
const Value V21556_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21556_329 = {S46, 0, &V21556_329_57323, &P21556_328};	/*Property = Video*/
const Value V21556_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21556_330 = {S47, 0, &V21556_330_57326, &P21556_329};	/*Property = Viewport*/
const Value V21556_332_57338 = {S2, NULL};	/*Value = True*/
const Property P21556_332 = {S48, 0, &V21556_332_57338, &P21556_330};	/*Property = WebWorkers*/
const Value V21556_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21556_333 = {S49, 0, &V21556_333_57342, &P21556_332};	/*Property = Xhr2*/
const Profile P21556 = {"21556", &P21556_333};	/*Profile ID - 21556*/
const Value V24003_1_1 = {S2, NULL};	/*Value = True*/
const Property P24003_1 = {S50, 0, &V24003_1_1, NULL};	/*Property = IsMobile*/
const Value V24003_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24003_5 = {S52, 0, &V24003_5_1277, &P24003_1};	/*Property = ScreenPixelsHeight*/
const Value V24003_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24003_4 = {S54, 0, &V24003_4_1279, &P24003_5};	/*Property = ScreenPixelsWidth*/
const Profile P24003 = {"24003", &P24003_4};	/*Profile ID - 24003*/
const Value V13188_1_1 = {S2, NULL};	/*Value = True*/
const Property P13188_1 = {S50, 0, &V13188_1_1, NULL};	/*Property = IsMobile*/
const Value V13188_5_37453 = {S91, NULL};	/*Value = 65*/
const Property P13188_5 = {S52, 0, &V13188_5_37453, &P13188_1};	/*Property = ScreenPixelsHeight*/
const Value V13188_4_31956 = {S92, NULL};	/*Value = 96*/
const Property P13188_4 = {S54, 0, &V13188_4_31956, &P13188_5};	/*Property = ScreenPixelsWidth*/
const Profile P13188 = {"13188", &P13188_4};	/*Profile ID - 13188*/
const Value V16332_1_1 = {S2, NULL};	/*Value = True*/
const Property P16332_1 = {S50, 0, &V16332_1_1, NULL};	/*Property = IsMobile*/
const Value V16332_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P16332_5 = {S52, 0, &V16332_5_9981, &P16332_1};	/*Property = ScreenPixelsHeight*/
const Value V16332_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P16332_4 = {S54, 0, &V16332_4_39098, &P16332_5};	/*Property = ScreenPixelsWidth*/
const Profile P16332 = {"16332", &P16332_4};	/*Profile ID - 16332*/
const Profile P22310 = {"22310", NULL};	/*Profile ID - 22310*/
const Value V13177_1_1 = {S2, NULL};	/*Value = True*/
const Property P13177_1 = {S50, 0, &V13177_1_1, NULL};	/*Property = IsMobile*/
const Value V13177_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13177_5 = {S52, 0, &V13177_5_1233, &P13177_1};	/*Property = ScreenPixelsHeight*/
const Value V13177_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13177_4 = {S54, 0, &V13177_4_1235, &P13177_5};	/*Property = ScreenPixelsWidth*/
const Profile P13177 = {"13177", &P13177_4};	/*Profile ID - 13177*/
const Value V13178_1_1 = {S2, NULL};	/*Value = True*/
const Property P13178_1 = {S50, 0, &V13178_1_1, NULL};	/*Property = IsMobile*/
const Value V13178_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13178_5 = {S52, 0, &V13178_5_1233, &P13178_1};	/*Property = ScreenPixelsHeight*/
const Value V13178_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13178_4 = {S54, 0, &V13178_4_1235, &P13178_5};	/*Property = ScreenPixelsWidth*/
const Profile P13178 = {"13178", &P13178_4};	/*Profile ID - 13178*/
const Value V13180_1_1 = {S2, NULL};	/*Value = True*/
const Property P13180_1 = {S50, 0, &V13180_1_1, NULL};	/*Property = IsMobile*/
const Value V13180_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13180_5 = {S52, 0, &V13180_5_1233, &P13180_1};	/*Property = ScreenPixelsHeight*/
const Value V13180_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13180_4 = {S54, 0, &V13180_4_1235, &P13180_5};	/*Property = ScreenPixelsWidth*/
const Profile P13180 = {"13180", &P13180_4};	/*Profile ID - 13180*/
const Value V18061_1_1 = {S2, NULL};	/*Value = True*/
const Property P18061_1 = {S50, 0, &V18061_1_1, NULL};	/*Property = IsMobile*/
const Value V18061_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P18061_5 = {S52, 0, &V18061_5_37, &P18061_1};	/*Property = ScreenPixelsHeight*/
const Value V18061_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P18061_4 = {S54, 0, &V18061_4_36, &P18061_5};	/*Property = ScreenPixelsWidth*/
const Profile P18061 = {"18061", &P18061_4};	/*Profile ID - 18061*/
const Value V13182_1_1 = {S2, NULL};	/*Value = True*/
const Property P13182_1 = {S50, 0, &V13182_1_1, NULL};	/*Property = IsMobile*/
const Value V13182_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13182_5 = {S52, 0, &V13182_5_1233, &P13182_1};	/*Property = ScreenPixelsHeight*/
const Value V13182_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13182_4 = {S54, 0, &V13182_4_1235, &P13182_5};	/*Property = ScreenPixelsWidth*/
const Profile P13182 = {"13182", &P13182_4};	/*Profile ID - 13182*/
const Value V13191_1_1 = {S2, NULL};	/*Value = True*/
const Property P13191_1 = {S50, 0, &V13191_1_1, NULL};	/*Property = IsMobile*/
const Value V13191_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13191_5 = {S52, 0, &V13191_5_1293, &P13191_1};	/*Property = ScreenPixelsHeight*/
const Value V13191_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13191_4 = {S54, 0, &V13191_4_1296, &P13191_5};	/*Property = ScreenPixelsWidth*/
const Profile P13191 = {"13191", &P13191_4};	/*Profile ID - 13191*/
const Value V13194_1_1 = {S2, NULL};	/*Value = True*/
const Property P13194_1 = {S50, 0, &V13194_1_1, NULL};	/*Property = IsMobile*/
const Value V13194_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13194_5 = {S52, 0, &V13194_5_1222, &P13194_1};	/*Property = ScreenPixelsHeight*/
const Value V13194_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13194_4 = {S54, 0, &V13194_4_1224, &P13194_5};	/*Property = ScreenPixelsWidth*/
const Profile P13194 = {"13194", &P13194_4};	/*Profile ID - 13194*/
const Value V13196_1_1 = {S2, NULL};	/*Value = True*/
const Property P13196_1 = {S50, 0, &V13196_1_1, NULL};	/*Property = IsMobile*/
const Value V13196_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13196_5 = {S52, 0, &V13196_5_1294, &P13196_1};	/*Property = ScreenPixelsHeight*/
const Value V13196_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13196_4 = {S54, 0, &V13196_4_1296, &P13196_5};	/*Property = ScreenPixelsWidth*/
const Profile P13196 = {"13196", &P13196_4};	/*Profile ID - 13196*/
const Value V17943_1_1 = {S2, NULL};	/*Value = True*/
const Property P17943_1 = {S50, 0, &V17943_1_1, NULL};	/*Property = IsMobile*/
const Value V17943_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17943_5 = {S52, 0, &V17943_5_1277, &P17943_1};	/*Property = ScreenPixelsHeight*/
const Value V17943_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17943_4 = {S54, 0, &V17943_4_1224, &P17943_5};	/*Property = ScreenPixelsWidth*/
const Profile P17943 = {"17943", &P17943_4};	/*Profile ID - 17943*/
const Value V13198_1_1 = {S2, NULL};	/*Value = True*/
const Property P13198_1 = {S50, 0, &V13198_1_1, NULL};	/*Property = IsMobile*/
const Value V13198_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P13198_5 = {S52, 0, &V13198_5_1269, &P13198_1};	/*Property = ScreenPixelsHeight*/
const Value V13198_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13198_4 = {S54, 0, &V13198_4_1229, &P13198_5};	/*Property = ScreenPixelsWidth*/
const Profile P13198 = {"13198", &P13198_4};	/*Profile ID - 13198*/
const Profile P17658 = {"17658", NULL};	/*Profile ID - 17658*/
const Value V18392_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_279 = {S1, 0, &V18392_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18392_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_282 = {S3, 0, &V18392_282_57155, &P18392_279};	/*Property = BlobBuilder*/
const Value V18392_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_350 = {S4, 0, &V18392_350_57446, &P18392_282};	/*Property = Canvas*/
const Value V18392_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_283 = {S5, 0, &V18392_283_57161, &P18392_350};	/*Property = CssBackground*/
const Value V18392_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_284 = {S6, 0, &V18392_284_57164, &P18392_283};	/*Property = CssBorderImage*/
const Value V18392_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_285 = {S7, 0, &V18392_285_57167, &P18392_284};	/*Property = CssCanvas*/
const Value V18392_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_286 = {S8, 0, &V18392_286_57170, &P18392_285};	/*Property = CssColor*/
const Value V18392_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_237 = {S9, 0, &V18392_237_55786, &P18392_286};	/*Property = CssColumn*/
const Value V18392_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_287 = {S10, 0, &V18392_287_57173, &P18392_237};	/*Property = CssFlexbox*/
const Value V18392_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_288 = {S11, 0, &V18392_288_57176, &P18392_287};	/*Property = CssFont*/
const Value V18392_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_289 = {S12, 0, &V18392_289_86899, &P18392_288};	/*Property = CssImages*/
const Value V18392_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_291 = {S13, 0, &V18392_291_57182, &P18392_289};	/*Property = CssMediaQueries*/
const Value V18392_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_292 = {S14, 0, &V18392_292_57185, &P18392_291};	/*Property = CssMinMax*/
const Value V18392_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_293 = {S15, 0, &V18392_293_57188, &P18392_292};	/*Property = CssOverflow*/
const Value V18392_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_294 = {S16, 0, &V18392_294_57191, &P18392_293};	/*Property = CssPosition*/
const Value V18392_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_295 = {S17, 0, &V18392_295_57194, &P18392_294};	/*Property = CssText*/
const Value V18392_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_297 = {S18, 0, &V18392_297_57197, &P18392_295};	/*Property = CssTransforms*/
const Value V18392_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_241 = {S19, 0, &V18392_241_55801, &P18392_297};	/*Property = CssTransitions*/
const Value V18392_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_298 = {S20, 0, &V18392_298_57208, &P18392_241};	/*Property = CssUI*/
const Value V18392_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_300 = {S21, 0, &V18392_300_57217, &P18392_298};	/*Property = DataSet*/
const Value V18392_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_301 = {S22, 0, &V18392_301_57220, &P18392_300};	/*Property = DataUrl*/
const Value V18392_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_302 = {S23, 0, &V18392_302_57223, &P18392_301};	/*Property = DeviceOrientation*/
const Value V18392_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_304 = {S24, 0, &V18392_304_57229, &P18392_302};	/*Property = FileReader*/
const Value V18392_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_305 = {S25, 0, &V18392_305_57232, &P18392_304};	/*Property = FileSaver*/
const Value V18392_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_306 = {S26, 0, &V18392_306_57238, &P18392_305};	/*Property = FileWriter*/
const Value V18392_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_307 = {S27, 0, &V18392_307_57241, &P18392_306};	/*Property = FormData*/
const Value V18392_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_309 = {S28, 0, &V18392_309_57247, &P18392_307};	/*Property = Fullscreen*/
const Value V18392_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_310 = {S29, 0, &V18392_310_57250, &P18392_309};	/*Property = GeoLocation*/
const Value V18392_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_312 = {S30, 0, &V18392_312_57256, &P18392_310};	/*Property = History*/
const Value V18392_314_57260 = {S2, NULL};	/*Value = True*/
const Property P18392_314 = {S31, 0, &V18392_314_57260, &P18392_312};	/*Property = Html5*/
const Value V18392_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_313 = {S32, 0, &V18392_313_57259, &P18392_314};	/*Property = Html-Media-Capture*/
const Value V18392_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_315 = {S33, 0, &V18392_315_57265, &P18392_313};	/*Property = Iframe*/
const Value V18392_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_316 = {S34, 0, &V18392_316_57268, &P18392_315};	/*Property = IndexedDB*/
const Value V18392_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_317 = {S35, 0, &V18392_317_57271, &P18392_316};	/*Property = Json*/
const Value V18392_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18392_15 = {S37, 0, &V18392_15_28, &P18392_317};	/*Property = LayoutEngine*/
const Value V18392_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_344 = {S38, 0, &V18392_344_57274, &P18392_15};	/*Property = Masking*/
const Value V18392_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_320 = {S39, 0, &V18392_320_57292, &P18392_344};	/*Property = PostMessage*/
const Value V18392_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_321 = {S40, 0, &V18392_321_57295, &P18392_320};	/*Property = Progress*/
const Value V18392_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_322 = {S41, 0, &V18392_322_57298, &P18392_321};	/*Property = Prompts*/
const Value V18392_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_323 = {S42, 0, &V18392_323_57301, &P18392_322};	/*Property = Selector*/
const Value V18392_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_324 = {S43, 0, &V18392_324_57307, &P18392_323};	/*Property = Svg*/
const Value V18392_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_327 = {S44, 0, &V18392_327_57316, &P18392_324};	/*Property = TouchEvents*/
const Value V18392_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_328 = {S45, 0, &V18392_328_57319, &P18392_327};	/*Property = Track*/
const Value V18392_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_329 = {S46, 0, &V18392_329_57325, &P18392_328};	/*Property = Video*/
const Value V18392_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_330 = {S47, 0, &V18392_330_57328, &P18392_329};	/*Property = Viewport*/
const Value V18392_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_332 = {S48, 0, &V18392_332_57340, &P18392_330};	/*Property = WebWorkers*/
const Value V18392_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18392_333 = {S49, 0, &V18392_333_57343, &P18392_332};	/*Property = Xhr2*/
const Profile P18392 = {"18392", &P18392_333};	/*Profile ID - 18392*/
const Value V13200_1_1 = {S2, NULL};	/*Value = True*/
const Property P13200_1 = {S50, 0, &V13200_1_1, NULL};	/*Property = IsMobile*/
const Value V13200_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13200_5 = {S52, 0, &V13200_5_1277, &P13200_1};	/*Property = ScreenPixelsHeight*/
const Value V13200_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13200_4 = {S54, 0, &V13200_4_1279, &P13200_5};	/*Property = ScreenPixelsWidth*/
const Profile P13200 = {"13200", &P13200_4};	/*Profile ID - 13200*/
const Value V14310_1_1 = {S2, NULL};	/*Value = True*/
const Property P14310_1 = {S50, 0, &V14310_1_1, NULL};	/*Property = IsMobile*/
const Value V14310_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14310_5 = {S52, 0, &V14310_5_1294, &P14310_1};	/*Property = ScreenPixelsHeight*/
const Value V14310_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14310_4 = {S54, 0, &V14310_4_1296, &P14310_5};	/*Property = ScreenPixelsWidth*/
const Profile P14310 = {"14310", &P14310_4};	/*Profile ID - 14310*/
const Value V13202_1_1 = {S2, NULL};	/*Value = True*/
const Property P13202_1 = {S50, 0, &V13202_1_1, NULL};	/*Property = IsMobile*/
const Value V13202_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P13202_5 = {S52, 0, &V13202_5_1269, &P13202_1};	/*Property = ScreenPixelsHeight*/
const Value V13202_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13202_4 = {S54, 0, &V13202_4_1229, &P13202_5};	/*Property = ScreenPixelsWidth*/
const Profile P13202 = {"13202", &P13202_4};	/*Profile ID - 13202*/
const Value V14834_1_1 = {S2, NULL};	/*Value = True*/
const Property P14834_1 = {S50, 0, &V14834_1_1, NULL};	/*Property = IsMobile*/
const Value V14834_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14834_5 = {S52, 0, &V14834_5_1222, &P14834_1};	/*Property = ScreenPixelsHeight*/
const Value V14834_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14834_4 = {S54, 0, &V14834_4_1224, &P14834_5};	/*Property = ScreenPixelsWidth*/
const Profile P14834 = {"14834", &P14834_4};	/*Profile ID - 14834*/
const Value V24001_1_1 = {S2, NULL};	/*Value = True*/
const Property P24001_1 = {S50, 0, &V24001_1_1, NULL};	/*Property = IsMobile*/
const Value V24001_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24001_5 = {S52, 0, &V24001_5_1277, &P24001_1};	/*Property = ScreenPixelsHeight*/
const Value V24001_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24001_4 = {S54, 0, &V24001_4_1279, &P24001_5};	/*Property = ScreenPixelsWidth*/
const Profile P24001 = {"24001", &P24001_4};	/*Profile ID - 24001*/
const Value V13207_1_1 = {S2, NULL};	/*Value = True*/
const Property P13207_1 = {S50, 0, &V13207_1_1, NULL};	/*Property = IsMobile*/
const Value V13207_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13207_5 = {S52, 0, &V13207_5_1233, &P13207_1};	/*Property = ScreenPixelsHeight*/
const Value V13207_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13207_4 = {S54, 0, &V13207_4_1235, &P13207_5};	/*Property = ScreenPixelsWidth*/
const Profile P13207 = {"13207", &P13207_4};	/*Profile ID - 13207*/
const Value V13208_1_1 = {S2, NULL};	/*Value = True*/
const Property P13208_1 = {S50, 0, &V13208_1_1, NULL};	/*Property = IsMobile*/
const Value V13208_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13208_5 = {S52, 0, &V13208_5_1304, &P13208_1};	/*Property = ScreenPixelsHeight*/
const Value V13208_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13208_4 = {S54, 0, &V13208_4_1224, &P13208_5};	/*Property = ScreenPixelsWidth*/
const Profile P13208 = {"13208", &P13208_4};	/*Profile ID - 13208*/
const Value V13209_1_1 = {S2, NULL};	/*Value = True*/
const Property P13209_1 = {S50, 0, &V13209_1_1, NULL};	/*Property = IsMobile*/
const Value V13209_5_37514 = {S107, NULL};	/*Value = 266*/
const Property P13209_5 = {S52, 0, &V13209_5_37514, &P13209_1};	/*Property = ScreenPixelsHeight*/
const Value V13209_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13209_4 = {S54, 0, &V13209_4_1224, &P13209_5};	/*Property = ScreenPixelsWidth*/
const Profile P13209 = {"13209", &P13209_4};	/*Profile ID - 13209*/
const Profile P18168 = {"18168", NULL};	/*Profile ID - 18168*/
const Value V13211_1_1 = {S2, NULL};	/*Value = True*/
const Property P13211_1 = {S50, 0, &V13211_1_1, NULL};	/*Property = IsMobile*/
const Value V13211_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P13211_5 = {S52, 0, &V13211_5_37, &P13211_1};	/*Property = ScreenPixelsHeight*/
const Value V13211_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P13211_4 = {S54, 0, &V13211_4_36, &P13211_5};	/*Property = ScreenPixelsWidth*/
const Profile P13211 = {"13211", &P13211_4};	/*Profile ID - 13211*/
const Value V14250_1_1 = {S2, NULL};	/*Value = True*/
const Property P14250_1 = {S50, 0, &V14250_1_1, NULL};	/*Property = IsMobile*/
const Value V14250_5_37453 = {S91, NULL};	/*Value = 65*/
const Property P14250_5 = {S52, 0, &V14250_5_37453, &P14250_1};	/*Property = ScreenPixelsHeight*/
const Value V14250_4_31956 = {S92, NULL};	/*Value = 96*/
const Property P14250_4 = {S54, 0, &V14250_4_31956, &P14250_5};	/*Property = ScreenPixelsWidth*/
const Profile P14250 = {"14250", &P14250_4};	/*Profile ID - 14250*/
const Value V13217_1_1 = {S2, NULL};	/*Value = True*/
const Property P13217_1 = {S50, 0, &V13217_1_1, NULL};	/*Property = IsMobile*/
const Value V13217_5_31826 = {S75, NULL};	/*Value = 260*/
const Property P13217_5 = {S52, 0, &V13217_5_31826, &P13217_1};	/*Property = ScreenPixelsHeight*/
const Value V13217_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P13217_4 = {S54, 0, &V13217_4_1943, &P13217_5};	/*Property = ScreenPixelsWidth*/
const Profile P13217 = {"13217", &P13217_4};	/*Profile ID - 13217*/
const Value V13249_1_1 = {S2, NULL};	/*Value = True*/
const Property P13249_1 = {S50, 0, &V13249_1_1, NULL};	/*Property = IsMobile*/
const Value V13249_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13249_5 = {S52, 0, &V13249_5_1304, &P13249_1};	/*Property = ScreenPixelsHeight*/
const Value V13249_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13249_4 = {S54, 0, &V13249_4_1224, &P13249_5};	/*Property = ScreenPixelsWidth*/
const Profile P13249 = {"13249", &P13249_4};	/*Profile ID - 13249*/
const Value V13234_1_1 = {S2, NULL};	/*Value = True*/
const Property P13234_1 = {S50, 0, &V13234_1_1, NULL};	/*Property = IsMobile*/
const Value V13234_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13234_5 = {S52, 0, &V13234_5_1277, &P13234_1};	/*Property = ScreenPixelsHeight*/
const Value V13234_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13234_4 = {S54, 0, &V13234_4_1279, &P13234_5};	/*Property = ScreenPixelsWidth*/
const Profile P13234 = {"13234", &P13234_4};	/*Profile ID - 13234*/
const Value V13221_1_1 = {S2, NULL};	/*Value = True*/
const Property P13221_1 = {S50, 0, &V13221_1_1, NULL};	/*Property = IsMobile*/
const Value V13221_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13221_5 = {S52, 0, &V13221_5_1304, &P13221_1};	/*Property = ScreenPixelsHeight*/
const Value V13221_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13221_4 = {S54, 0, &V13221_4_1224, &P13221_5};	/*Property = ScreenPixelsWidth*/
const Profile P13221 = {"13221", &P13221_4};	/*Profile ID - 13221*/
const Value V13222_1_1 = {S2, NULL};	/*Value = True*/
const Property P13222_1 = {S50, 0, &V13222_1_1, NULL};	/*Property = IsMobile*/
const Value V13222_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13222_5 = {S52, 0, &V13222_5_1227, &P13222_1};	/*Property = ScreenPixelsHeight*/
const Value V13222_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13222_4 = {S54, 0, &V13222_4_1229, &P13222_5};	/*Property = ScreenPixelsWidth*/
const Profile P13222 = {"13222", &P13222_4};	/*Profile ID - 13222*/
const Value V13223_1_1 = {S2, NULL};	/*Value = True*/
const Property P13223_1 = {S50, 0, &V13223_1_1, NULL};	/*Property = IsMobile*/
const Value V13223_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13223_5 = {S52, 0, &V13223_5_1304, &P13223_1};	/*Property = ScreenPixelsHeight*/
const Value V13223_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13223_4 = {S54, 0, &V13223_4_1224, &P13223_5};	/*Property = ScreenPixelsWidth*/
const Profile P13223 = {"13223", &P13223_4};	/*Profile ID - 13223*/
const Profile P6729 = {"6729", NULL};	/*Profile ID - 6729*/
const Value V18000_1_1 = {S2, NULL};	/*Value = True*/
const Property P18000_1 = {S50, 0, &V18000_1_1, NULL};	/*Property = IsMobile*/
const Value V18000_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18000_5 = {S52, 0, &V18000_5_1227, &P18000_1};	/*Property = ScreenPixelsHeight*/
const Value V18000_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18000_4 = {S54, 0, &V18000_4_1229, &P18000_5};	/*Property = ScreenPixelsWidth*/
const Profile P18000 = {"18000", &P18000_4};	/*Profile ID - 18000*/
const Value V14657_1_1 = {S2, NULL};	/*Value = True*/
const Property P14657_1 = {S50, 0, &V14657_1_1, NULL};	/*Property = IsMobile*/
const Value V14657_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14657_5 = {S52, 0, &V14657_5_1227, &P14657_1};	/*Property = ScreenPixelsHeight*/
const Value V14657_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14657_4 = {S54, 0, &V14657_4_1229, &P14657_5};	/*Property = ScreenPixelsWidth*/
const Profile P14657 = {"14657", &P14657_4};	/*Profile ID - 14657*/
const Value V13231_1_1 = {S2, NULL};	/*Value = True*/
const Property P13231_1 = {S50, 0, &V13231_1_1, NULL};	/*Property = IsMobile*/
const Value V13231_5_37524 = {S79, NULL};	/*Value = 440*/
const Property P13231_5 = {S52, 0, &V13231_5_37524, &P13231_1};	/*Property = ScreenPixelsHeight*/
const Value V13231_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13231_4 = {S54, 0, &V13231_4_1224, &P13231_5};	/*Property = ScreenPixelsWidth*/
const Profile P13231 = {"13231", &P13231_4};	/*Profile ID - 13231*/
const Value V18004_1_1 = {S2, NULL};	/*Value = True*/
const Property P18004_1 = {S50, 0, &V18004_1_1, NULL};	/*Property = IsMobile*/
const Value V18004_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P18004_5 = {S52, 0, &V18004_5_1304, &P18004_1};	/*Property = ScreenPixelsHeight*/
const Value V18004_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18004_4 = {S54, 0, &V18004_4_1279, &P18004_5};	/*Property = ScreenPixelsWidth*/
const Profile P18004 = {"18004", &P18004_4};	/*Profile ID - 18004*/
const Value V13244_1_1 = {S2, NULL};	/*Value = True*/
const Property P13244_1 = {S50, 0, &V13244_1_1, NULL};	/*Property = IsMobile*/
const Value V13244_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13244_5 = {S52, 0, &V13244_5_1277, &P13244_1};	/*Property = ScreenPixelsHeight*/
const Value V13244_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13244_4 = {S54, 0, &V13244_4_1279, &P13244_5};	/*Property = ScreenPixelsWidth*/
const Profile P13244 = {"13244", &P13244_4};	/*Profile ID - 13244*/
const Value V13251_1_1 = {S2, NULL};	/*Value = True*/
const Property P13251_1 = {S50, 0, &V13251_1_1, NULL};	/*Property = IsMobile*/
const Value V13251_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13251_5 = {S52, 0, &V13251_5_1227, &P13251_1};	/*Property = ScreenPixelsHeight*/
const Value V13251_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13251_4 = {S54, 0, &V13251_4_1229, &P13251_5};	/*Property = ScreenPixelsWidth*/
const Profile P13251 = {"13251", &P13251_4};	/*Profile ID - 13251*/
const Value V13242_1_1 = {S2, NULL};	/*Value = True*/
const Property P13242_1 = {S50, 0, &V13242_1_1, NULL};	/*Property = IsMobile*/
const Value V13242_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P13242_5 = {S52, 0, &V13242_5_1269, &P13242_1};	/*Property = ScreenPixelsHeight*/
const Value V13242_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13242_4 = {S54, 0, &V13242_4_1229, &P13242_5};	/*Property = ScreenPixelsWidth*/
const Profile P13242 = {"13242", &P13242_4};	/*Profile ID - 13242*/
const Value V13246_1_1 = {S2, NULL};	/*Value = True*/
const Property P13246_1 = {S50, 0, &V13246_1_1, NULL};	/*Property = IsMobile*/
const Value V13246_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13246_5 = {S52, 0, &V13246_5_1222, &P13246_1};	/*Property = ScreenPixelsHeight*/
const Value V13246_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13246_4 = {S54, 0, &V13246_4_1224, &P13246_5};	/*Property = ScreenPixelsWidth*/
const Profile P13246 = {"13246", &P13246_4};	/*Profile ID - 13246*/
const Value V13248_1_1 = {S2, NULL};	/*Value = True*/
const Property P13248_1 = {S50, 0, &V13248_1_1, NULL};	/*Property = IsMobile*/
const Value V13248_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13248_5 = {S52, 0, &V13248_5_1222, &P13248_1};	/*Property = ScreenPixelsHeight*/
const Value V13248_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13248_4 = {S54, 0, &V13248_4_1224, &P13248_5};	/*Property = ScreenPixelsWidth*/
const Profile P13248 = {"13248", &P13248_4};	/*Profile ID - 13248*/
const Value V13250_1_1 = {S2, NULL};	/*Value = True*/
const Property P13250_1 = {S50, 0, &V13250_1_1, NULL};	/*Property = IsMobile*/
const Value V13250_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13250_5 = {S52, 0, &V13250_5_1277, &P13250_1};	/*Property = ScreenPixelsHeight*/
const Value V13250_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13250_4 = {S54, 0, &V13250_4_1279, &P13250_5};	/*Property = ScreenPixelsWidth*/
const Profile P13250 = {"13250", &P13250_4};	/*Profile ID - 13250*/
const Value V13255_1_1 = {S2, NULL};	/*Value = True*/
const Property P13255_1 = {S50, 0, &V13255_1_1, NULL};	/*Property = IsMobile*/
const Value V13255_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13255_5 = {S52, 0, &V13255_5_1304, &P13255_1};	/*Property = ScreenPixelsHeight*/
const Value V13255_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13255_4 = {S54, 0, &V13255_4_1224, &P13255_5};	/*Property = ScreenPixelsWidth*/
const Profile P13255 = {"13255", &P13255_4};	/*Profile ID - 13255*/
const Value V13256_1_1 = {S2, NULL};	/*Value = True*/
const Property P13256_1 = {S50, 0, &V13256_1_1, NULL};	/*Property = IsMobile*/
const Value V13256_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13256_5 = {S52, 0, &V13256_5_1222, &P13256_1};	/*Property = ScreenPixelsHeight*/
const Value V13256_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13256_4 = {S54, 0, &V13256_4_1224, &P13256_5};	/*Property = ScreenPixelsWidth*/
const Profile P13256 = {"13256", &P13256_4};	/*Profile ID - 13256*/
const Value V13259_1_1 = {S2, NULL};	/*Value = True*/
const Property P13259_1 = {S50, 0, &V13259_1_1, NULL};	/*Property = IsMobile*/
const Value V13259_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13259_5 = {S52, 0, &V13259_5_1222, &P13259_1};	/*Property = ScreenPixelsHeight*/
const Value V13259_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13259_4 = {S54, 0, &V13259_4_1224, &P13259_5};	/*Property = ScreenPixelsWidth*/
const Profile P13259 = {"13259", &P13259_4};	/*Profile ID - 13259*/
const Value V14401_1_1 = {S2, NULL};	/*Value = True*/
const Property P14401_1 = {S50, 0, &V14401_1_1, NULL};	/*Property = IsMobile*/
const Value V14401_5_37453 = {S91, NULL};	/*Value = 65*/
const Property P14401_5 = {S52, 0, &V14401_5_37453, &P14401_1};	/*Property = ScreenPixelsHeight*/
const Value V14401_4_31956 = {S92, NULL};	/*Value = 96*/
const Property P14401_4 = {S54, 0, &V14401_4_31956, &P14401_5};	/*Property = ScreenPixelsWidth*/
const Profile P14401 = {"14401", &P14401_4};	/*Profile ID - 14401*/
const Value V13262_1_1 = {S2, NULL};	/*Value = True*/
const Property P13262_1 = {S50, 0, &V13262_1_1, NULL};	/*Property = IsMobile*/
const Value V13262_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13262_5 = {S52, 0, &V13262_5_1293, &P13262_1};	/*Property = ScreenPixelsHeight*/
const Value V13262_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13262_4 = {S54, 0, &V13262_4_1296, &P13262_5};	/*Property = ScreenPixelsWidth*/
const Profile P13262 = {"13262", &P13262_4};	/*Profile ID - 13262*/
const Value V13289_1_1 = {S2, NULL};	/*Value = True*/
const Property P13289_1 = {S50, 0, &V13289_1_1, NULL};	/*Property = IsMobile*/
const Value V13289_5_37453 = {S91, NULL};	/*Value = 65*/
const Property P13289_5 = {S52, 0, &V13289_5_37453, &P13289_1};	/*Property = ScreenPixelsHeight*/
const Value V13289_4_31956 = {S92, NULL};	/*Value = 96*/
const Property P13289_4 = {S54, 0, &V13289_4_31956, &P13289_5};	/*Property = ScreenPixelsWidth*/
const Profile P13289 = {"13289", &P13289_4};	/*Profile ID - 13289*/
const Value V17551_1_1 = {S2, NULL};	/*Value = True*/
const Property P17551_1 = {S50, 0, &V17551_1_1, NULL};	/*Property = IsMobile*/
const Value V17551_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17551_5 = {S52, 0, &V17551_5_1269, &P17551_1};	/*Property = ScreenPixelsHeight*/
const Value V17551_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17551_4 = {S54, 0, &V17551_4_1229, &P17551_5};	/*Property = ScreenPixelsWidth*/
const Profile P17551 = {"17551", &P17551_4};	/*Profile ID - 17551*/
const Value V13864_1_1 = {S2, NULL};	/*Value = True*/
const Property P13864_1 = {S50, 0, &V13864_1_1, NULL};	/*Property = IsMobile*/
const Value V13864_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P13864_5 = {S52, 0, &V13864_5_1333, &P13864_1};	/*Property = ScreenPixelsHeight*/
const Value V13864_4_37572 = {S93, NULL};	/*Value = 132*/
const Property P13864_4 = {S54, 0, &V13864_4_37572, &P13864_5};	/*Property = ScreenPixelsWidth*/
const Profile P13864 = {"13864", &P13864_4};	/*Profile ID - 13864*/
const Value V13920_1_1 = {S2, NULL};	/*Value = True*/
const Property P13920_1 = {S50, 0, &V13920_1_1, NULL};	/*Property = IsMobile*/
const Value V13920_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13920_5 = {S52, 0, &V13920_5_1222, &P13920_1};	/*Property = ScreenPixelsHeight*/
const Value V13920_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13920_4 = {S54, 0, &V13920_4_1224, &P13920_5};	/*Property = ScreenPixelsWidth*/
const Profile P13920 = {"13920", &P13920_4};	/*Profile ID - 13920*/
const Value V21682_1_1 = {S2, NULL};	/*Value = True*/
const Property P21682_1 = {S50, 0, &V21682_1_1, NULL};	/*Property = IsMobile*/
const Value V21682_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P21682_5 = {S52, 0, &V21682_5_1294, &P21682_1};	/*Property = ScreenPixelsHeight*/
const Value V21682_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P21682_4 = {S54, 0, &V21682_4_1296, &P21682_5};	/*Property = ScreenPixelsWidth*/
const Profile P21682 = {"21682", &P21682_4};	/*Profile ID - 21682*/
const Value V14672_1_1 = {S2, NULL};	/*Value = True*/
const Property P14672_1 = {S50, 0, &V14672_1_1, NULL};	/*Property = IsMobile*/
const Value V14672_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14672_5 = {S52, 0, &V14672_5_1304, &P14672_1};	/*Property = ScreenPixelsHeight*/
const Value V14672_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14672_4 = {S54, 0, &V14672_4_1224, &P14672_5};	/*Property = ScreenPixelsWidth*/
const Profile P14672 = {"14672", &P14672_4};	/*Profile ID - 14672*/
const Value V13934_1_1 = {S2, NULL};	/*Value = True*/
const Property P13934_1 = {S50, 0, &V13934_1_1, NULL};	/*Property = IsMobile*/
const Value V13934_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13934_5 = {S52, 0, &V13934_5_1222, &P13934_1};	/*Property = ScreenPixelsHeight*/
const Value V13934_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13934_4 = {S54, 0, &V13934_4_1224, &P13934_5};	/*Property = ScreenPixelsWidth*/
const Profile P13934 = {"13934", &P13934_4};	/*Profile ID - 13934*/
const Value V13938_1_1 = {S2, NULL};	/*Value = True*/
const Property P13938_1 = {S50, 0, &V13938_1_1, NULL};	/*Property = IsMobile*/
const Value V13938_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13938_5 = {S52, 0, &V13938_5_1233, &P13938_1};	/*Property = ScreenPixelsHeight*/
const Value V13938_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13938_4 = {S54, 0, &V13938_4_1235, &P13938_5};	/*Property = ScreenPixelsWidth*/
const Profile P13938 = {"13938", &P13938_4};	/*Profile ID - 13938*/
const Value V14484_1_1 = {S2, NULL};	/*Value = True*/
const Property P14484_1 = {S50, 0, &V14484_1_1, NULL};	/*Property = IsMobile*/
const Value V14484_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14484_5 = {S52, 0, &V14484_5_1222, &P14484_1};	/*Property = ScreenPixelsHeight*/
const Value V14484_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14484_4 = {S54, 0, &V14484_4_1224, &P14484_5};	/*Property = ScreenPixelsWidth*/
const Profile P14484 = {"14484", &P14484_4};	/*Profile ID - 14484*/
const Value V14520_1_1 = {S2, NULL};	/*Value = True*/
const Property P14520_1 = {S50, 0, &V14520_1_1, NULL};	/*Property = IsMobile*/
const Value V14520_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14520_5 = {S52, 0, &V14520_5_1294, &P14520_1};	/*Property = ScreenPixelsHeight*/
const Value V14520_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14520_4 = {S54, 0, &V14520_4_1296, &P14520_5};	/*Property = ScreenPixelsWidth*/
const Profile P14520 = {"14520", &P14520_4};	/*Profile ID - 14520*/
const Value V13293_1_1 = {S2, NULL};	/*Value = True*/
const Property P13293_1 = {S50, 0, &V13293_1_1, NULL};	/*Property = IsMobile*/
const Value V13293_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13293_5 = {S52, 0, &V13293_5_1222, &P13293_1};	/*Property = ScreenPixelsHeight*/
const Value V13293_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13293_4 = {S54, 0, &V13293_4_1224, &P13293_5};	/*Property = ScreenPixelsWidth*/
const Profile P13293 = {"13293", &P13293_4};	/*Profile ID - 13293*/
const Value V13943_1_1 = {S2, NULL};	/*Value = True*/
const Property P13943_1 = {S50, 0, &V13943_1_1, NULL};	/*Property = IsMobile*/
const Value V13943_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13943_5 = {S52, 0, &V13943_5_1222, &P13943_1};	/*Property = ScreenPixelsHeight*/
const Value V13943_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13943_4 = {S54, 0, &V13943_4_1224, &P13943_5};	/*Property = ScreenPixelsWidth*/
const Profile P13943 = {"13943", &P13943_4};	/*Profile ID - 13943*/
const Value V13944_1_1 = {S2, NULL};	/*Value = True*/
const Property P13944_1 = {S50, 0, &V13944_1_1, NULL};	/*Property = IsMobile*/
const Value V13944_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P13944_5 = {S52, 0, &V13944_5_1275, &P13944_1};	/*Property = ScreenPixelsHeight*/
const Value V13944_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P13944_4 = {S54, 0, &V13944_4_1278, &P13944_5};	/*Property = ScreenPixelsWidth*/
const Profile P13944 = {"13944", &P13944_4};	/*Profile ID - 13944*/
const Value V13950_1_1 = {S2, NULL};	/*Value = True*/
const Property P13950_1 = {S50, 0, &V13950_1_1, NULL};	/*Property = IsMobile*/
const Value V13950_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13950_5 = {S52, 0, &V13950_5_1222, &P13950_1};	/*Property = ScreenPixelsHeight*/
const Value V13950_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13950_4 = {S54, 0, &V13950_4_1224, &P13950_5};	/*Property = ScreenPixelsWidth*/
const Profile P13950 = {"13950", &P13950_4};	/*Profile ID - 13950*/
const Value V13952_1_1 = {S2, NULL};	/*Value = True*/
const Property P13952_1 = {S50, 0, &V13952_1_1, NULL};	/*Property = IsMobile*/
const Value V13952_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13952_5 = {S52, 0, &V13952_5_1222, &P13952_1};	/*Property = ScreenPixelsHeight*/
const Value V13952_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13952_4 = {S54, 0, &V13952_4_1224, &P13952_5};	/*Property = ScreenPixelsWidth*/
const Profile P13952 = {"13952", &P13952_4};	/*Profile ID - 13952*/
const Value V13954_1_1 = {S2, NULL};	/*Value = True*/
const Property P13954_1 = {S50, 0, &V13954_1_1, NULL};	/*Property = IsMobile*/
const Value V13954_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13954_5 = {S52, 0, &V13954_5_1227, &P13954_1};	/*Property = ScreenPixelsHeight*/
const Value V13954_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13954_4 = {S54, 0, &V13954_4_1229, &P13954_5};	/*Property = ScreenPixelsWidth*/
const Profile P13954 = {"13954", &P13954_4};	/*Profile ID - 13954*/
const Value V13955_1_1 = {S2, NULL};	/*Value = True*/
const Property P13955_1 = {S50, 0, &V13955_1_1, NULL};	/*Property = IsMobile*/
const Value V13955_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13955_5 = {S52, 0, &V13955_5_1293, &P13955_1};	/*Property = ScreenPixelsHeight*/
const Value V13955_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13955_4 = {S54, 0, &V13955_4_1296, &P13955_5};	/*Property = ScreenPixelsWidth*/
const Profile P13955 = {"13955", &P13955_4};	/*Profile ID - 13955*/
const Value V20801_1_1 = {S2, NULL};	/*Value = True*/
const Property P20801_1 = {S50, 0, &V20801_1_1, NULL};	/*Property = IsMobile*/
const Value V20801_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20801_5 = {S52, 0, &V20801_5_1403, &P20801_1};	/*Property = ScreenPixelsHeight*/
const Value V20801_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20801_4 = {S54, 0, &V20801_4_1404, &P20801_5};	/*Property = ScreenPixelsWidth*/
const Profile P20801 = {"20801", &P20801_4};	/*Profile ID - 20801*/
const Value V15819_1_1 = {S2, NULL};	/*Value = True*/
const Property P15819_1 = {S50, 0, &V15819_1_1, NULL};	/*Property = IsMobile*/
const Value V15819_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P15819_5 = {S52, 0, &V15819_5_1403, &P15819_1};	/*Property = ScreenPixelsHeight*/
const Value V15819_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P15819_4 = {S54, 0, &V15819_4_1404, &P15819_5};	/*Property = ScreenPixelsWidth*/
const Profile P15819 = {"15819", &P15819_4};	/*Profile ID - 15819*/
const Value V13961_1_1 = {S2, NULL};	/*Value = True*/
const Property P13961_1 = {S50, 0, &V13961_1_1, NULL};	/*Property = IsMobile*/
const Value V13961_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13961_5 = {S52, 0, &V13961_5_1222, &P13961_1};	/*Property = ScreenPixelsHeight*/
const Value V13961_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13961_4 = {S54, 0, &V13961_4_1224, &P13961_5};	/*Property = ScreenPixelsWidth*/
const Profile P13961 = {"13961", &P13961_4};	/*Profile ID - 13961*/
const Value V17941_1_1 = {S2, NULL};	/*Value = True*/
const Property P17941_1 = {S50, 0, &V17941_1_1, NULL};	/*Property = IsMobile*/
const Value V17941_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17941_5 = {S52, 0, &V17941_5_1269, &P17941_1};	/*Property = ScreenPixelsHeight*/
const Value V17941_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17941_4 = {S54, 0, &V17941_4_1229, &P17941_5};	/*Property = ScreenPixelsWidth*/
const Profile P17941 = {"17941", &P17941_4};	/*Profile ID - 17941*/
const Value V13429_1_1 = {S2, NULL};	/*Value = True*/
const Property P13429_1 = {S50, 0, &V13429_1_1, NULL};	/*Property = IsMobile*/
const Value V13429_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P13429_5 = {S52, 0, &V13429_5_1269, &P13429_1};	/*Property = ScreenPixelsHeight*/
const Value V13429_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13429_4 = {S54, 0, &V13429_4_1229, &P13429_5};	/*Property = ScreenPixelsWidth*/
const Profile P13429 = {"13429", &P13429_4};	/*Profile ID - 13429*/
const Value V21676_1_1 = {S2, NULL};	/*Value = True*/
const Property P21676_1 = {S50, 0, &V21676_1_1, NULL};	/*Property = IsMobile*/
const Value V21676_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21676_5 = {S52, 0, &V21676_5_1227, &P21676_1};	/*Property = ScreenPixelsHeight*/
const Value V21676_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21676_4 = {S54, 0, &V21676_4_1229, &P21676_5};	/*Property = ScreenPixelsWidth*/
const Profile P21676 = {"21676", &P21676_4};	/*Profile ID - 21676*/
const Value V13432_1_1 = {S2, NULL};	/*Value = True*/
const Property P13432_1 = {S50, 0, &V13432_1_1, NULL};	/*Property = IsMobile*/
const Value V13432_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P13432_5 = {S52, 0, &V13432_5_1269, &P13432_1};	/*Property = ScreenPixelsHeight*/
const Value V13432_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13432_4 = {S54, 0, &V13432_4_1229, &P13432_5};	/*Property = ScreenPixelsWidth*/
const Profile P13432 = {"13432", &P13432_4};	/*Profile ID - 13432*/
const Value V13433_1_1 = {S2, NULL};	/*Value = True*/
const Property P13433_1 = {S50, 0, &V13433_1_1, NULL};	/*Property = IsMobile*/
const Value V13433_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13433_5 = {S52, 0, &V13433_5_1277, &P13433_1};	/*Property = ScreenPixelsHeight*/
const Value V13433_4_37610 = {S69, NULL};	/*Value = 854*/
const Property P13433_4 = {S54, 0, &V13433_4_37610, &P13433_5};	/*Property = ScreenPixelsWidth*/
const Profile P13433 = {"13433", &P13433_4};	/*Profile ID - 13433*/
const Value V13434_1_1 = {S2, NULL};	/*Value = True*/
const Property P13434_1 = {S50, 0, &V13434_1_1, NULL};	/*Property = IsMobile*/
const Value V13434_5_37534 = {S108, NULL};	/*Value = 864*/
const Property P13434_5 = {S52, 0, &V13434_5_37534, &P13434_1};	/*Property = ScreenPixelsHeight*/
const Value V13434_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13434_4 = {S54, 0, &V13434_4_1229, &P13434_5};	/*Property = ScreenPixelsWidth*/
const Profile P13434 = {"13434", &P13434_4};	/*Profile ID - 13434*/
const Value V13435_1_1 = {S2, NULL};	/*Value = True*/
const Property P13435_1 = {S50, 0, &V13435_1_1, NULL};	/*Property = IsMobile*/
const Value V13435_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P13435_5 = {S52, 0, &V13435_5_1269, &P13435_1};	/*Property = ScreenPixelsHeight*/
const Value V13435_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13435_4 = {S54, 0, &V13435_4_1229, &P13435_5};	/*Property = ScreenPixelsWidth*/
const Profile P13435 = {"13435", &P13435_4};	/*Profile ID - 13435*/
const Value V13444_1_1 = {S2, NULL};	/*Value = True*/
const Property P13444_1 = {S50, 0, &V13444_1_1, NULL};	/*Property = IsMobile*/
const Value V13444_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13444_5 = {S52, 0, &V13444_5_1294, &P13444_1};	/*Property = ScreenPixelsHeight*/
const Value V13444_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13444_4 = {S54, 0, &V13444_4_1296, &P13444_5};	/*Property = ScreenPixelsWidth*/
const Profile P13444 = {"13444", &P13444_4};	/*Profile ID - 13444*/
const Value V14599_1_1 = {S2, NULL};	/*Value = True*/
const Property P14599_1 = {S50, 0, &V14599_1_1, NULL};	/*Property = IsMobile*/
const Value V14599_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P14599_5 = {S52, 0, &V14599_5_1269, &P14599_1};	/*Property = ScreenPixelsHeight*/
const Value V14599_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14599_4 = {S54, 0, &V14599_4_1229, &P14599_5};	/*Property = ScreenPixelsWidth*/
const Profile P14599 = {"14599", &P14599_4};	/*Profile ID - 14599*/
const Value V14600_1_1 = {S2, NULL};	/*Value = True*/
const Property P14600_1 = {S50, 0, &V14600_1_1, NULL};	/*Property = IsMobile*/
const Value V14600_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14600_5 = {S52, 0, &V14600_5_1222, &P14600_1};	/*Property = ScreenPixelsHeight*/
const Value V14600_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14600_4 = {S54, 0, &V14600_4_1224, &P14600_5};	/*Property = ScreenPixelsWidth*/
const Profile P14600 = {"14600", &P14600_4};	/*Profile ID - 14600*/
const Profile P6830 = {"6830", NULL};	/*Profile ID - 6830*/
const Value V14662_1_1 = {S2, NULL};	/*Value = True*/
const Property P14662_1 = {S50, 0, &V14662_1_1, NULL};	/*Property = IsMobile*/
const Value V14662_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14662_5 = {S52, 0, &V14662_5_1222, &P14662_1};	/*Property = ScreenPixelsHeight*/
const Value V14662_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14662_4 = {S54, 0, &V14662_4_1224, &P14662_5};	/*Property = ScreenPixelsWidth*/
const Profile P14662 = {"14662", &P14662_4};	/*Profile ID - 14662*/
const Value V24644_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_279 = {S1, 0, &V24644_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24644_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_282 = {S3, 0, &V24644_282_57155, &P24644_279};	/*Property = BlobBuilder*/
const Value V24644_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_350 = {S4, 0, &V24644_350_57446, &P24644_282};	/*Property = Canvas*/
const Value V24644_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_283 = {S5, 0, &V24644_283_57161, &P24644_350};	/*Property = CssBackground*/
const Value V24644_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_284 = {S6, 0, &V24644_284_57164, &P24644_283};	/*Property = CssBorderImage*/
const Value V24644_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_285 = {S7, 0, &V24644_285_57167, &P24644_284};	/*Property = CssCanvas*/
const Value V24644_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_286 = {S8, 0, &V24644_286_57170, &P24644_285};	/*Property = CssColor*/
const Value V24644_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_237 = {S9, 0, &V24644_237_55786, &P24644_286};	/*Property = CssColumn*/
const Value V24644_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_287 = {S10, 0, &V24644_287_57173, &P24644_237};	/*Property = CssFlexbox*/
const Value V24644_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_288 = {S11, 0, &V24644_288_57176, &P24644_287};	/*Property = CssFont*/
const Value V24644_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_289 = {S12, 0, &V24644_289_86899, &P24644_288};	/*Property = CssImages*/
const Value V24644_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_291 = {S13, 0, &V24644_291_57182, &P24644_289};	/*Property = CssMediaQueries*/
const Value V24644_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_292 = {S14, 0, &V24644_292_57185, &P24644_291};	/*Property = CssMinMax*/
const Value V24644_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_293 = {S15, 0, &V24644_293_57188, &P24644_292};	/*Property = CssOverflow*/
const Value V24644_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_294 = {S16, 0, &V24644_294_57191, &P24644_293};	/*Property = CssPosition*/
const Value V24644_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_295 = {S17, 0, &V24644_295_57194, &P24644_294};	/*Property = CssText*/
const Value V24644_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_297 = {S18, 0, &V24644_297_57197, &P24644_295};	/*Property = CssTransforms*/
const Value V24644_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_241 = {S19, 0, &V24644_241_55801, &P24644_297};	/*Property = CssTransitions*/
const Value V24644_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_298 = {S20, 0, &V24644_298_57208, &P24644_241};	/*Property = CssUI*/
const Value V24644_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_300 = {S21, 0, &V24644_300_57217, &P24644_298};	/*Property = DataSet*/
const Value V24644_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_301 = {S22, 0, &V24644_301_57220, &P24644_300};	/*Property = DataUrl*/
const Value V24644_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_302 = {S23, 0, &V24644_302_57223, &P24644_301};	/*Property = DeviceOrientation*/
const Value V24644_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_304 = {S24, 0, &V24644_304_57229, &P24644_302};	/*Property = FileReader*/
const Value V24644_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_305 = {S25, 0, &V24644_305_57232, &P24644_304};	/*Property = FileSaver*/
const Value V24644_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_306 = {S26, 0, &V24644_306_57238, &P24644_305};	/*Property = FileWriter*/
const Value V24644_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_307 = {S27, 0, &V24644_307_57241, &P24644_306};	/*Property = FormData*/
const Value V24644_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_309 = {S28, 0, &V24644_309_57247, &P24644_307};	/*Property = Fullscreen*/
const Value V24644_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_310 = {S29, 0, &V24644_310_57250, &P24644_309};	/*Property = GeoLocation*/
const Value V24644_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_312 = {S30, 0, &V24644_312_57256, &P24644_310};	/*Property = History*/
const Value V24644_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_314 = {S31, 0, &V24644_314_57262, &P24644_312};	/*Property = Html5*/
const Value V24644_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_313 = {S32, 0, &V24644_313_57259, &P24644_314};	/*Property = Html-Media-Capture*/
const Value V24644_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_315 = {S33, 0, &V24644_315_57265, &P24644_313};	/*Property = Iframe*/
const Value V24644_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_316 = {S34, 0, &V24644_316_57268, &P24644_315};	/*Property = IndexedDB*/
const Value V24644_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_317 = {S35, 0, &V24644_317_57271, &P24644_316};	/*Property = Json*/
const Value V24644_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24644_15 = {S37, 0, &V24644_15_90837, &P24644_317};	/*Property = LayoutEngine*/
const Value V24644_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_344 = {S38, 0, &V24644_344_57274, &P24644_15};	/*Property = Masking*/
const Value V24644_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_320 = {S39, 0, &V24644_320_57292, &P24644_344};	/*Property = PostMessage*/
const Value V24644_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_321 = {S40, 0, &V24644_321_57295, &P24644_320};	/*Property = Progress*/
const Value V24644_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_322 = {S41, 0, &V24644_322_57298, &P24644_321};	/*Property = Prompts*/
const Value V24644_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_323 = {S42, 0, &V24644_323_57301, &P24644_322};	/*Property = Selector*/
const Value V24644_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_324 = {S43, 0, &V24644_324_57307, &P24644_323};	/*Property = Svg*/
const Value V24644_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_327 = {S44, 0, &V24644_327_57316, &P24644_324};	/*Property = TouchEvents*/
const Value V24644_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_328 = {S45, 0, &V24644_328_57319, &P24644_327};	/*Property = Track*/
const Value V24644_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_329 = {S46, 0, &V24644_329_57325, &P24644_328};	/*Property = Video*/
const Value V24644_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_330 = {S47, 0, &V24644_330_57328, &P24644_329};	/*Property = Viewport*/
const Value V24644_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_332 = {S48, 0, &V24644_332_57340, &P24644_330};	/*Property = WebWorkers*/
const Value V24644_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24644_333 = {S49, 0, &V24644_333_57343, &P24644_332};	/*Property = Xhr2*/
const Profile P24644 = {"24644", &P24644_333};	/*Profile ID - 24644*/
const Value V16193_1_1 = {S2, NULL};	/*Value = True*/
const Property P16193_1 = {S50, 0, &V16193_1_1, NULL};	/*Property = IsMobile*/
const Value V16193_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P16193_5 = {S52, 0, &V16193_5_1275, &P16193_1};	/*Property = ScreenPixelsHeight*/
const Value V16193_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16193_4 = {S54, 0, &V16193_4_1229, &P16193_5};	/*Property = ScreenPixelsWidth*/
const Profile P16193 = {"16193", &P16193_4};	/*Profile ID - 16193*/
const Value V17544_1_1 = {S2, NULL};	/*Value = True*/
const Property P17544_1 = {S50, 0, &V17544_1_1, NULL};	/*Property = IsMobile*/
const Value V17544_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17544_5 = {S52, 0, &V17544_5_1222, &P17544_1};	/*Property = ScreenPixelsHeight*/
const Value V17544_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17544_4 = {S54, 0, &V17544_4_1224, &P17544_5};	/*Property = ScreenPixelsWidth*/
const Profile P17544 = {"17544", &P17544_4};	/*Profile ID - 17544*/
const Value V13967_1_1 = {S2, NULL};	/*Value = True*/
const Property P13967_1 = {S50, 0, &V13967_1_1, NULL};	/*Property = IsMobile*/
const Value V13967_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13967_5 = {S52, 0, &V13967_5_1222, &P13967_1};	/*Property = ScreenPixelsHeight*/
const Value V13967_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13967_4 = {S54, 0, &V13967_4_1224, &P13967_5};	/*Property = ScreenPixelsWidth*/
const Profile P13967 = {"13967", &P13967_4};	/*Profile ID - 13967*/
const Value V13970_1_1 = {S2, NULL};	/*Value = True*/
const Property P13970_1 = {S50, 0, &V13970_1_1, NULL};	/*Property = IsMobile*/
const Value V13970_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P13970_5 = {S52, 0, &V13970_5_1269, &P13970_1};	/*Property = ScreenPixelsHeight*/
const Value V13970_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13970_4 = {S54, 0, &V13970_4_1229, &P13970_5};	/*Property = ScreenPixelsWidth*/
const Profile P13970 = {"13970", &P13970_4};	/*Profile ID - 13970*/
const Value V13969_1_1 = {S2, NULL};	/*Value = True*/
const Property P13969_1 = {S50, 0, &V13969_1_1, NULL};	/*Property = IsMobile*/
const Value V13969_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13969_5 = {S52, 0, &V13969_5_1222, &P13969_1};	/*Property = ScreenPixelsHeight*/
const Value V13969_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13969_4 = {S54, 0, &V13969_4_1224, &P13969_5};	/*Property = ScreenPixelsWidth*/
const Profile P13969 = {"13969", &P13969_4};	/*Profile ID - 13969*/
const Value V13975_1_1 = {S2, NULL};	/*Value = True*/
const Property P13975_1 = {S50, 0, &V13975_1_1, NULL};	/*Property = IsMobile*/
const Value V13975_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13975_5 = {S52, 0, &V13975_5_1222, &P13975_1};	/*Property = ScreenPixelsHeight*/
const Value V13975_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13975_4 = {S54, 0, &V13975_4_1224, &P13975_5};	/*Property = ScreenPixelsWidth*/
const Profile P13975 = {"13975", &P13975_4};	/*Profile ID - 13975*/
const Value V13976_1_1 = {S2, NULL};	/*Value = True*/
const Property P13976_1 = {S50, 0, &V13976_1_1, NULL};	/*Property = IsMobile*/
const Value V13976_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P13976_5 = {S52, 0, &V13976_5_1304, &P13976_1};	/*Property = ScreenPixelsHeight*/
const Value V13976_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13976_4 = {S54, 0, &V13976_4_1224, &P13976_5};	/*Property = ScreenPixelsWidth*/
const Profile P13976 = {"13976", &P13976_4};	/*Profile ID - 13976*/
const Value V13980_1_1 = {S2, NULL};	/*Value = True*/
const Property P13980_1 = {S50, 0, &V13980_1_1, NULL};	/*Property = IsMobile*/
const Value V13980_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13980_5 = {S52, 0, &V13980_5_1233, &P13980_1};	/*Property = ScreenPixelsHeight*/
const Value V13980_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13980_4 = {S54, 0, &V13980_4_1235, &P13980_5};	/*Property = ScreenPixelsWidth*/
const Profile P13980 = {"13980", &P13980_4};	/*Profile ID - 13980*/
const Value V13982_1_1 = {S2, NULL};	/*Value = True*/
const Property P13982_1 = {S50, 0, &V13982_1_1, NULL};	/*Property = IsMobile*/
const Value V13982_5_49771 = {S109, NULL};	/*Value = 276*/
const Property P13982_5 = {S52, 0, &V13982_5_49771, &P13982_1};	/*Property = ScreenPixelsHeight*/
const Value V13982_4_37586 = {S110, NULL};	/*Value = 172*/
const Property P13982_4 = {S54, 0, &V13982_4_37586, &P13982_5};	/*Property = ScreenPixelsWidth*/
const Profile P13982 = {"13982", &P13982_4};	/*Profile ID - 13982*/
const Value V13983_1_1 = {S2, NULL};	/*Value = True*/
const Property P13983_1 = {S50, 0, &V13983_1_1, NULL};	/*Property = IsMobile*/
const Value V13983_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13983_5 = {S52, 0, &V13983_5_1294, &P13983_1};	/*Property = ScreenPixelsHeight*/
const Value V13983_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13983_4 = {S54, 0, &V13983_4_1296, &P13983_5};	/*Property = ScreenPixelsWidth*/
const Profile P13983 = {"13983", &P13983_4};	/*Profile ID - 13983*/
const Value V13984_1_1 = {S2, NULL};	/*Value = True*/
const Property P13984_1 = {S50, 0, &V13984_1_1, NULL};	/*Property = IsMobile*/
const Value V13984_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13984_5 = {S52, 0, &V13984_5_1222, &P13984_1};	/*Property = ScreenPixelsHeight*/
const Value V13984_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13984_4 = {S54, 0, &V13984_4_1224, &P13984_5};	/*Property = ScreenPixelsWidth*/
const Profile P13984 = {"13984", &P13984_4};	/*Profile ID - 13984*/
const Value V13985_1_1 = {S2, NULL};	/*Value = True*/
const Property P13985_1 = {S50, 0, &V13985_1_1, NULL};	/*Property = IsMobile*/
const Value V13985_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13985_5 = {S52, 0, &V13985_5_1222, &P13985_1};	/*Property = ScreenPixelsHeight*/
const Value V13985_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13985_4 = {S54, 0, &V13985_4_1224, &P13985_5};	/*Property = ScreenPixelsWidth*/
const Profile P13985 = {"13985", &P13985_4};	/*Profile ID - 13985*/
const Value V17919_1_1 = {S2, NULL};	/*Value = True*/
const Property P17919_1 = {S50, 0, &V17919_1_1, NULL};	/*Property = IsMobile*/
const Value V17919_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17919_5 = {S52, 0, &V17919_5_1222, &P17919_1};	/*Property = ScreenPixelsHeight*/
const Value V17919_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17919_4 = {S54, 0, &V17919_4_1224, &P17919_5};	/*Property = ScreenPixelsWidth*/
const Profile P17919 = {"17919", &P17919_4};	/*Profile ID - 17919*/
const Value V13988_1_1 = {S2, NULL};	/*Value = True*/
const Property P13988_1 = {S50, 0, &V13988_1_1, NULL};	/*Property = IsMobile*/
const Value V13988_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13988_5 = {S52, 0, &V13988_5_1227, &P13988_1};	/*Property = ScreenPixelsHeight*/
const Value V13988_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13988_4 = {S54, 0, &V13988_4_1229, &P13988_5};	/*Property = ScreenPixelsWidth*/
const Profile P13988 = {"13988", &P13988_4};	/*Profile ID - 13988*/
const Value V13990_1_1 = {S2, NULL};	/*Value = True*/
const Property P13990_1 = {S50, 0, &V13990_1_1, NULL};	/*Property = IsMobile*/
const Value V13990_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13990_5 = {S52, 0, &V13990_5_1222, &P13990_1};	/*Property = ScreenPixelsHeight*/
const Value V13990_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13990_4 = {S54, 0, &V13990_4_1224, &P13990_5};	/*Property = ScreenPixelsWidth*/
const Profile P13990 = {"13990", &P13990_4};	/*Profile ID - 13990*/
const Value V13992_1_1 = {S2, NULL};	/*Value = True*/
const Property P13992_1 = {S50, 0, &V13992_1_1, NULL};	/*Property = IsMobile*/
const Value V13992_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P13992_5 = {S52, 0, &V13992_5_1294, &P13992_1};	/*Property = ScreenPixelsHeight*/
const Value V13992_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13992_4 = {S54, 0, &V13992_4_1296, &P13992_5};	/*Property = ScreenPixelsWidth*/
const Profile P13992 = {"13992", &P13992_4};	/*Profile ID - 13992*/
const Value V13995_1_1 = {S2, NULL};	/*Value = True*/
const Property P13995_1 = {S50, 0, &V13995_1_1, NULL};	/*Property = IsMobile*/
const Value V13995_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13995_5 = {S52, 0, &V13995_5_1233, &P13995_1};	/*Property = ScreenPixelsHeight*/
const Value V13995_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13995_4 = {S54, 0, &V13995_4_1235, &P13995_5};	/*Property = ScreenPixelsWidth*/
const Profile P13995 = {"13995", &P13995_4};	/*Profile ID - 13995*/
const Value V12038_1_1 = {S2, NULL};	/*Value = True*/
const Property P12038_1 = {S50, 0, &V12038_1_1, NULL};	/*Property = IsMobile*/
const Value V12038_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12038_5 = {S52, 0, &V12038_5_1222, &P12038_1};	/*Property = ScreenPixelsHeight*/
const Value V12038_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12038_4 = {S54, 0, &V12038_4_1224, &P12038_5};	/*Property = ScreenPixelsWidth*/
const Profile P12038 = {"12038", &P12038_4};	/*Profile ID - 12038*/
const Value V13994_1_1 = {S2, NULL};	/*Value = True*/
const Property P13994_1 = {S50, 0, &V13994_1_1, NULL};	/*Property = IsMobile*/
const Value V13994_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13994_5 = {S52, 0, &V13994_5_1222, &P13994_1};	/*Property = ScreenPixelsHeight*/
const Value V13994_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13994_4 = {S54, 0, &V13994_4_1224, &P13994_5};	/*Property = ScreenPixelsWidth*/
const Profile P13994 = {"13994", &P13994_4};	/*Profile ID - 13994*/
const Value V13997_1_1 = {S2, NULL};	/*Value = True*/
const Property P13997_1 = {S50, 0, &V13997_1_1, NULL};	/*Property = IsMobile*/
const Value V13997_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13997_5 = {S52, 0, &V13997_5_1222, &P13997_1};	/*Property = ScreenPixelsHeight*/
const Value V13997_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13997_4 = {S54, 0, &V13997_4_1224, &P13997_5};	/*Property = ScreenPixelsWidth*/
const Profile P13997 = {"13997", &P13997_4};	/*Profile ID - 13997*/
const Value V13642_1_1 = {S2, NULL};	/*Value = True*/
const Property P13642_1 = {S50, 0, &V13642_1_1, NULL};	/*Property = IsMobile*/
const Value V13642_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13642_5 = {S52, 0, &V13642_5_1233, &P13642_1};	/*Property = ScreenPixelsHeight*/
const Value V13642_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13642_4 = {S54, 0, &V13642_4_1235, &P13642_5};	/*Property = ScreenPixelsWidth*/
const Profile P13642 = {"13642", &P13642_4};	/*Profile ID - 13642*/
const Value V14001_1_1 = {S2, NULL};	/*Value = True*/
const Property P14001_1 = {S50, 0, &V14001_1_1, NULL};	/*Property = IsMobile*/
const Value V14001_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14001_5 = {S52, 0, &V14001_5_1293, &P14001_1};	/*Property = ScreenPixelsHeight*/
const Value V14001_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14001_4 = {S54, 0, &V14001_4_1296, &P14001_5};	/*Property = ScreenPixelsWidth*/
const Profile P14001 = {"14001", &P14001_4};	/*Profile ID - 14001*/
const Value V14004_1_1 = {S2, NULL};	/*Value = True*/
const Property P14004_1 = {S50, 0, &V14004_1_1, NULL};	/*Property = IsMobile*/
const Value V14004_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14004_5 = {S52, 0, &V14004_5_1222, &P14004_1};	/*Property = ScreenPixelsHeight*/
const Value V14004_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14004_4 = {S54, 0, &V14004_4_1224, &P14004_5};	/*Property = ScreenPixelsWidth*/
const Profile P14004 = {"14004", &P14004_4};	/*Profile ID - 14004*/
const Value V17404_1_1 = {S2, NULL};	/*Value = True*/
const Property P17404_1 = {S50, 0, &V17404_1_1, NULL};	/*Property = IsMobile*/
const Value V17404_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17404_5 = {S52, 0, &V17404_5_1293, &P17404_1};	/*Property = ScreenPixelsHeight*/
const Value V17404_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17404_4 = {S54, 0, &V17404_4_1296, &P17404_5};	/*Property = ScreenPixelsWidth*/
const Profile P17404 = {"17404", &P17404_4};	/*Profile ID - 17404*/
const Value V14005_1_1 = {S2, NULL};	/*Value = True*/
const Property P14005_1 = {S50, 0, &V14005_1_1, NULL};	/*Property = IsMobile*/
const Value V14005_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14005_5 = {S52, 0, &V14005_5_1222, &P14005_1};	/*Property = ScreenPixelsHeight*/
const Value V14005_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14005_4 = {S54, 0, &V14005_4_1224, &P14005_5};	/*Property = ScreenPixelsWidth*/
const Profile P14005 = {"14005", &P14005_4};	/*Profile ID - 14005*/
const Value V14070_1_1 = {S2, NULL};	/*Value = True*/
const Property P14070_1 = {S50, 0, &V14070_1_1, NULL};	/*Property = IsMobile*/
const Value V14070_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14070_5 = {S52, 0, &V14070_5_1222, &P14070_1};	/*Property = ScreenPixelsHeight*/
const Value V14070_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14070_4 = {S54, 0, &V14070_4_1224, &P14070_5};	/*Property = ScreenPixelsWidth*/
const Profile P14070 = {"14070", &P14070_4};	/*Profile ID - 14070*/
const Value V14008_1_1 = {S2, NULL};	/*Value = True*/
const Property P14008_1 = {S50, 0, &V14008_1_1, NULL};	/*Property = IsMobile*/
const Value V14008_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14008_5 = {S52, 0, &V14008_5_1222, &P14008_1};	/*Property = ScreenPixelsHeight*/
const Value V14008_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14008_4 = {S54, 0, &V14008_4_1224, &P14008_5};	/*Property = ScreenPixelsWidth*/
const Profile P14008 = {"14008", &P14008_4};	/*Profile ID - 14008*/
const Value V14015_1_1 = {S2, NULL};	/*Value = True*/
const Property P14015_1 = {S50, 0, &V14015_1_1, NULL};	/*Property = IsMobile*/
const Value V14015_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14015_5 = {S52, 0, &V14015_5_1233, &P14015_1};	/*Property = ScreenPixelsHeight*/
const Value V14015_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14015_4 = {S54, 0, &V14015_4_1235, &P14015_5};	/*Property = ScreenPixelsWidth*/
const Profile P14015 = {"14015", &P14015_4};	/*Profile ID - 14015*/
const Value V14023_1_1 = {S2, NULL};	/*Value = True*/
const Property P14023_1 = {S50, 0, &V14023_1_1, NULL};	/*Property = IsMobile*/
const Value V14023_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14023_5 = {S52, 0, &V14023_5_1233, &P14023_1};	/*Property = ScreenPixelsHeight*/
const Value V14023_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14023_4 = {S54, 0, &V14023_4_1235, &P14023_5};	/*Property = ScreenPixelsWidth*/
const Profile P14023 = {"14023", &P14023_4};	/*Profile ID - 14023*/
const Value V14024_1_1 = {S2, NULL};	/*Value = True*/
const Property P14024_1 = {S50, 0, &V14024_1_1, NULL};	/*Property = IsMobile*/
const Value V14024_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14024_5 = {S52, 0, &V14024_5_1277, &P14024_1};	/*Property = ScreenPixelsHeight*/
const Value V14024_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P14024_4 = {S54, 0, &V14024_4_2336, &P14024_5};	/*Property = ScreenPixelsWidth*/
const Profile P14024 = {"14024", &P14024_4};	/*Profile ID - 14024*/
const Value V14035_1_1 = {S2, NULL};	/*Value = True*/
const Property P14035_1 = {S50, 0, &V14035_1_1, NULL};	/*Property = IsMobile*/
const Value V14035_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14035_5 = {S52, 0, &V14035_5_1380, &P14035_1};	/*Property = ScreenPixelsHeight*/
const Value V14035_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14035_4 = {S54, 0, &V14035_4_1279, &P14035_5};	/*Property = ScreenPixelsWidth*/
const Profile P14035 = {"14035", &P14035_4};	/*Profile ID - 14035*/
const Value V14040_1_1 = {S2, NULL};	/*Value = True*/
const Property P14040_1 = {S50, 0, &V14040_1_1, NULL};	/*Property = IsMobile*/
const Value V14040_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14040_5 = {S52, 0, &V14040_5_1380, &P14040_1};	/*Property = ScreenPixelsHeight*/
const Value V14040_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14040_4 = {S54, 0, &V14040_4_1279, &P14040_5};	/*Property = ScreenPixelsWidth*/
const Profile P14040 = {"14040", &P14040_4};	/*Profile ID - 14040*/
const Value V14043_1_1 = {S2, NULL};	/*Value = True*/
const Property P14043_1 = {S50, 0, &V14043_1_1, NULL};	/*Property = IsMobile*/
const Value V14043_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14043_5 = {S52, 0, &V14043_5_1233, &P14043_1};	/*Property = ScreenPixelsHeight*/
const Value V14043_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14043_4 = {S54, 0, &V14043_4_1235, &P14043_5};	/*Property = ScreenPixelsWidth*/
const Profile P14043 = {"14043", &P14043_4};	/*Profile ID - 14043*/
const Value V14777_1_1 = {S2, NULL};	/*Value = True*/
const Property P14777_1 = {S50, 0, &V14777_1_1, NULL};	/*Property = IsMobile*/
const Value V14777_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14777_5 = {S52, 0, &V14777_5_1304, &P14777_1};	/*Property = ScreenPixelsHeight*/
const Value V14777_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14777_4 = {S54, 0, &V14777_4_1224, &P14777_5};	/*Property = ScreenPixelsWidth*/
const Profile P14777 = {"14777", &P14777_4};	/*Profile ID - 14777*/
const Value V12703_1_1 = {S2, NULL};	/*Value = True*/
const Property P12703_1 = {S50, 0, &V12703_1_1, NULL};	/*Property = IsMobile*/
const Value V12703_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12703_5 = {S52, 0, &V12703_5_1222, &P12703_1};	/*Property = ScreenPixelsHeight*/
const Value V12703_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12703_4 = {S54, 0, &V12703_4_1224, &P12703_5};	/*Property = ScreenPixelsWidth*/
const Profile P12703 = {"12703", &P12703_4};	/*Profile ID - 12703*/
const Value V14045_1_1 = {S2, NULL};	/*Value = True*/
const Property P14045_1 = {S50, 0, &V14045_1_1, NULL};	/*Property = IsMobile*/
const Value V14045_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14045_5 = {S52, 0, &V14045_5_1222, &P14045_1};	/*Property = ScreenPixelsHeight*/
const Value V14045_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14045_4 = {S54, 0, &V14045_4_1224, &P14045_5};	/*Property = ScreenPixelsWidth*/
const Profile P14045 = {"14045", &P14045_4};	/*Profile ID - 14045*/
const Value V14048_1_1 = {S2, NULL};	/*Value = True*/
const Property P14048_1 = {S50, 0, &V14048_1_1, NULL};	/*Property = IsMobile*/
const Value V14048_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14048_5 = {S52, 0, &V14048_5_1293, &P14048_1};	/*Property = ScreenPixelsHeight*/
const Value V14048_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14048_4 = {S54, 0, &V14048_4_1296, &P14048_5};	/*Property = ScreenPixelsWidth*/
const Profile P14048 = {"14048", &P14048_4};	/*Profile ID - 14048*/
const Value V14049_1_1 = {S2, NULL};	/*Value = True*/
const Property P14049_1 = {S50, 0, &V14049_1_1, NULL};	/*Property = IsMobile*/
const Value V14049_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14049_5 = {S52, 0, &V14049_5_1222, &P14049_1};	/*Property = ScreenPixelsHeight*/
const Value V14049_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14049_4 = {S54, 0, &V14049_4_1224, &P14049_5};	/*Property = ScreenPixelsWidth*/
const Profile P14049 = {"14049", &P14049_4};	/*Profile ID - 14049*/
const Value V14050_1_1 = {S2, NULL};	/*Value = True*/
const Property P14050_1 = {S50, 0, &V14050_1_1, NULL};	/*Property = IsMobile*/
const Value V14050_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14050_5 = {S52, 0, &V14050_5_1294, &P14050_1};	/*Property = ScreenPixelsHeight*/
const Value V14050_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14050_4 = {S54, 0, &V14050_4_1296, &P14050_5};	/*Property = ScreenPixelsWidth*/
const Profile P14050 = {"14050", &P14050_4};	/*Profile ID - 14050*/
const Value V14051_1_1 = {S2, NULL};	/*Value = True*/
const Property P14051_1 = {S50, 0, &V14051_1_1, NULL};	/*Property = IsMobile*/
const Value V14051_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14051_5 = {S52, 0, &V14051_5_1222, &P14051_1};	/*Property = ScreenPixelsHeight*/
const Value V14051_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14051_4 = {S54, 0, &V14051_4_1224, &P14051_5};	/*Property = ScreenPixelsWidth*/
const Profile P14051 = {"14051", &P14051_4};	/*Profile ID - 14051*/
const Value V14061_1_1 = {S2, NULL};	/*Value = True*/
const Property P14061_1 = {S50, 0, &V14061_1_1, NULL};	/*Property = IsMobile*/
const Value V14061_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14061_5 = {S52, 0, &V14061_5_1233, &P14061_1};	/*Property = ScreenPixelsHeight*/
const Value V14061_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14061_4 = {S54, 0, &V14061_4_1235, &P14061_5};	/*Property = ScreenPixelsWidth*/
const Profile P14061 = {"14061", &P14061_4};	/*Profile ID - 14061*/
const Value V20211_1_1 = {S2, NULL};	/*Value = True*/
const Property P20211_1 = {S50, 0, &V20211_1_1, NULL};	/*Property = IsMobile*/
const Value V20211_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20211_5 = {S52, 0, &V20211_5_1222, &P20211_1};	/*Property = ScreenPixelsHeight*/
const Value V20211_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20211_4 = {S54, 0, &V20211_4_1224, &P20211_5};	/*Property = ScreenPixelsWidth*/
const Profile P20211 = {"20211", &P20211_4};	/*Profile ID - 20211*/
const Value V14067_1_1 = {S2, NULL};	/*Value = True*/
const Property P14067_1 = {S50, 0, &V14067_1_1, NULL};	/*Property = IsMobile*/
const Value V14067_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14067_5 = {S52, 0, &V14067_5_1222, &P14067_1};	/*Property = ScreenPixelsHeight*/
const Value V14067_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14067_4 = {S54, 0, &V14067_4_1224, &P14067_5};	/*Property = ScreenPixelsWidth*/
const Profile P14067 = {"14067", &P14067_4};	/*Profile ID - 14067*/
const Value V14069_1_1 = {S2, NULL};	/*Value = True*/
const Property P14069_1 = {S50, 0, &V14069_1_1, NULL};	/*Property = IsMobile*/
const Value V14069_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14069_5 = {S52, 0, &V14069_5_1222, &P14069_1};	/*Property = ScreenPixelsHeight*/
const Value V14069_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14069_4 = {S54, 0, &V14069_4_1224, &P14069_5};	/*Property = ScreenPixelsWidth*/
const Profile P14069 = {"14069", &P14069_4};	/*Profile ID - 14069*/
const Value V14073_1_1 = {S2, NULL};	/*Value = True*/
const Property P14073_1 = {S50, 0, &V14073_1_1, NULL};	/*Property = IsMobile*/
const Value V14073_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14073_5 = {S52, 0, &V14073_5_1294, &P14073_1};	/*Property = ScreenPixelsHeight*/
const Value V14073_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14073_4 = {S54, 0, &V14073_4_1296, &P14073_5};	/*Property = ScreenPixelsWidth*/
const Profile P14073 = {"14073", &P14073_4};	/*Profile ID - 14073*/
const Value V14077_1_1 = {S2, NULL};	/*Value = True*/
const Property P14077_1 = {S50, 0, &V14077_1_1, NULL};	/*Property = IsMobile*/
const Value V14077_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14077_5 = {S52, 0, &V14077_5_1304, &P14077_1};	/*Property = ScreenPixelsHeight*/
const Value V14077_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14077_4 = {S54, 0, &V14077_4_1224, &P14077_5};	/*Property = ScreenPixelsWidth*/
const Profile P14077 = {"14077", &P14077_4};	/*Profile ID - 14077*/
const Value V14082_1_1 = {S2, NULL};	/*Value = True*/
const Property P14082_1 = {S50, 0, &V14082_1_1, NULL};	/*Property = IsMobile*/
const Value V14082_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14082_5 = {S52, 0, &V14082_5_1304, &P14082_1};	/*Property = ScreenPixelsHeight*/
const Value V14082_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14082_4 = {S54, 0, &V14082_4_1224, &P14082_5};	/*Property = ScreenPixelsWidth*/
const Profile P14082 = {"14082", &P14082_4};	/*Profile ID - 14082*/
const Value V17906_1_1 = {S2, NULL};	/*Value = True*/
const Property P17906_1 = {S50, 0, &V17906_1_1, NULL};	/*Property = IsMobile*/
const Value V17906_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17906_5 = {S52, 0, &V17906_5_1222, &P17906_1};	/*Property = ScreenPixelsHeight*/
const Value V17906_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17906_4 = {S54, 0, &V17906_4_1224, &P17906_5};	/*Property = ScreenPixelsWidth*/
const Profile P17906 = {"17906", &P17906_4};	/*Profile ID - 17906*/
const Value V14092_1_1 = {S2, NULL};	/*Value = True*/
const Property P14092_1 = {S50, 0, &V14092_1_1, NULL};	/*Property = IsMobile*/
const Value V14092_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14092_5 = {S52, 0, &V14092_5_1294, &P14092_1};	/*Property = ScreenPixelsHeight*/
const Value V14092_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14092_4 = {S54, 0, &V14092_4_1296, &P14092_5};	/*Property = ScreenPixelsWidth*/
const Profile P14092 = {"14092", &P14092_4};	/*Profile ID - 14092*/
const Value V14095_1_1 = {S2, NULL};	/*Value = True*/
const Property P14095_1 = {S50, 0, &V14095_1_1, NULL};	/*Property = IsMobile*/
const Value V14095_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14095_5 = {S52, 0, &V14095_5_1233, &P14095_1};	/*Property = ScreenPixelsHeight*/
const Value V14095_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14095_4 = {S54, 0, &V14095_4_1235, &P14095_5};	/*Property = ScreenPixelsWidth*/
const Profile P14095 = {"14095", &P14095_4};	/*Profile ID - 14095*/
const Value V14100_1_1 = {S2, NULL};	/*Value = True*/
const Property P14100_1 = {S50, 0, &V14100_1_1, NULL};	/*Property = IsMobile*/
const Value V14100_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14100_5 = {S52, 0, &V14100_5_1380, &P14100_1};	/*Property = ScreenPixelsHeight*/
const Value V14100_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14100_4 = {S54, 0, &V14100_4_1279, &P14100_5};	/*Property = ScreenPixelsWidth*/
const Profile P14100 = {"14100", &P14100_4};	/*Profile ID - 14100*/
const Value V14102_1_1 = {S2, NULL};	/*Value = True*/
const Property P14102_1 = {S50, 0, &V14102_1_1, NULL};	/*Property = IsMobile*/
const Value V14102_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14102_5 = {S52, 0, &V14102_5_1222, &P14102_1};	/*Property = ScreenPixelsHeight*/
const Value V14102_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14102_4 = {S54, 0, &V14102_4_1224, &P14102_5};	/*Property = ScreenPixelsWidth*/
const Profile P14102 = {"14102", &P14102_4};	/*Profile ID - 14102*/
const Value V14108_1_1 = {S2, NULL};	/*Value = True*/
const Property P14108_1 = {S50, 0, &V14108_1_1, NULL};	/*Property = IsMobile*/
const Value V14108_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14108_5 = {S52, 0, &V14108_5_1294, &P14108_1};	/*Property = ScreenPixelsHeight*/
const Value V14108_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14108_4 = {S54, 0, &V14108_4_1296, &P14108_5};	/*Property = ScreenPixelsWidth*/
const Profile P14108 = {"14108", &P14108_4};	/*Profile ID - 14108*/
const Value V14109_1_1 = {S2, NULL};	/*Value = True*/
const Property P14109_1 = {S50, 0, &V14109_1_1, NULL};	/*Property = IsMobile*/
const Value V14109_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P14109_5 = {S52, 0, &V14109_5_37, &P14109_1};	/*Property = ScreenPixelsHeight*/
const Value V14109_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P14109_4 = {S54, 0, &V14109_4_36, &P14109_5};	/*Property = ScreenPixelsWidth*/
const Profile P14109 = {"14109", &P14109_4};	/*Profile ID - 14109*/
const Value V14113_1_1 = {S2, NULL};	/*Value = True*/
const Property P14113_1 = {S50, 0, &V14113_1_1, NULL};	/*Property = IsMobile*/
const Value V14113_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14113_5 = {S52, 0, &V14113_5_1294, &P14113_1};	/*Property = ScreenPixelsHeight*/
const Value V14113_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14113_4 = {S54, 0, &V14113_4_1296, &P14113_5};	/*Property = ScreenPixelsWidth*/
const Profile P14113 = {"14113", &P14113_4};	/*Profile ID - 14113*/
const Value V14120_1_1 = {S2, NULL};	/*Value = True*/
const Property P14120_1 = {S50, 0, &V14120_1_1, NULL};	/*Property = IsMobile*/
const Value V14120_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14120_5 = {S52, 0, &V14120_5_1304, &P14120_1};	/*Property = ScreenPixelsHeight*/
const Value V14120_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14120_4 = {S54, 0, &V14120_4_1224, &P14120_5};	/*Property = ScreenPixelsWidth*/
const Profile P14120 = {"14120", &P14120_4};	/*Profile ID - 14120*/
const Value V14122_1_1 = {S2, NULL};	/*Value = True*/
const Property P14122_1 = {S50, 0, &V14122_1_1, NULL};	/*Property = IsMobile*/
const Value V14122_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14122_5 = {S52, 0, &V14122_5_1233, &P14122_1};	/*Property = ScreenPixelsHeight*/
const Value V14122_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14122_4 = {S54, 0, &V14122_4_1235, &P14122_5};	/*Property = ScreenPixelsWidth*/
const Profile P14122 = {"14122", &P14122_4};	/*Profile ID - 14122*/
const Value V14126_1_1 = {S2, NULL};	/*Value = True*/
const Property P14126_1 = {S50, 0, &V14126_1_1, NULL};	/*Property = IsMobile*/
const Value V14126_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14126_5 = {S52, 0, &V14126_5_1293, &P14126_1};	/*Property = ScreenPixelsHeight*/
const Value V14126_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14126_4 = {S54, 0, &V14126_4_1296, &P14126_5};	/*Property = ScreenPixelsWidth*/
const Profile P14126 = {"14126", &P14126_4};	/*Profile ID - 14126*/
const Value V14127_1_1 = {S2, NULL};	/*Value = True*/
const Property P14127_1 = {S50, 0, &V14127_1_1, NULL};	/*Property = IsMobile*/
const Value V14127_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14127_5 = {S52, 0, &V14127_5_1222, &P14127_1};	/*Property = ScreenPixelsHeight*/
const Value V14127_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14127_4 = {S54, 0, &V14127_4_1224, &P14127_5};	/*Property = ScreenPixelsWidth*/
const Profile P14127 = {"14127", &P14127_4};	/*Profile ID - 14127*/
const Value V14129_1_1 = {S2, NULL};	/*Value = True*/
const Property P14129_1 = {S50, 0, &V14129_1_1, NULL};	/*Property = IsMobile*/
const Value V14129_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14129_5 = {S52, 0, &V14129_5_1222, &P14129_1};	/*Property = ScreenPixelsHeight*/
const Value V14129_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14129_4 = {S54, 0, &V14129_4_1224, &P14129_5};	/*Property = ScreenPixelsWidth*/
const Profile P14129 = {"14129", &P14129_4};	/*Profile ID - 14129*/
const Value V14130_1_1 = {S2, NULL};	/*Value = True*/
const Property P14130_1 = {S50, 0, &V14130_1_1, NULL};	/*Property = IsMobile*/
const Value V14130_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14130_5 = {S52, 0, &V14130_5_1222, &P14130_1};	/*Property = ScreenPixelsHeight*/
const Value V14130_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14130_4 = {S54, 0, &V14130_4_1224, &P14130_5};	/*Property = ScreenPixelsWidth*/
const Profile P14130 = {"14130", &P14130_4};	/*Profile ID - 14130*/
const Value V14133_1_1 = {S2, NULL};	/*Value = True*/
const Property P14133_1 = {S50, 0, &V14133_1_1, NULL};	/*Property = IsMobile*/
const Value V14133_5_37465 = {S96, NULL};	/*Value = 80*/
const Property P14133_5 = {S52, 0, &V14133_5_37465, &P14133_1};	/*Property = ScreenPixelsHeight*/
const Value V14133_4_37576 = {S97, NULL};	/*Value = 101*/
const Property P14133_4 = {S54, 0, &V14133_4_37576, &P14133_5};	/*Property = ScreenPixelsWidth*/
const Profile P14133 = {"14133", &P14133_4};	/*Profile ID - 14133*/
const Value V14142_1_1 = {S2, NULL};	/*Value = True*/
const Property P14142_1 = {S50, 0, &V14142_1_1, NULL};	/*Property = IsMobile*/
const Value V14142_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14142_5 = {S52, 0, &V14142_5_1294, &P14142_1};	/*Property = ScreenPixelsHeight*/
const Value V14142_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14142_4 = {S54, 0, &V14142_4_1296, &P14142_5};	/*Property = ScreenPixelsWidth*/
const Profile P14142 = {"14142", &P14142_4};	/*Profile ID - 14142*/
const Value V14145_1_1 = {S2, NULL};	/*Value = True*/
const Property P14145_1 = {S50, 0, &V14145_1_1, NULL};	/*Property = IsMobile*/
const Value V14145_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14145_5 = {S52, 0, &V14145_5_1293, &P14145_1};	/*Property = ScreenPixelsHeight*/
const Value V14145_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14145_4 = {S54, 0, &V14145_4_1296, &P14145_5};	/*Property = ScreenPixelsWidth*/
const Profile P14145 = {"14145", &P14145_4};	/*Profile ID - 14145*/
const Value V18012_1_1 = {S2, NULL};	/*Value = True*/
const Property P18012_1 = {S50, 0, &V18012_1_1, NULL};	/*Property = IsMobile*/
const Value V18012_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18012_5 = {S52, 0, &V18012_5_1277, &P18012_1};	/*Property = ScreenPixelsHeight*/
const Value V18012_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18012_4 = {S54, 0, &V18012_4_1279, &P18012_5};	/*Property = ScreenPixelsWidth*/
const Profile P18012 = {"18012", &P18012_4};	/*Profile ID - 18012*/
const Value V14147_1_1 = {S2, NULL};	/*Value = True*/
const Property P14147_1 = {S50, 0, &V14147_1_1, NULL};	/*Property = IsMobile*/
const Value V14147_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P14147_5 = {S52, 0, &V14147_5_1269, &P14147_1};	/*Property = ScreenPixelsHeight*/
const Value V14147_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14147_4 = {S54, 0, &V14147_4_1229, &P14147_5};	/*Property = ScreenPixelsWidth*/
const Profile P14147 = {"14147", &P14147_4};	/*Profile ID - 14147*/
const Value V14149_1_1 = {S2, NULL};	/*Value = True*/
const Property P14149_1 = {S50, 0, &V14149_1_1, NULL};	/*Property = IsMobile*/
const Value V14149_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14149_5 = {S52, 0, &V14149_5_1233, &P14149_1};	/*Property = ScreenPixelsHeight*/
const Value V14149_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14149_4 = {S54, 0, &V14149_4_1235, &P14149_5};	/*Property = ScreenPixelsWidth*/
const Profile P14149 = {"14149", &P14149_4};	/*Profile ID - 14149*/
const Value V14150_1_1 = {S2, NULL};	/*Value = True*/
const Property P14150_1 = {S50, 0, &V14150_1_1, NULL};	/*Property = IsMobile*/
const Value V14150_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14150_5 = {S52, 0, &V14150_5_1222, &P14150_1};	/*Property = ScreenPixelsHeight*/
const Value V14150_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14150_4 = {S54, 0, &V14150_4_1224, &P14150_5};	/*Property = ScreenPixelsWidth*/
const Profile P14150 = {"14150", &P14150_4};	/*Profile ID - 14150*/
const Value V14151_1_1 = {S2, NULL};	/*Value = True*/
const Property P14151_1 = {S50, 0, &V14151_1_1, NULL};	/*Property = IsMobile*/
const Value V14151_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14151_5 = {S52, 0, &V14151_5_1222, &P14151_1};	/*Property = ScreenPixelsHeight*/
const Value V14151_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14151_4 = {S54, 0, &V14151_4_1224, &P14151_5};	/*Property = ScreenPixelsWidth*/
const Profile P14151 = {"14151", &P14151_4};	/*Profile ID - 14151*/
const Value V14152_1_1 = {S2, NULL};	/*Value = True*/
const Property P14152_1 = {S50, 0, &V14152_1_1, NULL};	/*Property = IsMobile*/
const Value V14152_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14152_5 = {S52, 0, &V14152_5_1293, &P14152_1};	/*Property = ScreenPixelsHeight*/
const Value V14152_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14152_4 = {S54, 0, &V14152_4_1296, &P14152_5};	/*Property = ScreenPixelsWidth*/
const Profile P14152 = {"14152", &P14152_4};	/*Profile ID - 14152*/
const Value V14153_1_1 = {S2, NULL};	/*Value = True*/
const Property P14153_1 = {S50, 0, &V14153_1_1, NULL};	/*Property = IsMobile*/
const Value V14153_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14153_5 = {S52, 0, &V14153_5_1233, &P14153_1};	/*Property = ScreenPixelsHeight*/
const Value V14153_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14153_4 = {S54, 0, &V14153_4_1235, &P14153_5};	/*Property = ScreenPixelsWidth*/
const Profile P14153 = {"14153", &P14153_4};	/*Profile ID - 14153*/
const Value V14155_1_1 = {S2, NULL};	/*Value = True*/
const Property P14155_1 = {S50, 0, &V14155_1_1, NULL};	/*Property = IsMobile*/
const Value V14155_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14155_5 = {S52, 0, &V14155_5_1304, &P14155_1};	/*Property = ScreenPixelsHeight*/
const Value V14155_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14155_4 = {S54, 0, &V14155_4_1224, &P14155_5};	/*Property = ScreenPixelsWidth*/
const Profile P14155 = {"14155", &P14155_4};	/*Profile ID - 14155*/
const Value V14157_1_1 = {S2, NULL};	/*Value = True*/
const Property P14157_1 = {S50, 0, &V14157_1_1, NULL};	/*Property = IsMobile*/
const Value V14157_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14157_5 = {S52, 0, &V14157_5_1222, &P14157_1};	/*Property = ScreenPixelsHeight*/
const Value V14157_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14157_4 = {S54, 0, &V14157_4_1224, &P14157_5};	/*Property = ScreenPixelsWidth*/
const Profile P14157 = {"14157", &P14157_4};	/*Profile ID - 14157*/
const Value V14166_1_1 = {S2, NULL};	/*Value = True*/
const Property P14166_1 = {S50, 0, &V14166_1_1, NULL};	/*Property = IsMobile*/
const Value V14166_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14166_5 = {S52, 0, &V14166_5_1227, &P14166_1};	/*Property = ScreenPixelsHeight*/
const Value V14166_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14166_4 = {S54, 0, &V14166_4_1229, &P14166_5};	/*Property = ScreenPixelsWidth*/
const Profile P14166 = {"14166", &P14166_4};	/*Profile ID - 14166*/
const Value V14170_1_1 = {S2, NULL};	/*Value = True*/
const Property P14170_1 = {S50, 0, &V14170_1_1, NULL};	/*Property = IsMobile*/
const Value V14170_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14170_5 = {S52, 0, &V14170_5_1294, &P14170_1};	/*Property = ScreenPixelsHeight*/
const Value V14170_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14170_4 = {S54, 0, &V14170_4_1296, &P14170_5};	/*Property = ScreenPixelsWidth*/
const Profile P14170 = {"14170", &P14170_4};	/*Profile ID - 14170*/
const Value V14177_1_1 = {S2, NULL};	/*Value = True*/
const Property P14177_1 = {S50, 0, &V14177_1_1, NULL};	/*Property = IsMobile*/
const Value V14177_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P14177_5 = {S52, 0, &V14177_5_1333, &P14177_1};	/*Property = ScreenPixelsHeight*/
const Value V14177_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P14177_4 = {S54, 0, &V14177_4_1335, &P14177_5};	/*Property = ScreenPixelsWidth*/
const Profile P14177 = {"14177", &P14177_4};	/*Profile ID - 14177*/
const Value V14179_1_1 = {S2, NULL};	/*Value = True*/
const Property P14179_1 = {S50, 0, &V14179_1_1, NULL};	/*Property = IsMobile*/
const Value V14179_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14179_5 = {S52, 0, &V14179_5_1233, &P14179_1};	/*Property = ScreenPixelsHeight*/
const Value V14179_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14179_4 = {S54, 0, &V14179_4_1235, &P14179_5};	/*Property = ScreenPixelsWidth*/
const Profile P14179 = {"14179", &P14179_4};	/*Profile ID - 14179*/
const Value V14172_1_1 = {S2, NULL};	/*Value = True*/
const Property P14172_1 = {S50, 0, &V14172_1_1, NULL};	/*Property = IsMobile*/
const Value V14172_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14172_5 = {S52, 0, &V14172_5_1233, &P14172_1};	/*Property = ScreenPixelsHeight*/
const Value V14172_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14172_4 = {S54, 0, &V14172_4_1235, &P14172_5};	/*Property = ScreenPixelsWidth*/
const Profile P14172 = {"14172", &P14172_4};	/*Profile ID - 14172*/
const Value V14191_1_1 = {S2, NULL};	/*Value = True*/
const Property P14191_1 = {S50, 0, &V14191_1_1, NULL};	/*Property = IsMobile*/
const Value V14191_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14191_5 = {S52, 0, &V14191_5_1293, &P14191_1};	/*Property = ScreenPixelsHeight*/
const Value V14191_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14191_4 = {S54, 0, &V14191_4_1296, &P14191_5};	/*Property = ScreenPixelsWidth*/
const Profile P14191 = {"14191", &P14191_4};	/*Profile ID - 14191*/
const Value V14194_1_1 = {S2, NULL};	/*Value = True*/
const Property P14194_1 = {S50, 0, &V14194_1_1, NULL};	/*Property = IsMobile*/
const Value V14194_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14194_5 = {S52, 0, &V14194_5_1233, &P14194_1};	/*Property = ScreenPixelsHeight*/
const Value V14194_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14194_4 = {S54, 0, &V14194_4_1235, &P14194_5};	/*Property = ScreenPixelsWidth*/
const Profile P14194 = {"14194", &P14194_4};	/*Profile ID - 14194*/
const Value V14858_1_1 = {S2, NULL};	/*Value = True*/
const Property P14858_1 = {S50, 0, &V14858_1_1, NULL};	/*Property = IsMobile*/
const Value V14858_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14858_5 = {S52, 0, &V14858_5_1233, &P14858_1};	/*Property = ScreenPixelsHeight*/
const Value V14858_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14858_4 = {S54, 0, &V14858_4_1235, &P14858_5};	/*Property = ScreenPixelsWidth*/
const Profile P14858 = {"14858", &P14858_4};	/*Profile ID - 14858*/
const Value V14193_1_1 = {S2, NULL};	/*Value = True*/
const Property P14193_1 = {S50, 0, &V14193_1_1, NULL};	/*Property = IsMobile*/
const Value V14193_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14193_5 = {S52, 0, &V14193_5_1222, &P14193_1};	/*Property = ScreenPixelsHeight*/
const Value V14193_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14193_4 = {S54, 0, &V14193_4_1224, &P14193_5};	/*Property = ScreenPixelsWidth*/
const Profile P14193 = {"14193", &P14193_4};	/*Profile ID - 14193*/
const Value V14200_1_1 = {S2, NULL};	/*Value = True*/
const Property P14200_1 = {S50, 0, &V14200_1_1, NULL};	/*Property = IsMobile*/
const Value V14200_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14200_5 = {S52, 0, &V14200_5_1304, &P14200_1};	/*Property = ScreenPixelsHeight*/
const Value V14200_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14200_4 = {S54, 0, &V14200_4_1224, &P14200_5};	/*Property = ScreenPixelsWidth*/
const Profile P14200 = {"14200", &P14200_4};	/*Profile ID - 14200*/
const Value V14201_1_1 = {S2, NULL};	/*Value = True*/
const Property P14201_1 = {S50, 0, &V14201_1_1, NULL};	/*Property = IsMobile*/
const Value V14201_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14201_5 = {S52, 0, &V14201_5_1233, &P14201_1};	/*Property = ScreenPixelsHeight*/
const Value V14201_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14201_4 = {S54, 0, &V14201_4_1235, &P14201_5};	/*Property = ScreenPixelsWidth*/
const Profile P14201 = {"14201", &P14201_4};	/*Profile ID - 14201*/
const Value V14204_1_1 = {S2, NULL};	/*Value = True*/
const Property P14204_1 = {S50, 0, &V14204_1_1, NULL};	/*Property = IsMobile*/
const Value V14204_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14204_5 = {S52, 0, &V14204_5_1233, &P14204_1};	/*Property = ScreenPixelsHeight*/
const Value V14204_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14204_4 = {S54, 0, &V14204_4_1235, &P14204_5};	/*Property = ScreenPixelsWidth*/
const Profile P14204 = {"14204", &P14204_4};	/*Profile ID - 14204*/
const Value V14203_1_1 = {S2, NULL};	/*Value = True*/
const Property P14203_1 = {S50, 0, &V14203_1_1, NULL};	/*Property = IsMobile*/
const Value V14203_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14203_5 = {S52, 0, &V14203_5_1222, &P14203_1};	/*Property = ScreenPixelsHeight*/
const Value V14203_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14203_4 = {S54, 0, &V14203_4_1224, &P14203_5};	/*Property = ScreenPixelsWidth*/
const Profile P14203 = {"14203", &P14203_4};	/*Profile ID - 14203*/
const Value V14205_1_1 = {S2, NULL};	/*Value = True*/
const Property P14205_1 = {S50, 0, &V14205_1_1, NULL};	/*Property = IsMobile*/
const Value V14205_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14205_5 = {S52, 0, &V14205_5_1380, &P14205_1};	/*Property = ScreenPixelsHeight*/
const Value V14205_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14205_4 = {S54, 0, &V14205_4_1279, &P14205_5};	/*Property = ScreenPixelsWidth*/
const Profile P14205 = {"14205", &P14205_4};	/*Profile ID - 14205*/
const Value V14206_1_1 = {S2, NULL};	/*Value = True*/
const Property P14206_1 = {S50, 0, &V14206_1_1, NULL};	/*Property = IsMobile*/
const Value V14206_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14206_5 = {S52, 0, &V14206_5_1380, &P14206_1};	/*Property = ScreenPixelsHeight*/
const Value V14206_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14206_4 = {S54, 0, &V14206_4_1279, &P14206_5};	/*Property = ScreenPixelsWidth*/
const Profile P14206 = {"14206", &P14206_4};	/*Profile ID - 14206*/
const Value V14212_1_1 = {S2, NULL};	/*Value = True*/
const Property P14212_1 = {S50, 0, &V14212_1_1, NULL};	/*Property = IsMobile*/
const Value V14212_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14212_5 = {S52, 0, &V14212_5_1233, &P14212_1};	/*Property = ScreenPixelsHeight*/
const Value V14212_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14212_4 = {S54, 0, &V14212_4_1235, &P14212_5};	/*Property = ScreenPixelsWidth*/
const Profile P14212 = {"14212", &P14212_4};	/*Profile ID - 14212*/
const Value V14213_1_1 = {S2, NULL};	/*Value = True*/
const Property P14213_1 = {S50, 0, &V14213_1_1, NULL};	/*Property = IsMobile*/
const Value V14213_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14213_5 = {S52, 0, &V14213_5_1222, &P14213_1};	/*Property = ScreenPixelsHeight*/
const Value V14213_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14213_4 = {S54, 0, &V14213_4_1224, &P14213_5};	/*Property = ScreenPixelsWidth*/
const Profile P14213 = {"14213", &P14213_4};	/*Profile ID - 14213*/
const Value V14215_1_1 = {S2, NULL};	/*Value = True*/
const Property P14215_1 = {S50, 0, &V14215_1_1, NULL};	/*Property = IsMobile*/
const Value V14215_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14215_5 = {S52, 0, &V14215_5_1304, &P14215_1};	/*Property = ScreenPixelsHeight*/
const Value V14215_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14215_4 = {S54, 0, &V14215_4_1224, &P14215_5};	/*Property = ScreenPixelsWidth*/
const Profile P14215 = {"14215", &P14215_4};	/*Profile ID - 14215*/
const Value V17018_1_1 = {S2, NULL};	/*Value = True*/
const Property P17018_1 = {S50, 0, &V17018_1_1, NULL};	/*Property = IsMobile*/
const Value V17018_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P17018_5 = {S52, 0, &V17018_5_1294, &P17018_1};	/*Property = ScreenPixelsHeight*/
const Value V17018_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17018_4 = {S54, 0, &V17018_4_1296, &P17018_5};	/*Property = ScreenPixelsWidth*/
const Profile P17018 = {"17018", &P17018_4};	/*Profile ID - 17018*/
const Value V14216_1_1 = {S2, NULL};	/*Value = True*/
const Property P14216_1 = {S50, 0, &V14216_1_1, NULL};	/*Property = IsMobile*/
const Value V14216_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14216_5 = {S52, 0, &V14216_5_1233, &P14216_1};	/*Property = ScreenPixelsHeight*/
const Value V14216_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14216_4 = {S54, 0, &V14216_4_1235, &P14216_5};	/*Property = ScreenPixelsWidth*/
const Profile P14216 = {"14216", &P14216_4};	/*Profile ID - 14216*/
const Value V14220_1_1 = {S2, NULL};	/*Value = True*/
const Property P14220_1 = {S50, 0, &V14220_1_1, NULL};	/*Property = IsMobile*/
const Value V14220_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14220_5 = {S52, 0, &V14220_5_1294, &P14220_1};	/*Property = ScreenPixelsHeight*/
const Value V14220_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14220_4 = {S54, 0, &V14220_4_1296, &P14220_5};	/*Property = ScreenPixelsWidth*/
const Profile P14220 = {"14220", &P14220_4};	/*Profile ID - 14220*/
const Value V14221_1_1 = {S2, NULL};	/*Value = True*/
const Property P14221_1 = {S50, 0, &V14221_1_1, NULL};	/*Property = IsMobile*/
const Value V14221_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14221_5 = {S52, 0, &V14221_5_1380, &P14221_1};	/*Property = ScreenPixelsHeight*/
const Value V14221_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14221_4 = {S54, 0, &V14221_4_1279, &P14221_5};	/*Property = ScreenPixelsWidth*/
const Profile P14221 = {"14221", &P14221_4};	/*Profile ID - 14221*/
const Value V14226_1_1 = {S2, NULL};	/*Value = True*/
const Property P14226_1 = {S50, 0, &V14226_1_1, NULL};	/*Property = IsMobile*/
const Value V14226_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14226_5 = {S52, 0, &V14226_5_1222, &P14226_1};	/*Property = ScreenPixelsHeight*/
const Value V14226_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14226_4 = {S54, 0, &V14226_4_1224, &P14226_5};	/*Property = ScreenPixelsWidth*/
const Profile P14226 = {"14226", &P14226_4};	/*Profile ID - 14226*/
const Value V14228_1_1 = {S2, NULL};	/*Value = True*/
const Property P14228_1 = {S50, 0, &V14228_1_1, NULL};	/*Property = IsMobile*/
const Value V14228_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14228_5 = {S52, 0, &V14228_5_1380, &P14228_1};	/*Property = ScreenPixelsHeight*/
const Value V14228_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P14228_4 = {S54, 0, &V14228_4_1943, &P14228_5};	/*Property = ScreenPixelsWidth*/
const Profile P14228 = {"14228", &P14228_4};	/*Profile ID - 14228*/
const Value V14224_1_1 = {S2, NULL};	/*Value = True*/
const Property P14224_1 = {S50, 0, &V14224_1_1, NULL};	/*Property = IsMobile*/
const Value V14224_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14224_5 = {S52, 0, &V14224_5_1227, &P14224_1};	/*Property = ScreenPixelsHeight*/
const Value V14224_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14224_4 = {S54, 0, &V14224_4_1229, &P14224_5};	/*Property = ScreenPixelsWidth*/
const Profile P14224 = {"14224", &P14224_4};	/*Profile ID - 14224*/
const Value V14231_1_1 = {S2, NULL};	/*Value = True*/
const Property P14231_1 = {S50, 0, &V14231_1_1, NULL};	/*Property = IsMobile*/
const Value V14231_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14231_5 = {S52, 0, &V14231_5_1380, &P14231_1};	/*Property = ScreenPixelsHeight*/
const Value V14231_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14231_4 = {S54, 0, &V14231_4_1279, &P14231_5};	/*Property = ScreenPixelsWidth*/
const Profile P14231 = {"14231", &P14231_4};	/*Profile ID - 14231*/
const Value V14235_1_1 = {S2, NULL};	/*Value = True*/
const Property P14235_1 = {S50, 0, &V14235_1_1, NULL};	/*Property = IsMobile*/
const Value V14235_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14235_5 = {S52, 0, &V14235_5_1293, &P14235_1};	/*Property = ScreenPixelsHeight*/
const Value V14235_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14235_4 = {S54, 0, &V14235_4_1296, &P14235_5};	/*Property = ScreenPixelsWidth*/
const Profile P14235 = {"14235", &P14235_4};	/*Profile ID - 14235*/
const Value V13311_1_1 = {S2, NULL};	/*Value = True*/
const Property P13311_1 = {S50, 0, &V13311_1_1, NULL};	/*Property = IsMobile*/
const Value V13311_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13311_5 = {S52, 0, &V13311_5_1222, &P13311_1};	/*Property = ScreenPixelsHeight*/
const Value V13311_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13311_4 = {S54, 0, &V13311_4_1224, &P13311_5};	/*Property = ScreenPixelsWidth*/
const Profile P13311 = {"13311", &P13311_4};	/*Profile ID - 13311*/
const Value V14237_1_1 = {S2, NULL};	/*Value = True*/
const Property P14237_1 = {S50, 0, &V14237_1_1, NULL};	/*Property = IsMobile*/
const Value V14237_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14237_5 = {S52, 0, &V14237_5_1380, &P14237_1};	/*Property = ScreenPixelsHeight*/
const Value V14237_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14237_4 = {S54, 0, &V14237_4_1279, &P14237_5};	/*Property = ScreenPixelsWidth*/
const Profile P14237 = {"14237", &P14237_4};	/*Profile ID - 14237*/
const Value V14238_1_1 = {S2, NULL};	/*Value = True*/
const Property P14238_1 = {S50, 0, &V14238_1_1, NULL};	/*Property = IsMobile*/
const Value V14238_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14238_5 = {S52, 0, &V14238_5_1222, &P14238_1};	/*Property = ScreenPixelsHeight*/
const Value V14238_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14238_4 = {S54, 0, &V14238_4_1224, &P14238_5};	/*Property = ScreenPixelsWidth*/
const Profile P14238 = {"14238", &P14238_4};	/*Profile ID - 14238*/
const Value V14134_1_1 = {S2, NULL};	/*Value = True*/
const Property P14134_1 = {S50, 0, &V14134_1_1, NULL};	/*Property = IsMobile*/
const Value V14134_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14134_5 = {S52, 0, &V14134_5_1294, &P14134_1};	/*Property = ScreenPixelsHeight*/
const Value V14134_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14134_4 = {S54, 0, &V14134_4_1296, &P14134_5};	/*Property = ScreenPixelsWidth*/
const Profile P14134 = {"14134", &P14134_4};	/*Profile ID - 14134*/
const Value V14239_1_1 = {S2, NULL};	/*Value = True*/
const Property P14239_1 = {S50, 0, &V14239_1_1, NULL};	/*Property = IsMobile*/
const Value V14239_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14239_5 = {S52, 0, &V14239_5_1222, &P14239_1};	/*Property = ScreenPixelsHeight*/
const Value V14239_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14239_4 = {S54, 0, &V14239_4_1224, &P14239_5};	/*Property = ScreenPixelsWidth*/
const Profile P14239 = {"14239", &P14239_4};	/*Profile ID - 14239*/
const Value V14240_1_1 = {S2, NULL};	/*Value = True*/
const Property P14240_1 = {S50, 0, &V14240_1_1, NULL};	/*Property = IsMobile*/
const Value V14240_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14240_5 = {S52, 0, &V14240_5_1304, &P14240_1};	/*Property = ScreenPixelsHeight*/
const Value V14240_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14240_4 = {S54, 0, &V14240_4_1224, &P14240_5};	/*Property = ScreenPixelsWidth*/
const Profile P14240 = {"14240", &P14240_4};	/*Profile ID - 14240*/
const Value V13477_1_1 = {S2, NULL};	/*Value = True*/
const Property P13477_1 = {S50, 0, &V13477_1_1, NULL};	/*Property = IsMobile*/
const Value V13477_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13477_5 = {S52, 0, &V13477_5_1222, &P13477_1};	/*Property = ScreenPixelsHeight*/
const Value V13477_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13477_4 = {S54, 0, &V13477_4_1224, &P13477_5};	/*Property = ScreenPixelsWidth*/
const Profile P13477 = {"13477", &P13477_4};	/*Profile ID - 13477*/
const Value V14242_1_1 = {S2, NULL};	/*Value = True*/
const Property P14242_1 = {S50, 0, &V14242_1_1, NULL};	/*Property = IsMobile*/
const Value V14242_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14242_5 = {S52, 0, &V14242_5_1277, &P14242_1};	/*Property = ScreenPixelsHeight*/
const Value V14242_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14242_4 = {S54, 0, &V14242_4_1224, &P14242_5};	/*Property = ScreenPixelsWidth*/
const Profile P14242 = {"14242", &P14242_4};	/*Profile ID - 14242*/
const Value V14248_1_1 = {S2, NULL};	/*Value = True*/
const Property P14248_1 = {S50, 0, &V14248_1_1, NULL};	/*Property = IsMobile*/
const Value V14248_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P14248_5 = {S52, 0, &V14248_5_1275, &P14248_1};	/*Property = ScreenPixelsHeight*/
const Value V14248_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P14248_4 = {S54, 0, &V14248_4_1278, &P14248_5};	/*Property = ScreenPixelsWidth*/
const Profile P14248 = {"14248", &P14248_4};	/*Profile ID - 14248*/
const Value V14255_1_1 = {S2, NULL};	/*Value = True*/
const Property P14255_1 = {S50, 0, &V14255_1_1, NULL};	/*Property = IsMobile*/
const Value V14255_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14255_5 = {S52, 0, &V14255_5_1222, &P14255_1};	/*Property = ScreenPixelsHeight*/
const Value V14255_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14255_4 = {S54, 0, &V14255_4_1224, &P14255_5};	/*Property = ScreenPixelsWidth*/
const Profile P14255 = {"14255", &P14255_4};	/*Profile ID - 14255*/
const Value V14259_1_1 = {S2, NULL};	/*Value = True*/
const Property P14259_1 = {S50, 0, &V14259_1_1, NULL};	/*Property = IsMobile*/
const Value V14259_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14259_5 = {S52, 0, &V14259_5_1233, &P14259_1};	/*Property = ScreenPixelsHeight*/
const Value V14259_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14259_4 = {S54, 0, &V14259_4_1235, &P14259_5};	/*Property = ScreenPixelsWidth*/
const Profile P14259 = {"14259", &P14259_4};	/*Profile ID - 14259*/
const Value V14260_1_1 = {S2, NULL};	/*Value = True*/
const Property P14260_1 = {S50, 0, &V14260_1_1, NULL};	/*Property = IsMobile*/
const Value V14260_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14260_5 = {S52, 0, &V14260_5_1304, &P14260_1};	/*Property = ScreenPixelsHeight*/
const Value V14260_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14260_4 = {S54, 0, &V14260_4_1224, &P14260_5};	/*Property = ScreenPixelsWidth*/
const Profile P14260 = {"14260", &P14260_4};	/*Profile ID - 14260*/
const Value V14262_1_1 = {S2, NULL};	/*Value = True*/
const Property P14262_1 = {S50, 0, &V14262_1_1, NULL};	/*Property = IsMobile*/
const Value V14262_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14262_5 = {S52, 0, &V14262_5_1222, &P14262_1};	/*Property = ScreenPixelsHeight*/
const Value V14262_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14262_4 = {S54, 0, &V14262_4_1224, &P14262_5};	/*Property = ScreenPixelsWidth*/
const Profile P14262 = {"14262", &P14262_4};	/*Profile ID - 14262*/
const Value V14263_1_1 = {S2, NULL};	/*Value = True*/
const Property P14263_1 = {S50, 0, &V14263_1_1, NULL};	/*Property = IsMobile*/
const Value V14263_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14263_5 = {S52, 0, &V14263_5_1233, &P14263_1};	/*Property = ScreenPixelsHeight*/
const Value V14263_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14263_4 = {S54, 0, &V14263_4_1235, &P14263_5};	/*Property = ScreenPixelsWidth*/
const Profile P14263 = {"14263", &P14263_4};	/*Profile ID - 14263*/
const Value V14265_1_1 = {S2, NULL};	/*Value = True*/
const Property P14265_1 = {S50, 0, &V14265_1_1, NULL};	/*Property = IsMobile*/
const Value V14265_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14265_5 = {S52, 0, &V14265_5_1233, &P14265_1};	/*Property = ScreenPixelsHeight*/
const Value V14265_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14265_4 = {S54, 0, &V14265_4_1235, &P14265_5};	/*Property = ScreenPixelsWidth*/
const Profile P14265 = {"14265", &P14265_4};	/*Profile ID - 14265*/
const Value V14267_1_1 = {S2, NULL};	/*Value = True*/
const Property P14267_1 = {S50, 0, &V14267_1_1, NULL};	/*Property = IsMobile*/
const Value V14267_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14267_5 = {S52, 0, &V14267_5_1293, &P14267_1};	/*Property = ScreenPixelsHeight*/
const Value V14267_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14267_4 = {S54, 0, &V14267_4_1296, &P14267_5};	/*Property = ScreenPixelsWidth*/
const Profile P14267 = {"14267", &P14267_4};	/*Profile ID - 14267*/
const Value V14268_1_1 = {S2, NULL};	/*Value = True*/
const Property P14268_1 = {S50, 0, &V14268_1_1, NULL};	/*Property = IsMobile*/
const Value V14268_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14268_5 = {S52, 0, &V14268_5_1222, &P14268_1};	/*Property = ScreenPixelsHeight*/
const Value V14268_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14268_4 = {S54, 0, &V14268_4_1224, &P14268_5};	/*Property = ScreenPixelsWidth*/
const Profile P14268 = {"14268", &P14268_4};	/*Profile ID - 14268*/
const Value V16972_1_1 = {S2, NULL};	/*Value = True*/
const Property P16972_1 = {S50, 0, &V16972_1_1, NULL};	/*Property = IsMobile*/
const Value V16972_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16972_5 = {S52, 0, &V16972_5_1222, &P16972_1};	/*Property = ScreenPixelsHeight*/
const Value V16972_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16972_4 = {S54, 0, &V16972_4_1224, &P16972_5};	/*Property = ScreenPixelsWidth*/
const Profile P16972 = {"16972", &P16972_4};	/*Profile ID - 16972*/
const Value V12415_1_1 = {S2, NULL};	/*Value = True*/
const Property P12415_1 = {S50, 0, &V12415_1_1, NULL};	/*Property = IsMobile*/
const Value V12415_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12415_5 = {S52, 0, &V12415_5_1222, &P12415_1};	/*Property = ScreenPixelsHeight*/
const Value V12415_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12415_4 = {S54, 0, &V12415_4_1224, &P12415_5};	/*Property = ScreenPixelsWidth*/
const Profile P12415 = {"12415", &P12415_4};	/*Profile ID - 12415*/
const Value V17540_1_1 = {S2, NULL};	/*Value = True*/
const Property P17540_1 = {S50, 0, &V17540_1_1, NULL};	/*Property = IsMobile*/
const Value V17540_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17540_5 = {S52, 0, &V17540_5_1304, &P17540_1};	/*Property = ScreenPixelsHeight*/
const Value V17540_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17540_4 = {S54, 0, &V17540_4_1224, &P17540_5};	/*Property = ScreenPixelsWidth*/
const Profile P17540 = {"17540", &P17540_4};	/*Profile ID - 17540*/
const Value V12071_1_1 = {S2, NULL};	/*Value = True*/
const Property P12071_1 = {S50, 0, &V12071_1_1, NULL};	/*Property = IsMobile*/
const Value V12071_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12071_5 = {S52, 0, &V12071_5_1304, &P12071_1};	/*Property = ScreenPixelsHeight*/
const Value V12071_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12071_4 = {S54, 0, &V12071_4_1224, &P12071_5};	/*Property = ScreenPixelsWidth*/
const Profile P12071 = {"12071", &P12071_4};	/*Profile ID - 12071*/
const Value V14273_1_1 = {S2, NULL};	/*Value = True*/
const Property P14273_1 = {S50, 0, &V14273_1_1, NULL};	/*Property = IsMobile*/
const Value V14273_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14273_5 = {S52, 0, &V14273_5_1222, &P14273_1};	/*Property = ScreenPixelsHeight*/
const Value V14273_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14273_4 = {S54, 0, &V14273_4_1224, &P14273_5};	/*Property = ScreenPixelsWidth*/
const Profile P14273 = {"14273", &P14273_4};	/*Profile ID - 14273*/
const Value V14274_1_1 = {S2, NULL};	/*Value = True*/
const Property P14274_1 = {S50, 0, &V14274_1_1, NULL};	/*Property = IsMobile*/
const Value V14274_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14274_5 = {S52, 0, &V14274_5_1304, &P14274_1};	/*Property = ScreenPixelsHeight*/
const Value V14274_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14274_4 = {S54, 0, &V14274_4_1224, &P14274_5};	/*Property = ScreenPixelsWidth*/
const Profile P14274 = {"14274", &P14274_4};	/*Profile ID - 14274*/
const Value V14275_1_1 = {S2, NULL};	/*Value = True*/
const Property P14275_1 = {S50, 0, &V14275_1_1, NULL};	/*Property = IsMobile*/
const Value V14275_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14275_5 = {S52, 0, &V14275_5_1233, &P14275_1};	/*Property = ScreenPixelsHeight*/
const Value V14275_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14275_4 = {S54, 0, &V14275_4_1235, &P14275_5};	/*Property = ScreenPixelsWidth*/
const Profile P14275 = {"14275", &P14275_4};	/*Profile ID - 14275*/
const Value V13609_1_1 = {S2, NULL};	/*Value = True*/
const Property P13609_1 = {S50, 0, &V13609_1_1, NULL};	/*Property = IsMobile*/
const Value V13609_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13609_5 = {S52, 0, &V13609_5_1293, &P13609_1};	/*Property = ScreenPixelsHeight*/
const Value V13609_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13609_4 = {S54, 0, &V13609_4_1224, &P13609_5};	/*Property = ScreenPixelsWidth*/
const Profile P13609 = {"13609", &P13609_4};	/*Profile ID - 13609*/
const Value V14276_1_1 = {S2, NULL};	/*Value = True*/
const Property P14276_1 = {S50, 0, &V14276_1_1, NULL};	/*Property = IsMobile*/
const Value V14276_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14276_5 = {S52, 0, &V14276_5_1222, &P14276_1};	/*Property = ScreenPixelsHeight*/
const Value V14276_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14276_4 = {S54, 0, &V14276_4_1224, &P14276_5};	/*Property = ScreenPixelsWidth*/
const Profile P14276 = {"14276", &P14276_4};	/*Profile ID - 14276*/
const Value V14277_1_1 = {S2, NULL};	/*Value = True*/
const Property P14277_1 = {S50, 0, &V14277_1_1, NULL};	/*Property = IsMobile*/
const Value V14277_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14277_5 = {S52, 0, &V14277_5_1233, &P14277_1};	/*Property = ScreenPixelsHeight*/
const Value V14277_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14277_4 = {S54, 0, &V14277_4_1235, &P14277_5};	/*Property = ScreenPixelsWidth*/
const Profile P14277 = {"14277", &P14277_4};	/*Profile ID - 14277*/
const Value V13630_1_1 = {S2, NULL};	/*Value = True*/
const Property P13630_1 = {S50, 0, &V13630_1_1, NULL};	/*Property = IsMobile*/
const Value V13630_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13630_5 = {S52, 0, &V13630_5_1233, &P13630_1};	/*Property = ScreenPixelsHeight*/
const Value V13630_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13630_4 = {S54, 0, &V13630_4_1235, &P13630_5};	/*Property = ScreenPixelsWidth*/
const Profile P13630 = {"13630", &P13630_4};	/*Profile ID - 13630*/
const Value V14278_1_1 = {S2, NULL};	/*Value = True*/
const Property P14278_1 = {S50, 0, &V14278_1_1, NULL};	/*Property = IsMobile*/
const Value V14278_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14278_5 = {S52, 0, &V14278_5_1304, &P14278_1};	/*Property = ScreenPixelsHeight*/
const Value V14278_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14278_4 = {S54, 0, &V14278_4_1224, &P14278_5};	/*Property = ScreenPixelsWidth*/
const Profile P14278 = {"14278", &P14278_4};	/*Profile ID - 14278*/
const Value V14279_1_1 = {S2, NULL};	/*Value = True*/
const Property P14279_1 = {S50, 0, &V14279_1_1, NULL};	/*Property = IsMobile*/
const Value V14279_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14279_5 = {S52, 0, &V14279_5_1294, &P14279_1};	/*Property = ScreenPixelsHeight*/
const Value V14279_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14279_4 = {S54, 0, &V14279_4_1296, &P14279_5};	/*Property = ScreenPixelsWidth*/
const Profile P14279 = {"14279", &P14279_4};	/*Profile ID - 14279*/
const Value V14281_1_1 = {S2, NULL};	/*Value = True*/
const Property P14281_1 = {S50, 0, &V14281_1_1, NULL};	/*Property = IsMobile*/
const Value V14281_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14281_5 = {S52, 0, &V14281_5_1222, &P14281_1};	/*Property = ScreenPixelsHeight*/
const Value V14281_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14281_4 = {S54, 0, &V14281_4_1224, &P14281_5};	/*Property = ScreenPixelsWidth*/
const Profile P14281 = {"14281", &P14281_4};	/*Profile ID - 14281*/
const Value V14284_1_1 = {S2, NULL};	/*Value = True*/
const Property P14284_1 = {S50, 0, &V14284_1_1, NULL};	/*Property = IsMobile*/
const Value V14284_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14284_5 = {S52, 0, &V14284_5_1294, &P14284_1};	/*Property = ScreenPixelsHeight*/
const Value V14284_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14284_4 = {S54, 0, &V14284_4_1296, &P14284_5};	/*Property = ScreenPixelsWidth*/
const Profile P14284 = {"14284", &P14284_4};	/*Profile ID - 14284*/
const Value V17204_1_1 = {S2, NULL};	/*Value = True*/
const Property P17204_1 = {S50, 0, &V17204_1_1, NULL};	/*Property = IsMobile*/
const Value V17204_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P17204_5 = {S52, 0, &V17204_5_1294, &P17204_1};	/*Property = ScreenPixelsHeight*/
const Value V17204_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17204_4 = {S54, 0, &V17204_4_1296, &P17204_5};	/*Property = ScreenPixelsWidth*/
const Profile P17204 = {"17204", &P17204_4};	/*Profile ID - 17204*/
const Value V14287_1_1 = {S2, NULL};	/*Value = True*/
const Property P14287_1 = {S50, 0, &V14287_1_1, NULL};	/*Property = IsMobile*/
const Value V14287_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14287_5 = {S52, 0, &V14287_5_1294, &P14287_1};	/*Property = ScreenPixelsHeight*/
const Value V14287_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14287_4 = {S54, 0, &V14287_4_1296, &P14287_5};	/*Property = ScreenPixelsWidth*/
const Profile P14287 = {"14287", &P14287_4};	/*Profile ID - 14287*/
const Value V13939_1_1 = {S2, NULL};	/*Value = True*/
const Property P13939_1 = {S50, 0, &V13939_1_1, NULL};	/*Property = IsMobile*/
const Value V13939_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13939_5 = {S52, 0, &V13939_5_1233, &P13939_1};	/*Property = ScreenPixelsHeight*/
const Value V13939_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13939_4 = {S54, 0, &V13939_4_1235, &P13939_5};	/*Property = ScreenPixelsWidth*/
const Profile P13939 = {"13939", &P13939_4};	/*Profile ID - 13939*/
const Value V14289_1_1 = {S2, NULL};	/*Value = True*/
const Property P14289_1 = {S50, 0, &V14289_1_1, NULL};	/*Property = IsMobile*/
const Value V14289_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14289_5 = {S52, 0, &V14289_5_1222, &P14289_1};	/*Property = ScreenPixelsHeight*/
const Value V14289_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14289_4 = {S54, 0, &V14289_4_1224, &P14289_5};	/*Property = ScreenPixelsWidth*/
const Profile P14289 = {"14289", &P14289_4};	/*Profile ID - 14289*/
const Value V14293_1_1 = {S2, NULL};	/*Value = True*/
const Property P14293_1 = {S50, 0, &V14293_1_1, NULL};	/*Property = IsMobile*/
const Value V14293_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14293_5 = {S52, 0, &V14293_5_1222, &P14293_1};	/*Property = ScreenPixelsHeight*/
const Value V14293_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14293_4 = {S54, 0, &V14293_4_1224, &P14293_5};	/*Property = ScreenPixelsWidth*/
const Profile P14293 = {"14293", &P14293_4};	/*Profile ID - 14293*/
const Value V14296_1_1 = {S2, NULL};	/*Value = True*/
const Property P14296_1 = {S50, 0, &V14296_1_1, NULL};	/*Property = IsMobile*/
const Value V14296_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14296_5 = {S52, 0, &V14296_5_1222, &P14296_1};	/*Property = ScreenPixelsHeight*/
const Value V14296_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14296_4 = {S54, 0, &V14296_4_1224, &P14296_5};	/*Property = ScreenPixelsWidth*/
const Profile P14296 = {"14296", &P14296_4};	/*Profile ID - 14296*/
const Value V14298_1_1 = {S2, NULL};	/*Value = True*/
const Property P14298_1 = {S50, 0, &V14298_1_1, NULL};	/*Property = IsMobile*/
const Value V14298_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14298_5 = {S52, 0, &V14298_5_1233, &P14298_1};	/*Property = ScreenPixelsHeight*/
const Value V14298_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14298_4 = {S54, 0, &V14298_4_1235, &P14298_5};	/*Property = ScreenPixelsWidth*/
const Profile P14298 = {"14298", &P14298_4};	/*Profile ID - 14298*/
const Value V14299_1_1 = {S2, NULL};	/*Value = True*/
const Property P14299_1 = {S50, 0, &V14299_1_1, NULL};	/*Property = IsMobile*/
const Value V14299_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14299_5 = {S52, 0, &V14299_5_1233, &P14299_1};	/*Property = ScreenPixelsHeight*/
const Value V14299_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14299_4 = {S54, 0, &V14299_4_1235, &P14299_5};	/*Property = ScreenPixelsWidth*/
const Profile P14299 = {"14299", &P14299_4};	/*Profile ID - 14299*/
const Value V14300_1_1 = {S2, NULL};	/*Value = True*/
const Property P14300_1 = {S50, 0, &V14300_1_1, NULL};	/*Property = IsMobile*/
const Value V14300_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14300_5 = {S52, 0, &V14300_5_1293, &P14300_1};	/*Property = ScreenPixelsHeight*/
const Value V14300_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14300_4 = {S54, 0, &V14300_4_1296, &P14300_5};	/*Property = ScreenPixelsWidth*/
const Profile P14300 = {"14300", &P14300_4};	/*Profile ID - 14300*/
const Value V14301_1_1 = {S2, NULL};	/*Value = True*/
const Property P14301_1 = {S50, 0, &V14301_1_1, NULL};	/*Property = IsMobile*/
const Value V14301_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14301_5 = {S52, 0, &V14301_5_1233, &P14301_1};	/*Property = ScreenPixelsHeight*/
const Value V14301_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14301_4 = {S54, 0, &V14301_4_1235, &P14301_5};	/*Property = ScreenPixelsWidth*/
const Profile P14301 = {"14301", &P14301_4};	/*Profile ID - 14301*/
const Value V14303_1_1 = {S2, NULL};	/*Value = True*/
const Property P14303_1 = {S50, 0, &V14303_1_1, NULL};	/*Property = IsMobile*/
const Value V14303_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P14303_5 = {S52, 0, &V14303_5_1333, &P14303_1};	/*Property = ScreenPixelsHeight*/
const Value V14303_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P14303_4 = {S54, 0, &V14303_4_1335, &P14303_5};	/*Property = ScreenPixelsWidth*/
const Profile P14303 = {"14303", &P14303_4};	/*Profile ID - 14303*/
const Value V23012_1_1 = {S2, NULL};	/*Value = True*/
const Property P23012_1 = {S50, 0, &V23012_1_1, NULL};	/*Property = IsMobile*/
const Value V23012_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23012_5 = {S52, 0, &V23012_5_1222, &P23012_1};	/*Property = ScreenPixelsHeight*/
const Value V23012_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23012_4 = {S54, 0, &V23012_4_1224, &P23012_5};	/*Property = ScreenPixelsWidth*/
const Profile P23012 = {"23012", &P23012_4};	/*Profile ID - 23012*/
const Value V14339_1_1 = {S2, NULL};	/*Value = True*/
const Property P14339_1 = {S50, 0, &V14339_1_1, NULL};	/*Property = IsMobile*/
const Value V14339_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14339_5 = {S52, 0, &V14339_5_1222, &P14339_1};	/*Property = ScreenPixelsHeight*/
const Value V14339_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14339_4 = {S54, 0, &V14339_4_1224, &P14339_5};	/*Property = ScreenPixelsWidth*/
const Profile P14339 = {"14339", &P14339_4};	/*Profile ID - 14339*/
const Value V17995_1_1 = {S2, NULL};	/*Value = True*/
const Property P17995_1 = {S50, 0, &V17995_1_1, NULL};	/*Property = IsMobile*/
const Value V17995_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17995_5 = {S52, 0, &V17995_5_1269, &P17995_1};	/*Property = ScreenPixelsHeight*/
const Value V17995_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17995_4 = {S54, 0, &V17995_4_1229, &P17995_5};	/*Property = ScreenPixelsWidth*/
const Profile P17995 = {"17995", &P17995_4};	/*Profile ID - 17995*/
const Profile P17297 = {"17297", NULL};	/*Profile ID - 17297*/
const Value V14609_1_1 = {S2, NULL};	/*Value = True*/
const Property P14609_1 = {S50, 0, &V14609_1_1, NULL};	/*Property = IsMobile*/
const Value V14609_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14609_5 = {S52, 0, &V14609_5_1294, &P14609_1};	/*Property = ScreenPixelsHeight*/
const Value V14609_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14609_4 = {S54, 0, &V14609_4_1296, &P14609_5};	/*Property = ScreenPixelsWidth*/
const Profile P14609 = {"14609", &P14609_4};	/*Profile ID - 14609*/
const Value V14726_1_1 = {S2, NULL};	/*Value = True*/
const Property P14726_1 = {S50, 0, &V14726_1_1, NULL};	/*Property = IsMobile*/
const Value V14726_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14726_5 = {S52, 0, &V14726_5_1227, &P14726_1};	/*Property = ScreenPixelsHeight*/
const Value V14726_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14726_4 = {S54, 0, &V14726_4_1229, &P14726_5};	/*Property = ScreenPixelsWidth*/
const Profile P14726 = {"14726", &P14726_4};	/*Profile ID - 14726*/
const Value V14611_1_1 = {S2, NULL};	/*Value = True*/
const Property P14611_1 = {S50, 0, &V14611_1_1, NULL};	/*Property = IsMobile*/
const Value V14611_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14611_5 = {S52, 0, &V14611_5_1277, &P14611_1};	/*Property = ScreenPixelsHeight*/
const Value V14611_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14611_4 = {S54, 0, &V14611_4_1279, &P14611_5};	/*Property = ScreenPixelsWidth*/
const Profile P14611 = {"14611", &P14611_4};	/*Profile ID - 14611*/
const Value V14645_1_1 = {S2, NULL};	/*Value = True*/
const Property P14645_1 = {S50, 0, &V14645_1_1, NULL};	/*Property = IsMobile*/
const Value V14645_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P14645_5 = {S52, 0, &V14645_5_1269, &P14645_1};	/*Property = ScreenPixelsHeight*/
const Value V14645_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14645_4 = {S54, 0, &V14645_4_1229, &P14645_5};	/*Property = ScreenPixelsWidth*/
const Profile P14645 = {"14645", &P14645_4};	/*Profile ID - 14645*/
const Value V14614_1_1 = {S2, NULL};	/*Value = True*/
const Property P14614_1 = {S50, 0, &V14614_1_1, NULL};	/*Property = IsMobile*/
const Value V14614_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14614_5 = {S52, 0, &V14614_5_1293, &P14614_1};	/*Property = ScreenPixelsHeight*/
const Value V14614_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14614_4 = {S54, 0, &V14614_4_1296, &P14614_5};	/*Property = ScreenPixelsWidth*/
const Profile P14614 = {"14614", &P14614_4};	/*Profile ID - 14614*/
const Value V12307_1_1 = {S2, NULL};	/*Value = True*/
const Property P12307_1 = {S50, 0, &V12307_1_1, NULL};	/*Property = IsMobile*/
const Value V12307_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12307_5 = {S52, 0, &V12307_5_1294, &P12307_1};	/*Property = ScreenPixelsHeight*/
const Value V12307_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12307_4 = {S54, 0, &V12307_4_1296, &P12307_5};	/*Property = ScreenPixelsWidth*/
const Profile P12307 = {"12307", &P12307_4};	/*Profile ID - 12307*/
const Value V14640_1_1 = {S2, NULL};	/*Value = True*/
const Property P14640_1 = {S50, 0, &V14640_1_1, NULL};	/*Property = IsMobile*/
const Value V14640_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14640_5 = {S52, 0, &V14640_5_1227, &P14640_1};	/*Property = ScreenPixelsHeight*/
const Value V14640_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14640_4 = {S54, 0, &V14640_4_1229, &P14640_5};	/*Property = ScreenPixelsWidth*/
const Profile P14640 = {"14640", &P14640_4};	/*Profile ID - 14640*/
const Value V14616_1_1 = {S2, NULL};	/*Value = True*/
const Property P14616_1 = {S50, 0, &V14616_1_1, NULL};	/*Property = IsMobile*/
const Value V14616_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14616_5 = {S52, 0, &V14616_5_1222, &P14616_1};	/*Property = ScreenPixelsHeight*/
const Value V14616_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14616_4 = {S54, 0, &V14616_4_1224, &P14616_5};	/*Property = ScreenPixelsWidth*/
const Profile P14616 = {"14616", &P14616_4};	/*Profile ID - 14616*/
const Value V14617_1_1 = {S2, NULL};	/*Value = True*/
const Property P14617_1 = {S50, 0, &V14617_1_1, NULL};	/*Property = IsMobile*/
const Value V14617_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P14617_5 = {S52, 0, &V14617_5_1269, &P14617_1};	/*Property = ScreenPixelsHeight*/
const Value V14617_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14617_4 = {S54, 0, &V14617_4_1229, &P14617_5};	/*Property = ScreenPixelsWidth*/
const Profile P14617 = {"14617", &P14617_4};	/*Profile ID - 14617*/
const Value V14618_1_1 = {S2, NULL};	/*Value = True*/
const Property P14618_1 = {S50, 0, &V14618_1_1, NULL};	/*Property = IsMobile*/
const Value V14618_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14618_5 = {S52, 0, &V14618_5_1293, &P14618_1};	/*Property = ScreenPixelsHeight*/
const Value V14618_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14618_4 = {S54, 0, &V14618_4_1296, &P14618_5};	/*Property = ScreenPixelsWidth*/
const Profile P14618 = {"14618", &P14618_4};	/*Profile ID - 14618*/
const Value V14621_1_1 = {S2, NULL};	/*Value = True*/
const Property P14621_1 = {S50, 0, &V14621_1_1, NULL};	/*Property = IsMobile*/
const Value V14621_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14621_5 = {S52, 0, &V14621_5_1294, &P14621_1};	/*Property = ScreenPixelsHeight*/
const Value V14621_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14621_4 = {S54, 0, &V14621_4_1296, &P14621_5};	/*Property = ScreenPixelsWidth*/
const Profile P14621 = {"14621", &P14621_4};	/*Profile ID - 14621*/
const Value V14630_1_1 = {S2, NULL};	/*Value = True*/
const Property P14630_1 = {S50, 0, &V14630_1_1, NULL};	/*Property = IsMobile*/
const Value V14630_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14630_5 = {S52, 0, &V14630_5_1227, &P14630_1};	/*Property = ScreenPixelsHeight*/
const Value V14630_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14630_4 = {S54, 0, &V14630_4_1229, &P14630_5};	/*Property = ScreenPixelsWidth*/
const Profile P14630 = {"14630", &P14630_4};	/*Profile ID - 14630*/
const Value V14632_1_1 = {S2, NULL};	/*Value = True*/
const Property P14632_1 = {S50, 0, &V14632_1_1, NULL};	/*Property = IsMobile*/
const Value V14632_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14632_5 = {S52, 0, &V14632_5_1227, &P14632_1};	/*Property = ScreenPixelsHeight*/
const Value V14632_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14632_4 = {S54, 0, &V14632_4_1229, &P14632_5};	/*Property = ScreenPixelsWidth*/
const Profile P14632 = {"14632", &P14632_4};	/*Profile ID - 14632*/
const Value V14764_1_1 = {S2, NULL};	/*Value = True*/
const Property P14764_1 = {S50, 0, &V14764_1_1, NULL};	/*Property = IsMobile*/
const Value V14764_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P14764_5 = {S52, 0, &V14764_5_1275, &P14764_1};	/*Property = ScreenPixelsHeight*/
const Value V14764_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P14764_4 = {S54, 0, &V14764_4_1278, &P14764_5};	/*Property = ScreenPixelsWidth*/
const Profile P14764 = {"14764", &P14764_4};	/*Profile ID - 14764*/
const Value V14634_1_1 = {S2, NULL};	/*Value = True*/
const Property P14634_1 = {S50, 0, &V14634_1_1, NULL};	/*Property = IsMobile*/
const Value V14634_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14634_5 = {S52, 0, &V14634_5_1227, &P14634_1};	/*Property = ScreenPixelsHeight*/
const Value V14634_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14634_4 = {S54, 0, &V14634_4_1229, &P14634_5};	/*Property = ScreenPixelsWidth*/
const Profile P14634 = {"14634", &P14634_4};	/*Profile ID - 14634*/
const Value V14738_1_1 = {S2, NULL};	/*Value = True*/
const Property P14738_1 = {S50, 0, &V14738_1_1, NULL};	/*Property = IsMobile*/
const Value V14738_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14738_5 = {S52, 0, &V14738_5_1227, &P14738_1};	/*Property = ScreenPixelsHeight*/
const Value V14738_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14738_4 = {S54, 0, &V14738_4_1229, &P14738_5};	/*Property = ScreenPixelsWidth*/
const Profile P14738 = {"14738", &P14738_4};	/*Profile ID - 14738*/
const Value V14638_1_1 = {S2, NULL};	/*Value = True*/
const Property P14638_1 = {S50, 0, &V14638_1_1, NULL};	/*Property = IsMobile*/
const Value V14638_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14638_5 = {S52, 0, &V14638_5_1233, &P14638_1};	/*Property = ScreenPixelsHeight*/
const Value V14638_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14638_4 = {S54, 0, &V14638_4_1235, &P14638_5};	/*Property = ScreenPixelsWidth*/
const Profile P14638 = {"14638", &P14638_4};	/*Profile ID - 14638*/
const Value V14637_1_1 = {S2, NULL};	/*Value = True*/
const Property P14637_1 = {S50, 0, &V14637_1_1, NULL};	/*Property = IsMobile*/
const Value V14637_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14637_5 = {S52, 0, &V14637_5_1227, &P14637_1};	/*Property = ScreenPixelsHeight*/
const Value V14637_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14637_4 = {S54, 0, &V14637_4_1229, &P14637_5};	/*Property = ScreenPixelsWidth*/
const Profile P14637 = {"14637", &P14637_4};	/*Profile ID - 14637*/
const Value V13868_1_1 = {S2, NULL};	/*Value = True*/
const Property P13868_1 = {S50, 0, &V13868_1_1, NULL};	/*Property = IsMobile*/
const Value V13868_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13868_5 = {S52, 0, &V13868_5_1227, &P13868_1};	/*Property = ScreenPixelsHeight*/
const Value V13868_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13868_4 = {S54, 0, &V13868_4_1229, &P13868_5};	/*Property = ScreenPixelsWidth*/
const Profile P13868 = {"13868", &P13868_4};	/*Profile ID - 13868*/
const Value V14642_1_1 = {S2, NULL};	/*Value = True*/
const Property P14642_1 = {S50, 0, &V14642_1_1, NULL};	/*Property = IsMobile*/
const Value V14642_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14642_5 = {S52, 0, &V14642_5_1277, &P14642_1};	/*Property = ScreenPixelsHeight*/
const Value V14642_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14642_4 = {S54, 0, &V14642_4_1279, &P14642_5};	/*Property = ScreenPixelsWidth*/
const Profile P14642 = {"14642", &P14642_4};	/*Profile ID - 14642*/
const Value V13230_1_1 = {S2, NULL};	/*Value = True*/
const Property P13230_1 = {S50, 0, &V13230_1_1, NULL};	/*Property = IsMobile*/
const Value V13230_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13230_5 = {S52, 0, &V13230_5_1233, &P13230_1};	/*Property = ScreenPixelsHeight*/
const Value V13230_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13230_4 = {S54, 0, &V13230_4_1235, &P13230_5};	/*Property = ScreenPixelsWidth*/
const Profile P13230 = {"13230", &P13230_4};	/*Profile ID - 13230*/
const Value V17720_1_1 = {S2, NULL};	/*Value = True*/
const Property P17720_1 = {S50, 0, &V17720_1_1, NULL};	/*Property = IsMobile*/
const Value V17720_5_37540 = {S105, NULL};	/*Value = 427*/
const Property P17720_5 = {S52, 0, &V17720_5_37540, &P17720_1};	/*Property = ScreenPixelsHeight*/
const Value V17720_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17720_4 = {S54, 0, &V17720_4_1224, &P17720_5};	/*Property = ScreenPixelsWidth*/
const Profile P17720 = {"17720", &P17720_4};	/*Profile ID - 17720*/
const Value V14646_1_1 = {S2, NULL};	/*Value = True*/
const Property P14646_1 = {S50, 0, &V14646_1_1, NULL};	/*Property = IsMobile*/
const Value V14646_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P14646_5 = {S52, 0, &V14646_5_1269, &P14646_1};	/*Property = ScreenPixelsHeight*/
const Value V14646_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14646_4 = {S54, 0, &V14646_4_1229, &P14646_5};	/*Property = ScreenPixelsWidth*/
const Profile P14646 = {"14646", &P14646_4};	/*Profile ID - 14646*/
const Value V14648_1_1 = {S2, NULL};	/*Value = True*/
const Property P14648_1 = {S50, 0, &V14648_1_1, NULL};	/*Property = IsMobile*/
const Value V14648_5_4286 = {S73, NULL};	/*Value = 432*/
const Property P14648_5 = {S52, 0, &V14648_5_4286, &P14648_1};	/*Property = ScreenPixelsHeight*/
const Value V14648_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14648_4 = {S54, 0, &V14648_4_1224, &P14648_5};	/*Property = ScreenPixelsWidth*/
const Profile P14648 = {"14648", &P14648_4};	/*Profile ID - 14648*/
const Value V14644_1_1 = {S2, NULL};	/*Value = True*/
const Property P14644_1 = {S50, 0, &V14644_1_1, NULL};	/*Property = IsMobile*/
const Value V14644_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P14644_5 = {S52, 0, &V14644_5_1269, &P14644_1};	/*Property = ScreenPixelsHeight*/
const Value V14644_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14644_4 = {S54, 0, &V14644_4_1229, &P14644_5};	/*Property = ScreenPixelsWidth*/
const Profile P14644 = {"14644", &P14644_4};	/*Profile ID - 14644*/
const Value V14649_1_1 = {S2, NULL};	/*Value = True*/
const Property P14649_1 = {S50, 0, &V14649_1_1, NULL};	/*Property = IsMobile*/
const Value V14649_5_37540 = {S105, NULL};	/*Value = 427*/
const Property P14649_5 = {S52, 0, &V14649_5_37540, &P14649_1};	/*Property = ScreenPixelsHeight*/
const Value V14649_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14649_4 = {S54, 0, &V14649_4_1224, &P14649_5};	/*Property = ScreenPixelsWidth*/
const Profile P14649 = {"14649", &P14649_4};	/*Profile ID - 14649*/
const Value V17570_1_1 = {S2, NULL};	/*Value = True*/
const Property P17570_1 = {S50, 0, &V17570_1_1, NULL};	/*Property = IsMobile*/
const Value V17570_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17570_5 = {S52, 0, &V17570_5_1269, &P17570_1};	/*Property = ScreenPixelsHeight*/
const Value V17570_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17570_4 = {S54, 0, &V17570_4_1229, &P17570_5};	/*Property = ScreenPixelsWidth*/
const Profile P17570 = {"17570", &P17570_4};	/*Profile ID - 17570*/
const Profile P19941 = {"19941", NULL};	/*Profile ID - 19941*/
const Value V17760_1_1 = {S2, NULL};	/*Value = True*/
const Property P17760_1 = {S50, 0, &V17760_1_1, NULL};	/*Property = IsMobile*/
const Value V17760_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17760_5 = {S52, 0, &V17760_5_1269, &P17760_1};	/*Property = ScreenPixelsHeight*/
const Value V17760_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17760_4 = {S54, 0, &V17760_4_1229, &P17760_5};	/*Property = ScreenPixelsWidth*/
const Profile P17760 = {"17760", &P17760_4};	/*Profile ID - 17760*/
const Value V14651_1_1 = {S2, NULL};	/*Value = True*/
const Property P14651_1 = {S50, 0, &V14651_1_1, NULL};	/*Property = IsMobile*/
const Value V14651_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14651_5 = {S52, 0, &V14651_5_1227, &P14651_1};	/*Property = ScreenPixelsHeight*/
const Value V14651_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14651_4 = {S54, 0, &V14651_4_1229, &P14651_5};	/*Property = ScreenPixelsWidth*/
const Profile P14651 = {"14651", &P14651_4};	/*Profile ID - 14651*/
const Value V14652_1_1 = {S2, NULL};	/*Value = True*/
const Property P14652_1 = {S50, 0, &V14652_1_1, NULL};	/*Property = IsMobile*/
const Value V14652_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P14652_5 = {S52, 0, &V14652_5_1269, &P14652_1};	/*Property = ScreenPixelsHeight*/
const Value V14652_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14652_4 = {S54, 0, &V14652_4_1229, &P14652_5};	/*Property = ScreenPixelsWidth*/
const Profile P14652 = {"14652", &P14652_4};	/*Profile ID - 14652*/
const Value V17758_1_1 = {S2, NULL};	/*Value = True*/
const Property P17758_1 = {S50, 0, &V17758_1_1, NULL};	/*Property = IsMobile*/
const Value V17758_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17758_5 = {S52, 0, &V17758_5_1269, &P17758_1};	/*Property = ScreenPixelsHeight*/
const Value V17758_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17758_4 = {S54, 0, &V17758_4_1229, &P17758_5};	/*Property = ScreenPixelsWidth*/
const Profile P17758 = {"17758", &P17758_4};	/*Profile ID - 17758*/
const Value V14762_1_1 = {S2, NULL};	/*Value = True*/
const Property P14762_1 = {S50, 0, &V14762_1_1, NULL};	/*Property = IsMobile*/
const Value V14762_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14762_5 = {S52, 0, &V14762_5_1222, &P14762_1};	/*Property = ScreenPixelsHeight*/
const Value V14762_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14762_4 = {S54, 0, &V14762_4_1224, &P14762_5};	/*Property = ScreenPixelsWidth*/
const Profile P14762 = {"14762", &P14762_4};	/*Profile ID - 14762*/
const Value V14655_1_1 = {S2, NULL};	/*Value = True*/
const Property P14655_1 = {S50, 0, &V14655_1_1, NULL};	/*Property = IsMobile*/
const Value V14655_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P14655_5 = {S52, 0, &V14655_5_1269, &P14655_1};	/*Property = ScreenPixelsHeight*/
const Value V14655_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14655_4 = {S54, 0, &V14655_4_1229, &P14655_5};	/*Property = ScreenPixelsWidth*/
const Profile P14655 = {"14655", &P14655_4};	/*Profile ID - 14655*/
const Value V14551_1_1 = {S2, NULL};	/*Value = True*/
const Property P14551_1 = {S50, 0, &V14551_1_1, NULL};	/*Property = IsMobile*/
const Value V14551_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14551_5 = {S52, 0, &V14551_5_1380, &P14551_1};	/*Property = ScreenPixelsHeight*/
const Value V14551_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14551_4 = {S54, 0, &V14551_4_1279, &P14551_5};	/*Property = ScreenPixelsWidth*/
const Profile P14551 = {"14551", &P14551_4};	/*Profile ID - 14551*/
const Value V14656_1_1 = {S2, NULL};	/*Value = True*/
const Property P14656_1 = {S50, 0, &V14656_1_1, NULL};	/*Property = IsMobile*/
const Value V14656_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14656_5 = {S52, 0, &V14656_5_1227, &P14656_1};	/*Property = ScreenPixelsHeight*/
const Value V14656_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14656_4 = {S54, 0, &V14656_4_1229, &P14656_5};	/*Property = ScreenPixelsWidth*/
const Profile P14656 = {"14656", &P14656_4};	/*Profile ID - 14656*/
const Value V13679_1_1 = {S2, NULL};	/*Value = True*/
const Property P13679_1 = {S50, 0, &V13679_1_1, NULL};	/*Property = IsMobile*/
const Value V13679_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13679_5 = {S52, 0, &V13679_5_1222, &P13679_1};	/*Property = ScreenPixelsHeight*/
const Value V13679_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13679_4 = {S54, 0, &V13679_4_1224, &P13679_5};	/*Property = ScreenPixelsWidth*/
const Profile P13679 = {"13679", &P13679_4};	/*Profile ID - 13679*/
const Value V17567_1_1 = {S2, NULL};	/*Value = True*/
const Property P17567_1 = {S50, 0, &V17567_1_1, NULL};	/*Property = IsMobile*/
const Value V17567_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17567_5 = {S52, 0, &V17567_5_1227, &P17567_1};	/*Property = ScreenPixelsHeight*/
const Value V17567_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17567_4 = {S54, 0, &V17567_4_1229, &P17567_5};	/*Property = ScreenPixelsWidth*/
const Profile P17567 = {"17567", &P17567_4};	/*Profile ID - 17567*/
const Value V14697_1_1 = {S2, NULL};	/*Value = True*/
const Property P14697_1 = {S50, 0, &V14697_1_1, NULL};	/*Property = IsMobile*/
const Value V14697_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14697_5 = {S52, 0, &V14697_5_1227, &P14697_1};	/*Property = ScreenPixelsHeight*/
const Value V14697_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14697_4 = {S54, 0, &V14697_4_1229, &P14697_5};	/*Property = ScreenPixelsWidth*/
const Profile P14697 = {"14697", &P14697_4};	/*Profile ID - 14697*/
const Value V12513_1_1 = {S2, NULL};	/*Value = True*/
const Property P12513_1 = {S50, 0, &V12513_1_1, NULL};	/*Property = IsMobile*/
const Value V12513_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12513_5 = {S52, 0, &V12513_5_1222, &P12513_1};	/*Property = ScreenPixelsHeight*/
const Value V12513_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12513_4 = {S54, 0, &V12513_4_1224, &P12513_5};	/*Property = ScreenPixelsWidth*/
const Profile P12513 = {"12513", &P12513_4};	/*Profile ID - 12513*/
const Value V14700_1_1 = {S2, NULL};	/*Value = True*/
const Property P14700_1 = {S50, 0, &V14700_1_1, NULL};	/*Property = IsMobile*/
const Value V14700_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P14700_5 = {S52, 0, &V14700_5_1269, &P14700_1};	/*Property = ScreenPixelsHeight*/
const Value V14700_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14700_4 = {S54, 0, &V14700_4_1229, &P14700_5};	/*Property = ScreenPixelsWidth*/
const Profile P14700 = {"14700", &P14700_4};	/*Profile ID - 14700*/
const Value V18008_1_1 = {S2, NULL};	/*Value = True*/
const Property P18008_1 = {S50, 0, &V18008_1_1, NULL};	/*Property = IsMobile*/
const Value V18008_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18008_5 = {S52, 0, &V18008_5_1222, &P18008_1};	/*Property = ScreenPixelsHeight*/
const Value V18008_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18008_4 = {S54, 0, &V18008_4_1224, &P18008_5};	/*Property = ScreenPixelsWidth*/
const Profile P18008 = {"18008", &P18008_4};	/*Profile ID - 18008*/
const Value V14554_1_1 = {S2, NULL};	/*Value = True*/
const Property P14554_1 = {S50, 0, &V14554_1_1, NULL};	/*Property = IsMobile*/
const Value V14554_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14554_5 = {S52, 0, &V14554_5_1304, &P14554_1};	/*Property = ScreenPixelsHeight*/
const Value V14554_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14554_4 = {S54, 0, &V14554_4_1224, &P14554_5};	/*Property = ScreenPixelsWidth*/
const Profile P14554 = {"14554", &P14554_4};	/*Profile ID - 14554*/
const Value V14381_1_1 = {S2, NULL};	/*Value = True*/
const Property P14381_1 = {S50, 0, &V14381_1_1, NULL};	/*Property = IsMobile*/
const Value V14381_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14381_5 = {S52, 0, &V14381_5_1222, &P14381_1};	/*Property = ScreenPixelsHeight*/
const Value V14381_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14381_4 = {S54, 0, &V14381_4_1224, &P14381_5};	/*Property = ScreenPixelsWidth*/
const Profile P14381 = {"14381", &P14381_4};	/*Profile ID - 14381*/
const Value V14719_1_1 = {S2, NULL};	/*Value = True*/
const Property P14719_1 = {S50, 0, &V14719_1_1, NULL};	/*Property = IsMobile*/
const Value V14719_5_37453 = {S91, NULL};	/*Value = 65*/
const Property P14719_5 = {S52, 0, &V14719_5_37453, &P14719_1};	/*Property = ScreenPixelsHeight*/
const Value V14719_4_31956 = {S92, NULL};	/*Value = 96*/
const Property P14719_4 = {S54, 0, &V14719_4_31956, &P14719_5};	/*Property = ScreenPixelsWidth*/
const Profile P14719 = {"14719", &P14719_4};	/*Profile ID - 14719*/
const Value V14721_1_1 = {S2, NULL};	/*Value = True*/
const Property P14721_1 = {S50, 0, &V14721_1_1, NULL};	/*Property = IsMobile*/
const Value V14721_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P14721_5 = {S52, 0, &V14721_5_1269, &P14721_1};	/*Property = ScreenPixelsHeight*/
const Value V14721_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14721_4 = {S54, 0, &V14721_4_1229, &P14721_5};	/*Property = ScreenPixelsWidth*/
const Profile P14721 = {"14721", &P14721_4};	/*Profile ID - 14721*/
const Value V14722_1_1 = {S2, NULL};	/*Value = True*/
const Property P14722_1 = {S50, 0, &V14722_1_1, NULL};	/*Property = IsMobile*/
const Value V14722_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P14722_5 = {S52, 0, &V14722_5_1269, &P14722_1};	/*Property = ScreenPixelsHeight*/
const Value V14722_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14722_4 = {S54, 0, &V14722_4_1229, &P14722_5};	/*Property = ScreenPixelsWidth*/
const Profile P14722 = {"14722", &P14722_4};	/*Profile ID - 14722*/
const Value V14725_1_1 = {S2, NULL};	/*Value = True*/
const Property P14725_1 = {S50, 0, &V14725_1_1, NULL};	/*Property = IsMobile*/
const Value V14725_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P14725_5 = {S52, 0, &V14725_5_1269, &P14725_1};	/*Property = ScreenPixelsHeight*/
const Value V14725_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P14725_4 = {S54, 0, &V14725_4_8554, &P14725_5};	/*Property = ScreenPixelsWidth*/
const Profile P14725 = {"14725", &P14725_4};	/*Profile ID - 14725*/
const Value V14868_1_1 = {S2, NULL};	/*Value = True*/
const Property P14868_1 = {S50, 0, &V14868_1_1, NULL};	/*Property = IsMobile*/
const Value V14868_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P14868_5 = {S52, 0, &V14868_5_37, &P14868_1};	/*Property = ScreenPixelsHeight*/
const Value V14868_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P14868_4 = {S54, 0, &V14868_4_36, &P14868_5};	/*Property = ScreenPixelsWidth*/
const Profile P14868 = {"14868", &P14868_4};	/*Profile ID - 14868*/
const Value V14413_1_1 = {S2, NULL};	/*Value = True*/
const Property P14413_1 = {S50, 0, &V14413_1_1, NULL};	/*Property = IsMobile*/
const Value V14413_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14413_5 = {S52, 0, &V14413_5_1380, &P14413_1};	/*Property = ScreenPixelsHeight*/
const Value V14413_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14413_4 = {S54, 0, &V14413_4_1279, &P14413_5};	/*Property = ScreenPixelsWidth*/
const Profile P14413 = {"14413", &P14413_4};	/*Profile ID - 14413*/
const Value V22570_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_279 = {S1, 0, &V22570_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22570_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_282 = {S3, 0, &V22570_282_57155, &P22570_279};	/*Property = BlobBuilder*/
const Value V22570_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_350 = {S4, 0, &V22570_350_57446, &P22570_282};	/*Property = Canvas*/
const Value V22570_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_283 = {S5, 0, &V22570_283_57161, &P22570_350};	/*Property = CssBackground*/
const Value V22570_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_284 = {S6, 0, &V22570_284_57164, &P22570_283};	/*Property = CssBorderImage*/
const Value V22570_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_285 = {S7, 0, &V22570_285_57167, &P22570_284};	/*Property = CssCanvas*/
const Value V22570_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_286 = {S8, 0, &V22570_286_57170, &P22570_285};	/*Property = CssColor*/
const Value V22570_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_237 = {S9, 0, &V22570_237_55786, &P22570_286};	/*Property = CssColumn*/
const Value V22570_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_287 = {S10, 0, &V22570_287_57173, &P22570_237};	/*Property = CssFlexbox*/
const Value V22570_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_288 = {S11, 0, &V22570_288_57176, &P22570_287};	/*Property = CssFont*/
const Value V22570_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_289 = {S12, 0, &V22570_289_86899, &P22570_288};	/*Property = CssImages*/
const Value V22570_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_291 = {S13, 0, &V22570_291_57182, &P22570_289};	/*Property = CssMediaQueries*/
const Value V22570_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_292 = {S14, 0, &V22570_292_57185, &P22570_291};	/*Property = CssMinMax*/
const Value V22570_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_293 = {S15, 0, &V22570_293_57188, &P22570_292};	/*Property = CssOverflow*/
const Value V22570_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_294 = {S16, 0, &V22570_294_57191, &P22570_293};	/*Property = CssPosition*/
const Value V22570_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_295 = {S17, 0, &V22570_295_57194, &P22570_294};	/*Property = CssText*/
const Value V22570_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_297 = {S18, 0, &V22570_297_57197, &P22570_295};	/*Property = CssTransforms*/
const Value V22570_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_241 = {S19, 0, &V22570_241_55801, &P22570_297};	/*Property = CssTransitions*/
const Value V22570_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_298 = {S20, 0, &V22570_298_57208, &P22570_241};	/*Property = CssUI*/
const Value V22570_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_300 = {S21, 0, &V22570_300_57217, &P22570_298};	/*Property = DataSet*/
const Value V22570_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_301 = {S22, 0, &V22570_301_57220, &P22570_300};	/*Property = DataUrl*/
const Value V22570_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_302 = {S23, 0, &V22570_302_57223, &P22570_301};	/*Property = DeviceOrientation*/
const Value V22570_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_304 = {S24, 0, &V22570_304_57229, &P22570_302};	/*Property = FileReader*/
const Value V22570_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_305 = {S25, 0, &V22570_305_57232, &P22570_304};	/*Property = FileSaver*/
const Value V22570_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_306 = {S26, 0, &V22570_306_57238, &P22570_305};	/*Property = FileWriter*/
const Value V22570_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_307 = {S27, 0, &V22570_307_57241, &P22570_306};	/*Property = FormData*/
const Value V22570_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_309 = {S28, 0, &V22570_309_57247, &P22570_307};	/*Property = Fullscreen*/
const Value V22570_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_310 = {S29, 0, &V22570_310_57250, &P22570_309};	/*Property = GeoLocation*/
const Value V22570_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_312 = {S30, 0, &V22570_312_57256, &P22570_310};	/*Property = History*/
const Value V22570_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_314 = {S31, 0, &V22570_314_57262, &P22570_312};	/*Property = Html5*/
const Value V22570_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_313 = {S32, 0, &V22570_313_57259, &P22570_314};	/*Property = Html-Media-Capture*/
const Value V22570_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_315 = {S33, 0, &V22570_315_57265, &P22570_313};	/*Property = Iframe*/
const Value V22570_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_316 = {S34, 0, &V22570_316_57268, &P22570_315};	/*Property = IndexedDB*/
const Value V22570_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_317 = {S35, 0, &V22570_317_57271, &P22570_316};	/*Property = Json*/
const Value V22570_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P22570_15 = {S37, 0, &V22570_15_31, &P22570_317};	/*Property = LayoutEngine*/
const Value V22570_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_344 = {S38, 0, &V22570_344_57274, &P22570_15};	/*Property = Masking*/
const Value V22570_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_320 = {S39, 0, &V22570_320_57292, &P22570_344};	/*Property = PostMessage*/
const Value V22570_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_321 = {S40, 0, &V22570_321_57295, &P22570_320};	/*Property = Progress*/
const Value V22570_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_322 = {S41, 0, &V22570_322_57298, &P22570_321};	/*Property = Prompts*/
const Value V22570_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_323 = {S42, 0, &V22570_323_57301, &P22570_322};	/*Property = Selector*/
const Value V22570_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_324 = {S43, 0, &V22570_324_57307, &P22570_323};	/*Property = Svg*/
const Value V22570_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_327 = {S44, 0, &V22570_327_57316, &P22570_324};	/*Property = TouchEvents*/
const Value V22570_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_328 = {S45, 0, &V22570_328_57319, &P22570_327};	/*Property = Track*/
const Value V22570_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_329 = {S46, 0, &V22570_329_57325, &P22570_328};	/*Property = Video*/
const Value V22570_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_330 = {S47, 0, &V22570_330_57328, &P22570_329};	/*Property = Viewport*/
const Value V22570_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_332 = {S48, 0, &V22570_332_57340, &P22570_330};	/*Property = WebWorkers*/
const Value V22570_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22570_333 = {S49, 0, &V22570_333_57343, &P22570_332};	/*Property = Xhr2*/
const Profile P22570 = {"22570", &P22570_333};	/*Profile ID - 22570*/
const Value V14382_1_1 = {S2, NULL};	/*Value = True*/
const Property P14382_1 = {S50, 0, &V14382_1_1, NULL};	/*Property = IsMobile*/
const Value V14382_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14382_5 = {S52, 0, &V14382_5_1294, &P14382_1};	/*Property = ScreenPixelsHeight*/
const Value V14382_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14382_4 = {S54, 0, &V14382_4_1296, &P14382_5};	/*Property = ScreenPixelsWidth*/
const Profile P14382 = {"14382", &P14382_4};	/*Profile ID - 14382*/
const Value V16537_1_1 = {S2, NULL};	/*Value = True*/
const Property P16537_1 = {S50, 0, &V16537_1_1, NULL};	/*Property = IsMobile*/
const Value V16537_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16537_5 = {S52, 0, &V16537_5_1293, &P16537_1};	/*Property = ScreenPixelsHeight*/
const Value V16537_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16537_4 = {S54, 0, &V16537_4_1296, &P16537_5};	/*Property = ScreenPixelsWidth*/
const Profile P16537 = {"16537", &P16537_4};	/*Profile ID - 16537*/
const Value V20062_1_1 = {S2, NULL};	/*Value = True*/
const Property P20062_1 = {S50, 0, &V20062_1_1, NULL};	/*Property = IsMobile*/
const Value V20062_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P20062_5 = {S52, 0, &V20062_5_1275, &P20062_1};	/*Property = ScreenPixelsHeight*/
const Value V20062_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P20062_4 = {S54, 0, &V20062_4_1278, &P20062_5};	/*Property = ScreenPixelsWidth*/
const Profile P20062 = {"20062", &P20062_4};	/*Profile ID - 20062*/
const Value V12531_1_1 = {S2, NULL};	/*Value = True*/
const Property P12531_1 = {S50, 0, &V12531_1_1, NULL};	/*Property = IsMobile*/
const Value V12531_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12531_5 = {S52, 0, &V12531_5_1293, &P12531_1};	/*Property = ScreenPixelsHeight*/
const Value V12531_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12531_4 = {S54, 0, &V12531_4_1296, &P12531_5};	/*Property = ScreenPixelsWidth*/
const Profile P12531 = {"12531", &P12531_4};	/*Profile ID - 12531*/
const Value V17208_279_57147 = {S2, NULL};	/*Value = True*/
const Property P17208_279 = {S1, 0, &V17208_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V17208_282_57153 = {S2, NULL};	/*Value = True*/
const Property P17208_282 = {S3, 0, &V17208_282_57153, &P17208_279};	/*Property = BlobBuilder*/
const Value V17208_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17208_350 = {S4, 0, &V17208_350_57444, &P17208_282};	/*Property = Canvas*/
const Value V17208_283_57160 = {S0, NULL};	/*Value = False*/
const Property P17208_283 = {S5, 0, &V17208_283_57160, &P17208_350};	/*Property = CssBackground*/
const Value V17208_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17208_284 = {S6, 0, &V17208_284_57163, &P17208_283};	/*Property = CssBorderImage*/
const Value V17208_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17208_285 = {S7, 0, &V17208_285_57166, &P17208_284};	/*Property = CssCanvas*/
const Value V17208_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17208_286 = {S8, 0, &V17208_286_57168, &P17208_285};	/*Property = CssColor*/
const Value V17208_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17208_237 = {S9, 0, &V17208_237_55784, &P17208_286};	/*Property = CssColumn*/
const Value V17208_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17208_287 = {S10, 0, &V17208_287_57172, &P17208_237};	/*Property = CssFlexbox*/
const Value V17208_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17208_288 = {S11, 0, &V17208_288_57175, &P17208_287};	/*Property = CssFont*/
const Value V17208_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17208_289 = {S12, 0, &V17208_289_86899, &P17208_288};	/*Property = CssImages*/
const Value V17208_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17208_291 = {S13, 0, &V17208_291_57181, &P17208_289};	/*Property = CssMediaQueries*/
const Value V17208_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17208_292 = {S14, 0, &V17208_292_57183, &P17208_291};	/*Property = CssMinMax*/
const Value V17208_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17208_293 = {S15, 0, &V17208_293_57187, &P17208_292};	/*Property = CssOverflow*/
const Value V17208_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17208_294 = {S16, 0, &V17208_294_57189, &P17208_293};	/*Property = CssPosition*/
const Value V17208_295_57193 = {S0, NULL};	/*Value = False*/
const Property P17208_295 = {S17, 0, &V17208_295_57193, &P17208_294};	/*Property = CssText*/
const Value V17208_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17208_297 = {S18, 0, &V17208_297_57195, &P17208_295};	/*Property = CssTransforms*/
const Value V17208_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17208_241 = {S19, 0, &V17208_241_55799, &P17208_297};	/*Property = CssTransitions*/
const Value V17208_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17208_298 = {S20, 0, &V17208_298_57206, &P17208_241};	/*Property = CssUI*/
const Value V17208_300_57215 = {S2, NULL};	/*Value = True*/
const Property P17208_300 = {S21, 0, &V17208_300_57215, &P17208_298};	/*Property = DataSet*/
const Value V17208_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17208_301 = {S22, 0, &V17208_301_57218, &P17208_300};	/*Property = DataUrl*/
const Value V17208_302_57221 = {S2, NULL};	/*Value = True*/
const Property P17208_302 = {S23, 0, &V17208_302_57221, &P17208_301};	/*Property = DeviceOrientation*/
const Value V17208_304_57227 = {S2, NULL};	/*Value = True*/
const Property P17208_304 = {S24, 0, &V17208_304_57227, &P17208_302};	/*Property = FileReader*/
const Value V17208_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17208_305 = {S25, 0, &V17208_305_57231, &P17208_304};	/*Property = FileSaver*/
const Value V17208_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17208_306 = {S26, 0, &V17208_306_57237, &P17208_305};	/*Property = FileWriter*/
const Value V17208_307_57240 = {S0, NULL};	/*Value = False*/
const Property P17208_307 = {S27, 0, &V17208_307_57240, &P17208_306};	/*Property = FormData*/
const Value V17208_309_57245 = {S2, NULL};	/*Value = True*/
const Property P17208_309 = {S28, 0, &V17208_309_57245, &P17208_307};	/*Property = Fullscreen*/
const Value V17208_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17208_310 = {S29, 0, &V17208_310_57248, &P17208_309};	/*Property = GeoLocation*/
const Value V17208_312_57254 = {S2, NULL};	/*Value = True*/
const Property P17208_312 = {S30, 0, &V17208_312_57254, &P17208_310};	/*Property = History*/
const Value V17208_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17208_314 = {S31, 0, &V17208_314_57261, &P17208_312};	/*Property = Html5*/
const Value V17208_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17208_313 = {S32, 0, &V17208_313_57258, &P17208_314};	/*Property = Html-Media-Capture*/
const Value V17208_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17208_315 = {S33, 0, &V17208_315_57264, &P17208_313};	/*Property = Iframe*/
const Value V17208_316_57266 = {S2, NULL};	/*Value = True*/
const Property P17208_316 = {S34, 0, &V17208_316_57266, &P17208_315};	/*Property = IndexedDB*/
const Value V17208_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17208_317 = {S35, 0, &V17208_317_57269, &P17208_316};	/*Property = Json*/
const Value V17208_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P17208_15 = {S37, 0, &V17208_15_29, &P17208_317};	/*Property = LayoutEngine*/
const Value V17208_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17208_344 = {S38, 0, &V17208_344_57272, &P17208_15};	/*Property = Masking*/
const Value V17208_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17208_320 = {S39, 0, &V17208_320_57290, &P17208_344};	/*Property = PostMessage*/
const Value V17208_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17208_321 = {S40, 0, &V17208_321_57293, &P17208_320};	/*Property = Progress*/
const Value V17208_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17208_322 = {S41, 0, &V17208_322_57296, &P17208_321};	/*Property = Prompts*/
const Value V17208_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17208_323 = {S42, 0, &V17208_323_57299, &P17208_322};	/*Property = Selector*/
const Value V17208_324_57305 = {S2, NULL};	/*Value = True*/
const Property P17208_324 = {S43, 0, &V17208_324_57305, &P17208_323};	/*Property = Svg*/
const Value V17208_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17208_327 = {S44, 0, &V17208_327_57315, &P17208_324};	/*Property = TouchEvents*/
const Value V17208_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17208_328 = {S45, 0, &V17208_328_57318, &P17208_327};	/*Property = Track*/
const Value V17208_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17208_329 = {S46, 0, &V17208_329_57323, &P17208_328};	/*Property = Video*/
const Value V17208_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17208_330 = {S47, 0, &V17208_330_57326, &P17208_329};	/*Property = Viewport*/
const Value V17208_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17208_332 = {S48, 0, &V17208_332_57339, &P17208_330};	/*Property = WebWorkers*/
const Value V17208_333_57341 = {S2, NULL};	/*Value = True*/
const Property P17208_333 = {S49, 0, &V17208_333_57341, &P17208_332};	/*Property = Xhr2*/
const Profile P17208 = {"17208", &P17208_333};	/*Profile ID - 17208*/
const Value V14314_1_1 = {S2, NULL};	/*Value = True*/
const Property P14314_1 = {S50, 0, &V14314_1_1, NULL};	/*Property = IsMobile*/
const Value V14314_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14314_5 = {S52, 0, &V14314_5_1222, &P14314_1};	/*Property = ScreenPixelsHeight*/
const Value V14314_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14314_4 = {S54, 0, &V14314_4_1224, &P14314_5};	/*Property = ScreenPixelsWidth*/
const Profile P14314 = {"14314", &P14314_4};	/*Profile ID - 14314*/
const Value V20502_1_1 = {S2, NULL};	/*Value = True*/
const Property P20502_1 = {S50, 0, &V20502_1_1, NULL};	/*Property = IsMobile*/
const Value V20502_5_64659 = {S111, NULL};	/*Value = 420*/
const Property P20502_5 = {S52, 0, &V20502_5_64659, &P20502_1};	/*Property = ScreenPixelsHeight*/
const Value V20502_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20502_4 = {S54, 0, &V20502_4_1279, &P20502_5};	/*Property = ScreenPixelsWidth*/
const Profile P20502 = {"20502", &P20502_4};	/*Profile ID - 20502*/
const Value V20384_1_2 = {S0, NULL};	/*Value = False*/
const Property P20384_1 = {S50, 0, &V20384_1_2, NULL};	/*Property = IsMobile*/
const Value V20384_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20384_5 = {S52, 0, &V20384_5_37, &P20384_1};	/*Property = ScreenPixelsHeight*/
const Value V20384_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20384_4 = {S54, 0, &V20384_4_36, &P20384_5};	/*Property = ScreenPixelsWidth*/
const Profile P20384 = {"20384", &P20384_4};	/*Profile ID - 20384*/
const Value V17706_1_1 = {S2, NULL};	/*Value = True*/
const Property P17706_1 = {S50, 0, &V17706_1_1, NULL};	/*Property = IsMobile*/
const Value V17706_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17706_5 = {S52, 0, &V17706_5_1293, &P17706_1};	/*Property = ScreenPixelsHeight*/
const Value V17706_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17706_4 = {S54, 0, &V17706_4_1296, &P17706_5};	/*Property = ScreenPixelsWidth*/
const Profile P17706 = {"17706", &P17706_4};	/*Profile ID - 17706*/
const Value V14443_1_1 = {S2, NULL};	/*Value = True*/
const Property P14443_1 = {S50, 0, &V14443_1_1, NULL};	/*Property = IsMobile*/
const Value V14443_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14443_5 = {S52, 0, &V14443_5_1293, &P14443_1};	/*Property = ScreenPixelsHeight*/
const Value V14443_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14443_4 = {S54, 0, &V14443_4_1296, &P14443_5};	/*Property = ScreenPixelsWidth*/
const Profile P14443 = {"14443", &P14443_4};	/*Profile ID - 14443*/
const Value V15124_1_1 = {S2, NULL};	/*Value = True*/
const Property P15124_1 = {S50, 0, &V15124_1_1, NULL};	/*Property = IsMobile*/
const Value V15124_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15124_5 = {S52, 0, &V15124_5_1294, &P15124_1};	/*Property = ScreenPixelsHeight*/
const Value V15124_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15124_4 = {S54, 0, &V15124_4_1296, &P15124_5};	/*Property = ScreenPixelsWidth*/
const Profile P15124 = {"15124", &P15124_4};	/*Profile ID - 15124*/
const Value V17770_1_1 = {S2, NULL};	/*Value = True*/
const Property P17770_1 = {S50, 0, &V17770_1_1, NULL};	/*Property = IsMobile*/
const Value V17770_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17770_5 = {S52, 0, &V17770_5_1227, &P17770_1};	/*Property = ScreenPixelsHeight*/
const Value V17770_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17770_4 = {S54, 0, &V17770_4_1229, &P17770_5};	/*Property = ScreenPixelsWidth*/
const Profile P17770 = {"17770", &P17770_4};	/*Profile ID - 17770*/
const Value V15165_1_1 = {S2, NULL};	/*Value = True*/
const Property P15165_1 = {S50, 0, &V15165_1_1, NULL};	/*Property = IsMobile*/
const Value V15165_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15165_5 = {S52, 0, &V15165_5_1293, &P15165_1};	/*Property = ScreenPixelsHeight*/
const Value V15165_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15165_4 = {S54, 0, &V15165_4_1296, &P15165_5};	/*Property = ScreenPixelsWidth*/
const Profile P15165 = {"15165", &P15165_4};	/*Profile ID - 15165*/
const Value V16988_1_1 = {S2, NULL};	/*Value = True*/
const Property P16988_1 = {S50, 0, &V16988_1_1, NULL};	/*Property = IsMobile*/
const Value V16988_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16988_5 = {S52, 0, &V16988_5_1227, &P16988_1};	/*Property = ScreenPixelsHeight*/
const Value V16988_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16988_4 = {S54, 0, &V16988_4_1229, &P16988_5};	/*Property = ScreenPixelsWidth*/
const Profile P16988 = {"16988", &P16988_4};	/*Profile ID - 16988*/
const Value V17801_1_1 = {S2, NULL};	/*Value = True*/
const Property P17801_1 = {S50, 0, &V17801_1_1, NULL};	/*Property = IsMobile*/
const Value V17801_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17801_5 = {S52, 0, &V17801_5_1222, &P17801_1};	/*Property = ScreenPixelsHeight*/
const Value V17801_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17801_4 = {S54, 0, &V17801_4_1224, &P17801_5};	/*Property = ScreenPixelsWidth*/
const Profile P17801 = {"17801", &P17801_4};	/*Profile ID - 17801*/
const Value V16145_1_1 = {S2, NULL};	/*Value = True*/
const Property P16145_1 = {S50, 0, &V16145_1_1, NULL};	/*Property = IsMobile*/
const Value V16145_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16145_5 = {S52, 0, &V16145_5_1277, &P16145_1};	/*Property = ScreenPixelsHeight*/
const Value V16145_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P16145_4 = {S54, 0, &V16145_4_8554, &P16145_5};	/*Property = ScreenPixelsWidth*/
const Profile P16145 = {"16145", &P16145_4};	/*Profile ID - 16145*/
const Value V12534_1_1 = {S2, NULL};	/*Value = True*/
const Property P12534_1 = {S50, 0, &V12534_1_1, NULL};	/*Property = IsMobile*/
const Value V12534_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12534_5 = {S52, 0, &V12534_5_1380, &P12534_1};	/*Property = ScreenPixelsHeight*/
const Value V12534_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12534_4 = {S54, 0, &V12534_4_1279, &P12534_5};	/*Property = ScreenPixelsWidth*/
const Profile P12534 = {"12534", &P12534_4};	/*Profile ID - 12534*/
const Value V15699_1_1 = {S2, NULL};	/*Value = True*/
const Property P15699_1 = {S50, 0, &V15699_1_1, NULL};	/*Property = IsMobile*/
const Value V15699_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15699_5 = {S52, 0, &V15699_5_1227, &P15699_1};	/*Property = ScreenPixelsHeight*/
const Value V15699_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15699_4 = {S54, 0, &V15699_4_1229, &P15699_5};	/*Property = ScreenPixelsWidth*/
const Profile P15699 = {"15699", &P15699_4};	/*Profile ID - 15699*/
const Value V15481_1_1 = {S2, NULL};	/*Value = True*/
const Property P15481_1 = {S50, 0, &V15481_1_1, NULL};	/*Property = IsMobile*/
const Value V15481_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15481_5 = {S52, 0, &V15481_5_1380, &P15481_1};	/*Property = ScreenPixelsHeight*/
const Value V15481_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15481_4 = {S54, 0, &V15481_4_1224, &P15481_5};	/*Property = ScreenPixelsWidth*/
const Profile P15481 = {"15481", &P15481_4};	/*Profile ID - 15481*/
const Value V12536_1_1 = {S2, NULL};	/*Value = True*/
const Property P12536_1 = {S50, 0, &V12536_1_1, NULL};	/*Property = IsMobile*/
const Value V12536_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12536_5 = {S52, 0, &V12536_5_1294, &P12536_1};	/*Property = ScreenPixelsHeight*/
const Value V12536_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12536_4 = {S54, 0, &V12536_4_1296, &P12536_5};	/*Property = ScreenPixelsWidth*/
const Profile P12536 = {"12536", &P12536_4};	/*Profile ID - 12536*/
const Value V12537_1_1 = {S2, NULL};	/*Value = True*/
const Property P12537_1 = {S50, 0, &V12537_1_1, NULL};	/*Property = IsMobile*/
const Value V12537_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12537_5 = {S52, 0, &V12537_5_1222, &P12537_1};	/*Property = ScreenPixelsHeight*/
const Value V12537_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12537_4 = {S54, 0, &V12537_4_1224, &P12537_5};	/*Property = ScreenPixelsWidth*/
const Profile P12537 = {"12537", &P12537_4};	/*Profile ID - 12537*/
const Value V12539_1_1 = {S2, NULL};	/*Value = True*/
const Property P12539_1 = {S50, 0, &V12539_1_1, NULL};	/*Property = IsMobile*/
const Value V12539_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12539_5 = {S52, 0, &V12539_5_1294, &P12539_1};	/*Property = ScreenPixelsHeight*/
const Value V12539_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12539_4 = {S54, 0, &V12539_4_1296, &P12539_5};	/*Property = ScreenPixelsWidth*/
const Profile P12539 = {"12539", &P12539_4};	/*Profile ID - 12539*/
const Value V12540_1_1 = {S2, NULL};	/*Value = True*/
const Property P12540_1 = {S50, 0, &V12540_1_1, NULL};	/*Property = IsMobile*/
const Value V12540_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12540_5 = {S52, 0, &V12540_5_1222, &P12540_1};	/*Property = ScreenPixelsHeight*/
const Value V12540_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12540_4 = {S54, 0, &V12540_4_1224, &P12540_5};	/*Property = ScreenPixelsWidth*/
const Profile P12540 = {"12540", &P12540_4};	/*Profile ID - 12540*/
const Value V12541_1_1 = {S2, NULL};	/*Value = True*/
const Property P12541_1 = {S50, 0, &V12541_1_1, NULL};	/*Property = IsMobile*/
const Value V12541_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12541_5 = {S52, 0, &V12541_5_1304, &P12541_1};	/*Property = ScreenPixelsHeight*/
const Value V12541_4_43771 = {S80, NULL};	/*Value = 200*/
const Property P12541_4 = {S54, 0, &V12541_4_43771, &P12541_5};	/*Property = ScreenPixelsWidth*/
const Profile P12541 = {"12541", &P12541_4};	/*Profile ID - 12541*/
const Value V12546_1_1 = {S2, NULL};	/*Value = True*/
const Property P12546_1 = {S50, 0, &V12546_1_1, NULL};	/*Property = IsMobile*/
const Value V12546_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12546_5 = {S52, 0, &V12546_5_1293, &P12546_1};	/*Property = ScreenPixelsHeight*/
const Value V12546_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12546_4 = {S54, 0, &V12546_4_1296, &P12546_5};	/*Property = ScreenPixelsWidth*/
const Profile P12546 = {"12546", &P12546_4};	/*Profile ID - 12546*/
const Value V12547_1_1 = {S2, NULL};	/*Value = True*/
const Property P12547_1 = {S50, 0, &V12547_1_1, NULL};	/*Property = IsMobile*/
const Value V12547_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12547_5 = {S52, 0, &V12547_5_1233, &P12547_1};	/*Property = ScreenPixelsHeight*/
const Value V12547_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12547_4 = {S54, 0, &V12547_4_1235, &P12547_5};	/*Property = ScreenPixelsWidth*/
const Profile P12547 = {"12547", &P12547_4};	/*Profile ID - 12547*/
const Value V12549_1_1 = {S2, NULL};	/*Value = True*/
const Property P12549_1 = {S50, 0, &V12549_1_1, NULL};	/*Property = IsMobile*/
const Value V12549_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12549_5 = {S52, 0, &V12549_5_1304, &P12549_1};	/*Property = ScreenPixelsHeight*/
const Value V12549_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12549_4 = {S54, 0, &V12549_4_1224, &P12549_5};	/*Property = ScreenPixelsWidth*/
const Profile P12549 = {"12549", &P12549_4};	/*Profile ID - 12549*/
const Value V12551_1_1 = {S2, NULL};	/*Value = True*/
const Property P12551_1 = {S50, 0, &V12551_1_1, NULL};	/*Property = IsMobile*/
const Value V12551_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12551_5 = {S52, 0, &V12551_5_1222, &P12551_1};	/*Property = ScreenPixelsHeight*/
const Value V12551_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12551_4 = {S54, 0, &V12551_4_1224, &P12551_5};	/*Property = ScreenPixelsWidth*/
const Profile P12551 = {"12551", &P12551_4};	/*Profile ID - 12551*/
const Value V12553_1_1 = {S2, NULL};	/*Value = True*/
const Property P12553_1 = {S50, 0, &V12553_1_1, NULL};	/*Property = IsMobile*/
const Value V12553_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P12553_5 = {S52, 0, &V12553_5_1333, &P12553_1};	/*Property = ScreenPixelsHeight*/
const Value V12553_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P12553_4 = {S54, 0, &V12553_4_1335, &P12553_5};	/*Property = ScreenPixelsWidth*/
const Profile P12553 = {"12553", &P12553_4};	/*Profile ID - 12553*/
const Value V15312_1_1 = {S2, NULL};	/*Value = True*/
const Property P15312_1 = {S50, 0, &V15312_1_1, NULL};	/*Property = IsMobile*/
const Value V15312_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15312_5 = {S52, 0, &V15312_5_1222, &P15312_1};	/*Property = ScreenPixelsHeight*/
const Value V15312_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15312_4 = {S54, 0, &V15312_4_1224, &P15312_5};	/*Property = ScreenPixelsWidth*/
const Profile P15312 = {"15312", &P15312_4};	/*Profile ID - 15312*/
const Value V12563_1_1 = {S2, NULL};	/*Value = True*/
const Property P12563_1 = {S50, 0, &V12563_1_1, NULL};	/*Property = IsMobile*/
const Value V12563_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12563_5 = {S52, 0, &V12563_5_1304, &P12563_1};	/*Property = ScreenPixelsHeight*/
const Value V12563_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12563_4 = {S54, 0, &V12563_4_1224, &P12563_5};	/*Property = ScreenPixelsWidth*/
const Profile P12563 = {"12563", &P12563_4};	/*Profile ID - 12563*/
const Value V12567_1_1 = {S2, NULL};	/*Value = True*/
const Property P12567_1 = {S50, 0, &V12567_1_1, NULL};	/*Property = IsMobile*/
const Value V12567_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12567_5 = {S52, 0, &V12567_5_1222, &P12567_1};	/*Property = ScreenPixelsHeight*/
const Value V12567_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12567_4 = {S54, 0, &V12567_4_1224, &P12567_5};	/*Property = ScreenPixelsWidth*/
const Profile P12567 = {"12567", &P12567_4};	/*Profile ID - 12567*/
const Value V12556_1_1 = {S2, NULL};	/*Value = True*/
const Property P12556_1 = {S50, 0, &V12556_1_1, NULL};	/*Property = IsMobile*/
const Value V12556_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12556_5 = {S52, 0, &V12556_5_1293, &P12556_1};	/*Property = ScreenPixelsHeight*/
const Value V12556_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12556_4 = {S54, 0, &V12556_4_1296, &P12556_5};	/*Property = ScreenPixelsWidth*/
const Profile P12556 = {"12556", &P12556_4};	/*Profile ID - 12556*/
const Value V12569_1_1 = {S2, NULL};	/*Value = True*/
const Property P12569_1 = {S50, 0, &V12569_1_1, NULL};	/*Property = IsMobile*/
const Value V12569_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12569_5 = {S52, 0, &V12569_5_1293, &P12569_1};	/*Property = ScreenPixelsHeight*/
const Value V12569_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12569_4 = {S54, 0, &V12569_4_1296, &P12569_5};	/*Property = ScreenPixelsWidth*/
const Profile P12569 = {"12569", &P12569_4};	/*Profile ID - 12569*/
const Value V16378_1_1 = {S2, NULL};	/*Value = True*/
const Property P16378_1 = {S50, 0, &V16378_1_1, NULL};	/*Property = IsMobile*/
const Value V16378_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16378_5 = {S52, 0, &V16378_5_1233, &P16378_1};	/*Property = ScreenPixelsHeight*/
const Value V16378_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16378_4 = {S54, 0, &V16378_4_1235, &P16378_5};	/*Property = ScreenPixelsWidth*/
const Profile P16378 = {"16378", &P16378_4};	/*Profile ID - 16378*/
const Value V14440_1_1 = {S2, NULL};	/*Value = True*/
const Property P14440_1 = {S50, 0, &V14440_1_1, NULL};	/*Property = IsMobile*/
const Value V14440_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14440_5 = {S52, 0, &V14440_5_1294, &P14440_1};	/*Property = ScreenPixelsHeight*/
const Value V14440_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14440_4 = {S54, 0, &V14440_4_1296, &P14440_5};	/*Property = ScreenPixelsWidth*/
const Profile P14440 = {"14440", &P14440_4};	/*Profile ID - 14440*/
const Value V12529_1_1 = {S2, NULL};	/*Value = True*/
const Property P12529_1 = {S50, 0, &V12529_1_1, NULL};	/*Property = IsMobile*/
const Value V12529_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12529_5 = {S52, 0, &V12529_5_1233, &P12529_1};	/*Property = ScreenPixelsHeight*/
const Value V12529_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12529_4 = {S54, 0, &V12529_4_1235, &P12529_5};	/*Property = ScreenPixelsWidth*/
const Profile P12529 = {"12529", &P12529_4};	/*Profile ID - 12529*/
const Value V12574_1_1 = {S2, NULL};	/*Value = True*/
const Property P12574_1 = {S50, 0, &V12574_1_1, NULL};	/*Property = IsMobile*/
const Value V12574_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12574_5 = {S52, 0, &V12574_5_1222, &P12574_1};	/*Property = ScreenPixelsHeight*/
const Value V12574_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12574_4 = {S54, 0, &V12574_4_1224, &P12574_5};	/*Property = ScreenPixelsWidth*/
const Profile P12574 = {"12574", &P12574_4};	/*Profile ID - 12574*/
const Value V12576_1_1 = {S2, NULL};	/*Value = True*/
const Property P12576_1 = {S50, 0, &V12576_1_1, NULL};	/*Property = IsMobile*/
const Value V12576_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12576_5 = {S52, 0, &V12576_5_1222, &P12576_1};	/*Property = ScreenPixelsHeight*/
const Value V12576_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12576_4 = {S54, 0, &V12576_4_1224, &P12576_5};	/*Property = ScreenPixelsWidth*/
const Profile P12576 = {"12576", &P12576_4};	/*Profile ID - 12576*/
const Value V12575_1_1 = {S2, NULL};	/*Value = True*/
const Property P12575_1 = {S50, 0, &V12575_1_1, NULL};	/*Property = IsMobile*/
const Value V12575_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12575_5 = {S52, 0, &V12575_5_1227, &P12575_1};	/*Property = ScreenPixelsHeight*/
const Value V12575_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12575_4 = {S54, 0, &V12575_4_1229, &P12575_5};	/*Property = ScreenPixelsWidth*/
const Profile P12575 = {"12575", &P12575_4};	/*Profile ID - 12575*/
const Value V12577_1_1 = {S2, NULL};	/*Value = True*/
const Property P12577_1 = {S50, 0, &V12577_1_1, NULL};	/*Property = IsMobile*/
const Value V12577_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12577_5 = {S52, 0, &V12577_5_1233, &P12577_1};	/*Property = ScreenPixelsHeight*/
const Value V12577_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12577_4 = {S54, 0, &V12577_4_1235, &P12577_5};	/*Property = ScreenPixelsWidth*/
const Profile P12577 = {"12577", &P12577_4};	/*Profile ID - 12577*/
const Value V12578_1_1 = {S2, NULL};	/*Value = True*/
const Property P12578_1 = {S50, 0, &V12578_1_1, NULL};	/*Property = IsMobile*/
const Value V12578_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12578_5 = {S52, 0, &V12578_5_1233, &P12578_1};	/*Property = ScreenPixelsHeight*/
const Value V12578_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12578_4 = {S54, 0, &V12578_4_1235, &P12578_5};	/*Property = ScreenPixelsWidth*/
const Profile P12578 = {"12578", &P12578_4};	/*Profile ID - 12578*/
const Value V12582_1_1 = {S2, NULL};	/*Value = True*/
const Property P12582_1 = {S50, 0, &V12582_1_1, NULL};	/*Property = IsMobile*/
const Value V12582_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12582_5 = {S52, 0, &V12582_5_1277, &P12582_1};	/*Property = ScreenPixelsHeight*/
const Value V12582_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12582_4 = {S54, 0, &V12582_4_1279, &P12582_5};	/*Property = ScreenPixelsWidth*/
const Profile P12582 = {"12582", &P12582_4};	/*Profile ID - 12582*/
const Value V12585_1_1 = {S2, NULL};	/*Value = True*/
const Property P12585_1 = {S50, 0, &V12585_1_1, NULL};	/*Property = IsMobile*/
const Value V12585_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12585_5 = {S52, 0, &V12585_5_1293, &P12585_1};	/*Property = ScreenPixelsHeight*/
const Value V12585_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12585_4 = {S54, 0, &V12585_4_1296, &P12585_5};	/*Property = ScreenPixelsWidth*/
const Profile P12585 = {"12585", &P12585_4};	/*Profile ID - 12585*/
const Value V24375_1_1 = {S2, NULL};	/*Value = True*/
const Property P24375_1 = {S50, 0, &V24375_1_1, NULL};	/*Property = IsMobile*/
const Value V24375_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P24375_5 = {S52, 0, &V24375_5_1293, &P24375_1};	/*Property = ScreenPixelsHeight*/
const Value V24375_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P24375_4 = {S54, 0, &V24375_4_1296, &P24375_5};	/*Property = ScreenPixelsWidth*/
const Profile P24375 = {"24375", &P24375_4};	/*Profile ID - 24375*/
const Value V12586_1_1 = {S2, NULL};	/*Value = True*/
const Property P12586_1 = {S50, 0, &V12586_1_1, NULL};	/*Property = IsMobile*/
const Value V12586_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12586_5 = {S52, 0, &V12586_5_1222, &P12586_1};	/*Property = ScreenPixelsHeight*/
const Value V12586_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12586_4 = {S54, 0, &V12586_4_1224, &P12586_5};	/*Property = ScreenPixelsWidth*/
const Profile P12586 = {"12586", &P12586_4};	/*Profile ID - 12586*/
const Value V12589_1_1 = {S2, NULL};	/*Value = True*/
const Property P12589_1 = {S50, 0, &V12589_1_1, NULL};	/*Property = IsMobile*/
const Value V12589_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P12589_5 = {S52, 0, &V12589_5_1275, &P12589_1};	/*Property = ScreenPixelsHeight*/
const Value V12589_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P12589_4 = {S54, 0, &V12589_4_1278, &P12589_5};	/*Property = ScreenPixelsWidth*/
const Profile P12589 = {"12589", &P12589_4};	/*Profile ID - 12589*/
const Value V12591_1_1 = {S2, NULL};	/*Value = True*/
const Property P12591_1 = {S50, 0, &V12591_1_1, NULL};	/*Property = IsMobile*/
const Value V12591_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12591_5 = {S52, 0, &V12591_5_1233, &P12591_1};	/*Property = ScreenPixelsHeight*/
const Value V12591_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12591_4 = {S54, 0, &V12591_4_1235, &P12591_5};	/*Property = ScreenPixelsWidth*/
const Profile P12591 = {"12591", &P12591_4};	/*Profile ID - 12591*/
const Value V12587_1_1 = {S2, NULL};	/*Value = True*/
const Property P12587_1 = {S50, 0, &V12587_1_1, NULL};	/*Property = IsMobile*/
const Value V12587_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12587_5 = {S52, 0, &V12587_5_1222, &P12587_1};	/*Property = ScreenPixelsHeight*/
const Value V12587_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12587_4 = {S54, 0, &V12587_4_1224, &P12587_5};	/*Property = ScreenPixelsWidth*/
const Profile P12587 = {"12587", &P12587_4};	/*Profile ID - 12587*/
const Value V17410_1_1 = {S2, NULL};	/*Value = True*/
const Property P17410_1 = {S50, 0, &V17410_1_1, NULL};	/*Property = IsMobile*/
const Value V17410_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17410_5 = {S52, 0, &V17410_5_1293, &P17410_1};	/*Property = ScreenPixelsHeight*/
const Value V17410_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17410_4 = {S54, 0, &V17410_4_1296, &P17410_5};	/*Property = ScreenPixelsWidth*/
const Profile P17410 = {"17410", &P17410_4};	/*Profile ID - 17410*/
const Value V12598_1_1 = {S2, NULL};	/*Value = True*/
const Property P12598_1 = {S50, 0, &V12598_1_1, NULL};	/*Property = IsMobile*/
const Value V12598_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12598_5 = {S52, 0, &V12598_5_1222, &P12598_1};	/*Property = ScreenPixelsHeight*/
const Value V12598_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12598_4 = {S54, 0, &V12598_4_1224, &P12598_5};	/*Property = ScreenPixelsWidth*/
const Profile P12598 = {"12598", &P12598_4};	/*Profile ID - 12598*/
const Value V12603_1_1 = {S2, NULL};	/*Value = True*/
const Property P12603_1 = {S50, 0, &V12603_1_1, NULL};	/*Property = IsMobile*/
const Value V12603_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12603_5 = {S52, 0, &V12603_5_1222, &P12603_1};	/*Property = ScreenPixelsHeight*/
const Value V12603_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12603_4 = {S54, 0, &V12603_4_1224, &P12603_5};	/*Property = ScreenPixelsWidth*/
const Profile P12603 = {"12603", &P12603_4};	/*Profile ID - 12603*/
const Value V15552_1_1 = {S2, NULL};	/*Value = True*/
const Property P15552_1 = {S50, 0, &V15552_1_1, NULL};	/*Property = IsMobile*/
const Value V15552_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15552_5 = {S52, 0, &V15552_5_1233, &P15552_1};	/*Property = ScreenPixelsHeight*/
const Value V15552_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15552_4 = {S54, 0, &V15552_4_1235, &P15552_5};	/*Property = ScreenPixelsWidth*/
const Profile P15552 = {"15552", &P15552_4};	/*Profile ID - 15552*/
const Value V12606_1_1 = {S2, NULL};	/*Value = True*/
const Property P12606_1 = {S50, 0, &V12606_1_1, NULL};	/*Property = IsMobile*/
const Value V12606_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12606_5 = {S52, 0, &V12606_5_1233, &P12606_1};	/*Property = ScreenPixelsHeight*/
const Value V12606_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12606_4 = {S54, 0, &V12606_4_1235, &P12606_5};	/*Property = ScreenPixelsWidth*/
const Profile P12606 = {"12606", &P12606_4};	/*Profile ID - 12606*/
const Value V14590_1_1 = {S2, NULL};	/*Value = True*/
const Property P14590_1 = {S50, 0, &V14590_1_1, NULL};	/*Property = IsMobile*/
const Value V14590_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14590_5 = {S52, 0, &V14590_5_1233, &P14590_1};	/*Property = ScreenPixelsHeight*/
const Value V14590_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14590_4 = {S54, 0, &V14590_4_1235, &P14590_5};	/*Property = ScreenPixelsWidth*/
const Profile P14590 = {"14590", &P14590_4};	/*Profile ID - 14590*/
const Value V12597_1_1 = {S2, NULL};	/*Value = True*/
const Property P12597_1 = {S50, 0, &V12597_1_1, NULL};	/*Property = IsMobile*/
const Value V12597_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12597_5 = {S52, 0, &V12597_5_1293, &P12597_1};	/*Property = ScreenPixelsHeight*/
const Value V12597_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12597_4 = {S54, 0, &V12597_4_1296, &P12597_5};	/*Property = ScreenPixelsWidth*/
const Profile P12597 = {"12597", &P12597_4};	/*Profile ID - 12597*/
const Value V12609_1_1 = {S2, NULL};	/*Value = True*/
const Property P12609_1 = {S50, 0, &V12609_1_1, NULL};	/*Property = IsMobile*/
const Value V12609_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12609_5 = {S52, 0, &V12609_5_1222, &P12609_1};	/*Property = ScreenPixelsHeight*/
const Value V12609_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12609_4 = {S54, 0, &V12609_4_1224, &P12609_5};	/*Property = ScreenPixelsWidth*/
const Profile P12609 = {"12609", &P12609_4};	/*Profile ID - 12609*/
const Value V12611_1_1 = {S2, NULL};	/*Value = True*/
const Property P12611_1 = {S50, 0, &V12611_1_1, NULL};	/*Property = IsMobile*/
const Value V12611_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12611_5 = {S52, 0, &V12611_5_1222, &P12611_1};	/*Property = ScreenPixelsHeight*/
const Value V12611_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12611_4 = {S54, 0, &V12611_4_1224, &P12611_5};	/*Property = ScreenPixelsWidth*/
const Profile P12611 = {"12611", &P12611_4};	/*Profile ID - 12611*/
const Value V12612_1_1 = {S2, NULL};	/*Value = True*/
const Property P12612_1 = {S50, 0, &V12612_1_1, NULL};	/*Property = IsMobile*/
const Value V12612_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12612_5 = {S52, 0, &V12612_5_1294, &P12612_1};	/*Property = ScreenPixelsHeight*/
const Value V12612_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12612_4 = {S54, 0, &V12612_4_1296, &P12612_5};	/*Property = ScreenPixelsWidth*/
const Profile P12612 = {"12612", &P12612_4};	/*Profile ID - 12612*/
const Value V12613_1_1 = {S2, NULL};	/*Value = True*/
const Property P12613_1 = {S50, 0, &V12613_1_1, NULL};	/*Property = IsMobile*/
const Value V12613_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12613_5 = {S52, 0, &V12613_5_1380, &P12613_1};	/*Property = ScreenPixelsHeight*/
const Value V12613_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12613_4 = {S54, 0, &V12613_4_1279, &P12613_5};	/*Property = ScreenPixelsWidth*/
const Profile P12613 = {"12613", &P12613_4};	/*Profile ID - 12613*/
const Value V12614_1_1 = {S2, NULL};	/*Value = True*/
const Property P12614_1 = {S50, 0, &V12614_1_1, NULL};	/*Property = IsMobile*/
const Value V12614_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12614_5 = {S52, 0, &V12614_5_1222, &P12614_1};	/*Property = ScreenPixelsHeight*/
const Value V12614_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12614_4 = {S54, 0, &V12614_4_1224, &P12614_5};	/*Property = ScreenPixelsWidth*/
const Profile P12614 = {"12614", &P12614_4};	/*Profile ID - 12614*/
const Value V12615_1_1 = {S2, NULL};	/*Value = True*/
const Property P12615_1 = {S50, 0, &V12615_1_1, NULL};	/*Property = IsMobile*/
const Value V12615_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12615_5 = {S52, 0, &V12615_5_1227, &P12615_1};	/*Property = ScreenPixelsHeight*/
const Value V12615_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P12615_4 = {S54, 0, &V12615_4_1943, &P12615_5};	/*Property = ScreenPixelsWidth*/
const Profile P12615 = {"12615", &P12615_4};	/*Profile ID - 12615*/
const Value V12617_1_1 = {S2, NULL};	/*Value = True*/
const Property P12617_1 = {S50, 0, &V12617_1_1, NULL};	/*Property = IsMobile*/
const Value V12617_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12617_5 = {S52, 0, &V12617_5_1294, &P12617_1};	/*Property = ScreenPixelsHeight*/
const Value V12617_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12617_4 = {S54, 0, &V12617_4_1296, &P12617_5};	/*Property = ScreenPixelsWidth*/
const Profile P12617 = {"12617", &P12617_4};	/*Profile ID - 12617*/
const Value V12631_1_1 = {S2, NULL};	/*Value = True*/
const Property P12631_1 = {S50, 0, &V12631_1_1, NULL};	/*Property = IsMobile*/
const Value V12631_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12631_5 = {S52, 0, &V12631_5_1222, &P12631_1};	/*Property = ScreenPixelsHeight*/
const Value V12631_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12631_4 = {S54, 0, &V12631_4_1224, &P12631_5};	/*Property = ScreenPixelsWidth*/
const Profile P12631 = {"12631", &P12631_4};	/*Profile ID - 12631*/
const Value V12619_1_1 = {S2, NULL};	/*Value = True*/
const Property P12619_1 = {S50, 0, &V12619_1_1, NULL};	/*Property = IsMobile*/
const Value V12619_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12619_5 = {S52, 0, &V12619_5_1294, &P12619_1};	/*Property = ScreenPixelsHeight*/
const Value V12619_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12619_4 = {S54, 0, &V12619_4_1296, &P12619_5};	/*Property = ScreenPixelsWidth*/
const Profile P12619 = {"12619", &P12619_4};	/*Profile ID - 12619*/
const Value V14390_1_1 = {S2, NULL};	/*Value = True*/
const Property P14390_1 = {S50, 0, &V14390_1_1, NULL};	/*Property = IsMobile*/
const Value V14390_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14390_5 = {S52, 0, &V14390_5_1233, &P14390_1};	/*Property = ScreenPixelsHeight*/
const Value V14390_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14390_4 = {S54, 0, &V14390_4_1235, &P14390_5};	/*Property = ScreenPixelsWidth*/
const Profile P14390 = {"14390", &P14390_4};	/*Profile ID - 14390*/
const Value V12623_1_1 = {S2, NULL};	/*Value = True*/
const Property P12623_1 = {S50, 0, &V12623_1_1, NULL};	/*Property = IsMobile*/
const Value V12623_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12623_5 = {S52, 0, &V12623_5_1233, &P12623_1};	/*Property = ScreenPixelsHeight*/
const Value V12623_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12623_4 = {S54, 0, &V12623_4_1235, &P12623_5};	/*Property = ScreenPixelsWidth*/
const Profile P12623 = {"12623", &P12623_4};	/*Profile ID - 12623*/
const Value V12624_1_1 = {S2, NULL};	/*Value = True*/
const Property P12624_1 = {S50, 0, &V12624_1_1, NULL};	/*Property = IsMobile*/
const Value V12624_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12624_5 = {S52, 0, &V12624_5_1233, &P12624_1};	/*Property = ScreenPixelsHeight*/
const Value V12624_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12624_4 = {S54, 0, &V12624_4_1235, &P12624_5};	/*Property = ScreenPixelsWidth*/
const Profile P12624 = {"12624", &P12624_4};	/*Profile ID - 12624*/
const Value V12626_1_1 = {S2, NULL};	/*Value = True*/
const Property P12626_1 = {S50, 0, &V12626_1_1, NULL};	/*Property = IsMobile*/
const Value V12626_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12626_5 = {S52, 0, &V12626_5_1304, &P12626_1};	/*Property = ScreenPixelsHeight*/
const Value V12626_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12626_4 = {S54, 0, &V12626_4_1224, &P12626_5};	/*Property = ScreenPixelsWidth*/
const Profile P12626 = {"12626", &P12626_4};	/*Profile ID - 12626*/
const Value V12630_1_1 = {S2, NULL};	/*Value = True*/
const Property P12630_1 = {S50, 0, &V12630_1_1, NULL};	/*Property = IsMobile*/
const Value V12630_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12630_5 = {S52, 0, &V12630_5_1293, &P12630_1};	/*Property = ScreenPixelsHeight*/
const Value V12630_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12630_4 = {S54, 0, &V12630_4_1296, &P12630_5};	/*Property = ScreenPixelsWidth*/
const Profile P12630 = {"12630", &P12630_4};	/*Profile ID - 12630*/
const Value V14398_1_1 = {S2, NULL};	/*Value = True*/
const Property P14398_1 = {S50, 0, &V14398_1_1, NULL};	/*Property = IsMobile*/
const Value V14398_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14398_5 = {S52, 0, &V14398_5_1293, &P14398_1};	/*Property = ScreenPixelsHeight*/
const Value V14398_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14398_4 = {S54, 0, &V14398_4_1296, &P14398_5};	/*Property = ScreenPixelsWidth*/
const Profile P14398 = {"14398", &P14398_4};	/*Profile ID - 14398*/
const Value V12636_1_1 = {S2, NULL};	/*Value = True*/
const Property P12636_1 = {S50, 0, &V12636_1_1, NULL};	/*Property = IsMobile*/
const Value V12636_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12636_5 = {S52, 0, &V12636_5_1380, &P12636_1};	/*Property = ScreenPixelsHeight*/
const Value V12636_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12636_4 = {S54, 0, &V12636_4_1279, &P12636_5};	/*Property = ScreenPixelsWidth*/
const Profile P12636 = {"12636", &P12636_4};	/*Profile ID - 12636*/
const Value V12639_1_1 = {S2, NULL};	/*Value = True*/
const Property P12639_1 = {S50, 0, &V12639_1_1, NULL};	/*Property = IsMobile*/
const Value V12639_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12639_5 = {S52, 0, &V12639_5_1233, &P12639_1};	/*Property = ScreenPixelsHeight*/
const Value V12639_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12639_4 = {S54, 0, &V12639_4_1235, &P12639_5};	/*Property = ScreenPixelsWidth*/
const Profile P12639 = {"12639", &P12639_4};	/*Profile ID - 12639*/
const Value V12641_1_1 = {S2, NULL};	/*Value = True*/
const Property P12641_1 = {S50, 0, &V12641_1_1, NULL};	/*Property = IsMobile*/
const Value V12641_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12641_5 = {S52, 0, &V12641_5_1222, &P12641_1};	/*Property = ScreenPixelsHeight*/
const Value V12641_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12641_4 = {S54, 0, &V12641_4_1224, &P12641_5};	/*Property = ScreenPixelsWidth*/
const Profile P12641 = {"12641", &P12641_4};	/*Profile ID - 12641*/
const Value V12642_1_1 = {S2, NULL};	/*Value = True*/
const Property P12642_1 = {S50, 0, &V12642_1_1, NULL};	/*Property = IsMobile*/
const Value V12642_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12642_5 = {S52, 0, &V12642_5_1233, &P12642_1};	/*Property = ScreenPixelsHeight*/
const Value V12642_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12642_4 = {S54, 0, &V12642_4_1235, &P12642_5};	/*Property = ScreenPixelsWidth*/
const Profile P12642 = {"12642", &P12642_4};	/*Profile ID - 12642*/
const Value V12644_1_1 = {S2, NULL};	/*Value = True*/
const Property P12644_1 = {S50, 0, &V12644_1_1, NULL};	/*Property = IsMobile*/
const Value V12644_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12644_5 = {S52, 0, &V12644_5_1222, &P12644_1};	/*Property = ScreenPixelsHeight*/
const Value V12644_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12644_4 = {S54, 0, &V12644_4_1224, &P12644_5};	/*Property = ScreenPixelsWidth*/
const Profile P12644 = {"12644", &P12644_4};	/*Profile ID - 12644*/
const Value V12645_1_1 = {S2, NULL};	/*Value = True*/
const Property P12645_1 = {S50, 0, &V12645_1_1, NULL};	/*Property = IsMobile*/
const Value V12645_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12645_5 = {S52, 0, &V12645_5_1294, &P12645_1};	/*Property = ScreenPixelsHeight*/
const Value V12645_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12645_4 = {S54, 0, &V12645_4_1296, &P12645_5};	/*Property = ScreenPixelsWidth*/
const Profile P12645 = {"12645", &P12645_4};	/*Profile ID - 12645*/
const Value V12646_1_1 = {S2, NULL};	/*Value = True*/
const Property P12646_1 = {S50, 0, &V12646_1_1, NULL};	/*Property = IsMobile*/
const Value V12646_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12646_5 = {S52, 0, &V12646_5_1222, &P12646_1};	/*Property = ScreenPixelsHeight*/
const Value V12646_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12646_4 = {S54, 0, &V12646_4_1224, &P12646_5};	/*Property = ScreenPixelsWidth*/
const Profile P12646 = {"12646", &P12646_4};	/*Profile ID - 12646*/
const Value V12648_1_1 = {S2, NULL};	/*Value = True*/
const Property P12648_1 = {S50, 0, &V12648_1_1, NULL};	/*Property = IsMobile*/
const Value V12648_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12648_5 = {S52, 0, &V12648_5_1233, &P12648_1};	/*Property = ScreenPixelsHeight*/
const Value V12648_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12648_4 = {S54, 0, &V12648_4_1235, &P12648_5};	/*Property = ScreenPixelsWidth*/
const Profile P12648 = {"12648", &P12648_4};	/*Profile ID - 12648*/
const Value V12649_1_1 = {S2, NULL};	/*Value = True*/
const Property P12649_1 = {S50, 0, &V12649_1_1, NULL};	/*Property = IsMobile*/
const Value V12649_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12649_5 = {S52, 0, &V12649_5_1233, &P12649_1};	/*Property = ScreenPixelsHeight*/
const Value V12649_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12649_4 = {S54, 0, &V12649_4_1235, &P12649_5};	/*Property = ScreenPixelsWidth*/
const Profile P12649 = {"12649", &P12649_4};	/*Profile ID - 12649*/
const Value V12651_1_1 = {S2, NULL};	/*Value = True*/
const Property P12651_1 = {S50, 0, &V12651_1_1, NULL};	/*Property = IsMobile*/
const Value V12651_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12651_5 = {S52, 0, &V12651_5_1293, &P12651_1};	/*Property = ScreenPixelsHeight*/
const Value V12651_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12651_4 = {S54, 0, &V12651_4_1296, &P12651_5};	/*Property = ScreenPixelsWidth*/
const Profile P12651 = {"12651", &P12651_4};	/*Profile ID - 12651*/
const Value V3692_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_279 = {S1, 0, &V3692_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3692_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_282 = {S3, 0, &V3692_282_57155, &P3692_279};	/*Property = BlobBuilder*/
const Value V3692_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_350 = {S4, 0, &V3692_350_57446, &P3692_282};	/*Property = Canvas*/
const Value V3692_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_283 = {S5, 0, &V3692_283_57161, &P3692_350};	/*Property = CssBackground*/
const Value V3692_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_284 = {S6, 0, &V3692_284_57164, &P3692_283};	/*Property = CssBorderImage*/
const Value V3692_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_285 = {S7, 0, &V3692_285_57167, &P3692_284};	/*Property = CssCanvas*/
const Value V3692_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_286 = {S8, 0, &V3692_286_57170, &P3692_285};	/*Property = CssColor*/
const Value V3692_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_237 = {S9, 0, &V3692_237_55786, &P3692_286};	/*Property = CssColumn*/
const Value V3692_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_287 = {S10, 0, &V3692_287_57173, &P3692_237};	/*Property = CssFlexbox*/
const Value V3692_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_288 = {S11, 0, &V3692_288_57176, &P3692_287};	/*Property = CssFont*/
const Value V3692_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_289 = {S12, 0, &V3692_289_86899, &P3692_288};	/*Property = CssImages*/
const Value V3692_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_291 = {S13, 0, &V3692_291_57182, &P3692_289};	/*Property = CssMediaQueries*/
const Value V3692_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_292 = {S14, 0, &V3692_292_57185, &P3692_291};	/*Property = CssMinMax*/
const Value V3692_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_293 = {S15, 0, &V3692_293_57188, &P3692_292};	/*Property = CssOverflow*/
const Value V3692_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_294 = {S16, 0, &V3692_294_57191, &P3692_293};	/*Property = CssPosition*/
const Value V3692_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_295 = {S17, 0, &V3692_295_57194, &P3692_294};	/*Property = CssText*/
const Value V3692_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_297 = {S18, 0, &V3692_297_57197, &P3692_295};	/*Property = CssTransforms*/
const Value V3692_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_241 = {S19, 0, &V3692_241_55801, &P3692_297};	/*Property = CssTransitions*/
const Value V3692_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_298 = {S20, 0, &V3692_298_57208, &P3692_241};	/*Property = CssUI*/
const Value V3692_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_300 = {S21, 0, &V3692_300_57217, &P3692_298};	/*Property = DataSet*/
const Value V3692_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_301 = {S22, 0, &V3692_301_57220, &P3692_300};	/*Property = DataUrl*/
const Value V3692_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_302 = {S23, 0, &V3692_302_57223, &P3692_301};	/*Property = DeviceOrientation*/
const Value V3692_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_304 = {S24, 0, &V3692_304_57229, &P3692_302};	/*Property = FileReader*/
const Value V3692_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_305 = {S25, 0, &V3692_305_57232, &P3692_304};	/*Property = FileSaver*/
const Value V3692_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_306 = {S26, 0, &V3692_306_57238, &P3692_305};	/*Property = FileWriter*/
const Value V3692_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_307 = {S27, 0, &V3692_307_57241, &P3692_306};	/*Property = FormData*/
const Value V3692_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_309 = {S28, 0, &V3692_309_57247, &P3692_307};	/*Property = Fullscreen*/
const Value V3692_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_310 = {S29, 0, &V3692_310_57250, &P3692_309};	/*Property = GeoLocation*/
const Value V3692_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_312 = {S30, 0, &V3692_312_57256, &P3692_310};	/*Property = History*/
const Value V3692_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_314 = {S31, 0, &V3692_314_57262, &P3692_312};	/*Property = Html5*/
const Value V3692_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_313 = {S32, 0, &V3692_313_57259, &P3692_314};	/*Property = Html-Media-Capture*/
const Value V3692_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_315 = {S33, 0, &V3692_315_57265, &P3692_313};	/*Property = Iframe*/
const Value V3692_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_316 = {S34, 0, &V3692_316_57268, &P3692_315};	/*Property = IndexedDB*/
const Value V3692_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_317 = {S35, 0, &V3692_317_57271, &P3692_316};	/*Property = Json*/
const Value V3692_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_15 = {S37, 0, &V3692_15_72, &P3692_317};	/*Property = LayoutEngine*/
const Value V3692_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_344 = {S38, 0, &V3692_344_57274, &P3692_15};	/*Property = Masking*/
const Value V3692_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_320 = {S39, 0, &V3692_320_57292, &P3692_344};	/*Property = PostMessage*/
const Value V3692_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_321 = {S40, 0, &V3692_321_57295, &P3692_320};	/*Property = Progress*/
const Value V3692_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_322 = {S41, 0, &V3692_322_57298, &P3692_321};	/*Property = Prompts*/
const Value V3692_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_323 = {S42, 0, &V3692_323_57301, &P3692_322};	/*Property = Selector*/
const Value V3692_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_324 = {S43, 0, &V3692_324_57307, &P3692_323};	/*Property = Svg*/
const Value V3692_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_327 = {S44, 0, &V3692_327_57316, &P3692_324};	/*Property = TouchEvents*/
const Value V3692_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_328 = {S45, 0, &V3692_328_57319, &P3692_327};	/*Property = Track*/
const Value V3692_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_329 = {S46, 0, &V3692_329_57325, &P3692_328};	/*Property = Video*/
const Value V3692_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_330 = {S47, 0, &V3692_330_57328, &P3692_329};	/*Property = Viewport*/
const Value V3692_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_332 = {S48, 0, &V3692_332_57340, &P3692_330};	/*Property = WebWorkers*/
const Value V3692_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3692_333 = {S49, 0, &V3692_333_57343, &P3692_332};	/*Property = Xhr2*/
const Profile P3692 = {"3692", &P3692_333};	/*Profile ID - 3692*/
const Value V12652_1_1 = {S2, NULL};	/*Value = True*/
const Property P12652_1 = {S50, 0, &V12652_1_1, NULL};	/*Property = IsMobile*/
const Value V12652_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12652_5 = {S52, 0, &V12652_5_1233, &P12652_1};	/*Property = ScreenPixelsHeight*/
const Value V12652_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12652_4 = {S54, 0, &V12652_4_1235, &P12652_5};	/*Property = ScreenPixelsWidth*/
const Profile P12652 = {"12652", &P12652_4};	/*Profile ID - 12652*/
const Value V12653_1_1 = {S2, NULL};	/*Value = True*/
const Property P12653_1 = {S50, 0, &V12653_1_1, NULL};	/*Property = IsMobile*/
const Value V12653_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12653_5 = {S52, 0, &V12653_5_1293, &P12653_1};	/*Property = ScreenPixelsHeight*/
const Value V12653_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12653_4 = {S54, 0, &V12653_4_1296, &P12653_5};	/*Property = ScreenPixelsWidth*/
const Profile P12653 = {"12653", &P12653_4};	/*Profile ID - 12653*/
const Value V24376_1_1 = {S2, NULL};	/*Value = True*/
const Property P24376_1 = {S50, 0, &V24376_1_1, NULL};	/*Property = IsMobile*/
const Value V24376_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P24376_5 = {S52, 0, &V24376_5_1293, &P24376_1};	/*Property = ScreenPixelsHeight*/
const Value V24376_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P24376_4 = {S54, 0, &V24376_4_1296, &P24376_5};	/*Property = ScreenPixelsWidth*/
const Profile P24376 = {"24376", &P24376_4};	/*Profile ID - 24376*/
const Value V12658_1_1 = {S2, NULL};	/*Value = True*/
const Property P12658_1 = {S50, 0, &V12658_1_1, NULL};	/*Property = IsMobile*/
const Value V12658_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12658_5 = {S52, 0, &V12658_5_1222, &P12658_1};	/*Property = ScreenPixelsHeight*/
const Value V12658_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12658_4 = {S54, 0, &V12658_4_1224, &P12658_5};	/*Property = ScreenPixelsWidth*/
const Profile P12658 = {"12658", &P12658_4};	/*Profile ID - 12658*/
const Value V12660_1_1 = {S2, NULL};	/*Value = True*/
const Property P12660_1 = {S50, 0, &V12660_1_1, NULL};	/*Property = IsMobile*/
const Value V12660_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12660_5 = {S52, 0, &V12660_5_1222, &P12660_1};	/*Property = ScreenPixelsHeight*/
const Value V12660_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12660_4 = {S54, 0, &V12660_4_1224, &P12660_5};	/*Property = ScreenPixelsWidth*/
const Profile P12660 = {"12660", &P12660_4};	/*Profile ID - 12660*/
const Value V12661_1_1 = {S2, NULL};	/*Value = True*/
const Property P12661_1 = {S50, 0, &V12661_1_1, NULL};	/*Property = IsMobile*/
const Value V12661_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12661_5 = {S52, 0, &V12661_5_1294, &P12661_1};	/*Property = ScreenPixelsHeight*/
const Value V12661_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12661_4 = {S54, 0, &V12661_4_1296, &P12661_5};	/*Property = ScreenPixelsWidth*/
const Profile P12661 = {"12661", &P12661_4};	/*Profile ID - 12661*/
const Value V12662_1_1 = {S2, NULL};	/*Value = True*/
const Property P12662_1 = {S50, 0, &V12662_1_1, NULL};	/*Property = IsMobile*/
const Value V12662_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12662_5 = {S52, 0, &V12662_5_1222, &P12662_1};	/*Property = ScreenPixelsHeight*/
const Value V12662_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12662_4 = {S54, 0, &V12662_4_1224, &P12662_5};	/*Property = ScreenPixelsWidth*/
const Profile P12662 = {"12662", &P12662_4};	/*Profile ID - 12662*/
const Value V12664_1_1 = {S2, NULL};	/*Value = True*/
const Property P12664_1 = {S50, 0, &V12664_1_1, NULL};	/*Property = IsMobile*/
const Value V12664_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12664_5 = {S52, 0, &V12664_5_1293, &P12664_1};	/*Property = ScreenPixelsHeight*/
const Value V12664_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12664_4 = {S54, 0, &V12664_4_1296, &P12664_5};	/*Property = ScreenPixelsWidth*/
const Profile P12664 = {"12664", &P12664_4};	/*Profile ID - 12664*/
const Value V12665_1_1 = {S2, NULL};	/*Value = True*/
const Property P12665_1 = {S50, 0, &V12665_1_1, NULL};	/*Property = IsMobile*/
const Value V12665_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12665_5 = {S52, 0, &V12665_5_1233, &P12665_1};	/*Property = ScreenPixelsHeight*/
const Value V12665_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12665_4 = {S54, 0, &V12665_4_1235, &P12665_5};	/*Property = ScreenPixelsWidth*/
const Profile P12665 = {"12665", &P12665_4};	/*Profile ID - 12665*/
const Value V12668_1_1 = {S2, NULL};	/*Value = True*/
const Property P12668_1 = {S50, 0, &V12668_1_1, NULL};	/*Property = IsMobile*/
const Value V12668_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12668_5 = {S52, 0, &V12668_5_1293, &P12668_1};	/*Property = ScreenPixelsHeight*/
const Value V12668_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12668_4 = {S54, 0, &V12668_4_1296, &P12668_5};	/*Property = ScreenPixelsWidth*/
const Profile P12668 = {"12668", &P12668_4};	/*Profile ID - 12668*/
const Value V12670_1_1 = {S2, NULL};	/*Value = True*/
const Property P12670_1 = {S50, 0, &V12670_1_1, NULL};	/*Property = IsMobile*/
const Value V12670_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12670_5 = {S52, 0, &V12670_5_1304, &P12670_1};	/*Property = ScreenPixelsHeight*/
const Value V12670_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12670_4 = {S54, 0, &V12670_4_1224, &P12670_5};	/*Property = ScreenPixelsWidth*/
const Profile P12670 = {"12670", &P12670_4};	/*Profile ID - 12670*/
const Value V15382_1_1 = {S2, NULL};	/*Value = True*/
const Property P15382_1 = {S50, 0, &V15382_1_1, NULL};	/*Property = IsMobile*/
const Value V15382_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15382_5 = {S52, 0, &V15382_5_1222, &P15382_1};	/*Property = ScreenPixelsHeight*/
const Value V15382_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15382_4 = {S54, 0, &V15382_4_1224, &P15382_5};	/*Property = ScreenPixelsWidth*/
const Profile P15382 = {"15382", &P15382_4};	/*Profile ID - 15382*/
const Value V16670_1_1 = {S2, NULL};	/*Value = True*/
const Property P16670_1 = {S50, 0, &V16670_1_1, NULL};	/*Property = IsMobile*/
const Value V16670_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16670_5 = {S52, 0, &V16670_5_1277, &P16670_1};	/*Property = ScreenPixelsHeight*/
const Value V16670_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16670_4 = {S54, 0, &V16670_4_1279, &P16670_5};	/*Property = ScreenPixelsWidth*/
const Profile P16670 = {"16670", &P16670_4};	/*Profile ID - 16670*/
const Value V12672_1_1 = {S2, NULL};	/*Value = True*/
const Property P12672_1 = {S50, 0, &V12672_1_1, NULL};	/*Property = IsMobile*/
const Value V12672_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12672_5 = {S52, 0, &V12672_5_1222, &P12672_1};	/*Property = ScreenPixelsHeight*/
const Value V12672_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12672_4 = {S54, 0, &V12672_4_1224, &P12672_5};	/*Property = ScreenPixelsWidth*/
const Profile P12672 = {"12672", &P12672_4};	/*Profile ID - 12672*/
const Value V20210_1_1 = {S2, NULL};	/*Value = True*/
const Property P20210_1 = {S50, 0, &V20210_1_1, NULL};	/*Property = IsMobile*/
const Value V20210_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20210_5 = {S52, 0, &V20210_5_37, &P20210_1};	/*Property = ScreenPixelsHeight*/
const Value V20210_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20210_4 = {S54, 0, &V20210_4_36, &P20210_5};	/*Property = ScreenPixelsWidth*/
const Profile P20210 = {"20210", &P20210_4};	/*Profile ID - 20210*/
const Value V12676_1_1 = {S2, NULL};	/*Value = True*/
const Property P12676_1 = {S50, 0, &V12676_1_1, NULL};	/*Property = IsMobile*/
const Value V12676_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12676_5 = {S52, 0, &V12676_5_1233, &P12676_1};	/*Property = ScreenPixelsHeight*/
const Value V12676_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12676_4 = {S54, 0, &V12676_4_1235, &P12676_5};	/*Property = ScreenPixelsWidth*/
const Profile P12676 = {"12676", &P12676_4};	/*Profile ID - 12676*/
const Value V12678_1_1 = {S2, NULL};	/*Value = True*/
const Property P12678_1 = {S50, 0, &V12678_1_1, NULL};	/*Property = IsMobile*/
const Value V12678_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12678_5 = {S52, 0, &V12678_5_1233, &P12678_1};	/*Property = ScreenPixelsHeight*/
const Value V12678_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12678_4 = {S54, 0, &V12678_4_1235, &P12678_5};	/*Property = ScreenPixelsWidth*/
const Profile P12678 = {"12678", &P12678_4};	/*Profile ID - 12678*/
const Value V12680_1_1 = {S2, NULL};	/*Value = True*/
const Property P12680_1 = {S50, 0, &V12680_1_1, NULL};	/*Property = IsMobile*/
const Value V12680_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12680_5 = {S52, 0, &V12680_5_1293, &P12680_1};	/*Property = ScreenPixelsHeight*/
const Value V12680_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12680_4 = {S54, 0, &V12680_4_1296, &P12680_5};	/*Property = ScreenPixelsWidth*/
const Profile P12680 = {"12680", &P12680_4};	/*Profile ID - 12680*/
const Value V12682_1_1 = {S2, NULL};	/*Value = True*/
const Property P12682_1 = {S50, 0, &V12682_1_1, NULL};	/*Property = IsMobile*/
const Value V12682_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P12682_5 = {S52, 0, &V12682_5_1275, &P12682_1};	/*Property = ScreenPixelsHeight*/
const Value V12682_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12682_4 = {S54, 0, &V12682_4_1229, &P12682_5};	/*Property = ScreenPixelsWidth*/
const Profile P12682 = {"12682", &P12682_4};	/*Profile ID - 12682*/
const Value V12683_1_1 = {S2, NULL};	/*Value = True*/
const Property P12683_1 = {S50, 0, &V12683_1_1, NULL};	/*Property = IsMobile*/
const Value V12683_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12683_5 = {S52, 0, &V12683_5_1233, &P12683_1};	/*Property = ScreenPixelsHeight*/
const Value V12683_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12683_4 = {S54, 0, &V12683_4_1235, &P12683_5};	/*Property = ScreenPixelsWidth*/
const Profile P12683 = {"12683", &P12683_4};	/*Profile ID - 12683*/
const Value V12684_1_1 = {S2, NULL};	/*Value = True*/
const Property P12684_1 = {S50, 0, &V12684_1_1, NULL};	/*Property = IsMobile*/
const Value V12684_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12684_5 = {S52, 0, &V12684_5_1227, &P12684_1};	/*Property = ScreenPixelsHeight*/
const Value V12684_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12684_4 = {S54, 0, &V12684_4_1229, &P12684_5};	/*Property = ScreenPixelsWidth*/
const Profile P12684 = {"12684", &P12684_4};	/*Profile ID - 12684*/
const Value V12686_1_1 = {S2, NULL};	/*Value = True*/
const Property P12686_1 = {S50, 0, &V12686_1_1, NULL};	/*Property = IsMobile*/
const Value V12686_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12686_5 = {S52, 0, &V12686_5_1293, &P12686_1};	/*Property = ScreenPixelsHeight*/
const Value V12686_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12686_4 = {S54, 0, &V12686_4_1296, &P12686_5};	/*Property = ScreenPixelsWidth*/
const Profile P12686 = {"12686", &P12686_4};	/*Profile ID - 12686*/
const Value V12687_1_1 = {S2, NULL};	/*Value = True*/
const Property P12687_1 = {S50, 0, &V12687_1_1, NULL};	/*Property = IsMobile*/
const Value V12687_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12687_5 = {S52, 0, &V12687_5_1233, &P12687_1};	/*Property = ScreenPixelsHeight*/
const Value V12687_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12687_4 = {S54, 0, &V12687_4_1235, &P12687_5};	/*Property = ScreenPixelsWidth*/
const Profile P12687 = {"12687", &P12687_4};	/*Profile ID - 12687*/
const Value V12691_1_1 = {S2, NULL};	/*Value = True*/
const Property P12691_1 = {S50, 0, &V12691_1_1, NULL};	/*Property = IsMobile*/
const Value V12691_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12691_5 = {S52, 0, &V12691_5_1222, &P12691_1};	/*Property = ScreenPixelsHeight*/
const Value V12691_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12691_4 = {S54, 0, &V12691_4_1224, &P12691_5};	/*Property = ScreenPixelsWidth*/
const Profile P12691 = {"12691", &P12691_4};	/*Profile ID - 12691*/
const Value V12693_1_1 = {S2, NULL};	/*Value = True*/
const Property P12693_1 = {S50, 0, &V12693_1_1, NULL};	/*Property = IsMobile*/
const Value V12693_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12693_5 = {S52, 0, &V12693_5_1233, &P12693_1};	/*Property = ScreenPixelsHeight*/
const Value V12693_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12693_4 = {S54, 0, &V12693_4_1235, &P12693_5};	/*Property = ScreenPixelsWidth*/
const Profile P12693 = {"12693", &P12693_4};	/*Profile ID - 12693*/
const Value V12694_1_1 = {S2, NULL};	/*Value = True*/
const Property P12694_1 = {S50, 0, &V12694_1_1, NULL};	/*Property = IsMobile*/
const Value V12694_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12694_5 = {S52, 0, &V12694_5_1233, &P12694_1};	/*Property = ScreenPixelsHeight*/
const Value V12694_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12694_4 = {S54, 0, &V12694_4_1235, &P12694_5};	/*Property = ScreenPixelsWidth*/
const Profile P12694 = {"12694", &P12694_4};	/*Profile ID - 12694*/
const Value V12696_1_1 = {S2, NULL};	/*Value = True*/
const Property P12696_1 = {S50, 0, &V12696_1_1, NULL};	/*Property = IsMobile*/
const Value V12696_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12696_5 = {S52, 0, &V12696_5_1380, &P12696_1};	/*Property = ScreenPixelsHeight*/
const Value V12696_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12696_4 = {S54, 0, &V12696_4_1279, &P12696_5};	/*Property = ScreenPixelsWidth*/
const Profile P12696 = {"12696", &P12696_4};	/*Profile ID - 12696*/
const Value V20080_1_2 = {S0, NULL};	/*Value = False*/
const Property P20080_1 = {S50, 0, &V20080_1_2, NULL};	/*Property = IsMobile*/
const Value V20080_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20080_5 = {S52, 0, &V20080_5_37, &P20080_1};	/*Property = ScreenPixelsHeight*/
const Value V20080_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20080_4 = {S54, 0, &V20080_4_36, &P20080_5};	/*Property = ScreenPixelsWidth*/
const Profile P20080 = {"20080", &P20080_4};	/*Profile ID - 20080*/
const Value V12699_1_1 = {S2, NULL};	/*Value = True*/
const Property P12699_1 = {S50, 0, &V12699_1_1, NULL};	/*Property = IsMobile*/
const Value V12699_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12699_5 = {S52, 0, &V12699_5_1233, &P12699_1};	/*Property = ScreenPixelsHeight*/
const Value V12699_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12699_4 = {S54, 0, &V12699_4_1235, &P12699_5};	/*Property = ScreenPixelsWidth*/
const Profile P12699 = {"12699", &P12699_4};	/*Profile ID - 12699*/
const Value V14395_1_1 = {S2, NULL};	/*Value = True*/
const Property P14395_1 = {S50, 0, &V14395_1_1, NULL};	/*Property = IsMobile*/
const Value V14395_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14395_5 = {S52, 0, &V14395_5_1222, &P14395_1};	/*Property = ScreenPixelsHeight*/
const Value V14395_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14395_4 = {S54, 0, &V14395_4_1224, &P14395_5};	/*Property = ScreenPixelsWidth*/
const Profile P14395 = {"14395", &P14395_4};	/*Profile ID - 14395*/
const Value V12708_1_1 = {S2, NULL};	/*Value = True*/
const Property P12708_1 = {S50, 0, &V12708_1_1, NULL};	/*Property = IsMobile*/
const Value V12708_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12708_5 = {S52, 0, &V12708_5_1293, &P12708_1};	/*Property = ScreenPixelsHeight*/
const Value V12708_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12708_4 = {S54, 0, &V12708_4_1296, &P12708_5};	/*Property = ScreenPixelsWidth*/
const Profile P12708 = {"12708", &P12708_4};	/*Profile ID - 12708*/
const Value V12712_1_1 = {S2, NULL};	/*Value = True*/
const Property P12712_1 = {S50, 0, &V12712_1_1, NULL};	/*Property = IsMobile*/
const Value V12712_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12712_5 = {S52, 0, &V12712_5_1233, &P12712_1};	/*Property = ScreenPixelsHeight*/
const Value V12712_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12712_4 = {S54, 0, &V12712_4_1235, &P12712_5};	/*Property = ScreenPixelsWidth*/
const Profile P12712 = {"12712", &P12712_4};	/*Profile ID - 12712*/
const Value V12692_1_1 = {S2, NULL};	/*Value = True*/
const Property P12692_1 = {S50, 0, &V12692_1_1, NULL};	/*Property = IsMobile*/
const Value V12692_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12692_5 = {S52, 0, &V12692_5_1222, &P12692_1};	/*Property = ScreenPixelsHeight*/
const Value V12692_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12692_4 = {S54, 0, &V12692_4_1224, &P12692_5};	/*Property = ScreenPixelsWidth*/
const Profile P12692 = {"12692", &P12692_4};	/*Profile ID - 12692*/
const Value V12716_1_1 = {S2, NULL};	/*Value = True*/
const Property P12716_1 = {S50, 0, &V12716_1_1, NULL};	/*Property = IsMobile*/
const Value V12716_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12716_5 = {S52, 0, &V12716_5_1293, &P12716_1};	/*Property = ScreenPixelsHeight*/
const Value V12716_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12716_4 = {S54, 0, &V12716_4_1296, &P12716_5};	/*Property = ScreenPixelsWidth*/
const Profile P12716 = {"12716", &P12716_4};	/*Profile ID - 12716*/
const Value V12721_1_1 = {S2, NULL};	/*Value = True*/
const Property P12721_1 = {S50, 0, &V12721_1_1, NULL};	/*Property = IsMobile*/
const Value V12721_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12721_5 = {S52, 0, &V12721_5_1293, &P12721_1};	/*Property = ScreenPixelsHeight*/
const Value V12721_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12721_4 = {S54, 0, &V12721_4_1296, &P12721_5};	/*Property = ScreenPixelsWidth*/
const Profile P12721 = {"12721", &P12721_4};	/*Profile ID - 12721*/
const Value V12724_1_1 = {S2, NULL};	/*Value = True*/
const Property P12724_1 = {S50, 0, &V12724_1_1, NULL};	/*Property = IsMobile*/
const Value V12724_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12724_5 = {S52, 0, &V12724_5_1233, &P12724_1};	/*Property = ScreenPixelsHeight*/
const Value V12724_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12724_4 = {S54, 0, &V12724_4_1235, &P12724_5};	/*Property = ScreenPixelsWidth*/
const Profile P12724 = {"12724", &P12724_4};	/*Profile ID - 12724*/
const Value V12726_1_1 = {S2, NULL};	/*Value = True*/
const Property P12726_1 = {S50, 0, &V12726_1_1, NULL};	/*Property = IsMobile*/
const Value V12726_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12726_5 = {S52, 0, &V12726_5_1277, &P12726_1};	/*Property = ScreenPixelsHeight*/
const Value V12726_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12726_4 = {S54, 0, &V12726_4_1279, &P12726_5};	/*Property = ScreenPixelsWidth*/
const Profile P12726 = {"12726", &P12726_4};	/*Profile ID - 12726*/
const Value V12728_1_1 = {S2, NULL};	/*Value = True*/
const Property P12728_1 = {S50, 0, &V12728_1_1, NULL};	/*Property = IsMobile*/
const Value V12728_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12728_5 = {S52, 0, &V12728_5_1222, &P12728_1};	/*Property = ScreenPixelsHeight*/
const Value V12728_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12728_4 = {S54, 0, &V12728_4_1224, &P12728_5};	/*Property = ScreenPixelsWidth*/
const Profile P12728 = {"12728", &P12728_4};	/*Profile ID - 12728*/
const Value V12729_1_1 = {S2, NULL};	/*Value = True*/
const Property P12729_1 = {S50, 0, &V12729_1_1, NULL};	/*Property = IsMobile*/
const Value V12729_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12729_5 = {S52, 0, &V12729_5_1293, &P12729_1};	/*Property = ScreenPixelsHeight*/
const Value V12729_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12729_4 = {S54, 0, &V12729_4_1296, &P12729_5};	/*Property = ScreenPixelsWidth*/
const Profile P12729 = {"12729", &P12729_4};	/*Profile ID - 12729*/
const Value V12732_1_1 = {S2, NULL};	/*Value = True*/
const Property P12732_1 = {S50, 0, &V12732_1_1, NULL};	/*Property = IsMobile*/
const Value V12732_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P12732_5 = {S52, 0, &V12732_5_1294, &P12732_1};	/*Property = ScreenPixelsHeight*/
const Value V12732_4_33019 = {S61, NULL};	/*Value = 160*/
const Property P12732_4 = {S54, 0, &V12732_4_33019, &P12732_5};	/*Property = ScreenPixelsWidth*/
const Profile P12732 = {"12732", &P12732_4};	/*Profile ID - 12732*/
const Value V12736_1_1 = {S2, NULL};	/*Value = True*/
const Property P12736_1 = {S50, 0, &V12736_1_1, NULL};	/*Property = IsMobile*/
const Value V12736_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12736_5 = {S52, 0, &V12736_5_1222, &P12736_1};	/*Property = ScreenPixelsHeight*/
const Value V12736_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12736_4 = {S54, 0, &V12736_4_1224, &P12736_5};	/*Property = ScreenPixelsWidth*/
const Profile P12736 = {"12736", &P12736_4};	/*Profile ID - 12736*/
const Value V13014_1_1 = {S2, NULL};	/*Value = True*/
const Property P13014_1 = {S50, 0, &V13014_1_1, NULL};	/*Property = IsMobile*/
const Value V13014_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13014_5 = {S52, 0, &V13014_5_1222, &P13014_1};	/*Property = ScreenPixelsHeight*/
const Value V13014_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13014_4 = {S54, 0, &V13014_4_1224, &P13014_5};	/*Property = ScreenPixelsWidth*/
const Profile P13014 = {"13014", &P13014_4};	/*Profile ID - 13014*/
const Value V12737_1_1 = {S2, NULL};	/*Value = True*/
const Property P12737_1 = {S50, 0, &V12737_1_1, NULL};	/*Property = IsMobile*/
const Value V12737_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12737_5 = {S52, 0, &V12737_5_1233, &P12737_1};	/*Property = ScreenPixelsHeight*/
const Value V12737_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12737_4 = {S54, 0, &V12737_4_1235, &P12737_5};	/*Property = ScreenPixelsWidth*/
const Profile P12737 = {"12737", &P12737_4};	/*Profile ID - 12737*/
const Value V12739_1_1 = {S2, NULL};	/*Value = True*/
const Property P12739_1 = {S50, 0, &V12739_1_1, NULL};	/*Property = IsMobile*/
const Value V12739_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12739_5 = {S52, 0, &V12739_5_1380, &P12739_1};	/*Property = ScreenPixelsHeight*/
const Value V12739_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12739_4 = {S54, 0, &V12739_4_1279, &P12739_5};	/*Property = ScreenPixelsWidth*/
const Profile P12739 = {"12739", &P12739_4};	/*Profile ID - 12739*/
const Value V12741_1_1 = {S2, NULL};	/*Value = True*/
const Property P12741_1 = {S50, 0, &V12741_1_1, NULL};	/*Property = IsMobile*/
const Value V12741_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12741_5 = {S52, 0, &V12741_5_1233, &P12741_1};	/*Property = ScreenPixelsHeight*/
const Value V12741_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12741_4 = {S54, 0, &V12741_4_1235, &P12741_5};	/*Property = ScreenPixelsWidth*/
const Profile P12741 = {"12741", &P12741_4};	/*Profile ID - 12741*/
const Value V12745_1_1 = {S2, NULL};	/*Value = True*/
const Property P12745_1 = {S50, 0, &V12745_1_1, NULL};	/*Property = IsMobile*/
const Value V12745_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12745_5 = {S52, 0, &V12745_5_1233, &P12745_1};	/*Property = ScreenPixelsHeight*/
const Value V12745_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12745_4 = {S54, 0, &V12745_4_1235, &P12745_5};	/*Property = ScreenPixelsWidth*/
const Profile P12745 = {"12745", &P12745_4};	/*Profile ID - 12745*/
const Value V12746_1_1 = {S2, NULL};	/*Value = True*/
const Property P12746_1 = {S50, 0, &V12746_1_1, NULL};	/*Property = IsMobile*/
const Value V12746_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12746_5 = {S52, 0, &V12746_5_1222, &P12746_1};	/*Property = ScreenPixelsHeight*/
const Value V12746_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12746_4 = {S54, 0, &V12746_4_1224, &P12746_5};	/*Property = ScreenPixelsWidth*/
const Profile P12746 = {"12746", &P12746_4};	/*Profile ID - 12746*/
const Value V12747_1_1 = {S2, NULL};	/*Value = True*/
const Property P12747_1 = {S50, 0, &V12747_1_1, NULL};	/*Property = IsMobile*/
const Value V12747_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P12747_5 = {S52, 0, &V12747_5_1277, &P12747_1};	/*Property = ScreenPixelsHeight*/
const Value V12747_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12747_4 = {S54, 0, &V12747_4_1279, &P12747_5};	/*Property = ScreenPixelsWidth*/
const Profile P12747 = {"12747", &P12747_4};	/*Profile ID - 12747*/
const Value V12748_1_1 = {S2, NULL};	/*Value = True*/
const Property P12748_1 = {S50, 0, &V12748_1_1, NULL};	/*Property = IsMobile*/
const Value V12748_5_37540 = {S105, NULL};	/*Value = 427*/
const Property P12748_5 = {S52, 0, &V12748_5_37540, &P12748_1};	/*Property = ScreenPixelsHeight*/
const Value V12748_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12748_4 = {S54, 0, &V12748_4_1224, &P12748_5};	/*Property = ScreenPixelsWidth*/
const Profile P12748 = {"12748", &P12748_4};	/*Profile ID - 12748*/
const Value V14311_1_1 = {S2, NULL};	/*Value = True*/
const Property P14311_1 = {S50, 0, &V14311_1_1, NULL};	/*Property = IsMobile*/
const Value V14311_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14311_5 = {S52, 0, &V14311_5_1293, &P14311_1};	/*Property = ScreenPixelsHeight*/
const Value V14311_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14311_4 = {S54, 0, &V14311_4_1296, &P14311_5};	/*Property = ScreenPixelsWidth*/
const Profile P14311 = {"14311", &P14311_4};	/*Profile ID - 14311*/
const Value V11824_1_1 = {S2, NULL};	/*Value = True*/
const Property P11824_1 = {S50, 0, &V11824_1_1, NULL};	/*Property = IsMobile*/
const Value V11824_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P11824_5 = {S52, 0, &V11824_5_1304, &P11824_1};	/*Property = ScreenPixelsHeight*/
const Value V11824_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11824_4 = {S54, 0, &V11824_4_1224, &P11824_5};	/*Property = ScreenPixelsWidth*/
const Profile P11824 = {"11824", &P11824_4};	/*Profile ID - 11824*/
const Profile P24739 = {"24739", NULL};	/*Profile ID - 24739*/
const Value V12324_1_1 = {S2, NULL};	/*Value = True*/
const Property P12324_1 = {S50, 0, &V12324_1_1, NULL};	/*Property = IsMobile*/
const Value V12324_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12324_5 = {S52, 0, &V12324_5_1304, &P12324_1};	/*Property = ScreenPixelsHeight*/
const Value V12324_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12324_4 = {S54, 0, &V12324_4_1224, &P12324_5};	/*Property = ScreenPixelsWidth*/
const Profile P12324 = {"12324", &P12324_4};	/*Profile ID - 12324*/
const Value V14315_1_1 = {S2, NULL};	/*Value = True*/
const Property P14315_1 = {S50, 0, &V14315_1_1, NULL};	/*Property = IsMobile*/
const Value V14315_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14315_5 = {S52, 0, &V14315_5_1294, &P14315_1};	/*Property = ScreenPixelsHeight*/
const Value V14315_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14315_4 = {S54, 0, &V14315_4_1296, &P14315_5};	/*Property = ScreenPixelsWidth*/
const Profile P14315 = {"14315", &P14315_4};	/*Profile ID - 14315*/
const Value V14319_1_1 = {S2, NULL};	/*Value = True*/
const Property P14319_1 = {S50, 0, &V14319_1_1, NULL};	/*Property = IsMobile*/
const Value V14319_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P14319_5 = {S52, 0, &V14319_5_1333, &P14319_1};	/*Property = ScreenPixelsHeight*/
const Value V14319_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P14319_4 = {S54, 0, &V14319_4_1335, &P14319_5};	/*Property = ScreenPixelsWidth*/
const Profile P14319 = {"14319", &P14319_4};	/*Profile ID - 14319*/
const Value V14321_1_1 = {S2, NULL};	/*Value = True*/
const Property P14321_1 = {S50, 0, &V14321_1_1, NULL};	/*Property = IsMobile*/
const Value V14321_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P14321_5 = {S52, 0, &V14321_5_1275, &P14321_1};	/*Property = ScreenPixelsHeight*/
const Value V14321_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14321_4 = {S54, 0, &V14321_4_1229, &P14321_5};	/*Property = ScreenPixelsWidth*/
const Profile P14321 = {"14321", &P14321_4};	/*Profile ID - 14321*/
const Value V14322_1_1 = {S2, NULL};	/*Value = True*/
const Property P14322_1 = {S50, 0, &V14322_1_1, NULL};	/*Property = IsMobile*/
const Value V14322_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14322_5 = {S52, 0, &V14322_5_1233, &P14322_1};	/*Property = ScreenPixelsHeight*/
const Value V14322_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14322_4 = {S54, 0, &V14322_4_1235, &P14322_5};	/*Property = ScreenPixelsWidth*/
const Profile P14322 = {"14322", &P14322_4};	/*Profile ID - 14322*/
const Value V14326_1_1 = {S2, NULL};	/*Value = True*/
const Property P14326_1 = {S50, 0, &V14326_1_1, NULL};	/*Property = IsMobile*/
const Value V14326_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14326_5 = {S52, 0, &V14326_5_1233, &P14326_1};	/*Property = ScreenPixelsHeight*/
const Value V14326_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14326_4 = {S54, 0, &V14326_4_1235, &P14326_5};	/*Property = ScreenPixelsWidth*/
const Profile P14326 = {"14326", &P14326_4};	/*Profile ID - 14326*/
const Value V14327_1_1 = {S2, NULL};	/*Value = True*/
const Property P14327_1 = {S50, 0, &V14327_1_1, NULL};	/*Property = IsMobile*/
const Value V14327_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14327_5 = {S52, 0, &V14327_5_1222, &P14327_1};	/*Property = ScreenPixelsHeight*/
const Value V14327_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14327_4 = {S54, 0, &V14327_4_1224, &P14327_5};	/*Property = ScreenPixelsWidth*/
const Profile P14327 = {"14327", &P14327_4};	/*Profile ID - 14327*/
const Value V14328_1_1 = {S2, NULL};	/*Value = True*/
const Property P14328_1 = {S50, 0, &V14328_1_1, NULL};	/*Property = IsMobile*/
const Value V14328_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14328_5 = {S52, 0, &V14328_5_1294, &P14328_1};	/*Property = ScreenPixelsHeight*/
const Value V14328_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14328_4 = {S54, 0, &V14328_4_1296, &P14328_5};	/*Property = ScreenPixelsWidth*/
const Profile P14328 = {"14328", &P14328_4};	/*Profile ID - 14328*/
const Value V14329_1_1 = {S2, NULL};	/*Value = True*/
const Property P14329_1 = {S50, 0, &V14329_1_1, NULL};	/*Property = IsMobile*/
const Value V14329_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14329_5 = {S52, 0, &V14329_5_1222, &P14329_1};	/*Property = ScreenPixelsHeight*/
const Value V14329_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14329_4 = {S54, 0, &V14329_4_1224, &P14329_5};	/*Property = ScreenPixelsWidth*/
const Profile P14329 = {"14329", &P14329_4};	/*Profile ID - 14329*/
const Value V14330_1_1 = {S2, NULL};	/*Value = True*/
const Property P14330_1 = {S50, 0, &V14330_1_1, NULL};	/*Property = IsMobile*/
const Value V14330_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14330_5 = {S52, 0, &V14330_5_1233, &P14330_1};	/*Property = ScreenPixelsHeight*/
const Value V14330_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14330_4 = {S54, 0, &V14330_4_1235, &P14330_5};	/*Property = ScreenPixelsWidth*/
const Profile P14330 = {"14330", &P14330_4};	/*Profile ID - 14330*/
const Value V14331_1_1 = {S2, NULL};	/*Value = True*/
const Property P14331_1 = {S50, 0, &V14331_1_1, NULL};	/*Property = IsMobile*/
const Value V14331_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14331_5 = {S52, 0, &V14331_5_1222, &P14331_1};	/*Property = ScreenPixelsHeight*/
const Value V14331_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14331_4 = {S54, 0, &V14331_4_1224, &P14331_5};	/*Property = ScreenPixelsWidth*/
const Profile P14331 = {"14331", &P14331_4};	/*Profile ID - 14331*/
const Value V14337_1_1 = {S2, NULL};	/*Value = True*/
const Property P14337_1 = {S50, 0, &V14337_1_1, NULL};	/*Property = IsMobile*/
const Value V14337_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14337_5 = {S52, 0, &V14337_5_1233, &P14337_1};	/*Property = ScreenPixelsHeight*/
const Value V14337_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14337_4 = {S54, 0, &V14337_4_1235, &P14337_5};	/*Property = ScreenPixelsWidth*/
const Profile P14337 = {"14337", &P14337_4};	/*Profile ID - 14337*/
const Value V17557_1_1 = {S2, NULL};	/*Value = True*/
const Property P17557_1 = {S50, 0, &V17557_1_1, NULL};	/*Property = IsMobile*/
const Value V17557_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17557_5 = {S52, 0, &V17557_5_1222, &P17557_1};	/*Property = ScreenPixelsHeight*/
const Value V17557_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17557_4 = {S54, 0, &V17557_4_1224, &P17557_5};	/*Property = ScreenPixelsWidth*/
const Profile P17557 = {"17557", &P17557_4};	/*Profile ID - 17557*/
const Value V14338_1_1 = {S2, NULL};	/*Value = True*/
const Property P14338_1 = {S50, 0, &V14338_1_1, NULL};	/*Property = IsMobile*/
const Value V14338_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14338_5 = {S52, 0, &V14338_5_1380, &P14338_1};	/*Property = ScreenPixelsHeight*/
const Value V14338_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14338_4 = {S54, 0, &V14338_4_1279, &P14338_5};	/*Property = ScreenPixelsWidth*/
const Profile P14338 = {"14338", &P14338_4};	/*Profile ID - 14338*/
const Value V17616_1_1 = {S2, NULL};	/*Value = True*/
const Property P17616_1 = {S50, 0, &V17616_1_1, NULL};	/*Property = IsMobile*/
const Value V17616_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17616_5 = {S52, 0, &V17616_5_1233, &P17616_1};	/*Property = ScreenPixelsHeight*/
const Value V17616_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17616_4 = {S54, 0, &V17616_4_1235, &P17616_5};	/*Property = ScreenPixelsWidth*/
const Profile P17616 = {"17616", &P17616_4};	/*Profile ID - 17616*/
const Value V16616_1_1 = {S2, NULL};	/*Value = True*/
const Property P16616_1 = {S50, 0, &V16616_1_1, NULL};	/*Property = IsMobile*/
const Value V16616_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16616_5 = {S52, 0, &V16616_5_1293, &P16616_1};	/*Property = ScreenPixelsHeight*/
const Value V16616_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16616_4 = {S54, 0, &V16616_4_1296, &P16616_5};	/*Property = ScreenPixelsWidth*/
const Profile P16616 = {"16616", &P16616_4};	/*Profile ID - 16616*/
const Value V14344_1_1 = {S2, NULL};	/*Value = True*/
const Property P14344_1 = {S50, 0, &V14344_1_1, NULL};	/*Property = IsMobile*/
const Value V14344_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14344_5 = {S52, 0, &V14344_5_1222, &P14344_1};	/*Property = ScreenPixelsHeight*/
const Value V14344_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14344_4 = {S54, 0, &V14344_4_1224, &P14344_5};	/*Property = ScreenPixelsWidth*/
const Profile P14344 = {"14344", &P14344_4};	/*Profile ID - 14344*/
const Value V14348_1_1 = {S2, NULL};	/*Value = True*/
const Property P14348_1 = {S50, 0, &V14348_1_1, NULL};	/*Property = IsMobile*/
const Value V14348_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14348_5 = {S52, 0, &V14348_5_1222, &P14348_1};	/*Property = ScreenPixelsHeight*/
const Value V14348_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14348_4 = {S54, 0, &V14348_4_1224, &P14348_5};	/*Property = ScreenPixelsWidth*/
const Profile P14348 = {"14348", &P14348_4};	/*Profile ID - 14348*/
const Value V17907_1_1 = {S2, NULL};	/*Value = True*/
const Property P17907_1 = {S50, 0, &V17907_1_1, NULL};	/*Property = IsMobile*/
const Value V17907_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P17907_5 = {S52, 0, &V17907_5_1380, &P17907_1};	/*Property = ScreenPixelsHeight*/
const Value V17907_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17907_4 = {S54, 0, &V17907_4_1279, &P17907_5};	/*Property = ScreenPixelsWidth*/
const Profile P17907 = {"17907", &P17907_4};	/*Profile ID - 17907*/
const Value V14355_1_1 = {S2, NULL};	/*Value = True*/
const Property P14355_1 = {S50, 0, &V14355_1_1, NULL};	/*Property = IsMobile*/
const Value V14355_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14355_5 = {S52, 0, &V14355_5_1233, &P14355_1};	/*Property = ScreenPixelsHeight*/
const Value V14355_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14355_4 = {S54, 0, &V14355_4_1235, &P14355_5};	/*Property = ScreenPixelsWidth*/
const Profile P14355 = {"14355", &P14355_4};	/*Profile ID - 14355*/
const Value V14357_1_1 = {S2, NULL};	/*Value = True*/
const Property P14357_1 = {S50, 0, &V14357_1_1, NULL};	/*Property = IsMobile*/
const Value V14357_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14357_5 = {S52, 0, &V14357_5_1233, &P14357_1};	/*Property = ScreenPixelsHeight*/
const Value V14357_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14357_4 = {S54, 0, &V14357_4_1235, &P14357_5};	/*Property = ScreenPixelsWidth*/
const Profile P14357 = {"14357", &P14357_4};	/*Profile ID - 14357*/
const Value V14053_1_1 = {S2, NULL};	/*Value = True*/
const Property P14053_1 = {S50, 0, &V14053_1_1, NULL};	/*Property = IsMobile*/
const Value V14053_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14053_5 = {S52, 0, &V14053_5_1294, &P14053_1};	/*Property = ScreenPixelsHeight*/
const Value V14053_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14053_4 = {S54, 0, &V14053_4_1296, &P14053_5};	/*Property = ScreenPixelsWidth*/
const Profile P14053 = {"14053", &P14053_4};	/*Profile ID - 14053*/
const Value V14361_1_1 = {S2, NULL};	/*Value = True*/
const Property P14361_1 = {S50, 0, &V14361_1_1, NULL};	/*Property = IsMobile*/
const Value V14361_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14361_5 = {S52, 0, &V14361_5_1277, &P14361_1};	/*Property = ScreenPixelsHeight*/
const Value V14361_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14361_4 = {S54, 0, &V14361_4_1279, &P14361_5};	/*Property = ScreenPixelsWidth*/
const Profile P14361 = {"14361", &P14361_4};	/*Profile ID - 14361*/
const Value V14363_1_1 = {S2, NULL};	/*Value = True*/
const Property P14363_1 = {S50, 0, &V14363_1_1, NULL};	/*Property = IsMobile*/
const Value V14363_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14363_5 = {S52, 0, &V14363_5_1233, &P14363_1};	/*Property = ScreenPixelsHeight*/
const Value V14363_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14363_4 = {S54, 0, &V14363_4_1235, &P14363_5};	/*Property = ScreenPixelsWidth*/
const Profile P14363 = {"14363", &P14363_4};	/*Profile ID - 14363*/
const Value V14365_1_1 = {S2, NULL};	/*Value = True*/
const Property P14365_1 = {S50, 0, &V14365_1_1, NULL};	/*Property = IsMobile*/
const Value V14365_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P14365_5 = {S52, 0, &V14365_5_1333, &P14365_1};	/*Property = ScreenPixelsHeight*/
const Value V14365_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P14365_4 = {S54, 0, &V14365_4_1335, &P14365_5};	/*Property = ScreenPixelsWidth*/
const Profile P14365 = {"14365", &P14365_4};	/*Profile ID - 14365*/
const Value V14367_1_1 = {S2, NULL};	/*Value = True*/
const Property P14367_1 = {S50, 0, &V14367_1_1, NULL};	/*Property = IsMobile*/
const Value V14367_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14367_5 = {S52, 0, &V14367_5_1222, &P14367_1};	/*Property = ScreenPixelsHeight*/
const Value V14367_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14367_4 = {S54, 0, &V14367_4_1224, &P14367_5};	/*Property = ScreenPixelsWidth*/
const Profile P14367 = {"14367", &P14367_4};	/*Profile ID - 14367*/
const Value V14370_1_1 = {S2, NULL};	/*Value = True*/
const Property P14370_1 = {S50, 0, &V14370_1_1, NULL};	/*Property = IsMobile*/
const Value V14370_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14370_5 = {S52, 0, &V14370_5_1233, &P14370_1};	/*Property = ScreenPixelsHeight*/
const Value V14370_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14370_4 = {S54, 0, &V14370_4_1235, &P14370_5};	/*Property = ScreenPixelsWidth*/
const Profile P14370 = {"14370", &P14370_4};	/*Profile ID - 14370*/
const Value V14374_1_1 = {S2, NULL};	/*Value = True*/
const Property P14374_1 = {S50, 0, &V14374_1_1, NULL};	/*Property = IsMobile*/
const Value V14374_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14374_5 = {S52, 0, &V14374_5_1233, &P14374_1};	/*Property = ScreenPixelsHeight*/
const Value V14374_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14374_4 = {S54, 0, &V14374_4_1235, &P14374_5};	/*Property = ScreenPixelsWidth*/
const Profile P14374 = {"14374", &P14374_4};	/*Profile ID - 14374*/
const Value V14378_1_1 = {S2, NULL};	/*Value = True*/
const Property P14378_1 = {S50, 0, &V14378_1_1, NULL};	/*Property = IsMobile*/
const Value V14378_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14378_5 = {S52, 0, &V14378_5_1222, &P14378_1};	/*Property = ScreenPixelsHeight*/
const Value V14378_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14378_4 = {S54, 0, &V14378_4_1224, &P14378_5};	/*Property = ScreenPixelsWidth*/
const Profile P14378 = {"14378", &P14378_4};	/*Profile ID - 14378*/
const Value V14380_1_1 = {S2, NULL};	/*Value = True*/
const Property P14380_1 = {S50, 0, &V14380_1_1, NULL};	/*Property = IsMobile*/
const Value V14380_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14380_5 = {S52, 0, &V14380_5_1222, &P14380_1};	/*Property = ScreenPixelsHeight*/
const Value V14380_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14380_4 = {S54, 0, &V14380_4_1224, &P14380_5};	/*Property = ScreenPixelsWidth*/
const Profile P14380 = {"14380", &P14380_4};	/*Profile ID - 14380*/
const Value V14383_1_1 = {S2, NULL};	/*Value = True*/
const Property P14383_1 = {S50, 0, &V14383_1_1, NULL};	/*Property = IsMobile*/
const Value V14383_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14383_5 = {S52, 0, &V14383_5_1233, &P14383_1};	/*Property = ScreenPixelsHeight*/
const Value V14383_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14383_4 = {S54, 0, &V14383_4_1235, &P14383_5};	/*Property = ScreenPixelsWidth*/
const Profile P14383 = {"14383", &P14383_4};	/*Profile ID - 14383*/
const Value V14385_1_1 = {S2, NULL};	/*Value = True*/
const Property P14385_1 = {S50, 0, &V14385_1_1, NULL};	/*Property = IsMobile*/
const Value V14385_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14385_5 = {S52, 0, &V14385_5_1222, &P14385_1};	/*Property = ScreenPixelsHeight*/
const Value V14385_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14385_4 = {S54, 0, &V14385_4_1224, &P14385_5};	/*Property = ScreenPixelsWidth*/
const Profile P14385 = {"14385", &P14385_4};	/*Profile ID - 14385*/
const Value V14387_1_1 = {S2, NULL};	/*Value = True*/
const Property P14387_1 = {S50, 0, &V14387_1_1, NULL};	/*Property = IsMobile*/
const Value V14387_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14387_5 = {S52, 0, &V14387_5_1293, &P14387_1};	/*Property = ScreenPixelsHeight*/
const Value V14387_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14387_4 = {S54, 0, &V14387_4_1296, &P14387_5};	/*Property = ScreenPixelsWidth*/
const Profile P14387 = {"14387", &P14387_4};	/*Profile ID - 14387*/
const Value V16413_1_1 = {S2, NULL};	/*Value = True*/
const Property P16413_1 = {S50, 0, &V16413_1_1, NULL};	/*Property = IsMobile*/
const Value V16413_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P16413_5 = {S52, 0, &V16413_5_1304, &P16413_1};	/*Property = ScreenPixelsHeight*/
const Value V16413_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16413_4 = {S54, 0, &V16413_4_1224, &P16413_5};	/*Property = ScreenPixelsWidth*/
const Profile P16413 = {"16413", &P16413_4};	/*Profile ID - 16413*/
const Value V14388_1_1 = {S2, NULL};	/*Value = True*/
const Property P14388_1 = {S50, 0, &V14388_1_1, NULL};	/*Property = IsMobile*/
const Value V14388_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14388_5 = {S52, 0, &V14388_5_1222, &P14388_1};	/*Property = ScreenPixelsHeight*/
const Value V14388_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14388_4 = {S54, 0, &V14388_4_1224, &P14388_5};	/*Property = ScreenPixelsWidth*/
const Profile P14388 = {"14388", &P14388_4};	/*Profile ID - 14388*/
const Value V14389_1_1 = {S2, NULL};	/*Value = True*/
const Property P14389_1 = {S50, 0, &V14389_1_1, NULL};	/*Property = IsMobile*/
const Value V14389_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14389_5 = {S52, 0, &V14389_5_1222, &P14389_1};	/*Property = ScreenPixelsHeight*/
const Value V14389_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14389_4 = {S54, 0, &V14389_4_1224, &P14389_5};	/*Property = ScreenPixelsWidth*/
const Profile P14389 = {"14389", &P14389_4};	/*Profile ID - 14389*/
const Value V14391_1_1 = {S2, NULL};	/*Value = True*/
const Property P14391_1 = {S50, 0, &V14391_1_1, NULL};	/*Property = IsMobile*/
const Value V14391_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14391_5 = {S52, 0, &V14391_5_1294, &P14391_1};	/*Property = ScreenPixelsHeight*/
const Value V14391_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14391_4 = {S54, 0, &V14391_4_1296, &P14391_5};	/*Property = ScreenPixelsWidth*/
const Profile P14391 = {"14391", &P14391_4};	/*Profile ID - 14391*/
const Value V14393_1_1 = {S2, NULL};	/*Value = True*/
const Property P14393_1 = {S50, 0, &V14393_1_1, NULL};	/*Property = IsMobile*/
const Value V14393_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14393_5 = {S52, 0, &V14393_5_1222, &P14393_1};	/*Property = ScreenPixelsHeight*/
const Value V14393_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14393_4 = {S54, 0, &V14393_4_1224, &P14393_5};	/*Property = ScreenPixelsWidth*/
const Profile P14393 = {"14393", &P14393_4};	/*Profile ID - 14393*/
const Value V14396_1_1 = {S2, NULL};	/*Value = True*/
const Property P14396_1 = {S50, 0, &V14396_1_1, NULL};	/*Property = IsMobile*/
const Value V14396_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14396_5 = {S52, 0, &V14396_5_1233, &P14396_1};	/*Property = ScreenPixelsHeight*/
const Value V14396_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14396_4 = {S54, 0, &V14396_4_1235, &P14396_5};	/*Property = ScreenPixelsWidth*/
const Profile P14396 = {"14396", &P14396_4};	/*Profile ID - 14396*/
const Value V14542_1_1 = {S2, NULL};	/*Value = True*/
const Property P14542_1 = {S50, 0, &V14542_1_1, NULL};	/*Property = IsMobile*/
const Value V14542_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14542_5 = {S52, 0, &V14542_5_1222, &P14542_1};	/*Property = ScreenPixelsHeight*/
const Value V14542_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14542_4 = {S54, 0, &V14542_4_1224, &P14542_5};	/*Property = ScreenPixelsWidth*/
const Profile P14542 = {"14542", &P14542_4};	/*Profile ID - 14542*/
const Value V14589_1_1 = {S2, NULL};	/*Value = True*/
const Property P14589_1 = {S50, 0, &V14589_1_1, NULL};	/*Property = IsMobile*/
const Value V14589_5_37471 = {S98, NULL};	/*Value = 144*/
const Property P14589_5 = {S52, 0, &V14589_5_37471, &P14589_1};	/*Property = ScreenPixelsHeight*/
const Value V14589_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14589_4 = {S54, 0, &V14589_4_1235, &P14589_5};	/*Property = ScreenPixelsWidth*/
const Profile P14589 = {"14589", &P14589_4};	/*Profile ID - 14589*/
const Profile P6478 = {"6478", NULL};	/*Profile ID - 6478*/
const Value V14400_1_1 = {S2, NULL};	/*Value = True*/
const Property P14400_1 = {S50, 0, &V14400_1_1, NULL};	/*Property = IsMobile*/
const Value V14400_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14400_5 = {S52, 0, &V14400_5_1233, &P14400_1};	/*Property = ScreenPixelsHeight*/
const Value V14400_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14400_4 = {S54, 0, &V14400_4_1235, &P14400_5};	/*Property = ScreenPixelsWidth*/
const Profile P14400 = {"14400", &P14400_4};	/*Profile ID - 14400*/
const Value V16256_1_1 = {S2, NULL};	/*Value = True*/
const Property P16256_1 = {S50, 0, &V16256_1_1, NULL};	/*Property = IsMobile*/
const Value V16256_5_37465 = {S96, NULL};	/*Value = 80*/
const Property P16256_5 = {S52, 0, &V16256_5_37465, &P16256_1};	/*Property = ScreenPixelsHeight*/
const Value V16256_4_37576 = {S97, NULL};	/*Value = 101*/
const Property P16256_4 = {S54, 0, &V16256_4_37576, &P16256_5};	/*Property = ScreenPixelsWidth*/
const Profile P16256 = {"16256", &P16256_4};	/*Profile ID - 16256*/
const Value V16119_1_1 = {S2, NULL};	/*Value = True*/
const Property P16119_1 = {S50, 0, &V16119_1_1, NULL};	/*Property = IsMobile*/
const Value V16119_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16119_5 = {S52, 0, &V16119_5_1293, &P16119_1};	/*Property = ScreenPixelsHeight*/
const Value V16119_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16119_4 = {S54, 0, &V16119_4_1296, &P16119_5};	/*Property = ScreenPixelsWidth*/
const Profile P16119 = {"16119", &P16119_4};	/*Profile ID - 16119*/
const Value V17899_1_1 = {S2, NULL};	/*Value = True*/
const Property P17899_1 = {S50, 0, &V17899_1_1, NULL};	/*Property = IsMobile*/
const Value V17899_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P17899_5 = {S52, 0, &V17899_5_1380, &P17899_1};	/*Property = ScreenPixelsHeight*/
const Value V17899_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17899_4 = {S54, 0, &V17899_4_1279, &P17899_5};	/*Property = ScreenPixelsWidth*/
const Profile P17899 = {"17899", &P17899_4};	/*Profile ID - 17899*/
const Value V14402_1_1 = {S2, NULL};	/*Value = True*/
const Property P14402_1 = {S50, 0, &V14402_1_1, NULL};	/*Property = IsMobile*/
const Value V14402_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14402_5 = {S52, 0, &V14402_5_1233, &P14402_1};	/*Property = ScreenPixelsHeight*/
const Value V14402_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14402_4 = {S54, 0, &V14402_4_1235, &P14402_5};	/*Property = ScreenPixelsWidth*/
const Profile P14402 = {"14402", &P14402_4};	/*Profile ID - 14402*/
const Value V14403_1_1 = {S2, NULL};	/*Value = True*/
const Property P14403_1 = {S50, 0, &V14403_1_1, NULL};	/*Property = IsMobile*/
const Value V14403_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14403_5 = {S52, 0, &V14403_5_1380, &P14403_1};	/*Property = ScreenPixelsHeight*/
const Value V14403_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14403_4 = {S54, 0, &V14403_4_1279, &P14403_5};	/*Property = ScreenPixelsWidth*/
const Profile P14403 = {"14403", &P14403_4};	/*Profile ID - 14403*/
const Value V14405_1_1 = {S2, NULL};	/*Value = True*/
const Property P14405_1 = {S50, 0, &V14405_1_1, NULL};	/*Property = IsMobile*/
const Value V14405_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14405_5 = {S52, 0, &V14405_5_1222, &P14405_1};	/*Property = ScreenPixelsHeight*/
const Value V14405_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14405_4 = {S54, 0, &V14405_4_1224, &P14405_5};	/*Property = ScreenPixelsWidth*/
const Profile P14405 = {"14405", &P14405_4};	/*Profile ID - 14405*/
const Value V14407_1_1 = {S2, NULL};	/*Value = True*/
const Property P14407_1 = {S50, 0, &V14407_1_1, NULL};	/*Property = IsMobile*/
const Value V14407_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14407_5 = {S52, 0, &V14407_5_1222, &P14407_1};	/*Property = ScreenPixelsHeight*/
const Value V14407_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14407_4 = {S54, 0, &V14407_4_1224, &P14407_5};	/*Property = ScreenPixelsWidth*/
const Profile P14407 = {"14407", &P14407_4};	/*Profile ID - 14407*/
const Value V14412_1_1 = {S2, NULL};	/*Value = True*/
const Property P14412_1 = {S50, 0, &V14412_1_1, NULL};	/*Property = IsMobile*/
const Value V14412_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14412_5 = {S52, 0, &V14412_5_1293, &P14412_1};	/*Property = ScreenPixelsHeight*/
const Value V14412_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14412_4 = {S54, 0, &V14412_4_1296, &P14412_5};	/*Property = ScreenPixelsWidth*/
const Profile P14412 = {"14412", &P14412_4};	/*Profile ID - 14412*/
const Value V16434_1_1 = {S2, NULL};	/*Value = True*/
const Property P16434_1 = {S50, 0, &V16434_1_1, NULL};	/*Property = IsMobile*/
const Value V16434_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16434_5 = {S52, 0, &V16434_5_1233, &P16434_1};	/*Property = ScreenPixelsHeight*/
const Value V16434_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16434_4 = {S54, 0, &V16434_4_1235, &P16434_5};	/*Property = ScreenPixelsWidth*/
const Profile P16434 = {"16434", &P16434_4};	/*Profile ID - 16434*/
const Value V14416_1_1 = {S2, NULL};	/*Value = True*/
const Property P14416_1 = {S50, 0, &V14416_1_1, NULL};	/*Property = IsMobile*/
const Value V14416_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14416_5 = {S52, 0, &V14416_5_1222, &P14416_1};	/*Property = ScreenPixelsHeight*/
const Value V14416_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14416_4 = {S54, 0, &V14416_4_1224, &P14416_5};	/*Property = ScreenPixelsWidth*/
const Profile P14416 = {"14416", &P14416_4};	/*Profile ID - 14416*/
const Value V14476_1_1 = {S2, NULL};	/*Value = True*/
const Property P14476_1 = {S50, 0, &V14476_1_1, NULL};	/*Property = IsMobile*/
const Value V14476_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14476_5 = {S52, 0, &V14476_5_1293, &P14476_1};	/*Property = ScreenPixelsHeight*/
const Value V14476_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14476_4 = {S54, 0, &V14476_4_1296, &P14476_5};	/*Property = ScreenPixelsWidth*/
const Profile P14476 = {"14476", &P14476_4};	/*Profile ID - 14476*/
const Value V14420_1_1 = {S2, NULL};	/*Value = True*/
const Property P14420_1 = {S50, 0, &V14420_1_1, NULL};	/*Property = IsMobile*/
const Value V14420_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14420_5 = {S52, 0, &V14420_5_1233, &P14420_1};	/*Property = ScreenPixelsHeight*/
const Value V14420_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14420_4 = {S54, 0, &V14420_4_1235, &P14420_5};	/*Property = ScreenPixelsWidth*/
const Profile P14420 = {"14420", &P14420_4};	/*Profile ID - 14420*/
const Value V16481_1_1 = {S2, NULL};	/*Value = True*/
const Property P16481_1 = {S50, 0, &V16481_1_1, NULL};	/*Property = IsMobile*/
const Value V16481_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16481_5 = {S52, 0, &V16481_5_1293, &P16481_1};	/*Property = ScreenPixelsHeight*/
const Value V16481_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16481_4 = {S54, 0, &V16481_4_1296, &P16481_5};	/*Property = ScreenPixelsWidth*/
const Profile P16481 = {"16481", &P16481_4};	/*Profile ID - 16481*/
const Value V16613_1_1 = {S2, NULL};	/*Value = True*/
const Property P16613_1 = {S50, 0, &V16613_1_1, NULL};	/*Property = IsMobile*/
const Value V16613_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16613_5 = {S52, 0, &V16613_5_1293, &P16613_1};	/*Property = ScreenPixelsHeight*/
const Value V16613_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16613_4 = {S54, 0, &V16613_4_1296, &P16613_5};	/*Property = ScreenPixelsWidth*/
const Profile P16613 = {"16613", &P16613_4};	/*Profile ID - 16613*/
const Value V12590_1_1 = {S2, NULL};	/*Value = True*/
const Property P12590_1 = {S50, 0, &V12590_1_1, NULL};	/*Property = IsMobile*/
const Value V12590_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12590_5 = {S52, 0, &V12590_5_1380, &P12590_1};	/*Property = ScreenPixelsHeight*/
const Value V12590_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12590_4 = {S54, 0, &V12590_4_1279, &P12590_5};	/*Property = ScreenPixelsWidth*/
const Profile P12590 = {"12590", &P12590_4};	/*Profile ID - 12590*/
const Value V14422_1_1 = {S2, NULL};	/*Value = True*/
const Property P14422_1 = {S50, 0, &V14422_1_1, NULL};	/*Property = IsMobile*/
const Value V14422_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14422_5 = {S52, 0, &V14422_5_1222, &P14422_1};	/*Property = ScreenPixelsHeight*/
const Value V14422_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14422_4 = {S54, 0, &V14422_4_1224, &P14422_5};	/*Property = ScreenPixelsWidth*/
const Profile P14422 = {"14422", &P14422_4};	/*Profile ID - 14422*/
const Value V14423_1_1 = {S2, NULL};	/*Value = True*/
const Property P14423_1 = {S50, 0, &V14423_1_1, NULL};	/*Property = IsMobile*/
const Value V14423_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14423_5 = {S52, 0, &V14423_5_1233, &P14423_1};	/*Property = ScreenPixelsHeight*/
const Value V14423_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14423_4 = {S54, 0, &V14423_4_1235, &P14423_5};	/*Property = ScreenPixelsWidth*/
const Profile P14423 = {"14423", &P14423_4};	/*Profile ID - 14423*/
const Value V12340_1_1 = {S2, NULL};	/*Value = True*/
const Property P12340_1 = {S50, 0, &V12340_1_1, NULL};	/*Property = IsMobile*/
const Value V12340_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12340_5 = {S52, 0, &V12340_5_1222, &P12340_1};	/*Property = ScreenPixelsHeight*/
const Value V12340_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12340_4 = {S54, 0, &V12340_4_1224, &P12340_5};	/*Property = ScreenPixelsWidth*/
const Profile P12340 = {"12340", &P12340_4};	/*Profile ID - 12340*/
const Value V14429_1_1 = {S2, NULL};	/*Value = True*/
const Property P14429_1 = {S50, 0, &V14429_1_1, NULL};	/*Property = IsMobile*/
const Value V14429_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14429_5 = {S52, 0, &V14429_5_1304, &P14429_1};	/*Property = ScreenPixelsHeight*/
const Value V14429_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14429_4 = {S54, 0, &V14429_4_1224, &P14429_5};	/*Property = ScreenPixelsWidth*/
const Profile P14429 = {"14429", &P14429_4};	/*Profile ID - 14429*/
const Value V14430_1_1 = {S2, NULL};	/*Value = True*/
const Property P14430_1 = {S50, 0, &V14430_1_1, NULL};	/*Property = IsMobile*/
const Value V14430_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14430_5 = {S52, 0, &V14430_5_1222, &P14430_1};	/*Property = ScreenPixelsHeight*/
const Value V14430_4_1368 = {S74, NULL};	/*Value = 208*/
const Property P14430_4 = {S54, 0, &V14430_4_1368, &P14430_5};	/*Property = ScreenPixelsWidth*/
const Profile P14430 = {"14430", &P14430_4};	/*Profile ID - 14430*/
const Value V14435_1_1 = {S2, NULL};	/*Value = True*/
const Property P14435_1 = {S50, 0, &V14435_1_1, NULL};	/*Property = IsMobile*/
const Value V14435_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14435_5 = {S52, 0, &V14435_5_1222, &P14435_1};	/*Property = ScreenPixelsHeight*/
const Value V14435_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14435_4 = {S54, 0, &V14435_4_1224, &P14435_5};	/*Property = ScreenPixelsWidth*/
const Profile P14435 = {"14435", &P14435_4};	/*Profile ID - 14435*/
const Value V12115_1_1 = {S2, NULL};	/*Value = True*/
const Property P12115_1 = {S50, 0, &V12115_1_1, NULL};	/*Property = IsMobile*/
const Value V12115_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12115_5 = {S52, 0, &V12115_5_1304, &P12115_1};	/*Property = ScreenPixelsHeight*/
const Value V12115_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12115_4 = {S54, 0, &V12115_4_1224, &P12115_5};	/*Property = ScreenPixelsWidth*/
const Profile P12115 = {"12115", &P12115_4};	/*Profile ID - 12115*/
const Value V14442_1_1 = {S2, NULL};	/*Value = True*/
const Property P14442_1 = {S50, 0, &V14442_1_1, NULL};	/*Property = IsMobile*/
const Value V14442_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14442_5 = {S52, 0, &V14442_5_1233, &P14442_1};	/*Property = ScreenPixelsHeight*/
const Value V14442_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14442_4 = {S54, 0, &V14442_4_1235, &P14442_5};	/*Property = ScreenPixelsWidth*/
const Profile P14442 = {"14442", &P14442_4};	/*Profile ID - 14442*/
const Value V14448_1_1 = {S2, NULL};	/*Value = True*/
const Property P14448_1 = {S50, 0, &V14448_1_1, NULL};	/*Property = IsMobile*/
const Value V14448_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14448_5 = {S52, 0, &V14448_5_1293, &P14448_1};	/*Property = ScreenPixelsHeight*/
const Value V14448_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14448_4 = {S54, 0, &V14448_4_1296, &P14448_5};	/*Property = ScreenPixelsWidth*/
const Profile P14448 = {"14448", &P14448_4};	/*Profile ID - 14448*/
const Value V11922_1_1 = {S2, NULL};	/*Value = True*/
const Property P11922_1 = {S50, 0, &V11922_1_1, NULL};	/*Property = IsMobile*/
const Value V11922_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P11922_5 = {S52, 0, &V11922_5_1233, &P11922_1};	/*Property = ScreenPixelsHeight*/
const Value V11922_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P11922_4 = {S54, 0, &V11922_4_1235, &P11922_5};	/*Property = ScreenPixelsWidth*/
const Profile P11922 = {"11922", &P11922_4};	/*Profile ID - 11922*/
const Value V14449_1_1 = {S2, NULL};	/*Value = True*/
const Property P14449_1 = {S50, 0, &V14449_1_1, NULL};	/*Property = IsMobile*/
const Value V14449_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14449_5 = {S52, 0, &V14449_5_1222, &P14449_1};	/*Property = ScreenPixelsHeight*/
const Value V14449_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14449_4 = {S54, 0, &V14449_4_1224, &P14449_5};	/*Property = ScreenPixelsWidth*/
const Profile P14449 = {"14449", &P14449_4};	/*Profile ID - 14449*/
const Value V17081_1_1 = {S2, NULL};	/*Value = True*/
const Property P17081_1 = {S50, 0, &V17081_1_1, NULL};	/*Property = IsMobile*/
const Value V17081_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17081_5 = {S52, 0, &V17081_5_1233, &P17081_1};	/*Property = ScreenPixelsHeight*/
const Value V17081_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17081_4 = {S54, 0, &V17081_4_1235, &P17081_5};	/*Property = ScreenPixelsWidth*/
const Profile P17081 = {"17081", &P17081_4};	/*Profile ID - 17081*/
const Value V14965_1_1 = {S2, NULL};	/*Value = True*/
const Property P14965_1 = {S50, 0, &V14965_1_1, NULL};	/*Property = IsMobile*/
const Value V14965_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14965_5 = {S52, 0, &V14965_5_1233, &P14965_1};	/*Property = ScreenPixelsHeight*/
const Value V14965_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14965_4 = {S54, 0, &V14965_4_1235, &P14965_5};	/*Property = ScreenPixelsWidth*/
const Profile P14965 = {"14965", &P14965_4};	/*Profile ID - 14965*/
const Value V14458_1_1 = {S2, NULL};	/*Value = True*/
const Property P14458_1 = {S50, 0, &V14458_1_1, NULL};	/*Property = IsMobile*/
const Value V14458_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14458_5 = {S52, 0, &V14458_5_1233, &P14458_1};	/*Property = ScreenPixelsHeight*/
const Value V14458_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14458_4 = {S54, 0, &V14458_4_1235, &P14458_5};	/*Property = ScreenPixelsWidth*/
const Profile P14458 = {"14458", &P14458_4};	/*Profile ID - 14458*/
const Value V16189_1_1 = {S2, NULL};	/*Value = True*/
const Property P16189_1 = {S50, 0, &V16189_1_1, NULL};	/*Property = IsMobile*/
const Value V16189_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16189_5 = {S52, 0, &V16189_5_1222, &P16189_1};	/*Property = ScreenPixelsHeight*/
const Value V16189_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16189_4 = {S54, 0, &V16189_4_1224, &P16189_5};	/*Property = ScreenPixelsWidth*/
const Profile P16189 = {"16189", &P16189_4};	/*Profile ID - 16189*/
const Value V14461_1_1 = {S2, NULL};	/*Value = True*/
const Property P14461_1 = {S50, 0, &V14461_1_1, NULL};	/*Property = IsMobile*/
const Value V14461_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14461_5 = {S52, 0, &V14461_5_1222, &P14461_1};	/*Property = ScreenPixelsHeight*/
const Value V14461_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14461_4 = {S54, 0, &V14461_4_1224, &P14461_5};	/*Property = ScreenPixelsWidth*/
const Profile P14461 = {"14461", &P14461_4};	/*Profile ID - 14461*/
const Value V14464_1_1 = {S2, NULL};	/*Value = True*/
const Property P14464_1 = {S50, 0, &V14464_1_1, NULL};	/*Property = IsMobile*/
const Value V14464_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14464_5 = {S52, 0, &V14464_5_1380, &P14464_1};	/*Property = ScreenPixelsHeight*/
const Value V14464_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14464_4 = {S54, 0, &V14464_4_1224, &P14464_5};	/*Property = ScreenPixelsWidth*/
const Profile P14464 = {"14464", &P14464_4};	/*Profile ID - 14464*/
const Value V14469_1_1 = {S2, NULL};	/*Value = True*/
const Property P14469_1 = {S50, 0, &V14469_1_1, NULL};	/*Property = IsMobile*/
const Value V14469_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14469_5 = {S52, 0, &V14469_5_1293, &P14469_1};	/*Property = ScreenPixelsHeight*/
const Value V14469_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14469_4 = {S54, 0, &V14469_4_1296, &P14469_5};	/*Property = ScreenPixelsWidth*/
const Profile P14469 = {"14469", &P14469_4};	/*Profile ID - 14469*/
const Value V14466_1_1 = {S2, NULL};	/*Value = True*/
const Property P14466_1 = {S50, 0, &V14466_1_1, NULL};	/*Property = IsMobile*/
const Value V14466_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14466_5 = {S52, 0, &V14466_5_1222, &P14466_1};	/*Property = ScreenPixelsHeight*/
const Value V14466_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14466_4 = {S54, 0, &V14466_4_1224, &P14466_5};	/*Property = ScreenPixelsWidth*/
const Profile P14466 = {"14466", &P14466_4};	/*Profile ID - 14466*/
const Value V14470_1_1 = {S2, NULL};	/*Value = True*/
const Property P14470_1 = {S50, 0, &V14470_1_1, NULL};	/*Property = IsMobile*/
const Value V14470_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14470_5 = {S52, 0, &V14470_5_1380, &P14470_1};	/*Property = ScreenPixelsHeight*/
const Value V14470_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14470_4 = {S54, 0, &V14470_4_1279, &P14470_5};	/*Property = ScreenPixelsWidth*/
const Profile P14470 = {"14470", &P14470_4};	/*Profile ID - 14470*/
const Value V14471_1_1 = {S2, NULL};	/*Value = True*/
const Property P14471_1 = {S50, 0, &V14471_1_1, NULL};	/*Property = IsMobile*/
const Value V14471_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14471_5 = {S52, 0, &V14471_5_1233, &P14471_1};	/*Property = ScreenPixelsHeight*/
const Value V14471_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14471_4 = {S54, 0, &V14471_4_1235, &P14471_5};	/*Property = ScreenPixelsWidth*/
const Profile P14471 = {"14471", &P14471_4};	/*Profile ID - 14471*/
const Value V16204_1_1 = {S2, NULL};	/*Value = True*/
const Property P16204_1 = {S50, 0, &V16204_1_1, NULL};	/*Property = IsMobile*/
const Value V16204_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16204_5 = {S52, 0, &V16204_5_1222, &P16204_1};	/*Property = ScreenPixelsHeight*/
const Value V16204_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16204_4 = {S54, 0, &V16204_4_1224, &P16204_5};	/*Property = ScreenPixelsWidth*/
const Profile P16204 = {"16204", &P16204_4};	/*Profile ID - 16204*/
const Value V14477_1_1 = {S2, NULL};	/*Value = True*/
const Property P14477_1 = {S50, 0, &V14477_1_1, NULL};	/*Property = IsMobile*/
const Value V14477_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14477_5 = {S52, 0, &V14477_5_1233, &P14477_1};	/*Property = ScreenPixelsHeight*/
const Value V14477_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14477_4 = {S54, 0, &V14477_4_1235, &P14477_5};	/*Property = ScreenPixelsWidth*/
const Profile P14477 = {"14477", &P14477_4};	/*Profile ID - 14477*/
const Value V14479_1_1 = {S2, NULL};	/*Value = True*/
const Property P14479_1 = {S50, 0, &V14479_1_1, NULL};	/*Property = IsMobile*/
const Value V14479_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14479_5 = {S52, 0, &V14479_5_1233, &P14479_1};	/*Property = ScreenPixelsHeight*/
const Value V14479_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14479_4 = {S54, 0, &V14479_4_1235, &P14479_5};	/*Property = ScreenPixelsWidth*/
const Profile P14479 = {"14479", &P14479_4};	/*Profile ID - 14479*/
const Value V14480_1_1 = {S2, NULL};	/*Value = True*/
const Property P14480_1 = {S50, 0, &V14480_1_1, NULL};	/*Property = IsMobile*/
const Value V14480_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14480_5 = {S52, 0, &V14480_5_1222, &P14480_1};	/*Property = ScreenPixelsHeight*/
const Value V14480_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14480_4 = {S54, 0, &V14480_4_1224, &P14480_5};	/*Property = ScreenPixelsWidth*/
const Profile P14480 = {"14480", &P14480_4};	/*Profile ID - 14480*/
const Value V14481_1_1 = {S2, NULL};	/*Value = True*/
const Property P14481_1 = {S50, 0, &V14481_1_1, NULL};	/*Property = IsMobile*/
const Value V14481_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14481_5 = {S52, 0, &V14481_5_1233, &P14481_1};	/*Property = ScreenPixelsHeight*/
const Value V14481_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14481_4 = {S54, 0, &V14481_4_1235, &P14481_5};	/*Property = ScreenPixelsWidth*/
const Profile P14481 = {"14481", &P14481_4};	/*Profile ID - 14481*/
const Value V12474_1_1 = {S2, NULL};	/*Value = True*/
const Property P12474_1 = {S50, 0, &V12474_1_1, NULL};	/*Property = IsMobile*/
const Value V12474_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12474_5 = {S52, 0, &V12474_5_1233, &P12474_1};	/*Property = ScreenPixelsHeight*/
const Value V12474_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12474_4 = {S54, 0, &V12474_4_1235, &P12474_5};	/*Property = ScreenPixelsWidth*/
const Profile P12474 = {"12474", &P12474_4};	/*Profile ID - 12474*/
const Value V14485_1_1 = {S2, NULL};	/*Value = True*/
const Property P14485_1 = {S50, 0, &V14485_1_1, NULL};	/*Property = IsMobile*/
const Value V14485_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14485_5 = {S52, 0, &V14485_5_1222, &P14485_1};	/*Property = ScreenPixelsHeight*/
const Value V14485_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14485_4 = {S54, 0, &V14485_4_1224, &P14485_5};	/*Property = ScreenPixelsWidth*/
const Profile P14485 = {"14485", &P14485_4};	/*Profile ID - 14485*/
const Value V14486_1_1 = {S2, NULL};	/*Value = True*/
const Property P14486_1 = {S50, 0, &V14486_1_1, NULL};	/*Property = IsMobile*/
const Value V14486_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14486_5 = {S52, 0, &V14486_5_1304, &P14486_1};	/*Property = ScreenPixelsHeight*/
const Value V14486_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14486_4 = {S54, 0, &V14486_4_1224, &P14486_5};	/*Property = ScreenPixelsWidth*/
const Profile P14486 = {"14486", &P14486_4};	/*Profile ID - 14486*/
const Value V13652_1_1 = {S2, NULL};	/*Value = True*/
const Property P13652_1 = {S50, 0, &V13652_1_1, NULL};	/*Property = IsMobile*/
const Value V13652_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13652_5 = {S52, 0, &V13652_5_1222, &P13652_1};	/*Property = ScreenPixelsHeight*/
const Value V13652_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13652_4 = {S54, 0, &V13652_4_1224, &P13652_5};	/*Property = ScreenPixelsWidth*/
const Profile P13652 = {"13652", &P13652_4};	/*Profile ID - 13652*/
const Value V14487_1_1 = {S2, NULL};	/*Value = True*/
const Property P14487_1 = {S50, 0, &V14487_1_1, NULL};	/*Property = IsMobile*/
const Value V14487_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14487_5 = {S52, 0, &V14487_5_1222, &P14487_1};	/*Property = ScreenPixelsHeight*/
const Value V14487_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14487_4 = {S54, 0, &V14487_4_1224, &P14487_5};	/*Property = ScreenPixelsWidth*/
const Profile P14487 = {"14487", &P14487_4};	/*Profile ID - 14487*/
const Value V13318_1_1 = {S2, NULL};	/*Value = True*/
const Property P13318_1 = {S50, 0, &V13318_1_1, NULL};	/*Property = IsMobile*/
const Value V13318_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P13318_5 = {S52, 0, &V13318_5_1333, &P13318_1};	/*Property = ScreenPixelsHeight*/
const Value V13318_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P13318_4 = {S54, 0, &V13318_4_1335, &P13318_5};	/*Property = ScreenPixelsWidth*/
const Profile P13318 = {"13318", &P13318_4};	/*Profile ID - 13318*/
const Value V14494_1_1 = {S2, NULL};	/*Value = True*/
const Property P14494_1 = {S50, 0, &V14494_1_1, NULL};	/*Property = IsMobile*/
const Value V14494_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14494_5 = {S52, 0, &V14494_5_1293, &P14494_1};	/*Property = ScreenPixelsHeight*/
const Value V14494_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14494_4 = {S54, 0, &V14494_4_1296, &P14494_5};	/*Property = ScreenPixelsWidth*/
const Profile P14494 = {"14494", &P14494_4};	/*Profile ID - 14494*/
const Value V14498_1_1 = {S2, NULL};	/*Value = True*/
const Property P14498_1 = {S50, 0, &V14498_1_1, NULL};	/*Property = IsMobile*/
const Value V14498_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14498_5 = {S52, 0, &V14498_5_1304, &P14498_1};	/*Property = ScreenPixelsHeight*/
const Value V14498_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14498_4 = {S54, 0, &V14498_4_1224, &P14498_5};	/*Property = ScreenPixelsWidth*/
const Profile P14498 = {"14498", &P14498_4};	/*Profile ID - 14498*/
const Value V14501_1_1 = {S2, NULL};	/*Value = True*/
const Property P14501_1 = {S50, 0, &V14501_1_1, NULL};	/*Property = IsMobile*/
const Value V14501_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14501_5 = {S52, 0, &V14501_5_1233, &P14501_1};	/*Property = ScreenPixelsHeight*/
const Value V14501_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14501_4 = {S54, 0, &V14501_4_1235, &P14501_5};	/*Property = ScreenPixelsWidth*/
const Profile P14501 = {"14501", &P14501_4};	/*Profile ID - 14501*/
const Value V14502_1_1 = {S2, NULL};	/*Value = True*/
const Property P14502_1 = {S50, 0, &V14502_1_1, NULL};	/*Property = IsMobile*/
const Value V14502_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14502_5 = {S52, 0, &V14502_5_1233, &P14502_1};	/*Property = ScreenPixelsHeight*/
const Value V14502_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14502_4 = {S54, 0, &V14502_4_1235, &P14502_5};	/*Property = ScreenPixelsWidth*/
const Profile P14502 = {"14502", &P14502_4};	/*Profile ID - 14502*/
const Value V14505_1_1 = {S2, NULL};	/*Value = True*/
const Property P14505_1 = {S50, 0, &V14505_1_1, NULL};	/*Property = IsMobile*/
const Value V14505_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14505_5 = {S52, 0, &V14505_5_1294, &P14505_1};	/*Property = ScreenPixelsHeight*/
const Value V14505_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14505_4 = {S54, 0, &V14505_4_1296, &P14505_5};	/*Property = ScreenPixelsWidth*/
const Profile P14505 = {"14505", &P14505_4};	/*Profile ID - 14505*/
const Value V14575_1_1 = {S2, NULL};	/*Value = True*/
const Property P14575_1 = {S50, 0, &V14575_1_1, NULL};	/*Property = IsMobile*/
const Value V14575_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14575_5 = {S52, 0, &V14575_5_1293, &P14575_1};	/*Property = ScreenPixelsHeight*/
const Value V14575_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14575_4 = {S54, 0, &V14575_4_1296, &P14575_5};	/*Property = ScreenPixelsWidth*/
const Profile P14575 = {"14575", &P14575_4};	/*Profile ID - 14575*/
const Value V14507_1_1 = {S2, NULL};	/*Value = True*/
const Property P14507_1 = {S50, 0, &V14507_1_1, NULL};	/*Property = IsMobile*/
const Value V14507_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14507_5 = {S52, 0, &V14507_5_1304, &P14507_1};	/*Property = ScreenPixelsHeight*/
const Value V14507_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14507_4 = {S54, 0, &V14507_4_1224, &P14507_5};	/*Property = ScreenPixelsWidth*/
const Profile P14507 = {"14507", &P14507_4};	/*Profile ID - 14507*/
const Value V14508_1_1 = {S2, NULL};	/*Value = True*/
const Property P14508_1 = {S50, 0, &V14508_1_1, NULL};	/*Property = IsMobile*/
const Value V14508_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14508_5 = {S52, 0, &V14508_5_1294, &P14508_1};	/*Property = ScreenPixelsHeight*/
const Value V14508_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14508_4 = {S54, 0, &V14508_4_1296, &P14508_5};	/*Property = ScreenPixelsWidth*/
const Profile P14508 = {"14508", &P14508_4};	/*Profile ID - 14508*/
const Value V14245_1_1 = {S2, NULL};	/*Value = True*/
const Property P14245_1 = {S50, 0, &V14245_1_1, NULL};	/*Property = IsMobile*/
const Value V14245_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14245_5 = {S52, 0, &V14245_5_1233, &P14245_1};	/*Property = ScreenPixelsHeight*/
const Value V14245_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P14245_4 = {S54, 0, &V14245_4_1335, &P14245_5};	/*Property = ScreenPixelsWidth*/
const Profile P14245 = {"14245", &P14245_4};	/*Profile ID - 14245*/
const Value V14509_1_1 = {S2, NULL};	/*Value = True*/
const Property P14509_1 = {S50, 0, &V14509_1_1, NULL};	/*Property = IsMobile*/
const Value V14509_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P14509_5 = {S52, 0, &V14509_5_1333, &P14509_1};	/*Property = ScreenPixelsHeight*/
const Value V14509_4_37572 = {S93, NULL};	/*Value = 132*/
const Property P14509_4 = {S54, 0, &V14509_4_37572, &P14509_5};	/*Property = ScreenPixelsWidth*/
const Profile P14509 = {"14509", &P14509_4};	/*Profile ID - 14509*/
const Value V14511_1_1 = {S2, NULL};	/*Value = True*/
const Property P14511_1 = {S50, 0, &V14511_1_1, NULL};	/*Property = IsMobile*/
const Value V14511_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14511_5 = {S52, 0, &V14511_5_1380, &P14511_1};	/*Property = ScreenPixelsHeight*/
const Value V14511_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14511_4 = {S54, 0, &V14511_4_1279, &P14511_5};	/*Property = ScreenPixelsWidth*/
const Profile P14511 = {"14511", &P14511_4};	/*Profile ID - 14511*/
const Value V14513_1_1 = {S2, NULL};	/*Value = True*/
const Property P14513_1 = {S50, 0, &V14513_1_1, NULL};	/*Property = IsMobile*/
const Value V14513_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14513_5 = {S52, 0, &V14513_5_1222, &P14513_1};	/*Property = ScreenPixelsHeight*/
const Value V14513_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14513_4 = {S54, 0, &V14513_4_1224, &P14513_5};	/*Property = ScreenPixelsWidth*/
const Profile P14513 = {"14513", &P14513_4};	/*Profile ID - 14513*/
const Value V16601_1_1 = {S2, NULL};	/*Value = True*/
const Property P16601_1 = {S50, 0, &V16601_1_1, NULL};	/*Property = IsMobile*/
const Value V16601_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16601_5 = {S52, 0, &V16601_5_1222, &P16601_1};	/*Property = ScreenPixelsHeight*/
const Value V16601_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16601_4 = {S54, 0, &V16601_4_1224, &P16601_5};	/*Property = ScreenPixelsWidth*/
const Profile P16601 = {"16601", &P16601_4};	/*Profile ID - 16601*/
const Value V17707_1_1 = {S2, NULL};	/*Value = True*/
const Property P17707_1 = {S50, 0, &V17707_1_1, NULL};	/*Property = IsMobile*/
const Value V17707_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17707_5 = {S52, 0, &V17707_5_1293, &P17707_1};	/*Property = ScreenPixelsHeight*/
const Value V17707_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17707_4 = {S54, 0, &V17707_4_1296, &P17707_5};	/*Property = ScreenPixelsWidth*/
const Profile P17707 = {"17707", &P17707_4};	/*Profile ID - 17707*/
const Value V14517_1_1 = {S2, NULL};	/*Value = True*/
const Property P14517_1 = {S50, 0, &V14517_1_1, NULL};	/*Property = IsMobile*/
const Value V14517_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14517_5 = {S52, 0, &V14517_5_1222, &P14517_1};	/*Property = ScreenPixelsHeight*/
const Value V14517_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14517_4 = {S54, 0, &V14517_4_1224, &P14517_5};	/*Property = ScreenPixelsWidth*/
const Profile P14517 = {"14517", &P14517_4};	/*Profile ID - 14517*/
const Value V12050_1_1 = {S2, NULL};	/*Value = True*/
const Property P12050_1 = {S50, 0, &V12050_1_1, NULL};	/*Property = IsMobile*/
const Value V12050_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12050_5 = {S52, 0, &V12050_5_1304, &P12050_1};	/*Property = ScreenPixelsHeight*/
const Value V12050_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12050_4 = {S54, 0, &V12050_4_1224, &P12050_5};	/*Property = ScreenPixelsWidth*/
const Profile P12050 = {"12050", &P12050_4};	/*Profile ID - 12050*/
const Value V17806_1_1 = {S2, NULL};	/*Value = True*/
const Property P17806_1 = {S50, 0, &V17806_1_1, NULL};	/*Property = IsMobile*/
const Value V17806_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17806_5 = {S52, 0, &V17806_5_1222, &P17806_1};	/*Property = ScreenPixelsHeight*/
const Value V17806_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17806_4 = {S54, 0, &V17806_4_1224, &P17806_5};	/*Property = ScreenPixelsWidth*/
const Profile P17806 = {"17806", &P17806_4};	/*Profile ID - 17806*/
const Value V14529_1_1 = {S2, NULL};	/*Value = True*/
const Property P14529_1 = {S50, 0, &V14529_1_1, NULL};	/*Property = IsMobile*/
const Value V14529_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14529_5 = {S52, 0, &V14529_5_1294, &P14529_1};	/*Property = ScreenPixelsHeight*/
const Value V14529_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14529_4 = {S54, 0, &V14529_4_1296, &P14529_5};	/*Property = ScreenPixelsWidth*/
const Profile P14529 = {"14529", &P14529_4};	/*Profile ID - 14529*/
const Value V17113_1_1 = {S2, NULL};	/*Value = True*/
const Property P17113_1 = {S50, 0, &V17113_1_1, NULL};	/*Property = IsMobile*/
const Value V17113_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17113_5 = {S52, 0, &V17113_5_1222, &P17113_1};	/*Property = ScreenPixelsHeight*/
const Value V17113_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17113_4 = {S54, 0, &V17113_4_1224, &P17113_5};	/*Property = ScreenPixelsWidth*/
const Profile P17113 = {"17113", &P17113_4};	/*Profile ID - 17113*/
const Value V14522_1_1 = {S2, NULL};	/*Value = True*/
const Property P14522_1 = {S50, 0, &V14522_1_1, NULL};	/*Property = IsMobile*/
const Value V14522_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14522_5 = {S52, 0, &V14522_5_1222, &P14522_1};	/*Property = ScreenPixelsHeight*/
const Value V14522_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14522_4 = {S54, 0, &V14522_4_1224, &P14522_5};	/*Property = ScreenPixelsWidth*/
const Profile P14522 = {"14522", &P14522_4};	/*Profile ID - 14522*/
const Value V14526_1_1 = {S2, NULL};	/*Value = True*/
const Property P14526_1 = {S50, 0, &V14526_1_1, NULL};	/*Property = IsMobile*/
const Value V14526_5_37464 = {S112, NULL};	/*Value = 130*/
const Property P14526_5 = {S52, 0, &V14526_5_37464, &P14526_1};	/*Property = ScreenPixelsHeight*/
const Value V14526_4_37577 = {S112, NULL};	/*Value = 130*/
const Property P14526_4 = {S54, 0, &V14526_4_37577, &P14526_5};	/*Property = ScreenPixelsWidth*/
const Profile P14526 = {"14526", &P14526_4};	/*Profile ID - 14526*/
const Value V17618_1_1 = {S2, NULL};	/*Value = True*/
const Property P17618_1 = {S50, 0, &V17618_1_1, NULL};	/*Property = IsMobile*/
const Value V17618_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17618_5 = {S52, 0, &V17618_5_1222, &P17618_1};	/*Property = ScreenPixelsHeight*/
const Value V17618_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17618_4 = {S54, 0, &V17618_4_1224, &P17618_5};	/*Property = ScreenPixelsWidth*/
const Profile P17618 = {"17618", &P17618_4};	/*Profile ID - 17618*/
const Value V14530_1_1 = {S2, NULL};	/*Value = True*/
const Property P14530_1 = {S50, 0, &V14530_1_1, NULL};	/*Property = IsMobile*/
const Value V14530_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P14530_5 = {S52, 0, &V14530_5_1333, &P14530_1};	/*Property = ScreenPixelsHeight*/
const Value V14530_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P14530_4 = {S54, 0, &V14530_4_1335, &P14530_5};	/*Property = ScreenPixelsWidth*/
const Profile P14530 = {"14530", &P14530_4};	/*Profile ID - 14530*/
const Value V14532_1_1 = {S2, NULL};	/*Value = True*/
const Property P14532_1 = {S50, 0, &V14532_1_1, NULL};	/*Property = IsMobile*/
const Value V14532_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14532_5 = {S52, 0, &V14532_5_1222, &P14532_1};	/*Property = ScreenPixelsHeight*/
const Value V14532_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14532_4 = {S54, 0, &V14532_4_1224, &P14532_5};	/*Property = ScreenPixelsWidth*/
const Profile P14532 = {"14532", &P14532_4};	/*Profile ID - 14532*/
const Value V16024_1_1 = {S2, NULL};	/*Value = True*/
const Property P16024_1 = {S50, 0, &V16024_1_1, NULL};	/*Property = IsMobile*/
const Value V16024_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16024_5 = {S52, 0, &V16024_5_1222, &P16024_1};	/*Property = ScreenPixelsHeight*/
const Value V16024_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16024_4 = {S54, 0, &V16024_4_1224, &P16024_5};	/*Property = ScreenPixelsWidth*/
const Profile P16024 = {"16024", &P16024_4};	/*Profile ID - 16024*/
const Value V14535_1_1 = {S2, NULL};	/*Value = True*/
const Property P14535_1 = {S50, 0, &V14535_1_1, NULL};	/*Property = IsMobile*/
const Value V14535_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14535_5 = {S52, 0, &V14535_5_1293, &P14535_1};	/*Property = ScreenPixelsHeight*/
const Value V14535_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14535_4 = {S54, 0, &V14535_4_1296, &P14535_5};	/*Property = ScreenPixelsWidth*/
const Profile P14535 = {"14535", &P14535_4};	/*Profile ID - 14535*/
const Value V13583_1_1 = {S2, NULL};	/*Value = True*/
const Property P13583_1 = {S50, 0, &V13583_1_1, NULL};	/*Property = IsMobile*/
const Value V13583_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13583_5 = {S52, 0, &V13583_5_1293, &P13583_1};	/*Property = ScreenPixelsHeight*/
const Value V13583_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13583_4 = {S54, 0, &V13583_4_1296, &P13583_5};	/*Property = ScreenPixelsWidth*/
const Profile P13583 = {"13583", &P13583_4};	/*Profile ID - 13583*/
const Value V14537_1_1 = {S2, NULL};	/*Value = True*/
const Property P14537_1 = {S50, 0, &V14537_1_1, NULL};	/*Property = IsMobile*/
const Value V14537_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14537_5 = {S52, 0, &V14537_5_1380, &P14537_1};	/*Property = ScreenPixelsHeight*/
const Value V14537_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14537_4 = {S54, 0, &V14537_4_1279, &P14537_5};	/*Property = ScreenPixelsWidth*/
const Profile P14537 = {"14537", &P14537_4};	/*Profile ID - 14537*/
const Value V14539_1_1 = {S2, NULL};	/*Value = True*/
const Property P14539_1 = {S50, 0, &V14539_1_1, NULL};	/*Property = IsMobile*/
const Value V14539_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14539_5 = {S52, 0, &V14539_5_1304, &P14539_1};	/*Property = ScreenPixelsHeight*/
const Value V14539_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14539_4 = {S54, 0, &V14539_4_1224, &P14539_5};	/*Property = ScreenPixelsWidth*/
const Profile P14539 = {"14539", &P14539_4};	/*Profile ID - 14539*/
const Value V18058_1_1 = {S2, NULL};	/*Value = True*/
const Property P18058_1 = {S50, 0, &V18058_1_1, NULL};	/*Property = IsMobile*/
const Value V18058_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P18058_5 = {S52, 0, &V18058_5_1233, &P18058_1};	/*Property = ScreenPixelsHeight*/
const Value V18058_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P18058_4 = {S54, 0, &V18058_4_1235, &P18058_5};	/*Property = ScreenPixelsWidth*/
const Profile P18058 = {"18058", &P18058_4};	/*Profile ID - 18058*/
const Value V14541_1_1 = {S2, NULL};	/*Value = True*/
const Property P14541_1 = {S50, 0, &V14541_1_1, NULL};	/*Property = IsMobile*/
const Value V14541_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14541_5 = {S52, 0, &V14541_5_1222, &P14541_1};	/*Property = ScreenPixelsHeight*/
const Value V14541_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14541_4 = {S54, 0, &V14541_4_1224, &P14541_5};	/*Property = ScreenPixelsWidth*/
const Profile P14541 = {"14541", &P14541_4};	/*Profile ID - 14541*/
const Value V16856_1_1 = {S2, NULL};	/*Value = True*/
const Property P16856_1 = {S50, 0, &V16856_1_1, NULL};	/*Property = IsMobile*/
const Value V16856_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16856_5 = {S52, 0, &V16856_5_1233, &P16856_1};	/*Property = ScreenPixelsHeight*/
const Value V16856_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16856_4 = {S54, 0, &V16856_4_1235, &P16856_5};	/*Property = ScreenPixelsWidth*/
const Profile P16856 = {"16856", &P16856_4};	/*Profile ID - 16856*/
const Value V14555_1_1 = {S2, NULL};	/*Value = True*/
const Property P14555_1 = {S50, 0, &V14555_1_1, NULL};	/*Property = IsMobile*/
const Value V14555_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14555_5 = {S52, 0, &V14555_5_1380, &P14555_1};	/*Property = ScreenPixelsHeight*/
const Value V14555_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14555_4 = {S54, 0, &V14555_4_1279, &P14555_5};	/*Property = ScreenPixelsWidth*/
const Profile P14555 = {"14555", &P14555_4};	/*Profile ID - 14555*/
const Value V14556_1_1 = {S2, NULL};	/*Value = True*/
const Property P14556_1 = {S50, 0, &V14556_1_1, NULL};	/*Property = IsMobile*/
const Value V14556_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14556_5 = {S52, 0, &V14556_5_1222, &P14556_1};	/*Property = ScreenPixelsHeight*/
const Value V14556_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14556_4 = {S54, 0, &V14556_4_1224, &P14556_5};	/*Property = ScreenPixelsWidth*/
const Profile P14556 = {"14556", &P14556_4};	/*Profile ID - 14556*/
const Value V17798_1_1 = {S2, NULL};	/*Value = True*/
const Property P17798_1 = {S50, 0, &V17798_1_1, NULL};	/*Property = IsMobile*/
const Value V17798_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17798_5 = {S52, 0, &V17798_5_1304, &P17798_1};	/*Property = ScreenPixelsHeight*/
const Value V17798_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17798_4 = {S54, 0, &V17798_4_1224, &P17798_5};	/*Property = ScreenPixelsWidth*/
const Profile P17798 = {"17798", &P17798_4};	/*Profile ID - 17798*/
const Value V14560_1_1 = {S2, NULL};	/*Value = True*/
const Property P14560_1 = {S50, 0, &V14560_1_1, NULL};	/*Property = IsMobile*/
const Value V14560_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14560_5 = {S52, 0, &V14560_5_1293, &P14560_1};	/*Property = ScreenPixelsHeight*/
const Value V14560_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14560_4 = {S54, 0, &V14560_4_1296, &P14560_5};	/*Property = ScreenPixelsWidth*/
const Profile P14560 = {"14560", &P14560_4};	/*Profile ID - 14560*/
const Value V13508_1_1 = {S2, NULL};	/*Value = True*/
const Property P13508_1 = {S50, 0, &V13508_1_1, NULL};	/*Property = IsMobile*/
const Value V13508_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P13508_5 = {S52, 0, &V13508_5_1380, &P13508_1};	/*Property = ScreenPixelsHeight*/
const Value V13508_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13508_4 = {S54, 0, &V13508_4_1279, &P13508_5};	/*Property = ScreenPixelsWidth*/
const Profile P13508 = {"13508", &P13508_4};	/*Profile ID - 13508*/
const Value V14561_1_1 = {S2, NULL};	/*Value = True*/
const Property P14561_1 = {S50, 0, &V14561_1_1, NULL};	/*Property = IsMobile*/
const Value V14561_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14561_5 = {S52, 0, &V14561_5_1222, &P14561_1};	/*Property = ScreenPixelsHeight*/
const Value V14561_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14561_4 = {S54, 0, &V14561_4_1224, &P14561_5};	/*Property = ScreenPixelsWidth*/
const Profile P14561 = {"14561", &P14561_4};	/*Profile ID - 14561*/
const Value V14565_1_1 = {S2, NULL};	/*Value = True*/
const Property P14565_1 = {S50, 0, &V14565_1_1, NULL};	/*Property = IsMobile*/
const Value V14565_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14565_5 = {S52, 0, &V14565_5_1222, &P14565_1};	/*Property = ScreenPixelsHeight*/
const Value V14565_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14565_4 = {S54, 0, &V14565_4_1224, &P14565_5};	/*Property = ScreenPixelsWidth*/
const Profile P14565 = {"14565", &P14565_4};	/*Profile ID - 14565*/
const Value V18055_1_1 = {S2, NULL};	/*Value = True*/
const Property P18055_1 = {S50, 0, &V18055_1_1, NULL};	/*Property = IsMobile*/
const Value V18055_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P18055_5 = {S52, 0, &V18055_5_1304, &P18055_1};	/*Property = ScreenPixelsHeight*/
const Value V18055_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18055_4 = {S54, 0, &V18055_4_1224, &P18055_5};	/*Property = ScreenPixelsWidth*/
const Profile P18055 = {"18055", &P18055_4};	/*Profile ID - 18055*/
const Value V14567_1_1 = {S2, NULL};	/*Value = True*/
const Property P14567_1 = {S50, 0, &V14567_1_1, NULL};	/*Property = IsMobile*/
const Value V14567_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P14567_5 = {S52, 0, &V14567_5_1333, &P14567_1};	/*Property = ScreenPixelsHeight*/
const Value V14567_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P14567_4 = {S54, 0, &V14567_4_1335, &P14567_5};	/*Property = ScreenPixelsWidth*/
const Profile P14567 = {"14567", &P14567_4};	/*Profile ID - 14567*/
const Value V14568_1_1 = {S2, NULL};	/*Value = True*/
const Property P14568_1 = {S50, 0, &V14568_1_1, NULL};	/*Property = IsMobile*/
const Value V14568_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14568_5 = {S52, 0, &V14568_5_1233, &P14568_1};	/*Property = ScreenPixelsHeight*/
const Value V14568_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14568_4 = {S54, 0, &V14568_4_1235, &P14568_5};	/*Property = ScreenPixelsWidth*/
const Profile P14568 = {"14568", &P14568_4};	/*Profile ID - 14568*/
const Value V14570_1_1 = {S2, NULL};	/*Value = True*/
const Property P14570_1 = {S50, 0, &V14570_1_1, NULL};	/*Property = IsMobile*/
const Value V14570_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14570_5 = {S52, 0, &V14570_5_1380, &P14570_1};	/*Property = ScreenPixelsHeight*/
const Value V14570_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14570_4 = {S54, 0, &V14570_4_1279, &P14570_5};	/*Property = ScreenPixelsWidth*/
const Profile P14570 = {"14570", &P14570_4};	/*Profile ID - 14570*/
const Value V14571_1_1 = {S2, NULL};	/*Value = True*/
const Property P14571_1 = {S50, 0, &V14571_1_1, NULL};	/*Property = IsMobile*/
const Value V14571_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14571_5 = {S52, 0, &V14571_5_1294, &P14571_1};	/*Property = ScreenPixelsHeight*/
const Value V14571_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14571_4 = {S54, 0, &V14571_4_1296, &P14571_5};	/*Property = ScreenPixelsWidth*/
const Profile P14571 = {"14571", &P14571_4};	/*Profile ID - 14571*/
const Value V13199_1_1 = {S2, NULL};	/*Value = True*/
const Property P13199_1 = {S50, 0, &V13199_1_1, NULL};	/*Property = IsMobile*/
const Value V13199_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13199_5 = {S52, 0, &V13199_5_1293, &P13199_1};	/*Property = ScreenPixelsHeight*/
const Value V13199_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13199_4 = {S54, 0, &V13199_4_1296, &P13199_5};	/*Property = ScreenPixelsWidth*/
const Profile P13199 = {"13199", &P13199_4};	/*Profile ID - 13199*/
const Value V14576_1_1 = {S2, NULL};	/*Value = True*/
const Property P14576_1 = {S50, 0, &V14576_1_1, NULL};	/*Property = IsMobile*/
const Value V14576_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14576_5 = {S52, 0, &V14576_5_1277, &P14576_1};	/*Property = ScreenPixelsHeight*/
const Value V14576_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14576_4 = {S54, 0, &V14576_4_1224, &P14576_5};	/*Property = ScreenPixelsWidth*/
const Profile P14576 = {"14576", &P14576_4};	/*Profile ID - 14576*/
const Value V14577_1_1 = {S2, NULL};	/*Value = True*/
const Property P14577_1 = {S50, 0, &V14577_1_1, NULL};	/*Property = IsMobile*/
const Value V14577_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14577_5 = {S52, 0, &V14577_5_1233, &P14577_1};	/*Property = ScreenPixelsHeight*/
const Value V14577_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14577_4 = {S54, 0, &V14577_4_1235, &P14577_5};	/*Property = ScreenPixelsWidth*/
const Profile P14577 = {"14577", &P14577_4};	/*Profile ID - 14577*/
const Value V17332_1_1 = {S2, NULL};	/*Value = True*/
const Property P17332_1 = {S50, 0, &V17332_1_1, NULL};	/*Property = IsMobile*/
const Value V17332_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17332_5 = {S52, 0, &V17332_5_1222, &P17332_1};	/*Property = ScreenPixelsHeight*/
const Value V17332_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17332_4 = {S54, 0, &V17332_4_1224, &P17332_5};	/*Property = ScreenPixelsWidth*/
const Profile P17332 = {"17332", &P17332_4};	/*Profile ID - 17332*/
const Value V23539_1_1 = {S2, NULL};	/*Value = True*/
const Property P23539_1 = {S50, 0, &V23539_1_1, NULL};	/*Property = IsMobile*/
const Value V23539_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23539_5 = {S52, 0, &V23539_5_1222, &P23539_1};	/*Property = ScreenPixelsHeight*/
const Value V23539_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23539_4 = {S54, 0, &V23539_4_1224, &P23539_5};	/*Property = ScreenPixelsWidth*/
const Profile P23539 = {"23539", &P23539_4};	/*Profile ID - 23539*/
const Value V14578_1_1 = {S2, NULL};	/*Value = True*/
const Property P14578_1 = {S50, 0, &V14578_1_1, NULL};	/*Property = IsMobile*/
const Value V14578_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14578_5 = {S52, 0, &V14578_5_1222, &P14578_1};	/*Property = ScreenPixelsHeight*/
const Value V14578_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14578_4 = {S54, 0, &V14578_4_1224, &P14578_5};	/*Property = ScreenPixelsWidth*/
const Profile P14578 = {"14578", &P14578_4};	/*Profile ID - 14578*/
const Value V14580_1_1 = {S2, NULL};	/*Value = True*/
const Property P14580_1 = {S50, 0, &V14580_1_1, NULL};	/*Property = IsMobile*/
const Value V14580_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14580_5 = {S52, 0, &V14580_5_1222, &P14580_1};	/*Property = ScreenPixelsHeight*/
const Value V14580_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14580_4 = {S54, 0, &V14580_4_1224, &P14580_5};	/*Property = ScreenPixelsWidth*/
const Profile P14580 = {"14580", &P14580_4};	/*Profile ID - 14580*/
const Value V12298_1_1 = {S2, NULL};	/*Value = True*/
const Property P12298_1 = {S50, 0, &V12298_1_1, NULL};	/*Property = IsMobile*/
const Value V12298_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12298_5 = {S52, 0, &V12298_5_1304, &P12298_1};	/*Property = ScreenPixelsHeight*/
const Value V12298_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12298_4 = {S54, 0, &V12298_4_1224, &P12298_5};	/*Property = ScreenPixelsWidth*/
const Profile P12298 = {"12298", &P12298_4};	/*Profile ID - 12298*/
const Value V14581_1_1 = {S2, NULL};	/*Value = True*/
const Property P14581_1 = {S50, 0, &V14581_1_1, NULL};	/*Property = IsMobile*/
const Value V14581_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14581_5 = {S52, 0, &V14581_5_1233, &P14581_1};	/*Property = ScreenPixelsHeight*/
const Value V14581_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14581_4 = {S54, 0, &V14581_4_1235, &P14581_5};	/*Property = ScreenPixelsWidth*/
const Profile P14581 = {"14581", &P14581_4};	/*Profile ID - 14581*/
const Value V13365_1_1 = {S2, NULL};	/*Value = True*/
const Property P13365_1 = {S50, 0, &V13365_1_1, NULL};	/*Property = IsMobile*/
const Value V13365_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13365_5 = {S52, 0, &V13365_5_1233, &P13365_1};	/*Property = ScreenPixelsHeight*/
const Value V13365_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13365_4 = {S54, 0, &V13365_4_1235, &P13365_5};	/*Property = ScreenPixelsWidth*/
const Profile P13365 = {"13365", &P13365_4};	/*Profile ID - 13365*/
const Value V14585_1_1 = {S2, NULL};	/*Value = True*/
const Property P14585_1 = {S50, 0, &V14585_1_1, NULL};	/*Property = IsMobile*/
const Value V14585_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14585_5 = {S52, 0, &V14585_5_1222, &P14585_1};	/*Property = ScreenPixelsHeight*/
const Value V14585_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14585_4 = {S54, 0, &V14585_4_1224, &P14585_5};	/*Property = ScreenPixelsWidth*/
const Profile P14585 = {"14585", &P14585_4};	/*Profile ID - 14585*/
const Value V14586_1_1 = {S2, NULL};	/*Value = True*/
const Property P14586_1 = {S50, 0, &V14586_1_1, NULL};	/*Property = IsMobile*/
const Value V14586_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14586_5 = {S52, 0, &V14586_5_1233, &P14586_1};	/*Property = ScreenPixelsHeight*/
const Value V14586_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14586_4 = {S54, 0, &V14586_4_1235, &P14586_5};	/*Property = ScreenPixelsWidth*/
const Profile P14586 = {"14586", &P14586_4};	/*Profile ID - 14586*/
const Value V14591_1_1 = {S2, NULL};	/*Value = True*/
const Property P14591_1 = {S50, 0, &V14591_1_1, NULL};	/*Property = IsMobile*/
const Value V14591_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14591_5 = {S52, 0, &V14591_5_1293, &P14591_1};	/*Property = ScreenPixelsHeight*/
const Value V14591_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14591_4 = {S54, 0, &V14591_4_1296, &P14591_5};	/*Property = ScreenPixelsWidth*/
const Profile P14591 = {"14591", &P14591_4};	/*Profile ID - 14591*/
const Value V16646_1_1 = {S2, NULL};	/*Value = True*/
const Property P16646_1 = {S50, 0, &V16646_1_1, NULL};	/*Property = IsMobile*/
const Value V16646_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16646_5 = {S52, 0, &V16646_5_1293, &P16646_1};	/*Property = ScreenPixelsHeight*/
const Value V16646_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16646_4 = {S54, 0, &V16646_4_1296, &P16646_5};	/*Property = ScreenPixelsWidth*/
const Profile P16646 = {"16646", &P16646_4};	/*Profile ID - 16646*/
const Value V14594_1_1 = {S2, NULL};	/*Value = True*/
const Property P14594_1 = {S50, 0, &V14594_1_1, NULL};	/*Property = IsMobile*/
const Value V14594_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14594_5 = {S52, 0, &V14594_5_1233, &P14594_1};	/*Property = ScreenPixelsHeight*/
const Value V14594_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14594_4 = {S54, 0, &V14594_4_1235, &P14594_5};	/*Property = ScreenPixelsWidth*/
const Profile P14594 = {"14594", &P14594_4};	/*Profile ID - 14594*/
const Value V14595_1_1 = {S2, NULL};	/*Value = True*/
const Property P14595_1 = {S50, 0, &V14595_1_1, NULL};	/*Property = IsMobile*/
const Value V14595_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14595_5 = {S52, 0, &V14595_5_1277, &P14595_1};	/*Property = ScreenPixelsHeight*/
const Value V14595_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14595_4 = {S54, 0, &V14595_4_1279, &P14595_5};	/*Property = ScreenPixelsWidth*/
const Profile P14595 = {"14595", &P14595_4};	/*Profile ID - 14595*/
const Value V14663_1_1 = {S2, NULL};	/*Value = True*/
const Property P14663_1 = {S50, 0, &V14663_1_1, NULL};	/*Property = IsMobile*/
const Value V14663_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14663_5 = {S52, 0, &V14663_5_1380, &P14663_1};	/*Property = ScreenPixelsHeight*/
const Value V14663_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14663_4 = {S54, 0, &V14663_4_1279, &P14663_5};	/*Property = ScreenPixelsWidth*/
const Profile P14663 = {"14663", &P14663_4};	/*Profile ID - 14663*/
const Value V14665_1_1 = {S2, NULL};	/*Value = True*/
const Property P14665_1 = {S50, 0, &V14665_1_1, NULL};	/*Property = IsMobile*/
const Value V14665_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14665_5 = {S52, 0, &V14665_5_1304, &P14665_1};	/*Property = ScreenPixelsHeight*/
const Value V14665_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14665_4 = {S54, 0, &V14665_4_1224, &P14665_5};	/*Property = ScreenPixelsWidth*/
const Profile P14665 = {"14665", &P14665_4};	/*Profile ID - 14665*/
const Value V14667_1_1 = {S2, NULL};	/*Value = True*/
const Property P14667_1 = {S50, 0, &V14667_1_1, NULL};	/*Property = IsMobile*/
const Value V14667_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14667_5 = {S52, 0, &V14667_5_1304, &P14667_1};	/*Property = ScreenPixelsHeight*/
const Value V14667_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14667_4 = {S54, 0, &V14667_4_1224, &P14667_5};	/*Property = ScreenPixelsWidth*/
const Profile P14667 = {"14667", &P14667_4};	/*Profile ID - 14667*/
const Value V14670_1_1 = {S2, NULL};	/*Value = True*/
const Property P14670_1 = {S50, 0, &V14670_1_1, NULL};	/*Property = IsMobile*/
const Value V14670_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14670_5 = {S52, 0, &V14670_5_1233, &P14670_1};	/*Property = ScreenPixelsHeight*/
const Value V14670_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14670_4 = {S54, 0, &V14670_4_1235, &P14670_5};	/*Property = ScreenPixelsWidth*/
const Profile P14670 = {"14670", &P14670_4};	/*Profile ID - 14670*/
const Value V12679_1_1 = {S2, NULL};	/*Value = True*/
const Property P12679_1 = {S50, 0, &V12679_1_1, NULL};	/*Property = IsMobile*/
const Value V12679_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12679_5 = {S52, 0, &V12679_5_1222, &P12679_1};	/*Property = ScreenPixelsHeight*/
const Value V12679_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12679_4 = {S54, 0, &V12679_4_1224, &P12679_5};	/*Property = ScreenPixelsWidth*/
const Profile P12679 = {"12679", &P12679_4};	/*Profile ID - 12679*/
const Value V14676_1_1 = {S2, NULL};	/*Value = True*/
const Property P14676_1 = {S50, 0, &V14676_1_1, NULL};	/*Property = IsMobile*/
const Value V14676_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P14676_5 = {S52, 0, &V14676_5_1280, &P14676_1};	/*Property = ScreenPixelsHeight*/
const Value V14676_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14676_4 = {S54, 0, &V14676_4_1235, &P14676_5};	/*Property = ScreenPixelsWidth*/
const Profile P14676 = {"14676", &P14676_4};	/*Profile ID - 14676*/
const Value V14680_1_1 = {S2, NULL};	/*Value = True*/
const Property P14680_1 = {S50, 0, &V14680_1_1, NULL};	/*Property = IsMobile*/
const Value V14680_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14680_5 = {S52, 0, &V14680_5_1233, &P14680_1};	/*Property = ScreenPixelsHeight*/
const Value V14680_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14680_4 = {S54, 0, &V14680_4_1235, &P14680_5};	/*Property = ScreenPixelsWidth*/
const Profile P14680 = {"14680", &P14680_4};	/*Profile ID - 14680*/
const Value V14681_1_1 = {S2, NULL};	/*Value = True*/
const Property P14681_1 = {S50, 0, &V14681_1_1, NULL};	/*Property = IsMobile*/
const Value V14681_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14681_5 = {S52, 0, &V14681_5_1293, &P14681_1};	/*Property = ScreenPixelsHeight*/
const Value V14681_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14681_4 = {S54, 0, &V14681_4_1296, &P14681_5};	/*Property = ScreenPixelsWidth*/
const Profile P14681 = {"14681", &P14681_4};	/*Profile ID - 14681*/
const Value V14683_1_1 = {S2, NULL};	/*Value = True*/
const Property P14683_1 = {S50, 0, &V14683_1_1, NULL};	/*Property = IsMobile*/
const Value V14683_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14683_5 = {S52, 0, &V14683_5_1293, &P14683_1};	/*Property = ScreenPixelsHeight*/
const Value V14683_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14683_4 = {S54, 0, &V14683_4_1296, &P14683_5};	/*Property = ScreenPixelsWidth*/
const Profile P14683 = {"14683", &P14683_4};	/*Profile ID - 14683*/
const Value V14685_1_1 = {S2, NULL};	/*Value = True*/
const Property P14685_1 = {S50, 0, &V14685_1_1, NULL};	/*Property = IsMobile*/
const Value V14685_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14685_5 = {S52, 0, &V14685_5_1294, &P14685_1};	/*Property = ScreenPixelsHeight*/
const Value V14685_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14685_4 = {S54, 0, &V14685_4_1296, &P14685_5};	/*Property = ScreenPixelsWidth*/
const Profile P14685 = {"14685", &P14685_4};	/*Profile ID - 14685*/
const Value V14688_1_1 = {S2, NULL};	/*Value = True*/
const Property P14688_1 = {S50, 0, &V14688_1_1, NULL};	/*Property = IsMobile*/
const Value V14688_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14688_5 = {S52, 0, &V14688_5_1293, &P14688_1};	/*Property = ScreenPixelsHeight*/
const Value V14688_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14688_4 = {S54, 0, &V14688_4_1296, &P14688_5};	/*Property = ScreenPixelsWidth*/
const Profile P14688 = {"14688", &P14688_4};	/*Profile ID - 14688*/
const Value V14689_1_1 = {S2, NULL};	/*Value = True*/
const Property P14689_1 = {S50, 0, &V14689_1_1, NULL};	/*Property = IsMobile*/
const Value V14689_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14689_5 = {S52, 0, &V14689_5_1293, &P14689_1};	/*Property = ScreenPixelsHeight*/
const Value V14689_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14689_4 = {S54, 0, &V14689_4_1296, &P14689_5};	/*Property = ScreenPixelsWidth*/
const Profile P14689 = {"14689", &P14689_4};	/*Profile ID - 14689*/
const Value V16217_1_1 = {S2, NULL};	/*Value = True*/
const Property P16217_1 = {S50, 0, &V16217_1_1, NULL};	/*Property = IsMobile*/
const Value V16217_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16217_5 = {S52, 0, &V16217_5_1227, &P16217_1};	/*Property = ScreenPixelsHeight*/
const Value V16217_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16217_4 = {S54, 0, &V16217_4_1229, &P16217_5};	/*Property = ScreenPixelsWidth*/
const Profile P16217 = {"16217", &P16217_4};	/*Profile ID - 16217*/
const Value V14690_1_1 = {S2, NULL};	/*Value = True*/
const Property P14690_1 = {S50, 0, &V14690_1_1, NULL};	/*Property = IsMobile*/
const Value V14690_5_31826 = {S75, NULL};	/*Value = 260*/
const Property P14690_5 = {S52, 0, &V14690_5_31826, &P14690_1};	/*Property = ScreenPixelsHeight*/
const Value V14690_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14690_4 = {S54, 0, &V14690_4_1224, &P14690_5};	/*Property = ScreenPixelsWidth*/
const Profile P14690 = {"14690", &P14690_4};	/*Profile ID - 14690*/
const Value V14701_1_1 = {S2, NULL};	/*Value = True*/
const Property P14701_1 = {S50, 0, &V14701_1_1, NULL};	/*Property = IsMobile*/
const Value V14701_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14701_5 = {S52, 0, &V14701_5_1233, &P14701_1};	/*Property = ScreenPixelsHeight*/
const Value V14701_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14701_4 = {S54, 0, &V14701_4_1235, &P14701_5};	/*Property = ScreenPixelsWidth*/
const Profile P14701 = {"14701", &P14701_4};	/*Profile ID - 14701*/
const Value V14702_1_1 = {S2, NULL};	/*Value = True*/
const Property P14702_1 = {S50, 0, &V14702_1_1, NULL};	/*Property = IsMobile*/
const Value V14702_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14702_5 = {S52, 0, &V14702_5_1222, &P14702_1};	/*Property = ScreenPixelsHeight*/
const Value V14702_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14702_4 = {S54, 0, &V14702_4_1224, &P14702_5};	/*Property = ScreenPixelsWidth*/
const Profile P14702 = {"14702", &P14702_4};	/*Profile ID - 14702*/
const Value V14703_1_1 = {S2, NULL};	/*Value = True*/
const Property P14703_1 = {S50, 0, &V14703_1_1, NULL};	/*Property = IsMobile*/
const Value V14703_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14703_5 = {S52, 0, &V14703_5_1222, &P14703_1};	/*Property = ScreenPixelsHeight*/
const Value V14703_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14703_4 = {S54, 0, &V14703_4_1224, &P14703_5};	/*Property = ScreenPixelsWidth*/
const Profile P14703 = {"14703", &P14703_4};	/*Profile ID - 14703*/
const Value V14704_1_1 = {S2, NULL};	/*Value = True*/
const Property P14704_1 = {S50, 0, &V14704_1_1, NULL};	/*Property = IsMobile*/
const Value V14704_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14704_5 = {S52, 0, &V14704_5_1222, &P14704_1};	/*Property = ScreenPixelsHeight*/
const Value V14704_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14704_4 = {S54, 0, &V14704_4_1224, &P14704_5};	/*Property = ScreenPixelsWidth*/
const Profile P14704 = {"14704", &P14704_4};	/*Profile ID - 14704*/
const Value V14707_1_1 = {S2, NULL};	/*Value = True*/
const Property P14707_1 = {S50, 0, &V14707_1_1, NULL};	/*Property = IsMobile*/
const Value V14707_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14707_5 = {S52, 0, &V14707_5_1293, &P14707_1};	/*Property = ScreenPixelsHeight*/
const Value V14707_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14707_4 = {S54, 0, &V14707_4_1296, &P14707_5};	/*Property = ScreenPixelsWidth*/
const Profile P14707 = {"14707", &P14707_4};	/*Profile ID - 14707*/
const Value V17692_1_1 = {S2, NULL};	/*Value = True*/
const Property P17692_1 = {S50, 0, &V17692_1_1, NULL};	/*Property = IsMobile*/
const Value V17692_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17692_5 = {S52, 0, &V17692_5_1293, &P17692_1};	/*Property = ScreenPixelsHeight*/
const Value V17692_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17692_4 = {S54, 0, &V17692_4_1296, &P17692_5};	/*Property = ScreenPixelsWidth*/
const Profile P17692 = {"17692", &P17692_4};	/*Profile ID - 17692*/
const Value V14710_1_1 = {S2, NULL};	/*Value = True*/
const Property P14710_1 = {S50, 0, &V14710_1_1, NULL};	/*Property = IsMobile*/
const Value V14710_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14710_5 = {S52, 0, &V14710_5_1293, &P14710_1};	/*Property = ScreenPixelsHeight*/
const Value V14710_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14710_4 = {S54, 0, &V14710_4_1296, &P14710_5};	/*Property = ScreenPixelsWidth*/
const Profile P14710 = {"14710", &P14710_4};	/*Profile ID - 14710*/
const Value V14712_1_1 = {S2, NULL};	/*Value = True*/
const Property P14712_1 = {S50, 0, &V14712_1_1, NULL};	/*Property = IsMobile*/
const Value V14712_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14712_5 = {S52, 0, &V14712_5_1304, &P14712_1};	/*Property = ScreenPixelsHeight*/
const Value V14712_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14712_4 = {S54, 0, &V14712_4_1224, &P14712_5};	/*Property = ScreenPixelsWidth*/
const Profile P14712 = {"14712", &P14712_4};	/*Profile ID - 14712*/
const Value V17333_1_1 = {S2, NULL};	/*Value = True*/
const Property P17333_1 = {S50, 0, &V17333_1_1, NULL};	/*Property = IsMobile*/
const Value V17333_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17333_5 = {S52, 0, &V17333_5_1293, &P17333_1};	/*Property = ScreenPixelsHeight*/
const Value V17333_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17333_4 = {S54, 0, &V17333_4_1296, &P17333_5};	/*Property = ScreenPixelsWidth*/
const Profile P17333 = {"17333", &P17333_4};	/*Profile ID - 17333*/
const Value V14715_1_1 = {S2, NULL};	/*Value = True*/
const Property P14715_1 = {S50, 0, &V14715_1_1, NULL};	/*Property = IsMobile*/
const Value V14715_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14715_5 = {S52, 0, &V14715_5_1293, &P14715_1};	/*Property = ScreenPixelsHeight*/
const Value V14715_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14715_4 = {S54, 0, &V14715_4_1296, &P14715_5};	/*Property = ScreenPixelsWidth*/
const Profile P14715 = {"14715", &P14715_4};	/*Profile ID - 14715*/
const Value V18405_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_279 = {S1, 0, &V18405_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18405_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_282 = {S3, 0, &V18405_282_57155, &P18405_279};	/*Property = BlobBuilder*/
const Value V18405_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_350 = {S4, 0, &V18405_350_57446, &P18405_282};	/*Property = Canvas*/
const Value V18405_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_283 = {S5, 0, &V18405_283_57161, &P18405_350};	/*Property = CssBackground*/
const Value V18405_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_284 = {S6, 0, &V18405_284_57164, &P18405_283};	/*Property = CssBorderImage*/
const Value V18405_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_285 = {S7, 0, &V18405_285_57167, &P18405_284};	/*Property = CssCanvas*/
const Value V18405_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_286 = {S8, 0, &V18405_286_57170, &P18405_285};	/*Property = CssColor*/
const Value V18405_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_237 = {S9, 0, &V18405_237_55786, &P18405_286};	/*Property = CssColumn*/
const Value V18405_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_287 = {S10, 0, &V18405_287_57173, &P18405_237};	/*Property = CssFlexbox*/
const Value V18405_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_288 = {S11, 0, &V18405_288_57176, &P18405_287};	/*Property = CssFont*/
const Value V18405_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_289 = {S12, 0, &V18405_289_86899, &P18405_288};	/*Property = CssImages*/
const Value V18405_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_291 = {S13, 0, &V18405_291_57182, &P18405_289};	/*Property = CssMediaQueries*/
const Value V18405_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_292 = {S14, 0, &V18405_292_57185, &P18405_291};	/*Property = CssMinMax*/
const Value V18405_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_293 = {S15, 0, &V18405_293_57188, &P18405_292};	/*Property = CssOverflow*/
const Value V18405_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_294 = {S16, 0, &V18405_294_57191, &P18405_293};	/*Property = CssPosition*/
const Value V18405_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_295 = {S17, 0, &V18405_295_57194, &P18405_294};	/*Property = CssText*/
const Value V18405_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_297 = {S18, 0, &V18405_297_57197, &P18405_295};	/*Property = CssTransforms*/
const Value V18405_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_241 = {S19, 0, &V18405_241_55801, &P18405_297};	/*Property = CssTransitions*/
const Value V18405_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_298 = {S20, 0, &V18405_298_57208, &P18405_241};	/*Property = CssUI*/
const Value V18405_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_300 = {S21, 0, &V18405_300_57217, &P18405_298};	/*Property = DataSet*/
const Value V18405_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_301 = {S22, 0, &V18405_301_57220, &P18405_300};	/*Property = DataUrl*/
const Value V18405_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_302 = {S23, 0, &V18405_302_57223, &P18405_301};	/*Property = DeviceOrientation*/
const Value V18405_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_304 = {S24, 0, &V18405_304_57229, &P18405_302};	/*Property = FileReader*/
const Value V18405_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_305 = {S25, 0, &V18405_305_57232, &P18405_304};	/*Property = FileSaver*/
const Value V18405_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_306 = {S26, 0, &V18405_306_57238, &P18405_305};	/*Property = FileWriter*/
const Value V18405_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_307 = {S27, 0, &V18405_307_57241, &P18405_306};	/*Property = FormData*/
const Value V18405_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_309 = {S28, 0, &V18405_309_57247, &P18405_307};	/*Property = Fullscreen*/
const Value V18405_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_310 = {S29, 0, &V18405_310_57250, &P18405_309};	/*Property = GeoLocation*/
const Value V18405_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_312 = {S30, 0, &V18405_312_57256, &P18405_310};	/*Property = History*/
const Value V18405_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_314 = {S31, 0, &V18405_314_57262, &P18405_312};	/*Property = Html5*/
const Value V18405_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_313 = {S32, 0, &V18405_313_57259, &P18405_314};	/*Property = Html-Media-Capture*/
const Value V18405_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_315 = {S33, 0, &V18405_315_57265, &P18405_313};	/*Property = Iframe*/
const Value V18405_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_316 = {S34, 0, &V18405_316_57268, &P18405_315};	/*Property = IndexedDB*/
const Value V18405_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_317 = {S35, 0, &V18405_317_57271, &P18405_316};	/*Property = Json*/
const Value V18405_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18405_15 = {S37, 0, &V18405_15_31, &P18405_317};	/*Property = LayoutEngine*/
const Value V18405_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_344 = {S38, 0, &V18405_344_57274, &P18405_15};	/*Property = Masking*/
const Value V18405_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_320 = {S39, 0, &V18405_320_57292, &P18405_344};	/*Property = PostMessage*/
const Value V18405_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_321 = {S40, 0, &V18405_321_57295, &P18405_320};	/*Property = Progress*/
const Value V18405_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_322 = {S41, 0, &V18405_322_57298, &P18405_321};	/*Property = Prompts*/
const Value V18405_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_323 = {S42, 0, &V18405_323_57301, &P18405_322};	/*Property = Selector*/
const Value V18405_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_324 = {S43, 0, &V18405_324_57307, &P18405_323};	/*Property = Svg*/
const Value V18405_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_327 = {S44, 0, &V18405_327_57316, &P18405_324};	/*Property = TouchEvents*/
const Value V18405_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_328 = {S45, 0, &V18405_328_57319, &P18405_327};	/*Property = Track*/
const Value V18405_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_329 = {S46, 0, &V18405_329_57325, &P18405_328};	/*Property = Video*/
const Value V18405_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_330 = {S47, 0, &V18405_330_57328, &P18405_329};	/*Property = Viewport*/
const Value V18405_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_332 = {S48, 0, &V18405_332_57340, &P18405_330};	/*Property = WebWorkers*/
const Value V18405_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18405_333 = {S49, 0, &V18405_333_57343, &P18405_332};	/*Property = Xhr2*/
const Profile P18405 = {"18405", &P18405_333};	/*Profile ID - 18405*/
const Value V16137_1_1 = {S2, NULL};	/*Value = True*/
const Property P16137_1 = {S50, 0, &V16137_1_1, NULL};	/*Property = IsMobile*/
const Value V16137_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16137_5 = {S52, 0, &V16137_5_1222, &P16137_1};	/*Property = ScreenPixelsHeight*/
const Value V16137_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16137_4 = {S54, 0, &V16137_4_1224, &P16137_5};	/*Property = ScreenPixelsWidth*/
const Profile P16137 = {"16137", &P16137_4};	/*Profile ID - 16137*/
const Value V16205_1_1 = {S2, NULL};	/*Value = True*/
const Property P16205_1 = {S50, 0, &V16205_1_1, NULL};	/*Property = IsMobile*/
const Value V16205_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P16205_5 = {S52, 0, &V16205_5_1275, &P16205_1};	/*Property = ScreenPixelsHeight*/
const Value V16205_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16205_4 = {S54, 0, &V16205_4_1229, &P16205_5};	/*Property = ScreenPixelsWidth*/
const Profile P16205 = {"16205", &P16205_4};	/*Profile ID - 16205*/
const Value V14733_1_1 = {S2, NULL};	/*Value = True*/
const Property P14733_1 = {S50, 0, &V14733_1_1, NULL};	/*Property = IsMobile*/
const Value V14733_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14733_5 = {S52, 0, &V14733_5_1277, &P14733_1};	/*Property = ScreenPixelsHeight*/
const Value V14733_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14733_4 = {S54, 0, &V14733_4_1279, &P14733_5};	/*Property = ScreenPixelsWidth*/
const Profile P14733 = {"14733", &P14733_4};	/*Profile ID - 14733*/
const Value V14800_1_1 = {S2, NULL};	/*Value = True*/
const Property P14800_1 = {S50, 0, &V14800_1_1, NULL};	/*Property = IsMobile*/
const Value V14800_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P14800_5 = {S52, 0, &V14800_5_1269, &P14800_1};	/*Property = ScreenPixelsHeight*/
const Value V14800_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14800_4 = {S54, 0, &V14800_4_1229, &P14800_5};	/*Property = ScreenPixelsWidth*/
const Profile P14800 = {"14800", &P14800_4};	/*Profile ID - 14800*/
const Value V14802_1_1 = {S2, NULL};	/*Value = True*/
const Property P14802_1 = {S50, 0, &V14802_1_1, NULL};	/*Property = IsMobile*/
const Value V14802_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14802_5 = {S52, 0, &V14802_5_1227, &P14802_1};	/*Property = ScreenPixelsHeight*/
const Value V14802_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14802_4 = {S54, 0, &V14802_4_1229, &P14802_5};	/*Property = ScreenPixelsWidth*/
const Profile P14802 = {"14802", &P14802_4};	/*Profile ID - 14802*/
const Value V14803_1_1 = {S2, NULL};	/*Value = True*/
const Property P14803_1 = {S50, 0, &V14803_1_1, NULL};	/*Property = IsMobile*/
const Value V14803_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P14803_5 = {S52, 0, &V14803_5_1269, &P14803_1};	/*Property = ScreenPixelsHeight*/
const Value V14803_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14803_4 = {S54, 0, &V14803_4_1229, &P14803_5};	/*Property = ScreenPixelsWidth*/
const Profile P14803 = {"14803", &P14803_4};	/*Profile ID - 14803*/
const Value V17228_1_1 = {S2, NULL};	/*Value = True*/
const Property P17228_1 = {S50, 0, &V17228_1_1, NULL};	/*Property = IsMobile*/
const Value V17228_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P17228_5 = {S52, 0, &V17228_5_1832, &P17228_1};	/*Property = ScreenPixelsHeight*/
const Value V17228_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P17228_4 = {S54, 0, &V17228_4_2336, &P17228_5};	/*Property = ScreenPixelsWidth*/
const Profile P17228 = {"17228", &P17228_4};	/*Profile ID - 17228*/
const Value V14805_1_1 = {S2, NULL};	/*Value = True*/
const Property P14805_1 = {S50, 0, &V14805_1_1, NULL};	/*Property = IsMobile*/
const Value V14805_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P14805_5 = {S52, 0, &V14805_5_1275, &P14805_1};	/*Property = ScreenPixelsHeight*/
const Value V14805_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14805_4 = {S54, 0, &V14805_4_1229, &P14805_5};	/*Property = ScreenPixelsWidth*/
const Profile P14805 = {"14805", &P14805_4};	/*Profile ID - 14805*/
const Value V17960_1_1 = {S2, NULL};	/*Value = True*/
const Property P17960_1 = {S50, 0, &V17960_1_1, NULL};	/*Property = IsMobile*/
const Value V17960_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17960_5 = {S52, 0, &V17960_5_1227, &P17960_1};	/*Property = ScreenPixelsHeight*/
const Value V17960_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17960_4 = {S54, 0, &V17960_4_1229, &P17960_5};	/*Property = ScreenPixelsWidth*/
const Profile P17960 = {"17960", &P17960_4};	/*Profile ID - 17960*/
const Value V14742_1_1 = {S2, NULL};	/*Value = True*/
const Property P14742_1 = {S50, 0, &V14742_1_1, NULL};	/*Property = IsMobile*/
const Value V14742_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14742_5 = {S52, 0, &V14742_5_1222, &P14742_1};	/*Property = ScreenPixelsHeight*/
const Value V14742_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14742_4 = {S54, 0, &V14742_4_1229, &P14742_5};	/*Property = ScreenPixelsWidth*/
const Profile P14742 = {"14742", &P14742_4};	/*Profile ID - 14742*/
const Value V14746_1_1 = {S2, NULL};	/*Value = True*/
const Property P14746_1 = {S50, 0, &V14746_1_1, NULL};	/*Property = IsMobile*/
const Value V14746_5_37540 = {S105, NULL};	/*Value = 427*/
const Property P14746_5 = {S52, 0, &V14746_5_37540, &P14746_1};	/*Property = ScreenPixelsHeight*/
const Value V14746_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14746_4 = {S54, 0, &V14746_4_1224, &P14746_5};	/*Property = ScreenPixelsWidth*/
const Profile P14746 = {"14746", &P14746_4};	/*Profile ID - 14746*/
const Value V14745_1_1 = {S2, NULL};	/*Value = True*/
const Property P14745_1 = {S50, 0, &V14745_1_1, NULL};	/*Property = IsMobile*/
const Value V14745_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14745_5 = {S52, 0, &V14745_5_1222, &P14745_1};	/*Property = ScreenPixelsHeight*/
const Value V14745_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14745_4 = {S54, 0, &V14745_4_1224, &P14745_5};	/*Property = ScreenPixelsWidth*/
const Profile P14745 = {"14745", &P14745_4};	/*Profile ID - 14745*/
const Value V14747_1_1 = {S2, NULL};	/*Value = True*/
const Property P14747_1 = {S50, 0, &V14747_1_1, NULL};	/*Property = IsMobile*/
const Value V14747_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P14747_5 = {S52, 0, &V14747_5_1403, &P14747_1};	/*Property = ScreenPixelsHeight*/
const Value V14747_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14747_4 = {S54, 0, &V14747_4_1229, &P14747_5};	/*Property = ScreenPixelsWidth*/
const Profile P14747 = {"14747", &P14747_4};	/*Profile ID - 14747*/
const Value V14808_1_1 = {S2, NULL};	/*Value = True*/
const Property P14808_1 = {S50, 0, &V14808_1_1, NULL};	/*Property = IsMobile*/
const Value V14808_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14808_5 = {S52, 0, &V14808_5_1277, &P14808_1};	/*Property = ScreenPixelsHeight*/
const Value V14808_4_2299 = {S82, NULL};	/*Value = 960*/
const Property P14808_4 = {S54, 0, &V14808_4_2299, &P14808_5};	/*Property = ScreenPixelsWidth*/
const Profile P14808 = {"14808", &P14808_4};	/*Profile ID - 14808*/
const Value V14752_1_1 = {S2, NULL};	/*Value = True*/
const Property P14752_1 = {S50, 0, &V14752_1_1, NULL};	/*Property = IsMobile*/
const Value V14752_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14752_5 = {S52, 0, &V14752_5_1222, &P14752_1};	/*Property = ScreenPixelsHeight*/
const Value V14752_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14752_4 = {S54, 0, &V14752_4_1224, &P14752_5};	/*Property = ScreenPixelsWidth*/
const Profile P14752 = {"14752", &P14752_4};	/*Profile ID - 14752*/
const Value V17755_1_1 = {S2, NULL};	/*Value = True*/
const Property P17755_1 = {S50, 0, &V17755_1_1, NULL};	/*Property = IsMobile*/
const Value V17755_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P17755_5 = {S52, 0, &V17755_5_1403, &P17755_1};	/*Property = ScreenPixelsHeight*/
const Value V17755_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17755_4 = {S54, 0, &V17755_4_1229, &P17755_5};	/*Property = ScreenPixelsWidth*/
const Profile P17755 = {"17755", &P17755_4};	/*Profile ID - 17755*/
const Value V24645_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_279 = {S1, 0, &V24645_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24645_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_282 = {S3, 0, &V24645_282_57155, &P24645_279};	/*Property = BlobBuilder*/
const Value V24645_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_350 = {S4, 0, &V24645_350_57446, &P24645_282};	/*Property = Canvas*/
const Value V24645_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_283 = {S5, 0, &V24645_283_57161, &P24645_350};	/*Property = CssBackground*/
const Value V24645_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_284 = {S6, 0, &V24645_284_57164, &P24645_283};	/*Property = CssBorderImage*/
const Value V24645_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_285 = {S7, 0, &V24645_285_57167, &P24645_284};	/*Property = CssCanvas*/
const Value V24645_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_286 = {S8, 0, &V24645_286_57170, &P24645_285};	/*Property = CssColor*/
const Value V24645_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_237 = {S9, 0, &V24645_237_55786, &P24645_286};	/*Property = CssColumn*/
const Value V24645_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_287 = {S10, 0, &V24645_287_57173, &P24645_237};	/*Property = CssFlexbox*/
const Value V24645_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_288 = {S11, 0, &V24645_288_57176, &P24645_287};	/*Property = CssFont*/
const Value V24645_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_289 = {S12, 0, &V24645_289_86899, &P24645_288};	/*Property = CssImages*/
const Value V24645_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_291 = {S13, 0, &V24645_291_57182, &P24645_289};	/*Property = CssMediaQueries*/
const Value V24645_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_292 = {S14, 0, &V24645_292_57185, &P24645_291};	/*Property = CssMinMax*/
const Value V24645_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_293 = {S15, 0, &V24645_293_57188, &P24645_292};	/*Property = CssOverflow*/
const Value V24645_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_294 = {S16, 0, &V24645_294_57191, &P24645_293};	/*Property = CssPosition*/
const Value V24645_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_295 = {S17, 0, &V24645_295_57194, &P24645_294};	/*Property = CssText*/
const Value V24645_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_297 = {S18, 0, &V24645_297_57197, &P24645_295};	/*Property = CssTransforms*/
const Value V24645_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_241 = {S19, 0, &V24645_241_55801, &P24645_297};	/*Property = CssTransitions*/
const Value V24645_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_298 = {S20, 0, &V24645_298_57208, &P24645_241};	/*Property = CssUI*/
const Value V24645_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_300 = {S21, 0, &V24645_300_57217, &P24645_298};	/*Property = DataSet*/
const Value V24645_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_301 = {S22, 0, &V24645_301_57220, &P24645_300};	/*Property = DataUrl*/
const Value V24645_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_302 = {S23, 0, &V24645_302_57223, &P24645_301};	/*Property = DeviceOrientation*/
const Value V24645_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_304 = {S24, 0, &V24645_304_57229, &P24645_302};	/*Property = FileReader*/
const Value V24645_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_305 = {S25, 0, &V24645_305_57232, &P24645_304};	/*Property = FileSaver*/
const Value V24645_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_306 = {S26, 0, &V24645_306_57238, &P24645_305};	/*Property = FileWriter*/
const Value V24645_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_307 = {S27, 0, &V24645_307_57241, &P24645_306};	/*Property = FormData*/
const Value V24645_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_309 = {S28, 0, &V24645_309_57247, &P24645_307};	/*Property = Fullscreen*/
const Value V24645_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_310 = {S29, 0, &V24645_310_57250, &P24645_309};	/*Property = GeoLocation*/
const Value V24645_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_312 = {S30, 0, &V24645_312_57256, &P24645_310};	/*Property = History*/
const Value V24645_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_314 = {S31, 0, &V24645_314_57262, &P24645_312};	/*Property = Html5*/
const Value V24645_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_313 = {S32, 0, &V24645_313_57259, &P24645_314};	/*Property = Html-Media-Capture*/
const Value V24645_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_315 = {S33, 0, &V24645_315_57265, &P24645_313};	/*Property = Iframe*/
const Value V24645_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_316 = {S34, 0, &V24645_316_57268, &P24645_315};	/*Property = IndexedDB*/
const Value V24645_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_317 = {S35, 0, &V24645_317_57271, &P24645_316};	/*Property = Json*/
const Value V24645_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24645_15 = {S37, 0, &V24645_15_90837, &P24645_317};	/*Property = LayoutEngine*/
const Value V24645_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_344 = {S38, 0, &V24645_344_57274, &P24645_15};	/*Property = Masking*/
const Value V24645_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_320 = {S39, 0, &V24645_320_57292, &P24645_344};	/*Property = PostMessage*/
const Value V24645_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_321 = {S40, 0, &V24645_321_57295, &P24645_320};	/*Property = Progress*/
const Value V24645_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_322 = {S41, 0, &V24645_322_57298, &P24645_321};	/*Property = Prompts*/
const Value V24645_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_323 = {S42, 0, &V24645_323_57301, &P24645_322};	/*Property = Selector*/
const Value V24645_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_324 = {S43, 0, &V24645_324_57307, &P24645_323};	/*Property = Svg*/
const Value V24645_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_327 = {S44, 0, &V24645_327_57316, &P24645_324};	/*Property = TouchEvents*/
const Value V24645_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_328 = {S45, 0, &V24645_328_57319, &P24645_327};	/*Property = Track*/
const Value V24645_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_329 = {S46, 0, &V24645_329_57325, &P24645_328};	/*Property = Video*/
const Value V24645_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_330 = {S47, 0, &V24645_330_57328, &P24645_329};	/*Property = Viewport*/
const Value V24645_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_332 = {S48, 0, &V24645_332_57340, &P24645_330};	/*Property = WebWorkers*/
const Value V24645_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24645_333 = {S49, 0, &V24645_333_57343, &P24645_332};	/*Property = Xhr2*/
const Profile P24645 = {"24645", &P24645_333};	/*Profile ID - 24645*/
const Value V14778_1_1 = {S2, NULL};	/*Value = True*/
const Property P14778_1 = {S50, 0, &V14778_1_1, NULL};	/*Property = IsMobile*/
const Value V14778_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14778_5 = {S52, 0, &V14778_5_1222, &P14778_1};	/*Property = ScreenPixelsHeight*/
const Value V14778_4_1368 = {S74, NULL};	/*Value = 208*/
const Property P14778_4 = {S54, 0, &V14778_4_1368, &P14778_5};	/*Property = ScreenPixelsWidth*/
const Profile P14778 = {"14778", &P14778_4};	/*Profile ID - 14778*/
const Value V14761_1_1 = {S2, NULL};	/*Value = True*/
const Property P14761_1 = {S50, 0, &V14761_1_1, NULL};	/*Property = IsMobile*/
const Value V14761_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P14761_5 = {S52, 0, &V14761_5_1403, &P14761_1};	/*Property = ScreenPixelsHeight*/
const Value V14761_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14761_4 = {S54, 0, &V14761_4_1229, &P14761_5};	/*Property = ScreenPixelsWidth*/
const Profile P14761 = {"14761", &P14761_4};	/*Profile ID - 14761*/
const Value V12505_1_1 = {S2, NULL};	/*Value = True*/
const Property P12505_1 = {S50, 0, &V12505_1_1, NULL};	/*Property = IsMobile*/
const Value V12505_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12505_5 = {S52, 0, &V12505_5_1304, &P12505_1};	/*Property = ScreenPixelsHeight*/
const Value V12505_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12505_4 = {S54, 0, &V12505_4_1224, &P12505_5};	/*Property = ScreenPixelsWidth*/
const Profile P12505 = {"12505", &P12505_4};	/*Profile ID - 12505*/
const Value V14766_1_1 = {S2, NULL};	/*Value = True*/
const Property P14766_1 = {S50, 0, &V14766_1_1, NULL};	/*Property = IsMobile*/
const Value V14766_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P14766_5 = {S52, 0, &V14766_5_1403, &P14766_1};	/*Property = ScreenPixelsHeight*/
const Value V14766_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14766_4 = {S54, 0, &V14766_4_1229, &P14766_5};	/*Property = ScreenPixelsWidth*/
const Profile P14766 = {"14766", &P14766_4};	/*Profile ID - 14766*/
const Value V17716_1_1 = {S2, NULL};	/*Value = True*/
const Property P17716_1 = {S50, 0, &V17716_1_1, NULL};	/*Property = IsMobile*/
const Value V17716_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17716_5 = {S52, 0, &V17716_5_1277, &P17716_1};	/*Property = ScreenPixelsHeight*/
const Value V17716_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17716_4 = {S54, 0, &V17716_4_1279, &P17716_5};	/*Property = ScreenPixelsWidth*/
const Profile P17716 = {"17716", &P17716_4};	/*Profile ID - 17716*/
const Value V14770_1_1 = {S2, NULL};	/*Value = True*/
const Property P14770_1 = {S50, 0, &V14770_1_1, NULL};	/*Property = IsMobile*/
const Value V14770_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14770_5 = {S52, 0, &V14770_5_1304, &P14770_1};	/*Property = ScreenPixelsHeight*/
const Value V14770_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14770_4 = {S54, 0, &V14770_4_1224, &P14770_5};	/*Property = ScreenPixelsWidth*/
const Profile P14770 = {"14770", &P14770_4};	/*Profile ID - 14770*/
const Value V14781_1_1 = {S2, NULL};	/*Value = True*/
const Property P14781_1 = {S50, 0, &V14781_1_1, NULL};	/*Property = IsMobile*/
const Value V14781_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14781_5 = {S52, 0, &V14781_5_1227, &P14781_1};	/*Property = ScreenPixelsHeight*/
const Value V14781_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14781_4 = {S54, 0, &V14781_4_1229, &P14781_5};	/*Property = ScreenPixelsWidth*/
const Profile P14781 = {"14781", &P14781_4};	/*Profile ID - 14781*/
const Value V14783_1_1 = {S2, NULL};	/*Value = True*/
const Property P14783_1 = {S50, 0, &V14783_1_1, NULL};	/*Property = IsMobile*/
const Value V14783_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14783_5 = {S52, 0, &V14783_5_1304, &P14783_1};	/*Property = ScreenPixelsHeight*/
const Value V14783_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14783_4 = {S54, 0, &V14783_4_1224, &P14783_5};	/*Property = ScreenPixelsWidth*/
const Profile P14783 = {"14783", &P14783_4};	/*Profile ID - 14783*/
const Value V14787_1_1 = {S2, NULL};	/*Value = True*/
const Property P14787_1 = {S50, 0, &V14787_1_1, NULL};	/*Property = IsMobile*/
const Value V14787_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P14787_5 = {S52, 0, &V14787_5_1269, &P14787_1};	/*Property = ScreenPixelsHeight*/
const Value V14787_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14787_4 = {S54, 0, &V14787_4_1229, &P14787_5};	/*Property = ScreenPixelsWidth*/
const Profile P14787 = {"14787", &P14787_4};	/*Profile ID - 14787*/
const Value V14852_1_1 = {S2, NULL};	/*Value = True*/
const Property P14852_1 = {S50, 0, &V14852_1_1, NULL};	/*Property = IsMobile*/
const Value V14852_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14852_5 = {S52, 0, &V14852_5_1304, &P14852_1};	/*Property = ScreenPixelsHeight*/
const Value V14852_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14852_4 = {S54, 0, &V14852_4_1279, &P14852_5};	/*Property = ScreenPixelsWidth*/
const Profile P14852 = {"14852", &P14852_4};	/*Profile ID - 14852*/
const Value V14793_1_1 = {S2, NULL};	/*Value = True*/
const Property P14793_1 = {S50, 0, &V14793_1_1, NULL};	/*Property = IsMobile*/
const Value V14793_5_37540 = {S105, NULL};	/*Value = 427*/
const Property P14793_5 = {S52, 0, &V14793_5_37540, &P14793_1};	/*Property = ScreenPixelsHeight*/
const Value V14793_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14793_4 = {S54, 0, &V14793_4_1224, &P14793_5};	/*Property = ScreenPixelsWidth*/
const Profile P14793 = {"14793", &P14793_4};	/*Profile ID - 14793*/
const Value V14797_1_1 = {S2, NULL};	/*Value = True*/
const Property P14797_1 = {S50, 0, &V14797_1_1, NULL};	/*Property = IsMobile*/
const Value V14797_5_37540 = {S105, NULL};	/*Value = 427*/
const Property P14797_5 = {S52, 0, &V14797_5_37540, &P14797_1};	/*Property = ScreenPixelsHeight*/
const Value V14797_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14797_4 = {S54, 0, &V14797_4_1224, &P14797_5};	/*Property = ScreenPixelsWidth*/
const Profile P14797 = {"14797", &P14797_4};	/*Profile ID - 14797*/
const Value V14796_1_1 = {S2, NULL};	/*Value = True*/
const Property P14796_1 = {S50, 0, &V14796_1_1, NULL};	/*Property = IsMobile*/
const Value V14796_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P14796_5 = {S52, 0, &V14796_5_1832, &P14796_1};	/*Property = ScreenPixelsHeight*/
const Value V14796_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P14796_4 = {S54, 0, &V14796_4_5645, &P14796_5};	/*Property = ScreenPixelsWidth*/
const Profile P14796 = {"14796", &P14796_4};	/*Profile ID - 14796*/
const Value V12973_1_1 = {S2, NULL};	/*Value = True*/
const Property P12973_1 = {S50, 0, &V12973_1_1, NULL};	/*Property = IsMobile*/
const Value V12973_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P12973_5 = {S52, 0, &V12973_5_1458, &P12973_1};	/*Property = ScreenPixelsHeight*/
const Value V12973_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P12973_4 = {S54, 0, &V12973_4_1460, &P12973_5};	/*Property = ScreenPixelsWidth*/
const Profile P12973 = {"12973", &P12973_4};	/*Profile ID - 12973*/
const Value V14798_1_1 = {S2, NULL};	/*Value = True*/
const Property P14798_1 = {S50, 0, &V14798_1_1, NULL};	/*Property = IsMobile*/
const Value V14798_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P14798_5 = {S52, 0, &V14798_5_1269, &P14798_1};	/*Property = ScreenPixelsHeight*/
const Value V14798_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14798_4 = {S54, 0, &V14798_4_1229, &P14798_5};	/*Property = ScreenPixelsWidth*/
const Profile P14798 = {"14798", &P14798_4};	/*Profile ID - 14798*/
const Value V14812_1_1 = {S2, NULL};	/*Value = True*/
const Property P14812_1 = {S50, 0, &V14812_1_1, NULL};	/*Property = IsMobile*/
const Value V14812_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14812_5 = {S52, 0, &V14812_5_1277, &P14812_1};	/*Property = ScreenPixelsHeight*/
const Value V14812_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14812_4 = {S54, 0, &V14812_4_1279, &P14812_5};	/*Property = ScreenPixelsWidth*/
const Profile P14812 = {"14812", &P14812_4};	/*Profile ID - 14812*/
const Value V14813_1_1 = {S2, NULL};	/*Value = True*/
const Property P14813_1 = {S50, 0, &V14813_1_1, NULL};	/*Property = IsMobile*/
const Value V14813_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14813_5 = {S52, 0, &V14813_5_1277, &P14813_1};	/*Property = ScreenPixelsHeight*/
const Value V14813_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14813_4 = {S54, 0, &V14813_4_1279, &P14813_5};	/*Property = ScreenPixelsWidth*/
const Profile P14813 = {"14813", &P14813_4};	/*Profile ID - 14813*/
const Value V14815_1_1 = {S2, NULL};	/*Value = True*/
const Property P14815_1 = {S50, 0, &V14815_1_1, NULL};	/*Property = IsMobile*/
const Value V14815_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14815_5 = {S52, 0, &V14815_5_1304, &P14815_1};	/*Property = ScreenPixelsHeight*/
const Value V14815_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14815_4 = {S54, 0, &V14815_4_1224, &P14815_5};	/*Property = ScreenPixelsWidth*/
const Profile P14815 = {"14815", &P14815_4};	/*Profile ID - 14815*/
const Value V14816_1_1 = {S2, NULL};	/*Value = True*/
const Property P14816_1 = {S50, 0, &V14816_1_1, NULL};	/*Property = IsMobile*/
const Value V14816_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14816_5 = {S52, 0, &V14816_5_1277, &P14816_1};	/*Property = ScreenPixelsHeight*/
const Value V14816_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14816_4 = {S54, 0, &V14816_4_1279, &P14816_5};	/*Property = ScreenPixelsWidth*/
const Profile P14816 = {"14816", &P14816_4};	/*Profile ID - 14816*/
const Value V14979_1_1 = {S2, NULL};	/*Value = True*/
const Property P14979_1 = {S50, 0, &V14979_1_1, NULL};	/*Property = IsMobile*/
const Value V14979_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14979_5 = {S52, 0, &V14979_5_1227, &P14979_1};	/*Property = ScreenPixelsHeight*/
const Value V14979_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14979_4 = {S54, 0, &V14979_4_1229, &P14979_5};	/*Property = ScreenPixelsWidth*/
const Profile P14979 = {"14979", &P14979_4};	/*Profile ID - 14979*/
const Value V14809_1_1 = {S2, NULL};	/*Value = True*/
const Property P14809_1 = {S50, 0, &V14809_1_1, NULL};	/*Property = IsMobile*/
const Value V14809_5_37453 = {S91, NULL};	/*Value = 65*/
const Property P14809_5 = {S52, 0, &V14809_5_37453, &P14809_1};	/*Property = ScreenPixelsHeight*/
const Value V14809_4_31956 = {S92, NULL};	/*Value = 96*/
const Property P14809_4 = {S54, 0, &V14809_4_31956, &P14809_5};	/*Property = ScreenPixelsWidth*/
const Profile P14809 = {"14809", &P14809_4};	/*Profile ID - 14809*/
const Value V14821_1_1 = {S2, NULL};	/*Value = True*/
const Property P14821_1 = {S50, 0, &V14821_1_1, NULL};	/*Property = IsMobile*/
const Value V14821_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14821_5 = {S52, 0, &V14821_5_1227, &P14821_1};	/*Property = ScreenPixelsHeight*/
const Value V14821_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14821_4 = {S54, 0, &V14821_4_1229, &P14821_5};	/*Property = ScreenPixelsWidth*/
const Profile P14821 = {"14821", &P14821_4};	/*Profile ID - 14821*/
const Value V14854_1_1 = {S2, NULL};	/*Value = True*/
const Property P14854_1 = {S50, 0, &V14854_1_1, NULL};	/*Property = IsMobile*/
const Value V14854_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14854_5 = {S52, 0, &V14854_5_1227, &P14854_1};	/*Property = ScreenPixelsHeight*/
const Value V14854_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14854_4 = {S54, 0, &V14854_4_1229, &P14854_5};	/*Property = ScreenPixelsWidth*/
const Profile P14854 = {"14854", &P14854_4};	/*Profile ID - 14854*/
const Value V14823_1_1 = {S2, NULL};	/*Value = True*/
const Property P14823_1 = {S50, 0, &V14823_1_1, NULL};	/*Property = IsMobile*/
const Value V14823_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14823_5 = {S52, 0, &V14823_5_1277, &P14823_1};	/*Property = ScreenPixelsHeight*/
const Value V14823_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14823_4 = {S54, 0, &V14823_4_1279, &P14823_5};	/*Property = ScreenPixelsWidth*/
const Profile P14823 = {"14823", &P14823_4};	/*Profile ID - 14823*/
const Value V16638_1_1 = {S2, NULL};	/*Value = True*/
const Property P16638_1 = {S50, 0, &V16638_1_1, NULL};	/*Property = IsMobile*/
const Value V16638_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16638_5 = {S52, 0, &V16638_5_1233, &P16638_1};	/*Property = ScreenPixelsHeight*/
const Value V16638_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16638_4 = {S54, 0, &V16638_4_1235, &P16638_5};	/*Property = ScreenPixelsWidth*/
const Profile P16638 = {"16638", &P16638_4};	/*Profile ID - 16638*/
const Value V14900_1_1 = {S2, NULL};	/*Value = True*/
const Property P14900_1 = {S50, 0, &V14900_1_1, NULL};	/*Property = IsMobile*/
const Value V14900_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14900_5 = {S52, 0, &V14900_5_1277, &P14900_1};	/*Property = ScreenPixelsHeight*/
const Value V14900_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14900_4 = {S54, 0, &V14900_4_1279, &P14900_5};	/*Property = ScreenPixelsWidth*/
const Profile P14900 = {"14900", &P14900_4};	/*Profile ID - 14900*/
const Value V24660_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_279 = {S1, 0, &V24660_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24660_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_282 = {S3, 0, &V24660_282_57155, &P24660_279};	/*Property = BlobBuilder*/
const Value V24660_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_350 = {S4, 0, &V24660_350_57446, &P24660_282};	/*Property = Canvas*/
const Value V24660_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_283 = {S5, 0, &V24660_283_57161, &P24660_350};	/*Property = CssBackground*/
const Value V24660_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_284 = {S6, 0, &V24660_284_57164, &P24660_283};	/*Property = CssBorderImage*/
const Value V24660_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_285 = {S7, 0, &V24660_285_57167, &P24660_284};	/*Property = CssCanvas*/
const Value V24660_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_286 = {S8, 0, &V24660_286_57170, &P24660_285};	/*Property = CssColor*/
const Value V24660_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_237 = {S9, 0, &V24660_237_55786, &P24660_286};	/*Property = CssColumn*/
const Value V24660_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_287 = {S10, 0, &V24660_287_57173, &P24660_237};	/*Property = CssFlexbox*/
const Value V24660_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_288 = {S11, 0, &V24660_288_57176, &P24660_287};	/*Property = CssFont*/
const Value V24660_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_289 = {S12, 0, &V24660_289_86899, &P24660_288};	/*Property = CssImages*/
const Value V24660_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_291 = {S13, 0, &V24660_291_57182, &P24660_289};	/*Property = CssMediaQueries*/
const Value V24660_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_292 = {S14, 0, &V24660_292_57185, &P24660_291};	/*Property = CssMinMax*/
const Value V24660_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_293 = {S15, 0, &V24660_293_57188, &P24660_292};	/*Property = CssOverflow*/
const Value V24660_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_294 = {S16, 0, &V24660_294_57191, &P24660_293};	/*Property = CssPosition*/
const Value V24660_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_295 = {S17, 0, &V24660_295_57194, &P24660_294};	/*Property = CssText*/
const Value V24660_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_297 = {S18, 0, &V24660_297_57197, &P24660_295};	/*Property = CssTransforms*/
const Value V24660_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_241 = {S19, 0, &V24660_241_55801, &P24660_297};	/*Property = CssTransitions*/
const Value V24660_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_298 = {S20, 0, &V24660_298_57208, &P24660_241};	/*Property = CssUI*/
const Value V24660_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_300 = {S21, 0, &V24660_300_57217, &P24660_298};	/*Property = DataSet*/
const Value V24660_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_301 = {S22, 0, &V24660_301_57220, &P24660_300};	/*Property = DataUrl*/
const Value V24660_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_302 = {S23, 0, &V24660_302_57223, &P24660_301};	/*Property = DeviceOrientation*/
const Value V24660_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_304 = {S24, 0, &V24660_304_57229, &P24660_302};	/*Property = FileReader*/
const Value V24660_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_305 = {S25, 0, &V24660_305_57232, &P24660_304};	/*Property = FileSaver*/
const Value V24660_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_306 = {S26, 0, &V24660_306_57238, &P24660_305};	/*Property = FileWriter*/
const Value V24660_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_307 = {S27, 0, &V24660_307_57241, &P24660_306};	/*Property = FormData*/
const Value V24660_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_309 = {S28, 0, &V24660_309_57247, &P24660_307};	/*Property = Fullscreen*/
const Value V24660_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_310 = {S29, 0, &V24660_310_57250, &P24660_309};	/*Property = GeoLocation*/
const Value V24660_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_312 = {S30, 0, &V24660_312_57256, &P24660_310};	/*Property = History*/
const Value V24660_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_314 = {S31, 0, &V24660_314_57262, &P24660_312};	/*Property = Html5*/
const Value V24660_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_313 = {S32, 0, &V24660_313_57259, &P24660_314};	/*Property = Html-Media-Capture*/
const Value V24660_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_315 = {S33, 0, &V24660_315_57265, &P24660_313};	/*Property = Iframe*/
const Value V24660_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_316 = {S34, 0, &V24660_316_57268, &P24660_315};	/*Property = IndexedDB*/
const Value V24660_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_317 = {S35, 0, &V24660_317_57271, &P24660_316};	/*Property = Json*/
const Value V24660_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24660_15 = {S37, 0, &V24660_15_90837, &P24660_317};	/*Property = LayoutEngine*/
const Value V24660_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_344 = {S38, 0, &V24660_344_57274, &P24660_15};	/*Property = Masking*/
const Value V24660_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_320 = {S39, 0, &V24660_320_57292, &P24660_344};	/*Property = PostMessage*/
const Value V24660_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_321 = {S40, 0, &V24660_321_57295, &P24660_320};	/*Property = Progress*/
const Value V24660_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_322 = {S41, 0, &V24660_322_57298, &P24660_321};	/*Property = Prompts*/
const Value V24660_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_323 = {S42, 0, &V24660_323_57301, &P24660_322};	/*Property = Selector*/
const Value V24660_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_324 = {S43, 0, &V24660_324_57307, &P24660_323};	/*Property = Svg*/
const Value V24660_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_327 = {S44, 0, &V24660_327_57316, &P24660_324};	/*Property = TouchEvents*/
const Value V24660_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_328 = {S45, 0, &V24660_328_57319, &P24660_327};	/*Property = Track*/
const Value V24660_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_329 = {S46, 0, &V24660_329_57325, &P24660_328};	/*Property = Video*/
const Value V24660_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_330 = {S47, 0, &V24660_330_57328, &P24660_329};	/*Property = Viewport*/
const Value V24660_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_332 = {S48, 0, &V24660_332_57340, &P24660_330};	/*Property = WebWorkers*/
const Value V24660_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24660_333 = {S49, 0, &V24660_333_57343, &P24660_332};	/*Property = Xhr2*/
const Profile P24660 = {"24660", &P24660_333};	/*Profile ID - 24660*/
const Value V14835_1_1 = {S2, NULL};	/*Value = True*/
const Property P14835_1 = {S50, 0, &V14835_1_1, NULL};	/*Property = IsMobile*/
const Value V14835_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14835_5 = {S52, 0, &V14835_5_1222, &P14835_1};	/*Property = ScreenPixelsHeight*/
const Value V14835_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14835_4 = {S54, 0, &V14835_4_1224, &P14835_5};	/*Property = ScreenPixelsWidth*/
const Profile P14835 = {"14835", &P14835_4};	/*Profile ID - 14835*/
const Value V21988_1_1 = {S2, NULL};	/*Value = True*/
const Property P21988_1 = {S50, 0, &V21988_1_1, NULL};	/*Property = IsMobile*/
const Value V21988_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P21988_5 = {S52, 0, &V21988_5_1832, &P21988_1};	/*Property = ScreenPixelsHeight*/
const Value V21988_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21988_4 = {S54, 0, &V21988_4_2336, &P21988_5};	/*Property = ScreenPixelsWidth*/
const Profile P21988 = {"21988", &P21988_4};	/*Profile ID - 21988*/
const Value V15261_1_1 = {S2, NULL};	/*Value = True*/
const Property P15261_1 = {S50, 0, &V15261_1_1, NULL};	/*Property = IsMobile*/
const Value V15261_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15261_5 = {S52, 0, &V15261_5_1222, &P15261_1};	/*Property = ScreenPixelsHeight*/
const Value V15261_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15261_4 = {S54, 0, &V15261_4_1224, &P15261_5};	/*Property = ScreenPixelsWidth*/
const Profile P15261 = {"15261", &P15261_4};	/*Profile ID - 15261*/
const Value V14856_1_1 = {S2, NULL};	/*Value = True*/
const Property P14856_1 = {S50, 0, &V14856_1_1, NULL};	/*Property = IsMobile*/
const Value V14856_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14856_5 = {S52, 0, &V14856_5_1277, &P14856_1};	/*Property = ScreenPixelsHeight*/
const Value V14856_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14856_4 = {S54, 0, &V14856_4_1279, &P14856_5};	/*Property = ScreenPixelsWidth*/
const Profile P14856 = {"14856", &P14856_4};	/*Profile ID - 14856*/
const Value V14849_1_1 = {S2, NULL};	/*Value = True*/
const Property P14849_1 = {S50, 0, &V14849_1_1, NULL};	/*Property = IsMobile*/
const Value V14849_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P14849_5 = {S52, 0, &V14849_5_1458, &P14849_1};	/*Property = ScreenPixelsHeight*/
const Value V14849_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P14849_4 = {S54, 0, &V14849_4_1460, &P14849_5};	/*Property = ScreenPixelsWidth*/
const Profile P14849 = {"14849", &P14849_4};	/*Profile ID - 14849*/
const Value V14863_1_1 = {S2, NULL};	/*Value = True*/
const Property P14863_1 = {S50, 0, &V14863_1_1, NULL};	/*Property = IsMobile*/
const Value V14863_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14863_5 = {S52, 0, &V14863_5_1222, &P14863_1};	/*Property = ScreenPixelsHeight*/
const Value V14863_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14863_4 = {S54, 0, &V14863_4_1224, &P14863_5};	/*Property = ScreenPixelsWidth*/
const Profile P14863 = {"14863", &P14863_4};	/*Profile ID - 14863*/
const Value V14886_1_1 = {S2, NULL};	/*Value = True*/
const Property P14886_1 = {S50, 0, &V14886_1_1, NULL};	/*Property = IsMobile*/
const Value V14886_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14886_5 = {S52, 0, &V14886_5_1227, &P14886_1};	/*Property = ScreenPixelsHeight*/
const Value V14886_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14886_4 = {S54, 0, &V14886_4_1229, &P14886_5};	/*Property = ScreenPixelsWidth*/
const Profile P14886 = {"14886", &P14886_4};	/*Profile ID - 14886*/
const Value V18026_1_1 = {S2, NULL};	/*Value = True*/
const Property P18026_1 = {S50, 0, &V18026_1_1, NULL};	/*Property = IsMobile*/
const Value V18026_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18026_5 = {S52, 0, &V18026_5_1227, &P18026_1};	/*Property = ScreenPixelsHeight*/
const Value V18026_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18026_4 = {S54, 0, &V18026_4_1229, &P18026_5};	/*Property = ScreenPixelsWidth*/
const Profile P18026 = {"18026", &P18026_4};	/*Profile ID - 18026*/
const Value V14887_1_1 = {S2, NULL};	/*Value = True*/
const Property P14887_1 = {S50, 0, &V14887_1_1, NULL};	/*Property = IsMobile*/
const Value V14887_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14887_5 = {S52, 0, &V14887_5_1227, &P14887_1};	/*Property = ScreenPixelsHeight*/
const Value V14887_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14887_4 = {S54, 0, &V14887_4_1229, &P14887_5};	/*Property = ScreenPixelsWidth*/
const Profile P14887 = {"14887", &P14887_4};	/*Profile ID - 14887*/
const Value V18073_1_1 = {S2, NULL};	/*Value = True*/
const Property P18073_1 = {S50, 0, &V18073_1_1, NULL};	/*Property = IsMobile*/
const Value V18073_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18073_5 = {S52, 0, &V18073_5_1227, &P18073_1};	/*Property = ScreenPixelsHeight*/
const Value V18073_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18073_4 = {S54, 0, &V18073_4_1229, &P18073_5};	/*Property = ScreenPixelsWidth*/
const Profile P18073 = {"18073", &P18073_4};	/*Profile ID - 18073*/
const Value V21017_1_1 = {S2, NULL};	/*Value = True*/
const Property P21017_1 = {S50, 0, &V21017_1_1, NULL};	/*Property = IsMobile*/
const Value V21017_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21017_5 = {S52, 0, &V21017_5_9981, &P21017_1};	/*Property = ScreenPixelsHeight*/
const Value V21017_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21017_4 = {S54, 0, &V21017_4_39098, &P21017_5};	/*Property = ScreenPixelsWidth*/
const Profile P21017 = {"21017", &P21017_4};	/*Profile ID - 21017*/
const Value V14867_1_1 = {S2, NULL};	/*Value = True*/
const Property P14867_1 = {S50, 0, &V14867_1_1, NULL};	/*Property = IsMobile*/
const Value V14867_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14867_5 = {S52, 0, &V14867_5_1304, &P14867_1};	/*Property = ScreenPixelsHeight*/
const Value V14867_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14867_4 = {S54, 0, &V14867_4_1224, &P14867_5};	/*Property = ScreenPixelsWidth*/
const Profile P14867 = {"14867", &P14867_4};	/*Profile ID - 14867*/
const Value V22699_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_279 = {S1, 0, &V22699_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22699_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_282 = {S3, 0, &V22699_282_57155, &P22699_279};	/*Property = BlobBuilder*/
const Value V22699_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_350 = {S4, 0, &V22699_350_57446, &P22699_282};	/*Property = Canvas*/
const Value V22699_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_283 = {S5, 0, &V22699_283_57161, &P22699_350};	/*Property = CssBackground*/
const Value V22699_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_284 = {S6, 0, &V22699_284_57164, &P22699_283};	/*Property = CssBorderImage*/
const Value V22699_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_285 = {S7, 0, &V22699_285_57167, &P22699_284};	/*Property = CssCanvas*/
const Value V22699_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_286 = {S8, 0, &V22699_286_57170, &P22699_285};	/*Property = CssColor*/
const Value V22699_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_237 = {S9, 0, &V22699_237_55786, &P22699_286};	/*Property = CssColumn*/
const Value V22699_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_287 = {S10, 0, &V22699_287_57173, &P22699_237};	/*Property = CssFlexbox*/
const Value V22699_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_288 = {S11, 0, &V22699_288_57176, &P22699_287};	/*Property = CssFont*/
const Value V22699_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_289 = {S12, 0, &V22699_289_86899, &P22699_288};	/*Property = CssImages*/
const Value V22699_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_291 = {S13, 0, &V22699_291_57182, &P22699_289};	/*Property = CssMediaQueries*/
const Value V22699_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_292 = {S14, 0, &V22699_292_57185, &P22699_291};	/*Property = CssMinMax*/
const Value V22699_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_293 = {S15, 0, &V22699_293_57188, &P22699_292};	/*Property = CssOverflow*/
const Value V22699_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_294 = {S16, 0, &V22699_294_57191, &P22699_293};	/*Property = CssPosition*/
const Value V22699_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_295 = {S17, 0, &V22699_295_57194, &P22699_294};	/*Property = CssText*/
const Value V22699_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_297 = {S18, 0, &V22699_297_57197, &P22699_295};	/*Property = CssTransforms*/
const Value V22699_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_241 = {S19, 0, &V22699_241_55801, &P22699_297};	/*Property = CssTransitions*/
const Value V22699_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_298 = {S20, 0, &V22699_298_57208, &P22699_241};	/*Property = CssUI*/
const Value V22699_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_300 = {S21, 0, &V22699_300_57217, &P22699_298};	/*Property = DataSet*/
const Value V22699_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_301 = {S22, 0, &V22699_301_57220, &P22699_300};	/*Property = DataUrl*/
const Value V22699_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_302 = {S23, 0, &V22699_302_57223, &P22699_301};	/*Property = DeviceOrientation*/
const Value V22699_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_304 = {S24, 0, &V22699_304_57229, &P22699_302};	/*Property = FileReader*/
const Value V22699_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_305 = {S25, 0, &V22699_305_57232, &P22699_304};	/*Property = FileSaver*/
const Value V22699_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_306 = {S26, 0, &V22699_306_57238, &P22699_305};	/*Property = FileWriter*/
const Value V22699_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_307 = {S27, 0, &V22699_307_57241, &P22699_306};	/*Property = FormData*/
const Value V22699_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_309 = {S28, 0, &V22699_309_57247, &P22699_307};	/*Property = Fullscreen*/
const Value V22699_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_310 = {S29, 0, &V22699_310_57250, &P22699_309};	/*Property = GeoLocation*/
const Value V22699_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_312 = {S30, 0, &V22699_312_57256, &P22699_310};	/*Property = History*/
const Value V22699_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_314 = {S31, 0, &V22699_314_57262, &P22699_312};	/*Property = Html5*/
const Value V22699_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_313 = {S32, 0, &V22699_313_57259, &P22699_314};	/*Property = Html-Media-Capture*/
const Value V22699_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_315 = {S33, 0, &V22699_315_57265, &P22699_313};	/*Property = Iframe*/
const Value V22699_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_316 = {S34, 0, &V22699_316_57268, &P22699_315};	/*Property = IndexedDB*/
const Value V22699_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_317 = {S35, 0, &V22699_317_57271, &P22699_316};	/*Property = Json*/
const Value V22699_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_15 = {S37, 0, &V22699_15_72, &P22699_317};	/*Property = LayoutEngine*/
const Value V22699_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_344 = {S38, 0, &V22699_344_57274, &P22699_15};	/*Property = Masking*/
const Value V22699_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_320 = {S39, 0, &V22699_320_57292, &P22699_344};	/*Property = PostMessage*/
const Value V22699_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_321 = {S40, 0, &V22699_321_57295, &P22699_320};	/*Property = Progress*/
const Value V22699_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_322 = {S41, 0, &V22699_322_57298, &P22699_321};	/*Property = Prompts*/
const Value V22699_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_323 = {S42, 0, &V22699_323_57301, &P22699_322};	/*Property = Selector*/
const Value V22699_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_324 = {S43, 0, &V22699_324_57307, &P22699_323};	/*Property = Svg*/
const Value V22699_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_327 = {S44, 0, &V22699_327_57316, &P22699_324};	/*Property = TouchEvents*/
const Value V22699_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_328 = {S45, 0, &V22699_328_57319, &P22699_327};	/*Property = Track*/
const Value V22699_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_329 = {S46, 0, &V22699_329_57325, &P22699_328};	/*Property = Video*/
const Value V22699_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_330 = {S47, 0, &V22699_330_57328, &P22699_329};	/*Property = Viewport*/
const Value V22699_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_332 = {S48, 0, &V22699_332_57340, &P22699_330};	/*Property = WebWorkers*/
const Value V22699_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22699_333 = {S49, 0, &V22699_333_57343, &P22699_332};	/*Property = Xhr2*/
const Profile P22699 = {"22699", &P22699_333};	/*Profile ID - 22699*/
const Value V14878_1_1 = {S2, NULL};	/*Value = True*/
const Property P14878_1 = {S50, 0, &V14878_1_1, NULL};	/*Property = IsMobile*/
const Value V14878_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14878_5 = {S52, 0, &V14878_5_1277, &P14878_1};	/*Property = ScreenPixelsHeight*/
const Value V14878_4_2299 = {S82, NULL};	/*Value = 960*/
const Property P14878_4 = {S54, 0, &V14878_4_2299, &P14878_5};	/*Property = ScreenPixelsWidth*/
const Profile P14878 = {"14878", &P14878_4};	/*Profile ID - 14878*/
const Value V12807_1_1 = {S2, NULL};	/*Value = True*/
const Property P12807_1 = {S50, 0, &V12807_1_1, NULL};	/*Property = IsMobile*/
const Value V12807_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12807_5 = {S52, 0, &V12807_5_1304, &P12807_1};	/*Property = ScreenPixelsHeight*/
const Value V12807_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12807_4 = {S54, 0, &V12807_4_1224, &P12807_5};	/*Property = ScreenPixelsWidth*/
const Profile P12807 = {"12807", &P12807_4};	/*Profile ID - 12807*/
const Profile P7048 = {"7048", NULL};	/*Profile ID - 7048*/
const Value V14881_1_1 = {S2, NULL};	/*Value = True*/
const Property P14881_1 = {S50, 0, &V14881_1_1, NULL};	/*Property = IsMobile*/
const Value V14881_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P14881_5 = {S52, 0, &V14881_5_1277, &P14881_1};	/*Property = ScreenPixelsHeight*/
const Value V14881_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14881_4 = {S54, 0, &V14881_4_1279, &P14881_5};	/*Property = ScreenPixelsWidth*/
const Profile P14881 = {"14881", &P14881_4};	/*Profile ID - 14881*/
const Value V17269_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17269_279 = {S1, 0, &V17269_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17269_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17269_282 = {S3, 0, &V17269_282_57154, &P17269_279};	/*Property = BlobBuilder*/
const Value V17269_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17269_350 = {S4, 0, &V17269_350_57444, &P17269_282};	/*Property = Canvas*/
const Value V17269_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17269_283 = {S5, 0, &V17269_283_57159, &P17269_350};	/*Property = CssBackground*/
const Value V17269_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17269_284 = {S6, 0, &V17269_284_57163, &P17269_283};	/*Property = CssBorderImage*/
const Value V17269_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17269_285 = {S7, 0, &V17269_285_57166, &P17269_284};	/*Property = CssCanvas*/
const Value V17269_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17269_286 = {S8, 0, &V17269_286_57168, &P17269_285};	/*Property = CssColor*/
const Value V17269_237_55785 = {S0, NULL};	/*Value = False*/
const Property P17269_237 = {S9, 0, &V17269_237_55785, &P17269_286};	/*Property = CssColumn*/
const Value V17269_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17269_287 = {S10, 0, &V17269_287_57172, &P17269_237};	/*Property = CssFlexbox*/
const Value V17269_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17269_288 = {S11, 0, &V17269_288_57175, &P17269_287};	/*Property = CssFont*/
const Value V17269_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17269_289 = {S12, 0, &V17269_289_86899, &P17269_288};	/*Property = CssImages*/
const Value V17269_291_57180 = {S2, NULL};	/*Value = True*/
const Property P17269_291 = {S13, 0, &V17269_291_57180, &P17269_289};	/*Property = CssMediaQueries*/
const Value V17269_292_57184 = {S0, NULL};	/*Value = False*/
const Property P17269_292 = {S14, 0, &V17269_292_57184, &P17269_291};	/*Property = CssMinMax*/
const Value V17269_293_57186 = {S2, NULL};	/*Value = True*/
const Property P17269_293 = {S15, 0, &V17269_293_57186, &P17269_292};	/*Property = CssOverflow*/
const Value V17269_294_57190 = {S0, NULL};	/*Value = False*/
const Property P17269_294 = {S16, 0, &V17269_294_57190, &P17269_293};	/*Property = CssPosition*/
const Value V17269_295_57193 = {S0, NULL};	/*Value = False*/
const Property P17269_295 = {S17, 0, &V17269_295_57193, &P17269_294};	/*Property = CssText*/
const Value V17269_297_57196 = {S0, NULL};	/*Value = False*/
const Property P17269_297 = {S18, 0, &V17269_297_57196, &P17269_295};	/*Property = CssTransforms*/
const Value V17269_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17269_241 = {S19, 0, &V17269_241_55799, &P17269_297};	/*Property = CssTransitions*/
const Value V17269_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17269_298 = {S20, 0, &V17269_298_57206, &P17269_241};	/*Property = CssUI*/
const Value V17269_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17269_300 = {S21, 0, &V17269_300_57216, &P17269_298};	/*Property = DataSet*/
const Value V17269_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17269_301 = {S22, 0, &V17269_301_57218, &P17269_300};	/*Property = DataUrl*/
const Value V17269_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17269_302 = {S23, 0, &V17269_302_57223, &P17269_301};	/*Property = DeviceOrientation*/
const Value V17269_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17269_304 = {S24, 0, &V17269_304_57229, &P17269_302};	/*Property = FileReader*/
const Value V17269_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17269_305 = {S25, 0, &V17269_305_57232, &P17269_304};	/*Property = FileSaver*/
const Value V17269_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17269_306 = {S26, 0, &V17269_306_57238, &P17269_305};	/*Property = FileWriter*/
const Value V17269_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17269_307 = {S27, 0, &V17269_307_57241, &P17269_306};	/*Property = FormData*/
const Value V17269_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17269_309 = {S28, 0, &V17269_309_57247, &P17269_307};	/*Property = Fullscreen*/
const Value V17269_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17269_310 = {S29, 0, &V17269_310_57248, &P17269_309};	/*Property = GeoLocation*/
const Value V17269_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17269_312 = {S30, 0, &V17269_312_57256, &P17269_310};	/*Property = History*/
const Value V17269_314_57260 = {S2, NULL};	/*Value = True*/
const Property P17269_314 = {S31, 0, &V17269_314_57260, &P17269_312};	/*Property = Html5*/
const Value V17269_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17269_313 = {S32, 0, &V17269_313_57259, &P17269_314};	/*Property = Html-Media-Capture*/
const Value V17269_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17269_315 = {S33, 0, &V17269_315_57265, &P17269_313};	/*Property = Iframe*/
const Value V17269_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17269_316 = {S34, 0, &V17269_316_57268, &P17269_315};	/*Property = IndexedDB*/
const Value V17269_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17269_317 = {S35, 0, &V17269_317_57269, &P17269_316};	/*Property = Json*/
const Value V17269_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P17269_15 = {S37, 0, &V17269_15_31, &P17269_317};	/*Property = LayoutEngine*/
const Value V17269_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17269_344 = {S38, 0, &V17269_344_57272, &P17269_15};	/*Property = Masking*/
const Value V17269_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17269_320 = {S39, 0, &V17269_320_57292, &P17269_344};	/*Property = PostMessage*/
const Value V17269_321_57294 = {S0, NULL};	/*Value = False*/
const Property P17269_321 = {S40, 0, &V17269_321_57294, &P17269_320};	/*Property = Progress*/
const Value V17269_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17269_322 = {S41, 0, &V17269_322_57296, &P17269_321};	/*Property = Prompts*/
const Value V17269_323_57300 = {S0, NULL};	/*Value = False*/
const Property P17269_323 = {S42, 0, &V17269_323_57300, &P17269_322};	/*Property = Selector*/
const Value V17269_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17269_324 = {S43, 0, &V17269_324_57307, &P17269_323};	/*Property = Svg*/
const Value V17269_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17269_327 = {S44, 0, &V17269_327_57316, &P17269_324};	/*Property = TouchEvents*/
const Value V17269_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17269_328 = {S45, 0, &V17269_328_57319, &P17269_327};	/*Property = Track*/
const Value V17269_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17269_329 = {S46, 0, &V17269_329_57323, &P17269_328};	/*Property = Video*/
const Value V17269_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17269_330 = {S47, 0, &V17269_330_57326, &P17269_329};	/*Property = Viewport*/
const Value V17269_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17269_332 = {S48, 0, &V17269_332_57340, &P17269_330};	/*Property = WebWorkers*/
const Value V17269_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17269_333 = {S49, 0, &V17269_333_57343, &P17269_332};	/*Property = Xhr2*/
const Profile P17269 = {"17269", &P17269_333};	/*Profile ID - 17269*/
const Value V14908_1_1 = {S2, NULL};	/*Value = True*/
const Property P14908_1 = {S50, 0, &V14908_1_1, NULL};	/*Property = IsMobile*/
const Value V14908_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P14908_5 = {S52, 0, &V14908_5_1832, &P14908_1};	/*Property = ScreenPixelsHeight*/
const Value V14908_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P14908_4 = {S54, 0, &V14908_4_5645, &P14908_5};	/*Property = ScreenPixelsWidth*/
const Profile P14908 = {"14908", &P14908_4};	/*Profile ID - 14908*/
const Value V14901_1_1 = {S2, NULL};	/*Value = True*/
const Property P14901_1 = {S50, 0, &V14901_1_1, NULL};	/*Property = IsMobile*/
const Value V14901_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14901_5 = {S52, 0, &V14901_5_1233, &P14901_1};	/*Property = ScreenPixelsHeight*/
const Value V14901_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14901_4 = {S54, 0, &V14901_4_1235, &P14901_5};	/*Property = ScreenPixelsWidth*/
const Profile P14901 = {"14901", &P14901_4};	/*Profile ID - 14901*/
const Value V14903_1_1 = {S2, NULL};	/*Value = True*/
const Property P14903_1 = {S50, 0, &V14903_1_1, NULL};	/*Property = IsMobile*/
const Value V14903_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P14903_5 = {S52, 0, &V14903_5_9981, &P14903_1};	/*Property = ScreenPixelsHeight*/
const Value V14903_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P14903_4 = {S54, 0, &V14903_4_37617, &P14903_5};	/*Property = ScreenPixelsWidth*/
const Profile P14903 = {"14903", &P14903_4};	/*Profile ID - 14903*/
const Value V17976_1_1 = {S2, NULL};	/*Value = True*/
const Property P17976_1 = {S50, 0, &V17976_1_1, NULL};	/*Property = IsMobile*/
const Value V17976_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P17976_5 = {S52, 0, &V17976_5_37, &P17976_1};	/*Property = ScreenPixelsHeight*/
const Value V17976_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P17976_4 = {S54, 0, &V17976_4_36, &P17976_5};	/*Property = ScreenPixelsWidth*/
const Profile P17976 = {"17976", &P17976_4};	/*Profile ID - 17976*/
const Value V4702_279_57148 = {S0, NULL};	/*Value = False*/
const Property P4702_279 = {S1, 0, &V4702_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V4702_282_57154 = {S0, NULL};	/*Value = False*/
const Property P4702_282 = {S3, 0, &V4702_282_57154, &P4702_279};	/*Property = BlobBuilder*/
const Value V4702_350_57444 = {S2, NULL};	/*Value = True*/
const Property P4702_350 = {S4, 0, &V4702_350_57444, &P4702_282};	/*Property = Canvas*/
const Value V4702_283_57159 = {S2, NULL};	/*Value = True*/
const Property P4702_283 = {S5, 0, &V4702_283_57159, &P4702_350};	/*Property = CssBackground*/
const Value V4702_284_57163 = {S0, NULL};	/*Value = False*/
const Property P4702_284 = {S6, 0, &V4702_284_57163, &P4702_283};	/*Property = CssBorderImage*/
const Value V4702_285_57166 = {S0, NULL};	/*Value = False*/
const Property P4702_285 = {S7, 0, &V4702_285_57166, &P4702_284};	/*Property = CssCanvas*/
const Value V4702_286_57168 = {S2, NULL};	/*Value = True*/
const Property P4702_286 = {S8, 0, &V4702_286_57168, &P4702_285};	/*Property = CssColor*/
const Value V4702_237_55785 = {S0, NULL};	/*Value = False*/
const Property P4702_237 = {S9, 0, &V4702_237_55785, &P4702_286};	/*Property = CssColumn*/
const Value V4702_287_57172 = {S0, NULL};	/*Value = False*/
const Property P4702_287 = {S10, 0, &V4702_287_57172, &P4702_237};	/*Property = CssFlexbox*/
const Value V4702_288_57175 = {S0, NULL};	/*Value = False*/
const Property P4702_288 = {S11, 0, &V4702_288_57175, &P4702_287};	/*Property = CssFont*/
const Value V4702_289_57759 = {S0, NULL};	/*Value = False*/
const Property P4702_289 = {S12, 0, &V4702_289_57759, &P4702_288};	/*Property = CssImages*/
const Value V4702_291_57181 = {S0, NULL};	/*Value = False*/
const Property P4702_291 = {S13, 0, &V4702_291_57181, &P4702_289};	/*Property = CssMediaQueries*/
const Value V4702_292_57183 = {S2, NULL};	/*Value = True*/
const Property P4702_292 = {S14, 0, &V4702_292_57183, &P4702_291};	/*Property = CssMinMax*/
const Value V4702_293_57187 = {S0, NULL};	/*Value = False*/
const Property P4702_293 = {S15, 0, &V4702_293_57187, &P4702_292};	/*Property = CssOverflow*/
const Value V4702_294_57190 = {S0, NULL};	/*Value = False*/
const Property P4702_294 = {S16, 0, &V4702_294_57190, &P4702_293};	/*Property = CssPosition*/
const Value V4702_295_57193 = {S0, NULL};	/*Value = False*/
const Property P4702_295 = {S17, 0, &V4702_295_57193, &P4702_294};	/*Property = CssText*/
const Value V4702_297_57196 = {S0, NULL};	/*Value = False*/
const Property P4702_297 = {S18, 0, &V4702_297_57196, &P4702_295};	/*Property = CssTransforms*/
const Value V4702_241_55800 = {S0, NULL};	/*Value = False*/
const Property P4702_241 = {S19, 0, &V4702_241_55800, &P4702_297};	/*Property = CssTransitions*/
const Value V4702_298_57206 = {S2, NULL};	/*Value = True*/
const Property P4702_298 = {S20, 0, &V4702_298_57206, &P4702_241};	/*Property = CssUI*/
const Value V4702_300_57216 = {S0, NULL};	/*Value = False*/
const Property P4702_300 = {S21, 0, &V4702_300_57216, &P4702_298};	/*Property = DataSet*/
const Value V4702_301_57218 = {S2, NULL};	/*Value = True*/
const Property P4702_301 = {S22, 0, &V4702_301_57218, &P4702_300};	/*Property = DataUrl*/
const Value V4702_302_57222 = {S0, NULL};	/*Value = False*/
const Property P4702_302 = {S23, 0, &V4702_302_57222, &P4702_301};	/*Property = DeviceOrientation*/
const Value V4702_304_57228 = {S0, NULL};	/*Value = False*/
const Property P4702_304 = {S24, 0, &V4702_304_57228, &P4702_302};	/*Property = FileReader*/
const Value V4702_305_57231 = {S0, NULL};	/*Value = False*/
const Property P4702_305 = {S25, 0, &V4702_305_57231, &P4702_304};	/*Property = FileSaver*/
const Value V4702_306_57237 = {S0, NULL};	/*Value = False*/
const Property P4702_306 = {S26, 0, &V4702_306_57237, &P4702_305};	/*Property = FileWriter*/
const Value V4702_307_57240 = {S0, NULL};	/*Value = False*/
const Property P4702_307 = {S27, 0, &V4702_307_57240, &P4702_306};	/*Property = FormData*/
const Value V4702_309_57246 = {S0, NULL};	/*Value = False*/
const Property P4702_309 = {S28, 0, &V4702_309_57246, &P4702_307};	/*Property = Fullscreen*/
const Value V4702_310_57248 = {S2, NULL};	/*Value = True*/
const Property P4702_310 = {S29, 0, &V4702_310_57248, &P4702_309};	/*Property = GeoLocation*/
const Value V4702_312_57255 = {S0, NULL};	/*Value = False*/
const Property P4702_312 = {S30, 0, &V4702_312_57255, &P4702_310};	/*Property = History*/
const Value V4702_314_57261 = {S0, NULL};	/*Value = False*/
const Property P4702_314 = {S31, 0, &V4702_314_57261, &P4702_312};	/*Property = Html5*/
const Value V4702_313_57258 = {S0, NULL};	/*Value = False*/
const Property P4702_313 = {S32, 0, &V4702_313_57258, &P4702_314};	/*Property = Html-Media-Capture*/
const Value V4702_315_57264 = {S0, NULL};	/*Value = False*/
const Property P4702_315 = {S33, 0, &V4702_315_57264, &P4702_313};	/*Property = Iframe*/
const Value V4702_316_57267 = {S0, NULL};	/*Value = False*/
const Property P4702_316 = {S34, 0, &V4702_316_57267, &P4702_315};	/*Property = IndexedDB*/
const Value V4702_317_57269 = {S2, NULL};	/*Value = True*/
const Property P4702_317 = {S35, 0, &V4702_317_57269, &P4702_316};	/*Property = Json*/
const Value V4702_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P4702_15 = {S37, 0, &V4702_15_30, &P4702_317};	/*Property = LayoutEngine*/
const Value V4702_344_57272 = {S2, NULL};	/*Value = True*/
const Property P4702_344 = {S38, 0, &V4702_344_57272, &P4702_15};	/*Property = Masking*/
const Value V4702_320_57290 = {S2, NULL};	/*Value = True*/
const Property P4702_320 = {S39, 0, &V4702_320_57290, &P4702_344};	/*Property = PostMessage*/
const Value V4702_321_57294 = {S0, NULL};	/*Value = False*/
const Property P4702_321 = {S40, 0, &V4702_321_57294, &P4702_320};	/*Property = Progress*/
const Value V4702_322_57297 = {S0, NULL};	/*Value = False*/
const Property P4702_322 = {S41, 0, &V4702_322_57297, &P4702_321};	/*Property = Prompts*/
const Value V4702_323_57299 = {S2, NULL};	/*Value = True*/
const Property P4702_323 = {S42, 0, &V4702_323_57299, &P4702_322};	/*Property = Selector*/
const Value V4702_324_57306 = {S0, NULL};	/*Value = False*/
const Property P4702_324 = {S43, 0, &V4702_324_57306, &P4702_323};	/*Property = Svg*/
const Value V4702_327_57315 = {S0, NULL};	/*Value = False*/
const Property P4702_327 = {S44, 0, &V4702_327_57315, &P4702_324};	/*Property = TouchEvents*/
const Value V4702_328_57318 = {S0, NULL};	/*Value = False*/
const Property P4702_328 = {S45, 0, &V4702_328_57318, &P4702_327};	/*Property = Track*/
const Value V4702_329_57323 = {S2, NULL};	/*Value = True*/
const Property P4702_329 = {S46, 0, &V4702_329_57323, &P4702_328};	/*Property = Video*/
const Value V4702_330_57326 = {S2, NULL};	/*Value = True*/
const Property P4702_330 = {S47, 0, &V4702_330_57326, &P4702_329};	/*Property = Viewport*/
const Value V4702_332_57339 = {S0, NULL};	/*Value = False*/
const Property P4702_332 = {S48, 0, &V4702_332_57339, &P4702_330};	/*Property = WebWorkers*/
const Value V4702_333_57342 = {S0, NULL};	/*Value = False*/
const Property P4702_333 = {S49, 0, &V4702_333_57342, &P4702_332};	/*Property = Xhr2*/
const Profile P4702 = {"4702", &P4702_333};	/*Profile ID - 4702*/
const Value V14904_1_1 = {S2, NULL};	/*Value = True*/
const Property P14904_1 = {S50, 0, &V14904_1_1, NULL};	/*Property = IsMobile*/
const Value V14904_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14904_5 = {S52, 0, &V14904_5_1227, &P14904_1};	/*Property = ScreenPixelsHeight*/
const Value V14904_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14904_4 = {S54, 0, &V14904_4_1229, &P14904_5};	/*Property = ScreenPixelsWidth*/
const Profile P14904 = {"14904", &P14904_4};	/*Profile ID - 14904*/
const Profile P15924 = {"15924", NULL};	/*Profile ID - 15924*/
const Value V14905_1_1 = {S2, NULL};	/*Value = True*/
const Property P14905_1 = {S50, 0, &V14905_1_1, NULL};	/*Property = IsMobile*/
const Value V14905_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14905_5 = {S52, 0, &V14905_5_1227, &P14905_1};	/*Property = ScreenPixelsHeight*/
const Value V14905_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P14905_4 = {S54, 0, &V14905_4_1477, &P14905_5};	/*Property = ScreenPixelsWidth*/
const Profile P14905 = {"14905", &P14905_4};	/*Profile ID - 14905*/
const Profile P24694 = {"24694", NULL};	/*Profile ID - 24694*/
const Value V14906_1_1 = {S2, NULL};	/*Value = True*/
const Property P14906_1 = {S50, 0, &V14906_1_1, NULL};	/*Property = IsMobile*/
const Value V14906_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14906_5 = {S52, 0, &V14906_5_1293, &P14906_1};	/*Property = ScreenPixelsHeight*/
const Value V14906_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14906_4 = {S54, 0, &V14906_4_1296, &P14906_5};	/*Property = ScreenPixelsWidth*/
const Profile P14906 = {"14906", &P14906_4};	/*Profile ID - 14906*/
const Value V14922_1_1 = {S2, NULL};	/*Value = True*/
const Property P14922_1 = {S50, 0, &V14922_1_1, NULL};	/*Property = IsMobile*/
const Value V14922_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14922_5 = {S52, 0, &V14922_5_1227, &P14922_1};	/*Property = ScreenPixelsHeight*/
const Value V14922_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14922_4 = {S54, 0, &V14922_4_1229, &P14922_5};	/*Property = ScreenPixelsWidth*/
const Profile P14922 = {"14922", &P14922_4};	/*Profile ID - 14922*/
const Value V14927_1_1 = {S2, NULL};	/*Value = True*/
const Property P14927_1 = {S50, 0, &V14927_1_1, NULL};	/*Property = IsMobile*/
const Value V14927_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14927_5 = {S52, 0, &V14927_5_1294, &P14927_1};	/*Property = ScreenPixelsHeight*/
const Value V14927_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14927_4 = {S54, 0, &V14927_4_1296, &P14927_5};	/*Property = ScreenPixelsWidth*/
const Profile P14927 = {"14927", &P14927_4};	/*Profile ID - 14927*/
const Value V14917_1_1 = {S2, NULL};	/*Value = True*/
const Property P14917_1 = {S50, 0, &V14917_1_1, NULL};	/*Property = IsMobile*/
const Value V14917_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14917_5 = {S52, 0, &V14917_5_1227, &P14917_1};	/*Property = ScreenPixelsHeight*/
const Value V14917_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14917_4 = {S54, 0, &V14917_4_1229, &P14917_5};	/*Property = ScreenPixelsWidth*/
const Profile P14917 = {"14917", &P14917_4};	/*Profile ID - 14917*/
const Value V14918_1_1 = {S2, NULL};	/*Value = True*/
const Property P14918_1 = {S50, 0, &V14918_1_1, NULL};	/*Property = IsMobile*/
const Value V14918_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14918_5 = {S52, 0, &V14918_5_1304, &P14918_1};	/*Property = ScreenPixelsHeight*/
const Value V14918_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14918_4 = {S54, 0, &V14918_4_1224, &P14918_5};	/*Property = ScreenPixelsWidth*/
const Profile P14918 = {"14918", &P14918_4};	/*Profile ID - 14918*/
const Value V14923_1_1 = {S2, NULL};	/*Value = True*/
const Property P14923_1 = {S50, 0, &V14923_1_1, NULL};	/*Property = IsMobile*/
const Value V14923_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P14923_5 = {S52, 0, &V14923_5_1458, &P14923_1};	/*Property = ScreenPixelsHeight*/
const Value V14923_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P14923_4 = {S54, 0, &V14923_4_1460, &P14923_5};	/*Property = ScreenPixelsWidth*/
const Profile P14923 = {"14923", &P14923_4};	/*Profile ID - 14923*/
const Value V14924_1_1 = {S2, NULL};	/*Value = True*/
const Property P14924_1 = {S50, 0, &V14924_1_1, NULL};	/*Property = IsMobile*/
const Value V14924_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14924_5 = {S52, 0, &V14924_5_1227, &P14924_1};	/*Property = ScreenPixelsHeight*/
const Value V14924_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P14924_4 = {S54, 0, &V14924_4_1229, &P14924_5};	/*Property = ScreenPixelsWidth*/
const Profile P14924 = {"14924", &P14924_4};	/*Profile ID - 14924*/
const Value V14925_1_1 = {S2, NULL};	/*Value = True*/
const Property P14925_1 = {S50, 0, &V14925_1_1, NULL};	/*Property = IsMobile*/
const Value V14925_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14925_5 = {S52, 0, &V14925_5_1222, &P14925_1};	/*Property = ScreenPixelsHeight*/
const Value V14925_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14925_4 = {S54, 0, &V14925_4_1224, &P14925_5};	/*Property = ScreenPixelsWidth*/
const Profile P14925 = {"14925", &P14925_4};	/*Profile ID - 14925*/
const Value V17712_1_1 = {S2, NULL};	/*Value = True*/
const Property P17712_1 = {S50, 0, &V17712_1_1, NULL};	/*Property = IsMobile*/
const Value V17712_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P17712_5 = {S52, 0, &V17712_5_9981, &P17712_1};	/*Property = ScreenPixelsHeight*/
const Value V17712_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P17712_4 = {S54, 0, &V17712_4_37617, &P17712_5};	/*Property = ScreenPixelsWidth*/
const Profile P17712 = {"17712", &P17712_4};	/*Profile ID - 17712*/
const Value V18001_1_1 = {S2, NULL};	/*Value = True*/
const Property P18001_1 = {S50, 0, &V18001_1_1, NULL};	/*Property = IsMobile*/
const Value V18001_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P18001_5 = {S52, 0, &V18001_5_1304, &P18001_1};	/*Property = ScreenPixelsHeight*/
const Value V18001_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18001_4 = {S54, 0, &V18001_4_1224, &P18001_5};	/*Property = ScreenPixelsWidth*/
const Profile P18001 = {"18001", &P18001_4};	/*Profile ID - 18001*/
const Value V14933_1_1 = {S2, NULL};	/*Value = True*/
const Property P14933_1 = {S50, 0, &V14933_1_1, NULL};	/*Property = IsMobile*/
const Value V14933_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14933_5 = {S52, 0, &V14933_5_1294, &P14933_1};	/*Property = ScreenPixelsHeight*/
const Value V14933_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14933_4 = {S54, 0, &V14933_4_1296, &P14933_5};	/*Property = ScreenPixelsWidth*/
const Profile P14933 = {"14933", &P14933_4};	/*Profile ID - 14933*/
const Value V14934_1_1 = {S2, NULL};	/*Value = True*/
const Property P14934_1 = {S50, 0, &V14934_1_1, NULL};	/*Property = IsMobile*/
const Value V14934_5_37463 = {S100, NULL};	/*Value = 64*/
const Property P14934_5 = {S52, 0, &V14934_5_37463, &P14934_1};	/*Property = ScreenPixelsHeight*/
const Value V14934_4_31956 = {S92, NULL};	/*Value = 96*/
const Property P14934_4 = {S54, 0, &V14934_4_31956, &P14934_5};	/*Property = ScreenPixelsWidth*/
const Profile P14934 = {"14934", &P14934_4};	/*Profile ID - 14934*/
const Value V14936_1_1 = {S2, NULL};	/*Value = True*/
const Property P14936_1 = {S50, 0, &V14936_1_1, NULL};	/*Property = IsMobile*/
const Value V14936_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14936_5 = {S52, 0, &V14936_5_1294, &P14936_1};	/*Property = ScreenPixelsHeight*/
const Value V14936_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14936_4 = {S54, 0, &V14936_4_1296, &P14936_5};	/*Property = ScreenPixelsWidth*/
const Profile P14936 = {"14936", &P14936_4};	/*Profile ID - 14936*/
const Value V14937_1_1 = {S2, NULL};	/*Value = True*/
const Property P14937_1 = {S50, 0, &V14937_1_1, NULL};	/*Property = IsMobile*/
const Value V14937_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14937_5 = {S52, 0, &V14937_5_1294, &P14937_1};	/*Property = ScreenPixelsHeight*/
const Value V14937_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14937_4 = {S54, 0, &V14937_4_1296, &P14937_5};	/*Property = ScreenPixelsWidth*/
const Profile P14937 = {"14937", &P14937_4};	/*Profile ID - 14937*/
const Value V14939_1_1 = {S2, NULL};	/*Value = True*/
const Property P14939_1 = {S50, 0, &V14939_1_1, NULL};	/*Property = IsMobile*/
const Value V14939_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14939_5 = {S52, 0, &V14939_5_1293, &P14939_1};	/*Property = ScreenPixelsHeight*/
const Value V14939_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14939_4 = {S54, 0, &V14939_4_1296, &P14939_5};	/*Property = ScreenPixelsWidth*/
const Profile P14939 = {"14939", &P14939_4};	/*Profile ID - 14939*/
const Value V24661_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_279 = {S1, 0, &V24661_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24661_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_282 = {S3, 0, &V24661_282_57155, &P24661_279};	/*Property = BlobBuilder*/
const Value V24661_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_350 = {S4, 0, &V24661_350_57446, &P24661_282};	/*Property = Canvas*/
const Value V24661_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_283 = {S5, 0, &V24661_283_57161, &P24661_350};	/*Property = CssBackground*/
const Value V24661_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_284 = {S6, 0, &V24661_284_57164, &P24661_283};	/*Property = CssBorderImage*/
const Value V24661_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_285 = {S7, 0, &V24661_285_57167, &P24661_284};	/*Property = CssCanvas*/
const Value V24661_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_286 = {S8, 0, &V24661_286_57170, &P24661_285};	/*Property = CssColor*/
const Value V24661_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_237 = {S9, 0, &V24661_237_55786, &P24661_286};	/*Property = CssColumn*/
const Value V24661_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_287 = {S10, 0, &V24661_287_57173, &P24661_237};	/*Property = CssFlexbox*/
const Value V24661_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_288 = {S11, 0, &V24661_288_57176, &P24661_287};	/*Property = CssFont*/
const Value V24661_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_289 = {S12, 0, &V24661_289_86899, &P24661_288};	/*Property = CssImages*/
const Value V24661_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_291 = {S13, 0, &V24661_291_57182, &P24661_289};	/*Property = CssMediaQueries*/
const Value V24661_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_292 = {S14, 0, &V24661_292_57185, &P24661_291};	/*Property = CssMinMax*/
const Value V24661_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_293 = {S15, 0, &V24661_293_57188, &P24661_292};	/*Property = CssOverflow*/
const Value V24661_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_294 = {S16, 0, &V24661_294_57191, &P24661_293};	/*Property = CssPosition*/
const Value V24661_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_295 = {S17, 0, &V24661_295_57194, &P24661_294};	/*Property = CssText*/
const Value V24661_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_297 = {S18, 0, &V24661_297_57197, &P24661_295};	/*Property = CssTransforms*/
const Value V24661_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_241 = {S19, 0, &V24661_241_55801, &P24661_297};	/*Property = CssTransitions*/
const Value V24661_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_298 = {S20, 0, &V24661_298_57208, &P24661_241};	/*Property = CssUI*/
const Value V24661_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_300 = {S21, 0, &V24661_300_57217, &P24661_298};	/*Property = DataSet*/
const Value V24661_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_301 = {S22, 0, &V24661_301_57220, &P24661_300};	/*Property = DataUrl*/
const Value V24661_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_302 = {S23, 0, &V24661_302_57223, &P24661_301};	/*Property = DeviceOrientation*/
const Value V24661_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_304 = {S24, 0, &V24661_304_57229, &P24661_302};	/*Property = FileReader*/
const Value V24661_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_305 = {S25, 0, &V24661_305_57232, &P24661_304};	/*Property = FileSaver*/
const Value V24661_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_306 = {S26, 0, &V24661_306_57238, &P24661_305};	/*Property = FileWriter*/
const Value V24661_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_307 = {S27, 0, &V24661_307_57241, &P24661_306};	/*Property = FormData*/
const Value V24661_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_309 = {S28, 0, &V24661_309_57247, &P24661_307};	/*Property = Fullscreen*/
const Value V24661_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_310 = {S29, 0, &V24661_310_57250, &P24661_309};	/*Property = GeoLocation*/
const Value V24661_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_312 = {S30, 0, &V24661_312_57256, &P24661_310};	/*Property = History*/
const Value V24661_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_314 = {S31, 0, &V24661_314_57262, &P24661_312};	/*Property = Html5*/
const Value V24661_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_313 = {S32, 0, &V24661_313_57259, &P24661_314};	/*Property = Html-Media-Capture*/
const Value V24661_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_315 = {S33, 0, &V24661_315_57265, &P24661_313};	/*Property = Iframe*/
const Value V24661_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_316 = {S34, 0, &V24661_316_57268, &P24661_315};	/*Property = IndexedDB*/
const Value V24661_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_317 = {S35, 0, &V24661_317_57271, &P24661_316};	/*Property = Json*/
const Value V24661_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24661_15 = {S37, 0, &V24661_15_90837, &P24661_317};	/*Property = LayoutEngine*/
const Value V24661_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_344 = {S38, 0, &V24661_344_57274, &P24661_15};	/*Property = Masking*/
const Value V24661_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_320 = {S39, 0, &V24661_320_57292, &P24661_344};	/*Property = PostMessage*/
const Value V24661_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_321 = {S40, 0, &V24661_321_57295, &P24661_320};	/*Property = Progress*/
const Value V24661_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_322 = {S41, 0, &V24661_322_57298, &P24661_321};	/*Property = Prompts*/
const Value V24661_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_323 = {S42, 0, &V24661_323_57301, &P24661_322};	/*Property = Selector*/
const Value V24661_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_324 = {S43, 0, &V24661_324_57307, &P24661_323};	/*Property = Svg*/
const Value V24661_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_327 = {S44, 0, &V24661_327_57316, &P24661_324};	/*Property = TouchEvents*/
const Value V24661_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_328 = {S45, 0, &V24661_328_57319, &P24661_327};	/*Property = Track*/
const Value V24661_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_329 = {S46, 0, &V24661_329_57325, &P24661_328};	/*Property = Video*/
const Value V24661_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_330 = {S47, 0, &V24661_330_57328, &P24661_329};	/*Property = Viewport*/
const Value V24661_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_332 = {S48, 0, &V24661_332_57340, &P24661_330};	/*Property = WebWorkers*/
const Value V24661_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24661_333 = {S49, 0, &V24661_333_57343, &P24661_332};	/*Property = Xhr2*/
const Profile P24661 = {"24661", &P24661_333};	/*Profile ID - 24661*/
const Value V13771_1_1 = {S2, NULL};	/*Value = True*/
const Property P13771_1 = {S50, 0, &V13771_1_1, NULL};	/*Property = IsMobile*/
const Value V13771_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13771_5 = {S52, 0, &V13771_5_1293, &P13771_1};	/*Property = ScreenPixelsHeight*/
const Value V13771_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13771_4 = {S54, 0, &V13771_4_1296, &P13771_5};	/*Property = ScreenPixelsWidth*/
const Profile P13771 = {"13771", &P13771_4};	/*Profile ID - 13771*/
const Value V14941_1_1 = {S2, NULL};	/*Value = True*/
const Property P14941_1 = {S50, 0, &V14941_1_1, NULL};	/*Property = IsMobile*/
const Value V14941_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14941_5 = {S52, 0, &V14941_5_1233, &P14941_1};	/*Property = ScreenPixelsHeight*/
const Value V14941_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14941_4 = {S54, 0, &V14941_4_1235, &P14941_5};	/*Property = ScreenPixelsWidth*/
const Profile P14941 = {"14941", &P14941_4};	/*Profile ID - 14941*/
const Value V14943_1_1 = {S2, NULL};	/*Value = True*/
const Property P14943_1 = {S50, 0, &V14943_1_1, NULL};	/*Property = IsMobile*/
const Value V14943_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14943_5 = {S52, 0, &V14943_5_1293, &P14943_1};	/*Property = ScreenPixelsHeight*/
const Value V14943_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14943_4 = {S54, 0, &V14943_4_1296, &P14943_5};	/*Property = ScreenPixelsWidth*/
const Profile P14943 = {"14943", &P14943_4};	/*Profile ID - 14943*/
const Value V14944_1_1 = {S2, NULL};	/*Value = True*/
const Property P14944_1 = {S50, 0, &V14944_1_1, NULL};	/*Property = IsMobile*/
const Value V14944_5_37483 = {S113, NULL};	/*Value = 67*/
const Property P14944_5 = {S52, 0, &V14944_5_37483, &P14944_1};	/*Property = ScreenPixelsHeight*/
const Value V14944_4_37587 = {S114, NULL};	/*Value = 98*/
const Property P14944_4 = {S54, 0, &V14944_4_37587, &P14944_5};	/*Property = ScreenPixelsWidth*/
const Profile P14944 = {"14944", &P14944_4};	/*Profile ID - 14944*/
const Value V14935_1_1 = {S2, NULL};	/*Value = True*/
const Property P14935_1 = {S50, 0, &V14935_1_1, NULL};	/*Property = IsMobile*/
const Value V14935_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14935_5 = {S52, 0, &V14935_5_1233, &P14935_1};	/*Property = ScreenPixelsHeight*/
const Value V14935_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14935_4 = {S54, 0, &V14935_4_1235, &P14935_5};	/*Property = ScreenPixelsWidth*/
const Profile P14935 = {"14935", &P14935_4};	/*Profile ID - 14935*/
const Value V14973_1_1 = {S2, NULL};	/*Value = True*/
const Property P14973_1 = {S50, 0, &V14973_1_1, NULL};	/*Property = IsMobile*/
const Value V14973_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P14973_5 = {S52, 0, &V14973_5_37569, &P14973_1};	/*Property = ScreenPixelsHeight*/
const Value V14973_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P14973_4 = {S54, 0, &V14973_4_1477, &P14973_5};	/*Property = ScreenPixelsWidth*/
const Profile P14973 = {"14973", &P14973_4};	/*Profile ID - 14973*/
const Value V14945_1_1 = {S2, NULL};	/*Value = True*/
const Property P14945_1 = {S50, 0, &V14945_1_1, NULL};	/*Property = IsMobile*/
const Value V14945_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14945_5 = {S52, 0, &V14945_5_1293, &P14945_1};	/*Property = ScreenPixelsHeight*/
const Value V14945_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14945_4 = {S54, 0, &V14945_4_1296, &P14945_5};	/*Property = ScreenPixelsWidth*/
const Profile P14945 = {"14945", &P14945_4};	/*Profile ID - 14945*/
const Value V12834_1_1 = {S2, NULL};	/*Value = True*/
const Property P12834_1 = {S50, 0, &V12834_1_1, NULL};	/*Property = IsMobile*/
const Value V12834_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12834_5 = {S52, 0, &V12834_5_1293, &P12834_1};	/*Property = ScreenPixelsHeight*/
const Value V12834_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12834_4 = {S54, 0, &V12834_4_1296, &P12834_5};	/*Property = ScreenPixelsWidth*/
const Profile P12834 = {"12834", &P12834_4};	/*Profile ID - 12834*/
const Value V14950_1_1 = {S2, NULL};	/*Value = True*/
const Property P14950_1 = {S50, 0, &V14950_1_1, NULL};	/*Property = IsMobile*/
const Value V14950_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P14950_5 = {S52, 0, &V14950_5_1294, &P14950_1};	/*Property = ScreenPixelsHeight*/
const Value V14950_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14950_4 = {S54, 0, &V14950_4_1296, &P14950_5};	/*Property = ScreenPixelsWidth*/
const Profile P14950 = {"14950", &P14950_4};	/*Profile ID - 14950*/
const Value V24665_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_279 = {S1, 0, &V24665_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24665_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_282 = {S3, 0, &V24665_282_57155, &P24665_279};	/*Property = BlobBuilder*/
const Value V24665_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_350 = {S4, 0, &V24665_350_57446, &P24665_282};	/*Property = Canvas*/
const Value V24665_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_283 = {S5, 0, &V24665_283_57161, &P24665_350};	/*Property = CssBackground*/
const Value V24665_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_284 = {S6, 0, &V24665_284_57164, &P24665_283};	/*Property = CssBorderImage*/
const Value V24665_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_285 = {S7, 0, &V24665_285_57167, &P24665_284};	/*Property = CssCanvas*/
const Value V24665_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_286 = {S8, 0, &V24665_286_57170, &P24665_285};	/*Property = CssColor*/
const Value V24665_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_237 = {S9, 0, &V24665_237_55786, &P24665_286};	/*Property = CssColumn*/
const Value V24665_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_287 = {S10, 0, &V24665_287_57173, &P24665_237};	/*Property = CssFlexbox*/
const Value V24665_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_288 = {S11, 0, &V24665_288_57176, &P24665_287};	/*Property = CssFont*/
const Value V24665_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_289 = {S12, 0, &V24665_289_86899, &P24665_288};	/*Property = CssImages*/
const Value V24665_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_291 = {S13, 0, &V24665_291_57182, &P24665_289};	/*Property = CssMediaQueries*/
const Value V24665_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_292 = {S14, 0, &V24665_292_57185, &P24665_291};	/*Property = CssMinMax*/
const Value V24665_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_293 = {S15, 0, &V24665_293_57188, &P24665_292};	/*Property = CssOverflow*/
const Value V24665_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_294 = {S16, 0, &V24665_294_57191, &P24665_293};	/*Property = CssPosition*/
const Value V24665_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_295 = {S17, 0, &V24665_295_57194, &P24665_294};	/*Property = CssText*/
const Value V24665_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_297 = {S18, 0, &V24665_297_57197, &P24665_295};	/*Property = CssTransforms*/
const Value V24665_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_241 = {S19, 0, &V24665_241_55801, &P24665_297};	/*Property = CssTransitions*/
const Value V24665_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_298 = {S20, 0, &V24665_298_57208, &P24665_241};	/*Property = CssUI*/
const Value V24665_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_300 = {S21, 0, &V24665_300_57217, &P24665_298};	/*Property = DataSet*/
const Value V24665_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_301 = {S22, 0, &V24665_301_57220, &P24665_300};	/*Property = DataUrl*/
const Value V24665_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_302 = {S23, 0, &V24665_302_57223, &P24665_301};	/*Property = DeviceOrientation*/
const Value V24665_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_304 = {S24, 0, &V24665_304_57229, &P24665_302};	/*Property = FileReader*/
const Value V24665_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_305 = {S25, 0, &V24665_305_57232, &P24665_304};	/*Property = FileSaver*/
const Value V24665_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_306 = {S26, 0, &V24665_306_57238, &P24665_305};	/*Property = FileWriter*/
const Value V24665_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_307 = {S27, 0, &V24665_307_57241, &P24665_306};	/*Property = FormData*/
const Value V24665_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_309 = {S28, 0, &V24665_309_57247, &P24665_307};	/*Property = Fullscreen*/
const Value V24665_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_310 = {S29, 0, &V24665_310_57250, &P24665_309};	/*Property = GeoLocation*/
const Value V24665_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_312 = {S30, 0, &V24665_312_57256, &P24665_310};	/*Property = History*/
const Value V24665_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_314 = {S31, 0, &V24665_314_57262, &P24665_312};	/*Property = Html5*/
const Value V24665_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_313 = {S32, 0, &V24665_313_57259, &P24665_314};	/*Property = Html-Media-Capture*/
const Value V24665_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_315 = {S33, 0, &V24665_315_57265, &P24665_313};	/*Property = Iframe*/
const Value V24665_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_316 = {S34, 0, &V24665_316_57268, &P24665_315};	/*Property = IndexedDB*/
const Value V24665_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_317 = {S35, 0, &V24665_317_57271, &P24665_316};	/*Property = Json*/
const Value V24665_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24665_15 = {S37, 0, &V24665_15_90837, &P24665_317};	/*Property = LayoutEngine*/
const Value V24665_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_344 = {S38, 0, &V24665_344_57274, &P24665_15};	/*Property = Masking*/
const Value V24665_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_320 = {S39, 0, &V24665_320_57292, &P24665_344};	/*Property = PostMessage*/
const Value V24665_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_321 = {S40, 0, &V24665_321_57295, &P24665_320};	/*Property = Progress*/
const Value V24665_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_322 = {S41, 0, &V24665_322_57298, &P24665_321};	/*Property = Prompts*/
const Value V24665_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_323 = {S42, 0, &V24665_323_57301, &P24665_322};	/*Property = Selector*/
const Value V24665_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_324 = {S43, 0, &V24665_324_57307, &P24665_323};	/*Property = Svg*/
const Value V24665_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_327 = {S44, 0, &V24665_327_57316, &P24665_324};	/*Property = TouchEvents*/
const Value V24665_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_328 = {S45, 0, &V24665_328_57319, &P24665_327};	/*Property = Track*/
const Value V24665_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_329 = {S46, 0, &V24665_329_57325, &P24665_328};	/*Property = Video*/
const Value V24665_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_330 = {S47, 0, &V24665_330_57328, &P24665_329};	/*Property = Viewport*/
const Value V24665_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_332 = {S48, 0, &V24665_332_57340, &P24665_330};	/*Property = WebWorkers*/
const Value V24665_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24665_333 = {S49, 0, &V24665_333_57343, &P24665_332};	/*Property = Xhr2*/
const Profile P24665 = {"24665", &P24665_333};	/*Profile ID - 24665*/
const Value V14954_1_1 = {S2, NULL};	/*Value = True*/
const Property P14954_1 = {S50, 0, &V14954_1_1, NULL};	/*Property = IsMobile*/
const Value V14954_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14954_5 = {S52, 0, &V14954_5_1293, &P14954_1};	/*Property = ScreenPixelsHeight*/
const Value V14954_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P14954_4 = {S54, 0, &V14954_4_1296, &P14954_5};	/*Property = ScreenPixelsWidth*/
const Profile P14954 = {"14954", &P14954_4};	/*Profile ID - 14954*/
const Value V17578_1_1 = {S2, NULL};	/*Value = True*/
const Property P17578_1 = {S50, 0, &V17578_1_1, NULL};	/*Property = IsMobile*/
const Value V17578_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17578_5 = {S52, 0, &V17578_5_1269, &P17578_1};	/*Property = ScreenPixelsHeight*/
const Value V17578_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17578_4 = {S54, 0, &V17578_4_1229, &P17578_5};	/*Property = ScreenPixelsWidth*/
const Profile P17578 = {"17578", &P17578_4};	/*Profile ID - 17578*/
const Value V16653_1_1 = {S2, NULL};	/*Value = True*/
const Property P16653_1 = {S50, 0, &V16653_1_1, NULL};	/*Property = IsMobile*/
const Value V16653_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16653_5 = {S52, 0, &V16653_5_1222, &P16653_1};	/*Property = ScreenPixelsHeight*/
const Value V16653_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16653_4 = {S54, 0, &V16653_4_1224, &P16653_5};	/*Property = ScreenPixelsWidth*/
const Profile P16653 = {"16653", &P16653_4};	/*Profile ID - 16653*/
const Value V17374_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_279 = {S1, 0, &V17374_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17374_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_282 = {S3, 0, &V17374_282_57155, &P17374_279};	/*Property = BlobBuilder*/
const Value V17374_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_350 = {S4, 0, &V17374_350_57446, &P17374_282};	/*Property = Canvas*/
const Value V17374_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_283 = {S5, 0, &V17374_283_57161, &P17374_350};	/*Property = CssBackground*/
const Value V17374_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_284 = {S6, 0, &V17374_284_57164, &P17374_283};	/*Property = CssBorderImage*/
const Value V17374_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_285 = {S7, 0, &V17374_285_57167, &P17374_284};	/*Property = CssCanvas*/
const Value V17374_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_286 = {S8, 0, &V17374_286_57170, &P17374_285};	/*Property = CssColor*/
const Value V17374_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_237 = {S9, 0, &V17374_237_55786, &P17374_286};	/*Property = CssColumn*/
const Value V17374_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_287 = {S10, 0, &V17374_287_57173, &P17374_237};	/*Property = CssFlexbox*/
const Value V17374_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_288 = {S11, 0, &V17374_288_57176, &P17374_287};	/*Property = CssFont*/
const Value V17374_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_289 = {S12, 0, &V17374_289_86899, &P17374_288};	/*Property = CssImages*/
const Value V17374_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_291 = {S13, 0, &V17374_291_57182, &P17374_289};	/*Property = CssMediaQueries*/
const Value V17374_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_292 = {S14, 0, &V17374_292_57185, &P17374_291};	/*Property = CssMinMax*/
const Value V17374_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_293 = {S15, 0, &V17374_293_57188, &P17374_292};	/*Property = CssOverflow*/
const Value V17374_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_294 = {S16, 0, &V17374_294_57191, &P17374_293};	/*Property = CssPosition*/
const Value V17374_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_295 = {S17, 0, &V17374_295_57194, &P17374_294};	/*Property = CssText*/
const Value V17374_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_297 = {S18, 0, &V17374_297_57197, &P17374_295};	/*Property = CssTransforms*/
const Value V17374_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_241 = {S19, 0, &V17374_241_55801, &P17374_297};	/*Property = CssTransitions*/
const Value V17374_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_298 = {S20, 0, &V17374_298_57208, &P17374_241};	/*Property = CssUI*/
const Value V17374_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_300 = {S21, 0, &V17374_300_57217, &P17374_298};	/*Property = DataSet*/
const Value V17374_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_301 = {S22, 0, &V17374_301_57220, &P17374_300};	/*Property = DataUrl*/
const Value V17374_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_302 = {S23, 0, &V17374_302_57223, &P17374_301};	/*Property = DeviceOrientation*/
const Value V17374_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_304 = {S24, 0, &V17374_304_57229, &P17374_302};	/*Property = FileReader*/
const Value V17374_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_305 = {S25, 0, &V17374_305_57232, &P17374_304};	/*Property = FileSaver*/
const Value V17374_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_306 = {S26, 0, &V17374_306_57238, &P17374_305};	/*Property = FileWriter*/
const Value V17374_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_307 = {S27, 0, &V17374_307_57241, &P17374_306};	/*Property = FormData*/
const Value V17374_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_309 = {S28, 0, &V17374_309_57247, &P17374_307};	/*Property = Fullscreen*/
const Value V17374_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_310 = {S29, 0, &V17374_310_57250, &P17374_309};	/*Property = GeoLocation*/
const Value V17374_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_312 = {S30, 0, &V17374_312_57256, &P17374_310};	/*Property = History*/
const Value V17374_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_314 = {S31, 0, &V17374_314_57262, &P17374_312};	/*Property = Html5*/
const Value V17374_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_313 = {S32, 0, &V17374_313_57259, &P17374_314};	/*Property = Html-Media-Capture*/
const Value V17374_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_315 = {S33, 0, &V17374_315_57265, &P17374_313};	/*Property = Iframe*/
const Value V17374_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_316 = {S34, 0, &V17374_316_57268, &P17374_315};	/*Property = IndexedDB*/
const Value V17374_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_317 = {S35, 0, &V17374_317_57271, &P17374_316};	/*Property = Json*/
const Value V17374_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P17374_15 = {S37, 0, &V17374_15_31, &P17374_317};	/*Property = LayoutEngine*/
const Value V17374_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_344 = {S38, 0, &V17374_344_57274, &P17374_15};	/*Property = Masking*/
const Value V17374_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_320 = {S39, 0, &V17374_320_57292, &P17374_344};	/*Property = PostMessage*/
const Value V17374_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_321 = {S40, 0, &V17374_321_57295, &P17374_320};	/*Property = Progress*/
const Value V17374_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_322 = {S41, 0, &V17374_322_57298, &P17374_321};	/*Property = Prompts*/
const Value V17374_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_323 = {S42, 0, &V17374_323_57301, &P17374_322};	/*Property = Selector*/
const Value V17374_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_324 = {S43, 0, &V17374_324_57307, &P17374_323};	/*Property = Svg*/
const Value V17374_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_327 = {S44, 0, &V17374_327_57316, &P17374_324};	/*Property = TouchEvents*/
const Value V17374_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_328 = {S45, 0, &V17374_328_57319, &P17374_327};	/*Property = Track*/
const Value V17374_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_329 = {S46, 0, &V17374_329_57325, &P17374_328};	/*Property = Video*/
const Value V17374_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_330 = {S47, 0, &V17374_330_57328, &P17374_329};	/*Property = Viewport*/
const Value V17374_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_332 = {S48, 0, &V17374_332_57340, &P17374_330};	/*Property = WebWorkers*/
const Value V17374_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17374_333 = {S49, 0, &V17374_333_57343, &P17374_332};	/*Property = Xhr2*/
const Profile P17374 = {"17374", &P17374_333};	/*Profile ID - 17374*/
const Value V18036_1_1 = {S2, NULL};	/*Value = True*/
const Property P18036_1 = {S50, 0, &V18036_1_1, NULL};	/*Property = IsMobile*/
const Value V18036_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18036_5 = {S52, 0, &V18036_5_1227, &P18036_1};	/*Property = ScreenPixelsHeight*/
const Value V18036_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18036_4 = {S54, 0, &V18036_4_1229, &P18036_5};	/*Property = ScreenPixelsWidth*/
const Profile P18036 = {"18036", &P18036_4};	/*Profile ID - 18036*/
const Value V17573_1_1 = {S2, NULL};	/*Value = True*/
const Property P17573_1 = {S50, 0, &V17573_1_1, NULL};	/*Property = IsMobile*/
const Value V17573_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17573_5 = {S52, 0, &V17573_5_1269, &P17573_1};	/*Property = ScreenPixelsHeight*/
const Value V17573_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17573_4 = {S54, 0, &V17573_4_1229, &P17573_5};	/*Property = ScreenPixelsWidth*/
const Profile P17573 = {"17573", &P17573_4};	/*Profile ID - 17573*/
const Value V15211_1_1 = {S2, NULL};	/*Value = True*/
const Property P15211_1 = {S50, 0, &V15211_1_1, NULL};	/*Property = IsMobile*/
const Value V15211_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15211_5 = {S52, 0, &V15211_5_1222, &P15211_1};	/*Property = ScreenPixelsHeight*/
const Value V15211_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15211_4 = {S54, 0, &V15211_4_1224, &P15211_5};	/*Property = ScreenPixelsWidth*/
const Profile P15211 = {"15211", &P15211_4};	/*Profile ID - 15211*/
const Value V15208_1_1 = {S2, NULL};	/*Value = True*/
const Property P15208_1 = {S50, 0, &V15208_1_1, NULL};	/*Property = IsMobile*/
const Value V15208_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15208_5 = {S52, 0, &V15208_5_1227, &P15208_1};	/*Property = ScreenPixelsHeight*/
const Value V15208_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15208_4 = {S54, 0, &V15208_4_1229, &P15208_5};	/*Property = ScreenPixelsWidth*/
const Profile P15208 = {"15208", &P15208_4};	/*Profile ID - 15208*/
const Profile P7149 = {"7149", NULL};	/*Profile ID - 7149*/
const Value V15463_1_1 = {S2, NULL};	/*Value = True*/
const Property P15463_1 = {S50, 0, &V15463_1_1, NULL};	/*Property = IsMobile*/
const Value V15463_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15463_5 = {S52, 0, &V15463_5_1294, &P15463_1};	/*Property = ScreenPixelsHeight*/
const Value V15463_4_37571 = {S115, NULL};	/*Value = 90*/
const Property P15463_4 = {S54, 0, &V15463_4_37571, &P15463_5};	/*Property = ScreenPixelsWidth*/
const Profile P15463 = {"15463", &P15463_4};	/*Profile ID - 15463*/
const Value V15224_1_1 = {S2, NULL};	/*Value = True*/
const Property P15224_1 = {S50, 0, &V15224_1_1, NULL};	/*Property = IsMobile*/
const Value V15224_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15224_5 = {S52, 0, &V15224_5_1380, &P15224_1};	/*Property = ScreenPixelsHeight*/
const Value V15224_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15224_4 = {S54, 0, &V15224_4_1279, &P15224_5};	/*Property = ScreenPixelsWidth*/
const Profile P15224 = {"15224", &P15224_4};	/*Profile ID - 15224*/
const Value V15226_1_1 = {S2, NULL};	/*Value = True*/
const Property P15226_1 = {S50, 0, &V15226_1_1, NULL};	/*Property = IsMobile*/
const Value V15226_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15226_5 = {S52, 0, &V15226_5_1233, &P15226_1};	/*Property = ScreenPixelsHeight*/
const Value V15226_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15226_4 = {S54, 0, &V15226_4_1235, &P15226_5};	/*Property = ScreenPixelsWidth*/
const Profile P15226 = {"15226", &P15226_4};	/*Profile ID - 15226*/
const Value V15026_1_1 = {S2, NULL};	/*Value = True*/
const Property P15026_1 = {S50, 0, &V15026_1_1, NULL};	/*Property = IsMobile*/
const Value V15026_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P15026_5 = {S52, 0, &V15026_5_1294, &P15026_1};	/*Property = ScreenPixelsHeight*/
const Value V15026_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15026_4 = {S54, 0, &V15026_4_1296, &P15026_5};	/*Property = ScreenPixelsWidth*/
const Profile P15026 = {"15026", &P15026_4};	/*Profile ID - 15026*/
const Value V15476_1_1 = {S2, NULL};	/*Value = True*/
const Property P15476_1 = {S50, 0, &V15476_1_1, NULL};	/*Property = IsMobile*/
const Value V15476_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15476_5 = {S52, 0, &V15476_5_1222, &P15476_1};	/*Property = ScreenPixelsHeight*/
const Value V15476_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15476_4 = {S54, 0, &V15476_4_1224, &P15476_5};	/*Property = ScreenPixelsWidth*/
const Profile P15476 = {"15476", &P15476_4};	/*Profile ID - 15476*/
const Value V16055_1_1 = {S2, NULL};	/*Value = True*/
const Property P16055_1 = {S50, 0, &V16055_1_1, NULL};	/*Property = IsMobile*/
const Value V16055_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16055_5 = {S52, 0, &V16055_5_1233, &P16055_1};	/*Property = ScreenPixelsHeight*/
const Value V16055_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16055_4 = {S54, 0, &V16055_4_1235, &P16055_5};	/*Property = ScreenPixelsWidth*/
const Profile P16055 = {"16055", &P16055_4};	/*Profile ID - 16055*/
const Value V17922_1_1 = {S2, NULL};	/*Value = True*/
const Property P17922_1 = {S50, 0, &V17922_1_1, NULL};	/*Property = IsMobile*/
const Value V17922_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17922_5 = {S52, 0, &V17922_5_1222, &P17922_1};	/*Property = ScreenPixelsHeight*/
const Value V17922_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17922_4 = {S54, 0, &V17922_4_1224, &P17922_5};	/*Property = ScreenPixelsWidth*/
const Profile P17922 = {"17922", &P17922_4};	/*Profile ID - 17922*/
const Value V13143_1_1 = {S2, NULL};	/*Value = True*/
const Property P13143_1 = {S50, 0, &V13143_1_1, NULL};	/*Property = IsMobile*/
const Value V13143_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13143_5 = {S52, 0, &V13143_5_1233, &P13143_1};	/*Property = ScreenPixelsHeight*/
const Value V13143_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13143_4 = {S54, 0, &V13143_4_1235, &P13143_5};	/*Property = ScreenPixelsWidth*/
const Profile P13143 = {"13143", &P13143_4};	/*Profile ID - 13143*/
const Value V15794_1_1 = {S2, NULL};	/*Value = True*/
const Property P15794_1 = {S50, 0, &V15794_1_1, NULL};	/*Property = IsMobile*/
const Value V15794_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15794_5 = {S52, 0, &V15794_5_1227, &P15794_1};	/*Property = ScreenPixelsHeight*/
const Value V15794_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15794_4 = {S54, 0, &V15794_4_1229, &P15794_5};	/*Property = ScreenPixelsWidth*/
const Profile P15794 = {"15794", &P15794_4};	/*Profile ID - 15794*/
const Value V15159_1_1 = {S2, NULL};	/*Value = True*/
const Property P15159_1 = {S50, 0, &V15159_1_1, NULL};	/*Property = IsMobile*/
const Value V15159_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15159_5 = {S52, 0, &V15159_5_1293, &P15159_1};	/*Property = ScreenPixelsHeight*/
const Value V15159_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15159_4 = {S54, 0, &V15159_4_1296, &P15159_5};	/*Property = ScreenPixelsWidth*/
const Profile P15159 = {"15159", &P15159_4};	/*Profile ID - 15159*/
const Value V18113_1_1 = {S2, NULL};	/*Value = True*/
const Property P18113_1 = {S50, 0, &V18113_1_1, NULL};	/*Property = IsMobile*/
const Value V18113_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18113_5 = {S52, 0, &V18113_5_1403, &P18113_1};	/*Property = ScreenPixelsHeight*/
const Value V18113_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18113_4 = {S54, 0, &V18113_4_1404, &P18113_5};	/*Property = ScreenPixelsWidth*/
const Profile P18113 = {"18113", &P18113_4};	/*Profile ID - 18113*/
const Value V15233_1_1 = {S2, NULL};	/*Value = True*/
const Property P15233_1 = {S50, 0, &V15233_1_1, NULL};	/*Property = IsMobile*/
const Value V15233_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15233_5 = {S52, 0, &V15233_5_1233, &P15233_1};	/*Property = ScreenPixelsHeight*/
const Value V15233_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15233_4 = {S54, 0, &V15233_4_1235, &P15233_5};	/*Property = ScreenPixelsWidth*/
const Profile P15233 = {"15233", &P15233_4};	/*Profile ID - 15233*/
const Value V21143_1_1 = {S2, NULL};	/*Value = True*/
const Property P21143_1 = {S50, 0, &V21143_1_1, NULL};	/*Property = IsMobile*/
const Value V21143_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21143_5 = {S52, 0, &V21143_5_1227, &P21143_1};	/*Property = ScreenPixelsHeight*/
const Value V21143_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21143_4 = {S54, 0, &V21143_4_1229, &P21143_5};	/*Property = ScreenPixelsWidth*/
const Profile P21143 = {"21143", &P21143_4};	/*Profile ID - 21143*/
const Value V21221_1_1 = {S2, NULL};	/*Value = True*/
const Property P21221_1 = {S50, 0, &V21221_1_1, NULL};	/*Property = IsMobile*/
const Value V21221_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21221_5 = {S52, 0, &V21221_5_1277, &P21221_1};	/*Property = ScreenPixelsHeight*/
const Value V21221_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21221_4 = {S54, 0, &V21221_4_1279, &P21221_5};	/*Property = ScreenPixelsWidth*/
const Profile P21221 = {"21221", &P21221_4};	/*Profile ID - 21221*/
const Value V17338_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_279 = {S1, 0, &V17338_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17338_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_282 = {S3, 0, &V17338_282_57155, &P17338_279};	/*Property = BlobBuilder*/
const Value V17338_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_350 = {S4, 0, &V17338_350_57446, &P17338_282};	/*Property = Canvas*/
const Value V17338_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_283 = {S5, 0, &V17338_283_57161, &P17338_350};	/*Property = CssBackground*/
const Value V17338_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_284 = {S6, 0, &V17338_284_57164, &P17338_283};	/*Property = CssBorderImage*/
const Value V17338_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_285 = {S7, 0, &V17338_285_57167, &P17338_284};	/*Property = CssCanvas*/
const Value V17338_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_286 = {S8, 0, &V17338_286_57170, &P17338_285};	/*Property = CssColor*/
const Value V17338_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_237 = {S9, 0, &V17338_237_55786, &P17338_286};	/*Property = CssColumn*/
const Value V17338_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_287 = {S10, 0, &V17338_287_57173, &P17338_237};	/*Property = CssFlexbox*/
const Value V17338_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_288 = {S11, 0, &V17338_288_57176, &P17338_287};	/*Property = CssFont*/
const Value V17338_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_289 = {S12, 0, &V17338_289_86899, &P17338_288};	/*Property = CssImages*/
const Value V17338_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_291 = {S13, 0, &V17338_291_57182, &P17338_289};	/*Property = CssMediaQueries*/
const Value V17338_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_292 = {S14, 0, &V17338_292_57185, &P17338_291};	/*Property = CssMinMax*/
const Value V17338_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_293 = {S15, 0, &V17338_293_57188, &P17338_292};	/*Property = CssOverflow*/
const Value V17338_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_294 = {S16, 0, &V17338_294_57191, &P17338_293};	/*Property = CssPosition*/
const Value V17338_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_295 = {S17, 0, &V17338_295_57194, &P17338_294};	/*Property = CssText*/
const Value V17338_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_297 = {S18, 0, &V17338_297_57197, &P17338_295};	/*Property = CssTransforms*/
const Value V17338_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_241 = {S19, 0, &V17338_241_55801, &P17338_297};	/*Property = CssTransitions*/
const Value V17338_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_298 = {S20, 0, &V17338_298_57208, &P17338_241};	/*Property = CssUI*/
const Value V17338_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_300 = {S21, 0, &V17338_300_57217, &P17338_298};	/*Property = DataSet*/
const Value V17338_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_301 = {S22, 0, &V17338_301_57220, &P17338_300};	/*Property = DataUrl*/
const Value V17338_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_302 = {S23, 0, &V17338_302_57223, &P17338_301};	/*Property = DeviceOrientation*/
const Value V17338_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_304 = {S24, 0, &V17338_304_57229, &P17338_302};	/*Property = FileReader*/
const Value V17338_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_305 = {S25, 0, &V17338_305_57232, &P17338_304};	/*Property = FileSaver*/
const Value V17338_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_306 = {S26, 0, &V17338_306_57238, &P17338_305};	/*Property = FileWriter*/
const Value V17338_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_307 = {S27, 0, &V17338_307_57241, &P17338_306};	/*Property = FormData*/
const Value V17338_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_309 = {S28, 0, &V17338_309_57247, &P17338_307};	/*Property = Fullscreen*/
const Value V17338_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_310 = {S29, 0, &V17338_310_57250, &P17338_309};	/*Property = GeoLocation*/
const Value V17338_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_312 = {S30, 0, &V17338_312_57256, &P17338_310};	/*Property = History*/
const Value V17338_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_314 = {S31, 0, &V17338_314_57262, &P17338_312};	/*Property = Html5*/
const Value V17338_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_313 = {S32, 0, &V17338_313_57259, &P17338_314};	/*Property = Html-Media-Capture*/
const Value V17338_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_315 = {S33, 0, &V17338_315_57265, &P17338_313};	/*Property = Iframe*/
const Value V17338_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_316 = {S34, 0, &V17338_316_57268, &P17338_315};	/*Property = IndexedDB*/
const Value V17338_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_317 = {S35, 0, &V17338_317_57271, &P17338_316};	/*Property = Json*/
const Value V17338_15_90838 = {S70, NULL};	/*Value = U3*/
const Property P17338_15 = {S37, 0, &V17338_15_90838, &P17338_317};	/*Property = LayoutEngine*/
const Value V17338_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_344 = {S38, 0, &V17338_344_57274, &P17338_15};	/*Property = Masking*/
const Value V17338_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_320 = {S39, 0, &V17338_320_57292, &P17338_344};	/*Property = PostMessage*/
const Value V17338_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_321 = {S40, 0, &V17338_321_57295, &P17338_320};	/*Property = Progress*/
const Value V17338_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_322 = {S41, 0, &V17338_322_57298, &P17338_321};	/*Property = Prompts*/
const Value V17338_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_323 = {S42, 0, &V17338_323_57301, &P17338_322};	/*Property = Selector*/
const Value V17338_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_324 = {S43, 0, &V17338_324_57307, &P17338_323};	/*Property = Svg*/
const Value V17338_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_327 = {S44, 0, &V17338_327_57316, &P17338_324};	/*Property = TouchEvents*/
const Value V17338_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_328 = {S45, 0, &V17338_328_57319, &P17338_327};	/*Property = Track*/
const Value V17338_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_329 = {S46, 0, &V17338_329_57325, &P17338_328};	/*Property = Video*/
const Value V17338_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_330 = {S47, 0, &V17338_330_57328, &P17338_329};	/*Property = Viewport*/
const Value V17338_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_332 = {S48, 0, &V17338_332_57340, &P17338_330};	/*Property = WebWorkers*/
const Value V17338_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17338_333 = {S49, 0, &V17338_333_57343, &P17338_332};	/*Property = Xhr2*/
const Profile P17338 = {"17338", &P17338_333};	/*Profile ID - 17338*/
const Value V15235_1_1 = {S2, NULL};	/*Value = True*/
const Property P15235_1 = {S50, 0, &V15235_1_1, NULL};	/*Property = IsMobile*/
const Value V15235_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15235_5 = {S52, 0, &V15235_5_1233, &P15235_1};	/*Property = ScreenPixelsHeight*/
const Value V15235_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15235_4 = {S54, 0, &V15235_4_1235, &P15235_5};	/*Property = ScreenPixelsWidth*/
const Profile P15235 = {"15235", &P15235_4};	/*Profile ID - 15235*/
const Value V21142_1_1 = {S2, NULL};	/*Value = True*/
const Property P21142_1 = {S50, 0, &V21142_1_1, NULL};	/*Property = IsMobile*/
const Value V21142_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21142_5 = {S52, 0, &V21142_5_1227, &P21142_1};	/*Property = ScreenPixelsHeight*/
const Value V21142_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21142_4 = {S54, 0, &V21142_4_1229, &P21142_5};	/*Property = ScreenPixelsWidth*/
const Profile P21142 = {"21142", &P21142_4};	/*Profile ID - 21142*/
const Value V21371_1_1 = {S2, NULL};	/*Value = True*/
const Property P21371_1 = {S50, 0, &V21371_1_1, NULL};	/*Property = IsMobile*/
const Value V21371_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21371_5 = {S52, 0, &V21371_5_1277, &P21371_1};	/*Property = ScreenPixelsHeight*/
const Value V21371_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21371_4 = {S54, 0, &V21371_4_1279, &P21371_5};	/*Property = ScreenPixelsWidth*/
const Profile P21371 = {"21371", &P21371_4};	/*Profile ID - 21371*/
const Value V12835_1_1 = {S2, NULL};	/*Value = True*/
const Property P12835_1 = {S50, 0, &V12835_1_1, NULL};	/*Property = IsMobile*/
const Value V12835_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12835_5 = {S52, 0, &V12835_5_1222, &P12835_1};	/*Property = ScreenPixelsHeight*/
const Value V12835_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12835_4 = {S54, 0, &V12835_4_1224, &P12835_5};	/*Property = ScreenPixelsWidth*/
const Profile P12835 = {"12835", &P12835_4};	/*Profile ID - 12835*/
const Value V17920_1_1 = {S2, NULL};	/*Value = True*/
const Property P17920_1 = {S50, 0, &V17920_1_1, NULL};	/*Property = IsMobile*/
const Value V17920_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17920_5 = {S52, 0, &V17920_5_1222, &P17920_1};	/*Property = ScreenPixelsHeight*/
const Value V17920_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17920_4 = {S54, 0, &V17920_4_1224, &P17920_5};	/*Property = ScreenPixelsWidth*/
const Profile P17920 = {"17920", &P17920_4};	/*Profile ID - 17920*/
const Value V15562_1_1 = {S2, NULL};	/*Value = True*/
const Property P15562_1 = {S50, 0, &V15562_1_1, NULL};	/*Property = IsMobile*/
const Value V15562_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15562_5 = {S52, 0, &V15562_5_1233, &P15562_1};	/*Property = ScreenPixelsHeight*/
const Value V15562_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15562_4 = {S54, 0, &V15562_4_1296, &P15562_5};	/*Property = ScreenPixelsWidth*/
const Profile P15562 = {"15562", &P15562_4};	/*Profile ID - 15562*/
const Value V16946_1_1 = {S2, NULL};	/*Value = True*/
const Property P16946_1 = {S50, 0, &V16946_1_1, NULL};	/*Property = IsMobile*/
const Value V16946_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16946_5 = {S52, 0, &V16946_5_1222, &P16946_1};	/*Property = ScreenPixelsHeight*/
const Value V16946_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16946_4 = {S54, 0, &V16946_4_1279, &P16946_5};	/*Property = ScreenPixelsWidth*/
const Profile P16946 = {"16946", &P16946_4};	/*Profile ID - 16946*/
const Value V15238_1_1 = {S2, NULL};	/*Value = True*/
const Property P15238_1 = {S50, 0, &V15238_1_1, NULL};	/*Property = IsMobile*/
const Value V15238_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15238_5 = {S52, 0, &V15238_5_1380, &P15238_1};	/*Property = ScreenPixelsHeight*/
const Value V15238_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15238_4 = {S54, 0, &V15238_4_1279, &P15238_5};	/*Property = ScreenPixelsWidth*/
const Profile P15238 = {"15238", &P15238_4};	/*Profile ID - 15238*/
const Value V15798_1_1 = {S2, NULL};	/*Value = True*/
const Property P15798_1 = {S50, 0, &V15798_1_1, NULL};	/*Property = IsMobile*/
const Value V15798_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15798_5 = {S52, 0, &V15798_5_1277, &P15798_1};	/*Property = ScreenPixelsHeight*/
const Value V15798_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15798_4 = {S54, 0, &V15798_4_1279, &P15798_5};	/*Property = ScreenPixelsWidth*/
const Profile P15798 = {"15798", &P15798_4};	/*Profile ID - 15798*/
const Value V20776_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_279 = {S1, 0, &V20776_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20776_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_282 = {S3, 0, &V20776_282_57155, &P20776_279};	/*Property = BlobBuilder*/
const Value V20776_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_350 = {S4, 0, &V20776_350_57446, &P20776_282};	/*Property = Canvas*/
const Value V20776_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_283 = {S5, 0, &V20776_283_57161, &P20776_350};	/*Property = CssBackground*/
const Value V20776_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_284 = {S6, 0, &V20776_284_57164, &P20776_283};	/*Property = CssBorderImage*/
const Value V20776_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_285 = {S7, 0, &V20776_285_57167, &P20776_284};	/*Property = CssCanvas*/
const Value V20776_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_286 = {S8, 0, &V20776_286_57170, &P20776_285};	/*Property = CssColor*/
const Value V20776_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_237 = {S9, 0, &V20776_237_55786, &P20776_286};	/*Property = CssColumn*/
const Value V20776_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_287 = {S10, 0, &V20776_287_57173, &P20776_237};	/*Property = CssFlexbox*/
const Value V20776_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_288 = {S11, 0, &V20776_288_57176, &P20776_287};	/*Property = CssFont*/
const Value V20776_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_289 = {S12, 0, &V20776_289_86899, &P20776_288};	/*Property = CssImages*/
const Value V20776_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_291 = {S13, 0, &V20776_291_57182, &P20776_289};	/*Property = CssMediaQueries*/
const Value V20776_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_292 = {S14, 0, &V20776_292_57185, &P20776_291};	/*Property = CssMinMax*/
const Value V20776_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_293 = {S15, 0, &V20776_293_57188, &P20776_292};	/*Property = CssOverflow*/
const Value V20776_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_294 = {S16, 0, &V20776_294_57191, &P20776_293};	/*Property = CssPosition*/
const Value V20776_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_295 = {S17, 0, &V20776_295_57194, &P20776_294};	/*Property = CssText*/
const Value V20776_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_297 = {S18, 0, &V20776_297_57197, &P20776_295};	/*Property = CssTransforms*/
const Value V20776_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_241 = {S19, 0, &V20776_241_55801, &P20776_297};	/*Property = CssTransitions*/
const Value V20776_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_298 = {S20, 0, &V20776_298_57208, &P20776_241};	/*Property = CssUI*/
const Value V20776_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_300 = {S21, 0, &V20776_300_57217, &P20776_298};	/*Property = DataSet*/
const Value V20776_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_301 = {S22, 0, &V20776_301_57220, &P20776_300};	/*Property = DataUrl*/
const Value V20776_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_302 = {S23, 0, &V20776_302_57223, &P20776_301};	/*Property = DeviceOrientation*/
const Value V20776_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_304 = {S24, 0, &V20776_304_57229, &P20776_302};	/*Property = FileReader*/
const Value V20776_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_305 = {S25, 0, &V20776_305_57232, &P20776_304};	/*Property = FileSaver*/
const Value V20776_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_306 = {S26, 0, &V20776_306_57238, &P20776_305};	/*Property = FileWriter*/
const Value V20776_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_307 = {S27, 0, &V20776_307_57241, &P20776_306};	/*Property = FormData*/
const Value V20776_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_309 = {S28, 0, &V20776_309_57247, &P20776_307};	/*Property = Fullscreen*/
const Value V20776_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_310 = {S29, 0, &V20776_310_57250, &P20776_309};	/*Property = GeoLocation*/
const Value V20776_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_312 = {S30, 0, &V20776_312_57256, &P20776_310};	/*Property = History*/
const Value V20776_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_314 = {S31, 0, &V20776_314_57262, &P20776_312};	/*Property = Html5*/
const Value V20776_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_313 = {S32, 0, &V20776_313_57259, &P20776_314};	/*Property = Html-Media-Capture*/
const Value V20776_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_315 = {S33, 0, &V20776_315_57265, &P20776_313};	/*Property = Iframe*/
const Value V20776_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_316 = {S34, 0, &V20776_316_57268, &P20776_315};	/*Property = IndexedDB*/
const Value V20776_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_317 = {S35, 0, &V20776_317_57271, &P20776_316};	/*Property = Json*/
const Value V20776_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_15 = {S37, 0, &V20776_15_72, &P20776_317};	/*Property = LayoutEngine*/
const Value V20776_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_344 = {S38, 0, &V20776_344_57274, &P20776_15};	/*Property = Masking*/
const Value V20776_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_320 = {S39, 0, &V20776_320_57292, &P20776_344};	/*Property = PostMessage*/
const Value V20776_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_321 = {S40, 0, &V20776_321_57295, &P20776_320};	/*Property = Progress*/
const Value V20776_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_322 = {S41, 0, &V20776_322_57298, &P20776_321};	/*Property = Prompts*/
const Value V20776_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_323 = {S42, 0, &V20776_323_57301, &P20776_322};	/*Property = Selector*/
const Value V20776_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_324 = {S43, 0, &V20776_324_57307, &P20776_323};	/*Property = Svg*/
const Value V20776_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_327 = {S44, 0, &V20776_327_57316, &P20776_324};	/*Property = TouchEvents*/
const Value V20776_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_328 = {S45, 0, &V20776_328_57319, &P20776_327};	/*Property = Track*/
const Value V20776_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_329 = {S46, 0, &V20776_329_57325, &P20776_328};	/*Property = Video*/
const Value V20776_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_330 = {S47, 0, &V20776_330_57328, &P20776_329};	/*Property = Viewport*/
const Value V20776_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_332 = {S48, 0, &V20776_332_57340, &P20776_330};	/*Property = WebWorkers*/
const Value V20776_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20776_333 = {S49, 0, &V20776_333_57343, &P20776_332};	/*Property = Xhr2*/
const Profile P20776 = {"20776", &P20776_333};	/*Profile ID - 20776*/
const Value V12832_1_1 = {S2, NULL};	/*Value = True*/
const Property P12832_1 = {S50, 0, &V12832_1_1, NULL};	/*Property = IsMobile*/
const Value V12832_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12832_5 = {S52, 0, &V12832_5_1380, &P12832_1};	/*Property = ScreenPixelsHeight*/
const Value V12832_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P12832_4 = {S54, 0, &V12832_4_1279, &P12832_5};	/*Property = ScreenPixelsWidth*/
const Profile P12832 = {"12832", &P12832_4};	/*Profile ID - 12832*/
const Value V17172_1_1 = {S2, NULL};	/*Value = True*/
const Property P17172_1 = {S50, 0, &V17172_1_1, NULL};	/*Property = IsMobile*/
const Value V17172_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17172_5 = {S52, 0, &V17172_5_1227, &P17172_1};	/*Property = ScreenPixelsHeight*/
const Value V17172_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P17172_4 = {S54, 0, &V17172_4_1477, &P17172_5};	/*Property = ScreenPixelsWidth*/
const Profile P17172 = {"17172", &P17172_4};	/*Profile ID - 17172*/
const Value V15244_1_1 = {S2, NULL};	/*Value = True*/
const Property P15244_1 = {S50, 0, &V15244_1_1, NULL};	/*Property = IsMobile*/
const Value V15244_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15244_5 = {S52, 0, &V15244_5_1304, &P15244_1};	/*Property = ScreenPixelsHeight*/
const Value V15244_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15244_4 = {S54, 0, &V15244_4_1224, &P15244_5};	/*Property = ScreenPixelsWidth*/
const Profile P15244 = {"15244", &P15244_4};	/*Profile ID - 15244*/
const Value V15877_1_1 = {S2, NULL};	/*Value = True*/
const Property P15877_1 = {S50, 0, &V15877_1_1, NULL};	/*Property = IsMobile*/
const Value V15877_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P15877_5 = {S52, 0, &V15877_5_1403, &P15877_1};	/*Property = ScreenPixelsHeight*/
const Value V15877_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P15877_4 = {S54, 0, &V15877_4_1404, &P15877_5};	/*Property = ScreenPixelsWidth*/
const Profile P15877 = {"15877", &P15877_4};	/*Profile ID - 15877*/
const Value V20777_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_279 = {S1, 0, &V20777_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20777_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_282 = {S3, 0, &V20777_282_57155, &P20777_279};	/*Property = BlobBuilder*/
const Value V20777_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_350 = {S4, 0, &V20777_350_57446, &P20777_282};	/*Property = Canvas*/
const Value V20777_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_283 = {S5, 0, &V20777_283_57161, &P20777_350};	/*Property = CssBackground*/
const Value V20777_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_284 = {S6, 0, &V20777_284_57164, &P20777_283};	/*Property = CssBorderImage*/
const Value V20777_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_285 = {S7, 0, &V20777_285_57167, &P20777_284};	/*Property = CssCanvas*/
const Value V20777_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_286 = {S8, 0, &V20777_286_57170, &P20777_285};	/*Property = CssColor*/
const Value V20777_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_237 = {S9, 0, &V20777_237_55786, &P20777_286};	/*Property = CssColumn*/
const Value V20777_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_287 = {S10, 0, &V20777_287_57173, &P20777_237};	/*Property = CssFlexbox*/
const Value V20777_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_288 = {S11, 0, &V20777_288_57176, &P20777_287};	/*Property = CssFont*/
const Value V20777_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_289 = {S12, 0, &V20777_289_86899, &P20777_288};	/*Property = CssImages*/
const Value V20777_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_291 = {S13, 0, &V20777_291_57182, &P20777_289};	/*Property = CssMediaQueries*/
const Value V20777_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_292 = {S14, 0, &V20777_292_57185, &P20777_291};	/*Property = CssMinMax*/
const Value V20777_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_293 = {S15, 0, &V20777_293_57188, &P20777_292};	/*Property = CssOverflow*/
const Value V20777_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_294 = {S16, 0, &V20777_294_57191, &P20777_293};	/*Property = CssPosition*/
const Value V20777_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_295 = {S17, 0, &V20777_295_57194, &P20777_294};	/*Property = CssText*/
const Value V20777_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_297 = {S18, 0, &V20777_297_57197, &P20777_295};	/*Property = CssTransforms*/
const Value V20777_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_241 = {S19, 0, &V20777_241_55801, &P20777_297};	/*Property = CssTransitions*/
const Value V20777_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_298 = {S20, 0, &V20777_298_57208, &P20777_241};	/*Property = CssUI*/
const Value V20777_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_300 = {S21, 0, &V20777_300_57217, &P20777_298};	/*Property = DataSet*/
const Value V20777_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_301 = {S22, 0, &V20777_301_57220, &P20777_300};	/*Property = DataUrl*/
const Value V20777_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_302 = {S23, 0, &V20777_302_57223, &P20777_301};	/*Property = DeviceOrientation*/
const Value V20777_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_304 = {S24, 0, &V20777_304_57229, &P20777_302};	/*Property = FileReader*/
const Value V20777_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_305 = {S25, 0, &V20777_305_57232, &P20777_304};	/*Property = FileSaver*/
const Value V20777_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_306 = {S26, 0, &V20777_306_57238, &P20777_305};	/*Property = FileWriter*/
const Value V20777_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_307 = {S27, 0, &V20777_307_57241, &P20777_306};	/*Property = FormData*/
const Value V20777_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_309 = {S28, 0, &V20777_309_57247, &P20777_307};	/*Property = Fullscreen*/
const Value V20777_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_310 = {S29, 0, &V20777_310_57250, &P20777_309};	/*Property = GeoLocation*/
const Value V20777_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_312 = {S30, 0, &V20777_312_57256, &P20777_310};	/*Property = History*/
const Value V20777_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_314 = {S31, 0, &V20777_314_57262, &P20777_312};	/*Property = Html5*/
const Value V20777_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_313 = {S32, 0, &V20777_313_57259, &P20777_314};	/*Property = Html-Media-Capture*/
const Value V20777_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_315 = {S33, 0, &V20777_315_57265, &P20777_313};	/*Property = Iframe*/
const Value V20777_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_316 = {S34, 0, &V20777_316_57268, &P20777_315};	/*Property = IndexedDB*/
const Value V20777_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_317 = {S35, 0, &V20777_317_57271, &P20777_316};	/*Property = Json*/
const Value V20777_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P20777_15 = {S37, 0, &V20777_15_54167, &P20777_317};	/*Property = LayoutEngine*/
const Value V20777_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_344 = {S38, 0, &V20777_344_57274, &P20777_15};	/*Property = Masking*/
const Value V20777_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_320 = {S39, 0, &V20777_320_57292, &P20777_344};	/*Property = PostMessage*/
const Value V20777_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_321 = {S40, 0, &V20777_321_57295, &P20777_320};	/*Property = Progress*/
const Value V20777_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_322 = {S41, 0, &V20777_322_57298, &P20777_321};	/*Property = Prompts*/
const Value V20777_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_323 = {S42, 0, &V20777_323_57301, &P20777_322};	/*Property = Selector*/
const Value V20777_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_324 = {S43, 0, &V20777_324_57307, &P20777_323};	/*Property = Svg*/
const Value V20777_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_327 = {S44, 0, &V20777_327_57316, &P20777_324};	/*Property = TouchEvents*/
const Value V20777_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_328 = {S45, 0, &V20777_328_57319, &P20777_327};	/*Property = Track*/
const Value V20777_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_329 = {S46, 0, &V20777_329_57325, &P20777_328};	/*Property = Video*/
const Value V20777_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_330 = {S47, 0, &V20777_330_57328, &P20777_329};	/*Property = Viewport*/
const Value V20777_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_332 = {S48, 0, &V20777_332_57340, &P20777_330};	/*Property = WebWorkers*/
const Value V20777_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20777_333 = {S49, 0, &V20777_333_57343, &P20777_332};	/*Property = Xhr2*/
const Profile P20777 = {"20777", &P20777_333};	/*Profile ID - 20777*/
const Value V15249_1_1 = {S2, NULL};	/*Value = True*/
const Property P15249_1 = {S50, 0, &V15249_1_1, NULL};	/*Property = IsMobile*/
const Value V15249_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P15249_5 = {S52, 0, &V15249_5_1304, &P15249_1};	/*Property = ScreenPixelsHeight*/
const Value V15249_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15249_4 = {S54, 0, &V15249_4_1224, &P15249_5};	/*Property = ScreenPixelsWidth*/
const Profile P15249 = {"15249", &P15249_4};	/*Profile ID - 15249*/
const Value V12685_1_1 = {S2, NULL};	/*Value = True*/
const Property P12685_1 = {S50, 0, &V12685_1_1, NULL};	/*Property = IsMobile*/
const Value V12685_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12685_5 = {S52, 0, &V12685_5_1222, &P12685_1};	/*Property = ScreenPixelsHeight*/
const Value V12685_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12685_4 = {S54, 0, &V12685_4_1224, &P12685_5};	/*Property = ScreenPixelsWidth*/
const Profile P12685 = {"12685", &P12685_4};	/*Profile ID - 12685*/
const Value V16356_1_1 = {S2, NULL};	/*Value = True*/
const Property P16356_1 = {S50, 0, &V16356_1_1, NULL};	/*Property = IsMobile*/
const Value V16356_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P16356_5 = {S52, 0, &V16356_5_9981, &P16356_1};	/*Property = ScreenPixelsHeight*/
const Value V16356_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P16356_4 = {S54, 0, &V16356_4_2336, &P16356_5};	/*Property = ScreenPixelsWidth*/
const Profile P16356 = {"16356", &P16356_4};	/*Profile ID - 16356*/
const Profile P7114 = {"7114", NULL};	/*Profile ID - 7114*/
const Value V15253_1_1 = {S2, NULL};	/*Value = True*/
const Property P15253_1 = {S50, 0, &V15253_1_1, NULL};	/*Property = IsMobile*/
const Value V15253_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15253_5 = {S52, 0, &V15253_5_1293, &P15253_1};	/*Property = ScreenPixelsHeight*/
const Value V15253_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15253_4 = {S54, 0, &V15253_4_1296, &P15253_5};	/*Property = ScreenPixelsWidth*/
const Profile P15253 = {"15253", &P15253_4};	/*Profile ID - 15253*/
const Value V16366_1_1 = {S2, NULL};	/*Value = True*/
const Property P16366_1 = {S50, 0, &V16366_1_1, NULL};	/*Property = IsMobile*/
const Value V16366_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16366_5 = {S52, 0, &V16366_5_1227, &P16366_1};	/*Property = ScreenPixelsHeight*/
const Value V16366_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16366_4 = {S54, 0, &V16366_4_1229, &P16366_5};	/*Property = ScreenPixelsWidth*/
const Profile P16366 = {"16366", &P16366_4};	/*Profile ID - 16366*/
const Profile P17342 = {"17342", NULL};	/*Profile ID - 17342*/
const Value V17454_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17454_279 = {S1, 0, &V17454_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17454_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17454_282 = {S3, 0, &V17454_282_57154, &P17454_279};	/*Property = BlobBuilder*/
const Value V17454_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17454_350 = {S4, 0, &V17454_350_57444, &P17454_282};	/*Property = Canvas*/
const Value V17454_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17454_283 = {S5, 0, &V17454_283_57159, &P17454_350};	/*Property = CssBackground*/
const Value V17454_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17454_284 = {S6, 0, &V17454_284_57163, &P17454_283};	/*Property = CssBorderImage*/
const Value V17454_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17454_285 = {S7, 0, &V17454_285_57166, &P17454_284};	/*Property = CssCanvas*/
const Value V17454_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17454_286 = {S8, 0, &V17454_286_57168, &P17454_285};	/*Property = CssColor*/
const Value V17454_237_55785 = {S0, NULL};	/*Value = False*/
const Property P17454_237 = {S9, 0, &V17454_237_55785, &P17454_286};	/*Property = CssColumn*/
const Value V17454_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17454_287 = {S10, 0, &V17454_287_57172, &P17454_237};	/*Property = CssFlexbox*/
const Value V17454_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17454_288 = {S11, 0, &V17454_288_57175, &P17454_287};	/*Property = CssFont*/
const Value V17454_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17454_289 = {S12, 0, &V17454_289_57759, &P17454_288};	/*Property = CssImages*/
const Value V17454_291_57180 = {S2, NULL};	/*Value = True*/
const Property P17454_291 = {S13, 0, &V17454_291_57180, &P17454_289};	/*Property = CssMediaQueries*/
const Value V17454_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17454_292 = {S14, 0, &V17454_292_57183, &P17454_291};	/*Property = CssMinMax*/
const Value V17454_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17454_293 = {S15, 0, &V17454_293_57187, &P17454_292};	/*Property = CssOverflow*/
const Value V17454_294_57190 = {S0, NULL};	/*Value = False*/
const Property P17454_294 = {S16, 0, &V17454_294_57190, &P17454_293};	/*Property = CssPosition*/
const Value V17454_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17454_295 = {S17, 0, &V17454_295_57192, &P17454_294};	/*Property = CssText*/
const Value V17454_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17454_297 = {S18, 0, &V17454_297_57195, &P17454_295};	/*Property = CssTransforms*/
const Value V17454_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17454_241 = {S19, 0, &V17454_241_55799, &P17454_297};	/*Property = CssTransitions*/
const Value V17454_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17454_298 = {S20, 0, &V17454_298_57206, &P17454_241};	/*Property = CssUI*/
const Value V17454_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17454_300 = {S21, 0, &V17454_300_57216, &P17454_298};	/*Property = DataSet*/
const Value V17454_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17454_301 = {S22, 0, &V17454_301_57218, &P17454_300};	/*Property = DataUrl*/
const Value V17454_302_57221 = {S2, NULL};	/*Value = True*/
const Property P17454_302 = {S23, 0, &V17454_302_57221, &P17454_301};	/*Property = DeviceOrientation*/
const Value V17454_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17454_304 = {S24, 0, &V17454_304_57228, &P17454_302};	/*Property = FileReader*/
const Value V17454_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17454_305 = {S25, 0, &V17454_305_57231, &P17454_304};	/*Property = FileSaver*/
const Value V17454_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17454_306 = {S26, 0, &V17454_306_57237, &P17454_305};	/*Property = FileWriter*/
const Value V17454_307_57239 = {S2, NULL};	/*Value = True*/
const Property P17454_307 = {S27, 0, &V17454_307_57239, &P17454_306};	/*Property = FormData*/
const Value V17454_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17454_309 = {S28, 0, &V17454_309_57246, &P17454_307};	/*Property = Fullscreen*/
const Value V17454_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17454_310 = {S29, 0, &V17454_310_57248, &P17454_309};	/*Property = GeoLocation*/
const Value V17454_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17454_312 = {S30, 0, &V17454_312_57255, &P17454_310};	/*Property = History*/
const Value V17454_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17454_314 = {S31, 0, &V17454_314_57261, &P17454_312};	/*Property = Html5*/
const Value V17454_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17454_313 = {S32, 0, &V17454_313_57258, &P17454_314};	/*Property = Html-Media-Capture*/
const Value V17454_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17454_315 = {S33, 0, &V17454_315_57264, &P17454_313};	/*Property = Iframe*/
const Value V17454_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17454_316 = {S34, 0, &V17454_316_57267, &P17454_315};	/*Property = IndexedDB*/
const Value V17454_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17454_317 = {S35, 0, &V17454_317_57269, &P17454_316};	/*Property = Json*/
const Value V17454_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17454_15 = {S37, 0, &V17454_15_28, &P17454_317};	/*Property = LayoutEngine*/
const Value V17454_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17454_344 = {S38, 0, &V17454_344_57272, &P17454_15};	/*Property = Masking*/
const Value V17454_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17454_320 = {S39, 0, &V17454_320_57290, &P17454_344};	/*Property = PostMessage*/
const Value V17454_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17454_321 = {S40, 0, &V17454_321_57293, &P17454_320};	/*Property = Progress*/
const Value V17454_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17454_322 = {S41, 0, &V17454_322_57296, &P17454_321};	/*Property = Prompts*/
const Value V17454_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17454_323 = {S42, 0, &V17454_323_57299, &P17454_322};	/*Property = Selector*/
const Value V17454_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17454_324 = {S43, 0, &V17454_324_57306, &P17454_323};	/*Property = Svg*/
const Value V17454_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17454_327 = {S44, 0, &V17454_327_57315, &P17454_324};	/*Property = TouchEvents*/
const Value V17454_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17454_328 = {S45, 0, &V17454_328_57318, &P17454_327};	/*Property = Track*/
const Value V17454_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17454_329 = {S46, 0, &V17454_329_57323, &P17454_328};	/*Property = Video*/
const Value V17454_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17454_330 = {S47, 0, &V17454_330_57326, &P17454_329};	/*Property = Viewport*/
const Value V17454_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17454_332 = {S48, 0, &V17454_332_57339, &P17454_330};	/*Property = WebWorkers*/
const Value V17454_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17454_333 = {S49, 0, &V17454_333_57342, &P17454_332};	/*Property = Xhr2*/
const Profile P17454 = {"17454", &P17454_333};	/*Profile ID - 17454*/
const Value V17927_1_1 = {S2, NULL};	/*Value = True*/
const Property P17927_1 = {S50, 0, &V17927_1_1, NULL};	/*Property = IsMobile*/
const Value V17927_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17927_5 = {S52, 0, &V17927_5_1277, &P17927_1};	/*Property = ScreenPixelsHeight*/
const Value V17927_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17927_4 = {S54, 0, &V17927_4_1279, &P17927_5};	/*Property = ScreenPixelsWidth*/
const Profile P17927 = {"17927", &P17927_4};	/*Profile ID - 17927*/
const Value V12681_1_1 = {S2, NULL};	/*Value = True*/
const Property P12681_1 = {S50, 0, &V12681_1_1, NULL};	/*Property = IsMobile*/
const Value V12681_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P12681_5 = {S52, 0, &V12681_5_1227, &P12681_1};	/*Property = ScreenPixelsHeight*/
const Value V12681_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12681_4 = {S54, 0, &V12681_4_1229, &P12681_5};	/*Property = ScreenPixelsWidth*/
const Profile P12681 = {"12681", &P12681_4};	/*Profile ID - 12681*/
const Value V18084_1_1 = {S2, NULL};	/*Value = True*/
const Property P18084_1 = {S50, 0, &V18084_1_1, NULL};	/*Property = IsMobile*/
const Value V18084_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18084_5 = {S52, 0, &V18084_5_1227, &P18084_1};	/*Property = ScreenPixelsHeight*/
const Value V18084_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P18084_4 = {S54, 0, &V18084_4_1477, &P18084_5};	/*Property = ScreenPixelsWidth*/
const Profile P18084 = {"18084", &P18084_4};	/*Profile ID - 18084*/
const Value V15255_1_1 = {S2, NULL};	/*Value = True*/
const Property P15255_1 = {S50, 0, &V15255_1_1, NULL};	/*Property = IsMobile*/
const Value V15255_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15255_5 = {S52, 0, &V15255_5_1380, &P15255_1};	/*Property = ScreenPixelsHeight*/
const Value V15255_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15255_4 = {S54, 0, &V15255_4_1279, &P15255_5};	/*Property = ScreenPixelsWidth*/
const Profile P15255 = {"15255", &P15255_4};	/*Profile ID - 15255*/
const Value V17168_1_1 = {S2, NULL};	/*Value = True*/
const Property P17168_1 = {S50, 0, &V17168_1_1, NULL};	/*Property = IsMobile*/
const Value V17168_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17168_5 = {S52, 0, &V17168_5_1227, &P17168_1};	/*Property = ScreenPixelsHeight*/
const Value V17168_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P17168_4 = {S54, 0, &V17168_4_1477, &P17168_5};	/*Property = ScreenPixelsWidth*/
const Profile P17168 = {"17168", &P17168_4};	/*Profile ID - 17168*/
const Value V13128_1_1 = {S2, NULL};	/*Value = True*/
const Property P13128_1 = {S50, 0, &V13128_1_1, NULL};	/*Property = IsMobile*/
const Value V13128_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13128_5 = {S52, 0, &V13128_5_1277, &P13128_1};	/*Property = ScreenPixelsHeight*/
const Value V13128_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13128_4 = {S54, 0, &V13128_4_1279, &P13128_5};	/*Property = ScreenPixelsWidth*/
const Profile P13128 = {"13128", &P13128_4};	/*Profile ID - 13128*/
const Value V17160_1_1 = {S2, NULL};	/*Value = True*/
const Property P17160_1 = {S50, 0, &V17160_1_1, NULL};	/*Property = IsMobile*/
const Value V17160_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17160_5 = {S52, 0, &V17160_5_1227, &P17160_1};	/*Property = ScreenPixelsHeight*/
const Value V17160_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P17160_4 = {S54, 0, &V17160_4_1477, &P17160_5};	/*Property = ScreenPixelsWidth*/
const Profile P17160 = {"17160", &P17160_4};	/*Profile ID - 17160*/
const Value V18660_1_1 = {S2, NULL};	/*Value = True*/
const Property P18660_1 = {S50, 0, &V18660_1_1, NULL};	/*Property = IsMobile*/
const Value V18660_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18660_5 = {S52, 0, &V18660_5_1403, &P18660_1};	/*Property = ScreenPixelsHeight*/
const Value V18660_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18660_4 = {S54, 0, &V18660_4_1404, &P18660_5};	/*Property = ScreenPixelsWidth*/
const Profile P18660 = {"18660", &P18660_4};	/*Profile ID - 18660*/
const Value V15241_1_1 = {S2, NULL};	/*Value = True*/
const Property P15241_1 = {S50, 0, &V15241_1_1, NULL};	/*Property = IsMobile*/
const Value V15241_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15241_5 = {S52, 0, &V15241_5_1222, &P15241_1};	/*Property = ScreenPixelsHeight*/
const Value V15241_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15241_4 = {S54, 0, &V15241_4_1224, &P15241_5};	/*Property = ScreenPixelsWidth*/
const Profile P15241 = {"15241", &P15241_4};	/*Profile ID - 15241*/
const Value V15931_1_1 = {S2, NULL};	/*Value = True*/
const Property P15931_1 = {S50, 0, &V15931_1_1, NULL};	/*Property = IsMobile*/
const Value V15931_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P15931_5 = {S52, 0, &V15931_5_1403, &P15931_1};	/*Property = ScreenPixelsHeight*/
const Value V15931_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P15931_4 = {S54, 0, &V15931_4_1404, &P15931_5};	/*Property = ScreenPixelsWidth*/
const Profile P15931 = {"15931", &P15931_4};	/*Profile ID - 15931*/
const Value V16321_1_1 = {S2, NULL};	/*Value = True*/
const Property P16321_1 = {S50, 0, &V16321_1_1, NULL};	/*Property = IsMobile*/
const Value V16321_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16321_5 = {S52, 0, &V16321_5_1277, &P16321_1};	/*Property = ScreenPixelsHeight*/
const Value V16321_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16321_4 = {S54, 0, &V16321_4_1279, &P16321_5};	/*Property = ScreenPixelsWidth*/
const Profile P16321 = {"16321", &P16321_4};	/*Profile ID - 16321*/
const Value V15985_1_1 = {S2, NULL};	/*Value = True*/
const Property P15985_1 = {S50, 0, &V15985_1_1, NULL};	/*Property = IsMobile*/
const Value V15985_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P15985_5 = {S52, 0, &V15985_5_1832, &P15985_1};	/*Property = ScreenPixelsHeight*/
const Value V15985_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P15985_4 = {S54, 0, &V15985_4_5645, &P15985_5};	/*Property = ScreenPixelsWidth*/
const Profile P15985 = {"15985", &P15985_4};	/*Profile ID - 15985*/
const Value V21058_1_1 = {S2, NULL};	/*Value = True*/
const Property P21058_1 = {S50, 0, &V21058_1_1, NULL};	/*Property = IsMobile*/
const Value V21058_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21058_5 = {S52, 0, &V21058_5_1227, &P21058_1};	/*Property = ScreenPixelsHeight*/
const Value V21058_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21058_4 = {S54, 0, &V21058_4_1229, &P21058_5};	/*Property = ScreenPixelsWidth*/
const Profile P21058 = {"21058", &P21058_4};	/*Profile ID - 21058*/
const Profile P7106 = {"7106", NULL};	/*Profile ID - 7106*/
const Value V15829_279_57148 = {S0, NULL};	/*Value = False*/
const Property P15829_279 = {S1, 0, &V15829_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V15829_282_57154 = {S0, NULL};	/*Value = False*/
const Property P15829_282 = {S3, 0, &V15829_282_57154, &P15829_279};	/*Property = BlobBuilder*/
const Value V15829_350_57444 = {S2, NULL};	/*Value = True*/
const Property P15829_350 = {S4, 0, &V15829_350_57444, &P15829_282};	/*Property = Canvas*/
const Value V15829_283_57159 = {S2, NULL};	/*Value = True*/
const Property P15829_283 = {S5, 0, &V15829_283_57159, &P15829_350};	/*Property = CssBackground*/
const Value V15829_284_57163 = {S0, NULL};	/*Value = False*/
const Property P15829_284 = {S6, 0, &V15829_284_57163, &P15829_283};	/*Property = CssBorderImage*/
const Value V15829_285_57166 = {S0, NULL};	/*Value = False*/
const Property P15829_285 = {S7, 0, &V15829_285_57166, &P15829_284};	/*Property = CssCanvas*/
const Value V15829_286_57168 = {S2, NULL};	/*Value = True*/
const Property P15829_286 = {S8, 0, &V15829_286_57168, &P15829_285};	/*Property = CssColor*/
const Value V15829_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P15829_237 = {S9, 0, &V15829_237_55786, &P15829_286};	/*Property = CssColumn*/
const Value V15829_287_57172 = {S0, NULL};	/*Value = False*/
const Property P15829_287 = {S10, 0, &V15829_287_57172, &P15829_237};	/*Property = CssFlexbox*/
const Value V15829_288_57175 = {S0, NULL};	/*Value = False*/
const Property P15829_288 = {S11, 0, &V15829_288_57175, &P15829_287};	/*Property = CssFont*/
const Value V15829_289_57759 = {S0, NULL};	/*Value = False*/
const Property P15829_289 = {S12, 0, &V15829_289_57759, &P15829_288};	/*Property = CssImages*/
const Value V15829_291_57181 = {S0, NULL};	/*Value = False*/
const Property P15829_291 = {S13, 0, &V15829_291_57181, &P15829_289};	/*Property = CssMediaQueries*/
const Value V15829_292_57183 = {S2, NULL};	/*Value = True*/
const Property P15829_292 = {S14, 0, &V15829_292_57183, &P15829_291};	/*Property = CssMinMax*/
const Value V15829_293_57187 = {S0, NULL};	/*Value = False*/
const Property P15829_293 = {S15, 0, &V15829_293_57187, &P15829_292};	/*Property = CssOverflow*/
const Value V15829_294_57189 = {S2, NULL};	/*Value = True*/
const Property P15829_294 = {S16, 0, &V15829_294_57189, &P15829_293};	/*Property = CssPosition*/
const Value V15829_295_57192 = {S2, NULL};	/*Value = True*/
const Property P15829_295 = {S17, 0, &V15829_295_57192, &P15829_294};	/*Property = CssText*/
const Value V15829_297_57195 = {S2, NULL};	/*Value = True*/
const Property P15829_297 = {S18, 0, &V15829_297_57195, &P15829_295};	/*Property = CssTransforms*/
const Value V15829_241_55799 = {S2, NULL};	/*Value = True*/
const Property P15829_241 = {S19, 0, &V15829_241_55799, &P15829_297};	/*Property = CssTransitions*/
const Value V15829_298_57206 = {S2, NULL};	/*Value = True*/
const Property P15829_298 = {S20, 0, &V15829_298_57206, &P15829_241};	/*Property = CssUI*/
const Value V15829_300_57216 = {S0, NULL};	/*Value = False*/
const Property P15829_300 = {S21, 0, &V15829_300_57216, &P15829_298};	/*Property = DataSet*/
const Value V15829_301_57218 = {S2, NULL};	/*Value = True*/
const Property P15829_301 = {S22, 0, &V15829_301_57218, &P15829_300};	/*Property = DataUrl*/
const Value V15829_302_57222 = {S0, NULL};	/*Value = False*/
const Property P15829_302 = {S23, 0, &V15829_302_57222, &P15829_301};	/*Property = DeviceOrientation*/
const Value V15829_304_57228 = {S0, NULL};	/*Value = False*/
const Property P15829_304 = {S24, 0, &V15829_304_57228, &P15829_302};	/*Property = FileReader*/
const Value V15829_305_57231 = {S0, NULL};	/*Value = False*/
const Property P15829_305 = {S25, 0, &V15829_305_57231, &P15829_304};	/*Property = FileSaver*/
const Value V15829_306_57237 = {S0, NULL};	/*Value = False*/
const Property P15829_306 = {S26, 0, &V15829_306_57237, &P15829_305};	/*Property = FileWriter*/
const Value V15829_307_57239 = {S2, NULL};	/*Value = True*/
const Property P15829_307 = {S27, 0, &V15829_307_57239, &P15829_306};	/*Property = FormData*/
const Value V15829_309_57246 = {S0, NULL};	/*Value = False*/
const Property P15829_309 = {S28, 0, &V15829_309_57246, &P15829_307};	/*Property = Fullscreen*/
const Value V15829_310_57248 = {S2, NULL};	/*Value = True*/
const Property P15829_310 = {S29, 0, &V15829_310_57248, &P15829_309};	/*Property = GeoLocation*/
const Value V15829_312_57255 = {S0, NULL};	/*Value = False*/
const Property P15829_312 = {S30, 0, &V15829_312_57255, &P15829_310};	/*Property = History*/
const Value V15829_314_57261 = {S0, NULL};	/*Value = False*/
const Property P15829_314 = {S31, 0, &V15829_314_57261, &P15829_312};	/*Property = Html5*/
const Value V15829_313_57258 = {S0, NULL};	/*Value = False*/
const Property P15829_313 = {S32, 0, &V15829_313_57258, &P15829_314};	/*Property = Html-Media-Capture*/
const Value V15829_315_57264 = {S0, NULL};	/*Value = False*/
const Property P15829_315 = {S33, 0, &V15829_315_57264, &P15829_313};	/*Property = Iframe*/
const Value V15829_316_57267 = {S0, NULL};	/*Value = False*/
const Property P15829_316 = {S34, 0, &V15829_316_57267, &P15829_315};	/*Property = IndexedDB*/
const Value V15829_317_57269 = {S2, NULL};	/*Value = True*/
const Property P15829_317 = {S35, 0, &V15829_317_57269, &P15829_316};	/*Property = Json*/
const Value V15829_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P15829_15 = {S37, 0, &V15829_15_28, &P15829_317};	/*Property = LayoutEngine*/
const Value V15829_344_57272 = {S2, NULL};	/*Value = True*/
const Property P15829_344 = {S38, 0, &V15829_344_57272, &P15829_15};	/*Property = Masking*/
const Value V15829_320_57290 = {S2, NULL};	/*Value = True*/
const Property P15829_320 = {S39, 0, &V15829_320_57290, &P15829_344};	/*Property = PostMessage*/
const Value V15829_321_57293 = {S2, NULL};	/*Value = True*/
const Property P15829_321 = {S40, 0, &V15829_321_57293, &P15829_320};	/*Property = Progress*/
const Value V15829_322_57296 = {S2, NULL};	/*Value = True*/
const Property P15829_322 = {S41, 0, &V15829_322_57296, &P15829_321};	/*Property = Prompts*/
const Value V15829_323_57299 = {S2, NULL};	/*Value = True*/
const Property P15829_323 = {S42, 0, &V15829_323_57299, &P15829_322};	/*Property = Selector*/
const Value V15829_324_57306 = {S0, NULL};	/*Value = False*/
const Property P15829_324 = {S43, 0, &V15829_324_57306, &P15829_323};	/*Property = Svg*/
const Value V15829_327_57314 = {S2, NULL};	/*Value = True*/
const Property P15829_327 = {S44, 0, &V15829_327_57314, &P15829_324};	/*Property = TouchEvents*/
const Value V15829_328_57318 = {S0, NULL};	/*Value = False*/
const Property P15829_328 = {S45, 0, &V15829_328_57318, &P15829_327};	/*Property = Track*/
const Value V15829_329_57323 = {S2, NULL};	/*Value = True*/
const Property P15829_329 = {S46, 0, &V15829_329_57323, &P15829_328};	/*Property = Video*/
const Value V15829_330_57326 = {S2, NULL};	/*Value = True*/
const Property P15829_330 = {S47, 0, &V15829_330_57326, &P15829_329};	/*Property = Viewport*/
const Value V15829_332_57339 = {S0, NULL};	/*Value = False*/
const Property P15829_332 = {S48, 0, &V15829_332_57339, &P15829_330};	/*Property = WebWorkers*/
const Value V15829_333_57342 = {S0, NULL};	/*Value = False*/
const Property P15829_333 = {S49, 0, &V15829_333_57342, &P15829_332};	/*Property = Xhr2*/
const Profile P15829 = {"15829", &P15829_333};	/*Profile ID - 15829*/
const Value V17989_1_1 = {S2, NULL};	/*Value = True*/
const Property P17989_1 = {S50, 0, &V17989_1_1, NULL};	/*Property = IsMobile*/
const Value V17989_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17989_5 = {S52, 0, &V17989_5_1277, &P17989_1};	/*Property = ScreenPixelsHeight*/
const Value V17989_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17989_4 = {S54, 0, &V17989_4_1279, &P17989_5};	/*Property = ScreenPixelsWidth*/
const Profile P17989 = {"17989", &P17989_4};	/*Profile ID - 17989*/
const Value V16696_1_1 = {S2, NULL};	/*Value = True*/
const Property P16696_1 = {S50, 0, &V16696_1_1, NULL};	/*Property = IsMobile*/
const Value V16696_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16696_5 = {S52, 0, &V16696_5_1277, &P16696_1};	/*Property = ScreenPixelsHeight*/
const Value V16696_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16696_4 = {S54, 0, &V16696_4_1279, &P16696_5};	/*Property = ScreenPixelsWidth*/
const Profile P16696 = {"16696", &P16696_4};	/*Profile ID - 16696*/
const Value V16384_1_1 = {S2, NULL};	/*Value = True*/
const Property P16384_1 = {S50, 0, &V16384_1_1, NULL};	/*Property = IsMobile*/
const Value V16384_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16384_5 = {S52, 0, &V16384_5_1277, &P16384_1};	/*Property = ScreenPixelsHeight*/
const Value V16384_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16384_4 = {S54, 0, &V16384_4_1279, &P16384_5};	/*Property = ScreenPixelsWidth*/
const Profile P16384 = {"16384", &P16384_4};	/*Profile ID - 16384*/
const Value V15262_1_1 = {S2, NULL};	/*Value = True*/
const Property P15262_1 = {S50, 0, &V15262_1_1, NULL};	/*Property = IsMobile*/
const Value V15262_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15262_5 = {S52, 0, &V15262_5_1233, &P15262_1};	/*Property = ScreenPixelsHeight*/
const Value V15262_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15262_4 = {S54, 0, &V15262_4_1235, &P15262_5};	/*Property = ScreenPixelsWidth*/
const Profile P15262 = {"15262", &P15262_4};	/*Profile ID - 15262*/
const Value V15805_1_1 = {S2, NULL};	/*Value = True*/
const Property P15805_1 = {S50, 0, &V15805_1_1, NULL};	/*Property = IsMobile*/
const Value V15805_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P15805_5 = {S52, 0, &V15805_5_1403, &P15805_1};	/*Property = ScreenPixelsHeight*/
const Value V15805_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P15805_4 = {S54, 0, &V15805_4_1404, &P15805_5};	/*Property = ScreenPixelsWidth*/
const Profile P15805 = {"15805", &P15805_4};	/*Profile ID - 15805*/
const Value V17438_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_279 = {S1, 0, &V17438_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17438_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_282 = {S3, 0, &V17438_282_57155, &P17438_279};	/*Property = BlobBuilder*/
const Value V17438_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_350 = {S4, 0, &V17438_350_57446, &P17438_282};	/*Property = Canvas*/
const Value V17438_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_283 = {S5, 0, &V17438_283_57161, &P17438_350};	/*Property = CssBackground*/
const Value V17438_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_284 = {S6, 0, &V17438_284_57164, &P17438_283};	/*Property = CssBorderImage*/
const Value V17438_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_285 = {S7, 0, &V17438_285_57167, &P17438_284};	/*Property = CssCanvas*/
const Value V17438_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_286 = {S8, 0, &V17438_286_57170, &P17438_285};	/*Property = CssColor*/
const Value V17438_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_237 = {S9, 0, &V17438_237_55786, &P17438_286};	/*Property = CssColumn*/
const Value V17438_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_287 = {S10, 0, &V17438_287_57173, &P17438_237};	/*Property = CssFlexbox*/
const Value V17438_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_288 = {S11, 0, &V17438_288_57176, &P17438_287};	/*Property = CssFont*/
const Value V17438_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_289 = {S12, 0, &V17438_289_86899, &P17438_288};	/*Property = CssImages*/
const Value V17438_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_291 = {S13, 0, &V17438_291_57182, &P17438_289};	/*Property = CssMediaQueries*/
const Value V17438_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_292 = {S14, 0, &V17438_292_57185, &P17438_291};	/*Property = CssMinMax*/
const Value V17438_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_293 = {S15, 0, &V17438_293_57188, &P17438_292};	/*Property = CssOverflow*/
const Value V17438_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_294 = {S16, 0, &V17438_294_57191, &P17438_293};	/*Property = CssPosition*/
const Value V17438_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_295 = {S17, 0, &V17438_295_57194, &P17438_294};	/*Property = CssText*/
const Value V17438_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_297 = {S18, 0, &V17438_297_57197, &P17438_295};	/*Property = CssTransforms*/
const Value V17438_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_241 = {S19, 0, &V17438_241_55801, &P17438_297};	/*Property = CssTransitions*/
const Value V17438_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_298 = {S20, 0, &V17438_298_57208, &P17438_241};	/*Property = CssUI*/
const Value V17438_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_300 = {S21, 0, &V17438_300_57217, &P17438_298};	/*Property = DataSet*/
const Value V17438_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_301 = {S22, 0, &V17438_301_57220, &P17438_300};	/*Property = DataUrl*/
const Value V17438_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_302 = {S23, 0, &V17438_302_57223, &P17438_301};	/*Property = DeviceOrientation*/
const Value V17438_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_304 = {S24, 0, &V17438_304_57229, &P17438_302};	/*Property = FileReader*/
const Value V17438_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_305 = {S25, 0, &V17438_305_57232, &P17438_304};	/*Property = FileSaver*/
const Value V17438_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_306 = {S26, 0, &V17438_306_57238, &P17438_305};	/*Property = FileWriter*/
const Value V17438_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_307 = {S27, 0, &V17438_307_57241, &P17438_306};	/*Property = FormData*/
const Value V17438_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_309 = {S28, 0, &V17438_309_57247, &P17438_307};	/*Property = Fullscreen*/
const Value V17438_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_310 = {S29, 0, &V17438_310_57250, &P17438_309};	/*Property = GeoLocation*/
const Value V17438_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_312 = {S30, 0, &V17438_312_57256, &P17438_310};	/*Property = History*/
const Value V17438_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_314 = {S31, 0, &V17438_314_57262, &P17438_312};	/*Property = Html5*/
const Value V17438_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_313 = {S32, 0, &V17438_313_57259, &P17438_314};	/*Property = Html-Media-Capture*/
const Value V17438_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_315 = {S33, 0, &V17438_315_57265, &P17438_313};	/*Property = Iframe*/
const Value V17438_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_316 = {S34, 0, &V17438_316_57268, &P17438_315};	/*Property = IndexedDB*/
const Value V17438_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_317 = {S35, 0, &V17438_317_57271, &P17438_316};	/*Property = Json*/
const Value V17438_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P17438_15 = {S37, 0, &V17438_15_29, &P17438_317};	/*Property = LayoutEngine*/
const Value V17438_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_344 = {S38, 0, &V17438_344_57274, &P17438_15};	/*Property = Masking*/
const Value V17438_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_320 = {S39, 0, &V17438_320_57292, &P17438_344};	/*Property = PostMessage*/
const Value V17438_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_321 = {S40, 0, &V17438_321_57295, &P17438_320};	/*Property = Progress*/
const Value V17438_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_322 = {S41, 0, &V17438_322_57298, &P17438_321};	/*Property = Prompts*/
const Value V17438_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_323 = {S42, 0, &V17438_323_57301, &P17438_322};	/*Property = Selector*/
const Value V17438_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_324 = {S43, 0, &V17438_324_57307, &P17438_323};	/*Property = Svg*/
const Value V17438_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_327 = {S44, 0, &V17438_327_57316, &P17438_324};	/*Property = TouchEvents*/
const Value V17438_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_328 = {S45, 0, &V17438_328_57319, &P17438_327};	/*Property = Track*/
const Value V17438_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_329 = {S46, 0, &V17438_329_57325, &P17438_328};	/*Property = Video*/
const Value V17438_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_330 = {S47, 0, &V17438_330_57328, &P17438_329};	/*Property = Viewport*/
const Value V17438_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_332 = {S48, 0, &V17438_332_57340, &P17438_330};	/*Property = WebWorkers*/
const Value V17438_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17438_333 = {S49, 0, &V17438_333_57343, &P17438_332};	/*Property = Xhr2*/
const Profile P17438 = {"17438", &P17438_333};	/*Profile ID - 17438*/
const Value V17200_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17200_279 = {S1, 0, &V17200_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17200_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17200_282 = {S3, 0, &V17200_282_57154, &P17200_279};	/*Property = BlobBuilder*/
const Value V17200_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17200_350 = {S4, 0, &V17200_350_57444, &P17200_282};	/*Property = Canvas*/
const Value V17200_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17200_283 = {S5, 0, &V17200_283_57159, &P17200_350};	/*Property = CssBackground*/
const Value V17200_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17200_284 = {S6, 0, &V17200_284_57163, &P17200_283};	/*Property = CssBorderImage*/
const Value V17200_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17200_285 = {S7, 0, &V17200_285_57166, &P17200_284};	/*Property = CssCanvas*/
const Value V17200_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17200_286 = {S8, 0, &V17200_286_57168, &P17200_285};	/*Property = CssColor*/
const Value V17200_237_55785 = {S0, NULL};	/*Value = False*/
const Property P17200_237 = {S9, 0, &V17200_237_55785, &P17200_286};	/*Property = CssColumn*/
const Value V17200_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17200_287 = {S10, 0, &V17200_287_57172, &P17200_237};	/*Property = CssFlexbox*/
const Value V17200_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17200_288 = {S11, 0, &V17200_288_57175, &P17200_287};	/*Property = CssFont*/
const Value V17200_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17200_289 = {S12, 0, &V17200_289_57759, &P17200_288};	/*Property = CssImages*/
const Value V17200_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17200_291 = {S13, 0, &V17200_291_57181, &P17200_289};	/*Property = CssMediaQueries*/
const Value V17200_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17200_292 = {S14, 0, &V17200_292_57183, &P17200_291};	/*Property = CssMinMax*/
const Value V17200_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17200_293 = {S15, 0, &V17200_293_57187, &P17200_292};	/*Property = CssOverflow*/
const Value V17200_294_57190 = {S0, NULL};	/*Value = False*/
const Property P17200_294 = {S16, 0, &V17200_294_57190, &P17200_293};	/*Property = CssPosition*/
const Value V17200_295_57193 = {S0, NULL};	/*Value = False*/
const Property P17200_295 = {S17, 0, &V17200_295_57193, &P17200_294};	/*Property = CssText*/
const Value V17200_297_57196 = {S0, NULL};	/*Value = False*/
const Property P17200_297 = {S18, 0, &V17200_297_57196, &P17200_295};	/*Property = CssTransforms*/
const Value V17200_241_55800 = {S0, NULL};	/*Value = False*/
const Property P17200_241 = {S19, 0, &V17200_241_55800, &P17200_297};	/*Property = CssTransitions*/
const Value V17200_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17200_298 = {S20, 0, &V17200_298_57206, &P17200_241};	/*Property = CssUI*/
const Value V17200_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17200_300 = {S21, 0, &V17200_300_57216, &P17200_298};	/*Property = DataSet*/
const Value V17200_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17200_301 = {S22, 0, &V17200_301_57218, &P17200_300};	/*Property = DataUrl*/
const Value V17200_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17200_302 = {S23, 0, &V17200_302_57222, &P17200_301};	/*Property = DeviceOrientation*/
const Value V17200_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17200_304 = {S24, 0, &V17200_304_57228, &P17200_302};	/*Property = FileReader*/
const Value V17200_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17200_305 = {S25, 0, &V17200_305_57231, &P17200_304};	/*Property = FileSaver*/
const Value V17200_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17200_306 = {S26, 0, &V17200_306_57237, &P17200_305};	/*Property = FileWriter*/
const Value V17200_307_57240 = {S0, NULL};	/*Value = False*/
const Property P17200_307 = {S27, 0, &V17200_307_57240, &P17200_306};	/*Property = FormData*/
const Value V17200_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17200_309 = {S28, 0, &V17200_309_57246, &P17200_307};	/*Property = Fullscreen*/
const Value V17200_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17200_310 = {S29, 0, &V17200_310_57248, &P17200_309};	/*Property = GeoLocation*/
const Value V17200_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17200_312 = {S30, 0, &V17200_312_57255, &P17200_310};	/*Property = History*/
const Value V17200_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17200_314 = {S31, 0, &V17200_314_57261, &P17200_312};	/*Property = Html5*/
const Value V17200_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17200_313 = {S32, 0, &V17200_313_57258, &P17200_314};	/*Property = Html-Media-Capture*/
const Value V17200_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17200_315 = {S33, 0, &V17200_315_57264, &P17200_313};	/*Property = Iframe*/
const Value V17200_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17200_316 = {S34, 0, &V17200_316_57267, &P17200_315};	/*Property = IndexedDB*/
const Value V17200_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17200_317 = {S35, 0, &V17200_317_57269, &P17200_316};	/*Property = Json*/
const Value V17200_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P17200_15 = {S37, 0, &V17200_15_30, &P17200_317};	/*Property = LayoutEngine*/
const Value V17200_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17200_344 = {S38, 0, &V17200_344_57272, &P17200_15};	/*Property = Masking*/
const Value V17200_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17200_320 = {S39, 0, &V17200_320_57290, &P17200_344};	/*Property = PostMessage*/
const Value V17200_321_57294 = {S0, NULL};	/*Value = False*/
const Property P17200_321 = {S40, 0, &V17200_321_57294, &P17200_320};	/*Property = Progress*/
const Value V17200_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17200_322 = {S41, 0, &V17200_322_57296, &P17200_321};	/*Property = Prompts*/
const Value V17200_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17200_323 = {S42, 0, &V17200_323_57299, &P17200_322};	/*Property = Selector*/
const Value V17200_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17200_324 = {S43, 0, &V17200_324_57306, &P17200_323};	/*Property = Svg*/
const Value V17200_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17200_327 = {S44, 0, &V17200_327_57315, &P17200_324};	/*Property = TouchEvents*/
const Value V17200_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17200_328 = {S45, 0, &V17200_328_57318, &P17200_327};	/*Property = Track*/
const Value V17200_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17200_329 = {S46, 0, &V17200_329_57323, &P17200_328};	/*Property = Video*/
const Value V17200_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17200_330 = {S47, 0, &V17200_330_57326, &P17200_329};	/*Property = Viewport*/
const Value V17200_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17200_332 = {S48, 0, &V17200_332_57339, &P17200_330};	/*Property = WebWorkers*/
const Value V17200_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17200_333 = {S49, 0, &V17200_333_57342, &P17200_332};	/*Property = Xhr2*/
const Profile P17200 = {"17200", &P17200_333};	/*Profile ID - 17200*/
const Value V4811_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_279 = {S1, 0, &V4811_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4811_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_282 = {S3, 0, &V4811_282_57155, &P4811_279};	/*Property = BlobBuilder*/
const Value V4811_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_350 = {S4, 0, &V4811_350_57446, &P4811_282};	/*Property = Canvas*/
const Value V4811_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_283 = {S5, 0, &V4811_283_57161, &P4811_350};	/*Property = CssBackground*/
const Value V4811_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_284 = {S6, 0, &V4811_284_57164, &P4811_283};	/*Property = CssBorderImage*/
const Value V4811_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_285 = {S7, 0, &V4811_285_57167, &P4811_284};	/*Property = CssCanvas*/
const Value V4811_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_286 = {S8, 0, &V4811_286_57170, &P4811_285};	/*Property = CssColor*/
const Value V4811_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_237 = {S9, 0, &V4811_237_55786, &P4811_286};	/*Property = CssColumn*/
const Value V4811_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_287 = {S10, 0, &V4811_287_57173, &P4811_237};	/*Property = CssFlexbox*/
const Value V4811_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_288 = {S11, 0, &V4811_288_57176, &P4811_287};	/*Property = CssFont*/
const Value V4811_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_289 = {S12, 0, &V4811_289_86899, &P4811_288};	/*Property = CssImages*/
const Value V4811_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_291 = {S13, 0, &V4811_291_57182, &P4811_289};	/*Property = CssMediaQueries*/
const Value V4811_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_292 = {S14, 0, &V4811_292_57185, &P4811_291};	/*Property = CssMinMax*/
const Value V4811_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_293 = {S15, 0, &V4811_293_57188, &P4811_292};	/*Property = CssOverflow*/
const Value V4811_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_294 = {S16, 0, &V4811_294_57191, &P4811_293};	/*Property = CssPosition*/
const Value V4811_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_295 = {S17, 0, &V4811_295_57194, &P4811_294};	/*Property = CssText*/
const Value V4811_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_297 = {S18, 0, &V4811_297_57197, &P4811_295};	/*Property = CssTransforms*/
const Value V4811_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_241 = {S19, 0, &V4811_241_55801, &P4811_297};	/*Property = CssTransitions*/
const Value V4811_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_298 = {S20, 0, &V4811_298_57208, &P4811_241};	/*Property = CssUI*/
const Value V4811_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_300 = {S21, 0, &V4811_300_57217, &P4811_298};	/*Property = DataSet*/
const Value V4811_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_301 = {S22, 0, &V4811_301_57220, &P4811_300};	/*Property = DataUrl*/
const Value V4811_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_302 = {S23, 0, &V4811_302_57223, &P4811_301};	/*Property = DeviceOrientation*/
const Value V4811_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_304 = {S24, 0, &V4811_304_57229, &P4811_302};	/*Property = FileReader*/
const Value V4811_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_305 = {S25, 0, &V4811_305_57232, &P4811_304};	/*Property = FileSaver*/
const Value V4811_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_306 = {S26, 0, &V4811_306_57238, &P4811_305};	/*Property = FileWriter*/
const Value V4811_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_307 = {S27, 0, &V4811_307_57241, &P4811_306};	/*Property = FormData*/
const Value V4811_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_309 = {S28, 0, &V4811_309_57247, &P4811_307};	/*Property = Fullscreen*/
const Value V4811_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_310 = {S29, 0, &V4811_310_57250, &P4811_309};	/*Property = GeoLocation*/
const Value V4811_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_312 = {S30, 0, &V4811_312_57256, &P4811_310};	/*Property = History*/
const Value V4811_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_314 = {S31, 0, &V4811_314_57262, &P4811_312};	/*Property = Html5*/
const Value V4811_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_313 = {S32, 0, &V4811_313_57259, &P4811_314};	/*Property = Html-Media-Capture*/
const Value V4811_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_315 = {S33, 0, &V4811_315_57265, &P4811_313};	/*Property = Iframe*/
const Value V4811_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_316 = {S34, 0, &V4811_316_57268, &P4811_315};	/*Property = IndexedDB*/
const Value V4811_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_317 = {S35, 0, &V4811_317_57271, &P4811_316};	/*Property = Json*/
const Value V4811_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P4811_15 = {S37, 0, &V4811_15_31, &P4811_317};	/*Property = LayoutEngine*/
const Value V4811_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_344 = {S38, 0, &V4811_344_57274, &P4811_15};	/*Property = Masking*/
const Value V4811_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_320 = {S39, 0, &V4811_320_57292, &P4811_344};	/*Property = PostMessage*/
const Value V4811_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_321 = {S40, 0, &V4811_321_57295, &P4811_320};	/*Property = Progress*/
const Value V4811_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_322 = {S41, 0, &V4811_322_57298, &P4811_321};	/*Property = Prompts*/
const Value V4811_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_323 = {S42, 0, &V4811_323_57301, &P4811_322};	/*Property = Selector*/
const Value V4811_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_324 = {S43, 0, &V4811_324_57307, &P4811_323};	/*Property = Svg*/
const Value V4811_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_327 = {S44, 0, &V4811_327_57316, &P4811_324};	/*Property = TouchEvents*/
const Value V4811_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_328 = {S45, 0, &V4811_328_57319, &P4811_327};	/*Property = Track*/
const Value V4811_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_329 = {S46, 0, &V4811_329_57325, &P4811_328};	/*Property = Video*/
const Value V4811_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_330 = {S47, 0, &V4811_330_57328, &P4811_329};	/*Property = Viewport*/
const Value V4811_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_332 = {S48, 0, &V4811_332_57340, &P4811_330};	/*Property = WebWorkers*/
const Value V4811_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4811_333 = {S49, 0, &V4811_333_57343, &P4811_332};	/*Property = Xhr2*/
const Profile P4811 = {"4811", &P4811_333};	/*Profile ID - 4811*/
const Value V17232_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_279 = {S1, 0, &V17232_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17232_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_282 = {S3, 0, &V17232_282_57155, &P17232_279};	/*Property = BlobBuilder*/
const Value V17232_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_350 = {S4, 0, &V17232_350_57446, &P17232_282};	/*Property = Canvas*/
const Value V17232_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_283 = {S5, 0, &V17232_283_57161, &P17232_350};	/*Property = CssBackground*/
const Value V17232_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_284 = {S6, 0, &V17232_284_57164, &P17232_283};	/*Property = CssBorderImage*/
const Value V17232_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_285 = {S7, 0, &V17232_285_57167, &P17232_284};	/*Property = CssCanvas*/
const Value V17232_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_286 = {S8, 0, &V17232_286_57170, &P17232_285};	/*Property = CssColor*/
const Value V17232_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_237 = {S9, 0, &V17232_237_55786, &P17232_286};	/*Property = CssColumn*/
const Value V17232_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_287 = {S10, 0, &V17232_287_57173, &P17232_237};	/*Property = CssFlexbox*/
const Value V17232_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_288 = {S11, 0, &V17232_288_57176, &P17232_287};	/*Property = CssFont*/
const Value V17232_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_289 = {S12, 0, &V17232_289_86899, &P17232_288};	/*Property = CssImages*/
const Value V17232_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_291 = {S13, 0, &V17232_291_57182, &P17232_289};	/*Property = CssMediaQueries*/
const Value V17232_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_292 = {S14, 0, &V17232_292_57185, &P17232_291};	/*Property = CssMinMax*/
const Value V17232_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_293 = {S15, 0, &V17232_293_57188, &P17232_292};	/*Property = CssOverflow*/
const Value V17232_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_294 = {S16, 0, &V17232_294_57191, &P17232_293};	/*Property = CssPosition*/
const Value V17232_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_295 = {S17, 0, &V17232_295_57194, &P17232_294};	/*Property = CssText*/
const Value V17232_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_297 = {S18, 0, &V17232_297_57197, &P17232_295};	/*Property = CssTransforms*/
const Value V17232_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_241 = {S19, 0, &V17232_241_55801, &P17232_297};	/*Property = CssTransitions*/
const Value V17232_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_298 = {S20, 0, &V17232_298_57208, &P17232_241};	/*Property = CssUI*/
const Value V17232_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_300 = {S21, 0, &V17232_300_57217, &P17232_298};	/*Property = DataSet*/
const Value V17232_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_301 = {S22, 0, &V17232_301_57220, &P17232_300};	/*Property = DataUrl*/
const Value V17232_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_302 = {S23, 0, &V17232_302_57223, &P17232_301};	/*Property = DeviceOrientation*/
const Value V17232_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_304 = {S24, 0, &V17232_304_57229, &P17232_302};	/*Property = FileReader*/
const Value V17232_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_305 = {S25, 0, &V17232_305_57232, &P17232_304};	/*Property = FileSaver*/
const Value V17232_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_306 = {S26, 0, &V17232_306_57238, &P17232_305};	/*Property = FileWriter*/
const Value V17232_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_307 = {S27, 0, &V17232_307_57241, &P17232_306};	/*Property = FormData*/
const Value V17232_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_309 = {S28, 0, &V17232_309_57247, &P17232_307};	/*Property = Fullscreen*/
const Value V17232_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_310 = {S29, 0, &V17232_310_57250, &P17232_309};	/*Property = GeoLocation*/
const Value V17232_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_312 = {S30, 0, &V17232_312_57256, &P17232_310};	/*Property = History*/
const Value V17232_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_314 = {S31, 0, &V17232_314_57262, &P17232_312};	/*Property = Html5*/
const Value V17232_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_313 = {S32, 0, &V17232_313_57259, &P17232_314};	/*Property = Html-Media-Capture*/
const Value V17232_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_315 = {S33, 0, &V17232_315_57265, &P17232_313};	/*Property = Iframe*/
const Value V17232_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_316 = {S34, 0, &V17232_316_57268, &P17232_315};	/*Property = IndexedDB*/
const Value V17232_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_317 = {S35, 0, &V17232_317_57271, &P17232_316};	/*Property = Json*/
const Value V17232_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17232_15 = {S37, 0, &V17232_15_28, &P17232_317};	/*Property = LayoutEngine*/
const Value V17232_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_344 = {S38, 0, &V17232_344_57274, &P17232_15};	/*Property = Masking*/
const Value V17232_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_320 = {S39, 0, &V17232_320_57292, &P17232_344};	/*Property = PostMessage*/
const Value V17232_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_321 = {S40, 0, &V17232_321_57295, &P17232_320};	/*Property = Progress*/
const Value V17232_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_322 = {S41, 0, &V17232_322_57298, &P17232_321};	/*Property = Prompts*/
const Value V17232_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_323 = {S42, 0, &V17232_323_57301, &P17232_322};	/*Property = Selector*/
const Value V17232_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_324 = {S43, 0, &V17232_324_57307, &P17232_323};	/*Property = Svg*/
const Value V17232_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_327 = {S44, 0, &V17232_327_57316, &P17232_324};	/*Property = TouchEvents*/
const Value V17232_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_328 = {S45, 0, &V17232_328_57319, &P17232_327};	/*Property = Track*/
const Value V17232_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_329 = {S46, 0, &V17232_329_57325, &P17232_328};	/*Property = Video*/
const Value V17232_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_330 = {S47, 0, &V17232_330_57328, &P17232_329};	/*Property = Viewport*/
const Value V17232_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_332 = {S48, 0, &V17232_332_57340, &P17232_330};	/*Property = WebWorkers*/
const Value V17232_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17232_333 = {S49, 0, &V17232_333_57343, &P17232_332};	/*Property = Xhr2*/
const Profile P17232 = {"17232", &P17232_333};	/*Profile ID - 17232*/
const Value V17240_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17240_279 = {S1, 0, &V17240_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17240_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17240_282 = {S3, 0, &V17240_282_57154, &P17240_279};	/*Property = BlobBuilder*/
const Value V17240_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17240_350 = {S4, 0, &V17240_350_57444, &P17240_282};	/*Property = Canvas*/
const Value V17240_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17240_283 = {S5, 0, &V17240_283_57159, &P17240_350};	/*Property = CssBackground*/
const Value V17240_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17240_284 = {S6, 0, &V17240_284_57163, &P17240_283};	/*Property = CssBorderImage*/
const Value V17240_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17240_285 = {S7, 0, &V17240_285_57166, &P17240_284};	/*Property = CssCanvas*/
const Value V17240_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17240_286 = {S8, 0, &V17240_286_57168, &P17240_285};	/*Property = CssColor*/
const Value V17240_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17240_237 = {S9, 0, &V17240_237_55786, &P17240_286};	/*Property = CssColumn*/
const Value V17240_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17240_287 = {S10, 0, &V17240_287_57172, &P17240_237};	/*Property = CssFlexbox*/
const Value V17240_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17240_288 = {S11, 0, &V17240_288_57175, &P17240_287};	/*Property = CssFont*/
const Value V17240_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17240_289 = {S12, 0, &V17240_289_57759, &P17240_288};	/*Property = CssImages*/
const Value V17240_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17240_291 = {S13, 0, &V17240_291_57181, &P17240_289};	/*Property = CssMediaQueries*/
const Value V17240_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17240_292 = {S14, 0, &V17240_292_57183, &P17240_291};	/*Property = CssMinMax*/
const Value V17240_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17240_293 = {S15, 0, &V17240_293_57187, &P17240_292};	/*Property = CssOverflow*/
const Value V17240_294_57190 = {S0, NULL};	/*Value = False*/
const Property P17240_294 = {S16, 0, &V17240_294_57190, &P17240_293};	/*Property = CssPosition*/
const Value V17240_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17240_295 = {S17, 0, &V17240_295_57192, &P17240_294};	/*Property = CssText*/
const Value V17240_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17240_297 = {S18, 0, &V17240_297_57195, &P17240_295};	/*Property = CssTransforms*/
const Value V17240_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17240_241 = {S19, 0, &V17240_241_55799, &P17240_297};	/*Property = CssTransitions*/
const Value V17240_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17240_298 = {S20, 0, &V17240_298_57206, &P17240_241};	/*Property = CssUI*/
const Value V17240_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17240_300 = {S21, 0, &V17240_300_57216, &P17240_298};	/*Property = DataSet*/
const Value V17240_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17240_301 = {S22, 0, &V17240_301_57218, &P17240_300};	/*Property = DataUrl*/
const Value V17240_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17240_302 = {S23, 0, &V17240_302_57222, &P17240_301};	/*Property = DeviceOrientation*/
const Value V17240_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17240_304 = {S24, 0, &V17240_304_57228, &P17240_302};	/*Property = FileReader*/
const Value V17240_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17240_305 = {S25, 0, &V17240_305_57231, &P17240_304};	/*Property = FileSaver*/
const Value V17240_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17240_306 = {S26, 0, &V17240_306_57237, &P17240_305};	/*Property = FileWriter*/
const Value V17240_307_57240 = {S0, NULL};	/*Value = False*/
const Property P17240_307 = {S27, 0, &V17240_307_57240, &P17240_306};	/*Property = FormData*/
const Value V17240_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17240_309 = {S28, 0, &V17240_309_57246, &P17240_307};	/*Property = Fullscreen*/
const Value V17240_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17240_310 = {S29, 0, &V17240_310_57248, &P17240_309};	/*Property = GeoLocation*/
const Value V17240_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17240_312 = {S30, 0, &V17240_312_57255, &P17240_310};	/*Property = History*/
const Value V17240_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17240_314 = {S31, 0, &V17240_314_57261, &P17240_312};	/*Property = Html5*/
const Value V17240_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17240_313 = {S32, 0, &V17240_313_57258, &P17240_314};	/*Property = Html-Media-Capture*/
const Value V17240_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17240_315 = {S33, 0, &V17240_315_57264, &P17240_313};	/*Property = Iframe*/
const Value V17240_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17240_316 = {S34, 0, &V17240_316_57267, &P17240_315};	/*Property = IndexedDB*/
const Value V17240_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17240_317 = {S35, 0, &V17240_317_57269, &P17240_316};	/*Property = Json*/
const Value V17240_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17240_15 = {S37, 0, &V17240_15_28, &P17240_317};	/*Property = LayoutEngine*/
const Value V17240_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17240_344 = {S38, 0, &V17240_344_57272, &P17240_15};	/*Property = Masking*/
const Value V17240_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17240_320 = {S39, 0, &V17240_320_57290, &P17240_344};	/*Property = PostMessage*/
const Value V17240_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17240_321 = {S40, 0, &V17240_321_57293, &P17240_320};	/*Property = Progress*/
const Value V17240_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17240_322 = {S41, 0, &V17240_322_57296, &P17240_321};	/*Property = Prompts*/
const Value V17240_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17240_323 = {S42, 0, &V17240_323_57299, &P17240_322};	/*Property = Selector*/
const Value V17240_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17240_324 = {S43, 0, &V17240_324_57306, &P17240_323};	/*Property = Svg*/
const Value V17240_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17240_327 = {S44, 0, &V17240_327_57315, &P17240_324};	/*Property = TouchEvents*/
const Value V17240_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17240_328 = {S45, 0, &V17240_328_57318, &P17240_327};	/*Property = Track*/
const Value V17240_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17240_329 = {S46, 0, &V17240_329_57323, &P17240_328};	/*Property = Video*/
const Value V17240_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17240_330 = {S47, 0, &V17240_330_57326, &P17240_329};	/*Property = Viewport*/
const Value V17240_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17240_332 = {S48, 0, &V17240_332_57339, &P17240_330};	/*Property = WebWorkers*/
const Value V17240_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17240_333 = {S49, 0, &V17240_333_57342, &P17240_332};	/*Property = Xhr2*/
const Profile P17240 = {"17240", &P17240_333};	/*Profile ID - 17240*/
const Value V17237_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_279 = {S1, 0, &V17237_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17237_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_282 = {S3, 0, &V17237_282_57155, &P17237_279};	/*Property = BlobBuilder*/
const Value V17237_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_350 = {S4, 0, &V17237_350_57446, &P17237_282};	/*Property = Canvas*/
const Value V17237_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_283 = {S5, 0, &V17237_283_57161, &P17237_350};	/*Property = CssBackground*/
const Value V17237_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_284 = {S6, 0, &V17237_284_57164, &P17237_283};	/*Property = CssBorderImage*/
const Value V17237_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_285 = {S7, 0, &V17237_285_57167, &P17237_284};	/*Property = CssCanvas*/
const Value V17237_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_286 = {S8, 0, &V17237_286_57170, &P17237_285};	/*Property = CssColor*/
const Value V17237_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_237 = {S9, 0, &V17237_237_55786, &P17237_286};	/*Property = CssColumn*/
const Value V17237_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_287 = {S10, 0, &V17237_287_57173, &P17237_237};	/*Property = CssFlexbox*/
const Value V17237_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_288 = {S11, 0, &V17237_288_57176, &P17237_287};	/*Property = CssFont*/
const Value V17237_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_289 = {S12, 0, &V17237_289_86899, &P17237_288};	/*Property = CssImages*/
const Value V17237_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_291 = {S13, 0, &V17237_291_57182, &P17237_289};	/*Property = CssMediaQueries*/
const Value V17237_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_292 = {S14, 0, &V17237_292_57185, &P17237_291};	/*Property = CssMinMax*/
const Value V17237_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_293 = {S15, 0, &V17237_293_57188, &P17237_292};	/*Property = CssOverflow*/
const Value V17237_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_294 = {S16, 0, &V17237_294_57191, &P17237_293};	/*Property = CssPosition*/
const Value V17237_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_295 = {S17, 0, &V17237_295_57194, &P17237_294};	/*Property = CssText*/
const Value V17237_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_297 = {S18, 0, &V17237_297_57197, &P17237_295};	/*Property = CssTransforms*/
const Value V17237_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_241 = {S19, 0, &V17237_241_55801, &P17237_297};	/*Property = CssTransitions*/
const Value V17237_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_298 = {S20, 0, &V17237_298_57208, &P17237_241};	/*Property = CssUI*/
const Value V17237_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_300 = {S21, 0, &V17237_300_57217, &P17237_298};	/*Property = DataSet*/
const Value V17237_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_301 = {S22, 0, &V17237_301_57220, &P17237_300};	/*Property = DataUrl*/
const Value V17237_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_302 = {S23, 0, &V17237_302_57223, &P17237_301};	/*Property = DeviceOrientation*/
const Value V17237_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_304 = {S24, 0, &V17237_304_57229, &P17237_302};	/*Property = FileReader*/
const Value V17237_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_305 = {S25, 0, &V17237_305_57232, &P17237_304};	/*Property = FileSaver*/
const Value V17237_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_306 = {S26, 0, &V17237_306_57238, &P17237_305};	/*Property = FileWriter*/
const Value V17237_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_307 = {S27, 0, &V17237_307_57241, &P17237_306};	/*Property = FormData*/
const Value V17237_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_309 = {S28, 0, &V17237_309_57247, &P17237_307};	/*Property = Fullscreen*/
const Value V17237_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_310 = {S29, 0, &V17237_310_57250, &P17237_309};	/*Property = GeoLocation*/
const Value V17237_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_312 = {S30, 0, &V17237_312_57256, &P17237_310};	/*Property = History*/
const Value V17237_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_314 = {S31, 0, &V17237_314_57262, &P17237_312};	/*Property = Html5*/
const Value V17237_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_313 = {S32, 0, &V17237_313_57259, &P17237_314};	/*Property = Html-Media-Capture*/
const Value V17237_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_315 = {S33, 0, &V17237_315_57265, &P17237_313};	/*Property = Iframe*/
const Value V17237_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_316 = {S34, 0, &V17237_316_57268, &P17237_315};	/*Property = IndexedDB*/
const Value V17237_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_317 = {S35, 0, &V17237_317_57271, &P17237_316};	/*Property = Json*/
const Value V17237_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17237_15 = {S37, 0, &V17237_15_28, &P17237_317};	/*Property = LayoutEngine*/
const Value V17237_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_344 = {S38, 0, &V17237_344_57274, &P17237_15};	/*Property = Masking*/
const Value V17237_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_320 = {S39, 0, &V17237_320_57292, &P17237_344};	/*Property = PostMessage*/
const Value V17237_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_321 = {S40, 0, &V17237_321_57295, &P17237_320};	/*Property = Progress*/
const Value V17237_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_322 = {S41, 0, &V17237_322_57298, &P17237_321};	/*Property = Prompts*/
const Value V17237_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_323 = {S42, 0, &V17237_323_57301, &P17237_322};	/*Property = Selector*/
const Value V17237_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_324 = {S43, 0, &V17237_324_57307, &P17237_323};	/*Property = Svg*/
const Value V17237_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_327 = {S44, 0, &V17237_327_57316, &P17237_324};	/*Property = TouchEvents*/
const Value V17237_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_328 = {S45, 0, &V17237_328_57319, &P17237_327};	/*Property = Track*/
const Value V17237_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_329 = {S46, 0, &V17237_329_57325, &P17237_328};	/*Property = Video*/
const Value V17237_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_330 = {S47, 0, &V17237_330_57328, &P17237_329};	/*Property = Viewport*/
const Value V17237_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_332 = {S48, 0, &V17237_332_57340, &P17237_330};	/*Property = WebWorkers*/
const Value V17237_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17237_333 = {S49, 0, &V17237_333_57343, &P17237_332};	/*Property = Xhr2*/
const Profile P17237 = {"17237", &P17237_333};	/*Profile ID - 17237*/
const Value V17242_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17242_279 = {S1, 0, &V17242_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17242_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17242_282 = {S3, 0, &V17242_282_57154, &P17242_279};	/*Property = BlobBuilder*/
const Value V17242_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17242_350 = {S4, 0, &V17242_350_57444, &P17242_282};	/*Property = Canvas*/
const Value V17242_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17242_283 = {S5, 0, &V17242_283_57159, &P17242_350};	/*Property = CssBackground*/
const Value V17242_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17242_284 = {S6, 0, &V17242_284_57163, &P17242_283};	/*Property = CssBorderImage*/
const Value V17242_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17242_285 = {S7, 0, &V17242_285_57166, &P17242_284};	/*Property = CssCanvas*/
const Value V17242_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17242_286 = {S8, 0, &V17242_286_57168, &P17242_285};	/*Property = CssColor*/
const Value V17242_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17242_237 = {S9, 0, &V17242_237_55786, &P17242_286};	/*Property = CssColumn*/
const Value V17242_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17242_287 = {S10, 0, &V17242_287_57172, &P17242_237};	/*Property = CssFlexbox*/
const Value V17242_288_57174 = {S2, NULL};	/*Value = True*/
const Property P17242_288 = {S11, 0, &V17242_288_57174, &P17242_287};	/*Property = CssFont*/
const Value V17242_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17242_289 = {S12, 0, &V17242_289_57759, &P17242_288};	/*Property = CssImages*/
const Value V17242_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17242_291 = {S13, 0, &V17242_291_57181, &P17242_289};	/*Property = CssMediaQueries*/
const Value V17242_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17242_292 = {S14, 0, &V17242_292_57183, &P17242_291};	/*Property = CssMinMax*/
const Value V17242_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17242_293 = {S15, 0, &V17242_293_57187, &P17242_292};	/*Property = CssOverflow*/
const Value V17242_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17242_294 = {S16, 0, &V17242_294_57189, &P17242_293};	/*Property = CssPosition*/
const Value V17242_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17242_295 = {S17, 0, &V17242_295_57192, &P17242_294};	/*Property = CssText*/
const Value V17242_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17242_297 = {S18, 0, &V17242_297_57195, &P17242_295};	/*Property = CssTransforms*/
const Value V17242_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17242_241 = {S19, 0, &V17242_241_55799, &P17242_297};	/*Property = CssTransitions*/
const Value V17242_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17242_298 = {S20, 0, &V17242_298_57206, &P17242_241};	/*Property = CssUI*/
const Value V17242_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17242_300 = {S21, 0, &V17242_300_57216, &P17242_298};	/*Property = DataSet*/
const Value V17242_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17242_301 = {S22, 0, &V17242_301_57218, &P17242_300};	/*Property = DataUrl*/
const Value V17242_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17242_302 = {S23, 0, &V17242_302_57222, &P17242_301};	/*Property = DeviceOrientation*/
const Value V17242_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17242_304 = {S24, 0, &V17242_304_57228, &P17242_302};	/*Property = FileReader*/
const Value V17242_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17242_305 = {S25, 0, &V17242_305_57231, &P17242_304};	/*Property = FileSaver*/
const Value V17242_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17242_306 = {S26, 0, &V17242_306_57237, &P17242_305};	/*Property = FileWriter*/
const Value V17242_307_57240 = {S0, NULL};	/*Value = False*/
const Property P17242_307 = {S27, 0, &V17242_307_57240, &P17242_306};	/*Property = FormData*/
const Value V17242_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17242_309 = {S28, 0, &V17242_309_57246, &P17242_307};	/*Property = Fullscreen*/
const Value V17242_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17242_310 = {S29, 0, &V17242_310_57248, &P17242_309};	/*Property = GeoLocation*/
const Value V17242_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17242_312 = {S30, 0, &V17242_312_57255, &P17242_310};	/*Property = History*/
const Value V17242_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17242_314 = {S31, 0, &V17242_314_57261, &P17242_312};	/*Property = Html5*/
const Value V17242_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17242_313 = {S32, 0, &V17242_313_57258, &P17242_314};	/*Property = Html-Media-Capture*/
const Value V17242_315_57263 = {S2, NULL};	/*Value = True*/
const Property P17242_315 = {S33, 0, &V17242_315_57263, &P17242_313};	/*Property = Iframe*/
const Value V17242_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17242_316 = {S34, 0, &V17242_316_57267, &P17242_315};	/*Property = IndexedDB*/
const Value V17242_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17242_317 = {S35, 0, &V17242_317_57269, &P17242_316};	/*Property = Json*/
const Value V17242_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17242_15 = {S37, 0, &V17242_15_28, &P17242_317};	/*Property = LayoutEngine*/
const Value V17242_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17242_344 = {S38, 0, &V17242_344_57272, &P17242_15};	/*Property = Masking*/
const Value V17242_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17242_320 = {S39, 0, &V17242_320_57290, &P17242_344};	/*Property = PostMessage*/
const Value V17242_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17242_321 = {S40, 0, &V17242_321_57293, &P17242_320};	/*Property = Progress*/
const Value V17242_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17242_322 = {S41, 0, &V17242_322_57296, &P17242_321};	/*Property = Prompts*/
const Value V17242_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17242_323 = {S42, 0, &V17242_323_57299, &P17242_322};	/*Property = Selector*/
const Value V17242_324_57305 = {S2, NULL};	/*Value = True*/
const Property P17242_324 = {S43, 0, &V17242_324_57305, &P17242_323};	/*Property = Svg*/
const Value V17242_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17242_327 = {S44, 0, &V17242_327_57315, &P17242_324};	/*Property = TouchEvents*/
const Value V17242_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17242_328 = {S45, 0, &V17242_328_57318, &P17242_327};	/*Property = Track*/
const Value V17242_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17242_329 = {S46, 0, &V17242_329_57323, &P17242_328};	/*Property = Video*/
const Value V17242_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17242_330 = {S47, 0, &V17242_330_57326, &P17242_329};	/*Property = Viewport*/
const Value V17242_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17242_332 = {S48, 0, &V17242_332_57339, &P17242_330};	/*Property = WebWorkers*/
const Value V17242_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17242_333 = {S49, 0, &V17242_333_57342, &P17242_332};	/*Property = Xhr2*/
const Profile P17242 = {"17242", &P17242_333};	/*Profile ID - 17242*/
const Value V17249_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17249_279 = {S1, 0, &V17249_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17249_282_57153 = {S2, NULL};	/*Value = True*/
const Property P17249_282 = {S3, 0, &V17249_282_57153, &P17249_279};	/*Property = BlobBuilder*/
const Value V17249_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17249_350 = {S4, 0, &V17249_350_57444, &P17249_282};	/*Property = Canvas*/
const Value V17249_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17249_283 = {S5, 0, &V17249_283_57159, &P17249_350};	/*Property = CssBackground*/
const Value V17249_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17249_284 = {S6, 0, &V17249_284_57163, &P17249_283};	/*Property = CssBorderImage*/
const Value V17249_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17249_285 = {S7, 0, &V17249_285_57166, &P17249_284};	/*Property = CssCanvas*/
const Value V17249_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17249_286 = {S8, 0, &V17249_286_57168, &P17249_285};	/*Property = CssColor*/
const Value V17249_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17249_237 = {S9, 0, &V17249_237_55784, &P17249_286};	/*Property = CssColumn*/
const Value V17249_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17249_287 = {S10, 0, &V17249_287_57172, &P17249_237};	/*Property = CssFlexbox*/
const Value V17249_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17249_288 = {S11, 0, &V17249_288_57175, &P17249_287};	/*Property = CssFont*/
const Value V17249_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17249_289 = {S12, 0, &V17249_289_57759, &P17249_288};	/*Property = CssImages*/
const Value V17249_291_57180 = {S2, NULL};	/*Value = True*/
const Property P17249_291 = {S13, 0, &V17249_291_57180, &P17249_289};	/*Property = CssMediaQueries*/
const Value V17249_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17249_292 = {S14, 0, &V17249_292_57183, &P17249_291};	/*Property = CssMinMax*/
const Value V17249_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17249_293 = {S15, 0, &V17249_293_57187, &P17249_292};	/*Property = CssOverflow*/
const Value V17249_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17249_294 = {S16, 0, &V17249_294_57189, &P17249_293};	/*Property = CssPosition*/
const Value V17249_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17249_295 = {S17, 0, &V17249_295_57192, &P17249_294};	/*Property = CssText*/
const Value V17249_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17249_297 = {S18, 0, &V17249_297_57195, &P17249_295};	/*Property = CssTransforms*/
const Value V17249_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17249_241 = {S19, 0, &V17249_241_55799, &P17249_297};	/*Property = CssTransitions*/
const Value V17249_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17249_298 = {S20, 0, &V17249_298_57206, &P17249_241};	/*Property = CssUI*/
const Value V17249_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17249_300 = {S21, 0, &V17249_300_57216, &P17249_298};	/*Property = DataSet*/
const Value V17249_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17249_301 = {S22, 0, &V17249_301_57218, &P17249_300};	/*Property = DataUrl*/
const Value V17249_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17249_302 = {S23, 0, &V17249_302_57222, &P17249_301};	/*Property = DeviceOrientation*/
const Value V17249_304_57227 = {S2, NULL};	/*Value = True*/
const Property P17249_304 = {S24, 0, &V17249_304_57227, &P17249_302};	/*Property = FileReader*/
const Value V17249_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17249_305 = {S25, 0, &V17249_305_57231, &P17249_304};	/*Property = FileSaver*/
const Value V17249_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17249_306 = {S26, 0, &V17249_306_57237, &P17249_305};	/*Property = FileWriter*/
const Value V17249_307_57239 = {S2, NULL};	/*Value = True*/
const Property P17249_307 = {S27, 0, &V17249_307_57239, &P17249_306};	/*Property = FormData*/
const Value V17249_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17249_309 = {S28, 0, &V17249_309_57246, &P17249_307};	/*Property = Fullscreen*/
const Value V17249_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17249_310 = {S29, 0, &V17249_310_57248, &P17249_309};	/*Property = GeoLocation*/
const Value V17249_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17249_312 = {S30, 0, &V17249_312_57255, &P17249_310};	/*Property = History*/
const Value V17249_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17249_314 = {S31, 0, &V17249_314_57261, &P17249_312};	/*Property = Html5*/
const Value V17249_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17249_313 = {S32, 0, &V17249_313_57258, &P17249_314};	/*Property = Html-Media-Capture*/
const Value V17249_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17249_315 = {S33, 0, &V17249_315_57264, &P17249_313};	/*Property = Iframe*/
const Value V17249_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17249_316 = {S34, 0, &V17249_316_57267, &P17249_315};	/*Property = IndexedDB*/
const Value V17249_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17249_317 = {S35, 0, &V17249_317_57269, &P17249_316};	/*Property = Json*/
const Value V17249_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17249_15 = {S37, 0, &V17249_15_28, &P17249_317};	/*Property = LayoutEngine*/
const Value V17249_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17249_344 = {S38, 0, &V17249_344_57272, &P17249_15};	/*Property = Masking*/
const Value V17249_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17249_320 = {S39, 0, &V17249_320_57290, &P17249_344};	/*Property = PostMessage*/
const Value V17249_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17249_321 = {S40, 0, &V17249_321_57293, &P17249_320};	/*Property = Progress*/
const Value V17249_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17249_322 = {S41, 0, &V17249_322_57296, &P17249_321};	/*Property = Prompts*/
const Value V17249_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17249_323 = {S42, 0, &V17249_323_57299, &P17249_322};	/*Property = Selector*/
const Value V17249_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17249_324 = {S43, 0, &V17249_324_57306, &P17249_323};	/*Property = Svg*/
const Value V17249_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17249_327 = {S44, 0, &V17249_327_57315, &P17249_324};	/*Property = TouchEvents*/
const Value V17249_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17249_328 = {S45, 0, &V17249_328_57318, &P17249_327};	/*Property = Track*/
const Value V17249_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17249_329 = {S46, 0, &V17249_329_57323, &P17249_328};	/*Property = Video*/
const Value V17249_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17249_330 = {S47, 0, &V17249_330_57326, &P17249_329};	/*Property = Viewport*/
const Value V17249_332_57338 = {S2, NULL};	/*Value = True*/
const Property P17249_332 = {S48, 0, &V17249_332_57338, &P17249_330};	/*Property = WebWorkers*/
const Value V17249_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17249_333 = {S49, 0, &V17249_333_57342, &P17249_332};	/*Property = Xhr2*/
const Profile P17249 = {"17249", &P17249_333};	/*Profile ID - 17249*/
const Profile P17291 = {"17291", NULL};	/*Profile ID - 17291*/
const Value V17252_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17252_279 = {S1, 0, &V17252_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17252_282_57153 = {S2, NULL};	/*Value = True*/
const Property P17252_282 = {S3, 0, &V17252_282_57153, &P17252_279};	/*Property = BlobBuilder*/
const Value V17252_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17252_350 = {S4, 0, &V17252_350_57444, &P17252_282};	/*Property = Canvas*/
const Value V17252_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17252_283 = {S5, 0, &V17252_283_57159, &P17252_350};	/*Property = CssBackground*/
const Value V17252_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17252_284 = {S6, 0, &V17252_284_57163, &P17252_283};	/*Property = CssBorderImage*/
const Value V17252_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17252_285 = {S7, 0, &V17252_285_57166, &P17252_284};	/*Property = CssCanvas*/
const Value V17252_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17252_286 = {S8, 0, &V17252_286_57168, &P17252_285};	/*Property = CssColor*/
const Value V17252_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17252_237 = {S9, 0, &V17252_237_55784, &P17252_286};	/*Property = CssColumn*/
const Value V17252_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17252_287 = {S10, 0, &V17252_287_57172, &P17252_237};	/*Property = CssFlexbox*/
const Value V17252_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17252_288 = {S11, 0, &V17252_288_57175, &P17252_287};	/*Property = CssFont*/
const Value V17252_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17252_289 = {S12, 0, &V17252_289_57759, &P17252_288};	/*Property = CssImages*/
const Value V17252_291_57180 = {S2, NULL};	/*Value = True*/
const Property P17252_291 = {S13, 0, &V17252_291_57180, &P17252_289};	/*Property = CssMediaQueries*/
const Value V17252_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17252_292 = {S14, 0, &V17252_292_57183, &P17252_291};	/*Property = CssMinMax*/
const Value V17252_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17252_293 = {S15, 0, &V17252_293_57187, &P17252_292};	/*Property = CssOverflow*/
const Value V17252_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17252_294 = {S16, 0, &V17252_294_57189, &P17252_293};	/*Property = CssPosition*/
const Value V17252_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17252_295 = {S17, 0, &V17252_295_57192, &P17252_294};	/*Property = CssText*/
const Value V17252_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17252_297 = {S18, 0, &V17252_297_57195, &P17252_295};	/*Property = CssTransforms*/
const Value V17252_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17252_241 = {S19, 0, &V17252_241_55799, &P17252_297};	/*Property = CssTransitions*/
const Value V17252_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17252_298 = {S20, 0, &V17252_298_57206, &P17252_241};	/*Property = CssUI*/
const Value V17252_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17252_300 = {S21, 0, &V17252_300_57216, &P17252_298};	/*Property = DataSet*/
const Value V17252_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17252_301 = {S22, 0, &V17252_301_57218, &P17252_300};	/*Property = DataUrl*/
const Value V17252_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17252_302 = {S23, 0, &V17252_302_57222, &P17252_301};	/*Property = DeviceOrientation*/
const Value V17252_304_57227 = {S2, NULL};	/*Value = True*/
const Property P17252_304 = {S24, 0, &V17252_304_57227, &P17252_302};	/*Property = FileReader*/
const Value V17252_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17252_305 = {S25, 0, &V17252_305_57231, &P17252_304};	/*Property = FileSaver*/
const Value V17252_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17252_306 = {S26, 0, &V17252_306_57237, &P17252_305};	/*Property = FileWriter*/
const Value V17252_307_57239 = {S2, NULL};	/*Value = True*/
const Property P17252_307 = {S27, 0, &V17252_307_57239, &P17252_306};	/*Property = FormData*/
const Value V17252_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17252_309 = {S28, 0, &V17252_309_57246, &P17252_307};	/*Property = Fullscreen*/
const Value V17252_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17252_310 = {S29, 0, &V17252_310_57248, &P17252_309};	/*Property = GeoLocation*/
const Value V17252_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17252_312 = {S30, 0, &V17252_312_57255, &P17252_310};	/*Property = History*/
const Value V17252_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17252_314 = {S31, 0, &V17252_314_57261, &P17252_312};	/*Property = Html5*/
const Value V17252_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17252_313 = {S32, 0, &V17252_313_57258, &P17252_314};	/*Property = Html-Media-Capture*/
const Value V17252_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17252_315 = {S33, 0, &V17252_315_57264, &P17252_313};	/*Property = Iframe*/
const Value V17252_316_57266 = {S2, NULL};	/*Value = True*/
const Property P17252_316 = {S34, 0, &V17252_316_57266, &P17252_315};	/*Property = IndexedDB*/
const Value V17252_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17252_317 = {S35, 0, &V17252_317_57269, &P17252_316};	/*Property = Json*/
const Value V17252_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17252_15 = {S37, 0, &V17252_15_28, &P17252_317};	/*Property = LayoutEngine*/
const Value V17252_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17252_344 = {S38, 0, &V17252_344_57272, &P17252_15};	/*Property = Masking*/
const Value V17252_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17252_320 = {S39, 0, &V17252_320_57290, &P17252_344};	/*Property = PostMessage*/
const Value V17252_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17252_321 = {S40, 0, &V17252_321_57293, &P17252_320};	/*Property = Progress*/
const Value V17252_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17252_322 = {S41, 0, &V17252_322_57296, &P17252_321};	/*Property = Prompts*/
const Value V17252_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17252_323 = {S42, 0, &V17252_323_57299, &P17252_322};	/*Property = Selector*/
const Value V17252_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17252_324 = {S43, 0, &V17252_324_57306, &P17252_323};	/*Property = Svg*/
const Value V17252_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17252_327 = {S44, 0, &V17252_327_57315, &P17252_324};	/*Property = TouchEvents*/
const Value V17252_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17252_328 = {S45, 0, &V17252_328_57318, &P17252_327};	/*Property = Track*/
const Value V17252_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17252_329 = {S46, 0, &V17252_329_57323, &P17252_328};	/*Property = Video*/
const Value V17252_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17252_330 = {S47, 0, &V17252_330_57326, &P17252_329};	/*Property = Viewport*/
const Value V17252_332_57338 = {S2, NULL};	/*Value = True*/
const Property P17252_332 = {S48, 0, &V17252_332_57338, &P17252_330};	/*Property = WebWorkers*/
const Value V17252_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17252_333 = {S49, 0, &V17252_333_57342, &P17252_332};	/*Property = Xhr2*/
const Profile P17252 = {"17252", &P17252_333};	/*Profile ID - 17252*/
const Value V17253_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17253_279 = {S1, 0, &V17253_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17253_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17253_282 = {S3, 0, &V17253_282_57154, &P17253_279};	/*Property = BlobBuilder*/
const Value V17253_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17253_350 = {S4, 0, &V17253_350_57444, &P17253_282};	/*Property = Canvas*/
const Value V17253_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17253_283 = {S5, 0, &V17253_283_57159, &P17253_350};	/*Property = CssBackground*/
const Value V17253_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17253_284 = {S6, 0, &V17253_284_57163, &P17253_283};	/*Property = CssBorderImage*/
const Value V17253_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17253_285 = {S7, 0, &V17253_285_57166, &P17253_284};	/*Property = CssCanvas*/
const Value V17253_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17253_286 = {S8, 0, &V17253_286_57168, &P17253_285};	/*Property = CssColor*/
const Value V17253_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17253_237 = {S9, 0, &V17253_237_55784, &P17253_286};	/*Property = CssColumn*/
const Value V17253_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17253_287 = {S10, 0, &V17253_287_57172, &P17253_237};	/*Property = CssFlexbox*/
const Value V17253_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17253_288 = {S11, 0, &V17253_288_57175, &P17253_287};	/*Property = CssFont*/
const Value V17253_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17253_289 = {S12, 0, &V17253_289_57759, &P17253_288};	/*Property = CssImages*/
const Value V17253_291_57180 = {S2, NULL};	/*Value = True*/
const Property P17253_291 = {S13, 0, &V17253_291_57180, &P17253_289};	/*Property = CssMediaQueries*/
const Value V17253_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17253_292 = {S14, 0, &V17253_292_57183, &P17253_291};	/*Property = CssMinMax*/
const Value V17253_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17253_293 = {S15, 0, &V17253_293_57187, &P17253_292};	/*Property = CssOverflow*/
const Value V17253_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17253_294 = {S16, 0, &V17253_294_57189, &P17253_293};	/*Property = CssPosition*/
const Value V17253_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17253_295 = {S17, 0, &V17253_295_57192, &P17253_294};	/*Property = CssText*/
const Value V17253_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17253_297 = {S18, 0, &V17253_297_57195, &P17253_295};	/*Property = CssTransforms*/
const Value V17253_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17253_241 = {S19, 0, &V17253_241_55799, &P17253_297};	/*Property = CssTransitions*/
const Value V17253_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17253_298 = {S20, 0, &V17253_298_57206, &P17253_241};	/*Property = CssUI*/
const Value V17253_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17253_300 = {S21, 0, &V17253_300_57216, &P17253_298};	/*Property = DataSet*/
const Value V17253_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17253_301 = {S22, 0, &V17253_301_57218, &P17253_300};	/*Property = DataUrl*/
const Value V17253_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17253_302 = {S23, 0, &V17253_302_57222, &P17253_301};	/*Property = DeviceOrientation*/
const Value V17253_304_57227 = {S2, NULL};	/*Value = True*/
const Property P17253_304 = {S24, 0, &V17253_304_57227, &P17253_302};	/*Property = FileReader*/
const Value V17253_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17253_305 = {S25, 0, &V17253_305_57231, &P17253_304};	/*Property = FileSaver*/
const Value V17253_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17253_306 = {S26, 0, &V17253_306_57237, &P17253_305};	/*Property = FileWriter*/
const Value V17253_307_57239 = {S2, NULL};	/*Value = True*/
const Property P17253_307 = {S27, 0, &V17253_307_57239, &P17253_306};	/*Property = FormData*/
const Value V17253_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17253_309 = {S28, 0, &V17253_309_57246, &P17253_307};	/*Property = Fullscreen*/
const Value V17253_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17253_310 = {S29, 0, &V17253_310_57248, &P17253_309};	/*Property = GeoLocation*/
const Value V17253_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17253_312 = {S30, 0, &V17253_312_57255, &P17253_310};	/*Property = History*/
const Value V17253_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17253_314 = {S31, 0, &V17253_314_57261, &P17253_312};	/*Property = Html5*/
const Value V17253_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17253_313 = {S32, 0, &V17253_313_57258, &P17253_314};	/*Property = Html-Media-Capture*/
const Value V17253_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17253_315 = {S33, 0, &V17253_315_57264, &P17253_313};	/*Property = Iframe*/
const Value V17253_316_57266 = {S2, NULL};	/*Value = True*/
const Property P17253_316 = {S34, 0, &V17253_316_57266, &P17253_315};	/*Property = IndexedDB*/
const Value V17253_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17253_317 = {S35, 0, &V17253_317_57269, &P17253_316};	/*Property = Json*/
const Value V17253_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17253_15 = {S37, 0, &V17253_15_28, &P17253_317};	/*Property = LayoutEngine*/
const Value V17253_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17253_344 = {S38, 0, &V17253_344_57272, &P17253_15};	/*Property = Masking*/
const Value V17253_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17253_320 = {S39, 0, &V17253_320_57290, &P17253_344};	/*Property = PostMessage*/
const Value V17253_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17253_321 = {S40, 0, &V17253_321_57293, &P17253_320};	/*Property = Progress*/
const Value V17253_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17253_322 = {S41, 0, &V17253_322_57296, &P17253_321};	/*Property = Prompts*/
const Value V17253_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17253_323 = {S42, 0, &V17253_323_57299, &P17253_322};	/*Property = Selector*/
const Value V17253_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17253_324 = {S43, 0, &V17253_324_57306, &P17253_323};	/*Property = Svg*/
const Value V17253_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17253_327 = {S44, 0, &V17253_327_57315, &P17253_324};	/*Property = TouchEvents*/
const Value V17253_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17253_328 = {S45, 0, &V17253_328_57318, &P17253_327};	/*Property = Track*/
const Value V17253_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17253_329 = {S46, 0, &V17253_329_57323, &P17253_328};	/*Property = Video*/
const Value V17253_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17253_330 = {S47, 0, &V17253_330_57326, &P17253_329};	/*Property = Viewport*/
const Value V17253_332_57338 = {S2, NULL};	/*Value = True*/
const Property P17253_332 = {S48, 0, &V17253_332_57338, &P17253_330};	/*Property = WebWorkers*/
const Value V17253_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17253_333 = {S49, 0, &V17253_333_57342, &P17253_332};	/*Property = Xhr2*/
const Profile P17253 = {"17253", &P17253_333};	/*Profile ID - 17253*/
const Value V17254_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17254_279 = {S1, 0, &V17254_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17254_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17254_282 = {S3, 0, &V17254_282_57154, &P17254_279};	/*Property = BlobBuilder*/
const Value V17254_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17254_350 = {S4, 0, &V17254_350_57444, &P17254_282};	/*Property = Canvas*/
const Value V17254_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17254_283 = {S5, 0, &V17254_283_57159, &P17254_350};	/*Property = CssBackground*/
const Value V17254_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17254_284 = {S6, 0, &V17254_284_57163, &P17254_283};	/*Property = CssBorderImage*/
const Value V17254_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17254_285 = {S7, 0, &V17254_285_57166, &P17254_284};	/*Property = CssCanvas*/
const Value V17254_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17254_286 = {S8, 0, &V17254_286_57168, &P17254_285};	/*Property = CssColor*/
const Value V17254_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17254_237 = {S9, 0, &V17254_237_55784, &P17254_286};	/*Property = CssColumn*/
const Value V17254_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17254_287 = {S10, 0, &V17254_287_57172, &P17254_237};	/*Property = CssFlexbox*/
const Value V17254_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17254_288 = {S11, 0, &V17254_288_57175, &P17254_287};	/*Property = CssFont*/
const Value V17254_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17254_289 = {S12, 0, &V17254_289_57759, &P17254_288};	/*Property = CssImages*/
const Value V17254_291_57180 = {S2, NULL};	/*Value = True*/
const Property P17254_291 = {S13, 0, &V17254_291_57180, &P17254_289};	/*Property = CssMediaQueries*/
const Value V17254_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17254_292 = {S14, 0, &V17254_292_57183, &P17254_291};	/*Property = CssMinMax*/
const Value V17254_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17254_293 = {S15, 0, &V17254_293_57187, &P17254_292};	/*Property = CssOverflow*/
const Value V17254_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17254_294 = {S16, 0, &V17254_294_57189, &P17254_293};	/*Property = CssPosition*/
const Value V17254_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17254_295 = {S17, 0, &V17254_295_57192, &P17254_294};	/*Property = CssText*/
const Value V17254_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17254_297 = {S18, 0, &V17254_297_57195, &P17254_295};	/*Property = CssTransforms*/
const Value V17254_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17254_241 = {S19, 0, &V17254_241_55799, &P17254_297};	/*Property = CssTransitions*/
const Value V17254_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17254_298 = {S20, 0, &V17254_298_57206, &P17254_241};	/*Property = CssUI*/
const Value V17254_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17254_300 = {S21, 0, &V17254_300_57216, &P17254_298};	/*Property = DataSet*/
const Value V17254_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17254_301 = {S22, 0, &V17254_301_57218, &P17254_300};	/*Property = DataUrl*/
const Value V17254_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17254_302 = {S23, 0, &V17254_302_57222, &P17254_301};	/*Property = DeviceOrientation*/
const Value V17254_304_57227 = {S2, NULL};	/*Value = True*/
const Property P17254_304 = {S24, 0, &V17254_304_57227, &P17254_302};	/*Property = FileReader*/
const Value V17254_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17254_305 = {S25, 0, &V17254_305_57231, &P17254_304};	/*Property = FileSaver*/
const Value V17254_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17254_306 = {S26, 0, &V17254_306_57237, &P17254_305};	/*Property = FileWriter*/
const Value V17254_307_57239 = {S2, NULL};	/*Value = True*/
const Property P17254_307 = {S27, 0, &V17254_307_57239, &P17254_306};	/*Property = FormData*/
const Value V17254_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17254_309 = {S28, 0, &V17254_309_57246, &P17254_307};	/*Property = Fullscreen*/
const Value V17254_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17254_310 = {S29, 0, &V17254_310_57248, &P17254_309};	/*Property = GeoLocation*/
const Value V17254_312_57254 = {S2, NULL};	/*Value = True*/
const Property P17254_312 = {S30, 0, &V17254_312_57254, &P17254_310};	/*Property = History*/
const Value V17254_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17254_314 = {S31, 0, &V17254_314_57261, &P17254_312};	/*Property = Html5*/
const Value V17254_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17254_313 = {S32, 0, &V17254_313_57258, &P17254_314};	/*Property = Html-Media-Capture*/
const Value V17254_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17254_315 = {S33, 0, &V17254_315_57264, &P17254_313};	/*Property = Iframe*/
const Value V17254_316_57266 = {S2, NULL};	/*Value = True*/
const Property P17254_316 = {S34, 0, &V17254_316_57266, &P17254_315};	/*Property = IndexedDB*/
const Value V17254_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17254_317 = {S35, 0, &V17254_317_57269, &P17254_316};	/*Property = Json*/
const Value V17254_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17254_15 = {S37, 0, &V17254_15_28, &P17254_317};	/*Property = LayoutEngine*/
const Value V17254_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17254_344 = {S38, 0, &V17254_344_57272, &P17254_15};	/*Property = Masking*/
const Value V17254_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17254_320 = {S39, 0, &V17254_320_57290, &P17254_344};	/*Property = PostMessage*/
const Value V17254_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17254_321 = {S40, 0, &V17254_321_57293, &P17254_320};	/*Property = Progress*/
const Value V17254_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17254_322 = {S41, 0, &V17254_322_57296, &P17254_321};	/*Property = Prompts*/
const Value V17254_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17254_323 = {S42, 0, &V17254_323_57299, &P17254_322};	/*Property = Selector*/
const Value V17254_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17254_324 = {S43, 0, &V17254_324_57306, &P17254_323};	/*Property = Svg*/
const Value V17254_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17254_327 = {S44, 0, &V17254_327_57315, &P17254_324};	/*Property = TouchEvents*/
const Value V17254_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17254_328 = {S45, 0, &V17254_328_57318, &P17254_327};	/*Property = Track*/
const Value V17254_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17254_329 = {S46, 0, &V17254_329_57323, &P17254_328};	/*Property = Video*/
const Value V17254_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17254_330 = {S47, 0, &V17254_330_57326, &P17254_329};	/*Property = Viewport*/
const Value V17254_332_57338 = {S2, NULL};	/*Value = True*/
const Property P17254_332 = {S48, 0, &V17254_332_57338, &P17254_330};	/*Property = WebWorkers*/
const Value V17254_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17254_333 = {S49, 0, &V17254_333_57342, &P17254_332};	/*Property = Xhr2*/
const Profile P17254 = {"17254", &P17254_333};	/*Profile ID - 17254*/
const Value V17314_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_279 = {S1, 0, &V17314_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17314_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_282 = {S3, 0, &V17314_282_57155, &P17314_279};	/*Property = BlobBuilder*/
const Value V17314_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_350 = {S4, 0, &V17314_350_57446, &P17314_282};	/*Property = Canvas*/
const Value V17314_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_283 = {S5, 0, &V17314_283_57161, &P17314_350};	/*Property = CssBackground*/
const Value V17314_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_284 = {S6, 0, &V17314_284_57164, &P17314_283};	/*Property = CssBorderImage*/
const Value V17314_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_285 = {S7, 0, &V17314_285_57167, &P17314_284};	/*Property = CssCanvas*/
const Value V17314_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_286 = {S8, 0, &V17314_286_57170, &P17314_285};	/*Property = CssColor*/
const Value V17314_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_237 = {S9, 0, &V17314_237_55786, &P17314_286};	/*Property = CssColumn*/
const Value V17314_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_287 = {S10, 0, &V17314_287_57173, &P17314_237};	/*Property = CssFlexbox*/
const Value V17314_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_288 = {S11, 0, &V17314_288_57176, &P17314_287};	/*Property = CssFont*/
const Value V17314_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_289 = {S12, 0, &V17314_289_86899, &P17314_288};	/*Property = CssImages*/
const Value V17314_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_291 = {S13, 0, &V17314_291_57182, &P17314_289};	/*Property = CssMediaQueries*/
const Value V17314_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_292 = {S14, 0, &V17314_292_57185, &P17314_291};	/*Property = CssMinMax*/
const Value V17314_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_293 = {S15, 0, &V17314_293_57188, &P17314_292};	/*Property = CssOverflow*/
const Value V17314_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_294 = {S16, 0, &V17314_294_57191, &P17314_293};	/*Property = CssPosition*/
const Value V17314_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_295 = {S17, 0, &V17314_295_57194, &P17314_294};	/*Property = CssText*/
const Value V17314_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_297 = {S18, 0, &V17314_297_57197, &P17314_295};	/*Property = CssTransforms*/
const Value V17314_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_241 = {S19, 0, &V17314_241_55801, &P17314_297};	/*Property = CssTransitions*/
const Value V17314_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_298 = {S20, 0, &V17314_298_57208, &P17314_241};	/*Property = CssUI*/
const Value V17314_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_300 = {S21, 0, &V17314_300_57217, &P17314_298};	/*Property = DataSet*/
const Value V17314_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_301 = {S22, 0, &V17314_301_57220, &P17314_300};	/*Property = DataUrl*/
const Value V17314_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_302 = {S23, 0, &V17314_302_57223, &P17314_301};	/*Property = DeviceOrientation*/
const Value V17314_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_304 = {S24, 0, &V17314_304_57229, &P17314_302};	/*Property = FileReader*/
const Value V17314_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_305 = {S25, 0, &V17314_305_57232, &P17314_304};	/*Property = FileSaver*/
const Value V17314_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_306 = {S26, 0, &V17314_306_57238, &P17314_305};	/*Property = FileWriter*/
const Value V17314_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_307 = {S27, 0, &V17314_307_57241, &P17314_306};	/*Property = FormData*/
const Value V17314_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_309 = {S28, 0, &V17314_309_57247, &P17314_307};	/*Property = Fullscreen*/
const Value V17314_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_310 = {S29, 0, &V17314_310_57250, &P17314_309};	/*Property = GeoLocation*/
const Value V17314_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_312 = {S30, 0, &V17314_312_57256, &P17314_310};	/*Property = History*/
const Value V17314_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_314 = {S31, 0, &V17314_314_57262, &P17314_312};	/*Property = Html5*/
const Value V17314_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_313 = {S32, 0, &V17314_313_57259, &P17314_314};	/*Property = Html-Media-Capture*/
const Value V17314_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_315 = {S33, 0, &V17314_315_57265, &P17314_313};	/*Property = Iframe*/
const Value V17314_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_316 = {S34, 0, &V17314_316_57268, &P17314_315};	/*Property = IndexedDB*/
const Value V17314_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_317 = {S35, 0, &V17314_317_57271, &P17314_316};	/*Property = Json*/
const Value V17314_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17314_15 = {S37, 0, &V17314_15_28, &P17314_317};	/*Property = LayoutEngine*/
const Value V17314_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_344 = {S38, 0, &V17314_344_57274, &P17314_15};	/*Property = Masking*/
const Value V17314_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_320 = {S39, 0, &V17314_320_57292, &P17314_344};	/*Property = PostMessage*/
const Value V17314_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_321 = {S40, 0, &V17314_321_57295, &P17314_320};	/*Property = Progress*/
const Value V17314_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_322 = {S41, 0, &V17314_322_57298, &P17314_321};	/*Property = Prompts*/
const Value V17314_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_323 = {S42, 0, &V17314_323_57301, &P17314_322};	/*Property = Selector*/
const Value V17314_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_324 = {S43, 0, &V17314_324_57307, &P17314_323};	/*Property = Svg*/
const Value V17314_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_327 = {S44, 0, &V17314_327_57316, &P17314_324};	/*Property = TouchEvents*/
const Value V17314_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_328 = {S45, 0, &V17314_328_57319, &P17314_327};	/*Property = Track*/
const Value V17314_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_329 = {S46, 0, &V17314_329_57325, &P17314_328};	/*Property = Video*/
const Value V17314_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_330 = {S47, 0, &V17314_330_57328, &P17314_329};	/*Property = Viewport*/
const Value V17314_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_332 = {S48, 0, &V17314_332_57340, &P17314_330};	/*Property = WebWorkers*/
const Value V17314_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17314_333 = {S49, 0, &V17314_333_57343, &P17314_332};	/*Property = Xhr2*/
const Profile P17314 = {"17314", &P17314_333};	/*Profile ID - 17314*/
const Value V17673_1_1 = {S2, NULL};	/*Value = True*/
const Property P17673_1 = {S50, 0, &V17673_1_1, NULL};	/*Property = IsMobile*/
const Value V17673_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P17673_5 = {S52, 0, &V17673_5_1275, &P17673_1};	/*Property = ScreenPixelsHeight*/
const Value V17673_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17673_4 = {S54, 0, &V17673_4_1229, &P17673_5};	/*Property = ScreenPixelsWidth*/
const Profile P17673 = {"17673", &P17673_4};	/*Profile ID - 17673*/
const Value V3955_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_279 = {S1, 0, &V3955_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V3955_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_282 = {S3, 0, &V3955_282_57155, &P3955_279};	/*Property = BlobBuilder*/
const Value V3955_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_350 = {S4, 0, &V3955_350_57446, &P3955_282};	/*Property = Canvas*/
const Value V3955_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_283 = {S5, 0, &V3955_283_57161, &P3955_350};	/*Property = CssBackground*/
const Value V3955_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_284 = {S6, 0, &V3955_284_57164, &P3955_283};	/*Property = CssBorderImage*/
const Value V3955_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_285 = {S7, 0, &V3955_285_57167, &P3955_284};	/*Property = CssCanvas*/
const Value V3955_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_286 = {S8, 0, &V3955_286_57170, &P3955_285};	/*Property = CssColor*/
const Value V3955_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_237 = {S9, 0, &V3955_237_55786, &P3955_286};	/*Property = CssColumn*/
const Value V3955_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_287 = {S10, 0, &V3955_287_57173, &P3955_237};	/*Property = CssFlexbox*/
const Value V3955_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_288 = {S11, 0, &V3955_288_57176, &P3955_287};	/*Property = CssFont*/
const Value V3955_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_289 = {S12, 0, &V3955_289_86899, &P3955_288};	/*Property = CssImages*/
const Value V3955_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_291 = {S13, 0, &V3955_291_57182, &P3955_289};	/*Property = CssMediaQueries*/
const Value V3955_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_292 = {S14, 0, &V3955_292_57185, &P3955_291};	/*Property = CssMinMax*/
const Value V3955_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_293 = {S15, 0, &V3955_293_57188, &P3955_292};	/*Property = CssOverflow*/
const Value V3955_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_294 = {S16, 0, &V3955_294_57191, &P3955_293};	/*Property = CssPosition*/
const Value V3955_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_295 = {S17, 0, &V3955_295_57194, &P3955_294};	/*Property = CssText*/
const Value V3955_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_297 = {S18, 0, &V3955_297_57197, &P3955_295};	/*Property = CssTransforms*/
const Value V3955_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_241 = {S19, 0, &V3955_241_55801, &P3955_297};	/*Property = CssTransitions*/
const Value V3955_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_298 = {S20, 0, &V3955_298_57208, &P3955_241};	/*Property = CssUI*/
const Value V3955_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_300 = {S21, 0, &V3955_300_57217, &P3955_298};	/*Property = DataSet*/
const Value V3955_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_301 = {S22, 0, &V3955_301_57220, &P3955_300};	/*Property = DataUrl*/
const Value V3955_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_302 = {S23, 0, &V3955_302_57223, &P3955_301};	/*Property = DeviceOrientation*/
const Value V3955_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_304 = {S24, 0, &V3955_304_57229, &P3955_302};	/*Property = FileReader*/
const Value V3955_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_305 = {S25, 0, &V3955_305_57232, &P3955_304};	/*Property = FileSaver*/
const Value V3955_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_306 = {S26, 0, &V3955_306_57238, &P3955_305};	/*Property = FileWriter*/
const Value V3955_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_307 = {S27, 0, &V3955_307_57241, &P3955_306};	/*Property = FormData*/
const Value V3955_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_309 = {S28, 0, &V3955_309_57247, &P3955_307};	/*Property = Fullscreen*/
const Value V3955_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_310 = {S29, 0, &V3955_310_57250, &P3955_309};	/*Property = GeoLocation*/
const Value V3955_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_312 = {S30, 0, &V3955_312_57256, &P3955_310};	/*Property = History*/
const Value V3955_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_314 = {S31, 0, &V3955_314_57262, &P3955_312};	/*Property = Html5*/
const Value V3955_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_313 = {S32, 0, &V3955_313_57259, &P3955_314};	/*Property = Html-Media-Capture*/
const Value V3955_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_315 = {S33, 0, &V3955_315_57265, &P3955_313};	/*Property = Iframe*/
const Value V3955_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_316 = {S34, 0, &V3955_316_57268, &P3955_315};	/*Property = IndexedDB*/
const Value V3955_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_317 = {S35, 0, &V3955_317_57271, &P3955_316};	/*Property = Json*/
const Value V3955_15_43264 = {S59, NULL};	/*Value = Mango*/
const Property P3955_15 = {S37, 0, &V3955_15_43264, &P3955_317};	/*Property = LayoutEngine*/
const Value V3955_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_344 = {S38, 0, &V3955_344_57274, &P3955_15};	/*Property = Masking*/
const Value V3955_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_320 = {S39, 0, &V3955_320_57292, &P3955_344};	/*Property = PostMessage*/
const Value V3955_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_321 = {S40, 0, &V3955_321_57295, &P3955_320};	/*Property = Progress*/
const Value V3955_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_322 = {S41, 0, &V3955_322_57298, &P3955_321};	/*Property = Prompts*/
const Value V3955_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_323 = {S42, 0, &V3955_323_57301, &P3955_322};	/*Property = Selector*/
const Value V3955_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_324 = {S43, 0, &V3955_324_57307, &P3955_323};	/*Property = Svg*/
const Value V3955_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_327 = {S44, 0, &V3955_327_57316, &P3955_324};	/*Property = TouchEvents*/
const Value V3955_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_328 = {S45, 0, &V3955_328_57319, &P3955_327};	/*Property = Track*/
const Value V3955_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_329 = {S46, 0, &V3955_329_57325, &P3955_328};	/*Property = Video*/
const Value V3955_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_330 = {S47, 0, &V3955_330_57328, &P3955_329};	/*Property = Viewport*/
const Value V3955_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_332 = {S48, 0, &V3955_332_57340, &P3955_330};	/*Property = WebWorkers*/
const Value V3955_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P3955_333 = {S49, 0, &V3955_333_57343, &P3955_332};	/*Property = Xhr2*/
const Profile P3955 = {"3955", &P3955_333};	/*Profile ID - 3955*/
const Profile P5765 = {"5765", NULL};	/*Profile ID - 5765*/
const Value V15896_1_1 = {S2, NULL};	/*Value = True*/
const Property P15896_1 = {S50, 0, &V15896_1_1, NULL};	/*Property = IsMobile*/
const Value V15896_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15896_5 = {S52, 0, &V15896_5_1293, &P15896_1};	/*Property = ScreenPixelsHeight*/
const Value V15896_4_33019 = {S61, NULL};	/*Value = 160*/
const Property P15896_4 = {S54, 0, &V15896_4_33019, &P15896_5};	/*Property = ScreenPixelsWidth*/
const Profile P15896 = {"15896", &P15896_4};	/*Profile ID - 15896*/
const Value V12781_1_1 = {S2, NULL};	/*Value = True*/
const Property P12781_1 = {S50, 0, &V12781_1_1, NULL};	/*Property = IsMobile*/
const Value V12781_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12781_5 = {S52, 0, &V12781_5_1293, &P12781_1};	/*Property = ScreenPixelsHeight*/
const Value V12781_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12781_4 = {S54, 0, &V12781_4_1224, &P12781_5};	/*Property = ScreenPixelsWidth*/
const Profile P12781 = {"12781", &P12781_4};	/*Profile ID - 12781*/
const Value V14606_1_1 = {S2, NULL};	/*Value = True*/
const Property P14606_1 = {S50, 0, &V14606_1_1, NULL};	/*Property = IsMobile*/
const Value V14606_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P14606_5 = {S52, 0, &V14606_5_1293, &P14606_1};	/*Property = ScreenPixelsHeight*/
const Value V14606_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14606_4 = {S54, 0, &V14606_4_1224, &P14606_5};	/*Property = ScreenPixelsWidth*/
const Profile P14606 = {"14606", &P14606_4};	/*Profile ID - 14606*/
const Value V15864_1_1 = {S2, NULL};	/*Value = True*/
const Property P15864_1 = {S50, 0, &V15864_1_1, NULL};	/*Property = IsMobile*/
const Value V15864_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15864_5 = {S52, 0, &V15864_5_1380, &P15864_1};	/*Property = ScreenPixelsHeight*/
const Value V15864_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15864_4 = {S54, 0, &V15864_4_1224, &P15864_5};	/*Property = ScreenPixelsWidth*/
const Profile P15864 = {"15864", &P15864_4};	/*Profile ID - 15864*/
const Value V11665_1_1 = {S2, NULL};	/*Value = True*/
const Property P11665_1 = {S50, 0, &V11665_1_1, NULL};	/*Property = IsMobile*/
const Value V11665_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11665_5 = {S52, 0, &V11665_5_1293, &P11665_1};	/*Property = ScreenPixelsHeight*/
const Value V11665_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P11665_4 = {S54, 0, &V11665_4_1224, &P11665_5};	/*Property = ScreenPixelsWidth*/
const Profile P11665 = {"11665", &P11665_4};	/*Profile ID - 11665*/
const Value V16106_1_1 = {S2, NULL};	/*Value = True*/
const Property P16106_1 = {S50, 0, &V16106_1_1, NULL};	/*Property = IsMobile*/
const Value V16106_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16106_5 = {S52, 0, &V16106_5_1222, &P16106_1};	/*Property = ScreenPixelsHeight*/
const Value V16106_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16106_4 = {S54, 0, &V16106_4_1224, &P16106_5};	/*Property = ScreenPixelsWidth*/
const Profile P16106 = {"16106", &P16106_4};	/*Profile ID - 16106*/
const Value V16222_1_1 = {S2, NULL};	/*Value = True*/
const Property P16222_1 = {S50, 0, &V16222_1_1, NULL};	/*Property = IsMobile*/
const Value V16222_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P16222_5 = {S52, 0, &V16222_5_1280, &P16222_1};	/*Property = ScreenPixelsHeight*/
const Value V16222_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16222_4 = {S54, 0, &V16222_4_1235, &P16222_5};	/*Property = ScreenPixelsWidth*/
const Profile P16222 = {"16222", &P16222_4};	/*Profile ID - 16222*/
const Value V15036_1_1 = {S2, NULL};	/*Value = True*/
const Property P15036_1 = {S50, 0, &V15036_1_1, NULL};	/*Property = IsMobile*/
const Value V15036_5_37559 = {S93, NULL};	/*Value = 132*/
const Property P15036_5 = {S52, 0, &V15036_5_37559, &P15036_1};	/*Property = ScreenPixelsHeight*/
const Value V15036_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15036_4 = {S54, 0, &V15036_4_1235, &P15036_5};	/*Property = ScreenPixelsWidth*/
const Profile P15036 = {"15036", &P15036_4};	/*Profile ID - 15036*/
const Value V19947_1_1 = {S2, NULL};	/*Value = True*/
const Property P19947_1 = {S50, 0, &V19947_1_1, NULL};	/*Property = IsMobile*/
const Value V19947_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P19947_5 = {S52, 0, &V19947_5_1293, &P19947_1};	/*Property = ScreenPixelsHeight*/
const Value V19947_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P19947_4 = {S54, 0, &V19947_4_1296, &P19947_5};	/*Property = ScreenPixelsWidth*/
const Profile P19947 = {"19947", &P19947_4};	/*Profile ID - 19947*/
const Value V20338_1_1 = {S2, NULL};	/*Value = True*/
const Property P20338_1 = {S50, 0, &V20338_1_1, NULL};	/*Property = IsMobile*/
const Value V20338_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P20338_5 = {S52, 0, &V20338_5_1293, &P20338_1};	/*Property = ScreenPixelsHeight*/
const Value V20338_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P20338_4 = {S54, 0, &V20338_4_1296, &P20338_5};	/*Property = ScreenPixelsWidth*/
const Profile P20338 = {"20338", &P20338_4};	/*Profile ID - 20338*/
const Value V17339_1_1 = {S2, NULL};	/*Value = True*/
const Property P17339_1 = {S50, 0, &V17339_1_1, NULL};	/*Property = IsMobile*/
const Value V17339_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P17339_5 = {S52, 0, &V17339_5_1294, &P17339_1};	/*Property = ScreenPixelsHeight*/
const Value V17339_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17339_4 = {S54, 0, &V17339_4_1296, &P17339_5};	/*Property = ScreenPixelsWidth*/
const Profile P17339 = {"17339", &P17339_4};	/*Profile ID - 17339*/
const Value V20112_1_2 = {S0, NULL};	/*Value = False*/
const Property P20112_1 = {S50, 0, &V20112_1_2, NULL};	/*Property = IsMobile*/
const Value V20112_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20112_5 = {S52, 0, &V20112_5_37, &P20112_1};	/*Property = ScreenPixelsHeight*/
const Value V20112_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20112_4 = {S54, 0, &V20112_4_36, &P20112_5};	/*Property = ScreenPixelsWidth*/
const Profile P20112 = {"20112", &P20112_4};	/*Profile ID - 20112*/
const Value V17222_1_1 = {S2, NULL};	/*Value = True*/
const Property P17222_1 = {S50, 0, &V17222_1_1, NULL};	/*Property = IsMobile*/
const Value V17222_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17222_5 = {S52, 0, &V17222_5_1233, &P17222_1};	/*Property = ScreenPixelsHeight*/
const Value V17222_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17222_4 = {S54, 0, &V17222_4_1235, &P17222_5};	/*Property = ScreenPixelsWidth*/
const Profile P17222 = {"17222", &P17222_4};	/*Profile ID - 17222*/
const Value V17476_1_1 = {S2, NULL};	/*Value = True*/
const Property P17476_1 = {S50, 0, &V17476_1_1, NULL};	/*Property = IsMobile*/
const Value V17476_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17476_5 = {S52, 0, &V17476_5_1293, &P17476_1};	/*Property = ScreenPixelsHeight*/
const Value V17476_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17476_4 = {S54, 0, &V17476_4_1296, &P17476_5};	/*Property = ScreenPixelsWidth*/
const Profile P17476 = {"17476", &P17476_4};	/*Profile ID - 17476*/
const Value V19919_1_1 = {S2, NULL};	/*Value = True*/
const Property P19919_1 = {S50, 0, &V19919_1_1, NULL};	/*Property = IsMobile*/
const Value V19919_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P19919_5 = {S52, 0, &V19919_5_1222, &P19919_1};	/*Property = ScreenPixelsHeight*/
const Value V19919_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P19919_4 = {S54, 0, &V19919_4_1224, &P19919_5};	/*Property = ScreenPixelsWidth*/
const Profile P19919 = {"19919", &P19919_4};	/*Profile ID - 19919*/
const Value V13926_1_1 = {S2, NULL};	/*Value = True*/
const Property P13926_1 = {S50, 0, &V13926_1_1, NULL};	/*Property = IsMobile*/
const Value V13926_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13926_5 = {S52, 0, &V13926_5_1293, &P13926_1};	/*Property = ScreenPixelsHeight*/
const Value V13926_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13926_4 = {S54, 0, &V13926_4_1296, &P13926_5};	/*Property = ScreenPixelsWidth*/
const Profile P13926 = {"13926", &P13926_4};	/*Profile ID - 13926*/
const Value V20092_1_1 = {S2, NULL};	/*Value = True*/
const Property P20092_1 = {S50, 0, &V20092_1_1, NULL};	/*Property = IsMobile*/
const Value V20092_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20092_5 = {S52, 0, &V20092_5_1227, &P20092_1};	/*Property = ScreenPixelsHeight*/
const Value V20092_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20092_4 = {S54, 0, &V20092_4_1229, &P20092_5};	/*Property = ScreenPixelsWidth*/
const Profile P20092 = {"20092", &P20092_4};	/*Profile ID - 20092*/
const Value V19926_1_1 = {S2, NULL};	/*Value = True*/
const Property P19926_1 = {S50, 0, &V19926_1_1, NULL};	/*Property = IsMobile*/
const Value V19926_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P19926_5 = {S52, 0, &V19926_5_1293, &P19926_1};	/*Property = ScreenPixelsHeight*/
const Value V19926_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P19926_4 = {S54, 0, &V19926_4_1296, &P19926_5};	/*Property = ScreenPixelsWidth*/
const Profile P19926 = {"19926", &P19926_4};	/*Profile ID - 19926*/
const Value V17312_1_1 = {S2, NULL};	/*Value = True*/
const Property P17312_1 = {S50, 0, &V17312_1_1, NULL};	/*Property = IsMobile*/
const Value V17312_5_1280 = {S74, NULL};	/*Value = 208*/
const Property P17312_5 = {S52, 0, &V17312_5_1280, &P17312_1};	/*Property = ScreenPixelsHeight*/
const Value V17312_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17312_4 = {S54, 0, &V17312_4_1235, &P17312_5};	/*Property = ScreenPixelsWidth*/
const Profile P17312 = {"17312", &P17312_4};	/*Profile ID - 17312*/
const Value V17313_1_1 = {S2, NULL};	/*Value = True*/
const Property P17313_1 = {S50, 0, &V17313_1_1, NULL};	/*Property = IsMobile*/
const Value V17313_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17313_5 = {S52, 0, &V17313_5_1233, &P17313_1};	/*Property = ScreenPixelsHeight*/
const Value V17313_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17313_4 = {S54, 0, &V17313_4_1235, &P17313_5};	/*Property = ScreenPixelsWidth*/
const Profile P17313 = {"17313", &P17313_4};	/*Profile ID - 17313*/
const Value V17289_1_1 = {S2, NULL};	/*Value = True*/
const Property P17289_1 = {S50, 0, &V17289_1_1, NULL};	/*Property = IsMobile*/
const Value V17289_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17289_5 = {S52, 0, &V17289_5_1233, &P17289_1};	/*Property = ScreenPixelsHeight*/
const Value V17289_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17289_4 = {S54, 0, &V17289_4_1235, &P17289_5};	/*Property = ScreenPixelsWidth*/
const Profile P17289 = {"17289", &P17289_4};	/*Profile ID - 17289*/
const Value V20334_1_1 = {S2, NULL};	/*Value = True*/
const Property P20334_1 = {S50, 0, &V20334_1_1, NULL};	/*Property = IsMobile*/
const Value V20334_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P20334_5 = {S52, 0, &V20334_5_1293, &P20334_1};	/*Property = ScreenPixelsHeight*/
const Value V20334_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P20334_4 = {S54, 0, &V20334_4_1296, &P20334_5};	/*Property = ScreenPixelsWidth*/
const Profile P20334 = {"20334", &P20334_4};	/*Profile ID - 20334*/
const Value V20302_1_1 = {S2, NULL};	/*Value = True*/
const Property P20302_1 = {S50, 0, &V20302_1_1, NULL};	/*Property = IsMobile*/
const Value V20302_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P20302_5 = {S52, 0, &V20302_5_1293, &P20302_1};	/*Property = ScreenPixelsHeight*/
const Value V20302_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P20302_4 = {S54, 0, &V20302_4_1296, &P20302_5};	/*Property = ScreenPixelsWidth*/
const Profile P20302 = {"20302", &P20302_4};	/*Profile ID - 20302*/
const Value V20159_1_1 = {S2, NULL};	/*Value = True*/
const Property P20159_1 = {S50, 0, &V20159_1_1, NULL};	/*Property = IsMobile*/
const Value V20159_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P20159_5 = {S52, 0, &V20159_5_1293, &P20159_1};	/*Property = ScreenPixelsHeight*/
const Value V20159_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P20159_4 = {S54, 0, &V20159_4_1296, &P20159_5};	/*Property = ScreenPixelsWidth*/
const Profile P20159 = {"20159", &P20159_4};	/*Profile ID - 20159*/
const Value V15515_1_1 = {S2, NULL};	/*Value = True*/
const Property P15515_1 = {S50, 0, &V15515_1_1, NULL};	/*Property = IsMobile*/
const Value V15515_5_37453 = {S91, NULL};	/*Value = 65*/
const Property P15515_5 = {S52, 0, &V15515_5_37453, &P15515_1};	/*Property = ScreenPixelsHeight*/
const Value V15515_4_31956 = {S92, NULL};	/*Value = 96*/
const Property P15515_4 = {S54, 0, &V15515_4_31956, &P15515_5};	/*Property = ScreenPixelsWidth*/
const Profile P15515 = {"15515", &P15515_4};	/*Profile ID - 15515*/
const Value V17344_1_1 = {S2, NULL};	/*Value = True*/
const Property P17344_1 = {S50, 0, &V17344_1_1, NULL};	/*Property = IsMobile*/
const Value V17344_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17344_5 = {S52, 0, &V17344_5_1293, &P17344_1};	/*Property = ScreenPixelsHeight*/
const Value V17344_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17344_4 = {S54, 0, &V17344_4_1296, &P17344_5};	/*Property = ScreenPixelsWidth*/
const Profile P17344 = {"17344", &P17344_4};	/*Profile ID - 17344*/
const Value V17014_1_1 = {S2, NULL};	/*Value = True*/
const Property P17014_1 = {S50, 0, &V17014_1_1, NULL};	/*Property = IsMobile*/
const Value V17014_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17014_5 = {S52, 0, &V17014_5_1293, &P17014_1};	/*Property = ScreenPixelsHeight*/
const Value V17014_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17014_4 = {S54, 0, &V17014_4_1296, &P17014_5};	/*Property = ScreenPixelsWidth*/
const Profile P17014 = {"17014", &P17014_4};	/*Profile ID - 17014*/
const Value V17689_1_1 = {S2, NULL};	/*Value = True*/
const Property P17689_1 = {S50, 0, &V17689_1_1, NULL};	/*Property = IsMobile*/
const Value V17689_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17689_5 = {S52, 0, &V17689_5_1293, &P17689_1};	/*Property = ScreenPixelsHeight*/
const Value V17689_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17689_4 = {S54, 0, &V17689_4_1296, &P17689_5};	/*Property = ScreenPixelsWidth*/
const Profile P17689 = {"17689", &P17689_4};	/*Profile ID - 17689*/
const Value V12704_1_1 = {S2, NULL};	/*Value = True*/
const Property P12704_1 = {S50, 0, &V12704_1_1, NULL};	/*Property = IsMobile*/
const Value V12704_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12704_5 = {S52, 0, &V12704_5_1233, &P12704_1};	/*Property = ScreenPixelsHeight*/
const Value V12704_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12704_4 = {S54, 0, &V12704_4_1235, &P12704_5};	/*Property = ScreenPixelsWidth*/
const Profile P12704 = {"12704", &P12704_4};	/*Profile ID - 12704*/
const Value V16507_1_1 = {S2, NULL};	/*Value = True*/
const Property P16507_1 = {S50, 0, &V16507_1_1, NULL};	/*Property = IsMobile*/
const Value V16507_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16507_5 = {S52, 0, &V16507_5_1294, &P16507_1};	/*Property = ScreenPixelsHeight*/
const Value V16507_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16507_4 = {S54, 0, &V16507_4_1296, &P16507_5};	/*Property = ScreenPixelsWidth*/
const Profile P16507 = {"16507", &P16507_4};	/*Profile ID - 16507*/
const Value V16511_1_1 = {S2, NULL};	/*Value = True*/
const Property P16511_1 = {S50, 0, &V16511_1_1, NULL};	/*Property = IsMobile*/
const Value V16511_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16511_5 = {S52, 0, &V16511_5_1294, &P16511_1};	/*Property = ScreenPixelsHeight*/
const Value V16511_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16511_4 = {S54, 0, &V16511_4_1296, &P16511_5};	/*Property = ScreenPixelsWidth*/
const Profile P16511 = {"16511", &P16511_4};	/*Profile ID - 16511*/
const Value V14139_1_1 = {S2, NULL};	/*Value = True*/
const Property P14139_1 = {S50, 0, &V14139_1_1, NULL};	/*Property = IsMobile*/
const Value V14139_5_37905 = {S97, NULL};	/*Value = 101*/
const Property P14139_5 = {S52, 0, &V14139_5_37905, &P14139_1};	/*Property = ScreenPixelsHeight*/
const Value V14139_4_37593 = {S96, NULL};	/*Value = 80*/
const Property P14139_4 = {S54, 0, &V14139_4_37593, &P14139_5};	/*Property = ScreenPixelsWidth*/
const Profile P14139 = {"14139", &P14139_4};	/*Profile ID - 14139*/
const Value V13924_1_1 = {S2, NULL};	/*Value = True*/
const Property P13924_1 = {S50, 0, &V13924_1_1, NULL};	/*Property = IsMobile*/
const Value V13924_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P13924_5 = {S52, 0, &V13924_5_1333, &P13924_1};	/*Property = ScreenPixelsHeight*/
const Value V13924_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P13924_4 = {S54, 0, &V13924_4_1335, &P13924_5};	/*Property = ScreenPixelsWidth*/
const Profile P13924 = {"13924", &P13924_4};	/*Profile ID - 13924*/
const Value V12604_1_1 = {S2, NULL};	/*Value = True*/
const Property P12604_1 = {S50, 0, &V12604_1_1, NULL};	/*Property = IsMobile*/
const Value V12604_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12604_5 = {S52, 0, &V12604_5_1222, &P12604_1};	/*Property = ScreenPixelsHeight*/
const Value V12604_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12604_4 = {S54, 0, &V12604_4_1224, &P12604_5};	/*Property = ScreenPixelsWidth*/
const Profile P12604 = {"12604", &P12604_4};	/*Profile ID - 12604*/
const Value V19925_1_1 = {S2, NULL};	/*Value = True*/
const Property P19925_1 = {S50, 0, &V19925_1_1, NULL};	/*Property = IsMobile*/
const Value V19925_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P19925_5 = {S52, 0, &V19925_5_1233, &P19925_1};	/*Property = ScreenPixelsHeight*/
const Value V19925_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P19925_4 = {S54, 0, &V19925_4_1235, &P19925_5};	/*Property = ScreenPixelsWidth*/
const Profile P19925 = {"19925", &P19925_4};	/*Profile ID - 19925*/
const Value V17439_1_1 = {S2, NULL};	/*Value = True*/
const Property P17439_1 = {S50, 0, &V17439_1_1, NULL};	/*Property = IsMobile*/
const Value V17439_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17439_5 = {S52, 0, &V17439_5_1293, &P17439_1};	/*Property = ScreenPixelsHeight*/
const Value V17439_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17439_4 = {S54, 0, &V17439_4_1296, &P17439_5};	/*Property = ScreenPixelsWidth*/
const Profile P17439 = {"17439", &P17439_4};	/*Profile ID - 17439*/
const Value V16240_1_1 = {S2, NULL};	/*Value = True*/
const Property P16240_1 = {S50, 0, &V16240_1_1, NULL};	/*Property = IsMobile*/
const Value V16240_5_37483 = {S113, NULL};	/*Value = 67*/
const Property P16240_5 = {S52, 0, &V16240_5_37483, &P16240_1};	/*Property = ScreenPixelsHeight*/
const Value V16240_4_37576 = {S97, NULL};	/*Value = 101*/
const Property P16240_4 = {S54, 0, &V16240_4_37576, &P16240_5};	/*Property = ScreenPixelsWidth*/
const Profile P16240 = {"16240", &P16240_4};	/*Profile ID - 16240*/
const Value V16191_1_1 = {S2, NULL};	/*Value = True*/
const Property P16191_1 = {S50, 0, &V16191_1_1, NULL};	/*Property = IsMobile*/
const Value V16191_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16191_5 = {S52, 0, &V16191_5_1294, &P16191_1};	/*Property = ScreenPixelsHeight*/
const Value V16191_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16191_4 = {S54, 0, &V16191_4_1296, &P16191_5};	/*Property = ScreenPixelsWidth*/
const Profile P16191 = {"16191", &P16191_4};	/*Profile ID - 16191*/
const Value V16497_1_1 = {S2, NULL};	/*Value = True*/
const Property P16497_1 = {S50, 0, &V16497_1_1, NULL};	/*Property = IsMobile*/
const Value V16497_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16497_5 = {S52, 0, &V16497_5_1293, &P16497_1};	/*Property = ScreenPixelsHeight*/
const Value V16497_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16497_4 = {S54, 0, &V16497_4_1296, &P16497_5};	/*Property = ScreenPixelsWidth*/
const Profile P16497 = {"16497", &P16497_4};	/*Profile ID - 16497*/
const Value V16484_1_1 = {S2, NULL};	/*Value = True*/
const Property P16484_1 = {S50, 0, &V16484_1_1, NULL};	/*Property = IsMobile*/
const Value V16484_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16484_5 = {S52, 0, &V16484_5_1294, &P16484_1};	/*Property = ScreenPixelsHeight*/
const Value V16484_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16484_4 = {S54, 0, &V16484_4_1296, &P16484_5};	/*Property = ScreenPixelsWidth*/
const Profile P16484 = {"16484", &P16484_4};	/*Profile ID - 16484*/
const Value V16508_1_1 = {S2, NULL};	/*Value = True*/
const Property P16508_1 = {S50, 0, &V16508_1_1, NULL};	/*Property = IsMobile*/
const Value V16508_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16508_5 = {S52, 0, &V16508_5_1294, &P16508_1};	/*Property = ScreenPixelsHeight*/
const Value V16508_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16508_4 = {S54, 0, &V16508_4_1296, &P16508_5};	/*Property = ScreenPixelsWidth*/
const Profile P16508 = {"16508", &P16508_4};	/*Profile ID - 16508*/
const Value V16201_1_1 = {S2, NULL};	/*Value = True*/
const Property P16201_1 = {S50, 0, &V16201_1_1, NULL};	/*Property = IsMobile*/
const Value V16201_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16201_5 = {S52, 0, &V16201_5_1294, &P16201_1};	/*Property = ScreenPixelsHeight*/
const Value V16201_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16201_4 = {S54, 0, &V16201_4_1296, &P16201_5};	/*Property = ScreenPixelsWidth*/
const Profile P16201 = {"16201", &P16201_4};	/*Profile ID - 16201*/
const Value V16664_1_1 = {S2, NULL};	/*Value = True*/
const Property P16664_1 = {S50, 0, &V16664_1_1, NULL};	/*Property = IsMobile*/
const Value V16664_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16664_5 = {S52, 0, &V16664_5_1294, &P16664_1};	/*Property = ScreenPixelsHeight*/
const Value V16664_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16664_4 = {S54, 0, &V16664_4_1296, &P16664_5};	/*Property = ScreenPixelsWidth*/
const Profile P16664 = {"16664", &P16664_4};	/*Profile ID - 16664*/
const Value V16661_1_1 = {S2, NULL};	/*Value = True*/
const Property P16661_1 = {S50, 0, &V16661_1_1, NULL};	/*Property = IsMobile*/
const Value V16661_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16661_5 = {S52, 0, &V16661_5_1294, &P16661_1};	/*Property = ScreenPixelsHeight*/
const Value V16661_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16661_4 = {S54, 0, &V16661_4_1296, &P16661_5};	/*Property = ScreenPixelsWidth*/
const Profile P16661 = {"16661", &P16661_4};	/*Profile ID - 16661*/
const Value V20223_1_1 = {S2, NULL};	/*Value = True*/
const Property P20223_1 = {S50, 0, &V20223_1_1, NULL};	/*Property = IsMobile*/
const Value V20223_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P20223_5 = {S52, 0, &V20223_5_1294, &P20223_1};	/*Property = ScreenPixelsHeight*/
const Value V20223_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P20223_4 = {S54, 0, &V20223_4_1296, &P20223_5};	/*Property = ScreenPixelsWidth*/
const Profile P20223 = {"20223", &P20223_4};	/*Profile ID - 20223*/
const Value V20217_1_2 = {S0, NULL};	/*Value = False*/
const Property P20217_1 = {S50, 0, &V20217_1_2, NULL};	/*Property = IsMobile*/
const Value V20217_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20217_5 = {S52, 0, &V20217_5_37, &P20217_1};	/*Property = ScreenPixelsHeight*/
const Value V20217_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20217_4 = {S54, 0, &V20217_4_36, &P20217_5};	/*Property = ScreenPixelsWidth*/
const Profile P20217 = {"20217", &P20217_4};	/*Profile ID - 20217*/
const Value V16641_1_1 = {S2, NULL};	/*Value = True*/
const Property P16641_1 = {S50, 0, &V16641_1_1, NULL};	/*Property = IsMobile*/
const Value V16641_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16641_5 = {S52, 0, &V16641_5_1293, &P16641_1};	/*Property = ScreenPixelsHeight*/
const Value V16641_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16641_4 = {S54, 0, &V16641_4_1296, &P16641_5};	/*Property = ScreenPixelsWidth*/
const Profile P16641 = {"16641", &P16641_4};	/*Profile ID - 16641*/
const Value V16228_1_1 = {S2, NULL};	/*Value = True*/
const Property P16228_1 = {S50, 0, &V16228_1_1, NULL};	/*Property = IsMobile*/
const Value V16228_5_37483 = {S113, NULL};	/*Value = 67*/
const Property P16228_5 = {S52, 0, &V16228_5_37483, &P16228_1};	/*Property = ScreenPixelsHeight*/
const Value V16228_4_37576 = {S97, NULL};	/*Value = 101*/
const Property P16228_4 = {S54, 0, &V16228_4_37576, &P16228_5};	/*Property = ScreenPixelsWidth*/
const Profile P16228 = {"16228", &P16228_4};	/*Profile ID - 16228*/
const Value V16826_1_1 = {S2, NULL};	/*Value = True*/
const Property P16826_1 = {S50, 0, &V16826_1_1, NULL};	/*Property = IsMobile*/
const Value V16826_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16826_5 = {S52, 0, &V16826_5_1293, &P16826_1};	/*Property = ScreenPixelsHeight*/
const Value V16826_4_24031 = {S71, NULL};	/*Value = 120*/
const Property P16826_4 = {S54, 0, &V16826_4_24031, &P16826_5};	/*Property = ScreenPixelsWidth*/
const Profile P16826 = {"16826", &P16826_4};	/*Profile ID - 16826*/
const Value V16863_1_1 = {S2, NULL};	/*Value = True*/
const Property P16863_1 = {S50, 0, &V16863_1_1, NULL};	/*Property = IsMobile*/
const Value V16863_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16863_5 = {S52, 0, &V16863_5_1293, &P16863_1};	/*Property = ScreenPixelsHeight*/
const Value V16863_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16863_4 = {S54, 0, &V16863_4_1296, &P16863_5};	/*Property = ScreenPixelsWidth*/
const Profile P16863 = {"16863", &P16863_4};	/*Profile ID - 16863*/
const Value V17130_1_1 = {S2, NULL};	/*Value = True*/
const Property P17130_1 = {S50, 0, &V17130_1_1, NULL};	/*Property = IsMobile*/
const Value V17130_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17130_5 = {S52, 0, &V17130_5_1293, &P17130_1};	/*Property = ScreenPixelsHeight*/
const Value V17130_4_24031 = {S71, NULL};	/*Value = 120*/
const Property P17130_4 = {S54, 0, &V17130_4_24031, &P17130_5};	/*Property = ScreenPixelsWidth*/
const Profile P17130 = {"17130", &P17130_4};	/*Profile ID - 17130*/
const Value V17846_1_1 = {S2, NULL};	/*Value = True*/
const Property P17846_1 = {S50, 0, &V17846_1_1, NULL};	/*Property = IsMobile*/
const Value V17846_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P17846_5 = {S52, 0, &V17846_5_1333, &P17846_1};	/*Property = ScreenPixelsHeight*/
const Value V17846_4_37572 = {S93, NULL};	/*Value = 132*/
const Property P17846_4 = {S54, 0, &V17846_4_37572, &P17846_5};	/*Property = ScreenPixelsWidth*/
const Profile P17846 = {"17846", &P17846_4};	/*Profile ID - 17846*/
const Value V12022_1_1 = {S2, NULL};	/*Value = True*/
const Property P12022_1 = {S50, 0, &V12022_1_1, NULL};	/*Property = IsMobile*/
const Value V12022_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P12022_5 = {S52, 0, &V12022_5_1233, &P12022_1};	/*Property = ScreenPixelsHeight*/
const Value V12022_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P12022_4 = {S54, 0, &V12022_4_1235, &P12022_5};	/*Property = ScreenPixelsWidth*/
const Profile P12022 = {"12022", &P12022_4};	/*Profile ID - 12022*/
const Value V16333_1_1 = {S2, NULL};	/*Value = True*/
const Property P16333_1 = {S50, 0, &V16333_1_1, NULL};	/*Property = IsMobile*/
const Value V16333_5_37505 = {S83, NULL};	/*Value = 272*/
const Property P16333_5 = {S52, 0, &V16333_5_37505, &P16333_1};	/*Property = ScreenPixelsHeight*/
const Value V16333_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16333_4 = {S54, 0, &V16333_4_1229, &P16333_5};	/*Property = ScreenPixelsWidth*/
const Profile P16333 = {"16333", &P16333_4};	/*Profile ID - 16333*/
const Value V17652_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17652_279 = {S1, 0, &V17652_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17652_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17652_282 = {S3, 0, &V17652_282_57154, &P17652_279};	/*Property = BlobBuilder*/
const Value V17652_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17652_350 = {S4, 0, &V17652_350_57444, &P17652_282};	/*Property = Canvas*/
const Value V17652_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17652_283 = {S5, 0, &V17652_283_57159, &P17652_350};	/*Property = CssBackground*/
const Value V17652_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17652_284 = {S6, 0, &V17652_284_57163, &P17652_283};	/*Property = CssBorderImage*/
const Value V17652_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17652_285 = {S7, 0, &V17652_285_57166, &P17652_284};	/*Property = CssCanvas*/
const Value V17652_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17652_286 = {S8, 0, &V17652_286_57168, &P17652_285};	/*Property = CssColor*/
const Value V17652_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17652_237 = {S9, 0, &V17652_237_55786, &P17652_286};	/*Property = CssColumn*/
const Value V17652_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17652_287 = {S10, 0, &V17652_287_57172, &P17652_237};	/*Property = CssFlexbox*/
const Value V17652_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17652_288 = {S11, 0, &V17652_288_57175, &P17652_287};	/*Property = CssFont*/
const Value V17652_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17652_289 = {S12, 0, &V17652_289_57759, &P17652_288};	/*Property = CssImages*/
const Value V17652_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17652_291 = {S13, 0, &V17652_291_57181, &P17652_289};	/*Property = CssMediaQueries*/
const Value V17652_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17652_292 = {S14, 0, &V17652_292_57183, &P17652_291};	/*Property = CssMinMax*/
const Value V17652_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17652_293 = {S15, 0, &V17652_293_57187, &P17652_292};	/*Property = CssOverflow*/
const Value V17652_294_57190 = {S0, NULL};	/*Value = False*/
const Property P17652_294 = {S16, 0, &V17652_294_57190, &P17652_293};	/*Property = CssPosition*/
const Value V17652_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17652_295 = {S17, 0, &V17652_295_57192, &P17652_294};	/*Property = CssText*/
const Value V17652_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17652_297 = {S18, 0, &V17652_297_57195, &P17652_295};	/*Property = CssTransforms*/
const Value V17652_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17652_241 = {S19, 0, &V17652_241_55799, &P17652_297};	/*Property = CssTransitions*/
const Value V17652_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17652_298 = {S20, 0, &V17652_298_57206, &P17652_241};	/*Property = CssUI*/
const Value V17652_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17652_300 = {S21, 0, &V17652_300_57216, &P17652_298};	/*Property = DataSet*/
const Value V17652_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17652_301 = {S22, 0, &V17652_301_57218, &P17652_300};	/*Property = DataUrl*/
const Value V17652_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17652_302 = {S23, 0, &V17652_302_57222, &P17652_301};	/*Property = DeviceOrientation*/
const Value V17652_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17652_304 = {S24, 0, &V17652_304_57228, &P17652_302};	/*Property = FileReader*/
const Value V17652_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17652_305 = {S25, 0, &V17652_305_57231, &P17652_304};	/*Property = FileSaver*/
const Value V17652_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17652_306 = {S26, 0, &V17652_306_57237, &P17652_305};	/*Property = FileWriter*/
const Value V17652_307_57240 = {S0, NULL};	/*Value = False*/
const Property P17652_307 = {S27, 0, &V17652_307_57240, &P17652_306};	/*Property = FormData*/
const Value V17652_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17652_309 = {S28, 0, &V17652_309_57246, &P17652_307};	/*Property = Fullscreen*/
const Value V17652_310_57249 = {S0, NULL};	/*Value = False*/
const Property P17652_310 = {S29, 0, &V17652_310_57249, &P17652_309};	/*Property = GeoLocation*/
const Value V17652_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17652_312 = {S30, 0, &V17652_312_57255, &P17652_310};	/*Property = History*/
const Value V17652_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17652_314 = {S31, 0, &V17652_314_57261, &P17652_312};	/*Property = Html5*/
const Value V17652_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17652_313 = {S32, 0, &V17652_313_57258, &P17652_314};	/*Property = Html-Media-Capture*/
const Value V17652_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17652_315 = {S33, 0, &V17652_315_57264, &P17652_313};	/*Property = Iframe*/
const Value V17652_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17652_316 = {S34, 0, &V17652_316_57267, &P17652_315};	/*Property = IndexedDB*/
const Value V17652_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17652_317 = {S35, 0, &V17652_317_57269, &P17652_316};	/*Property = Json*/
const Value V17652_15_43263 = {S65, NULL};	/*Value = Netfront*/
const Property P17652_15 = {S37, 0, &V17652_15_43263, &P17652_317};	/*Property = LayoutEngine*/
const Value V17652_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17652_344 = {S38, 0, &V17652_344_57272, &P17652_15};	/*Property = Masking*/
const Value V17652_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17652_320 = {S39, 0, &V17652_320_57290, &P17652_344};	/*Property = PostMessage*/
const Value V17652_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17652_321 = {S40, 0, &V17652_321_57293, &P17652_320};	/*Property = Progress*/
const Value V17652_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17652_322 = {S41, 0, &V17652_322_57296, &P17652_321};	/*Property = Prompts*/
const Value V17652_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17652_323 = {S42, 0, &V17652_323_57299, &P17652_322};	/*Property = Selector*/
const Value V17652_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17652_324 = {S43, 0, &V17652_324_57306, &P17652_323};	/*Property = Svg*/
const Value V17652_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17652_327 = {S44, 0, &V17652_327_57315, &P17652_324};	/*Property = TouchEvents*/
const Value V17652_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17652_328 = {S45, 0, &V17652_328_57318, &P17652_327};	/*Property = Track*/
const Value V17652_329_57324 = {S0, NULL};	/*Value = False*/
const Property P17652_329 = {S46, 0, &V17652_329_57324, &P17652_328};	/*Property = Video*/
const Value V17652_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17652_330 = {S47, 0, &V17652_330_57326, &P17652_329};	/*Property = Viewport*/
const Value V17652_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17652_332 = {S48, 0, &V17652_332_57339, &P17652_330};	/*Property = WebWorkers*/
const Value V17652_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17652_333 = {S49, 0, &V17652_333_57342, &P17652_332};	/*Property = Xhr2*/
const Profile P17652 = {"17652", &P17652_333};	/*Profile ID - 17652*/
const Profile P18180 = {"18180", NULL};	/*Profile ID - 18180*/
const Value V16755_1_1 = {S2, NULL};	/*Value = True*/
const Property P16755_1 = {S50, 0, &V16755_1_1, NULL};	/*Property = IsMobile*/
const Value V16755_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16755_5 = {S52, 0, &V16755_5_1293, &P16755_1};	/*Property = ScreenPixelsHeight*/
const Value V16755_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16755_4 = {S54, 0, &V16755_4_1296, &P16755_5};	/*Property = ScreenPixelsWidth*/
const Profile P16755 = {"16755", &P16755_4};	/*Profile ID - 16755*/
const Value V17001_1_1 = {S2, NULL};	/*Value = True*/
const Property P17001_1 = {S50, 0, &V17001_1_1, NULL};	/*Property = IsMobile*/
const Value V17001_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P17001_5 = {S52, 0, &V17001_5_1294, &P17001_1};	/*Property = ScreenPixelsHeight*/
const Value V17001_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17001_4 = {S54, 0, &V17001_4_1296, &P17001_5};	/*Property = ScreenPixelsWidth*/
const Profile P17001 = {"17001", &P17001_4};	/*Profile ID - 17001*/
const Value V16997_1_1 = {S2, NULL};	/*Value = True*/
const Property P16997_1 = {S50, 0, &V16997_1_1, NULL};	/*Property = IsMobile*/
const Value V16997_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16997_5 = {S52, 0, &V16997_5_1293, &P16997_1};	/*Property = ScreenPixelsHeight*/
const Value V16997_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16997_4 = {S54, 0, &V16997_4_1296, &P16997_5};	/*Property = ScreenPixelsWidth*/
const Profile P16997 = {"16997", &P16997_4};	/*Profile ID - 16997*/
const Value V16493_1_1 = {S2, NULL};	/*Value = True*/
const Property P16493_1 = {S50, 0, &V16493_1_1, NULL};	/*Property = IsMobile*/
const Value V16493_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16493_5 = {S52, 0, &V16493_5_1294, &P16493_1};	/*Property = ScreenPixelsHeight*/
const Value V16493_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16493_4 = {S54, 0, &V16493_4_1296, &P16493_5};	/*Property = ScreenPixelsWidth*/
const Profile P16493 = {"16493", &P16493_4};	/*Profile ID - 16493*/
const Value V16692_1_1 = {S2, NULL};	/*Value = True*/
const Property P16692_1 = {S50, 0, &V16692_1_1, NULL};	/*Property = IsMobile*/
const Value V16692_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16692_5 = {S52, 0, &V16692_5_1233, &P16692_1};	/*Property = ScreenPixelsHeight*/
const Value V16692_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16692_4 = {S54, 0, &V16692_4_1235, &P16692_5};	/*Property = ScreenPixelsWidth*/
const Profile P16692 = {"16692", &P16692_4};	/*Profile ID - 16692*/
const Value V16504_1_1 = {S2, NULL};	/*Value = True*/
const Property P16504_1 = {S50, 0, &V16504_1_1, NULL};	/*Property = IsMobile*/
const Value V16504_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16504_5 = {S52, 0, &V16504_5_1294, &P16504_1};	/*Property = ScreenPixelsHeight*/
const Value V16504_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16504_4 = {S54, 0, &V16504_4_1296, &P16504_5};	/*Property = ScreenPixelsWidth*/
const Profile P16504 = {"16504", &P16504_4};	/*Profile ID - 16504*/
const Value V16079_1_1 = {S2, NULL};	/*Value = True*/
const Property P16079_1 = {S50, 0, &V16079_1_1, NULL};	/*Property = IsMobile*/
const Value V16079_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16079_5 = {S52, 0, &V16079_5_1294, &P16079_1};	/*Property = ScreenPixelsHeight*/
const Value V16079_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16079_4 = {S54, 0, &V16079_4_1296, &P16079_5};	/*Property = ScreenPixelsWidth*/
const Profile P16079 = {"16079", &P16079_4};	/*Profile ID - 16079*/
const Value V16505_1_1 = {S2, NULL};	/*Value = True*/
const Property P16505_1 = {S50, 0, &V16505_1_1, NULL};	/*Property = IsMobile*/
const Value V16505_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16505_5 = {S52, 0, &V16505_5_1293, &P16505_1};	/*Property = ScreenPixelsHeight*/
const Value V16505_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16505_4 = {S54, 0, &V16505_4_1296, &P16505_5};	/*Property = ScreenPixelsWidth*/
const Profile P16505 = {"16505", &P16505_4};	/*Profile ID - 16505*/
const Value V17656_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_279 = {S1, 0, &V17656_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17656_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_282 = {S3, 0, &V17656_282_57155, &P17656_279};	/*Property = BlobBuilder*/
const Value V17656_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_350 = {S4, 0, &V17656_350_57446, &P17656_282};	/*Property = Canvas*/
const Value V17656_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_283 = {S5, 0, &V17656_283_57161, &P17656_350};	/*Property = CssBackground*/
const Value V17656_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_284 = {S6, 0, &V17656_284_57164, &P17656_283};	/*Property = CssBorderImage*/
const Value V17656_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_285 = {S7, 0, &V17656_285_57167, &P17656_284};	/*Property = CssCanvas*/
const Value V17656_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_286 = {S8, 0, &V17656_286_57170, &P17656_285};	/*Property = CssColor*/
const Value V17656_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_237 = {S9, 0, &V17656_237_55786, &P17656_286};	/*Property = CssColumn*/
const Value V17656_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_287 = {S10, 0, &V17656_287_57173, &P17656_237};	/*Property = CssFlexbox*/
const Value V17656_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_288 = {S11, 0, &V17656_288_57176, &P17656_287};	/*Property = CssFont*/
const Value V17656_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_289 = {S12, 0, &V17656_289_86899, &P17656_288};	/*Property = CssImages*/
const Value V17656_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_291 = {S13, 0, &V17656_291_57182, &P17656_289};	/*Property = CssMediaQueries*/
const Value V17656_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_292 = {S14, 0, &V17656_292_57185, &P17656_291};	/*Property = CssMinMax*/
const Value V17656_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_293 = {S15, 0, &V17656_293_57188, &P17656_292};	/*Property = CssOverflow*/
const Value V17656_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_294 = {S16, 0, &V17656_294_57191, &P17656_293};	/*Property = CssPosition*/
const Value V17656_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_295 = {S17, 0, &V17656_295_57194, &P17656_294};	/*Property = CssText*/
const Value V17656_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_297 = {S18, 0, &V17656_297_57197, &P17656_295};	/*Property = CssTransforms*/
const Value V17656_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_241 = {S19, 0, &V17656_241_55801, &P17656_297};	/*Property = CssTransitions*/
const Value V17656_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_298 = {S20, 0, &V17656_298_57208, &P17656_241};	/*Property = CssUI*/
const Value V17656_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_300 = {S21, 0, &V17656_300_57217, &P17656_298};	/*Property = DataSet*/
const Value V17656_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_301 = {S22, 0, &V17656_301_57220, &P17656_300};	/*Property = DataUrl*/
const Value V17656_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_302 = {S23, 0, &V17656_302_57223, &P17656_301};	/*Property = DeviceOrientation*/
const Value V17656_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_304 = {S24, 0, &V17656_304_57229, &P17656_302};	/*Property = FileReader*/
const Value V17656_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_305 = {S25, 0, &V17656_305_57232, &P17656_304};	/*Property = FileSaver*/
const Value V17656_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_306 = {S26, 0, &V17656_306_57238, &P17656_305};	/*Property = FileWriter*/
const Value V17656_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_307 = {S27, 0, &V17656_307_57241, &P17656_306};	/*Property = FormData*/
const Value V17656_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_309 = {S28, 0, &V17656_309_57247, &P17656_307};	/*Property = Fullscreen*/
const Value V17656_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_310 = {S29, 0, &V17656_310_57250, &P17656_309};	/*Property = GeoLocation*/
const Value V17656_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_312 = {S30, 0, &V17656_312_57256, &P17656_310};	/*Property = History*/
const Value V17656_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_314 = {S31, 0, &V17656_314_57262, &P17656_312};	/*Property = Html5*/
const Value V17656_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_313 = {S32, 0, &V17656_313_57259, &P17656_314};	/*Property = Html-Media-Capture*/
const Value V17656_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_315 = {S33, 0, &V17656_315_57265, &P17656_313};	/*Property = Iframe*/
const Value V17656_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_316 = {S34, 0, &V17656_316_57268, &P17656_315};	/*Property = IndexedDB*/
const Value V17656_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_317 = {S35, 0, &V17656_317_57271, &P17656_316};	/*Property = Json*/
const Value V17656_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_15 = {S37, 0, &V17656_15_72, &P17656_317};	/*Property = LayoutEngine*/
const Value V17656_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_344 = {S38, 0, &V17656_344_57274, &P17656_15};	/*Property = Masking*/
const Value V17656_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_320 = {S39, 0, &V17656_320_57292, &P17656_344};	/*Property = PostMessage*/
const Value V17656_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_321 = {S40, 0, &V17656_321_57295, &P17656_320};	/*Property = Progress*/
const Value V17656_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_322 = {S41, 0, &V17656_322_57298, &P17656_321};	/*Property = Prompts*/
const Value V17656_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_323 = {S42, 0, &V17656_323_57301, &P17656_322};	/*Property = Selector*/
const Value V17656_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_324 = {S43, 0, &V17656_324_57307, &P17656_323};	/*Property = Svg*/
const Value V17656_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_327 = {S44, 0, &V17656_327_57316, &P17656_324};	/*Property = TouchEvents*/
const Value V17656_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_328 = {S45, 0, &V17656_328_57319, &P17656_327};	/*Property = Track*/
const Value V17656_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_329 = {S46, 0, &V17656_329_57325, &P17656_328};	/*Property = Video*/
const Value V17656_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_330 = {S47, 0, &V17656_330_57328, &P17656_329};	/*Property = Viewport*/
const Value V17656_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_332 = {S48, 0, &V17656_332_57340, &P17656_330};	/*Property = WebWorkers*/
const Value V17656_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17656_333 = {S49, 0, &V17656_333_57343, &P17656_332};	/*Property = Xhr2*/
const Profile P17656 = {"17656", &P17656_333};	/*Profile ID - 17656*/
const Value V19997_1_1 = {S2, NULL};	/*Value = True*/
const Property P19997_1 = {S50, 0, &V19997_1_1, NULL};	/*Property = IsMobile*/
const Value V19997_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19997_5 = {S52, 0, &V19997_5_1227, &P19997_1};	/*Property = ScreenPixelsHeight*/
const Value V19997_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19997_4 = {S54, 0, &V19997_4_1229, &P19997_5};	/*Property = ScreenPixelsWidth*/
const Profile P19997 = {"19997", &P19997_4};	/*Profile ID - 19997*/
const Value V17230_1_1 = {S2, NULL};	/*Value = True*/
const Property P17230_1 = {S50, 0, &V17230_1_1, NULL};	/*Property = IsMobile*/
const Value V17230_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17230_5 = {S52, 0, &V17230_5_1222, &P17230_1};	/*Property = ScreenPixelsHeight*/
const Value V17230_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17230_4 = {S54, 0, &V17230_4_1224, &P17230_5};	/*Property = ScreenPixelsWidth*/
const Profile P17230 = {"17230", &P17230_4};	/*Profile ID - 17230*/
const Value V16793_1_1 = {S2, NULL};	/*Value = True*/
const Property P16793_1 = {S50, 0, &V16793_1_1, NULL};	/*Property = IsMobile*/
const Value V16793_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16793_5 = {S52, 0, &V16793_5_1293, &P16793_1};	/*Property = ScreenPixelsHeight*/
const Value V16793_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16793_4 = {S54, 0, &V16793_4_1296, &P16793_5};	/*Property = ScreenPixelsWidth*/
const Profile P16793 = {"16793", &P16793_4};	/*Profile ID - 16793*/
const Value V17298_1_1 = {S2, NULL};	/*Value = True*/
const Property P17298_1 = {S50, 0, &V17298_1_1, NULL};	/*Property = IsMobile*/
const Value V17298_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17298_5 = {S52, 0, &V17298_5_1233, &P17298_1};	/*Property = ScreenPixelsHeight*/
const Value V17298_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17298_4 = {S54, 0, &V17298_4_1235, &P17298_5};	/*Property = ScreenPixelsWidth*/
const Profile P17298 = {"17298", &P17298_4};	/*Profile ID - 17298*/
const Value V17284_1_1 = {S2, NULL};	/*Value = True*/
const Property P17284_1 = {S50, 0, &V17284_1_1, NULL};	/*Property = IsMobile*/
const Value V17284_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17284_5 = {S52, 0, &V17284_5_1233, &P17284_1};	/*Property = ScreenPixelsHeight*/
const Value V17284_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17284_4 = {S54, 0, &V17284_4_1235, &P17284_5};	/*Property = ScreenPixelsWidth*/
const Profile P17284 = {"17284", &P17284_4};	/*Profile ID - 17284*/
const Value V17244_1_1 = {S2, NULL};	/*Value = True*/
const Property P17244_1 = {S50, 0, &V17244_1_1, NULL};	/*Property = IsMobile*/
const Value V17244_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17244_5 = {S52, 0, &V17244_5_1293, &P17244_1};	/*Property = ScreenPixelsHeight*/
const Value V17244_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17244_4 = {S54, 0, &V17244_4_1296, &P17244_5};	/*Property = ScreenPixelsWidth*/
const Profile P17244 = {"17244", &P17244_4};	/*Profile ID - 17244*/
const Value V17325_1_1 = {S2, NULL};	/*Value = True*/
const Property P17325_1 = {S50, 0, &V17325_1_1, NULL};	/*Property = IsMobile*/
const Value V17325_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17325_5 = {S52, 0, &V17325_5_1233, &P17325_1};	/*Property = ScreenPixelsHeight*/
const Value V17325_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17325_4 = {S54, 0, &V17325_4_1235, &P17325_5};	/*Property = ScreenPixelsWidth*/
const Profile P17325 = {"17325", &P17325_4};	/*Profile ID - 17325*/
const Value V17301_1_1 = {S2, NULL};	/*Value = True*/
const Property P17301_1 = {S50, 0, &V17301_1_1, NULL};	/*Property = IsMobile*/
const Value V17301_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17301_5 = {S52, 0, &V17301_5_1222, &P17301_1};	/*Property = ScreenPixelsHeight*/
const Value V17301_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17301_4 = {S54, 0, &V17301_4_1224, &P17301_5};	/*Property = ScreenPixelsWidth*/
const Profile P17301 = {"17301", &P17301_4};	/*Profile ID - 17301*/
const Value V17268_1_1 = {S2, NULL};	/*Value = True*/
const Property P17268_1 = {S50, 0, &V17268_1_1, NULL};	/*Property = IsMobile*/
const Value V17268_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17268_5 = {S52, 0, &V17268_5_1233, &P17268_1};	/*Property = ScreenPixelsHeight*/
const Value V17268_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17268_4 = {S54, 0, &V17268_4_1235, &P17268_5};	/*Property = ScreenPixelsWidth*/
const Profile P17268 = {"17268", &P17268_4};	/*Profile ID - 17268*/
const Value V17466_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_279 = {S1, 0, &V17466_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17466_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_282 = {S3, 0, &V17466_282_57155, &P17466_279};	/*Property = BlobBuilder*/
const Value V17466_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_350 = {S4, 0, &V17466_350_57446, &P17466_282};	/*Property = Canvas*/
const Value V17466_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_283 = {S5, 0, &V17466_283_57161, &P17466_350};	/*Property = CssBackground*/
const Value V17466_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_284 = {S6, 0, &V17466_284_57164, &P17466_283};	/*Property = CssBorderImage*/
const Value V17466_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_285 = {S7, 0, &V17466_285_57167, &P17466_284};	/*Property = CssCanvas*/
const Value V17466_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_286 = {S8, 0, &V17466_286_57170, &P17466_285};	/*Property = CssColor*/
const Value V17466_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_237 = {S9, 0, &V17466_237_55786, &P17466_286};	/*Property = CssColumn*/
const Value V17466_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_287 = {S10, 0, &V17466_287_57173, &P17466_237};	/*Property = CssFlexbox*/
const Value V17466_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_288 = {S11, 0, &V17466_288_57176, &P17466_287};	/*Property = CssFont*/
const Value V17466_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_289 = {S12, 0, &V17466_289_86899, &P17466_288};	/*Property = CssImages*/
const Value V17466_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_291 = {S13, 0, &V17466_291_57182, &P17466_289};	/*Property = CssMediaQueries*/
const Value V17466_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_292 = {S14, 0, &V17466_292_57185, &P17466_291};	/*Property = CssMinMax*/
const Value V17466_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_293 = {S15, 0, &V17466_293_57188, &P17466_292};	/*Property = CssOverflow*/
const Value V17466_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_294 = {S16, 0, &V17466_294_57191, &P17466_293};	/*Property = CssPosition*/
const Value V17466_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_295 = {S17, 0, &V17466_295_57194, &P17466_294};	/*Property = CssText*/
const Value V17466_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_297 = {S18, 0, &V17466_297_57197, &P17466_295};	/*Property = CssTransforms*/
const Value V17466_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_241 = {S19, 0, &V17466_241_55801, &P17466_297};	/*Property = CssTransitions*/
const Value V17466_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_298 = {S20, 0, &V17466_298_57208, &P17466_241};	/*Property = CssUI*/
const Value V17466_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_300 = {S21, 0, &V17466_300_57217, &P17466_298};	/*Property = DataSet*/
const Value V17466_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_301 = {S22, 0, &V17466_301_57220, &P17466_300};	/*Property = DataUrl*/
const Value V17466_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_302 = {S23, 0, &V17466_302_57223, &P17466_301};	/*Property = DeviceOrientation*/
const Value V17466_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_304 = {S24, 0, &V17466_304_57229, &P17466_302};	/*Property = FileReader*/
const Value V17466_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_305 = {S25, 0, &V17466_305_57232, &P17466_304};	/*Property = FileSaver*/
const Value V17466_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_306 = {S26, 0, &V17466_306_57238, &P17466_305};	/*Property = FileWriter*/
const Value V17466_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_307 = {S27, 0, &V17466_307_57241, &P17466_306};	/*Property = FormData*/
const Value V17466_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_309 = {S28, 0, &V17466_309_57247, &P17466_307};	/*Property = Fullscreen*/
const Value V17466_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_310 = {S29, 0, &V17466_310_57250, &P17466_309};	/*Property = GeoLocation*/
const Value V17466_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_312 = {S30, 0, &V17466_312_57256, &P17466_310};	/*Property = History*/
const Value V17466_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_314 = {S31, 0, &V17466_314_57262, &P17466_312};	/*Property = Html5*/
const Value V17466_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_313 = {S32, 0, &V17466_313_57259, &P17466_314};	/*Property = Html-Media-Capture*/
const Value V17466_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_315 = {S33, 0, &V17466_315_57265, &P17466_313};	/*Property = Iframe*/
const Value V17466_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_316 = {S34, 0, &V17466_316_57268, &P17466_315};	/*Property = IndexedDB*/
const Value V17466_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_317 = {S35, 0, &V17466_317_57271, &P17466_316};	/*Property = Json*/
const Value V17466_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17466_15 = {S37, 0, &V17466_15_28, &P17466_317};	/*Property = LayoutEngine*/
const Value V17466_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_344 = {S38, 0, &V17466_344_57274, &P17466_15};	/*Property = Masking*/
const Value V17466_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_320 = {S39, 0, &V17466_320_57292, &P17466_344};	/*Property = PostMessage*/
const Value V17466_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_321 = {S40, 0, &V17466_321_57295, &P17466_320};	/*Property = Progress*/
const Value V17466_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_322 = {S41, 0, &V17466_322_57298, &P17466_321};	/*Property = Prompts*/
const Value V17466_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_323 = {S42, 0, &V17466_323_57301, &P17466_322};	/*Property = Selector*/
const Value V17466_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_324 = {S43, 0, &V17466_324_57307, &P17466_323};	/*Property = Svg*/
const Value V17466_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_327 = {S44, 0, &V17466_327_57316, &P17466_324};	/*Property = TouchEvents*/
const Value V17466_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_328 = {S45, 0, &V17466_328_57319, &P17466_327};	/*Property = Track*/
const Value V17466_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_329 = {S46, 0, &V17466_329_57325, &P17466_328};	/*Property = Video*/
const Value V17466_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_330 = {S47, 0, &V17466_330_57328, &P17466_329};	/*Property = Viewport*/
const Value V17466_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_332 = {S48, 0, &V17466_332_57340, &P17466_330};	/*Property = WebWorkers*/
const Value V17466_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17466_333 = {S49, 0, &V17466_333_57343, &P17466_332};	/*Property = Xhr2*/
const Profile P17466 = {"17466", &P17466_333};	/*Profile ID - 17466*/
const Value V20448_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_279 = {S1, 0, &V20448_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20448_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_282 = {S3, 0, &V20448_282_57155, &P20448_279};	/*Property = BlobBuilder*/
const Value V20448_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_350 = {S4, 0, &V20448_350_57446, &P20448_282};	/*Property = Canvas*/
const Value V20448_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_283 = {S5, 0, &V20448_283_57161, &P20448_350};	/*Property = CssBackground*/
const Value V20448_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_284 = {S6, 0, &V20448_284_57164, &P20448_283};	/*Property = CssBorderImage*/
const Value V20448_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_285 = {S7, 0, &V20448_285_57167, &P20448_284};	/*Property = CssCanvas*/
const Value V20448_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_286 = {S8, 0, &V20448_286_57170, &P20448_285};	/*Property = CssColor*/
const Value V20448_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_237 = {S9, 0, &V20448_237_55786, &P20448_286};	/*Property = CssColumn*/
const Value V20448_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_287 = {S10, 0, &V20448_287_57173, &P20448_237};	/*Property = CssFlexbox*/
const Value V20448_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_288 = {S11, 0, &V20448_288_57176, &P20448_287};	/*Property = CssFont*/
const Value V20448_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_289 = {S12, 0, &V20448_289_86899, &P20448_288};	/*Property = CssImages*/
const Value V20448_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_291 = {S13, 0, &V20448_291_57182, &P20448_289};	/*Property = CssMediaQueries*/
const Value V20448_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_292 = {S14, 0, &V20448_292_57185, &P20448_291};	/*Property = CssMinMax*/
const Value V20448_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_293 = {S15, 0, &V20448_293_57188, &P20448_292};	/*Property = CssOverflow*/
const Value V20448_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_294 = {S16, 0, &V20448_294_57191, &P20448_293};	/*Property = CssPosition*/
const Value V20448_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_295 = {S17, 0, &V20448_295_57194, &P20448_294};	/*Property = CssText*/
const Value V20448_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_297 = {S18, 0, &V20448_297_57197, &P20448_295};	/*Property = CssTransforms*/
const Value V20448_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_241 = {S19, 0, &V20448_241_55801, &P20448_297};	/*Property = CssTransitions*/
const Value V20448_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_298 = {S20, 0, &V20448_298_57208, &P20448_241};	/*Property = CssUI*/
const Value V20448_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_300 = {S21, 0, &V20448_300_57217, &P20448_298};	/*Property = DataSet*/
const Value V20448_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_301 = {S22, 0, &V20448_301_57220, &P20448_300};	/*Property = DataUrl*/
const Value V20448_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_302 = {S23, 0, &V20448_302_57223, &P20448_301};	/*Property = DeviceOrientation*/
const Value V20448_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_304 = {S24, 0, &V20448_304_57229, &P20448_302};	/*Property = FileReader*/
const Value V20448_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_305 = {S25, 0, &V20448_305_57232, &P20448_304};	/*Property = FileSaver*/
const Value V20448_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_306 = {S26, 0, &V20448_306_57238, &P20448_305};	/*Property = FileWriter*/
const Value V20448_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_307 = {S27, 0, &V20448_307_57241, &P20448_306};	/*Property = FormData*/
const Value V20448_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_309 = {S28, 0, &V20448_309_57247, &P20448_307};	/*Property = Fullscreen*/
const Value V20448_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_310 = {S29, 0, &V20448_310_57250, &P20448_309};	/*Property = GeoLocation*/
const Value V20448_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_312 = {S30, 0, &V20448_312_57256, &P20448_310};	/*Property = History*/
const Value V20448_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_314 = {S31, 0, &V20448_314_57262, &P20448_312};	/*Property = Html5*/
const Value V20448_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_313 = {S32, 0, &V20448_313_57259, &P20448_314};	/*Property = Html-Media-Capture*/
const Value V20448_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_315 = {S33, 0, &V20448_315_57265, &P20448_313};	/*Property = Iframe*/
const Value V20448_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_316 = {S34, 0, &V20448_316_57268, &P20448_315};	/*Property = IndexedDB*/
const Value V20448_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_317 = {S35, 0, &V20448_317_57271, &P20448_316};	/*Property = Json*/
const Value V20448_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P20448_15 = {S37, 0, &V20448_15_31, &P20448_317};	/*Property = LayoutEngine*/
const Value V20448_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_344 = {S38, 0, &V20448_344_57274, &P20448_15};	/*Property = Masking*/
const Value V20448_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_320 = {S39, 0, &V20448_320_57292, &P20448_344};	/*Property = PostMessage*/
const Value V20448_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_321 = {S40, 0, &V20448_321_57295, &P20448_320};	/*Property = Progress*/
const Value V20448_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_322 = {S41, 0, &V20448_322_57298, &P20448_321};	/*Property = Prompts*/
const Value V20448_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_323 = {S42, 0, &V20448_323_57301, &P20448_322};	/*Property = Selector*/
const Value V20448_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_324 = {S43, 0, &V20448_324_57307, &P20448_323};	/*Property = Svg*/
const Value V20448_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_327 = {S44, 0, &V20448_327_57316, &P20448_324};	/*Property = TouchEvents*/
const Value V20448_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_328 = {S45, 0, &V20448_328_57319, &P20448_327};	/*Property = Track*/
const Value V20448_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_329 = {S46, 0, &V20448_329_57325, &P20448_328};	/*Property = Video*/
const Value V20448_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_330 = {S47, 0, &V20448_330_57328, &P20448_329};	/*Property = Viewport*/
const Value V20448_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_332 = {S48, 0, &V20448_332_57340, &P20448_330};	/*Property = WebWorkers*/
const Value V20448_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20448_333 = {S49, 0, &V20448_333_57343, &P20448_332};	/*Property = Xhr2*/
const Profile P20448 = {"20448", &P20448_333};	/*Profile ID - 20448*/
const Value V20447_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_279 = {S1, 0, &V20447_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20447_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_282 = {S3, 0, &V20447_282_57155, &P20447_279};	/*Property = BlobBuilder*/
const Value V20447_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_350 = {S4, 0, &V20447_350_57446, &P20447_282};	/*Property = Canvas*/
const Value V20447_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_283 = {S5, 0, &V20447_283_57161, &P20447_350};	/*Property = CssBackground*/
const Value V20447_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_284 = {S6, 0, &V20447_284_57164, &P20447_283};	/*Property = CssBorderImage*/
const Value V20447_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_285 = {S7, 0, &V20447_285_57167, &P20447_284};	/*Property = CssCanvas*/
const Value V20447_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_286 = {S8, 0, &V20447_286_57170, &P20447_285};	/*Property = CssColor*/
const Value V20447_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_237 = {S9, 0, &V20447_237_55786, &P20447_286};	/*Property = CssColumn*/
const Value V20447_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_287 = {S10, 0, &V20447_287_57173, &P20447_237};	/*Property = CssFlexbox*/
const Value V20447_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_288 = {S11, 0, &V20447_288_57176, &P20447_287};	/*Property = CssFont*/
const Value V20447_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_289 = {S12, 0, &V20447_289_86899, &P20447_288};	/*Property = CssImages*/
const Value V20447_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_291 = {S13, 0, &V20447_291_57182, &P20447_289};	/*Property = CssMediaQueries*/
const Value V20447_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_292 = {S14, 0, &V20447_292_57185, &P20447_291};	/*Property = CssMinMax*/
const Value V20447_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_293 = {S15, 0, &V20447_293_57188, &P20447_292};	/*Property = CssOverflow*/
const Value V20447_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_294 = {S16, 0, &V20447_294_57191, &P20447_293};	/*Property = CssPosition*/
const Value V20447_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_295 = {S17, 0, &V20447_295_57194, &P20447_294};	/*Property = CssText*/
const Value V20447_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_297 = {S18, 0, &V20447_297_57197, &P20447_295};	/*Property = CssTransforms*/
const Value V20447_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_241 = {S19, 0, &V20447_241_55801, &P20447_297};	/*Property = CssTransitions*/
const Value V20447_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_298 = {S20, 0, &V20447_298_57208, &P20447_241};	/*Property = CssUI*/
const Value V20447_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_300 = {S21, 0, &V20447_300_57217, &P20447_298};	/*Property = DataSet*/
const Value V20447_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_301 = {S22, 0, &V20447_301_57220, &P20447_300};	/*Property = DataUrl*/
const Value V20447_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_302 = {S23, 0, &V20447_302_57223, &P20447_301};	/*Property = DeviceOrientation*/
const Value V20447_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_304 = {S24, 0, &V20447_304_57229, &P20447_302};	/*Property = FileReader*/
const Value V20447_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_305 = {S25, 0, &V20447_305_57232, &P20447_304};	/*Property = FileSaver*/
const Value V20447_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_306 = {S26, 0, &V20447_306_57238, &P20447_305};	/*Property = FileWriter*/
const Value V20447_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_307 = {S27, 0, &V20447_307_57241, &P20447_306};	/*Property = FormData*/
const Value V20447_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_309 = {S28, 0, &V20447_309_57247, &P20447_307};	/*Property = Fullscreen*/
const Value V20447_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_310 = {S29, 0, &V20447_310_57250, &P20447_309};	/*Property = GeoLocation*/
const Value V20447_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_312 = {S30, 0, &V20447_312_57256, &P20447_310};	/*Property = History*/
const Value V20447_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_314 = {S31, 0, &V20447_314_57262, &P20447_312};	/*Property = Html5*/
const Value V20447_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_313 = {S32, 0, &V20447_313_57259, &P20447_314};	/*Property = Html-Media-Capture*/
const Value V20447_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_315 = {S33, 0, &V20447_315_57265, &P20447_313};	/*Property = Iframe*/
const Value V20447_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_316 = {S34, 0, &V20447_316_57268, &P20447_315};	/*Property = IndexedDB*/
const Value V20447_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_317 = {S35, 0, &V20447_317_57271, &P20447_316};	/*Property = Json*/
const Value V20447_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P20447_15 = {S37, 0, &V20447_15_31, &P20447_317};	/*Property = LayoutEngine*/
const Value V20447_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_344 = {S38, 0, &V20447_344_57274, &P20447_15};	/*Property = Masking*/
const Value V20447_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_320 = {S39, 0, &V20447_320_57292, &P20447_344};	/*Property = PostMessage*/
const Value V20447_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_321 = {S40, 0, &V20447_321_57295, &P20447_320};	/*Property = Progress*/
const Value V20447_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_322 = {S41, 0, &V20447_322_57298, &P20447_321};	/*Property = Prompts*/
const Value V20447_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_323 = {S42, 0, &V20447_323_57301, &P20447_322};	/*Property = Selector*/
const Value V20447_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_324 = {S43, 0, &V20447_324_57307, &P20447_323};	/*Property = Svg*/
const Value V20447_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_327 = {S44, 0, &V20447_327_57316, &P20447_324};	/*Property = TouchEvents*/
const Value V20447_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_328 = {S45, 0, &V20447_328_57319, &P20447_327};	/*Property = Track*/
const Value V20447_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_329 = {S46, 0, &V20447_329_57325, &P20447_328};	/*Property = Video*/
const Value V20447_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_330 = {S47, 0, &V20447_330_57328, &P20447_329};	/*Property = Viewport*/
const Value V20447_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_332 = {S48, 0, &V20447_332_57340, &P20447_330};	/*Property = WebWorkers*/
const Value V20447_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20447_333 = {S49, 0, &V20447_333_57343, &P20447_332};	/*Property = Xhr2*/
const Profile P20447 = {"20447", &P20447_333};	/*Profile ID - 20447*/
const Value V18397_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_279 = {S1, 0, &V18397_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18397_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_282 = {S3, 0, &V18397_282_57155, &P18397_279};	/*Property = BlobBuilder*/
const Value V18397_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_350 = {S4, 0, &V18397_350_57446, &P18397_282};	/*Property = Canvas*/
const Value V18397_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_283 = {S5, 0, &V18397_283_57161, &P18397_350};	/*Property = CssBackground*/
const Value V18397_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_284 = {S6, 0, &V18397_284_57164, &P18397_283};	/*Property = CssBorderImage*/
const Value V18397_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_285 = {S7, 0, &V18397_285_57167, &P18397_284};	/*Property = CssCanvas*/
const Value V18397_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_286 = {S8, 0, &V18397_286_57170, &P18397_285};	/*Property = CssColor*/
const Value V18397_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_237 = {S9, 0, &V18397_237_55786, &P18397_286};	/*Property = CssColumn*/
const Value V18397_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_287 = {S10, 0, &V18397_287_57173, &P18397_237};	/*Property = CssFlexbox*/
const Value V18397_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_288 = {S11, 0, &V18397_288_57176, &P18397_287};	/*Property = CssFont*/
const Value V18397_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_289 = {S12, 0, &V18397_289_86899, &P18397_288};	/*Property = CssImages*/
const Value V18397_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_291 = {S13, 0, &V18397_291_57182, &P18397_289};	/*Property = CssMediaQueries*/
const Value V18397_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_292 = {S14, 0, &V18397_292_57185, &P18397_291};	/*Property = CssMinMax*/
const Value V18397_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_293 = {S15, 0, &V18397_293_57188, &P18397_292};	/*Property = CssOverflow*/
const Value V18397_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_294 = {S16, 0, &V18397_294_57191, &P18397_293};	/*Property = CssPosition*/
const Value V18397_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_295 = {S17, 0, &V18397_295_57194, &P18397_294};	/*Property = CssText*/
const Value V18397_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_297 = {S18, 0, &V18397_297_57197, &P18397_295};	/*Property = CssTransforms*/
const Value V18397_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_241 = {S19, 0, &V18397_241_55801, &P18397_297};	/*Property = CssTransitions*/
const Value V18397_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_298 = {S20, 0, &V18397_298_57208, &P18397_241};	/*Property = CssUI*/
const Value V18397_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_300 = {S21, 0, &V18397_300_57217, &P18397_298};	/*Property = DataSet*/
const Value V18397_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_301 = {S22, 0, &V18397_301_57220, &P18397_300};	/*Property = DataUrl*/
const Value V18397_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_302 = {S23, 0, &V18397_302_57223, &P18397_301};	/*Property = DeviceOrientation*/
const Value V18397_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_304 = {S24, 0, &V18397_304_57229, &P18397_302};	/*Property = FileReader*/
const Value V18397_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_305 = {S25, 0, &V18397_305_57232, &P18397_304};	/*Property = FileSaver*/
const Value V18397_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_306 = {S26, 0, &V18397_306_57238, &P18397_305};	/*Property = FileWriter*/
const Value V18397_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_307 = {S27, 0, &V18397_307_57241, &P18397_306};	/*Property = FormData*/
const Value V18397_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_309 = {S28, 0, &V18397_309_57247, &P18397_307};	/*Property = Fullscreen*/
const Value V18397_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_310 = {S29, 0, &V18397_310_57250, &P18397_309};	/*Property = GeoLocation*/
const Value V18397_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_312 = {S30, 0, &V18397_312_57256, &P18397_310};	/*Property = History*/
const Value V18397_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_314 = {S31, 0, &V18397_314_57262, &P18397_312};	/*Property = Html5*/
const Value V18397_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_313 = {S32, 0, &V18397_313_57259, &P18397_314};	/*Property = Html-Media-Capture*/
const Value V18397_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_315 = {S33, 0, &V18397_315_57265, &P18397_313};	/*Property = Iframe*/
const Value V18397_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_316 = {S34, 0, &V18397_316_57268, &P18397_315};	/*Property = IndexedDB*/
const Value V18397_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_317 = {S35, 0, &V18397_317_57271, &P18397_316};	/*Property = Json*/
const Value V18397_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18397_15 = {S37, 0, &V18397_15_28, &P18397_317};	/*Property = LayoutEngine*/
const Value V18397_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_344 = {S38, 0, &V18397_344_57274, &P18397_15};	/*Property = Masking*/
const Value V18397_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_320 = {S39, 0, &V18397_320_57292, &P18397_344};	/*Property = PostMessage*/
const Value V18397_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_321 = {S40, 0, &V18397_321_57295, &P18397_320};	/*Property = Progress*/
const Value V18397_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_322 = {S41, 0, &V18397_322_57298, &P18397_321};	/*Property = Prompts*/
const Value V18397_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_323 = {S42, 0, &V18397_323_57301, &P18397_322};	/*Property = Selector*/
const Value V18397_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_324 = {S43, 0, &V18397_324_57307, &P18397_323};	/*Property = Svg*/
const Value V18397_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_327 = {S44, 0, &V18397_327_57316, &P18397_324};	/*Property = TouchEvents*/
const Value V18397_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_328 = {S45, 0, &V18397_328_57319, &P18397_327};	/*Property = Track*/
const Value V18397_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_329 = {S46, 0, &V18397_329_57325, &P18397_328};	/*Property = Video*/
const Value V18397_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_330 = {S47, 0, &V18397_330_57328, &P18397_329};	/*Property = Viewport*/
const Value V18397_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_332 = {S48, 0, &V18397_332_57340, &P18397_330};	/*Property = WebWorkers*/
const Value V18397_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18397_333 = {S49, 0, &V18397_333_57343, &P18397_332};	/*Property = Xhr2*/
const Profile P18397 = {"18397", &P18397_333};	/*Profile ID - 18397*/
const Profile P18091 = {"18091", NULL};	/*Profile ID - 18091*/
const Value V16486_1_1 = {S2, NULL};	/*Value = True*/
const Property P16486_1 = {S50, 0, &V16486_1_1, NULL};	/*Property = IsMobile*/
const Value V16486_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16486_5 = {S52, 0, &V16486_5_1294, &P16486_1};	/*Property = ScreenPixelsHeight*/
const Value V16486_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16486_4 = {S54, 0, &V16486_4_1296, &P16486_5};	/*Property = ScreenPixelsWidth*/
const Profile P16486 = {"16486", &P16486_4};	/*Profile ID - 16486*/
const Value V17478_1_1 = {S2, NULL};	/*Value = True*/
const Property P17478_1 = {S50, 0, &V17478_1_1, NULL};	/*Property = IsMobile*/
const Value V17478_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17478_5 = {S52, 0, &V17478_5_1222, &P17478_1};	/*Property = ScreenPixelsHeight*/
const Value V17478_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17478_4 = {S54, 0, &V17478_4_1224, &P17478_5};	/*Property = ScreenPixelsWidth*/
const Profile P17478 = {"17478", &P17478_4};	/*Profile ID - 17478*/
const Value V14003_1_1 = {S2, NULL};	/*Value = True*/
const Property P14003_1 = {S50, 0, &V14003_1_1, NULL};	/*Property = IsMobile*/
const Value V14003_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14003_5 = {S52, 0, &V14003_5_1233, &P14003_1};	/*Property = ScreenPixelsHeight*/
const Value V14003_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14003_4 = {S54, 0, &V14003_4_1235, &P14003_5};	/*Property = ScreenPixelsWidth*/
const Profile P14003 = {"14003", &P14003_4};	/*Profile ID - 14003*/
const Value V16489_1_1 = {S2, NULL};	/*Value = True*/
const Property P16489_1 = {S50, 0, &V16489_1_1, NULL};	/*Property = IsMobile*/
const Value V16489_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P16489_5 = {S52, 0, &V16489_5_1275, &P16489_1};	/*Property = ScreenPixelsHeight*/
const Value V16489_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16489_4 = {S54, 0, &V16489_4_1229, &P16489_5};	/*Property = ScreenPixelsWidth*/
const Profile P16489 = {"16489", &P16489_4};	/*Profile ID - 16489*/
const Value V16227_1_1 = {S2, NULL};	/*Value = True*/
const Property P16227_1 = {S50, 0, &V16227_1_1, NULL};	/*Property = IsMobile*/
const Value V16227_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16227_5 = {S52, 0, &V16227_5_1222, &P16227_1};	/*Property = ScreenPixelsHeight*/
const Value V16227_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16227_4 = {S54, 0, &V16227_4_1224, &P16227_5};	/*Property = ScreenPixelsWidth*/
const Profile P16227 = {"16227", &P16227_4};	/*Profile ID - 16227*/
const Value V19914_1_1 = {S2, NULL};	/*Value = True*/
const Property P19914_1 = {S50, 0, &V19914_1_1, NULL};	/*Property = IsMobile*/
const Value V19914_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P19914_5 = {S52, 0, &V19914_5_1293, &P19914_1};	/*Property = ScreenPixelsHeight*/
const Value V19914_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P19914_4 = {S54, 0, &V19914_4_1296, &P19914_5};	/*Property = ScreenPixelsWidth*/
const Profile P19914 = {"19914", &P19914_4};	/*Profile ID - 19914*/
const Value V20243_1_1 = {S2, NULL};	/*Value = True*/
const Property P20243_1 = {S50, 0, &V20243_1_1, NULL};	/*Property = IsMobile*/
const Value V20243_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P20243_5 = {S52, 0, &V20243_5_1294, &P20243_1};	/*Property = ScreenPixelsHeight*/
const Value V20243_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P20243_4 = {S54, 0, &V20243_4_1296, &P20243_5};	/*Property = ScreenPixelsWidth*/
const Profile P20243 = {"20243", &P20243_4};	/*Profile ID - 20243*/
const Value V19927_1_1 = {S2, NULL};	/*Value = True*/
const Property P19927_1 = {S50, 0, &V19927_1_1, NULL};	/*Property = IsMobile*/
const Value V19927_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P19927_5 = {S52, 0, &V19927_5_1233, &P19927_1};	/*Property = ScreenPixelsHeight*/
const Value V19927_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P19927_4 = {S54, 0, &V19927_4_1235, &P19927_5};	/*Property = ScreenPixelsWidth*/
const Profile P19927 = {"19927", &P19927_4};	/*Profile ID - 19927*/
const Value V16548_1_1 = {S2, NULL};	/*Value = True*/
const Property P16548_1 = {S50, 0, &V16548_1_1, NULL};	/*Property = IsMobile*/
const Value V16548_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16548_5 = {S52, 0, &V16548_5_1293, &P16548_1};	/*Property = ScreenPixelsHeight*/
const Value V16548_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16548_4 = {S54, 0, &V16548_4_1296, &P16548_5};	/*Property = ScreenPixelsWidth*/
const Profile P16548 = {"16548", &P16548_4};	/*Profile ID - 16548*/
const Value V18108_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18108_279 = {S1, 0, &V18108_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18108_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18108_282 = {S3, 0, &V18108_282_57154, &P18108_279};	/*Property = BlobBuilder*/
const Value V18108_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18108_350 = {S4, 0, &V18108_350_57444, &P18108_282};	/*Property = Canvas*/
const Value V18108_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18108_283 = {S5, 0, &V18108_283_57159, &P18108_350};	/*Property = CssBackground*/
const Value V18108_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18108_284 = {S6, 0, &V18108_284_57163, &P18108_283};	/*Property = CssBorderImage*/
const Value V18108_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18108_285 = {S7, 0, &V18108_285_57166, &P18108_284};	/*Property = CssCanvas*/
const Value V18108_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18108_286 = {S8, 0, &V18108_286_57168, &P18108_285};	/*Property = CssColor*/
const Value V18108_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18108_237 = {S9, 0, &V18108_237_55786, &P18108_286};	/*Property = CssColumn*/
const Value V18108_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18108_287 = {S10, 0, &V18108_287_57172, &P18108_237};	/*Property = CssFlexbox*/
const Value V18108_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18108_288 = {S11, 0, &V18108_288_57175, &P18108_287};	/*Property = CssFont*/
const Value V18108_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18108_289 = {S12, 0, &V18108_289_57759, &P18108_288};	/*Property = CssImages*/
const Value V18108_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18108_291 = {S13, 0, &V18108_291_57180, &P18108_289};	/*Property = CssMediaQueries*/
const Value V18108_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18108_292 = {S14, 0, &V18108_292_57183, &P18108_291};	/*Property = CssMinMax*/
const Value V18108_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18108_293 = {S15, 0, &V18108_293_57187, &P18108_292};	/*Property = CssOverflow*/
const Value V18108_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18108_294 = {S16, 0, &V18108_294_57189, &P18108_293};	/*Property = CssPosition*/
const Value V18108_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18108_295 = {S17, 0, &V18108_295_57192, &P18108_294};	/*Property = CssText*/
const Value V18108_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18108_297 = {S18, 0, &V18108_297_57195, &P18108_295};	/*Property = CssTransforms*/
const Value V18108_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18108_241 = {S19, 0, &V18108_241_55799, &P18108_297};	/*Property = CssTransitions*/
const Value V18108_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18108_298 = {S20, 0, &V18108_298_57206, &P18108_241};	/*Property = CssUI*/
const Value V18108_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18108_300 = {S21, 0, &V18108_300_57216, &P18108_298};	/*Property = DataSet*/
const Value V18108_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18108_301 = {S22, 0, &V18108_301_57218, &P18108_300};	/*Property = DataUrl*/
const Value V18108_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18108_302 = {S23, 0, &V18108_302_57222, &P18108_301};	/*Property = DeviceOrientation*/
const Value V18108_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18108_304 = {S24, 0, &V18108_304_57227, &P18108_302};	/*Property = FileReader*/
const Value V18108_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18108_305 = {S25, 0, &V18108_305_57231, &P18108_304};	/*Property = FileSaver*/
const Value V18108_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18108_306 = {S26, 0, &V18108_306_57237, &P18108_305};	/*Property = FileWriter*/
const Value V18108_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18108_307 = {S27, 0, &V18108_307_57239, &P18108_306};	/*Property = FormData*/
const Value V18108_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18108_309 = {S28, 0, &V18108_309_57246, &P18108_307};	/*Property = Fullscreen*/
const Value V18108_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18108_310 = {S29, 0, &V18108_310_57248, &P18108_309};	/*Property = GeoLocation*/
const Value V18108_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18108_312 = {S30, 0, &V18108_312_57254, &P18108_310};	/*Property = History*/
const Value V18108_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18108_314 = {S31, 0, &V18108_314_57261, &P18108_312};	/*Property = Html5*/
const Value V18108_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18108_313 = {S32, 0, &V18108_313_57258, &P18108_314};	/*Property = Html-Media-Capture*/
const Value V18108_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18108_315 = {S33, 0, &V18108_315_57264, &P18108_313};	/*Property = Iframe*/
const Value V18108_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18108_316 = {S34, 0, &V18108_316_57267, &P18108_315};	/*Property = IndexedDB*/
const Value V18108_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18108_317 = {S35, 0, &V18108_317_57269, &P18108_316};	/*Property = Json*/
const Value V18108_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18108_15 = {S37, 0, &V18108_15_28, &P18108_317};	/*Property = LayoutEngine*/
const Value V18108_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18108_344 = {S38, 0, &V18108_344_57272, &P18108_15};	/*Property = Masking*/
const Value V18108_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18108_320 = {S39, 0, &V18108_320_57290, &P18108_344};	/*Property = PostMessage*/
const Value V18108_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18108_321 = {S40, 0, &V18108_321_57293, &P18108_320};	/*Property = Progress*/
const Value V18108_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18108_322 = {S41, 0, &V18108_322_57296, &P18108_321};	/*Property = Prompts*/
const Value V18108_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18108_323 = {S42, 0, &V18108_323_57299, &P18108_322};	/*Property = Selector*/
const Value V18108_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18108_324 = {S43, 0, &V18108_324_57306, &P18108_323};	/*Property = Svg*/
const Value V18108_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18108_327 = {S44, 0, &V18108_327_57315, &P18108_324};	/*Property = TouchEvents*/
const Value V18108_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18108_328 = {S45, 0, &V18108_328_57318, &P18108_327};	/*Property = Track*/
const Value V18108_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18108_329 = {S46, 0, &V18108_329_57323, &P18108_328};	/*Property = Video*/
const Value V18108_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18108_330 = {S47, 0, &V18108_330_57326, &P18108_329};	/*Property = Viewport*/
const Value V18108_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18108_332 = {S48, 0, &V18108_332_57339, &P18108_330};	/*Property = WebWorkers*/
const Value V18108_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18108_333 = {S49, 0, &V18108_333_57342, &P18108_332};	/*Property = Xhr2*/
const Profile P18108 = {"18108", &P18108_333};	/*Profile ID - 18108*/
const Value V17456_1_1 = {S2, NULL};	/*Value = True*/
const Property P17456_1 = {S50, 0, &V17456_1_1, NULL};	/*Property = IsMobile*/
const Value V17456_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17456_5 = {S52, 0, &V17456_5_1222, &P17456_1};	/*Property = ScreenPixelsHeight*/
const Value V17456_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17456_4 = {S54, 0, &V17456_4_1224, &P17456_5};	/*Property = ScreenPixelsWidth*/
const Profile P17456 = {"17456", &P17456_4};	/*Profile ID - 17456*/
const Value V20208_1_1 = {S2, NULL};	/*Value = True*/
const Property P20208_1 = {S50, 0, &V20208_1_1, NULL};	/*Property = IsMobile*/
const Value V20208_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20208_5 = {S52, 0, &V20208_5_1222, &P20208_1};	/*Property = ScreenPixelsHeight*/
const Value V20208_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20208_4 = {S54, 0, &V20208_4_1224, &P20208_5};	/*Property = ScreenPixelsWidth*/
const Profile P20208 = {"20208", &P20208_4};	/*Profile ID - 20208*/
const Profile P22740 = {"22740", NULL};	/*Profile ID - 22740*/
const Value V16238_1_1 = {S2, NULL};	/*Value = True*/
const Property P16238_1 = {S50, 0, &V16238_1_1, NULL};	/*Property = IsMobile*/
const Value V16238_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16238_5 = {S52, 0, &V16238_5_1222, &P16238_1};	/*Property = ScreenPixelsHeight*/
const Value V16238_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16238_4 = {S54, 0, &V16238_4_1224, &P16238_5};	/*Property = ScreenPixelsWidth*/
const Profile P16238 = {"16238", &P16238_4};	/*Profile ID - 16238*/
const Value V17426_1_1 = {S2, NULL};	/*Value = True*/
const Property P17426_1 = {S50, 0, &V17426_1_1, NULL};	/*Property = IsMobile*/
const Value V17426_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17426_5 = {S52, 0, &V17426_5_1222, &P17426_1};	/*Property = ScreenPixelsHeight*/
const Value V17426_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17426_4 = {S54, 0, &V17426_4_1224, &P17426_5};	/*Property = ScreenPixelsWidth*/
const Profile P17426 = {"17426", &P17426_4};	/*Profile ID - 17426*/
const Value V11842_1_1 = {S2, NULL};	/*Value = True*/
const Property P11842_1 = {S50, 0, &V11842_1_1, NULL};	/*Property = IsMobile*/
const Value V11842_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P11842_5 = {S52, 0, &V11842_5_1293, &P11842_1};	/*Property = ScreenPixelsHeight*/
const Value V11842_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P11842_4 = {S54, 0, &V11842_4_1296, &P11842_5};	/*Property = ScreenPixelsWidth*/
const Profile P11842 = {"11842", &P11842_4};	/*Profile ID - 11842*/
const Value V16259_1_1 = {S2, NULL};	/*Value = True*/
const Property P16259_1 = {S50, 0, &V16259_1_1, NULL};	/*Property = IsMobile*/
const Value V16259_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16259_5 = {S52, 0, &V16259_5_1222, &P16259_1};	/*Property = ScreenPixelsHeight*/
const Value V16259_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16259_4 = {S54, 0, &V16259_4_1224, &P16259_5};	/*Property = ScreenPixelsWidth*/
const Profile P16259 = {"16259", &P16259_4};	/*Profile ID - 16259*/
const Value V14114_1_1 = {S2, NULL};	/*Value = True*/
const Property P14114_1 = {S50, 0, &V14114_1_1, NULL};	/*Property = IsMobile*/
const Value V14114_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P14114_5 = {S52, 0, &V14114_5_1227, &P14114_1};	/*Property = ScreenPixelsHeight*/
const Value V14114_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P14114_4 = {S54, 0, &V14114_4_1460, &P14114_5};	/*Property = ScreenPixelsWidth*/
const Profile P14114 = {"14114", &P14114_4};	/*Profile ID - 14114*/
const Value V17762_1_1 = {S2, NULL};	/*Value = True*/
const Property P17762_1 = {S50, 0, &V17762_1_1, NULL};	/*Property = IsMobile*/
const Value V17762_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17762_5 = {S52, 0, &V17762_5_1304, &P17762_1};	/*Property = ScreenPixelsHeight*/
const Value V17762_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17762_4 = {S54, 0, &V17762_4_1224, &P17762_5};	/*Property = ScreenPixelsWidth*/
const Profile P17762 = {"17762", &P17762_4};	/*Profile ID - 17762*/
const Value V16047_1_1 = {S2, NULL};	/*Value = True*/
const Property P16047_1 = {S50, 0, &V16047_1_1, NULL};	/*Property = IsMobile*/
const Value V16047_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16047_5 = {S52, 0, &V16047_5_1222, &P16047_1};	/*Property = ScreenPixelsHeight*/
const Value V16047_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16047_4 = {S54, 0, &V16047_4_1224, &P16047_5};	/*Property = ScreenPixelsWidth*/
const Profile P16047 = {"16047", &P16047_4};	/*Profile ID - 16047*/
const Value V16329_1_2 = {S0, NULL};	/*Value = False*/
const Property P16329_1 = {S50, 0, &V16329_1_2, NULL};	/*Property = IsMobile*/
const Value V16329_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P16329_5 = {S52, 0, &V16329_5_37, &P16329_1};	/*Property = ScreenPixelsHeight*/
const Value V16329_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P16329_4 = {S54, 0, &V16329_4_36, &P16329_5};	/*Property = ScreenPixelsWidth*/
const Profile P16329 = {"16329", &P16329_4};	/*Profile ID - 16329*/
const Profile P18178 = {"18178", NULL};	/*Profile ID - 18178*/
const Profile P18177 = {"18177", NULL};	/*Profile ID - 18177*/
const Value V17503_1_1 = {S2, NULL};	/*Value = True*/
const Property P17503_1 = {S50, 0, &V17503_1_1, NULL};	/*Property = IsMobile*/
const Value V17503_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17503_5 = {S52, 0, &V17503_5_1227, &P17503_1};	/*Property = ScreenPixelsHeight*/
const Value V17503_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17503_4 = {S54, 0, &V17503_4_1229, &P17503_5};	/*Property = ScreenPixelsWidth*/
const Profile P17503 = {"17503", &P17503_4};	/*Profile ID - 17503*/
const Value V17118_1_1 = {S2, NULL};	/*Value = True*/
const Property P17118_1 = {S50, 0, &V17118_1_1, NULL};	/*Property = IsMobile*/
const Value V17118_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17118_5 = {S52, 0, &V17118_5_1293, &P17118_1};	/*Property = ScreenPixelsHeight*/
const Value V17118_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17118_4 = {S54, 0, &V17118_4_1296, &P17118_5};	/*Property = ScreenPixelsWidth*/
const Profile P17118 = {"17118", &P17118_4};	/*Profile ID - 17118*/
const Value V13394_1_1 = {S2, NULL};	/*Value = True*/
const Property P13394_1 = {S50, 0, &V13394_1_1, NULL};	/*Property = IsMobile*/
const Value V13394_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13394_5 = {S52, 0, &V13394_5_1293, &P13394_1};	/*Property = ScreenPixelsHeight*/
const Value V13394_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13394_4 = {S54, 0, &V13394_4_1296, &P13394_5};	/*Property = ScreenPixelsWidth*/
const Profile P13394 = {"13394", &P13394_4};	/*Profile ID - 13394*/
const Value V16951_1_1 = {S2, NULL};	/*Value = True*/
const Property P16951_1 = {S50, 0, &V16951_1_1, NULL};	/*Property = IsMobile*/
const Value V16951_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16951_5 = {S52, 0, &V16951_5_1222, &P16951_1};	/*Property = ScreenPixelsHeight*/
const Value V16951_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16951_4 = {S54, 0, &V16951_4_1279, &P16951_5};	/*Property = ScreenPixelsWidth*/
const Profile P16951 = {"16951", &P16951_4};	/*Profile ID - 16951*/
const Value V17877_1_1 = {S2, NULL};	/*Value = True*/
const Property P17877_1 = {S50, 0, &V17877_1_1, NULL};	/*Property = IsMobile*/
const Value V17877_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17877_5 = {S52, 0, &V17877_5_1304, &P17877_1};	/*Property = ScreenPixelsHeight*/
const Value V17877_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17877_4 = {S54, 0, &V17877_4_1224, &P17877_5};	/*Property = ScreenPixelsWidth*/
const Profile P17877 = {"17877", &P17877_4};	/*Profile ID - 17877*/
const Value V17630_1_1 = {S2, NULL};	/*Value = True*/
const Property P17630_1 = {S50, 0, &V17630_1_1, NULL};	/*Property = IsMobile*/
const Value V17630_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17630_5 = {S52, 0, &V17630_5_1304, &P17630_1};	/*Property = ScreenPixelsHeight*/
const Value V17630_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17630_4 = {S54, 0, &V17630_4_1224, &P17630_5};	/*Property = ScreenPixelsWidth*/
const Profile P17630 = {"17630", &P17630_4};	/*Profile ID - 17630*/
const Value V17631_1_1 = {S2, NULL};	/*Value = True*/
const Property P17631_1 = {S50, 0, &V17631_1_1, NULL};	/*Property = IsMobile*/
const Value V17631_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17631_5 = {S52, 0, &V17631_5_1304, &P17631_1};	/*Property = ScreenPixelsHeight*/
const Value V17631_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17631_4 = {S54, 0, &V17631_4_1224, &P17631_5};	/*Property = ScreenPixelsWidth*/
const Profile P17631 = {"17631", &P17631_4};	/*Profile ID - 17631*/
const Value V17504_1_1 = {S2, NULL};	/*Value = True*/
const Property P17504_1 = {S50, 0, &V17504_1_1, NULL};	/*Property = IsMobile*/
const Value V17504_5_37528 = {S116, NULL};	/*Value = 323*/
const Property P17504_5 = {S52, 0, &V17504_5_37528, &P17504_1};	/*Property = ScreenPixelsHeight*/
const Value V17504_4_37608 = {S117, NULL};	/*Value = 232*/
const Property P17504_4 = {S54, 0, &V17504_4_37608, &P17504_5};	/*Property = ScreenPixelsWidth*/
const Profile P17504 = {"17504", &P17504_4};	/*Profile ID - 17504*/
const Value V17499_1_1 = {S2, NULL};	/*Value = True*/
const Property P17499_1 = {S50, 0, &V17499_1_1, NULL};	/*Property = IsMobile*/
const Value V17499_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17499_5 = {S52, 0, &V17499_5_1304, &P17499_1};	/*Property = ScreenPixelsHeight*/
const Value V17499_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17499_4 = {S54, 0, &V17499_4_1224, &P17499_5};	/*Property = ScreenPixelsWidth*/
const Profile P17499 = {"17499", &P17499_4};	/*Profile ID - 17499*/
const Value V15182_1_1 = {S2, NULL};	/*Value = True*/
const Property P15182_1 = {S50, 0, &V15182_1_1, NULL};	/*Property = IsMobile*/
const Value V15182_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P15182_5 = {S52, 0, &V15182_5_1293, &P15182_1};	/*Property = ScreenPixelsHeight*/
const Value V15182_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P15182_4 = {S54, 0, &V15182_4_1296, &P15182_5};	/*Property = ScreenPixelsWidth*/
const Profile P15182 = {"15182", &P15182_4};	/*Profile ID - 15182*/
const Value V17530_1_1 = {S2, NULL};	/*Value = True*/
const Property P17530_1 = {S50, 0, &V17530_1_1, NULL};	/*Property = IsMobile*/
const Value V17530_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17530_5 = {S52, 0, &V17530_5_1304, &P17530_1};	/*Property = ScreenPixelsHeight*/
const Value V17530_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17530_4 = {S54, 0, &V17530_4_1224, &P17530_5};	/*Property = ScreenPixelsWidth*/
const Profile P17530 = {"17530", &P17530_4};	/*Profile ID - 17530*/
const Value V17592_1_1 = {S2, NULL};	/*Value = True*/
const Property P17592_1 = {S50, 0, &V17592_1_1, NULL};	/*Property = IsMobile*/
const Value V17592_5_4286 = {S73, NULL};	/*Value = 432*/
const Property P17592_5 = {S52, 0, &V17592_5_4286, &P17592_1};	/*Property = ScreenPixelsHeight*/
const Value V17592_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17592_4 = {S54, 0, &V17592_4_1224, &P17592_5};	/*Property = ScreenPixelsWidth*/
const Profile P17592 = {"17592", &P17592_4};	/*Profile ID - 17592*/
const Value V17641_1_1 = {S2, NULL};	/*Value = True*/
const Property P17641_1 = {S50, 0, &V17641_1_1, NULL};	/*Property = IsMobile*/
const Value V17641_5_4286 = {S73, NULL};	/*Value = 432*/
const Property P17641_5 = {S52, 0, &V17641_5_4286, &P17641_1};	/*Property = ScreenPixelsHeight*/
const Value V17641_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17641_4 = {S54, 0, &V17641_4_1224, &P17641_5};	/*Property = ScreenPixelsWidth*/
const Profile P17641 = {"17641", &P17641_4};	/*Profile ID - 17641*/
const Value V17593_1_1 = {S2, NULL};	/*Value = True*/
const Property P17593_1 = {S50, 0, &V17593_1_1, NULL};	/*Property = IsMobile*/
const Value V17593_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17593_5 = {S52, 0, &V17593_5_1304, &P17593_1};	/*Property = ScreenPixelsHeight*/
const Value V17593_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17593_4 = {S54, 0, &V17593_4_1224, &P17593_5};	/*Property = ScreenPixelsWidth*/
const Profile P17593 = {"17593", &P17593_4};	/*Profile ID - 17593*/
const Value V17533_1_1 = {S2, NULL};	/*Value = True*/
const Property P17533_1 = {S50, 0, &V17533_1_1, NULL};	/*Property = IsMobile*/
const Value V17533_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17533_5 = {S52, 0, &V17533_5_1304, &P17533_1};	/*Property = ScreenPixelsHeight*/
const Value V17533_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17533_4 = {S54, 0, &V17533_4_1224, &P17533_5};	/*Property = ScreenPixelsWidth*/
const Profile P17533 = {"17533", &P17533_4};	/*Profile ID - 17533*/
const Value V17523_1_1 = {S2, NULL};	/*Value = True*/
const Property P17523_1 = {S50, 0, &V17523_1_1, NULL};	/*Property = IsMobile*/
const Value V17523_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17523_5 = {S52, 0, &V17523_5_1304, &P17523_1};	/*Property = ScreenPixelsHeight*/
const Value V17523_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17523_4 = {S54, 0, &V17523_4_1224, &P17523_5};	/*Property = ScreenPixelsWidth*/
const Profile P17523 = {"17523", &P17523_4};	/*Profile ID - 17523*/
const Value V17862_1_1 = {S2, NULL};	/*Value = True*/
const Property P17862_1 = {S50, 0, &V17862_1_1, NULL};	/*Property = IsMobile*/
const Value V17862_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17862_5 = {S52, 0, &V17862_5_1227, &P17862_1};	/*Property = ScreenPixelsHeight*/
const Value V17862_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17862_4 = {S54, 0, &V17862_4_1229, &P17862_5};	/*Property = ScreenPixelsWidth*/
const Profile P17862 = {"17862", &P17862_4};	/*Profile ID - 17862*/
const Value V17527_1_1 = {S2, NULL};	/*Value = True*/
const Property P17527_1 = {S50, 0, &V17527_1_1, NULL};	/*Property = IsMobile*/
const Value V17527_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17527_5 = {S52, 0, &V17527_5_1304, &P17527_1};	/*Property = ScreenPixelsHeight*/
const Value V17527_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17527_4 = {S54, 0, &V17527_4_1224, &P17527_5};	/*Property = ScreenPixelsWidth*/
const Profile P17527 = {"17527", &P17527_4};	/*Profile ID - 17527*/
const Value V17589_1_1 = {S2, NULL};	/*Value = True*/
const Property P17589_1 = {S50, 0, &V17589_1_1, NULL};	/*Property = IsMobile*/
const Value V17589_5_4349 = {S90, NULL};	/*Value = 345*/
const Property P17589_5 = {S52, 0, &V17589_5_4349, &P17589_1};	/*Property = ScreenPixelsHeight*/
const Value V17589_4_54625 = {S118, NULL};	/*Value = 228*/
const Property P17589_4 = {S54, 0, &V17589_4_54625, &P17589_5};	/*Property = ScreenPixelsWidth*/
const Profile P17589 = {"17589", &P17589_4};	/*Profile ID - 17589*/
const Value V17642_1_1 = {S2, NULL};	/*Value = True*/
const Property P17642_1 = {S50, 0, &V17642_1_1, NULL};	/*Property = IsMobile*/
const Value V17642_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17642_5 = {S52, 0, &V17642_5_1222, &P17642_1};	/*Property = ScreenPixelsHeight*/
const Value V17642_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17642_4 = {S54, 0, &V17642_4_1224, &P17642_5};	/*Property = ScreenPixelsWidth*/
const Profile P17642 = {"17642", &P17642_4};	/*Profile ID - 17642*/
const Value V17705_1_1 = {S2, NULL};	/*Value = True*/
const Property P17705_1 = {S50, 0, &V17705_1_1, NULL};	/*Property = IsMobile*/
const Value V17705_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17705_5 = {S52, 0, &V17705_5_1222, &P17705_1};	/*Property = ScreenPixelsHeight*/
const Value V17705_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17705_4 = {S54, 0, &V17705_4_1224, &P17705_5};	/*Property = ScreenPixelsWidth*/
const Profile P17705 = {"17705", &P17705_4};	/*Profile ID - 17705*/
const Value V17632_1_1 = {S2, NULL};	/*Value = True*/
const Property P17632_1 = {S50, 0, &V17632_1_1, NULL};	/*Property = IsMobile*/
const Value V17632_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17632_5 = {S52, 0, &V17632_5_1304, &P17632_1};	/*Property = ScreenPixelsHeight*/
const Value V17632_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17632_4 = {S54, 0, &V17632_4_1224, &P17632_5};	/*Property = ScreenPixelsWidth*/
const Profile P17632 = {"17632", &P17632_4};	/*Profile ID - 17632*/
const Value V17704_1_1 = {S2, NULL};	/*Value = True*/
const Property P17704_1 = {S50, 0, &V17704_1_1, NULL};	/*Property = IsMobile*/
const Value V17704_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17704_5 = {S52, 0, &V17704_5_1222, &P17704_1};	/*Property = ScreenPixelsHeight*/
const Value V17704_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17704_4 = {S54, 0, &V17704_4_1224, &P17704_5};	/*Property = ScreenPixelsWidth*/
const Profile P17704 = {"17704", &P17704_4};	/*Profile ID - 17704*/
const Value V17501_1_1 = {S2, NULL};	/*Value = True*/
const Property P17501_1 = {S50, 0, &V17501_1_1, NULL};	/*Property = IsMobile*/
const Value V17501_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17501_5 = {S52, 0, &V17501_5_1304, &P17501_1};	/*Property = ScreenPixelsHeight*/
const Value V17501_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17501_4 = {S54, 0, &V17501_4_1224, &P17501_5};	/*Property = ScreenPixelsWidth*/
const Profile P17501 = {"17501", &P17501_4};	/*Profile ID - 17501*/
const Value V17878_1_1 = {S2, NULL};	/*Value = True*/
const Property P17878_1 = {S50, 0, &V17878_1_1, NULL};	/*Property = IsMobile*/
const Value V17878_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17878_5 = {S52, 0, &V17878_5_1304, &P17878_1};	/*Property = ScreenPixelsHeight*/
const Value V17878_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17878_4 = {S54, 0, &V17878_4_1224, &P17878_5};	/*Property = ScreenPixelsWidth*/
const Profile P17878 = {"17878", &P17878_4};	/*Profile ID - 17878*/
const Value V17647_1_1 = {S2, NULL};	/*Value = True*/
const Property P17647_1 = {S50, 0, &V17647_1_1, NULL};	/*Property = IsMobile*/
const Value V17647_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17647_5 = {S52, 0, &V17647_5_1304, &P17647_1};	/*Property = ScreenPixelsHeight*/
const Value V17647_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17647_4 = {S54, 0, &V17647_4_1224, &P17647_5};	/*Property = ScreenPixelsWidth*/
const Profile P17647 = {"17647", &P17647_4};	/*Profile ID - 17647*/
const Value V17555_1_1 = {S2, NULL};	/*Value = True*/
const Property P17555_1 = {S50, 0, &V17555_1_1, NULL};	/*Property = IsMobile*/
const Value V17555_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17555_5 = {S52, 0, &V17555_5_1304, &P17555_1};	/*Property = ScreenPixelsHeight*/
const Value V17555_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17555_4 = {S54, 0, &V17555_4_1224, &P17555_5};	/*Property = ScreenPixelsWidth*/
const Profile P17555 = {"17555", &P17555_4};	/*Profile ID - 17555*/
const Value V16274_1_1 = {S2, NULL};	/*Value = True*/
const Property P16274_1 = {S50, 0, &V16274_1_1, NULL};	/*Property = IsMobile*/
const Value V16274_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16274_5 = {S52, 0, &V16274_5_1222, &P16274_1};	/*Property = ScreenPixelsHeight*/
const Value V16274_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16274_4 = {S54, 0, &V16274_4_1224, &P16274_5};	/*Property = ScreenPixelsWidth*/
const Profile P16274 = {"16274", &P16274_4};	/*Profile ID - 16274*/
const Value V16346_1_1 = {S2, NULL};	/*Value = True*/
const Property P16346_1 = {S50, 0, &V16346_1_1, NULL};	/*Property = IsMobile*/
const Value V16346_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P16346_5 = {S52, 0, &V16346_5_1380, &P16346_1};	/*Property = ScreenPixelsHeight*/
const Value V16346_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16346_4 = {S54, 0, &V16346_4_1279, &P16346_5};	/*Property = ScreenPixelsWidth*/
const Profile P16346 = {"16346", &P16346_4};	/*Profile ID - 16346*/
const Value V17525_1_1 = {S2, NULL};	/*Value = True*/
const Property P17525_1 = {S50, 0, &V17525_1_1, NULL};	/*Property = IsMobile*/
const Value V17525_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17525_5 = {S52, 0, &V17525_5_1304, &P17525_1};	/*Property = ScreenPixelsHeight*/
const Value V17525_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17525_4 = {S54, 0, &V17525_4_1224, &P17525_5};	/*Property = ScreenPixelsWidth*/
const Profile P17525 = {"17525", &P17525_4};	/*Profile ID - 17525*/
const Value V17498_1_1 = {S2, NULL};	/*Value = True*/
const Property P17498_1 = {S50, 0, &V17498_1_1, NULL};	/*Property = IsMobile*/
const Value V17498_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17498_5 = {S52, 0, &V17498_5_1304, &P17498_1};	/*Property = ScreenPixelsHeight*/
const Value V17498_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17498_4 = {S54, 0, &V17498_4_1224, &P17498_5};	/*Property = ScreenPixelsWidth*/
const Profile P17498 = {"17498", &P17498_4};	/*Profile ID - 17498*/
const Value V16492_1_1 = {S2, NULL};	/*Value = True*/
const Property P16492_1 = {S50, 0, &V16492_1_1, NULL};	/*Property = IsMobile*/
const Value V16492_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16492_5 = {S52, 0, &V16492_5_1222, &P16492_1};	/*Property = ScreenPixelsHeight*/
const Value V16492_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16492_4 = {S54, 0, &V16492_4_1224, &P16492_5};	/*Property = ScreenPixelsWidth*/
const Profile P16492 = {"16492", &P16492_4};	/*Profile ID - 16492*/
const Value V16231_1_1 = {S2, NULL};	/*Value = True*/
const Property P16231_1 = {S50, 0, &V16231_1_1, NULL};	/*Property = IsMobile*/
const Value V16231_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16231_5 = {S52, 0, &V16231_5_1222, &P16231_1};	/*Property = ScreenPixelsHeight*/
const Value V16231_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16231_4 = {S54, 0, &V16231_4_1224, &P16231_5};	/*Property = ScreenPixelsWidth*/
const Profile P16231 = {"16231", &P16231_4};	/*Profile ID - 16231*/
const Value V16262_1_1 = {S2, NULL};	/*Value = True*/
const Property P16262_1 = {S50, 0, &V16262_1_1, NULL};	/*Property = IsMobile*/
const Value V16262_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16262_5 = {S52, 0, &V16262_5_1233, &P16262_1};	/*Property = ScreenPixelsHeight*/
const Value V16262_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16262_4 = {S54, 0, &V16262_4_1235, &P16262_5};	/*Property = ScreenPixelsWidth*/
const Profile P16262 = {"16262", &P16262_4};	/*Profile ID - 16262*/
const Value V14105_1_1 = {S2, NULL};	/*Value = True*/
const Property P14105_1 = {S50, 0, &V14105_1_1, NULL};	/*Property = IsMobile*/
const Value V14105_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14105_5 = {S52, 0, &V14105_5_1222, &P14105_1};	/*Property = ScreenPixelsHeight*/
const Value V14105_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14105_4 = {S54, 0, &V14105_4_1224, &P14105_5};	/*Property = ScreenPixelsWidth*/
const Profile P14105 = {"14105", &P14105_4};	/*Profile ID - 14105*/
const Value V13837_1_1 = {S2, NULL};	/*Value = True*/
const Property P13837_1 = {S50, 0, &V13837_1_1, NULL};	/*Property = IsMobile*/
const Value V13837_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P13837_5 = {S52, 0, &V13837_5_1275, &P13837_1};	/*Property = ScreenPixelsHeight*/
const Value V13837_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13837_4 = {S54, 0, &V13837_4_1229, &P13837_5};	/*Property = ScreenPixelsWidth*/
const Profile P13837 = {"13837", &P13837_4};	/*Profile ID - 13837*/
const Value V16852_1_1 = {S2, NULL};	/*Value = True*/
const Property P16852_1 = {S50, 0, &V16852_1_1, NULL};	/*Property = IsMobile*/
const Value V16852_5_37463 = {S100, NULL};	/*Value = 64*/
const Property P16852_5 = {S52, 0, &V16852_5_37463, &P16852_1};	/*Property = ScreenPixelsHeight*/
const Value V16852_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16852_4 = {S54, 0, &V16852_4_1296, &P16852_5};	/*Property = ScreenPixelsWidth*/
const Profile P16852 = {"16852", &P16852_4};	/*Profile ID - 16852*/
const Value V16241_1_1 = {S2, NULL};	/*Value = True*/
const Property P16241_1 = {S50, 0, &V16241_1_1, NULL};	/*Property = IsMobile*/
const Value V16241_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P16241_5 = {S52, 0, &V16241_5_1275, &P16241_1};	/*Property = ScreenPixelsHeight*/
const Value V16241_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16241_4 = {S54, 0, &V16241_4_1229, &P16241_5};	/*Property = ScreenPixelsWidth*/
const Profile P16241 = {"16241", &P16241_4};	/*Profile ID - 16241*/
const Value V13572_1_1 = {S2, NULL};	/*Value = True*/
const Property P13572_1 = {S50, 0, &V13572_1_1, NULL};	/*Property = IsMobile*/
const Value V13572_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P13572_5 = {S52, 0, &V13572_5_1233, &P13572_1};	/*Property = ScreenPixelsHeight*/
const Value V13572_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P13572_4 = {S54, 0, &V13572_4_1235, &P13572_5};	/*Property = ScreenPixelsWidth*/
const Profile P13572 = {"13572", &P13572_4};	/*Profile ID - 13572*/
const Value V17100_1_1 = {S2, NULL};	/*Value = True*/
const Property P17100_1 = {S50, 0, &V17100_1_1, NULL};	/*Property = IsMobile*/
const Value V17100_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17100_5 = {S52, 0, &V17100_5_1293, &P17100_1};	/*Property = ScreenPixelsHeight*/
const Value V17100_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17100_4 = {S54, 0, &V17100_4_1296, &P17100_5};	/*Property = ScreenPixelsWidth*/
const Profile P17100 = {"17100", &P17100_4};	/*Profile ID - 17100*/
const Value V17619_1_1 = {S2, NULL};	/*Value = True*/
const Property P17619_1 = {S50, 0, &V17619_1_1, NULL};	/*Property = IsMobile*/
const Value V17619_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17619_5 = {S52, 0, &V17619_5_1233, &P17619_1};	/*Property = ScreenPixelsHeight*/
const Value V17619_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17619_4 = {S54, 0, &V17619_4_1235, &P17619_5};	/*Property = ScreenPixelsWidth*/
const Profile P17619 = {"17619", &P17619_4};	/*Profile ID - 17619*/
const Value V20101_1_2 = {S0, NULL};	/*Value = False*/
const Property P20101_1 = {S50, 0, &V20101_1_2, NULL};	/*Property = IsMobile*/
const Value V20101_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20101_5 = {S52, 0, &V20101_5_37, &P20101_1};	/*Property = ScreenPixelsHeight*/
const Value V20101_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20101_4 = {S54, 0, &V20101_4_36, &P20101_5};	/*Property = ScreenPixelsWidth*/
const Profile P20101 = {"20101", &P20101_4};	/*Profile ID - 20101*/
const Value V16483_1_1 = {S2, NULL};	/*Value = True*/
const Property P16483_1 = {S50, 0, &V16483_1_1, NULL};	/*Property = IsMobile*/
const Value V16483_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16483_5 = {S52, 0, &V16483_5_1222, &P16483_1};	/*Property = ScreenPixelsHeight*/
const Value V16483_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16483_4 = {S54, 0, &V16483_4_1224, &P16483_5};	/*Property = ScreenPixelsWidth*/
const Profile P16483 = {"16483", &P16483_4};	/*Profile ID - 16483*/
const Value V17804_1_1 = {S2, NULL};	/*Value = True*/
const Property P17804_1 = {S50, 0, &V17804_1_1, NULL};	/*Property = IsMobile*/
const Value V17804_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17804_5 = {S52, 0, &V17804_5_1233, &P17804_1};	/*Property = ScreenPixelsHeight*/
const Value V17804_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17804_4 = {S54, 0, &V17804_4_1235, &P17804_5};	/*Property = ScreenPixelsWidth*/
const Profile P17804 = {"17804", &P17804_4};	/*Profile ID - 17804*/
const Value V17688_1_1 = {S2, NULL};	/*Value = True*/
const Property P17688_1 = {S50, 0, &V17688_1_1, NULL};	/*Property = IsMobile*/
const Value V17688_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P17688_5 = {S52, 0, &V17688_5_1380, &P17688_1};	/*Property = ScreenPixelsHeight*/
const Value V17688_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17688_4 = {S54, 0, &V17688_4_1279, &P17688_5};	/*Property = ScreenPixelsWidth*/
const Profile P17688 = {"17688", &P17688_4};	/*Profile ID - 17688*/
const Value V17857_1_1 = {S2, NULL};	/*Value = True*/
const Property P17857_1 = {S50, 0, &V17857_1_1, NULL};	/*Property = IsMobile*/
const Value V17857_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17857_5 = {S52, 0, &V17857_5_1222, &P17857_1};	/*Property = ScreenPixelsHeight*/
const Value V17857_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17857_4 = {S54, 0, &V17857_4_1224, &P17857_5};	/*Property = ScreenPixelsWidth*/
const Profile P17857 = {"17857", &P17857_4};	/*Profile ID - 17857*/
const Value V17661_1_1 = {S2, NULL};	/*Value = True*/
const Property P17661_1 = {S50, 0, &V17661_1_1, NULL};	/*Property = IsMobile*/
const Value V17661_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17661_5 = {S52, 0, &V17661_5_1269, &P17661_1};	/*Property = ScreenPixelsHeight*/
const Value V17661_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17661_4 = {S54, 0, &V17661_4_1229, &P17661_5};	/*Property = ScreenPixelsWidth*/
const Profile P17661 = {"17661", &P17661_4};	/*Profile ID - 17661*/
const Value V17521_1_1 = {S2, NULL};	/*Value = True*/
const Property P17521_1 = {S50, 0, &V17521_1_1, NULL};	/*Property = IsMobile*/
const Value V17521_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17521_5 = {S52, 0, &V17521_5_1304, &P17521_1};	/*Property = ScreenPixelsHeight*/
const Value V17521_4_54464 = {S119, NULL};	/*Value = 280*/
const Property P17521_4 = {S54, 0, &V17521_4_54464, &P17521_5};	/*Property = ScreenPixelsWidth*/
const Profile P17521 = {"17521", &P17521_4};	/*Profile ID - 17521*/
const Value V17885_1_1 = {S2, NULL};	/*Value = True*/
const Property P17885_1 = {S50, 0, &V17885_1_1, NULL};	/*Property = IsMobile*/
const Value V17885_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17885_5 = {S52, 0, &V17885_5_1222, &P17885_1};	/*Property = ScreenPixelsHeight*/
const Value V17885_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17885_4 = {S54, 0, &V17885_4_1235, &P17885_5};	/*Property = ScreenPixelsWidth*/
const Profile P17885 = {"17885", &P17885_4};	/*Profile ID - 17885*/
const Value V23740_1_1 = {S2, NULL};	/*Value = True*/
const Property P23740_1 = {S50, 0, &V23740_1_1, NULL};	/*Property = IsMobile*/
const Value V23740_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P23740_5 = {S52, 0, &V23740_5_1294, &P23740_1};	/*Property = ScreenPixelsHeight*/
const Value V23740_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P23740_4 = {S54, 0, &V23740_4_1296, &P23740_5};	/*Property = ScreenPixelsWidth*/
const Profile P23740 = {"23740", &P23740_4};	/*Profile ID - 23740*/
const Value V17791_1_1 = {S2, NULL};	/*Value = True*/
const Property P17791_1 = {S50, 0, &V17791_1_1, NULL};	/*Property = IsMobile*/
const Value V17791_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17791_5 = {S52, 0, &V17791_5_1277, &P17791_1};	/*Property = ScreenPixelsHeight*/
const Value V17791_4_37610 = {S69, NULL};	/*Value = 854*/
const Property P17791_4 = {S54, 0, &V17791_4_37610, &P17791_5};	/*Property = ScreenPixelsWidth*/
const Profile P17791 = {"17791", &P17791_4};	/*Profile ID - 17791*/
const Value V17793_1_1 = {S2, NULL};	/*Value = True*/
const Property P17793_1 = {S50, 0, &V17793_1_1, NULL};	/*Property = IsMobile*/
const Value V17793_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17793_5 = {S52, 0, &V17793_5_1304, &P17793_1};	/*Property = ScreenPixelsHeight*/
const Value V17793_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17793_4 = {S54, 0, &V17793_4_1224, &P17793_5};	/*Property = ScreenPixelsWidth*/
const Profile P17793 = {"17793", &P17793_4};	/*Profile ID - 17793*/
const Value V17651_1_1 = {S2, NULL};	/*Value = True*/
const Property P17651_1 = {S50, 0, &V17651_1_1, NULL};	/*Property = IsMobile*/
const Value V17651_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17651_5 = {S52, 0, &V17651_5_1222, &P17651_1};	/*Property = ScreenPixelsHeight*/
const Value V17651_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17651_4 = {S54, 0, &V17651_4_1224, &P17651_5};	/*Property = ScreenPixelsWidth*/
const Profile P17651 = {"17651", &P17651_4};	/*Profile ID - 17651*/
const Value V19921_1_1 = {S2, NULL};	/*Value = True*/
const Property P19921_1 = {S50, 0, &V19921_1_1, NULL};	/*Property = IsMobile*/
const Value V19921_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P19921_5 = {S52, 0, &V19921_5_1233, &P19921_1};	/*Property = ScreenPixelsHeight*/
const Value V19921_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P19921_4 = {S54, 0, &V19921_4_1235, &P19921_5};	/*Property = ScreenPixelsWidth*/
const Profile P19921 = {"19921", &P19921_4};	/*Profile ID - 19921*/
const Value V17785_1_1 = {S2, NULL};	/*Value = True*/
const Property P17785_1 = {S50, 0, &V17785_1_1, NULL};	/*Property = IsMobile*/
const Value V17785_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17785_5 = {S52, 0, &V17785_5_1222, &P17785_1};	/*Property = ScreenPixelsHeight*/
const Value V17785_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17785_4 = {S54, 0, &V17785_4_1224, &P17785_5};	/*Property = ScreenPixelsWidth*/
const Profile P17785 = {"17785", &P17785_4};	/*Profile ID - 17785*/
const Value V17126_1_1 = {S2, NULL};	/*Value = True*/
const Property P17126_1 = {S50, 0, &V17126_1_1, NULL};	/*Property = IsMobile*/
const Value V17126_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17126_5 = {S52, 0, &V17126_5_1293, &P17126_1};	/*Property = ScreenPixelsHeight*/
const Value V17126_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17126_4 = {S54, 0, &V17126_4_1296, &P17126_5};	/*Property = ScreenPixelsWidth*/
const Profile P17126 = {"17126", &P17126_4};	/*Profile ID - 17126*/
const Value V13491_1_1 = {S2, NULL};	/*Value = True*/
const Property P13491_1 = {S50, 0, &V13491_1_1, NULL};	/*Property = IsMobile*/
const Value V13491_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P13491_5 = {S52, 0, &V13491_5_1293, &P13491_1};	/*Property = ScreenPixelsHeight*/
const Value V13491_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P13491_4 = {S54, 0, &V13491_4_1296, &P13491_5};	/*Property = ScreenPixelsWidth*/
const Profile P13491 = {"13491", &P13491_4};	/*Profile ID - 13491*/
const Value V17061_1_1 = {S2, NULL};	/*Value = True*/
const Property P17061_1 = {S50, 0, &V17061_1_1, NULL};	/*Property = IsMobile*/
const Value V17061_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P17061_5 = {S52, 0, &V17061_5_1294, &P17061_1};	/*Property = ScreenPixelsHeight*/
const Value V17061_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17061_4 = {S54, 0, &V17061_4_1296, &P17061_5};	/*Property = ScreenPixelsWidth*/
const Profile P17061 = {"17061", &P17061_4};	/*Profile ID - 17061*/
const Value V17506_1_1 = {S2, NULL};	/*Value = True*/
const Property P17506_1 = {S50, 0, &V17506_1_1, NULL};	/*Property = IsMobile*/
const Value V17506_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17506_5 = {S52, 0, &V17506_5_1222, &P17506_1};	/*Property = ScreenPixelsHeight*/
const Value V17506_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17506_4 = {S54, 0, &V17506_4_1224, &P17506_5};	/*Property = ScreenPixelsWidth*/
const Profile P17506 = {"17506", &P17506_4};	/*Profile ID - 17506*/
const Value V17502_1_1 = {S2, NULL};	/*Value = True*/
const Property P17502_1 = {S50, 0, &V17502_1_1, NULL};	/*Property = IsMobile*/
const Value V17502_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17502_5 = {S52, 0, &V17502_5_1222, &P17502_1};	/*Property = ScreenPixelsHeight*/
const Value V17502_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17502_4 = {S54, 0, &V17502_4_1224, &P17502_5};	/*Property = ScreenPixelsWidth*/
const Profile P17502 = {"17502", &P17502_4};	/*Profile ID - 17502*/
const Value V17792_1_1 = {S2, NULL};	/*Value = True*/
const Property P17792_1 = {S50, 0, &V17792_1_1, NULL};	/*Property = IsMobile*/
const Value V17792_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17792_5 = {S52, 0, &V17792_5_1222, &P17792_1};	/*Property = ScreenPixelsHeight*/
const Value V17792_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17792_4 = {S54, 0, &V17792_4_1224, &P17792_5};	/*Property = ScreenPixelsWidth*/
const Profile P17792 = {"17792", &P17792_4};	/*Profile ID - 17792*/
const Value V17789_1_1 = {S2, NULL};	/*Value = True*/
const Property P17789_1 = {S50, 0, &V17789_1_1, NULL};	/*Property = IsMobile*/
const Value V17789_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17789_5 = {S52, 0, &V17789_5_1304, &P17789_1};	/*Property = ScreenPixelsHeight*/
const Value V17789_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17789_4 = {S54, 0, &V17789_4_1224, &P17789_5};	/*Property = ScreenPixelsWidth*/
const Profile P17789 = {"17789", &P17789_4};	/*Profile ID - 17789*/
const Value V17990_1_1 = {S2, NULL};	/*Value = True*/
const Property P17990_1 = {S50, 0, &V17990_1_1, NULL};	/*Property = IsMobile*/
const Value V17990_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P17990_5 = {S52, 0, &V17990_5_1294, &P17990_1};	/*Property = ScreenPixelsHeight*/
const Value V17990_4_36436 = {S120, NULL};	/*Value = 92*/
const Property P17990_4 = {S54, 0, &V17990_4_36436, &P17990_5};	/*Property = ScreenPixelsWidth*/
const Profile P17990 = {"17990", &P17990_4};	/*Profile ID - 17990*/
const Value V15245_1_1 = {S2, NULL};	/*Value = True*/
const Property P15245_1 = {S50, 0, &V15245_1_1, NULL};	/*Property = IsMobile*/
const Value V15245_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15245_5 = {S52, 0, &V15245_5_1222, &P15245_1};	/*Property = ScreenPixelsHeight*/
const Value V15245_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15245_4 = {S54, 0, &V15245_4_1224, &P15245_5};	/*Property = ScreenPixelsWidth*/
const Profile P15245 = {"15245", &P15245_4};	/*Profile ID - 15245*/
const Value V16995_1_1 = {S2, NULL};	/*Value = True*/
const Property P16995_1 = {S50, 0, &V16995_1_1, NULL};	/*Property = IsMobile*/
const Value V16995_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16995_5 = {S52, 0, &V16995_5_1294, &P16995_1};	/*Property = ScreenPixelsHeight*/
const Value V16995_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16995_4 = {S54, 0, &V16995_4_1296, &P16995_5};	/*Property = ScreenPixelsWidth*/
const Profile P16995 = {"16995", &P16995_4};	/*Profile ID - 16995*/
const Profile P22840 = {"22840", NULL};	/*Profile ID - 22840*/
const Value V20218_1_1 = {S2, NULL};	/*Value = True*/
const Property P20218_1 = {S50, 0, &V20218_1_1, NULL};	/*Property = IsMobile*/
const Value V20218_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P20218_5 = {S52, 0, &V20218_5_1293, &P20218_1};	/*Property = ScreenPixelsHeight*/
const Value V20218_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P20218_4 = {S54, 0, &V20218_4_1296, &P20218_5};	/*Property = ScreenPixelsWidth*/
const Profile P20218 = {"20218", &P20218_4};	/*Profile ID - 20218*/
const Value V20336_1_1 = {S2, NULL};	/*Value = True*/
const Property P20336_1 = {S50, 0, &V20336_1_1, NULL};	/*Property = IsMobile*/
const Value V20336_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20336_5 = {S52, 0, &V20336_5_1222, &P20336_1};	/*Property = ScreenPixelsHeight*/
const Value V20336_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20336_4 = {S54, 0, &V20336_4_1224, &P20336_5};	/*Property = ScreenPixelsWidth*/
const Profile P20336 = {"20336", &P20336_4};	/*Profile ID - 20336*/
const Value V17794_1_1 = {S2, NULL};	/*Value = True*/
const Property P17794_1 = {S50, 0, &V17794_1_1, NULL};	/*Property = IsMobile*/
const Value V17794_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17794_5 = {S52, 0, &V17794_5_1222, &P17794_1};	/*Property = ScreenPixelsHeight*/
const Value V17794_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17794_4 = {S54, 0, &V17794_4_1224, &P17794_5};	/*Property = ScreenPixelsWidth*/
const Profile P17794 = {"17794", &P17794_4};	/*Profile ID - 17794*/
const Value V17496_1_1 = {S2, NULL};	/*Value = True*/
const Property P17496_1 = {S50, 0, &V17496_1_1, NULL};	/*Property = IsMobile*/
const Value V17496_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17496_5 = {S52, 0, &V17496_5_1222, &P17496_1};	/*Property = ScreenPixelsHeight*/
const Value V17496_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17496_4 = {S54, 0, &V17496_4_1224, &P17496_5};	/*Property = ScreenPixelsWidth*/
const Profile P17496 = {"17496", &P17496_4};	/*Profile ID - 17496*/
const Value V17663_1_1 = {S2, NULL};	/*Value = True*/
const Property P17663_1 = {S50, 0, &V17663_1_1, NULL};	/*Property = IsMobile*/
const Value V17663_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17663_5 = {S52, 0, &V17663_5_1304, &P17663_1};	/*Property = ScreenPixelsHeight*/
const Value V17663_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17663_4 = {S54, 0, &V17663_4_1224, &P17663_5};	/*Property = ScreenPixelsWidth*/
const Profile P17663 = {"17663", &P17663_4};	/*Profile ID - 17663*/
const Value V17664_1_1 = {S2, NULL};	/*Value = True*/
const Property P17664_1 = {S50, 0, &V17664_1_1, NULL};	/*Property = IsMobile*/
const Value V17664_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17664_5 = {S52, 0, &V17664_5_1222, &P17664_1};	/*Property = ScreenPixelsHeight*/
const Value V17664_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17664_4 = {S54, 0, &V17664_4_1224, &P17664_5};	/*Property = ScreenPixelsWidth*/
const Profile P17664 = {"17664", &P17664_4};	/*Profile ID - 17664*/
const Value V17800_1_1 = {S2, NULL};	/*Value = True*/
const Property P17800_1 = {S50, 0, &V17800_1_1, NULL};	/*Property = IsMobile*/
const Value V17800_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17800_5 = {S52, 0, &V17800_5_1233, &P17800_1};	/*Property = ScreenPixelsHeight*/
const Value V17800_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17800_4 = {S54, 0, &V17800_4_1235, &P17800_5};	/*Property = ScreenPixelsWidth*/
const Profile P17800 = {"17800", &P17800_4};	/*Profile ID - 17800*/
const Value V16989_1_1 = {S2, NULL};	/*Value = True*/
const Property P16989_1 = {S50, 0, &V16989_1_1, NULL};	/*Property = IsMobile*/
const Value V16989_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16989_5 = {S52, 0, &V16989_5_1233, &P16989_1};	/*Property = ScreenPixelsHeight*/
const Value V16989_4_37606 = {S121, NULL};	/*Value = 177*/
const Property P16989_4 = {S54, 0, &V16989_4_37606, &P16989_5};	/*Property = ScreenPixelsWidth*/
const Profile P16989 = {"16989", &P16989_4};	/*Profile ID - 16989*/
const Value V16048_1_1 = {S2, NULL};	/*Value = True*/
const Property P16048_1 = {S50, 0, &V16048_1_1, NULL};	/*Property = IsMobile*/
const Value V16048_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16048_5 = {S52, 0, &V16048_5_1222, &P16048_1};	/*Property = ScreenPixelsHeight*/
const Value V16048_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16048_4 = {S54, 0, &V16048_4_1224, &P16048_5};	/*Property = ScreenPixelsWidth*/
const Profile P16048 = {"16048", &P16048_4};	/*Profile ID - 16048*/
const Value V17645_1_1 = {S2, NULL};	/*Value = True*/
const Property P17645_1 = {S50, 0, &V17645_1_1, NULL};	/*Property = IsMobile*/
const Value V17645_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17645_5 = {S52, 0, &V17645_5_1304, &P17645_1};	/*Property = ScreenPixelsHeight*/
const Value V17645_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17645_4 = {S54, 0, &V17645_4_1224, &P17645_5};	/*Property = ScreenPixelsWidth*/
const Profile P17645 = {"17645", &P17645_4};	/*Profile ID - 17645*/
const Value V17633_1_1 = {S2, NULL};	/*Value = True*/
const Property P17633_1 = {S50, 0, &V17633_1_1, NULL};	/*Property = IsMobile*/
const Value V17633_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17633_5 = {S52, 0, &V17633_5_1304, &P17633_1};	/*Property = ScreenPixelsHeight*/
const Value V17633_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17633_4 = {S54, 0, &V17633_4_1224, &P17633_5};	/*Property = ScreenPixelsWidth*/
const Profile P17633 = {"17633", &P17633_4};	/*Profile ID - 17633*/
const Value V17864_1_1 = {S2, NULL};	/*Value = True*/
const Property P17864_1 = {S50, 0, &V17864_1_1, NULL};	/*Property = IsMobile*/
const Value V17864_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17864_5 = {S52, 0, &V17864_5_1222, &P17864_1};	/*Property = ScreenPixelsHeight*/
const Value V17864_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17864_4 = {S54, 0, &V17864_4_1224, &P17864_5};	/*Property = ScreenPixelsWidth*/
const Profile P17864 = {"17864", &P17864_4};	/*Profile ID - 17864*/
const Value V17528_1_1 = {S2, NULL};	/*Value = True*/
const Property P17528_1 = {S50, 0, &V17528_1_1, NULL};	/*Property = IsMobile*/
const Value V17528_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17528_5 = {S52, 0, &V17528_5_1304, &P17528_1};	/*Property = ScreenPixelsHeight*/
const Value V17528_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17528_4 = {S54, 0, &V17528_4_1224, &P17528_5};	/*Property = ScreenPixelsWidth*/
const Profile P17528 = {"17528", &P17528_4};	/*Profile ID - 17528*/
const Value V17495_1_1 = {S2, NULL};	/*Value = True*/
const Property P17495_1 = {S50, 0, &V17495_1_1, NULL};	/*Property = IsMobile*/
const Value V17495_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17495_5 = {S52, 0, &V17495_5_1222, &P17495_1};	/*Property = ScreenPixelsHeight*/
const Value V17495_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17495_4 = {S54, 0, &V17495_4_1224, &P17495_5};	/*Property = ScreenPixelsWidth*/
const Profile P17495 = {"17495", &P17495_4};	/*Profile ID - 17495*/
const Value V17655_1_1 = {S2, NULL};	/*Value = True*/
const Property P17655_1 = {S50, 0, &V17655_1_1, NULL};	/*Property = IsMobile*/
const Value V17655_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P17655_5 = {S52, 0, &V17655_5_1275, &P17655_1};	/*Property = ScreenPixelsHeight*/
const Value V17655_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17655_4 = {S54, 0, &V17655_4_1229, &P17655_5};	/*Property = ScreenPixelsWidth*/
const Profile P17655 = {"17655", &P17655_4};	/*Profile ID - 17655*/
const Value V17657_1_1 = {S2, NULL};	/*Value = True*/
const Property P17657_1 = {S50, 0, &V17657_1_1, NULL};	/*Property = IsMobile*/
const Value V17657_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17657_5 = {S52, 0, &V17657_5_1222, &P17657_1};	/*Property = ScreenPixelsHeight*/
const Value V17657_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17657_4 = {S54, 0, &V17657_4_1224, &P17657_5};	/*Property = ScreenPixelsWidth*/
const Profile P17657 = {"17657", &P17657_4};	/*Profile ID - 17657*/
const Value V17526_1_1 = {S2, NULL};	/*Value = True*/
const Property P17526_1 = {S50, 0, &V17526_1_1, NULL};	/*Property = IsMobile*/
const Value V17526_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17526_5 = {S52, 0, &V17526_5_1304, &P17526_1};	/*Property = ScreenPixelsHeight*/
const Value V17526_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17526_4 = {S54, 0, &V17526_4_1224, &P17526_5};	/*Property = ScreenPixelsWidth*/
const Profile P17526 = {"17526", &P17526_4};	/*Profile ID - 17526*/
const Value V17529_1_1 = {S2, NULL};	/*Value = True*/
const Property P17529_1 = {S50, 0, &V17529_1_1, NULL};	/*Property = IsMobile*/
const Value V17529_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17529_5 = {S52, 0, &V17529_5_1304, &P17529_1};	/*Property = ScreenPixelsHeight*/
const Value V17529_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17529_4 = {S54, 0, &V17529_4_1224, &P17529_5};	/*Property = ScreenPixelsWidth*/
const Profile P17529 = {"17529", &P17529_4};	/*Profile ID - 17529*/
const Value V16070_1_1 = {S2, NULL};	/*Value = True*/
const Property P16070_1 = {S50, 0, &V16070_1_1, NULL};	/*Property = IsMobile*/
const Value V16070_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16070_5 = {S52, 0, &V16070_5_1222, &P16070_1};	/*Property = ScreenPixelsHeight*/
const Value V16070_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16070_4 = {S54, 0, &V16070_4_1224, &P16070_5};	/*Property = ScreenPixelsWidth*/
const Profile P16070 = {"16070", &P16070_4};	/*Profile ID - 16070*/
const Value V17747_1_1 = {S2, NULL};	/*Value = True*/
const Property P17747_1 = {S50, 0, &V17747_1_1, NULL};	/*Property = IsMobile*/
const Value V17747_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17747_5 = {S52, 0, &V17747_5_1269, &P17747_1};	/*Property = ScreenPixelsHeight*/
const Value V17747_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17747_4 = {S54, 0, &V17747_4_1229, &P17747_5};	/*Property = ScreenPixelsWidth*/
const Profile P17747 = {"17747", &P17747_4};	/*Profile ID - 17747*/
const Value V19923_1_1 = {S2, NULL};	/*Value = True*/
const Property P19923_1 = {S50, 0, &V19923_1_1, NULL};	/*Property = IsMobile*/
const Value V19923_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P19923_5 = {S52, 0, &V19923_5_1293, &P19923_1};	/*Property = ScreenPixelsHeight*/
const Value V19923_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P19923_4 = {S54, 0, &V19923_4_1296, &P19923_5};	/*Property = ScreenPixelsWidth*/
const Profile P19923 = {"19923", &P19923_4};	/*Profile ID - 19923*/
const Value V17626_1_1 = {S2, NULL};	/*Value = True*/
const Property P17626_1 = {S50, 0, &V17626_1_1, NULL};	/*Property = IsMobile*/
const Value V17626_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17626_5 = {S52, 0, &V17626_5_1227, &P17626_1};	/*Property = ScreenPixelsHeight*/
const Value V17626_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17626_4 = {S54, 0, &V17626_4_1229, &P17626_5};	/*Property = ScreenPixelsWidth*/
const Profile P17626 = {"17626", &P17626_4};	/*Profile ID - 17626*/
const Value V17575_1_1 = {S2, NULL};	/*Value = True*/
const Property P17575_1 = {S50, 0, &V17575_1_1, NULL};	/*Property = IsMobile*/
const Value V17575_5_4286 = {S73, NULL};	/*Value = 432*/
const Property P17575_5 = {S52, 0, &V17575_5_4286, &P17575_1};	/*Property = ScreenPixelsHeight*/
const Value V17575_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17575_4 = {S54, 0, &V17575_4_1224, &P17575_5};	/*Property = ScreenPixelsWidth*/
const Profile P17575 = {"17575", &P17575_4};	/*Profile ID - 17575*/
const Value V17556_1_1 = {S2, NULL};	/*Value = True*/
const Property P17556_1 = {S50, 0, &V17556_1_1, NULL};	/*Property = IsMobile*/
const Value V17556_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17556_5 = {S52, 0, &V17556_5_1304, &P17556_1};	/*Property = ScreenPixelsHeight*/
const Value V17556_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17556_4 = {S54, 0, &V17556_4_1224, &P17556_5};	/*Property = ScreenPixelsWidth*/
const Profile P17556 = {"17556", &P17556_4};	/*Profile ID - 17556*/
const Value V17464_1_1 = {S2, NULL};	/*Value = True*/
const Property P17464_1 = {S50, 0, &V17464_1_1, NULL};	/*Property = IsMobile*/
const Value V17464_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17464_5 = {S52, 0, &V17464_5_1233, &P17464_1};	/*Property = ScreenPixelsHeight*/
const Value V17464_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17464_4 = {S54, 0, &V17464_4_1235, &P17464_5};	/*Property = ScreenPixelsWidth*/
const Profile P17464 = {"17464", &P17464_4};	/*Profile ID - 17464*/
const Value V17534_1_1 = {S2, NULL};	/*Value = True*/
const Property P17534_1 = {S50, 0, &V17534_1_1, NULL};	/*Property = IsMobile*/
const Value V17534_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17534_5 = {S52, 0, &V17534_5_1304, &P17534_1};	/*Property = ScreenPixelsHeight*/
const Value V17534_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17534_4 = {S54, 0, &V17534_4_1224, &P17534_5};	/*Property = ScreenPixelsWidth*/
const Profile P17534 = {"17534", &P17534_4};	/*Profile ID - 17534*/
const Value V17490_1_2 = {S0, NULL};	/*Value = False*/
const Property P17490_1 = {S50, 0, &V17490_1_2, NULL};	/*Property = IsMobile*/
const Value V17490_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P17490_5 = {S52, 0, &V17490_5_37, &P17490_1};	/*Property = ScreenPixelsHeight*/
const Value V17490_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P17490_4 = {S54, 0, &V17490_4_36, &P17490_5};	/*Property = ScreenPixelsWidth*/
const Profile P17490 = {"17490", &P17490_4};	/*Profile ID - 17490*/
const Profile P18378 = {"18378", NULL};	/*Profile ID - 18378*/
const Value V18403_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_279 = {S1, 0, &V18403_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18403_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_282 = {S3, 0, &V18403_282_57155, &P18403_279};	/*Property = BlobBuilder*/
const Value V18403_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_350 = {S4, 0, &V18403_350_57446, &P18403_282};	/*Property = Canvas*/
const Value V18403_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_283 = {S5, 0, &V18403_283_57161, &P18403_350};	/*Property = CssBackground*/
const Value V18403_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_284 = {S6, 0, &V18403_284_57164, &P18403_283};	/*Property = CssBorderImage*/
const Value V18403_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_285 = {S7, 0, &V18403_285_57167, &P18403_284};	/*Property = CssCanvas*/
const Value V18403_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_286 = {S8, 0, &V18403_286_57170, &P18403_285};	/*Property = CssColor*/
const Value V18403_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_237 = {S9, 0, &V18403_237_55786, &P18403_286};	/*Property = CssColumn*/
const Value V18403_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_287 = {S10, 0, &V18403_287_57173, &P18403_237};	/*Property = CssFlexbox*/
const Value V18403_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_288 = {S11, 0, &V18403_288_57176, &P18403_287};	/*Property = CssFont*/
const Value V18403_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_289 = {S12, 0, &V18403_289_86899, &P18403_288};	/*Property = CssImages*/
const Value V18403_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_291 = {S13, 0, &V18403_291_57182, &P18403_289};	/*Property = CssMediaQueries*/
const Value V18403_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_292 = {S14, 0, &V18403_292_57185, &P18403_291};	/*Property = CssMinMax*/
const Value V18403_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_293 = {S15, 0, &V18403_293_57188, &P18403_292};	/*Property = CssOverflow*/
const Value V18403_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_294 = {S16, 0, &V18403_294_57191, &P18403_293};	/*Property = CssPosition*/
const Value V18403_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_295 = {S17, 0, &V18403_295_57194, &P18403_294};	/*Property = CssText*/
const Value V18403_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_297 = {S18, 0, &V18403_297_57197, &P18403_295};	/*Property = CssTransforms*/
const Value V18403_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_241 = {S19, 0, &V18403_241_55801, &P18403_297};	/*Property = CssTransitions*/
const Value V18403_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_298 = {S20, 0, &V18403_298_57208, &P18403_241};	/*Property = CssUI*/
const Value V18403_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_300 = {S21, 0, &V18403_300_57217, &P18403_298};	/*Property = DataSet*/
const Value V18403_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_301 = {S22, 0, &V18403_301_57220, &P18403_300};	/*Property = DataUrl*/
const Value V18403_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_302 = {S23, 0, &V18403_302_57223, &P18403_301};	/*Property = DeviceOrientation*/
const Value V18403_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_304 = {S24, 0, &V18403_304_57229, &P18403_302};	/*Property = FileReader*/
const Value V18403_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_305 = {S25, 0, &V18403_305_57232, &P18403_304};	/*Property = FileSaver*/
const Value V18403_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_306 = {S26, 0, &V18403_306_57238, &P18403_305};	/*Property = FileWriter*/
const Value V18403_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_307 = {S27, 0, &V18403_307_57241, &P18403_306};	/*Property = FormData*/
const Value V18403_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_309 = {S28, 0, &V18403_309_57247, &P18403_307};	/*Property = Fullscreen*/
const Value V18403_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_310 = {S29, 0, &V18403_310_57250, &P18403_309};	/*Property = GeoLocation*/
const Value V18403_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_312 = {S30, 0, &V18403_312_57256, &P18403_310};	/*Property = History*/
const Value V18403_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_314 = {S31, 0, &V18403_314_57262, &P18403_312};	/*Property = Html5*/
const Value V18403_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_313 = {S32, 0, &V18403_313_57259, &P18403_314};	/*Property = Html-Media-Capture*/
const Value V18403_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_315 = {S33, 0, &V18403_315_57265, &P18403_313};	/*Property = Iframe*/
const Value V18403_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_316 = {S34, 0, &V18403_316_57268, &P18403_315};	/*Property = IndexedDB*/
const Value V18403_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_317 = {S35, 0, &V18403_317_57271, &P18403_316};	/*Property = Json*/
const Value V18403_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18403_15 = {S37, 0, &V18403_15_31, &P18403_317};	/*Property = LayoutEngine*/
const Value V18403_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_344 = {S38, 0, &V18403_344_57274, &P18403_15};	/*Property = Masking*/
const Value V18403_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_320 = {S39, 0, &V18403_320_57292, &P18403_344};	/*Property = PostMessage*/
const Value V18403_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_321 = {S40, 0, &V18403_321_57295, &P18403_320};	/*Property = Progress*/
const Value V18403_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_322 = {S41, 0, &V18403_322_57298, &P18403_321};	/*Property = Prompts*/
const Value V18403_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_323 = {S42, 0, &V18403_323_57301, &P18403_322};	/*Property = Selector*/
const Value V18403_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_324 = {S43, 0, &V18403_324_57307, &P18403_323};	/*Property = Svg*/
const Value V18403_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_327 = {S44, 0, &V18403_327_57316, &P18403_324};	/*Property = TouchEvents*/
const Value V18403_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_328 = {S45, 0, &V18403_328_57319, &P18403_327};	/*Property = Track*/
const Value V18403_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_329 = {S46, 0, &V18403_329_57325, &P18403_328};	/*Property = Video*/
const Value V18403_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_330 = {S47, 0, &V18403_330_57328, &P18403_329};	/*Property = Viewport*/
const Value V18403_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_332 = {S48, 0, &V18403_332_57340, &P18403_330};	/*Property = WebWorkers*/
const Value V18403_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18403_333 = {S49, 0, &V18403_333_57343, &P18403_332};	/*Property = Xhr2*/
const Profile P18403 = {"18403", &P18403_333};	/*Profile ID - 18403*/
const Value V12499_1_1 = {S2, NULL};	/*Value = True*/
const Property P12499_1 = {S50, 0, &V12499_1_1, NULL};	/*Property = IsMobile*/
const Value V12499_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P12499_5 = {S52, 0, &V12499_5_1380, &P12499_1};	/*Property = ScreenPixelsHeight*/
const Value V12499_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12499_4 = {S54, 0, &V12499_4_1224, &P12499_5};	/*Property = ScreenPixelsWidth*/
const Profile P12499 = {"12499", &P12499_4};	/*Profile ID - 12499*/
const Value V16150_1_1 = {S2, NULL};	/*Value = True*/
const Property P16150_1 = {S50, 0, &V16150_1_1, NULL};	/*Property = IsMobile*/
const Value V16150_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16150_5 = {S52, 0, &V16150_5_1227, &P16150_1};	/*Property = ScreenPixelsHeight*/
const Value V16150_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P16150_4 = {S54, 0, &V16150_4_1460, &P16150_5};	/*Property = ScreenPixelsWidth*/
const Profile P16150 = {"16150", &P16150_4};	/*Profile ID - 16150*/
const Value V14418_1_1 = {S2, NULL};	/*Value = True*/
const Property P14418_1 = {S50, 0, &V14418_1_1, NULL};	/*Property = IsMobile*/
const Value V14418_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P14418_5 = {S52, 0, &V14418_5_1222, &P14418_1};	/*Property = ScreenPixelsHeight*/
const Value V14418_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14418_4 = {S54, 0, &V14418_4_1224, &P14418_5};	/*Property = ScreenPixelsWidth*/
const Profile P14418 = {"14418", &P14418_4};	/*Profile ID - 14418*/
const Value V17012_1_1 = {S2, NULL};	/*Value = True*/
const Property P17012_1 = {S50, 0, &V17012_1_1, NULL};	/*Property = IsMobile*/
const Value V17012_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17012_5 = {S52, 0, &V17012_5_1222, &P17012_1};	/*Property = ScreenPixelsHeight*/
const Value V17012_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17012_4 = {S54, 0, &V17012_4_1279, &P17012_5};	/*Property = ScreenPixelsWidth*/
const Profile P17012 = {"17012", &P17012_4};	/*Profile ID - 17012*/
const Value V19841_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_279 = {S1, 0, &V19841_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V19841_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_282 = {S3, 0, &V19841_282_57155, &P19841_279};	/*Property = BlobBuilder*/
const Value V19841_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_350 = {S4, 0, &V19841_350_57446, &P19841_282};	/*Property = Canvas*/
const Value V19841_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_283 = {S5, 0, &V19841_283_57161, &P19841_350};	/*Property = CssBackground*/
const Value V19841_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_284 = {S6, 0, &V19841_284_57164, &P19841_283};	/*Property = CssBorderImage*/
const Value V19841_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_285 = {S7, 0, &V19841_285_57167, &P19841_284};	/*Property = CssCanvas*/
const Value V19841_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_286 = {S8, 0, &V19841_286_57170, &P19841_285};	/*Property = CssColor*/
const Value V19841_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_237 = {S9, 0, &V19841_237_55786, &P19841_286};	/*Property = CssColumn*/
const Value V19841_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_287 = {S10, 0, &V19841_287_57173, &P19841_237};	/*Property = CssFlexbox*/
const Value V19841_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_288 = {S11, 0, &V19841_288_57176, &P19841_287};	/*Property = CssFont*/
const Value V19841_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_289 = {S12, 0, &V19841_289_86899, &P19841_288};	/*Property = CssImages*/
const Value V19841_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_291 = {S13, 0, &V19841_291_57182, &P19841_289};	/*Property = CssMediaQueries*/
const Value V19841_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_292 = {S14, 0, &V19841_292_57185, &P19841_291};	/*Property = CssMinMax*/
const Value V19841_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_293 = {S15, 0, &V19841_293_57188, &P19841_292};	/*Property = CssOverflow*/
const Value V19841_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_294 = {S16, 0, &V19841_294_57191, &P19841_293};	/*Property = CssPosition*/
const Value V19841_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_295 = {S17, 0, &V19841_295_57194, &P19841_294};	/*Property = CssText*/
const Value V19841_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_297 = {S18, 0, &V19841_297_57197, &P19841_295};	/*Property = CssTransforms*/
const Value V19841_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_241 = {S19, 0, &V19841_241_55801, &P19841_297};	/*Property = CssTransitions*/
const Value V19841_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_298 = {S20, 0, &V19841_298_57208, &P19841_241};	/*Property = CssUI*/
const Value V19841_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_300 = {S21, 0, &V19841_300_57217, &P19841_298};	/*Property = DataSet*/
const Value V19841_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_301 = {S22, 0, &V19841_301_57220, &P19841_300};	/*Property = DataUrl*/
const Value V19841_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_302 = {S23, 0, &V19841_302_57223, &P19841_301};	/*Property = DeviceOrientation*/
const Value V19841_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_304 = {S24, 0, &V19841_304_57229, &P19841_302};	/*Property = FileReader*/
const Value V19841_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_305 = {S25, 0, &V19841_305_57232, &P19841_304};	/*Property = FileSaver*/
const Value V19841_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_306 = {S26, 0, &V19841_306_57238, &P19841_305};	/*Property = FileWriter*/
const Value V19841_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_307 = {S27, 0, &V19841_307_57241, &P19841_306};	/*Property = FormData*/
const Value V19841_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_309 = {S28, 0, &V19841_309_57247, &P19841_307};	/*Property = Fullscreen*/
const Value V19841_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_310 = {S29, 0, &V19841_310_57250, &P19841_309};	/*Property = GeoLocation*/
const Value V19841_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_312 = {S30, 0, &V19841_312_57256, &P19841_310};	/*Property = History*/
const Value V19841_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_314 = {S31, 0, &V19841_314_57262, &P19841_312};	/*Property = Html5*/
const Value V19841_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_313 = {S32, 0, &V19841_313_57259, &P19841_314};	/*Property = Html-Media-Capture*/
const Value V19841_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_315 = {S33, 0, &V19841_315_57265, &P19841_313};	/*Property = Iframe*/
const Value V19841_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_316 = {S34, 0, &V19841_316_57268, &P19841_315};	/*Property = IndexedDB*/
const Value V19841_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_317 = {S35, 0, &V19841_317_57271, &P19841_316};	/*Property = Json*/
const Value V19841_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P19841_15 = {S37, 0, &V19841_15_31, &P19841_317};	/*Property = LayoutEngine*/
const Value V19841_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_344 = {S38, 0, &V19841_344_57274, &P19841_15};	/*Property = Masking*/
const Value V19841_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_320 = {S39, 0, &V19841_320_57292, &P19841_344};	/*Property = PostMessage*/
const Value V19841_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_321 = {S40, 0, &V19841_321_57295, &P19841_320};	/*Property = Progress*/
const Value V19841_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_322 = {S41, 0, &V19841_322_57298, &P19841_321};	/*Property = Prompts*/
const Value V19841_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_323 = {S42, 0, &V19841_323_57301, &P19841_322};	/*Property = Selector*/
const Value V19841_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_324 = {S43, 0, &V19841_324_57307, &P19841_323};	/*Property = Svg*/
const Value V19841_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_327 = {S44, 0, &V19841_327_57316, &P19841_324};	/*Property = TouchEvents*/
const Value V19841_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_328 = {S45, 0, &V19841_328_57319, &P19841_327};	/*Property = Track*/
const Value V19841_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_329 = {S46, 0, &V19841_329_57325, &P19841_328};	/*Property = Video*/
const Value V19841_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_330 = {S47, 0, &V19841_330_57328, &P19841_329};	/*Property = Viewport*/
const Value V19841_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_332 = {S48, 0, &V19841_332_57340, &P19841_330};	/*Property = WebWorkers*/
const Value V19841_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P19841_333 = {S49, 0, &V19841_333_57343, &P19841_332};	/*Property = Xhr2*/
const Profile P19841 = {"19841", &P19841_333};	/*Profile ID - 19841*/
const Value V20271_1_1 = {S2, NULL};	/*Value = True*/
const Property P20271_1 = {S50, 0, &V20271_1_1, NULL};	/*Property = IsMobile*/
const Value V20271_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20271_5 = {S52, 0, &V20271_5_1222, &P20271_1};	/*Property = ScreenPixelsHeight*/
const Value V20271_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20271_4 = {S54, 0, &V20271_4_1224, &P20271_5};	/*Property = ScreenPixelsWidth*/
const Profile P20271 = {"20271", &P20271_4};	/*Profile ID - 20271*/
const Value V17315_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17315_279 = {S1, 0, &V17315_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17315_282_57153 = {S2, NULL};	/*Value = True*/
const Property P17315_282 = {S3, 0, &V17315_282_57153, &P17315_279};	/*Property = BlobBuilder*/
const Value V17315_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17315_350 = {S4, 0, &V17315_350_57444, &P17315_282};	/*Property = Canvas*/
const Value V17315_283_57160 = {S0, NULL};	/*Value = False*/
const Property P17315_283 = {S5, 0, &V17315_283_57160, &P17315_350};	/*Property = CssBackground*/
const Value V17315_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17315_284 = {S6, 0, &V17315_284_57163, &P17315_283};	/*Property = CssBorderImage*/
const Value V17315_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17315_285 = {S7, 0, &V17315_285_57166, &P17315_284};	/*Property = CssCanvas*/
const Value V17315_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17315_286 = {S8, 0, &V17315_286_57168, &P17315_285};	/*Property = CssColor*/
const Value V17315_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17315_237 = {S9, 0, &V17315_237_55784, &P17315_286};	/*Property = CssColumn*/
const Value V17315_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17315_287 = {S10, 0, &V17315_287_57172, &P17315_237};	/*Property = CssFlexbox*/
const Value V17315_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17315_288 = {S11, 0, &V17315_288_57175, &P17315_287};	/*Property = CssFont*/
const Value V17315_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17315_289 = {S12, 0, &V17315_289_86899, &P17315_288};	/*Property = CssImages*/
const Value V17315_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17315_291 = {S13, 0, &V17315_291_57181, &P17315_289};	/*Property = CssMediaQueries*/
const Value V17315_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17315_292 = {S14, 0, &V17315_292_57183, &P17315_291};	/*Property = CssMinMax*/
const Value V17315_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17315_293 = {S15, 0, &V17315_293_57187, &P17315_292};	/*Property = CssOverflow*/
const Value V17315_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17315_294 = {S16, 0, &V17315_294_57189, &P17315_293};	/*Property = CssPosition*/
const Value V17315_295_57193 = {S0, NULL};	/*Value = False*/
const Property P17315_295 = {S17, 0, &V17315_295_57193, &P17315_294};	/*Property = CssText*/
const Value V17315_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17315_297 = {S18, 0, &V17315_297_57195, &P17315_295};	/*Property = CssTransforms*/
const Value V17315_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17315_241 = {S19, 0, &V17315_241_55799, &P17315_297};	/*Property = CssTransitions*/
const Value V17315_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17315_298 = {S20, 0, &V17315_298_57206, &P17315_241};	/*Property = CssUI*/
const Value V17315_300_57215 = {S2, NULL};	/*Value = True*/
const Property P17315_300 = {S21, 0, &V17315_300_57215, &P17315_298};	/*Property = DataSet*/
const Value V17315_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17315_301 = {S22, 0, &V17315_301_57218, &P17315_300};	/*Property = DataUrl*/
const Value V17315_302_57221 = {S2, NULL};	/*Value = True*/
const Property P17315_302 = {S23, 0, &V17315_302_57221, &P17315_301};	/*Property = DeviceOrientation*/
const Value V17315_304_57227 = {S2, NULL};	/*Value = True*/
const Property P17315_304 = {S24, 0, &V17315_304_57227, &P17315_302};	/*Property = FileReader*/
const Value V17315_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17315_305 = {S25, 0, &V17315_305_57231, &P17315_304};	/*Property = FileSaver*/
const Value V17315_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17315_306 = {S26, 0, &V17315_306_57237, &P17315_305};	/*Property = FileWriter*/
const Value V17315_307_57239 = {S2, NULL};	/*Value = True*/
const Property P17315_307 = {S27, 0, &V17315_307_57239, &P17315_306};	/*Property = FormData*/
const Value V17315_309_57245 = {S2, NULL};	/*Value = True*/
const Property P17315_309 = {S28, 0, &V17315_309_57245, &P17315_307};	/*Property = Fullscreen*/
const Value V17315_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17315_310 = {S29, 0, &V17315_310_57248, &P17315_309};	/*Property = GeoLocation*/
const Value V17315_312_57254 = {S2, NULL};	/*Value = True*/
const Property P17315_312 = {S30, 0, &V17315_312_57254, &P17315_310};	/*Property = History*/
const Value V17315_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17315_314 = {S31, 0, &V17315_314_57261, &P17315_312};	/*Property = Html5*/
const Value V17315_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17315_313 = {S32, 0, &V17315_313_57258, &P17315_314};	/*Property = Html-Media-Capture*/
const Value V17315_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17315_315 = {S33, 0, &V17315_315_57264, &P17315_313};	/*Property = Iframe*/
const Value V17315_316_57266 = {S2, NULL};	/*Value = True*/
const Property P17315_316 = {S34, 0, &V17315_316_57266, &P17315_315};	/*Property = IndexedDB*/
const Value V17315_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17315_317 = {S35, 0, &V17315_317_57269, &P17315_316};	/*Property = Json*/
const Value V17315_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17315_15 = {S37, 0, &V17315_15_28, &P17315_317};	/*Property = LayoutEngine*/
const Value V17315_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17315_344 = {S38, 0, &V17315_344_57272, &P17315_15};	/*Property = Masking*/
const Value V17315_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17315_320 = {S39, 0, &V17315_320_57290, &P17315_344};	/*Property = PostMessage*/
const Value V17315_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17315_321 = {S40, 0, &V17315_321_57293, &P17315_320};	/*Property = Progress*/
const Value V17315_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17315_322 = {S41, 0, &V17315_322_57296, &P17315_321};	/*Property = Prompts*/
const Value V17315_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17315_323 = {S42, 0, &V17315_323_57299, &P17315_322};	/*Property = Selector*/
const Value V17315_324_57305 = {S2, NULL};	/*Value = True*/
const Property P17315_324 = {S43, 0, &V17315_324_57305, &P17315_323};	/*Property = Svg*/
const Value V17315_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17315_327 = {S44, 0, &V17315_327_57315, &P17315_324};	/*Property = TouchEvents*/
const Value V17315_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17315_328 = {S45, 0, &V17315_328_57318, &P17315_327};	/*Property = Track*/
const Value V17315_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17315_329 = {S46, 0, &V17315_329_57323, &P17315_328};	/*Property = Video*/
const Value V17315_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17315_330 = {S47, 0, &V17315_330_57326, &P17315_329};	/*Property = Viewport*/
const Value V17315_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17315_332 = {S48, 0, &V17315_332_57339, &P17315_330};	/*Property = WebWorkers*/
const Value V17315_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17315_333 = {S49, 0, &V17315_333_57342, &P17315_332};	/*Property = Xhr2*/
const Profile P17315 = {"17315", &P17315_333};	/*Profile ID - 17315*/
const Value V17381_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_279 = {S1, 0, &V17381_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17381_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_282 = {S3, 0, &V17381_282_57155, &P17381_279};	/*Property = BlobBuilder*/
const Value V17381_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_350 = {S4, 0, &V17381_350_57446, &P17381_282};	/*Property = Canvas*/
const Value V17381_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_283 = {S5, 0, &V17381_283_57161, &P17381_350};	/*Property = CssBackground*/
const Value V17381_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_284 = {S6, 0, &V17381_284_57164, &P17381_283};	/*Property = CssBorderImage*/
const Value V17381_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_285 = {S7, 0, &V17381_285_57167, &P17381_284};	/*Property = CssCanvas*/
const Value V17381_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_286 = {S8, 0, &V17381_286_57170, &P17381_285};	/*Property = CssColor*/
const Value V17381_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_237 = {S9, 0, &V17381_237_55786, &P17381_286};	/*Property = CssColumn*/
const Value V17381_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_287 = {S10, 0, &V17381_287_57173, &P17381_237};	/*Property = CssFlexbox*/
const Value V17381_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_288 = {S11, 0, &V17381_288_57176, &P17381_287};	/*Property = CssFont*/
const Value V17381_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_289 = {S12, 0, &V17381_289_86899, &P17381_288};	/*Property = CssImages*/
const Value V17381_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_291 = {S13, 0, &V17381_291_57182, &P17381_289};	/*Property = CssMediaQueries*/
const Value V17381_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_292 = {S14, 0, &V17381_292_57185, &P17381_291};	/*Property = CssMinMax*/
const Value V17381_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_293 = {S15, 0, &V17381_293_57188, &P17381_292};	/*Property = CssOverflow*/
const Value V17381_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_294 = {S16, 0, &V17381_294_57191, &P17381_293};	/*Property = CssPosition*/
const Value V17381_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_295 = {S17, 0, &V17381_295_57194, &P17381_294};	/*Property = CssText*/
const Value V17381_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_297 = {S18, 0, &V17381_297_57197, &P17381_295};	/*Property = CssTransforms*/
const Value V17381_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_241 = {S19, 0, &V17381_241_55801, &P17381_297};	/*Property = CssTransitions*/
const Value V17381_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_298 = {S20, 0, &V17381_298_57208, &P17381_241};	/*Property = CssUI*/
const Value V17381_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_300 = {S21, 0, &V17381_300_57217, &P17381_298};	/*Property = DataSet*/
const Value V17381_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_301 = {S22, 0, &V17381_301_57220, &P17381_300};	/*Property = DataUrl*/
const Value V17381_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_302 = {S23, 0, &V17381_302_57223, &P17381_301};	/*Property = DeviceOrientation*/
const Value V17381_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_304 = {S24, 0, &V17381_304_57229, &P17381_302};	/*Property = FileReader*/
const Value V17381_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_305 = {S25, 0, &V17381_305_57232, &P17381_304};	/*Property = FileSaver*/
const Value V17381_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_306 = {S26, 0, &V17381_306_57238, &P17381_305};	/*Property = FileWriter*/
const Value V17381_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_307 = {S27, 0, &V17381_307_57241, &P17381_306};	/*Property = FormData*/
const Value V17381_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_309 = {S28, 0, &V17381_309_57247, &P17381_307};	/*Property = Fullscreen*/
const Value V17381_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_310 = {S29, 0, &V17381_310_57250, &P17381_309};	/*Property = GeoLocation*/
const Value V17381_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_312 = {S30, 0, &V17381_312_57256, &P17381_310};	/*Property = History*/
const Value V17381_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_314 = {S31, 0, &V17381_314_57262, &P17381_312};	/*Property = Html5*/
const Value V17381_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_313 = {S32, 0, &V17381_313_57259, &P17381_314};	/*Property = Html-Media-Capture*/
const Value V17381_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_315 = {S33, 0, &V17381_315_57265, &P17381_313};	/*Property = Iframe*/
const Value V17381_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_316 = {S34, 0, &V17381_316_57268, &P17381_315};	/*Property = IndexedDB*/
const Value V17381_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_317 = {S35, 0, &V17381_317_57271, &P17381_316};	/*Property = Json*/
const Value V17381_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P17381_15 = {S37, 0, &V17381_15_29, &P17381_317};	/*Property = LayoutEngine*/
const Value V17381_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_344 = {S38, 0, &V17381_344_57274, &P17381_15};	/*Property = Masking*/
const Value V17381_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_320 = {S39, 0, &V17381_320_57292, &P17381_344};	/*Property = PostMessage*/
const Value V17381_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_321 = {S40, 0, &V17381_321_57295, &P17381_320};	/*Property = Progress*/
const Value V17381_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_322 = {S41, 0, &V17381_322_57298, &P17381_321};	/*Property = Prompts*/
const Value V17381_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_323 = {S42, 0, &V17381_323_57301, &P17381_322};	/*Property = Selector*/
const Value V17381_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_324 = {S43, 0, &V17381_324_57307, &P17381_323};	/*Property = Svg*/
const Value V17381_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_327 = {S44, 0, &V17381_327_57316, &P17381_324};	/*Property = TouchEvents*/
const Value V17381_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_328 = {S45, 0, &V17381_328_57319, &P17381_327};	/*Property = Track*/
const Value V17381_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_329 = {S46, 0, &V17381_329_57325, &P17381_328};	/*Property = Video*/
const Value V17381_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_330 = {S47, 0, &V17381_330_57328, &P17381_329};	/*Property = Viewport*/
const Value V17381_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_332 = {S48, 0, &V17381_332_57340, &P17381_330};	/*Property = WebWorkers*/
const Value V17381_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17381_333 = {S49, 0, &V17381_333_57343, &P17381_332};	/*Property = Xhr2*/
const Profile P17381 = {"17381", &P17381_333};	/*Profile ID - 17381*/
const Value V17772_1_1 = {S2, NULL};	/*Value = True*/
const Property P17772_1 = {S50, 0, &V17772_1_1, NULL};	/*Property = IsMobile*/
const Value V17772_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17772_5 = {S52, 0, &V17772_5_1222, &P17772_1};	/*Property = ScreenPixelsHeight*/
const Value V17772_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17772_4 = {S54, 0, &V17772_4_1224, &P17772_5};	/*Property = ScreenPixelsWidth*/
const Profile P17772 = {"17772", &P17772_4};	/*Profile ID - 17772*/
const Value V16940_1_1 = {S2, NULL};	/*Value = True*/
const Property P16940_1 = {S50, 0, &V16940_1_1, NULL};	/*Property = IsMobile*/
const Value V16940_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16940_5 = {S52, 0, &V16940_5_1277, &P16940_1};	/*Property = ScreenPixelsHeight*/
const Value V16940_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16940_4 = {S54, 0, &V16940_4_1279, &P16940_5};	/*Property = ScreenPixelsWidth*/
const Profile P16940 = {"16940", &P16940_4};	/*Profile ID - 16940*/
const Value V18175_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18175_279 = {S1, 0, &V18175_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18175_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18175_282 = {S3, 0, &V18175_282_57154, &P18175_279};	/*Property = BlobBuilder*/
const Value V18175_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18175_350 = {S4, 0, &V18175_350_57444, &P18175_282};	/*Property = Canvas*/
const Value V18175_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18175_283 = {S5, 0, &V18175_283_57159, &P18175_350};	/*Property = CssBackground*/
const Value V18175_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18175_284 = {S6, 0, &V18175_284_57163, &P18175_283};	/*Property = CssBorderImage*/
const Value V18175_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18175_285 = {S7, 0, &V18175_285_57166, &P18175_284};	/*Property = CssCanvas*/
const Value V18175_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18175_286 = {S8, 0, &V18175_286_57168, &P18175_285};	/*Property = CssColor*/
const Value V18175_237_55785 = {S0, NULL};	/*Value = False*/
const Property P18175_237 = {S9, 0, &V18175_237_55785, &P18175_286};	/*Property = CssColumn*/
const Value V18175_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18175_287 = {S10, 0, &V18175_287_57172, &P18175_237};	/*Property = CssFlexbox*/
const Value V18175_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18175_288 = {S11, 0, &V18175_288_57175, &P18175_287};	/*Property = CssFont*/
const Value V18175_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18175_289 = {S12, 0, &V18175_289_57759, &P18175_288};	/*Property = CssImages*/
const Value V18175_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18175_291 = {S13, 0, &V18175_291_57181, &P18175_289};	/*Property = CssMediaQueries*/
const Value V18175_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18175_292 = {S14, 0, &V18175_292_57183, &P18175_291};	/*Property = CssMinMax*/
const Value V18175_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18175_293 = {S15, 0, &V18175_293_57187, &P18175_292};	/*Property = CssOverflow*/
const Value V18175_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18175_294 = {S16, 0, &V18175_294_57189, &P18175_293};	/*Property = CssPosition*/
const Value V18175_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18175_295 = {S17, 0, &V18175_295_57192, &P18175_294};	/*Property = CssText*/
const Value V18175_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18175_297 = {S18, 0, &V18175_297_57195, &P18175_295};	/*Property = CssTransforms*/
const Value V18175_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18175_241 = {S19, 0, &V18175_241_55799, &P18175_297};	/*Property = CssTransitions*/
const Value V18175_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18175_298 = {S20, 0, &V18175_298_57206, &P18175_241};	/*Property = CssUI*/
const Value V18175_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18175_300 = {S21, 0, &V18175_300_57216, &P18175_298};	/*Property = DataSet*/
const Value V18175_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18175_301 = {S22, 0, &V18175_301_57218, &P18175_300};	/*Property = DataUrl*/
const Value V18175_302_57221 = {S2, NULL};	/*Value = True*/
const Property P18175_302 = {S23, 0, &V18175_302_57221, &P18175_301};	/*Property = DeviceOrientation*/
const Value V18175_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18175_304 = {S24, 0, &V18175_304_57228, &P18175_302};	/*Property = FileReader*/
const Value V18175_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18175_305 = {S25, 0, &V18175_305_57231, &P18175_304};	/*Property = FileSaver*/
const Value V18175_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18175_306 = {S26, 0, &V18175_306_57237, &P18175_305};	/*Property = FileWriter*/
const Value V18175_307_57240 = {S0, NULL};	/*Value = False*/
const Property P18175_307 = {S27, 0, &V18175_307_57240, &P18175_306};	/*Property = FormData*/
const Value V18175_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18175_309 = {S28, 0, &V18175_309_57246, &P18175_307};	/*Property = Fullscreen*/
const Value V18175_310_57249 = {S0, NULL};	/*Value = False*/
const Property P18175_310 = {S29, 0, &V18175_310_57249, &P18175_309};	/*Property = GeoLocation*/
const Value V18175_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18175_312 = {S30, 0, &V18175_312_57255, &P18175_310};	/*Property = History*/
const Value V18175_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18175_314 = {S31, 0, &V18175_314_57261, &P18175_312};	/*Property = Html5*/
const Value V18175_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18175_313 = {S32, 0, &V18175_313_57258, &P18175_314};	/*Property = Html-Media-Capture*/
const Value V18175_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18175_315 = {S33, 0, &V18175_315_57264, &P18175_313};	/*Property = Iframe*/
const Value V18175_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18175_316 = {S34, 0, &V18175_316_57267, &P18175_315};	/*Property = IndexedDB*/
const Value V18175_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18175_317 = {S35, 0, &V18175_317_57269, &P18175_316};	/*Property = Json*/
const Value V18175_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18175_15 = {S37, 0, &V18175_15_28, &P18175_317};	/*Property = LayoutEngine*/
const Value V18175_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18175_344 = {S38, 0, &V18175_344_57272, &P18175_15};	/*Property = Masking*/
const Value V18175_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18175_320 = {S39, 0, &V18175_320_57290, &P18175_344};	/*Property = PostMessage*/
const Value V18175_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18175_321 = {S40, 0, &V18175_321_57293, &P18175_320};	/*Property = Progress*/
const Value V18175_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18175_322 = {S41, 0, &V18175_322_57296, &P18175_321};	/*Property = Prompts*/
const Value V18175_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18175_323 = {S42, 0, &V18175_323_57299, &P18175_322};	/*Property = Selector*/
const Value V18175_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18175_324 = {S43, 0, &V18175_324_57306, &P18175_323};	/*Property = Svg*/
const Value V18175_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18175_327 = {S44, 0, &V18175_327_57315, &P18175_324};	/*Property = TouchEvents*/
const Value V18175_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18175_328 = {S45, 0, &V18175_328_57318, &P18175_327};	/*Property = Track*/
const Value V18175_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18175_329 = {S46, 0, &V18175_329_57323, &P18175_328};	/*Property = Video*/
const Value V18175_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18175_330 = {S47, 0, &V18175_330_57326, &P18175_329};	/*Property = Viewport*/
const Value V18175_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18175_332 = {S48, 0, &V18175_332_57339, &P18175_330};	/*Property = WebWorkers*/
const Value V18175_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18175_333 = {S49, 0, &V18175_333_57342, &P18175_332};	/*Property = Xhr2*/
const Profile P18175 = {"18175", &P18175_333};	/*Profile ID - 18175*/
const Profile P18187 = {"18187", NULL};	/*Profile ID - 18187*/
const Value V17754_1_1 = {S2, NULL};	/*Value = True*/
const Property P17754_1 = {S50, 0, &V17754_1_1, NULL};	/*Property = IsMobile*/
const Value V17754_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17754_5 = {S52, 0, &V17754_5_1304, &P17754_1};	/*Property = ScreenPixelsHeight*/
const Value V17754_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17754_4 = {S54, 0, &V17754_4_1224, &P17754_5};	/*Property = ScreenPixelsWidth*/
const Profile P17754 = {"17754", &P17754_4};	/*Profile ID - 17754*/
const Value V17576_1_1 = {S2, NULL};	/*Value = True*/
const Property P17576_1 = {S50, 0, &V17576_1_1, NULL};	/*Property = IsMobile*/
const Value V17576_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17576_5 = {S52, 0, &V17576_5_1269, &P17576_1};	/*Property = ScreenPixelsHeight*/
const Value V17576_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17576_4 = {S54, 0, &V17576_4_1229, &P17576_5};	/*Property = ScreenPixelsWidth*/
const Profile P17576 = {"17576", &P17576_4};	/*Profile ID - 17576*/
const Value V17855_1_1 = {S2, NULL};	/*Value = True*/
const Property P17855_1 = {S50, 0, &V17855_1_1, NULL};	/*Property = IsMobile*/
const Value V17855_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17855_5 = {S52, 0, &V17855_5_1227, &P17855_1};	/*Property = ScreenPixelsHeight*/
const Value V17855_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17855_4 = {S54, 0, &V17855_4_1229, &P17855_5};	/*Property = ScreenPixelsWidth*/
const Profile P17855 = {"17855", &P17855_4};	/*Profile ID - 17855*/
const Value V17702_1_1 = {S2, NULL};	/*Value = True*/
const Property P17702_1 = {S50, 0, &V17702_1_1, NULL};	/*Property = IsMobile*/
const Value V17702_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17702_5 = {S52, 0, &V17702_5_1227, &P17702_1};	/*Property = ScreenPixelsHeight*/
const Value V17702_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17702_4 = {S54, 0, &V17702_4_1229, &P17702_5};	/*Property = ScreenPixelsWidth*/
const Profile P17702 = {"17702", &P17702_4};	/*Profile ID - 17702*/
const Value V17858_1_1 = {S2, NULL};	/*Value = True*/
const Property P17858_1 = {S50, 0, &V17858_1_1, NULL};	/*Property = IsMobile*/
const Value V17858_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17858_5 = {S52, 0, &V17858_5_1304, &P17858_1};	/*Property = ScreenPixelsHeight*/
const Value V17858_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17858_4 = {S54, 0, &V17858_4_1224, &P17858_5};	/*Property = ScreenPixelsWidth*/
const Profile P17858 = {"17858", &P17858_4};	/*Profile ID - 17858*/
const Value V16612_1_1 = {S2, NULL};	/*Value = True*/
const Property P16612_1 = {S50, 0, &V16612_1_1, NULL};	/*Property = IsMobile*/
const Value V16612_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16612_5 = {S52, 0, &V16612_5_1233, &P16612_1};	/*Property = ScreenPixelsHeight*/
const Value V16612_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16612_4 = {S54, 0, &V16612_4_1235, &P16612_5};	/*Property = ScreenPixelsWidth*/
const Profile P16612 = {"16612", &P16612_4};	/*Profile ID - 16612*/
const Value V17536_1_1 = {S2, NULL};	/*Value = True*/
const Property P17536_1 = {S50, 0, &V17536_1_1, NULL};	/*Property = IsMobile*/
const Value V17536_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17536_5 = {S52, 0, &V17536_5_1304, &P17536_1};	/*Property = ScreenPixelsHeight*/
const Value V17536_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17536_4 = {S54, 0, &V17536_4_1224, &P17536_5};	/*Property = ScreenPixelsWidth*/
const Profile P17536 = {"17536", &P17536_4};	/*Profile ID - 17536*/
const Value V17068_1_1 = {S2, NULL};	/*Value = True*/
const Property P17068_1 = {S50, 0, &V17068_1_1, NULL};	/*Property = IsMobile*/
const Value V17068_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17068_5 = {S52, 0, &V17068_5_1233, &P17068_1};	/*Property = ScreenPixelsHeight*/
const Value V17068_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17068_4 = {S54, 0, &V17068_4_1235, &P17068_5};	/*Property = ScreenPixelsWidth*/
const Profile P17068 = {"17068", &P17068_4};	/*Profile ID - 17068*/
const Value V16933_1_1 = {S2, NULL};	/*Value = True*/
const Property P16933_1 = {S50, 0, &V16933_1_1, NULL};	/*Property = IsMobile*/
const Value V16933_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16933_5 = {S52, 0, &V16933_5_1233, &P16933_1};	/*Property = ScreenPixelsHeight*/
const Value V16933_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16933_4 = {S54, 0, &V16933_4_1235, &P16933_5};	/*Property = ScreenPixelsWidth*/
const Profile P16933 = {"16933", &P16933_4};	/*Profile ID - 16933*/
const Value V17749_1_1 = {S2, NULL};	/*Value = True*/
const Property P17749_1 = {S50, 0, &V17749_1_1, NULL};	/*Property = IsMobile*/
const Value V17749_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17749_5 = {S52, 0, &V17749_5_1222, &P17749_1};	/*Property = ScreenPixelsHeight*/
const Value V17749_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17749_4 = {S54, 0, &V17749_4_1224, &P17749_5};	/*Property = ScreenPixelsWidth*/
const Profile P17749 = {"17749", &P17749_4};	/*Profile ID - 17749*/
const Value V16929_1_1 = {S2, NULL};	/*Value = True*/
const Property P16929_1 = {S50, 0, &V16929_1_1, NULL};	/*Property = IsMobile*/
const Value V16929_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16929_5 = {S52, 0, &V16929_5_1293, &P16929_1};	/*Property = ScreenPixelsHeight*/
const Value V16929_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16929_4 = {S54, 0, &V16929_4_1296, &P16929_5};	/*Property = ScreenPixelsWidth*/
const Profile P16929 = {"16929", &P16929_4};	/*Profile ID - 16929*/
const Value V16391_1_1 = {S2, NULL};	/*Value = True*/
const Property P16391_1 = {S50, 0, &V16391_1_1, NULL};	/*Property = IsMobile*/
const Value V16391_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16391_5 = {S52, 0, &V16391_5_1222, &P16391_1};	/*Property = ScreenPixelsHeight*/
const Value V16391_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16391_4 = {S54, 0, &V16391_4_1224, &P16391_5};	/*Property = ScreenPixelsWidth*/
const Profile P16391 = {"16391", &P16391_4};	/*Profile ID - 16391*/
const Value V17751_1_1 = {S2, NULL};	/*Value = True*/
const Property P17751_1 = {S50, 0, &V17751_1_1, NULL};	/*Property = IsMobile*/
const Value V17751_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17751_5 = {S52, 0, &V17751_5_1222, &P17751_1};	/*Property = ScreenPixelsHeight*/
const Value V17751_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17751_4 = {S54, 0, &V17751_4_1224, &P17751_5};	/*Property = ScreenPixelsWidth*/
const Profile P17751 = {"17751", &P17751_4};	/*Profile ID - 17751*/
const Value V16986_1_1 = {S2, NULL};	/*Value = True*/
const Property P16986_1 = {S50, 0, &V16986_1_1, NULL};	/*Property = IsMobile*/
const Value V16986_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16986_5 = {S52, 0, &V16986_5_1294, &P16986_1};	/*Property = ScreenPixelsHeight*/
const Value V16986_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16986_4 = {S54, 0, &V16986_4_1296, &P16986_5};	/*Property = ScreenPixelsWidth*/
const Profile P16986 = {"16986", &P16986_4};	/*Profile ID - 16986*/
const Value V16688_1_1 = {S2, NULL};	/*Value = True*/
const Property P16688_1 = {S50, 0, &V16688_1_1, NULL};	/*Property = IsMobile*/
const Value V16688_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P16688_5 = {S52, 0, &V16688_5_1275, &P16688_1};	/*Property = ScreenPixelsHeight*/
const Value V16688_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16688_4 = {S54, 0, &V16688_4_1229, &P16688_5};	/*Property = ScreenPixelsWidth*/
const Profile P16688 = {"16688", &P16688_4};	/*Profile ID - 16688*/
const Value V17903_1_1 = {S2, NULL};	/*Value = True*/
const Property P17903_1 = {S50, 0, &V17903_1_1, NULL};	/*Property = IsMobile*/
const Value V17903_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P17903_5 = {S52, 0, &V17903_5_1380, &P17903_1};	/*Property = ScreenPixelsHeight*/
const Value V17903_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17903_4 = {S54, 0, &V17903_4_1279, &P17903_5};	/*Property = ScreenPixelsWidth*/
const Profile P17903 = {"17903", &P17903_4};	/*Profile ID - 17903*/
const Value V13906_1_1 = {S2, NULL};	/*Value = True*/
const Property P13906_1 = {S50, 0, &V13906_1_1, NULL};	/*Property = IsMobile*/
const Value V13906_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P13906_5 = {S52, 0, &V13906_5_1275, &P13906_1};	/*Property = ScreenPixelsHeight*/
const Value V13906_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P13906_4 = {S54, 0, &V13906_4_1278, &P13906_5};	/*Property = ScreenPixelsWidth*/
const Profile P13906 = {"13906", &P13906_4};	/*Profile ID - 13906*/
const Value V16961_1_1 = {S2, NULL};	/*Value = True*/
const Property P16961_1 = {S50, 0, &V16961_1_1, NULL};	/*Property = IsMobile*/
const Value V16961_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P16961_5 = {S52, 0, &V16961_5_1380, &P16961_1};	/*Property = ScreenPixelsHeight*/
const Value V16961_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16961_4 = {S54, 0, &V16961_4_1279, &P16961_5};	/*Property = ScreenPixelsWidth*/
const Profile P16961 = {"16961", &P16961_4};	/*Profile ID - 16961*/
const Value V16151_1_1 = {S2, NULL};	/*Value = True*/
const Property P16151_1 = {S50, 0, &V16151_1_1, NULL};	/*Property = IsMobile*/
const Value V16151_5_37526 = {S122, NULL};	/*Value = 1200*/
const Property P16151_5 = {S52, 0, &V16151_5_37526, &P16151_1};	/*Property = ScreenPixelsHeight*/
const Value V16151_4_53934 = {S123, NULL};	/*Value = 824*/
const Property P16151_4 = {S54, 0, &V16151_4_53934, &P16151_5};	/*Property = ScreenPixelsWidth*/
const Profile P16151 = {"16151", &P16151_4};	/*Profile ID - 16151*/
const Value V17412_1_1 = {S2, NULL};	/*Value = True*/
const Property P17412_1 = {S50, 0, &V17412_1_1, NULL};	/*Property = IsMobile*/
const Value V17412_5_4286 = {S73, NULL};	/*Value = 432*/
const Property P17412_5 = {S52, 0, &V17412_5_4286, &P17412_1};	/*Property = ScreenPixelsHeight*/
const Value V17412_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17412_4 = {S54, 0, &V17412_4_1224, &P17412_5};	/*Property = ScreenPixelsWidth*/
const Profile P17412 = {"17412", &P17412_4};	/*Profile ID - 17412*/
const Value V14171_1_1 = {S2, NULL};	/*Value = True*/
const Property P14171_1 = {S50, 0, &V14171_1_1, NULL};	/*Property = IsMobile*/
const Value V14171_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P14171_5 = {S52, 0, &V14171_5_1233, &P14171_1};	/*Property = ScreenPixelsHeight*/
const Value V14171_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P14171_4 = {S54, 0, &V14171_4_1235, &P14171_5};	/*Property = ScreenPixelsWidth*/
const Profile P14171 = {"14171", &P14171_4};	/*Profile ID - 14171*/
const Value V16924_1_1 = {S2, NULL};	/*Value = True*/
const Property P16924_1 = {S50, 0, &V16924_1_1, NULL};	/*Property = IsMobile*/
const Value V16924_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16924_5 = {S52, 0, &V16924_5_1222, &P16924_1};	/*Property = ScreenPixelsHeight*/
const Value V16924_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16924_4 = {S54, 0, &V16924_4_1224, &P16924_5};	/*Property = ScreenPixelsWidth*/
const Profile P16924 = {"16924", &P16924_4};	/*Profile ID - 16924*/
const Value V17753_1_1 = {S2, NULL};	/*Value = True*/
const Property P17753_1 = {S50, 0, &V17753_1_1, NULL};	/*Property = IsMobile*/
const Value V17753_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17753_5 = {S52, 0, &V17753_5_1222, &P17753_1};	/*Property = ScreenPixelsHeight*/
const Value V17753_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17753_4 = {S54, 0, &V17753_4_1224, &P17753_5};	/*Property = ScreenPixelsWidth*/
const Profile P17753 = {"17753", &P17753_4};	/*Profile ID - 17753*/
const Value V20113_1_2 = {S0, NULL};	/*Value = False*/
const Property P20113_1 = {S50, 0, &V20113_1_2, NULL};	/*Property = IsMobile*/
const Value V20113_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20113_5 = {S52, 0, &V20113_5_37, &P20113_1};	/*Property = ScreenPixelsHeight*/
const Value V20113_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20113_4 = {S54, 0, &V20113_4_36, &P20113_5};	/*Property = ScreenPixelsWidth*/
const Profile P20113 = {"20113", &P20113_4};	/*Profile ID - 20113*/
const Value V12996_1_1 = {S2, NULL};	/*Value = True*/
const Property P12996_1 = {S50, 0, &V12996_1_1, NULL};	/*Property = IsMobile*/
const Value V12996_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12996_5 = {S52, 0, &V12996_5_1222, &P12996_1};	/*Property = ScreenPixelsHeight*/
const Value V12996_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12996_4 = {S54, 0, &V12996_4_1224, &P12996_5};	/*Property = ScreenPixelsWidth*/
const Profile P12996 = {"12996", &P12996_4};	/*Profile ID - 12996*/
const Value V15761_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_279 = {S1, 0, &V15761_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V15761_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_282 = {S3, 0, &V15761_282_57155, &P15761_279};	/*Property = BlobBuilder*/
const Value V15761_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_350 = {S4, 0, &V15761_350_57446, &P15761_282};	/*Property = Canvas*/
const Value V15761_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_283 = {S5, 0, &V15761_283_57161, &P15761_350};	/*Property = CssBackground*/
const Value V15761_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_284 = {S6, 0, &V15761_284_57164, &P15761_283};	/*Property = CssBorderImage*/
const Value V15761_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_285 = {S7, 0, &V15761_285_57167, &P15761_284};	/*Property = CssCanvas*/
const Value V15761_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_286 = {S8, 0, &V15761_286_57170, &P15761_285};	/*Property = CssColor*/
const Value V15761_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_237 = {S9, 0, &V15761_237_55786, &P15761_286};	/*Property = CssColumn*/
const Value V15761_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_287 = {S10, 0, &V15761_287_57173, &P15761_237};	/*Property = CssFlexbox*/
const Value V15761_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_288 = {S11, 0, &V15761_288_57176, &P15761_287};	/*Property = CssFont*/
const Value V15761_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_289 = {S12, 0, &V15761_289_86899, &P15761_288};	/*Property = CssImages*/
const Value V15761_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_291 = {S13, 0, &V15761_291_57182, &P15761_289};	/*Property = CssMediaQueries*/
const Value V15761_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_292 = {S14, 0, &V15761_292_57185, &P15761_291};	/*Property = CssMinMax*/
const Value V15761_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_293 = {S15, 0, &V15761_293_57188, &P15761_292};	/*Property = CssOverflow*/
const Value V15761_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_294 = {S16, 0, &V15761_294_57191, &P15761_293};	/*Property = CssPosition*/
const Value V15761_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_295 = {S17, 0, &V15761_295_57194, &P15761_294};	/*Property = CssText*/
const Value V15761_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_297 = {S18, 0, &V15761_297_57197, &P15761_295};	/*Property = CssTransforms*/
const Value V15761_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_241 = {S19, 0, &V15761_241_55801, &P15761_297};	/*Property = CssTransitions*/
const Value V15761_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_298 = {S20, 0, &V15761_298_57208, &P15761_241};	/*Property = CssUI*/
const Value V15761_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_300 = {S21, 0, &V15761_300_57217, &P15761_298};	/*Property = DataSet*/
const Value V15761_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_301 = {S22, 0, &V15761_301_57220, &P15761_300};	/*Property = DataUrl*/
const Value V15761_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_302 = {S23, 0, &V15761_302_57223, &P15761_301};	/*Property = DeviceOrientation*/
const Value V15761_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_304 = {S24, 0, &V15761_304_57229, &P15761_302};	/*Property = FileReader*/
const Value V15761_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_305 = {S25, 0, &V15761_305_57232, &P15761_304};	/*Property = FileSaver*/
const Value V15761_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_306 = {S26, 0, &V15761_306_57238, &P15761_305};	/*Property = FileWriter*/
const Value V15761_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_307 = {S27, 0, &V15761_307_57241, &P15761_306};	/*Property = FormData*/
const Value V15761_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_309 = {S28, 0, &V15761_309_57247, &P15761_307};	/*Property = Fullscreen*/
const Value V15761_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_310 = {S29, 0, &V15761_310_57250, &P15761_309};	/*Property = GeoLocation*/
const Value V15761_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_312 = {S30, 0, &V15761_312_57256, &P15761_310};	/*Property = History*/
const Value V15761_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_314 = {S31, 0, &V15761_314_57262, &P15761_312};	/*Property = Html5*/
const Value V15761_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_313 = {S32, 0, &V15761_313_57259, &P15761_314};	/*Property = Html-Media-Capture*/
const Value V15761_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_315 = {S33, 0, &V15761_315_57265, &P15761_313};	/*Property = Iframe*/
const Value V15761_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_316 = {S34, 0, &V15761_316_57268, &P15761_315};	/*Property = IndexedDB*/
const Value V15761_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_317 = {S35, 0, &V15761_317_57271, &P15761_316};	/*Property = Json*/
const Value V15761_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P15761_15 = {S37, 0, &V15761_15_29, &P15761_317};	/*Property = LayoutEngine*/
const Value V15761_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_344 = {S38, 0, &V15761_344_57274, &P15761_15};	/*Property = Masking*/
const Value V15761_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_320 = {S39, 0, &V15761_320_57292, &P15761_344};	/*Property = PostMessage*/
const Value V15761_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_321 = {S40, 0, &V15761_321_57295, &P15761_320};	/*Property = Progress*/
const Value V15761_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_322 = {S41, 0, &V15761_322_57298, &P15761_321};	/*Property = Prompts*/
const Value V15761_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_323 = {S42, 0, &V15761_323_57301, &P15761_322};	/*Property = Selector*/
const Value V15761_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_324 = {S43, 0, &V15761_324_57307, &P15761_323};	/*Property = Svg*/
const Value V15761_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_327 = {S44, 0, &V15761_327_57316, &P15761_324};	/*Property = TouchEvents*/
const Value V15761_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_328 = {S45, 0, &V15761_328_57319, &P15761_327};	/*Property = Track*/
const Value V15761_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_329 = {S46, 0, &V15761_329_57325, &P15761_328};	/*Property = Video*/
const Value V15761_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_330 = {S47, 0, &V15761_330_57328, &P15761_329};	/*Property = Viewport*/
const Value V15761_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_332 = {S48, 0, &V15761_332_57340, &P15761_330};	/*Property = WebWorkers*/
const Value V15761_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P15761_333 = {S49, 0, &V15761_333_57343, &P15761_332};	/*Property = Xhr2*/
const Profile P15761 = {"15761", &P15761_333};	/*Profile ID - 15761*/
const Value V16912_1_1 = {S2, NULL};	/*Value = True*/
const Property P16912_1 = {S50, 0, &V16912_1_1, NULL};	/*Property = IsMobile*/
const Value V16912_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16912_5 = {S52, 0, &V16912_5_1294, &P16912_1};	/*Property = ScreenPixelsHeight*/
const Value V16912_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16912_4 = {S54, 0, &V16912_4_1296, &P16912_5};	/*Property = ScreenPixelsWidth*/
const Profile P16912 = {"16912", &P16912_4};	/*Profile ID - 16912*/
const Value V22744_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_279 = {S1, 0, &V22744_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22744_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_282 = {S3, 0, &V22744_282_57155, &P22744_279};	/*Property = BlobBuilder*/
const Value V22744_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_350 = {S4, 0, &V22744_350_57446, &P22744_282};	/*Property = Canvas*/
const Value V22744_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_283 = {S5, 0, &V22744_283_57161, &P22744_350};	/*Property = CssBackground*/
const Value V22744_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_284 = {S6, 0, &V22744_284_57164, &P22744_283};	/*Property = CssBorderImage*/
const Value V22744_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_285 = {S7, 0, &V22744_285_57167, &P22744_284};	/*Property = CssCanvas*/
const Value V22744_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_286 = {S8, 0, &V22744_286_57170, &P22744_285};	/*Property = CssColor*/
const Value V22744_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_237 = {S9, 0, &V22744_237_55786, &P22744_286};	/*Property = CssColumn*/
const Value V22744_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_287 = {S10, 0, &V22744_287_57173, &P22744_237};	/*Property = CssFlexbox*/
const Value V22744_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_288 = {S11, 0, &V22744_288_57176, &P22744_287};	/*Property = CssFont*/
const Value V22744_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_289 = {S12, 0, &V22744_289_86899, &P22744_288};	/*Property = CssImages*/
const Value V22744_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_291 = {S13, 0, &V22744_291_57182, &P22744_289};	/*Property = CssMediaQueries*/
const Value V22744_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_292 = {S14, 0, &V22744_292_57185, &P22744_291};	/*Property = CssMinMax*/
const Value V22744_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_293 = {S15, 0, &V22744_293_57188, &P22744_292};	/*Property = CssOverflow*/
const Value V22744_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_294 = {S16, 0, &V22744_294_57191, &P22744_293};	/*Property = CssPosition*/
const Value V22744_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_295 = {S17, 0, &V22744_295_57194, &P22744_294};	/*Property = CssText*/
const Value V22744_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_297 = {S18, 0, &V22744_297_57197, &P22744_295};	/*Property = CssTransforms*/
const Value V22744_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_241 = {S19, 0, &V22744_241_55801, &P22744_297};	/*Property = CssTransitions*/
const Value V22744_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_298 = {S20, 0, &V22744_298_57208, &P22744_241};	/*Property = CssUI*/
const Value V22744_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_300 = {S21, 0, &V22744_300_57217, &P22744_298};	/*Property = DataSet*/
const Value V22744_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_301 = {S22, 0, &V22744_301_57220, &P22744_300};	/*Property = DataUrl*/
const Value V22744_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_302 = {S23, 0, &V22744_302_57223, &P22744_301};	/*Property = DeviceOrientation*/
const Value V22744_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_304 = {S24, 0, &V22744_304_57229, &P22744_302};	/*Property = FileReader*/
const Value V22744_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_305 = {S25, 0, &V22744_305_57232, &P22744_304};	/*Property = FileSaver*/
const Value V22744_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_306 = {S26, 0, &V22744_306_57238, &P22744_305};	/*Property = FileWriter*/
const Value V22744_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_307 = {S27, 0, &V22744_307_57241, &P22744_306};	/*Property = FormData*/
const Value V22744_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_309 = {S28, 0, &V22744_309_57247, &P22744_307};	/*Property = Fullscreen*/
const Value V22744_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_310 = {S29, 0, &V22744_310_57250, &P22744_309};	/*Property = GeoLocation*/
const Value V22744_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_312 = {S30, 0, &V22744_312_57256, &P22744_310};	/*Property = History*/
const Value V22744_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_314 = {S31, 0, &V22744_314_57262, &P22744_312};	/*Property = Html5*/
const Value V22744_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_313 = {S32, 0, &V22744_313_57259, &P22744_314};	/*Property = Html-Media-Capture*/
const Value V22744_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_315 = {S33, 0, &V22744_315_57265, &P22744_313};	/*Property = Iframe*/
const Value V22744_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_316 = {S34, 0, &V22744_316_57268, &P22744_315};	/*Property = IndexedDB*/
const Value V22744_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_317 = {S35, 0, &V22744_317_57271, &P22744_316};	/*Property = Json*/
const Value V22744_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22744_15 = {S37, 0, &V22744_15_29, &P22744_317};	/*Property = LayoutEngine*/
const Value V22744_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_344 = {S38, 0, &V22744_344_57274, &P22744_15};	/*Property = Masking*/
const Value V22744_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_320 = {S39, 0, &V22744_320_57292, &P22744_344};	/*Property = PostMessage*/
const Value V22744_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_321 = {S40, 0, &V22744_321_57295, &P22744_320};	/*Property = Progress*/
const Value V22744_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_322 = {S41, 0, &V22744_322_57298, &P22744_321};	/*Property = Prompts*/
const Value V22744_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_323 = {S42, 0, &V22744_323_57301, &P22744_322};	/*Property = Selector*/
const Value V22744_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_324 = {S43, 0, &V22744_324_57307, &P22744_323};	/*Property = Svg*/
const Value V22744_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_327 = {S44, 0, &V22744_327_57316, &P22744_324};	/*Property = TouchEvents*/
const Value V22744_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_328 = {S45, 0, &V22744_328_57319, &P22744_327};	/*Property = Track*/
const Value V22744_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_329 = {S46, 0, &V22744_329_57325, &P22744_328};	/*Property = Video*/
const Value V22744_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_330 = {S47, 0, &V22744_330_57328, &P22744_329};	/*Property = Viewport*/
const Value V22744_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_332 = {S48, 0, &V22744_332_57340, &P22744_330};	/*Property = WebWorkers*/
const Value V22744_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22744_333 = {S49, 0, &V22744_333_57343, &P22744_332};	/*Property = Xhr2*/
const Profile P22744 = {"22744", &P22744_333};	/*Profile ID - 22744*/
const Profile P18089 = {"18089", NULL};	/*Profile ID - 18089*/
const Value V17737_1_1 = {S2, NULL};	/*Value = True*/
const Property P17737_1 = {S50, 0, &V17737_1_1, NULL};	/*Property = IsMobile*/
const Value V17737_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17737_5 = {S52, 0, &V17737_5_1269, &P17737_1};	/*Property = ScreenPixelsHeight*/
const Value V17737_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17737_4 = {S54, 0, &V17737_4_1229, &P17737_5};	/*Property = ScreenPixelsWidth*/
const Profile P17737 = {"17737", &P17737_4};	/*Profile ID - 17737*/
const Value V20212_1_2 = {S0, NULL};	/*Value = False*/
const Property P20212_1 = {S50, 0, &V20212_1_2, NULL};	/*Property = IsMobile*/
const Value V20212_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20212_5 = {S52, 0, &V20212_5_37, &P20212_1};	/*Property = ScreenPixelsHeight*/
const Value V20212_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20212_4 = {S54, 0, &V20212_4_36, &P20212_5};	/*Property = ScreenPixelsWidth*/
const Profile P20212 = {"20212", &P20212_4};	/*Profile ID - 20212*/
const Value V16941_1_1 = {S2, NULL};	/*Value = True*/
const Property P16941_1 = {S50, 0, &V16941_1_1, NULL};	/*Property = IsMobile*/
const Value V16941_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16941_5 = {S52, 0, &V16941_5_1277, &P16941_1};	/*Property = ScreenPixelsHeight*/
const Value V16941_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16941_4 = {S54, 0, &V16941_4_1229, &P16941_5};	/*Property = ScreenPixelsWidth*/
const Profile P16941 = {"16941", &P16941_4};	/*Profile ID - 16941*/
const Value V17220_1_1 = {S2, NULL};	/*Value = True*/
const Property P17220_1 = {S50, 0, &V17220_1_1, NULL};	/*Property = IsMobile*/
const Value V17220_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17220_5 = {S52, 0, &V17220_5_1304, &P17220_1};	/*Property = ScreenPixelsHeight*/
const Value V17220_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17220_4 = {S54, 0, &V17220_4_1224, &P17220_5};	/*Property = ScreenPixelsWidth*/
const Profile P17220 = {"17220", &P17220_4};	/*Profile ID - 17220*/
const Value V18490_1_1 = {S2, NULL};	/*Value = True*/
const Property P18490_1 = {S50, 0, &V18490_1_1, NULL};	/*Property = IsMobile*/
const Value V18490_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P18490_5 = {S52, 0, &V18490_5_1333, &P18490_1};	/*Property = ScreenPixelsHeight*/
const Value V18490_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P18490_4 = {S54, 0, &V18490_4_1335, &P18490_5};	/*Property = ScreenPixelsWidth*/
const Profile P18490 = {"18490", &P18490_4};	/*Profile ID - 18490*/
const Value V18546_279_57147 = {S2, NULL};	/*Value = True*/
const Property P18546_279 = {S1, 0, &V18546_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V18546_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18546_282 = {S3, 0, &V18546_282_57154, &P18546_279};	/*Property = BlobBuilder*/
const Value V18546_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18546_350 = {S4, 0, &V18546_350_57444, &P18546_282};	/*Property = Canvas*/
const Value V18546_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18546_283 = {S5, 0, &V18546_283_57159, &P18546_350};	/*Property = CssBackground*/
const Value V18546_284_57162 = {S2, NULL};	/*Value = True*/
const Property P18546_284 = {S6, 0, &V18546_284_57162, &P18546_283};	/*Property = CssBorderImage*/
const Value V18546_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18546_285 = {S7, 0, &V18546_285_57166, &P18546_284};	/*Property = CssCanvas*/
const Value V18546_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18546_286 = {S8, 0, &V18546_286_57168, &P18546_285};	/*Property = CssColor*/
const Value V18546_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18546_237 = {S9, 0, &V18546_237_55786, &P18546_286};	/*Property = CssColumn*/
const Value V18546_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18546_287 = {S10, 0, &V18546_287_57172, &P18546_237};	/*Property = CssFlexbox*/
const Value V18546_288_57174 = {S2, NULL};	/*Value = True*/
const Property P18546_288 = {S11, 0, &V18546_288_57174, &P18546_287};	/*Property = CssFont*/
const Value V18546_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18546_289 = {S12, 0, &V18546_289_57759, &P18546_288};	/*Property = CssImages*/
const Value V18546_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18546_291 = {S13, 0, &V18546_291_57180, &P18546_289};	/*Property = CssMediaQueries*/
const Value V18546_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18546_292 = {S14, 0, &V18546_292_57183, &P18546_291};	/*Property = CssMinMax*/
const Value V18546_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18546_293 = {S15, 0, &V18546_293_57187, &P18546_292};	/*Property = CssOverflow*/
const Value V18546_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18546_294 = {S16, 0, &V18546_294_57189, &P18546_293};	/*Property = CssPosition*/
const Value V18546_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18546_295 = {S17, 0, &V18546_295_57192, &P18546_294};	/*Property = CssText*/
const Value V18546_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18546_297 = {S18, 0, &V18546_297_57195, &P18546_295};	/*Property = CssTransforms*/
const Value V18546_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18546_241 = {S19, 0, &V18546_241_55799, &P18546_297};	/*Property = CssTransitions*/
const Value V18546_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18546_298 = {S20, 0, &V18546_298_57206, &P18546_241};	/*Property = CssUI*/
const Value V18546_300_57215 = {S2, NULL};	/*Value = True*/
const Property P18546_300 = {S21, 0, &V18546_300_57215, &P18546_298};	/*Property = DataSet*/
const Value V18546_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18546_301 = {S22, 0, &V18546_301_57218, &P18546_300};	/*Property = DataUrl*/
const Value V18546_302_57221 = {S2, NULL};	/*Value = True*/
const Property P18546_302 = {S23, 0, &V18546_302_57221, &P18546_301};	/*Property = DeviceOrientation*/
const Value V18546_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18546_304 = {S24, 0, &V18546_304_57227, &P18546_302};	/*Property = FileReader*/
const Value V18546_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18546_305 = {S25, 0, &V18546_305_57231, &P18546_304};	/*Property = FileSaver*/
const Value V18546_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18546_306 = {S26, 0, &V18546_306_57237, &P18546_305};	/*Property = FileWriter*/
const Value V18546_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18546_307 = {S27, 0, &V18546_307_57239, &P18546_306};	/*Property = FormData*/
const Value V18546_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18546_309 = {S28, 0, &V18546_309_57246, &P18546_307};	/*Property = Fullscreen*/
const Value V18546_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18546_310 = {S29, 0, &V18546_310_57248, &P18546_309};	/*Property = GeoLocation*/
const Value V18546_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18546_312 = {S30, 0, &V18546_312_57254, &P18546_310};	/*Property = History*/
const Value V18546_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18546_314 = {S31, 0, &V18546_314_57261, &P18546_312};	/*Property = Html5*/
const Value V18546_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18546_313 = {S32, 0, &V18546_313_57258, &P18546_314};	/*Property = Html-Media-Capture*/
const Value V18546_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18546_315 = {S33, 0, &V18546_315_57264, &P18546_313};	/*Property = Iframe*/
const Value V18546_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18546_316 = {S34, 0, &V18546_316_57266, &P18546_315};	/*Property = IndexedDB*/
const Value V18546_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18546_317 = {S35, 0, &V18546_317_57269, &P18546_316};	/*Property = Json*/
const Value V18546_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P18546_15 = {S37, 0, &V18546_15_72, &P18546_317};	/*Property = LayoutEngine*/
const Value V18546_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18546_344 = {S38, 0, &V18546_344_57272, &P18546_15};	/*Property = Masking*/
const Value V18546_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18546_320 = {S39, 0, &V18546_320_57290, &P18546_344};	/*Property = PostMessage*/
const Value V18546_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18546_321 = {S40, 0, &V18546_321_57293, &P18546_320};	/*Property = Progress*/
const Value V18546_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18546_322 = {S41, 0, &V18546_322_57296, &P18546_321};	/*Property = Prompts*/
const Value V18546_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18546_323 = {S42, 0, &V18546_323_57299, &P18546_322};	/*Property = Selector*/
const Value V18546_324_57305 = {S2, NULL};	/*Value = True*/
const Property P18546_324 = {S43, 0, &V18546_324_57305, &P18546_323};	/*Property = Svg*/
const Value V18546_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18546_327 = {S44, 0, &V18546_327_57315, &P18546_324};	/*Property = TouchEvents*/
const Value V18546_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18546_328 = {S45, 0, &V18546_328_57318, &P18546_327};	/*Property = Track*/
const Value V18546_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18546_329 = {S46, 0, &V18546_329_57323, &P18546_328};	/*Property = Video*/
const Value V18546_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18546_330 = {S47, 0, &V18546_330_57326, &P18546_329};	/*Property = Viewport*/
const Value V18546_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18546_332 = {S48, 0, &V18546_332_57339, &P18546_330};	/*Property = WebWorkers*/
const Value V18546_333_57341 = {S2, NULL};	/*Value = True*/
const Property P18546_333 = {S49, 0, &V18546_333_57341, &P18546_332};	/*Property = Xhr2*/
const Profile P18546 = {"18546", &P18546_333};	/*Profile ID - 18546*/
const Value V17853_1_1 = {S2, NULL};	/*Value = True*/
const Property P17853_1 = {S50, 0, &V17853_1_1, NULL};	/*Property = IsMobile*/
const Value V17853_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17853_5 = {S52, 0, &V17853_5_1304, &P17853_1};	/*Property = ScreenPixelsHeight*/
const Value V17853_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17853_4 = {S54, 0, &V17853_4_1224, &P17853_5};	/*Property = ScreenPixelsWidth*/
const Profile P17853 = {"17853", &P17853_4};	/*Profile ID - 17853*/
const Value V17700_1_1 = {S2, NULL};	/*Value = True*/
const Property P17700_1 = {S50, 0, &V17700_1_1, NULL};	/*Property = IsMobile*/
const Value V17700_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P17700_5 = {S52, 0, &V17700_5_1333, &P17700_1};	/*Property = ScreenPixelsHeight*/
const Value V17700_4_37596 = {S98, NULL};	/*Value = 144*/
const Property P17700_4 = {S54, 0, &V17700_4_37596, &P17700_5};	/*Property = ScreenPixelsWidth*/
const Profile P17700 = {"17700", &P17700_4};	/*Profile ID - 17700*/
const Value V20422_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_279 = {S1, 0, &V20422_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20422_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_282 = {S3, 0, &V20422_282_57155, &P20422_279};	/*Property = BlobBuilder*/
const Value V20422_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_350 = {S4, 0, &V20422_350_57446, &P20422_282};	/*Property = Canvas*/
const Value V20422_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_283 = {S5, 0, &V20422_283_57161, &P20422_350};	/*Property = CssBackground*/
const Value V20422_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_284 = {S6, 0, &V20422_284_57164, &P20422_283};	/*Property = CssBorderImage*/
const Value V20422_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_285 = {S7, 0, &V20422_285_57167, &P20422_284};	/*Property = CssCanvas*/
const Value V20422_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_286 = {S8, 0, &V20422_286_57170, &P20422_285};	/*Property = CssColor*/
const Value V20422_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_237 = {S9, 0, &V20422_237_55786, &P20422_286};	/*Property = CssColumn*/
const Value V20422_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_287 = {S10, 0, &V20422_287_57173, &P20422_237};	/*Property = CssFlexbox*/
const Value V20422_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_288 = {S11, 0, &V20422_288_57176, &P20422_287};	/*Property = CssFont*/
const Value V20422_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_289 = {S12, 0, &V20422_289_86899, &P20422_288};	/*Property = CssImages*/
const Value V20422_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_291 = {S13, 0, &V20422_291_57182, &P20422_289};	/*Property = CssMediaQueries*/
const Value V20422_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_292 = {S14, 0, &V20422_292_57185, &P20422_291};	/*Property = CssMinMax*/
const Value V20422_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_293 = {S15, 0, &V20422_293_57188, &P20422_292};	/*Property = CssOverflow*/
const Value V20422_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_294 = {S16, 0, &V20422_294_57191, &P20422_293};	/*Property = CssPosition*/
const Value V20422_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_295 = {S17, 0, &V20422_295_57194, &P20422_294};	/*Property = CssText*/
const Value V20422_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_297 = {S18, 0, &V20422_297_57197, &P20422_295};	/*Property = CssTransforms*/
const Value V20422_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_241 = {S19, 0, &V20422_241_55801, &P20422_297};	/*Property = CssTransitions*/
const Value V20422_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_298 = {S20, 0, &V20422_298_57208, &P20422_241};	/*Property = CssUI*/
const Value V20422_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_300 = {S21, 0, &V20422_300_57217, &P20422_298};	/*Property = DataSet*/
const Value V20422_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_301 = {S22, 0, &V20422_301_57220, &P20422_300};	/*Property = DataUrl*/
const Value V20422_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_302 = {S23, 0, &V20422_302_57223, &P20422_301};	/*Property = DeviceOrientation*/
const Value V20422_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_304 = {S24, 0, &V20422_304_57229, &P20422_302};	/*Property = FileReader*/
const Value V20422_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_305 = {S25, 0, &V20422_305_57232, &P20422_304};	/*Property = FileSaver*/
const Value V20422_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_306 = {S26, 0, &V20422_306_57238, &P20422_305};	/*Property = FileWriter*/
const Value V20422_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_307 = {S27, 0, &V20422_307_57241, &P20422_306};	/*Property = FormData*/
const Value V20422_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_309 = {S28, 0, &V20422_309_57247, &P20422_307};	/*Property = Fullscreen*/
const Value V20422_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_310 = {S29, 0, &V20422_310_57250, &P20422_309};	/*Property = GeoLocation*/
const Value V20422_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_312 = {S30, 0, &V20422_312_57256, &P20422_310};	/*Property = History*/
const Value V20422_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_314 = {S31, 0, &V20422_314_57262, &P20422_312};	/*Property = Html5*/
const Value V20422_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_313 = {S32, 0, &V20422_313_57259, &P20422_314};	/*Property = Html-Media-Capture*/
const Value V20422_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_315 = {S33, 0, &V20422_315_57265, &P20422_313};	/*Property = Iframe*/
const Value V20422_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_316 = {S34, 0, &V20422_316_57268, &P20422_315};	/*Property = IndexedDB*/
const Value V20422_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_317 = {S35, 0, &V20422_317_57271, &P20422_316};	/*Property = Json*/
const Value V20422_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_15 = {S37, 0, &V20422_15_72, &P20422_317};	/*Property = LayoutEngine*/
const Value V20422_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_344 = {S38, 0, &V20422_344_57274, &P20422_15};	/*Property = Masking*/
const Value V20422_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_320 = {S39, 0, &V20422_320_57292, &P20422_344};	/*Property = PostMessage*/
const Value V20422_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_321 = {S40, 0, &V20422_321_57295, &P20422_320};	/*Property = Progress*/
const Value V20422_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_322 = {S41, 0, &V20422_322_57298, &P20422_321};	/*Property = Prompts*/
const Value V20422_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_323 = {S42, 0, &V20422_323_57301, &P20422_322};	/*Property = Selector*/
const Value V20422_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_324 = {S43, 0, &V20422_324_57307, &P20422_323};	/*Property = Svg*/
const Value V20422_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_327 = {S44, 0, &V20422_327_57316, &P20422_324};	/*Property = TouchEvents*/
const Value V20422_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_328 = {S45, 0, &V20422_328_57319, &P20422_327};	/*Property = Track*/
const Value V20422_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_329 = {S46, 0, &V20422_329_57325, &P20422_328};	/*Property = Video*/
const Value V20422_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_330 = {S47, 0, &V20422_330_57328, &P20422_329};	/*Property = Viewport*/
const Value V20422_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_332 = {S48, 0, &V20422_332_57340, &P20422_330};	/*Property = WebWorkers*/
const Value V20422_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20422_333 = {S49, 0, &V20422_333_57343, &P20422_332};	/*Property = Xhr2*/
const Profile P20422 = {"20422", &P20422_333};	/*Profile ID - 20422*/
const Value V17541_1_1 = {S2, NULL};	/*Value = True*/
const Property P17541_1 = {S50, 0, &V17541_1_1, NULL};	/*Property = IsMobile*/
const Value V17541_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17541_5 = {S52, 0, &V17541_5_1304, &P17541_1};	/*Property = ScreenPixelsHeight*/
const Value V17541_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17541_4 = {S54, 0, &V17541_4_1224, &P17541_5};	/*Property = ScreenPixelsWidth*/
const Profile P17541 = {"17541", &P17541_4};	/*Profile ID - 17541*/
const Value V17537_1_1 = {S2, NULL};	/*Value = True*/
const Property P17537_1 = {S50, 0, &V17537_1_1, NULL};	/*Property = IsMobile*/
const Value V17537_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17537_5 = {S52, 0, &V17537_5_1304, &P17537_1};	/*Property = ScreenPixelsHeight*/
const Value V17537_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17537_4 = {S54, 0, &V17537_4_1224, &P17537_5};	/*Property = ScreenPixelsWidth*/
const Profile P17537 = {"17537", &P17537_4};	/*Profile ID - 17537*/
const Value V17873_1_1 = {S2, NULL};	/*Value = True*/
const Property P17873_1 = {S50, 0, &V17873_1_1, NULL};	/*Property = IsMobile*/
const Value V17873_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17873_5 = {S52, 0, &V17873_5_1269, &P17873_1};	/*Property = ScreenPixelsHeight*/
const Value V17873_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17873_4 = {S54, 0, &V17873_4_1229, &P17873_5};	/*Property = ScreenPixelsWidth*/
const Profile P17873 = {"17873", &P17873_4};	/*Profile ID - 17873*/
const Value V17636_1_1 = {S2, NULL};	/*Value = True*/
const Property P17636_1 = {S50, 0, &V17636_1_1, NULL};	/*Property = IsMobile*/
const Value V17636_5_4286 = {S73, NULL};	/*Value = 432*/
const Property P17636_5 = {S52, 0, &V17636_5_4286, &P17636_1};	/*Property = ScreenPixelsHeight*/
const Value V17636_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17636_4 = {S54, 0, &V17636_4_1224, &P17636_5};	/*Property = ScreenPixelsWidth*/
const Profile P17636 = {"17636", &P17636_4};	/*Profile ID - 17636*/
const Value V17535_1_1 = {S2, NULL};	/*Value = True*/
const Property P17535_1 = {S50, 0, &V17535_1_1, NULL};	/*Property = IsMobile*/
const Value V17535_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17535_5 = {S52, 0, &V17535_5_1304, &P17535_1};	/*Property = ScreenPixelsHeight*/
const Value V17535_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17535_4 = {S54, 0, &V17535_4_1224, &P17535_5};	/*Property = ScreenPixelsWidth*/
const Profile P17535 = {"17535", &P17535_4};	/*Profile ID - 17535*/
const Value V17543_1_1 = {S2, NULL};	/*Value = True*/
const Property P17543_1 = {S50, 0, &V17543_1_1, NULL};	/*Property = IsMobile*/
const Value V17543_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17543_5 = {S52, 0, &V17543_5_1269, &P17543_1};	/*Property = ScreenPixelsHeight*/
const Value V17543_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17543_4 = {S54, 0, &V17543_4_1229, &P17543_5};	/*Property = ScreenPixelsWidth*/
const Profile P17543 = {"17543", &P17543_4};	/*Profile ID - 17543*/
const Value V17545_1_1 = {S2, NULL};	/*Value = True*/
const Property P17545_1 = {S50, 0, &V17545_1_1, NULL};	/*Property = IsMobile*/
const Value V17545_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17545_5 = {S52, 0, &V17545_5_1304, &P17545_1};	/*Property = ScreenPixelsHeight*/
const Value V17545_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17545_4 = {S54, 0, &V17545_4_1224, &P17545_5};	/*Property = ScreenPixelsWidth*/
const Profile P17545 = {"17545", &P17545_4};	/*Profile ID - 17545*/
const Value V17509_1_1 = {S2, NULL};	/*Value = True*/
const Property P17509_1 = {S50, 0, &V17509_1_1, NULL};	/*Property = IsMobile*/
const Value V17509_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17509_5 = {S52, 0, &V17509_5_1269, &P17509_1};	/*Property = ScreenPixelsHeight*/
const Value V17509_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17509_4 = {S54, 0, &V17509_4_1229, &P17509_5};	/*Property = ScreenPixelsWidth*/
const Profile P17509 = {"17509", &P17509_4};	/*Profile ID - 17509*/
const Value V17569_1_1 = {S2, NULL};	/*Value = True*/
const Property P17569_1 = {S50, 0, &V17569_1_1, NULL};	/*Property = IsMobile*/
const Value V17569_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17569_5 = {S52, 0, &V17569_5_1227, &P17569_1};	/*Property = ScreenPixelsHeight*/
const Value V17569_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17569_4 = {S54, 0, &V17569_4_1229, &P17569_5};	/*Property = ScreenPixelsWidth*/
const Profile P17569 = {"17569", &P17569_4};	/*Profile ID - 17569*/
const Value V17511_1_1 = {S2, NULL};	/*Value = True*/
const Property P17511_1 = {S50, 0, &V17511_1_1, NULL};	/*Property = IsMobile*/
const Value V17511_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17511_5 = {S52, 0, &V17511_5_1227, &P17511_1};	/*Property = ScreenPixelsHeight*/
const Value V17511_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17511_4 = {S54, 0, &V17511_4_1229, &P17511_5};	/*Property = ScreenPixelsWidth*/
const Profile P17511 = {"17511", &P17511_4};	/*Profile ID - 17511*/
const Value V17564_1_1 = {S2, NULL};	/*Value = True*/
const Property P17564_1 = {S50, 0, &V17564_1_1, NULL};	/*Property = IsMobile*/
const Value V17564_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P17564_5 = {S52, 0, &V17564_5_1380, &P17564_1};	/*Property = ScreenPixelsHeight*/
const Value V17564_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17564_4 = {S54, 0, &V17564_4_1279, &P17564_5};	/*Property = ScreenPixelsWidth*/
const Profile P17564 = {"17564", &P17564_4};	/*Profile ID - 17564*/
const Value V21392_1_1 = {S2, NULL};	/*Value = True*/
const Property P21392_1 = {S50, 0, &V21392_1_1, NULL};	/*Property = IsMobile*/
const Value V21392_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P21392_5 = {S52, 0, &V21392_5_1233, &P21392_1};	/*Property = ScreenPixelsHeight*/
const Value V21392_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P21392_4 = {S54, 0, &V21392_4_1235, &P21392_5};	/*Property = ScreenPixelsWidth*/
const Profile P21392 = {"21392", &P21392_4};	/*Profile ID - 21392*/
const Value V21391_1_1 = {S2, NULL};	/*Value = True*/
const Property P21391_1 = {S50, 0, &V21391_1_1, NULL};	/*Property = IsMobile*/
const Value V21391_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P21391_5 = {S52, 0, &V21391_5_1293, &P21391_1};	/*Property = ScreenPixelsHeight*/
const Value V21391_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P21391_4 = {S54, 0, &V21391_4_1296, &P21391_5};	/*Property = ScreenPixelsWidth*/
const Profile P21391 = {"21391", &P21391_4};	/*Profile ID - 21391*/
const Value V16871_1_1 = {S2, NULL};	/*Value = True*/
const Property P16871_1 = {S50, 0, &V16871_1_1, NULL};	/*Property = IsMobile*/
const Value V16871_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P16871_5 = {S52, 0, &V16871_5_1380, &P16871_1};	/*Property = ScreenPixelsHeight*/
const Value V16871_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16871_4 = {S54, 0, &V16871_4_1279, &P16871_5};	/*Property = ScreenPixelsWidth*/
const Profile P16871 = {"16871", &P16871_4};	/*Profile ID - 16871*/
const Value V17562_1_1 = {S2, NULL};	/*Value = True*/
const Property P17562_1 = {S50, 0, &V17562_1_1, NULL};	/*Property = IsMobile*/
const Value V17562_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P17562_5 = {S52, 0, &V17562_5_1380, &P17562_1};	/*Property = ScreenPixelsHeight*/
const Value V17562_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17562_4 = {S54, 0, &V17562_4_1279, &P17562_5};	/*Property = ScreenPixelsWidth*/
const Profile P17562 = {"17562", &P17562_4};	/*Profile ID - 17562*/
const Value V17805_1_1 = {S2, NULL};	/*Value = True*/
const Property P17805_1 = {S50, 0, &V17805_1_1, NULL};	/*Property = IsMobile*/
const Value V17805_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17805_5 = {S52, 0, &V17805_5_1222, &P17805_1};	/*Property = ScreenPixelsHeight*/
const Value V17805_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17805_4 = {S54, 0, &V17805_4_1224, &P17805_5};	/*Property = ScreenPixelsWidth*/
const Profile P17805 = {"17805", &P17805_4};	/*Profile ID - 17805*/
const Value V16138_1_1 = {S2, NULL};	/*Value = True*/
const Property P16138_1 = {S50, 0, &V16138_1_1, NULL};	/*Property = IsMobile*/
const Value V16138_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P16138_5 = {S52, 0, &V16138_5_1269, &P16138_1};	/*Property = ScreenPixelsHeight*/
const Value V16138_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16138_4 = {S54, 0, &V16138_4_1229, &P16138_5};	/*Property = ScreenPixelsWidth*/
const Profile P16138 = {"16138", &P16138_4};	/*Profile ID - 16138*/
const Value V17568_1_1 = {S2, NULL};	/*Value = True*/
const Property P17568_1 = {S50, 0, &V17568_1_1, NULL};	/*Property = IsMobile*/
const Value V17568_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17568_5 = {S52, 0, &V17568_5_1227, &P17568_1};	/*Property = ScreenPixelsHeight*/
const Value V17568_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17568_4 = {S54, 0, &V17568_4_1229, &P17568_5};	/*Property = ScreenPixelsWidth*/
const Profile P17568 = {"17568", &P17568_4};	/*Profile ID - 17568*/
const Value V17874_1_1 = {S2, NULL};	/*Value = True*/
const Property P17874_1 = {S50, 0, &V17874_1_1, NULL};	/*Property = IsMobile*/
const Value V17874_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17874_5 = {S52, 0, &V17874_5_1269, &P17874_1};	/*Property = ScreenPixelsHeight*/
const Value V17874_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17874_4 = {S54, 0, &V17874_4_1229, &P17874_5};	/*Property = ScreenPixelsWidth*/
const Profile P17874 = {"17874", &P17874_4};	/*Profile ID - 17874*/
const Value V17549_1_1 = {S2, NULL};	/*Value = True*/
const Property P17549_1 = {S50, 0, &V17549_1_1, NULL};	/*Property = IsMobile*/
const Value V17549_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17549_5 = {S52, 0, &V17549_5_1304, &P17549_1};	/*Property = ScreenPixelsHeight*/
const Value V17549_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17549_4 = {S54, 0, &V17549_4_1224, &P17549_5};	/*Property = ScreenPixelsWidth*/
const Profile P17549 = {"17549", &P17549_4};	/*Profile ID - 17549*/
const Value V17547_1_1 = {S2, NULL};	/*Value = True*/
const Property P17547_1 = {S50, 0, &V17547_1_1, NULL};	/*Property = IsMobile*/
const Value V17547_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17547_5 = {S52, 0, &V17547_5_1304, &P17547_1};	/*Property = ScreenPixelsHeight*/
const Value V17547_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17547_4 = {S54, 0, &V17547_4_1224, &P17547_5};	/*Property = ScreenPixelsWidth*/
const Profile P17547 = {"17547", &P17547_4};	/*Profile ID - 17547*/
const Value V17572_1_1 = {S2, NULL};	/*Value = True*/
const Property P17572_1 = {S50, 0, &V17572_1_1, NULL};	/*Property = IsMobile*/
const Value V17572_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17572_5 = {S52, 0, &V17572_5_1269, &P17572_1};	/*Property = ScreenPixelsHeight*/
const Value V17572_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17572_4 = {S54, 0, &V17572_4_1229, &P17572_5};	/*Property = ScreenPixelsWidth*/
const Profile P17572 = {"17572", &P17572_4};	/*Profile ID - 17572*/
const Value V17634_1_1 = {S2, NULL};	/*Value = True*/
const Property P17634_1 = {S50, 0, &V17634_1_1, NULL};	/*Property = IsMobile*/
const Value V17634_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17634_5 = {S52, 0, &V17634_5_1304, &P17634_1};	/*Property = ScreenPixelsHeight*/
const Value V17634_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17634_4 = {S54, 0, &V17634_4_1224, &P17634_5};	/*Property = ScreenPixelsWidth*/
const Profile P17634 = {"17634", &P17634_4};	/*Profile ID - 17634*/
const Value V20232_1_1 = {S2, NULL};	/*Value = True*/
const Property P20232_1 = {S50, 0, &V20232_1_1, NULL};	/*Property = IsMobile*/
const Value V20232_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P20232_5 = {S52, 0, &V20232_5_1233, &P20232_1};	/*Property = ScreenPixelsHeight*/
const Value V20232_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P20232_4 = {S54, 0, &V20232_4_1235, &P20232_5};	/*Property = ScreenPixelsWidth*/
const Profile P20232 = {"20232", &P20232_4};	/*Profile ID - 20232*/
const Value V17904_1_1 = {S2, NULL};	/*Value = True*/
const Property P17904_1 = {S50, 0, &V17904_1_1, NULL};	/*Property = IsMobile*/
const Value V17904_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17904_5 = {S52, 0, &V17904_5_1233, &P17904_1};	/*Property = ScreenPixelsHeight*/
const Value V17904_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17904_4 = {S54, 0, &V17904_4_1235, &P17904_5};	/*Property = ScreenPixelsWidth*/
const Profile P17904 = {"17904", &P17904_4};	/*Profile ID - 17904*/
const Value V16626_1_1 = {S2, NULL};	/*Value = True*/
const Property P16626_1 = {S50, 0, &V16626_1_1, NULL};	/*Property = IsMobile*/
const Value V16626_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16626_5 = {S52, 0, &V16626_5_1233, &P16626_1};	/*Property = ScreenPixelsHeight*/
const Value V16626_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16626_4 = {S54, 0, &V16626_4_1235, &P16626_5};	/*Property = ScreenPixelsWidth*/
const Profile P16626 = {"16626", &P16626_4};	/*Profile ID - 16626*/
const Value V17375_1_1 = {S2, NULL};	/*Value = True*/
const Property P17375_1 = {S50, 0, &V17375_1_1, NULL};	/*Property = IsMobile*/
const Value V17375_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P17375_5 = {S52, 0, &V17375_5_1294, &P17375_1};	/*Property = ScreenPixelsHeight*/
const Value V17375_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17375_4 = {S54, 0, &V17375_4_1296, &P17375_5};	/*Property = ScreenPixelsWidth*/
const Profile P17375 = {"17375", &P17375_4};	/*Profile ID - 17375*/
const Value V16838_1_1 = {S2, NULL};	/*Value = True*/
const Property P16838_1 = {S50, 0, &V16838_1_1, NULL};	/*Property = IsMobile*/
const Value V16838_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16838_5 = {S52, 0, &V16838_5_1233, &P16838_1};	/*Property = ScreenPixelsHeight*/
const Value V16838_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16838_4 = {S54, 0, &V16838_4_1235, &P16838_5};	/*Property = ScreenPixelsWidth*/
const Profile P16838 = {"16838", &P16838_4};	/*Profile ID - 16838*/
const Value V17726_1_1 = {S2, NULL};	/*Value = True*/
const Property P17726_1 = {S50, 0, &V17726_1_1, NULL};	/*Property = IsMobile*/
const Value V17726_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17726_5 = {S52, 0, &V17726_5_1269, &P17726_1};	/*Property = ScreenPixelsHeight*/
const Value V17726_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17726_4 = {S54, 0, &V17726_4_1229, &P17726_5};	/*Property = ScreenPixelsWidth*/
const Profile P17726 = {"17726", &P17726_4};	/*Profile ID - 17726*/
const Value V17734_1_1 = {S2, NULL};	/*Value = True*/
const Property P17734_1 = {S50, 0, &V17734_1_1, NULL};	/*Property = IsMobile*/
const Value V17734_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17734_5 = {S52, 0, &V17734_5_1269, &P17734_1};	/*Property = ScreenPixelsHeight*/
const Value V17734_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17734_4 = {S54, 0, &V17734_4_1229, &P17734_5};	/*Property = ScreenPixelsWidth*/
const Profile P17734 = {"17734", &P17734_4};	/*Profile ID - 17734*/
const Value V17554_1_1 = {S2, NULL};	/*Value = True*/
const Property P17554_1 = {S50, 0, &V17554_1_1, NULL};	/*Property = IsMobile*/
const Value V17554_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17554_5 = {S52, 0, &V17554_5_1269, &P17554_1};	/*Property = ScreenPixelsHeight*/
const Value V17554_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17554_4 = {S54, 0, &V17554_4_1229, &P17554_5};	/*Property = ScreenPixelsWidth*/
const Profile P17554 = {"17554", &P17554_4};	/*Profile ID - 17554*/
const Value V17748_1_1 = {S2, NULL};	/*Value = True*/
const Property P17748_1 = {S50, 0, &V17748_1_1, NULL};	/*Property = IsMobile*/
const Value V17748_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17748_5 = {S52, 0, &V17748_5_1269, &P17748_1};	/*Property = ScreenPixelsHeight*/
const Value V17748_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17748_4 = {S54, 0, &V17748_4_1229, &P17748_5};	/*Property = ScreenPixelsWidth*/
const Profile P17748 = {"17748", &P17748_4};	/*Profile ID - 17748*/
const Value V17580_1_1 = {S2, NULL};	/*Value = True*/
const Property P17580_1 = {S50, 0, &V17580_1_1, NULL};	/*Property = IsMobile*/
const Value V17580_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17580_5 = {S52, 0, &V17580_5_1269, &P17580_1};	/*Property = ScreenPixelsHeight*/
const Value V17580_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17580_4 = {S54, 0, &V17580_4_1229, &P17580_5};	/*Property = ScreenPixelsWidth*/
const Profile P17580 = {"17580", &P17580_4};	/*Profile ID - 17580*/
const Value V17699_1_1 = {S2, NULL};	/*Value = True*/
const Property P17699_1 = {S50, 0, &V17699_1_1, NULL};	/*Property = IsMobile*/
const Value V17699_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17699_5 = {S52, 0, &V17699_5_1269, &P17699_1};	/*Property = ScreenPixelsHeight*/
const Value V17699_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17699_4 = {S54, 0, &V17699_4_1229, &P17699_5};	/*Property = ScreenPixelsWidth*/
const Profile P17699 = {"17699", &P17699_4};	/*Profile ID - 17699*/
const Value V17594_1_1 = {S2, NULL};	/*Value = True*/
const Property P17594_1 = {S50, 0, &V17594_1_1, NULL};	/*Property = IsMobile*/
const Value V17594_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17594_5 = {S52, 0, &V17594_5_1269, &P17594_1};	/*Property = ScreenPixelsHeight*/
const Value V17594_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17594_4 = {S54, 0, &V17594_4_1229, &P17594_5};	/*Property = ScreenPixelsWidth*/
const Profile P17594 = {"17594", &P17594_4};	/*Profile ID - 17594*/
const Value V17759_1_1 = {S2, NULL};	/*Value = True*/
const Property P17759_1 = {S50, 0, &V17759_1_1, NULL};	/*Property = IsMobile*/
const Value V17759_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17759_5 = {S52, 0, &V17759_5_1269, &P17759_1};	/*Property = ScreenPixelsHeight*/
const Value V17759_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17759_4 = {S54, 0, &V17759_4_1229, &P17759_5};	/*Property = ScreenPixelsWidth*/
const Profile P17759 = {"17759", &P17759_4};	/*Profile ID - 17759*/
const Value V17733_1_1 = {S2, NULL};	/*Value = True*/
const Property P17733_1 = {S50, 0, &V17733_1_1, NULL};	/*Property = IsMobile*/
const Value V17733_5_37540 = {S105, NULL};	/*Value = 427*/
const Property P17733_5 = {S52, 0, &V17733_5_37540, &P17733_1};	/*Property = ScreenPixelsHeight*/
const Value V17733_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17733_4 = {S54, 0, &V17733_4_1224, &P17733_5};	/*Property = ScreenPixelsWidth*/
const Profile P17733 = {"17733", &P17733_4};	/*Profile ID - 17733*/
const Value V16984_1_1 = {S2, NULL};	/*Value = True*/
const Property P16984_1 = {S50, 0, &V16984_1_1, NULL};	/*Property = IsMobile*/
const Value V16984_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P16984_5 = {S52, 0, &V16984_5_1403, &P16984_1};	/*Property = ScreenPixelsHeight*/
const Value V16984_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P16984_4 = {S54, 0, &V16984_4_1404, &P16984_5};	/*Property = ScreenPixelsWidth*/
const Profile P16984 = {"16984", &P16984_4};	/*Profile ID - 16984*/
const Value V16808_1_1 = {S2, NULL};	/*Value = True*/
const Property P16808_1 = {S50, 0, &V16808_1_1, NULL};	/*Property = IsMobile*/
const Value V16808_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P16808_5 = {S52, 0, &V16808_5_1380, &P16808_1};	/*Property = ScreenPixelsHeight*/
const Value V16808_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16808_4 = {S54, 0, &V16808_4_1279, &P16808_5};	/*Property = ScreenPixelsWidth*/
const Profile P16808 = {"16808", &P16808_4};	/*Profile ID - 16808*/
const Value V17736_1_1 = {S2, NULL};	/*Value = True*/
const Property P17736_1 = {S50, 0, &V17736_1_1, NULL};	/*Property = IsMobile*/
const Value V17736_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17736_5 = {S52, 0, &V17736_5_1269, &P17736_1};	/*Property = ScreenPixelsHeight*/
const Value V17736_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17736_4 = {S54, 0, &V17736_4_1229, &P17736_5};	/*Property = ScreenPixelsWidth*/
const Profile P17736 = {"17736", &P17736_4};	/*Profile ID - 17736*/
const Value V17727_1_1 = {S2, NULL};	/*Value = True*/
const Property P17727_1 = {S50, 0, &V17727_1_1, NULL};	/*Property = IsMobile*/
const Value V17727_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17727_5 = {S52, 0, &V17727_5_1269, &P17727_1};	/*Property = ScreenPixelsHeight*/
const Value V17727_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17727_4 = {S54, 0, &V17727_4_1229, &P17727_5};	/*Property = ScreenPixelsWidth*/
const Profile P17727 = {"17727", &P17727_4};	/*Profile ID - 17727*/
const Value V17867_1_1 = {S2, NULL};	/*Value = True*/
const Property P17867_1 = {S50, 0, &V17867_1_1, NULL};	/*Property = IsMobile*/
const Value V17867_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17867_5 = {S52, 0, &V17867_5_1269, &P17867_1};	/*Property = ScreenPixelsHeight*/
const Value V17867_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17867_4 = {S54, 0, &V17867_4_1229, &P17867_5};	/*Property = ScreenPixelsWidth*/
const Profile P17867 = {"17867", &P17867_4};	/*Profile ID - 17867*/
const Value V17756_1_1 = {S2, NULL};	/*Value = True*/
const Property P17756_1 = {S50, 0, &V17756_1_1, NULL};	/*Property = IsMobile*/
const Value V17756_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17756_5 = {S52, 0, &V17756_5_1222, &P17756_1};	/*Property = ScreenPixelsHeight*/
const Value V17756_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17756_4 = {S54, 0, &V17756_4_1224, &P17756_5};	/*Property = ScreenPixelsWidth*/
const Profile P17756 = {"17756", &P17756_4};	/*Profile ID - 17756*/
const Value V17627_1_1 = {S2, NULL};	/*Value = True*/
const Property P17627_1 = {S50, 0, &V17627_1_1, NULL};	/*Property = IsMobile*/
const Value V17627_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17627_5 = {S52, 0, &V17627_5_1304, &P17627_1};	/*Property = ScreenPixelsHeight*/
const Value V17627_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17627_4 = {S54, 0, &V17627_4_1224, &P17627_5};	/*Property = ScreenPixelsWidth*/
const Profile P17627 = {"17627", &P17627_4};	/*Profile ID - 17627*/
const Value V17742_1_1 = {S2, NULL};	/*Value = True*/
const Property P17742_1 = {S50, 0, &V17742_1_1, NULL};	/*Property = IsMobile*/
const Value V17742_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17742_5 = {S52, 0, &V17742_5_1269, &P17742_1};	/*Property = ScreenPixelsHeight*/
const Value V17742_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17742_4 = {S54, 0, &V17742_4_1229, &P17742_5};	/*Property = ScreenPixelsWidth*/
const Profile P17742 = {"17742", &P17742_4};	/*Profile ID - 17742*/
const Value V17729_1_1 = {S2, NULL};	/*Value = True*/
const Property P17729_1 = {S50, 0, &V17729_1_1, NULL};	/*Property = IsMobile*/
const Value V17729_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17729_5 = {S52, 0, &V17729_5_1227, &P17729_1};	/*Property = ScreenPixelsHeight*/
const Value V17729_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17729_4 = {S54, 0, &V17729_4_1229, &P17729_5};	/*Property = ScreenPixelsWidth*/
const Profile P17729 = {"17729", &P17729_4};	/*Profile ID - 17729*/
const Value V17370_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_279 = {S1, 0, &V17370_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17370_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_282 = {S3, 0, &V17370_282_57155, &P17370_279};	/*Property = BlobBuilder*/
const Value V17370_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_350 = {S4, 0, &V17370_350_57446, &P17370_282};	/*Property = Canvas*/
const Value V17370_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_283 = {S5, 0, &V17370_283_57161, &P17370_350};	/*Property = CssBackground*/
const Value V17370_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_284 = {S6, 0, &V17370_284_57164, &P17370_283};	/*Property = CssBorderImage*/
const Value V17370_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_285 = {S7, 0, &V17370_285_57167, &P17370_284};	/*Property = CssCanvas*/
const Value V17370_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_286 = {S8, 0, &V17370_286_57170, &P17370_285};	/*Property = CssColor*/
const Value V17370_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_237 = {S9, 0, &V17370_237_55786, &P17370_286};	/*Property = CssColumn*/
const Value V17370_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_287 = {S10, 0, &V17370_287_57173, &P17370_237};	/*Property = CssFlexbox*/
const Value V17370_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_288 = {S11, 0, &V17370_288_57176, &P17370_287};	/*Property = CssFont*/
const Value V17370_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_289 = {S12, 0, &V17370_289_86899, &P17370_288};	/*Property = CssImages*/
const Value V17370_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_291 = {S13, 0, &V17370_291_57182, &P17370_289};	/*Property = CssMediaQueries*/
const Value V17370_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_292 = {S14, 0, &V17370_292_57185, &P17370_291};	/*Property = CssMinMax*/
const Value V17370_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_293 = {S15, 0, &V17370_293_57188, &P17370_292};	/*Property = CssOverflow*/
const Value V17370_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_294 = {S16, 0, &V17370_294_57191, &P17370_293};	/*Property = CssPosition*/
const Value V17370_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_295 = {S17, 0, &V17370_295_57194, &P17370_294};	/*Property = CssText*/
const Value V17370_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_297 = {S18, 0, &V17370_297_57197, &P17370_295};	/*Property = CssTransforms*/
const Value V17370_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_241 = {S19, 0, &V17370_241_55801, &P17370_297};	/*Property = CssTransitions*/
const Value V17370_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_298 = {S20, 0, &V17370_298_57208, &P17370_241};	/*Property = CssUI*/
const Value V17370_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_300 = {S21, 0, &V17370_300_57217, &P17370_298};	/*Property = DataSet*/
const Value V17370_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_301 = {S22, 0, &V17370_301_57220, &P17370_300};	/*Property = DataUrl*/
const Value V17370_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_302 = {S23, 0, &V17370_302_57223, &P17370_301};	/*Property = DeviceOrientation*/
const Value V17370_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_304 = {S24, 0, &V17370_304_57229, &P17370_302};	/*Property = FileReader*/
const Value V17370_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_305 = {S25, 0, &V17370_305_57232, &P17370_304};	/*Property = FileSaver*/
const Value V17370_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_306 = {S26, 0, &V17370_306_57238, &P17370_305};	/*Property = FileWriter*/
const Value V17370_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_307 = {S27, 0, &V17370_307_57241, &P17370_306};	/*Property = FormData*/
const Value V17370_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_309 = {S28, 0, &V17370_309_57247, &P17370_307};	/*Property = Fullscreen*/
const Value V17370_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_310 = {S29, 0, &V17370_310_57250, &P17370_309};	/*Property = GeoLocation*/
const Value V17370_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_312 = {S30, 0, &V17370_312_57256, &P17370_310};	/*Property = History*/
const Value V17370_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_314 = {S31, 0, &V17370_314_57262, &P17370_312};	/*Property = Html5*/
const Value V17370_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_313 = {S32, 0, &V17370_313_57259, &P17370_314};	/*Property = Html-Media-Capture*/
const Value V17370_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_315 = {S33, 0, &V17370_315_57265, &P17370_313};	/*Property = Iframe*/
const Value V17370_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_316 = {S34, 0, &V17370_316_57268, &P17370_315};	/*Property = IndexedDB*/
const Value V17370_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_317 = {S35, 0, &V17370_317_57271, &P17370_316};	/*Property = Json*/
const Value V17370_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P17370_15 = {S37, 0, &V17370_15_31, &P17370_317};	/*Property = LayoutEngine*/
const Value V17370_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_344 = {S38, 0, &V17370_344_57274, &P17370_15};	/*Property = Masking*/
const Value V17370_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_320 = {S39, 0, &V17370_320_57292, &P17370_344};	/*Property = PostMessage*/
const Value V17370_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_321 = {S40, 0, &V17370_321_57295, &P17370_320};	/*Property = Progress*/
const Value V17370_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_322 = {S41, 0, &V17370_322_57298, &P17370_321};	/*Property = Prompts*/
const Value V17370_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_323 = {S42, 0, &V17370_323_57301, &P17370_322};	/*Property = Selector*/
const Value V17370_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_324 = {S43, 0, &V17370_324_57307, &P17370_323};	/*Property = Svg*/
const Value V17370_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_327 = {S44, 0, &V17370_327_57316, &P17370_324};	/*Property = TouchEvents*/
const Value V17370_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_328 = {S45, 0, &V17370_328_57319, &P17370_327};	/*Property = Track*/
const Value V17370_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_329 = {S46, 0, &V17370_329_57325, &P17370_328};	/*Property = Video*/
const Value V17370_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_330 = {S47, 0, &V17370_330_57328, &P17370_329};	/*Property = Viewport*/
const Value V17370_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_332 = {S48, 0, &V17370_332_57340, &P17370_330};	/*Property = WebWorkers*/
const Value V17370_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17370_333 = {S49, 0, &V17370_333_57343, &P17370_332};	/*Property = Xhr2*/
const Profile P17370 = {"17370", &P17370_333};	/*Profile ID - 17370*/
const Value V17741_1_1 = {S2, NULL};	/*Value = True*/
const Property P17741_1 = {S50, 0, &V17741_1_1, NULL};	/*Property = IsMobile*/
const Value V17741_5_37540 = {S105, NULL};	/*Value = 427*/
const Property P17741_5 = {S52, 0, &V17741_5_37540, &P17741_1};	/*Property = ScreenPixelsHeight*/
const Value V17741_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17741_4 = {S54, 0, &V17741_4_1224, &P17741_5};	/*Property = ScreenPixelsWidth*/
const Profile P17741 = {"17741", &P17741_4};	/*Profile ID - 17741*/
const Value V17746_1_1 = {S2, NULL};	/*Value = True*/
const Property P17746_1 = {S50, 0, &V17746_1_1, NULL};	/*Property = IsMobile*/
const Value V17746_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17746_5 = {S52, 0, &V17746_5_1269, &P17746_1};	/*Property = ScreenPixelsHeight*/
const Value V17746_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17746_4 = {S54, 0, &V17746_4_1229, &P17746_5};	/*Property = ScreenPixelsWidth*/
const Profile P17746 = {"17746", &P17746_4};	/*Profile ID - 17746*/
const Value V17875_1_1 = {S2, NULL};	/*Value = True*/
const Property P17875_1 = {S50, 0, &V17875_1_1, NULL};	/*Property = IsMobile*/
const Value V17875_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17875_5 = {S52, 0, &V17875_5_1269, &P17875_1};	/*Property = ScreenPixelsHeight*/
const Value V17875_4_51276 = {S124, NULL};	/*Value = 37*/
const Property P17875_4 = {S54, 0, &V17875_4_51276, &P17875_5};	/*Property = ScreenPixelsWidth*/
const Profile P17875 = {"17875", &P17875_4};	/*Profile ID - 17875*/
const Value V16757_1_1 = {S2, NULL};	/*Value = True*/
const Property P16757_1 = {S50, 0, &V16757_1_1, NULL};	/*Property = IsMobile*/
const Value V16757_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16757_5 = {S52, 0, &V16757_5_1227, &P16757_1};	/*Property = ScreenPixelsHeight*/
const Value V16757_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16757_4 = {S54, 0, &V16757_4_1229, &P16757_5};	/*Property = ScreenPixelsWidth*/
const Profile P16757 = {"16757", &P16757_4};	/*Profile ID - 16757*/
const Value V13110_1_1 = {S2, NULL};	/*Value = True*/
const Property P13110_1 = {S50, 0, &V13110_1_1, NULL};	/*Property = IsMobile*/
const Value V13110_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P13110_5 = {S52, 0, &V13110_5_37, &P13110_1};	/*Property = ScreenPixelsHeight*/
const Value V13110_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13110_4 = {S54, 0, &V13110_4_1224, &P13110_5};	/*Property = ScreenPixelsWidth*/
const Profile P13110 = {"13110", &P13110_4};	/*Profile ID - 13110*/
const Value V16600_1_1 = {S2, NULL};	/*Value = True*/
const Property P16600_1 = {S50, 0, &V16600_1_1, NULL};	/*Property = IsMobile*/
const Value V16600_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16600_5 = {S52, 0, &V16600_5_1222, &P16600_1};	/*Property = ScreenPixelsHeight*/
const Value V16600_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16600_4 = {S54, 0, &V16600_4_1224, &P16600_5};	/*Property = ScreenPixelsWidth*/
const Profile P16600 = {"16600", &P16600_4};	/*Profile ID - 16600*/
const Value V17744_1_1 = {S2, NULL};	/*Value = True*/
const Property P17744_1 = {S50, 0, &V17744_1_1, NULL};	/*Property = IsMobile*/
const Value V17744_5_37540 = {S105, NULL};	/*Value = 427*/
const Property P17744_5 = {S52, 0, &V17744_5_37540, &P17744_1};	/*Property = ScreenPixelsHeight*/
const Value V17744_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17744_4 = {S54, 0, &V17744_4_1224, &P17744_5};	/*Property = ScreenPixelsWidth*/
const Profile P17744 = {"17744", &P17744_4};	/*Profile ID - 17744*/
const Value V17591_1_1 = {S2, NULL};	/*Value = True*/
const Property P17591_1 = {S50, 0, &V17591_1_1, NULL};	/*Property = IsMobile*/
const Value V17591_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17591_5 = {S52, 0, &V17591_5_1269, &P17591_1};	/*Property = ScreenPixelsHeight*/
const Value V17591_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17591_4 = {S54, 0, &V17591_4_1229, &P17591_5};	/*Property = ScreenPixelsWidth*/
const Profile P17591 = {"17591", &P17591_4};	/*Profile ID - 17591*/
const Value V18284_1_1 = {S2, NULL};	/*Value = True*/
const Property P18284_1 = {S50, 0, &V18284_1_1, NULL};	/*Property = IsMobile*/
const Value V18284_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18284_5 = {S52, 0, &V18284_5_1222, &P18284_1};	/*Property = ScreenPixelsHeight*/
const Value V18284_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18284_4 = {S54, 0, &V18284_4_1224, &P18284_5};	/*Property = ScreenPixelsWidth*/
const Profile P18284 = {"18284", &P18284_4};	/*Profile ID - 18284*/
const Value V17761_1_1 = {S2, NULL};	/*Value = True*/
const Property P17761_1 = {S50, 0, &V17761_1_1, NULL};	/*Property = IsMobile*/
const Value V17761_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17761_5 = {S52, 0, &V17761_5_1269, &P17761_1};	/*Property = ScreenPixelsHeight*/
const Value V17761_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17761_4 = {S54, 0, &V17761_4_1229, &P17761_5};	/*Property = ScreenPixelsWidth*/
const Profile P17761 = {"17761", &P17761_4};	/*Profile ID - 17761*/
const Value V17581_1_1 = {S2, NULL};	/*Value = True*/
const Property P17581_1 = {S50, 0, &V17581_1_1, NULL};	/*Property = IsMobile*/
const Value V17581_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17581_5 = {S52, 0, &V17581_5_1269, &P17581_1};	/*Property = ScreenPixelsHeight*/
const Value V17581_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17581_4 = {S54, 0, &V17581_4_1229, &P17581_5};	/*Property = ScreenPixelsWidth*/
const Profile P17581 = {"17581", &P17581_4};	/*Profile ID - 17581*/
const Value V18272_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_279 = {S1, 0, &V18272_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18272_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_282 = {S3, 0, &V18272_282_57155, &P18272_279};	/*Property = BlobBuilder*/
const Value V18272_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_350 = {S4, 0, &V18272_350_57446, &P18272_282};	/*Property = Canvas*/
const Value V18272_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_283 = {S5, 0, &V18272_283_57161, &P18272_350};	/*Property = CssBackground*/
const Value V18272_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_284 = {S6, 0, &V18272_284_57164, &P18272_283};	/*Property = CssBorderImage*/
const Value V18272_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_285 = {S7, 0, &V18272_285_57167, &P18272_284};	/*Property = CssCanvas*/
const Value V18272_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_286 = {S8, 0, &V18272_286_57170, &P18272_285};	/*Property = CssColor*/
const Value V18272_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_237 = {S9, 0, &V18272_237_55786, &P18272_286};	/*Property = CssColumn*/
const Value V18272_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_287 = {S10, 0, &V18272_287_57173, &P18272_237};	/*Property = CssFlexbox*/
const Value V18272_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_288 = {S11, 0, &V18272_288_57176, &P18272_287};	/*Property = CssFont*/
const Value V18272_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_289 = {S12, 0, &V18272_289_86899, &P18272_288};	/*Property = CssImages*/
const Value V18272_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_291 = {S13, 0, &V18272_291_57182, &P18272_289};	/*Property = CssMediaQueries*/
const Value V18272_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_292 = {S14, 0, &V18272_292_57185, &P18272_291};	/*Property = CssMinMax*/
const Value V18272_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_293 = {S15, 0, &V18272_293_57188, &P18272_292};	/*Property = CssOverflow*/
const Value V18272_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_294 = {S16, 0, &V18272_294_57191, &P18272_293};	/*Property = CssPosition*/
const Value V18272_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_295 = {S17, 0, &V18272_295_57194, &P18272_294};	/*Property = CssText*/
const Value V18272_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_297 = {S18, 0, &V18272_297_57197, &P18272_295};	/*Property = CssTransforms*/
const Value V18272_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_241 = {S19, 0, &V18272_241_55801, &P18272_297};	/*Property = CssTransitions*/
const Value V18272_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_298 = {S20, 0, &V18272_298_57208, &P18272_241};	/*Property = CssUI*/
const Value V18272_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_300 = {S21, 0, &V18272_300_57217, &P18272_298};	/*Property = DataSet*/
const Value V18272_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_301 = {S22, 0, &V18272_301_57220, &P18272_300};	/*Property = DataUrl*/
const Value V18272_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_302 = {S23, 0, &V18272_302_57223, &P18272_301};	/*Property = DeviceOrientation*/
const Value V18272_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_304 = {S24, 0, &V18272_304_57229, &P18272_302};	/*Property = FileReader*/
const Value V18272_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_305 = {S25, 0, &V18272_305_57232, &P18272_304};	/*Property = FileSaver*/
const Value V18272_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_306 = {S26, 0, &V18272_306_57238, &P18272_305};	/*Property = FileWriter*/
const Value V18272_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_307 = {S27, 0, &V18272_307_57241, &P18272_306};	/*Property = FormData*/
const Value V18272_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_309 = {S28, 0, &V18272_309_57247, &P18272_307};	/*Property = Fullscreen*/
const Value V18272_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_310 = {S29, 0, &V18272_310_57250, &P18272_309};	/*Property = GeoLocation*/
const Value V18272_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_312 = {S30, 0, &V18272_312_57256, &P18272_310};	/*Property = History*/
const Value V18272_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_314 = {S31, 0, &V18272_314_57262, &P18272_312};	/*Property = Html5*/
const Value V18272_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_313 = {S32, 0, &V18272_313_57259, &P18272_314};	/*Property = Html-Media-Capture*/
const Value V18272_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_315 = {S33, 0, &V18272_315_57265, &P18272_313};	/*Property = Iframe*/
const Value V18272_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_316 = {S34, 0, &V18272_316_57268, &P18272_315};	/*Property = IndexedDB*/
const Value V18272_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_317 = {S35, 0, &V18272_317_57271, &P18272_316};	/*Property = Json*/
const Value V18272_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18272_15 = {S37, 0, &V18272_15_31, &P18272_317};	/*Property = LayoutEngine*/
const Value V18272_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_344 = {S38, 0, &V18272_344_57274, &P18272_15};	/*Property = Masking*/
const Value V18272_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_320 = {S39, 0, &V18272_320_57292, &P18272_344};	/*Property = PostMessage*/
const Value V18272_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_321 = {S40, 0, &V18272_321_57295, &P18272_320};	/*Property = Progress*/
const Value V18272_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_322 = {S41, 0, &V18272_322_57298, &P18272_321};	/*Property = Prompts*/
const Value V18272_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_323 = {S42, 0, &V18272_323_57301, &P18272_322};	/*Property = Selector*/
const Value V18272_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_324 = {S43, 0, &V18272_324_57307, &P18272_323};	/*Property = Svg*/
const Value V18272_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_327 = {S44, 0, &V18272_327_57316, &P18272_324};	/*Property = TouchEvents*/
const Value V18272_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_328 = {S45, 0, &V18272_328_57319, &P18272_327};	/*Property = Track*/
const Value V18272_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_329 = {S46, 0, &V18272_329_57325, &P18272_328};	/*Property = Video*/
const Value V18272_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_330 = {S47, 0, &V18272_330_57328, &P18272_329};	/*Property = Viewport*/
const Value V18272_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_332 = {S48, 0, &V18272_332_57340, &P18272_330};	/*Property = WebWorkers*/
const Value V18272_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18272_333 = {S49, 0, &V18272_333_57343, &P18272_332};	/*Property = Xhr2*/
const Profile P18272 = {"18272", &P18272_333};	/*Profile ID - 18272*/
const Value V22573_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_279 = {S1, 0, &V22573_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22573_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_282 = {S3, 0, &V22573_282_57155, &P22573_279};	/*Property = BlobBuilder*/
const Value V22573_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_350 = {S4, 0, &V22573_350_57446, &P22573_282};	/*Property = Canvas*/
const Value V22573_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_283 = {S5, 0, &V22573_283_57161, &P22573_350};	/*Property = CssBackground*/
const Value V22573_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_284 = {S6, 0, &V22573_284_57164, &P22573_283};	/*Property = CssBorderImage*/
const Value V22573_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_285 = {S7, 0, &V22573_285_57167, &P22573_284};	/*Property = CssCanvas*/
const Value V22573_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_286 = {S8, 0, &V22573_286_57170, &P22573_285};	/*Property = CssColor*/
const Value V22573_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_237 = {S9, 0, &V22573_237_55786, &P22573_286};	/*Property = CssColumn*/
const Value V22573_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_287 = {S10, 0, &V22573_287_57173, &P22573_237};	/*Property = CssFlexbox*/
const Value V22573_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_288 = {S11, 0, &V22573_288_57176, &P22573_287};	/*Property = CssFont*/
const Value V22573_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_289 = {S12, 0, &V22573_289_86899, &P22573_288};	/*Property = CssImages*/
const Value V22573_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_291 = {S13, 0, &V22573_291_57182, &P22573_289};	/*Property = CssMediaQueries*/
const Value V22573_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_292 = {S14, 0, &V22573_292_57185, &P22573_291};	/*Property = CssMinMax*/
const Value V22573_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_293 = {S15, 0, &V22573_293_57188, &P22573_292};	/*Property = CssOverflow*/
const Value V22573_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_294 = {S16, 0, &V22573_294_57191, &P22573_293};	/*Property = CssPosition*/
const Value V22573_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_295 = {S17, 0, &V22573_295_57194, &P22573_294};	/*Property = CssText*/
const Value V22573_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_297 = {S18, 0, &V22573_297_57197, &P22573_295};	/*Property = CssTransforms*/
const Value V22573_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_241 = {S19, 0, &V22573_241_55801, &P22573_297};	/*Property = CssTransitions*/
const Value V22573_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_298 = {S20, 0, &V22573_298_57208, &P22573_241};	/*Property = CssUI*/
const Value V22573_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_300 = {S21, 0, &V22573_300_57217, &P22573_298};	/*Property = DataSet*/
const Value V22573_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_301 = {S22, 0, &V22573_301_57220, &P22573_300};	/*Property = DataUrl*/
const Value V22573_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_302 = {S23, 0, &V22573_302_57223, &P22573_301};	/*Property = DeviceOrientation*/
const Value V22573_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_304 = {S24, 0, &V22573_304_57229, &P22573_302};	/*Property = FileReader*/
const Value V22573_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_305 = {S25, 0, &V22573_305_57232, &P22573_304};	/*Property = FileSaver*/
const Value V22573_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_306 = {S26, 0, &V22573_306_57238, &P22573_305};	/*Property = FileWriter*/
const Value V22573_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_307 = {S27, 0, &V22573_307_57241, &P22573_306};	/*Property = FormData*/
const Value V22573_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_309 = {S28, 0, &V22573_309_57247, &P22573_307};	/*Property = Fullscreen*/
const Value V22573_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_310 = {S29, 0, &V22573_310_57250, &P22573_309};	/*Property = GeoLocation*/
const Value V22573_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_312 = {S30, 0, &V22573_312_57256, &P22573_310};	/*Property = History*/
const Value V22573_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_314 = {S31, 0, &V22573_314_57262, &P22573_312};	/*Property = Html5*/
const Value V22573_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_313 = {S32, 0, &V22573_313_57259, &P22573_314};	/*Property = Html-Media-Capture*/
const Value V22573_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_315 = {S33, 0, &V22573_315_57265, &P22573_313};	/*Property = Iframe*/
const Value V22573_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_316 = {S34, 0, &V22573_316_57268, &P22573_315};	/*Property = IndexedDB*/
const Value V22573_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_317 = {S35, 0, &V22573_317_57271, &P22573_316};	/*Property = Json*/
const Value V22573_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_15 = {S37, 0, &V22573_15_72, &P22573_317};	/*Property = LayoutEngine*/
const Value V22573_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_344 = {S38, 0, &V22573_344_57274, &P22573_15};	/*Property = Masking*/
const Value V22573_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_320 = {S39, 0, &V22573_320_57292, &P22573_344};	/*Property = PostMessage*/
const Value V22573_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_321 = {S40, 0, &V22573_321_57295, &P22573_320};	/*Property = Progress*/
const Value V22573_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_322 = {S41, 0, &V22573_322_57298, &P22573_321};	/*Property = Prompts*/
const Value V22573_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_323 = {S42, 0, &V22573_323_57301, &P22573_322};	/*Property = Selector*/
const Value V22573_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_324 = {S43, 0, &V22573_324_57307, &P22573_323};	/*Property = Svg*/
const Value V22573_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_327 = {S44, 0, &V22573_327_57316, &P22573_324};	/*Property = TouchEvents*/
const Value V22573_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_328 = {S45, 0, &V22573_328_57319, &P22573_327};	/*Property = Track*/
const Value V22573_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_329 = {S46, 0, &V22573_329_57325, &P22573_328};	/*Property = Video*/
const Value V22573_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_330 = {S47, 0, &V22573_330_57328, &P22573_329};	/*Property = Viewport*/
const Value V22573_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_332 = {S48, 0, &V22573_332_57340, &P22573_330};	/*Property = WebWorkers*/
const Value V22573_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22573_333 = {S49, 0, &V22573_333_57343, &P22573_332};	/*Property = Xhr2*/
const Profile P22573 = {"22573", &P22573_333};	/*Profile ID - 22573*/
const Value V18123_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_279 = {S1, 0, &V18123_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18123_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_282 = {S3, 0, &V18123_282_57155, &P18123_279};	/*Property = BlobBuilder*/
const Value V18123_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_350 = {S4, 0, &V18123_350_57446, &P18123_282};	/*Property = Canvas*/
const Value V18123_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_283 = {S5, 0, &V18123_283_57161, &P18123_350};	/*Property = CssBackground*/
const Value V18123_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_284 = {S6, 0, &V18123_284_57164, &P18123_283};	/*Property = CssBorderImage*/
const Value V18123_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_285 = {S7, 0, &V18123_285_57167, &P18123_284};	/*Property = CssCanvas*/
const Value V18123_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_286 = {S8, 0, &V18123_286_57170, &P18123_285};	/*Property = CssColor*/
const Value V18123_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_237 = {S9, 0, &V18123_237_55786, &P18123_286};	/*Property = CssColumn*/
const Value V18123_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_287 = {S10, 0, &V18123_287_57173, &P18123_237};	/*Property = CssFlexbox*/
const Value V18123_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_288 = {S11, 0, &V18123_288_57176, &P18123_287};	/*Property = CssFont*/
const Value V18123_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_289 = {S12, 0, &V18123_289_86899, &P18123_288};	/*Property = CssImages*/
const Value V18123_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_291 = {S13, 0, &V18123_291_57182, &P18123_289};	/*Property = CssMediaQueries*/
const Value V18123_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_292 = {S14, 0, &V18123_292_57185, &P18123_291};	/*Property = CssMinMax*/
const Value V18123_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_293 = {S15, 0, &V18123_293_57188, &P18123_292};	/*Property = CssOverflow*/
const Value V18123_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_294 = {S16, 0, &V18123_294_57191, &P18123_293};	/*Property = CssPosition*/
const Value V18123_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_295 = {S17, 0, &V18123_295_57194, &P18123_294};	/*Property = CssText*/
const Value V18123_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_297 = {S18, 0, &V18123_297_57197, &P18123_295};	/*Property = CssTransforms*/
const Value V18123_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_241 = {S19, 0, &V18123_241_55801, &P18123_297};	/*Property = CssTransitions*/
const Value V18123_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_298 = {S20, 0, &V18123_298_57208, &P18123_241};	/*Property = CssUI*/
const Value V18123_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_300 = {S21, 0, &V18123_300_57217, &P18123_298};	/*Property = DataSet*/
const Value V18123_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_301 = {S22, 0, &V18123_301_57220, &P18123_300};	/*Property = DataUrl*/
const Value V18123_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_302 = {S23, 0, &V18123_302_57223, &P18123_301};	/*Property = DeviceOrientation*/
const Value V18123_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_304 = {S24, 0, &V18123_304_57229, &P18123_302};	/*Property = FileReader*/
const Value V18123_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_305 = {S25, 0, &V18123_305_57232, &P18123_304};	/*Property = FileSaver*/
const Value V18123_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_306 = {S26, 0, &V18123_306_57238, &P18123_305};	/*Property = FileWriter*/
const Value V18123_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_307 = {S27, 0, &V18123_307_57241, &P18123_306};	/*Property = FormData*/
const Value V18123_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_309 = {S28, 0, &V18123_309_57247, &P18123_307};	/*Property = Fullscreen*/
const Value V18123_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_310 = {S29, 0, &V18123_310_57250, &P18123_309};	/*Property = GeoLocation*/
const Value V18123_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_312 = {S30, 0, &V18123_312_57256, &P18123_310};	/*Property = History*/
const Value V18123_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_314 = {S31, 0, &V18123_314_57262, &P18123_312};	/*Property = Html5*/
const Value V18123_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_313 = {S32, 0, &V18123_313_57259, &P18123_314};	/*Property = Html-Media-Capture*/
const Value V18123_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_315 = {S33, 0, &V18123_315_57265, &P18123_313};	/*Property = Iframe*/
const Value V18123_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_316 = {S34, 0, &V18123_316_57268, &P18123_315};	/*Property = IndexedDB*/
const Value V18123_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_317 = {S35, 0, &V18123_317_57271, &P18123_316};	/*Property = Json*/
const Value V18123_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18123_15 = {S37, 0, &V18123_15_31, &P18123_317};	/*Property = LayoutEngine*/
const Value V18123_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_344 = {S38, 0, &V18123_344_57274, &P18123_15};	/*Property = Masking*/
const Value V18123_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_320 = {S39, 0, &V18123_320_57292, &P18123_344};	/*Property = PostMessage*/
const Value V18123_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_321 = {S40, 0, &V18123_321_57295, &P18123_320};	/*Property = Progress*/
const Value V18123_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_322 = {S41, 0, &V18123_322_57298, &P18123_321};	/*Property = Prompts*/
const Value V18123_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_323 = {S42, 0, &V18123_323_57301, &P18123_322};	/*Property = Selector*/
const Value V18123_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_324 = {S43, 0, &V18123_324_57307, &P18123_323};	/*Property = Svg*/
const Value V18123_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_327 = {S44, 0, &V18123_327_57316, &P18123_324};	/*Property = TouchEvents*/
const Value V18123_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_328 = {S45, 0, &V18123_328_57319, &P18123_327};	/*Property = Track*/
const Value V18123_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_329 = {S46, 0, &V18123_329_57325, &P18123_328};	/*Property = Video*/
const Value V18123_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_330 = {S47, 0, &V18123_330_57328, &P18123_329};	/*Property = Viewport*/
const Value V18123_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_332 = {S48, 0, &V18123_332_57340, &P18123_330};	/*Property = WebWorkers*/
const Value V18123_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18123_333 = {S49, 0, &V18123_333_57343, &P18123_332};	/*Property = Xhr2*/
const Profile P18123 = {"18123", &P18123_333};	/*Profile ID - 18123*/
const Value V17367_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_279 = {S1, 0, &V17367_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17367_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_282 = {S3, 0, &V17367_282_57155, &P17367_279};	/*Property = BlobBuilder*/
const Value V17367_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_350 = {S4, 0, &V17367_350_57446, &P17367_282};	/*Property = Canvas*/
const Value V17367_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_283 = {S5, 0, &V17367_283_57161, &P17367_350};	/*Property = CssBackground*/
const Value V17367_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_284 = {S6, 0, &V17367_284_57164, &P17367_283};	/*Property = CssBorderImage*/
const Value V17367_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_285 = {S7, 0, &V17367_285_57167, &P17367_284};	/*Property = CssCanvas*/
const Value V17367_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_286 = {S8, 0, &V17367_286_57170, &P17367_285};	/*Property = CssColor*/
const Value V17367_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_237 = {S9, 0, &V17367_237_55786, &P17367_286};	/*Property = CssColumn*/
const Value V17367_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_287 = {S10, 0, &V17367_287_57173, &P17367_237};	/*Property = CssFlexbox*/
const Value V17367_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_288 = {S11, 0, &V17367_288_57176, &P17367_287};	/*Property = CssFont*/
const Value V17367_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_289 = {S12, 0, &V17367_289_86899, &P17367_288};	/*Property = CssImages*/
const Value V17367_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_291 = {S13, 0, &V17367_291_57182, &P17367_289};	/*Property = CssMediaQueries*/
const Value V17367_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_292 = {S14, 0, &V17367_292_57185, &P17367_291};	/*Property = CssMinMax*/
const Value V17367_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_293 = {S15, 0, &V17367_293_57188, &P17367_292};	/*Property = CssOverflow*/
const Value V17367_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_294 = {S16, 0, &V17367_294_57191, &P17367_293};	/*Property = CssPosition*/
const Value V17367_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_295 = {S17, 0, &V17367_295_57194, &P17367_294};	/*Property = CssText*/
const Value V17367_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_297 = {S18, 0, &V17367_297_57197, &P17367_295};	/*Property = CssTransforms*/
const Value V17367_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_241 = {S19, 0, &V17367_241_55801, &P17367_297};	/*Property = CssTransitions*/
const Value V17367_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_298 = {S20, 0, &V17367_298_57208, &P17367_241};	/*Property = CssUI*/
const Value V17367_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_300 = {S21, 0, &V17367_300_57217, &P17367_298};	/*Property = DataSet*/
const Value V17367_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_301 = {S22, 0, &V17367_301_57220, &P17367_300};	/*Property = DataUrl*/
const Value V17367_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_302 = {S23, 0, &V17367_302_57223, &P17367_301};	/*Property = DeviceOrientation*/
const Value V17367_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_304 = {S24, 0, &V17367_304_57229, &P17367_302};	/*Property = FileReader*/
const Value V17367_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_305 = {S25, 0, &V17367_305_57232, &P17367_304};	/*Property = FileSaver*/
const Value V17367_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_306 = {S26, 0, &V17367_306_57238, &P17367_305};	/*Property = FileWriter*/
const Value V17367_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_307 = {S27, 0, &V17367_307_57241, &P17367_306};	/*Property = FormData*/
const Value V17367_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_309 = {S28, 0, &V17367_309_57247, &P17367_307};	/*Property = Fullscreen*/
const Value V17367_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_310 = {S29, 0, &V17367_310_57250, &P17367_309};	/*Property = GeoLocation*/
const Value V17367_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_312 = {S30, 0, &V17367_312_57256, &P17367_310};	/*Property = History*/
const Value V17367_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_314 = {S31, 0, &V17367_314_57262, &P17367_312};	/*Property = Html5*/
const Value V17367_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_313 = {S32, 0, &V17367_313_57259, &P17367_314};	/*Property = Html-Media-Capture*/
const Value V17367_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_315 = {S33, 0, &V17367_315_57265, &P17367_313};	/*Property = Iframe*/
const Value V17367_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_316 = {S34, 0, &V17367_316_57268, &P17367_315};	/*Property = IndexedDB*/
const Value V17367_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_317 = {S35, 0, &V17367_317_57271, &P17367_316};	/*Property = Json*/
const Value V17367_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P17367_15 = {S37, 0, &V17367_15_31, &P17367_317};	/*Property = LayoutEngine*/
const Value V17367_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_344 = {S38, 0, &V17367_344_57274, &P17367_15};	/*Property = Masking*/
const Value V17367_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_320 = {S39, 0, &V17367_320_57292, &P17367_344};	/*Property = PostMessage*/
const Value V17367_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_321 = {S40, 0, &V17367_321_57295, &P17367_320};	/*Property = Progress*/
const Value V17367_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_322 = {S41, 0, &V17367_322_57298, &P17367_321};	/*Property = Prompts*/
const Value V17367_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_323 = {S42, 0, &V17367_323_57301, &P17367_322};	/*Property = Selector*/
const Value V17367_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_324 = {S43, 0, &V17367_324_57307, &P17367_323};	/*Property = Svg*/
const Value V17367_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_327 = {S44, 0, &V17367_327_57316, &P17367_324};	/*Property = TouchEvents*/
const Value V17367_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_328 = {S45, 0, &V17367_328_57319, &P17367_327};	/*Property = Track*/
const Value V17367_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_329 = {S46, 0, &V17367_329_57325, &P17367_328};	/*Property = Video*/
const Value V17367_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_330 = {S47, 0, &V17367_330_57328, &P17367_329};	/*Property = Viewport*/
const Value V17367_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_332 = {S48, 0, &V17367_332_57340, &P17367_330};	/*Property = WebWorkers*/
const Value V17367_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17367_333 = {S49, 0, &V17367_333_57343, &P17367_332};	/*Property = Xhr2*/
const Profile P17367 = {"17367", &P17367_333};	/*Profile ID - 17367*/
const Value V21177_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_279 = {S1, 0, &V21177_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21177_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_282 = {S3, 0, &V21177_282_57155, &P21177_279};	/*Property = BlobBuilder*/
const Value V21177_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_350 = {S4, 0, &V21177_350_57446, &P21177_282};	/*Property = Canvas*/
const Value V21177_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_283 = {S5, 0, &V21177_283_57161, &P21177_350};	/*Property = CssBackground*/
const Value V21177_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_284 = {S6, 0, &V21177_284_57164, &P21177_283};	/*Property = CssBorderImage*/
const Value V21177_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_285 = {S7, 0, &V21177_285_57167, &P21177_284};	/*Property = CssCanvas*/
const Value V21177_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_286 = {S8, 0, &V21177_286_57170, &P21177_285};	/*Property = CssColor*/
const Value V21177_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_237 = {S9, 0, &V21177_237_55786, &P21177_286};	/*Property = CssColumn*/
const Value V21177_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_287 = {S10, 0, &V21177_287_57173, &P21177_237};	/*Property = CssFlexbox*/
const Value V21177_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_288 = {S11, 0, &V21177_288_57176, &P21177_287};	/*Property = CssFont*/
const Value V21177_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_289 = {S12, 0, &V21177_289_86899, &P21177_288};	/*Property = CssImages*/
const Value V21177_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_291 = {S13, 0, &V21177_291_57182, &P21177_289};	/*Property = CssMediaQueries*/
const Value V21177_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_292 = {S14, 0, &V21177_292_57185, &P21177_291};	/*Property = CssMinMax*/
const Value V21177_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_293 = {S15, 0, &V21177_293_57188, &P21177_292};	/*Property = CssOverflow*/
const Value V21177_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_294 = {S16, 0, &V21177_294_57191, &P21177_293};	/*Property = CssPosition*/
const Value V21177_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_295 = {S17, 0, &V21177_295_57194, &P21177_294};	/*Property = CssText*/
const Value V21177_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_297 = {S18, 0, &V21177_297_57197, &P21177_295};	/*Property = CssTransforms*/
const Value V21177_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_241 = {S19, 0, &V21177_241_55801, &P21177_297};	/*Property = CssTransitions*/
const Value V21177_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_298 = {S20, 0, &V21177_298_57208, &P21177_241};	/*Property = CssUI*/
const Value V21177_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_300 = {S21, 0, &V21177_300_57217, &P21177_298};	/*Property = DataSet*/
const Value V21177_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_301 = {S22, 0, &V21177_301_57220, &P21177_300};	/*Property = DataUrl*/
const Value V21177_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_302 = {S23, 0, &V21177_302_57223, &P21177_301};	/*Property = DeviceOrientation*/
const Value V21177_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_304 = {S24, 0, &V21177_304_57229, &P21177_302};	/*Property = FileReader*/
const Value V21177_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_305 = {S25, 0, &V21177_305_57232, &P21177_304};	/*Property = FileSaver*/
const Value V21177_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_306 = {S26, 0, &V21177_306_57238, &P21177_305};	/*Property = FileWriter*/
const Value V21177_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_307 = {S27, 0, &V21177_307_57241, &P21177_306};	/*Property = FormData*/
const Value V21177_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_309 = {S28, 0, &V21177_309_57247, &P21177_307};	/*Property = Fullscreen*/
const Value V21177_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_310 = {S29, 0, &V21177_310_57250, &P21177_309};	/*Property = GeoLocation*/
const Value V21177_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_312 = {S30, 0, &V21177_312_57256, &P21177_310};	/*Property = History*/
const Value V21177_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_314 = {S31, 0, &V21177_314_57262, &P21177_312};	/*Property = Html5*/
const Value V21177_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_313 = {S32, 0, &V21177_313_57259, &P21177_314};	/*Property = Html-Media-Capture*/
const Value V21177_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_315 = {S33, 0, &V21177_315_57265, &P21177_313};	/*Property = Iframe*/
const Value V21177_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_316 = {S34, 0, &V21177_316_57268, &P21177_315};	/*Property = IndexedDB*/
const Value V21177_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_317 = {S35, 0, &V21177_317_57271, &P21177_316};	/*Property = Json*/
const Value V21177_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P21177_15 = {S37, 0, &V21177_15_31, &P21177_317};	/*Property = LayoutEngine*/
const Value V21177_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_344 = {S38, 0, &V21177_344_57274, &P21177_15};	/*Property = Masking*/
const Value V21177_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_320 = {S39, 0, &V21177_320_57292, &P21177_344};	/*Property = PostMessage*/
const Value V21177_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_321 = {S40, 0, &V21177_321_57295, &P21177_320};	/*Property = Progress*/
const Value V21177_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_322 = {S41, 0, &V21177_322_57298, &P21177_321};	/*Property = Prompts*/
const Value V21177_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_323 = {S42, 0, &V21177_323_57301, &P21177_322};	/*Property = Selector*/
const Value V21177_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_324 = {S43, 0, &V21177_324_57307, &P21177_323};	/*Property = Svg*/
const Value V21177_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_327 = {S44, 0, &V21177_327_57316, &P21177_324};	/*Property = TouchEvents*/
const Value V21177_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_328 = {S45, 0, &V21177_328_57319, &P21177_327};	/*Property = Track*/
const Value V21177_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_329 = {S46, 0, &V21177_329_57325, &P21177_328};	/*Property = Video*/
const Value V21177_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_330 = {S47, 0, &V21177_330_57328, &P21177_329};	/*Property = Viewport*/
const Value V21177_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_332 = {S48, 0, &V21177_332_57340, &P21177_330};	/*Property = WebWorkers*/
const Value V21177_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21177_333 = {S49, 0, &V21177_333_57343, &P21177_332};	/*Property = Xhr2*/
const Profile P21177 = {"21177", &P21177_333};	/*Profile ID - 21177*/
const Value V18410_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_279 = {S1, 0, &V18410_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18410_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_282 = {S3, 0, &V18410_282_57155, &P18410_279};	/*Property = BlobBuilder*/
const Value V18410_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_350 = {S4, 0, &V18410_350_57446, &P18410_282};	/*Property = Canvas*/
const Value V18410_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_283 = {S5, 0, &V18410_283_57161, &P18410_350};	/*Property = CssBackground*/
const Value V18410_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_284 = {S6, 0, &V18410_284_57164, &P18410_283};	/*Property = CssBorderImage*/
const Value V18410_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_285 = {S7, 0, &V18410_285_57167, &P18410_284};	/*Property = CssCanvas*/
const Value V18410_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_286 = {S8, 0, &V18410_286_57170, &P18410_285};	/*Property = CssColor*/
const Value V18410_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_237 = {S9, 0, &V18410_237_55786, &P18410_286};	/*Property = CssColumn*/
const Value V18410_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_287 = {S10, 0, &V18410_287_57173, &P18410_237};	/*Property = CssFlexbox*/
const Value V18410_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_288 = {S11, 0, &V18410_288_57176, &P18410_287};	/*Property = CssFont*/
const Value V18410_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_289 = {S12, 0, &V18410_289_86899, &P18410_288};	/*Property = CssImages*/
const Value V18410_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_291 = {S13, 0, &V18410_291_57182, &P18410_289};	/*Property = CssMediaQueries*/
const Value V18410_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_292 = {S14, 0, &V18410_292_57185, &P18410_291};	/*Property = CssMinMax*/
const Value V18410_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_293 = {S15, 0, &V18410_293_57188, &P18410_292};	/*Property = CssOverflow*/
const Value V18410_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_294 = {S16, 0, &V18410_294_57191, &P18410_293};	/*Property = CssPosition*/
const Value V18410_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_295 = {S17, 0, &V18410_295_57194, &P18410_294};	/*Property = CssText*/
const Value V18410_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_297 = {S18, 0, &V18410_297_57197, &P18410_295};	/*Property = CssTransforms*/
const Value V18410_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_241 = {S19, 0, &V18410_241_55801, &P18410_297};	/*Property = CssTransitions*/
const Value V18410_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_298 = {S20, 0, &V18410_298_57208, &P18410_241};	/*Property = CssUI*/
const Value V18410_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_300 = {S21, 0, &V18410_300_57217, &P18410_298};	/*Property = DataSet*/
const Value V18410_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_301 = {S22, 0, &V18410_301_57220, &P18410_300};	/*Property = DataUrl*/
const Value V18410_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_302 = {S23, 0, &V18410_302_57223, &P18410_301};	/*Property = DeviceOrientation*/
const Value V18410_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_304 = {S24, 0, &V18410_304_57229, &P18410_302};	/*Property = FileReader*/
const Value V18410_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_305 = {S25, 0, &V18410_305_57232, &P18410_304};	/*Property = FileSaver*/
const Value V18410_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_306 = {S26, 0, &V18410_306_57238, &P18410_305};	/*Property = FileWriter*/
const Value V18410_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_307 = {S27, 0, &V18410_307_57241, &P18410_306};	/*Property = FormData*/
const Value V18410_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_309 = {S28, 0, &V18410_309_57247, &P18410_307};	/*Property = Fullscreen*/
const Value V18410_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_310 = {S29, 0, &V18410_310_57250, &P18410_309};	/*Property = GeoLocation*/
const Value V18410_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_312 = {S30, 0, &V18410_312_57256, &P18410_310};	/*Property = History*/
const Value V18410_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_314 = {S31, 0, &V18410_314_57262, &P18410_312};	/*Property = Html5*/
const Value V18410_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_313 = {S32, 0, &V18410_313_57259, &P18410_314};	/*Property = Html-Media-Capture*/
const Value V18410_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_315 = {S33, 0, &V18410_315_57265, &P18410_313};	/*Property = Iframe*/
const Value V18410_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_316 = {S34, 0, &V18410_316_57268, &P18410_315};	/*Property = IndexedDB*/
const Value V18410_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_317 = {S35, 0, &V18410_317_57271, &P18410_316};	/*Property = Json*/
const Value V18410_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18410_15 = {S37, 0, &V18410_15_31, &P18410_317};	/*Property = LayoutEngine*/
const Value V18410_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_344 = {S38, 0, &V18410_344_57274, &P18410_15};	/*Property = Masking*/
const Value V18410_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_320 = {S39, 0, &V18410_320_57292, &P18410_344};	/*Property = PostMessage*/
const Value V18410_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_321 = {S40, 0, &V18410_321_57295, &P18410_320};	/*Property = Progress*/
const Value V18410_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_322 = {S41, 0, &V18410_322_57298, &P18410_321};	/*Property = Prompts*/
const Value V18410_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_323 = {S42, 0, &V18410_323_57301, &P18410_322};	/*Property = Selector*/
const Value V18410_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_324 = {S43, 0, &V18410_324_57307, &P18410_323};	/*Property = Svg*/
const Value V18410_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_327 = {S44, 0, &V18410_327_57316, &P18410_324};	/*Property = TouchEvents*/
const Value V18410_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_328 = {S45, 0, &V18410_328_57319, &P18410_327};	/*Property = Track*/
const Value V18410_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_329 = {S46, 0, &V18410_329_57325, &P18410_328};	/*Property = Video*/
const Value V18410_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_330 = {S47, 0, &V18410_330_57328, &P18410_329};	/*Property = Viewport*/
const Value V18410_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_332 = {S48, 0, &V18410_332_57340, &P18410_330};	/*Property = WebWorkers*/
const Value V18410_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18410_333 = {S49, 0, &V18410_333_57343, &P18410_332};	/*Property = Xhr2*/
const Profile P18410 = {"18410", &P18410_333};	/*Profile ID - 18410*/
const Value V4307_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_279 = {S1, 0, &V4307_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4307_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_282 = {S3, 0, &V4307_282_57155, &P4307_279};	/*Property = BlobBuilder*/
const Value V4307_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_350 = {S4, 0, &V4307_350_57446, &P4307_282};	/*Property = Canvas*/
const Value V4307_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_283 = {S5, 0, &V4307_283_57161, &P4307_350};	/*Property = CssBackground*/
const Value V4307_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_284 = {S6, 0, &V4307_284_57164, &P4307_283};	/*Property = CssBorderImage*/
const Value V4307_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_285 = {S7, 0, &V4307_285_57167, &P4307_284};	/*Property = CssCanvas*/
const Value V4307_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_286 = {S8, 0, &V4307_286_57170, &P4307_285};	/*Property = CssColor*/
const Value V4307_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_237 = {S9, 0, &V4307_237_55786, &P4307_286};	/*Property = CssColumn*/
const Value V4307_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_287 = {S10, 0, &V4307_287_57173, &P4307_237};	/*Property = CssFlexbox*/
const Value V4307_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_288 = {S11, 0, &V4307_288_57176, &P4307_287};	/*Property = CssFont*/
const Value V4307_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_289 = {S12, 0, &V4307_289_86899, &P4307_288};	/*Property = CssImages*/
const Value V4307_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_291 = {S13, 0, &V4307_291_57182, &P4307_289};	/*Property = CssMediaQueries*/
const Value V4307_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_292 = {S14, 0, &V4307_292_57185, &P4307_291};	/*Property = CssMinMax*/
const Value V4307_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_293 = {S15, 0, &V4307_293_57188, &P4307_292};	/*Property = CssOverflow*/
const Value V4307_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_294 = {S16, 0, &V4307_294_57191, &P4307_293};	/*Property = CssPosition*/
const Value V4307_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_295 = {S17, 0, &V4307_295_57194, &P4307_294};	/*Property = CssText*/
const Value V4307_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_297 = {S18, 0, &V4307_297_57197, &P4307_295};	/*Property = CssTransforms*/
const Value V4307_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_241 = {S19, 0, &V4307_241_55801, &P4307_297};	/*Property = CssTransitions*/
const Value V4307_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_298 = {S20, 0, &V4307_298_57208, &P4307_241};	/*Property = CssUI*/
const Value V4307_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_300 = {S21, 0, &V4307_300_57217, &P4307_298};	/*Property = DataSet*/
const Value V4307_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_301 = {S22, 0, &V4307_301_57220, &P4307_300};	/*Property = DataUrl*/
const Value V4307_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_302 = {S23, 0, &V4307_302_57223, &P4307_301};	/*Property = DeviceOrientation*/
const Value V4307_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_304 = {S24, 0, &V4307_304_57229, &P4307_302};	/*Property = FileReader*/
const Value V4307_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_305 = {S25, 0, &V4307_305_57232, &P4307_304};	/*Property = FileSaver*/
const Value V4307_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_306 = {S26, 0, &V4307_306_57238, &P4307_305};	/*Property = FileWriter*/
const Value V4307_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_307 = {S27, 0, &V4307_307_57241, &P4307_306};	/*Property = FormData*/
const Value V4307_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_309 = {S28, 0, &V4307_309_57247, &P4307_307};	/*Property = Fullscreen*/
const Value V4307_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_310 = {S29, 0, &V4307_310_57250, &P4307_309};	/*Property = GeoLocation*/
const Value V4307_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_312 = {S30, 0, &V4307_312_57256, &P4307_310};	/*Property = History*/
const Value V4307_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_314 = {S31, 0, &V4307_314_57262, &P4307_312};	/*Property = Html5*/
const Value V4307_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_313 = {S32, 0, &V4307_313_57259, &P4307_314};	/*Property = Html-Media-Capture*/
const Value V4307_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_315 = {S33, 0, &V4307_315_57265, &P4307_313};	/*Property = Iframe*/
const Value V4307_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_316 = {S34, 0, &V4307_316_57268, &P4307_315};	/*Property = IndexedDB*/
const Value V4307_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_317 = {S35, 0, &V4307_317_57271, &P4307_316};	/*Property = Json*/
const Value V4307_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P4307_15 = {S37, 0, &V4307_15_31, &P4307_317};	/*Property = LayoutEngine*/
const Value V4307_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_344 = {S38, 0, &V4307_344_57274, &P4307_15};	/*Property = Masking*/
const Value V4307_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_320 = {S39, 0, &V4307_320_57292, &P4307_344};	/*Property = PostMessage*/
const Value V4307_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_321 = {S40, 0, &V4307_321_57295, &P4307_320};	/*Property = Progress*/
const Value V4307_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_322 = {S41, 0, &V4307_322_57298, &P4307_321};	/*Property = Prompts*/
const Value V4307_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_323 = {S42, 0, &V4307_323_57301, &P4307_322};	/*Property = Selector*/
const Value V4307_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_324 = {S43, 0, &V4307_324_57307, &P4307_323};	/*Property = Svg*/
const Value V4307_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_327 = {S44, 0, &V4307_327_57316, &P4307_324};	/*Property = TouchEvents*/
const Value V4307_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_328 = {S45, 0, &V4307_328_57319, &P4307_327};	/*Property = Track*/
const Value V4307_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_329 = {S46, 0, &V4307_329_57325, &P4307_328};	/*Property = Video*/
const Value V4307_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_330 = {S47, 0, &V4307_330_57328, &P4307_329};	/*Property = Viewport*/
const Value V4307_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_332 = {S48, 0, &V4307_332_57340, &P4307_330};	/*Property = WebWorkers*/
const Value V4307_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4307_333 = {S49, 0, &V4307_333_57343, &P4307_332};	/*Property = Xhr2*/
const Profile P4307 = {"4307", &P4307_333};	/*Profile ID - 4307*/
const Value V17538_1_1 = {S2, NULL};	/*Value = True*/
const Property P17538_1 = {S50, 0, &V17538_1_1, NULL};	/*Property = IsMobile*/
const Value V17538_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17538_5 = {S52, 0, &V17538_5_1304, &P17538_1};	/*Property = ScreenPixelsHeight*/
const Value V17538_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17538_4 = {S54, 0, &V17538_4_1224, &P17538_5};	/*Property = ScreenPixelsWidth*/
const Profile P17538 = {"17538", &P17538_4};	/*Profile ID - 17538*/
const Value V17588_1_1 = {S2, NULL};	/*Value = True*/
const Property P17588_1 = {S50, 0, &V17588_1_1, NULL};	/*Property = IsMobile*/
const Value V17588_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17588_5 = {S52, 0, &V17588_5_1269, &P17588_1};	/*Property = ScreenPixelsHeight*/
const Value V17588_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17588_4 = {S54, 0, &V17588_4_1229, &P17588_5};	/*Property = ScreenPixelsWidth*/
const Profile P17588 = {"17588", &P17588_4};	/*Profile ID - 17588*/
const Value V17876_1_1 = {S2, NULL};	/*Value = True*/
const Property P17876_1 = {S50, 0, &V17876_1_1, NULL};	/*Property = IsMobile*/
const Value V17876_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17876_5 = {S52, 0, &V17876_5_1269, &P17876_1};	/*Property = ScreenPixelsHeight*/
const Value V17876_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17876_4 = {S54, 0, &V17876_4_1229, &P17876_5};	/*Property = ScreenPixelsWidth*/
const Profile P17876 = {"17876", &P17876_4};	/*Profile ID - 17876*/
const Value V17961_1_1 = {S2, NULL};	/*Value = True*/
const Property P17961_1 = {S50, 0, &V17961_1_1, NULL};	/*Property = IsMobile*/
const Value V17961_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P17961_5 = {S52, 0, &V17961_5_1458, &P17961_1};	/*Property = ScreenPixelsHeight*/
const Value V17961_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P17961_4 = {S54, 0, &V17961_4_1460, &P17961_5};	/*Property = ScreenPixelsWidth*/
const Profile P17961 = {"17961", &P17961_4};	/*Profile ID - 17961*/
const Value V17639_1_1 = {S2, NULL};	/*Value = True*/
const Property P17639_1 = {S50, 0, &V17639_1_1, NULL};	/*Property = IsMobile*/
const Value V17639_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17639_5 = {S52, 0, &V17639_5_1269, &P17639_1};	/*Property = ScreenPixelsHeight*/
const Value V17639_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17639_4 = {S54, 0, &V17639_4_1229, &P17639_5};	/*Property = ScreenPixelsWidth*/
const Profile P17639 = {"17639", &P17639_4};	/*Profile ID - 17639*/
const Value V17553_1_1 = {S2, NULL};	/*Value = True*/
const Property P17553_1 = {S50, 0, &V17553_1_1, NULL};	/*Property = IsMobile*/
const Value V17553_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17553_5 = {S52, 0, &V17553_5_1269, &P17553_1};	/*Property = ScreenPixelsHeight*/
const Value V17553_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17553_4 = {S54, 0, &V17553_4_1229, &P17553_5};	/*Property = ScreenPixelsWidth*/
const Profile P17553 = {"17553", &P17553_4};	/*Profile ID - 17553*/
const Value V17757_1_1 = {S2, NULL};	/*Value = True*/
const Property P17757_1 = {S50, 0, &V17757_1_1, NULL};	/*Property = IsMobile*/
const Value V17757_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17757_5 = {S52, 0, &V17757_5_1269, &P17757_1};	/*Property = ScreenPixelsHeight*/
const Value V17757_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17757_4 = {S54, 0, &V17757_4_1229, &P17757_5};	/*Property = ScreenPixelsWidth*/
const Profile P17757 = {"17757", &P17757_4};	/*Profile ID - 17757*/
const Value V17739_1_1 = {S2, NULL};	/*Value = True*/
const Property P17739_1 = {S50, 0, &V17739_1_1, NULL};	/*Property = IsMobile*/
const Value V17739_5_37540 = {S105, NULL};	/*Value = 427*/
const Property P17739_5 = {S52, 0, &V17739_5_37540, &P17739_1};	/*Property = ScreenPixelsHeight*/
const Value V17739_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17739_4 = {S54, 0, &V17739_4_1224, &P17739_5};	/*Property = ScreenPixelsWidth*/
const Profile P17739 = {"17739", &P17739_4};	/*Profile ID - 17739*/
const Value V17648_1_1 = {S2, NULL};	/*Value = True*/
const Property P17648_1 = {S50, 0, &V17648_1_1, NULL};	/*Property = IsMobile*/
const Value V17648_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P17648_5 = {S52, 0, &V17648_5_1403, &P17648_1};	/*Property = ScreenPixelsHeight*/
const Value V17648_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17648_4 = {S54, 0, &V17648_4_1229, &P17648_5};	/*Property = ScreenPixelsWidth*/
const Profile P17648 = {"17648", &P17648_4};	/*Profile ID - 17648*/
const Value V17721_1_1 = {S2, NULL};	/*Value = True*/
const Property P17721_1 = {S50, 0, &V17721_1_1, NULL};	/*Property = IsMobile*/
const Value V17721_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17721_5 = {S52, 0, &V17721_5_1269, &P17721_1};	/*Property = ScreenPixelsHeight*/
const Value V17721_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17721_4 = {S54, 0, &V17721_4_1229, &P17721_5};	/*Property = ScreenPixelsWidth*/
const Profile P17721 = {"17721", &P17721_4};	/*Profile ID - 17721*/
const Value V17722_1_1 = {S2, NULL};	/*Value = True*/
const Property P17722_1 = {S50, 0, &V17722_1_1, NULL};	/*Property = IsMobile*/
const Value V17722_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17722_5 = {S52, 0, &V17722_5_1269, &P17722_1};	/*Property = ScreenPixelsHeight*/
const Value V17722_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17722_4 = {S54, 0, &V17722_4_1229, &P17722_5};	/*Property = ScreenPixelsWidth*/
const Profile P17722 = {"17722", &P17722_4};	/*Profile ID - 17722*/
const Value V17763_1_1 = {S2, NULL};	/*Value = True*/
const Property P17763_1 = {S50, 0, &V17763_1_1, NULL};	/*Property = IsMobile*/
const Value V17763_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17763_5 = {S52, 0, &V17763_5_1269, &P17763_1};	/*Property = ScreenPixelsHeight*/
const Value V17763_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17763_4 = {S54, 0, &V17763_4_1229, &P17763_5};	/*Property = ScreenPixelsWidth*/
const Profile P17763 = {"17763", &P17763_4};	/*Profile ID - 17763*/
const Value V17512_1_1 = {S2, NULL};	/*Value = True*/
const Property P17512_1 = {S50, 0, &V17512_1_1, NULL};	/*Property = IsMobile*/
const Value V17512_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17512_5 = {S52, 0, &V17512_5_1269, &P17512_1};	/*Property = ScreenPixelsHeight*/
const Value V17512_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17512_4 = {S54, 0, &V17512_4_1229, &P17512_5};	/*Property = ScreenPixelsWidth*/
const Profile P17512 = {"17512", &P17512_4};	/*Profile ID - 17512*/
const Value V17938_1_1 = {S2, NULL};	/*Value = True*/
const Property P17938_1 = {S50, 0, &V17938_1_1, NULL};	/*Property = IsMobile*/
const Value V17938_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17938_5 = {S52, 0, &V17938_5_1227, &P17938_1};	/*Property = ScreenPixelsHeight*/
const Value V17938_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17938_4 = {S54, 0, &V17938_4_1229, &P17938_5};	/*Property = ScreenPixelsWidth*/
const Profile P17938 = {"17938", &P17938_4};	/*Profile ID - 17938*/
const Value V18086_1_1 = {S2, NULL};	/*Value = True*/
const Property P18086_1 = {S50, 0, &V18086_1_1, NULL};	/*Property = IsMobile*/
const Value V18086_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P18086_5 = {S52, 0, &V18086_5_37, &P18086_1};	/*Property = ScreenPixelsHeight*/
const Value V18086_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P18086_4 = {S54, 0, &V18086_4_36, &P18086_5};	/*Property = ScreenPixelsWidth*/
const Profile P18086 = {"18086", &P18086_4};	/*Profile ID - 18086*/
const Value V17952_1_1 = {S2, NULL};	/*Value = True*/
const Property P17952_1 = {S50, 0, &V17952_1_1, NULL};	/*Property = IsMobile*/
const Value V17952_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17952_5 = {S52, 0, &V17952_5_1227, &P17952_1};	/*Property = ScreenPixelsHeight*/
const Value V17952_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17952_4 = {S54, 0, &V17952_4_1229, &P17952_5};	/*Property = ScreenPixelsWidth*/
const Profile P17952 = {"17952", &P17952_4};	/*Profile ID - 17952*/
const Value V16053_1_1 = {S2, NULL};	/*Value = True*/
const Property P16053_1 = {S50, 0, &V16053_1_1, NULL};	/*Property = IsMobile*/
const Value V16053_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16053_5 = {S52, 0, &V16053_5_1277, &P16053_1};	/*Property = ScreenPixelsHeight*/
const Value V16053_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16053_4 = {S54, 0, &V16053_4_1279, &P16053_5};	/*Property = ScreenPixelsWidth*/
const Profile P16053 = {"16053", &P16053_4};	/*Profile ID - 16053*/
const Value V16962_1_1 = {S2, NULL};	/*Value = True*/
const Property P16962_1 = {S50, 0, &V16962_1_1, NULL};	/*Property = IsMobile*/
const Value V16962_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P16962_5 = {S52, 0, &V16962_5_1294, &P16962_1};	/*Property = ScreenPixelsHeight*/
const Value V16962_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16962_4 = {S54, 0, &V16962_4_1296, &P16962_5};	/*Property = ScreenPixelsWidth*/
const Profile P16962 = {"16962", &P16962_4};	/*Profile ID - 16962*/
const Value V20315_1_1 = {S2, NULL};	/*Value = True*/
const Property P20315_1 = {S50, 0, &V20315_1_1, NULL};	/*Property = IsMobile*/
const Value V20315_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P20315_5 = {S52, 0, &V20315_5_1293, &P20315_1};	/*Property = ScreenPixelsHeight*/
const Value V20315_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P20315_4 = {S54, 0, &V20315_4_1296, &P20315_5};	/*Property = ScreenPixelsWidth*/
const Profile P20315 = {"20315", &P20315_4};	/*Profile ID - 20315*/
const Value V17993_1_1 = {S2, NULL};	/*Value = True*/
const Property P17993_1 = {S50, 0, &V17993_1_1, NULL};	/*Property = IsMobile*/
const Value V17993_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P17993_5 = {S52, 0, &V17993_5_1403, &P17993_1};	/*Property = ScreenPixelsHeight*/
const Value V17993_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P17993_4 = {S54, 0, &V17993_4_8554, &P17993_5};	/*Property = ScreenPixelsWidth*/
const Profile P17993 = {"17993", &P17993_4};	/*Profile ID - 17993*/
const Value V20804_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_279 = {S1, 0, &V20804_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20804_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_282 = {S3, 0, &V20804_282_57155, &P20804_279};	/*Property = BlobBuilder*/
const Value V20804_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_350 = {S4, 0, &V20804_350_57446, &P20804_282};	/*Property = Canvas*/
const Value V20804_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_283 = {S5, 0, &V20804_283_57161, &P20804_350};	/*Property = CssBackground*/
const Value V20804_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_284 = {S6, 0, &V20804_284_57164, &P20804_283};	/*Property = CssBorderImage*/
const Value V20804_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_285 = {S7, 0, &V20804_285_57167, &P20804_284};	/*Property = CssCanvas*/
const Value V20804_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_286 = {S8, 0, &V20804_286_57170, &P20804_285};	/*Property = CssColor*/
const Value V20804_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_237 = {S9, 0, &V20804_237_55786, &P20804_286};	/*Property = CssColumn*/
const Value V20804_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_287 = {S10, 0, &V20804_287_57173, &P20804_237};	/*Property = CssFlexbox*/
const Value V20804_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_288 = {S11, 0, &V20804_288_57176, &P20804_287};	/*Property = CssFont*/
const Value V20804_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_289 = {S12, 0, &V20804_289_86899, &P20804_288};	/*Property = CssImages*/
const Value V20804_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_291 = {S13, 0, &V20804_291_57182, &P20804_289};	/*Property = CssMediaQueries*/
const Value V20804_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_292 = {S14, 0, &V20804_292_57185, &P20804_291};	/*Property = CssMinMax*/
const Value V20804_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_293 = {S15, 0, &V20804_293_57188, &P20804_292};	/*Property = CssOverflow*/
const Value V20804_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_294 = {S16, 0, &V20804_294_57191, &P20804_293};	/*Property = CssPosition*/
const Value V20804_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_295 = {S17, 0, &V20804_295_57194, &P20804_294};	/*Property = CssText*/
const Value V20804_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_297 = {S18, 0, &V20804_297_57197, &P20804_295};	/*Property = CssTransforms*/
const Value V20804_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_241 = {S19, 0, &V20804_241_55801, &P20804_297};	/*Property = CssTransitions*/
const Value V20804_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_298 = {S20, 0, &V20804_298_57208, &P20804_241};	/*Property = CssUI*/
const Value V20804_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_300 = {S21, 0, &V20804_300_57217, &P20804_298};	/*Property = DataSet*/
const Value V20804_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_301 = {S22, 0, &V20804_301_57220, &P20804_300};	/*Property = DataUrl*/
const Value V20804_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_302 = {S23, 0, &V20804_302_57223, &P20804_301};	/*Property = DeviceOrientation*/
const Value V20804_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_304 = {S24, 0, &V20804_304_57229, &P20804_302};	/*Property = FileReader*/
const Value V20804_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_305 = {S25, 0, &V20804_305_57232, &P20804_304};	/*Property = FileSaver*/
const Value V20804_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_306 = {S26, 0, &V20804_306_57238, &P20804_305};	/*Property = FileWriter*/
const Value V20804_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_307 = {S27, 0, &V20804_307_57241, &P20804_306};	/*Property = FormData*/
const Value V20804_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_309 = {S28, 0, &V20804_309_57247, &P20804_307};	/*Property = Fullscreen*/
const Value V20804_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_310 = {S29, 0, &V20804_310_57250, &P20804_309};	/*Property = GeoLocation*/
const Value V20804_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_312 = {S30, 0, &V20804_312_57256, &P20804_310};	/*Property = History*/
const Value V20804_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_314 = {S31, 0, &V20804_314_57262, &P20804_312};	/*Property = Html5*/
const Value V20804_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_313 = {S32, 0, &V20804_313_57259, &P20804_314};	/*Property = Html-Media-Capture*/
const Value V20804_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_315 = {S33, 0, &V20804_315_57265, &P20804_313};	/*Property = Iframe*/
const Value V20804_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_316 = {S34, 0, &V20804_316_57268, &P20804_315};	/*Property = IndexedDB*/
const Value V20804_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_317 = {S35, 0, &V20804_317_57271, &P20804_316};	/*Property = Json*/
const Value V20804_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P20804_15 = {S37, 0, &V20804_15_28, &P20804_317};	/*Property = LayoutEngine*/
const Value V20804_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_344 = {S38, 0, &V20804_344_57274, &P20804_15};	/*Property = Masking*/
const Value V20804_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_320 = {S39, 0, &V20804_320_57292, &P20804_344};	/*Property = PostMessage*/
const Value V20804_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_321 = {S40, 0, &V20804_321_57295, &P20804_320};	/*Property = Progress*/
const Value V20804_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_322 = {S41, 0, &V20804_322_57298, &P20804_321};	/*Property = Prompts*/
const Value V20804_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_323 = {S42, 0, &V20804_323_57301, &P20804_322};	/*Property = Selector*/
const Value V20804_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_324 = {S43, 0, &V20804_324_57307, &P20804_323};	/*Property = Svg*/
const Value V20804_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_327 = {S44, 0, &V20804_327_57316, &P20804_324};	/*Property = TouchEvents*/
const Value V20804_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_328 = {S45, 0, &V20804_328_57319, &P20804_327};	/*Property = Track*/
const Value V20804_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_329 = {S46, 0, &V20804_329_57325, &P20804_328};	/*Property = Video*/
const Value V20804_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_330 = {S47, 0, &V20804_330_57328, &P20804_329};	/*Property = Viewport*/
const Value V20804_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_332 = {S48, 0, &V20804_332_57340, &P20804_330};	/*Property = WebWorkers*/
const Value V20804_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20804_333 = {S49, 0, &V20804_333_57343, &P20804_332};	/*Property = Xhr2*/
const Profile P20804 = {"20804", &P20804_333};	/*Profile ID - 20804*/
const Value V17723_1_1 = {S2, NULL};	/*Value = True*/
const Property P17723_1 = {S50, 0, &V17723_1_1, NULL};	/*Property = IsMobile*/
const Value V17723_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17723_5 = {S52, 0, &V17723_5_1269, &P17723_1};	/*Property = ScreenPixelsHeight*/
const Value V17723_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17723_4 = {S54, 0, &V17723_4_1229, &P17723_5};	/*Property = ScreenPixelsWidth*/
const Profile P17723 = {"17723", &P17723_4};	/*Profile ID - 17723*/
const Value V17579_1_1 = {S2, NULL};	/*Value = True*/
const Property P17579_1 = {S50, 0, &V17579_1_1, NULL};	/*Property = IsMobile*/
const Value V17579_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17579_5 = {S52, 0, &V17579_5_1269, &P17579_1};	/*Property = ScreenPixelsHeight*/
const Value V17579_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17579_4 = {S54, 0, &V17579_4_1229, &P17579_5};	/*Property = ScreenPixelsWidth*/
const Profile P17579 = {"17579", &P17579_4};	/*Profile ID - 17579*/
const Value V17516_1_1 = {S2, NULL};	/*Value = True*/
const Property P17516_1 = {S50, 0, &V17516_1_1, NULL};	/*Property = IsMobile*/
const Value V17516_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17516_5 = {S52, 0, &V17516_5_1269, &P17516_1};	/*Property = ScreenPixelsHeight*/
const Value V17516_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17516_4 = {S54, 0, &V17516_4_1229, &P17516_5};	/*Property = ScreenPixelsWidth*/
const Profile P17516 = {"17516", &P17516_4};	/*Profile ID - 17516*/
const Value V17590_1_1 = {S2, NULL};	/*Value = True*/
const Property P17590_1 = {S50, 0, &V17590_1_1, NULL};	/*Property = IsMobile*/
const Value V17590_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17590_5 = {S52, 0, &V17590_5_1269, &P17590_1};	/*Property = ScreenPixelsHeight*/
const Value V17590_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17590_4 = {S54, 0, &V17590_4_1229, &P17590_5};	/*Property = ScreenPixelsWidth*/
const Profile P17590 = {"17590", &P17590_4};	/*Profile ID - 17590*/
const Value V17638_1_1 = {S2, NULL};	/*Value = True*/
const Property P17638_1 = {S50, 0, &V17638_1_1, NULL};	/*Property = IsMobile*/
const Value V17638_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17638_5 = {S52, 0, &V17638_5_1269, &P17638_1};	/*Property = ScreenPixelsHeight*/
const Value V17638_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17638_4 = {S54, 0, &V17638_4_1229, &P17638_5};	/*Property = ScreenPixelsWidth*/
const Profile P17638 = {"17638", &P17638_4};	/*Profile ID - 17638*/
const Value V18025_1_1 = {S2, NULL};	/*Value = True*/
const Property P18025_1 = {S50, 0, &V18025_1_1, NULL};	/*Property = IsMobile*/
const Value V18025_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18025_5 = {S52, 0, &V18025_5_1227, &P18025_1};	/*Property = ScreenPixelsHeight*/
const Value V18025_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18025_4 = {S54, 0, &V18025_4_1229, &P18025_5};	/*Property = ScreenPixelsWidth*/
const Profile P18025 = {"18025", &P18025_4};	/*Profile ID - 18025*/
const Value V18045_1_1 = {S2, NULL};	/*Value = True*/
const Property P18045_1 = {S50, 0, &V18045_1_1, NULL};	/*Property = IsMobile*/
const Value V18045_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18045_5 = {S52, 0, &V18045_5_1403, &P18045_1};	/*Property = ScreenPixelsHeight*/
const Value V18045_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18045_4 = {S54, 0, &V18045_4_1229, &P18045_5};	/*Property = ScreenPixelsWidth*/
const Profile P18045 = {"18045", &P18045_4};	/*Profile ID - 18045*/
const Value V21292_1_1 = {S2, NULL};	/*Value = True*/
const Property P21292_1 = {S50, 0, &V21292_1_1, NULL};	/*Property = IsMobile*/
const Value V21292_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P21292_5 = {S52, 0, &V21292_5_1275, &P21292_1};	/*Property = ScreenPixelsHeight*/
const Value V21292_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21292_4 = {S54, 0, &V21292_4_1229, &P21292_5};	/*Property = ScreenPixelsWidth*/
const Profile P21292 = {"21292", &P21292_4};	/*Profile ID - 21292*/
const Value V15841_1_1 = {S2, NULL};	/*Value = True*/
const Property P15841_1 = {S50, 0, &V15841_1_1, NULL};	/*Property = IsMobile*/
const Value V15841_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P15841_5 = {S52, 0, &V15841_5_1832, &P15841_1};	/*Property = ScreenPixelsHeight*/
const Value V15841_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P15841_4 = {S54, 0, &V15841_4_5645, &P15841_5};	/*Property = ScreenPixelsWidth*/
const Profile P15841 = {"15841", &P15841_4};	/*Profile ID - 15841*/
const Profile P17352 = {"17352", NULL};	/*Profile ID - 17352*/
const Value V17360_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17360_279 = {S1, 0, &V17360_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17360_282_57153 = {S2, NULL};	/*Value = True*/
const Property P17360_282 = {S3, 0, &V17360_282_57153, &P17360_279};	/*Property = BlobBuilder*/
const Value V17360_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17360_350 = {S4, 0, &V17360_350_57444, &P17360_282};	/*Property = Canvas*/
const Value V17360_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17360_283 = {S5, 0, &V17360_283_57159, &P17360_350};	/*Property = CssBackground*/
const Value V17360_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17360_284 = {S6, 0, &V17360_284_57163, &P17360_283};	/*Property = CssBorderImage*/
const Value V17360_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17360_285 = {S7, 0, &V17360_285_57166, &P17360_284};	/*Property = CssCanvas*/
const Value V17360_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17360_286 = {S8, 0, &V17360_286_57168, &P17360_285};	/*Property = CssColor*/
const Value V17360_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17360_237 = {S9, 0, &V17360_237_55784, &P17360_286};	/*Property = CssColumn*/
const Value V17360_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17360_287 = {S10, 0, &V17360_287_57172, &P17360_237};	/*Property = CssFlexbox*/
const Value V17360_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17360_288 = {S11, 0, &V17360_288_57175, &P17360_287};	/*Property = CssFont*/
const Value V17360_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17360_289 = {S12, 0, &V17360_289_57759, &P17360_288};	/*Property = CssImages*/
const Value V17360_291_57180 = {S2, NULL};	/*Value = True*/
const Property P17360_291 = {S13, 0, &V17360_291_57180, &P17360_289};	/*Property = CssMediaQueries*/
const Value V17360_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17360_292 = {S14, 0, &V17360_292_57183, &P17360_291};	/*Property = CssMinMax*/
const Value V17360_293_57186 = {S2, NULL};	/*Value = True*/
const Property P17360_293 = {S15, 0, &V17360_293_57186, &P17360_292};	/*Property = CssOverflow*/
const Value V17360_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17360_294 = {S16, 0, &V17360_294_57189, &P17360_293};	/*Property = CssPosition*/
const Value V17360_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17360_295 = {S17, 0, &V17360_295_57192, &P17360_294};	/*Property = CssText*/
const Value V17360_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17360_297 = {S18, 0, &V17360_297_57195, &P17360_295};	/*Property = CssTransforms*/
const Value V17360_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17360_241 = {S19, 0, &V17360_241_55799, &P17360_297};	/*Property = CssTransitions*/
const Value V17360_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17360_298 = {S20, 0, &V17360_298_57206, &P17360_241};	/*Property = CssUI*/
const Value V17360_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17360_300 = {S21, 0, &V17360_300_57216, &P17360_298};	/*Property = DataSet*/
const Value V17360_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17360_301 = {S22, 0, &V17360_301_57218, &P17360_300};	/*Property = DataUrl*/
const Value V17360_302_57221 = {S2, NULL};	/*Value = True*/
const Property P17360_302 = {S23, 0, &V17360_302_57221, &P17360_301};	/*Property = DeviceOrientation*/
const Value V17360_304_57227 = {S2, NULL};	/*Value = True*/
const Property P17360_304 = {S24, 0, &V17360_304_57227, &P17360_302};	/*Property = FileReader*/
const Value V17360_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17360_305 = {S25, 0, &V17360_305_57231, &P17360_304};	/*Property = FileSaver*/
const Value V17360_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17360_306 = {S26, 0, &V17360_306_57237, &P17360_305};	/*Property = FileWriter*/
const Value V17360_307_57239 = {S2, NULL};	/*Value = True*/
const Property P17360_307 = {S27, 0, &V17360_307_57239, &P17360_306};	/*Property = FormData*/
const Value V17360_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17360_309 = {S28, 0, &V17360_309_57246, &P17360_307};	/*Property = Fullscreen*/
const Value V17360_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17360_310 = {S29, 0, &V17360_310_57248, &P17360_309};	/*Property = GeoLocation*/
const Value V17360_312_57254 = {S2, NULL};	/*Value = True*/
const Property P17360_312 = {S30, 0, &V17360_312_57254, &P17360_310};	/*Property = History*/
const Value V17360_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17360_314 = {S31, 0, &V17360_314_57261, &P17360_312};	/*Property = Html5*/
const Value V17360_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17360_313 = {S32, 0, &V17360_313_57258, &P17360_314};	/*Property = Html-Media-Capture*/
const Value V17360_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17360_315 = {S33, 0, &V17360_315_57264, &P17360_313};	/*Property = Iframe*/
const Value V17360_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17360_316 = {S34, 0, &V17360_316_57267, &P17360_315};	/*Property = IndexedDB*/
const Value V17360_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17360_317 = {S35, 0, &V17360_317_57269, &P17360_316};	/*Property = Json*/
const Value V17360_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17360_15 = {S37, 0, &V17360_15_28, &P17360_317};	/*Property = LayoutEngine*/
const Value V17360_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17360_344 = {S38, 0, &V17360_344_57272, &P17360_15};	/*Property = Masking*/
const Value V17360_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17360_320 = {S39, 0, &V17360_320_57290, &P17360_344};	/*Property = PostMessage*/
const Value V17360_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17360_321 = {S40, 0, &V17360_321_57293, &P17360_320};	/*Property = Progress*/
const Value V17360_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17360_322 = {S41, 0, &V17360_322_57296, &P17360_321};	/*Property = Prompts*/
const Value V17360_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17360_323 = {S42, 0, &V17360_323_57299, &P17360_322};	/*Property = Selector*/
const Value V17360_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17360_324 = {S43, 0, &V17360_324_57306, &P17360_323};	/*Property = Svg*/
const Value V17360_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17360_327 = {S44, 0, &V17360_327_57315, &P17360_324};	/*Property = TouchEvents*/
const Value V17360_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17360_328 = {S45, 0, &V17360_328_57318, &P17360_327};	/*Property = Track*/
const Value V17360_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17360_329 = {S46, 0, &V17360_329_57323, &P17360_328};	/*Property = Video*/
const Value V17360_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17360_330 = {S47, 0, &V17360_330_57326, &P17360_329};	/*Property = Viewport*/
const Value V17360_332_57338 = {S2, NULL};	/*Value = True*/
const Property P17360_332 = {S48, 0, &V17360_332_57338, &P17360_330};	/*Property = WebWorkers*/
const Value V17360_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17360_333 = {S49, 0, &V17360_333_57342, &P17360_332};	/*Property = Xhr2*/
const Profile P17360 = {"17360", &P17360_333};	/*Profile ID - 17360*/
const Value V18013_1_1 = {S2, NULL};	/*Value = True*/
const Property P18013_1 = {S50, 0, &V18013_1_1, NULL};	/*Property = IsMobile*/
const Value V18013_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18013_5 = {S52, 0, &V18013_5_1277, &P18013_1};	/*Property = ScreenPixelsHeight*/
const Value V18013_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18013_4 = {S54, 0, &V18013_4_1279, &P18013_5};	/*Property = ScreenPixelsWidth*/
const Profile P18013 = {"18013", &P18013_4};	/*Profile ID - 18013*/
const Value V17863_1_1 = {S2, NULL};	/*Value = True*/
const Property P17863_1 = {S50, 0, &V17863_1_1, NULL};	/*Property = IsMobile*/
const Value V17863_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17863_5 = {S52, 0, &V17863_5_1227, &P17863_1};	/*Property = ScreenPixelsHeight*/
const Value V17863_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17863_4 = {S54, 0, &V17863_4_1229, &P17863_5};	/*Property = ScreenPixelsWidth*/
const Profile P17863 = {"17863", &P17863_4};	/*Profile ID - 17863*/
const Value V18018_1_1 = {S2, NULL};	/*Value = True*/
const Property P18018_1 = {S50, 0, &V18018_1_1, NULL};	/*Property = IsMobile*/
const Value V18018_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18018_5 = {S52, 0, &V18018_5_1277, &P18018_1};	/*Property = ScreenPixelsHeight*/
const Value V18018_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18018_4 = {S54, 0, &V18018_4_1279, &P18018_5};	/*Property = ScreenPixelsWidth*/
const Profile P18018 = {"18018", &P18018_4};	/*Profile ID - 18018*/
const Value V15869_1_1 = {S2, NULL};	/*Value = True*/
const Property P15869_1 = {S50, 0, &V15869_1_1, NULL};	/*Property = IsMobile*/
const Value V15869_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P15869_5 = {S52, 0, &V15869_5_9981, &P15869_1};	/*Property = ScreenPixelsHeight*/
const Value V15869_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P15869_4 = {S54, 0, &V15869_4_2336, &P15869_5};	/*Property = ScreenPixelsWidth*/
const Profile P15869 = {"15869", &P15869_4};	/*Profile ID - 15869*/
const Value V17587_1_1 = {S2, NULL};	/*Value = True*/
const Property P17587_1 = {S50, 0, &V17587_1_1, NULL};	/*Property = IsMobile*/
const Value V17587_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17587_5 = {S52, 0, &V17587_5_1269, &P17587_1};	/*Property = ScreenPixelsHeight*/
const Value V17587_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17587_4 = {S54, 0, &V17587_4_1229, &P17587_5};	/*Property = ScreenPixelsWidth*/
const Profile P17587 = {"17587", &P17587_4};	/*Profile ID - 17587*/
const Value V17724_1_1 = {S2, NULL};	/*Value = True*/
const Property P17724_1 = {S50, 0, &V17724_1_1, NULL};	/*Property = IsMobile*/
const Value V17724_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17724_5 = {S52, 0, &V17724_5_1269, &P17724_1};	/*Property = ScreenPixelsHeight*/
const Value V17724_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17724_4 = {S54, 0, &V17724_4_1229, &P17724_5};	/*Property = ScreenPixelsWidth*/
const Profile P17724 = {"17724", &P17724_4};	/*Profile ID - 17724*/
const Value V18271_1_1 = {S2, NULL};	/*Value = True*/
const Property P18271_1 = {S50, 0, &V18271_1_1, NULL};	/*Property = IsMobile*/
const Value V18271_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18271_5 = {S52, 0, &V18271_5_1403, &P18271_1};	/*Property = ScreenPixelsHeight*/
const Value V18271_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18271_4 = {S54, 0, &V18271_4_1229, &P18271_5};	/*Property = ScreenPixelsWidth*/
const Profile P18271 = {"18271", &P18271_4};	/*Profile ID - 18271*/
const Value V17628_1_1 = {S2, NULL};	/*Value = True*/
const Property P17628_1 = {S50, 0, &V17628_1_1, NULL};	/*Property = IsMobile*/
const Value V17628_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17628_5 = {S52, 0, &V17628_5_1304, &P17628_1};	/*Property = ScreenPixelsHeight*/
const Value V17628_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17628_4 = {S54, 0, &V17628_4_1224, &P17628_5};	/*Property = ScreenPixelsWidth*/
const Profile P17628 = {"17628", &P17628_4};	/*Profile ID - 17628*/
const Value V18040_1_1 = {S2, NULL};	/*Value = True*/
const Property P18040_1 = {S50, 0, &V18040_1_1, NULL};	/*Property = IsMobile*/
const Value V18040_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18040_5 = {S52, 0, &V18040_5_1227, &P18040_1};	/*Property = ScreenPixelsHeight*/
const Value V18040_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18040_4 = {S54, 0, &V18040_4_1229, &P18040_5};	/*Property = ScreenPixelsWidth*/
const Profile P18040 = {"18040", &P18040_4};	/*Profile ID - 18040*/
const Value V18052_1_1 = {S2, NULL};	/*Value = True*/
const Property P18052_1 = {S50, 0, &V18052_1_1, NULL};	/*Property = IsMobile*/
const Value V18052_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18052_5 = {S52, 0, &V18052_5_1227, &P18052_1};	/*Property = ScreenPixelsHeight*/
const Value V18052_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18052_4 = {S54, 0, &V18052_4_1229, &P18052_5};	/*Property = ScreenPixelsWidth*/
const Profile P18052 = {"18052", &P18052_4};	/*Profile ID - 18052*/
const Value V20581_1_1 = {S2, NULL};	/*Value = True*/
const Property P20581_1 = {S50, 0, &V20581_1_1, NULL};	/*Property = IsMobile*/
const Value V20581_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20581_5 = {S52, 0, &V20581_5_9981, &P20581_1};	/*Property = ScreenPixelsHeight*/
const Value V20581_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20581_4 = {S54, 0, &V20581_4_37617, &P20581_5};	/*Property = ScreenPixelsWidth*/
const Profile P20581 = {"20581", &P20581_4};	/*Profile ID - 20581*/
const Value V17994_1_1 = {S2, NULL};	/*Value = True*/
const Property P17994_1 = {S50, 0, &V17994_1_1, NULL};	/*Property = IsMobile*/
const Value V17994_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17994_5 = {S52, 0, &V17994_5_1269, &P17994_1};	/*Property = ScreenPixelsHeight*/
const Value V17994_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17994_4 = {S54, 0, &V17994_4_1229, &P17994_5};	/*Property = ScreenPixelsWidth*/
const Profile P17994 = {"17994", &P17994_4};	/*Profile ID - 17994*/
const Value V18011_1_1 = {S2, NULL};	/*Value = True*/
const Property P18011_1 = {S50, 0, &V18011_1_1, NULL};	/*Property = IsMobile*/
const Value V18011_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P18011_5 = {S52, 0, &V18011_5_1269, &P18011_1};	/*Property = ScreenPixelsHeight*/
const Value V18011_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18011_4 = {S54, 0, &V18011_4_1229, &P18011_5};	/*Property = ScreenPixelsWidth*/
const Profile P18011 = {"18011", &P18011_4};	/*Profile ID - 18011*/
const Value V16609_1_1 = {S2, NULL};	/*Value = True*/
const Property P16609_1 = {S50, 0, &V16609_1_1, NULL};	/*Property = IsMobile*/
const Value V16609_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P16609_5 = {S52, 0, &V16609_5_1304, &P16609_1};	/*Property = ScreenPixelsHeight*/
const Value V16609_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16609_4 = {S54, 0, &V16609_4_1224, &P16609_5};	/*Property = ScreenPixelsWidth*/
const Profile P16609 = {"16609", &P16609_4};	/*Profile ID - 16609*/
const Value V17946_1_1 = {S2, NULL};	/*Value = True*/
const Property P17946_1 = {S50, 0, &V17946_1_1, NULL};	/*Property = IsMobile*/
const Value V17946_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17946_5 = {S52, 0, &V17946_5_1227, &P17946_1};	/*Property = ScreenPixelsHeight*/
const Value V17946_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17946_4 = {S54, 0, &V17946_4_1229, &P17946_5};	/*Property = ScreenPixelsWidth*/
const Profile P17946 = {"17946", &P17946_4};	/*Profile ID - 17946*/
const Value V22741_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_279 = {S1, 0, &V22741_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22741_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_282 = {S3, 0, &V22741_282_57155, &P22741_279};	/*Property = BlobBuilder*/
const Value V22741_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_350 = {S4, 0, &V22741_350_57446, &P22741_282};	/*Property = Canvas*/
const Value V22741_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_283 = {S5, 0, &V22741_283_57161, &P22741_350};	/*Property = CssBackground*/
const Value V22741_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_284 = {S6, 0, &V22741_284_57164, &P22741_283};	/*Property = CssBorderImage*/
const Value V22741_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_285 = {S7, 0, &V22741_285_57167, &P22741_284};	/*Property = CssCanvas*/
const Value V22741_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_286 = {S8, 0, &V22741_286_57170, &P22741_285};	/*Property = CssColor*/
const Value V22741_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_237 = {S9, 0, &V22741_237_55786, &P22741_286};	/*Property = CssColumn*/
const Value V22741_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_287 = {S10, 0, &V22741_287_57173, &P22741_237};	/*Property = CssFlexbox*/
const Value V22741_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_288 = {S11, 0, &V22741_288_57176, &P22741_287};	/*Property = CssFont*/
const Value V22741_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_289 = {S12, 0, &V22741_289_86899, &P22741_288};	/*Property = CssImages*/
const Value V22741_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_291 = {S13, 0, &V22741_291_57182, &P22741_289};	/*Property = CssMediaQueries*/
const Value V22741_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_292 = {S14, 0, &V22741_292_57185, &P22741_291};	/*Property = CssMinMax*/
const Value V22741_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_293 = {S15, 0, &V22741_293_57188, &P22741_292};	/*Property = CssOverflow*/
const Value V22741_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_294 = {S16, 0, &V22741_294_57191, &P22741_293};	/*Property = CssPosition*/
const Value V22741_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_295 = {S17, 0, &V22741_295_57194, &P22741_294};	/*Property = CssText*/
const Value V22741_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_297 = {S18, 0, &V22741_297_57197, &P22741_295};	/*Property = CssTransforms*/
const Value V22741_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_241 = {S19, 0, &V22741_241_55801, &P22741_297};	/*Property = CssTransitions*/
const Value V22741_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_298 = {S20, 0, &V22741_298_57208, &P22741_241};	/*Property = CssUI*/
const Value V22741_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_300 = {S21, 0, &V22741_300_57217, &P22741_298};	/*Property = DataSet*/
const Value V22741_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_301 = {S22, 0, &V22741_301_57220, &P22741_300};	/*Property = DataUrl*/
const Value V22741_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_302 = {S23, 0, &V22741_302_57223, &P22741_301};	/*Property = DeviceOrientation*/
const Value V22741_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_304 = {S24, 0, &V22741_304_57229, &P22741_302};	/*Property = FileReader*/
const Value V22741_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_305 = {S25, 0, &V22741_305_57232, &P22741_304};	/*Property = FileSaver*/
const Value V22741_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_306 = {S26, 0, &V22741_306_57238, &P22741_305};	/*Property = FileWriter*/
const Value V22741_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_307 = {S27, 0, &V22741_307_57241, &P22741_306};	/*Property = FormData*/
const Value V22741_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_309 = {S28, 0, &V22741_309_57247, &P22741_307};	/*Property = Fullscreen*/
const Value V22741_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_310 = {S29, 0, &V22741_310_57250, &P22741_309};	/*Property = GeoLocation*/
const Value V22741_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_312 = {S30, 0, &V22741_312_57256, &P22741_310};	/*Property = History*/
const Value V22741_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_314 = {S31, 0, &V22741_314_57262, &P22741_312};	/*Property = Html5*/
const Value V22741_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_313 = {S32, 0, &V22741_313_57259, &P22741_314};	/*Property = Html-Media-Capture*/
const Value V22741_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_315 = {S33, 0, &V22741_315_57265, &P22741_313};	/*Property = Iframe*/
const Value V22741_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_316 = {S34, 0, &V22741_316_57268, &P22741_315};	/*Property = IndexedDB*/
const Value V22741_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_317 = {S35, 0, &V22741_317_57271, &P22741_316};	/*Property = Json*/
const Value V22741_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22741_15 = {S37, 0, &V22741_15_29, &P22741_317};	/*Property = LayoutEngine*/
const Value V22741_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_344 = {S38, 0, &V22741_344_57274, &P22741_15};	/*Property = Masking*/
const Value V22741_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_320 = {S39, 0, &V22741_320_57292, &P22741_344};	/*Property = PostMessage*/
const Value V22741_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_321 = {S40, 0, &V22741_321_57295, &P22741_320};	/*Property = Progress*/
const Value V22741_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_322 = {S41, 0, &V22741_322_57298, &P22741_321};	/*Property = Prompts*/
const Value V22741_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_323 = {S42, 0, &V22741_323_57301, &P22741_322};	/*Property = Selector*/
const Value V22741_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_324 = {S43, 0, &V22741_324_57307, &P22741_323};	/*Property = Svg*/
const Value V22741_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_327 = {S44, 0, &V22741_327_57316, &P22741_324};	/*Property = TouchEvents*/
const Value V22741_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_328 = {S45, 0, &V22741_328_57319, &P22741_327};	/*Property = Track*/
const Value V22741_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_329 = {S46, 0, &V22741_329_57325, &P22741_328};	/*Property = Video*/
const Value V22741_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_330 = {S47, 0, &V22741_330_57328, &P22741_329};	/*Property = Viewport*/
const Value V22741_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_332 = {S48, 0, &V22741_332_57340, &P22741_330};	/*Property = WebWorkers*/
const Value V22741_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22741_333 = {S49, 0, &V22741_333_57343, &P22741_332};	/*Property = Xhr2*/
const Profile P22741 = {"22741", &P22741_333};	/*Profile ID - 22741*/
const Value V17643_1_1 = {S2, NULL};	/*Value = True*/
const Property P17643_1 = {S50, 0, &V17643_1_1, NULL};	/*Property = IsMobile*/
const Value V17643_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17643_5 = {S52, 0, &V17643_5_1269, &P17643_1};	/*Property = ScreenPixelsHeight*/
const Value V17643_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17643_4 = {S54, 0, &V17643_4_1229, &P17643_5};	/*Property = ScreenPixelsWidth*/
const Profile P17643 = {"17643", &P17643_4};	/*Profile ID - 17643*/
const Value V17728_1_1 = {S2, NULL};	/*Value = True*/
const Property P17728_1 = {S50, 0, &V17728_1_1, NULL};	/*Property = IsMobile*/
const Value V17728_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17728_5 = {S52, 0, &V17728_5_1304, &P17728_1};	/*Property = ScreenPixelsHeight*/
const Value V17728_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17728_4 = {S54, 0, &V17728_4_1224, &P17728_5};	/*Property = ScreenPixelsWidth*/
const Profile P17728 = {"17728", &P17728_4};	/*Profile ID - 17728*/
const Value V21541_1_1 = {S2, NULL};	/*Value = True*/
const Property P21541_1 = {S50, 0, &V21541_1_1, NULL};	/*Property = IsMobile*/
const Value V21541_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21541_5 = {S52, 0, &V21541_5_9981, &P21541_1};	/*Property = ScreenPixelsHeight*/
const Value V21541_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21541_4 = {S54, 0, &V21541_4_2336, &P21541_5};	/*Property = ScreenPixelsWidth*/
const Profile P21541 = {"21541", &P21541_4};	/*Profile ID - 21541*/
const Value V16566_1_1 = {S2, NULL};	/*Value = True*/
const Property P16566_1 = {S50, 0, &V16566_1_1, NULL};	/*Property = IsMobile*/
const Value V16566_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16566_5 = {S52, 0, &V16566_5_1233, &P16566_1};	/*Property = ScreenPixelsHeight*/
const Value V16566_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16566_4 = {S54, 0, &V16566_4_1235, &P16566_5};	/*Property = ScreenPixelsWidth*/
const Profile P16566 = {"16566", &P16566_4};	/*Profile ID - 16566*/
const Value V16796_1_1 = {S2, NULL};	/*Value = True*/
const Property P16796_1 = {S50, 0, &V16796_1_1, NULL};	/*Property = IsMobile*/
const Value V16796_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P16796_5 = {S52, 0, &V16796_5_1380, &P16796_1};	/*Property = ScreenPixelsHeight*/
const Value V16796_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16796_4 = {S54, 0, &V16796_4_1279, &P16796_5};	/*Property = ScreenPixelsWidth*/
const Profile P16796 = {"16796", &P16796_4};	/*Profile ID - 16796*/
const Value V18071_1_1 = {S2, NULL};	/*Value = True*/
const Property P18071_1 = {S50, 0, &V18071_1_1, NULL};	/*Property = IsMobile*/
const Value V18071_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18071_5 = {S52, 0, &V18071_5_1403, &P18071_1};	/*Property = ScreenPixelsHeight*/
const Value V18071_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18071_4 = {S54, 0, &V18071_4_1404, &P18071_5};	/*Property = ScreenPixelsWidth*/
const Profile P18071 = {"18071", &P18071_4};	/*Profile ID - 18071*/
const Value V17585_1_1 = {S2, NULL};	/*Value = True*/
const Property P17585_1 = {S50, 0, &V17585_1_1, NULL};	/*Property = IsMobile*/
const Value V17585_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P17585_5 = {S52, 0, &V17585_5_1832, &P17585_1};	/*Property = ScreenPixelsHeight*/
const Value V17585_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P17585_4 = {S54, 0, &V17585_4_5645, &P17585_5};	/*Property = ScreenPixelsWidth*/
const Profile P17585 = {"17585", &P17585_4};	/*Profile ID - 17585*/
const Value V12697_1_1 = {S2, NULL};	/*Value = True*/
const Property P12697_1 = {S50, 0, &V12697_1_1, NULL};	/*Property = IsMobile*/
const Value V12697_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P12697_5 = {S52, 0, &V12697_5_1403, &P12697_1};	/*Property = ScreenPixelsHeight*/
const Value V12697_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P12697_4 = {S54, 0, &V12697_4_1404, &P12697_5};	/*Property = ScreenPixelsWidth*/
const Profile P12697 = {"12697", &P12697_4};	/*Profile ID - 12697*/
const Value V15795_1_1 = {S2, NULL};	/*Value = True*/
const Property P15795_1 = {S50, 0, &V15795_1_1, NULL};	/*Property = IsMobile*/
const Value V15795_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15795_5 = {S52, 0, &V15795_5_1227, &P15795_1};	/*Property = ScreenPixelsHeight*/
const Value V15795_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15795_4 = {S54, 0, &V15795_4_1229, &P15795_5};	/*Property = ScreenPixelsWidth*/
const Profile P15795 = {"15795", &P15795_4};	/*Profile ID - 15795*/
const Value V15775_1_1 = {S2, NULL};	/*Value = True*/
const Property P15775_1 = {S50, 0, &V15775_1_1, NULL};	/*Property = IsMobile*/
const Value V15775_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15775_5 = {S52, 0, &V15775_5_1227, &P15775_1};	/*Property = ScreenPixelsHeight*/
const Value V15775_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15775_4 = {S54, 0, &V15775_4_1229, &P15775_5};	/*Property = ScreenPixelsWidth*/
const Profile P15775 = {"15775", &P15775_4};	/*Profile ID - 15775*/
const Value V18418_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_279 = {S1, 0, &V18418_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18418_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_282 = {S3, 0, &V18418_282_57155, &P18418_279};	/*Property = BlobBuilder*/
const Value V18418_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_350 = {S4, 0, &V18418_350_57446, &P18418_282};	/*Property = Canvas*/
const Value V18418_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_283 = {S5, 0, &V18418_283_57161, &P18418_350};	/*Property = CssBackground*/
const Value V18418_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_284 = {S6, 0, &V18418_284_57164, &P18418_283};	/*Property = CssBorderImage*/
const Value V18418_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_285 = {S7, 0, &V18418_285_57167, &P18418_284};	/*Property = CssCanvas*/
const Value V18418_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_286 = {S8, 0, &V18418_286_57170, &P18418_285};	/*Property = CssColor*/
const Value V18418_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_237 = {S9, 0, &V18418_237_55786, &P18418_286};	/*Property = CssColumn*/
const Value V18418_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_287 = {S10, 0, &V18418_287_57173, &P18418_237};	/*Property = CssFlexbox*/
const Value V18418_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_288 = {S11, 0, &V18418_288_57176, &P18418_287};	/*Property = CssFont*/
const Value V18418_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_289 = {S12, 0, &V18418_289_86899, &P18418_288};	/*Property = CssImages*/
const Value V18418_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_291 = {S13, 0, &V18418_291_57182, &P18418_289};	/*Property = CssMediaQueries*/
const Value V18418_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_292 = {S14, 0, &V18418_292_57185, &P18418_291};	/*Property = CssMinMax*/
const Value V18418_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_293 = {S15, 0, &V18418_293_57188, &P18418_292};	/*Property = CssOverflow*/
const Value V18418_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_294 = {S16, 0, &V18418_294_57191, &P18418_293};	/*Property = CssPosition*/
const Value V18418_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_295 = {S17, 0, &V18418_295_57194, &P18418_294};	/*Property = CssText*/
const Value V18418_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_297 = {S18, 0, &V18418_297_57197, &P18418_295};	/*Property = CssTransforms*/
const Value V18418_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_241 = {S19, 0, &V18418_241_55801, &P18418_297};	/*Property = CssTransitions*/
const Value V18418_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_298 = {S20, 0, &V18418_298_57208, &P18418_241};	/*Property = CssUI*/
const Value V18418_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_300 = {S21, 0, &V18418_300_57217, &P18418_298};	/*Property = DataSet*/
const Value V18418_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_301 = {S22, 0, &V18418_301_57220, &P18418_300};	/*Property = DataUrl*/
const Value V18418_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_302 = {S23, 0, &V18418_302_57223, &P18418_301};	/*Property = DeviceOrientation*/
const Value V18418_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_304 = {S24, 0, &V18418_304_57229, &P18418_302};	/*Property = FileReader*/
const Value V18418_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_305 = {S25, 0, &V18418_305_57232, &P18418_304};	/*Property = FileSaver*/
const Value V18418_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_306 = {S26, 0, &V18418_306_57238, &P18418_305};	/*Property = FileWriter*/
const Value V18418_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_307 = {S27, 0, &V18418_307_57241, &P18418_306};	/*Property = FormData*/
const Value V18418_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_309 = {S28, 0, &V18418_309_57247, &P18418_307};	/*Property = Fullscreen*/
const Value V18418_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_310 = {S29, 0, &V18418_310_57250, &P18418_309};	/*Property = GeoLocation*/
const Value V18418_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_312 = {S30, 0, &V18418_312_57256, &P18418_310};	/*Property = History*/
const Value V18418_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_314 = {S31, 0, &V18418_314_57262, &P18418_312};	/*Property = Html5*/
const Value V18418_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_313 = {S32, 0, &V18418_313_57259, &P18418_314};	/*Property = Html-Media-Capture*/
const Value V18418_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_315 = {S33, 0, &V18418_315_57265, &P18418_313};	/*Property = Iframe*/
const Value V18418_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_316 = {S34, 0, &V18418_316_57268, &P18418_315};	/*Property = IndexedDB*/
const Value V18418_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_317 = {S35, 0, &V18418_317_57271, &P18418_316};	/*Property = Json*/
const Value V18418_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18418_15 = {S37, 0, &V18418_15_28, &P18418_317};	/*Property = LayoutEngine*/
const Value V18418_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_344 = {S38, 0, &V18418_344_57274, &P18418_15};	/*Property = Masking*/
const Value V18418_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_320 = {S39, 0, &V18418_320_57292, &P18418_344};	/*Property = PostMessage*/
const Value V18418_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_321 = {S40, 0, &V18418_321_57295, &P18418_320};	/*Property = Progress*/
const Value V18418_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_322 = {S41, 0, &V18418_322_57298, &P18418_321};	/*Property = Prompts*/
const Value V18418_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_323 = {S42, 0, &V18418_323_57301, &P18418_322};	/*Property = Selector*/
const Value V18418_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_324 = {S43, 0, &V18418_324_57307, &P18418_323};	/*Property = Svg*/
const Value V18418_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_327 = {S44, 0, &V18418_327_57316, &P18418_324};	/*Property = TouchEvents*/
const Value V18418_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_328 = {S45, 0, &V18418_328_57319, &P18418_327};	/*Property = Track*/
const Value V18418_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_329 = {S46, 0, &V18418_329_57325, &P18418_328};	/*Property = Video*/
const Value V18418_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_330 = {S47, 0, &V18418_330_57328, &P18418_329};	/*Property = Viewport*/
const Value V18418_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_332 = {S48, 0, &V18418_332_57340, &P18418_330};	/*Property = WebWorkers*/
const Value V18418_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18418_333 = {S49, 0, &V18418_333_57343, &P18418_332};	/*Property = Xhr2*/
const Profile P18418 = {"18418", &P18418_333};	/*Profile ID - 18418*/
const Value V22702_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_279 = {S1, 0, &V22702_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22702_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_282 = {S3, 0, &V22702_282_57155, &P22702_279};	/*Property = BlobBuilder*/
const Value V22702_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_350 = {S4, 0, &V22702_350_57446, &P22702_282};	/*Property = Canvas*/
const Value V22702_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_283 = {S5, 0, &V22702_283_57161, &P22702_350};	/*Property = CssBackground*/
const Value V22702_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_284 = {S6, 0, &V22702_284_57164, &P22702_283};	/*Property = CssBorderImage*/
const Value V22702_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_285 = {S7, 0, &V22702_285_57167, &P22702_284};	/*Property = CssCanvas*/
const Value V22702_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_286 = {S8, 0, &V22702_286_57170, &P22702_285};	/*Property = CssColor*/
const Value V22702_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_237 = {S9, 0, &V22702_237_55786, &P22702_286};	/*Property = CssColumn*/
const Value V22702_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_287 = {S10, 0, &V22702_287_57173, &P22702_237};	/*Property = CssFlexbox*/
const Value V22702_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_288 = {S11, 0, &V22702_288_57176, &P22702_287};	/*Property = CssFont*/
const Value V22702_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_289 = {S12, 0, &V22702_289_86899, &P22702_288};	/*Property = CssImages*/
const Value V22702_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_291 = {S13, 0, &V22702_291_57182, &P22702_289};	/*Property = CssMediaQueries*/
const Value V22702_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_292 = {S14, 0, &V22702_292_57185, &P22702_291};	/*Property = CssMinMax*/
const Value V22702_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_293 = {S15, 0, &V22702_293_57188, &P22702_292};	/*Property = CssOverflow*/
const Value V22702_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_294 = {S16, 0, &V22702_294_57191, &P22702_293};	/*Property = CssPosition*/
const Value V22702_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_295 = {S17, 0, &V22702_295_57194, &P22702_294};	/*Property = CssText*/
const Value V22702_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_297 = {S18, 0, &V22702_297_57197, &P22702_295};	/*Property = CssTransforms*/
const Value V22702_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_241 = {S19, 0, &V22702_241_55801, &P22702_297};	/*Property = CssTransitions*/
const Value V22702_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_298 = {S20, 0, &V22702_298_57208, &P22702_241};	/*Property = CssUI*/
const Value V22702_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_300 = {S21, 0, &V22702_300_57217, &P22702_298};	/*Property = DataSet*/
const Value V22702_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_301 = {S22, 0, &V22702_301_57220, &P22702_300};	/*Property = DataUrl*/
const Value V22702_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_302 = {S23, 0, &V22702_302_57223, &P22702_301};	/*Property = DeviceOrientation*/
const Value V22702_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_304 = {S24, 0, &V22702_304_57229, &P22702_302};	/*Property = FileReader*/
const Value V22702_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_305 = {S25, 0, &V22702_305_57232, &P22702_304};	/*Property = FileSaver*/
const Value V22702_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_306 = {S26, 0, &V22702_306_57238, &P22702_305};	/*Property = FileWriter*/
const Value V22702_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_307 = {S27, 0, &V22702_307_57241, &P22702_306};	/*Property = FormData*/
const Value V22702_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_309 = {S28, 0, &V22702_309_57247, &P22702_307};	/*Property = Fullscreen*/
const Value V22702_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_310 = {S29, 0, &V22702_310_57250, &P22702_309};	/*Property = GeoLocation*/
const Value V22702_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_312 = {S30, 0, &V22702_312_57256, &P22702_310};	/*Property = History*/
const Value V22702_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_314 = {S31, 0, &V22702_314_57262, &P22702_312};	/*Property = Html5*/
const Value V22702_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_313 = {S32, 0, &V22702_313_57259, &P22702_314};	/*Property = Html-Media-Capture*/
const Value V22702_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_315 = {S33, 0, &V22702_315_57265, &P22702_313};	/*Property = Iframe*/
const Value V22702_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_316 = {S34, 0, &V22702_316_57268, &P22702_315};	/*Property = IndexedDB*/
const Value V22702_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_317 = {S35, 0, &V22702_317_57271, &P22702_316};	/*Property = Json*/
const Value V22702_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_15 = {S37, 0, &V22702_15_72, &P22702_317};	/*Property = LayoutEngine*/
const Value V22702_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_344 = {S38, 0, &V22702_344_57274, &P22702_15};	/*Property = Masking*/
const Value V22702_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_320 = {S39, 0, &V22702_320_57292, &P22702_344};	/*Property = PostMessage*/
const Value V22702_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_321 = {S40, 0, &V22702_321_57295, &P22702_320};	/*Property = Progress*/
const Value V22702_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_322 = {S41, 0, &V22702_322_57298, &P22702_321};	/*Property = Prompts*/
const Value V22702_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_323 = {S42, 0, &V22702_323_57301, &P22702_322};	/*Property = Selector*/
const Value V22702_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_324 = {S43, 0, &V22702_324_57307, &P22702_323};	/*Property = Svg*/
const Value V22702_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_327 = {S44, 0, &V22702_327_57316, &P22702_324};	/*Property = TouchEvents*/
const Value V22702_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_328 = {S45, 0, &V22702_328_57319, &P22702_327};	/*Property = Track*/
const Value V22702_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_329 = {S46, 0, &V22702_329_57325, &P22702_328};	/*Property = Video*/
const Value V22702_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_330 = {S47, 0, &V22702_330_57328, &P22702_329};	/*Property = Viewport*/
const Value V22702_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_332 = {S48, 0, &V22702_332_57340, &P22702_330};	/*Property = WebWorkers*/
const Value V22702_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22702_333 = {S49, 0, &V22702_333_57343, &P22702_332};	/*Property = Xhr2*/
const Profile P22702 = {"22702", &P22702_333};	/*Profile ID - 22702*/
const Value V17830_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17830_279 = {S1, 0, &V17830_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17830_282_57153 = {S2, NULL};	/*Value = True*/
const Property P17830_282 = {S3, 0, &V17830_282_57153, &P17830_279};	/*Property = BlobBuilder*/
const Value V17830_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17830_350 = {S4, 0, &V17830_350_57444, &P17830_282};	/*Property = Canvas*/
const Value V17830_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17830_283 = {S5, 0, &V17830_283_57159, &P17830_350};	/*Property = CssBackground*/
const Value V17830_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17830_284 = {S6, 0, &V17830_284_57163, &P17830_283};	/*Property = CssBorderImage*/
const Value V17830_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17830_285 = {S7, 0, &V17830_285_57166, &P17830_284};	/*Property = CssCanvas*/
const Value V17830_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17830_286 = {S8, 0, &V17830_286_57168, &P17830_285};	/*Property = CssColor*/
const Value V17830_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17830_237 = {S9, 0, &V17830_237_55784, &P17830_286};	/*Property = CssColumn*/
const Value V17830_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17830_287 = {S10, 0, &V17830_287_57172, &P17830_237};	/*Property = CssFlexbox*/
const Value V17830_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17830_288 = {S11, 0, &V17830_288_57175, &P17830_287};	/*Property = CssFont*/
const Value V17830_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17830_289 = {S12, 0, &V17830_289_57759, &P17830_288};	/*Property = CssImages*/
const Value V17830_291_57180 = {S2, NULL};	/*Value = True*/
const Property P17830_291 = {S13, 0, &V17830_291_57180, &P17830_289};	/*Property = CssMediaQueries*/
const Value V17830_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17830_292 = {S14, 0, &V17830_292_57183, &P17830_291};	/*Property = CssMinMax*/
const Value V17830_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17830_293 = {S15, 0, &V17830_293_57187, &P17830_292};	/*Property = CssOverflow*/
const Value V17830_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17830_294 = {S16, 0, &V17830_294_57189, &P17830_293};	/*Property = CssPosition*/
const Value V17830_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17830_295 = {S17, 0, &V17830_295_57192, &P17830_294};	/*Property = CssText*/
const Value V17830_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17830_297 = {S18, 0, &V17830_297_57195, &P17830_295};	/*Property = CssTransforms*/
const Value V17830_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17830_241 = {S19, 0, &V17830_241_55799, &P17830_297};	/*Property = CssTransitions*/
const Value V17830_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17830_298 = {S20, 0, &V17830_298_57206, &P17830_241};	/*Property = CssUI*/
const Value V17830_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17830_300 = {S21, 0, &V17830_300_57216, &P17830_298};	/*Property = DataSet*/
const Value V17830_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17830_301 = {S22, 0, &V17830_301_57218, &P17830_300};	/*Property = DataUrl*/
const Value V17830_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17830_302 = {S23, 0, &V17830_302_57222, &P17830_301};	/*Property = DeviceOrientation*/
const Value V17830_304_57227 = {S2, NULL};	/*Value = True*/
const Property P17830_304 = {S24, 0, &V17830_304_57227, &P17830_302};	/*Property = FileReader*/
const Value V17830_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17830_305 = {S25, 0, &V17830_305_57231, &P17830_304};	/*Property = FileSaver*/
const Value V17830_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17830_306 = {S26, 0, &V17830_306_57237, &P17830_305};	/*Property = FileWriter*/
const Value V17830_307_57239 = {S2, NULL};	/*Value = True*/
const Property P17830_307 = {S27, 0, &V17830_307_57239, &P17830_306};	/*Property = FormData*/
const Value V17830_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17830_309 = {S28, 0, &V17830_309_57246, &P17830_307};	/*Property = Fullscreen*/
const Value V17830_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17830_310 = {S29, 0, &V17830_310_57248, &P17830_309};	/*Property = GeoLocation*/
const Value V17830_312_57254 = {S2, NULL};	/*Value = True*/
const Property P17830_312 = {S30, 0, &V17830_312_57254, &P17830_310};	/*Property = History*/
const Value V17830_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17830_314 = {S31, 0, &V17830_314_57261, &P17830_312};	/*Property = Html5*/
const Value V17830_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17830_313 = {S32, 0, &V17830_313_57258, &P17830_314};	/*Property = Html-Media-Capture*/
const Value V17830_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17830_315 = {S33, 0, &V17830_315_57264, &P17830_313};	/*Property = Iframe*/
const Value V17830_316_57266 = {S2, NULL};	/*Value = True*/
const Property P17830_316 = {S34, 0, &V17830_316_57266, &P17830_315};	/*Property = IndexedDB*/
const Value V17830_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17830_317 = {S35, 0, &V17830_317_57269, &P17830_316};	/*Property = Json*/
const Value V17830_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17830_15 = {S37, 0, &V17830_15_28, &P17830_317};	/*Property = LayoutEngine*/
const Value V17830_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17830_344 = {S38, 0, &V17830_344_57272, &P17830_15};	/*Property = Masking*/
const Value V17830_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17830_320 = {S39, 0, &V17830_320_57290, &P17830_344};	/*Property = PostMessage*/
const Value V17830_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17830_321 = {S40, 0, &V17830_321_57293, &P17830_320};	/*Property = Progress*/
const Value V17830_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17830_322 = {S41, 0, &V17830_322_57296, &P17830_321};	/*Property = Prompts*/
const Value V17830_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17830_323 = {S42, 0, &V17830_323_57299, &P17830_322};	/*Property = Selector*/
const Value V17830_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17830_324 = {S43, 0, &V17830_324_57306, &P17830_323};	/*Property = Svg*/
const Value V17830_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17830_327 = {S44, 0, &V17830_327_57315, &P17830_324};	/*Property = TouchEvents*/
const Value V17830_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17830_328 = {S45, 0, &V17830_328_57318, &P17830_327};	/*Property = Track*/
const Value V17830_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17830_329 = {S46, 0, &V17830_329_57323, &P17830_328};	/*Property = Video*/
const Value V17830_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17830_330 = {S47, 0, &V17830_330_57326, &P17830_329};	/*Property = Viewport*/
const Value V17830_332_57338 = {S2, NULL};	/*Value = True*/
const Property P17830_332 = {S48, 0, &V17830_332_57338, &P17830_330};	/*Property = WebWorkers*/
const Value V17830_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17830_333 = {S49, 0, &V17830_333_57342, &P17830_332};	/*Property = Xhr2*/
const Profile P17830 = {"17830", &P17830_333};	/*Profile ID - 17830*/
const Value V17262_1_1 = {S2, NULL};	/*Value = True*/
const Property P17262_1 = {S50, 0, &V17262_1_1, NULL};	/*Property = IsMobile*/
const Value V17262_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P17262_5 = {S52, 0, &V17262_5_1458, &P17262_1};	/*Property = ScreenPixelsHeight*/
const Value V17262_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P17262_4 = {S54, 0, &V17262_4_1460, &P17262_5};	/*Property = ScreenPixelsWidth*/
const Profile P17262 = {"17262", &P17262_4};	/*Profile ID - 17262*/
const Value V17400_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17400_279 = {S1, 0, &V17400_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17400_282_57153 = {S2, NULL};	/*Value = True*/
const Property P17400_282 = {S3, 0, &V17400_282_57153, &P17400_279};	/*Property = BlobBuilder*/
const Value V17400_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17400_350 = {S4, 0, &V17400_350_57444, &P17400_282};	/*Property = Canvas*/
const Value V17400_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17400_283 = {S5, 0, &V17400_283_57159, &P17400_350};	/*Property = CssBackground*/
const Value V17400_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17400_284 = {S6, 0, &V17400_284_57163, &P17400_283};	/*Property = CssBorderImage*/
const Value V17400_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17400_285 = {S7, 0, &V17400_285_57166, &P17400_284};	/*Property = CssCanvas*/
const Value V17400_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17400_286 = {S8, 0, &V17400_286_57168, &P17400_285};	/*Property = CssColor*/
const Value V17400_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17400_237 = {S9, 0, &V17400_237_55784, &P17400_286};	/*Property = CssColumn*/
const Value V17400_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17400_287 = {S10, 0, &V17400_287_57172, &P17400_237};	/*Property = CssFlexbox*/
const Value V17400_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17400_288 = {S11, 0, &V17400_288_57175, &P17400_287};	/*Property = CssFont*/
const Value V17400_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17400_289 = {S12, 0, &V17400_289_57759, &P17400_288};	/*Property = CssImages*/
const Value V17400_291_57180 = {S2, NULL};	/*Value = True*/
const Property P17400_291 = {S13, 0, &V17400_291_57180, &P17400_289};	/*Property = CssMediaQueries*/
const Value V17400_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17400_292 = {S14, 0, &V17400_292_57183, &P17400_291};	/*Property = CssMinMax*/
const Value V17400_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17400_293 = {S15, 0, &V17400_293_57187, &P17400_292};	/*Property = CssOverflow*/
const Value V17400_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17400_294 = {S16, 0, &V17400_294_57189, &P17400_293};	/*Property = CssPosition*/
const Value V17400_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17400_295 = {S17, 0, &V17400_295_57192, &P17400_294};	/*Property = CssText*/
const Value V17400_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17400_297 = {S18, 0, &V17400_297_57195, &P17400_295};	/*Property = CssTransforms*/
const Value V17400_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17400_241 = {S19, 0, &V17400_241_55799, &P17400_297};	/*Property = CssTransitions*/
const Value V17400_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17400_298 = {S20, 0, &V17400_298_57206, &P17400_241};	/*Property = CssUI*/
const Value V17400_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17400_300 = {S21, 0, &V17400_300_57216, &P17400_298};	/*Property = DataSet*/
const Value V17400_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17400_301 = {S22, 0, &V17400_301_57218, &P17400_300};	/*Property = DataUrl*/
const Value V17400_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17400_302 = {S23, 0, &V17400_302_57222, &P17400_301};	/*Property = DeviceOrientation*/
const Value V17400_304_57227 = {S2, NULL};	/*Value = True*/
const Property P17400_304 = {S24, 0, &V17400_304_57227, &P17400_302};	/*Property = FileReader*/
const Value V17400_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17400_305 = {S25, 0, &V17400_305_57231, &P17400_304};	/*Property = FileSaver*/
const Value V17400_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17400_306 = {S26, 0, &V17400_306_57237, &P17400_305};	/*Property = FileWriter*/
const Value V17400_307_57239 = {S2, NULL};	/*Value = True*/
const Property P17400_307 = {S27, 0, &V17400_307_57239, &P17400_306};	/*Property = FormData*/
const Value V17400_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17400_309 = {S28, 0, &V17400_309_57246, &P17400_307};	/*Property = Fullscreen*/
const Value V17400_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17400_310 = {S29, 0, &V17400_310_57248, &P17400_309};	/*Property = GeoLocation*/
const Value V17400_312_57254 = {S2, NULL};	/*Value = True*/
const Property P17400_312 = {S30, 0, &V17400_312_57254, &P17400_310};	/*Property = History*/
const Value V17400_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17400_314 = {S31, 0, &V17400_314_57261, &P17400_312};	/*Property = Html5*/
const Value V17400_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17400_313 = {S32, 0, &V17400_313_57258, &P17400_314};	/*Property = Html-Media-Capture*/
const Value V17400_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17400_315 = {S33, 0, &V17400_315_57264, &P17400_313};	/*Property = Iframe*/
const Value V17400_316_57266 = {S2, NULL};	/*Value = True*/
const Property P17400_316 = {S34, 0, &V17400_316_57266, &P17400_315};	/*Property = IndexedDB*/
const Value V17400_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17400_317 = {S35, 0, &V17400_317_57269, &P17400_316};	/*Property = Json*/
const Value V17400_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17400_15 = {S37, 0, &V17400_15_28, &P17400_317};	/*Property = LayoutEngine*/
const Value V17400_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17400_344 = {S38, 0, &V17400_344_57272, &P17400_15};	/*Property = Masking*/
const Value V17400_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17400_320 = {S39, 0, &V17400_320_57290, &P17400_344};	/*Property = PostMessage*/
const Value V17400_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17400_321 = {S40, 0, &V17400_321_57293, &P17400_320};	/*Property = Progress*/
const Value V17400_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17400_322 = {S41, 0, &V17400_322_57296, &P17400_321};	/*Property = Prompts*/
const Value V17400_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17400_323 = {S42, 0, &V17400_323_57299, &P17400_322};	/*Property = Selector*/
const Value V17400_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17400_324 = {S43, 0, &V17400_324_57306, &P17400_323};	/*Property = Svg*/
const Value V17400_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17400_327 = {S44, 0, &V17400_327_57315, &P17400_324};	/*Property = TouchEvents*/
const Value V17400_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17400_328 = {S45, 0, &V17400_328_57318, &P17400_327};	/*Property = Track*/
const Value V17400_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17400_329 = {S46, 0, &V17400_329_57323, &P17400_328};	/*Property = Video*/
const Value V17400_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17400_330 = {S47, 0, &V17400_330_57326, &P17400_329};	/*Property = Viewport*/
const Value V17400_332_57338 = {S2, NULL};	/*Value = True*/
const Property P17400_332 = {S48, 0, &V17400_332_57338, &P17400_330};	/*Property = WebWorkers*/
const Value V17400_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17400_333 = {S49, 0, &V17400_333_57342, &P17400_332};	/*Property = Xhr2*/
const Profile P17400 = {"17400", &P17400_333};	/*Profile ID - 17400*/
const Value V21901_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21901_279 = {S1, 0, &V21901_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21901_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21901_282 = {S3, 0, &V21901_282_57154, &P21901_279};	/*Property = BlobBuilder*/
const Value V21901_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21901_350 = {S4, 0, &V21901_350_57444, &P21901_282};	/*Property = Canvas*/
const Value V21901_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21901_283 = {S5, 0, &V21901_283_57159, &P21901_350};	/*Property = CssBackground*/
const Value V21901_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21901_284 = {S6, 0, &V21901_284_57163, &P21901_283};	/*Property = CssBorderImage*/
const Value V21901_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21901_285 = {S7, 0, &V21901_285_57166, &P21901_284};	/*Property = CssCanvas*/
const Value V21901_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21901_286 = {S8, 0, &V21901_286_57168, &P21901_285};	/*Property = CssColor*/
const Value V21901_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21901_237 = {S9, 0, &V21901_237_55784, &P21901_286};	/*Property = CssColumn*/
const Value V21901_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21901_287 = {S10, 0, &V21901_287_57172, &P21901_237};	/*Property = CssFlexbox*/
const Value V21901_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21901_288 = {S11, 0, &V21901_288_57175, &P21901_287};	/*Property = CssFont*/
const Value V21901_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21901_289 = {S12, 0, &V21901_289_57759, &P21901_288};	/*Property = CssImages*/
const Value V21901_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21901_291 = {S13, 0, &V21901_291_57180, &P21901_289};	/*Property = CssMediaQueries*/
const Value V21901_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21901_292 = {S14, 0, &V21901_292_57183, &P21901_291};	/*Property = CssMinMax*/
const Value V21901_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21901_293 = {S15, 0, &V21901_293_57187, &P21901_292};	/*Property = CssOverflow*/
const Value V21901_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21901_294 = {S16, 0, &V21901_294_57189, &P21901_293};	/*Property = CssPosition*/
const Value V21901_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21901_295 = {S17, 0, &V21901_295_57192, &P21901_294};	/*Property = CssText*/
const Value V21901_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21901_297 = {S18, 0, &V21901_297_57195, &P21901_295};	/*Property = CssTransforms*/
const Value V21901_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21901_241 = {S19, 0, &V21901_241_55799, &P21901_297};	/*Property = CssTransitions*/
const Value V21901_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21901_298 = {S20, 0, &V21901_298_57206, &P21901_241};	/*Property = CssUI*/
const Value V21901_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21901_300 = {S21, 0, &V21901_300_57216, &P21901_298};	/*Property = DataSet*/
const Value V21901_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21901_301 = {S22, 0, &V21901_301_57218, &P21901_300};	/*Property = DataUrl*/
const Value V21901_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21901_302 = {S23, 0, &V21901_302_57222, &P21901_301};	/*Property = DeviceOrientation*/
const Value V21901_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21901_304 = {S24, 0, &V21901_304_57227, &P21901_302};	/*Property = FileReader*/
const Value V21901_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21901_305 = {S25, 0, &V21901_305_57231, &P21901_304};	/*Property = FileSaver*/
const Value V21901_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21901_306 = {S26, 0, &V21901_306_57237, &P21901_305};	/*Property = FileWriter*/
const Value V21901_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21901_307 = {S27, 0, &V21901_307_57239, &P21901_306};	/*Property = FormData*/
const Value V21901_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21901_309 = {S28, 0, &V21901_309_57246, &P21901_307};	/*Property = Fullscreen*/
const Value V21901_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21901_310 = {S29, 0, &V21901_310_57248, &P21901_309};	/*Property = GeoLocation*/
const Value V21901_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21901_312 = {S30, 0, &V21901_312_57254, &P21901_310};	/*Property = History*/
const Value V21901_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21901_314 = {S31, 0, &V21901_314_57261, &P21901_312};	/*Property = Html5*/
const Value V21901_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21901_313 = {S32, 0, &V21901_313_57258, &P21901_314};	/*Property = Html-Media-Capture*/
const Value V21901_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21901_315 = {S33, 0, &V21901_315_57264, &P21901_313};	/*Property = Iframe*/
const Value V21901_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21901_316 = {S34, 0, &V21901_316_57266, &P21901_315};	/*Property = IndexedDB*/
const Value V21901_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21901_317 = {S35, 0, &V21901_317_57269, &P21901_316};	/*Property = Json*/
const Value V21901_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21901_15 = {S37, 0, &V21901_15_28, &P21901_317};	/*Property = LayoutEngine*/
const Value V21901_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21901_344 = {S38, 0, &V21901_344_57272, &P21901_15};	/*Property = Masking*/
const Value V21901_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21901_320 = {S39, 0, &V21901_320_57290, &P21901_344};	/*Property = PostMessage*/
const Value V21901_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21901_321 = {S40, 0, &V21901_321_57293, &P21901_320};	/*Property = Progress*/
const Value V21901_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21901_322 = {S41, 0, &V21901_322_57296, &P21901_321};	/*Property = Prompts*/
const Value V21901_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21901_323 = {S42, 0, &V21901_323_57299, &P21901_322};	/*Property = Selector*/
const Value V21901_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21901_324 = {S43, 0, &V21901_324_57306, &P21901_323};	/*Property = Svg*/
const Value V21901_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21901_327 = {S44, 0, &V21901_327_57315, &P21901_324};	/*Property = TouchEvents*/
const Value V21901_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21901_328 = {S45, 0, &V21901_328_57318, &P21901_327};	/*Property = Track*/
const Value V21901_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21901_329 = {S46, 0, &V21901_329_57323, &P21901_328};	/*Property = Video*/
const Value V21901_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21901_330 = {S47, 0, &V21901_330_57326, &P21901_329};	/*Property = Viewport*/
const Value V21901_332_57338 = {S2, NULL};	/*Value = True*/
const Property P21901_332 = {S48, 0, &V21901_332_57338, &P21901_330};	/*Property = WebWorkers*/
const Value V21901_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21901_333 = {S49, 0, &V21901_333_57342, &P21901_332};	/*Property = Xhr2*/
const Profile P21901 = {"21901", &P21901_333};	/*Profile ID - 21901*/
const Profile P18570 = {"18570", NULL};	/*Profile ID - 18570*/
const Value V17216_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17216_279 = {S1, 0, &V17216_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17216_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17216_282 = {S3, 0, &V17216_282_57154, &P17216_279};	/*Property = BlobBuilder*/
const Value V17216_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17216_350 = {S4, 0, &V17216_350_57444, &P17216_282};	/*Property = Canvas*/
const Value V17216_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17216_283 = {S5, 0, &V17216_283_57159, &P17216_350};	/*Property = CssBackground*/
const Value V17216_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17216_284 = {S6, 0, &V17216_284_57163, &P17216_283};	/*Property = CssBorderImage*/
const Value V17216_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17216_285 = {S7, 0, &V17216_285_57166, &P17216_284};	/*Property = CssCanvas*/
const Value V17216_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17216_286 = {S8, 0, &V17216_286_57168, &P17216_285};	/*Property = CssColor*/
const Value V17216_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17216_237 = {S9, 0, &V17216_237_55784, &P17216_286};	/*Property = CssColumn*/
const Value V17216_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17216_287 = {S10, 0, &V17216_287_57172, &P17216_237};	/*Property = CssFlexbox*/
const Value V17216_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17216_288 = {S11, 0, &V17216_288_57175, &P17216_287};	/*Property = CssFont*/
const Value V17216_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17216_289 = {S12, 0, &V17216_289_57759, &P17216_288};	/*Property = CssImages*/
const Value V17216_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17216_291 = {S13, 0, &V17216_291_57181, &P17216_289};	/*Property = CssMediaQueries*/
const Value V17216_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17216_292 = {S14, 0, &V17216_292_57183, &P17216_291};	/*Property = CssMinMax*/
const Value V17216_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17216_293 = {S15, 0, &V17216_293_57187, &P17216_292};	/*Property = CssOverflow*/
const Value V17216_294_57190 = {S0, NULL};	/*Value = False*/
const Property P17216_294 = {S16, 0, &V17216_294_57190, &P17216_293};	/*Property = CssPosition*/
const Value V17216_295_57193 = {S0, NULL};	/*Value = False*/
const Property P17216_295 = {S17, 0, &V17216_295_57193, &P17216_294};	/*Property = CssText*/
const Value V17216_297_57196 = {S0, NULL};	/*Value = False*/
const Property P17216_297 = {S18, 0, &V17216_297_57196, &P17216_295};	/*Property = CssTransforms*/
const Value V17216_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17216_241 = {S19, 0, &V17216_241_55799, &P17216_297};	/*Property = CssTransitions*/
const Value V17216_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17216_298 = {S20, 0, &V17216_298_57206, &P17216_241};	/*Property = CssUI*/
const Value V17216_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17216_300 = {S21, 0, &V17216_300_57216, &P17216_298};	/*Property = DataSet*/
const Value V17216_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17216_301 = {S22, 0, &V17216_301_57218, &P17216_300};	/*Property = DataUrl*/
const Value V17216_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17216_302 = {S23, 0, &V17216_302_57222, &P17216_301};	/*Property = DeviceOrientation*/
const Value V17216_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17216_304 = {S24, 0, &V17216_304_57228, &P17216_302};	/*Property = FileReader*/
const Value V17216_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17216_305 = {S25, 0, &V17216_305_57231, &P17216_304};	/*Property = FileSaver*/
const Value V17216_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17216_306 = {S26, 0, &V17216_306_57237, &P17216_305};	/*Property = FileWriter*/
const Value V17216_307_57240 = {S0, NULL};	/*Value = False*/
const Property P17216_307 = {S27, 0, &V17216_307_57240, &P17216_306};	/*Property = FormData*/
const Value V17216_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17216_309 = {S28, 0, &V17216_309_57246, &P17216_307};	/*Property = Fullscreen*/
const Value V17216_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17216_310 = {S29, 0, &V17216_310_57248, &P17216_309};	/*Property = GeoLocation*/
const Value V17216_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17216_312 = {S30, 0, &V17216_312_57255, &P17216_310};	/*Property = History*/
const Value V17216_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17216_314 = {S31, 0, &V17216_314_57261, &P17216_312};	/*Property = Html5*/
const Value V17216_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17216_313 = {S32, 0, &V17216_313_57258, &P17216_314};	/*Property = Html-Media-Capture*/
const Value V17216_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17216_315 = {S33, 0, &V17216_315_57264, &P17216_313};	/*Property = Iframe*/
const Value V17216_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17216_316 = {S34, 0, &V17216_316_57267, &P17216_315};	/*Property = IndexedDB*/
const Value V17216_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17216_317 = {S35, 0, &V17216_317_57269, &P17216_316};	/*Property = Json*/
const Value V17216_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P17216_15 = {S37, 0, &V17216_15_31, &P17216_317};	/*Property = LayoutEngine*/
const Value V17216_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17216_344 = {S38, 0, &V17216_344_57272, &P17216_15};	/*Property = Masking*/
const Value V17216_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17216_320 = {S39, 0, &V17216_320_57290, &P17216_344};	/*Property = PostMessage*/
const Value V17216_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17216_321 = {S40, 0, &V17216_321_57293, &P17216_320};	/*Property = Progress*/
const Value V17216_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17216_322 = {S41, 0, &V17216_322_57296, &P17216_321};	/*Property = Prompts*/
const Value V17216_323_57300 = {S0, NULL};	/*Value = False*/
const Property P17216_323 = {S42, 0, &V17216_323_57300, &P17216_322};	/*Property = Selector*/
const Value V17216_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17216_324 = {S43, 0, &V17216_324_57306, &P17216_323};	/*Property = Svg*/
const Value V17216_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17216_327 = {S44, 0, &V17216_327_57315, &P17216_324};	/*Property = TouchEvents*/
const Value V17216_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17216_328 = {S45, 0, &V17216_328_57318, &P17216_327};	/*Property = Track*/
const Value V17216_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17216_329 = {S46, 0, &V17216_329_57323, &P17216_328};	/*Property = Video*/
const Value V17216_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17216_330 = {S47, 0, &V17216_330_57326, &P17216_329};	/*Property = Viewport*/
const Value V17216_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17216_332 = {S48, 0, &V17216_332_57339, &P17216_330};	/*Property = WebWorkers*/
const Value V17216_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17216_333 = {S49, 0, &V17216_333_57342, &P17216_332};	/*Property = Xhr2*/
const Profile P17216 = {"17216", &P17216_333};	/*Profile ID - 17216*/
const Profile P18389 = {"18389", NULL};	/*Profile ID - 18389*/
const Value V18390_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_279 = {S1, 0, &V18390_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18390_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_282 = {S3, 0, &V18390_282_57155, &P18390_279};	/*Property = BlobBuilder*/
const Value V18390_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_350 = {S4, 0, &V18390_350_57446, &P18390_282};	/*Property = Canvas*/
const Value V18390_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_283 = {S5, 0, &V18390_283_57161, &P18390_350};	/*Property = CssBackground*/
const Value V18390_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_284 = {S6, 0, &V18390_284_57164, &P18390_283};	/*Property = CssBorderImage*/
const Value V18390_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_285 = {S7, 0, &V18390_285_57167, &P18390_284};	/*Property = CssCanvas*/
const Value V18390_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_286 = {S8, 0, &V18390_286_57170, &P18390_285};	/*Property = CssColor*/
const Value V18390_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_237 = {S9, 0, &V18390_237_55786, &P18390_286};	/*Property = CssColumn*/
const Value V18390_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_287 = {S10, 0, &V18390_287_57173, &P18390_237};	/*Property = CssFlexbox*/
const Value V18390_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_288 = {S11, 0, &V18390_288_57176, &P18390_287};	/*Property = CssFont*/
const Value V18390_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_289 = {S12, 0, &V18390_289_86899, &P18390_288};	/*Property = CssImages*/
const Value V18390_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_291 = {S13, 0, &V18390_291_57182, &P18390_289};	/*Property = CssMediaQueries*/
const Value V18390_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_292 = {S14, 0, &V18390_292_57185, &P18390_291};	/*Property = CssMinMax*/
const Value V18390_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_293 = {S15, 0, &V18390_293_57188, &P18390_292};	/*Property = CssOverflow*/
const Value V18390_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_294 = {S16, 0, &V18390_294_57191, &P18390_293};	/*Property = CssPosition*/
const Value V18390_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_295 = {S17, 0, &V18390_295_57194, &P18390_294};	/*Property = CssText*/
const Value V18390_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_297 = {S18, 0, &V18390_297_57197, &P18390_295};	/*Property = CssTransforms*/
const Value V18390_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_241 = {S19, 0, &V18390_241_55801, &P18390_297};	/*Property = CssTransitions*/
const Value V18390_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_298 = {S20, 0, &V18390_298_57208, &P18390_241};	/*Property = CssUI*/
const Value V18390_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_300 = {S21, 0, &V18390_300_57217, &P18390_298};	/*Property = DataSet*/
const Value V18390_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_301 = {S22, 0, &V18390_301_57220, &P18390_300};	/*Property = DataUrl*/
const Value V18390_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_302 = {S23, 0, &V18390_302_57223, &P18390_301};	/*Property = DeviceOrientation*/
const Value V18390_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_304 = {S24, 0, &V18390_304_57229, &P18390_302};	/*Property = FileReader*/
const Value V18390_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_305 = {S25, 0, &V18390_305_57232, &P18390_304};	/*Property = FileSaver*/
const Value V18390_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_306 = {S26, 0, &V18390_306_57238, &P18390_305};	/*Property = FileWriter*/
const Value V18390_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_307 = {S27, 0, &V18390_307_57241, &P18390_306};	/*Property = FormData*/
const Value V18390_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_309 = {S28, 0, &V18390_309_57247, &P18390_307};	/*Property = Fullscreen*/
const Value V18390_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_310 = {S29, 0, &V18390_310_57250, &P18390_309};	/*Property = GeoLocation*/
const Value V18390_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_312 = {S30, 0, &V18390_312_57256, &P18390_310};	/*Property = History*/
const Value V18390_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_314 = {S31, 0, &V18390_314_57262, &P18390_312};	/*Property = Html5*/
const Value V18390_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_313 = {S32, 0, &V18390_313_57259, &P18390_314};	/*Property = Html-Media-Capture*/
const Value V18390_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_315 = {S33, 0, &V18390_315_57265, &P18390_313};	/*Property = Iframe*/
const Value V18390_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_316 = {S34, 0, &V18390_316_57268, &P18390_315};	/*Property = IndexedDB*/
const Value V18390_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_317 = {S35, 0, &V18390_317_57271, &P18390_316};	/*Property = Json*/
const Value V18390_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_15 = {S37, 0, &V18390_15_72, &P18390_317};	/*Property = LayoutEngine*/
const Value V18390_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_344 = {S38, 0, &V18390_344_57274, &P18390_15};	/*Property = Masking*/
const Value V18390_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_320 = {S39, 0, &V18390_320_57292, &P18390_344};	/*Property = PostMessage*/
const Value V18390_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_321 = {S40, 0, &V18390_321_57295, &P18390_320};	/*Property = Progress*/
const Value V18390_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_322 = {S41, 0, &V18390_322_57298, &P18390_321};	/*Property = Prompts*/
const Value V18390_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_323 = {S42, 0, &V18390_323_57301, &P18390_322};	/*Property = Selector*/
const Value V18390_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_324 = {S43, 0, &V18390_324_57307, &P18390_323};	/*Property = Svg*/
const Value V18390_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_327 = {S44, 0, &V18390_327_57316, &P18390_324};	/*Property = TouchEvents*/
const Value V18390_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_328 = {S45, 0, &V18390_328_57319, &P18390_327};	/*Property = Track*/
const Value V18390_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_329 = {S46, 0, &V18390_329_57325, &P18390_328};	/*Property = Video*/
const Value V18390_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_330 = {S47, 0, &V18390_330_57328, &P18390_329};	/*Property = Viewport*/
const Value V18390_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_332 = {S48, 0, &V18390_332_57340, &P18390_330};	/*Property = WebWorkers*/
const Value V18390_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18390_333 = {S49, 0, &V18390_333_57343, &P18390_332};	/*Property = Xhr2*/
const Profile P18390 = {"18390", &P18390_333};	/*Profile ID - 18390*/
const Value V22731_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22731_279 = {S1, 0, &V22731_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22731_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22731_282 = {S3, 0, &V22731_282_57154, &P22731_279};	/*Property = BlobBuilder*/
const Value V22731_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22731_350 = {S4, 0, &V22731_350_57444, &P22731_282};	/*Property = Canvas*/
const Value V22731_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22731_283 = {S5, 0, &V22731_283_57159, &P22731_350};	/*Property = CssBackground*/
const Value V22731_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22731_284 = {S6, 0, &V22731_284_57163, &P22731_283};	/*Property = CssBorderImage*/
const Value V22731_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22731_285 = {S7, 0, &V22731_285_57166, &P22731_284};	/*Property = CssCanvas*/
const Value V22731_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22731_286 = {S8, 0, &V22731_286_57168, &P22731_285};	/*Property = CssColor*/
const Value V22731_237_55785 = {S0, NULL};	/*Value = False*/
const Property P22731_237 = {S9, 0, &V22731_237_55785, &P22731_286};	/*Property = CssColumn*/
const Value V22731_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22731_287 = {S10, 0, &V22731_287_57172, &P22731_237};	/*Property = CssFlexbox*/
const Value V22731_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22731_288 = {S11, 0, &V22731_288_57175, &P22731_287};	/*Property = CssFont*/
const Value V22731_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22731_289 = {S12, 0, &V22731_289_57759, &P22731_288};	/*Property = CssImages*/
const Value V22731_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22731_291 = {S13, 0, &V22731_291_57181, &P22731_289};	/*Property = CssMediaQueries*/
const Value V22731_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22731_292 = {S14, 0, &V22731_292_57183, &P22731_291};	/*Property = CssMinMax*/
const Value V22731_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22731_293 = {S15, 0, &V22731_293_57187, &P22731_292};	/*Property = CssOverflow*/
const Value V22731_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22731_294 = {S16, 0, &V22731_294_57190, &P22731_293};	/*Property = CssPosition*/
const Value V22731_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22731_295 = {S17, 0, &V22731_295_57192, &P22731_294};	/*Property = CssText*/
const Value V22731_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22731_297 = {S18, 0, &V22731_297_57195, &P22731_295};	/*Property = CssTransforms*/
const Value V22731_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22731_241 = {S19, 0, &V22731_241_55799, &P22731_297};	/*Property = CssTransitions*/
const Value V22731_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22731_298 = {S20, 0, &V22731_298_57206, &P22731_241};	/*Property = CssUI*/
const Value V22731_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22731_300 = {S21, 0, &V22731_300_57216, &P22731_298};	/*Property = DataSet*/
const Value V22731_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22731_301 = {S22, 0, &V22731_301_57218, &P22731_300};	/*Property = DataUrl*/
const Value V22731_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22731_302 = {S23, 0, &V22731_302_57222, &P22731_301};	/*Property = DeviceOrientation*/
const Value V22731_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22731_304 = {S24, 0, &V22731_304_57228, &P22731_302};	/*Property = FileReader*/
const Value V22731_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22731_305 = {S25, 0, &V22731_305_57231, &P22731_304};	/*Property = FileSaver*/
const Value V22731_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22731_306 = {S26, 0, &V22731_306_57237, &P22731_305};	/*Property = FileWriter*/
const Value V22731_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22731_307 = {S27, 0, &V22731_307_57240, &P22731_306};	/*Property = FormData*/
const Value V22731_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22731_309 = {S28, 0, &V22731_309_57246, &P22731_307};	/*Property = Fullscreen*/
const Value V22731_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22731_310 = {S29, 0, &V22731_310_57248, &P22731_309};	/*Property = GeoLocation*/
const Value V22731_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22731_312 = {S30, 0, &V22731_312_57255, &P22731_310};	/*Property = History*/
const Value V22731_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22731_314 = {S31, 0, &V22731_314_57261, &P22731_312};	/*Property = Html5*/
const Value V22731_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22731_313 = {S32, 0, &V22731_313_57258, &P22731_314};	/*Property = Html-Media-Capture*/
const Value V22731_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22731_315 = {S33, 0, &V22731_315_57264, &P22731_313};	/*Property = Iframe*/
const Value V22731_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22731_316 = {S34, 0, &V22731_316_57267, &P22731_315};	/*Property = IndexedDB*/
const Value V22731_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22731_317 = {S35, 0, &V22731_317_57269, &P22731_316};	/*Property = Json*/
const Value V22731_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P22731_15 = {S37, 0, &V22731_15_30, &P22731_317};	/*Property = LayoutEngine*/
const Value V22731_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22731_344 = {S38, 0, &V22731_344_57272, &P22731_15};	/*Property = Masking*/
const Value V22731_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22731_320 = {S39, 0, &V22731_320_57290, &P22731_344};	/*Property = PostMessage*/
const Value V22731_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22731_321 = {S40, 0, &V22731_321_57293, &P22731_320};	/*Property = Progress*/
const Value V22731_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22731_322 = {S41, 0, &V22731_322_57296, &P22731_321};	/*Property = Prompts*/
const Value V22731_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22731_323 = {S42, 0, &V22731_323_57299, &P22731_322};	/*Property = Selector*/
const Value V22731_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22731_324 = {S43, 0, &V22731_324_57306, &P22731_323};	/*Property = Svg*/
const Value V22731_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22731_327 = {S44, 0, &V22731_327_57315, &P22731_324};	/*Property = TouchEvents*/
const Value V22731_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22731_328 = {S45, 0, &V22731_328_57318, &P22731_327};	/*Property = Track*/
const Value V22731_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22731_329 = {S46, 0, &V22731_329_57323, &P22731_328};	/*Property = Video*/
const Value V22731_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22731_330 = {S47, 0, &V22731_330_57326, &P22731_329};	/*Property = Viewport*/
const Value V22731_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22731_332 = {S48, 0, &V22731_332_57339, &P22731_330};	/*Property = WebWorkers*/
const Value V22731_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22731_333 = {S49, 0, &V22731_333_57342, &P22731_332};	/*Property = Xhr2*/
const Profile P22731 = {"22731", &P22731_333};	/*Profile ID - 22731*/
const Value V17891_1_1 = {S2, NULL};	/*Value = True*/
const Property P17891_1 = {S50, 0, &V17891_1_1, NULL};	/*Property = IsMobile*/
const Value V17891_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P17891_5 = {S52, 0, &V17891_5_1333, &P17891_1};	/*Property = ScreenPixelsHeight*/
const Value V17891_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P17891_4 = {S54, 0, &V17891_4_1335, &P17891_5};	/*Property = ScreenPixelsWidth*/
const Profile P17891 = {"17891", &P17891_4};	/*Profile ID - 17891*/
const Value V18096_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_279 = {S1, 0, &V18096_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18096_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_282 = {S3, 0, &V18096_282_57155, &P18096_279};	/*Property = BlobBuilder*/
const Value V18096_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_350 = {S4, 0, &V18096_350_57446, &P18096_282};	/*Property = Canvas*/
const Value V18096_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_283 = {S5, 0, &V18096_283_57161, &P18096_350};	/*Property = CssBackground*/
const Value V18096_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_284 = {S6, 0, &V18096_284_57164, &P18096_283};	/*Property = CssBorderImage*/
const Value V18096_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_285 = {S7, 0, &V18096_285_57167, &P18096_284};	/*Property = CssCanvas*/
const Value V18096_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_286 = {S8, 0, &V18096_286_57170, &P18096_285};	/*Property = CssColor*/
const Value V18096_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_237 = {S9, 0, &V18096_237_55786, &P18096_286};	/*Property = CssColumn*/
const Value V18096_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_287 = {S10, 0, &V18096_287_57173, &P18096_237};	/*Property = CssFlexbox*/
const Value V18096_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_288 = {S11, 0, &V18096_288_57176, &P18096_287};	/*Property = CssFont*/
const Value V18096_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_289 = {S12, 0, &V18096_289_86899, &P18096_288};	/*Property = CssImages*/
const Value V18096_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_291 = {S13, 0, &V18096_291_57182, &P18096_289};	/*Property = CssMediaQueries*/
const Value V18096_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_292 = {S14, 0, &V18096_292_57185, &P18096_291};	/*Property = CssMinMax*/
const Value V18096_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_293 = {S15, 0, &V18096_293_57188, &P18096_292};	/*Property = CssOverflow*/
const Value V18096_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_294 = {S16, 0, &V18096_294_57191, &P18096_293};	/*Property = CssPosition*/
const Value V18096_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_295 = {S17, 0, &V18096_295_57194, &P18096_294};	/*Property = CssText*/
const Value V18096_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_297 = {S18, 0, &V18096_297_57197, &P18096_295};	/*Property = CssTransforms*/
const Value V18096_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_241 = {S19, 0, &V18096_241_55801, &P18096_297};	/*Property = CssTransitions*/
const Value V18096_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_298 = {S20, 0, &V18096_298_57208, &P18096_241};	/*Property = CssUI*/
const Value V18096_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_300 = {S21, 0, &V18096_300_57217, &P18096_298};	/*Property = DataSet*/
const Value V18096_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_301 = {S22, 0, &V18096_301_57220, &P18096_300};	/*Property = DataUrl*/
const Value V18096_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_302 = {S23, 0, &V18096_302_57223, &P18096_301};	/*Property = DeviceOrientation*/
const Value V18096_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_304 = {S24, 0, &V18096_304_57229, &P18096_302};	/*Property = FileReader*/
const Value V18096_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_305 = {S25, 0, &V18096_305_57232, &P18096_304};	/*Property = FileSaver*/
const Value V18096_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_306 = {S26, 0, &V18096_306_57238, &P18096_305};	/*Property = FileWriter*/
const Value V18096_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_307 = {S27, 0, &V18096_307_57241, &P18096_306};	/*Property = FormData*/
const Value V18096_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_309 = {S28, 0, &V18096_309_57247, &P18096_307};	/*Property = Fullscreen*/
const Value V18096_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_310 = {S29, 0, &V18096_310_57250, &P18096_309};	/*Property = GeoLocation*/
const Value V18096_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_312 = {S30, 0, &V18096_312_57256, &P18096_310};	/*Property = History*/
const Value V18096_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_314 = {S31, 0, &V18096_314_57262, &P18096_312};	/*Property = Html5*/
const Value V18096_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_313 = {S32, 0, &V18096_313_57259, &P18096_314};	/*Property = Html-Media-Capture*/
const Value V18096_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_315 = {S33, 0, &V18096_315_57265, &P18096_313};	/*Property = Iframe*/
const Value V18096_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_316 = {S34, 0, &V18096_316_57268, &P18096_315};	/*Property = IndexedDB*/
const Value V18096_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_317 = {S35, 0, &V18096_317_57271, &P18096_316};	/*Property = Json*/
const Value V18096_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18096_15 = {S37, 0, &V18096_15_31, &P18096_317};	/*Property = LayoutEngine*/
const Value V18096_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_344 = {S38, 0, &V18096_344_57274, &P18096_15};	/*Property = Masking*/
const Value V18096_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_320 = {S39, 0, &V18096_320_57292, &P18096_344};	/*Property = PostMessage*/
const Value V18096_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_321 = {S40, 0, &V18096_321_57295, &P18096_320};	/*Property = Progress*/
const Value V18096_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_322 = {S41, 0, &V18096_322_57298, &P18096_321};	/*Property = Prompts*/
const Value V18096_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_323 = {S42, 0, &V18096_323_57301, &P18096_322};	/*Property = Selector*/
const Value V18096_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_324 = {S43, 0, &V18096_324_57307, &P18096_323};	/*Property = Svg*/
const Value V18096_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_327 = {S44, 0, &V18096_327_57316, &P18096_324};	/*Property = TouchEvents*/
const Value V18096_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_328 = {S45, 0, &V18096_328_57319, &P18096_327};	/*Property = Track*/
const Value V18096_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_329 = {S46, 0, &V18096_329_57325, &P18096_328};	/*Property = Video*/
const Value V18096_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_330 = {S47, 0, &V18096_330_57328, &P18096_329};	/*Property = Viewport*/
const Value V18096_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_332 = {S48, 0, &V18096_332_57340, &P18096_330};	/*Property = WebWorkers*/
const Value V18096_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18096_333 = {S49, 0, &V18096_333_57343, &P18096_332};	/*Property = Xhr2*/
const Profile P18096 = {"18096", &P18096_333};	/*Profile ID - 18096*/
const Value V22440_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_279 = {S1, 0, &V22440_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22440_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_282 = {S3, 0, &V22440_282_57155, &P22440_279};	/*Property = BlobBuilder*/
const Value V22440_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_350 = {S4, 0, &V22440_350_57446, &P22440_282};	/*Property = Canvas*/
const Value V22440_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_283 = {S5, 0, &V22440_283_57161, &P22440_350};	/*Property = CssBackground*/
const Value V22440_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_284 = {S6, 0, &V22440_284_57164, &P22440_283};	/*Property = CssBorderImage*/
const Value V22440_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_285 = {S7, 0, &V22440_285_57167, &P22440_284};	/*Property = CssCanvas*/
const Value V22440_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_286 = {S8, 0, &V22440_286_57170, &P22440_285};	/*Property = CssColor*/
const Value V22440_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_237 = {S9, 0, &V22440_237_55786, &P22440_286};	/*Property = CssColumn*/
const Value V22440_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_287 = {S10, 0, &V22440_287_57173, &P22440_237};	/*Property = CssFlexbox*/
const Value V22440_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_288 = {S11, 0, &V22440_288_57176, &P22440_287};	/*Property = CssFont*/
const Value V22440_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_289 = {S12, 0, &V22440_289_86899, &P22440_288};	/*Property = CssImages*/
const Value V22440_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_291 = {S13, 0, &V22440_291_57182, &P22440_289};	/*Property = CssMediaQueries*/
const Value V22440_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_292 = {S14, 0, &V22440_292_57185, &P22440_291};	/*Property = CssMinMax*/
const Value V22440_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_293 = {S15, 0, &V22440_293_57188, &P22440_292};	/*Property = CssOverflow*/
const Value V22440_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_294 = {S16, 0, &V22440_294_57191, &P22440_293};	/*Property = CssPosition*/
const Value V22440_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_295 = {S17, 0, &V22440_295_57194, &P22440_294};	/*Property = CssText*/
const Value V22440_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_297 = {S18, 0, &V22440_297_57197, &P22440_295};	/*Property = CssTransforms*/
const Value V22440_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_241 = {S19, 0, &V22440_241_55801, &P22440_297};	/*Property = CssTransitions*/
const Value V22440_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_298 = {S20, 0, &V22440_298_57208, &P22440_241};	/*Property = CssUI*/
const Value V22440_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_300 = {S21, 0, &V22440_300_57217, &P22440_298};	/*Property = DataSet*/
const Value V22440_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_301 = {S22, 0, &V22440_301_57220, &P22440_300};	/*Property = DataUrl*/
const Value V22440_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_302 = {S23, 0, &V22440_302_57223, &P22440_301};	/*Property = DeviceOrientation*/
const Value V22440_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_304 = {S24, 0, &V22440_304_57229, &P22440_302};	/*Property = FileReader*/
const Value V22440_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_305 = {S25, 0, &V22440_305_57232, &P22440_304};	/*Property = FileSaver*/
const Value V22440_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_306 = {S26, 0, &V22440_306_57238, &P22440_305};	/*Property = FileWriter*/
const Value V22440_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_307 = {S27, 0, &V22440_307_57241, &P22440_306};	/*Property = FormData*/
const Value V22440_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_309 = {S28, 0, &V22440_309_57247, &P22440_307};	/*Property = Fullscreen*/
const Value V22440_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_310 = {S29, 0, &V22440_310_57250, &P22440_309};	/*Property = GeoLocation*/
const Value V22440_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_312 = {S30, 0, &V22440_312_57256, &P22440_310};	/*Property = History*/
const Value V22440_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_314 = {S31, 0, &V22440_314_57262, &P22440_312};	/*Property = Html5*/
const Value V22440_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_313 = {S32, 0, &V22440_313_57259, &P22440_314};	/*Property = Html-Media-Capture*/
const Value V22440_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_315 = {S33, 0, &V22440_315_57265, &P22440_313};	/*Property = Iframe*/
const Value V22440_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_316 = {S34, 0, &V22440_316_57268, &P22440_315};	/*Property = IndexedDB*/
const Value V22440_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_317 = {S35, 0, &V22440_317_57271, &P22440_316};	/*Property = Json*/
const Value V22440_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22440_15 = {S37, 0, &V22440_15_28, &P22440_317};	/*Property = LayoutEngine*/
const Value V22440_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_344 = {S38, 0, &V22440_344_57274, &P22440_15};	/*Property = Masking*/
const Value V22440_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_320 = {S39, 0, &V22440_320_57292, &P22440_344};	/*Property = PostMessage*/
const Value V22440_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_321 = {S40, 0, &V22440_321_57295, &P22440_320};	/*Property = Progress*/
const Value V22440_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_322 = {S41, 0, &V22440_322_57298, &P22440_321};	/*Property = Prompts*/
const Value V22440_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_323 = {S42, 0, &V22440_323_57301, &P22440_322};	/*Property = Selector*/
const Value V22440_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_324 = {S43, 0, &V22440_324_57307, &P22440_323};	/*Property = Svg*/
const Value V22440_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_327 = {S44, 0, &V22440_327_57316, &P22440_324};	/*Property = TouchEvents*/
const Value V22440_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_328 = {S45, 0, &V22440_328_57319, &P22440_327};	/*Property = Track*/
const Value V22440_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_329 = {S46, 0, &V22440_329_57325, &P22440_328};	/*Property = Video*/
const Value V22440_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_330 = {S47, 0, &V22440_330_57328, &P22440_329};	/*Property = Viewport*/
const Value V22440_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_332 = {S48, 0, &V22440_332_57340, &P22440_330};	/*Property = WebWorkers*/
const Value V22440_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22440_333 = {S49, 0, &V22440_333_57343, &P22440_332};	/*Property = Xhr2*/
const Profile P22440 = {"22440", &P22440_333};	/*Profile ID - 22440*/
const Value V17825_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_279 = {S1, 0, &V17825_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17825_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_282 = {S3, 0, &V17825_282_57155, &P17825_279};	/*Property = BlobBuilder*/
const Value V17825_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_350 = {S4, 0, &V17825_350_57446, &P17825_282};	/*Property = Canvas*/
const Value V17825_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_283 = {S5, 0, &V17825_283_57161, &P17825_350};	/*Property = CssBackground*/
const Value V17825_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_284 = {S6, 0, &V17825_284_57164, &P17825_283};	/*Property = CssBorderImage*/
const Value V17825_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_285 = {S7, 0, &V17825_285_57167, &P17825_284};	/*Property = CssCanvas*/
const Value V17825_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_286 = {S8, 0, &V17825_286_57170, &P17825_285};	/*Property = CssColor*/
const Value V17825_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_237 = {S9, 0, &V17825_237_55786, &P17825_286};	/*Property = CssColumn*/
const Value V17825_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_287 = {S10, 0, &V17825_287_57173, &P17825_237};	/*Property = CssFlexbox*/
const Value V17825_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_288 = {S11, 0, &V17825_288_57176, &P17825_287};	/*Property = CssFont*/
const Value V17825_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_289 = {S12, 0, &V17825_289_86899, &P17825_288};	/*Property = CssImages*/
const Value V17825_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_291 = {S13, 0, &V17825_291_57182, &P17825_289};	/*Property = CssMediaQueries*/
const Value V17825_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_292 = {S14, 0, &V17825_292_57185, &P17825_291};	/*Property = CssMinMax*/
const Value V17825_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_293 = {S15, 0, &V17825_293_57188, &P17825_292};	/*Property = CssOverflow*/
const Value V17825_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_294 = {S16, 0, &V17825_294_57191, &P17825_293};	/*Property = CssPosition*/
const Value V17825_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_295 = {S17, 0, &V17825_295_57194, &P17825_294};	/*Property = CssText*/
const Value V17825_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_297 = {S18, 0, &V17825_297_57197, &P17825_295};	/*Property = CssTransforms*/
const Value V17825_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_241 = {S19, 0, &V17825_241_55801, &P17825_297};	/*Property = CssTransitions*/
const Value V17825_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_298 = {S20, 0, &V17825_298_57208, &P17825_241};	/*Property = CssUI*/
const Value V17825_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_300 = {S21, 0, &V17825_300_57217, &P17825_298};	/*Property = DataSet*/
const Value V17825_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_301 = {S22, 0, &V17825_301_57220, &P17825_300};	/*Property = DataUrl*/
const Value V17825_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_302 = {S23, 0, &V17825_302_57223, &P17825_301};	/*Property = DeviceOrientation*/
const Value V17825_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_304 = {S24, 0, &V17825_304_57229, &P17825_302};	/*Property = FileReader*/
const Value V17825_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_305 = {S25, 0, &V17825_305_57232, &P17825_304};	/*Property = FileSaver*/
const Value V17825_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_306 = {S26, 0, &V17825_306_57238, &P17825_305};	/*Property = FileWriter*/
const Value V17825_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_307 = {S27, 0, &V17825_307_57241, &P17825_306};	/*Property = FormData*/
const Value V17825_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_309 = {S28, 0, &V17825_309_57247, &P17825_307};	/*Property = Fullscreen*/
const Value V17825_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_310 = {S29, 0, &V17825_310_57250, &P17825_309};	/*Property = GeoLocation*/
const Value V17825_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_312 = {S30, 0, &V17825_312_57256, &P17825_310};	/*Property = History*/
const Value V17825_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_314 = {S31, 0, &V17825_314_57262, &P17825_312};	/*Property = Html5*/
const Value V17825_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_313 = {S32, 0, &V17825_313_57259, &P17825_314};	/*Property = Html-Media-Capture*/
const Value V17825_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_315 = {S33, 0, &V17825_315_57265, &P17825_313};	/*Property = Iframe*/
const Value V17825_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_316 = {S34, 0, &V17825_316_57268, &P17825_315};	/*Property = IndexedDB*/
const Value V17825_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_317 = {S35, 0, &V17825_317_57271, &P17825_316};	/*Property = Json*/
const Value V17825_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P17825_15 = {S37, 0, &V17825_15_54167, &P17825_317};	/*Property = LayoutEngine*/
const Value V17825_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_344 = {S38, 0, &V17825_344_57274, &P17825_15};	/*Property = Masking*/
const Value V17825_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_320 = {S39, 0, &V17825_320_57292, &P17825_344};	/*Property = PostMessage*/
const Value V17825_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_321 = {S40, 0, &V17825_321_57295, &P17825_320};	/*Property = Progress*/
const Value V17825_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_322 = {S41, 0, &V17825_322_57298, &P17825_321};	/*Property = Prompts*/
const Value V17825_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_323 = {S42, 0, &V17825_323_57301, &P17825_322};	/*Property = Selector*/
const Value V17825_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_324 = {S43, 0, &V17825_324_57307, &P17825_323};	/*Property = Svg*/
const Value V17825_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_327 = {S44, 0, &V17825_327_57316, &P17825_324};	/*Property = TouchEvents*/
const Value V17825_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_328 = {S45, 0, &V17825_328_57319, &P17825_327};	/*Property = Track*/
const Value V17825_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_329 = {S46, 0, &V17825_329_57325, &P17825_328};	/*Property = Video*/
const Value V17825_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_330 = {S47, 0, &V17825_330_57328, &P17825_329};	/*Property = Viewport*/
const Value V17825_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_332 = {S48, 0, &V17825_332_57340, &P17825_330};	/*Property = WebWorkers*/
const Value V17825_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17825_333 = {S49, 0, &V17825_333_57343, &P17825_332};	/*Property = Xhr2*/
const Profile P17825 = {"17825", &P17825_333};	/*Profile ID - 17825*/
const Profile P17843 = {"17843", NULL};	/*Profile ID - 17843*/
const Value V22770_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22770_279 = {S1, 0, &V22770_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22770_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22770_282 = {S3, 0, &V22770_282_57154, &P22770_279};	/*Property = BlobBuilder*/
const Value V22770_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22770_350 = {S4, 0, &V22770_350_57444, &P22770_282};	/*Property = Canvas*/
const Value V22770_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22770_283 = {S5, 0, &V22770_283_57159, &P22770_350};	/*Property = CssBackground*/
const Value V22770_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22770_284 = {S6, 0, &V22770_284_57163, &P22770_283};	/*Property = CssBorderImage*/
const Value V22770_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22770_285 = {S7, 0, &V22770_285_57166, &P22770_284};	/*Property = CssCanvas*/
const Value V22770_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22770_286 = {S8, 0, &V22770_286_57168, &P22770_285};	/*Property = CssColor*/
const Value V22770_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22770_237 = {S9, 0, &V22770_237_55786, &P22770_286};	/*Property = CssColumn*/
const Value V22770_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22770_287 = {S10, 0, &V22770_287_57172, &P22770_237};	/*Property = CssFlexbox*/
const Value V22770_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22770_288 = {S11, 0, &V22770_288_57174, &P22770_287};	/*Property = CssFont*/
const Value V22770_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22770_289 = {S12, 0, &V22770_289_57759, &P22770_288};	/*Property = CssImages*/
const Value V22770_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22770_291 = {S13, 0, &V22770_291_57180, &P22770_289};	/*Property = CssMediaQueries*/
const Value V22770_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22770_292 = {S14, 0, &V22770_292_57183, &P22770_291};	/*Property = CssMinMax*/
const Value V22770_293_57186 = {S2, NULL};	/*Value = True*/
const Property P22770_293 = {S15, 0, &V22770_293_57186, &P22770_292};	/*Property = CssOverflow*/
const Value V22770_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22770_294 = {S16, 0, &V22770_294_57189, &P22770_293};	/*Property = CssPosition*/
const Value V22770_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22770_295 = {S17, 0, &V22770_295_57192, &P22770_294};	/*Property = CssText*/
const Value V22770_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22770_297 = {S18, 0, &V22770_297_57195, &P22770_295};	/*Property = CssTransforms*/
const Value V22770_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22770_241 = {S19, 0, &V22770_241_55799, &P22770_297};	/*Property = CssTransitions*/
const Value V22770_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22770_298 = {S20, 0, &V22770_298_57206, &P22770_241};	/*Property = CssUI*/
const Value V22770_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22770_300 = {S21, 0, &V22770_300_57215, &P22770_298};	/*Property = DataSet*/
const Value V22770_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22770_301 = {S22, 0, &V22770_301_57218, &P22770_300};	/*Property = DataUrl*/
const Value V22770_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22770_302 = {S23, 0, &V22770_302_57221, &P22770_301};	/*Property = DeviceOrientation*/
const Value V22770_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22770_304 = {S24, 0, &V22770_304_57227, &P22770_302};	/*Property = FileReader*/
const Value V22770_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22770_305 = {S25, 0, &V22770_305_57231, &P22770_304};	/*Property = FileSaver*/
const Value V22770_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22770_306 = {S26, 0, &V22770_306_57237, &P22770_305};	/*Property = FileWriter*/
const Value V22770_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22770_307 = {S27, 0, &V22770_307_57239, &P22770_306};	/*Property = FormData*/
const Value V22770_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22770_309 = {S28, 0, &V22770_309_57246, &P22770_307};	/*Property = Fullscreen*/
const Value V22770_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22770_310 = {S29, 0, &V22770_310_57248, &P22770_309};	/*Property = GeoLocation*/
const Value V22770_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22770_312 = {S30, 0, &V22770_312_57255, &P22770_310};	/*Property = History*/
const Value V22770_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22770_314 = {S31, 0, &V22770_314_57260, &P22770_312};	/*Property = Html5*/
const Value V22770_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22770_313 = {S32, 0, &V22770_313_57258, &P22770_314};	/*Property = Html-Media-Capture*/
const Value V22770_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22770_315 = {S33, 0, &V22770_315_57264, &P22770_313};	/*Property = Iframe*/
const Value V22770_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22770_316 = {S34, 0, &V22770_316_57267, &P22770_315};	/*Property = IndexedDB*/
const Value V22770_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22770_317 = {S35, 0, &V22770_317_57269, &P22770_316};	/*Property = Json*/
const Value V22770_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P22770_15 = {S37, 0, &V22770_15_30, &P22770_317};	/*Property = LayoutEngine*/
const Value V22770_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22770_344 = {S38, 0, &V22770_344_57272, &P22770_15};	/*Property = Masking*/
const Value V22770_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22770_320 = {S39, 0, &V22770_320_57290, &P22770_344};	/*Property = PostMessage*/
const Value V22770_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22770_321 = {S40, 0, &V22770_321_57293, &P22770_320};	/*Property = Progress*/
const Value V22770_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22770_322 = {S41, 0, &V22770_322_57296, &P22770_321};	/*Property = Prompts*/
const Value V22770_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22770_323 = {S42, 0, &V22770_323_57299, &P22770_322};	/*Property = Selector*/
const Value V22770_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22770_324 = {S43, 0, &V22770_324_57305, &P22770_323};	/*Property = Svg*/
const Value V22770_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22770_327 = {S44, 0, &V22770_327_57314, &P22770_324};	/*Property = TouchEvents*/
const Value V22770_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22770_328 = {S45, 0, &V22770_328_57318, &P22770_327};	/*Property = Track*/
const Value V22770_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22770_329 = {S46, 0, &V22770_329_57323, &P22770_328};	/*Property = Video*/
const Value V22770_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22770_330 = {S47, 0, &V22770_330_57326, &P22770_329};	/*Property = Viewport*/
const Value V22770_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22770_332 = {S48, 0, &V22770_332_57339, &P22770_330};	/*Property = WebWorkers*/
const Value V22770_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22770_333 = {S49, 0, &V22770_333_57342, &P22770_332};	/*Property = Xhr2*/
const Profile P22770 = {"22770", &P22770_333};	/*Profile ID - 22770*/
const Profile P18156 = {"18156", NULL};	/*Profile ID - 18156*/
const Profile P21442 = {"21442", NULL};	/*Profile ID - 21442*/
const Value V22443_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22443_279 = {S1, 0, &V22443_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22443_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22443_282 = {S3, 0, &V22443_282_57153, &P22443_279};	/*Property = BlobBuilder*/
const Value V22443_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22443_350 = {S4, 0, &V22443_350_57444, &P22443_282};	/*Property = Canvas*/
const Value V22443_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22443_283 = {S5, 0, &V22443_283_57159, &P22443_350};	/*Property = CssBackground*/
const Value V22443_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22443_284 = {S6, 0, &V22443_284_57163, &P22443_283};	/*Property = CssBorderImage*/
const Value V22443_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22443_285 = {S7, 0, &V22443_285_57166, &P22443_284};	/*Property = CssCanvas*/
const Value V22443_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22443_286 = {S8, 0, &V22443_286_57168, &P22443_285};	/*Property = CssColor*/
const Value V22443_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22443_237 = {S9, 0, &V22443_237_55786, &P22443_286};	/*Property = CssColumn*/
const Value V22443_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22443_287 = {S10, 0, &V22443_287_57172, &P22443_237};	/*Property = CssFlexbox*/
const Value V22443_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22443_288 = {S11, 0, &V22443_288_57174, &P22443_287};	/*Property = CssFont*/
const Value V22443_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22443_289 = {S12, 0, &V22443_289_57759, &P22443_288};	/*Property = CssImages*/
const Value V22443_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22443_291 = {S13, 0, &V22443_291_57180, &P22443_289};	/*Property = CssMediaQueries*/
const Value V22443_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22443_292 = {S14, 0, &V22443_292_57183, &P22443_291};	/*Property = CssMinMax*/
const Value V22443_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22443_293 = {S15, 0, &V22443_293_57187, &P22443_292};	/*Property = CssOverflow*/
const Value V22443_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22443_294 = {S16, 0, &V22443_294_57190, &P22443_293};	/*Property = CssPosition*/
const Value V22443_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22443_295 = {S17, 0, &V22443_295_57192, &P22443_294};	/*Property = CssText*/
const Value V22443_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22443_297 = {S18, 0, &V22443_297_57195, &P22443_295};	/*Property = CssTransforms*/
const Value V22443_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22443_241 = {S19, 0, &V22443_241_55799, &P22443_297};	/*Property = CssTransitions*/
const Value V22443_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22443_298 = {S20, 0, &V22443_298_57206, &P22443_241};	/*Property = CssUI*/
const Value V22443_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22443_300 = {S21, 0, &V22443_300_57216, &P22443_298};	/*Property = DataSet*/
const Value V22443_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22443_301 = {S22, 0, &V22443_301_57218, &P22443_300};	/*Property = DataUrl*/
const Value V22443_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22443_302 = {S23, 0, &V22443_302_57221, &P22443_301};	/*Property = DeviceOrientation*/
const Value V22443_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22443_304 = {S24, 0, &V22443_304_57227, &P22443_302};	/*Property = FileReader*/
const Value V22443_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22443_305 = {S25, 0, &V22443_305_57231, &P22443_304};	/*Property = FileSaver*/
const Value V22443_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22443_306 = {S26, 0, &V22443_306_57237, &P22443_305};	/*Property = FileWriter*/
const Value V22443_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22443_307 = {S27, 0, &V22443_307_57239, &P22443_306};	/*Property = FormData*/
const Value V22443_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22443_309 = {S28, 0, &V22443_309_57246, &P22443_307};	/*Property = Fullscreen*/
const Value V22443_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22443_310 = {S29, 0, &V22443_310_57248, &P22443_309};	/*Property = GeoLocation*/
const Value V22443_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22443_312 = {S30, 0, &V22443_312_57255, &P22443_310};	/*Property = History*/
const Value V22443_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22443_314 = {S31, 0, &V22443_314_57261, &P22443_312};	/*Property = Html5*/
const Value V22443_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22443_313 = {S32, 0, &V22443_313_57258, &P22443_314};	/*Property = Html-Media-Capture*/
const Value V22443_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22443_315 = {S33, 0, &V22443_315_57264, &P22443_313};	/*Property = Iframe*/
const Value V22443_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22443_316 = {S34, 0, &V22443_316_57267, &P22443_315};	/*Property = IndexedDB*/
const Value V22443_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22443_317 = {S35, 0, &V22443_317_57269, &P22443_316};	/*Property = Json*/
const Value V22443_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22443_15 = {S37, 0, &V22443_15_28, &P22443_317};	/*Property = LayoutEngine*/
const Value V22443_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22443_344 = {S38, 0, &V22443_344_57272, &P22443_15};	/*Property = Masking*/
const Value V22443_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22443_320 = {S39, 0, &V22443_320_57290, &P22443_344};	/*Property = PostMessage*/
const Value V22443_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22443_321 = {S40, 0, &V22443_321_57293, &P22443_320};	/*Property = Progress*/
const Value V22443_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22443_322 = {S41, 0, &V22443_322_57296, &P22443_321};	/*Property = Prompts*/
const Value V22443_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22443_323 = {S42, 0, &V22443_323_57299, &P22443_322};	/*Property = Selector*/
const Value V22443_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22443_324 = {S43, 0, &V22443_324_57306, &P22443_323};	/*Property = Svg*/
const Value V22443_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22443_327 = {S44, 0, &V22443_327_57315, &P22443_324};	/*Property = TouchEvents*/
const Value V22443_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22443_328 = {S45, 0, &V22443_328_57318, &P22443_327};	/*Property = Track*/
const Value V22443_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22443_329 = {S46, 0, &V22443_329_57323, &P22443_328};	/*Property = Video*/
const Value V22443_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22443_330 = {S47, 0, &V22443_330_57326, &P22443_329};	/*Property = Viewport*/
const Value V22443_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22443_332 = {S48, 0, &V22443_332_57339, &P22443_330};	/*Property = WebWorkers*/
const Value V22443_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22443_333 = {S49, 0, &V22443_333_57342, &P22443_332};	/*Property = Xhr2*/
const Profile P22443 = {"22443", &P22443_333};	/*Profile ID - 22443*/
const Value V17334_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_279 = {S1, 0, &V17334_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17334_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_282 = {S3, 0, &V17334_282_57155, &P17334_279};	/*Property = BlobBuilder*/
const Value V17334_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_350 = {S4, 0, &V17334_350_57446, &P17334_282};	/*Property = Canvas*/
const Value V17334_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_283 = {S5, 0, &V17334_283_57161, &P17334_350};	/*Property = CssBackground*/
const Value V17334_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_284 = {S6, 0, &V17334_284_57164, &P17334_283};	/*Property = CssBorderImage*/
const Value V17334_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_285 = {S7, 0, &V17334_285_57167, &P17334_284};	/*Property = CssCanvas*/
const Value V17334_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_286 = {S8, 0, &V17334_286_57170, &P17334_285};	/*Property = CssColor*/
const Value V17334_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_237 = {S9, 0, &V17334_237_55786, &P17334_286};	/*Property = CssColumn*/
const Value V17334_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_287 = {S10, 0, &V17334_287_57173, &P17334_237};	/*Property = CssFlexbox*/
const Value V17334_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_288 = {S11, 0, &V17334_288_57176, &P17334_287};	/*Property = CssFont*/
const Value V17334_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_289 = {S12, 0, &V17334_289_86899, &P17334_288};	/*Property = CssImages*/
const Value V17334_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_291 = {S13, 0, &V17334_291_57182, &P17334_289};	/*Property = CssMediaQueries*/
const Value V17334_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_292 = {S14, 0, &V17334_292_57185, &P17334_291};	/*Property = CssMinMax*/
const Value V17334_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_293 = {S15, 0, &V17334_293_57188, &P17334_292};	/*Property = CssOverflow*/
const Value V17334_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_294 = {S16, 0, &V17334_294_57191, &P17334_293};	/*Property = CssPosition*/
const Value V17334_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_295 = {S17, 0, &V17334_295_57194, &P17334_294};	/*Property = CssText*/
const Value V17334_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_297 = {S18, 0, &V17334_297_57197, &P17334_295};	/*Property = CssTransforms*/
const Value V17334_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_241 = {S19, 0, &V17334_241_55801, &P17334_297};	/*Property = CssTransitions*/
const Value V17334_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_298 = {S20, 0, &V17334_298_57208, &P17334_241};	/*Property = CssUI*/
const Value V17334_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_300 = {S21, 0, &V17334_300_57217, &P17334_298};	/*Property = DataSet*/
const Value V17334_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_301 = {S22, 0, &V17334_301_57220, &P17334_300};	/*Property = DataUrl*/
const Value V17334_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_302 = {S23, 0, &V17334_302_57223, &P17334_301};	/*Property = DeviceOrientation*/
const Value V17334_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_304 = {S24, 0, &V17334_304_57229, &P17334_302};	/*Property = FileReader*/
const Value V17334_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_305 = {S25, 0, &V17334_305_57232, &P17334_304};	/*Property = FileSaver*/
const Value V17334_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_306 = {S26, 0, &V17334_306_57238, &P17334_305};	/*Property = FileWriter*/
const Value V17334_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_307 = {S27, 0, &V17334_307_57241, &P17334_306};	/*Property = FormData*/
const Value V17334_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_309 = {S28, 0, &V17334_309_57247, &P17334_307};	/*Property = Fullscreen*/
const Value V17334_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_310 = {S29, 0, &V17334_310_57250, &P17334_309};	/*Property = GeoLocation*/
const Value V17334_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_312 = {S30, 0, &V17334_312_57256, &P17334_310};	/*Property = History*/
const Value V17334_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_314 = {S31, 0, &V17334_314_57262, &P17334_312};	/*Property = Html5*/
const Value V17334_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_313 = {S32, 0, &V17334_313_57259, &P17334_314};	/*Property = Html-Media-Capture*/
const Value V17334_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_315 = {S33, 0, &V17334_315_57265, &P17334_313};	/*Property = Iframe*/
const Value V17334_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_316 = {S34, 0, &V17334_316_57268, &P17334_315};	/*Property = IndexedDB*/
const Value V17334_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_317 = {S35, 0, &V17334_317_57271, &P17334_316};	/*Property = Json*/
const Value V17334_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17334_15 = {S37, 0, &V17334_15_28, &P17334_317};	/*Property = LayoutEngine*/
const Value V17334_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_344 = {S38, 0, &V17334_344_57274, &P17334_15};	/*Property = Masking*/
const Value V17334_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_320 = {S39, 0, &V17334_320_57292, &P17334_344};	/*Property = PostMessage*/
const Value V17334_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_321 = {S40, 0, &V17334_321_57295, &P17334_320};	/*Property = Progress*/
const Value V17334_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_322 = {S41, 0, &V17334_322_57298, &P17334_321};	/*Property = Prompts*/
const Value V17334_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_323 = {S42, 0, &V17334_323_57301, &P17334_322};	/*Property = Selector*/
const Value V17334_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_324 = {S43, 0, &V17334_324_57307, &P17334_323};	/*Property = Svg*/
const Value V17334_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_327 = {S44, 0, &V17334_327_57316, &P17334_324};	/*Property = TouchEvents*/
const Value V17334_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_328 = {S45, 0, &V17334_328_57319, &P17334_327};	/*Property = Track*/
const Value V17334_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_329 = {S46, 0, &V17334_329_57325, &P17334_328};	/*Property = Video*/
const Value V17334_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_330 = {S47, 0, &V17334_330_57328, &P17334_329};	/*Property = Viewport*/
const Value V17334_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_332 = {S48, 0, &V17334_332_57340, &P17334_330};	/*Property = WebWorkers*/
const Value V17334_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17334_333 = {S49, 0, &V17334_333_57343, &P17334_332};	/*Property = Xhr2*/
const Profile P17334 = {"17334", &P17334_333};	/*Profile ID - 17334*/
const Value V4618_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_279 = {S1, 0, &V4618_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4618_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_282 = {S3, 0, &V4618_282_57155, &P4618_279};	/*Property = BlobBuilder*/
const Value V4618_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_350 = {S4, 0, &V4618_350_57446, &P4618_282};	/*Property = Canvas*/
const Value V4618_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_283 = {S5, 0, &V4618_283_57161, &P4618_350};	/*Property = CssBackground*/
const Value V4618_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_284 = {S6, 0, &V4618_284_57164, &P4618_283};	/*Property = CssBorderImage*/
const Value V4618_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_285 = {S7, 0, &V4618_285_57167, &P4618_284};	/*Property = CssCanvas*/
const Value V4618_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_286 = {S8, 0, &V4618_286_57170, &P4618_285};	/*Property = CssColor*/
const Value V4618_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_237 = {S9, 0, &V4618_237_55786, &P4618_286};	/*Property = CssColumn*/
const Value V4618_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_287 = {S10, 0, &V4618_287_57173, &P4618_237};	/*Property = CssFlexbox*/
const Value V4618_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_288 = {S11, 0, &V4618_288_57176, &P4618_287};	/*Property = CssFont*/
const Value V4618_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_289 = {S12, 0, &V4618_289_86899, &P4618_288};	/*Property = CssImages*/
const Value V4618_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_291 = {S13, 0, &V4618_291_57182, &P4618_289};	/*Property = CssMediaQueries*/
const Value V4618_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_292 = {S14, 0, &V4618_292_57185, &P4618_291};	/*Property = CssMinMax*/
const Value V4618_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_293 = {S15, 0, &V4618_293_57188, &P4618_292};	/*Property = CssOverflow*/
const Value V4618_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_294 = {S16, 0, &V4618_294_57191, &P4618_293};	/*Property = CssPosition*/
const Value V4618_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_295 = {S17, 0, &V4618_295_57194, &P4618_294};	/*Property = CssText*/
const Value V4618_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_297 = {S18, 0, &V4618_297_57197, &P4618_295};	/*Property = CssTransforms*/
const Value V4618_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_241 = {S19, 0, &V4618_241_55801, &P4618_297};	/*Property = CssTransitions*/
const Value V4618_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_298 = {S20, 0, &V4618_298_57208, &P4618_241};	/*Property = CssUI*/
const Value V4618_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_300 = {S21, 0, &V4618_300_57217, &P4618_298};	/*Property = DataSet*/
const Value V4618_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_301 = {S22, 0, &V4618_301_57220, &P4618_300};	/*Property = DataUrl*/
const Value V4618_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_302 = {S23, 0, &V4618_302_57223, &P4618_301};	/*Property = DeviceOrientation*/
const Value V4618_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_304 = {S24, 0, &V4618_304_57229, &P4618_302};	/*Property = FileReader*/
const Value V4618_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_305 = {S25, 0, &V4618_305_57232, &P4618_304};	/*Property = FileSaver*/
const Value V4618_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_306 = {S26, 0, &V4618_306_57238, &P4618_305};	/*Property = FileWriter*/
const Value V4618_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_307 = {S27, 0, &V4618_307_57241, &P4618_306};	/*Property = FormData*/
const Value V4618_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_309 = {S28, 0, &V4618_309_57247, &P4618_307};	/*Property = Fullscreen*/
const Value V4618_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_310 = {S29, 0, &V4618_310_57250, &P4618_309};	/*Property = GeoLocation*/
const Value V4618_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_312 = {S30, 0, &V4618_312_57256, &P4618_310};	/*Property = History*/
const Value V4618_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_314 = {S31, 0, &V4618_314_57262, &P4618_312};	/*Property = Html5*/
const Value V4618_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_313 = {S32, 0, &V4618_313_57259, &P4618_314};	/*Property = Html-Media-Capture*/
const Value V4618_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_315 = {S33, 0, &V4618_315_57265, &P4618_313};	/*Property = Iframe*/
const Value V4618_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_316 = {S34, 0, &V4618_316_57268, &P4618_315};	/*Property = IndexedDB*/
const Value V4618_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_317 = {S35, 0, &V4618_317_57271, &P4618_316};	/*Property = Json*/
const Value V4618_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_15 = {S37, 0, &V4618_15_72, &P4618_317};	/*Property = LayoutEngine*/
const Value V4618_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_344 = {S38, 0, &V4618_344_57274, &P4618_15};	/*Property = Masking*/
const Value V4618_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_320 = {S39, 0, &V4618_320_57292, &P4618_344};	/*Property = PostMessage*/
const Value V4618_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_321 = {S40, 0, &V4618_321_57295, &P4618_320};	/*Property = Progress*/
const Value V4618_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_322 = {S41, 0, &V4618_322_57298, &P4618_321};	/*Property = Prompts*/
const Value V4618_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_323 = {S42, 0, &V4618_323_57301, &P4618_322};	/*Property = Selector*/
const Value V4618_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_324 = {S43, 0, &V4618_324_57307, &P4618_323};	/*Property = Svg*/
const Value V4618_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_327 = {S44, 0, &V4618_327_57316, &P4618_324};	/*Property = TouchEvents*/
const Value V4618_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_328 = {S45, 0, &V4618_328_57319, &P4618_327};	/*Property = Track*/
const Value V4618_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_329 = {S46, 0, &V4618_329_57325, &P4618_328};	/*Property = Video*/
const Value V4618_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_330 = {S47, 0, &V4618_330_57328, &P4618_329};	/*Property = Viewport*/
const Value V4618_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_332 = {S48, 0, &V4618_332_57340, &P4618_330};	/*Property = WebWorkers*/
const Value V4618_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4618_333 = {S49, 0, &V4618_333_57343, &P4618_332};	/*Property = Xhr2*/
const Profile P4618 = {"4618", &P4618_333};	/*Profile ID - 4618*/
const Profile P18146 = {"18146", NULL};	/*Profile ID - 18146*/
const Value V22730_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22730_279 = {S1, 0, &V22730_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22730_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22730_282 = {S3, 0, &V22730_282_57154, &P22730_279};	/*Property = BlobBuilder*/
const Value V22730_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22730_350 = {S4, 0, &V22730_350_57444, &P22730_282};	/*Property = Canvas*/
const Value V22730_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22730_283 = {S5, 0, &V22730_283_57159, &P22730_350};	/*Property = CssBackground*/
const Value V22730_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22730_284 = {S6, 0, &V22730_284_57163, &P22730_283};	/*Property = CssBorderImage*/
const Value V22730_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22730_285 = {S7, 0, &V22730_285_57166, &P22730_284};	/*Property = CssCanvas*/
const Value V22730_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22730_286 = {S8, 0, &V22730_286_57168, &P22730_285};	/*Property = CssColor*/
const Value V22730_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22730_237 = {S9, 0, &V22730_237_55786, &P22730_286};	/*Property = CssColumn*/
const Value V22730_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22730_287 = {S10, 0, &V22730_287_57172, &P22730_237};	/*Property = CssFlexbox*/
const Value V22730_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22730_288 = {S11, 0, &V22730_288_57175, &P22730_287};	/*Property = CssFont*/
const Value V22730_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22730_289 = {S12, 0, &V22730_289_57759, &P22730_288};	/*Property = CssImages*/
const Value V22730_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22730_291 = {S13, 0, &V22730_291_57181, &P22730_289};	/*Property = CssMediaQueries*/
const Value V22730_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22730_292 = {S14, 0, &V22730_292_57183, &P22730_291};	/*Property = CssMinMax*/
const Value V22730_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22730_293 = {S15, 0, &V22730_293_57187, &P22730_292};	/*Property = CssOverflow*/
const Value V22730_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22730_294 = {S16, 0, &V22730_294_57190, &P22730_293};	/*Property = CssPosition*/
const Value V22730_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22730_295 = {S17, 0, &V22730_295_57193, &P22730_294};	/*Property = CssText*/
const Value V22730_297_57196 = {S0, NULL};	/*Value = False*/
const Property P22730_297 = {S18, 0, &V22730_297_57196, &P22730_295};	/*Property = CssTransforms*/
const Value V22730_241_55800 = {S0, NULL};	/*Value = False*/
const Property P22730_241 = {S19, 0, &V22730_241_55800, &P22730_297};	/*Property = CssTransitions*/
const Value V22730_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22730_298 = {S20, 0, &V22730_298_57206, &P22730_241};	/*Property = CssUI*/
const Value V22730_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22730_300 = {S21, 0, &V22730_300_57216, &P22730_298};	/*Property = DataSet*/
const Value V22730_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22730_301 = {S22, 0, &V22730_301_57218, &P22730_300};	/*Property = DataUrl*/
const Value V22730_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22730_302 = {S23, 0, &V22730_302_57222, &P22730_301};	/*Property = DeviceOrientation*/
const Value V22730_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22730_304 = {S24, 0, &V22730_304_57228, &P22730_302};	/*Property = FileReader*/
const Value V22730_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22730_305 = {S25, 0, &V22730_305_57231, &P22730_304};	/*Property = FileSaver*/
const Value V22730_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22730_306 = {S26, 0, &V22730_306_57237, &P22730_305};	/*Property = FileWriter*/
const Value V22730_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22730_307 = {S27, 0, &V22730_307_57240, &P22730_306};	/*Property = FormData*/
const Value V22730_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22730_309 = {S28, 0, &V22730_309_57246, &P22730_307};	/*Property = Fullscreen*/
const Value V22730_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22730_310 = {S29, 0, &V22730_310_57248, &P22730_309};	/*Property = GeoLocation*/
const Value V22730_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22730_312 = {S30, 0, &V22730_312_57255, &P22730_310};	/*Property = History*/
const Value V22730_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22730_314 = {S31, 0, &V22730_314_57261, &P22730_312};	/*Property = Html5*/
const Value V22730_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22730_313 = {S32, 0, &V22730_313_57258, &P22730_314};	/*Property = Html-Media-Capture*/
const Value V22730_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22730_315 = {S33, 0, &V22730_315_57264, &P22730_313};	/*Property = Iframe*/
const Value V22730_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22730_316 = {S34, 0, &V22730_316_57267, &P22730_315};	/*Property = IndexedDB*/
const Value V22730_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22730_317 = {S35, 0, &V22730_317_57269, &P22730_316};	/*Property = Json*/
const Value V22730_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P22730_15 = {S37, 0, &V22730_15_30, &P22730_317};	/*Property = LayoutEngine*/
const Value V22730_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22730_344 = {S38, 0, &V22730_344_57272, &P22730_15};	/*Property = Masking*/
const Value V22730_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22730_320 = {S39, 0, &V22730_320_57290, &P22730_344};	/*Property = PostMessage*/
const Value V22730_321_57294 = {S0, NULL};	/*Value = False*/
const Property P22730_321 = {S40, 0, &V22730_321_57294, &P22730_320};	/*Property = Progress*/
const Value V22730_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22730_322 = {S41, 0, &V22730_322_57296, &P22730_321};	/*Property = Prompts*/
const Value V22730_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22730_323 = {S42, 0, &V22730_323_57299, &P22730_322};	/*Property = Selector*/
const Value V22730_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22730_324 = {S43, 0, &V22730_324_57306, &P22730_323};	/*Property = Svg*/
const Value V22730_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22730_327 = {S44, 0, &V22730_327_57315, &P22730_324};	/*Property = TouchEvents*/
const Value V22730_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22730_328 = {S45, 0, &V22730_328_57318, &P22730_327};	/*Property = Track*/
const Value V22730_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22730_329 = {S46, 0, &V22730_329_57323, &P22730_328};	/*Property = Video*/
const Value V22730_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22730_330 = {S47, 0, &V22730_330_57326, &P22730_329};	/*Property = Viewport*/
const Value V22730_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22730_332 = {S48, 0, &V22730_332_57339, &P22730_330};	/*Property = WebWorkers*/
const Value V22730_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22730_333 = {S49, 0, &V22730_333_57342, &P22730_332};	/*Property = Xhr2*/
const Profile P22730 = {"22730", &P22730_333};	/*Profile ID - 22730*/
const Profile P18093 = {"18093", NULL};	/*Profile ID - 18093*/
const Value V22441_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22441_279 = {S1, 0, &V22441_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22441_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22441_282 = {S3, 0, &V22441_282_57154, &P22441_279};	/*Property = BlobBuilder*/
const Value V22441_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22441_350 = {S4, 0, &V22441_350_57444, &P22441_282};	/*Property = Canvas*/
const Value V22441_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22441_283 = {S5, 0, &V22441_283_57159, &P22441_350};	/*Property = CssBackground*/
const Value V22441_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22441_284 = {S6, 0, &V22441_284_57163, &P22441_283};	/*Property = CssBorderImage*/
const Value V22441_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22441_285 = {S7, 0, &V22441_285_57166, &P22441_284};	/*Property = CssCanvas*/
const Value V22441_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22441_286 = {S8, 0, &V22441_286_57168, &P22441_285};	/*Property = CssColor*/
const Value V22441_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22441_237 = {S9, 0, &V22441_237_55786, &P22441_286};	/*Property = CssColumn*/
const Value V22441_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22441_287 = {S10, 0, &V22441_287_57172, &P22441_237};	/*Property = CssFlexbox*/
const Value V22441_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22441_288 = {S11, 0, &V22441_288_57175, &P22441_287};	/*Property = CssFont*/
const Value V22441_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22441_289 = {S12, 0, &V22441_289_57759, &P22441_288};	/*Property = CssImages*/
const Value V22441_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22441_291 = {S13, 0, &V22441_291_57181, &P22441_289};	/*Property = CssMediaQueries*/
const Value V22441_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22441_292 = {S14, 0, &V22441_292_57183, &P22441_291};	/*Property = CssMinMax*/
const Value V22441_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22441_293 = {S15, 0, &V22441_293_57187, &P22441_292};	/*Property = CssOverflow*/
const Value V22441_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22441_294 = {S16, 0, &V22441_294_57190, &P22441_293};	/*Property = CssPosition*/
const Value V22441_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22441_295 = {S17, 0, &V22441_295_57192, &P22441_294};	/*Property = CssText*/
const Value V22441_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22441_297 = {S18, 0, &V22441_297_57195, &P22441_295};	/*Property = CssTransforms*/
const Value V22441_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22441_241 = {S19, 0, &V22441_241_55799, &P22441_297};	/*Property = CssTransitions*/
const Value V22441_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22441_298 = {S20, 0, &V22441_298_57206, &P22441_241};	/*Property = CssUI*/
const Value V22441_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22441_300 = {S21, 0, &V22441_300_57216, &P22441_298};	/*Property = DataSet*/
const Value V22441_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22441_301 = {S22, 0, &V22441_301_57218, &P22441_300};	/*Property = DataUrl*/
const Value V22441_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22441_302 = {S23, 0, &V22441_302_57222, &P22441_301};	/*Property = DeviceOrientation*/
const Value V22441_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22441_304 = {S24, 0, &V22441_304_57228, &P22441_302};	/*Property = FileReader*/
const Value V22441_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22441_305 = {S25, 0, &V22441_305_57231, &P22441_304};	/*Property = FileSaver*/
const Value V22441_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22441_306 = {S26, 0, &V22441_306_57237, &P22441_305};	/*Property = FileWriter*/
const Value V22441_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22441_307 = {S27, 0, &V22441_307_57240, &P22441_306};	/*Property = FormData*/
const Value V22441_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22441_309 = {S28, 0, &V22441_309_57246, &P22441_307};	/*Property = Fullscreen*/
const Value V22441_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22441_310 = {S29, 0, &V22441_310_57248, &P22441_309};	/*Property = GeoLocation*/
const Value V22441_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22441_312 = {S30, 0, &V22441_312_57255, &P22441_310};	/*Property = History*/
const Value V22441_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22441_314 = {S31, 0, &V22441_314_57261, &P22441_312};	/*Property = Html5*/
const Value V22441_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22441_313 = {S32, 0, &V22441_313_57258, &P22441_314};	/*Property = Html-Media-Capture*/
const Value V22441_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22441_315 = {S33, 0, &V22441_315_57264, &P22441_313};	/*Property = Iframe*/
const Value V22441_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22441_316 = {S34, 0, &V22441_316_57267, &P22441_315};	/*Property = IndexedDB*/
const Value V22441_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22441_317 = {S35, 0, &V22441_317_57269, &P22441_316};	/*Property = Json*/
const Value V22441_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22441_15 = {S37, 0, &V22441_15_28, &P22441_317};	/*Property = LayoutEngine*/
const Value V22441_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22441_344 = {S38, 0, &V22441_344_57272, &P22441_15};	/*Property = Masking*/
const Value V22441_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22441_320 = {S39, 0, &V22441_320_57290, &P22441_344};	/*Property = PostMessage*/
const Value V22441_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22441_321 = {S40, 0, &V22441_321_57293, &P22441_320};	/*Property = Progress*/
const Value V22441_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22441_322 = {S41, 0, &V22441_322_57296, &P22441_321};	/*Property = Prompts*/
const Value V22441_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22441_323 = {S42, 0, &V22441_323_57299, &P22441_322};	/*Property = Selector*/
const Value V22441_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22441_324 = {S43, 0, &V22441_324_57306, &P22441_323};	/*Property = Svg*/
const Value V22441_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22441_327 = {S44, 0, &V22441_327_57315, &P22441_324};	/*Property = TouchEvents*/
const Value V22441_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22441_328 = {S45, 0, &V22441_328_57318, &P22441_327};	/*Property = Track*/
const Value V22441_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22441_329 = {S46, 0, &V22441_329_57323, &P22441_328};	/*Property = Video*/
const Value V22441_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22441_330 = {S47, 0, &V22441_330_57326, &P22441_329};	/*Property = Viewport*/
const Value V22441_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22441_332 = {S48, 0, &V22441_332_57339, &P22441_330};	/*Property = WebWorkers*/
const Value V22441_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22441_333 = {S49, 0, &V22441_333_57342, &P22441_332};	/*Property = Xhr2*/
const Profile P22441 = {"22441", &P22441_333};	/*Profile ID - 22441*/
const Value V18068_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18068_279 = {S1, 0, &V18068_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18068_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18068_282 = {S3, 0, &V18068_282_57154, &P18068_279};	/*Property = BlobBuilder*/
const Value V18068_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18068_350 = {S4, 0, &V18068_350_57444, &P18068_282};	/*Property = Canvas*/
const Value V18068_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18068_283 = {S5, 0, &V18068_283_57159, &P18068_350};	/*Property = CssBackground*/
const Value V18068_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18068_284 = {S6, 0, &V18068_284_57163, &P18068_283};	/*Property = CssBorderImage*/
const Value V18068_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18068_285 = {S7, 0, &V18068_285_57166, &P18068_284};	/*Property = CssCanvas*/
const Value V18068_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18068_286 = {S8, 0, &V18068_286_57168, &P18068_285};	/*Property = CssColor*/
const Value V18068_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18068_237 = {S9, 0, &V18068_237_55786, &P18068_286};	/*Property = CssColumn*/
const Value V18068_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18068_287 = {S10, 0, &V18068_287_57172, &P18068_237};	/*Property = CssFlexbox*/
const Value V18068_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18068_288 = {S11, 0, &V18068_288_57175, &P18068_287};	/*Property = CssFont*/
const Value V18068_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18068_289 = {S12, 0, &V18068_289_57759, &P18068_288};	/*Property = CssImages*/
const Value V18068_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18068_291 = {S13, 0, &V18068_291_57181, &P18068_289};	/*Property = CssMediaQueries*/
const Value V18068_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18068_292 = {S14, 0, &V18068_292_57183, &P18068_291};	/*Property = CssMinMax*/
const Value V18068_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18068_293 = {S15, 0, &V18068_293_57187, &P18068_292};	/*Property = CssOverflow*/
const Value V18068_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18068_294 = {S16, 0, &V18068_294_57190, &P18068_293};	/*Property = CssPosition*/
const Value V18068_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18068_295 = {S17, 0, &V18068_295_57193, &P18068_294};	/*Property = CssText*/
const Value V18068_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18068_297 = {S18, 0, &V18068_297_57195, &P18068_295};	/*Property = CssTransforms*/
const Value V18068_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18068_241 = {S19, 0, &V18068_241_55799, &P18068_297};	/*Property = CssTransitions*/
const Value V18068_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18068_298 = {S20, 0, &V18068_298_57206, &P18068_241};	/*Property = CssUI*/
const Value V18068_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18068_300 = {S21, 0, &V18068_300_57216, &P18068_298};	/*Property = DataSet*/
const Value V18068_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18068_301 = {S22, 0, &V18068_301_57218, &P18068_300};	/*Property = DataUrl*/
const Value V18068_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18068_302 = {S23, 0, &V18068_302_57222, &P18068_301};	/*Property = DeviceOrientation*/
const Value V18068_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18068_304 = {S24, 0, &V18068_304_57228, &P18068_302};	/*Property = FileReader*/
const Value V18068_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18068_305 = {S25, 0, &V18068_305_57231, &P18068_304};	/*Property = FileSaver*/
const Value V18068_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18068_306 = {S26, 0, &V18068_306_57237, &P18068_305};	/*Property = FileWriter*/
const Value V18068_307_57240 = {S0, NULL};	/*Value = False*/
const Property P18068_307 = {S27, 0, &V18068_307_57240, &P18068_306};	/*Property = FormData*/
const Value V18068_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18068_309 = {S28, 0, &V18068_309_57246, &P18068_307};	/*Property = Fullscreen*/
const Value V18068_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18068_310 = {S29, 0, &V18068_310_57248, &P18068_309};	/*Property = GeoLocation*/
const Value V18068_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18068_312 = {S30, 0, &V18068_312_57255, &P18068_310};	/*Property = History*/
const Value V18068_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18068_314 = {S31, 0, &V18068_314_57261, &P18068_312};	/*Property = Html5*/
const Value V18068_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18068_313 = {S32, 0, &V18068_313_57258, &P18068_314};	/*Property = Html-Media-Capture*/
const Value V18068_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18068_315 = {S33, 0, &V18068_315_57264, &P18068_313};	/*Property = Iframe*/
const Value V18068_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18068_316 = {S34, 0, &V18068_316_57267, &P18068_315};	/*Property = IndexedDB*/
const Value V18068_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18068_317 = {S35, 0, &V18068_317_57269, &P18068_316};	/*Property = Json*/
const Value V18068_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P18068_15 = {S37, 0, &V18068_15_72, &P18068_317};	/*Property = LayoutEngine*/
const Value V18068_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18068_344 = {S38, 0, &V18068_344_57272, &P18068_15};	/*Property = Masking*/
const Value V18068_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18068_320 = {S39, 0, &V18068_320_57290, &P18068_344};	/*Property = PostMessage*/
const Value V18068_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18068_321 = {S40, 0, &V18068_321_57293, &P18068_320};	/*Property = Progress*/
const Value V18068_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18068_322 = {S41, 0, &V18068_322_57296, &P18068_321};	/*Property = Prompts*/
const Value V18068_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18068_323 = {S42, 0, &V18068_323_57299, &P18068_322};	/*Property = Selector*/
const Value V18068_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18068_324 = {S43, 0, &V18068_324_57306, &P18068_323};	/*Property = Svg*/
const Value V18068_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18068_327 = {S44, 0, &V18068_327_57315, &P18068_324};	/*Property = TouchEvents*/
const Value V18068_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18068_328 = {S45, 0, &V18068_328_57318, &P18068_327};	/*Property = Track*/
const Value V18068_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18068_329 = {S46, 0, &V18068_329_57323, &P18068_328};	/*Property = Video*/
const Value V18068_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18068_330 = {S47, 0, &V18068_330_57326, &P18068_329};	/*Property = Viewport*/
const Value V18068_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18068_332 = {S48, 0, &V18068_332_57339, &P18068_330};	/*Property = WebWorkers*/
const Value V18068_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18068_333 = {S49, 0, &V18068_333_57342, &P18068_332};	/*Property = Xhr2*/
const Profile P18068 = {"18068", &P18068_333};	/*Profile ID - 18068*/
const Value V17384_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_279 = {S1, 0, &V17384_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17384_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_282 = {S3, 0, &V17384_282_57155, &P17384_279};	/*Property = BlobBuilder*/
const Value V17384_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_350 = {S4, 0, &V17384_350_57446, &P17384_282};	/*Property = Canvas*/
const Value V17384_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_283 = {S5, 0, &V17384_283_57161, &P17384_350};	/*Property = CssBackground*/
const Value V17384_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_284 = {S6, 0, &V17384_284_57164, &P17384_283};	/*Property = CssBorderImage*/
const Value V17384_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_285 = {S7, 0, &V17384_285_57167, &P17384_284};	/*Property = CssCanvas*/
const Value V17384_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_286 = {S8, 0, &V17384_286_57170, &P17384_285};	/*Property = CssColor*/
const Value V17384_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_237 = {S9, 0, &V17384_237_55786, &P17384_286};	/*Property = CssColumn*/
const Value V17384_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_287 = {S10, 0, &V17384_287_57173, &P17384_237};	/*Property = CssFlexbox*/
const Value V17384_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_288 = {S11, 0, &V17384_288_57176, &P17384_287};	/*Property = CssFont*/
const Value V17384_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_289 = {S12, 0, &V17384_289_86899, &P17384_288};	/*Property = CssImages*/
const Value V17384_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_291 = {S13, 0, &V17384_291_57182, &P17384_289};	/*Property = CssMediaQueries*/
const Value V17384_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_292 = {S14, 0, &V17384_292_57185, &P17384_291};	/*Property = CssMinMax*/
const Value V17384_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_293 = {S15, 0, &V17384_293_57188, &P17384_292};	/*Property = CssOverflow*/
const Value V17384_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_294 = {S16, 0, &V17384_294_57191, &P17384_293};	/*Property = CssPosition*/
const Value V17384_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_295 = {S17, 0, &V17384_295_57194, &P17384_294};	/*Property = CssText*/
const Value V17384_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_297 = {S18, 0, &V17384_297_57197, &P17384_295};	/*Property = CssTransforms*/
const Value V17384_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_241 = {S19, 0, &V17384_241_55801, &P17384_297};	/*Property = CssTransitions*/
const Value V17384_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_298 = {S20, 0, &V17384_298_57208, &P17384_241};	/*Property = CssUI*/
const Value V17384_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_300 = {S21, 0, &V17384_300_57217, &P17384_298};	/*Property = DataSet*/
const Value V17384_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_301 = {S22, 0, &V17384_301_57220, &P17384_300};	/*Property = DataUrl*/
const Value V17384_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_302 = {S23, 0, &V17384_302_57223, &P17384_301};	/*Property = DeviceOrientation*/
const Value V17384_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_304 = {S24, 0, &V17384_304_57229, &P17384_302};	/*Property = FileReader*/
const Value V17384_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_305 = {S25, 0, &V17384_305_57232, &P17384_304};	/*Property = FileSaver*/
const Value V17384_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_306 = {S26, 0, &V17384_306_57238, &P17384_305};	/*Property = FileWriter*/
const Value V17384_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_307 = {S27, 0, &V17384_307_57241, &P17384_306};	/*Property = FormData*/
const Value V17384_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_309 = {S28, 0, &V17384_309_57247, &P17384_307};	/*Property = Fullscreen*/
const Value V17384_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_310 = {S29, 0, &V17384_310_57250, &P17384_309};	/*Property = GeoLocation*/
const Value V17384_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_312 = {S30, 0, &V17384_312_57256, &P17384_310};	/*Property = History*/
const Value V17384_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_314 = {S31, 0, &V17384_314_57262, &P17384_312};	/*Property = Html5*/
const Value V17384_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_313 = {S32, 0, &V17384_313_57259, &P17384_314};	/*Property = Html-Media-Capture*/
const Value V17384_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_315 = {S33, 0, &V17384_315_57265, &P17384_313};	/*Property = Iframe*/
const Value V17384_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_316 = {S34, 0, &V17384_316_57268, &P17384_315};	/*Property = IndexedDB*/
const Value V17384_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_317 = {S35, 0, &V17384_317_57271, &P17384_316};	/*Property = Json*/
const Value V17384_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P17384_15 = {S37, 0, &V17384_15_30, &P17384_317};	/*Property = LayoutEngine*/
const Value V17384_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_344 = {S38, 0, &V17384_344_57274, &P17384_15};	/*Property = Masking*/
const Value V17384_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_320 = {S39, 0, &V17384_320_57292, &P17384_344};	/*Property = PostMessage*/
const Value V17384_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_321 = {S40, 0, &V17384_321_57295, &P17384_320};	/*Property = Progress*/
const Value V17384_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_322 = {S41, 0, &V17384_322_57298, &P17384_321};	/*Property = Prompts*/
const Value V17384_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_323 = {S42, 0, &V17384_323_57301, &P17384_322};	/*Property = Selector*/
const Value V17384_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_324 = {S43, 0, &V17384_324_57307, &P17384_323};	/*Property = Svg*/
const Value V17384_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_327 = {S44, 0, &V17384_327_57316, &P17384_324};	/*Property = TouchEvents*/
const Value V17384_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_328 = {S45, 0, &V17384_328_57319, &P17384_327};	/*Property = Track*/
const Value V17384_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_329 = {S46, 0, &V17384_329_57325, &P17384_328};	/*Property = Video*/
const Value V17384_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_330 = {S47, 0, &V17384_330_57328, &P17384_329};	/*Property = Viewport*/
const Value V17384_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_332 = {S48, 0, &V17384_332_57340, &P17384_330};	/*Property = WebWorkers*/
const Value V17384_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17384_333 = {S49, 0, &V17384_333_57343, &P17384_332};	/*Property = Xhr2*/
const Profile P17384 = {"17384", &P17384_333};	/*Profile ID - 17384*/
const Value V17812_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_279 = {S1, 0, &V17812_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17812_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_282 = {S3, 0, &V17812_282_57155, &P17812_279};	/*Property = BlobBuilder*/
const Value V17812_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_350 = {S4, 0, &V17812_350_57446, &P17812_282};	/*Property = Canvas*/
const Value V17812_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_283 = {S5, 0, &V17812_283_57161, &P17812_350};	/*Property = CssBackground*/
const Value V17812_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_284 = {S6, 0, &V17812_284_57164, &P17812_283};	/*Property = CssBorderImage*/
const Value V17812_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_285 = {S7, 0, &V17812_285_57167, &P17812_284};	/*Property = CssCanvas*/
const Value V17812_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_286 = {S8, 0, &V17812_286_57170, &P17812_285};	/*Property = CssColor*/
const Value V17812_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_237 = {S9, 0, &V17812_237_55786, &P17812_286};	/*Property = CssColumn*/
const Value V17812_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_287 = {S10, 0, &V17812_287_57173, &P17812_237};	/*Property = CssFlexbox*/
const Value V17812_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_288 = {S11, 0, &V17812_288_57176, &P17812_287};	/*Property = CssFont*/
const Value V17812_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_289 = {S12, 0, &V17812_289_86899, &P17812_288};	/*Property = CssImages*/
const Value V17812_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_291 = {S13, 0, &V17812_291_57182, &P17812_289};	/*Property = CssMediaQueries*/
const Value V17812_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_292 = {S14, 0, &V17812_292_57185, &P17812_291};	/*Property = CssMinMax*/
const Value V17812_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_293 = {S15, 0, &V17812_293_57188, &P17812_292};	/*Property = CssOverflow*/
const Value V17812_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_294 = {S16, 0, &V17812_294_57191, &P17812_293};	/*Property = CssPosition*/
const Value V17812_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_295 = {S17, 0, &V17812_295_57194, &P17812_294};	/*Property = CssText*/
const Value V17812_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_297 = {S18, 0, &V17812_297_57197, &P17812_295};	/*Property = CssTransforms*/
const Value V17812_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_241 = {S19, 0, &V17812_241_55801, &P17812_297};	/*Property = CssTransitions*/
const Value V17812_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_298 = {S20, 0, &V17812_298_57208, &P17812_241};	/*Property = CssUI*/
const Value V17812_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_300 = {S21, 0, &V17812_300_57217, &P17812_298};	/*Property = DataSet*/
const Value V17812_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_301 = {S22, 0, &V17812_301_57220, &P17812_300};	/*Property = DataUrl*/
const Value V17812_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_302 = {S23, 0, &V17812_302_57223, &P17812_301};	/*Property = DeviceOrientation*/
const Value V17812_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_304 = {S24, 0, &V17812_304_57229, &P17812_302};	/*Property = FileReader*/
const Value V17812_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_305 = {S25, 0, &V17812_305_57232, &P17812_304};	/*Property = FileSaver*/
const Value V17812_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_306 = {S26, 0, &V17812_306_57238, &P17812_305};	/*Property = FileWriter*/
const Value V17812_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_307 = {S27, 0, &V17812_307_57241, &P17812_306};	/*Property = FormData*/
const Value V17812_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_309 = {S28, 0, &V17812_309_57247, &P17812_307};	/*Property = Fullscreen*/
const Value V17812_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_310 = {S29, 0, &V17812_310_57250, &P17812_309};	/*Property = GeoLocation*/
const Value V17812_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_312 = {S30, 0, &V17812_312_57256, &P17812_310};	/*Property = History*/
const Value V17812_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_314 = {S31, 0, &V17812_314_57262, &P17812_312};	/*Property = Html5*/
const Value V17812_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_313 = {S32, 0, &V17812_313_57259, &P17812_314};	/*Property = Html-Media-Capture*/
const Value V17812_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_315 = {S33, 0, &V17812_315_57265, &P17812_313};	/*Property = Iframe*/
const Value V17812_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_316 = {S34, 0, &V17812_316_57268, &P17812_315};	/*Property = IndexedDB*/
const Value V17812_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_317 = {S35, 0, &V17812_317_57271, &P17812_316};	/*Property = Json*/
const Value V17812_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P17812_15 = {S37, 0, &V17812_15_54167, &P17812_317};	/*Property = LayoutEngine*/
const Value V17812_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_344 = {S38, 0, &V17812_344_57274, &P17812_15};	/*Property = Masking*/
const Value V17812_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_320 = {S39, 0, &V17812_320_57292, &P17812_344};	/*Property = PostMessage*/
const Value V17812_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_321 = {S40, 0, &V17812_321_57295, &P17812_320};	/*Property = Progress*/
const Value V17812_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_322 = {S41, 0, &V17812_322_57298, &P17812_321};	/*Property = Prompts*/
const Value V17812_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_323 = {S42, 0, &V17812_323_57301, &P17812_322};	/*Property = Selector*/
const Value V17812_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_324 = {S43, 0, &V17812_324_57307, &P17812_323};	/*Property = Svg*/
const Value V17812_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_327 = {S44, 0, &V17812_327_57316, &P17812_324};	/*Property = TouchEvents*/
const Value V17812_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_328 = {S45, 0, &V17812_328_57319, &P17812_327};	/*Property = Track*/
const Value V17812_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_329 = {S46, 0, &V17812_329_57325, &P17812_328};	/*Property = Video*/
const Value V17812_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_330 = {S47, 0, &V17812_330_57328, &P17812_329};	/*Property = Viewport*/
const Value V17812_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_332 = {S48, 0, &V17812_332_57340, &P17812_330};	/*Property = WebWorkers*/
const Value V17812_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17812_333 = {S49, 0, &V17812_333_57343, &P17812_332};	/*Property = Xhr2*/
const Profile P17812 = {"17812", &P17812_333};	/*Profile ID - 17812*/
const Profile P18090 = {"18090", NULL};	/*Profile ID - 18090*/
const Value V22728_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22728_279 = {S1, 0, &V22728_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22728_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22728_282 = {S3, 0, &V22728_282_57154, &P22728_279};	/*Property = BlobBuilder*/
const Value V22728_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22728_350 = {S4, 0, &V22728_350_57444, &P22728_282};	/*Property = Canvas*/
const Value V22728_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22728_283 = {S5, 0, &V22728_283_57159, &P22728_350};	/*Property = CssBackground*/
const Value V22728_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22728_284 = {S6, 0, &V22728_284_57163, &P22728_283};	/*Property = CssBorderImage*/
const Value V22728_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22728_285 = {S7, 0, &V22728_285_57166, &P22728_284};	/*Property = CssCanvas*/
const Value V22728_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22728_286 = {S8, 0, &V22728_286_57168, &P22728_285};	/*Property = CssColor*/
const Value V22728_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22728_237 = {S9, 0, &V22728_237_55786, &P22728_286};	/*Property = CssColumn*/
const Value V22728_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22728_287 = {S10, 0, &V22728_287_57172, &P22728_237};	/*Property = CssFlexbox*/
const Value V22728_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22728_288 = {S11, 0, &V22728_288_57174, &P22728_287};	/*Property = CssFont*/
const Value V22728_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22728_289 = {S12, 0, &V22728_289_57759, &P22728_288};	/*Property = CssImages*/
const Value V22728_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22728_291 = {S13, 0, &V22728_291_57180, &P22728_289};	/*Property = CssMediaQueries*/
const Value V22728_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22728_292 = {S14, 0, &V22728_292_57183, &P22728_291};	/*Property = CssMinMax*/
const Value V22728_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22728_293 = {S15, 0, &V22728_293_57187, &P22728_292};	/*Property = CssOverflow*/
const Value V22728_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22728_294 = {S16, 0, &V22728_294_57189, &P22728_293};	/*Property = CssPosition*/
const Value V22728_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22728_295 = {S17, 0, &V22728_295_57192, &P22728_294};	/*Property = CssText*/
const Value V22728_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22728_297 = {S18, 0, &V22728_297_57195, &P22728_295};	/*Property = CssTransforms*/
const Value V22728_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22728_241 = {S19, 0, &V22728_241_55799, &P22728_297};	/*Property = CssTransitions*/
const Value V22728_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22728_298 = {S20, 0, &V22728_298_57206, &P22728_241};	/*Property = CssUI*/
const Value V22728_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22728_300 = {S21, 0, &V22728_300_57216, &P22728_298};	/*Property = DataSet*/
const Value V22728_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22728_301 = {S22, 0, &V22728_301_57218, &P22728_300};	/*Property = DataUrl*/
const Value V22728_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22728_302 = {S23, 0, &V22728_302_57222, &P22728_301};	/*Property = DeviceOrientation*/
const Value V22728_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22728_304 = {S24, 0, &V22728_304_57227, &P22728_302};	/*Property = FileReader*/
const Value V22728_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22728_305 = {S25, 0, &V22728_305_57231, &P22728_304};	/*Property = FileSaver*/
const Value V22728_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22728_306 = {S26, 0, &V22728_306_57237, &P22728_305};	/*Property = FileWriter*/
const Value V22728_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22728_307 = {S27, 0, &V22728_307_57239, &P22728_306};	/*Property = FormData*/
const Value V22728_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22728_309 = {S28, 0, &V22728_309_57246, &P22728_307};	/*Property = Fullscreen*/
const Value V22728_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22728_310 = {S29, 0, &V22728_310_57248, &P22728_309};	/*Property = GeoLocation*/
const Value V22728_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22728_312 = {S30, 0, &V22728_312_57254, &P22728_310};	/*Property = History*/
const Value V22728_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22728_314 = {S31, 0, &V22728_314_57261, &P22728_312};	/*Property = Html5*/
const Value V22728_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22728_313 = {S32, 0, &V22728_313_57258, &P22728_314};	/*Property = Html-Media-Capture*/
const Value V22728_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22728_315 = {S33, 0, &V22728_315_57264, &P22728_313};	/*Property = Iframe*/
const Value V22728_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22728_316 = {S34, 0, &V22728_316_57266, &P22728_315};	/*Property = IndexedDB*/
const Value V22728_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22728_317 = {S35, 0, &V22728_317_57269, &P22728_316};	/*Property = Json*/
const Value V22728_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P22728_15 = {S37, 0, &V22728_15_30, &P22728_317};	/*Property = LayoutEngine*/
const Value V22728_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22728_344 = {S38, 0, &V22728_344_57272, &P22728_15};	/*Property = Masking*/
const Value V22728_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22728_320 = {S39, 0, &V22728_320_57290, &P22728_344};	/*Property = PostMessage*/
const Value V22728_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22728_321 = {S40, 0, &V22728_321_57293, &P22728_320};	/*Property = Progress*/
const Value V22728_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22728_322 = {S41, 0, &V22728_322_57296, &P22728_321};	/*Property = Prompts*/
const Value V22728_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22728_323 = {S42, 0, &V22728_323_57299, &P22728_322};	/*Property = Selector*/
const Value V22728_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22728_324 = {S43, 0, &V22728_324_57306, &P22728_323};	/*Property = Svg*/
const Value V22728_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22728_327 = {S44, 0, &V22728_327_57315, &P22728_324};	/*Property = TouchEvents*/
const Value V22728_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22728_328 = {S45, 0, &V22728_328_57318, &P22728_327};	/*Property = Track*/
const Value V22728_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22728_329 = {S46, 0, &V22728_329_57323, &P22728_328};	/*Property = Video*/
const Value V22728_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22728_330 = {S47, 0, &V22728_330_57326, &P22728_329};	/*Property = Viewport*/
const Value V22728_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22728_332 = {S48, 0, &V22728_332_57339, &P22728_330};	/*Property = WebWorkers*/
const Value V22728_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22728_333 = {S49, 0, &V22728_333_57342, &P22728_332};	/*Property = Xhr2*/
const Profile P22728 = {"22728", &P22728_333};	/*Profile ID - 22728*/
const Value V17740_1_1 = {S2, NULL};	/*Value = True*/
const Property P17740_1 = {S50, 0, &V17740_1_1, NULL};	/*Property = IsMobile*/
const Value V17740_5_37540 = {S105, NULL};	/*Value = 427*/
const Property P17740_5 = {S52, 0, &V17740_5_37540, &P17740_1};	/*Property = ScreenPixelsHeight*/
const Value V17740_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17740_4 = {S54, 0, &V17740_4_1224, &P17740_5};	/*Property = ScreenPixelsWidth*/
const Profile P17740 = {"17740", &P17740_4};	/*Profile ID - 17740*/
const Profile P18189 = {"18189", NULL};	/*Profile ID - 18189*/
const Profile P18145 = {"18145", NULL};	/*Profile ID - 18145*/
const Value V22484_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_279 = {S1, 0, &V22484_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22484_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_282 = {S3, 0, &V22484_282_57155, &P22484_279};	/*Property = BlobBuilder*/
const Value V22484_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_350 = {S4, 0, &V22484_350_57446, &P22484_282};	/*Property = Canvas*/
const Value V22484_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_283 = {S5, 0, &V22484_283_57161, &P22484_350};	/*Property = CssBackground*/
const Value V22484_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_284 = {S6, 0, &V22484_284_57164, &P22484_283};	/*Property = CssBorderImage*/
const Value V22484_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_285 = {S7, 0, &V22484_285_57167, &P22484_284};	/*Property = CssCanvas*/
const Value V22484_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_286 = {S8, 0, &V22484_286_57170, &P22484_285};	/*Property = CssColor*/
const Value V22484_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_237 = {S9, 0, &V22484_237_55786, &P22484_286};	/*Property = CssColumn*/
const Value V22484_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_287 = {S10, 0, &V22484_287_57173, &P22484_237};	/*Property = CssFlexbox*/
const Value V22484_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_288 = {S11, 0, &V22484_288_57176, &P22484_287};	/*Property = CssFont*/
const Value V22484_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_289 = {S12, 0, &V22484_289_86899, &P22484_288};	/*Property = CssImages*/
const Value V22484_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_291 = {S13, 0, &V22484_291_57182, &P22484_289};	/*Property = CssMediaQueries*/
const Value V22484_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_292 = {S14, 0, &V22484_292_57185, &P22484_291};	/*Property = CssMinMax*/
const Value V22484_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_293 = {S15, 0, &V22484_293_57188, &P22484_292};	/*Property = CssOverflow*/
const Value V22484_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_294 = {S16, 0, &V22484_294_57191, &P22484_293};	/*Property = CssPosition*/
const Value V22484_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_295 = {S17, 0, &V22484_295_57194, &P22484_294};	/*Property = CssText*/
const Value V22484_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_297 = {S18, 0, &V22484_297_57197, &P22484_295};	/*Property = CssTransforms*/
const Value V22484_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_241 = {S19, 0, &V22484_241_55801, &P22484_297};	/*Property = CssTransitions*/
const Value V22484_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_298 = {S20, 0, &V22484_298_57208, &P22484_241};	/*Property = CssUI*/
const Value V22484_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_300 = {S21, 0, &V22484_300_57217, &P22484_298};	/*Property = DataSet*/
const Value V22484_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_301 = {S22, 0, &V22484_301_57220, &P22484_300};	/*Property = DataUrl*/
const Value V22484_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_302 = {S23, 0, &V22484_302_57223, &P22484_301};	/*Property = DeviceOrientation*/
const Value V22484_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_304 = {S24, 0, &V22484_304_57229, &P22484_302};	/*Property = FileReader*/
const Value V22484_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_305 = {S25, 0, &V22484_305_57232, &P22484_304};	/*Property = FileSaver*/
const Value V22484_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_306 = {S26, 0, &V22484_306_57238, &P22484_305};	/*Property = FileWriter*/
const Value V22484_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_307 = {S27, 0, &V22484_307_57241, &P22484_306};	/*Property = FormData*/
const Value V22484_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_309 = {S28, 0, &V22484_309_57247, &P22484_307};	/*Property = Fullscreen*/
const Value V22484_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_310 = {S29, 0, &V22484_310_57250, &P22484_309};	/*Property = GeoLocation*/
const Value V22484_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_312 = {S30, 0, &V22484_312_57256, &P22484_310};	/*Property = History*/
const Value V22484_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_314 = {S31, 0, &V22484_314_57262, &P22484_312};	/*Property = Html5*/
const Value V22484_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_313 = {S32, 0, &V22484_313_57259, &P22484_314};	/*Property = Html-Media-Capture*/
const Value V22484_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_315 = {S33, 0, &V22484_315_57265, &P22484_313};	/*Property = Iframe*/
const Value V22484_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_316 = {S34, 0, &V22484_316_57268, &P22484_315};	/*Property = IndexedDB*/
const Value V22484_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_317 = {S35, 0, &V22484_317_57271, &P22484_316};	/*Property = Json*/
const Value V22484_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P22484_15 = {S37, 0, &V22484_15_31, &P22484_317};	/*Property = LayoutEngine*/
const Value V22484_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_344 = {S38, 0, &V22484_344_57274, &P22484_15};	/*Property = Masking*/
const Value V22484_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_320 = {S39, 0, &V22484_320_57292, &P22484_344};	/*Property = PostMessage*/
const Value V22484_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_321 = {S40, 0, &V22484_321_57295, &P22484_320};	/*Property = Progress*/
const Value V22484_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_322 = {S41, 0, &V22484_322_57298, &P22484_321};	/*Property = Prompts*/
const Value V22484_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_323 = {S42, 0, &V22484_323_57301, &P22484_322};	/*Property = Selector*/
const Value V22484_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_324 = {S43, 0, &V22484_324_57307, &P22484_323};	/*Property = Svg*/
const Value V22484_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_327 = {S44, 0, &V22484_327_57316, &P22484_324};	/*Property = TouchEvents*/
const Value V22484_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_328 = {S45, 0, &V22484_328_57319, &P22484_327};	/*Property = Track*/
const Value V22484_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_329 = {S46, 0, &V22484_329_57325, &P22484_328};	/*Property = Video*/
const Value V22484_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_330 = {S47, 0, &V22484_330_57328, &P22484_329};	/*Property = Viewport*/
const Value V22484_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_332 = {S48, 0, &V22484_332_57340, &P22484_330};	/*Property = WebWorkers*/
const Value V22484_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22484_333 = {S49, 0, &V22484_333_57343, &P22484_332};	/*Property = Xhr2*/
const Profile P22484 = {"22484", &P22484_333};	/*Profile ID - 22484*/
const Profile P21445 = {"21445", NULL};	/*Profile ID - 21445*/
const Profile P18190 = {"18190", NULL};	/*Profile ID - 18190*/
const Value V17826_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_279 = {S1, 0, &V17826_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17826_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_282 = {S3, 0, &V17826_282_57155, &P17826_279};	/*Property = BlobBuilder*/
const Value V17826_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_350 = {S4, 0, &V17826_350_57446, &P17826_282};	/*Property = Canvas*/
const Value V17826_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_283 = {S5, 0, &V17826_283_57161, &P17826_350};	/*Property = CssBackground*/
const Value V17826_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_284 = {S6, 0, &V17826_284_57164, &P17826_283};	/*Property = CssBorderImage*/
const Value V17826_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_285 = {S7, 0, &V17826_285_57167, &P17826_284};	/*Property = CssCanvas*/
const Value V17826_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_286 = {S8, 0, &V17826_286_57170, &P17826_285};	/*Property = CssColor*/
const Value V17826_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_237 = {S9, 0, &V17826_237_55786, &P17826_286};	/*Property = CssColumn*/
const Value V17826_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_287 = {S10, 0, &V17826_287_57173, &P17826_237};	/*Property = CssFlexbox*/
const Value V17826_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_288 = {S11, 0, &V17826_288_57176, &P17826_287};	/*Property = CssFont*/
const Value V17826_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_289 = {S12, 0, &V17826_289_86899, &P17826_288};	/*Property = CssImages*/
const Value V17826_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_291 = {S13, 0, &V17826_291_57182, &P17826_289};	/*Property = CssMediaQueries*/
const Value V17826_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_292 = {S14, 0, &V17826_292_57185, &P17826_291};	/*Property = CssMinMax*/
const Value V17826_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_293 = {S15, 0, &V17826_293_57188, &P17826_292};	/*Property = CssOverflow*/
const Value V17826_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_294 = {S16, 0, &V17826_294_57191, &P17826_293};	/*Property = CssPosition*/
const Value V17826_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_295 = {S17, 0, &V17826_295_57194, &P17826_294};	/*Property = CssText*/
const Value V17826_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_297 = {S18, 0, &V17826_297_57197, &P17826_295};	/*Property = CssTransforms*/
const Value V17826_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_241 = {S19, 0, &V17826_241_55801, &P17826_297};	/*Property = CssTransitions*/
const Value V17826_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_298 = {S20, 0, &V17826_298_57208, &P17826_241};	/*Property = CssUI*/
const Value V17826_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_300 = {S21, 0, &V17826_300_57217, &P17826_298};	/*Property = DataSet*/
const Value V17826_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_301 = {S22, 0, &V17826_301_57220, &P17826_300};	/*Property = DataUrl*/
const Value V17826_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_302 = {S23, 0, &V17826_302_57223, &P17826_301};	/*Property = DeviceOrientation*/
const Value V17826_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_304 = {S24, 0, &V17826_304_57229, &P17826_302};	/*Property = FileReader*/
const Value V17826_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_305 = {S25, 0, &V17826_305_57232, &P17826_304};	/*Property = FileSaver*/
const Value V17826_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_306 = {S26, 0, &V17826_306_57238, &P17826_305};	/*Property = FileWriter*/
const Value V17826_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_307 = {S27, 0, &V17826_307_57241, &P17826_306};	/*Property = FormData*/
const Value V17826_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_309 = {S28, 0, &V17826_309_57247, &P17826_307};	/*Property = Fullscreen*/
const Value V17826_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_310 = {S29, 0, &V17826_310_57250, &P17826_309};	/*Property = GeoLocation*/
const Value V17826_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_312 = {S30, 0, &V17826_312_57256, &P17826_310};	/*Property = History*/
const Value V17826_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_314 = {S31, 0, &V17826_314_57262, &P17826_312};	/*Property = Html5*/
const Value V17826_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_313 = {S32, 0, &V17826_313_57259, &P17826_314};	/*Property = Html-Media-Capture*/
const Value V17826_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_315 = {S33, 0, &V17826_315_57265, &P17826_313};	/*Property = Iframe*/
const Value V17826_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_316 = {S34, 0, &V17826_316_57268, &P17826_315};	/*Property = IndexedDB*/
const Value V17826_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_317 = {S35, 0, &V17826_317_57271, &P17826_316};	/*Property = Json*/
const Value V17826_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P17826_15 = {S37, 0, &V17826_15_54167, &P17826_317};	/*Property = LayoutEngine*/
const Value V17826_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_344 = {S38, 0, &V17826_344_57274, &P17826_15};	/*Property = Masking*/
const Value V17826_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_320 = {S39, 0, &V17826_320_57292, &P17826_344};	/*Property = PostMessage*/
const Value V17826_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_321 = {S40, 0, &V17826_321_57295, &P17826_320};	/*Property = Progress*/
const Value V17826_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_322 = {S41, 0, &V17826_322_57298, &P17826_321};	/*Property = Prompts*/
const Value V17826_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_323 = {S42, 0, &V17826_323_57301, &P17826_322};	/*Property = Selector*/
const Value V17826_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_324 = {S43, 0, &V17826_324_57307, &P17826_323};	/*Property = Svg*/
const Value V17826_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_327 = {S44, 0, &V17826_327_57316, &P17826_324};	/*Property = TouchEvents*/
const Value V17826_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_328 = {S45, 0, &V17826_328_57319, &P17826_327};	/*Property = Track*/
const Value V17826_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_329 = {S46, 0, &V17826_329_57325, &P17826_328};	/*Property = Video*/
const Value V17826_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_330 = {S47, 0, &V17826_330_57328, &P17826_329};	/*Property = Viewport*/
const Value V17826_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_332 = {S48, 0, &V17826_332_57340, &P17826_330};	/*Property = WebWorkers*/
const Value V17826_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17826_333 = {S49, 0, &V17826_333_57343, &P17826_332};	/*Property = Xhr2*/
const Profile P17826 = {"17826", &P17826_333};	/*Profile ID - 17826*/
const Value V20800_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_279 = {S1, 0, &V20800_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20800_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_282 = {S3, 0, &V20800_282_57155, &P20800_279};	/*Property = BlobBuilder*/
const Value V20800_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_350 = {S4, 0, &V20800_350_57446, &P20800_282};	/*Property = Canvas*/
const Value V20800_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_283 = {S5, 0, &V20800_283_57161, &P20800_350};	/*Property = CssBackground*/
const Value V20800_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_284 = {S6, 0, &V20800_284_57164, &P20800_283};	/*Property = CssBorderImage*/
const Value V20800_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_285 = {S7, 0, &V20800_285_57167, &P20800_284};	/*Property = CssCanvas*/
const Value V20800_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_286 = {S8, 0, &V20800_286_57170, &P20800_285};	/*Property = CssColor*/
const Value V20800_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_237 = {S9, 0, &V20800_237_55786, &P20800_286};	/*Property = CssColumn*/
const Value V20800_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_287 = {S10, 0, &V20800_287_57173, &P20800_237};	/*Property = CssFlexbox*/
const Value V20800_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_288 = {S11, 0, &V20800_288_57176, &P20800_287};	/*Property = CssFont*/
const Value V20800_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_289 = {S12, 0, &V20800_289_86899, &P20800_288};	/*Property = CssImages*/
const Value V20800_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_291 = {S13, 0, &V20800_291_57182, &P20800_289};	/*Property = CssMediaQueries*/
const Value V20800_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_292 = {S14, 0, &V20800_292_57185, &P20800_291};	/*Property = CssMinMax*/
const Value V20800_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_293 = {S15, 0, &V20800_293_57188, &P20800_292};	/*Property = CssOverflow*/
const Value V20800_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_294 = {S16, 0, &V20800_294_57191, &P20800_293};	/*Property = CssPosition*/
const Value V20800_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_295 = {S17, 0, &V20800_295_57194, &P20800_294};	/*Property = CssText*/
const Value V20800_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_297 = {S18, 0, &V20800_297_57197, &P20800_295};	/*Property = CssTransforms*/
const Value V20800_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_241 = {S19, 0, &V20800_241_55801, &P20800_297};	/*Property = CssTransitions*/
const Value V20800_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_298 = {S20, 0, &V20800_298_57208, &P20800_241};	/*Property = CssUI*/
const Value V20800_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_300 = {S21, 0, &V20800_300_57217, &P20800_298};	/*Property = DataSet*/
const Value V20800_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_301 = {S22, 0, &V20800_301_57220, &P20800_300};	/*Property = DataUrl*/
const Value V20800_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_302 = {S23, 0, &V20800_302_57223, &P20800_301};	/*Property = DeviceOrientation*/
const Value V20800_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_304 = {S24, 0, &V20800_304_57229, &P20800_302};	/*Property = FileReader*/
const Value V20800_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_305 = {S25, 0, &V20800_305_57232, &P20800_304};	/*Property = FileSaver*/
const Value V20800_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_306 = {S26, 0, &V20800_306_57238, &P20800_305};	/*Property = FileWriter*/
const Value V20800_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_307 = {S27, 0, &V20800_307_57241, &P20800_306};	/*Property = FormData*/
const Value V20800_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_309 = {S28, 0, &V20800_309_57247, &P20800_307};	/*Property = Fullscreen*/
const Value V20800_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_310 = {S29, 0, &V20800_310_57250, &P20800_309};	/*Property = GeoLocation*/
const Value V20800_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_312 = {S30, 0, &V20800_312_57256, &P20800_310};	/*Property = History*/
const Value V20800_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_314 = {S31, 0, &V20800_314_57262, &P20800_312};	/*Property = Html5*/
const Value V20800_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_313 = {S32, 0, &V20800_313_57259, &P20800_314};	/*Property = Html-Media-Capture*/
const Value V20800_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_315 = {S33, 0, &V20800_315_57265, &P20800_313};	/*Property = Iframe*/
const Value V20800_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_316 = {S34, 0, &V20800_316_57268, &P20800_315};	/*Property = IndexedDB*/
const Value V20800_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_317 = {S35, 0, &V20800_317_57271, &P20800_316};	/*Property = Json*/
const Value V20800_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_15 = {S37, 0, &V20800_15_72, &P20800_317};	/*Property = LayoutEngine*/
const Value V20800_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_344 = {S38, 0, &V20800_344_57274, &P20800_15};	/*Property = Masking*/
const Value V20800_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_320 = {S39, 0, &V20800_320_57292, &P20800_344};	/*Property = PostMessage*/
const Value V20800_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_321 = {S40, 0, &V20800_321_57295, &P20800_320};	/*Property = Progress*/
const Value V20800_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_322 = {S41, 0, &V20800_322_57298, &P20800_321};	/*Property = Prompts*/
const Value V20800_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_323 = {S42, 0, &V20800_323_57301, &P20800_322};	/*Property = Selector*/
const Value V20800_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_324 = {S43, 0, &V20800_324_57307, &P20800_323};	/*Property = Svg*/
const Value V20800_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_327 = {S44, 0, &V20800_327_57316, &P20800_324};	/*Property = TouchEvents*/
const Value V20800_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_328 = {S45, 0, &V20800_328_57319, &P20800_327};	/*Property = Track*/
const Value V20800_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_329 = {S46, 0, &V20800_329_57325, &P20800_328};	/*Property = Video*/
const Value V20800_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_330 = {S47, 0, &V20800_330_57328, &P20800_329};	/*Property = Viewport*/
const Value V20800_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_332 = {S48, 0, &V20800_332_57340, &P20800_330};	/*Property = WebWorkers*/
const Value V20800_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20800_333 = {S49, 0, &V20800_333_57343, &P20800_332};	/*Property = Xhr2*/
const Profile P20800 = {"20800", &P20800_333};	/*Profile ID - 20800*/
const Value V16249_1_1 = {S2, NULL};	/*Value = True*/
const Property P16249_1 = {S50, 0, &V16249_1_1, NULL};	/*Property = IsMobile*/
const Value V16249_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16249_5 = {S52, 0, &V16249_5_1233, &P16249_1};	/*Property = ScreenPixelsHeight*/
const Value V16249_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16249_4 = {S54, 0, &V16249_4_1235, &P16249_5};	/*Property = ScreenPixelsWidth*/
const Profile P16249 = {"16249", &P16249_4};	/*Profile ID - 16249*/
const Value V18568_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_279 = {S1, 0, &V18568_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18568_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_282 = {S3, 0, &V18568_282_57155, &P18568_279};	/*Property = BlobBuilder*/
const Value V18568_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_350 = {S4, 0, &V18568_350_57446, &P18568_282};	/*Property = Canvas*/
const Value V18568_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_283 = {S5, 0, &V18568_283_57161, &P18568_350};	/*Property = CssBackground*/
const Value V18568_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_284 = {S6, 0, &V18568_284_57164, &P18568_283};	/*Property = CssBorderImage*/
const Value V18568_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_285 = {S7, 0, &V18568_285_57167, &P18568_284};	/*Property = CssCanvas*/
const Value V18568_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_286 = {S8, 0, &V18568_286_57170, &P18568_285};	/*Property = CssColor*/
const Value V18568_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_237 = {S9, 0, &V18568_237_55786, &P18568_286};	/*Property = CssColumn*/
const Value V18568_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_287 = {S10, 0, &V18568_287_57173, &P18568_237};	/*Property = CssFlexbox*/
const Value V18568_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_288 = {S11, 0, &V18568_288_57176, &P18568_287};	/*Property = CssFont*/
const Value V18568_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_289 = {S12, 0, &V18568_289_86899, &P18568_288};	/*Property = CssImages*/
const Value V18568_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_291 = {S13, 0, &V18568_291_57182, &P18568_289};	/*Property = CssMediaQueries*/
const Value V18568_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_292 = {S14, 0, &V18568_292_57185, &P18568_291};	/*Property = CssMinMax*/
const Value V18568_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_293 = {S15, 0, &V18568_293_57188, &P18568_292};	/*Property = CssOverflow*/
const Value V18568_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_294 = {S16, 0, &V18568_294_57191, &P18568_293};	/*Property = CssPosition*/
const Value V18568_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_295 = {S17, 0, &V18568_295_57194, &P18568_294};	/*Property = CssText*/
const Value V18568_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_297 = {S18, 0, &V18568_297_57197, &P18568_295};	/*Property = CssTransforms*/
const Value V18568_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_241 = {S19, 0, &V18568_241_55801, &P18568_297};	/*Property = CssTransitions*/
const Value V18568_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_298 = {S20, 0, &V18568_298_57208, &P18568_241};	/*Property = CssUI*/
const Value V18568_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_300 = {S21, 0, &V18568_300_57217, &P18568_298};	/*Property = DataSet*/
const Value V18568_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_301 = {S22, 0, &V18568_301_57220, &P18568_300};	/*Property = DataUrl*/
const Value V18568_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_302 = {S23, 0, &V18568_302_57223, &P18568_301};	/*Property = DeviceOrientation*/
const Value V18568_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_304 = {S24, 0, &V18568_304_57229, &P18568_302};	/*Property = FileReader*/
const Value V18568_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_305 = {S25, 0, &V18568_305_57232, &P18568_304};	/*Property = FileSaver*/
const Value V18568_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_306 = {S26, 0, &V18568_306_57238, &P18568_305};	/*Property = FileWriter*/
const Value V18568_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_307 = {S27, 0, &V18568_307_57241, &P18568_306};	/*Property = FormData*/
const Value V18568_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_309 = {S28, 0, &V18568_309_57247, &P18568_307};	/*Property = Fullscreen*/
const Value V18568_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_310 = {S29, 0, &V18568_310_57250, &P18568_309};	/*Property = GeoLocation*/
const Value V18568_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_312 = {S30, 0, &V18568_312_57256, &P18568_310};	/*Property = History*/
const Value V18568_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_314 = {S31, 0, &V18568_314_57262, &P18568_312};	/*Property = Html5*/
const Value V18568_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_313 = {S32, 0, &V18568_313_57259, &P18568_314};	/*Property = Html-Media-Capture*/
const Value V18568_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_315 = {S33, 0, &V18568_315_57265, &P18568_313};	/*Property = Iframe*/
const Value V18568_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_316 = {S34, 0, &V18568_316_57268, &P18568_315};	/*Property = IndexedDB*/
const Value V18568_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_317 = {S35, 0, &V18568_317_57271, &P18568_316};	/*Property = Json*/
const Value V18568_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_15 = {S37, 0, &V18568_15_72, &P18568_317};	/*Property = LayoutEngine*/
const Value V18568_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_344 = {S38, 0, &V18568_344_57274, &P18568_15};	/*Property = Masking*/
const Value V18568_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_320 = {S39, 0, &V18568_320_57292, &P18568_344};	/*Property = PostMessage*/
const Value V18568_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_321 = {S40, 0, &V18568_321_57295, &P18568_320};	/*Property = Progress*/
const Value V18568_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_322 = {S41, 0, &V18568_322_57298, &P18568_321};	/*Property = Prompts*/
const Value V18568_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_323 = {S42, 0, &V18568_323_57301, &P18568_322};	/*Property = Selector*/
const Value V18568_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_324 = {S43, 0, &V18568_324_57307, &P18568_323};	/*Property = Svg*/
const Value V18568_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_327 = {S44, 0, &V18568_327_57316, &P18568_324};	/*Property = TouchEvents*/
const Value V18568_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_328 = {S45, 0, &V18568_328_57319, &P18568_327};	/*Property = Track*/
const Value V18568_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_329 = {S46, 0, &V18568_329_57325, &P18568_328};	/*Property = Video*/
const Value V18568_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_330 = {S47, 0, &V18568_330_57328, &P18568_329};	/*Property = Viewport*/
const Value V18568_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_332 = {S48, 0, &V18568_332_57340, &P18568_330};	/*Property = WebWorkers*/
const Value V18568_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18568_333 = {S49, 0, &V18568_333_57343, &P18568_332};	/*Property = Xhr2*/
const Profile P18568 = {"18568", &P18568_333};	/*Profile ID - 18568*/
const Value V18353_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_279 = {S1, 0, &V18353_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18353_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_282 = {S3, 0, &V18353_282_57155, &P18353_279};	/*Property = BlobBuilder*/
const Value V18353_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_350 = {S4, 0, &V18353_350_57446, &P18353_282};	/*Property = Canvas*/
const Value V18353_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_283 = {S5, 0, &V18353_283_57161, &P18353_350};	/*Property = CssBackground*/
const Value V18353_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_284 = {S6, 0, &V18353_284_57164, &P18353_283};	/*Property = CssBorderImage*/
const Value V18353_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_285 = {S7, 0, &V18353_285_57167, &P18353_284};	/*Property = CssCanvas*/
const Value V18353_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_286 = {S8, 0, &V18353_286_57170, &P18353_285};	/*Property = CssColor*/
const Value V18353_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_237 = {S9, 0, &V18353_237_55786, &P18353_286};	/*Property = CssColumn*/
const Value V18353_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_287 = {S10, 0, &V18353_287_57173, &P18353_237};	/*Property = CssFlexbox*/
const Value V18353_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_288 = {S11, 0, &V18353_288_57176, &P18353_287};	/*Property = CssFont*/
const Value V18353_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_289 = {S12, 0, &V18353_289_86899, &P18353_288};	/*Property = CssImages*/
const Value V18353_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_291 = {S13, 0, &V18353_291_57182, &P18353_289};	/*Property = CssMediaQueries*/
const Value V18353_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_292 = {S14, 0, &V18353_292_57185, &P18353_291};	/*Property = CssMinMax*/
const Value V18353_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_293 = {S15, 0, &V18353_293_57188, &P18353_292};	/*Property = CssOverflow*/
const Value V18353_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_294 = {S16, 0, &V18353_294_57191, &P18353_293};	/*Property = CssPosition*/
const Value V18353_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_295 = {S17, 0, &V18353_295_57194, &P18353_294};	/*Property = CssText*/
const Value V18353_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_297 = {S18, 0, &V18353_297_57197, &P18353_295};	/*Property = CssTransforms*/
const Value V18353_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_241 = {S19, 0, &V18353_241_55801, &P18353_297};	/*Property = CssTransitions*/
const Value V18353_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_298 = {S20, 0, &V18353_298_57208, &P18353_241};	/*Property = CssUI*/
const Value V18353_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_300 = {S21, 0, &V18353_300_57217, &P18353_298};	/*Property = DataSet*/
const Value V18353_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_301 = {S22, 0, &V18353_301_57220, &P18353_300};	/*Property = DataUrl*/
const Value V18353_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_302 = {S23, 0, &V18353_302_57223, &P18353_301};	/*Property = DeviceOrientation*/
const Value V18353_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_304 = {S24, 0, &V18353_304_57229, &P18353_302};	/*Property = FileReader*/
const Value V18353_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_305 = {S25, 0, &V18353_305_57232, &P18353_304};	/*Property = FileSaver*/
const Value V18353_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_306 = {S26, 0, &V18353_306_57238, &P18353_305};	/*Property = FileWriter*/
const Value V18353_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_307 = {S27, 0, &V18353_307_57241, &P18353_306};	/*Property = FormData*/
const Value V18353_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_309 = {S28, 0, &V18353_309_57247, &P18353_307};	/*Property = Fullscreen*/
const Value V18353_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_310 = {S29, 0, &V18353_310_57250, &P18353_309};	/*Property = GeoLocation*/
const Value V18353_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_312 = {S30, 0, &V18353_312_57256, &P18353_310};	/*Property = History*/
const Value V18353_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_314 = {S31, 0, &V18353_314_57262, &P18353_312};	/*Property = Html5*/
const Value V18353_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_313 = {S32, 0, &V18353_313_57259, &P18353_314};	/*Property = Html-Media-Capture*/
const Value V18353_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_315 = {S33, 0, &V18353_315_57265, &P18353_313};	/*Property = Iframe*/
const Value V18353_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_316 = {S34, 0, &V18353_316_57268, &P18353_315};	/*Property = IndexedDB*/
const Value V18353_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_317 = {S35, 0, &V18353_317_57271, &P18353_316};	/*Property = Json*/
const Value V18353_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P18353_15 = {S37, 0, &V18353_15_30, &P18353_317};	/*Property = LayoutEngine*/
const Value V18353_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_344 = {S38, 0, &V18353_344_57274, &P18353_15};	/*Property = Masking*/
const Value V18353_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_320 = {S39, 0, &V18353_320_57292, &P18353_344};	/*Property = PostMessage*/
const Value V18353_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_321 = {S40, 0, &V18353_321_57295, &P18353_320};	/*Property = Progress*/
const Value V18353_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_322 = {S41, 0, &V18353_322_57298, &P18353_321};	/*Property = Prompts*/
const Value V18353_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_323 = {S42, 0, &V18353_323_57301, &P18353_322};	/*Property = Selector*/
const Value V18353_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_324 = {S43, 0, &V18353_324_57307, &P18353_323};	/*Property = Svg*/
const Value V18353_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_327 = {S44, 0, &V18353_327_57316, &P18353_324};	/*Property = TouchEvents*/
const Value V18353_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_328 = {S45, 0, &V18353_328_57319, &P18353_327};	/*Property = Track*/
const Value V18353_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_329 = {S46, 0, &V18353_329_57325, &P18353_328};	/*Property = Video*/
const Value V18353_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_330 = {S47, 0, &V18353_330_57328, &P18353_329};	/*Property = Viewport*/
const Value V18353_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_332 = {S48, 0, &V18353_332_57340, &P18353_330};	/*Property = WebWorkers*/
const Value V18353_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18353_333 = {S49, 0, &V18353_333_57343, &P18353_332};	/*Property = Xhr2*/
const Profile P18353 = {"18353", &P18353_333};	/*Profile ID - 18353*/
const Value V22444_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22444_279 = {S1, 0, &V22444_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22444_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22444_282 = {S3, 0, &V22444_282_57154, &P22444_279};	/*Property = BlobBuilder*/
const Value V22444_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22444_350 = {S4, 0, &V22444_350_57444, &P22444_282};	/*Property = Canvas*/
const Value V22444_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22444_283 = {S5, 0, &V22444_283_57159, &P22444_350};	/*Property = CssBackground*/
const Value V22444_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22444_284 = {S6, 0, &V22444_284_57163, &P22444_283};	/*Property = CssBorderImage*/
const Value V22444_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22444_285 = {S7, 0, &V22444_285_57166, &P22444_284};	/*Property = CssCanvas*/
const Value V22444_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22444_286 = {S8, 0, &V22444_286_57168, &P22444_285};	/*Property = CssColor*/
const Value V22444_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22444_237 = {S9, 0, &V22444_237_55786, &P22444_286};	/*Property = CssColumn*/
const Value V22444_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22444_287 = {S10, 0, &V22444_287_57172, &P22444_237};	/*Property = CssFlexbox*/
const Value V22444_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22444_288 = {S11, 0, &V22444_288_57174, &P22444_287};	/*Property = CssFont*/
const Value V22444_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22444_289 = {S12, 0, &V22444_289_57759, &P22444_288};	/*Property = CssImages*/
const Value V22444_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22444_291 = {S13, 0, &V22444_291_57181, &P22444_289};	/*Property = CssMediaQueries*/
const Value V22444_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22444_292 = {S14, 0, &V22444_292_57183, &P22444_291};	/*Property = CssMinMax*/
const Value V22444_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22444_293 = {S15, 0, &V22444_293_57187, &P22444_292};	/*Property = CssOverflow*/
const Value V22444_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22444_294 = {S16, 0, &V22444_294_57189, &P22444_293};	/*Property = CssPosition*/
const Value V22444_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22444_295 = {S17, 0, &V22444_295_57192, &P22444_294};	/*Property = CssText*/
const Value V22444_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22444_297 = {S18, 0, &V22444_297_57195, &P22444_295};	/*Property = CssTransforms*/
const Value V22444_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22444_241 = {S19, 0, &V22444_241_55799, &P22444_297};	/*Property = CssTransitions*/
const Value V22444_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22444_298 = {S20, 0, &V22444_298_57206, &P22444_241};	/*Property = CssUI*/
const Value V22444_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22444_300 = {S21, 0, &V22444_300_57216, &P22444_298};	/*Property = DataSet*/
const Value V22444_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22444_301 = {S22, 0, &V22444_301_57218, &P22444_300};	/*Property = DataUrl*/
const Value V22444_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22444_302 = {S23, 0, &V22444_302_57222, &P22444_301};	/*Property = DeviceOrientation*/
const Value V22444_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22444_304 = {S24, 0, &V22444_304_57228, &P22444_302};	/*Property = FileReader*/
const Value V22444_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22444_305 = {S25, 0, &V22444_305_57231, &P22444_304};	/*Property = FileSaver*/
const Value V22444_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22444_306 = {S26, 0, &V22444_306_57237, &P22444_305};	/*Property = FileWriter*/
const Value V22444_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22444_307 = {S27, 0, &V22444_307_57240, &P22444_306};	/*Property = FormData*/
const Value V22444_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22444_309 = {S28, 0, &V22444_309_57246, &P22444_307};	/*Property = Fullscreen*/
const Value V22444_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22444_310 = {S29, 0, &V22444_310_57248, &P22444_309};	/*Property = GeoLocation*/
const Value V22444_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22444_312 = {S30, 0, &V22444_312_57255, &P22444_310};	/*Property = History*/
const Value V22444_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22444_314 = {S31, 0, &V22444_314_57261, &P22444_312};	/*Property = Html5*/
const Value V22444_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22444_313 = {S32, 0, &V22444_313_57258, &P22444_314};	/*Property = Html-Media-Capture*/
const Value V22444_315_57263 = {S2, NULL};	/*Value = True*/
const Property P22444_315 = {S33, 0, &V22444_315_57263, &P22444_313};	/*Property = Iframe*/
const Value V22444_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22444_316 = {S34, 0, &V22444_316_57267, &P22444_315};	/*Property = IndexedDB*/
const Value V22444_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22444_317 = {S35, 0, &V22444_317_57269, &P22444_316};	/*Property = Json*/
const Value V22444_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22444_15 = {S37, 0, &V22444_15_28, &P22444_317};	/*Property = LayoutEngine*/
const Value V22444_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22444_344 = {S38, 0, &V22444_344_57272, &P22444_15};	/*Property = Masking*/
const Value V22444_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22444_320 = {S39, 0, &V22444_320_57290, &P22444_344};	/*Property = PostMessage*/
const Value V22444_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22444_321 = {S40, 0, &V22444_321_57293, &P22444_320};	/*Property = Progress*/
const Value V22444_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22444_322 = {S41, 0, &V22444_322_57296, &P22444_321};	/*Property = Prompts*/
const Value V22444_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22444_323 = {S42, 0, &V22444_323_57299, &P22444_322};	/*Property = Selector*/
const Value V22444_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22444_324 = {S43, 0, &V22444_324_57305, &P22444_323};	/*Property = Svg*/
const Value V22444_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22444_327 = {S44, 0, &V22444_327_57315, &P22444_324};	/*Property = TouchEvents*/
const Value V22444_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22444_328 = {S45, 0, &V22444_328_57318, &P22444_327};	/*Property = Track*/
const Value V22444_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22444_329 = {S46, 0, &V22444_329_57323, &P22444_328};	/*Property = Video*/
const Value V22444_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22444_330 = {S47, 0, &V22444_330_57326, &P22444_329};	/*Property = Viewport*/
const Value V22444_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22444_332 = {S48, 0, &V22444_332_57339, &P22444_330};	/*Property = WebWorkers*/
const Value V22444_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22444_333 = {S49, 0, &V22444_333_57342, &P22444_332};	/*Property = Xhr2*/
const Profile P22444 = {"22444", &P22444_333};	/*Profile ID - 22444*/
const Profile P18143 = {"18143", NULL};	/*Profile ID - 18143*/
const Value V20411_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_279 = {S1, 0, &V20411_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20411_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_282 = {S3, 0, &V20411_282_57155, &P20411_279};	/*Property = BlobBuilder*/
const Value V20411_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_350 = {S4, 0, &V20411_350_57446, &P20411_282};	/*Property = Canvas*/
const Value V20411_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_283 = {S5, 0, &V20411_283_57161, &P20411_350};	/*Property = CssBackground*/
const Value V20411_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_284 = {S6, 0, &V20411_284_57164, &P20411_283};	/*Property = CssBorderImage*/
const Value V20411_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_285 = {S7, 0, &V20411_285_57167, &P20411_284};	/*Property = CssCanvas*/
const Value V20411_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_286 = {S8, 0, &V20411_286_57170, &P20411_285};	/*Property = CssColor*/
const Value V20411_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_237 = {S9, 0, &V20411_237_55786, &P20411_286};	/*Property = CssColumn*/
const Value V20411_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_287 = {S10, 0, &V20411_287_57173, &P20411_237};	/*Property = CssFlexbox*/
const Value V20411_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_288 = {S11, 0, &V20411_288_57176, &P20411_287};	/*Property = CssFont*/
const Value V20411_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_289 = {S12, 0, &V20411_289_86899, &P20411_288};	/*Property = CssImages*/
const Value V20411_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_291 = {S13, 0, &V20411_291_57182, &P20411_289};	/*Property = CssMediaQueries*/
const Value V20411_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_292 = {S14, 0, &V20411_292_57185, &P20411_291};	/*Property = CssMinMax*/
const Value V20411_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_293 = {S15, 0, &V20411_293_57188, &P20411_292};	/*Property = CssOverflow*/
const Value V20411_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_294 = {S16, 0, &V20411_294_57191, &P20411_293};	/*Property = CssPosition*/
const Value V20411_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_295 = {S17, 0, &V20411_295_57194, &P20411_294};	/*Property = CssText*/
const Value V20411_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_297 = {S18, 0, &V20411_297_57197, &P20411_295};	/*Property = CssTransforms*/
const Value V20411_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_241 = {S19, 0, &V20411_241_55801, &P20411_297};	/*Property = CssTransitions*/
const Value V20411_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_298 = {S20, 0, &V20411_298_57208, &P20411_241};	/*Property = CssUI*/
const Value V20411_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_300 = {S21, 0, &V20411_300_57217, &P20411_298};	/*Property = DataSet*/
const Value V20411_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_301 = {S22, 0, &V20411_301_57220, &P20411_300};	/*Property = DataUrl*/
const Value V20411_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_302 = {S23, 0, &V20411_302_57223, &P20411_301};	/*Property = DeviceOrientation*/
const Value V20411_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_304 = {S24, 0, &V20411_304_57229, &P20411_302};	/*Property = FileReader*/
const Value V20411_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_305 = {S25, 0, &V20411_305_57232, &P20411_304};	/*Property = FileSaver*/
const Value V20411_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_306 = {S26, 0, &V20411_306_57238, &P20411_305};	/*Property = FileWriter*/
const Value V20411_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_307 = {S27, 0, &V20411_307_57241, &P20411_306};	/*Property = FormData*/
const Value V20411_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_309 = {S28, 0, &V20411_309_57247, &P20411_307};	/*Property = Fullscreen*/
const Value V20411_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_310 = {S29, 0, &V20411_310_57250, &P20411_309};	/*Property = GeoLocation*/
const Value V20411_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_312 = {S30, 0, &V20411_312_57256, &P20411_310};	/*Property = History*/
const Value V20411_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_314 = {S31, 0, &V20411_314_57262, &P20411_312};	/*Property = Html5*/
const Value V20411_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_313 = {S32, 0, &V20411_313_57259, &P20411_314};	/*Property = Html-Media-Capture*/
const Value V20411_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_315 = {S33, 0, &V20411_315_57265, &P20411_313};	/*Property = Iframe*/
const Value V20411_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_316 = {S34, 0, &V20411_316_57268, &P20411_315};	/*Property = IndexedDB*/
const Value V20411_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_317 = {S35, 0, &V20411_317_57271, &P20411_316};	/*Property = Json*/
const Value V20411_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P20411_15 = {S37, 0, &V20411_15_31, &P20411_317};	/*Property = LayoutEngine*/
const Value V20411_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_344 = {S38, 0, &V20411_344_57274, &P20411_15};	/*Property = Masking*/
const Value V20411_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_320 = {S39, 0, &V20411_320_57292, &P20411_344};	/*Property = PostMessage*/
const Value V20411_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_321 = {S40, 0, &V20411_321_57295, &P20411_320};	/*Property = Progress*/
const Value V20411_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_322 = {S41, 0, &V20411_322_57298, &P20411_321};	/*Property = Prompts*/
const Value V20411_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_323 = {S42, 0, &V20411_323_57301, &P20411_322};	/*Property = Selector*/
const Value V20411_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_324 = {S43, 0, &V20411_324_57307, &P20411_323};	/*Property = Svg*/
const Value V20411_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_327 = {S44, 0, &V20411_327_57316, &P20411_324};	/*Property = TouchEvents*/
const Value V20411_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_328 = {S45, 0, &V20411_328_57319, &P20411_327};	/*Property = Track*/
const Value V20411_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_329 = {S46, 0, &V20411_329_57325, &P20411_328};	/*Property = Video*/
const Value V20411_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_330 = {S47, 0, &V20411_330_57328, &P20411_329};	/*Property = Viewport*/
const Value V20411_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_332 = {S48, 0, &V20411_332_57340, &P20411_330};	/*Property = WebWorkers*/
const Value V20411_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20411_333 = {S49, 0, &V20411_333_57343, &P20411_332};	/*Property = Xhr2*/
const Profile P20411 = {"20411", &P20411_333};	/*Profile ID - 20411*/
const Value V4809_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_279 = {S1, 0, &V4809_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4809_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_282 = {S3, 0, &V4809_282_57155, &P4809_279};	/*Property = BlobBuilder*/
const Value V4809_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_350 = {S4, 0, &V4809_350_57446, &P4809_282};	/*Property = Canvas*/
const Value V4809_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_283 = {S5, 0, &V4809_283_57161, &P4809_350};	/*Property = CssBackground*/
const Value V4809_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_284 = {S6, 0, &V4809_284_57164, &P4809_283};	/*Property = CssBorderImage*/
const Value V4809_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_285 = {S7, 0, &V4809_285_57167, &P4809_284};	/*Property = CssCanvas*/
const Value V4809_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_286 = {S8, 0, &V4809_286_57170, &P4809_285};	/*Property = CssColor*/
const Value V4809_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_237 = {S9, 0, &V4809_237_55786, &P4809_286};	/*Property = CssColumn*/
const Value V4809_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_287 = {S10, 0, &V4809_287_57173, &P4809_237};	/*Property = CssFlexbox*/
const Value V4809_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_288 = {S11, 0, &V4809_288_57176, &P4809_287};	/*Property = CssFont*/
const Value V4809_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_289 = {S12, 0, &V4809_289_86899, &P4809_288};	/*Property = CssImages*/
const Value V4809_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_291 = {S13, 0, &V4809_291_57182, &P4809_289};	/*Property = CssMediaQueries*/
const Value V4809_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_292 = {S14, 0, &V4809_292_57185, &P4809_291};	/*Property = CssMinMax*/
const Value V4809_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_293 = {S15, 0, &V4809_293_57188, &P4809_292};	/*Property = CssOverflow*/
const Value V4809_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_294 = {S16, 0, &V4809_294_57191, &P4809_293};	/*Property = CssPosition*/
const Value V4809_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_295 = {S17, 0, &V4809_295_57194, &P4809_294};	/*Property = CssText*/
const Value V4809_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_297 = {S18, 0, &V4809_297_57197, &P4809_295};	/*Property = CssTransforms*/
const Value V4809_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_241 = {S19, 0, &V4809_241_55801, &P4809_297};	/*Property = CssTransitions*/
const Value V4809_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_298 = {S20, 0, &V4809_298_57208, &P4809_241};	/*Property = CssUI*/
const Value V4809_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_300 = {S21, 0, &V4809_300_57217, &P4809_298};	/*Property = DataSet*/
const Value V4809_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_301 = {S22, 0, &V4809_301_57220, &P4809_300};	/*Property = DataUrl*/
const Value V4809_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_302 = {S23, 0, &V4809_302_57223, &P4809_301};	/*Property = DeviceOrientation*/
const Value V4809_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_304 = {S24, 0, &V4809_304_57229, &P4809_302};	/*Property = FileReader*/
const Value V4809_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_305 = {S25, 0, &V4809_305_57232, &P4809_304};	/*Property = FileSaver*/
const Value V4809_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_306 = {S26, 0, &V4809_306_57238, &P4809_305};	/*Property = FileWriter*/
const Value V4809_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_307 = {S27, 0, &V4809_307_57241, &P4809_306};	/*Property = FormData*/
const Value V4809_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_309 = {S28, 0, &V4809_309_57247, &P4809_307};	/*Property = Fullscreen*/
const Value V4809_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_310 = {S29, 0, &V4809_310_57250, &P4809_309};	/*Property = GeoLocation*/
const Value V4809_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_312 = {S30, 0, &V4809_312_57256, &P4809_310};	/*Property = History*/
const Value V4809_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_314 = {S31, 0, &V4809_314_57262, &P4809_312};	/*Property = Html5*/
const Value V4809_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_313 = {S32, 0, &V4809_313_57259, &P4809_314};	/*Property = Html-Media-Capture*/
const Value V4809_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_315 = {S33, 0, &V4809_315_57265, &P4809_313};	/*Property = Iframe*/
const Value V4809_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_316 = {S34, 0, &V4809_316_57268, &P4809_315};	/*Property = IndexedDB*/
const Value V4809_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_317 = {S35, 0, &V4809_317_57271, &P4809_316};	/*Property = Json*/
const Value V4809_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P4809_15 = {S37, 0, &V4809_15_31, &P4809_317};	/*Property = LayoutEngine*/
const Value V4809_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_344 = {S38, 0, &V4809_344_57274, &P4809_15};	/*Property = Masking*/
const Value V4809_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_320 = {S39, 0, &V4809_320_57292, &P4809_344};	/*Property = PostMessage*/
const Value V4809_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_321 = {S40, 0, &V4809_321_57295, &P4809_320};	/*Property = Progress*/
const Value V4809_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_322 = {S41, 0, &V4809_322_57298, &P4809_321};	/*Property = Prompts*/
const Value V4809_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_323 = {S42, 0, &V4809_323_57301, &P4809_322};	/*Property = Selector*/
const Value V4809_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_324 = {S43, 0, &V4809_324_57307, &P4809_323};	/*Property = Svg*/
const Value V4809_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_327 = {S44, 0, &V4809_327_57316, &P4809_324};	/*Property = TouchEvents*/
const Value V4809_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_328 = {S45, 0, &V4809_328_57319, &P4809_327};	/*Property = Track*/
const Value V4809_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_329 = {S46, 0, &V4809_329_57325, &P4809_328};	/*Property = Video*/
const Value V4809_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_330 = {S47, 0, &V4809_330_57328, &P4809_329};	/*Property = Viewport*/
const Value V4809_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_332 = {S48, 0, &V4809_332_57340, &P4809_330};	/*Property = WebWorkers*/
const Value V4809_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4809_333 = {S49, 0, &V4809_333_57343, &P4809_332};	/*Property = Xhr2*/
const Profile P4809 = {"4809", &P4809_333};	/*Profile ID - 4809*/
const Value V17900_1_1 = {S2, NULL};	/*Value = True*/
const Property P17900_1 = {S50, 0, &V17900_1_1, NULL};	/*Property = IsMobile*/
const Value V17900_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17900_5 = {S52, 0, &V17900_5_1222, &P17900_1};	/*Property = ScreenPixelsHeight*/
const Value V17900_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17900_4 = {S54, 0, &V17900_4_1224, &P17900_5};	/*Property = ScreenPixelsWidth*/
const Profile P17900 = {"17900", &P17900_4};	/*Profile ID - 17900*/
const Value V18124_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_279 = {S1, 0, &V18124_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18124_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_282 = {S3, 0, &V18124_282_57155, &P18124_279};	/*Property = BlobBuilder*/
const Value V18124_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_350 = {S4, 0, &V18124_350_57446, &P18124_282};	/*Property = Canvas*/
const Value V18124_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_283 = {S5, 0, &V18124_283_57161, &P18124_350};	/*Property = CssBackground*/
const Value V18124_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_284 = {S6, 0, &V18124_284_57164, &P18124_283};	/*Property = CssBorderImage*/
const Value V18124_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_285 = {S7, 0, &V18124_285_57167, &P18124_284};	/*Property = CssCanvas*/
const Value V18124_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_286 = {S8, 0, &V18124_286_57170, &P18124_285};	/*Property = CssColor*/
const Value V18124_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_237 = {S9, 0, &V18124_237_55786, &P18124_286};	/*Property = CssColumn*/
const Value V18124_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_287 = {S10, 0, &V18124_287_57173, &P18124_237};	/*Property = CssFlexbox*/
const Value V18124_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_288 = {S11, 0, &V18124_288_57176, &P18124_287};	/*Property = CssFont*/
const Value V18124_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_289 = {S12, 0, &V18124_289_86899, &P18124_288};	/*Property = CssImages*/
const Value V18124_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_291 = {S13, 0, &V18124_291_57182, &P18124_289};	/*Property = CssMediaQueries*/
const Value V18124_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_292 = {S14, 0, &V18124_292_57185, &P18124_291};	/*Property = CssMinMax*/
const Value V18124_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_293 = {S15, 0, &V18124_293_57188, &P18124_292};	/*Property = CssOverflow*/
const Value V18124_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_294 = {S16, 0, &V18124_294_57191, &P18124_293};	/*Property = CssPosition*/
const Value V18124_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_295 = {S17, 0, &V18124_295_57194, &P18124_294};	/*Property = CssText*/
const Value V18124_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_297 = {S18, 0, &V18124_297_57197, &P18124_295};	/*Property = CssTransforms*/
const Value V18124_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_241 = {S19, 0, &V18124_241_55801, &P18124_297};	/*Property = CssTransitions*/
const Value V18124_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_298 = {S20, 0, &V18124_298_57208, &P18124_241};	/*Property = CssUI*/
const Value V18124_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_300 = {S21, 0, &V18124_300_57217, &P18124_298};	/*Property = DataSet*/
const Value V18124_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_301 = {S22, 0, &V18124_301_57220, &P18124_300};	/*Property = DataUrl*/
const Value V18124_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_302 = {S23, 0, &V18124_302_57223, &P18124_301};	/*Property = DeviceOrientation*/
const Value V18124_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_304 = {S24, 0, &V18124_304_57229, &P18124_302};	/*Property = FileReader*/
const Value V18124_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_305 = {S25, 0, &V18124_305_57232, &P18124_304};	/*Property = FileSaver*/
const Value V18124_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_306 = {S26, 0, &V18124_306_57238, &P18124_305};	/*Property = FileWriter*/
const Value V18124_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_307 = {S27, 0, &V18124_307_57241, &P18124_306};	/*Property = FormData*/
const Value V18124_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_309 = {S28, 0, &V18124_309_57247, &P18124_307};	/*Property = Fullscreen*/
const Value V18124_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_310 = {S29, 0, &V18124_310_57250, &P18124_309};	/*Property = GeoLocation*/
const Value V18124_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_312 = {S30, 0, &V18124_312_57256, &P18124_310};	/*Property = History*/
const Value V18124_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_314 = {S31, 0, &V18124_314_57262, &P18124_312};	/*Property = Html5*/
const Value V18124_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_313 = {S32, 0, &V18124_313_57259, &P18124_314};	/*Property = Html-Media-Capture*/
const Value V18124_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_315 = {S33, 0, &V18124_315_57265, &P18124_313};	/*Property = Iframe*/
const Value V18124_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_316 = {S34, 0, &V18124_316_57268, &P18124_315};	/*Property = IndexedDB*/
const Value V18124_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_317 = {S35, 0, &V18124_317_57271, &P18124_316};	/*Property = Json*/
const Value V18124_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18124_15 = {S37, 0, &V18124_15_31, &P18124_317};	/*Property = LayoutEngine*/
const Value V18124_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_344 = {S38, 0, &V18124_344_57274, &P18124_15};	/*Property = Masking*/
const Value V18124_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_320 = {S39, 0, &V18124_320_57292, &P18124_344};	/*Property = PostMessage*/
const Value V18124_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_321 = {S40, 0, &V18124_321_57295, &P18124_320};	/*Property = Progress*/
const Value V18124_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_322 = {S41, 0, &V18124_322_57298, &P18124_321};	/*Property = Prompts*/
const Value V18124_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_323 = {S42, 0, &V18124_323_57301, &P18124_322};	/*Property = Selector*/
const Value V18124_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_324 = {S43, 0, &V18124_324_57307, &P18124_323};	/*Property = Svg*/
const Value V18124_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_327 = {S44, 0, &V18124_327_57316, &P18124_324};	/*Property = TouchEvents*/
const Value V18124_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_328 = {S45, 0, &V18124_328_57319, &P18124_327};	/*Property = Track*/
const Value V18124_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_329 = {S46, 0, &V18124_329_57325, &P18124_328};	/*Property = Video*/
const Value V18124_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_330 = {S47, 0, &V18124_330_57328, &P18124_329};	/*Property = Viewport*/
const Value V18124_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_332 = {S48, 0, &V18124_332_57340, &P18124_330};	/*Property = WebWorkers*/
const Value V18124_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18124_333 = {S49, 0, &V18124_333_57343, &P18124_332};	/*Property = Xhr2*/
const Profile P18124 = {"18124", &P18124_333};	/*Profile ID - 18124*/
const Value V18269_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_279 = {S1, 0, &V18269_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18269_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_282 = {S3, 0, &V18269_282_57155, &P18269_279};	/*Property = BlobBuilder*/
const Value V18269_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_350 = {S4, 0, &V18269_350_57446, &P18269_282};	/*Property = Canvas*/
const Value V18269_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_283 = {S5, 0, &V18269_283_57161, &P18269_350};	/*Property = CssBackground*/
const Value V18269_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_284 = {S6, 0, &V18269_284_57164, &P18269_283};	/*Property = CssBorderImage*/
const Value V18269_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_285 = {S7, 0, &V18269_285_57167, &P18269_284};	/*Property = CssCanvas*/
const Value V18269_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_286 = {S8, 0, &V18269_286_57170, &P18269_285};	/*Property = CssColor*/
const Value V18269_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_237 = {S9, 0, &V18269_237_55786, &P18269_286};	/*Property = CssColumn*/
const Value V18269_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_287 = {S10, 0, &V18269_287_57173, &P18269_237};	/*Property = CssFlexbox*/
const Value V18269_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_288 = {S11, 0, &V18269_288_57176, &P18269_287};	/*Property = CssFont*/
const Value V18269_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_289 = {S12, 0, &V18269_289_86899, &P18269_288};	/*Property = CssImages*/
const Value V18269_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_291 = {S13, 0, &V18269_291_57182, &P18269_289};	/*Property = CssMediaQueries*/
const Value V18269_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_292 = {S14, 0, &V18269_292_57185, &P18269_291};	/*Property = CssMinMax*/
const Value V18269_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_293 = {S15, 0, &V18269_293_57188, &P18269_292};	/*Property = CssOverflow*/
const Value V18269_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_294 = {S16, 0, &V18269_294_57191, &P18269_293};	/*Property = CssPosition*/
const Value V18269_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_295 = {S17, 0, &V18269_295_57194, &P18269_294};	/*Property = CssText*/
const Value V18269_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_297 = {S18, 0, &V18269_297_57197, &P18269_295};	/*Property = CssTransforms*/
const Value V18269_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_241 = {S19, 0, &V18269_241_55801, &P18269_297};	/*Property = CssTransitions*/
const Value V18269_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_298 = {S20, 0, &V18269_298_57208, &P18269_241};	/*Property = CssUI*/
const Value V18269_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_300 = {S21, 0, &V18269_300_57217, &P18269_298};	/*Property = DataSet*/
const Value V18269_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_301 = {S22, 0, &V18269_301_57220, &P18269_300};	/*Property = DataUrl*/
const Value V18269_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_302 = {S23, 0, &V18269_302_57223, &P18269_301};	/*Property = DeviceOrientation*/
const Value V18269_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_304 = {S24, 0, &V18269_304_57229, &P18269_302};	/*Property = FileReader*/
const Value V18269_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_305 = {S25, 0, &V18269_305_57232, &P18269_304};	/*Property = FileSaver*/
const Value V18269_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_306 = {S26, 0, &V18269_306_57238, &P18269_305};	/*Property = FileWriter*/
const Value V18269_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_307 = {S27, 0, &V18269_307_57241, &P18269_306};	/*Property = FormData*/
const Value V18269_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_309 = {S28, 0, &V18269_309_57247, &P18269_307};	/*Property = Fullscreen*/
const Value V18269_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_310 = {S29, 0, &V18269_310_57250, &P18269_309};	/*Property = GeoLocation*/
const Value V18269_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_312 = {S30, 0, &V18269_312_57256, &P18269_310};	/*Property = History*/
const Value V18269_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_314 = {S31, 0, &V18269_314_57262, &P18269_312};	/*Property = Html5*/
const Value V18269_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_313 = {S32, 0, &V18269_313_57259, &P18269_314};	/*Property = Html-Media-Capture*/
const Value V18269_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_315 = {S33, 0, &V18269_315_57265, &P18269_313};	/*Property = Iframe*/
const Value V18269_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_316 = {S34, 0, &V18269_316_57268, &P18269_315};	/*Property = IndexedDB*/
const Value V18269_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_317 = {S35, 0, &V18269_317_57271, &P18269_316};	/*Property = Json*/
const Value V18269_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18269_15 = {S37, 0, &V18269_15_31, &P18269_317};	/*Property = LayoutEngine*/
const Value V18269_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_344 = {S38, 0, &V18269_344_57274, &P18269_15};	/*Property = Masking*/
const Value V18269_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_320 = {S39, 0, &V18269_320_57292, &P18269_344};	/*Property = PostMessage*/
const Value V18269_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_321 = {S40, 0, &V18269_321_57295, &P18269_320};	/*Property = Progress*/
const Value V18269_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_322 = {S41, 0, &V18269_322_57298, &P18269_321};	/*Property = Prompts*/
const Value V18269_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_323 = {S42, 0, &V18269_323_57301, &P18269_322};	/*Property = Selector*/
const Value V18269_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_324 = {S43, 0, &V18269_324_57307, &P18269_323};	/*Property = Svg*/
const Value V18269_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_327 = {S44, 0, &V18269_327_57316, &P18269_324};	/*Property = TouchEvents*/
const Value V18269_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_328 = {S45, 0, &V18269_328_57319, &P18269_327};	/*Property = Track*/
const Value V18269_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_329 = {S46, 0, &V18269_329_57325, &P18269_328};	/*Property = Video*/
const Value V18269_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_330 = {S47, 0, &V18269_330_57328, &P18269_329};	/*Property = Viewport*/
const Value V18269_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_332 = {S48, 0, &V18269_332_57340, &P18269_330};	/*Property = WebWorkers*/
const Value V18269_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18269_333 = {S49, 0, &V18269_333_57343, &P18269_332};	/*Property = Xhr2*/
const Profile P18269 = {"18269", &P18269_333};	/*Profile ID - 18269*/
const Value V17335_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17335_279 = {S1, 0, &V17335_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17335_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17335_282 = {S3, 0, &V17335_282_57154, &P17335_279};	/*Property = BlobBuilder*/
const Value V17335_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17335_350 = {S4, 0, &V17335_350_57444, &P17335_282};	/*Property = Canvas*/
const Value V17335_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17335_283 = {S5, 0, &V17335_283_57159, &P17335_350};	/*Property = CssBackground*/
const Value V17335_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17335_284 = {S6, 0, &V17335_284_57163, &P17335_283};	/*Property = CssBorderImage*/
const Value V17335_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17335_285 = {S7, 0, &V17335_285_57166, &P17335_284};	/*Property = CssCanvas*/
const Value V17335_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17335_286 = {S8, 0, &V17335_286_57168, &P17335_285};	/*Property = CssColor*/
const Value V17335_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17335_237 = {S9, 0, &V17335_237_55786, &P17335_286};	/*Property = CssColumn*/
const Value V17335_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17335_287 = {S10, 0, &V17335_287_57172, &P17335_237};	/*Property = CssFlexbox*/
const Value V17335_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17335_288 = {S11, 0, &V17335_288_57175, &P17335_287};	/*Property = CssFont*/
const Value V17335_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17335_289 = {S12, 0, &V17335_289_57759, &P17335_288};	/*Property = CssImages*/
const Value V17335_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17335_291 = {S13, 0, &V17335_291_57181, &P17335_289};	/*Property = CssMediaQueries*/
const Value V17335_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17335_292 = {S14, 0, &V17335_292_57183, &P17335_291};	/*Property = CssMinMax*/
const Value V17335_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17335_293 = {S15, 0, &V17335_293_57187, &P17335_292};	/*Property = CssOverflow*/
const Value V17335_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17335_294 = {S16, 0, &V17335_294_57189, &P17335_293};	/*Property = CssPosition*/
const Value V17335_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17335_295 = {S17, 0, &V17335_295_57192, &P17335_294};	/*Property = CssText*/
const Value V17335_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17335_297 = {S18, 0, &V17335_297_57195, &P17335_295};	/*Property = CssTransforms*/
const Value V17335_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17335_241 = {S19, 0, &V17335_241_55799, &P17335_297};	/*Property = CssTransitions*/
const Value V17335_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17335_298 = {S20, 0, &V17335_298_57206, &P17335_241};	/*Property = CssUI*/
const Value V17335_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17335_300 = {S21, 0, &V17335_300_57216, &P17335_298};	/*Property = DataSet*/
const Value V17335_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17335_301 = {S22, 0, &V17335_301_57218, &P17335_300};	/*Property = DataUrl*/
const Value V17335_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17335_302 = {S23, 0, &V17335_302_57222, &P17335_301};	/*Property = DeviceOrientation*/
const Value V17335_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17335_304 = {S24, 0, &V17335_304_57228, &P17335_302};	/*Property = FileReader*/
const Value V17335_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17335_305 = {S25, 0, &V17335_305_57231, &P17335_304};	/*Property = FileSaver*/
const Value V17335_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17335_306 = {S26, 0, &V17335_306_57237, &P17335_305};	/*Property = FileWriter*/
const Value V17335_307_57240 = {S0, NULL};	/*Value = False*/
const Property P17335_307 = {S27, 0, &V17335_307_57240, &P17335_306};	/*Property = FormData*/
const Value V17335_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17335_309 = {S28, 0, &V17335_309_57246, &P17335_307};	/*Property = Fullscreen*/
const Value V17335_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17335_310 = {S29, 0, &V17335_310_57248, &P17335_309};	/*Property = GeoLocation*/
const Value V17335_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17335_312 = {S30, 0, &V17335_312_57255, &P17335_310};	/*Property = History*/
const Value V17335_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17335_314 = {S31, 0, &V17335_314_57261, &P17335_312};	/*Property = Html5*/
const Value V17335_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17335_313 = {S32, 0, &V17335_313_57258, &P17335_314};	/*Property = Html-Media-Capture*/
const Value V17335_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17335_315 = {S33, 0, &V17335_315_57264, &P17335_313};	/*Property = Iframe*/
const Value V17335_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17335_316 = {S34, 0, &V17335_316_57267, &P17335_315};	/*Property = IndexedDB*/
const Value V17335_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17335_317 = {S35, 0, &V17335_317_57269, &P17335_316};	/*Property = Json*/
const Value V17335_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17335_15 = {S37, 0, &V17335_15_28, &P17335_317};	/*Property = LayoutEngine*/
const Value V17335_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17335_344 = {S38, 0, &V17335_344_57272, &P17335_15};	/*Property = Masking*/
const Value V17335_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17335_320 = {S39, 0, &V17335_320_57290, &P17335_344};	/*Property = PostMessage*/
const Value V17335_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17335_321 = {S40, 0, &V17335_321_57293, &P17335_320};	/*Property = Progress*/
const Value V17335_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17335_322 = {S41, 0, &V17335_322_57296, &P17335_321};	/*Property = Prompts*/
const Value V17335_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17335_323 = {S42, 0, &V17335_323_57299, &P17335_322};	/*Property = Selector*/
const Value V17335_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17335_324 = {S43, 0, &V17335_324_57306, &P17335_323};	/*Property = Svg*/
const Value V17335_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17335_327 = {S44, 0, &V17335_327_57315, &P17335_324};	/*Property = TouchEvents*/
const Value V17335_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17335_328 = {S45, 0, &V17335_328_57318, &P17335_327};	/*Property = Track*/
const Value V17335_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17335_329 = {S46, 0, &V17335_329_57323, &P17335_328};	/*Property = Video*/
const Value V17335_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17335_330 = {S47, 0, &V17335_330_57326, &P17335_329};	/*Property = Viewport*/
const Value V17335_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17335_332 = {S48, 0, &V17335_332_57339, &P17335_330};	/*Property = WebWorkers*/
const Value V17335_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17335_333 = {S49, 0, &V17335_333_57342, &P17335_332};	/*Property = Xhr2*/
const Profile P17335 = {"17335", &P17335_333};	/*Profile ID - 17335*/
const Value V22442_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_279 = {S1, 0, &V22442_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22442_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_282 = {S3, 0, &V22442_282_57155, &P22442_279};	/*Property = BlobBuilder*/
const Value V22442_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_350 = {S4, 0, &V22442_350_57446, &P22442_282};	/*Property = Canvas*/
const Value V22442_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_283 = {S5, 0, &V22442_283_57161, &P22442_350};	/*Property = CssBackground*/
const Value V22442_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_284 = {S6, 0, &V22442_284_57164, &P22442_283};	/*Property = CssBorderImage*/
const Value V22442_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_285 = {S7, 0, &V22442_285_57167, &P22442_284};	/*Property = CssCanvas*/
const Value V22442_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_286 = {S8, 0, &V22442_286_57170, &P22442_285};	/*Property = CssColor*/
const Value V22442_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_237 = {S9, 0, &V22442_237_55786, &P22442_286};	/*Property = CssColumn*/
const Value V22442_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_287 = {S10, 0, &V22442_287_57173, &P22442_237};	/*Property = CssFlexbox*/
const Value V22442_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_288 = {S11, 0, &V22442_288_57176, &P22442_287};	/*Property = CssFont*/
const Value V22442_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_289 = {S12, 0, &V22442_289_86899, &P22442_288};	/*Property = CssImages*/
const Value V22442_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_291 = {S13, 0, &V22442_291_57182, &P22442_289};	/*Property = CssMediaQueries*/
const Value V22442_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_292 = {S14, 0, &V22442_292_57185, &P22442_291};	/*Property = CssMinMax*/
const Value V22442_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_293 = {S15, 0, &V22442_293_57188, &P22442_292};	/*Property = CssOverflow*/
const Value V22442_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_294 = {S16, 0, &V22442_294_57191, &P22442_293};	/*Property = CssPosition*/
const Value V22442_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_295 = {S17, 0, &V22442_295_57194, &P22442_294};	/*Property = CssText*/
const Value V22442_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_297 = {S18, 0, &V22442_297_57197, &P22442_295};	/*Property = CssTransforms*/
const Value V22442_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_241 = {S19, 0, &V22442_241_55801, &P22442_297};	/*Property = CssTransitions*/
const Value V22442_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_298 = {S20, 0, &V22442_298_57208, &P22442_241};	/*Property = CssUI*/
const Value V22442_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_300 = {S21, 0, &V22442_300_57217, &P22442_298};	/*Property = DataSet*/
const Value V22442_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_301 = {S22, 0, &V22442_301_57220, &P22442_300};	/*Property = DataUrl*/
const Value V22442_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_302 = {S23, 0, &V22442_302_57223, &P22442_301};	/*Property = DeviceOrientation*/
const Value V22442_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_304 = {S24, 0, &V22442_304_57229, &P22442_302};	/*Property = FileReader*/
const Value V22442_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_305 = {S25, 0, &V22442_305_57232, &P22442_304};	/*Property = FileSaver*/
const Value V22442_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_306 = {S26, 0, &V22442_306_57238, &P22442_305};	/*Property = FileWriter*/
const Value V22442_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_307 = {S27, 0, &V22442_307_57241, &P22442_306};	/*Property = FormData*/
const Value V22442_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_309 = {S28, 0, &V22442_309_57247, &P22442_307};	/*Property = Fullscreen*/
const Value V22442_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_310 = {S29, 0, &V22442_310_57250, &P22442_309};	/*Property = GeoLocation*/
const Value V22442_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_312 = {S30, 0, &V22442_312_57256, &P22442_310};	/*Property = History*/
const Value V22442_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_314 = {S31, 0, &V22442_314_57262, &P22442_312};	/*Property = Html5*/
const Value V22442_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_313 = {S32, 0, &V22442_313_57259, &P22442_314};	/*Property = Html-Media-Capture*/
const Value V22442_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_315 = {S33, 0, &V22442_315_57265, &P22442_313};	/*Property = Iframe*/
const Value V22442_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_316 = {S34, 0, &V22442_316_57268, &P22442_315};	/*Property = IndexedDB*/
const Value V22442_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_317 = {S35, 0, &V22442_317_57271, &P22442_316};	/*Property = Json*/
const Value V22442_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22442_15 = {S37, 0, &V22442_15_28, &P22442_317};	/*Property = LayoutEngine*/
const Value V22442_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_344 = {S38, 0, &V22442_344_57274, &P22442_15};	/*Property = Masking*/
const Value V22442_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_320 = {S39, 0, &V22442_320_57292, &P22442_344};	/*Property = PostMessage*/
const Value V22442_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_321 = {S40, 0, &V22442_321_57295, &P22442_320};	/*Property = Progress*/
const Value V22442_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_322 = {S41, 0, &V22442_322_57298, &P22442_321};	/*Property = Prompts*/
const Value V22442_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_323 = {S42, 0, &V22442_323_57301, &P22442_322};	/*Property = Selector*/
const Value V22442_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_324 = {S43, 0, &V22442_324_57307, &P22442_323};	/*Property = Svg*/
const Value V22442_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_327 = {S44, 0, &V22442_327_57316, &P22442_324};	/*Property = TouchEvents*/
const Value V22442_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_328 = {S45, 0, &V22442_328_57319, &P22442_327};	/*Property = Track*/
const Value V22442_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_329 = {S46, 0, &V22442_329_57325, &P22442_328};	/*Property = Video*/
const Value V22442_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_330 = {S47, 0, &V22442_330_57328, &P22442_329};	/*Property = Viewport*/
const Value V22442_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_332 = {S48, 0, &V22442_332_57340, &P22442_330};	/*Property = WebWorkers*/
const Value V22442_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22442_333 = {S49, 0, &V22442_333_57343, &P22442_332};	/*Property = Xhr2*/
const Profile P22442 = {"22442", &P22442_333};	/*Profile ID - 22442*/
const Value V17884_1_1 = {S2, NULL};	/*Value = True*/
const Property P17884_1 = {S50, 0, &V17884_1_1, NULL};	/*Property = IsMobile*/
const Value V17884_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P17884_5 = {S52, 0, &V17884_5_1233, &P17884_1};	/*Property = ScreenPixelsHeight*/
const Value V17884_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P17884_4 = {S54, 0, &V17884_4_1235, &P17884_5};	/*Property = ScreenPixelsWidth*/
const Profile P17884 = {"17884", &P17884_4};	/*Profile ID - 17884*/
const Profile P18163 = {"18163", NULL};	/*Profile ID - 18163*/
const Value V17389_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_279 = {S1, 0, &V17389_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17389_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_282 = {S3, 0, &V17389_282_57155, &P17389_279};	/*Property = BlobBuilder*/
const Value V17389_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_350 = {S4, 0, &V17389_350_57446, &P17389_282};	/*Property = Canvas*/
const Value V17389_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_283 = {S5, 0, &V17389_283_57161, &P17389_350};	/*Property = CssBackground*/
const Value V17389_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_284 = {S6, 0, &V17389_284_57164, &P17389_283};	/*Property = CssBorderImage*/
const Value V17389_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_285 = {S7, 0, &V17389_285_57167, &P17389_284};	/*Property = CssCanvas*/
const Value V17389_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_286 = {S8, 0, &V17389_286_57170, &P17389_285};	/*Property = CssColor*/
const Value V17389_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_237 = {S9, 0, &V17389_237_55786, &P17389_286};	/*Property = CssColumn*/
const Value V17389_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_287 = {S10, 0, &V17389_287_57173, &P17389_237};	/*Property = CssFlexbox*/
const Value V17389_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_288 = {S11, 0, &V17389_288_57176, &P17389_287};	/*Property = CssFont*/
const Value V17389_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_289 = {S12, 0, &V17389_289_86899, &P17389_288};	/*Property = CssImages*/
const Value V17389_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_291 = {S13, 0, &V17389_291_57182, &P17389_289};	/*Property = CssMediaQueries*/
const Value V17389_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_292 = {S14, 0, &V17389_292_57185, &P17389_291};	/*Property = CssMinMax*/
const Value V17389_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_293 = {S15, 0, &V17389_293_57188, &P17389_292};	/*Property = CssOverflow*/
const Value V17389_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_294 = {S16, 0, &V17389_294_57191, &P17389_293};	/*Property = CssPosition*/
const Value V17389_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_295 = {S17, 0, &V17389_295_57194, &P17389_294};	/*Property = CssText*/
const Value V17389_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_297 = {S18, 0, &V17389_297_57197, &P17389_295};	/*Property = CssTransforms*/
const Value V17389_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_241 = {S19, 0, &V17389_241_55801, &P17389_297};	/*Property = CssTransitions*/
const Value V17389_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_298 = {S20, 0, &V17389_298_57208, &P17389_241};	/*Property = CssUI*/
const Value V17389_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_300 = {S21, 0, &V17389_300_57217, &P17389_298};	/*Property = DataSet*/
const Value V17389_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_301 = {S22, 0, &V17389_301_57220, &P17389_300};	/*Property = DataUrl*/
const Value V17389_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_302 = {S23, 0, &V17389_302_57223, &P17389_301};	/*Property = DeviceOrientation*/
const Value V17389_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_304 = {S24, 0, &V17389_304_57229, &P17389_302};	/*Property = FileReader*/
const Value V17389_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_305 = {S25, 0, &V17389_305_57232, &P17389_304};	/*Property = FileSaver*/
const Value V17389_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_306 = {S26, 0, &V17389_306_57238, &P17389_305};	/*Property = FileWriter*/
const Value V17389_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_307 = {S27, 0, &V17389_307_57241, &P17389_306};	/*Property = FormData*/
const Value V17389_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_309 = {S28, 0, &V17389_309_57247, &P17389_307};	/*Property = Fullscreen*/
const Value V17389_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_310 = {S29, 0, &V17389_310_57250, &P17389_309};	/*Property = GeoLocation*/
const Value V17389_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_312 = {S30, 0, &V17389_312_57256, &P17389_310};	/*Property = History*/
const Value V17389_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_314 = {S31, 0, &V17389_314_57262, &P17389_312};	/*Property = Html5*/
const Value V17389_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_313 = {S32, 0, &V17389_313_57259, &P17389_314};	/*Property = Html-Media-Capture*/
const Value V17389_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_315 = {S33, 0, &V17389_315_57265, &P17389_313};	/*Property = Iframe*/
const Value V17389_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_316 = {S34, 0, &V17389_316_57268, &P17389_315};	/*Property = IndexedDB*/
const Value V17389_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_317 = {S35, 0, &V17389_317_57271, &P17389_316};	/*Property = Json*/
const Value V17389_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P17389_15 = {S37, 0, &V17389_15_54167, &P17389_317};	/*Property = LayoutEngine*/
const Value V17389_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_344 = {S38, 0, &V17389_344_57274, &P17389_15};	/*Property = Masking*/
const Value V17389_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_320 = {S39, 0, &V17389_320_57292, &P17389_344};	/*Property = PostMessage*/
const Value V17389_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_321 = {S40, 0, &V17389_321_57295, &P17389_320};	/*Property = Progress*/
const Value V17389_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_322 = {S41, 0, &V17389_322_57298, &P17389_321};	/*Property = Prompts*/
const Value V17389_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_323 = {S42, 0, &V17389_323_57301, &P17389_322};	/*Property = Selector*/
const Value V17389_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_324 = {S43, 0, &V17389_324_57307, &P17389_323};	/*Property = Svg*/
const Value V17389_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_327 = {S44, 0, &V17389_327_57316, &P17389_324};	/*Property = TouchEvents*/
const Value V17389_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_328 = {S45, 0, &V17389_328_57319, &P17389_327};	/*Property = Track*/
const Value V17389_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_329 = {S46, 0, &V17389_329_57325, &P17389_328};	/*Property = Video*/
const Value V17389_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_330 = {S47, 0, &V17389_330_57328, &P17389_329};	/*Property = Viewport*/
const Value V17389_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_332 = {S48, 0, &V17389_332_57340, &P17389_330};	/*Property = WebWorkers*/
const Value V17389_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17389_333 = {S49, 0, &V17389_333_57343, &P17389_332};	/*Property = Xhr2*/
const Profile P17389 = {"17389", &P17389_333};	/*Profile ID - 17389*/
const Profile P19831 = {"19831", NULL};	/*Profile ID - 19831*/
const Value V18031_1_1 = {S2, NULL};	/*Value = True*/
const Property P18031_1 = {S50, 0, &V18031_1_1, NULL};	/*Property = IsMobile*/
const Value V18031_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18031_5 = {S52, 0, &V18031_5_1222, &P18031_1};	/*Property = ScreenPixelsHeight*/
const Value V18031_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18031_4 = {S54, 0, &V18031_4_1224, &P18031_5};	/*Property = ScreenPixelsWidth*/
const Profile P18031 = {"18031", &P18031_4};	/*Profile ID - 18031*/
const Value V18030_1_1 = {S2, NULL};	/*Value = True*/
const Property P18030_1 = {S50, 0, &V18030_1_1, NULL};	/*Property = IsMobile*/
const Value V18030_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18030_5 = {S52, 0, &V18030_5_1227, &P18030_1};	/*Property = ScreenPixelsHeight*/
const Value V18030_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18030_4 = {S54, 0, &V18030_4_1229, &P18030_5};	/*Property = ScreenPixelsWidth*/
const Profile P18030 = {"18030", &P18030_4};	/*Profile ID - 18030*/
const Value V18355_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_279 = {S1, 0, &V18355_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18355_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_282 = {S3, 0, &V18355_282_57155, &P18355_279};	/*Property = BlobBuilder*/
const Value V18355_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_350 = {S4, 0, &V18355_350_57446, &P18355_282};	/*Property = Canvas*/
const Value V18355_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_283 = {S5, 0, &V18355_283_57161, &P18355_350};	/*Property = CssBackground*/
const Value V18355_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_284 = {S6, 0, &V18355_284_57164, &P18355_283};	/*Property = CssBorderImage*/
const Value V18355_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_285 = {S7, 0, &V18355_285_57167, &P18355_284};	/*Property = CssCanvas*/
const Value V18355_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_286 = {S8, 0, &V18355_286_57170, &P18355_285};	/*Property = CssColor*/
const Value V18355_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_237 = {S9, 0, &V18355_237_55786, &P18355_286};	/*Property = CssColumn*/
const Value V18355_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_287 = {S10, 0, &V18355_287_57173, &P18355_237};	/*Property = CssFlexbox*/
const Value V18355_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_288 = {S11, 0, &V18355_288_57176, &P18355_287};	/*Property = CssFont*/
const Value V18355_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_289 = {S12, 0, &V18355_289_86899, &P18355_288};	/*Property = CssImages*/
const Value V18355_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_291 = {S13, 0, &V18355_291_57182, &P18355_289};	/*Property = CssMediaQueries*/
const Value V18355_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_292 = {S14, 0, &V18355_292_57185, &P18355_291};	/*Property = CssMinMax*/
const Value V18355_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_293 = {S15, 0, &V18355_293_57188, &P18355_292};	/*Property = CssOverflow*/
const Value V18355_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_294 = {S16, 0, &V18355_294_57191, &P18355_293};	/*Property = CssPosition*/
const Value V18355_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_295 = {S17, 0, &V18355_295_57194, &P18355_294};	/*Property = CssText*/
const Value V18355_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_297 = {S18, 0, &V18355_297_57197, &P18355_295};	/*Property = CssTransforms*/
const Value V18355_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_241 = {S19, 0, &V18355_241_55801, &P18355_297};	/*Property = CssTransitions*/
const Value V18355_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_298 = {S20, 0, &V18355_298_57208, &P18355_241};	/*Property = CssUI*/
const Value V18355_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_300 = {S21, 0, &V18355_300_57217, &P18355_298};	/*Property = DataSet*/
const Value V18355_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_301 = {S22, 0, &V18355_301_57220, &P18355_300};	/*Property = DataUrl*/
const Value V18355_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_302 = {S23, 0, &V18355_302_57223, &P18355_301};	/*Property = DeviceOrientation*/
const Value V18355_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_304 = {S24, 0, &V18355_304_57229, &P18355_302};	/*Property = FileReader*/
const Value V18355_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_305 = {S25, 0, &V18355_305_57232, &P18355_304};	/*Property = FileSaver*/
const Value V18355_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_306 = {S26, 0, &V18355_306_57238, &P18355_305};	/*Property = FileWriter*/
const Value V18355_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_307 = {S27, 0, &V18355_307_57241, &P18355_306};	/*Property = FormData*/
const Value V18355_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_309 = {S28, 0, &V18355_309_57247, &P18355_307};	/*Property = Fullscreen*/
const Value V18355_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_310 = {S29, 0, &V18355_310_57250, &P18355_309};	/*Property = GeoLocation*/
const Value V18355_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_312 = {S30, 0, &V18355_312_57256, &P18355_310};	/*Property = History*/
const Value V18355_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_314 = {S31, 0, &V18355_314_57262, &P18355_312};	/*Property = Html5*/
const Value V18355_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_313 = {S32, 0, &V18355_313_57259, &P18355_314};	/*Property = Html-Media-Capture*/
const Value V18355_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_315 = {S33, 0, &V18355_315_57265, &P18355_313};	/*Property = Iframe*/
const Value V18355_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_316 = {S34, 0, &V18355_316_57268, &P18355_315};	/*Property = IndexedDB*/
const Value V18355_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_317 = {S35, 0, &V18355_317_57271, &P18355_316};	/*Property = Json*/
const Value V18355_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P18355_15 = {S37, 0, &V18355_15_30, &P18355_317};	/*Property = LayoutEngine*/
const Value V18355_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_344 = {S38, 0, &V18355_344_57274, &P18355_15};	/*Property = Masking*/
const Value V18355_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_320 = {S39, 0, &V18355_320_57292, &P18355_344};	/*Property = PostMessage*/
const Value V18355_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_321 = {S40, 0, &V18355_321_57295, &P18355_320};	/*Property = Progress*/
const Value V18355_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_322 = {S41, 0, &V18355_322_57298, &P18355_321};	/*Property = Prompts*/
const Value V18355_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_323 = {S42, 0, &V18355_323_57301, &P18355_322};	/*Property = Selector*/
const Value V18355_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_324 = {S43, 0, &V18355_324_57307, &P18355_323};	/*Property = Svg*/
const Value V18355_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_327 = {S44, 0, &V18355_327_57316, &P18355_324};	/*Property = TouchEvents*/
const Value V18355_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_328 = {S45, 0, &V18355_328_57319, &P18355_327};	/*Property = Track*/
const Value V18355_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_329 = {S46, 0, &V18355_329_57325, &P18355_328};	/*Property = Video*/
const Value V18355_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_330 = {S47, 0, &V18355_330_57328, &P18355_329};	/*Property = Viewport*/
const Value V18355_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_332 = {S48, 0, &V18355_332_57340, &P18355_330};	/*Property = WebWorkers*/
const Value V18355_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18355_333 = {S49, 0, &V18355_333_57343, &P18355_332};	/*Property = Xhr2*/
const Profile P18355 = {"18355", &P18355_333};	/*Profile ID - 18355*/
const Value V21929_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_279 = {S1, 0, &V21929_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21929_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_282 = {S3, 0, &V21929_282_57155, &P21929_279};	/*Property = BlobBuilder*/
const Value V21929_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_350 = {S4, 0, &V21929_350_57446, &P21929_282};	/*Property = Canvas*/
const Value V21929_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_283 = {S5, 0, &V21929_283_57161, &P21929_350};	/*Property = CssBackground*/
const Value V21929_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_284 = {S6, 0, &V21929_284_57164, &P21929_283};	/*Property = CssBorderImage*/
const Value V21929_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_285 = {S7, 0, &V21929_285_57167, &P21929_284};	/*Property = CssCanvas*/
const Value V21929_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_286 = {S8, 0, &V21929_286_57170, &P21929_285};	/*Property = CssColor*/
const Value V21929_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_237 = {S9, 0, &V21929_237_55786, &P21929_286};	/*Property = CssColumn*/
const Value V21929_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_287 = {S10, 0, &V21929_287_57173, &P21929_237};	/*Property = CssFlexbox*/
const Value V21929_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_288 = {S11, 0, &V21929_288_57176, &P21929_287};	/*Property = CssFont*/
const Value V21929_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_289 = {S12, 0, &V21929_289_86899, &P21929_288};	/*Property = CssImages*/
const Value V21929_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_291 = {S13, 0, &V21929_291_57182, &P21929_289};	/*Property = CssMediaQueries*/
const Value V21929_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_292 = {S14, 0, &V21929_292_57185, &P21929_291};	/*Property = CssMinMax*/
const Value V21929_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_293 = {S15, 0, &V21929_293_57188, &P21929_292};	/*Property = CssOverflow*/
const Value V21929_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_294 = {S16, 0, &V21929_294_57191, &P21929_293};	/*Property = CssPosition*/
const Value V21929_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_295 = {S17, 0, &V21929_295_57194, &P21929_294};	/*Property = CssText*/
const Value V21929_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_297 = {S18, 0, &V21929_297_57197, &P21929_295};	/*Property = CssTransforms*/
const Value V21929_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_241 = {S19, 0, &V21929_241_55801, &P21929_297};	/*Property = CssTransitions*/
const Value V21929_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_298 = {S20, 0, &V21929_298_57208, &P21929_241};	/*Property = CssUI*/
const Value V21929_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_300 = {S21, 0, &V21929_300_57217, &P21929_298};	/*Property = DataSet*/
const Value V21929_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_301 = {S22, 0, &V21929_301_57220, &P21929_300};	/*Property = DataUrl*/
const Value V21929_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_302 = {S23, 0, &V21929_302_57223, &P21929_301};	/*Property = DeviceOrientation*/
const Value V21929_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_304 = {S24, 0, &V21929_304_57229, &P21929_302};	/*Property = FileReader*/
const Value V21929_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_305 = {S25, 0, &V21929_305_57232, &P21929_304};	/*Property = FileSaver*/
const Value V21929_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_306 = {S26, 0, &V21929_306_57238, &P21929_305};	/*Property = FileWriter*/
const Value V21929_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_307 = {S27, 0, &V21929_307_57241, &P21929_306};	/*Property = FormData*/
const Value V21929_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_309 = {S28, 0, &V21929_309_57247, &P21929_307};	/*Property = Fullscreen*/
const Value V21929_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_310 = {S29, 0, &V21929_310_57250, &P21929_309};	/*Property = GeoLocation*/
const Value V21929_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_312 = {S30, 0, &V21929_312_57256, &P21929_310};	/*Property = History*/
const Value V21929_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_314 = {S31, 0, &V21929_314_57262, &P21929_312};	/*Property = Html5*/
const Value V21929_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_313 = {S32, 0, &V21929_313_57259, &P21929_314};	/*Property = Html-Media-Capture*/
const Value V21929_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_315 = {S33, 0, &V21929_315_57265, &P21929_313};	/*Property = Iframe*/
const Value V21929_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_316 = {S34, 0, &V21929_316_57268, &P21929_315};	/*Property = IndexedDB*/
const Value V21929_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_317 = {S35, 0, &V21929_317_57271, &P21929_316};	/*Property = Json*/
const Value V21929_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21929_15 = {S37, 0, &V21929_15_28, &P21929_317};	/*Property = LayoutEngine*/
const Value V21929_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_344 = {S38, 0, &V21929_344_57274, &P21929_15};	/*Property = Masking*/
const Value V21929_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_320 = {S39, 0, &V21929_320_57292, &P21929_344};	/*Property = PostMessage*/
const Value V21929_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_321 = {S40, 0, &V21929_321_57295, &P21929_320};	/*Property = Progress*/
const Value V21929_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_322 = {S41, 0, &V21929_322_57298, &P21929_321};	/*Property = Prompts*/
const Value V21929_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_323 = {S42, 0, &V21929_323_57301, &P21929_322};	/*Property = Selector*/
const Value V21929_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_324 = {S43, 0, &V21929_324_57307, &P21929_323};	/*Property = Svg*/
const Value V21929_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_327 = {S44, 0, &V21929_327_57316, &P21929_324};	/*Property = TouchEvents*/
const Value V21929_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_328 = {S45, 0, &V21929_328_57319, &P21929_327};	/*Property = Track*/
const Value V21929_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_329 = {S46, 0, &V21929_329_57325, &P21929_328};	/*Property = Video*/
const Value V21929_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_330 = {S47, 0, &V21929_330_57328, &P21929_329};	/*Property = Viewport*/
const Value V21929_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_332 = {S48, 0, &V21929_332_57340, &P21929_330};	/*Property = WebWorkers*/
const Value V21929_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21929_333 = {S49, 0, &V21929_333_57343, &P21929_332};	/*Property = Xhr2*/
const Profile P21929 = {"21929", &P21929_333};	/*Profile ID - 21929*/
const Value V16305_1_1 = {S2, NULL};	/*Value = True*/
const Property P16305_1 = {S50, 0, &V16305_1_1, NULL};	/*Property = IsMobile*/
const Value V16305_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P16305_5 = {S52, 0, &V16305_5_1380, &P16305_1};	/*Property = ScreenPixelsHeight*/
const Value V16305_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16305_4 = {S54, 0, &V16305_4_1279, &P16305_5};	/*Property = ScreenPixelsWidth*/
const Profile P16305 = {"16305", &P16305_4};	/*Profile ID - 16305*/
const Value V17388_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_279 = {S1, 0, &V17388_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17388_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_282 = {S3, 0, &V17388_282_57155, &P17388_279};	/*Property = BlobBuilder*/
const Value V17388_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_350 = {S4, 0, &V17388_350_57446, &P17388_282};	/*Property = Canvas*/
const Value V17388_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_283 = {S5, 0, &V17388_283_57161, &P17388_350};	/*Property = CssBackground*/
const Value V17388_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_284 = {S6, 0, &V17388_284_57164, &P17388_283};	/*Property = CssBorderImage*/
const Value V17388_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_285 = {S7, 0, &V17388_285_57167, &P17388_284};	/*Property = CssCanvas*/
const Value V17388_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_286 = {S8, 0, &V17388_286_57170, &P17388_285};	/*Property = CssColor*/
const Value V17388_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_237 = {S9, 0, &V17388_237_55786, &P17388_286};	/*Property = CssColumn*/
const Value V17388_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_287 = {S10, 0, &V17388_287_57173, &P17388_237};	/*Property = CssFlexbox*/
const Value V17388_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_288 = {S11, 0, &V17388_288_57176, &P17388_287};	/*Property = CssFont*/
const Value V17388_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_289 = {S12, 0, &V17388_289_86899, &P17388_288};	/*Property = CssImages*/
const Value V17388_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_291 = {S13, 0, &V17388_291_57182, &P17388_289};	/*Property = CssMediaQueries*/
const Value V17388_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_292 = {S14, 0, &V17388_292_57185, &P17388_291};	/*Property = CssMinMax*/
const Value V17388_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_293 = {S15, 0, &V17388_293_57188, &P17388_292};	/*Property = CssOverflow*/
const Value V17388_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_294 = {S16, 0, &V17388_294_57191, &P17388_293};	/*Property = CssPosition*/
const Value V17388_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_295 = {S17, 0, &V17388_295_57194, &P17388_294};	/*Property = CssText*/
const Value V17388_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_297 = {S18, 0, &V17388_297_57197, &P17388_295};	/*Property = CssTransforms*/
const Value V17388_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_241 = {S19, 0, &V17388_241_55801, &P17388_297};	/*Property = CssTransitions*/
const Value V17388_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_298 = {S20, 0, &V17388_298_57208, &P17388_241};	/*Property = CssUI*/
const Value V17388_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_300 = {S21, 0, &V17388_300_57217, &P17388_298};	/*Property = DataSet*/
const Value V17388_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_301 = {S22, 0, &V17388_301_57220, &P17388_300};	/*Property = DataUrl*/
const Value V17388_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_302 = {S23, 0, &V17388_302_57223, &P17388_301};	/*Property = DeviceOrientation*/
const Value V17388_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_304 = {S24, 0, &V17388_304_57229, &P17388_302};	/*Property = FileReader*/
const Value V17388_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_305 = {S25, 0, &V17388_305_57232, &P17388_304};	/*Property = FileSaver*/
const Value V17388_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_306 = {S26, 0, &V17388_306_57238, &P17388_305};	/*Property = FileWriter*/
const Value V17388_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_307 = {S27, 0, &V17388_307_57241, &P17388_306};	/*Property = FormData*/
const Value V17388_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_309 = {S28, 0, &V17388_309_57247, &P17388_307};	/*Property = Fullscreen*/
const Value V17388_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_310 = {S29, 0, &V17388_310_57250, &P17388_309};	/*Property = GeoLocation*/
const Value V17388_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_312 = {S30, 0, &V17388_312_57256, &P17388_310};	/*Property = History*/
const Value V17388_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_314 = {S31, 0, &V17388_314_57262, &P17388_312};	/*Property = Html5*/
const Value V17388_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_313 = {S32, 0, &V17388_313_57259, &P17388_314};	/*Property = Html-Media-Capture*/
const Value V17388_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_315 = {S33, 0, &V17388_315_57265, &P17388_313};	/*Property = Iframe*/
const Value V17388_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_316 = {S34, 0, &V17388_316_57268, &P17388_315};	/*Property = IndexedDB*/
const Value V17388_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_317 = {S35, 0, &V17388_317_57271, &P17388_316};	/*Property = Json*/
const Value V17388_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P17388_15 = {S37, 0, &V17388_15_54167, &P17388_317};	/*Property = LayoutEngine*/
const Value V17388_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_344 = {S38, 0, &V17388_344_57274, &P17388_15};	/*Property = Masking*/
const Value V17388_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_320 = {S39, 0, &V17388_320_57292, &P17388_344};	/*Property = PostMessage*/
const Value V17388_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_321 = {S40, 0, &V17388_321_57295, &P17388_320};	/*Property = Progress*/
const Value V17388_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_322 = {S41, 0, &V17388_322_57298, &P17388_321};	/*Property = Prompts*/
const Value V17388_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_323 = {S42, 0, &V17388_323_57301, &P17388_322};	/*Property = Selector*/
const Value V17388_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_324 = {S43, 0, &V17388_324_57307, &P17388_323};	/*Property = Svg*/
const Value V17388_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_327 = {S44, 0, &V17388_327_57316, &P17388_324};	/*Property = TouchEvents*/
const Value V17388_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_328 = {S45, 0, &V17388_328_57319, &P17388_327};	/*Property = Track*/
const Value V17388_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_329 = {S46, 0, &V17388_329_57325, &P17388_328};	/*Property = Video*/
const Value V17388_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_330 = {S47, 0, &V17388_330_57328, &P17388_329};	/*Property = Viewport*/
const Value V17388_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_332 = {S48, 0, &V17388_332_57340, &P17388_330};	/*Property = WebWorkers*/
const Value V17388_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17388_333 = {S49, 0, &V17388_333_57343, &P17388_332};	/*Property = Xhr2*/
const Profile P17388 = {"17388", &P17388_333};	/*Profile ID - 17388*/
const Value V17889_1_1 = {S2, NULL};	/*Value = True*/
const Property P17889_1 = {S50, 0, &V17889_1_1, NULL};	/*Property = IsMobile*/
const Value V17889_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P17889_5 = {S52, 0, &V17889_5_1333, &P17889_1};	/*Property = ScreenPixelsHeight*/
const Value V17889_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P17889_4 = {S54, 0, &V17889_4_1335, &P17889_5};	/*Property = ScreenPixelsWidth*/
const Profile P17889 = {"17889", &P17889_4};	/*Profile ID - 17889*/
const Value V17893_1_1 = {S2, NULL};	/*Value = True*/
const Property P17893_1 = {S50, 0, &V17893_1_1, NULL};	/*Property = IsMobile*/
const Value V17893_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P17893_5 = {S52, 0, &V17893_5_1333, &P17893_1};	/*Property = ScreenPixelsHeight*/
const Value V17893_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P17893_4 = {S54, 0, &V17893_4_1335, &P17893_5};	/*Property = ScreenPixelsWidth*/
const Profile P17893 = {"17893", &P17893_4};	/*Profile ID - 17893*/
const Value V17981_1_1 = {S2, NULL};	/*Value = True*/
const Property P17981_1 = {S50, 0, &V17981_1_1, NULL};	/*Property = IsMobile*/
const Value V17981_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P17981_5 = {S52, 0, &V17981_5_1294, &P17981_1};	/*Property = ScreenPixelsHeight*/
const Value V17981_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17981_4 = {S54, 0, &V17981_4_1296, &P17981_5};	/*Property = ScreenPixelsWidth*/
const Profile P17981 = {"17981", &P17981_4};	/*Profile ID - 17981*/
const Profile P18165 = {"18165", NULL};	/*Profile ID - 18165*/
const Value V20909_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20909_279 = {S1, 0, &V20909_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20909_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20909_282 = {S3, 0, &V20909_282_57154, &P20909_279};	/*Property = BlobBuilder*/
const Value V20909_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20909_350 = {S4, 0, &V20909_350_57444, &P20909_282};	/*Property = Canvas*/
const Value V20909_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20909_283 = {S5, 0, &V20909_283_57159, &P20909_350};	/*Property = CssBackground*/
const Value V20909_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20909_284 = {S6, 0, &V20909_284_57163, &P20909_283};	/*Property = CssBorderImage*/
const Value V20909_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20909_285 = {S7, 0, &V20909_285_57166, &P20909_284};	/*Property = CssCanvas*/
const Value V20909_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20909_286 = {S8, 0, &V20909_286_57168, &P20909_285};	/*Property = CssColor*/
const Value V20909_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20909_237 = {S9, 0, &V20909_237_55786, &P20909_286};	/*Property = CssColumn*/
const Value V20909_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20909_287 = {S10, 0, &V20909_287_57172, &P20909_237};	/*Property = CssFlexbox*/
const Value V20909_288_57175 = {S0, NULL};	/*Value = False*/
const Property P20909_288 = {S11, 0, &V20909_288_57175, &P20909_287};	/*Property = CssFont*/
const Value V20909_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20909_289 = {S12, 0, &V20909_289_57759, &P20909_288};	/*Property = CssImages*/
const Value V20909_291_57181 = {S0, NULL};	/*Value = False*/
const Property P20909_291 = {S13, 0, &V20909_291_57181, &P20909_289};	/*Property = CssMediaQueries*/
const Value V20909_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20909_292 = {S14, 0, &V20909_292_57183, &P20909_291};	/*Property = CssMinMax*/
const Value V20909_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20909_293 = {S15, 0, &V20909_293_57187, &P20909_292};	/*Property = CssOverflow*/
const Value V20909_294_57190 = {S0, NULL};	/*Value = False*/
const Property P20909_294 = {S16, 0, &V20909_294_57190, &P20909_293};	/*Property = CssPosition*/
const Value V20909_295_57192 = {S2, NULL};	/*Value = True*/
const Property P20909_295 = {S17, 0, &V20909_295_57192, &P20909_294};	/*Property = CssText*/
const Value V20909_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20909_297 = {S18, 0, &V20909_297_57195, &P20909_295};	/*Property = CssTransforms*/
const Value V20909_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20909_241 = {S19, 0, &V20909_241_55799, &P20909_297};	/*Property = CssTransitions*/
const Value V20909_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20909_298 = {S20, 0, &V20909_298_57206, &P20909_241};	/*Property = CssUI*/
const Value V20909_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20909_300 = {S21, 0, &V20909_300_57216, &P20909_298};	/*Property = DataSet*/
const Value V20909_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20909_301 = {S22, 0, &V20909_301_57218, &P20909_300};	/*Property = DataUrl*/
const Value V20909_302_57222 = {S0, NULL};	/*Value = False*/
const Property P20909_302 = {S23, 0, &V20909_302_57222, &P20909_301};	/*Property = DeviceOrientation*/
const Value V20909_304_57228 = {S0, NULL};	/*Value = False*/
const Property P20909_304 = {S24, 0, &V20909_304_57228, &P20909_302};	/*Property = FileReader*/
const Value V20909_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20909_305 = {S25, 0, &V20909_305_57231, &P20909_304};	/*Property = FileSaver*/
const Value V20909_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20909_306 = {S26, 0, &V20909_306_57237, &P20909_305};	/*Property = FileWriter*/
const Value V20909_307_57240 = {S0, NULL};	/*Value = False*/
const Property P20909_307 = {S27, 0, &V20909_307_57240, &P20909_306};	/*Property = FormData*/
const Value V20909_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20909_309 = {S28, 0, &V20909_309_57246, &P20909_307};	/*Property = Fullscreen*/
const Value V20909_310_57249 = {S0, NULL};	/*Value = False*/
const Property P20909_310 = {S29, 0, &V20909_310_57249, &P20909_309};	/*Property = GeoLocation*/
const Value V20909_312_57255 = {S0, NULL};	/*Value = False*/
const Property P20909_312 = {S30, 0, &V20909_312_57255, &P20909_310};	/*Property = History*/
const Value V20909_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20909_314 = {S31, 0, &V20909_314_57261, &P20909_312};	/*Property = Html5*/
const Value V20909_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20909_313 = {S32, 0, &V20909_313_57258, &P20909_314};	/*Property = Html-Media-Capture*/
const Value V20909_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20909_315 = {S33, 0, &V20909_315_57264, &P20909_313};	/*Property = Iframe*/
const Value V20909_316_57267 = {S0, NULL};	/*Value = False*/
const Property P20909_316 = {S34, 0, &V20909_316_57267, &P20909_315};	/*Property = IndexedDB*/
const Value V20909_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20909_317 = {S35, 0, &V20909_317_57269, &P20909_316};	/*Property = Json*/
const Value V20909_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P20909_15 = {S37, 0, &V20909_15_54167, &P20909_317};	/*Property = LayoutEngine*/
const Value V20909_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20909_344 = {S38, 0, &V20909_344_57272, &P20909_15};	/*Property = Masking*/
const Value V20909_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20909_320 = {S39, 0, &V20909_320_57290, &P20909_344};	/*Property = PostMessage*/
const Value V20909_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20909_321 = {S40, 0, &V20909_321_57293, &P20909_320};	/*Property = Progress*/
const Value V20909_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20909_322 = {S41, 0, &V20909_322_57296, &P20909_321};	/*Property = Prompts*/
const Value V20909_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20909_323 = {S42, 0, &V20909_323_57299, &P20909_322};	/*Property = Selector*/
const Value V20909_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20909_324 = {S43, 0, &V20909_324_57306, &P20909_323};	/*Property = Svg*/
const Value V20909_327_57315 = {S0, NULL};	/*Value = False*/
const Property P20909_327 = {S44, 0, &V20909_327_57315, &P20909_324};	/*Property = TouchEvents*/
const Value V20909_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20909_328 = {S45, 0, &V20909_328_57318, &P20909_327};	/*Property = Track*/
const Value V20909_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20909_329 = {S46, 0, &V20909_329_57323, &P20909_328};	/*Property = Video*/
const Value V20909_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20909_330 = {S47, 0, &V20909_330_57326, &P20909_329};	/*Property = Viewport*/
const Value V20909_332_57339 = {S0, NULL};	/*Value = False*/
const Property P20909_332 = {S48, 0, &V20909_332_57339, &P20909_330};	/*Property = WebWorkers*/
const Value V20909_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20909_333 = {S49, 0, &V20909_333_57342, &P20909_332};	/*Property = Xhr2*/
const Profile P20909 = {"20909", &P20909_333};	/*Profile ID - 20909*/
const Value V22398_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22398_279 = {S1, 0, &V22398_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22398_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22398_282 = {S3, 0, &V22398_282_57153, &P22398_279};	/*Property = BlobBuilder*/
const Value V22398_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22398_350 = {S4, 0, &V22398_350_57444, &P22398_282};	/*Property = Canvas*/
const Value V22398_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22398_283 = {S5, 0, &V22398_283_57159, &P22398_350};	/*Property = CssBackground*/
const Value V22398_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22398_284 = {S6, 0, &V22398_284_57163, &P22398_283};	/*Property = CssBorderImage*/
const Value V22398_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22398_285 = {S7, 0, &V22398_285_57166, &P22398_284};	/*Property = CssCanvas*/
const Value V22398_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22398_286 = {S8, 0, &V22398_286_57168, &P22398_285};	/*Property = CssColor*/
const Value V22398_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22398_237 = {S9, 0, &V22398_237_55786, &P22398_286};	/*Property = CssColumn*/
const Value V22398_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22398_287 = {S10, 0, &V22398_287_57172, &P22398_237};	/*Property = CssFlexbox*/
const Value V22398_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22398_288 = {S11, 0, &V22398_288_57174, &P22398_287};	/*Property = CssFont*/
const Value V22398_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22398_289 = {S12, 0, &V22398_289_57759, &P22398_288};	/*Property = CssImages*/
const Value V22398_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22398_291 = {S13, 0, &V22398_291_57180, &P22398_289};	/*Property = CssMediaQueries*/
const Value V22398_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22398_292 = {S14, 0, &V22398_292_57183, &P22398_291};	/*Property = CssMinMax*/
const Value V22398_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22398_293 = {S15, 0, &V22398_293_57187, &P22398_292};	/*Property = CssOverflow*/
const Value V22398_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22398_294 = {S16, 0, &V22398_294_57190, &P22398_293};	/*Property = CssPosition*/
const Value V22398_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22398_295 = {S17, 0, &V22398_295_57192, &P22398_294};	/*Property = CssText*/
const Value V22398_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22398_297 = {S18, 0, &V22398_297_57195, &P22398_295};	/*Property = CssTransforms*/
const Value V22398_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22398_241 = {S19, 0, &V22398_241_55799, &P22398_297};	/*Property = CssTransitions*/
const Value V22398_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22398_298 = {S20, 0, &V22398_298_57206, &P22398_241};	/*Property = CssUI*/
const Value V22398_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22398_300 = {S21, 0, &V22398_300_57216, &P22398_298};	/*Property = DataSet*/
const Value V22398_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22398_301 = {S22, 0, &V22398_301_57218, &P22398_300};	/*Property = DataUrl*/
const Value V22398_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22398_302 = {S23, 0, &V22398_302_57221, &P22398_301};	/*Property = DeviceOrientation*/
const Value V22398_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22398_304 = {S24, 0, &V22398_304_57227, &P22398_302};	/*Property = FileReader*/
const Value V22398_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22398_305 = {S25, 0, &V22398_305_57231, &P22398_304};	/*Property = FileSaver*/
const Value V22398_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22398_306 = {S26, 0, &V22398_306_57237, &P22398_305};	/*Property = FileWriter*/
const Value V22398_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22398_307 = {S27, 0, &V22398_307_57239, &P22398_306};	/*Property = FormData*/
const Value V22398_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22398_309 = {S28, 0, &V22398_309_57246, &P22398_307};	/*Property = Fullscreen*/
const Value V22398_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22398_310 = {S29, 0, &V22398_310_57248, &P22398_309};	/*Property = GeoLocation*/
const Value V22398_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22398_312 = {S30, 0, &V22398_312_57255, &P22398_310};	/*Property = History*/
const Value V22398_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22398_314 = {S31, 0, &V22398_314_57261, &P22398_312};	/*Property = Html5*/
const Value V22398_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22398_313 = {S32, 0, &V22398_313_57258, &P22398_314};	/*Property = Html-Media-Capture*/
const Value V22398_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22398_315 = {S33, 0, &V22398_315_57264, &P22398_313};	/*Property = Iframe*/
const Value V22398_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22398_316 = {S34, 0, &V22398_316_57267, &P22398_315};	/*Property = IndexedDB*/
const Value V22398_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22398_317 = {S35, 0, &V22398_317_57269, &P22398_316};	/*Property = Json*/
const Value V22398_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22398_15 = {S37, 0, &V22398_15_28, &P22398_317};	/*Property = LayoutEngine*/
const Value V22398_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22398_344 = {S38, 0, &V22398_344_57272, &P22398_15};	/*Property = Masking*/
const Value V22398_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22398_320 = {S39, 0, &V22398_320_57290, &P22398_344};	/*Property = PostMessage*/
const Value V22398_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22398_321 = {S40, 0, &V22398_321_57293, &P22398_320};	/*Property = Progress*/
const Value V22398_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22398_322 = {S41, 0, &V22398_322_57296, &P22398_321};	/*Property = Prompts*/
const Value V22398_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22398_323 = {S42, 0, &V22398_323_57299, &P22398_322};	/*Property = Selector*/
const Value V22398_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22398_324 = {S43, 0, &V22398_324_57306, &P22398_323};	/*Property = Svg*/
const Value V22398_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22398_327 = {S44, 0, &V22398_327_57315, &P22398_324};	/*Property = TouchEvents*/
const Value V22398_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22398_328 = {S45, 0, &V22398_328_57318, &P22398_327};	/*Property = Track*/
const Value V22398_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22398_329 = {S46, 0, &V22398_329_57323, &P22398_328};	/*Property = Video*/
const Value V22398_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22398_330 = {S47, 0, &V22398_330_57326, &P22398_329};	/*Property = Viewport*/
const Value V22398_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22398_332 = {S48, 0, &V22398_332_57339, &P22398_330};	/*Property = WebWorkers*/
const Value V22398_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22398_333 = {S49, 0, &V22398_333_57342, &P22398_332};	/*Property = Xhr2*/
const Profile P22398 = {"22398", &P22398_333};	/*Profile ID - 22398*/
const Value V18374_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_279 = {S1, 0, &V18374_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18374_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_282 = {S3, 0, &V18374_282_57155, &P18374_279};	/*Property = BlobBuilder*/
const Value V18374_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_350 = {S4, 0, &V18374_350_57446, &P18374_282};	/*Property = Canvas*/
const Value V18374_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_283 = {S5, 0, &V18374_283_57161, &P18374_350};	/*Property = CssBackground*/
const Value V18374_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_284 = {S6, 0, &V18374_284_57164, &P18374_283};	/*Property = CssBorderImage*/
const Value V18374_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_285 = {S7, 0, &V18374_285_57167, &P18374_284};	/*Property = CssCanvas*/
const Value V18374_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_286 = {S8, 0, &V18374_286_57170, &P18374_285};	/*Property = CssColor*/
const Value V18374_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_237 = {S9, 0, &V18374_237_55786, &P18374_286};	/*Property = CssColumn*/
const Value V18374_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_287 = {S10, 0, &V18374_287_57173, &P18374_237};	/*Property = CssFlexbox*/
const Value V18374_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_288 = {S11, 0, &V18374_288_57176, &P18374_287};	/*Property = CssFont*/
const Value V18374_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_289 = {S12, 0, &V18374_289_86899, &P18374_288};	/*Property = CssImages*/
const Value V18374_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_291 = {S13, 0, &V18374_291_57182, &P18374_289};	/*Property = CssMediaQueries*/
const Value V18374_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_292 = {S14, 0, &V18374_292_57185, &P18374_291};	/*Property = CssMinMax*/
const Value V18374_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_293 = {S15, 0, &V18374_293_57188, &P18374_292};	/*Property = CssOverflow*/
const Value V18374_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_294 = {S16, 0, &V18374_294_57191, &P18374_293};	/*Property = CssPosition*/
const Value V18374_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_295 = {S17, 0, &V18374_295_57194, &P18374_294};	/*Property = CssText*/
const Value V18374_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_297 = {S18, 0, &V18374_297_57197, &P18374_295};	/*Property = CssTransforms*/
const Value V18374_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_241 = {S19, 0, &V18374_241_55801, &P18374_297};	/*Property = CssTransitions*/
const Value V18374_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_298 = {S20, 0, &V18374_298_57208, &P18374_241};	/*Property = CssUI*/
const Value V18374_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_300 = {S21, 0, &V18374_300_57217, &P18374_298};	/*Property = DataSet*/
const Value V18374_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_301 = {S22, 0, &V18374_301_57220, &P18374_300};	/*Property = DataUrl*/
const Value V18374_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_302 = {S23, 0, &V18374_302_57223, &P18374_301};	/*Property = DeviceOrientation*/
const Value V18374_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_304 = {S24, 0, &V18374_304_57229, &P18374_302};	/*Property = FileReader*/
const Value V18374_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_305 = {S25, 0, &V18374_305_57232, &P18374_304};	/*Property = FileSaver*/
const Value V18374_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_306 = {S26, 0, &V18374_306_57238, &P18374_305};	/*Property = FileWriter*/
const Value V18374_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_307 = {S27, 0, &V18374_307_57241, &P18374_306};	/*Property = FormData*/
const Value V18374_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_309 = {S28, 0, &V18374_309_57247, &P18374_307};	/*Property = Fullscreen*/
const Value V18374_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_310 = {S29, 0, &V18374_310_57250, &P18374_309};	/*Property = GeoLocation*/
const Value V18374_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_312 = {S30, 0, &V18374_312_57256, &P18374_310};	/*Property = History*/
const Value V18374_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_314 = {S31, 0, &V18374_314_57262, &P18374_312};	/*Property = Html5*/
const Value V18374_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_313 = {S32, 0, &V18374_313_57259, &P18374_314};	/*Property = Html-Media-Capture*/
const Value V18374_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_315 = {S33, 0, &V18374_315_57265, &P18374_313};	/*Property = Iframe*/
const Value V18374_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_316 = {S34, 0, &V18374_316_57268, &P18374_315};	/*Property = IndexedDB*/
const Value V18374_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_317 = {S35, 0, &V18374_317_57271, &P18374_316};	/*Property = Json*/
const Value V18374_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18374_15 = {S37, 0, &V18374_15_31, &P18374_317};	/*Property = LayoutEngine*/
const Value V18374_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_344 = {S38, 0, &V18374_344_57274, &P18374_15};	/*Property = Masking*/
const Value V18374_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_320 = {S39, 0, &V18374_320_57292, &P18374_344};	/*Property = PostMessage*/
const Value V18374_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_321 = {S40, 0, &V18374_321_57295, &P18374_320};	/*Property = Progress*/
const Value V18374_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_322 = {S41, 0, &V18374_322_57298, &P18374_321};	/*Property = Prompts*/
const Value V18374_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_323 = {S42, 0, &V18374_323_57301, &P18374_322};	/*Property = Selector*/
const Value V18374_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_324 = {S43, 0, &V18374_324_57307, &P18374_323};	/*Property = Svg*/
const Value V18374_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_327 = {S44, 0, &V18374_327_57316, &P18374_324};	/*Property = TouchEvents*/
const Value V18374_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_328 = {S45, 0, &V18374_328_57319, &P18374_327};	/*Property = Track*/
const Value V18374_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_329 = {S46, 0, &V18374_329_57325, &P18374_328};	/*Property = Video*/
const Value V18374_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_330 = {S47, 0, &V18374_330_57328, &P18374_329};	/*Property = Viewport*/
const Value V18374_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_332 = {S48, 0, &V18374_332_57340, &P18374_330};	/*Property = WebWorkers*/
const Value V18374_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18374_333 = {S49, 0, &V18374_333_57343, &P18374_332};	/*Property = Xhr2*/
const Profile P18374 = {"18374", &P18374_333};	/*Profile ID - 18374*/
const Value V22267_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_279 = {S1, 0, &V22267_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22267_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_282 = {S3, 0, &V22267_282_57155, &P22267_279};	/*Property = BlobBuilder*/
const Value V22267_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_350 = {S4, 0, &V22267_350_57446, &P22267_282};	/*Property = Canvas*/
const Value V22267_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_283 = {S5, 0, &V22267_283_57161, &P22267_350};	/*Property = CssBackground*/
const Value V22267_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_284 = {S6, 0, &V22267_284_57164, &P22267_283};	/*Property = CssBorderImage*/
const Value V22267_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_285 = {S7, 0, &V22267_285_57167, &P22267_284};	/*Property = CssCanvas*/
const Value V22267_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_286 = {S8, 0, &V22267_286_57170, &P22267_285};	/*Property = CssColor*/
const Value V22267_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_237 = {S9, 0, &V22267_237_55786, &P22267_286};	/*Property = CssColumn*/
const Value V22267_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_287 = {S10, 0, &V22267_287_57173, &P22267_237};	/*Property = CssFlexbox*/
const Value V22267_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_288 = {S11, 0, &V22267_288_57176, &P22267_287};	/*Property = CssFont*/
const Value V22267_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_289 = {S12, 0, &V22267_289_86899, &P22267_288};	/*Property = CssImages*/
const Value V22267_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_291 = {S13, 0, &V22267_291_57182, &P22267_289};	/*Property = CssMediaQueries*/
const Value V22267_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_292 = {S14, 0, &V22267_292_57185, &P22267_291};	/*Property = CssMinMax*/
const Value V22267_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_293 = {S15, 0, &V22267_293_57188, &P22267_292};	/*Property = CssOverflow*/
const Value V22267_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_294 = {S16, 0, &V22267_294_57191, &P22267_293};	/*Property = CssPosition*/
const Value V22267_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_295 = {S17, 0, &V22267_295_57194, &P22267_294};	/*Property = CssText*/
const Value V22267_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_297 = {S18, 0, &V22267_297_57197, &P22267_295};	/*Property = CssTransforms*/
const Value V22267_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_241 = {S19, 0, &V22267_241_55801, &P22267_297};	/*Property = CssTransitions*/
const Value V22267_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_298 = {S20, 0, &V22267_298_57208, &P22267_241};	/*Property = CssUI*/
const Value V22267_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_300 = {S21, 0, &V22267_300_57217, &P22267_298};	/*Property = DataSet*/
const Value V22267_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_301 = {S22, 0, &V22267_301_57220, &P22267_300};	/*Property = DataUrl*/
const Value V22267_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_302 = {S23, 0, &V22267_302_57223, &P22267_301};	/*Property = DeviceOrientation*/
const Value V22267_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_304 = {S24, 0, &V22267_304_57229, &P22267_302};	/*Property = FileReader*/
const Value V22267_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_305 = {S25, 0, &V22267_305_57232, &P22267_304};	/*Property = FileSaver*/
const Value V22267_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_306 = {S26, 0, &V22267_306_57238, &P22267_305};	/*Property = FileWriter*/
const Value V22267_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_307 = {S27, 0, &V22267_307_57241, &P22267_306};	/*Property = FormData*/
const Value V22267_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_309 = {S28, 0, &V22267_309_57247, &P22267_307};	/*Property = Fullscreen*/
const Value V22267_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_310 = {S29, 0, &V22267_310_57250, &P22267_309};	/*Property = GeoLocation*/
const Value V22267_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_312 = {S30, 0, &V22267_312_57256, &P22267_310};	/*Property = History*/
const Value V22267_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_314 = {S31, 0, &V22267_314_57262, &P22267_312};	/*Property = Html5*/
const Value V22267_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_313 = {S32, 0, &V22267_313_57259, &P22267_314};	/*Property = Html-Media-Capture*/
const Value V22267_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_315 = {S33, 0, &V22267_315_57265, &P22267_313};	/*Property = Iframe*/
const Value V22267_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_316 = {S34, 0, &V22267_316_57268, &P22267_315};	/*Property = IndexedDB*/
const Value V22267_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_317 = {S35, 0, &V22267_317_57271, &P22267_316};	/*Property = Json*/
const Value V22267_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P22267_15 = {S37, 0, &V22267_15_30, &P22267_317};	/*Property = LayoutEngine*/
const Value V22267_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_344 = {S38, 0, &V22267_344_57274, &P22267_15};	/*Property = Masking*/
const Value V22267_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_320 = {S39, 0, &V22267_320_57292, &P22267_344};	/*Property = PostMessage*/
const Value V22267_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_321 = {S40, 0, &V22267_321_57295, &P22267_320};	/*Property = Progress*/
const Value V22267_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_322 = {S41, 0, &V22267_322_57298, &P22267_321};	/*Property = Prompts*/
const Value V22267_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_323 = {S42, 0, &V22267_323_57301, &P22267_322};	/*Property = Selector*/
const Value V22267_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_324 = {S43, 0, &V22267_324_57307, &P22267_323};	/*Property = Svg*/
const Value V22267_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_327 = {S44, 0, &V22267_327_57316, &P22267_324};	/*Property = TouchEvents*/
const Value V22267_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_328 = {S45, 0, &V22267_328_57319, &P22267_327};	/*Property = Track*/
const Value V22267_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_329 = {S46, 0, &V22267_329_57325, &P22267_328};	/*Property = Video*/
const Value V22267_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_330 = {S47, 0, &V22267_330_57328, &P22267_329};	/*Property = Viewport*/
const Value V22267_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_332 = {S48, 0, &V22267_332_57340, &P22267_330};	/*Property = WebWorkers*/
const Value V22267_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22267_333 = {S49, 0, &V22267_333_57343, &P22267_332};	/*Property = Xhr2*/
const Profile P22267 = {"22267", &P22267_333};	/*Profile ID - 22267*/
const Value V20901_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20901_279 = {S1, 0, &V20901_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20901_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20901_282 = {S3, 0, &V20901_282_57154, &P20901_279};	/*Property = BlobBuilder*/
const Value V20901_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20901_350 = {S4, 0, &V20901_350_57444, &P20901_282};	/*Property = Canvas*/
const Value V20901_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20901_283 = {S5, 0, &V20901_283_57159, &P20901_350};	/*Property = CssBackground*/
const Value V20901_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20901_284 = {S6, 0, &V20901_284_57163, &P20901_283};	/*Property = CssBorderImage*/
const Value V20901_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20901_285 = {S7, 0, &V20901_285_57166, &P20901_284};	/*Property = CssCanvas*/
const Value V20901_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20901_286 = {S8, 0, &V20901_286_57168, &P20901_285};	/*Property = CssColor*/
const Value V20901_237_55785 = {S0, NULL};	/*Value = False*/
const Property P20901_237 = {S9, 0, &V20901_237_55785, &P20901_286};	/*Property = CssColumn*/
const Value V20901_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20901_287 = {S10, 0, &V20901_287_57172, &P20901_237};	/*Property = CssFlexbox*/
const Value V20901_288_57175 = {S0, NULL};	/*Value = False*/
const Property P20901_288 = {S11, 0, &V20901_288_57175, &P20901_287};	/*Property = CssFont*/
const Value V20901_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20901_289 = {S12, 0, &V20901_289_57759, &P20901_288};	/*Property = CssImages*/
const Value V20901_291_57181 = {S0, NULL};	/*Value = False*/
const Property P20901_291 = {S13, 0, &V20901_291_57181, &P20901_289};	/*Property = CssMediaQueries*/
const Value V20901_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20901_292 = {S14, 0, &V20901_292_57183, &P20901_291};	/*Property = CssMinMax*/
const Value V20901_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20901_293 = {S15, 0, &V20901_293_57187, &P20901_292};	/*Property = CssOverflow*/
const Value V20901_294_57190 = {S0, NULL};	/*Value = False*/
const Property P20901_294 = {S16, 0, &V20901_294_57190, &P20901_293};	/*Property = CssPosition*/
const Value V20901_295_57192 = {S2, NULL};	/*Value = True*/
const Property P20901_295 = {S17, 0, &V20901_295_57192, &P20901_294};	/*Property = CssText*/
const Value V20901_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20901_297 = {S18, 0, &V20901_297_57195, &P20901_295};	/*Property = CssTransforms*/
const Value V20901_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20901_241 = {S19, 0, &V20901_241_55799, &P20901_297};	/*Property = CssTransitions*/
const Value V20901_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20901_298 = {S20, 0, &V20901_298_57206, &P20901_241};	/*Property = CssUI*/
const Value V20901_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20901_300 = {S21, 0, &V20901_300_57216, &P20901_298};	/*Property = DataSet*/
const Value V20901_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20901_301 = {S22, 0, &V20901_301_57218, &P20901_300};	/*Property = DataUrl*/
const Value V20901_302_57222 = {S0, NULL};	/*Value = False*/
const Property P20901_302 = {S23, 0, &V20901_302_57222, &P20901_301};	/*Property = DeviceOrientation*/
const Value V20901_304_57228 = {S0, NULL};	/*Value = False*/
const Property P20901_304 = {S24, 0, &V20901_304_57228, &P20901_302};	/*Property = FileReader*/
const Value V20901_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20901_305 = {S25, 0, &V20901_305_57231, &P20901_304};	/*Property = FileSaver*/
const Value V20901_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20901_306 = {S26, 0, &V20901_306_57237, &P20901_305};	/*Property = FileWriter*/
const Value V20901_307_57240 = {S0, NULL};	/*Value = False*/
const Property P20901_307 = {S27, 0, &V20901_307_57240, &P20901_306};	/*Property = FormData*/
const Value V20901_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20901_309 = {S28, 0, &V20901_309_57246, &P20901_307};	/*Property = Fullscreen*/
const Value V20901_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20901_310 = {S29, 0, &V20901_310_57248, &P20901_309};	/*Property = GeoLocation*/
const Value V20901_312_57255 = {S0, NULL};	/*Value = False*/
const Property P20901_312 = {S30, 0, &V20901_312_57255, &P20901_310};	/*Property = History*/
const Value V20901_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20901_314 = {S31, 0, &V20901_314_57261, &P20901_312};	/*Property = Html5*/
const Value V20901_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20901_313 = {S32, 0, &V20901_313_57258, &P20901_314};	/*Property = Html-Media-Capture*/
const Value V20901_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20901_315 = {S33, 0, &V20901_315_57264, &P20901_313};	/*Property = Iframe*/
const Value V20901_316_57267 = {S0, NULL};	/*Value = False*/
const Property P20901_316 = {S34, 0, &V20901_316_57267, &P20901_315};	/*Property = IndexedDB*/
const Value V20901_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20901_317 = {S35, 0, &V20901_317_57269, &P20901_316};	/*Property = Json*/
const Value V20901_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P20901_15 = {S37, 0, &V20901_15_30, &P20901_317};	/*Property = LayoutEngine*/
const Value V20901_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20901_344 = {S38, 0, &V20901_344_57272, &P20901_15};	/*Property = Masking*/
const Value V20901_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20901_320 = {S39, 0, &V20901_320_57290, &P20901_344};	/*Property = PostMessage*/
const Value V20901_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20901_321 = {S40, 0, &V20901_321_57293, &P20901_320};	/*Property = Progress*/
const Value V20901_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20901_322 = {S41, 0, &V20901_322_57296, &P20901_321};	/*Property = Prompts*/
const Value V20901_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20901_323 = {S42, 0, &V20901_323_57299, &P20901_322};	/*Property = Selector*/
const Value V20901_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20901_324 = {S43, 0, &V20901_324_57306, &P20901_323};	/*Property = Svg*/
const Value V20901_327_57315 = {S0, NULL};	/*Value = False*/
const Property P20901_327 = {S44, 0, &V20901_327_57315, &P20901_324};	/*Property = TouchEvents*/
const Value V20901_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20901_328 = {S45, 0, &V20901_328_57318, &P20901_327};	/*Property = Track*/
const Value V20901_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20901_329 = {S46, 0, &V20901_329_57323, &P20901_328};	/*Property = Video*/
const Value V20901_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20901_330 = {S47, 0, &V20901_330_57326, &P20901_329};	/*Property = Viewport*/
const Value V20901_332_57339 = {S0, NULL};	/*Value = False*/
const Property P20901_332 = {S48, 0, &V20901_332_57339, &P20901_330};	/*Property = WebWorkers*/
const Value V20901_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20901_333 = {S49, 0, &V20901_333_57342, &P20901_332};	/*Property = Xhr2*/
const Profile P20901 = {"20901", &P20901_333};	/*Profile ID - 20901*/
const Value V17901_1_1 = {S2, NULL};	/*Value = True*/
const Property P17901_1 = {S50, 0, &V17901_1_1, NULL};	/*Property = IsMobile*/
const Value V17901_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17901_5 = {S52, 0, &V17901_5_1304, &P17901_1};	/*Property = ScreenPixelsHeight*/
const Value V17901_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17901_4 = {S54, 0, &V17901_4_1224, &P17901_5};	/*Property = ScreenPixelsWidth*/
const Profile P17901 = {"17901", &P17901_4};	/*Profile ID - 17901*/
const Value V21640_1_1 = {S2, NULL};	/*Value = True*/
const Property P21640_1 = {S50, 0, &V21640_1_1, NULL};	/*Property = IsMobile*/
const Value V21640_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21640_5 = {S52, 0, &V21640_5_37, &P21640_1};	/*Property = ScreenPixelsHeight*/
const Value V21640_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21640_4 = {S54, 0, &V21640_4_36, &P21640_5};	/*Property = ScreenPixelsWidth*/
const Profile P21640 = {"21640", &P21640_4};	/*Profile ID - 21640*/
const Value V18419_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_279 = {S1, 0, &V18419_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18419_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_282 = {S3, 0, &V18419_282_57155, &P18419_279};	/*Property = BlobBuilder*/
const Value V18419_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_350 = {S4, 0, &V18419_350_57446, &P18419_282};	/*Property = Canvas*/
const Value V18419_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_283 = {S5, 0, &V18419_283_57161, &P18419_350};	/*Property = CssBackground*/
const Value V18419_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_284 = {S6, 0, &V18419_284_57164, &P18419_283};	/*Property = CssBorderImage*/
const Value V18419_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_285 = {S7, 0, &V18419_285_57167, &P18419_284};	/*Property = CssCanvas*/
const Value V18419_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_286 = {S8, 0, &V18419_286_57170, &P18419_285};	/*Property = CssColor*/
const Value V18419_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_237 = {S9, 0, &V18419_237_55786, &P18419_286};	/*Property = CssColumn*/
const Value V18419_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_287 = {S10, 0, &V18419_287_57173, &P18419_237};	/*Property = CssFlexbox*/
const Value V18419_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_288 = {S11, 0, &V18419_288_57176, &P18419_287};	/*Property = CssFont*/
const Value V18419_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_289 = {S12, 0, &V18419_289_86899, &P18419_288};	/*Property = CssImages*/
const Value V18419_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_291 = {S13, 0, &V18419_291_57182, &P18419_289};	/*Property = CssMediaQueries*/
const Value V18419_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_292 = {S14, 0, &V18419_292_57185, &P18419_291};	/*Property = CssMinMax*/
const Value V18419_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_293 = {S15, 0, &V18419_293_57188, &P18419_292};	/*Property = CssOverflow*/
const Value V18419_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_294 = {S16, 0, &V18419_294_57191, &P18419_293};	/*Property = CssPosition*/
const Value V18419_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_295 = {S17, 0, &V18419_295_57194, &P18419_294};	/*Property = CssText*/
const Value V18419_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_297 = {S18, 0, &V18419_297_57197, &P18419_295};	/*Property = CssTransforms*/
const Value V18419_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_241 = {S19, 0, &V18419_241_55801, &P18419_297};	/*Property = CssTransitions*/
const Value V18419_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_298 = {S20, 0, &V18419_298_57208, &P18419_241};	/*Property = CssUI*/
const Value V18419_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_300 = {S21, 0, &V18419_300_57217, &P18419_298};	/*Property = DataSet*/
const Value V18419_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_301 = {S22, 0, &V18419_301_57220, &P18419_300};	/*Property = DataUrl*/
const Value V18419_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_302 = {S23, 0, &V18419_302_57223, &P18419_301};	/*Property = DeviceOrientation*/
const Value V18419_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_304 = {S24, 0, &V18419_304_57229, &P18419_302};	/*Property = FileReader*/
const Value V18419_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_305 = {S25, 0, &V18419_305_57232, &P18419_304};	/*Property = FileSaver*/
const Value V18419_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_306 = {S26, 0, &V18419_306_57238, &P18419_305};	/*Property = FileWriter*/
const Value V18419_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_307 = {S27, 0, &V18419_307_57241, &P18419_306};	/*Property = FormData*/
const Value V18419_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_309 = {S28, 0, &V18419_309_57247, &P18419_307};	/*Property = Fullscreen*/
const Value V18419_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_310 = {S29, 0, &V18419_310_57250, &P18419_309};	/*Property = GeoLocation*/
const Value V18419_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_312 = {S30, 0, &V18419_312_57256, &P18419_310};	/*Property = History*/
const Value V18419_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_314 = {S31, 0, &V18419_314_57262, &P18419_312};	/*Property = Html5*/
const Value V18419_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_313 = {S32, 0, &V18419_313_57259, &P18419_314};	/*Property = Html-Media-Capture*/
const Value V18419_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_315 = {S33, 0, &V18419_315_57265, &P18419_313};	/*Property = Iframe*/
const Value V18419_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_316 = {S34, 0, &V18419_316_57268, &P18419_315};	/*Property = IndexedDB*/
const Value V18419_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_317 = {S35, 0, &V18419_317_57271, &P18419_316};	/*Property = Json*/
const Value V18419_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18419_15 = {S37, 0, &V18419_15_28, &P18419_317};	/*Property = LayoutEngine*/
const Value V18419_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_344 = {S38, 0, &V18419_344_57274, &P18419_15};	/*Property = Masking*/
const Value V18419_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_320 = {S39, 0, &V18419_320_57292, &P18419_344};	/*Property = PostMessage*/
const Value V18419_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_321 = {S40, 0, &V18419_321_57295, &P18419_320};	/*Property = Progress*/
const Value V18419_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_322 = {S41, 0, &V18419_322_57298, &P18419_321};	/*Property = Prompts*/
const Value V18419_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_323 = {S42, 0, &V18419_323_57301, &P18419_322};	/*Property = Selector*/
const Value V18419_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_324 = {S43, 0, &V18419_324_57307, &P18419_323};	/*Property = Svg*/
const Value V18419_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_327 = {S44, 0, &V18419_327_57316, &P18419_324};	/*Property = TouchEvents*/
const Value V18419_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_328 = {S45, 0, &V18419_328_57319, &P18419_327};	/*Property = Track*/
const Value V18419_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_329 = {S46, 0, &V18419_329_57325, &P18419_328};	/*Property = Video*/
const Value V18419_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_330 = {S47, 0, &V18419_330_57328, &P18419_329};	/*Property = Viewport*/
const Value V18419_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_332 = {S48, 0, &V18419_332_57340, &P18419_330};	/*Property = WebWorkers*/
const Value V18419_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18419_333 = {S49, 0, &V18419_333_57343, &P18419_332};	/*Property = Xhr2*/
const Profile P18419 = {"18419", &P18419_333};	/*Profile ID - 18419*/
const Value V17797_1_1 = {S2, NULL};	/*Value = True*/
const Property P17797_1 = {S50, 0, &V17797_1_1, NULL};	/*Property = IsMobile*/
const Value V17797_5_37505 = {S83, NULL};	/*Value = 272*/
const Property P17797_5 = {S52, 0, &V17797_5_37505, &P17797_1};	/*Property = ScreenPixelsHeight*/
const Value V17797_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17797_4 = {S54, 0, &V17797_4_1224, &P17797_5};	/*Property = ScreenPixelsWidth*/
const Profile P17797 = {"17797", &P17797_4};	/*Profile ID - 17797*/
const Value V17888_1_1 = {S2, NULL};	/*Value = True*/
const Property P17888_1 = {S50, 0, &V17888_1_1, NULL};	/*Property = IsMobile*/
const Value V17888_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P17888_5 = {S52, 0, &V17888_5_1380, &P17888_1};	/*Property = ScreenPixelsHeight*/
const Value V17888_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17888_4 = {S54, 0, &V17888_4_1279, &P17888_5};	/*Property = ScreenPixelsWidth*/
const Profile P17888 = {"17888", &P17888_4};	/*Profile ID - 17888*/
const Value V17390_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_279 = {S1, 0, &V17390_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17390_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_282 = {S3, 0, &V17390_282_57155, &P17390_279};	/*Property = BlobBuilder*/
const Value V17390_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_350 = {S4, 0, &V17390_350_57446, &P17390_282};	/*Property = Canvas*/
const Value V17390_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_283 = {S5, 0, &V17390_283_57161, &P17390_350};	/*Property = CssBackground*/
const Value V17390_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_284 = {S6, 0, &V17390_284_57164, &P17390_283};	/*Property = CssBorderImage*/
const Value V17390_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_285 = {S7, 0, &V17390_285_57167, &P17390_284};	/*Property = CssCanvas*/
const Value V17390_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_286 = {S8, 0, &V17390_286_57170, &P17390_285};	/*Property = CssColor*/
const Value V17390_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_237 = {S9, 0, &V17390_237_55786, &P17390_286};	/*Property = CssColumn*/
const Value V17390_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_287 = {S10, 0, &V17390_287_57173, &P17390_237};	/*Property = CssFlexbox*/
const Value V17390_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_288 = {S11, 0, &V17390_288_57176, &P17390_287};	/*Property = CssFont*/
const Value V17390_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_289 = {S12, 0, &V17390_289_86899, &P17390_288};	/*Property = CssImages*/
const Value V17390_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_291 = {S13, 0, &V17390_291_57182, &P17390_289};	/*Property = CssMediaQueries*/
const Value V17390_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_292 = {S14, 0, &V17390_292_57185, &P17390_291};	/*Property = CssMinMax*/
const Value V17390_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_293 = {S15, 0, &V17390_293_57188, &P17390_292};	/*Property = CssOverflow*/
const Value V17390_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_294 = {S16, 0, &V17390_294_57191, &P17390_293};	/*Property = CssPosition*/
const Value V17390_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_295 = {S17, 0, &V17390_295_57194, &P17390_294};	/*Property = CssText*/
const Value V17390_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_297 = {S18, 0, &V17390_297_57197, &P17390_295};	/*Property = CssTransforms*/
const Value V17390_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_241 = {S19, 0, &V17390_241_55801, &P17390_297};	/*Property = CssTransitions*/
const Value V17390_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_298 = {S20, 0, &V17390_298_57208, &P17390_241};	/*Property = CssUI*/
const Value V17390_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_300 = {S21, 0, &V17390_300_57217, &P17390_298};	/*Property = DataSet*/
const Value V17390_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_301 = {S22, 0, &V17390_301_57220, &P17390_300};	/*Property = DataUrl*/
const Value V17390_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_302 = {S23, 0, &V17390_302_57223, &P17390_301};	/*Property = DeviceOrientation*/
const Value V17390_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_304 = {S24, 0, &V17390_304_57229, &P17390_302};	/*Property = FileReader*/
const Value V17390_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_305 = {S25, 0, &V17390_305_57232, &P17390_304};	/*Property = FileSaver*/
const Value V17390_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_306 = {S26, 0, &V17390_306_57238, &P17390_305};	/*Property = FileWriter*/
const Value V17390_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_307 = {S27, 0, &V17390_307_57241, &P17390_306};	/*Property = FormData*/
const Value V17390_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_309 = {S28, 0, &V17390_309_57247, &P17390_307};	/*Property = Fullscreen*/
const Value V17390_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_310 = {S29, 0, &V17390_310_57250, &P17390_309};	/*Property = GeoLocation*/
const Value V17390_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_312 = {S30, 0, &V17390_312_57256, &P17390_310};	/*Property = History*/
const Value V17390_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_314 = {S31, 0, &V17390_314_57262, &P17390_312};	/*Property = Html5*/
const Value V17390_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_313 = {S32, 0, &V17390_313_57259, &P17390_314};	/*Property = Html-Media-Capture*/
const Value V17390_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_315 = {S33, 0, &V17390_315_57265, &P17390_313};	/*Property = Iframe*/
const Value V17390_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_316 = {S34, 0, &V17390_316_57268, &P17390_315};	/*Property = IndexedDB*/
const Value V17390_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_317 = {S35, 0, &V17390_317_57271, &P17390_316};	/*Property = Json*/
const Value V17390_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P17390_15 = {S37, 0, &V17390_15_54167, &P17390_317};	/*Property = LayoutEngine*/
const Value V17390_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_344 = {S38, 0, &V17390_344_57274, &P17390_15};	/*Property = Masking*/
const Value V17390_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_320 = {S39, 0, &V17390_320_57292, &P17390_344};	/*Property = PostMessage*/
const Value V17390_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_321 = {S40, 0, &V17390_321_57295, &P17390_320};	/*Property = Progress*/
const Value V17390_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_322 = {S41, 0, &V17390_322_57298, &P17390_321};	/*Property = Prompts*/
const Value V17390_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_323 = {S42, 0, &V17390_323_57301, &P17390_322};	/*Property = Selector*/
const Value V17390_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_324 = {S43, 0, &V17390_324_57307, &P17390_323};	/*Property = Svg*/
const Value V17390_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_327 = {S44, 0, &V17390_327_57316, &P17390_324};	/*Property = TouchEvents*/
const Value V17390_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_328 = {S45, 0, &V17390_328_57319, &P17390_327};	/*Property = Track*/
const Value V17390_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_329 = {S46, 0, &V17390_329_57325, &P17390_328};	/*Property = Video*/
const Value V17390_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_330 = {S47, 0, &V17390_330_57328, &P17390_329};	/*Property = Viewport*/
const Value V17390_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_332 = {S48, 0, &V17390_332_57340, &P17390_330};	/*Property = WebWorkers*/
const Value V17390_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17390_333 = {S49, 0, &V17390_333_57343, &P17390_332};	/*Property = Xhr2*/
const Profile P17390 = {"17390", &P17390_333};	/*Profile ID - 17390*/
const Value V21955_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21955_279 = {S1, 0, &V21955_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21955_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21955_282 = {S3, 0, &V21955_282_57154, &P21955_279};	/*Property = BlobBuilder*/
const Value V21955_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21955_350 = {S4, 0, &V21955_350_57444, &P21955_282};	/*Property = Canvas*/
const Value V21955_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21955_283 = {S5, 0, &V21955_283_57159, &P21955_350};	/*Property = CssBackground*/
const Value V21955_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21955_284 = {S6, 0, &V21955_284_57163, &P21955_283};	/*Property = CssBorderImage*/
const Value V21955_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21955_285 = {S7, 0, &V21955_285_57166, &P21955_284};	/*Property = CssCanvas*/
const Value V21955_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21955_286 = {S8, 0, &V21955_286_57168, &P21955_285};	/*Property = CssColor*/
const Value V21955_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21955_237 = {S9, 0, &V21955_237_55784, &P21955_286};	/*Property = CssColumn*/
const Value V21955_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21955_287 = {S10, 0, &V21955_287_57172, &P21955_237};	/*Property = CssFlexbox*/
const Value V21955_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21955_288 = {S11, 0, &V21955_288_57175, &P21955_287};	/*Property = CssFont*/
const Value V21955_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21955_289 = {S12, 0, &V21955_289_57759, &P21955_288};	/*Property = CssImages*/
const Value V21955_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21955_291 = {S13, 0, &V21955_291_57181, &P21955_289};	/*Property = CssMediaQueries*/
const Value V21955_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21955_292 = {S14, 0, &V21955_292_57183, &P21955_291};	/*Property = CssMinMax*/
const Value V21955_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21955_293 = {S15, 0, &V21955_293_57187, &P21955_292};	/*Property = CssOverflow*/
const Value V21955_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21955_294 = {S16, 0, &V21955_294_57190, &P21955_293};	/*Property = CssPosition*/
const Value V21955_295_57193 = {S0, NULL};	/*Value = False*/
const Property P21955_295 = {S17, 0, &V21955_295_57193, &P21955_294};	/*Property = CssText*/
const Value V21955_297_57196 = {S0, NULL};	/*Value = False*/
const Property P21955_297 = {S18, 0, &V21955_297_57196, &P21955_295};	/*Property = CssTransforms*/
const Value V21955_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21955_241 = {S19, 0, &V21955_241_55799, &P21955_297};	/*Property = CssTransitions*/
const Value V21955_298_57207 = {S0, NULL};	/*Value = False*/
const Property P21955_298 = {S20, 0, &V21955_298_57207, &P21955_241};	/*Property = CssUI*/
const Value V21955_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21955_300 = {S21, 0, &V21955_300_57216, &P21955_298};	/*Property = DataSet*/
const Value V21955_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21955_301 = {S22, 0, &V21955_301_57218, &P21955_300};	/*Property = DataUrl*/
const Value V21955_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21955_302 = {S23, 0, &V21955_302_57222, &P21955_301};	/*Property = DeviceOrientation*/
const Value V21955_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21955_304 = {S24, 0, &V21955_304_57228, &P21955_302};	/*Property = FileReader*/
const Value V21955_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21955_305 = {S25, 0, &V21955_305_57231, &P21955_304};	/*Property = FileSaver*/
const Value V21955_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21955_306 = {S26, 0, &V21955_306_57237, &P21955_305};	/*Property = FileWriter*/
const Value V21955_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21955_307 = {S27, 0, &V21955_307_57240, &P21955_306};	/*Property = FormData*/
const Value V21955_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21955_309 = {S28, 0, &V21955_309_57246, &P21955_307};	/*Property = Fullscreen*/
const Value V21955_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21955_310 = {S29, 0, &V21955_310_57248, &P21955_309};	/*Property = GeoLocation*/
const Value V21955_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21955_312 = {S30, 0, &V21955_312_57255, &P21955_310};	/*Property = History*/
const Value V21955_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21955_314 = {S31, 0, &V21955_314_57261, &P21955_312};	/*Property = Html5*/
const Value V21955_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21955_313 = {S32, 0, &V21955_313_57258, &P21955_314};	/*Property = Html-Media-Capture*/
const Value V21955_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21955_315 = {S33, 0, &V21955_315_57264, &P21955_313};	/*Property = Iframe*/
const Value V21955_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21955_316 = {S34, 0, &V21955_316_57267, &P21955_315};	/*Property = IndexedDB*/
const Value V21955_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21955_317 = {S35, 0, &V21955_317_57269, &P21955_316};	/*Property = Json*/
const Value V21955_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P21955_15 = {S37, 0, &V21955_15_29, &P21955_317};	/*Property = LayoutEngine*/
const Value V21955_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21955_344 = {S38, 0, &V21955_344_57272, &P21955_15};	/*Property = Masking*/
const Value V21955_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21955_320 = {S39, 0, &V21955_320_57290, &P21955_344};	/*Property = PostMessage*/
const Value V21955_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21955_321 = {S40, 0, &V21955_321_57293, &P21955_320};	/*Property = Progress*/
const Value V21955_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21955_322 = {S41, 0, &V21955_322_57296, &P21955_321};	/*Property = Prompts*/
const Value V21955_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21955_323 = {S42, 0, &V21955_323_57299, &P21955_322};	/*Property = Selector*/
const Value V21955_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21955_324 = {S43, 0, &V21955_324_57306, &P21955_323};	/*Property = Svg*/
const Value V21955_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21955_327 = {S44, 0, &V21955_327_57315, &P21955_324};	/*Property = TouchEvents*/
const Value V21955_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21955_328 = {S45, 0, &V21955_328_57318, &P21955_327};	/*Property = Track*/
const Value V21955_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21955_329 = {S46, 0, &V21955_329_57323, &P21955_328};	/*Property = Video*/
const Value V21955_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21955_330 = {S47, 0, &V21955_330_57326, &P21955_329};	/*Property = Viewport*/
const Value V21955_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21955_332 = {S48, 0, &V21955_332_57339, &P21955_330};	/*Property = WebWorkers*/
const Value V21955_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21955_333 = {S49, 0, &V21955_333_57342, &P21955_332};	/*Property = Xhr2*/
const Profile P21955 = {"21955", &P21955_333};	/*Profile ID - 21955*/
const Value V17890_1_1 = {S2, NULL};	/*Value = True*/
const Property P17890_1 = {S50, 0, &V17890_1_1, NULL};	/*Property = IsMobile*/
const Value V17890_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P17890_5 = {S52, 0, &V17890_5_1380, &P17890_1};	/*Property = ScreenPixelsHeight*/
const Value V17890_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17890_4 = {S54, 0, &V17890_4_1279, &P17890_5};	/*Property = ScreenPixelsWidth*/
const Profile P17890 = {"17890", &P17890_4};	/*Profile ID - 17890*/
const Value V17886_1_1 = {S2, NULL};	/*Value = True*/
const Property P17886_1 = {S50, 0, &V17886_1_1, NULL};	/*Property = IsMobile*/
const Value V17886_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17886_5 = {S52, 0, &V17886_5_1222, &P17886_1};	/*Property = ScreenPixelsHeight*/
const Value V17886_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17886_4 = {S54, 0, &V17886_4_1224, &P17886_5};	/*Property = ScreenPixelsWidth*/
const Profile P17886 = {"17886", &P17886_4};	/*Profile ID - 17886*/
const Value V22373_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22373_279 = {S1, 0, &V22373_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22373_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22373_282 = {S3, 0, &V22373_282_57154, &P22373_279};	/*Property = BlobBuilder*/
const Value V22373_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22373_350 = {S4, 0, &V22373_350_57444, &P22373_282};	/*Property = Canvas*/
const Value V22373_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22373_283 = {S5, 0, &V22373_283_57159, &P22373_350};	/*Property = CssBackground*/
const Value V22373_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22373_284 = {S6, 0, &V22373_284_57163, &P22373_283};	/*Property = CssBorderImage*/
const Value V22373_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22373_285 = {S7, 0, &V22373_285_57166, &P22373_284};	/*Property = CssCanvas*/
const Value V22373_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22373_286 = {S8, 0, &V22373_286_57168, &P22373_285};	/*Property = CssColor*/
const Value V22373_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22373_237 = {S9, 0, &V22373_237_55786, &P22373_286};	/*Property = CssColumn*/
const Value V22373_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22373_287 = {S10, 0, &V22373_287_57172, &P22373_237};	/*Property = CssFlexbox*/
const Value V22373_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22373_288 = {S11, 0, &V22373_288_57176, &P22373_287};	/*Property = CssFont*/
const Value V22373_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22373_289 = {S12, 0, &V22373_289_57759, &P22373_288};	/*Property = CssImages*/
const Value V22373_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22373_291 = {S13, 0, &V22373_291_57180, &P22373_289};	/*Property = CssMediaQueries*/
const Value V22373_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22373_292 = {S14, 0, &V22373_292_57183, &P22373_291};	/*Property = CssMinMax*/
const Value V22373_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22373_293 = {S15, 0, &V22373_293_57187, &P22373_292};	/*Property = CssOverflow*/
const Value V22373_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22373_294 = {S16, 0, &V22373_294_57189, &P22373_293};	/*Property = CssPosition*/
const Value V22373_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22373_295 = {S17, 0, &V22373_295_57192, &P22373_294};	/*Property = CssText*/
const Value V22373_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22373_297 = {S18, 0, &V22373_297_57195, &P22373_295};	/*Property = CssTransforms*/
const Value V22373_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22373_241 = {S19, 0, &V22373_241_55799, &P22373_297};	/*Property = CssTransitions*/
const Value V22373_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22373_298 = {S20, 0, &V22373_298_57206, &P22373_241};	/*Property = CssUI*/
const Value V22373_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22373_300 = {S21, 0, &V22373_300_57216, &P22373_298};	/*Property = DataSet*/
const Value V22373_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22373_301 = {S22, 0, &V22373_301_57218, &P22373_300};	/*Property = DataUrl*/
const Value V22373_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22373_302 = {S23, 0, &V22373_302_57222, &P22373_301};	/*Property = DeviceOrientation*/
const Value V22373_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22373_304 = {S24, 0, &V22373_304_57227, &P22373_302};	/*Property = FileReader*/
const Value V22373_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22373_305 = {S25, 0, &V22373_305_57231, &P22373_304};	/*Property = FileSaver*/
const Value V22373_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22373_306 = {S26, 0, &V22373_306_57237, &P22373_305};	/*Property = FileWriter*/
const Value V22373_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22373_307 = {S27, 0, &V22373_307_57239, &P22373_306};	/*Property = FormData*/
const Value V22373_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22373_309 = {S28, 0, &V22373_309_57246, &P22373_307};	/*Property = Fullscreen*/
const Value V22373_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22373_310 = {S29, 0, &V22373_310_57248, &P22373_309};	/*Property = GeoLocation*/
const Value V22373_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22373_312 = {S30, 0, &V22373_312_57254, &P22373_310};	/*Property = History*/
const Value V22373_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22373_314 = {S31, 0, &V22373_314_57261, &P22373_312};	/*Property = Html5*/
const Value V22373_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22373_313 = {S32, 0, &V22373_313_57258, &P22373_314};	/*Property = Html-Media-Capture*/
const Value V22373_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22373_315 = {S33, 0, &V22373_315_57264, &P22373_313};	/*Property = Iframe*/
const Value V22373_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22373_316 = {S34, 0, &V22373_316_57266, &P22373_315};	/*Property = IndexedDB*/
const Value V22373_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22373_317 = {S35, 0, &V22373_317_57269, &P22373_316};	/*Property = Json*/
const Value V22373_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22373_15 = {S37, 0, &V22373_15_28, &P22373_317};	/*Property = LayoutEngine*/
const Value V22373_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22373_344 = {S38, 0, &V22373_344_57272, &P22373_15};	/*Property = Masking*/
const Value V22373_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22373_320 = {S39, 0, &V22373_320_57290, &P22373_344};	/*Property = PostMessage*/
const Value V22373_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22373_321 = {S40, 0, &V22373_321_57293, &P22373_320};	/*Property = Progress*/
const Value V22373_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22373_322 = {S41, 0, &V22373_322_57296, &P22373_321};	/*Property = Prompts*/
const Value V22373_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22373_323 = {S42, 0, &V22373_323_57299, &P22373_322};	/*Property = Selector*/
const Value V22373_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22373_324 = {S43, 0, &V22373_324_57306, &P22373_323};	/*Property = Svg*/
const Value V22373_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22373_327 = {S44, 0, &V22373_327_57315, &P22373_324};	/*Property = TouchEvents*/
const Value V22373_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22373_328 = {S45, 0, &V22373_328_57318, &P22373_327};	/*Property = Track*/
const Value V22373_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22373_329 = {S46, 0, &V22373_329_57323, &P22373_328};	/*Property = Video*/
const Value V22373_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22373_330 = {S47, 0, &V22373_330_57326, &P22373_329};	/*Property = Viewport*/
const Value V22373_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22373_332 = {S48, 0, &V22373_332_57340, &P22373_330};	/*Property = WebWorkers*/
const Value V22373_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22373_333 = {S49, 0, &V22373_333_57343, &P22373_332};	/*Property = Xhr2*/
const Profile P22373 = {"22373", &P22373_333};	/*Profile ID - 22373*/
const Value V21409_1_1 = {S2, NULL};	/*Value = True*/
const Property P21409_1 = {S50, 0, &V21409_1_1, NULL};	/*Property = IsMobile*/
const Value V21409_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21409_5 = {S52, 0, &V21409_5_1227, &P21409_1};	/*Property = ScreenPixelsHeight*/
const Value V21409_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21409_4 = {S54, 0, &V21409_4_1229, &P21409_5};	/*Property = ScreenPixelsWidth*/
const Profile P21409 = {"21409", &P21409_4};	/*Profile ID - 21409*/
const Profile P24094 = {"24094", NULL};	/*Profile ID - 24094*/
const Value V17565_1_1 = {S2, NULL};	/*Value = True*/
const Property P17565_1 = {S50, 0, &V17565_1_1, NULL};	/*Property = IsMobile*/
const Value V17565_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17565_5 = {S52, 0, &V17565_5_1269, &P17565_1};	/*Property = ScreenPixelsHeight*/
const Value V17565_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17565_4 = {S54, 0, &V17565_4_1229, &P17565_5};	/*Property = ScreenPixelsWidth*/
const Profile P17565 = {"17565", &P17565_4};	/*Profile ID - 17565*/
const Profile P17816 = {"17816", NULL};	/*Profile ID - 17816*/
const Value V18211_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_279 = {S1, 0, &V18211_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18211_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_282 = {S3, 0, &V18211_282_57155, &P18211_279};	/*Property = BlobBuilder*/
const Value V18211_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_350 = {S4, 0, &V18211_350_57446, &P18211_282};	/*Property = Canvas*/
const Value V18211_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_283 = {S5, 0, &V18211_283_57161, &P18211_350};	/*Property = CssBackground*/
const Value V18211_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_284 = {S6, 0, &V18211_284_57164, &P18211_283};	/*Property = CssBorderImage*/
const Value V18211_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_285 = {S7, 0, &V18211_285_57167, &P18211_284};	/*Property = CssCanvas*/
const Value V18211_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_286 = {S8, 0, &V18211_286_57170, &P18211_285};	/*Property = CssColor*/
const Value V18211_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_237 = {S9, 0, &V18211_237_55786, &P18211_286};	/*Property = CssColumn*/
const Value V18211_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_287 = {S10, 0, &V18211_287_57173, &P18211_237};	/*Property = CssFlexbox*/
const Value V18211_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_288 = {S11, 0, &V18211_288_57176, &P18211_287};	/*Property = CssFont*/
const Value V18211_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_289 = {S12, 0, &V18211_289_86899, &P18211_288};	/*Property = CssImages*/
const Value V18211_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_291 = {S13, 0, &V18211_291_57182, &P18211_289};	/*Property = CssMediaQueries*/
const Value V18211_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_292 = {S14, 0, &V18211_292_57185, &P18211_291};	/*Property = CssMinMax*/
const Value V18211_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_293 = {S15, 0, &V18211_293_57188, &P18211_292};	/*Property = CssOverflow*/
const Value V18211_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_294 = {S16, 0, &V18211_294_57191, &P18211_293};	/*Property = CssPosition*/
const Value V18211_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_295 = {S17, 0, &V18211_295_57194, &P18211_294};	/*Property = CssText*/
const Value V18211_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_297 = {S18, 0, &V18211_297_57197, &P18211_295};	/*Property = CssTransforms*/
const Value V18211_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_241 = {S19, 0, &V18211_241_55801, &P18211_297};	/*Property = CssTransitions*/
const Value V18211_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_298 = {S20, 0, &V18211_298_57208, &P18211_241};	/*Property = CssUI*/
const Value V18211_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_300 = {S21, 0, &V18211_300_57217, &P18211_298};	/*Property = DataSet*/
const Value V18211_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_301 = {S22, 0, &V18211_301_57220, &P18211_300};	/*Property = DataUrl*/
const Value V18211_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_302 = {S23, 0, &V18211_302_57223, &P18211_301};	/*Property = DeviceOrientation*/
const Value V18211_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_304 = {S24, 0, &V18211_304_57229, &P18211_302};	/*Property = FileReader*/
const Value V18211_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_305 = {S25, 0, &V18211_305_57232, &P18211_304};	/*Property = FileSaver*/
const Value V18211_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_306 = {S26, 0, &V18211_306_57238, &P18211_305};	/*Property = FileWriter*/
const Value V18211_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_307 = {S27, 0, &V18211_307_57241, &P18211_306};	/*Property = FormData*/
const Value V18211_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_309 = {S28, 0, &V18211_309_57247, &P18211_307};	/*Property = Fullscreen*/
const Value V18211_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_310 = {S29, 0, &V18211_310_57250, &P18211_309};	/*Property = GeoLocation*/
const Value V18211_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_312 = {S30, 0, &V18211_312_57256, &P18211_310};	/*Property = History*/
const Value V18211_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_314 = {S31, 0, &V18211_314_57262, &P18211_312};	/*Property = Html5*/
const Value V18211_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_313 = {S32, 0, &V18211_313_57259, &P18211_314};	/*Property = Html-Media-Capture*/
const Value V18211_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_315 = {S33, 0, &V18211_315_57265, &P18211_313};	/*Property = Iframe*/
const Value V18211_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_316 = {S34, 0, &V18211_316_57268, &P18211_315};	/*Property = IndexedDB*/
const Value V18211_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_317 = {S35, 0, &V18211_317_57271, &P18211_316};	/*Property = Json*/
const Value V18211_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18211_15 = {S37, 0, &V18211_15_29, &P18211_317};	/*Property = LayoutEngine*/
const Value V18211_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_344 = {S38, 0, &V18211_344_57274, &P18211_15};	/*Property = Masking*/
const Value V18211_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_320 = {S39, 0, &V18211_320_57292, &P18211_344};	/*Property = PostMessage*/
const Value V18211_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_321 = {S40, 0, &V18211_321_57295, &P18211_320};	/*Property = Progress*/
const Value V18211_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_322 = {S41, 0, &V18211_322_57298, &P18211_321};	/*Property = Prompts*/
const Value V18211_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_323 = {S42, 0, &V18211_323_57301, &P18211_322};	/*Property = Selector*/
const Value V18211_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_324 = {S43, 0, &V18211_324_57307, &P18211_323};	/*Property = Svg*/
const Value V18211_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_327 = {S44, 0, &V18211_327_57316, &P18211_324};	/*Property = TouchEvents*/
const Value V18211_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_328 = {S45, 0, &V18211_328_57319, &P18211_327};	/*Property = Track*/
const Value V18211_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_329 = {S46, 0, &V18211_329_57325, &P18211_328};	/*Property = Video*/
const Value V18211_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_330 = {S47, 0, &V18211_330_57328, &P18211_329};	/*Property = Viewport*/
const Value V18211_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_332 = {S48, 0, &V18211_332_57340, &P18211_330};	/*Property = WebWorkers*/
const Value V18211_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18211_333 = {S49, 0, &V18211_333_57343, &P18211_332};	/*Property = Xhr2*/
const Profile P18211 = {"18211", &P18211_333};	/*Profile ID - 18211*/
const Value V20805_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_279 = {S1, 0, &V20805_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20805_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_282 = {S3, 0, &V20805_282_57155, &P20805_279};	/*Property = BlobBuilder*/
const Value V20805_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_350 = {S4, 0, &V20805_350_57446, &P20805_282};	/*Property = Canvas*/
const Value V20805_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_283 = {S5, 0, &V20805_283_57161, &P20805_350};	/*Property = CssBackground*/
const Value V20805_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_284 = {S6, 0, &V20805_284_57164, &P20805_283};	/*Property = CssBorderImage*/
const Value V20805_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_285 = {S7, 0, &V20805_285_57167, &P20805_284};	/*Property = CssCanvas*/
const Value V20805_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_286 = {S8, 0, &V20805_286_57170, &P20805_285};	/*Property = CssColor*/
const Value V20805_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_237 = {S9, 0, &V20805_237_55786, &P20805_286};	/*Property = CssColumn*/
const Value V20805_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_287 = {S10, 0, &V20805_287_57173, &P20805_237};	/*Property = CssFlexbox*/
const Value V20805_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_288 = {S11, 0, &V20805_288_57176, &P20805_287};	/*Property = CssFont*/
const Value V20805_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_289 = {S12, 0, &V20805_289_86899, &P20805_288};	/*Property = CssImages*/
const Value V20805_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_291 = {S13, 0, &V20805_291_57182, &P20805_289};	/*Property = CssMediaQueries*/
const Value V20805_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_292 = {S14, 0, &V20805_292_57185, &P20805_291};	/*Property = CssMinMax*/
const Value V20805_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_293 = {S15, 0, &V20805_293_57188, &P20805_292};	/*Property = CssOverflow*/
const Value V20805_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_294 = {S16, 0, &V20805_294_57191, &P20805_293};	/*Property = CssPosition*/
const Value V20805_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_295 = {S17, 0, &V20805_295_57194, &P20805_294};	/*Property = CssText*/
const Value V20805_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_297 = {S18, 0, &V20805_297_57197, &P20805_295};	/*Property = CssTransforms*/
const Value V20805_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_241 = {S19, 0, &V20805_241_55801, &P20805_297};	/*Property = CssTransitions*/
const Value V20805_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_298 = {S20, 0, &V20805_298_57208, &P20805_241};	/*Property = CssUI*/
const Value V20805_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_300 = {S21, 0, &V20805_300_57217, &P20805_298};	/*Property = DataSet*/
const Value V20805_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_301 = {S22, 0, &V20805_301_57220, &P20805_300};	/*Property = DataUrl*/
const Value V20805_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_302 = {S23, 0, &V20805_302_57223, &P20805_301};	/*Property = DeviceOrientation*/
const Value V20805_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_304 = {S24, 0, &V20805_304_57229, &P20805_302};	/*Property = FileReader*/
const Value V20805_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_305 = {S25, 0, &V20805_305_57232, &P20805_304};	/*Property = FileSaver*/
const Value V20805_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_306 = {S26, 0, &V20805_306_57238, &P20805_305};	/*Property = FileWriter*/
const Value V20805_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_307 = {S27, 0, &V20805_307_57241, &P20805_306};	/*Property = FormData*/
const Value V20805_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_309 = {S28, 0, &V20805_309_57247, &P20805_307};	/*Property = Fullscreen*/
const Value V20805_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_310 = {S29, 0, &V20805_310_57250, &P20805_309};	/*Property = GeoLocation*/
const Value V20805_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_312 = {S30, 0, &V20805_312_57256, &P20805_310};	/*Property = History*/
const Value V20805_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_314 = {S31, 0, &V20805_314_57262, &P20805_312};	/*Property = Html5*/
const Value V20805_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_313 = {S32, 0, &V20805_313_57259, &P20805_314};	/*Property = Html-Media-Capture*/
const Value V20805_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_315 = {S33, 0, &V20805_315_57265, &P20805_313};	/*Property = Iframe*/
const Value V20805_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_316 = {S34, 0, &V20805_316_57268, &P20805_315};	/*Property = IndexedDB*/
const Value V20805_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_317 = {S35, 0, &V20805_317_57271, &P20805_316};	/*Property = Json*/
const Value V20805_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_15 = {S37, 0, &V20805_15_72, &P20805_317};	/*Property = LayoutEngine*/
const Value V20805_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_344 = {S38, 0, &V20805_344_57274, &P20805_15};	/*Property = Masking*/
const Value V20805_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_320 = {S39, 0, &V20805_320_57292, &P20805_344};	/*Property = PostMessage*/
const Value V20805_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_321 = {S40, 0, &V20805_321_57295, &P20805_320};	/*Property = Progress*/
const Value V20805_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_322 = {S41, 0, &V20805_322_57298, &P20805_321};	/*Property = Prompts*/
const Value V20805_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_323 = {S42, 0, &V20805_323_57301, &P20805_322};	/*Property = Selector*/
const Value V20805_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_324 = {S43, 0, &V20805_324_57307, &P20805_323};	/*Property = Svg*/
const Value V20805_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_327 = {S44, 0, &V20805_327_57316, &P20805_324};	/*Property = TouchEvents*/
const Value V20805_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_328 = {S45, 0, &V20805_328_57319, &P20805_327};	/*Property = Track*/
const Value V20805_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_329 = {S46, 0, &V20805_329_57325, &P20805_328};	/*Property = Video*/
const Value V20805_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_330 = {S47, 0, &V20805_330_57328, &P20805_329};	/*Property = Viewport*/
const Value V20805_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_332 = {S48, 0, &V20805_332_57340, &P20805_330};	/*Property = WebWorkers*/
const Value V20805_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20805_333 = {S49, 0, &V20805_333_57343, &P20805_332};	/*Property = Xhr2*/
const Profile P20805 = {"20805", &P20805_333};	/*Profile ID - 20805*/
const Value V18020_1_1 = {S2, NULL};	/*Value = True*/
const Property P18020_1 = {S50, 0, &V18020_1_1, NULL};	/*Property = IsMobile*/
const Value V18020_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18020_5 = {S52, 0, &V18020_5_1277, &P18020_1};	/*Property = ScreenPixelsHeight*/
const Value V18020_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18020_4 = {S54, 0, &V18020_4_1224, &P18020_5};	/*Property = ScreenPixelsWidth*/
const Profile P18020 = {"18020", &P18020_4};	/*Profile ID - 18020*/
const Value V17892_1_1 = {S2, NULL};	/*Value = True*/
const Property P17892_1 = {S50, 0, &V17892_1_1, NULL};	/*Property = IsMobile*/
const Value V17892_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P17892_5 = {S52, 0, &V17892_5_1333, &P17892_1};	/*Property = ScreenPixelsHeight*/
const Value V17892_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P17892_4 = {S54, 0, &V17892_4_1335, &P17892_5};	/*Property = ScreenPixelsWidth*/
const Profile P17892 = {"17892", &P17892_4};	/*Profile ID - 17892*/
const Value V17584_1_1 = {S2, NULL};	/*Value = True*/
const Property P17584_1 = {S50, 0, &V17584_1_1, NULL};	/*Property = IsMobile*/
const Value V17584_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17584_5 = {S52, 0, &V17584_5_1269, &P17584_1};	/*Property = ScreenPixelsHeight*/
const Value V17584_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17584_4 = {S54, 0, &V17584_4_1229, &P17584_5};	/*Property = ScreenPixelsWidth*/
const Profile P17584 = {"17584", &P17584_4};	/*Profile ID - 17584*/
const Value V23545_1_1 = {S2, NULL};	/*Value = True*/
const Property P23545_1 = {S50, 0, &V23545_1_1, NULL};	/*Property = IsMobile*/
const Value V23545_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P23545_5 = {S52, 0, &V23545_5_1304, &P23545_1};	/*Property = ScreenPixelsHeight*/
const Value V23545_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23545_4 = {S54, 0, &V23545_4_1224, &P23545_5};	/*Property = ScreenPixelsWidth*/
const Profile P23545 = {"23545", &P23545_4};	/*Profile ID - 23545*/
const Profile P18121 = {"18121", NULL};	/*Profile ID - 18121*/
const Value V17898_1_1 = {S2, NULL};	/*Value = True*/
const Property P17898_1 = {S50, 0, &V17898_1_1, NULL};	/*Property = IsMobile*/
const Value V17898_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P17898_5 = {S52, 0, &V17898_5_1294, &P17898_1};	/*Property = ScreenPixelsHeight*/
const Value V17898_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17898_4 = {S54, 0, &V17898_4_1296, &P17898_5};	/*Property = ScreenPixelsWidth*/
const Profile P17898 = {"17898", &P17898_4};	/*Profile ID - 17898*/
const Profile P17817 = {"17817", NULL};	/*Profile ID - 17817*/
const Value V16744_1_1 = {S2, NULL};	/*Value = True*/
const Property P16744_1 = {S50, 0, &V16744_1_1, NULL};	/*Property = IsMobile*/
const Value V16744_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16744_5 = {S52, 0, &V16744_5_1227, &P16744_1};	/*Property = ScreenPixelsHeight*/
const Value V16744_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16744_4 = {S54, 0, &V16744_4_1229, &P16744_5};	/*Property = ScreenPixelsWidth*/
const Profile P16744 = {"16744", &P16744_4};	/*Profile ID - 16744*/
const Value V17552_1_1 = {S2, NULL};	/*Value = True*/
const Property P17552_1 = {S50, 0, &V17552_1_1, NULL};	/*Property = IsMobile*/
const Value V17552_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17552_5 = {S52, 0, &V17552_5_1269, &P17552_1};	/*Property = ScreenPixelsHeight*/
const Value V17552_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17552_4 = {S54, 0, &V17552_4_1229, &P17552_5};	/*Property = ScreenPixelsWidth*/
const Profile P17552 = {"17552", &P17552_4};	/*Profile ID - 17552*/
const Profile P18585 = {"18585", NULL};	/*Profile ID - 18585*/
const Value V18335_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_279 = {S1, 0, &V18335_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18335_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_282 = {S3, 0, &V18335_282_57155, &P18335_279};	/*Property = BlobBuilder*/
const Value V18335_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_350 = {S4, 0, &V18335_350_57446, &P18335_282};	/*Property = Canvas*/
const Value V18335_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_283 = {S5, 0, &V18335_283_57161, &P18335_350};	/*Property = CssBackground*/
const Value V18335_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_284 = {S6, 0, &V18335_284_57164, &P18335_283};	/*Property = CssBorderImage*/
const Value V18335_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_285 = {S7, 0, &V18335_285_57167, &P18335_284};	/*Property = CssCanvas*/
const Value V18335_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_286 = {S8, 0, &V18335_286_57170, &P18335_285};	/*Property = CssColor*/
const Value V18335_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_237 = {S9, 0, &V18335_237_55786, &P18335_286};	/*Property = CssColumn*/
const Value V18335_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_287 = {S10, 0, &V18335_287_57173, &P18335_237};	/*Property = CssFlexbox*/
const Value V18335_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_288 = {S11, 0, &V18335_288_57176, &P18335_287};	/*Property = CssFont*/
const Value V18335_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_289 = {S12, 0, &V18335_289_86899, &P18335_288};	/*Property = CssImages*/
const Value V18335_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_291 = {S13, 0, &V18335_291_57182, &P18335_289};	/*Property = CssMediaQueries*/
const Value V18335_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_292 = {S14, 0, &V18335_292_57185, &P18335_291};	/*Property = CssMinMax*/
const Value V18335_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_293 = {S15, 0, &V18335_293_57188, &P18335_292};	/*Property = CssOverflow*/
const Value V18335_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_294 = {S16, 0, &V18335_294_57191, &P18335_293};	/*Property = CssPosition*/
const Value V18335_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_295 = {S17, 0, &V18335_295_57194, &P18335_294};	/*Property = CssText*/
const Value V18335_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_297 = {S18, 0, &V18335_297_57197, &P18335_295};	/*Property = CssTransforms*/
const Value V18335_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_241 = {S19, 0, &V18335_241_55801, &P18335_297};	/*Property = CssTransitions*/
const Value V18335_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_298 = {S20, 0, &V18335_298_57208, &P18335_241};	/*Property = CssUI*/
const Value V18335_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_300 = {S21, 0, &V18335_300_57217, &P18335_298};	/*Property = DataSet*/
const Value V18335_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_301 = {S22, 0, &V18335_301_57220, &P18335_300};	/*Property = DataUrl*/
const Value V18335_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_302 = {S23, 0, &V18335_302_57223, &P18335_301};	/*Property = DeviceOrientation*/
const Value V18335_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_304 = {S24, 0, &V18335_304_57229, &P18335_302};	/*Property = FileReader*/
const Value V18335_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_305 = {S25, 0, &V18335_305_57232, &P18335_304};	/*Property = FileSaver*/
const Value V18335_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_306 = {S26, 0, &V18335_306_57238, &P18335_305};	/*Property = FileWriter*/
const Value V18335_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_307 = {S27, 0, &V18335_307_57241, &P18335_306};	/*Property = FormData*/
const Value V18335_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_309 = {S28, 0, &V18335_309_57247, &P18335_307};	/*Property = Fullscreen*/
const Value V18335_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_310 = {S29, 0, &V18335_310_57250, &P18335_309};	/*Property = GeoLocation*/
const Value V18335_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_312 = {S30, 0, &V18335_312_57256, &P18335_310};	/*Property = History*/
const Value V18335_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_314 = {S31, 0, &V18335_314_57262, &P18335_312};	/*Property = Html5*/
const Value V18335_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_313 = {S32, 0, &V18335_313_57259, &P18335_314};	/*Property = Html-Media-Capture*/
const Value V18335_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_315 = {S33, 0, &V18335_315_57265, &P18335_313};	/*Property = Iframe*/
const Value V18335_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_316 = {S34, 0, &V18335_316_57268, &P18335_315};	/*Property = IndexedDB*/
const Value V18335_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_317 = {S35, 0, &V18335_317_57271, &P18335_316};	/*Property = Json*/
const Value V18335_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_15 = {S37, 0, &V18335_15_72, &P18335_317};	/*Property = LayoutEngine*/
const Value V18335_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_344 = {S38, 0, &V18335_344_57274, &P18335_15};	/*Property = Masking*/
const Value V18335_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_320 = {S39, 0, &V18335_320_57292, &P18335_344};	/*Property = PostMessage*/
const Value V18335_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_321 = {S40, 0, &V18335_321_57295, &P18335_320};	/*Property = Progress*/
const Value V18335_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_322 = {S41, 0, &V18335_322_57298, &P18335_321};	/*Property = Prompts*/
const Value V18335_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_323 = {S42, 0, &V18335_323_57301, &P18335_322};	/*Property = Selector*/
const Value V18335_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_324 = {S43, 0, &V18335_324_57307, &P18335_323};	/*Property = Svg*/
const Value V18335_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_327 = {S44, 0, &V18335_327_57316, &P18335_324};	/*Property = TouchEvents*/
const Value V18335_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_328 = {S45, 0, &V18335_328_57319, &P18335_327};	/*Property = Track*/
const Value V18335_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_329 = {S46, 0, &V18335_329_57325, &P18335_328};	/*Property = Video*/
const Value V18335_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_330 = {S47, 0, &V18335_330_57328, &P18335_329};	/*Property = Viewport*/
const Value V18335_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_332 = {S48, 0, &V18335_332_57340, &P18335_330};	/*Property = WebWorkers*/
const Value V18335_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18335_333 = {S49, 0, &V18335_333_57343, &P18335_332};	/*Property = Xhr2*/
const Profile P18335 = {"18335", &P18335_333};	/*Profile ID - 18335*/
const Value V17982_1_1 = {S2, NULL};	/*Value = True*/
const Property P17982_1 = {S50, 0, &V17982_1_1, NULL};	/*Property = IsMobile*/
const Value V17982_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P17982_5 = {S52, 0, &V17982_5_1294, &P17982_1};	/*Property = ScreenPixelsHeight*/
const Value V17982_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17982_4 = {S54, 0, &V17982_4_1224, &P17982_5};	/*Property = ScreenPixelsWidth*/
const Profile P17982 = {"17982", &P17982_4};	/*Profile ID - 17982*/
const Value V18350_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18350_279 = {S1, 0, &V18350_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18350_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18350_282 = {S3, 0, &V18350_282_57154, &P18350_279};	/*Property = BlobBuilder*/
const Value V18350_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18350_350 = {S4, 0, &V18350_350_57444, &P18350_282};	/*Property = Canvas*/
const Value V18350_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18350_283 = {S5, 0, &V18350_283_57159, &P18350_350};	/*Property = CssBackground*/
const Value V18350_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18350_284 = {S6, 0, &V18350_284_57163, &P18350_283};	/*Property = CssBorderImage*/
const Value V18350_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18350_285 = {S7, 0, &V18350_285_57166, &P18350_284};	/*Property = CssCanvas*/
const Value V18350_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18350_286 = {S8, 0, &V18350_286_57168, &P18350_285};	/*Property = CssColor*/
const Value V18350_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18350_237 = {S9, 0, &V18350_237_55786, &P18350_286};	/*Property = CssColumn*/
const Value V18350_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18350_287 = {S10, 0, &V18350_287_57172, &P18350_237};	/*Property = CssFlexbox*/
const Value V18350_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18350_288 = {S11, 0, &V18350_288_57175, &P18350_287};	/*Property = CssFont*/
const Value V18350_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18350_289 = {S12, 0, &V18350_289_57759, &P18350_288};	/*Property = CssImages*/
const Value V18350_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18350_291 = {S13, 0, &V18350_291_57181, &P18350_289};	/*Property = CssMediaQueries*/
const Value V18350_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18350_292 = {S14, 0, &V18350_292_57183, &P18350_291};	/*Property = CssMinMax*/
const Value V18350_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18350_293 = {S15, 0, &V18350_293_57187, &P18350_292};	/*Property = CssOverflow*/
const Value V18350_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18350_294 = {S16, 0, &V18350_294_57190, &P18350_293};	/*Property = CssPosition*/
const Value V18350_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18350_295 = {S17, 0, &V18350_295_57193, &P18350_294};	/*Property = CssText*/
const Value V18350_297_57196 = {S0, NULL};	/*Value = False*/
const Property P18350_297 = {S18, 0, &V18350_297_57196, &P18350_295};	/*Property = CssTransforms*/
const Value V18350_241_55800 = {S0, NULL};	/*Value = False*/
const Property P18350_241 = {S19, 0, &V18350_241_55800, &P18350_297};	/*Property = CssTransitions*/
const Value V18350_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18350_298 = {S20, 0, &V18350_298_57206, &P18350_241};	/*Property = CssUI*/
const Value V18350_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18350_300 = {S21, 0, &V18350_300_57216, &P18350_298};	/*Property = DataSet*/
const Value V18350_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18350_301 = {S22, 0, &V18350_301_57218, &P18350_300};	/*Property = DataUrl*/
const Value V18350_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18350_302 = {S23, 0, &V18350_302_57222, &P18350_301};	/*Property = DeviceOrientation*/
const Value V18350_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18350_304 = {S24, 0, &V18350_304_57228, &P18350_302};	/*Property = FileReader*/
const Value V18350_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18350_305 = {S25, 0, &V18350_305_57231, &P18350_304};	/*Property = FileSaver*/
const Value V18350_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18350_306 = {S26, 0, &V18350_306_57237, &P18350_305};	/*Property = FileWriter*/
const Value V18350_307_57240 = {S0, NULL};	/*Value = False*/
const Property P18350_307 = {S27, 0, &V18350_307_57240, &P18350_306};	/*Property = FormData*/
const Value V18350_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18350_309 = {S28, 0, &V18350_309_57246, &P18350_307};	/*Property = Fullscreen*/
const Value V18350_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18350_310 = {S29, 0, &V18350_310_57248, &P18350_309};	/*Property = GeoLocation*/
const Value V18350_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18350_312 = {S30, 0, &V18350_312_57255, &P18350_310};	/*Property = History*/
const Value V18350_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18350_314 = {S31, 0, &V18350_314_57261, &P18350_312};	/*Property = Html5*/
const Value V18350_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18350_313 = {S32, 0, &V18350_313_57258, &P18350_314};	/*Property = Html-Media-Capture*/
const Value V18350_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18350_315 = {S33, 0, &V18350_315_57264, &P18350_313};	/*Property = Iframe*/
const Value V18350_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18350_316 = {S34, 0, &V18350_316_57267, &P18350_315};	/*Property = IndexedDB*/
const Value V18350_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18350_317 = {S35, 0, &V18350_317_57269, &P18350_316};	/*Property = Json*/
const Value V18350_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P18350_15 = {S37, 0, &V18350_15_30, &P18350_317};	/*Property = LayoutEngine*/
const Value V18350_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18350_344 = {S38, 0, &V18350_344_57272, &P18350_15};	/*Property = Masking*/
const Value V18350_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18350_320 = {S39, 0, &V18350_320_57290, &P18350_344};	/*Property = PostMessage*/
const Value V18350_321_57294 = {S0, NULL};	/*Value = False*/
const Property P18350_321 = {S40, 0, &V18350_321_57294, &P18350_320};	/*Property = Progress*/
const Value V18350_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18350_322 = {S41, 0, &V18350_322_57296, &P18350_321};	/*Property = Prompts*/
const Value V18350_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18350_323 = {S42, 0, &V18350_323_57299, &P18350_322};	/*Property = Selector*/
const Value V18350_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18350_324 = {S43, 0, &V18350_324_57306, &P18350_323};	/*Property = Svg*/
const Value V18350_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18350_327 = {S44, 0, &V18350_327_57315, &P18350_324};	/*Property = TouchEvents*/
const Value V18350_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18350_328 = {S45, 0, &V18350_328_57318, &P18350_327};	/*Property = Track*/
const Value V18350_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18350_329 = {S46, 0, &V18350_329_57323, &P18350_328};	/*Property = Video*/
const Value V18350_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18350_330 = {S47, 0, &V18350_330_57326, &P18350_329};	/*Property = Viewport*/
const Value V18350_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18350_332 = {S48, 0, &V18350_332_57339, &P18350_330};	/*Property = WebWorkers*/
const Value V18350_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18350_333 = {S49, 0, &V18350_333_57342, &P18350_332};	/*Property = Xhr2*/
const Profile P18350 = {"18350", &P18350_333};	/*Profile ID - 18350*/
const Value V17217_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_279 = {S1, 0, &V17217_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17217_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_282 = {S3, 0, &V17217_282_57155, &P17217_279};	/*Property = BlobBuilder*/
const Value V17217_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_350 = {S4, 0, &V17217_350_57446, &P17217_282};	/*Property = Canvas*/
const Value V17217_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_283 = {S5, 0, &V17217_283_57161, &P17217_350};	/*Property = CssBackground*/
const Value V17217_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_284 = {S6, 0, &V17217_284_57164, &P17217_283};	/*Property = CssBorderImage*/
const Value V17217_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_285 = {S7, 0, &V17217_285_57167, &P17217_284};	/*Property = CssCanvas*/
const Value V17217_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_286 = {S8, 0, &V17217_286_57170, &P17217_285};	/*Property = CssColor*/
const Value V17217_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_237 = {S9, 0, &V17217_237_55786, &P17217_286};	/*Property = CssColumn*/
const Value V17217_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_287 = {S10, 0, &V17217_287_57173, &P17217_237};	/*Property = CssFlexbox*/
const Value V17217_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_288 = {S11, 0, &V17217_288_57176, &P17217_287};	/*Property = CssFont*/
const Value V17217_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_289 = {S12, 0, &V17217_289_86899, &P17217_288};	/*Property = CssImages*/
const Value V17217_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_291 = {S13, 0, &V17217_291_57182, &P17217_289};	/*Property = CssMediaQueries*/
const Value V17217_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_292 = {S14, 0, &V17217_292_57185, &P17217_291};	/*Property = CssMinMax*/
const Value V17217_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_293 = {S15, 0, &V17217_293_57188, &P17217_292};	/*Property = CssOverflow*/
const Value V17217_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_294 = {S16, 0, &V17217_294_57191, &P17217_293};	/*Property = CssPosition*/
const Value V17217_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_295 = {S17, 0, &V17217_295_57194, &P17217_294};	/*Property = CssText*/
const Value V17217_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_297 = {S18, 0, &V17217_297_57197, &P17217_295};	/*Property = CssTransforms*/
const Value V17217_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_241 = {S19, 0, &V17217_241_55801, &P17217_297};	/*Property = CssTransitions*/
const Value V17217_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_298 = {S20, 0, &V17217_298_57208, &P17217_241};	/*Property = CssUI*/
const Value V17217_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_300 = {S21, 0, &V17217_300_57217, &P17217_298};	/*Property = DataSet*/
const Value V17217_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_301 = {S22, 0, &V17217_301_57220, &P17217_300};	/*Property = DataUrl*/
const Value V17217_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_302 = {S23, 0, &V17217_302_57223, &P17217_301};	/*Property = DeviceOrientation*/
const Value V17217_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_304 = {S24, 0, &V17217_304_57229, &P17217_302};	/*Property = FileReader*/
const Value V17217_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_305 = {S25, 0, &V17217_305_57232, &P17217_304};	/*Property = FileSaver*/
const Value V17217_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_306 = {S26, 0, &V17217_306_57238, &P17217_305};	/*Property = FileWriter*/
const Value V17217_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_307 = {S27, 0, &V17217_307_57241, &P17217_306};	/*Property = FormData*/
const Value V17217_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_309 = {S28, 0, &V17217_309_57247, &P17217_307};	/*Property = Fullscreen*/
const Value V17217_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_310 = {S29, 0, &V17217_310_57250, &P17217_309};	/*Property = GeoLocation*/
const Value V17217_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_312 = {S30, 0, &V17217_312_57256, &P17217_310};	/*Property = History*/
const Value V17217_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_314 = {S31, 0, &V17217_314_57262, &P17217_312};	/*Property = Html5*/
const Value V17217_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_313 = {S32, 0, &V17217_313_57259, &P17217_314};	/*Property = Html-Media-Capture*/
const Value V17217_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_315 = {S33, 0, &V17217_315_57265, &P17217_313};	/*Property = Iframe*/
const Value V17217_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_316 = {S34, 0, &V17217_316_57268, &P17217_315};	/*Property = IndexedDB*/
const Value V17217_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_317 = {S35, 0, &V17217_317_57271, &P17217_316};	/*Property = Json*/
const Value V17217_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P17217_15 = {S37, 0, &V17217_15_31, &P17217_317};	/*Property = LayoutEngine*/
const Value V17217_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_344 = {S38, 0, &V17217_344_57274, &P17217_15};	/*Property = Masking*/
const Value V17217_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_320 = {S39, 0, &V17217_320_57292, &P17217_344};	/*Property = PostMessage*/
const Value V17217_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_321 = {S40, 0, &V17217_321_57295, &P17217_320};	/*Property = Progress*/
const Value V17217_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_322 = {S41, 0, &V17217_322_57298, &P17217_321};	/*Property = Prompts*/
const Value V17217_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_323 = {S42, 0, &V17217_323_57301, &P17217_322};	/*Property = Selector*/
const Value V17217_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_324 = {S43, 0, &V17217_324_57307, &P17217_323};	/*Property = Svg*/
const Value V17217_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_327 = {S44, 0, &V17217_327_57316, &P17217_324};	/*Property = TouchEvents*/
const Value V17217_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_328 = {S45, 0, &V17217_328_57319, &P17217_327};	/*Property = Track*/
const Value V17217_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_329 = {S46, 0, &V17217_329_57325, &P17217_328};	/*Property = Video*/
const Value V17217_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_330 = {S47, 0, &V17217_330_57328, &P17217_329};	/*Property = Viewport*/
const Value V17217_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_332 = {S48, 0, &V17217_332_57340, &P17217_330};	/*Property = WebWorkers*/
const Value V17217_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17217_333 = {S49, 0, &V17217_333_57343, &P17217_332};	/*Property = Xhr2*/
const Profile P17217 = {"17217", &P17217_333};	/*Profile ID - 17217*/
const Value V22445_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22445_279 = {S1, 0, &V22445_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22445_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22445_282 = {S3, 0, &V22445_282_57154, &P22445_279};	/*Property = BlobBuilder*/
const Value V22445_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22445_350 = {S4, 0, &V22445_350_57444, &P22445_282};	/*Property = Canvas*/
const Value V22445_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22445_283 = {S5, 0, &V22445_283_57159, &P22445_350};	/*Property = CssBackground*/
const Value V22445_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22445_284 = {S6, 0, &V22445_284_57163, &P22445_283};	/*Property = CssBorderImage*/
const Value V22445_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22445_285 = {S7, 0, &V22445_285_57166, &P22445_284};	/*Property = CssCanvas*/
const Value V22445_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22445_286 = {S8, 0, &V22445_286_57168, &P22445_285};	/*Property = CssColor*/
const Value V22445_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22445_237 = {S9, 0, &V22445_237_55784, &P22445_286};	/*Property = CssColumn*/
const Value V22445_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22445_287 = {S10, 0, &V22445_287_57172, &P22445_237};	/*Property = CssFlexbox*/
const Value V22445_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22445_288 = {S11, 0, &V22445_288_57175, &P22445_287};	/*Property = CssFont*/
const Value V22445_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22445_289 = {S12, 0, &V22445_289_57759, &P22445_288};	/*Property = CssImages*/
const Value V22445_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22445_291 = {S13, 0, &V22445_291_57181, &P22445_289};	/*Property = CssMediaQueries*/
const Value V22445_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22445_292 = {S14, 0, &V22445_292_57183, &P22445_291};	/*Property = CssMinMax*/
const Value V22445_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22445_293 = {S15, 0, &V22445_293_57187, &P22445_292};	/*Property = CssOverflow*/
const Value V22445_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22445_294 = {S16, 0, &V22445_294_57189, &P22445_293};	/*Property = CssPosition*/
const Value V22445_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22445_295 = {S17, 0, &V22445_295_57192, &P22445_294};	/*Property = CssText*/
const Value V22445_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22445_297 = {S18, 0, &V22445_297_57195, &P22445_295};	/*Property = CssTransforms*/
const Value V22445_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22445_241 = {S19, 0, &V22445_241_55799, &P22445_297};	/*Property = CssTransitions*/
const Value V22445_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22445_298 = {S20, 0, &V22445_298_57206, &P22445_241};	/*Property = CssUI*/
const Value V22445_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22445_300 = {S21, 0, &V22445_300_57216, &P22445_298};	/*Property = DataSet*/
const Value V22445_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22445_301 = {S22, 0, &V22445_301_57218, &P22445_300};	/*Property = DataUrl*/
const Value V22445_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22445_302 = {S23, 0, &V22445_302_57222, &P22445_301};	/*Property = DeviceOrientation*/
const Value V22445_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22445_304 = {S24, 0, &V22445_304_57228, &P22445_302};	/*Property = FileReader*/
const Value V22445_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22445_305 = {S25, 0, &V22445_305_57231, &P22445_304};	/*Property = FileSaver*/
const Value V22445_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22445_306 = {S26, 0, &V22445_306_57237, &P22445_305};	/*Property = FileWriter*/
const Value V22445_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22445_307 = {S27, 0, &V22445_307_57240, &P22445_306};	/*Property = FormData*/
const Value V22445_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22445_309 = {S28, 0, &V22445_309_57246, &P22445_307};	/*Property = Fullscreen*/
const Value V22445_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22445_310 = {S29, 0, &V22445_310_57248, &P22445_309};	/*Property = GeoLocation*/
const Value V22445_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22445_312 = {S30, 0, &V22445_312_57255, &P22445_310};	/*Property = History*/
const Value V22445_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22445_314 = {S31, 0, &V22445_314_57261, &P22445_312};	/*Property = Html5*/
const Value V22445_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22445_313 = {S32, 0, &V22445_313_57258, &P22445_314};	/*Property = Html-Media-Capture*/
const Value V22445_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22445_315 = {S33, 0, &V22445_315_57264, &P22445_313};	/*Property = Iframe*/
const Value V22445_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22445_316 = {S34, 0, &V22445_316_57267, &P22445_315};	/*Property = IndexedDB*/
const Value V22445_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22445_317 = {S35, 0, &V22445_317_57269, &P22445_316};	/*Property = Json*/
const Value V22445_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22445_15 = {S37, 0, &V22445_15_28, &P22445_317};	/*Property = LayoutEngine*/
const Value V22445_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22445_344 = {S38, 0, &V22445_344_57272, &P22445_15};	/*Property = Masking*/
const Value V22445_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22445_320 = {S39, 0, &V22445_320_57290, &P22445_344};	/*Property = PostMessage*/
const Value V22445_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22445_321 = {S40, 0, &V22445_321_57293, &P22445_320};	/*Property = Progress*/
const Value V22445_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22445_322 = {S41, 0, &V22445_322_57296, &P22445_321};	/*Property = Prompts*/
const Value V22445_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22445_323 = {S42, 0, &V22445_323_57299, &P22445_322};	/*Property = Selector*/
const Value V22445_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22445_324 = {S43, 0, &V22445_324_57306, &P22445_323};	/*Property = Svg*/
const Value V22445_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22445_327 = {S44, 0, &V22445_327_57315, &P22445_324};	/*Property = TouchEvents*/
const Value V22445_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22445_328 = {S45, 0, &V22445_328_57318, &P22445_327};	/*Property = Track*/
const Value V22445_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22445_329 = {S46, 0, &V22445_329_57323, &P22445_328};	/*Property = Video*/
const Value V22445_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22445_330 = {S47, 0, &V22445_330_57326, &P22445_329};	/*Property = Viewport*/
const Value V22445_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22445_332 = {S48, 0, &V22445_332_57339, &P22445_330};	/*Property = WebWorkers*/
const Value V22445_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22445_333 = {S49, 0, &V22445_333_57342, &P22445_332};	/*Property = Xhr2*/
const Profile P22445 = {"22445", &P22445_333};	/*Profile ID - 22445*/
const Profile P19832 = {"19832", NULL};	/*Profile ID - 19832*/
const Value V22546_1_1 = {S2, NULL};	/*Value = True*/
const Property P22546_1 = {S50, 0, &V22546_1_1, NULL};	/*Property = IsMobile*/
const Value V22546_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P22546_5 = {S52, 0, &V22546_5_1304, &P22546_1};	/*Property = ScreenPixelsHeight*/
const Value V22546_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22546_4 = {S54, 0, &V22546_4_2336, &P22546_5};	/*Property = ScreenPixelsWidth*/
const Profile P22546 = {"22546", &P22546_4};	/*Profile ID - 22546*/
const Value V17507_1_1 = {S2, NULL};	/*Value = True*/
const Property P17507_1 = {S50, 0, &V17507_1_1, NULL};	/*Property = IsMobile*/
const Value V17507_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17507_5 = {S52, 0, &V17507_5_1304, &P17507_1};	/*Property = ScreenPixelsHeight*/
const Value V17507_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17507_4 = {S54, 0, &V17507_4_1224, &P17507_5};	/*Property = ScreenPixelsWidth*/
const Profile P17507 = {"17507", &P17507_4};	/*Profile ID - 17507*/
const Value V17828_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_279 = {S1, 0, &V17828_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17828_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_282 = {S3, 0, &V17828_282_57155, &P17828_279};	/*Property = BlobBuilder*/
const Value V17828_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_350 = {S4, 0, &V17828_350_57446, &P17828_282};	/*Property = Canvas*/
const Value V17828_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_283 = {S5, 0, &V17828_283_57161, &P17828_350};	/*Property = CssBackground*/
const Value V17828_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_284 = {S6, 0, &V17828_284_57164, &P17828_283};	/*Property = CssBorderImage*/
const Value V17828_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_285 = {S7, 0, &V17828_285_57167, &P17828_284};	/*Property = CssCanvas*/
const Value V17828_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_286 = {S8, 0, &V17828_286_57170, &P17828_285};	/*Property = CssColor*/
const Value V17828_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_237 = {S9, 0, &V17828_237_55786, &P17828_286};	/*Property = CssColumn*/
const Value V17828_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_287 = {S10, 0, &V17828_287_57173, &P17828_237};	/*Property = CssFlexbox*/
const Value V17828_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_288 = {S11, 0, &V17828_288_57176, &P17828_287};	/*Property = CssFont*/
const Value V17828_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_289 = {S12, 0, &V17828_289_86899, &P17828_288};	/*Property = CssImages*/
const Value V17828_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_291 = {S13, 0, &V17828_291_57182, &P17828_289};	/*Property = CssMediaQueries*/
const Value V17828_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_292 = {S14, 0, &V17828_292_57185, &P17828_291};	/*Property = CssMinMax*/
const Value V17828_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_293 = {S15, 0, &V17828_293_57188, &P17828_292};	/*Property = CssOverflow*/
const Value V17828_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_294 = {S16, 0, &V17828_294_57191, &P17828_293};	/*Property = CssPosition*/
const Value V17828_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_295 = {S17, 0, &V17828_295_57194, &P17828_294};	/*Property = CssText*/
const Value V17828_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_297 = {S18, 0, &V17828_297_57197, &P17828_295};	/*Property = CssTransforms*/
const Value V17828_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_241 = {S19, 0, &V17828_241_55801, &P17828_297};	/*Property = CssTransitions*/
const Value V17828_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_298 = {S20, 0, &V17828_298_57208, &P17828_241};	/*Property = CssUI*/
const Value V17828_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_300 = {S21, 0, &V17828_300_57217, &P17828_298};	/*Property = DataSet*/
const Value V17828_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_301 = {S22, 0, &V17828_301_57220, &P17828_300};	/*Property = DataUrl*/
const Value V17828_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_302 = {S23, 0, &V17828_302_57223, &P17828_301};	/*Property = DeviceOrientation*/
const Value V17828_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_304 = {S24, 0, &V17828_304_57229, &P17828_302};	/*Property = FileReader*/
const Value V17828_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_305 = {S25, 0, &V17828_305_57232, &P17828_304};	/*Property = FileSaver*/
const Value V17828_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_306 = {S26, 0, &V17828_306_57238, &P17828_305};	/*Property = FileWriter*/
const Value V17828_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_307 = {S27, 0, &V17828_307_57241, &P17828_306};	/*Property = FormData*/
const Value V17828_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_309 = {S28, 0, &V17828_309_57247, &P17828_307};	/*Property = Fullscreen*/
const Value V17828_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_310 = {S29, 0, &V17828_310_57250, &P17828_309};	/*Property = GeoLocation*/
const Value V17828_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_312 = {S30, 0, &V17828_312_57256, &P17828_310};	/*Property = History*/
const Value V17828_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_314 = {S31, 0, &V17828_314_57262, &P17828_312};	/*Property = Html5*/
const Value V17828_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_313 = {S32, 0, &V17828_313_57259, &P17828_314};	/*Property = Html-Media-Capture*/
const Value V17828_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_315 = {S33, 0, &V17828_315_57265, &P17828_313};	/*Property = Iframe*/
const Value V17828_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_316 = {S34, 0, &V17828_316_57268, &P17828_315};	/*Property = IndexedDB*/
const Value V17828_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_317 = {S35, 0, &V17828_317_57271, &P17828_316};	/*Property = Json*/
const Value V17828_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P17828_15 = {S37, 0, &V17828_15_54167, &P17828_317};	/*Property = LayoutEngine*/
const Value V17828_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_344 = {S38, 0, &V17828_344_57274, &P17828_15};	/*Property = Masking*/
const Value V17828_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_320 = {S39, 0, &V17828_320_57292, &P17828_344};	/*Property = PostMessage*/
const Value V17828_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_321 = {S40, 0, &V17828_321_57295, &P17828_320};	/*Property = Progress*/
const Value V17828_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_322 = {S41, 0, &V17828_322_57298, &P17828_321};	/*Property = Prompts*/
const Value V17828_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_323 = {S42, 0, &V17828_323_57301, &P17828_322};	/*Property = Selector*/
const Value V17828_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_324 = {S43, 0, &V17828_324_57307, &P17828_323};	/*Property = Svg*/
const Value V17828_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_327 = {S44, 0, &V17828_327_57316, &P17828_324};	/*Property = TouchEvents*/
const Value V17828_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_328 = {S45, 0, &V17828_328_57319, &P17828_327};	/*Property = Track*/
const Value V17828_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_329 = {S46, 0, &V17828_329_57325, &P17828_328};	/*Property = Video*/
const Value V17828_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_330 = {S47, 0, &V17828_330_57328, &P17828_329};	/*Property = Viewport*/
const Value V17828_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_332 = {S48, 0, &V17828_332_57340, &P17828_330};	/*Property = WebWorkers*/
const Value V17828_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17828_333 = {S49, 0, &V17828_333_57343, &P17828_332};	/*Property = Xhr2*/
const Profile P17828 = {"17828", &P17828_333};	/*Profile ID - 17828*/
const Value V17942_1_1 = {S2, NULL};	/*Value = True*/
const Property P17942_1 = {S50, 0, &V17942_1_1, NULL};	/*Property = IsMobile*/
const Value V17942_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17942_5 = {S52, 0, &V17942_5_1277, &P17942_1};	/*Property = ScreenPixelsHeight*/
const Value V17942_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17942_4 = {S54, 0, &V17942_4_1279, &P17942_5};	/*Property = ScreenPixelsWidth*/
const Profile P17942 = {"17942", &P17942_4};	/*Profile ID - 17942*/
const Profile P23112 = {"23112", NULL};	/*Profile ID - 23112*/
const Value V22939_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_279 = {S1, 0, &V22939_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22939_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_282 = {S3, 0, &V22939_282_57155, &P22939_279};	/*Property = BlobBuilder*/
const Value V22939_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_350 = {S4, 0, &V22939_350_57446, &P22939_282};	/*Property = Canvas*/
const Value V22939_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_283 = {S5, 0, &V22939_283_57161, &P22939_350};	/*Property = CssBackground*/
const Value V22939_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_284 = {S6, 0, &V22939_284_57164, &P22939_283};	/*Property = CssBorderImage*/
const Value V22939_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_285 = {S7, 0, &V22939_285_57167, &P22939_284};	/*Property = CssCanvas*/
const Value V22939_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_286 = {S8, 0, &V22939_286_57170, &P22939_285};	/*Property = CssColor*/
const Value V22939_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_237 = {S9, 0, &V22939_237_55786, &P22939_286};	/*Property = CssColumn*/
const Value V22939_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_287 = {S10, 0, &V22939_287_57173, &P22939_237};	/*Property = CssFlexbox*/
const Value V22939_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_288 = {S11, 0, &V22939_288_57176, &P22939_287};	/*Property = CssFont*/
const Value V22939_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_289 = {S12, 0, &V22939_289_86899, &P22939_288};	/*Property = CssImages*/
const Value V22939_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_291 = {S13, 0, &V22939_291_57182, &P22939_289};	/*Property = CssMediaQueries*/
const Value V22939_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_292 = {S14, 0, &V22939_292_57185, &P22939_291};	/*Property = CssMinMax*/
const Value V22939_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_293 = {S15, 0, &V22939_293_57188, &P22939_292};	/*Property = CssOverflow*/
const Value V22939_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_294 = {S16, 0, &V22939_294_57191, &P22939_293};	/*Property = CssPosition*/
const Value V22939_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_295 = {S17, 0, &V22939_295_57194, &P22939_294};	/*Property = CssText*/
const Value V22939_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_297 = {S18, 0, &V22939_297_57197, &P22939_295};	/*Property = CssTransforms*/
const Value V22939_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_241 = {S19, 0, &V22939_241_55801, &P22939_297};	/*Property = CssTransitions*/
const Value V22939_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_298 = {S20, 0, &V22939_298_57208, &P22939_241};	/*Property = CssUI*/
const Value V22939_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_300 = {S21, 0, &V22939_300_57217, &P22939_298};	/*Property = DataSet*/
const Value V22939_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_301 = {S22, 0, &V22939_301_57220, &P22939_300};	/*Property = DataUrl*/
const Value V22939_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_302 = {S23, 0, &V22939_302_57223, &P22939_301};	/*Property = DeviceOrientation*/
const Value V22939_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_304 = {S24, 0, &V22939_304_57229, &P22939_302};	/*Property = FileReader*/
const Value V22939_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_305 = {S25, 0, &V22939_305_57232, &P22939_304};	/*Property = FileSaver*/
const Value V22939_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_306 = {S26, 0, &V22939_306_57238, &P22939_305};	/*Property = FileWriter*/
const Value V22939_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_307 = {S27, 0, &V22939_307_57241, &P22939_306};	/*Property = FormData*/
const Value V22939_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_309 = {S28, 0, &V22939_309_57247, &P22939_307};	/*Property = Fullscreen*/
const Value V22939_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_310 = {S29, 0, &V22939_310_57250, &P22939_309};	/*Property = GeoLocation*/
const Value V22939_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_312 = {S30, 0, &V22939_312_57256, &P22939_310};	/*Property = History*/
const Value V22939_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_314 = {S31, 0, &V22939_314_57262, &P22939_312};	/*Property = Html5*/
const Value V22939_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_313 = {S32, 0, &V22939_313_57259, &P22939_314};	/*Property = Html-Media-Capture*/
const Value V22939_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_315 = {S33, 0, &V22939_315_57265, &P22939_313};	/*Property = Iframe*/
const Value V22939_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_316 = {S34, 0, &V22939_316_57268, &P22939_315};	/*Property = IndexedDB*/
const Value V22939_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_317 = {S35, 0, &V22939_317_57271, &P22939_316};	/*Property = Json*/
const Value V22939_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_15 = {S37, 0, &V22939_15_72, &P22939_317};	/*Property = LayoutEngine*/
const Value V22939_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_344 = {S38, 0, &V22939_344_57274, &P22939_15};	/*Property = Masking*/
const Value V22939_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_320 = {S39, 0, &V22939_320_57292, &P22939_344};	/*Property = PostMessage*/
const Value V22939_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_321 = {S40, 0, &V22939_321_57295, &P22939_320};	/*Property = Progress*/
const Value V22939_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_322 = {S41, 0, &V22939_322_57298, &P22939_321};	/*Property = Prompts*/
const Value V22939_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_323 = {S42, 0, &V22939_323_57301, &P22939_322};	/*Property = Selector*/
const Value V22939_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_324 = {S43, 0, &V22939_324_57307, &P22939_323};	/*Property = Svg*/
const Value V22939_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_327 = {S44, 0, &V22939_327_57316, &P22939_324};	/*Property = TouchEvents*/
const Value V22939_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_328 = {S45, 0, &V22939_328_57319, &P22939_327};	/*Property = Track*/
const Value V22939_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_329 = {S46, 0, &V22939_329_57325, &P22939_328};	/*Property = Video*/
const Value V22939_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_330 = {S47, 0, &V22939_330_57328, &P22939_329};	/*Property = Viewport*/
const Value V22939_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_332 = {S48, 0, &V22939_332_57340, &P22939_330};	/*Property = WebWorkers*/
const Value V22939_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22939_333 = {S49, 0, &V22939_333_57343, &P22939_332};	/*Property = Xhr2*/
const Profile P22939 = {"22939", &P22939_333};	/*Profile ID - 22939*/
const Value V17571_1_1 = {S2, NULL};	/*Value = True*/
const Property P17571_1 = {S50, 0, &V17571_1_1, NULL};	/*Property = IsMobile*/
const Value V17571_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17571_5 = {S52, 0, &V17571_5_1269, &P17571_1};	/*Property = ScreenPixelsHeight*/
const Value V17571_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17571_4 = {S54, 0, &V17571_4_1229, &P17571_5};	/*Property = ScreenPixelsWidth*/
const Profile P17571 = {"17571", &P17571_4};	/*Profile ID - 17571*/
const Value V16963_1_1 = {S2, NULL};	/*Value = True*/
const Property P16963_1 = {S50, 0, &V16963_1_1, NULL};	/*Property = IsMobile*/
const Value V16963_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16963_5 = {S52, 0, &V16963_5_1227, &P16963_1};	/*Property = ScreenPixelsHeight*/
const Value V16963_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16963_4 = {S54, 0, &V16963_4_1229, &P16963_5};	/*Property = ScreenPixelsWidth*/
const Profile P16963 = {"16963", &P16963_4};	/*Profile ID - 16963*/
const Value V22399_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22399_279 = {S1, 0, &V22399_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22399_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22399_282 = {S3, 0, &V22399_282_57154, &P22399_279};	/*Property = BlobBuilder*/
const Value V22399_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22399_350 = {S4, 0, &V22399_350_57444, &P22399_282};	/*Property = Canvas*/
const Value V22399_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22399_283 = {S5, 0, &V22399_283_57159, &P22399_350};	/*Property = CssBackground*/
const Value V22399_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22399_284 = {S6, 0, &V22399_284_57163, &P22399_283};	/*Property = CssBorderImage*/
const Value V22399_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22399_285 = {S7, 0, &V22399_285_57166, &P22399_284};	/*Property = CssCanvas*/
const Value V22399_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22399_286 = {S8, 0, &V22399_286_57168, &P22399_285};	/*Property = CssColor*/
const Value V22399_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22399_237 = {S9, 0, &V22399_237_55784, &P22399_286};	/*Property = CssColumn*/
const Value V22399_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22399_287 = {S10, 0, &V22399_287_57172, &P22399_237};	/*Property = CssFlexbox*/
const Value V22399_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22399_288 = {S11, 0, &V22399_288_57174, &P22399_287};	/*Property = CssFont*/
const Value V22399_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22399_289 = {S12, 0, &V22399_289_57759, &P22399_288};	/*Property = CssImages*/
const Value V22399_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22399_291 = {S13, 0, &V22399_291_57181, &P22399_289};	/*Property = CssMediaQueries*/
const Value V22399_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22399_292 = {S14, 0, &V22399_292_57183, &P22399_291};	/*Property = CssMinMax*/
const Value V22399_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22399_293 = {S15, 0, &V22399_293_57187, &P22399_292};	/*Property = CssOverflow*/
const Value V22399_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22399_294 = {S16, 0, &V22399_294_57189, &P22399_293};	/*Property = CssPosition*/
const Value V22399_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22399_295 = {S17, 0, &V22399_295_57192, &P22399_294};	/*Property = CssText*/
const Value V22399_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22399_297 = {S18, 0, &V22399_297_57195, &P22399_295};	/*Property = CssTransforms*/
const Value V22399_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22399_241 = {S19, 0, &V22399_241_55799, &P22399_297};	/*Property = CssTransitions*/
const Value V22399_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22399_298 = {S20, 0, &V22399_298_57206, &P22399_241};	/*Property = CssUI*/
const Value V22399_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22399_300 = {S21, 0, &V22399_300_57216, &P22399_298};	/*Property = DataSet*/
const Value V22399_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22399_301 = {S22, 0, &V22399_301_57218, &P22399_300};	/*Property = DataUrl*/
const Value V22399_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22399_302 = {S23, 0, &V22399_302_57222, &P22399_301};	/*Property = DeviceOrientation*/
const Value V22399_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22399_304 = {S24, 0, &V22399_304_57228, &P22399_302};	/*Property = FileReader*/
const Value V22399_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22399_305 = {S25, 0, &V22399_305_57231, &P22399_304};	/*Property = FileSaver*/
const Value V22399_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22399_306 = {S26, 0, &V22399_306_57237, &P22399_305};	/*Property = FileWriter*/
const Value V22399_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22399_307 = {S27, 0, &V22399_307_57240, &P22399_306};	/*Property = FormData*/
const Value V22399_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22399_309 = {S28, 0, &V22399_309_57246, &P22399_307};	/*Property = Fullscreen*/
const Value V22399_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22399_310 = {S29, 0, &V22399_310_57248, &P22399_309};	/*Property = GeoLocation*/
const Value V22399_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22399_312 = {S30, 0, &V22399_312_57255, &P22399_310};	/*Property = History*/
const Value V22399_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22399_314 = {S31, 0, &V22399_314_57261, &P22399_312};	/*Property = Html5*/
const Value V22399_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22399_313 = {S32, 0, &V22399_313_57258, &P22399_314};	/*Property = Html-Media-Capture*/
const Value V22399_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22399_315 = {S33, 0, &V22399_315_57264, &P22399_313};	/*Property = Iframe*/
const Value V22399_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22399_316 = {S34, 0, &V22399_316_57267, &P22399_315};	/*Property = IndexedDB*/
const Value V22399_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22399_317 = {S35, 0, &V22399_317_57269, &P22399_316};	/*Property = Json*/
const Value V22399_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22399_15 = {S37, 0, &V22399_15_28, &P22399_317};	/*Property = LayoutEngine*/
const Value V22399_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22399_344 = {S38, 0, &V22399_344_57272, &P22399_15};	/*Property = Masking*/
const Value V22399_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22399_320 = {S39, 0, &V22399_320_57290, &P22399_344};	/*Property = PostMessage*/
const Value V22399_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22399_321 = {S40, 0, &V22399_321_57293, &P22399_320};	/*Property = Progress*/
const Value V22399_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22399_322 = {S41, 0, &V22399_322_57296, &P22399_321};	/*Property = Prompts*/
const Value V22399_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22399_323 = {S42, 0, &V22399_323_57299, &P22399_322};	/*Property = Selector*/
const Value V22399_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22399_324 = {S43, 0, &V22399_324_57306, &P22399_323};	/*Property = Svg*/
const Value V22399_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22399_327 = {S44, 0, &V22399_327_57315, &P22399_324};	/*Property = TouchEvents*/
const Value V22399_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22399_328 = {S45, 0, &V22399_328_57318, &P22399_327};	/*Property = Track*/
const Value V22399_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22399_329 = {S46, 0, &V22399_329_57323, &P22399_328};	/*Property = Video*/
const Value V22399_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22399_330 = {S47, 0, &V22399_330_57326, &P22399_329};	/*Property = Viewport*/
const Value V22399_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22399_332 = {S48, 0, &V22399_332_57339, &P22399_330};	/*Property = WebWorkers*/
const Value V22399_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22399_333 = {S49, 0, &V22399_333_57342, &P22399_332};	/*Property = Xhr2*/
const Profile P22399 = {"22399", &P22399_333};	/*Profile ID - 22399*/
const Value V18015_1_1 = {S2, NULL};	/*Value = True*/
const Property P18015_1 = {S50, 0, &V18015_1_1, NULL};	/*Property = IsMobile*/
const Value V18015_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P18015_5 = {S52, 0, &V18015_5_1458, &P18015_1};	/*Property = ScreenPixelsHeight*/
const Value V18015_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P18015_4 = {S54, 0, &V18015_4_1460, &P18015_5};	/*Property = ScreenPixelsWidth*/
const Profile P18015 = {"18015", &P18015_4};	/*Profile ID - 18015*/
const Value V24643_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_279 = {S1, 0, &V24643_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24643_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_282 = {S3, 0, &V24643_282_57155, &P24643_279};	/*Property = BlobBuilder*/
const Value V24643_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_350 = {S4, 0, &V24643_350_57446, &P24643_282};	/*Property = Canvas*/
const Value V24643_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_283 = {S5, 0, &V24643_283_57161, &P24643_350};	/*Property = CssBackground*/
const Value V24643_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_284 = {S6, 0, &V24643_284_57164, &P24643_283};	/*Property = CssBorderImage*/
const Value V24643_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_285 = {S7, 0, &V24643_285_57167, &P24643_284};	/*Property = CssCanvas*/
const Value V24643_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_286 = {S8, 0, &V24643_286_57170, &P24643_285};	/*Property = CssColor*/
const Value V24643_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_237 = {S9, 0, &V24643_237_55786, &P24643_286};	/*Property = CssColumn*/
const Value V24643_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_287 = {S10, 0, &V24643_287_57173, &P24643_237};	/*Property = CssFlexbox*/
const Value V24643_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_288 = {S11, 0, &V24643_288_57176, &P24643_287};	/*Property = CssFont*/
const Value V24643_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_289 = {S12, 0, &V24643_289_86899, &P24643_288};	/*Property = CssImages*/
const Value V24643_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_291 = {S13, 0, &V24643_291_57182, &P24643_289};	/*Property = CssMediaQueries*/
const Value V24643_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_292 = {S14, 0, &V24643_292_57185, &P24643_291};	/*Property = CssMinMax*/
const Value V24643_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_293 = {S15, 0, &V24643_293_57188, &P24643_292};	/*Property = CssOverflow*/
const Value V24643_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_294 = {S16, 0, &V24643_294_57191, &P24643_293};	/*Property = CssPosition*/
const Value V24643_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_295 = {S17, 0, &V24643_295_57194, &P24643_294};	/*Property = CssText*/
const Value V24643_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_297 = {S18, 0, &V24643_297_57197, &P24643_295};	/*Property = CssTransforms*/
const Value V24643_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_241 = {S19, 0, &V24643_241_55801, &P24643_297};	/*Property = CssTransitions*/
const Value V24643_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_298 = {S20, 0, &V24643_298_57208, &P24643_241};	/*Property = CssUI*/
const Value V24643_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_300 = {S21, 0, &V24643_300_57217, &P24643_298};	/*Property = DataSet*/
const Value V24643_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_301 = {S22, 0, &V24643_301_57220, &P24643_300};	/*Property = DataUrl*/
const Value V24643_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_302 = {S23, 0, &V24643_302_57223, &P24643_301};	/*Property = DeviceOrientation*/
const Value V24643_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_304 = {S24, 0, &V24643_304_57229, &P24643_302};	/*Property = FileReader*/
const Value V24643_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_305 = {S25, 0, &V24643_305_57232, &P24643_304};	/*Property = FileSaver*/
const Value V24643_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_306 = {S26, 0, &V24643_306_57238, &P24643_305};	/*Property = FileWriter*/
const Value V24643_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_307 = {S27, 0, &V24643_307_57241, &P24643_306};	/*Property = FormData*/
const Value V24643_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_309 = {S28, 0, &V24643_309_57247, &P24643_307};	/*Property = Fullscreen*/
const Value V24643_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_310 = {S29, 0, &V24643_310_57250, &P24643_309};	/*Property = GeoLocation*/
const Value V24643_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_312 = {S30, 0, &V24643_312_57256, &P24643_310};	/*Property = History*/
const Value V24643_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_314 = {S31, 0, &V24643_314_57262, &P24643_312};	/*Property = Html5*/
const Value V24643_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_313 = {S32, 0, &V24643_313_57259, &P24643_314};	/*Property = Html-Media-Capture*/
const Value V24643_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_315 = {S33, 0, &V24643_315_57265, &P24643_313};	/*Property = Iframe*/
const Value V24643_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_316 = {S34, 0, &V24643_316_57268, &P24643_315};	/*Property = IndexedDB*/
const Value V24643_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_317 = {S35, 0, &V24643_317_57271, &P24643_316};	/*Property = Json*/
const Value V24643_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24643_15 = {S37, 0, &V24643_15_90837, &P24643_317};	/*Property = LayoutEngine*/
const Value V24643_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_344 = {S38, 0, &V24643_344_57274, &P24643_15};	/*Property = Masking*/
const Value V24643_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_320 = {S39, 0, &V24643_320_57292, &P24643_344};	/*Property = PostMessage*/
const Value V24643_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_321 = {S40, 0, &V24643_321_57295, &P24643_320};	/*Property = Progress*/
const Value V24643_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_322 = {S41, 0, &V24643_322_57298, &P24643_321};	/*Property = Prompts*/
const Value V24643_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_323 = {S42, 0, &V24643_323_57301, &P24643_322};	/*Property = Selector*/
const Value V24643_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_324 = {S43, 0, &V24643_324_57307, &P24643_323};	/*Property = Svg*/
const Value V24643_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_327 = {S44, 0, &V24643_327_57316, &P24643_324};	/*Property = TouchEvents*/
const Value V24643_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_328 = {S45, 0, &V24643_328_57319, &P24643_327};	/*Property = Track*/
const Value V24643_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_329 = {S46, 0, &V24643_329_57325, &P24643_328};	/*Property = Video*/
const Value V24643_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_330 = {S47, 0, &V24643_330_57328, &P24643_329};	/*Property = Viewport*/
const Value V24643_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_332 = {S48, 0, &V24643_332_57340, &P24643_330};	/*Property = WebWorkers*/
const Value V24643_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24643_333 = {S49, 0, &V24643_333_57343, &P24643_332};	/*Property = Xhr2*/
const Profile P24643 = {"24643", &P24643_333};	/*Profile ID - 24643*/
const Value V17949_1_1 = {S2, NULL};	/*Value = True*/
const Property P17949_1 = {S50, 0, &V17949_1_1, NULL};	/*Property = IsMobile*/
const Value V17949_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17949_5 = {S52, 0, &V17949_5_1227, &P17949_1};	/*Property = ScreenPixelsHeight*/
const Value V17949_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17949_4 = {S54, 0, &V17949_4_1229, &P17949_5};	/*Property = ScreenPixelsWidth*/
const Profile P17949 = {"17949", &P17949_4};	/*Profile ID - 17949*/
const Value V16954_1_1 = {S2, NULL};	/*Value = True*/
const Property P16954_1 = {S50, 0, &V16954_1_1, NULL};	/*Property = IsMobile*/
const Value V16954_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16954_5 = {S52, 0, &V16954_5_1227, &P16954_1};	/*Property = ScreenPixelsHeight*/
const Value V16954_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16954_4 = {S54, 0, &V16954_4_1229, &P16954_5};	/*Property = ScreenPixelsWidth*/
const Profile P16954 = {"16954", &P16954_4};	/*Profile ID - 16954*/
const Value V17392_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_279 = {S1, 0, &V17392_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17392_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_282 = {S3, 0, &V17392_282_57155, &P17392_279};	/*Property = BlobBuilder*/
const Value V17392_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_350 = {S4, 0, &V17392_350_57446, &P17392_282};	/*Property = Canvas*/
const Value V17392_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_283 = {S5, 0, &V17392_283_57161, &P17392_350};	/*Property = CssBackground*/
const Value V17392_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_284 = {S6, 0, &V17392_284_57164, &P17392_283};	/*Property = CssBorderImage*/
const Value V17392_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_285 = {S7, 0, &V17392_285_57167, &P17392_284};	/*Property = CssCanvas*/
const Value V17392_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_286 = {S8, 0, &V17392_286_57170, &P17392_285};	/*Property = CssColor*/
const Value V17392_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_237 = {S9, 0, &V17392_237_55786, &P17392_286};	/*Property = CssColumn*/
const Value V17392_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_287 = {S10, 0, &V17392_287_57173, &P17392_237};	/*Property = CssFlexbox*/
const Value V17392_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_288 = {S11, 0, &V17392_288_57176, &P17392_287};	/*Property = CssFont*/
const Value V17392_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_289 = {S12, 0, &V17392_289_86899, &P17392_288};	/*Property = CssImages*/
const Value V17392_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_291 = {S13, 0, &V17392_291_57182, &P17392_289};	/*Property = CssMediaQueries*/
const Value V17392_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_292 = {S14, 0, &V17392_292_57185, &P17392_291};	/*Property = CssMinMax*/
const Value V17392_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_293 = {S15, 0, &V17392_293_57188, &P17392_292};	/*Property = CssOverflow*/
const Value V17392_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_294 = {S16, 0, &V17392_294_57191, &P17392_293};	/*Property = CssPosition*/
const Value V17392_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_295 = {S17, 0, &V17392_295_57194, &P17392_294};	/*Property = CssText*/
const Value V17392_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_297 = {S18, 0, &V17392_297_57197, &P17392_295};	/*Property = CssTransforms*/
const Value V17392_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_241 = {S19, 0, &V17392_241_55801, &P17392_297};	/*Property = CssTransitions*/
const Value V17392_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_298 = {S20, 0, &V17392_298_57208, &P17392_241};	/*Property = CssUI*/
const Value V17392_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_300 = {S21, 0, &V17392_300_57217, &P17392_298};	/*Property = DataSet*/
const Value V17392_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_301 = {S22, 0, &V17392_301_57220, &P17392_300};	/*Property = DataUrl*/
const Value V17392_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_302 = {S23, 0, &V17392_302_57223, &P17392_301};	/*Property = DeviceOrientation*/
const Value V17392_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_304 = {S24, 0, &V17392_304_57229, &P17392_302};	/*Property = FileReader*/
const Value V17392_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_305 = {S25, 0, &V17392_305_57232, &P17392_304};	/*Property = FileSaver*/
const Value V17392_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_306 = {S26, 0, &V17392_306_57238, &P17392_305};	/*Property = FileWriter*/
const Value V17392_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_307 = {S27, 0, &V17392_307_57241, &P17392_306};	/*Property = FormData*/
const Value V17392_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_309 = {S28, 0, &V17392_309_57247, &P17392_307};	/*Property = Fullscreen*/
const Value V17392_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_310 = {S29, 0, &V17392_310_57250, &P17392_309};	/*Property = GeoLocation*/
const Value V17392_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_312 = {S30, 0, &V17392_312_57256, &P17392_310};	/*Property = History*/
const Value V17392_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_314 = {S31, 0, &V17392_314_57262, &P17392_312};	/*Property = Html5*/
const Value V17392_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_313 = {S32, 0, &V17392_313_57259, &P17392_314};	/*Property = Html-Media-Capture*/
const Value V17392_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_315 = {S33, 0, &V17392_315_57265, &P17392_313};	/*Property = Iframe*/
const Value V17392_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_316 = {S34, 0, &V17392_316_57268, &P17392_315};	/*Property = IndexedDB*/
const Value V17392_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_317 = {S35, 0, &V17392_317_57271, &P17392_316};	/*Property = Json*/
const Value V17392_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17392_15 = {S37, 0, &V17392_15_28, &P17392_317};	/*Property = LayoutEngine*/
const Value V17392_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_344 = {S38, 0, &V17392_344_57274, &P17392_15};	/*Property = Masking*/
const Value V17392_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_320 = {S39, 0, &V17392_320_57292, &P17392_344};	/*Property = PostMessage*/
const Value V17392_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_321 = {S40, 0, &V17392_321_57295, &P17392_320};	/*Property = Progress*/
const Value V17392_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_322 = {S41, 0, &V17392_322_57298, &P17392_321};	/*Property = Prompts*/
const Value V17392_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_323 = {S42, 0, &V17392_323_57301, &P17392_322};	/*Property = Selector*/
const Value V17392_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_324 = {S43, 0, &V17392_324_57307, &P17392_323};	/*Property = Svg*/
const Value V17392_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_327 = {S44, 0, &V17392_327_57316, &P17392_324};	/*Property = TouchEvents*/
const Value V17392_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_328 = {S45, 0, &V17392_328_57319, &P17392_327};	/*Property = Track*/
const Value V17392_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_329 = {S46, 0, &V17392_329_57325, &P17392_328};	/*Property = Video*/
const Value V17392_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_330 = {S47, 0, &V17392_330_57328, &P17392_329};	/*Property = Viewport*/
const Value V17392_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_332 = {S48, 0, &V17392_332_57340, &P17392_330};	/*Property = WebWorkers*/
const Value V17392_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17392_333 = {S49, 0, &V17392_333_57343, &P17392_332};	/*Property = Xhr2*/
const Profile P17392 = {"17392", &P17392_333};	/*Profile ID - 17392*/
const Value V16703_1_1 = {S2, NULL};	/*Value = True*/
const Property P16703_1 = {S50, 0, &V16703_1_1, NULL};	/*Property = IsMobile*/
const Value V16703_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16703_5 = {S52, 0, &V16703_5_1233, &P16703_1};	/*Property = ScreenPixelsHeight*/
const Value V16703_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16703_4 = {S54, 0, &V16703_4_1235, &P16703_5};	/*Property = ScreenPixelsWidth*/
const Profile P16703 = {"16703", &P16703_4};	/*Profile ID - 16703*/
const Value V21560_1_1 = {S2, NULL};	/*Value = True*/
const Property P21560_1 = {S50, 0, &V21560_1_1, NULL};	/*Property = IsMobile*/
const Value V21560_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21560_5 = {S52, 0, &V21560_5_1227, &P21560_1};	/*Property = ScreenPixelsHeight*/
const Value V21560_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21560_4 = {S54, 0, &V21560_4_1229, &P21560_5};	/*Property = ScreenPixelsWidth*/
const Profile P21560 = {"21560", &P21560_4};	/*Profile ID - 21560*/
const Value V16168_1_1 = {S2, NULL};	/*Value = True*/
const Property P16168_1 = {S50, 0, &V16168_1_1, NULL};	/*Property = IsMobile*/
const Value V16168_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P16168_5 = {S52, 0, &V16168_5_1458, &P16168_1};	/*Property = ScreenPixelsHeight*/
const Value V16168_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P16168_4 = {S54, 0, &V16168_4_1460, &P16168_5};	/*Property = ScreenPixelsWidth*/
const Profile P16168 = {"16168", &P16168_4};	/*Profile ID - 16168*/
const Value V17719_1_1 = {S2, NULL};	/*Value = True*/
const Property P17719_1 = {S50, 0, &V17719_1_1, NULL};	/*Property = IsMobile*/
const Value V17719_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17719_5 = {S52, 0, &V17719_5_1269, &P17719_1};	/*Property = ScreenPixelsHeight*/
const Value V17719_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17719_4 = {S54, 0, &V17719_4_1229, &P17719_5};	/*Property = ScreenPixelsWidth*/
const Profile P17719 = {"17719", &P17719_4};	/*Profile ID - 17719*/
const Value V17973_1_1 = {S2, NULL};	/*Value = True*/
const Property P17973_1 = {S50, 0, &V17973_1_1, NULL};	/*Property = IsMobile*/
const Value V17973_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17973_5 = {S52, 0, &V17973_5_1227, &P17973_1};	/*Property = ScreenPixelsHeight*/
const Value V17973_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17973_4 = {S54, 0, &V17973_4_1229, &P17973_5};	/*Property = ScreenPixelsWidth*/
const Profile P17973 = {"17973", &P17973_4};	/*Profile ID - 17973*/
const Value V17950_1_1 = {S2, NULL};	/*Value = True*/
const Property P17950_1 = {S50, 0, &V17950_1_1, NULL};	/*Property = IsMobile*/
const Value V17950_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17950_5 = {S52, 0, &V17950_5_1227, &P17950_1};	/*Property = ScreenPixelsHeight*/
const Value V17950_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17950_4 = {S54, 0, &V17950_4_1229, &P17950_5};	/*Property = ScreenPixelsWidth*/
const Profile P17950 = {"17950", &P17950_4};	/*Profile ID - 17950*/
const Value V18081_1_1 = {S2, NULL};	/*Value = True*/
const Property P18081_1 = {S50, 0, &V18081_1_1, NULL};	/*Property = IsMobile*/
const Value V18081_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18081_5 = {S52, 0, &V18081_5_1277, &P18081_1};	/*Property = ScreenPixelsHeight*/
const Value V18081_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18081_4 = {S54, 0, &V18081_4_1279, &P18081_5};	/*Property = ScreenPixelsWidth*/
const Profile P18081 = {"18081", &P18081_4};	/*Profile ID - 18081*/
const Value V18106_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18106_279 = {S1, 0, &V18106_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18106_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18106_282 = {S3, 0, &V18106_282_57154, &P18106_279};	/*Property = BlobBuilder*/
const Value V18106_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18106_350 = {S4, 0, &V18106_350_57444, &P18106_282};	/*Property = Canvas*/
const Value V18106_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18106_283 = {S5, 0, &V18106_283_57159, &P18106_350};	/*Property = CssBackground*/
const Value V18106_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18106_284 = {S6, 0, &V18106_284_57163, &P18106_283};	/*Property = CssBorderImage*/
const Value V18106_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18106_285 = {S7, 0, &V18106_285_57166, &P18106_284};	/*Property = CssCanvas*/
const Value V18106_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18106_286 = {S8, 0, &V18106_286_57168, &P18106_285};	/*Property = CssColor*/
const Value V18106_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18106_237 = {S9, 0, &V18106_237_55786, &P18106_286};	/*Property = CssColumn*/
const Value V18106_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18106_287 = {S10, 0, &V18106_287_57172, &P18106_237};	/*Property = CssFlexbox*/
const Value V18106_288_57174 = {S2, NULL};	/*Value = True*/
const Property P18106_288 = {S11, 0, &V18106_288_57174, &P18106_287};	/*Property = CssFont*/
const Value V18106_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18106_289 = {S12, 0, &V18106_289_57759, &P18106_288};	/*Property = CssImages*/
const Value V18106_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18106_291 = {S13, 0, &V18106_291_57180, &P18106_289};	/*Property = CssMediaQueries*/
const Value V18106_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18106_292 = {S14, 0, &V18106_292_57183, &P18106_291};	/*Property = CssMinMax*/
const Value V18106_293_57186 = {S2, NULL};	/*Value = True*/
const Property P18106_293 = {S15, 0, &V18106_293_57186, &P18106_292};	/*Property = CssOverflow*/
const Value V18106_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18106_294 = {S16, 0, &V18106_294_57189, &P18106_293};	/*Property = CssPosition*/
const Value V18106_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18106_295 = {S17, 0, &V18106_295_57192, &P18106_294};	/*Property = CssText*/
const Value V18106_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18106_297 = {S18, 0, &V18106_297_57195, &P18106_295};	/*Property = CssTransforms*/
const Value V18106_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18106_241 = {S19, 0, &V18106_241_55799, &P18106_297};	/*Property = CssTransitions*/
const Value V18106_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18106_298 = {S20, 0, &V18106_298_57206, &P18106_241};	/*Property = CssUI*/
const Value V18106_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18106_300 = {S21, 0, &V18106_300_57216, &P18106_298};	/*Property = DataSet*/
const Value V18106_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18106_301 = {S22, 0, &V18106_301_57218, &P18106_300};	/*Property = DataUrl*/
const Value V18106_302_57221 = {S2, NULL};	/*Value = True*/
const Property P18106_302 = {S23, 0, &V18106_302_57221, &P18106_301};	/*Property = DeviceOrientation*/
const Value V18106_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18106_304 = {S24, 0, &V18106_304_57228, &P18106_302};	/*Property = FileReader*/
const Value V18106_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18106_305 = {S25, 0, &V18106_305_57231, &P18106_304};	/*Property = FileSaver*/
const Value V18106_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18106_306 = {S26, 0, &V18106_306_57237, &P18106_305};	/*Property = FileWriter*/
const Value V18106_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18106_307 = {S27, 0, &V18106_307_57239, &P18106_306};	/*Property = FormData*/
const Value V18106_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18106_309 = {S28, 0, &V18106_309_57246, &P18106_307};	/*Property = Fullscreen*/
const Value V18106_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18106_310 = {S29, 0, &V18106_310_57248, &P18106_309};	/*Property = GeoLocation*/
const Value V18106_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18106_312 = {S30, 0, &V18106_312_57255, &P18106_310};	/*Property = History*/
const Value V18106_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18106_314 = {S31, 0, &V18106_314_57261, &P18106_312};	/*Property = Html5*/
const Value V18106_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18106_313 = {S32, 0, &V18106_313_57258, &P18106_314};	/*Property = Html-Media-Capture*/
const Value V18106_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18106_315 = {S33, 0, &V18106_315_57264, &P18106_313};	/*Property = Iframe*/
const Value V18106_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18106_316 = {S34, 0, &V18106_316_57267, &P18106_315};	/*Property = IndexedDB*/
const Value V18106_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18106_317 = {S35, 0, &V18106_317_57269, &P18106_316};	/*Property = Json*/
const Value V18106_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18106_15 = {S37, 0, &V18106_15_29, &P18106_317};	/*Property = LayoutEngine*/
const Value V18106_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18106_344 = {S38, 0, &V18106_344_57272, &P18106_15};	/*Property = Masking*/
const Value V18106_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18106_320 = {S39, 0, &V18106_320_57290, &P18106_344};	/*Property = PostMessage*/
const Value V18106_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18106_321 = {S40, 0, &V18106_321_57293, &P18106_320};	/*Property = Progress*/
const Value V18106_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18106_322 = {S41, 0, &V18106_322_57296, &P18106_321};	/*Property = Prompts*/
const Value V18106_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18106_323 = {S42, 0, &V18106_323_57299, &P18106_322};	/*Property = Selector*/
const Value V18106_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18106_324 = {S43, 0, &V18106_324_57306, &P18106_323};	/*Property = Svg*/
const Value V18106_327_57314 = {S2, NULL};	/*Value = True*/
const Property P18106_327 = {S44, 0, &V18106_327_57314, &P18106_324};	/*Property = TouchEvents*/
const Value V18106_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18106_328 = {S45, 0, &V18106_328_57318, &P18106_327};	/*Property = Track*/
const Value V18106_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18106_329 = {S46, 0, &V18106_329_57323, &P18106_328};	/*Property = Video*/
const Value V18106_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18106_330 = {S47, 0, &V18106_330_57326, &P18106_329};	/*Property = Viewport*/
const Value V18106_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18106_332 = {S48, 0, &V18106_332_57339, &P18106_330};	/*Property = WebWorkers*/
const Value V18106_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18106_333 = {S49, 0, &V18106_333_57342, &P18106_332};	/*Property = Xhr2*/
const Profile P18106 = {"18106", &P18106_333};	/*Profile ID - 18106*/
const Profile P22843 = {"22843", NULL};	/*Profile ID - 22843*/
const Value V13036_1_1 = {S2, NULL};	/*Value = True*/
const Property P13036_1 = {S50, 0, &V13036_1_1, NULL};	/*Property = IsMobile*/
const Value V13036_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13036_5 = {S52, 0, &V13036_5_1227, &P13036_1};	/*Property = ScreenPixelsHeight*/
const Value V13036_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13036_4 = {S54, 0, &V13036_4_1229, &P13036_5};	/*Property = ScreenPixelsWidth*/
const Profile P13036 = {"13036", &P13036_4};	/*Profile ID - 13036*/
const Value V21928_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21928_279 = {S1, 0, &V21928_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21928_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21928_282 = {S3, 0, &V21928_282_57154, &P21928_279};	/*Property = BlobBuilder*/
const Value V21928_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21928_350 = {S4, 0, &V21928_350_57444, &P21928_282};	/*Property = Canvas*/
const Value V21928_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21928_283 = {S5, 0, &V21928_283_57159, &P21928_350};	/*Property = CssBackground*/
const Value V21928_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21928_284 = {S6, 0, &V21928_284_57163, &P21928_283};	/*Property = CssBorderImage*/
const Value V21928_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21928_285 = {S7, 0, &V21928_285_57166, &P21928_284};	/*Property = CssCanvas*/
const Value V21928_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21928_286 = {S8, 0, &V21928_286_57168, &P21928_285};	/*Property = CssColor*/
const Value V21928_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21928_237 = {S9, 0, &V21928_237_55784, &P21928_286};	/*Property = CssColumn*/
const Value V21928_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21928_287 = {S10, 0, &V21928_287_57172, &P21928_237};	/*Property = CssFlexbox*/
const Value V21928_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21928_288 = {S11, 0, &V21928_288_57175, &P21928_287};	/*Property = CssFont*/
const Value V21928_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21928_289 = {S12, 0, &V21928_289_57759, &P21928_288};	/*Property = CssImages*/
const Value V21928_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21928_291 = {S13, 0, &V21928_291_57181, &P21928_289};	/*Property = CssMediaQueries*/
const Value V21928_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21928_292 = {S14, 0, &V21928_292_57183, &P21928_291};	/*Property = CssMinMax*/
const Value V21928_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21928_293 = {S15, 0, &V21928_293_57187, &P21928_292};	/*Property = CssOverflow*/
const Value V21928_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21928_294 = {S16, 0, &V21928_294_57190, &P21928_293};	/*Property = CssPosition*/
const Value V21928_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21928_295 = {S17, 0, &V21928_295_57192, &P21928_294};	/*Property = CssText*/
const Value V21928_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21928_297 = {S18, 0, &V21928_297_57195, &P21928_295};	/*Property = CssTransforms*/
const Value V21928_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21928_241 = {S19, 0, &V21928_241_55799, &P21928_297};	/*Property = CssTransitions*/
const Value V21928_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21928_298 = {S20, 0, &V21928_298_57206, &P21928_241};	/*Property = CssUI*/
const Value V21928_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21928_300 = {S21, 0, &V21928_300_57216, &P21928_298};	/*Property = DataSet*/
const Value V21928_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21928_301 = {S22, 0, &V21928_301_57218, &P21928_300};	/*Property = DataUrl*/
const Value V21928_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21928_302 = {S23, 0, &V21928_302_57222, &P21928_301};	/*Property = DeviceOrientation*/
const Value V21928_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21928_304 = {S24, 0, &V21928_304_57228, &P21928_302};	/*Property = FileReader*/
const Value V21928_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21928_305 = {S25, 0, &V21928_305_57231, &P21928_304};	/*Property = FileSaver*/
const Value V21928_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21928_306 = {S26, 0, &V21928_306_57237, &P21928_305};	/*Property = FileWriter*/
const Value V21928_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21928_307 = {S27, 0, &V21928_307_57240, &P21928_306};	/*Property = FormData*/
const Value V21928_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21928_309 = {S28, 0, &V21928_309_57246, &P21928_307};	/*Property = Fullscreen*/
const Value V21928_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21928_310 = {S29, 0, &V21928_310_57248, &P21928_309};	/*Property = GeoLocation*/
const Value V21928_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21928_312 = {S30, 0, &V21928_312_57255, &P21928_310};	/*Property = History*/
const Value V21928_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21928_314 = {S31, 0, &V21928_314_57261, &P21928_312};	/*Property = Html5*/
const Value V21928_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21928_313 = {S32, 0, &V21928_313_57258, &P21928_314};	/*Property = Html-Media-Capture*/
const Value V21928_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21928_315 = {S33, 0, &V21928_315_57264, &P21928_313};	/*Property = Iframe*/
const Value V21928_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21928_316 = {S34, 0, &V21928_316_57267, &P21928_315};	/*Property = IndexedDB*/
const Value V21928_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21928_317 = {S35, 0, &V21928_317_57269, &P21928_316};	/*Property = Json*/
const Value V21928_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21928_15 = {S37, 0, &V21928_15_28, &P21928_317};	/*Property = LayoutEngine*/
const Value V21928_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21928_344 = {S38, 0, &V21928_344_57272, &P21928_15};	/*Property = Masking*/
const Value V21928_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21928_320 = {S39, 0, &V21928_320_57290, &P21928_344};	/*Property = PostMessage*/
const Value V21928_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21928_321 = {S40, 0, &V21928_321_57293, &P21928_320};	/*Property = Progress*/
const Value V21928_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21928_322 = {S41, 0, &V21928_322_57296, &P21928_321};	/*Property = Prompts*/
const Value V21928_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21928_323 = {S42, 0, &V21928_323_57299, &P21928_322};	/*Property = Selector*/
const Value V21928_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21928_324 = {S43, 0, &V21928_324_57306, &P21928_323};	/*Property = Svg*/
const Value V21928_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21928_327 = {S44, 0, &V21928_327_57315, &P21928_324};	/*Property = TouchEvents*/
const Value V21928_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21928_328 = {S45, 0, &V21928_328_57318, &P21928_327};	/*Property = Track*/
const Value V21928_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21928_329 = {S46, 0, &V21928_329_57323, &P21928_328};	/*Property = Video*/
const Value V21928_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21928_330 = {S47, 0, &V21928_330_57326, &P21928_329};	/*Property = Viewport*/
const Value V21928_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21928_332 = {S48, 0, &V21928_332_57339, &P21928_330};	/*Property = WebWorkers*/
const Value V21928_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21928_333 = {S49, 0, &V21928_333_57342, &P21928_332};	/*Property = Xhr2*/
const Profile P21928 = {"21928", &P21928_333};	/*Profile ID - 21928*/
const Value V22492_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22492_279 = {S1, 0, &V22492_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22492_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22492_282 = {S3, 0, &V22492_282_57154, &P22492_279};	/*Property = BlobBuilder*/
const Value V22492_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22492_350 = {S4, 0, &V22492_350_57444, &P22492_282};	/*Property = Canvas*/
const Value V22492_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22492_283 = {S5, 0, &V22492_283_57159, &P22492_350};	/*Property = CssBackground*/
const Value V22492_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22492_284 = {S6, 0, &V22492_284_57163, &P22492_283};	/*Property = CssBorderImage*/
const Value V22492_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22492_285 = {S7, 0, &V22492_285_57166, &P22492_284};	/*Property = CssCanvas*/
const Value V22492_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22492_286 = {S8, 0, &V22492_286_57168, &P22492_285};	/*Property = CssColor*/
const Value V22492_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22492_237 = {S9, 0, &V22492_237_55784, &P22492_286};	/*Property = CssColumn*/
const Value V22492_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22492_287 = {S10, 0, &V22492_287_57172, &P22492_237};	/*Property = CssFlexbox*/
const Value V22492_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22492_288 = {S11, 0, &V22492_288_57175, &P22492_287};	/*Property = CssFont*/
const Value V22492_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22492_289 = {S12, 0, &V22492_289_57759, &P22492_288};	/*Property = CssImages*/
const Value V22492_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22492_291 = {S13, 0, &V22492_291_57181, &P22492_289};	/*Property = CssMediaQueries*/
const Value V22492_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22492_292 = {S14, 0, &V22492_292_57183, &P22492_291};	/*Property = CssMinMax*/
const Value V22492_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22492_293 = {S15, 0, &V22492_293_57187, &P22492_292};	/*Property = CssOverflow*/
const Value V22492_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22492_294 = {S16, 0, &V22492_294_57189, &P22492_293};	/*Property = CssPosition*/
const Value V22492_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22492_295 = {S17, 0, &V22492_295_57192, &P22492_294};	/*Property = CssText*/
const Value V22492_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22492_297 = {S18, 0, &V22492_297_57195, &P22492_295};	/*Property = CssTransforms*/
const Value V22492_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22492_241 = {S19, 0, &V22492_241_55799, &P22492_297};	/*Property = CssTransitions*/
const Value V22492_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22492_298 = {S20, 0, &V22492_298_57206, &P22492_241};	/*Property = CssUI*/
const Value V22492_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22492_300 = {S21, 0, &V22492_300_57216, &P22492_298};	/*Property = DataSet*/
const Value V22492_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22492_301 = {S22, 0, &V22492_301_57218, &P22492_300};	/*Property = DataUrl*/
const Value V22492_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22492_302 = {S23, 0, &V22492_302_57222, &P22492_301};	/*Property = DeviceOrientation*/
const Value V22492_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22492_304 = {S24, 0, &V22492_304_57228, &P22492_302};	/*Property = FileReader*/
const Value V22492_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22492_305 = {S25, 0, &V22492_305_57231, &P22492_304};	/*Property = FileSaver*/
const Value V22492_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22492_306 = {S26, 0, &V22492_306_57237, &P22492_305};	/*Property = FileWriter*/
const Value V22492_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22492_307 = {S27, 0, &V22492_307_57240, &P22492_306};	/*Property = FormData*/
const Value V22492_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22492_309 = {S28, 0, &V22492_309_57246, &P22492_307};	/*Property = Fullscreen*/
const Value V22492_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22492_310 = {S29, 0, &V22492_310_57248, &P22492_309};	/*Property = GeoLocation*/
const Value V22492_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22492_312 = {S30, 0, &V22492_312_57255, &P22492_310};	/*Property = History*/
const Value V22492_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22492_314 = {S31, 0, &V22492_314_57261, &P22492_312};	/*Property = Html5*/
const Value V22492_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22492_313 = {S32, 0, &V22492_313_57258, &P22492_314};	/*Property = Html-Media-Capture*/
const Value V22492_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22492_315 = {S33, 0, &V22492_315_57264, &P22492_313};	/*Property = Iframe*/
const Value V22492_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22492_316 = {S34, 0, &V22492_316_57267, &P22492_315};	/*Property = IndexedDB*/
const Value V22492_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22492_317 = {S35, 0, &V22492_317_57269, &P22492_316};	/*Property = Json*/
const Value V22492_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22492_15 = {S37, 0, &V22492_15_28, &P22492_317};	/*Property = LayoutEngine*/
const Value V22492_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22492_344 = {S38, 0, &V22492_344_57272, &P22492_15};	/*Property = Masking*/
const Value V22492_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22492_320 = {S39, 0, &V22492_320_57290, &P22492_344};	/*Property = PostMessage*/
const Value V22492_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22492_321 = {S40, 0, &V22492_321_57293, &P22492_320};	/*Property = Progress*/
const Value V22492_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22492_322 = {S41, 0, &V22492_322_57296, &P22492_321};	/*Property = Prompts*/
const Value V22492_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22492_323 = {S42, 0, &V22492_323_57299, &P22492_322};	/*Property = Selector*/
const Value V22492_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22492_324 = {S43, 0, &V22492_324_57306, &P22492_323};	/*Property = Svg*/
const Value V22492_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22492_327 = {S44, 0, &V22492_327_57315, &P22492_324};	/*Property = TouchEvents*/
const Value V22492_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22492_328 = {S45, 0, &V22492_328_57318, &P22492_327};	/*Property = Track*/
const Value V22492_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22492_329 = {S46, 0, &V22492_329_57323, &P22492_328};	/*Property = Video*/
const Value V22492_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22492_330 = {S47, 0, &V22492_330_57326, &P22492_329};	/*Property = Viewport*/
const Value V22492_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22492_332 = {S48, 0, &V22492_332_57339, &P22492_330};	/*Property = WebWorkers*/
const Value V22492_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22492_333 = {S49, 0, &V22492_333_57342, &P22492_332};	/*Property = Xhr2*/
const Profile P22492 = {"22492", &P22492_333};	/*Profile ID - 22492*/
const Value V22446_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22446_279 = {S1, 0, &V22446_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22446_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22446_282 = {S3, 0, &V22446_282_57154, &P22446_279};	/*Property = BlobBuilder*/
const Value V22446_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22446_350 = {S4, 0, &V22446_350_57444, &P22446_282};	/*Property = Canvas*/
const Value V22446_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22446_283 = {S5, 0, &V22446_283_57159, &P22446_350};	/*Property = CssBackground*/
const Value V22446_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22446_284 = {S6, 0, &V22446_284_57163, &P22446_283};	/*Property = CssBorderImage*/
const Value V22446_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22446_285 = {S7, 0, &V22446_285_57166, &P22446_284};	/*Property = CssCanvas*/
const Value V22446_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22446_286 = {S8, 0, &V22446_286_57168, &P22446_285};	/*Property = CssColor*/
const Value V22446_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22446_237 = {S9, 0, &V22446_237_55784, &P22446_286};	/*Property = CssColumn*/
const Value V22446_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22446_287 = {S10, 0, &V22446_287_57172, &P22446_237};	/*Property = CssFlexbox*/
const Value V22446_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22446_288 = {S11, 0, &V22446_288_57175, &P22446_287};	/*Property = CssFont*/
const Value V22446_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22446_289 = {S12, 0, &V22446_289_57759, &P22446_288};	/*Property = CssImages*/
const Value V22446_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22446_291 = {S13, 0, &V22446_291_57181, &P22446_289};	/*Property = CssMediaQueries*/
const Value V22446_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22446_292 = {S14, 0, &V22446_292_57183, &P22446_291};	/*Property = CssMinMax*/
const Value V22446_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22446_293 = {S15, 0, &V22446_293_57187, &P22446_292};	/*Property = CssOverflow*/
const Value V22446_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22446_294 = {S16, 0, &V22446_294_57190, &P22446_293};	/*Property = CssPosition*/
const Value V22446_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22446_295 = {S17, 0, &V22446_295_57192, &P22446_294};	/*Property = CssText*/
const Value V22446_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22446_297 = {S18, 0, &V22446_297_57195, &P22446_295};	/*Property = CssTransforms*/
const Value V22446_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22446_241 = {S19, 0, &V22446_241_55799, &P22446_297};	/*Property = CssTransitions*/
const Value V22446_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22446_298 = {S20, 0, &V22446_298_57206, &P22446_241};	/*Property = CssUI*/
const Value V22446_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22446_300 = {S21, 0, &V22446_300_57216, &P22446_298};	/*Property = DataSet*/
const Value V22446_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22446_301 = {S22, 0, &V22446_301_57218, &P22446_300};	/*Property = DataUrl*/
const Value V22446_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22446_302 = {S23, 0, &V22446_302_57222, &P22446_301};	/*Property = DeviceOrientation*/
const Value V22446_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22446_304 = {S24, 0, &V22446_304_57228, &P22446_302};	/*Property = FileReader*/
const Value V22446_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22446_305 = {S25, 0, &V22446_305_57231, &P22446_304};	/*Property = FileSaver*/
const Value V22446_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22446_306 = {S26, 0, &V22446_306_57237, &P22446_305};	/*Property = FileWriter*/
const Value V22446_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22446_307 = {S27, 0, &V22446_307_57240, &P22446_306};	/*Property = FormData*/
const Value V22446_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22446_309 = {S28, 0, &V22446_309_57246, &P22446_307};	/*Property = Fullscreen*/
const Value V22446_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22446_310 = {S29, 0, &V22446_310_57248, &P22446_309};	/*Property = GeoLocation*/
const Value V22446_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22446_312 = {S30, 0, &V22446_312_57255, &P22446_310};	/*Property = History*/
const Value V22446_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22446_314 = {S31, 0, &V22446_314_57261, &P22446_312};	/*Property = Html5*/
const Value V22446_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22446_313 = {S32, 0, &V22446_313_57258, &P22446_314};	/*Property = Html-Media-Capture*/
const Value V22446_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22446_315 = {S33, 0, &V22446_315_57264, &P22446_313};	/*Property = Iframe*/
const Value V22446_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22446_316 = {S34, 0, &V22446_316_57267, &P22446_315};	/*Property = IndexedDB*/
const Value V22446_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22446_317 = {S35, 0, &V22446_317_57269, &P22446_316};	/*Property = Json*/
const Value V22446_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22446_15 = {S37, 0, &V22446_15_28, &P22446_317};	/*Property = LayoutEngine*/
const Value V22446_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22446_344 = {S38, 0, &V22446_344_57272, &P22446_15};	/*Property = Masking*/
const Value V22446_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22446_320 = {S39, 0, &V22446_320_57290, &P22446_344};	/*Property = PostMessage*/
const Value V22446_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22446_321 = {S40, 0, &V22446_321_57293, &P22446_320};	/*Property = Progress*/
const Value V22446_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22446_322 = {S41, 0, &V22446_322_57296, &P22446_321};	/*Property = Prompts*/
const Value V22446_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22446_323 = {S42, 0, &V22446_323_57299, &P22446_322};	/*Property = Selector*/
const Value V22446_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22446_324 = {S43, 0, &V22446_324_57306, &P22446_323};	/*Property = Svg*/
const Value V22446_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22446_327 = {S44, 0, &V22446_327_57315, &P22446_324};	/*Property = TouchEvents*/
const Value V22446_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22446_328 = {S45, 0, &V22446_328_57318, &P22446_327};	/*Property = Track*/
const Value V22446_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22446_329 = {S46, 0, &V22446_329_57323, &P22446_328};	/*Property = Video*/
const Value V22446_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22446_330 = {S47, 0, &V22446_330_57326, &P22446_329};	/*Property = Viewport*/
const Value V22446_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22446_332 = {S48, 0, &V22446_332_57339, &P22446_330};	/*Property = WebWorkers*/
const Value V22446_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22446_333 = {S49, 0, &V22446_333_57342, &P22446_332};	/*Property = Xhr2*/
const Profile P22446 = {"22446", &P22446_333};	/*Profile ID - 22446*/
const Value V21926_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21926_279 = {S1, 0, &V21926_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21926_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21926_282 = {S3, 0, &V21926_282_57154, &P21926_279};	/*Property = BlobBuilder*/
const Value V21926_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21926_350 = {S4, 0, &V21926_350_57444, &P21926_282};	/*Property = Canvas*/
const Value V21926_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21926_283 = {S5, 0, &V21926_283_57159, &P21926_350};	/*Property = CssBackground*/
const Value V21926_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21926_284 = {S6, 0, &V21926_284_57163, &P21926_283};	/*Property = CssBorderImage*/
const Value V21926_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21926_285 = {S7, 0, &V21926_285_57166, &P21926_284};	/*Property = CssCanvas*/
const Value V21926_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21926_286 = {S8, 0, &V21926_286_57168, &P21926_285};	/*Property = CssColor*/
const Value V21926_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21926_237 = {S9, 0, &V21926_237_55784, &P21926_286};	/*Property = CssColumn*/
const Value V21926_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21926_287 = {S10, 0, &V21926_287_57172, &P21926_237};	/*Property = CssFlexbox*/
const Value V21926_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21926_288 = {S11, 0, &V21926_288_57175, &P21926_287};	/*Property = CssFont*/
const Value V21926_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21926_289 = {S12, 0, &V21926_289_57759, &P21926_288};	/*Property = CssImages*/
const Value V21926_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21926_291 = {S13, 0, &V21926_291_57181, &P21926_289};	/*Property = CssMediaQueries*/
const Value V21926_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21926_292 = {S14, 0, &V21926_292_57183, &P21926_291};	/*Property = CssMinMax*/
const Value V21926_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21926_293 = {S15, 0, &V21926_293_57187, &P21926_292};	/*Property = CssOverflow*/
const Value V21926_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21926_294 = {S16, 0, &V21926_294_57190, &P21926_293};	/*Property = CssPosition*/
const Value V21926_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21926_295 = {S17, 0, &V21926_295_57192, &P21926_294};	/*Property = CssText*/
const Value V21926_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21926_297 = {S18, 0, &V21926_297_57195, &P21926_295};	/*Property = CssTransforms*/
const Value V21926_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21926_241 = {S19, 0, &V21926_241_55799, &P21926_297};	/*Property = CssTransitions*/
const Value V21926_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21926_298 = {S20, 0, &V21926_298_57206, &P21926_241};	/*Property = CssUI*/
const Value V21926_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21926_300 = {S21, 0, &V21926_300_57216, &P21926_298};	/*Property = DataSet*/
const Value V21926_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21926_301 = {S22, 0, &V21926_301_57218, &P21926_300};	/*Property = DataUrl*/
const Value V21926_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21926_302 = {S23, 0, &V21926_302_57222, &P21926_301};	/*Property = DeviceOrientation*/
const Value V21926_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21926_304 = {S24, 0, &V21926_304_57228, &P21926_302};	/*Property = FileReader*/
const Value V21926_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21926_305 = {S25, 0, &V21926_305_57231, &P21926_304};	/*Property = FileSaver*/
const Value V21926_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21926_306 = {S26, 0, &V21926_306_57237, &P21926_305};	/*Property = FileWriter*/
const Value V21926_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21926_307 = {S27, 0, &V21926_307_57240, &P21926_306};	/*Property = FormData*/
const Value V21926_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21926_309 = {S28, 0, &V21926_309_57246, &P21926_307};	/*Property = Fullscreen*/
const Value V21926_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21926_310 = {S29, 0, &V21926_310_57248, &P21926_309};	/*Property = GeoLocation*/
const Value V21926_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21926_312 = {S30, 0, &V21926_312_57255, &P21926_310};	/*Property = History*/
const Value V21926_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21926_314 = {S31, 0, &V21926_314_57261, &P21926_312};	/*Property = Html5*/
const Value V21926_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21926_313 = {S32, 0, &V21926_313_57258, &P21926_314};	/*Property = Html-Media-Capture*/
const Value V21926_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21926_315 = {S33, 0, &V21926_315_57264, &P21926_313};	/*Property = Iframe*/
const Value V21926_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21926_316 = {S34, 0, &V21926_316_57267, &P21926_315};	/*Property = IndexedDB*/
const Value V21926_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21926_317 = {S35, 0, &V21926_317_57269, &P21926_316};	/*Property = Json*/
const Value V21926_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21926_15 = {S37, 0, &V21926_15_28, &P21926_317};	/*Property = LayoutEngine*/
const Value V21926_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21926_344 = {S38, 0, &V21926_344_57272, &P21926_15};	/*Property = Masking*/
const Value V21926_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21926_320 = {S39, 0, &V21926_320_57290, &P21926_344};	/*Property = PostMessage*/
const Value V21926_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21926_321 = {S40, 0, &V21926_321_57293, &P21926_320};	/*Property = Progress*/
const Value V21926_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21926_322 = {S41, 0, &V21926_322_57296, &P21926_321};	/*Property = Prompts*/
const Value V21926_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21926_323 = {S42, 0, &V21926_323_57299, &P21926_322};	/*Property = Selector*/
const Value V21926_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21926_324 = {S43, 0, &V21926_324_57306, &P21926_323};	/*Property = Svg*/
const Value V21926_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21926_327 = {S44, 0, &V21926_327_57315, &P21926_324};	/*Property = TouchEvents*/
const Value V21926_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21926_328 = {S45, 0, &V21926_328_57318, &P21926_327};	/*Property = Track*/
const Value V21926_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21926_329 = {S46, 0, &V21926_329_57323, &P21926_328};	/*Property = Video*/
const Value V21926_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21926_330 = {S47, 0, &V21926_330_57326, &P21926_329};	/*Property = Viewport*/
const Value V21926_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21926_332 = {S48, 0, &V21926_332_57339, &P21926_330};	/*Property = WebWorkers*/
const Value V21926_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21926_333 = {S49, 0, &V21926_333_57342, &P21926_332};	/*Property = Xhr2*/
const Profile P21926 = {"21926", &P21926_333};	/*Profile ID - 21926*/
const Value V16591_1_1 = {S2, NULL};	/*Value = True*/
const Property P16591_1 = {S50, 0, &V16591_1_1, NULL};	/*Property = IsMobile*/
const Value V16591_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P16591_5 = {S52, 0, &V16591_5_1380, &P16591_1};	/*Property = ScreenPixelsHeight*/
const Value V16591_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16591_4 = {S54, 0, &V16591_4_1279, &P16591_5};	/*Property = ScreenPixelsWidth*/
const Profile P16591 = {"16591", &P16591_4};	/*Profile ID - 16591*/
const Value V16950_1_1 = {S2, NULL};	/*Value = True*/
const Property P16950_1 = {S50, 0, &V16950_1_1, NULL};	/*Property = IsMobile*/
const Value V16950_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16950_5 = {S52, 0, &V16950_5_1227, &P16950_1};	/*Property = ScreenPixelsHeight*/
const Value V16950_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16950_4 = {S54, 0, &V16950_4_1229, &P16950_5};	/*Property = ScreenPixelsWidth*/
const Profile P16950 = {"16950", &P16950_4};	/*Profile ID - 16950*/
const Value V23417_1_1 = {S2, NULL};	/*Value = True*/
const Property P23417_1 = {S50, 0, &V23417_1_1, NULL};	/*Property = IsMobile*/
const Value V23417_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23417_5 = {S52, 0, &V23417_5_1222, &P23417_1};	/*Property = ScreenPixelsHeight*/
const Value V23417_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23417_4 = {S54, 0, &V23417_4_1224, &P23417_5};	/*Property = ScreenPixelsWidth*/
const Profile P23417 = {"23417", &P23417_4};	/*Profile ID - 23417*/
const Value V17964_1_1 = {S2, NULL};	/*Value = True*/
const Property P17964_1 = {S50, 0, &V17964_1_1, NULL};	/*Property = IsMobile*/
const Value V17964_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P17964_5 = {S52, 0, &V17964_5_1458, &P17964_1};	/*Property = ScreenPixelsHeight*/
const Value V17964_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P17964_4 = {S54, 0, &V17964_4_1460, &P17964_5};	/*Property = ScreenPixelsWidth*/
const Profile P17964 = {"17964", &P17964_4};	/*Profile ID - 17964*/
const Value V17998_1_1 = {S2, NULL};	/*Value = True*/
const Property P17998_1 = {S50, 0, &V17998_1_1, NULL};	/*Property = IsMobile*/
const Value V17998_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17998_5 = {S52, 0, &V17998_5_1222, &P17998_1};	/*Property = ScreenPixelsHeight*/
const Value V17998_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17998_4 = {S54, 0, &V17998_4_1224, &P17998_5};	/*Property = ScreenPixelsWidth*/
const Profile P17998 = {"17998", &P17998_4};	/*Profile ID - 17998*/
const Value V17487_1_1 = {S2, NULL};	/*Value = True*/
const Property P17487_1 = {S50, 0, &V17487_1_1, NULL};	/*Property = IsMobile*/
const Value V17487_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17487_5 = {S52, 0, &V17487_5_1227, &P17487_1};	/*Property = ScreenPixelsHeight*/
const Value V17487_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17487_4 = {S54, 0, &V17487_4_1229, &P17487_5};	/*Property = ScreenPixelsWidth*/
const Profile P17487 = {"17487", &P17487_4};	/*Profile ID - 17487*/
const Value V18054_1_1 = {S2, NULL};	/*Value = True*/
const Property P18054_1 = {S50, 0, &V18054_1_1, NULL};	/*Property = IsMobile*/
const Value V18054_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P18054_5 = {S52, 0, &V18054_5_1304, &P18054_1};	/*Property = ScreenPixelsHeight*/
const Value V18054_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18054_4 = {S54, 0, &V18054_4_1224, &P18054_5};	/*Property = ScreenPixelsWidth*/
const Profile P18054 = {"18054", &P18054_4};	/*Profile ID - 18054*/
const Value V17582_1_1 = {S2, NULL};	/*Value = True*/
const Property P17582_1 = {S50, 0, &V17582_1_1, NULL};	/*Property = IsMobile*/
const Value V17582_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17582_5 = {S52, 0, &V17582_5_1269, &P17582_1};	/*Property = ScreenPixelsHeight*/
const Value V17582_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17582_4 = {S54, 0, &V17582_4_1229, &P17582_5};	/*Property = ScreenPixelsWidth*/
const Profile P17582 = {"17582", &P17582_4};	/*Profile ID - 17582*/
const Value V17959_1_1 = {S2, NULL};	/*Value = True*/
const Property P17959_1 = {S50, 0, &V17959_1_1, NULL};	/*Property = IsMobile*/
const Value V17959_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17959_5 = {S52, 0, &V17959_5_1227, &P17959_1};	/*Property = ScreenPixelsHeight*/
const Value V17959_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P17959_4 = {S54, 0, &V17959_4_1460, &P17959_5};	/*Property = ScreenPixelsWidth*/
const Profile P17959 = {"17959", &P17959_4};	/*Profile ID - 17959*/
const Value V17969_1_1 = {S2, NULL};	/*Value = True*/
const Property P17969_1 = {S50, 0, &V17969_1_1, NULL};	/*Property = IsMobile*/
const Value V17969_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17969_5 = {S52, 0, &V17969_5_1227, &P17969_1};	/*Property = ScreenPixelsHeight*/
const Value V17969_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17969_4 = {S54, 0, &V17969_4_1229, &P17969_5};	/*Property = ScreenPixelsWidth*/
const Profile P17969 = {"17969", &P17969_4};	/*Profile ID - 17969*/
const Value V18209_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_279 = {S1, 0, &V18209_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18209_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_282 = {S3, 0, &V18209_282_57155, &P18209_279};	/*Property = BlobBuilder*/
const Value V18209_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_350 = {S4, 0, &V18209_350_57446, &P18209_282};	/*Property = Canvas*/
const Value V18209_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_283 = {S5, 0, &V18209_283_57161, &P18209_350};	/*Property = CssBackground*/
const Value V18209_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_284 = {S6, 0, &V18209_284_57164, &P18209_283};	/*Property = CssBorderImage*/
const Value V18209_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_285 = {S7, 0, &V18209_285_57167, &P18209_284};	/*Property = CssCanvas*/
const Value V18209_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_286 = {S8, 0, &V18209_286_57170, &P18209_285};	/*Property = CssColor*/
const Value V18209_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_237 = {S9, 0, &V18209_237_55786, &P18209_286};	/*Property = CssColumn*/
const Value V18209_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_287 = {S10, 0, &V18209_287_57173, &P18209_237};	/*Property = CssFlexbox*/
const Value V18209_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_288 = {S11, 0, &V18209_288_57176, &P18209_287};	/*Property = CssFont*/
const Value V18209_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_289 = {S12, 0, &V18209_289_86899, &P18209_288};	/*Property = CssImages*/
const Value V18209_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_291 = {S13, 0, &V18209_291_57182, &P18209_289};	/*Property = CssMediaQueries*/
const Value V18209_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_292 = {S14, 0, &V18209_292_57185, &P18209_291};	/*Property = CssMinMax*/
const Value V18209_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_293 = {S15, 0, &V18209_293_57188, &P18209_292};	/*Property = CssOverflow*/
const Value V18209_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_294 = {S16, 0, &V18209_294_57191, &P18209_293};	/*Property = CssPosition*/
const Value V18209_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_295 = {S17, 0, &V18209_295_57194, &P18209_294};	/*Property = CssText*/
const Value V18209_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_297 = {S18, 0, &V18209_297_57197, &P18209_295};	/*Property = CssTransforms*/
const Value V18209_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_241 = {S19, 0, &V18209_241_55801, &P18209_297};	/*Property = CssTransitions*/
const Value V18209_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_298 = {S20, 0, &V18209_298_57208, &P18209_241};	/*Property = CssUI*/
const Value V18209_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_300 = {S21, 0, &V18209_300_57217, &P18209_298};	/*Property = DataSet*/
const Value V18209_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_301 = {S22, 0, &V18209_301_57220, &P18209_300};	/*Property = DataUrl*/
const Value V18209_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_302 = {S23, 0, &V18209_302_57223, &P18209_301};	/*Property = DeviceOrientation*/
const Value V18209_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_304 = {S24, 0, &V18209_304_57229, &P18209_302};	/*Property = FileReader*/
const Value V18209_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_305 = {S25, 0, &V18209_305_57232, &P18209_304};	/*Property = FileSaver*/
const Value V18209_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_306 = {S26, 0, &V18209_306_57238, &P18209_305};	/*Property = FileWriter*/
const Value V18209_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_307 = {S27, 0, &V18209_307_57241, &P18209_306};	/*Property = FormData*/
const Value V18209_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_309 = {S28, 0, &V18209_309_57247, &P18209_307};	/*Property = Fullscreen*/
const Value V18209_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_310 = {S29, 0, &V18209_310_57250, &P18209_309};	/*Property = GeoLocation*/
const Value V18209_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_312 = {S30, 0, &V18209_312_57256, &P18209_310};	/*Property = History*/
const Value V18209_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_314 = {S31, 0, &V18209_314_57262, &P18209_312};	/*Property = Html5*/
const Value V18209_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_313 = {S32, 0, &V18209_313_57259, &P18209_314};	/*Property = Html-Media-Capture*/
const Value V18209_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_315 = {S33, 0, &V18209_315_57265, &P18209_313};	/*Property = Iframe*/
const Value V18209_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_316 = {S34, 0, &V18209_316_57268, &P18209_315};	/*Property = IndexedDB*/
const Value V18209_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_317 = {S35, 0, &V18209_317_57271, &P18209_316};	/*Property = Json*/
const Value V18209_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18209_15 = {S37, 0, &V18209_15_29, &P18209_317};	/*Property = LayoutEngine*/
const Value V18209_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_344 = {S38, 0, &V18209_344_57274, &P18209_15};	/*Property = Masking*/
const Value V18209_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_320 = {S39, 0, &V18209_320_57292, &P18209_344};	/*Property = PostMessage*/
const Value V18209_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_321 = {S40, 0, &V18209_321_57295, &P18209_320};	/*Property = Progress*/
const Value V18209_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_322 = {S41, 0, &V18209_322_57298, &P18209_321};	/*Property = Prompts*/
const Value V18209_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_323 = {S42, 0, &V18209_323_57301, &P18209_322};	/*Property = Selector*/
const Value V18209_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_324 = {S43, 0, &V18209_324_57307, &P18209_323};	/*Property = Svg*/
const Value V18209_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_327 = {S44, 0, &V18209_327_57316, &P18209_324};	/*Property = TouchEvents*/
const Value V18209_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_328 = {S45, 0, &V18209_328_57319, &P18209_327};	/*Property = Track*/
const Value V18209_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_329 = {S46, 0, &V18209_329_57325, &P18209_328};	/*Property = Video*/
const Value V18209_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_330 = {S47, 0, &V18209_330_57328, &P18209_329};	/*Property = Viewport*/
const Value V18209_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_332 = {S48, 0, &V18209_332_57340, &P18209_330};	/*Property = WebWorkers*/
const Value V18209_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18209_333 = {S49, 0, &V18209_333_57343, &P18209_332};	/*Property = Xhr2*/
const Profile P18209 = {"18209", &P18209_333};	/*Profile ID - 18209*/
const Value V18569_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_279 = {S1, 0, &V18569_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18569_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_282 = {S3, 0, &V18569_282_57155, &P18569_279};	/*Property = BlobBuilder*/
const Value V18569_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_350 = {S4, 0, &V18569_350_57446, &P18569_282};	/*Property = Canvas*/
const Value V18569_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_283 = {S5, 0, &V18569_283_57161, &P18569_350};	/*Property = CssBackground*/
const Value V18569_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_284 = {S6, 0, &V18569_284_57164, &P18569_283};	/*Property = CssBorderImage*/
const Value V18569_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_285 = {S7, 0, &V18569_285_57167, &P18569_284};	/*Property = CssCanvas*/
const Value V18569_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_286 = {S8, 0, &V18569_286_57170, &P18569_285};	/*Property = CssColor*/
const Value V18569_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_237 = {S9, 0, &V18569_237_55786, &P18569_286};	/*Property = CssColumn*/
const Value V18569_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_287 = {S10, 0, &V18569_287_57173, &P18569_237};	/*Property = CssFlexbox*/
const Value V18569_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_288 = {S11, 0, &V18569_288_57176, &P18569_287};	/*Property = CssFont*/
const Value V18569_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_289 = {S12, 0, &V18569_289_86899, &P18569_288};	/*Property = CssImages*/
const Value V18569_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_291 = {S13, 0, &V18569_291_57182, &P18569_289};	/*Property = CssMediaQueries*/
const Value V18569_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_292 = {S14, 0, &V18569_292_57185, &P18569_291};	/*Property = CssMinMax*/
const Value V18569_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_293 = {S15, 0, &V18569_293_57188, &P18569_292};	/*Property = CssOverflow*/
const Value V18569_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_294 = {S16, 0, &V18569_294_57191, &P18569_293};	/*Property = CssPosition*/
const Value V18569_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_295 = {S17, 0, &V18569_295_57194, &P18569_294};	/*Property = CssText*/
const Value V18569_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_297 = {S18, 0, &V18569_297_57197, &P18569_295};	/*Property = CssTransforms*/
const Value V18569_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_241 = {S19, 0, &V18569_241_55801, &P18569_297};	/*Property = CssTransitions*/
const Value V18569_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_298 = {S20, 0, &V18569_298_57208, &P18569_241};	/*Property = CssUI*/
const Value V18569_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_300 = {S21, 0, &V18569_300_57217, &P18569_298};	/*Property = DataSet*/
const Value V18569_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_301 = {S22, 0, &V18569_301_57220, &P18569_300};	/*Property = DataUrl*/
const Value V18569_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_302 = {S23, 0, &V18569_302_57223, &P18569_301};	/*Property = DeviceOrientation*/
const Value V18569_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_304 = {S24, 0, &V18569_304_57229, &P18569_302};	/*Property = FileReader*/
const Value V18569_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_305 = {S25, 0, &V18569_305_57232, &P18569_304};	/*Property = FileSaver*/
const Value V18569_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_306 = {S26, 0, &V18569_306_57238, &P18569_305};	/*Property = FileWriter*/
const Value V18569_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_307 = {S27, 0, &V18569_307_57241, &P18569_306};	/*Property = FormData*/
const Value V18569_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_309 = {S28, 0, &V18569_309_57247, &P18569_307};	/*Property = Fullscreen*/
const Value V18569_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_310 = {S29, 0, &V18569_310_57250, &P18569_309};	/*Property = GeoLocation*/
const Value V18569_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_312 = {S30, 0, &V18569_312_57256, &P18569_310};	/*Property = History*/
const Value V18569_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_314 = {S31, 0, &V18569_314_57262, &P18569_312};	/*Property = Html5*/
const Value V18569_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_313 = {S32, 0, &V18569_313_57259, &P18569_314};	/*Property = Html-Media-Capture*/
const Value V18569_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_315 = {S33, 0, &V18569_315_57265, &P18569_313};	/*Property = Iframe*/
const Value V18569_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_316 = {S34, 0, &V18569_316_57268, &P18569_315};	/*Property = IndexedDB*/
const Value V18569_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_317 = {S35, 0, &V18569_317_57271, &P18569_316};	/*Property = Json*/
const Value V18569_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_15 = {S37, 0, &V18569_15_72, &P18569_317};	/*Property = LayoutEngine*/
const Value V18569_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_344 = {S38, 0, &V18569_344_57274, &P18569_15};	/*Property = Masking*/
const Value V18569_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_320 = {S39, 0, &V18569_320_57292, &P18569_344};	/*Property = PostMessage*/
const Value V18569_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_321 = {S40, 0, &V18569_321_57295, &P18569_320};	/*Property = Progress*/
const Value V18569_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_322 = {S41, 0, &V18569_322_57298, &P18569_321};	/*Property = Prompts*/
const Value V18569_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_323 = {S42, 0, &V18569_323_57301, &P18569_322};	/*Property = Selector*/
const Value V18569_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_324 = {S43, 0, &V18569_324_57307, &P18569_323};	/*Property = Svg*/
const Value V18569_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_327 = {S44, 0, &V18569_327_57316, &P18569_324};	/*Property = TouchEvents*/
const Value V18569_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_328 = {S45, 0, &V18569_328_57319, &P18569_327};	/*Property = Track*/
const Value V18569_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_329 = {S46, 0, &V18569_329_57325, &P18569_328};	/*Property = Video*/
const Value V18569_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_330 = {S47, 0, &V18569_330_57328, &P18569_329};	/*Property = Viewport*/
const Value V18569_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_332 = {S48, 0, &V18569_332_57340, &P18569_330};	/*Property = WebWorkers*/
const Value V18569_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18569_333 = {S49, 0, &V18569_333_57343, &P18569_332};	/*Property = Xhr2*/
const Profile P18569 = {"18569", &P18569_333};	/*Profile ID - 18569*/
const Value V16595_1_1 = {S2, NULL};	/*Value = True*/
const Property P16595_1 = {S50, 0, &V16595_1_1, NULL};	/*Property = IsMobile*/
const Value V16595_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16595_5 = {S52, 0, &V16595_5_1222, &P16595_1};	/*Property = ScreenPixelsHeight*/
const Value V16595_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16595_4 = {S54, 0, &V16595_4_1224, &P16595_5};	/*Property = ScreenPixelsWidth*/
const Profile P16595 = {"16595", &P16595_4};	/*Profile ID - 16595*/
const Value V18105_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18105_279 = {S1, 0, &V18105_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18105_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18105_282 = {S3, 0, &V18105_282_57154, &P18105_279};	/*Property = BlobBuilder*/
const Value V18105_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18105_350 = {S4, 0, &V18105_350_57444, &P18105_282};	/*Property = Canvas*/
const Value V18105_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18105_283 = {S5, 0, &V18105_283_57159, &P18105_350};	/*Property = CssBackground*/
const Value V18105_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18105_284 = {S6, 0, &V18105_284_57163, &P18105_283};	/*Property = CssBorderImage*/
const Value V18105_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18105_285 = {S7, 0, &V18105_285_57166, &P18105_284};	/*Property = CssCanvas*/
const Value V18105_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18105_286 = {S8, 0, &V18105_286_57168, &P18105_285};	/*Property = CssColor*/
const Value V18105_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18105_237 = {S9, 0, &V18105_237_55786, &P18105_286};	/*Property = CssColumn*/
const Value V18105_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18105_287 = {S10, 0, &V18105_287_57172, &P18105_237};	/*Property = CssFlexbox*/
const Value V18105_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18105_288 = {S11, 0, &V18105_288_57175, &P18105_287};	/*Property = CssFont*/
const Value V18105_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18105_289 = {S12, 0, &V18105_289_57759, &P18105_288};	/*Property = CssImages*/
const Value V18105_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18105_291 = {S13, 0, &V18105_291_57181, &P18105_289};	/*Property = CssMediaQueries*/
const Value V18105_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18105_292 = {S14, 0, &V18105_292_57183, &P18105_291};	/*Property = CssMinMax*/
const Value V18105_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18105_293 = {S15, 0, &V18105_293_57187, &P18105_292};	/*Property = CssOverflow*/
const Value V18105_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18105_294 = {S16, 0, &V18105_294_57190, &P18105_293};	/*Property = CssPosition*/
const Value V18105_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18105_295 = {S17, 0, &V18105_295_57193, &P18105_294};	/*Property = CssText*/
const Value V18105_297_57196 = {S0, NULL};	/*Value = False*/
const Property P18105_297 = {S18, 0, &V18105_297_57196, &P18105_295};	/*Property = CssTransforms*/
const Value V18105_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18105_241 = {S19, 0, &V18105_241_55799, &P18105_297};	/*Property = CssTransitions*/
const Value V18105_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18105_298 = {S20, 0, &V18105_298_57206, &P18105_241};	/*Property = CssUI*/
const Value V18105_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18105_300 = {S21, 0, &V18105_300_57216, &P18105_298};	/*Property = DataSet*/
const Value V18105_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18105_301 = {S22, 0, &V18105_301_57218, &P18105_300};	/*Property = DataUrl*/
const Value V18105_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18105_302 = {S23, 0, &V18105_302_57222, &P18105_301};	/*Property = DeviceOrientation*/
const Value V18105_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18105_304 = {S24, 0, &V18105_304_57228, &P18105_302};	/*Property = FileReader*/
const Value V18105_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18105_305 = {S25, 0, &V18105_305_57231, &P18105_304};	/*Property = FileSaver*/
const Value V18105_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18105_306 = {S26, 0, &V18105_306_57237, &P18105_305};	/*Property = FileWriter*/
const Value V18105_307_57240 = {S0, NULL};	/*Value = False*/
const Property P18105_307 = {S27, 0, &V18105_307_57240, &P18105_306};	/*Property = FormData*/
const Value V18105_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18105_309 = {S28, 0, &V18105_309_57246, &P18105_307};	/*Property = Fullscreen*/
const Value V18105_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18105_310 = {S29, 0, &V18105_310_57248, &P18105_309};	/*Property = GeoLocation*/
const Value V18105_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18105_312 = {S30, 0, &V18105_312_57255, &P18105_310};	/*Property = History*/
const Value V18105_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18105_314 = {S31, 0, &V18105_314_57261, &P18105_312};	/*Property = Html5*/
const Value V18105_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18105_313 = {S32, 0, &V18105_313_57258, &P18105_314};	/*Property = Html-Media-Capture*/
const Value V18105_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18105_315 = {S33, 0, &V18105_315_57264, &P18105_313};	/*Property = Iframe*/
const Value V18105_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18105_316 = {S34, 0, &V18105_316_57267, &P18105_315};	/*Property = IndexedDB*/
const Value V18105_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18105_317 = {S35, 0, &V18105_317_57269, &P18105_316};	/*Property = Json*/
const Value V18105_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18105_15 = {S37, 0, &V18105_15_31, &P18105_317};	/*Property = LayoutEngine*/
const Value V18105_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18105_344 = {S38, 0, &V18105_344_57272, &P18105_15};	/*Property = Masking*/
const Value V18105_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18105_320 = {S39, 0, &V18105_320_57290, &P18105_344};	/*Property = PostMessage*/
const Value V18105_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18105_321 = {S40, 0, &V18105_321_57293, &P18105_320};	/*Property = Progress*/
const Value V18105_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18105_322 = {S41, 0, &V18105_322_57296, &P18105_321};	/*Property = Prompts*/
const Value V18105_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18105_323 = {S42, 0, &V18105_323_57301, &P18105_322};	/*Property = Selector*/
const Value V18105_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18105_324 = {S43, 0, &V18105_324_57306, &P18105_323};	/*Property = Svg*/
const Value V18105_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18105_327 = {S44, 0, &V18105_327_57315, &P18105_324};	/*Property = TouchEvents*/
const Value V18105_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18105_328 = {S45, 0, &V18105_328_57318, &P18105_327};	/*Property = Track*/
const Value V18105_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18105_329 = {S46, 0, &V18105_329_57323, &P18105_328};	/*Property = Video*/
const Value V18105_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18105_330 = {S47, 0, &V18105_330_57326, &P18105_329};	/*Property = Viewport*/
const Value V18105_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18105_332 = {S48, 0, &V18105_332_57339, &P18105_330};	/*Property = WebWorkers*/
const Value V18105_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18105_333 = {S49, 0, &V18105_333_57342, &P18105_332};	/*Property = Xhr2*/
const Profile P18105 = {"18105", &P18105_333};	/*Profile ID - 18105*/
const Value V22400_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22400_279 = {S1, 0, &V22400_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22400_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22400_282 = {S3, 0, &V22400_282_57154, &P22400_279};	/*Property = BlobBuilder*/
const Value V22400_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22400_350 = {S4, 0, &V22400_350_57444, &P22400_282};	/*Property = Canvas*/
const Value V22400_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22400_283 = {S5, 0, &V22400_283_57159, &P22400_350};	/*Property = CssBackground*/
const Value V22400_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22400_284 = {S6, 0, &V22400_284_57163, &P22400_283};	/*Property = CssBorderImage*/
const Value V22400_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22400_285 = {S7, 0, &V22400_285_57166, &P22400_284};	/*Property = CssCanvas*/
const Value V22400_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22400_286 = {S8, 0, &V22400_286_57168, &P22400_285};	/*Property = CssColor*/
const Value V22400_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22400_237 = {S9, 0, &V22400_237_55784, &P22400_286};	/*Property = CssColumn*/
const Value V22400_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22400_287 = {S10, 0, &V22400_287_57172, &P22400_237};	/*Property = CssFlexbox*/
const Value V22400_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22400_288 = {S11, 0, &V22400_288_57175, &P22400_287};	/*Property = CssFont*/
const Value V22400_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22400_289 = {S12, 0, &V22400_289_57759, &P22400_288};	/*Property = CssImages*/
const Value V22400_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22400_291 = {S13, 0, &V22400_291_57181, &P22400_289};	/*Property = CssMediaQueries*/
const Value V22400_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22400_292 = {S14, 0, &V22400_292_57183, &P22400_291};	/*Property = CssMinMax*/
const Value V22400_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22400_293 = {S15, 0, &V22400_293_57187, &P22400_292};	/*Property = CssOverflow*/
const Value V22400_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22400_294 = {S16, 0, &V22400_294_57190, &P22400_293};	/*Property = CssPosition*/
const Value V22400_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22400_295 = {S17, 0, &V22400_295_57192, &P22400_294};	/*Property = CssText*/
const Value V22400_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22400_297 = {S18, 0, &V22400_297_57195, &P22400_295};	/*Property = CssTransforms*/
const Value V22400_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22400_241 = {S19, 0, &V22400_241_55799, &P22400_297};	/*Property = CssTransitions*/
const Value V22400_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22400_298 = {S20, 0, &V22400_298_57206, &P22400_241};	/*Property = CssUI*/
const Value V22400_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22400_300 = {S21, 0, &V22400_300_57216, &P22400_298};	/*Property = DataSet*/
const Value V22400_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22400_301 = {S22, 0, &V22400_301_57218, &P22400_300};	/*Property = DataUrl*/
const Value V22400_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22400_302 = {S23, 0, &V22400_302_57222, &P22400_301};	/*Property = DeviceOrientation*/
const Value V22400_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22400_304 = {S24, 0, &V22400_304_57228, &P22400_302};	/*Property = FileReader*/
const Value V22400_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22400_305 = {S25, 0, &V22400_305_57231, &P22400_304};	/*Property = FileSaver*/
const Value V22400_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22400_306 = {S26, 0, &V22400_306_57237, &P22400_305};	/*Property = FileWriter*/
const Value V22400_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22400_307 = {S27, 0, &V22400_307_57240, &P22400_306};	/*Property = FormData*/
const Value V22400_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22400_309 = {S28, 0, &V22400_309_57246, &P22400_307};	/*Property = Fullscreen*/
const Value V22400_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22400_310 = {S29, 0, &V22400_310_57248, &P22400_309};	/*Property = GeoLocation*/
const Value V22400_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22400_312 = {S30, 0, &V22400_312_57255, &P22400_310};	/*Property = History*/
const Value V22400_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22400_314 = {S31, 0, &V22400_314_57261, &P22400_312};	/*Property = Html5*/
const Value V22400_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22400_313 = {S32, 0, &V22400_313_57258, &P22400_314};	/*Property = Html-Media-Capture*/
const Value V22400_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22400_315 = {S33, 0, &V22400_315_57264, &P22400_313};	/*Property = Iframe*/
const Value V22400_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22400_316 = {S34, 0, &V22400_316_57267, &P22400_315};	/*Property = IndexedDB*/
const Value V22400_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22400_317 = {S35, 0, &V22400_317_57269, &P22400_316};	/*Property = Json*/
const Value V22400_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22400_15 = {S37, 0, &V22400_15_28, &P22400_317};	/*Property = LayoutEngine*/
const Value V22400_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22400_344 = {S38, 0, &V22400_344_57272, &P22400_15};	/*Property = Masking*/
const Value V22400_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22400_320 = {S39, 0, &V22400_320_57290, &P22400_344};	/*Property = PostMessage*/
const Value V22400_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22400_321 = {S40, 0, &V22400_321_57293, &P22400_320};	/*Property = Progress*/
const Value V22400_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22400_322 = {S41, 0, &V22400_322_57296, &P22400_321};	/*Property = Prompts*/
const Value V22400_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22400_323 = {S42, 0, &V22400_323_57299, &P22400_322};	/*Property = Selector*/
const Value V22400_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22400_324 = {S43, 0, &V22400_324_57306, &P22400_323};	/*Property = Svg*/
const Value V22400_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22400_327 = {S44, 0, &V22400_327_57315, &P22400_324};	/*Property = TouchEvents*/
const Value V22400_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22400_328 = {S45, 0, &V22400_328_57318, &P22400_327};	/*Property = Track*/
const Value V22400_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22400_329 = {S46, 0, &V22400_329_57323, &P22400_328};	/*Property = Video*/
const Value V22400_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22400_330 = {S47, 0, &V22400_330_57326, &P22400_329};	/*Property = Viewport*/
const Value V22400_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22400_332 = {S48, 0, &V22400_332_57339, &P22400_330};	/*Property = WebWorkers*/
const Value V22400_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22400_333 = {S49, 0, &V22400_333_57342, &P22400_332};	/*Property = Xhr2*/
const Profile P22400 = {"22400", &P22400_333};	/*Profile ID - 22400*/
const Value V17649_1_1 = {S2, NULL};	/*Value = True*/
const Property P17649_1 = {S50, 0, &V17649_1_1, NULL};	/*Property = IsMobile*/
const Value V17649_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17649_5 = {S52, 0, &V17649_5_1269, &P17649_1};	/*Property = ScreenPixelsHeight*/
const Value V17649_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17649_4 = {S54, 0, &V17649_4_1229, &P17649_5};	/*Property = ScreenPixelsWidth*/
const Profile P17649 = {"17649", &P17649_4};	/*Profile ID - 17649*/
const Value V18032_1_1 = {S2, NULL};	/*Value = True*/
const Property P18032_1 = {S50, 0, &V18032_1_1, NULL};	/*Property = IsMobile*/
const Value V18032_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18032_5 = {S52, 0, &V18032_5_1227, &P18032_1};	/*Property = ScreenPixelsHeight*/
const Value V18032_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18032_4 = {S54, 0, &V18032_4_1229, &P18032_5};	/*Property = ScreenPixelsWidth*/
const Profile P18032 = {"18032", &P18032_4};	/*Profile ID - 18032*/
const Value V18404_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_279 = {S1, 0, &V18404_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18404_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_282 = {S3, 0, &V18404_282_57155, &P18404_279};	/*Property = BlobBuilder*/
const Value V18404_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_350 = {S4, 0, &V18404_350_57446, &P18404_282};	/*Property = Canvas*/
const Value V18404_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_283 = {S5, 0, &V18404_283_57161, &P18404_350};	/*Property = CssBackground*/
const Value V18404_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_284 = {S6, 0, &V18404_284_57164, &P18404_283};	/*Property = CssBorderImage*/
const Value V18404_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_285 = {S7, 0, &V18404_285_57167, &P18404_284};	/*Property = CssCanvas*/
const Value V18404_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_286 = {S8, 0, &V18404_286_57170, &P18404_285};	/*Property = CssColor*/
const Value V18404_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_237 = {S9, 0, &V18404_237_55786, &P18404_286};	/*Property = CssColumn*/
const Value V18404_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_287 = {S10, 0, &V18404_287_57173, &P18404_237};	/*Property = CssFlexbox*/
const Value V18404_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_288 = {S11, 0, &V18404_288_57176, &P18404_287};	/*Property = CssFont*/
const Value V18404_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_289 = {S12, 0, &V18404_289_86899, &P18404_288};	/*Property = CssImages*/
const Value V18404_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_291 = {S13, 0, &V18404_291_57182, &P18404_289};	/*Property = CssMediaQueries*/
const Value V18404_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_292 = {S14, 0, &V18404_292_57185, &P18404_291};	/*Property = CssMinMax*/
const Value V18404_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_293 = {S15, 0, &V18404_293_57188, &P18404_292};	/*Property = CssOverflow*/
const Value V18404_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_294 = {S16, 0, &V18404_294_57191, &P18404_293};	/*Property = CssPosition*/
const Value V18404_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_295 = {S17, 0, &V18404_295_57194, &P18404_294};	/*Property = CssText*/
const Value V18404_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_297 = {S18, 0, &V18404_297_57197, &P18404_295};	/*Property = CssTransforms*/
const Value V18404_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_241 = {S19, 0, &V18404_241_55801, &P18404_297};	/*Property = CssTransitions*/
const Value V18404_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_298 = {S20, 0, &V18404_298_57208, &P18404_241};	/*Property = CssUI*/
const Value V18404_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_300 = {S21, 0, &V18404_300_57217, &P18404_298};	/*Property = DataSet*/
const Value V18404_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_301 = {S22, 0, &V18404_301_57220, &P18404_300};	/*Property = DataUrl*/
const Value V18404_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_302 = {S23, 0, &V18404_302_57223, &P18404_301};	/*Property = DeviceOrientation*/
const Value V18404_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_304 = {S24, 0, &V18404_304_57229, &P18404_302};	/*Property = FileReader*/
const Value V18404_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_305 = {S25, 0, &V18404_305_57232, &P18404_304};	/*Property = FileSaver*/
const Value V18404_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_306 = {S26, 0, &V18404_306_57238, &P18404_305};	/*Property = FileWriter*/
const Value V18404_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_307 = {S27, 0, &V18404_307_57241, &P18404_306};	/*Property = FormData*/
const Value V18404_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_309 = {S28, 0, &V18404_309_57247, &P18404_307};	/*Property = Fullscreen*/
const Value V18404_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_310 = {S29, 0, &V18404_310_57250, &P18404_309};	/*Property = GeoLocation*/
const Value V18404_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_312 = {S30, 0, &V18404_312_57256, &P18404_310};	/*Property = History*/
const Value V18404_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_314 = {S31, 0, &V18404_314_57262, &P18404_312};	/*Property = Html5*/
const Value V18404_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_313 = {S32, 0, &V18404_313_57259, &P18404_314};	/*Property = Html-Media-Capture*/
const Value V18404_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_315 = {S33, 0, &V18404_315_57265, &P18404_313};	/*Property = Iframe*/
const Value V18404_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_316 = {S34, 0, &V18404_316_57268, &P18404_315};	/*Property = IndexedDB*/
const Value V18404_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_317 = {S35, 0, &V18404_317_57271, &P18404_316};	/*Property = Json*/
const Value V18404_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18404_15 = {S37, 0, &V18404_15_31, &P18404_317};	/*Property = LayoutEngine*/
const Value V18404_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_344 = {S38, 0, &V18404_344_57274, &P18404_15};	/*Property = Masking*/
const Value V18404_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_320 = {S39, 0, &V18404_320_57292, &P18404_344};	/*Property = PostMessage*/
const Value V18404_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_321 = {S40, 0, &V18404_321_57295, &P18404_320};	/*Property = Progress*/
const Value V18404_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_322 = {S41, 0, &V18404_322_57298, &P18404_321};	/*Property = Prompts*/
const Value V18404_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_323 = {S42, 0, &V18404_323_57301, &P18404_322};	/*Property = Selector*/
const Value V18404_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_324 = {S43, 0, &V18404_324_57307, &P18404_323};	/*Property = Svg*/
const Value V18404_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_327 = {S44, 0, &V18404_327_57316, &P18404_324};	/*Property = TouchEvents*/
const Value V18404_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_328 = {S45, 0, &V18404_328_57319, &P18404_327};	/*Property = Track*/
const Value V18404_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_329 = {S46, 0, &V18404_329_57325, &P18404_328};	/*Property = Video*/
const Value V18404_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_330 = {S47, 0, &V18404_330_57328, &P18404_329};	/*Property = Viewport*/
const Value V18404_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_332 = {S48, 0, &V18404_332_57340, &P18404_330};	/*Property = WebWorkers*/
const Value V18404_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18404_333 = {S49, 0, &V18404_333_57343, &P18404_332};	/*Property = Xhr2*/
const Profile P18404 = {"18404", &P18404_333};	/*Profile ID - 18404*/
const Value V17962_1_1 = {S2, NULL};	/*Value = True*/
const Property P17962_1 = {S50, 0, &V17962_1_1, NULL};	/*Property = IsMobile*/
const Value V17962_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17962_5 = {S52, 0, &V17962_5_1227, &P17962_1};	/*Property = ScreenPixelsHeight*/
const Value V17962_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17962_4 = {S54, 0, &V17962_4_1229, &P17962_5};	/*Property = ScreenPixelsWidth*/
const Profile P17962 = {"17962", &P17962_4};	/*Profile ID - 17962*/
const Value V16593_1_1 = {S2, NULL};	/*Value = True*/
const Property P16593_1 = {S50, 0, &V16593_1_1, NULL};	/*Property = IsMobile*/
const Value V16593_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P16593_5 = {S52, 0, &V16593_5_1304, &P16593_1};	/*Property = ScreenPixelsHeight*/
const Value V16593_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16593_4 = {S54, 0, &V16593_4_1279, &P16593_5};	/*Property = ScreenPixelsWidth*/
const Profile P16593 = {"16593", &P16593_4};	/*Profile ID - 16593*/
const Value V17977_1_1 = {S2, NULL};	/*Value = True*/
const Property P17977_1 = {S50, 0, &V17977_1_1, NULL};	/*Property = IsMobile*/
const Value V17977_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17977_5 = {S52, 0, &V17977_5_1277, &P17977_1};	/*Property = ScreenPixelsHeight*/
const Value V17977_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17977_4 = {S54, 0, &V17977_4_1279, &P17977_5};	/*Property = ScreenPixelsWidth*/
const Profile P17977 = {"17977", &P17977_4};	/*Profile ID - 17977*/
const Value V18010_1_1 = {S2, NULL};	/*Value = True*/
const Property P18010_1 = {S50, 0, &V18010_1_1, NULL};	/*Property = IsMobile*/
const Value V18010_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18010_5 = {S52, 0, &V18010_5_1403, &P18010_1};	/*Property = ScreenPixelsHeight*/
const Value V18010_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P18010_4 = {S54, 0, &V18010_4_8554, &P18010_5};	/*Property = ScreenPixelsWidth*/
const Profile P18010 = {"18010", &P18010_4};	/*Profile ID - 18010*/
const Value V20803_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_279 = {S1, 0, &V20803_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20803_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_282 = {S3, 0, &V20803_282_57155, &P20803_279};	/*Property = BlobBuilder*/
const Value V20803_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_350 = {S4, 0, &V20803_350_57446, &P20803_282};	/*Property = Canvas*/
const Value V20803_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_283 = {S5, 0, &V20803_283_57161, &P20803_350};	/*Property = CssBackground*/
const Value V20803_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_284 = {S6, 0, &V20803_284_57164, &P20803_283};	/*Property = CssBorderImage*/
const Value V20803_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_285 = {S7, 0, &V20803_285_57167, &P20803_284};	/*Property = CssCanvas*/
const Value V20803_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_286 = {S8, 0, &V20803_286_57170, &P20803_285};	/*Property = CssColor*/
const Value V20803_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_237 = {S9, 0, &V20803_237_55786, &P20803_286};	/*Property = CssColumn*/
const Value V20803_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_287 = {S10, 0, &V20803_287_57173, &P20803_237};	/*Property = CssFlexbox*/
const Value V20803_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_288 = {S11, 0, &V20803_288_57176, &P20803_287};	/*Property = CssFont*/
const Value V20803_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_289 = {S12, 0, &V20803_289_86899, &P20803_288};	/*Property = CssImages*/
const Value V20803_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_291 = {S13, 0, &V20803_291_57182, &P20803_289};	/*Property = CssMediaQueries*/
const Value V20803_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_292 = {S14, 0, &V20803_292_57185, &P20803_291};	/*Property = CssMinMax*/
const Value V20803_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_293 = {S15, 0, &V20803_293_57188, &P20803_292};	/*Property = CssOverflow*/
const Value V20803_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_294 = {S16, 0, &V20803_294_57191, &P20803_293};	/*Property = CssPosition*/
const Value V20803_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_295 = {S17, 0, &V20803_295_57194, &P20803_294};	/*Property = CssText*/
const Value V20803_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_297 = {S18, 0, &V20803_297_57197, &P20803_295};	/*Property = CssTransforms*/
const Value V20803_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_241 = {S19, 0, &V20803_241_55801, &P20803_297};	/*Property = CssTransitions*/
const Value V20803_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_298 = {S20, 0, &V20803_298_57208, &P20803_241};	/*Property = CssUI*/
const Value V20803_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_300 = {S21, 0, &V20803_300_57217, &P20803_298};	/*Property = DataSet*/
const Value V20803_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_301 = {S22, 0, &V20803_301_57220, &P20803_300};	/*Property = DataUrl*/
const Value V20803_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_302 = {S23, 0, &V20803_302_57223, &P20803_301};	/*Property = DeviceOrientation*/
const Value V20803_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_304 = {S24, 0, &V20803_304_57229, &P20803_302};	/*Property = FileReader*/
const Value V20803_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_305 = {S25, 0, &V20803_305_57232, &P20803_304};	/*Property = FileSaver*/
const Value V20803_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_306 = {S26, 0, &V20803_306_57238, &P20803_305};	/*Property = FileWriter*/
const Value V20803_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_307 = {S27, 0, &V20803_307_57241, &P20803_306};	/*Property = FormData*/
const Value V20803_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_309 = {S28, 0, &V20803_309_57247, &P20803_307};	/*Property = Fullscreen*/
const Value V20803_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_310 = {S29, 0, &V20803_310_57250, &P20803_309};	/*Property = GeoLocation*/
const Value V20803_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_312 = {S30, 0, &V20803_312_57256, &P20803_310};	/*Property = History*/
const Value V20803_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_314 = {S31, 0, &V20803_314_57262, &P20803_312};	/*Property = Html5*/
const Value V20803_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_313 = {S32, 0, &V20803_313_57259, &P20803_314};	/*Property = Html-Media-Capture*/
const Value V20803_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_315 = {S33, 0, &V20803_315_57265, &P20803_313};	/*Property = Iframe*/
const Value V20803_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_316 = {S34, 0, &V20803_316_57268, &P20803_315};	/*Property = IndexedDB*/
const Value V20803_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_317 = {S35, 0, &V20803_317_57271, &P20803_316};	/*Property = Json*/
const Value V20803_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P20803_15 = {S37, 0, &V20803_15_54167, &P20803_317};	/*Property = LayoutEngine*/
const Value V20803_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_344 = {S38, 0, &V20803_344_57274, &P20803_15};	/*Property = Masking*/
const Value V20803_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_320 = {S39, 0, &V20803_320_57292, &P20803_344};	/*Property = PostMessage*/
const Value V20803_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_321 = {S40, 0, &V20803_321_57295, &P20803_320};	/*Property = Progress*/
const Value V20803_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_322 = {S41, 0, &V20803_322_57298, &P20803_321};	/*Property = Prompts*/
const Value V20803_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_323 = {S42, 0, &V20803_323_57301, &P20803_322};	/*Property = Selector*/
const Value V20803_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_324 = {S43, 0, &V20803_324_57307, &P20803_323};	/*Property = Svg*/
const Value V20803_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_327 = {S44, 0, &V20803_327_57316, &P20803_324};	/*Property = TouchEvents*/
const Value V20803_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_328 = {S45, 0, &V20803_328_57319, &P20803_327};	/*Property = Track*/
const Value V20803_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_329 = {S46, 0, &V20803_329_57325, &P20803_328};	/*Property = Video*/
const Value V20803_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_330 = {S47, 0, &V20803_330_57328, &P20803_329};	/*Property = Viewport*/
const Value V20803_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_332 = {S48, 0, &V20803_332_57340, &P20803_330};	/*Property = WebWorkers*/
const Value V20803_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20803_333 = {S49, 0, &V20803_333_57343, &P20803_332};	/*Property = Xhr2*/
const Profile P20803 = {"20803", &P20803_333};	/*Profile ID - 20803*/
const Value V17963_1_1 = {S2, NULL};	/*Value = True*/
const Property P17963_1 = {S50, 0, &V17963_1_1, NULL};	/*Property = IsMobile*/
const Value V17963_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17963_5 = {S52, 0, &V17963_5_1227, &P17963_1};	/*Property = ScreenPixelsHeight*/
const Value V17963_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17963_4 = {S54, 0, &V17963_4_1229, &P17963_5};	/*Property = ScreenPixelsWidth*/
const Profile P17963 = {"17963", &P17963_4};	/*Profile ID - 17963*/
const Value V16587_1_1 = {S2, NULL};	/*Value = True*/
const Property P16587_1 = {S50, 0, &V16587_1_1, NULL};	/*Property = IsMobile*/
const Value V16587_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16587_5 = {S52, 0, &V16587_5_1227, &P16587_1};	/*Property = ScreenPixelsHeight*/
const Value V16587_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16587_4 = {S54, 0, &V16587_4_1229, &P16587_5};	/*Property = ScreenPixelsWidth*/
const Profile P16587 = {"16587", &P16587_4};	/*Profile ID - 16587*/
const Value V18034_1_1 = {S2, NULL};	/*Value = True*/
const Property P18034_1 = {S50, 0, &V18034_1_1, NULL};	/*Property = IsMobile*/
const Value V18034_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18034_5 = {S52, 0, &V18034_5_1227, &P18034_1};	/*Property = ScreenPixelsHeight*/
const Value V18034_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18034_4 = {S54, 0, &V18034_4_1229, &P18034_5};	/*Property = ScreenPixelsWidth*/
const Profile P18034 = {"18034", &P18034_4};	/*Profile ID - 18034*/
const Value V16541_1_1 = {S2, NULL};	/*Value = True*/
const Property P16541_1 = {S50, 0, &V16541_1_1, NULL};	/*Property = IsMobile*/
const Value V16541_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16541_5 = {S52, 0, &V16541_5_1227, &P16541_1};	/*Property = ScreenPixelsHeight*/
const Value V16541_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16541_4 = {S54, 0, &V16541_4_1229, &P16541_5};	/*Property = ScreenPixelsWidth*/
const Profile P16541 = {"16541", &P16541_4};	/*Profile ID - 16541*/
const Value V18041_1_1 = {S2, NULL};	/*Value = True*/
const Property P18041_1 = {S50, 0, &V18041_1_1, NULL};	/*Property = IsMobile*/
const Value V18041_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18041_5 = {S52, 0, &V18041_5_1227, &P18041_1};	/*Property = ScreenPixelsHeight*/
const Value V18041_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18041_4 = {S54, 0, &V18041_4_1229, &P18041_5};	/*Property = ScreenPixelsWidth*/
const Profile P18041 = {"18041", &P18041_4};	/*Profile ID - 18041*/
const Value V18019_1_1 = {S2, NULL};	/*Value = True*/
const Property P18019_1 = {S50, 0, &V18019_1_1, NULL};	/*Property = IsMobile*/
const Value V18019_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18019_5 = {S52, 0, &V18019_5_1227, &P18019_1};	/*Property = ScreenPixelsHeight*/
const Value V18019_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18019_4 = {S54, 0, &V18019_4_1229, &P18019_5};	/*Property = ScreenPixelsWidth*/
const Profile P18019 = {"18019", &P18019_4};	/*Profile ID - 18019*/
const Value V22447_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22447_279 = {S1, 0, &V22447_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22447_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22447_282 = {S3, 0, &V22447_282_57153, &P22447_279};	/*Property = BlobBuilder*/
const Value V22447_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22447_350 = {S4, 0, &V22447_350_57444, &P22447_282};	/*Property = Canvas*/
const Value V22447_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22447_283 = {S5, 0, &V22447_283_57159, &P22447_350};	/*Property = CssBackground*/
const Value V22447_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22447_284 = {S6, 0, &V22447_284_57163, &P22447_283};	/*Property = CssBorderImage*/
const Value V22447_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22447_285 = {S7, 0, &V22447_285_57166, &P22447_284};	/*Property = CssCanvas*/
const Value V22447_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22447_286 = {S8, 0, &V22447_286_57168, &P22447_285};	/*Property = CssColor*/
const Value V22447_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22447_237 = {S9, 0, &V22447_237_55786, &P22447_286};	/*Property = CssColumn*/
const Value V22447_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22447_287 = {S10, 0, &V22447_287_57172, &P22447_237};	/*Property = CssFlexbox*/
const Value V22447_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22447_288 = {S11, 0, &V22447_288_57175, &P22447_287};	/*Property = CssFont*/
const Value V22447_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22447_289 = {S12, 0, &V22447_289_57759, &P22447_288};	/*Property = CssImages*/
const Value V22447_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22447_291 = {S13, 0, &V22447_291_57180, &P22447_289};	/*Property = CssMediaQueries*/
const Value V22447_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22447_292 = {S14, 0, &V22447_292_57183, &P22447_291};	/*Property = CssMinMax*/
const Value V22447_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22447_293 = {S15, 0, &V22447_293_57187, &P22447_292};	/*Property = CssOverflow*/
const Value V22447_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22447_294 = {S16, 0, &V22447_294_57189, &P22447_293};	/*Property = CssPosition*/
const Value V22447_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22447_295 = {S17, 0, &V22447_295_57192, &P22447_294};	/*Property = CssText*/
const Value V22447_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22447_297 = {S18, 0, &V22447_297_57195, &P22447_295};	/*Property = CssTransforms*/
const Value V22447_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22447_241 = {S19, 0, &V22447_241_55799, &P22447_297};	/*Property = CssTransitions*/
const Value V22447_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22447_298 = {S20, 0, &V22447_298_57206, &P22447_241};	/*Property = CssUI*/
const Value V22447_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22447_300 = {S21, 0, &V22447_300_57216, &P22447_298};	/*Property = DataSet*/
const Value V22447_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22447_301 = {S22, 0, &V22447_301_57218, &P22447_300};	/*Property = DataUrl*/
const Value V22447_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22447_302 = {S23, 0, &V22447_302_57222, &P22447_301};	/*Property = DeviceOrientation*/
const Value V22447_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22447_304 = {S24, 0, &V22447_304_57227, &P22447_302};	/*Property = FileReader*/
const Value V22447_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22447_305 = {S25, 0, &V22447_305_57231, &P22447_304};	/*Property = FileSaver*/
const Value V22447_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22447_306 = {S26, 0, &V22447_306_57237, &P22447_305};	/*Property = FileWriter*/
const Value V22447_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22447_307 = {S27, 0, &V22447_307_57239, &P22447_306};	/*Property = FormData*/
const Value V22447_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22447_309 = {S28, 0, &V22447_309_57246, &P22447_307};	/*Property = Fullscreen*/
const Value V22447_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22447_310 = {S29, 0, &V22447_310_57248, &P22447_309};	/*Property = GeoLocation*/
const Value V22447_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22447_312 = {S30, 0, &V22447_312_57255, &P22447_310};	/*Property = History*/
const Value V22447_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22447_314 = {S31, 0, &V22447_314_57261, &P22447_312};	/*Property = Html5*/
const Value V22447_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22447_313 = {S32, 0, &V22447_313_57258, &P22447_314};	/*Property = Html-Media-Capture*/
const Value V22447_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22447_315 = {S33, 0, &V22447_315_57264, &P22447_313};	/*Property = Iframe*/
const Value V22447_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22447_316 = {S34, 0, &V22447_316_57267, &P22447_315};	/*Property = IndexedDB*/
const Value V22447_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22447_317 = {S35, 0, &V22447_317_57269, &P22447_316};	/*Property = Json*/
const Value V22447_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22447_15 = {S37, 0, &V22447_15_28, &P22447_317};	/*Property = LayoutEngine*/
const Value V22447_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22447_344 = {S38, 0, &V22447_344_57272, &P22447_15};	/*Property = Masking*/
const Value V22447_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22447_320 = {S39, 0, &V22447_320_57290, &P22447_344};	/*Property = PostMessage*/
const Value V22447_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22447_321 = {S40, 0, &V22447_321_57293, &P22447_320};	/*Property = Progress*/
const Value V22447_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22447_322 = {S41, 0, &V22447_322_57296, &P22447_321};	/*Property = Prompts*/
const Value V22447_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22447_323 = {S42, 0, &V22447_323_57299, &P22447_322};	/*Property = Selector*/
const Value V22447_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22447_324 = {S43, 0, &V22447_324_57306, &P22447_323};	/*Property = Svg*/
const Value V22447_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22447_327 = {S44, 0, &V22447_327_57315, &P22447_324};	/*Property = TouchEvents*/
const Value V22447_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22447_328 = {S45, 0, &V22447_328_57318, &P22447_327};	/*Property = Track*/
const Value V22447_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22447_329 = {S46, 0, &V22447_329_57323, &P22447_328};	/*Property = Video*/
const Value V22447_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22447_330 = {S47, 0, &V22447_330_57326, &P22447_329};	/*Property = Viewport*/
const Value V22447_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22447_332 = {S48, 0, &V22447_332_57339, &P22447_330};	/*Property = WebWorkers*/
const Value V22447_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22447_333 = {S49, 0, &V22447_333_57342, &P22447_332};	/*Property = Xhr2*/
const Profile P22447 = {"22447", &P22447_333};	/*Profile ID - 22447*/
const Value V17391_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17391_279 = {S1, 0, &V17391_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17391_282_57153 = {S2, NULL};	/*Value = True*/
const Property P17391_282 = {S3, 0, &V17391_282_57153, &P17391_279};	/*Property = BlobBuilder*/
const Value V17391_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17391_350 = {S4, 0, &V17391_350_57444, &P17391_282};	/*Property = Canvas*/
const Value V17391_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17391_283 = {S5, 0, &V17391_283_57159, &P17391_350};	/*Property = CssBackground*/
const Value V17391_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17391_284 = {S6, 0, &V17391_284_57163, &P17391_283};	/*Property = CssBorderImage*/
const Value V17391_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17391_285 = {S7, 0, &V17391_285_57166, &P17391_284};	/*Property = CssCanvas*/
const Value V17391_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17391_286 = {S8, 0, &V17391_286_57168, &P17391_285};	/*Property = CssColor*/
const Value V17391_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17391_237 = {S9, 0, &V17391_237_55784, &P17391_286};	/*Property = CssColumn*/
const Value V17391_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17391_287 = {S10, 0, &V17391_287_57172, &P17391_237};	/*Property = CssFlexbox*/
const Value V17391_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17391_288 = {S11, 0, &V17391_288_57175, &P17391_287};	/*Property = CssFont*/
const Value V17391_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17391_289 = {S12, 0, &V17391_289_57759, &P17391_288};	/*Property = CssImages*/
const Value V17391_291_57180 = {S2, NULL};	/*Value = True*/
const Property P17391_291 = {S13, 0, &V17391_291_57180, &P17391_289};	/*Property = CssMediaQueries*/
const Value V17391_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17391_292 = {S14, 0, &V17391_292_57183, &P17391_291};	/*Property = CssMinMax*/
const Value V17391_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17391_293 = {S15, 0, &V17391_293_57187, &P17391_292};	/*Property = CssOverflow*/
const Value V17391_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17391_294 = {S16, 0, &V17391_294_57189, &P17391_293};	/*Property = CssPosition*/
const Value V17391_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17391_295 = {S17, 0, &V17391_295_57192, &P17391_294};	/*Property = CssText*/
const Value V17391_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17391_297 = {S18, 0, &V17391_297_57195, &P17391_295};	/*Property = CssTransforms*/
const Value V17391_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17391_241 = {S19, 0, &V17391_241_55799, &P17391_297};	/*Property = CssTransitions*/
const Value V17391_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17391_298 = {S20, 0, &V17391_298_57206, &P17391_241};	/*Property = CssUI*/
const Value V17391_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17391_300 = {S21, 0, &V17391_300_57216, &P17391_298};	/*Property = DataSet*/
const Value V17391_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17391_301 = {S22, 0, &V17391_301_57218, &P17391_300};	/*Property = DataUrl*/
const Value V17391_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17391_302 = {S23, 0, &V17391_302_57222, &P17391_301};	/*Property = DeviceOrientation*/
const Value V17391_304_57227 = {S2, NULL};	/*Value = True*/
const Property P17391_304 = {S24, 0, &V17391_304_57227, &P17391_302};	/*Property = FileReader*/
const Value V17391_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17391_305 = {S25, 0, &V17391_305_57231, &P17391_304};	/*Property = FileSaver*/
const Value V17391_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17391_306 = {S26, 0, &V17391_306_57237, &P17391_305};	/*Property = FileWriter*/
const Value V17391_307_57239 = {S2, NULL};	/*Value = True*/
const Property P17391_307 = {S27, 0, &V17391_307_57239, &P17391_306};	/*Property = FormData*/
const Value V17391_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17391_309 = {S28, 0, &V17391_309_57246, &P17391_307};	/*Property = Fullscreen*/
const Value V17391_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17391_310 = {S29, 0, &V17391_310_57248, &P17391_309};	/*Property = GeoLocation*/
const Value V17391_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17391_312 = {S30, 0, &V17391_312_57255, &P17391_310};	/*Property = History*/
const Value V17391_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17391_314 = {S31, 0, &V17391_314_57261, &P17391_312};	/*Property = Html5*/
const Value V17391_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17391_313 = {S32, 0, &V17391_313_57258, &P17391_314};	/*Property = Html-Media-Capture*/
const Value V17391_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17391_315 = {S33, 0, &V17391_315_57264, &P17391_313};	/*Property = Iframe*/
const Value V17391_316_57266 = {S2, NULL};	/*Value = True*/
const Property P17391_316 = {S34, 0, &V17391_316_57266, &P17391_315};	/*Property = IndexedDB*/
const Value V17391_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17391_317 = {S35, 0, &V17391_317_57269, &P17391_316};	/*Property = Json*/
const Value V17391_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17391_15 = {S37, 0, &V17391_15_28, &P17391_317};	/*Property = LayoutEngine*/
const Value V17391_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17391_344 = {S38, 0, &V17391_344_57272, &P17391_15};	/*Property = Masking*/
const Value V17391_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17391_320 = {S39, 0, &V17391_320_57290, &P17391_344};	/*Property = PostMessage*/
const Value V17391_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17391_321 = {S40, 0, &V17391_321_57293, &P17391_320};	/*Property = Progress*/
const Value V17391_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17391_322 = {S41, 0, &V17391_322_57296, &P17391_321};	/*Property = Prompts*/
const Value V17391_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17391_323 = {S42, 0, &V17391_323_57299, &P17391_322};	/*Property = Selector*/
const Value V17391_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17391_324 = {S43, 0, &V17391_324_57306, &P17391_323};	/*Property = Svg*/
const Value V17391_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17391_327 = {S44, 0, &V17391_327_57315, &P17391_324};	/*Property = TouchEvents*/
const Value V17391_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17391_328 = {S45, 0, &V17391_328_57318, &P17391_327};	/*Property = Track*/
const Value V17391_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17391_329 = {S46, 0, &V17391_329_57323, &P17391_328};	/*Property = Video*/
const Value V17391_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17391_330 = {S47, 0, &V17391_330_57326, &P17391_329};	/*Property = Viewport*/
const Value V17391_332_57338 = {S2, NULL};	/*Value = True*/
const Property P17391_332 = {S48, 0, &V17391_332_57338, &P17391_330};	/*Property = WebWorkers*/
const Value V17391_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17391_333 = {S49, 0, &V17391_333_57342, &P17391_332};	/*Property = Xhr2*/
const Profile P17391 = {"17391", &P17391_333};	/*Profile ID - 17391*/
const Value V17365_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_279 = {S1, 0, &V17365_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17365_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_282 = {S3, 0, &V17365_282_57155, &P17365_279};	/*Property = BlobBuilder*/
const Value V17365_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_350 = {S4, 0, &V17365_350_57446, &P17365_282};	/*Property = Canvas*/
const Value V17365_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_283 = {S5, 0, &V17365_283_57161, &P17365_350};	/*Property = CssBackground*/
const Value V17365_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_284 = {S6, 0, &V17365_284_57164, &P17365_283};	/*Property = CssBorderImage*/
const Value V17365_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_285 = {S7, 0, &V17365_285_57167, &P17365_284};	/*Property = CssCanvas*/
const Value V17365_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_286 = {S8, 0, &V17365_286_57170, &P17365_285};	/*Property = CssColor*/
const Value V17365_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_237 = {S9, 0, &V17365_237_55786, &P17365_286};	/*Property = CssColumn*/
const Value V17365_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_287 = {S10, 0, &V17365_287_57173, &P17365_237};	/*Property = CssFlexbox*/
const Value V17365_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_288 = {S11, 0, &V17365_288_57176, &P17365_287};	/*Property = CssFont*/
const Value V17365_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_289 = {S12, 0, &V17365_289_86899, &P17365_288};	/*Property = CssImages*/
const Value V17365_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_291 = {S13, 0, &V17365_291_57182, &P17365_289};	/*Property = CssMediaQueries*/
const Value V17365_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_292 = {S14, 0, &V17365_292_57185, &P17365_291};	/*Property = CssMinMax*/
const Value V17365_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_293 = {S15, 0, &V17365_293_57188, &P17365_292};	/*Property = CssOverflow*/
const Value V17365_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_294 = {S16, 0, &V17365_294_57191, &P17365_293};	/*Property = CssPosition*/
const Value V17365_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_295 = {S17, 0, &V17365_295_57194, &P17365_294};	/*Property = CssText*/
const Value V17365_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_297 = {S18, 0, &V17365_297_57197, &P17365_295};	/*Property = CssTransforms*/
const Value V17365_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_241 = {S19, 0, &V17365_241_55801, &P17365_297};	/*Property = CssTransitions*/
const Value V17365_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_298 = {S20, 0, &V17365_298_57208, &P17365_241};	/*Property = CssUI*/
const Value V17365_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_300 = {S21, 0, &V17365_300_57217, &P17365_298};	/*Property = DataSet*/
const Value V17365_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_301 = {S22, 0, &V17365_301_57220, &P17365_300};	/*Property = DataUrl*/
const Value V17365_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_302 = {S23, 0, &V17365_302_57223, &P17365_301};	/*Property = DeviceOrientation*/
const Value V17365_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_304 = {S24, 0, &V17365_304_57229, &P17365_302};	/*Property = FileReader*/
const Value V17365_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_305 = {S25, 0, &V17365_305_57232, &P17365_304};	/*Property = FileSaver*/
const Value V17365_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_306 = {S26, 0, &V17365_306_57238, &P17365_305};	/*Property = FileWriter*/
const Value V17365_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_307 = {S27, 0, &V17365_307_57241, &P17365_306};	/*Property = FormData*/
const Value V17365_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_309 = {S28, 0, &V17365_309_57247, &P17365_307};	/*Property = Fullscreen*/
const Value V17365_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_310 = {S29, 0, &V17365_310_57250, &P17365_309};	/*Property = GeoLocation*/
const Value V17365_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_312 = {S30, 0, &V17365_312_57256, &P17365_310};	/*Property = History*/
const Value V17365_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_314 = {S31, 0, &V17365_314_57262, &P17365_312};	/*Property = Html5*/
const Value V17365_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_313 = {S32, 0, &V17365_313_57259, &P17365_314};	/*Property = Html-Media-Capture*/
const Value V17365_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_315 = {S33, 0, &V17365_315_57265, &P17365_313};	/*Property = Iframe*/
const Value V17365_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_316 = {S34, 0, &V17365_316_57268, &P17365_315};	/*Property = IndexedDB*/
const Value V17365_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_317 = {S35, 0, &V17365_317_57271, &P17365_316};	/*Property = Json*/
const Value V17365_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P17365_15 = {S37, 0, &V17365_15_31, &P17365_317};	/*Property = LayoutEngine*/
const Value V17365_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_344 = {S38, 0, &V17365_344_57274, &P17365_15};	/*Property = Masking*/
const Value V17365_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_320 = {S39, 0, &V17365_320_57292, &P17365_344};	/*Property = PostMessage*/
const Value V17365_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_321 = {S40, 0, &V17365_321_57295, &P17365_320};	/*Property = Progress*/
const Value V17365_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_322 = {S41, 0, &V17365_322_57298, &P17365_321};	/*Property = Prompts*/
const Value V17365_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_323 = {S42, 0, &V17365_323_57301, &P17365_322};	/*Property = Selector*/
const Value V17365_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_324 = {S43, 0, &V17365_324_57307, &P17365_323};	/*Property = Svg*/
const Value V17365_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_327 = {S44, 0, &V17365_327_57316, &P17365_324};	/*Property = TouchEvents*/
const Value V17365_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_328 = {S45, 0, &V17365_328_57319, &P17365_327};	/*Property = Track*/
const Value V17365_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_329 = {S46, 0, &V17365_329_57325, &P17365_328};	/*Property = Video*/
const Value V17365_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_330 = {S47, 0, &V17365_330_57328, &P17365_329};	/*Property = Viewport*/
const Value V17365_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_332 = {S48, 0, &V17365_332_57340, &P17365_330};	/*Property = WebWorkers*/
const Value V17365_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17365_333 = {S49, 0, &V17365_333_57343, &P17365_332};	/*Property = Xhr2*/
const Profile P17365 = {"17365", &P17365_333};	/*Profile ID - 17365*/
const Value V16561_1_1 = {S2, NULL};	/*Value = True*/
const Property P16561_1 = {S50, 0, &V16561_1_1, NULL};	/*Property = IsMobile*/
const Value V16561_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16561_5 = {S52, 0, &V16561_5_1277, &P16561_1};	/*Property = ScreenPixelsHeight*/
const Value V16561_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16561_4 = {S54, 0, &V16561_4_1279, &P16561_5};	/*Property = ScreenPixelsWidth*/
const Profile P16561 = {"16561", &P16561_4};	/*Profile ID - 16561*/
const Value V23729_279_57148 = {S0, NULL};	/*Value = False*/
const Property P23729_279 = {S1, 0, &V23729_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V23729_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23729_282 = {S3, 0, &V23729_282_57154, &P23729_279};	/*Property = BlobBuilder*/
const Value V23729_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23729_350 = {S4, 0, &V23729_350_57444, &P23729_282};	/*Property = Canvas*/
const Value V23729_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23729_283 = {S5, 0, &V23729_283_57159, &P23729_350};	/*Property = CssBackground*/
const Value V23729_284_57163 = {S0, NULL};	/*Value = False*/
const Property P23729_284 = {S6, 0, &V23729_284_57163, &P23729_283};	/*Property = CssBorderImage*/
const Value V23729_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23729_285 = {S7, 0, &V23729_285_57166, &P23729_284};	/*Property = CssCanvas*/
const Value V23729_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23729_286 = {S8, 0, &V23729_286_57168, &P23729_285};	/*Property = CssColor*/
const Value V23729_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23729_237 = {S9, 0, &V23729_237_55786, &P23729_286};	/*Property = CssColumn*/
const Value V23729_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23729_287 = {S10, 0, &V23729_287_57172, &P23729_237};	/*Property = CssFlexbox*/
const Value V23729_288_57175 = {S0, NULL};	/*Value = False*/
const Property P23729_288 = {S11, 0, &V23729_288_57175, &P23729_287};	/*Property = CssFont*/
const Value V23729_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23729_289 = {S12, 0, &V23729_289_57759, &P23729_288};	/*Property = CssImages*/
const Value V23729_291_57181 = {S0, NULL};	/*Value = False*/
const Property P23729_291 = {S13, 0, &V23729_291_57181, &P23729_289};	/*Property = CssMediaQueries*/
const Value V23729_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23729_292 = {S14, 0, &V23729_292_57183, &P23729_291};	/*Property = CssMinMax*/
const Value V23729_293_57187 = {S0, NULL};	/*Value = False*/
const Property P23729_293 = {S15, 0, &V23729_293_57187, &P23729_292};	/*Property = CssOverflow*/
const Value V23729_294_57190 = {S0, NULL};	/*Value = False*/
const Property P23729_294 = {S16, 0, &V23729_294_57190, &P23729_293};	/*Property = CssPosition*/
const Value V23729_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23729_295 = {S17, 0, &V23729_295_57192, &P23729_294};	/*Property = CssText*/
const Value V23729_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23729_297 = {S18, 0, &V23729_297_57195, &P23729_295};	/*Property = CssTransforms*/
const Value V23729_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23729_241 = {S19, 0, &V23729_241_55799, &P23729_297};	/*Property = CssTransitions*/
const Value V23729_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23729_298 = {S20, 0, &V23729_298_57206, &P23729_241};	/*Property = CssUI*/
const Value V23729_300_57216 = {S0, NULL};	/*Value = False*/
const Property P23729_300 = {S21, 0, &V23729_300_57216, &P23729_298};	/*Property = DataSet*/
const Value V23729_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23729_301 = {S22, 0, &V23729_301_57218, &P23729_300};	/*Property = DataUrl*/
const Value V23729_302_57222 = {S0, NULL};	/*Value = False*/
const Property P23729_302 = {S23, 0, &V23729_302_57222, &P23729_301};	/*Property = DeviceOrientation*/
const Value V23729_304_57228 = {S0, NULL};	/*Value = False*/
const Property P23729_304 = {S24, 0, &V23729_304_57228, &P23729_302};	/*Property = FileReader*/
const Value V23729_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23729_305 = {S25, 0, &V23729_305_57231, &P23729_304};	/*Property = FileSaver*/
const Value V23729_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23729_306 = {S26, 0, &V23729_306_57237, &P23729_305};	/*Property = FileWriter*/
const Value V23729_307_57240 = {S0, NULL};	/*Value = False*/
const Property P23729_307 = {S27, 0, &V23729_307_57240, &P23729_306};	/*Property = FormData*/
const Value V23729_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23729_309 = {S28, 0, &V23729_309_57246, &P23729_307};	/*Property = Fullscreen*/
const Value V23729_310_57249 = {S0, NULL};	/*Value = False*/
const Property P23729_310 = {S29, 0, &V23729_310_57249, &P23729_309};	/*Property = GeoLocation*/
const Value V23729_312_57255 = {S0, NULL};	/*Value = False*/
const Property P23729_312 = {S30, 0, &V23729_312_57255, &P23729_310};	/*Property = History*/
const Value V23729_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23729_314 = {S31, 0, &V23729_314_57261, &P23729_312};	/*Property = Html5*/
const Value V23729_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23729_313 = {S32, 0, &V23729_313_57258, &P23729_314};	/*Property = Html-Media-Capture*/
const Value V23729_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23729_315 = {S33, 0, &V23729_315_57264, &P23729_313};	/*Property = Iframe*/
const Value V23729_316_57267 = {S0, NULL};	/*Value = False*/
const Property P23729_316 = {S34, 0, &V23729_316_57267, &P23729_315};	/*Property = IndexedDB*/
const Value V23729_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23729_317 = {S35, 0, &V23729_317_57269, &P23729_316};	/*Property = Json*/
const Value V23729_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P23729_15 = {S37, 0, &V23729_15_28, &P23729_317};	/*Property = LayoutEngine*/
const Value V23729_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23729_344 = {S38, 0, &V23729_344_57272, &P23729_15};	/*Property = Masking*/
const Value V23729_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23729_320 = {S39, 0, &V23729_320_57290, &P23729_344};	/*Property = PostMessage*/
const Value V23729_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23729_321 = {S40, 0, &V23729_321_57293, &P23729_320};	/*Property = Progress*/
const Value V23729_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23729_322 = {S41, 0, &V23729_322_57296, &P23729_321};	/*Property = Prompts*/
const Value V23729_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23729_323 = {S42, 0, &V23729_323_57299, &P23729_322};	/*Property = Selector*/
const Value V23729_324_57306 = {S0, NULL};	/*Value = False*/
const Property P23729_324 = {S43, 0, &V23729_324_57306, &P23729_323};	/*Property = Svg*/
const Value V23729_327_57315 = {S0, NULL};	/*Value = False*/
const Property P23729_327 = {S44, 0, &V23729_327_57315, &P23729_324};	/*Property = TouchEvents*/
const Value V23729_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23729_328 = {S45, 0, &V23729_328_57318, &P23729_327};	/*Property = Track*/
const Value V23729_329_57324 = {S0, NULL};	/*Value = False*/
const Property P23729_329 = {S46, 0, &V23729_329_57324, &P23729_328};	/*Property = Video*/
const Value V23729_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23729_330 = {S47, 0, &V23729_330_57326, &P23729_329};	/*Property = Viewport*/
const Value V23729_332_57339 = {S0, NULL};	/*Value = False*/
const Property P23729_332 = {S48, 0, &V23729_332_57339, &P23729_330};	/*Property = WebWorkers*/
const Value V23729_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23729_333 = {S49, 0, &V23729_333_57342, &P23729_332};	/*Property = Xhr2*/
const Profile P23729 = {"23729", &P23729_333};	/*Profile ID - 23729*/
const Value V17574_1_1 = {S2, NULL};	/*Value = True*/
const Property P17574_1 = {S50, 0, &V17574_1_1, NULL};	/*Property = IsMobile*/
const Value V17574_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17574_5 = {S52, 0, &V17574_5_1269, &P17574_1};	/*Property = ScreenPixelsHeight*/
const Value V17574_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17574_4 = {S54, 0, &V17574_4_1229, &P17574_5};	/*Property = ScreenPixelsWidth*/
const Profile P17574 = {"17574", &P17574_4};	/*Profile ID - 17574*/
const Value V18082_1_1 = {S2, NULL};	/*Value = True*/
const Property P18082_1 = {S50, 0, &V18082_1_1, NULL};	/*Property = IsMobile*/
const Value V18082_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18082_5 = {S52, 0, &V18082_5_1227, &P18082_1};	/*Property = ScreenPixelsHeight*/
const Value V18082_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18082_4 = {S54, 0, &V18082_4_1229, &P18082_5};	/*Property = ScreenPixelsWidth*/
const Profile P18082 = {"18082", &P18082_4};	/*Profile ID - 18082*/
const Value V18044_1_1 = {S2, NULL};	/*Value = True*/
const Property P18044_1 = {S50, 0, &V18044_1_1, NULL};	/*Property = IsMobile*/
const Value V18044_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18044_5 = {S52, 0, &V18044_5_1222, &P18044_1};	/*Property = ScreenPixelsHeight*/
const Value V18044_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18044_4 = {S54, 0, &V18044_4_1224, &P18044_5};	/*Property = ScreenPixelsWidth*/
const Profile P18044 = {"18044", &P18044_4};	/*Profile ID - 18044*/
const Value V16534_1_1 = {S2, NULL};	/*Value = True*/
const Property P16534_1 = {S50, 0, &V16534_1_1, NULL};	/*Property = IsMobile*/
const Value V16534_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16534_5 = {S52, 0, &V16534_5_1227, &P16534_1};	/*Property = ScreenPixelsHeight*/
const Value V16534_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16534_4 = {S54, 0, &V16534_4_1229, &P16534_5};	/*Property = ScreenPixelsWidth*/
const Profile P16534 = {"16534", &P16534_4};	/*Profile ID - 16534*/
const Value V21925_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21925_279 = {S1, 0, &V21925_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21925_282_57153 = {S2, NULL};	/*Value = True*/
const Property P21925_282 = {S3, 0, &V21925_282_57153, &P21925_279};	/*Property = BlobBuilder*/
const Value V21925_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21925_350 = {S4, 0, &V21925_350_57444, &P21925_282};	/*Property = Canvas*/
const Value V21925_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21925_283 = {S5, 0, &V21925_283_57159, &P21925_350};	/*Property = CssBackground*/
const Value V21925_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21925_284 = {S6, 0, &V21925_284_57163, &P21925_283};	/*Property = CssBorderImage*/
const Value V21925_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21925_285 = {S7, 0, &V21925_285_57166, &P21925_284};	/*Property = CssCanvas*/
const Value V21925_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21925_286 = {S8, 0, &V21925_286_57168, &P21925_285};	/*Property = CssColor*/
const Value V21925_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21925_237 = {S9, 0, &V21925_237_55786, &P21925_286};	/*Property = CssColumn*/
const Value V21925_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21925_287 = {S10, 0, &V21925_287_57172, &P21925_237};	/*Property = CssFlexbox*/
const Value V21925_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21925_288 = {S11, 0, &V21925_288_57175, &P21925_287};	/*Property = CssFont*/
const Value V21925_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21925_289 = {S12, 0, &V21925_289_57759, &P21925_288};	/*Property = CssImages*/
const Value V21925_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21925_291 = {S13, 0, &V21925_291_57180, &P21925_289};	/*Property = CssMediaQueries*/
const Value V21925_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21925_292 = {S14, 0, &V21925_292_57183, &P21925_291};	/*Property = CssMinMax*/
const Value V21925_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21925_293 = {S15, 0, &V21925_293_57187, &P21925_292};	/*Property = CssOverflow*/
const Value V21925_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21925_294 = {S16, 0, &V21925_294_57189, &P21925_293};	/*Property = CssPosition*/
const Value V21925_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21925_295 = {S17, 0, &V21925_295_57192, &P21925_294};	/*Property = CssText*/
const Value V21925_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21925_297 = {S18, 0, &V21925_297_57195, &P21925_295};	/*Property = CssTransforms*/
const Value V21925_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21925_241 = {S19, 0, &V21925_241_55799, &P21925_297};	/*Property = CssTransitions*/
const Value V21925_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21925_298 = {S20, 0, &V21925_298_57206, &P21925_241};	/*Property = CssUI*/
const Value V21925_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21925_300 = {S21, 0, &V21925_300_57216, &P21925_298};	/*Property = DataSet*/
const Value V21925_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21925_301 = {S22, 0, &V21925_301_57218, &P21925_300};	/*Property = DataUrl*/
const Value V21925_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21925_302 = {S23, 0, &V21925_302_57222, &P21925_301};	/*Property = DeviceOrientation*/
const Value V21925_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21925_304 = {S24, 0, &V21925_304_57227, &P21925_302};	/*Property = FileReader*/
const Value V21925_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21925_305 = {S25, 0, &V21925_305_57231, &P21925_304};	/*Property = FileSaver*/
const Value V21925_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21925_306 = {S26, 0, &V21925_306_57237, &P21925_305};	/*Property = FileWriter*/
const Value V21925_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21925_307 = {S27, 0, &V21925_307_57239, &P21925_306};	/*Property = FormData*/
const Value V21925_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21925_309 = {S28, 0, &V21925_309_57246, &P21925_307};	/*Property = Fullscreen*/
const Value V21925_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21925_310 = {S29, 0, &V21925_310_57248, &P21925_309};	/*Property = GeoLocation*/
const Value V21925_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21925_312 = {S30, 0, &V21925_312_57255, &P21925_310};	/*Property = History*/
const Value V21925_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21925_314 = {S31, 0, &V21925_314_57261, &P21925_312};	/*Property = Html5*/
const Value V21925_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21925_313 = {S32, 0, &V21925_313_57258, &P21925_314};	/*Property = Html-Media-Capture*/
const Value V21925_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21925_315 = {S33, 0, &V21925_315_57264, &P21925_313};	/*Property = Iframe*/
const Value V21925_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21925_316 = {S34, 0, &V21925_316_57267, &P21925_315};	/*Property = IndexedDB*/
const Value V21925_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21925_317 = {S35, 0, &V21925_317_57269, &P21925_316};	/*Property = Json*/
const Value V21925_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21925_15 = {S37, 0, &V21925_15_28, &P21925_317};	/*Property = LayoutEngine*/
const Value V21925_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21925_344 = {S38, 0, &V21925_344_57272, &P21925_15};	/*Property = Masking*/
const Value V21925_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21925_320 = {S39, 0, &V21925_320_57290, &P21925_344};	/*Property = PostMessage*/
const Value V21925_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21925_321 = {S40, 0, &V21925_321_57293, &P21925_320};	/*Property = Progress*/
const Value V21925_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21925_322 = {S41, 0, &V21925_322_57296, &P21925_321};	/*Property = Prompts*/
const Value V21925_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21925_323 = {S42, 0, &V21925_323_57299, &P21925_322};	/*Property = Selector*/
const Value V21925_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21925_324 = {S43, 0, &V21925_324_57306, &P21925_323};	/*Property = Svg*/
const Value V21925_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21925_327 = {S44, 0, &V21925_327_57315, &P21925_324};	/*Property = TouchEvents*/
const Value V21925_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21925_328 = {S45, 0, &V21925_328_57318, &P21925_327};	/*Property = Track*/
const Value V21925_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21925_329 = {S46, 0, &V21925_329_57323, &P21925_328};	/*Property = Video*/
const Value V21925_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21925_330 = {S47, 0, &V21925_330_57326, &P21925_329};	/*Property = Viewport*/
const Value V21925_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21925_332 = {S48, 0, &V21925_332_57339, &P21925_330};	/*Property = WebWorkers*/
const Value V21925_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21925_333 = {S49, 0, &V21925_333_57342, &P21925_332};	/*Property = Xhr2*/
const Profile P21925 = {"21925", &P21925_333};	/*Profile ID - 21925*/
const Value V22401_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22401_279 = {S1, 0, &V22401_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22401_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22401_282 = {S3, 0, &V22401_282_57153, &P22401_279};	/*Property = BlobBuilder*/
const Value V22401_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22401_350 = {S4, 0, &V22401_350_57444, &P22401_282};	/*Property = Canvas*/
const Value V22401_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22401_283 = {S5, 0, &V22401_283_57159, &P22401_350};	/*Property = CssBackground*/
const Value V22401_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22401_284 = {S6, 0, &V22401_284_57163, &P22401_283};	/*Property = CssBorderImage*/
const Value V22401_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22401_285 = {S7, 0, &V22401_285_57166, &P22401_284};	/*Property = CssCanvas*/
const Value V22401_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22401_286 = {S8, 0, &V22401_286_57168, &P22401_285};	/*Property = CssColor*/
const Value V22401_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22401_237 = {S9, 0, &V22401_237_55786, &P22401_286};	/*Property = CssColumn*/
const Value V22401_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22401_287 = {S10, 0, &V22401_287_57172, &P22401_237};	/*Property = CssFlexbox*/
const Value V22401_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22401_288 = {S11, 0, &V22401_288_57175, &P22401_287};	/*Property = CssFont*/
const Value V22401_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22401_289 = {S12, 0, &V22401_289_57759, &P22401_288};	/*Property = CssImages*/
const Value V22401_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22401_291 = {S13, 0, &V22401_291_57180, &P22401_289};	/*Property = CssMediaQueries*/
const Value V22401_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22401_292 = {S14, 0, &V22401_292_57183, &P22401_291};	/*Property = CssMinMax*/
const Value V22401_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22401_293 = {S15, 0, &V22401_293_57187, &P22401_292};	/*Property = CssOverflow*/
const Value V22401_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22401_294 = {S16, 0, &V22401_294_57189, &P22401_293};	/*Property = CssPosition*/
const Value V22401_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22401_295 = {S17, 0, &V22401_295_57192, &P22401_294};	/*Property = CssText*/
const Value V22401_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22401_297 = {S18, 0, &V22401_297_57195, &P22401_295};	/*Property = CssTransforms*/
const Value V22401_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22401_241 = {S19, 0, &V22401_241_55799, &P22401_297};	/*Property = CssTransitions*/
const Value V22401_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22401_298 = {S20, 0, &V22401_298_57206, &P22401_241};	/*Property = CssUI*/
const Value V22401_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22401_300 = {S21, 0, &V22401_300_57216, &P22401_298};	/*Property = DataSet*/
const Value V22401_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22401_301 = {S22, 0, &V22401_301_57218, &P22401_300};	/*Property = DataUrl*/
const Value V22401_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22401_302 = {S23, 0, &V22401_302_57222, &P22401_301};	/*Property = DeviceOrientation*/
const Value V22401_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22401_304 = {S24, 0, &V22401_304_57227, &P22401_302};	/*Property = FileReader*/
const Value V22401_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22401_305 = {S25, 0, &V22401_305_57231, &P22401_304};	/*Property = FileSaver*/
const Value V22401_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22401_306 = {S26, 0, &V22401_306_57237, &P22401_305};	/*Property = FileWriter*/
const Value V22401_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22401_307 = {S27, 0, &V22401_307_57239, &P22401_306};	/*Property = FormData*/
const Value V22401_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22401_309 = {S28, 0, &V22401_309_57246, &P22401_307};	/*Property = Fullscreen*/
const Value V22401_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22401_310 = {S29, 0, &V22401_310_57248, &P22401_309};	/*Property = GeoLocation*/
const Value V22401_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22401_312 = {S30, 0, &V22401_312_57255, &P22401_310};	/*Property = History*/
const Value V22401_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22401_314 = {S31, 0, &V22401_314_57261, &P22401_312};	/*Property = Html5*/
const Value V22401_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22401_313 = {S32, 0, &V22401_313_57258, &P22401_314};	/*Property = Html-Media-Capture*/
const Value V22401_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22401_315 = {S33, 0, &V22401_315_57264, &P22401_313};	/*Property = Iframe*/
const Value V22401_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22401_316 = {S34, 0, &V22401_316_57267, &P22401_315};	/*Property = IndexedDB*/
const Value V22401_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22401_317 = {S35, 0, &V22401_317_57269, &P22401_316};	/*Property = Json*/
const Value V22401_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22401_15 = {S37, 0, &V22401_15_28, &P22401_317};	/*Property = LayoutEngine*/
const Value V22401_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22401_344 = {S38, 0, &V22401_344_57272, &P22401_15};	/*Property = Masking*/
const Value V22401_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22401_320 = {S39, 0, &V22401_320_57290, &P22401_344};	/*Property = PostMessage*/
const Value V22401_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22401_321 = {S40, 0, &V22401_321_57293, &P22401_320};	/*Property = Progress*/
const Value V22401_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22401_322 = {S41, 0, &V22401_322_57296, &P22401_321};	/*Property = Prompts*/
const Value V22401_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22401_323 = {S42, 0, &V22401_323_57299, &P22401_322};	/*Property = Selector*/
const Value V22401_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22401_324 = {S43, 0, &V22401_324_57306, &P22401_323};	/*Property = Svg*/
const Value V22401_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22401_327 = {S44, 0, &V22401_327_57315, &P22401_324};	/*Property = TouchEvents*/
const Value V22401_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22401_328 = {S45, 0, &V22401_328_57318, &P22401_327};	/*Property = Track*/
const Value V22401_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22401_329 = {S46, 0, &V22401_329_57323, &P22401_328};	/*Property = Video*/
const Value V22401_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22401_330 = {S47, 0, &V22401_330_57326, &P22401_329};	/*Property = Viewport*/
const Value V22401_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22401_332 = {S48, 0, &V22401_332_57339, &P22401_330};	/*Property = WebWorkers*/
const Value V22401_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22401_333 = {S49, 0, &V22401_333_57342, &P22401_332};	/*Property = Xhr2*/
const Profile P22401 = {"22401", &P22401_333};	/*Profile ID - 22401*/
const Value V21923_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21923_279 = {S1, 0, &V21923_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21923_282_57153 = {S2, NULL};	/*Value = True*/
const Property P21923_282 = {S3, 0, &V21923_282_57153, &P21923_279};	/*Property = BlobBuilder*/
const Value V21923_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21923_350 = {S4, 0, &V21923_350_57444, &P21923_282};	/*Property = Canvas*/
const Value V21923_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21923_283 = {S5, 0, &V21923_283_57159, &P21923_350};	/*Property = CssBackground*/
const Value V21923_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21923_284 = {S6, 0, &V21923_284_57163, &P21923_283};	/*Property = CssBorderImage*/
const Value V21923_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21923_285 = {S7, 0, &V21923_285_57166, &P21923_284};	/*Property = CssCanvas*/
const Value V21923_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21923_286 = {S8, 0, &V21923_286_57168, &P21923_285};	/*Property = CssColor*/
const Value V21923_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21923_237 = {S9, 0, &V21923_237_55784, &P21923_286};	/*Property = CssColumn*/
const Value V21923_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21923_287 = {S10, 0, &V21923_287_57172, &P21923_237};	/*Property = CssFlexbox*/
const Value V21923_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21923_288 = {S11, 0, &V21923_288_57175, &P21923_287};	/*Property = CssFont*/
const Value V21923_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21923_289 = {S12, 0, &V21923_289_57759, &P21923_288};	/*Property = CssImages*/
const Value V21923_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21923_291 = {S13, 0, &V21923_291_57180, &P21923_289};	/*Property = CssMediaQueries*/
const Value V21923_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21923_292 = {S14, 0, &V21923_292_57183, &P21923_291};	/*Property = CssMinMax*/
const Value V21923_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21923_293 = {S15, 0, &V21923_293_57187, &P21923_292};	/*Property = CssOverflow*/
const Value V21923_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21923_294 = {S16, 0, &V21923_294_57189, &P21923_293};	/*Property = CssPosition*/
const Value V21923_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21923_295 = {S17, 0, &V21923_295_57192, &P21923_294};	/*Property = CssText*/
const Value V21923_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21923_297 = {S18, 0, &V21923_297_57195, &P21923_295};	/*Property = CssTransforms*/
const Value V21923_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21923_241 = {S19, 0, &V21923_241_55799, &P21923_297};	/*Property = CssTransitions*/
const Value V21923_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21923_298 = {S20, 0, &V21923_298_57206, &P21923_241};	/*Property = CssUI*/
const Value V21923_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21923_300 = {S21, 0, &V21923_300_57216, &P21923_298};	/*Property = DataSet*/
const Value V21923_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21923_301 = {S22, 0, &V21923_301_57218, &P21923_300};	/*Property = DataUrl*/
const Value V21923_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21923_302 = {S23, 0, &V21923_302_57222, &P21923_301};	/*Property = DeviceOrientation*/
const Value V21923_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21923_304 = {S24, 0, &V21923_304_57227, &P21923_302};	/*Property = FileReader*/
const Value V21923_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21923_305 = {S25, 0, &V21923_305_57231, &P21923_304};	/*Property = FileSaver*/
const Value V21923_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21923_306 = {S26, 0, &V21923_306_57237, &P21923_305};	/*Property = FileWriter*/
const Value V21923_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21923_307 = {S27, 0, &V21923_307_57239, &P21923_306};	/*Property = FormData*/
const Value V21923_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21923_309 = {S28, 0, &V21923_309_57246, &P21923_307};	/*Property = Fullscreen*/
const Value V21923_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21923_310 = {S29, 0, &V21923_310_57248, &P21923_309};	/*Property = GeoLocation*/
const Value V21923_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21923_312 = {S30, 0, &V21923_312_57255, &P21923_310};	/*Property = History*/
const Value V21923_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21923_314 = {S31, 0, &V21923_314_57261, &P21923_312};	/*Property = Html5*/
const Value V21923_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21923_313 = {S32, 0, &V21923_313_57258, &P21923_314};	/*Property = Html-Media-Capture*/
const Value V21923_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21923_315 = {S33, 0, &V21923_315_57264, &P21923_313};	/*Property = Iframe*/
const Value V21923_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21923_316 = {S34, 0, &V21923_316_57267, &P21923_315};	/*Property = IndexedDB*/
const Value V21923_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21923_317 = {S35, 0, &V21923_317_57269, &P21923_316};	/*Property = Json*/
const Value V21923_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21923_15 = {S37, 0, &V21923_15_28, &P21923_317};	/*Property = LayoutEngine*/
const Value V21923_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21923_344 = {S38, 0, &V21923_344_57272, &P21923_15};	/*Property = Masking*/
const Value V21923_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21923_320 = {S39, 0, &V21923_320_57290, &P21923_344};	/*Property = PostMessage*/
const Value V21923_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21923_321 = {S40, 0, &V21923_321_57293, &P21923_320};	/*Property = Progress*/
const Value V21923_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21923_322 = {S41, 0, &V21923_322_57296, &P21923_321};	/*Property = Prompts*/
const Value V21923_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21923_323 = {S42, 0, &V21923_323_57299, &P21923_322};	/*Property = Selector*/
const Value V21923_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21923_324 = {S43, 0, &V21923_324_57306, &P21923_323};	/*Property = Svg*/
const Value V21923_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21923_327 = {S44, 0, &V21923_327_57315, &P21923_324};	/*Property = TouchEvents*/
const Value V21923_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21923_328 = {S45, 0, &V21923_328_57318, &P21923_327};	/*Property = Track*/
const Value V21923_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21923_329 = {S46, 0, &V21923_329_57323, &P21923_328};	/*Property = Video*/
const Value V21923_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21923_330 = {S47, 0, &V21923_330_57326, &P21923_329};	/*Property = Viewport*/
const Value V21923_332_57338 = {S2, NULL};	/*Value = True*/
const Property P21923_332 = {S48, 0, &V21923_332_57338, &P21923_330};	/*Property = WebWorkers*/
const Value V21923_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21923_333 = {S49, 0, &V21923_333_57342, &P21923_332};	/*Property = Xhr2*/
const Profile P21923 = {"21923", &P21923_333};	/*Profile ID - 21923*/
const Value V17149_1_1 = {S2, NULL};	/*Value = True*/
const Property P17149_1 = {S50, 0, &V17149_1_1, NULL};	/*Property = IsMobile*/
const Value V17149_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17149_5 = {S52, 0, &V17149_5_1227, &P17149_1};	/*Property = ScreenPixelsHeight*/
const Value V17149_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17149_4 = {S54, 0, &V17149_4_1229, &P17149_5};	/*Property = ScreenPixelsWidth*/
const Profile P17149 = {"17149", &P17149_4};	/*Profile ID - 17149*/
const Value V18619_1_1 = {S2, NULL};	/*Value = True*/
const Property P18619_1 = {S50, 0, &V18619_1_1, NULL};	/*Property = IsMobile*/
const Value V18619_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18619_5 = {S52, 0, &V18619_5_1227, &P18619_1};	/*Property = ScreenPixelsHeight*/
const Value V18619_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18619_4 = {S54, 0, &V18619_4_1229, &P18619_5};	/*Property = ScreenPixelsWidth*/
const Profile P18619 = {"18619", &P18619_4};	/*Profile ID - 18619*/
const Value V17669_1_1 = {S2, NULL};	/*Value = True*/
const Property P17669_1 = {S50, 0, &V17669_1_1, NULL};	/*Property = IsMobile*/
const Value V17669_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17669_5 = {S52, 0, &V17669_5_1227, &P17669_1};	/*Property = ScreenPixelsHeight*/
const Value V17669_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17669_4 = {S54, 0, &V17669_4_1229, &P17669_5};	/*Property = ScreenPixelsWidth*/
const Profile P17669 = {"17669", &P17669_4};	/*Profile ID - 17669*/
const Value V16588_1_1 = {S2, NULL};	/*Value = True*/
const Property P16588_1 = {S50, 0, &V16588_1_1, NULL};	/*Property = IsMobile*/
const Value V16588_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16588_5 = {S52, 0, &V16588_5_1293, &P16588_1};	/*Property = ScreenPixelsHeight*/
const Value V16588_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16588_4 = {S54, 0, &V16588_4_1296, &P16588_5};	/*Property = ScreenPixelsWidth*/
const Profile P16588 = {"16588", &P16588_4};	/*Profile ID - 16588*/
const Value V16555_1_1 = {S2, NULL};	/*Value = True*/
const Property P16555_1 = {S50, 0, &V16555_1_1, NULL};	/*Property = IsMobile*/
const Value V16555_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16555_5 = {S52, 0, &V16555_5_1222, &P16555_1};	/*Property = ScreenPixelsHeight*/
const Value V16555_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16555_4 = {S54, 0, &V16555_4_1224, &P16555_5};	/*Property = ScreenPixelsWidth*/
const Profile P16555 = {"16555", &P16555_4};	/*Profile ID - 16555*/
const Value V18625_1_2 = {S0, NULL};	/*Value = False*/
const Property P18625_1 = {S50, 0, &V18625_1_2, NULL};	/*Property = IsMobile*/
const Value V18625_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P18625_5 = {S52, 0, &V18625_5_37, &P18625_1};	/*Property = ScreenPixelsHeight*/
const Value V18625_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P18625_4 = {S54, 0, &V18625_4_36, &P18625_5};	/*Property = ScreenPixelsWidth*/
const Profile P18625 = {"18625", &P18625_4};	/*Profile ID - 18625*/
const Profile P18576 = {"18576", NULL};	/*Profile ID - 18576*/
const Value V17201_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17201_279 = {S1, 0, &V17201_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17201_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17201_282 = {S3, 0, &V17201_282_57154, &P17201_279};	/*Property = BlobBuilder*/
const Value V17201_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17201_350 = {S4, 0, &V17201_350_57444, &P17201_282};	/*Property = Canvas*/
const Value V17201_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17201_283 = {S5, 0, &V17201_283_57159, &P17201_350};	/*Property = CssBackground*/
const Value V17201_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17201_284 = {S6, 0, &V17201_284_57163, &P17201_283};	/*Property = CssBorderImage*/
const Value V17201_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17201_285 = {S7, 0, &V17201_285_57166, &P17201_284};	/*Property = CssCanvas*/
const Value V17201_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17201_286 = {S8, 0, &V17201_286_57168, &P17201_285};	/*Property = CssColor*/
const Value V17201_237_55785 = {S0, NULL};	/*Value = False*/
const Property P17201_237 = {S9, 0, &V17201_237_55785, &P17201_286};	/*Property = CssColumn*/
const Value V17201_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17201_287 = {S10, 0, &V17201_287_57172, &P17201_237};	/*Property = CssFlexbox*/
const Value V17201_288_57174 = {S2, NULL};	/*Value = True*/
const Property P17201_288 = {S11, 0, &V17201_288_57174, &P17201_287};	/*Property = CssFont*/
const Value V17201_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17201_289 = {S12, 0, &V17201_289_57759, &P17201_288};	/*Property = CssImages*/
const Value V17201_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17201_291 = {S13, 0, &V17201_291_57181, &P17201_289};	/*Property = CssMediaQueries*/
const Value V17201_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17201_292 = {S14, 0, &V17201_292_57183, &P17201_291};	/*Property = CssMinMax*/
const Value V17201_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17201_293 = {S15, 0, &V17201_293_57187, &P17201_292};	/*Property = CssOverflow*/
const Value V17201_294_57189 = {S2, NULL};	/*Value = True*/
const Property P17201_294 = {S16, 0, &V17201_294_57189, &P17201_293};	/*Property = CssPosition*/
const Value V17201_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17201_295 = {S17, 0, &V17201_295_57192, &P17201_294};	/*Property = CssText*/
const Value V17201_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17201_297 = {S18, 0, &V17201_297_57195, &P17201_295};	/*Property = CssTransforms*/
const Value V17201_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17201_241 = {S19, 0, &V17201_241_55799, &P17201_297};	/*Property = CssTransitions*/
const Value V17201_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17201_298 = {S20, 0, &V17201_298_57206, &P17201_241};	/*Property = CssUI*/
const Value V17201_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17201_300 = {S21, 0, &V17201_300_57216, &P17201_298};	/*Property = DataSet*/
const Value V17201_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17201_301 = {S22, 0, &V17201_301_57218, &P17201_300};	/*Property = DataUrl*/
const Value V17201_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17201_302 = {S23, 0, &V17201_302_57222, &P17201_301};	/*Property = DeviceOrientation*/
const Value V17201_304_57227 = {S2, NULL};	/*Value = True*/
const Property P17201_304 = {S24, 0, &V17201_304_57227, &P17201_302};	/*Property = FileReader*/
const Value V17201_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17201_305 = {S25, 0, &V17201_305_57231, &P17201_304};	/*Property = FileSaver*/
const Value V17201_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17201_306 = {S26, 0, &V17201_306_57237, &P17201_305};	/*Property = FileWriter*/
const Value V17201_307_57239 = {S2, NULL};	/*Value = True*/
const Property P17201_307 = {S27, 0, &V17201_307_57239, &P17201_306};	/*Property = FormData*/
const Value V17201_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17201_309 = {S28, 0, &V17201_309_57246, &P17201_307};	/*Property = Fullscreen*/
const Value V17201_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17201_310 = {S29, 0, &V17201_310_57248, &P17201_309};	/*Property = GeoLocation*/
const Value V17201_312_57254 = {S2, NULL};	/*Value = True*/
const Property P17201_312 = {S30, 0, &V17201_312_57254, &P17201_310};	/*Property = History*/
const Value V17201_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17201_314 = {S31, 0, &V17201_314_57261, &P17201_312};	/*Property = Html5*/
const Value V17201_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17201_313 = {S32, 0, &V17201_313_57258, &P17201_314};	/*Property = Html-Media-Capture*/
const Value V17201_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17201_315 = {S33, 0, &V17201_315_57264, &P17201_313};	/*Property = Iframe*/
const Value V17201_316_57266 = {S2, NULL};	/*Value = True*/
const Property P17201_316 = {S34, 0, &V17201_316_57266, &P17201_315};	/*Property = IndexedDB*/
const Value V17201_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17201_317 = {S35, 0, &V17201_317_57269, &P17201_316};	/*Property = Json*/
const Value V17201_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P17201_15 = {S37, 0, &V17201_15_30, &P17201_317};	/*Property = LayoutEngine*/
const Value V17201_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17201_344 = {S38, 0, &V17201_344_57272, &P17201_15};	/*Property = Masking*/
const Value V17201_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17201_320 = {S39, 0, &V17201_320_57290, &P17201_344};	/*Property = PostMessage*/
const Value V17201_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17201_321 = {S40, 0, &V17201_321_57293, &P17201_320};	/*Property = Progress*/
const Value V17201_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17201_322 = {S41, 0, &V17201_322_57296, &P17201_321};	/*Property = Prompts*/
const Value V17201_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17201_323 = {S42, 0, &V17201_323_57299, &P17201_322};	/*Property = Selector*/
const Value V17201_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17201_324 = {S43, 0, &V17201_324_57306, &P17201_323};	/*Property = Svg*/
const Value V17201_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17201_327 = {S44, 0, &V17201_327_57315, &P17201_324};	/*Property = TouchEvents*/
const Value V17201_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17201_328 = {S45, 0, &V17201_328_57318, &P17201_327};	/*Property = Track*/
const Value V17201_329_57323 = {S2, NULL};	/*Value = True*/
const Property P17201_329 = {S46, 0, &V17201_329_57323, &P17201_328};	/*Property = Video*/
const Value V17201_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17201_330 = {S47, 0, &V17201_330_57326, &P17201_329};	/*Property = Viewport*/
const Value V17201_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17201_332 = {S48, 0, &V17201_332_57339, &P17201_330};	/*Property = WebWorkers*/
const Value V17201_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17201_333 = {S49, 0, &V17201_333_57342, &P17201_332};	/*Property = Xhr2*/
const Profile P17201 = {"17201", &P17201_333};	/*Profile ID - 17201*/
const Value V17415_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17415_279 = {S1, 0, &V17415_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17415_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17415_282 = {S3, 0, &V17415_282_57154, &P17415_279};	/*Property = BlobBuilder*/
const Value V17415_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17415_350 = {S4, 0, &V17415_350_57444, &P17415_282};	/*Property = Canvas*/
const Value V17415_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17415_283 = {S5, 0, &V17415_283_57159, &P17415_350};	/*Property = CssBackground*/
const Value V17415_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17415_284 = {S6, 0, &V17415_284_57163, &P17415_283};	/*Property = CssBorderImage*/
const Value V17415_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17415_285 = {S7, 0, &V17415_285_57166, &P17415_284};	/*Property = CssCanvas*/
const Value V17415_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17415_286 = {S8, 0, &V17415_286_57168, &P17415_285};	/*Property = CssColor*/
const Value V17415_237_55784 = {S2, NULL};	/*Value = True*/
const Property P17415_237 = {S9, 0, &V17415_237_55784, &P17415_286};	/*Property = CssColumn*/
const Value V17415_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17415_287 = {S10, 0, &V17415_287_57172, &P17415_237};	/*Property = CssFlexbox*/
const Value V17415_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17415_288 = {S11, 0, &V17415_288_57175, &P17415_287};	/*Property = CssFont*/
const Value V17415_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17415_289 = {S12, 0, &V17415_289_57759, &P17415_288};	/*Property = CssImages*/
const Value V17415_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17415_291 = {S13, 0, &V17415_291_57181, &P17415_289};	/*Property = CssMediaQueries*/
const Value V17415_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17415_292 = {S14, 0, &V17415_292_57183, &P17415_291};	/*Property = CssMinMax*/
const Value V17415_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17415_293 = {S15, 0, &V17415_293_57187, &P17415_292};	/*Property = CssOverflow*/
const Value V17415_294_57190 = {S0, NULL};	/*Value = False*/
const Property P17415_294 = {S16, 0, &V17415_294_57190, &P17415_293};	/*Property = CssPosition*/
const Value V17415_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17415_295 = {S17, 0, &V17415_295_57192, &P17415_294};	/*Property = CssText*/
const Value V17415_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17415_297 = {S18, 0, &V17415_297_57195, &P17415_295};	/*Property = CssTransforms*/
const Value V17415_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17415_241 = {S19, 0, &V17415_241_55799, &P17415_297};	/*Property = CssTransitions*/
const Value V17415_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17415_298 = {S20, 0, &V17415_298_57206, &P17415_241};	/*Property = CssUI*/
const Value V17415_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17415_300 = {S21, 0, &V17415_300_57216, &P17415_298};	/*Property = DataSet*/
const Value V17415_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17415_301 = {S22, 0, &V17415_301_57218, &P17415_300};	/*Property = DataUrl*/
const Value V17415_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17415_302 = {S23, 0, &V17415_302_57222, &P17415_301};	/*Property = DeviceOrientation*/
const Value V17415_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17415_304 = {S24, 0, &V17415_304_57228, &P17415_302};	/*Property = FileReader*/
const Value V17415_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17415_305 = {S25, 0, &V17415_305_57231, &P17415_304};	/*Property = FileSaver*/
const Value V17415_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17415_306 = {S26, 0, &V17415_306_57237, &P17415_305};	/*Property = FileWriter*/
const Value V17415_307_57240 = {S0, NULL};	/*Value = False*/
const Property P17415_307 = {S27, 0, &V17415_307_57240, &P17415_306};	/*Property = FormData*/
const Value V17415_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17415_309 = {S28, 0, &V17415_309_57246, &P17415_307};	/*Property = Fullscreen*/
const Value V17415_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17415_310 = {S29, 0, &V17415_310_57248, &P17415_309};	/*Property = GeoLocation*/
const Value V17415_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17415_312 = {S30, 0, &V17415_312_57255, &P17415_310};	/*Property = History*/
const Value V17415_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17415_314 = {S31, 0, &V17415_314_57261, &P17415_312};	/*Property = Html5*/
const Value V17415_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17415_313 = {S32, 0, &V17415_313_57258, &P17415_314};	/*Property = Html-Media-Capture*/
const Value V17415_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17415_315 = {S33, 0, &V17415_315_57264, &P17415_313};	/*Property = Iframe*/
const Value V17415_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17415_316 = {S34, 0, &V17415_316_57267, &P17415_315};	/*Property = IndexedDB*/
const Value V17415_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17415_317 = {S35, 0, &V17415_317_57269, &P17415_316};	/*Property = Json*/
const Value V17415_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P17415_15 = {S37, 0, &V17415_15_28, &P17415_317};	/*Property = LayoutEngine*/
const Value V17415_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17415_344 = {S38, 0, &V17415_344_57272, &P17415_15};	/*Property = Masking*/
const Value V17415_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17415_320 = {S39, 0, &V17415_320_57290, &P17415_344};	/*Property = PostMessage*/
const Value V17415_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17415_321 = {S40, 0, &V17415_321_57293, &P17415_320};	/*Property = Progress*/
const Value V17415_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17415_322 = {S41, 0, &V17415_322_57296, &P17415_321};	/*Property = Prompts*/
const Value V17415_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17415_323 = {S42, 0, &V17415_323_57299, &P17415_322};	/*Property = Selector*/
const Value V17415_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17415_324 = {S43, 0, &V17415_324_57306, &P17415_323};	/*Property = Svg*/
const Value V17415_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17415_327 = {S44, 0, &V17415_327_57315, &P17415_324};	/*Property = TouchEvents*/
const Value V17415_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17415_328 = {S45, 0, &V17415_328_57318, &P17415_327};	/*Property = Track*/
const Value V17415_329_57324 = {S0, NULL};	/*Value = False*/
const Property P17415_329 = {S46, 0, &V17415_329_57324, &P17415_328};	/*Property = Video*/
const Value V17415_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17415_330 = {S47, 0, &V17415_330_57326, &P17415_329};	/*Property = Viewport*/
const Value V17415_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17415_332 = {S48, 0, &V17415_332_57339, &P17415_330};	/*Property = WebWorkers*/
const Value V17415_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17415_333 = {S49, 0, &V17415_333_57342, &P17415_332};	/*Property = Xhr2*/
const Profile P17415 = {"17415", &P17415_333};	/*Profile ID - 17415*/
const Value V17687_1_1 = {S2, NULL};	/*Value = True*/
const Property P17687_1 = {S50, 0, &V17687_1_1, NULL};	/*Property = IsMobile*/
const Value V17687_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P17687_5 = {S52, 0, &V17687_5_9981, &P17687_1};	/*Property = ScreenPixelsHeight*/
const Value V17687_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P17687_4 = {S54, 0, &V17687_4_2336, &P17687_5};	/*Property = ScreenPixelsWidth*/
const Profile P17687 = {"17687", &P17687_4};	/*Profile ID - 17687*/
const Value V23995_1_1 = {S2, NULL};	/*Value = True*/
const Property P23995_1 = {S50, 0, &V23995_1_1, NULL};	/*Property = IsMobile*/
const Value V23995_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23995_5 = {S52, 0, &V23995_5_1458, &P23995_1};	/*Property = ScreenPixelsHeight*/
const Value V23995_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23995_4 = {S54, 0, &V23995_4_1460, &P23995_5};	/*Property = ScreenPixelsWidth*/
const Profile P23995 = {"23995", &P23995_4};	/*Profile ID - 23995*/
const Value V17366_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17366_279 = {S1, 0, &V17366_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17366_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17366_282 = {S3, 0, &V17366_282_57154, &P17366_279};	/*Property = BlobBuilder*/
const Value V17366_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17366_350 = {S4, 0, &V17366_350_57444, &P17366_282};	/*Property = Canvas*/
const Value V17366_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17366_283 = {S5, 0, &V17366_283_57159, &P17366_350};	/*Property = CssBackground*/
const Value V17366_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17366_284 = {S6, 0, &V17366_284_57163, &P17366_283};	/*Property = CssBorderImage*/
const Value V17366_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17366_285 = {S7, 0, &V17366_285_57166, &P17366_284};	/*Property = CssCanvas*/
const Value V17366_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17366_286 = {S8, 0, &V17366_286_57168, &P17366_285};	/*Property = CssColor*/
const Value V17366_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17366_237 = {S9, 0, &V17366_237_55786, &P17366_286};	/*Property = CssColumn*/
const Value V17366_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17366_287 = {S10, 0, &V17366_287_57172, &P17366_237};	/*Property = CssFlexbox*/
const Value V17366_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17366_288 = {S11, 0, &V17366_288_57175, &P17366_287};	/*Property = CssFont*/
const Value V17366_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17366_289 = {S12, 0, &V17366_289_57759, &P17366_288};	/*Property = CssImages*/
const Value V17366_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17366_291 = {S13, 0, &V17366_291_57181, &P17366_289};	/*Property = CssMediaQueries*/
const Value V17366_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17366_292 = {S14, 0, &V17366_292_57183, &P17366_291};	/*Property = CssMinMax*/
const Value V17366_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17366_293 = {S15, 0, &V17366_293_57187, &P17366_292};	/*Property = CssOverflow*/
const Value V17366_294_57190 = {S0, NULL};	/*Value = False*/
const Property P17366_294 = {S16, 0, &V17366_294_57190, &P17366_293};	/*Property = CssPosition*/
const Value V17366_295_57193 = {S0, NULL};	/*Value = False*/
const Property P17366_295 = {S17, 0, &V17366_295_57193, &P17366_294};	/*Property = CssText*/
const Value V17366_297_57196 = {S0, NULL};	/*Value = False*/
const Property P17366_297 = {S18, 0, &V17366_297_57196, &P17366_295};	/*Property = CssTransforms*/
const Value V17366_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17366_241 = {S19, 0, &V17366_241_55799, &P17366_297};	/*Property = CssTransitions*/
const Value V17366_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17366_298 = {S20, 0, &V17366_298_57206, &P17366_241};	/*Property = CssUI*/
const Value V17366_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17366_300 = {S21, 0, &V17366_300_57216, &P17366_298};	/*Property = DataSet*/
const Value V17366_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17366_301 = {S22, 0, &V17366_301_57218, &P17366_300};	/*Property = DataUrl*/
const Value V17366_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17366_302 = {S23, 0, &V17366_302_57222, &P17366_301};	/*Property = DeviceOrientation*/
const Value V17366_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17366_304 = {S24, 0, &V17366_304_57228, &P17366_302};	/*Property = FileReader*/
const Value V17366_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17366_305 = {S25, 0, &V17366_305_57231, &P17366_304};	/*Property = FileSaver*/
const Value V17366_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17366_306 = {S26, 0, &V17366_306_57237, &P17366_305};	/*Property = FileWriter*/
const Value V17366_307_57240 = {S0, NULL};	/*Value = False*/
const Property P17366_307 = {S27, 0, &V17366_307_57240, &P17366_306};	/*Property = FormData*/
const Value V17366_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17366_309 = {S28, 0, &V17366_309_57246, &P17366_307};	/*Property = Fullscreen*/
const Value V17366_310_57248 = {S2, NULL};	/*Value = True*/
const Property P17366_310 = {S29, 0, &V17366_310_57248, &P17366_309};	/*Property = GeoLocation*/
const Value V17366_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17366_312 = {S30, 0, &V17366_312_57255, &P17366_310};	/*Property = History*/
const Value V17366_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17366_314 = {S31, 0, &V17366_314_57261, &P17366_312};	/*Property = Html5*/
const Value V17366_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17366_313 = {S32, 0, &V17366_313_57258, &P17366_314};	/*Property = Html-Media-Capture*/
const Value V17366_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17366_315 = {S33, 0, &V17366_315_57264, &P17366_313};	/*Property = Iframe*/
const Value V17366_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17366_316 = {S34, 0, &V17366_316_57267, &P17366_315};	/*Property = IndexedDB*/
const Value V17366_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17366_317 = {S35, 0, &V17366_317_57269, &P17366_316};	/*Property = Json*/
const Value V17366_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P17366_15 = {S37, 0, &V17366_15_31, &P17366_317};	/*Property = LayoutEngine*/
const Value V17366_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17366_344 = {S38, 0, &V17366_344_57272, &P17366_15};	/*Property = Masking*/
const Value V17366_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17366_320 = {S39, 0, &V17366_320_57290, &P17366_344};	/*Property = PostMessage*/
const Value V17366_321_57294 = {S0, NULL};	/*Value = False*/
const Property P17366_321 = {S40, 0, &V17366_321_57294, &P17366_320};	/*Property = Progress*/
const Value V17366_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17366_322 = {S41, 0, &V17366_322_57296, &P17366_321};	/*Property = Prompts*/
const Value V17366_323_57300 = {S0, NULL};	/*Value = False*/
const Property P17366_323 = {S42, 0, &V17366_323_57300, &P17366_322};	/*Property = Selector*/
const Value V17366_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17366_324 = {S43, 0, &V17366_324_57306, &P17366_323};	/*Property = Svg*/
const Value V17366_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17366_327 = {S44, 0, &V17366_327_57315, &P17366_324};	/*Property = TouchEvents*/
const Value V17366_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17366_328 = {S45, 0, &V17366_328_57318, &P17366_327};	/*Property = Track*/
const Value V17366_329_57324 = {S0, NULL};	/*Value = False*/
const Property P17366_329 = {S46, 0, &V17366_329_57324, &P17366_328};	/*Property = Video*/
const Value V17366_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17366_330 = {S47, 0, &V17366_330_57326, &P17366_329};	/*Property = Viewport*/
const Value V17366_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17366_332 = {S48, 0, &V17366_332_57339, &P17366_330};	/*Property = WebWorkers*/
const Value V17366_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17366_333 = {S49, 0, &V17366_333_57342, &P17366_332};	/*Property = Xhr2*/
const Profile P17366 = {"17366", &P17366_333};	/*Profile ID - 17366*/
const Value V22889_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_279 = {S1, 0, &V22889_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22889_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_282 = {S3, 0, &V22889_282_57155, &P22889_279};	/*Property = BlobBuilder*/
const Value V22889_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_350 = {S4, 0, &V22889_350_57446, &P22889_282};	/*Property = Canvas*/
const Value V22889_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_283 = {S5, 0, &V22889_283_57161, &P22889_350};	/*Property = CssBackground*/
const Value V22889_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_284 = {S6, 0, &V22889_284_57164, &P22889_283};	/*Property = CssBorderImage*/
const Value V22889_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_285 = {S7, 0, &V22889_285_57167, &P22889_284};	/*Property = CssCanvas*/
const Value V22889_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_286 = {S8, 0, &V22889_286_57170, &P22889_285};	/*Property = CssColor*/
const Value V22889_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_237 = {S9, 0, &V22889_237_55786, &P22889_286};	/*Property = CssColumn*/
const Value V22889_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_287 = {S10, 0, &V22889_287_57173, &P22889_237};	/*Property = CssFlexbox*/
const Value V22889_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_288 = {S11, 0, &V22889_288_57176, &P22889_287};	/*Property = CssFont*/
const Value V22889_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_289 = {S12, 0, &V22889_289_86899, &P22889_288};	/*Property = CssImages*/
const Value V22889_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_291 = {S13, 0, &V22889_291_57182, &P22889_289};	/*Property = CssMediaQueries*/
const Value V22889_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_292 = {S14, 0, &V22889_292_57185, &P22889_291};	/*Property = CssMinMax*/
const Value V22889_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_293 = {S15, 0, &V22889_293_57188, &P22889_292};	/*Property = CssOverflow*/
const Value V22889_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_294 = {S16, 0, &V22889_294_57191, &P22889_293};	/*Property = CssPosition*/
const Value V22889_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_295 = {S17, 0, &V22889_295_57194, &P22889_294};	/*Property = CssText*/
const Value V22889_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_297 = {S18, 0, &V22889_297_57197, &P22889_295};	/*Property = CssTransforms*/
const Value V22889_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_241 = {S19, 0, &V22889_241_55801, &P22889_297};	/*Property = CssTransitions*/
const Value V22889_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_298 = {S20, 0, &V22889_298_57208, &P22889_241};	/*Property = CssUI*/
const Value V22889_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_300 = {S21, 0, &V22889_300_57217, &P22889_298};	/*Property = DataSet*/
const Value V22889_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_301 = {S22, 0, &V22889_301_57220, &P22889_300};	/*Property = DataUrl*/
const Value V22889_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_302 = {S23, 0, &V22889_302_57223, &P22889_301};	/*Property = DeviceOrientation*/
const Value V22889_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_304 = {S24, 0, &V22889_304_57229, &P22889_302};	/*Property = FileReader*/
const Value V22889_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_305 = {S25, 0, &V22889_305_57232, &P22889_304};	/*Property = FileSaver*/
const Value V22889_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_306 = {S26, 0, &V22889_306_57238, &P22889_305};	/*Property = FileWriter*/
const Value V22889_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_307 = {S27, 0, &V22889_307_57241, &P22889_306};	/*Property = FormData*/
const Value V22889_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_309 = {S28, 0, &V22889_309_57247, &P22889_307};	/*Property = Fullscreen*/
const Value V22889_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_310 = {S29, 0, &V22889_310_57250, &P22889_309};	/*Property = GeoLocation*/
const Value V22889_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_312 = {S30, 0, &V22889_312_57256, &P22889_310};	/*Property = History*/
const Value V22889_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_314 = {S31, 0, &V22889_314_57262, &P22889_312};	/*Property = Html5*/
const Value V22889_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_313 = {S32, 0, &V22889_313_57259, &P22889_314};	/*Property = Html-Media-Capture*/
const Value V22889_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_315 = {S33, 0, &V22889_315_57265, &P22889_313};	/*Property = Iframe*/
const Value V22889_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_316 = {S34, 0, &V22889_316_57268, &P22889_315};	/*Property = IndexedDB*/
const Value V22889_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_317 = {S35, 0, &V22889_317_57271, &P22889_316};	/*Property = Json*/
const Value V22889_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_15 = {S37, 0, &V22889_15_72, &P22889_317};	/*Property = LayoutEngine*/
const Value V22889_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_344 = {S38, 0, &V22889_344_57274, &P22889_15};	/*Property = Masking*/
const Value V22889_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_320 = {S39, 0, &V22889_320_57292, &P22889_344};	/*Property = PostMessage*/
const Value V22889_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_321 = {S40, 0, &V22889_321_57295, &P22889_320};	/*Property = Progress*/
const Value V22889_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_322 = {S41, 0, &V22889_322_57298, &P22889_321};	/*Property = Prompts*/
const Value V22889_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_323 = {S42, 0, &V22889_323_57301, &P22889_322};	/*Property = Selector*/
const Value V22889_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_324 = {S43, 0, &V22889_324_57307, &P22889_323};	/*Property = Svg*/
const Value V22889_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_327 = {S44, 0, &V22889_327_57316, &P22889_324};	/*Property = TouchEvents*/
const Value V22889_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_328 = {S45, 0, &V22889_328_57319, &P22889_327};	/*Property = Track*/
const Value V22889_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_329 = {S46, 0, &V22889_329_57325, &P22889_328};	/*Property = Video*/
const Value V22889_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_330 = {S47, 0, &V22889_330_57328, &P22889_329};	/*Property = Viewport*/
const Value V22889_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_332 = {S48, 0, &V22889_332_57340, &P22889_330};	/*Property = WebWorkers*/
const Value V22889_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22889_333 = {S49, 0, &V22889_333_57343, &P22889_332};	/*Property = Xhr2*/
const Profile P22889 = {"22889", &P22889_333};	/*Profile ID - 22889*/
const Value V17958_1_1 = {S2, NULL};	/*Value = True*/
const Property P17958_1 = {S50, 0, &V17958_1_1, NULL};	/*Property = IsMobile*/
const Value V17958_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17958_5 = {S52, 0, &V17958_5_1227, &P17958_1};	/*Property = ScreenPixelsHeight*/
const Value V17958_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P17958_4 = {S54, 0, &V17958_4_1460, &P17958_5};	/*Property = ScreenPixelsWidth*/
const Profile P17958 = {"17958", &P17958_4};	/*Profile ID - 17958*/
const Profile P22190 = {"22190", NULL};	/*Profile ID - 22190*/
const Profile P18575 = {"18575", NULL};	/*Profile ID - 18575*/
const Value V21214_1_1 = {S2, NULL};	/*Value = True*/
const Property P21214_1 = {S50, 0, &V21214_1_1, NULL};	/*Property = IsMobile*/
const Value V21214_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21214_5 = {S52, 0, &V21214_5_1227, &P21214_1};	/*Property = ScreenPixelsHeight*/
const Value V21214_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21214_4 = {S54, 0, &V21214_4_1229, &P21214_5};	/*Property = ScreenPixelsWidth*/
const Profile P21214 = {"21214", &P21214_4};	/*Profile ID - 21214*/
const Value V17829_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_279 = {S1, 0, &V17829_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17829_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_282 = {S3, 0, &V17829_282_57155, &P17829_279};	/*Property = BlobBuilder*/
const Value V17829_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_350 = {S4, 0, &V17829_350_57446, &P17829_282};	/*Property = Canvas*/
const Value V17829_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_283 = {S5, 0, &V17829_283_57161, &P17829_350};	/*Property = CssBackground*/
const Value V17829_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_284 = {S6, 0, &V17829_284_57164, &P17829_283};	/*Property = CssBorderImage*/
const Value V17829_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_285 = {S7, 0, &V17829_285_57167, &P17829_284};	/*Property = CssCanvas*/
const Value V17829_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_286 = {S8, 0, &V17829_286_57170, &P17829_285};	/*Property = CssColor*/
const Value V17829_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_237 = {S9, 0, &V17829_237_55786, &P17829_286};	/*Property = CssColumn*/
const Value V17829_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_287 = {S10, 0, &V17829_287_57173, &P17829_237};	/*Property = CssFlexbox*/
const Value V17829_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_288 = {S11, 0, &V17829_288_57176, &P17829_287};	/*Property = CssFont*/
const Value V17829_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_289 = {S12, 0, &V17829_289_86899, &P17829_288};	/*Property = CssImages*/
const Value V17829_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_291 = {S13, 0, &V17829_291_57182, &P17829_289};	/*Property = CssMediaQueries*/
const Value V17829_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_292 = {S14, 0, &V17829_292_57185, &P17829_291};	/*Property = CssMinMax*/
const Value V17829_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_293 = {S15, 0, &V17829_293_57188, &P17829_292};	/*Property = CssOverflow*/
const Value V17829_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_294 = {S16, 0, &V17829_294_57191, &P17829_293};	/*Property = CssPosition*/
const Value V17829_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_295 = {S17, 0, &V17829_295_57194, &P17829_294};	/*Property = CssText*/
const Value V17829_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_297 = {S18, 0, &V17829_297_57197, &P17829_295};	/*Property = CssTransforms*/
const Value V17829_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_241 = {S19, 0, &V17829_241_55801, &P17829_297};	/*Property = CssTransitions*/
const Value V17829_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_298 = {S20, 0, &V17829_298_57208, &P17829_241};	/*Property = CssUI*/
const Value V17829_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_300 = {S21, 0, &V17829_300_57217, &P17829_298};	/*Property = DataSet*/
const Value V17829_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_301 = {S22, 0, &V17829_301_57220, &P17829_300};	/*Property = DataUrl*/
const Value V17829_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_302 = {S23, 0, &V17829_302_57223, &P17829_301};	/*Property = DeviceOrientation*/
const Value V17829_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_304 = {S24, 0, &V17829_304_57229, &P17829_302};	/*Property = FileReader*/
const Value V17829_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_305 = {S25, 0, &V17829_305_57232, &P17829_304};	/*Property = FileSaver*/
const Value V17829_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_306 = {S26, 0, &V17829_306_57238, &P17829_305};	/*Property = FileWriter*/
const Value V17829_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_307 = {S27, 0, &V17829_307_57241, &P17829_306};	/*Property = FormData*/
const Value V17829_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_309 = {S28, 0, &V17829_309_57247, &P17829_307};	/*Property = Fullscreen*/
const Value V17829_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_310 = {S29, 0, &V17829_310_57250, &P17829_309};	/*Property = GeoLocation*/
const Value V17829_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_312 = {S30, 0, &V17829_312_57256, &P17829_310};	/*Property = History*/
const Value V17829_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_314 = {S31, 0, &V17829_314_57262, &P17829_312};	/*Property = Html5*/
const Value V17829_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_313 = {S32, 0, &V17829_313_57259, &P17829_314};	/*Property = Html-Media-Capture*/
const Value V17829_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_315 = {S33, 0, &V17829_315_57265, &P17829_313};	/*Property = Iframe*/
const Value V17829_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_316 = {S34, 0, &V17829_316_57268, &P17829_315};	/*Property = IndexedDB*/
const Value V17829_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_317 = {S35, 0, &V17829_317_57271, &P17829_316};	/*Property = Json*/
const Value V17829_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P17829_15 = {S37, 0, &V17829_15_54167, &P17829_317};	/*Property = LayoutEngine*/
const Value V17829_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_344 = {S38, 0, &V17829_344_57274, &P17829_15};	/*Property = Masking*/
const Value V17829_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_320 = {S39, 0, &V17829_320_57292, &P17829_344};	/*Property = PostMessage*/
const Value V17829_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_321 = {S40, 0, &V17829_321_57295, &P17829_320};	/*Property = Progress*/
const Value V17829_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_322 = {S41, 0, &V17829_322_57298, &P17829_321};	/*Property = Prompts*/
const Value V17829_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_323 = {S42, 0, &V17829_323_57301, &P17829_322};	/*Property = Selector*/
const Value V17829_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_324 = {S43, 0, &V17829_324_57307, &P17829_323};	/*Property = Svg*/
const Value V17829_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_327 = {S44, 0, &V17829_327_57316, &P17829_324};	/*Property = TouchEvents*/
const Value V17829_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_328 = {S45, 0, &V17829_328_57319, &P17829_327};	/*Property = Track*/
const Value V17829_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_329 = {S46, 0, &V17829_329_57325, &P17829_328};	/*Property = Video*/
const Value V17829_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_330 = {S47, 0, &V17829_330_57328, &P17829_329};	/*Property = Viewport*/
const Value V17829_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_332 = {S48, 0, &V17829_332_57340, &P17829_330};	/*Property = WebWorkers*/
const Value V17829_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17829_333 = {S49, 0, &V17829_333_57343, &P17829_332};	/*Property = Xhr2*/
const Profile P17829 = {"17829", &P17829_333};	/*Profile ID - 17829*/
const Value V16607_1_1 = {S2, NULL};	/*Value = True*/
const Property P16607_1 = {S50, 0, &V16607_1_1, NULL};	/*Property = IsMobile*/
const Value V16607_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16607_5 = {S52, 0, &V16607_5_1222, &P16607_1};	/*Property = ScreenPixelsHeight*/
const Value V16607_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16607_4 = {S54, 0, &V16607_4_1224, &P16607_5};	/*Property = ScreenPixelsWidth*/
const Profile P16607 = {"16607", &P16607_4};	/*Profile ID - 16607*/
const Value V16897_1_1 = {S2, NULL};	/*Value = True*/
const Property P16897_1 = {S50, 0, &V16897_1_1, NULL};	/*Property = IsMobile*/
const Value V16897_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16897_5 = {S52, 0, &V16897_5_1293, &P16897_1};	/*Property = ScreenPixelsHeight*/
const Value V16897_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16897_4 = {S54, 0, &V16897_4_1296, &P16897_5};	/*Property = ScreenPixelsWidth*/
const Profile P16897 = {"16897", &P16897_4};	/*Profile ID - 16897*/
const Value V22448_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22448_279 = {S1, 0, &V22448_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22448_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22448_282 = {S3, 0, &V22448_282_57153, &P22448_279};	/*Property = BlobBuilder*/
const Value V22448_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22448_350 = {S4, 0, &V22448_350_57444, &P22448_282};	/*Property = Canvas*/
const Value V22448_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22448_283 = {S5, 0, &V22448_283_57159, &P22448_350};	/*Property = CssBackground*/
const Value V22448_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22448_284 = {S6, 0, &V22448_284_57163, &P22448_283};	/*Property = CssBorderImage*/
const Value V22448_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22448_285 = {S7, 0, &V22448_285_57166, &P22448_284};	/*Property = CssCanvas*/
const Value V22448_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22448_286 = {S8, 0, &V22448_286_57168, &P22448_285};	/*Property = CssColor*/
const Value V22448_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22448_237 = {S9, 0, &V22448_237_55784, &P22448_286};	/*Property = CssColumn*/
const Value V22448_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22448_287 = {S10, 0, &V22448_287_57172, &P22448_237};	/*Property = CssFlexbox*/
const Value V22448_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22448_288 = {S11, 0, &V22448_288_57175, &P22448_287};	/*Property = CssFont*/
const Value V22448_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22448_289 = {S12, 0, &V22448_289_57759, &P22448_288};	/*Property = CssImages*/
const Value V22448_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22448_291 = {S13, 0, &V22448_291_57180, &P22448_289};	/*Property = CssMediaQueries*/
const Value V22448_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22448_292 = {S14, 0, &V22448_292_57183, &P22448_291};	/*Property = CssMinMax*/
const Value V22448_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22448_293 = {S15, 0, &V22448_293_57187, &P22448_292};	/*Property = CssOverflow*/
const Value V22448_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22448_294 = {S16, 0, &V22448_294_57189, &P22448_293};	/*Property = CssPosition*/
const Value V22448_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22448_295 = {S17, 0, &V22448_295_57192, &P22448_294};	/*Property = CssText*/
const Value V22448_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22448_297 = {S18, 0, &V22448_297_57195, &P22448_295};	/*Property = CssTransforms*/
const Value V22448_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22448_241 = {S19, 0, &V22448_241_55799, &P22448_297};	/*Property = CssTransitions*/
const Value V22448_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22448_298 = {S20, 0, &V22448_298_57206, &P22448_241};	/*Property = CssUI*/
const Value V22448_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22448_300 = {S21, 0, &V22448_300_57216, &P22448_298};	/*Property = DataSet*/
const Value V22448_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22448_301 = {S22, 0, &V22448_301_57218, &P22448_300};	/*Property = DataUrl*/
const Value V22448_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22448_302 = {S23, 0, &V22448_302_57222, &P22448_301};	/*Property = DeviceOrientation*/
const Value V22448_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22448_304 = {S24, 0, &V22448_304_57227, &P22448_302};	/*Property = FileReader*/
const Value V22448_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22448_305 = {S25, 0, &V22448_305_57231, &P22448_304};	/*Property = FileSaver*/
const Value V22448_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22448_306 = {S26, 0, &V22448_306_57237, &P22448_305};	/*Property = FileWriter*/
const Value V22448_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22448_307 = {S27, 0, &V22448_307_57239, &P22448_306};	/*Property = FormData*/
const Value V22448_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22448_309 = {S28, 0, &V22448_309_57246, &P22448_307};	/*Property = Fullscreen*/
const Value V22448_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22448_310 = {S29, 0, &V22448_310_57248, &P22448_309};	/*Property = GeoLocation*/
const Value V22448_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22448_312 = {S30, 0, &V22448_312_57255, &P22448_310};	/*Property = History*/
const Value V22448_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22448_314 = {S31, 0, &V22448_314_57261, &P22448_312};	/*Property = Html5*/
const Value V22448_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22448_313 = {S32, 0, &V22448_313_57258, &P22448_314};	/*Property = Html-Media-Capture*/
const Value V22448_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22448_315 = {S33, 0, &V22448_315_57264, &P22448_313};	/*Property = Iframe*/
const Value V22448_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22448_316 = {S34, 0, &V22448_316_57267, &P22448_315};	/*Property = IndexedDB*/
const Value V22448_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22448_317 = {S35, 0, &V22448_317_57269, &P22448_316};	/*Property = Json*/
const Value V22448_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22448_15 = {S37, 0, &V22448_15_28, &P22448_317};	/*Property = LayoutEngine*/
const Value V22448_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22448_344 = {S38, 0, &V22448_344_57272, &P22448_15};	/*Property = Masking*/
const Value V22448_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22448_320 = {S39, 0, &V22448_320_57290, &P22448_344};	/*Property = PostMessage*/
const Value V22448_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22448_321 = {S40, 0, &V22448_321_57293, &P22448_320};	/*Property = Progress*/
const Value V22448_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22448_322 = {S41, 0, &V22448_322_57296, &P22448_321};	/*Property = Prompts*/
const Value V22448_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22448_323 = {S42, 0, &V22448_323_57299, &P22448_322};	/*Property = Selector*/
const Value V22448_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22448_324 = {S43, 0, &V22448_324_57306, &P22448_323};	/*Property = Svg*/
const Value V22448_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22448_327 = {S44, 0, &V22448_327_57315, &P22448_324};	/*Property = TouchEvents*/
const Value V22448_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22448_328 = {S45, 0, &V22448_328_57318, &P22448_327};	/*Property = Track*/
const Value V22448_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22448_329 = {S46, 0, &V22448_329_57323, &P22448_328};	/*Property = Video*/
const Value V22448_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22448_330 = {S47, 0, &V22448_330_57326, &P22448_329};	/*Property = Viewport*/
const Value V22448_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22448_332 = {S48, 0, &V22448_332_57338, &P22448_330};	/*Property = WebWorkers*/
const Value V22448_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22448_333 = {S49, 0, &V22448_333_57342, &P22448_332};	/*Property = Xhr2*/
const Profile P22448 = {"22448", &P22448_333};	/*Profile ID - 22448*/
const Value V17577_1_1 = {S2, NULL};	/*Value = True*/
const Property P17577_1 = {S50, 0, &V17577_1_1, NULL};	/*Property = IsMobile*/
const Value V17577_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17577_5 = {S52, 0, &V17577_5_1269, &P17577_1};	/*Property = ScreenPixelsHeight*/
const Value V17577_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17577_4 = {S54, 0, &V17577_4_1229, &P17577_5};	/*Property = ScreenPixelsWidth*/
const Profile P17577 = {"17577", &P17577_4};	/*Profile ID - 17577*/
const Profile P17818 = {"17818", NULL};	/*Profile ID - 17818*/
const Value V23863_1_1 = {S2, NULL};	/*Value = True*/
const Property P23863_1 = {S50, 0, &V23863_1_1, NULL};	/*Property = IsMobile*/
const Value V23863_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23863_5 = {S52, 0, &V23863_5_1227, &P23863_1};	/*Property = ScreenPixelsHeight*/
const Value V23863_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23863_4 = {S54, 0, &V23863_4_1229, &P23863_5};	/*Property = ScreenPixelsWidth*/
const Profile P23863 = {"23863", &P23863_4};	/*Profile ID - 23863*/
const Value V17980_1_1 = {S2, NULL};	/*Value = True*/
const Property P17980_1 = {S50, 0, &V17980_1_1, NULL};	/*Property = IsMobile*/
const Value V17980_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17980_5 = {S52, 0, &V17980_5_1227, &P17980_1};	/*Property = ScreenPixelsHeight*/
const Value V17980_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17980_4 = {S54, 0, &V17980_4_1229, &P17980_5};	/*Property = ScreenPixelsWidth*/
const Profile P17980 = {"17980", &P17980_4};	/*Profile ID - 17980*/
const Value V16558_1_1 = {S2, NULL};	/*Value = True*/
const Property P16558_1 = {S50, 0, &V16558_1_1, NULL};	/*Property = IsMobile*/
const Value V16558_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16558_5 = {S52, 0, &V16558_5_1222, &P16558_1};	/*Property = ScreenPixelsHeight*/
const Value V16558_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16558_4 = {S54, 0, &V16558_4_1224, &P16558_5};	/*Property = ScreenPixelsWidth*/
const Profile P16558 = {"16558", &P16558_4};	/*Profile ID - 16558*/
const Value V24163_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_279 = {S1, 0, &V24163_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24163_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_282 = {S3, 0, &V24163_282_57155, &P24163_279};	/*Property = BlobBuilder*/
const Value V24163_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_350 = {S4, 0, &V24163_350_57446, &P24163_282};	/*Property = Canvas*/
const Value V24163_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_283 = {S5, 0, &V24163_283_57161, &P24163_350};	/*Property = CssBackground*/
const Value V24163_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_284 = {S6, 0, &V24163_284_57164, &P24163_283};	/*Property = CssBorderImage*/
const Value V24163_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_285 = {S7, 0, &V24163_285_57167, &P24163_284};	/*Property = CssCanvas*/
const Value V24163_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_286 = {S8, 0, &V24163_286_57170, &P24163_285};	/*Property = CssColor*/
const Value V24163_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_237 = {S9, 0, &V24163_237_55786, &P24163_286};	/*Property = CssColumn*/
const Value V24163_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_287 = {S10, 0, &V24163_287_57173, &P24163_237};	/*Property = CssFlexbox*/
const Value V24163_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_288 = {S11, 0, &V24163_288_57176, &P24163_287};	/*Property = CssFont*/
const Value V24163_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_289 = {S12, 0, &V24163_289_86899, &P24163_288};	/*Property = CssImages*/
const Value V24163_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_291 = {S13, 0, &V24163_291_57182, &P24163_289};	/*Property = CssMediaQueries*/
const Value V24163_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_292 = {S14, 0, &V24163_292_57185, &P24163_291};	/*Property = CssMinMax*/
const Value V24163_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_293 = {S15, 0, &V24163_293_57188, &P24163_292};	/*Property = CssOverflow*/
const Value V24163_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_294 = {S16, 0, &V24163_294_57191, &P24163_293};	/*Property = CssPosition*/
const Value V24163_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_295 = {S17, 0, &V24163_295_57194, &P24163_294};	/*Property = CssText*/
const Value V24163_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_297 = {S18, 0, &V24163_297_57197, &P24163_295};	/*Property = CssTransforms*/
const Value V24163_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_241 = {S19, 0, &V24163_241_55801, &P24163_297};	/*Property = CssTransitions*/
const Value V24163_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_298 = {S20, 0, &V24163_298_57208, &P24163_241};	/*Property = CssUI*/
const Value V24163_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_300 = {S21, 0, &V24163_300_57217, &P24163_298};	/*Property = DataSet*/
const Value V24163_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_301 = {S22, 0, &V24163_301_57220, &P24163_300};	/*Property = DataUrl*/
const Value V24163_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_302 = {S23, 0, &V24163_302_57223, &P24163_301};	/*Property = DeviceOrientation*/
const Value V24163_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_304 = {S24, 0, &V24163_304_57229, &P24163_302};	/*Property = FileReader*/
const Value V24163_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_305 = {S25, 0, &V24163_305_57232, &P24163_304};	/*Property = FileSaver*/
const Value V24163_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_306 = {S26, 0, &V24163_306_57238, &P24163_305};	/*Property = FileWriter*/
const Value V24163_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_307 = {S27, 0, &V24163_307_57241, &P24163_306};	/*Property = FormData*/
const Value V24163_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_309 = {S28, 0, &V24163_309_57247, &P24163_307};	/*Property = Fullscreen*/
const Value V24163_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_310 = {S29, 0, &V24163_310_57250, &P24163_309};	/*Property = GeoLocation*/
const Value V24163_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_312 = {S30, 0, &V24163_312_57256, &P24163_310};	/*Property = History*/
const Value V24163_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_314 = {S31, 0, &V24163_314_57262, &P24163_312};	/*Property = Html5*/
const Value V24163_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_313 = {S32, 0, &V24163_313_57259, &P24163_314};	/*Property = Html-Media-Capture*/
const Value V24163_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_315 = {S33, 0, &V24163_315_57265, &P24163_313};	/*Property = Iframe*/
const Value V24163_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_316 = {S34, 0, &V24163_316_57268, &P24163_315};	/*Property = IndexedDB*/
const Value V24163_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_317 = {S35, 0, &V24163_317_57271, &P24163_316};	/*Property = Json*/
const Value V24163_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P24163_15 = {S37, 0, &V24163_15_31, &P24163_317};	/*Property = LayoutEngine*/
const Value V24163_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_344 = {S38, 0, &V24163_344_57274, &P24163_15};	/*Property = Masking*/
const Value V24163_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_320 = {S39, 0, &V24163_320_57292, &P24163_344};	/*Property = PostMessage*/
const Value V24163_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_321 = {S40, 0, &V24163_321_57295, &P24163_320};	/*Property = Progress*/
const Value V24163_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_322 = {S41, 0, &V24163_322_57298, &P24163_321};	/*Property = Prompts*/
const Value V24163_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_323 = {S42, 0, &V24163_323_57301, &P24163_322};	/*Property = Selector*/
const Value V24163_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_324 = {S43, 0, &V24163_324_57307, &P24163_323};	/*Property = Svg*/
const Value V24163_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_327 = {S44, 0, &V24163_327_57316, &P24163_324};	/*Property = TouchEvents*/
const Value V24163_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_328 = {S45, 0, &V24163_328_57319, &P24163_327};	/*Property = Track*/
const Value V24163_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_329 = {S46, 0, &V24163_329_57325, &P24163_328};	/*Property = Video*/
const Value V24163_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_330 = {S47, 0, &V24163_330_57328, &P24163_329};	/*Property = Viewport*/
const Value V24163_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_332 = {S48, 0, &V24163_332_57340, &P24163_330};	/*Property = WebWorkers*/
const Value V24163_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24163_333 = {S49, 0, &V24163_333_57343, &P24163_332};	/*Property = Xhr2*/
const Profile P24163 = {"24163", &P24163_333};	/*Profile ID - 24163*/
const Profile P18144 = {"18144", NULL};	/*Profile ID - 18144*/
const Value V17999_1_1 = {S2, NULL};	/*Value = True*/
const Property P17999_1 = {S50, 0, &V17999_1_1, NULL};	/*Property = IsMobile*/
const Value V17999_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17999_5 = {S52, 0, &V17999_5_1227, &P17999_1};	/*Property = ScreenPixelsHeight*/
const Value V17999_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P17999_4 = {S54, 0, &V17999_4_1460, &P17999_5};	/*Property = ScreenPixelsWidth*/
const Profile P17999 = {"17999", &P17999_4};	/*Profile ID - 17999*/
const Value V17933_1_1 = {S2, NULL};	/*Value = True*/
const Property P17933_1 = {S50, 0, &V17933_1_1, NULL};	/*Property = IsMobile*/
const Value V17933_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17933_5 = {S52, 0, &V17933_5_1277, &P17933_1};	/*Property = ScreenPixelsHeight*/
const Value V17933_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17933_4 = {S54, 0, &V17933_4_1279, &P17933_5};	/*Property = ScreenPixelsWidth*/
const Profile P17933 = {"17933", &P17933_4};	/*Profile ID - 17933*/
const Value V22402_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22402_279 = {S1, 0, &V22402_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22402_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22402_282 = {S3, 0, &V22402_282_57153, &P22402_279};	/*Property = BlobBuilder*/
const Value V22402_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22402_350 = {S4, 0, &V22402_350_57444, &P22402_282};	/*Property = Canvas*/
const Value V22402_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22402_283 = {S5, 0, &V22402_283_57159, &P22402_350};	/*Property = CssBackground*/
const Value V22402_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22402_284 = {S6, 0, &V22402_284_57163, &P22402_283};	/*Property = CssBorderImage*/
const Value V22402_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22402_285 = {S7, 0, &V22402_285_57166, &P22402_284};	/*Property = CssCanvas*/
const Value V22402_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22402_286 = {S8, 0, &V22402_286_57168, &P22402_285};	/*Property = CssColor*/
const Value V22402_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22402_237 = {S9, 0, &V22402_237_55784, &P22402_286};	/*Property = CssColumn*/
const Value V22402_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22402_287 = {S10, 0, &V22402_287_57172, &P22402_237};	/*Property = CssFlexbox*/
const Value V22402_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22402_288 = {S11, 0, &V22402_288_57175, &P22402_287};	/*Property = CssFont*/
const Value V22402_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22402_289 = {S12, 0, &V22402_289_57759, &P22402_288};	/*Property = CssImages*/
const Value V22402_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22402_291 = {S13, 0, &V22402_291_57180, &P22402_289};	/*Property = CssMediaQueries*/
const Value V22402_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22402_292 = {S14, 0, &V22402_292_57183, &P22402_291};	/*Property = CssMinMax*/
const Value V22402_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22402_293 = {S15, 0, &V22402_293_57187, &P22402_292};	/*Property = CssOverflow*/
const Value V22402_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22402_294 = {S16, 0, &V22402_294_57189, &P22402_293};	/*Property = CssPosition*/
const Value V22402_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22402_295 = {S17, 0, &V22402_295_57192, &P22402_294};	/*Property = CssText*/
const Value V22402_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22402_297 = {S18, 0, &V22402_297_57195, &P22402_295};	/*Property = CssTransforms*/
const Value V22402_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22402_241 = {S19, 0, &V22402_241_55799, &P22402_297};	/*Property = CssTransitions*/
const Value V22402_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22402_298 = {S20, 0, &V22402_298_57206, &P22402_241};	/*Property = CssUI*/
const Value V22402_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22402_300 = {S21, 0, &V22402_300_57216, &P22402_298};	/*Property = DataSet*/
const Value V22402_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22402_301 = {S22, 0, &V22402_301_57218, &P22402_300};	/*Property = DataUrl*/
const Value V22402_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22402_302 = {S23, 0, &V22402_302_57222, &P22402_301};	/*Property = DeviceOrientation*/
const Value V22402_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22402_304 = {S24, 0, &V22402_304_57227, &P22402_302};	/*Property = FileReader*/
const Value V22402_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22402_305 = {S25, 0, &V22402_305_57231, &P22402_304};	/*Property = FileSaver*/
const Value V22402_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22402_306 = {S26, 0, &V22402_306_57237, &P22402_305};	/*Property = FileWriter*/
const Value V22402_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22402_307 = {S27, 0, &V22402_307_57239, &P22402_306};	/*Property = FormData*/
const Value V22402_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22402_309 = {S28, 0, &V22402_309_57246, &P22402_307};	/*Property = Fullscreen*/
const Value V22402_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22402_310 = {S29, 0, &V22402_310_57248, &P22402_309};	/*Property = GeoLocation*/
const Value V22402_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22402_312 = {S30, 0, &V22402_312_57255, &P22402_310};	/*Property = History*/
const Value V22402_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22402_314 = {S31, 0, &V22402_314_57261, &P22402_312};	/*Property = Html5*/
const Value V22402_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22402_313 = {S32, 0, &V22402_313_57258, &P22402_314};	/*Property = Html-Media-Capture*/
const Value V22402_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22402_315 = {S33, 0, &V22402_315_57264, &P22402_313};	/*Property = Iframe*/
const Value V22402_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22402_316 = {S34, 0, &V22402_316_57267, &P22402_315};	/*Property = IndexedDB*/
const Value V22402_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22402_317 = {S35, 0, &V22402_317_57269, &P22402_316};	/*Property = Json*/
const Value V22402_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22402_15 = {S37, 0, &V22402_15_28, &P22402_317};	/*Property = LayoutEngine*/
const Value V22402_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22402_344 = {S38, 0, &V22402_344_57272, &P22402_15};	/*Property = Masking*/
const Value V22402_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22402_320 = {S39, 0, &V22402_320_57290, &P22402_344};	/*Property = PostMessage*/
const Value V22402_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22402_321 = {S40, 0, &V22402_321_57293, &P22402_320};	/*Property = Progress*/
const Value V22402_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22402_322 = {S41, 0, &V22402_322_57296, &P22402_321};	/*Property = Prompts*/
const Value V22402_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22402_323 = {S42, 0, &V22402_323_57299, &P22402_322};	/*Property = Selector*/
const Value V22402_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22402_324 = {S43, 0, &V22402_324_57306, &P22402_323};	/*Property = Svg*/
const Value V22402_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22402_327 = {S44, 0, &V22402_327_57315, &P22402_324};	/*Property = TouchEvents*/
const Value V22402_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22402_328 = {S45, 0, &V22402_328_57318, &P22402_327};	/*Property = Track*/
const Value V22402_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22402_329 = {S46, 0, &V22402_329_57323, &P22402_328};	/*Property = Video*/
const Value V22402_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22402_330 = {S47, 0, &V22402_330_57326, &P22402_329};	/*Property = Viewport*/
const Value V22402_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22402_332 = {S48, 0, &V22402_332_57338, &P22402_330};	/*Property = WebWorkers*/
const Value V22402_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22402_333 = {S49, 0, &V22402_333_57342, &P22402_332};	/*Property = Xhr2*/
const Profile P22402 = {"22402", &P22402_333};	/*Profile ID - 22402*/
const Value V18029_1_1 = {S2, NULL};	/*Value = True*/
const Property P18029_1 = {S50, 0, &V18029_1_1, NULL};	/*Property = IsMobile*/
const Value V18029_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18029_5 = {S52, 0, &V18029_5_1227, &P18029_1};	/*Property = ScreenPixelsHeight*/
const Value V18029_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18029_4 = {S54, 0, &V18029_4_1229, &P18029_5};	/*Property = ScreenPixelsWidth*/
const Profile P18029 = {"18029", &P18029_4};	/*Profile ID - 18029*/
const Value V22403_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22403_279 = {S1, 0, &V22403_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22403_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22403_282 = {S3, 0, &V22403_282_57154, &P22403_279};	/*Property = BlobBuilder*/
const Value V22403_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22403_350 = {S4, 0, &V22403_350_57444, &P22403_282};	/*Property = Canvas*/
const Value V22403_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22403_283 = {S5, 0, &V22403_283_57159, &P22403_350};	/*Property = CssBackground*/
const Value V22403_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22403_284 = {S6, 0, &V22403_284_57163, &P22403_283};	/*Property = CssBorderImage*/
const Value V22403_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22403_285 = {S7, 0, &V22403_285_57166, &P22403_284};	/*Property = CssCanvas*/
const Value V22403_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22403_286 = {S8, 0, &V22403_286_57168, &P22403_285};	/*Property = CssColor*/
const Value V22403_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22403_237 = {S9, 0, &V22403_237_55784, &P22403_286};	/*Property = CssColumn*/
const Value V22403_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22403_287 = {S10, 0, &V22403_287_57172, &P22403_237};	/*Property = CssFlexbox*/
const Value V22403_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22403_288 = {S11, 0, &V22403_288_57175, &P22403_287};	/*Property = CssFont*/
const Value V22403_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22403_289 = {S12, 0, &V22403_289_57759, &P22403_288};	/*Property = CssImages*/
const Value V22403_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22403_291 = {S13, 0, &V22403_291_57180, &P22403_289};	/*Property = CssMediaQueries*/
const Value V22403_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22403_292 = {S14, 0, &V22403_292_57183, &P22403_291};	/*Property = CssMinMax*/
const Value V22403_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22403_293 = {S15, 0, &V22403_293_57187, &P22403_292};	/*Property = CssOverflow*/
const Value V22403_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22403_294 = {S16, 0, &V22403_294_57190, &P22403_293};	/*Property = CssPosition*/
const Value V22403_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22403_295 = {S17, 0, &V22403_295_57192, &P22403_294};	/*Property = CssText*/
const Value V22403_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22403_297 = {S18, 0, &V22403_297_57195, &P22403_295};	/*Property = CssTransforms*/
const Value V22403_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22403_241 = {S19, 0, &V22403_241_55799, &P22403_297};	/*Property = CssTransitions*/
const Value V22403_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22403_298 = {S20, 0, &V22403_298_57206, &P22403_241};	/*Property = CssUI*/
const Value V22403_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22403_300 = {S21, 0, &V22403_300_57216, &P22403_298};	/*Property = DataSet*/
const Value V22403_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22403_301 = {S22, 0, &V22403_301_57218, &P22403_300};	/*Property = DataUrl*/
const Value V22403_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22403_302 = {S23, 0, &V22403_302_57222, &P22403_301};	/*Property = DeviceOrientation*/
const Value V22403_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22403_304 = {S24, 0, &V22403_304_57227, &P22403_302};	/*Property = FileReader*/
const Value V22403_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22403_305 = {S25, 0, &V22403_305_57231, &P22403_304};	/*Property = FileSaver*/
const Value V22403_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22403_306 = {S26, 0, &V22403_306_57237, &P22403_305};	/*Property = FileWriter*/
const Value V22403_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22403_307 = {S27, 0, &V22403_307_57239, &P22403_306};	/*Property = FormData*/
const Value V22403_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22403_309 = {S28, 0, &V22403_309_57246, &P22403_307};	/*Property = Fullscreen*/
const Value V22403_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22403_310 = {S29, 0, &V22403_310_57248, &P22403_309};	/*Property = GeoLocation*/
const Value V22403_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22403_312 = {S30, 0, &V22403_312_57255, &P22403_310};	/*Property = History*/
const Value V22403_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22403_314 = {S31, 0, &V22403_314_57261, &P22403_312};	/*Property = Html5*/
const Value V22403_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22403_313 = {S32, 0, &V22403_313_57258, &P22403_314};	/*Property = Html-Media-Capture*/
const Value V22403_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22403_315 = {S33, 0, &V22403_315_57264, &P22403_313};	/*Property = Iframe*/
const Value V22403_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22403_316 = {S34, 0, &V22403_316_57266, &P22403_315};	/*Property = IndexedDB*/
const Value V22403_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22403_317 = {S35, 0, &V22403_317_57269, &P22403_316};	/*Property = Json*/
const Value V22403_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22403_15 = {S37, 0, &V22403_15_28, &P22403_317};	/*Property = LayoutEngine*/
const Value V22403_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22403_344 = {S38, 0, &V22403_344_57272, &P22403_15};	/*Property = Masking*/
const Value V22403_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22403_320 = {S39, 0, &V22403_320_57290, &P22403_344};	/*Property = PostMessage*/
const Value V22403_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22403_321 = {S40, 0, &V22403_321_57293, &P22403_320};	/*Property = Progress*/
const Value V22403_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22403_322 = {S41, 0, &V22403_322_57296, &P22403_321};	/*Property = Prompts*/
const Value V22403_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22403_323 = {S42, 0, &V22403_323_57299, &P22403_322};	/*Property = Selector*/
const Value V22403_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22403_324 = {S43, 0, &V22403_324_57306, &P22403_323};	/*Property = Svg*/
const Value V22403_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22403_327 = {S44, 0, &V22403_327_57315, &P22403_324};	/*Property = TouchEvents*/
const Value V22403_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22403_328 = {S45, 0, &V22403_328_57318, &P22403_327};	/*Property = Track*/
const Value V22403_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22403_329 = {S46, 0, &V22403_329_57323, &P22403_328};	/*Property = Video*/
const Value V22403_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22403_330 = {S47, 0, &V22403_330_57326, &P22403_329};	/*Property = Viewport*/
const Value V22403_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22403_332 = {S48, 0, &V22403_332_57339, &P22403_330};	/*Property = WebWorkers*/
const Value V22403_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22403_333 = {S49, 0, &V22403_333_57342, &P22403_332};	/*Property = Xhr2*/
const Profile P22403 = {"22403", &P22403_333};	/*Profile ID - 22403*/
const Value V18195_1_1 = {S2, NULL};	/*Value = True*/
const Property P18195_1 = {S50, 0, &V18195_1_1, NULL};	/*Property = IsMobile*/
const Value V18195_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18195_5 = {S52, 0, &V18195_5_1277, &P18195_1};	/*Property = ScreenPixelsHeight*/
const Value V18195_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18195_4 = {S54, 0, &V18195_4_2336, &P18195_5};	/*Property = ScreenPixelsWidth*/
const Profile P18195 = {"18195", &P18195_4};	/*Profile ID - 18195*/
const Value V21896_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21896_279 = {S1, 0, &V21896_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21896_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21896_282 = {S3, 0, &V21896_282_57154, &P21896_279};	/*Property = BlobBuilder*/
const Value V21896_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21896_350 = {S4, 0, &V21896_350_57444, &P21896_282};	/*Property = Canvas*/
const Value V21896_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21896_283 = {S5, 0, &V21896_283_57159, &P21896_350};	/*Property = CssBackground*/
const Value V21896_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21896_284 = {S6, 0, &V21896_284_57163, &P21896_283};	/*Property = CssBorderImage*/
const Value V21896_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21896_285 = {S7, 0, &V21896_285_57166, &P21896_284};	/*Property = CssCanvas*/
const Value V21896_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21896_286 = {S8, 0, &V21896_286_57168, &P21896_285};	/*Property = CssColor*/
const Value V21896_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21896_237 = {S9, 0, &V21896_237_55786, &P21896_286};	/*Property = CssColumn*/
const Value V21896_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21896_287 = {S10, 0, &V21896_287_57172, &P21896_237};	/*Property = CssFlexbox*/
const Value V21896_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21896_288 = {S11, 0, &V21896_288_57175, &P21896_287};	/*Property = CssFont*/
const Value V21896_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21896_289 = {S12, 0, &V21896_289_57759, &P21896_288};	/*Property = CssImages*/
const Value V21896_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21896_291 = {S13, 0, &V21896_291_57181, &P21896_289};	/*Property = CssMediaQueries*/
const Value V21896_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21896_292 = {S14, 0, &V21896_292_57183, &P21896_291};	/*Property = CssMinMax*/
const Value V21896_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21896_293 = {S15, 0, &V21896_293_57187, &P21896_292};	/*Property = CssOverflow*/
const Value V21896_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21896_294 = {S16, 0, &V21896_294_57190, &P21896_293};	/*Property = CssPosition*/
const Value V21896_295_57193 = {S0, NULL};	/*Value = False*/
const Property P21896_295 = {S17, 0, &V21896_295_57193, &P21896_294};	/*Property = CssText*/
const Value V21896_297_57196 = {S0, NULL};	/*Value = False*/
const Property P21896_297 = {S18, 0, &V21896_297_57196, &P21896_295};	/*Property = CssTransforms*/
const Value V21896_241_55800 = {S0, NULL};	/*Value = False*/
const Property P21896_241 = {S19, 0, &V21896_241_55800, &P21896_297};	/*Property = CssTransitions*/
const Value V21896_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21896_298 = {S20, 0, &V21896_298_57206, &P21896_241};	/*Property = CssUI*/
const Value V21896_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21896_300 = {S21, 0, &V21896_300_57216, &P21896_298};	/*Property = DataSet*/
const Value V21896_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21896_301 = {S22, 0, &V21896_301_57218, &P21896_300};	/*Property = DataUrl*/
const Value V21896_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21896_302 = {S23, 0, &V21896_302_57222, &P21896_301};	/*Property = DeviceOrientation*/
const Value V21896_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21896_304 = {S24, 0, &V21896_304_57228, &P21896_302};	/*Property = FileReader*/
const Value V21896_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21896_305 = {S25, 0, &V21896_305_57231, &P21896_304};	/*Property = FileSaver*/
const Value V21896_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21896_306 = {S26, 0, &V21896_306_57237, &P21896_305};	/*Property = FileWriter*/
const Value V21896_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21896_307 = {S27, 0, &V21896_307_57240, &P21896_306};	/*Property = FormData*/
const Value V21896_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21896_309 = {S28, 0, &V21896_309_57246, &P21896_307};	/*Property = Fullscreen*/
const Value V21896_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21896_310 = {S29, 0, &V21896_310_57248, &P21896_309};	/*Property = GeoLocation*/
const Value V21896_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21896_312 = {S30, 0, &V21896_312_57255, &P21896_310};	/*Property = History*/
const Value V21896_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21896_314 = {S31, 0, &V21896_314_57261, &P21896_312};	/*Property = Html5*/
const Value V21896_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21896_313 = {S32, 0, &V21896_313_57258, &P21896_314};	/*Property = Html-Media-Capture*/
const Value V21896_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21896_315 = {S33, 0, &V21896_315_57264, &P21896_313};	/*Property = Iframe*/
const Value V21896_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21896_316 = {S34, 0, &V21896_316_57267, &P21896_315};	/*Property = IndexedDB*/
const Value V21896_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21896_317 = {S35, 0, &V21896_317_57269, &P21896_316};	/*Property = Json*/
const Value V21896_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21896_15 = {S37, 0, &V21896_15_28, &P21896_317};	/*Property = LayoutEngine*/
const Value V21896_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21896_344 = {S38, 0, &V21896_344_57272, &P21896_15};	/*Property = Masking*/
const Value V21896_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21896_320 = {S39, 0, &V21896_320_57290, &P21896_344};	/*Property = PostMessage*/
const Value V21896_321_57294 = {S0, NULL};	/*Value = False*/
const Property P21896_321 = {S40, 0, &V21896_321_57294, &P21896_320};	/*Property = Progress*/
const Value V21896_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21896_322 = {S41, 0, &V21896_322_57296, &P21896_321};	/*Property = Prompts*/
const Value V21896_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21896_323 = {S42, 0, &V21896_323_57299, &P21896_322};	/*Property = Selector*/
const Value V21896_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21896_324 = {S43, 0, &V21896_324_57306, &P21896_323};	/*Property = Svg*/
const Value V21896_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21896_327 = {S44, 0, &V21896_327_57315, &P21896_324};	/*Property = TouchEvents*/
const Value V21896_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21896_328 = {S45, 0, &V21896_328_57318, &P21896_327};	/*Property = Track*/
const Value V21896_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21896_329 = {S46, 0, &V21896_329_57323, &P21896_328};	/*Property = Video*/
const Value V21896_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21896_330 = {S47, 0, &V21896_330_57326, &P21896_329};	/*Property = Viewport*/
const Value V21896_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21896_332 = {S48, 0, &V21896_332_57339, &P21896_330};	/*Property = WebWorkers*/
const Value V21896_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21896_333 = {S49, 0, &V21896_333_57342, &P21896_332};	/*Property = Xhr2*/
const Profile P21896 = {"21896", &P21896_333};	/*Profile ID - 21896*/
const Profile P6596 = {"6596", NULL};	/*Profile ID - 6596*/
const Value V18033_1_1 = {S2, NULL};	/*Value = True*/
const Property P18033_1 = {S50, 0, &V18033_1_1, NULL};	/*Property = IsMobile*/
const Value V18033_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18033_5 = {S52, 0, &V18033_5_1277, &P18033_1};	/*Property = ScreenPixelsHeight*/
const Value V18033_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18033_4 = {S54, 0, &V18033_4_1279, &P18033_5};	/*Property = ScreenPixelsWidth*/
const Profile P18033 = {"18033", &P18033_4};	/*Profile ID - 18033*/
const Profile P22192 = {"22192", NULL};	/*Profile ID - 22192*/
const Value V22449_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22449_279 = {S1, 0, &V22449_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22449_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22449_282 = {S3, 0, &V22449_282_57154, &P22449_279};	/*Property = BlobBuilder*/
const Value V22449_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22449_350 = {S4, 0, &V22449_350_57444, &P22449_282};	/*Property = Canvas*/
const Value V22449_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22449_283 = {S5, 0, &V22449_283_57159, &P22449_350};	/*Property = CssBackground*/
const Value V22449_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22449_284 = {S6, 0, &V22449_284_57163, &P22449_283};	/*Property = CssBorderImage*/
const Value V22449_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22449_285 = {S7, 0, &V22449_285_57166, &P22449_284};	/*Property = CssCanvas*/
const Value V22449_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22449_286 = {S8, 0, &V22449_286_57168, &P22449_285};	/*Property = CssColor*/
const Value V22449_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22449_237 = {S9, 0, &V22449_237_55784, &P22449_286};	/*Property = CssColumn*/
const Value V22449_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22449_287 = {S10, 0, &V22449_287_57172, &P22449_237};	/*Property = CssFlexbox*/
const Value V22449_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22449_288 = {S11, 0, &V22449_288_57175, &P22449_287};	/*Property = CssFont*/
const Value V22449_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22449_289 = {S12, 0, &V22449_289_57759, &P22449_288};	/*Property = CssImages*/
const Value V22449_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22449_291 = {S13, 0, &V22449_291_57180, &P22449_289};	/*Property = CssMediaQueries*/
const Value V22449_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22449_292 = {S14, 0, &V22449_292_57183, &P22449_291};	/*Property = CssMinMax*/
const Value V22449_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22449_293 = {S15, 0, &V22449_293_57187, &P22449_292};	/*Property = CssOverflow*/
const Value V22449_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22449_294 = {S16, 0, &V22449_294_57190, &P22449_293};	/*Property = CssPosition*/
const Value V22449_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22449_295 = {S17, 0, &V22449_295_57192, &P22449_294};	/*Property = CssText*/
const Value V22449_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22449_297 = {S18, 0, &V22449_297_57195, &P22449_295};	/*Property = CssTransforms*/
const Value V22449_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22449_241 = {S19, 0, &V22449_241_55799, &P22449_297};	/*Property = CssTransitions*/
const Value V22449_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22449_298 = {S20, 0, &V22449_298_57206, &P22449_241};	/*Property = CssUI*/
const Value V22449_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22449_300 = {S21, 0, &V22449_300_57216, &P22449_298};	/*Property = DataSet*/
const Value V22449_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22449_301 = {S22, 0, &V22449_301_57218, &P22449_300};	/*Property = DataUrl*/
const Value V22449_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22449_302 = {S23, 0, &V22449_302_57222, &P22449_301};	/*Property = DeviceOrientation*/
const Value V22449_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22449_304 = {S24, 0, &V22449_304_57227, &P22449_302};	/*Property = FileReader*/
const Value V22449_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22449_305 = {S25, 0, &V22449_305_57231, &P22449_304};	/*Property = FileSaver*/
const Value V22449_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22449_306 = {S26, 0, &V22449_306_57237, &P22449_305};	/*Property = FileWriter*/
const Value V22449_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22449_307 = {S27, 0, &V22449_307_57239, &P22449_306};	/*Property = FormData*/
const Value V22449_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22449_309 = {S28, 0, &V22449_309_57246, &P22449_307};	/*Property = Fullscreen*/
const Value V22449_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22449_310 = {S29, 0, &V22449_310_57248, &P22449_309};	/*Property = GeoLocation*/
const Value V22449_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22449_312 = {S30, 0, &V22449_312_57255, &P22449_310};	/*Property = History*/
const Value V22449_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22449_314 = {S31, 0, &V22449_314_57261, &P22449_312};	/*Property = Html5*/
const Value V22449_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22449_313 = {S32, 0, &V22449_313_57258, &P22449_314};	/*Property = Html-Media-Capture*/
const Value V22449_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22449_315 = {S33, 0, &V22449_315_57264, &P22449_313};	/*Property = Iframe*/
const Value V22449_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22449_316 = {S34, 0, &V22449_316_57266, &P22449_315};	/*Property = IndexedDB*/
const Value V22449_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22449_317 = {S35, 0, &V22449_317_57269, &P22449_316};	/*Property = Json*/
const Value V22449_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22449_15 = {S37, 0, &V22449_15_28, &P22449_317};	/*Property = LayoutEngine*/
const Value V22449_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22449_344 = {S38, 0, &V22449_344_57272, &P22449_15};	/*Property = Masking*/
const Value V22449_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22449_320 = {S39, 0, &V22449_320_57290, &P22449_344};	/*Property = PostMessage*/
const Value V22449_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22449_321 = {S40, 0, &V22449_321_57293, &P22449_320};	/*Property = Progress*/
const Value V22449_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22449_322 = {S41, 0, &V22449_322_57296, &P22449_321};	/*Property = Prompts*/
const Value V22449_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22449_323 = {S42, 0, &V22449_323_57299, &P22449_322};	/*Property = Selector*/
const Value V22449_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22449_324 = {S43, 0, &V22449_324_57306, &P22449_323};	/*Property = Svg*/
const Value V22449_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22449_327 = {S44, 0, &V22449_327_57315, &P22449_324};	/*Property = TouchEvents*/
const Value V22449_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22449_328 = {S45, 0, &V22449_328_57318, &P22449_327};	/*Property = Track*/
const Value V22449_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22449_329 = {S46, 0, &V22449_329_57323, &P22449_328};	/*Property = Video*/
const Value V22449_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22449_330 = {S47, 0, &V22449_330_57326, &P22449_329};	/*Property = Viewport*/
const Value V22449_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22449_332 = {S48, 0, &V22449_332_57339, &P22449_330};	/*Property = WebWorkers*/
const Value V22449_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22449_333 = {S49, 0, &V22449_333_57342, &P22449_332};	/*Property = Xhr2*/
const Profile P22449 = {"22449", &P22449_333};	/*Profile ID - 22449*/
const Value V16044_1_1 = {S2, NULL};	/*Value = True*/
const Property P16044_1 = {S50, 0, &V16044_1_1, NULL};	/*Property = IsMobile*/
const Value V16044_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P16044_5 = {S52, 0, &V16044_5_1458, &P16044_1};	/*Property = ScreenPixelsHeight*/
const Value V16044_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P16044_4 = {S54, 0, &V16044_4_37617, &P16044_5};	/*Property = ScreenPixelsWidth*/
const Profile P16044 = {"16044", &P16044_4};	/*Profile ID - 16044*/
const Value V17926_1_1 = {S2, NULL};	/*Value = True*/
const Property P17926_1 = {S50, 0, &V17926_1_1, NULL};	/*Property = IsMobile*/
const Value V17926_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P17926_5 = {S52, 0, &V17926_5_1458, &P17926_1};	/*Property = ScreenPixelsHeight*/
const Value V17926_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P17926_4 = {S54, 0, &V17926_4_1460, &P17926_5};	/*Property = ScreenPixelsWidth*/
const Profile P17926 = {"17926", &P17926_4};	/*Profile ID - 17926*/
const Value V17158_1_1 = {S2, NULL};	/*Value = True*/
const Property P17158_1 = {S50, 0, &V17158_1_1, NULL};	/*Property = IsMobile*/
const Value V17158_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17158_5 = {S52, 0, &V17158_5_1227, &P17158_1};	/*Property = ScreenPixelsHeight*/
const Value V17158_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17158_4 = {S54, 0, &V17158_4_1229, &P17158_5};	/*Property = ScreenPixelsWidth*/
const Profile P17158 = {"17158", &P17158_4};	/*Profile ID - 17158*/
const Value V21897_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21897_279 = {S1, 0, &V21897_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21897_282_57153 = {S2, NULL};	/*Value = True*/
const Property P21897_282 = {S3, 0, &V21897_282_57153, &P21897_279};	/*Property = BlobBuilder*/
const Value V21897_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21897_350 = {S4, 0, &V21897_350_57444, &P21897_282};	/*Property = Canvas*/
const Value V21897_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21897_283 = {S5, 0, &V21897_283_57159, &P21897_350};	/*Property = CssBackground*/
const Value V21897_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21897_284 = {S6, 0, &V21897_284_57163, &P21897_283};	/*Property = CssBorderImage*/
const Value V21897_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21897_285 = {S7, 0, &V21897_285_57166, &P21897_284};	/*Property = CssCanvas*/
const Value V21897_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21897_286 = {S8, 0, &V21897_286_57168, &P21897_285};	/*Property = CssColor*/
const Value V21897_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21897_237 = {S9, 0, &V21897_237_55784, &P21897_286};	/*Property = CssColumn*/
const Value V21897_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21897_287 = {S10, 0, &V21897_287_57172, &P21897_237};	/*Property = CssFlexbox*/
const Value V21897_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21897_288 = {S11, 0, &V21897_288_57175, &P21897_287};	/*Property = CssFont*/
const Value V21897_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21897_289 = {S12, 0, &V21897_289_57759, &P21897_288};	/*Property = CssImages*/
const Value V21897_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21897_291 = {S13, 0, &V21897_291_57180, &P21897_289};	/*Property = CssMediaQueries*/
const Value V21897_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21897_292 = {S14, 0, &V21897_292_57183, &P21897_291};	/*Property = CssMinMax*/
const Value V21897_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21897_293 = {S15, 0, &V21897_293_57187, &P21897_292};	/*Property = CssOverflow*/
const Value V21897_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21897_294 = {S16, 0, &V21897_294_57189, &P21897_293};	/*Property = CssPosition*/
const Value V21897_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21897_295 = {S17, 0, &V21897_295_57192, &P21897_294};	/*Property = CssText*/
const Value V21897_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21897_297 = {S18, 0, &V21897_297_57195, &P21897_295};	/*Property = CssTransforms*/
const Value V21897_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21897_241 = {S19, 0, &V21897_241_55799, &P21897_297};	/*Property = CssTransitions*/
const Value V21897_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21897_298 = {S20, 0, &V21897_298_57206, &P21897_241};	/*Property = CssUI*/
const Value V21897_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21897_300 = {S21, 0, &V21897_300_57216, &P21897_298};	/*Property = DataSet*/
const Value V21897_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21897_301 = {S22, 0, &V21897_301_57218, &P21897_300};	/*Property = DataUrl*/
const Value V21897_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21897_302 = {S23, 0, &V21897_302_57222, &P21897_301};	/*Property = DeviceOrientation*/
const Value V21897_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21897_304 = {S24, 0, &V21897_304_57227, &P21897_302};	/*Property = FileReader*/
const Value V21897_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21897_305 = {S25, 0, &V21897_305_57231, &P21897_304};	/*Property = FileSaver*/
const Value V21897_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21897_306 = {S26, 0, &V21897_306_57237, &P21897_305};	/*Property = FileWriter*/
const Value V21897_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21897_307 = {S27, 0, &V21897_307_57239, &P21897_306};	/*Property = FormData*/
const Value V21897_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21897_309 = {S28, 0, &V21897_309_57246, &P21897_307};	/*Property = Fullscreen*/
const Value V21897_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21897_310 = {S29, 0, &V21897_310_57248, &P21897_309};	/*Property = GeoLocation*/
const Value V21897_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21897_312 = {S30, 0, &V21897_312_57255, &P21897_310};	/*Property = History*/
const Value V21897_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21897_314 = {S31, 0, &V21897_314_57261, &P21897_312};	/*Property = Html5*/
const Value V21897_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21897_313 = {S32, 0, &V21897_313_57258, &P21897_314};	/*Property = Html-Media-Capture*/
const Value V21897_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21897_315 = {S33, 0, &V21897_315_57264, &P21897_313};	/*Property = Iframe*/
const Value V21897_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21897_316 = {S34, 0, &V21897_316_57266, &P21897_315};	/*Property = IndexedDB*/
const Value V21897_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21897_317 = {S35, 0, &V21897_317_57269, &P21897_316};	/*Property = Json*/
const Value V21897_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21897_15 = {S37, 0, &V21897_15_28, &P21897_317};	/*Property = LayoutEngine*/
const Value V21897_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21897_344 = {S38, 0, &V21897_344_57272, &P21897_15};	/*Property = Masking*/
const Value V21897_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21897_320 = {S39, 0, &V21897_320_57290, &P21897_344};	/*Property = PostMessage*/
const Value V21897_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21897_321 = {S40, 0, &V21897_321_57293, &P21897_320};	/*Property = Progress*/
const Value V21897_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21897_322 = {S41, 0, &V21897_322_57296, &P21897_321};	/*Property = Prompts*/
const Value V21897_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21897_323 = {S42, 0, &V21897_323_57299, &P21897_322};	/*Property = Selector*/
const Value V21897_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21897_324 = {S43, 0, &V21897_324_57306, &P21897_323};	/*Property = Svg*/
const Value V21897_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21897_327 = {S44, 0, &V21897_327_57315, &P21897_324};	/*Property = TouchEvents*/
const Value V21897_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21897_328 = {S45, 0, &V21897_328_57318, &P21897_327};	/*Property = Track*/
const Value V21897_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21897_329 = {S46, 0, &V21897_329_57323, &P21897_328};	/*Property = Video*/
const Value V21897_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21897_330 = {S47, 0, &V21897_330_57326, &P21897_329};	/*Property = Viewport*/
const Value V21897_332_57338 = {S2, NULL};	/*Value = True*/
const Property P21897_332 = {S48, 0, &V21897_332_57338, &P21897_330};	/*Property = WebWorkers*/
const Value V21897_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21897_333 = {S49, 0, &V21897_333_57342, &P21897_332};	/*Property = Xhr2*/
const Profile P21897 = {"21897", &P21897_333};	/*Profile ID - 21897*/
const Value V19789_1_1 = {S2, NULL};	/*Value = True*/
const Property P19789_1 = {S50, 0, &V19789_1_1, NULL};	/*Property = IsMobile*/
const Value V19789_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19789_5 = {S52, 0, &V19789_5_1227, &P19789_1};	/*Property = ScreenPixelsHeight*/
const Value V19789_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19789_4 = {S54, 0, &V19789_4_1229, &P19789_5};	/*Property = ScreenPixelsWidth*/
const Profile P19789 = {"19789", &P19789_4};	/*Profile ID - 19789*/
const Value V16352_1_1 = {S2, NULL};	/*Value = True*/
const Property P16352_1 = {S50, 0, &V16352_1_1, NULL};	/*Property = IsMobile*/
const Value V16352_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16352_5 = {S52, 0, &V16352_5_1227, &P16352_1};	/*Property = ScreenPixelsHeight*/
const Value V16352_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16352_4 = {S54, 0, &V16352_4_1229, &P16352_5};	/*Property = ScreenPixelsWidth*/
const Profile P16352 = {"16352", &P16352_4};	/*Profile ID - 16352*/
const Value V16156_1_1 = {S2, NULL};	/*Value = True*/
const Property P16156_1 = {S50, 0, &V16156_1_1, NULL};	/*Property = IsMobile*/
const Value V16156_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16156_5 = {S52, 0, &V16156_5_1227, &P16156_1};	/*Property = ScreenPixelsHeight*/
const Value V16156_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P16156_4 = {S54, 0, &V16156_4_1460, &P16156_5};	/*Property = ScreenPixelsWidth*/
const Profile P16156 = {"16156", &P16156_4};	/*Profile ID - 16156*/
const Value V17717_1_1 = {S2, NULL};	/*Value = True*/
const Property P17717_1 = {S50, 0, &V17717_1_1, NULL};	/*Property = IsMobile*/
const Value V17717_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17717_5 = {S52, 0, &V17717_5_1227, &P17717_1};	/*Property = ScreenPixelsHeight*/
const Value V17717_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17717_4 = {S54, 0, &V17717_4_1229, &P17717_5};	/*Property = ScreenPixelsWidth*/
const Profile P17717 = {"17717", &P17717_4};	/*Profile ID - 17717*/
const Value V18602_1_1 = {S2, NULL};	/*Value = True*/
const Property P18602_1 = {S50, 0, &V18602_1_1, NULL};	/*Property = IsMobile*/
const Value V18602_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18602_5 = {S52, 0, &V18602_5_1227, &P18602_1};	/*Property = ScreenPixelsHeight*/
const Value V18602_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18602_4 = {S54, 0, &V18602_4_1229, &P18602_5};	/*Property = ScreenPixelsWidth*/
const Profile P18602 = {"18602", &P18602_4};	/*Profile ID - 18602*/
const Value V18085_1_1 = {S2, NULL};	/*Value = True*/
const Property P18085_1 = {S50, 0, &V18085_1_1, NULL};	/*Property = IsMobile*/
const Value V18085_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18085_5 = {S52, 0, &V18085_5_1277, &P18085_1};	/*Property = ScreenPixelsHeight*/
const Value V18085_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18085_4 = {S54, 0, &V18085_4_1279, &P18085_5};	/*Property = ScreenPixelsWidth*/
const Profile P18085 = {"18085", &P18085_4};	/*Profile ID - 18085*/
const Value V22450_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22450_279 = {S1, 0, &V22450_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22450_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22450_282 = {S3, 0, &V22450_282_57154, &P22450_279};	/*Property = BlobBuilder*/
const Value V22450_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22450_350 = {S4, 0, &V22450_350_57444, &P22450_282};	/*Property = Canvas*/
const Value V22450_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22450_283 = {S5, 0, &V22450_283_57159, &P22450_350};	/*Property = CssBackground*/
const Value V22450_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22450_284 = {S6, 0, &V22450_284_57163, &P22450_283};	/*Property = CssBorderImage*/
const Value V22450_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22450_285 = {S7, 0, &V22450_285_57166, &P22450_284};	/*Property = CssCanvas*/
const Value V22450_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22450_286 = {S8, 0, &V22450_286_57168, &P22450_285};	/*Property = CssColor*/
const Value V22450_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22450_237 = {S9, 0, &V22450_237_55784, &P22450_286};	/*Property = CssColumn*/
const Value V22450_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22450_287 = {S10, 0, &V22450_287_57172, &P22450_237};	/*Property = CssFlexbox*/
const Value V22450_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22450_288 = {S11, 0, &V22450_288_57174, &P22450_287};	/*Property = CssFont*/
const Value V22450_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22450_289 = {S12, 0, &V22450_289_57759, &P22450_288};	/*Property = CssImages*/
const Value V22450_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22450_291 = {S13, 0, &V22450_291_57180, &P22450_289};	/*Property = CssMediaQueries*/
const Value V22450_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22450_292 = {S14, 0, &V22450_292_57183, &P22450_291};	/*Property = CssMinMax*/
const Value V22450_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22450_293 = {S15, 0, &V22450_293_57187, &P22450_292};	/*Property = CssOverflow*/
const Value V22450_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22450_294 = {S16, 0, &V22450_294_57189, &P22450_293};	/*Property = CssPosition*/
const Value V22450_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22450_295 = {S17, 0, &V22450_295_57192, &P22450_294};	/*Property = CssText*/
const Value V22450_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22450_297 = {S18, 0, &V22450_297_57195, &P22450_295};	/*Property = CssTransforms*/
const Value V22450_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22450_241 = {S19, 0, &V22450_241_55799, &P22450_297};	/*Property = CssTransitions*/
const Value V22450_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22450_298 = {S20, 0, &V22450_298_57206, &P22450_241};	/*Property = CssUI*/
const Value V22450_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22450_300 = {S21, 0, &V22450_300_57216, &P22450_298};	/*Property = DataSet*/
const Value V22450_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22450_301 = {S22, 0, &V22450_301_57218, &P22450_300};	/*Property = DataUrl*/
const Value V22450_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22450_302 = {S23, 0, &V22450_302_57222, &P22450_301};	/*Property = DeviceOrientation*/
const Value V22450_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22450_304 = {S24, 0, &V22450_304_57227, &P22450_302};	/*Property = FileReader*/
const Value V22450_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22450_305 = {S25, 0, &V22450_305_57231, &P22450_304};	/*Property = FileSaver*/
const Value V22450_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22450_306 = {S26, 0, &V22450_306_57237, &P22450_305};	/*Property = FileWriter*/
const Value V22450_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22450_307 = {S27, 0, &V22450_307_57239, &P22450_306};	/*Property = FormData*/
const Value V22450_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22450_309 = {S28, 0, &V22450_309_57246, &P22450_307};	/*Property = Fullscreen*/
const Value V22450_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22450_310 = {S29, 0, &V22450_310_57248, &P22450_309};	/*Property = GeoLocation*/
const Value V22450_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22450_312 = {S30, 0, &V22450_312_57255, &P22450_310};	/*Property = History*/
const Value V22450_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22450_314 = {S31, 0, &V22450_314_57261, &P22450_312};	/*Property = Html5*/
const Value V22450_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22450_313 = {S32, 0, &V22450_313_57258, &P22450_314};	/*Property = Html-Media-Capture*/
const Value V22450_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22450_315 = {S33, 0, &V22450_315_57264, &P22450_313};	/*Property = Iframe*/
const Value V22450_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22450_316 = {S34, 0, &V22450_316_57266, &P22450_315};	/*Property = IndexedDB*/
const Value V22450_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22450_317 = {S35, 0, &V22450_317_57269, &P22450_316};	/*Property = Json*/
const Value V22450_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22450_15 = {S37, 0, &V22450_15_28, &P22450_317};	/*Property = LayoutEngine*/
const Value V22450_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22450_344 = {S38, 0, &V22450_344_57272, &P22450_15};	/*Property = Masking*/
const Value V22450_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22450_320 = {S39, 0, &V22450_320_57290, &P22450_344};	/*Property = PostMessage*/
const Value V22450_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22450_321 = {S40, 0, &V22450_321_57293, &P22450_320};	/*Property = Progress*/
const Value V22450_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22450_322 = {S41, 0, &V22450_322_57296, &P22450_321};	/*Property = Prompts*/
const Value V22450_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22450_323 = {S42, 0, &V22450_323_57299, &P22450_322};	/*Property = Selector*/
const Value V22450_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22450_324 = {S43, 0, &V22450_324_57306, &P22450_323};	/*Property = Svg*/
const Value V22450_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22450_327 = {S44, 0, &V22450_327_57315, &P22450_324};	/*Property = TouchEvents*/
const Value V22450_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22450_328 = {S45, 0, &V22450_328_57318, &P22450_327};	/*Property = Track*/
const Value V22450_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22450_329 = {S46, 0, &V22450_329_57323, &P22450_328};	/*Property = Video*/
const Value V22450_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22450_330 = {S47, 0, &V22450_330_57326, &P22450_329};	/*Property = Viewport*/
const Value V22450_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22450_332 = {S48, 0, &V22450_332_57339, &P22450_330};	/*Property = WebWorkers*/
const Value V22450_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22450_333 = {S49, 0, &V22450_333_57342, &P22450_332};	/*Property = Xhr2*/
const Profile P22450 = {"22450", &P22450_333};	/*Profile ID - 22450*/
const Value V18157_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_279 = {S1, 0, &V18157_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18157_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_282 = {S3, 0, &V18157_282_57155, &P18157_279};	/*Property = BlobBuilder*/
const Value V18157_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_350 = {S4, 0, &V18157_350_57446, &P18157_282};	/*Property = Canvas*/
const Value V18157_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_283 = {S5, 0, &V18157_283_57161, &P18157_350};	/*Property = CssBackground*/
const Value V18157_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_284 = {S6, 0, &V18157_284_57164, &P18157_283};	/*Property = CssBorderImage*/
const Value V18157_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_285 = {S7, 0, &V18157_285_57167, &P18157_284};	/*Property = CssCanvas*/
const Value V18157_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_286 = {S8, 0, &V18157_286_57170, &P18157_285};	/*Property = CssColor*/
const Value V18157_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_237 = {S9, 0, &V18157_237_55786, &P18157_286};	/*Property = CssColumn*/
const Value V18157_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_287 = {S10, 0, &V18157_287_57173, &P18157_237};	/*Property = CssFlexbox*/
const Value V18157_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_288 = {S11, 0, &V18157_288_57176, &P18157_287};	/*Property = CssFont*/
const Value V18157_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_289 = {S12, 0, &V18157_289_86899, &P18157_288};	/*Property = CssImages*/
const Value V18157_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_291 = {S13, 0, &V18157_291_57182, &P18157_289};	/*Property = CssMediaQueries*/
const Value V18157_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_292 = {S14, 0, &V18157_292_57185, &P18157_291};	/*Property = CssMinMax*/
const Value V18157_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_293 = {S15, 0, &V18157_293_57188, &P18157_292};	/*Property = CssOverflow*/
const Value V18157_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_294 = {S16, 0, &V18157_294_57191, &P18157_293};	/*Property = CssPosition*/
const Value V18157_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_295 = {S17, 0, &V18157_295_57194, &P18157_294};	/*Property = CssText*/
const Value V18157_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_297 = {S18, 0, &V18157_297_57197, &P18157_295};	/*Property = CssTransforms*/
const Value V18157_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_241 = {S19, 0, &V18157_241_55801, &P18157_297};	/*Property = CssTransitions*/
const Value V18157_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_298 = {S20, 0, &V18157_298_57208, &P18157_241};	/*Property = CssUI*/
const Value V18157_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_300 = {S21, 0, &V18157_300_57217, &P18157_298};	/*Property = DataSet*/
const Value V18157_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_301 = {S22, 0, &V18157_301_57220, &P18157_300};	/*Property = DataUrl*/
const Value V18157_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_302 = {S23, 0, &V18157_302_57223, &P18157_301};	/*Property = DeviceOrientation*/
const Value V18157_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_304 = {S24, 0, &V18157_304_57229, &P18157_302};	/*Property = FileReader*/
const Value V18157_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_305 = {S25, 0, &V18157_305_57232, &P18157_304};	/*Property = FileSaver*/
const Value V18157_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_306 = {S26, 0, &V18157_306_57238, &P18157_305};	/*Property = FileWriter*/
const Value V18157_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_307 = {S27, 0, &V18157_307_57241, &P18157_306};	/*Property = FormData*/
const Value V18157_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_309 = {S28, 0, &V18157_309_57247, &P18157_307};	/*Property = Fullscreen*/
const Value V18157_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_310 = {S29, 0, &V18157_310_57250, &P18157_309};	/*Property = GeoLocation*/
const Value V18157_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_312 = {S30, 0, &V18157_312_57256, &P18157_310};	/*Property = History*/
const Value V18157_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_314 = {S31, 0, &V18157_314_57262, &P18157_312};	/*Property = Html5*/
const Value V18157_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_313 = {S32, 0, &V18157_313_57259, &P18157_314};	/*Property = Html-Media-Capture*/
const Value V18157_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_315 = {S33, 0, &V18157_315_57265, &P18157_313};	/*Property = Iframe*/
const Value V18157_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_316 = {S34, 0, &V18157_316_57268, &P18157_315};	/*Property = IndexedDB*/
const Value V18157_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_317 = {S35, 0, &V18157_317_57271, &P18157_316};	/*Property = Json*/
const Value V18157_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_15 = {S37, 0, &V18157_15_72, &P18157_317};	/*Property = LayoutEngine*/
const Value V18157_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_344 = {S38, 0, &V18157_344_57274, &P18157_15};	/*Property = Masking*/
const Value V18157_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_320 = {S39, 0, &V18157_320_57292, &P18157_344};	/*Property = PostMessage*/
const Value V18157_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_321 = {S40, 0, &V18157_321_57295, &P18157_320};	/*Property = Progress*/
const Value V18157_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_322 = {S41, 0, &V18157_322_57298, &P18157_321};	/*Property = Prompts*/
const Value V18157_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_323 = {S42, 0, &V18157_323_57301, &P18157_322};	/*Property = Selector*/
const Value V18157_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_324 = {S43, 0, &V18157_324_57307, &P18157_323};	/*Property = Svg*/
const Value V18157_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_327 = {S44, 0, &V18157_327_57316, &P18157_324};	/*Property = TouchEvents*/
const Value V18157_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_328 = {S45, 0, &V18157_328_57319, &P18157_327};	/*Property = Track*/
const Value V18157_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_329 = {S46, 0, &V18157_329_57325, &P18157_328};	/*Property = Video*/
const Value V18157_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_330 = {S47, 0, &V18157_330_57328, &P18157_329};	/*Property = Viewport*/
const Value V18157_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_332 = {S48, 0, &V18157_332_57340, &P18157_330};	/*Property = WebWorkers*/
const Value V18157_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18157_333 = {S49, 0, &V18157_333_57343, &P18157_332};	/*Property = Xhr2*/
const Profile P18157 = {"18157", &P18157_333};	/*Profile ID - 18157*/
const Value V18006_1_1 = {S2, NULL};	/*Value = True*/
const Property P18006_1 = {S50, 0, &V18006_1_1, NULL};	/*Property = IsMobile*/
const Value V18006_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P18006_5 = {S52, 0, &V18006_5_1458, &P18006_1};	/*Property = ScreenPixelsHeight*/
const Value V18006_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18006_4 = {S54, 0, &V18006_4_1229, &P18006_5};	/*Property = ScreenPixelsWidth*/
const Profile P18006 = {"18006", &P18006_4};	/*Profile ID - 18006*/
const Value V21043_1_1 = {S2, NULL};	/*Value = True*/
const Property P21043_1 = {S50, 0, &V21043_1_1, NULL};	/*Property = IsMobile*/
const Value V21043_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21043_5 = {S52, 0, &V21043_5_1269, &P21043_1};	/*Property = ScreenPixelsHeight*/
const Value V21043_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21043_4 = {S54, 0, &V21043_4_1229, &P21043_5};	/*Property = ScreenPixelsWidth*/
const Profile P21043 = {"21043", &P21043_4};	/*Profile ID - 21043*/
const Value V17948_1_1 = {S2, NULL};	/*Value = True*/
const Property P17948_1 = {S50, 0, &V17948_1_1, NULL};	/*Property = IsMobile*/
const Value V17948_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17948_5 = {S52, 0, &V17948_5_1227, &P17948_1};	/*Property = ScreenPixelsHeight*/
const Value V17948_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17948_4 = {S54, 0, &V17948_4_1229, &P17948_5};	/*Property = ScreenPixelsWidth*/
const Profile P17948 = {"17948", &P17948_4};	/*Profile ID - 17948*/
const Value V18080_1_1 = {S2, NULL};	/*Value = True*/
const Property P18080_1 = {S50, 0, &V18080_1_1, NULL};	/*Property = IsMobile*/
const Value V18080_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18080_5 = {S52, 0, &V18080_5_1277, &P18080_1};	/*Property = ScreenPixelsHeight*/
const Value V18080_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18080_4 = {S54, 0, &V18080_4_1279, &P18080_5};	/*Property = ScreenPixelsWidth*/
const Profile P18080 = {"18080", &P18080_4};	/*Profile ID - 18080*/
const Value V18078_1_1 = {S2, NULL};	/*Value = True*/
const Property P18078_1 = {S50, 0, &V18078_1_1, NULL};	/*Property = IsMobile*/
const Value V18078_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18078_5 = {S52, 0, &V18078_5_1403, &P18078_1};	/*Property = ScreenPixelsHeight*/
const Value V18078_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18078_4 = {S54, 0, &V18078_4_1404, &P18078_5};	/*Property = ScreenPixelsWidth*/
const Profile P18078 = {"18078", &P18078_4};	/*Profile ID - 18078*/
const Value V21196_1_1 = {S2, NULL};	/*Value = True*/
const Property P21196_1 = {S50, 0, &V21196_1_1, NULL};	/*Property = IsMobile*/
const Value V21196_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21196_5 = {S52, 0, &V21196_5_1269, &P21196_1};	/*Property = ScreenPixelsHeight*/
const Value V21196_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21196_4 = {S54, 0, &V21196_4_1229, &P21196_5};	/*Property = ScreenPixelsWidth*/
const Profile P21196 = {"21196", &P21196_4};	/*Profile ID - 21196*/
const Value V16760_1_1 = {S2, NULL};	/*Value = True*/
const Property P16760_1 = {S50, 0, &V16760_1_1, NULL};	/*Property = IsMobile*/
const Value V16760_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16760_5 = {S52, 0, &V16760_5_1222, &P16760_1};	/*Property = ScreenPixelsHeight*/
const Value V16760_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16760_4 = {S54, 0, &V16760_4_1224, &P16760_5};	/*Property = ScreenPixelsWidth*/
const Profile P16760 = {"16760", &P16760_4};	/*Profile ID - 16760*/
const Value V18002_1_1 = {S2, NULL};	/*Value = True*/
const Property P18002_1 = {S50, 0, &V18002_1_1, NULL};	/*Property = IsMobile*/
const Value V18002_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18002_5 = {S52, 0, &V18002_5_1227, &P18002_1};	/*Property = ScreenPixelsHeight*/
const Value V18002_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18002_4 = {S54, 0, &V18002_4_1229, &P18002_5};	/*Property = ScreenPixelsWidth*/
const Profile P18002 = {"18002", &P18002_4};	/*Profile ID - 18002*/
const Value V18049_1_1 = {S2, NULL};	/*Value = True*/
const Property P18049_1 = {S50, 0, &V18049_1_1, NULL};	/*Property = IsMobile*/
const Value V18049_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P18049_5 = {S52, 0, &V18049_5_1458, &P18049_1};	/*Property = ScreenPixelsHeight*/
const Value V18049_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P18049_4 = {S54, 0, &V18049_4_1460, &P18049_5};	/*Property = ScreenPixelsWidth*/
const Profile P18049 = {"18049", &P18049_4};	/*Profile ID - 18049*/
const Value V22255_1_1 = {S2, NULL};	/*Value = True*/
const Property P22255_1 = {S50, 0, &V22255_1_1, NULL};	/*Property = IsMobile*/
const Value V22255_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22255_5 = {S52, 0, &V22255_5_1832, &P22255_1};	/*Property = ScreenPixelsHeight*/
const Value V22255_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22255_4 = {S54, 0, &V22255_4_5645, &P22255_5};	/*Property = ScreenPixelsWidth*/
const Profile P22255 = {"22255", &P22255_4};	/*Profile ID - 22255*/
const Value V18021_1_1 = {S2, NULL};	/*Value = True*/
const Property P18021_1 = {S50, 0, &V18021_1_1, NULL};	/*Property = IsMobile*/
const Value V18021_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18021_5 = {S52, 0, &V18021_5_1227, &P18021_1};	/*Property = ScreenPixelsHeight*/
const Value V18021_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18021_4 = {S54, 0, &V18021_4_1229, &P18021_5};	/*Property = ScreenPixelsWidth*/
const Profile P18021 = {"18021", &P18021_4};	/*Profile ID - 18021*/
const Value V18620_1_1 = {S2, NULL};	/*Value = True*/
const Property P18620_1 = {S50, 0, &V18620_1_1, NULL};	/*Property = IsMobile*/
const Value V18620_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18620_5 = {S52, 0, &V18620_5_1227, &P18620_1};	/*Property = ScreenPixelsHeight*/
const Value V18620_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18620_4 = {S54, 0, &V18620_4_1229, &P18620_5};	/*Property = ScreenPixelsWidth*/
const Profile P18620 = {"18620", &P18620_4};	/*Profile ID - 18620*/
const Value V17955_1_1 = {S2, NULL};	/*Value = True*/
const Property P17955_1 = {S50, 0, &V17955_1_1, NULL};	/*Property = IsMobile*/
const Value V17955_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P17955_5 = {S52, 0, &V17955_5_1458, &P17955_1};	/*Property = ScreenPixelsHeight*/
const Value V17955_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P17955_4 = {S54, 0, &V17955_4_1460, &P17955_5};	/*Property = ScreenPixelsWidth*/
const Profile P17955 = {"17955", &P17955_4};	/*Profile ID - 17955*/
const Value V21576_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_279 = {S1, 0, &V21576_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21576_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_282 = {S3, 0, &V21576_282_57155, &P21576_279};	/*Property = BlobBuilder*/
const Value V21576_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_350 = {S4, 0, &V21576_350_57446, &P21576_282};	/*Property = Canvas*/
const Value V21576_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_283 = {S5, 0, &V21576_283_57161, &P21576_350};	/*Property = CssBackground*/
const Value V21576_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_284 = {S6, 0, &V21576_284_57164, &P21576_283};	/*Property = CssBorderImage*/
const Value V21576_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_285 = {S7, 0, &V21576_285_57167, &P21576_284};	/*Property = CssCanvas*/
const Value V21576_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_286 = {S8, 0, &V21576_286_57170, &P21576_285};	/*Property = CssColor*/
const Value V21576_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_237 = {S9, 0, &V21576_237_55786, &P21576_286};	/*Property = CssColumn*/
const Value V21576_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_287 = {S10, 0, &V21576_287_57173, &P21576_237};	/*Property = CssFlexbox*/
const Value V21576_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_288 = {S11, 0, &V21576_288_57176, &P21576_287};	/*Property = CssFont*/
const Value V21576_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_289 = {S12, 0, &V21576_289_86899, &P21576_288};	/*Property = CssImages*/
const Value V21576_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_291 = {S13, 0, &V21576_291_57182, &P21576_289};	/*Property = CssMediaQueries*/
const Value V21576_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_292 = {S14, 0, &V21576_292_57185, &P21576_291};	/*Property = CssMinMax*/
const Value V21576_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_293 = {S15, 0, &V21576_293_57188, &P21576_292};	/*Property = CssOverflow*/
const Value V21576_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_294 = {S16, 0, &V21576_294_57191, &P21576_293};	/*Property = CssPosition*/
const Value V21576_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_295 = {S17, 0, &V21576_295_57194, &P21576_294};	/*Property = CssText*/
const Value V21576_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_297 = {S18, 0, &V21576_297_57197, &P21576_295};	/*Property = CssTransforms*/
const Value V21576_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_241 = {S19, 0, &V21576_241_55801, &P21576_297};	/*Property = CssTransitions*/
const Value V21576_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_298 = {S20, 0, &V21576_298_57208, &P21576_241};	/*Property = CssUI*/
const Value V21576_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_300 = {S21, 0, &V21576_300_57217, &P21576_298};	/*Property = DataSet*/
const Value V21576_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_301 = {S22, 0, &V21576_301_57220, &P21576_300};	/*Property = DataUrl*/
const Value V21576_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_302 = {S23, 0, &V21576_302_57223, &P21576_301};	/*Property = DeviceOrientation*/
const Value V21576_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_304 = {S24, 0, &V21576_304_57229, &P21576_302};	/*Property = FileReader*/
const Value V21576_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_305 = {S25, 0, &V21576_305_57232, &P21576_304};	/*Property = FileSaver*/
const Value V21576_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_306 = {S26, 0, &V21576_306_57238, &P21576_305};	/*Property = FileWriter*/
const Value V21576_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_307 = {S27, 0, &V21576_307_57241, &P21576_306};	/*Property = FormData*/
const Value V21576_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_309 = {S28, 0, &V21576_309_57247, &P21576_307};	/*Property = Fullscreen*/
const Value V21576_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_310 = {S29, 0, &V21576_310_57250, &P21576_309};	/*Property = GeoLocation*/
const Value V21576_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_312 = {S30, 0, &V21576_312_57256, &P21576_310};	/*Property = History*/
const Value V21576_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_314 = {S31, 0, &V21576_314_57262, &P21576_312};	/*Property = Html5*/
const Value V21576_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_313 = {S32, 0, &V21576_313_57259, &P21576_314};	/*Property = Html-Media-Capture*/
const Value V21576_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_315 = {S33, 0, &V21576_315_57265, &P21576_313};	/*Property = Iframe*/
const Value V21576_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_316 = {S34, 0, &V21576_316_57268, &P21576_315};	/*Property = IndexedDB*/
const Value V21576_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_317 = {S35, 0, &V21576_317_57271, &P21576_316};	/*Property = Json*/
const Value V21576_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21576_15 = {S37, 0, &V21576_15_28, &P21576_317};	/*Property = LayoutEngine*/
const Value V21576_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_344 = {S38, 0, &V21576_344_57274, &P21576_15};	/*Property = Masking*/
const Value V21576_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_320 = {S39, 0, &V21576_320_57292, &P21576_344};	/*Property = PostMessage*/
const Value V21576_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_321 = {S40, 0, &V21576_321_57295, &P21576_320};	/*Property = Progress*/
const Value V21576_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_322 = {S41, 0, &V21576_322_57298, &P21576_321};	/*Property = Prompts*/
const Value V21576_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_323 = {S42, 0, &V21576_323_57301, &P21576_322};	/*Property = Selector*/
const Value V21576_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_324 = {S43, 0, &V21576_324_57307, &P21576_323};	/*Property = Svg*/
const Value V21576_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_327 = {S44, 0, &V21576_327_57316, &P21576_324};	/*Property = TouchEvents*/
const Value V21576_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_328 = {S45, 0, &V21576_328_57319, &P21576_327};	/*Property = Track*/
const Value V21576_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_329 = {S46, 0, &V21576_329_57325, &P21576_328};	/*Property = Video*/
const Value V21576_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_330 = {S47, 0, &V21576_330_57328, &P21576_329};	/*Property = Viewport*/
const Value V21576_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_332 = {S48, 0, &V21576_332_57340, &P21576_330};	/*Property = WebWorkers*/
const Value V21576_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21576_333 = {S49, 0, &V21576_333_57343, &P21576_332};	/*Property = Xhr2*/
const Profile P21576 = {"21576", &P21576_333};	/*Profile ID - 21576*/
const Value V21898_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21898_279 = {S1, 0, &V21898_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21898_282_57153 = {S2, NULL};	/*Value = True*/
const Property P21898_282 = {S3, 0, &V21898_282_57153, &P21898_279};	/*Property = BlobBuilder*/
const Value V21898_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21898_350 = {S4, 0, &V21898_350_57444, &P21898_282};	/*Property = Canvas*/
const Value V21898_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21898_283 = {S5, 0, &V21898_283_57159, &P21898_350};	/*Property = CssBackground*/
const Value V21898_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21898_284 = {S6, 0, &V21898_284_57163, &P21898_283};	/*Property = CssBorderImage*/
const Value V21898_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21898_285 = {S7, 0, &V21898_285_57166, &P21898_284};	/*Property = CssCanvas*/
const Value V21898_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21898_286 = {S8, 0, &V21898_286_57168, &P21898_285};	/*Property = CssColor*/
const Value V21898_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21898_237 = {S9, 0, &V21898_237_55784, &P21898_286};	/*Property = CssColumn*/
const Value V21898_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21898_287 = {S10, 0, &V21898_287_57172, &P21898_237};	/*Property = CssFlexbox*/
const Value V21898_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21898_288 = {S11, 0, &V21898_288_57175, &P21898_287};	/*Property = CssFont*/
const Value V21898_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21898_289 = {S12, 0, &V21898_289_57759, &P21898_288};	/*Property = CssImages*/
const Value V21898_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21898_291 = {S13, 0, &V21898_291_57180, &P21898_289};	/*Property = CssMediaQueries*/
const Value V21898_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21898_292 = {S14, 0, &V21898_292_57183, &P21898_291};	/*Property = CssMinMax*/
const Value V21898_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21898_293 = {S15, 0, &V21898_293_57187, &P21898_292};	/*Property = CssOverflow*/
const Value V21898_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21898_294 = {S16, 0, &V21898_294_57189, &P21898_293};	/*Property = CssPosition*/
const Value V21898_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21898_295 = {S17, 0, &V21898_295_57192, &P21898_294};	/*Property = CssText*/
const Value V21898_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21898_297 = {S18, 0, &V21898_297_57195, &P21898_295};	/*Property = CssTransforms*/
const Value V21898_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21898_241 = {S19, 0, &V21898_241_55799, &P21898_297};	/*Property = CssTransitions*/
const Value V21898_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21898_298 = {S20, 0, &V21898_298_57206, &P21898_241};	/*Property = CssUI*/
const Value V21898_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21898_300 = {S21, 0, &V21898_300_57216, &P21898_298};	/*Property = DataSet*/
const Value V21898_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21898_301 = {S22, 0, &V21898_301_57218, &P21898_300};	/*Property = DataUrl*/
const Value V21898_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21898_302 = {S23, 0, &V21898_302_57222, &P21898_301};	/*Property = DeviceOrientation*/
const Value V21898_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21898_304 = {S24, 0, &V21898_304_57227, &P21898_302};	/*Property = FileReader*/
const Value V21898_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21898_305 = {S25, 0, &V21898_305_57231, &P21898_304};	/*Property = FileSaver*/
const Value V21898_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21898_306 = {S26, 0, &V21898_306_57237, &P21898_305};	/*Property = FileWriter*/
const Value V21898_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21898_307 = {S27, 0, &V21898_307_57239, &P21898_306};	/*Property = FormData*/
const Value V21898_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21898_309 = {S28, 0, &V21898_309_57246, &P21898_307};	/*Property = Fullscreen*/
const Value V21898_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21898_310 = {S29, 0, &V21898_310_57248, &P21898_309};	/*Property = GeoLocation*/
const Value V21898_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21898_312 = {S30, 0, &V21898_312_57255, &P21898_310};	/*Property = History*/
const Value V21898_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21898_314 = {S31, 0, &V21898_314_57261, &P21898_312};	/*Property = Html5*/
const Value V21898_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21898_313 = {S32, 0, &V21898_313_57258, &P21898_314};	/*Property = Html-Media-Capture*/
const Value V21898_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21898_315 = {S33, 0, &V21898_315_57264, &P21898_313};	/*Property = Iframe*/
const Value V21898_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21898_316 = {S34, 0, &V21898_316_57266, &P21898_315};	/*Property = IndexedDB*/
const Value V21898_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21898_317 = {S35, 0, &V21898_317_57269, &P21898_316};	/*Property = Json*/
const Value V21898_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21898_15 = {S37, 0, &V21898_15_28, &P21898_317};	/*Property = LayoutEngine*/
const Value V21898_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21898_344 = {S38, 0, &V21898_344_57272, &P21898_15};	/*Property = Masking*/
const Value V21898_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21898_320 = {S39, 0, &V21898_320_57290, &P21898_344};	/*Property = PostMessage*/
const Value V21898_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21898_321 = {S40, 0, &V21898_321_57293, &P21898_320};	/*Property = Progress*/
const Value V21898_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21898_322 = {S41, 0, &V21898_322_57296, &P21898_321};	/*Property = Prompts*/
const Value V21898_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21898_323 = {S42, 0, &V21898_323_57299, &P21898_322};	/*Property = Selector*/
const Value V21898_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21898_324 = {S43, 0, &V21898_324_57306, &P21898_323};	/*Property = Svg*/
const Value V21898_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21898_327 = {S44, 0, &V21898_327_57315, &P21898_324};	/*Property = TouchEvents*/
const Value V21898_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21898_328 = {S45, 0, &V21898_328_57318, &P21898_327};	/*Property = Track*/
const Value V21898_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21898_329 = {S46, 0, &V21898_329_57323, &P21898_328};	/*Property = Video*/
const Value V21898_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21898_330 = {S47, 0, &V21898_330_57326, &P21898_329};	/*Property = Viewport*/
const Value V21898_332_57338 = {S2, NULL};	/*Value = True*/
const Property P21898_332 = {S48, 0, &V21898_332_57338, &P21898_330};	/*Property = WebWorkers*/
const Value V21898_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21898_333 = {S49, 0, &V21898_333_57342, &P21898_332};	/*Property = Xhr2*/
const Profile P21898 = {"21898", &P21898_333};	/*Profile ID - 21898*/
const Value V17491_1_1 = {S2, NULL};	/*Value = True*/
const Property P17491_1 = {S50, 0, &V17491_1_1, NULL};	/*Property = IsMobile*/
const Value V17491_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P17491_5 = {S52, 0, &V17491_5_1380, &P17491_1};	/*Property = ScreenPixelsHeight*/
const Value V17491_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17491_4 = {S54, 0, &V17491_4_1279, &P17491_5};	/*Property = ScreenPixelsWidth*/
const Profile P17491 = {"17491", &P17491_4};	/*Profile ID - 17491*/
const Value V17750_279_57148 = {S0, NULL};	/*Value = False*/
const Property P17750_279 = {S1, 0, &V17750_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V17750_282_57154 = {S0, NULL};	/*Value = False*/
const Property P17750_282 = {S3, 0, &V17750_282_57154, &P17750_279};	/*Property = BlobBuilder*/
const Value V17750_350_57444 = {S2, NULL};	/*Value = True*/
const Property P17750_350 = {S4, 0, &V17750_350_57444, &P17750_282};	/*Property = Canvas*/
const Value V17750_283_57159 = {S2, NULL};	/*Value = True*/
const Property P17750_283 = {S5, 0, &V17750_283_57159, &P17750_350};	/*Property = CssBackground*/
const Value V17750_284_57163 = {S0, NULL};	/*Value = False*/
const Property P17750_284 = {S6, 0, &V17750_284_57163, &P17750_283};	/*Property = CssBorderImage*/
const Value V17750_285_57166 = {S0, NULL};	/*Value = False*/
const Property P17750_285 = {S7, 0, &V17750_285_57166, &P17750_284};	/*Property = CssCanvas*/
const Value V17750_286_57168 = {S2, NULL};	/*Value = True*/
const Property P17750_286 = {S8, 0, &V17750_286_57168, &P17750_285};	/*Property = CssColor*/
const Value V17750_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17750_237 = {S9, 0, &V17750_237_55786, &P17750_286};	/*Property = CssColumn*/
const Value V17750_287_57172 = {S0, NULL};	/*Value = False*/
const Property P17750_287 = {S10, 0, &V17750_287_57172, &P17750_237};	/*Property = CssFlexbox*/
const Value V17750_288_57175 = {S0, NULL};	/*Value = False*/
const Property P17750_288 = {S11, 0, &V17750_288_57175, &P17750_287};	/*Property = CssFont*/
const Value V17750_289_57759 = {S0, NULL};	/*Value = False*/
const Property P17750_289 = {S12, 0, &V17750_289_57759, &P17750_288};	/*Property = CssImages*/
const Value V17750_291_57181 = {S0, NULL};	/*Value = False*/
const Property P17750_291 = {S13, 0, &V17750_291_57181, &P17750_289};	/*Property = CssMediaQueries*/
const Value V17750_292_57183 = {S2, NULL};	/*Value = True*/
const Property P17750_292 = {S14, 0, &V17750_292_57183, &P17750_291};	/*Property = CssMinMax*/
const Value V17750_293_57187 = {S0, NULL};	/*Value = False*/
const Property P17750_293 = {S15, 0, &V17750_293_57187, &P17750_292};	/*Property = CssOverflow*/
const Value V17750_294_57190 = {S0, NULL};	/*Value = False*/
const Property P17750_294 = {S16, 0, &V17750_294_57190, &P17750_293};	/*Property = CssPosition*/
const Value V17750_295_57192 = {S2, NULL};	/*Value = True*/
const Property P17750_295 = {S17, 0, &V17750_295_57192, &P17750_294};	/*Property = CssText*/
const Value V17750_297_57195 = {S2, NULL};	/*Value = True*/
const Property P17750_297 = {S18, 0, &V17750_297_57195, &P17750_295};	/*Property = CssTransforms*/
const Value V17750_241_55799 = {S2, NULL};	/*Value = True*/
const Property P17750_241 = {S19, 0, &V17750_241_55799, &P17750_297};	/*Property = CssTransitions*/
const Value V17750_298_57206 = {S2, NULL};	/*Value = True*/
const Property P17750_298 = {S20, 0, &V17750_298_57206, &P17750_241};	/*Property = CssUI*/
const Value V17750_300_57216 = {S0, NULL};	/*Value = False*/
const Property P17750_300 = {S21, 0, &V17750_300_57216, &P17750_298};	/*Property = DataSet*/
const Value V17750_301_57218 = {S2, NULL};	/*Value = True*/
const Property P17750_301 = {S22, 0, &V17750_301_57218, &P17750_300};	/*Property = DataUrl*/
const Value V17750_302_57222 = {S0, NULL};	/*Value = False*/
const Property P17750_302 = {S23, 0, &V17750_302_57222, &P17750_301};	/*Property = DeviceOrientation*/
const Value V17750_304_57228 = {S0, NULL};	/*Value = False*/
const Property P17750_304 = {S24, 0, &V17750_304_57228, &P17750_302};	/*Property = FileReader*/
const Value V17750_305_57231 = {S0, NULL};	/*Value = False*/
const Property P17750_305 = {S25, 0, &V17750_305_57231, &P17750_304};	/*Property = FileSaver*/
const Value V17750_306_57237 = {S0, NULL};	/*Value = False*/
const Property P17750_306 = {S26, 0, &V17750_306_57237, &P17750_305};	/*Property = FileWriter*/
const Value V17750_307_57240 = {S0, NULL};	/*Value = False*/
const Property P17750_307 = {S27, 0, &V17750_307_57240, &P17750_306};	/*Property = FormData*/
const Value V17750_309_57246 = {S0, NULL};	/*Value = False*/
const Property P17750_309 = {S28, 0, &V17750_309_57246, &P17750_307};	/*Property = Fullscreen*/
const Value V17750_310_57249 = {S0, NULL};	/*Value = False*/
const Property P17750_310 = {S29, 0, &V17750_310_57249, &P17750_309};	/*Property = GeoLocation*/
const Value V17750_312_57255 = {S0, NULL};	/*Value = False*/
const Property P17750_312 = {S30, 0, &V17750_312_57255, &P17750_310};	/*Property = History*/
const Value V17750_314_57261 = {S0, NULL};	/*Value = False*/
const Property P17750_314 = {S31, 0, &V17750_314_57261, &P17750_312};	/*Property = Html5*/
const Value V17750_313_57258 = {S0, NULL};	/*Value = False*/
const Property P17750_313 = {S32, 0, &V17750_313_57258, &P17750_314};	/*Property = Html-Media-Capture*/
const Value V17750_315_57264 = {S0, NULL};	/*Value = False*/
const Property P17750_315 = {S33, 0, &V17750_315_57264, &P17750_313};	/*Property = Iframe*/
const Value V17750_316_57267 = {S0, NULL};	/*Value = False*/
const Property P17750_316 = {S34, 0, &V17750_316_57267, &P17750_315};	/*Property = IndexedDB*/
const Value V17750_317_57269 = {S2, NULL};	/*Value = True*/
const Property P17750_317 = {S35, 0, &V17750_317_57269, &P17750_316};	/*Property = Json*/
const Value V17750_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P17750_15 = {S37, 0, &V17750_15_72, &P17750_317};	/*Property = LayoutEngine*/
const Value V17750_344_57272 = {S2, NULL};	/*Value = True*/
const Property P17750_344 = {S38, 0, &V17750_344_57272, &P17750_15};	/*Property = Masking*/
const Value V17750_320_57290 = {S2, NULL};	/*Value = True*/
const Property P17750_320 = {S39, 0, &V17750_320_57290, &P17750_344};	/*Property = PostMessage*/
const Value V17750_321_57293 = {S2, NULL};	/*Value = True*/
const Property P17750_321 = {S40, 0, &V17750_321_57293, &P17750_320};	/*Property = Progress*/
const Value V17750_322_57296 = {S2, NULL};	/*Value = True*/
const Property P17750_322 = {S41, 0, &V17750_322_57296, &P17750_321};	/*Property = Prompts*/
const Value V17750_323_57299 = {S2, NULL};	/*Value = True*/
const Property P17750_323 = {S42, 0, &V17750_323_57299, &P17750_322};	/*Property = Selector*/
const Value V17750_324_57306 = {S0, NULL};	/*Value = False*/
const Property P17750_324 = {S43, 0, &V17750_324_57306, &P17750_323};	/*Property = Svg*/
const Value V17750_327_57315 = {S0, NULL};	/*Value = False*/
const Property P17750_327 = {S44, 0, &V17750_327_57315, &P17750_324};	/*Property = TouchEvents*/
const Value V17750_328_57318 = {S0, NULL};	/*Value = False*/
const Property P17750_328 = {S45, 0, &V17750_328_57318, &P17750_327};	/*Property = Track*/
const Value V17750_329_57324 = {S0, NULL};	/*Value = False*/
const Property P17750_329 = {S46, 0, &V17750_329_57324, &P17750_328};	/*Property = Video*/
const Value V17750_330_57326 = {S2, NULL};	/*Value = True*/
const Property P17750_330 = {S47, 0, &V17750_330_57326, &P17750_329};	/*Property = Viewport*/
const Value V17750_332_57339 = {S0, NULL};	/*Value = False*/
const Property P17750_332 = {S48, 0, &V17750_332_57339, &P17750_330};	/*Property = WebWorkers*/
const Value V17750_333_57342 = {S0, NULL};	/*Value = False*/
const Property P17750_333 = {S49, 0, &V17750_333_57342, &P17750_332};	/*Property = Xhr2*/
const Profile P17750 = {"17750", &P17750_333};	/*Profile ID - 17750*/
const Profile P18377 = {"18377", NULL};	/*Profile ID - 18377*/
const Value V16479_1_1 = {S2, NULL};	/*Value = True*/
const Property P16479_1 = {S50, 0, &V16479_1_1, NULL};	/*Property = IsMobile*/
const Value V16479_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16479_5 = {S52, 0, &V16479_5_1277, &P16479_1};	/*Property = ScreenPixelsHeight*/
const Value V16479_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16479_4 = {S54, 0, &V16479_4_1279, &P16479_5};	/*Property = ScreenPixelsWidth*/
const Profile P16479 = {"16479", &P16479_4};	/*Profile ID - 16479*/
const Value V17937_1_1 = {S2, NULL};	/*Value = True*/
const Property P17937_1 = {S50, 0, &V17937_1_1, NULL};	/*Property = IsMobile*/
const Value V17937_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17937_5 = {S52, 0, &V17937_5_1227, &P17937_1};	/*Property = ScreenPixelsHeight*/
const Value V17937_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17937_4 = {S54, 0, &V17937_4_1229, &P17937_5};	/*Property = ScreenPixelsWidth*/
const Profile P17937 = {"17937", &P17937_4};	/*Profile ID - 17937*/
const Value V22525_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22525_279 = {S1, 0, &V22525_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22525_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22525_282 = {S3, 0, &V22525_282_57154, &P22525_279};	/*Property = BlobBuilder*/
const Value V22525_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22525_350 = {S4, 0, &V22525_350_57444, &P22525_282};	/*Property = Canvas*/
const Value V22525_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22525_283 = {S5, 0, &V22525_283_57159, &P22525_350};	/*Property = CssBackground*/
const Value V22525_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22525_284 = {S6, 0, &V22525_284_57163, &P22525_283};	/*Property = CssBorderImage*/
const Value V22525_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22525_285 = {S7, 0, &V22525_285_57166, &P22525_284};	/*Property = CssCanvas*/
const Value V22525_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22525_286 = {S8, 0, &V22525_286_57168, &P22525_285};	/*Property = CssColor*/
const Value V22525_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22525_237 = {S9, 0, &V22525_237_55786, &P22525_286};	/*Property = CssColumn*/
const Value V22525_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22525_287 = {S10, 0, &V22525_287_57172, &P22525_237};	/*Property = CssFlexbox*/
const Value V22525_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22525_288 = {S11, 0, &V22525_288_57176, &P22525_287};	/*Property = CssFont*/
const Value V22525_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22525_289 = {S12, 0, &V22525_289_57759, &P22525_288};	/*Property = CssImages*/
const Value V22525_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22525_291 = {S13, 0, &V22525_291_57182, &P22525_289};	/*Property = CssMediaQueries*/
const Value V22525_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22525_292 = {S14, 0, &V22525_292_57183, &P22525_291};	/*Property = CssMinMax*/
const Value V22525_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22525_293 = {S15, 0, &V22525_293_57187, &P22525_292};	/*Property = CssOverflow*/
const Value V22525_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22525_294 = {S16, 0, &V22525_294_57191, &P22525_293};	/*Property = CssPosition*/
const Value V22525_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22525_295 = {S17, 0, &V22525_295_57192, &P22525_294};	/*Property = CssText*/
const Value V22525_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22525_297 = {S18, 0, &V22525_297_57195, &P22525_295};	/*Property = CssTransforms*/
const Value V22525_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22525_241 = {S19, 0, &V22525_241_55799, &P22525_297};	/*Property = CssTransitions*/
const Value V22525_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22525_298 = {S20, 0, &V22525_298_57206, &P22525_241};	/*Property = CssUI*/
const Value V22525_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22525_300 = {S21, 0, &V22525_300_57216, &P22525_298};	/*Property = DataSet*/
const Value V22525_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22525_301 = {S22, 0, &V22525_301_57218, &P22525_300};	/*Property = DataUrl*/
const Value V22525_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22525_302 = {S23, 0, &V22525_302_57222, &P22525_301};	/*Property = DeviceOrientation*/
const Value V22525_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22525_304 = {S24, 0, &V22525_304_57229, &P22525_302};	/*Property = FileReader*/
const Value V22525_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22525_305 = {S25, 0, &V22525_305_57231, &P22525_304};	/*Property = FileSaver*/
const Value V22525_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22525_306 = {S26, 0, &V22525_306_57237, &P22525_305};	/*Property = FileWriter*/
const Value V22525_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22525_307 = {S27, 0, &V22525_307_57241, &P22525_306};	/*Property = FormData*/
const Value V22525_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22525_309 = {S28, 0, &V22525_309_57246, &P22525_307};	/*Property = Fullscreen*/
const Value V22525_310_57249 = {S0, NULL};	/*Value = False*/
const Property P22525_310 = {S29, 0, &V22525_310_57249, &P22525_309};	/*Property = GeoLocation*/
const Value V22525_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22525_312 = {S30, 0, &V22525_312_57256, &P22525_310};	/*Property = History*/
const Value V22525_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22525_314 = {S31, 0, &V22525_314_57261, &P22525_312};	/*Property = Html5*/
const Value V22525_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22525_313 = {S32, 0, &V22525_313_57258, &P22525_314};	/*Property = Html-Media-Capture*/
const Value V22525_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22525_315 = {S33, 0, &V22525_315_57264, &P22525_313};	/*Property = Iframe*/
const Value V22525_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22525_316 = {S34, 0, &V22525_316_57267, &P22525_315};	/*Property = IndexedDB*/
const Value V22525_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22525_317 = {S35, 0, &V22525_317_57269, &P22525_316};	/*Property = Json*/
const Value V22525_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22525_15 = {S37, 0, &V22525_15_28, &P22525_317};	/*Property = LayoutEngine*/
const Value V22525_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22525_344 = {S38, 0, &V22525_344_57272, &P22525_15};	/*Property = Masking*/
const Value V22525_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22525_320 = {S39, 0, &V22525_320_57290, &P22525_344};	/*Property = PostMessage*/
const Value V22525_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22525_321 = {S40, 0, &V22525_321_57293, &P22525_320};	/*Property = Progress*/
const Value V22525_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22525_322 = {S41, 0, &V22525_322_57296, &P22525_321};	/*Property = Prompts*/
const Value V22525_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22525_323 = {S42, 0, &V22525_323_57299, &P22525_322};	/*Property = Selector*/
const Value V22525_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22525_324 = {S43, 0, &V22525_324_57306, &P22525_323};	/*Property = Svg*/
const Value V22525_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22525_327 = {S44, 0, &V22525_327_57315, &P22525_324};	/*Property = TouchEvents*/
const Value V22525_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22525_328 = {S45, 0, &V22525_328_57318, &P22525_327};	/*Property = Track*/
const Value V22525_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22525_329 = {S46, 0, &V22525_329_57323, &P22525_328};	/*Property = Video*/
const Value V22525_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22525_330 = {S47, 0, &V22525_330_57326, &P22525_329};	/*Property = Viewport*/
const Value V22525_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22525_332 = {S48, 0, &V22525_332_57339, &P22525_330};	/*Property = WebWorkers*/
const Value V22525_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22525_333 = {S49, 0, &V22525_333_57342, &P22525_332};	/*Property = Xhr2*/
const Profile P22525 = {"22525", &P22525_333};	/*Profile ID - 22525*/
const Value V22247_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22247_279 = {S1, 0, &V22247_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22247_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22247_282 = {S3, 0, &V22247_282_57154, &P22247_279};	/*Property = BlobBuilder*/
const Value V22247_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22247_350 = {S4, 0, &V22247_350_57444, &P22247_282};	/*Property = Canvas*/
const Value V22247_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22247_283 = {S5, 0, &V22247_283_57159, &P22247_350};	/*Property = CssBackground*/
const Value V22247_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22247_284 = {S6, 0, &V22247_284_57163, &P22247_283};	/*Property = CssBorderImage*/
const Value V22247_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22247_285 = {S7, 0, &V22247_285_57166, &P22247_284};	/*Property = CssCanvas*/
const Value V22247_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22247_286 = {S8, 0, &V22247_286_57168, &P22247_285};	/*Property = CssColor*/
const Value V22247_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22247_237 = {S9, 0, &V22247_237_55786, &P22247_286};	/*Property = CssColumn*/
const Value V22247_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22247_287 = {S10, 0, &V22247_287_57172, &P22247_237};	/*Property = CssFlexbox*/
const Value V22247_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22247_288 = {S11, 0, &V22247_288_57175, &P22247_287};	/*Property = CssFont*/
const Value V22247_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22247_289 = {S12, 0, &V22247_289_57759, &P22247_288};	/*Property = CssImages*/
const Value V22247_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22247_291 = {S13, 0, &V22247_291_57181, &P22247_289};	/*Property = CssMediaQueries*/
const Value V22247_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22247_292 = {S14, 0, &V22247_292_57183, &P22247_291};	/*Property = CssMinMax*/
const Value V22247_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22247_293 = {S15, 0, &V22247_293_57187, &P22247_292};	/*Property = CssOverflow*/
const Value V22247_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22247_294 = {S16, 0, &V22247_294_57190, &P22247_293};	/*Property = CssPosition*/
const Value V22247_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22247_295 = {S17, 0, &V22247_295_57192, &P22247_294};	/*Property = CssText*/
const Value V22247_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22247_297 = {S18, 0, &V22247_297_57195, &P22247_295};	/*Property = CssTransforms*/
const Value V22247_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22247_241 = {S19, 0, &V22247_241_55799, &P22247_297};	/*Property = CssTransitions*/
const Value V22247_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22247_298 = {S20, 0, &V22247_298_57206, &P22247_241};	/*Property = CssUI*/
const Value V22247_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22247_300 = {S21, 0, &V22247_300_57216, &P22247_298};	/*Property = DataSet*/
const Value V22247_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22247_301 = {S22, 0, &V22247_301_57218, &P22247_300};	/*Property = DataUrl*/
const Value V22247_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22247_302 = {S23, 0, &V22247_302_57222, &P22247_301};	/*Property = DeviceOrientation*/
const Value V22247_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22247_304 = {S24, 0, &V22247_304_57228, &P22247_302};	/*Property = FileReader*/
const Value V22247_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22247_305 = {S25, 0, &V22247_305_57231, &P22247_304};	/*Property = FileSaver*/
const Value V22247_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22247_306 = {S26, 0, &V22247_306_57237, &P22247_305};	/*Property = FileWriter*/
const Value V22247_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22247_307 = {S27, 0, &V22247_307_57240, &P22247_306};	/*Property = FormData*/
const Value V22247_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22247_309 = {S28, 0, &V22247_309_57246, &P22247_307};	/*Property = Fullscreen*/
const Value V22247_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22247_310 = {S29, 0, &V22247_310_57248, &P22247_309};	/*Property = GeoLocation*/
const Value V22247_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22247_312 = {S30, 0, &V22247_312_57255, &P22247_310};	/*Property = History*/
const Value V22247_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22247_314 = {S31, 0, &V22247_314_57261, &P22247_312};	/*Property = Html5*/
const Value V22247_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22247_313 = {S32, 0, &V22247_313_57258, &P22247_314};	/*Property = Html-Media-Capture*/
const Value V22247_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22247_315 = {S33, 0, &V22247_315_57264, &P22247_313};	/*Property = Iframe*/
const Value V22247_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22247_316 = {S34, 0, &V22247_316_57267, &P22247_315};	/*Property = IndexedDB*/
const Value V22247_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22247_317 = {S35, 0, &V22247_317_57269, &P22247_316};	/*Property = Json*/
const Value V22247_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22247_15 = {S37, 0, &V22247_15_72, &P22247_317};	/*Property = LayoutEngine*/
const Value V22247_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22247_344 = {S38, 0, &V22247_344_57272, &P22247_15};	/*Property = Masking*/
const Value V22247_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22247_320 = {S39, 0, &V22247_320_57290, &P22247_344};	/*Property = PostMessage*/
const Value V22247_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22247_321 = {S40, 0, &V22247_321_57293, &P22247_320};	/*Property = Progress*/
const Value V22247_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22247_322 = {S41, 0, &V22247_322_57296, &P22247_321};	/*Property = Prompts*/
const Value V22247_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22247_323 = {S42, 0, &V22247_323_57299, &P22247_322};	/*Property = Selector*/
const Value V22247_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22247_324 = {S43, 0, &V22247_324_57306, &P22247_323};	/*Property = Svg*/
const Value V22247_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22247_327 = {S44, 0, &V22247_327_57315, &P22247_324};	/*Property = TouchEvents*/
const Value V22247_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22247_328 = {S45, 0, &V22247_328_57318, &P22247_327};	/*Property = Track*/
const Value V22247_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22247_329 = {S46, 0, &V22247_329_57323, &P22247_328};	/*Property = Video*/
const Value V22247_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22247_330 = {S47, 0, &V22247_330_57326, &P22247_329};	/*Property = Viewport*/
const Value V22247_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22247_332 = {S48, 0, &V22247_332_57339, &P22247_330};	/*Property = WebWorkers*/
const Value V22247_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22247_333 = {S49, 0, &V22247_333_57342, &P22247_332};	/*Property = Xhr2*/
const Profile P22247 = {"22247", &P22247_333};	/*Profile ID - 22247*/
const Value V15003_1_1 = {S2, NULL};	/*Value = True*/
const Property P15003_1 = {S50, 0, &V15003_1_1, NULL};	/*Property = IsMobile*/
const Value V15003_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P15003_5 = {S52, 0, &V15003_5_1458, &P15003_1};	/*Property = ScreenPixelsHeight*/
const Value V15003_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P15003_4 = {S54, 0, &V15003_4_1460, &P15003_5};	/*Property = ScreenPixelsWidth*/
const Profile P15003 = {"15003", &P15003_4};	/*Profile ID - 15003*/
const Value V17956_1_1 = {S2, NULL};	/*Value = True*/
const Property P17956_1 = {S50, 0, &V17956_1_1, NULL};	/*Property = IsMobile*/
const Value V17956_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17956_5 = {S52, 0, &V17956_5_1227, &P17956_1};	/*Property = ScreenPixelsHeight*/
const Value V17956_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17956_4 = {S54, 0, &V17956_4_1229, &P17956_5};	/*Property = ScreenPixelsWidth*/
const Profile P17956 = {"17956", &P17956_4};	/*Profile ID - 17956*/
const Value V18023_1_1 = {S2, NULL};	/*Value = True*/
const Property P18023_1 = {S50, 0, &V18023_1_1, NULL};	/*Property = IsMobile*/
const Value V18023_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18023_5 = {S52, 0, &V18023_5_1227, &P18023_1};	/*Property = ScreenPixelsHeight*/
const Value V18023_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18023_4 = {S54, 0, &V18023_4_1229, &P18023_5};	/*Property = ScreenPixelsWidth*/
const Profile P18023 = {"18023", &P18023_4};	/*Profile ID - 18023*/
const Value V16237_1_1 = {S2, NULL};	/*Value = True*/
const Property P16237_1 = {S50, 0, &V16237_1_1, NULL};	/*Property = IsMobile*/
const Value V16237_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16237_5 = {S52, 0, &V16237_5_1227, &P16237_1};	/*Property = ScreenPixelsHeight*/
const Value V16237_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16237_4 = {S54, 0, &V16237_4_1229, &P16237_5};	/*Property = ScreenPixelsWidth*/
const Profile P16237 = {"16237", &P16237_4};	/*Profile ID - 16237*/
const Value V18079_1_1 = {S2, NULL};	/*Value = True*/
const Property P18079_1 = {S50, 0, &V18079_1_1, NULL};	/*Property = IsMobile*/
const Value V18079_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18079_5 = {S52, 0, &V18079_5_1403, &P18079_1};	/*Property = ScreenPixelsHeight*/
const Value V18079_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18079_4 = {S54, 0, &V18079_4_1404, &P18079_5};	/*Property = ScreenPixelsWidth*/
const Profile P18079 = {"18079", &P18079_4};	/*Profile ID - 18079*/
const Value V22451_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22451_279 = {S1, 0, &V22451_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22451_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22451_282 = {S3, 0, &V22451_282_57154, &P22451_279};	/*Property = BlobBuilder*/
const Value V22451_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22451_350 = {S4, 0, &V22451_350_57444, &P22451_282};	/*Property = Canvas*/
const Value V22451_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22451_283 = {S5, 0, &V22451_283_57159, &P22451_350};	/*Property = CssBackground*/
const Value V22451_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22451_284 = {S6, 0, &V22451_284_57163, &P22451_283};	/*Property = CssBorderImage*/
const Value V22451_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22451_285 = {S7, 0, &V22451_285_57166, &P22451_284};	/*Property = CssCanvas*/
const Value V22451_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22451_286 = {S8, 0, &V22451_286_57168, &P22451_285};	/*Property = CssColor*/
const Value V22451_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22451_237 = {S9, 0, &V22451_237_55784, &P22451_286};	/*Property = CssColumn*/
const Value V22451_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22451_287 = {S10, 0, &V22451_287_57172, &P22451_237};	/*Property = CssFlexbox*/
const Value V22451_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22451_288 = {S11, 0, &V22451_288_57174, &P22451_287};	/*Property = CssFont*/
const Value V22451_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22451_289 = {S12, 0, &V22451_289_57759, &P22451_288};	/*Property = CssImages*/
const Value V22451_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22451_291 = {S13, 0, &V22451_291_57180, &P22451_289};	/*Property = CssMediaQueries*/
const Value V22451_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22451_292 = {S14, 0, &V22451_292_57183, &P22451_291};	/*Property = CssMinMax*/
const Value V22451_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22451_293 = {S15, 0, &V22451_293_57187, &P22451_292};	/*Property = CssOverflow*/
const Value V22451_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22451_294 = {S16, 0, &V22451_294_57189, &P22451_293};	/*Property = CssPosition*/
const Value V22451_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22451_295 = {S17, 0, &V22451_295_57192, &P22451_294};	/*Property = CssText*/
const Value V22451_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22451_297 = {S18, 0, &V22451_297_57195, &P22451_295};	/*Property = CssTransforms*/
const Value V22451_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22451_241 = {S19, 0, &V22451_241_55799, &P22451_297};	/*Property = CssTransitions*/
const Value V22451_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22451_298 = {S20, 0, &V22451_298_57206, &P22451_241};	/*Property = CssUI*/
const Value V22451_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22451_300 = {S21, 0, &V22451_300_57215, &P22451_298};	/*Property = DataSet*/
const Value V22451_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22451_301 = {S22, 0, &V22451_301_57218, &P22451_300};	/*Property = DataUrl*/
const Value V22451_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22451_302 = {S23, 0, &V22451_302_57222, &P22451_301};	/*Property = DeviceOrientation*/
const Value V22451_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22451_304 = {S24, 0, &V22451_304_57227, &P22451_302};	/*Property = FileReader*/
const Value V22451_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22451_305 = {S25, 0, &V22451_305_57231, &P22451_304};	/*Property = FileSaver*/
const Value V22451_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22451_306 = {S26, 0, &V22451_306_57237, &P22451_305};	/*Property = FileWriter*/
const Value V22451_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22451_307 = {S27, 0, &V22451_307_57239, &P22451_306};	/*Property = FormData*/
const Value V22451_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22451_309 = {S28, 0, &V22451_309_57246, &P22451_307};	/*Property = Fullscreen*/
const Value V22451_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22451_310 = {S29, 0, &V22451_310_57248, &P22451_309};	/*Property = GeoLocation*/
const Value V22451_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22451_312 = {S30, 0, &V22451_312_57255, &P22451_310};	/*Property = History*/
const Value V22451_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22451_314 = {S31, 0, &V22451_314_57260, &P22451_312};	/*Property = Html5*/
const Value V22451_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22451_313 = {S32, 0, &V22451_313_57258, &P22451_314};	/*Property = Html-Media-Capture*/
const Value V22451_315_57263 = {S2, NULL};	/*Value = True*/
const Property P22451_315 = {S33, 0, &V22451_315_57263, &P22451_313};	/*Property = Iframe*/
const Value V22451_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22451_316 = {S34, 0, &V22451_316_57266, &P22451_315};	/*Property = IndexedDB*/
const Value V22451_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22451_317 = {S35, 0, &V22451_317_57269, &P22451_316};	/*Property = Json*/
const Value V22451_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22451_15 = {S37, 0, &V22451_15_28, &P22451_317};	/*Property = LayoutEngine*/
const Value V22451_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22451_344 = {S38, 0, &V22451_344_57272, &P22451_15};	/*Property = Masking*/
const Value V22451_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22451_320 = {S39, 0, &V22451_320_57290, &P22451_344};	/*Property = PostMessage*/
const Value V22451_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22451_321 = {S40, 0, &V22451_321_57293, &P22451_320};	/*Property = Progress*/
const Value V22451_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22451_322 = {S41, 0, &V22451_322_57296, &P22451_321};	/*Property = Prompts*/
const Value V22451_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22451_323 = {S42, 0, &V22451_323_57299, &P22451_322};	/*Property = Selector*/
const Value V22451_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22451_324 = {S43, 0, &V22451_324_57305, &P22451_323};	/*Property = Svg*/
const Value V22451_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22451_327 = {S44, 0, &V22451_327_57315, &P22451_324};	/*Property = TouchEvents*/
const Value V22451_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22451_328 = {S45, 0, &V22451_328_57318, &P22451_327};	/*Property = Track*/
const Value V22451_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22451_329 = {S46, 0, &V22451_329_57323, &P22451_328};	/*Property = Video*/
const Value V22451_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22451_330 = {S47, 0, &V22451_330_57326, &P22451_329};	/*Property = Viewport*/
const Value V22451_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22451_332 = {S48, 0, &V22451_332_57339, &P22451_330};	/*Property = WebWorkers*/
const Value V22451_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22451_333 = {S49, 0, &V22451_333_57342, &P22451_332};	/*Property = Xhr2*/
const Profile P22451 = {"22451", &P22451_333};	/*Profile ID - 22451*/
const Value V13135_1_1 = {S2, NULL};	/*Value = True*/
const Property P13135_1 = {S50, 0, &V13135_1_1, NULL};	/*Property = IsMobile*/
const Value V13135_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13135_5 = {S52, 0, &V13135_5_1222, &P13135_1};	/*Property = ScreenPixelsHeight*/
const Value V13135_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13135_4 = {S54, 0, &V13135_4_1224, &P13135_5};	/*Property = ScreenPixelsWidth*/
const Profile P13135 = {"13135", &P13135_4};	/*Profile ID - 13135*/
const Value V18048_1_1 = {S2, NULL};	/*Value = True*/
const Property P18048_1 = {S50, 0, &V18048_1_1, NULL};	/*Property = IsMobile*/
const Value V18048_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18048_5 = {S52, 0, &V18048_5_1403, &P18048_1};	/*Property = ScreenPixelsHeight*/
const Value V18048_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18048_4 = {S54, 0, &V18048_4_1404, &P18048_5};	/*Property = ScreenPixelsWidth*/
const Profile P18048 = {"18048", &P18048_4};	/*Profile ID - 18048*/
const Value V18017_1_1 = {S2, NULL};	/*Value = True*/
const Property P18017_1 = {S50, 0, &V18017_1_1, NULL};	/*Property = IsMobile*/
const Value V18017_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P18017_5 = {S52, 0, &V18017_5_1458, &P18017_1};	/*Property = ScreenPixelsHeight*/
const Value V18017_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P18017_4 = {S54, 0, &V18017_4_1460, &P18017_5};	/*Property = ScreenPixelsWidth*/
const Profile P18017 = {"18017", &P18017_4};	/*Profile ID - 18017*/
const Value V18039_1_1 = {S2, NULL};	/*Value = True*/
const Property P18039_1 = {S50, 0, &V18039_1_1, NULL};	/*Property = IsMobile*/
const Value V18039_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18039_5 = {S52, 0, &V18039_5_1227, &P18039_1};	/*Property = ScreenPixelsHeight*/
const Value V18039_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18039_4 = {S54, 0, &V18039_4_1229, &P18039_5};	/*Property = ScreenPixelsWidth*/
const Profile P18039 = {"18039", &P18039_4};	/*Profile ID - 18039*/
const Value V22550_1_1 = {S2, NULL};	/*Value = True*/
const Property P22550_1 = {S50, 0, &V22550_1_1, NULL};	/*Property = IsMobile*/
const Value V22550_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22550_5 = {S52, 0, &V22550_5_1277, &P22550_1};	/*Property = ScreenPixelsHeight*/
const Value V22550_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22550_4 = {S54, 0, &V22550_4_1279, &P22550_5};	/*Property = ScreenPixelsWidth*/
const Profile P22550 = {"22550", &P22550_4};	/*Profile ID - 22550*/
const Value V16552_1_1 = {S2, NULL};	/*Value = True*/
const Property P16552_1 = {S50, 0, &V16552_1_1, NULL};	/*Property = IsMobile*/
const Value V16552_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16552_5 = {S52, 0, &V16552_5_1277, &P16552_1};	/*Property = ScreenPixelsHeight*/
const Value V16552_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16552_4 = {S54, 0, &V16552_4_1279, &P16552_5};	/*Property = ScreenPixelsWidth*/
const Profile P16552 = {"16552", &P16552_4};	/*Profile ID - 16552*/
const Value V19780_1_2 = {S0, NULL};	/*Value = False*/
const Property P19780_1 = {S50, 0, &V19780_1_2, NULL};	/*Property = IsMobile*/
const Value V19780_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P19780_5 = {S52, 0, &V19780_5_37, &P19780_1};	/*Property = ScreenPixelsHeight*/
const Value V19780_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P19780_4 = {S54, 0, &V19780_4_36, &P19780_5};	/*Property = ScreenPixelsWidth*/
const Profile P19780 = {"19780", &P19780_4};	/*Profile ID - 19780*/
const Value V18074_1_1 = {S2, NULL};	/*Value = True*/
const Property P18074_1 = {S50, 0, &V18074_1_1, NULL};	/*Property = IsMobile*/
const Value V18074_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18074_5 = {S52, 0, &V18074_5_1227, &P18074_1};	/*Property = ScreenPixelsHeight*/
const Value V18074_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18074_4 = {S54, 0, &V18074_4_1229, &P18074_5};	/*Property = ScreenPixelsWidth*/
const Profile P18074 = {"18074", &P18074_4};	/*Profile ID - 18074*/
const Value V22405_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22405_279 = {S1, 0, &V22405_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22405_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22405_282 = {S3, 0, &V22405_282_57153, &P22405_279};	/*Property = BlobBuilder*/
const Value V22405_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22405_350 = {S4, 0, &V22405_350_57444, &P22405_282};	/*Property = Canvas*/
const Value V22405_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22405_283 = {S5, 0, &V22405_283_57159, &P22405_350};	/*Property = CssBackground*/
const Value V22405_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22405_284 = {S6, 0, &V22405_284_57163, &P22405_283};	/*Property = CssBorderImage*/
const Value V22405_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22405_285 = {S7, 0, &V22405_285_57166, &P22405_284};	/*Property = CssCanvas*/
const Value V22405_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22405_286 = {S8, 0, &V22405_286_57168, &P22405_285};	/*Property = CssColor*/
const Value V22405_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22405_237 = {S9, 0, &V22405_237_55784, &P22405_286};	/*Property = CssColumn*/
const Value V22405_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22405_287 = {S10, 0, &V22405_287_57172, &P22405_237};	/*Property = CssFlexbox*/
const Value V22405_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22405_288 = {S11, 0, &V22405_288_57175, &P22405_287};	/*Property = CssFont*/
const Value V22405_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22405_289 = {S12, 0, &V22405_289_57759, &P22405_288};	/*Property = CssImages*/
const Value V22405_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22405_291 = {S13, 0, &V22405_291_57180, &P22405_289};	/*Property = CssMediaQueries*/
const Value V22405_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22405_292 = {S14, 0, &V22405_292_57183, &P22405_291};	/*Property = CssMinMax*/
const Value V22405_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22405_293 = {S15, 0, &V22405_293_57187, &P22405_292};	/*Property = CssOverflow*/
const Value V22405_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22405_294 = {S16, 0, &V22405_294_57189, &P22405_293};	/*Property = CssPosition*/
const Value V22405_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22405_295 = {S17, 0, &V22405_295_57192, &P22405_294};	/*Property = CssText*/
const Value V22405_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22405_297 = {S18, 0, &V22405_297_57195, &P22405_295};	/*Property = CssTransforms*/
const Value V22405_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22405_241 = {S19, 0, &V22405_241_55799, &P22405_297};	/*Property = CssTransitions*/
const Value V22405_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22405_298 = {S20, 0, &V22405_298_57206, &P22405_241};	/*Property = CssUI*/
const Value V22405_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22405_300 = {S21, 0, &V22405_300_57216, &P22405_298};	/*Property = DataSet*/
const Value V22405_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22405_301 = {S22, 0, &V22405_301_57218, &P22405_300};	/*Property = DataUrl*/
const Value V22405_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22405_302 = {S23, 0, &V22405_302_57222, &P22405_301};	/*Property = DeviceOrientation*/
const Value V22405_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22405_304 = {S24, 0, &V22405_304_57227, &P22405_302};	/*Property = FileReader*/
const Value V22405_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22405_305 = {S25, 0, &V22405_305_57231, &P22405_304};	/*Property = FileSaver*/
const Value V22405_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22405_306 = {S26, 0, &V22405_306_57237, &P22405_305};	/*Property = FileWriter*/
const Value V22405_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22405_307 = {S27, 0, &V22405_307_57239, &P22405_306};	/*Property = FormData*/
const Value V22405_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22405_309 = {S28, 0, &V22405_309_57246, &P22405_307};	/*Property = Fullscreen*/
const Value V22405_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22405_310 = {S29, 0, &V22405_310_57248, &P22405_309};	/*Property = GeoLocation*/
const Value V22405_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22405_312 = {S30, 0, &V22405_312_57255, &P22405_310};	/*Property = History*/
const Value V22405_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22405_314 = {S31, 0, &V22405_314_57261, &P22405_312};	/*Property = Html5*/
const Value V22405_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22405_313 = {S32, 0, &V22405_313_57258, &P22405_314};	/*Property = Html-Media-Capture*/
const Value V22405_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22405_315 = {S33, 0, &V22405_315_57264, &P22405_313};	/*Property = Iframe*/
const Value V22405_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22405_316 = {S34, 0, &V22405_316_57266, &P22405_315};	/*Property = IndexedDB*/
const Value V22405_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22405_317 = {S35, 0, &V22405_317_57269, &P22405_316};	/*Property = Json*/
const Value V22405_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22405_15 = {S37, 0, &V22405_15_28, &P22405_317};	/*Property = LayoutEngine*/
const Value V22405_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22405_344 = {S38, 0, &V22405_344_57272, &P22405_15};	/*Property = Masking*/
const Value V22405_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22405_320 = {S39, 0, &V22405_320_57290, &P22405_344};	/*Property = PostMessage*/
const Value V22405_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22405_321 = {S40, 0, &V22405_321_57293, &P22405_320};	/*Property = Progress*/
const Value V22405_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22405_322 = {S41, 0, &V22405_322_57296, &P22405_321};	/*Property = Prompts*/
const Value V22405_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22405_323 = {S42, 0, &V22405_323_57299, &P22405_322};	/*Property = Selector*/
const Value V22405_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22405_324 = {S43, 0, &V22405_324_57306, &P22405_323};	/*Property = Svg*/
const Value V22405_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22405_327 = {S44, 0, &V22405_327_57315, &P22405_324};	/*Property = TouchEvents*/
const Value V22405_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22405_328 = {S45, 0, &V22405_328_57318, &P22405_327};	/*Property = Track*/
const Value V22405_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22405_329 = {S46, 0, &V22405_329_57323, &P22405_328};	/*Property = Video*/
const Value V22405_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22405_330 = {S47, 0, &V22405_330_57326, &P22405_329};	/*Property = Viewport*/
const Value V22405_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22405_332 = {S48, 0, &V22405_332_57338, &P22405_330};	/*Property = WebWorkers*/
const Value V22405_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22405_333 = {S49, 0, &V22405_333_57342, &P22405_332};	/*Property = Xhr2*/
const Profile P22405 = {"22405", &P22405_333};	/*Profile ID - 22405*/
const Value V18016_1_1 = {S2, NULL};	/*Value = True*/
const Property P18016_1 = {S50, 0, &V18016_1_1, NULL};	/*Property = IsMobile*/
const Value V18016_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P18016_5 = {S52, 0, &V18016_5_1832, &P18016_1};	/*Property = ScreenPixelsHeight*/
const Value V18016_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P18016_4 = {S54, 0, &V18016_4_5645, &P18016_5};	/*Property = ScreenPixelsWidth*/
const Profile P18016 = {"18016", &P18016_4};	/*Profile ID - 18016*/
const Value V17623_1_1 = {S2, NULL};	/*Value = True*/
const Property P17623_1 = {S50, 0, &V17623_1_1, NULL};	/*Property = IsMobile*/
const Value V17623_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P17623_5 = {S52, 0, &V17623_5_37569, &P17623_1};	/*Property = ScreenPixelsHeight*/
const Value V17623_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P17623_4 = {S54, 0, &V17623_4_5645, &P17623_5};	/*Property = ScreenPixelsWidth*/
const Profile P17623 = {"17623", &P17623_4};	/*Profile ID - 17623*/
const Value V16530_1_1 = {S2, NULL};	/*Value = True*/
const Property P16530_1 = {S50, 0, &V16530_1_1, NULL};	/*Property = IsMobile*/
const Value V16530_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16530_5 = {S52, 0, &V16530_5_1222, &P16530_1};	/*Property = ScreenPixelsHeight*/
const Value V16530_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16530_4 = {S54, 0, &V16530_4_1224, &P16530_5};	/*Property = ScreenPixelsWidth*/
const Profile P16530 = {"16530", &P16530_4};	/*Profile ID - 16530*/
const Value V17715_1_1 = {S2, NULL};	/*Value = True*/
const Property P17715_1 = {S50, 0, &V17715_1_1, NULL};	/*Property = IsMobile*/
const Value V17715_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17715_5 = {S52, 0, &V17715_5_1277, &P17715_1};	/*Property = ScreenPixelsHeight*/
const Value V17715_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17715_4 = {S54, 0, &V17715_4_1279, &P17715_5};	/*Property = ScreenPixelsWidth*/
const Profile P17715 = {"17715", &P17715_4};	/*Profile ID - 17715*/
const Value V21099_1_1 = {S2, NULL};	/*Value = True*/
const Property P21099_1 = {S50, 0, &V21099_1_1, NULL};	/*Property = IsMobile*/
const Value V21099_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21099_5 = {S52, 0, &V21099_5_1269, &P21099_1};	/*Property = ScreenPixelsHeight*/
const Value V21099_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21099_4 = {S54, 0, &V21099_4_1229, &P21099_5};	/*Property = ScreenPixelsWidth*/
const Profile P21099 = {"21099", &P21099_4};	/*Profile ID - 21099*/
const Profile P19827 = {"19827", NULL};	/*Profile ID - 19827*/
const Value V16349_1_1 = {S2, NULL};	/*Value = True*/
const Property P16349_1 = {S50, 0, &V16349_1_1, NULL};	/*Property = IsMobile*/
const Value V16349_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P16349_5 = {S52, 0, &V16349_5_9981, &P16349_1};	/*Property = ScreenPixelsHeight*/
const Value V16349_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P16349_4 = {S54, 0, &V16349_4_2336, &P16349_5};	/*Property = ScreenPixelsWidth*/
const Profile P16349 = {"16349", &P16349_4};	/*Profile ID - 16349*/
const Value V18701_1_1 = {S2, NULL};	/*Value = True*/
const Property P18701_1 = {S50, 0, &V18701_1_1, NULL};	/*Property = IsMobile*/
const Value V18701_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18701_5 = {S52, 0, &V18701_5_1227, &P18701_1};	/*Property = ScreenPixelsHeight*/
const Value V18701_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18701_4 = {S54, 0, &V18701_4_1229, &P18701_5};	/*Property = ScreenPixelsWidth*/
const Profile P18701 = {"18701", &P18701_4};	/*Profile ID - 18701*/
const Value V22407_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22407_279 = {S1, 0, &V22407_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22407_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22407_282 = {S3, 0, &V22407_282_57153, &P22407_279};	/*Property = BlobBuilder*/
const Value V22407_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22407_350 = {S4, 0, &V22407_350_57444, &P22407_282};	/*Property = Canvas*/
const Value V22407_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22407_283 = {S5, 0, &V22407_283_57159, &P22407_350};	/*Property = CssBackground*/
const Value V22407_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22407_284 = {S6, 0, &V22407_284_57163, &P22407_283};	/*Property = CssBorderImage*/
const Value V22407_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22407_285 = {S7, 0, &V22407_285_57166, &P22407_284};	/*Property = CssCanvas*/
const Value V22407_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22407_286 = {S8, 0, &V22407_286_57168, &P22407_285};	/*Property = CssColor*/
const Value V22407_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22407_237 = {S9, 0, &V22407_237_55784, &P22407_286};	/*Property = CssColumn*/
const Value V22407_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22407_287 = {S10, 0, &V22407_287_57172, &P22407_237};	/*Property = CssFlexbox*/
const Value V22407_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22407_288 = {S11, 0, &V22407_288_57175, &P22407_287};	/*Property = CssFont*/
const Value V22407_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22407_289 = {S12, 0, &V22407_289_57759, &P22407_288};	/*Property = CssImages*/
const Value V22407_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22407_291 = {S13, 0, &V22407_291_57180, &P22407_289};	/*Property = CssMediaQueries*/
const Value V22407_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22407_292 = {S14, 0, &V22407_292_57183, &P22407_291};	/*Property = CssMinMax*/
const Value V22407_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22407_293 = {S15, 0, &V22407_293_57187, &P22407_292};	/*Property = CssOverflow*/
const Value V22407_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22407_294 = {S16, 0, &V22407_294_57189, &P22407_293};	/*Property = CssPosition*/
const Value V22407_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22407_295 = {S17, 0, &V22407_295_57192, &P22407_294};	/*Property = CssText*/
const Value V22407_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22407_297 = {S18, 0, &V22407_297_57195, &P22407_295};	/*Property = CssTransforms*/
const Value V22407_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22407_241 = {S19, 0, &V22407_241_55799, &P22407_297};	/*Property = CssTransitions*/
const Value V22407_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22407_298 = {S20, 0, &V22407_298_57206, &P22407_241};	/*Property = CssUI*/
const Value V22407_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22407_300 = {S21, 0, &V22407_300_57216, &P22407_298};	/*Property = DataSet*/
const Value V22407_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22407_301 = {S22, 0, &V22407_301_57218, &P22407_300};	/*Property = DataUrl*/
const Value V22407_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22407_302 = {S23, 0, &V22407_302_57222, &P22407_301};	/*Property = DeviceOrientation*/
const Value V22407_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22407_304 = {S24, 0, &V22407_304_57227, &P22407_302};	/*Property = FileReader*/
const Value V22407_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22407_305 = {S25, 0, &V22407_305_57231, &P22407_304};	/*Property = FileSaver*/
const Value V22407_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22407_306 = {S26, 0, &V22407_306_57237, &P22407_305};	/*Property = FileWriter*/
const Value V22407_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22407_307 = {S27, 0, &V22407_307_57239, &P22407_306};	/*Property = FormData*/
const Value V22407_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22407_309 = {S28, 0, &V22407_309_57246, &P22407_307};	/*Property = Fullscreen*/
const Value V22407_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22407_310 = {S29, 0, &V22407_310_57248, &P22407_309};	/*Property = GeoLocation*/
const Value V22407_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22407_312 = {S30, 0, &V22407_312_57255, &P22407_310};	/*Property = History*/
const Value V22407_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22407_314 = {S31, 0, &V22407_314_57261, &P22407_312};	/*Property = Html5*/
const Value V22407_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22407_313 = {S32, 0, &V22407_313_57258, &P22407_314};	/*Property = Html-Media-Capture*/
const Value V22407_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22407_315 = {S33, 0, &V22407_315_57264, &P22407_313};	/*Property = Iframe*/
const Value V22407_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22407_316 = {S34, 0, &V22407_316_57266, &P22407_315};	/*Property = IndexedDB*/
const Value V22407_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22407_317 = {S35, 0, &V22407_317_57269, &P22407_316};	/*Property = Json*/
const Value V22407_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22407_15 = {S37, 0, &V22407_15_28, &P22407_317};	/*Property = LayoutEngine*/
const Value V22407_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22407_344 = {S38, 0, &V22407_344_57272, &P22407_15};	/*Property = Masking*/
const Value V22407_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22407_320 = {S39, 0, &V22407_320_57290, &P22407_344};	/*Property = PostMessage*/
const Value V22407_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22407_321 = {S40, 0, &V22407_321_57293, &P22407_320};	/*Property = Progress*/
const Value V22407_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22407_322 = {S41, 0, &V22407_322_57296, &P22407_321};	/*Property = Prompts*/
const Value V22407_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22407_323 = {S42, 0, &V22407_323_57299, &P22407_322};	/*Property = Selector*/
const Value V22407_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22407_324 = {S43, 0, &V22407_324_57306, &P22407_323};	/*Property = Svg*/
const Value V22407_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22407_327 = {S44, 0, &V22407_327_57315, &P22407_324};	/*Property = TouchEvents*/
const Value V22407_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22407_328 = {S45, 0, &V22407_328_57318, &P22407_327};	/*Property = Track*/
const Value V22407_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22407_329 = {S46, 0, &V22407_329_57323, &P22407_328};	/*Property = Video*/
const Value V22407_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22407_330 = {S47, 0, &V22407_330_57326, &P22407_329};	/*Property = Viewport*/
const Value V22407_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22407_332 = {S48, 0, &V22407_332_57338, &P22407_330};	/*Property = WebWorkers*/
const Value V22407_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22407_333 = {S49, 0, &V22407_333_57342, &P22407_332};	/*Property = Xhr2*/
const Profile P22407 = {"22407", &P22407_333};	/*Profile ID - 22407*/
const Value V20451_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20451_279 = {S1, 0, &V20451_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20451_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20451_282 = {S3, 0, &V20451_282_57154, &P20451_279};	/*Property = BlobBuilder*/
const Value V20451_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20451_350 = {S4, 0, &V20451_350_57444, &P20451_282};	/*Property = Canvas*/
const Value V20451_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20451_283 = {S5, 0, &V20451_283_57159, &P20451_350};	/*Property = CssBackground*/
const Value V20451_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20451_284 = {S6, 0, &V20451_284_57163, &P20451_283};	/*Property = CssBorderImage*/
const Value V20451_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20451_285 = {S7, 0, &V20451_285_57166, &P20451_284};	/*Property = CssCanvas*/
const Value V20451_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20451_286 = {S8, 0, &V20451_286_57168, &P20451_285};	/*Property = CssColor*/
const Value V20451_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20451_237 = {S9, 0, &V20451_237_55786, &P20451_286};	/*Property = CssColumn*/
const Value V20451_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20451_287 = {S10, 0, &V20451_287_57172, &P20451_237};	/*Property = CssFlexbox*/
const Value V20451_288_57175 = {S0, NULL};	/*Value = False*/
const Property P20451_288 = {S11, 0, &V20451_288_57175, &P20451_287};	/*Property = CssFont*/
const Value V20451_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20451_289 = {S12, 0, &V20451_289_57759, &P20451_288};	/*Property = CssImages*/
const Value V20451_291_57181 = {S0, NULL};	/*Value = False*/
const Property P20451_291 = {S13, 0, &V20451_291_57181, &P20451_289};	/*Property = CssMediaQueries*/
const Value V20451_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20451_292 = {S14, 0, &V20451_292_57183, &P20451_291};	/*Property = CssMinMax*/
const Value V20451_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20451_293 = {S15, 0, &V20451_293_57187, &P20451_292};	/*Property = CssOverflow*/
const Value V20451_294_57190 = {S0, NULL};	/*Value = False*/
const Property P20451_294 = {S16, 0, &V20451_294_57190, &P20451_293};	/*Property = CssPosition*/
const Value V20451_295_57193 = {S0, NULL};	/*Value = False*/
const Property P20451_295 = {S17, 0, &V20451_295_57193, &P20451_294};	/*Property = CssText*/
const Value V20451_297_57196 = {S0, NULL};	/*Value = False*/
const Property P20451_297 = {S18, 0, &V20451_297_57196, &P20451_295};	/*Property = CssTransforms*/
const Value V20451_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20451_241 = {S19, 0, &V20451_241_55799, &P20451_297};	/*Property = CssTransitions*/
const Value V20451_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20451_298 = {S20, 0, &V20451_298_57206, &P20451_241};	/*Property = CssUI*/
const Value V20451_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20451_300 = {S21, 0, &V20451_300_57216, &P20451_298};	/*Property = DataSet*/
const Value V20451_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20451_301 = {S22, 0, &V20451_301_57218, &P20451_300};	/*Property = DataUrl*/
const Value V20451_302_57222 = {S0, NULL};	/*Value = False*/
const Property P20451_302 = {S23, 0, &V20451_302_57222, &P20451_301};	/*Property = DeviceOrientation*/
const Value V20451_304_57228 = {S0, NULL};	/*Value = False*/
const Property P20451_304 = {S24, 0, &V20451_304_57228, &P20451_302};	/*Property = FileReader*/
const Value V20451_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20451_305 = {S25, 0, &V20451_305_57231, &P20451_304};	/*Property = FileSaver*/
const Value V20451_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20451_306 = {S26, 0, &V20451_306_57237, &P20451_305};	/*Property = FileWriter*/
const Value V20451_307_57240 = {S0, NULL};	/*Value = False*/
const Property P20451_307 = {S27, 0, &V20451_307_57240, &P20451_306};	/*Property = FormData*/
const Value V20451_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20451_309 = {S28, 0, &V20451_309_57246, &P20451_307};	/*Property = Fullscreen*/
const Value V20451_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20451_310 = {S29, 0, &V20451_310_57248, &P20451_309};	/*Property = GeoLocation*/
const Value V20451_312_57255 = {S0, NULL};	/*Value = False*/
const Property P20451_312 = {S30, 0, &V20451_312_57255, &P20451_310};	/*Property = History*/
const Value V20451_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20451_314 = {S31, 0, &V20451_314_57261, &P20451_312};	/*Property = Html5*/
const Value V20451_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20451_313 = {S32, 0, &V20451_313_57258, &P20451_314};	/*Property = Html-Media-Capture*/
const Value V20451_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20451_315 = {S33, 0, &V20451_315_57264, &P20451_313};	/*Property = Iframe*/
const Value V20451_316_57267 = {S0, NULL};	/*Value = False*/
const Property P20451_316 = {S34, 0, &V20451_316_57267, &P20451_315};	/*Property = IndexedDB*/
const Value V20451_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20451_317 = {S35, 0, &V20451_317_57269, &P20451_316};	/*Property = Json*/
const Value V20451_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P20451_15 = {S37, 0, &V20451_15_31, &P20451_317};	/*Property = LayoutEngine*/
const Value V20451_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20451_344 = {S38, 0, &V20451_344_57272, &P20451_15};	/*Property = Masking*/
const Value V20451_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20451_320 = {S39, 0, &V20451_320_57290, &P20451_344};	/*Property = PostMessage*/
const Value V20451_321_57294 = {S0, NULL};	/*Value = False*/
const Property P20451_321 = {S40, 0, &V20451_321_57294, &P20451_320};	/*Property = Progress*/
const Value V20451_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20451_322 = {S41, 0, &V20451_322_57296, &P20451_321};	/*Property = Prompts*/
const Value V20451_323_57300 = {S0, NULL};	/*Value = False*/
const Property P20451_323 = {S42, 0, &V20451_323_57300, &P20451_322};	/*Property = Selector*/
const Value V20451_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20451_324 = {S43, 0, &V20451_324_57306, &P20451_323};	/*Property = Svg*/
const Value V20451_327_57315 = {S0, NULL};	/*Value = False*/
const Property P20451_327 = {S44, 0, &V20451_327_57315, &P20451_324};	/*Property = TouchEvents*/
const Value V20451_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20451_328 = {S45, 0, &V20451_328_57318, &P20451_327};	/*Property = Track*/
const Value V20451_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20451_329 = {S46, 0, &V20451_329_57323, &P20451_328};	/*Property = Video*/
const Value V20451_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20451_330 = {S47, 0, &V20451_330_57326, &P20451_329};	/*Property = Viewport*/
const Value V20451_332_57339 = {S0, NULL};	/*Value = False*/
const Property P20451_332 = {S48, 0, &V20451_332_57339, &P20451_330};	/*Property = WebWorkers*/
const Value V20451_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20451_333 = {S49, 0, &V20451_333_57342, &P20451_332};	/*Property = Xhr2*/
const Profile P20451 = {"20451", &P20451_333};	/*Profile ID - 20451*/
const Value V17505_1_1 = {S2, NULL};	/*Value = True*/
const Property P17505_1 = {S50, 0, &V17505_1_1, NULL};	/*Property = IsMobile*/
const Value V17505_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17505_5 = {S52, 0, &V17505_5_1227, &P17505_1};	/*Property = ScreenPixelsHeight*/
const Value V17505_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17505_4 = {S54, 0, &V17505_4_1229, &P17505_5};	/*Property = ScreenPixelsWidth*/
const Profile P17505 = {"17505", &P17505_4};	/*Profile ID - 17505*/
const Value V22406_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22406_279 = {S1, 0, &V22406_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22406_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22406_282 = {S3, 0, &V22406_282_57153, &P22406_279};	/*Property = BlobBuilder*/
const Value V22406_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22406_350 = {S4, 0, &V22406_350_57444, &P22406_282};	/*Property = Canvas*/
const Value V22406_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22406_283 = {S5, 0, &V22406_283_57159, &P22406_350};	/*Property = CssBackground*/
const Value V22406_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22406_284 = {S6, 0, &V22406_284_57163, &P22406_283};	/*Property = CssBorderImage*/
const Value V22406_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22406_285 = {S7, 0, &V22406_285_57166, &P22406_284};	/*Property = CssCanvas*/
const Value V22406_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22406_286 = {S8, 0, &V22406_286_57168, &P22406_285};	/*Property = CssColor*/
const Value V22406_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22406_237 = {S9, 0, &V22406_237_55784, &P22406_286};	/*Property = CssColumn*/
const Value V22406_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22406_287 = {S10, 0, &V22406_287_57172, &P22406_237};	/*Property = CssFlexbox*/
const Value V22406_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22406_288 = {S11, 0, &V22406_288_57175, &P22406_287};	/*Property = CssFont*/
const Value V22406_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22406_289 = {S12, 0, &V22406_289_57759, &P22406_288};	/*Property = CssImages*/
const Value V22406_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22406_291 = {S13, 0, &V22406_291_57180, &P22406_289};	/*Property = CssMediaQueries*/
const Value V22406_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22406_292 = {S14, 0, &V22406_292_57183, &P22406_291};	/*Property = CssMinMax*/
const Value V22406_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22406_293 = {S15, 0, &V22406_293_57187, &P22406_292};	/*Property = CssOverflow*/
const Value V22406_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22406_294 = {S16, 0, &V22406_294_57189, &P22406_293};	/*Property = CssPosition*/
const Value V22406_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22406_295 = {S17, 0, &V22406_295_57192, &P22406_294};	/*Property = CssText*/
const Value V22406_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22406_297 = {S18, 0, &V22406_297_57195, &P22406_295};	/*Property = CssTransforms*/
const Value V22406_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22406_241 = {S19, 0, &V22406_241_55799, &P22406_297};	/*Property = CssTransitions*/
const Value V22406_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22406_298 = {S20, 0, &V22406_298_57206, &P22406_241};	/*Property = CssUI*/
const Value V22406_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22406_300 = {S21, 0, &V22406_300_57216, &P22406_298};	/*Property = DataSet*/
const Value V22406_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22406_301 = {S22, 0, &V22406_301_57218, &P22406_300};	/*Property = DataUrl*/
const Value V22406_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22406_302 = {S23, 0, &V22406_302_57222, &P22406_301};	/*Property = DeviceOrientation*/
const Value V22406_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22406_304 = {S24, 0, &V22406_304_57227, &P22406_302};	/*Property = FileReader*/
const Value V22406_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22406_305 = {S25, 0, &V22406_305_57231, &P22406_304};	/*Property = FileSaver*/
const Value V22406_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22406_306 = {S26, 0, &V22406_306_57237, &P22406_305};	/*Property = FileWriter*/
const Value V22406_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22406_307 = {S27, 0, &V22406_307_57239, &P22406_306};	/*Property = FormData*/
const Value V22406_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22406_309 = {S28, 0, &V22406_309_57246, &P22406_307};	/*Property = Fullscreen*/
const Value V22406_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22406_310 = {S29, 0, &V22406_310_57248, &P22406_309};	/*Property = GeoLocation*/
const Value V22406_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22406_312 = {S30, 0, &V22406_312_57255, &P22406_310};	/*Property = History*/
const Value V22406_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22406_314 = {S31, 0, &V22406_314_57261, &P22406_312};	/*Property = Html5*/
const Value V22406_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22406_313 = {S32, 0, &V22406_313_57258, &P22406_314};	/*Property = Html-Media-Capture*/
const Value V22406_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22406_315 = {S33, 0, &V22406_315_57264, &P22406_313};	/*Property = Iframe*/
const Value V22406_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22406_316 = {S34, 0, &V22406_316_57266, &P22406_315};	/*Property = IndexedDB*/
const Value V22406_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22406_317 = {S35, 0, &V22406_317_57269, &P22406_316};	/*Property = Json*/
const Value V22406_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22406_15 = {S37, 0, &V22406_15_28, &P22406_317};	/*Property = LayoutEngine*/
const Value V22406_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22406_344 = {S38, 0, &V22406_344_57272, &P22406_15};	/*Property = Masking*/
const Value V22406_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22406_320 = {S39, 0, &V22406_320_57290, &P22406_344};	/*Property = PostMessage*/
const Value V22406_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22406_321 = {S40, 0, &V22406_321_57293, &P22406_320};	/*Property = Progress*/
const Value V22406_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22406_322 = {S41, 0, &V22406_322_57296, &P22406_321};	/*Property = Prompts*/
const Value V22406_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22406_323 = {S42, 0, &V22406_323_57299, &P22406_322};	/*Property = Selector*/
const Value V22406_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22406_324 = {S43, 0, &V22406_324_57306, &P22406_323};	/*Property = Svg*/
const Value V22406_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22406_327 = {S44, 0, &V22406_327_57315, &P22406_324};	/*Property = TouchEvents*/
const Value V22406_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22406_328 = {S45, 0, &V22406_328_57318, &P22406_327};	/*Property = Track*/
const Value V22406_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22406_329 = {S46, 0, &V22406_329_57323, &P22406_328};	/*Property = Video*/
const Value V22406_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22406_330 = {S47, 0, &V22406_330_57326, &P22406_329};	/*Property = Viewport*/
const Value V22406_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22406_332 = {S48, 0, &V22406_332_57338, &P22406_330};	/*Property = WebWorkers*/
const Value V22406_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22406_333 = {S49, 0, &V22406_333_57342, &P22406_332};	/*Property = Xhr2*/
const Profile P22406 = {"22406", &P22406_333};	/*Profile ID - 22406*/
const Profile P19828 = {"19828", NULL};	/*Profile ID - 19828*/
const Profile P18346 = {"18346", NULL};	/*Profile ID - 18346*/
const Value V15802_1_1 = {S2, NULL};	/*Value = True*/
const Property P15802_1 = {S50, 0, &V15802_1_1, NULL};	/*Property = IsMobile*/
const Value V15802_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P15802_5 = {S52, 0, &V15802_5_1403, &P15802_1};	/*Property = ScreenPixelsHeight*/
const Value V15802_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P15802_4 = {S54, 0, &V15802_4_1404, &P15802_5};	/*Property = ScreenPixelsWidth*/
const Profile P15802 = {"15802", &P15802_4};	/*Profile ID - 15802*/
const Value V16304_1_1 = {S2, NULL};	/*Value = True*/
const Property P16304_1 = {S50, 0, &V16304_1_1, NULL};	/*Property = IsMobile*/
const Value V16304_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16304_5 = {S52, 0, &V16304_5_1277, &P16304_1};	/*Property = ScreenPixelsHeight*/
const Value V16304_4_37613 = {S125, NULL};	/*Value = 230*/
const Property P16304_4 = {S54, 0, &V16304_4_37613, &P16304_5};	/*Property = ScreenPixelsWidth*/
const Profile P16304 = {"16304", &P16304_4};	/*Profile ID - 16304*/
const Value V23144_1_1 = {S2, NULL};	/*Value = True*/
const Property P23144_1 = {S50, 0, &V23144_1_1, NULL};	/*Property = IsMobile*/
const Value V23144_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23144_5 = {S52, 0, &V23144_5_37, &P23144_1};	/*Property = ScreenPixelsHeight*/
const Value V23144_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23144_4 = {S54, 0, &V23144_4_36, &P23144_5};	/*Property = ScreenPixelsWidth*/
const Profile P23144 = {"23144", &P23144_4};	/*Profile ID - 23144*/
const Value V20971_1_1 = {S2, NULL};	/*Value = True*/
const Property P20971_1 = {S50, 0, &V20971_1_1, NULL};	/*Property = IsMobile*/
const Value V20971_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20971_5 = {S52, 0, &V20971_5_1227, &P20971_1};	/*Property = ScreenPixelsHeight*/
const Value V20971_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20971_4 = {S54, 0, &V20971_4_1229, &P20971_5};	/*Property = ScreenPixelsWidth*/
const Profile P20971 = {"20971", &P20971_4};	/*Profile ID - 20971*/
const Value V24203_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24203_279 = {S1, 0, &V24203_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24203_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24203_282 = {S3, 0, &V24203_282_57154, &P24203_279};	/*Property = BlobBuilder*/
const Value V24203_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24203_350 = {S4, 0, &V24203_350_57444, &P24203_282};	/*Property = Canvas*/
const Value V24203_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24203_283 = {S5, 0, &V24203_283_57159, &P24203_350};	/*Property = CssBackground*/
const Value V24203_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24203_284 = {S6, 0, &V24203_284_57162, &P24203_283};	/*Property = CssBorderImage*/
const Value V24203_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24203_285 = {S7, 0, &V24203_285_57166, &P24203_284};	/*Property = CssCanvas*/
const Value V24203_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24203_286 = {S8, 0, &V24203_286_57168, &P24203_285};	/*Property = CssColor*/
const Value V24203_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24203_237 = {S9, 0, &V24203_237_55786, &P24203_286};	/*Property = CssColumn*/
const Value V24203_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24203_287 = {S10, 0, &V24203_287_57172, &P24203_237};	/*Property = CssFlexbox*/
const Value V24203_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24203_288 = {S11, 0, &V24203_288_57175, &P24203_287};	/*Property = CssFont*/
const Value V24203_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24203_289 = {S12, 0, &V24203_289_57759, &P24203_288};	/*Property = CssImages*/
const Value V24203_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24203_291 = {S13, 0, &V24203_291_57180, &P24203_289};	/*Property = CssMediaQueries*/
const Value V24203_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24203_292 = {S14, 0, &V24203_292_57183, &P24203_291};	/*Property = CssMinMax*/
const Value V24203_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24203_293 = {S15, 0, &V24203_293_57187, &P24203_292};	/*Property = CssOverflow*/
const Value V24203_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24203_294 = {S16, 0, &V24203_294_57189, &P24203_293};	/*Property = CssPosition*/
const Value V24203_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24203_295 = {S17, 0, &V24203_295_57192, &P24203_294};	/*Property = CssText*/
const Value V24203_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24203_297 = {S18, 0, &V24203_297_57195, &P24203_295};	/*Property = CssTransforms*/
const Value V24203_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24203_241 = {S19, 0, &V24203_241_55799, &P24203_297};	/*Property = CssTransitions*/
const Value V24203_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24203_298 = {S20, 0, &V24203_298_57206, &P24203_241};	/*Property = CssUI*/
const Value V24203_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24203_300 = {S21, 0, &V24203_300_57215, &P24203_298};	/*Property = DataSet*/
const Value V24203_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24203_301 = {S22, 0, &V24203_301_57218, &P24203_300};	/*Property = DataUrl*/
const Value V24203_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24203_302 = {S23, 0, &V24203_302_57221, &P24203_301};	/*Property = DeviceOrientation*/
const Value V24203_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24203_304 = {S24, 0, &V24203_304_57227, &P24203_302};	/*Property = FileReader*/
const Value V24203_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24203_305 = {S25, 0, &V24203_305_57231, &P24203_304};	/*Property = FileSaver*/
const Value V24203_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24203_306 = {S26, 0, &V24203_306_57237, &P24203_305};	/*Property = FileWriter*/
const Value V24203_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24203_307 = {S27, 0, &V24203_307_57239, &P24203_306};	/*Property = FormData*/
const Value V24203_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24203_309 = {S28, 0, &V24203_309_57246, &P24203_307};	/*Property = Fullscreen*/
const Value V24203_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24203_310 = {S29, 0, &V24203_310_57248, &P24203_309};	/*Property = GeoLocation*/
const Value V24203_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24203_312 = {S30, 0, &V24203_312_57254, &P24203_310};	/*Property = History*/
const Value V24203_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24203_314 = {S31, 0, &V24203_314_57260, &P24203_312};	/*Property = Html5*/
const Value V24203_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24203_313 = {S32, 0, &V24203_313_57257, &P24203_314};	/*Property = Html-Media-Capture*/
const Value V24203_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24203_315 = {S33, 0, &V24203_315_57264, &P24203_313};	/*Property = Iframe*/
const Value V24203_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24203_316 = {S34, 0, &V24203_316_57266, &P24203_315};	/*Property = IndexedDB*/
const Value V24203_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24203_317 = {S35, 0, &V24203_317_57269, &P24203_316};	/*Property = Json*/
const Value V24203_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24203_15 = {S37, 0, &V24203_15_54167, &P24203_317};	/*Property = LayoutEngine*/
const Value V24203_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24203_344 = {S38, 0, &V24203_344_57272, &P24203_15};	/*Property = Masking*/
const Value V24203_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24203_320 = {S39, 0, &V24203_320_57290, &P24203_344};	/*Property = PostMessage*/
const Value V24203_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24203_321 = {S40, 0, &V24203_321_57293, &P24203_320};	/*Property = Progress*/
const Value V24203_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24203_322 = {S41, 0, &V24203_322_57296, &P24203_321};	/*Property = Prompts*/
const Value V24203_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24203_323 = {S42, 0, &V24203_323_57299, &P24203_322};	/*Property = Selector*/
const Value V24203_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24203_324 = {S43, 0, &V24203_324_57306, &P24203_323};	/*Property = Svg*/
const Value V24203_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24203_327 = {S44, 0, &V24203_327_57314, &P24203_324};	/*Property = TouchEvents*/
const Value V24203_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24203_328 = {S45, 0, &V24203_328_57318, &P24203_327};	/*Property = Track*/
const Value V24203_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24203_329 = {S46, 0, &V24203_329_57323, &P24203_328};	/*Property = Video*/
const Value V24203_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24203_330 = {S47, 0, &V24203_330_57326, &P24203_329};	/*Property = Viewport*/
const Value V24203_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24203_332 = {S48, 0, &V24203_332_57339, &P24203_330};	/*Property = WebWorkers*/
const Value V24203_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24203_333 = {S49, 0, &V24203_333_57342, &P24203_332};	/*Property = Xhr2*/
const Profile P24203 = {"24203", &P24203_333};	/*Profile ID - 24203*/
const Value V22409_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22409_279 = {S1, 0, &V22409_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22409_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22409_282 = {S3, 0, &V22409_282_57153, &P22409_279};	/*Property = BlobBuilder*/
const Value V22409_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22409_350 = {S4, 0, &V22409_350_57444, &P22409_282};	/*Property = Canvas*/
const Value V22409_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22409_283 = {S5, 0, &V22409_283_57159, &P22409_350};	/*Property = CssBackground*/
const Value V22409_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22409_284 = {S6, 0, &V22409_284_57163, &P22409_283};	/*Property = CssBorderImage*/
const Value V22409_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22409_285 = {S7, 0, &V22409_285_57166, &P22409_284};	/*Property = CssCanvas*/
const Value V22409_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22409_286 = {S8, 0, &V22409_286_57168, &P22409_285};	/*Property = CssColor*/
const Value V22409_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22409_237 = {S9, 0, &V22409_237_55784, &P22409_286};	/*Property = CssColumn*/
const Value V22409_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22409_287 = {S10, 0, &V22409_287_57172, &P22409_237};	/*Property = CssFlexbox*/
const Value V22409_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22409_288 = {S11, 0, &V22409_288_57175, &P22409_287};	/*Property = CssFont*/
const Value V22409_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22409_289 = {S12, 0, &V22409_289_57759, &P22409_288};	/*Property = CssImages*/
const Value V22409_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22409_291 = {S13, 0, &V22409_291_57180, &P22409_289};	/*Property = CssMediaQueries*/
const Value V22409_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22409_292 = {S14, 0, &V22409_292_57183, &P22409_291};	/*Property = CssMinMax*/
const Value V22409_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22409_293 = {S15, 0, &V22409_293_57187, &P22409_292};	/*Property = CssOverflow*/
const Value V22409_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22409_294 = {S16, 0, &V22409_294_57189, &P22409_293};	/*Property = CssPosition*/
const Value V22409_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22409_295 = {S17, 0, &V22409_295_57192, &P22409_294};	/*Property = CssText*/
const Value V22409_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22409_297 = {S18, 0, &V22409_297_57195, &P22409_295};	/*Property = CssTransforms*/
const Value V22409_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22409_241 = {S19, 0, &V22409_241_55799, &P22409_297};	/*Property = CssTransitions*/
const Value V22409_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22409_298 = {S20, 0, &V22409_298_57206, &P22409_241};	/*Property = CssUI*/
const Value V22409_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22409_300 = {S21, 0, &V22409_300_57216, &P22409_298};	/*Property = DataSet*/
const Value V22409_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22409_301 = {S22, 0, &V22409_301_57218, &P22409_300};	/*Property = DataUrl*/
const Value V22409_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22409_302 = {S23, 0, &V22409_302_57222, &P22409_301};	/*Property = DeviceOrientation*/
const Value V22409_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22409_304 = {S24, 0, &V22409_304_57227, &P22409_302};	/*Property = FileReader*/
const Value V22409_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22409_305 = {S25, 0, &V22409_305_57231, &P22409_304};	/*Property = FileSaver*/
const Value V22409_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22409_306 = {S26, 0, &V22409_306_57237, &P22409_305};	/*Property = FileWriter*/
const Value V22409_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22409_307 = {S27, 0, &V22409_307_57239, &P22409_306};	/*Property = FormData*/
const Value V22409_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22409_309 = {S28, 0, &V22409_309_57246, &P22409_307};	/*Property = Fullscreen*/
const Value V22409_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22409_310 = {S29, 0, &V22409_310_57248, &P22409_309};	/*Property = GeoLocation*/
const Value V22409_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22409_312 = {S30, 0, &V22409_312_57255, &P22409_310};	/*Property = History*/
const Value V22409_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22409_314 = {S31, 0, &V22409_314_57261, &P22409_312};	/*Property = Html5*/
const Value V22409_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22409_313 = {S32, 0, &V22409_313_57258, &P22409_314};	/*Property = Html-Media-Capture*/
const Value V22409_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22409_315 = {S33, 0, &V22409_315_57264, &P22409_313};	/*Property = Iframe*/
const Value V22409_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22409_316 = {S34, 0, &V22409_316_57266, &P22409_315};	/*Property = IndexedDB*/
const Value V22409_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22409_317 = {S35, 0, &V22409_317_57269, &P22409_316};	/*Property = Json*/
const Value V22409_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22409_15 = {S37, 0, &V22409_15_28, &P22409_317};	/*Property = LayoutEngine*/
const Value V22409_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22409_344 = {S38, 0, &V22409_344_57272, &P22409_15};	/*Property = Masking*/
const Value V22409_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22409_320 = {S39, 0, &V22409_320_57290, &P22409_344};	/*Property = PostMessage*/
const Value V22409_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22409_321 = {S40, 0, &V22409_321_57293, &P22409_320};	/*Property = Progress*/
const Value V22409_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22409_322 = {S41, 0, &V22409_322_57296, &P22409_321};	/*Property = Prompts*/
const Value V22409_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22409_323 = {S42, 0, &V22409_323_57299, &P22409_322};	/*Property = Selector*/
const Value V22409_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22409_324 = {S43, 0, &V22409_324_57306, &P22409_323};	/*Property = Svg*/
const Value V22409_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22409_327 = {S44, 0, &V22409_327_57315, &P22409_324};	/*Property = TouchEvents*/
const Value V22409_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22409_328 = {S45, 0, &V22409_328_57318, &P22409_327};	/*Property = Track*/
const Value V22409_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22409_329 = {S46, 0, &V22409_329_57323, &P22409_328};	/*Property = Video*/
const Value V22409_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22409_330 = {S47, 0, &V22409_330_57326, &P22409_329};	/*Property = Viewport*/
const Value V22409_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22409_332 = {S48, 0, &V22409_332_57338, &P22409_330};	/*Property = WebWorkers*/
const Value V22409_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22409_333 = {S49, 0, &V22409_333_57342, &P22409_332};	/*Property = Xhr2*/
const Profile P22409 = {"22409", &P22409_333};	/*Profile ID - 22409*/
const Profile P18140 = {"18140", NULL};	/*Profile ID - 18140*/
const Value V14377_1_1 = {S2, NULL};	/*Value = True*/
const Property P14377_1 = {S50, 0, &V14377_1_1, NULL};	/*Property = IsMobile*/
const Value V14377_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P14377_5 = {S52, 0, &V14377_5_1380, &P14377_1};	/*Property = ScreenPixelsHeight*/
const Value V14377_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P14377_4 = {S54, 0, &V14377_4_1279, &P14377_5};	/*Property = ScreenPixelsWidth*/
const Profile P14377 = {"14377", &P14377_4};	/*Profile ID - 14377*/
const Value V15851_1_1 = {S2, NULL};	/*Value = True*/
const Property P15851_1 = {S50, 0, &V15851_1_1, NULL};	/*Property = IsMobile*/
const Value V15851_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15851_5 = {S52, 0, &V15851_5_1227, &P15851_1};	/*Property = ScreenPixelsHeight*/
const Value V15851_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15851_4 = {S54, 0, &V15851_4_1229, &P15851_5};	/*Property = ScreenPixelsWidth*/
const Profile P15851 = {"15851", &P15851_4};	/*Profile ID - 15851*/
const Value V22408_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22408_279 = {S1, 0, &V22408_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22408_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22408_282 = {S3, 0, &V22408_282_57153, &P22408_279};	/*Property = BlobBuilder*/
const Value V22408_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22408_350 = {S4, 0, &V22408_350_57444, &P22408_282};	/*Property = Canvas*/
const Value V22408_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22408_283 = {S5, 0, &V22408_283_57159, &P22408_350};	/*Property = CssBackground*/
const Value V22408_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22408_284 = {S6, 0, &V22408_284_57163, &P22408_283};	/*Property = CssBorderImage*/
const Value V22408_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22408_285 = {S7, 0, &V22408_285_57166, &P22408_284};	/*Property = CssCanvas*/
const Value V22408_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22408_286 = {S8, 0, &V22408_286_57168, &P22408_285};	/*Property = CssColor*/
const Value V22408_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22408_237 = {S9, 0, &V22408_237_55784, &P22408_286};	/*Property = CssColumn*/
const Value V22408_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22408_287 = {S10, 0, &V22408_287_57172, &P22408_237};	/*Property = CssFlexbox*/
const Value V22408_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22408_288 = {S11, 0, &V22408_288_57175, &P22408_287};	/*Property = CssFont*/
const Value V22408_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22408_289 = {S12, 0, &V22408_289_57759, &P22408_288};	/*Property = CssImages*/
const Value V22408_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22408_291 = {S13, 0, &V22408_291_57180, &P22408_289};	/*Property = CssMediaQueries*/
const Value V22408_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22408_292 = {S14, 0, &V22408_292_57183, &P22408_291};	/*Property = CssMinMax*/
const Value V22408_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22408_293 = {S15, 0, &V22408_293_57187, &P22408_292};	/*Property = CssOverflow*/
const Value V22408_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22408_294 = {S16, 0, &V22408_294_57189, &P22408_293};	/*Property = CssPosition*/
const Value V22408_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22408_295 = {S17, 0, &V22408_295_57192, &P22408_294};	/*Property = CssText*/
const Value V22408_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22408_297 = {S18, 0, &V22408_297_57195, &P22408_295};	/*Property = CssTransforms*/
const Value V22408_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22408_241 = {S19, 0, &V22408_241_55799, &P22408_297};	/*Property = CssTransitions*/
const Value V22408_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22408_298 = {S20, 0, &V22408_298_57206, &P22408_241};	/*Property = CssUI*/
const Value V22408_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22408_300 = {S21, 0, &V22408_300_57216, &P22408_298};	/*Property = DataSet*/
const Value V22408_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22408_301 = {S22, 0, &V22408_301_57218, &P22408_300};	/*Property = DataUrl*/
const Value V22408_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22408_302 = {S23, 0, &V22408_302_57222, &P22408_301};	/*Property = DeviceOrientation*/
const Value V22408_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22408_304 = {S24, 0, &V22408_304_57227, &P22408_302};	/*Property = FileReader*/
const Value V22408_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22408_305 = {S25, 0, &V22408_305_57231, &P22408_304};	/*Property = FileSaver*/
const Value V22408_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22408_306 = {S26, 0, &V22408_306_57237, &P22408_305};	/*Property = FileWriter*/
const Value V22408_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22408_307 = {S27, 0, &V22408_307_57239, &P22408_306};	/*Property = FormData*/
const Value V22408_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22408_309 = {S28, 0, &V22408_309_57246, &P22408_307};	/*Property = Fullscreen*/
const Value V22408_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22408_310 = {S29, 0, &V22408_310_57248, &P22408_309};	/*Property = GeoLocation*/
const Value V22408_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22408_312 = {S30, 0, &V22408_312_57255, &P22408_310};	/*Property = History*/
const Value V22408_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22408_314 = {S31, 0, &V22408_314_57261, &P22408_312};	/*Property = Html5*/
const Value V22408_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22408_313 = {S32, 0, &V22408_313_57258, &P22408_314};	/*Property = Html-Media-Capture*/
const Value V22408_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22408_315 = {S33, 0, &V22408_315_57264, &P22408_313};	/*Property = Iframe*/
const Value V22408_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22408_316 = {S34, 0, &V22408_316_57266, &P22408_315};	/*Property = IndexedDB*/
const Value V22408_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22408_317 = {S35, 0, &V22408_317_57269, &P22408_316};	/*Property = Json*/
const Value V22408_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22408_15 = {S37, 0, &V22408_15_28, &P22408_317};	/*Property = LayoutEngine*/
const Value V22408_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22408_344 = {S38, 0, &V22408_344_57272, &P22408_15};	/*Property = Masking*/
const Value V22408_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22408_320 = {S39, 0, &V22408_320_57290, &P22408_344};	/*Property = PostMessage*/
const Value V22408_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22408_321 = {S40, 0, &V22408_321_57293, &P22408_320};	/*Property = Progress*/
const Value V22408_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22408_322 = {S41, 0, &V22408_322_57296, &P22408_321};	/*Property = Prompts*/
const Value V22408_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22408_323 = {S42, 0, &V22408_323_57299, &P22408_322};	/*Property = Selector*/
const Value V22408_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22408_324 = {S43, 0, &V22408_324_57306, &P22408_323};	/*Property = Svg*/
const Value V22408_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22408_327 = {S44, 0, &V22408_327_57315, &P22408_324};	/*Property = TouchEvents*/
const Value V22408_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22408_328 = {S45, 0, &V22408_328_57318, &P22408_327};	/*Property = Track*/
const Value V22408_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22408_329 = {S46, 0, &V22408_329_57323, &P22408_328};	/*Property = Video*/
const Value V22408_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22408_330 = {S47, 0, &V22408_330_57326, &P22408_329};	/*Property = Viewport*/
const Value V22408_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22408_332 = {S48, 0, &V22408_332_57338, &P22408_330};	/*Property = WebWorkers*/
const Value V22408_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22408_333 = {S49, 0, &V22408_333_57342, &P22408_332};	/*Property = Xhr2*/
const Profile P22408 = {"22408", &P22408_333};	/*Profile ID - 22408*/
const Value V17957_1_1 = {S2, NULL};	/*Value = True*/
const Property P17957_1 = {S50, 0, &V17957_1_1, NULL};	/*Property = IsMobile*/
const Value V17957_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17957_5 = {S52, 0, &V17957_5_1227, &P17957_1};	/*Property = ScreenPixelsHeight*/
const Value V17957_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P17957_4 = {S54, 0, &V17957_4_1460, &P17957_5};	/*Property = ScreenPixelsWidth*/
const Profile P17957 = {"17957", &P17957_4};	/*Profile ID - 17957*/
const Value V22452_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22452_279 = {S1, 0, &V22452_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22452_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22452_282 = {S3, 0, &V22452_282_57154, &P22452_279};	/*Property = BlobBuilder*/
const Value V22452_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22452_350 = {S4, 0, &V22452_350_57444, &P22452_282};	/*Property = Canvas*/
const Value V22452_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22452_283 = {S5, 0, &V22452_283_57159, &P22452_350};	/*Property = CssBackground*/
const Value V22452_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22452_284 = {S6, 0, &V22452_284_57163, &P22452_283};	/*Property = CssBorderImage*/
const Value V22452_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22452_285 = {S7, 0, &V22452_285_57166, &P22452_284};	/*Property = CssCanvas*/
const Value V22452_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22452_286 = {S8, 0, &V22452_286_57168, &P22452_285};	/*Property = CssColor*/
const Value V22452_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22452_237 = {S9, 0, &V22452_237_55784, &P22452_286};	/*Property = CssColumn*/
const Value V22452_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22452_287 = {S10, 0, &V22452_287_57172, &P22452_237};	/*Property = CssFlexbox*/
const Value V22452_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22452_288 = {S11, 0, &V22452_288_57175, &P22452_287};	/*Property = CssFont*/
const Value V22452_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22452_289 = {S12, 0, &V22452_289_57759, &P22452_288};	/*Property = CssImages*/
const Value V22452_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22452_291 = {S13, 0, &V22452_291_57180, &P22452_289};	/*Property = CssMediaQueries*/
const Value V22452_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22452_292 = {S14, 0, &V22452_292_57183, &P22452_291};	/*Property = CssMinMax*/
const Value V22452_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22452_293 = {S15, 0, &V22452_293_57187, &P22452_292};	/*Property = CssOverflow*/
const Value V22452_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22452_294 = {S16, 0, &V22452_294_57189, &P22452_293};	/*Property = CssPosition*/
const Value V22452_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22452_295 = {S17, 0, &V22452_295_57192, &P22452_294};	/*Property = CssText*/
const Value V22452_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22452_297 = {S18, 0, &V22452_297_57195, &P22452_295};	/*Property = CssTransforms*/
const Value V22452_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22452_241 = {S19, 0, &V22452_241_55799, &P22452_297};	/*Property = CssTransitions*/
const Value V22452_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22452_298 = {S20, 0, &V22452_298_57206, &P22452_241};	/*Property = CssUI*/
const Value V22452_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22452_300 = {S21, 0, &V22452_300_57216, &P22452_298};	/*Property = DataSet*/
const Value V22452_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22452_301 = {S22, 0, &V22452_301_57218, &P22452_300};	/*Property = DataUrl*/
const Value V22452_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22452_302 = {S23, 0, &V22452_302_57222, &P22452_301};	/*Property = DeviceOrientation*/
const Value V22452_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22452_304 = {S24, 0, &V22452_304_57227, &P22452_302};	/*Property = FileReader*/
const Value V22452_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22452_305 = {S25, 0, &V22452_305_57231, &P22452_304};	/*Property = FileSaver*/
const Value V22452_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22452_306 = {S26, 0, &V22452_306_57237, &P22452_305};	/*Property = FileWriter*/
const Value V22452_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22452_307 = {S27, 0, &V22452_307_57239, &P22452_306};	/*Property = FormData*/
const Value V22452_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22452_309 = {S28, 0, &V22452_309_57246, &P22452_307};	/*Property = Fullscreen*/
const Value V22452_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22452_310 = {S29, 0, &V22452_310_57248, &P22452_309};	/*Property = GeoLocation*/
const Value V22452_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22452_312 = {S30, 0, &V22452_312_57255, &P22452_310};	/*Property = History*/
const Value V22452_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22452_314 = {S31, 0, &V22452_314_57261, &P22452_312};	/*Property = Html5*/
const Value V22452_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22452_313 = {S32, 0, &V22452_313_57258, &P22452_314};	/*Property = Html-Media-Capture*/
const Value V22452_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22452_315 = {S33, 0, &V22452_315_57264, &P22452_313};	/*Property = Iframe*/
const Value V22452_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22452_316 = {S34, 0, &V22452_316_57266, &P22452_315};	/*Property = IndexedDB*/
const Value V22452_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22452_317 = {S35, 0, &V22452_317_57269, &P22452_316};	/*Property = Json*/
const Value V22452_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22452_15 = {S37, 0, &V22452_15_28, &P22452_317};	/*Property = LayoutEngine*/
const Value V22452_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22452_344 = {S38, 0, &V22452_344_57272, &P22452_15};	/*Property = Masking*/
const Value V22452_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22452_320 = {S39, 0, &V22452_320_57290, &P22452_344};	/*Property = PostMessage*/
const Value V22452_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22452_321 = {S40, 0, &V22452_321_57293, &P22452_320};	/*Property = Progress*/
const Value V22452_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22452_322 = {S41, 0, &V22452_322_57296, &P22452_321};	/*Property = Prompts*/
const Value V22452_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22452_323 = {S42, 0, &V22452_323_57299, &P22452_322};	/*Property = Selector*/
const Value V22452_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22452_324 = {S43, 0, &V22452_324_57306, &P22452_323};	/*Property = Svg*/
const Value V22452_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22452_327 = {S44, 0, &V22452_327_57315, &P22452_324};	/*Property = TouchEvents*/
const Value V22452_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22452_328 = {S45, 0, &V22452_328_57318, &P22452_327};	/*Property = Track*/
const Value V22452_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22452_329 = {S46, 0, &V22452_329_57323, &P22452_328};	/*Property = Video*/
const Value V22452_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22452_330 = {S47, 0, &V22452_330_57326, &P22452_329};	/*Property = Viewport*/
const Value V22452_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22452_332 = {S48, 0, &V22452_332_57339, &P22452_330};	/*Property = WebWorkers*/
const Value V22452_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22452_333 = {S49, 0, &V22452_333_57342, &P22452_332};	/*Property = Xhr2*/
const Profile P22452 = {"22452", &P22452_333};	/*Profile ID - 22452*/
const Value V16273_1_1 = {S2, NULL};	/*Value = True*/
const Property P16273_1 = {S50, 0, &V16273_1_1, NULL};	/*Property = IsMobile*/
const Value V16273_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16273_5 = {S52, 0, &V16273_5_1227, &P16273_1};	/*Property = ScreenPixelsHeight*/
const Value V16273_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16273_4 = {S54, 0, &V16273_4_1229, &P16273_5};	/*Property = ScreenPixelsWidth*/
const Profile P16273 = {"16273", &P16273_4};	/*Profile ID - 16273*/
const Value V23859_1_1 = {S2, NULL};	/*Value = True*/
const Property P23859_1 = {S50, 0, &V23859_1_1, NULL};	/*Property = IsMobile*/
const Value V23859_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23859_5 = {S52, 0, &V23859_5_1458, &P23859_1};	/*Property = ScreenPixelsHeight*/
const Value V23859_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23859_4 = {S54, 0, &V23859_4_1460, &P23859_5};	/*Property = ScreenPixelsWidth*/
const Profile P23859 = {"23859", &P23859_4};	/*Profile ID - 23859*/
const Value V17713_1_1 = {S2, NULL};	/*Value = True*/
const Property P17713_1 = {S50, 0, &V17713_1_1, NULL};	/*Property = IsMobile*/
const Value V17713_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17713_5 = {S52, 0, &V17713_5_1277, &P17713_1};	/*Property = ScreenPixelsHeight*/
const Value V17713_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17713_4 = {S54, 0, &V17713_4_1279, &P17713_5};	/*Property = ScreenPixelsWidth*/
const Profile P17713 = {"17713", &P17713_4};	/*Profile ID - 17713*/
const Value V20722_1_1 = {S2, NULL};	/*Value = True*/
const Property P20722_1 = {S50, 0, &V20722_1_1, NULL};	/*Property = IsMobile*/
const Value V20722_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20722_5 = {S52, 0, &V20722_5_1227, &P20722_1};	/*Property = ScreenPixelsHeight*/
const Value V20722_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20722_4 = {S54, 0, &V20722_4_1229, &P20722_5};	/*Property = ScreenPixelsWidth*/
const Profile P20722 = {"20722", &P20722_4};	/*Profile ID - 20722*/
const Profile P17358 = {"17358", NULL};	/*Profile ID - 17358*/
const Value V24416_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24416_279 = {S1, 0, &V24416_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24416_282_57153 = {S2, NULL};	/*Value = True*/
const Property P24416_282 = {S3, 0, &V24416_282_57153, &P24416_279};	/*Property = BlobBuilder*/
const Value V24416_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24416_350 = {S4, 0, &V24416_350_57444, &P24416_282};	/*Property = Canvas*/
const Value V24416_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24416_283 = {S5, 0, &V24416_283_57159, &P24416_350};	/*Property = CssBackground*/
const Value V24416_284_57163 = {S0, NULL};	/*Value = False*/
const Property P24416_284 = {S6, 0, &V24416_284_57163, &P24416_283};	/*Property = CssBorderImage*/
const Value V24416_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24416_285 = {S7, 0, &V24416_285_57166, &P24416_284};	/*Property = CssCanvas*/
const Value V24416_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24416_286 = {S8, 0, &V24416_286_57168, &P24416_285};	/*Property = CssColor*/
const Value V24416_237_55784 = {S2, NULL};	/*Value = True*/
const Property P24416_237 = {S9, 0, &V24416_237_55784, &P24416_286};	/*Property = CssColumn*/
const Value V24416_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24416_287 = {S10, 0, &V24416_287_57172, &P24416_237};	/*Property = CssFlexbox*/
const Value V24416_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24416_288 = {S11, 0, &V24416_288_57175, &P24416_287};	/*Property = CssFont*/
const Value V24416_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24416_289 = {S12, 0, &V24416_289_57759, &P24416_288};	/*Property = CssImages*/
const Value V24416_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24416_291 = {S13, 0, &V24416_291_57180, &P24416_289};	/*Property = CssMediaQueries*/
const Value V24416_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24416_292 = {S14, 0, &V24416_292_57183, &P24416_291};	/*Property = CssMinMax*/
const Value V24416_293_57186 = {S2, NULL};	/*Value = True*/
const Property P24416_293 = {S15, 0, &V24416_293_57186, &P24416_292};	/*Property = CssOverflow*/
const Value V24416_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24416_294 = {S16, 0, &V24416_294_57189, &P24416_293};	/*Property = CssPosition*/
const Value V24416_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24416_295 = {S17, 0, &V24416_295_57192, &P24416_294};	/*Property = CssText*/
const Value V24416_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24416_297 = {S18, 0, &V24416_297_57195, &P24416_295};	/*Property = CssTransforms*/
const Value V24416_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24416_241 = {S19, 0, &V24416_241_55799, &P24416_297};	/*Property = CssTransitions*/
const Value V24416_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24416_298 = {S20, 0, &V24416_298_57206, &P24416_241};	/*Property = CssUI*/
const Value V24416_300_57216 = {S0, NULL};	/*Value = False*/
const Property P24416_300 = {S21, 0, &V24416_300_57216, &P24416_298};	/*Property = DataSet*/
const Value V24416_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24416_301 = {S22, 0, &V24416_301_57218, &P24416_300};	/*Property = DataUrl*/
const Value V24416_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24416_302 = {S23, 0, &V24416_302_57221, &P24416_301};	/*Property = DeviceOrientation*/
const Value V24416_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24416_304 = {S24, 0, &V24416_304_57227, &P24416_302};	/*Property = FileReader*/
const Value V24416_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24416_305 = {S25, 0, &V24416_305_57231, &P24416_304};	/*Property = FileSaver*/
const Value V24416_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24416_306 = {S26, 0, &V24416_306_57237, &P24416_305};	/*Property = FileWriter*/
const Value V24416_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24416_307 = {S27, 0, &V24416_307_57239, &P24416_306};	/*Property = FormData*/
const Value V24416_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24416_309 = {S28, 0, &V24416_309_57246, &P24416_307};	/*Property = Fullscreen*/
const Value V24416_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24416_310 = {S29, 0, &V24416_310_57248, &P24416_309};	/*Property = GeoLocation*/
const Value V24416_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24416_312 = {S30, 0, &V24416_312_57254, &P24416_310};	/*Property = History*/
const Value V24416_314_57261 = {S0, NULL};	/*Value = False*/
const Property P24416_314 = {S31, 0, &V24416_314_57261, &P24416_312};	/*Property = Html5*/
const Value V24416_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24416_313 = {S32, 0, &V24416_313_57258, &P24416_314};	/*Property = Html-Media-Capture*/
const Value V24416_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24416_315 = {S33, 0, &V24416_315_57264, &P24416_313};	/*Property = Iframe*/
const Value V24416_316_57267 = {S0, NULL};	/*Value = False*/
const Property P24416_316 = {S34, 0, &V24416_316_57267, &P24416_315};	/*Property = IndexedDB*/
const Value V24416_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24416_317 = {S35, 0, &V24416_317_57269, &P24416_316};	/*Property = Json*/
const Value V24416_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24416_15 = {S37, 0, &V24416_15_28, &P24416_317};	/*Property = LayoutEngine*/
const Value V24416_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24416_344 = {S38, 0, &V24416_344_57272, &P24416_15};	/*Property = Masking*/
const Value V24416_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24416_320 = {S39, 0, &V24416_320_57290, &P24416_344};	/*Property = PostMessage*/
const Value V24416_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24416_321 = {S40, 0, &V24416_321_57293, &P24416_320};	/*Property = Progress*/
const Value V24416_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24416_322 = {S41, 0, &V24416_322_57296, &P24416_321};	/*Property = Prompts*/
const Value V24416_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24416_323 = {S42, 0, &V24416_323_57299, &P24416_322};	/*Property = Selector*/
const Value V24416_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24416_324 = {S43, 0, &V24416_324_57306, &P24416_323};	/*Property = Svg*/
const Value V24416_327_57315 = {S0, NULL};	/*Value = False*/
const Property P24416_327 = {S44, 0, &V24416_327_57315, &P24416_324};	/*Property = TouchEvents*/
const Value V24416_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24416_328 = {S45, 0, &V24416_328_57318, &P24416_327};	/*Property = Track*/
const Value V24416_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24416_329 = {S46, 0, &V24416_329_57323, &P24416_328};	/*Property = Video*/
const Value V24416_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24416_330 = {S47, 0, &V24416_330_57326, &P24416_329};	/*Property = Viewport*/
const Value V24416_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24416_332 = {S48, 0, &V24416_332_57338, &P24416_330};	/*Property = WebWorkers*/
const Value V24416_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24416_333 = {S49, 0, &V24416_333_57342, &P24416_332};	/*Property = Xhr2*/
const Profile P24416 = {"24416", &P24416_333};	/*Profile ID - 24416*/
const Value V21900_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21900_279 = {S1, 0, &V21900_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21900_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21900_282 = {S3, 0, &V21900_282_57154, &P21900_279};	/*Property = BlobBuilder*/
const Value V21900_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21900_350 = {S4, 0, &V21900_350_57444, &P21900_282};	/*Property = Canvas*/
const Value V21900_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21900_283 = {S5, 0, &V21900_283_57159, &P21900_350};	/*Property = CssBackground*/
const Value V21900_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21900_284 = {S6, 0, &V21900_284_57163, &P21900_283};	/*Property = CssBorderImage*/
const Value V21900_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21900_285 = {S7, 0, &V21900_285_57166, &P21900_284};	/*Property = CssCanvas*/
const Value V21900_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21900_286 = {S8, 0, &V21900_286_57168, &P21900_285};	/*Property = CssColor*/
const Value V21900_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21900_237 = {S9, 0, &V21900_237_55784, &P21900_286};	/*Property = CssColumn*/
const Value V21900_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21900_287 = {S10, 0, &V21900_287_57172, &P21900_237};	/*Property = CssFlexbox*/
const Value V21900_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21900_288 = {S11, 0, &V21900_288_57175, &P21900_287};	/*Property = CssFont*/
const Value V21900_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21900_289 = {S12, 0, &V21900_289_57759, &P21900_288};	/*Property = CssImages*/
const Value V21900_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21900_291 = {S13, 0, &V21900_291_57180, &P21900_289};	/*Property = CssMediaQueries*/
const Value V21900_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21900_292 = {S14, 0, &V21900_292_57183, &P21900_291};	/*Property = CssMinMax*/
const Value V21900_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21900_293 = {S15, 0, &V21900_293_57187, &P21900_292};	/*Property = CssOverflow*/
const Value V21900_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21900_294 = {S16, 0, &V21900_294_57189, &P21900_293};	/*Property = CssPosition*/
const Value V21900_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21900_295 = {S17, 0, &V21900_295_57192, &P21900_294};	/*Property = CssText*/
const Value V21900_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21900_297 = {S18, 0, &V21900_297_57195, &P21900_295};	/*Property = CssTransforms*/
const Value V21900_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21900_241 = {S19, 0, &V21900_241_55799, &P21900_297};	/*Property = CssTransitions*/
const Value V21900_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21900_298 = {S20, 0, &V21900_298_57206, &P21900_241};	/*Property = CssUI*/
const Value V21900_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21900_300 = {S21, 0, &V21900_300_57216, &P21900_298};	/*Property = DataSet*/
const Value V21900_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21900_301 = {S22, 0, &V21900_301_57218, &P21900_300};	/*Property = DataUrl*/
const Value V21900_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21900_302 = {S23, 0, &V21900_302_57222, &P21900_301};	/*Property = DeviceOrientation*/
const Value V21900_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21900_304 = {S24, 0, &V21900_304_57227, &P21900_302};	/*Property = FileReader*/
const Value V21900_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21900_305 = {S25, 0, &V21900_305_57231, &P21900_304};	/*Property = FileSaver*/
const Value V21900_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21900_306 = {S26, 0, &V21900_306_57237, &P21900_305};	/*Property = FileWriter*/
const Value V21900_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21900_307 = {S27, 0, &V21900_307_57239, &P21900_306};	/*Property = FormData*/
const Value V21900_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21900_309 = {S28, 0, &V21900_309_57246, &P21900_307};	/*Property = Fullscreen*/
const Value V21900_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21900_310 = {S29, 0, &V21900_310_57248, &P21900_309};	/*Property = GeoLocation*/
const Value V21900_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21900_312 = {S30, 0, &V21900_312_57255, &P21900_310};	/*Property = History*/
const Value V21900_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21900_314 = {S31, 0, &V21900_314_57261, &P21900_312};	/*Property = Html5*/
const Value V21900_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21900_313 = {S32, 0, &V21900_313_57258, &P21900_314};	/*Property = Html-Media-Capture*/
const Value V21900_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21900_315 = {S33, 0, &V21900_315_57264, &P21900_313};	/*Property = Iframe*/
const Value V21900_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21900_316 = {S34, 0, &V21900_316_57266, &P21900_315};	/*Property = IndexedDB*/
const Value V21900_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21900_317 = {S35, 0, &V21900_317_57269, &P21900_316};	/*Property = Json*/
const Value V21900_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21900_15 = {S37, 0, &V21900_15_28, &P21900_317};	/*Property = LayoutEngine*/
const Value V21900_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21900_344 = {S38, 0, &V21900_344_57272, &P21900_15};	/*Property = Masking*/
const Value V21900_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21900_320 = {S39, 0, &V21900_320_57290, &P21900_344};	/*Property = PostMessage*/
const Value V21900_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21900_321 = {S40, 0, &V21900_321_57293, &P21900_320};	/*Property = Progress*/
const Value V21900_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21900_322 = {S41, 0, &V21900_322_57296, &P21900_321};	/*Property = Prompts*/
const Value V21900_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21900_323 = {S42, 0, &V21900_323_57299, &P21900_322};	/*Property = Selector*/
const Value V21900_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21900_324 = {S43, 0, &V21900_324_57306, &P21900_323};	/*Property = Svg*/
const Value V21900_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21900_327 = {S44, 0, &V21900_327_57315, &P21900_324};	/*Property = TouchEvents*/
const Value V21900_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21900_328 = {S45, 0, &V21900_328_57318, &P21900_327};	/*Property = Track*/
const Value V21900_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21900_329 = {S46, 0, &V21900_329_57323, &P21900_328};	/*Property = Video*/
const Value V21900_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21900_330 = {S47, 0, &V21900_330_57326, &P21900_329};	/*Property = Viewport*/
const Value V21900_332_57338 = {S2, NULL};	/*Value = True*/
const Property P21900_332 = {S48, 0, &V21900_332_57338, &P21900_330};	/*Property = WebWorkers*/
const Value V21900_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21900_333 = {S49, 0, &V21900_333_57342, &P21900_332};	/*Property = Xhr2*/
const Profile P21900 = {"21900", &P21900_333};	/*Profile ID - 21900*/
const Value V16436_1_1 = {S2, NULL};	/*Value = True*/
const Property P16436_1 = {S50, 0, &V16436_1_1, NULL};	/*Property = IsMobile*/
const Value V16436_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16436_5 = {S52, 0, &V16436_5_1227, &P16436_1};	/*Property = ScreenPixelsHeight*/
const Value V16436_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16436_4 = {S54, 0, &V16436_4_1229, &P16436_5};	/*Property = ScreenPixelsWidth*/
const Profile P16436 = {"16436", &P16436_4};	/*Profile ID - 16436*/
const Value V17482_1_1 = {S2, NULL};	/*Value = True*/
const Property P17482_1 = {S50, 0, &V17482_1_1, NULL};	/*Property = IsMobile*/
const Value V17482_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P17482_5 = {S52, 0, &V17482_5_1458, &P17482_1};	/*Property = ScreenPixelsHeight*/
const Value V17482_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P17482_4 = {S54, 0, &V17482_4_37617, &P17482_5};	/*Property = ScreenPixelsWidth*/
const Profile P17482 = {"17482", &P17482_4};	/*Profile ID - 17482*/
const Value V21529_1_1 = {S2, NULL};	/*Value = True*/
const Property P21529_1 = {S50, 0, &V21529_1_1, NULL};	/*Property = IsMobile*/
const Value V21529_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21529_5 = {S52, 0, &V21529_5_37, &P21529_1};	/*Property = ScreenPixelsHeight*/
const Value V21529_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21529_4 = {S54, 0, &V21529_4_36, &P21529_5};	/*Property = ScreenPixelsWidth*/
const Profile P21529 = {"21529", &P21529_4};	/*Profile ID - 21529*/
const Value V23884_1_1 = {S2, NULL};	/*Value = True*/
const Property P23884_1 = {S50, 0, &V23884_1_1, NULL};	/*Property = IsMobile*/
const Value V23884_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23884_5 = {S52, 0, &V23884_5_1227, &P23884_1};	/*Property = ScreenPixelsHeight*/
const Value V23884_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23884_4 = {S54, 0, &V23884_4_1229, &P23884_5};	/*Property = ScreenPixelsWidth*/
const Profile P23884 = {"23884", &P23884_4};	/*Profile ID - 23884*/
const Value V17954_1_1 = {S2, NULL};	/*Value = True*/
const Property P17954_1 = {S50, 0, &V17954_1_1, NULL};	/*Property = IsMobile*/
const Value V17954_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P17954_5 = {S52, 0, &V17954_5_1458, &P17954_1};	/*Property = ScreenPixelsHeight*/
const Value V17954_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P17954_4 = {S54, 0, &V17954_4_1460, &P17954_5};	/*Property = ScreenPixelsWidth*/
const Profile P17954 = {"17954", &P17954_4};	/*Profile ID - 17954*/
const Value V15842_1_1 = {S2, NULL};	/*Value = True*/
const Property P15842_1 = {S50, 0, &V15842_1_1, NULL};	/*Property = IsMobile*/
const Value V15842_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15842_5 = {S52, 0, &V15842_5_1227, &P15842_1};	/*Property = ScreenPixelsHeight*/
const Value V15842_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P15842_4 = {S54, 0, &V15842_4_1943, &P15842_5};	/*Property = ScreenPixelsWidth*/
const Profile P15842 = {"15842", &P15842_4};	/*Profile ID - 15842*/
const Value V24666_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_279 = {S1, 0, &V24666_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24666_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_282 = {S3, 0, &V24666_282_57155, &P24666_279};	/*Property = BlobBuilder*/
const Value V24666_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_350 = {S4, 0, &V24666_350_57446, &P24666_282};	/*Property = Canvas*/
const Value V24666_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_283 = {S5, 0, &V24666_283_57161, &P24666_350};	/*Property = CssBackground*/
const Value V24666_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_284 = {S6, 0, &V24666_284_57164, &P24666_283};	/*Property = CssBorderImage*/
const Value V24666_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_285 = {S7, 0, &V24666_285_57167, &P24666_284};	/*Property = CssCanvas*/
const Value V24666_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_286 = {S8, 0, &V24666_286_57170, &P24666_285};	/*Property = CssColor*/
const Value V24666_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_237 = {S9, 0, &V24666_237_55786, &P24666_286};	/*Property = CssColumn*/
const Value V24666_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_287 = {S10, 0, &V24666_287_57173, &P24666_237};	/*Property = CssFlexbox*/
const Value V24666_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_288 = {S11, 0, &V24666_288_57176, &P24666_287};	/*Property = CssFont*/
const Value V24666_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_289 = {S12, 0, &V24666_289_86899, &P24666_288};	/*Property = CssImages*/
const Value V24666_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_291 = {S13, 0, &V24666_291_57182, &P24666_289};	/*Property = CssMediaQueries*/
const Value V24666_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_292 = {S14, 0, &V24666_292_57185, &P24666_291};	/*Property = CssMinMax*/
const Value V24666_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_293 = {S15, 0, &V24666_293_57188, &P24666_292};	/*Property = CssOverflow*/
const Value V24666_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_294 = {S16, 0, &V24666_294_57191, &P24666_293};	/*Property = CssPosition*/
const Value V24666_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_295 = {S17, 0, &V24666_295_57194, &P24666_294};	/*Property = CssText*/
const Value V24666_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_297 = {S18, 0, &V24666_297_57197, &P24666_295};	/*Property = CssTransforms*/
const Value V24666_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_241 = {S19, 0, &V24666_241_55801, &P24666_297};	/*Property = CssTransitions*/
const Value V24666_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_298 = {S20, 0, &V24666_298_57208, &P24666_241};	/*Property = CssUI*/
const Value V24666_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_300 = {S21, 0, &V24666_300_57217, &P24666_298};	/*Property = DataSet*/
const Value V24666_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_301 = {S22, 0, &V24666_301_57220, &P24666_300};	/*Property = DataUrl*/
const Value V24666_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_302 = {S23, 0, &V24666_302_57223, &P24666_301};	/*Property = DeviceOrientation*/
const Value V24666_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_304 = {S24, 0, &V24666_304_57229, &P24666_302};	/*Property = FileReader*/
const Value V24666_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_305 = {S25, 0, &V24666_305_57232, &P24666_304};	/*Property = FileSaver*/
const Value V24666_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_306 = {S26, 0, &V24666_306_57238, &P24666_305};	/*Property = FileWriter*/
const Value V24666_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_307 = {S27, 0, &V24666_307_57241, &P24666_306};	/*Property = FormData*/
const Value V24666_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_309 = {S28, 0, &V24666_309_57247, &P24666_307};	/*Property = Fullscreen*/
const Value V24666_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_310 = {S29, 0, &V24666_310_57250, &P24666_309};	/*Property = GeoLocation*/
const Value V24666_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_312 = {S30, 0, &V24666_312_57256, &P24666_310};	/*Property = History*/
const Value V24666_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_314 = {S31, 0, &V24666_314_57262, &P24666_312};	/*Property = Html5*/
const Value V24666_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_313 = {S32, 0, &V24666_313_57259, &P24666_314};	/*Property = Html-Media-Capture*/
const Value V24666_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_315 = {S33, 0, &V24666_315_57265, &P24666_313};	/*Property = Iframe*/
const Value V24666_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_316 = {S34, 0, &V24666_316_57268, &P24666_315};	/*Property = IndexedDB*/
const Value V24666_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_317 = {S35, 0, &V24666_317_57271, &P24666_316};	/*Property = Json*/
const Value V24666_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24666_15 = {S37, 0, &V24666_15_90837, &P24666_317};	/*Property = LayoutEngine*/
const Value V24666_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_344 = {S38, 0, &V24666_344_57274, &P24666_15};	/*Property = Masking*/
const Value V24666_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_320 = {S39, 0, &V24666_320_57292, &P24666_344};	/*Property = PostMessage*/
const Value V24666_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_321 = {S40, 0, &V24666_321_57295, &P24666_320};	/*Property = Progress*/
const Value V24666_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_322 = {S41, 0, &V24666_322_57298, &P24666_321};	/*Property = Prompts*/
const Value V24666_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_323 = {S42, 0, &V24666_323_57301, &P24666_322};	/*Property = Selector*/
const Value V24666_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_324 = {S43, 0, &V24666_324_57307, &P24666_323};	/*Property = Svg*/
const Value V24666_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_327 = {S44, 0, &V24666_327_57316, &P24666_324};	/*Property = TouchEvents*/
const Value V24666_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_328 = {S45, 0, &V24666_328_57319, &P24666_327};	/*Property = Track*/
const Value V24666_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_329 = {S46, 0, &V24666_329_57325, &P24666_328};	/*Property = Video*/
const Value V24666_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_330 = {S47, 0, &V24666_330_57328, &P24666_329};	/*Property = Viewport*/
const Value V24666_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_332 = {S48, 0, &V24666_332_57340, &P24666_330};	/*Property = WebWorkers*/
const Value V24666_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24666_333 = {S49, 0, &V24666_333_57343, &P24666_332};	/*Property = Xhr2*/
const Profile P24666 = {"24666", &P24666_333};	/*Profile ID - 24666*/
const Value V17997_1_1 = {S2, NULL};	/*Value = True*/
const Property P17997_1 = {S50, 0, &V17997_1_1, NULL};	/*Property = IsMobile*/
const Value V17997_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17997_5 = {S52, 0, &V17997_5_1277, &P17997_1};	/*Property = ScreenPixelsHeight*/
const Value V17997_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P17997_4 = {S54, 0, &V17997_4_2336, &P17997_5};	/*Property = ScreenPixelsWidth*/
const Profile P17997 = {"17997", &P17997_4};	/*Profile ID - 17997*/
const Value V22410_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22410_279 = {S1, 0, &V22410_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22410_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22410_282 = {S3, 0, &V22410_282_57154, &P22410_279};	/*Property = BlobBuilder*/
const Value V22410_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22410_350 = {S4, 0, &V22410_350_57444, &P22410_282};	/*Property = Canvas*/
const Value V22410_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22410_283 = {S5, 0, &V22410_283_57159, &P22410_350};	/*Property = CssBackground*/
const Value V22410_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22410_284 = {S6, 0, &V22410_284_57163, &P22410_283};	/*Property = CssBorderImage*/
const Value V22410_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22410_285 = {S7, 0, &V22410_285_57166, &P22410_284};	/*Property = CssCanvas*/
const Value V22410_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22410_286 = {S8, 0, &V22410_286_57168, &P22410_285};	/*Property = CssColor*/
const Value V22410_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22410_237 = {S9, 0, &V22410_237_55784, &P22410_286};	/*Property = CssColumn*/
const Value V22410_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22410_287 = {S10, 0, &V22410_287_57172, &P22410_237};	/*Property = CssFlexbox*/
const Value V22410_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22410_288 = {S11, 0, &V22410_288_57175, &P22410_287};	/*Property = CssFont*/
const Value V22410_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22410_289 = {S12, 0, &V22410_289_57759, &P22410_288};	/*Property = CssImages*/
const Value V22410_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22410_291 = {S13, 0, &V22410_291_57180, &P22410_289};	/*Property = CssMediaQueries*/
const Value V22410_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22410_292 = {S14, 0, &V22410_292_57183, &P22410_291};	/*Property = CssMinMax*/
const Value V22410_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22410_293 = {S15, 0, &V22410_293_57187, &P22410_292};	/*Property = CssOverflow*/
const Value V22410_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22410_294 = {S16, 0, &V22410_294_57189, &P22410_293};	/*Property = CssPosition*/
const Value V22410_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22410_295 = {S17, 0, &V22410_295_57192, &P22410_294};	/*Property = CssText*/
const Value V22410_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22410_297 = {S18, 0, &V22410_297_57195, &P22410_295};	/*Property = CssTransforms*/
const Value V22410_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22410_241 = {S19, 0, &V22410_241_55799, &P22410_297};	/*Property = CssTransitions*/
const Value V22410_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22410_298 = {S20, 0, &V22410_298_57206, &P22410_241};	/*Property = CssUI*/
const Value V22410_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22410_300 = {S21, 0, &V22410_300_57216, &P22410_298};	/*Property = DataSet*/
const Value V22410_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22410_301 = {S22, 0, &V22410_301_57218, &P22410_300};	/*Property = DataUrl*/
const Value V22410_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22410_302 = {S23, 0, &V22410_302_57222, &P22410_301};	/*Property = DeviceOrientation*/
const Value V22410_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22410_304 = {S24, 0, &V22410_304_57227, &P22410_302};	/*Property = FileReader*/
const Value V22410_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22410_305 = {S25, 0, &V22410_305_57231, &P22410_304};	/*Property = FileSaver*/
const Value V22410_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22410_306 = {S26, 0, &V22410_306_57237, &P22410_305};	/*Property = FileWriter*/
const Value V22410_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22410_307 = {S27, 0, &V22410_307_57239, &P22410_306};	/*Property = FormData*/
const Value V22410_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22410_309 = {S28, 0, &V22410_309_57246, &P22410_307};	/*Property = Fullscreen*/
const Value V22410_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22410_310 = {S29, 0, &V22410_310_57248, &P22410_309};	/*Property = GeoLocation*/
const Value V22410_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22410_312 = {S30, 0, &V22410_312_57255, &P22410_310};	/*Property = History*/
const Value V22410_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22410_314 = {S31, 0, &V22410_314_57261, &P22410_312};	/*Property = Html5*/
const Value V22410_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22410_313 = {S32, 0, &V22410_313_57258, &P22410_314};	/*Property = Html-Media-Capture*/
const Value V22410_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22410_315 = {S33, 0, &V22410_315_57264, &P22410_313};	/*Property = Iframe*/
const Value V22410_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22410_316 = {S34, 0, &V22410_316_57266, &P22410_315};	/*Property = IndexedDB*/
const Value V22410_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22410_317 = {S35, 0, &V22410_317_57269, &P22410_316};	/*Property = Json*/
const Value V22410_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22410_15 = {S37, 0, &V22410_15_28, &P22410_317};	/*Property = LayoutEngine*/
const Value V22410_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22410_344 = {S38, 0, &V22410_344_57272, &P22410_15};	/*Property = Masking*/
const Value V22410_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22410_320 = {S39, 0, &V22410_320_57290, &P22410_344};	/*Property = PostMessage*/
const Value V22410_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22410_321 = {S40, 0, &V22410_321_57293, &P22410_320};	/*Property = Progress*/
const Value V22410_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22410_322 = {S41, 0, &V22410_322_57296, &P22410_321};	/*Property = Prompts*/
const Value V22410_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22410_323 = {S42, 0, &V22410_323_57299, &P22410_322};	/*Property = Selector*/
const Value V22410_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22410_324 = {S43, 0, &V22410_324_57306, &P22410_323};	/*Property = Svg*/
const Value V22410_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22410_327 = {S44, 0, &V22410_327_57315, &P22410_324};	/*Property = TouchEvents*/
const Value V22410_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22410_328 = {S45, 0, &V22410_328_57318, &P22410_327};	/*Property = Track*/
const Value V22410_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22410_329 = {S46, 0, &V22410_329_57323, &P22410_328};	/*Property = Video*/
const Value V22410_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22410_330 = {S47, 0, &V22410_330_57326, &P22410_329};	/*Property = Viewport*/
const Value V22410_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22410_332 = {S48, 0, &V22410_332_57338, &P22410_330};	/*Property = WebWorkers*/
const Value V22410_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22410_333 = {S49, 0, &V22410_333_57342, &P22410_332};	/*Property = Xhr2*/
const Profile P22410 = {"22410", &P22410_333};	/*Profile ID - 22410*/
const Profile P17819 = {"17819", NULL};	/*Profile ID - 17819*/
const Value V22411_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22411_279 = {S1, 0, &V22411_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22411_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22411_282 = {S3, 0, &V22411_282_57154, &P22411_279};	/*Property = BlobBuilder*/
const Value V22411_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22411_350 = {S4, 0, &V22411_350_57444, &P22411_282};	/*Property = Canvas*/
const Value V22411_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22411_283 = {S5, 0, &V22411_283_57159, &P22411_350};	/*Property = CssBackground*/
const Value V22411_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22411_284 = {S6, 0, &V22411_284_57163, &P22411_283};	/*Property = CssBorderImage*/
const Value V22411_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22411_285 = {S7, 0, &V22411_285_57166, &P22411_284};	/*Property = CssCanvas*/
const Value V22411_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22411_286 = {S8, 0, &V22411_286_57168, &P22411_285};	/*Property = CssColor*/
const Value V22411_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22411_237 = {S9, 0, &V22411_237_55784, &P22411_286};	/*Property = CssColumn*/
const Value V22411_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22411_287 = {S10, 0, &V22411_287_57172, &P22411_237};	/*Property = CssFlexbox*/
const Value V22411_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22411_288 = {S11, 0, &V22411_288_57175, &P22411_287};	/*Property = CssFont*/
const Value V22411_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22411_289 = {S12, 0, &V22411_289_57759, &P22411_288};	/*Property = CssImages*/
const Value V22411_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22411_291 = {S13, 0, &V22411_291_57180, &P22411_289};	/*Property = CssMediaQueries*/
const Value V22411_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22411_292 = {S14, 0, &V22411_292_57183, &P22411_291};	/*Property = CssMinMax*/
const Value V22411_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22411_293 = {S15, 0, &V22411_293_57187, &P22411_292};	/*Property = CssOverflow*/
const Value V22411_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22411_294 = {S16, 0, &V22411_294_57189, &P22411_293};	/*Property = CssPosition*/
const Value V22411_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22411_295 = {S17, 0, &V22411_295_57192, &P22411_294};	/*Property = CssText*/
const Value V22411_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22411_297 = {S18, 0, &V22411_297_57195, &P22411_295};	/*Property = CssTransforms*/
const Value V22411_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22411_241 = {S19, 0, &V22411_241_55799, &P22411_297};	/*Property = CssTransitions*/
const Value V22411_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22411_298 = {S20, 0, &V22411_298_57206, &P22411_241};	/*Property = CssUI*/
const Value V22411_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22411_300 = {S21, 0, &V22411_300_57216, &P22411_298};	/*Property = DataSet*/
const Value V22411_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22411_301 = {S22, 0, &V22411_301_57218, &P22411_300};	/*Property = DataUrl*/
const Value V22411_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22411_302 = {S23, 0, &V22411_302_57222, &P22411_301};	/*Property = DeviceOrientation*/
const Value V22411_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22411_304 = {S24, 0, &V22411_304_57227, &P22411_302};	/*Property = FileReader*/
const Value V22411_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22411_305 = {S25, 0, &V22411_305_57231, &P22411_304};	/*Property = FileSaver*/
const Value V22411_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22411_306 = {S26, 0, &V22411_306_57237, &P22411_305};	/*Property = FileWriter*/
const Value V22411_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22411_307 = {S27, 0, &V22411_307_57239, &P22411_306};	/*Property = FormData*/
const Value V22411_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22411_309 = {S28, 0, &V22411_309_57246, &P22411_307};	/*Property = Fullscreen*/
const Value V22411_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22411_310 = {S29, 0, &V22411_310_57248, &P22411_309};	/*Property = GeoLocation*/
const Value V22411_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22411_312 = {S30, 0, &V22411_312_57255, &P22411_310};	/*Property = History*/
const Value V22411_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22411_314 = {S31, 0, &V22411_314_57261, &P22411_312};	/*Property = Html5*/
const Value V22411_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22411_313 = {S32, 0, &V22411_313_57258, &P22411_314};	/*Property = Html-Media-Capture*/
const Value V22411_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22411_315 = {S33, 0, &V22411_315_57264, &P22411_313};	/*Property = Iframe*/
const Value V22411_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22411_316 = {S34, 0, &V22411_316_57266, &P22411_315};	/*Property = IndexedDB*/
const Value V22411_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22411_317 = {S35, 0, &V22411_317_57269, &P22411_316};	/*Property = Json*/
const Value V22411_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22411_15 = {S37, 0, &V22411_15_28, &P22411_317};	/*Property = LayoutEngine*/
const Value V22411_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22411_344 = {S38, 0, &V22411_344_57272, &P22411_15};	/*Property = Masking*/
const Value V22411_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22411_320 = {S39, 0, &V22411_320_57290, &P22411_344};	/*Property = PostMessage*/
const Value V22411_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22411_321 = {S40, 0, &V22411_321_57293, &P22411_320};	/*Property = Progress*/
const Value V22411_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22411_322 = {S41, 0, &V22411_322_57296, &P22411_321};	/*Property = Prompts*/
const Value V22411_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22411_323 = {S42, 0, &V22411_323_57299, &P22411_322};	/*Property = Selector*/
const Value V22411_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22411_324 = {S43, 0, &V22411_324_57306, &P22411_323};	/*Property = Svg*/
const Value V22411_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22411_327 = {S44, 0, &V22411_327_57315, &P22411_324};	/*Property = TouchEvents*/
const Value V22411_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22411_328 = {S45, 0, &V22411_328_57318, &P22411_327};	/*Property = Track*/
const Value V22411_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22411_329 = {S46, 0, &V22411_329_57323, &P22411_328};	/*Property = Video*/
const Value V22411_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22411_330 = {S47, 0, &V22411_330_57326, &P22411_329};	/*Property = Viewport*/
const Value V22411_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22411_332 = {S48, 0, &V22411_332_57338, &P22411_330};	/*Property = WebWorkers*/
const Value V22411_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22411_333 = {S49, 0, &V22411_333_57342, &P22411_332};	/*Property = Xhr2*/
const Profile P22411 = {"22411", &P22411_333};	/*Profile ID - 22411*/
const Value V13141_1_1 = {S2, NULL};	/*Value = True*/
const Property P13141_1 = {S50, 0, &V13141_1_1, NULL};	/*Property = IsMobile*/
const Value V13141_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13141_5 = {S52, 0, &V13141_5_1227, &P13141_1};	/*Property = ScreenPixelsHeight*/
const Value V13141_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13141_4 = {S54, 0, &V13141_4_1229, &P13141_5};	/*Property = ScreenPixelsWidth*/
const Profile P13141 = {"13141", &P13141_4};	/*Profile ID - 13141*/
const Value V18210_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_279 = {S1, 0, &V18210_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18210_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_282 = {S3, 0, &V18210_282_57155, &P18210_279};	/*Property = BlobBuilder*/
const Value V18210_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_350 = {S4, 0, &V18210_350_57446, &P18210_282};	/*Property = Canvas*/
const Value V18210_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_283 = {S5, 0, &V18210_283_57161, &P18210_350};	/*Property = CssBackground*/
const Value V18210_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_284 = {S6, 0, &V18210_284_57164, &P18210_283};	/*Property = CssBorderImage*/
const Value V18210_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_285 = {S7, 0, &V18210_285_57167, &P18210_284};	/*Property = CssCanvas*/
const Value V18210_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_286 = {S8, 0, &V18210_286_57170, &P18210_285};	/*Property = CssColor*/
const Value V18210_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_237 = {S9, 0, &V18210_237_55786, &P18210_286};	/*Property = CssColumn*/
const Value V18210_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_287 = {S10, 0, &V18210_287_57173, &P18210_237};	/*Property = CssFlexbox*/
const Value V18210_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_288 = {S11, 0, &V18210_288_57176, &P18210_287};	/*Property = CssFont*/
const Value V18210_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_289 = {S12, 0, &V18210_289_86899, &P18210_288};	/*Property = CssImages*/
const Value V18210_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_291 = {S13, 0, &V18210_291_57182, &P18210_289};	/*Property = CssMediaQueries*/
const Value V18210_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_292 = {S14, 0, &V18210_292_57185, &P18210_291};	/*Property = CssMinMax*/
const Value V18210_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_293 = {S15, 0, &V18210_293_57188, &P18210_292};	/*Property = CssOverflow*/
const Value V18210_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_294 = {S16, 0, &V18210_294_57191, &P18210_293};	/*Property = CssPosition*/
const Value V18210_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_295 = {S17, 0, &V18210_295_57194, &P18210_294};	/*Property = CssText*/
const Value V18210_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_297 = {S18, 0, &V18210_297_57197, &P18210_295};	/*Property = CssTransforms*/
const Value V18210_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_241 = {S19, 0, &V18210_241_55801, &P18210_297};	/*Property = CssTransitions*/
const Value V18210_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_298 = {S20, 0, &V18210_298_57208, &P18210_241};	/*Property = CssUI*/
const Value V18210_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_300 = {S21, 0, &V18210_300_57217, &P18210_298};	/*Property = DataSet*/
const Value V18210_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_301 = {S22, 0, &V18210_301_57220, &P18210_300};	/*Property = DataUrl*/
const Value V18210_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_302 = {S23, 0, &V18210_302_57223, &P18210_301};	/*Property = DeviceOrientation*/
const Value V18210_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_304 = {S24, 0, &V18210_304_57229, &P18210_302};	/*Property = FileReader*/
const Value V18210_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_305 = {S25, 0, &V18210_305_57232, &P18210_304};	/*Property = FileSaver*/
const Value V18210_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_306 = {S26, 0, &V18210_306_57238, &P18210_305};	/*Property = FileWriter*/
const Value V18210_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_307 = {S27, 0, &V18210_307_57241, &P18210_306};	/*Property = FormData*/
const Value V18210_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_309 = {S28, 0, &V18210_309_57247, &P18210_307};	/*Property = Fullscreen*/
const Value V18210_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_310 = {S29, 0, &V18210_310_57250, &P18210_309};	/*Property = GeoLocation*/
const Value V18210_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_312 = {S30, 0, &V18210_312_57256, &P18210_310};	/*Property = History*/
const Value V18210_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_314 = {S31, 0, &V18210_314_57262, &P18210_312};	/*Property = Html5*/
const Value V18210_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_313 = {S32, 0, &V18210_313_57259, &P18210_314};	/*Property = Html-Media-Capture*/
const Value V18210_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_315 = {S33, 0, &V18210_315_57265, &P18210_313};	/*Property = Iframe*/
const Value V18210_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_316 = {S34, 0, &V18210_316_57268, &P18210_315};	/*Property = IndexedDB*/
const Value V18210_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_317 = {S35, 0, &V18210_317_57271, &P18210_316};	/*Property = Json*/
const Value V18210_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18210_15 = {S37, 0, &V18210_15_29, &P18210_317};	/*Property = LayoutEngine*/
const Value V18210_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_344 = {S38, 0, &V18210_344_57274, &P18210_15};	/*Property = Masking*/
const Value V18210_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_320 = {S39, 0, &V18210_320_57292, &P18210_344};	/*Property = PostMessage*/
const Value V18210_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_321 = {S40, 0, &V18210_321_57295, &P18210_320};	/*Property = Progress*/
const Value V18210_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_322 = {S41, 0, &V18210_322_57298, &P18210_321};	/*Property = Prompts*/
const Value V18210_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_323 = {S42, 0, &V18210_323_57301, &P18210_322};	/*Property = Selector*/
const Value V18210_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_324 = {S43, 0, &V18210_324_57307, &P18210_323};	/*Property = Svg*/
const Value V18210_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_327 = {S44, 0, &V18210_327_57316, &P18210_324};	/*Property = TouchEvents*/
const Value V18210_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_328 = {S45, 0, &V18210_328_57319, &P18210_327};	/*Property = Track*/
const Value V18210_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_329 = {S46, 0, &V18210_329_57325, &P18210_328};	/*Property = Video*/
const Value V18210_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_330 = {S47, 0, &V18210_330_57328, &P18210_329};	/*Property = Viewport*/
const Value V18210_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_332 = {S48, 0, &V18210_332_57340, &P18210_330};	/*Property = WebWorkers*/
const Value V18210_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18210_333 = {S49, 0, &V18210_333_57343, &P18210_332};	/*Property = Xhr2*/
const Profile P18210 = {"18210", &P18210_333};	/*Profile ID - 18210*/
const Value V21902_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21902_279 = {S1, 0, &V21902_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21902_282_57153 = {S2, NULL};	/*Value = True*/
const Property P21902_282 = {S3, 0, &V21902_282_57153, &P21902_279};	/*Property = BlobBuilder*/
const Value V21902_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21902_350 = {S4, 0, &V21902_350_57444, &P21902_282};	/*Property = Canvas*/
const Value V21902_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21902_283 = {S5, 0, &V21902_283_57159, &P21902_350};	/*Property = CssBackground*/
const Value V21902_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21902_284 = {S6, 0, &V21902_284_57163, &P21902_283};	/*Property = CssBorderImage*/
const Value V21902_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21902_285 = {S7, 0, &V21902_285_57166, &P21902_284};	/*Property = CssCanvas*/
const Value V21902_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21902_286 = {S8, 0, &V21902_286_57168, &P21902_285};	/*Property = CssColor*/
const Value V21902_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21902_237 = {S9, 0, &V21902_237_55784, &P21902_286};	/*Property = CssColumn*/
const Value V21902_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21902_287 = {S10, 0, &V21902_287_57172, &P21902_237};	/*Property = CssFlexbox*/
const Value V21902_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21902_288 = {S11, 0, &V21902_288_57175, &P21902_287};	/*Property = CssFont*/
const Value V21902_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21902_289 = {S12, 0, &V21902_289_57759, &P21902_288};	/*Property = CssImages*/
const Value V21902_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21902_291 = {S13, 0, &V21902_291_57180, &P21902_289};	/*Property = CssMediaQueries*/
const Value V21902_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21902_292 = {S14, 0, &V21902_292_57183, &P21902_291};	/*Property = CssMinMax*/
const Value V21902_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21902_293 = {S15, 0, &V21902_293_57187, &P21902_292};	/*Property = CssOverflow*/
const Value V21902_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21902_294 = {S16, 0, &V21902_294_57189, &P21902_293};	/*Property = CssPosition*/
const Value V21902_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21902_295 = {S17, 0, &V21902_295_57192, &P21902_294};	/*Property = CssText*/
const Value V21902_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21902_297 = {S18, 0, &V21902_297_57195, &P21902_295};	/*Property = CssTransforms*/
const Value V21902_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21902_241 = {S19, 0, &V21902_241_55799, &P21902_297};	/*Property = CssTransitions*/
const Value V21902_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21902_298 = {S20, 0, &V21902_298_57206, &P21902_241};	/*Property = CssUI*/
const Value V21902_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21902_300 = {S21, 0, &V21902_300_57216, &P21902_298};	/*Property = DataSet*/
const Value V21902_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21902_301 = {S22, 0, &V21902_301_57218, &P21902_300};	/*Property = DataUrl*/
const Value V21902_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21902_302 = {S23, 0, &V21902_302_57222, &P21902_301};	/*Property = DeviceOrientation*/
const Value V21902_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21902_304 = {S24, 0, &V21902_304_57227, &P21902_302};	/*Property = FileReader*/
const Value V21902_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21902_305 = {S25, 0, &V21902_305_57231, &P21902_304};	/*Property = FileSaver*/
const Value V21902_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21902_306 = {S26, 0, &V21902_306_57237, &P21902_305};	/*Property = FileWriter*/
const Value V21902_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21902_307 = {S27, 0, &V21902_307_57239, &P21902_306};	/*Property = FormData*/
const Value V21902_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21902_309 = {S28, 0, &V21902_309_57246, &P21902_307};	/*Property = Fullscreen*/
const Value V21902_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21902_310 = {S29, 0, &V21902_310_57248, &P21902_309};	/*Property = GeoLocation*/
const Value V21902_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21902_312 = {S30, 0, &V21902_312_57254, &P21902_310};	/*Property = History*/
const Value V21902_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21902_314 = {S31, 0, &V21902_314_57261, &P21902_312};	/*Property = Html5*/
const Value V21902_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21902_313 = {S32, 0, &V21902_313_57258, &P21902_314};	/*Property = Html-Media-Capture*/
const Value V21902_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21902_315 = {S33, 0, &V21902_315_57264, &P21902_313};	/*Property = Iframe*/
const Value V21902_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21902_316 = {S34, 0, &V21902_316_57266, &P21902_315};	/*Property = IndexedDB*/
const Value V21902_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21902_317 = {S35, 0, &V21902_317_57269, &P21902_316};	/*Property = Json*/
const Value V21902_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21902_15 = {S37, 0, &V21902_15_28, &P21902_317};	/*Property = LayoutEngine*/
const Value V21902_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21902_344 = {S38, 0, &V21902_344_57272, &P21902_15};	/*Property = Masking*/
const Value V21902_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21902_320 = {S39, 0, &V21902_320_57290, &P21902_344};	/*Property = PostMessage*/
const Value V21902_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21902_321 = {S40, 0, &V21902_321_57293, &P21902_320};	/*Property = Progress*/
const Value V21902_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21902_322 = {S41, 0, &V21902_322_57296, &P21902_321};	/*Property = Prompts*/
const Value V21902_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21902_323 = {S42, 0, &V21902_323_57299, &P21902_322};	/*Property = Selector*/
const Value V21902_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21902_324 = {S43, 0, &V21902_324_57306, &P21902_323};	/*Property = Svg*/
const Value V21902_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21902_327 = {S44, 0, &V21902_327_57315, &P21902_324};	/*Property = TouchEvents*/
const Value V21902_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21902_328 = {S45, 0, &V21902_328_57318, &P21902_327};	/*Property = Track*/
const Value V21902_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21902_329 = {S46, 0, &V21902_329_57323, &P21902_328};	/*Property = Video*/
const Value V21902_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21902_330 = {S47, 0, &V21902_330_57326, &P21902_329};	/*Property = Viewport*/
const Value V21902_332_57338 = {S2, NULL};	/*Value = True*/
const Property P21902_332 = {S48, 0, &V21902_332_57338, &P21902_330};	/*Property = WebWorkers*/
const Value V21902_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21902_333 = {S49, 0, &V21902_333_57342, &P21902_332};	/*Property = Xhr2*/
const Profile P21902 = {"21902", &P21902_333};	/*Profile ID - 21902*/
const Value V22207_1_1 = {S2, NULL};	/*Value = True*/
const Property P22207_1 = {S50, 0, &V22207_1_1, NULL};	/*Property = IsMobile*/
const Value V22207_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22207_5 = {S52, 0, &V22207_5_1832, &P22207_1};	/*Property = ScreenPixelsHeight*/
const Value V22207_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22207_4 = {S54, 0, &V22207_4_2336, &P22207_5};	/*Property = ScreenPixelsWidth*/
const Profile P22207 = {"22207", &P22207_4};	/*Profile ID - 22207*/
const Value V21580_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_279 = {S1, 0, &V21580_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21580_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_282 = {S3, 0, &V21580_282_57155, &P21580_279};	/*Property = BlobBuilder*/
const Value V21580_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_350 = {S4, 0, &V21580_350_57446, &P21580_282};	/*Property = Canvas*/
const Value V21580_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_283 = {S5, 0, &V21580_283_57161, &P21580_350};	/*Property = CssBackground*/
const Value V21580_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_284 = {S6, 0, &V21580_284_57164, &P21580_283};	/*Property = CssBorderImage*/
const Value V21580_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_285 = {S7, 0, &V21580_285_57167, &P21580_284};	/*Property = CssCanvas*/
const Value V21580_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_286 = {S8, 0, &V21580_286_57170, &P21580_285};	/*Property = CssColor*/
const Value V21580_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_237 = {S9, 0, &V21580_237_55786, &P21580_286};	/*Property = CssColumn*/
const Value V21580_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_287 = {S10, 0, &V21580_287_57173, &P21580_237};	/*Property = CssFlexbox*/
const Value V21580_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_288 = {S11, 0, &V21580_288_57176, &P21580_287};	/*Property = CssFont*/
const Value V21580_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_289 = {S12, 0, &V21580_289_86899, &P21580_288};	/*Property = CssImages*/
const Value V21580_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_291 = {S13, 0, &V21580_291_57182, &P21580_289};	/*Property = CssMediaQueries*/
const Value V21580_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_292 = {S14, 0, &V21580_292_57185, &P21580_291};	/*Property = CssMinMax*/
const Value V21580_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_293 = {S15, 0, &V21580_293_57188, &P21580_292};	/*Property = CssOverflow*/
const Value V21580_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_294 = {S16, 0, &V21580_294_57191, &P21580_293};	/*Property = CssPosition*/
const Value V21580_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_295 = {S17, 0, &V21580_295_57194, &P21580_294};	/*Property = CssText*/
const Value V21580_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_297 = {S18, 0, &V21580_297_57197, &P21580_295};	/*Property = CssTransforms*/
const Value V21580_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_241 = {S19, 0, &V21580_241_55801, &P21580_297};	/*Property = CssTransitions*/
const Value V21580_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_298 = {S20, 0, &V21580_298_57208, &P21580_241};	/*Property = CssUI*/
const Value V21580_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_300 = {S21, 0, &V21580_300_57217, &P21580_298};	/*Property = DataSet*/
const Value V21580_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_301 = {S22, 0, &V21580_301_57220, &P21580_300};	/*Property = DataUrl*/
const Value V21580_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_302 = {S23, 0, &V21580_302_57223, &P21580_301};	/*Property = DeviceOrientation*/
const Value V21580_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_304 = {S24, 0, &V21580_304_57229, &P21580_302};	/*Property = FileReader*/
const Value V21580_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_305 = {S25, 0, &V21580_305_57232, &P21580_304};	/*Property = FileSaver*/
const Value V21580_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_306 = {S26, 0, &V21580_306_57238, &P21580_305};	/*Property = FileWriter*/
const Value V21580_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_307 = {S27, 0, &V21580_307_57241, &P21580_306};	/*Property = FormData*/
const Value V21580_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_309 = {S28, 0, &V21580_309_57247, &P21580_307};	/*Property = Fullscreen*/
const Value V21580_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_310 = {S29, 0, &V21580_310_57250, &P21580_309};	/*Property = GeoLocation*/
const Value V21580_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_312 = {S30, 0, &V21580_312_57256, &P21580_310};	/*Property = History*/
const Value V21580_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_314 = {S31, 0, &V21580_314_57262, &P21580_312};	/*Property = Html5*/
const Value V21580_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_313 = {S32, 0, &V21580_313_57259, &P21580_314};	/*Property = Html-Media-Capture*/
const Value V21580_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_315 = {S33, 0, &V21580_315_57265, &P21580_313};	/*Property = Iframe*/
const Value V21580_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_316 = {S34, 0, &V21580_316_57268, &P21580_315};	/*Property = IndexedDB*/
const Value V21580_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_317 = {S35, 0, &V21580_317_57271, &P21580_316};	/*Property = Json*/
const Value V21580_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21580_15 = {S37, 0, &V21580_15_28, &P21580_317};	/*Property = LayoutEngine*/
const Value V21580_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_344 = {S38, 0, &V21580_344_57274, &P21580_15};	/*Property = Masking*/
const Value V21580_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_320 = {S39, 0, &V21580_320_57292, &P21580_344};	/*Property = PostMessage*/
const Value V21580_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_321 = {S40, 0, &V21580_321_57295, &P21580_320};	/*Property = Progress*/
const Value V21580_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_322 = {S41, 0, &V21580_322_57298, &P21580_321};	/*Property = Prompts*/
const Value V21580_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_323 = {S42, 0, &V21580_323_57301, &P21580_322};	/*Property = Selector*/
const Value V21580_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_324 = {S43, 0, &V21580_324_57307, &P21580_323};	/*Property = Svg*/
const Value V21580_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_327 = {S44, 0, &V21580_327_57316, &P21580_324};	/*Property = TouchEvents*/
const Value V21580_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_328 = {S45, 0, &V21580_328_57319, &P21580_327};	/*Property = Track*/
const Value V21580_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_329 = {S46, 0, &V21580_329_57325, &P21580_328};	/*Property = Video*/
const Value V21580_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_330 = {S47, 0, &V21580_330_57328, &P21580_329};	/*Property = Viewport*/
const Value V21580_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_332 = {S48, 0, &V21580_332_57340, &P21580_330};	/*Property = WebWorkers*/
const Value V21580_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21580_333 = {S49, 0, &V21580_333_57343, &P21580_332};	/*Property = Xhr2*/
const Profile P21580 = {"21580", &P21580_333};	/*Profile ID - 21580*/
const Value V15850_1_1 = {S2, NULL};	/*Value = True*/
const Property P15850_1 = {S50, 0, &V15850_1_1, NULL};	/*Property = IsMobile*/
const Value V15850_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P15850_5 = {S52, 0, &V15850_5_9981, &P15850_1};	/*Property = ScreenPixelsHeight*/
const Value V15850_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P15850_4 = {S54, 0, &V15850_4_39098, &P15850_5};	/*Property = ScreenPixelsWidth*/
const Profile P15850 = {"15850", &P15850_4};	/*Profile ID - 15850*/
const Value V22412_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22412_279 = {S1, 0, &V22412_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22412_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22412_282 = {S3, 0, &V22412_282_57154, &P22412_279};	/*Property = BlobBuilder*/
const Value V22412_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22412_350 = {S4, 0, &V22412_350_57444, &P22412_282};	/*Property = Canvas*/
const Value V22412_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22412_283 = {S5, 0, &V22412_283_57159, &P22412_350};	/*Property = CssBackground*/
const Value V22412_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22412_284 = {S6, 0, &V22412_284_57163, &P22412_283};	/*Property = CssBorderImage*/
const Value V22412_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22412_285 = {S7, 0, &V22412_285_57166, &P22412_284};	/*Property = CssCanvas*/
const Value V22412_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22412_286 = {S8, 0, &V22412_286_57168, &P22412_285};	/*Property = CssColor*/
const Value V22412_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22412_237 = {S9, 0, &V22412_237_55784, &P22412_286};	/*Property = CssColumn*/
const Value V22412_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22412_287 = {S10, 0, &V22412_287_57172, &P22412_237};	/*Property = CssFlexbox*/
const Value V22412_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22412_288 = {S11, 0, &V22412_288_57175, &P22412_287};	/*Property = CssFont*/
const Value V22412_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22412_289 = {S12, 0, &V22412_289_57759, &P22412_288};	/*Property = CssImages*/
const Value V22412_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22412_291 = {S13, 0, &V22412_291_57180, &P22412_289};	/*Property = CssMediaQueries*/
const Value V22412_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22412_292 = {S14, 0, &V22412_292_57183, &P22412_291};	/*Property = CssMinMax*/
const Value V22412_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22412_293 = {S15, 0, &V22412_293_57187, &P22412_292};	/*Property = CssOverflow*/
const Value V22412_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22412_294 = {S16, 0, &V22412_294_57189, &P22412_293};	/*Property = CssPosition*/
const Value V22412_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22412_295 = {S17, 0, &V22412_295_57192, &P22412_294};	/*Property = CssText*/
const Value V22412_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22412_297 = {S18, 0, &V22412_297_57195, &P22412_295};	/*Property = CssTransforms*/
const Value V22412_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22412_241 = {S19, 0, &V22412_241_55799, &P22412_297};	/*Property = CssTransitions*/
const Value V22412_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22412_298 = {S20, 0, &V22412_298_57206, &P22412_241};	/*Property = CssUI*/
const Value V22412_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22412_300 = {S21, 0, &V22412_300_57216, &P22412_298};	/*Property = DataSet*/
const Value V22412_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22412_301 = {S22, 0, &V22412_301_57218, &P22412_300};	/*Property = DataUrl*/
const Value V22412_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22412_302 = {S23, 0, &V22412_302_57222, &P22412_301};	/*Property = DeviceOrientation*/
const Value V22412_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22412_304 = {S24, 0, &V22412_304_57227, &P22412_302};	/*Property = FileReader*/
const Value V22412_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22412_305 = {S25, 0, &V22412_305_57231, &P22412_304};	/*Property = FileSaver*/
const Value V22412_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22412_306 = {S26, 0, &V22412_306_57237, &P22412_305};	/*Property = FileWriter*/
const Value V22412_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22412_307 = {S27, 0, &V22412_307_57239, &P22412_306};	/*Property = FormData*/
const Value V22412_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22412_309 = {S28, 0, &V22412_309_57246, &P22412_307};	/*Property = Fullscreen*/
const Value V22412_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22412_310 = {S29, 0, &V22412_310_57248, &P22412_309};	/*Property = GeoLocation*/
const Value V22412_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22412_312 = {S30, 0, &V22412_312_57255, &P22412_310};	/*Property = History*/
const Value V22412_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22412_314 = {S31, 0, &V22412_314_57261, &P22412_312};	/*Property = Html5*/
const Value V22412_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22412_313 = {S32, 0, &V22412_313_57258, &P22412_314};	/*Property = Html-Media-Capture*/
const Value V22412_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22412_315 = {S33, 0, &V22412_315_57264, &P22412_313};	/*Property = Iframe*/
const Value V22412_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22412_316 = {S34, 0, &V22412_316_57266, &P22412_315};	/*Property = IndexedDB*/
const Value V22412_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22412_317 = {S35, 0, &V22412_317_57269, &P22412_316};	/*Property = Json*/
const Value V22412_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22412_15 = {S37, 0, &V22412_15_28, &P22412_317};	/*Property = LayoutEngine*/
const Value V22412_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22412_344 = {S38, 0, &V22412_344_57272, &P22412_15};	/*Property = Masking*/
const Value V22412_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22412_320 = {S39, 0, &V22412_320_57290, &P22412_344};	/*Property = PostMessage*/
const Value V22412_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22412_321 = {S40, 0, &V22412_321_57293, &P22412_320};	/*Property = Progress*/
const Value V22412_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22412_322 = {S41, 0, &V22412_322_57296, &P22412_321};	/*Property = Prompts*/
const Value V22412_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22412_323 = {S42, 0, &V22412_323_57299, &P22412_322};	/*Property = Selector*/
const Value V22412_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22412_324 = {S43, 0, &V22412_324_57306, &P22412_323};	/*Property = Svg*/
const Value V22412_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22412_327 = {S44, 0, &V22412_327_57315, &P22412_324};	/*Property = TouchEvents*/
const Value V22412_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22412_328 = {S45, 0, &V22412_328_57318, &P22412_327};	/*Property = Track*/
const Value V22412_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22412_329 = {S46, 0, &V22412_329_57323, &P22412_328};	/*Property = Video*/
const Value V22412_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22412_330 = {S47, 0, &V22412_330_57326, &P22412_329};	/*Property = Viewport*/
const Value V22412_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22412_332 = {S48, 0, &V22412_332_57338, &P22412_330};	/*Property = WebWorkers*/
const Value V22412_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22412_333 = {S49, 0, &V22412_333_57342, &P22412_332};	/*Property = Xhr2*/
const Profile P22412 = {"22412", &P22412_333};	/*Profile ID - 22412*/
const Value V17939_1_1 = {S2, NULL};	/*Value = True*/
const Property P17939_1 = {S50, 0, &V17939_1_1, NULL};	/*Property = IsMobile*/
const Value V17939_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P17939_5 = {S52, 0, &V17939_5_1832, &P17939_1};	/*Property = ScreenPixelsHeight*/
const Value V17939_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P17939_4 = {S54, 0, &V17939_4_5645, &P17939_5};	/*Property = ScreenPixelsWidth*/
const Profile P17939 = {"17939", &P17939_4};	/*Profile ID - 17939*/
const Value V24206_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24206_279 = {S1, 0, &V24206_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24206_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24206_282 = {S3, 0, &V24206_282_57154, &P24206_279};	/*Property = BlobBuilder*/
const Value V24206_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24206_350 = {S4, 0, &V24206_350_57444, &P24206_282};	/*Property = Canvas*/
const Value V24206_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24206_283 = {S5, 0, &V24206_283_57159, &P24206_350};	/*Property = CssBackground*/
const Value V24206_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24206_284 = {S6, 0, &V24206_284_57162, &P24206_283};	/*Property = CssBorderImage*/
const Value V24206_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24206_285 = {S7, 0, &V24206_285_57166, &P24206_284};	/*Property = CssCanvas*/
const Value V24206_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24206_286 = {S8, 0, &V24206_286_57168, &P24206_285};	/*Property = CssColor*/
const Value V24206_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24206_237 = {S9, 0, &V24206_237_55786, &P24206_286};	/*Property = CssColumn*/
const Value V24206_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24206_287 = {S10, 0, &V24206_287_57172, &P24206_237};	/*Property = CssFlexbox*/
const Value V24206_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24206_288 = {S11, 0, &V24206_288_57175, &P24206_287};	/*Property = CssFont*/
const Value V24206_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24206_289 = {S12, 0, &V24206_289_57759, &P24206_288};	/*Property = CssImages*/
const Value V24206_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24206_291 = {S13, 0, &V24206_291_57180, &P24206_289};	/*Property = CssMediaQueries*/
const Value V24206_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24206_292 = {S14, 0, &V24206_292_57183, &P24206_291};	/*Property = CssMinMax*/
const Value V24206_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24206_293 = {S15, 0, &V24206_293_57187, &P24206_292};	/*Property = CssOverflow*/
const Value V24206_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24206_294 = {S16, 0, &V24206_294_57189, &P24206_293};	/*Property = CssPosition*/
const Value V24206_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24206_295 = {S17, 0, &V24206_295_57192, &P24206_294};	/*Property = CssText*/
const Value V24206_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24206_297 = {S18, 0, &V24206_297_57195, &P24206_295};	/*Property = CssTransforms*/
const Value V24206_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24206_241 = {S19, 0, &V24206_241_55799, &P24206_297};	/*Property = CssTransitions*/
const Value V24206_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24206_298 = {S20, 0, &V24206_298_57206, &P24206_241};	/*Property = CssUI*/
const Value V24206_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24206_300 = {S21, 0, &V24206_300_57215, &P24206_298};	/*Property = DataSet*/
const Value V24206_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24206_301 = {S22, 0, &V24206_301_57218, &P24206_300};	/*Property = DataUrl*/
const Value V24206_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24206_302 = {S23, 0, &V24206_302_57221, &P24206_301};	/*Property = DeviceOrientation*/
const Value V24206_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24206_304 = {S24, 0, &V24206_304_57227, &P24206_302};	/*Property = FileReader*/
const Value V24206_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24206_305 = {S25, 0, &V24206_305_57231, &P24206_304};	/*Property = FileSaver*/
const Value V24206_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24206_306 = {S26, 0, &V24206_306_57237, &P24206_305};	/*Property = FileWriter*/
const Value V24206_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24206_307 = {S27, 0, &V24206_307_57239, &P24206_306};	/*Property = FormData*/
const Value V24206_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24206_309 = {S28, 0, &V24206_309_57246, &P24206_307};	/*Property = Fullscreen*/
const Value V24206_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24206_310 = {S29, 0, &V24206_310_57248, &P24206_309};	/*Property = GeoLocation*/
const Value V24206_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24206_312 = {S30, 0, &V24206_312_57254, &P24206_310};	/*Property = History*/
const Value V24206_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24206_314 = {S31, 0, &V24206_314_57260, &P24206_312};	/*Property = Html5*/
const Value V24206_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24206_313 = {S32, 0, &V24206_313_57257, &P24206_314};	/*Property = Html-Media-Capture*/
const Value V24206_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24206_315 = {S33, 0, &V24206_315_57264, &P24206_313};	/*Property = Iframe*/
const Value V24206_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24206_316 = {S34, 0, &V24206_316_57266, &P24206_315};	/*Property = IndexedDB*/
const Value V24206_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24206_317 = {S35, 0, &V24206_317_57269, &P24206_316};	/*Property = Json*/
const Value V24206_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24206_15 = {S37, 0, &V24206_15_54167, &P24206_317};	/*Property = LayoutEngine*/
const Value V24206_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24206_344 = {S38, 0, &V24206_344_57272, &P24206_15};	/*Property = Masking*/
const Value V24206_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24206_320 = {S39, 0, &V24206_320_57290, &P24206_344};	/*Property = PostMessage*/
const Value V24206_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24206_321 = {S40, 0, &V24206_321_57293, &P24206_320};	/*Property = Progress*/
const Value V24206_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24206_322 = {S41, 0, &V24206_322_57296, &P24206_321};	/*Property = Prompts*/
const Value V24206_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24206_323 = {S42, 0, &V24206_323_57299, &P24206_322};	/*Property = Selector*/
const Value V24206_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24206_324 = {S43, 0, &V24206_324_57306, &P24206_323};	/*Property = Svg*/
const Value V24206_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24206_327 = {S44, 0, &V24206_327_57314, &P24206_324};	/*Property = TouchEvents*/
const Value V24206_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24206_328 = {S45, 0, &V24206_328_57318, &P24206_327};	/*Property = Track*/
const Value V24206_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24206_329 = {S46, 0, &V24206_329_57323, &P24206_328};	/*Property = Video*/
const Value V24206_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24206_330 = {S47, 0, &V24206_330_57326, &P24206_329};	/*Property = Viewport*/
const Value V24206_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24206_332 = {S48, 0, &V24206_332_57339, &P24206_330};	/*Property = WebWorkers*/
const Value V24206_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24206_333 = {S49, 0, &V24206_333_57342, &P24206_332};	/*Property = Xhr2*/
const Profile P24206 = {"24206", &P24206_333};	/*Profile ID - 24206*/
const Value V22542_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22542_279 = {S1, 0, &V22542_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22542_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22542_282 = {S3, 0, &V22542_282_57154, &P22542_279};	/*Property = BlobBuilder*/
const Value V22542_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22542_350 = {S4, 0, &V22542_350_57444, &P22542_282};	/*Property = Canvas*/
const Value V22542_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22542_283 = {S5, 0, &V22542_283_57159, &P22542_350};	/*Property = CssBackground*/
const Value V22542_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22542_284 = {S6, 0, &V22542_284_57163, &P22542_283};	/*Property = CssBorderImage*/
const Value V22542_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22542_285 = {S7, 0, &V22542_285_57166, &P22542_284};	/*Property = CssCanvas*/
const Value V22542_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22542_286 = {S8, 0, &V22542_286_57168, &P22542_285};	/*Property = CssColor*/
const Value V22542_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22542_237 = {S9, 0, &V22542_237_55786, &P22542_286};	/*Property = CssColumn*/
const Value V22542_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22542_287 = {S10, 0, &V22542_287_57172, &P22542_237};	/*Property = CssFlexbox*/
const Value V22542_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22542_288 = {S11, 0, &V22542_288_57175, &P22542_287};	/*Property = CssFont*/
const Value V22542_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22542_289 = {S12, 0, &V22542_289_57759, &P22542_288};	/*Property = CssImages*/
const Value V22542_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22542_291 = {S13, 0, &V22542_291_57181, &P22542_289};	/*Property = CssMediaQueries*/
const Value V22542_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22542_292 = {S14, 0, &V22542_292_57183, &P22542_291};	/*Property = CssMinMax*/
const Value V22542_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22542_293 = {S15, 0, &V22542_293_57187, &P22542_292};	/*Property = CssOverflow*/
const Value V22542_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22542_294 = {S16, 0, &V22542_294_57190, &P22542_293};	/*Property = CssPosition*/
const Value V22542_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22542_295 = {S17, 0, &V22542_295_57193, &P22542_294};	/*Property = CssText*/
const Value V22542_297_57196 = {S0, NULL};	/*Value = False*/
const Property P22542_297 = {S18, 0, &V22542_297_57196, &P22542_295};	/*Property = CssTransforms*/
const Value V22542_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22542_241 = {S19, 0, &V22542_241_55799, &P22542_297};	/*Property = CssTransitions*/
const Value V22542_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22542_298 = {S20, 0, &V22542_298_57206, &P22542_241};	/*Property = CssUI*/
const Value V22542_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22542_300 = {S21, 0, &V22542_300_57216, &P22542_298};	/*Property = DataSet*/
const Value V22542_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22542_301 = {S22, 0, &V22542_301_57218, &P22542_300};	/*Property = DataUrl*/
const Value V22542_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22542_302 = {S23, 0, &V22542_302_57222, &P22542_301};	/*Property = DeviceOrientation*/
const Value V22542_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22542_304 = {S24, 0, &V22542_304_57228, &P22542_302};	/*Property = FileReader*/
const Value V22542_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22542_305 = {S25, 0, &V22542_305_57231, &P22542_304};	/*Property = FileSaver*/
const Value V22542_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22542_306 = {S26, 0, &V22542_306_57237, &P22542_305};	/*Property = FileWriter*/
const Value V22542_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22542_307 = {S27, 0, &V22542_307_57240, &P22542_306};	/*Property = FormData*/
const Value V22542_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22542_309 = {S28, 0, &V22542_309_57246, &P22542_307};	/*Property = Fullscreen*/
const Value V22542_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22542_310 = {S29, 0, &V22542_310_57248, &P22542_309};	/*Property = GeoLocation*/
const Value V22542_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22542_312 = {S30, 0, &V22542_312_57255, &P22542_310};	/*Property = History*/
const Value V22542_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22542_314 = {S31, 0, &V22542_314_57261, &P22542_312};	/*Property = Html5*/
const Value V22542_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22542_313 = {S32, 0, &V22542_313_57258, &P22542_314};	/*Property = Html-Media-Capture*/
const Value V22542_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22542_315 = {S33, 0, &V22542_315_57264, &P22542_313};	/*Property = Iframe*/
const Value V22542_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22542_316 = {S34, 0, &V22542_316_57267, &P22542_315};	/*Property = IndexedDB*/
const Value V22542_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22542_317 = {S35, 0, &V22542_317_57269, &P22542_316};	/*Property = Json*/
const Value V22542_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P22542_15 = {S37, 0, &V22542_15_31, &P22542_317};	/*Property = LayoutEngine*/
const Value V22542_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22542_344 = {S38, 0, &V22542_344_57272, &P22542_15};	/*Property = Masking*/
const Value V22542_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22542_320 = {S39, 0, &V22542_320_57290, &P22542_344};	/*Property = PostMessage*/
const Value V22542_321_57294 = {S0, NULL};	/*Value = False*/
const Property P22542_321 = {S40, 0, &V22542_321_57294, &P22542_320};	/*Property = Progress*/
const Value V22542_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22542_322 = {S41, 0, &V22542_322_57296, &P22542_321};	/*Property = Prompts*/
const Value V22542_323_57300 = {S0, NULL};	/*Value = False*/
const Property P22542_323 = {S42, 0, &V22542_323_57300, &P22542_322};	/*Property = Selector*/
const Value V22542_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22542_324 = {S43, 0, &V22542_324_57306, &P22542_323};	/*Property = Svg*/
const Value V22542_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22542_327 = {S44, 0, &V22542_327_57315, &P22542_324};	/*Property = TouchEvents*/
const Value V22542_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22542_328 = {S45, 0, &V22542_328_57318, &P22542_327};	/*Property = Track*/
const Value V22542_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22542_329 = {S46, 0, &V22542_329_57323, &P22542_328};	/*Property = Video*/
const Value V22542_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22542_330 = {S47, 0, &V22542_330_57326, &P22542_329};	/*Property = Viewport*/
const Value V22542_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22542_332 = {S48, 0, &V22542_332_57339, &P22542_330};	/*Property = WebWorkers*/
const Value V22542_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22542_333 = {S49, 0, &V22542_333_57342, &P22542_332};	/*Property = Xhr2*/
const Profile P22542 = {"22542", &P22542_333};	/*Profile ID - 22542*/
const Profile P18066 = {"18066", NULL};	/*Profile ID - 18066*/
const Value V18062_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18062_279 = {S1, 0, &V18062_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18062_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18062_282 = {S3, 0, &V18062_282_57154, &P18062_279};	/*Property = BlobBuilder*/
const Value V18062_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18062_350 = {S4, 0, &V18062_350_57444, &P18062_282};	/*Property = Canvas*/
const Value V18062_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18062_283 = {S5, 0, &V18062_283_57159, &P18062_350};	/*Property = CssBackground*/
const Value V18062_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18062_284 = {S6, 0, &V18062_284_57163, &P18062_283};	/*Property = CssBorderImage*/
const Value V18062_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18062_285 = {S7, 0, &V18062_285_57166, &P18062_284};	/*Property = CssCanvas*/
const Value V18062_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18062_286 = {S8, 0, &V18062_286_57168, &P18062_285};	/*Property = CssColor*/
const Value V18062_237_55784 = {S2, NULL};	/*Value = True*/
const Property P18062_237 = {S9, 0, &V18062_237_55784, &P18062_286};	/*Property = CssColumn*/
const Value V18062_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18062_287 = {S10, 0, &V18062_287_57172, &P18062_237};	/*Property = CssFlexbox*/
const Value V18062_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18062_288 = {S11, 0, &V18062_288_57175, &P18062_287};	/*Property = CssFont*/
const Value V18062_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18062_289 = {S12, 0, &V18062_289_57759, &P18062_288};	/*Property = CssImages*/
const Value V18062_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18062_291 = {S13, 0, &V18062_291_57180, &P18062_289};	/*Property = CssMediaQueries*/
const Value V18062_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18062_292 = {S14, 0, &V18062_292_57183, &P18062_291};	/*Property = CssMinMax*/
const Value V18062_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18062_293 = {S15, 0, &V18062_293_57187, &P18062_292};	/*Property = CssOverflow*/
const Value V18062_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18062_294 = {S16, 0, &V18062_294_57190, &P18062_293};	/*Property = CssPosition*/
const Value V18062_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18062_295 = {S17, 0, &V18062_295_57192, &P18062_294};	/*Property = CssText*/
const Value V18062_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18062_297 = {S18, 0, &V18062_297_57195, &P18062_295};	/*Property = CssTransforms*/
const Value V18062_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18062_241 = {S19, 0, &V18062_241_55799, &P18062_297};	/*Property = CssTransitions*/
const Value V18062_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18062_298 = {S20, 0, &V18062_298_57206, &P18062_241};	/*Property = CssUI*/
const Value V18062_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18062_300 = {S21, 0, &V18062_300_57216, &P18062_298};	/*Property = DataSet*/
const Value V18062_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18062_301 = {S22, 0, &V18062_301_57218, &P18062_300};	/*Property = DataUrl*/
const Value V18062_302_57221 = {S2, NULL};	/*Value = True*/
const Property P18062_302 = {S23, 0, &V18062_302_57221, &P18062_301};	/*Property = DeviceOrientation*/
const Value V18062_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18062_304 = {S24, 0, &V18062_304_57227, &P18062_302};	/*Property = FileReader*/
const Value V18062_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18062_305 = {S25, 0, &V18062_305_57231, &P18062_304};	/*Property = FileSaver*/
const Value V18062_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18062_306 = {S26, 0, &V18062_306_57237, &P18062_305};	/*Property = FileWriter*/
const Value V18062_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18062_307 = {S27, 0, &V18062_307_57239, &P18062_306};	/*Property = FormData*/
const Value V18062_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18062_309 = {S28, 0, &V18062_309_57246, &P18062_307};	/*Property = Fullscreen*/
const Value V18062_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18062_310 = {S29, 0, &V18062_310_57248, &P18062_309};	/*Property = GeoLocation*/
const Value V18062_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18062_312 = {S30, 0, &V18062_312_57255, &P18062_310};	/*Property = History*/
const Value V18062_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18062_314 = {S31, 0, &V18062_314_57261, &P18062_312};	/*Property = Html5*/
const Value V18062_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18062_313 = {S32, 0, &V18062_313_57258, &P18062_314};	/*Property = Html-Media-Capture*/
const Value V18062_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18062_315 = {S33, 0, &V18062_315_57264, &P18062_313};	/*Property = Iframe*/
const Value V18062_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18062_316 = {S34, 0, &V18062_316_57267, &P18062_315};	/*Property = IndexedDB*/
const Value V18062_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18062_317 = {S35, 0, &V18062_317_57269, &P18062_316};	/*Property = Json*/
const Value V18062_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18062_15 = {S37, 0, &V18062_15_28, &P18062_317};	/*Property = LayoutEngine*/
const Value V18062_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18062_344 = {S38, 0, &V18062_344_57272, &P18062_15};	/*Property = Masking*/
const Value V18062_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18062_320 = {S39, 0, &V18062_320_57290, &P18062_344};	/*Property = PostMessage*/
const Value V18062_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18062_321 = {S40, 0, &V18062_321_57293, &P18062_320};	/*Property = Progress*/
const Value V18062_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18062_322 = {S41, 0, &V18062_322_57296, &P18062_321};	/*Property = Prompts*/
const Value V18062_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18062_323 = {S42, 0, &V18062_323_57299, &P18062_322};	/*Property = Selector*/
const Value V18062_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18062_324 = {S43, 0, &V18062_324_57306, &P18062_323};	/*Property = Svg*/
const Value V18062_327_57314 = {S2, NULL};	/*Value = True*/
const Property P18062_327 = {S44, 0, &V18062_327_57314, &P18062_324};	/*Property = TouchEvents*/
const Value V18062_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18062_328 = {S45, 0, &V18062_328_57318, &P18062_327};	/*Property = Track*/
const Value V18062_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18062_329 = {S46, 0, &V18062_329_57323, &P18062_328};	/*Property = Video*/
const Value V18062_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18062_330 = {S47, 0, &V18062_330_57326, &P18062_329};	/*Property = Viewport*/
const Value V18062_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18062_332 = {S48, 0, &V18062_332_57339, &P18062_330};	/*Property = WebWorkers*/
const Value V18062_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18062_333 = {S49, 0, &V18062_333_57342, &P18062_332};	/*Property = Xhr2*/
const Profile P18062 = {"18062", &P18062_333};	/*Profile ID - 18062*/
const Value V18051_1_1 = {S2, NULL};	/*Value = True*/
const Property P18051_1 = {S50, 0, &V18051_1_1, NULL};	/*Property = IsMobile*/
const Value V18051_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18051_5 = {S52, 0, &V18051_5_9981, &P18051_1};	/*Property = ScreenPixelsHeight*/
const Value V18051_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18051_4 = {S54, 0, &V18051_4_2336, &P18051_5};	/*Property = ScreenPixelsWidth*/
const Profile P18051 = {"18051", &P18051_4};	/*Profile ID - 18051*/
const Profile P18698 = {"18698", NULL};	/*Profile ID - 18698*/
const Value V21317_1_1 = {S2, NULL};	/*Value = True*/
const Property P21317_1 = {S50, 0, &V21317_1_1, NULL};	/*Property = IsMobile*/
const Value V21317_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21317_5 = {S52, 0, &V21317_5_1277, &P21317_1};	/*Property = ScreenPixelsHeight*/
const Value V21317_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21317_4 = {S54, 0, &V21317_4_1279, &P21317_5};	/*Property = ScreenPixelsWidth*/
const Profile P21317 = {"21317", &P21317_4};	/*Profile ID - 21317*/
const Value V15885_1_1 = {S2, NULL};	/*Value = True*/
const Property P15885_1 = {S50, 0, &V15885_1_1, NULL};	/*Property = IsMobile*/
const Value V15885_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P15885_5 = {S52, 0, &V15885_5_1275, &P15885_1};	/*Property = ScreenPixelsHeight*/
const Value V15885_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15885_4 = {S54, 0, &V15885_4_1229, &P15885_5};	/*Property = ScreenPixelsWidth*/
const Profile P15885 = {"15885", &P15885_4};	/*Profile ID - 15885*/
const Value V22413_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22413_279 = {S1, 0, &V22413_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22413_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22413_282 = {S3, 0, &V22413_282_57154, &P22413_279};	/*Property = BlobBuilder*/
const Value V22413_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22413_350 = {S4, 0, &V22413_350_57444, &P22413_282};	/*Property = Canvas*/
const Value V22413_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22413_283 = {S5, 0, &V22413_283_57159, &P22413_350};	/*Property = CssBackground*/
const Value V22413_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22413_284 = {S6, 0, &V22413_284_57163, &P22413_283};	/*Property = CssBorderImage*/
const Value V22413_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22413_285 = {S7, 0, &V22413_285_57166, &P22413_284};	/*Property = CssCanvas*/
const Value V22413_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22413_286 = {S8, 0, &V22413_286_57168, &P22413_285};	/*Property = CssColor*/
const Value V22413_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22413_237 = {S9, 0, &V22413_237_55784, &P22413_286};	/*Property = CssColumn*/
const Value V22413_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22413_287 = {S10, 0, &V22413_287_57172, &P22413_237};	/*Property = CssFlexbox*/
const Value V22413_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22413_288 = {S11, 0, &V22413_288_57175, &P22413_287};	/*Property = CssFont*/
const Value V22413_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22413_289 = {S12, 0, &V22413_289_57759, &P22413_288};	/*Property = CssImages*/
const Value V22413_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22413_291 = {S13, 0, &V22413_291_57180, &P22413_289};	/*Property = CssMediaQueries*/
const Value V22413_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22413_292 = {S14, 0, &V22413_292_57183, &P22413_291};	/*Property = CssMinMax*/
const Value V22413_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22413_293 = {S15, 0, &V22413_293_57187, &P22413_292};	/*Property = CssOverflow*/
const Value V22413_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22413_294 = {S16, 0, &V22413_294_57189, &P22413_293};	/*Property = CssPosition*/
const Value V22413_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22413_295 = {S17, 0, &V22413_295_57192, &P22413_294};	/*Property = CssText*/
const Value V22413_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22413_297 = {S18, 0, &V22413_297_57195, &P22413_295};	/*Property = CssTransforms*/
const Value V22413_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22413_241 = {S19, 0, &V22413_241_55799, &P22413_297};	/*Property = CssTransitions*/
const Value V22413_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22413_298 = {S20, 0, &V22413_298_57206, &P22413_241};	/*Property = CssUI*/
const Value V22413_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22413_300 = {S21, 0, &V22413_300_57216, &P22413_298};	/*Property = DataSet*/
const Value V22413_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22413_301 = {S22, 0, &V22413_301_57218, &P22413_300};	/*Property = DataUrl*/
const Value V22413_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22413_302 = {S23, 0, &V22413_302_57222, &P22413_301};	/*Property = DeviceOrientation*/
const Value V22413_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22413_304 = {S24, 0, &V22413_304_57227, &P22413_302};	/*Property = FileReader*/
const Value V22413_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22413_305 = {S25, 0, &V22413_305_57231, &P22413_304};	/*Property = FileSaver*/
const Value V22413_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22413_306 = {S26, 0, &V22413_306_57237, &P22413_305};	/*Property = FileWriter*/
const Value V22413_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22413_307 = {S27, 0, &V22413_307_57239, &P22413_306};	/*Property = FormData*/
const Value V22413_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22413_309 = {S28, 0, &V22413_309_57246, &P22413_307};	/*Property = Fullscreen*/
const Value V22413_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22413_310 = {S29, 0, &V22413_310_57248, &P22413_309};	/*Property = GeoLocation*/
const Value V22413_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22413_312 = {S30, 0, &V22413_312_57254, &P22413_310};	/*Property = History*/
const Value V22413_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22413_314 = {S31, 0, &V22413_314_57261, &P22413_312};	/*Property = Html5*/
const Value V22413_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22413_313 = {S32, 0, &V22413_313_57258, &P22413_314};	/*Property = Html-Media-Capture*/
const Value V22413_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22413_315 = {S33, 0, &V22413_315_57264, &P22413_313};	/*Property = Iframe*/
const Value V22413_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22413_316 = {S34, 0, &V22413_316_57266, &P22413_315};	/*Property = IndexedDB*/
const Value V22413_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22413_317 = {S35, 0, &V22413_317_57269, &P22413_316};	/*Property = Json*/
const Value V22413_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22413_15 = {S37, 0, &V22413_15_28, &P22413_317};	/*Property = LayoutEngine*/
const Value V22413_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22413_344 = {S38, 0, &V22413_344_57272, &P22413_15};	/*Property = Masking*/
const Value V22413_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22413_320 = {S39, 0, &V22413_320_57290, &P22413_344};	/*Property = PostMessage*/
const Value V22413_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22413_321 = {S40, 0, &V22413_321_57293, &P22413_320};	/*Property = Progress*/
const Value V22413_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22413_322 = {S41, 0, &V22413_322_57296, &P22413_321};	/*Property = Prompts*/
const Value V22413_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22413_323 = {S42, 0, &V22413_323_57299, &P22413_322};	/*Property = Selector*/
const Value V22413_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22413_324 = {S43, 0, &V22413_324_57306, &P22413_323};	/*Property = Svg*/
const Value V22413_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22413_327 = {S44, 0, &V22413_327_57315, &P22413_324};	/*Property = TouchEvents*/
const Value V22413_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22413_328 = {S45, 0, &V22413_328_57318, &P22413_327};	/*Property = Track*/
const Value V22413_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22413_329 = {S46, 0, &V22413_329_57323, &P22413_328};	/*Property = Video*/
const Value V22413_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22413_330 = {S47, 0, &V22413_330_57326, &P22413_329};	/*Property = Viewport*/
const Value V22413_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22413_332 = {S48, 0, &V22413_332_57338, &P22413_330};	/*Property = WebWorkers*/
const Value V22413_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22413_333 = {S49, 0, &V22413_333_57342, &P22413_332};	/*Property = Xhr2*/
const Profile P22413 = {"22413", &P22413_333};	/*Profile ID - 22413*/
const Value V17780_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_279 = {S1, 0, &V17780_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17780_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_282 = {S3, 0, &V17780_282_57155, &P17780_279};	/*Property = BlobBuilder*/
const Value V17780_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_350 = {S4, 0, &V17780_350_57446, &P17780_282};	/*Property = Canvas*/
const Value V17780_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_283 = {S5, 0, &V17780_283_57161, &P17780_350};	/*Property = CssBackground*/
const Value V17780_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_284 = {S6, 0, &V17780_284_57164, &P17780_283};	/*Property = CssBorderImage*/
const Value V17780_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_285 = {S7, 0, &V17780_285_57167, &P17780_284};	/*Property = CssCanvas*/
const Value V17780_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_286 = {S8, 0, &V17780_286_57170, &P17780_285};	/*Property = CssColor*/
const Value V17780_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_237 = {S9, 0, &V17780_237_55786, &P17780_286};	/*Property = CssColumn*/
const Value V17780_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_287 = {S10, 0, &V17780_287_57173, &P17780_237};	/*Property = CssFlexbox*/
const Value V17780_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_288 = {S11, 0, &V17780_288_57176, &P17780_287};	/*Property = CssFont*/
const Value V17780_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_289 = {S12, 0, &V17780_289_86899, &P17780_288};	/*Property = CssImages*/
const Value V17780_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_291 = {S13, 0, &V17780_291_57182, &P17780_289};	/*Property = CssMediaQueries*/
const Value V17780_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_292 = {S14, 0, &V17780_292_57185, &P17780_291};	/*Property = CssMinMax*/
const Value V17780_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_293 = {S15, 0, &V17780_293_57188, &P17780_292};	/*Property = CssOverflow*/
const Value V17780_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_294 = {S16, 0, &V17780_294_57191, &P17780_293};	/*Property = CssPosition*/
const Value V17780_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_295 = {S17, 0, &V17780_295_57194, &P17780_294};	/*Property = CssText*/
const Value V17780_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_297 = {S18, 0, &V17780_297_57197, &P17780_295};	/*Property = CssTransforms*/
const Value V17780_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_241 = {S19, 0, &V17780_241_55801, &P17780_297};	/*Property = CssTransitions*/
const Value V17780_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_298 = {S20, 0, &V17780_298_57208, &P17780_241};	/*Property = CssUI*/
const Value V17780_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_300 = {S21, 0, &V17780_300_57217, &P17780_298};	/*Property = DataSet*/
const Value V17780_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_301 = {S22, 0, &V17780_301_57220, &P17780_300};	/*Property = DataUrl*/
const Value V17780_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_302 = {S23, 0, &V17780_302_57223, &P17780_301};	/*Property = DeviceOrientation*/
const Value V17780_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_304 = {S24, 0, &V17780_304_57229, &P17780_302};	/*Property = FileReader*/
const Value V17780_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_305 = {S25, 0, &V17780_305_57232, &P17780_304};	/*Property = FileSaver*/
const Value V17780_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_306 = {S26, 0, &V17780_306_57238, &P17780_305};	/*Property = FileWriter*/
const Value V17780_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_307 = {S27, 0, &V17780_307_57241, &P17780_306};	/*Property = FormData*/
const Value V17780_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_309 = {S28, 0, &V17780_309_57247, &P17780_307};	/*Property = Fullscreen*/
const Value V17780_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_310 = {S29, 0, &V17780_310_57250, &P17780_309};	/*Property = GeoLocation*/
const Value V17780_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_312 = {S30, 0, &V17780_312_57256, &P17780_310};	/*Property = History*/
const Value V17780_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_314 = {S31, 0, &V17780_314_57262, &P17780_312};	/*Property = Html5*/
const Value V17780_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_313 = {S32, 0, &V17780_313_57259, &P17780_314};	/*Property = Html-Media-Capture*/
const Value V17780_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_315 = {S33, 0, &V17780_315_57265, &P17780_313};	/*Property = Iframe*/
const Value V17780_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_316 = {S34, 0, &V17780_316_57268, &P17780_315};	/*Property = IndexedDB*/
const Value V17780_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_317 = {S35, 0, &V17780_317_57271, &P17780_316};	/*Property = Json*/
const Value V17780_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P17780_15 = {S37, 0, &V17780_15_31, &P17780_317};	/*Property = LayoutEngine*/
const Value V17780_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_344 = {S38, 0, &V17780_344_57274, &P17780_15};	/*Property = Masking*/
const Value V17780_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_320 = {S39, 0, &V17780_320_57292, &P17780_344};	/*Property = PostMessage*/
const Value V17780_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_321 = {S40, 0, &V17780_321_57295, &P17780_320};	/*Property = Progress*/
const Value V17780_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_322 = {S41, 0, &V17780_322_57298, &P17780_321};	/*Property = Prompts*/
const Value V17780_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_323 = {S42, 0, &V17780_323_57301, &P17780_322};	/*Property = Selector*/
const Value V17780_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_324 = {S43, 0, &V17780_324_57307, &P17780_323};	/*Property = Svg*/
const Value V17780_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_327 = {S44, 0, &V17780_327_57316, &P17780_324};	/*Property = TouchEvents*/
const Value V17780_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_328 = {S45, 0, &V17780_328_57319, &P17780_327};	/*Property = Track*/
const Value V17780_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_329 = {S46, 0, &V17780_329_57325, &P17780_328};	/*Property = Video*/
const Value V17780_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_330 = {S47, 0, &V17780_330_57328, &P17780_329};	/*Property = Viewport*/
const Value V17780_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_332 = {S48, 0, &V17780_332_57340, &P17780_330};	/*Property = WebWorkers*/
const Value V17780_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17780_333 = {S49, 0, &V17780_333_57343, &P17780_332};	/*Property = Xhr2*/
const Profile P17780 = {"17780", &P17780_333};	/*Profile ID - 17780*/
const Profile P18109 = {"18109", NULL};	/*Profile ID - 18109*/
const Value V18104_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18104_279 = {S1, 0, &V18104_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18104_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18104_282 = {S3, 0, &V18104_282_57154, &P18104_279};	/*Property = BlobBuilder*/
const Value V18104_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18104_350 = {S4, 0, &V18104_350_57444, &P18104_282};	/*Property = Canvas*/
const Value V18104_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18104_283 = {S5, 0, &V18104_283_57159, &P18104_350};	/*Property = CssBackground*/
const Value V18104_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18104_284 = {S6, 0, &V18104_284_57163, &P18104_283};	/*Property = CssBorderImage*/
const Value V18104_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18104_285 = {S7, 0, &V18104_285_57166, &P18104_284};	/*Property = CssCanvas*/
const Value V18104_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18104_286 = {S8, 0, &V18104_286_57168, &P18104_285};	/*Property = CssColor*/
const Value V18104_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18104_237 = {S9, 0, &V18104_237_55786, &P18104_286};	/*Property = CssColumn*/
const Value V18104_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18104_287 = {S10, 0, &V18104_287_57172, &P18104_237};	/*Property = CssFlexbox*/
const Value V18104_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18104_288 = {S11, 0, &V18104_288_57175, &P18104_287};	/*Property = CssFont*/
const Value V18104_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18104_289 = {S12, 0, &V18104_289_57759, &P18104_288};	/*Property = CssImages*/
const Value V18104_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18104_291 = {S13, 0, &V18104_291_57181, &P18104_289};	/*Property = CssMediaQueries*/
const Value V18104_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18104_292 = {S14, 0, &V18104_292_57183, &P18104_291};	/*Property = CssMinMax*/
const Value V18104_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18104_293 = {S15, 0, &V18104_293_57187, &P18104_292};	/*Property = CssOverflow*/
const Value V18104_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18104_294 = {S16, 0, &V18104_294_57190, &P18104_293};	/*Property = CssPosition*/
const Value V18104_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18104_295 = {S17, 0, &V18104_295_57193, &P18104_294};	/*Property = CssText*/
const Value V18104_297_57196 = {S0, NULL};	/*Value = False*/
const Property P18104_297 = {S18, 0, &V18104_297_57196, &P18104_295};	/*Property = CssTransforms*/
const Value V18104_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18104_241 = {S19, 0, &V18104_241_55799, &P18104_297};	/*Property = CssTransitions*/
const Value V18104_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18104_298 = {S20, 0, &V18104_298_57206, &P18104_241};	/*Property = CssUI*/
const Value V18104_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18104_300 = {S21, 0, &V18104_300_57216, &P18104_298};	/*Property = DataSet*/
const Value V18104_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18104_301 = {S22, 0, &V18104_301_57218, &P18104_300};	/*Property = DataUrl*/
const Value V18104_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18104_302 = {S23, 0, &V18104_302_57222, &P18104_301};	/*Property = DeviceOrientation*/
const Value V18104_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18104_304 = {S24, 0, &V18104_304_57228, &P18104_302};	/*Property = FileReader*/
const Value V18104_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18104_305 = {S25, 0, &V18104_305_57231, &P18104_304};	/*Property = FileSaver*/
const Value V18104_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18104_306 = {S26, 0, &V18104_306_57237, &P18104_305};	/*Property = FileWriter*/
const Value V18104_307_57240 = {S0, NULL};	/*Value = False*/
const Property P18104_307 = {S27, 0, &V18104_307_57240, &P18104_306};	/*Property = FormData*/
const Value V18104_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18104_309 = {S28, 0, &V18104_309_57246, &P18104_307};	/*Property = Fullscreen*/
const Value V18104_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18104_310 = {S29, 0, &V18104_310_57248, &P18104_309};	/*Property = GeoLocation*/
const Value V18104_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18104_312 = {S30, 0, &V18104_312_57255, &P18104_310};	/*Property = History*/
const Value V18104_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18104_314 = {S31, 0, &V18104_314_57261, &P18104_312};	/*Property = Html5*/
const Value V18104_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18104_313 = {S32, 0, &V18104_313_57258, &P18104_314};	/*Property = Html-Media-Capture*/
const Value V18104_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18104_315 = {S33, 0, &V18104_315_57264, &P18104_313};	/*Property = Iframe*/
const Value V18104_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18104_316 = {S34, 0, &V18104_316_57267, &P18104_315};	/*Property = IndexedDB*/
const Value V18104_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18104_317 = {S35, 0, &V18104_317_57269, &P18104_316};	/*Property = Json*/
const Value V18104_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18104_15 = {S37, 0, &V18104_15_31, &P18104_317};	/*Property = LayoutEngine*/
const Value V18104_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18104_344 = {S38, 0, &V18104_344_57272, &P18104_15};	/*Property = Masking*/
const Value V18104_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18104_320 = {S39, 0, &V18104_320_57290, &P18104_344};	/*Property = PostMessage*/
const Value V18104_321_57294 = {S0, NULL};	/*Value = False*/
const Property P18104_321 = {S40, 0, &V18104_321_57294, &P18104_320};	/*Property = Progress*/
const Value V18104_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18104_322 = {S41, 0, &V18104_322_57296, &P18104_321};	/*Property = Prompts*/
const Value V18104_323_57300 = {S0, NULL};	/*Value = False*/
const Property P18104_323 = {S42, 0, &V18104_323_57300, &P18104_322};	/*Property = Selector*/
const Value V18104_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18104_324 = {S43, 0, &V18104_324_57306, &P18104_323};	/*Property = Svg*/
const Value V18104_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18104_327 = {S44, 0, &V18104_327_57315, &P18104_324};	/*Property = TouchEvents*/
const Value V18104_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18104_328 = {S45, 0, &V18104_328_57318, &P18104_327};	/*Property = Track*/
const Value V18104_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18104_329 = {S46, 0, &V18104_329_57323, &P18104_328};	/*Property = Video*/
const Value V18104_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18104_330 = {S47, 0, &V18104_330_57326, &P18104_329};	/*Property = Viewport*/
const Value V18104_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18104_332 = {S48, 0, &V18104_332_57339, &P18104_330};	/*Property = WebWorkers*/
const Value V18104_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18104_333 = {S49, 0, &V18104_333_57342, &P18104_332};	/*Property = Xhr2*/
const Profile P18104 = {"18104", &P18104_333};	/*Profile ID - 18104*/
const Value V17373_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_279 = {S1, 0, &V17373_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17373_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_282 = {S3, 0, &V17373_282_57155, &P17373_279};	/*Property = BlobBuilder*/
const Value V17373_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_350 = {S4, 0, &V17373_350_57446, &P17373_282};	/*Property = Canvas*/
const Value V17373_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_283 = {S5, 0, &V17373_283_57161, &P17373_350};	/*Property = CssBackground*/
const Value V17373_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_284 = {S6, 0, &V17373_284_57164, &P17373_283};	/*Property = CssBorderImage*/
const Value V17373_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_285 = {S7, 0, &V17373_285_57167, &P17373_284};	/*Property = CssCanvas*/
const Value V17373_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_286 = {S8, 0, &V17373_286_57170, &P17373_285};	/*Property = CssColor*/
const Value V17373_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_237 = {S9, 0, &V17373_237_55786, &P17373_286};	/*Property = CssColumn*/
const Value V17373_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_287 = {S10, 0, &V17373_287_57173, &P17373_237};	/*Property = CssFlexbox*/
const Value V17373_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_288 = {S11, 0, &V17373_288_57176, &P17373_287};	/*Property = CssFont*/
const Value V17373_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_289 = {S12, 0, &V17373_289_86899, &P17373_288};	/*Property = CssImages*/
const Value V17373_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_291 = {S13, 0, &V17373_291_57182, &P17373_289};	/*Property = CssMediaQueries*/
const Value V17373_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_292 = {S14, 0, &V17373_292_57185, &P17373_291};	/*Property = CssMinMax*/
const Value V17373_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_293 = {S15, 0, &V17373_293_57188, &P17373_292};	/*Property = CssOverflow*/
const Value V17373_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_294 = {S16, 0, &V17373_294_57191, &P17373_293};	/*Property = CssPosition*/
const Value V17373_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_295 = {S17, 0, &V17373_295_57194, &P17373_294};	/*Property = CssText*/
const Value V17373_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_297 = {S18, 0, &V17373_297_57197, &P17373_295};	/*Property = CssTransforms*/
const Value V17373_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_241 = {S19, 0, &V17373_241_55801, &P17373_297};	/*Property = CssTransitions*/
const Value V17373_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_298 = {S20, 0, &V17373_298_57208, &P17373_241};	/*Property = CssUI*/
const Value V17373_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_300 = {S21, 0, &V17373_300_57217, &P17373_298};	/*Property = DataSet*/
const Value V17373_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_301 = {S22, 0, &V17373_301_57220, &P17373_300};	/*Property = DataUrl*/
const Value V17373_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_302 = {S23, 0, &V17373_302_57223, &P17373_301};	/*Property = DeviceOrientation*/
const Value V17373_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_304 = {S24, 0, &V17373_304_57229, &P17373_302};	/*Property = FileReader*/
const Value V17373_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_305 = {S25, 0, &V17373_305_57232, &P17373_304};	/*Property = FileSaver*/
const Value V17373_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_306 = {S26, 0, &V17373_306_57238, &P17373_305};	/*Property = FileWriter*/
const Value V17373_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_307 = {S27, 0, &V17373_307_57241, &P17373_306};	/*Property = FormData*/
const Value V17373_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_309 = {S28, 0, &V17373_309_57247, &P17373_307};	/*Property = Fullscreen*/
const Value V17373_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_310 = {S29, 0, &V17373_310_57250, &P17373_309};	/*Property = GeoLocation*/
const Value V17373_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_312 = {S30, 0, &V17373_312_57256, &P17373_310};	/*Property = History*/
const Value V17373_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_314 = {S31, 0, &V17373_314_57262, &P17373_312};	/*Property = Html5*/
const Value V17373_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_313 = {S32, 0, &V17373_313_57259, &P17373_314};	/*Property = Html-Media-Capture*/
const Value V17373_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_315 = {S33, 0, &V17373_315_57265, &P17373_313};	/*Property = Iframe*/
const Value V17373_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_316 = {S34, 0, &V17373_316_57268, &P17373_315};	/*Property = IndexedDB*/
const Value V17373_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_317 = {S35, 0, &V17373_317_57271, &P17373_316};	/*Property = Json*/
const Value V17373_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P17373_15 = {S37, 0, &V17373_15_31, &P17373_317};	/*Property = LayoutEngine*/
const Value V17373_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_344 = {S38, 0, &V17373_344_57274, &P17373_15};	/*Property = Masking*/
const Value V17373_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_320 = {S39, 0, &V17373_320_57292, &P17373_344};	/*Property = PostMessage*/
const Value V17373_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_321 = {S40, 0, &V17373_321_57295, &P17373_320};	/*Property = Progress*/
const Value V17373_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_322 = {S41, 0, &V17373_322_57298, &P17373_321};	/*Property = Prompts*/
const Value V17373_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_323 = {S42, 0, &V17373_323_57301, &P17373_322};	/*Property = Selector*/
const Value V17373_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_324 = {S43, 0, &V17373_324_57307, &P17373_323};	/*Property = Svg*/
const Value V17373_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_327 = {S44, 0, &V17373_327_57316, &P17373_324};	/*Property = TouchEvents*/
const Value V17373_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_328 = {S45, 0, &V17373_328_57319, &P17373_327};	/*Property = Track*/
const Value V17373_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_329 = {S46, 0, &V17373_329_57325, &P17373_328};	/*Property = Video*/
const Value V17373_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_330 = {S47, 0, &V17373_330_57328, &P17373_329};	/*Property = Viewport*/
const Value V17373_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_332 = {S48, 0, &V17373_332_57340, &P17373_330};	/*Property = WebWorkers*/
const Value V17373_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17373_333 = {S49, 0, &V17373_333_57343, &P17373_332};	/*Property = Xhr2*/
const Profile P17373 = {"17373", &P17373_333};	/*Profile ID - 17373*/
const Value V24472_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_279 = {S1, 0, &V24472_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24472_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_282 = {S3, 0, &V24472_282_57155, &P24472_279};	/*Property = BlobBuilder*/
const Value V24472_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_350 = {S4, 0, &V24472_350_57446, &P24472_282};	/*Property = Canvas*/
const Value V24472_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_283 = {S5, 0, &V24472_283_57161, &P24472_350};	/*Property = CssBackground*/
const Value V24472_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_284 = {S6, 0, &V24472_284_57164, &P24472_283};	/*Property = CssBorderImage*/
const Value V24472_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_285 = {S7, 0, &V24472_285_57167, &P24472_284};	/*Property = CssCanvas*/
const Value V24472_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_286 = {S8, 0, &V24472_286_57170, &P24472_285};	/*Property = CssColor*/
const Value V24472_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_237 = {S9, 0, &V24472_237_55786, &P24472_286};	/*Property = CssColumn*/
const Value V24472_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_287 = {S10, 0, &V24472_287_57173, &P24472_237};	/*Property = CssFlexbox*/
const Value V24472_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_288 = {S11, 0, &V24472_288_57176, &P24472_287};	/*Property = CssFont*/
const Value V24472_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_289 = {S12, 0, &V24472_289_86899, &P24472_288};	/*Property = CssImages*/
const Value V24472_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_291 = {S13, 0, &V24472_291_57182, &P24472_289};	/*Property = CssMediaQueries*/
const Value V24472_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_292 = {S14, 0, &V24472_292_57185, &P24472_291};	/*Property = CssMinMax*/
const Value V24472_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_293 = {S15, 0, &V24472_293_57188, &P24472_292};	/*Property = CssOverflow*/
const Value V24472_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_294 = {S16, 0, &V24472_294_57191, &P24472_293};	/*Property = CssPosition*/
const Value V24472_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_295 = {S17, 0, &V24472_295_57194, &P24472_294};	/*Property = CssText*/
const Value V24472_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_297 = {S18, 0, &V24472_297_57197, &P24472_295};	/*Property = CssTransforms*/
const Value V24472_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_241 = {S19, 0, &V24472_241_55801, &P24472_297};	/*Property = CssTransitions*/
const Value V24472_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_298 = {S20, 0, &V24472_298_57208, &P24472_241};	/*Property = CssUI*/
const Value V24472_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_300 = {S21, 0, &V24472_300_57217, &P24472_298};	/*Property = DataSet*/
const Value V24472_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_301 = {S22, 0, &V24472_301_57220, &P24472_300};	/*Property = DataUrl*/
const Value V24472_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_302 = {S23, 0, &V24472_302_57223, &P24472_301};	/*Property = DeviceOrientation*/
const Value V24472_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_304 = {S24, 0, &V24472_304_57229, &P24472_302};	/*Property = FileReader*/
const Value V24472_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_305 = {S25, 0, &V24472_305_57232, &P24472_304};	/*Property = FileSaver*/
const Value V24472_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_306 = {S26, 0, &V24472_306_57238, &P24472_305};	/*Property = FileWriter*/
const Value V24472_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_307 = {S27, 0, &V24472_307_57241, &P24472_306};	/*Property = FormData*/
const Value V24472_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_309 = {S28, 0, &V24472_309_57247, &P24472_307};	/*Property = Fullscreen*/
const Value V24472_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_310 = {S29, 0, &V24472_310_57250, &P24472_309};	/*Property = GeoLocation*/
const Value V24472_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_312 = {S30, 0, &V24472_312_57256, &P24472_310};	/*Property = History*/
const Value V24472_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_314 = {S31, 0, &V24472_314_57262, &P24472_312};	/*Property = Html5*/
const Value V24472_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_313 = {S32, 0, &V24472_313_57259, &P24472_314};	/*Property = Html-Media-Capture*/
const Value V24472_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_315 = {S33, 0, &V24472_315_57265, &P24472_313};	/*Property = Iframe*/
const Value V24472_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_316 = {S34, 0, &V24472_316_57268, &P24472_315};	/*Property = IndexedDB*/
const Value V24472_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_317 = {S35, 0, &V24472_317_57271, &P24472_316};	/*Property = Json*/
const Value V24472_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24472_15 = {S37, 0, &V24472_15_54167, &P24472_317};	/*Property = LayoutEngine*/
const Value V24472_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_344 = {S38, 0, &V24472_344_57274, &P24472_15};	/*Property = Masking*/
const Value V24472_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_320 = {S39, 0, &V24472_320_57292, &P24472_344};	/*Property = PostMessage*/
const Value V24472_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_321 = {S40, 0, &V24472_321_57295, &P24472_320};	/*Property = Progress*/
const Value V24472_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_322 = {S41, 0, &V24472_322_57298, &P24472_321};	/*Property = Prompts*/
const Value V24472_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_323 = {S42, 0, &V24472_323_57301, &P24472_322};	/*Property = Selector*/
const Value V24472_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_324 = {S43, 0, &V24472_324_57307, &P24472_323};	/*Property = Svg*/
const Value V24472_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_327 = {S44, 0, &V24472_327_57316, &P24472_324};	/*Property = TouchEvents*/
const Value V24472_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_328 = {S45, 0, &V24472_328_57319, &P24472_327};	/*Property = Track*/
const Value V24472_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_329 = {S46, 0, &V24472_329_57325, &P24472_328};	/*Property = Video*/
const Value V24472_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_330 = {S47, 0, &V24472_330_57328, &P24472_329};	/*Property = Viewport*/
const Value V24472_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_332 = {S48, 0, &V24472_332_57340, &P24472_330};	/*Property = WebWorkers*/
const Value V24472_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24472_333 = {S49, 0, &V24472_333_57343, &P24472_332};	/*Property = Xhr2*/
const Profile P24472 = {"24472", &P24472_333};	/*Profile ID - 24472*/
const Profile P22470 = {"22470", NULL};	/*Profile ID - 22470*/
const Profile P22472 = {"22472", NULL};	/*Profile ID - 22472*/
const Value V17299_1_1 = {S2, NULL};	/*Value = True*/
const Property P17299_1 = {S50, 0, &V17299_1_1, NULL};	/*Property = IsMobile*/
const Value V17299_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17299_5 = {S52, 0, &V17299_5_1277, &P17299_1};	/*Property = ScreenPixelsHeight*/
const Value V17299_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17299_4 = {S54, 0, &V17299_4_1279, &P17299_5};	/*Property = ScreenPixelsWidth*/
const Profile P17299 = {"17299", &P17299_4};	/*Profile ID - 17299*/
const Value V24207_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24207_279 = {S1, 0, &V24207_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24207_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24207_282 = {S3, 0, &V24207_282_57154, &P24207_279};	/*Property = BlobBuilder*/
const Value V24207_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24207_350 = {S4, 0, &V24207_350_57444, &P24207_282};	/*Property = Canvas*/
const Value V24207_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24207_283 = {S5, 0, &V24207_283_57159, &P24207_350};	/*Property = CssBackground*/
const Value V24207_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24207_284 = {S6, 0, &V24207_284_57162, &P24207_283};	/*Property = CssBorderImage*/
const Value V24207_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24207_285 = {S7, 0, &V24207_285_57166, &P24207_284};	/*Property = CssCanvas*/
const Value V24207_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24207_286 = {S8, 0, &V24207_286_57168, &P24207_285};	/*Property = CssColor*/
const Value V24207_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24207_237 = {S9, 0, &V24207_237_55786, &P24207_286};	/*Property = CssColumn*/
const Value V24207_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24207_287 = {S10, 0, &V24207_287_57172, &P24207_237};	/*Property = CssFlexbox*/
const Value V24207_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24207_288 = {S11, 0, &V24207_288_57175, &P24207_287};	/*Property = CssFont*/
const Value V24207_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24207_289 = {S12, 0, &V24207_289_57759, &P24207_288};	/*Property = CssImages*/
const Value V24207_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24207_291 = {S13, 0, &V24207_291_57180, &P24207_289};	/*Property = CssMediaQueries*/
const Value V24207_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24207_292 = {S14, 0, &V24207_292_57183, &P24207_291};	/*Property = CssMinMax*/
const Value V24207_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24207_293 = {S15, 0, &V24207_293_57187, &P24207_292};	/*Property = CssOverflow*/
const Value V24207_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24207_294 = {S16, 0, &V24207_294_57189, &P24207_293};	/*Property = CssPosition*/
const Value V24207_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24207_295 = {S17, 0, &V24207_295_57192, &P24207_294};	/*Property = CssText*/
const Value V24207_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24207_297 = {S18, 0, &V24207_297_57195, &P24207_295};	/*Property = CssTransforms*/
const Value V24207_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24207_241 = {S19, 0, &V24207_241_55799, &P24207_297};	/*Property = CssTransitions*/
const Value V24207_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24207_298 = {S20, 0, &V24207_298_57206, &P24207_241};	/*Property = CssUI*/
const Value V24207_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24207_300 = {S21, 0, &V24207_300_57215, &P24207_298};	/*Property = DataSet*/
const Value V24207_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24207_301 = {S22, 0, &V24207_301_57218, &P24207_300};	/*Property = DataUrl*/
const Value V24207_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24207_302 = {S23, 0, &V24207_302_57221, &P24207_301};	/*Property = DeviceOrientation*/
const Value V24207_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24207_304 = {S24, 0, &V24207_304_57227, &P24207_302};	/*Property = FileReader*/
const Value V24207_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24207_305 = {S25, 0, &V24207_305_57231, &P24207_304};	/*Property = FileSaver*/
const Value V24207_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24207_306 = {S26, 0, &V24207_306_57237, &P24207_305};	/*Property = FileWriter*/
const Value V24207_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24207_307 = {S27, 0, &V24207_307_57239, &P24207_306};	/*Property = FormData*/
const Value V24207_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24207_309 = {S28, 0, &V24207_309_57246, &P24207_307};	/*Property = Fullscreen*/
const Value V24207_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24207_310 = {S29, 0, &V24207_310_57248, &P24207_309};	/*Property = GeoLocation*/
const Value V24207_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24207_312 = {S30, 0, &V24207_312_57254, &P24207_310};	/*Property = History*/
const Value V24207_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24207_314 = {S31, 0, &V24207_314_57260, &P24207_312};	/*Property = Html5*/
const Value V24207_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24207_313 = {S32, 0, &V24207_313_57257, &P24207_314};	/*Property = Html-Media-Capture*/
const Value V24207_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24207_315 = {S33, 0, &V24207_315_57264, &P24207_313};	/*Property = Iframe*/
const Value V24207_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24207_316 = {S34, 0, &V24207_316_57266, &P24207_315};	/*Property = IndexedDB*/
const Value V24207_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24207_317 = {S35, 0, &V24207_317_57269, &P24207_316};	/*Property = Json*/
const Value V24207_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24207_15 = {S37, 0, &V24207_15_54167, &P24207_317};	/*Property = LayoutEngine*/
const Value V24207_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24207_344 = {S38, 0, &V24207_344_57272, &P24207_15};	/*Property = Masking*/
const Value V24207_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24207_320 = {S39, 0, &V24207_320_57290, &P24207_344};	/*Property = PostMessage*/
const Value V24207_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24207_321 = {S40, 0, &V24207_321_57293, &P24207_320};	/*Property = Progress*/
const Value V24207_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24207_322 = {S41, 0, &V24207_322_57296, &P24207_321};	/*Property = Prompts*/
const Value V24207_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24207_323 = {S42, 0, &V24207_323_57299, &P24207_322};	/*Property = Selector*/
const Value V24207_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24207_324 = {S43, 0, &V24207_324_57306, &P24207_323};	/*Property = Svg*/
const Value V24207_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24207_327 = {S44, 0, &V24207_327_57314, &P24207_324};	/*Property = TouchEvents*/
const Value V24207_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24207_328 = {S45, 0, &V24207_328_57318, &P24207_327};	/*Property = Track*/
const Value V24207_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24207_329 = {S46, 0, &V24207_329_57323, &P24207_328};	/*Property = Video*/
const Value V24207_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24207_330 = {S47, 0, &V24207_330_57326, &P24207_329};	/*Property = Viewport*/
const Value V24207_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24207_332 = {S48, 0, &V24207_332_57339, &P24207_330};	/*Property = WebWorkers*/
const Value V24207_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24207_333 = {S49, 0, &V24207_333_57342, &P24207_332};	/*Property = Xhr2*/
const Profile P24207 = {"24207", &P24207_333};	/*Profile ID - 24207*/
const Value V22248_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22248_279 = {S1, 0, &V22248_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22248_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22248_282 = {S3, 0, &V22248_282_57154, &P22248_279};	/*Property = BlobBuilder*/
const Value V22248_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22248_350 = {S4, 0, &V22248_350_57444, &P22248_282};	/*Property = Canvas*/
const Value V22248_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22248_283 = {S5, 0, &V22248_283_57159, &P22248_350};	/*Property = CssBackground*/
const Value V22248_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22248_284 = {S6, 0, &V22248_284_57163, &P22248_283};	/*Property = CssBorderImage*/
const Value V22248_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22248_285 = {S7, 0, &V22248_285_57166, &P22248_284};	/*Property = CssCanvas*/
const Value V22248_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22248_286 = {S8, 0, &V22248_286_57168, &P22248_285};	/*Property = CssColor*/
const Value V22248_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22248_237 = {S9, 0, &V22248_237_55786, &P22248_286};	/*Property = CssColumn*/
const Value V22248_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22248_287 = {S10, 0, &V22248_287_57172, &P22248_237};	/*Property = CssFlexbox*/
const Value V22248_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22248_288 = {S11, 0, &V22248_288_57175, &P22248_287};	/*Property = CssFont*/
const Value V22248_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22248_289 = {S12, 0, &V22248_289_57759, &P22248_288};	/*Property = CssImages*/
const Value V22248_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22248_291 = {S13, 0, &V22248_291_57181, &P22248_289};	/*Property = CssMediaQueries*/
const Value V22248_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22248_292 = {S14, 0, &V22248_292_57183, &P22248_291};	/*Property = CssMinMax*/
const Value V22248_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22248_293 = {S15, 0, &V22248_293_57187, &P22248_292};	/*Property = CssOverflow*/
const Value V22248_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22248_294 = {S16, 0, &V22248_294_57190, &P22248_293};	/*Property = CssPosition*/
const Value V22248_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22248_295 = {S17, 0, &V22248_295_57192, &P22248_294};	/*Property = CssText*/
const Value V22248_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22248_297 = {S18, 0, &V22248_297_57195, &P22248_295};	/*Property = CssTransforms*/
const Value V22248_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22248_241 = {S19, 0, &V22248_241_55799, &P22248_297};	/*Property = CssTransitions*/
const Value V22248_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22248_298 = {S20, 0, &V22248_298_57206, &P22248_241};	/*Property = CssUI*/
const Value V22248_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22248_300 = {S21, 0, &V22248_300_57216, &P22248_298};	/*Property = DataSet*/
const Value V22248_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22248_301 = {S22, 0, &V22248_301_57218, &P22248_300};	/*Property = DataUrl*/
const Value V22248_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22248_302 = {S23, 0, &V22248_302_57222, &P22248_301};	/*Property = DeviceOrientation*/
const Value V22248_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22248_304 = {S24, 0, &V22248_304_57228, &P22248_302};	/*Property = FileReader*/
const Value V22248_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22248_305 = {S25, 0, &V22248_305_57231, &P22248_304};	/*Property = FileSaver*/
const Value V22248_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22248_306 = {S26, 0, &V22248_306_57237, &P22248_305};	/*Property = FileWriter*/
const Value V22248_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22248_307 = {S27, 0, &V22248_307_57240, &P22248_306};	/*Property = FormData*/
const Value V22248_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22248_309 = {S28, 0, &V22248_309_57246, &P22248_307};	/*Property = Fullscreen*/
const Value V22248_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22248_310 = {S29, 0, &V22248_310_57248, &P22248_309};	/*Property = GeoLocation*/
const Value V22248_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22248_312 = {S30, 0, &V22248_312_57255, &P22248_310};	/*Property = History*/
const Value V22248_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22248_314 = {S31, 0, &V22248_314_57261, &P22248_312};	/*Property = Html5*/
const Value V22248_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22248_313 = {S32, 0, &V22248_313_57258, &P22248_314};	/*Property = Html-Media-Capture*/
const Value V22248_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22248_315 = {S33, 0, &V22248_315_57264, &P22248_313};	/*Property = Iframe*/
const Value V22248_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22248_316 = {S34, 0, &V22248_316_57267, &P22248_315};	/*Property = IndexedDB*/
const Value V22248_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22248_317 = {S35, 0, &V22248_317_57269, &P22248_316};	/*Property = Json*/
const Value V22248_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22248_15 = {S37, 0, &V22248_15_72, &P22248_317};	/*Property = LayoutEngine*/
const Value V22248_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22248_344 = {S38, 0, &V22248_344_57272, &P22248_15};	/*Property = Masking*/
const Value V22248_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22248_320 = {S39, 0, &V22248_320_57290, &P22248_344};	/*Property = PostMessage*/
const Value V22248_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22248_321 = {S40, 0, &V22248_321_57293, &P22248_320};	/*Property = Progress*/
const Value V22248_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22248_322 = {S41, 0, &V22248_322_57296, &P22248_321};	/*Property = Prompts*/
const Value V22248_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22248_323 = {S42, 0, &V22248_323_57299, &P22248_322};	/*Property = Selector*/
const Value V22248_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22248_324 = {S43, 0, &V22248_324_57306, &P22248_323};	/*Property = Svg*/
const Value V22248_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22248_327 = {S44, 0, &V22248_327_57315, &P22248_324};	/*Property = TouchEvents*/
const Value V22248_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22248_328 = {S45, 0, &V22248_328_57318, &P22248_327};	/*Property = Track*/
const Value V22248_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22248_329 = {S46, 0, &V22248_329_57323, &P22248_328};	/*Property = Video*/
const Value V22248_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22248_330 = {S47, 0, &V22248_330_57326, &P22248_329};	/*Property = Viewport*/
const Value V22248_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22248_332 = {S48, 0, &V22248_332_57339, &P22248_330};	/*Property = WebWorkers*/
const Value V22248_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22248_333 = {S49, 0, &V22248_333_57342, &P22248_332};	/*Property = Xhr2*/
const Profile P22248 = {"22248", &P22248_333};	/*Profile ID - 22248*/
const Value V21904_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21904_279 = {S1, 0, &V21904_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21904_282_57153 = {S2, NULL};	/*Value = True*/
const Property P21904_282 = {S3, 0, &V21904_282_57153, &P21904_279};	/*Property = BlobBuilder*/
const Value V21904_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21904_350 = {S4, 0, &V21904_350_57444, &P21904_282};	/*Property = Canvas*/
const Value V21904_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21904_283 = {S5, 0, &V21904_283_57159, &P21904_350};	/*Property = CssBackground*/
const Value V21904_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21904_284 = {S6, 0, &V21904_284_57163, &P21904_283};	/*Property = CssBorderImage*/
const Value V21904_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21904_285 = {S7, 0, &V21904_285_57166, &P21904_284};	/*Property = CssCanvas*/
const Value V21904_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21904_286 = {S8, 0, &V21904_286_57168, &P21904_285};	/*Property = CssColor*/
const Value V21904_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21904_237 = {S9, 0, &V21904_237_55784, &P21904_286};	/*Property = CssColumn*/
const Value V21904_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21904_287 = {S10, 0, &V21904_287_57172, &P21904_237};	/*Property = CssFlexbox*/
const Value V21904_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21904_288 = {S11, 0, &V21904_288_57175, &P21904_287};	/*Property = CssFont*/
const Value V21904_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21904_289 = {S12, 0, &V21904_289_57759, &P21904_288};	/*Property = CssImages*/
const Value V21904_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21904_291 = {S13, 0, &V21904_291_57180, &P21904_289};	/*Property = CssMediaQueries*/
const Value V21904_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21904_292 = {S14, 0, &V21904_292_57183, &P21904_291};	/*Property = CssMinMax*/
const Value V21904_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21904_293 = {S15, 0, &V21904_293_57187, &P21904_292};	/*Property = CssOverflow*/
const Value V21904_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21904_294 = {S16, 0, &V21904_294_57189, &P21904_293};	/*Property = CssPosition*/
const Value V21904_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21904_295 = {S17, 0, &V21904_295_57192, &P21904_294};	/*Property = CssText*/
const Value V21904_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21904_297 = {S18, 0, &V21904_297_57195, &P21904_295};	/*Property = CssTransforms*/
const Value V21904_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21904_241 = {S19, 0, &V21904_241_55799, &P21904_297};	/*Property = CssTransitions*/
const Value V21904_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21904_298 = {S20, 0, &V21904_298_57206, &P21904_241};	/*Property = CssUI*/
const Value V21904_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21904_300 = {S21, 0, &V21904_300_57216, &P21904_298};	/*Property = DataSet*/
const Value V21904_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21904_301 = {S22, 0, &V21904_301_57218, &P21904_300};	/*Property = DataUrl*/
const Value V21904_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21904_302 = {S23, 0, &V21904_302_57222, &P21904_301};	/*Property = DeviceOrientation*/
const Value V21904_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21904_304 = {S24, 0, &V21904_304_57227, &P21904_302};	/*Property = FileReader*/
const Value V21904_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21904_305 = {S25, 0, &V21904_305_57231, &P21904_304};	/*Property = FileSaver*/
const Value V21904_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21904_306 = {S26, 0, &V21904_306_57237, &P21904_305};	/*Property = FileWriter*/
const Value V21904_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21904_307 = {S27, 0, &V21904_307_57239, &P21904_306};	/*Property = FormData*/
const Value V21904_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21904_309 = {S28, 0, &V21904_309_57246, &P21904_307};	/*Property = Fullscreen*/
const Value V21904_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21904_310 = {S29, 0, &V21904_310_57248, &P21904_309};	/*Property = GeoLocation*/
const Value V21904_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21904_312 = {S30, 0, &V21904_312_57254, &P21904_310};	/*Property = History*/
const Value V21904_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21904_314 = {S31, 0, &V21904_314_57261, &P21904_312};	/*Property = Html5*/
const Value V21904_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21904_313 = {S32, 0, &V21904_313_57258, &P21904_314};	/*Property = Html-Media-Capture*/
const Value V21904_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21904_315 = {S33, 0, &V21904_315_57264, &P21904_313};	/*Property = Iframe*/
const Value V21904_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21904_316 = {S34, 0, &V21904_316_57266, &P21904_315};	/*Property = IndexedDB*/
const Value V21904_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21904_317 = {S35, 0, &V21904_317_57269, &P21904_316};	/*Property = Json*/
const Value V21904_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21904_15 = {S37, 0, &V21904_15_28, &P21904_317};	/*Property = LayoutEngine*/
const Value V21904_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21904_344 = {S38, 0, &V21904_344_57272, &P21904_15};	/*Property = Masking*/
const Value V21904_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21904_320 = {S39, 0, &V21904_320_57290, &P21904_344};	/*Property = PostMessage*/
const Value V21904_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21904_321 = {S40, 0, &V21904_321_57293, &P21904_320};	/*Property = Progress*/
const Value V21904_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21904_322 = {S41, 0, &V21904_322_57296, &P21904_321};	/*Property = Prompts*/
const Value V21904_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21904_323 = {S42, 0, &V21904_323_57299, &P21904_322};	/*Property = Selector*/
const Value V21904_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21904_324 = {S43, 0, &V21904_324_57306, &P21904_323};	/*Property = Svg*/
const Value V21904_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21904_327 = {S44, 0, &V21904_327_57315, &P21904_324};	/*Property = TouchEvents*/
const Value V21904_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21904_328 = {S45, 0, &V21904_328_57318, &P21904_327};	/*Property = Track*/
const Value V21904_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21904_329 = {S46, 0, &V21904_329_57323, &P21904_328};	/*Property = Video*/
const Value V21904_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21904_330 = {S47, 0, &V21904_330_57326, &P21904_329};	/*Property = Viewport*/
const Value V21904_332_57338 = {S2, NULL};	/*Value = True*/
const Property P21904_332 = {S48, 0, &V21904_332_57338, &P21904_330};	/*Property = WebWorkers*/
const Value V21904_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21904_333 = {S49, 0, &V21904_333_57342, &P21904_332};	/*Property = Xhr2*/
const Profile P21904 = {"21904", &P21904_333};	/*Profile ID - 21904*/
const Value V16292_1_1 = {S2, NULL};	/*Value = True*/
const Property P16292_1 = {S50, 0, &V16292_1_1, NULL};	/*Property = IsMobile*/
const Value V16292_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P16292_5 = {S52, 0, &V16292_5_9981, &P16292_1};	/*Property = ScreenPixelsHeight*/
const Value V16292_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P16292_4 = {S54, 0, &V16292_4_39098, &P16292_5};	/*Property = ScreenPixelsWidth*/
const Profile P16292 = {"16292", &P16292_4};	/*Profile ID - 16292*/
const Value V18351_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_279 = {S1, 0, &V18351_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18351_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_282 = {S3, 0, &V18351_282_57155, &P18351_279};	/*Property = BlobBuilder*/
const Value V18351_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_350 = {S4, 0, &V18351_350_57446, &P18351_282};	/*Property = Canvas*/
const Value V18351_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_283 = {S5, 0, &V18351_283_57161, &P18351_350};	/*Property = CssBackground*/
const Value V18351_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_284 = {S6, 0, &V18351_284_57164, &P18351_283};	/*Property = CssBorderImage*/
const Value V18351_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_285 = {S7, 0, &V18351_285_57167, &P18351_284};	/*Property = CssCanvas*/
const Value V18351_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_286 = {S8, 0, &V18351_286_57170, &P18351_285};	/*Property = CssColor*/
const Value V18351_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_237 = {S9, 0, &V18351_237_55786, &P18351_286};	/*Property = CssColumn*/
const Value V18351_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_287 = {S10, 0, &V18351_287_57173, &P18351_237};	/*Property = CssFlexbox*/
const Value V18351_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_288 = {S11, 0, &V18351_288_57176, &P18351_287};	/*Property = CssFont*/
const Value V18351_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_289 = {S12, 0, &V18351_289_86899, &P18351_288};	/*Property = CssImages*/
const Value V18351_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_291 = {S13, 0, &V18351_291_57182, &P18351_289};	/*Property = CssMediaQueries*/
const Value V18351_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_292 = {S14, 0, &V18351_292_57185, &P18351_291};	/*Property = CssMinMax*/
const Value V18351_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_293 = {S15, 0, &V18351_293_57188, &P18351_292};	/*Property = CssOverflow*/
const Value V18351_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_294 = {S16, 0, &V18351_294_57191, &P18351_293};	/*Property = CssPosition*/
const Value V18351_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_295 = {S17, 0, &V18351_295_57194, &P18351_294};	/*Property = CssText*/
const Value V18351_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_297 = {S18, 0, &V18351_297_57197, &P18351_295};	/*Property = CssTransforms*/
const Value V18351_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_241 = {S19, 0, &V18351_241_55801, &P18351_297};	/*Property = CssTransitions*/
const Value V18351_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_298 = {S20, 0, &V18351_298_57208, &P18351_241};	/*Property = CssUI*/
const Value V18351_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_300 = {S21, 0, &V18351_300_57217, &P18351_298};	/*Property = DataSet*/
const Value V18351_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_301 = {S22, 0, &V18351_301_57220, &P18351_300};	/*Property = DataUrl*/
const Value V18351_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_302 = {S23, 0, &V18351_302_57223, &P18351_301};	/*Property = DeviceOrientation*/
const Value V18351_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_304 = {S24, 0, &V18351_304_57229, &P18351_302};	/*Property = FileReader*/
const Value V18351_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_305 = {S25, 0, &V18351_305_57232, &P18351_304};	/*Property = FileSaver*/
const Value V18351_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_306 = {S26, 0, &V18351_306_57238, &P18351_305};	/*Property = FileWriter*/
const Value V18351_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_307 = {S27, 0, &V18351_307_57241, &P18351_306};	/*Property = FormData*/
const Value V18351_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_309 = {S28, 0, &V18351_309_57247, &P18351_307};	/*Property = Fullscreen*/
const Value V18351_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_310 = {S29, 0, &V18351_310_57250, &P18351_309};	/*Property = GeoLocation*/
const Value V18351_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_312 = {S30, 0, &V18351_312_57256, &P18351_310};	/*Property = History*/
const Value V18351_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_314 = {S31, 0, &V18351_314_57262, &P18351_312};	/*Property = Html5*/
const Value V18351_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_313 = {S32, 0, &V18351_313_57259, &P18351_314};	/*Property = Html-Media-Capture*/
const Value V18351_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_315 = {S33, 0, &V18351_315_57265, &P18351_313};	/*Property = Iframe*/
const Value V18351_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_316 = {S34, 0, &V18351_316_57268, &P18351_315};	/*Property = IndexedDB*/
const Value V18351_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_317 = {S35, 0, &V18351_317_57271, &P18351_316};	/*Property = Json*/
const Value V18351_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P18351_15 = {S37, 0, &V18351_15_30, &P18351_317};	/*Property = LayoutEngine*/
const Value V18351_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_344 = {S38, 0, &V18351_344_57274, &P18351_15};	/*Property = Masking*/
const Value V18351_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_320 = {S39, 0, &V18351_320_57292, &P18351_344};	/*Property = PostMessage*/
const Value V18351_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_321 = {S40, 0, &V18351_321_57295, &P18351_320};	/*Property = Progress*/
const Value V18351_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_322 = {S41, 0, &V18351_322_57298, &P18351_321};	/*Property = Prompts*/
const Value V18351_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_323 = {S42, 0, &V18351_323_57301, &P18351_322};	/*Property = Selector*/
const Value V18351_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_324 = {S43, 0, &V18351_324_57307, &P18351_323};	/*Property = Svg*/
const Value V18351_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_327 = {S44, 0, &V18351_327_57316, &P18351_324};	/*Property = TouchEvents*/
const Value V18351_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_328 = {S45, 0, &V18351_328_57319, &P18351_327};	/*Property = Track*/
const Value V18351_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_329 = {S46, 0, &V18351_329_57325, &P18351_328};	/*Property = Video*/
const Value V18351_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_330 = {S47, 0, &V18351_330_57328, &P18351_329};	/*Property = Viewport*/
const Value V18351_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_332 = {S48, 0, &V18351_332_57340, &P18351_330};	/*Property = WebWorkers*/
const Value V18351_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18351_333 = {S49, 0, &V18351_333_57343, &P18351_332};	/*Property = Xhr2*/
const Profile P18351 = {"18351", &P18351_333};	/*Profile ID - 18351*/
const Value V18070_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18070_279 = {S1, 0, &V18070_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18070_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18070_282 = {S3, 0, &V18070_282_57154, &P18070_279};	/*Property = BlobBuilder*/
const Value V18070_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18070_350 = {S4, 0, &V18070_350_57444, &P18070_282};	/*Property = Canvas*/
const Value V18070_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18070_283 = {S5, 0, &V18070_283_57159, &P18070_350};	/*Property = CssBackground*/
const Value V18070_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18070_284 = {S6, 0, &V18070_284_57163, &P18070_283};	/*Property = CssBorderImage*/
const Value V18070_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18070_285 = {S7, 0, &V18070_285_57166, &P18070_284};	/*Property = CssCanvas*/
const Value V18070_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18070_286 = {S8, 0, &V18070_286_57168, &P18070_285};	/*Property = CssColor*/
const Value V18070_237_55784 = {S2, NULL};	/*Value = True*/
const Property P18070_237 = {S9, 0, &V18070_237_55784, &P18070_286};	/*Property = CssColumn*/
const Value V18070_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18070_287 = {S10, 0, &V18070_287_57172, &P18070_237};	/*Property = CssFlexbox*/
const Value V18070_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18070_288 = {S11, 0, &V18070_288_57175, &P18070_287};	/*Property = CssFont*/
const Value V18070_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18070_289 = {S12, 0, &V18070_289_57759, &P18070_288};	/*Property = CssImages*/
const Value V18070_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18070_291 = {S13, 0, &V18070_291_57181, &P18070_289};	/*Property = CssMediaQueries*/
const Value V18070_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18070_292 = {S14, 0, &V18070_292_57183, &P18070_291};	/*Property = CssMinMax*/
const Value V18070_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18070_293 = {S15, 0, &V18070_293_57187, &P18070_292};	/*Property = CssOverflow*/
const Value V18070_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18070_294 = {S16, 0, &V18070_294_57190, &P18070_293};	/*Property = CssPosition*/
const Value V18070_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18070_295 = {S17, 0, &V18070_295_57193, &P18070_294};	/*Property = CssText*/
const Value V18070_297_57196 = {S0, NULL};	/*Value = False*/
const Property P18070_297 = {S18, 0, &V18070_297_57196, &P18070_295};	/*Property = CssTransforms*/
const Value V18070_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18070_241 = {S19, 0, &V18070_241_55799, &P18070_297};	/*Property = CssTransitions*/
const Value V18070_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18070_298 = {S20, 0, &V18070_298_57206, &P18070_241};	/*Property = CssUI*/
const Value V18070_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18070_300 = {S21, 0, &V18070_300_57216, &P18070_298};	/*Property = DataSet*/
const Value V18070_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18070_301 = {S22, 0, &V18070_301_57218, &P18070_300};	/*Property = DataUrl*/
const Value V18070_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18070_302 = {S23, 0, &V18070_302_57222, &P18070_301};	/*Property = DeviceOrientation*/
const Value V18070_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18070_304 = {S24, 0, &V18070_304_57228, &P18070_302};	/*Property = FileReader*/
const Value V18070_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18070_305 = {S25, 0, &V18070_305_57231, &P18070_304};	/*Property = FileSaver*/
const Value V18070_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18070_306 = {S26, 0, &V18070_306_57237, &P18070_305};	/*Property = FileWriter*/
const Value V18070_307_57240 = {S0, NULL};	/*Value = False*/
const Property P18070_307 = {S27, 0, &V18070_307_57240, &P18070_306};	/*Property = FormData*/
const Value V18070_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18070_309 = {S28, 0, &V18070_309_57246, &P18070_307};	/*Property = Fullscreen*/
const Value V18070_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18070_310 = {S29, 0, &V18070_310_57248, &P18070_309};	/*Property = GeoLocation*/
const Value V18070_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18070_312 = {S30, 0, &V18070_312_57255, &P18070_310};	/*Property = History*/
const Value V18070_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18070_314 = {S31, 0, &V18070_314_57261, &P18070_312};	/*Property = Html5*/
const Value V18070_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18070_313 = {S32, 0, &V18070_313_57258, &P18070_314};	/*Property = Html-Media-Capture*/
const Value V18070_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18070_315 = {S33, 0, &V18070_315_57264, &P18070_313};	/*Property = Iframe*/
const Value V18070_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18070_316 = {S34, 0, &V18070_316_57267, &P18070_315};	/*Property = IndexedDB*/
const Value V18070_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18070_317 = {S35, 0, &V18070_317_57269, &P18070_316};	/*Property = Json*/
const Value V18070_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18070_15 = {S37, 0, &V18070_15_31, &P18070_317};	/*Property = LayoutEngine*/
const Value V18070_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18070_344 = {S38, 0, &V18070_344_57272, &P18070_15};	/*Property = Masking*/
const Value V18070_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18070_320 = {S39, 0, &V18070_320_57290, &P18070_344};	/*Property = PostMessage*/
const Value V18070_321_57294 = {S0, NULL};	/*Value = False*/
const Property P18070_321 = {S40, 0, &V18070_321_57294, &P18070_320};	/*Property = Progress*/
const Value V18070_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18070_322 = {S41, 0, &V18070_322_57296, &P18070_321};	/*Property = Prompts*/
const Value V18070_323_57300 = {S0, NULL};	/*Value = False*/
const Property P18070_323 = {S42, 0, &V18070_323_57300, &P18070_322};	/*Property = Selector*/
const Value V18070_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18070_324 = {S43, 0, &V18070_324_57306, &P18070_323};	/*Property = Svg*/
const Value V18070_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18070_327 = {S44, 0, &V18070_327_57315, &P18070_324};	/*Property = TouchEvents*/
const Value V18070_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18070_328 = {S45, 0, &V18070_328_57318, &P18070_327};	/*Property = Track*/
const Value V18070_329_57324 = {S0, NULL};	/*Value = False*/
const Property P18070_329 = {S46, 0, &V18070_329_57324, &P18070_328};	/*Property = Video*/
const Value V18070_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18070_330 = {S47, 0, &V18070_330_57326, &P18070_329};	/*Property = Viewport*/
const Value V18070_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18070_332 = {S48, 0, &V18070_332_57339, &P18070_330};	/*Property = WebWorkers*/
const Value V18070_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18070_333 = {S49, 0, &V18070_333_57342, &P18070_332};	/*Property = Xhr2*/
const Profile P18070 = {"18070", &P18070_333};	/*Profile ID - 18070*/
const Value V16877_1_1 = {S2, NULL};	/*Value = True*/
const Property P16877_1 = {S50, 0, &V16877_1_1, NULL};	/*Property = IsMobile*/
const Value V16877_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16877_5 = {S52, 0, &V16877_5_1277, &P16877_1};	/*Property = ScreenPixelsHeight*/
const Value V16877_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P16877_4 = {S54, 0, &V16877_4_8554, &P16877_5};	/*Property = ScreenPixelsWidth*/
const Profile P16877 = {"16877", &P16877_4};	/*Profile ID - 16877*/
const Value V24474_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_279 = {S1, 0, &V24474_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24474_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_282 = {S3, 0, &V24474_282_57155, &P24474_279};	/*Property = BlobBuilder*/
const Value V24474_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_350 = {S4, 0, &V24474_350_57446, &P24474_282};	/*Property = Canvas*/
const Value V24474_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_283 = {S5, 0, &V24474_283_57161, &P24474_350};	/*Property = CssBackground*/
const Value V24474_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_284 = {S6, 0, &V24474_284_57164, &P24474_283};	/*Property = CssBorderImage*/
const Value V24474_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_285 = {S7, 0, &V24474_285_57167, &P24474_284};	/*Property = CssCanvas*/
const Value V24474_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_286 = {S8, 0, &V24474_286_57170, &P24474_285};	/*Property = CssColor*/
const Value V24474_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_237 = {S9, 0, &V24474_237_55786, &P24474_286};	/*Property = CssColumn*/
const Value V24474_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_287 = {S10, 0, &V24474_287_57173, &P24474_237};	/*Property = CssFlexbox*/
const Value V24474_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_288 = {S11, 0, &V24474_288_57176, &P24474_287};	/*Property = CssFont*/
const Value V24474_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_289 = {S12, 0, &V24474_289_86899, &P24474_288};	/*Property = CssImages*/
const Value V24474_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_291 = {S13, 0, &V24474_291_57182, &P24474_289};	/*Property = CssMediaQueries*/
const Value V24474_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_292 = {S14, 0, &V24474_292_57185, &P24474_291};	/*Property = CssMinMax*/
const Value V24474_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_293 = {S15, 0, &V24474_293_57188, &P24474_292};	/*Property = CssOverflow*/
const Value V24474_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_294 = {S16, 0, &V24474_294_57191, &P24474_293};	/*Property = CssPosition*/
const Value V24474_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_295 = {S17, 0, &V24474_295_57194, &P24474_294};	/*Property = CssText*/
const Value V24474_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_297 = {S18, 0, &V24474_297_57197, &P24474_295};	/*Property = CssTransforms*/
const Value V24474_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_241 = {S19, 0, &V24474_241_55801, &P24474_297};	/*Property = CssTransitions*/
const Value V24474_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_298 = {S20, 0, &V24474_298_57208, &P24474_241};	/*Property = CssUI*/
const Value V24474_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_300 = {S21, 0, &V24474_300_57217, &P24474_298};	/*Property = DataSet*/
const Value V24474_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_301 = {S22, 0, &V24474_301_57220, &P24474_300};	/*Property = DataUrl*/
const Value V24474_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_302 = {S23, 0, &V24474_302_57223, &P24474_301};	/*Property = DeviceOrientation*/
const Value V24474_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_304 = {S24, 0, &V24474_304_57229, &P24474_302};	/*Property = FileReader*/
const Value V24474_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_305 = {S25, 0, &V24474_305_57232, &P24474_304};	/*Property = FileSaver*/
const Value V24474_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_306 = {S26, 0, &V24474_306_57238, &P24474_305};	/*Property = FileWriter*/
const Value V24474_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_307 = {S27, 0, &V24474_307_57241, &P24474_306};	/*Property = FormData*/
const Value V24474_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_309 = {S28, 0, &V24474_309_57247, &P24474_307};	/*Property = Fullscreen*/
const Value V24474_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_310 = {S29, 0, &V24474_310_57250, &P24474_309};	/*Property = GeoLocation*/
const Value V24474_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_312 = {S30, 0, &V24474_312_57256, &P24474_310};	/*Property = History*/
const Value V24474_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_314 = {S31, 0, &V24474_314_57262, &P24474_312};	/*Property = Html5*/
const Value V24474_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_313 = {S32, 0, &V24474_313_57259, &P24474_314};	/*Property = Html-Media-Capture*/
const Value V24474_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_315 = {S33, 0, &V24474_315_57265, &P24474_313};	/*Property = Iframe*/
const Value V24474_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_316 = {S34, 0, &V24474_316_57268, &P24474_315};	/*Property = IndexedDB*/
const Value V24474_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_317 = {S35, 0, &V24474_317_57271, &P24474_316};	/*Property = Json*/
const Value V24474_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24474_15 = {S37, 0, &V24474_15_54167, &P24474_317};	/*Property = LayoutEngine*/
const Value V24474_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_344 = {S38, 0, &V24474_344_57274, &P24474_15};	/*Property = Masking*/
const Value V24474_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_320 = {S39, 0, &V24474_320_57292, &P24474_344};	/*Property = PostMessage*/
const Value V24474_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_321 = {S40, 0, &V24474_321_57295, &P24474_320};	/*Property = Progress*/
const Value V24474_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_322 = {S41, 0, &V24474_322_57298, &P24474_321};	/*Property = Prompts*/
const Value V24474_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_323 = {S42, 0, &V24474_323_57301, &P24474_322};	/*Property = Selector*/
const Value V24474_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_324 = {S43, 0, &V24474_324_57307, &P24474_323};	/*Property = Svg*/
const Value V24474_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_327 = {S44, 0, &V24474_327_57316, &P24474_324};	/*Property = TouchEvents*/
const Value V24474_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_328 = {S45, 0, &V24474_328_57319, &P24474_327};	/*Property = Track*/
const Value V24474_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_329 = {S46, 0, &V24474_329_57325, &P24474_328};	/*Property = Video*/
const Value V24474_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_330 = {S47, 0, &V24474_330_57328, &P24474_329};	/*Property = Viewport*/
const Value V24474_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_332 = {S48, 0, &V24474_332_57340, &P24474_330};	/*Property = WebWorkers*/
const Value V24474_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24474_333 = {S49, 0, &V24474_333_57343, &P24474_332};	/*Property = Xhr2*/
const Profile P24474 = {"24474", &P24474_333};	/*Profile ID - 24474*/
const Value V22524_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22524_279 = {S1, 0, &V22524_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22524_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22524_282 = {S3, 0, &V22524_282_57154, &P22524_279};	/*Property = BlobBuilder*/
const Value V22524_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22524_350 = {S4, 0, &V22524_350_57444, &P22524_282};	/*Property = Canvas*/
const Value V22524_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22524_283 = {S5, 0, &V22524_283_57159, &P22524_350};	/*Property = CssBackground*/
const Value V22524_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22524_284 = {S6, 0, &V22524_284_57163, &P22524_283};	/*Property = CssBorderImage*/
const Value V22524_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22524_285 = {S7, 0, &V22524_285_57166, &P22524_284};	/*Property = CssCanvas*/
const Value V22524_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22524_286 = {S8, 0, &V22524_286_57168, &P22524_285};	/*Property = CssColor*/
const Value V22524_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22524_237 = {S9, 0, &V22524_237_55786, &P22524_286};	/*Property = CssColumn*/
const Value V22524_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22524_287 = {S10, 0, &V22524_287_57172, &P22524_237};	/*Property = CssFlexbox*/
const Value V22524_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22524_288 = {S11, 0, &V22524_288_57175, &P22524_287};	/*Property = CssFont*/
const Value V22524_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22524_289 = {S12, 0, &V22524_289_57759, &P22524_288};	/*Property = CssImages*/
const Value V22524_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22524_291 = {S13, 0, &V22524_291_57181, &P22524_289};	/*Property = CssMediaQueries*/
const Value V22524_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22524_292 = {S14, 0, &V22524_292_57183, &P22524_291};	/*Property = CssMinMax*/
const Value V22524_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22524_293 = {S15, 0, &V22524_293_57187, &P22524_292};	/*Property = CssOverflow*/
const Value V22524_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22524_294 = {S16, 0, &V22524_294_57190, &P22524_293};	/*Property = CssPosition*/
const Value V22524_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22524_295 = {S17, 0, &V22524_295_57192, &P22524_294};	/*Property = CssText*/
const Value V22524_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22524_297 = {S18, 0, &V22524_297_57195, &P22524_295};	/*Property = CssTransforms*/
const Value V22524_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22524_241 = {S19, 0, &V22524_241_55799, &P22524_297};	/*Property = CssTransitions*/
const Value V22524_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22524_298 = {S20, 0, &V22524_298_57206, &P22524_241};	/*Property = CssUI*/
const Value V22524_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22524_300 = {S21, 0, &V22524_300_57216, &P22524_298};	/*Property = DataSet*/
const Value V22524_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22524_301 = {S22, 0, &V22524_301_57218, &P22524_300};	/*Property = DataUrl*/
const Value V22524_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22524_302 = {S23, 0, &V22524_302_57222, &P22524_301};	/*Property = DeviceOrientation*/
const Value V22524_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22524_304 = {S24, 0, &V22524_304_57228, &P22524_302};	/*Property = FileReader*/
const Value V22524_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22524_305 = {S25, 0, &V22524_305_57231, &P22524_304};	/*Property = FileSaver*/
const Value V22524_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22524_306 = {S26, 0, &V22524_306_57237, &P22524_305};	/*Property = FileWriter*/
const Value V22524_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22524_307 = {S27, 0, &V22524_307_57240, &P22524_306};	/*Property = FormData*/
const Value V22524_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22524_309 = {S28, 0, &V22524_309_57246, &P22524_307};	/*Property = Fullscreen*/
const Value V22524_310_57249 = {S0, NULL};	/*Value = False*/
const Property P22524_310 = {S29, 0, &V22524_310_57249, &P22524_309};	/*Property = GeoLocation*/
const Value V22524_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22524_312 = {S30, 0, &V22524_312_57255, &P22524_310};	/*Property = History*/
const Value V22524_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22524_314 = {S31, 0, &V22524_314_57261, &P22524_312};	/*Property = Html5*/
const Value V22524_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22524_313 = {S32, 0, &V22524_313_57258, &P22524_314};	/*Property = Html-Media-Capture*/
const Value V22524_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22524_315 = {S33, 0, &V22524_315_57264, &P22524_313};	/*Property = Iframe*/
const Value V22524_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22524_316 = {S34, 0, &V22524_316_57267, &P22524_315};	/*Property = IndexedDB*/
const Value V22524_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22524_317 = {S35, 0, &V22524_317_57269, &P22524_316};	/*Property = Json*/
const Value V22524_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22524_15 = {S37, 0, &V22524_15_28, &P22524_317};	/*Property = LayoutEngine*/
const Value V22524_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22524_344 = {S38, 0, &V22524_344_57272, &P22524_15};	/*Property = Masking*/
const Value V22524_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22524_320 = {S39, 0, &V22524_320_57290, &P22524_344};	/*Property = PostMessage*/
const Value V22524_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22524_321 = {S40, 0, &V22524_321_57293, &P22524_320};	/*Property = Progress*/
const Value V22524_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22524_322 = {S41, 0, &V22524_322_57296, &P22524_321};	/*Property = Prompts*/
const Value V22524_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22524_323 = {S42, 0, &V22524_323_57299, &P22524_322};	/*Property = Selector*/
const Value V22524_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22524_324 = {S43, 0, &V22524_324_57306, &P22524_323};	/*Property = Svg*/
const Value V22524_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22524_327 = {S44, 0, &V22524_327_57315, &P22524_324};	/*Property = TouchEvents*/
const Value V22524_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22524_328 = {S45, 0, &V22524_328_57318, &P22524_327};	/*Property = Track*/
const Value V22524_329_57324 = {S0, NULL};	/*Value = False*/
const Property P22524_329 = {S46, 0, &V22524_329_57324, &P22524_328};	/*Property = Video*/
const Value V22524_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22524_330 = {S47, 0, &V22524_330_57326, &P22524_329};	/*Property = Viewport*/
const Value V22524_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22524_332 = {S48, 0, &V22524_332_57339, &P22524_330};	/*Property = WebWorkers*/
const Value V22524_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22524_333 = {S49, 0, &V22524_333_57342, &P22524_332};	/*Property = Xhr2*/
const Profile P22524 = {"22524", &P22524_333};	/*Profile ID - 22524*/
const Value V24642_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_279 = {S1, 0, &V24642_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24642_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_282 = {S3, 0, &V24642_282_57155, &P24642_279};	/*Property = BlobBuilder*/
const Value V24642_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_350 = {S4, 0, &V24642_350_57446, &P24642_282};	/*Property = Canvas*/
const Value V24642_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_283 = {S5, 0, &V24642_283_57161, &P24642_350};	/*Property = CssBackground*/
const Value V24642_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_284 = {S6, 0, &V24642_284_57164, &P24642_283};	/*Property = CssBorderImage*/
const Value V24642_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_285 = {S7, 0, &V24642_285_57167, &P24642_284};	/*Property = CssCanvas*/
const Value V24642_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_286 = {S8, 0, &V24642_286_57170, &P24642_285};	/*Property = CssColor*/
const Value V24642_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_237 = {S9, 0, &V24642_237_55786, &P24642_286};	/*Property = CssColumn*/
const Value V24642_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_287 = {S10, 0, &V24642_287_57173, &P24642_237};	/*Property = CssFlexbox*/
const Value V24642_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_288 = {S11, 0, &V24642_288_57176, &P24642_287};	/*Property = CssFont*/
const Value V24642_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_289 = {S12, 0, &V24642_289_86899, &P24642_288};	/*Property = CssImages*/
const Value V24642_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_291 = {S13, 0, &V24642_291_57182, &P24642_289};	/*Property = CssMediaQueries*/
const Value V24642_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_292 = {S14, 0, &V24642_292_57185, &P24642_291};	/*Property = CssMinMax*/
const Value V24642_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_293 = {S15, 0, &V24642_293_57188, &P24642_292};	/*Property = CssOverflow*/
const Value V24642_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_294 = {S16, 0, &V24642_294_57191, &P24642_293};	/*Property = CssPosition*/
const Value V24642_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_295 = {S17, 0, &V24642_295_57194, &P24642_294};	/*Property = CssText*/
const Value V24642_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_297 = {S18, 0, &V24642_297_57197, &P24642_295};	/*Property = CssTransforms*/
const Value V24642_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_241 = {S19, 0, &V24642_241_55801, &P24642_297};	/*Property = CssTransitions*/
const Value V24642_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_298 = {S20, 0, &V24642_298_57208, &P24642_241};	/*Property = CssUI*/
const Value V24642_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_300 = {S21, 0, &V24642_300_57217, &P24642_298};	/*Property = DataSet*/
const Value V24642_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_301 = {S22, 0, &V24642_301_57220, &P24642_300};	/*Property = DataUrl*/
const Value V24642_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_302 = {S23, 0, &V24642_302_57223, &P24642_301};	/*Property = DeviceOrientation*/
const Value V24642_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_304 = {S24, 0, &V24642_304_57229, &P24642_302};	/*Property = FileReader*/
const Value V24642_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_305 = {S25, 0, &V24642_305_57232, &P24642_304};	/*Property = FileSaver*/
const Value V24642_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_306 = {S26, 0, &V24642_306_57238, &P24642_305};	/*Property = FileWriter*/
const Value V24642_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_307 = {S27, 0, &V24642_307_57241, &P24642_306};	/*Property = FormData*/
const Value V24642_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_309 = {S28, 0, &V24642_309_57247, &P24642_307};	/*Property = Fullscreen*/
const Value V24642_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_310 = {S29, 0, &V24642_310_57250, &P24642_309};	/*Property = GeoLocation*/
const Value V24642_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_312 = {S30, 0, &V24642_312_57256, &P24642_310};	/*Property = History*/
const Value V24642_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_314 = {S31, 0, &V24642_314_57262, &P24642_312};	/*Property = Html5*/
const Value V24642_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_313 = {S32, 0, &V24642_313_57259, &P24642_314};	/*Property = Html-Media-Capture*/
const Value V24642_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_315 = {S33, 0, &V24642_315_57265, &P24642_313};	/*Property = Iframe*/
const Value V24642_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_316 = {S34, 0, &V24642_316_57268, &P24642_315};	/*Property = IndexedDB*/
const Value V24642_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_317 = {S35, 0, &V24642_317_57271, &P24642_316};	/*Property = Json*/
const Value V24642_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24642_15 = {S37, 0, &V24642_15_90837, &P24642_317};	/*Property = LayoutEngine*/
const Value V24642_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_344 = {S38, 0, &V24642_344_57274, &P24642_15};	/*Property = Masking*/
const Value V24642_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_320 = {S39, 0, &V24642_320_57292, &P24642_344};	/*Property = PostMessage*/
const Value V24642_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_321 = {S40, 0, &V24642_321_57295, &P24642_320};	/*Property = Progress*/
const Value V24642_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_322 = {S41, 0, &V24642_322_57298, &P24642_321};	/*Property = Prompts*/
const Value V24642_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_323 = {S42, 0, &V24642_323_57301, &P24642_322};	/*Property = Selector*/
const Value V24642_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_324 = {S43, 0, &V24642_324_57307, &P24642_323};	/*Property = Svg*/
const Value V24642_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_327 = {S44, 0, &V24642_327_57316, &P24642_324};	/*Property = TouchEvents*/
const Value V24642_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_328 = {S45, 0, &V24642_328_57319, &P24642_327};	/*Property = Track*/
const Value V24642_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_329 = {S46, 0, &V24642_329_57325, &P24642_328};	/*Property = Video*/
const Value V24642_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_330 = {S47, 0, &V24642_330_57328, &P24642_329};	/*Property = Viewport*/
const Value V24642_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_332 = {S48, 0, &V24642_332_57340, &P24642_330};	/*Property = WebWorkers*/
const Value V24642_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24642_333 = {S49, 0, &V24642_333_57343, &P24642_332};	/*Property = Xhr2*/
const Profile P24642 = {"24642", &P24642_333};	/*Profile ID - 24642*/
const Value V18147_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18147_279 = {S1, 0, &V18147_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18147_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18147_282 = {S3, 0, &V18147_282_57154, &P18147_279};	/*Property = BlobBuilder*/
const Value V18147_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18147_350 = {S4, 0, &V18147_350_57444, &P18147_282};	/*Property = Canvas*/
const Value V18147_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18147_283 = {S5, 0, &V18147_283_57159, &P18147_350};	/*Property = CssBackground*/
const Value V18147_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18147_284 = {S6, 0, &V18147_284_57163, &P18147_283};	/*Property = CssBorderImage*/
const Value V18147_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18147_285 = {S7, 0, &V18147_285_57166, &P18147_284};	/*Property = CssCanvas*/
const Value V18147_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18147_286 = {S8, 0, &V18147_286_57168, &P18147_285};	/*Property = CssColor*/
const Value V18147_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18147_237 = {S9, 0, &V18147_237_55786, &P18147_286};	/*Property = CssColumn*/
const Value V18147_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18147_287 = {S10, 0, &V18147_287_57172, &P18147_237};	/*Property = CssFlexbox*/
const Value V18147_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18147_288 = {S11, 0, &V18147_288_57175, &P18147_287};	/*Property = CssFont*/
const Value V18147_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18147_289 = {S12, 0, &V18147_289_57759, &P18147_288};	/*Property = CssImages*/
const Value V18147_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18147_291 = {S13, 0, &V18147_291_57181, &P18147_289};	/*Property = CssMediaQueries*/
const Value V18147_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18147_292 = {S14, 0, &V18147_292_57183, &P18147_291};	/*Property = CssMinMax*/
const Value V18147_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18147_293 = {S15, 0, &V18147_293_57187, &P18147_292};	/*Property = CssOverflow*/
const Value V18147_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18147_294 = {S16, 0, &V18147_294_57190, &P18147_293};	/*Property = CssPosition*/
const Value V18147_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18147_295 = {S17, 0, &V18147_295_57193, &P18147_294};	/*Property = CssText*/
const Value V18147_297_57196 = {S0, NULL};	/*Value = False*/
const Property P18147_297 = {S18, 0, &V18147_297_57196, &P18147_295};	/*Property = CssTransforms*/
const Value V18147_241_55800 = {S0, NULL};	/*Value = False*/
const Property P18147_241 = {S19, 0, &V18147_241_55800, &P18147_297};	/*Property = CssTransitions*/
const Value V18147_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18147_298 = {S20, 0, &V18147_298_57206, &P18147_241};	/*Property = CssUI*/
const Value V18147_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18147_300 = {S21, 0, &V18147_300_57216, &P18147_298};	/*Property = DataSet*/
const Value V18147_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18147_301 = {S22, 0, &V18147_301_57218, &P18147_300};	/*Property = DataUrl*/
const Value V18147_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18147_302 = {S23, 0, &V18147_302_57222, &P18147_301};	/*Property = DeviceOrientation*/
const Value V18147_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18147_304 = {S24, 0, &V18147_304_57228, &P18147_302};	/*Property = FileReader*/
const Value V18147_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18147_305 = {S25, 0, &V18147_305_57231, &P18147_304};	/*Property = FileSaver*/
const Value V18147_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18147_306 = {S26, 0, &V18147_306_57237, &P18147_305};	/*Property = FileWriter*/
const Value V18147_307_57240 = {S0, NULL};	/*Value = False*/
const Property P18147_307 = {S27, 0, &V18147_307_57240, &P18147_306};	/*Property = FormData*/
const Value V18147_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18147_309 = {S28, 0, &V18147_309_57246, &P18147_307};	/*Property = Fullscreen*/
const Value V18147_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18147_310 = {S29, 0, &V18147_310_57248, &P18147_309};	/*Property = GeoLocation*/
const Value V18147_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18147_312 = {S30, 0, &V18147_312_57255, &P18147_310};	/*Property = History*/
const Value V18147_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18147_314 = {S31, 0, &V18147_314_57261, &P18147_312};	/*Property = Html5*/
const Value V18147_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18147_313 = {S32, 0, &V18147_313_57258, &P18147_314};	/*Property = Html-Media-Capture*/
const Value V18147_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18147_315 = {S33, 0, &V18147_315_57264, &P18147_313};	/*Property = Iframe*/
const Value V18147_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18147_316 = {S34, 0, &V18147_316_57267, &P18147_315};	/*Property = IndexedDB*/
const Value V18147_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18147_317 = {S35, 0, &V18147_317_57269, &P18147_316};	/*Property = Json*/
const Value V18147_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18147_15 = {S37, 0, &V18147_15_28, &P18147_317};	/*Property = LayoutEngine*/
const Value V18147_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18147_344 = {S38, 0, &V18147_344_57272, &P18147_15};	/*Property = Masking*/
const Value V18147_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18147_320 = {S39, 0, &V18147_320_57290, &P18147_344};	/*Property = PostMessage*/
const Value V18147_321_57294 = {S0, NULL};	/*Value = False*/
const Property P18147_321 = {S40, 0, &V18147_321_57294, &P18147_320};	/*Property = Progress*/
const Value V18147_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18147_322 = {S41, 0, &V18147_322_57296, &P18147_321};	/*Property = Prompts*/
const Value V18147_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18147_323 = {S42, 0, &V18147_323_57299, &P18147_322};	/*Property = Selector*/
const Value V18147_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18147_324 = {S43, 0, &V18147_324_57306, &P18147_323};	/*Property = Svg*/
const Value V18147_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18147_327 = {S44, 0, &V18147_327_57315, &P18147_324};	/*Property = TouchEvents*/
const Value V18147_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18147_328 = {S45, 0, &V18147_328_57318, &P18147_327};	/*Property = Track*/
const Value V18147_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18147_329 = {S46, 0, &V18147_329_57323, &P18147_328};	/*Property = Video*/
const Value V18147_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18147_330 = {S47, 0, &V18147_330_57326, &P18147_329};	/*Property = Viewport*/
const Value V18147_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18147_332 = {S48, 0, &V18147_332_57339, &P18147_330};	/*Property = WebWorkers*/
const Value V18147_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18147_333 = {S49, 0, &V18147_333_57342, &P18147_332};	/*Property = Xhr2*/
const Profile P18147 = {"18147", &P18147_333};	/*Profile ID - 18147*/
const Value V16135_1_1 = {S2, NULL};	/*Value = True*/
const Property P16135_1 = {S50, 0, &V16135_1_1, NULL};	/*Property = IsMobile*/
const Value V16135_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P16135_5 = {S52, 0, &V16135_5_1269, &P16135_1};	/*Property = ScreenPixelsHeight*/
const Value V16135_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16135_4 = {S54, 0, &V16135_4_1229, &P16135_5};	/*Property = ScreenPixelsWidth*/
const Profile P16135 = {"16135", &P16135_4};	/*Profile ID - 16135*/
const Value V22380_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22380_279 = {S1, 0, &V22380_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22380_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22380_282 = {S3, 0, &V22380_282_57153, &P22380_279};	/*Property = BlobBuilder*/
const Value V22380_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22380_350 = {S4, 0, &V22380_350_57444, &P22380_282};	/*Property = Canvas*/
const Value V22380_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22380_283 = {S5, 0, &V22380_283_57159, &P22380_350};	/*Property = CssBackground*/
const Value V22380_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22380_284 = {S6, 0, &V22380_284_57163, &P22380_283};	/*Property = CssBorderImage*/
const Value V22380_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22380_285 = {S7, 0, &V22380_285_57166, &P22380_284};	/*Property = CssCanvas*/
const Value V22380_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22380_286 = {S8, 0, &V22380_286_57168, &P22380_285};	/*Property = CssColor*/
const Value V22380_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22380_237 = {S9, 0, &V22380_237_55786, &P22380_286};	/*Property = CssColumn*/
const Value V22380_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22380_287 = {S10, 0, &V22380_287_57172, &P22380_237};	/*Property = CssFlexbox*/
const Value V22380_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22380_288 = {S11, 0, &V22380_288_57174, &P22380_287};	/*Property = CssFont*/
const Value V22380_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22380_289 = {S12, 0, &V22380_289_57759, &P22380_288};	/*Property = CssImages*/
const Value V22380_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22380_291 = {S13, 0, &V22380_291_57180, &P22380_289};	/*Property = CssMediaQueries*/
const Value V22380_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22380_292 = {S14, 0, &V22380_292_57183, &P22380_291};	/*Property = CssMinMax*/
const Value V22380_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22380_293 = {S15, 0, &V22380_293_57187, &P22380_292};	/*Property = CssOverflow*/
const Value V22380_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22380_294 = {S16, 0, &V22380_294_57190, &P22380_293};	/*Property = CssPosition*/
const Value V22380_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22380_295 = {S17, 0, &V22380_295_57192, &P22380_294};	/*Property = CssText*/
const Value V22380_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22380_297 = {S18, 0, &V22380_297_57195, &P22380_295};	/*Property = CssTransforms*/
const Value V22380_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22380_241 = {S19, 0, &V22380_241_55799, &P22380_297};	/*Property = CssTransitions*/
const Value V22380_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22380_298 = {S20, 0, &V22380_298_57206, &P22380_241};	/*Property = CssUI*/
const Value V22380_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22380_300 = {S21, 0, &V22380_300_57216, &P22380_298};	/*Property = DataSet*/
const Value V22380_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22380_301 = {S22, 0, &V22380_301_57218, &P22380_300};	/*Property = DataUrl*/
const Value V22380_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22380_302 = {S23, 0, &V22380_302_57221, &P22380_301};	/*Property = DeviceOrientation*/
const Value V22380_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22380_304 = {S24, 0, &V22380_304_57227, &P22380_302};	/*Property = FileReader*/
const Value V22380_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22380_305 = {S25, 0, &V22380_305_57231, &P22380_304};	/*Property = FileSaver*/
const Value V22380_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22380_306 = {S26, 0, &V22380_306_57237, &P22380_305};	/*Property = FileWriter*/
const Value V22380_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22380_307 = {S27, 0, &V22380_307_57239, &P22380_306};	/*Property = FormData*/
const Value V22380_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22380_309 = {S28, 0, &V22380_309_57246, &P22380_307};	/*Property = Fullscreen*/
const Value V22380_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22380_310 = {S29, 0, &V22380_310_57248, &P22380_309};	/*Property = GeoLocation*/
const Value V22380_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22380_312 = {S30, 0, &V22380_312_57255, &P22380_310};	/*Property = History*/
const Value V22380_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22380_314 = {S31, 0, &V22380_314_57261, &P22380_312};	/*Property = Html5*/
const Value V22380_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22380_313 = {S32, 0, &V22380_313_57258, &P22380_314};	/*Property = Html-Media-Capture*/
const Value V22380_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22380_315 = {S33, 0, &V22380_315_57264, &P22380_313};	/*Property = Iframe*/
const Value V22380_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22380_316 = {S34, 0, &V22380_316_57267, &P22380_315};	/*Property = IndexedDB*/
const Value V22380_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22380_317 = {S35, 0, &V22380_317_57269, &P22380_316};	/*Property = Json*/
const Value V22380_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22380_15 = {S37, 0, &V22380_15_28, &P22380_317};	/*Property = LayoutEngine*/
const Value V22380_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22380_344 = {S38, 0, &V22380_344_57272, &P22380_15};	/*Property = Masking*/
const Value V22380_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22380_320 = {S39, 0, &V22380_320_57290, &P22380_344};	/*Property = PostMessage*/
const Value V22380_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22380_321 = {S40, 0, &V22380_321_57293, &P22380_320};	/*Property = Progress*/
const Value V22380_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22380_322 = {S41, 0, &V22380_322_57296, &P22380_321};	/*Property = Prompts*/
const Value V22380_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22380_323 = {S42, 0, &V22380_323_57299, &P22380_322};	/*Property = Selector*/
const Value V22380_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22380_324 = {S43, 0, &V22380_324_57306, &P22380_323};	/*Property = Svg*/
const Value V22380_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22380_327 = {S44, 0, &V22380_327_57315, &P22380_324};	/*Property = TouchEvents*/
const Value V22380_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22380_328 = {S45, 0, &V22380_328_57318, &P22380_327};	/*Property = Track*/
const Value V22380_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22380_329 = {S46, 0, &V22380_329_57323, &P22380_328};	/*Property = Video*/
const Value V22380_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22380_330 = {S47, 0, &V22380_330_57326, &P22380_329};	/*Property = Viewport*/
const Value V22380_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22380_332 = {S48, 0, &V22380_332_57339, &P22380_330};	/*Property = WebWorkers*/
const Value V22380_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22380_333 = {S49, 0, &V22380_333_57342, &P22380_332};	/*Property = Xhr2*/
const Profile P22380 = {"22380", &P22380_333};	/*Profile ID - 22380*/
const Value V16088_1_1 = {S2, NULL};	/*Value = True*/
const Property P16088_1 = {S50, 0, &V16088_1_1, NULL};	/*Property = IsMobile*/
const Value V16088_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P16088_5 = {S52, 0, &V16088_5_1275, &P16088_1};	/*Property = ScreenPixelsHeight*/
const Value V16088_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16088_4 = {S54, 0, &V16088_4_1229, &P16088_5};	/*Property = ScreenPixelsWidth*/
const Profile P16088 = {"16088", &P16088_4};	/*Profile ID - 16088*/
const Profile P17815 = {"17815", NULL};	/*Profile ID - 17815*/
const Profile P17293 = {"17293", NULL};	/*Profile ID - 17293*/
const Value V16140_1_1 = {S2, NULL};	/*Value = True*/
const Property P16140_1 = {S50, 0, &V16140_1_1, NULL};	/*Property = IsMobile*/
const Value V16140_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16140_5 = {S52, 0, &V16140_5_1277, &P16140_1};	/*Property = ScreenPixelsHeight*/
const Value V16140_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16140_4 = {S54, 0, &V16140_4_1279, &P16140_5};	/*Property = ScreenPixelsWidth*/
const Profile P16140 = {"16140", &P16140_4};	/*Profile ID - 16140*/
const Value V22491_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22491_279 = {S1, 0, &V22491_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22491_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22491_282 = {S3, 0, &V22491_282_57154, &P22491_279};	/*Property = BlobBuilder*/
const Value V22491_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22491_350 = {S4, 0, &V22491_350_57444, &P22491_282};	/*Property = Canvas*/
const Value V22491_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22491_283 = {S5, 0, &V22491_283_57159, &P22491_350};	/*Property = CssBackground*/
const Value V22491_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22491_284 = {S6, 0, &V22491_284_57163, &P22491_283};	/*Property = CssBorderImage*/
const Value V22491_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22491_285 = {S7, 0, &V22491_285_57166, &P22491_284};	/*Property = CssCanvas*/
const Value V22491_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22491_286 = {S8, 0, &V22491_286_57168, &P22491_285};	/*Property = CssColor*/
const Value V22491_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22491_237 = {S9, 0, &V22491_237_55786, &P22491_286};	/*Property = CssColumn*/
const Value V22491_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22491_287 = {S10, 0, &V22491_287_57172, &P22491_237};	/*Property = CssFlexbox*/
const Value V22491_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22491_288 = {S11, 0, &V22491_288_57174, &P22491_287};	/*Property = CssFont*/
const Value V22491_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22491_289 = {S12, 0, &V22491_289_57759, &P22491_288};	/*Property = CssImages*/
const Value V22491_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22491_291 = {S13, 0, &V22491_291_57181, &P22491_289};	/*Property = CssMediaQueries*/
const Value V22491_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22491_292 = {S14, 0, &V22491_292_57183, &P22491_291};	/*Property = CssMinMax*/
const Value V22491_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22491_293 = {S15, 0, &V22491_293_57187, &P22491_292};	/*Property = CssOverflow*/
const Value V22491_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22491_294 = {S16, 0, &V22491_294_57189, &P22491_293};	/*Property = CssPosition*/
const Value V22491_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22491_295 = {S17, 0, &V22491_295_57192, &P22491_294};	/*Property = CssText*/
const Value V22491_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22491_297 = {S18, 0, &V22491_297_57195, &P22491_295};	/*Property = CssTransforms*/
const Value V22491_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22491_241 = {S19, 0, &V22491_241_55799, &P22491_297};	/*Property = CssTransitions*/
const Value V22491_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22491_298 = {S20, 0, &V22491_298_57206, &P22491_241};	/*Property = CssUI*/
const Value V22491_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22491_300 = {S21, 0, &V22491_300_57216, &P22491_298};	/*Property = DataSet*/
const Value V22491_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22491_301 = {S22, 0, &V22491_301_57218, &P22491_300};	/*Property = DataUrl*/
const Value V22491_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22491_302 = {S23, 0, &V22491_302_57222, &P22491_301};	/*Property = DeviceOrientation*/
const Value V22491_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22491_304 = {S24, 0, &V22491_304_57228, &P22491_302};	/*Property = FileReader*/
const Value V22491_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22491_305 = {S25, 0, &V22491_305_57231, &P22491_304};	/*Property = FileSaver*/
const Value V22491_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22491_306 = {S26, 0, &V22491_306_57237, &P22491_305};	/*Property = FileWriter*/
const Value V22491_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22491_307 = {S27, 0, &V22491_307_57240, &P22491_306};	/*Property = FormData*/
const Value V22491_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22491_309 = {S28, 0, &V22491_309_57246, &P22491_307};	/*Property = Fullscreen*/
const Value V22491_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22491_310 = {S29, 0, &V22491_310_57248, &P22491_309};	/*Property = GeoLocation*/
const Value V22491_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22491_312 = {S30, 0, &V22491_312_57255, &P22491_310};	/*Property = History*/
const Value V22491_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22491_314 = {S31, 0, &V22491_314_57261, &P22491_312};	/*Property = Html5*/
const Value V22491_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22491_313 = {S32, 0, &V22491_313_57258, &P22491_314};	/*Property = Html-Media-Capture*/
const Value V22491_315_57263 = {S2, NULL};	/*Value = True*/
const Property P22491_315 = {S33, 0, &V22491_315_57263, &P22491_313};	/*Property = Iframe*/
const Value V22491_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22491_316 = {S34, 0, &V22491_316_57267, &P22491_315};	/*Property = IndexedDB*/
const Value V22491_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22491_317 = {S35, 0, &V22491_317_57269, &P22491_316};	/*Property = Json*/
const Value V22491_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22491_15 = {S37, 0, &V22491_15_28, &P22491_317};	/*Property = LayoutEngine*/
const Value V22491_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22491_344 = {S38, 0, &V22491_344_57272, &P22491_15};	/*Property = Masking*/
const Value V22491_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22491_320 = {S39, 0, &V22491_320_57290, &P22491_344};	/*Property = PostMessage*/
const Value V22491_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22491_321 = {S40, 0, &V22491_321_57293, &P22491_320};	/*Property = Progress*/
const Value V22491_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22491_322 = {S41, 0, &V22491_322_57296, &P22491_321};	/*Property = Prompts*/
const Value V22491_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22491_323 = {S42, 0, &V22491_323_57299, &P22491_322};	/*Property = Selector*/
const Value V22491_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22491_324 = {S43, 0, &V22491_324_57305, &P22491_323};	/*Property = Svg*/
const Value V22491_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22491_327 = {S44, 0, &V22491_327_57315, &P22491_324};	/*Property = TouchEvents*/
const Value V22491_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22491_328 = {S45, 0, &V22491_328_57318, &P22491_327};	/*Property = Track*/
const Value V22491_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22491_329 = {S46, 0, &V22491_329_57323, &P22491_328};	/*Property = Video*/
const Value V22491_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22491_330 = {S47, 0, &V22491_330_57326, &P22491_329};	/*Property = Viewport*/
const Value V22491_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22491_332 = {S48, 0, &V22491_332_57339, &P22491_330};	/*Property = WebWorkers*/
const Value V22491_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22491_333 = {S49, 0, &V22491_333_57342, &P22491_332};	/*Property = Xhr2*/
const Profile P22491 = {"22491", &P22491_333};	/*Profile ID - 22491*/
const Value V17731_1_1 = {S2, NULL};	/*Value = True*/
const Property P17731_1 = {S50, 0, &V17731_1_1, NULL};	/*Property = IsMobile*/
const Value V17731_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17731_5 = {S52, 0, &V17731_5_1269, &P17731_1};	/*Property = ScreenPixelsHeight*/
const Value V17731_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17731_4 = {S54, 0, &V17731_4_1229, &P17731_5};	/*Property = ScreenPixelsWidth*/
const Profile P17731 = {"17731", &P17731_4};	/*Profile ID - 17731*/
const Value V17703_1_1 = {S2, NULL};	/*Value = True*/
const Property P17703_1 = {S50, 0, &V17703_1_1, NULL};	/*Property = IsMobile*/
const Value V17703_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P17703_5 = {S52, 0, &V17703_5_1403, &P17703_1};	/*Property = ScreenPixelsHeight*/
const Value V17703_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17703_4 = {S54, 0, &V17703_4_1229, &P17703_5};	/*Property = ScreenPixelsWidth*/
const Profile P17703 = {"17703", &P17703_4};	/*Profile ID - 17703*/
const Value V17635_1_1 = {S2, NULL};	/*Value = True*/
const Property P17635_1 = {S50, 0, &V17635_1_1, NULL};	/*Property = IsMobile*/
const Value V17635_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17635_5 = {S52, 0, &V17635_5_1277, &P17635_1};	/*Property = ScreenPixelsHeight*/
const Value V17635_4_37610 = {S69, NULL};	/*Value = 854*/
const Property P17635_4 = {S54, 0, &V17635_4_37610, &P17635_5};	/*Property = ScreenPixelsWidth*/
const Profile P17635 = {"17635", &P17635_4};	/*Profile ID - 17635*/
const Value V18059_1_1 = {S2, NULL};	/*Value = True*/
const Property P18059_1 = {S50, 0, &V18059_1_1, NULL};	/*Property = IsMobile*/
const Value V18059_5_1294 = {S62, NULL};	/*Value = 128*/
const Property P18059_5 = {S52, 0, &V18059_5_1294, &P18059_1};	/*Property = ScreenPixelsHeight*/
const Value V18059_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P18059_4 = {S54, 0, &V18059_4_1296, &P18059_5};	/*Property = ScreenPixelsWidth*/
const Profile P18059 = {"18059", &P18059_4};	/*Profile ID - 18059*/
const Value V17738_1_1 = {S2, NULL};	/*Value = True*/
const Property P17738_1 = {S50, 0, &V17738_1_1, NULL};	/*Property = IsMobile*/
const Value V17738_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17738_5 = {S52, 0, &V17738_5_1269, &P17738_1};	/*Property = ScreenPixelsHeight*/
const Value V17738_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17738_4 = {S54, 0, &V17738_4_1229, &P17738_5};	/*Property = ScreenPixelsWidth*/
const Profile P17738 = {"17738", &P17738_4};	/*Profile ID - 17738*/
const Value V17743_1_1 = {S2, NULL};	/*Value = True*/
const Property P17743_1 = {S50, 0, &V17743_1_1, NULL};	/*Property = IsMobile*/
const Value V17743_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17743_5 = {S52, 0, &V17743_5_1269, &P17743_1};	/*Property = ScreenPixelsHeight*/
const Value V17743_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17743_4 = {S54, 0, &V17743_4_1229, &P17743_5};	/*Property = ScreenPixelsWidth*/
const Profile P17743 = {"17743", &P17743_4};	/*Profile ID - 17743*/
const Value V22391_1_1 = {S2, NULL};	/*Value = True*/
const Property P22391_1 = {S50, 0, &V22391_1_1, NULL};	/*Property = IsMobile*/
const Value V22391_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22391_5 = {S52, 0, &V22391_5_37569, &P22391_1};	/*Property = ScreenPixelsHeight*/
const Value V22391_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22391_4 = {S54, 0, &V22391_4_5645, &P22391_5};	/*Property = ScreenPixelsWidth*/
const Profile P22391 = {"22391", &P22391_4};	/*Profile ID - 22391*/
const Value V17745_1_1 = {S2, NULL};	/*Value = True*/
const Property P17745_1 = {S50, 0, &V17745_1_1, NULL};	/*Property = IsMobile*/
const Value V17745_5_37540 = {S105, NULL};	/*Value = 427*/
const Property P17745_5 = {S52, 0, &V17745_5_37540, &P17745_1};	/*Property = ScreenPixelsHeight*/
const Value V17745_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17745_4 = {S54, 0, &V17745_4_1224, &P17745_5};	/*Property = ScreenPixelsWidth*/
const Profile P17745 = {"17745", &P17745_4};	/*Profile ID - 17745*/
const Value V16633_1_1 = {S2, NULL};	/*Value = True*/
const Property P16633_1 = {S50, 0, &V16633_1_1, NULL};	/*Property = IsMobile*/
const Value V16633_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16633_5 = {S52, 0, &V16633_5_1222, &P16633_1};	/*Property = ScreenPixelsHeight*/
const Value V16633_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16633_4 = {S54, 0, &V16633_4_1224, &P16633_5};	/*Property = ScreenPixelsWidth*/
const Profile P16633 = {"16633", &P16633_4};	/*Profile ID - 16633*/
const Value V22127_1_1 = {S2, NULL};	/*Value = True*/
const Property P22127_1 = {S50, 0, &V22127_1_1, NULL};	/*Property = IsMobile*/
const Value V22127_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22127_5 = {S52, 0, &V22127_5_1227, &P22127_1};	/*Property = ScreenPixelsHeight*/
const Value V22127_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22127_4 = {S54, 0, &V22127_4_1229, &P22127_5};	/*Property = ScreenPixelsWidth*/
const Profile P22127 = {"22127", &P22127_4};	/*Profile ID - 22127*/
const Value V16874_1_1 = {S2, NULL};	/*Value = True*/
const Property P16874_1 = {S50, 0, &V16874_1_1, NULL};	/*Property = IsMobile*/
const Value V16874_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P16874_5 = {S52, 0, &V16874_5_1380, &P16874_1};	/*Property = ScreenPixelsHeight*/
const Value V16874_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16874_4 = {S54, 0, &V16874_4_1279, &P16874_5};	/*Property = ScreenPixelsWidth*/
const Profile P16874 = {"16874", &P16874_4};	/*Profile ID - 16874*/
const Value V21361_1_1 = {S2, NULL};	/*Value = True*/
const Property P21361_1 = {S50, 0, &V21361_1_1, NULL};	/*Property = IsMobile*/
const Value V21361_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P21361_5 = {S52, 0, &V21361_5_1304, &P21361_1};	/*Property = ScreenPixelsHeight*/
const Value V21361_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21361_4 = {S54, 0, &V21361_4_1224, &P21361_5};	/*Property = ScreenPixelsWidth*/
const Profile P21361 = {"21361", &P21361_4};	/*Profile ID - 21361*/
const Value V16819_1_1 = {S2, NULL};	/*Value = True*/
const Property P16819_1 = {S50, 0, &V16819_1_1, NULL};	/*Property = IsMobile*/
const Value V16819_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16819_5 = {S52, 0, &V16819_5_1233, &P16819_1};	/*Property = ScreenPixelsHeight*/
const Value V16819_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16819_4 = {S54, 0, &V16819_4_1235, &P16819_5};	/*Property = ScreenPixelsWidth*/
const Profile P16819 = {"16819", &P16819_4};	/*Profile ID - 16819*/
const Value V20906_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20906_279 = {S1, 0, &V20906_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20906_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20906_282 = {S3, 0, &V20906_282_57154, &P20906_279};	/*Property = BlobBuilder*/
const Value V20906_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20906_350 = {S4, 0, &V20906_350_57444, &P20906_282};	/*Property = Canvas*/
const Value V20906_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20906_283 = {S5, 0, &V20906_283_57159, &P20906_350};	/*Property = CssBackground*/
const Value V20906_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20906_284 = {S6, 0, &V20906_284_57163, &P20906_283};	/*Property = CssBorderImage*/
const Value V20906_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20906_285 = {S7, 0, &V20906_285_57166, &P20906_284};	/*Property = CssCanvas*/
const Value V20906_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20906_286 = {S8, 0, &V20906_286_57168, &P20906_285};	/*Property = CssColor*/
const Value V20906_237_55785 = {S0, NULL};	/*Value = False*/
const Property P20906_237 = {S9, 0, &V20906_237_55785, &P20906_286};	/*Property = CssColumn*/
const Value V20906_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20906_287 = {S10, 0, &V20906_287_57172, &P20906_237};	/*Property = CssFlexbox*/
const Value V20906_288_57175 = {S0, NULL};	/*Value = False*/
const Property P20906_288 = {S11, 0, &V20906_288_57175, &P20906_287};	/*Property = CssFont*/
const Value V20906_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20906_289 = {S12, 0, &V20906_289_57759, &P20906_288};	/*Property = CssImages*/
const Value V20906_291_57181 = {S0, NULL};	/*Value = False*/
const Property P20906_291 = {S13, 0, &V20906_291_57181, &P20906_289};	/*Property = CssMediaQueries*/
const Value V20906_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20906_292 = {S14, 0, &V20906_292_57183, &P20906_291};	/*Property = CssMinMax*/
const Value V20906_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20906_293 = {S15, 0, &V20906_293_57187, &P20906_292};	/*Property = CssOverflow*/
const Value V20906_294_57190 = {S0, NULL};	/*Value = False*/
const Property P20906_294 = {S16, 0, &V20906_294_57190, &P20906_293};	/*Property = CssPosition*/
const Value V20906_295_57192 = {S2, NULL};	/*Value = True*/
const Property P20906_295 = {S17, 0, &V20906_295_57192, &P20906_294};	/*Property = CssText*/
const Value V20906_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20906_297 = {S18, 0, &V20906_297_57195, &P20906_295};	/*Property = CssTransforms*/
const Value V20906_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20906_241 = {S19, 0, &V20906_241_55799, &P20906_297};	/*Property = CssTransitions*/
const Value V20906_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20906_298 = {S20, 0, &V20906_298_57206, &P20906_241};	/*Property = CssUI*/
const Value V20906_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20906_300 = {S21, 0, &V20906_300_57216, &P20906_298};	/*Property = DataSet*/
const Value V20906_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20906_301 = {S22, 0, &V20906_301_57218, &P20906_300};	/*Property = DataUrl*/
const Value V20906_302_57222 = {S0, NULL};	/*Value = False*/
const Property P20906_302 = {S23, 0, &V20906_302_57222, &P20906_301};	/*Property = DeviceOrientation*/
const Value V20906_304_57228 = {S0, NULL};	/*Value = False*/
const Property P20906_304 = {S24, 0, &V20906_304_57228, &P20906_302};	/*Property = FileReader*/
const Value V20906_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20906_305 = {S25, 0, &V20906_305_57231, &P20906_304};	/*Property = FileSaver*/
const Value V20906_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20906_306 = {S26, 0, &V20906_306_57237, &P20906_305};	/*Property = FileWriter*/
const Value V20906_307_57240 = {S0, NULL};	/*Value = False*/
const Property P20906_307 = {S27, 0, &V20906_307_57240, &P20906_306};	/*Property = FormData*/
const Value V20906_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20906_309 = {S28, 0, &V20906_309_57246, &P20906_307};	/*Property = Fullscreen*/
const Value V20906_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20906_310 = {S29, 0, &V20906_310_57248, &P20906_309};	/*Property = GeoLocation*/
const Value V20906_312_57255 = {S0, NULL};	/*Value = False*/
const Property P20906_312 = {S30, 0, &V20906_312_57255, &P20906_310};	/*Property = History*/
const Value V20906_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20906_314 = {S31, 0, &V20906_314_57261, &P20906_312};	/*Property = Html5*/
const Value V20906_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20906_313 = {S32, 0, &V20906_313_57258, &P20906_314};	/*Property = Html-Media-Capture*/
const Value V20906_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20906_315 = {S33, 0, &V20906_315_57264, &P20906_313};	/*Property = Iframe*/
const Value V20906_316_57267 = {S0, NULL};	/*Value = False*/
const Property P20906_316 = {S34, 0, &V20906_316_57267, &P20906_315};	/*Property = IndexedDB*/
const Value V20906_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20906_317 = {S35, 0, &V20906_317_57269, &P20906_316};	/*Property = Json*/
const Value V20906_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P20906_15 = {S37, 0, &V20906_15_30, &P20906_317};	/*Property = LayoutEngine*/
const Value V20906_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20906_344 = {S38, 0, &V20906_344_57272, &P20906_15};	/*Property = Masking*/
const Value V20906_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20906_320 = {S39, 0, &V20906_320_57290, &P20906_344};	/*Property = PostMessage*/
const Value V20906_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20906_321 = {S40, 0, &V20906_321_57293, &P20906_320};	/*Property = Progress*/
const Value V20906_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20906_322 = {S41, 0, &V20906_322_57296, &P20906_321};	/*Property = Prompts*/
const Value V20906_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20906_323 = {S42, 0, &V20906_323_57299, &P20906_322};	/*Property = Selector*/
const Value V20906_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20906_324 = {S43, 0, &V20906_324_57306, &P20906_323};	/*Property = Svg*/
const Value V20906_327_57315 = {S0, NULL};	/*Value = False*/
const Property P20906_327 = {S44, 0, &V20906_327_57315, &P20906_324};	/*Property = TouchEvents*/
const Value V20906_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20906_328 = {S45, 0, &V20906_328_57318, &P20906_327};	/*Property = Track*/
const Value V20906_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20906_329 = {S46, 0, &V20906_329_57323, &P20906_328};	/*Property = Video*/
const Value V20906_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20906_330 = {S47, 0, &V20906_330_57326, &P20906_329};	/*Property = Viewport*/
const Value V20906_332_57339 = {S0, NULL};	/*Value = False*/
const Property P20906_332 = {S48, 0, &V20906_332_57339, &P20906_330};	/*Property = WebWorkers*/
const Value V20906_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20906_333 = {S49, 0, &V20906_333_57342, &P20906_332};	/*Property = Xhr2*/
const Profile P20906 = {"20906", &P20906_333};	/*Profile ID - 20906*/
const Value V16620_1_1 = {S2, NULL};	/*Value = True*/
const Property P16620_1 = {S50, 0, &V16620_1_1, NULL};	/*Property = IsMobile*/
const Value V16620_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P16620_5 = {S52, 0, &V16620_5_1304, &P16620_1};	/*Property = ScreenPixelsHeight*/
const Value V16620_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16620_4 = {S54, 0, &V16620_4_1224, &P16620_5};	/*Property = ScreenPixelsWidth*/
const Profile P16620 = {"16620", &P16620_4};	/*Profile ID - 16620*/
const Value V16869_1_1 = {S2, NULL};	/*Value = True*/
const Property P16869_1 = {S50, 0, &V16869_1_1, NULL};	/*Property = IsMobile*/
const Value V16869_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P16869_5 = {S52, 0, &V16869_5_1333, &P16869_1};	/*Property = ScreenPixelsHeight*/
const Value V16869_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P16869_4 = {S54, 0, &V16869_4_1335, &P16869_5};	/*Property = ScreenPixelsWidth*/
const Profile P16869 = {"16869", &P16869_4};	/*Profile ID - 16869*/
const Profile P24695 = {"24695", NULL};	/*Profile ID - 24695*/
const Value V22520_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_279 = {S1, 0, &V22520_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22520_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_282 = {S3, 0, &V22520_282_57155, &P22520_279};	/*Property = BlobBuilder*/
const Value V22520_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_350 = {S4, 0, &V22520_350_57446, &P22520_282};	/*Property = Canvas*/
const Value V22520_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_283 = {S5, 0, &V22520_283_57161, &P22520_350};	/*Property = CssBackground*/
const Value V22520_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_284 = {S6, 0, &V22520_284_57164, &P22520_283};	/*Property = CssBorderImage*/
const Value V22520_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_285 = {S7, 0, &V22520_285_57167, &P22520_284};	/*Property = CssCanvas*/
const Value V22520_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_286 = {S8, 0, &V22520_286_57170, &P22520_285};	/*Property = CssColor*/
const Value V22520_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_237 = {S9, 0, &V22520_237_55786, &P22520_286};	/*Property = CssColumn*/
const Value V22520_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_287 = {S10, 0, &V22520_287_57173, &P22520_237};	/*Property = CssFlexbox*/
const Value V22520_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_288 = {S11, 0, &V22520_288_57176, &P22520_287};	/*Property = CssFont*/
const Value V22520_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_289 = {S12, 0, &V22520_289_86899, &P22520_288};	/*Property = CssImages*/
const Value V22520_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_291 = {S13, 0, &V22520_291_57182, &P22520_289};	/*Property = CssMediaQueries*/
const Value V22520_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_292 = {S14, 0, &V22520_292_57185, &P22520_291};	/*Property = CssMinMax*/
const Value V22520_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_293 = {S15, 0, &V22520_293_57188, &P22520_292};	/*Property = CssOverflow*/
const Value V22520_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_294 = {S16, 0, &V22520_294_57191, &P22520_293};	/*Property = CssPosition*/
const Value V22520_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_295 = {S17, 0, &V22520_295_57194, &P22520_294};	/*Property = CssText*/
const Value V22520_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_297 = {S18, 0, &V22520_297_57197, &P22520_295};	/*Property = CssTransforms*/
const Value V22520_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_241 = {S19, 0, &V22520_241_55801, &P22520_297};	/*Property = CssTransitions*/
const Value V22520_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_298 = {S20, 0, &V22520_298_57208, &P22520_241};	/*Property = CssUI*/
const Value V22520_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_300 = {S21, 0, &V22520_300_57217, &P22520_298};	/*Property = DataSet*/
const Value V22520_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_301 = {S22, 0, &V22520_301_57220, &P22520_300};	/*Property = DataUrl*/
const Value V22520_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_302 = {S23, 0, &V22520_302_57223, &P22520_301};	/*Property = DeviceOrientation*/
const Value V22520_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_304 = {S24, 0, &V22520_304_57229, &P22520_302};	/*Property = FileReader*/
const Value V22520_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_305 = {S25, 0, &V22520_305_57232, &P22520_304};	/*Property = FileSaver*/
const Value V22520_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_306 = {S26, 0, &V22520_306_57238, &P22520_305};	/*Property = FileWriter*/
const Value V22520_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_307 = {S27, 0, &V22520_307_57241, &P22520_306};	/*Property = FormData*/
const Value V22520_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_309 = {S28, 0, &V22520_309_57247, &P22520_307};	/*Property = Fullscreen*/
const Value V22520_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_310 = {S29, 0, &V22520_310_57250, &P22520_309};	/*Property = GeoLocation*/
const Value V22520_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_312 = {S30, 0, &V22520_312_57256, &P22520_310};	/*Property = History*/
const Value V22520_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_314 = {S31, 0, &V22520_314_57262, &P22520_312};	/*Property = Html5*/
const Value V22520_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_313 = {S32, 0, &V22520_313_57259, &P22520_314};	/*Property = Html-Media-Capture*/
const Value V22520_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_315 = {S33, 0, &V22520_315_57265, &P22520_313};	/*Property = Iframe*/
const Value V22520_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_316 = {S34, 0, &V22520_316_57268, &P22520_315};	/*Property = IndexedDB*/
const Value V22520_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_317 = {S35, 0, &V22520_317_57271, &P22520_316};	/*Property = Json*/
const Value V22520_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22520_15 = {S37, 0, &V22520_15_28, &P22520_317};	/*Property = LayoutEngine*/
const Value V22520_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_344 = {S38, 0, &V22520_344_57274, &P22520_15};	/*Property = Masking*/
const Value V22520_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_320 = {S39, 0, &V22520_320_57292, &P22520_344};	/*Property = PostMessage*/
const Value V22520_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_321 = {S40, 0, &V22520_321_57295, &P22520_320};	/*Property = Progress*/
const Value V22520_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_322 = {S41, 0, &V22520_322_57298, &P22520_321};	/*Property = Prompts*/
const Value V22520_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_323 = {S42, 0, &V22520_323_57301, &P22520_322};	/*Property = Selector*/
const Value V22520_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_324 = {S43, 0, &V22520_324_57307, &P22520_323};	/*Property = Svg*/
const Value V22520_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_327 = {S44, 0, &V22520_327_57316, &P22520_324};	/*Property = TouchEvents*/
const Value V22520_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_328 = {S45, 0, &V22520_328_57319, &P22520_327};	/*Property = Track*/
const Value V22520_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_329 = {S46, 0, &V22520_329_57325, &P22520_328};	/*Property = Video*/
const Value V22520_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_330 = {S47, 0, &V22520_330_57328, &P22520_329};	/*Property = Viewport*/
const Value V22520_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_332 = {S48, 0, &V22520_332_57340, &P22520_330};	/*Property = WebWorkers*/
const Value V22520_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22520_333 = {S49, 0, &V22520_333_57343, &P22520_332};	/*Property = Xhr2*/
const Profile P22520 = {"22520", &P22520_333};	/*Profile ID - 22520*/
const Value V18047_1_1 = {S2, NULL};	/*Value = True*/
const Property P18047_1 = {S50, 0, &V18047_1_1, NULL};	/*Property = IsMobile*/
const Value V18047_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18047_5 = {S52, 0, &V18047_5_1277, &P18047_1};	/*Property = ScreenPixelsHeight*/
const Value V18047_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18047_4 = {S54, 0, &V18047_4_1279, &P18047_5};	/*Property = ScreenPixelsWidth*/
const Profile P18047 = {"18047", &P18047_4};	/*Profile ID - 18047*/
const Value V17637_1_1 = {S2, NULL};	/*Value = True*/
const Property P17637_1 = {S50, 0, &V17637_1_1, NULL};	/*Property = IsMobile*/
const Value V17637_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17637_5 = {S52, 0, &V17637_5_1269, &P17637_1};	/*Property = ScreenPixelsHeight*/
const Value V17637_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17637_4 = {S54, 0, &V17637_4_1229, &P17637_5};	/*Property = ScreenPixelsWidth*/
const Profile P17637 = {"17637", &P17637_4};	/*Profile ID - 17637*/
const Value V18077_1_1 = {S2, NULL};	/*Value = True*/
const Property P18077_1 = {S50, 0, &V18077_1_1, NULL};	/*Property = IsMobile*/
const Value V18077_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18077_5 = {S52, 0, &V18077_5_1222, &P18077_1};	/*Property = ScreenPixelsHeight*/
const Value V18077_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18077_4 = {S54, 0, &V18077_4_1224, &P18077_5};	/*Property = ScreenPixelsWidth*/
const Profile P18077 = {"18077", &P18077_4};	/*Profile ID - 18077*/
const Value V17465_1_1 = {S2, NULL};	/*Value = True*/
const Property P17465_1 = {S50, 0, &V17465_1_1, NULL};	/*Property = IsMobile*/
const Value V17465_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P17465_5 = {S52, 0, &V17465_5_37, &P17465_1};	/*Property = ScreenPixelsHeight*/
const Value V17465_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P17465_4 = {S54, 0, &V17465_4_36, &P17465_5};	/*Property = ScreenPixelsWidth*/
const Profile P17465 = {"17465", &P17465_4};	/*Profile ID - 17465*/
const Value V18024_1_1 = {S2, NULL};	/*Value = True*/
const Property P18024_1 = {S50, 0, &V18024_1_1, NULL};	/*Property = IsMobile*/
const Value V18024_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18024_5 = {S52, 0, &V18024_5_1222, &P18024_1};	/*Property = ScreenPixelsHeight*/
const Value V18024_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18024_4 = {S54, 0, &V18024_4_1224, &P18024_5};	/*Property = ScreenPixelsWidth*/
const Profile P18024 = {"18024", &P18024_4};	/*Profile ID - 18024*/
const Value V17566_1_1 = {S2, NULL};	/*Value = True*/
const Property P17566_1 = {S50, 0, &V17566_1_1, NULL};	/*Property = IsMobile*/
const Value V17566_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17566_5 = {S52, 0, &V17566_5_1269, &P17566_1};	/*Property = ScreenPixelsHeight*/
const Value V17566_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17566_4 = {S54, 0, &V17566_4_1229, &P17566_5};	/*Property = ScreenPixelsWidth*/
const Profile P17566 = {"17566", &P17566_4};	/*Profile ID - 17566*/
const Profile P22194 = {"22194", NULL};	/*Profile ID - 22194*/
const Value V23331_1_2 = {S0, NULL};	/*Value = False*/
const Property P23331_1 = {S50, 0, &V23331_1_2, NULL};	/*Property = IsMobile*/
const Value V23331_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23331_5 = {S52, 0, &V23331_5_37, &P23331_1};	/*Property = ScreenPixelsHeight*/
const Value V23331_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23331_4 = {S54, 0, &V23331_4_36, &P23331_5};	/*Property = ScreenPixelsWidth*/
const Profile P23331 = {"23331", &P23331_4};	/*Profile ID - 23331*/
const Value V24304_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_279 = {S1, 0, &V24304_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24304_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_282 = {S3, 0, &V24304_282_57155, &P24304_279};	/*Property = BlobBuilder*/
const Value V24304_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_350 = {S4, 0, &V24304_350_57446, &P24304_282};	/*Property = Canvas*/
const Value V24304_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_283 = {S5, 0, &V24304_283_57161, &P24304_350};	/*Property = CssBackground*/
const Value V24304_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_284 = {S6, 0, &V24304_284_57164, &P24304_283};	/*Property = CssBorderImage*/
const Value V24304_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_285 = {S7, 0, &V24304_285_57167, &P24304_284};	/*Property = CssCanvas*/
const Value V24304_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_286 = {S8, 0, &V24304_286_57170, &P24304_285};	/*Property = CssColor*/
const Value V24304_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_237 = {S9, 0, &V24304_237_55786, &P24304_286};	/*Property = CssColumn*/
const Value V24304_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_287 = {S10, 0, &V24304_287_57173, &P24304_237};	/*Property = CssFlexbox*/
const Value V24304_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_288 = {S11, 0, &V24304_288_57176, &P24304_287};	/*Property = CssFont*/
const Value V24304_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_289 = {S12, 0, &V24304_289_86899, &P24304_288};	/*Property = CssImages*/
const Value V24304_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_291 = {S13, 0, &V24304_291_57182, &P24304_289};	/*Property = CssMediaQueries*/
const Value V24304_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_292 = {S14, 0, &V24304_292_57185, &P24304_291};	/*Property = CssMinMax*/
const Value V24304_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_293 = {S15, 0, &V24304_293_57188, &P24304_292};	/*Property = CssOverflow*/
const Value V24304_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_294 = {S16, 0, &V24304_294_57191, &P24304_293};	/*Property = CssPosition*/
const Value V24304_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_295 = {S17, 0, &V24304_295_57194, &P24304_294};	/*Property = CssText*/
const Value V24304_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_297 = {S18, 0, &V24304_297_57197, &P24304_295};	/*Property = CssTransforms*/
const Value V24304_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_241 = {S19, 0, &V24304_241_55801, &P24304_297};	/*Property = CssTransitions*/
const Value V24304_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_298 = {S20, 0, &V24304_298_57208, &P24304_241};	/*Property = CssUI*/
const Value V24304_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_300 = {S21, 0, &V24304_300_57217, &P24304_298};	/*Property = DataSet*/
const Value V24304_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_301 = {S22, 0, &V24304_301_57220, &P24304_300};	/*Property = DataUrl*/
const Value V24304_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_302 = {S23, 0, &V24304_302_57223, &P24304_301};	/*Property = DeviceOrientation*/
const Value V24304_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_304 = {S24, 0, &V24304_304_57229, &P24304_302};	/*Property = FileReader*/
const Value V24304_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_305 = {S25, 0, &V24304_305_57232, &P24304_304};	/*Property = FileSaver*/
const Value V24304_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_306 = {S26, 0, &V24304_306_57238, &P24304_305};	/*Property = FileWriter*/
const Value V24304_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_307 = {S27, 0, &V24304_307_57241, &P24304_306};	/*Property = FormData*/
const Value V24304_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_309 = {S28, 0, &V24304_309_57247, &P24304_307};	/*Property = Fullscreen*/
const Value V24304_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_310 = {S29, 0, &V24304_310_57250, &P24304_309};	/*Property = GeoLocation*/
const Value V24304_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_312 = {S30, 0, &V24304_312_57256, &P24304_310};	/*Property = History*/
const Value V24304_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_314 = {S31, 0, &V24304_314_57262, &P24304_312};	/*Property = Html5*/
const Value V24304_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_313 = {S32, 0, &V24304_313_57259, &P24304_314};	/*Property = Html-Media-Capture*/
const Value V24304_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_315 = {S33, 0, &V24304_315_57265, &P24304_313};	/*Property = Iframe*/
const Value V24304_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_316 = {S34, 0, &V24304_316_57268, &P24304_315};	/*Property = IndexedDB*/
const Value V24304_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_317 = {S35, 0, &V24304_317_57271, &P24304_316};	/*Property = Json*/
const Value V24304_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24304_15 = {S37, 0, &V24304_15_28, &P24304_317};	/*Property = LayoutEngine*/
const Value V24304_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_344 = {S38, 0, &V24304_344_57274, &P24304_15};	/*Property = Masking*/
const Value V24304_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_320 = {S39, 0, &V24304_320_57292, &P24304_344};	/*Property = PostMessage*/
const Value V24304_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_321 = {S40, 0, &V24304_321_57295, &P24304_320};	/*Property = Progress*/
const Value V24304_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_322 = {S41, 0, &V24304_322_57298, &P24304_321};	/*Property = Prompts*/
const Value V24304_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_323 = {S42, 0, &V24304_323_57301, &P24304_322};	/*Property = Selector*/
const Value V24304_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_324 = {S43, 0, &V24304_324_57307, &P24304_323};	/*Property = Svg*/
const Value V24304_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_327 = {S44, 0, &V24304_327_57316, &P24304_324};	/*Property = TouchEvents*/
const Value V24304_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_328 = {S45, 0, &V24304_328_57319, &P24304_327};	/*Property = Track*/
const Value V24304_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_329 = {S46, 0, &V24304_329_57325, &P24304_328};	/*Property = Video*/
const Value V24304_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_330 = {S47, 0, &V24304_330_57328, &P24304_329};	/*Property = Viewport*/
const Value V24304_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_332 = {S48, 0, &V24304_332_57340, &P24304_330};	/*Property = WebWorkers*/
const Value V24304_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24304_333 = {S49, 0, &V24304_333_57343, &P24304_332};	/*Property = Xhr2*/
const Profile P24304 = {"24304", &P24304_333};	/*Profile ID - 24304*/
const Value V22643_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_279 = {S1, 0, &V22643_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22643_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_282 = {S3, 0, &V22643_282_57155, &P22643_279};	/*Property = BlobBuilder*/
const Value V22643_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_350 = {S4, 0, &V22643_350_57446, &P22643_282};	/*Property = Canvas*/
const Value V22643_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_283 = {S5, 0, &V22643_283_57161, &P22643_350};	/*Property = CssBackground*/
const Value V22643_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_284 = {S6, 0, &V22643_284_57164, &P22643_283};	/*Property = CssBorderImage*/
const Value V22643_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_285 = {S7, 0, &V22643_285_57167, &P22643_284};	/*Property = CssCanvas*/
const Value V22643_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_286 = {S8, 0, &V22643_286_57170, &P22643_285};	/*Property = CssColor*/
const Value V22643_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_237 = {S9, 0, &V22643_237_55786, &P22643_286};	/*Property = CssColumn*/
const Value V22643_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_287 = {S10, 0, &V22643_287_57173, &P22643_237};	/*Property = CssFlexbox*/
const Value V22643_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_288 = {S11, 0, &V22643_288_57176, &P22643_287};	/*Property = CssFont*/
const Value V22643_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_289 = {S12, 0, &V22643_289_86899, &P22643_288};	/*Property = CssImages*/
const Value V22643_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_291 = {S13, 0, &V22643_291_57182, &P22643_289};	/*Property = CssMediaQueries*/
const Value V22643_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_292 = {S14, 0, &V22643_292_57185, &P22643_291};	/*Property = CssMinMax*/
const Value V22643_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_293 = {S15, 0, &V22643_293_57188, &P22643_292};	/*Property = CssOverflow*/
const Value V22643_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_294 = {S16, 0, &V22643_294_57191, &P22643_293};	/*Property = CssPosition*/
const Value V22643_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_295 = {S17, 0, &V22643_295_57194, &P22643_294};	/*Property = CssText*/
const Value V22643_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_297 = {S18, 0, &V22643_297_57197, &P22643_295};	/*Property = CssTransforms*/
const Value V22643_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_241 = {S19, 0, &V22643_241_55801, &P22643_297};	/*Property = CssTransitions*/
const Value V22643_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_298 = {S20, 0, &V22643_298_57208, &P22643_241};	/*Property = CssUI*/
const Value V22643_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_300 = {S21, 0, &V22643_300_57217, &P22643_298};	/*Property = DataSet*/
const Value V22643_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_301 = {S22, 0, &V22643_301_57220, &P22643_300};	/*Property = DataUrl*/
const Value V22643_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_302 = {S23, 0, &V22643_302_57223, &P22643_301};	/*Property = DeviceOrientation*/
const Value V22643_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_304 = {S24, 0, &V22643_304_57229, &P22643_302};	/*Property = FileReader*/
const Value V22643_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_305 = {S25, 0, &V22643_305_57232, &P22643_304};	/*Property = FileSaver*/
const Value V22643_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_306 = {S26, 0, &V22643_306_57238, &P22643_305};	/*Property = FileWriter*/
const Value V22643_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_307 = {S27, 0, &V22643_307_57241, &P22643_306};	/*Property = FormData*/
const Value V22643_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_309 = {S28, 0, &V22643_309_57247, &P22643_307};	/*Property = Fullscreen*/
const Value V22643_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_310 = {S29, 0, &V22643_310_57250, &P22643_309};	/*Property = GeoLocation*/
const Value V22643_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_312 = {S30, 0, &V22643_312_57256, &P22643_310};	/*Property = History*/
const Value V22643_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_314 = {S31, 0, &V22643_314_57262, &P22643_312};	/*Property = Html5*/
const Value V22643_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_313 = {S32, 0, &V22643_313_57259, &P22643_314};	/*Property = Html-Media-Capture*/
const Value V22643_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_315 = {S33, 0, &V22643_315_57265, &P22643_313};	/*Property = Iframe*/
const Value V22643_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_316 = {S34, 0, &V22643_316_57268, &P22643_315};	/*Property = IndexedDB*/
const Value V22643_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_317 = {S35, 0, &V22643_317_57271, &P22643_316};	/*Property = Json*/
const Value V22643_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P22643_15 = {S37, 0, &V22643_15_54167, &P22643_317};	/*Property = LayoutEngine*/
const Value V22643_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_344 = {S38, 0, &V22643_344_57274, &P22643_15};	/*Property = Masking*/
const Value V22643_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_320 = {S39, 0, &V22643_320_57292, &P22643_344};	/*Property = PostMessage*/
const Value V22643_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_321 = {S40, 0, &V22643_321_57295, &P22643_320};	/*Property = Progress*/
const Value V22643_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_322 = {S41, 0, &V22643_322_57298, &P22643_321};	/*Property = Prompts*/
const Value V22643_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_323 = {S42, 0, &V22643_323_57301, &P22643_322};	/*Property = Selector*/
const Value V22643_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_324 = {S43, 0, &V22643_324_57307, &P22643_323};	/*Property = Svg*/
const Value V22643_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_327 = {S44, 0, &V22643_327_57316, &P22643_324};	/*Property = TouchEvents*/
const Value V22643_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_328 = {S45, 0, &V22643_328_57319, &P22643_327};	/*Property = Track*/
const Value V22643_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_329 = {S46, 0, &V22643_329_57325, &P22643_328};	/*Property = Video*/
const Value V22643_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_330 = {S47, 0, &V22643_330_57328, &P22643_329};	/*Property = Viewport*/
const Value V22643_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_332 = {S48, 0, &V22643_332_57340, &P22643_330};	/*Property = WebWorkers*/
const Value V22643_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22643_333 = {S49, 0, &V22643_333_57343, &P22643_332};	/*Property = Xhr2*/
const Profile P22643 = {"22643", &P22643_333};	/*Profile ID - 22643*/
const Value V16968_1_1 = {S2, NULL};	/*Value = True*/
const Property P16968_1 = {S50, 0, &V16968_1_1, NULL};	/*Property = IsMobile*/
const Value V16968_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16968_5 = {S52, 0, &V16968_5_1227, &P16968_1};	/*Property = ScreenPixelsHeight*/
const Value V16968_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16968_4 = {S54, 0, &V16968_4_1229, &P16968_5};	/*Property = ScreenPixelsWidth*/
const Profile P16968 = {"16968", &P16968_4};	/*Profile ID - 16968*/
const Value V22381_1_1 = {S2, NULL};	/*Value = True*/
const Property P22381_1 = {S50, 0, &V22381_1_1, NULL};	/*Property = IsMobile*/
const Value V22381_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22381_5 = {S52, 0, &V22381_5_1832, &P22381_1};	/*Property = ScreenPixelsHeight*/
const Value V22381_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22381_4 = {S54, 0, &V22381_4_5645, &P22381_5};	/*Property = ScreenPixelsWidth*/
const Profile P22381 = {"22381", &P22381_4};	/*Profile ID - 22381*/
const Value V15897_1_1 = {S2, NULL};	/*Value = True*/
const Property P15897_1 = {S50, 0, &V15897_1_1, NULL};	/*Property = IsMobile*/
const Value V15897_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15897_5 = {S52, 0, &V15897_5_1277, &P15897_1};	/*Property = ScreenPixelsHeight*/
const Value V15897_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15897_4 = {S54, 0, &V15897_4_1279, &P15897_5};	/*Property = ScreenPixelsWidth*/
const Profile P15897 = {"15897", &P15897_4};	/*Profile ID - 15897*/
const Value V17691_1_1 = {S2, NULL};	/*Value = True*/
const Property P17691_1 = {S50, 0, &V17691_1_1, NULL};	/*Property = IsMobile*/
const Value V17691_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17691_5 = {S52, 0, &V17691_5_1227, &P17691_1};	/*Property = ScreenPixelsHeight*/
const Value V17691_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P17691_4 = {S54, 0, &V17691_4_1477, &P17691_5};	/*Property = ScreenPixelsWidth*/
const Profile P17691 = {"17691", &P17691_4};	/*Profile ID - 17691*/
const Value V16172_1_1 = {S2, NULL};	/*Value = True*/
const Property P16172_1 = {S50, 0, &V16172_1_1, NULL};	/*Property = IsMobile*/
const Value V16172_5_37540 = {S105, NULL};	/*Value = 427*/
const Property P16172_5 = {S52, 0, &V16172_5_37540, &P16172_1};	/*Property = ScreenPixelsHeight*/
const Value V16172_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16172_4 = {S54, 0, &V16172_4_1224, &P16172_5};	/*Property = ScreenPixelsWidth*/
const Profile P16172 = {"16172", &P16172_4};	/*Profile ID - 16172*/
const Value V21034_1_1 = {S2, NULL};	/*Value = True*/
const Property P21034_1 = {S50, 0, &V21034_1_1, NULL};	/*Property = IsMobile*/
const Value V21034_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21034_5 = {S52, 0, &V21034_5_1403, &P21034_1};	/*Property = ScreenPixelsHeight*/
const Value V21034_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21034_4 = {S54, 0, &V21034_4_1404, &P21034_5};	/*Property = ScreenPixelsWidth*/
const Profile P21034 = {"21034", &P21034_4};	/*Profile ID - 21034*/
const Value V17934_1_1 = {S2, NULL};	/*Value = True*/
const Property P17934_1 = {S50, 0, &V17934_1_1, NULL};	/*Property = IsMobile*/
const Value V17934_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P17934_5 = {S52, 0, &V17934_5_37, &P17934_1};	/*Property = ScreenPixelsHeight*/
const Value V17934_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P17934_4 = {S54, 0, &V17934_4_36, &P17934_5};	/*Property = ScreenPixelsWidth*/
const Profile P17934 = {"17934", &P17934_4};	/*Profile ID - 17934*/
const Value V24205_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24205_279 = {S1, 0, &V24205_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24205_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24205_282 = {S3, 0, &V24205_282_57154, &P24205_279};	/*Property = BlobBuilder*/
const Value V24205_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24205_350 = {S4, 0, &V24205_350_57444, &P24205_282};	/*Property = Canvas*/
const Value V24205_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24205_283 = {S5, 0, &V24205_283_57159, &P24205_350};	/*Property = CssBackground*/
const Value V24205_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24205_284 = {S6, 0, &V24205_284_57162, &P24205_283};	/*Property = CssBorderImage*/
const Value V24205_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24205_285 = {S7, 0, &V24205_285_57166, &P24205_284};	/*Property = CssCanvas*/
const Value V24205_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24205_286 = {S8, 0, &V24205_286_57168, &P24205_285};	/*Property = CssColor*/
const Value V24205_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24205_237 = {S9, 0, &V24205_237_55786, &P24205_286};	/*Property = CssColumn*/
const Value V24205_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24205_287 = {S10, 0, &V24205_287_57172, &P24205_237};	/*Property = CssFlexbox*/
const Value V24205_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24205_288 = {S11, 0, &V24205_288_57175, &P24205_287};	/*Property = CssFont*/
const Value V24205_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24205_289 = {S12, 0, &V24205_289_57759, &P24205_288};	/*Property = CssImages*/
const Value V24205_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24205_291 = {S13, 0, &V24205_291_57180, &P24205_289};	/*Property = CssMediaQueries*/
const Value V24205_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24205_292 = {S14, 0, &V24205_292_57183, &P24205_291};	/*Property = CssMinMax*/
const Value V24205_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24205_293 = {S15, 0, &V24205_293_57187, &P24205_292};	/*Property = CssOverflow*/
const Value V24205_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24205_294 = {S16, 0, &V24205_294_57189, &P24205_293};	/*Property = CssPosition*/
const Value V24205_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24205_295 = {S17, 0, &V24205_295_57192, &P24205_294};	/*Property = CssText*/
const Value V24205_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24205_297 = {S18, 0, &V24205_297_57195, &P24205_295};	/*Property = CssTransforms*/
const Value V24205_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24205_241 = {S19, 0, &V24205_241_55799, &P24205_297};	/*Property = CssTransitions*/
const Value V24205_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24205_298 = {S20, 0, &V24205_298_57206, &P24205_241};	/*Property = CssUI*/
const Value V24205_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24205_300 = {S21, 0, &V24205_300_57215, &P24205_298};	/*Property = DataSet*/
const Value V24205_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24205_301 = {S22, 0, &V24205_301_57218, &P24205_300};	/*Property = DataUrl*/
const Value V24205_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24205_302 = {S23, 0, &V24205_302_57221, &P24205_301};	/*Property = DeviceOrientation*/
const Value V24205_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24205_304 = {S24, 0, &V24205_304_57227, &P24205_302};	/*Property = FileReader*/
const Value V24205_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24205_305 = {S25, 0, &V24205_305_57231, &P24205_304};	/*Property = FileSaver*/
const Value V24205_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24205_306 = {S26, 0, &V24205_306_57237, &P24205_305};	/*Property = FileWriter*/
const Value V24205_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24205_307 = {S27, 0, &V24205_307_57239, &P24205_306};	/*Property = FormData*/
const Value V24205_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24205_309 = {S28, 0, &V24205_309_57246, &P24205_307};	/*Property = Fullscreen*/
const Value V24205_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24205_310 = {S29, 0, &V24205_310_57248, &P24205_309};	/*Property = GeoLocation*/
const Value V24205_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24205_312 = {S30, 0, &V24205_312_57254, &P24205_310};	/*Property = History*/
const Value V24205_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24205_314 = {S31, 0, &V24205_314_57260, &P24205_312};	/*Property = Html5*/
const Value V24205_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24205_313 = {S32, 0, &V24205_313_57257, &P24205_314};	/*Property = Html-Media-Capture*/
const Value V24205_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24205_315 = {S33, 0, &V24205_315_57264, &P24205_313};	/*Property = Iframe*/
const Value V24205_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24205_316 = {S34, 0, &V24205_316_57266, &P24205_315};	/*Property = IndexedDB*/
const Value V24205_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24205_317 = {S35, 0, &V24205_317_57269, &P24205_316};	/*Property = Json*/
const Value V24205_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24205_15 = {S37, 0, &V24205_15_54167, &P24205_317};	/*Property = LayoutEngine*/
const Value V24205_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24205_344 = {S38, 0, &V24205_344_57272, &P24205_15};	/*Property = Masking*/
const Value V24205_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24205_320 = {S39, 0, &V24205_320_57290, &P24205_344};	/*Property = PostMessage*/
const Value V24205_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24205_321 = {S40, 0, &V24205_321_57293, &P24205_320};	/*Property = Progress*/
const Value V24205_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24205_322 = {S41, 0, &V24205_322_57296, &P24205_321};	/*Property = Prompts*/
const Value V24205_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24205_323 = {S42, 0, &V24205_323_57299, &P24205_322};	/*Property = Selector*/
const Value V24205_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24205_324 = {S43, 0, &V24205_324_57306, &P24205_323};	/*Property = Svg*/
const Value V24205_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24205_327 = {S44, 0, &V24205_327_57314, &P24205_324};	/*Property = TouchEvents*/
const Value V24205_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24205_328 = {S45, 0, &V24205_328_57318, &P24205_327};	/*Property = Track*/
const Value V24205_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24205_329 = {S46, 0, &V24205_329_57323, &P24205_328};	/*Property = Video*/
const Value V24205_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24205_330 = {S47, 0, &V24205_330_57326, &P24205_329};	/*Property = Viewport*/
const Value V24205_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24205_332 = {S48, 0, &V24205_332_57339, &P24205_330};	/*Property = WebWorkers*/
const Value V24205_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24205_333 = {S49, 0, &V24205_333_57342, &P24205_332};	/*Property = Xhr2*/
const Profile P24205 = {"24205", &P24205_333};	/*Profile ID - 24205*/
const Value V22765_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22765_279 = {S1, 0, &V22765_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22765_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22765_282 = {S3, 0, &V22765_282_57154, &P22765_279};	/*Property = BlobBuilder*/
const Value V22765_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22765_350 = {S4, 0, &V22765_350_57444, &P22765_282};	/*Property = Canvas*/
const Value V22765_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22765_283 = {S5, 0, &V22765_283_57159, &P22765_350};	/*Property = CssBackground*/
const Value V22765_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22765_284 = {S6, 0, &V22765_284_57163, &P22765_283};	/*Property = CssBorderImage*/
const Value V22765_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22765_285 = {S7, 0, &V22765_285_57166, &P22765_284};	/*Property = CssCanvas*/
const Value V22765_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22765_286 = {S8, 0, &V22765_286_57168, &P22765_285};	/*Property = CssColor*/
const Value V22765_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22765_237 = {S9, 0, &V22765_237_55786, &P22765_286};	/*Property = CssColumn*/
const Value V22765_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22765_287 = {S10, 0, &V22765_287_57172, &P22765_237};	/*Property = CssFlexbox*/
const Value V22765_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22765_288 = {S11, 0, &V22765_288_57175, &P22765_287};	/*Property = CssFont*/
const Value V22765_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22765_289 = {S12, 0, &V22765_289_57759, &P22765_288};	/*Property = CssImages*/
const Value V22765_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22765_291 = {S13, 0, &V22765_291_57181, &P22765_289};	/*Property = CssMediaQueries*/
const Value V22765_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22765_292 = {S14, 0, &V22765_292_57183, &P22765_291};	/*Property = CssMinMax*/
const Value V22765_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22765_293 = {S15, 0, &V22765_293_57187, &P22765_292};	/*Property = CssOverflow*/
const Value V22765_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22765_294 = {S16, 0, &V22765_294_57190, &P22765_293};	/*Property = CssPosition*/
const Value V22765_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22765_295 = {S17, 0, &V22765_295_57193, &P22765_294};	/*Property = CssText*/
const Value V22765_297_57196 = {S0, NULL};	/*Value = False*/
const Property P22765_297 = {S18, 0, &V22765_297_57196, &P22765_295};	/*Property = CssTransforms*/
const Value V22765_241_55800 = {S0, NULL};	/*Value = False*/
const Property P22765_241 = {S19, 0, &V22765_241_55800, &P22765_297};	/*Property = CssTransitions*/
const Value V22765_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22765_298 = {S20, 0, &V22765_298_57206, &P22765_241};	/*Property = CssUI*/
const Value V22765_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22765_300 = {S21, 0, &V22765_300_57216, &P22765_298};	/*Property = DataSet*/
const Value V22765_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22765_301 = {S22, 0, &V22765_301_57218, &P22765_300};	/*Property = DataUrl*/
const Value V22765_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22765_302 = {S23, 0, &V22765_302_57222, &P22765_301};	/*Property = DeviceOrientation*/
const Value V22765_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22765_304 = {S24, 0, &V22765_304_57228, &P22765_302};	/*Property = FileReader*/
const Value V22765_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22765_305 = {S25, 0, &V22765_305_57231, &P22765_304};	/*Property = FileSaver*/
const Value V22765_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22765_306 = {S26, 0, &V22765_306_57237, &P22765_305};	/*Property = FileWriter*/
const Value V22765_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22765_307 = {S27, 0, &V22765_307_57240, &P22765_306};	/*Property = FormData*/
const Value V22765_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22765_309 = {S28, 0, &V22765_309_57246, &P22765_307};	/*Property = Fullscreen*/
const Value V22765_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22765_310 = {S29, 0, &V22765_310_57248, &P22765_309};	/*Property = GeoLocation*/
const Value V22765_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22765_312 = {S30, 0, &V22765_312_57255, &P22765_310};	/*Property = History*/
const Value V22765_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22765_314 = {S31, 0, &V22765_314_57261, &P22765_312};	/*Property = Html5*/
const Value V22765_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22765_313 = {S32, 0, &V22765_313_57258, &P22765_314};	/*Property = Html-Media-Capture*/
const Value V22765_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22765_315 = {S33, 0, &V22765_315_57264, &P22765_313};	/*Property = Iframe*/
const Value V22765_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22765_316 = {S34, 0, &V22765_316_57267, &P22765_315};	/*Property = IndexedDB*/
const Value V22765_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22765_317 = {S35, 0, &V22765_317_57269, &P22765_316};	/*Property = Json*/
const Value V22765_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P22765_15 = {S37, 0, &V22765_15_30, &P22765_317};	/*Property = LayoutEngine*/
const Value V22765_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22765_344 = {S38, 0, &V22765_344_57272, &P22765_15};	/*Property = Masking*/
const Value V22765_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22765_320 = {S39, 0, &V22765_320_57290, &P22765_344};	/*Property = PostMessage*/
const Value V22765_321_57294 = {S0, NULL};	/*Value = False*/
const Property P22765_321 = {S40, 0, &V22765_321_57294, &P22765_320};	/*Property = Progress*/
const Value V22765_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22765_322 = {S41, 0, &V22765_322_57296, &P22765_321};	/*Property = Prompts*/
const Value V22765_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22765_323 = {S42, 0, &V22765_323_57299, &P22765_322};	/*Property = Selector*/
const Value V22765_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22765_324 = {S43, 0, &V22765_324_57306, &P22765_323};	/*Property = Svg*/
const Value V22765_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22765_327 = {S44, 0, &V22765_327_57315, &P22765_324};	/*Property = TouchEvents*/
const Value V22765_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22765_328 = {S45, 0, &V22765_328_57318, &P22765_327};	/*Property = Track*/
const Value V22765_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22765_329 = {S46, 0, &V22765_329_57323, &P22765_328};	/*Property = Video*/
const Value V22765_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22765_330 = {S47, 0, &V22765_330_57326, &P22765_329};	/*Property = Viewport*/
const Value V22765_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22765_332 = {S48, 0, &V22765_332_57339, &P22765_330};	/*Property = WebWorkers*/
const Value V22765_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22765_333 = {S49, 0, &V22765_333_57342, &P22765_332};	/*Property = Xhr2*/
const Profile P22765 = {"22765", &P22765_333};	/*Profile ID - 22765*/
const Value V17965_1_1 = {S2, NULL};	/*Value = True*/
const Property P17965_1 = {S50, 0, &V17965_1_1, NULL};	/*Property = IsMobile*/
const Value V17965_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P17965_5 = {S52, 0, &V17965_5_1458, &P17965_1};	/*Property = ScreenPixelsHeight*/
const Value V17965_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P17965_4 = {S54, 0, &V17965_4_1460, &P17965_5};	/*Property = ScreenPixelsWidth*/
const Profile P17965 = {"17965", &P17965_4};	/*Profile ID - 17965*/
const Value V22453_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22453_279 = {S1, 0, &V22453_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22453_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22453_282 = {S3, 0, &V22453_282_57154, &P22453_279};	/*Property = BlobBuilder*/
const Value V22453_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22453_350 = {S4, 0, &V22453_350_57444, &P22453_282};	/*Property = Canvas*/
const Value V22453_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22453_283 = {S5, 0, &V22453_283_57159, &P22453_350};	/*Property = CssBackground*/
const Value V22453_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22453_284 = {S6, 0, &V22453_284_57163, &P22453_283};	/*Property = CssBorderImage*/
const Value V22453_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22453_285 = {S7, 0, &V22453_285_57166, &P22453_284};	/*Property = CssCanvas*/
const Value V22453_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22453_286 = {S8, 0, &V22453_286_57168, &P22453_285};	/*Property = CssColor*/
const Value V22453_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22453_237 = {S9, 0, &V22453_237_55784, &P22453_286};	/*Property = CssColumn*/
const Value V22453_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22453_287 = {S10, 0, &V22453_287_57172, &P22453_237};	/*Property = CssFlexbox*/
const Value V22453_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22453_288 = {S11, 0, &V22453_288_57174, &P22453_287};	/*Property = CssFont*/
const Value V22453_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22453_289 = {S12, 0, &V22453_289_57759, &P22453_288};	/*Property = CssImages*/
const Value V22453_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22453_291 = {S13, 0, &V22453_291_57180, &P22453_289};	/*Property = CssMediaQueries*/
const Value V22453_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22453_292 = {S14, 0, &V22453_292_57183, &P22453_291};	/*Property = CssMinMax*/
const Value V22453_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22453_293 = {S15, 0, &V22453_293_57187, &P22453_292};	/*Property = CssOverflow*/
const Value V22453_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22453_294 = {S16, 0, &V22453_294_57189, &P22453_293};	/*Property = CssPosition*/
const Value V22453_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22453_295 = {S17, 0, &V22453_295_57192, &P22453_294};	/*Property = CssText*/
const Value V22453_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22453_297 = {S18, 0, &V22453_297_57195, &P22453_295};	/*Property = CssTransforms*/
const Value V22453_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22453_241 = {S19, 0, &V22453_241_55799, &P22453_297};	/*Property = CssTransitions*/
const Value V22453_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22453_298 = {S20, 0, &V22453_298_57206, &P22453_241};	/*Property = CssUI*/
const Value V22453_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22453_300 = {S21, 0, &V22453_300_57216, &P22453_298};	/*Property = DataSet*/
const Value V22453_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22453_301 = {S22, 0, &V22453_301_57218, &P22453_300};	/*Property = DataUrl*/
const Value V22453_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22453_302 = {S23, 0, &V22453_302_57222, &P22453_301};	/*Property = DeviceOrientation*/
const Value V22453_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22453_304 = {S24, 0, &V22453_304_57227, &P22453_302};	/*Property = FileReader*/
const Value V22453_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22453_305 = {S25, 0, &V22453_305_57231, &P22453_304};	/*Property = FileSaver*/
const Value V22453_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22453_306 = {S26, 0, &V22453_306_57237, &P22453_305};	/*Property = FileWriter*/
const Value V22453_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22453_307 = {S27, 0, &V22453_307_57239, &P22453_306};	/*Property = FormData*/
const Value V22453_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22453_309 = {S28, 0, &V22453_309_57246, &P22453_307};	/*Property = Fullscreen*/
const Value V22453_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22453_310 = {S29, 0, &V22453_310_57248, &P22453_309};	/*Property = GeoLocation*/
const Value V22453_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22453_312 = {S30, 0, &V22453_312_57254, &P22453_310};	/*Property = History*/
const Value V22453_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22453_314 = {S31, 0, &V22453_314_57261, &P22453_312};	/*Property = Html5*/
const Value V22453_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22453_313 = {S32, 0, &V22453_313_57258, &P22453_314};	/*Property = Html-Media-Capture*/
const Value V22453_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22453_315 = {S33, 0, &V22453_315_57264, &P22453_313};	/*Property = Iframe*/
const Value V22453_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22453_316 = {S34, 0, &V22453_316_57266, &P22453_315};	/*Property = IndexedDB*/
const Value V22453_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22453_317 = {S35, 0, &V22453_317_57269, &P22453_316};	/*Property = Json*/
const Value V22453_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22453_15 = {S37, 0, &V22453_15_28, &P22453_317};	/*Property = LayoutEngine*/
const Value V22453_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22453_344 = {S38, 0, &V22453_344_57272, &P22453_15};	/*Property = Masking*/
const Value V22453_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22453_320 = {S39, 0, &V22453_320_57290, &P22453_344};	/*Property = PostMessage*/
const Value V22453_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22453_321 = {S40, 0, &V22453_321_57293, &P22453_320};	/*Property = Progress*/
const Value V22453_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22453_322 = {S41, 0, &V22453_322_57296, &P22453_321};	/*Property = Prompts*/
const Value V22453_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22453_323 = {S42, 0, &V22453_323_57299, &P22453_322};	/*Property = Selector*/
const Value V22453_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22453_324 = {S43, 0, &V22453_324_57306, &P22453_323};	/*Property = Svg*/
const Value V22453_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22453_327 = {S44, 0, &V22453_327_57315, &P22453_324};	/*Property = TouchEvents*/
const Value V22453_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22453_328 = {S45, 0, &V22453_328_57318, &P22453_327};	/*Property = Track*/
const Value V22453_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22453_329 = {S46, 0, &V22453_329_57323, &P22453_328};	/*Property = Video*/
const Value V22453_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22453_330 = {S47, 0, &V22453_330_57326, &P22453_329};	/*Property = Viewport*/
const Value V22453_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22453_332 = {S48, 0, &V22453_332_57339, &P22453_330};	/*Property = WebWorkers*/
const Value V22453_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22453_333 = {S49, 0, &V22453_333_57342, &P22453_332};	/*Property = Xhr2*/
const Profile P22453 = {"22453", &P22453_333};	/*Profile ID - 22453*/
const Value V17369_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_279 = {S1, 0, &V17369_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17369_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_282 = {S3, 0, &V17369_282_57155, &P17369_279};	/*Property = BlobBuilder*/
const Value V17369_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_350 = {S4, 0, &V17369_350_57446, &P17369_282};	/*Property = Canvas*/
const Value V17369_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_283 = {S5, 0, &V17369_283_57161, &P17369_350};	/*Property = CssBackground*/
const Value V17369_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_284 = {S6, 0, &V17369_284_57164, &P17369_283};	/*Property = CssBorderImage*/
const Value V17369_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_285 = {S7, 0, &V17369_285_57167, &P17369_284};	/*Property = CssCanvas*/
const Value V17369_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_286 = {S8, 0, &V17369_286_57170, &P17369_285};	/*Property = CssColor*/
const Value V17369_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_237 = {S9, 0, &V17369_237_55786, &P17369_286};	/*Property = CssColumn*/
const Value V17369_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_287 = {S10, 0, &V17369_287_57173, &P17369_237};	/*Property = CssFlexbox*/
const Value V17369_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_288 = {S11, 0, &V17369_288_57176, &P17369_287};	/*Property = CssFont*/
const Value V17369_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_289 = {S12, 0, &V17369_289_86899, &P17369_288};	/*Property = CssImages*/
const Value V17369_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_291 = {S13, 0, &V17369_291_57182, &P17369_289};	/*Property = CssMediaQueries*/
const Value V17369_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_292 = {S14, 0, &V17369_292_57185, &P17369_291};	/*Property = CssMinMax*/
const Value V17369_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_293 = {S15, 0, &V17369_293_57188, &P17369_292};	/*Property = CssOverflow*/
const Value V17369_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_294 = {S16, 0, &V17369_294_57191, &P17369_293};	/*Property = CssPosition*/
const Value V17369_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_295 = {S17, 0, &V17369_295_57194, &P17369_294};	/*Property = CssText*/
const Value V17369_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_297 = {S18, 0, &V17369_297_57197, &P17369_295};	/*Property = CssTransforms*/
const Value V17369_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_241 = {S19, 0, &V17369_241_55801, &P17369_297};	/*Property = CssTransitions*/
const Value V17369_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_298 = {S20, 0, &V17369_298_57208, &P17369_241};	/*Property = CssUI*/
const Value V17369_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_300 = {S21, 0, &V17369_300_57217, &P17369_298};	/*Property = DataSet*/
const Value V17369_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_301 = {S22, 0, &V17369_301_57220, &P17369_300};	/*Property = DataUrl*/
const Value V17369_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_302 = {S23, 0, &V17369_302_57223, &P17369_301};	/*Property = DeviceOrientation*/
const Value V17369_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_304 = {S24, 0, &V17369_304_57229, &P17369_302};	/*Property = FileReader*/
const Value V17369_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_305 = {S25, 0, &V17369_305_57232, &P17369_304};	/*Property = FileSaver*/
const Value V17369_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_306 = {S26, 0, &V17369_306_57238, &P17369_305};	/*Property = FileWriter*/
const Value V17369_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_307 = {S27, 0, &V17369_307_57241, &P17369_306};	/*Property = FormData*/
const Value V17369_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_309 = {S28, 0, &V17369_309_57247, &P17369_307};	/*Property = Fullscreen*/
const Value V17369_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_310 = {S29, 0, &V17369_310_57250, &P17369_309};	/*Property = GeoLocation*/
const Value V17369_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_312 = {S30, 0, &V17369_312_57256, &P17369_310};	/*Property = History*/
const Value V17369_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_314 = {S31, 0, &V17369_314_57262, &P17369_312};	/*Property = Html5*/
const Value V17369_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_313 = {S32, 0, &V17369_313_57259, &P17369_314};	/*Property = Html-Media-Capture*/
const Value V17369_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_315 = {S33, 0, &V17369_315_57265, &P17369_313};	/*Property = Iframe*/
const Value V17369_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_316 = {S34, 0, &V17369_316_57268, &P17369_315};	/*Property = IndexedDB*/
const Value V17369_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_317 = {S35, 0, &V17369_317_57271, &P17369_316};	/*Property = Json*/
const Value V17369_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P17369_15 = {S37, 0, &V17369_15_31, &P17369_317};	/*Property = LayoutEngine*/
const Value V17369_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_344 = {S38, 0, &V17369_344_57274, &P17369_15};	/*Property = Masking*/
const Value V17369_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_320 = {S39, 0, &V17369_320_57292, &P17369_344};	/*Property = PostMessage*/
const Value V17369_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_321 = {S40, 0, &V17369_321_57295, &P17369_320};	/*Property = Progress*/
const Value V17369_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_322 = {S41, 0, &V17369_322_57298, &P17369_321};	/*Property = Prompts*/
const Value V17369_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_323 = {S42, 0, &V17369_323_57301, &P17369_322};	/*Property = Selector*/
const Value V17369_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_324 = {S43, 0, &V17369_324_57307, &P17369_323};	/*Property = Svg*/
const Value V17369_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_327 = {S44, 0, &V17369_327_57316, &P17369_324};	/*Property = TouchEvents*/
const Value V17369_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_328 = {S45, 0, &V17369_328_57319, &P17369_327};	/*Property = Track*/
const Value V17369_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_329 = {S46, 0, &V17369_329_57325, &P17369_328};	/*Property = Video*/
const Value V17369_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_330 = {S47, 0, &V17369_330_57328, &P17369_329};	/*Property = Viewport*/
const Value V17369_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_332 = {S48, 0, &V17369_332_57340, &P17369_330};	/*Property = WebWorkers*/
const Value V17369_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17369_333 = {S49, 0, &V17369_333_57343, &P17369_332};	/*Property = Xhr2*/
const Profile P17369 = {"17369", &P17369_333};	/*Profile ID - 17369*/
const Value V18205_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_279 = {S1, 0, &V18205_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18205_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_282 = {S3, 0, &V18205_282_57155, &P18205_279};	/*Property = BlobBuilder*/
const Value V18205_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_350 = {S4, 0, &V18205_350_57446, &P18205_282};	/*Property = Canvas*/
const Value V18205_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_283 = {S5, 0, &V18205_283_57161, &P18205_350};	/*Property = CssBackground*/
const Value V18205_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_284 = {S6, 0, &V18205_284_57164, &P18205_283};	/*Property = CssBorderImage*/
const Value V18205_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_285 = {S7, 0, &V18205_285_57167, &P18205_284};	/*Property = CssCanvas*/
const Value V18205_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_286 = {S8, 0, &V18205_286_57170, &P18205_285};	/*Property = CssColor*/
const Value V18205_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_237 = {S9, 0, &V18205_237_55786, &P18205_286};	/*Property = CssColumn*/
const Value V18205_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_287 = {S10, 0, &V18205_287_57173, &P18205_237};	/*Property = CssFlexbox*/
const Value V18205_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_288 = {S11, 0, &V18205_288_57176, &P18205_287};	/*Property = CssFont*/
const Value V18205_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_289 = {S12, 0, &V18205_289_86899, &P18205_288};	/*Property = CssImages*/
const Value V18205_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_291 = {S13, 0, &V18205_291_57182, &P18205_289};	/*Property = CssMediaQueries*/
const Value V18205_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_292 = {S14, 0, &V18205_292_57185, &P18205_291};	/*Property = CssMinMax*/
const Value V18205_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_293 = {S15, 0, &V18205_293_57188, &P18205_292};	/*Property = CssOverflow*/
const Value V18205_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_294 = {S16, 0, &V18205_294_57191, &P18205_293};	/*Property = CssPosition*/
const Value V18205_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_295 = {S17, 0, &V18205_295_57194, &P18205_294};	/*Property = CssText*/
const Value V18205_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_297 = {S18, 0, &V18205_297_57197, &P18205_295};	/*Property = CssTransforms*/
const Value V18205_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_241 = {S19, 0, &V18205_241_55801, &P18205_297};	/*Property = CssTransitions*/
const Value V18205_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_298 = {S20, 0, &V18205_298_57208, &P18205_241};	/*Property = CssUI*/
const Value V18205_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_300 = {S21, 0, &V18205_300_57217, &P18205_298};	/*Property = DataSet*/
const Value V18205_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_301 = {S22, 0, &V18205_301_57220, &P18205_300};	/*Property = DataUrl*/
const Value V18205_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_302 = {S23, 0, &V18205_302_57223, &P18205_301};	/*Property = DeviceOrientation*/
const Value V18205_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_304 = {S24, 0, &V18205_304_57229, &P18205_302};	/*Property = FileReader*/
const Value V18205_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_305 = {S25, 0, &V18205_305_57232, &P18205_304};	/*Property = FileSaver*/
const Value V18205_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_306 = {S26, 0, &V18205_306_57238, &P18205_305};	/*Property = FileWriter*/
const Value V18205_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_307 = {S27, 0, &V18205_307_57241, &P18205_306};	/*Property = FormData*/
const Value V18205_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_309 = {S28, 0, &V18205_309_57247, &P18205_307};	/*Property = Fullscreen*/
const Value V18205_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_310 = {S29, 0, &V18205_310_57250, &P18205_309};	/*Property = GeoLocation*/
const Value V18205_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_312 = {S30, 0, &V18205_312_57256, &P18205_310};	/*Property = History*/
const Value V18205_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_314 = {S31, 0, &V18205_314_57262, &P18205_312};	/*Property = Html5*/
const Value V18205_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_313 = {S32, 0, &V18205_313_57259, &P18205_314};	/*Property = Html-Media-Capture*/
const Value V18205_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_315 = {S33, 0, &V18205_315_57265, &P18205_313};	/*Property = Iframe*/
const Value V18205_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_316 = {S34, 0, &V18205_316_57268, &P18205_315};	/*Property = IndexedDB*/
const Value V18205_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_317 = {S35, 0, &V18205_317_57271, &P18205_316};	/*Property = Json*/
const Value V18205_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18205_15 = {S37, 0, &V18205_15_28, &P18205_317};	/*Property = LayoutEngine*/
const Value V18205_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_344 = {S38, 0, &V18205_344_57274, &P18205_15};	/*Property = Masking*/
const Value V18205_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_320 = {S39, 0, &V18205_320_57292, &P18205_344};	/*Property = PostMessage*/
const Value V18205_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_321 = {S40, 0, &V18205_321_57295, &P18205_320};	/*Property = Progress*/
const Value V18205_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_322 = {S41, 0, &V18205_322_57298, &P18205_321};	/*Property = Prompts*/
const Value V18205_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_323 = {S42, 0, &V18205_323_57301, &P18205_322};	/*Property = Selector*/
const Value V18205_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_324 = {S43, 0, &V18205_324_57307, &P18205_323};	/*Property = Svg*/
const Value V18205_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_327 = {S44, 0, &V18205_327_57316, &P18205_324};	/*Property = TouchEvents*/
const Value V18205_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_328 = {S45, 0, &V18205_328_57319, &P18205_327};	/*Property = Track*/
const Value V18205_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_329 = {S46, 0, &V18205_329_57325, &P18205_328};	/*Property = Video*/
const Value V18205_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_330 = {S47, 0, &V18205_330_57328, &P18205_329};	/*Property = Viewport*/
const Value V18205_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_332 = {S48, 0, &V18205_332_57340, &P18205_330};	/*Property = WebWorkers*/
const Value V18205_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18205_333 = {S49, 0, &V18205_333_57343, &P18205_332};	/*Property = Xhr2*/
const Profile P18205 = {"18205", &P18205_333};	/*Profile ID - 18205*/
const Value V18166_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_279 = {S1, 0, &V18166_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18166_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_282 = {S3, 0, &V18166_282_57155, &P18166_279};	/*Property = BlobBuilder*/
const Value V18166_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_350 = {S4, 0, &V18166_350_57446, &P18166_282};	/*Property = Canvas*/
const Value V18166_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_283 = {S5, 0, &V18166_283_57161, &P18166_350};	/*Property = CssBackground*/
const Value V18166_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_284 = {S6, 0, &V18166_284_57164, &P18166_283};	/*Property = CssBorderImage*/
const Value V18166_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_285 = {S7, 0, &V18166_285_57167, &P18166_284};	/*Property = CssCanvas*/
const Value V18166_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_286 = {S8, 0, &V18166_286_57170, &P18166_285};	/*Property = CssColor*/
const Value V18166_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_237 = {S9, 0, &V18166_237_55786, &P18166_286};	/*Property = CssColumn*/
const Value V18166_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_287 = {S10, 0, &V18166_287_57173, &P18166_237};	/*Property = CssFlexbox*/
const Value V18166_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_288 = {S11, 0, &V18166_288_57176, &P18166_287};	/*Property = CssFont*/
const Value V18166_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_289 = {S12, 0, &V18166_289_86899, &P18166_288};	/*Property = CssImages*/
const Value V18166_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_291 = {S13, 0, &V18166_291_57182, &P18166_289};	/*Property = CssMediaQueries*/
const Value V18166_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_292 = {S14, 0, &V18166_292_57185, &P18166_291};	/*Property = CssMinMax*/
const Value V18166_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_293 = {S15, 0, &V18166_293_57188, &P18166_292};	/*Property = CssOverflow*/
const Value V18166_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_294 = {S16, 0, &V18166_294_57191, &P18166_293};	/*Property = CssPosition*/
const Value V18166_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_295 = {S17, 0, &V18166_295_57194, &P18166_294};	/*Property = CssText*/
const Value V18166_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_297 = {S18, 0, &V18166_297_57197, &P18166_295};	/*Property = CssTransforms*/
const Value V18166_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_241 = {S19, 0, &V18166_241_55801, &P18166_297};	/*Property = CssTransitions*/
const Value V18166_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_298 = {S20, 0, &V18166_298_57208, &P18166_241};	/*Property = CssUI*/
const Value V18166_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_300 = {S21, 0, &V18166_300_57217, &P18166_298};	/*Property = DataSet*/
const Value V18166_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_301 = {S22, 0, &V18166_301_57220, &P18166_300};	/*Property = DataUrl*/
const Value V18166_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_302 = {S23, 0, &V18166_302_57223, &P18166_301};	/*Property = DeviceOrientation*/
const Value V18166_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_304 = {S24, 0, &V18166_304_57229, &P18166_302};	/*Property = FileReader*/
const Value V18166_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_305 = {S25, 0, &V18166_305_57232, &P18166_304};	/*Property = FileSaver*/
const Value V18166_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_306 = {S26, 0, &V18166_306_57238, &P18166_305};	/*Property = FileWriter*/
const Value V18166_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_307 = {S27, 0, &V18166_307_57241, &P18166_306};	/*Property = FormData*/
const Value V18166_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_309 = {S28, 0, &V18166_309_57247, &P18166_307};	/*Property = Fullscreen*/
const Value V18166_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_310 = {S29, 0, &V18166_310_57250, &P18166_309};	/*Property = GeoLocation*/
const Value V18166_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_312 = {S30, 0, &V18166_312_57256, &P18166_310};	/*Property = History*/
const Value V18166_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_314 = {S31, 0, &V18166_314_57262, &P18166_312};	/*Property = Html5*/
const Value V18166_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_313 = {S32, 0, &V18166_313_57259, &P18166_314};	/*Property = Html-Media-Capture*/
const Value V18166_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_315 = {S33, 0, &V18166_315_57265, &P18166_313};	/*Property = Iframe*/
const Value V18166_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_316 = {S34, 0, &V18166_316_57268, &P18166_315};	/*Property = IndexedDB*/
const Value V18166_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_317 = {S35, 0, &V18166_317_57271, &P18166_316};	/*Property = Json*/
const Value V18166_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18166_15 = {S37, 0, &V18166_15_28, &P18166_317};	/*Property = LayoutEngine*/
const Value V18166_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_344 = {S38, 0, &V18166_344_57274, &P18166_15};	/*Property = Masking*/
const Value V18166_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_320 = {S39, 0, &V18166_320_57292, &P18166_344};	/*Property = PostMessage*/
const Value V18166_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_321 = {S40, 0, &V18166_321_57295, &P18166_320};	/*Property = Progress*/
const Value V18166_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_322 = {S41, 0, &V18166_322_57298, &P18166_321};	/*Property = Prompts*/
const Value V18166_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_323 = {S42, 0, &V18166_323_57301, &P18166_322};	/*Property = Selector*/
const Value V18166_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_324 = {S43, 0, &V18166_324_57307, &P18166_323};	/*Property = Svg*/
const Value V18166_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_327 = {S44, 0, &V18166_327_57316, &P18166_324};	/*Property = TouchEvents*/
const Value V18166_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_328 = {S45, 0, &V18166_328_57319, &P18166_327};	/*Property = Track*/
const Value V18166_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_329 = {S46, 0, &V18166_329_57325, &P18166_328};	/*Property = Video*/
const Value V18166_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_330 = {S47, 0, &V18166_330_57328, &P18166_329};	/*Property = Viewport*/
const Value V18166_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_332 = {S48, 0, &V18166_332_57340, &P18166_330};	/*Property = WebWorkers*/
const Value V18166_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18166_333 = {S49, 0, &V18166_333_57343, &P18166_332};	/*Property = Xhr2*/
const Profile P18166 = {"18166", &P18166_333};	/*Profile ID - 18166*/
const Value V18600_1_1 = {S2, NULL};	/*Value = True*/
const Property P18600_1 = {S50, 0, &V18600_1_1, NULL};	/*Property = IsMobile*/
const Value V18600_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18600_5 = {S52, 0, &V18600_5_1227, &P18600_1};	/*Property = ScreenPixelsHeight*/
const Value V18600_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18600_4 = {S54, 0, &V18600_4_1229, &P18600_5};	/*Property = ScreenPixelsWidth*/
const Profile P18600 = {"18600", &P18600_4};	/*Profile ID - 18600*/
const Value V18467_1_1 = {S2, NULL};	/*Value = True*/
const Property P18467_1 = {S50, 0, &V18467_1_1, NULL};	/*Property = IsMobile*/
const Value V18467_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18467_5 = {S52, 0, &V18467_5_9981, &P18467_1};	/*Property = ScreenPixelsHeight*/
const Value V18467_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18467_4 = {S54, 0, &V18467_4_2336, &P18467_5};	/*Property = ScreenPixelsWidth*/
const Profile P18467 = {"18467", &P18467_4};	/*Profile ID - 18467*/
const Value V22374_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22374_279 = {S1, 0, &V22374_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22374_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22374_282 = {S3, 0, &V22374_282_57153, &P22374_279};	/*Property = BlobBuilder*/
const Value V22374_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22374_350 = {S4, 0, &V22374_350_57444, &P22374_282};	/*Property = Canvas*/
const Value V22374_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22374_283 = {S5, 0, &V22374_283_57159, &P22374_350};	/*Property = CssBackground*/
const Value V22374_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22374_284 = {S6, 0, &V22374_284_57163, &P22374_283};	/*Property = CssBorderImage*/
const Value V22374_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22374_285 = {S7, 0, &V22374_285_57166, &P22374_284};	/*Property = CssCanvas*/
const Value V22374_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22374_286 = {S8, 0, &V22374_286_57168, &P22374_285};	/*Property = CssColor*/
const Value V22374_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22374_237 = {S9, 0, &V22374_237_55786, &P22374_286};	/*Property = CssColumn*/
const Value V22374_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22374_287 = {S10, 0, &V22374_287_57172, &P22374_237};	/*Property = CssFlexbox*/
const Value V22374_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22374_288 = {S11, 0, &V22374_288_57175, &P22374_287};	/*Property = CssFont*/
const Value V22374_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22374_289 = {S12, 0, &V22374_289_57759, &P22374_288};	/*Property = CssImages*/
const Value V22374_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22374_291 = {S13, 0, &V22374_291_57180, &P22374_289};	/*Property = CssMediaQueries*/
const Value V22374_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22374_292 = {S14, 0, &V22374_292_57183, &P22374_291};	/*Property = CssMinMax*/
const Value V22374_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22374_293 = {S15, 0, &V22374_293_57187, &P22374_292};	/*Property = CssOverflow*/
const Value V22374_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22374_294 = {S16, 0, &V22374_294_57189, &P22374_293};	/*Property = CssPosition*/
const Value V22374_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22374_295 = {S17, 0, &V22374_295_57192, &P22374_294};	/*Property = CssText*/
const Value V22374_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22374_297 = {S18, 0, &V22374_297_57195, &P22374_295};	/*Property = CssTransforms*/
const Value V22374_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22374_241 = {S19, 0, &V22374_241_55799, &P22374_297};	/*Property = CssTransitions*/
const Value V22374_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22374_298 = {S20, 0, &V22374_298_57206, &P22374_241};	/*Property = CssUI*/
const Value V22374_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22374_300 = {S21, 0, &V22374_300_57216, &P22374_298};	/*Property = DataSet*/
const Value V22374_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22374_301 = {S22, 0, &V22374_301_57218, &P22374_300};	/*Property = DataUrl*/
const Value V22374_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22374_302 = {S23, 0, &V22374_302_57222, &P22374_301};	/*Property = DeviceOrientation*/
const Value V22374_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22374_304 = {S24, 0, &V22374_304_57227, &P22374_302};	/*Property = FileReader*/
const Value V22374_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22374_305 = {S25, 0, &V22374_305_57231, &P22374_304};	/*Property = FileSaver*/
const Value V22374_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22374_306 = {S26, 0, &V22374_306_57237, &P22374_305};	/*Property = FileWriter*/
const Value V22374_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22374_307 = {S27, 0, &V22374_307_57239, &P22374_306};	/*Property = FormData*/
const Value V22374_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22374_309 = {S28, 0, &V22374_309_57246, &P22374_307};	/*Property = Fullscreen*/
const Value V22374_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22374_310 = {S29, 0, &V22374_310_57248, &P22374_309};	/*Property = GeoLocation*/
const Value V22374_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22374_312 = {S30, 0, &V22374_312_57254, &P22374_310};	/*Property = History*/
const Value V22374_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22374_314 = {S31, 0, &V22374_314_57261, &P22374_312};	/*Property = Html5*/
const Value V22374_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22374_313 = {S32, 0, &V22374_313_57258, &P22374_314};	/*Property = Html-Media-Capture*/
const Value V22374_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22374_315 = {S33, 0, &V22374_315_57264, &P22374_313};	/*Property = Iframe*/
const Value V22374_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22374_316 = {S34, 0, &V22374_316_57266, &P22374_315};	/*Property = IndexedDB*/
const Value V22374_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22374_317 = {S35, 0, &V22374_317_57269, &P22374_316};	/*Property = Json*/
const Value V22374_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22374_15 = {S37, 0, &V22374_15_28, &P22374_317};	/*Property = LayoutEngine*/
const Value V22374_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22374_344 = {S38, 0, &V22374_344_57272, &P22374_15};	/*Property = Masking*/
const Value V22374_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22374_320 = {S39, 0, &V22374_320_57290, &P22374_344};	/*Property = PostMessage*/
const Value V22374_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22374_321 = {S40, 0, &V22374_321_57293, &P22374_320};	/*Property = Progress*/
const Value V22374_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22374_322 = {S41, 0, &V22374_322_57296, &P22374_321};	/*Property = Prompts*/
const Value V22374_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22374_323 = {S42, 0, &V22374_323_57299, &P22374_322};	/*Property = Selector*/
const Value V22374_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22374_324 = {S43, 0, &V22374_324_57306, &P22374_323};	/*Property = Svg*/
const Value V22374_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22374_327 = {S44, 0, &V22374_327_57315, &P22374_324};	/*Property = TouchEvents*/
const Value V22374_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22374_328 = {S45, 0, &V22374_328_57318, &P22374_327};	/*Property = Track*/
const Value V22374_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22374_329 = {S46, 0, &V22374_329_57323, &P22374_328};	/*Property = Video*/
const Value V22374_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22374_330 = {S47, 0, &V22374_330_57326, &P22374_329};	/*Property = Viewport*/
const Value V22374_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22374_332 = {S48, 0, &V22374_332_57338, &P22374_330};	/*Property = WebWorkers*/
const Value V22374_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22374_333 = {S49, 0, &V22374_333_57341, &P22374_332};	/*Property = Xhr2*/
const Profile P22374 = {"22374", &P22374_333};	/*Profile ID - 22374*/
const Value V15978_1_1 = {S2, NULL};	/*Value = True*/
const Property P15978_1 = {S50, 0, &V15978_1_1, NULL};	/*Property = IsMobile*/
const Value V15978_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15978_5 = {S52, 0, &V15978_5_1222, &P15978_1};	/*Property = ScreenPixelsHeight*/
const Value V15978_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15978_4 = {S54, 0, &V15978_4_1224, &P15978_5};	/*Property = ScreenPixelsWidth*/
const Profile P15978 = {"15978", &P15978_4};	/*Profile ID - 15978*/
const Value V22414_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22414_279 = {S1, 0, &V22414_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22414_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22414_282 = {S3, 0, &V22414_282_57153, &P22414_279};	/*Property = BlobBuilder*/
const Value V22414_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22414_350 = {S4, 0, &V22414_350_57444, &P22414_282};	/*Property = Canvas*/
const Value V22414_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22414_283 = {S5, 0, &V22414_283_57159, &P22414_350};	/*Property = CssBackground*/
const Value V22414_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22414_284 = {S6, 0, &V22414_284_57163, &P22414_283};	/*Property = CssBorderImage*/
const Value V22414_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22414_285 = {S7, 0, &V22414_285_57166, &P22414_284};	/*Property = CssCanvas*/
const Value V22414_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22414_286 = {S8, 0, &V22414_286_57168, &P22414_285};	/*Property = CssColor*/
const Value V22414_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22414_237 = {S9, 0, &V22414_237_55784, &P22414_286};	/*Property = CssColumn*/
const Value V22414_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22414_287 = {S10, 0, &V22414_287_57172, &P22414_237};	/*Property = CssFlexbox*/
const Value V22414_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22414_288 = {S11, 0, &V22414_288_57175, &P22414_287};	/*Property = CssFont*/
const Value V22414_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22414_289 = {S12, 0, &V22414_289_57759, &P22414_288};	/*Property = CssImages*/
const Value V22414_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22414_291 = {S13, 0, &V22414_291_57180, &P22414_289};	/*Property = CssMediaQueries*/
const Value V22414_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22414_292 = {S14, 0, &V22414_292_57183, &P22414_291};	/*Property = CssMinMax*/
const Value V22414_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22414_293 = {S15, 0, &V22414_293_57187, &P22414_292};	/*Property = CssOverflow*/
const Value V22414_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22414_294 = {S16, 0, &V22414_294_57189, &P22414_293};	/*Property = CssPosition*/
const Value V22414_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22414_295 = {S17, 0, &V22414_295_57192, &P22414_294};	/*Property = CssText*/
const Value V22414_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22414_297 = {S18, 0, &V22414_297_57195, &P22414_295};	/*Property = CssTransforms*/
const Value V22414_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22414_241 = {S19, 0, &V22414_241_55799, &P22414_297};	/*Property = CssTransitions*/
const Value V22414_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22414_298 = {S20, 0, &V22414_298_57206, &P22414_241};	/*Property = CssUI*/
const Value V22414_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22414_300 = {S21, 0, &V22414_300_57216, &P22414_298};	/*Property = DataSet*/
const Value V22414_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22414_301 = {S22, 0, &V22414_301_57218, &P22414_300};	/*Property = DataUrl*/
const Value V22414_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22414_302 = {S23, 0, &V22414_302_57222, &P22414_301};	/*Property = DeviceOrientation*/
const Value V22414_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22414_304 = {S24, 0, &V22414_304_57227, &P22414_302};	/*Property = FileReader*/
const Value V22414_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22414_305 = {S25, 0, &V22414_305_57231, &P22414_304};	/*Property = FileSaver*/
const Value V22414_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22414_306 = {S26, 0, &V22414_306_57237, &P22414_305};	/*Property = FileWriter*/
const Value V22414_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22414_307 = {S27, 0, &V22414_307_57239, &P22414_306};	/*Property = FormData*/
const Value V22414_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22414_309 = {S28, 0, &V22414_309_57246, &P22414_307};	/*Property = Fullscreen*/
const Value V22414_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22414_310 = {S29, 0, &V22414_310_57248, &P22414_309};	/*Property = GeoLocation*/
const Value V22414_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22414_312 = {S30, 0, &V22414_312_57254, &P22414_310};	/*Property = History*/
const Value V22414_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22414_314 = {S31, 0, &V22414_314_57261, &P22414_312};	/*Property = Html5*/
const Value V22414_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22414_313 = {S32, 0, &V22414_313_57258, &P22414_314};	/*Property = Html-Media-Capture*/
const Value V22414_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22414_315 = {S33, 0, &V22414_315_57264, &P22414_313};	/*Property = Iframe*/
const Value V22414_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22414_316 = {S34, 0, &V22414_316_57266, &P22414_315};	/*Property = IndexedDB*/
const Value V22414_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22414_317 = {S35, 0, &V22414_317_57269, &P22414_316};	/*Property = Json*/
const Value V22414_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22414_15 = {S37, 0, &V22414_15_28, &P22414_317};	/*Property = LayoutEngine*/
const Value V22414_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22414_344 = {S38, 0, &V22414_344_57272, &P22414_15};	/*Property = Masking*/
const Value V22414_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22414_320 = {S39, 0, &V22414_320_57290, &P22414_344};	/*Property = PostMessage*/
const Value V22414_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22414_321 = {S40, 0, &V22414_321_57293, &P22414_320};	/*Property = Progress*/
const Value V22414_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22414_322 = {S41, 0, &V22414_322_57296, &P22414_321};	/*Property = Prompts*/
const Value V22414_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22414_323 = {S42, 0, &V22414_323_57299, &P22414_322};	/*Property = Selector*/
const Value V22414_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22414_324 = {S43, 0, &V22414_324_57306, &P22414_323};	/*Property = Svg*/
const Value V22414_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22414_327 = {S44, 0, &V22414_327_57315, &P22414_324};	/*Property = TouchEvents*/
const Value V22414_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22414_328 = {S45, 0, &V22414_328_57318, &P22414_327};	/*Property = Track*/
const Value V22414_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22414_329 = {S46, 0, &V22414_329_57323, &P22414_328};	/*Property = Video*/
const Value V22414_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22414_330 = {S47, 0, &V22414_330_57326, &P22414_329};	/*Property = Viewport*/
const Value V22414_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22414_332 = {S48, 0, &V22414_332_57338, &P22414_330};	/*Property = WebWorkers*/
const Value V22414_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22414_333 = {S49, 0, &V22414_333_57342, &P22414_332};	/*Property = Xhr2*/
const Profile P22414 = {"22414", &P22414_333};	/*Profile ID - 22414*/
const Value V22246_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22246_279 = {S1, 0, &V22246_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22246_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22246_282 = {S3, 0, &V22246_282_57154, &P22246_279};	/*Property = BlobBuilder*/
const Value V22246_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22246_350 = {S4, 0, &V22246_350_57444, &P22246_282};	/*Property = Canvas*/
const Value V22246_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22246_283 = {S5, 0, &V22246_283_57159, &P22246_350};	/*Property = CssBackground*/
const Value V22246_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22246_284 = {S6, 0, &V22246_284_57163, &P22246_283};	/*Property = CssBorderImage*/
const Value V22246_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22246_285 = {S7, 0, &V22246_285_57166, &P22246_284};	/*Property = CssCanvas*/
const Value V22246_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22246_286 = {S8, 0, &V22246_286_57168, &P22246_285};	/*Property = CssColor*/
const Value V22246_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22246_237 = {S9, 0, &V22246_237_55786, &P22246_286};	/*Property = CssColumn*/
const Value V22246_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22246_287 = {S10, 0, &V22246_287_57172, &P22246_237};	/*Property = CssFlexbox*/
const Value V22246_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22246_288 = {S11, 0, &V22246_288_57175, &P22246_287};	/*Property = CssFont*/
const Value V22246_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22246_289 = {S12, 0, &V22246_289_57759, &P22246_288};	/*Property = CssImages*/
const Value V22246_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22246_291 = {S13, 0, &V22246_291_57181, &P22246_289};	/*Property = CssMediaQueries*/
const Value V22246_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22246_292 = {S14, 0, &V22246_292_57183, &P22246_291};	/*Property = CssMinMax*/
const Value V22246_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22246_293 = {S15, 0, &V22246_293_57187, &P22246_292};	/*Property = CssOverflow*/
const Value V22246_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22246_294 = {S16, 0, &V22246_294_57190, &P22246_293};	/*Property = CssPosition*/
const Value V22246_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22246_295 = {S17, 0, &V22246_295_57192, &P22246_294};	/*Property = CssText*/
const Value V22246_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22246_297 = {S18, 0, &V22246_297_57195, &P22246_295};	/*Property = CssTransforms*/
const Value V22246_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22246_241 = {S19, 0, &V22246_241_55799, &P22246_297};	/*Property = CssTransitions*/
const Value V22246_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22246_298 = {S20, 0, &V22246_298_57206, &P22246_241};	/*Property = CssUI*/
const Value V22246_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22246_300 = {S21, 0, &V22246_300_57216, &P22246_298};	/*Property = DataSet*/
const Value V22246_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22246_301 = {S22, 0, &V22246_301_57218, &P22246_300};	/*Property = DataUrl*/
const Value V22246_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22246_302 = {S23, 0, &V22246_302_57222, &P22246_301};	/*Property = DeviceOrientation*/
const Value V22246_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22246_304 = {S24, 0, &V22246_304_57228, &P22246_302};	/*Property = FileReader*/
const Value V22246_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22246_305 = {S25, 0, &V22246_305_57231, &P22246_304};	/*Property = FileSaver*/
const Value V22246_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22246_306 = {S26, 0, &V22246_306_57237, &P22246_305};	/*Property = FileWriter*/
const Value V22246_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22246_307 = {S27, 0, &V22246_307_57240, &P22246_306};	/*Property = FormData*/
const Value V22246_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22246_309 = {S28, 0, &V22246_309_57246, &P22246_307};	/*Property = Fullscreen*/
const Value V22246_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22246_310 = {S29, 0, &V22246_310_57248, &P22246_309};	/*Property = GeoLocation*/
const Value V22246_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22246_312 = {S30, 0, &V22246_312_57255, &P22246_310};	/*Property = History*/
const Value V22246_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22246_314 = {S31, 0, &V22246_314_57261, &P22246_312};	/*Property = Html5*/
const Value V22246_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22246_313 = {S32, 0, &V22246_313_57258, &P22246_314};	/*Property = Html-Media-Capture*/
const Value V22246_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22246_315 = {S33, 0, &V22246_315_57264, &P22246_313};	/*Property = Iframe*/
const Value V22246_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22246_316 = {S34, 0, &V22246_316_57267, &P22246_315};	/*Property = IndexedDB*/
const Value V22246_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22246_317 = {S35, 0, &V22246_317_57269, &P22246_316};	/*Property = Json*/
const Value V22246_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22246_15 = {S37, 0, &V22246_15_72, &P22246_317};	/*Property = LayoutEngine*/
const Value V22246_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22246_344 = {S38, 0, &V22246_344_57272, &P22246_15};	/*Property = Masking*/
const Value V22246_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22246_320 = {S39, 0, &V22246_320_57290, &P22246_344};	/*Property = PostMessage*/
const Value V22246_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22246_321 = {S40, 0, &V22246_321_57293, &P22246_320};	/*Property = Progress*/
const Value V22246_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22246_322 = {S41, 0, &V22246_322_57296, &P22246_321};	/*Property = Prompts*/
const Value V22246_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22246_323 = {S42, 0, &V22246_323_57299, &P22246_322};	/*Property = Selector*/
const Value V22246_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22246_324 = {S43, 0, &V22246_324_57306, &P22246_323};	/*Property = Svg*/
const Value V22246_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22246_327 = {S44, 0, &V22246_327_57315, &P22246_324};	/*Property = TouchEvents*/
const Value V22246_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22246_328 = {S45, 0, &V22246_328_57318, &P22246_327};	/*Property = Track*/
const Value V22246_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22246_329 = {S46, 0, &V22246_329_57323, &P22246_328};	/*Property = Video*/
const Value V22246_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22246_330 = {S47, 0, &V22246_330_57326, &P22246_329};	/*Property = Viewport*/
const Value V22246_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22246_332 = {S48, 0, &V22246_332_57339, &P22246_330};	/*Property = WebWorkers*/
const Value V22246_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22246_333 = {S49, 0, &V22246_333_57342, &P22246_332};	/*Property = Xhr2*/
const Profile P22246 = {"22246", &P22246_333};	/*Profile ID - 22246*/
const Value V24208_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24208_279 = {S1, 0, &V24208_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24208_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24208_282 = {S3, 0, &V24208_282_57154, &P24208_279};	/*Property = BlobBuilder*/
const Value V24208_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24208_350 = {S4, 0, &V24208_350_57444, &P24208_282};	/*Property = Canvas*/
const Value V24208_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24208_283 = {S5, 0, &V24208_283_57159, &P24208_350};	/*Property = CssBackground*/
const Value V24208_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24208_284 = {S6, 0, &V24208_284_57162, &P24208_283};	/*Property = CssBorderImage*/
const Value V24208_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24208_285 = {S7, 0, &V24208_285_57166, &P24208_284};	/*Property = CssCanvas*/
const Value V24208_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24208_286 = {S8, 0, &V24208_286_57168, &P24208_285};	/*Property = CssColor*/
const Value V24208_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24208_237 = {S9, 0, &V24208_237_55786, &P24208_286};	/*Property = CssColumn*/
const Value V24208_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24208_287 = {S10, 0, &V24208_287_57172, &P24208_237};	/*Property = CssFlexbox*/
const Value V24208_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24208_288 = {S11, 0, &V24208_288_57175, &P24208_287};	/*Property = CssFont*/
const Value V24208_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24208_289 = {S12, 0, &V24208_289_57759, &P24208_288};	/*Property = CssImages*/
const Value V24208_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24208_291 = {S13, 0, &V24208_291_57180, &P24208_289};	/*Property = CssMediaQueries*/
const Value V24208_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24208_292 = {S14, 0, &V24208_292_57183, &P24208_291};	/*Property = CssMinMax*/
const Value V24208_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24208_293 = {S15, 0, &V24208_293_57187, &P24208_292};	/*Property = CssOverflow*/
const Value V24208_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24208_294 = {S16, 0, &V24208_294_57189, &P24208_293};	/*Property = CssPosition*/
const Value V24208_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24208_295 = {S17, 0, &V24208_295_57192, &P24208_294};	/*Property = CssText*/
const Value V24208_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24208_297 = {S18, 0, &V24208_297_57195, &P24208_295};	/*Property = CssTransforms*/
const Value V24208_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24208_241 = {S19, 0, &V24208_241_55799, &P24208_297};	/*Property = CssTransitions*/
const Value V24208_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24208_298 = {S20, 0, &V24208_298_57206, &P24208_241};	/*Property = CssUI*/
const Value V24208_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24208_300 = {S21, 0, &V24208_300_57215, &P24208_298};	/*Property = DataSet*/
const Value V24208_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24208_301 = {S22, 0, &V24208_301_57218, &P24208_300};	/*Property = DataUrl*/
const Value V24208_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24208_302 = {S23, 0, &V24208_302_57221, &P24208_301};	/*Property = DeviceOrientation*/
const Value V24208_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24208_304 = {S24, 0, &V24208_304_57227, &P24208_302};	/*Property = FileReader*/
const Value V24208_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24208_305 = {S25, 0, &V24208_305_57231, &P24208_304};	/*Property = FileSaver*/
const Value V24208_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24208_306 = {S26, 0, &V24208_306_57237, &P24208_305};	/*Property = FileWriter*/
const Value V24208_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24208_307 = {S27, 0, &V24208_307_57239, &P24208_306};	/*Property = FormData*/
const Value V24208_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24208_309 = {S28, 0, &V24208_309_57246, &P24208_307};	/*Property = Fullscreen*/
const Value V24208_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24208_310 = {S29, 0, &V24208_310_57248, &P24208_309};	/*Property = GeoLocation*/
const Value V24208_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24208_312 = {S30, 0, &V24208_312_57254, &P24208_310};	/*Property = History*/
const Value V24208_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24208_314 = {S31, 0, &V24208_314_57260, &P24208_312};	/*Property = Html5*/
const Value V24208_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24208_313 = {S32, 0, &V24208_313_57257, &P24208_314};	/*Property = Html-Media-Capture*/
const Value V24208_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24208_315 = {S33, 0, &V24208_315_57264, &P24208_313};	/*Property = Iframe*/
const Value V24208_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24208_316 = {S34, 0, &V24208_316_57266, &P24208_315};	/*Property = IndexedDB*/
const Value V24208_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24208_317 = {S35, 0, &V24208_317_57269, &P24208_316};	/*Property = Json*/
const Value V24208_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24208_15 = {S37, 0, &V24208_15_54167, &P24208_317};	/*Property = LayoutEngine*/
const Value V24208_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24208_344 = {S38, 0, &V24208_344_57272, &P24208_15};	/*Property = Masking*/
const Value V24208_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24208_320 = {S39, 0, &V24208_320_57290, &P24208_344};	/*Property = PostMessage*/
const Value V24208_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24208_321 = {S40, 0, &V24208_321_57293, &P24208_320};	/*Property = Progress*/
const Value V24208_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24208_322 = {S41, 0, &V24208_322_57296, &P24208_321};	/*Property = Prompts*/
const Value V24208_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24208_323 = {S42, 0, &V24208_323_57299, &P24208_322};	/*Property = Selector*/
const Value V24208_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24208_324 = {S43, 0, &V24208_324_57306, &P24208_323};	/*Property = Svg*/
const Value V24208_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24208_327 = {S44, 0, &V24208_327_57314, &P24208_324};	/*Property = TouchEvents*/
const Value V24208_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24208_328 = {S45, 0, &V24208_328_57318, &P24208_327};	/*Property = Track*/
const Value V24208_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24208_329 = {S46, 0, &V24208_329_57323, &P24208_328};	/*Property = Video*/
const Value V24208_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24208_330 = {S47, 0, &V24208_330_57326, &P24208_329};	/*Property = Viewport*/
const Value V24208_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24208_332 = {S48, 0, &V24208_332_57339, &P24208_330};	/*Property = WebWorkers*/
const Value V24208_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24208_333 = {S49, 0, &V24208_333_57342, &P24208_332};	/*Property = Xhr2*/
const Profile P24208 = {"24208", &P24208_333};	/*Profile ID - 24208*/
const Value V22415_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22415_279 = {S1, 0, &V22415_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22415_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22415_282 = {S3, 0, &V22415_282_57153, &P22415_279};	/*Property = BlobBuilder*/
const Value V22415_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22415_350 = {S4, 0, &V22415_350_57444, &P22415_282};	/*Property = Canvas*/
const Value V22415_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22415_283 = {S5, 0, &V22415_283_57159, &P22415_350};	/*Property = CssBackground*/
const Value V22415_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22415_284 = {S6, 0, &V22415_284_57163, &P22415_283};	/*Property = CssBorderImage*/
const Value V22415_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22415_285 = {S7, 0, &V22415_285_57166, &P22415_284};	/*Property = CssCanvas*/
const Value V22415_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22415_286 = {S8, 0, &V22415_286_57168, &P22415_285};	/*Property = CssColor*/
const Value V22415_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22415_237 = {S9, 0, &V22415_237_55784, &P22415_286};	/*Property = CssColumn*/
const Value V22415_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22415_287 = {S10, 0, &V22415_287_57172, &P22415_237};	/*Property = CssFlexbox*/
const Value V22415_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22415_288 = {S11, 0, &V22415_288_57175, &P22415_287};	/*Property = CssFont*/
const Value V22415_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22415_289 = {S12, 0, &V22415_289_57759, &P22415_288};	/*Property = CssImages*/
const Value V22415_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22415_291 = {S13, 0, &V22415_291_57180, &P22415_289};	/*Property = CssMediaQueries*/
const Value V22415_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22415_292 = {S14, 0, &V22415_292_57183, &P22415_291};	/*Property = CssMinMax*/
const Value V22415_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22415_293 = {S15, 0, &V22415_293_57187, &P22415_292};	/*Property = CssOverflow*/
const Value V22415_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22415_294 = {S16, 0, &V22415_294_57189, &P22415_293};	/*Property = CssPosition*/
const Value V22415_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22415_295 = {S17, 0, &V22415_295_57192, &P22415_294};	/*Property = CssText*/
const Value V22415_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22415_297 = {S18, 0, &V22415_297_57195, &P22415_295};	/*Property = CssTransforms*/
const Value V22415_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22415_241 = {S19, 0, &V22415_241_55799, &P22415_297};	/*Property = CssTransitions*/
const Value V22415_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22415_298 = {S20, 0, &V22415_298_57206, &P22415_241};	/*Property = CssUI*/
const Value V22415_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22415_300 = {S21, 0, &V22415_300_57216, &P22415_298};	/*Property = DataSet*/
const Value V22415_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22415_301 = {S22, 0, &V22415_301_57218, &P22415_300};	/*Property = DataUrl*/
const Value V22415_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22415_302 = {S23, 0, &V22415_302_57222, &P22415_301};	/*Property = DeviceOrientation*/
const Value V22415_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22415_304 = {S24, 0, &V22415_304_57227, &P22415_302};	/*Property = FileReader*/
const Value V22415_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22415_305 = {S25, 0, &V22415_305_57231, &P22415_304};	/*Property = FileSaver*/
const Value V22415_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22415_306 = {S26, 0, &V22415_306_57237, &P22415_305};	/*Property = FileWriter*/
const Value V22415_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22415_307 = {S27, 0, &V22415_307_57239, &P22415_306};	/*Property = FormData*/
const Value V22415_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22415_309 = {S28, 0, &V22415_309_57246, &P22415_307};	/*Property = Fullscreen*/
const Value V22415_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22415_310 = {S29, 0, &V22415_310_57248, &P22415_309};	/*Property = GeoLocation*/
const Value V22415_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22415_312 = {S30, 0, &V22415_312_57254, &P22415_310};	/*Property = History*/
const Value V22415_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22415_314 = {S31, 0, &V22415_314_57261, &P22415_312};	/*Property = Html5*/
const Value V22415_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22415_313 = {S32, 0, &V22415_313_57258, &P22415_314};	/*Property = Html-Media-Capture*/
const Value V22415_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22415_315 = {S33, 0, &V22415_315_57264, &P22415_313};	/*Property = Iframe*/
const Value V22415_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22415_316 = {S34, 0, &V22415_316_57266, &P22415_315};	/*Property = IndexedDB*/
const Value V22415_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22415_317 = {S35, 0, &V22415_317_57269, &P22415_316};	/*Property = Json*/
const Value V22415_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22415_15 = {S37, 0, &V22415_15_28, &P22415_317};	/*Property = LayoutEngine*/
const Value V22415_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22415_344 = {S38, 0, &V22415_344_57272, &P22415_15};	/*Property = Masking*/
const Value V22415_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22415_320 = {S39, 0, &V22415_320_57290, &P22415_344};	/*Property = PostMessage*/
const Value V22415_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22415_321 = {S40, 0, &V22415_321_57293, &P22415_320};	/*Property = Progress*/
const Value V22415_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22415_322 = {S41, 0, &V22415_322_57296, &P22415_321};	/*Property = Prompts*/
const Value V22415_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22415_323 = {S42, 0, &V22415_323_57299, &P22415_322};	/*Property = Selector*/
const Value V22415_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22415_324 = {S43, 0, &V22415_324_57306, &P22415_323};	/*Property = Svg*/
const Value V22415_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22415_327 = {S44, 0, &V22415_327_57315, &P22415_324};	/*Property = TouchEvents*/
const Value V22415_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22415_328 = {S45, 0, &V22415_328_57318, &P22415_327};	/*Property = Track*/
const Value V22415_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22415_329 = {S46, 0, &V22415_329_57323, &P22415_328};	/*Property = Video*/
const Value V22415_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22415_330 = {S47, 0, &V22415_330_57326, &P22415_329};	/*Property = Viewport*/
const Value V22415_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22415_332 = {S48, 0, &V22415_332_57338, &P22415_330};	/*Property = WebWorkers*/
const Value V22415_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22415_333 = {S49, 0, &V22415_333_57342, &P22415_332};	/*Property = Xhr2*/
const Profile P22415 = {"22415", &P22415_333};	/*Profile ID - 22415*/
const Value V24197_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24197_279 = {S1, 0, &V24197_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24197_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24197_282 = {S3, 0, &V24197_282_57154, &P24197_279};	/*Property = BlobBuilder*/
const Value V24197_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24197_350 = {S4, 0, &V24197_350_57444, &P24197_282};	/*Property = Canvas*/
const Value V24197_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24197_283 = {S5, 0, &V24197_283_57159, &P24197_350};	/*Property = CssBackground*/
const Value V24197_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24197_284 = {S6, 0, &V24197_284_57162, &P24197_283};	/*Property = CssBorderImage*/
const Value V24197_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24197_285 = {S7, 0, &V24197_285_57166, &P24197_284};	/*Property = CssCanvas*/
const Value V24197_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24197_286 = {S8, 0, &V24197_286_57168, &P24197_285};	/*Property = CssColor*/
const Value V24197_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24197_237 = {S9, 0, &V24197_237_55786, &P24197_286};	/*Property = CssColumn*/
const Value V24197_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24197_287 = {S10, 0, &V24197_287_57172, &P24197_237};	/*Property = CssFlexbox*/
const Value V24197_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24197_288 = {S11, 0, &V24197_288_57175, &P24197_287};	/*Property = CssFont*/
const Value V24197_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24197_289 = {S12, 0, &V24197_289_57759, &P24197_288};	/*Property = CssImages*/
const Value V24197_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24197_291 = {S13, 0, &V24197_291_57180, &P24197_289};	/*Property = CssMediaQueries*/
const Value V24197_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24197_292 = {S14, 0, &V24197_292_57183, &P24197_291};	/*Property = CssMinMax*/
const Value V24197_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24197_293 = {S15, 0, &V24197_293_57187, &P24197_292};	/*Property = CssOverflow*/
const Value V24197_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24197_294 = {S16, 0, &V24197_294_57189, &P24197_293};	/*Property = CssPosition*/
const Value V24197_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24197_295 = {S17, 0, &V24197_295_57192, &P24197_294};	/*Property = CssText*/
const Value V24197_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24197_297 = {S18, 0, &V24197_297_57195, &P24197_295};	/*Property = CssTransforms*/
const Value V24197_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24197_241 = {S19, 0, &V24197_241_55799, &P24197_297};	/*Property = CssTransitions*/
const Value V24197_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24197_298 = {S20, 0, &V24197_298_57206, &P24197_241};	/*Property = CssUI*/
const Value V24197_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24197_300 = {S21, 0, &V24197_300_57215, &P24197_298};	/*Property = DataSet*/
const Value V24197_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24197_301 = {S22, 0, &V24197_301_57218, &P24197_300};	/*Property = DataUrl*/
const Value V24197_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24197_302 = {S23, 0, &V24197_302_57221, &P24197_301};	/*Property = DeviceOrientation*/
const Value V24197_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24197_304 = {S24, 0, &V24197_304_57227, &P24197_302};	/*Property = FileReader*/
const Value V24197_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24197_305 = {S25, 0, &V24197_305_57231, &P24197_304};	/*Property = FileSaver*/
const Value V24197_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24197_306 = {S26, 0, &V24197_306_57237, &P24197_305};	/*Property = FileWriter*/
const Value V24197_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24197_307 = {S27, 0, &V24197_307_57239, &P24197_306};	/*Property = FormData*/
const Value V24197_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24197_309 = {S28, 0, &V24197_309_57246, &P24197_307};	/*Property = Fullscreen*/
const Value V24197_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24197_310 = {S29, 0, &V24197_310_57248, &P24197_309};	/*Property = GeoLocation*/
const Value V24197_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24197_312 = {S30, 0, &V24197_312_57254, &P24197_310};	/*Property = History*/
const Value V24197_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24197_314 = {S31, 0, &V24197_314_57260, &P24197_312};	/*Property = Html5*/
const Value V24197_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24197_313 = {S32, 0, &V24197_313_57257, &P24197_314};	/*Property = Html-Media-Capture*/
const Value V24197_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24197_315 = {S33, 0, &V24197_315_57264, &P24197_313};	/*Property = Iframe*/
const Value V24197_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24197_316 = {S34, 0, &V24197_316_57266, &P24197_315};	/*Property = IndexedDB*/
const Value V24197_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24197_317 = {S35, 0, &V24197_317_57269, &P24197_316};	/*Property = Json*/
const Value V24197_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24197_15 = {S37, 0, &V24197_15_54167, &P24197_317};	/*Property = LayoutEngine*/
const Value V24197_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24197_344 = {S38, 0, &V24197_344_57272, &P24197_15};	/*Property = Masking*/
const Value V24197_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24197_320 = {S39, 0, &V24197_320_57290, &P24197_344};	/*Property = PostMessage*/
const Value V24197_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24197_321 = {S40, 0, &V24197_321_57293, &P24197_320};	/*Property = Progress*/
const Value V24197_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24197_322 = {S41, 0, &V24197_322_57296, &P24197_321};	/*Property = Prompts*/
const Value V24197_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24197_323 = {S42, 0, &V24197_323_57299, &P24197_322};	/*Property = Selector*/
const Value V24197_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24197_324 = {S43, 0, &V24197_324_57306, &P24197_323};	/*Property = Svg*/
const Value V24197_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24197_327 = {S44, 0, &V24197_327_57314, &P24197_324};	/*Property = TouchEvents*/
const Value V24197_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24197_328 = {S45, 0, &V24197_328_57318, &P24197_327};	/*Property = Track*/
const Value V24197_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24197_329 = {S46, 0, &V24197_329_57323, &P24197_328};	/*Property = Video*/
const Value V24197_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24197_330 = {S47, 0, &V24197_330_57326, &P24197_329};	/*Property = Viewport*/
const Value V24197_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24197_332 = {S48, 0, &V24197_332_57339, &P24197_330};	/*Property = WebWorkers*/
const Value V24197_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24197_333 = {S49, 0, &V24197_333_57342, &P24197_332};	/*Property = Xhr2*/
const Profile P24197 = {"24197", &P24197_333};	/*Profile ID - 24197*/
const Profile P18117 = {"18117", NULL};	/*Profile ID - 18117*/
const Value V17347_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_279 = {S1, 0, &V17347_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17347_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_282 = {S3, 0, &V17347_282_57155, &P17347_279};	/*Property = BlobBuilder*/
const Value V17347_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_350 = {S4, 0, &V17347_350_57446, &P17347_282};	/*Property = Canvas*/
const Value V17347_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_283 = {S5, 0, &V17347_283_57161, &P17347_350};	/*Property = CssBackground*/
const Value V17347_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_284 = {S6, 0, &V17347_284_57164, &P17347_283};	/*Property = CssBorderImage*/
const Value V17347_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_285 = {S7, 0, &V17347_285_57167, &P17347_284};	/*Property = CssCanvas*/
const Value V17347_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_286 = {S8, 0, &V17347_286_57170, &P17347_285};	/*Property = CssColor*/
const Value V17347_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_237 = {S9, 0, &V17347_237_55786, &P17347_286};	/*Property = CssColumn*/
const Value V17347_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_287 = {S10, 0, &V17347_287_57173, &P17347_237};	/*Property = CssFlexbox*/
const Value V17347_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_288 = {S11, 0, &V17347_288_57176, &P17347_287};	/*Property = CssFont*/
const Value V17347_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_289 = {S12, 0, &V17347_289_86899, &P17347_288};	/*Property = CssImages*/
const Value V17347_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_291 = {S13, 0, &V17347_291_57182, &P17347_289};	/*Property = CssMediaQueries*/
const Value V17347_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_292 = {S14, 0, &V17347_292_57185, &P17347_291};	/*Property = CssMinMax*/
const Value V17347_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_293 = {S15, 0, &V17347_293_57188, &P17347_292};	/*Property = CssOverflow*/
const Value V17347_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_294 = {S16, 0, &V17347_294_57191, &P17347_293};	/*Property = CssPosition*/
const Value V17347_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_295 = {S17, 0, &V17347_295_57194, &P17347_294};	/*Property = CssText*/
const Value V17347_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_297 = {S18, 0, &V17347_297_57197, &P17347_295};	/*Property = CssTransforms*/
const Value V17347_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_241 = {S19, 0, &V17347_241_55801, &P17347_297};	/*Property = CssTransitions*/
const Value V17347_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_298 = {S20, 0, &V17347_298_57208, &P17347_241};	/*Property = CssUI*/
const Value V17347_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_300 = {S21, 0, &V17347_300_57217, &P17347_298};	/*Property = DataSet*/
const Value V17347_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_301 = {S22, 0, &V17347_301_57220, &P17347_300};	/*Property = DataUrl*/
const Value V17347_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_302 = {S23, 0, &V17347_302_57223, &P17347_301};	/*Property = DeviceOrientation*/
const Value V17347_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_304 = {S24, 0, &V17347_304_57229, &P17347_302};	/*Property = FileReader*/
const Value V17347_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_305 = {S25, 0, &V17347_305_57232, &P17347_304};	/*Property = FileSaver*/
const Value V17347_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_306 = {S26, 0, &V17347_306_57238, &P17347_305};	/*Property = FileWriter*/
const Value V17347_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_307 = {S27, 0, &V17347_307_57241, &P17347_306};	/*Property = FormData*/
const Value V17347_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_309 = {S28, 0, &V17347_309_57247, &P17347_307};	/*Property = Fullscreen*/
const Value V17347_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_310 = {S29, 0, &V17347_310_57250, &P17347_309};	/*Property = GeoLocation*/
const Value V17347_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_312 = {S30, 0, &V17347_312_57256, &P17347_310};	/*Property = History*/
const Value V17347_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_314 = {S31, 0, &V17347_314_57262, &P17347_312};	/*Property = Html5*/
const Value V17347_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_313 = {S32, 0, &V17347_313_57259, &P17347_314};	/*Property = Html-Media-Capture*/
const Value V17347_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_315 = {S33, 0, &V17347_315_57265, &P17347_313};	/*Property = Iframe*/
const Value V17347_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_316 = {S34, 0, &V17347_316_57268, &P17347_315};	/*Property = IndexedDB*/
const Value V17347_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_317 = {S35, 0, &V17347_317_57271, &P17347_316};	/*Property = Json*/
const Value V17347_15_90838 = {S70, NULL};	/*Value = U3*/
const Property P17347_15 = {S37, 0, &V17347_15_90838, &P17347_317};	/*Property = LayoutEngine*/
const Value V17347_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_344 = {S38, 0, &V17347_344_57274, &P17347_15};	/*Property = Masking*/
const Value V17347_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_320 = {S39, 0, &V17347_320_57292, &P17347_344};	/*Property = PostMessage*/
const Value V17347_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_321 = {S40, 0, &V17347_321_57295, &P17347_320};	/*Property = Progress*/
const Value V17347_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_322 = {S41, 0, &V17347_322_57298, &P17347_321};	/*Property = Prompts*/
const Value V17347_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_323 = {S42, 0, &V17347_323_57301, &P17347_322};	/*Property = Selector*/
const Value V17347_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_324 = {S43, 0, &V17347_324_57307, &P17347_323};	/*Property = Svg*/
const Value V17347_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_327 = {S44, 0, &V17347_327_57316, &P17347_324};	/*Property = TouchEvents*/
const Value V17347_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_328 = {S45, 0, &V17347_328_57319, &P17347_327};	/*Property = Track*/
const Value V17347_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_329 = {S46, 0, &V17347_329_57325, &P17347_328};	/*Property = Video*/
const Value V17347_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_330 = {S47, 0, &V17347_330_57328, &P17347_329};	/*Property = Viewport*/
const Value V17347_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_332 = {S48, 0, &V17347_332_57340, &P17347_330};	/*Property = WebWorkers*/
const Value V17347_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17347_333 = {S49, 0, &V17347_333_57343, &P17347_332};	/*Property = Xhr2*/
const Profile P17347 = {"17347", &P17347_333};	/*Profile ID - 17347*/
const Profile P24697 = {"24697", NULL};	/*Profile ID - 24697*/
const Value V13074_1_1 = {S2, NULL};	/*Value = True*/
const Property P13074_1 = {S50, 0, &V13074_1_1, NULL};	/*Property = IsMobile*/
const Value V13074_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P13074_5 = {S52, 0, &V13074_5_9981, &P13074_1};	/*Property = ScreenPixelsHeight*/
const Value V13074_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P13074_4 = {S54, 0, &V13074_4_39098, &P13074_5};	/*Property = ScreenPixelsWidth*/
const Profile P13074 = {"13074", &P13074_4};	/*Profile ID - 13074*/
const Value V18599_1_1 = {S2, NULL};	/*Value = True*/
const Property P18599_1 = {S50, 0, &V18599_1_1, NULL};	/*Property = IsMobile*/
const Value V18599_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18599_5 = {S52, 0, &V18599_5_1277, &P18599_1};	/*Property = ScreenPixelsHeight*/
const Value V18599_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18599_4 = {S54, 0, &V18599_4_1279, &P18599_5};	/*Property = ScreenPixelsWidth*/
const Profile P18599 = {"18599", &P18599_4};	/*Profile ID - 18599*/
const Value V16263_1_1 = {S2, NULL};	/*Value = True*/
const Property P16263_1 = {S50, 0, &V16263_1_1, NULL};	/*Property = IsMobile*/
const Value V16263_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16263_5 = {S52, 0, &V16263_5_1227, &P16263_1};	/*Property = ScreenPixelsHeight*/
const Value V16263_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P16263_4 = {S54, 0, &V16263_4_1477, &P16263_5};	/*Property = ScreenPixelsWidth*/
const Profile P16263 = {"16263", &P16263_4};	/*Profile ID - 16263*/
const Profile P18207 = {"18207", NULL};	/*Profile ID - 18207*/
const Value V18363_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18363_279 = {S1, 0, &V18363_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18363_282_57153 = {S2, NULL};	/*Value = True*/
const Property P18363_282 = {S3, 0, &V18363_282_57153, &P18363_279};	/*Property = BlobBuilder*/
const Value V18363_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18363_350 = {S4, 0, &V18363_350_57444, &P18363_282};	/*Property = Canvas*/
const Value V18363_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18363_283 = {S5, 0, &V18363_283_57159, &P18363_350};	/*Property = CssBackground*/
const Value V18363_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18363_284 = {S6, 0, &V18363_284_57163, &P18363_283};	/*Property = CssBorderImage*/
const Value V18363_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18363_285 = {S7, 0, &V18363_285_57166, &P18363_284};	/*Property = CssCanvas*/
const Value V18363_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18363_286 = {S8, 0, &V18363_286_57168, &P18363_285};	/*Property = CssColor*/
const Value V18363_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18363_237 = {S9, 0, &V18363_237_55786, &P18363_286};	/*Property = CssColumn*/
const Value V18363_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18363_287 = {S10, 0, &V18363_287_57172, &P18363_237};	/*Property = CssFlexbox*/
const Value V18363_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18363_288 = {S11, 0, &V18363_288_57175, &P18363_287};	/*Property = CssFont*/
const Value V18363_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18363_289 = {S12, 0, &V18363_289_57759, &P18363_288};	/*Property = CssImages*/
const Value V18363_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18363_291 = {S13, 0, &V18363_291_57180, &P18363_289};	/*Property = CssMediaQueries*/
const Value V18363_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18363_292 = {S14, 0, &V18363_292_57183, &P18363_291};	/*Property = CssMinMax*/
const Value V18363_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18363_293 = {S15, 0, &V18363_293_57187, &P18363_292};	/*Property = CssOverflow*/
const Value V18363_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18363_294 = {S16, 0, &V18363_294_57189, &P18363_293};	/*Property = CssPosition*/
const Value V18363_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18363_295 = {S17, 0, &V18363_295_57192, &P18363_294};	/*Property = CssText*/
const Value V18363_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18363_297 = {S18, 0, &V18363_297_57195, &P18363_295};	/*Property = CssTransforms*/
const Value V18363_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18363_241 = {S19, 0, &V18363_241_55799, &P18363_297};	/*Property = CssTransitions*/
const Value V18363_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18363_298 = {S20, 0, &V18363_298_57206, &P18363_241};	/*Property = CssUI*/
const Value V18363_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18363_300 = {S21, 0, &V18363_300_57216, &P18363_298};	/*Property = DataSet*/
const Value V18363_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18363_301 = {S22, 0, &V18363_301_57218, &P18363_300};	/*Property = DataUrl*/
const Value V18363_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18363_302 = {S23, 0, &V18363_302_57222, &P18363_301};	/*Property = DeviceOrientation*/
const Value V18363_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18363_304 = {S24, 0, &V18363_304_57227, &P18363_302};	/*Property = FileReader*/
const Value V18363_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18363_305 = {S25, 0, &V18363_305_57231, &P18363_304};	/*Property = FileSaver*/
const Value V18363_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18363_306 = {S26, 0, &V18363_306_57237, &P18363_305};	/*Property = FileWriter*/
const Value V18363_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18363_307 = {S27, 0, &V18363_307_57239, &P18363_306};	/*Property = FormData*/
const Value V18363_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18363_309 = {S28, 0, &V18363_309_57246, &P18363_307};	/*Property = Fullscreen*/
const Value V18363_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18363_310 = {S29, 0, &V18363_310_57248, &P18363_309};	/*Property = GeoLocation*/
const Value V18363_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18363_312 = {S30, 0, &V18363_312_57254, &P18363_310};	/*Property = History*/
const Value V18363_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18363_314 = {S31, 0, &V18363_314_57261, &P18363_312};	/*Property = Html5*/
const Value V18363_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18363_313 = {S32, 0, &V18363_313_57258, &P18363_314};	/*Property = Html-Media-Capture*/
const Value V18363_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18363_315 = {S33, 0, &V18363_315_57264, &P18363_313};	/*Property = Iframe*/
const Value V18363_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18363_316 = {S34, 0, &V18363_316_57267, &P18363_315};	/*Property = IndexedDB*/
const Value V18363_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18363_317 = {S35, 0, &V18363_317_57269, &P18363_316};	/*Property = Json*/
const Value V18363_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18363_15 = {S37, 0, &V18363_15_28, &P18363_317};	/*Property = LayoutEngine*/
const Value V18363_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18363_344 = {S38, 0, &V18363_344_57272, &P18363_15};	/*Property = Masking*/
const Value V18363_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18363_320 = {S39, 0, &V18363_320_57290, &P18363_344};	/*Property = PostMessage*/
const Value V18363_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18363_321 = {S40, 0, &V18363_321_57293, &P18363_320};	/*Property = Progress*/
const Value V18363_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18363_322 = {S41, 0, &V18363_322_57296, &P18363_321};	/*Property = Prompts*/
const Value V18363_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18363_323 = {S42, 0, &V18363_323_57299, &P18363_322};	/*Property = Selector*/
const Value V18363_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18363_324 = {S43, 0, &V18363_324_57306, &P18363_323};	/*Property = Svg*/
const Value V18363_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18363_327 = {S44, 0, &V18363_327_57315, &P18363_324};	/*Property = TouchEvents*/
const Value V18363_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18363_328 = {S45, 0, &V18363_328_57318, &P18363_327};	/*Property = Track*/
const Value V18363_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18363_329 = {S46, 0, &V18363_329_57323, &P18363_328};	/*Property = Video*/
const Value V18363_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18363_330 = {S47, 0, &V18363_330_57326, &P18363_329};	/*Property = Viewport*/
const Value V18363_332_57338 = {S2, NULL};	/*Value = True*/
const Property P18363_332 = {S48, 0, &V18363_332_57338, &P18363_330};	/*Property = WebWorkers*/
const Value V18363_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18363_333 = {S49, 0, &V18363_333_57342, &P18363_332};	/*Property = Xhr2*/
const Profile P18363 = {"18363", &P18363_333};	/*Profile ID - 18363*/
const Value V18349_1_1 = {S2, NULL};	/*Value = True*/
const Property P18349_1 = {S50, 0, &V18349_1_1, NULL};	/*Property = IsMobile*/
const Value V18349_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18349_5 = {S52, 0, &V18349_5_1277, &P18349_1};	/*Property = ScreenPixelsHeight*/
const Value V18349_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18349_4 = {S54, 0, &V18349_4_1279, &P18349_5};	/*Property = ScreenPixelsWidth*/
const Profile P18349 = {"18349", &P18349_4};	/*Profile ID - 18349*/
const Value V20858_1_1 = {S2, NULL};	/*Value = True*/
const Property P20858_1 = {S50, 0, &V20858_1_1, NULL};	/*Property = IsMobile*/
const Value V20858_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20858_5 = {S52, 0, &V20858_5_1227, &P20858_1};	/*Property = ScreenPixelsHeight*/
const Value V20858_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20858_4 = {S54, 0, &V20858_4_1229, &P20858_5};	/*Property = ScreenPixelsWidth*/
const Profile P20858 = {"20858", &P20858_4};	/*Profile ID - 20858*/
const Value V18230_1_1 = {S2, NULL};	/*Value = True*/
const Property P18230_1 = {S50, 0, &V18230_1_1, NULL};	/*Property = IsMobile*/
const Value V18230_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P18230_5 = {S52, 0, &V18230_5_1458, &P18230_1};	/*Property = ScreenPixelsHeight*/
const Value V18230_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P18230_4 = {S54, 0, &V18230_4_1460, &P18230_5};	/*Property = ScreenPixelsWidth*/
const Profile P18230 = {"18230", &P18230_4};	/*Profile ID - 18230*/
const Value V18659_1_1 = {S2, NULL};	/*Value = True*/
const Property P18659_1 = {S50, 0, &V18659_1_1, NULL};	/*Property = IsMobile*/
const Value V18659_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P18659_5 = {S52, 0, &V18659_5_37, &P18659_1};	/*Property = ScreenPixelsHeight*/
const Value V18659_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P18659_4 = {S54, 0, &V18659_4_36, &P18659_5};	/*Property = ScreenPixelsWidth*/
const Profile P18659 = {"18659", &P18659_4};	/*Profile ID - 18659*/
const Value V18601_1_1 = {S2, NULL};	/*Value = True*/
const Property P18601_1 = {S50, 0, &V18601_1_1, NULL};	/*Property = IsMobile*/
const Value V18601_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18601_5 = {S52, 0, &V18601_5_1277, &P18601_1};	/*Property = ScreenPixelsHeight*/
const Value V18601_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18601_4 = {S54, 0, &V18601_4_1279, &P18601_5};	/*Property = ScreenPixelsWidth*/
const Profile P18601 = {"18601", &P18601_4};	/*Profile ID - 18601*/
const Value V23600_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_279 = {S1, 0, &V23600_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23600_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_282 = {S3, 0, &V23600_282_57155, &P23600_279};	/*Property = BlobBuilder*/
const Value V23600_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_350 = {S4, 0, &V23600_350_57446, &P23600_282};	/*Property = Canvas*/
const Value V23600_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_283 = {S5, 0, &V23600_283_57161, &P23600_350};	/*Property = CssBackground*/
const Value V23600_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_284 = {S6, 0, &V23600_284_57164, &P23600_283};	/*Property = CssBorderImage*/
const Value V23600_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_285 = {S7, 0, &V23600_285_57167, &P23600_284};	/*Property = CssCanvas*/
const Value V23600_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_286 = {S8, 0, &V23600_286_57170, &P23600_285};	/*Property = CssColor*/
const Value V23600_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_237 = {S9, 0, &V23600_237_55786, &P23600_286};	/*Property = CssColumn*/
const Value V23600_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_287 = {S10, 0, &V23600_287_57173, &P23600_237};	/*Property = CssFlexbox*/
const Value V23600_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_288 = {S11, 0, &V23600_288_57176, &P23600_287};	/*Property = CssFont*/
const Value V23600_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_289 = {S12, 0, &V23600_289_86899, &P23600_288};	/*Property = CssImages*/
const Value V23600_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_291 = {S13, 0, &V23600_291_57182, &P23600_289};	/*Property = CssMediaQueries*/
const Value V23600_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_292 = {S14, 0, &V23600_292_57185, &P23600_291};	/*Property = CssMinMax*/
const Value V23600_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_293 = {S15, 0, &V23600_293_57188, &P23600_292};	/*Property = CssOverflow*/
const Value V23600_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_294 = {S16, 0, &V23600_294_57191, &P23600_293};	/*Property = CssPosition*/
const Value V23600_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_295 = {S17, 0, &V23600_295_57194, &P23600_294};	/*Property = CssText*/
const Value V23600_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_297 = {S18, 0, &V23600_297_57197, &P23600_295};	/*Property = CssTransforms*/
const Value V23600_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_241 = {S19, 0, &V23600_241_55801, &P23600_297};	/*Property = CssTransitions*/
const Value V23600_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_298 = {S20, 0, &V23600_298_57208, &P23600_241};	/*Property = CssUI*/
const Value V23600_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_300 = {S21, 0, &V23600_300_57217, &P23600_298};	/*Property = DataSet*/
const Value V23600_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_301 = {S22, 0, &V23600_301_57220, &P23600_300};	/*Property = DataUrl*/
const Value V23600_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_302 = {S23, 0, &V23600_302_57223, &P23600_301};	/*Property = DeviceOrientation*/
const Value V23600_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_304 = {S24, 0, &V23600_304_57229, &P23600_302};	/*Property = FileReader*/
const Value V23600_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_305 = {S25, 0, &V23600_305_57232, &P23600_304};	/*Property = FileSaver*/
const Value V23600_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_306 = {S26, 0, &V23600_306_57238, &P23600_305};	/*Property = FileWriter*/
const Value V23600_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_307 = {S27, 0, &V23600_307_57241, &P23600_306};	/*Property = FormData*/
const Value V23600_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_309 = {S28, 0, &V23600_309_57247, &P23600_307};	/*Property = Fullscreen*/
const Value V23600_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_310 = {S29, 0, &V23600_310_57250, &P23600_309};	/*Property = GeoLocation*/
const Value V23600_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_312 = {S30, 0, &V23600_312_57256, &P23600_310};	/*Property = History*/
const Value V23600_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_314 = {S31, 0, &V23600_314_57262, &P23600_312};	/*Property = Html5*/
const Value V23600_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_313 = {S32, 0, &V23600_313_57259, &P23600_314};	/*Property = Html-Media-Capture*/
const Value V23600_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_315 = {S33, 0, &V23600_315_57265, &P23600_313};	/*Property = Iframe*/
const Value V23600_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_316 = {S34, 0, &V23600_316_57268, &P23600_315};	/*Property = IndexedDB*/
const Value V23600_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_317 = {S35, 0, &V23600_317_57271, &P23600_316};	/*Property = Json*/
const Value V23600_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P23600_15 = {S37, 0, &V23600_15_28, &P23600_317};	/*Property = LayoutEngine*/
const Value V23600_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_344 = {S38, 0, &V23600_344_57274, &P23600_15};	/*Property = Masking*/
const Value V23600_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_320 = {S39, 0, &V23600_320_57292, &P23600_344};	/*Property = PostMessage*/
const Value V23600_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_321 = {S40, 0, &V23600_321_57295, &P23600_320};	/*Property = Progress*/
const Value V23600_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_322 = {S41, 0, &V23600_322_57298, &P23600_321};	/*Property = Prompts*/
const Value V23600_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_323 = {S42, 0, &V23600_323_57301, &P23600_322};	/*Property = Selector*/
const Value V23600_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_324 = {S43, 0, &V23600_324_57307, &P23600_323};	/*Property = Svg*/
const Value V23600_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_327 = {S44, 0, &V23600_327_57316, &P23600_324};	/*Property = TouchEvents*/
const Value V23600_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_328 = {S45, 0, &V23600_328_57319, &P23600_327};	/*Property = Track*/
const Value V23600_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_329 = {S46, 0, &V23600_329_57325, &P23600_328};	/*Property = Video*/
const Value V23600_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_330 = {S47, 0, &V23600_330_57328, &P23600_329};	/*Property = Viewport*/
const Value V23600_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_332 = {S48, 0, &V23600_332_57340, &P23600_330};	/*Property = WebWorkers*/
const Value V23600_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23600_333 = {S49, 0, &V23600_333_57343, &P23600_332};	/*Property = Xhr2*/
const Profile P23600 = {"23600", &P23600_333};	/*Profile ID - 23600*/
const Value V20778_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_279 = {S1, 0, &V20778_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20778_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_282 = {S3, 0, &V20778_282_57155, &P20778_279};	/*Property = BlobBuilder*/
const Value V20778_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_350 = {S4, 0, &V20778_350_57446, &P20778_282};	/*Property = Canvas*/
const Value V20778_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_283 = {S5, 0, &V20778_283_57161, &P20778_350};	/*Property = CssBackground*/
const Value V20778_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_284 = {S6, 0, &V20778_284_57164, &P20778_283};	/*Property = CssBorderImage*/
const Value V20778_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_285 = {S7, 0, &V20778_285_57167, &P20778_284};	/*Property = CssCanvas*/
const Value V20778_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_286 = {S8, 0, &V20778_286_57170, &P20778_285};	/*Property = CssColor*/
const Value V20778_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_237 = {S9, 0, &V20778_237_55786, &P20778_286};	/*Property = CssColumn*/
const Value V20778_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_287 = {S10, 0, &V20778_287_57173, &P20778_237};	/*Property = CssFlexbox*/
const Value V20778_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_288 = {S11, 0, &V20778_288_57176, &P20778_287};	/*Property = CssFont*/
const Value V20778_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_289 = {S12, 0, &V20778_289_86899, &P20778_288};	/*Property = CssImages*/
const Value V20778_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_291 = {S13, 0, &V20778_291_57182, &P20778_289};	/*Property = CssMediaQueries*/
const Value V20778_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_292 = {S14, 0, &V20778_292_57185, &P20778_291};	/*Property = CssMinMax*/
const Value V20778_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_293 = {S15, 0, &V20778_293_57188, &P20778_292};	/*Property = CssOverflow*/
const Value V20778_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_294 = {S16, 0, &V20778_294_57191, &P20778_293};	/*Property = CssPosition*/
const Value V20778_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_295 = {S17, 0, &V20778_295_57194, &P20778_294};	/*Property = CssText*/
const Value V20778_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_297 = {S18, 0, &V20778_297_57197, &P20778_295};	/*Property = CssTransforms*/
const Value V20778_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_241 = {S19, 0, &V20778_241_55801, &P20778_297};	/*Property = CssTransitions*/
const Value V20778_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_298 = {S20, 0, &V20778_298_57208, &P20778_241};	/*Property = CssUI*/
const Value V20778_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_300 = {S21, 0, &V20778_300_57217, &P20778_298};	/*Property = DataSet*/
const Value V20778_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_301 = {S22, 0, &V20778_301_57220, &P20778_300};	/*Property = DataUrl*/
const Value V20778_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_302 = {S23, 0, &V20778_302_57223, &P20778_301};	/*Property = DeviceOrientation*/
const Value V20778_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_304 = {S24, 0, &V20778_304_57229, &P20778_302};	/*Property = FileReader*/
const Value V20778_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_305 = {S25, 0, &V20778_305_57232, &P20778_304};	/*Property = FileSaver*/
const Value V20778_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_306 = {S26, 0, &V20778_306_57238, &P20778_305};	/*Property = FileWriter*/
const Value V20778_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_307 = {S27, 0, &V20778_307_57241, &P20778_306};	/*Property = FormData*/
const Value V20778_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_309 = {S28, 0, &V20778_309_57247, &P20778_307};	/*Property = Fullscreen*/
const Value V20778_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_310 = {S29, 0, &V20778_310_57250, &P20778_309};	/*Property = GeoLocation*/
const Value V20778_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_312 = {S30, 0, &V20778_312_57256, &P20778_310};	/*Property = History*/
const Value V20778_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_314 = {S31, 0, &V20778_314_57262, &P20778_312};	/*Property = Html5*/
const Value V20778_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_313 = {S32, 0, &V20778_313_57259, &P20778_314};	/*Property = Html-Media-Capture*/
const Value V20778_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_315 = {S33, 0, &V20778_315_57265, &P20778_313};	/*Property = Iframe*/
const Value V20778_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_316 = {S34, 0, &V20778_316_57268, &P20778_315};	/*Property = IndexedDB*/
const Value V20778_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_317 = {S35, 0, &V20778_317_57271, &P20778_316};	/*Property = Json*/
const Value V20778_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P20778_15 = {S37, 0, &V20778_15_54167, &P20778_317};	/*Property = LayoutEngine*/
const Value V20778_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_344 = {S38, 0, &V20778_344_57274, &P20778_15};	/*Property = Masking*/
const Value V20778_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_320 = {S39, 0, &V20778_320_57292, &P20778_344};	/*Property = PostMessage*/
const Value V20778_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_321 = {S40, 0, &V20778_321_57295, &P20778_320};	/*Property = Progress*/
const Value V20778_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_322 = {S41, 0, &V20778_322_57298, &P20778_321};	/*Property = Prompts*/
const Value V20778_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_323 = {S42, 0, &V20778_323_57301, &P20778_322};	/*Property = Selector*/
const Value V20778_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_324 = {S43, 0, &V20778_324_57307, &P20778_323};	/*Property = Svg*/
const Value V20778_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_327 = {S44, 0, &V20778_327_57316, &P20778_324};	/*Property = TouchEvents*/
const Value V20778_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_328 = {S45, 0, &V20778_328_57319, &P20778_327};	/*Property = Track*/
const Value V20778_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_329 = {S46, 0, &V20778_329_57325, &P20778_328};	/*Property = Video*/
const Value V20778_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_330 = {S47, 0, &V20778_330_57328, &P20778_329};	/*Property = Viewport*/
const Value V20778_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_332 = {S48, 0, &V20778_332_57340, &P20778_330};	/*Property = WebWorkers*/
const Value V20778_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20778_333 = {S49, 0, &V20778_333_57343, &P20778_332};	/*Property = Xhr2*/
const Profile P20778 = {"20778", &P20778_333};	/*Profile ID - 20778*/
const Profile P18111 = {"18111", NULL};	/*Profile ID - 18111*/
const Value V18112_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18112_279 = {S1, 0, &V18112_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18112_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18112_282 = {S3, 0, &V18112_282_57154, &P18112_279};	/*Property = BlobBuilder*/
const Value V18112_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18112_350 = {S4, 0, &V18112_350_57444, &P18112_282};	/*Property = Canvas*/
const Value V18112_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18112_283 = {S5, 0, &V18112_283_57159, &P18112_350};	/*Property = CssBackground*/
const Value V18112_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18112_284 = {S6, 0, &V18112_284_57163, &P18112_283};	/*Property = CssBorderImage*/
const Value V18112_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18112_285 = {S7, 0, &V18112_285_57166, &P18112_284};	/*Property = CssCanvas*/
const Value V18112_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18112_286 = {S8, 0, &V18112_286_57168, &P18112_285};	/*Property = CssColor*/
const Value V18112_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18112_237 = {S9, 0, &V18112_237_55786, &P18112_286};	/*Property = CssColumn*/
const Value V18112_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18112_287 = {S10, 0, &V18112_287_57172, &P18112_237};	/*Property = CssFlexbox*/
const Value V18112_288_57174 = {S2, NULL};	/*Value = True*/
const Property P18112_288 = {S11, 0, &V18112_288_57174, &P18112_287};	/*Property = CssFont*/
const Value V18112_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18112_289 = {S12, 0, &V18112_289_57759, &P18112_288};	/*Property = CssImages*/
const Value V18112_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18112_291 = {S13, 0, &V18112_291_57180, &P18112_289};	/*Property = CssMediaQueries*/
const Value V18112_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18112_292 = {S14, 0, &V18112_292_57183, &P18112_291};	/*Property = CssMinMax*/
const Value V18112_293_57186 = {S2, NULL};	/*Value = True*/
const Property P18112_293 = {S15, 0, &V18112_293_57186, &P18112_292};	/*Property = CssOverflow*/
const Value V18112_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18112_294 = {S16, 0, &V18112_294_57189, &P18112_293};	/*Property = CssPosition*/
const Value V18112_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18112_295 = {S17, 0, &V18112_295_57192, &P18112_294};	/*Property = CssText*/
const Value V18112_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18112_297 = {S18, 0, &V18112_297_57195, &P18112_295};	/*Property = CssTransforms*/
const Value V18112_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18112_241 = {S19, 0, &V18112_241_55799, &P18112_297};	/*Property = CssTransitions*/
const Value V18112_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18112_298 = {S20, 0, &V18112_298_57206, &P18112_241};	/*Property = CssUI*/
const Value V18112_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18112_300 = {S21, 0, &V18112_300_57216, &P18112_298};	/*Property = DataSet*/
const Value V18112_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18112_301 = {S22, 0, &V18112_301_57218, &P18112_300};	/*Property = DataUrl*/
const Value V18112_302_57221 = {S2, NULL};	/*Value = True*/
const Property P18112_302 = {S23, 0, &V18112_302_57221, &P18112_301};	/*Property = DeviceOrientation*/
const Value V18112_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18112_304 = {S24, 0, &V18112_304_57227, &P18112_302};	/*Property = FileReader*/
const Value V18112_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18112_305 = {S25, 0, &V18112_305_57231, &P18112_304};	/*Property = FileSaver*/
const Value V18112_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18112_306 = {S26, 0, &V18112_306_57237, &P18112_305};	/*Property = FileWriter*/
const Value V18112_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18112_307 = {S27, 0, &V18112_307_57239, &P18112_306};	/*Property = FormData*/
const Value V18112_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18112_309 = {S28, 0, &V18112_309_57246, &P18112_307};	/*Property = Fullscreen*/
const Value V18112_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18112_310 = {S29, 0, &V18112_310_57248, &P18112_309};	/*Property = GeoLocation*/
const Value V18112_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18112_312 = {S30, 0, &V18112_312_57255, &P18112_310};	/*Property = History*/
const Value V18112_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18112_314 = {S31, 0, &V18112_314_57261, &P18112_312};	/*Property = Html5*/
const Value V18112_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18112_313 = {S32, 0, &V18112_313_57258, &P18112_314};	/*Property = Html-Media-Capture*/
const Value V18112_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18112_315 = {S33, 0, &V18112_315_57264, &P18112_313};	/*Property = Iframe*/
const Value V18112_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18112_316 = {S34, 0, &V18112_316_57267, &P18112_315};	/*Property = IndexedDB*/
const Value V18112_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18112_317 = {S35, 0, &V18112_317_57269, &P18112_316};	/*Property = Json*/
const Value V18112_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18112_15 = {S37, 0, &V18112_15_28, &P18112_317};	/*Property = LayoutEngine*/
const Value V18112_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18112_344 = {S38, 0, &V18112_344_57272, &P18112_15};	/*Property = Masking*/
const Value V18112_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18112_320 = {S39, 0, &V18112_320_57290, &P18112_344};	/*Property = PostMessage*/
const Value V18112_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18112_321 = {S40, 0, &V18112_321_57293, &P18112_320};	/*Property = Progress*/
const Value V18112_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18112_322 = {S41, 0, &V18112_322_57296, &P18112_321};	/*Property = Prompts*/
const Value V18112_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18112_323 = {S42, 0, &V18112_323_57299, &P18112_322};	/*Property = Selector*/
const Value V18112_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18112_324 = {S43, 0, &V18112_324_57306, &P18112_323};	/*Property = Svg*/
const Value V18112_327_57314 = {S2, NULL};	/*Value = True*/
const Property P18112_327 = {S44, 0, &V18112_327_57314, &P18112_324};	/*Property = TouchEvents*/
const Value V18112_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18112_328 = {S45, 0, &V18112_328_57318, &P18112_327};	/*Property = Track*/
const Value V18112_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18112_329 = {S46, 0, &V18112_329_57323, &P18112_328};	/*Property = Video*/
const Value V18112_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18112_330 = {S47, 0, &V18112_330_57326, &P18112_329};	/*Property = Viewport*/
const Value V18112_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18112_332 = {S48, 0, &V18112_332_57339, &P18112_330};	/*Property = WebWorkers*/
const Value V18112_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18112_333 = {S49, 0, &V18112_333_57342, &P18112_332};	/*Property = Xhr2*/
const Profile P18112 = {"18112", &P18112_333};	/*Profile ID - 18112*/
const Value V21225_1_1 = {S2, NULL};	/*Value = True*/
const Property P21225_1 = {S50, 0, &V21225_1_1, NULL};	/*Property = IsMobile*/
const Value V21225_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21225_5 = {S52, 0, &V21225_5_1403, &P21225_1};	/*Property = ScreenPixelsHeight*/
const Value V21225_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21225_4 = {S54, 0, &V21225_4_1404, &P21225_5};	/*Property = ScreenPixelsWidth*/
const Profile P21225 = {"21225", &P21225_4};	/*Profile ID - 21225*/
const Value V18102_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18102_279 = {S1, 0, &V18102_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18102_282_57153 = {S2, NULL};	/*Value = True*/
const Property P18102_282 = {S3, 0, &V18102_282_57153, &P18102_279};	/*Property = BlobBuilder*/
const Value V18102_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18102_350 = {S4, 0, &V18102_350_57444, &P18102_282};	/*Property = Canvas*/
const Value V18102_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18102_283 = {S5, 0, &V18102_283_57159, &P18102_350};	/*Property = CssBackground*/
const Value V18102_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18102_284 = {S6, 0, &V18102_284_57163, &P18102_283};	/*Property = CssBorderImage*/
const Value V18102_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18102_285 = {S7, 0, &V18102_285_57166, &P18102_284};	/*Property = CssCanvas*/
const Value V18102_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18102_286 = {S8, 0, &V18102_286_57168, &P18102_285};	/*Property = CssColor*/
const Value V18102_237_55784 = {S2, NULL};	/*Value = True*/
const Property P18102_237 = {S9, 0, &V18102_237_55784, &P18102_286};	/*Property = CssColumn*/
const Value V18102_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18102_287 = {S10, 0, &V18102_287_57172, &P18102_237};	/*Property = CssFlexbox*/
const Value V18102_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18102_288 = {S11, 0, &V18102_288_57175, &P18102_287};	/*Property = CssFont*/
const Value V18102_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18102_289 = {S12, 0, &V18102_289_57759, &P18102_288};	/*Property = CssImages*/
const Value V18102_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18102_291 = {S13, 0, &V18102_291_57180, &P18102_289};	/*Property = CssMediaQueries*/
const Value V18102_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18102_292 = {S14, 0, &V18102_292_57183, &P18102_291};	/*Property = CssMinMax*/
const Value V18102_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18102_293 = {S15, 0, &V18102_293_57187, &P18102_292};	/*Property = CssOverflow*/
const Value V18102_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18102_294 = {S16, 0, &V18102_294_57189, &P18102_293};	/*Property = CssPosition*/
const Value V18102_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18102_295 = {S17, 0, &V18102_295_57192, &P18102_294};	/*Property = CssText*/
const Value V18102_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18102_297 = {S18, 0, &V18102_297_57195, &P18102_295};	/*Property = CssTransforms*/
const Value V18102_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18102_241 = {S19, 0, &V18102_241_55799, &P18102_297};	/*Property = CssTransitions*/
const Value V18102_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18102_298 = {S20, 0, &V18102_298_57206, &P18102_241};	/*Property = CssUI*/
const Value V18102_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18102_300 = {S21, 0, &V18102_300_57216, &P18102_298};	/*Property = DataSet*/
const Value V18102_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18102_301 = {S22, 0, &V18102_301_57218, &P18102_300};	/*Property = DataUrl*/
const Value V18102_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18102_302 = {S23, 0, &V18102_302_57222, &P18102_301};	/*Property = DeviceOrientation*/
const Value V18102_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18102_304 = {S24, 0, &V18102_304_57227, &P18102_302};	/*Property = FileReader*/
const Value V18102_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18102_305 = {S25, 0, &V18102_305_57231, &P18102_304};	/*Property = FileSaver*/
const Value V18102_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18102_306 = {S26, 0, &V18102_306_57237, &P18102_305};	/*Property = FileWriter*/
const Value V18102_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18102_307 = {S27, 0, &V18102_307_57239, &P18102_306};	/*Property = FormData*/
const Value V18102_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18102_309 = {S28, 0, &V18102_309_57246, &P18102_307};	/*Property = Fullscreen*/
const Value V18102_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18102_310 = {S29, 0, &V18102_310_57248, &P18102_309};	/*Property = GeoLocation*/
const Value V18102_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18102_312 = {S30, 0, &V18102_312_57254, &P18102_310};	/*Property = History*/
const Value V18102_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18102_314 = {S31, 0, &V18102_314_57261, &P18102_312};	/*Property = Html5*/
const Value V18102_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18102_313 = {S32, 0, &V18102_313_57258, &P18102_314};	/*Property = Html-Media-Capture*/
const Value V18102_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18102_315 = {S33, 0, &V18102_315_57264, &P18102_313};	/*Property = Iframe*/
const Value V18102_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18102_316 = {S34, 0, &V18102_316_57266, &P18102_315};	/*Property = IndexedDB*/
const Value V18102_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18102_317 = {S35, 0, &V18102_317_57269, &P18102_316};	/*Property = Json*/
const Value V18102_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18102_15 = {S37, 0, &V18102_15_28, &P18102_317};	/*Property = LayoutEngine*/
const Value V18102_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18102_344 = {S38, 0, &V18102_344_57272, &P18102_15};	/*Property = Masking*/
const Value V18102_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18102_320 = {S39, 0, &V18102_320_57290, &P18102_344};	/*Property = PostMessage*/
const Value V18102_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18102_321 = {S40, 0, &V18102_321_57293, &P18102_320};	/*Property = Progress*/
const Value V18102_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18102_322 = {S41, 0, &V18102_322_57296, &P18102_321};	/*Property = Prompts*/
const Value V18102_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18102_323 = {S42, 0, &V18102_323_57299, &P18102_322};	/*Property = Selector*/
const Value V18102_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18102_324 = {S43, 0, &V18102_324_57306, &P18102_323};	/*Property = Svg*/
const Value V18102_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18102_327 = {S44, 0, &V18102_327_57315, &P18102_324};	/*Property = TouchEvents*/
const Value V18102_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18102_328 = {S45, 0, &V18102_328_57318, &P18102_327};	/*Property = Track*/
const Value V18102_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18102_329 = {S46, 0, &V18102_329_57323, &P18102_328};	/*Property = Video*/
const Value V18102_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18102_330 = {S47, 0, &V18102_330_57326, &P18102_329};	/*Property = Viewport*/
const Value V18102_332_57338 = {S2, NULL};	/*Value = True*/
const Property P18102_332 = {S48, 0, &V18102_332_57338, &P18102_330};	/*Property = WebWorkers*/
const Value V18102_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18102_333 = {S49, 0, &V18102_333_57342, &P18102_332};	/*Property = Xhr2*/
const Profile P18102 = {"18102", &P18102_333};	/*Profile ID - 18102*/
const Value V18290_1_1 = {S2, NULL};	/*Value = True*/
const Property P18290_1 = {S50, 0, &V18290_1_1, NULL};	/*Property = IsMobile*/
const Value V18290_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18290_5 = {S52, 0, &V18290_5_9981, &P18290_1};	/*Property = ScreenPixelsHeight*/
const Value V18290_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18290_4 = {S54, 0, &V18290_4_2336, &P18290_5};	/*Property = ScreenPixelsWidth*/
const Profile P18290 = {"18290", &P18290_4};	/*Profile ID - 18290*/
const Value V18292_1_1 = {S2, NULL};	/*Value = True*/
const Property P18292_1 = {S50, 0, &V18292_1_1, NULL};	/*Property = IsMobile*/
const Value V18292_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18292_5 = {S52, 0, &V18292_5_9981, &P18292_1};	/*Property = ScreenPixelsHeight*/
const Value V18292_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18292_4 = {S54, 0, &V18292_4_2336, &P18292_5};	/*Property = ScreenPixelsWidth*/
const Profile P18292 = {"18292", &P18292_4};	/*Profile ID - 18292*/
const Value V19784_1_1 = {S2, NULL};	/*Value = True*/
const Property P19784_1 = {S50, 0, &V19784_1_1, NULL};	/*Property = IsMobile*/
const Value V19784_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19784_5 = {S52, 0, &V19784_5_9981, &P19784_1};	/*Property = ScreenPixelsHeight*/
const Value V19784_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P19784_4 = {S54, 0, &V19784_4_2336, &P19784_5};	/*Property = ScreenPixelsWidth*/
const Profile P19784 = {"19784", &P19784_4};	/*Profile ID - 19784*/
const Value V22890_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22890_279 = {S1, 0, &V22890_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22890_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22890_282 = {S3, 0, &V22890_282_57154, &P22890_279};	/*Property = BlobBuilder*/
const Value V22890_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22890_350 = {S4, 0, &V22890_350_57444, &P22890_282};	/*Property = Canvas*/
const Value V22890_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22890_283 = {S5, 0, &V22890_283_57159, &P22890_350};	/*Property = CssBackground*/
const Value V22890_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22890_284 = {S6, 0, &V22890_284_57163, &P22890_283};	/*Property = CssBorderImage*/
const Value V22890_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22890_285 = {S7, 0, &V22890_285_57166, &P22890_284};	/*Property = CssCanvas*/
const Value V22890_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22890_286 = {S8, 0, &V22890_286_57168, &P22890_285};	/*Property = CssColor*/
const Value V22890_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22890_237 = {S9, 0, &V22890_237_55786, &P22890_286};	/*Property = CssColumn*/
const Value V22890_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22890_287 = {S10, 0, &V22890_287_57172, &P22890_237};	/*Property = CssFlexbox*/
const Value V22890_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22890_288 = {S11, 0, &V22890_288_57175, &P22890_287};	/*Property = CssFont*/
const Value V22890_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22890_289 = {S12, 0, &V22890_289_57759, &P22890_288};	/*Property = CssImages*/
const Value V22890_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22890_291 = {S13, 0, &V22890_291_57180, &P22890_289};	/*Property = CssMediaQueries*/
const Value V22890_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22890_292 = {S14, 0, &V22890_292_57183, &P22890_291};	/*Property = CssMinMax*/
const Value V22890_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22890_293 = {S15, 0, &V22890_293_57187, &P22890_292};	/*Property = CssOverflow*/
const Value V22890_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22890_294 = {S16, 0, &V22890_294_57189, &P22890_293};	/*Property = CssPosition*/
const Value V22890_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22890_295 = {S17, 0, &V22890_295_57192, &P22890_294};	/*Property = CssText*/
const Value V22890_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22890_297 = {S18, 0, &V22890_297_57195, &P22890_295};	/*Property = CssTransforms*/
const Value V22890_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22890_241 = {S19, 0, &V22890_241_55799, &P22890_297};	/*Property = CssTransitions*/
const Value V22890_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22890_298 = {S20, 0, &V22890_298_57206, &P22890_241};	/*Property = CssUI*/
const Value V22890_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22890_300 = {S21, 0, &V22890_300_57216, &P22890_298};	/*Property = DataSet*/
const Value V22890_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22890_301 = {S22, 0, &V22890_301_57218, &P22890_300};	/*Property = DataUrl*/
const Value V22890_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22890_302 = {S23, 0, &V22890_302_57222, &P22890_301};	/*Property = DeviceOrientation*/
const Value V22890_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22890_304 = {S24, 0, &V22890_304_57227, &P22890_302};	/*Property = FileReader*/
const Value V22890_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22890_305 = {S25, 0, &V22890_305_57231, &P22890_304};	/*Property = FileSaver*/
const Value V22890_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22890_306 = {S26, 0, &V22890_306_57237, &P22890_305};	/*Property = FileWriter*/
const Value V22890_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22890_307 = {S27, 0, &V22890_307_57239, &P22890_306};	/*Property = FormData*/
const Value V22890_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22890_309 = {S28, 0, &V22890_309_57246, &P22890_307};	/*Property = Fullscreen*/
const Value V22890_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22890_310 = {S29, 0, &V22890_310_57248, &P22890_309};	/*Property = GeoLocation*/
const Value V22890_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22890_312 = {S30, 0, &V22890_312_57254, &P22890_310};	/*Property = History*/
const Value V22890_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22890_314 = {S31, 0, &V22890_314_57261, &P22890_312};	/*Property = Html5*/
const Value V22890_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22890_313 = {S32, 0, &V22890_313_57258, &P22890_314};	/*Property = Html-Media-Capture*/
const Value V22890_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22890_315 = {S33, 0, &V22890_315_57264, &P22890_313};	/*Property = Iframe*/
const Value V22890_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22890_316 = {S34, 0, &V22890_316_57267, &P22890_315};	/*Property = IndexedDB*/
const Value V22890_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22890_317 = {S35, 0, &V22890_317_57269, &P22890_316};	/*Property = Json*/
const Value V22890_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22890_15 = {S37, 0, &V22890_15_72, &P22890_317};	/*Property = LayoutEngine*/
const Value V22890_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22890_344 = {S38, 0, &V22890_344_57272, &P22890_15};	/*Property = Masking*/
const Value V22890_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22890_320 = {S39, 0, &V22890_320_57290, &P22890_344};	/*Property = PostMessage*/
const Value V22890_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22890_321 = {S40, 0, &V22890_321_57293, &P22890_320};	/*Property = Progress*/
const Value V22890_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22890_322 = {S41, 0, &V22890_322_57296, &P22890_321};	/*Property = Prompts*/
const Value V22890_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22890_323 = {S42, 0, &V22890_323_57299, &P22890_322};	/*Property = Selector*/
const Value V22890_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22890_324 = {S43, 0, &V22890_324_57306, &P22890_323};	/*Property = Svg*/
const Value V22890_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22890_327 = {S44, 0, &V22890_327_57315, &P22890_324};	/*Property = TouchEvents*/
const Value V22890_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22890_328 = {S45, 0, &V22890_328_57318, &P22890_327};	/*Property = Track*/
const Value V22890_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22890_329 = {S46, 0, &V22890_329_57323, &P22890_328};	/*Property = Video*/
const Value V22890_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22890_330 = {S47, 0, &V22890_330_57326, &P22890_329};	/*Property = Viewport*/
const Value V22890_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22890_332 = {S48, 0, &V22890_332_57338, &P22890_330};	/*Property = WebWorkers*/
const Value V22890_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22890_333 = {S49, 0, &V22890_333_57342, &P22890_332};	/*Property = Xhr2*/
const Profile P22890 = {"22890", &P22890_333};	/*Profile ID - 22890*/
const Value V18072_1_1 = {S2, NULL};	/*Value = True*/
const Property P18072_1 = {S50, 0, &V18072_1_1, NULL};	/*Property = IsMobile*/
const Value V18072_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P18072_5 = {S52, 0, &V18072_5_1458, &P18072_1};	/*Property = ScreenPixelsHeight*/
const Value V18072_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18072_4 = {S54, 0, &V18072_4_39098, &P18072_5};	/*Property = ScreenPixelsWidth*/
const Profile P18072 = {"18072", &P18072_4};	/*Profile ID - 18072*/
const Value V24745_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24745_279 = {S1, 0, &V24745_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24745_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24745_282 = {S3, 0, &V24745_282_57154, &P24745_279};	/*Property = BlobBuilder*/
const Value V24745_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24745_350 = {S4, 0, &V24745_350_57444, &P24745_282};	/*Property = Canvas*/
const Value V24745_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24745_283 = {S5, 0, &V24745_283_57159, &P24745_350};	/*Property = CssBackground*/
const Value V24745_284_57163 = {S0, NULL};	/*Value = False*/
const Property P24745_284 = {S6, 0, &V24745_284_57163, &P24745_283};	/*Property = CssBorderImage*/
const Value V24745_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24745_285 = {S7, 0, &V24745_285_57166, &P24745_284};	/*Property = CssCanvas*/
const Value V24745_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24745_286 = {S8, 0, &V24745_286_57168, &P24745_285};	/*Property = CssColor*/
const Value V24745_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24745_287 = {S10, 0, &V24745_287_57172, &P24745_286};	/*Property = CssFlexbox*/
const Value V24745_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24745_288 = {S11, 0, &V24745_288_57175, &P24745_287};	/*Property = CssFont*/
const Value V24745_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24745_289 = {S12, 0, &V24745_289_57759, &P24745_288};	/*Property = CssImages*/
const Value V24745_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24745_291 = {S13, 0, &V24745_291_57180, &P24745_289};	/*Property = CssMediaQueries*/
const Value V24745_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24745_292 = {S14, 0, &V24745_292_57183, &P24745_291};	/*Property = CssMinMax*/
const Value V24745_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24745_293 = {S15, 0, &V24745_293_57187, &P24745_292};	/*Property = CssOverflow*/
const Value V24745_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24745_294 = {S16, 0, &V24745_294_57189, &P24745_293};	/*Property = CssPosition*/
const Value V24745_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24745_295 = {S17, 0, &V24745_295_57192, &P24745_294};	/*Property = CssText*/
const Value V24745_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24745_297 = {S18, 0, &V24745_297_57195, &P24745_295};	/*Property = CssTransforms*/
const Value V24745_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24745_241 = {S19, 0, &V24745_241_55799, &P24745_297};	/*Property = CssTransitions*/
const Value V24745_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24745_298 = {S20, 0, &V24745_298_57206, &P24745_241};	/*Property = CssUI*/
const Value V24745_300_57216 = {S0, NULL};	/*Value = False*/
const Property P24745_300 = {S21, 0, &V24745_300_57216, &P24745_298};	/*Property = DataSet*/
const Value V24745_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24745_301 = {S22, 0, &V24745_301_57218, &P24745_300};	/*Property = DataUrl*/
const Value V24745_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24745_302 = {S23, 0, &V24745_302_57222, &P24745_301};	/*Property = DeviceOrientation*/
const Value V24745_304_57228 = {S0, NULL};	/*Value = False*/
const Property P24745_304 = {S24, 0, &V24745_304_57228, &P24745_302};	/*Property = FileReader*/
const Value V24745_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24745_305 = {S25, 0, &V24745_305_57231, &P24745_304};	/*Property = FileSaver*/
const Value V24745_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24745_306 = {S26, 0, &V24745_306_57237, &P24745_305};	/*Property = FileWriter*/
const Value V24745_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24745_307 = {S27, 0, &V24745_307_57239, &P24745_306};	/*Property = FormData*/
const Value V24745_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24745_309 = {S28, 0, &V24745_309_57246, &P24745_307};	/*Property = Fullscreen*/
const Value V24745_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24745_310 = {S29, 0, &V24745_310_57248, &P24745_309};	/*Property = GeoLocation*/
const Value V24745_312_57255 = {S0, NULL};	/*Value = False*/
const Property P24745_312 = {S30, 0, &V24745_312_57255, &P24745_310};	/*Property = History*/
const Value V24745_314_57261 = {S0, NULL};	/*Value = False*/
const Property P24745_314 = {S31, 0, &V24745_314_57261, &P24745_312};	/*Property = Html5*/
const Value V24745_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24745_313 = {S32, 0, &V24745_313_57258, &P24745_314};	/*Property = Html-Media-Capture*/
const Value V24745_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24745_315 = {S33, 0, &V24745_315_57264, &P24745_313};	/*Property = Iframe*/
const Value V24745_316_57267 = {S0, NULL};	/*Value = False*/
const Property P24745_316 = {S34, 0, &V24745_316_57267, &P24745_315};	/*Property = IndexedDB*/
const Value V24745_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24745_317 = {S35, 0, &V24745_317_57269, &P24745_316};	/*Property = Json*/
const Value V24745_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24745_15 = {S37, 0, &V24745_15_28, &P24745_317};	/*Property = LayoutEngine*/
const Value V24745_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24745_344 = {S38, 0, &V24745_344_57272, &P24745_15};	/*Property = Masking*/
const Value V24745_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24745_320 = {S39, 0, &V24745_320_57290, &P24745_344};	/*Property = PostMessage*/
const Value V24745_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24745_321 = {S40, 0, &V24745_321_57293, &P24745_320};	/*Property = Progress*/
const Value V24745_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24745_322 = {S41, 0, &V24745_322_57296, &P24745_321};	/*Property = Prompts*/
const Value V24745_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24745_323 = {S42, 0, &V24745_323_57299, &P24745_322};	/*Property = Selector*/
const Value V24745_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24745_324 = {S43, 0, &V24745_324_57306, &P24745_323};	/*Property = Svg*/
const Value V24745_327_57315 = {S0, NULL};	/*Value = False*/
const Property P24745_327 = {S44, 0, &V24745_327_57315, &P24745_324};	/*Property = TouchEvents*/
const Value V24745_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24745_328 = {S45, 0, &V24745_328_57318, &P24745_327};	/*Property = Track*/
const Value V24745_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24745_329 = {S46, 0, &V24745_329_57323, &P24745_328};	/*Property = Video*/
const Value V24745_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24745_330 = {S47, 0, &V24745_330_57326, &P24745_329};	/*Property = Viewport*/
const Value V24745_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24745_332 = {S48, 0, &V24745_332_57339, &P24745_330};	/*Property = WebWorkers*/
const Value V24745_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24745_333 = {S49, 0, &V24745_333_57342, &P24745_332};	/*Property = Xhr2*/
const Profile P24745 = {"24745", &P24745_333};	/*Profile ID - 24745*/
const Value V16542_1_1 = {S2, NULL};	/*Value = True*/
const Property P16542_1 = {S50, 0, &V16542_1_1, NULL};	/*Property = IsMobile*/
const Value V16542_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16542_5 = {S52, 0, &V16542_5_1222, &P16542_1};	/*Property = ScreenPixelsHeight*/
const Value V16542_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16542_4 = {S54, 0, &V16542_4_1224, &P16542_5};	/*Property = ScreenPixelsWidth*/
const Profile P16542 = {"16542", &P16542_4};	/*Profile ID - 16542*/
const Value V24200_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24200_279 = {S1, 0, &V24200_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24200_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24200_282 = {S3, 0, &V24200_282_57154, &P24200_279};	/*Property = BlobBuilder*/
const Value V24200_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24200_350 = {S4, 0, &V24200_350_57444, &P24200_282};	/*Property = Canvas*/
const Value V24200_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24200_283 = {S5, 0, &V24200_283_57159, &P24200_350};	/*Property = CssBackground*/
const Value V24200_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24200_284 = {S6, 0, &V24200_284_57162, &P24200_283};	/*Property = CssBorderImage*/
const Value V24200_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24200_285 = {S7, 0, &V24200_285_57166, &P24200_284};	/*Property = CssCanvas*/
const Value V24200_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24200_286 = {S8, 0, &V24200_286_57168, &P24200_285};	/*Property = CssColor*/
const Value V24200_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24200_237 = {S9, 0, &V24200_237_55786, &P24200_286};	/*Property = CssColumn*/
const Value V24200_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24200_287 = {S10, 0, &V24200_287_57172, &P24200_237};	/*Property = CssFlexbox*/
const Value V24200_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24200_288 = {S11, 0, &V24200_288_57175, &P24200_287};	/*Property = CssFont*/
const Value V24200_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24200_289 = {S12, 0, &V24200_289_57759, &P24200_288};	/*Property = CssImages*/
const Value V24200_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24200_291 = {S13, 0, &V24200_291_57180, &P24200_289};	/*Property = CssMediaQueries*/
const Value V24200_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24200_292 = {S14, 0, &V24200_292_57183, &P24200_291};	/*Property = CssMinMax*/
const Value V24200_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24200_293 = {S15, 0, &V24200_293_57187, &P24200_292};	/*Property = CssOverflow*/
const Value V24200_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24200_294 = {S16, 0, &V24200_294_57189, &P24200_293};	/*Property = CssPosition*/
const Value V24200_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24200_295 = {S17, 0, &V24200_295_57192, &P24200_294};	/*Property = CssText*/
const Value V24200_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24200_297 = {S18, 0, &V24200_297_57195, &P24200_295};	/*Property = CssTransforms*/
const Value V24200_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24200_241 = {S19, 0, &V24200_241_55799, &P24200_297};	/*Property = CssTransitions*/
const Value V24200_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24200_298 = {S20, 0, &V24200_298_57206, &P24200_241};	/*Property = CssUI*/
const Value V24200_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24200_300 = {S21, 0, &V24200_300_57215, &P24200_298};	/*Property = DataSet*/
const Value V24200_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24200_301 = {S22, 0, &V24200_301_57218, &P24200_300};	/*Property = DataUrl*/
const Value V24200_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24200_302 = {S23, 0, &V24200_302_57221, &P24200_301};	/*Property = DeviceOrientation*/
const Value V24200_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24200_304 = {S24, 0, &V24200_304_57227, &P24200_302};	/*Property = FileReader*/
const Value V24200_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24200_305 = {S25, 0, &V24200_305_57231, &P24200_304};	/*Property = FileSaver*/
const Value V24200_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24200_306 = {S26, 0, &V24200_306_57237, &P24200_305};	/*Property = FileWriter*/
const Value V24200_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24200_307 = {S27, 0, &V24200_307_57239, &P24200_306};	/*Property = FormData*/
const Value V24200_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24200_309 = {S28, 0, &V24200_309_57246, &P24200_307};	/*Property = Fullscreen*/
const Value V24200_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24200_310 = {S29, 0, &V24200_310_57248, &P24200_309};	/*Property = GeoLocation*/
const Value V24200_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24200_312 = {S30, 0, &V24200_312_57254, &P24200_310};	/*Property = History*/
const Value V24200_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24200_314 = {S31, 0, &V24200_314_57260, &P24200_312};	/*Property = Html5*/
const Value V24200_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24200_313 = {S32, 0, &V24200_313_57257, &P24200_314};	/*Property = Html-Media-Capture*/
const Value V24200_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24200_315 = {S33, 0, &V24200_315_57264, &P24200_313};	/*Property = Iframe*/
const Value V24200_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24200_316 = {S34, 0, &V24200_316_57266, &P24200_315};	/*Property = IndexedDB*/
const Value V24200_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24200_317 = {S35, 0, &V24200_317_57269, &P24200_316};	/*Property = Json*/
const Value V24200_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24200_15 = {S37, 0, &V24200_15_54167, &P24200_317};	/*Property = LayoutEngine*/
const Value V24200_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24200_344 = {S38, 0, &V24200_344_57272, &P24200_15};	/*Property = Masking*/
const Value V24200_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24200_320 = {S39, 0, &V24200_320_57290, &P24200_344};	/*Property = PostMessage*/
const Value V24200_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24200_321 = {S40, 0, &V24200_321_57293, &P24200_320};	/*Property = Progress*/
const Value V24200_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24200_322 = {S41, 0, &V24200_322_57296, &P24200_321};	/*Property = Prompts*/
const Value V24200_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24200_323 = {S42, 0, &V24200_323_57299, &P24200_322};	/*Property = Selector*/
const Value V24200_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24200_324 = {S43, 0, &V24200_324_57306, &P24200_323};	/*Property = Svg*/
const Value V24200_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24200_327 = {S44, 0, &V24200_327_57314, &P24200_324};	/*Property = TouchEvents*/
const Value V24200_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24200_328 = {S45, 0, &V24200_328_57318, &P24200_327};	/*Property = Track*/
const Value V24200_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24200_329 = {S46, 0, &V24200_329_57323, &P24200_328};	/*Property = Video*/
const Value V24200_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24200_330 = {S47, 0, &V24200_330_57326, &P24200_329};	/*Property = Viewport*/
const Value V24200_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24200_332 = {S48, 0, &V24200_332_57339, &P24200_330};	/*Property = WebWorkers*/
const Value V24200_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24200_333 = {S49, 0, &V24200_333_57342, &P24200_332};	/*Property = Xhr2*/
const Profile P24200 = {"24200", &P24200_333};	/*Profile ID - 24200*/
const Value V18202_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_279 = {S1, 0, &V18202_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18202_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_282 = {S3, 0, &V18202_282_57155, &P18202_279};	/*Property = BlobBuilder*/
const Value V18202_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_350 = {S4, 0, &V18202_350_57446, &P18202_282};	/*Property = Canvas*/
const Value V18202_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_283 = {S5, 0, &V18202_283_57161, &P18202_350};	/*Property = CssBackground*/
const Value V18202_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_284 = {S6, 0, &V18202_284_57164, &P18202_283};	/*Property = CssBorderImage*/
const Value V18202_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_285 = {S7, 0, &V18202_285_57167, &P18202_284};	/*Property = CssCanvas*/
const Value V18202_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_286 = {S8, 0, &V18202_286_57170, &P18202_285};	/*Property = CssColor*/
const Value V18202_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_237 = {S9, 0, &V18202_237_55786, &P18202_286};	/*Property = CssColumn*/
const Value V18202_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_287 = {S10, 0, &V18202_287_57173, &P18202_237};	/*Property = CssFlexbox*/
const Value V18202_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_288 = {S11, 0, &V18202_288_57176, &P18202_287};	/*Property = CssFont*/
const Value V18202_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_289 = {S12, 0, &V18202_289_86899, &P18202_288};	/*Property = CssImages*/
const Value V18202_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_291 = {S13, 0, &V18202_291_57182, &P18202_289};	/*Property = CssMediaQueries*/
const Value V18202_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_292 = {S14, 0, &V18202_292_57185, &P18202_291};	/*Property = CssMinMax*/
const Value V18202_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_293 = {S15, 0, &V18202_293_57188, &P18202_292};	/*Property = CssOverflow*/
const Value V18202_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_294 = {S16, 0, &V18202_294_57191, &P18202_293};	/*Property = CssPosition*/
const Value V18202_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_295 = {S17, 0, &V18202_295_57194, &P18202_294};	/*Property = CssText*/
const Value V18202_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_297 = {S18, 0, &V18202_297_57197, &P18202_295};	/*Property = CssTransforms*/
const Value V18202_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_241 = {S19, 0, &V18202_241_55801, &P18202_297};	/*Property = CssTransitions*/
const Value V18202_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_298 = {S20, 0, &V18202_298_57208, &P18202_241};	/*Property = CssUI*/
const Value V18202_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_300 = {S21, 0, &V18202_300_57217, &P18202_298};	/*Property = DataSet*/
const Value V18202_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_301 = {S22, 0, &V18202_301_57220, &P18202_300};	/*Property = DataUrl*/
const Value V18202_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_302 = {S23, 0, &V18202_302_57223, &P18202_301};	/*Property = DeviceOrientation*/
const Value V18202_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_304 = {S24, 0, &V18202_304_57229, &P18202_302};	/*Property = FileReader*/
const Value V18202_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_305 = {S25, 0, &V18202_305_57232, &P18202_304};	/*Property = FileSaver*/
const Value V18202_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_306 = {S26, 0, &V18202_306_57238, &P18202_305};	/*Property = FileWriter*/
const Value V18202_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_307 = {S27, 0, &V18202_307_57241, &P18202_306};	/*Property = FormData*/
const Value V18202_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_309 = {S28, 0, &V18202_309_57247, &P18202_307};	/*Property = Fullscreen*/
const Value V18202_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_310 = {S29, 0, &V18202_310_57250, &P18202_309};	/*Property = GeoLocation*/
const Value V18202_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_312 = {S30, 0, &V18202_312_57256, &P18202_310};	/*Property = History*/
const Value V18202_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_314 = {S31, 0, &V18202_314_57262, &P18202_312};	/*Property = Html5*/
const Value V18202_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_313 = {S32, 0, &V18202_313_57259, &P18202_314};	/*Property = Html-Media-Capture*/
const Value V18202_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_315 = {S33, 0, &V18202_315_57265, &P18202_313};	/*Property = Iframe*/
const Value V18202_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_316 = {S34, 0, &V18202_316_57268, &P18202_315};	/*Property = IndexedDB*/
const Value V18202_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_317 = {S35, 0, &V18202_317_57271, &P18202_316};	/*Property = Json*/
const Value V18202_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18202_15 = {S37, 0, &V18202_15_28, &P18202_317};	/*Property = LayoutEngine*/
const Value V18202_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_344 = {S38, 0, &V18202_344_57274, &P18202_15};	/*Property = Masking*/
const Value V18202_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_320 = {S39, 0, &V18202_320_57292, &P18202_344};	/*Property = PostMessage*/
const Value V18202_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_321 = {S40, 0, &V18202_321_57295, &P18202_320};	/*Property = Progress*/
const Value V18202_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_322 = {S41, 0, &V18202_322_57298, &P18202_321};	/*Property = Prompts*/
const Value V18202_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_323 = {S42, 0, &V18202_323_57301, &P18202_322};	/*Property = Selector*/
const Value V18202_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_324 = {S43, 0, &V18202_324_57307, &P18202_323};	/*Property = Svg*/
const Value V18202_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_327 = {S44, 0, &V18202_327_57316, &P18202_324};	/*Property = TouchEvents*/
const Value V18202_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_328 = {S45, 0, &V18202_328_57319, &P18202_327};	/*Property = Track*/
const Value V18202_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_329 = {S46, 0, &V18202_329_57325, &P18202_328};	/*Property = Video*/
const Value V18202_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_330 = {S47, 0, &V18202_330_57328, &P18202_329};	/*Property = Viewport*/
const Value V18202_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_332 = {S48, 0, &V18202_332_57340, &P18202_330};	/*Property = WebWorkers*/
const Value V18202_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18202_333 = {S49, 0, &V18202_333_57343, &P18202_332};	/*Property = Xhr2*/
const Profile P18202 = {"18202", &P18202_333};	/*Profile ID - 18202*/
const Value V4860_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_279 = {S1, 0, &V4860_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4860_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_282 = {S3, 0, &V4860_282_57155, &P4860_279};	/*Property = BlobBuilder*/
const Value V4860_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_350 = {S4, 0, &V4860_350_57446, &P4860_282};	/*Property = Canvas*/
const Value V4860_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_283 = {S5, 0, &V4860_283_57161, &P4860_350};	/*Property = CssBackground*/
const Value V4860_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_284 = {S6, 0, &V4860_284_57164, &P4860_283};	/*Property = CssBorderImage*/
const Value V4860_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_285 = {S7, 0, &V4860_285_57167, &P4860_284};	/*Property = CssCanvas*/
const Value V4860_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_286 = {S8, 0, &V4860_286_57170, &P4860_285};	/*Property = CssColor*/
const Value V4860_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_237 = {S9, 0, &V4860_237_55786, &P4860_286};	/*Property = CssColumn*/
const Value V4860_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_287 = {S10, 0, &V4860_287_57173, &P4860_237};	/*Property = CssFlexbox*/
const Value V4860_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_288 = {S11, 0, &V4860_288_57176, &P4860_287};	/*Property = CssFont*/
const Value V4860_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_289 = {S12, 0, &V4860_289_86899, &P4860_288};	/*Property = CssImages*/
const Value V4860_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_291 = {S13, 0, &V4860_291_57182, &P4860_289};	/*Property = CssMediaQueries*/
const Value V4860_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_292 = {S14, 0, &V4860_292_57185, &P4860_291};	/*Property = CssMinMax*/
const Value V4860_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_293 = {S15, 0, &V4860_293_57188, &P4860_292};	/*Property = CssOverflow*/
const Value V4860_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_294 = {S16, 0, &V4860_294_57191, &P4860_293};	/*Property = CssPosition*/
const Value V4860_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_295 = {S17, 0, &V4860_295_57194, &P4860_294};	/*Property = CssText*/
const Value V4860_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_297 = {S18, 0, &V4860_297_57197, &P4860_295};	/*Property = CssTransforms*/
const Value V4860_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_241 = {S19, 0, &V4860_241_55801, &P4860_297};	/*Property = CssTransitions*/
const Value V4860_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_298 = {S20, 0, &V4860_298_57208, &P4860_241};	/*Property = CssUI*/
const Value V4860_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_300 = {S21, 0, &V4860_300_57217, &P4860_298};	/*Property = DataSet*/
const Value V4860_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_301 = {S22, 0, &V4860_301_57220, &P4860_300};	/*Property = DataUrl*/
const Value V4860_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_302 = {S23, 0, &V4860_302_57223, &P4860_301};	/*Property = DeviceOrientation*/
const Value V4860_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_304 = {S24, 0, &V4860_304_57229, &P4860_302};	/*Property = FileReader*/
const Value V4860_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_305 = {S25, 0, &V4860_305_57232, &P4860_304};	/*Property = FileSaver*/
const Value V4860_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_306 = {S26, 0, &V4860_306_57238, &P4860_305};	/*Property = FileWriter*/
const Value V4860_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_307 = {S27, 0, &V4860_307_57241, &P4860_306};	/*Property = FormData*/
const Value V4860_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_309 = {S28, 0, &V4860_309_57247, &P4860_307};	/*Property = Fullscreen*/
const Value V4860_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_310 = {S29, 0, &V4860_310_57250, &P4860_309};	/*Property = GeoLocation*/
const Value V4860_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_312 = {S30, 0, &V4860_312_57256, &P4860_310};	/*Property = History*/
const Value V4860_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_314 = {S31, 0, &V4860_314_57262, &P4860_312};	/*Property = Html5*/
const Value V4860_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_313 = {S32, 0, &V4860_313_57259, &P4860_314};	/*Property = Html-Media-Capture*/
const Value V4860_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_315 = {S33, 0, &V4860_315_57265, &P4860_313};	/*Property = Iframe*/
const Value V4860_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_316 = {S34, 0, &V4860_316_57268, &P4860_315};	/*Property = IndexedDB*/
const Value V4860_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_317 = {S35, 0, &V4860_317_57271, &P4860_316};	/*Property = Json*/
const Value V4860_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P4860_15 = {S37, 0, &V4860_15_31, &P4860_317};	/*Property = LayoutEngine*/
const Value V4860_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_344 = {S38, 0, &V4860_344_57274, &P4860_15};	/*Property = Masking*/
const Value V4860_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_320 = {S39, 0, &V4860_320_57292, &P4860_344};	/*Property = PostMessage*/
const Value V4860_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_321 = {S40, 0, &V4860_321_57295, &P4860_320};	/*Property = Progress*/
const Value V4860_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_322 = {S41, 0, &V4860_322_57298, &P4860_321};	/*Property = Prompts*/
const Value V4860_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_323 = {S42, 0, &V4860_323_57301, &P4860_322};	/*Property = Selector*/
const Value V4860_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_324 = {S43, 0, &V4860_324_57307, &P4860_323};	/*Property = Svg*/
const Value V4860_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_327 = {S44, 0, &V4860_327_57316, &P4860_324};	/*Property = TouchEvents*/
const Value V4860_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_328 = {S45, 0, &V4860_328_57319, &P4860_327};	/*Property = Track*/
const Value V4860_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_329 = {S46, 0, &V4860_329_57325, &P4860_328};	/*Property = Video*/
const Value V4860_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_330 = {S47, 0, &V4860_330_57328, &P4860_329};	/*Property = Viewport*/
const Value V4860_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_332 = {S48, 0, &V4860_332_57340, &P4860_330};	/*Property = WebWorkers*/
const Value V4860_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4860_333 = {S49, 0, &V4860_333_57343, &P4860_332};	/*Property = Xhr2*/
const Profile P4860 = {"4860", &P4860_333};	/*Profile ID - 4860*/
const Value V17371_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_279 = {S1, 0, &V17371_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17371_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_282 = {S3, 0, &V17371_282_57155, &P17371_279};	/*Property = BlobBuilder*/
const Value V17371_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_350 = {S4, 0, &V17371_350_57446, &P17371_282};	/*Property = Canvas*/
const Value V17371_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_283 = {S5, 0, &V17371_283_57161, &P17371_350};	/*Property = CssBackground*/
const Value V17371_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_284 = {S6, 0, &V17371_284_57164, &P17371_283};	/*Property = CssBorderImage*/
const Value V17371_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_285 = {S7, 0, &V17371_285_57167, &P17371_284};	/*Property = CssCanvas*/
const Value V17371_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_286 = {S8, 0, &V17371_286_57170, &P17371_285};	/*Property = CssColor*/
const Value V17371_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_237 = {S9, 0, &V17371_237_55786, &P17371_286};	/*Property = CssColumn*/
const Value V17371_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_287 = {S10, 0, &V17371_287_57173, &P17371_237};	/*Property = CssFlexbox*/
const Value V17371_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_288 = {S11, 0, &V17371_288_57176, &P17371_287};	/*Property = CssFont*/
const Value V17371_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_289 = {S12, 0, &V17371_289_86899, &P17371_288};	/*Property = CssImages*/
const Value V17371_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_291 = {S13, 0, &V17371_291_57182, &P17371_289};	/*Property = CssMediaQueries*/
const Value V17371_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_292 = {S14, 0, &V17371_292_57185, &P17371_291};	/*Property = CssMinMax*/
const Value V17371_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_293 = {S15, 0, &V17371_293_57188, &P17371_292};	/*Property = CssOverflow*/
const Value V17371_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_294 = {S16, 0, &V17371_294_57191, &P17371_293};	/*Property = CssPosition*/
const Value V17371_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_295 = {S17, 0, &V17371_295_57194, &P17371_294};	/*Property = CssText*/
const Value V17371_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_297 = {S18, 0, &V17371_297_57197, &P17371_295};	/*Property = CssTransforms*/
const Value V17371_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_241 = {S19, 0, &V17371_241_55801, &P17371_297};	/*Property = CssTransitions*/
const Value V17371_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_298 = {S20, 0, &V17371_298_57208, &P17371_241};	/*Property = CssUI*/
const Value V17371_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_300 = {S21, 0, &V17371_300_57217, &P17371_298};	/*Property = DataSet*/
const Value V17371_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_301 = {S22, 0, &V17371_301_57220, &P17371_300};	/*Property = DataUrl*/
const Value V17371_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_302 = {S23, 0, &V17371_302_57223, &P17371_301};	/*Property = DeviceOrientation*/
const Value V17371_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_304 = {S24, 0, &V17371_304_57229, &P17371_302};	/*Property = FileReader*/
const Value V17371_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_305 = {S25, 0, &V17371_305_57232, &P17371_304};	/*Property = FileSaver*/
const Value V17371_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_306 = {S26, 0, &V17371_306_57238, &P17371_305};	/*Property = FileWriter*/
const Value V17371_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_307 = {S27, 0, &V17371_307_57241, &P17371_306};	/*Property = FormData*/
const Value V17371_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_309 = {S28, 0, &V17371_309_57247, &P17371_307};	/*Property = Fullscreen*/
const Value V17371_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_310 = {S29, 0, &V17371_310_57250, &P17371_309};	/*Property = GeoLocation*/
const Value V17371_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_312 = {S30, 0, &V17371_312_57256, &P17371_310};	/*Property = History*/
const Value V17371_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_314 = {S31, 0, &V17371_314_57262, &P17371_312};	/*Property = Html5*/
const Value V17371_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_313 = {S32, 0, &V17371_313_57259, &P17371_314};	/*Property = Html-Media-Capture*/
const Value V17371_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_315 = {S33, 0, &V17371_315_57265, &P17371_313};	/*Property = Iframe*/
const Value V17371_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_316 = {S34, 0, &V17371_316_57268, &P17371_315};	/*Property = IndexedDB*/
const Value V17371_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_317 = {S35, 0, &V17371_317_57271, &P17371_316};	/*Property = Json*/
const Value V17371_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P17371_15 = {S37, 0, &V17371_15_31, &P17371_317};	/*Property = LayoutEngine*/
const Value V17371_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_344 = {S38, 0, &V17371_344_57274, &P17371_15};	/*Property = Masking*/
const Value V17371_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_320 = {S39, 0, &V17371_320_57292, &P17371_344};	/*Property = PostMessage*/
const Value V17371_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_321 = {S40, 0, &V17371_321_57295, &P17371_320};	/*Property = Progress*/
const Value V17371_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_322 = {S41, 0, &V17371_322_57298, &P17371_321};	/*Property = Prompts*/
const Value V17371_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_323 = {S42, 0, &V17371_323_57301, &P17371_322};	/*Property = Selector*/
const Value V17371_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_324 = {S43, 0, &V17371_324_57307, &P17371_323};	/*Property = Svg*/
const Value V17371_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_327 = {S44, 0, &V17371_327_57316, &P17371_324};	/*Property = TouchEvents*/
const Value V17371_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_328 = {S45, 0, &V17371_328_57319, &P17371_327};	/*Property = Track*/
const Value V17371_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_329 = {S46, 0, &V17371_329_57325, &P17371_328};	/*Property = Video*/
const Value V17371_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_330 = {S47, 0, &V17371_330_57328, &P17371_329};	/*Property = Viewport*/
const Value V17371_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_332 = {S48, 0, &V17371_332_57340, &P17371_330};	/*Property = WebWorkers*/
const Value V17371_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17371_333 = {S49, 0, &V17371_333_57343, &P17371_332};	/*Property = Xhr2*/
const Profile P17371 = {"17371", &P17371_333};	/*Profile ID - 17371*/
const Value V21084_1_1 = {S2, NULL};	/*Value = True*/
const Property P21084_1 = {S50, 0, &V21084_1_1, NULL};	/*Property = IsMobile*/
const Value V21084_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21084_5 = {S52, 0, &V21084_5_1227, &P21084_1};	/*Property = ScreenPixelsHeight*/
const Value V21084_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21084_4 = {S54, 0, &V21084_4_1229, &P21084_5};	/*Property = ScreenPixelsWidth*/
const Profile P21084 = {"21084", &P21084_4};	/*Profile ID - 21084*/
const Value V21180_1_1 = {S2, NULL};	/*Value = True*/
const Property P21180_1 = {S50, 0, &V21180_1_1, NULL};	/*Property = IsMobile*/
const Value V21180_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21180_5 = {S52, 0, &V21180_5_9981, &P21180_1};	/*Property = ScreenPixelsHeight*/
const Value V21180_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21180_4 = {S54, 0, &V21180_4_2336, &P21180_5};	/*Property = ScreenPixelsWidth*/
const Profile P21180 = {"21180", &P21180_4};	/*Profile ID - 21180*/
const Value V18125_1_1 = {S2, NULL};	/*Value = True*/
const Property P18125_1 = {S50, 0, &V18125_1_1, NULL};	/*Property = IsMobile*/
const Value V18125_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18125_5 = {S52, 0, &V18125_5_1403, &P18125_1};	/*Property = ScreenPixelsHeight*/
const Value V18125_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18125_4 = {S54, 0, &V18125_4_1404, &P18125_5};	/*Property = ScreenPixelsWidth*/
const Profile P18125 = {"18125", &P18125_4};	/*Profile ID - 18125*/
const Value V18651_1_1 = {S2, NULL};	/*Value = True*/
const Property P18651_1 = {S50, 0, &V18651_1_1, NULL};	/*Property = IsMobile*/
const Value V18651_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18651_5 = {S52, 0, &V18651_5_9981, &P18651_1};	/*Property = ScreenPixelsHeight*/
const Value V18651_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18651_4 = {S54, 0, &V18651_4_39098, &P18651_5};	/*Property = ScreenPixelsWidth*/
const Profile P18651 = {"18651", &P18651_4};	/*Profile ID - 18651*/
const Value V17211_1_1 = {S2, NULL};	/*Value = True*/
const Property P17211_1 = {S50, 0, &V17211_1_1, NULL};	/*Property = IsMobile*/
const Value V17211_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17211_5 = {S52, 0, &V17211_5_1227, &P17211_1};	/*Property = ScreenPixelsHeight*/
const Value V17211_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P17211_4 = {S54, 0, &V17211_4_1477, &P17211_5};	/*Property = ScreenPixelsWidth*/
const Profile P17211 = {"17211", &P17211_4};	/*Profile ID - 17211*/
const Value V21473_1_1 = {S2, NULL};	/*Value = True*/
const Property P21473_1 = {S50, 0, &V21473_1_1, NULL};	/*Property = IsMobile*/
const Value V21473_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21473_5 = {S52, 0, &V21473_5_1277, &P21473_1};	/*Property = ScreenPixelsHeight*/
const Value V21473_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21473_4 = {S54, 0, &V21473_4_1279, &P21473_5};	/*Property = ScreenPixelsWidth*/
const Profile P21473 = {"21473", &P21473_4};	/*Profile ID - 21473*/
const Value V23991_1_1 = {S2, NULL};	/*Value = True*/
const Property P23991_1 = {S50, 0, &V23991_1_1, NULL};	/*Property = IsMobile*/
const Value V23991_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23991_5 = {S52, 0, &V23991_5_1227, &P23991_1};	/*Property = ScreenPixelsHeight*/
const Value V23991_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23991_4 = {S54, 0, &V23991_4_1229, &P23991_5};	/*Property = ScreenPixelsWidth*/
const Profile P23991 = {"23991", &P23991_4};	/*Profile ID - 23991*/
const Value V22732_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22732_279 = {S1, 0, &V22732_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22732_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22732_282 = {S3, 0, &V22732_282_57154, &P22732_279};	/*Property = BlobBuilder*/
const Value V22732_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22732_350 = {S4, 0, &V22732_350_57444, &P22732_282};	/*Property = Canvas*/
const Value V22732_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22732_283 = {S5, 0, &V22732_283_57159, &P22732_350};	/*Property = CssBackground*/
const Value V22732_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22732_284 = {S6, 0, &V22732_284_57163, &P22732_283};	/*Property = CssBorderImage*/
const Value V22732_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22732_285 = {S7, 0, &V22732_285_57166, &P22732_284};	/*Property = CssCanvas*/
const Value V22732_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22732_286 = {S8, 0, &V22732_286_57168, &P22732_285};	/*Property = CssColor*/
const Value V22732_237_55785 = {S0, NULL};	/*Value = False*/
const Property P22732_237 = {S9, 0, &V22732_237_55785, &P22732_286};	/*Property = CssColumn*/
const Value V22732_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22732_287 = {S10, 0, &V22732_287_57172, &P22732_237};	/*Property = CssFlexbox*/
const Value V22732_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22732_288 = {S11, 0, &V22732_288_57175, &P22732_287};	/*Property = CssFont*/
const Value V22732_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22732_289 = {S12, 0, &V22732_289_57759, &P22732_288};	/*Property = CssImages*/
const Value V22732_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22732_291 = {S13, 0, &V22732_291_57181, &P22732_289};	/*Property = CssMediaQueries*/
const Value V22732_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22732_292 = {S14, 0, &V22732_292_57183, &P22732_291};	/*Property = CssMinMax*/
const Value V22732_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22732_293 = {S15, 0, &V22732_293_57187, &P22732_292};	/*Property = CssOverflow*/
const Value V22732_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22732_294 = {S16, 0, &V22732_294_57190, &P22732_293};	/*Property = CssPosition*/
const Value V22732_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22732_295 = {S17, 0, &V22732_295_57193, &P22732_294};	/*Property = CssText*/
const Value V22732_297_57196 = {S0, NULL};	/*Value = False*/
const Property P22732_297 = {S18, 0, &V22732_297_57196, &P22732_295};	/*Property = CssTransforms*/
const Value V22732_241_55800 = {S0, NULL};	/*Value = False*/
const Property P22732_241 = {S19, 0, &V22732_241_55800, &P22732_297};	/*Property = CssTransitions*/
const Value V22732_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22732_298 = {S20, 0, &V22732_298_57206, &P22732_241};	/*Property = CssUI*/
const Value V22732_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22732_300 = {S21, 0, &V22732_300_57216, &P22732_298};	/*Property = DataSet*/
const Value V22732_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22732_301 = {S22, 0, &V22732_301_57218, &P22732_300};	/*Property = DataUrl*/
const Value V22732_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22732_302 = {S23, 0, &V22732_302_57222, &P22732_301};	/*Property = DeviceOrientation*/
const Value V22732_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22732_304 = {S24, 0, &V22732_304_57228, &P22732_302};	/*Property = FileReader*/
const Value V22732_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22732_305 = {S25, 0, &V22732_305_57231, &P22732_304};	/*Property = FileSaver*/
const Value V22732_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22732_306 = {S26, 0, &V22732_306_57237, &P22732_305};	/*Property = FileWriter*/
const Value V22732_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22732_307 = {S27, 0, &V22732_307_57240, &P22732_306};	/*Property = FormData*/
const Value V22732_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22732_309 = {S28, 0, &V22732_309_57246, &P22732_307};	/*Property = Fullscreen*/
const Value V22732_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22732_310 = {S29, 0, &V22732_310_57248, &P22732_309};	/*Property = GeoLocation*/
const Value V22732_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22732_312 = {S30, 0, &V22732_312_57255, &P22732_310};	/*Property = History*/
const Value V22732_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22732_314 = {S31, 0, &V22732_314_57261, &P22732_312};	/*Property = Html5*/
const Value V22732_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22732_313 = {S32, 0, &V22732_313_57258, &P22732_314};	/*Property = Html-Media-Capture*/
const Value V22732_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22732_315 = {S33, 0, &V22732_315_57264, &P22732_313};	/*Property = Iframe*/
const Value V22732_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22732_316 = {S34, 0, &V22732_316_57267, &P22732_315};	/*Property = IndexedDB*/
const Value V22732_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22732_317 = {S35, 0, &V22732_317_57269, &P22732_316};	/*Property = Json*/
const Value V22732_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P22732_15 = {S37, 0, &V22732_15_30, &P22732_317};	/*Property = LayoutEngine*/
const Value V22732_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22732_344 = {S38, 0, &V22732_344_57272, &P22732_15};	/*Property = Masking*/
const Value V22732_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22732_320 = {S39, 0, &V22732_320_57290, &P22732_344};	/*Property = PostMessage*/
const Value V22732_321_57294 = {S0, NULL};	/*Value = False*/
const Property P22732_321 = {S40, 0, &V22732_321_57294, &P22732_320};	/*Property = Progress*/
const Value V22732_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22732_322 = {S41, 0, &V22732_322_57296, &P22732_321};	/*Property = Prompts*/
const Value V22732_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22732_323 = {S42, 0, &V22732_323_57299, &P22732_322};	/*Property = Selector*/
const Value V22732_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22732_324 = {S43, 0, &V22732_324_57306, &P22732_323};	/*Property = Svg*/
const Value V22732_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22732_327 = {S44, 0, &V22732_327_57315, &P22732_324};	/*Property = TouchEvents*/
const Value V22732_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22732_328 = {S45, 0, &V22732_328_57318, &P22732_327};	/*Property = Track*/
const Value V22732_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22732_329 = {S46, 0, &V22732_329_57323, &P22732_328};	/*Property = Video*/
const Value V22732_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22732_330 = {S47, 0, &V22732_330_57326, &P22732_329};	/*Property = Viewport*/
const Value V22732_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22732_332 = {S48, 0, &V22732_332_57339, &P22732_330};	/*Property = WebWorkers*/
const Value V22732_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22732_333 = {S49, 0, &V22732_333_57342, &P22732_332};	/*Property = Xhr2*/
const Profile P22732 = {"22732", &P22732_333};	/*Profile ID - 22732*/
const Value V16319_1_1 = {S2, NULL};	/*Value = True*/
const Property P16319_1 = {S50, 0, &V16319_1_1, NULL};	/*Property = IsMobile*/
const Value V16319_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P16319_5 = {S52, 0, &V16319_5_1269, &P16319_1};	/*Property = ScreenPixelsHeight*/
const Value V16319_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16319_4 = {S54, 0, &V16319_4_1229, &P16319_5};	/*Property = ScreenPixelsWidth*/
const Profile P16319 = {"16319", &P16319_4};	/*Profile ID - 16319*/
const Value V18630_1_1 = {S2, NULL};	/*Value = True*/
const Property P18630_1 = {S50, 0, &V18630_1_1, NULL};	/*Property = IsMobile*/
const Value V18630_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18630_5 = {S52, 0, &V18630_5_9981, &P18630_1};	/*Property = ScreenPixelsHeight*/
const Value V18630_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18630_4 = {S54, 0, &V18630_4_2336, &P18630_5};	/*Property = ScreenPixelsWidth*/
const Profile P18630 = {"18630", &P18630_4};	/*Profile ID - 18630*/
const Value V21558_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_279 = {S1, 0, &V21558_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21558_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_282 = {S3, 0, &V21558_282_57155, &P21558_279};	/*Property = BlobBuilder*/
const Value V21558_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_350 = {S4, 0, &V21558_350_57446, &P21558_282};	/*Property = Canvas*/
const Value V21558_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_283 = {S5, 0, &V21558_283_57161, &P21558_350};	/*Property = CssBackground*/
const Value V21558_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_284 = {S6, 0, &V21558_284_57164, &P21558_283};	/*Property = CssBorderImage*/
const Value V21558_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_285 = {S7, 0, &V21558_285_57167, &P21558_284};	/*Property = CssCanvas*/
const Value V21558_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_286 = {S8, 0, &V21558_286_57170, &P21558_285};	/*Property = CssColor*/
const Value V21558_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_237 = {S9, 0, &V21558_237_55786, &P21558_286};	/*Property = CssColumn*/
const Value V21558_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_287 = {S10, 0, &V21558_287_57173, &P21558_237};	/*Property = CssFlexbox*/
const Value V21558_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_288 = {S11, 0, &V21558_288_57176, &P21558_287};	/*Property = CssFont*/
const Value V21558_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_289 = {S12, 0, &V21558_289_86899, &P21558_288};	/*Property = CssImages*/
const Value V21558_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_291 = {S13, 0, &V21558_291_57182, &P21558_289};	/*Property = CssMediaQueries*/
const Value V21558_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_292 = {S14, 0, &V21558_292_57185, &P21558_291};	/*Property = CssMinMax*/
const Value V21558_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_293 = {S15, 0, &V21558_293_57188, &P21558_292};	/*Property = CssOverflow*/
const Value V21558_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_294 = {S16, 0, &V21558_294_57191, &P21558_293};	/*Property = CssPosition*/
const Value V21558_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_295 = {S17, 0, &V21558_295_57194, &P21558_294};	/*Property = CssText*/
const Value V21558_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_297 = {S18, 0, &V21558_297_57197, &P21558_295};	/*Property = CssTransforms*/
const Value V21558_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_241 = {S19, 0, &V21558_241_55801, &P21558_297};	/*Property = CssTransitions*/
const Value V21558_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_298 = {S20, 0, &V21558_298_57208, &P21558_241};	/*Property = CssUI*/
const Value V21558_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_300 = {S21, 0, &V21558_300_57217, &P21558_298};	/*Property = DataSet*/
const Value V21558_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_301 = {S22, 0, &V21558_301_57220, &P21558_300};	/*Property = DataUrl*/
const Value V21558_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_302 = {S23, 0, &V21558_302_57223, &P21558_301};	/*Property = DeviceOrientation*/
const Value V21558_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_304 = {S24, 0, &V21558_304_57229, &P21558_302};	/*Property = FileReader*/
const Value V21558_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_305 = {S25, 0, &V21558_305_57232, &P21558_304};	/*Property = FileSaver*/
const Value V21558_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_306 = {S26, 0, &V21558_306_57238, &P21558_305};	/*Property = FileWriter*/
const Value V21558_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_307 = {S27, 0, &V21558_307_57241, &P21558_306};	/*Property = FormData*/
const Value V21558_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_309 = {S28, 0, &V21558_309_57247, &P21558_307};	/*Property = Fullscreen*/
const Value V21558_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_310 = {S29, 0, &V21558_310_57250, &P21558_309};	/*Property = GeoLocation*/
const Value V21558_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_312 = {S30, 0, &V21558_312_57256, &P21558_310};	/*Property = History*/
const Value V21558_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_314 = {S31, 0, &V21558_314_57262, &P21558_312};	/*Property = Html5*/
const Value V21558_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_313 = {S32, 0, &V21558_313_57259, &P21558_314};	/*Property = Html-Media-Capture*/
const Value V21558_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_315 = {S33, 0, &V21558_315_57265, &P21558_313};	/*Property = Iframe*/
const Value V21558_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_316 = {S34, 0, &V21558_316_57268, &P21558_315};	/*Property = IndexedDB*/
const Value V21558_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_317 = {S35, 0, &V21558_317_57271, &P21558_316};	/*Property = Json*/
const Value V21558_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21558_15 = {S37, 0, &V21558_15_28, &P21558_317};	/*Property = LayoutEngine*/
const Value V21558_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_344 = {S38, 0, &V21558_344_57274, &P21558_15};	/*Property = Masking*/
const Value V21558_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_320 = {S39, 0, &V21558_320_57292, &P21558_344};	/*Property = PostMessage*/
const Value V21558_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_321 = {S40, 0, &V21558_321_57295, &P21558_320};	/*Property = Progress*/
const Value V21558_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_322 = {S41, 0, &V21558_322_57298, &P21558_321};	/*Property = Prompts*/
const Value V21558_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_323 = {S42, 0, &V21558_323_57301, &P21558_322};	/*Property = Selector*/
const Value V21558_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_324 = {S43, 0, &V21558_324_57307, &P21558_323};	/*Property = Svg*/
const Value V21558_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_327 = {S44, 0, &V21558_327_57316, &P21558_324};	/*Property = TouchEvents*/
const Value V21558_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_328 = {S45, 0, &V21558_328_57319, &P21558_327};	/*Property = Track*/
const Value V21558_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_329 = {S46, 0, &V21558_329_57325, &P21558_328};	/*Property = Video*/
const Value V21558_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_330 = {S47, 0, &V21558_330_57328, &P21558_329};	/*Property = Viewport*/
const Value V21558_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_332 = {S48, 0, &V21558_332_57340, &P21558_330};	/*Property = WebWorkers*/
const Value V21558_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21558_333 = {S49, 0, &V21558_333_57343, &P21558_332};	/*Property = Xhr2*/
const Profile P21558 = {"21558", &P21558_333};	/*Profile ID - 21558*/
const Value V18216_1_1 = {S2, NULL};	/*Value = True*/
const Property P18216_1 = {S50, 0, &V18216_1_1, NULL};	/*Property = IsMobile*/
const Value V18216_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18216_5 = {S52, 0, &V18216_5_1277, &P18216_1};	/*Property = ScreenPixelsHeight*/
const Value V18216_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P18216_4 = {S54, 0, &V18216_4_5645, &P18216_5};	/*Property = ScreenPixelsWidth*/
const Profile P18216 = {"18216", &P18216_4};	/*Profile ID - 18216*/
const Value V18678_1_1 = {S2, NULL};	/*Value = True*/
const Property P18678_1 = {S50, 0, &V18678_1_1, NULL};	/*Property = IsMobile*/
const Value V18678_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18678_5 = {S52, 0, &V18678_5_9981, &P18678_1};	/*Property = ScreenPixelsHeight*/
const Value V18678_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18678_4 = {S54, 0, &V18678_4_39098, &P18678_5};	/*Property = ScreenPixelsWidth*/
const Profile P18678 = {"18678", &P18678_4};	/*Profile ID - 18678*/
const Value V23844_1_1 = {S2, NULL};	/*Value = True*/
const Property P23844_1 = {S50, 0, &V23844_1_1, NULL};	/*Property = IsMobile*/
const Value V23844_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23844_5 = {S52, 0, &V23844_5_1227, &P23844_1};	/*Property = ScreenPixelsHeight*/
const Value V23844_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23844_4 = {S54, 0, &V23844_4_1229, &P23844_5};	/*Property = ScreenPixelsWidth*/
const Profile P23844 = {"23844", &P23844_4};	/*Profile ID - 23844*/
const Value V20779_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20779_279 = {S1, 0, &V20779_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20779_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20779_282 = {S3, 0, &V20779_282_57154, &P20779_279};	/*Property = BlobBuilder*/
const Value V20779_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20779_350 = {S4, 0, &V20779_350_57444, &P20779_282};	/*Property = Canvas*/
const Value V20779_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20779_283 = {S5, 0, &V20779_283_57159, &P20779_350};	/*Property = CssBackground*/
const Value V20779_284_57162 = {S2, NULL};	/*Value = True*/
const Property P20779_284 = {S6, 0, &V20779_284_57162, &P20779_283};	/*Property = CssBorderImage*/
const Value V20779_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20779_285 = {S7, 0, &V20779_285_57166, &P20779_284};	/*Property = CssCanvas*/
const Value V20779_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20779_286 = {S8, 0, &V20779_286_57168, &P20779_285};	/*Property = CssColor*/
const Value V20779_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20779_237 = {S9, 0, &V20779_237_55786, &P20779_286};	/*Property = CssColumn*/
const Value V20779_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20779_287 = {S10, 0, &V20779_287_57172, &P20779_237};	/*Property = CssFlexbox*/
const Value V20779_288_57174 = {S2, NULL};	/*Value = True*/
const Property P20779_288 = {S11, 0, &V20779_288_57174, &P20779_287};	/*Property = CssFont*/
const Value V20779_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20779_289 = {S12, 0, &V20779_289_57759, &P20779_288};	/*Property = CssImages*/
const Value V20779_291_57180 = {S2, NULL};	/*Value = True*/
const Property P20779_291 = {S13, 0, &V20779_291_57180, &P20779_289};	/*Property = CssMediaQueries*/
const Value V20779_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20779_292 = {S14, 0, &V20779_292_57183, &P20779_291};	/*Property = CssMinMax*/
const Value V20779_293_57186 = {S2, NULL};	/*Value = True*/
const Property P20779_293 = {S15, 0, &V20779_293_57186, &P20779_292};	/*Property = CssOverflow*/
const Value V20779_294_57189 = {S2, NULL};	/*Value = True*/
const Property P20779_294 = {S16, 0, &V20779_294_57189, &P20779_293};	/*Property = CssPosition*/
const Value V20779_295_57192 = {S2, NULL};	/*Value = True*/
const Property P20779_295 = {S17, 0, &V20779_295_57192, &P20779_294};	/*Property = CssText*/
const Value V20779_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20779_297 = {S18, 0, &V20779_297_57195, &P20779_295};	/*Property = CssTransforms*/
const Value V20779_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20779_241 = {S19, 0, &V20779_241_55799, &P20779_297};	/*Property = CssTransitions*/
const Value V20779_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20779_298 = {S20, 0, &V20779_298_57206, &P20779_241};	/*Property = CssUI*/
const Value V20779_300_57215 = {S2, NULL};	/*Value = True*/
const Property P20779_300 = {S21, 0, &V20779_300_57215, &P20779_298};	/*Property = DataSet*/
const Value V20779_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20779_301 = {S22, 0, &V20779_301_57218, &P20779_300};	/*Property = DataUrl*/
const Value V20779_302_57221 = {S2, NULL};	/*Value = True*/
const Property P20779_302 = {S23, 0, &V20779_302_57221, &P20779_301};	/*Property = DeviceOrientation*/
const Value V20779_304_57228 = {S0, NULL};	/*Value = False*/
const Property P20779_304 = {S24, 0, &V20779_304_57228, &P20779_302};	/*Property = FileReader*/
const Value V20779_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20779_305 = {S25, 0, &V20779_305_57231, &P20779_304};	/*Property = FileSaver*/
const Value V20779_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20779_306 = {S26, 0, &V20779_306_57237, &P20779_305};	/*Property = FileWriter*/
const Value V20779_307_57239 = {S2, NULL};	/*Value = True*/
const Property P20779_307 = {S27, 0, &V20779_307_57239, &P20779_306};	/*Property = FormData*/
const Value V20779_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20779_309 = {S28, 0, &V20779_309_57246, &P20779_307};	/*Property = Fullscreen*/
const Value V20779_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20779_310 = {S29, 0, &V20779_310_57248, &P20779_309};	/*Property = GeoLocation*/
const Value V20779_312_57255 = {S0, NULL};	/*Value = False*/
const Property P20779_312 = {S30, 0, &V20779_312_57255, &P20779_310};	/*Property = History*/
const Value V20779_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20779_314 = {S31, 0, &V20779_314_57261, &P20779_312};	/*Property = Html5*/
const Value V20779_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20779_313 = {S32, 0, &V20779_313_57258, &P20779_314};	/*Property = Html-Media-Capture*/
const Value V20779_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20779_315 = {S33, 0, &V20779_315_57264, &P20779_313};	/*Property = Iframe*/
const Value V20779_316_57267 = {S0, NULL};	/*Value = False*/
const Property P20779_316 = {S34, 0, &V20779_316_57267, &P20779_315};	/*Property = IndexedDB*/
const Value V20779_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20779_317 = {S35, 0, &V20779_317_57269, &P20779_316};	/*Property = Json*/
const Value V20779_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P20779_15 = {S37, 0, &V20779_15_54167, &P20779_317};	/*Property = LayoutEngine*/
const Value V20779_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20779_344 = {S38, 0, &V20779_344_57272, &P20779_15};	/*Property = Masking*/
const Value V20779_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20779_320 = {S39, 0, &V20779_320_57290, &P20779_344};	/*Property = PostMessage*/
const Value V20779_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20779_321 = {S40, 0, &V20779_321_57293, &P20779_320};	/*Property = Progress*/
const Value V20779_322_57297 = {S0, NULL};	/*Value = False*/
const Property P20779_322 = {S41, 0, &V20779_322_57297, &P20779_321};	/*Property = Prompts*/
const Value V20779_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20779_323 = {S42, 0, &V20779_323_57299, &P20779_322};	/*Property = Selector*/
const Value V20779_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20779_324 = {S43, 0, &V20779_324_57306, &P20779_323};	/*Property = Svg*/
const Value V20779_327_57314 = {S2, NULL};	/*Value = True*/
const Property P20779_327 = {S44, 0, &V20779_327_57314, &P20779_324};	/*Property = TouchEvents*/
const Value V20779_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20779_328 = {S45, 0, &V20779_328_57318, &P20779_327};	/*Property = Track*/
const Value V20779_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20779_329 = {S46, 0, &V20779_329_57323, &P20779_328};	/*Property = Video*/
const Value V20779_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20779_330 = {S47, 0, &V20779_330_57326, &P20779_329};	/*Property = Viewport*/
const Value V20779_332_57339 = {S0, NULL};	/*Value = False*/
const Property P20779_332 = {S48, 0, &V20779_332_57339, &P20779_330};	/*Property = WebWorkers*/
const Value V20779_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20779_333 = {S49, 0, &V20779_333_57342, &P20779_332};	/*Property = Xhr2*/
const Profile P20779 = {"20779", &P20779_333};	/*Profile ID - 20779*/
const Value V16005_1_1 = {S2, NULL};	/*Value = True*/
const Property P16005_1 = {S50, 0, &V16005_1_1, NULL};	/*Property = IsMobile*/
const Value V16005_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16005_5 = {S52, 0, &V16005_5_1227, &P16005_1};	/*Property = ScreenPixelsHeight*/
const Value V16005_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16005_4 = {S54, 0, &V16005_4_1229, &P16005_5};	/*Property = ScreenPixelsWidth*/
const Profile P16005 = {"16005", &P16005_4};	/*Profile ID - 16005*/
const Value V21452_1_1 = {S2, NULL};	/*Value = True*/
const Property P21452_1 = {S50, 0, &V21452_1_1, NULL};	/*Property = IsMobile*/
const Value V21452_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21452_5 = {S52, 0, &V21452_5_1269, &P21452_1};	/*Property = ScreenPixelsHeight*/
const Value V21452_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21452_4 = {S54, 0, &V21452_4_1229, &P21452_5};	/*Property = ScreenPixelsWidth*/
const Profile P21452 = {"21452", &P21452_4};	/*Profile ID - 21452*/
const Value V22264_1_1 = {S2, NULL};	/*Value = True*/
const Property P22264_1 = {S50, 0, &V22264_1_1, NULL};	/*Property = IsMobile*/
const Value V22264_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22264_5 = {S52, 0, &V22264_5_1277, &P22264_1};	/*Property = ScreenPixelsHeight*/
const Value V22264_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22264_4 = {S54, 0, &V22264_4_2336, &P22264_5};	/*Property = ScreenPixelsWidth*/
const Profile P22264 = {"22264", &P22264_4};	/*Profile ID - 22264*/
const Value V18114_1_1 = {S2, NULL};	/*Value = True*/
const Property P18114_1 = {S50, 0, &V18114_1_1, NULL};	/*Property = IsMobile*/
const Value V18114_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18114_5 = {S52, 0, &V18114_5_1227, &P18114_1};	/*Property = ScreenPixelsHeight*/
const Value V18114_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P18114_4 = {S54, 0, &V18114_4_1477, &P18114_5};	/*Property = ScreenPixelsWidth*/
const Profile P18114 = {"18114", &P18114_4};	/*Profile ID - 18114*/
const Value V21426_1_1 = {S2, NULL};	/*Value = True*/
const Property P21426_1 = {S50, 0, &V21426_1_1, NULL};	/*Property = IsMobile*/
const Value V21426_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21426_5 = {S52, 0, &V21426_5_37, &P21426_1};	/*Property = ScreenPixelsHeight*/
const Value V21426_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21426_4 = {S54, 0, &V21426_4_36, &P21426_5};	/*Property = ScreenPixelsWidth*/
const Profile P21426 = {"21426", &P21426_4};	/*Profile ID - 21426*/
const Value V22214_1_1 = {S2, NULL};	/*Value = True*/
const Property P22214_1 = {S50, 0, &V22214_1_1, NULL};	/*Property = IsMobile*/
const Value V22214_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22214_5 = {S52, 0, &V22214_5_1277, &P22214_1};	/*Property = ScreenPixelsHeight*/
const Value V22214_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22214_4 = {S54, 0, &V22214_4_1279, &P22214_5};	/*Property = ScreenPixelsWidth*/
const Profile P22214 = {"22214", &P22214_4};	/*Profile ID - 22214*/
const Value V16373_1_1 = {S2, NULL};	/*Value = True*/
const Property P16373_1 = {S50, 0, &V16373_1_1, NULL};	/*Property = IsMobile*/
const Value V16373_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16373_5 = {S52, 0, &V16373_5_1277, &P16373_1};	/*Property = ScreenPixelsHeight*/
const Value V16373_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16373_4 = {S54, 0, &V16373_4_1279, &P16373_5};	/*Property = ScreenPixelsWidth*/
const Profile P16373 = {"16373", &P16373_4};	/*Profile ID - 16373*/
const Value V18383_1_1 = {S2, NULL};	/*Value = True*/
const Property P18383_1 = {S50, 0, &V18383_1_1, NULL};	/*Property = IsMobile*/
const Value V18383_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18383_5 = {S52, 0, &V18383_5_1403, &P18383_1};	/*Property = ScreenPixelsHeight*/
const Value V18383_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18383_4 = {S54, 0, &V18383_4_1404, &P18383_5};	/*Property = ScreenPixelsWidth*/
const Profile P18383 = {"18383", &P18383_4};	/*Profile ID - 18383*/
const Value V22454_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22454_279 = {S1, 0, &V22454_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22454_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22454_282 = {S3, 0, &V22454_282_57153, &P22454_279};	/*Property = BlobBuilder*/
const Value V22454_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22454_350 = {S4, 0, &V22454_350_57444, &P22454_282};	/*Property = Canvas*/
const Value V22454_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22454_283 = {S5, 0, &V22454_283_57159, &P22454_350};	/*Property = CssBackground*/
const Value V22454_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22454_284 = {S6, 0, &V22454_284_57163, &P22454_283};	/*Property = CssBorderImage*/
const Value V22454_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22454_285 = {S7, 0, &V22454_285_57166, &P22454_284};	/*Property = CssCanvas*/
const Value V22454_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22454_286 = {S8, 0, &V22454_286_57168, &P22454_285};	/*Property = CssColor*/
const Value V22454_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22454_237 = {S9, 0, &V22454_237_55784, &P22454_286};	/*Property = CssColumn*/
const Value V22454_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22454_287 = {S10, 0, &V22454_287_57172, &P22454_237};	/*Property = CssFlexbox*/
const Value V22454_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22454_288 = {S11, 0, &V22454_288_57175, &P22454_287};	/*Property = CssFont*/
const Value V22454_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22454_289 = {S12, 0, &V22454_289_57759, &P22454_288};	/*Property = CssImages*/
const Value V22454_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22454_291 = {S13, 0, &V22454_291_57180, &P22454_289};	/*Property = CssMediaQueries*/
const Value V22454_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22454_292 = {S14, 0, &V22454_292_57183, &P22454_291};	/*Property = CssMinMax*/
const Value V22454_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22454_293 = {S15, 0, &V22454_293_57187, &P22454_292};	/*Property = CssOverflow*/
const Value V22454_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22454_294 = {S16, 0, &V22454_294_57189, &P22454_293};	/*Property = CssPosition*/
const Value V22454_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22454_295 = {S17, 0, &V22454_295_57192, &P22454_294};	/*Property = CssText*/
const Value V22454_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22454_297 = {S18, 0, &V22454_297_57195, &P22454_295};	/*Property = CssTransforms*/
const Value V22454_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22454_241 = {S19, 0, &V22454_241_55799, &P22454_297};	/*Property = CssTransitions*/
const Value V22454_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22454_298 = {S20, 0, &V22454_298_57206, &P22454_241};	/*Property = CssUI*/
const Value V22454_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22454_300 = {S21, 0, &V22454_300_57216, &P22454_298};	/*Property = DataSet*/
const Value V22454_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22454_301 = {S22, 0, &V22454_301_57218, &P22454_300};	/*Property = DataUrl*/
const Value V22454_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22454_302 = {S23, 0, &V22454_302_57222, &P22454_301};	/*Property = DeviceOrientation*/
const Value V22454_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22454_304 = {S24, 0, &V22454_304_57227, &P22454_302};	/*Property = FileReader*/
const Value V22454_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22454_305 = {S25, 0, &V22454_305_57231, &P22454_304};	/*Property = FileSaver*/
const Value V22454_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22454_306 = {S26, 0, &V22454_306_57237, &P22454_305};	/*Property = FileWriter*/
const Value V22454_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22454_307 = {S27, 0, &V22454_307_57239, &P22454_306};	/*Property = FormData*/
const Value V22454_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22454_309 = {S28, 0, &V22454_309_57246, &P22454_307};	/*Property = Fullscreen*/
const Value V22454_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22454_310 = {S29, 0, &V22454_310_57248, &P22454_309};	/*Property = GeoLocation*/
const Value V22454_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22454_312 = {S30, 0, &V22454_312_57254, &P22454_310};	/*Property = History*/
const Value V22454_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22454_314 = {S31, 0, &V22454_314_57261, &P22454_312};	/*Property = Html5*/
const Value V22454_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22454_313 = {S32, 0, &V22454_313_57258, &P22454_314};	/*Property = Html-Media-Capture*/
const Value V22454_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22454_315 = {S33, 0, &V22454_315_57264, &P22454_313};	/*Property = Iframe*/
const Value V22454_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22454_316 = {S34, 0, &V22454_316_57266, &P22454_315};	/*Property = IndexedDB*/
const Value V22454_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22454_317 = {S35, 0, &V22454_317_57269, &P22454_316};	/*Property = Json*/
const Value V22454_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22454_15 = {S37, 0, &V22454_15_28, &P22454_317};	/*Property = LayoutEngine*/
const Value V22454_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22454_344 = {S38, 0, &V22454_344_57272, &P22454_15};	/*Property = Masking*/
const Value V22454_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22454_320 = {S39, 0, &V22454_320_57290, &P22454_344};	/*Property = PostMessage*/
const Value V22454_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22454_321 = {S40, 0, &V22454_321_57293, &P22454_320};	/*Property = Progress*/
const Value V22454_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22454_322 = {S41, 0, &V22454_322_57296, &P22454_321};	/*Property = Prompts*/
const Value V22454_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22454_323 = {S42, 0, &V22454_323_57299, &P22454_322};	/*Property = Selector*/
const Value V22454_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22454_324 = {S43, 0, &V22454_324_57306, &P22454_323};	/*Property = Svg*/
const Value V22454_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22454_327 = {S44, 0, &V22454_327_57315, &P22454_324};	/*Property = TouchEvents*/
const Value V22454_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22454_328 = {S45, 0, &V22454_328_57318, &P22454_327};	/*Property = Track*/
const Value V22454_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22454_329 = {S46, 0, &V22454_329_57323, &P22454_328};	/*Property = Video*/
const Value V22454_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22454_330 = {S47, 0, &V22454_330_57326, &P22454_329};	/*Property = Viewport*/
const Value V22454_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22454_332 = {S48, 0, &V22454_332_57338, &P22454_330};	/*Property = WebWorkers*/
const Value V22454_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22454_333 = {S49, 0, &V22454_333_57342, &P22454_332};	/*Property = Xhr2*/
const Profile P22454 = {"22454", &P22454_333};	/*Profile ID - 22454*/
const Value V13079_1_1 = {S2, NULL};	/*Value = True*/
const Property P13079_1 = {S50, 0, &V13079_1_1, NULL};	/*Property = IsMobile*/
const Value V13079_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P13079_5 = {S52, 0, &V13079_5_1227, &P13079_1};	/*Property = ScreenPixelsHeight*/
const Value V13079_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P13079_4 = {S54, 0, &V13079_4_1229, &P13079_5};	/*Property = ScreenPixelsWidth*/
const Profile P13079 = {"13079", &P13079_4};	/*Profile ID - 13079*/
const Value V18150_1_2 = {S0, NULL};	/*Value = False*/
const Property P18150_1 = {S50, 0, &V18150_1_2, NULL};	/*Property = IsMobile*/
const Value V18150_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P18150_5 = {S52, 0, &V18150_5_37, &P18150_1};	/*Property = ScreenPixelsHeight*/
const Value V18150_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P18150_4 = {S54, 0, &V18150_4_36, &P18150_5};	/*Property = ScreenPixelsWidth*/
const Profile P18150 = {"18150", &P18150_4};	/*Profile ID - 18150*/
const Value V21213_1_1 = {S2, NULL};	/*Value = True*/
const Property P21213_1 = {S50, 0, &V21213_1_1, NULL};	/*Property = IsMobile*/
const Value V21213_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21213_5 = {S52, 0, &V21213_5_1227, &P21213_1};	/*Property = ScreenPixelsHeight*/
const Value V21213_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21213_4 = {S54, 0, &V21213_4_1460, &P21213_5};	/*Property = ScreenPixelsWidth*/
const Profile P21213 = {"21213", &P21213_4};	/*Profile ID - 21213*/
const Value V3403_279_57148 = {S0, NULL};	/*Value = False*/
const Property P3403_279 = {S1, 0, &V3403_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V3403_282_57154 = {S0, NULL};	/*Value = False*/
const Property P3403_282 = {S3, 0, &V3403_282_57154, &P3403_279};	/*Property = BlobBuilder*/
const Value V3403_350_57444 = {S2, NULL};	/*Value = True*/
const Property P3403_350 = {S4, 0, &V3403_350_57444, &P3403_282};	/*Property = Canvas*/
const Value V3403_283_57159 = {S2, NULL};	/*Value = True*/
const Property P3403_283 = {S5, 0, &V3403_283_57159, &P3403_350};	/*Property = CssBackground*/
const Value V3403_284_57163 = {S0, NULL};	/*Value = False*/
const Property P3403_284 = {S6, 0, &V3403_284_57163, &P3403_283};	/*Property = CssBorderImage*/
const Value V3403_285_57166 = {S0, NULL};	/*Value = False*/
const Property P3403_285 = {S7, 0, &V3403_285_57166, &P3403_284};	/*Property = CssCanvas*/
const Value V3403_286_57168 = {S2, NULL};	/*Value = True*/
const Property P3403_286 = {S8, 0, &V3403_286_57168, &P3403_285};	/*Property = CssColor*/
const Value V3403_237_55784 = {S2, NULL};	/*Value = True*/
const Property P3403_237 = {S9, 0, &V3403_237_55784, &P3403_286};	/*Property = CssColumn*/
const Value V3403_287_57172 = {S0, NULL};	/*Value = False*/
const Property P3403_287 = {S10, 0, &V3403_287_57172, &P3403_237};	/*Property = CssFlexbox*/
const Value V3403_288_57175 = {S0, NULL};	/*Value = False*/
const Property P3403_288 = {S11, 0, &V3403_288_57175, &P3403_287};	/*Property = CssFont*/
const Value V3403_289_57759 = {S0, NULL};	/*Value = False*/
const Property P3403_289 = {S12, 0, &V3403_289_57759, &P3403_288};	/*Property = CssImages*/
const Value V3403_291_57181 = {S0, NULL};	/*Value = False*/
const Property P3403_291 = {S13, 0, &V3403_291_57181, &P3403_289};	/*Property = CssMediaQueries*/
const Value V3403_292_57183 = {S2, NULL};	/*Value = True*/
const Property P3403_292 = {S14, 0, &V3403_292_57183, &P3403_291};	/*Property = CssMinMax*/
const Value V3403_293_57187 = {S0, NULL};	/*Value = False*/
const Property P3403_293 = {S15, 0, &V3403_293_57187, &P3403_292};	/*Property = CssOverflow*/
const Value V3403_294_57190 = {S0, NULL};	/*Value = False*/
const Property P3403_294 = {S16, 0, &V3403_294_57190, &P3403_293};	/*Property = CssPosition*/
const Value V3403_295_57193 = {S0, NULL};	/*Value = False*/
const Property P3403_295 = {S17, 0, &V3403_295_57193, &P3403_294};	/*Property = CssText*/
const Value V3403_297_57196 = {S0, NULL};	/*Value = False*/
const Property P3403_297 = {S18, 0, &V3403_297_57196, &P3403_295};	/*Property = CssTransforms*/
const Value V3403_241_55799 = {S2, NULL};	/*Value = True*/
const Property P3403_241 = {S19, 0, &V3403_241_55799, &P3403_297};	/*Property = CssTransitions*/
const Value V3403_298_57206 = {S2, NULL};	/*Value = True*/
const Property P3403_298 = {S20, 0, &V3403_298_57206, &P3403_241};	/*Property = CssUI*/
const Value V3403_300_57216 = {S0, NULL};	/*Value = False*/
const Property P3403_300 = {S21, 0, &V3403_300_57216, &P3403_298};	/*Property = DataSet*/
const Value V3403_301_57218 = {S2, NULL};	/*Value = True*/
const Property P3403_301 = {S22, 0, &V3403_301_57218, &P3403_300};	/*Property = DataUrl*/
const Value V3403_302_57222 = {S0, NULL};	/*Value = False*/
const Property P3403_302 = {S23, 0, &V3403_302_57222, &P3403_301};	/*Property = DeviceOrientation*/
const Value V3403_304_57228 = {S0, NULL};	/*Value = False*/
const Property P3403_304 = {S24, 0, &V3403_304_57228, &P3403_302};	/*Property = FileReader*/
const Value V3403_305_57231 = {S0, NULL};	/*Value = False*/
const Property P3403_305 = {S25, 0, &V3403_305_57231, &P3403_304};	/*Property = FileSaver*/
const Value V3403_306_57237 = {S0, NULL};	/*Value = False*/
const Property P3403_306 = {S26, 0, &V3403_306_57237, &P3403_305};	/*Property = FileWriter*/
const Value V3403_307_57240 = {S0, NULL};	/*Value = False*/
const Property P3403_307 = {S27, 0, &V3403_307_57240, &P3403_306};	/*Property = FormData*/
const Value V3403_309_57246 = {S0, NULL};	/*Value = False*/
const Property P3403_309 = {S28, 0, &V3403_309_57246, &P3403_307};	/*Property = Fullscreen*/
const Value V3403_310_57248 = {S2, NULL};	/*Value = True*/
const Property P3403_310 = {S29, 0, &V3403_310_57248, &P3403_309};	/*Property = GeoLocation*/
const Value V3403_312_57255 = {S0, NULL};	/*Value = False*/
const Property P3403_312 = {S30, 0, &V3403_312_57255, &P3403_310};	/*Property = History*/
const Value V3403_314_57261 = {S0, NULL};	/*Value = False*/
const Property P3403_314 = {S31, 0, &V3403_314_57261, &P3403_312};	/*Property = Html5*/
const Value V3403_313_57258 = {S0, NULL};	/*Value = False*/
const Property P3403_313 = {S32, 0, &V3403_313_57258, &P3403_314};	/*Property = Html-Media-Capture*/
const Value V3403_315_57264 = {S0, NULL};	/*Value = False*/
const Property P3403_315 = {S33, 0, &V3403_315_57264, &P3403_313};	/*Property = Iframe*/
const Value V3403_316_57267 = {S0, NULL};	/*Value = False*/
const Property P3403_316 = {S34, 0, &V3403_316_57267, &P3403_315};	/*Property = IndexedDB*/
const Value V3403_317_57269 = {S2, NULL};	/*Value = True*/
const Property P3403_317 = {S35, 0, &V3403_317_57269, &P3403_316};	/*Property = Json*/
const Value V3403_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P3403_15 = {S37, 0, &V3403_15_31, &P3403_317};	/*Property = LayoutEngine*/
const Value V3403_344_57272 = {S2, NULL};	/*Value = True*/
const Property P3403_344 = {S38, 0, &V3403_344_57272, &P3403_15};	/*Property = Masking*/
const Value V3403_320_57290 = {S2, NULL};	/*Value = True*/
const Property P3403_320 = {S39, 0, &V3403_320_57290, &P3403_344};	/*Property = PostMessage*/
const Value V3403_321_57294 = {S0, NULL};	/*Value = False*/
const Property P3403_321 = {S40, 0, &V3403_321_57294, &P3403_320};	/*Property = Progress*/
const Value V3403_322_57296 = {S2, NULL};	/*Value = True*/
const Property P3403_322 = {S41, 0, &V3403_322_57296, &P3403_321};	/*Property = Prompts*/
const Value V3403_323_57300 = {S0, NULL};	/*Value = False*/
const Property P3403_323 = {S42, 0, &V3403_323_57300, &P3403_322};	/*Property = Selector*/
const Value V3403_324_57306 = {S0, NULL};	/*Value = False*/
const Property P3403_324 = {S43, 0, &V3403_324_57306, &P3403_323};	/*Property = Svg*/
const Value V3403_327_57315 = {S0, NULL};	/*Value = False*/
const Property P3403_327 = {S44, 0, &V3403_327_57315, &P3403_324};	/*Property = TouchEvents*/
const Value V3403_328_57318 = {S0, NULL};	/*Value = False*/
const Property P3403_328 = {S45, 0, &V3403_328_57318, &P3403_327};	/*Property = Track*/
const Value V3403_329_57323 = {S2, NULL};	/*Value = True*/
const Property P3403_329 = {S46, 0, &V3403_329_57323, &P3403_328};	/*Property = Video*/
const Value V3403_330_57326 = {S2, NULL};	/*Value = True*/
const Property P3403_330 = {S47, 0, &V3403_330_57326, &P3403_329};	/*Property = Viewport*/
const Value V3403_332_57339 = {S0, NULL};	/*Value = False*/
const Property P3403_332 = {S48, 0, &V3403_332_57339, &P3403_330};	/*Property = WebWorkers*/
const Value V3403_333_57342 = {S0, NULL};	/*Value = False*/
const Property P3403_333 = {S49, 0, &V3403_333_57342, &P3403_332};	/*Property = Xhr2*/
const Profile P3403 = {"3403", &P3403_333};	/*Profile ID - 3403*/
const Value V18479_1_1 = {S2, NULL};	/*Value = True*/
const Property P18479_1 = {S50, 0, &V18479_1_1, NULL};	/*Property = IsMobile*/
const Value V18479_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18479_5 = {S52, 0, &V18479_5_1227, &P18479_1};	/*Property = ScreenPixelsHeight*/
const Value V18479_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18479_4 = {S54, 0, &V18479_4_1229, &P18479_5};	/*Property = ScreenPixelsWidth*/
const Profile P18479 = {"18479", &P18479_4};	/*Profile ID - 18479*/
const Value V18497_1_1 = {S2, NULL};	/*Value = True*/
const Property P18497_1 = {S50, 0, &V18497_1_1, NULL};	/*Property = IsMobile*/
const Value V18497_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P18497_5 = {S52, 0, &V18497_5_1269, &P18497_1};	/*Property = ScreenPixelsHeight*/
const Value V18497_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18497_4 = {S54, 0, &V18497_4_1229, &P18497_5};	/*Property = ScreenPixelsWidth*/
const Profile P18497 = {"18497", &P18497_4};	/*Profile ID - 18497*/
const Value V18282_1_1 = {S2, NULL};	/*Value = True*/
const Property P18282_1 = {S50, 0, &V18282_1_1, NULL};	/*Property = IsMobile*/
const Value V18282_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18282_5 = {S52, 0, &V18282_5_1227, &P18282_1};	/*Property = ScreenPixelsHeight*/
const Value V18282_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P18282_4 = {S54, 0, &V18282_4_1477, &P18282_5};	/*Property = ScreenPixelsWidth*/
const Profile P18282 = {"18282", &P18282_4};	/*Profile ID - 18282*/
const Profile P18169 = {"18169", NULL};	/*Profile ID - 18169*/
const Value V23031_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_279 = {S1, 0, &V23031_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23031_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_282 = {S3, 0, &V23031_282_57155, &P23031_279};	/*Property = BlobBuilder*/
const Value V23031_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_350 = {S4, 0, &V23031_350_57446, &P23031_282};	/*Property = Canvas*/
const Value V23031_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_283 = {S5, 0, &V23031_283_57161, &P23031_350};	/*Property = CssBackground*/
const Value V23031_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_284 = {S6, 0, &V23031_284_57164, &P23031_283};	/*Property = CssBorderImage*/
const Value V23031_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_285 = {S7, 0, &V23031_285_57167, &P23031_284};	/*Property = CssCanvas*/
const Value V23031_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_286 = {S8, 0, &V23031_286_57170, &P23031_285};	/*Property = CssColor*/
const Value V23031_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_237 = {S9, 0, &V23031_237_55786, &P23031_286};	/*Property = CssColumn*/
const Value V23031_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_287 = {S10, 0, &V23031_287_57173, &P23031_237};	/*Property = CssFlexbox*/
const Value V23031_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_288 = {S11, 0, &V23031_288_57176, &P23031_287};	/*Property = CssFont*/
const Value V23031_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_289 = {S12, 0, &V23031_289_86899, &P23031_288};	/*Property = CssImages*/
const Value V23031_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_291 = {S13, 0, &V23031_291_57182, &P23031_289};	/*Property = CssMediaQueries*/
const Value V23031_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_292 = {S14, 0, &V23031_292_57185, &P23031_291};	/*Property = CssMinMax*/
const Value V23031_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_293 = {S15, 0, &V23031_293_57188, &P23031_292};	/*Property = CssOverflow*/
const Value V23031_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_294 = {S16, 0, &V23031_294_57191, &P23031_293};	/*Property = CssPosition*/
const Value V23031_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_295 = {S17, 0, &V23031_295_57194, &P23031_294};	/*Property = CssText*/
const Value V23031_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_297 = {S18, 0, &V23031_297_57197, &P23031_295};	/*Property = CssTransforms*/
const Value V23031_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_241 = {S19, 0, &V23031_241_55801, &P23031_297};	/*Property = CssTransitions*/
const Value V23031_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_298 = {S20, 0, &V23031_298_57208, &P23031_241};	/*Property = CssUI*/
const Value V23031_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_300 = {S21, 0, &V23031_300_57217, &P23031_298};	/*Property = DataSet*/
const Value V23031_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_301 = {S22, 0, &V23031_301_57220, &P23031_300};	/*Property = DataUrl*/
const Value V23031_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_302 = {S23, 0, &V23031_302_57223, &P23031_301};	/*Property = DeviceOrientation*/
const Value V23031_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_304 = {S24, 0, &V23031_304_57229, &P23031_302};	/*Property = FileReader*/
const Value V23031_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_305 = {S25, 0, &V23031_305_57232, &P23031_304};	/*Property = FileSaver*/
const Value V23031_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_306 = {S26, 0, &V23031_306_57238, &P23031_305};	/*Property = FileWriter*/
const Value V23031_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_307 = {S27, 0, &V23031_307_57241, &P23031_306};	/*Property = FormData*/
const Value V23031_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_309 = {S28, 0, &V23031_309_57247, &P23031_307};	/*Property = Fullscreen*/
const Value V23031_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_310 = {S29, 0, &V23031_310_57250, &P23031_309};	/*Property = GeoLocation*/
const Value V23031_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_312 = {S30, 0, &V23031_312_57256, &P23031_310};	/*Property = History*/
const Value V23031_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_314 = {S31, 0, &V23031_314_57262, &P23031_312};	/*Property = Html5*/
const Value V23031_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_313 = {S32, 0, &V23031_313_57259, &P23031_314};	/*Property = Html-Media-Capture*/
const Value V23031_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_315 = {S33, 0, &V23031_315_57265, &P23031_313};	/*Property = Iframe*/
const Value V23031_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_316 = {S34, 0, &V23031_316_57268, &P23031_315};	/*Property = IndexedDB*/
const Value V23031_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_317 = {S35, 0, &V23031_317_57271, &P23031_316};	/*Property = Json*/
const Value V23031_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P23031_15 = {S37, 0, &V23031_15_28, &P23031_317};	/*Property = LayoutEngine*/
const Value V23031_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_344 = {S38, 0, &V23031_344_57274, &P23031_15};	/*Property = Masking*/
const Value V23031_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_320 = {S39, 0, &V23031_320_57292, &P23031_344};	/*Property = PostMessage*/
const Value V23031_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_321 = {S40, 0, &V23031_321_57295, &P23031_320};	/*Property = Progress*/
const Value V23031_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_322 = {S41, 0, &V23031_322_57298, &P23031_321};	/*Property = Prompts*/
const Value V23031_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_323 = {S42, 0, &V23031_323_57301, &P23031_322};	/*Property = Selector*/
const Value V23031_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_324 = {S43, 0, &V23031_324_57307, &P23031_323};	/*Property = Svg*/
const Value V23031_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_327 = {S44, 0, &V23031_327_57316, &P23031_324};	/*Property = TouchEvents*/
const Value V23031_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_328 = {S45, 0, &V23031_328_57319, &P23031_327};	/*Property = Track*/
const Value V23031_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_329 = {S46, 0, &V23031_329_57325, &P23031_328};	/*Property = Video*/
const Value V23031_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_330 = {S47, 0, &V23031_330_57328, &P23031_329};	/*Property = Viewport*/
const Value V23031_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_332 = {S48, 0, &V23031_332_57340, &P23031_330};	/*Property = WebWorkers*/
const Value V23031_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23031_333 = {S49, 0, &V23031_333_57343, &P23031_332};	/*Property = Xhr2*/
const Profile P23031 = {"23031", &P23031_333};	/*Profile ID - 23031*/
const Profile P18155 = {"18155", NULL};	/*Profile ID - 18155*/
const Value V21905_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21905_279 = {S1, 0, &V21905_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21905_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21905_282 = {S3, 0, &V21905_282_57154, &P21905_279};	/*Property = BlobBuilder*/
const Value V21905_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21905_350 = {S4, 0, &V21905_350_57444, &P21905_282};	/*Property = Canvas*/
const Value V21905_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21905_283 = {S5, 0, &V21905_283_57159, &P21905_350};	/*Property = CssBackground*/
const Value V21905_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21905_284 = {S6, 0, &V21905_284_57163, &P21905_283};	/*Property = CssBorderImage*/
const Value V21905_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21905_285 = {S7, 0, &V21905_285_57166, &P21905_284};	/*Property = CssCanvas*/
const Value V21905_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21905_286 = {S8, 0, &V21905_286_57168, &P21905_285};	/*Property = CssColor*/
const Value V21905_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21905_237 = {S9, 0, &V21905_237_55784, &P21905_286};	/*Property = CssColumn*/
const Value V21905_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21905_287 = {S10, 0, &V21905_287_57172, &P21905_237};	/*Property = CssFlexbox*/
const Value V21905_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21905_288 = {S11, 0, &V21905_288_57175, &P21905_287};	/*Property = CssFont*/
const Value V21905_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21905_289 = {S12, 0, &V21905_289_57759, &P21905_288};	/*Property = CssImages*/
const Value V21905_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21905_291 = {S13, 0, &V21905_291_57181, &P21905_289};	/*Property = CssMediaQueries*/
const Value V21905_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21905_292 = {S14, 0, &V21905_292_57183, &P21905_291};	/*Property = CssMinMax*/
const Value V21905_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21905_293 = {S15, 0, &V21905_293_57187, &P21905_292};	/*Property = CssOverflow*/
const Value V21905_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21905_294 = {S16, 0, &V21905_294_57190, &P21905_293};	/*Property = CssPosition*/
const Value V21905_295_57193 = {S0, NULL};	/*Value = False*/
const Property P21905_295 = {S17, 0, &V21905_295_57193, &P21905_294};	/*Property = CssText*/
const Value V21905_297_57196 = {S0, NULL};	/*Value = False*/
const Property P21905_297 = {S18, 0, &V21905_297_57196, &P21905_295};	/*Property = CssTransforms*/
const Value V21905_241_55800 = {S0, NULL};	/*Value = False*/
const Property P21905_241 = {S19, 0, &V21905_241_55800, &P21905_297};	/*Property = CssTransitions*/
const Value V21905_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21905_298 = {S20, 0, &V21905_298_57206, &P21905_241};	/*Property = CssUI*/
const Value V21905_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21905_300 = {S21, 0, &V21905_300_57216, &P21905_298};	/*Property = DataSet*/
const Value V21905_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21905_301 = {S22, 0, &V21905_301_57218, &P21905_300};	/*Property = DataUrl*/
const Value V21905_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21905_302 = {S23, 0, &V21905_302_57222, &P21905_301};	/*Property = DeviceOrientation*/
const Value V21905_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21905_304 = {S24, 0, &V21905_304_57228, &P21905_302};	/*Property = FileReader*/
const Value V21905_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21905_305 = {S25, 0, &V21905_305_57231, &P21905_304};	/*Property = FileSaver*/
const Value V21905_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21905_306 = {S26, 0, &V21905_306_57237, &P21905_305};	/*Property = FileWriter*/
const Value V21905_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21905_307 = {S27, 0, &V21905_307_57240, &P21905_306};	/*Property = FormData*/
const Value V21905_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21905_309 = {S28, 0, &V21905_309_57246, &P21905_307};	/*Property = Fullscreen*/
const Value V21905_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21905_310 = {S29, 0, &V21905_310_57248, &P21905_309};	/*Property = GeoLocation*/
const Value V21905_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21905_312 = {S30, 0, &V21905_312_57255, &P21905_310};	/*Property = History*/
const Value V21905_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21905_314 = {S31, 0, &V21905_314_57261, &P21905_312};	/*Property = Html5*/
const Value V21905_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21905_313 = {S32, 0, &V21905_313_57258, &P21905_314};	/*Property = Html-Media-Capture*/
const Value V21905_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21905_315 = {S33, 0, &V21905_315_57264, &P21905_313};	/*Property = Iframe*/
const Value V21905_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21905_316 = {S34, 0, &V21905_316_57267, &P21905_315};	/*Property = IndexedDB*/
const Value V21905_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21905_317 = {S35, 0, &V21905_317_57269, &P21905_316};	/*Property = Json*/
const Value V21905_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21905_15 = {S37, 0, &V21905_15_28, &P21905_317};	/*Property = LayoutEngine*/
const Value V21905_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21905_344 = {S38, 0, &V21905_344_57272, &P21905_15};	/*Property = Masking*/
const Value V21905_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21905_320 = {S39, 0, &V21905_320_57290, &P21905_344};	/*Property = PostMessage*/
const Value V21905_321_57294 = {S0, NULL};	/*Value = False*/
const Property P21905_321 = {S40, 0, &V21905_321_57294, &P21905_320};	/*Property = Progress*/
const Value V21905_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21905_322 = {S41, 0, &V21905_322_57296, &P21905_321};	/*Property = Prompts*/
const Value V21905_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21905_323 = {S42, 0, &V21905_323_57299, &P21905_322};	/*Property = Selector*/
const Value V21905_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21905_324 = {S43, 0, &V21905_324_57306, &P21905_323};	/*Property = Svg*/
const Value V21905_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21905_327 = {S44, 0, &V21905_327_57315, &P21905_324};	/*Property = TouchEvents*/
const Value V21905_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21905_328 = {S45, 0, &V21905_328_57318, &P21905_327};	/*Property = Track*/
const Value V21905_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21905_329 = {S46, 0, &V21905_329_57323, &P21905_328};	/*Property = Video*/
const Value V21905_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21905_330 = {S47, 0, &V21905_330_57326, &P21905_329};	/*Property = Viewport*/
const Value V21905_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21905_332 = {S48, 0, &V21905_332_57339, &P21905_330};	/*Property = WebWorkers*/
const Value V21905_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21905_333 = {S49, 0, &V21905_333_57342, &P21905_332};	/*Property = Xhr2*/
const Profile P21905 = {"21905", &P21905_333};	/*Profile ID - 21905*/
const Value V17698_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_279 = {S1, 0, &V17698_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17698_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_282 = {S3, 0, &V17698_282_57155, &P17698_279};	/*Property = BlobBuilder*/
const Value V17698_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_350 = {S4, 0, &V17698_350_57446, &P17698_282};	/*Property = Canvas*/
const Value V17698_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_283 = {S5, 0, &V17698_283_57161, &P17698_350};	/*Property = CssBackground*/
const Value V17698_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_284 = {S6, 0, &V17698_284_57164, &P17698_283};	/*Property = CssBorderImage*/
const Value V17698_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_285 = {S7, 0, &V17698_285_57167, &P17698_284};	/*Property = CssCanvas*/
const Value V17698_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_286 = {S8, 0, &V17698_286_57170, &P17698_285};	/*Property = CssColor*/
const Value V17698_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_237 = {S9, 0, &V17698_237_55786, &P17698_286};	/*Property = CssColumn*/
const Value V17698_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_287 = {S10, 0, &V17698_287_57173, &P17698_237};	/*Property = CssFlexbox*/
const Value V17698_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_288 = {S11, 0, &V17698_288_57176, &P17698_287};	/*Property = CssFont*/
const Value V17698_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_289 = {S12, 0, &V17698_289_86899, &P17698_288};	/*Property = CssImages*/
const Value V17698_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_291 = {S13, 0, &V17698_291_57182, &P17698_289};	/*Property = CssMediaQueries*/
const Value V17698_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_292 = {S14, 0, &V17698_292_57185, &P17698_291};	/*Property = CssMinMax*/
const Value V17698_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_293 = {S15, 0, &V17698_293_57188, &P17698_292};	/*Property = CssOverflow*/
const Value V17698_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_294 = {S16, 0, &V17698_294_57191, &P17698_293};	/*Property = CssPosition*/
const Value V17698_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_295 = {S17, 0, &V17698_295_57194, &P17698_294};	/*Property = CssText*/
const Value V17698_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_297 = {S18, 0, &V17698_297_57197, &P17698_295};	/*Property = CssTransforms*/
const Value V17698_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_241 = {S19, 0, &V17698_241_55801, &P17698_297};	/*Property = CssTransitions*/
const Value V17698_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_298 = {S20, 0, &V17698_298_57208, &P17698_241};	/*Property = CssUI*/
const Value V17698_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_300 = {S21, 0, &V17698_300_57217, &P17698_298};	/*Property = DataSet*/
const Value V17698_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_301 = {S22, 0, &V17698_301_57220, &P17698_300};	/*Property = DataUrl*/
const Value V17698_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_302 = {S23, 0, &V17698_302_57223, &P17698_301};	/*Property = DeviceOrientation*/
const Value V17698_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_304 = {S24, 0, &V17698_304_57229, &P17698_302};	/*Property = FileReader*/
const Value V17698_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_305 = {S25, 0, &V17698_305_57232, &P17698_304};	/*Property = FileSaver*/
const Value V17698_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_306 = {S26, 0, &V17698_306_57238, &P17698_305};	/*Property = FileWriter*/
const Value V17698_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_307 = {S27, 0, &V17698_307_57241, &P17698_306};	/*Property = FormData*/
const Value V17698_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_309 = {S28, 0, &V17698_309_57247, &P17698_307};	/*Property = Fullscreen*/
const Value V17698_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_310 = {S29, 0, &V17698_310_57250, &P17698_309};	/*Property = GeoLocation*/
const Value V17698_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_312 = {S30, 0, &V17698_312_57256, &P17698_310};	/*Property = History*/
const Value V17698_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_314 = {S31, 0, &V17698_314_57262, &P17698_312};	/*Property = Html5*/
const Value V17698_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_313 = {S32, 0, &V17698_313_57259, &P17698_314};	/*Property = Html-Media-Capture*/
const Value V17698_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_315 = {S33, 0, &V17698_315_57265, &P17698_313};	/*Property = Iframe*/
const Value V17698_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_316 = {S34, 0, &V17698_316_57268, &P17698_315};	/*Property = IndexedDB*/
const Value V17698_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_317 = {S35, 0, &V17698_317_57271, &P17698_316};	/*Property = Json*/
const Value V17698_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P17698_15 = {S37, 0, &V17698_15_29, &P17698_317};	/*Property = LayoutEngine*/
const Value V17698_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_344 = {S38, 0, &V17698_344_57274, &P17698_15};	/*Property = Masking*/
const Value V17698_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_320 = {S39, 0, &V17698_320_57292, &P17698_344};	/*Property = PostMessage*/
const Value V17698_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_321 = {S40, 0, &V17698_321_57295, &P17698_320};	/*Property = Progress*/
const Value V17698_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_322 = {S41, 0, &V17698_322_57298, &P17698_321};	/*Property = Prompts*/
const Value V17698_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_323 = {S42, 0, &V17698_323_57301, &P17698_322};	/*Property = Selector*/
const Value V17698_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_324 = {S43, 0, &V17698_324_57307, &P17698_323};	/*Property = Svg*/
const Value V17698_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_327 = {S44, 0, &V17698_327_57316, &P17698_324};	/*Property = TouchEvents*/
const Value V17698_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_328 = {S45, 0, &V17698_328_57319, &P17698_327};	/*Property = Track*/
const Value V17698_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_329 = {S46, 0, &V17698_329_57325, &P17698_328};	/*Property = Video*/
const Value V17698_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_330 = {S47, 0, &V17698_330_57328, &P17698_329};	/*Property = Viewport*/
const Value V17698_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_332 = {S48, 0, &V17698_332_57340, &P17698_330};	/*Property = WebWorkers*/
const Value V17698_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17698_333 = {S49, 0, &V17698_333_57343, &P17698_332};	/*Property = Xhr2*/
const Profile P17698 = {"17698", &P17698_333};	/*Profile ID - 17698*/
const Value V22417_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22417_279 = {S1, 0, &V22417_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22417_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22417_282 = {S3, 0, &V22417_282_57153, &P22417_279};	/*Property = BlobBuilder*/
const Value V22417_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22417_350 = {S4, 0, &V22417_350_57444, &P22417_282};	/*Property = Canvas*/
const Value V22417_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22417_283 = {S5, 0, &V22417_283_57159, &P22417_350};	/*Property = CssBackground*/
const Value V22417_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22417_284 = {S6, 0, &V22417_284_57163, &P22417_283};	/*Property = CssBorderImage*/
const Value V22417_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22417_285 = {S7, 0, &V22417_285_57166, &P22417_284};	/*Property = CssCanvas*/
const Value V22417_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22417_286 = {S8, 0, &V22417_286_57168, &P22417_285};	/*Property = CssColor*/
const Value V22417_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22417_237 = {S9, 0, &V22417_237_55784, &P22417_286};	/*Property = CssColumn*/
const Value V22417_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22417_287 = {S10, 0, &V22417_287_57172, &P22417_237};	/*Property = CssFlexbox*/
const Value V22417_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22417_288 = {S11, 0, &V22417_288_57175, &P22417_287};	/*Property = CssFont*/
const Value V22417_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22417_289 = {S12, 0, &V22417_289_57759, &P22417_288};	/*Property = CssImages*/
const Value V22417_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22417_291 = {S13, 0, &V22417_291_57180, &P22417_289};	/*Property = CssMediaQueries*/
const Value V22417_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22417_292 = {S14, 0, &V22417_292_57183, &P22417_291};	/*Property = CssMinMax*/
const Value V22417_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22417_293 = {S15, 0, &V22417_293_57187, &P22417_292};	/*Property = CssOverflow*/
const Value V22417_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22417_294 = {S16, 0, &V22417_294_57189, &P22417_293};	/*Property = CssPosition*/
const Value V22417_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22417_295 = {S17, 0, &V22417_295_57192, &P22417_294};	/*Property = CssText*/
const Value V22417_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22417_297 = {S18, 0, &V22417_297_57195, &P22417_295};	/*Property = CssTransforms*/
const Value V22417_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22417_241 = {S19, 0, &V22417_241_55799, &P22417_297};	/*Property = CssTransitions*/
const Value V22417_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22417_298 = {S20, 0, &V22417_298_57206, &P22417_241};	/*Property = CssUI*/
const Value V22417_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22417_300 = {S21, 0, &V22417_300_57216, &P22417_298};	/*Property = DataSet*/
const Value V22417_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22417_301 = {S22, 0, &V22417_301_57218, &P22417_300};	/*Property = DataUrl*/
const Value V22417_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22417_302 = {S23, 0, &V22417_302_57222, &P22417_301};	/*Property = DeviceOrientation*/
const Value V22417_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22417_304 = {S24, 0, &V22417_304_57227, &P22417_302};	/*Property = FileReader*/
const Value V22417_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22417_305 = {S25, 0, &V22417_305_57231, &P22417_304};	/*Property = FileSaver*/
const Value V22417_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22417_306 = {S26, 0, &V22417_306_57237, &P22417_305};	/*Property = FileWriter*/
const Value V22417_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22417_307 = {S27, 0, &V22417_307_57239, &P22417_306};	/*Property = FormData*/
const Value V22417_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22417_309 = {S28, 0, &V22417_309_57246, &P22417_307};	/*Property = Fullscreen*/
const Value V22417_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22417_310 = {S29, 0, &V22417_310_57248, &P22417_309};	/*Property = GeoLocation*/
const Value V22417_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22417_312 = {S30, 0, &V22417_312_57254, &P22417_310};	/*Property = History*/
const Value V22417_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22417_314 = {S31, 0, &V22417_314_57261, &P22417_312};	/*Property = Html5*/
const Value V22417_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22417_313 = {S32, 0, &V22417_313_57258, &P22417_314};	/*Property = Html-Media-Capture*/
const Value V22417_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22417_315 = {S33, 0, &V22417_315_57264, &P22417_313};	/*Property = Iframe*/
const Value V22417_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22417_316 = {S34, 0, &V22417_316_57266, &P22417_315};	/*Property = IndexedDB*/
const Value V22417_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22417_317 = {S35, 0, &V22417_317_57269, &P22417_316};	/*Property = Json*/
const Value V22417_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22417_15 = {S37, 0, &V22417_15_28, &P22417_317};	/*Property = LayoutEngine*/
const Value V22417_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22417_344 = {S38, 0, &V22417_344_57272, &P22417_15};	/*Property = Masking*/
const Value V22417_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22417_320 = {S39, 0, &V22417_320_57290, &P22417_344};	/*Property = PostMessage*/
const Value V22417_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22417_321 = {S40, 0, &V22417_321_57293, &P22417_320};	/*Property = Progress*/
const Value V22417_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22417_322 = {S41, 0, &V22417_322_57296, &P22417_321};	/*Property = Prompts*/
const Value V22417_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22417_323 = {S42, 0, &V22417_323_57299, &P22417_322};	/*Property = Selector*/
const Value V22417_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22417_324 = {S43, 0, &V22417_324_57306, &P22417_323};	/*Property = Svg*/
const Value V22417_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22417_327 = {S44, 0, &V22417_327_57315, &P22417_324};	/*Property = TouchEvents*/
const Value V22417_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22417_328 = {S45, 0, &V22417_328_57318, &P22417_327};	/*Property = Track*/
const Value V22417_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22417_329 = {S46, 0, &V22417_329_57323, &P22417_328};	/*Property = Video*/
const Value V22417_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22417_330 = {S47, 0, &V22417_330_57326, &P22417_329};	/*Property = Viewport*/
const Value V22417_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22417_332 = {S48, 0, &V22417_332_57338, &P22417_330};	/*Property = WebWorkers*/
const Value V22417_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22417_333 = {S49, 0, &V22417_333_57342, &P22417_332};	/*Property = Xhr2*/
const Profile P22417 = {"22417", &P22417_333};	/*Profile ID - 22417*/
const Value V22581_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_279 = {S1, 0, &V22581_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22581_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_282 = {S3, 0, &V22581_282_57155, &P22581_279};	/*Property = BlobBuilder*/
const Value V22581_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_350 = {S4, 0, &V22581_350_57446, &P22581_282};	/*Property = Canvas*/
const Value V22581_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_283 = {S5, 0, &V22581_283_57161, &P22581_350};	/*Property = CssBackground*/
const Value V22581_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_284 = {S6, 0, &V22581_284_57164, &P22581_283};	/*Property = CssBorderImage*/
const Value V22581_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_285 = {S7, 0, &V22581_285_57167, &P22581_284};	/*Property = CssCanvas*/
const Value V22581_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_286 = {S8, 0, &V22581_286_57170, &P22581_285};	/*Property = CssColor*/
const Value V22581_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_237 = {S9, 0, &V22581_237_55786, &P22581_286};	/*Property = CssColumn*/
const Value V22581_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_287 = {S10, 0, &V22581_287_57173, &P22581_237};	/*Property = CssFlexbox*/
const Value V22581_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_288 = {S11, 0, &V22581_288_57176, &P22581_287};	/*Property = CssFont*/
const Value V22581_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_289 = {S12, 0, &V22581_289_86899, &P22581_288};	/*Property = CssImages*/
const Value V22581_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_291 = {S13, 0, &V22581_291_57182, &P22581_289};	/*Property = CssMediaQueries*/
const Value V22581_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_292 = {S14, 0, &V22581_292_57185, &P22581_291};	/*Property = CssMinMax*/
const Value V22581_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_293 = {S15, 0, &V22581_293_57188, &P22581_292};	/*Property = CssOverflow*/
const Value V22581_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_294 = {S16, 0, &V22581_294_57191, &P22581_293};	/*Property = CssPosition*/
const Value V22581_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_295 = {S17, 0, &V22581_295_57194, &P22581_294};	/*Property = CssText*/
const Value V22581_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_297 = {S18, 0, &V22581_297_57197, &P22581_295};	/*Property = CssTransforms*/
const Value V22581_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_241 = {S19, 0, &V22581_241_55801, &P22581_297};	/*Property = CssTransitions*/
const Value V22581_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_298 = {S20, 0, &V22581_298_57208, &P22581_241};	/*Property = CssUI*/
const Value V22581_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_300 = {S21, 0, &V22581_300_57217, &P22581_298};	/*Property = DataSet*/
const Value V22581_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_301 = {S22, 0, &V22581_301_57220, &P22581_300};	/*Property = DataUrl*/
const Value V22581_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_302 = {S23, 0, &V22581_302_57223, &P22581_301};	/*Property = DeviceOrientation*/
const Value V22581_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_304 = {S24, 0, &V22581_304_57229, &P22581_302};	/*Property = FileReader*/
const Value V22581_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_305 = {S25, 0, &V22581_305_57232, &P22581_304};	/*Property = FileSaver*/
const Value V22581_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_306 = {S26, 0, &V22581_306_57238, &P22581_305};	/*Property = FileWriter*/
const Value V22581_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_307 = {S27, 0, &V22581_307_57241, &P22581_306};	/*Property = FormData*/
const Value V22581_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_309 = {S28, 0, &V22581_309_57247, &P22581_307};	/*Property = Fullscreen*/
const Value V22581_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_310 = {S29, 0, &V22581_310_57250, &P22581_309};	/*Property = GeoLocation*/
const Value V22581_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_312 = {S30, 0, &V22581_312_57256, &P22581_310};	/*Property = History*/
const Value V22581_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_314 = {S31, 0, &V22581_314_57262, &P22581_312};	/*Property = Html5*/
const Value V22581_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_313 = {S32, 0, &V22581_313_57259, &P22581_314};	/*Property = Html-Media-Capture*/
const Value V22581_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_315 = {S33, 0, &V22581_315_57265, &P22581_313};	/*Property = Iframe*/
const Value V22581_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_316 = {S34, 0, &V22581_316_57268, &P22581_315};	/*Property = IndexedDB*/
const Value V22581_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_317 = {S35, 0, &V22581_317_57271, &P22581_316};	/*Property = Json*/
const Value V22581_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P22581_15 = {S37, 0, &V22581_15_54167, &P22581_317};	/*Property = LayoutEngine*/
const Value V22581_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_344 = {S38, 0, &V22581_344_57274, &P22581_15};	/*Property = Masking*/
const Value V22581_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_320 = {S39, 0, &V22581_320_57292, &P22581_344};	/*Property = PostMessage*/
const Value V22581_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_321 = {S40, 0, &V22581_321_57295, &P22581_320};	/*Property = Progress*/
const Value V22581_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_322 = {S41, 0, &V22581_322_57298, &P22581_321};	/*Property = Prompts*/
const Value V22581_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_323 = {S42, 0, &V22581_323_57301, &P22581_322};	/*Property = Selector*/
const Value V22581_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_324 = {S43, 0, &V22581_324_57307, &P22581_323};	/*Property = Svg*/
const Value V22581_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_327 = {S44, 0, &V22581_327_57316, &P22581_324};	/*Property = TouchEvents*/
const Value V22581_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_328 = {S45, 0, &V22581_328_57319, &P22581_327};	/*Property = Track*/
const Value V22581_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_329 = {S46, 0, &V22581_329_57325, &P22581_328};	/*Property = Video*/
const Value V22581_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_330 = {S47, 0, &V22581_330_57328, &P22581_329};	/*Property = Viewport*/
const Value V22581_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_332 = {S48, 0, &V22581_332_57340, &P22581_330};	/*Property = WebWorkers*/
const Value V22581_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22581_333 = {S49, 0, &V22581_333_57343, &P22581_332};	/*Property = Xhr2*/
const Profile P22581 = {"22581", &P22581_333};	/*Profile ID - 22581*/
const Value V24201_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24201_279 = {S1, 0, &V24201_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24201_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24201_282 = {S3, 0, &V24201_282_57154, &P24201_279};	/*Property = BlobBuilder*/
const Value V24201_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24201_350 = {S4, 0, &V24201_350_57444, &P24201_282};	/*Property = Canvas*/
const Value V24201_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24201_283 = {S5, 0, &V24201_283_57159, &P24201_350};	/*Property = CssBackground*/
const Value V24201_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24201_284 = {S6, 0, &V24201_284_57162, &P24201_283};	/*Property = CssBorderImage*/
const Value V24201_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24201_285 = {S7, 0, &V24201_285_57166, &P24201_284};	/*Property = CssCanvas*/
const Value V24201_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24201_286 = {S8, 0, &V24201_286_57168, &P24201_285};	/*Property = CssColor*/
const Value V24201_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24201_237 = {S9, 0, &V24201_237_55786, &P24201_286};	/*Property = CssColumn*/
const Value V24201_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24201_287 = {S10, 0, &V24201_287_57172, &P24201_237};	/*Property = CssFlexbox*/
const Value V24201_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24201_288 = {S11, 0, &V24201_288_57175, &P24201_287};	/*Property = CssFont*/
const Value V24201_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24201_289 = {S12, 0, &V24201_289_57759, &P24201_288};	/*Property = CssImages*/
const Value V24201_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24201_291 = {S13, 0, &V24201_291_57180, &P24201_289};	/*Property = CssMediaQueries*/
const Value V24201_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24201_292 = {S14, 0, &V24201_292_57183, &P24201_291};	/*Property = CssMinMax*/
const Value V24201_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24201_293 = {S15, 0, &V24201_293_57187, &P24201_292};	/*Property = CssOverflow*/
const Value V24201_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24201_294 = {S16, 0, &V24201_294_57189, &P24201_293};	/*Property = CssPosition*/
const Value V24201_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24201_295 = {S17, 0, &V24201_295_57192, &P24201_294};	/*Property = CssText*/
const Value V24201_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24201_297 = {S18, 0, &V24201_297_57195, &P24201_295};	/*Property = CssTransforms*/
const Value V24201_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24201_241 = {S19, 0, &V24201_241_55799, &P24201_297};	/*Property = CssTransitions*/
const Value V24201_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24201_298 = {S20, 0, &V24201_298_57206, &P24201_241};	/*Property = CssUI*/
const Value V24201_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24201_300 = {S21, 0, &V24201_300_57215, &P24201_298};	/*Property = DataSet*/
const Value V24201_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24201_301 = {S22, 0, &V24201_301_57218, &P24201_300};	/*Property = DataUrl*/
const Value V24201_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24201_302 = {S23, 0, &V24201_302_57221, &P24201_301};	/*Property = DeviceOrientation*/
const Value V24201_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24201_304 = {S24, 0, &V24201_304_57227, &P24201_302};	/*Property = FileReader*/
const Value V24201_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24201_305 = {S25, 0, &V24201_305_57231, &P24201_304};	/*Property = FileSaver*/
const Value V24201_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24201_306 = {S26, 0, &V24201_306_57237, &P24201_305};	/*Property = FileWriter*/
const Value V24201_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24201_307 = {S27, 0, &V24201_307_57239, &P24201_306};	/*Property = FormData*/
const Value V24201_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24201_309 = {S28, 0, &V24201_309_57246, &P24201_307};	/*Property = Fullscreen*/
const Value V24201_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24201_310 = {S29, 0, &V24201_310_57248, &P24201_309};	/*Property = GeoLocation*/
const Value V24201_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24201_312 = {S30, 0, &V24201_312_57254, &P24201_310};	/*Property = History*/
const Value V24201_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24201_314 = {S31, 0, &V24201_314_57260, &P24201_312};	/*Property = Html5*/
const Value V24201_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24201_313 = {S32, 0, &V24201_313_57257, &P24201_314};	/*Property = Html-Media-Capture*/
const Value V24201_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24201_315 = {S33, 0, &V24201_315_57264, &P24201_313};	/*Property = Iframe*/
const Value V24201_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24201_316 = {S34, 0, &V24201_316_57266, &P24201_315};	/*Property = IndexedDB*/
const Value V24201_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24201_317 = {S35, 0, &V24201_317_57269, &P24201_316};	/*Property = Json*/
const Value V24201_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24201_15 = {S37, 0, &V24201_15_54167, &P24201_317};	/*Property = LayoutEngine*/
const Value V24201_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24201_344 = {S38, 0, &V24201_344_57272, &P24201_15};	/*Property = Masking*/
const Value V24201_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24201_320 = {S39, 0, &V24201_320_57290, &P24201_344};	/*Property = PostMessage*/
const Value V24201_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24201_321 = {S40, 0, &V24201_321_57293, &P24201_320};	/*Property = Progress*/
const Value V24201_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24201_322 = {S41, 0, &V24201_322_57296, &P24201_321};	/*Property = Prompts*/
const Value V24201_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24201_323 = {S42, 0, &V24201_323_57299, &P24201_322};	/*Property = Selector*/
const Value V24201_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24201_324 = {S43, 0, &V24201_324_57306, &P24201_323};	/*Property = Svg*/
const Value V24201_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24201_327 = {S44, 0, &V24201_327_57314, &P24201_324};	/*Property = TouchEvents*/
const Value V24201_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24201_328 = {S45, 0, &V24201_328_57318, &P24201_327};	/*Property = Track*/
const Value V24201_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24201_329 = {S46, 0, &V24201_329_57323, &P24201_328};	/*Property = Video*/
const Value V24201_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24201_330 = {S47, 0, &V24201_330_57326, &P24201_329};	/*Property = Viewport*/
const Value V24201_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24201_332 = {S48, 0, &V24201_332_57339, &P24201_330};	/*Property = WebWorkers*/
const Value V24201_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24201_333 = {S49, 0, &V24201_333_57342, &P24201_332};	/*Property = Xhr2*/
const Profile P24201 = {"24201", &P24201_333};	/*Profile ID - 24201*/
const Value V17323_1_1 = {S2, NULL};	/*Value = True*/
const Property P17323_1 = {S50, 0, &V17323_1_1, NULL};	/*Property = IsMobile*/
const Value V17323_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P17323_5 = {S52, 0, &V17323_5_9981, &P17323_1};	/*Property = ScreenPixelsHeight*/
const Value V17323_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P17323_4 = {S54, 0, &V17323_4_2336, &P17323_5};	/*Property = ScreenPixelsWidth*/
const Profile P17323 = {"17323", &P17323_4};	/*Profile ID - 17323*/
const Value V22419_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22419_279 = {S1, 0, &V22419_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22419_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22419_282 = {S3, 0, &V22419_282_57153, &P22419_279};	/*Property = BlobBuilder*/
const Value V22419_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22419_350 = {S4, 0, &V22419_350_57444, &P22419_282};	/*Property = Canvas*/
const Value V22419_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22419_283 = {S5, 0, &V22419_283_57159, &P22419_350};	/*Property = CssBackground*/
const Value V22419_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22419_284 = {S6, 0, &V22419_284_57163, &P22419_283};	/*Property = CssBorderImage*/
const Value V22419_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22419_285 = {S7, 0, &V22419_285_57166, &P22419_284};	/*Property = CssCanvas*/
const Value V22419_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22419_286 = {S8, 0, &V22419_286_57168, &P22419_285};	/*Property = CssColor*/
const Value V22419_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22419_237 = {S9, 0, &V22419_237_55784, &P22419_286};	/*Property = CssColumn*/
const Value V22419_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22419_287 = {S10, 0, &V22419_287_57172, &P22419_237};	/*Property = CssFlexbox*/
const Value V22419_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22419_288 = {S11, 0, &V22419_288_57175, &P22419_287};	/*Property = CssFont*/
const Value V22419_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22419_289 = {S12, 0, &V22419_289_57759, &P22419_288};	/*Property = CssImages*/
const Value V22419_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22419_291 = {S13, 0, &V22419_291_57180, &P22419_289};	/*Property = CssMediaQueries*/
const Value V22419_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22419_292 = {S14, 0, &V22419_292_57183, &P22419_291};	/*Property = CssMinMax*/
const Value V22419_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22419_293 = {S15, 0, &V22419_293_57187, &P22419_292};	/*Property = CssOverflow*/
const Value V22419_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22419_294 = {S16, 0, &V22419_294_57189, &P22419_293};	/*Property = CssPosition*/
const Value V22419_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22419_295 = {S17, 0, &V22419_295_57192, &P22419_294};	/*Property = CssText*/
const Value V22419_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22419_297 = {S18, 0, &V22419_297_57195, &P22419_295};	/*Property = CssTransforms*/
const Value V22419_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22419_241 = {S19, 0, &V22419_241_55799, &P22419_297};	/*Property = CssTransitions*/
const Value V22419_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22419_298 = {S20, 0, &V22419_298_57206, &P22419_241};	/*Property = CssUI*/
const Value V22419_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22419_300 = {S21, 0, &V22419_300_57216, &P22419_298};	/*Property = DataSet*/
const Value V22419_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22419_301 = {S22, 0, &V22419_301_57218, &P22419_300};	/*Property = DataUrl*/
const Value V22419_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22419_302 = {S23, 0, &V22419_302_57222, &P22419_301};	/*Property = DeviceOrientation*/
const Value V22419_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22419_304 = {S24, 0, &V22419_304_57227, &P22419_302};	/*Property = FileReader*/
const Value V22419_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22419_305 = {S25, 0, &V22419_305_57231, &P22419_304};	/*Property = FileSaver*/
const Value V22419_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22419_306 = {S26, 0, &V22419_306_57237, &P22419_305};	/*Property = FileWriter*/
const Value V22419_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22419_307 = {S27, 0, &V22419_307_57239, &P22419_306};	/*Property = FormData*/
const Value V22419_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22419_309 = {S28, 0, &V22419_309_57246, &P22419_307};	/*Property = Fullscreen*/
const Value V22419_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22419_310 = {S29, 0, &V22419_310_57248, &P22419_309};	/*Property = GeoLocation*/
const Value V22419_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22419_312 = {S30, 0, &V22419_312_57254, &P22419_310};	/*Property = History*/
const Value V22419_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22419_314 = {S31, 0, &V22419_314_57261, &P22419_312};	/*Property = Html5*/
const Value V22419_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22419_313 = {S32, 0, &V22419_313_57258, &P22419_314};	/*Property = Html-Media-Capture*/
const Value V22419_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22419_315 = {S33, 0, &V22419_315_57264, &P22419_313};	/*Property = Iframe*/
const Value V22419_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22419_316 = {S34, 0, &V22419_316_57266, &P22419_315};	/*Property = IndexedDB*/
const Value V22419_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22419_317 = {S35, 0, &V22419_317_57269, &P22419_316};	/*Property = Json*/
const Value V22419_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22419_15 = {S37, 0, &V22419_15_28, &P22419_317};	/*Property = LayoutEngine*/
const Value V22419_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22419_344 = {S38, 0, &V22419_344_57272, &P22419_15};	/*Property = Masking*/
const Value V22419_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22419_320 = {S39, 0, &V22419_320_57290, &P22419_344};	/*Property = PostMessage*/
const Value V22419_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22419_321 = {S40, 0, &V22419_321_57293, &P22419_320};	/*Property = Progress*/
const Value V22419_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22419_322 = {S41, 0, &V22419_322_57296, &P22419_321};	/*Property = Prompts*/
const Value V22419_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22419_323 = {S42, 0, &V22419_323_57299, &P22419_322};	/*Property = Selector*/
const Value V22419_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22419_324 = {S43, 0, &V22419_324_57306, &P22419_323};	/*Property = Svg*/
const Value V22419_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22419_327 = {S44, 0, &V22419_327_57315, &P22419_324};	/*Property = TouchEvents*/
const Value V22419_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22419_328 = {S45, 0, &V22419_328_57318, &P22419_327};	/*Property = Track*/
const Value V22419_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22419_329 = {S46, 0, &V22419_329_57323, &P22419_328};	/*Property = Video*/
const Value V22419_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22419_330 = {S47, 0, &V22419_330_57326, &P22419_329};	/*Property = Viewport*/
const Value V22419_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22419_332 = {S48, 0, &V22419_332_57338, &P22419_330};	/*Property = WebWorkers*/
const Value V22419_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22419_333 = {S49, 0, &V22419_333_57342, &P22419_332};	/*Property = Xhr2*/
const Profile P22419 = {"22419", &P22419_333};	/*Profile ID - 22419*/
const Value V22420_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22420_279 = {S1, 0, &V22420_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22420_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22420_282 = {S3, 0, &V22420_282_57153, &P22420_279};	/*Property = BlobBuilder*/
const Value V22420_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22420_350 = {S4, 0, &V22420_350_57444, &P22420_282};	/*Property = Canvas*/
const Value V22420_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22420_283 = {S5, 0, &V22420_283_57159, &P22420_350};	/*Property = CssBackground*/
const Value V22420_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22420_284 = {S6, 0, &V22420_284_57163, &P22420_283};	/*Property = CssBorderImage*/
const Value V22420_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22420_285 = {S7, 0, &V22420_285_57166, &P22420_284};	/*Property = CssCanvas*/
const Value V22420_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22420_286 = {S8, 0, &V22420_286_57168, &P22420_285};	/*Property = CssColor*/
const Value V22420_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22420_237 = {S9, 0, &V22420_237_55784, &P22420_286};	/*Property = CssColumn*/
const Value V22420_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22420_287 = {S10, 0, &V22420_287_57172, &P22420_237};	/*Property = CssFlexbox*/
const Value V22420_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22420_288 = {S11, 0, &V22420_288_57175, &P22420_287};	/*Property = CssFont*/
const Value V22420_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22420_289 = {S12, 0, &V22420_289_57759, &P22420_288};	/*Property = CssImages*/
const Value V22420_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22420_291 = {S13, 0, &V22420_291_57180, &P22420_289};	/*Property = CssMediaQueries*/
const Value V22420_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22420_292 = {S14, 0, &V22420_292_57183, &P22420_291};	/*Property = CssMinMax*/
const Value V22420_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22420_293 = {S15, 0, &V22420_293_57187, &P22420_292};	/*Property = CssOverflow*/
const Value V22420_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22420_294 = {S16, 0, &V22420_294_57189, &P22420_293};	/*Property = CssPosition*/
const Value V22420_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22420_295 = {S17, 0, &V22420_295_57192, &P22420_294};	/*Property = CssText*/
const Value V22420_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22420_297 = {S18, 0, &V22420_297_57195, &P22420_295};	/*Property = CssTransforms*/
const Value V22420_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22420_241 = {S19, 0, &V22420_241_55799, &P22420_297};	/*Property = CssTransitions*/
const Value V22420_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22420_298 = {S20, 0, &V22420_298_57206, &P22420_241};	/*Property = CssUI*/
const Value V22420_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22420_300 = {S21, 0, &V22420_300_57216, &P22420_298};	/*Property = DataSet*/
const Value V22420_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22420_301 = {S22, 0, &V22420_301_57218, &P22420_300};	/*Property = DataUrl*/
const Value V22420_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22420_302 = {S23, 0, &V22420_302_57222, &P22420_301};	/*Property = DeviceOrientation*/
const Value V22420_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22420_304 = {S24, 0, &V22420_304_57227, &P22420_302};	/*Property = FileReader*/
const Value V22420_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22420_305 = {S25, 0, &V22420_305_57231, &P22420_304};	/*Property = FileSaver*/
const Value V22420_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22420_306 = {S26, 0, &V22420_306_57237, &P22420_305};	/*Property = FileWriter*/
const Value V22420_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22420_307 = {S27, 0, &V22420_307_57239, &P22420_306};	/*Property = FormData*/
const Value V22420_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22420_309 = {S28, 0, &V22420_309_57246, &P22420_307};	/*Property = Fullscreen*/
const Value V22420_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22420_310 = {S29, 0, &V22420_310_57248, &P22420_309};	/*Property = GeoLocation*/
const Value V22420_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22420_312 = {S30, 0, &V22420_312_57254, &P22420_310};	/*Property = History*/
const Value V22420_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22420_314 = {S31, 0, &V22420_314_57261, &P22420_312};	/*Property = Html5*/
const Value V22420_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22420_313 = {S32, 0, &V22420_313_57258, &P22420_314};	/*Property = Html-Media-Capture*/
const Value V22420_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22420_315 = {S33, 0, &V22420_315_57264, &P22420_313};	/*Property = Iframe*/
const Value V22420_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22420_316 = {S34, 0, &V22420_316_57266, &P22420_315};	/*Property = IndexedDB*/
const Value V22420_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22420_317 = {S35, 0, &V22420_317_57269, &P22420_316};	/*Property = Json*/
const Value V22420_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22420_15 = {S37, 0, &V22420_15_28, &P22420_317};	/*Property = LayoutEngine*/
const Value V22420_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22420_344 = {S38, 0, &V22420_344_57272, &P22420_15};	/*Property = Masking*/
const Value V22420_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22420_320 = {S39, 0, &V22420_320_57290, &P22420_344};	/*Property = PostMessage*/
const Value V22420_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22420_321 = {S40, 0, &V22420_321_57293, &P22420_320};	/*Property = Progress*/
const Value V22420_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22420_322 = {S41, 0, &V22420_322_57296, &P22420_321};	/*Property = Prompts*/
const Value V22420_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22420_323 = {S42, 0, &V22420_323_57299, &P22420_322};	/*Property = Selector*/
const Value V22420_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22420_324 = {S43, 0, &V22420_324_57306, &P22420_323};	/*Property = Svg*/
const Value V22420_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22420_327 = {S44, 0, &V22420_327_57315, &P22420_324};	/*Property = TouchEvents*/
const Value V22420_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22420_328 = {S45, 0, &V22420_328_57318, &P22420_327};	/*Property = Track*/
const Value V22420_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22420_329 = {S46, 0, &V22420_329_57323, &P22420_328};	/*Property = Video*/
const Value V22420_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22420_330 = {S47, 0, &V22420_330_57326, &P22420_329};	/*Property = Viewport*/
const Value V22420_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22420_332 = {S48, 0, &V22420_332_57338, &P22420_330};	/*Property = WebWorkers*/
const Value V22420_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22420_333 = {S49, 0, &V22420_333_57342, &P22420_332};	/*Property = Xhr2*/
const Profile P22420 = {"22420", &P22420_333};	/*Profile ID - 22420*/
const Value V18161_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_279 = {S1, 0, &V18161_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18161_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_282 = {S3, 0, &V18161_282_57155, &P18161_279};	/*Property = BlobBuilder*/
const Value V18161_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_350 = {S4, 0, &V18161_350_57446, &P18161_282};	/*Property = Canvas*/
const Value V18161_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_283 = {S5, 0, &V18161_283_57161, &P18161_350};	/*Property = CssBackground*/
const Value V18161_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_284 = {S6, 0, &V18161_284_57164, &P18161_283};	/*Property = CssBorderImage*/
const Value V18161_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_285 = {S7, 0, &V18161_285_57167, &P18161_284};	/*Property = CssCanvas*/
const Value V18161_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_286 = {S8, 0, &V18161_286_57170, &P18161_285};	/*Property = CssColor*/
const Value V18161_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_237 = {S9, 0, &V18161_237_55786, &P18161_286};	/*Property = CssColumn*/
const Value V18161_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_287 = {S10, 0, &V18161_287_57173, &P18161_237};	/*Property = CssFlexbox*/
const Value V18161_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_288 = {S11, 0, &V18161_288_57176, &P18161_287};	/*Property = CssFont*/
const Value V18161_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_289 = {S12, 0, &V18161_289_86899, &P18161_288};	/*Property = CssImages*/
const Value V18161_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_291 = {S13, 0, &V18161_291_57182, &P18161_289};	/*Property = CssMediaQueries*/
const Value V18161_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_292 = {S14, 0, &V18161_292_57185, &P18161_291};	/*Property = CssMinMax*/
const Value V18161_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_293 = {S15, 0, &V18161_293_57188, &P18161_292};	/*Property = CssOverflow*/
const Value V18161_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_294 = {S16, 0, &V18161_294_57191, &P18161_293};	/*Property = CssPosition*/
const Value V18161_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_295 = {S17, 0, &V18161_295_57194, &P18161_294};	/*Property = CssText*/
const Value V18161_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_297 = {S18, 0, &V18161_297_57197, &P18161_295};	/*Property = CssTransforms*/
const Value V18161_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_241 = {S19, 0, &V18161_241_55801, &P18161_297};	/*Property = CssTransitions*/
const Value V18161_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_298 = {S20, 0, &V18161_298_57208, &P18161_241};	/*Property = CssUI*/
const Value V18161_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_300 = {S21, 0, &V18161_300_57217, &P18161_298};	/*Property = DataSet*/
const Value V18161_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_301 = {S22, 0, &V18161_301_57220, &P18161_300};	/*Property = DataUrl*/
const Value V18161_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_302 = {S23, 0, &V18161_302_57223, &P18161_301};	/*Property = DeviceOrientation*/
const Value V18161_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_304 = {S24, 0, &V18161_304_57229, &P18161_302};	/*Property = FileReader*/
const Value V18161_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_305 = {S25, 0, &V18161_305_57232, &P18161_304};	/*Property = FileSaver*/
const Value V18161_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_306 = {S26, 0, &V18161_306_57238, &P18161_305};	/*Property = FileWriter*/
const Value V18161_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_307 = {S27, 0, &V18161_307_57241, &P18161_306};	/*Property = FormData*/
const Value V18161_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_309 = {S28, 0, &V18161_309_57247, &P18161_307};	/*Property = Fullscreen*/
const Value V18161_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_310 = {S29, 0, &V18161_310_57250, &P18161_309};	/*Property = GeoLocation*/
const Value V18161_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_312 = {S30, 0, &V18161_312_57256, &P18161_310};	/*Property = History*/
const Value V18161_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_314 = {S31, 0, &V18161_314_57262, &P18161_312};	/*Property = Html5*/
const Value V18161_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_313 = {S32, 0, &V18161_313_57259, &P18161_314};	/*Property = Html-Media-Capture*/
const Value V18161_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_315 = {S33, 0, &V18161_315_57265, &P18161_313};	/*Property = Iframe*/
const Value V18161_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_316 = {S34, 0, &V18161_316_57268, &P18161_315};	/*Property = IndexedDB*/
const Value V18161_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_317 = {S35, 0, &V18161_317_57271, &P18161_316};	/*Property = Json*/
const Value V18161_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18161_15 = {S37, 0, &V18161_15_28, &P18161_317};	/*Property = LayoutEngine*/
const Value V18161_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_344 = {S38, 0, &V18161_344_57274, &P18161_15};	/*Property = Masking*/
const Value V18161_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_320 = {S39, 0, &V18161_320_57292, &P18161_344};	/*Property = PostMessage*/
const Value V18161_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_321 = {S40, 0, &V18161_321_57295, &P18161_320};	/*Property = Progress*/
const Value V18161_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_322 = {S41, 0, &V18161_322_57298, &P18161_321};	/*Property = Prompts*/
const Value V18161_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_323 = {S42, 0, &V18161_323_57301, &P18161_322};	/*Property = Selector*/
const Value V18161_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_324 = {S43, 0, &V18161_324_57307, &P18161_323};	/*Property = Svg*/
const Value V18161_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_327 = {S44, 0, &V18161_327_57316, &P18161_324};	/*Property = TouchEvents*/
const Value V18161_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_328 = {S45, 0, &V18161_328_57319, &P18161_327};	/*Property = Track*/
const Value V18161_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_329 = {S46, 0, &V18161_329_57325, &P18161_328};	/*Property = Video*/
const Value V18161_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_330 = {S47, 0, &V18161_330_57328, &P18161_329};	/*Property = Viewport*/
const Value V18161_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_332 = {S48, 0, &V18161_332_57340, &P18161_330};	/*Property = WebWorkers*/
const Value V18161_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18161_333 = {S49, 0, &V18161_333_57343, &P18161_332};	/*Property = Xhr2*/
const Profile P18161 = {"18161", &P18161_333};	/*Profile ID - 18161*/
const Profile P18149 = {"18149", NULL};	/*Profile ID - 18149*/
const Value V20167_1_1 = {S2, NULL};	/*Value = True*/
const Property P20167_1 = {S50, 0, &V20167_1_1, NULL};	/*Property = IsMobile*/
const Value V20167_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20167_5 = {S52, 0, &V20167_5_1277, &P20167_1};	/*Property = ScreenPixelsHeight*/
const Value V20167_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20167_4 = {S54, 0, &V20167_4_1279, &P20167_5};	/*Property = ScreenPixelsWidth*/
const Profile P20167 = {"20167", &P20167_4};	/*Profile ID - 20167*/
const Value V22393_1_1 = {S2, NULL};	/*Value = True*/
const Property P22393_1 = {S50, 0, &V22393_1_1, NULL};	/*Property = IsMobile*/
const Value V22393_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22393_5 = {S52, 0, &V22393_5_1458, &P22393_1};	/*Property = ScreenPixelsHeight*/
const Value V22393_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22393_4 = {S54, 0, &V22393_4_1460, &P22393_5};	/*Property = ScreenPixelsWidth*/
const Profile P22393 = {"22393", &P22393_4};	/*Profile ID - 22393*/
const Value V18741_1_1 = {S2, NULL};	/*Value = True*/
const Property P18741_1 = {S50, 0, &V18741_1_1, NULL};	/*Property = IsMobile*/
const Value V18741_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18741_5 = {S52, 0, &V18741_5_1277, &P18741_1};	/*Property = ScreenPixelsHeight*/
const Value V18741_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18741_4 = {S54, 0, &V18741_4_1279, &P18741_5};	/*Property = ScreenPixelsWidth*/
const Profile P18741 = {"18741", &P18741_4};	/*Profile ID - 18741*/
const Value V16049_1_1 = {S2, NULL};	/*Value = True*/
const Property P16049_1 = {S50, 0, &V16049_1_1, NULL};	/*Property = IsMobile*/
const Value V16049_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16049_5 = {S52, 0, &V16049_5_1277, &P16049_1};	/*Property = ScreenPixelsHeight*/
const Value V16049_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16049_4 = {S54, 0, &V16049_4_1279, &P16049_5};	/*Property = ScreenPixelsWidth*/
const Profile P16049 = {"16049", &P16049_4};	/*Profile ID - 16049*/
const Value V22041_1_1 = {S2, NULL};	/*Value = True*/
const Property P22041_1 = {S50, 0, &V22041_1_1, NULL};	/*Property = IsMobile*/
const Value V22041_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22041_5 = {S52, 0, &V22041_5_9981, &P22041_1};	/*Property = ScreenPixelsHeight*/
const Value V22041_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22041_4 = {S54, 0, &V22041_4_2336, &P22041_5};	/*Property = ScreenPixelsWidth*/
const Profile P22041 = {"22041", &P22041_4};	/*Profile ID - 22041*/
const Value V18298_1_1 = {S2, NULL};	/*Value = True*/
const Property P18298_1 = {S50, 0, &V18298_1_1, NULL};	/*Property = IsMobile*/
const Value V18298_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18298_5 = {S52, 0, &V18298_5_9981, &P18298_1};	/*Property = ScreenPixelsHeight*/
const Value V18298_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18298_4 = {S54, 0, &V18298_4_39098, &P18298_5};	/*Property = ScreenPixelsWidth*/
const Profile P18298 = {"18298", &P18298_4};	/*Profile ID - 18298*/
const Value V18235_1_1 = {S2, NULL};	/*Value = True*/
const Property P18235_1 = {S50, 0, &V18235_1_1, NULL};	/*Property = IsMobile*/
const Value V18235_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18235_5 = {S52, 0, &V18235_5_1227, &P18235_1};	/*Property = ScreenPixelsHeight*/
const Value V18235_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18235_4 = {S54, 0, &V18235_4_1229, &P18235_5};	/*Property = ScreenPixelsWidth*/
const Profile P18235 = {"18235", &P18235_4};	/*Profile ID - 18235*/
const Value V21746_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_279 = {S1, 0, &V21746_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21746_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_282 = {S3, 0, &V21746_282_57155, &P21746_279};	/*Property = BlobBuilder*/
const Value V21746_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_350 = {S4, 0, &V21746_350_57446, &P21746_282};	/*Property = Canvas*/
const Value V21746_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_283 = {S5, 0, &V21746_283_57161, &P21746_350};	/*Property = CssBackground*/
const Value V21746_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_284 = {S6, 0, &V21746_284_57164, &P21746_283};	/*Property = CssBorderImage*/
const Value V21746_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_285 = {S7, 0, &V21746_285_57167, &P21746_284};	/*Property = CssCanvas*/
const Value V21746_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_286 = {S8, 0, &V21746_286_57170, &P21746_285};	/*Property = CssColor*/
const Value V21746_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_237 = {S9, 0, &V21746_237_55786, &P21746_286};	/*Property = CssColumn*/
const Value V21746_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_287 = {S10, 0, &V21746_287_57173, &P21746_237};	/*Property = CssFlexbox*/
const Value V21746_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_288 = {S11, 0, &V21746_288_57176, &P21746_287};	/*Property = CssFont*/
const Value V21746_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_289 = {S12, 0, &V21746_289_86899, &P21746_288};	/*Property = CssImages*/
const Value V21746_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_291 = {S13, 0, &V21746_291_57182, &P21746_289};	/*Property = CssMediaQueries*/
const Value V21746_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_292 = {S14, 0, &V21746_292_57185, &P21746_291};	/*Property = CssMinMax*/
const Value V21746_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_293 = {S15, 0, &V21746_293_57188, &P21746_292};	/*Property = CssOverflow*/
const Value V21746_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_294 = {S16, 0, &V21746_294_57191, &P21746_293};	/*Property = CssPosition*/
const Value V21746_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_295 = {S17, 0, &V21746_295_57194, &P21746_294};	/*Property = CssText*/
const Value V21746_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_297 = {S18, 0, &V21746_297_57197, &P21746_295};	/*Property = CssTransforms*/
const Value V21746_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_241 = {S19, 0, &V21746_241_55801, &P21746_297};	/*Property = CssTransitions*/
const Value V21746_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_298 = {S20, 0, &V21746_298_57208, &P21746_241};	/*Property = CssUI*/
const Value V21746_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_300 = {S21, 0, &V21746_300_57217, &P21746_298};	/*Property = DataSet*/
const Value V21746_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_301 = {S22, 0, &V21746_301_57220, &P21746_300};	/*Property = DataUrl*/
const Value V21746_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_302 = {S23, 0, &V21746_302_57223, &P21746_301};	/*Property = DeviceOrientation*/
const Value V21746_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_304 = {S24, 0, &V21746_304_57229, &P21746_302};	/*Property = FileReader*/
const Value V21746_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_305 = {S25, 0, &V21746_305_57232, &P21746_304};	/*Property = FileSaver*/
const Value V21746_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_306 = {S26, 0, &V21746_306_57238, &P21746_305};	/*Property = FileWriter*/
const Value V21746_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_307 = {S27, 0, &V21746_307_57241, &P21746_306};	/*Property = FormData*/
const Value V21746_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_309 = {S28, 0, &V21746_309_57247, &P21746_307};	/*Property = Fullscreen*/
const Value V21746_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_310 = {S29, 0, &V21746_310_57250, &P21746_309};	/*Property = GeoLocation*/
const Value V21746_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_312 = {S30, 0, &V21746_312_57256, &P21746_310};	/*Property = History*/
const Value V21746_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_314 = {S31, 0, &V21746_314_57262, &P21746_312};	/*Property = Html5*/
const Value V21746_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_313 = {S32, 0, &V21746_313_57259, &P21746_314};	/*Property = Html-Media-Capture*/
const Value V21746_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_315 = {S33, 0, &V21746_315_57265, &P21746_313};	/*Property = Iframe*/
const Value V21746_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_316 = {S34, 0, &V21746_316_57268, &P21746_315};	/*Property = IndexedDB*/
const Value V21746_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_317 = {S35, 0, &V21746_317_57271, &P21746_316};	/*Property = Json*/
const Value V21746_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21746_15 = {S37, 0, &V21746_15_54167, &P21746_317};	/*Property = LayoutEngine*/
const Value V21746_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_344 = {S38, 0, &V21746_344_57274, &P21746_15};	/*Property = Masking*/
const Value V21746_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_320 = {S39, 0, &V21746_320_57292, &P21746_344};	/*Property = PostMessage*/
const Value V21746_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_321 = {S40, 0, &V21746_321_57295, &P21746_320};	/*Property = Progress*/
const Value V21746_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_322 = {S41, 0, &V21746_322_57298, &P21746_321};	/*Property = Prompts*/
const Value V21746_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_323 = {S42, 0, &V21746_323_57301, &P21746_322};	/*Property = Selector*/
const Value V21746_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_324 = {S43, 0, &V21746_324_57307, &P21746_323};	/*Property = Svg*/
const Value V21746_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_327 = {S44, 0, &V21746_327_57316, &P21746_324};	/*Property = TouchEvents*/
const Value V21746_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_328 = {S45, 0, &V21746_328_57319, &P21746_327};	/*Property = Track*/
const Value V21746_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_329 = {S46, 0, &V21746_329_57325, &P21746_328};	/*Property = Video*/
const Value V21746_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_330 = {S47, 0, &V21746_330_57328, &P21746_329};	/*Property = Viewport*/
const Value V21746_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_332 = {S48, 0, &V21746_332_57340, &P21746_330};	/*Property = WebWorkers*/
const Value V21746_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21746_333 = {S49, 0, &V21746_333_57343, &P21746_332};	/*Property = Xhr2*/
const Profile P21746 = {"21746", &P21746_333};	/*Profile ID - 21746*/
const Value V21139_1_1 = {S2, NULL};	/*Value = True*/
const Property P21139_1 = {S50, 0, &V21139_1_1, NULL};	/*Property = IsMobile*/
const Value V21139_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21139_5 = {S52, 0, &V21139_5_9981, &P21139_1};	/*Property = ScreenPixelsHeight*/
const Value V21139_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21139_4 = {S54, 0, &V21139_4_2336, &P21139_5};	/*Property = ScreenPixelsWidth*/
const Profile P21139 = {"21139", &P21139_4};	/*Profile ID - 21139*/
const Value V19852_1_1 = {S2, NULL};	/*Value = True*/
const Property P19852_1 = {S50, 0, &V19852_1_1, NULL};	/*Property = IsMobile*/
const Value V19852_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19852_5 = {S52, 0, &V19852_5_9981, &P19852_1};	/*Property = ScreenPixelsHeight*/
const Value V19852_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P19852_4 = {S54, 0, &V19852_4_2336, &P19852_5};	/*Property = ScreenPixelsWidth*/
const Profile P19852 = {"19852", &P19852_4};	/*Profile ID - 19852*/
const Value V22052_1_1 = {S2, NULL};	/*Value = True*/
const Property P22052_1 = {S50, 0, &V22052_1_1, NULL};	/*Property = IsMobile*/
const Value V22052_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22052_5 = {S52, 0, &V22052_5_9981, &P22052_1};	/*Property = ScreenPixelsHeight*/
const Value V22052_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22052_4 = {S54, 0, &V22052_4_2336, &P22052_5};	/*Property = ScreenPixelsWidth*/
const Profile P22052 = {"22052", &P22052_4};	/*Profile ID - 22052*/
const Value V18281_1_1 = {S2, NULL};	/*Value = True*/
const Property P18281_1 = {S50, 0, &V18281_1_1, NULL};	/*Property = IsMobile*/
const Value V18281_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18281_5 = {S52, 0, &V18281_5_1227, &P18281_1};	/*Property = ScreenPixelsHeight*/
const Value V18281_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18281_4 = {S54, 0, &V18281_4_1229, &P18281_5};	/*Property = ScreenPixelsWidth*/
const Profile P18281 = {"18281", &P18281_4};	/*Profile ID - 18281*/
const Value V18265_1_1 = {S2, NULL};	/*Value = True*/
const Property P18265_1 = {S50, 0, &V18265_1_1, NULL};	/*Property = IsMobile*/
const Value V18265_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18265_5 = {S52, 0, &V18265_5_1403, &P18265_1};	/*Property = ScreenPixelsHeight*/
const Value V18265_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18265_4 = {S54, 0, &V18265_4_1229, &P18265_5};	/*Property = ScreenPixelsWidth*/
const Profile P18265 = {"18265", &P18265_4};	/*Profile ID - 18265*/
const Value V18498_1_1 = {S2, NULL};	/*Value = True*/
const Property P18498_1 = {S50, 0, &V18498_1_1, NULL};	/*Property = IsMobile*/
const Value V18498_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18498_5 = {S52, 0, &V18498_5_1227, &P18498_1};	/*Property = ScreenPixelsHeight*/
const Value V18498_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18498_4 = {S54, 0, &V18498_4_1229, &P18498_5};	/*Property = ScreenPixelsWidth*/
const Profile P18498 = {"18498", &P18498_4};	/*Profile ID - 18498*/
const Value V18266_1_1 = {S2, NULL};	/*Value = True*/
const Property P18266_1 = {S50, 0, &V18266_1_1, NULL};	/*Property = IsMobile*/
const Value V18266_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18266_5 = {S52, 0, &V18266_5_1227, &P18266_1};	/*Property = ScreenPixelsHeight*/
const Value V18266_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18266_4 = {S54, 0, &V18266_4_1229, &P18266_5};	/*Property = ScreenPixelsWidth*/
const Profile P18266 = {"18266", &P18266_4};	/*Profile ID - 18266*/
const Value V20134_1_1 = {S2, NULL};	/*Value = True*/
const Property P20134_1 = {S50, 0, &V20134_1_1, NULL};	/*Property = IsMobile*/
const Value V20134_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20134_5 = {S52, 0, &V20134_5_37, &P20134_1};	/*Property = ScreenPixelsHeight*/
const Value V20134_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20134_4 = {S54, 0, &V20134_4_36, &P20134_5};	/*Property = ScreenPixelsWidth*/
const Profile P20134 = {"20134", &P20134_4};	/*Profile ID - 20134*/
const Value V21567_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_279 = {S1, 0, &V21567_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21567_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_282 = {S3, 0, &V21567_282_57155, &P21567_279};	/*Property = BlobBuilder*/
const Value V21567_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_350 = {S4, 0, &V21567_350_57446, &P21567_282};	/*Property = Canvas*/
const Value V21567_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_283 = {S5, 0, &V21567_283_57161, &P21567_350};	/*Property = CssBackground*/
const Value V21567_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_284 = {S6, 0, &V21567_284_57164, &P21567_283};	/*Property = CssBorderImage*/
const Value V21567_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_285 = {S7, 0, &V21567_285_57167, &P21567_284};	/*Property = CssCanvas*/
const Value V21567_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_286 = {S8, 0, &V21567_286_57170, &P21567_285};	/*Property = CssColor*/
const Value V21567_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_237 = {S9, 0, &V21567_237_55786, &P21567_286};	/*Property = CssColumn*/
const Value V21567_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_287 = {S10, 0, &V21567_287_57173, &P21567_237};	/*Property = CssFlexbox*/
const Value V21567_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_288 = {S11, 0, &V21567_288_57176, &P21567_287};	/*Property = CssFont*/
const Value V21567_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_289 = {S12, 0, &V21567_289_86899, &P21567_288};	/*Property = CssImages*/
const Value V21567_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_291 = {S13, 0, &V21567_291_57182, &P21567_289};	/*Property = CssMediaQueries*/
const Value V21567_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_292 = {S14, 0, &V21567_292_57185, &P21567_291};	/*Property = CssMinMax*/
const Value V21567_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_293 = {S15, 0, &V21567_293_57188, &P21567_292};	/*Property = CssOverflow*/
const Value V21567_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_294 = {S16, 0, &V21567_294_57191, &P21567_293};	/*Property = CssPosition*/
const Value V21567_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_295 = {S17, 0, &V21567_295_57194, &P21567_294};	/*Property = CssText*/
const Value V21567_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_297 = {S18, 0, &V21567_297_57197, &P21567_295};	/*Property = CssTransforms*/
const Value V21567_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_241 = {S19, 0, &V21567_241_55801, &P21567_297};	/*Property = CssTransitions*/
const Value V21567_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_298 = {S20, 0, &V21567_298_57208, &P21567_241};	/*Property = CssUI*/
const Value V21567_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_300 = {S21, 0, &V21567_300_57217, &P21567_298};	/*Property = DataSet*/
const Value V21567_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_301 = {S22, 0, &V21567_301_57220, &P21567_300};	/*Property = DataUrl*/
const Value V21567_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_302 = {S23, 0, &V21567_302_57223, &P21567_301};	/*Property = DeviceOrientation*/
const Value V21567_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_304 = {S24, 0, &V21567_304_57229, &P21567_302};	/*Property = FileReader*/
const Value V21567_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_305 = {S25, 0, &V21567_305_57232, &P21567_304};	/*Property = FileSaver*/
const Value V21567_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_306 = {S26, 0, &V21567_306_57238, &P21567_305};	/*Property = FileWriter*/
const Value V21567_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_307 = {S27, 0, &V21567_307_57241, &P21567_306};	/*Property = FormData*/
const Value V21567_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_309 = {S28, 0, &V21567_309_57247, &P21567_307};	/*Property = Fullscreen*/
const Value V21567_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_310 = {S29, 0, &V21567_310_57250, &P21567_309};	/*Property = GeoLocation*/
const Value V21567_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_312 = {S30, 0, &V21567_312_57256, &P21567_310};	/*Property = History*/
const Value V21567_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_314 = {S31, 0, &V21567_314_57262, &P21567_312};	/*Property = Html5*/
const Value V21567_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_313 = {S32, 0, &V21567_313_57259, &P21567_314};	/*Property = Html-Media-Capture*/
const Value V21567_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_315 = {S33, 0, &V21567_315_57265, &P21567_313};	/*Property = Iframe*/
const Value V21567_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_316 = {S34, 0, &V21567_316_57268, &P21567_315};	/*Property = IndexedDB*/
const Value V21567_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_317 = {S35, 0, &V21567_317_57271, &P21567_316};	/*Property = Json*/
const Value V21567_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_15 = {S37, 0, &V21567_15_72, &P21567_317};	/*Property = LayoutEngine*/
const Value V21567_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_344 = {S38, 0, &V21567_344_57274, &P21567_15};	/*Property = Masking*/
const Value V21567_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_320 = {S39, 0, &V21567_320_57292, &P21567_344};	/*Property = PostMessage*/
const Value V21567_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_321 = {S40, 0, &V21567_321_57295, &P21567_320};	/*Property = Progress*/
const Value V21567_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_322 = {S41, 0, &V21567_322_57298, &P21567_321};	/*Property = Prompts*/
const Value V21567_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_323 = {S42, 0, &V21567_323_57301, &P21567_322};	/*Property = Selector*/
const Value V21567_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_324 = {S43, 0, &V21567_324_57307, &P21567_323};	/*Property = Svg*/
const Value V21567_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_327 = {S44, 0, &V21567_327_57316, &P21567_324};	/*Property = TouchEvents*/
const Value V21567_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_328 = {S45, 0, &V21567_328_57319, &P21567_327};	/*Property = Track*/
const Value V21567_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_329 = {S46, 0, &V21567_329_57325, &P21567_328};	/*Property = Video*/
const Value V21567_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_330 = {S47, 0, &V21567_330_57328, &P21567_329};	/*Property = Viewport*/
const Value V21567_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_332 = {S48, 0, &V21567_332_57340, &P21567_330};	/*Property = WebWorkers*/
const Value V21567_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21567_333 = {S49, 0, &V21567_333_57343, &P21567_332};	/*Property = Xhr2*/
const Profile P21567 = {"21567", &P21567_333};	/*Profile ID - 21567*/
const Value V21116_1_1 = {S2, NULL};	/*Value = True*/
const Property P21116_1 = {S50, 0, &V21116_1_1, NULL};	/*Property = IsMobile*/
const Value V21116_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21116_5 = {S52, 0, &V21116_5_1403, &P21116_1};	/*Property = ScreenPixelsHeight*/
const Value V21116_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21116_4 = {S54, 0, &V21116_4_1404, &P21116_5};	/*Property = ScreenPixelsWidth*/
const Profile P21116 = {"21116", &P21116_4};	/*Profile ID - 21116*/
const Value V20780_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_279 = {S1, 0, &V20780_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20780_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_282 = {S3, 0, &V20780_282_57155, &P20780_279};	/*Property = BlobBuilder*/
const Value V20780_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_350 = {S4, 0, &V20780_350_57446, &P20780_282};	/*Property = Canvas*/
const Value V20780_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_283 = {S5, 0, &V20780_283_57161, &P20780_350};	/*Property = CssBackground*/
const Value V20780_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_284 = {S6, 0, &V20780_284_57164, &P20780_283};	/*Property = CssBorderImage*/
const Value V20780_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_285 = {S7, 0, &V20780_285_57167, &P20780_284};	/*Property = CssCanvas*/
const Value V20780_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_286 = {S8, 0, &V20780_286_57170, &P20780_285};	/*Property = CssColor*/
const Value V20780_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_237 = {S9, 0, &V20780_237_55786, &P20780_286};	/*Property = CssColumn*/
const Value V20780_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_287 = {S10, 0, &V20780_287_57173, &P20780_237};	/*Property = CssFlexbox*/
const Value V20780_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_288 = {S11, 0, &V20780_288_57176, &P20780_287};	/*Property = CssFont*/
const Value V20780_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_289 = {S12, 0, &V20780_289_86899, &P20780_288};	/*Property = CssImages*/
const Value V20780_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_291 = {S13, 0, &V20780_291_57182, &P20780_289};	/*Property = CssMediaQueries*/
const Value V20780_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_292 = {S14, 0, &V20780_292_57185, &P20780_291};	/*Property = CssMinMax*/
const Value V20780_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_293 = {S15, 0, &V20780_293_57188, &P20780_292};	/*Property = CssOverflow*/
const Value V20780_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_294 = {S16, 0, &V20780_294_57191, &P20780_293};	/*Property = CssPosition*/
const Value V20780_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_295 = {S17, 0, &V20780_295_57194, &P20780_294};	/*Property = CssText*/
const Value V20780_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_297 = {S18, 0, &V20780_297_57197, &P20780_295};	/*Property = CssTransforms*/
const Value V20780_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_241 = {S19, 0, &V20780_241_55801, &P20780_297};	/*Property = CssTransitions*/
const Value V20780_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_298 = {S20, 0, &V20780_298_57208, &P20780_241};	/*Property = CssUI*/
const Value V20780_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_300 = {S21, 0, &V20780_300_57217, &P20780_298};	/*Property = DataSet*/
const Value V20780_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_301 = {S22, 0, &V20780_301_57220, &P20780_300};	/*Property = DataUrl*/
const Value V20780_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_302 = {S23, 0, &V20780_302_57223, &P20780_301};	/*Property = DeviceOrientation*/
const Value V20780_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_304 = {S24, 0, &V20780_304_57229, &P20780_302};	/*Property = FileReader*/
const Value V20780_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_305 = {S25, 0, &V20780_305_57232, &P20780_304};	/*Property = FileSaver*/
const Value V20780_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_306 = {S26, 0, &V20780_306_57238, &P20780_305};	/*Property = FileWriter*/
const Value V20780_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_307 = {S27, 0, &V20780_307_57241, &P20780_306};	/*Property = FormData*/
const Value V20780_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_309 = {S28, 0, &V20780_309_57247, &P20780_307};	/*Property = Fullscreen*/
const Value V20780_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_310 = {S29, 0, &V20780_310_57250, &P20780_309};	/*Property = GeoLocation*/
const Value V20780_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_312 = {S30, 0, &V20780_312_57256, &P20780_310};	/*Property = History*/
const Value V20780_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_314 = {S31, 0, &V20780_314_57262, &P20780_312};	/*Property = Html5*/
const Value V20780_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_313 = {S32, 0, &V20780_313_57259, &P20780_314};	/*Property = Html-Media-Capture*/
const Value V20780_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_315 = {S33, 0, &V20780_315_57265, &P20780_313};	/*Property = Iframe*/
const Value V20780_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_316 = {S34, 0, &V20780_316_57268, &P20780_315};	/*Property = IndexedDB*/
const Value V20780_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_317 = {S35, 0, &V20780_317_57271, &P20780_316};	/*Property = Json*/
const Value V20780_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P20780_15 = {S37, 0, &V20780_15_54167, &P20780_317};	/*Property = LayoutEngine*/
const Value V20780_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_344 = {S38, 0, &V20780_344_57274, &P20780_15};	/*Property = Masking*/
const Value V20780_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_320 = {S39, 0, &V20780_320_57292, &P20780_344};	/*Property = PostMessage*/
const Value V20780_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_321 = {S40, 0, &V20780_321_57295, &P20780_320};	/*Property = Progress*/
const Value V20780_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_322 = {S41, 0, &V20780_322_57298, &P20780_321};	/*Property = Prompts*/
const Value V20780_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_323 = {S42, 0, &V20780_323_57301, &P20780_322};	/*Property = Selector*/
const Value V20780_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_324 = {S43, 0, &V20780_324_57307, &P20780_323};	/*Property = Svg*/
const Value V20780_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_327 = {S44, 0, &V20780_327_57316, &P20780_324};	/*Property = TouchEvents*/
const Value V20780_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_328 = {S45, 0, &V20780_328_57319, &P20780_327};	/*Property = Track*/
const Value V20780_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_329 = {S46, 0, &V20780_329_57325, &P20780_328};	/*Property = Video*/
const Value V20780_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_330 = {S47, 0, &V20780_330_57328, &P20780_329};	/*Property = Viewport*/
const Value V20780_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_332 = {S48, 0, &V20780_332_57340, &P20780_330};	/*Property = WebWorkers*/
const Value V20780_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20780_333 = {S49, 0, &V20780_333_57343, &P20780_332};	/*Property = Xhr2*/
const Profile P20780 = {"20780", &P20780_333};	/*Profile ID - 20780*/
const Value V18661_1_1 = {S2, NULL};	/*Value = True*/
const Property P18661_1 = {S50, 0, &V18661_1_1, NULL};	/*Property = IsMobile*/
const Value V18661_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18661_5 = {S52, 0, &V18661_5_1227, &P18661_1};	/*Property = ScreenPixelsHeight*/
const Value V18661_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18661_4 = {S54, 0, &V18661_4_1229, &P18661_5};	/*Property = ScreenPixelsWidth*/
const Profile P18661 = {"18661", &P18661_4};	/*Profile ID - 18661*/
const Value V18686_1_1 = {S2, NULL};	/*Value = True*/
const Property P18686_1 = {S50, 0, &V18686_1_1, NULL};	/*Property = IsMobile*/
const Value V18686_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18686_5 = {S52, 0, &V18686_5_9981, &P18686_1};	/*Property = ScreenPixelsHeight*/
const Value V18686_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18686_4 = {S54, 0, &V18686_4_2336, &P18686_5};	/*Property = ScreenPixelsWidth*/
const Profile P18686 = {"18686", &P18686_4};	/*Profile ID - 18686*/
const Value V15854_1_1 = {S2, NULL};	/*Value = True*/
const Property P15854_1 = {S50, 0, &V15854_1_1, NULL};	/*Property = IsMobile*/
const Value V15854_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P15854_5 = {S52, 0, &V15854_5_9981, &P15854_1};	/*Property = ScreenPixelsHeight*/
const Value V15854_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P15854_4 = {S54, 0, &V15854_4_2336, &P15854_5};	/*Property = ScreenPixelsWidth*/
const Profile P15854 = {"15854", &P15854_4};	/*Profile ID - 15854*/
const Value V20582_1_1 = {S2, NULL};	/*Value = True*/
const Property P20582_1 = {S50, 0, &V20582_1_1, NULL};	/*Property = IsMobile*/
const Value V20582_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20582_5 = {S52, 0, &V20582_5_1458, &P20582_1};	/*Property = ScreenPixelsHeight*/
const Value V20582_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20582_4 = {S54, 0, &V20582_4_1460, &P20582_5};	/*Property = ScreenPixelsWidth*/
const Profile P20582 = {"20582", &P20582_4};	/*Profile ID - 20582*/
const Value V18223_1_1 = {S2, NULL};	/*Value = True*/
const Property P18223_1 = {S50, 0, &V18223_1_1, NULL};	/*Property = IsMobile*/
const Value V18223_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18223_5 = {S52, 0, &V18223_5_1403, &P18223_1};	/*Property = ScreenPixelsHeight*/
const Value V18223_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18223_4 = {S54, 0, &V18223_4_1404, &P18223_5};	/*Property = ScreenPixelsWidth*/
const Profile P18223 = {"18223", &P18223_4};	/*Profile ID - 18223*/
const Value V20724_1_1 = {S2, NULL};	/*Value = True*/
const Property P20724_1 = {S50, 0, &V20724_1_1, NULL};	/*Property = IsMobile*/
const Value V20724_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20724_5 = {S52, 0, &V20724_5_1227, &P20724_1};	/*Property = ScreenPixelsHeight*/
const Value V20724_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20724_4 = {S54, 0, &V20724_4_1460, &P20724_5};	/*Property = ScreenPixelsWidth*/
const Profile P20724 = {"20724", &P20724_4};	/*Profile ID - 20724*/
const Value V21665_1_1 = {S2, NULL};	/*Value = True*/
const Property P21665_1 = {S50, 0, &V21665_1_1, NULL};	/*Property = IsMobile*/
const Value V21665_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21665_5 = {S52, 0, &V21665_5_37, &P21665_1};	/*Property = ScreenPixelsHeight*/
const Value V21665_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21665_4 = {S54, 0, &V21665_4_36, &P21665_5};	/*Property = ScreenPixelsWidth*/
const Profile P21665 = {"21665", &P21665_4};	/*Profile ID - 21665*/
const Value V18603_1_1 = {S2, NULL};	/*Value = True*/
const Property P18603_1 = {S50, 0, &V18603_1_1, NULL};	/*Property = IsMobile*/
const Value V18603_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18603_5 = {S52, 0, &V18603_5_1227, &P18603_1};	/*Property = ScreenPixelsHeight*/
const Value V18603_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18603_4 = {S54, 0, &V18603_4_1229, &P18603_5};	/*Property = ScreenPixelsWidth*/
const Profile P18603 = {"18603", &P18603_4};	/*Profile ID - 18603*/
const Value V18380_1_1 = {S2, NULL};	/*Value = True*/
const Property P18380_1 = {S50, 0, &V18380_1_1, NULL};	/*Property = IsMobile*/
const Value V18380_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18380_5 = {S52, 0, &V18380_5_1227, &P18380_1};	/*Property = ScreenPixelsHeight*/
const Value V18380_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18380_4 = {S54, 0, &V18380_4_1229, &P18380_5};	/*Property = ScreenPixelsWidth*/
const Profile P18380 = {"18380", &P18380_4};	/*Profile ID - 18380*/
const Value V18499_1_1 = {S2, NULL};	/*Value = True*/
const Property P18499_1 = {S50, 0, &V18499_1_1, NULL};	/*Property = IsMobile*/
const Value V18499_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18499_5 = {S52, 0, &V18499_5_9981, &P18499_1};	/*Property = ScreenPixelsHeight*/
const Value V18499_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18499_4 = {S54, 0, &V18499_4_39098, &P18499_5};	/*Property = ScreenPixelsWidth*/
const Profile P18499 = {"18499", &P18499_4};	/*Profile ID - 18499*/
const Value V18151_1_1 = {S2, NULL};	/*Value = True*/
const Property P18151_1 = {S50, 0, &V18151_1_1, NULL};	/*Property = IsMobile*/
const Value V18151_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P18151_5 = {S52, 0, &V18151_5_1304, &P18151_1};	/*Property = ScreenPixelsHeight*/
const Value V18151_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18151_4 = {S54, 0, &V18151_4_2336, &P18151_5};	/*Property = ScreenPixelsWidth*/
const Profile P18151 = {"18151", &P18151_4};	/*Profile ID - 18151*/
const Value V20286_1_1 = {S2, NULL};	/*Value = True*/
const Property P20286_1 = {S50, 0, &V20286_1_1, NULL};	/*Property = IsMobile*/
const Value V20286_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20286_5 = {S52, 0, &V20286_5_9981, &P20286_1};	/*Property = ScreenPixelsHeight*/
const Value V20286_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20286_4 = {S54, 0, &V20286_4_39098, &P20286_5};	/*Property = ScreenPixelsWidth*/
const Profile P20286 = {"20286", &P20286_4};	/*Profile ID - 20286*/
const Value V22907_1_1 = {S2, NULL};	/*Value = True*/
const Property P22907_1 = {S50, 0, &V22907_1_1, NULL};	/*Property = IsMobile*/
const Value V22907_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22907_5 = {S52, 0, &V22907_5_1227, &P22907_1};	/*Property = ScreenPixelsHeight*/
const Value V22907_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22907_4 = {S54, 0, &V22907_4_1229, &P22907_5};	/*Property = ScreenPixelsWidth*/
const Profile P22907 = {"22907", &P22907_4};	/*Profile ID - 22907*/
const Value V23874_1_1 = {S2, NULL};	/*Value = True*/
const Property P23874_1 = {S50, 0, &V23874_1_1, NULL};	/*Property = IsMobile*/
const Value V23874_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23874_5 = {S52, 0, &V23874_5_1227, &P23874_1};	/*Property = ScreenPixelsHeight*/
const Value V23874_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23874_4 = {S54, 0, &V23874_4_1229, &P23874_5};	/*Property = ScreenPixelsWidth*/
const Profile P23874 = {"23874", &P23874_4};	/*Profile ID - 23874*/
const Value V21860_1_1 = {S2, NULL};	/*Value = True*/
const Property P21860_1 = {S50, 0, &V21860_1_1, NULL};	/*Property = IsMobile*/
const Value V21860_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21860_5 = {S52, 0, &V21860_5_1227, &P21860_1};	/*Property = ScreenPixelsHeight*/
const Value V21860_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21860_4 = {S54, 0, &V21860_4_1229, &P21860_5};	/*Property = ScreenPixelsWidth*/
const Profile P21860 = {"21860", &P21860_4};	/*Profile ID - 21860*/
const Value V22119_1_1 = {S2, NULL};	/*Value = True*/
const Property P22119_1 = {S50, 0, &V22119_1_1, NULL};	/*Property = IsMobile*/
const Value V22119_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22119_5 = {S52, 0, &V22119_5_1277, &P22119_1};	/*Property = ScreenPixelsHeight*/
const Value V22119_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22119_4 = {S54, 0, &V22119_4_1279, &P22119_5};	/*Property = ScreenPixelsWidth*/
const Profile P22119 = {"22119", &P22119_4};	/*Profile ID - 22119*/
const Value V15873_1_1 = {S2, NULL};	/*Value = True*/
const Property P15873_1 = {S50, 0, &V15873_1_1, NULL};	/*Property = IsMobile*/
const Value V15873_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P15873_5 = {S52, 0, &V15873_5_1832, &P15873_1};	/*Property = ScreenPixelsHeight*/
const Value V15873_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P15873_4 = {S54, 0, &V15873_4_36, &P15873_5};	/*Property = ScreenPixelsWidth*/
const Profile P15873 = {"15873", &P15873_4};	/*Profile ID - 15873*/
const Value V21239_1_1 = {S2, NULL};	/*Value = True*/
const Property P21239_1 = {S50, 0, &V21239_1_1, NULL};	/*Property = IsMobile*/
const Value V21239_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21239_5 = {S52, 0, &V21239_5_1403, &P21239_1};	/*Property = ScreenPixelsHeight*/
const Value V21239_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P21239_4 = {S54, 0, &V21239_4_8554, &P21239_5};	/*Property = ScreenPixelsWidth*/
const Profile P21239 = {"21239", &P21239_4};	/*Profile ID - 21239*/
const Value V21861_1_1 = {S2, NULL};	/*Value = True*/
const Property P21861_1 = {S50, 0, &V21861_1_1, NULL};	/*Property = IsMobile*/
const Value V21861_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21861_5 = {S52, 0, &V21861_5_1227, &P21861_1};	/*Property = ScreenPixelsHeight*/
const Value V21861_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21861_4 = {S54, 0, &V21861_4_1460, &P21861_5};	/*Property = ScreenPixelsWidth*/
const Profile P21861 = {"21861", &P21861_4};	/*Profile ID - 21861*/
const Value V21863_1_1 = {S2, NULL};	/*Value = True*/
const Property P21863_1 = {S50, 0, &V21863_1_1, NULL};	/*Property = IsMobile*/
const Value V21863_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21863_5 = {S52, 0, &V21863_5_1227, &P21863_1};	/*Property = ScreenPixelsHeight*/
const Value V21863_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21863_4 = {S54, 0, &V21863_4_1229, &P21863_5};	/*Property = ScreenPixelsWidth*/
const Profile P21863 = {"21863", &P21863_4};	/*Profile ID - 21863*/
const Value V17835_1_1 = {S2, NULL};	/*Value = True*/
const Property P17835_1 = {S50, 0, &V17835_1_1, NULL};	/*Property = IsMobile*/
const Value V17835_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P17835_5 = {S52, 0, &V17835_5_1458, &P17835_1};	/*Property = ScreenPixelsHeight*/
const Value V17835_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P17835_4 = {S54, 0, &V17835_4_1460, &P17835_5};	/*Property = ScreenPixelsWidth*/
const Profile P17835 = {"17835", &P17835_4};	/*Profile ID - 17835*/
const Value V18224_1_1 = {S2, NULL};	/*Value = True*/
const Property P18224_1 = {S50, 0, &V18224_1_1, NULL};	/*Property = IsMobile*/
const Value V18224_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18224_5 = {S52, 0, &V18224_5_1227, &P18224_1};	/*Property = ScreenPixelsHeight*/
const Value V18224_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18224_4 = {S54, 0, &V18224_4_1229, &P18224_5};	/*Property = ScreenPixelsWidth*/
const Profile P18224 = {"18224", &P18224_4};	/*Profile ID - 18224*/
const Value V18154_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_279 = {S1, 0, &V18154_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18154_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_282 = {S3, 0, &V18154_282_57155, &P18154_279};	/*Property = BlobBuilder*/
const Value V18154_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_350 = {S4, 0, &V18154_350_57446, &P18154_282};	/*Property = Canvas*/
const Value V18154_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_283 = {S5, 0, &V18154_283_57161, &P18154_350};	/*Property = CssBackground*/
const Value V18154_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_284 = {S6, 0, &V18154_284_57164, &P18154_283};	/*Property = CssBorderImage*/
const Value V18154_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_285 = {S7, 0, &V18154_285_57167, &P18154_284};	/*Property = CssCanvas*/
const Value V18154_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_286 = {S8, 0, &V18154_286_57170, &P18154_285};	/*Property = CssColor*/
const Value V18154_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_237 = {S9, 0, &V18154_237_55786, &P18154_286};	/*Property = CssColumn*/
const Value V18154_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_287 = {S10, 0, &V18154_287_57173, &P18154_237};	/*Property = CssFlexbox*/
const Value V18154_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_288 = {S11, 0, &V18154_288_57176, &P18154_287};	/*Property = CssFont*/
const Value V18154_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_289 = {S12, 0, &V18154_289_86899, &P18154_288};	/*Property = CssImages*/
const Value V18154_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_291 = {S13, 0, &V18154_291_57182, &P18154_289};	/*Property = CssMediaQueries*/
const Value V18154_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_292 = {S14, 0, &V18154_292_57185, &P18154_291};	/*Property = CssMinMax*/
const Value V18154_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_293 = {S15, 0, &V18154_293_57188, &P18154_292};	/*Property = CssOverflow*/
const Value V18154_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_294 = {S16, 0, &V18154_294_57191, &P18154_293};	/*Property = CssPosition*/
const Value V18154_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_295 = {S17, 0, &V18154_295_57194, &P18154_294};	/*Property = CssText*/
const Value V18154_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_297 = {S18, 0, &V18154_297_57197, &P18154_295};	/*Property = CssTransforms*/
const Value V18154_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_241 = {S19, 0, &V18154_241_55801, &P18154_297};	/*Property = CssTransitions*/
const Value V18154_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_298 = {S20, 0, &V18154_298_57208, &P18154_241};	/*Property = CssUI*/
const Value V18154_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_300 = {S21, 0, &V18154_300_57217, &P18154_298};	/*Property = DataSet*/
const Value V18154_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_301 = {S22, 0, &V18154_301_57220, &P18154_300};	/*Property = DataUrl*/
const Value V18154_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_302 = {S23, 0, &V18154_302_57223, &P18154_301};	/*Property = DeviceOrientation*/
const Value V18154_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_304 = {S24, 0, &V18154_304_57229, &P18154_302};	/*Property = FileReader*/
const Value V18154_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_305 = {S25, 0, &V18154_305_57232, &P18154_304};	/*Property = FileSaver*/
const Value V18154_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_306 = {S26, 0, &V18154_306_57238, &P18154_305};	/*Property = FileWriter*/
const Value V18154_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_307 = {S27, 0, &V18154_307_57241, &P18154_306};	/*Property = FormData*/
const Value V18154_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_309 = {S28, 0, &V18154_309_57247, &P18154_307};	/*Property = Fullscreen*/
const Value V18154_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_310 = {S29, 0, &V18154_310_57250, &P18154_309};	/*Property = GeoLocation*/
const Value V18154_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_312 = {S30, 0, &V18154_312_57256, &P18154_310};	/*Property = History*/
const Value V18154_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_314 = {S31, 0, &V18154_314_57262, &P18154_312};	/*Property = Html5*/
const Value V18154_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_313 = {S32, 0, &V18154_313_57259, &P18154_314};	/*Property = Html-Media-Capture*/
const Value V18154_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_315 = {S33, 0, &V18154_315_57265, &P18154_313};	/*Property = Iframe*/
const Value V18154_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_316 = {S34, 0, &V18154_316_57268, &P18154_315};	/*Property = IndexedDB*/
const Value V18154_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_317 = {S35, 0, &V18154_317_57271, &P18154_316};	/*Property = Json*/
const Value V18154_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_15 = {S37, 0, &V18154_15_72, &P18154_317};	/*Property = LayoutEngine*/
const Value V18154_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_344 = {S38, 0, &V18154_344_57274, &P18154_15};	/*Property = Masking*/
const Value V18154_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_320 = {S39, 0, &V18154_320_57292, &P18154_344};	/*Property = PostMessage*/
const Value V18154_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_321 = {S40, 0, &V18154_321_57295, &P18154_320};	/*Property = Progress*/
const Value V18154_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_322 = {S41, 0, &V18154_322_57298, &P18154_321};	/*Property = Prompts*/
const Value V18154_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_323 = {S42, 0, &V18154_323_57301, &P18154_322};	/*Property = Selector*/
const Value V18154_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_324 = {S43, 0, &V18154_324_57307, &P18154_323};	/*Property = Svg*/
const Value V18154_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_327 = {S44, 0, &V18154_327_57316, &P18154_324};	/*Property = TouchEvents*/
const Value V18154_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_328 = {S45, 0, &V18154_328_57319, &P18154_327};	/*Property = Track*/
const Value V18154_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_329 = {S46, 0, &V18154_329_57325, &P18154_328};	/*Property = Video*/
const Value V18154_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_330 = {S47, 0, &V18154_330_57328, &P18154_329};	/*Property = Viewport*/
const Value V18154_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_332 = {S48, 0, &V18154_332_57340, &P18154_330};	/*Property = WebWorkers*/
const Value V18154_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18154_333 = {S49, 0, &V18154_333_57343, &P18154_332};	/*Property = Xhr2*/
const Profile P18154 = {"18154", &P18154_333};	/*Profile ID - 18154*/
const Value V15975_1_1 = {S2, NULL};	/*Value = True*/
const Property P15975_1 = {S50, 0, &V15975_1_1, NULL};	/*Property = IsMobile*/
const Value V15975_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P15975_5 = {S52, 0, &V15975_5_1458, &P15975_1};	/*Property = ScreenPixelsHeight*/
const Value V15975_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P15975_4 = {S54, 0, &V15975_4_1460, &P15975_5};	/*Property = ScreenPixelsWidth*/
const Profile P15975 = {"15975", &P15975_4};	/*Profile ID - 15975*/
const Value V22235_1_1 = {S2, NULL};	/*Value = True*/
const Property P22235_1 = {S50, 0, &V22235_1_1, NULL};	/*Property = IsMobile*/
const Value V22235_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22235_5 = {S52, 0, &V22235_5_1277, &P22235_1};	/*Property = ScreenPixelsHeight*/
const Value V22235_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22235_4 = {S54, 0, &V22235_4_2336, &P22235_5};	/*Property = ScreenPixelsWidth*/
const Profile P22235 = {"22235", &P22235_4};	/*Profile ID - 22235*/
const Value V21340_1_1 = {S2, NULL};	/*Value = True*/
const Property P21340_1 = {S50, 0, &V21340_1_1, NULL};	/*Property = IsMobile*/
const Value V21340_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21340_5 = {S52, 0, &V21340_5_1277, &P21340_1};	/*Property = ScreenPixelsHeight*/
const Value V21340_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21340_4 = {S54, 0, &V21340_4_1279, &P21340_5};	/*Property = ScreenPixelsWidth*/
const Profile P21340 = {"21340", &P21340_4};	/*Profile ID - 21340*/
const Value V22287_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_279 = {S1, 0, &V22287_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22287_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_282 = {S3, 0, &V22287_282_57155, &P22287_279};	/*Property = BlobBuilder*/
const Value V22287_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_350 = {S4, 0, &V22287_350_57446, &P22287_282};	/*Property = Canvas*/
const Value V22287_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_283 = {S5, 0, &V22287_283_57161, &P22287_350};	/*Property = CssBackground*/
const Value V22287_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_284 = {S6, 0, &V22287_284_57164, &P22287_283};	/*Property = CssBorderImage*/
const Value V22287_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_285 = {S7, 0, &V22287_285_57167, &P22287_284};	/*Property = CssCanvas*/
const Value V22287_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_286 = {S8, 0, &V22287_286_57170, &P22287_285};	/*Property = CssColor*/
const Value V22287_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_237 = {S9, 0, &V22287_237_55786, &P22287_286};	/*Property = CssColumn*/
const Value V22287_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_287 = {S10, 0, &V22287_287_57173, &P22287_237};	/*Property = CssFlexbox*/
const Value V22287_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_288 = {S11, 0, &V22287_288_57176, &P22287_287};	/*Property = CssFont*/
const Value V22287_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_289 = {S12, 0, &V22287_289_86899, &P22287_288};	/*Property = CssImages*/
const Value V22287_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_291 = {S13, 0, &V22287_291_57182, &P22287_289};	/*Property = CssMediaQueries*/
const Value V22287_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_292 = {S14, 0, &V22287_292_57185, &P22287_291};	/*Property = CssMinMax*/
const Value V22287_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_293 = {S15, 0, &V22287_293_57188, &P22287_292};	/*Property = CssOverflow*/
const Value V22287_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_294 = {S16, 0, &V22287_294_57191, &P22287_293};	/*Property = CssPosition*/
const Value V22287_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_295 = {S17, 0, &V22287_295_57194, &P22287_294};	/*Property = CssText*/
const Value V22287_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_297 = {S18, 0, &V22287_297_57197, &P22287_295};	/*Property = CssTransforms*/
const Value V22287_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_241 = {S19, 0, &V22287_241_55801, &P22287_297};	/*Property = CssTransitions*/
const Value V22287_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_298 = {S20, 0, &V22287_298_57208, &P22287_241};	/*Property = CssUI*/
const Value V22287_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_300 = {S21, 0, &V22287_300_57217, &P22287_298};	/*Property = DataSet*/
const Value V22287_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_301 = {S22, 0, &V22287_301_57220, &P22287_300};	/*Property = DataUrl*/
const Value V22287_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_302 = {S23, 0, &V22287_302_57223, &P22287_301};	/*Property = DeviceOrientation*/
const Value V22287_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_304 = {S24, 0, &V22287_304_57229, &P22287_302};	/*Property = FileReader*/
const Value V22287_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_305 = {S25, 0, &V22287_305_57232, &P22287_304};	/*Property = FileSaver*/
const Value V22287_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_306 = {S26, 0, &V22287_306_57238, &P22287_305};	/*Property = FileWriter*/
const Value V22287_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_307 = {S27, 0, &V22287_307_57241, &P22287_306};	/*Property = FormData*/
const Value V22287_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_309 = {S28, 0, &V22287_309_57247, &P22287_307};	/*Property = Fullscreen*/
const Value V22287_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_310 = {S29, 0, &V22287_310_57250, &P22287_309};	/*Property = GeoLocation*/
const Value V22287_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_312 = {S30, 0, &V22287_312_57256, &P22287_310};	/*Property = History*/
const Value V22287_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_314 = {S31, 0, &V22287_314_57262, &P22287_312};	/*Property = Html5*/
const Value V22287_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_313 = {S32, 0, &V22287_313_57259, &P22287_314};	/*Property = Html-Media-Capture*/
const Value V22287_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_315 = {S33, 0, &V22287_315_57265, &P22287_313};	/*Property = Iframe*/
const Value V22287_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_316 = {S34, 0, &V22287_316_57268, &P22287_315};	/*Property = IndexedDB*/
const Value V22287_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_317 = {S35, 0, &V22287_317_57271, &P22287_316};	/*Property = Json*/
const Value V22287_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_15 = {S37, 0, &V22287_15_72, &P22287_317};	/*Property = LayoutEngine*/
const Value V22287_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_344 = {S38, 0, &V22287_344_57274, &P22287_15};	/*Property = Masking*/
const Value V22287_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_320 = {S39, 0, &V22287_320_57292, &P22287_344};	/*Property = PostMessage*/
const Value V22287_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_321 = {S40, 0, &V22287_321_57295, &P22287_320};	/*Property = Progress*/
const Value V22287_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_322 = {S41, 0, &V22287_322_57298, &P22287_321};	/*Property = Prompts*/
const Value V22287_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_323 = {S42, 0, &V22287_323_57301, &P22287_322};	/*Property = Selector*/
const Value V22287_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_324 = {S43, 0, &V22287_324_57307, &P22287_323};	/*Property = Svg*/
const Value V22287_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_327 = {S44, 0, &V22287_327_57316, &P22287_324};	/*Property = TouchEvents*/
const Value V22287_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_328 = {S45, 0, &V22287_328_57319, &P22287_327};	/*Property = Track*/
const Value V22287_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_329 = {S46, 0, &V22287_329_57325, &P22287_328};	/*Property = Video*/
const Value V22287_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_330 = {S47, 0, &V22287_330_57328, &P22287_329};	/*Property = Viewport*/
const Value V22287_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_332 = {S48, 0, &V22287_332_57340, &P22287_330};	/*Property = WebWorkers*/
const Value V22287_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22287_333 = {S49, 0, &V22287_333_57343, &P22287_332};	/*Property = Xhr2*/
const Profile P22287 = {"22287", &P22287_333};	/*Profile ID - 22287*/
const Value V18631_1_1 = {S2, NULL};	/*Value = True*/
const Property P18631_1 = {S50, 0, &V18631_1_1, NULL};	/*Property = IsMobile*/
const Value V18631_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18631_5 = {S52, 0, &V18631_5_1227, &P18631_1};	/*Property = ScreenPixelsHeight*/
const Value V18631_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P18631_4 = {S54, 0, &V18631_4_1460, &P18631_5};	/*Property = ScreenPixelsWidth*/
const Profile P18631 = {"18631", &P18631_4};	/*Profile ID - 18631*/
const Value V17187_1_1 = {S2, NULL};	/*Value = True*/
const Property P17187_1 = {S50, 0, &V17187_1_1, NULL};	/*Property = IsMobile*/
const Value V17187_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P17187_5 = {S52, 0, &V17187_5_1277, &P17187_1};	/*Property = ScreenPixelsHeight*/
const Value V17187_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P17187_4 = {S54, 0, &V17187_4_1279, &P17187_5};	/*Property = ScreenPixelsWidth*/
const Profile P17187 = {"17187", &P17187_4};	/*Profile ID - 17187*/
const Value V18278_1_1 = {S2, NULL};	/*Value = True*/
const Property P18278_1 = {S50, 0, &V18278_1_1, NULL};	/*Property = IsMobile*/
const Value V18278_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18278_5 = {S52, 0, &V18278_5_9981, &P18278_1};	/*Property = ScreenPixelsHeight*/
const Value V18278_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18278_4 = {S54, 0, &V18278_4_39098, &P18278_5};	/*Property = ScreenPixelsWidth*/
const Profile P18278 = {"18278", &P18278_4};	/*Profile ID - 18278*/
const Value V20537_1_1 = {S2, NULL};	/*Value = True*/
const Property P20537_1 = {S50, 0, &V20537_1_1, NULL};	/*Property = IsMobile*/
const Value V20537_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20537_5 = {S52, 0, &V20537_5_1227, &P20537_1};	/*Property = ScreenPixelsHeight*/
const Value V20537_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20537_4 = {S54, 0, &V20537_4_1229, &P20537_5};	/*Property = ScreenPixelsWidth*/
const Profile P20537 = {"20537", &P20537_4};	/*Profile ID - 20537*/
const Value V18334_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_279 = {S1, 0, &V18334_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18334_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_282 = {S3, 0, &V18334_282_57155, &P18334_279};	/*Property = BlobBuilder*/
const Value V18334_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_350 = {S4, 0, &V18334_350_57446, &P18334_282};	/*Property = Canvas*/
const Value V18334_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_283 = {S5, 0, &V18334_283_57161, &P18334_350};	/*Property = CssBackground*/
const Value V18334_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_284 = {S6, 0, &V18334_284_57164, &P18334_283};	/*Property = CssBorderImage*/
const Value V18334_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_285 = {S7, 0, &V18334_285_57167, &P18334_284};	/*Property = CssCanvas*/
const Value V18334_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_286 = {S8, 0, &V18334_286_57170, &P18334_285};	/*Property = CssColor*/
const Value V18334_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_237 = {S9, 0, &V18334_237_55786, &P18334_286};	/*Property = CssColumn*/
const Value V18334_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_287 = {S10, 0, &V18334_287_57173, &P18334_237};	/*Property = CssFlexbox*/
const Value V18334_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_288 = {S11, 0, &V18334_288_57176, &P18334_287};	/*Property = CssFont*/
const Value V18334_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_289 = {S12, 0, &V18334_289_86899, &P18334_288};	/*Property = CssImages*/
const Value V18334_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_291 = {S13, 0, &V18334_291_57182, &P18334_289};	/*Property = CssMediaQueries*/
const Value V18334_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_292 = {S14, 0, &V18334_292_57185, &P18334_291};	/*Property = CssMinMax*/
const Value V18334_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_293 = {S15, 0, &V18334_293_57188, &P18334_292};	/*Property = CssOverflow*/
const Value V18334_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_294 = {S16, 0, &V18334_294_57191, &P18334_293};	/*Property = CssPosition*/
const Value V18334_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_295 = {S17, 0, &V18334_295_57194, &P18334_294};	/*Property = CssText*/
const Value V18334_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_297 = {S18, 0, &V18334_297_57197, &P18334_295};	/*Property = CssTransforms*/
const Value V18334_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_241 = {S19, 0, &V18334_241_55801, &P18334_297};	/*Property = CssTransitions*/
const Value V18334_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_298 = {S20, 0, &V18334_298_57208, &P18334_241};	/*Property = CssUI*/
const Value V18334_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_300 = {S21, 0, &V18334_300_57217, &P18334_298};	/*Property = DataSet*/
const Value V18334_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_301 = {S22, 0, &V18334_301_57220, &P18334_300};	/*Property = DataUrl*/
const Value V18334_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_302 = {S23, 0, &V18334_302_57223, &P18334_301};	/*Property = DeviceOrientation*/
const Value V18334_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_304 = {S24, 0, &V18334_304_57229, &P18334_302};	/*Property = FileReader*/
const Value V18334_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_305 = {S25, 0, &V18334_305_57232, &P18334_304};	/*Property = FileSaver*/
const Value V18334_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_306 = {S26, 0, &V18334_306_57238, &P18334_305};	/*Property = FileWriter*/
const Value V18334_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_307 = {S27, 0, &V18334_307_57241, &P18334_306};	/*Property = FormData*/
const Value V18334_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_309 = {S28, 0, &V18334_309_57247, &P18334_307};	/*Property = Fullscreen*/
const Value V18334_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_310 = {S29, 0, &V18334_310_57250, &P18334_309};	/*Property = GeoLocation*/
const Value V18334_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_312 = {S30, 0, &V18334_312_57256, &P18334_310};	/*Property = History*/
const Value V18334_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_314 = {S31, 0, &V18334_314_57262, &P18334_312};	/*Property = Html5*/
const Value V18334_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_313 = {S32, 0, &V18334_313_57259, &P18334_314};	/*Property = Html-Media-Capture*/
const Value V18334_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_315 = {S33, 0, &V18334_315_57265, &P18334_313};	/*Property = Iframe*/
const Value V18334_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_316 = {S34, 0, &V18334_316_57268, &P18334_315};	/*Property = IndexedDB*/
const Value V18334_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_317 = {S35, 0, &V18334_317_57271, &P18334_316};	/*Property = Json*/
const Value V18334_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_15 = {S37, 0, &V18334_15_72, &P18334_317};	/*Property = LayoutEngine*/
const Value V18334_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_344 = {S38, 0, &V18334_344_57274, &P18334_15};	/*Property = Masking*/
const Value V18334_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_320 = {S39, 0, &V18334_320_57292, &P18334_344};	/*Property = PostMessage*/
const Value V18334_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_321 = {S40, 0, &V18334_321_57295, &P18334_320};	/*Property = Progress*/
const Value V18334_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_322 = {S41, 0, &V18334_322_57298, &P18334_321};	/*Property = Prompts*/
const Value V18334_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_323 = {S42, 0, &V18334_323_57301, &P18334_322};	/*Property = Selector*/
const Value V18334_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_324 = {S43, 0, &V18334_324_57307, &P18334_323};	/*Property = Svg*/
const Value V18334_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_327 = {S44, 0, &V18334_327_57316, &P18334_324};	/*Property = TouchEvents*/
const Value V18334_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_328 = {S45, 0, &V18334_328_57319, &P18334_327};	/*Property = Track*/
const Value V18334_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_329 = {S46, 0, &V18334_329_57325, &P18334_328};	/*Property = Video*/
const Value V18334_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_330 = {S47, 0, &V18334_330_57328, &P18334_329};	/*Property = Viewport*/
const Value V18334_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_332 = {S48, 0, &V18334_332_57340, &P18334_330};	/*Property = WebWorkers*/
const Value V18334_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18334_333 = {S49, 0, &V18334_333_57343, &P18334_332};	/*Property = Xhr2*/
const Profile P18334 = {"18334", &P18334_333};	/*Profile ID - 18334*/
const Value V24502_1_1 = {S2, NULL};	/*Value = True*/
const Property P24502_1 = {S50, 0, &V24502_1_1, NULL};	/*Property = IsMobile*/
const Value V24502_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24502_5 = {S52, 0, &V24502_5_1277, &P24502_1};	/*Property = ScreenPixelsHeight*/
const Value V24502_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24502_4 = {S54, 0, &V24502_4_1279, &P24502_5};	/*Property = ScreenPixelsWidth*/
const Profile P24502 = {"24502", &P24502_4};	/*Profile ID - 24502*/
const Value V23948_1_1 = {S2, NULL};	/*Value = True*/
const Property P23948_1 = {S50, 0, &V23948_1_1, NULL};	/*Property = IsMobile*/
const Value V23948_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23948_5 = {S52, 0, &V23948_5_1227, &P23948_1};	/*Property = ScreenPixelsHeight*/
const Value V23948_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23948_4 = {S54, 0, &V23948_4_1229, &P23948_5};	/*Property = ScreenPixelsWidth*/
const Profile P23948 = {"23948", &P23948_4};	/*Profile ID - 23948*/
const Value V16376_1_1 = {S2, NULL};	/*Value = True*/
const Property P16376_1 = {S50, 0, &V16376_1_1, NULL};	/*Property = IsMobile*/
const Value V16376_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16376_5 = {S52, 0, &V16376_5_1227, &P16376_1};	/*Property = ScreenPixelsHeight*/
const Value V16376_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P16376_4 = {S54, 0, &V16376_4_1460, &P16376_5};	/*Property = ScreenPixelsWidth*/
const Profile P16376 = {"16376", &P16376_4};	/*Profile ID - 16376*/
const Value V18213_1_1 = {S2, NULL};	/*Value = True*/
const Property P18213_1 = {S50, 0, &V18213_1_1, NULL};	/*Property = IsMobile*/
const Value V18213_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18213_5 = {S52, 0, &V18213_5_1222, &P18213_1};	/*Property = ScreenPixelsHeight*/
const Value V18213_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18213_4 = {S54, 0, &V18213_4_1229, &P18213_5};	/*Property = ScreenPixelsWidth*/
const Profile P18213 = {"18213", &P18213_4};	/*Profile ID - 18213*/
const Value V18465_1_1 = {S2, NULL};	/*Value = True*/
const Property P18465_1 = {S50, 0, &V18465_1_1, NULL};	/*Property = IsMobile*/
const Value V18465_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P18465_5 = {S52, 0, &V18465_5_1458, &P18465_1};	/*Property = ScreenPixelsHeight*/
const Value V18465_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P18465_4 = {S54, 0, &V18465_4_1460, &P18465_5};	/*Property = ScreenPixelsWidth*/
const Profile P18465 = {"18465", &P18465_4};	/*Profile ID - 18465*/
const Value V21483_1_1 = {S2, NULL};	/*Value = True*/
const Property P21483_1 = {S50, 0, &V21483_1_1, NULL};	/*Property = IsMobile*/
const Value V21483_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21483_5 = {S52, 0, &V21483_5_1277, &P21483_1};	/*Property = ScreenPixelsHeight*/
const Value V21483_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21483_4 = {S54, 0, &V21483_4_1279, &P21483_5};	/*Property = ScreenPixelsWidth*/
const Profile P21483 = {"21483", &P21483_4};	/*Profile ID - 21483*/
const Value V21979_1_1 = {S2, NULL};	/*Value = True*/
const Property P21979_1 = {S50, 0, &V21979_1_1, NULL};	/*Property = IsMobile*/
const Value V21979_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P21979_5 = {S52, 0, &V21979_5_1832, &P21979_1};	/*Property = ScreenPixelsHeight*/
const Value V21979_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21979_4 = {S54, 0, &V21979_4_5645, &P21979_5};	/*Property = ScreenPixelsWidth*/
const Profile P21979 = {"21979", &P21979_4};	/*Profile ID - 21979*/
const Value V20554_1_1 = {S2, NULL};	/*Value = True*/
const Property P20554_1 = {S50, 0, &V20554_1_1, NULL};	/*Property = IsMobile*/
const Value V20554_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20554_5 = {S52, 0, &V20554_5_1227, &P20554_1};	/*Property = ScreenPixelsHeight*/
const Value V20554_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20554_4 = {S54, 0, &V20554_4_1229, &P20554_5};	/*Property = ScreenPixelsWidth*/
const Profile P20554 = {"20554", &P20554_4};	/*Profile ID - 20554*/
const Value V22263_1_1 = {S2, NULL};	/*Value = True*/
const Property P22263_1 = {S50, 0, &V22263_1_1, NULL};	/*Property = IsMobile*/
const Value V22263_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22263_5 = {S52, 0, &V22263_5_1227, &P22263_1};	/*Property = ScreenPixelsHeight*/
const Value V22263_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22263_4 = {S54, 0, &V22263_4_1229, &P22263_5};	/*Property = ScreenPixelsWidth*/
const Profile P22263 = {"22263", &P22263_4};	/*Profile ID - 22263*/
const Value V13025_1_1 = {S2, NULL};	/*Value = True*/
const Property P13025_1 = {S50, 0, &V13025_1_1, NULL};	/*Property = IsMobile*/
const Value V13025_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P13025_5 = {S52, 0, &V13025_5_1277, &P13025_1};	/*Property = ScreenPixelsHeight*/
const Value V13025_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P13025_4 = {S54, 0, &V13025_4_1279, &P13025_5};	/*Property = ScreenPixelsWidth*/
const Profile P13025 = {"13025", &P13025_4};	/*Profile ID - 13025*/
const Profile P22200 = {"22200", NULL};	/*Profile ID - 22200*/
const Value V23046_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_279 = {S1, 0, &V23046_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23046_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_282 = {S3, 0, &V23046_282_57155, &P23046_279};	/*Property = BlobBuilder*/
const Value V23046_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_350 = {S4, 0, &V23046_350_57446, &P23046_282};	/*Property = Canvas*/
const Value V23046_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_283 = {S5, 0, &V23046_283_57161, &P23046_350};	/*Property = CssBackground*/
const Value V23046_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_284 = {S6, 0, &V23046_284_57164, &P23046_283};	/*Property = CssBorderImage*/
const Value V23046_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_285 = {S7, 0, &V23046_285_57167, &P23046_284};	/*Property = CssCanvas*/
const Value V23046_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_286 = {S8, 0, &V23046_286_57170, &P23046_285};	/*Property = CssColor*/
const Value V23046_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_237 = {S9, 0, &V23046_237_55786, &P23046_286};	/*Property = CssColumn*/
const Value V23046_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_287 = {S10, 0, &V23046_287_57173, &P23046_237};	/*Property = CssFlexbox*/
const Value V23046_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_288 = {S11, 0, &V23046_288_57176, &P23046_287};	/*Property = CssFont*/
const Value V23046_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_289 = {S12, 0, &V23046_289_86899, &P23046_288};	/*Property = CssImages*/
const Value V23046_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_291 = {S13, 0, &V23046_291_57182, &P23046_289};	/*Property = CssMediaQueries*/
const Value V23046_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_292 = {S14, 0, &V23046_292_57185, &P23046_291};	/*Property = CssMinMax*/
const Value V23046_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_293 = {S15, 0, &V23046_293_57188, &P23046_292};	/*Property = CssOverflow*/
const Value V23046_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_294 = {S16, 0, &V23046_294_57191, &P23046_293};	/*Property = CssPosition*/
const Value V23046_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_295 = {S17, 0, &V23046_295_57194, &P23046_294};	/*Property = CssText*/
const Value V23046_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_297 = {S18, 0, &V23046_297_57197, &P23046_295};	/*Property = CssTransforms*/
const Value V23046_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_241 = {S19, 0, &V23046_241_55801, &P23046_297};	/*Property = CssTransitions*/
const Value V23046_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_298 = {S20, 0, &V23046_298_57208, &P23046_241};	/*Property = CssUI*/
const Value V23046_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_300 = {S21, 0, &V23046_300_57217, &P23046_298};	/*Property = DataSet*/
const Value V23046_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_301 = {S22, 0, &V23046_301_57220, &P23046_300};	/*Property = DataUrl*/
const Value V23046_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_302 = {S23, 0, &V23046_302_57223, &P23046_301};	/*Property = DeviceOrientation*/
const Value V23046_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_304 = {S24, 0, &V23046_304_57229, &P23046_302};	/*Property = FileReader*/
const Value V23046_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_305 = {S25, 0, &V23046_305_57232, &P23046_304};	/*Property = FileSaver*/
const Value V23046_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_306 = {S26, 0, &V23046_306_57238, &P23046_305};	/*Property = FileWriter*/
const Value V23046_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_307 = {S27, 0, &V23046_307_57241, &P23046_306};	/*Property = FormData*/
const Value V23046_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_309 = {S28, 0, &V23046_309_57247, &P23046_307};	/*Property = Fullscreen*/
const Value V23046_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_310 = {S29, 0, &V23046_310_57250, &P23046_309};	/*Property = GeoLocation*/
const Value V23046_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_312 = {S30, 0, &V23046_312_57256, &P23046_310};	/*Property = History*/
const Value V23046_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_314 = {S31, 0, &V23046_314_57262, &P23046_312};	/*Property = Html5*/
const Value V23046_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_313 = {S32, 0, &V23046_313_57259, &P23046_314};	/*Property = Html-Media-Capture*/
const Value V23046_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_315 = {S33, 0, &V23046_315_57265, &P23046_313};	/*Property = Iframe*/
const Value V23046_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_316 = {S34, 0, &V23046_316_57268, &P23046_315};	/*Property = IndexedDB*/
const Value V23046_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_317 = {S35, 0, &V23046_317_57271, &P23046_316};	/*Property = Json*/
const Value V23046_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_15 = {S37, 0, &V23046_15_72, &P23046_317};	/*Property = LayoutEngine*/
const Value V23046_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_344 = {S38, 0, &V23046_344_57274, &P23046_15};	/*Property = Masking*/
const Value V23046_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_320 = {S39, 0, &V23046_320_57292, &P23046_344};	/*Property = PostMessage*/
const Value V23046_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_321 = {S40, 0, &V23046_321_57295, &P23046_320};	/*Property = Progress*/
const Value V23046_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_322 = {S41, 0, &V23046_322_57298, &P23046_321};	/*Property = Prompts*/
const Value V23046_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_323 = {S42, 0, &V23046_323_57301, &P23046_322};	/*Property = Selector*/
const Value V23046_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_324 = {S43, 0, &V23046_324_57307, &P23046_323};	/*Property = Svg*/
const Value V23046_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_327 = {S44, 0, &V23046_327_57316, &P23046_324};	/*Property = TouchEvents*/
const Value V23046_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_328 = {S45, 0, &V23046_328_57319, &P23046_327};	/*Property = Track*/
const Value V23046_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_329 = {S46, 0, &V23046_329_57325, &P23046_328};	/*Property = Video*/
const Value V23046_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_330 = {S47, 0, &V23046_330_57328, &P23046_329};	/*Property = Viewport*/
const Value V23046_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_332 = {S48, 0, &V23046_332_57340, &P23046_330};	/*Property = WebWorkers*/
const Value V23046_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23046_333 = {S49, 0, &V23046_333_57343, &P23046_332};	/*Property = Xhr2*/
const Profile P23046 = {"23046", &P23046_333};	/*Profile ID - 23046*/
const Value V21927_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21927_279 = {S1, 0, &V21927_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21927_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21927_282 = {S3, 0, &V21927_282_57154, &P21927_279};	/*Property = BlobBuilder*/
const Value V21927_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21927_350 = {S4, 0, &V21927_350_57444, &P21927_282};	/*Property = Canvas*/
const Value V21927_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21927_283 = {S5, 0, &V21927_283_57159, &P21927_350};	/*Property = CssBackground*/
const Value V21927_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21927_284 = {S6, 0, &V21927_284_57163, &P21927_283};	/*Property = CssBorderImage*/
const Value V21927_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21927_285 = {S7, 0, &V21927_285_57166, &P21927_284};	/*Property = CssCanvas*/
const Value V21927_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21927_286 = {S8, 0, &V21927_286_57168, &P21927_285};	/*Property = CssColor*/
const Value V21927_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21927_237 = {S9, 0, &V21927_237_55784, &P21927_286};	/*Property = CssColumn*/
const Value V21927_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21927_287 = {S10, 0, &V21927_287_57172, &P21927_237};	/*Property = CssFlexbox*/
const Value V21927_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21927_288 = {S11, 0, &V21927_288_57175, &P21927_287};	/*Property = CssFont*/
const Value V21927_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21927_289 = {S12, 0, &V21927_289_57759, &P21927_288};	/*Property = CssImages*/
const Value V21927_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21927_291 = {S13, 0, &V21927_291_57181, &P21927_289};	/*Property = CssMediaQueries*/
const Value V21927_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21927_292 = {S14, 0, &V21927_292_57183, &P21927_291};	/*Property = CssMinMax*/
const Value V21927_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21927_293 = {S15, 0, &V21927_293_57187, &P21927_292};	/*Property = CssOverflow*/
const Value V21927_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21927_294 = {S16, 0, &V21927_294_57189, &P21927_293};	/*Property = CssPosition*/
const Value V21927_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21927_295 = {S17, 0, &V21927_295_57192, &P21927_294};	/*Property = CssText*/
const Value V21927_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21927_297 = {S18, 0, &V21927_297_57195, &P21927_295};	/*Property = CssTransforms*/
const Value V21927_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21927_241 = {S19, 0, &V21927_241_55799, &P21927_297};	/*Property = CssTransitions*/
const Value V21927_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21927_298 = {S20, 0, &V21927_298_57206, &P21927_241};	/*Property = CssUI*/
const Value V21927_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21927_300 = {S21, 0, &V21927_300_57216, &P21927_298};	/*Property = DataSet*/
const Value V21927_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21927_301 = {S22, 0, &V21927_301_57218, &P21927_300};	/*Property = DataUrl*/
const Value V21927_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21927_302 = {S23, 0, &V21927_302_57222, &P21927_301};	/*Property = DeviceOrientation*/
const Value V21927_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21927_304 = {S24, 0, &V21927_304_57228, &P21927_302};	/*Property = FileReader*/
const Value V21927_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21927_305 = {S25, 0, &V21927_305_57231, &P21927_304};	/*Property = FileSaver*/
const Value V21927_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21927_306 = {S26, 0, &V21927_306_57237, &P21927_305};	/*Property = FileWriter*/
const Value V21927_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21927_307 = {S27, 0, &V21927_307_57240, &P21927_306};	/*Property = FormData*/
const Value V21927_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21927_309 = {S28, 0, &V21927_309_57246, &P21927_307};	/*Property = Fullscreen*/
const Value V21927_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21927_310 = {S29, 0, &V21927_310_57248, &P21927_309};	/*Property = GeoLocation*/
const Value V21927_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21927_312 = {S30, 0, &V21927_312_57255, &P21927_310};	/*Property = History*/
const Value V21927_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21927_314 = {S31, 0, &V21927_314_57261, &P21927_312};	/*Property = Html5*/
const Value V21927_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21927_313 = {S32, 0, &V21927_313_57258, &P21927_314};	/*Property = Html-Media-Capture*/
const Value V21927_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21927_315 = {S33, 0, &V21927_315_57264, &P21927_313};	/*Property = Iframe*/
const Value V21927_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21927_316 = {S34, 0, &V21927_316_57267, &P21927_315};	/*Property = IndexedDB*/
const Value V21927_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21927_317 = {S35, 0, &V21927_317_57269, &P21927_316};	/*Property = Json*/
const Value V21927_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21927_15 = {S37, 0, &V21927_15_28, &P21927_317};	/*Property = LayoutEngine*/
const Value V21927_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21927_344 = {S38, 0, &V21927_344_57272, &P21927_15};	/*Property = Masking*/
const Value V21927_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21927_320 = {S39, 0, &V21927_320_57290, &P21927_344};	/*Property = PostMessage*/
const Value V21927_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21927_321 = {S40, 0, &V21927_321_57293, &P21927_320};	/*Property = Progress*/
const Value V21927_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21927_322 = {S41, 0, &V21927_322_57296, &P21927_321};	/*Property = Prompts*/
const Value V21927_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21927_323 = {S42, 0, &V21927_323_57299, &P21927_322};	/*Property = Selector*/
const Value V21927_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21927_324 = {S43, 0, &V21927_324_57306, &P21927_323};	/*Property = Svg*/
const Value V21927_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21927_327 = {S44, 0, &V21927_327_57315, &P21927_324};	/*Property = TouchEvents*/
const Value V21927_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21927_328 = {S45, 0, &V21927_328_57318, &P21927_327};	/*Property = Track*/
const Value V21927_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21927_329 = {S46, 0, &V21927_329_57323, &P21927_328};	/*Property = Video*/
const Value V21927_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21927_330 = {S47, 0, &V21927_330_57326, &P21927_329};	/*Property = Viewport*/
const Value V21927_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21927_332 = {S48, 0, &V21927_332_57339, &P21927_330};	/*Property = WebWorkers*/
const Value V21927_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21927_333 = {S49, 0, &V21927_333_57342, &P21927_332};	/*Property = Xhr2*/
const Profile P21927 = {"21927", &P21927_333};	/*Profile ID - 21927*/
const Profile P23099 = {"23099", NULL};	/*Profile ID - 23099*/
const Profile P18577 = {"18577", NULL};	/*Profile ID - 18577*/
const Value V21790_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_279 = {S1, 0, &V21790_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21790_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_282 = {S3, 0, &V21790_282_57155, &P21790_279};	/*Property = BlobBuilder*/
const Value V21790_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_350 = {S4, 0, &V21790_350_57446, &P21790_282};	/*Property = Canvas*/
const Value V21790_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_283 = {S5, 0, &V21790_283_57161, &P21790_350};	/*Property = CssBackground*/
const Value V21790_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_284 = {S6, 0, &V21790_284_57164, &P21790_283};	/*Property = CssBorderImage*/
const Value V21790_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_285 = {S7, 0, &V21790_285_57167, &P21790_284};	/*Property = CssCanvas*/
const Value V21790_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_286 = {S8, 0, &V21790_286_57170, &P21790_285};	/*Property = CssColor*/
const Value V21790_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_237 = {S9, 0, &V21790_237_55786, &P21790_286};	/*Property = CssColumn*/
const Value V21790_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_287 = {S10, 0, &V21790_287_57173, &P21790_237};	/*Property = CssFlexbox*/
const Value V21790_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_288 = {S11, 0, &V21790_288_57176, &P21790_287};	/*Property = CssFont*/
const Value V21790_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_289 = {S12, 0, &V21790_289_86899, &P21790_288};	/*Property = CssImages*/
const Value V21790_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_291 = {S13, 0, &V21790_291_57182, &P21790_289};	/*Property = CssMediaQueries*/
const Value V21790_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_292 = {S14, 0, &V21790_292_57185, &P21790_291};	/*Property = CssMinMax*/
const Value V21790_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_293 = {S15, 0, &V21790_293_57188, &P21790_292};	/*Property = CssOverflow*/
const Value V21790_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_294 = {S16, 0, &V21790_294_57191, &P21790_293};	/*Property = CssPosition*/
const Value V21790_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_295 = {S17, 0, &V21790_295_57194, &P21790_294};	/*Property = CssText*/
const Value V21790_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_297 = {S18, 0, &V21790_297_57197, &P21790_295};	/*Property = CssTransforms*/
const Value V21790_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_241 = {S19, 0, &V21790_241_55801, &P21790_297};	/*Property = CssTransitions*/
const Value V21790_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_298 = {S20, 0, &V21790_298_57208, &P21790_241};	/*Property = CssUI*/
const Value V21790_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_300 = {S21, 0, &V21790_300_57217, &P21790_298};	/*Property = DataSet*/
const Value V21790_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_301 = {S22, 0, &V21790_301_57220, &P21790_300};	/*Property = DataUrl*/
const Value V21790_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_302 = {S23, 0, &V21790_302_57223, &P21790_301};	/*Property = DeviceOrientation*/
const Value V21790_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_304 = {S24, 0, &V21790_304_57229, &P21790_302};	/*Property = FileReader*/
const Value V21790_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_305 = {S25, 0, &V21790_305_57232, &P21790_304};	/*Property = FileSaver*/
const Value V21790_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_306 = {S26, 0, &V21790_306_57238, &P21790_305};	/*Property = FileWriter*/
const Value V21790_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_307 = {S27, 0, &V21790_307_57241, &P21790_306};	/*Property = FormData*/
const Value V21790_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_309 = {S28, 0, &V21790_309_57247, &P21790_307};	/*Property = Fullscreen*/
const Value V21790_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_310 = {S29, 0, &V21790_310_57250, &P21790_309};	/*Property = GeoLocation*/
const Value V21790_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_312 = {S30, 0, &V21790_312_57256, &P21790_310};	/*Property = History*/
const Value V21790_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_314 = {S31, 0, &V21790_314_57262, &P21790_312};	/*Property = Html5*/
const Value V21790_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_313 = {S32, 0, &V21790_313_57259, &P21790_314};	/*Property = Html-Media-Capture*/
const Value V21790_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_315 = {S33, 0, &V21790_315_57265, &P21790_313};	/*Property = Iframe*/
const Value V21790_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_316 = {S34, 0, &V21790_316_57268, &P21790_315};	/*Property = IndexedDB*/
const Value V21790_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_317 = {S35, 0, &V21790_317_57271, &P21790_316};	/*Property = Json*/
const Value V21790_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_15 = {S37, 0, &V21790_15_72, &P21790_317};	/*Property = LayoutEngine*/
const Value V21790_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_344 = {S38, 0, &V21790_344_57274, &P21790_15};	/*Property = Masking*/
const Value V21790_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_320 = {S39, 0, &V21790_320_57292, &P21790_344};	/*Property = PostMessage*/
const Value V21790_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_321 = {S40, 0, &V21790_321_57295, &P21790_320};	/*Property = Progress*/
const Value V21790_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_322 = {S41, 0, &V21790_322_57298, &P21790_321};	/*Property = Prompts*/
const Value V21790_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_323 = {S42, 0, &V21790_323_57301, &P21790_322};	/*Property = Selector*/
const Value V21790_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_324 = {S43, 0, &V21790_324_57307, &P21790_323};	/*Property = Svg*/
const Value V21790_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_327 = {S44, 0, &V21790_327_57316, &P21790_324};	/*Property = TouchEvents*/
const Value V21790_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_328 = {S45, 0, &V21790_328_57319, &P21790_327};	/*Property = Track*/
const Value V21790_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_329 = {S46, 0, &V21790_329_57325, &P21790_328};	/*Property = Video*/
const Value V21790_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_330 = {S47, 0, &V21790_330_57328, &P21790_329};	/*Property = Viewport*/
const Value V21790_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_332 = {S48, 0, &V21790_332_57340, &P21790_330};	/*Property = WebWorkers*/
const Value V21790_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21790_333 = {S49, 0, &V21790_333_57343, &P21790_332};	/*Property = Xhr2*/
const Profile P21790 = {"21790", &P21790_333};	/*Profile ID - 21790*/
const Value V24746_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24746_15 = {S37, 0, &V24746_15_28, NULL};	/*Property = LayoutEngine*/
const Profile P24746 = {"24746", &P24746_15};	/*Profile ID - 24746*/
const Profile P22475 = {"22475", NULL};	/*Profile ID - 22475*/
const Value V20065_1_1 = {S2, NULL};	/*Value = True*/
const Property P20065_1 = {S50, 0, &V20065_1_1, NULL};	/*Property = IsMobile*/
const Value V20065_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P20065_5 = {S52, 0, &V20065_5_1293, &P20065_1};	/*Property = ScreenPixelsHeight*/
const Value V20065_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P20065_4 = {S54, 0, &V20065_4_1296, &P20065_5};	/*Property = ScreenPixelsWidth*/
const Profile P20065 = {"20065", &P20065_4};	/*Profile ID - 20065*/
const Value V19785_1_1 = {S2, NULL};	/*Value = True*/
const Property P19785_1 = {S50, 0, &V19785_1_1, NULL};	/*Property = IsMobile*/
const Value V19785_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P19785_5 = {S52, 0, &V19785_5_1458, &P19785_1};	/*Property = ScreenPixelsHeight*/
const Value V19785_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P19785_4 = {S54, 0, &V19785_4_37617, &P19785_5};	/*Property = ScreenPixelsWidth*/
const Profile P19785 = {"19785", &P19785_4};	/*Profile ID - 19785*/
const Value V20965_1_1 = {S2, NULL};	/*Value = True*/
const Property P20965_1 = {S50, 0, &V20965_1_1, NULL};	/*Property = IsMobile*/
const Value V20965_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20965_5 = {S52, 0, &V20965_5_1227, &P20965_1};	/*Property = ScreenPixelsHeight*/
const Value V20965_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20965_4 = {S54, 0, &V20965_4_1229, &P20965_5};	/*Property = ScreenPixelsWidth*/
const Profile P20965 = {"20965", &P20965_4};	/*Profile ID - 20965*/
const Value V20368_1_2 = {S0, NULL};	/*Value = False*/
const Property P20368_1 = {S50, 0, &V20368_1_2, NULL};	/*Property = IsMobile*/
const Value V20368_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20368_5 = {S52, 0, &V20368_5_37, &P20368_1};	/*Property = ScreenPixelsHeight*/
const Value V20368_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20368_4 = {S54, 0, &V20368_4_36, &P20368_5};	/*Property = ScreenPixelsWidth*/
const Profile P20368 = {"20368", &P20368_4};	/*Profile ID - 20368*/
const Value V15803_1_1 = {S2, NULL};	/*Value = True*/
const Property P15803_1 = {S50, 0, &V15803_1_1, NULL};	/*Property = IsMobile*/
const Value V15803_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15803_5 = {S52, 0, &V15803_5_1380, &P15803_1};	/*Property = ScreenPixelsHeight*/
const Value V15803_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15803_4 = {S54, 0, &V15803_4_1279, &P15803_5};	/*Property = ScreenPixelsWidth*/
const Profile P15803 = {"15803", &P15803_4};	/*Profile ID - 15803*/
const Value V18208_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_279 = {S1, 0, &V18208_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18208_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_282 = {S3, 0, &V18208_282_57155, &P18208_279};	/*Property = BlobBuilder*/
const Value V18208_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_350 = {S4, 0, &V18208_350_57446, &P18208_282};	/*Property = Canvas*/
const Value V18208_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_283 = {S5, 0, &V18208_283_57161, &P18208_350};	/*Property = CssBackground*/
const Value V18208_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_284 = {S6, 0, &V18208_284_57164, &P18208_283};	/*Property = CssBorderImage*/
const Value V18208_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_285 = {S7, 0, &V18208_285_57167, &P18208_284};	/*Property = CssCanvas*/
const Value V18208_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_286 = {S8, 0, &V18208_286_57170, &P18208_285};	/*Property = CssColor*/
const Value V18208_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_237 = {S9, 0, &V18208_237_55786, &P18208_286};	/*Property = CssColumn*/
const Value V18208_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_287 = {S10, 0, &V18208_287_57173, &P18208_237};	/*Property = CssFlexbox*/
const Value V18208_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_288 = {S11, 0, &V18208_288_57176, &P18208_287};	/*Property = CssFont*/
const Value V18208_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_289 = {S12, 0, &V18208_289_86899, &P18208_288};	/*Property = CssImages*/
const Value V18208_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_291 = {S13, 0, &V18208_291_57182, &P18208_289};	/*Property = CssMediaQueries*/
const Value V18208_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_292 = {S14, 0, &V18208_292_57185, &P18208_291};	/*Property = CssMinMax*/
const Value V18208_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_293 = {S15, 0, &V18208_293_57188, &P18208_292};	/*Property = CssOverflow*/
const Value V18208_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_294 = {S16, 0, &V18208_294_57191, &P18208_293};	/*Property = CssPosition*/
const Value V18208_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_295 = {S17, 0, &V18208_295_57194, &P18208_294};	/*Property = CssText*/
const Value V18208_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_297 = {S18, 0, &V18208_297_57197, &P18208_295};	/*Property = CssTransforms*/
const Value V18208_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_241 = {S19, 0, &V18208_241_55801, &P18208_297};	/*Property = CssTransitions*/
const Value V18208_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_298 = {S20, 0, &V18208_298_57208, &P18208_241};	/*Property = CssUI*/
const Value V18208_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_300 = {S21, 0, &V18208_300_57217, &P18208_298};	/*Property = DataSet*/
const Value V18208_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_301 = {S22, 0, &V18208_301_57220, &P18208_300};	/*Property = DataUrl*/
const Value V18208_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_302 = {S23, 0, &V18208_302_57223, &P18208_301};	/*Property = DeviceOrientation*/
const Value V18208_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_304 = {S24, 0, &V18208_304_57229, &P18208_302};	/*Property = FileReader*/
const Value V18208_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_305 = {S25, 0, &V18208_305_57232, &P18208_304};	/*Property = FileSaver*/
const Value V18208_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_306 = {S26, 0, &V18208_306_57238, &P18208_305};	/*Property = FileWriter*/
const Value V18208_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_307 = {S27, 0, &V18208_307_57241, &P18208_306};	/*Property = FormData*/
const Value V18208_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_309 = {S28, 0, &V18208_309_57247, &P18208_307};	/*Property = Fullscreen*/
const Value V18208_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_310 = {S29, 0, &V18208_310_57250, &P18208_309};	/*Property = GeoLocation*/
const Value V18208_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_312 = {S30, 0, &V18208_312_57256, &P18208_310};	/*Property = History*/
const Value V18208_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_314 = {S31, 0, &V18208_314_57262, &P18208_312};	/*Property = Html5*/
const Value V18208_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_313 = {S32, 0, &V18208_313_57259, &P18208_314};	/*Property = Html-Media-Capture*/
const Value V18208_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_315 = {S33, 0, &V18208_315_57265, &P18208_313};	/*Property = Iframe*/
const Value V18208_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_316 = {S34, 0, &V18208_316_57268, &P18208_315};	/*Property = IndexedDB*/
const Value V18208_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_317 = {S35, 0, &V18208_317_57271, &P18208_316};	/*Property = Json*/
const Value V18208_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18208_15 = {S37, 0, &V18208_15_29, &P18208_317};	/*Property = LayoutEngine*/
const Value V18208_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_344 = {S38, 0, &V18208_344_57274, &P18208_15};	/*Property = Masking*/
const Value V18208_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_320 = {S39, 0, &V18208_320_57292, &P18208_344};	/*Property = PostMessage*/
const Value V18208_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_321 = {S40, 0, &V18208_321_57295, &P18208_320};	/*Property = Progress*/
const Value V18208_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_322 = {S41, 0, &V18208_322_57298, &P18208_321};	/*Property = Prompts*/
const Value V18208_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_323 = {S42, 0, &V18208_323_57301, &P18208_322};	/*Property = Selector*/
const Value V18208_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_324 = {S43, 0, &V18208_324_57307, &P18208_323};	/*Property = Svg*/
const Value V18208_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_327 = {S44, 0, &V18208_327_57316, &P18208_324};	/*Property = TouchEvents*/
const Value V18208_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_328 = {S45, 0, &V18208_328_57319, &P18208_327};	/*Property = Track*/
const Value V18208_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_329 = {S46, 0, &V18208_329_57325, &P18208_328};	/*Property = Video*/
const Value V18208_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_330 = {S47, 0, &V18208_330_57328, &P18208_329};	/*Property = Viewport*/
const Value V18208_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_332 = {S48, 0, &V18208_332_57340, &P18208_330};	/*Property = WebWorkers*/
const Value V18208_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18208_333 = {S49, 0, &V18208_333_57343, &P18208_332};	/*Property = Xhr2*/
const Profile P18208 = {"18208", &P18208_333};	/*Profile ID - 18208*/
const Profile P18116 = {"18116", NULL};	/*Profile ID - 18116*/
const Value V24667_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_279 = {S1, 0, &V24667_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24667_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_282 = {S3, 0, &V24667_282_57155, &P24667_279};	/*Property = BlobBuilder*/
const Value V24667_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_350 = {S4, 0, &V24667_350_57446, &P24667_282};	/*Property = Canvas*/
const Value V24667_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_283 = {S5, 0, &V24667_283_57161, &P24667_350};	/*Property = CssBackground*/
const Value V24667_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_284 = {S6, 0, &V24667_284_57164, &P24667_283};	/*Property = CssBorderImage*/
const Value V24667_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_285 = {S7, 0, &V24667_285_57167, &P24667_284};	/*Property = CssCanvas*/
const Value V24667_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_286 = {S8, 0, &V24667_286_57170, &P24667_285};	/*Property = CssColor*/
const Value V24667_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_237 = {S9, 0, &V24667_237_55786, &P24667_286};	/*Property = CssColumn*/
const Value V24667_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_287 = {S10, 0, &V24667_287_57173, &P24667_237};	/*Property = CssFlexbox*/
const Value V24667_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_288 = {S11, 0, &V24667_288_57176, &P24667_287};	/*Property = CssFont*/
const Value V24667_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_289 = {S12, 0, &V24667_289_86899, &P24667_288};	/*Property = CssImages*/
const Value V24667_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_291 = {S13, 0, &V24667_291_57182, &P24667_289};	/*Property = CssMediaQueries*/
const Value V24667_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_292 = {S14, 0, &V24667_292_57185, &P24667_291};	/*Property = CssMinMax*/
const Value V24667_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_293 = {S15, 0, &V24667_293_57188, &P24667_292};	/*Property = CssOverflow*/
const Value V24667_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_294 = {S16, 0, &V24667_294_57191, &P24667_293};	/*Property = CssPosition*/
const Value V24667_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_295 = {S17, 0, &V24667_295_57194, &P24667_294};	/*Property = CssText*/
const Value V24667_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_297 = {S18, 0, &V24667_297_57197, &P24667_295};	/*Property = CssTransforms*/
const Value V24667_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_241 = {S19, 0, &V24667_241_55801, &P24667_297};	/*Property = CssTransitions*/
const Value V24667_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_298 = {S20, 0, &V24667_298_57208, &P24667_241};	/*Property = CssUI*/
const Value V24667_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_300 = {S21, 0, &V24667_300_57217, &P24667_298};	/*Property = DataSet*/
const Value V24667_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_301 = {S22, 0, &V24667_301_57220, &P24667_300};	/*Property = DataUrl*/
const Value V24667_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_302 = {S23, 0, &V24667_302_57223, &P24667_301};	/*Property = DeviceOrientation*/
const Value V24667_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_304 = {S24, 0, &V24667_304_57229, &P24667_302};	/*Property = FileReader*/
const Value V24667_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_305 = {S25, 0, &V24667_305_57232, &P24667_304};	/*Property = FileSaver*/
const Value V24667_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_306 = {S26, 0, &V24667_306_57238, &P24667_305};	/*Property = FileWriter*/
const Value V24667_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_307 = {S27, 0, &V24667_307_57241, &P24667_306};	/*Property = FormData*/
const Value V24667_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_309 = {S28, 0, &V24667_309_57247, &P24667_307};	/*Property = Fullscreen*/
const Value V24667_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_310 = {S29, 0, &V24667_310_57250, &P24667_309};	/*Property = GeoLocation*/
const Value V24667_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_312 = {S30, 0, &V24667_312_57256, &P24667_310};	/*Property = History*/
const Value V24667_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_314 = {S31, 0, &V24667_314_57262, &P24667_312};	/*Property = Html5*/
const Value V24667_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_313 = {S32, 0, &V24667_313_57259, &P24667_314};	/*Property = Html-Media-Capture*/
const Value V24667_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_315 = {S33, 0, &V24667_315_57265, &P24667_313};	/*Property = Iframe*/
const Value V24667_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_316 = {S34, 0, &V24667_316_57268, &P24667_315};	/*Property = IndexedDB*/
const Value V24667_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_317 = {S35, 0, &V24667_317_57271, &P24667_316};	/*Property = Json*/
const Value V24667_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24667_15 = {S37, 0, &V24667_15_90837, &P24667_317};	/*Property = LayoutEngine*/
const Value V24667_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_344 = {S38, 0, &V24667_344_57274, &P24667_15};	/*Property = Masking*/
const Value V24667_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_320 = {S39, 0, &V24667_320_57292, &P24667_344};	/*Property = PostMessage*/
const Value V24667_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_321 = {S40, 0, &V24667_321_57295, &P24667_320};	/*Property = Progress*/
const Value V24667_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_322 = {S41, 0, &V24667_322_57298, &P24667_321};	/*Property = Prompts*/
const Value V24667_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_323 = {S42, 0, &V24667_323_57301, &P24667_322};	/*Property = Selector*/
const Value V24667_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_324 = {S43, 0, &V24667_324_57307, &P24667_323};	/*Property = Svg*/
const Value V24667_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_327 = {S44, 0, &V24667_327_57316, &P24667_324};	/*Property = TouchEvents*/
const Value V24667_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_328 = {S45, 0, &V24667_328_57319, &P24667_327};	/*Property = Track*/
const Value V24667_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_329 = {S46, 0, &V24667_329_57325, &P24667_328};	/*Property = Video*/
const Value V24667_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_330 = {S47, 0, &V24667_330_57328, &P24667_329};	/*Property = Viewport*/
const Value V24667_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_332 = {S48, 0, &V24667_332_57340, &P24667_330};	/*Property = WebWorkers*/
const Value V24667_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24667_333 = {S49, 0, &V24667_333_57343, &P24667_332};	/*Property = Xhr2*/
const Profile P24667 = {"24667", &P24667_333};	/*Profile ID - 24667*/
const Profile P18179 = {"18179", NULL};	/*Profile ID - 18179*/
const Value V18167_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18167_279 = {S1, 0, &V18167_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18167_282_57153 = {S2, NULL};	/*Value = True*/
const Property P18167_282 = {S3, 0, &V18167_282_57153, &P18167_279};	/*Property = BlobBuilder*/
const Value V18167_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18167_350 = {S4, 0, &V18167_350_57444, &P18167_282};	/*Property = Canvas*/
const Value V18167_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18167_283 = {S5, 0, &V18167_283_57159, &P18167_350};	/*Property = CssBackground*/
const Value V18167_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18167_284 = {S6, 0, &V18167_284_57163, &P18167_283};	/*Property = CssBorderImage*/
const Value V18167_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18167_285 = {S7, 0, &V18167_285_57166, &P18167_284};	/*Property = CssCanvas*/
const Value V18167_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18167_286 = {S8, 0, &V18167_286_57168, &P18167_285};	/*Property = CssColor*/
const Value V18167_237_55784 = {S2, NULL};	/*Value = True*/
const Property P18167_237 = {S9, 0, &V18167_237_55784, &P18167_286};	/*Property = CssColumn*/
const Value V18167_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18167_287 = {S10, 0, &V18167_287_57172, &P18167_237};	/*Property = CssFlexbox*/
const Value V18167_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18167_288 = {S11, 0, &V18167_288_57175, &P18167_287};	/*Property = CssFont*/
const Value V18167_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18167_289 = {S12, 0, &V18167_289_57759, &P18167_288};	/*Property = CssImages*/
const Value V18167_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18167_291 = {S13, 0, &V18167_291_57180, &P18167_289};	/*Property = CssMediaQueries*/
const Value V18167_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18167_292 = {S14, 0, &V18167_292_57183, &P18167_291};	/*Property = CssMinMax*/
const Value V18167_293_57186 = {S2, NULL};	/*Value = True*/
const Property P18167_293 = {S15, 0, &V18167_293_57186, &P18167_292};	/*Property = CssOverflow*/
const Value V18167_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18167_294 = {S16, 0, &V18167_294_57189, &P18167_293};	/*Property = CssPosition*/
const Value V18167_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18167_295 = {S17, 0, &V18167_295_57192, &P18167_294};	/*Property = CssText*/
const Value V18167_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18167_297 = {S18, 0, &V18167_297_57195, &P18167_295};	/*Property = CssTransforms*/
const Value V18167_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18167_241 = {S19, 0, &V18167_241_55799, &P18167_297};	/*Property = CssTransitions*/
const Value V18167_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18167_298 = {S20, 0, &V18167_298_57206, &P18167_241};	/*Property = CssUI*/
const Value V18167_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18167_300 = {S21, 0, &V18167_300_57216, &P18167_298};	/*Property = DataSet*/
const Value V18167_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18167_301 = {S22, 0, &V18167_301_57218, &P18167_300};	/*Property = DataUrl*/
const Value V18167_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18167_302 = {S23, 0, &V18167_302_57222, &P18167_301};	/*Property = DeviceOrientation*/
const Value V18167_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18167_304 = {S24, 0, &V18167_304_57227, &P18167_302};	/*Property = FileReader*/
const Value V18167_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18167_305 = {S25, 0, &V18167_305_57231, &P18167_304};	/*Property = FileSaver*/
const Value V18167_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18167_306 = {S26, 0, &V18167_306_57237, &P18167_305};	/*Property = FileWriter*/
const Value V18167_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18167_307 = {S27, 0, &V18167_307_57239, &P18167_306};	/*Property = FormData*/
const Value V18167_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18167_309 = {S28, 0, &V18167_309_57246, &P18167_307};	/*Property = Fullscreen*/
const Value V18167_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18167_310 = {S29, 0, &V18167_310_57248, &P18167_309};	/*Property = GeoLocation*/
const Value V18167_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18167_312 = {S30, 0, &V18167_312_57254, &P18167_310};	/*Property = History*/
const Value V18167_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18167_314 = {S31, 0, &V18167_314_57261, &P18167_312};	/*Property = Html5*/
const Value V18167_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18167_313 = {S32, 0, &V18167_313_57258, &P18167_314};	/*Property = Html-Media-Capture*/
const Value V18167_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18167_315 = {S33, 0, &V18167_315_57264, &P18167_313};	/*Property = Iframe*/
const Value V18167_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18167_316 = {S34, 0, &V18167_316_57266, &P18167_315};	/*Property = IndexedDB*/
const Value V18167_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18167_317 = {S35, 0, &V18167_317_57269, &P18167_316};	/*Property = Json*/
const Value V18167_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18167_15 = {S37, 0, &V18167_15_28, &P18167_317};	/*Property = LayoutEngine*/
const Value V18167_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18167_344 = {S38, 0, &V18167_344_57272, &P18167_15};	/*Property = Masking*/
const Value V18167_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18167_320 = {S39, 0, &V18167_320_57290, &P18167_344};	/*Property = PostMessage*/
const Value V18167_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18167_321 = {S40, 0, &V18167_321_57293, &P18167_320};	/*Property = Progress*/
const Value V18167_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18167_322 = {S41, 0, &V18167_322_57296, &P18167_321};	/*Property = Prompts*/
const Value V18167_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18167_323 = {S42, 0, &V18167_323_57299, &P18167_322};	/*Property = Selector*/
const Value V18167_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18167_324 = {S43, 0, &V18167_324_57306, &P18167_323};	/*Property = Svg*/
const Value V18167_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18167_327 = {S44, 0, &V18167_327_57315, &P18167_324};	/*Property = TouchEvents*/
const Value V18167_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18167_328 = {S45, 0, &V18167_328_57318, &P18167_327};	/*Property = Track*/
const Value V18167_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18167_329 = {S46, 0, &V18167_329_57323, &P18167_328};	/*Property = Video*/
const Value V18167_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18167_330 = {S47, 0, &V18167_330_57326, &P18167_329};	/*Property = Viewport*/
const Value V18167_332_57338 = {S2, NULL};	/*Value = True*/
const Property P18167_332 = {S48, 0, &V18167_332_57338, &P18167_330};	/*Property = WebWorkers*/
const Value V18167_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18167_333 = {S49, 0, &V18167_333_57342, &P18167_332};	/*Property = Xhr2*/
const Profile P18167 = {"18167", &P18167_333};	/*Profile ID - 18167*/
const Value V18162_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18162_279 = {S1, 0, &V18162_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18162_282_57153 = {S2, NULL};	/*Value = True*/
const Property P18162_282 = {S3, 0, &V18162_282_57153, &P18162_279};	/*Property = BlobBuilder*/
const Value V18162_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18162_350 = {S4, 0, &V18162_350_57444, &P18162_282};	/*Property = Canvas*/
const Value V18162_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18162_283 = {S5, 0, &V18162_283_57159, &P18162_350};	/*Property = CssBackground*/
const Value V18162_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18162_284 = {S6, 0, &V18162_284_57163, &P18162_283};	/*Property = CssBorderImage*/
const Value V18162_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18162_285 = {S7, 0, &V18162_285_57166, &P18162_284};	/*Property = CssCanvas*/
const Value V18162_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18162_286 = {S8, 0, &V18162_286_57168, &P18162_285};	/*Property = CssColor*/
const Value V18162_237_55784 = {S2, NULL};	/*Value = True*/
const Property P18162_237 = {S9, 0, &V18162_237_55784, &P18162_286};	/*Property = CssColumn*/
const Value V18162_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18162_287 = {S10, 0, &V18162_287_57172, &P18162_237};	/*Property = CssFlexbox*/
const Value V18162_288_57174 = {S2, NULL};	/*Value = True*/
const Property P18162_288 = {S11, 0, &V18162_288_57174, &P18162_287};	/*Property = CssFont*/
const Value V18162_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18162_289 = {S12, 0, &V18162_289_57759, &P18162_288};	/*Property = CssImages*/
const Value V18162_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18162_291 = {S13, 0, &V18162_291_57180, &P18162_289};	/*Property = CssMediaQueries*/
const Value V18162_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18162_292 = {S14, 0, &V18162_292_57183, &P18162_291};	/*Property = CssMinMax*/
const Value V18162_293_57186 = {S2, NULL};	/*Value = True*/
const Property P18162_293 = {S15, 0, &V18162_293_57186, &P18162_292};	/*Property = CssOverflow*/
const Value V18162_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18162_294 = {S16, 0, &V18162_294_57189, &P18162_293};	/*Property = CssPosition*/
const Value V18162_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18162_295 = {S17, 0, &V18162_295_57192, &P18162_294};	/*Property = CssText*/
const Value V18162_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18162_297 = {S18, 0, &V18162_297_57195, &P18162_295};	/*Property = CssTransforms*/
const Value V18162_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18162_241 = {S19, 0, &V18162_241_55799, &P18162_297};	/*Property = CssTransitions*/
const Value V18162_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18162_298 = {S20, 0, &V18162_298_57206, &P18162_241};	/*Property = CssUI*/
const Value V18162_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18162_300 = {S21, 0, &V18162_300_57216, &P18162_298};	/*Property = DataSet*/
const Value V18162_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18162_301 = {S22, 0, &V18162_301_57218, &P18162_300};	/*Property = DataUrl*/
const Value V18162_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18162_302 = {S23, 0, &V18162_302_57222, &P18162_301};	/*Property = DeviceOrientation*/
const Value V18162_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18162_304 = {S24, 0, &V18162_304_57227, &P18162_302};	/*Property = FileReader*/
const Value V18162_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18162_305 = {S25, 0, &V18162_305_57231, &P18162_304};	/*Property = FileSaver*/
const Value V18162_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18162_306 = {S26, 0, &V18162_306_57237, &P18162_305};	/*Property = FileWriter*/
const Value V18162_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18162_307 = {S27, 0, &V18162_307_57239, &P18162_306};	/*Property = FormData*/
const Value V18162_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18162_309 = {S28, 0, &V18162_309_57246, &P18162_307};	/*Property = Fullscreen*/
const Value V18162_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18162_310 = {S29, 0, &V18162_310_57248, &P18162_309};	/*Property = GeoLocation*/
const Value V18162_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18162_312 = {S30, 0, &V18162_312_57254, &P18162_310};	/*Property = History*/
const Value V18162_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18162_314 = {S31, 0, &V18162_314_57261, &P18162_312};	/*Property = Html5*/
const Value V18162_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18162_313 = {S32, 0, &V18162_313_57258, &P18162_314};	/*Property = Html-Media-Capture*/
const Value V18162_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18162_315 = {S33, 0, &V18162_315_57264, &P18162_313};	/*Property = Iframe*/
const Value V18162_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18162_316 = {S34, 0, &V18162_316_57266, &P18162_315};	/*Property = IndexedDB*/
const Value V18162_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18162_317 = {S35, 0, &V18162_317_57269, &P18162_316};	/*Property = Json*/
const Value V18162_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18162_15 = {S37, 0, &V18162_15_28, &P18162_317};	/*Property = LayoutEngine*/
const Value V18162_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18162_344 = {S38, 0, &V18162_344_57272, &P18162_15};	/*Property = Masking*/
const Value V18162_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18162_320 = {S39, 0, &V18162_320_57290, &P18162_344};	/*Property = PostMessage*/
const Value V18162_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18162_321 = {S40, 0, &V18162_321_57293, &P18162_320};	/*Property = Progress*/
const Value V18162_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18162_322 = {S41, 0, &V18162_322_57296, &P18162_321};	/*Property = Prompts*/
const Value V18162_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18162_323 = {S42, 0, &V18162_323_57299, &P18162_322};	/*Property = Selector*/
const Value V18162_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18162_324 = {S43, 0, &V18162_324_57306, &P18162_323};	/*Property = Svg*/
const Value V18162_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18162_327 = {S44, 0, &V18162_327_57315, &P18162_324};	/*Property = TouchEvents*/
const Value V18162_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18162_328 = {S45, 0, &V18162_328_57318, &P18162_327};	/*Property = Track*/
const Value V18162_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18162_329 = {S46, 0, &V18162_329_57323, &P18162_328};	/*Property = Video*/
const Value V18162_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18162_330 = {S47, 0, &V18162_330_57326, &P18162_329};	/*Property = Viewport*/
const Value V18162_332_57338 = {S2, NULL};	/*Value = True*/
const Property P18162_332 = {S48, 0, &V18162_332_57338, &P18162_330};	/*Property = WebWorkers*/
const Value V18162_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18162_333 = {S49, 0, &V18162_333_57342, &P18162_332};	/*Property = Xhr2*/
const Profile P18162 = {"18162", &P18162_333};	/*Profile ID - 18162*/
const Value V24473_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_279 = {S1, 0, &V24473_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24473_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_282 = {S3, 0, &V24473_282_57155, &P24473_279};	/*Property = BlobBuilder*/
const Value V24473_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_350 = {S4, 0, &V24473_350_57446, &P24473_282};	/*Property = Canvas*/
const Value V24473_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_283 = {S5, 0, &V24473_283_57161, &P24473_350};	/*Property = CssBackground*/
const Value V24473_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_284 = {S6, 0, &V24473_284_57164, &P24473_283};	/*Property = CssBorderImage*/
const Value V24473_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_285 = {S7, 0, &V24473_285_57167, &P24473_284};	/*Property = CssCanvas*/
const Value V24473_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_286 = {S8, 0, &V24473_286_57170, &P24473_285};	/*Property = CssColor*/
const Value V24473_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_237 = {S9, 0, &V24473_237_55786, &P24473_286};	/*Property = CssColumn*/
const Value V24473_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_287 = {S10, 0, &V24473_287_57173, &P24473_237};	/*Property = CssFlexbox*/
const Value V24473_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_288 = {S11, 0, &V24473_288_57176, &P24473_287};	/*Property = CssFont*/
const Value V24473_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_289 = {S12, 0, &V24473_289_86899, &P24473_288};	/*Property = CssImages*/
const Value V24473_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_291 = {S13, 0, &V24473_291_57182, &P24473_289};	/*Property = CssMediaQueries*/
const Value V24473_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_292 = {S14, 0, &V24473_292_57185, &P24473_291};	/*Property = CssMinMax*/
const Value V24473_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_293 = {S15, 0, &V24473_293_57188, &P24473_292};	/*Property = CssOverflow*/
const Value V24473_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_294 = {S16, 0, &V24473_294_57191, &P24473_293};	/*Property = CssPosition*/
const Value V24473_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_295 = {S17, 0, &V24473_295_57194, &P24473_294};	/*Property = CssText*/
const Value V24473_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_297 = {S18, 0, &V24473_297_57197, &P24473_295};	/*Property = CssTransforms*/
const Value V24473_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_241 = {S19, 0, &V24473_241_55801, &P24473_297};	/*Property = CssTransitions*/
const Value V24473_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_298 = {S20, 0, &V24473_298_57208, &P24473_241};	/*Property = CssUI*/
const Value V24473_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_300 = {S21, 0, &V24473_300_57217, &P24473_298};	/*Property = DataSet*/
const Value V24473_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_301 = {S22, 0, &V24473_301_57220, &P24473_300};	/*Property = DataUrl*/
const Value V24473_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_302 = {S23, 0, &V24473_302_57223, &P24473_301};	/*Property = DeviceOrientation*/
const Value V24473_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_304 = {S24, 0, &V24473_304_57229, &P24473_302};	/*Property = FileReader*/
const Value V24473_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_305 = {S25, 0, &V24473_305_57232, &P24473_304};	/*Property = FileSaver*/
const Value V24473_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_306 = {S26, 0, &V24473_306_57238, &P24473_305};	/*Property = FileWriter*/
const Value V24473_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_307 = {S27, 0, &V24473_307_57241, &P24473_306};	/*Property = FormData*/
const Value V24473_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_309 = {S28, 0, &V24473_309_57247, &P24473_307};	/*Property = Fullscreen*/
const Value V24473_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_310 = {S29, 0, &V24473_310_57250, &P24473_309};	/*Property = GeoLocation*/
const Value V24473_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_312 = {S30, 0, &V24473_312_57256, &P24473_310};	/*Property = History*/
const Value V24473_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_314 = {S31, 0, &V24473_314_57262, &P24473_312};	/*Property = Html5*/
const Value V24473_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_313 = {S32, 0, &V24473_313_57259, &P24473_314};	/*Property = Html-Media-Capture*/
const Value V24473_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_315 = {S33, 0, &V24473_315_57265, &P24473_313};	/*Property = Iframe*/
const Value V24473_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_316 = {S34, 0, &V24473_316_57268, &P24473_315};	/*Property = IndexedDB*/
const Value V24473_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_317 = {S35, 0, &V24473_317_57271, &P24473_316};	/*Property = Json*/
const Value V24473_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24473_15 = {S37, 0, &V24473_15_54167, &P24473_317};	/*Property = LayoutEngine*/
const Value V24473_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_344 = {S38, 0, &V24473_344_57274, &P24473_15};	/*Property = Masking*/
const Value V24473_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_320 = {S39, 0, &V24473_320_57292, &P24473_344};	/*Property = PostMessage*/
const Value V24473_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_321 = {S40, 0, &V24473_321_57295, &P24473_320};	/*Property = Progress*/
const Value V24473_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_322 = {S41, 0, &V24473_322_57298, &P24473_321};	/*Property = Prompts*/
const Value V24473_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_323 = {S42, 0, &V24473_323_57301, &P24473_322};	/*Property = Selector*/
const Value V24473_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_324 = {S43, 0, &V24473_324_57307, &P24473_323};	/*Property = Svg*/
const Value V24473_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_327 = {S44, 0, &V24473_327_57316, &P24473_324};	/*Property = TouchEvents*/
const Value V24473_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_328 = {S45, 0, &V24473_328_57319, &P24473_327};	/*Property = Track*/
const Value V24473_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_329 = {S46, 0, &V24473_329_57325, &P24473_328};	/*Property = Video*/
const Value V24473_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_330 = {S47, 0, &V24473_330_57328, &P24473_329};	/*Property = Viewport*/
const Value V24473_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_332 = {S48, 0, &V24473_332_57340, &P24473_330};	/*Property = WebWorkers*/
const Value V24473_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24473_333 = {S49, 0, &V24473_333_57343, &P24473_332};	/*Property = Xhr2*/
const Profile P24473 = {"24473", &P24473_333};	/*Profile ID - 24473*/
const Value V23048_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_279 = {S1, 0, &V23048_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23048_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_282 = {S3, 0, &V23048_282_57155, &P23048_279};	/*Property = BlobBuilder*/
const Value V23048_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_350 = {S4, 0, &V23048_350_57446, &P23048_282};	/*Property = Canvas*/
const Value V23048_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_283 = {S5, 0, &V23048_283_57161, &P23048_350};	/*Property = CssBackground*/
const Value V23048_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_284 = {S6, 0, &V23048_284_57164, &P23048_283};	/*Property = CssBorderImage*/
const Value V23048_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_285 = {S7, 0, &V23048_285_57167, &P23048_284};	/*Property = CssCanvas*/
const Value V23048_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_286 = {S8, 0, &V23048_286_57170, &P23048_285};	/*Property = CssColor*/
const Value V23048_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_237 = {S9, 0, &V23048_237_55786, &P23048_286};	/*Property = CssColumn*/
const Value V23048_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_287 = {S10, 0, &V23048_287_57173, &P23048_237};	/*Property = CssFlexbox*/
const Value V23048_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_288 = {S11, 0, &V23048_288_57176, &P23048_287};	/*Property = CssFont*/
const Value V23048_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_289 = {S12, 0, &V23048_289_86899, &P23048_288};	/*Property = CssImages*/
const Value V23048_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_291 = {S13, 0, &V23048_291_57182, &P23048_289};	/*Property = CssMediaQueries*/
const Value V23048_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_292 = {S14, 0, &V23048_292_57185, &P23048_291};	/*Property = CssMinMax*/
const Value V23048_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_293 = {S15, 0, &V23048_293_57188, &P23048_292};	/*Property = CssOverflow*/
const Value V23048_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_294 = {S16, 0, &V23048_294_57191, &P23048_293};	/*Property = CssPosition*/
const Value V23048_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_295 = {S17, 0, &V23048_295_57194, &P23048_294};	/*Property = CssText*/
const Value V23048_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_297 = {S18, 0, &V23048_297_57197, &P23048_295};	/*Property = CssTransforms*/
const Value V23048_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_241 = {S19, 0, &V23048_241_55801, &P23048_297};	/*Property = CssTransitions*/
const Value V23048_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_298 = {S20, 0, &V23048_298_57208, &P23048_241};	/*Property = CssUI*/
const Value V23048_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_300 = {S21, 0, &V23048_300_57217, &P23048_298};	/*Property = DataSet*/
const Value V23048_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_301 = {S22, 0, &V23048_301_57220, &P23048_300};	/*Property = DataUrl*/
const Value V23048_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_302 = {S23, 0, &V23048_302_57223, &P23048_301};	/*Property = DeviceOrientation*/
const Value V23048_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_304 = {S24, 0, &V23048_304_57229, &P23048_302};	/*Property = FileReader*/
const Value V23048_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_305 = {S25, 0, &V23048_305_57232, &P23048_304};	/*Property = FileSaver*/
const Value V23048_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_306 = {S26, 0, &V23048_306_57238, &P23048_305};	/*Property = FileWriter*/
const Value V23048_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_307 = {S27, 0, &V23048_307_57241, &P23048_306};	/*Property = FormData*/
const Value V23048_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_309 = {S28, 0, &V23048_309_57247, &P23048_307};	/*Property = Fullscreen*/
const Value V23048_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_310 = {S29, 0, &V23048_310_57250, &P23048_309};	/*Property = GeoLocation*/
const Value V23048_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_312 = {S30, 0, &V23048_312_57256, &P23048_310};	/*Property = History*/
const Value V23048_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_314 = {S31, 0, &V23048_314_57262, &P23048_312};	/*Property = Html5*/
const Value V23048_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_313 = {S32, 0, &V23048_313_57259, &P23048_314};	/*Property = Html-Media-Capture*/
const Value V23048_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_315 = {S33, 0, &V23048_315_57265, &P23048_313};	/*Property = Iframe*/
const Value V23048_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_316 = {S34, 0, &V23048_316_57268, &P23048_315};	/*Property = IndexedDB*/
const Value V23048_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_317 = {S35, 0, &V23048_317_57271, &P23048_316};	/*Property = Json*/
const Value V23048_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_15 = {S37, 0, &V23048_15_72, &P23048_317};	/*Property = LayoutEngine*/
const Value V23048_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_344 = {S38, 0, &V23048_344_57274, &P23048_15};	/*Property = Masking*/
const Value V23048_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_320 = {S39, 0, &V23048_320_57292, &P23048_344};	/*Property = PostMessage*/
const Value V23048_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_321 = {S40, 0, &V23048_321_57295, &P23048_320};	/*Property = Progress*/
const Value V23048_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_322 = {S41, 0, &V23048_322_57298, &P23048_321};	/*Property = Prompts*/
const Value V23048_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_323 = {S42, 0, &V23048_323_57301, &P23048_322};	/*Property = Selector*/
const Value V23048_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_324 = {S43, 0, &V23048_324_57307, &P23048_323};	/*Property = Svg*/
const Value V23048_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_327 = {S44, 0, &V23048_327_57316, &P23048_324};	/*Property = TouchEvents*/
const Value V23048_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_328 = {S45, 0, &V23048_328_57319, &P23048_327};	/*Property = Track*/
const Value V23048_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_329 = {S46, 0, &V23048_329_57325, &P23048_328};	/*Property = Video*/
const Value V23048_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_330 = {S47, 0, &V23048_330_57328, &P23048_329};	/*Property = Viewport*/
const Value V23048_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_332 = {S48, 0, &V23048_332_57340, &P23048_330};	/*Property = WebWorkers*/
const Value V23048_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23048_333 = {S49, 0, &V23048_333_57343, &P23048_332};	/*Property = Xhr2*/
const Profile P23048 = {"23048", &P23048_333};	/*Profile ID - 23048*/
const Value V20064_1_1 = {S2, NULL};	/*Value = True*/
const Property P20064_1 = {S50, 0, &V20064_1_1, NULL};	/*Property = IsMobile*/
const Value V20064_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P20064_5 = {S52, 0, &V20064_5_1233, &P20064_1};	/*Property = ScreenPixelsHeight*/
const Value V20064_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P20064_4 = {S54, 0, &V20064_4_1235, &P20064_5};	/*Property = ScreenPixelsWidth*/
const Profile P20064 = {"20064", &P20064_4};	/*Profile ID - 20064*/
const Value V22518_1_1 = {S2, NULL};	/*Value = True*/
const Property P22518_1 = {S50, 0, &V22518_1_1, NULL};	/*Property = IsMobile*/
const Value V22518_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22518_5 = {S52, 0, &V22518_5_1277, &P22518_1};	/*Property = ScreenPixelsHeight*/
const Value V22518_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22518_4 = {S54, 0, &V22518_4_2336, &P22518_5};	/*Property = ScreenPixelsWidth*/
const Profile P22518 = {"22518", &P22518_4};	/*Profile ID - 22518*/
const Value V24556_1_1 = {S2, NULL};	/*Value = True*/
const Property P24556_1 = {S50, 0, &V24556_1_1, NULL};	/*Property = IsMobile*/
const Value V24556_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24556_5 = {S52, 0, &V24556_5_1227, &P24556_1};	/*Property = ScreenPixelsHeight*/
const Value V24556_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24556_4 = {S54, 0, &V24556_4_1229, &P24556_5};	/*Property = ScreenPixelsWidth*/
const Profile P24556 = {"24556", &P24556_4};	/*Profile ID - 24556*/
const Value V22726_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22726_279 = {S1, 0, &V22726_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22726_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22726_282 = {S3, 0, &V22726_282_57154, &P22726_279};	/*Property = BlobBuilder*/
const Value V22726_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22726_350 = {S4, 0, &V22726_350_57444, &P22726_282};	/*Property = Canvas*/
const Value V22726_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22726_283 = {S5, 0, &V22726_283_57159, &P22726_350};	/*Property = CssBackground*/
const Value V22726_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22726_284 = {S6, 0, &V22726_284_57163, &P22726_283};	/*Property = CssBorderImage*/
const Value V22726_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22726_285 = {S7, 0, &V22726_285_57166, &P22726_284};	/*Property = CssCanvas*/
const Value V22726_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22726_286 = {S8, 0, &V22726_286_57168, &P22726_285};	/*Property = CssColor*/
const Value V22726_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22726_237 = {S9, 0, &V22726_237_55786, &P22726_286};	/*Property = CssColumn*/
const Value V22726_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22726_287 = {S10, 0, &V22726_287_57172, &P22726_237};	/*Property = CssFlexbox*/
const Value V22726_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22726_288 = {S11, 0, &V22726_288_57175, &P22726_287};	/*Property = CssFont*/
const Value V22726_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22726_289 = {S12, 0, &V22726_289_57759, &P22726_288};	/*Property = CssImages*/
const Value V22726_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22726_291 = {S13, 0, &V22726_291_57181, &P22726_289};	/*Property = CssMediaQueries*/
const Value V22726_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22726_292 = {S14, 0, &V22726_292_57183, &P22726_291};	/*Property = CssMinMax*/
const Value V22726_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22726_293 = {S15, 0, &V22726_293_57187, &P22726_292};	/*Property = CssOverflow*/
const Value V22726_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22726_294 = {S16, 0, &V22726_294_57190, &P22726_293};	/*Property = CssPosition*/
const Value V22726_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22726_295 = {S17, 0, &V22726_295_57192, &P22726_294};	/*Property = CssText*/
const Value V22726_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22726_297 = {S18, 0, &V22726_297_57195, &P22726_295};	/*Property = CssTransforms*/
const Value V22726_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22726_241 = {S19, 0, &V22726_241_55799, &P22726_297};	/*Property = CssTransitions*/
const Value V22726_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22726_298 = {S20, 0, &V22726_298_57206, &P22726_241};	/*Property = CssUI*/
const Value V22726_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22726_300 = {S21, 0, &V22726_300_57216, &P22726_298};	/*Property = DataSet*/
const Value V22726_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22726_301 = {S22, 0, &V22726_301_57218, &P22726_300};	/*Property = DataUrl*/
const Value V22726_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22726_302 = {S23, 0, &V22726_302_57222, &P22726_301};	/*Property = DeviceOrientation*/
const Value V22726_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22726_304 = {S24, 0, &V22726_304_57228, &P22726_302};	/*Property = FileReader*/
const Value V22726_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22726_305 = {S25, 0, &V22726_305_57231, &P22726_304};	/*Property = FileSaver*/
const Value V22726_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22726_306 = {S26, 0, &V22726_306_57237, &P22726_305};	/*Property = FileWriter*/
const Value V22726_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22726_307 = {S27, 0, &V22726_307_57240, &P22726_306};	/*Property = FormData*/
const Value V22726_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22726_309 = {S28, 0, &V22726_309_57246, &P22726_307};	/*Property = Fullscreen*/
const Value V22726_310_57249 = {S0, NULL};	/*Value = False*/
const Property P22726_310 = {S29, 0, &V22726_310_57249, &P22726_309};	/*Property = GeoLocation*/
const Value V22726_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22726_312 = {S30, 0, &V22726_312_57255, &P22726_310};	/*Property = History*/
const Value V22726_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22726_314 = {S31, 0, &V22726_314_57261, &P22726_312};	/*Property = Html5*/
const Value V22726_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22726_313 = {S32, 0, &V22726_313_57258, &P22726_314};	/*Property = Html-Media-Capture*/
const Value V22726_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22726_315 = {S33, 0, &V22726_315_57264, &P22726_313};	/*Property = Iframe*/
const Value V22726_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22726_316 = {S34, 0, &V22726_316_57267, &P22726_315};	/*Property = IndexedDB*/
const Value V22726_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22726_317 = {S35, 0, &V22726_317_57269, &P22726_316};	/*Property = Json*/
const Value V22726_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22726_15 = {S37, 0, &V22726_15_72, &P22726_317};	/*Property = LayoutEngine*/
const Value V22726_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22726_344 = {S38, 0, &V22726_344_57272, &P22726_15};	/*Property = Masking*/
const Value V22726_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22726_320 = {S39, 0, &V22726_320_57290, &P22726_344};	/*Property = PostMessage*/
const Value V22726_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22726_321 = {S40, 0, &V22726_321_57293, &P22726_320};	/*Property = Progress*/
const Value V22726_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22726_322 = {S41, 0, &V22726_322_57296, &P22726_321};	/*Property = Prompts*/
const Value V22726_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22726_323 = {S42, 0, &V22726_323_57299, &P22726_322};	/*Property = Selector*/
const Value V22726_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22726_324 = {S43, 0, &V22726_324_57306, &P22726_323};	/*Property = Svg*/
const Value V22726_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22726_327 = {S44, 0, &V22726_327_57315, &P22726_324};	/*Property = TouchEvents*/
const Value V22726_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22726_328 = {S45, 0, &V22726_328_57318, &P22726_327};	/*Property = Track*/
const Value V22726_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22726_329 = {S46, 0, &V22726_329_57323, &P22726_328};	/*Property = Video*/
const Value V22726_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22726_330 = {S47, 0, &V22726_330_57326, &P22726_329};	/*Property = Viewport*/
const Value V22726_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22726_332 = {S48, 0, &V22726_332_57339, &P22726_330};	/*Property = WebWorkers*/
const Value V22726_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22726_333 = {S49, 0, &V22726_333_57342, &P22726_332};	/*Property = Xhr2*/
const Profile P22726 = {"22726", &P22726_333};	/*Profile ID - 22726*/
const Value V21251_1_1 = {S2, NULL};	/*Value = True*/
const Property P21251_1 = {S50, 0, &V21251_1_1, NULL};	/*Property = IsMobile*/
const Value V21251_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21251_5 = {S52, 0, &V21251_5_1227, &P21251_1};	/*Property = ScreenPixelsHeight*/
const Value V21251_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21251_4 = {S54, 0, &V21251_4_1229, &P21251_5};	/*Property = ScreenPixelsWidth*/
const Profile P21251 = {"21251", &P21251_4};	/*Profile ID - 21251*/
const Value V20372_1_2 = {S0, NULL};	/*Value = False*/
const Property P20372_1 = {S50, 0, &V20372_1_2, NULL};	/*Property = IsMobile*/
const Value V20372_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20372_5 = {S52, 0, &V20372_5_37, &P20372_1};	/*Property = ScreenPixelsHeight*/
const Value V20372_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20372_4 = {S54, 0, &V20372_4_36, &P20372_5};	/*Property = ScreenPixelsWidth*/
const Profile P20372 = {"20372", &P20372_4};	/*Profile ID - 20372*/
const Value V21201_1_1 = {S2, NULL};	/*Value = True*/
const Property P21201_1 = {S50, 0, &V21201_1_1, NULL};	/*Property = IsMobile*/
const Value V21201_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21201_5 = {S52, 0, &V21201_5_1222, &P21201_1};	/*Property = ScreenPixelsHeight*/
const Value V21201_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21201_4 = {S54, 0, &V21201_4_1224, &P21201_5};	/*Property = ScreenPixelsWidth*/
const Profile P21201 = {"21201", &P21201_4};	/*Profile ID - 21201*/
const Value V21425_1_1 = {S2, NULL};	/*Value = True*/
const Property P21425_1 = {S50, 0, &V21425_1_1, NULL};	/*Property = IsMobile*/
const Value V21425_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21425_5 = {S52, 0, &V21425_5_1227, &P21425_1};	/*Property = ScreenPixelsHeight*/
const Value V21425_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21425_4 = {S54, 0, &V21425_4_1460, &P21425_5};	/*Property = ScreenPixelsWidth*/
const Profile P21425 = {"21425", &P21425_4};	/*Profile ID - 21425*/
const Value V20806_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20806_279 = {S1, 0, &V20806_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20806_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20806_282 = {S3, 0, &V20806_282_57154, &P20806_279};	/*Property = BlobBuilder*/
const Value V20806_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20806_350 = {S4, 0, &V20806_350_57444, &P20806_282};	/*Property = Canvas*/
const Value V20806_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20806_283 = {S5, 0, &V20806_283_57159, &P20806_350};	/*Property = CssBackground*/
const Value V20806_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20806_284 = {S6, 0, &V20806_284_57163, &P20806_283};	/*Property = CssBorderImage*/
const Value V20806_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20806_285 = {S7, 0, &V20806_285_57166, &P20806_284};	/*Property = CssCanvas*/
const Value V20806_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20806_286 = {S8, 0, &V20806_286_57168, &P20806_285};	/*Property = CssColor*/
const Value V20806_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20806_237 = {S9, 0, &V20806_237_55786, &P20806_286};	/*Property = CssColumn*/
const Value V20806_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20806_287 = {S10, 0, &V20806_287_57172, &P20806_237};	/*Property = CssFlexbox*/
const Value V20806_288_57175 = {S0, NULL};	/*Value = False*/
const Property P20806_288 = {S11, 0, &V20806_288_57175, &P20806_287};	/*Property = CssFont*/
const Value V20806_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20806_289 = {S12, 0, &V20806_289_57759, &P20806_288};	/*Property = CssImages*/
const Value V20806_291_57181 = {S0, NULL};	/*Value = False*/
const Property P20806_291 = {S13, 0, &V20806_291_57181, &P20806_289};	/*Property = CssMediaQueries*/
const Value V20806_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20806_292 = {S14, 0, &V20806_292_57183, &P20806_291};	/*Property = CssMinMax*/
const Value V20806_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20806_293 = {S15, 0, &V20806_293_57187, &P20806_292};	/*Property = CssOverflow*/
const Value V20806_294_57190 = {S0, NULL};	/*Value = False*/
const Property P20806_294 = {S16, 0, &V20806_294_57190, &P20806_293};	/*Property = CssPosition*/
const Value V20806_295_57192 = {S2, NULL};	/*Value = True*/
const Property P20806_295 = {S17, 0, &V20806_295_57192, &P20806_294};	/*Property = CssText*/
const Value V20806_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20806_297 = {S18, 0, &V20806_297_57195, &P20806_295};	/*Property = CssTransforms*/
const Value V20806_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20806_241 = {S19, 0, &V20806_241_55799, &P20806_297};	/*Property = CssTransitions*/
const Value V20806_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20806_298 = {S20, 0, &V20806_298_57206, &P20806_241};	/*Property = CssUI*/
const Value V20806_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20806_300 = {S21, 0, &V20806_300_57216, &P20806_298};	/*Property = DataSet*/
const Value V20806_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20806_301 = {S22, 0, &V20806_301_57218, &P20806_300};	/*Property = DataUrl*/
const Value V20806_302_57222 = {S0, NULL};	/*Value = False*/
const Property P20806_302 = {S23, 0, &V20806_302_57222, &P20806_301};	/*Property = DeviceOrientation*/
const Value V20806_304_57228 = {S0, NULL};	/*Value = False*/
const Property P20806_304 = {S24, 0, &V20806_304_57228, &P20806_302};	/*Property = FileReader*/
const Value V20806_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20806_305 = {S25, 0, &V20806_305_57231, &P20806_304};	/*Property = FileSaver*/
const Value V20806_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20806_306 = {S26, 0, &V20806_306_57237, &P20806_305};	/*Property = FileWriter*/
const Value V20806_307_57240 = {S0, NULL};	/*Value = False*/
const Property P20806_307 = {S27, 0, &V20806_307_57240, &P20806_306};	/*Property = FormData*/
const Value V20806_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20806_309 = {S28, 0, &V20806_309_57246, &P20806_307};	/*Property = Fullscreen*/
const Value V20806_310_57249 = {S0, NULL};	/*Value = False*/
const Property P20806_310 = {S29, 0, &V20806_310_57249, &P20806_309};	/*Property = GeoLocation*/
const Value V20806_312_57255 = {S0, NULL};	/*Value = False*/
const Property P20806_312 = {S30, 0, &V20806_312_57255, &P20806_310};	/*Property = History*/
const Value V20806_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20806_314 = {S31, 0, &V20806_314_57261, &P20806_312};	/*Property = Html5*/
const Value V20806_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20806_313 = {S32, 0, &V20806_313_57258, &P20806_314};	/*Property = Html-Media-Capture*/
const Value V20806_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20806_315 = {S33, 0, &V20806_315_57264, &P20806_313};	/*Property = Iframe*/
const Value V20806_316_57267 = {S0, NULL};	/*Value = False*/
const Property P20806_316 = {S34, 0, &V20806_316_57267, &P20806_315};	/*Property = IndexedDB*/
const Value V20806_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20806_317 = {S35, 0, &V20806_317_57269, &P20806_316};	/*Property = Json*/
const Value V20806_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P20806_15 = {S37, 0, &V20806_15_28, &P20806_317};	/*Property = LayoutEngine*/
const Value V20806_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20806_344 = {S38, 0, &V20806_344_57272, &P20806_15};	/*Property = Masking*/
const Value V20806_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20806_320 = {S39, 0, &V20806_320_57290, &P20806_344};	/*Property = PostMessage*/
const Value V20806_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20806_321 = {S40, 0, &V20806_321_57293, &P20806_320};	/*Property = Progress*/
const Value V20806_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20806_322 = {S41, 0, &V20806_322_57296, &P20806_321};	/*Property = Prompts*/
const Value V20806_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20806_323 = {S42, 0, &V20806_323_57299, &P20806_322};	/*Property = Selector*/
const Value V20806_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20806_324 = {S43, 0, &V20806_324_57306, &P20806_323};	/*Property = Svg*/
const Value V20806_327_57315 = {S0, NULL};	/*Value = False*/
const Property P20806_327 = {S44, 0, &V20806_327_57315, &P20806_324};	/*Property = TouchEvents*/
const Value V20806_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20806_328 = {S45, 0, &V20806_328_57318, &P20806_327};	/*Property = Track*/
const Value V20806_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20806_329 = {S46, 0, &V20806_329_57323, &P20806_328};	/*Property = Video*/
const Value V20806_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20806_330 = {S47, 0, &V20806_330_57326, &P20806_329};	/*Property = Viewport*/
const Value V20806_332_57339 = {S0, NULL};	/*Value = False*/
const Property P20806_332 = {S48, 0, &V20806_332_57339, &P20806_330};	/*Property = WebWorkers*/
const Value V20806_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20806_333 = {S49, 0, &V20806_333_57342, &P20806_332};	/*Property = Xhr2*/
const Profile P20806 = {"20806", &P20806_333};	/*Profile ID - 20806*/
const Value V20230_1_1 = {S2, NULL};	/*Value = True*/
const Property P20230_1 = {S50, 0, &V20230_1_1, NULL};	/*Property = IsMobile*/
const Value V20230_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20230_5 = {S52, 0, &V20230_5_1227, &P20230_1};	/*Property = ScreenPixelsHeight*/
const Value V20230_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20230_4 = {S54, 0, &V20230_4_1229, &P20230_5};	/*Property = ScreenPixelsWidth*/
const Profile P20230 = {"20230", &P20230_4};	/*Profile ID - 20230*/
const Value V16455_1_1 = {S2, NULL};	/*Value = True*/
const Property P16455_1 = {S50, 0, &V16455_1_1, NULL};	/*Property = IsMobile*/
const Value V16455_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16455_5 = {S52, 0, &V16455_5_1222, &P16455_1};	/*Property = ScreenPixelsHeight*/
const Value V16455_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16455_4 = {S54, 0, &V16455_4_1224, &P16455_5};	/*Property = ScreenPixelsWidth*/
const Profile P16455 = {"16455", &P16455_4};	/*Profile ID - 16455*/
const Profile P18584 = {"18584", NULL};	/*Profile ID - 18584*/
const Profile P17820 = {"17820", NULL};	/*Profile ID - 17820*/
const Profile P5764 = {"5764", NULL};	/*Profile ID - 5764*/
const Profile P18170 = {"18170", NULL};	/*Profile ID - 18170*/
const Value V22580_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22580_279 = {S1, 0, &V22580_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22580_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22580_282 = {S3, 0, &V22580_282_57154, &P22580_279};	/*Property = BlobBuilder*/
const Value V22580_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22580_350 = {S4, 0, &V22580_350_57444, &P22580_282};	/*Property = Canvas*/
const Value V22580_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22580_283 = {S5, 0, &V22580_283_57159, &P22580_350};	/*Property = CssBackground*/
const Value V22580_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22580_284 = {S6, 0, &V22580_284_57163, &P22580_283};	/*Property = CssBorderImage*/
const Value V22580_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22580_285 = {S7, 0, &V22580_285_57166, &P22580_284};	/*Property = CssCanvas*/
const Value V22580_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22580_286 = {S8, 0, &V22580_286_57168, &P22580_285};	/*Property = CssColor*/
const Value V22580_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22580_237 = {S9, 0, &V22580_237_55786, &P22580_286};	/*Property = CssColumn*/
const Value V22580_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22580_287 = {S10, 0, &V22580_287_57172, &P22580_237};	/*Property = CssFlexbox*/
const Value V22580_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22580_288 = {S11, 0, &V22580_288_57175, &P22580_287};	/*Property = CssFont*/
const Value V22580_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22580_289 = {S12, 0, &V22580_289_57759, &P22580_288};	/*Property = CssImages*/
const Value V22580_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22580_291 = {S13, 0, &V22580_291_57181, &P22580_289};	/*Property = CssMediaQueries*/
const Value V22580_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22580_292 = {S14, 0, &V22580_292_57183, &P22580_291};	/*Property = CssMinMax*/
const Value V22580_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22580_293 = {S15, 0, &V22580_293_57187, &P22580_292};	/*Property = CssOverflow*/
const Value V22580_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22580_294 = {S16, 0, &V22580_294_57190, &P22580_293};	/*Property = CssPosition*/
const Value V22580_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22580_295 = {S17, 0, &V22580_295_57192, &P22580_294};	/*Property = CssText*/
const Value V22580_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22580_297 = {S18, 0, &V22580_297_57195, &P22580_295};	/*Property = CssTransforms*/
const Value V22580_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22580_241 = {S19, 0, &V22580_241_55799, &P22580_297};	/*Property = CssTransitions*/
const Value V22580_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22580_298 = {S20, 0, &V22580_298_57206, &P22580_241};	/*Property = CssUI*/
const Value V22580_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22580_300 = {S21, 0, &V22580_300_57216, &P22580_298};	/*Property = DataSet*/
const Value V22580_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22580_301 = {S22, 0, &V22580_301_57218, &P22580_300};	/*Property = DataUrl*/
const Value V22580_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22580_302 = {S23, 0, &V22580_302_57222, &P22580_301};	/*Property = DeviceOrientation*/
const Value V22580_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22580_304 = {S24, 0, &V22580_304_57228, &P22580_302};	/*Property = FileReader*/
const Value V22580_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22580_305 = {S25, 0, &V22580_305_57231, &P22580_304};	/*Property = FileSaver*/
const Value V22580_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22580_306 = {S26, 0, &V22580_306_57237, &P22580_305};	/*Property = FileWriter*/
const Value V22580_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22580_307 = {S27, 0, &V22580_307_57240, &P22580_306};	/*Property = FormData*/
const Value V22580_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22580_309 = {S28, 0, &V22580_309_57246, &P22580_307};	/*Property = Fullscreen*/
const Value V22580_310_57249 = {S0, NULL};	/*Value = False*/
const Property P22580_310 = {S29, 0, &V22580_310_57249, &P22580_309};	/*Property = GeoLocation*/
const Value V22580_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22580_312 = {S30, 0, &V22580_312_57255, &P22580_310};	/*Property = History*/
const Value V22580_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22580_314 = {S31, 0, &V22580_314_57261, &P22580_312};	/*Property = Html5*/
const Value V22580_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22580_313 = {S32, 0, &V22580_313_57258, &P22580_314};	/*Property = Html-Media-Capture*/
const Value V22580_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22580_315 = {S33, 0, &V22580_315_57264, &P22580_313};	/*Property = Iframe*/
const Value V22580_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22580_316 = {S34, 0, &V22580_316_57267, &P22580_315};	/*Property = IndexedDB*/
const Value V22580_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22580_317 = {S35, 0, &V22580_317_57269, &P22580_316};	/*Property = Json*/
const Value V22580_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P22580_15 = {S37, 0, &V22580_15_54167, &P22580_317};	/*Property = LayoutEngine*/
const Value V22580_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22580_344 = {S38, 0, &V22580_344_57272, &P22580_15};	/*Property = Masking*/
const Value V22580_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22580_320 = {S39, 0, &V22580_320_57290, &P22580_344};	/*Property = PostMessage*/
const Value V22580_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22580_321 = {S40, 0, &V22580_321_57293, &P22580_320};	/*Property = Progress*/
const Value V22580_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22580_322 = {S41, 0, &V22580_322_57296, &P22580_321};	/*Property = Prompts*/
const Value V22580_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22580_323 = {S42, 0, &V22580_323_57299, &P22580_322};	/*Property = Selector*/
const Value V22580_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22580_324 = {S43, 0, &V22580_324_57306, &P22580_323};	/*Property = Svg*/
const Value V22580_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22580_327 = {S44, 0, &V22580_327_57315, &P22580_324};	/*Property = TouchEvents*/
const Value V22580_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22580_328 = {S45, 0, &V22580_328_57318, &P22580_327};	/*Property = Track*/
const Value V22580_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22580_329 = {S46, 0, &V22580_329_57323, &P22580_328};	/*Property = Video*/
const Value V22580_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22580_330 = {S47, 0, &V22580_330_57326, &P22580_329};	/*Property = Viewport*/
const Value V22580_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22580_332 = {S48, 0, &V22580_332_57339, &P22580_330};	/*Property = WebWorkers*/
const Value V22580_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22580_333 = {S49, 0, &V22580_333_57342, &P22580_332};	/*Property = Xhr2*/
const Profile P22580 = {"22580", &P22580_333};	/*Profile ID - 22580*/
const Value V18176_1_1 = {S2, NULL};	/*Value = True*/
const Property P18176_1 = {S50, 0, &V18176_1_1, NULL};	/*Property = IsMobile*/
const Value V18176_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18176_5 = {S52, 0, &V18176_5_1227, &P18176_1};	/*Property = ScreenPixelsHeight*/
const Value V18176_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18176_4 = {S54, 0, &V18176_4_1229, &P18176_5};	/*Property = ScreenPixelsWidth*/
const Profile P18176 = {"18176", &P18176_4};	/*Profile ID - 18176*/
const Value V20079_1_2 = {S0, NULL};	/*Value = False*/
const Property P20079_1 = {S50, 0, &V20079_1_2, NULL};	/*Property = IsMobile*/
const Value V20079_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20079_5 = {S52, 0, &V20079_5_37, &P20079_1};	/*Property = ScreenPixelsHeight*/
const Value V20079_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20079_4 = {S54, 0, &V20079_4_36, &P20079_5};	/*Property = ScreenPixelsWidth*/
const Profile P20079 = {"20079", &P20079_4};	/*Profile ID - 20079*/
const Value V18206_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_279 = {S1, 0, &V18206_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18206_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_282 = {S3, 0, &V18206_282_57155, &P18206_279};	/*Property = BlobBuilder*/
const Value V18206_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_350 = {S4, 0, &V18206_350_57446, &P18206_282};	/*Property = Canvas*/
const Value V18206_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_283 = {S5, 0, &V18206_283_57161, &P18206_350};	/*Property = CssBackground*/
const Value V18206_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_284 = {S6, 0, &V18206_284_57164, &P18206_283};	/*Property = CssBorderImage*/
const Value V18206_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_285 = {S7, 0, &V18206_285_57167, &P18206_284};	/*Property = CssCanvas*/
const Value V18206_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_286 = {S8, 0, &V18206_286_57170, &P18206_285};	/*Property = CssColor*/
const Value V18206_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_237 = {S9, 0, &V18206_237_55786, &P18206_286};	/*Property = CssColumn*/
const Value V18206_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_287 = {S10, 0, &V18206_287_57173, &P18206_237};	/*Property = CssFlexbox*/
const Value V18206_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_288 = {S11, 0, &V18206_288_57176, &P18206_287};	/*Property = CssFont*/
const Value V18206_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_289 = {S12, 0, &V18206_289_86899, &P18206_288};	/*Property = CssImages*/
const Value V18206_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_291 = {S13, 0, &V18206_291_57182, &P18206_289};	/*Property = CssMediaQueries*/
const Value V18206_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_292 = {S14, 0, &V18206_292_57185, &P18206_291};	/*Property = CssMinMax*/
const Value V18206_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_293 = {S15, 0, &V18206_293_57188, &P18206_292};	/*Property = CssOverflow*/
const Value V18206_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_294 = {S16, 0, &V18206_294_57191, &P18206_293};	/*Property = CssPosition*/
const Value V18206_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_295 = {S17, 0, &V18206_295_57194, &P18206_294};	/*Property = CssText*/
const Value V18206_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_297 = {S18, 0, &V18206_297_57197, &P18206_295};	/*Property = CssTransforms*/
const Value V18206_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_241 = {S19, 0, &V18206_241_55801, &P18206_297};	/*Property = CssTransitions*/
const Value V18206_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_298 = {S20, 0, &V18206_298_57208, &P18206_241};	/*Property = CssUI*/
const Value V18206_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_300 = {S21, 0, &V18206_300_57217, &P18206_298};	/*Property = DataSet*/
const Value V18206_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_301 = {S22, 0, &V18206_301_57220, &P18206_300};	/*Property = DataUrl*/
const Value V18206_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_302 = {S23, 0, &V18206_302_57223, &P18206_301};	/*Property = DeviceOrientation*/
const Value V18206_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_304 = {S24, 0, &V18206_304_57229, &P18206_302};	/*Property = FileReader*/
const Value V18206_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_305 = {S25, 0, &V18206_305_57232, &P18206_304};	/*Property = FileSaver*/
const Value V18206_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_306 = {S26, 0, &V18206_306_57238, &P18206_305};	/*Property = FileWriter*/
const Value V18206_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_307 = {S27, 0, &V18206_307_57241, &P18206_306};	/*Property = FormData*/
const Value V18206_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_309 = {S28, 0, &V18206_309_57247, &P18206_307};	/*Property = Fullscreen*/
const Value V18206_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_310 = {S29, 0, &V18206_310_57250, &P18206_309};	/*Property = GeoLocation*/
const Value V18206_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_312 = {S30, 0, &V18206_312_57256, &P18206_310};	/*Property = History*/
const Value V18206_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_314 = {S31, 0, &V18206_314_57262, &P18206_312};	/*Property = Html5*/
const Value V18206_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_313 = {S32, 0, &V18206_313_57259, &P18206_314};	/*Property = Html-Media-Capture*/
const Value V18206_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_315 = {S33, 0, &V18206_315_57265, &P18206_313};	/*Property = Iframe*/
const Value V18206_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_316 = {S34, 0, &V18206_316_57268, &P18206_315};	/*Property = IndexedDB*/
const Value V18206_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_317 = {S35, 0, &V18206_317_57271, &P18206_316};	/*Property = Json*/
const Value V18206_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18206_15 = {S37, 0, &V18206_15_28, &P18206_317};	/*Property = LayoutEngine*/
const Value V18206_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_344 = {S38, 0, &V18206_344_57274, &P18206_15};	/*Property = Masking*/
const Value V18206_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_320 = {S39, 0, &V18206_320_57292, &P18206_344};	/*Property = PostMessage*/
const Value V18206_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_321 = {S40, 0, &V18206_321_57295, &P18206_320};	/*Property = Progress*/
const Value V18206_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_322 = {S41, 0, &V18206_322_57298, &P18206_321};	/*Property = Prompts*/
const Value V18206_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_323 = {S42, 0, &V18206_323_57301, &P18206_322};	/*Property = Selector*/
const Value V18206_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_324 = {S43, 0, &V18206_324_57307, &P18206_323};	/*Property = Svg*/
const Value V18206_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_327 = {S44, 0, &V18206_327_57316, &P18206_324};	/*Property = TouchEvents*/
const Value V18206_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_328 = {S45, 0, &V18206_328_57319, &P18206_327};	/*Property = Track*/
const Value V18206_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_329 = {S46, 0, &V18206_329_57325, &P18206_328};	/*Property = Video*/
const Value V18206_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_330 = {S47, 0, &V18206_330_57328, &P18206_329};	/*Property = Viewport*/
const Value V18206_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_332 = {S48, 0, &V18206_332_57340, &P18206_330};	/*Property = WebWorkers*/
const Value V18206_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18206_333 = {S49, 0, &V18206_333_57343, &P18206_332};	/*Property = Xhr2*/
const Profile P18206 = {"18206", &P18206_333};	/*Profile ID - 18206*/
const Value V20318_1_2 = {S0, NULL};	/*Value = False*/
const Property P20318_1 = {S50, 0, &V20318_1_2, NULL};	/*Property = IsMobile*/
const Value V20318_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20318_5 = {S52, 0, &V20318_5_37, &P20318_1};	/*Property = ScreenPixelsHeight*/
const Value V20318_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20318_4 = {S54, 0, &V20318_4_36, &P20318_5};	/*Property = ScreenPixelsWidth*/
const Profile P20318 = {"20318", &P20318_4};	/*Profile ID - 20318*/
const Value V21478_1_1 = {S2, NULL};	/*Value = True*/
const Property P21478_1 = {S50, 0, &V21478_1_1, NULL};	/*Property = IsMobile*/
const Value V21478_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21478_5 = {S52, 0, &V21478_5_1222, &P21478_1};	/*Property = ScreenPixelsHeight*/
const Value V21478_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21478_4 = {S54, 0, &V21478_4_1224, &P21478_5};	/*Property = ScreenPixelsWidth*/
const Profile P21478 = {"21478", &P21478_4};	/*Profile ID - 21478*/
const Value V22422_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22422_279 = {S1, 0, &V22422_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22422_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22422_282 = {S3, 0, &V22422_282_57154, &P22422_279};	/*Property = BlobBuilder*/
const Value V22422_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22422_350 = {S4, 0, &V22422_350_57444, &P22422_282};	/*Property = Canvas*/
const Value V22422_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22422_283 = {S5, 0, &V22422_283_57159, &P22422_350};	/*Property = CssBackground*/
const Value V22422_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22422_284 = {S6, 0, &V22422_284_57163, &P22422_283};	/*Property = CssBorderImage*/
const Value V22422_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22422_285 = {S7, 0, &V22422_285_57166, &P22422_284};	/*Property = CssCanvas*/
const Value V22422_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22422_286 = {S8, 0, &V22422_286_57168, &P22422_285};	/*Property = CssColor*/
const Value V22422_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22422_237 = {S9, 0, &V22422_237_55784, &P22422_286};	/*Property = CssColumn*/
const Value V22422_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22422_287 = {S10, 0, &V22422_287_57172, &P22422_237};	/*Property = CssFlexbox*/
const Value V22422_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22422_288 = {S11, 0, &V22422_288_57174, &P22422_287};	/*Property = CssFont*/
const Value V22422_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22422_289 = {S12, 0, &V22422_289_57759, &P22422_288};	/*Property = CssImages*/
const Value V22422_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22422_291 = {S13, 0, &V22422_291_57180, &P22422_289};	/*Property = CssMediaQueries*/
const Value V22422_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22422_292 = {S14, 0, &V22422_292_57183, &P22422_291};	/*Property = CssMinMax*/
const Value V22422_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22422_293 = {S15, 0, &V22422_293_57187, &P22422_292};	/*Property = CssOverflow*/
const Value V22422_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22422_294 = {S16, 0, &V22422_294_57189, &P22422_293};	/*Property = CssPosition*/
const Value V22422_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22422_295 = {S17, 0, &V22422_295_57192, &P22422_294};	/*Property = CssText*/
const Value V22422_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22422_297 = {S18, 0, &V22422_297_57195, &P22422_295};	/*Property = CssTransforms*/
const Value V22422_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22422_241 = {S19, 0, &V22422_241_55799, &P22422_297};	/*Property = CssTransitions*/
const Value V22422_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22422_298 = {S20, 0, &V22422_298_57206, &P22422_241};	/*Property = CssUI*/
const Value V22422_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22422_300 = {S21, 0, &V22422_300_57216, &P22422_298};	/*Property = DataSet*/
const Value V22422_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22422_301 = {S22, 0, &V22422_301_57218, &P22422_300};	/*Property = DataUrl*/
const Value V22422_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22422_302 = {S23, 0, &V22422_302_57222, &P22422_301};	/*Property = DeviceOrientation*/
const Value V22422_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22422_304 = {S24, 0, &V22422_304_57227, &P22422_302};	/*Property = FileReader*/
const Value V22422_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22422_305 = {S25, 0, &V22422_305_57231, &P22422_304};	/*Property = FileSaver*/
const Value V22422_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22422_306 = {S26, 0, &V22422_306_57237, &P22422_305};	/*Property = FileWriter*/
const Value V22422_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22422_307 = {S27, 0, &V22422_307_57239, &P22422_306};	/*Property = FormData*/
const Value V22422_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22422_309 = {S28, 0, &V22422_309_57246, &P22422_307};	/*Property = Fullscreen*/
const Value V22422_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22422_310 = {S29, 0, &V22422_310_57248, &P22422_309};	/*Property = GeoLocation*/
const Value V22422_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22422_312 = {S30, 0, &V22422_312_57254, &P22422_310};	/*Property = History*/
const Value V22422_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22422_314 = {S31, 0, &V22422_314_57261, &P22422_312};	/*Property = Html5*/
const Value V22422_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22422_313 = {S32, 0, &V22422_313_57258, &P22422_314};	/*Property = Html-Media-Capture*/
const Value V22422_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22422_315 = {S33, 0, &V22422_315_57264, &P22422_313};	/*Property = Iframe*/
const Value V22422_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22422_316 = {S34, 0, &V22422_316_57266, &P22422_315};	/*Property = IndexedDB*/
const Value V22422_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22422_317 = {S35, 0, &V22422_317_57269, &P22422_316};	/*Property = Json*/
const Value V22422_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22422_15 = {S37, 0, &V22422_15_28, &P22422_317};	/*Property = LayoutEngine*/
const Value V22422_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22422_344 = {S38, 0, &V22422_344_57272, &P22422_15};	/*Property = Masking*/
const Value V22422_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22422_320 = {S39, 0, &V22422_320_57290, &P22422_344};	/*Property = PostMessage*/
const Value V22422_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22422_321 = {S40, 0, &V22422_321_57293, &P22422_320};	/*Property = Progress*/
const Value V22422_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22422_322 = {S41, 0, &V22422_322_57296, &P22422_321};	/*Property = Prompts*/
const Value V22422_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22422_323 = {S42, 0, &V22422_323_57299, &P22422_322};	/*Property = Selector*/
const Value V22422_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22422_324 = {S43, 0, &V22422_324_57306, &P22422_323};	/*Property = Svg*/
const Value V22422_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22422_327 = {S44, 0, &V22422_327_57315, &P22422_324};	/*Property = TouchEvents*/
const Value V22422_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22422_328 = {S45, 0, &V22422_328_57318, &P22422_327};	/*Property = Track*/
const Value V22422_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22422_329 = {S46, 0, &V22422_329_57323, &P22422_328};	/*Property = Video*/
const Value V22422_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22422_330 = {S47, 0, &V22422_330_57326, &P22422_329};	/*Property = Viewport*/
const Value V22422_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22422_332 = {S48, 0, &V22422_332_57339, &P22422_330};	/*Property = WebWorkers*/
const Value V22422_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22422_333 = {S49, 0, &V22422_333_57342, &P22422_332};	/*Property = Xhr2*/
const Profile P22422 = {"22422", &P22422_333};	/*Profile ID - 22422*/
const Value V21908_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21908_279 = {S1, 0, &V21908_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21908_282_57153 = {S2, NULL};	/*Value = True*/
const Property P21908_282 = {S3, 0, &V21908_282_57153, &P21908_279};	/*Property = BlobBuilder*/
const Value V21908_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21908_350 = {S4, 0, &V21908_350_57444, &P21908_282};	/*Property = Canvas*/
const Value V21908_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21908_283 = {S5, 0, &V21908_283_57159, &P21908_350};	/*Property = CssBackground*/
const Value V21908_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21908_284 = {S6, 0, &V21908_284_57163, &P21908_283};	/*Property = CssBorderImage*/
const Value V21908_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21908_285 = {S7, 0, &V21908_285_57166, &P21908_284};	/*Property = CssCanvas*/
const Value V21908_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21908_286 = {S8, 0, &V21908_286_57168, &P21908_285};	/*Property = CssColor*/
const Value V21908_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21908_237 = {S9, 0, &V21908_237_55784, &P21908_286};	/*Property = CssColumn*/
const Value V21908_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21908_287 = {S10, 0, &V21908_287_57172, &P21908_237};	/*Property = CssFlexbox*/
const Value V21908_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21908_288 = {S11, 0, &V21908_288_57175, &P21908_287};	/*Property = CssFont*/
const Value V21908_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21908_289 = {S12, 0, &V21908_289_57759, &P21908_288};	/*Property = CssImages*/
const Value V21908_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21908_291 = {S13, 0, &V21908_291_57181, &P21908_289};	/*Property = CssMediaQueries*/
const Value V21908_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21908_292 = {S14, 0, &V21908_292_57183, &P21908_291};	/*Property = CssMinMax*/
const Value V21908_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21908_293 = {S15, 0, &V21908_293_57187, &P21908_292};	/*Property = CssOverflow*/
const Value V21908_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21908_294 = {S16, 0, &V21908_294_57189, &P21908_293};	/*Property = CssPosition*/
const Value V21908_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21908_295 = {S17, 0, &V21908_295_57192, &P21908_294};	/*Property = CssText*/
const Value V21908_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21908_297 = {S18, 0, &V21908_297_57195, &P21908_295};	/*Property = CssTransforms*/
const Value V21908_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21908_241 = {S19, 0, &V21908_241_55799, &P21908_297};	/*Property = CssTransitions*/
const Value V21908_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21908_298 = {S20, 0, &V21908_298_57206, &P21908_241};	/*Property = CssUI*/
const Value V21908_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21908_300 = {S21, 0, &V21908_300_57216, &P21908_298};	/*Property = DataSet*/
const Value V21908_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21908_301 = {S22, 0, &V21908_301_57218, &P21908_300};	/*Property = DataUrl*/
const Value V21908_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21908_302 = {S23, 0, &V21908_302_57222, &P21908_301};	/*Property = DeviceOrientation*/
const Value V21908_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21908_304 = {S24, 0, &V21908_304_57227, &P21908_302};	/*Property = FileReader*/
const Value V21908_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21908_305 = {S25, 0, &V21908_305_57231, &P21908_304};	/*Property = FileSaver*/
const Value V21908_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21908_306 = {S26, 0, &V21908_306_57237, &P21908_305};	/*Property = FileWriter*/
const Value V21908_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21908_307 = {S27, 0, &V21908_307_57239, &P21908_306};	/*Property = FormData*/
const Value V21908_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21908_309 = {S28, 0, &V21908_309_57246, &P21908_307};	/*Property = Fullscreen*/
const Value V21908_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21908_310 = {S29, 0, &V21908_310_57248, &P21908_309};	/*Property = GeoLocation*/
const Value V21908_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21908_312 = {S30, 0, &V21908_312_57254, &P21908_310};	/*Property = History*/
const Value V21908_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21908_314 = {S31, 0, &V21908_314_57261, &P21908_312};	/*Property = Html5*/
const Value V21908_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21908_313 = {S32, 0, &V21908_313_57258, &P21908_314};	/*Property = Html-Media-Capture*/
const Value V21908_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21908_315 = {S33, 0, &V21908_315_57264, &P21908_313};	/*Property = Iframe*/
const Value V21908_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21908_316 = {S34, 0, &V21908_316_57266, &P21908_315};	/*Property = IndexedDB*/
const Value V21908_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21908_317 = {S35, 0, &V21908_317_57269, &P21908_316};	/*Property = Json*/
const Value V21908_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21908_15 = {S37, 0, &V21908_15_28, &P21908_317};	/*Property = LayoutEngine*/
const Value V21908_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21908_344 = {S38, 0, &V21908_344_57272, &P21908_15};	/*Property = Masking*/
const Value V21908_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21908_320 = {S39, 0, &V21908_320_57290, &P21908_344};	/*Property = PostMessage*/
const Value V21908_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21908_321 = {S40, 0, &V21908_321_57293, &P21908_320};	/*Property = Progress*/
const Value V21908_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21908_322 = {S41, 0, &V21908_322_57296, &P21908_321};	/*Property = Prompts*/
const Value V21908_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21908_323 = {S42, 0, &V21908_323_57299, &P21908_322};	/*Property = Selector*/
const Value V21908_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21908_324 = {S43, 0, &V21908_324_57306, &P21908_323};	/*Property = Svg*/
const Value V21908_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21908_327 = {S44, 0, &V21908_327_57315, &P21908_324};	/*Property = TouchEvents*/
const Value V21908_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21908_328 = {S45, 0, &V21908_328_57318, &P21908_327};	/*Property = Track*/
const Value V21908_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21908_329 = {S46, 0, &V21908_329_57323, &P21908_328};	/*Property = Video*/
const Value V21908_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21908_330 = {S47, 0, &V21908_330_57326, &P21908_329};	/*Property = Viewport*/
const Value V21908_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21908_332 = {S48, 0, &V21908_332_57339, &P21908_330};	/*Property = WebWorkers*/
const Value V21908_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21908_333 = {S49, 0, &V21908_333_57342, &P21908_332};	/*Property = Xhr2*/
const Profile P21908 = {"21908", &P21908_333};	/*Profile ID - 21908*/
const Profile P18203 = {"18203", NULL};	/*Profile ID - 18203*/
const Value V18171_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18171_279 = {S1, 0, &V18171_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18171_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18171_282 = {S3, 0, &V18171_282_57154, &P18171_279};	/*Property = BlobBuilder*/
const Value V18171_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18171_350 = {S4, 0, &V18171_350_57444, &P18171_282};	/*Property = Canvas*/
const Value V18171_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18171_283 = {S5, 0, &V18171_283_57159, &P18171_350};	/*Property = CssBackground*/
const Value V18171_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18171_284 = {S6, 0, &V18171_284_57163, &P18171_283};	/*Property = CssBorderImage*/
const Value V18171_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18171_285 = {S7, 0, &V18171_285_57166, &P18171_284};	/*Property = CssCanvas*/
const Value V18171_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18171_286 = {S8, 0, &V18171_286_57168, &P18171_285};	/*Property = CssColor*/
const Value V18171_237_55784 = {S2, NULL};	/*Value = True*/
const Property P18171_237 = {S9, 0, &V18171_237_55784, &P18171_286};	/*Property = CssColumn*/
const Value V18171_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18171_287 = {S10, 0, &V18171_287_57172, &P18171_237};	/*Property = CssFlexbox*/
const Value V18171_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18171_288 = {S11, 0, &V18171_288_57175, &P18171_287};	/*Property = CssFont*/
const Value V18171_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18171_289 = {S12, 0, &V18171_289_57759, &P18171_288};	/*Property = CssImages*/
const Value V18171_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18171_291 = {S13, 0, &V18171_291_57181, &P18171_289};	/*Property = CssMediaQueries*/
const Value V18171_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18171_292 = {S14, 0, &V18171_292_57183, &P18171_291};	/*Property = CssMinMax*/
const Value V18171_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18171_293 = {S15, 0, &V18171_293_57187, &P18171_292};	/*Property = CssOverflow*/
const Value V18171_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18171_294 = {S16, 0, &V18171_294_57190, &P18171_293};	/*Property = CssPosition*/
const Value V18171_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18171_295 = {S17, 0, &V18171_295_57193, &P18171_294};	/*Property = CssText*/
const Value V18171_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18171_297 = {S18, 0, &V18171_297_57195, &P18171_295};	/*Property = CssTransforms*/
const Value V18171_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18171_241 = {S19, 0, &V18171_241_55799, &P18171_297};	/*Property = CssTransitions*/
const Value V18171_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18171_298 = {S20, 0, &V18171_298_57206, &P18171_241};	/*Property = CssUI*/
const Value V18171_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18171_300 = {S21, 0, &V18171_300_57216, &P18171_298};	/*Property = DataSet*/
const Value V18171_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18171_301 = {S22, 0, &V18171_301_57218, &P18171_300};	/*Property = DataUrl*/
const Value V18171_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18171_302 = {S23, 0, &V18171_302_57222, &P18171_301};	/*Property = DeviceOrientation*/
const Value V18171_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18171_304 = {S24, 0, &V18171_304_57228, &P18171_302};	/*Property = FileReader*/
const Value V18171_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18171_305 = {S25, 0, &V18171_305_57231, &P18171_304};	/*Property = FileSaver*/
const Value V18171_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18171_306 = {S26, 0, &V18171_306_57237, &P18171_305};	/*Property = FileWriter*/
const Value V18171_307_57240 = {S0, NULL};	/*Value = False*/
const Property P18171_307 = {S27, 0, &V18171_307_57240, &P18171_306};	/*Property = FormData*/
const Value V18171_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18171_309 = {S28, 0, &V18171_309_57246, &P18171_307};	/*Property = Fullscreen*/
const Value V18171_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18171_310 = {S29, 0, &V18171_310_57248, &P18171_309};	/*Property = GeoLocation*/
const Value V18171_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18171_312 = {S30, 0, &V18171_312_57255, &P18171_310};	/*Property = History*/
const Value V18171_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18171_314 = {S31, 0, &V18171_314_57261, &P18171_312};	/*Property = Html5*/
const Value V18171_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18171_313 = {S32, 0, &V18171_313_57258, &P18171_314};	/*Property = Html-Media-Capture*/
const Value V18171_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18171_315 = {S33, 0, &V18171_315_57264, &P18171_313};	/*Property = Iframe*/
const Value V18171_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18171_316 = {S34, 0, &V18171_316_57267, &P18171_315};	/*Property = IndexedDB*/
const Value V18171_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18171_317 = {S35, 0, &V18171_317_57269, &P18171_316};	/*Property = Json*/
const Value V18171_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18171_15 = {S37, 0, &V18171_15_29, &P18171_317};	/*Property = LayoutEngine*/
const Value V18171_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18171_344 = {S38, 0, &V18171_344_57272, &P18171_15};	/*Property = Masking*/
const Value V18171_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18171_320 = {S39, 0, &V18171_320_57290, &P18171_344};	/*Property = PostMessage*/
const Value V18171_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18171_321 = {S40, 0, &V18171_321_57293, &P18171_320};	/*Property = Progress*/
const Value V18171_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18171_322 = {S41, 0, &V18171_322_57296, &P18171_321};	/*Property = Prompts*/
const Value V18171_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18171_323 = {S42, 0, &V18171_323_57299, &P18171_322};	/*Property = Selector*/
const Value V18171_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18171_324 = {S43, 0, &V18171_324_57306, &P18171_323};	/*Property = Svg*/
const Value V18171_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18171_327 = {S44, 0, &V18171_327_57315, &P18171_324};	/*Property = TouchEvents*/
const Value V18171_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18171_328 = {S45, 0, &V18171_328_57318, &P18171_327};	/*Property = Track*/
const Value V18171_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18171_329 = {S46, 0, &V18171_329_57323, &P18171_328};	/*Property = Video*/
const Value V18171_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18171_330 = {S47, 0, &V18171_330_57326, &P18171_329};	/*Property = Viewport*/
const Value V18171_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18171_332 = {S48, 0, &V18171_332_57339, &P18171_330};	/*Property = WebWorkers*/
const Value V18171_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18171_333 = {S49, 0, &V18171_333_57342, &P18171_332};	/*Property = Xhr2*/
const Profile P18171 = {"18171", &P18171_333};	/*Profile ID - 18171*/
const Profile P18204 = {"18204", NULL};	/*Profile ID - 18204*/
const Value V18141_1_1 = {S2, NULL};	/*Value = True*/
const Property P18141_1 = {S50, 0, &V18141_1_1, NULL};	/*Property = IsMobile*/
const Value V18141_5_56838 = {S126, NULL};	/*Value = 544*/
const Property P18141_5 = {S52, 0, &V18141_5_56838, &P18141_1};	/*Property = ScreenPixelsHeight*/
const Value V18141_4_2299 = {S82, NULL};	/*Value = 960*/
const Property P18141_4 = {S54, 0, &V18141_4_2299, &P18141_5};	/*Property = ScreenPixelsWidth*/
const Profile P18141 = {"18141", &P18141_4};	/*Profile ID - 18141*/
const Value V23485_279_57147 = {S2, NULL};	/*Value = True*/
const Property P23485_279 = {S1, 0, &V23485_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V23485_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23485_282 = {S3, 0, &V23485_282_57154, &P23485_279};	/*Property = BlobBuilder*/
const Value V23485_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23485_350 = {S4, 0, &V23485_350_57444, &P23485_282};	/*Property = Canvas*/
const Value V23485_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23485_283 = {S5, 0, &V23485_283_57159, &P23485_350};	/*Property = CssBackground*/
const Value V23485_284_57162 = {S2, NULL};	/*Value = True*/
const Property P23485_284 = {S6, 0, &V23485_284_57162, &P23485_283};	/*Property = CssBorderImage*/
const Value V23485_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23485_285 = {S7, 0, &V23485_285_57166, &P23485_284};	/*Property = CssCanvas*/
const Value V23485_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23485_286 = {S8, 0, &V23485_286_57168, &P23485_285};	/*Property = CssColor*/
const Value V23485_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23485_237 = {S9, 0, &V23485_237_55786, &P23485_286};	/*Property = CssColumn*/
const Value V23485_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23485_287 = {S10, 0, &V23485_287_57172, &P23485_237};	/*Property = CssFlexbox*/
const Value V23485_288_57175 = {S0, NULL};	/*Value = False*/
const Property P23485_288 = {S11, 0, &V23485_288_57175, &P23485_287};	/*Property = CssFont*/
const Value V23485_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23485_289 = {S12, 0, &V23485_289_57759, &P23485_288};	/*Property = CssImages*/
const Value V23485_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23485_291 = {S13, 0, &V23485_291_57180, &P23485_289};	/*Property = CssMediaQueries*/
const Value V23485_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23485_292 = {S14, 0, &V23485_292_57183, &P23485_291};	/*Property = CssMinMax*/
const Value V23485_293_57187 = {S0, NULL};	/*Value = False*/
const Property P23485_293 = {S15, 0, &V23485_293_57187, &P23485_292};	/*Property = CssOverflow*/
const Value V23485_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23485_294 = {S16, 0, &V23485_294_57189, &P23485_293};	/*Property = CssPosition*/
const Value V23485_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23485_295 = {S17, 0, &V23485_295_57192, &P23485_294};	/*Property = CssText*/
const Value V23485_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23485_297 = {S18, 0, &V23485_297_57195, &P23485_295};	/*Property = CssTransforms*/
const Value V23485_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23485_241 = {S19, 0, &V23485_241_55799, &P23485_297};	/*Property = CssTransitions*/
const Value V23485_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23485_298 = {S20, 0, &V23485_298_57206, &P23485_241};	/*Property = CssUI*/
const Value V23485_300_57215 = {S2, NULL};	/*Value = True*/
const Property P23485_300 = {S21, 0, &V23485_300_57215, &P23485_298};	/*Property = DataSet*/
const Value V23485_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23485_301 = {S22, 0, &V23485_301_57218, &P23485_300};	/*Property = DataUrl*/
const Value V23485_302_57222 = {S0, NULL};	/*Value = False*/
const Property P23485_302 = {S23, 0, &V23485_302_57222, &P23485_301};	/*Property = DeviceOrientation*/
const Value V23485_304_57228 = {S0, NULL};	/*Value = False*/
const Property P23485_304 = {S24, 0, &V23485_304_57228, &P23485_302};	/*Property = FileReader*/
const Value V23485_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23485_305 = {S25, 0, &V23485_305_57231, &P23485_304};	/*Property = FileSaver*/
const Value V23485_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23485_306 = {S26, 0, &V23485_306_57237, &P23485_305};	/*Property = FileWriter*/
const Value V23485_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23485_307 = {S27, 0, &V23485_307_57239, &P23485_306};	/*Property = FormData*/
const Value V23485_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23485_309 = {S28, 0, &V23485_309_57246, &P23485_307};	/*Property = Fullscreen*/
const Value V23485_310_57249 = {S0, NULL};	/*Value = False*/
const Property P23485_310 = {S29, 0, &V23485_310_57249, &P23485_309};	/*Property = GeoLocation*/
const Value V23485_312_57254 = {S2, NULL};	/*Value = True*/
const Property P23485_312 = {S30, 0, &V23485_312_57254, &P23485_310};	/*Property = History*/
const Value V23485_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23485_314 = {S31, 0, &V23485_314_57261, &P23485_312};	/*Property = Html5*/
const Value V23485_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23485_313 = {S32, 0, &V23485_313_57258, &P23485_314};	/*Property = Html-Media-Capture*/
const Value V23485_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23485_315 = {S33, 0, &V23485_315_57264, &P23485_313};	/*Property = Iframe*/
const Value V23485_316_57267 = {S0, NULL};	/*Value = False*/
const Property P23485_316 = {S34, 0, &V23485_316_57267, &P23485_315};	/*Property = IndexedDB*/
const Value V23485_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23485_317 = {S35, 0, &V23485_317_57269, &P23485_316};	/*Property = Json*/
const Value V23485_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P23485_15 = {S37, 0, &V23485_15_28, &P23485_317};	/*Property = LayoutEngine*/
const Value V23485_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23485_344 = {S38, 0, &V23485_344_57272, &P23485_15};	/*Property = Masking*/
const Value V23485_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23485_320 = {S39, 0, &V23485_320_57290, &P23485_344};	/*Property = PostMessage*/
const Value V23485_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23485_321 = {S40, 0, &V23485_321_57293, &P23485_320};	/*Property = Progress*/
const Value V23485_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23485_322 = {S41, 0, &V23485_322_57296, &P23485_321};	/*Property = Prompts*/
const Value V23485_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23485_323 = {S42, 0, &V23485_323_57299, &P23485_322};	/*Property = Selector*/
const Value V23485_324_57305 = {S2, NULL};	/*Value = True*/
const Property P23485_324 = {S43, 0, &V23485_324_57305, &P23485_323};	/*Property = Svg*/
const Value V23485_327_57314 = {S2, NULL};	/*Value = True*/
const Property P23485_327 = {S44, 0, &V23485_327_57314, &P23485_324};	/*Property = TouchEvents*/
const Value V23485_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23485_328 = {S45, 0, &V23485_328_57318, &P23485_327};	/*Property = Track*/
const Value V23485_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23485_329 = {S46, 0, &V23485_329_57323, &P23485_328};	/*Property = Video*/
const Value V23485_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23485_330 = {S47, 0, &V23485_330_57326, &P23485_329};	/*Property = Viewport*/
const Value V23485_332_57339 = {S0, NULL};	/*Value = False*/
const Property P23485_332 = {S48, 0, &V23485_332_57339, &P23485_330};	/*Property = WebWorkers*/
const Value V23485_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23485_333 = {S49, 0, &V23485_333_57342, &P23485_332};	/*Property = Xhr2*/
const Profile P23485 = {"23485", &P23485_333};	/*Profile ID - 23485*/
const Value V22455_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22455_279 = {S1, 0, &V22455_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22455_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22455_282 = {S3, 0, &V22455_282_57153, &P22455_279};	/*Property = BlobBuilder*/
const Value V22455_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22455_350 = {S4, 0, &V22455_350_57444, &P22455_282};	/*Property = Canvas*/
const Value V22455_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22455_283 = {S5, 0, &V22455_283_57159, &P22455_350};	/*Property = CssBackground*/
const Value V22455_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22455_284 = {S6, 0, &V22455_284_57163, &P22455_283};	/*Property = CssBorderImage*/
const Value V22455_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22455_285 = {S7, 0, &V22455_285_57166, &P22455_284};	/*Property = CssCanvas*/
const Value V22455_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22455_286 = {S8, 0, &V22455_286_57168, &P22455_285};	/*Property = CssColor*/
const Value V22455_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22455_237 = {S9, 0, &V22455_237_55784, &P22455_286};	/*Property = CssColumn*/
const Value V22455_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22455_287 = {S10, 0, &V22455_287_57172, &P22455_237};	/*Property = CssFlexbox*/
const Value V22455_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22455_288 = {S11, 0, &V22455_288_57175, &P22455_287};	/*Property = CssFont*/
const Value V22455_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22455_289 = {S12, 0, &V22455_289_57759, &P22455_288};	/*Property = CssImages*/
const Value V22455_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22455_291 = {S13, 0, &V22455_291_57180, &P22455_289};	/*Property = CssMediaQueries*/
const Value V22455_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22455_292 = {S14, 0, &V22455_292_57183, &P22455_291};	/*Property = CssMinMax*/
const Value V22455_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22455_293 = {S15, 0, &V22455_293_57187, &P22455_292};	/*Property = CssOverflow*/
const Value V22455_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22455_294 = {S16, 0, &V22455_294_57189, &P22455_293};	/*Property = CssPosition*/
const Value V22455_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22455_295 = {S17, 0, &V22455_295_57192, &P22455_294};	/*Property = CssText*/
const Value V22455_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22455_297 = {S18, 0, &V22455_297_57195, &P22455_295};	/*Property = CssTransforms*/
const Value V22455_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22455_241 = {S19, 0, &V22455_241_55799, &P22455_297};	/*Property = CssTransitions*/
const Value V22455_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22455_298 = {S20, 0, &V22455_298_57206, &P22455_241};	/*Property = CssUI*/
const Value V22455_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22455_300 = {S21, 0, &V22455_300_57216, &P22455_298};	/*Property = DataSet*/
const Value V22455_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22455_301 = {S22, 0, &V22455_301_57218, &P22455_300};	/*Property = DataUrl*/
const Value V22455_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22455_302 = {S23, 0, &V22455_302_57222, &P22455_301};	/*Property = DeviceOrientation*/
const Value V22455_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22455_304 = {S24, 0, &V22455_304_57227, &P22455_302};	/*Property = FileReader*/
const Value V22455_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22455_305 = {S25, 0, &V22455_305_57231, &P22455_304};	/*Property = FileSaver*/
const Value V22455_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22455_306 = {S26, 0, &V22455_306_57237, &P22455_305};	/*Property = FileWriter*/
const Value V22455_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22455_307 = {S27, 0, &V22455_307_57239, &P22455_306};	/*Property = FormData*/
const Value V22455_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22455_309 = {S28, 0, &V22455_309_57246, &P22455_307};	/*Property = Fullscreen*/
const Value V22455_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22455_310 = {S29, 0, &V22455_310_57248, &P22455_309};	/*Property = GeoLocation*/
const Value V22455_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22455_312 = {S30, 0, &V22455_312_57254, &P22455_310};	/*Property = History*/
const Value V22455_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22455_314 = {S31, 0, &V22455_314_57261, &P22455_312};	/*Property = Html5*/
const Value V22455_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22455_313 = {S32, 0, &V22455_313_57258, &P22455_314};	/*Property = Html-Media-Capture*/
const Value V22455_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22455_315 = {S33, 0, &V22455_315_57264, &P22455_313};	/*Property = Iframe*/
const Value V22455_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22455_316 = {S34, 0, &V22455_316_57266, &P22455_315};	/*Property = IndexedDB*/
const Value V22455_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22455_317 = {S35, 0, &V22455_317_57269, &P22455_316};	/*Property = Json*/
const Value V22455_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22455_15 = {S37, 0, &V22455_15_28, &P22455_317};	/*Property = LayoutEngine*/
const Value V22455_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22455_344 = {S38, 0, &V22455_344_57272, &P22455_15};	/*Property = Masking*/
const Value V22455_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22455_320 = {S39, 0, &V22455_320_57290, &P22455_344};	/*Property = PostMessage*/
const Value V22455_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22455_321 = {S40, 0, &V22455_321_57293, &P22455_320};	/*Property = Progress*/
const Value V22455_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22455_322 = {S41, 0, &V22455_322_57296, &P22455_321};	/*Property = Prompts*/
const Value V22455_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22455_323 = {S42, 0, &V22455_323_57299, &P22455_322};	/*Property = Selector*/
const Value V22455_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22455_324 = {S43, 0, &V22455_324_57306, &P22455_323};	/*Property = Svg*/
const Value V22455_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22455_327 = {S44, 0, &V22455_327_57315, &P22455_324};	/*Property = TouchEvents*/
const Value V22455_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22455_328 = {S45, 0, &V22455_328_57318, &P22455_327};	/*Property = Track*/
const Value V22455_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22455_329 = {S46, 0, &V22455_329_57323, &P22455_328};	/*Property = Video*/
const Value V22455_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22455_330 = {S47, 0, &V22455_330_57326, &P22455_329};	/*Property = Viewport*/
const Value V22455_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22455_332 = {S48, 0, &V22455_332_57338, &P22455_330};	/*Property = WebWorkers*/
const Value V22455_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22455_333 = {S49, 0, &V22455_333_57342, &P22455_332};	/*Property = Xhr2*/
const Profile P22455 = {"22455", &P22455_333};	/*Profile ID - 22455*/
const Value V18186_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_279 = {S1, 0, &V18186_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18186_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_282 = {S3, 0, &V18186_282_57155, &P18186_279};	/*Property = BlobBuilder*/
const Value V18186_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_350 = {S4, 0, &V18186_350_57446, &P18186_282};	/*Property = Canvas*/
const Value V18186_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_283 = {S5, 0, &V18186_283_57161, &P18186_350};	/*Property = CssBackground*/
const Value V18186_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_284 = {S6, 0, &V18186_284_57164, &P18186_283};	/*Property = CssBorderImage*/
const Value V18186_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_285 = {S7, 0, &V18186_285_57167, &P18186_284};	/*Property = CssCanvas*/
const Value V18186_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_286 = {S8, 0, &V18186_286_57170, &P18186_285};	/*Property = CssColor*/
const Value V18186_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_237 = {S9, 0, &V18186_237_55786, &P18186_286};	/*Property = CssColumn*/
const Value V18186_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_287 = {S10, 0, &V18186_287_57173, &P18186_237};	/*Property = CssFlexbox*/
const Value V18186_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_288 = {S11, 0, &V18186_288_57176, &P18186_287};	/*Property = CssFont*/
const Value V18186_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_289 = {S12, 0, &V18186_289_86899, &P18186_288};	/*Property = CssImages*/
const Value V18186_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_291 = {S13, 0, &V18186_291_57182, &P18186_289};	/*Property = CssMediaQueries*/
const Value V18186_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_292 = {S14, 0, &V18186_292_57185, &P18186_291};	/*Property = CssMinMax*/
const Value V18186_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_293 = {S15, 0, &V18186_293_57188, &P18186_292};	/*Property = CssOverflow*/
const Value V18186_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_294 = {S16, 0, &V18186_294_57191, &P18186_293};	/*Property = CssPosition*/
const Value V18186_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_295 = {S17, 0, &V18186_295_57194, &P18186_294};	/*Property = CssText*/
const Value V18186_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_297 = {S18, 0, &V18186_297_57197, &P18186_295};	/*Property = CssTransforms*/
const Value V18186_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_241 = {S19, 0, &V18186_241_55801, &P18186_297};	/*Property = CssTransitions*/
const Value V18186_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_298 = {S20, 0, &V18186_298_57208, &P18186_241};	/*Property = CssUI*/
const Value V18186_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_300 = {S21, 0, &V18186_300_57217, &P18186_298};	/*Property = DataSet*/
const Value V18186_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_301 = {S22, 0, &V18186_301_57220, &P18186_300};	/*Property = DataUrl*/
const Value V18186_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_302 = {S23, 0, &V18186_302_57223, &P18186_301};	/*Property = DeviceOrientation*/
const Value V18186_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_304 = {S24, 0, &V18186_304_57229, &P18186_302};	/*Property = FileReader*/
const Value V18186_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_305 = {S25, 0, &V18186_305_57232, &P18186_304};	/*Property = FileSaver*/
const Value V18186_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_306 = {S26, 0, &V18186_306_57238, &P18186_305};	/*Property = FileWriter*/
const Value V18186_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_307 = {S27, 0, &V18186_307_57241, &P18186_306};	/*Property = FormData*/
const Value V18186_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_309 = {S28, 0, &V18186_309_57247, &P18186_307};	/*Property = Fullscreen*/
const Value V18186_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_310 = {S29, 0, &V18186_310_57250, &P18186_309};	/*Property = GeoLocation*/
const Value V18186_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_312 = {S30, 0, &V18186_312_57256, &P18186_310};	/*Property = History*/
const Value V18186_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_314 = {S31, 0, &V18186_314_57262, &P18186_312};	/*Property = Html5*/
const Value V18186_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_313 = {S32, 0, &V18186_313_57259, &P18186_314};	/*Property = Html-Media-Capture*/
const Value V18186_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_315 = {S33, 0, &V18186_315_57265, &P18186_313};	/*Property = Iframe*/
const Value V18186_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_316 = {S34, 0, &V18186_316_57268, &P18186_315};	/*Property = IndexedDB*/
const Value V18186_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_317 = {S35, 0, &V18186_317_57271, &P18186_316};	/*Property = Json*/
const Value V18186_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18186_15 = {S37, 0, &V18186_15_29, &P18186_317};	/*Property = LayoutEngine*/
const Value V18186_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_344 = {S38, 0, &V18186_344_57274, &P18186_15};	/*Property = Masking*/
const Value V18186_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_320 = {S39, 0, &V18186_320_57292, &P18186_344};	/*Property = PostMessage*/
const Value V18186_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_321 = {S40, 0, &V18186_321_57295, &P18186_320};	/*Property = Progress*/
const Value V18186_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_322 = {S41, 0, &V18186_322_57298, &P18186_321};	/*Property = Prompts*/
const Value V18186_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_323 = {S42, 0, &V18186_323_57301, &P18186_322};	/*Property = Selector*/
const Value V18186_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_324 = {S43, 0, &V18186_324_57307, &P18186_323};	/*Property = Svg*/
const Value V18186_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_327 = {S44, 0, &V18186_327_57316, &P18186_324};	/*Property = TouchEvents*/
const Value V18186_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_328 = {S45, 0, &V18186_328_57319, &P18186_327};	/*Property = Track*/
const Value V18186_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_329 = {S46, 0, &V18186_329_57325, &P18186_328};	/*Property = Video*/
const Value V18186_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_330 = {S47, 0, &V18186_330_57328, &P18186_329};	/*Property = Viewport*/
const Value V18186_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_332 = {S48, 0, &V18186_332_57340, &P18186_330};	/*Property = WebWorkers*/
const Value V18186_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18186_333 = {S49, 0, &V18186_333_57343, &P18186_332};	/*Property = Xhr2*/
const Profile P18186 = {"18186", &P18186_333};	/*Profile ID - 18186*/
const Value V18183_1_1 = {S2, NULL};	/*Value = True*/
const Property P18183_1 = {S50, 0, &V18183_1_1, NULL};	/*Property = IsMobile*/
const Value V18183_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18183_5 = {S52, 0, &V18183_5_1227, &P18183_1};	/*Property = ScreenPixelsHeight*/
const Value V18183_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P18183_4 = {S54, 0, &V18183_4_1477, &P18183_5};	/*Property = ScreenPixelsWidth*/
const Profile P18183 = {"18183", &P18183_4};	/*Profile ID - 18183*/
const Value V20285_1_1 = {S2, NULL};	/*Value = True*/
const Property P20285_1 = {S50, 0, &V20285_1_1, NULL};	/*Property = IsMobile*/
const Value V20285_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20285_5 = {S52, 0, &V20285_5_9981, &P20285_1};	/*Property = ScreenPixelsHeight*/
const Value V20285_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20285_4 = {S54, 0, &V20285_4_2336, &P20285_5};	/*Property = ScreenPixelsWidth*/
const Profile P20285 = {"20285", &P20285_4};	/*Profile ID - 20285*/
const Value V20558_1_1 = {S2, NULL};	/*Value = True*/
const Property P20558_1 = {S50, 0, &V20558_1_1, NULL};	/*Property = IsMobile*/
const Value V20558_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20558_5 = {S52, 0, &V20558_5_9981, &P20558_1};	/*Property = ScreenPixelsHeight*/
const Value V20558_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20558_4 = {S54, 0, &V20558_4_2336, &P20558_5};	/*Property = ScreenPixelsWidth*/
const Profile P20558 = {"20558", &P20558_4};	/*Profile ID - 20558*/
const Value V20557_1_1 = {S2, NULL};	/*Value = True*/
const Property P20557_1 = {S50, 0, &V20557_1_1, NULL};	/*Property = IsMobile*/
const Value V20557_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20557_5 = {S52, 0, &V20557_5_9981, &P20557_1};	/*Property = ScreenPixelsHeight*/
const Value V20557_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20557_4 = {S54, 0, &V20557_4_2336, &P20557_5};	/*Property = ScreenPixelsWidth*/
const Profile P20557 = {"20557", &P20557_4};	/*Profile ID - 20557*/
const Value V18184_1_1 = {S2, NULL};	/*Value = True*/
const Property P18184_1 = {S50, 0, &V18184_1_1, NULL};	/*Property = IsMobile*/
const Value V18184_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18184_5 = {S52, 0, &V18184_5_1222, &P18184_1};	/*Property = ScreenPixelsHeight*/
const Value V18184_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18184_4 = {S54, 0, &V18184_4_1229, &P18184_5};	/*Property = ScreenPixelsWidth*/
const Profile P18184 = {"18184", &P18184_4};	/*Profile ID - 18184*/
const Value V18185_1_1 = {S2, NULL};	/*Value = True*/
const Property P18185_1 = {S50, 0, &V18185_1_1, NULL};	/*Property = IsMobile*/
const Value V18185_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18185_5 = {S52, 0, &V18185_5_1277, &P18185_1};	/*Property = ScreenPixelsHeight*/
const Value V18185_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18185_4 = {S54, 0, &V18185_4_2336, &P18185_5};	/*Property = ScreenPixelsWidth*/
const Profile P18185 = {"18185", &P18185_4};	/*Profile ID - 18185*/
const Value V16463_1_1 = {S2, NULL};	/*Value = True*/
const Property P16463_1 = {S50, 0, &V16463_1_1, NULL};	/*Property = IsMobile*/
const Value V16463_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16463_5 = {S52, 0, &V16463_5_1227, &P16463_1};	/*Property = ScreenPixelsHeight*/
const Value V16463_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16463_4 = {S54, 0, &V16463_4_1229, &P16463_5};	/*Property = ScreenPixelsWidth*/
const Profile P16463 = {"16463", &P16463_4};	/*Profile ID - 16463*/
const Value V18218_1_1 = {S2, NULL};	/*Value = True*/
const Property P18218_1 = {S50, 0, &V18218_1_1, NULL};	/*Property = IsMobile*/
const Value V18218_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18218_5 = {S52, 0, &V18218_5_1227, &P18218_1};	/*Property = ScreenPixelsHeight*/
const Value V18218_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18218_4 = {S54, 0, &V18218_4_1229, &P18218_5};	/*Property = ScreenPixelsWidth*/
const Profile P18218 = {"18218", &P18218_4};	/*Profile ID - 18218*/
const Value V15791_1_1 = {S2, NULL};	/*Value = True*/
const Property P15791_1 = {S50, 0, &V15791_1_1, NULL};	/*Property = IsMobile*/
const Value V15791_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P15791_5 = {S52, 0, &V15791_5_1275, &P15791_1};	/*Property = ScreenPixelsHeight*/
const Value V15791_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P15791_4 = {S54, 0, &V15791_4_1278, &P15791_5};	/*Property = ScreenPixelsWidth*/
const Profile P15791 = {"15791", &P15791_4};	/*Profile ID - 15791*/
const Value V18225_1_1 = {S2, NULL};	/*Value = True*/
const Property P18225_1 = {S50, 0, &V18225_1_1, NULL};	/*Property = IsMobile*/
const Value V18225_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18225_5 = {S52, 0, &V18225_5_1403, &P18225_1};	/*Property = ScreenPixelsHeight*/
const Value V18225_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18225_4 = {S54, 0, &V18225_4_1404, &P18225_5};	/*Property = ScreenPixelsWidth*/
const Profile P18225 = {"18225", &P18225_4};	/*Profile ID - 18225*/
const Value V18226_1_1 = {S2, NULL};	/*Value = True*/
const Property P18226_1 = {S50, 0, &V18226_1_1, NULL};	/*Property = IsMobile*/
const Value V18226_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18226_5 = {S52, 0, &V18226_5_9981, &P18226_1};	/*Property = ScreenPixelsHeight*/
const Value V18226_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18226_4 = {S54, 0, &V18226_4_39098, &P18226_5};	/*Property = ScreenPixelsWidth*/
const Profile P18226 = {"18226", &P18226_4};	/*Profile ID - 18226*/
const Value V18239_1_1 = {S2, NULL};	/*Value = True*/
const Property P18239_1 = {S50, 0, &V18239_1_1, NULL};	/*Property = IsMobile*/
const Value V18239_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18239_5 = {S52, 0, &V18239_5_1227, &P18239_1};	/*Property = ScreenPixelsHeight*/
const Value V18239_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P18239_4 = {S54, 0, &V18239_4_1477, &P18239_5};	/*Property = ScreenPixelsWidth*/
const Profile P18239 = {"18239", &P18239_4};	/*Profile ID - 18239*/
const Value V18248_1_1 = {S2, NULL};	/*Value = True*/
const Property P18248_1 = {S50, 0, &V18248_1_1, NULL};	/*Property = IsMobile*/
const Value V18248_5_37526 = {S122, NULL};	/*Value = 1200*/
const Property P18248_5 = {S52, 0, &V18248_5_37526, &P18248_1};	/*Property = ScreenPixelsHeight*/
const Value V18248_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P18248_4 = {S54, 0, &V18248_4_57095, &P18248_5};	/*Property = ScreenPixelsWidth*/
const Profile P18248 = {"18248", &P18248_4};	/*Profile ID - 18248*/
const Value V18280_1_1 = {S2, NULL};	/*Value = True*/
const Property P18280_1 = {S50, 0, &V18280_1_1, NULL};	/*Property = IsMobile*/
const Value V18280_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18280_5 = {S52, 0, &V18280_5_1403, &P18280_1};	/*Property = ScreenPixelsHeight*/
const Value V18280_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18280_4 = {S54, 0, &V18280_4_1404, &P18280_5};	/*Property = ScreenPixelsWidth*/
const Profile P18280 = {"18280", &P18280_4};	/*Profile ID - 18280*/
const Value V18251_1_1 = {S2, NULL};	/*Value = True*/
const Property P18251_1 = {S50, 0, &V18251_1_1, NULL};	/*Property = IsMobile*/
const Value V18251_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18251_5 = {S52, 0, &V18251_5_1277, &P18251_1};	/*Property = ScreenPixelsHeight*/
const Value V18251_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18251_4 = {S54, 0, &V18251_4_1279, &P18251_5};	/*Property = ScreenPixelsWidth*/
const Profile P18251 = {"18251", &P18251_4};	/*Profile ID - 18251*/
const Value V18262_1_1 = {S2, NULL};	/*Value = True*/
const Property P18262_1 = {S50, 0, &V18262_1_1, NULL};	/*Property = IsMobile*/
const Value V18262_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18262_5 = {S52, 0, &V18262_5_1403, &P18262_1};	/*Property = ScreenPixelsHeight*/
const Value V18262_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18262_4 = {S54, 0, &V18262_4_1404, &P18262_5};	/*Property = ScreenPixelsWidth*/
const Profile P18262 = {"18262", &P18262_4};	/*Profile ID - 18262*/
const Value V22423_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22423_279 = {S1, 0, &V22423_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22423_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22423_282 = {S3, 0, &V22423_282_57153, &P22423_279};	/*Property = BlobBuilder*/
const Value V22423_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22423_350 = {S4, 0, &V22423_350_57444, &P22423_282};	/*Property = Canvas*/
const Value V22423_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22423_283 = {S5, 0, &V22423_283_57159, &P22423_350};	/*Property = CssBackground*/
const Value V22423_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22423_284 = {S6, 0, &V22423_284_57163, &P22423_283};	/*Property = CssBorderImage*/
const Value V22423_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22423_285 = {S7, 0, &V22423_285_57166, &P22423_284};	/*Property = CssCanvas*/
const Value V22423_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22423_286 = {S8, 0, &V22423_286_57168, &P22423_285};	/*Property = CssColor*/
const Value V22423_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22423_237 = {S9, 0, &V22423_237_55784, &P22423_286};	/*Property = CssColumn*/
const Value V22423_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22423_287 = {S10, 0, &V22423_287_57172, &P22423_237};	/*Property = CssFlexbox*/
const Value V22423_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22423_288 = {S11, 0, &V22423_288_57175, &P22423_287};	/*Property = CssFont*/
const Value V22423_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22423_289 = {S12, 0, &V22423_289_57759, &P22423_288};	/*Property = CssImages*/
const Value V22423_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22423_291 = {S13, 0, &V22423_291_57180, &P22423_289};	/*Property = CssMediaQueries*/
const Value V22423_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22423_292 = {S14, 0, &V22423_292_57183, &P22423_291};	/*Property = CssMinMax*/
const Value V22423_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22423_293 = {S15, 0, &V22423_293_57187, &P22423_292};	/*Property = CssOverflow*/
const Value V22423_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22423_294 = {S16, 0, &V22423_294_57189, &P22423_293};	/*Property = CssPosition*/
const Value V22423_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22423_295 = {S17, 0, &V22423_295_57192, &P22423_294};	/*Property = CssText*/
const Value V22423_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22423_297 = {S18, 0, &V22423_297_57195, &P22423_295};	/*Property = CssTransforms*/
const Value V22423_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22423_241 = {S19, 0, &V22423_241_55799, &P22423_297};	/*Property = CssTransitions*/
const Value V22423_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22423_298 = {S20, 0, &V22423_298_57206, &P22423_241};	/*Property = CssUI*/
const Value V22423_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22423_300 = {S21, 0, &V22423_300_57216, &P22423_298};	/*Property = DataSet*/
const Value V22423_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22423_301 = {S22, 0, &V22423_301_57218, &P22423_300};	/*Property = DataUrl*/
const Value V22423_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22423_302 = {S23, 0, &V22423_302_57222, &P22423_301};	/*Property = DeviceOrientation*/
const Value V22423_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22423_304 = {S24, 0, &V22423_304_57227, &P22423_302};	/*Property = FileReader*/
const Value V22423_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22423_305 = {S25, 0, &V22423_305_57231, &P22423_304};	/*Property = FileSaver*/
const Value V22423_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22423_306 = {S26, 0, &V22423_306_57237, &P22423_305};	/*Property = FileWriter*/
const Value V22423_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22423_307 = {S27, 0, &V22423_307_57239, &P22423_306};	/*Property = FormData*/
const Value V22423_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22423_309 = {S28, 0, &V22423_309_57246, &P22423_307};	/*Property = Fullscreen*/
const Value V22423_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22423_310 = {S29, 0, &V22423_310_57248, &P22423_309};	/*Property = GeoLocation*/
const Value V22423_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22423_312 = {S30, 0, &V22423_312_57254, &P22423_310};	/*Property = History*/
const Value V22423_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22423_314 = {S31, 0, &V22423_314_57261, &P22423_312};	/*Property = Html5*/
const Value V22423_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22423_313 = {S32, 0, &V22423_313_57258, &P22423_314};	/*Property = Html-Media-Capture*/
const Value V22423_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22423_315 = {S33, 0, &V22423_315_57264, &P22423_313};	/*Property = Iframe*/
const Value V22423_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22423_316 = {S34, 0, &V22423_316_57266, &P22423_315};	/*Property = IndexedDB*/
const Value V22423_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22423_317 = {S35, 0, &V22423_317_57269, &P22423_316};	/*Property = Json*/
const Value V22423_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22423_15 = {S37, 0, &V22423_15_28, &P22423_317};	/*Property = LayoutEngine*/
const Value V22423_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22423_344 = {S38, 0, &V22423_344_57272, &P22423_15};	/*Property = Masking*/
const Value V22423_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22423_320 = {S39, 0, &V22423_320_57290, &P22423_344};	/*Property = PostMessage*/
const Value V22423_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22423_321 = {S40, 0, &V22423_321_57293, &P22423_320};	/*Property = Progress*/
const Value V22423_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22423_322 = {S41, 0, &V22423_322_57296, &P22423_321};	/*Property = Prompts*/
const Value V22423_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22423_323 = {S42, 0, &V22423_323_57299, &P22423_322};	/*Property = Selector*/
const Value V22423_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22423_324 = {S43, 0, &V22423_324_57306, &P22423_323};	/*Property = Svg*/
const Value V22423_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22423_327 = {S44, 0, &V22423_327_57315, &P22423_324};	/*Property = TouchEvents*/
const Value V22423_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22423_328 = {S45, 0, &V22423_328_57318, &P22423_327};	/*Property = Track*/
const Value V22423_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22423_329 = {S46, 0, &V22423_329_57323, &P22423_328};	/*Property = Video*/
const Value V22423_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22423_330 = {S47, 0, &V22423_330_57326, &P22423_329};	/*Property = Viewport*/
const Value V22423_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22423_332 = {S48, 0, &V22423_332_57338, &P22423_330};	/*Property = WebWorkers*/
const Value V22423_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22423_333 = {S49, 0, &V22423_333_57342, &P22423_332};	/*Property = Xhr2*/
const Profile P22423 = {"22423", &P22423_333};	/*Profile ID - 22423*/
const Value V18276_1_1 = {S2, NULL};	/*Value = True*/
const Property P18276_1 = {S50, 0, &V18276_1_1, NULL};	/*Property = IsMobile*/
const Value V18276_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18276_5 = {S52, 0, &V18276_5_9981, &P18276_1};	/*Property = ScreenPixelsHeight*/
const Value V18276_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18276_4 = {S54, 0, &V18276_4_39098, &P18276_5};	/*Property = ScreenPixelsWidth*/
const Profile P18276 = {"18276", &P18276_4};	/*Profile ID - 18276*/
const Value V22321_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22321_279 = {S1, 0, &V22321_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22321_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22321_282 = {S3, 0, &V22321_282_57153, &P22321_279};	/*Property = BlobBuilder*/
const Value V22321_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22321_350 = {S4, 0, &V22321_350_57444, &P22321_282};	/*Property = Canvas*/
const Value V22321_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22321_283 = {S5, 0, &V22321_283_57159, &P22321_350};	/*Property = CssBackground*/
const Value V22321_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22321_284 = {S6, 0, &V22321_284_57163, &P22321_283};	/*Property = CssBorderImage*/
const Value V22321_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22321_285 = {S7, 0, &V22321_285_57166, &P22321_284};	/*Property = CssCanvas*/
const Value V22321_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22321_286 = {S8, 0, &V22321_286_57168, &P22321_285};	/*Property = CssColor*/
const Value V22321_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22321_237 = {S9, 0, &V22321_237_55784, &P22321_286};	/*Property = CssColumn*/
const Value V22321_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22321_287 = {S10, 0, &V22321_287_57172, &P22321_237};	/*Property = CssFlexbox*/
const Value V22321_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22321_288 = {S11, 0, &V22321_288_57175, &P22321_287};	/*Property = CssFont*/
const Value V22321_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22321_289 = {S12, 0, &V22321_289_57759, &P22321_288};	/*Property = CssImages*/
const Value V22321_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22321_291 = {S13, 0, &V22321_291_57180, &P22321_289};	/*Property = CssMediaQueries*/
const Value V22321_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22321_292 = {S14, 0, &V22321_292_57183, &P22321_291};	/*Property = CssMinMax*/
const Value V22321_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22321_293 = {S15, 0, &V22321_293_57187, &P22321_292};	/*Property = CssOverflow*/
const Value V22321_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22321_294 = {S16, 0, &V22321_294_57189, &P22321_293};	/*Property = CssPosition*/
const Value V22321_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22321_295 = {S17, 0, &V22321_295_57192, &P22321_294};	/*Property = CssText*/
const Value V22321_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22321_297 = {S18, 0, &V22321_297_57195, &P22321_295};	/*Property = CssTransforms*/
const Value V22321_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22321_241 = {S19, 0, &V22321_241_55799, &P22321_297};	/*Property = CssTransitions*/
const Value V22321_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22321_298 = {S20, 0, &V22321_298_57206, &P22321_241};	/*Property = CssUI*/
const Value V22321_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22321_300 = {S21, 0, &V22321_300_57216, &P22321_298};	/*Property = DataSet*/
const Value V22321_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22321_301 = {S22, 0, &V22321_301_57218, &P22321_300};	/*Property = DataUrl*/
const Value V22321_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22321_302 = {S23, 0, &V22321_302_57222, &P22321_301};	/*Property = DeviceOrientation*/
const Value V22321_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22321_304 = {S24, 0, &V22321_304_57227, &P22321_302};	/*Property = FileReader*/
const Value V22321_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22321_305 = {S25, 0, &V22321_305_57231, &P22321_304};	/*Property = FileSaver*/
const Value V22321_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22321_306 = {S26, 0, &V22321_306_57237, &P22321_305};	/*Property = FileWriter*/
const Value V22321_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22321_307 = {S27, 0, &V22321_307_57239, &P22321_306};	/*Property = FormData*/
const Value V22321_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22321_309 = {S28, 0, &V22321_309_57246, &P22321_307};	/*Property = Fullscreen*/
const Value V22321_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22321_310 = {S29, 0, &V22321_310_57248, &P22321_309};	/*Property = GeoLocation*/
const Value V22321_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22321_312 = {S30, 0, &V22321_312_57254, &P22321_310};	/*Property = History*/
const Value V22321_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22321_314 = {S31, 0, &V22321_314_57261, &P22321_312};	/*Property = Html5*/
const Value V22321_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22321_313 = {S32, 0, &V22321_313_57258, &P22321_314};	/*Property = Html-Media-Capture*/
const Value V22321_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22321_315 = {S33, 0, &V22321_315_57264, &P22321_313};	/*Property = Iframe*/
const Value V22321_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22321_316 = {S34, 0, &V22321_316_57266, &P22321_315};	/*Property = IndexedDB*/
const Value V22321_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22321_317 = {S35, 0, &V22321_317_57269, &P22321_316};	/*Property = Json*/
const Value V22321_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22321_15 = {S37, 0, &V22321_15_28, &P22321_317};	/*Property = LayoutEngine*/
const Value V22321_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22321_344 = {S38, 0, &V22321_344_57272, &P22321_15};	/*Property = Masking*/
const Value V22321_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22321_320 = {S39, 0, &V22321_320_57290, &P22321_344};	/*Property = PostMessage*/
const Value V22321_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22321_321 = {S40, 0, &V22321_321_57293, &P22321_320};	/*Property = Progress*/
const Value V22321_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22321_322 = {S41, 0, &V22321_322_57296, &P22321_321};	/*Property = Prompts*/
const Value V22321_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22321_323 = {S42, 0, &V22321_323_57299, &P22321_322};	/*Property = Selector*/
const Value V22321_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22321_324 = {S43, 0, &V22321_324_57306, &P22321_323};	/*Property = Svg*/
const Value V22321_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22321_327 = {S44, 0, &V22321_327_57315, &P22321_324};	/*Property = TouchEvents*/
const Value V22321_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22321_328 = {S45, 0, &V22321_328_57318, &P22321_327};	/*Property = Track*/
const Value V22321_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22321_329 = {S46, 0, &V22321_329_57323, &P22321_328};	/*Property = Video*/
const Value V22321_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22321_330 = {S47, 0, &V22321_330_57326, &P22321_329};	/*Property = Viewport*/
const Value V22321_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22321_332 = {S48, 0, &V22321_332_57338, &P22321_330};	/*Property = WebWorkers*/
const Value V22321_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22321_333 = {S49, 0, &V22321_333_57342, &P22321_332};	/*Property = Xhr2*/
const Profile P22321 = {"22321", &P22321_333};	/*Profile ID - 22321*/
const Value V22244_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22244_279 = {S1, 0, &V22244_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22244_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22244_282 = {S3, 0, &V22244_282_57154, &P22244_279};	/*Property = BlobBuilder*/
const Value V22244_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22244_350 = {S4, 0, &V22244_350_57444, &P22244_282};	/*Property = Canvas*/
const Value V22244_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22244_283 = {S5, 0, &V22244_283_57159, &P22244_350};	/*Property = CssBackground*/
const Value V22244_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22244_284 = {S6, 0, &V22244_284_57163, &P22244_283};	/*Property = CssBorderImage*/
const Value V22244_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22244_285 = {S7, 0, &V22244_285_57166, &P22244_284};	/*Property = CssCanvas*/
const Value V22244_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22244_286 = {S8, 0, &V22244_286_57168, &P22244_285};	/*Property = CssColor*/
const Value V22244_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22244_237 = {S9, 0, &V22244_237_55786, &P22244_286};	/*Property = CssColumn*/
const Value V22244_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22244_287 = {S10, 0, &V22244_287_57172, &P22244_237};	/*Property = CssFlexbox*/
const Value V22244_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22244_288 = {S11, 0, &V22244_288_57175, &P22244_287};	/*Property = CssFont*/
const Value V22244_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22244_289 = {S12, 0, &V22244_289_57759, &P22244_288};	/*Property = CssImages*/
const Value V22244_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22244_291 = {S13, 0, &V22244_291_57181, &P22244_289};	/*Property = CssMediaQueries*/
const Value V22244_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22244_292 = {S14, 0, &V22244_292_57183, &P22244_291};	/*Property = CssMinMax*/
const Value V22244_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22244_293 = {S15, 0, &V22244_293_57187, &P22244_292};	/*Property = CssOverflow*/
const Value V22244_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22244_294 = {S16, 0, &V22244_294_57190, &P22244_293};	/*Property = CssPosition*/
const Value V22244_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22244_295 = {S17, 0, &V22244_295_57192, &P22244_294};	/*Property = CssText*/
const Value V22244_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22244_297 = {S18, 0, &V22244_297_57195, &P22244_295};	/*Property = CssTransforms*/
const Value V22244_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22244_241 = {S19, 0, &V22244_241_55799, &P22244_297};	/*Property = CssTransitions*/
const Value V22244_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22244_298 = {S20, 0, &V22244_298_57206, &P22244_241};	/*Property = CssUI*/
const Value V22244_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22244_300 = {S21, 0, &V22244_300_57216, &P22244_298};	/*Property = DataSet*/
const Value V22244_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22244_301 = {S22, 0, &V22244_301_57218, &P22244_300};	/*Property = DataUrl*/
const Value V22244_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22244_302 = {S23, 0, &V22244_302_57222, &P22244_301};	/*Property = DeviceOrientation*/
const Value V22244_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22244_304 = {S24, 0, &V22244_304_57228, &P22244_302};	/*Property = FileReader*/
const Value V22244_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22244_305 = {S25, 0, &V22244_305_57231, &P22244_304};	/*Property = FileSaver*/
const Value V22244_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22244_306 = {S26, 0, &V22244_306_57237, &P22244_305};	/*Property = FileWriter*/
const Value V22244_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22244_307 = {S27, 0, &V22244_307_57240, &P22244_306};	/*Property = FormData*/
const Value V22244_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22244_309 = {S28, 0, &V22244_309_57246, &P22244_307};	/*Property = Fullscreen*/
const Value V22244_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22244_310 = {S29, 0, &V22244_310_57248, &P22244_309};	/*Property = GeoLocation*/
const Value V22244_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22244_312 = {S30, 0, &V22244_312_57255, &P22244_310};	/*Property = History*/
const Value V22244_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22244_314 = {S31, 0, &V22244_314_57261, &P22244_312};	/*Property = Html5*/
const Value V22244_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22244_313 = {S32, 0, &V22244_313_57258, &P22244_314};	/*Property = Html-Media-Capture*/
const Value V22244_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22244_315 = {S33, 0, &V22244_315_57264, &P22244_313};	/*Property = Iframe*/
const Value V22244_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22244_316 = {S34, 0, &V22244_316_57267, &P22244_315};	/*Property = IndexedDB*/
const Value V22244_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22244_317 = {S35, 0, &V22244_317_57269, &P22244_316};	/*Property = Json*/
const Value V22244_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22244_15 = {S37, 0, &V22244_15_72, &P22244_317};	/*Property = LayoutEngine*/
const Value V22244_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22244_344 = {S38, 0, &V22244_344_57272, &P22244_15};	/*Property = Masking*/
const Value V22244_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22244_320 = {S39, 0, &V22244_320_57290, &P22244_344};	/*Property = PostMessage*/
const Value V22244_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22244_321 = {S40, 0, &V22244_321_57293, &P22244_320};	/*Property = Progress*/
const Value V22244_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22244_322 = {S41, 0, &V22244_322_57296, &P22244_321};	/*Property = Prompts*/
const Value V22244_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22244_323 = {S42, 0, &V22244_323_57299, &P22244_322};	/*Property = Selector*/
const Value V22244_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22244_324 = {S43, 0, &V22244_324_57306, &P22244_323};	/*Property = Svg*/
const Value V22244_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22244_327 = {S44, 0, &V22244_327_57315, &P22244_324};	/*Property = TouchEvents*/
const Value V22244_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22244_328 = {S45, 0, &V22244_328_57318, &P22244_327};	/*Property = Track*/
const Value V22244_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22244_329 = {S46, 0, &V22244_329_57323, &P22244_328};	/*Property = Video*/
const Value V22244_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22244_330 = {S47, 0, &V22244_330_57326, &P22244_329};	/*Property = Viewport*/
const Value V22244_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22244_332 = {S48, 0, &V22244_332_57339, &P22244_330};	/*Property = WebWorkers*/
const Value V22244_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22244_333 = {S49, 0, &V22244_333_57342, &P22244_332};	/*Property = Xhr2*/
const Profile P22244 = {"22244", &P22244_333};	/*Profile ID - 22244*/
const Value V23045_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_279 = {S1, 0, &V23045_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23045_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_282 = {S3, 0, &V23045_282_57155, &P23045_279};	/*Property = BlobBuilder*/
const Value V23045_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_350 = {S4, 0, &V23045_350_57446, &P23045_282};	/*Property = Canvas*/
const Value V23045_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_283 = {S5, 0, &V23045_283_57161, &P23045_350};	/*Property = CssBackground*/
const Value V23045_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_284 = {S6, 0, &V23045_284_57164, &P23045_283};	/*Property = CssBorderImage*/
const Value V23045_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_285 = {S7, 0, &V23045_285_57167, &P23045_284};	/*Property = CssCanvas*/
const Value V23045_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_286 = {S8, 0, &V23045_286_57170, &P23045_285};	/*Property = CssColor*/
const Value V23045_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_237 = {S9, 0, &V23045_237_55786, &P23045_286};	/*Property = CssColumn*/
const Value V23045_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_287 = {S10, 0, &V23045_287_57173, &P23045_237};	/*Property = CssFlexbox*/
const Value V23045_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_288 = {S11, 0, &V23045_288_57176, &P23045_287};	/*Property = CssFont*/
const Value V23045_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_289 = {S12, 0, &V23045_289_86899, &P23045_288};	/*Property = CssImages*/
const Value V23045_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_291 = {S13, 0, &V23045_291_57182, &P23045_289};	/*Property = CssMediaQueries*/
const Value V23045_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_292 = {S14, 0, &V23045_292_57185, &P23045_291};	/*Property = CssMinMax*/
const Value V23045_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_293 = {S15, 0, &V23045_293_57188, &P23045_292};	/*Property = CssOverflow*/
const Value V23045_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_294 = {S16, 0, &V23045_294_57191, &P23045_293};	/*Property = CssPosition*/
const Value V23045_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_295 = {S17, 0, &V23045_295_57194, &P23045_294};	/*Property = CssText*/
const Value V23045_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_297 = {S18, 0, &V23045_297_57197, &P23045_295};	/*Property = CssTransforms*/
const Value V23045_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_241 = {S19, 0, &V23045_241_55801, &P23045_297};	/*Property = CssTransitions*/
const Value V23045_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_298 = {S20, 0, &V23045_298_57208, &P23045_241};	/*Property = CssUI*/
const Value V23045_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_300 = {S21, 0, &V23045_300_57217, &P23045_298};	/*Property = DataSet*/
const Value V23045_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_301 = {S22, 0, &V23045_301_57220, &P23045_300};	/*Property = DataUrl*/
const Value V23045_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_302 = {S23, 0, &V23045_302_57223, &P23045_301};	/*Property = DeviceOrientation*/
const Value V23045_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_304 = {S24, 0, &V23045_304_57229, &P23045_302};	/*Property = FileReader*/
const Value V23045_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_305 = {S25, 0, &V23045_305_57232, &P23045_304};	/*Property = FileSaver*/
const Value V23045_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_306 = {S26, 0, &V23045_306_57238, &P23045_305};	/*Property = FileWriter*/
const Value V23045_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_307 = {S27, 0, &V23045_307_57241, &P23045_306};	/*Property = FormData*/
const Value V23045_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_309 = {S28, 0, &V23045_309_57247, &P23045_307};	/*Property = Fullscreen*/
const Value V23045_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_310 = {S29, 0, &V23045_310_57250, &P23045_309};	/*Property = GeoLocation*/
const Value V23045_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_312 = {S30, 0, &V23045_312_57256, &P23045_310};	/*Property = History*/
const Value V23045_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_314 = {S31, 0, &V23045_314_57262, &P23045_312};	/*Property = Html5*/
const Value V23045_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_313 = {S32, 0, &V23045_313_57259, &P23045_314};	/*Property = Html-Media-Capture*/
const Value V23045_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_315 = {S33, 0, &V23045_315_57265, &P23045_313};	/*Property = Iframe*/
const Value V23045_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_316 = {S34, 0, &V23045_316_57268, &P23045_315};	/*Property = IndexedDB*/
const Value V23045_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_317 = {S35, 0, &V23045_317_57271, &P23045_316};	/*Property = Json*/
const Value V23045_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_15 = {S37, 0, &V23045_15_72, &P23045_317};	/*Property = LayoutEngine*/
const Value V23045_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_344 = {S38, 0, &V23045_344_57274, &P23045_15};	/*Property = Masking*/
const Value V23045_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_320 = {S39, 0, &V23045_320_57292, &P23045_344};	/*Property = PostMessage*/
const Value V23045_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_321 = {S40, 0, &V23045_321_57295, &P23045_320};	/*Property = Progress*/
const Value V23045_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_322 = {S41, 0, &V23045_322_57298, &P23045_321};	/*Property = Prompts*/
const Value V23045_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_323 = {S42, 0, &V23045_323_57301, &P23045_322};	/*Property = Selector*/
const Value V23045_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_324 = {S43, 0, &V23045_324_57307, &P23045_323};	/*Property = Svg*/
const Value V23045_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_327 = {S44, 0, &V23045_327_57316, &P23045_324};	/*Property = TouchEvents*/
const Value V23045_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_328 = {S45, 0, &V23045_328_57319, &P23045_327};	/*Property = Track*/
const Value V23045_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_329 = {S46, 0, &V23045_329_57325, &P23045_328};	/*Property = Video*/
const Value V23045_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_330 = {S47, 0, &V23045_330_57328, &P23045_329};	/*Property = Viewport*/
const Value V23045_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_332 = {S48, 0, &V23045_332_57340, &P23045_330};	/*Property = WebWorkers*/
const Value V23045_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23045_333 = {S49, 0, &V23045_333_57343, &P23045_332};	/*Property = Xhr2*/
const Profile P23045 = {"23045", &P23045_333};	/*Profile ID - 23045*/
const Value V21327_1_1 = {S2, NULL};	/*Value = True*/
const Property P21327_1 = {S50, 0, &V21327_1_1, NULL};	/*Property = IsMobile*/
const Value V21327_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P21327_5 = {S52, 0, &V21327_5_1304, &P21327_1};	/*Property = ScreenPixelsHeight*/
const Value V21327_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21327_4 = {S54, 0, &V21327_4_1224, &P21327_5};	/*Property = ScreenPixelsWidth*/
const Profile P21327 = {"21327", &P21327_4};	/*Profile ID - 21327*/
const Value V20144_1_1 = {S2, NULL};	/*Value = True*/
const Property P20144_1 = {S50, 0, &V20144_1_1, NULL};	/*Property = IsMobile*/
const Value V20144_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20144_5 = {S52, 0, &V20144_5_1222, &P20144_1};	/*Property = ScreenPixelsHeight*/
const Value V20144_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20144_4 = {S54, 0, &V20144_4_1224, &P20144_5};	/*Property = ScreenPixelsWidth*/
const Profile P20144 = {"20144", &P20144_4};	/*Profile ID - 20144*/
const Profile P23100 = {"23100", NULL};	/*Profile ID - 23100*/
const Value V21163_1_1 = {S2, NULL};	/*Value = True*/
const Property P21163_1 = {S50, 0, &V21163_1_1, NULL};	/*Property = IsMobile*/
const Value V21163_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21163_5 = {S52, 0, &V21163_5_1458, &P21163_1};	/*Property = ScreenPixelsHeight*/
const Value V21163_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21163_4 = {S54, 0, &V21163_4_1460, &P21163_5};	/*Property = ScreenPixelsWidth*/
const Profile P21163 = {"21163", &P21163_4};	/*Profile ID - 21163*/
const Profile P17838 = {"17838", NULL};	/*Profile ID - 17838*/
const Profile P22725 = {"22725", NULL};	/*Profile ID - 22725*/
const Value V20500_1_1 = {S2, NULL};	/*Value = True*/
const Property P20500_1 = {S50, 0, &V20500_1_1, NULL};	/*Property = IsMobile*/
const Value V20500_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20500_5 = {S52, 0, &V20500_5_1277, &P20500_1};	/*Property = ScreenPixelsHeight*/
const Value V20500_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20500_4 = {S54, 0, &V20500_4_1279, &P20500_5};	/*Property = ScreenPixelsWidth*/
const Profile P20500 = {"20500", &P20500_4};	/*Profile ID - 20500*/
const Value V21324_1_1 = {S2, NULL};	/*Value = True*/
const Property P21324_1 = {S50, 0, &V21324_1_1, NULL};	/*Property = IsMobile*/
const Value V21324_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21324_5 = {S52, 0, &V21324_5_1277, &P21324_1};	/*Property = ScreenPixelsHeight*/
const Value V21324_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21324_4 = {S54, 0, &V21324_4_1279, &P21324_5};	/*Property = ScreenPixelsWidth*/
const Profile P21324 = {"21324", &P21324_4};	/*Profile ID - 21324*/
const Value V21053_1_1 = {S2, NULL};	/*Value = True*/
const Property P21053_1 = {S50, 0, &V21053_1_1, NULL};	/*Property = IsMobile*/
const Value V21053_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21053_5 = {S52, 0, &V21053_5_1277, &P21053_1};	/*Property = ScreenPixelsHeight*/
const Value V21053_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21053_4 = {S54, 0, &V21053_4_1279, &P21053_5};	/*Property = ScreenPixelsWidth*/
const Profile P21053 = {"21053", &P21053_4};	/*Profile ID - 21053*/
const Value V12968_1_1 = {S2, NULL};	/*Value = True*/
const Property P12968_1 = {S50, 0, &V12968_1_1, NULL};	/*Property = IsMobile*/
const Value V12968_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P12968_5 = {S52, 0, &V12968_5_9981, &P12968_1};	/*Property = ScreenPixelsHeight*/
const Value V12968_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P12968_4 = {S54, 0, &V12968_4_39098, &P12968_5};	/*Property = ScreenPixelsWidth*/
const Profile P12968 = {"12968", &P12968_4};	/*Profile ID - 12968*/
const Value V18357_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_279 = {S1, 0, &V18357_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18357_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_282 = {S3, 0, &V18357_282_57155, &P18357_279};	/*Property = BlobBuilder*/
const Value V18357_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_350 = {S4, 0, &V18357_350_57446, &P18357_282};	/*Property = Canvas*/
const Value V18357_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_283 = {S5, 0, &V18357_283_57161, &P18357_350};	/*Property = CssBackground*/
const Value V18357_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_284 = {S6, 0, &V18357_284_57164, &P18357_283};	/*Property = CssBorderImage*/
const Value V18357_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_285 = {S7, 0, &V18357_285_57167, &P18357_284};	/*Property = CssCanvas*/
const Value V18357_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_286 = {S8, 0, &V18357_286_57170, &P18357_285};	/*Property = CssColor*/
const Value V18357_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_237 = {S9, 0, &V18357_237_55786, &P18357_286};	/*Property = CssColumn*/
const Value V18357_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_287 = {S10, 0, &V18357_287_57173, &P18357_237};	/*Property = CssFlexbox*/
const Value V18357_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_288 = {S11, 0, &V18357_288_57176, &P18357_287};	/*Property = CssFont*/
const Value V18357_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_289 = {S12, 0, &V18357_289_86899, &P18357_288};	/*Property = CssImages*/
const Value V18357_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_291 = {S13, 0, &V18357_291_57182, &P18357_289};	/*Property = CssMediaQueries*/
const Value V18357_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_292 = {S14, 0, &V18357_292_57185, &P18357_291};	/*Property = CssMinMax*/
const Value V18357_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_293 = {S15, 0, &V18357_293_57188, &P18357_292};	/*Property = CssOverflow*/
const Value V18357_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_294 = {S16, 0, &V18357_294_57191, &P18357_293};	/*Property = CssPosition*/
const Value V18357_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_295 = {S17, 0, &V18357_295_57194, &P18357_294};	/*Property = CssText*/
const Value V18357_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_297 = {S18, 0, &V18357_297_57197, &P18357_295};	/*Property = CssTransforms*/
const Value V18357_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_241 = {S19, 0, &V18357_241_55801, &P18357_297};	/*Property = CssTransitions*/
const Value V18357_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_298 = {S20, 0, &V18357_298_57208, &P18357_241};	/*Property = CssUI*/
const Value V18357_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_300 = {S21, 0, &V18357_300_57217, &P18357_298};	/*Property = DataSet*/
const Value V18357_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_301 = {S22, 0, &V18357_301_57220, &P18357_300};	/*Property = DataUrl*/
const Value V18357_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_302 = {S23, 0, &V18357_302_57223, &P18357_301};	/*Property = DeviceOrientation*/
const Value V18357_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_304 = {S24, 0, &V18357_304_57229, &P18357_302};	/*Property = FileReader*/
const Value V18357_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_305 = {S25, 0, &V18357_305_57232, &P18357_304};	/*Property = FileSaver*/
const Value V18357_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_306 = {S26, 0, &V18357_306_57238, &P18357_305};	/*Property = FileWriter*/
const Value V18357_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_307 = {S27, 0, &V18357_307_57241, &P18357_306};	/*Property = FormData*/
const Value V18357_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_309 = {S28, 0, &V18357_309_57247, &P18357_307};	/*Property = Fullscreen*/
const Value V18357_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_310 = {S29, 0, &V18357_310_57250, &P18357_309};	/*Property = GeoLocation*/
const Value V18357_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_312 = {S30, 0, &V18357_312_57256, &P18357_310};	/*Property = History*/
const Value V18357_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_314 = {S31, 0, &V18357_314_57262, &P18357_312};	/*Property = Html5*/
const Value V18357_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_313 = {S32, 0, &V18357_313_57259, &P18357_314};	/*Property = Html-Media-Capture*/
const Value V18357_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_315 = {S33, 0, &V18357_315_57265, &P18357_313};	/*Property = Iframe*/
const Value V18357_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_316 = {S34, 0, &V18357_316_57268, &P18357_315};	/*Property = IndexedDB*/
const Value V18357_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_317 = {S35, 0, &V18357_317_57271, &P18357_316};	/*Property = Json*/
const Value V18357_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18357_15 = {S37, 0, &V18357_15_29, &P18357_317};	/*Property = LayoutEngine*/
const Value V18357_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_344 = {S38, 0, &V18357_344_57274, &P18357_15};	/*Property = Masking*/
const Value V18357_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_320 = {S39, 0, &V18357_320_57292, &P18357_344};	/*Property = PostMessage*/
const Value V18357_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_321 = {S40, 0, &V18357_321_57295, &P18357_320};	/*Property = Progress*/
const Value V18357_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_322 = {S41, 0, &V18357_322_57298, &P18357_321};	/*Property = Prompts*/
const Value V18357_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_323 = {S42, 0, &V18357_323_57301, &P18357_322};	/*Property = Selector*/
const Value V18357_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_324 = {S43, 0, &V18357_324_57307, &P18357_323};	/*Property = Svg*/
const Value V18357_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_327 = {S44, 0, &V18357_327_57316, &P18357_324};	/*Property = TouchEvents*/
const Value V18357_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_328 = {S45, 0, &V18357_328_57319, &P18357_327};	/*Property = Track*/
const Value V18357_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_329 = {S46, 0, &V18357_329_57325, &P18357_328};	/*Property = Video*/
const Value V18357_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_330 = {S47, 0, &V18357_330_57328, &P18357_329};	/*Property = Viewport*/
const Value V18357_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_332 = {S48, 0, &V18357_332_57340, &P18357_330};	/*Property = WebWorkers*/
const Value V18357_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18357_333 = {S49, 0, &V18357_333_57343, &P18357_332};	/*Property = Xhr2*/
const Profile P18357 = {"18357", &P18357_333};	/*Profile ID - 18357*/
const Value V24198_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24198_279 = {S1, 0, &V24198_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24198_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24198_282 = {S3, 0, &V24198_282_57154, &P24198_279};	/*Property = BlobBuilder*/
const Value V24198_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24198_350 = {S4, 0, &V24198_350_57444, &P24198_282};	/*Property = Canvas*/
const Value V24198_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24198_283 = {S5, 0, &V24198_283_57159, &P24198_350};	/*Property = CssBackground*/
const Value V24198_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24198_284 = {S6, 0, &V24198_284_57162, &P24198_283};	/*Property = CssBorderImage*/
const Value V24198_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24198_285 = {S7, 0, &V24198_285_57166, &P24198_284};	/*Property = CssCanvas*/
const Value V24198_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24198_286 = {S8, 0, &V24198_286_57168, &P24198_285};	/*Property = CssColor*/
const Value V24198_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24198_237 = {S9, 0, &V24198_237_55786, &P24198_286};	/*Property = CssColumn*/
const Value V24198_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24198_287 = {S10, 0, &V24198_287_57172, &P24198_237};	/*Property = CssFlexbox*/
const Value V24198_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24198_288 = {S11, 0, &V24198_288_57175, &P24198_287};	/*Property = CssFont*/
const Value V24198_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24198_289 = {S12, 0, &V24198_289_57759, &P24198_288};	/*Property = CssImages*/
const Value V24198_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24198_291 = {S13, 0, &V24198_291_57180, &P24198_289};	/*Property = CssMediaQueries*/
const Value V24198_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24198_292 = {S14, 0, &V24198_292_57183, &P24198_291};	/*Property = CssMinMax*/
const Value V24198_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24198_293 = {S15, 0, &V24198_293_57187, &P24198_292};	/*Property = CssOverflow*/
const Value V24198_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24198_294 = {S16, 0, &V24198_294_57189, &P24198_293};	/*Property = CssPosition*/
const Value V24198_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24198_295 = {S17, 0, &V24198_295_57192, &P24198_294};	/*Property = CssText*/
const Value V24198_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24198_297 = {S18, 0, &V24198_297_57195, &P24198_295};	/*Property = CssTransforms*/
const Value V24198_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24198_241 = {S19, 0, &V24198_241_55799, &P24198_297};	/*Property = CssTransitions*/
const Value V24198_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24198_298 = {S20, 0, &V24198_298_57206, &P24198_241};	/*Property = CssUI*/
const Value V24198_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24198_300 = {S21, 0, &V24198_300_57215, &P24198_298};	/*Property = DataSet*/
const Value V24198_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24198_301 = {S22, 0, &V24198_301_57218, &P24198_300};	/*Property = DataUrl*/
const Value V24198_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24198_302 = {S23, 0, &V24198_302_57221, &P24198_301};	/*Property = DeviceOrientation*/
const Value V24198_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24198_304 = {S24, 0, &V24198_304_57227, &P24198_302};	/*Property = FileReader*/
const Value V24198_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24198_305 = {S25, 0, &V24198_305_57231, &P24198_304};	/*Property = FileSaver*/
const Value V24198_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24198_306 = {S26, 0, &V24198_306_57237, &P24198_305};	/*Property = FileWriter*/
const Value V24198_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24198_307 = {S27, 0, &V24198_307_57239, &P24198_306};	/*Property = FormData*/
const Value V24198_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24198_309 = {S28, 0, &V24198_309_57246, &P24198_307};	/*Property = Fullscreen*/
const Value V24198_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24198_310 = {S29, 0, &V24198_310_57248, &P24198_309};	/*Property = GeoLocation*/
const Value V24198_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24198_312 = {S30, 0, &V24198_312_57254, &P24198_310};	/*Property = History*/
const Value V24198_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24198_314 = {S31, 0, &V24198_314_57260, &P24198_312};	/*Property = Html5*/
const Value V24198_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24198_313 = {S32, 0, &V24198_313_57257, &P24198_314};	/*Property = Html-Media-Capture*/
const Value V24198_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24198_315 = {S33, 0, &V24198_315_57264, &P24198_313};	/*Property = Iframe*/
const Value V24198_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24198_316 = {S34, 0, &V24198_316_57266, &P24198_315};	/*Property = IndexedDB*/
const Value V24198_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24198_317 = {S35, 0, &V24198_317_57269, &P24198_316};	/*Property = Json*/
const Value V24198_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24198_15 = {S37, 0, &V24198_15_54167, &P24198_317};	/*Property = LayoutEngine*/
const Value V24198_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24198_344 = {S38, 0, &V24198_344_57272, &P24198_15};	/*Property = Masking*/
const Value V24198_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24198_320 = {S39, 0, &V24198_320_57290, &P24198_344};	/*Property = PostMessage*/
const Value V24198_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24198_321 = {S40, 0, &V24198_321_57293, &P24198_320};	/*Property = Progress*/
const Value V24198_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24198_322 = {S41, 0, &V24198_322_57296, &P24198_321};	/*Property = Prompts*/
const Value V24198_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24198_323 = {S42, 0, &V24198_323_57299, &P24198_322};	/*Property = Selector*/
const Value V24198_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24198_324 = {S43, 0, &V24198_324_57306, &P24198_323};	/*Property = Svg*/
const Value V24198_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24198_327 = {S44, 0, &V24198_327_57314, &P24198_324};	/*Property = TouchEvents*/
const Value V24198_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24198_328 = {S45, 0, &V24198_328_57318, &P24198_327};	/*Property = Track*/
const Value V24198_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24198_329 = {S46, 0, &V24198_329_57323, &P24198_328};	/*Property = Video*/
const Value V24198_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24198_330 = {S47, 0, &V24198_330_57326, &P24198_329};	/*Property = Viewport*/
const Value V24198_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24198_332 = {S48, 0, &V24198_332_57339, &P24198_330};	/*Property = WebWorkers*/
const Value V24198_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24198_333 = {S49, 0, &V24198_333_57342, &P24198_332};	/*Property = Xhr2*/
const Profile P24198 = {"24198", &P24198_333};	/*Profile ID - 24198*/
const Value V12521_1_1 = {S2, NULL};	/*Value = True*/
const Property P12521_1 = {S50, 0, &V12521_1_1, NULL};	/*Property = IsMobile*/
const Value V12521_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P12521_5 = {S52, 0, &V12521_5_9981, &P12521_1};	/*Property = ScreenPixelsHeight*/
const Value V12521_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P12521_4 = {S54, 0, &V12521_4_39098, &P12521_5};	/*Property = ScreenPixelsWidth*/
const Profile P12521 = {"12521", &P12521_4};	/*Profile ID - 12521*/
const Value V20782_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_279 = {S1, 0, &V20782_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20782_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_282 = {S3, 0, &V20782_282_57155, &P20782_279};	/*Property = BlobBuilder*/
const Value V20782_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_350 = {S4, 0, &V20782_350_57446, &P20782_282};	/*Property = Canvas*/
const Value V20782_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_283 = {S5, 0, &V20782_283_57161, &P20782_350};	/*Property = CssBackground*/
const Value V20782_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_284 = {S6, 0, &V20782_284_57164, &P20782_283};	/*Property = CssBorderImage*/
const Value V20782_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_285 = {S7, 0, &V20782_285_57167, &P20782_284};	/*Property = CssCanvas*/
const Value V20782_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_286 = {S8, 0, &V20782_286_57170, &P20782_285};	/*Property = CssColor*/
const Value V20782_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_237 = {S9, 0, &V20782_237_55786, &P20782_286};	/*Property = CssColumn*/
const Value V20782_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_287 = {S10, 0, &V20782_287_57173, &P20782_237};	/*Property = CssFlexbox*/
const Value V20782_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_288 = {S11, 0, &V20782_288_57176, &P20782_287};	/*Property = CssFont*/
const Value V20782_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_289 = {S12, 0, &V20782_289_86899, &P20782_288};	/*Property = CssImages*/
const Value V20782_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_291 = {S13, 0, &V20782_291_57182, &P20782_289};	/*Property = CssMediaQueries*/
const Value V20782_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_292 = {S14, 0, &V20782_292_57185, &P20782_291};	/*Property = CssMinMax*/
const Value V20782_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_293 = {S15, 0, &V20782_293_57188, &P20782_292};	/*Property = CssOverflow*/
const Value V20782_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_294 = {S16, 0, &V20782_294_57191, &P20782_293};	/*Property = CssPosition*/
const Value V20782_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_295 = {S17, 0, &V20782_295_57194, &P20782_294};	/*Property = CssText*/
const Value V20782_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_297 = {S18, 0, &V20782_297_57197, &P20782_295};	/*Property = CssTransforms*/
const Value V20782_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_241 = {S19, 0, &V20782_241_55801, &P20782_297};	/*Property = CssTransitions*/
const Value V20782_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_298 = {S20, 0, &V20782_298_57208, &P20782_241};	/*Property = CssUI*/
const Value V20782_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_300 = {S21, 0, &V20782_300_57217, &P20782_298};	/*Property = DataSet*/
const Value V20782_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_301 = {S22, 0, &V20782_301_57220, &P20782_300};	/*Property = DataUrl*/
const Value V20782_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_302 = {S23, 0, &V20782_302_57223, &P20782_301};	/*Property = DeviceOrientation*/
const Value V20782_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_304 = {S24, 0, &V20782_304_57229, &P20782_302};	/*Property = FileReader*/
const Value V20782_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_305 = {S25, 0, &V20782_305_57232, &P20782_304};	/*Property = FileSaver*/
const Value V20782_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_306 = {S26, 0, &V20782_306_57238, &P20782_305};	/*Property = FileWriter*/
const Value V20782_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_307 = {S27, 0, &V20782_307_57241, &P20782_306};	/*Property = FormData*/
const Value V20782_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_309 = {S28, 0, &V20782_309_57247, &P20782_307};	/*Property = Fullscreen*/
const Value V20782_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_310 = {S29, 0, &V20782_310_57250, &P20782_309};	/*Property = GeoLocation*/
const Value V20782_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_312 = {S30, 0, &V20782_312_57256, &P20782_310};	/*Property = History*/
const Value V20782_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_314 = {S31, 0, &V20782_314_57262, &P20782_312};	/*Property = Html5*/
const Value V20782_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_313 = {S32, 0, &V20782_313_57259, &P20782_314};	/*Property = Html-Media-Capture*/
const Value V20782_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_315 = {S33, 0, &V20782_315_57265, &P20782_313};	/*Property = Iframe*/
const Value V20782_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_316 = {S34, 0, &V20782_316_57268, &P20782_315};	/*Property = IndexedDB*/
const Value V20782_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_317 = {S35, 0, &V20782_317_57271, &P20782_316};	/*Property = Json*/
const Value V20782_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P20782_15 = {S37, 0, &V20782_15_54167, &P20782_317};	/*Property = LayoutEngine*/
const Value V20782_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_344 = {S38, 0, &V20782_344_57274, &P20782_15};	/*Property = Masking*/
const Value V20782_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_320 = {S39, 0, &V20782_320_57292, &P20782_344};	/*Property = PostMessage*/
const Value V20782_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_321 = {S40, 0, &V20782_321_57295, &P20782_320};	/*Property = Progress*/
const Value V20782_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_322 = {S41, 0, &V20782_322_57298, &P20782_321};	/*Property = Prompts*/
const Value V20782_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_323 = {S42, 0, &V20782_323_57301, &P20782_322};	/*Property = Selector*/
const Value V20782_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_324 = {S43, 0, &V20782_324_57307, &P20782_323};	/*Property = Svg*/
const Value V20782_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_327 = {S44, 0, &V20782_327_57316, &P20782_324};	/*Property = TouchEvents*/
const Value V20782_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_328 = {S45, 0, &V20782_328_57319, &P20782_327};	/*Property = Track*/
const Value V20782_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_329 = {S46, 0, &V20782_329_57325, &P20782_328};	/*Property = Video*/
const Value V20782_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_330 = {S47, 0, &V20782_330_57328, &P20782_329};	/*Property = Viewport*/
const Value V20782_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_332 = {S48, 0, &V20782_332_57340, &P20782_330};	/*Property = WebWorkers*/
const Value V20782_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20782_333 = {S49, 0, &V20782_333_57343, &P20782_332};	/*Property = Xhr2*/
const Profile P20782 = {"20782", &P20782_333};	/*Profile ID - 20782*/
const Value V18307_1_1 = {S2, NULL};	/*Value = True*/
const Property P18307_1 = {S50, 0, &V18307_1_1, NULL};	/*Property = IsMobile*/
const Value V18307_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P18307_5 = {S52, 0, &V18307_5_1832, &P18307_1};	/*Property = ScreenPixelsHeight*/
const Value V18307_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P18307_4 = {S54, 0, &V18307_4_5645, &P18307_5};	/*Property = ScreenPixelsWidth*/
const Profile P18307 = {"18307", &P18307_4};	/*Profile ID - 18307*/
const Value V18398_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_279 = {S1, 0, &V18398_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18398_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_282 = {S3, 0, &V18398_282_57155, &P18398_279};	/*Property = BlobBuilder*/
const Value V18398_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_350 = {S4, 0, &V18398_350_57446, &P18398_282};	/*Property = Canvas*/
const Value V18398_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_283 = {S5, 0, &V18398_283_57161, &P18398_350};	/*Property = CssBackground*/
const Value V18398_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_284 = {S6, 0, &V18398_284_57164, &P18398_283};	/*Property = CssBorderImage*/
const Value V18398_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_285 = {S7, 0, &V18398_285_57167, &P18398_284};	/*Property = CssCanvas*/
const Value V18398_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_286 = {S8, 0, &V18398_286_57170, &P18398_285};	/*Property = CssColor*/
const Value V18398_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_237 = {S9, 0, &V18398_237_55786, &P18398_286};	/*Property = CssColumn*/
const Value V18398_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_287 = {S10, 0, &V18398_287_57173, &P18398_237};	/*Property = CssFlexbox*/
const Value V18398_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_288 = {S11, 0, &V18398_288_57176, &P18398_287};	/*Property = CssFont*/
const Value V18398_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_289 = {S12, 0, &V18398_289_86899, &P18398_288};	/*Property = CssImages*/
const Value V18398_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_291 = {S13, 0, &V18398_291_57182, &P18398_289};	/*Property = CssMediaQueries*/
const Value V18398_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_292 = {S14, 0, &V18398_292_57185, &P18398_291};	/*Property = CssMinMax*/
const Value V18398_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_293 = {S15, 0, &V18398_293_57188, &P18398_292};	/*Property = CssOverflow*/
const Value V18398_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_294 = {S16, 0, &V18398_294_57191, &P18398_293};	/*Property = CssPosition*/
const Value V18398_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_295 = {S17, 0, &V18398_295_57194, &P18398_294};	/*Property = CssText*/
const Value V18398_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_297 = {S18, 0, &V18398_297_57197, &P18398_295};	/*Property = CssTransforms*/
const Value V18398_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_241 = {S19, 0, &V18398_241_55801, &P18398_297};	/*Property = CssTransitions*/
const Value V18398_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_298 = {S20, 0, &V18398_298_57208, &P18398_241};	/*Property = CssUI*/
const Value V18398_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_300 = {S21, 0, &V18398_300_57217, &P18398_298};	/*Property = DataSet*/
const Value V18398_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_301 = {S22, 0, &V18398_301_57220, &P18398_300};	/*Property = DataUrl*/
const Value V18398_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_302 = {S23, 0, &V18398_302_57223, &P18398_301};	/*Property = DeviceOrientation*/
const Value V18398_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_304 = {S24, 0, &V18398_304_57229, &P18398_302};	/*Property = FileReader*/
const Value V18398_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_305 = {S25, 0, &V18398_305_57232, &P18398_304};	/*Property = FileSaver*/
const Value V18398_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_306 = {S26, 0, &V18398_306_57238, &P18398_305};	/*Property = FileWriter*/
const Value V18398_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_307 = {S27, 0, &V18398_307_57241, &P18398_306};	/*Property = FormData*/
const Value V18398_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_309 = {S28, 0, &V18398_309_57247, &P18398_307};	/*Property = Fullscreen*/
const Value V18398_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_310 = {S29, 0, &V18398_310_57250, &P18398_309};	/*Property = GeoLocation*/
const Value V18398_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_312 = {S30, 0, &V18398_312_57256, &P18398_310};	/*Property = History*/
const Value V18398_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_314 = {S31, 0, &V18398_314_57262, &P18398_312};	/*Property = Html5*/
const Value V18398_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_313 = {S32, 0, &V18398_313_57259, &P18398_314};	/*Property = Html-Media-Capture*/
const Value V18398_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_315 = {S33, 0, &V18398_315_57265, &P18398_313};	/*Property = Iframe*/
const Value V18398_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_316 = {S34, 0, &V18398_316_57268, &P18398_315};	/*Property = IndexedDB*/
const Value V18398_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_317 = {S35, 0, &V18398_317_57271, &P18398_316};	/*Property = Json*/
const Value V18398_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18398_15 = {S37, 0, &V18398_15_31, &P18398_317};	/*Property = LayoutEngine*/
const Value V18398_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_344 = {S38, 0, &V18398_344_57274, &P18398_15};	/*Property = Masking*/
const Value V18398_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_320 = {S39, 0, &V18398_320_57292, &P18398_344};	/*Property = PostMessage*/
const Value V18398_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_321 = {S40, 0, &V18398_321_57295, &P18398_320};	/*Property = Progress*/
const Value V18398_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_322 = {S41, 0, &V18398_322_57298, &P18398_321};	/*Property = Prompts*/
const Value V18398_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_323 = {S42, 0, &V18398_323_57301, &P18398_322};	/*Property = Selector*/
const Value V18398_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_324 = {S43, 0, &V18398_324_57307, &P18398_323};	/*Property = Svg*/
const Value V18398_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_327 = {S44, 0, &V18398_327_57316, &P18398_324};	/*Property = TouchEvents*/
const Value V18398_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_328 = {S45, 0, &V18398_328_57319, &P18398_327};	/*Property = Track*/
const Value V18398_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_329 = {S46, 0, &V18398_329_57325, &P18398_328};	/*Property = Video*/
const Value V18398_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_330 = {S47, 0, &V18398_330_57328, &P18398_329};	/*Property = Viewport*/
const Value V18398_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_332 = {S48, 0, &V18398_332_57340, &P18398_330};	/*Property = WebWorkers*/
const Value V18398_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18398_333 = {S49, 0, &V18398_333_57343, &P18398_332};	/*Property = Xhr2*/
const Profile P18398 = {"18398", &P18398_333};	/*Profile ID - 18398*/
const Value V24429_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_279 = {S1, 0, &V24429_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24429_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_282 = {S3, 0, &V24429_282_57155, &P24429_279};	/*Property = BlobBuilder*/
const Value V24429_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_350 = {S4, 0, &V24429_350_57446, &P24429_282};	/*Property = Canvas*/
const Value V24429_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_283 = {S5, 0, &V24429_283_57161, &P24429_350};	/*Property = CssBackground*/
const Value V24429_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_284 = {S6, 0, &V24429_284_57164, &P24429_283};	/*Property = CssBorderImage*/
const Value V24429_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_285 = {S7, 0, &V24429_285_57167, &P24429_284};	/*Property = CssCanvas*/
const Value V24429_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_286 = {S8, 0, &V24429_286_57170, &P24429_285};	/*Property = CssColor*/
const Value V24429_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_237 = {S9, 0, &V24429_237_55786, &P24429_286};	/*Property = CssColumn*/
const Value V24429_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_287 = {S10, 0, &V24429_287_57173, &P24429_237};	/*Property = CssFlexbox*/
const Value V24429_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_288 = {S11, 0, &V24429_288_57176, &P24429_287};	/*Property = CssFont*/
const Value V24429_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_289 = {S12, 0, &V24429_289_86899, &P24429_288};	/*Property = CssImages*/
const Value V24429_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_291 = {S13, 0, &V24429_291_57182, &P24429_289};	/*Property = CssMediaQueries*/
const Value V24429_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_292 = {S14, 0, &V24429_292_57185, &P24429_291};	/*Property = CssMinMax*/
const Value V24429_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_293 = {S15, 0, &V24429_293_57188, &P24429_292};	/*Property = CssOverflow*/
const Value V24429_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_294 = {S16, 0, &V24429_294_57191, &P24429_293};	/*Property = CssPosition*/
const Value V24429_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_295 = {S17, 0, &V24429_295_57194, &P24429_294};	/*Property = CssText*/
const Value V24429_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_297 = {S18, 0, &V24429_297_57197, &P24429_295};	/*Property = CssTransforms*/
const Value V24429_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_241 = {S19, 0, &V24429_241_55801, &P24429_297};	/*Property = CssTransitions*/
const Value V24429_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_298 = {S20, 0, &V24429_298_57208, &P24429_241};	/*Property = CssUI*/
const Value V24429_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_300 = {S21, 0, &V24429_300_57217, &P24429_298};	/*Property = DataSet*/
const Value V24429_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_301 = {S22, 0, &V24429_301_57220, &P24429_300};	/*Property = DataUrl*/
const Value V24429_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_302 = {S23, 0, &V24429_302_57223, &P24429_301};	/*Property = DeviceOrientation*/
const Value V24429_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_304 = {S24, 0, &V24429_304_57229, &P24429_302};	/*Property = FileReader*/
const Value V24429_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_305 = {S25, 0, &V24429_305_57232, &P24429_304};	/*Property = FileSaver*/
const Value V24429_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_306 = {S26, 0, &V24429_306_57238, &P24429_305};	/*Property = FileWriter*/
const Value V24429_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_307 = {S27, 0, &V24429_307_57241, &P24429_306};	/*Property = FormData*/
const Value V24429_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_309 = {S28, 0, &V24429_309_57247, &P24429_307};	/*Property = Fullscreen*/
const Value V24429_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_310 = {S29, 0, &V24429_310_57250, &P24429_309};	/*Property = GeoLocation*/
const Value V24429_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_312 = {S30, 0, &V24429_312_57256, &P24429_310};	/*Property = History*/
const Value V24429_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_314 = {S31, 0, &V24429_314_57262, &P24429_312};	/*Property = Html5*/
const Value V24429_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_313 = {S32, 0, &V24429_313_57259, &P24429_314};	/*Property = Html-Media-Capture*/
const Value V24429_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_315 = {S33, 0, &V24429_315_57265, &P24429_313};	/*Property = Iframe*/
const Value V24429_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_316 = {S34, 0, &V24429_316_57268, &P24429_315};	/*Property = IndexedDB*/
const Value V24429_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_317 = {S35, 0, &V24429_317_57271, &P24429_316};	/*Property = Json*/
const Value V24429_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24429_15 = {S37, 0, &V24429_15_28, &P24429_317};	/*Property = LayoutEngine*/
const Value V24429_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_344 = {S38, 0, &V24429_344_57274, &P24429_15};	/*Property = Masking*/
const Value V24429_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_320 = {S39, 0, &V24429_320_57292, &P24429_344};	/*Property = PostMessage*/
const Value V24429_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_321 = {S40, 0, &V24429_321_57295, &P24429_320};	/*Property = Progress*/
const Value V24429_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_322 = {S41, 0, &V24429_322_57298, &P24429_321};	/*Property = Prompts*/
const Value V24429_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_323 = {S42, 0, &V24429_323_57301, &P24429_322};	/*Property = Selector*/
const Value V24429_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_324 = {S43, 0, &V24429_324_57307, &P24429_323};	/*Property = Svg*/
const Value V24429_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_327 = {S44, 0, &V24429_327_57316, &P24429_324};	/*Property = TouchEvents*/
const Value V24429_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_328 = {S45, 0, &V24429_328_57319, &P24429_327};	/*Property = Track*/
const Value V24429_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_329 = {S46, 0, &V24429_329_57325, &P24429_328};	/*Property = Video*/
const Value V24429_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_330 = {S47, 0, &V24429_330_57328, &P24429_329};	/*Property = Viewport*/
const Value V24429_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_332 = {S48, 0, &V24429_332_57340, &P24429_330};	/*Property = WebWorkers*/
const Value V24429_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24429_333 = {S49, 0, &V24429_333_57343, &P24429_332};	/*Property = Xhr2*/
const Profile P24429 = {"24429", &P24429_333};	/*Profile ID - 24429*/
const Value V24430_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_279 = {S1, 0, &V24430_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24430_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_282 = {S3, 0, &V24430_282_57155, &P24430_279};	/*Property = BlobBuilder*/
const Value V24430_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_350 = {S4, 0, &V24430_350_57446, &P24430_282};	/*Property = Canvas*/
const Value V24430_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_283 = {S5, 0, &V24430_283_57161, &P24430_350};	/*Property = CssBackground*/
const Value V24430_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_284 = {S6, 0, &V24430_284_57164, &P24430_283};	/*Property = CssBorderImage*/
const Value V24430_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_285 = {S7, 0, &V24430_285_57167, &P24430_284};	/*Property = CssCanvas*/
const Value V24430_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_286 = {S8, 0, &V24430_286_57170, &P24430_285};	/*Property = CssColor*/
const Value V24430_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_237 = {S9, 0, &V24430_237_55786, &P24430_286};	/*Property = CssColumn*/
const Value V24430_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_287 = {S10, 0, &V24430_287_57173, &P24430_237};	/*Property = CssFlexbox*/
const Value V24430_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_288 = {S11, 0, &V24430_288_57176, &P24430_287};	/*Property = CssFont*/
const Value V24430_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_289 = {S12, 0, &V24430_289_86899, &P24430_288};	/*Property = CssImages*/
const Value V24430_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_291 = {S13, 0, &V24430_291_57182, &P24430_289};	/*Property = CssMediaQueries*/
const Value V24430_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_292 = {S14, 0, &V24430_292_57185, &P24430_291};	/*Property = CssMinMax*/
const Value V24430_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_293 = {S15, 0, &V24430_293_57188, &P24430_292};	/*Property = CssOverflow*/
const Value V24430_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_294 = {S16, 0, &V24430_294_57191, &P24430_293};	/*Property = CssPosition*/
const Value V24430_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_295 = {S17, 0, &V24430_295_57194, &P24430_294};	/*Property = CssText*/
const Value V24430_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_297 = {S18, 0, &V24430_297_57197, &P24430_295};	/*Property = CssTransforms*/
const Value V24430_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_241 = {S19, 0, &V24430_241_55801, &P24430_297};	/*Property = CssTransitions*/
const Value V24430_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_298 = {S20, 0, &V24430_298_57208, &P24430_241};	/*Property = CssUI*/
const Value V24430_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_300 = {S21, 0, &V24430_300_57217, &P24430_298};	/*Property = DataSet*/
const Value V24430_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_301 = {S22, 0, &V24430_301_57220, &P24430_300};	/*Property = DataUrl*/
const Value V24430_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_302 = {S23, 0, &V24430_302_57223, &P24430_301};	/*Property = DeviceOrientation*/
const Value V24430_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_304 = {S24, 0, &V24430_304_57229, &P24430_302};	/*Property = FileReader*/
const Value V24430_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_305 = {S25, 0, &V24430_305_57232, &P24430_304};	/*Property = FileSaver*/
const Value V24430_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_306 = {S26, 0, &V24430_306_57238, &P24430_305};	/*Property = FileWriter*/
const Value V24430_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_307 = {S27, 0, &V24430_307_57241, &P24430_306};	/*Property = FormData*/
const Value V24430_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_309 = {S28, 0, &V24430_309_57247, &P24430_307};	/*Property = Fullscreen*/
const Value V24430_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_310 = {S29, 0, &V24430_310_57250, &P24430_309};	/*Property = GeoLocation*/
const Value V24430_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_312 = {S30, 0, &V24430_312_57256, &P24430_310};	/*Property = History*/
const Value V24430_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_314 = {S31, 0, &V24430_314_57262, &P24430_312};	/*Property = Html5*/
const Value V24430_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_313 = {S32, 0, &V24430_313_57259, &P24430_314};	/*Property = Html-Media-Capture*/
const Value V24430_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_315 = {S33, 0, &V24430_315_57265, &P24430_313};	/*Property = Iframe*/
const Value V24430_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_316 = {S34, 0, &V24430_316_57268, &P24430_315};	/*Property = IndexedDB*/
const Value V24430_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_317 = {S35, 0, &V24430_317_57271, &P24430_316};	/*Property = Json*/
const Value V24430_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24430_15 = {S37, 0, &V24430_15_28, &P24430_317};	/*Property = LayoutEngine*/
const Value V24430_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_344 = {S38, 0, &V24430_344_57274, &P24430_15};	/*Property = Masking*/
const Value V24430_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_320 = {S39, 0, &V24430_320_57292, &P24430_344};	/*Property = PostMessage*/
const Value V24430_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_321 = {S40, 0, &V24430_321_57295, &P24430_320};	/*Property = Progress*/
const Value V24430_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_322 = {S41, 0, &V24430_322_57298, &P24430_321};	/*Property = Prompts*/
const Value V24430_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_323 = {S42, 0, &V24430_323_57301, &P24430_322};	/*Property = Selector*/
const Value V24430_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_324 = {S43, 0, &V24430_324_57307, &P24430_323};	/*Property = Svg*/
const Value V24430_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_327 = {S44, 0, &V24430_327_57316, &P24430_324};	/*Property = TouchEvents*/
const Value V24430_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_328 = {S45, 0, &V24430_328_57319, &P24430_327};	/*Property = Track*/
const Value V24430_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_329 = {S46, 0, &V24430_329_57325, &P24430_328};	/*Property = Video*/
const Value V24430_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_330 = {S47, 0, &V24430_330_57328, &P24430_329};	/*Property = Viewport*/
const Value V24430_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_332 = {S48, 0, &V24430_332_57340, &P24430_330};	/*Property = WebWorkers*/
const Value V24430_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24430_333 = {S49, 0, &V24430_333_57343, &P24430_332};	/*Property = Xhr2*/
const Profile P24430 = {"24430", &P24430_333};	/*Profile ID - 24430*/
const Value V18255_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18255_279 = {S1, 0, &V18255_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18255_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18255_282 = {S3, 0, &V18255_282_57154, &P18255_279};	/*Property = BlobBuilder*/
const Value V18255_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18255_350 = {S4, 0, &V18255_350_57444, &P18255_282};	/*Property = Canvas*/
const Value V18255_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18255_283 = {S5, 0, &V18255_283_57159, &P18255_350};	/*Property = CssBackground*/
const Value V18255_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18255_284 = {S6, 0, &V18255_284_57163, &P18255_283};	/*Property = CssBorderImage*/
const Value V18255_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18255_285 = {S7, 0, &V18255_285_57166, &P18255_284};	/*Property = CssCanvas*/
const Value V18255_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18255_286 = {S8, 0, &V18255_286_57168, &P18255_285};	/*Property = CssColor*/
const Value V18255_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18255_237 = {S9, 0, &V18255_237_55786, &P18255_286};	/*Property = CssColumn*/
const Value V18255_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18255_287 = {S10, 0, &V18255_287_57172, &P18255_237};	/*Property = CssFlexbox*/
const Value V18255_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18255_288 = {S11, 0, &V18255_288_57175, &P18255_287};	/*Property = CssFont*/
const Value V18255_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18255_289 = {S12, 0, &V18255_289_57759, &P18255_288};	/*Property = CssImages*/
const Value V18255_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18255_291 = {S13, 0, &V18255_291_57181, &P18255_289};	/*Property = CssMediaQueries*/
const Value V18255_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18255_292 = {S14, 0, &V18255_292_57183, &P18255_291};	/*Property = CssMinMax*/
const Value V18255_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18255_293 = {S15, 0, &V18255_293_57187, &P18255_292};	/*Property = CssOverflow*/
const Value V18255_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18255_294 = {S16, 0, &V18255_294_57190, &P18255_293};	/*Property = CssPosition*/
const Value V18255_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18255_295 = {S17, 0, &V18255_295_57192, &P18255_294};	/*Property = CssText*/
const Value V18255_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18255_297 = {S18, 0, &V18255_297_57195, &P18255_295};	/*Property = CssTransforms*/
const Value V18255_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18255_241 = {S19, 0, &V18255_241_55799, &P18255_297};	/*Property = CssTransitions*/
const Value V18255_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18255_298 = {S20, 0, &V18255_298_57206, &P18255_241};	/*Property = CssUI*/
const Value V18255_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18255_300 = {S21, 0, &V18255_300_57216, &P18255_298};	/*Property = DataSet*/
const Value V18255_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18255_301 = {S22, 0, &V18255_301_57218, &P18255_300};	/*Property = DataUrl*/
const Value V18255_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18255_302 = {S23, 0, &V18255_302_57222, &P18255_301};	/*Property = DeviceOrientation*/
const Value V18255_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18255_304 = {S24, 0, &V18255_304_57227, &P18255_302};	/*Property = FileReader*/
const Value V18255_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18255_305 = {S25, 0, &V18255_305_57231, &P18255_304};	/*Property = FileSaver*/
const Value V18255_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18255_306 = {S26, 0, &V18255_306_57237, &P18255_305};	/*Property = FileWriter*/
const Value V18255_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18255_307 = {S27, 0, &V18255_307_57239, &P18255_306};	/*Property = FormData*/
const Value V18255_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18255_309 = {S28, 0, &V18255_309_57246, &P18255_307};	/*Property = Fullscreen*/
const Value V18255_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18255_310 = {S29, 0, &V18255_310_57248, &P18255_309};	/*Property = GeoLocation*/
const Value V18255_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18255_312 = {S30, 0, &V18255_312_57255, &P18255_310};	/*Property = History*/
const Value V18255_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18255_314 = {S31, 0, &V18255_314_57261, &P18255_312};	/*Property = Html5*/
const Value V18255_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18255_313 = {S32, 0, &V18255_313_57258, &P18255_314};	/*Property = Html-Media-Capture*/
const Value V18255_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18255_315 = {S33, 0, &V18255_315_57264, &P18255_313};	/*Property = Iframe*/
const Value V18255_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18255_316 = {S34, 0, &V18255_316_57267, &P18255_315};	/*Property = IndexedDB*/
const Value V18255_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18255_317 = {S35, 0, &V18255_317_57269, &P18255_316};	/*Property = Json*/
const Value V18255_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18255_15 = {S37, 0, &V18255_15_31, &P18255_317};	/*Property = LayoutEngine*/
const Value V18255_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18255_344 = {S38, 0, &V18255_344_57272, &P18255_15};	/*Property = Masking*/
const Value V18255_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18255_320 = {S39, 0, &V18255_320_57290, &P18255_344};	/*Property = PostMessage*/
const Value V18255_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18255_321 = {S40, 0, &V18255_321_57293, &P18255_320};	/*Property = Progress*/
const Value V18255_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18255_322 = {S41, 0, &V18255_322_57296, &P18255_321};	/*Property = Prompts*/
const Value V18255_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18255_323 = {S42, 0, &V18255_323_57299, &P18255_322};	/*Property = Selector*/
const Value V18255_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18255_324 = {S43, 0, &V18255_324_57306, &P18255_323};	/*Property = Svg*/
const Value V18255_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18255_327 = {S44, 0, &V18255_327_57315, &P18255_324};	/*Property = TouchEvents*/
const Value V18255_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18255_328 = {S45, 0, &V18255_328_57318, &P18255_327};	/*Property = Track*/
const Value V18255_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18255_329 = {S46, 0, &V18255_329_57323, &P18255_328};	/*Property = Video*/
const Value V18255_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18255_330 = {S47, 0, &V18255_330_57326, &P18255_329};	/*Property = Viewport*/
const Value V18255_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18255_332 = {S48, 0, &V18255_332_57339, &P18255_330};	/*Property = WebWorkers*/
const Value V18255_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18255_333 = {S49, 0, &V18255_333_57342, &P18255_332};	/*Property = Xhr2*/
const Profile P18255 = {"18255", &P18255_333};	/*Profile ID - 18255*/
const Value V20773_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_279 = {S1, 0, &V20773_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20773_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_282 = {S3, 0, &V20773_282_57155, &P20773_279};	/*Property = BlobBuilder*/
const Value V20773_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_350 = {S4, 0, &V20773_350_57446, &P20773_282};	/*Property = Canvas*/
const Value V20773_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_283 = {S5, 0, &V20773_283_57161, &P20773_350};	/*Property = CssBackground*/
const Value V20773_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_284 = {S6, 0, &V20773_284_57164, &P20773_283};	/*Property = CssBorderImage*/
const Value V20773_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_285 = {S7, 0, &V20773_285_57167, &P20773_284};	/*Property = CssCanvas*/
const Value V20773_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_286 = {S8, 0, &V20773_286_57170, &P20773_285};	/*Property = CssColor*/
const Value V20773_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_237 = {S9, 0, &V20773_237_55786, &P20773_286};	/*Property = CssColumn*/
const Value V20773_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_287 = {S10, 0, &V20773_287_57173, &P20773_237};	/*Property = CssFlexbox*/
const Value V20773_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_288 = {S11, 0, &V20773_288_57176, &P20773_287};	/*Property = CssFont*/
const Value V20773_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_289 = {S12, 0, &V20773_289_86899, &P20773_288};	/*Property = CssImages*/
const Value V20773_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_291 = {S13, 0, &V20773_291_57182, &P20773_289};	/*Property = CssMediaQueries*/
const Value V20773_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_292 = {S14, 0, &V20773_292_57185, &P20773_291};	/*Property = CssMinMax*/
const Value V20773_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_293 = {S15, 0, &V20773_293_57188, &P20773_292};	/*Property = CssOverflow*/
const Value V20773_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_294 = {S16, 0, &V20773_294_57191, &P20773_293};	/*Property = CssPosition*/
const Value V20773_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_295 = {S17, 0, &V20773_295_57194, &P20773_294};	/*Property = CssText*/
const Value V20773_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_297 = {S18, 0, &V20773_297_57197, &P20773_295};	/*Property = CssTransforms*/
const Value V20773_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_241 = {S19, 0, &V20773_241_55801, &P20773_297};	/*Property = CssTransitions*/
const Value V20773_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_298 = {S20, 0, &V20773_298_57208, &P20773_241};	/*Property = CssUI*/
const Value V20773_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_300 = {S21, 0, &V20773_300_57217, &P20773_298};	/*Property = DataSet*/
const Value V20773_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_301 = {S22, 0, &V20773_301_57220, &P20773_300};	/*Property = DataUrl*/
const Value V20773_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_302 = {S23, 0, &V20773_302_57223, &P20773_301};	/*Property = DeviceOrientation*/
const Value V20773_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_304 = {S24, 0, &V20773_304_57229, &P20773_302};	/*Property = FileReader*/
const Value V20773_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_305 = {S25, 0, &V20773_305_57232, &P20773_304};	/*Property = FileSaver*/
const Value V20773_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_306 = {S26, 0, &V20773_306_57238, &P20773_305};	/*Property = FileWriter*/
const Value V20773_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_307 = {S27, 0, &V20773_307_57241, &P20773_306};	/*Property = FormData*/
const Value V20773_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_309 = {S28, 0, &V20773_309_57247, &P20773_307};	/*Property = Fullscreen*/
const Value V20773_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_310 = {S29, 0, &V20773_310_57250, &P20773_309};	/*Property = GeoLocation*/
const Value V20773_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_312 = {S30, 0, &V20773_312_57256, &P20773_310};	/*Property = History*/
const Value V20773_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_314 = {S31, 0, &V20773_314_57262, &P20773_312};	/*Property = Html5*/
const Value V20773_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_313 = {S32, 0, &V20773_313_57259, &P20773_314};	/*Property = Html-Media-Capture*/
const Value V20773_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_315 = {S33, 0, &V20773_315_57265, &P20773_313};	/*Property = Iframe*/
const Value V20773_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_316 = {S34, 0, &V20773_316_57268, &P20773_315};	/*Property = IndexedDB*/
const Value V20773_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_317 = {S35, 0, &V20773_317_57271, &P20773_316};	/*Property = Json*/
const Value V20773_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_15 = {S37, 0, &V20773_15_72, &P20773_317};	/*Property = LayoutEngine*/
const Value V20773_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_344 = {S38, 0, &V20773_344_57274, &P20773_15};	/*Property = Masking*/
const Value V20773_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_320 = {S39, 0, &V20773_320_57292, &P20773_344};	/*Property = PostMessage*/
const Value V20773_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_321 = {S40, 0, &V20773_321_57295, &P20773_320};	/*Property = Progress*/
const Value V20773_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_322 = {S41, 0, &V20773_322_57298, &P20773_321};	/*Property = Prompts*/
const Value V20773_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_323 = {S42, 0, &V20773_323_57301, &P20773_322};	/*Property = Selector*/
const Value V20773_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_324 = {S43, 0, &V20773_324_57307, &P20773_323};	/*Property = Svg*/
const Value V20773_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_327 = {S44, 0, &V20773_327_57316, &P20773_324};	/*Property = TouchEvents*/
const Value V20773_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_328 = {S45, 0, &V20773_328_57319, &P20773_327};	/*Property = Track*/
const Value V20773_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_329 = {S46, 0, &V20773_329_57325, &P20773_328};	/*Property = Video*/
const Value V20773_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_330 = {S47, 0, &V20773_330_57328, &P20773_329};	/*Property = Viewport*/
const Value V20773_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_332 = {S48, 0, &V20773_332_57340, &P20773_330};	/*Property = WebWorkers*/
const Value V20773_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20773_333 = {S49, 0, &V20773_333_57343, &P20773_332};	/*Property = Xhr2*/
const Profile P20773 = {"20773", &P20773_333};	/*Profile ID - 20773*/
const Value V22424_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22424_279 = {S1, 0, &V22424_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22424_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22424_282 = {S3, 0, &V22424_282_57153, &P22424_279};	/*Property = BlobBuilder*/
const Value V22424_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22424_350 = {S4, 0, &V22424_350_57444, &P22424_282};	/*Property = Canvas*/
const Value V22424_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22424_283 = {S5, 0, &V22424_283_57159, &P22424_350};	/*Property = CssBackground*/
const Value V22424_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22424_284 = {S6, 0, &V22424_284_57163, &P22424_283};	/*Property = CssBorderImage*/
const Value V22424_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22424_285 = {S7, 0, &V22424_285_57166, &P22424_284};	/*Property = CssCanvas*/
const Value V22424_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22424_286 = {S8, 0, &V22424_286_57168, &P22424_285};	/*Property = CssColor*/
const Value V22424_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22424_237 = {S9, 0, &V22424_237_55784, &P22424_286};	/*Property = CssColumn*/
const Value V22424_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22424_287 = {S10, 0, &V22424_287_57172, &P22424_237};	/*Property = CssFlexbox*/
const Value V22424_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22424_288 = {S11, 0, &V22424_288_57175, &P22424_287};	/*Property = CssFont*/
const Value V22424_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22424_289 = {S12, 0, &V22424_289_57759, &P22424_288};	/*Property = CssImages*/
const Value V22424_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22424_291 = {S13, 0, &V22424_291_57180, &P22424_289};	/*Property = CssMediaQueries*/
const Value V22424_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22424_292 = {S14, 0, &V22424_292_57183, &P22424_291};	/*Property = CssMinMax*/
const Value V22424_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22424_293 = {S15, 0, &V22424_293_57187, &P22424_292};	/*Property = CssOverflow*/
const Value V22424_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22424_294 = {S16, 0, &V22424_294_57189, &P22424_293};	/*Property = CssPosition*/
const Value V22424_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22424_295 = {S17, 0, &V22424_295_57192, &P22424_294};	/*Property = CssText*/
const Value V22424_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22424_297 = {S18, 0, &V22424_297_57195, &P22424_295};	/*Property = CssTransforms*/
const Value V22424_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22424_241 = {S19, 0, &V22424_241_55799, &P22424_297};	/*Property = CssTransitions*/
const Value V22424_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22424_298 = {S20, 0, &V22424_298_57206, &P22424_241};	/*Property = CssUI*/
const Value V22424_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22424_300 = {S21, 0, &V22424_300_57216, &P22424_298};	/*Property = DataSet*/
const Value V22424_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22424_301 = {S22, 0, &V22424_301_57218, &P22424_300};	/*Property = DataUrl*/
const Value V22424_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22424_302 = {S23, 0, &V22424_302_57222, &P22424_301};	/*Property = DeviceOrientation*/
const Value V22424_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22424_304 = {S24, 0, &V22424_304_57227, &P22424_302};	/*Property = FileReader*/
const Value V22424_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22424_305 = {S25, 0, &V22424_305_57231, &P22424_304};	/*Property = FileSaver*/
const Value V22424_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22424_306 = {S26, 0, &V22424_306_57237, &P22424_305};	/*Property = FileWriter*/
const Value V22424_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22424_307 = {S27, 0, &V22424_307_57239, &P22424_306};	/*Property = FormData*/
const Value V22424_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22424_309 = {S28, 0, &V22424_309_57246, &P22424_307};	/*Property = Fullscreen*/
const Value V22424_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22424_310 = {S29, 0, &V22424_310_57248, &P22424_309};	/*Property = GeoLocation*/
const Value V22424_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22424_312 = {S30, 0, &V22424_312_57254, &P22424_310};	/*Property = History*/
const Value V22424_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22424_314 = {S31, 0, &V22424_314_57261, &P22424_312};	/*Property = Html5*/
const Value V22424_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22424_313 = {S32, 0, &V22424_313_57258, &P22424_314};	/*Property = Html-Media-Capture*/
const Value V22424_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22424_315 = {S33, 0, &V22424_315_57264, &P22424_313};	/*Property = Iframe*/
const Value V22424_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22424_316 = {S34, 0, &V22424_316_57266, &P22424_315};	/*Property = IndexedDB*/
const Value V22424_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22424_317 = {S35, 0, &V22424_317_57269, &P22424_316};	/*Property = Json*/
const Value V22424_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22424_15 = {S37, 0, &V22424_15_28, &P22424_317};	/*Property = LayoutEngine*/
const Value V22424_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22424_344 = {S38, 0, &V22424_344_57272, &P22424_15};	/*Property = Masking*/
const Value V22424_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22424_320 = {S39, 0, &V22424_320_57290, &P22424_344};	/*Property = PostMessage*/
const Value V22424_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22424_321 = {S40, 0, &V22424_321_57293, &P22424_320};	/*Property = Progress*/
const Value V22424_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22424_322 = {S41, 0, &V22424_322_57296, &P22424_321};	/*Property = Prompts*/
const Value V22424_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22424_323 = {S42, 0, &V22424_323_57299, &P22424_322};	/*Property = Selector*/
const Value V22424_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22424_324 = {S43, 0, &V22424_324_57306, &P22424_323};	/*Property = Svg*/
const Value V22424_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22424_327 = {S44, 0, &V22424_327_57315, &P22424_324};	/*Property = TouchEvents*/
const Value V22424_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22424_328 = {S45, 0, &V22424_328_57318, &P22424_327};	/*Property = Track*/
const Value V22424_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22424_329 = {S46, 0, &V22424_329_57323, &P22424_328};	/*Property = Video*/
const Value V22424_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22424_330 = {S47, 0, &V22424_330_57326, &P22424_329};	/*Property = Viewport*/
const Value V22424_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22424_332 = {S48, 0, &V22424_332_57338, &P22424_330};	/*Property = WebWorkers*/
const Value V22424_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22424_333 = {S49, 0, &V22424_333_57342, &P22424_332};	/*Property = Xhr2*/
const Profile P22424 = {"22424", &P22424_333};	/*Profile ID - 22424*/
const Value V15830_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_279 = {S1, 0, &V15830_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V15830_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_282 = {S3, 0, &V15830_282_57155, &P15830_279};	/*Property = BlobBuilder*/
const Value V15830_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_350 = {S4, 0, &V15830_350_57446, &P15830_282};	/*Property = Canvas*/
const Value V15830_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_283 = {S5, 0, &V15830_283_57161, &P15830_350};	/*Property = CssBackground*/
const Value V15830_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_284 = {S6, 0, &V15830_284_57164, &P15830_283};	/*Property = CssBorderImage*/
const Value V15830_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_285 = {S7, 0, &V15830_285_57167, &P15830_284};	/*Property = CssCanvas*/
const Value V15830_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_286 = {S8, 0, &V15830_286_57170, &P15830_285};	/*Property = CssColor*/
const Value V15830_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_237 = {S9, 0, &V15830_237_55786, &P15830_286};	/*Property = CssColumn*/
const Value V15830_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_287 = {S10, 0, &V15830_287_57173, &P15830_237};	/*Property = CssFlexbox*/
const Value V15830_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_288 = {S11, 0, &V15830_288_57176, &P15830_287};	/*Property = CssFont*/
const Value V15830_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_289 = {S12, 0, &V15830_289_86899, &P15830_288};	/*Property = CssImages*/
const Value V15830_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_291 = {S13, 0, &V15830_291_57182, &P15830_289};	/*Property = CssMediaQueries*/
const Value V15830_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_292 = {S14, 0, &V15830_292_57185, &P15830_291};	/*Property = CssMinMax*/
const Value V15830_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_293 = {S15, 0, &V15830_293_57188, &P15830_292};	/*Property = CssOverflow*/
const Value V15830_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_294 = {S16, 0, &V15830_294_57191, &P15830_293};	/*Property = CssPosition*/
const Value V15830_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_295 = {S17, 0, &V15830_295_57194, &P15830_294};	/*Property = CssText*/
const Value V15830_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_297 = {S18, 0, &V15830_297_57197, &P15830_295};	/*Property = CssTransforms*/
const Value V15830_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_241 = {S19, 0, &V15830_241_55801, &P15830_297};	/*Property = CssTransitions*/
const Value V15830_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_298 = {S20, 0, &V15830_298_57208, &P15830_241};	/*Property = CssUI*/
const Value V15830_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_300 = {S21, 0, &V15830_300_57217, &P15830_298};	/*Property = DataSet*/
const Value V15830_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_301 = {S22, 0, &V15830_301_57220, &P15830_300};	/*Property = DataUrl*/
const Value V15830_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_302 = {S23, 0, &V15830_302_57223, &P15830_301};	/*Property = DeviceOrientation*/
const Value V15830_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_304 = {S24, 0, &V15830_304_57229, &P15830_302};	/*Property = FileReader*/
const Value V15830_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_305 = {S25, 0, &V15830_305_57232, &P15830_304};	/*Property = FileSaver*/
const Value V15830_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_306 = {S26, 0, &V15830_306_57238, &P15830_305};	/*Property = FileWriter*/
const Value V15830_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_307 = {S27, 0, &V15830_307_57241, &P15830_306};	/*Property = FormData*/
const Value V15830_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_309 = {S28, 0, &V15830_309_57247, &P15830_307};	/*Property = Fullscreen*/
const Value V15830_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_310 = {S29, 0, &V15830_310_57250, &P15830_309};	/*Property = GeoLocation*/
const Value V15830_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_312 = {S30, 0, &V15830_312_57256, &P15830_310};	/*Property = History*/
const Value V15830_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_314 = {S31, 0, &V15830_314_57262, &P15830_312};	/*Property = Html5*/
const Value V15830_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_313 = {S32, 0, &V15830_313_57259, &P15830_314};	/*Property = Html-Media-Capture*/
const Value V15830_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_315 = {S33, 0, &V15830_315_57265, &P15830_313};	/*Property = Iframe*/
const Value V15830_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_316 = {S34, 0, &V15830_316_57268, &P15830_315};	/*Property = IndexedDB*/
const Value V15830_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_317 = {S35, 0, &V15830_317_57271, &P15830_316};	/*Property = Json*/
const Value V15830_15_43264 = {S59, NULL};	/*Value = Mango*/
const Property P15830_15 = {S37, 0, &V15830_15_43264, &P15830_317};	/*Property = LayoutEngine*/
const Value V15830_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_344 = {S38, 0, &V15830_344_57274, &P15830_15};	/*Property = Masking*/
const Value V15830_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_320 = {S39, 0, &V15830_320_57292, &P15830_344};	/*Property = PostMessage*/
const Value V15830_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_321 = {S40, 0, &V15830_321_57295, &P15830_320};	/*Property = Progress*/
const Value V15830_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_322 = {S41, 0, &V15830_322_57298, &P15830_321};	/*Property = Prompts*/
const Value V15830_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_323 = {S42, 0, &V15830_323_57301, &P15830_322};	/*Property = Selector*/
const Value V15830_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_324 = {S43, 0, &V15830_324_57307, &P15830_323};	/*Property = Svg*/
const Value V15830_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_327 = {S44, 0, &V15830_327_57316, &P15830_324};	/*Property = TouchEvents*/
const Value V15830_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_328 = {S45, 0, &V15830_328_57319, &P15830_327};	/*Property = Track*/
const Value V15830_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_329 = {S46, 0, &V15830_329_57325, &P15830_328};	/*Property = Video*/
const Value V15830_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_330 = {S47, 0, &V15830_330_57328, &P15830_329};	/*Property = Viewport*/
const Value V15830_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_332 = {S48, 0, &V15830_332_57340, &P15830_330};	/*Property = WebWorkers*/
const Value V15830_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P15830_333 = {S49, 0, &V15830_333_57343, &P15830_332};	/*Property = Xhr2*/
const Profile P15830 = {"15830", &P15830_333};	/*Profile ID - 15830*/
const Value V15254_1_1 = {S2, NULL};	/*Value = True*/
const Property P15254_1 = {S50, 0, &V15254_1_1, NULL};	/*Property = IsMobile*/
const Value V15254_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15254_5 = {S52, 0, &V15254_5_1277, &P15254_1};	/*Property = ScreenPixelsHeight*/
const Value V15254_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P15254_4 = {S54, 0, &V15254_4_8554, &P15254_5};	/*Property = ScreenPixelsWidth*/
const Profile P15254 = {"15254", &P15254_4};	/*Profile ID - 15254*/
const Value V12705_1_1 = {S2, NULL};	/*Value = True*/
const Property P12705_1 = {S50, 0, &V12705_1_1, NULL};	/*Property = IsMobile*/
const Value V12705_5_1796 = {S64, NULL};	/*Value = 360*/
const Property P12705_5 = {S52, 0, &V12705_5_1796, &P12705_1};	/*Property = ScreenPixelsHeight*/
const Value V12705_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P12705_4 = {S54, 0, &V12705_4_1229, &P12705_5};	/*Property = ScreenPixelsWidth*/
const Profile P12705 = {"12705", &P12705_4};	/*Profile ID - 12705*/
const Profile P17361 = {"17361", NULL};	/*Profile ID - 17361*/
const Value V18293_1_1 = {S2, NULL};	/*Value = True*/
const Property P18293_1 = {S50, 0, &V18293_1_1, NULL};	/*Property = IsMobile*/
const Value V18293_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18293_5 = {S52, 0, &V18293_5_1227, &P18293_1};	/*Property = ScreenPixelsHeight*/
const Value V18293_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18293_4 = {S54, 0, &V18293_4_1229, &P18293_5};	/*Property = ScreenPixelsWidth*/
const Profile P18293 = {"18293", &P18293_4};	/*Profile ID - 18293*/
const Value V23044_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_279 = {S1, 0, &V23044_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23044_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_282 = {S3, 0, &V23044_282_57155, &P23044_279};	/*Property = BlobBuilder*/
const Value V23044_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_350 = {S4, 0, &V23044_350_57446, &P23044_282};	/*Property = Canvas*/
const Value V23044_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_283 = {S5, 0, &V23044_283_57161, &P23044_350};	/*Property = CssBackground*/
const Value V23044_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_284 = {S6, 0, &V23044_284_57164, &P23044_283};	/*Property = CssBorderImage*/
const Value V23044_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_285 = {S7, 0, &V23044_285_57167, &P23044_284};	/*Property = CssCanvas*/
const Value V23044_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_286 = {S8, 0, &V23044_286_57170, &P23044_285};	/*Property = CssColor*/
const Value V23044_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_237 = {S9, 0, &V23044_237_55786, &P23044_286};	/*Property = CssColumn*/
const Value V23044_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_287 = {S10, 0, &V23044_287_57173, &P23044_237};	/*Property = CssFlexbox*/
const Value V23044_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_288 = {S11, 0, &V23044_288_57176, &P23044_287};	/*Property = CssFont*/
const Value V23044_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_289 = {S12, 0, &V23044_289_86899, &P23044_288};	/*Property = CssImages*/
const Value V23044_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_291 = {S13, 0, &V23044_291_57182, &P23044_289};	/*Property = CssMediaQueries*/
const Value V23044_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_292 = {S14, 0, &V23044_292_57185, &P23044_291};	/*Property = CssMinMax*/
const Value V23044_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_293 = {S15, 0, &V23044_293_57188, &P23044_292};	/*Property = CssOverflow*/
const Value V23044_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_294 = {S16, 0, &V23044_294_57191, &P23044_293};	/*Property = CssPosition*/
const Value V23044_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_295 = {S17, 0, &V23044_295_57194, &P23044_294};	/*Property = CssText*/
const Value V23044_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_297 = {S18, 0, &V23044_297_57197, &P23044_295};	/*Property = CssTransforms*/
const Value V23044_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_241 = {S19, 0, &V23044_241_55801, &P23044_297};	/*Property = CssTransitions*/
const Value V23044_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_298 = {S20, 0, &V23044_298_57208, &P23044_241};	/*Property = CssUI*/
const Value V23044_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_300 = {S21, 0, &V23044_300_57217, &P23044_298};	/*Property = DataSet*/
const Value V23044_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_301 = {S22, 0, &V23044_301_57220, &P23044_300};	/*Property = DataUrl*/
const Value V23044_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_302 = {S23, 0, &V23044_302_57223, &P23044_301};	/*Property = DeviceOrientation*/
const Value V23044_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_304 = {S24, 0, &V23044_304_57229, &P23044_302};	/*Property = FileReader*/
const Value V23044_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_305 = {S25, 0, &V23044_305_57232, &P23044_304};	/*Property = FileSaver*/
const Value V23044_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_306 = {S26, 0, &V23044_306_57238, &P23044_305};	/*Property = FileWriter*/
const Value V23044_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_307 = {S27, 0, &V23044_307_57241, &P23044_306};	/*Property = FormData*/
const Value V23044_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_309 = {S28, 0, &V23044_309_57247, &P23044_307};	/*Property = Fullscreen*/
const Value V23044_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_310 = {S29, 0, &V23044_310_57250, &P23044_309};	/*Property = GeoLocation*/
const Value V23044_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_312 = {S30, 0, &V23044_312_57256, &P23044_310};	/*Property = History*/
const Value V23044_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_314 = {S31, 0, &V23044_314_57262, &P23044_312};	/*Property = Html5*/
const Value V23044_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_313 = {S32, 0, &V23044_313_57259, &P23044_314};	/*Property = Html-Media-Capture*/
const Value V23044_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_315 = {S33, 0, &V23044_315_57265, &P23044_313};	/*Property = Iframe*/
const Value V23044_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_316 = {S34, 0, &V23044_316_57268, &P23044_315};	/*Property = IndexedDB*/
const Value V23044_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_317 = {S35, 0, &V23044_317_57271, &P23044_316};	/*Property = Json*/
const Value V23044_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_15 = {S37, 0, &V23044_15_72, &P23044_317};	/*Property = LayoutEngine*/
const Value V23044_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_344 = {S38, 0, &V23044_344_57274, &P23044_15};	/*Property = Masking*/
const Value V23044_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_320 = {S39, 0, &V23044_320_57292, &P23044_344};	/*Property = PostMessage*/
const Value V23044_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_321 = {S40, 0, &V23044_321_57295, &P23044_320};	/*Property = Progress*/
const Value V23044_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_322 = {S41, 0, &V23044_322_57298, &P23044_321};	/*Property = Prompts*/
const Value V23044_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_323 = {S42, 0, &V23044_323_57301, &P23044_322};	/*Property = Selector*/
const Value V23044_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_324 = {S43, 0, &V23044_324_57307, &P23044_323};	/*Property = Svg*/
const Value V23044_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_327 = {S44, 0, &V23044_327_57316, &P23044_324};	/*Property = TouchEvents*/
const Value V23044_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_328 = {S45, 0, &V23044_328_57319, &P23044_327};	/*Property = Track*/
const Value V23044_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_329 = {S46, 0, &V23044_329_57325, &P23044_328};	/*Property = Video*/
const Value V23044_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_330 = {S47, 0, &V23044_330_57328, &P23044_329};	/*Property = Viewport*/
const Value V23044_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_332 = {S48, 0, &V23044_332_57340, &P23044_330};	/*Property = WebWorkers*/
const Value V23044_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23044_333 = {S49, 0, &V23044_333_57343, &P23044_332};	/*Property = Xhr2*/
const Profile P23044 = {"23044", &P23044_333};	/*Profile ID - 23044*/
const Value V23788_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_279 = {S1, 0, &V23788_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23788_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_282 = {S3, 0, &V23788_282_57155, &P23788_279};	/*Property = BlobBuilder*/
const Value V23788_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_350 = {S4, 0, &V23788_350_57446, &P23788_282};	/*Property = Canvas*/
const Value V23788_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_283 = {S5, 0, &V23788_283_57161, &P23788_350};	/*Property = CssBackground*/
const Value V23788_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_284 = {S6, 0, &V23788_284_57164, &P23788_283};	/*Property = CssBorderImage*/
const Value V23788_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_285 = {S7, 0, &V23788_285_57167, &P23788_284};	/*Property = CssCanvas*/
const Value V23788_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_286 = {S8, 0, &V23788_286_57170, &P23788_285};	/*Property = CssColor*/
const Value V23788_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_237 = {S9, 0, &V23788_237_55786, &P23788_286};	/*Property = CssColumn*/
const Value V23788_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_287 = {S10, 0, &V23788_287_57173, &P23788_237};	/*Property = CssFlexbox*/
const Value V23788_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_288 = {S11, 0, &V23788_288_57176, &P23788_287};	/*Property = CssFont*/
const Value V23788_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_289 = {S12, 0, &V23788_289_86899, &P23788_288};	/*Property = CssImages*/
const Value V23788_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_291 = {S13, 0, &V23788_291_57182, &P23788_289};	/*Property = CssMediaQueries*/
const Value V23788_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_292 = {S14, 0, &V23788_292_57185, &P23788_291};	/*Property = CssMinMax*/
const Value V23788_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_293 = {S15, 0, &V23788_293_57188, &P23788_292};	/*Property = CssOverflow*/
const Value V23788_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_294 = {S16, 0, &V23788_294_57191, &P23788_293};	/*Property = CssPosition*/
const Value V23788_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_295 = {S17, 0, &V23788_295_57194, &P23788_294};	/*Property = CssText*/
const Value V23788_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_297 = {S18, 0, &V23788_297_57197, &P23788_295};	/*Property = CssTransforms*/
const Value V23788_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_241 = {S19, 0, &V23788_241_55801, &P23788_297};	/*Property = CssTransitions*/
const Value V23788_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_298 = {S20, 0, &V23788_298_57208, &P23788_241};	/*Property = CssUI*/
const Value V23788_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_300 = {S21, 0, &V23788_300_57217, &P23788_298};	/*Property = DataSet*/
const Value V23788_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_301 = {S22, 0, &V23788_301_57220, &P23788_300};	/*Property = DataUrl*/
const Value V23788_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_302 = {S23, 0, &V23788_302_57223, &P23788_301};	/*Property = DeviceOrientation*/
const Value V23788_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_304 = {S24, 0, &V23788_304_57229, &P23788_302};	/*Property = FileReader*/
const Value V23788_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_305 = {S25, 0, &V23788_305_57232, &P23788_304};	/*Property = FileSaver*/
const Value V23788_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_306 = {S26, 0, &V23788_306_57238, &P23788_305};	/*Property = FileWriter*/
const Value V23788_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_307 = {S27, 0, &V23788_307_57241, &P23788_306};	/*Property = FormData*/
const Value V23788_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_309 = {S28, 0, &V23788_309_57247, &P23788_307};	/*Property = Fullscreen*/
const Value V23788_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_310 = {S29, 0, &V23788_310_57250, &P23788_309};	/*Property = GeoLocation*/
const Value V23788_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_312 = {S30, 0, &V23788_312_57256, &P23788_310};	/*Property = History*/
const Value V23788_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_314 = {S31, 0, &V23788_314_57262, &P23788_312};	/*Property = Html5*/
const Value V23788_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_313 = {S32, 0, &V23788_313_57259, &P23788_314};	/*Property = Html-Media-Capture*/
const Value V23788_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_315 = {S33, 0, &V23788_315_57265, &P23788_313};	/*Property = Iframe*/
const Value V23788_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_316 = {S34, 0, &V23788_316_57268, &P23788_315};	/*Property = IndexedDB*/
const Value V23788_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_317 = {S35, 0, &V23788_317_57271, &P23788_316};	/*Property = Json*/
const Value V23788_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_15 = {S37, 0, &V23788_15_72, &P23788_317};	/*Property = LayoutEngine*/
const Value V23788_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_344 = {S38, 0, &V23788_344_57274, &P23788_15};	/*Property = Masking*/
const Value V23788_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_320 = {S39, 0, &V23788_320_57292, &P23788_344};	/*Property = PostMessage*/
const Value V23788_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_321 = {S40, 0, &V23788_321_57295, &P23788_320};	/*Property = Progress*/
const Value V23788_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_322 = {S41, 0, &V23788_322_57298, &P23788_321};	/*Property = Prompts*/
const Value V23788_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_323 = {S42, 0, &V23788_323_57301, &P23788_322};	/*Property = Selector*/
const Value V23788_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_324 = {S43, 0, &V23788_324_57307, &P23788_323};	/*Property = Svg*/
const Value V23788_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_327 = {S44, 0, &V23788_327_57316, &P23788_324};	/*Property = TouchEvents*/
const Value V23788_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_328 = {S45, 0, &V23788_328_57319, &P23788_327};	/*Property = Track*/
const Value V23788_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_329 = {S46, 0, &V23788_329_57325, &P23788_328};	/*Property = Video*/
const Value V23788_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_330 = {S47, 0, &V23788_330_57328, &P23788_329};	/*Property = Viewport*/
const Value V23788_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_332 = {S48, 0, &V23788_332_57340, &P23788_330};	/*Property = WebWorkers*/
const Value V23788_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23788_333 = {S49, 0, &V23788_333_57343, &P23788_332};	/*Property = Xhr2*/
const Profile P23788 = {"23788", &P23788_333};	/*Profile ID - 23788*/
const Profile P17292 = {"17292", NULL};	/*Profile ID - 17292*/
const Value V15900_1_1 = {S2, NULL};	/*Value = True*/
const Property P15900_1 = {S50, 0, &V15900_1_1, NULL};	/*Property = IsMobile*/
const Value V15900_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15900_5 = {S52, 0, &V15900_5_1222, &P15900_1};	/*Property = ScreenPixelsHeight*/
const Value V15900_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15900_4 = {S54, 0, &V15900_4_1224, &P15900_5};	/*Property = ScreenPixelsWidth*/
const Profile P15900 = {"15900", &P15900_4};	/*Profile ID - 15900*/
const Value V23399_279_57148 = {S0, NULL};	/*Value = False*/
const Property P23399_279 = {S1, 0, &V23399_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V23399_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23399_282 = {S3, 0, &V23399_282_57154, &P23399_279};	/*Property = BlobBuilder*/
const Value V23399_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23399_350 = {S4, 0, &V23399_350_57444, &P23399_282};	/*Property = Canvas*/
const Value V23399_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23399_283 = {S5, 0, &V23399_283_57159, &P23399_350};	/*Property = CssBackground*/
const Value V23399_284_57162 = {S2, NULL};	/*Value = True*/
const Property P23399_284 = {S6, 0, &V23399_284_57162, &P23399_283};	/*Property = CssBorderImage*/
const Value V23399_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23399_285 = {S7, 0, &V23399_285_57166, &P23399_284};	/*Property = CssCanvas*/
const Value V23399_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23399_286 = {S8, 0, &V23399_286_57168, &P23399_285};	/*Property = CssColor*/
const Value V23399_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23399_237 = {S9, 0, &V23399_237_55786, &P23399_286};	/*Property = CssColumn*/
const Value V23399_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23399_287 = {S10, 0, &V23399_287_57172, &P23399_237};	/*Property = CssFlexbox*/
const Value V23399_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23399_288 = {S11, 0, &V23399_288_57174, &P23399_287};	/*Property = CssFont*/
const Value V23399_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23399_289 = {S12, 0, &V23399_289_57759, &P23399_288};	/*Property = CssImages*/
const Value V23399_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23399_291 = {S13, 0, &V23399_291_57180, &P23399_289};	/*Property = CssMediaQueries*/
const Value V23399_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23399_292 = {S14, 0, &V23399_292_57183, &P23399_291};	/*Property = CssMinMax*/
const Value V23399_293_57186 = {S2, NULL};	/*Value = True*/
const Property P23399_293 = {S15, 0, &V23399_293_57186, &P23399_292};	/*Property = CssOverflow*/
const Value V23399_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23399_294 = {S16, 0, &V23399_294_57189, &P23399_293};	/*Property = CssPosition*/
const Value V23399_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23399_295 = {S17, 0, &V23399_295_57192, &P23399_294};	/*Property = CssText*/
const Value V23399_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23399_297 = {S18, 0, &V23399_297_57195, &P23399_295};	/*Property = CssTransforms*/
const Value V23399_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23399_241 = {S19, 0, &V23399_241_55799, &P23399_297};	/*Property = CssTransitions*/
const Value V23399_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23399_298 = {S20, 0, &V23399_298_57206, &P23399_241};	/*Property = CssUI*/
const Value V23399_300_57215 = {S2, NULL};	/*Value = True*/
const Property P23399_300 = {S21, 0, &V23399_300_57215, &P23399_298};	/*Property = DataSet*/
const Value V23399_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23399_301 = {S22, 0, &V23399_301_57218, &P23399_300};	/*Property = DataUrl*/
const Value V23399_302_57221 = {S2, NULL};	/*Value = True*/
const Property P23399_302 = {S23, 0, &V23399_302_57221, &P23399_301};	/*Property = DeviceOrientation*/
const Value V23399_304_57228 = {S0, NULL};	/*Value = False*/
const Property P23399_304 = {S24, 0, &V23399_304_57228, &P23399_302};	/*Property = FileReader*/
const Value V23399_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23399_305 = {S25, 0, &V23399_305_57231, &P23399_304};	/*Property = FileSaver*/
const Value V23399_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23399_306 = {S26, 0, &V23399_306_57237, &P23399_305};	/*Property = FileWriter*/
const Value V23399_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23399_307 = {S27, 0, &V23399_307_57239, &P23399_306};	/*Property = FormData*/
const Value V23399_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23399_309 = {S28, 0, &V23399_309_57246, &P23399_307};	/*Property = Fullscreen*/
const Value V23399_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23399_310 = {S29, 0, &V23399_310_57248, &P23399_309};	/*Property = GeoLocation*/
const Value V23399_312_57255 = {S0, NULL};	/*Value = False*/
const Property P23399_312 = {S30, 0, &V23399_312_57255, &P23399_310};	/*Property = History*/
const Value V23399_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23399_314 = {S31, 0, &V23399_314_57261, &P23399_312};	/*Property = Html5*/
const Value V23399_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23399_313 = {S32, 0, &V23399_313_57258, &P23399_314};	/*Property = Html-Media-Capture*/
const Value V23399_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23399_315 = {S33, 0, &V23399_315_57264, &P23399_313};	/*Property = Iframe*/
const Value V23399_316_57267 = {S0, NULL};	/*Value = False*/
const Property P23399_316 = {S34, 0, &V23399_316_57267, &P23399_315};	/*Property = IndexedDB*/
const Value V23399_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23399_317 = {S35, 0, &V23399_317_57269, &P23399_316};	/*Property = Json*/
const Value V23399_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P23399_15 = {S37, 0, &V23399_15_54167, &P23399_317};	/*Property = LayoutEngine*/
const Value V23399_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23399_344 = {S38, 0, &V23399_344_57272, &P23399_15};	/*Property = Masking*/
const Value V23399_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23399_320 = {S39, 0, &V23399_320_57290, &P23399_344};	/*Property = PostMessage*/
const Value V23399_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23399_321 = {S40, 0, &V23399_321_57293, &P23399_320};	/*Property = Progress*/
const Value V23399_322_57297 = {S0, NULL};	/*Value = False*/
const Property P23399_322 = {S41, 0, &V23399_322_57297, &P23399_321};	/*Property = Prompts*/
const Value V23399_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23399_323 = {S42, 0, &V23399_323_57299, &P23399_322};	/*Property = Selector*/
const Value V23399_324_57306 = {S0, NULL};	/*Value = False*/
const Property P23399_324 = {S43, 0, &V23399_324_57306, &P23399_323};	/*Property = Svg*/
const Value V23399_327_57314 = {S2, NULL};	/*Value = True*/
const Property P23399_327 = {S44, 0, &V23399_327_57314, &P23399_324};	/*Property = TouchEvents*/
const Value V23399_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23399_328 = {S45, 0, &V23399_328_57318, &P23399_327};	/*Property = Track*/
const Value V23399_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23399_329 = {S46, 0, &V23399_329_57323, &P23399_328};	/*Property = Video*/
const Value V23399_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23399_330 = {S47, 0, &V23399_330_57326, &P23399_329};	/*Property = Viewport*/
const Value V23399_332_57339 = {S0, NULL};	/*Value = False*/
const Property P23399_332 = {S48, 0, &V23399_332_57339, &P23399_330};	/*Property = WebWorkers*/
const Value V23399_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23399_333 = {S49, 0, &V23399_333_57342, &P23399_332};	/*Property = Xhr2*/
const Profile P23399 = {"23399", &P23399_333};	/*Profile ID - 23399*/
const Value V22875_1_1 = {S2, NULL};	/*Value = True*/
const Property P22875_1 = {S50, 0, &V22875_1_1, NULL};	/*Property = IsMobile*/
const Value V22875_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22875_5 = {S52, 0, &V22875_5_9981, &P22875_1};	/*Property = ScreenPixelsHeight*/
const Value V22875_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22875_4 = {S54, 0, &V22875_4_2336, &P22875_5};	/*Property = ScreenPixelsWidth*/
const Profile P22875 = {"22875", &P22875_4};	/*Profile ID - 22875*/
const Value V15860_1_1 = {S2, NULL};	/*Value = True*/
const Property P15860_1 = {S50, 0, &V15860_1_1, NULL};	/*Property = IsMobile*/
const Value V15860_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15860_5 = {S52, 0, &V15860_5_1227, &P15860_1};	/*Property = ScreenPixelsHeight*/
const Value V15860_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15860_4 = {S54, 0, &V15860_4_1229, &P15860_5};	/*Property = ScreenPixelsWidth*/
const Profile P15860 = {"15860", &P15860_4};	/*Profile ID - 15860*/
const Value V18331_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18331_279 = {S1, 0, &V18331_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18331_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18331_282 = {S3, 0, &V18331_282_57154, &P18331_279};	/*Property = BlobBuilder*/
const Value V18331_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18331_350 = {S4, 0, &V18331_350_57444, &P18331_282};	/*Property = Canvas*/
const Value V18331_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18331_283 = {S5, 0, &V18331_283_57159, &P18331_350};	/*Property = CssBackground*/
const Value V18331_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18331_284 = {S6, 0, &V18331_284_57163, &P18331_283};	/*Property = CssBorderImage*/
const Value V18331_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18331_285 = {S7, 0, &V18331_285_57166, &P18331_284};	/*Property = CssCanvas*/
const Value V18331_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18331_286 = {S8, 0, &V18331_286_57168, &P18331_285};	/*Property = CssColor*/
const Value V18331_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18331_237 = {S9, 0, &V18331_237_55786, &P18331_286};	/*Property = CssColumn*/
const Value V18331_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18331_287 = {S10, 0, &V18331_287_57172, &P18331_237};	/*Property = CssFlexbox*/
const Value V18331_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18331_288 = {S11, 0, &V18331_288_57175, &P18331_287};	/*Property = CssFont*/
const Value V18331_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18331_289 = {S12, 0, &V18331_289_57759, &P18331_288};	/*Property = CssImages*/
const Value V18331_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18331_291 = {S13, 0, &V18331_291_57181, &P18331_289};	/*Property = CssMediaQueries*/
const Value V18331_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18331_292 = {S14, 0, &V18331_292_57183, &P18331_291};	/*Property = CssMinMax*/
const Value V18331_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18331_293 = {S15, 0, &V18331_293_57187, &P18331_292};	/*Property = CssOverflow*/
const Value V18331_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18331_294 = {S16, 0, &V18331_294_57190, &P18331_293};	/*Property = CssPosition*/
const Value V18331_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18331_295 = {S17, 0, &V18331_295_57193, &P18331_294};	/*Property = CssText*/
const Value V18331_297_57196 = {S0, NULL};	/*Value = False*/
const Property P18331_297 = {S18, 0, &V18331_297_57196, &P18331_295};	/*Property = CssTransforms*/
const Value V18331_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18331_241 = {S19, 0, &V18331_241_55799, &P18331_297};	/*Property = CssTransitions*/
const Value V18331_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18331_298 = {S20, 0, &V18331_298_57206, &P18331_241};	/*Property = CssUI*/
const Value V18331_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18331_300 = {S21, 0, &V18331_300_57216, &P18331_298};	/*Property = DataSet*/
const Value V18331_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18331_301 = {S22, 0, &V18331_301_57218, &P18331_300};	/*Property = DataUrl*/
const Value V18331_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18331_302 = {S23, 0, &V18331_302_57222, &P18331_301};	/*Property = DeviceOrientation*/
const Value V18331_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18331_304 = {S24, 0, &V18331_304_57228, &P18331_302};	/*Property = FileReader*/
const Value V18331_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18331_305 = {S25, 0, &V18331_305_57231, &P18331_304};	/*Property = FileSaver*/
const Value V18331_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18331_306 = {S26, 0, &V18331_306_57237, &P18331_305};	/*Property = FileWriter*/
const Value V18331_307_57240 = {S0, NULL};	/*Value = False*/
const Property P18331_307 = {S27, 0, &V18331_307_57240, &P18331_306};	/*Property = FormData*/
const Value V18331_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18331_309 = {S28, 0, &V18331_309_57246, &P18331_307};	/*Property = Fullscreen*/
const Value V18331_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18331_310 = {S29, 0, &V18331_310_57248, &P18331_309};	/*Property = GeoLocation*/
const Value V18331_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18331_312 = {S30, 0, &V18331_312_57255, &P18331_310};	/*Property = History*/
const Value V18331_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18331_314 = {S31, 0, &V18331_314_57261, &P18331_312};	/*Property = Html5*/
const Value V18331_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18331_313 = {S32, 0, &V18331_313_57258, &P18331_314};	/*Property = Html-Media-Capture*/
const Value V18331_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18331_315 = {S33, 0, &V18331_315_57264, &P18331_313};	/*Property = Iframe*/
const Value V18331_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18331_316 = {S34, 0, &V18331_316_57267, &P18331_315};	/*Property = IndexedDB*/
const Value V18331_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18331_317 = {S35, 0, &V18331_317_57269, &P18331_316};	/*Property = Json*/
const Value V18331_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18331_15 = {S37, 0, &V18331_15_31, &P18331_317};	/*Property = LayoutEngine*/
const Value V18331_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18331_344 = {S38, 0, &V18331_344_57272, &P18331_15};	/*Property = Masking*/
const Value V18331_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18331_320 = {S39, 0, &V18331_320_57290, &P18331_344};	/*Property = PostMessage*/
const Value V18331_321_57294 = {S0, NULL};	/*Value = False*/
const Property P18331_321 = {S40, 0, &V18331_321_57294, &P18331_320};	/*Property = Progress*/
const Value V18331_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18331_322 = {S41, 0, &V18331_322_57296, &P18331_321};	/*Property = Prompts*/
const Value V18331_323_57300 = {S0, NULL};	/*Value = False*/
const Property P18331_323 = {S42, 0, &V18331_323_57300, &P18331_322};	/*Property = Selector*/
const Value V18331_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18331_324 = {S43, 0, &V18331_324_57306, &P18331_323};	/*Property = Svg*/
const Value V18331_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18331_327 = {S44, 0, &V18331_327_57315, &P18331_324};	/*Property = TouchEvents*/
const Value V18331_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18331_328 = {S45, 0, &V18331_328_57318, &P18331_327};	/*Property = Track*/
const Value V18331_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18331_329 = {S46, 0, &V18331_329_57323, &P18331_328};	/*Property = Video*/
const Value V18331_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18331_330 = {S47, 0, &V18331_330_57326, &P18331_329};	/*Property = Viewport*/
const Value V18331_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18331_332 = {S48, 0, &V18331_332_57339, &P18331_330};	/*Property = WebWorkers*/
const Value V18331_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18331_333 = {S49, 0, &V18331_333_57342, &P18331_332};	/*Property = Xhr2*/
const Profile P18331 = {"18331", &P18331_333};	/*Profile ID - 18331*/
const Value V18537_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_279 = {S1, 0, &V18537_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18537_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_282 = {S3, 0, &V18537_282_57155, &P18537_279};	/*Property = BlobBuilder*/
const Value V18537_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_350 = {S4, 0, &V18537_350_57446, &P18537_282};	/*Property = Canvas*/
const Value V18537_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_283 = {S5, 0, &V18537_283_57161, &P18537_350};	/*Property = CssBackground*/
const Value V18537_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_284 = {S6, 0, &V18537_284_57164, &P18537_283};	/*Property = CssBorderImage*/
const Value V18537_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_285 = {S7, 0, &V18537_285_57167, &P18537_284};	/*Property = CssCanvas*/
const Value V18537_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_286 = {S8, 0, &V18537_286_57170, &P18537_285};	/*Property = CssColor*/
const Value V18537_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_237 = {S9, 0, &V18537_237_55786, &P18537_286};	/*Property = CssColumn*/
const Value V18537_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_287 = {S10, 0, &V18537_287_57173, &P18537_237};	/*Property = CssFlexbox*/
const Value V18537_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_288 = {S11, 0, &V18537_288_57176, &P18537_287};	/*Property = CssFont*/
const Value V18537_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_289 = {S12, 0, &V18537_289_86899, &P18537_288};	/*Property = CssImages*/
const Value V18537_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_291 = {S13, 0, &V18537_291_57182, &P18537_289};	/*Property = CssMediaQueries*/
const Value V18537_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_292 = {S14, 0, &V18537_292_57185, &P18537_291};	/*Property = CssMinMax*/
const Value V18537_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_293 = {S15, 0, &V18537_293_57188, &P18537_292};	/*Property = CssOverflow*/
const Value V18537_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_294 = {S16, 0, &V18537_294_57191, &P18537_293};	/*Property = CssPosition*/
const Value V18537_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_295 = {S17, 0, &V18537_295_57194, &P18537_294};	/*Property = CssText*/
const Value V18537_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_297 = {S18, 0, &V18537_297_57197, &P18537_295};	/*Property = CssTransforms*/
const Value V18537_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_241 = {S19, 0, &V18537_241_55801, &P18537_297};	/*Property = CssTransitions*/
const Value V18537_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_298 = {S20, 0, &V18537_298_57208, &P18537_241};	/*Property = CssUI*/
const Value V18537_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_300 = {S21, 0, &V18537_300_57217, &P18537_298};	/*Property = DataSet*/
const Value V18537_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_301 = {S22, 0, &V18537_301_57220, &P18537_300};	/*Property = DataUrl*/
const Value V18537_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_302 = {S23, 0, &V18537_302_57223, &P18537_301};	/*Property = DeviceOrientation*/
const Value V18537_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_304 = {S24, 0, &V18537_304_57229, &P18537_302};	/*Property = FileReader*/
const Value V18537_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_305 = {S25, 0, &V18537_305_57232, &P18537_304};	/*Property = FileSaver*/
const Value V18537_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_306 = {S26, 0, &V18537_306_57238, &P18537_305};	/*Property = FileWriter*/
const Value V18537_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_307 = {S27, 0, &V18537_307_57241, &P18537_306};	/*Property = FormData*/
const Value V18537_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_309 = {S28, 0, &V18537_309_57247, &P18537_307};	/*Property = Fullscreen*/
const Value V18537_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_310 = {S29, 0, &V18537_310_57250, &P18537_309};	/*Property = GeoLocation*/
const Value V18537_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_312 = {S30, 0, &V18537_312_57256, &P18537_310};	/*Property = History*/
const Value V18537_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_314 = {S31, 0, &V18537_314_57262, &P18537_312};	/*Property = Html5*/
const Value V18537_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_313 = {S32, 0, &V18537_313_57259, &P18537_314};	/*Property = Html-Media-Capture*/
const Value V18537_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_315 = {S33, 0, &V18537_315_57265, &P18537_313};	/*Property = Iframe*/
const Value V18537_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_316 = {S34, 0, &V18537_316_57268, &P18537_315};	/*Property = IndexedDB*/
const Value V18537_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_317 = {S35, 0, &V18537_317_57271, &P18537_316};	/*Property = Json*/
const Value V18537_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18537_15 = {S37, 0, &V18537_15_28, &P18537_317};	/*Property = LayoutEngine*/
const Value V18537_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_344 = {S38, 0, &V18537_344_57274, &P18537_15};	/*Property = Masking*/
const Value V18537_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_320 = {S39, 0, &V18537_320_57292, &P18537_344};	/*Property = PostMessage*/
const Value V18537_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_321 = {S40, 0, &V18537_321_57295, &P18537_320};	/*Property = Progress*/
const Value V18537_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_322 = {S41, 0, &V18537_322_57298, &P18537_321};	/*Property = Prompts*/
const Value V18537_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_323 = {S42, 0, &V18537_323_57301, &P18537_322};	/*Property = Selector*/
const Value V18537_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_324 = {S43, 0, &V18537_324_57307, &P18537_323};	/*Property = Svg*/
const Value V18537_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_327 = {S44, 0, &V18537_327_57316, &P18537_324};	/*Property = TouchEvents*/
const Value V18537_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_328 = {S45, 0, &V18537_328_57319, &P18537_327};	/*Property = Track*/
const Value V18537_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_329 = {S46, 0, &V18537_329_57325, &P18537_328};	/*Property = Video*/
const Value V18537_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_330 = {S47, 0, &V18537_330_57328, &P18537_329};	/*Property = Viewport*/
const Value V18537_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_332 = {S48, 0, &V18537_332_57340, &P18537_330};	/*Property = WebWorkers*/
const Value V18537_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18537_333 = {S49, 0, &V18537_333_57343, &P18537_332};	/*Property = Xhr2*/
const Profile P18537 = {"18537", &P18537_333};	/*Profile ID - 18537*/
const Value V18301_1_1 = {S2, NULL};	/*Value = True*/
const Property P18301_1 = {S50, 0, &V18301_1_1, NULL};	/*Property = IsMobile*/
const Value V18301_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18301_5 = {S52, 0, &V18301_5_1227, &P18301_1};	/*Property = ScreenPixelsHeight*/
const Value V18301_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18301_4 = {S54, 0, &V18301_4_1229, &P18301_5};	/*Property = ScreenPixelsWidth*/
const Profile P18301 = {"18301", &P18301_4};	/*Profile ID - 18301*/
const Value V18304_1_1 = {S2, NULL};	/*Value = True*/
const Property P18304_1 = {S50, 0, &V18304_1_1, NULL};	/*Property = IsMobile*/
const Value V18304_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18304_5 = {S52, 0, &V18304_5_1227, &P18304_1};	/*Property = ScreenPixelsHeight*/
const Value V18304_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18304_4 = {S54, 0, &V18304_4_1229, &P18304_5};	/*Property = ScreenPixelsWidth*/
const Profile P18304 = {"18304", &P18304_4};	/*Profile ID - 18304*/
const Value V18295_1_1 = {S2, NULL};	/*Value = True*/
const Property P18295_1 = {S50, 0, &V18295_1_1, NULL};	/*Property = IsMobile*/
const Value V18295_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18295_5 = {S52, 0, &V18295_5_9981, &P18295_1};	/*Property = ScreenPixelsHeight*/
const Value V18295_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18295_4 = {S54, 0, &V18295_4_39098, &P18295_5};	/*Property = ScreenPixelsWidth*/
const Profile P18295 = {"18295", &P18295_4};	/*Profile ID - 18295*/
const Value V18296_1_1 = {S2, NULL};	/*Value = True*/
const Property P18296_1 = {S50, 0, &V18296_1_1, NULL};	/*Property = IsMobile*/
const Value V18296_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18296_5 = {S52, 0, &V18296_5_1227, &P18296_1};	/*Property = ScreenPixelsHeight*/
const Value V18296_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18296_4 = {S54, 0, &V18296_4_1229, &P18296_5};	/*Property = ScreenPixelsWidth*/
const Profile P18296 = {"18296", &P18296_4};	/*Profile ID - 18296*/
const Value V18297_1_1 = {S2, NULL};	/*Value = True*/
const Property P18297_1 = {S50, 0, &V18297_1_1, NULL};	/*Property = IsMobile*/
const Value V18297_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18297_5 = {S52, 0, &V18297_5_1227, &P18297_1};	/*Property = ScreenPixelsHeight*/
const Value V18297_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18297_4 = {S54, 0, &V18297_4_1229, &P18297_5};	/*Property = ScreenPixelsWidth*/
const Profile P18297 = {"18297", &P18297_4};	/*Profile ID - 18297*/
const Value V18305_1_1 = {S2, NULL};	/*Value = True*/
const Property P18305_1 = {S50, 0, &V18305_1_1, NULL};	/*Property = IsMobile*/
const Value V18305_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18305_5 = {S52, 0, &V18305_5_9981, &P18305_1};	/*Property = ScreenPixelsHeight*/
const Value V18305_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18305_4 = {S54, 0, &V18305_4_39098, &P18305_5};	/*Property = ScreenPixelsWidth*/
const Profile P18305 = {"18305", &P18305_4};	/*Profile ID - 18305*/
const Value V20356_1_2 = {S0, NULL};	/*Value = False*/
const Property P20356_1 = {S50, 0, &V20356_1_2, NULL};	/*Property = IsMobile*/
const Value V20356_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20356_5 = {S52, 0, &V20356_5_37, &P20356_1};	/*Property = ScreenPixelsHeight*/
const Value V20356_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20356_4 = {S54, 0, &V20356_4_36, &P20356_5};	/*Property = ScreenPixelsWidth*/
const Profile P20356 = {"20356", &P20356_4};	/*Profile ID - 20356*/
const Value V20774_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_279 = {S1, 0, &V20774_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20774_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_282 = {S3, 0, &V20774_282_57155, &P20774_279};	/*Property = BlobBuilder*/
const Value V20774_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_350 = {S4, 0, &V20774_350_57446, &P20774_282};	/*Property = Canvas*/
const Value V20774_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_283 = {S5, 0, &V20774_283_57161, &P20774_350};	/*Property = CssBackground*/
const Value V20774_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_284 = {S6, 0, &V20774_284_57164, &P20774_283};	/*Property = CssBorderImage*/
const Value V20774_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_285 = {S7, 0, &V20774_285_57167, &P20774_284};	/*Property = CssCanvas*/
const Value V20774_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_286 = {S8, 0, &V20774_286_57170, &P20774_285};	/*Property = CssColor*/
const Value V20774_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_237 = {S9, 0, &V20774_237_55786, &P20774_286};	/*Property = CssColumn*/
const Value V20774_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_287 = {S10, 0, &V20774_287_57173, &P20774_237};	/*Property = CssFlexbox*/
const Value V20774_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_288 = {S11, 0, &V20774_288_57176, &P20774_287};	/*Property = CssFont*/
const Value V20774_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_289 = {S12, 0, &V20774_289_86899, &P20774_288};	/*Property = CssImages*/
const Value V20774_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_291 = {S13, 0, &V20774_291_57182, &P20774_289};	/*Property = CssMediaQueries*/
const Value V20774_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_292 = {S14, 0, &V20774_292_57185, &P20774_291};	/*Property = CssMinMax*/
const Value V20774_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_293 = {S15, 0, &V20774_293_57188, &P20774_292};	/*Property = CssOverflow*/
const Value V20774_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_294 = {S16, 0, &V20774_294_57191, &P20774_293};	/*Property = CssPosition*/
const Value V20774_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_295 = {S17, 0, &V20774_295_57194, &P20774_294};	/*Property = CssText*/
const Value V20774_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_297 = {S18, 0, &V20774_297_57197, &P20774_295};	/*Property = CssTransforms*/
const Value V20774_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_241 = {S19, 0, &V20774_241_55801, &P20774_297};	/*Property = CssTransitions*/
const Value V20774_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_298 = {S20, 0, &V20774_298_57208, &P20774_241};	/*Property = CssUI*/
const Value V20774_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_300 = {S21, 0, &V20774_300_57217, &P20774_298};	/*Property = DataSet*/
const Value V20774_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_301 = {S22, 0, &V20774_301_57220, &P20774_300};	/*Property = DataUrl*/
const Value V20774_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_302 = {S23, 0, &V20774_302_57223, &P20774_301};	/*Property = DeviceOrientation*/
const Value V20774_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_304 = {S24, 0, &V20774_304_57229, &P20774_302};	/*Property = FileReader*/
const Value V20774_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_305 = {S25, 0, &V20774_305_57232, &P20774_304};	/*Property = FileSaver*/
const Value V20774_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_306 = {S26, 0, &V20774_306_57238, &P20774_305};	/*Property = FileWriter*/
const Value V20774_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_307 = {S27, 0, &V20774_307_57241, &P20774_306};	/*Property = FormData*/
const Value V20774_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_309 = {S28, 0, &V20774_309_57247, &P20774_307};	/*Property = Fullscreen*/
const Value V20774_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_310 = {S29, 0, &V20774_310_57250, &P20774_309};	/*Property = GeoLocation*/
const Value V20774_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_312 = {S30, 0, &V20774_312_57256, &P20774_310};	/*Property = History*/
const Value V20774_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_314 = {S31, 0, &V20774_314_57262, &P20774_312};	/*Property = Html5*/
const Value V20774_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_313 = {S32, 0, &V20774_313_57259, &P20774_314};	/*Property = Html-Media-Capture*/
const Value V20774_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_315 = {S33, 0, &V20774_315_57265, &P20774_313};	/*Property = Iframe*/
const Value V20774_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_316 = {S34, 0, &V20774_316_57268, &P20774_315};	/*Property = IndexedDB*/
const Value V20774_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_317 = {S35, 0, &V20774_317_57271, &P20774_316};	/*Property = Json*/
const Value V20774_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_15 = {S37, 0, &V20774_15_72, &P20774_317};	/*Property = LayoutEngine*/
const Value V20774_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_344 = {S38, 0, &V20774_344_57274, &P20774_15};	/*Property = Masking*/
const Value V20774_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_320 = {S39, 0, &V20774_320_57292, &P20774_344};	/*Property = PostMessage*/
const Value V20774_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_321 = {S40, 0, &V20774_321_57295, &P20774_320};	/*Property = Progress*/
const Value V20774_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_322 = {S41, 0, &V20774_322_57298, &P20774_321};	/*Property = Prompts*/
const Value V20774_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_323 = {S42, 0, &V20774_323_57301, &P20774_322};	/*Property = Selector*/
const Value V20774_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_324 = {S43, 0, &V20774_324_57307, &P20774_323};	/*Property = Svg*/
const Value V20774_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_327 = {S44, 0, &V20774_327_57316, &P20774_324};	/*Property = TouchEvents*/
const Value V20774_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_328 = {S45, 0, &V20774_328_57319, &P20774_327};	/*Property = Track*/
const Value V20774_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_329 = {S46, 0, &V20774_329_57325, &P20774_328};	/*Property = Video*/
const Value V20774_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_330 = {S47, 0, &V20774_330_57328, &P20774_329};	/*Property = Viewport*/
const Value V20774_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_332 = {S48, 0, &V20774_332_57340, &P20774_330};	/*Property = WebWorkers*/
const Value V20774_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20774_333 = {S49, 0, &V20774_333_57343, &P20774_332};	/*Property = Xhr2*/
const Profile P20774 = {"20774", &P20774_333};	/*Profile ID - 20774*/
const Value V18256_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18256_279 = {S1, 0, &V18256_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18256_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18256_282 = {S3, 0, &V18256_282_57154, &P18256_279};	/*Property = BlobBuilder*/
const Value V18256_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18256_350 = {S4, 0, &V18256_350_57444, &P18256_282};	/*Property = Canvas*/
const Value V18256_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18256_283 = {S5, 0, &V18256_283_57159, &P18256_350};	/*Property = CssBackground*/
const Value V18256_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18256_284 = {S6, 0, &V18256_284_57163, &P18256_283};	/*Property = CssBorderImage*/
const Value V18256_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18256_285 = {S7, 0, &V18256_285_57166, &P18256_284};	/*Property = CssCanvas*/
const Value V18256_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18256_286 = {S8, 0, &V18256_286_57168, &P18256_285};	/*Property = CssColor*/
const Value V18256_237_55784 = {S2, NULL};	/*Value = True*/
const Property P18256_237 = {S9, 0, &V18256_237_55784, &P18256_286};	/*Property = CssColumn*/
const Value V18256_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18256_287 = {S10, 0, &V18256_287_57172, &P18256_237};	/*Property = CssFlexbox*/
const Value V18256_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18256_288 = {S11, 0, &V18256_288_57175, &P18256_287};	/*Property = CssFont*/
const Value V18256_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18256_289 = {S12, 0, &V18256_289_57759, &P18256_288};	/*Property = CssImages*/
const Value V18256_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18256_291 = {S13, 0, &V18256_291_57181, &P18256_289};	/*Property = CssMediaQueries*/
const Value V18256_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18256_292 = {S14, 0, &V18256_292_57183, &P18256_291};	/*Property = CssMinMax*/
const Value V18256_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18256_293 = {S15, 0, &V18256_293_57187, &P18256_292};	/*Property = CssOverflow*/
const Value V18256_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18256_294 = {S16, 0, &V18256_294_57190, &P18256_293};	/*Property = CssPosition*/
const Value V18256_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18256_295 = {S17, 0, &V18256_295_57193, &P18256_294};	/*Property = CssText*/
const Value V18256_297_57196 = {S0, NULL};	/*Value = False*/
const Property P18256_297 = {S18, 0, &V18256_297_57196, &P18256_295};	/*Property = CssTransforms*/
const Value V18256_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18256_241 = {S19, 0, &V18256_241_55799, &P18256_297};	/*Property = CssTransitions*/
const Value V18256_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18256_298 = {S20, 0, &V18256_298_57206, &P18256_241};	/*Property = CssUI*/
const Value V18256_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18256_300 = {S21, 0, &V18256_300_57216, &P18256_298};	/*Property = DataSet*/
const Value V18256_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18256_301 = {S22, 0, &V18256_301_57218, &P18256_300};	/*Property = DataUrl*/
const Value V18256_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18256_302 = {S23, 0, &V18256_302_57222, &P18256_301};	/*Property = DeviceOrientation*/
const Value V18256_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18256_304 = {S24, 0, &V18256_304_57228, &P18256_302};	/*Property = FileReader*/
const Value V18256_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18256_305 = {S25, 0, &V18256_305_57231, &P18256_304};	/*Property = FileSaver*/
const Value V18256_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18256_306 = {S26, 0, &V18256_306_57237, &P18256_305};	/*Property = FileWriter*/
const Value V18256_307_57240 = {S0, NULL};	/*Value = False*/
const Property P18256_307 = {S27, 0, &V18256_307_57240, &P18256_306};	/*Property = FormData*/
const Value V18256_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18256_309 = {S28, 0, &V18256_309_57246, &P18256_307};	/*Property = Fullscreen*/
const Value V18256_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18256_310 = {S29, 0, &V18256_310_57248, &P18256_309};	/*Property = GeoLocation*/
const Value V18256_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18256_312 = {S30, 0, &V18256_312_57255, &P18256_310};	/*Property = History*/
const Value V18256_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18256_314 = {S31, 0, &V18256_314_57261, &P18256_312};	/*Property = Html5*/
const Value V18256_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18256_313 = {S32, 0, &V18256_313_57258, &P18256_314};	/*Property = Html-Media-Capture*/
const Value V18256_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18256_315 = {S33, 0, &V18256_315_57264, &P18256_313};	/*Property = Iframe*/
const Value V18256_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18256_316 = {S34, 0, &V18256_316_57267, &P18256_315};	/*Property = IndexedDB*/
const Value V18256_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18256_317 = {S35, 0, &V18256_317_57269, &P18256_316};	/*Property = Json*/
const Value V18256_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18256_15 = {S37, 0, &V18256_15_31, &P18256_317};	/*Property = LayoutEngine*/
const Value V18256_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18256_344 = {S38, 0, &V18256_344_57272, &P18256_15};	/*Property = Masking*/
const Value V18256_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18256_320 = {S39, 0, &V18256_320_57290, &P18256_344};	/*Property = PostMessage*/
const Value V18256_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18256_321 = {S40, 0, &V18256_321_57293, &P18256_320};	/*Property = Progress*/
const Value V18256_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18256_322 = {S41, 0, &V18256_322_57296, &P18256_321};	/*Property = Prompts*/
const Value V18256_323_57300 = {S0, NULL};	/*Value = False*/
const Property P18256_323 = {S42, 0, &V18256_323_57300, &P18256_322};	/*Property = Selector*/
const Value V18256_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18256_324 = {S43, 0, &V18256_324_57306, &P18256_323};	/*Property = Svg*/
const Value V18256_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18256_327 = {S44, 0, &V18256_327_57315, &P18256_324};	/*Property = TouchEvents*/
const Value V18256_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18256_328 = {S45, 0, &V18256_328_57318, &P18256_327};	/*Property = Track*/
const Value V18256_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18256_329 = {S46, 0, &V18256_329_57323, &P18256_328};	/*Property = Video*/
const Value V18256_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18256_330 = {S47, 0, &V18256_330_57326, &P18256_329};	/*Property = Viewport*/
const Value V18256_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18256_332 = {S48, 0, &V18256_332_57339, &P18256_330};	/*Property = WebWorkers*/
const Value V18256_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18256_333 = {S49, 0, &V18256_333_57342, &P18256_332};	/*Property = Xhr2*/
const Profile P18256 = {"18256", &P18256_333};	/*Profile ID - 18256*/
const Value V18534_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_279 = {S1, 0, &V18534_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18534_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_282 = {S3, 0, &V18534_282_57155, &P18534_279};	/*Property = BlobBuilder*/
const Value V18534_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_350 = {S4, 0, &V18534_350_57446, &P18534_282};	/*Property = Canvas*/
const Value V18534_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_283 = {S5, 0, &V18534_283_57161, &P18534_350};	/*Property = CssBackground*/
const Value V18534_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_284 = {S6, 0, &V18534_284_57164, &P18534_283};	/*Property = CssBorderImage*/
const Value V18534_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_285 = {S7, 0, &V18534_285_57167, &P18534_284};	/*Property = CssCanvas*/
const Value V18534_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_286 = {S8, 0, &V18534_286_57170, &P18534_285};	/*Property = CssColor*/
const Value V18534_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_237 = {S9, 0, &V18534_237_55786, &P18534_286};	/*Property = CssColumn*/
const Value V18534_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_287 = {S10, 0, &V18534_287_57173, &P18534_237};	/*Property = CssFlexbox*/
const Value V18534_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_288 = {S11, 0, &V18534_288_57176, &P18534_287};	/*Property = CssFont*/
const Value V18534_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_289 = {S12, 0, &V18534_289_86899, &P18534_288};	/*Property = CssImages*/
const Value V18534_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_291 = {S13, 0, &V18534_291_57182, &P18534_289};	/*Property = CssMediaQueries*/
const Value V18534_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_292 = {S14, 0, &V18534_292_57185, &P18534_291};	/*Property = CssMinMax*/
const Value V18534_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_293 = {S15, 0, &V18534_293_57188, &P18534_292};	/*Property = CssOverflow*/
const Value V18534_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_294 = {S16, 0, &V18534_294_57191, &P18534_293};	/*Property = CssPosition*/
const Value V18534_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_295 = {S17, 0, &V18534_295_57194, &P18534_294};	/*Property = CssText*/
const Value V18534_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_297 = {S18, 0, &V18534_297_57197, &P18534_295};	/*Property = CssTransforms*/
const Value V18534_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_241 = {S19, 0, &V18534_241_55801, &P18534_297};	/*Property = CssTransitions*/
const Value V18534_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_298 = {S20, 0, &V18534_298_57208, &P18534_241};	/*Property = CssUI*/
const Value V18534_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_300 = {S21, 0, &V18534_300_57217, &P18534_298};	/*Property = DataSet*/
const Value V18534_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_301 = {S22, 0, &V18534_301_57220, &P18534_300};	/*Property = DataUrl*/
const Value V18534_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_302 = {S23, 0, &V18534_302_57223, &P18534_301};	/*Property = DeviceOrientation*/
const Value V18534_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_304 = {S24, 0, &V18534_304_57229, &P18534_302};	/*Property = FileReader*/
const Value V18534_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_305 = {S25, 0, &V18534_305_57232, &P18534_304};	/*Property = FileSaver*/
const Value V18534_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_306 = {S26, 0, &V18534_306_57238, &P18534_305};	/*Property = FileWriter*/
const Value V18534_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_307 = {S27, 0, &V18534_307_57241, &P18534_306};	/*Property = FormData*/
const Value V18534_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_309 = {S28, 0, &V18534_309_57247, &P18534_307};	/*Property = Fullscreen*/
const Value V18534_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_310 = {S29, 0, &V18534_310_57250, &P18534_309};	/*Property = GeoLocation*/
const Value V18534_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_312 = {S30, 0, &V18534_312_57256, &P18534_310};	/*Property = History*/
const Value V18534_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_314 = {S31, 0, &V18534_314_57262, &P18534_312};	/*Property = Html5*/
const Value V18534_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_313 = {S32, 0, &V18534_313_57259, &P18534_314};	/*Property = Html-Media-Capture*/
const Value V18534_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_315 = {S33, 0, &V18534_315_57265, &P18534_313};	/*Property = Iframe*/
const Value V18534_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_316 = {S34, 0, &V18534_316_57268, &P18534_315};	/*Property = IndexedDB*/
const Value V18534_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_317 = {S35, 0, &V18534_317_57271, &P18534_316};	/*Property = Json*/
const Value V18534_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18534_15 = {S37, 0, &V18534_15_28, &P18534_317};	/*Property = LayoutEngine*/
const Value V18534_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_344 = {S38, 0, &V18534_344_57274, &P18534_15};	/*Property = Masking*/
const Value V18534_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_320 = {S39, 0, &V18534_320_57292, &P18534_344};	/*Property = PostMessage*/
const Value V18534_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_321 = {S40, 0, &V18534_321_57295, &P18534_320};	/*Property = Progress*/
const Value V18534_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_322 = {S41, 0, &V18534_322_57298, &P18534_321};	/*Property = Prompts*/
const Value V18534_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_323 = {S42, 0, &V18534_323_57301, &P18534_322};	/*Property = Selector*/
const Value V18534_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_324 = {S43, 0, &V18534_324_57307, &P18534_323};	/*Property = Svg*/
const Value V18534_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_327 = {S44, 0, &V18534_327_57316, &P18534_324};	/*Property = TouchEvents*/
const Value V18534_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_328 = {S45, 0, &V18534_328_57319, &P18534_327};	/*Property = Track*/
const Value V18534_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_329 = {S46, 0, &V18534_329_57325, &P18534_328};	/*Property = Video*/
const Value V18534_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_330 = {S47, 0, &V18534_330_57328, &P18534_329};	/*Property = Viewport*/
const Value V18534_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_332 = {S48, 0, &V18534_332_57340, &P18534_330};	/*Property = WebWorkers*/
const Value V18534_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18534_333 = {S49, 0, &V18534_333_57343, &P18534_332};	/*Property = Xhr2*/
const Profile P18534 = {"18534", &P18534_333};	/*Profile ID - 18534*/
const Value V22426_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22426_279 = {S1, 0, &V22426_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22426_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22426_282 = {S3, 0, &V22426_282_57153, &P22426_279};	/*Property = BlobBuilder*/
const Value V22426_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22426_350 = {S4, 0, &V22426_350_57444, &P22426_282};	/*Property = Canvas*/
const Value V22426_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22426_283 = {S5, 0, &V22426_283_57159, &P22426_350};	/*Property = CssBackground*/
const Value V22426_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22426_284 = {S6, 0, &V22426_284_57163, &P22426_283};	/*Property = CssBorderImage*/
const Value V22426_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22426_285 = {S7, 0, &V22426_285_57166, &P22426_284};	/*Property = CssCanvas*/
const Value V22426_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22426_286 = {S8, 0, &V22426_286_57168, &P22426_285};	/*Property = CssColor*/
const Value V22426_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22426_237 = {S9, 0, &V22426_237_55784, &P22426_286};	/*Property = CssColumn*/
const Value V22426_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22426_287 = {S10, 0, &V22426_287_57172, &P22426_237};	/*Property = CssFlexbox*/
const Value V22426_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22426_288 = {S11, 0, &V22426_288_57175, &P22426_287};	/*Property = CssFont*/
const Value V22426_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22426_289 = {S12, 0, &V22426_289_57759, &P22426_288};	/*Property = CssImages*/
const Value V22426_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22426_291 = {S13, 0, &V22426_291_57180, &P22426_289};	/*Property = CssMediaQueries*/
const Value V22426_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22426_292 = {S14, 0, &V22426_292_57183, &P22426_291};	/*Property = CssMinMax*/
const Value V22426_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22426_293 = {S15, 0, &V22426_293_57187, &P22426_292};	/*Property = CssOverflow*/
const Value V22426_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22426_294 = {S16, 0, &V22426_294_57189, &P22426_293};	/*Property = CssPosition*/
const Value V22426_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22426_295 = {S17, 0, &V22426_295_57192, &P22426_294};	/*Property = CssText*/
const Value V22426_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22426_297 = {S18, 0, &V22426_297_57195, &P22426_295};	/*Property = CssTransforms*/
const Value V22426_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22426_241 = {S19, 0, &V22426_241_55799, &P22426_297};	/*Property = CssTransitions*/
const Value V22426_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22426_298 = {S20, 0, &V22426_298_57206, &P22426_241};	/*Property = CssUI*/
const Value V22426_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22426_300 = {S21, 0, &V22426_300_57216, &P22426_298};	/*Property = DataSet*/
const Value V22426_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22426_301 = {S22, 0, &V22426_301_57218, &P22426_300};	/*Property = DataUrl*/
const Value V22426_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22426_302 = {S23, 0, &V22426_302_57222, &P22426_301};	/*Property = DeviceOrientation*/
const Value V22426_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22426_304 = {S24, 0, &V22426_304_57227, &P22426_302};	/*Property = FileReader*/
const Value V22426_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22426_305 = {S25, 0, &V22426_305_57231, &P22426_304};	/*Property = FileSaver*/
const Value V22426_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22426_306 = {S26, 0, &V22426_306_57237, &P22426_305};	/*Property = FileWriter*/
const Value V22426_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22426_307 = {S27, 0, &V22426_307_57239, &P22426_306};	/*Property = FormData*/
const Value V22426_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22426_309 = {S28, 0, &V22426_309_57246, &P22426_307};	/*Property = Fullscreen*/
const Value V22426_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22426_310 = {S29, 0, &V22426_310_57248, &P22426_309};	/*Property = GeoLocation*/
const Value V22426_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22426_312 = {S30, 0, &V22426_312_57254, &P22426_310};	/*Property = History*/
const Value V22426_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22426_314 = {S31, 0, &V22426_314_57261, &P22426_312};	/*Property = Html5*/
const Value V22426_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22426_313 = {S32, 0, &V22426_313_57258, &P22426_314};	/*Property = Html-Media-Capture*/
const Value V22426_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22426_315 = {S33, 0, &V22426_315_57264, &P22426_313};	/*Property = Iframe*/
const Value V22426_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22426_316 = {S34, 0, &V22426_316_57266, &P22426_315};	/*Property = IndexedDB*/
const Value V22426_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22426_317 = {S35, 0, &V22426_317_57269, &P22426_316};	/*Property = Json*/
const Value V22426_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22426_15 = {S37, 0, &V22426_15_28, &P22426_317};	/*Property = LayoutEngine*/
const Value V22426_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22426_344 = {S38, 0, &V22426_344_57272, &P22426_15};	/*Property = Masking*/
const Value V22426_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22426_320 = {S39, 0, &V22426_320_57290, &P22426_344};	/*Property = PostMessage*/
const Value V22426_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22426_321 = {S40, 0, &V22426_321_57293, &P22426_320};	/*Property = Progress*/
const Value V22426_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22426_322 = {S41, 0, &V22426_322_57296, &P22426_321};	/*Property = Prompts*/
const Value V22426_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22426_323 = {S42, 0, &V22426_323_57299, &P22426_322};	/*Property = Selector*/
const Value V22426_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22426_324 = {S43, 0, &V22426_324_57306, &P22426_323};	/*Property = Svg*/
const Value V22426_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22426_327 = {S44, 0, &V22426_327_57315, &P22426_324};	/*Property = TouchEvents*/
const Value V22426_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22426_328 = {S45, 0, &V22426_328_57318, &P22426_327};	/*Property = Track*/
const Value V22426_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22426_329 = {S46, 0, &V22426_329_57323, &P22426_328};	/*Property = Video*/
const Value V22426_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22426_330 = {S47, 0, &V22426_330_57326, &P22426_329};	/*Property = Viewport*/
const Value V22426_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22426_332 = {S48, 0, &V22426_332_57338, &P22426_330};	/*Property = WebWorkers*/
const Value V22426_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22426_333 = {S49, 0, &V22426_333_57342, &P22426_332};	/*Property = Xhr2*/
const Profile P22426 = {"22426", &P22426_333};	/*Profile ID - 22426*/
const Value V24195_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24195_279 = {S1, 0, &V24195_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24195_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24195_282 = {S3, 0, &V24195_282_57154, &P24195_279};	/*Property = BlobBuilder*/
const Value V24195_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24195_350 = {S4, 0, &V24195_350_57444, &P24195_282};	/*Property = Canvas*/
const Value V24195_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24195_283 = {S5, 0, &V24195_283_57159, &P24195_350};	/*Property = CssBackground*/
const Value V24195_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24195_284 = {S6, 0, &V24195_284_57162, &P24195_283};	/*Property = CssBorderImage*/
const Value V24195_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24195_285 = {S7, 0, &V24195_285_57166, &P24195_284};	/*Property = CssCanvas*/
const Value V24195_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24195_286 = {S8, 0, &V24195_286_57168, &P24195_285};	/*Property = CssColor*/
const Value V24195_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24195_237 = {S9, 0, &V24195_237_55786, &P24195_286};	/*Property = CssColumn*/
const Value V24195_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24195_287 = {S10, 0, &V24195_287_57172, &P24195_237};	/*Property = CssFlexbox*/
const Value V24195_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24195_288 = {S11, 0, &V24195_288_57175, &P24195_287};	/*Property = CssFont*/
const Value V24195_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24195_289 = {S12, 0, &V24195_289_57759, &P24195_288};	/*Property = CssImages*/
const Value V24195_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24195_291 = {S13, 0, &V24195_291_57180, &P24195_289};	/*Property = CssMediaQueries*/
const Value V24195_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24195_292 = {S14, 0, &V24195_292_57183, &P24195_291};	/*Property = CssMinMax*/
const Value V24195_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24195_293 = {S15, 0, &V24195_293_57187, &P24195_292};	/*Property = CssOverflow*/
const Value V24195_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24195_294 = {S16, 0, &V24195_294_57189, &P24195_293};	/*Property = CssPosition*/
const Value V24195_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24195_295 = {S17, 0, &V24195_295_57192, &P24195_294};	/*Property = CssText*/
const Value V24195_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24195_297 = {S18, 0, &V24195_297_57195, &P24195_295};	/*Property = CssTransforms*/
const Value V24195_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24195_241 = {S19, 0, &V24195_241_55799, &P24195_297};	/*Property = CssTransitions*/
const Value V24195_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24195_298 = {S20, 0, &V24195_298_57206, &P24195_241};	/*Property = CssUI*/
const Value V24195_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24195_300 = {S21, 0, &V24195_300_57215, &P24195_298};	/*Property = DataSet*/
const Value V24195_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24195_301 = {S22, 0, &V24195_301_57218, &P24195_300};	/*Property = DataUrl*/
const Value V24195_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24195_302 = {S23, 0, &V24195_302_57221, &P24195_301};	/*Property = DeviceOrientation*/
const Value V24195_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24195_304 = {S24, 0, &V24195_304_57227, &P24195_302};	/*Property = FileReader*/
const Value V24195_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24195_305 = {S25, 0, &V24195_305_57231, &P24195_304};	/*Property = FileSaver*/
const Value V24195_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24195_306 = {S26, 0, &V24195_306_57237, &P24195_305};	/*Property = FileWriter*/
const Value V24195_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24195_307 = {S27, 0, &V24195_307_57239, &P24195_306};	/*Property = FormData*/
const Value V24195_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24195_309 = {S28, 0, &V24195_309_57246, &P24195_307};	/*Property = Fullscreen*/
const Value V24195_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24195_310 = {S29, 0, &V24195_310_57248, &P24195_309};	/*Property = GeoLocation*/
const Value V24195_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24195_312 = {S30, 0, &V24195_312_57254, &P24195_310};	/*Property = History*/
const Value V24195_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24195_314 = {S31, 0, &V24195_314_57260, &P24195_312};	/*Property = Html5*/
const Value V24195_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24195_313 = {S32, 0, &V24195_313_57257, &P24195_314};	/*Property = Html-Media-Capture*/
const Value V24195_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24195_315 = {S33, 0, &V24195_315_57264, &P24195_313};	/*Property = Iframe*/
const Value V24195_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24195_316 = {S34, 0, &V24195_316_57266, &P24195_315};	/*Property = IndexedDB*/
const Value V24195_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24195_317 = {S35, 0, &V24195_317_57269, &P24195_316};	/*Property = Json*/
const Value V24195_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24195_15 = {S37, 0, &V24195_15_54167, &P24195_317};	/*Property = LayoutEngine*/
const Value V24195_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24195_344 = {S38, 0, &V24195_344_57272, &P24195_15};	/*Property = Masking*/
const Value V24195_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24195_320 = {S39, 0, &V24195_320_57290, &P24195_344};	/*Property = PostMessage*/
const Value V24195_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24195_321 = {S40, 0, &V24195_321_57293, &P24195_320};	/*Property = Progress*/
const Value V24195_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24195_322 = {S41, 0, &V24195_322_57296, &P24195_321};	/*Property = Prompts*/
const Value V24195_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24195_323 = {S42, 0, &V24195_323_57299, &P24195_322};	/*Property = Selector*/
const Value V24195_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24195_324 = {S43, 0, &V24195_324_57306, &P24195_323};	/*Property = Svg*/
const Value V24195_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24195_327 = {S44, 0, &V24195_327_57314, &P24195_324};	/*Property = TouchEvents*/
const Value V24195_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24195_328 = {S45, 0, &V24195_328_57318, &P24195_327};	/*Property = Track*/
const Value V24195_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24195_329 = {S46, 0, &V24195_329_57323, &P24195_328};	/*Property = Video*/
const Value V24195_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24195_330 = {S47, 0, &V24195_330_57326, &P24195_329};	/*Property = Viewport*/
const Value V24195_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24195_332 = {S48, 0, &V24195_332_57339, &P24195_330};	/*Property = WebWorkers*/
const Value V24195_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24195_333 = {S49, 0, &V24195_333_57342, &P24195_332};	/*Property = Xhr2*/
const Profile P24195 = {"24195", &P24195_333};	/*Profile ID - 24195*/
const Value V18275_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18275_279 = {S1, 0, &V18275_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18275_282_57153 = {S2, NULL};	/*Value = True*/
const Property P18275_282 = {S3, 0, &V18275_282_57153, &P18275_279};	/*Property = BlobBuilder*/
const Value V18275_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18275_350 = {S4, 0, &V18275_350_57444, &P18275_282};	/*Property = Canvas*/
const Value V18275_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18275_283 = {S5, 0, &V18275_283_57159, &P18275_350};	/*Property = CssBackground*/
const Value V18275_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18275_284 = {S6, 0, &V18275_284_57163, &P18275_283};	/*Property = CssBorderImage*/
const Value V18275_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18275_285 = {S7, 0, &V18275_285_57166, &P18275_284};	/*Property = CssCanvas*/
const Value V18275_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18275_286 = {S8, 0, &V18275_286_57168, &P18275_285};	/*Property = CssColor*/
const Value V18275_237_55784 = {S2, NULL};	/*Value = True*/
const Property P18275_237 = {S9, 0, &V18275_237_55784, &P18275_286};	/*Property = CssColumn*/
const Value V18275_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18275_287 = {S10, 0, &V18275_287_57172, &P18275_237};	/*Property = CssFlexbox*/
const Value V18275_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18275_288 = {S11, 0, &V18275_288_57175, &P18275_287};	/*Property = CssFont*/
const Value V18275_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18275_289 = {S12, 0, &V18275_289_57759, &P18275_288};	/*Property = CssImages*/
const Value V18275_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18275_291 = {S13, 0, &V18275_291_57180, &P18275_289};	/*Property = CssMediaQueries*/
const Value V18275_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18275_292 = {S14, 0, &V18275_292_57183, &P18275_291};	/*Property = CssMinMax*/
const Value V18275_293_57186 = {S2, NULL};	/*Value = True*/
const Property P18275_293 = {S15, 0, &V18275_293_57186, &P18275_292};	/*Property = CssOverflow*/
const Value V18275_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18275_294 = {S16, 0, &V18275_294_57189, &P18275_293};	/*Property = CssPosition*/
const Value V18275_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18275_295 = {S17, 0, &V18275_295_57192, &P18275_294};	/*Property = CssText*/
const Value V18275_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18275_297 = {S18, 0, &V18275_297_57195, &P18275_295};	/*Property = CssTransforms*/
const Value V18275_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18275_241 = {S19, 0, &V18275_241_55799, &P18275_297};	/*Property = CssTransitions*/
const Value V18275_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18275_298 = {S20, 0, &V18275_298_57206, &P18275_241};	/*Property = CssUI*/
const Value V18275_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18275_300 = {S21, 0, &V18275_300_57216, &P18275_298};	/*Property = DataSet*/
const Value V18275_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18275_301 = {S22, 0, &V18275_301_57218, &P18275_300};	/*Property = DataUrl*/
const Value V18275_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18275_302 = {S23, 0, &V18275_302_57222, &P18275_301};	/*Property = DeviceOrientation*/
const Value V18275_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18275_304 = {S24, 0, &V18275_304_57227, &P18275_302};	/*Property = FileReader*/
const Value V18275_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18275_305 = {S25, 0, &V18275_305_57231, &P18275_304};	/*Property = FileSaver*/
const Value V18275_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18275_306 = {S26, 0, &V18275_306_57237, &P18275_305};	/*Property = FileWriter*/
const Value V18275_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18275_307 = {S27, 0, &V18275_307_57239, &P18275_306};	/*Property = FormData*/
const Value V18275_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18275_309 = {S28, 0, &V18275_309_57246, &P18275_307};	/*Property = Fullscreen*/
const Value V18275_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18275_310 = {S29, 0, &V18275_310_57248, &P18275_309};	/*Property = GeoLocation*/
const Value V18275_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18275_312 = {S30, 0, &V18275_312_57254, &P18275_310};	/*Property = History*/
const Value V18275_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18275_314 = {S31, 0, &V18275_314_57261, &P18275_312};	/*Property = Html5*/
const Value V18275_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18275_313 = {S32, 0, &V18275_313_57258, &P18275_314};	/*Property = Html-Media-Capture*/
const Value V18275_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18275_315 = {S33, 0, &V18275_315_57264, &P18275_313};	/*Property = Iframe*/
const Value V18275_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18275_316 = {S34, 0, &V18275_316_57266, &P18275_315};	/*Property = IndexedDB*/
const Value V18275_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18275_317 = {S35, 0, &V18275_317_57269, &P18275_316};	/*Property = Json*/
const Value V18275_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18275_15 = {S37, 0, &V18275_15_28, &P18275_317};	/*Property = LayoutEngine*/
const Value V18275_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18275_344 = {S38, 0, &V18275_344_57272, &P18275_15};	/*Property = Masking*/
const Value V18275_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18275_320 = {S39, 0, &V18275_320_57290, &P18275_344};	/*Property = PostMessage*/
const Value V18275_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18275_321 = {S40, 0, &V18275_321_57293, &P18275_320};	/*Property = Progress*/
const Value V18275_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18275_322 = {S41, 0, &V18275_322_57296, &P18275_321};	/*Property = Prompts*/
const Value V18275_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18275_323 = {S42, 0, &V18275_323_57299, &P18275_322};	/*Property = Selector*/
const Value V18275_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18275_324 = {S43, 0, &V18275_324_57306, &P18275_323};	/*Property = Svg*/
const Value V18275_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18275_327 = {S44, 0, &V18275_327_57315, &P18275_324};	/*Property = TouchEvents*/
const Value V18275_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18275_328 = {S45, 0, &V18275_328_57318, &P18275_327};	/*Property = Track*/
const Value V18275_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18275_329 = {S46, 0, &V18275_329_57323, &P18275_328};	/*Property = Video*/
const Value V18275_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18275_330 = {S47, 0, &V18275_330_57326, &P18275_329};	/*Property = Viewport*/
const Value V18275_332_57338 = {S2, NULL};	/*Value = True*/
const Property P18275_332 = {S48, 0, &V18275_332_57338, &P18275_330};	/*Property = WebWorkers*/
const Value V18275_333_57341 = {S2, NULL};	/*Value = True*/
const Property P18275_333 = {S49, 0, &V18275_333_57341, &P18275_332};	/*Property = Xhr2*/
const Profile P18275 = {"18275", &P18275_333};	/*Profile ID - 18275*/
const Value V18321_1_1 = {S2, NULL};	/*Value = True*/
const Property P18321_1 = {S50, 0, &V18321_1_1, NULL};	/*Property = IsMobile*/
const Value V18321_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18321_5 = {S52, 0, &V18321_5_1222, &P18321_1};	/*Property = ScreenPixelsHeight*/
const Value V18321_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18321_4 = {S54, 0, &V18321_4_1224, &P18321_5};	/*Property = ScreenPixelsWidth*/
const Profile P18321 = {"18321", &P18321_4};	/*Profile ID - 18321*/
const Value V18197_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_279 = {S1, 0, &V18197_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18197_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_282 = {S3, 0, &V18197_282_57155, &P18197_279};	/*Property = BlobBuilder*/
const Value V18197_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_350 = {S4, 0, &V18197_350_57446, &P18197_282};	/*Property = Canvas*/
const Value V18197_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_283 = {S5, 0, &V18197_283_57161, &P18197_350};	/*Property = CssBackground*/
const Value V18197_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_284 = {S6, 0, &V18197_284_57164, &P18197_283};	/*Property = CssBorderImage*/
const Value V18197_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_285 = {S7, 0, &V18197_285_57167, &P18197_284};	/*Property = CssCanvas*/
const Value V18197_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_286 = {S8, 0, &V18197_286_57170, &P18197_285};	/*Property = CssColor*/
const Value V18197_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_237 = {S9, 0, &V18197_237_55786, &P18197_286};	/*Property = CssColumn*/
const Value V18197_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_287 = {S10, 0, &V18197_287_57173, &P18197_237};	/*Property = CssFlexbox*/
const Value V18197_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_288 = {S11, 0, &V18197_288_57176, &P18197_287};	/*Property = CssFont*/
const Value V18197_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_289 = {S12, 0, &V18197_289_86899, &P18197_288};	/*Property = CssImages*/
const Value V18197_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_291 = {S13, 0, &V18197_291_57182, &P18197_289};	/*Property = CssMediaQueries*/
const Value V18197_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_292 = {S14, 0, &V18197_292_57185, &P18197_291};	/*Property = CssMinMax*/
const Value V18197_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_293 = {S15, 0, &V18197_293_57188, &P18197_292};	/*Property = CssOverflow*/
const Value V18197_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_294 = {S16, 0, &V18197_294_57191, &P18197_293};	/*Property = CssPosition*/
const Value V18197_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_295 = {S17, 0, &V18197_295_57194, &P18197_294};	/*Property = CssText*/
const Value V18197_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_297 = {S18, 0, &V18197_297_57197, &P18197_295};	/*Property = CssTransforms*/
const Value V18197_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_241 = {S19, 0, &V18197_241_55801, &P18197_297};	/*Property = CssTransitions*/
const Value V18197_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_298 = {S20, 0, &V18197_298_57208, &P18197_241};	/*Property = CssUI*/
const Value V18197_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_300 = {S21, 0, &V18197_300_57217, &P18197_298};	/*Property = DataSet*/
const Value V18197_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_301 = {S22, 0, &V18197_301_57220, &P18197_300};	/*Property = DataUrl*/
const Value V18197_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_302 = {S23, 0, &V18197_302_57223, &P18197_301};	/*Property = DeviceOrientation*/
const Value V18197_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_304 = {S24, 0, &V18197_304_57229, &P18197_302};	/*Property = FileReader*/
const Value V18197_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_305 = {S25, 0, &V18197_305_57232, &P18197_304};	/*Property = FileSaver*/
const Value V18197_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_306 = {S26, 0, &V18197_306_57238, &P18197_305};	/*Property = FileWriter*/
const Value V18197_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_307 = {S27, 0, &V18197_307_57241, &P18197_306};	/*Property = FormData*/
const Value V18197_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_309 = {S28, 0, &V18197_309_57247, &P18197_307};	/*Property = Fullscreen*/
const Value V18197_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_310 = {S29, 0, &V18197_310_57250, &P18197_309};	/*Property = GeoLocation*/
const Value V18197_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_312 = {S30, 0, &V18197_312_57256, &P18197_310};	/*Property = History*/
const Value V18197_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_314 = {S31, 0, &V18197_314_57262, &P18197_312};	/*Property = Html5*/
const Value V18197_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_313 = {S32, 0, &V18197_313_57259, &P18197_314};	/*Property = Html-Media-Capture*/
const Value V18197_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_315 = {S33, 0, &V18197_315_57265, &P18197_313};	/*Property = Iframe*/
const Value V18197_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_316 = {S34, 0, &V18197_316_57268, &P18197_315};	/*Property = IndexedDB*/
const Value V18197_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_317 = {S35, 0, &V18197_317_57271, &P18197_316};	/*Property = Json*/
const Value V18197_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18197_15 = {S37, 0, &V18197_15_29, &P18197_317};	/*Property = LayoutEngine*/
const Value V18197_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_344 = {S38, 0, &V18197_344_57274, &P18197_15};	/*Property = Masking*/
const Value V18197_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_320 = {S39, 0, &V18197_320_57292, &P18197_344};	/*Property = PostMessage*/
const Value V18197_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_321 = {S40, 0, &V18197_321_57295, &P18197_320};	/*Property = Progress*/
const Value V18197_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_322 = {S41, 0, &V18197_322_57298, &P18197_321};	/*Property = Prompts*/
const Value V18197_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_323 = {S42, 0, &V18197_323_57301, &P18197_322};	/*Property = Selector*/
const Value V18197_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_324 = {S43, 0, &V18197_324_57307, &P18197_323};	/*Property = Svg*/
const Value V18197_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_327 = {S44, 0, &V18197_327_57316, &P18197_324};	/*Property = TouchEvents*/
const Value V18197_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_328 = {S45, 0, &V18197_328_57319, &P18197_327};	/*Property = Track*/
const Value V18197_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_329 = {S46, 0, &V18197_329_57325, &P18197_328};	/*Property = Video*/
const Value V18197_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_330 = {S47, 0, &V18197_330_57328, &P18197_329};	/*Property = Viewport*/
const Value V18197_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_332 = {S48, 0, &V18197_332_57340, &P18197_330};	/*Property = WebWorkers*/
const Value V18197_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18197_333 = {S49, 0, &V18197_333_57343, &P18197_332};	/*Property = Xhr2*/
const Profile P18197 = {"18197", &P18197_333};	/*Profile ID - 18197*/
const Profile P22187 = {"22187", NULL};	/*Profile ID - 22187*/
const Value V19772_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_279 = {S1, 0, &V19772_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V19772_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_282 = {S3, 0, &V19772_282_57155, &P19772_279};	/*Property = BlobBuilder*/
const Value V19772_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_350 = {S4, 0, &V19772_350_57446, &P19772_282};	/*Property = Canvas*/
const Value V19772_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_283 = {S5, 0, &V19772_283_57161, &P19772_350};	/*Property = CssBackground*/
const Value V19772_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_284 = {S6, 0, &V19772_284_57164, &P19772_283};	/*Property = CssBorderImage*/
const Value V19772_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_285 = {S7, 0, &V19772_285_57167, &P19772_284};	/*Property = CssCanvas*/
const Value V19772_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_286 = {S8, 0, &V19772_286_57170, &P19772_285};	/*Property = CssColor*/
const Value V19772_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_237 = {S9, 0, &V19772_237_55786, &P19772_286};	/*Property = CssColumn*/
const Value V19772_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_287 = {S10, 0, &V19772_287_57173, &P19772_237};	/*Property = CssFlexbox*/
const Value V19772_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_288 = {S11, 0, &V19772_288_57176, &P19772_287};	/*Property = CssFont*/
const Value V19772_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_289 = {S12, 0, &V19772_289_86899, &P19772_288};	/*Property = CssImages*/
const Value V19772_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_291 = {S13, 0, &V19772_291_57182, &P19772_289};	/*Property = CssMediaQueries*/
const Value V19772_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_292 = {S14, 0, &V19772_292_57185, &P19772_291};	/*Property = CssMinMax*/
const Value V19772_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_293 = {S15, 0, &V19772_293_57188, &P19772_292};	/*Property = CssOverflow*/
const Value V19772_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_294 = {S16, 0, &V19772_294_57191, &P19772_293};	/*Property = CssPosition*/
const Value V19772_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_295 = {S17, 0, &V19772_295_57194, &P19772_294};	/*Property = CssText*/
const Value V19772_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_297 = {S18, 0, &V19772_297_57197, &P19772_295};	/*Property = CssTransforms*/
const Value V19772_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_241 = {S19, 0, &V19772_241_55801, &P19772_297};	/*Property = CssTransitions*/
const Value V19772_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_298 = {S20, 0, &V19772_298_57208, &P19772_241};	/*Property = CssUI*/
const Value V19772_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_300 = {S21, 0, &V19772_300_57217, &P19772_298};	/*Property = DataSet*/
const Value V19772_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_301 = {S22, 0, &V19772_301_57220, &P19772_300};	/*Property = DataUrl*/
const Value V19772_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_302 = {S23, 0, &V19772_302_57223, &P19772_301};	/*Property = DeviceOrientation*/
const Value V19772_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_304 = {S24, 0, &V19772_304_57229, &P19772_302};	/*Property = FileReader*/
const Value V19772_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_305 = {S25, 0, &V19772_305_57232, &P19772_304};	/*Property = FileSaver*/
const Value V19772_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_306 = {S26, 0, &V19772_306_57238, &P19772_305};	/*Property = FileWriter*/
const Value V19772_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_307 = {S27, 0, &V19772_307_57241, &P19772_306};	/*Property = FormData*/
const Value V19772_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_309 = {S28, 0, &V19772_309_57247, &P19772_307};	/*Property = Fullscreen*/
const Value V19772_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_310 = {S29, 0, &V19772_310_57250, &P19772_309};	/*Property = GeoLocation*/
const Value V19772_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_312 = {S30, 0, &V19772_312_57256, &P19772_310};	/*Property = History*/
const Value V19772_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_314 = {S31, 0, &V19772_314_57262, &P19772_312};	/*Property = Html5*/
const Value V19772_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_313 = {S32, 0, &V19772_313_57259, &P19772_314};	/*Property = Html-Media-Capture*/
const Value V19772_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_315 = {S33, 0, &V19772_315_57265, &P19772_313};	/*Property = Iframe*/
const Value V19772_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_316 = {S34, 0, &V19772_316_57268, &P19772_315};	/*Property = IndexedDB*/
const Value V19772_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_317 = {S35, 0, &V19772_317_57271, &P19772_316};	/*Property = Json*/
const Value V19772_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P19772_15 = {S37, 0, &V19772_15_29, &P19772_317};	/*Property = LayoutEngine*/
const Value V19772_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_344 = {S38, 0, &V19772_344_57274, &P19772_15};	/*Property = Masking*/
const Value V19772_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_320 = {S39, 0, &V19772_320_57292, &P19772_344};	/*Property = PostMessage*/
const Value V19772_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_321 = {S40, 0, &V19772_321_57295, &P19772_320};	/*Property = Progress*/
const Value V19772_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_322 = {S41, 0, &V19772_322_57298, &P19772_321};	/*Property = Prompts*/
const Value V19772_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_323 = {S42, 0, &V19772_323_57301, &P19772_322};	/*Property = Selector*/
const Value V19772_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_324 = {S43, 0, &V19772_324_57307, &P19772_323};	/*Property = Svg*/
const Value V19772_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_327 = {S44, 0, &V19772_327_57316, &P19772_324};	/*Property = TouchEvents*/
const Value V19772_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_328 = {S45, 0, &V19772_328_57319, &P19772_327};	/*Property = Track*/
const Value V19772_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_329 = {S46, 0, &V19772_329_57325, &P19772_328};	/*Property = Video*/
const Value V19772_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_330 = {S47, 0, &V19772_330_57328, &P19772_329};	/*Property = Viewport*/
const Value V19772_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_332 = {S48, 0, &V19772_332_57340, &P19772_330};	/*Property = WebWorkers*/
const Value V19772_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P19772_333 = {S49, 0, &V19772_333_57343, &P19772_332};	/*Property = Xhr2*/
const Profile P19772 = {"19772", &P19772_333};	/*Profile ID - 19772*/
const Value V20917_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_279 = {S1, 0, &V20917_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20917_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_282 = {S3, 0, &V20917_282_57155, &P20917_279};	/*Property = BlobBuilder*/
const Value V20917_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_350 = {S4, 0, &V20917_350_57446, &P20917_282};	/*Property = Canvas*/
const Value V20917_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_283 = {S5, 0, &V20917_283_57161, &P20917_350};	/*Property = CssBackground*/
const Value V20917_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_284 = {S6, 0, &V20917_284_57164, &P20917_283};	/*Property = CssBorderImage*/
const Value V20917_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_285 = {S7, 0, &V20917_285_57167, &P20917_284};	/*Property = CssCanvas*/
const Value V20917_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_286 = {S8, 0, &V20917_286_57170, &P20917_285};	/*Property = CssColor*/
const Value V20917_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_237 = {S9, 0, &V20917_237_55786, &P20917_286};	/*Property = CssColumn*/
const Value V20917_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_287 = {S10, 0, &V20917_287_57173, &P20917_237};	/*Property = CssFlexbox*/
const Value V20917_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_288 = {S11, 0, &V20917_288_57176, &P20917_287};	/*Property = CssFont*/
const Value V20917_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_289 = {S12, 0, &V20917_289_86899, &P20917_288};	/*Property = CssImages*/
const Value V20917_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_291 = {S13, 0, &V20917_291_57182, &P20917_289};	/*Property = CssMediaQueries*/
const Value V20917_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_292 = {S14, 0, &V20917_292_57185, &P20917_291};	/*Property = CssMinMax*/
const Value V20917_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_293 = {S15, 0, &V20917_293_57188, &P20917_292};	/*Property = CssOverflow*/
const Value V20917_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_294 = {S16, 0, &V20917_294_57191, &P20917_293};	/*Property = CssPosition*/
const Value V20917_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_295 = {S17, 0, &V20917_295_57194, &P20917_294};	/*Property = CssText*/
const Value V20917_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_297 = {S18, 0, &V20917_297_57197, &P20917_295};	/*Property = CssTransforms*/
const Value V20917_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_241 = {S19, 0, &V20917_241_55801, &P20917_297};	/*Property = CssTransitions*/
const Value V20917_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_298 = {S20, 0, &V20917_298_57208, &P20917_241};	/*Property = CssUI*/
const Value V20917_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_300 = {S21, 0, &V20917_300_57217, &P20917_298};	/*Property = DataSet*/
const Value V20917_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_301 = {S22, 0, &V20917_301_57220, &P20917_300};	/*Property = DataUrl*/
const Value V20917_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_302 = {S23, 0, &V20917_302_57223, &P20917_301};	/*Property = DeviceOrientation*/
const Value V20917_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_304 = {S24, 0, &V20917_304_57229, &P20917_302};	/*Property = FileReader*/
const Value V20917_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_305 = {S25, 0, &V20917_305_57232, &P20917_304};	/*Property = FileSaver*/
const Value V20917_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_306 = {S26, 0, &V20917_306_57238, &P20917_305};	/*Property = FileWriter*/
const Value V20917_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_307 = {S27, 0, &V20917_307_57241, &P20917_306};	/*Property = FormData*/
const Value V20917_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_309 = {S28, 0, &V20917_309_57247, &P20917_307};	/*Property = Fullscreen*/
const Value V20917_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_310 = {S29, 0, &V20917_310_57250, &P20917_309};	/*Property = GeoLocation*/
const Value V20917_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_312 = {S30, 0, &V20917_312_57256, &P20917_310};	/*Property = History*/
const Value V20917_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_314 = {S31, 0, &V20917_314_57262, &P20917_312};	/*Property = Html5*/
const Value V20917_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_313 = {S32, 0, &V20917_313_57259, &P20917_314};	/*Property = Html-Media-Capture*/
const Value V20917_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_315 = {S33, 0, &V20917_315_57265, &P20917_313};	/*Property = Iframe*/
const Value V20917_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_316 = {S34, 0, &V20917_316_57268, &P20917_315};	/*Property = IndexedDB*/
const Value V20917_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_317 = {S35, 0, &V20917_317_57271, &P20917_316};	/*Property = Json*/
const Value V20917_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20917_15 = {S37, 0, &V20917_15_29, &P20917_317};	/*Property = LayoutEngine*/
const Value V20917_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_344 = {S38, 0, &V20917_344_57274, &P20917_15};	/*Property = Masking*/
const Value V20917_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_320 = {S39, 0, &V20917_320_57292, &P20917_344};	/*Property = PostMessage*/
const Value V20917_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_321 = {S40, 0, &V20917_321_57295, &P20917_320};	/*Property = Progress*/
const Value V20917_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_322 = {S41, 0, &V20917_322_57298, &P20917_321};	/*Property = Prompts*/
const Value V20917_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_323 = {S42, 0, &V20917_323_57301, &P20917_322};	/*Property = Selector*/
const Value V20917_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_324 = {S43, 0, &V20917_324_57307, &P20917_323};	/*Property = Svg*/
const Value V20917_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_327 = {S44, 0, &V20917_327_57316, &P20917_324};	/*Property = TouchEvents*/
const Value V20917_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_328 = {S45, 0, &V20917_328_57319, &P20917_327};	/*Property = Track*/
const Value V20917_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_329 = {S46, 0, &V20917_329_57325, &P20917_328};	/*Property = Video*/
const Value V20917_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_330 = {S47, 0, &V20917_330_57328, &P20917_329};	/*Property = Viewport*/
const Value V20917_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_332 = {S48, 0, &V20917_332_57340, &P20917_330};	/*Property = WebWorkers*/
const Value V20917_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20917_333 = {S49, 0, &V20917_333_57343, &P20917_332};	/*Property = Xhr2*/
const Profile P20917 = {"20917", &P20917_333};	/*Profile ID - 20917*/
const Profile P18135 = {"18135", NULL};	/*Profile ID - 18135*/
const Profile P22186 = {"22186", NULL};	/*Profile ID - 22186*/
const Value V18198_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_279 = {S1, 0, &V18198_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18198_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_282 = {S3, 0, &V18198_282_57155, &P18198_279};	/*Property = BlobBuilder*/
const Value V18198_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_350 = {S4, 0, &V18198_350_57446, &P18198_282};	/*Property = Canvas*/
const Value V18198_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_283 = {S5, 0, &V18198_283_57161, &P18198_350};	/*Property = CssBackground*/
const Value V18198_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_284 = {S6, 0, &V18198_284_57164, &P18198_283};	/*Property = CssBorderImage*/
const Value V18198_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_285 = {S7, 0, &V18198_285_57167, &P18198_284};	/*Property = CssCanvas*/
const Value V18198_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_286 = {S8, 0, &V18198_286_57170, &P18198_285};	/*Property = CssColor*/
const Value V18198_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_237 = {S9, 0, &V18198_237_55786, &P18198_286};	/*Property = CssColumn*/
const Value V18198_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_287 = {S10, 0, &V18198_287_57173, &P18198_237};	/*Property = CssFlexbox*/
const Value V18198_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_288 = {S11, 0, &V18198_288_57176, &P18198_287};	/*Property = CssFont*/
const Value V18198_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_289 = {S12, 0, &V18198_289_86899, &P18198_288};	/*Property = CssImages*/
const Value V18198_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_291 = {S13, 0, &V18198_291_57182, &P18198_289};	/*Property = CssMediaQueries*/
const Value V18198_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_292 = {S14, 0, &V18198_292_57185, &P18198_291};	/*Property = CssMinMax*/
const Value V18198_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_293 = {S15, 0, &V18198_293_57188, &P18198_292};	/*Property = CssOverflow*/
const Value V18198_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_294 = {S16, 0, &V18198_294_57191, &P18198_293};	/*Property = CssPosition*/
const Value V18198_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_295 = {S17, 0, &V18198_295_57194, &P18198_294};	/*Property = CssText*/
const Value V18198_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_297 = {S18, 0, &V18198_297_57197, &P18198_295};	/*Property = CssTransforms*/
const Value V18198_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_241 = {S19, 0, &V18198_241_55801, &P18198_297};	/*Property = CssTransitions*/
const Value V18198_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_298 = {S20, 0, &V18198_298_57208, &P18198_241};	/*Property = CssUI*/
const Value V18198_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_300 = {S21, 0, &V18198_300_57217, &P18198_298};	/*Property = DataSet*/
const Value V18198_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_301 = {S22, 0, &V18198_301_57220, &P18198_300};	/*Property = DataUrl*/
const Value V18198_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_302 = {S23, 0, &V18198_302_57223, &P18198_301};	/*Property = DeviceOrientation*/
const Value V18198_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_304 = {S24, 0, &V18198_304_57229, &P18198_302};	/*Property = FileReader*/
const Value V18198_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_305 = {S25, 0, &V18198_305_57232, &P18198_304};	/*Property = FileSaver*/
const Value V18198_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_306 = {S26, 0, &V18198_306_57238, &P18198_305};	/*Property = FileWriter*/
const Value V18198_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_307 = {S27, 0, &V18198_307_57241, &P18198_306};	/*Property = FormData*/
const Value V18198_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_309 = {S28, 0, &V18198_309_57247, &P18198_307};	/*Property = Fullscreen*/
const Value V18198_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_310 = {S29, 0, &V18198_310_57250, &P18198_309};	/*Property = GeoLocation*/
const Value V18198_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_312 = {S30, 0, &V18198_312_57256, &P18198_310};	/*Property = History*/
const Value V18198_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_314 = {S31, 0, &V18198_314_57262, &P18198_312};	/*Property = Html5*/
const Value V18198_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_313 = {S32, 0, &V18198_313_57259, &P18198_314};	/*Property = Html-Media-Capture*/
const Value V18198_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_315 = {S33, 0, &V18198_315_57265, &P18198_313};	/*Property = Iframe*/
const Value V18198_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_316 = {S34, 0, &V18198_316_57268, &P18198_315};	/*Property = IndexedDB*/
const Value V18198_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_317 = {S35, 0, &V18198_317_57271, &P18198_316};	/*Property = Json*/
const Value V18198_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18198_15 = {S37, 0, &V18198_15_29, &P18198_317};	/*Property = LayoutEngine*/
const Value V18198_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_344 = {S38, 0, &V18198_344_57274, &P18198_15};	/*Property = Masking*/
const Value V18198_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_320 = {S39, 0, &V18198_320_57292, &P18198_344};	/*Property = PostMessage*/
const Value V18198_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_321 = {S40, 0, &V18198_321_57295, &P18198_320};	/*Property = Progress*/
const Value V18198_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_322 = {S41, 0, &V18198_322_57298, &P18198_321};	/*Property = Prompts*/
const Value V18198_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_323 = {S42, 0, &V18198_323_57301, &P18198_322};	/*Property = Selector*/
const Value V18198_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_324 = {S43, 0, &V18198_324_57307, &P18198_323};	/*Property = Svg*/
const Value V18198_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_327 = {S44, 0, &V18198_327_57316, &P18198_324};	/*Property = TouchEvents*/
const Value V18198_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_328 = {S45, 0, &V18198_328_57319, &P18198_327};	/*Property = Track*/
const Value V18198_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_329 = {S46, 0, &V18198_329_57325, &P18198_328};	/*Property = Video*/
const Value V18198_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_330 = {S47, 0, &V18198_330_57328, &P18198_329};	/*Property = Viewport*/
const Value V18198_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_332 = {S48, 0, &V18198_332_57340, &P18198_330};	/*Property = WebWorkers*/
const Value V18198_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18198_333 = {S49, 0, &V18198_333_57343, &P18198_332};	/*Property = Xhr2*/
const Profile P18198 = {"18198", &P18198_333};	/*Profile ID - 18198*/
const Value V19773_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_279 = {S1, 0, &V19773_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V19773_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_282 = {S3, 0, &V19773_282_57155, &P19773_279};	/*Property = BlobBuilder*/
const Value V19773_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_350 = {S4, 0, &V19773_350_57446, &P19773_282};	/*Property = Canvas*/
const Value V19773_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_283 = {S5, 0, &V19773_283_57161, &P19773_350};	/*Property = CssBackground*/
const Value V19773_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_284 = {S6, 0, &V19773_284_57164, &P19773_283};	/*Property = CssBorderImage*/
const Value V19773_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_285 = {S7, 0, &V19773_285_57167, &P19773_284};	/*Property = CssCanvas*/
const Value V19773_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_286 = {S8, 0, &V19773_286_57170, &P19773_285};	/*Property = CssColor*/
const Value V19773_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_237 = {S9, 0, &V19773_237_55786, &P19773_286};	/*Property = CssColumn*/
const Value V19773_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_287 = {S10, 0, &V19773_287_57173, &P19773_237};	/*Property = CssFlexbox*/
const Value V19773_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_288 = {S11, 0, &V19773_288_57176, &P19773_287};	/*Property = CssFont*/
const Value V19773_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_289 = {S12, 0, &V19773_289_86899, &P19773_288};	/*Property = CssImages*/
const Value V19773_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_291 = {S13, 0, &V19773_291_57182, &P19773_289};	/*Property = CssMediaQueries*/
const Value V19773_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_292 = {S14, 0, &V19773_292_57185, &P19773_291};	/*Property = CssMinMax*/
const Value V19773_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_293 = {S15, 0, &V19773_293_57188, &P19773_292};	/*Property = CssOverflow*/
const Value V19773_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_294 = {S16, 0, &V19773_294_57191, &P19773_293};	/*Property = CssPosition*/
const Value V19773_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_295 = {S17, 0, &V19773_295_57194, &P19773_294};	/*Property = CssText*/
const Value V19773_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_297 = {S18, 0, &V19773_297_57197, &P19773_295};	/*Property = CssTransforms*/
const Value V19773_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_241 = {S19, 0, &V19773_241_55801, &P19773_297};	/*Property = CssTransitions*/
const Value V19773_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_298 = {S20, 0, &V19773_298_57208, &P19773_241};	/*Property = CssUI*/
const Value V19773_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_300 = {S21, 0, &V19773_300_57217, &P19773_298};	/*Property = DataSet*/
const Value V19773_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_301 = {S22, 0, &V19773_301_57220, &P19773_300};	/*Property = DataUrl*/
const Value V19773_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_302 = {S23, 0, &V19773_302_57223, &P19773_301};	/*Property = DeviceOrientation*/
const Value V19773_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_304 = {S24, 0, &V19773_304_57229, &P19773_302};	/*Property = FileReader*/
const Value V19773_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_305 = {S25, 0, &V19773_305_57232, &P19773_304};	/*Property = FileSaver*/
const Value V19773_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_306 = {S26, 0, &V19773_306_57238, &P19773_305};	/*Property = FileWriter*/
const Value V19773_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_307 = {S27, 0, &V19773_307_57241, &P19773_306};	/*Property = FormData*/
const Value V19773_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_309 = {S28, 0, &V19773_309_57247, &P19773_307};	/*Property = Fullscreen*/
const Value V19773_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_310 = {S29, 0, &V19773_310_57250, &P19773_309};	/*Property = GeoLocation*/
const Value V19773_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_312 = {S30, 0, &V19773_312_57256, &P19773_310};	/*Property = History*/
const Value V19773_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_314 = {S31, 0, &V19773_314_57262, &P19773_312};	/*Property = Html5*/
const Value V19773_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_313 = {S32, 0, &V19773_313_57259, &P19773_314};	/*Property = Html-Media-Capture*/
const Value V19773_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_315 = {S33, 0, &V19773_315_57265, &P19773_313};	/*Property = Iframe*/
const Value V19773_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_316 = {S34, 0, &V19773_316_57268, &P19773_315};	/*Property = IndexedDB*/
const Value V19773_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_317 = {S35, 0, &V19773_317_57271, &P19773_316};	/*Property = Json*/
const Value V19773_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P19773_15 = {S37, 0, &V19773_15_29, &P19773_317};	/*Property = LayoutEngine*/
const Value V19773_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_344 = {S38, 0, &V19773_344_57274, &P19773_15};	/*Property = Masking*/
const Value V19773_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_320 = {S39, 0, &V19773_320_57292, &P19773_344};	/*Property = PostMessage*/
const Value V19773_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_321 = {S40, 0, &V19773_321_57295, &P19773_320};	/*Property = Progress*/
const Value V19773_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_322 = {S41, 0, &V19773_322_57298, &P19773_321};	/*Property = Prompts*/
const Value V19773_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_323 = {S42, 0, &V19773_323_57301, &P19773_322};	/*Property = Selector*/
const Value V19773_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_324 = {S43, 0, &V19773_324_57307, &P19773_323};	/*Property = Svg*/
const Value V19773_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_327 = {S44, 0, &V19773_327_57316, &P19773_324};	/*Property = TouchEvents*/
const Value V19773_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_328 = {S45, 0, &V19773_328_57319, &P19773_327};	/*Property = Track*/
const Value V19773_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_329 = {S46, 0, &V19773_329_57325, &P19773_328};	/*Property = Video*/
const Value V19773_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_330 = {S47, 0, &V19773_330_57328, &P19773_329};	/*Property = Viewport*/
const Value V19773_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_332 = {S48, 0, &V19773_332_57340, &P19773_330};	/*Property = WebWorkers*/
const Value V19773_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P19773_333 = {S49, 0, &V19773_333_57343, &P19773_332};	/*Property = Xhr2*/
const Profile P19773 = {"19773", &P19773_333};	/*Profile ID - 19773*/
const Profile P22197 = {"22197", NULL};	/*Profile ID - 22197*/
const Profile P22191 = {"22191", NULL};	/*Profile ID - 22191*/
const Value V18199_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_279 = {S1, 0, &V18199_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18199_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_282 = {S3, 0, &V18199_282_57155, &P18199_279};	/*Property = BlobBuilder*/
const Value V18199_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_350 = {S4, 0, &V18199_350_57446, &P18199_282};	/*Property = Canvas*/
const Value V18199_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_283 = {S5, 0, &V18199_283_57161, &P18199_350};	/*Property = CssBackground*/
const Value V18199_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_284 = {S6, 0, &V18199_284_57164, &P18199_283};	/*Property = CssBorderImage*/
const Value V18199_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_285 = {S7, 0, &V18199_285_57167, &P18199_284};	/*Property = CssCanvas*/
const Value V18199_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_286 = {S8, 0, &V18199_286_57170, &P18199_285};	/*Property = CssColor*/
const Value V18199_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_237 = {S9, 0, &V18199_237_55786, &P18199_286};	/*Property = CssColumn*/
const Value V18199_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_287 = {S10, 0, &V18199_287_57173, &P18199_237};	/*Property = CssFlexbox*/
const Value V18199_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_288 = {S11, 0, &V18199_288_57176, &P18199_287};	/*Property = CssFont*/
const Value V18199_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_289 = {S12, 0, &V18199_289_86899, &P18199_288};	/*Property = CssImages*/
const Value V18199_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_291 = {S13, 0, &V18199_291_57182, &P18199_289};	/*Property = CssMediaQueries*/
const Value V18199_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_292 = {S14, 0, &V18199_292_57185, &P18199_291};	/*Property = CssMinMax*/
const Value V18199_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_293 = {S15, 0, &V18199_293_57188, &P18199_292};	/*Property = CssOverflow*/
const Value V18199_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_294 = {S16, 0, &V18199_294_57191, &P18199_293};	/*Property = CssPosition*/
const Value V18199_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_295 = {S17, 0, &V18199_295_57194, &P18199_294};	/*Property = CssText*/
const Value V18199_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_297 = {S18, 0, &V18199_297_57197, &P18199_295};	/*Property = CssTransforms*/
const Value V18199_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_241 = {S19, 0, &V18199_241_55801, &P18199_297};	/*Property = CssTransitions*/
const Value V18199_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_298 = {S20, 0, &V18199_298_57208, &P18199_241};	/*Property = CssUI*/
const Value V18199_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_300 = {S21, 0, &V18199_300_57217, &P18199_298};	/*Property = DataSet*/
const Value V18199_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_301 = {S22, 0, &V18199_301_57220, &P18199_300};	/*Property = DataUrl*/
const Value V18199_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_302 = {S23, 0, &V18199_302_57223, &P18199_301};	/*Property = DeviceOrientation*/
const Value V18199_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_304 = {S24, 0, &V18199_304_57229, &P18199_302};	/*Property = FileReader*/
const Value V18199_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_305 = {S25, 0, &V18199_305_57232, &P18199_304};	/*Property = FileSaver*/
const Value V18199_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_306 = {S26, 0, &V18199_306_57238, &P18199_305};	/*Property = FileWriter*/
const Value V18199_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_307 = {S27, 0, &V18199_307_57241, &P18199_306};	/*Property = FormData*/
const Value V18199_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_309 = {S28, 0, &V18199_309_57247, &P18199_307};	/*Property = Fullscreen*/
const Value V18199_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_310 = {S29, 0, &V18199_310_57250, &P18199_309};	/*Property = GeoLocation*/
const Value V18199_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_312 = {S30, 0, &V18199_312_57256, &P18199_310};	/*Property = History*/
const Value V18199_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_314 = {S31, 0, &V18199_314_57262, &P18199_312};	/*Property = Html5*/
const Value V18199_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_313 = {S32, 0, &V18199_313_57259, &P18199_314};	/*Property = Html-Media-Capture*/
const Value V18199_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_315 = {S33, 0, &V18199_315_57265, &P18199_313};	/*Property = Iframe*/
const Value V18199_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_316 = {S34, 0, &V18199_316_57268, &P18199_315};	/*Property = IndexedDB*/
const Value V18199_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_317 = {S35, 0, &V18199_317_57271, &P18199_316};	/*Property = Json*/
const Value V18199_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18199_15 = {S37, 0, &V18199_15_29, &P18199_317};	/*Property = LayoutEngine*/
const Value V18199_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_344 = {S38, 0, &V18199_344_57274, &P18199_15};	/*Property = Masking*/
const Value V18199_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_320 = {S39, 0, &V18199_320_57292, &P18199_344};	/*Property = PostMessage*/
const Value V18199_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_321 = {S40, 0, &V18199_321_57295, &P18199_320};	/*Property = Progress*/
const Value V18199_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_322 = {S41, 0, &V18199_322_57298, &P18199_321};	/*Property = Prompts*/
const Value V18199_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_323 = {S42, 0, &V18199_323_57301, &P18199_322};	/*Property = Selector*/
const Value V18199_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_324 = {S43, 0, &V18199_324_57307, &P18199_323};	/*Property = Svg*/
const Value V18199_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_327 = {S44, 0, &V18199_327_57316, &P18199_324};	/*Property = TouchEvents*/
const Value V18199_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_328 = {S45, 0, &V18199_328_57319, &P18199_327};	/*Property = Track*/
const Value V18199_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_329 = {S46, 0, &V18199_329_57325, &P18199_328};	/*Property = Video*/
const Value V18199_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_330 = {S47, 0, &V18199_330_57328, &P18199_329};	/*Property = Viewport*/
const Value V18199_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_332 = {S48, 0, &V18199_332_57340, &P18199_330};	/*Property = WebWorkers*/
const Value V18199_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18199_333 = {S49, 0, &V18199_333_57343, &P18199_332};	/*Property = Xhr2*/
const Profile P18199 = {"18199", &P18199_333};	/*Profile ID - 18199*/
const Value V22343_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22343_279 = {S1, 0, &V22343_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22343_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22343_282 = {S3, 0, &V22343_282_57153, &P22343_279};	/*Property = BlobBuilder*/
const Value V22343_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22343_350 = {S4, 0, &V22343_350_57444, &P22343_282};	/*Property = Canvas*/
const Value V22343_283_57160 = {S0, NULL};	/*Value = False*/
const Property P22343_283 = {S5, 0, &V22343_283_57160, &P22343_350};	/*Property = CssBackground*/
const Value V22343_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22343_284 = {S6, 0, &V22343_284_57163, &P22343_283};	/*Property = CssBorderImage*/
const Value V22343_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22343_285 = {S7, 0, &V22343_285_57166, &P22343_284};	/*Property = CssCanvas*/
const Value V22343_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22343_286 = {S8, 0, &V22343_286_57168, &P22343_285};	/*Property = CssColor*/
const Value V22343_237_55785 = {S0, NULL};	/*Value = False*/
const Property P22343_237 = {S9, 0, &V22343_237_55785, &P22343_286};	/*Property = CssColumn*/
const Value V22343_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22343_287 = {S10, 0, &V22343_287_57172, &P22343_237};	/*Property = CssFlexbox*/
const Value V22343_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22343_288 = {S11, 0, &V22343_288_57175, &P22343_287};	/*Property = CssFont*/
const Value V22343_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22343_289 = {S12, 0, &V22343_289_86899, &P22343_288};	/*Property = CssImages*/
const Value V22343_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22343_291 = {S13, 0, &V22343_291_57181, &P22343_289};	/*Property = CssMediaQueries*/
const Value V22343_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22343_292 = {S14, 0, &V22343_292_57183, &P22343_291};	/*Property = CssMinMax*/
const Value V22343_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22343_293 = {S15, 0, &V22343_293_57187, &P22343_292};	/*Property = CssOverflow*/
const Value V22343_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22343_294 = {S16, 0, &V22343_294_57189, &P22343_293};	/*Property = CssPosition*/
const Value V22343_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22343_295 = {S17, 0, &V22343_295_57193, &P22343_294};	/*Property = CssText*/
const Value V22343_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22343_297 = {S18, 0, &V22343_297_57195, &P22343_295};	/*Property = CssTransforms*/
const Value V22343_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22343_241 = {S19, 0, &V22343_241_55799, &P22343_297};	/*Property = CssTransitions*/
const Value V22343_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22343_298 = {S20, 0, &V22343_298_57206, &P22343_241};	/*Property = CssUI*/
const Value V22343_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22343_300 = {S21, 0, &V22343_300_57215, &P22343_298};	/*Property = DataSet*/
const Value V22343_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22343_301 = {S22, 0, &V22343_301_57218, &P22343_300};	/*Property = DataUrl*/
const Value V22343_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22343_302 = {S23, 0, &V22343_302_57221, &P22343_301};	/*Property = DeviceOrientation*/
const Value V22343_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22343_304 = {S24, 0, &V22343_304_57227, &P22343_302};	/*Property = FileReader*/
const Value V22343_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22343_305 = {S25, 0, &V22343_305_57231, &P22343_304};	/*Property = FileSaver*/
const Value V22343_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22343_306 = {S26, 0, &V22343_306_57237, &P22343_305};	/*Property = FileWriter*/
const Value V22343_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22343_307 = {S27, 0, &V22343_307_57239, &P22343_306};	/*Property = FormData*/
const Value V22343_309_57245 = {S2, NULL};	/*Value = True*/
const Property P22343_309 = {S28, 0, &V22343_309_57245, &P22343_307};	/*Property = Fullscreen*/
const Value V22343_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22343_310 = {S29, 0, &V22343_310_57248, &P22343_309};	/*Property = GeoLocation*/
const Value V22343_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22343_312 = {S30, 0, &V22343_312_57254, &P22343_310};	/*Property = History*/
const Value V22343_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22343_314 = {S31, 0, &V22343_314_57261, &P22343_312};	/*Property = Html5*/
const Value V22343_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22343_313 = {S32, 0, &V22343_313_57258, &P22343_314};	/*Property = Html-Media-Capture*/
const Value V22343_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22343_315 = {S33, 0, &V22343_315_57264, &P22343_313};	/*Property = Iframe*/
const Value V22343_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22343_316 = {S34, 0, &V22343_316_57266, &P22343_315};	/*Property = IndexedDB*/
const Value V22343_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22343_317 = {S35, 0, &V22343_317_57269, &P22343_316};	/*Property = Json*/
const Value V22343_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22343_15 = {S37, 0, &V22343_15_29, &P22343_317};	/*Property = LayoutEngine*/
const Value V22343_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22343_344 = {S38, 0, &V22343_344_57272, &P22343_15};	/*Property = Masking*/
const Value V22343_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22343_320 = {S39, 0, &V22343_320_57290, &P22343_344};	/*Property = PostMessage*/
const Value V22343_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22343_321 = {S40, 0, &V22343_321_57293, &P22343_320};	/*Property = Progress*/
const Value V22343_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22343_322 = {S41, 0, &V22343_322_57296, &P22343_321};	/*Property = Prompts*/
const Value V22343_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22343_323 = {S42, 0, &V22343_323_57299, &P22343_322};	/*Property = Selector*/
const Value V22343_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22343_324 = {S43, 0, &V22343_324_57305, &P22343_323};	/*Property = Svg*/
const Value V22343_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22343_327 = {S44, 0, &V22343_327_57315, &P22343_324};	/*Property = TouchEvents*/
const Value V22343_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22343_328 = {S45, 0, &V22343_328_57318, &P22343_327};	/*Property = Track*/
const Value V22343_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22343_329 = {S46, 0, &V22343_329_57323, &P22343_328};	/*Property = Video*/
const Value V22343_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22343_330 = {S47, 0, &V22343_330_57326, &P22343_329};	/*Property = Viewport*/
const Value V22343_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22343_332 = {S48, 0, &V22343_332_57339, &P22343_330};	/*Property = WebWorkers*/
const Value V22343_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22343_333 = {S49, 0, &V22343_333_57342, &P22343_332};	/*Property = Xhr2*/
const Profile P22343 = {"22343", &P22343_333};	/*Profile ID - 22343*/
const Value V19905_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19905_279 = {S1, 0, &V19905_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19905_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19905_282 = {S3, 0, &V19905_282_57154, &P19905_279};	/*Property = BlobBuilder*/
const Value V19905_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19905_350 = {S4, 0, &V19905_350_57444, &P19905_282};	/*Property = Canvas*/
const Value V19905_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19905_283 = {S5, 0, &V19905_283_57159, &P19905_350};	/*Property = CssBackground*/
const Value V19905_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19905_284 = {S6, 0, &V19905_284_57163, &P19905_283};	/*Property = CssBorderImage*/
const Value V19905_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19905_285 = {S7, 0, &V19905_285_57166, &P19905_284};	/*Property = CssCanvas*/
const Value V19905_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19905_286 = {S8, 0, &V19905_286_57168, &P19905_285};	/*Property = CssColor*/
const Value V19905_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19905_237 = {S9, 0, &V19905_237_55786, &P19905_286};	/*Property = CssColumn*/
const Value V19905_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19905_287 = {S10, 0, &V19905_287_57172, &P19905_237};	/*Property = CssFlexbox*/
const Value V19905_288_57175 = {S0, NULL};	/*Value = False*/
const Property P19905_288 = {S11, 0, &V19905_288_57175, &P19905_287};	/*Property = CssFont*/
const Value V19905_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19905_289 = {S12, 0, &V19905_289_57759, &P19905_288};	/*Property = CssImages*/
const Value V19905_291_57181 = {S0, NULL};	/*Value = False*/
const Property P19905_291 = {S13, 0, &V19905_291_57181, &P19905_289};	/*Property = CssMediaQueries*/
const Value V19905_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19905_292 = {S14, 0, &V19905_292_57183, &P19905_291};	/*Property = CssMinMax*/
const Value V19905_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19905_293 = {S15, 0, &V19905_293_57187, &P19905_292};	/*Property = CssOverflow*/
const Value V19905_294_57190 = {S0, NULL};	/*Value = False*/
const Property P19905_294 = {S16, 0, &V19905_294_57190, &P19905_293};	/*Property = CssPosition*/
const Value V19905_295_57193 = {S0, NULL};	/*Value = False*/
const Property P19905_295 = {S17, 0, &V19905_295_57193, &P19905_294};	/*Property = CssText*/
const Value V19905_297_57196 = {S0, NULL};	/*Value = False*/
const Property P19905_297 = {S18, 0, &V19905_297_57196, &P19905_295};	/*Property = CssTransforms*/
const Value V19905_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19905_241 = {S19, 0, &V19905_241_55799, &P19905_297};	/*Property = CssTransitions*/
const Value V19905_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19905_298 = {S20, 0, &V19905_298_57206, &P19905_241};	/*Property = CssUI*/
const Value V19905_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19905_300 = {S21, 0, &V19905_300_57216, &P19905_298};	/*Property = DataSet*/
const Value V19905_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19905_301 = {S22, 0, &V19905_301_57218, &P19905_300};	/*Property = DataUrl*/
const Value V19905_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19905_302 = {S23, 0, &V19905_302_57222, &P19905_301};	/*Property = DeviceOrientation*/
const Value V19905_304_57228 = {S0, NULL};	/*Value = False*/
const Property P19905_304 = {S24, 0, &V19905_304_57228, &P19905_302};	/*Property = FileReader*/
const Value V19905_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19905_305 = {S25, 0, &V19905_305_57231, &P19905_304};	/*Property = FileSaver*/
const Value V19905_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19905_306 = {S26, 0, &V19905_306_57237, &P19905_305};	/*Property = FileWriter*/
const Value V19905_307_57240 = {S0, NULL};	/*Value = False*/
const Property P19905_307 = {S27, 0, &V19905_307_57240, &P19905_306};	/*Property = FormData*/
const Value V19905_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19905_309 = {S28, 0, &V19905_309_57246, &P19905_307};	/*Property = Fullscreen*/
const Value V19905_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19905_310 = {S29, 0, &V19905_310_57248, &P19905_309};	/*Property = GeoLocation*/
const Value V19905_312_57255 = {S0, NULL};	/*Value = False*/
const Property P19905_312 = {S30, 0, &V19905_312_57255, &P19905_310};	/*Property = History*/
const Value V19905_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19905_314 = {S31, 0, &V19905_314_57261, &P19905_312};	/*Property = Html5*/
const Value V19905_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19905_313 = {S32, 0, &V19905_313_57258, &P19905_314};	/*Property = Html-Media-Capture*/
const Value V19905_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19905_315 = {S33, 0, &V19905_315_57264, &P19905_313};	/*Property = Iframe*/
const Value V19905_316_57267 = {S0, NULL};	/*Value = False*/
const Property P19905_316 = {S34, 0, &V19905_316_57267, &P19905_315};	/*Property = IndexedDB*/
const Value V19905_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19905_317 = {S35, 0, &V19905_317_57269, &P19905_316};	/*Property = Json*/
const Value V19905_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P19905_15 = {S37, 0, &V19905_15_31, &P19905_317};	/*Property = LayoutEngine*/
const Value V19905_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19905_344 = {S38, 0, &V19905_344_57272, &P19905_15};	/*Property = Masking*/
const Value V19905_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19905_320 = {S39, 0, &V19905_320_57290, &P19905_344};	/*Property = PostMessage*/
const Value V19905_321_57294 = {S0, NULL};	/*Value = False*/
const Property P19905_321 = {S40, 0, &V19905_321_57294, &P19905_320};	/*Property = Progress*/
const Value V19905_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19905_322 = {S41, 0, &V19905_322_57296, &P19905_321};	/*Property = Prompts*/
const Value V19905_323_57300 = {S0, NULL};	/*Value = False*/
const Property P19905_323 = {S42, 0, &V19905_323_57300, &P19905_322};	/*Property = Selector*/
const Value V19905_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19905_324 = {S43, 0, &V19905_324_57306, &P19905_323};	/*Property = Svg*/
const Value V19905_327_57315 = {S0, NULL};	/*Value = False*/
const Property P19905_327 = {S44, 0, &V19905_327_57315, &P19905_324};	/*Property = TouchEvents*/
const Value V19905_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19905_328 = {S45, 0, &V19905_328_57318, &P19905_327};	/*Property = Track*/
const Value V19905_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19905_329 = {S46, 0, &V19905_329_57323, &P19905_328};	/*Property = Video*/
const Value V19905_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19905_330 = {S47, 0, &V19905_330_57326, &P19905_329};	/*Property = Viewport*/
const Value V19905_332_57339 = {S0, NULL};	/*Value = False*/
const Property P19905_332 = {S48, 0, &V19905_332_57339, &P19905_330};	/*Property = WebWorkers*/
const Value V19905_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19905_333 = {S49, 0, &V19905_333_57342, &P19905_332};	/*Property = Xhr2*/
const Profile P19905 = {"19905", &P19905_333};	/*Profile ID - 19905*/
const Value V22479_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22479_279 = {S1, 0, &V22479_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22479_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22479_282 = {S3, 0, &V22479_282_57154, &P22479_279};	/*Property = BlobBuilder*/
const Value V22479_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22479_350 = {S4, 0, &V22479_350_57444, &P22479_282};	/*Property = Canvas*/
const Value V22479_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22479_283 = {S5, 0, &V22479_283_57159, &P22479_350};	/*Property = CssBackground*/
const Value V22479_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22479_284 = {S6, 0, &V22479_284_57163, &P22479_283};	/*Property = CssBorderImage*/
const Value V22479_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22479_285 = {S7, 0, &V22479_285_57166, &P22479_284};	/*Property = CssCanvas*/
const Value V22479_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22479_286 = {S8, 0, &V22479_286_57168, &P22479_285};	/*Property = CssColor*/
const Value V22479_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22479_237 = {S9, 0, &V22479_237_55784, &P22479_286};	/*Property = CssColumn*/
const Value V22479_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22479_287 = {S10, 0, &V22479_287_57172, &P22479_237};	/*Property = CssFlexbox*/
const Value V22479_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22479_288 = {S11, 0, &V22479_288_57174, &P22479_287};	/*Property = CssFont*/
const Value V22479_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22479_289 = {S12, 0, &V22479_289_57759, &P22479_288};	/*Property = CssImages*/
const Value V22479_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22479_291 = {S13, 0, &V22479_291_57180, &P22479_289};	/*Property = CssMediaQueries*/
const Value V22479_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22479_292 = {S14, 0, &V22479_292_57183, &P22479_291};	/*Property = CssMinMax*/
const Value V22479_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22479_293 = {S15, 0, &V22479_293_57187, &P22479_292};	/*Property = CssOverflow*/
const Value V22479_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22479_294 = {S16, 0, &V22479_294_57189, &P22479_293};	/*Property = CssPosition*/
const Value V22479_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22479_295 = {S17, 0, &V22479_295_57192, &P22479_294};	/*Property = CssText*/
const Value V22479_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22479_297 = {S18, 0, &V22479_297_57195, &P22479_295};	/*Property = CssTransforms*/
const Value V22479_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22479_241 = {S19, 0, &V22479_241_55799, &P22479_297};	/*Property = CssTransitions*/
const Value V22479_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22479_298 = {S20, 0, &V22479_298_57206, &P22479_241};	/*Property = CssUI*/
const Value V22479_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22479_300 = {S21, 0, &V22479_300_57215, &P22479_298};	/*Property = DataSet*/
const Value V22479_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22479_301 = {S22, 0, &V22479_301_57218, &P22479_300};	/*Property = DataUrl*/
const Value V22479_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22479_302 = {S23, 0, &V22479_302_57221, &P22479_301};	/*Property = DeviceOrientation*/
const Value V22479_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22479_304 = {S24, 0, &V22479_304_57227, &P22479_302};	/*Property = FileReader*/
const Value V22479_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22479_305 = {S25, 0, &V22479_305_57231, &P22479_304};	/*Property = FileSaver*/
const Value V22479_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22479_306 = {S26, 0, &V22479_306_57237, &P22479_305};	/*Property = FileWriter*/
const Value V22479_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22479_307 = {S27, 0, &V22479_307_57239, &P22479_306};	/*Property = FormData*/
const Value V22479_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22479_309 = {S28, 0, &V22479_309_57246, &P22479_307};	/*Property = Fullscreen*/
const Value V22479_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22479_310 = {S29, 0, &V22479_310_57248, &P22479_309};	/*Property = GeoLocation*/
const Value V22479_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22479_312 = {S30, 0, &V22479_312_57254, &P22479_310};	/*Property = History*/
const Value V22479_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22479_314 = {S31, 0, &V22479_314_57261, &P22479_312};	/*Property = Html5*/
const Value V22479_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22479_313 = {S32, 0, &V22479_313_57258, &P22479_314};	/*Property = Html-Media-Capture*/
const Value V22479_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22479_315 = {S33, 0, &V22479_315_57264, &P22479_313};	/*Property = Iframe*/
const Value V22479_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22479_316 = {S34, 0, &V22479_316_57266, &P22479_315};	/*Property = IndexedDB*/
const Value V22479_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22479_317 = {S35, 0, &V22479_317_57269, &P22479_316};	/*Property = Json*/
const Value V22479_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22479_15 = {S37, 0, &V22479_15_29, &P22479_317};	/*Property = LayoutEngine*/
const Value V22479_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22479_344 = {S38, 0, &V22479_344_57272, &P22479_15};	/*Property = Masking*/
const Value V22479_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22479_320 = {S39, 0, &V22479_320_57290, &P22479_344};	/*Property = PostMessage*/
const Value V22479_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22479_321 = {S40, 0, &V22479_321_57293, &P22479_320};	/*Property = Progress*/
const Value V22479_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22479_322 = {S41, 0, &V22479_322_57296, &P22479_321};	/*Property = Prompts*/
const Value V22479_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22479_323 = {S42, 0, &V22479_323_57299, &P22479_322};	/*Property = Selector*/
const Value V22479_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22479_324 = {S43, 0, &V22479_324_57305, &P22479_323};	/*Property = Svg*/
const Value V22479_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22479_327 = {S44, 0, &V22479_327_57315, &P22479_324};	/*Property = TouchEvents*/
const Value V22479_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22479_328 = {S45, 0, &V22479_328_57318, &P22479_327};	/*Property = Track*/
const Value V22479_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22479_329 = {S46, 0, &V22479_329_57323, &P22479_328};	/*Property = Video*/
const Value V22479_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22479_330 = {S47, 0, &V22479_330_57326, &P22479_329};	/*Property = Viewport*/
const Value V22479_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22479_332 = {S48, 0, &V22479_332_57338, &P22479_330};	/*Property = WebWorkers*/
const Value V22479_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22479_333 = {S49, 0, &V22479_333_57341, &P22479_332};	/*Property = Xhr2*/
const Profile P22479 = {"22479", &P22479_333};	/*Profile ID - 22479*/
const Value V22481_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_279 = {S1, 0, &V22481_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22481_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_282 = {S3, 0, &V22481_282_57155, &P22481_279};	/*Property = BlobBuilder*/
const Value V22481_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_350 = {S4, 0, &V22481_350_57446, &P22481_282};	/*Property = Canvas*/
const Value V22481_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_283 = {S5, 0, &V22481_283_57161, &P22481_350};	/*Property = CssBackground*/
const Value V22481_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_284 = {S6, 0, &V22481_284_57164, &P22481_283};	/*Property = CssBorderImage*/
const Value V22481_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_285 = {S7, 0, &V22481_285_57167, &P22481_284};	/*Property = CssCanvas*/
const Value V22481_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_286 = {S8, 0, &V22481_286_57170, &P22481_285};	/*Property = CssColor*/
const Value V22481_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_237 = {S9, 0, &V22481_237_55786, &P22481_286};	/*Property = CssColumn*/
const Value V22481_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_287 = {S10, 0, &V22481_287_57173, &P22481_237};	/*Property = CssFlexbox*/
const Value V22481_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_288 = {S11, 0, &V22481_288_57176, &P22481_287};	/*Property = CssFont*/
const Value V22481_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_289 = {S12, 0, &V22481_289_86899, &P22481_288};	/*Property = CssImages*/
const Value V22481_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_291 = {S13, 0, &V22481_291_57182, &P22481_289};	/*Property = CssMediaQueries*/
const Value V22481_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_292 = {S14, 0, &V22481_292_57185, &P22481_291};	/*Property = CssMinMax*/
const Value V22481_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_293 = {S15, 0, &V22481_293_57188, &P22481_292};	/*Property = CssOverflow*/
const Value V22481_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_294 = {S16, 0, &V22481_294_57191, &P22481_293};	/*Property = CssPosition*/
const Value V22481_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_295 = {S17, 0, &V22481_295_57194, &P22481_294};	/*Property = CssText*/
const Value V22481_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_297 = {S18, 0, &V22481_297_57197, &P22481_295};	/*Property = CssTransforms*/
const Value V22481_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_241 = {S19, 0, &V22481_241_55801, &P22481_297};	/*Property = CssTransitions*/
const Value V22481_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_298 = {S20, 0, &V22481_298_57208, &P22481_241};	/*Property = CssUI*/
const Value V22481_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_300 = {S21, 0, &V22481_300_57217, &P22481_298};	/*Property = DataSet*/
const Value V22481_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_301 = {S22, 0, &V22481_301_57220, &P22481_300};	/*Property = DataUrl*/
const Value V22481_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_302 = {S23, 0, &V22481_302_57223, &P22481_301};	/*Property = DeviceOrientation*/
const Value V22481_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_304 = {S24, 0, &V22481_304_57229, &P22481_302};	/*Property = FileReader*/
const Value V22481_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_305 = {S25, 0, &V22481_305_57232, &P22481_304};	/*Property = FileSaver*/
const Value V22481_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_306 = {S26, 0, &V22481_306_57238, &P22481_305};	/*Property = FileWriter*/
const Value V22481_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_307 = {S27, 0, &V22481_307_57241, &P22481_306};	/*Property = FormData*/
const Value V22481_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_309 = {S28, 0, &V22481_309_57247, &P22481_307};	/*Property = Fullscreen*/
const Value V22481_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_310 = {S29, 0, &V22481_310_57250, &P22481_309};	/*Property = GeoLocation*/
const Value V22481_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_312 = {S30, 0, &V22481_312_57256, &P22481_310};	/*Property = History*/
const Value V22481_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_314 = {S31, 0, &V22481_314_57262, &P22481_312};	/*Property = Html5*/
const Value V22481_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_313 = {S32, 0, &V22481_313_57259, &P22481_314};	/*Property = Html-Media-Capture*/
const Value V22481_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_315 = {S33, 0, &V22481_315_57265, &P22481_313};	/*Property = Iframe*/
const Value V22481_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_316 = {S34, 0, &V22481_316_57268, &P22481_315};	/*Property = IndexedDB*/
const Value V22481_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_317 = {S35, 0, &V22481_317_57271, &P22481_316};	/*Property = Json*/
const Value V22481_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22481_15 = {S37, 0, &V22481_15_29, &P22481_317};	/*Property = LayoutEngine*/
const Value V22481_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_344 = {S38, 0, &V22481_344_57274, &P22481_15};	/*Property = Masking*/
const Value V22481_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_320 = {S39, 0, &V22481_320_57292, &P22481_344};	/*Property = PostMessage*/
const Value V22481_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_321 = {S40, 0, &V22481_321_57295, &P22481_320};	/*Property = Progress*/
const Value V22481_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_322 = {S41, 0, &V22481_322_57298, &P22481_321};	/*Property = Prompts*/
const Value V22481_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_323 = {S42, 0, &V22481_323_57301, &P22481_322};	/*Property = Selector*/
const Value V22481_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_324 = {S43, 0, &V22481_324_57307, &P22481_323};	/*Property = Svg*/
const Value V22481_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_327 = {S44, 0, &V22481_327_57316, &P22481_324};	/*Property = TouchEvents*/
const Value V22481_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_328 = {S45, 0, &V22481_328_57319, &P22481_327};	/*Property = Track*/
const Value V22481_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_329 = {S46, 0, &V22481_329_57325, &P22481_328};	/*Property = Video*/
const Value V22481_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_330 = {S47, 0, &V22481_330_57328, &P22481_329};	/*Property = Viewport*/
const Value V22481_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_332 = {S48, 0, &V22481_332_57340, &P22481_330};	/*Property = WebWorkers*/
const Value V22481_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22481_333 = {S49, 0, &V22481_333_57343, &P22481_332};	/*Property = Xhr2*/
const Profile P22481 = {"22481", &P22481_333};	/*Profile ID - 22481*/
const Profile P17832 = {"17832", NULL};	/*Profile ID - 17832*/
const Profile P17814 = {"17814", NULL};	/*Profile ID - 17814*/
const Value V21806_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_279 = {S1, 0, &V21806_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21806_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_282 = {S3, 0, &V21806_282_57155, &P21806_279};	/*Property = BlobBuilder*/
const Value V21806_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_350 = {S4, 0, &V21806_350_57446, &P21806_282};	/*Property = Canvas*/
const Value V21806_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_283 = {S5, 0, &V21806_283_57161, &P21806_350};	/*Property = CssBackground*/
const Value V21806_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_284 = {S6, 0, &V21806_284_57164, &P21806_283};	/*Property = CssBorderImage*/
const Value V21806_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_285 = {S7, 0, &V21806_285_57167, &P21806_284};	/*Property = CssCanvas*/
const Value V21806_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_286 = {S8, 0, &V21806_286_57170, &P21806_285};	/*Property = CssColor*/
const Value V21806_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_237 = {S9, 0, &V21806_237_55786, &P21806_286};	/*Property = CssColumn*/
const Value V21806_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_287 = {S10, 0, &V21806_287_57173, &P21806_237};	/*Property = CssFlexbox*/
const Value V21806_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_288 = {S11, 0, &V21806_288_57176, &P21806_287};	/*Property = CssFont*/
const Value V21806_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_289 = {S12, 0, &V21806_289_86899, &P21806_288};	/*Property = CssImages*/
const Value V21806_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_291 = {S13, 0, &V21806_291_57182, &P21806_289};	/*Property = CssMediaQueries*/
const Value V21806_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_292 = {S14, 0, &V21806_292_57185, &P21806_291};	/*Property = CssMinMax*/
const Value V21806_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_293 = {S15, 0, &V21806_293_57188, &P21806_292};	/*Property = CssOverflow*/
const Value V21806_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_294 = {S16, 0, &V21806_294_57191, &P21806_293};	/*Property = CssPosition*/
const Value V21806_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_295 = {S17, 0, &V21806_295_57194, &P21806_294};	/*Property = CssText*/
const Value V21806_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_297 = {S18, 0, &V21806_297_57197, &P21806_295};	/*Property = CssTransforms*/
const Value V21806_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_241 = {S19, 0, &V21806_241_55801, &P21806_297};	/*Property = CssTransitions*/
const Value V21806_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_298 = {S20, 0, &V21806_298_57208, &P21806_241};	/*Property = CssUI*/
const Value V21806_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_300 = {S21, 0, &V21806_300_57217, &P21806_298};	/*Property = DataSet*/
const Value V21806_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_301 = {S22, 0, &V21806_301_57220, &P21806_300};	/*Property = DataUrl*/
const Value V21806_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_302 = {S23, 0, &V21806_302_57223, &P21806_301};	/*Property = DeviceOrientation*/
const Value V21806_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_304 = {S24, 0, &V21806_304_57229, &P21806_302};	/*Property = FileReader*/
const Value V21806_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_305 = {S25, 0, &V21806_305_57232, &P21806_304};	/*Property = FileSaver*/
const Value V21806_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_306 = {S26, 0, &V21806_306_57238, &P21806_305};	/*Property = FileWriter*/
const Value V21806_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_307 = {S27, 0, &V21806_307_57241, &P21806_306};	/*Property = FormData*/
const Value V21806_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_309 = {S28, 0, &V21806_309_57247, &P21806_307};	/*Property = Fullscreen*/
const Value V21806_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_310 = {S29, 0, &V21806_310_57250, &P21806_309};	/*Property = GeoLocation*/
const Value V21806_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_312 = {S30, 0, &V21806_312_57256, &P21806_310};	/*Property = History*/
const Value V21806_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_314 = {S31, 0, &V21806_314_57262, &P21806_312};	/*Property = Html5*/
const Value V21806_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_313 = {S32, 0, &V21806_313_57259, &P21806_314};	/*Property = Html-Media-Capture*/
const Value V21806_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_315 = {S33, 0, &V21806_315_57265, &P21806_313};	/*Property = Iframe*/
const Value V21806_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_316 = {S34, 0, &V21806_316_57268, &P21806_315};	/*Property = IndexedDB*/
const Value V21806_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_317 = {S35, 0, &V21806_317_57271, &P21806_316};	/*Property = Json*/
const Value V21806_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P21806_15 = {S37, 0, &V21806_15_29, &P21806_317};	/*Property = LayoutEngine*/
const Value V21806_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_344 = {S38, 0, &V21806_344_57274, &P21806_15};	/*Property = Masking*/
const Value V21806_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_320 = {S39, 0, &V21806_320_57292, &P21806_344};	/*Property = PostMessage*/
const Value V21806_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_321 = {S40, 0, &V21806_321_57295, &P21806_320};	/*Property = Progress*/
const Value V21806_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_322 = {S41, 0, &V21806_322_57298, &P21806_321};	/*Property = Prompts*/
const Value V21806_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_323 = {S42, 0, &V21806_323_57301, &P21806_322};	/*Property = Selector*/
const Value V21806_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_324 = {S43, 0, &V21806_324_57307, &P21806_323};	/*Property = Svg*/
const Value V21806_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_327 = {S44, 0, &V21806_327_57316, &P21806_324};	/*Property = TouchEvents*/
const Value V21806_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_328 = {S45, 0, &V21806_328_57319, &P21806_327};	/*Property = Track*/
const Value V21806_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_329 = {S46, 0, &V21806_329_57325, &P21806_328};	/*Property = Video*/
const Value V21806_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_330 = {S47, 0, &V21806_330_57328, &P21806_329};	/*Property = Viewport*/
const Value V21806_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_332 = {S48, 0, &V21806_332_57340, &P21806_330};	/*Property = WebWorkers*/
const Value V21806_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21806_333 = {S49, 0, &V21806_333_57343, &P21806_332};	/*Property = Xhr2*/
const Profile P21806 = {"21806", &P21806_333};	/*Profile ID - 21806*/
const Value V22504_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_279 = {S1, 0, &V22504_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22504_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_282 = {S3, 0, &V22504_282_57155, &P22504_279};	/*Property = BlobBuilder*/
const Value V22504_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_350 = {S4, 0, &V22504_350_57446, &P22504_282};	/*Property = Canvas*/
const Value V22504_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_283 = {S5, 0, &V22504_283_57161, &P22504_350};	/*Property = CssBackground*/
const Value V22504_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_284 = {S6, 0, &V22504_284_57164, &P22504_283};	/*Property = CssBorderImage*/
const Value V22504_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_285 = {S7, 0, &V22504_285_57167, &P22504_284};	/*Property = CssCanvas*/
const Value V22504_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_286 = {S8, 0, &V22504_286_57170, &P22504_285};	/*Property = CssColor*/
const Value V22504_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_237 = {S9, 0, &V22504_237_55786, &P22504_286};	/*Property = CssColumn*/
const Value V22504_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_287 = {S10, 0, &V22504_287_57173, &P22504_237};	/*Property = CssFlexbox*/
const Value V22504_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_288 = {S11, 0, &V22504_288_57176, &P22504_287};	/*Property = CssFont*/
const Value V22504_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_289 = {S12, 0, &V22504_289_86899, &P22504_288};	/*Property = CssImages*/
const Value V22504_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_291 = {S13, 0, &V22504_291_57182, &P22504_289};	/*Property = CssMediaQueries*/
const Value V22504_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_292 = {S14, 0, &V22504_292_57185, &P22504_291};	/*Property = CssMinMax*/
const Value V22504_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_293 = {S15, 0, &V22504_293_57188, &P22504_292};	/*Property = CssOverflow*/
const Value V22504_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_294 = {S16, 0, &V22504_294_57191, &P22504_293};	/*Property = CssPosition*/
const Value V22504_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_295 = {S17, 0, &V22504_295_57194, &P22504_294};	/*Property = CssText*/
const Value V22504_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_297 = {S18, 0, &V22504_297_57197, &P22504_295};	/*Property = CssTransforms*/
const Value V22504_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_241 = {S19, 0, &V22504_241_55801, &P22504_297};	/*Property = CssTransitions*/
const Value V22504_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_298 = {S20, 0, &V22504_298_57208, &P22504_241};	/*Property = CssUI*/
const Value V22504_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_300 = {S21, 0, &V22504_300_57217, &P22504_298};	/*Property = DataSet*/
const Value V22504_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_301 = {S22, 0, &V22504_301_57220, &P22504_300};	/*Property = DataUrl*/
const Value V22504_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_302 = {S23, 0, &V22504_302_57223, &P22504_301};	/*Property = DeviceOrientation*/
const Value V22504_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_304 = {S24, 0, &V22504_304_57229, &P22504_302};	/*Property = FileReader*/
const Value V22504_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_305 = {S25, 0, &V22504_305_57232, &P22504_304};	/*Property = FileSaver*/
const Value V22504_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_306 = {S26, 0, &V22504_306_57238, &P22504_305};	/*Property = FileWriter*/
const Value V22504_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_307 = {S27, 0, &V22504_307_57241, &P22504_306};	/*Property = FormData*/
const Value V22504_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_309 = {S28, 0, &V22504_309_57247, &P22504_307};	/*Property = Fullscreen*/
const Value V22504_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_310 = {S29, 0, &V22504_310_57250, &P22504_309};	/*Property = GeoLocation*/
const Value V22504_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_312 = {S30, 0, &V22504_312_57256, &P22504_310};	/*Property = History*/
const Value V22504_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_314 = {S31, 0, &V22504_314_57262, &P22504_312};	/*Property = Html5*/
const Value V22504_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_313 = {S32, 0, &V22504_313_57259, &P22504_314};	/*Property = Html-Media-Capture*/
const Value V22504_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_315 = {S33, 0, &V22504_315_57265, &P22504_313};	/*Property = Iframe*/
const Value V22504_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_316 = {S34, 0, &V22504_316_57268, &P22504_315};	/*Property = IndexedDB*/
const Value V22504_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_317 = {S35, 0, &V22504_317_57271, &P22504_316};	/*Property = Json*/
const Value V22504_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22504_15 = {S37, 0, &V22504_15_29, &P22504_317};	/*Property = LayoutEngine*/
const Value V22504_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_344 = {S38, 0, &V22504_344_57274, &P22504_15};	/*Property = Masking*/
const Value V22504_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_320 = {S39, 0, &V22504_320_57292, &P22504_344};	/*Property = PostMessage*/
const Value V22504_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_321 = {S40, 0, &V22504_321_57295, &P22504_320};	/*Property = Progress*/
const Value V22504_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_322 = {S41, 0, &V22504_322_57298, &P22504_321};	/*Property = Prompts*/
const Value V22504_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_323 = {S42, 0, &V22504_323_57301, &P22504_322};	/*Property = Selector*/
const Value V22504_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_324 = {S43, 0, &V22504_324_57307, &P22504_323};	/*Property = Svg*/
const Value V22504_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_327 = {S44, 0, &V22504_327_57316, &P22504_324};	/*Property = TouchEvents*/
const Value V22504_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_328 = {S45, 0, &V22504_328_57319, &P22504_327};	/*Property = Track*/
const Value V22504_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_329 = {S46, 0, &V22504_329_57325, &P22504_328};	/*Property = Video*/
const Value V22504_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_330 = {S47, 0, &V22504_330_57328, &P22504_329};	/*Property = Viewport*/
const Value V22504_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_332 = {S48, 0, &V22504_332_57340, &P22504_330};	/*Property = WebWorkers*/
const Value V22504_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22504_333 = {S49, 0, &V22504_333_57343, &P22504_332};	/*Property = Xhr2*/
const Profile P22504 = {"22504", &P22504_333};	/*Profile ID - 22504*/
const Value V21601_1_1 = {S2, NULL};	/*Value = True*/
const Property P21601_1 = {S50, 0, &V21601_1_1, NULL};	/*Property = IsMobile*/
const Value V21601_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21601_5 = {S52, 0, &V21601_5_1227, &P21601_1};	/*Property = ScreenPixelsHeight*/
const Value V21601_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21601_4 = {S54, 0, &V21601_4_1229, &P21601_5};	/*Property = ScreenPixelsWidth*/
const Profile P21601 = {"21601", &P21601_4};	/*Profile ID - 21601*/
const Value V22331_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22331_279 = {S1, 0, &V22331_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22331_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22331_282 = {S3, 0, &V22331_282_57153, &P22331_279};	/*Property = BlobBuilder*/
const Value V22331_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22331_350 = {S4, 0, &V22331_350_57444, &P22331_282};	/*Property = Canvas*/
const Value V22331_283_57160 = {S0, NULL};	/*Value = False*/
const Property P22331_283 = {S5, 0, &V22331_283_57160, &P22331_350};	/*Property = CssBackground*/
const Value V22331_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22331_284 = {S6, 0, &V22331_284_57163, &P22331_283};	/*Property = CssBorderImage*/
const Value V22331_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22331_285 = {S7, 0, &V22331_285_57166, &P22331_284};	/*Property = CssCanvas*/
const Value V22331_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22331_286 = {S8, 0, &V22331_286_57168, &P22331_285};	/*Property = CssColor*/
const Value V22331_237_55785 = {S0, NULL};	/*Value = False*/
const Property P22331_237 = {S9, 0, &V22331_237_55785, &P22331_286};	/*Property = CssColumn*/
const Value V22331_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22331_287 = {S10, 0, &V22331_287_57172, &P22331_237};	/*Property = CssFlexbox*/
const Value V22331_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22331_288 = {S11, 0, &V22331_288_57175, &P22331_287};	/*Property = CssFont*/
const Value V22331_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22331_289 = {S12, 0, &V22331_289_86899, &P22331_288};	/*Property = CssImages*/
const Value V22331_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22331_291 = {S13, 0, &V22331_291_57181, &P22331_289};	/*Property = CssMediaQueries*/
const Value V22331_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22331_292 = {S14, 0, &V22331_292_57183, &P22331_291};	/*Property = CssMinMax*/
const Value V22331_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22331_293 = {S15, 0, &V22331_293_57187, &P22331_292};	/*Property = CssOverflow*/
const Value V22331_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22331_294 = {S16, 0, &V22331_294_57189, &P22331_293};	/*Property = CssPosition*/
const Value V22331_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22331_295 = {S17, 0, &V22331_295_57193, &P22331_294};	/*Property = CssText*/
const Value V22331_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22331_297 = {S18, 0, &V22331_297_57195, &P22331_295};	/*Property = CssTransforms*/
const Value V22331_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22331_241 = {S19, 0, &V22331_241_55799, &P22331_297};	/*Property = CssTransitions*/
const Value V22331_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22331_298 = {S20, 0, &V22331_298_57206, &P22331_241};	/*Property = CssUI*/
const Value V22331_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22331_300 = {S21, 0, &V22331_300_57215, &P22331_298};	/*Property = DataSet*/
const Value V22331_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22331_301 = {S22, 0, &V22331_301_57218, &P22331_300};	/*Property = DataUrl*/
const Value V22331_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22331_302 = {S23, 0, &V22331_302_57221, &P22331_301};	/*Property = DeviceOrientation*/
const Value V22331_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22331_304 = {S24, 0, &V22331_304_57227, &P22331_302};	/*Property = FileReader*/
const Value V22331_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22331_305 = {S25, 0, &V22331_305_57231, &P22331_304};	/*Property = FileSaver*/
const Value V22331_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22331_306 = {S26, 0, &V22331_306_57237, &P22331_305};	/*Property = FileWriter*/
const Value V22331_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22331_307 = {S27, 0, &V22331_307_57239, &P22331_306};	/*Property = FormData*/
const Value V22331_309_57245 = {S2, NULL};	/*Value = True*/
const Property P22331_309 = {S28, 0, &V22331_309_57245, &P22331_307};	/*Property = Fullscreen*/
const Value V22331_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22331_310 = {S29, 0, &V22331_310_57248, &P22331_309};	/*Property = GeoLocation*/
const Value V22331_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22331_312 = {S30, 0, &V22331_312_57254, &P22331_310};	/*Property = History*/
const Value V22331_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22331_314 = {S31, 0, &V22331_314_57261, &P22331_312};	/*Property = Html5*/
const Value V22331_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22331_313 = {S32, 0, &V22331_313_57258, &P22331_314};	/*Property = Html-Media-Capture*/
const Value V22331_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22331_315 = {S33, 0, &V22331_315_57264, &P22331_313};	/*Property = Iframe*/
const Value V22331_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22331_316 = {S34, 0, &V22331_316_57266, &P22331_315};	/*Property = IndexedDB*/
const Value V22331_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22331_317 = {S35, 0, &V22331_317_57269, &P22331_316};	/*Property = Json*/
const Value V22331_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22331_15 = {S37, 0, &V22331_15_29, &P22331_317};	/*Property = LayoutEngine*/
const Value V22331_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22331_344 = {S38, 0, &V22331_344_57272, &P22331_15};	/*Property = Masking*/
const Value V22331_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22331_320 = {S39, 0, &V22331_320_57290, &P22331_344};	/*Property = PostMessage*/
const Value V22331_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22331_321 = {S40, 0, &V22331_321_57293, &P22331_320};	/*Property = Progress*/
const Value V22331_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22331_322 = {S41, 0, &V22331_322_57296, &P22331_321};	/*Property = Prompts*/
const Value V22331_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22331_323 = {S42, 0, &V22331_323_57299, &P22331_322};	/*Property = Selector*/
const Value V22331_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22331_324 = {S43, 0, &V22331_324_57305, &P22331_323};	/*Property = Svg*/
const Value V22331_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22331_327 = {S44, 0, &V22331_327_57315, &P22331_324};	/*Property = TouchEvents*/
const Value V22331_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22331_328 = {S45, 0, &V22331_328_57318, &P22331_327};	/*Property = Track*/
const Value V22331_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22331_329 = {S46, 0, &V22331_329_57323, &P22331_328};	/*Property = Video*/
const Value V22331_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22331_330 = {S47, 0, &V22331_330_57326, &P22331_329};	/*Property = Viewport*/
const Value V22331_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22331_332 = {S48, 0, &V22331_332_57339, &P22331_330};	/*Property = WebWorkers*/
const Value V22331_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22331_333 = {S49, 0, &V22331_333_57342, &P22331_332};	/*Property = Xhr2*/
const Profile P22331 = {"22331", &P22331_333};	/*Profile ID - 22331*/
const Value V18201_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_279 = {S1, 0, &V18201_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18201_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_282 = {S3, 0, &V18201_282_57155, &P18201_279};	/*Property = BlobBuilder*/
const Value V18201_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_350 = {S4, 0, &V18201_350_57446, &P18201_282};	/*Property = Canvas*/
const Value V18201_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_283 = {S5, 0, &V18201_283_57161, &P18201_350};	/*Property = CssBackground*/
const Value V18201_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_284 = {S6, 0, &V18201_284_57164, &P18201_283};	/*Property = CssBorderImage*/
const Value V18201_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_285 = {S7, 0, &V18201_285_57167, &P18201_284};	/*Property = CssCanvas*/
const Value V18201_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_286 = {S8, 0, &V18201_286_57170, &P18201_285};	/*Property = CssColor*/
const Value V18201_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_237 = {S9, 0, &V18201_237_55786, &P18201_286};	/*Property = CssColumn*/
const Value V18201_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_287 = {S10, 0, &V18201_287_57173, &P18201_237};	/*Property = CssFlexbox*/
const Value V18201_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_288 = {S11, 0, &V18201_288_57176, &P18201_287};	/*Property = CssFont*/
const Value V18201_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_289 = {S12, 0, &V18201_289_86899, &P18201_288};	/*Property = CssImages*/
const Value V18201_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_291 = {S13, 0, &V18201_291_57182, &P18201_289};	/*Property = CssMediaQueries*/
const Value V18201_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_292 = {S14, 0, &V18201_292_57185, &P18201_291};	/*Property = CssMinMax*/
const Value V18201_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_293 = {S15, 0, &V18201_293_57188, &P18201_292};	/*Property = CssOverflow*/
const Value V18201_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_294 = {S16, 0, &V18201_294_57191, &P18201_293};	/*Property = CssPosition*/
const Value V18201_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_295 = {S17, 0, &V18201_295_57194, &P18201_294};	/*Property = CssText*/
const Value V18201_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_297 = {S18, 0, &V18201_297_57197, &P18201_295};	/*Property = CssTransforms*/
const Value V18201_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_241 = {S19, 0, &V18201_241_55801, &P18201_297};	/*Property = CssTransitions*/
const Value V18201_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_298 = {S20, 0, &V18201_298_57208, &P18201_241};	/*Property = CssUI*/
const Value V18201_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_300 = {S21, 0, &V18201_300_57217, &P18201_298};	/*Property = DataSet*/
const Value V18201_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_301 = {S22, 0, &V18201_301_57220, &P18201_300};	/*Property = DataUrl*/
const Value V18201_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_302 = {S23, 0, &V18201_302_57223, &P18201_301};	/*Property = DeviceOrientation*/
const Value V18201_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_304 = {S24, 0, &V18201_304_57229, &P18201_302};	/*Property = FileReader*/
const Value V18201_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_305 = {S25, 0, &V18201_305_57232, &P18201_304};	/*Property = FileSaver*/
const Value V18201_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_306 = {S26, 0, &V18201_306_57238, &P18201_305};	/*Property = FileWriter*/
const Value V18201_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_307 = {S27, 0, &V18201_307_57241, &P18201_306};	/*Property = FormData*/
const Value V18201_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_309 = {S28, 0, &V18201_309_57247, &P18201_307};	/*Property = Fullscreen*/
const Value V18201_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_310 = {S29, 0, &V18201_310_57250, &P18201_309};	/*Property = GeoLocation*/
const Value V18201_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_312 = {S30, 0, &V18201_312_57256, &P18201_310};	/*Property = History*/
const Value V18201_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_314 = {S31, 0, &V18201_314_57262, &P18201_312};	/*Property = Html5*/
const Value V18201_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_313 = {S32, 0, &V18201_313_57259, &P18201_314};	/*Property = Html-Media-Capture*/
const Value V18201_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_315 = {S33, 0, &V18201_315_57265, &P18201_313};	/*Property = Iframe*/
const Value V18201_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_316 = {S34, 0, &V18201_316_57268, &P18201_315};	/*Property = IndexedDB*/
const Value V18201_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_317 = {S35, 0, &V18201_317_57271, &P18201_316};	/*Property = Json*/
const Value V18201_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18201_15 = {S37, 0, &V18201_15_29, &P18201_317};	/*Property = LayoutEngine*/
const Value V18201_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_344 = {S38, 0, &V18201_344_57274, &P18201_15};	/*Property = Masking*/
const Value V18201_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_320 = {S39, 0, &V18201_320_57292, &P18201_344};	/*Property = PostMessage*/
const Value V18201_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_321 = {S40, 0, &V18201_321_57295, &P18201_320};	/*Property = Progress*/
const Value V18201_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_322 = {S41, 0, &V18201_322_57298, &P18201_321};	/*Property = Prompts*/
const Value V18201_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_323 = {S42, 0, &V18201_323_57301, &P18201_322};	/*Property = Selector*/
const Value V18201_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_324 = {S43, 0, &V18201_324_57307, &P18201_323};	/*Property = Svg*/
const Value V18201_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_327 = {S44, 0, &V18201_327_57316, &P18201_324};	/*Property = TouchEvents*/
const Value V18201_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_328 = {S45, 0, &V18201_328_57319, &P18201_327};	/*Property = Track*/
const Value V18201_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_329 = {S46, 0, &V18201_329_57325, &P18201_328};	/*Property = Video*/
const Value V18201_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_330 = {S47, 0, &V18201_330_57328, &P18201_329};	/*Property = Viewport*/
const Value V18201_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_332 = {S48, 0, &V18201_332_57340, &P18201_330};	/*Property = WebWorkers*/
const Value V18201_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18201_333 = {S49, 0, &V18201_333_57343, &P18201_332};	/*Property = Xhr2*/
const Profile P18201 = {"18201", &P18201_333};	/*Profile ID - 18201*/
const Value V22480_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_279 = {S1, 0, &V22480_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22480_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_282 = {S3, 0, &V22480_282_57155, &P22480_279};	/*Property = BlobBuilder*/
const Value V22480_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_350 = {S4, 0, &V22480_350_57446, &P22480_282};	/*Property = Canvas*/
const Value V22480_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_283 = {S5, 0, &V22480_283_57161, &P22480_350};	/*Property = CssBackground*/
const Value V22480_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_284 = {S6, 0, &V22480_284_57164, &P22480_283};	/*Property = CssBorderImage*/
const Value V22480_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_285 = {S7, 0, &V22480_285_57167, &P22480_284};	/*Property = CssCanvas*/
const Value V22480_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_286 = {S8, 0, &V22480_286_57170, &P22480_285};	/*Property = CssColor*/
const Value V22480_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_237 = {S9, 0, &V22480_237_55786, &P22480_286};	/*Property = CssColumn*/
const Value V22480_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_287 = {S10, 0, &V22480_287_57173, &P22480_237};	/*Property = CssFlexbox*/
const Value V22480_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_288 = {S11, 0, &V22480_288_57176, &P22480_287};	/*Property = CssFont*/
const Value V22480_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_289 = {S12, 0, &V22480_289_86899, &P22480_288};	/*Property = CssImages*/
const Value V22480_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_291 = {S13, 0, &V22480_291_57182, &P22480_289};	/*Property = CssMediaQueries*/
const Value V22480_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_292 = {S14, 0, &V22480_292_57185, &P22480_291};	/*Property = CssMinMax*/
const Value V22480_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_293 = {S15, 0, &V22480_293_57188, &P22480_292};	/*Property = CssOverflow*/
const Value V22480_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_294 = {S16, 0, &V22480_294_57191, &P22480_293};	/*Property = CssPosition*/
const Value V22480_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_295 = {S17, 0, &V22480_295_57194, &P22480_294};	/*Property = CssText*/
const Value V22480_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_297 = {S18, 0, &V22480_297_57197, &P22480_295};	/*Property = CssTransforms*/
const Value V22480_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_241 = {S19, 0, &V22480_241_55801, &P22480_297};	/*Property = CssTransitions*/
const Value V22480_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_298 = {S20, 0, &V22480_298_57208, &P22480_241};	/*Property = CssUI*/
const Value V22480_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_300 = {S21, 0, &V22480_300_57217, &P22480_298};	/*Property = DataSet*/
const Value V22480_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_301 = {S22, 0, &V22480_301_57220, &P22480_300};	/*Property = DataUrl*/
const Value V22480_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_302 = {S23, 0, &V22480_302_57223, &P22480_301};	/*Property = DeviceOrientation*/
const Value V22480_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_304 = {S24, 0, &V22480_304_57229, &P22480_302};	/*Property = FileReader*/
const Value V22480_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_305 = {S25, 0, &V22480_305_57232, &P22480_304};	/*Property = FileSaver*/
const Value V22480_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_306 = {S26, 0, &V22480_306_57238, &P22480_305};	/*Property = FileWriter*/
const Value V22480_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_307 = {S27, 0, &V22480_307_57241, &P22480_306};	/*Property = FormData*/
const Value V22480_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_309 = {S28, 0, &V22480_309_57247, &P22480_307};	/*Property = Fullscreen*/
const Value V22480_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_310 = {S29, 0, &V22480_310_57250, &P22480_309};	/*Property = GeoLocation*/
const Value V22480_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_312 = {S30, 0, &V22480_312_57256, &P22480_310};	/*Property = History*/
const Value V22480_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_314 = {S31, 0, &V22480_314_57262, &P22480_312};	/*Property = Html5*/
const Value V22480_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_313 = {S32, 0, &V22480_313_57259, &P22480_314};	/*Property = Html-Media-Capture*/
const Value V22480_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_315 = {S33, 0, &V22480_315_57265, &P22480_313};	/*Property = Iframe*/
const Value V22480_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_316 = {S34, 0, &V22480_316_57268, &P22480_315};	/*Property = IndexedDB*/
const Value V22480_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_317 = {S35, 0, &V22480_317_57271, &P22480_316};	/*Property = Json*/
const Value V22480_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22480_15 = {S37, 0, &V22480_15_29, &P22480_317};	/*Property = LayoutEngine*/
const Value V22480_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_344 = {S38, 0, &V22480_344_57274, &P22480_15};	/*Property = Masking*/
const Value V22480_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_320 = {S39, 0, &V22480_320_57292, &P22480_344};	/*Property = PostMessage*/
const Value V22480_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_321 = {S40, 0, &V22480_321_57295, &P22480_320};	/*Property = Progress*/
const Value V22480_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_322 = {S41, 0, &V22480_322_57298, &P22480_321};	/*Property = Prompts*/
const Value V22480_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_323 = {S42, 0, &V22480_323_57301, &P22480_322};	/*Property = Selector*/
const Value V22480_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_324 = {S43, 0, &V22480_324_57307, &P22480_323};	/*Property = Svg*/
const Value V22480_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_327 = {S44, 0, &V22480_327_57316, &P22480_324};	/*Property = TouchEvents*/
const Value V22480_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_328 = {S45, 0, &V22480_328_57319, &P22480_327};	/*Property = Track*/
const Value V22480_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_329 = {S46, 0, &V22480_329_57325, &P22480_328};	/*Property = Video*/
const Value V22480_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_330 = {S47, 0, &V22480_330_57328, &P22480_329};	/*Property = Viewport*/
const Value V22480_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_332 = {S48, 0, &V22480_332_57340, &P22480_330};	/*Property = WebWorkers*/
const Value V22480_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22480_333 = {S49, 0, &V22480_333_57343, &P22480_332};	/*Property = Xhr2*/
const Profile P22480 = {"22480", &P22480_333};	/*Profile ID - 22480*/
const Value V22503_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22503_279 = {S1, 0, &V22503_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22503_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22503_282 = {S3, 0, &V22503_282_57154, &P22503_279};	/*Property = BlobBuilder*/
const Value V22503_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22503_350 = {S4, 0, &V22503_350_57444, &P22503_282};	/*Property = Canvas*/
const Value V22503_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22503_283 = {S5, 0, &V22503_283_57159, &P22503_350};	/*Property = CssBackground*/
const Value V22503_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22503_284 = {S6, 0, &V22503_284_57163, &P22503_283};	/*Property = CssBorderImage*/
const Value V22503_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22503_285 = {S7, 0, &V22503_285_57166, &P22503_284};	/*Property = CssCanvas*/
const Value V22503_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22503_286 = {S8, 0, &V22503_286_57168, &P22503_285};	/*Property = CssColor*/
const Value V22503_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22503_237 = {S9, 0, &V22503_237_55784, &P22503_286};	/*Property = CssColumn*/
const Value V22503_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22503_287 = {S10, 0, &V22503_287_57172, &P22503_237};	/*Property = CssFlexbox*/
const Value V22503_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22503_288 = {S11, 0, &V22503_288_57174, &P22503_287};	/*Property = CssFont*/
const Value V22503_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22503_289 = {S12, 0, &V22503_289_57759, &P22503_288};	/*Property = CssImages*/
const Value V22503_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22503_291 = {S13, 0, &V22503_291_57180, &P22503_289};	/*Property = CssMediaQueries*/
const Value V22503_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22503_292 = {S14, 0, &V22503_292_57183, &P22503_291};	/*Property = CssMinMax*/
const Value V22503_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22503_293 = {S15, 0, &V22503_293_57187, &P22503_292};	/*Property = CssOverflow*/
const Value V22503_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22503_294 = {S16, 0, &V22503_294_57189, &P22503_293};	/*Property = CssPosition*/
const Value V22503_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22503_295 = {S17, 0, &V22503_295_57192, &P22503_294};	/*Property = CssText*/
const Value V22503_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22503_297 = {S18, 0, &V22503_297_57195, &P22503_295};	/*Property = CssTransforms*/
const Value V22503_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22503_241 = {S19, 0, &V22503_241_55799, &P22503_297};	/*Property = CssTransitions*/
const Value V22503_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22503_298 = {S20, 0, &V22503_298_57206, &P22503_241};	/*Property = CssUI*/
const Value V22503_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22503_300 = {S21, 0, &V22503_300_57215, &P22503_298};	/*Property = DataSet*/
const Value V22503_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22503_301 = {S22, 0, &V22503_301_57218, &P22503_300};	/*Property = DataUrl*/
const Value V22503_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22503_302 = {S23, 0, &V22503_302_57221, &P22503_301};	/*Property = DeviceOrientation*/
const Value V22503_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22503_304 = {S24, 0, &V22503_304_57227, &P22503_302};	/*Property = FileReader*/
const Value V22503_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22503_305 = {S25, 0, &V22503_305_57231, &P22503_304};	/*Property = FileSaver*/
const Value V22503_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22503_306 = {S26, 0, &V22503_306_57237, &P22503_305};	/*Property = FileWriter*/
const Value V22503_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22503_307 = {S27, 0, &V22503_307_57239, &P22503_306};	/*Property = FormData*/
const Value V22503_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22503_309 = {S28, 0, &V22503_309_57246, &P22503_307};	/*Property = Fullscreen*/
const Value V22503_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22503_310 = {S29, 0, &V22503_310_57248, &P22503_309};	/*Property = GeoLocation*/
const Value V22503_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22503_312 = {S30, 0, &V22503_312_57254, &P22503_310};	/*Property = History*/
const Value V22503_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22503_314 = {S31, 0, &V22503_314_57261, &P22503_312};	/*Property = Html5*/
const Value V22503_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22503_313 = {S32, 0, &V22503_313_57258, &P22503_314};	/*Property = Html-Media-Capture*/
const Value V22503_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22503_315 = {S33, 0, &V22503_315_57264, &P22503_313};	/*Property = Iframe*/
const Value V22503_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22503_316 = {S34, 0, &V22503_316_57266, &P22503_315};	/*Property = IndexedDB*/
const Value V22503_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22503_317 = {S35, 0, &V22503_317_57269, &P22503_316};	/*Property = Json*/
const Value V22503_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22503_15 = {S37, 0, &V22503_15_29, &P22503_317};	/*Property = LayoutEngine*/
const Value V22503_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22503_344 = {S38, 0, &V22503_344_57272, &P22503_15};	/*Property = Masking*/
const Value V22503_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22503_320 = {S39, 0, &V22503_320_57290, &P22503_344};	/*Property = PostMessage*/
const Value V22503_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22503_321 = {S40, 0, &V22503_321_57293, &P22503_320};	/*Property = Progress*/
const Value V22503_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22503_322 = {S41, 0, &V22503_322_57296, &P22503_321};	/*Property = Prompts*/
const Value V22503_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22503_323 = {S42, 0, &V22503_323_57299, &P22503_322};	/*Property = Selector*/
const Value V22503_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22503_324 = {S43, 0, &V22503_324_57305, &P22503_323};	/*Property = Svg*/
const Value V22503_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22503_327 = {S44, 0, &V22503_327_57315, &P22503_324};	/*Property = TouchEvents*/
const Value V22503_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22503_328 = {S45, 0, &V22503_328_57318, &P22503_327};	/*Property = Track*/
const Value V22503_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22503_329 = {S46, 0, &V22503_329_57323, &P22503_328};	/*Property = Video*/
const Value V22503_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22503_330 = {S47, 0, &V22503_330_57326, &P22503_329};	/*Property = Viewport*/
const Value V22503_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22503_332 = {S48, 0, &V22503_332_57338, &P22503_330};	/*Property = WebWorkers*/
const Value V22503_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22503_333 = {S49, 0, &V22503_333_57341, &P22503_332};	/*Property = Xhr2*/
const Profile P22503 = {"22503", &P22503_333};	/*Profile ID - 22503*/
const Profile P18133 = {"18133", NULL};	/*Profile ID - 18133*/
const Profile P18137 = {"18137", NULL};	/*Profile ID - 18137*/
const Value V21930_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21930_279 = {S1, 0, &V21930_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21930_282_57153 = {S2, NULL};	/*Value = True*/
const Property P21930_282 = {S3, 0, &V21930_282_57153, &P21930_279};	/*Property = BlobBuilder*/
const Value V21930_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21930_350 = {S4, 0, &V21930_350_57444, &P21930_282};	/*Property = Canvas*/
const Value V21930_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21930_283 = {S5, 0, &V21930_283_57159, &P21930_350};	/*Property = CssBackground*/
const Value V21930_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21930_284 = {S6, 0, &V21930_284_57163, &P21930_283};	/*Property = CssBorderImage*/
const Value V21930_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21930_285 = {S7, 0, &V21930_285_57166, &P21930_284};	/*Property = CssCanvas*/
const Value V21930_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21930_286 = {S8, 0, &V21930_286_57168, &P21930_285};	/*Property = CssColor*/
const Value V21930_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21930_237 = {S9, 0, &V21930_237_55786, &P21930_286};	/*Property = CssColumn*/
const Value V21930_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21930_287 = {S10, 0, &V21930_287_57172, &P21930_237};	/*Property = CssFlexbox*/
const Value V21930_288_57174 = {S2, NULL};	/*Value = True*/
const Property P21930_288 = {S11, 0, &V21930_288_57174, &P21930_287};	/*Property = CssFont*/
const Value V21930_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21930_289 = {S12, 0, &V21930_289_57759, &P21930_288};	/*Property = CssImages*/
const Value V21930_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21930_291 = {S13, 0, &V21930_291_57180, &P21930_289};	/*Property = CssMediaQueries*/
const Value V21930_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21930_292 = {S14, 0, &V21930_292_57183, &P21930_291};	/*Property = CssMinMax*/
const Value V21930_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21930_293 = {S15, 0, &V21930_293_57187, &P21930_292};	/*Property = CssOverflow*/
const Value V21930_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21930_294 = {S16, 0, &V21930_294_57190, &P21930_293};	/*Property = CssPosition*/
const Value V21930_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21930_295 = {S17, 0, &V21930_295_57192, &P21930_294};	/*Property = CssText*/
const Value V21930_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21930_297 = {S18, 0, &V21930_297_57195, &P21930_295};	/*Property = CssTransforms*/
const Value V21930_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21930_241 = {S19, 0, &V21930_241_55799, &P21930_297};	/*Property = CssTransitions*/
const Value V21930_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21930_298 = {S20, 0, &V21930_298_57206, &P21930_241};	/*Property = CssUI*/
const Value V21930_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21930_300 = {S21, 0, &V21930_300_57216, &P21930_298};	/*Property = DataSet*/
const Value V21930_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21930_301 = {S22, 0, &V21930_301_57218, &P21930_300};	/*Property = DataUrl*/
const Value V21930_302_57221 = {S2, NULL};	/*Value = True*/
const Property P21930_302 = {S23, 0, &V21930_302_57221, &P21930_301};	/*Property = DeviceOrientation*/
const Value V21930_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21930_304 = {S24, 0, &V21930_304_57227, &P21930_302};	/*Property = FileReader*/
const Value V21930_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21930_305 = {S25, 0, &V21930_305_57231, &P21930_304};	/*Property = FileSaver*/
const Value V21930_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21930_306 = {S26, 0, &V21930_306_57237, &P21930_305};	/*Property = FileWriter*/
const Value V21930_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21930_307 = {S27, 0, &V21930_307_57239, &P21930_306};	/*Property = FormData*/
const Value V21930_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21930_309 = {S28, 0, &V21930_309_57246, &P21930_307};	/*Property = Fullscreen*/
const Value V21930_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21930_310 = {S29, 0, &V21930_310_57248, &P21930_309};	/*Property = GeoLocation*/
const Value V21930_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21930_312 = {S30, 0, &V21930_312_57255, &P21930_310};	/*Property = History*/
const Value V21930_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21930_314 = {S31, 0, &V21930_314_57261, &P21930_312};	/*Property = Html5*/
const Value V21930_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21930_313 = {S32, 0, &V21930_313_57258, &P21930_314};	/*Property = Html-Media-Capture*/
const Value V21930_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21930_315 = {S33, 0, &V21930_315_57264, &P21930_313};	/*Property = Iframe*/
const Value V21930_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21930_316 = {S34, 0, &V21930_316_57267, &P21930_315};	/*Property = IndexedDB*/
const Value V21930_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21930_317 = {S35, 0, &V21930_317_57269, &P21930_316};	/*Property = Json*/
const Value V21930_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21930_15 = {S37, 0, &V21930_15_28, &P21930_317};	/*Property = LayoutEngine*/
const Value V21930_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21930_344 = {S38, 0, &V21930_344_57272, &P21930_15};	/*Property = Masking*/
const Value V21930_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21930_320 = {S39, 0, &V21930_320_57290, &P21930_344};	/*Property = PostMessage*/
const Value V21930_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21930_321 = {S40, 0, &V21930_321_57293, &P21930_320};	/*Property = Progress*/
const Value V21930_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21930_322 = {S41, 0, &V21930_322_57296, &P21930_321};	/*Property = Prompts*/
const Value V21930_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21930_323 = {S42, 0, &V21930_323_57299, &P21930_322};	/*Property = Selector*/
const Value V21930_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21930_324 = {S43, 0, &V21930_324_57306, &P21930_323};	/*Property = Svg*/
const Value V21930_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21930_327 = {S44, 0, &V21930_327_57315, &P21930_324};	/*Property = TouchEvents*/
const Value V21930_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21930_328 = {S45, 0, &V21930_328_57318, &P21930_327};	/*Property = Track*/
const Value V21930_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21930_329 = {S46, 0, &V21930_329_57323, &P21930_328};	/*Property = Video*/
const Value V21930_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21930_330 = {S47, 0, &V21930_330_57326, &P21930_329};	/*Property = Viewport*/
const Value V21930_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21930_332 = {S48, 0, &V21930_332_57339, &P21930_330};	/*Property = WebWorkers*/
const Value V21930_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21930_333 = {S49, 0, &V21930_333_57342, &P21930_332};	/*Property = Xhr2*/
const Profile P21930 = {"21930", &P21930_333};	/*Profile ID - 21930*/
const Value V23790_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_279 = {S1, 0, &V23790_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23790_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_282 = {S3, 0, &V23790_282_57155, &P23790_279};	/*Property = BlobBuilder*/
const Value V23790_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_350 = {S4, 0, &V23790_350_57446, &P23790_282};	/*Property = Canvas*/
const Value V23790_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_283 = {S5, 0, &V23790_283_57161, &P23790_350};	/*Property = CssBackground*/
const Value V23790_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_284 = {S6, 0, &V23790_284_57164, &P23790_283};	/*Property = CssBorderImage*/
const Value V23790_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_285 = {S7, 0, &V23790_285_57167, &P23790_284};	/*Property = CssCanvas*/
const Value V23790_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_286 = {S8, 0, &V23790_286_57170, &P23790_285};	/*Property = CssColor*/
const Value V23790_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_237 = {S9, 0, &V23790_237_55786, &P23790_286};	/*Property = CssColumn*/
const Value V23790_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_287 = {S10, 0, &V23790_287_57173, &P23790_237};	/*Property = CssFlexbox*/
const Value V23790_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_288 = {S11, 0, &V23790_288_57176, &P23790_287};	/*Property = CssFont*/
const Value V23790_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_289 = {S12, 0, &V23790_289_86899, &P23790_288};	/*Property = CssImages*/
const Value V23790_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_291 = {S13, 0, &V23790_291_57182, &P23790_289};	/*Property = CssMediaQueries*/
const Value V23790_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_292 = {S14, 0, &V23790_292_57185, &P23790_291};	/*Property = CssMinMax*/
const Value V23790_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_293 = {S15, 0, &V23790_293_57188, &P23790_292};	/*Property = CssOverflow*/
const Value V23790_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_294 = {S16, 0, &V23790_294_57191, &P23790_293};	/*Property = CssPosition*/
const Value V23790_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_295 = {S17, 0, &V23790_295_57194, &P23790_294};	/*Property = CssText*/
const Value V23790_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_297 = {S18, 0, &V23790_297_57197, &P23790_295};	/*Property = CssTransforms*/
const Value V23790_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_241 = {S19, 0, &V23790_241_55801, &P23790_297};	/*Property = CssTransitions*/
const Value V23790_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_298 = {S20, 0, &V23790_298_57208, &P23790_241};	/*Property = CssUI*/
const Value V23790_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_300 = {S21, 0, &V23790_300_57217, &P23790_298};	/*Property = DataSet*/
const Value V23790_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_301 = {S22, 0, &V23790_301_57220, &P23790_300};	/*Property = DataUrl*/
const Value V23790_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_302 = {S23, 0, &V23790_302_57223, &P23790_301};	/*Property = DeviceOrientation*/
const Value V23790_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_304 = {S24, 0, &V23790_304_57229, &P23790_302};	/*Property = FileReader*/
const Value V23790_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_305 = {S25, 0, &V23790_305_57232, &P23790_304};	/*Property = FileSaver*/
const Value V23790_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_306 = {S26, 0, &V23790_306_57238, &P23790_305};	/*Property = FileWriter*/
const Value V23790_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_307 = {S27, 0, &V23790_307_57241, &P23790_306};	/*Property = FormData*/
const Value V23790_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_309 = {S28, 0, &V23790_309_57247, &P23790_307};	/*Property = Fullscreen*/
const Value V23790_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_310 = {S29, 0, &V23790_310_57250, &P23790_309};	/*Property = GeoLocation*/
const Value V23790_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_312 = {S30, 0, &V23790_312_57256, &P23790_310};	/*Property = History*/
const Value V23790_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_314 = {S31, 0, &V23790_314_57262, &P23790_312};	/*Property = Html5*/
const Value V23790_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_313 = {S32, 0, &V23790_313_57259, &P23790_314};	/*Property = Html-Media-Capture*/
const Value V23790_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_315 = {S33, 0, &V23790_315_57265, &P23790_313};	/*Property = Iframe*/
const Value V23790_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_316 = {S34, 0, &V23790_316_57268, &P23790_315};	/*Property = IndexedDB*/
const Value V23790_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_317 = {S35, 0, &V23790_317_57271, &P23790_316};	/*Property = Json*/
const Value V23790_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_15 = {S37, 0, &V23790_15_72, &P23790_317};	/*Property = LayoutEngine*/
const Value V23790_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_344 = {S38, 0, &V23790_344_57274, &P23790_15};	/*Property = Masking*/
const Value V23790_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_320 = {S39, 0, &V23790_320_57292, &P23790_344};	/*Property = PostMessage*/
const Value V23790_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_321 = {S40, 0, &V23790_321_57295, &P23790_320};	/*Property = Progress*/
const Value V23790_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_322 = {S41, 0, &V23790_322_57298, &P23790_321};	/*Property = Prompts*/
const Value V23790_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_323 = {S42, 0, &V23790_323_57301, &P23790_322};	/*Property = Selector*/
const Value V23790_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_324 = {S43, 0, &V23790_324_57307, &P23790_323};	/*Property = Svg*/
const Value V23790_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_327 = {S44, 0, &V23790_327_57316, &P23790_324};	/*Property = TouchEvents*/
const Value V23790_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_328 = {S45, 0, &V23790_328_57319, &P23790_327};	/*Property = Track*/
const Value V23790_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_329 = {S46, 0, &V23790_329_57325, &P23790_328};	/*Property = Video*/
const Value V23790_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_330 = {S47, 0, &V23790_330_57328, &P23790_329};	/*Property = Viewport*/
const Value V23790_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_332 = {S48, 0, &V23790_332_57340, &P23790_330};	/*Property = WebWorkers*/
const Value V23790_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23790_333 = {S49, 0, &V23790_333_57343, &P23790_332};	/*Property = Xhr2*/
const Profile P23790 = {"23790", &P23790_333};	/*Profile ID - 23790*/
const Value V20441_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20441_279 = {S1, 0, &V20441_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20441_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20441_282 = {S3, 0, &V20441_282_57154, &P20441_279};	/*Property = BlobBuilder*/
const Value V20441_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20441_350 = {S4, 0, &V20441_350_57444, &P20441_282};	/*Property = Canvas*/
const Value V20441_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20441_283 = {S5, 0, &V20441_283_57159, &P20441_350};	/*Property = CssBackground*/
const Value V20441_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20441_284 = {S6, 0, &V20441_284_57163, &P20441_283};	/*Property = CssBorderImage*/
const Value V20441_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20441_285 = {S7, 0, &V20441_285_57166, &P20441_284};	/*Property = CssCanvas*/
const Value V20441_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20441_286 = {S8, 0, &V20441_286_57168, &P20441_285};	/*Property = CssColor*/
const Value V20441_237_55784 = {S2, NULL};	/*Value = True*/
const Property P20441_237 = {S9, 0, &V20441_237_55784, &P20441_286};	/*Property = CssColumn*/
const Value V20441_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20441_287 = {S10, 0, &V20441_287_57172, &P20441_237};	/*Property = CssFlexbox*/
const Value V20441_288_57174 = {S2, NULL};	/*Value = True*/
const Property P20441_288 = {S11, 0, &V20441_288_57174, &P20441_287};	/*Property = CssFont*/
const Value V20441_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20441_289 = {S12, 0, &V20441_289_57759, &P20441_288};	/*Property = CssImages*/
const Value V20441_291_57180 = {S2, NULL};	/*Value = True*/
const Property P20441_291 = {S13, 0, &V20441_291_57180, &P20441_289};	/*Property = CssMediaQueries*/
const Value V20441_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20441_292 = {S14, 0, &V20441_292_57183, &P20441_291};	/*Property = CssMinMax*/
const Value V20441_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20441_293 = {S15, 0, &V20441_293_57187, &P20441_292};	/*Property = CssOverflow*/
const Value V20441_294_57189 = {S2, NULL};	/*Value = True*/
const Property P20441_294 = {S16, 0, &V20441_294_57189, &P20441_293};	/*Property = CssPosition*/
const Value V20441_295_57192 = {S2, NULL};	/*Value = True*/
const Property P20441_295 = {S17, 0, &V20441_295_57192, &P20441_294};	/*Property = CssText*/
const Value V20441_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20441_297 = {S18, 0, &V20441_297_57195, &P20441_295};	/*Property = CssTransforms*/
const Value V20441_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20441_241 = {S19, 0, &V20441_241_55799, &P20441_297};	/*Property = CssTransitions*/
const Value V20441_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20441_298 = {S20, 0, &V20441_298_57206, &P20441_241};	/*Property = CssUI*/
const Value V20441_300_57215 = {S2, NULL};	/*Value = True*/
const Property P20441_300 = {S21, 0, &V20441_300_57215, &P20441_298};	/*Property = DataSet*/
const Value V20441_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20441_301 = {S22, 0, &V20441_301_57218, &P20441_300};	/*Property = DataUrl*/
const Value V20441_302_57221 = {S2, NULL};	/*Value = True*/
const Property P20441_302 = {S23, 0, &V20441_302_57221, &P20441_301};	/*Property = DeviceOrientation*/
const Value V20441_304_57227 = {S2, NULL};	/*Value = True*/
const Property P20441_304 = {S24, 0, &V20441_304_57227, &P20441_302};	/*Property = FileReader*/
const Value V20441_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20441_305 = {S25, 0, &V20441_305_57231, &P20441_304};	/*Property = FileSaver*/
const Value V20441_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20441_306 = {S26, 0, &V20441_306_57237, &P20441_305};	/*Property = FileWriter*/
const Value V20441_307_57239 = {S2, NULL};	/*Value = True*/
const Property P20441_307 = {S27, 0, &V20441_307_57239, &P20441_306};	/*Property = FormData*/
const Value V20441_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20441_309 = {S28, 0, &V20441_309_57246, &P20441_307};	/*Property = Fullscreen*/
const Value V20441_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20441_310 = {S29, 0, &V20441_310_57248, &P20441_309};	/*Property = GeoLocation*/
const Value V20441_312_57254 = {S2, NULL};	/*Value = True*/
const Property P20441_312 = {S30, 0, &V20441_312_57254, &P20441_310};	/*Property = History*/
const Value V20441_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20441_314 = {S31, 0, &V20441_314_57261, &P20441_312};	/*Property = Html5*/
const Value V20441_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20441_313 = {S32, 0, &V20441_313_57258, &P20441_314};	/*Property = Html-Media-Capture*/
const Value V20441_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20441_315 = {S33, 0, &V20441_315_57264, &P20441_313};	/*Property = Iframe*/
const Value V20441_316_57266 = {S2, NULL};	/*Value = True*/
const Property P20441_316 = {S34, 0, &V20441_316_57266, &P20441_315};	/*Property = IndexedDB*/
const Value V20441_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20441_317 = {S35, 0, &V20441_317_57269, &P20441_316};	/*Property = Json*/
const Value V20441_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20441_15 = {S37, 0, &V20441_15_29, &P20441_317};	/*Property = LayoutEngine*/
const Value V20441_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20441_344 = {S38, 0, &V20441_344_57272, &P20441_15};	/*Property = Masking*/
const Value V20441_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20441_320 = {S39, 0, &V20441_320_57290, &P20441_344};	/*Property = PostMessage*/
const Value V20441_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20441_321 = {S40, 0, &V20441_321_57293, &P20441_320};	/*Property = Progress*/
const Value V20441_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20441_322 = {S41, 0, &V20441_322_57296, &P20441_321};	/*Property = Prompts*/
const Value V20441_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20441_323 = {S42, 0, &V20441_323_57299, &P20441_322};	/*Property = Selector*/
const Value V20441_324_57305 = {S2, NULL};	/*Value = True*/
const Property P20441_324 = {S43, 0, &V20441_324_57305, &P20441_323};	/*Property = Svg*/
const Value V20441_327_57315 = {S0, NULL};	/*Value = False*/
const Property P20441_327 = {S44, 0, &V20441_327_57315, &P20441_324};	/*Property = TouchEvents*/
const Value V20441_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20441_328 = {S45, 0, &V20441_328_57318, &P20441_327};	/*Property = Track*/
const Value V20441_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20441_329 = {S46, 0, &V20441_329_57323, &P20441_328};	/*Property = Video*/
const Value V20441_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20441_330 = {S47, 0, &V20441_330_57326, &P20441_329};	/*Property = Viewport*/
const Value V20441_332_57338 = {S2, NULL};	/*Value = True*/
const Property P20441_332 = {S48, 0, &V20441_332_57338, &P20441_330};	/*Property = WebWorkers*/
const Value V20441_333_57341 = {S2, NULL};	/*Value = True*/
const Property P20441_333 = {S49, 0, &V20441_333_57341, &P20441_332};	/*Property = Xhr2*/
const Profile P20441 = {"20441", &P20441_333};	/*Profile ID - 20441*/
const Value V23376_1_1 = {S2, NULL};	/*Value = True*/
const Property P23376_1 = {S50, 0, &V23376_1_1, NULL};	/*Property = IsMobile*/
const Value V23376_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P23376_5 = {S52, 0, &V23376_5_1304, &P23376_1};	/*Property = ScreenPixelsHeight*/
const Value V23376_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23376_4 = {S54, 0, &V23376_4_2336, &P23376_5};	/*Property = ScreenPixelsWidth*/
const Profile P23376 = {"23376", &P23376_4};	/*Profile ID - 23376*/
const Value V20600_1_1 = {S2, NULL};	/*Value = True*/
const Property P20600_1 = {S50, 0, &V20600_1_1, NULL};	/*Property = IsMobile*/
const Value V20600_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20600_5 = {S52, 0, &V20600_5_1227, &P20600_1};	/*Property = ScreenPixelsHeight*/
const Value V20600_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20600_4 = {S54, 0, &V20600_4_1229, &P20600_5};	/*Property = ScreenPixelsWidth*/
const Profile P20600 = {"20600", &P20600_4};	/*Profile ID - 20600*/
const Value V17974_1_1 = {S2, NULL};	/*Value = True*/
const Property P17974_1 = {S50, 0, &V17974_1_1, NULL};	/*Property = IsMobile*/
const Value V17974_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17974_5 = {S52, 0, &V17974_5_1227, &P17974_1};	/*Property = ScreenPixelsHeight*/
const Value V17974_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17974_4 = {S54, 0, &V17974_4_1229, &P17974_5};	/*Property = ScreenPixelsWidth*/
const Profile P17974 = {"17974", &P17974_4};	/*Profile ID - 17974*/
const Value V22306_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_279 = {S1, 0, &V22306_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22306_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_282 = {S3, 0, &V22306_282_57155, &P22306_279};	/*Property = BlobBuilder*/
const Value V22306_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_350 = {S4, 0, &V22306_350_57446, &P22306_282};	/*Property = Canvas*/
const Value V22306_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_283 = {S5, 0, &V22306_283_57161, &P22306_350};	/*Property = CssBackground*/
const Value V22306_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_284 = {S6, 0, &V22306_284_57164, &P22306_283};	/*Property = CssBorderImage*/
const Value V22306_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_285 = {S7, 0, &V22306_285_57167, &P22306_284};	/*Property = CssCanvas*/
const Value V22306_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_286 = {S8, 0, &V22306_286_57170, &P22306_285};	/*Property = CssColor*/
const Value V22306_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_237 = {S9, 0, &V22306_237_55786, &P22306_286};	/*Property = CssColumn*/
const Value V22306_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_287 = {S10, 0, &V22306_287_57173, &P22306_237};	/*Property = CssFlexbox*/
const Value V22306_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_288 = {S11, 0, &V22306_288_57176, &P22306_287};	/*Property = CssFont*/
const Value V22306_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_289 = {S12, 0, &V22306_289_86899, &P22306_288};	/*Property = CssImages*/
const Value V22306_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_291 = {S13, 0, &V22306_291_57182, &P22306_289};	/*Property = CssMediaQueries*/
const Value V22306_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_292 = {S14, 0, &V22306_292_57185, &P22306_291};	/*Property = CssMinMax*/
const Value V22306_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_293 = {S15, 0, &V22306_293_57188, &P22306_292};	/*Property = CssOverflow*/
const Value V22306_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_294 = {S16, 0, &V22306_294_57191, &P22306_293};	/*Property = CssPosition*/
const Value V22306_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_295 = {S17, 0, &V22306_295_57194, &P22306_294};	/*Property = CssText*/
const Value V22306_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_297 = {S18, 0, &V22306_297_57197, &P22306_295};	/*Property = CssTransforms*/
const Value V22306_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_241 = {S19, 0, &V22306_241_55801, &P22306_297};	/*Property = CssTransitions*/
const Value V22306_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_298 = {S20, 0, &V22306_298_57208, &P22306_241};	/*Property = CssUI*/
const Value V22306_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_300 = {S21, 0, &V22306_300_57217, &P22306_298};	/*Property = DataSet*/
const Value V22306_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_301 = {S22, 0, &V22306_301_57220, &P22306_300};	/*Property = DataUrl*/
const Value V22306_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_302 = {S23, 0, &V22306_302_57223, &P22306_301};	/*Property = DeviceOrientation*/
const Value V22306_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_304 = {S24, 0, &V22306_304_57229, &P22306_302};	/*Property = FileReader*/
const Value V22306_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_305 = {S25, 0, &V22306_305_57232, &P22306_304};	/*Property = FileSaver*/
const Value V22306_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_306 = {S26, 0, &V22306_306_57238, &P22306_305};	/*Property = FileWriter*/
const Value V22306_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_307 = {S27, 0, &V22306_307_57241, &P22306_306};	/*Property = FormData*/
const Value V22306_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_309 = {S28, 0, &V22306_309_57247, &P22306_307};	/*Property = Fullscreen*/
const Value V22306_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_310 = {S29, 0, &V22306_310_57250, &P22306_309};	/*Property = GeoLocation*/
const Value V22306_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_312 = {S30, 0, &V22306_312_57256, &P22306_310};	/*Property = History*/
const Value V22306_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_314 = {S31, 0, &V22306_314_57262, &P22306_312};	/*Property = Html5*/
const Value V22306_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_313 = {S32, 0, &V22306_313_57259, &P22306_314};	/*Property = Html-Media-Capture*/
const Value V22306_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_315 = {S33, 0, &V22306_315_57265, &P22306_313};	/*Property = Iframe*/
const Value V22306_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_316 = {S34, 0, &V22306_316_57268, &P22306_315};	/*Property = IndexedDB*/
const Value V22306_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_317 = {S35, 0, &V22306_317_57271, &P22306_316};	/*Property = Json*/
const Value V22306_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22306_15 = {S37, 0, &V22306_15_29, &P22306_317};	/*Property = LayoutEngine*/
const Value V22306_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_344 = {S38, 0, &V22306_344_57274, &P22306_15};	/*Property = Masking*/
const Value V22306_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_320 = {S39, 0, &V22306_320_57292, &P22306_344};	/*Property = PostMessage*/
const Value V22306_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_321 = {S40, 0, &V22306_321_57295, &P22306_320};	/*Property = Progress*/
const Value V22306_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_322 = {S41, 0, &V22306_322_57298, &P22306_321};	/*Property = Prompts*/
const Value V22306_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_323 = {S42, 0, &V22306_323_57301, &P22306_322};	/*Property = Selector*/
const Value V22306_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_324 = {S43, 0, &V22306_324_57307, &P22306_323};	/*Property = Svg*/
const Value V22306_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_327 = {S44, 0, &V22306_327_57316, &P22306_324};	/*Property = TouchEvents*/
const Value V22306_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_328 = {S45, 0, &V22306_328_57319, &P22306_327};	/*Property = Track*/
const Value V22306_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_329 = {S46, 0, &V22306_329_57325, &P22306_328};	/*Property = Video*/
const Value V22306_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_330 = {S47, 0, &V22306_330_57328, &P22306_329};	/*Property = Viewport*/
const Value V22306_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_332 = {S48, 0, &V22306_332_57340, &P22306_330};	/*Property = WebWorkers*/
const Value V22306_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22306_333 = {S49, 0, &V22306_333_57343, &P22306_332};	/*Property = Xhr2*/
const Profile P22306 = {"22306", &P22306_333};	/*Profile ID - 22306*/
const Profile P22188 = {"22188", NULL};	/*Profile ID - 22188*/
const Profile P18119 = {"18119", NULL};	/*Profile ID - 18119*/
const Value V21549_1_1 = {S2, NULL};	/*Value = True*/
const Property P21549_1 = {S50, 0, &V21549_1_1, NULL};	/*Property = IsMobile*/
const Value V21549_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21549_5 = {S52, 0, &V21549_5_1277, &P21549_1};	/*Property = ScreenPixelsHeight*/
const Value V21549_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21549_4 = {S54, 0, &V21549_4_1279, &P21549_5};	/*Property = ScreenPixelsWidth*/
const Profile P21549 = {"21549", &P21549_4};	/*Profile ID - 21549*/
const Value V17209_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_279 = {S1, 0, &V17209_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17209_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_282 = {S3, 0, &V17209_282_57155, &P17209_279};	/*Property = BlobBuilder*/
const Value V17209_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_350 = {S4, 0, &V17209_350_57446, &P17209_282};	/*Property = Canvas*/
const Value V17209_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_283 = {S5, 0, &V17209_283_57161, &P17209_350};	/*Property = CssBackground*/
const Value V17209_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_284 = {S6, 0, &V17209_284_57164, &P17209_283};	/*Property = CssBorderImage*/
const Value V17209_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_285 = {S7, 0, &V17209_285_57167, &P17209_284};	/*Property = CssCanvas*/
const Value V17209_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_286 = {S8, 0, &V17209_286_57170, &P17209_285};	/*Property = CssColor*/
const Value V17209_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_237 = {S9, 0, &V17209_237_55786, &P17209_286};	/*Property = CssColumn*/
const Value V17209_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_287 = {S10, 0, &V17209_287_57173, &P17209_237};	/*Property = CssFlexbox*/
const Value V17209_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_288 = {S11, 0, &V17209_288_57176, &P17209_287};	/*Property = CssFont*/
const Value V17209_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_289 = {S12, 0, &V17209_289_86899, &P17209_288};	/*Property = CssImages*/
const Value V17209_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_291 = {S13, 0, &V17209_291_57182, &P17209_289};	/*Property = CssMediaQueries*/
const Value V17209_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_292 = {S14, 0, &V17209_292_57185, &P17209_291};	/*Property = CssMinMax*/
const Value V17209_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_293 = {S15, 0, &V17209_293_57188, &P17209_292};	/*Property = CssOverflow*/
const Value V17209_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_294 = {S16, 0, &V17209_294_57191, &P17209_293};	/*Property = CssPosition*/
const Value V17209_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_295 = {S17, 0, &V17209_295_57194, &P17209_294};	/*Property = CssText*/
const Value V17209_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_297 = {S18, 0, &V17209_297_57197, &P17209_295};	/*Property = CssTransforms*/
const Value V17209_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_241 = {S19, 0, &V17209_241_55801, &P17209_297};	/*Property = CssTransitions*/
const Value V17209_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_298 = {S20, 0, &V17209_298_57208, &P17209_241};	/*Property = CssUI*/
const Value V17209_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_300 = {S21, 0, &V17209_300_57217, &P17209_298};	/*Property = DataSet*/
const Value V17209_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_301 = {S22, 0, &V17209_301_57220, &P17209_300};	/*Property = DataUrl*/
const Value V17209_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_302 = {S23, 0, &V17209_302_57223, &P17209_301};	/*Property = DeviceOrientation*/
const Value V17209_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_304 = {S24, 0, &V17209_304_57229, &P17209_302};	/*Property = FileReader*/
const Value V17209_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_305 = {S25, 0, &V17209_305_57232, &P17209_304};	/*Property = FileSaver*/
const Value V17209_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_306 = {S26, 0, &V17209_306_57238, &P17209_305};	/*Property = FileWriter*/
const Value V17209_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_307 = {S27, 0, &V17209_307_57241, &P17209_306};	/*Property = FormData*/
const Value V17209_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_309 = {S28, 0, &V17209_309_57247, &P17209_307};	/*Property = Fullscreen*/
const Value V17209_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_310 = {S29, 0, &V17209_310_57250, &P17209_309};	/*Property = GeoLocation*/
const Value V17209_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_312 = {S30, 0, &V17209_312_57256, &P17209_310};	/*Property = History*/
const Value V17209_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_314 = {S31, 0, &V17209_314_57262, &P17209_312};	/*Property = Html5*/
const Value V17209_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_313 = {S32, 0, &V17209_313_57259, &P17209_314};	/*Property = Html-Media-Capture*/
const Value V17209_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_315 = {S33, 0, &V17209_315_57265, &P17209_313};	/*Property = Iframe*/
const Value V17209_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_316 = {S34, 0, &V17209_316_57268, &P17209_315};	/*Property = IndexedDB*/
const Value V17209_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_317 = {S35, 0, &V17209_317_57271, &P17209_316};	/*Property = Json*/
const Value V17209_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P17209_15 = {S37, 0, &V17209_15_29, &P17209_317};	/*Property = LayoutEngine*/
const Value V17209_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_344 = {S38, 0, &V17209_344_57274, &P17209_15};	/*Property = Masking*/
const Value V17209_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_320 = {S39, 0, &V17209_320_57292, &P17209_344};	/*Property = PostMessage*/
const Value V17209_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_321 = {S40, 0, &V17209_321_57295, &P17209_320};	/*Property = Progress*/
const Value V17209_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_322 = {S41, 0, &V17209_322_57298, &P17209_321};	/*Property = Prompts*/
const Value V17209_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_323 = {S42, 0, &V17209_323_57301, &P17209_322};	/*Property = Selector*/
const Value V17209_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_324 = {S43, 0, &V17209_324_57307, &P17209_323};	/*Property = Svg*/
const Value V17209_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_327 = {S44, 0, &V17209_327_57316, &P17209_324};	/*Property = TouchEvents*/
const Value V17209_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_328 = {S45, 0, &V17209_328_57319, &P17209_327};	/*Property = Track*/
const Value V17209_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_329 = {S46, 0, &V17209_329_57325, &P17209_328};	/*Property = Video*/
const Value V17209_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_330 = {S47, 0, &V17209_330_57328, &P17209_329};	/*Property = Viewport*/
const Value V17209_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_332 = {S48, 0, &V17209_332_57340, &P17209_330};	/*Property = WebWorkers*/
const Value V17209_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17209_333 = {S49, 0, &V17209_333_57343, &P17209_332};	/*Property = Xhr2*/
const Profile P17209 = {"17209", &P17209_333};	/*Profile ID - 17209*/
const Value V22897_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_279 = {S1, 0, &V22897_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22897_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_282 = {S3, 0, &V22897_282_57155, &P22897_279};	/*Property = BlobBuilder*/
const Value V22897_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_350 = {S4, 0, &V22897_350_57446, &P22897_282};	/*Property = Canvas*/
const Value V22897_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_283 = {S5, 0, &V22897_283_57161, &P22897_350};	/*Property = CssBackground*/
const Value V22897_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_284 = {S6, 0, &V22897_284_57164, &P22897_283};	/*Property = CssBorderImage*/
const Value V22897_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_285 = {S7, 0, &V22897_285_57167, &P22897_284};	/*Property = CssCanvas*/
const Value V22897_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_286 = {S8, 0, &V22897_286_57170, &P22897_285};	/*Property = CssColor*/
const Value V22897_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_237 = {S9, 0, &V22897_237_55786, &P22897_286};	/*Property = CssColumn*/
const Value V22897_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_287 = {S10, 0, &V22897_287_57173, &P22897_237};	/*Property = CssFlexbox*/
const Value V22897_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_288 = {S11, 0, &V22897_288_57176, &P22897_287};	/*Property = CssFont*/
const Value V22897_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_289 = {S12, 0, &V22897_289_86899, &P22897_288};	/*Property = CssImages*/
const Value V22897_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_291 = {S13, 0, &V22897_291_57182, &P22897_289};	/*Property = CssMediaQueries*/
const Value V22897_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_292 = {S14, 0, &V22897_292_57185, &P22897_291};	/*Property = CssMinMax*/
const Value V22897_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_293 = {S15, 0, &V22897_293_57188, &P22897_292};	/*Property = CssOverflow*/
const Value V22897_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_294 = {S16, 0, &V22897_294_57191, &P22897_293};	/*Property = CssPosition*/
const Value V22897_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_295 = {S17, 0, &V22897_295_57194, &P22897_294};	/*Property = CssText*/
const Value V22897_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_297 = {S18, 0, &V22897_297_57197, &P22897_295};	/*Property = CssTransforms*/
const Value V22897_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_241 = {S19, 0, &V22897_241_55801, &P22897_297};	/*Property = CssTransitions*/
const Value V22897_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_298 = {S20, 0, &V22897_298_57208, &P22897_241};	/*Property = CssUI*/
const Value V22897_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_300 = {S21, 0, &V22897_300_57217, &P22897_298};	/*Property = DataSet*/
const Value V22897_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_301 = {S22, 0, &V22897_301_57220, &P22897_300};	/*Property = DataUrl*/
const Value V22897_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_302 = {S23, 0, &V22897_302_57223, &P22897_301};	/*Property = DeviceOrientation*/
const Value V22897_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_304 = {S24, 0, &V22897_304_57229, &P22897_302};	/*Property = FileReader*/
const Value V22897_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_305 = {S25, 0, &V22897_305_57232, &P22897_304};	/*Property = FileSaver*/
const Value V22897_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_306 = {S26, 0, &V22897_306_57238, &P22897_305};	/*Property = FileWriter*/
const Value V22897_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_307 = {S27, 0, &V22897_307_57241, &P22897_306};	/*Property = FormData*/
const Value V22897_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_309 = {S28, 0, &V22897_309_57247, &P22897_307};	/*Property = Fullscreen*/
const Value V22897_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_310 = {S29, 0, &V22897_310_57250, &P22897_309};	/*Property = GeoLocation*/
const Value V22897_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_312 = {S30, 0, &V22897_312_57256, &P22897_310};	/*Property = History*/
const Value V22897_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_314 = {S31, 0, &V22897_314_57262, &P22897_312};	/*Property = Html5*/
const Value V22897_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_313 = {S32, 0, &V22897_313_57259, &P22897_314};	/*Property = Html-Media-Capture*/
const Value V22897_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_315 = {S33, 0, &V22897_315_57265, &P22897_313};	/*Property = Iframe*/
const Value V22897_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_316 = {S34, 0, &V22897_316_57268, &P22897_315};	/*Property = IndexedDB*/
const Value V22897_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_317 = {S35, 0, &V22897_317_57271, &P22897_316};	/*Property = Json*/
const Value V22897_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_15 = {S37, 0, &V22897_15_72, &P22897_317};	/*Property = LayoutEngine*/
const Value V22897_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_344 = {S38, 0, &V22897_344_57274, &P22897_15};	/*Property = Masking*/
const Value V22897_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_320 = {S39, 0, &V22897_320_57292, &P22897_344};	/*Property = PostMessage*/
const Value V22897_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_321 = {S40, 0, &V22897_321_57295, &P22897_320};	/*Property = Progress*/
const Value V22897_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_322 = {S41, 0, &V22897_322_57298, &P22897_321};	/*Property = Prompts*/
const Value V22897_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_323 = {S42, 0, &V22897_323_57301, &P22897_322};	/*Property = Selector*/
const Value V22897_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_324 = {S43, 0, &V22897_324_57307, &P22897_323};	/*Property = Svg*/
const Value V22897_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_327 = {S44, 0, &V22897_327_57316, &P22897_324};	/*Property = TouchEvents*/
const Value V22897_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_328 = {S45, 0, &V22897_328_57319, &P22897_327};	/*Property = Track*/
const Value V22897_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_329 = {S46, 0, &V22897_329_57325, &P22897_328};	/*Property = Video*/
const Value V22897_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_330 = {S47, 0, &V22897_330_57328, &P22897_329};	/*Property = Viewport*/
const Value V22897_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_332 = {S48, 0, &V22897_332_57340, &P22897_330};	/*Property = WebWorkers*/
const Value V22897_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22897_333 = {S49, 0, &V22897_333_57343, &P22897_332};	/*Property = Xhr2*/
const Profile P22897 = {"22897", &P22897_333};	/*Profile ID - 22897*/
const Value V18194_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_279 = {S1, 0, &V18194_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18194_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_282 = {S3, 0, &V18194_282_57155, &P18194_279};	/*Property = BlobBuilder*/
const Value V18194_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_350 = {S4, 0, &V18194_350_57446, &P18194_282};	/*Property = Canvas*/
const Value V18194_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_283 = {S5, 0, &V18194_283_57161, &P18194_350};	/*Property = CssBackground*/
const Value V18194_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_284 = {S6, 0, &V18194_284_57164, &P18194_283};	/*Property = CssBorderImage*/
const Value V18194_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_285 = {S7, 0, &V18194_285_57167, &P18194_284};	/*Property = CssCanvas*/
const Value V18194_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_286 = {S8, 0, &V18194_286_57170, &P18194_285};	/*Property = CssColor*/
const Value V18194_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_237 = {S9, 0, &V18194_237_55786, &P18194_286};	/*Property = CssColumn*/
const Value V18194_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_287 = {S10, 0, &V18194_287_57173, &P18194_237};	/*Property = CssFlexbox*/
const Value V18194_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_288 = {S11, 0, &V18194_288_57176, &P18194_287};	/*Property = CssFont*/
const Value V18194_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_289 = {S12, 0, &V18194_289_86899, &P18194_288};	/*Property = CssImages*/
const Value V18194_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_291 = {S13, 0, &V18194_291_57182, &P18194_289};	/*Property = CssMediaQueries*/
const Value V18194_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_292 = {S14, 0, &V18194_292_57185, &P18194_291};	/*Property = CssMinMax*/
const Value V18194_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_293 = {S15, 0, &V18194_293_57188, &P18194_292};	/*Property = CssOverflow*/
const Value V18194_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_294 = {S16, 0, &V18194_294_57191, &P18194_293};	/*Property = CssPosition*/
const Value V18194_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_295 = {S17, 0, &V18194_295_57194, &P18194_294};	/*Property = CssText*/
const Value V18194_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_297 = {S18, 0, &V18194_297_57197, &P18194_295};	/*Property = CssTransforms*/
const Value V18194_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_241 = {S19, 0, &V18194_241_55801, &P18194_297};	/*Property = CssTransitions*/
const Value V18194_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_298 = {S20, 0, &V18194_298_57208, &P18194_241};	/*Property = CssUI*/
const Value V18194_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_300 = {S21, 0, &V18194_300_57217, &P18194_298};	/*Property = DataSet*/
const Value V18194_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_301 = {S22, 0, &V18194_301_57220, &P18194_300};	/*Property = DataUrl*/
const Value V18194_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_302 = {S23, 0, &V18194_302_57223, &P18194_301};	/*Property = DeviceOrientation*/
const Value V18194_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_304 = {S24, 0, &V18194_304_57229, &P18194_302};	/*Property = FileReader*/
const Value V18194_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_305 = {S25, 0, &V18194_305_57232, &P18194_304};	/*Property = FileSaver*/
const Value V18194_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_306 = {S26, 0, &V18194_306_57238, &P18194_305};	/*Property = FileWriter*/
const Value V18194_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_307 = {S27, 0, &V18194_307_57241, &P18194_306};	/*Property = FormData*/
const Value V18194_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_309 = {S28, 0, &V18194_309_57247, &P18194_307};	/*Property = Fullscreen*/
const Value V18194_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_310 = {S29, 0, &V18194_310_57250, &P18194_309};	/*Property = GeoLocation*/
const Value V18194_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_312 = {S30, 0, &V18194_312_57256, &P18194_310};	/*Property = History*/
const Value V18194_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_314 = {S31, 0, &V18194_314_57262, &P18194_312};	/*Property = Html5*/
const Value V18194_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_313 = {S32, 0, &V18194_313_57259, &P18194_314};	/*Property = Html-Media-Capture*/
const Value V18194_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_315 = {S33, 0, &V18194_315_57265, &P18194_313};	/*Property = Iframe*/
const Value V18194_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_316 = {S34, 0, &V18194_316_57268, &P18194_315};	/*Property = IndexedDB*/
const Value V18194_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_317 = {S35, 0, &V18194_317_57271, &P18194_316};	/*Property = Json*/
const Value V18194_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18194_15 = {S37, 0, &V18194_15_29, &P18194_317};	/*Property = LayoutEngine*/
const Value V18194_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_344 = {S38, 0, &V18194_344_57274, &P18194_15};	/*Property = Masking*/
const Value V18194_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_320 = {S39, 0, &V18194_320_57292, &P18194_344};	/*Property = PostMessage*/
const Value V18194_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_321 = {S40, 0, &V18194_321_57295, &P18194_320};	/*Property = Progress*/
const Value V18194_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_322 = {S41, 0, &V18194_322_57298, &P18194_321};	/*Property = Prompts*/
const Value V18194_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_323 = {S42, 0, &V18194_323_57301, &P18194_322};	/*Property = Selector*/
const Value V18194_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_324 = {S43, 0, &V18194_324_57307, &P18194_323};	/*Property = Svg*/
const Value V18194_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_327 = {S44, 0, &V18194_327_57316, &P18194_324};	/*Property = TouchEvents*/
const Value V18194_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_328 = {S45, 0, &V18194_328_57319, &P18194_327};	/*Property = Track*/
const Value V18194_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_329 = {S46, 0, &V18194_329_57325, &P18194_328};	/*Property = Video*/
const Value V18194_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_330 = {S47, 0, &V18194_330_57328, &P18194_329};	/*Property = Viewport*/
const Value V18194_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_332 = {S48, 0, &V18194_332_57340, &P18194_330};	/*Property = WebWorkers*/
const Value V18194_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18194_333 = {S49, 0, &V18194_333_57343, &P18194_332};	/*Property = Xhr2*/
const Profile P18194 = {"18194", &P18194_333};	/*Profile ID - 18194*/
const Value V15820_1_1 = {S2, NULL};	/*Value = True*/
const Property P15820_1 = {S50, 0, &V15820_1_1, NULL};	/*Property = IsMobile*/
const Value V15820_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P15820_5 = {S52, 0, &V15820_5_9981, &P15820_1};	/*Property = ScreenPixelsHeight*/
const Value V15820_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P15820_4 = {S54, 0, &V15820_4_39098, &P15820_5};	/*Property = ScreenPixelsWidth*/
const Profile P15820 = {"15820", &P15820_4};	/*Profile ID - 15820*/
const Value V20490_1_1 = {S2, NULL};	/*Value = True*/
const Property P20490_1 = {S50, 0, &V20490_1_1, NULL};	/*Property = IsMobile*/
const Value V20490_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20490_5 = {S52, 0, &V20490_5_1222, &P20490_1};	/*Property = ScreenPixelsHeight*/
const Value V20490_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20490_4 = {S54, 0, &V20490_4_1224, &P20490_5};	/*Property = ScreenPixelsWidth*/
const Profile P20490 = {"20490", &P20490_4};	/*Profile ID - 20490*/
const Value V20795_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_279 = {S1, 0, &V20795_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20795_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_282 = {S3, 0, &V20795_282_57155, &P20795_279};	/*Property = BlobBuilder*/
const Value V20795_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_350 = {S4, 0, &V20795_350_57446, &P20795_282};	/*Property = Canvas*/
const Value V20795_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_283 = {S5, 0, &V20795_283_57161, &P20795_350};	/*Property = CssBackground*/
const Value V20795_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_284 = {S6, 0, &V20795_284_57164, &P20795_283};	/*Property = CssBorderImage*/
const Value V20795_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_285 = {S7, 0, &V20795_285_57167, &P20795_284};	/*Property = CssCanvas*/
const Value V20795_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_286 = {S8, 0, &V20795_286_57170, &P20795_285};	/*Property = CssColor*/
const Value V20795_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_237 = {S9, 0, &V20795_237_55786, &P20795_286};	/*Property = CssColumn*/
const Value V20795_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_287 = {S10, 0, &V20795_287_57173, &P20795_237};	/*Property = CssFlexbox*/
const Value V20795_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_288 = {S11, 0, &V20795_288_57176, &P20795_287};	/*Property = CssFont*/
const Value V20795_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_289 = {S12, 0, &V20795_289_86899, &P20795_288};	/*Property = CssImages*/
const Value V20795_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_291 = {S13, 0, &V20795_291_57182, &P20795_289};	/*Property = CssMediaQueries*/
const Value V20795_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_292 = {S14, 0, &V20795_292_57185, &P20795_291};	/*Property = CssMinMax*/
const Value V20795_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_293 = {S15, 0, &V20795_293_57188, &P20795_292};	/*Property = CssOverflow*/
const Value V20795_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_294 = {S16, 0, &V20795_294_57191, &P20795_293};	/*Property = CssPosition*/
const Value V20795_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_295 = {S17, 0, &V20795_295_57194, &P20795_294};	/*Property = CssText*/
const Value V20795_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_297 = {S18, 0, &V20795_297_57197, &P20795_295};	/*Property = CssTransforms*/
const Value V20795_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_241 = {S19, 0, &V20795_241_55801, &P20795_297};	/*Property = CssTransitions*/
const Value V20795_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_298 = {S20, 0, &V20795_298_57208, &P20795_241};	/*Property = CssUI*/
const Value V20795_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_300 = {S21, 0, &V20795_300_57217, &P20795_298};	/*Property = DataSet*/
const Value V20795_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_301 = {S22, 0, &V20795_301_57220, &P20795_300};	/*Property = DataUrl*/
const Value V20795_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_302 = {S23, 0, &V20795_302_57223, &P20795_301};	/*Property = DeviceOrientation*/
const Value V20795_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_304 = {S24, 0, &V20795_304_57229, &P20795_302};	/*Property = FileReader*/
const Value V20795_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_305 = {S25, 0, &V20795_305_57232, &P20795_304};	/*Property = FileSaver*/
const Value V20795_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_306 = {S26, 0, &V20795_306_57238, &P20795_305};	/*Property = FileWriter*/
const Value V20795_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_307 = {S27, 0, &V20795_307_57241, &P20795_306};	/*Property = FormData*/
const Value V20795_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_309 = {S28, 0, &V20795_309_57247, &P20795_307};	/*Property = Fullscreen*/
const Value V20795_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_310 = {S29, 0, &V20795_310_57250, &P20795_309};	/*Property = GeoLocation*/
const Value V20795_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_312 = {S30, 0, &V20795_312_57256, &P20795_310};	/*Property = History*/
const Value V20795_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_314 = {S31, 0, &V20795_314_57262, &P20795_312};	/*Property = Html5*/
const Value V20795_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_313 = {S32, 0, &V20795_313_57259, &P20795_314};	/*Property = Html-Media-Capture*/
const Value V20795_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_315 = {S33, 0, &V20795_315_57265, &P20795_313};	/*Property = Iframe*/
const Value V20795_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_316 = {S34, 0, &V20795_316_57268, &P20795_315};	/*Property = IndexedDB*/
const Value V20795_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_317 = {S35, 0, &V20795_317_57271, &P20795_316};	/*Property = Json*/
const Value V20795_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20795_15 = {S37, 0, &V20795_15_29, &P20795_317};	/*Property = LayoutEngine*/
const Value V20795_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_344 = {S38, 0, &V20795_344_57274, &P20795_15};	/*Property = Masking*/
const Value V20795_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_320 = {S39, 0, &V20795_320_57292, &P20795_344};	/*Property = PostMessage*/
const Value V20795_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_321 = {S40, 0, &V20795_321_57295, &P20795_320};	/*Property = Progress*/
const Value V20795_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_322 = {S41, 0, &V20795_322_57298, &P20795_321};	/*Property = Prompts*/
const Value V20795_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_323 = {S42, 0, &V20795_323_57301, &P20795_322};	/*Property = Selector*/
const Value V20795_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_324 = {S43, 0, &V20795_324_57307, &P20795_323};	/*Property = Svg*/
const Value V20795_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_327 = {S44, 0, &V20795_327_57316, &P20795_324};	/*Property = TouchEvents*/
const Value V20795_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_328 = {S45, 0, &V20795_328_57319, &P20795_327};	/*Property = Track*/
const Value V20795_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_329 = {S46, 0, &V20795_329_57325, &P20795_328};	/*Property = Video*/
const Value V20795_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_330 = {S47, 0, &V20795_330_57328, &P20795_329};	/*Property = Viewport*/
const Value V20795_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_332 = {S48, 0, &V20795_332_57340, &P20795_330};	/*Property = WebWorkers*/
const Value V20795_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20795_333 = {S49, 0, &V20795_333_57343, &P20795_332};	/*Property = Xhr2*/
const Profile P20795 = {"20795", &P20795_333};	/*Profile ID - 20795*/
const Value V20249_1_1 = {S2, NULL};	/*Value = True*/
const Property P20249_1 = {S50, 0, &V20249_1_1, NULL};	/*Property = IsMobile*/
const Value V20249_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20249_5 = {S52, 0, &V20249_5_9981, &P20249_1};	/*Property = ScreenPixelsHeight*/
const Value V20249_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20249_4 = {S54, 0, &V20249_4_2336, &P20249_5};	/*Property = ScreenPixelsWidth*/
const Profile P20249 = {"20249", &P20249_4};	/*Profile ID - 20249*/
const Value V21198_1_1 = {S2, NULL};	/*Value = True*/
const Property P21198_1 = {S50, 0, &V21198_1_1, NULL};	/*Property = IsMobile*/
const Value V21198_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21198_5 = {S52, 0, &V21198_5_9981, &P21198_1};	/*Property = ScreenPixelsHeight*/
const Value V21198_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21198_4 = {S54, 0, &V21198_4_2336, &P21198_5};	/*Property = ScreenPixelsWidth*/
const Profile P21198 = {"21198", &P21198_4};	/*Profile ID - 21198*/
const Value V22313_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_279 = {S1, 0, &V22313_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22313_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_282 = {S3, 0, &V22313_282_57155, &P22313_279};	/*Property = BlobBuilder*/
const Value V22313_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_350 = {S4, 0, &V22313_350_57446, &P22313_282};	/*Property = Canvas*/
const Value V22313_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_283 = {S5, 0, &V22313_283_57161, &P22313_350};	/*Property = CssBackground*/
const Value V22313_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_284 = {S6, 0, &V22313_284_57164, &P22313_283};	/*Property = CssBorderImage*/
const Value V22313_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_285 = {S7, 0, &V22313_285_57167, &P22313_284};	/*Property = CssCanvas*/
const Value V22313_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_286 = {S8, 0, &V22313_286_57170, &P22313_285};	/*Property = CssColor*/
const Value V22313_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_237 = {S9, 0, &V22313_237_55786, &P22313_286};	/*Property = CssColumn*/
const Value V22313_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_287 = {S10, 0, &V22313_287_57173, &P22313_237};	/*Property = CssFlexbox*/
const Value V22313_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_288 = {S11, 0, &V22313_288_57176, &P22313_287};	/*Property = CssFont*/
const Value V22313_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_289 = {S12, 0, &V22313_289_86899, &P22313_288};	/*Property = CssImages*/
const Value V22313_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_291 = {S13, 0, &V22313_291_57182, &P22313_289};	/*Property = CssMediaQueries*/
const Value V22313_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_292 = {S14, 0, &V22313_292_57185, &P22313_291};	/*Property = CssMinMax*/
const Value V22313_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_293 = {S15, 0, &V22313_293_57188, &P22313_292};	/*Property = CssOverflow*/
const Value V22313_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_294 = {S16, 0, &V22313_294_57191, &P22313_293};	/*Property = CssPosition*/
const Value V22313_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_295 = {S17, 0, &V22313_295_57194, &P22313_294};	/*Property = CssText*/
const Value V22313_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_297 = {S18, 0, &V22313_297_57197, &P22313_295};	/*Property = CssTransforms*/
const Value V22313_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_241 = {S19, 0, &V22313_241_55801, &P22313_297};	/*Property = CssTransitions*/
const Value V22313_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_298 = {S20, 0, &V22313_298_57208, &P22313_241};	/*Property = CssUI*/
const Value V22313_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_300 = {S21, 0, &V22313_300_57217, &P22313_298};	/*Property = DataSet*/
const Value V22313_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_301 = {S22, 0, &V22313_301_57220, &P22313_300};	/*Property = DataUrl*/
const Value V22313_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_302 = {S23, 0, &V22313_302_57223, &P22313_301};	/*Property = DeviceOrientation*/
const Value V22313_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_304 = {S24, 0, &V22313_304_57229, &P22313_302};	/*Property = FileReader*/
const Value V22313_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_305 = {S25, 0, &V22313_305_57232, &P22313_304};	/*Property = FileSaver*/
const Value V22313_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_306 = {S26, 0, &V22313_306_57238, &P22313_305};	/*Property = FileWriter*/
const Value V22313_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_307 = {S27, 0, &V22313_307_57241, &P22313_306};	/*Property = FormData*/
const Value V22313_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_309 = {S28, 0, &V22313_309_57247, &P22313_307};	/*Property = Fullscreen*/
const Value V22313_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_310 = {S29, 0, &V22313_310_57250, &P22313_309};	/*Property = GeoLocation*/
const Value V22313_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_312 = {S30, 0, &V22313_312_57256, &P22313_310};	/*Property = History*/
const Value V22313_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_314 = {S31, 0, &V22313_314_57262, &P22313_312};	/*Property = Html5*/
const Value V22313_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_313 = {S32, 0, &V22313_313_57259, &P22313_314};	/*Property = Html-Media-Capture*/
const Value V22313_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_315 = {S33, 0, &V22313_315_57265, &P22313_313};	/*Property = Iframe*/
const Value V22313_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_316 = {S34, 0, &V22313_316_57268, &P22313_315};	/*Property = IndexedDB*/
const Value V22313_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_317 = {S35, 0, &V22313_317_57271, &P22313_316};	/*Property = Json*/
const Value V22313_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22313_15 = {S37, 0, &V22313_15_29, &P22313_317};	/*Property = LayoutEngine*/
const Value V22313_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_344 = {S38, 0, &V22313_344_57274, &P22313_15};	/*Property = Masking*/
const Value V22313_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_320 = {S39, 0, &V22313_320_57292, &P22313_344};	/*Property = PostMessage*/
const Value V22313_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_321 = {S40, 0, &V22313_321_57295, &P22313_320};	/*Property = Progress*/
const Value V22313_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_322 = {S41, 0, &V22313_322_57298, &P22313_321};	/*Property = Prompts*/
const Value V22313_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_323 = {S42, 0, &V22313_323_57301, &P22313_322};	/*Property = Selector*/
const Value V22313_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_324 = {S43, 0, &V22313_324_57307, &P22313_323};	/*Property = Svg*/
const Value V22313_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_327 = {S44, 0, &V22313_327_57316, &P22313_324};	/*Property = TouchEvents*/
const Value V22313_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_328 = {S45, 0, &V22313_328_57319, &P22313_327};	/*Property = Track*/
const Value V22313_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_329 = {S46, 0, &V22313_329_57325, &P22313_328};	/*Property = Video*/
const Value V22313_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_330 = {S47, 0, &V22313_330_57328, &P22313_329};	/*Property = Viewport*/
const Value V22313_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_332 = {S48, 0, &V22313_332_57340, &P22313_330};	/*Property = WebWorkers*/
const Value V22313_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22313_333 = {S49, 0, &V22313_333_57343, &P22313_332};	/*Property = Xhr2*/
const Profile P22313 = {"22313", &P22313_333};	/*Profile ID - 22313*/
const Value V20802_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_279 = {S1, 0, &V20802_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20802_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_282 = {S3, 0, &V20802_282_57155, &P20802_279};	/*Property = BlobBuilder*/
const Value V20802_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_350 = {S4, 0, &V20802_350_57446, &P20802_282};	/*Property = Canvas*/
const Value V20802_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_283 = {S5, 0, &V20802_283_57161, &P20802_350};	/*Property = CssBackground*/
const Value V20802_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_284 = {S6, 0, &V20802_284_57164, &P20802_283};	/*Property = CssBorderImage*/
const Value V20802_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_285 = {S7, 0, &V20802_285_57167, &P20802_284};	/*Property = CssCanvas*/
const Value V20802_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_286 = {S8, 0, &V20802_286_57170, &P20802_285};	/*Property = CssColor*/
const Value V20802_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_237 = {S9, 0, &V20802_237_55786, &P20802_286};	/*Property = CssColumn*/
const Value V20802_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_287 = {S10, 0, &V20802_287_57173, &P20802_237};	/*Property = CssFlexbox*/
const Value V20802_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_288 = {S11, 0, &V20802_288_57176, &P20802_287};	/*Property = CssFont*/
const Value V20802_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_289 = {S12, 0, &V20802_289_86899, &P20802_288};	/*Property = CssImages*/
const Value V20802_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_291 = {S13, 0, &V20802_291_57182, &P20802_289};	/*Property = CssMediaQueries*/
const Value V20802_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_292 = {S14, 0, &V20802_292_57185, &P20802_291};	/*Property = CssMinMax*/
const Value V20802_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_293 = {S15, 0, &V20802_293_57188, &P20802_292};	/*Property = CssOverflow*/
const Value V20802_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_294 = {S16, 0, &V20802_294_57191, &P20802_293};	/*Property = CssPosition*/
const Value V20802_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_295 = {S17, 0, &V20802_295_57194, &P20802_294};	/*Property = CssText*/
const Value V20802_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_297 = {S18, 0, &V20802_297_57197, &P20802_295};	/*Property = CssTransforms*/
const Value V20802_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_241 = {S19, 0, &V20802_241_55801, &P20802_297};	/*Property = CssTransitions*/
const Value V20802_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_298 = {S20, 0, &V20802_298_57208, &P20802_241};	/*Property = CssUI*/
const Value V20802_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_300 = {S21, 0, &V20802_300_57217, &P20802_298};	/*Property = DataSet*/
const Value V20802_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_301 = {S22, 0, &V20802_301_57220, &P20802_300};	/*Property = DataUrl*/
const Value V20802_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_302 = {S23, 0, &V20802_302_57223, &P20802_301};	/*Property = DeviceOrientation*/
const Value V20802_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_304 = {S24, 0, &V20802_304_57229, &P20802_302};	/*Property = FileReader*/
const Value V20802_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_305 = {S25, 0, &V20802_305_57232, &P20802_304};	/*Property = FileSaver*/
const Value V20802_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_306 = {S26, 0, &V20802_306_57238, &P20802_305};	/*Property = FileWriter*/
const Value V20802_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_307 = {S27, 0, &V20802_307_57241, &P20802_306};	/*Property = FormData*/
const Value V20802_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_309 = {S28, 0, &V20802_309_57247, &P20802_307};	/*Property = Fullscreen*/
const Value V20802_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_310 = {S29, 0, &V20802_310_57250, &P20802_309};	/*Property = GeoLocation*/
const Value V20802_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_312 = {S30, 0, &V20802_312_57256, &P20802_310};	/*Property = History*/
const Value V20802_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_314 = {S31, 0, &V20802_314_57262, &P20802_312};	/*Property = Html5*/
const Value V20802_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_313 = {S32, 0, &V20802_313_57259, &P20802_314};	/*Property = Html-Media-Capture*/
const Value V20802_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_315 = {S33, 0, &V20802_315_57265, &P20802_313};	/*Property = Iframe*/
const Value V20802_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_316 = {S34, 0, &V20802_316_57268, &P20802_315};	/*Property = IndexedDB*/
const Value V20802_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_317 = {S35, 0, &V20802_317_57271, &P20802_316};	/*Property = Json*/
const Value V20802_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20802_15 = {S37, 0, &V20802_15_29, &P20802_317};	/*Property = LayoutEngine*/
const Value V20802_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_344 = {S38, 0, &V20802_344_57274, &P20802_15};	/*Property = Masking*/
const Value V20802_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_320 = {S39, 0, &V20802_320_57292, &P20802_344};	/*Property = PostMessage*/
const Value V20802_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_321 = {S40, 0, &V20802_321_57295, &P20802_320};	/*Property = Progress*/
const Value V20802_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_322 = {S41, 0, &V20802_322_57298, &P20802_321};	/*Property = Prompts*/
const Value V20802_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_323 = {S42, 0, &V20802_323_57301, &P20802_322};	/*Property = Selector*/
const Value V20802_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_324 = {S43, 0, &V20802_324_57307, &P20802_323};	/*Property = Svg*/
const Value V20802_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_327 = {S44, 0, &V20802_327_57316, &P20802_324};	/*Property = TouchEvents*/
const Value V20802_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_328 = {S45, 0, &V20802_328_57319, &P20802_327};	/*Property = Track*/
const Value V20802_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_329 = {S46, 0, &V20802_329_57325, &P20802_328};	/*Property = Video*/
const Value V20802_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_330 = {S47, 0, &V20802_330_57328, &P20802_329};	/*Property = Viewport*/
const Value V20802_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_332 = {S48, 0, &V20802_332_57340, &P20802_330};	/*Property = WebWorkers*/
const Value V20802_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20802_333 = {S49, 0, &V20802_333_57343, &P20802_332};	/*Property = Xhr2*/
const Profile P20802 = {"20802", &P20802_333};	/*Profile ID - 20802*/
const Value V22590_1_1 = {S2, NULL};	/*Value = True*/
const Property P22590_1 = {S50, 0, &V22590_1_1, NULL};	/*Property = IsMobile*/
const Value V22590_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22590_5 = {S52, 0, &V22590_5_9981, &P22590_1};	/*Property = ScreenPixelsHeight*/
const Value V22590_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22590_4 = {S54, 0, &V22590_4_2336, &P22590_5};	/*Property = ScreenPixelsWidth*/
const Profile P22590 = {"22590", &P22590_4};	/*Profile ID - 22590*/
const Profile P23097 = {"23097", NULL};	/*Profile ID - 23097*/
const Value V18231_1_1 = {S2, NULL};	/*Value = True*/
const Property P18231_1 = {S50, 0, &V18231_1_1, NULL};	/*Property = IsMobile*/
const Value V18231_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18231_5 = {S52, 0, &V18231_5_1277, &P18231_1};	/*Property = ScreenPixelsHeight*/
const Value V18231_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18231_4 = {S54, 0, &V18231_4_1279, &P18231_5};	/*Property = ScreenPixelsWidth*/
const Profile P18231 = {"18231", &P18231_4};	/*Profile ID - 18231*/
const Value V18097_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18097_279 = {S1, 0, &V18097_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18097_282_57153 = {S2, NULL};	/*Value = True*/
const Property P18097_282 = {S3, 0, &V18097_282_57153, &P18097_279};	/*Property = BlobBuilder*/
const Value V18097_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18097_350 = {S4, 0, &V18097_350_57444, &P18097_282};	/*Property = Canvas*/
const Value V18097_283_57160 = {S0, NULL};	/*Value = False*/
const Property P18097_283 = {S5, 0, &V18097_283_57160, &P18097_350};	/*Property = CssBackground*/
const Value V18097_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18097_284 = {S6, 0, &V18097_284_57163, &P18097_283};	/*Property = CssBorderImage*/
const Value V18097_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18097_285 = {S7, 0, &V18097_285_57166, &P18097_284};	/*Property = CssCanvas*/
const Value V18097_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18097_286 = {S8, 0, &V18097_286_57168, &P18097_285};	/*Property = CssColor*/
const Value V18097_237_55784 = {S2, NULL};	/*Value = True*/
const Property P18097_237 = {S9, 0, &V18097_237_55784, &P18097_286};	/*Property = CssColumn*/
const Value V18097_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18097_287 = {S10, 0, &V18097_287_57172, &P18097_237};	/*Property = CssFlexbox*/
const Value V18097_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18097_288 = {S11, 0, &V18097_288_57175, &P18097_287};	/*Property = CssFont*/
const Value V18097_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18097_289 = {S12, 0, &V18097_289_86899, &P18097_288};	/*Property = CssImages*/
const Value V18097_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18097_291 = {S13, 0, &V18097_291_57181, &P18097_289};	/*Property = CssMediaQueries*/
const Value V18097_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18097_292 = {S14, 0, &V18097_292_57183, &P18097_291};	/*Property = CssMinMax*/
const Value V18097_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18097_293 = {S15, 0, &V18097_293_57187, &P18097_292};	/*Property = CssOverflow*/
const Value V18097_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18097_294 = {S16, 0, &V18097_294_57189, &P18097_293};	/*Property = CssPosition*/
const Value V18097_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18097_295 = {S17, 0, &V18097_295_57193, &P18097_294};	/*Property = CssText*/
const Value V18097_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18097_297 = {S18, 0, &V18097_297_57195, &P18097_295};	/*Property = CssTransforms*/
const Value V18097_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18097_241 = {S19, 0, &V18097_241_55799, &P18097_297};	/*Property = CssTransitions*/
const Value V18097_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18097_298 = {S20, 0, &V18097_298_57206, &P18097_241};	/*Property = CssUI*/
const Value V18097_300_57215 = {S2, NULL};	/*Value = True*/
const Property P18097_300 = {S21, 0, &V18097_300_57215, &P18097_298};	/*Property = DataSet*/
const Value V18097_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18097_301 = {S22, 0, &V18097_301_57218, &P18097_300};	/*Property = DataUrl*/
const Value V18097_302_57221 = {S2, NULL};	/*Value = True*/
const Property P18097_302 = {S23, 0, &V18097_302_57221, &P18097_301};	/*Property = DeviceOrientation*/
const Value V18097_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18097_304 = {S24, 0, &V18097_304_57227, &P18097_302};	/*Property = FileReader*/
const Value V18097_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18097_305 = {S25, 0, &V18097_305_57231, &P18097_304};	/*Property = FileSaver*/
const Value V18097_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18097_306 = {S26, 0, &V18097_306_57237, &P18097_305};	/*Property = FileWriter*/
const Value V18097_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18097_307 = {S27, 0, &V18097_307_57239, &P18097_306};	/*Property = FormData*/
const Value V18097_309_57245 = {S2, NULL};	/*Value = True*/
const Property P18097_309 = {S28, 0, &V18097_309_57245, &P18097_307};	/*Property = Fullscreen*/
const Value V18097_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18097_310 = {S29, 0, &V18097_310_57248, &P18097_309};	/*Property = GeoLocation*/
const Value V18097_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18097_312 = {S30, 0, &V18097_312_57254, &P18097_310};	/*Property = History*/
const Value V18097_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18097_314 = {S31, 0, &V18097_314_57261, &P18097_312};	/*Property = Html5*/
const Value V18097_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18097_313 = {S32, 0, &V18097_313_57258, &P18097_314};	/*Property = Html-Media-Capture*/
const Value V18097_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18097_315 = {S33, 0, &V18097_315_57264, &P18097_313};	/*Property = Iframe*/
const Value V18097_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18097_316 = {S34, 0, &V18097_316_57266, &P18097_315};	/*Property = IndexedDB*/
const Value V18097_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18097_317 = {S35, 0, &V18097_317_57269, &P18097_316};	/*Property = Json*/
const Value V18097_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18097_15 = {S37, 0, &V18097_15_29, &P18097_317};	/*Property = LayoutEngine*/
const Value V18097_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18097_344 = {S38, 0, &V18097_344_57272, &P18097_15};	/*Property = Masking*/
const Value V18097_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18097_320 = {S39, 0, &V18097_320_57290, &P18097_344};	/*Property = PostMessage*/
const Value V18097_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18097_321 = {S40, 0, &V18097_321_57293, &P18097_320};	/*Property = Progress*/
const Value V18097_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18097_322 = {S41, 0, &V18097_322_57296, &P18097_321};	/*Property = Prompts*/
const Value V18097_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18097_323 = {S42, 0, &V18097_323_57299, &P18097_322};	/*Property = Selector*/
const Value V18097_324_57305 = {S2, NULL};	/*Value = True*/
const Property P18097_324 = {S43, 0, &V18097_324_57305, &P18097_323};	/*Property = Svg*/
const Value V18097_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18097_327 = {S44, 0, &V18097_327_57315, &P18097_324};	/*Property = TouchEvents*/
const Value V18097_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18097_328 = {S45, 0, &V18097_328_57318, &P18097_327};	/*Property = Track*/
const Value V18097_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18097_329 = {S46, 0, &V18097_329_57323, &P18097_328};	/*Property = Video*/
const Value V18097_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18097_330 = {S47, 0, &V18097_330_57326, &P18097_329};	/*Property = Viewport*/
const Value V18097_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18097_332 = {S48, 0, &V18097_332_57339, &P18097_330};	/*Property = WebWorkers*/
const Value V18097_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18097_333 = {S49, 0, &V18097_333_57342, &P18097_332};	/*Property = Xhr2*/
const Profile P18097 = {"18097", &P18097_333};	/*Profile ID - 18097*/
const Value V18341_1_1 = {S2, NULL};	/*Value = True*/
const Property P18341_1 = {S50, 0, &V18341_1_1, NULL};	/*Property = IsMobile*/
const Value V18341_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P18341_5 = {S52, 0, &V18341_5_1269, &P18341_1};	/*Property = ScreenPixelsHeight*/
const Value V18341_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18341_4 = {S54, 0, &V18341_4_1229, &P18341_5};	/*Property = ScreenPixelsWidth*/
const Profile P18341 = {"18341", &P18341_4};	/*Profile ID - 18341*/
const Value V22356_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22356_279 = {S1, 0, &V22356_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22356_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22356_282 = {S3, 0, &V22356_282_57153, &P22356_279};	/*Property = BlobBuilder*/
const Value V22356_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22356_350 = {S4, 0, &V22356_350_57444, &P22356_282};	/*Property = Canvas*/
const Value V22356_283_57160 = {S0, NULL};	/*Value = False*/
const Property P22356_283 = {S5, 0, &V22356_283_57160, &P22356_350};	/*Property = CssBackground*/
const Value V22356_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22356_284 = {S6, 0, &V22356_284_57163, &P22356_283};	/*Property = CssBorderImage*/
const Value V22356_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22356_285 = {S7, 0, &V22356_285_57166, &P22356_284};	/*Property = CssCanvas*/
const Value V22356_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22356_286 = {S8, 0, &V22356_286_57168, &P22356_285};	/*Property = CssColor*/
const Value V22356_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22356_237 = {S9, 0, &V22356_237_55784, &P22356_286};	/*Property = CssColumn*/
const Value V22356_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22356_287 = {S10, 0, &V22356_287_57172, &P22356_237};	/*Property = CssFlexbox*/
const Value V22356_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22356_288 = {S11, 0, &V22356_288_57175, &P22356_287};	/*Property = CssFont*/
const Value V22356_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22356_289 = {S12, 0, &V22356_289_86899, &P22356_288};	/*Property = CssImages*/
const Value V22356_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22356_291 = {S13, 0, &V22356_291_57181, &P22356_289};	/*Property = CssMediaQueries*/
const Value V22356_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22356_292 = {S14, 0, &V22356_292_57183, &P22356_291};	/*Property = CssMinMax*/
const Value V22356_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22356_293 = {S15, 0, &V22356_293_57187, &P22356_292};	/*Property = CssOverflow*/
const Value V22356_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22356_294 = {S16, 0, &V22356_294_57189, &P22356_293};	/*Property = CssPosition*/
const Value V22356_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22356_295 = {S17, 0, &V22356_295_57193, &P22356_294};	/*Property = CssText*/
const Value V22356_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22356_297 = {S18, 0, &V22356_297_57195, &P22356_295};	/*Property = CssTransforms*/
const Value V22356_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22356_241 = {S19, 0, &V22356_241_55799, &P22356_297};	/*Property = CssTransitions*/
const Value V22356_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22356_298 = {S20, 0, &V22356_298_57206, &P22356_241};	/*Property = CssUI*/
const Value V22356_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22356_300 = {S21, 0, &V22356_300_57215, &P22356_298};	/*Property = DataSet*/
const Value V22356_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22356_301 = {S22, 0, &V22356_301_57218, &P22356_300};	/*Property = DataUrl*/
const Value V22356_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22356_302 = {S23, 0, &V22356_302_57221, &P22356_301};	/*Property = DeviceOrientation*/
const Value V22356_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22356_304 = {S24, 0, &V22356_304_57227, &P22356_302};	/*Property = FileReader*/
const Value V22356_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22356_305 = {S25, 0, &V22356_305_57231, &P22356_304};	/*Property = FileSaver*/
const Value V22356_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22356_306 = {S26, 0, &V22356_306_57237, &P22356_305};	/*Property = FileWriter*/
const Value V22356_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22356_307 = {S27, 0, &V22356_307_57239, &P22356_306};	/*Property = FormData*/
const Value V22356_309_57245 = {S2, NULL};	/*Value = True*/
const Property P22356_309 = {S28, 0, &V22356_309_57245, &P22356_307};	/*Property = Fullscreen*/
const Value V22356_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22356_310 = {S29, 0, &V22356_310_57248, &P22356_309};	/*Property = GeoLocation*/
const Value V22356_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22356_312 = {S30, 0, &V22356_312_57254, &P22356_310};	/*Property = History*/
const Value V22356_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22356_314 = {S31, 0, &V22356_314_57261, &P22356_312};	/*Property = Html5*/
const Value V22356_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22356_313 = {S32, 0, &V22356_313_57258, &P22356_314};	/*Property = Html-Media-Capture*/
const Value V22356_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22356_315 = {S33, 0, &V22356_315_57264, &P22356_313};	/*Property = Iframe*/
const Value V22356_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22356_316 = {S34, 0, &V22356_316_57266, &P22356_315};	/*Property = IndexedDB*/
const Value V22356_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22356_317 = {S35, 0, &V22356_317_57269, &P22356_316};	/*Property = Json*/
const Value V22356_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22356_15 = {S37, 0, &V22356_15_29, &P22356_317};	/*Property = LayoutEngine*/
const Value V22356_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22356_344 = {S38, 0, &V22356_344_57272, &P22356_15};	/*Property = Masking*/
const Value V22356_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22356_320 = {S39, 0, &V22356_320_57290, &P22356_344};	/*Property = PostMessage*/
const Value V22356_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22356_321 = {S40, 0, &V22356_321_57293, &P22356_320};	/*Property = Progress*/
const Value V22356_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22356_322 = {S41, 0, &V22356_322_57296, &P22356_321};	/*Property = Prompts*/
const Value V22356_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22356_323 = {S42, 0, &V22356_323_57299, &P22356_322};	/*Property = Selector*/
const Value V22356_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22356_324 = {S43, 0, &V22356_324_57305, &P22356_323};	/*Property = Svg*/
const Value V22356_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22356_327 = {S44, 0, &V22356_327_57315, &P22356_324};	/*Property = TouchEvents*/
const Value V22356_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22356_328 = {S45, 0, &V22356_328_57318, &P22356_327};	/*Property = Track*/
const Value V22356_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22356_329 = {S46, 0, &V22356_329_57323, &P22356_328};	/*Property = Video*/
const Value V22356_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22356_330 = {S47, 0, &V22356_330_57326, &P22356_329};	/*Property = Viewport*/
const Value V22356_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22356_332 = {S48, 0, &V22356_332_57339, &P22356_330};	/*Property = WebWorkers*/
const Value V22356_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22356_333 = {S49, 0, &V22356_333_57341, &P22356_332};	/*Property = Xhr2*/
const Profile P22356 = {"22356", &P22356_333};	/*Profile ID - 22356*/
const Value V24668_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_279 = {S1, 0, &V24668_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24668_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_282 = {S3, 0, &V24668_282_57155, &P24668_279};	/*Property = BlobBuilder*/
const Value V24668_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_350 = {S4, 0, &V24668_350_57446, &P24668_282};	/*Property = Canvas*/
const Value V24668_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_283 = {S5, 0, &V24668_283_57161, &P24668_350};	/*Property = CssBackground*/
const Value V24668_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_284 = {S6, 0, &V24668_284_57164, &P24668_283};	/*Property = CssBorderImage*/
const Value V24668_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_285 = {S7, 0, &V24668_285_57167, &P24668_284};	/*Property = CssCanvas*/
const Value V24668_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_286 = {S8, 0, &V24668_286_57170, &P24668_285};	/*Property = CssColor*/
const Value V24668_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_237 = {S9, 0, &V24668_237_55786, &P24668_286};	/*Property = CssColumn*/
const Value V24668_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_287 = {S10, 0, &V24668_287_57173, &P24668_237};	/*Property = CssFlexbox*/
const Value V24668_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_288 = {S11, 0, &V24668_288_57176, &P24668_287};	/*Property = CssFont*/
const Value V24668_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_289 = {S12, 0, &V24668_289_86899, &P24668_288};	/*Property = CssImages*/
const Value V24668_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_291 = {S13, 0, &V24668_291_57182, &P24668_289};	/*Property = CssMediaQueries*/
const Value V24668_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_292 = {S14, 0, &V24668_292_57185, &P24668_291};	/*Property = CssMinMax*/
const Value V24668_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_293 = {S15, 0, &V24668_293_57188, &P24668_292};	/*Property = CssOverflow*/
const Value V24668_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_294 = {S16, 0, &V24668_294_57191, &P24668_293};	/*Property = CssPosition*/
const Value V24668_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_295 = {S17, 0, &V24668_295_57194, &P24668_294};	/*Property = CssText*/
const Value V24668_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_297 = {S18, 0, &V24668_297_57197, &P24668_295};	/*Property = CssTransforms*/
const Value V24668_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_241 = {S19, 0, &V24668_241_55801, &P24668_297};	/*Property = CssTransitions*/
const Value V24668_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_298 = {S20, 0, &V24668_298_57208, &P24668_241};	/*Property = CssUI*/
const Value V24668_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_300 = {S21, 0, &V24668_300_57217, &P24668_298};	/*Property = DataSet*/
const Value V24668_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_301 = {S22, 0, &V24668_301_57220, &P24668_300};	/*Property = DataUrl*/
const Value V24668_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_302 = {S23, 0, &V24668_302_57223, &P24668_301};	/*Property = DeviceOrientation*/
const Value V24668_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_304 = {S24, 0, &V24668_304_57229, &P24668_302};	/*Property = FileReader*/
const Value V24668_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_305 = {S25, 0, &V24668_305_57232, &P24668_304};	/*Property = FileSaver*/
const Value V24668_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_306 = {S26, 0, &V24668_306_57238, &P24668_305};	/*Property = FileWriter*/
const Value V24668_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_307 = {S27, 0, &V24668_307_57241, &P24668_306};	/*Property = FormData*/
const Value V24668_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_309 = {S28, 0, &V24668_309_57247, &P24668_307};	/*Property = Fullscreen*/
const Value V24668_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_310 = {S29, 0, &V24668_310_57250, &P24668_309};	/*Property = GeoLocation*/
const Value V24668_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_312 = {S30, 0, &V24668_312_57256, &P24668_310};	/*Property = History*/
const Value V24668_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_314 = {S31, 0, &V24668_314_57262, &P24668_312};	/*Property = Html5*/
const Value V24668_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_313 = {S32, 0, &V24668_313_57259, &P24668_314};	/*Property = Html-Media-Capture*/
const Value V24668_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_315 = {S33, 0, &V24668_315_57265, &P24668_313};	/*Property = Iframe*/
const Value V24668_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_316 = {S34, 0, &V24668_316_57268, &P24668_315};	/*Property = IndexedDB*/
const Value V24668_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_317 = {S35, 0, &V24668_317_57271, &P24668_316};	/*Property = Json*/
const Value V24668_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24668_15 = {S37, 0, &V24668_15_90837, &P24668_317};	/*Property = LayoutEngine*/
const Value V24668_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_344 = {S38, 0, &V24668_344_57274, &P24668_15};	/*Property = Masking*/
const Value V24668_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_320 = {S39, 0, &V24668_320_57292, &P24668_344};	/*Property = PostMessage*/
const Value V24668_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_321 = {S40, 0, &V24668_321_57295, &P24668_320};	/*Property = Progress*/
const Value V24668_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_322 = {S41, 0, &V24668_322_57298, &P24668_321};	/*Property = Prompts*/
const Value V24668_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_323 = {S42, 0, &V24668_323_57301, &P24668_322};	/*Property = Selector*/
const Value V24668_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_324 = {S43, 0, &V24668_324_57307, &P24668_323};	/*Property = Svg*/
const Value V24668_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_327 = {S44, 0, &V24668_327_57316, &P24668_324};	/*Property = TouchEvents*/
const Value V24668_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_328 = {S45, 0, &V24668_328_57319, &P24668_327};	/*Property = Track*/
const Value V24668_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_329 = {S46, 0, &V24668_329_57325, &P24668_328};	/*Property = Video*/
const Value V24668_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_330 = {S47, 0, &V24668_330_57328, &P24668_329};	/*Property = Viewport*/
const Value V24668_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_332 = {S48, 0, &V24668_332_57340, &P24668_330};	/*Property = WebWorkers*/
const Value V24668_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24668_333 = {S49, 0, &V24668_333_57343, &P24668_332};	/*Property = Xhr2*/
const Profile P24668 = {"24668", &P24668_333};	/*Profile ID - 24668*/
const Value V18712_1_1 = {S2, NULL};	/*Value = True*/
const Property P18712_1 = {S50, 0, &V18712_1_1, NULL};	/*Property = IsMobile*/
const Value V18712_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18712_5 = {S52, 0, &V18712_5_1227, &P18712_1};	/*Property = ScreenPixelsHeight*/
const Value V18712_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18712_4 = {S54, 0, &V18712_4_1229, &P18712_5};	/*Property = ScreenPixelsWidth*/
const Profile P18712 = {"18712", &P18712_4};	/*Profile ID - 18712*/
const Value V18100_279_57147 = {S2, NULL};	/*Value = True*/
const Property P18100_279 = {S1, 0, &V18100_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V18100_282_57153 = {S2, NULL};	/*Value = True*/
const Property P18100_282 = {S3, 0, &V18100_282_57153, &P18100_279};	/*Property = BlobBuilder*/
const Value V18100_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18100_350 = {S4, 0, &V18100_350_57444, &P18100_282};	/*Property = Canvas*/
const Value V18100_283_57160 = {S0, NULL};	/*Value = False*/
const Property P18100_283 = {S5, 0, &V18100_283_57160, &P18100_350};	/*Property = CssBackground*/
const Value V18100_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18100_284 = {S6, 0, &V18100_284_57163, &P18100_283};	/*Property = CssBorderImage*/
const Value V18100_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18100_285 = {S7, 0, &V18100_285_57166, &P18100_284};	/*Property = CssCanvas*/
const Value V18100_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18100_286 = {S8, 0, &V18100_286_57168, &P18100_285};	/*Property = CssColor*/
const Value V18100_237_55784 = {S2, NULL};	/*Value = True*/
const Property P18100_237 = {S9, 0, &V18100_237_55784, &P18100_286};	/*Property = CssColumn*/
const Value V18100_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18100_287 = {S10, 0, &V18100_287_57172, &P18100_237};	/*Property = CssFlexbox*/
const Value V18100_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18100_288 = {S11, 0, &V18100_288_57175, &P18100_287};	/*Property = CssFont*/
const Value V18100_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18100_289 = {S12, 0, &V18100_289_86899, &P18100_288};	/*Property = CssImages*/
const Value V18100_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18100_291 = {S13, 0, &V18100_291_57181, &P18100_289};	/*Property = CssMediaQueries*/
const Value V18100_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18100_292 = {S14, 0, &V18100_292_57183, &P18100_291};	/*Property = CssMinMax*/
const Value V18100_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18100_293 = {S15, 0, &V18100_293_57187, &P18100_292};	/*Property = CssOverflow*/
const Value V18100_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18100_294 = {S16, 0, &V18100_294_57189, &P18100_293};	/*Property = CssPosition*/
const Value V18100_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18100_295 = {S17, 0, &V18100_295_57193, &P18100_294};	/*Property = CssText*/
const Value V18100_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18100_297 = {S18, 0, &V18100_297_57195, &P18100_295};	/*Property = CssTransforms*/
const Value V18100_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18100_241 = {S19, 0, &V18100_241_55799, &P18100_297};	/*Property = CssTransitions*/
const Value V18100_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18100_298 = {S20, 0, &V18100_298_57206, &P18100_241};	/*Property = CssUI*/
const Value V18100_300_57215 = {S2, NULL};	/*Value = True*/
const Property P18100_300 = {S21, 0, &V18100_300_57215, &P18100_298};	/*Property = DataSet*/
const Value V18100_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18100_301 = {S22, 0, &V18100_301_57218, &P18100_300};	/*Property = DataUrl*/
const Value V18100_302_57221 = {S2, NULL};	/*Value = True*/
const Property P18100_302 = {S23, 0, &V18100_302_57221, &P18100_301};	/*Property = DeviceOrientation*/
const Value V18100_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18100_304 = {S24, 0, &V18100_304_57227, &P18100_302};	/*Property = FileReader*/
const Value V18100_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18100_305 = {S25, 0, &V18100_305_57231, &P18100_304};	/*Property = FileSaver*/
const Value V18100_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18100_306 = {S26, 0, &V18100_306_57237, &P18100_305};	/*Property = FileWriter*/
const Value V18100_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18100_307 = {S27, 0, &V18100_307_57239, &P18100_306};	/*Property = FormData*/
const Value V18100_309_57245 = {S2, NULL};	/*Value = True*/
const Property P18100_309 = {S28, 0, &V18100_309_57245, &P18100_307};	/*Property = Fullscreen*/
const Value V18100_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18100_310 = {S29, 0, &V18100_310_57248, &P18100_309};	/*Property = GeoLocation*/
const Value V18100_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18100_312 = {S30, 0, &V18100_312_57254, &P18100_310};	/*Property = History*/
const Value V18100_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18100_314 = {S31, 0, &V18100_314_57261, &P18100_312};	/*Property = Html5*/
const Value V18100_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18100_313 = {S32, 0, &V18100_313_57258, &P18100_314};	/*Property = Html-Media-Capture*/
const Value V18100_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18100_315 = {S33, 0, &V18100_315_57264, &P18100_313};	/*Property = Iframe*/
const Value V18100_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18100_316 = {S34, 0, &V18100_316_57266, &P18100_315};	/*Property = IndexedDB*/
const Value V18100_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18100_317 = {S35, 0, &V18100_317_57269, &P18100_316};	/*Property = Json*/
const Value V18100_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18100_15 = {S37, 0, &V18100_15_29, &P18100_317};	/*Property = LayoutEngine*/
const Value V18100_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18100_344 = {S38, 0, &V18100_344_57272, &P18100_15};	/*Property = Masking*/
const Value V18100_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18100_320 = {S39, 0, &V18100_320_57290, &P18100_344};	/*Property = PostMessage*/
const Value V18100_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18100_321 = {S40, 0, &V18100_321_57293, &P18100_320};	/*Property = Progress*/
const Value V18100_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18100_322 = {S41, 0, &V18100_322_57296, &P18100_321};	/*Property = Prompts*/
const Value V18100_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18100_323 = {S42, 0, &V18100_323_57299, &P18100_322};	/*Property = Selector*/
const Value V18100_324_57305 = {S2, NULL};	/*Value = True*/
const Property P18100_324 = {S43, 0, &V18100_324_57305, &P18100_323};	/*Property = Svg*/
const Value V18100_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18100_327 = {S44, 0, &V18100_327_57315, &P18100_324};	/*Property = TouchEvents*/
const Value V18100_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18100_328 = {S45, 0, &V18100_328_57318, &P18100_327};	/*Property = Track*/
const Value V18100_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18100_329 = {S46, 0, &V18100_329_57323, &P18100_328};	/*Property = Video*/
const Value V18100_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18100_330 = {S47, 0, &V18100_330_57326, &P18100_329};	/*Property = Viewport*/
const Value V18100_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18100_332 = {S48, 0, &V18100_332_57339, &P18100_330};	/*Property = WebWorkers*/
const Value V18100_333_57341 = {S2, NULL};	/*Value = True*/
const Property P18100_333 = {S49, 0, &V18100_333_57341, &P18100_332};	/*Property = Xhr2*/
const Profile P18100 = {"18100", &P18100_333};	/*Profile ID - 18100*/
const Value V21171_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_279 = {S1, 0, &V21171_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21171_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_282 = {S3, 0, &V21171_282_57155, &P21171_279};	/*Property = BlobBuilder*/
const Value V21171_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_350 = {S4, 0, &V21171_350_57446, &P21171_282};	/*Property = Canvas*/
const Value V21171_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_283 = {S5, 0, &V21171_283_57161, &P21171_350};	/*Property = CssBackground*/
const Value V21171_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_284 = {S6, 0, &V21171_284_57164, &P21171_283};	/*Property = CssBorderImage*/
const Value V21171_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_285 = {S7, 0, &V21171_285_57167, &P21171_284};	/*Property = CssCanvas*/
const Value V21171_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_286 = {S8, 0, &V21171_286_57170, &P21171_285};	/*Property = CssColor*/
const Value V21171_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_237 = {S9, 0, &V21171_237_55786, &P21171_286};	/*Property = CssColumn*/
const Value V21171_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_287 = {S10, 0, &V21171_287_57173, &P21171_237};	/*Property = CssFlexbox*/
const Value V21171_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_288 = {S11, 0, &V21171_288_57176, &P21171_287};	/*Property = CssFont*/
const Value V21171_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_289 = {S12, 0, &V21171_289_86899, &P21171_288};	/*Property = CssImages*/
const Value V21171_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_291 = {S13, 0, &V21171_291_57182, &P21171_289};	/*Property = CssMediaQueries*/
const Value V21171_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_292 = {S14, 0, &V21171_292_57185, &P21171_291};	/*Property = CssMinMax*/
const Value V21171_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_293 = {S15, 0, &V21171_293_57188, &P21171_292};	/*Property = CssOverflow*/
const Value V21171_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_294 = {S16, 0, &V21171_294_57191, &P21171_293};	/*Property = CssPosition*/
const Value V21171_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_295 = {S17, 0, &V21171_295_57194, &P21171_294};	/*Property = CssText*/
const Value V21171_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_297 = {S18, 0, &V21171_297_57197, &P21171_295};	/*Property = CssTransforms*/
const Value V21171_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_241 = {S19, 0, &V21171_241_55801, &P21171_297};	/*Property = CssTransitions*/
const Value V21171_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_298 = {S20, 0, &V21171_298_57208, &P21171_241};	/*Property = CssUI*/
const Value V21171_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_300 = {S21, 0, &V21171_300_57217, &P21171_298};	/*Property = DataSet*/
const Value V21171_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_301 = {S22, 0, &V21171_301_57220, &P21171_300};	/*Property = DataUrl*/
const Value V21171_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_302 = {S23, 0, &V21171_302_57223, &P21171_301};	/*Property = DeviceOrientation*/
const Value V21171_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_304 = {S24, 0, &V21171_304_57229, &P21171_302};	/*Property = FileReader*/
const Value V21171_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_305 = {S25, 0, &V21171_305_57232, &P21171_304};	/*Property = FileSaver*/
const Value V21171_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_306 = {S26, 0, &V21171_306_57238, &P21171_305};	/*Property = FileWriter*/
const Value V21171_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_307 = {S27, 0, &V21171_307_57241, &P21171_306};	/*Property = FormData*/
const Value V21171_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_309 = {S28, 0, &V21171_309_57247, &P21171_307};	/*Property = Fullscreen*/
const Value V21171_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_310 = {S29, 0, &V21171_310_57250, &P21171_309};	/*Property = GeoLocation*/
const Value V21171_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_312 = {S30, 0, &V21171_312_57256, &P21171_310};	/*Property = History*/
const Value V21171_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_314 = {S31, 0, &V21171_314_57262, &P21171_312};	/*Property = Html5*/
const Value V21171_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_313 = {S32, 0, &V21171_313_57259, &P21171_314};	/*Property = Html-Media-Capture*/
const Value V21171_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_315 = {S33, 0, &V21171_315_57265, &P21171_313};	/*Property = Iframe*/
const Value V21171_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_316 = {S34, 0, &V21171_316_57268, &P21171_315};	/*Property = IndexedDB*/
const Value V21171_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_317 = {S35, 0, &V21171_317_57271, &P21171_316};	/*Property = Json*/
const Value V21171_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P21171_15 = {S37, 0, &V21171_15_29, &P21171_317};	/*Property = LayoutEngine*/
const Value V21171_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_344 = {S38, 0, &V21171_344_57274, &P21171_15};	/*Property = Masking*/
const Value V21171_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_320 = {S39, 0, &V21171_320_57292, &P21171_344};	/*Property = PostMessage*/
const Value V21171_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_321 = {S40, 0, &V21171_321_57295, &P21171_320};	/*Property = Progress*/
const Value V21171_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_322 = {S41, 0, &V21171_322_57298, &P21171_321};	/*Property = Prompts*/
const Value V21171_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_323 = {S42, 0, &V21171_323_57301, &P21171_322};	/*Property = Selector*/
const Value V21171_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_324 = {S43, 0, &V21171_324_57307, &P21171_323};	/*Property = Svg*/
const Value V21171_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_327 = {S44, 0, &V21171_327_57316, &P21171_324};	/*Property = TouchEvents*/
const Value V21171_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_328 = {S45, 0, &V21171_328_57319, &P21171_327};	/*Property = Track*/
const Value V21171_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_329 = {S46, 0, &V21171_329_57325, &P21171_328};	/*Property = Video*/
const Value V21171_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_330 = {S47, 0, &V21171_330_57328, &P21171_329};	/*Property = Viewport*/
const Value V21171_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_332 = {S48, 0, &V21171_332_57340, &P21171_330};	/*Property = WebWorkers*/
const Value V21171_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21171_333 = {S49, 0, &V21171_333_57343, &P21171_332};	/*Property = Xhr2*/
const Profile P21171 = {"21171", &P21171_333};	/*Profile ID - 21171*/
const Value V18330_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18330_279 = {S1, 0, &V18330_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18330_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18330_282 = {S3, 0, &V18330_282_57154, &P18330_279};	/*Property = BlobBuilder*/
const Value V18330_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18330_350 = {S4, 0, &V18330_350_57444, &P18330_282};	/*Property = Canvas*/
const Value V18330_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18330_283 = {S5, 0, &V18330_283_57159, &P18330_350};	/*Property = CssBackground*/
const Value V18330_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18330_284 = {S6, 0, &V18330_284_57163, &P18330_283};	/*Property = CssBorderImage*/
const Value V18330_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18330_285 = {S7, 0, &V18330_285_57166, &P18330_284};	/*Property = CssCanvas*/
const Value V18330_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18330_286 = {S8, 0, &V18330_286_57168, &P18330_285};	/*Property = CssColor*/
const Value V18330_237_55784 = {S2, NULL};	/*Value = True*/
const Property P18330_237 = {S9, 0, &V18330_237_55784, &P18330_286};	/*Property = CssColumn*/
const Value V18330_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18330_287 = {S10, 0, &V18330_287_57172, &P18330_237};	/*Property = CssFlexbox*/
const Value V18330_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18330_288 = {S11, 0, &V18330_288_57175, &P18330_287};	/*Property = CssFont*/
const Value V18330_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18330_289 = {S12, 0, &V18330_289_57759, &P18330_288};	/*Property = CssImages*/
const Value V18330_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18330_291 = {S13, 0, &V18330_291_57181, &P18330_289};	/*Property = CssMediaQueries*/
const Value V18330_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18330_292 = {S14, 0, &V18330_292_57183, &P18330_291};	/*Property = CssMinMax*/
const Value V18330_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18330_293 = {S15, 0, &V18330_293_57187, &P18330_292};	/*Property = CssOverflow*/
const Value V18330_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18330_294 = {S16, 0, &V18330_294_57190, &P18330_293};	/*Property = CssPosition*/
const Value V18330_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18330_295 = {S17, 0, &V18330_295_57193, &P18330_294};	/*Property = CssText*/
const Value V18330_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18330_297 = {S18, 0, &V18330_297_57195, &P18330_295};	/*Property = CssTransforms*/
const Value V18330_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18330_241 = {S19, 0, &V18330_241_55799, &P18330_297};	/*Property = CssTransitions*/
const Value V18330_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18330_298 = {S20, 0, &V18330_298_57206, &P18330_241};	/*Property = CssUI*/
const Value V18330_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18330_300 = {S21, 0, &V18330_300_57216, &P18330_298};	/*Property = DataSet*/
const Value V18330_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18330_301 = {S22, 0, &V18330_301_57218, &P18330_300};	/*Property = DataUrl*/
const Value V18330_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18330_302 = {S23, 0, &V18330_302_57222, &P18330_301};	/*Property = DeviceOrientation*/
const Value V18330_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18330_304 = {S24, 0, &V18330_304_57228, &P18330_302};	/*Property = FileReader*/
const Value V18330_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18330_305 = {S25, 0, &V18330_305_57231, &P18330_304};	/*Property = FileSaver*/
const Value V18330_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18330_306 = {S26, 0, &V18330_306_57237, &P18330_305};	/*Property = FileWriter*/
const Value V18330_307_57240 = {S0, NULL};	/*Value = False*/
const Property P18330_307 = {S27, 0, &V18330_307_57240, &P18330_306};	/*Property = FormData*/
const Value V18330_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18330_309 = {S28, 0, &V18330_309_57246, &P18330_307};	/*Property = Fullscreen*/
const Value V18330_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18330_310 = {S29, 0, &V18330_310_57248, &P18330_309};	/*Property = GeoLocation*/
const Value V18330_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18330_312 = {S30, 0, &V18330_312_57255, &P18330_310};	/*Property = History*/
const Value V18330_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18330_314 = {S31, 0, &V18330_314_57261, &P18330_312};	/*Property = Html5*/
const Value V18330_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18330_313 = {S32, 0, &V18330_313_57258, &P18330_314};	/*Property = Html-Media-Capture*/
const Value V18330_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18330_315 = {S33, 0, &V18330_315_57264, &P18330_313};	/*Property = Iframe*/
const Value V18330_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18330_316 = {S34, 0, &V18330_316_57267, &P18330_315};	/*Property = IndexedDB*/
const Value V18330_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18330_317 = {S35, 0, &V18330_317_57269, &P18330_316};	/*Property = Json*/
const Value V18330_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18330_15 = {S37, 0, &V18330_15_29, &P18330_317};	/*Property = LayoutEngine*/
const Value V18330_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18330_344 = {S38, 0, &V18330_344_57272, &P18330_15};	/*Property = Masking*/
const Value V18330_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18330_320 = {S39, 0, &V18330_320_57290, &P18330_344};	/*Property = PostMessage*/
const Value V18330_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18330_321 = {S40, 0, &V18330_321_57293, &P18330_320};	/*Property = Progress*/
const Value V18330_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18330_322 = {S41, 0, &V18330_322_57296, &P18330_321};	/*Property = Prompts*/
const Value V18330_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18330_323 = {S42, 0, &V18330_323_57299, &P18330_322};	/*Property = Selector*/
const Value V18330_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18330_324 = {S43, 0, &V18330_324_57306, &P18330_323};	/*Property = Svg*/
const Value V18330_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18330_327 = {S44, 0, &V18330_327_57315, &P18330_324};	/*Property = TouchEvents*/
const Value V18330_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18330_328 = {S45, 0, &V18330_328_57318, &P18330_327};	/*Property = Track*/
const Value V18330_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18330_329 = {S46, 0, &V18330_329_57323, &P18330_328};	/*Property = Video*/
const Value V18330_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18330_330 = {S47, 0, &V18330_330_57326, &P18330_329};	/*Property = Viewport*/
const Value V18330_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18330_332 = {S48, 0, &V18330_332_57339, &P18330_330};	/*Property = WebWorkers*/
const Value V18330_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18330_333 = {S49, 0, &V18330_333_57342, &P18330_332};	/*Property = Xhr2*/
const Profile P18330 = {"18330", &P18330_333};	/*Profile ID - 18330*/
const Value V18329_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_279 = {S1, 0, &V18329_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18329_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_282 = {S3, 0, &V18329_282_57155, &P18329_279};	/*Property = BlobBuilder*/
const Value V18329_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_350 = {S4, 0, &V18329_350_57446, &P18329_282};	/*Property = Canvas*/
const Value V18329_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_283 = {S5, 0, &V18329_283_57161, &P18329_350};	/*Property = CssBackground*/
const Value V18329_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_284 = {S6, 0, &V18329_284_57164, &P18329_283};	/*Property = CssBorderImage*/
const Value V18329_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_285 = {S7, 0, &V18329_285_57167, &P18329_284};	/*Property = CssCanvas*/
const Value V18329_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_286 = {S8, 0, &V18329_286_57170, &P18329_285};	/*Property = CssColor*/
const Value V18329_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_237 = {S9, 0, &V18329_237_55786, &P18329_286};	/*Property = CssColumn*/
const Value V18329_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_287 = {S10, 0, &V18329_287_57173, &P18329_237};	/*Property = CssFlexbox*/
const Value V18329_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_288 = {S11, 0, &V18329_288_57176, &P18329_287};	/*Property = CssFont*/
const Value V18329_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_289 = {S12, 0, &V18329_289_86899, &P18329_288};	/*Property = CssImages*/
const Value V18329_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_291 = {S13, 0, &V18329_291_57182, &P18329_289};	/*Property = CssMediaQueries*/
const Value V18329_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_292 = {S14, 0, &V18329_292_57185, &P18329_291};	/*Property = CssMinMax*/
const Value V18329_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_293 = {S15, 0, &V18329_293_57188, &P18329_292};	/*Property = CssOverflow*/
const Value V18329_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_294 = {S16, 0, &V18329_294_57191, &P18329_293};	/*Property = CssPosition*/
const Value V18329_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_295 = {S17, 0, &V18329_295_57194, &P18329_294};	/*Property = CssText*/
const Value V18329_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_297 = {S18, 0, &V18329_297_57197, &P18329_295};	/*Property = CssTransforms*/
const Value V18329_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_241 = {S19, 0, &V18329_241_55801, &P18329_297};	/*Property = CssTransitions*/
const Value V18329_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_298 = {S20, 0, &V18329_298_57208, &P18329_241};	/*Property = CssUI*/
const Value V18329_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_300 = {S21, 0, &V18329_300_57217, &P18329_298};	/*Property = DataSet*/
const Value V18329_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_301 = {S22, 0, &V18329_301_57220, &P18329_300};	/*Property = DataUrl*/
const Value V18329_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_302 = {S23, 0, &V18329_302_57223, &P18329_301};	/*Property = DeviceOrientation*/
const Value V18329_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_304 = {S24, 0, &V18329_304_57229, &P18329_302};	/*Property = FileReader*/
const Value V18329_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_305 = {S25, 0, &V18329_305_57232, &P18329_304};	/*Property = FileSaver*/
const Value V18329_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_306 = {S26, 0, &V18329_306_57238, &P18329_305};	/*Property = FileWriter*/
const Value V18329_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_307 = {S27, 0, &V18329_307_57241, &P18329_306};	/*Property = FormData*/
const Value V18329_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_309 = {S28, 0, &V18329_309_57247, &P18329_307};	/*Property = Fullscreen*/
const Value V18329_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_310 = {S29, 0, &V18329_310_57250, &P18329_309};	/*Property = GeoLocation*/
const Value V18329_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_312 = {S30, 0, &V18329_312_57256, &P18329_310};	/*Property = History*/
const Value V18329_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_314 = {S31, 0, &V18329_314_57262, &P18329_312};	/*Property = Html5*/
const Value V18329_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_313 = {S32, 0, &V18329_313_57259, &P18329_314};	/*Property = Html-Media-Capture*/
const Value V18329_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_315 = {S33, 0, &V18329_315_57265, &P18329_313};	/*Property = Iframe*/
const Value V18329_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_316 = {S34, 0, &V18329_316_57268, &P18329_315};	/*Property = IndexedDB*/
const Value V18329_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_317 = {S35, 0, &V18329_317_57271, &P18329_316};	/*Property = Json*/
const Value V18329_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18329_15 = {S37, 0, &V18329_15_29, &P18329_317};	/*Property = LayoutEngine*/
const Value V18329_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_344 = {S38, 0, &V18329_344_57274, &P18329_15};	/*Property = Masking*/
const Value V18329_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_320 = {S39, 0, &V18329_320_57292, &P18329_344};	/*Property = PostMessage*/
const Value V18329_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_321 = {S40, 0, &V18329_321_57295, &P18329_320};	/*Property = Progress*/
const Value V18329_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_322 = {S41, 0, &V18329_322_57298, &P18329_321};	/*Property = Prompts*/
const Value V18329_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_323 = {S42, 0, &V18329_323_57301, &P18329_322};	/*Property = Selector*/
const Value V18329_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_324 = {S43, 0, &V18329_324_57307, &P18329_323};	/*Property = Svg*/
const Value V18329_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_327 = {S44, 0, &V18329_327_57316, &P18329_324};	/*Property = TouchEvents*/
const Value V18329_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_328 = {S45, 0, &V18329_328_57319, &P18329_327};	/*Property = Track*/
const Value V18329_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_329 = {S46, 0, &V18329_329_57325, &P18329_328};	/*Property = Video*/
const Value V18329_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_330 = {S47, 0, &V18329_330_57328, &P18329_329};	/*Property = Viewport*/
const Value V18329_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_332 = {S48, 0, &V18329_332_57340, &P18329_330};	/*Property = WebWorkers*/
const Value V18329_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18329_333 = {S49, 0, &V18329_333_57343, &P18329_332};	/*Property = Xhr2*/
const Profile P18329 = {"18329", &P18329_333};	/*Profile ID - 18329*/
const Profile P18118 = {"18118", NULL};	/*Profile ID - 18118*/
const Value V18099_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18099_279 = {S1, 0, &V18099_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18099_282_57153 = {S2, NULL};	/*Value = True*/
const Property P18099_282 = {S3, 0, &V18099_282_57153, &P18099_279};	/*Property = BlobBuilder*/
const Value V18099_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18099_350 = {S4, 0, &V18099_350_57444, &P18099_282};	/*Property = Canvas*/
const Value V18099_283_57160 = {S0, NULL};	/*Value = False*/
const Property P18099_283 = {S5, 0, &V18099_283_57160, &P18099_350};	/*Property = CssBackground*/
const Value V18099_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18099_284 = {S6, 0, &V18099_284_57163, &P18099_283};	/*Property = CssBorderImage*/
const Value V18099_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18099_285 = {S7, 0, &V18099_285_57166, &P18099_284};	/*Property = CssCanvas*/
const Value V18099_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18099_286 = {S8, 0, &V18099_286_57168, &P18099_285};	/*Property = CssColor*/
const Value V18099_237_55784 = {S2, NULL};	/*Value = True*/
const Property P18099_237 = {S9, 0, &V18099_237_55784, &P18099_286};	/*Property = CssColumn*/
const Value V18099_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18099_287 = {S10, 0, &V18099_287_57172, &P18099_237};	/*Property = CssFlexbox*/
const Value V18099_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18099_288 = {S11, 0, &V18099_288_57175, &P18099_287};	/*Property = CssFont*/
const Value V18099_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18099_289 = {S12, 0, &V18099_289_86899, &P18099_288};	/*Property = CssImages*/
const Value V18099_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18099_291 = {S13, 0, &V18099_291_57181, &P18099_289};	/*Property = CssMediaQueries*/
const Value V18099_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18099_292 = {S14, 0, &V18099_292_57183, &P18099_291};	/*Property = CssMinMax*/
const Value V18099_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18099_293 = {S15, 0, &V18099_293_57187, &P18099_292};	/*Property = CssOverflow*/
const Value V18099_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18099_294 = {S16, 0, &V18099_294_57189, &P18099_293};	/*Property = CssPosition*/
const Value V18099_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18099_295 = {S17, 0, &V18099_295_57193, &P18099_294};	/*Property = CssText*/
const Value V18099_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18099_297 = {S18, 0, &V18099_297_57195, &P18099_295};	/*Property = CssTransforms*/
const Value V18099_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18099_241 = {S19, 0, &V18099_241_55799, &P18099_297};	/*Property = CssTransitions*/
const Value V18099_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18099_298 = {S20, 0, &V18099_298_57206, &P18099_241};	/*Property = CssUI*/
const Value V18099_300_57215 = {S2, NULL};	/*Value = True*/
const Property P18099_300 = {S21, 0, &V18099_300_57215, &P18099_298};	/*Property = DataSet*/
const Value V18099_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18099_301 = {S22, 0, &V18099_301_57218, &P18099_300};	/*Property = DataUrl*/
const Value V18099_302_57221 = {S2, NULL};	/*Value = True*/
const Property P18099_302 = {S23, 0, &V18099_302_57221, &P18099_301};	/*Property = DeviceOrientation*/
const Value V18099_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18099_304 = {S24, 0, &V18099_304_57227, &P18099_302};	/*Property = FileReader*/
const Value V18099_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18099_305 = {S25, 0, &V18099_305_57231, &P18099_304};	/*Property = FileSaver*/
const Value V18099_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18099_306 = {S26, 0, &V18099_306_57237, &P18099_305};	/*Property = FileWriter*/
const Value V18099_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18099_307 = {S27, 0, &V18099_307_57239, &P18099_306};	/*Property = FormData*/
const Value V18099_309_57245 = {S2, NULL};	/*Value = True*/
const Property P18099_309 = {S28, 0, &V18099_309_57245, &P18099_307};	/*Property = Fullscreen*/
const Value V18099_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18099_310 = {S29, 0, &V18099_310_57248, &P18099_309};	/*Property = GeoLocation*/
const Value V18099_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18099_312 = {S30, 0, &V18099_312_57254, &P18099_310};	/*Property = History*/
const Value V18099_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18099_314 = {S31, 0, &V18099_314_57261, &P18099_312};	/*Property = Html5*/
const Value V18099_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18099_313 = {S32, 0, &V18099_313_57258, &P18099_314};	/*Property = Html-Media-Capture*/
const Value V18099_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18099_315 = {S33, 0, &V18099_315_57264, &P18099_313};	/*Property = Iframe*/
const Value V18099_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18099_316 = {S34, 0, &V18099_316_57266, &P18099_315};	/*Property = IndexedDB*/
const Value V18099_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18099_317 = {S35, 0, &V18099_317_57269, &P18099_316};	/*Property = Json*/
const Value V18099_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18099_15 = {S37, 0, &V18099_15_29, &P18099_317};	/*Property = LayoutEngine*/
const Value V18099_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18099_344 = {S38, 0, &V18099_344_57272, &P18099_15};	/*Property = Masking*/
const Value V18099_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18099_320 = {S39, 0, &V18099_320_57290, &P18099_344};	/*Property = PostMessage*/
const Value V18099_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18099_321 = {S40, 0, &V18099_321_57293, &P18099_320};	/*Property = Progress*/
const Value V18099_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18099_322 = {S41, 0, &V18099_322_57296, &P18099_321};	/*Property = Prompts*/
const Value V18099_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18099_323 = {S42, 0, &V18099_323_57299, &P18099_322};	/*Property = Selector*/
const Value V18099_324_57305 = {S2, NULL};	/*Value = True*/
const Property P18099_324 = {S43, 0, &V18099_324_57305, &P18099_323};	/*Property = Svg*/
const Value V18099_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18099_327 = {S44, 0, &V18099_327_57315, &P18099_324};	/*Property = TouchEvents*/
const Value V18099_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18099_328 = {S45, 0, &V18099_328_57318, &P18099_327};	/*Property = Track*/
const Value V18099_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18099_329 = {S46, 0, &V18099_329_57323, &P18099_328};	/*Property = Video*/
const Value V18099_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18099_330 = {S47, 0, &V18099_330_57326, &P18099_329};	/*Property = Viewport*/
const Value V18099_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18099_332 = {S48, 0, &V18099_332_57339, &P18099_330};	/*Property = WebWorkers*/
const Value V18099_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18099_333 = {S49, 0, &V18099_333_57342, &P18099_332};	/*Property = Xhr2*/
const Profile P18099 = {"18099", &P18099_333};	/*Profile ID - 18099*/
const Value V20794_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_279 = {S1, 0, &V20794_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20794_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_282 = {S3, 0, &V20794_282_57155, &P20794_279};	/*Property = BlobBuilder*/
const Value V20794_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_350 = {S4, 0, &V20794_350_57446, &P20794_282};	/*Property = Canvas*/
const Value V20794_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_283 = {S5, 0, &V20794_283_57161, &P20794_350};	/*Property = CssBackground*/
const Value V20794_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_284 = {S6, 0, &V20794_284_57164, &P20794_283};	/*Property = CssBorderImage*/
const Value V20794_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_285 = {S7, 0, &V20794_285_57167, &P20794_284};	/*Property = CssCanvas*/
const Value V20794_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_286 = {S8, 0, &V20794_286_57170, &P20794_285};	/*Property = CssColor*/
const Value V20794_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_237 = {S9, 0, &V20794_237_55786, &P20794_286};	/*Property = CssColumn*/
const Value V20794_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_287 = {S10, 0, &V20794_287_57173, &P20794_237};	/*Property = CssFlexbox*/
const Value V20794_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_288 = {S11, 0, &V20794_288_57176, &P20794_287};	/*Property = CssFont*/
const Value V20794_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_289 = {S12, 0, &V20794_289_86899, &P20794_288};	/*Property = CssImages*/
const Value V20794_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_291 = {S13, 0, &V20794_291_57182, &P20794_289};	/*Property = CssMediaQueries*/
const Value V20794_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_292 = {S14, 0, &V20794_292_57185, &P20794_291};	/*Property = CssMinMax*/
const Value V20794_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_293 = {S15, 0, &V20794_293_57188, &P20794_292};	/*Property = CssOverflow*/
const Value V20794_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_294 = {S16, 0, &V20794_294_57191, &P20794_293};	/*Property = CssPosition*/
const Value V20794_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_295 = {S17, 0, &V20794_295_57194, &P20794_294};	/*Property = CssText*/
const Value V20794_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_297 = {S18, 0, &V20794_297_57197, &P20794_295};	/*Property = CssTransforms*/
const Value V20794_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_241 = {S19, 0, &V20794_241_55801, &P20794_297};	/*Property = CssTransitions*/
const Value V20794_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_298 = {S20, 0, &V20794_298_57208, &P20794_241};	/*Property = CssUI*/
const Value V20794_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_300 = {S21, 0, &V20794_300_57217, &P20794_298};	/*Property = DataSet*/
const Value V20794_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_301 = {S22, 0, &V20794_301_57220, &P20794_300};	/*Property = DataUrl*/
const Value V20794_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_302 = {S23, 0, &V20794_302_57223, &P20794_301};	/*Property = DeviceOrientation*/
const Value V20794_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_304 = {S24, 0, &V20794_304_57229, &P20794_302};	/*Property = FileReader*/
const Value V20794_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_305 = {S25, 0, &V20794_305_57232, &P20794_304};	/*Property = FileSaver*/
const Value V20794_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_306 = {S26, 0, &V20794_306_57238, &P20794_305};	/*Property = FileWriter*/
const Value V20794_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_307 = {S27, 0, &V20794_307_57241, &P20794_306};	/*Property = FormData*/
const Value V20794_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_309 = {S28, 0, &V20794_309_57247, &P20794_307};	/*Property = Fullscreen*/
const Value V20794_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_310 = {S29, 0, &V20794_310_57250, &P20794_309};	/*Property = GeoLocation*/
const Value V20794_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_312 = {S30, 0, &V20794_312_57256, &P20794_310};	/*Property = History*/
const Value V20794_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_314 = {S31, 0, &V20794_314_57262, &P20794_312};	/*Property = Html5*/
const Value V20794_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_313 = {S32, 0, &V20794_313_57259, &P20794_314};	/*Property = Html-Media-Capture*/
const Value V20794_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_315 = {S33, 0, &V20794_315_57265, &P20794_313};	/*Property = Iframe*/
const Value V20794_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_316 = {S34, 0, &V20794_316_57268, &P20794_315};	/*Property = IndexedDB*/
const Value V20794_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_317 = {S35, 0, &V20794_317_57271, &P20794_316};	/*Property = Json*/
const Value V20794_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20794_15 = {S37, 0, &V20794_15_29, &P20794_317};	/*Property = LayoutEngine*/
const Value V20794_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_344 = {S38, 0, &V20794_344_57274, &P20794_15};	/*Property = Masking*/
const Value V20794_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_320 = {S39, 0, &V20794_320_57292, &P20794_344};	/*Property = PostMessage*/
const Value V20794_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_321 = {S40, 0, &V20794_321_57295, &P20794_320};	/*Property = Progress*/
const Value V20794_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_322 = {S41, 0, &V20794_322_57298, &P20794_321};	/*Property = Prompts*/
const Value V20794_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_323 = {S42, 0, &V20794_323_57301, &P20794_322};	/*Property = Selector*/
const Value V20794_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_324 = {S43, 0, &V20794_324_57307, &P20794_323};	/*Property = Svg*/
const Value V20794_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_327 = {S44, 0, &V20794_327_57316, &P20794_324};	/*Property = TouchEvents*/
const Value V20794_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_328 = {S45, 0, &V20794_328_57319, &P20794_327};	/*Property = Track*/
const Value V20794_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_329 = {S46, 0, &V20794_329_57325, &P20794_328};	/*Property = Video*/
const Value V20794_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_330 = {S47, 0, &V20794_330_57328, &P20794_329};	/*Property = Viewport*/
const Value V20794_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_332 = {S48, 0, &V20794_332_57340, &P20794_330};	/*Property = WebWorkers*/
const Value V20794_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20794_333 = {S49, 0, &V20794_333_57343, &P20794_332};	/*Property = Xhr2*/
const Profile P20794 = {"20794", &P20794_333};	/*Profile ID - 20794*/
const Value V18565_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_279 = {S1, 0, &V18565_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18565_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_282 = {S3, 0, &V18565_282_57155, &P18565_279};	/*Property = BlobBuilder*/
const Value V18565_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_350 = {S4, 0, &V18565_350_57446, &P18565_282};	/*Property = Canvas*/
const Value V18565_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_283 = {S5, 0, &V18565_283_57161, &P18565_350};	/*Property = CssBackground*/
const Value V18565_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_284 = {S6, 0, &V18565_284_57164, &P18565_283};	/*Property = CssBorderImage*/
const Value V18565_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_285 = {S7, 0, &V18565_285_57167, &P18565_284};	/*Property = CssCanvas*/
const Value V18565_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_286 = {S8, 0, &V18565_286_57170, &P18565_285};	/*Property = CssColor*/
const Value V18565_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_237 = {S9, 0, &V18565_237_55786, &P18565_286};	/*Property = CssColumn*/
const Value V18565_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_287 = {S10, 0, &V18565_287_57173, &P18565_237};	/*Property = CssFlexbox*/
const Value V18565_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_288 = {S11, 0, &V18565_288_57176, &P18565_287};	/*Property = CssFont*/
const Value V18565_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_289 = {S12, 0, &V18565_289_86899, &P18565_288};	/*Property = CssImages*/
const Value V18565_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_291 = {S13, 0, &V18565_291_57182, &P18565_289};	/*Property = CssMediaQueries*/
const Value V18565_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_292 = {S14, 0, &V18565_292_57185, &P18565_291};	/*Property = CssMinMax*/
const Value V18565_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_293 = {S15, 0, &V18565_293_57188, &P18565_292};	/*Property = CssOverflow*/
const Value V18565_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_294 = {S16, 0, &V18565_294_57191, &P18565_293};	/*Property = CssPosition*/
const Value V18565_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_295 = {S17, 0, &V18565_295_57194, &P18565_294};	/*Property = CssText*/
const Value V18565_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_297 = {S18, 0, &V18565_297_57197, &P18565_295};	/*Property = CssTransforms*/
const Value V18565_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_241 = {S19, 0, &V18565_241_55801, &P18565_297};	/*Property = CssTransitions*/
const Value V18565_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_298 = {S20, 0, &V18565_298_57208, &P18565_241};	/*Property = CssUI*/
const Value V18565_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_300 = {S21, 0, &V18565_300_57217, &P18565_298};	/*Property = DataSet*/
const Value V18565_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_301 = {S22, 0, &V18565_301_57220, &P18565_300};	/*Property = DataUrl*/
const Value V18565_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_302 = {S23, 0, &V18565_302_57223, &P18565_301};	/*Property = DeviceOrientation*/
const Value V18565_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_304 = {S24, 0, &V18565_304_57229, &P18565_302};	/*Property = FileReader*/
const Value V18565_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_305 = {S25, 0, &V18565_305_57232, &P18565_304};	/*Property = FileSaver*/
const Value V18565_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_306 = {S26, 0, &V18565_306_57238, &P18565_305};	/*Property = FileWriter*/
const Value V18565_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_307 = {S27, 0, &V18565_307_57241, &P18565_306};	/*Property = FormData*/
const Value V18565_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_309 = {S28, 0, &V18565_309_57247, &P18565_307};	/*Property = Fullscreen*/
const Value V18565_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_310 = {S29, 0, &V18565_310_57250, &P18565_309};	/*Property = GeoLocation*/
const Value V18565_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_312 = {S30, 0, &V18565_312_57256, &P18565_310};	/*Property = History*/
const Value V18565_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_314 = {S31, 0, &V18565_314_57262, &P18565_312};	/*Property = Html5*/
const Value V18565_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_313 = {S32, 0, &V18565_313_57259, &P18565_314};	/*Property = Html-Media-Capture*/
const Value V18565_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_315 = {S33, 0, &V18565_315_57265, &P18565_313};	/*Property = Iframe*/
const Value V18565_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_316 = {S34, 0, &V18565_316_57268, &P18565_315};	/*Property = IndexedDB*/
const Value V18565_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_317 = {S35, 0, &V18565_317_57271, &P18565_316};	/*Property = Json*/
const Value V18565_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P18565_15 = {S37, 0, &V18565_15_54167, &P18565_317};	/*Property = LayoutEngine*/
const Value V18565_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_344 = {S38, 0, &V18565_344_57274, &P18565_15};	/*Property = Masking*/
const Value V18565_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_320 = {S39, 0, &V18565_320_57292, &P18565_344};	/*Property = PostMessage*/
const Value V18565_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_321 = {S40, 0, &V18565_321_57295, &P18565_320};	/*Property = Progress*/
const Value V18565_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_322 = {S41, 0, &V18565_322_57298, &P18565_321};	/*Property = Prompts*/
const Value V18565_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_323 = {S42, 0, &V18565_323_57301, &P18565_322};	/*Property = Selector*/
const Value V18565_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_324 = {S43, 0, &V18565_324_57307, &P18565_323};	/*Property = Svg*/
const Value V18565_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_327 = {S44, 0, &V18565_327_57316, &P18565_324};	/*Property = TouchEvents*/
const Value V18565_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_328 = {S45, 0, &V18565_328_57319, &P18565_327};	/*Property = Track*/
const Value V18565_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_329 = {S46, 0, &V18565_329_57325, &P18565_328};	/*Property = Video*/
const Value V18565_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_330 = {S47, 0, &V18565_330_57328, &P18565_329};	/*Property = Viewport*/
const Value V18565_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_332 = {S48, 0, &V18565_332_57340, &P18565_330};	/*Property = WebWorkers*/
const Value V18565_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18565_333 = {S49, 0, &V18565_333_57343, &P18565_332};	/*Property = Xhr2*/
const Profile P18565 = {"18565", &P18565_333};	/*Profile ID - 18565*/
const Value V18191_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_279 = {S1, 0, &V18191_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18191_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_282 = {S3, 0, &V18191_282_57155, &P18191_279};	/*Property = BlobBuilder*/
const Value V18191_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_350 = {S4, 0, &V18191_350_57446, &P18191_282};	/*Property = Canvas*/
const Value V18191_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_283 = {S5, 0, &V18191_283_57161, &P18191_350};	/*Property = CssBackground*/
const Value V18191_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_284 = {S6, 0, &V18191_284_57164, &P18191_283};	/*Property = CssBorderImage*/
const Value V18191_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_285 = {S7, 0, &V18191_285_57167, &P18191_284};	/*Property = CssCanvas*/
const Value V18191_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_286 = {S8, 0, &V18191_286_57170, &P18191_285};	/*Property = CssColor*/
const Value V18191_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_237 = {S9, 0, &V18191_237_55786, &P18191_286};	/*Property = CssColumn*/
const Value V18191_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_287 = {S10, 0, &V18191_287_57173, &P18191_237};	/*Property = CssFlexbox*/
const Value V18191_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_288 = {S11, 0, &V18191_288_57176, &P18191_287};	/*Property = CssFont*/
const Value V18191_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_289 = {S12, 0, &V18191_289_86899, &P18191_288};	/*Property = CssImages*/
const Value V18191_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_291 = {S13, 0, &V18191_291_57182, &P18191_289};	/*Property = CssMediaQueries*/
const Value V18191_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_292 = {S14, 0, &V18191_292_57185, &P18191_291};	/*Property = CssMinMax*/
const Value V18191_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_293 = {S15, 0, &V18191_293_57188, &P18191_292};	/*Property = CssOverflow*/
const Value V18191_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_294 = {S16, 0, &V18191_294_57191, &P18191_293};	/*Property = CssPosition*/
const Value V18191_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_295 = {S17, 0, &V18191_295_57194, &P18191_294};	/*Property = CssText*/
const Value V18191_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_297 = {S18, 0, &V18191_297_57197, &P18191_295};	/*Property = CssTransforms*/
const Value V18191_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_241 = {S19, 0, &V18191_241_55801, &P18191_297};	/*Property = CssTransitions*/
const Value V18191_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_298 = {S20, 0, &V18191_298_57208, &P18191_241};	/*Property = CssUI*/
const Value V18191_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_300 = {S21, 0, &V18191_300_57217, &P18191_298};	/*Property = DataSet*/
const Value V18191_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_301 = {S22, 0, &V18191_301_57220, &P18191_300};	/*Property = DataUrl*/
const Value V18191_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_302 = {S23, 0, &V18191_302_57223, &P18191_301};	/*Property = DeviceOrientation*/
const Value V18191_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_304 = {S24, 0, &V18191_304_57229, &P18191_302};	/*Property = FileReader*/
const Value V18191_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_305 = {S25, 0, &V18191_305_57232, &P18191_304};	/*Property = FileSaver*/
const Value V18191_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_306 = {S26, 0, &V18191_306_57238, &P18191_305};	/*Property = FileWriter*/
const Value V18191_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_307 = {S27, 0, &V18191_307_57241, &P18191_306};	/*Property = FormData*/
const Value V18191_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_309 = {S28, 0, &V18191_309_57247, &P18191_307};	/*Property = Fullscreen*/
const Value V18191_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_310 = {S29, 0, &V18191_310_57250, &P18191_309};	/*Property = GeoLocation*/
const Value V18191_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_312 = {S30, 0, &V18191_312_57256, &P18191_310};	/*Property = History*/
const Value V18191_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_314 = {S31, 0, &V18191_314_57262, &P18191_312};	/*Property = Html5*/
const Value V18191_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_313 = {S32, 0, &V18191_313_57259, &P18191_314};	/*Property = Html-Media-Capture*/
const Value V18191_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_315 = {S33, 0, &V18191_315_57265, &P18191_313};	/*Property = Iframe*/
const Value V18191_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_316 = {S34, 0, &V18191_316_57268, &P18191_315};	/*Property = IndexedDB*/
const Value V18191_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_317 = {S35, 0, &V18191_317_57271, &P18191_316};	/*Property = Json*/
const Value V18191_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18191_15 = {S37, 0, &V18191_15_29, &P18191_317};	/*Property = LayoutEngine*/
const Value V18191_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_344 = {S38, 0, &V18191_344_57274, &P18191_15};	/*Property = Masking*/
const Value V18191_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_320 = {S39, 0, &V18191_320_57292, &P18191_344};	/*Property = PostMessage*/
const Value V18191_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_321 = {S40, 0, &V18191_321_57295, &P18191_320};	/*Property = Progress*/
const Value V18191_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_322 = {S41, 0, &V18191_322_57298, &P18191_321};	/*Property = Prompts*/
const Value V18191_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_323 = {S42, 0, &V18191_323_57301, &P18191_322};	/*Property = Selector*/
const Value V18191_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_324 = {S43, 0, &V18191_324_57307, &P18191_323};	/*Property = Svg*/
const Value V18191_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_327 = {S44, 0, &V18191_327_57316, &P18191_324};	/*Property = TouchEvents*/
const Value V18191_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_328 = {S45, 0, &V18191_328_57319, &P18191_327};	/*Property = Track*/
const Value V18191_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_329 = {S46, 0, &V18191_329_57325, &P18191_328};	/*Property = Video*/
const Value V18191_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_330 = {S47, 0, &V18191_330_57328, &P18191_329};	/*Property = Viewport*/
const Value V18191_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_332 = {S48, 0, &V18191_332_57340, &P18191_330};	/*Property = WebWorkers*/
const Value V18191_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18191_333 = {S49, 0, &V18191_333_57343, &P18191_332};	/*Property = Xhr2*/
const Profile P18191 = {"18191", &P18191_333};	/*Profile ID - 18191*/
const Value V18267_1_1 = {S2, NULL};	/*Value = True*/
const Property P18267_1 = {S50, 0, &V18267_1_1, NULL};	/*Property = IsMobile*/
const Value V18267_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18267_5 = {S52, 0, &V18267_5_1222, &P18267_1};	/*Property = ScreenPixelsHeight*/
const Value V18267_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18267_4 = {S54, 0, &V18267_4_1224, &P18267_5};	/*Property = ScreenPixelsWidth*/
const Profile P18267 = {"18267", &P18267_4};	/*Profile ID - 18267*/
const Value V22314_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_279 = {S1, 0, &V22314_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22314_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_282 = {S3, 0, &V22314_282_57155, &P22314_279};	/*Property = BlobBuilder*/
const Value V22314_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_350 = {S4, 0, &V22314_350_57446, &P22314_282};	/*Property = Canvas*/
const Value V22314_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_283 = {S5, 0, &V22314_283_57161, &P22314_350};	/*Property = CssBackground*/
const Value V22314_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_284 = {S6, 0, &V22314_284_57164, &P22314_283};	/*Property = CssBorderImage*/
const Value V22314_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_285 = {S7, 0, &V22314_285_57167, &P22314_284};	/*Property = CssCanvas*/
const Value V22314_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_286 = {S8, 0, &V22314_286_57170, &P22314_285};	/*Property = CssColor*/
const Value V22314_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_237 = {S9, 0, &V22314_237_55786, &P22314_286};	/*Property = CssColumn*/
const Value V22314_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_287 = {S10, 0, &V22314_287_57173, &P22314_237};	/*Property = CssFlexbox*/
const Value V22314_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_288 = {S11, 0, &V22314_288_57176, &P22314_287};	/*Property = CssFont*/
const Value V22314_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_289 = {S12, 0, &V22314_289_86899, &P22314_288};	/*Property = CssImages*/
const Value V22314_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_291 = {S13, 0, &V22314_291_57182, &P22314_289};	/*Property = CssMediaQueries*/
const Value V22314_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_292 = {S14, 0, &V22314_292_57185, &P22314_291};	/*Property = CssMinMax*/
const Value V22314_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_293 = {S15, 0, &V22314_293_57188, &P22314_292};	/*Property = CssOverflow*/
const Value V22314_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_294 = {S16, 0, &V22314_294_57191, &P22314_293};	/*Property = CssPosition*/
const Value V22314_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_295 = {S17, 0, &V22314_295_57194, &P22314_294};	/*Property = CssText*/
const Value V22314_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_297 = {S18, 0, &V22314_297_57197, &P22314_295};	/*Property = CssTransforms*/
const Value V22314_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_241 = {S19, 0, &V22314_241_55801, &P22314_297};	/*Property = CssTransitions*/
const Value V22314_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_298 = {S20, 0, &V22314_298_57208, &P22314_241};	/*Property = CssUI*/
const Value V22314_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_300 = {S21, 0, &V22314_300_57217, &P22314_298};	/*Property = DataSet*/
const Value V22314_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_301 = {S22, 0, &V22314_301_57220, &P22314_300};	/*Property = DataUrl*/
const Value V22314_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_302 = {S23, 0, &V22314_302_57223, &P22314_301};	/*Property = DeviceOrientation*/
const Value V22314_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_304 = {S24, 0, &V22314_304_57229, &P22314_302};	/*Property = FileReader*/
const Value V22314_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_305 = {S25, 0, &V22314_305_57232, &P22314_304};	/*Property = FileSaver*/
const Value V22314_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_306 = {S26, 0, &V22314_306_57238, &P22314_305};	/*Property = FileWriter*/
const Value V22314_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_307 = {S27, 0, &V22314_307_57241, &P22314_306};	/*Property = FormData*/
const Value V22314_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_309 = {S28, 0, &V22314_309_57247, &P22314_307};	/*Property = Fullscreen*/
const Value V22314_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_310 = {S29, 0, &V22314_310_57250, &P22314_309};	/*Property = GeoLocation*/
const Value V22314_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_312 = {S30, 0, &V22314_312_57256, &P22314_310};	/*Property = History*/
const Value V22314_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_314 = {S31, 0, &V22314_314_57262, &P22314_312};	/*Property = Html5*/
const Value V22314_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_313 = {S32, 0, &V22314_313_57259, &P22314_314};	/*Property = Html-Media-Capture*/
const Value V22314_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_315 = {S33, 0, &V22314_315_57265, &P22314_313};	/*Property = Iframe*/
const Value V22314_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_316 = {S34, 0, &V22314_316_57268, &P22314_315};	/*Property = IndexedDB*/
const Value V22314_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_317 = {S35, 0, &V22314_317_57271, &P22314_316};	/*Property = Json*/
const Value V22314_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22314_15 = {S37, 0, &V22314_15_29, &P22314_317};	/*Property = LayoutEngine*/
const Value V22314_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_344 = {S38, 0, &V22314_344_57274, &P22314_15};	/*Property = Masking*/
const Value V22314_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_320 = {S39, 0, &V22314_320_57292, &P22314_344};	/*Property = PostMessage*/
const Value V22314_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_321 = {S40, 0, &V22314_321_57295, &P22314_320};	/*Property = Progress*/
const Value V22314_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_322 = {S41, 0, &V22314_322_57298, &P22314_321};	/*Property = Prompts*/
const Value V22314_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_323 = {S42, 0, &V22314_323_57301, &P22314_322};	/*Property = Selector*/
const Value V22314_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_324 = {S43, 0, &V22314_324_57307, &P22314_323};	/*Property = Svg*/
const Value V22314_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_327 = {S44, 0, &V22314_327_57316, &P22314_324};	/*Property = TouchEvents*/
const Value V22314_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_328 = {S45, 0, &V22314_328_57319, &P22314_327};	/*Property = Track*/
const Value V22314_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_329 = {S46, 0, &V22314_329_57325, &P22314_328};	/*Property = Video*/
const Value V22314_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_330 = {S47, 0, &V22314_330_57328, &P22314_329};	/*Property = Viewport*/
const Value V22314_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_332 = {S48, 0, &V22314_332_57340, &P22314_330};	/*Property = WebWorkers*/
const Value V22314_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22314_333 = {S49, 0, &V22314_333_57343, &P22314_332};	/*Property = Xhr2*/
const Profile P22314 = {"22314", &P22314_333};	/*Profile ID - 22314*/
const Value V16001_1_1 = {S2, NULL};	/*Value = True*/
const Property P16001_1 = {S50, 0, &V16001_1_1, NULL};	/*Property = IsMobile*/
const Value V16001_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P16001_5 = {S52, 0, &V16001_5_9981, &P16001_1};	/*Property = ScreenPixelsHeight*/
const Value V16001_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P16001_4 = {S54, 0, &V16001_4_39098, &P16001_5};	/*Property = ScreenPixelsWidth*/
const Profile P16001 = {"16001", &P16001_4};	/*Profile ID - 16001*/
const Value V19891_1_1 = {S2, NULL};	/*Value = True*/
const Property P19891_1 = {S50, 0, &V19891_1_1, NULL};	/*Property = IsMobile*/
const Value V19891_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19891_5 = {S52, 0, &V19891_5_9981, &P19891_1};	/*Property = ScreenPixelsHeight*/
const Value V19891_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P19891_4 = {S54, 0, &V19891_4_2336, &P19891_5};	/*Property = ScreenPixelsWidth*/
const Profile P19891 = {"19891", &P19891_4};	/*Profile ID - 19891*/
const Value V22505_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22505_279 = {S1, 0, &V22505_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22505_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22505_282 = {S3, 0, &V22505_282_57153, &P22505_279};	/*Property = BlobBuilder*/
const Value V22505_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22505_350 = {S4, 0, &V22505_350_57444, &P22505_282};	/*Property = Canvas*/
const Value V22505_283_57160 = {S0, NULL};	/*Value = False*/
const Property P22505_283 = {S5, 0, &V22505_283_57160, &P22505_350};	/*Property = CssBackground*/
const Value V22505_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22505_284 = {S6, 0, &V22505_284_57163, &P22505_283};	/*Property = CssBorderImage*/
const Value V22505_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22505_285 = {S7, 0, &V22505_285_57166, &P22505_284};	/*Property = CssCanvas*/
const Value V22505_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22505_286 = {S8, 0, &V22505_286_57168, &P22505_285};	/*Property = CssColor*/
const Value V22505_237_55785 = {S0, NULL};	/*Value = False*/
const Property P22505_237 = {S9, 0, &V22505_237_55785, &P22505_286};	/*Property = CssColumn*/
const Value V22505_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22505_287 = {S10, 0, &V22505_287_57172, &P22505_237};	/*Property = CssFlexbox*/
const Value V22505_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22505_288 = {S11, 0, &V22505_288_57175, &P22505_287};	/*Property = CssFont*/
const Value V22505_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22505_289 = {S12, 0, &V22505_289_86899, &P22505_288};	/*Property = CssImages*/
const Value V22505_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22505_291 = {S13, 0, &V22505_291_57181, &P22505_289};	/*Property = CssMediaQueries*/
const Value V22505_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22505_292 = {S14, 0, &V22505_292_57183, &P22505_291};	/*Property = CssMinMax*/
const Value V22505_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22505_293 = {S15, 0, &V22505_293_57187, &P22505_292};	/*Property = CssOverflow*/
const Value V22505_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22505_294 = {S16, 0, &V22505_294_57189, &P22505_293};	/*Property = CssPosition*/
const Value V22505_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22505_295 = {S17, 0, &V22505_295_57193, &P22505_294};	/*Property = CssText*/
const Value V22505_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22505_297 = {S18, 0, &V22505_297_57195, &P22505_295};	/*Property = CssTransforms*/
const Value V22505_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22505_241 = {S19, 0, &V22505_241_55799, &P22505_297};	/*Property = CssTransitions*/
const Value V22505_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22505_298 = {S20, 0, &V22505_298_57206, &P22505_241};	/*Property = CssUI*/
const Value V22505_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22505_300 = {S21, 0, &V22505_300_57215, &P22505_298};	/*Property = DataSet*/
const Value V22505_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22505_301 = {S22, 0, &V22505_301_57218, &P22505_300};	/*Property = DataUrl*/
const Value V22505_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22505_302 = {S23, 0, &V22505_302_57221, &P22505_301};	/*Property = DeviceOrientation*/
const Value V22505_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22505_304 = {S24, 0, &V22505_304_57227, &P22505_302};	/*Property = FileReader*/
const Value V22505_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22505_305 = {S25, 0, &V22505_305_57231, &P22505_304};	/*Property = FileSaver*/
const Value V22505_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22505_306 = {S26, 0, &V22505_306_57237, &P22505_305};	/*Property = FileWriter*/
const Value V22505_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22505_307 = {S27, 0, &V22505_307_57239, &P22505_306};	/*Property = FormData*/
const Value V22505_309_57245 = {S2, NULL};	/*Value = True*/
const Property P22505_309 = {S28, 0, &V22505_309_57245, &P22505_307};	/*Property = Fullscreen*/
const Value V22505_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22505_310 = {S29, 0, &V22505_310_57248, &P22505_309};	/*Property = GeoLocation*/
const Value V22505_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22505_312 = {S30, 0, &V22505_312_57254, &P22505_310};	/*Property = History*/
const Value V22505_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22505_314 = {S31, 0, &V22505_314_57261, &P22505_312};	/*Property = Html5*/
const Value V22505_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22505_313 = {S32, 0, &V22505_313_57258, &P22505_314};	/*Property = Html-Media-Capture*/
const Value V22505_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22505_315 = {S33, 0, &V22505_315_57264, &P22505_313};	/*Property = Iframe*/
const Value V22505_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22505_316 = {S34, 0, &V22505_316_57266, &P22505_315};	/*Property = IndexedDB*/
const Value V22505_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22505_317 = {S35, 0, &V22505_317_57269, &P22505_316};	/*Property = Json*/
const Value V22505_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22505_15 = {S37, 0, &V22505_15_29, &P22505_317};	/*Property = LayoutEngine*/
const Value V22505_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22505_344 = {S38, 0, &V22505_344_57272, &P22505_15};	/*Property = Masking*/
const Value V22505_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22505_320 = {S39, 0, &V22505_320_57290, &P22505_344};	/*Property = PostMessage*/
const Value V22505_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22505_321 = {S40, 0, &V22505_321_57293, &P22505_320};	/*Property = Progress*/
const Value V22505_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22505_322 = {S41, 0, &V22505_322_57296, &P22505_321};	/*Property = Prompts*/
const Value V22505_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22505_323 = {S42, 0, &V22505_323_57299, &P22505_322};	/*Property = Selector*/
const Value V22505_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22505_324 = {S43, 0, &V22505_324_57305, &P22505_323};	/*Property = Svg*/
const Value V22505_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22505_327 = {S44, 0, &V22505_327_57315, &P22505_324};	/*Property = TouchEvents*/
const Value V22505_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22505_328 = {S45, 0, &V22505_328_57318, &P22505_327};	/*Property = Track*/
const Value V22505_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22505_329 = {S46, 0, &V22505_329_57323, &P22505_328};	/*Property = Video*/
const Value V22505_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22505_330 = {S47, 0, &V22505_330_57326, &P22505_329};	/*Property = Viewport*/
const Value V22505_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22505_332 = {S48, 0, &V22505_332_57339, &P22505_330};	/*Property = WebWorkers*/
const Value V22505_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22505_333 = {S49, 0, &V22505_333_57342, &P22505_332};	/*Property = Xhr2*/
const Profile P22505 = {"22505", &P22505_333};	/*Profile ID - 22505*/
const Value V18348_1_1 = {S2, NULL};	/*Value = True*/
const Property P18348_1 = {S50, 0, &V18348_1_1, NULL};	/*Property = IsMobile*/
const Value V18348_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18348_5 = {S52, 0, &V18348_5_1403, &P18348_1};	/*Property = ScreenPixelsHeight*/
const Value V18348_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18348_4 = {S54, 0, &V18348_4_1404, &P18348_5};	/*Property = ScreenPixelsWidth*/
const Profile P18348 = {"18348", &P18348_4};	/*Profile ID - 18348*/
const Value V18326_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_279 = {S1, 0, &V18326_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18326_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_282 = {S3, 0, &V18326_282_57155, &P18326_279};	/*Property = BlobBuilder*/
const Value V18326_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_350 = {S4, 0, &V18326_350_57446, &P18326_282};	/*Property = Canvas*/
const Value V18326_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_283 = {S5, 0, &V18326_283_57161, &P18326_350};	/*Property = CssBackground*/
const Value V18326_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_284 = {S6, 0, &V18326_284_57164, &P18326_283};	/*Property = CssBorderImage*/
const Value V18326_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_285 = {S7, 0, &V18326_285_57167, &P18326_284};	/*Property = CssCanvas*/
const Value V18326_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_286 = {S8, 0, &V18326_286_57170, &P18326_285};	/*Property = CssColor*/
const Value V18326_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_237 = {S9, 0, &V18326_237_55786, &P18326_286};	/*Property = CssColumn*/
const Value V18326_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_287 = {S10, 0, &V18326_287_57173, &P18326_237};	/*Property = CssFlexbox*/
const Value V18326_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_288 = {S11, 0, &V18326_288_57176, &P18326_287};	/*Property = CssFont*/
const Value V18326_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_289 = {S12, 0, &V18326_289_86899, &P18326_288};	/*Property = CssImages*/
const Value V18326_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_291 = {S13, 0, &V18326_291_57182, &P18326_289};	/*Property = CssMediaQueries*/
const Value V18326_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_292 = {S14, 0, &V18326_292_57185, &P18326_291};	/*Property = CssMinMax*/
const Value V18326_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_293 = {S15, 0, &V18326_293_57188, &P18326_292};	/*Property = CssOverflow*/
const Value V18326_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_294 = {S16, 0, &V18326_294_57191, &P18326_293};	/*Property = CssPosition*/
const Value V18326_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_295 = {S17, 0, &V18326_295_57194, &P18326_294};	/*Property = CssText*/
const Value V18326_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_297 = {S18, 0, &V18326_297_57197, &P18326_295};	/*Property = CssTransforms*/
const Value V18326_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_241 = {S19, 0, &V18326_241_55801, &P18326_297};	/*Property = CssTransitions*/
const Value V18326_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_298 = {S20, 0, &V18326_298_57208, &P18326_241};	/*Property = CssUI*/
const Value V18326_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_300 = {S21, 0, &V18326_300_57217, &P18326_298};	/*Property = DataSet*/
const Value V18326_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_301 = {S22, 0, &V18326_301_57220, &P18326_300};	/*Property = DataUrl*/
const Value V18326_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_302 = {S23, 0, &V18326_302_57223, &P18326_301};	/*Property = DeviceOrientation*/
const Value V18326_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_304 = {S24, 0, &V18326_304_57229, &P18326_302};	/*Property = FileReader*/
const Value V18326_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_305 = {S25, 0, &V18326_305_57232, &P18326_304};	/*Property = FileSaver*/
const Value V18326_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_306 = {S26, 0, &V18326_306_57238, &P18326_305};	/*Property = FileWriter*/
const Value V18326_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_307 = {S27, 0, &V18326_307_57241, &P18326_306};	/*Property = FormData*/
const Value V18326_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_309 = {S28, 0, &V18326_309_57247, &P18326_307};	/*Property = Fullscreen*/
const Value V18326_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_310 = {S29, 0, &V18326_310_57250, &P18326_309};	/*Property = GeoLocation*/
const Value V18326_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_312 = {S30, 0, &V18326_312_57256, &P18326_310};	/*Property = History*/
const Value V18326_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_314 = {S31, 0, &V18326_314_57262, &P18326_312};	/*Property = Html5*/
const Value V18326_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_313 = {S32, 0, &V18326_313_57259, &P18326_314};	/*Property = Html-Media-Capture*/
const Value V18326_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_315 = {S33, 0, &V18326_315_57265, &P18326_313};	/*Property = Iframe*/
const Value V18326_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_316 = {S34, 0, &V18326_316_57268, &P18326_315};	/*Property = IndexedDB*/
const Value V18326_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_317 = {S35, 0, &V18326_317_57271, &P18326_316};	/*Property = Json*/
const Value V18326_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18326_15 = {S37, 0, &V18326_15_28, &P18326_317};	/*Property = LayoutEngine*/
const Value V18326_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_344 = {S38, 0, &V18326_344_57274, &P18326_15};	/*Property = Masking*/
const Value V18326_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_320 = {S39, 0, &V18326_320_57292, &P18326_344};	/*Property = PostMessage*/
const Value V18326_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_321 = {S40, 0, &V18326_321_57295, &P18326_320};	/*Property = Progress*/
const Value V18326_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_322 = {S41, 0, &V18326_322_57298, &P18326_321};	/*Property = Prompts*/
const Value V18326_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_323 = {S42, 0, &V18326_323_57301, &P18326_322};	/*Property = Selector*/
const Value V18326_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_324 = {S43, 0, &V18326_324_57307, &P18326_323};	/*Property = Svg*/
const Value V18326_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_327 = {S44, 0, &V18326_327_57316, &P18326_324};	/*Property = TouchEvents*/
const Value V18326_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_328 = {S45, 0, &V18326_328_57319, &P18326_327};	/*Property = Track*/
const Value V18326_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_329 = {S46, 0, &V18326_329_57325, &P18326_328};	/*Property = Video*/
const Value V18326_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_330 = {S47, 0, &V18326_330_57328, &P18326_329};	/*Property = Viewport*/
const Value V18326_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_332 = {S48, 0, &V18326_332_57340, &P18326_330};	/*Property = WebWorkers*/
const Value V18326_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18326_333 = {S49, 0, &V18326_333_57343, &P18326_332};	/*Property = Xhr2*/
const Profile P18326 = {"18326", &P18326_333};	/*Profile ID - 18326*/
const Value V15837_1_1 = {S2, NULL};	/*Value = True*/
const Property P15837_1 = {S50, 0, &V15837_1_1, NULL};	/*Property = IsMobile*/
const Value V15837_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15837_5 = {S52, 0, &V15837_5_1222, &P15837_1};	/*Property = ScreenPixelsHeight*/
const Value V15837_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15837_4 = {S54, 0, &V15837_4_1224, &P15837_5};	/*Property = ScreenPixelsWidth*/
const Profile P15837 = {"15837", &P15837_4};	/*Profile ID - 15837*/
const Value V18273_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_279 = {S1, 0, &V18273_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18273_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_282 = {S3, 0, &V18273_282_57155, &P18273_279};	/*Property = BlobBuilder*/
const Value V18273_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_350 = {S4, 0, &V18273_350_57446, &P18273_282};	/*Property = Canvas*/
const Value V18273_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_283 = {S5, 0, &V18273_283_57161, &P18273_350};	/*Property = CssBackground*/
const Value V18273_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_284 = {S6, 0, &V18273_284_57164, &P18273_283};	/*Property = CssBorderImage*/
const Value V18273_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_285 = {S7, 0, &V18273_285_57167, &P18273_284};	/*Property = CssCanvas*/
const Value V18273_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_286 = {S8, 0, &V18273_286_57170, &P18273_285};	/*Property = CssColor*/
const Value V18273_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_237 = {S9, 0, &V18273_237_55786, &P18273_286};	/*Property = CssColumn*/
const Value V18273_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_287 = {S10, 0, &V18273_287_57173, &P18273_237};	/*Property = CssFlexbox*/
const Value V18273_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_288 = {S11, 0, &V18273_288_57176, &P18273_287};	/*Property = CssFont*/
const Value V18273_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_289 = {S12, 0, &V18273_289_86899, &P18273_288};	/*Property = CssImages*/
const Value V18273_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_291 = {S13, 0, &V18273_291_57182, &P18273_289};	/*Property = CssMediaQueries*/
const Value V18273_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_292 = {S14, 0, &V18273_292_57185, &P18273_291};	/*Property = CssMinMax*/
const Value V18273_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_293 = {S15, 0, &V18273_293_57188, &P18273_292};	/*Property = CssOverflow*/
const Value V18273_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_294 = {S16, 0, &V18273_294_57191, &P18273_293};	/*Property = CssPosition*/
const Value V18273_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_295 = {S17, 0, &V18273_295_57194, &P18273_294};	/*Property = CssText*/
const Value V18273_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_297 = {S18, 0, &V18273_297_57197, &P18273_295};	/*Property = CssTransforms*/
const Value V18273_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_241 = {S19, 0, &V18273_241_55801, &P18273_297};	/*Property = CssTransitions*/
const Value V18273_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_298 = {S20, 0, &V18273_298_57208, &P18273_241};	/*Property = CssUI*/
const Value V18273_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_300 = {S21, 0, &V18273_300_57217, &P18273_298};	/*Property = DataSet*/
const Value V18273_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_301 = {S22, 0, &V18273_301_57220, &P18273_300};	/*Property = DataUrl*/
const Value V18273_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_302 = {S23, 0, &V18273_302_57223, &P18273_301};	/*Property = DeviceOrientation*/
const Value V18273_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_304 = {S24, 0, &V18273_304_57229, &P18273_302};	/*Property = FileReader*/
const Value V18273_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_305 = {S25, 0, &V18273_305_57232, &P18273_304};	/*Property = FileSaver*/
const Value V18273_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_306 = {S26, 0, &V18273_306_57238, &P18273_305};	/*Property = FileWriter*/
const Value V18273_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_307 = {S27, 0, &V18273_307_57241, &P18273_306};	/*Property = FormData*/
const Value V18273_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_309 = {S28, 0, &V18273_309_57247, &P18273_307};	/*Property = Fullscreen*/
const Value V18273_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_310 = {S29, 0, &V18273_310_57250, &P18273_309};	/*Property = GeoLocation*/
const Value V18273_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_312 = {S30, 0, &V18273_312_57256, &P18273_310};	/*Property = History*/
const Value V18273_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_314 = {S31, 0, &V18273_314_57262, &P18273_312};	/*Property = Html5*/
const Value V18273_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_313 = {S32, 0, &V18273_313_57259, &P18273_314};	/*Property = Html-Media-Capture*/
const Value V18273_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_315 = {S33, 0, &V18273_315_57265, &P18273_313};	/*Property = Iframe*/
const Value V18273_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_316 = {S34, 0, &V18273_316_57268, &P18273_315};	/*Property = IndexedDB*/
const Value V18273_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_317 = {S35, 0, &V18273_317_57271, &P18273_316};	/*Property = Json*/
const Value V18273_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18273_15 = {S37, 0, &V18273_15_29, &P18273_317};	/*Property = LayoutEngine*/
const Value V18273_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_344 = {S38, 0, &V18273_344_57274, &P18273_15};	/*Property = Masking*/
const Value V18273_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_320 = {S39, 0, &V18273_320_57292, &P18273_344};	/*Property = PostMessage*/
const Value V18273_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_321 = {S40, 0, &V18273_321_57295, &P18273_320};	/*Property = Progress*/
const Value V18273_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_322 = {S41, 0, &V18273_322_57298, &P18273_321};	/*Property = Prompts*/
const Value V18273_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_323 = {S42, 0, &V18273_323_57301, &P18273_322};	/*Property = Selector*/
const Value V18273_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_324 = {S43, 0, &V18273_324_57307, &P18273_323};	/*Property = Svg*/
const Value V18273_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_327 = {S44, 0, &V18273_327_57316, &P18273_324};	/*Property = TouchEvents*/
const Value V18273_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_328 = {S45, 0, &V18273_328_57319, &P18273_327};	/*Property = Track*/
const Value V18273_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_329 = {S46, 0, &V18273_329_57325, &P18273_328};	/*Property = Video*/
const Value V18273_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_330 = {S47, 0, &V18273_330_57328, &P18273_329};	/*Property = Viewport*/
const Value V18273_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_332 = {S48, 0, &V18273_332_57340, &P18273_330};	/*Property = WebWorkers*/
const Value V18273_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18273_333 = {S49, 0, &V18273_333_57343, &P18273_332};	/*Property = Xhr2*/
const Profile P18273 = {"18273", &P18273_333};	/*Profile ID - 18273*/
const Value V18101_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18101_279 = {S1, 0, &V18101_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18101_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18101_282 = {S3, 0, &V18101_282_57154, &P18101_279};	/*Property = BlobBuilder*/
const Value V18101_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18101_350 = {S4, 0, &V18101_350_57444, &P18101_282};	/*Property = Canvas*/
const Value V18101_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18101_283 = {S5, 0, &V18101_283_57159, &P18101_350};	/*Property = CssBackground*/
const Value V18101_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18101_284 = {S6, 0, &V18101_284_57163, &P18101_283};	/*Property = CssBorderImage*/
const Value V18101_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18101_285 = {S7, 0, &V18101_285_57166, &P18101_284};	/*Property = CssCanvas*/
const Value V18101_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18101_286 = {S8, 0, &V18101_286_57168, &P18101_285};	/*Property = CssColor*/
const Value V18101_237_55784 = {S2, NULL};	/*Value = True*/
const Property P18101_237 = {S9, 0, &V18101_237_55784, &P18101_286};	/*Property = CssColumn*/
const Value V18101_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18101_287 = {S10, 0, &V18101_287_57172, &P18101_237};	/*Property = CssFlexbox*/
const Value V18101_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18101_288 = {S11, 0, &V18101_288_57175, &P18101_287};	/*Property = CssFont*/
const Value V18101_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18101_289 = {S12, 0, &V18101_289_57759, &P18101_288};	/*Property = CssImages*/
const Value V18101_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18101_291 = {S13, 0, &V18101_291_57181, &P18101_289};	/*Property = CssMediaQueries*/
const Value V18101_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18101_292 = {S14, 0, &V18101_292_57183, &P18101_291};	/*Property = CssMinMax*/
const Value V18101_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18101_293 = {S15, 0, &V18101_293_57187, &P18101_292};	/*Property = CssOverflow*/
const Value V18101_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18101_294 = {S16, 0, &V18101_294_57190, &P18101_293};	/*Property = CssPosition*/
const Value V18101_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18101_295 = {S17, 0, &V18101_295_57193, &P18101_294};	/*Property = CssText*/
const Value V18101_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18101_297 = {S18, 0, &V18101_297_57195, &P18101_295};	/*Property = CssTransforms*/
const Value V18101_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18101_241 = {S19, 0, &V18101_241_55799, &P18101_297};	/*Property = CssTransitions*/
const Value V18101_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18101_298 = {S20, 0, &V18101_298_57206, &P18101_241};	/*Property = CssUI*/
const Value V18101_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18101_300 = {S21, 0, &V18101_300_57216, &P18101_298};	/*Property = DataSet*/
const Value V18101_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18101_301 = {S22, 0, &V18101_301_57218, &P18101_300};	/*Property = DataUrl*/
const Value V18101_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18101_302 = {S23, 0, &V18101_302_57222, &P18101_301};	/*Property = DeviceOrientation*/
const Value V18101_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18101_304 = {S24, 0, &V18101_304_57228, &P18101_302};	/*Property = FileReader*/
const Value V18101_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18101_305 = {S25, 0, &V18101_305_57231, &P18101_304};	/*Property = FileSaver*/
const Value V18101_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18101_306 = {S26, 0, &V18101_306_57237, &P18101_305};	/*Property = FileWriter*/
const Value V18101_307_57240 = {S0, NULL};	/*Value = False*/
const Property P18101_307 = {S27, 0, &V18101_307_57240, &P18101_306};	/*Property = FormData*/
const Value V18101_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18101_309 = {S28, 0, &V18101_309_57246, &P18101_307};	/*Property = Fullscreen*/
const Value V18101_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18101_310 = {S29, 0, &V18101_310_57248, &P18101_309};	/*Property = GeoLocation*/
const Value V18101_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18101_312 = {S30, 0, &V18101_312_57255, &P18101_310};	/*Property = History*/
const Value V18101_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18101_314 = {S31, 0, &V18101_314_57261, &P18101_312};	/*Property = Html5*/
const Value V18101_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18101_313 = {S32, 0, &V18101_313_57258, &P18101_314};	/*Property = Html-Media-Capture*/
const Value V18101_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18101_315 = {S33, 0, &V18101_315_57264, &P18101_313};	/*Property = Iframe*/
const Value V18101_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18101_316 = {S34, 0, &V18101_316_57267, &P18101_315};	/*Property = IndexedDB*/
const Value V18101_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18101_317 = {S35, 0, &V18101_317_57269, &P18101_316};	/*Property = Json*/
const Value V18101_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18101_15 = {S37, 0, &V18101_15_29, &P18101_317};	/*Property = LayoutEngine*/
const Value V18101_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18101_344 = {S38, 0, &V18101_344_57272, &P18101_15};	/*Property = Masking*/
const Value V18101_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18101_320 = {S39, 0, &V18101_320_57290, &P18101_344};	/*Property = PostMessage*/
const Value V18101_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18101_321 = {S40, 0, &V18101_321_57293, &P18101_320};	/*Property = Progress*/
const Value V18101_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18101_322 = {S41, 0, &V18101_322_57296, &P18101_321};	/*Property = Prompts*/
const Value V18101_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18101_323 = {S42, 0, &V18101_323_57299, &P18101_322};	/*Property = Selector*/
const Value V18101_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18101_324 = {S43, 0, &V18101_324_57306, &P18101_323};	/*Property = Svg*/
const Value V18101_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18101_327 = {S44, 0, &V18101_327_57315, &P18101_324};	/*Property = TouchEvents*/
const Value V18101_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18101_328 = {S45, 0, &V18101_328_57318, &P18101_327};	/*Property = Track*/
const Value V18101_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18101_329 = {S46, 0, &V18101_329_57323, &P18101_328};	/*Property = Video*/
const Value V18101_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18101_330 = {S47, 0, &V18101_330_57326, &P18101_329};	/*Property = Viewport*/
const Value V18101_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18101_332 = {S48, 0, &V18101_332_57339, &P18101_330};	/*Property = WebWorkers*/
const Value V18101_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18101_333 = {S49, 0, &V18101_333_57342, &P18101_332};	/*Property = Xhr2*/
const Profile P18101 = {"18101", &P18101_333};	/*Profile ID - 18101*/
const Profile P17095 = {"17095", NULL};	/*Profile ID - 17095*/
const Value V18222_1_1 = {S2, NULL};	/*Value = True*/
const Property P18222_1 = {S50, 0, &V18222_1_1, NULL};	/*Property = IsMobile*/
const Value V18222_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18222_5 = {S52, 0, &V18222_5_9981, &P18222_1};	/*Property = ScreenPixelsHeight*/
const Value V18222_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18222_4 = {S54, 0, &V18222_4_39098, &P18222_5};	/*Property = ScreenPixelsWidth*/
const Profile P18222 = {"18222", &P18222_4};	/*Profile ID - 18222*/
const Value V18328_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18328_279 = {S1, 0, &V18328_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18328_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18328_282 = {S3, 0, &V18328_282_57154, &P18328_279};	/*Property = BlobBuilder*/
const Value V18328_350_57445 = {S0, NULL};	/*Value = False*/
const Property P18328_350 = {S4, 0, &V18328_350_57445, &P18328_282};	/*Property = Canvas*/
const Value V18328_283_57160 = {S0, NULL};	/*Value = False*/
const Property P18328_283 = {S5, 0, &V18328_283_57160, &P18328_350};	/*Property = CssBackground*/
const Value V18328_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18328_284 = {S6, 0, &V18328_284_57163, &P18328_283};	/*Property = CssBorderImage*/
const Value V18328_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18328_285 = {S7, 0, &V18328_285_57166, &P18328_284};	/*Property = CssCanvas*/
const Value V18328_286_57169 = {S0, NULL};	/*Value = False*/
const Property P18328_286 = {S8, 0, &V18328_286_57169, &P18328_285};	/*Property = CssColor*/
const Value V18328_237_55785 = {S0, NULL};	/*Value = False*/
const Property P18328_237 = {S9, 0, &V18328_237_55785, &P18328_286};	/*Property = CssColumn*/
const Value V18328_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18328_287 = {S10, 0, &V18328_287_57172, &P18328_237};	/*Property = CssFlexbox*/
const Value V18328_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18328_288 = {S11, 0, &V18328_288_57175, &P18328_287};	/*Property = CssFont*/
const Value V18328_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_289 = {S12, 0, &V18328_289_86899, &P18328_288};	/*Property = CssImages*/
const Value V18328_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18328_291 = {S13, 0, &V18328_291_57181, &P18328_289};	/*Property = CssMediaQueries*/
const Value V18328_292_57184 = {S0, NULL};	/*Value = False*/
const Property P18328_292 = {S14, 0, &V18328_292_57184, &P18328_291};	/*Property = CssMinMax*/
const Value V18328_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18328_293 = {S15, 0, &V18328_293_57187, &P18328_292};	/*Property = CssOverflow*/
const Value V18328_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18328_294 = {S16, 0, &V18328_294_57190, &P18328_293};	/*Property = CssPosition*/
const Value V18328_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18328_295 = {S17, 0, &V18328_295_57193, &P18328_294};	/*Property = CssText*/
const Value V18328_297_57196 = {S0, NULL};	/*Value = False*/
const Property P18328_297 = {S18, 0, &V18328_297_57196, &P18328_295};	/*Property = CssTransforms*/
const Value V18328_241_55800 = {S0, NULL};	/*Value = False*/
const Property P18328_241 = {S19, 0, &V18328_241_55800, &P18328_297};	/*Property = CssTransitions*/
const Value V18328_298_57207 = {S0, NULL};	/*Value = False*/
const Property P18328_298 = {S20, 0, &V18328_298_57207, &P18328_241};	/*Property = CssUI*/
const Value V18328_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_300 = {S21, 0, &V18328_300_57217, &P18328_298};	/*Property = DataSet*/
const Value V18328_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_301 = {S22, 0, &V18328_301_57220, &P18328_300};	/*Property = DataUrl*/
const Value V18328_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_302 = {S23, 0, &V18328_302_57223, &P18328_301};	/*Property = DeviceOrientation*/
const Value V18328_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_304 = {S24, 0, &V18328_304_57229, &P18328_302};	/*Property = FileReader*/
const Value V18328_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_305 = {S25, 0, &V18328_305_57232, &P18328_304};	/*Property = FileSaver*/
const Value V18328_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_306 = {S26, 0, &V18328_306_57238, &P18328_305};	/*Property = FileWriter*/
const Value V18328_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_307 = {S27, 0, &V18328_307_57241, &P18328_306};	/*Property = FormData*/
const Value V18328_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_309 = {S28, 0, &V18328_309_57247, &P18328_307};	/*Property = Fullscreen*/
const Value V18328_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_310 = {S29, 0, &V18328_310_57250, &P18328_309};	/*Property = GeoLocation*/
const Value V18328_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_312 = {S30, 0, &V18328_312_57256, &P18328_310};	/*Property = History*/
const Value V18328_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_314 = {S31, 0, &V18328_314_57262, &P18328_312};	/*Property = Html5*/
const Value V18328_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_313 = {S32, 0, &V18328_313_57259, &P18328_314};	/*Property = Html-Media-Capture*/
const Value V18328_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_315 = {S33, 0, &V18328_315_57265, &P18328_313};	/*Property = Iframe*/
const Value V18328_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_316 = {S34, 0, &V18328_316_57268, &P18328_315};	/*Property = IndexedDB*/
const Value V18328_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_317 = {S35, 0, &V18328_317_57271, &P18328_316};	/*Property = Json*/
const Value V18328_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18328_15 = {S37, 0, &V18328_15_31, &P18328_317};	/*Property = LayoutEngine*/
const Value V18328_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_344 = {S38, 0, &V18328_344_57274, &P18328_15};	/*Property = Masking*/
const Value V18328_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_320 = {S39, 0, &V18328_320_57292, &P18328_344};	/*Property = PostMessage*/
const Value V18328_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_321 = {S40, 0, &V18328_321_57295, &P18328_320};	/*Property = Progress*/
const Value V18328_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_322 = {S41, 0, &V18328_322_57298, &P18328_321};	/*Property = Prompts*/
const Value V18328_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_323 = {S42, 0, &V18328_323_57301, &P18328_322};	/*Property = Selector*/
const Value V18328_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_324 = {S43, 0, &V18328_324_57307, &P18328_323};	/*Property = Svg*/
const Value V18328_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_327 = {S44, 0, &V18328_327_57316, &P18328_324};	/*Property = TouchEvents*/
const Value V18328_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_328 = {S45, 0, &V18328_328_57319, &P18328_327};	/*Property = Track*/
const Value V18328_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_329 = {S46, 0, &V18328_329_57325, &P18328_328};	/*Property = Video*/
const Value V18328_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_330 = {S47, 0, &V18328_330_57328, &P18328_329};	/*Property = Viewport*/
const Value V18328_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_332 = {S48, 0, &V18328_332_57340, &P18328_330};	/*Property = WebWorkers*/
const Value V18328_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18328_333 = {S49, 0, &V18328_333_57343, &P18328_332};	/*Property = Xhr2*/
const Profile P18328 = {"18328", &P18328_333};	/*Profile ID - 18328*/
const Value V19840_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_279 = {S1, 0, &V19840_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V19840_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_282 = {S3, 0, &V19840_282_57155, &P19840_279};	/*Property = BlobBuilder*/
const Value V19840_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_350 = {S4, 0, &V19840_350_57446, &P19840_282};	/*Property = Canvas*/
const Value V19840_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_283 = {S5, 0, &V19840_283_57161, &P19840_350};	/*Property = CssBackground*/
const Value V19840_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_284 = {S6, 0, &V19840_284_57164, &P19840_283};	/*Property = CssBorderImage*/
const Value V19840_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_285 = {S7, 0, &V19840_285_57167, &P19840_284};	/*Property = CssCanvas*/
const Value V19840_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_286 = {S8, 0, &V19840_286_57170, &P19840_285};	/*Property = CssColor*/
const Value V19840_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_237 = {S9, 0, &V19840_237_55786, &P19840_286};	/*Property = CssColumn*/
const Value V19840_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_287 = {S10, 0, &V19840_287_57173, &P19840_237};	/*Property = CssFlexbox*/
const Value V19840_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_288 = {S11, 0, &V19840_288_57176, &P19840_287};	/*Property = CssFont*/
const Value V19840_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_289 = {S12, 0, &V19840_289_86899, &P19840_288};	/*Property = CssImages*/
const Value V19840_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_291 = {S13, 0, &V19840_291_57182, &P19840_289};	/*Property = CssMediaQueries*/
const Value V19840_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_292 = {S14, 0, &V19840_292_57185, &P19840_291};	/*Property = CssMinMax*/
const Value V19840_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_293 = {S15, 0, &V19840_293_57188, &P19840_292};	/*Property = CssOverflow*/
const Value V19840_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_294 = {S16, 0, &V19840_294_57191, &P19840_293};	/*Property = CssPosition*/
const Value V19840_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_295 = {S17, 0, &V19840_295_57194, &P19840_294};	/*Property = CssText*/
const Value V19840_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_297 = {S18, 0, &V19840_297_57197, &P19840_295};	/*Property = CssTransforms*/
const Value V19840_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_241 = {S19, 0, &V19840_241_55801, &P19840_297};	/*Property = CssTransitions*/
const Value V19840_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_298 = {S20, 0, &V19840_298_57208, &P19840_241};	/*Property = CssUI*/
const Value V19840_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_300 = {S21, 0, &V19840_300_57217, &P19840_298};	/*Property = DataSet*/
const Value V19840_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_301 = {S22, 0, &V19840_301_57220, &P19840_300};	/*Property = DataUrl*/
const Value V19840_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_302 = {S23, 0, &V19840_302_57223, &P19840_301};	/*Property = DeviceOrientation*/
const Value V19840_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_304 = {S24, 0, &V19840_304_57229, &P19840_302};	/*Property = FileReader*/
const Value V19840_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_305 = {S25, 0, &V19840_305_57232, &P19840_304};	/*Property = FileSaver*/
const Value V19840_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_306 = {S26, 0, &V19840_306_57238, &P19840_305};	/*Property = FileWriter*/
const Value V19840_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_307 = {S27, 0, &V19840_307_57241, &P19840_306};	/*Property = FormData*/
const Value V19840_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_309 = {S28, 0, &V19840_309_57247, &P19840_307};	/*Property = Fullscreen*/
const Value V19840_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_310 = {S29, 0, &V19840_310_57250, &P19840_309};	/*Property = GeoLocation*/
const Value V19840_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_312 = {S30, 0, &V19840_312_57256, &P19840_310};	/*Property = History*/
const Value V19840_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_314 = {S31, 0, &V19840_314_57262, &P19840_312};	/*Property = Html5*/
const Value V19840_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_313 = {S32, 0, &V19840_313_57259, &P19840_314};	/*Property = Html-Media-Capture*/
const Value V19840_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_315 = {S33, 0, &V19840_315_57265, &P19840_313};	/*Property = Iframe*/
const Value V19840_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_316 = {S34, 0, &V19840_316_57268, &P19840_315};	/*Property = IndexedDB*/
const Value V19840_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_317 = {S35, 0, &V19840_317_57271, &P19840_316};	/*Property = Json*/
const Value V19840_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19840_15 = {S37, 0, &V19840_15_28, &P19840_317};	/*Property = LayoutEngine*/
const Value V19840_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_344 = {S38, 0, &V19840_344_57274, &P19840_15};	/*Property = Masking*/
const Value V19840_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_320 = {S39, 0, &V19840_320_57292, &P19840_344};	/*Property = PostMessage*/
const Value V19840_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_321 = {S40, 0, &V19840_321_57295, &P19840_320};	/*Property = Progress*/
const Value V19840_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_322 = {S41, 0, &V19840_322_57298, &P19840_321};	/*Property = Prompts*/
const Value V19840_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_323 = {S42, 0, &V19840_323_57301, &P19840_322};	/*Property = Selector*/
const Value V19840_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_324 = {S43, 0, &V19840_324_57307, &P19840_323};	/*Property = Svg*/
const Value V19840_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_327 = {S44, 0, &V19840_327_57316, &P19840_324};	/*Property = TouchEvents*/
const Value V19840_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_328 = {S45, 0, &V19840_328_57319, &P19840_327};	/*Property = Track*/
const Value V19840_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_329 = {S46, 0, &V19840_329_57325, &P19840_328};	/*Property = Video*/
const Value V19840_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_330 = {S47, 0, &V19840_330_57328, &P19840_329};	/*Property = Viewport*/
const Value V19840_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_332 = {S48, 0, &V19840_332_57340, &P19840_330};	/*Property = WebWorkers*/
const Value V19840_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P19840_333 = {S49, 0, &V19840_333_57343, &P19840_332};	/*Property = Xhr2*/
const Profile P19840 = {"19840", &P19840_333};	/*Profile ID - 19840*/
const Value V24449_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_279 = {S1, 0, &V24449_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24449_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_282 = {S3, 0, &V24449_282_57155, &P24449_279};	/*Property = BlobBuilder*/
const Value V24449_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_350 = {S4, 0, &V24449_350_57446, &P24449_282};	/*Property = Canvas*/
const Value V24449_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_283 = {S5, 0, &V24449_283_57161, &P24449_350};	/*Property = CssBackground*/
const Value V24449_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_284 = {S6, 0, &V24449_284_57164, &P24449_283};	/*Property = CssBorderImage*/
const Value V24449_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_285 = {S7, 0, &V24449_285_57167, &P24449_284};	/*Property = CssCanvas*/
const Value V24449_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_286 = {S8, 0, &V24449_286_57170, &P24449_285};	/*Property = CssColor*/
const Value V24449_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_237 = {S9, 0, &V24449_237_55786, &P24449_286};	/*Property = CssColumn*/
const Value V24449_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_287 = {S10, 0, &V24449_287_57173, &P24449_237};	/*Property = CssFlexbox*/
const Value V24449_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_288 = {S11, 0, &V24449_288_57176, &P24449_287};	/*Property = CssFont*/
const Value V24449_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_289 = {S12, 0, &V24449_289_86899, &P24449_288};	/*Property = CssImages*/
const Value V24449_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_291 = {S13, 0, &V24449_291_57182, &P24449_289};	/*Property = CssMediaQueries*/
const Value V24449_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_292 = {S14, 0, &V24449_292_57185, &P24449_291};	/*Property = CssMinMax*/
const Value V24449_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_293 = {S15, 0, &V24449_293_57188, &P24449_292};	/*Property = CssOverflow*/
const Value V24449_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_294 = {S16, 0, &V24449_294_57191, &P24449_293};	/*Property = CssPosition*/
const Value V24449_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_295 = {S17, 0, &V24449_295_57194, &P24449_294};	/*Property = CssText*/
const Value V24449_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_297 = {S18, 0, &V24449_297_57197, &P24449_295};	/*Property = CssTransforms*/
const Value V24449_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_241 = {S19, 0, &V24449_241_55801, &P24449_297};	/*Property = CssTransitions*/
const Value V24449_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_298 = {S20, 0, &V24449_298_57208, &P24449_241};	/*Property = CssUI*/
const Value V24449_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_300 = {S21, 0, &V24449_300_57217, &P24449_298};	/*Property = DataSet*/
const Value V24449_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_301 = {S22, 0, &V24449_301_57220, &P24449_300};	/*Property = DataUrl*/
const Value V24449_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_302 = {S23, 0, &V24449_302_57223, &P24449_301};	/*Property = DeviceOrientation*/
const Value V24449_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_304 = {S24, 0, &V24449_304_57229, &P24449_302};	/*Property = FileReader*/
const Value V24449_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_305 = {S25, 0, &V24449_305_57232, &P24449_304};	/*Property = FileSaver*/
const Value V24449_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_306 = {S26, 0, &V24449_306_57238, &P24449_305};	/*Property = FileWriter*/
const Value V24449_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_307 = {S27, 0, &V24449_307_57241, &P24449_306};	/*Property = FormData*/
const Value V24449_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_309 = {S28, 0, &V24449_309_57247, &P24449_307};	/*Property = Fullscreen*/
const Value V24449_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_310 = {S29, 0, &V24449_310_57250, &P24449_309};	/*Property = GeoLocation*/
const Value V24449_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_312 = {S30, 0, &V24449_312_57256, &P24449_310};	/*Property = History*/
const Value V24449_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_314 = {S31, 0, &V24449_314_57262, &P24449_312};	/*Property = Html5*/
const Value V24449_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_313 = {S32, 0, &V24449_313_57259, &P24449_314};	/*Property = Html-Media-Capture*/
const Value V24449_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_315 = {S33, 0, &V24449_315_57265, &P24449_313};	/*Property = Iframe*/
const Value V24449_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_316 = {S34, 0, &V24449_316_57268, &P24449_315};	/*Property = IndexedDB*/
const Value V24449_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_317 = {S35, 0, &V24449_317_57271, &P24449_316};	/*Property = Json*/
const Value V24449_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24449_15 = {S37, 0, &V24449_15_28, &P24449_317};	/*Property = LayoutEngine*/
const Value V24449_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_344 = {S38, 0, &V24449_344_57274, &P24449_15};	/*Property = Masking*/
const Value V24449_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_320 = {S39, 0, &V24449_320_57292, &P24449_344};	/*Property = PostMessage*/
const Value V24449_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_321 = {S40, 0, &V24449_321_57295, &P24449_320};	/*Property = Progress*/
const Value V24449_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_322 = {S41, 0, &V24449_322_57298, &P24449_321};	/*Property = Prompts*/
const Value V24449_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_323 = {S42, 0, &V24449_323_57301, &P24449_322};	/*Property = Selector*/
const Value V24449_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_324 = {S43, 0, &V24449_324_57307, &P24449_323};	/*Property = Svg*/
const Value V24449_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_327 = {S44, 0, &V24449_327_57316, &P24449_324};	/*Property = TouchEvents*/
const Value V24449_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_328 = {S45, 0, &V24449_328_57319, &P24449_327};	/*Property = Track*/
const Value V24449_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_329 = {S46, 0, &V24449_329_57325, &P24449_328};	/*Property = Video*/
const Value V24449_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_330 = {S47, 0, &V24449_330_57328, &P24449_329};	/*Property = Viewport*/
const Value V24449_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_332 = {S48, 0, &V24449_332_57340, &P24449_330};	/*Property = WebWorkers*/
const Value V24449_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24449_333 = {S49, 0, &V24449_333_57343, &P24449_332};	/*Property = Xhr2*/
const Profile P24449 = {"24449", &P24449_333};	/*Profile ID - 24449*/
const Value V20434_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20434_279 = {S1, 0, &V20434_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20434_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20434_282 = {S3, 0, &V20434_282_57154, &P20434_279};	/*Property = BlobBuilder*/
const Value V20434_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20434_350 = {S4, 0, &V20434_350_57444, &P20434_282};	/*Property = Canvas*/
const Value V20434_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20434_283 = {S5, 0, &V20434_283_57159, &P20434_350};	/*Property = CssBackground*/
const Value V20434_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20434_284 = {S6, 0, &V20434_284_57163, &P20434_283};	/*Property = CssBorderImage*/
const Value V20434_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20434_285 = {S7, 0, &V20434_285_57166, &P20434_284};	/*Property = CssCanvas*/
const Value V20434_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20434_286 = {S8, 0, &V20434_286_57168, &P20434_285};	/*Property = CssColor*/
const Value V20434_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20434_237 = {S9, 0, &V20434_237_55786, &P20434_286};	/*Property = CssColumn*/
const Value V20434_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20434_287 = {S10, 0, &V20434_287_57172, &P20434_237};	/*Property = CssFlexbox*/
const Value V20434_288_57175 = {S0, NULL};	/*Value = False*/
const Property P20434_288 = {S11, 0, &V20434_288_57175, &P20434_287};	/*Property = CssFont*/
const Value V20434_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20434_289 = {S12, 0, &V20434_289_57759, &P20434_288};	/*Property = CssImages*/
const Value V20434_291_57181 = {S0, NULL};	/*Value = False*/
const Property P20434_291 = {S13, 0, &V20434_291_57181, &P20434_289};	/*Property = CssMediaQueries*/
const Value V20434_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20434_292 = {S14, 0, &V20434_292_57183, &P20434_291};	/*Property = CssMinMax*/
const Value V20434_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20434_293 = {S15, 0, &V20434_293_57187, &P20434_292};	/*Property = CssOverflow*/
const Value V20434_294_57190 = {S0, NULL};	/*Value = False*/
const Property P20434_294 = {S16, 0, &V20434_294_57190, &P20434_293};	/*Property = CssPosition*/
const Value V20434_295_57193 = {S0, NULL};	/*Value = False*/
const Property P20434_295 = {S17, 0, &V20434_295_57193, &P20434_294};	/*Property = CssText*/
const Value V20434_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20434_297 = {S18, 0, &V20434_297_57195, &P20434_295};	/*Property = CssTransforms*/
const Value V20434_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20434_241 = {S19, 0, &V20434_241_55799, &P20434_297};	/*Property = CssTransitions*/
const Value V20434_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20434_298 = {S20, 0, &V20434_298_57206, &P20434_241};	/*Property = CssUI*/
const Value V20434_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20434_300 = {S21, 0, &V20434_300_57216, &P20434_298};	/*Property = DataSet*/
const Value V20434_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20434_301 = {S22, 0, &V20434_301_57218, &P20434_300};	/*Property = DataUrl*/
const Value V20434_302_57222 = {S0, NULL};	/*Value = False*/
const Property P20434_302 = {S23, 0, &V20434_302_57222, &P20434_301};	/*Property = DeviceOrientation*/
const Value V20434_304_57228 = {S0, NULL};	/*Value = False*/
const Property P20434_304 = {S24, 0, &V20434_304_57228, &P20434_302};	/*Property = FileReader*/
const Value V20434_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20434_305 = {S25, 0, &V20434_305_57231, &P20434_304};	/*Property = FileSaver*/
const Value V20434_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20434_306 = {S26, 0, &V20434_306_57237, &P20434_305};	/*Property = FileWriter*/
const Value V20434_307_57240 = {S0, NULL};	/*Value = False*/
const Property P20434_307 = {S27, 0, &V20434_307_57240, &P20434_306};	/*Property = FormData*/
const Value V20434_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20434_309 = {S28, 0, &V20434_309_57246, &P20434_307};	/*Property = Fullscreen*/
const Value V20434_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20434_310 = {S29, 0, &V20434_310_57248, &P20434_309};	/*Property = GeoLocation*/
const Value V20434_312_57255 = {S0, NULL};	/*Value = False*/
const Property P20434_312 = {S30, 0, &V20434_312_57255, &P20434_310};	/*Property = History*/
const Value V20434_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20434_314 = {S31, 0, &V20434_314_57261, &P20434_312};	/*Property = Html5*/
const Value V20434_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20434_313 = {S32, 0, &V20434_313_57258, &P20434_314};	/*Property = Html-Media-Capture*/
const Value V20434_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20434_315 = {S33, 0, &V20434_315_57264, &P20434_313};	/*Property = Iframe*/
const Value V20434_316_57267 = {S0, NULL};	/*Value = False*/
const Property P20434_316 = {S34, 0, &V20434_316_57267, &P20434_315};	/*Property = IndexedDB*/
const Value V20434_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20434_317 = {S35, 0, &V20434_317_57269, &P20434_316};	/*Property = Json*/
const Value V20434_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20434_15 = {S37, 0, &V20434_15_29, &P20434_317};	/*Property = LayoutEngine*/
const Value V20434_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20434_344 = {S38, 0, &V20434_344_57272, &P20434_15};	/*Property = Masking*/
const Value V20434_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20434_320 = {S39, 0, &V20434_320_57290, &P20434_344};	/*Property = PostMessage*/
const Value V20434_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20434_321 = {S40, 0, &V20434_321_57293, &P20434_320};	/*Property = Progress*/
const Value V20434_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20434_322 = {S41, 0, &V20434_322_57296, &P20434_321};	/*Property = Prompts*/
const Value V20434_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20434_323 = {S42, 0, &V20434_323_57299, &P20434_322};	/*Property = Selector*/
const Value V20434_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20434_324 = {S43, 0, &V20434_324_57306, &P20434_323};	/*Property = Svg*/
const Value V20434_327_57315 = {S0, NULL};	/*Value = False*/
const Property P20434_327 = {S44, 0, &V20434_327_57315, &P20434_324};	/*Property = TouchEvents*/
const Value V20434_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20434_328 = {S45, 0, &V20434_328_57318, &P20434_327};	/*Property = Track*/
const Value V20434_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20434_329 = {S46, 0, &V20434_329_57323, &P20434_328};	/*Property = Video*/
const Value V20434_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20434_330 = {S47, 0, &V20434_330_57326, &P20434_329};	/*Property = Viewport*/
const Value V20434_332_57339 = {S0, NULL};	/*Value = False*/
const Property P20434_332 = {S48, 0, &V20434_332_57339, &P20434_330};	/*Property = WebWorkers*/
const Value V20434_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20434_333 = {S49, 0, &V20434_333_57342, &P20434_332};	/*Property = Xhr2*/
const Profile P20434 = {"20434", &P20434_333};	/*Profile ID - 20434*/
const Value V17905_1_1 = {S2, NULL};	/*Value = True*/
const Property P17905_1 = {S50, 0, &V17905_1_1, NULL};	/*Property = IsMobile*/
const Value V17905_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P17905_5 = {S52, 0, &V17905_5_9981, &P17905_1};	/*Property = ScreenPixelsHeight*/
const Value V17905_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P17905_4 = {S54, 0, &V17905_4_39098, &P17905_5};	/*Property = ScreenPixelsWidth*/
const Profile P17905 = {"17905", &P17905_4};	/*Profile ID - 17905*/
const Value V22534_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22534_279 = {S1, 0, &V22534_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22534_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22534_282 = {S3, 0, &V22534_282_57153, &P22534_279};	/*Property = BlobBuilder*/
const Value V22534_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22534_350 = {S4, 0, &V22534_350_57444, &P22534_282};	/*Property = Canvas*/
const Value V22534_283_57160 = {S0, NULL};	/*Value = False*/
const Property P22534_283 = {S5, 0, &V22534_283_57160, &P22534_350};	/*Property = CssBackground*/
const Value V22534_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22534_284 = {S6, 0, &V22534_284_57163, &P22534_283};	/*Property = CssBorderImage*/
const Value V22534_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22534_285 = {S7, 0, &V22534_285_57166, &P22534_284};	/*Property = CssCanvas*/
const Value V22534_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22534_286 = {S8, 0, &V22534_286_57168, &P22534_285};	/*Property = CssColor*/
const Value V22534_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22534_237 = {S9, 0, &V22534_237_55784, &P22534_286};	/*Property = CssColumn*/
const Value V22534_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22534_287 = {S10, 0, &V22534_287_57172, &P22534_237};	/*Property = CssFlexbox*/
const Value V22534_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22534_288 = {S11, 0, &V22534_288_57175, &P22534_287};	/*Property = CssFont*/
const Value V22534_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22534_289 = {S12, 0, &V22534_289_86899, &P22534_288};	/*Property = CssImages*/
const Value V22534_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22534_291 = {S13, 0, &V22534_291_57181, &P22534_289};	/*Property = CssMediaQueries*/
const Value V22534_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22534_292 = {S14, 0, &V22534_292_57183, &P22534_291};	/*Property = CssMinMax*/
const Value V22534_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22534_293 = {S15, 0, &V22534_293_57187, &P22534_292};	/*Property = CssOverflow*/
const Value V22534_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22534_294 = {S16, 0, &V22534_294_57189, &P22534_293};	/*Property = CssPosition*/
const Value V22534_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22534_295 = {S17, 0, &V22534_295_57193, &P22534_294};	/*Property = CssText*/
const Value V22534_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22534_297 = {S18, 0, &V22534_297_57195, &P22534_295};	/*Property = CssTransforms*/
const Value V22534_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22534_241 = {S19, 0, &V22534_241_55799, &P22534_297};	/*Property = CssTransitions*/
const Value V22534_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22534_298 = {S20, 0, &V22534_298_57206, &P22534_241};	/*Property = CssUI*/
const Value V22534_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22534_300 = {S21, 0, &V22534_300_57215, &P22534_298};	/*Property = DataSet*/
const Value V22534_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22534_301 = {S22, 0, &V22534_301_57218, &P22534_300};	/*Property = DataUrl*/
const Value V22534_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22534_302 = {S23, 0, &V22534_302_57221, &P22534_301};	/*Property = DeviceOrientation*/
const Value V22534_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22534_304 = {S24, 0, &V22534_304_57227, &P22534_302};	/*Property = FileReader*/
const Value V22534_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22534_305 = {S25, 0, &V22534_305_57231, &P22534_304};	/*Property = FileSaver*/
const Value V22534_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22534_306 = {S26, 0, &V22534_306_57237, &P22534_305};	/*Property = FileWriter*/
const Value V22534_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22534_307 = {S27, 0, &V22534_307_57239, &P22534_306};	/*Property = FormData*/
const Value V22534_309_57245 = {S2, NULL};	/*Value = True*/
const Property P22534_309 = {S28, 0, &V22534_309_57245, &P22534_307};	/*Property = Fullscreen*/
const Value V22534_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22534_310 = {S29, 0, &V22534_310_57248, &P22534_309};	/*Property = GeoLocation*/
const Value V22534_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22534_312 = {S30, 0, &V22534_312_57254, &P22534_310};	/*Property = History*/
const Value V22534_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22534_314 = {S31, 0, &V22534_314_57261, &P22534_312};	/*Property = Html5*/
const Value V22534_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22534_313 = {S32, 0, &V22534_313_57258, &P22534_314};	/*Property = Html-Media-Capture*/
const Value V22534_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22534_315 = {S33, 0, &V22534_315_57264, &P22534_313};	/*Property = Iframe*/
const Value V22534_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22534_316 = {S34, 0, &V22534_316_57266, &P22534_315};	/*Property = IndexedDB*/
const Value V22534_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22534_317 = {S35, 0, &V22534_317_57269, &P22534_316};	/*Property = Json*/
const Value V22534_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22534_15 = {S37, 0, &V22534_15_29, &P22534_317};	/*Property = LayoutEngine*/
const Value V22534_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22534_344 = {S38, 0, &V22534_344_57272, &P22534_15};	/*Property = Masking*/
const Value V22534_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22534_320 = {S39, 0, &V22534_320_57290, &P22534_344};	/*Property = PostMessage*/
const Value V22534_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22534_321 = {S40, 0, &V22534_321_57293, &P22534_320};	/*Property = Progress*/
const Value V22534_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22534_322 = {S41, 0, &V22534_322_57296, &P22534_321};	/*Property = Prompts*/
const Value V22534_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22534_323 = {S42, 0, &V22534_323_57299, &P22534_322};	/*Property = Selector*/
const Value V22534_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22534_324 = {S43, 0, &V22534_324_57305, &P22534_323};	/*Property = Svg*/
const Value V22534_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22534_327 = {S44, 0, &V22534_327_57315, &P22534_324};	/*Property = TouchEvents*/
const Value V22534_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22534_328 = {S45, 0, &V22534_328_57318, &P22534_327};	/*Property = Track*/
const Value V22534_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22534_329 = {S46, 0, &V22534_329_57323, &P22534_328};	/*Property = Video*/
const Value V22534_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22534_330 = {S47, 0, &V22534_330_57326, &P22534_329};	/*Property = Viewport*/
const Value V22534_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22534_332 = {S48, 0, &V22534_332_57339, &P22534_330};	/*Property = WebWorkers*/
const Value V22534_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22534_333 = {S49, 0, &V22534_333_57342, &P22534_332};	/*Property = Xhr2*/
const Profile P22534 = {"22534", &P22534_333};	/*Profile ID - 22534*/
const Value V21559_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_279 = {S1, 0, &V21559_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21559_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_282 = {S3, 0, &V21559_282_57155, &P21559_279};	/*Property = BlobBuilder*/
const Value V21559_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_350 = {S4, 0, &V21559_350_57446, &P21559_282};	/*Property = Canvas*/
const Value V21559_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_283 = {S5, 0, &V21559_283_57161, &P21559_350};	/*Property = CssBackground*/
const Value V21559_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_284 = {S6, 0, &V21559_284_57164, &P21559_283};	/*Property = CssBorderImage*/
const Value V21559_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_285 = {S7, 0, &V21559_285_57167, &P21559_284};	/*Property = CssCanvas*/
const Value V21559_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_286 = {S8, 0, &V21559_286_57170, &P21559_285};	/*Property = CssColor*/
const Value V21559_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_237 = {S9, 0, &V21559_237_55786, &P21559_286};	/*Property = CssColumn*/
const Value V21559_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_287 = {S10, 0, &V21559_287_57173, &P21559_237};	/*Property = CssFlexbox*/
const Value V21559_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_288 = {S11, 0, &V21559_288_57176, &P21559_287};	/*Property = CssFont*/
const Value V21559_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_289 = {S12, 0, &V21559_289_86899, &P21559_288};	/*Property = CssImages*/
const Value V21559_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_291 = {S13, 0, &V21559_291_57182, &P21559_289};	/*Property = CssMediaQueries*/
const Value V21559_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_292 = {S14, 0, &V21559_292_57185, &P21559_291};	/*Property = CssMinMax*/
const Value V21559_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_293 = {S15, 0, &V21559_293_57188, &P21559_292};	/*Property = CssOverflow*/
const Value V21559_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_294 = {S16, 0, &V21559_294_57191, &P21559_293};	/*Property = CssPosition*/
const Value V21559_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_295 = {S17, 0, &V21559_295_57194, &P21559_294};	/*Property = CssText*/
const Value V21559_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_297 = {S18, 0, &V21559_297_57197, &P21559_295};	/*Property = CssTransforms*/
const Value V21559_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_241 = {S19, 0, &V21559_241_55801, &P21559_297};	/*Property = CssTransitions*/
const Value V21559_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_298 = {S20, 0, &V21559_298_57208, &P21559_241};	/*Property = CssUI*/
const Value V21559_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_300 = {S21, 0, &V21559_300_57217, &P21559_298};	/*Property = DataSet*/
const Value V21559_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_301 = {S22, 0, &V21559_301_57220, &P21559_300};	/*Property = DataUrl*/
const Value V21559_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_302 = {S23, 0, &V21559_302_57223, &P21559_301};	/*Property = DeviceOrientation*/
const Value V21559_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_304 = {S24, 0, &V21559_304_57229, &P21559_302};	/*Property = FileReader*/
const Value V21559_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_305 = {S25, 0, &V21559_305_57232, &P21559_304};	/*Property = FileSaver*/
const Value V21559_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_306 = {S26, 0, &V21559_306_57238, &P21559_305};	/*Property = FileWriter*/
const Value V21559_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_307 = {S27, 0, &V21559_307_57241, &P21559_306};	/*Property = FormData*/
const Value V21559_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_309 = {S28, 0, &V21559_309_57247, &P21559_307};	/*Property = Fullscreen*/
const Value V21559_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_310 = {S29, 0, &V21559_310_57250, &P21559_309};	/*Property = GeoLocation*/
const Value V21559_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_312 = {S30, 0, &V21559_312_57256, &P21559_310};	/*Property = History*/
const Value V21559_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_314 = {S31, 0, &V21559_314_57262, &P21559_312};	/*Property = Html5*/
const Value V21559_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_313 = {S32, 0, &V21559_313_57259, &P21559_314};	/*Property = Html-Media-Capture*/
const Value V21559_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_315 = {S33, 0, &V21559_315_57265, &P21559_313};	/*Property = Iframe*/
const Value V21559_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_316 = {S34, 0, &V21559_316_57268, &P21559_315};	/*Property = IndexedDB*/
const Value V21559_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_317 = {S35, 0, &V21559_317_57271, &P21559_316};	/*Property = Json*/
const Value V21559_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21559_15 = {S37, 0, &V21559_15_28, &P21559_317};	/*Property = LayoutEngine*/
const Value V21559_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_344 = {S38, 0, &V21559_344_57274, &P21559_15};	/*Property = Masking*/
const Value V21559_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_320 = {S39, 0, &V21559_320_57292, &P21559_344};	/*Property = PostMessage*/
const Value V21559_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_321 = {S40, 0, &V21559_321_57295, &P21559_320};	/*Property = Progress*/
const Value V21559_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_322 = {S41, 0, &V21559_322_57298, &P21559_321};	/*Property = Prompts*/
const Value V21559_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_323 = {S42, 0, &V21559_323_57301, &P21559_322};	/*Property = Selector*/
const Value V21559_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_324 = {S43, 0, &V21559_324_57307, &P21559_323};	/*Property = Svg*/
const Value V21559_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_327 = {S44, 0, &V21559_327_57316, &P21559_324};	/*Property = TouchEvents*/
const Value V21559_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_328 = {S45, 0, &V21559_328_57319, &P21559_327};	/*Property = Track*/
const Value V21559_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_329 = {S46, 0, &V21559_329_57325, &P21559_328};	/*Property = Video*/
const Value V21559_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_330 = {S47, 0, &V21559_330_57328, &P21559_329};	/*Property = Viewport*/
const Value V21559_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_332 = {S48, 0, &V21559_332_57340, &P21559_330};	/*Property = WebWorkers*/
const Value V21559_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21559_333 = {S49, 0, &V21559_333_57343, &P21559_332};	/*Property = Xhr2*/
const Profile P21559 = {"21559", &P21559_333};	/*Profile ID - 21559*/
const Value V15876_1_1 = {S2, NULL};	/*Value = True*/
const Property P15876_1 = {S50, 0, &V15876_1_1, NULL};	/*Property = IsMobile*/
const Value V15876_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P15876_5 = {S52, 0, &V15876_5_9981, &P15876_1};	/*Property = ScreenPixelsHeight*/
const Value V15876_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P15876_4 = {S54, 0, &V15876_4_2336, &P15876_5};	/*Property = ScreenPixelsWidth*/
const Profile P15876 = {"15876", &P15876_4};	/*Profile ID - 15876*/
const Value V15882_1_1 = {S2, NULL};	/*Value = True*/
const Property P15882_1 = {S50, 0, &V15882_1_1, NULL};	/*Property = IsMobile*/
const Value V15882_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P15882_5 = {S52, 0, &V15882_5_9981, &P15882_1};	/*Property = ScreenPixelsHeight*/
const Value V15882_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P15882_4 = {S54, 0, &V15882_4_2336, &P15882_5};	/*Property = ScreenPixelsWidth*/
const Profile P15882 = {"15882", &P15882_4};	/*Profile ID - 15882*/
const Value V18320_1_1 = {S2, NULL};	/*Value = True*/
const Property P18320_1 = {S50, 0, &V18320_1_1, NULL};	/*Property = IsMobile*/
const Value V18320_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18320_5 = {S52, 0, &V18320_5_1403, &P18320_1};	/*Property = ScreenPixelsHeight*/
const Value V18320_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18320_4 = {S54, 0, &V18320_4_1404, &P18320_5};	/*Property = ScreenPixelsWidth*/
const Profile P18320 = {"18320", &P18320_4};	/*Profile ID - 18320*/
const Value V15919_1_1 = {S2, NULL};	/*Value = True*/
const Property P15919_1 = {S50, 0, &V15919_1_1, NULL};	/*Property = IsMobile*/
const Value V15919_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P15919_5 = {S52, 0, &V15919_5_1458, &P15919_1};	/*Property = ScreenPixelsHeight*/
const Value V15919_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P15919_4 = {S54, 0, &V15919_4_1460, &P15919_5};	/*Property = ScreenPixelsWidth*/
const Profile P15919 = {"15919", &P15919_4};	/*Profile ID - 15919*/
const Value V15932_1_1 = {S2, NULL};	/*Value = True*/
const Property P15932_1 = {S50, 0, &V15932_1_1, NULL};	/*Property = IsMobile*/
const Value V15932_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P15932_5 = {S52, 0, &V15932_5_1458, &P15932_1};	/*Property = ScreenPixelsHeight*/
const Value V15932_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P15932_4 = {S54, 0, &V15932_4_1460, &P15932_5};	/*Property = ScreenPixelsWidth*/
const Profile P15932 = {"15932", &P15932_4};	/*Profile ID - 15932*/
const Value V18317_1_1 = {S2, NULL};	/*Value = True*/
const Property P18317_1 = {S50, 0, &V18317_1_1, NULL};	/*Property = IsMobile*/
const Value V18317_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18317_5 = {S52, 0, &V18317_5_1227, &P18317_1};	/*Property = ScreenPixelsHeight*/
const Value V18317_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P18317_4 = {S54, 0, &V18317_4_1477, &P18317_5};	/*Property = ScreenPixelsWidth*/
const Profile P18317 = {"18317", &P18317_4};	/*Profile ID - 18317*/
const Value V18318_1_1 = {S2, NULL};	/*Value = True*/
const Property P18318_1 = {S50, 0, &V18318_1_1, NULL};	/*Property = IsMobile*/
const Value V18318_5_37526 = {S122, NULL};	/*Value = 1200*/
const Property P18318_5 = {S52, 0, &V18318_5_37526, &P18318_1};	/*Property = ScreenPixelsHeight*/
const Value V18318_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P18318_4 = {S54, 0, &V18318_4_57095, &P18318_5};	/*Property = ScreenPixelsWidth*/
const Profile P18318 = {"18318", &P18318_4};	/*Profile ID - 18318*/
const Value V18309_1_1 = {S2, NULL};	/*Value = True*/
const Property P18309_1 = {S50, 0, &V18309_1_1, NULL};	/*Property = IsMobile*/
const Value V18309_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18309_5 = {S52, 0, &V18309_5_1227, &P18309_1};	/*Property = ScreenPixelsHeight*/
const Value V18309_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18309_4 = {S54, 0, &V18309_4_1229, &P18309_5};	/*Property = ScreenPixelsWidth*/
const Profile P18309 = {"18309", &P18309_4};	/*Profile ID - 18309*/
const Value V18035_1_1 = {S2, NULL};	/*Value = True*/
const Property P18035_1 = {S50, 0, &V18035_1_1, NULL};	/*Property = IsMobile*/
const Value V18035_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18035_5 = {S52, 0, &V18035_5_1277, &P18035_1};	/*Property = ScreenPixelsHeight*/
const Value V18035_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18035_4 = {S54, 0, &V18035_4_2336, &P18035_5};	/*Property = ScreenPixelsWidth*/
const Profile P18035 = {"18035", &P18035_4};	/*Profile ID - 18035*/
const Value V20978_1_1 = {S2, NULL};	/*Value = True*/
const Property P20978_1 = {S50, 0, &V20978_1_1, NULL};	/*Property = IsMobile*/
const Value V20978_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20978_5 = {S52, 0, &V20978_5_1227, &P20978_1};	/*Property = ScreenPixelsHeight*/
const Value V20978_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20978_4 = {S54, 0, &V20978_4_1229, &P20978_5};	/*Property = ScreenPixelsWidth*/
const Profile P20978 = {"20978", &P20978_4};	/*Profile ID - 20978*/
const Value V17996_1_1 = {S2, NULL};	/*Value = True*/
const Property P17996_1 = {S50, 0, &V17996_1_1, NULL};	/*Property = IsMobile*/
const Value V17996_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P17996_5 = {S52, 0, &V17996_5_37569, &P17996_1};	/*Property = ScreenPixelsHeight*/
const Value V17996_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P17996_4 = {S54, 0, &V17996_4_5645, &P17996_5};	/*Property = ScreenPixelsWidth*/
const Profile P17996 = {"17996", &P17996_4};	/*Profile ID - 17996*/
const Value V21077_1_1 = {S2, NULL};	/*Value = True*/
const Property P21077_1 = {S50, 0, &V21077_1_1, NULL};	/*Property = IsMobile*/
const Value V21077_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21077_5 = {S52, 0, &V21077_5_1227, &P21077_1};	/*Property = ScreenPixelsHeight*/
const Value V21077_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21077_4 = {S54, 0, &V21077_4_1460, &P21077_5};	/*Property = ScreenPixelsWidth*/
const Profile P21077 = {"21077", &P21077_4};	/*Profile ID - 21077*/
const Value V21208_1_1 = {S2, NULL};	/*Value = True*/
const Property P21208_1 = {S50, 0, &V21208_1_1, NULL};	/*Property = IsMobile*/
const Value V21208_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21208_5 = {S52, 0, &V21208_5_1227, &P21208_1};	/*Property = ScreenPixelsHeight*/
const Value V21208_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21208_4 = {S54, 0, &V21208_4_1460, &P21208_5};	/*Property = ScreenPixelsWidth*/
const Profile P21208 = {"21208", &P21208_4};	/*Profile ID - 21208*/
const Value V20979_1_1 = {S2, NULL};	/*Value = True*/
const Property P20979_1 = {S50, 0, &V20979_1_1, NULL};	/*Property = IsMobile*/
const Value V20979_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20979_5 = {S52, 0, &V20979_5_1227, &P20979_1};	/*Property = ScreenPixelsHeight*/
const Value V20979_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20979_4 = {S54, 0, &V20979_4_1460, &P20979_5};	/*Property = ScreenPixelsWidth*/
const Profile P20979 = {"20979", &P20979_4};	/*Profile ID - 20979*/
const Value V21482_1_1 = {S2, NULL};	/*Value = True*/
const Property P21482_1 = {S50, 0, &V21482_1_1, NULL};	/*Property = IsMobile*/
const Value V21482_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21482_5 = {S52, 0, &V21482_5_1458, &P21482_1};	/*Property = ScreenPixelsHeight*/
const Value V21482_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21482_4 = {S54, 0, &V21482_4_1460, &P21482_5};	/*Property = ScreenPixelsWidth*/
const Profile P21482 = {"21482", &P21482_4};	/*Profile ID - 21482*/
const Value V21609_1_1 = {S2, NULL};	/*Value = True*/
const Property P21609_1 = {S50, 0, &V21609_1_1, NULL};	/*Property = IsMobile*/
const Value V21609_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21609_5 = {S52, 0, &V21609_5_37, &P21609_1};	/*Property = ScreenPixelsHeight*/
const Value V21609_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21609_4 = {S54, 0, &V21609_4_36, &P21609_5};	/*Property = ScreenPixelsWidth*/
const Profile P21609 = {"21609", &P21609_4};	/*Profile ID - 21609*/
const Value V22547_1_1 = {S2, NULL};	/*Value = True*/
const Property P22547_1 = {S50, 0, &V22547_1_1, NULL};	/*Property = IsMobile*/
const Value V22547_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22547_5 = {S52, 0, &V22547_5_1277, &P22547_1};	/*Property = ScreenPixelsHeight*/
const Value V22547_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22547_4 = {S54, 0, &V22547_4_2336, &P22547_5};	/*Property = ScreenPixelsWidth*/
const Profile P22547 = {"22547", &P22547_4};	/*Profile ID - 22547*/
const Value V18027_1_1 = {S2, NULL};	/*Value = True*/
const Property P18027_1 = {S50, 0, &V18027_1_1, NULL};	/*Property = IsMobile*/
const Value V18027_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18027_5 = {S52, 0, &V18027_5_1277, &P18027_1};	/*Property = ScreenPixelsHeight*/
const Value V18027_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18027_4 = {S54, 0, &V18027_4_2336, &P18027_5};	/*Property = ScreenPixelsWidth*/
const Profile P18027 = {"18027", &P18027_4};	/*Profile ID - 18027*/
const Value V18014_1_1 = {S2, NULL};	/*Value = True*/
const Property P18014_1 = {S50, 0, &V18014_1_1, NULL};	/*Property = IsMobile*/
const Value V18014_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18014_5 = {S52, 0, &V18014_5_1277, &P18014_1};	/*Property = ScreenPixelsHeight*/
const Value V18014_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18014_4 = {S54, 0, &V18014_4_2336, &P18014_5};	/*Property = ScreenPixelsWidth*/
const Profile P18014 = {"18014", &P18014_4};	/*Profile ID - 18014*/
const Value V20470_1_1 = {S2, NULL};	/*Value = True*/
const Property P20470_1 = {S50, 0, &V20470_1_1, NULL};	/*Property = IsMobile*/
const Value V20470_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20470_5 = {S52, 0, &V20470_5_1458, &P20470_1};	/*Property = ScreenPixelsHeight*/
const Value V20470_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20470_4 = {S54, 0, &V20470_4_37617, &P20470_5};	/*Property = ScreenPixelsWidth*/
const Profile P20470 = {"20470", &P20470_4};	/*Profile ID - 20470*/
const Value V18314_1_1 = {S2, NULL};	/*Value = True*/
const Property P18314_1 = {S50, 0, &V18314_1_1, NULL};	/*Property = IsMobile*/
const Value V18314_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18314_5 = {S52, 0, &V18314_5_1403, &P18314_1};	/*Property = ScreenPixelsHeight*/
const Value V18314_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P18314_4 = {S54, 0, &V18314_4_8554, &P18314_5};	/*Property = ScreenPixelsWidth*/
const Profile P18314 = {"18314", &P18314_4};	/*Profile ID - 18314*/
const Value V18283_1_1 = {S2, NULL};	/*Value = True*/
const Property P18283_1 = {S50, 0, &V18283_1_1, NULL};	/*Property = IsMobile*/
const Value V18283_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18283_5 = {S52, 0, &V18283_5_9981, &P18283_1};	/*Property = ScreenPixelsHeight*/
const Value V18283_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18283_4 = {S54, 0, &V18283_4_39098, &P18283_5};	/*Property = ScreenPixelsWidth*/
const Profile P18283 = {"18283", &P18283_4};	/*Profile ID - 18283*/
const Value V18323_1_1 = {S2, NULL};	/*Value = True*/
const Property P18323_1 = {S50, 0, &V18323_1_1, NULL};	/*Property = IsMobile*/
const Value V18323_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P18323_5 = {S52, 0, &V18323_5_1380, &P18323_1};	/*Property = ScreenPixelsHeight*/
const Value V18323_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18323_4 = {S54, 0, &V18323_4_1279, &P18323_5};	/*Property = ScreenPixelsWidth*/
const Profile P18323 = {"18323", &P18323_4};	/*Profile ID - 18323*/
const Value V23401_279_57148 = {S0, NULL};	/*Value = False*/
const Property P23401_279 = {S1, 0, &V23401_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V23401_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23401_282 = {S3, 0, &V23401_282_57154, &P23401_279};	/*Property = BlobBuilder*/
const Value V23401_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23401_350 = {S4, 0, &V23401_350_57444, &P23401_282};	/*Property = Canvas*/
const Value V23401_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23401_283 = {S5, 0, &V23401_283_57159, &P23401_350};	/*Property = CssBackground*/
const Value V23401_284_57162 = {S2, NULL};	/*Value = True*/
const Property P23401_284 = {S6, 0, &V23401_284_57162, &P23401_283};	/*Property = CssBorderImage*/
const Value V23401_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23401_285 = {S7, 0, &V23401_285_57166, &P23401_284};	/*Property = CssCanvas*/
const Value V23401_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23401_286 = {S8, 0, &V23401_286_57168, &P23401_285};	/*Property = CssColor*/
const Value V23401_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23401_237 = {S9, 0, &V23401_237_55786, &P23401_286};	/*Property = CssColumn*/
const Value V23401_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23401_287 = {S10, 0, &V23401_287_57172, &P23401_237};	/*Property = CssFlexbox*/
const Value V23401_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23401_288 = {S11, 0, &V23401_288_57174, &P23401_287};	/*Property = CssFont*/
const Value V23401_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23401_289 = {S12, 0, &V23401_289_57759, &P23401_288};	/*Property = CssImages*/
const Value V23401_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23401_291 = {S13, 0, &V23401_291_57180, &P23401_289};	/*Property = CssMediaQueries*/
const Value V23401_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23401_292 = {S14, 0, &V23401_292_57183, &P23401_291};	/*Property = CssMinMax*/
const Value V23401_293_57186 = {S2, NULL};	/*Value = True*/
const Property P23401_293 = {S15, 0, &V23401_293_57186, &P23401_292};	/*Property = CssOverflow*/
const Value V23401_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23401_294 = {S16, 0, &V23401_294_57189, &P23401_293};	/*Property = CssPosition*/
const Value V23401_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23401_295 = {S17, 0, &V23401_295_57192, &P23401_294};	/*Property = CssText*/
const Value V23401_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23401_297 = {S18, 0, &V23401_297_57195, &P23401_295};	/*Property = CssTransforms*/
const Value V23401_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23401_241 = {S19, 0, &V23401_241_55799, &P23401_297};	/*Property = CssTransitions*/
const Value V23401_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23401_298 = {S20, 0, &V23401_298_57206, &P23401_241};	/*Property = CssUI*/
const Value V23401_300_57215 = {S2, NULL};	/*Value = True*/
const Property P23401_300 = {S21, 0, &V23401_300_57215, &P23401_298};	/*Property = DataSet*/
const Value V23401_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23401_301 = {S22, 0, &V23401_301_57218, &P23401_300};	/*Property = DataUrl*/
const Value V23401_302_57221 = {S2, NULL};	/*Value = True*/
const Property P23401_302 = {S23, 0, &V23401_302_57221, &P23401_301};	/*Property = DeviceOrientation*/
const Value V23401_304_57228 = {S0, NULL};	/*Value = False*/
const Property P23401_304 = {S24, 0, &V23401_304_57228, &P23401_302};	/*Property = FileReader*/
const Value V23401_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23401_305 = {S25, 0, &V23401_305_57231, &P23401_304};	/*Property = FileSaver*/
const Value V23401_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23401_306 = {S26, 0, &V23401_306_57237, &P23401_305};	/*Property = FileWriter*/
const Value V23401_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23401_307 = {S27, 0, &V23401_307_57239, &P23401_306};	/*Property = FormData*/
const Value V23401_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23401_309 = {S28, 0, &V23401_309_57246, &P23401_307};	/*Property = Fullscreen*/
const Value V23401_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23401_310 = {S29, 0, &V23401_310_57248, &P23401_309};	/*Property = GeoLocation*/
const Value V23401_312_57255 = {S0, NULL};	/*Value = False*/
const Property P23401_312 = {S30, 0, &V23401_312_57255, &P23401_310};	/*Property = History*/
const Value V23401_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23401_314 = {S31, 0, &V23401_314_57261, &P23401_312};	/*Property = Html5*/
const Value V23401_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23401_313 = {S32, 0, &V23401_313_57258, &P23401_314};	/*Property = Html-Media-Capture*/
const Value V23401_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23401_315 = {S33, 0, &V23401_315_57264, &P23401_313};	/*Property = Iframe*/
const Value V23401_316_57267 = {S0, NULL};	/*Value = False*/
const Property P23401_316 = {S34, 0, &V23401_316_57267, &P23401_315};	/*Property = IndexedDB*/
const Value V23401_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23401_317 = {S35, 0, &V23401_317_57269, &P23401_316};	/*Property = Json*/
const Value V23401_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P23401_15 = {S37, 0, &V23401_15_54167, &P23401_317};	/*Property = LayoutEngine*/
const Value V23401_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23401_344 = {S38, 0, &V23401_344_57272, &P23401_15};	/*Property = Masking*/
const Value V23401_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23401_320 = {S39, 0, &V23401_320_57290, &P23401_344};	/*Property = PostMessage*/
const Value V23401_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23401_321 = {S40, 0, &V23401_321_57293, &P23401_320};	/*Property = Progress*/
const Value V23401_322_57297 = {S0, NULL};	/*Value = False*/
const Property P23401_322 = {S41, 0, &V23401_322_57297, &P23401_321};	/*Property = Prompts*/
const Value V23401_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23401_323 = {S42, 0, &V23401_323_57299, &P23401_322};	/*Property = Selector*/
const Value V23401_324_57306 = {S0, NULL};	/*Value = False*/
const Property P23401_324 = {S43, 0, &V23401_324_57306, &P23401_323};	/*Property = Svg*/
const Value V23401_327_57314 = {S2, NULL};	/*Value = True*/
const Property P23401_327 = {S44, 0, &V23401_327_57314, &P23401_324};	/*Property = TouchEvents*/
const Value V23401_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23401_328 = {S45, 0, &V23401_328_57318, &P23401_327};	/*Property = Track*/
const Value V23401_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23401_329 = {S46, 0, &V23401_329_57323, &P23401_328};	/*Property = Video*/
const Value V23401_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23401_330 = {S47, 0, &V23401_330_57326, &P23401_329};	/*Property = Viewport*/
const Value V23401_332_57339 = {S0, NULL};	/*Value = False*/
const Property P23401_332 = {S48, 0, &V23401_332_57339, &P23401_330};	/*Property = WebWorkers*/
const Value V23401_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23401_333 = {S49, 0, &V23401_333_57342, &P23401_332};	/*Property = Xhr2*/
const Profile P23401 = {"23401", &P23401_333};	/*Profile ID - 23401*/
const Value V15983_1_1 = {S2, NULL};	/*Value = True*/
const Property P15983_1 = {S50, 0, &V15983_1_1, NULL};	/*Property = IsMobile*/
const Value V15983_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15983_5 = {S52, 0, &V15983_5_1277, &P15983_1};	/*Property = ScreenPixelsHeight*/
const Value V15983_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15983_4 = {S54, 0, &V15983_4_1279, &P15983_5};	/*Property = ScreenPixelsWidth*/
const Profile P15983 = {"15983", &P15983_4};	/*Profile ID - 15983*/
const Value V15976_1_1 = {S2, NULL};	/*Value = True*/
const Property P15976_1 = {S50, 0, &V15976_1_1, NULL};	/*Property = IsMobile*/
const Value V15976_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P15976_5 = {S52, 0, &V15976_5_1275, &P15976_1};	/*Property = ScreenPixelsHeight*/
const Value V15976_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15976_4 = {S54, 0, &V15976_4_1279, &P15976_5};	/*Property = ScreenPixelsWidth*/
const Profile P15976 = {"15976", &P15976_4};	/*Profile ID - 15976*/
const Value V15988_1_1 = {S2, NULL};	/*Value = True*/
const Property P15988_1 = {S50, 0, &V15988_1_1, NULL};	/*Property = IsMobile*/
const Value V15988_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15988_5 = {S52, 0, &V15988_5_1222, &P15988_1};	/*Property = ScreenPixelsHeight*/
const Value V15988_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15988_4 = {S54, 0, &V15988_4_1224, &P15988_5};	/*Property = ScreenPixelsWidth*/
const Profile P15988 = {"15988", &P15988_4};	/*Profile ID - 15988*/
const Value V21757_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_279 = {S1, 0, &V21757_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21757_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_282 = {S3, 0, &V21757_282_57155, &P21757_279};	/*Property = BlobBuilder*/
const Value V21757_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_350 = {S4, 0, &V21757_350_57446, &P21757_282};	/*Property = Canvas*/
const Value V21757_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_283 = {S5, 0, &V21757_283_57161, &P21757_350};	/*Property = CssBackground*/
const Value V21757_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_284 = {S6, 0, &V21757_284_57164, &P21757_283};	/*Property = CssBorderImage*/
const Value V21757_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_285 = {S7, 0, &V21757_285_57167, &P21757_284};	/*Property = CssCanvas*/
const Value V21757_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_286 = {S8, 0, &V21757_286_57170, &P21757_285};	/*Property = CssColor*/
const Value V21757_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_237 = {S9, 0, &V21757_237_55786, &P21757_286};	/*Property = CssColumn*/
const Value V21757_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_287 = {S10, 0, &V21757_287_57173, &P21757_237};	/*Property = CssFlexbox*/
const Value V21757_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_288 = {S11, 0, &V21757_288_57176, &P21757_287};	/*Property = CssFont*/
const Value V21757_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_289 = {S12, 0, &V21757_289_86899, &P21757_288};	/*Property = CssImages*/
const Value V21757_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_291 = {S13, 0, &V21757_291_57182, &P21757_289};	/*Property = CssMediaQueries*/
const Value V21757_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_292 = {S14, 0, &V21757_292_57185, &P21757_291};	/*Property = CssMinMax*/
const Value V21757_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_293 = {S15, 0, &V21757_293_57188, &P21757_292};	/*Property = CssOverflow*/
const Value V21757_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_294 = {S16, 0, &V21757_294_57191, &P21757_293};	/*Property = CssPosition*/
const Value V21757_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_295 = {S17, 0, &V21757_295_57194, &P21757_294};	/*Property = CssText*/
const Value V21757_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_297 = {S18, 0, &V21757_297_57197, &P21757_295};	/*Property = CssTransforms*/
const Value V21757_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_241 = {S19, 0, &V21757_241_55801, &P21757_297};	/*Property = CssTransitions*/
const Value V21757_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_298 = {S20, 0, &V21757_298_57208, &P21757_241};	/*Property = CssUI*/
const Value V21757_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_300 = {S21, 0, &V21757_300_57217, &P21757_298};	/*Property = DataSet*/
const Value V21757_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_301 = {S22, 0, &V21757_301_57220, &P21757_300};	/*Property = DataUrl*/
const Value V21757_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_302 = {S23, 0, &V21757_302_57223, &P21757_301};	/*Property = DeviceOrientation*/
const Value V21757_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_304 = {S24, 0, &V21757_304_57229, &P21757_302};	/*Property = FileReader*/
const Value V21757_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_305 = {S25, 0, &V21757_305_57232, &P21757_304};	/*Property = FileSaver*/
const Value V21757_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_306 = {S26, 0, &V21757_306_57238, &P21757_305};	/*Property = FileWriter*/
const Value V21757_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_307 = {S27, 0, &V21757_307_57241, &P21757_306};	/*Property = FormData*/
const Value V21757_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_309 = {S28, 0, &V21757_309_57247, &P21757_307};	/*Property = Fullscreen*/
const Value V21757_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_310 = {S29, 0, &V21757_310_57250, &P21757_309};	/*Property = GeoLocation*/
const Value V21757_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_312 = {S30, 0, &V21757_312_57256, &P21757_310};	/*Property = History*/
const Value V21757_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_314 = {S31, 0, &V21757_314_57262, &P21757_312};	/*Property = Html5*/
const Value V21757_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_313 = {S32, 0, &V21757_313_57259, &P21757_314};	/*Property = Html-Media-Capture*/
const Value V21757_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_315 = {S33, 0, &V21757_315_57265, &P21757_313};	/*Property = Iframe*/
const Value V21757_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_316 = {S34, 0, &V21757_316_57268, &P21757_315};	/*Property = IndexedDB*/
const Value V21757_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_317 = {S35, 0, &V21757_317_57271, &P21757_316};	/*Property = Json*/
const Value V21757_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21757_15 = {S37, 0, &V21757_15_54167, &P21757_317};	/*Property = LayoutEngine*/
const Value V21757_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_344 = {S38, 0, &V21757_344_57274, &P21757_15};	/*Property = Masking*/
const Value V21757_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_320 = {S39, 0, &V21757_320_57292, &P21757_344};	/*Property = PostMessage*/
const Value V21757_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_321 = {S40, 0, &V21757_321_57295, &P21757_320};	/*Property = Progress*/
const Value V21757_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_322 = {S41, 0, &V21757_322_57298, &P21757_321};	/*Property = Prompts*/
const Value V21757_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_323 = {S42, 0, &V21757_323_57301, &P21757_322};	/*Property = Selector*/
const Value V21757_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_324 = {S43, 0, &V21757_324_57307, &P21757_323};	/*Property = Svg*/
const Value V21757_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_327 = {S44, 0, &V21757_327_57316, &P21757_324};	/*Property = TouchEvents*/
const Value V21757_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_328 = {S45, 0, &V21757_328_57319, &P21757_327};	/*Property = Track*/
const Value V21757_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_329 = {S46, 0, &V21757_329_57325, &P21757_328};	/*Property = Video*/
const Value V21757_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_330 = {S47, 0, &V21757_330_57328, &P21757_329};	/*Property = Viewport*/
const Value V21757_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_332 = {S48, 0, &V21757_332_57340, &P21757_330};	/*Property = WebWorkers*/
const Value V21757_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21757_333 = {S49, 0, &V21757_333_57343, &P21757_332};	/*Property = Xhr2*/
const Profile P21757 = {"21757", &P21757_333};	/*Profile ID - 21757*/
const Value V23400_279_57148 = {S0, NULL};	/*Value = False*/
const Property P23400_279 = {S1, 0, &V23400_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V23400_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23400_282 = {S3, 0, &V23400_282_57154, &P23400_279};	/*Property = BlobBuilder*/
const Value V23400_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23400_350 = {S4, 0, &V23400_350_57444, &P23400_282};	/*Property = Canvas*/
const Value V23400_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23400_283 = {S5, 0, &V23400_283_57159, &P23400_350};	/*Property = CssBackground*/
const Value V23400_284_57162 = {S2, NULL};	/*Value = True*/
const Property P23400_284 = {S6, 0, &V23400_284_57162, &P23400_283};	/*Property = CssBorderImage*/
const Value V23400_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23400_285 = {S7, 0, &V23400_285_57166, &P23400_284};	/*Property = CssCanvas*/
const Value V23400_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23400_286 = {S8, 0, &V23400_286_57168, &P23400_285};	/*Property = CssColor*/
const Value V23400_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23400_237 = {S9, 0, &V23400_237_55786, &P23400_286};	/*Property = CssColumn*/
const Value V23400_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23400_287 = {S10, 0, &V23400_287_57172, &P23400_237};	/*Property = CssFlexbox*/
const Value V23400_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23400_288 = {S11, 0, &V23400_288_57174, &P23400_287};	/*Property = CssFont*/
const Value V23400_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23400_289 = {S12, 0, &V23400_289_57759, &P23400_288};	/*Property = CssImages*/
const Value V23400_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23400_291 = {S13, 0, &V23400_291_57180, &P23400_289};	/*Property = CssMediaQueries*/
const Value V23400_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23400_292 = {S14, 0, &V23400_292_57183, &P23400_291};	/*Property = CssMinMax*/
const Value V23400_293_57186 = {S2, NULL};	/*Value = True*/
const Property P23400_293 = {S15, 0, &V23400_293_57186, &P23400_292};	/*Property = CssOverflow*/
const Value V23400_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23400_294 = {S16, 0, &V23400_294_57189, &P23400_293};	/*Property = CssPosition*/
const Value V23400_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23400_295 = {S17, 0, &V23400_295_57192, &P23400_294};	/*Property = CssText*/
const Value V23400_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23400_297 = {S18, 0, &V23400_297_57195, &P23400_295};	/*Property = CssTransforms*/
const Value V23400_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23400_241 = {S19, 0, &V23400_241_55799, &P23400_297};	/*Property = CssTransitions*/
const Value V23400_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23400_298 = {S20, 0, &V23400_298_57206, &P23400_241};	/*Property = CssUI*/
const Value V23400_300_57215 = {S2, NULL};	/*Value = True*/
const Property P23400_300 = {S21, 0, &V23400_300_57215, &P23400_298};	/*Property = DataSet*/
const Value V23400_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23400_301 = {S22, 0, &V23400_301_57218, &P23400_300};	/*Property = DataUrl*/
const Value V23400_302_57221 = {S2, NULL};	/*Value = True*/
const Property P23400_302 = {S23, 0, &V23400_302_57221, &P23400_301};	/*Property = DeviceOrientation*/
const Value V23400_304_57228 = {S0, NULL};	/*Value = False*/
const Property P23400_304 = {S24, 0, &V23400_304_57228, &P23400_302};	/*Property = FileReader*/
const Value V23400_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23400_305 = {S25, 0, &V23400_305_57231, &P23400_304};	/*Property = FileSaver*/
const Value V23400_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23400_306 = {S26, 0, &V23400_306_57237, &P23400_305};	/*Property = FileWriter*/
const Value V23400_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23400_307 = {S27, 0, &V23400_307_57239, &P23400_306};	/*Property = FormData*/
const Value V23400_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23400_309 = {S28, 0, &V23400_309_57246, &P23400_307};	/*Property = Fullscreen*/
const Value V23400_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23400_310 = {S29, 0, &V23400_310_57248, &P23400_309};	/*Property = GeoLocation*/
const Value V23400_312_57255 = {S0, NULL};	/*Value = False*/
const Property P23400_312 = {S30, 0, &V23400_312_57255, &P23400_310};	/*Property = History*/
const Value V23400_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23400_314 = {S31, 0, &V23400_314_57261, &P23400_312};	/*Property = Html5*/
const Value V23400_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23400_313 = {S32, 0, &V23400_313_57258, &P23400_314};	/*Property = Html-Media-Capture*/
const Value V23400_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23400_315 = {S33, 0, &V23400_315_57264, &P23400_313};	/*Property = Iframe*/
const Value V23400_316_57267 = {S0, NULL};	/*Value = False*/
const Property P23400_316 = {S34, 0, &V23400_316_57267, &P23400_315};	/*Property = IndexedDB*/
const Value V23400_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23400_317 = {S35, 0, &V23400_317_57269, &P23400_316};	/*Property = Json*/
const Value V23400_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P23400_15 = {S37, 0, &V23400_15_54167, &P23400_317};	/*Property = LayoutEngine*/
const Value V23400_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23400_344 = {S38, 0, &V23400_344_57272, &P23400_15};	/*Property = Masking*/
const Value V23400_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23400_320 = {S39, 0, &V23400_320_57290, &P23400_344};	/*Property = PostMessage*/
const Value V23400_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23400_321 = {S40, 0, &V23400_321_57293, &P23400_320};	/*Property = Progress*/
const Value V23400_322_57297 = {S0, NULL};	/*Value = False*/
const Property P23400_322 = {S41, 0, &V23400_322_57297, &P23400_321};	/*Property = Prompts*/
const Value V23400_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23400_323 = {S42, 0, &V23400_323_57299, &P23400_322};	/*Property = Selector*/
const Value V23400_324_57306 = {S0, NULL};	/*Value = False*/
const Property P23400_324 = {S43, 0, &V23400_324_57306, &P23400_323};	/*Property = Svg*/
const Value V23400_327_57314 = {S2, NULL};	/*Value = True*/
const Property P23400_327 = {S44, 0, &V23400_327_57314, &P23400_324};	/*Property = TouchEvents*/
const Value V23400_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23400_328 = {S45, 0, &V23400_328_57318, &P23400_327};	/*Property = Track*/
const Value V23400_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23400_329 = {S46, 0, &V23400_329_57323, &P23400_328};	/*Property = Video*/
const Value V23400_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23400_330 = {S47, 0, &V23400_330_57326, &P23400_329};	/*Property = Viewport*/
const Value V23400_332_57339 = {S0, NULL};	/*Value = False*/
const Property P23400_332 = {S48, 0, &V23400_332_57339, &P23400_330};	/*Property = WebWorkers*/
const Value V23400_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23400_333 = {S49, 0, &V23400_333_57342, &P23400_332};	/*Property = Xhr2*/
const Profile P23400 = {"23400", &P23400_333};	/*Profile ID - 23400*/
const Value V15991_1_1 = {S2, NULL};	/*Value = True*/
const Property P15991_1 = {S50, 0, &V15991_1_1, NULL};	/*Property = IsMobile*/
const Value V15991_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15991_5 = {S52, 0, &V15991_5_1222, &P15991_1};	/*Property = ScreenPixelsHeight*/
const Value V15991_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15991_4 = {S54, 0, &V15991_4_1224, &P15991_5};	/*Property = ScreenPixelsWidth*/
const Profile P15991 = {"15991", &P15991_4};	/*Profile ID - 15991*/
const Value V21750_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_279 = {S1, 0, &V21750_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21750_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_282 = {S3, 0, &V21750_282_57155, &P21750_279};	/*Property = BlobBuilder*/
const Value V21750_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_350 = {S4, 0, &V21750_350_57446, &P21750_282};	/*Property = Canvas*/
const Value V21750_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_283 = {S5, 0, &V21750_283_57161, &P21750_350};	/*Property = CssBackground*/
const Value V21750_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_284 = {S6, 0, &V21750_284_57164, &P21750_283};	/*Property = CssBorderImage*/
const Value V21750_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_285 = {S7, 0, &V21750_285_57167, &P21750_284};	/*Property = CssCanvas*/
const Value V21750_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_286 = {S8, 0, &V21750_286_57170, &P21750_285};	/*Property = CssColor*/
const Value V21750_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_237 = {S9, 0, &V21750_237_55786, &P21750_286};	/*Property = CssColumn*/
const Value V21750_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_287 = {S10, 0, &V21750_287_57173, &P21750_237};	/*Property = CssFlexbox*/
const Value V21750_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_288 = {S11, 0, &V21750_288_57176, &P21750_287};	/*Property = CssFont*/
const Value V21750_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_289 = {S12, 0, &V21750_289_86899, &P21750_288};	/*Property = CssImages*/
const Value V21750_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_291 = {S13, 0, &V21750_291_57182, &P21750_289};	/*Property = CssMediaQueries*/
const Value V21750_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_292 = {S14, 0, &V21750_292_57185, &P21750_291};	/*Property = CssMinMax*/
const Value V21750_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_293 = {S15, 0, &V21750_293_57188, &P21750_292};	/*Property = CssOverflow*/
const Value V21750_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_294 = {S16, 0, &V21750_294_57191, &P21750_293};	/*Property = CssPosition*/
const Value V21750_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_295 = {S17, 0, &V21750_295_57194, &P21750_294};	/*Property = CssText*/
const Value V21750_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_297 = {S18, 0, &V21750_297_57197, &P21750_295};	/*Property = CssTransforms*/
const Value V21750_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_241 = {S19, 0, &V21750_241_55801, &P21750_297};	/*Property = CssTransitions*/
const Value V21750_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_298 = {S20, 0, &V21750_298_57208, &P21750_241};	/*Property = CssUI*/
const Value V21750_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_300 = {S21, 0, &V21750_300_57217, &P21750_298};	/*Property = DataSet*/
const Value V21750_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_301 = {S22, 0, &V21750_301_57220, &P21750_300};	/*Property = DataUrl*/
const Value V21750_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_302 = {S23, 0, &V21750_302_57223, &P21750_301};	/*Property = DeviceOrientation*/
const Value V21750_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_304 = {S24, 0, &V21750_304_57229, &P21750_302};	/*Property = FileReader*/
const Value V21750_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_305 = {S25, 0, &V21750_305_57232, &P21750_304};	/*Property = FileSaver*/
const Value V21750_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_306 = {S26, 0, &V21750_306_57238, &P21750_305};	/*Property = FileWriter*/
const Value V21750_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_307 = {S27, 0, &V21750_307_57241, &P21750_306};	/*Property = FormData*/
const Value V21750_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_309 = {S28, 0, &V21750_309_57247, &P21750_307};	/*Property = Fullscreen*/
const Value V21750_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_310 = {S29, 0, &V21750_310_57250, &P21750_309};	/*Property = GeoLocation*/
const Value V21750_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_312 = {S30, 0, &V21750_312_57256, &P21750_310};	/*Property = History*/
const Value V21750_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_314 = {S31, 0, &V21750_314_57262, &P21750_312};	/*Property = Html5*/
const Value V21750_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_313 = {S32, 0, &V21750_313_57259, &P21750_314};	/*Property = Html-Media-Capture*/
const Value V21750_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_315 = {S33, 0, &V21750_315_57265, &P21750_313};	/*Property = Iframe*/
const Value V21750_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_316 = {S34, 0, &V21750_316_57268, &P21750_315};	/*Property = IndexedDB*/
const Value V21750_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_317 = {S35, 0, &V21750_317_57271, &P21750_316};	/*Property = Json*/
const Value V21750_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21750_15 = {S37, 0, &V21750_15_54167, &P21750_317};	/*Property = LayoutEngine*/
const Value V21750_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_344 = {S38, 0, &V21750_344_57274, &P21750_15};	/*Property = Masking*/
const Value V21750_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_320 = {S39, 0, &V21750_320_57292, &P21750_344};	/*Property = PostMessage*/
const Value V21750_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_321 = {S40, 0, &V21750_321_57295, &P21750_320};	/*Property = Progress*/
const Value V21750_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_322 = {S41, 0, &V21750_322_57298, &P21750_321};	/*Property = Prompts*/
const Value V21750_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_323 = {S42, 0, &V21750_323_57301, &P21750_322};	/*Property = Selector*/
const Value V21750_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_324 = {S43, 0, &V21750_324_57307, &P21750_323};	/*Property = Svg*/
const Value V21750_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_327 = {S44, 0, &V21750_327_57316, &P21750_324};	/*Property = TouchEvents*/
const Value V21750_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_328 = {S45, 0, &V21750_328_57319, &P21750_327};	/*Property = Track*/
const Value V21750_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_329 = {S46, 0, &V21750_329_57325, &P21750_328};	/*Property = Video*/
const Value V21750_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_330 = {S47, 0, &V21750_330_57328, &P21750_329};	/*Property = Viewport*/
const Value V21750_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_332 = {S48, 0, &V21750_332_57340, &P21750_330};	/*Property = WebWorkers*/
const Value V21750_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21750_333 = {S49, 0, &V21750_333_57343, &P21750_332};	/*Property = Xhr2*/
const Profile P21750 = {"21750", &P21750_333};	/*Profile ID - 21750*/
const Value V18338_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18338_279 = {S1, 0, &V18338_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18338_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18338_282 = {S3, 0, &V18338_282_57154, &P18338_279};	/*Property = BlobBuilder*/
const Value V18338_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18338_350 = {S4, 0, &V18338_350_57444, &P18338_282};	/*Property = Canvas*/
const Value V18338_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18338_283 = {S5, 0, &V18338_283_57159, &P18338_350};	/*Property = CssBackground*/
const Value V18338_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18338_284 = {S6, 0, &V18338_284_57163, &P18338_283};	/*Property = CssBorderImage*/
const Value V18338_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18338_285 = {S7, 0, &V18338_285_57166, &P18338_284};	/*Property = CssCanvas*/
const Value V18338_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18338_286 = {S8, 0, &V18338_286_57168, &P18338_285};	/*Property = CssColor*/
const Value V18338_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18338_237 = {S9, 0, &V18338_237_55786, &P18338_286};	/*Property = CssColumn*/
const Value V18338_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18338_287 = {S10, 0, &V18338_287_57172, &P18338_237};	/*Property = CssFlexbox*/
const Value V18338_288_57174 = {S2, NULL};	/*Value = True*/
const Property P18338_288 = {S11, 0, &V18338_288_57174, &P18338_287};	/*Property = CssFont*/
const Value V18338_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18338_289 = {S12, 0, &V18338_289_57759, &P18338_288};	/*Property = CssImages*/
const Value V18338_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18338_291 = {S13, 0, &V18338_291_57180, &P18338_289};	/*Property = CssMediaQueries*/
const Value V18338_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18338_292 = {S14, 0, &V18338_292_57183, &P18338_291};	/*Property = CssMinMax*/
const Value V18338_293_57186 = {S2, NULL};	/*Value = True*/
const Property P18338_293 = {S15, 0, &V18338_293_57186, &P18338_292};	/*Property = CssOverflow*/
const Value V18338_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18338_294 = {S16, 0, &V18338_294_57189, &P18338_293};	/*Property = CssPosition*/
const Value V18338_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18338_295 = {S17, 0, &V18338_295_57192, &P18338_294};	/*Property = CssText*/
const Value V18338_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18338_297 = {S18, 0, &V18338_297_57195, &P18338_295};	/*Property = CssTransforms*/
const Value V18338_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18338_241 = {S19, 0, &V18338_241_55799, &P18338_297};	/*Property = CssTransitions*/
const Value V18338_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18338_298 = {S20, 0, &V18338_298_57206, &P18338_241};	/*Property = CssUI*/
const Value V18338_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18338_300 = {S21, 0, &V18338_300_57216, &P18338_298};	/*Property = DataSet*/
const Value V18338_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18338_301 = {S22, 0, &V18338_301_57218, &P18338_300};	/*Property = DataUrl*/
const Value V18338_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18338_302 = {S23, 0, &V18338_302_57222, &P18338_301};	/*Property = DeviceOrientation*/
const Value V18338_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18338_304 = {S24, 0, &V18338_304_57227, &P18338_302};	/*Property = FileReader*/
const Value V18338_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18338_305 = {S25, 0, &V18338_305_57231, &P18338_304};	/*Property = FileSaver*/
const Value V18338_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18338_306 = {S26, 0, &V18338_306_57237, &P18338_305};	/*Property = FileWriter*/
const Value V18338_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18338_307 = {S27, 0, &V18338_307_57239, &P18338_306};	/*Property = FormData*/
const Value V18338_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18338_309 = {S28, 0, &V18338_309_57246, &P18338_307};	/*Property = Fullscreen*/
const Value V18338_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18338_310 = {S29, 0, &V18338_310_57248, &P18338_309};	/*Property = GeoLocation*/
const Value V18338_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18338_312 = {S30, 0, &V18338_312_57254, &P18338_310};	/*Property = History*/
const Value V18338_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18338_314 = {S31, 0, &V18338_314_57261, &P18338_312};	/*Property = Html5*/
const Value V18338_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18338_313 = {S32, 0, &V18338_313_57258, &P18338_314};	/*Property = Html-Media-Capture*/
const Value V18338_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18338_315 = {S33, 0, &V18338_315_57264, &P18338_313};	/*Property = Iframe*/
const Value V18338_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18338_316 = {S34, 0, &V18338_316_57266, &P18338_315};	/*Property = IndexedDB*/
const Value V18338_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18338_317 = {S35, 0, &V18338_317_57269, &P18338_316};	/*Property = Json*/
const Value V18338_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18338_15 = {S37, 0, &V18338_15_28, &P18338_317};	/*Property = LayoutEngine*/
const Value V18338_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18338_344 = {S38, 0, &V18338_344_57272, &P18338_15};	/*Property = Masking*/
const Value V18338_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18338_320 = {S39, 0, &V18338_320_57290, &P18338_344};	/*Property = PostMessage*/
const Value V18338_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18338_321 = {S40, 0, &V18338_321_57293, &P18338_320};	/*Property = Progress*/
const Value V18338_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18338_322 = {S41, 0, &V18338_322_57296, &P18338_321};	/*Property = Prompts*/
const Value V18338_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18338_323 = {S42, 0, &V18338_323_57299, &P18338_322};	/*Property = Selector*/
const Value V18338_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18338_324 = {S43, 0, &V18338_324_57306, &P18338_323};	/*Property = Svg*/
const Value V18338_327_57314 = {S2, NULL};	/*Value = True*/
const Property P18338_327 = {S44, 0, &V18338_327_57314, &P18338_324};	/*Property = TouchEvents*/
const Value V18338_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18338_328 = {S45, 0, &V18338_328_57318, &P18338_327};	/*Property = Track*/
const Value V18338_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18338_329 = {S46, 0, &V18338_329_57323, &P18338_328};	/*Property = Video*/
const Value V18338_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18338_330 = {S47, 0, &V18338_330_57326, &P18338_329};	/*Property = Viewport*/
const Value V18338_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18338_332 = {S48, 0, &V18338_332_57339, &P18338_330};	/*Property = WebWorkers*/
const Value V18338_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18338_333 = {S49, 0, &V18338_333_57342, &P18338_332};	/*Property = Xhr2*/
const Profile P18338 = {"18338", &P18338_333};	/*Profile ID - 18338*/
const Value V22500_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_279 = {S1, 0, &V22500_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22500_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_282 = {S3, 0, &V22500_282_57155, &P22500_279};	/*Property = BlobBuilder*/
const Value V22500_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_350 = {S4, 0, &V22500_350_57446, &P22500_282};	/*Property = Canvas*/
const Value V22500_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_283 = {S5, 0, &V22500_283_57161, &P22500_350};	/*Property = CssBackground*/
const Value V22500_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_284 = {S6, 0, &V22500_284_57164, &P22500_283};	/*Property = CssBorderImage*/
const Value V22500_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_285 = {S7, 0, &V22500_285_57167, &P22500_284};	/*Property = CssCanvas*/
const Value V22500_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_286 = {S8, 0, &V22500_286_57170, &P22500_285};	/*Property = CssColor*/
const Value V22500_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_237 = {S9, 0, &V22500_237_55786, &P22500_286};	/*Property = CssColumn*/
const Value V22500_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_287 = {S10, 0, &V22500_287_57173, &P22500_237};	/*Property = CssFlexbox*/
const Value V22500_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_288 = {S11, 0, &V22500_288_57176, &P22500_287};	/*Property = CssFont*/
const Value V22500_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_289 = {S12, 0, &V22500_289_86899, &P22500_288};	/*Property = CssImages*/
const Value V22500_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_291 = {S13, 0, &V22500_291_57182, &P22500_289};	/*Property = CssMediaQueries*/
const Value V22500_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_292 = {S14, 0, &V22500_292_57185, &P22500_291};	/*Property = CssMinMax*/
const Value V22500_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_293 = {S15, 0, &V22500_293_57188, &P22500_292};	/*Property = CssOverflow*/
const Value V22500_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_294 = {S16, 0, &V22500_294_57191, &P22500_293};	/*Property = CssPosition*/
const Value V22500_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_295 = {S17, 0, &V22500_295_57194, &P22500_294};	/*Property = CssText*/
const Value V22500_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_297 = {S18, 0, &V22500_297_57197, &P22500_295};	/*Property = CssTransforms*/
const Value V22500_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_241 = {S19, 0, &V22500_241_55801, &P22500_297};	/*Property = CssTransitions*/
const Value V22500_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_298 = {S20, 0, &V22500_298_57208, &P22500_241};	/*Property = CssUI*/
const Value V22500_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_300 = {S21, 0, &V22500_300_57217, &P22500_298};	/*Property = DataSet*/
const Value V22500_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_301 = {S22, 0, &V22500_301_57220, &P22500_300};	/*Property = DataUrl*/
const Value V22500_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_302 = {S23, 0, &V22500_302_57223, &P22500_301};	/*Property = DeviceOrientation*/
const Value V22500_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_304 = {S24, 0, &V22500_304_57229, &P22500_302};	/*Property = FileReader*/
const Value V22500_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_305 = {S25, 0, &V22500_305_57232, &P22500_304};	/*Property = FileSaver*/
const Value V22500_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_306 = {S26, 0, &V22500_306_57238, &P22500_305};	/*Property = FileWriter*/
const Value V22500_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_307 = {S27, 0, &V22500_307_57241, &P22500_306};	/*Property = FormData*/
const Value V22500_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_309 = {S28, 0, &V22500_309_57247, &P22500_307};	/*Property = Fullscreen*/
const Value V22500_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_310 = {S29, 0, &V22500_310_57250, &P22500_309};	/*Property = GeoLocation*/
const Value V22500_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_312 = {S30, 0, &V22500_312_57256, &P22500_310};	/*Property = History*/
const Value V22500_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_314 = {S31, 0, &V22500_314_57262, &P22500_312};	/*Property = Html5*/
const Value V22500_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_313 = {S32, 0, &V22500_313_57259, &P22500_314};	/*Property = Html-Media-Capture*/
const Value V22500_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_315 = {S33, 0, &V22500_315_57265, &P22500_313};	/*Property = Iframe*/
const Value V22500_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_316 = {S34, 0, &V22500_316_57268, &P22500_315};	/*Property = IndexedDB*/
const Value V22500_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_317 = {S35, 0, &V22500_317_57271, &P22500_316};	/*Property = Json*/
const Value V22500_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22500_15 = {S37, 0, &V22500_15_29, &P22500_317};	/*Property = LayoutEngine*/
const Value V22500_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_344 = {S38, 0, &V22500_344_57274, &P22500_15};	/*Property = Masking*/
const Value V22500_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_320 = {S39, 0, &V22500_320_57292, &P22500_344};	/*Property = PostMessage*/
const Value V22500_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_321 = {S40, 0, &V22500_321_57295, &P22500_320};	/*Property = Progress*/
const Value V22500_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_322 = {S41, 0, &V22500_322_57298, &P22500_321};	/*Property = Prompts*/
const Value V22500_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_323 = {S42, 0, &V22500_323_57301, &P22500_322};	/*Property = Selector*/
const Value V22500_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_324 = {S43, 0, &V22500_324_57307, &P22500_323};	/*Property = Svg*/
const Value V22500_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_327 = {S44, 0, &V22500_327_57316, &P22500_324};	/*Property = TouchEvents*/
const Value V22500_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_328 = {S45, 0, &V22500_328_57319, &P22500_327};	/*Property = Track*/
const Value V22500_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_329 = {S46, 0, &V22500_329_57325, &P22500_328};	/*Property = Video*/
const Value V22500_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_330 = {S47, 0, &V22500_330_57328, &P22500_329};	/*Property = Viewport*/
const Value V22500_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_332 = {S48, 0, &V22500_332_57340, &P22500_330};	/*Property = WebWorkers*/
const Value V22500_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22500_333 = {S49, 0, &V22500_333_57343, &P22500_332};	/*Property = Xhr2*/
const Profile P22500 = {"22500", &P22500_333};	/*Profile ID - 22500*/
const Value V20900_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20900_279 = {S1, 0, &V20900_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20900_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20900_282 = {S3, 0, &V20900_282_57154, &P20900_279};	/*Property = BlobBuilder*/
const Value V20900_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20900_350 = {S4, 0, &V20900_350_57444, &P20900_282};	/*Property = Canvas*/
const Value V20900_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20900_283 = {S5, 0, &V20900_283_57159, &P20900_350};	/*Property = CssBackground*/
const Value V20900_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20900_284 = {S6, 0, &V20900_284_57163, &P20900_283};	/*Property = CssBorderImage*/
const Value V20900_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20900_285 = {S7, 0, &V20900_285_57166, &P20900_284};	/*Property = CssCanvas*/
const Value V20900_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20900_286 = {S8, 0, &V20900_286_57168, &P20900_285};	/*Property = CssColor*/
const Value V20900_237_55785 = {S0, NULL};	/*Value = False*/
const Property P20900_237 = {S9, 0, &V20900_237_55785, &P20900_286};	/*Property = CssColumn*/
const Value V20900_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20900_287 = {S10, 0, &V20900_287_57172, &P20900_237};	/*Property = CssFlexbox*/
const Value V20900_288_57175 = {S0, NULL};	/*Value = False*/
const Property P20900_288 = {S11, 0, &V20900_288_57175, &P20900_287};	/*Property = CssFont*/
const Value V20900_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20900_289 = {S12, 0, &V20900_289_57759, &P20900_288};	/*Property = CssImages*/
const Value V20900_291_57181 = {S0, NULL};	/*Value = False*/
const Property P20900_291 = {S13, 0, &V20900_291_57181, &P20900_289};	/*Property = CssMediaQueries*/
const Value V20900_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20900_292 = {S14, 0, &V20900_292_57183, &P20900_291};	/*Property = CssMinMax*/
const Value V20900_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20900_293 = {S15, 0, &V20900_293_57187, &P20900_292};	/*Property = CssOverflow*/
const Value V20900_294_57190 = {S0, NULL};	/*Value = False*/
const Property P20900_294 = {S16, 0, &V20900_294_57190, &P20900_293};	/*Property = CssPosition*/
const Value V20900_295_57192 = {S2, NULL};	/*Value = True*/
const Property P20900_295 = {S17, 0, &V20900_295_57192, &P20900_294};	/*Property = CssText*/
const Value V20900_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20900_297 = {S18, 0, &V20900_297_57195, &P20900_295};	/*Property = CssTransforms*/
const Value V20900_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20900_241 = {S19, 0, &V20900_241_55799, &P20900_297};	/*Property = CssTransitions*/
const Value V20900_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20900_298 = {S20, 0, &V20900_298_57206, &P20900_241};	/*Property = CssUI*/
const Value V20900_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20900_300 = {S21, 0, &V20900_300_57216, &P20900_298};	/*Property = DataSet*/
const Value V20900_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20900_301 = {S22, 0, &V20900_301_57218, &P20900_300};	/*Property = DataUrl*/
const Value V20900_302_57222 = {S0, NULL};	/*Value = False*/
const Property P20900_302 = {S23, 0, &V20900_302_57222, &P20900_301};	/*Property = DeviceOrientation*/
const Value V20900_304_57228 = {S0, NULL};	/*Value = False*/
const Property P20900_304 = {S24, 0, &V20900_304_57228, &P20900_302};	/*Property = FileReader*/
const Value V20900_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20900_305 = {S25, 0, &V20900_305_57231, &P20900_304};	/*Property = FileSaver*/
const Value V20900_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20900_306 = {S26, 0, &V20900_306_57237, &P20900_305};	/*Property = FileWriter*/
const Value V20900_307_57240 = {S0, NULL};	/*Value = False*/
const Property P20900_307 = {S27, 0, &V20900_307_57240, &P20900_306};	/*Property = FormData*/
const Value V20900_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20900_309 = {S28, 0, &V20900_309_57246, &P20900_307};	/*Property = Fullscreen*/
const Value V20900_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20900_310 = {S29, 0, &V20900_310_57248, &P20900_309};	/*Property = GeoLocation*/
const Value V20900_312_57255 = {S0, NULL};	/*Value = False*/
const Property P20900_312 = {S30, 0, &V20900_312_57255, &P20900_310};	/*Property = History*/
const Value V20900_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20900_314 = {S31, 0, &V20900_314_57261, &P20900_312};	/*Property = Html5*/
const Value V20900_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20900_313 = {S32, 0, &V20900_313_57258, &P20900_314};	/*Property = Html-Media-Capture*/
const Value V20900_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20900_315 = {S33, 0, &V20900_315_57264, &P20900_313};	/*Property = Iframe*/
const Value V20900_316_57267 = {S0, NULL};	/*Value = False*/
const Property P20900_316 = {S34, 0, &V20900_316_57267, &P20900_315};	/*Property = IndexedDB*/
const Value V20900_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20900_317 = {S35, 0, &V20900_317_57269, &P20900_316};	/*Property = Json*/
const Value V20900_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P20900_15 = {S37, 0, &V20900_15_30, &P20900_317};	/*Property = LayoutEngine*/
const Value V20900_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20900_344 = {S38, 0, &V20900_344_57272, &P20900_15};	/*Property = Masking*/
const Value V20900_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20900_320 = {S39, 0, &V20900_320_57290, &P20900_344};	/*Property = PostMessage*/
const Value V20900_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20900_321 = {S40, 0, &V20900_321_57293, &P20900_320};	/*Property = Progress*/
const Value V20900_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20900_322 = {S41, 0, &V20900_322_57296, &P20900_321};	/*Property = Prompts*/
const Value V20900_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20900_323 = {S42, 0, &V20900_323_57299, &P20900_322};	/*Property = Selector*/
const Value V20900_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20900_324 = {S43, 0, &V20900_324_57306, &P20900_323};	/*Property = Svg*/
const Value V20900_327_57315 = {S0, NULL};	/*Value = False*/
const Property P20900_327 = {S44, 0, &V20900_327_57315, &P20900_324};	/*Property = TouchEvents*/
const Value V20900_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20900_328 = {S45, 0, &V20900_328_57318, &P20900_327};	/*Property = Track*/
const Value V20900_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20900_329 = {S46, 0, &V20900_329_57323, &P20900_328};	/*Property = Video*/
const Value V20900_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20900_330 = {S47, 0, &V20900_330_57326, &P20900_329};	/*Property = Viewport*/
const Value V20900_332_57339 = {S0, NULL};	/*Value = False*/
const Property P20900_332 = {S48, 0, &V20900_332_57339, &P20900_330};	/*Property = WebWorkers*/
const Value V20900_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20900_333 = {S49, 0, &V20900_333_57342, &P20900_332};	/*Property = Xhr2*/
const Profile P20900 = {"20900", &P20900_333};	/*Profile ID - 20900*/
const Value V20903_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20903_279 = {S1, 0, &V20903_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20903_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20903_282 = {S3, 0, &V20903_282_57154, &P20903_279};	/*Property = BlobBuilder*/
const Value V20903_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20903_350 = {S4, 0, &V20903_350_57444, &P20903_282};	/*Property = Canvas*/
const Value V20903_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20903_283 = {S5, 0, &V20903_283_57159, &P20903_350};	/*Property = CssBackground*/
const Value V20903_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20903_284 = {S6, 0, &V20903_284_57163, &P20903_283};	/*Property = CssBorderImage*/
const Value V20903_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20903_285 = {S7, 0, &V20903_285_57166, &P20903_284};	/*Property = CssCanvas*/
const Value V20903_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20903_286 = {S8, 0, &V20903_286_57168, &P20903_285};	/*Property = CssColor*/
const Value V20903_237_55785 = {S0, NULL};	/*Value = False*/
const Property P20903_237 = {S9, 0, &V20903_237_55785, &P20903_286};	/*Property = CssColumn*/
const Value V20903_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20903_287 = {S10, 0, &V20903_287_57172, &P20903_237};	/*Property = CssFlexbox*/
const Value V20903_288_57175 = {S0, NULL};	/*Value = False*/
const Property P20903_288 = {S11, 0, &V20903_288_57175, &P20903_287};	/*Property = CssFont*/
const Value V20903_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20903_289 = {S12, 0, &V20903_289_57759, &P20903_288};	/*Property = CssImages*/
const Value V20903_291_57181 = {S0, NULL};	/*Value = False*/
const Property P20903_291 = {S13, 0, &V20903_291_57181, &P20903_289};	/*Property = CssMediaQueries*/
const Value V20903_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20903_292 = {S14, 0, &V20903_292_57183, &P20903_291};	/*Property = CssMinMax*/
const Value V20903_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20903_293 = {S15, 0, &V20903_293_57187, &P20903_292};	/*Property = CssOverflow*/
const Value V20903_294_57190 = {S0, NULL};	/*Value = False*/
const Property P20903_294 = {S16, 0, &V20903_294_57190, &P20903_293};	/*Property = CssPosition*/
const Value V20903_295_57192 = {S2, NULL};	/*Value = True*/
const Property P20903_295 = {S17, 0, &V20903_295_57192, &P20903_294};	/*Property = CssText*/
const Value V20903_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20903_297 = {S18, 0, &V20903_297_57195, &P20903_295};	/*Property = CssTransforms*/
const Value V20903_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20903_241 = {S19, 0, &V20903_241_55799, &P20903_297};	/*Property = CssTransitions*/
const Value V20903_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20903_298 = {S20, 0, &V20903_298_57206, &P20903_241};	/*Property = CssUI*/
const Value V20903_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20903_300 = {S21, 0, &V20903_300_57216, &P20903_298};	/*Property = DataSet*/
const Value V20903_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20903_301 = {S22, 0, &V20903_301_57218, &P20903_300};	/*Property = DataUrl*/
const Value V20903_302_57222 = {S0, NULL};	/*Value = False*/
const Property P20903_302 = {S23, 0, &V20903_302_57222, &P20903_301};	/*Property = DeviceOrientation*/
const Value V20903_304_57228 = {S0, NULL};	/*Value = False*/
const Property P20903_304 = {S24, 0, &V20903_304_57228, &P20903_302};	/*Property = FileReader*/
const Value V20903_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20903_305 = {S25, 0, &V20903_305_57231, &P20903_304};	/*Property = FileSaver*/
const Value V20903_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20903_306 = {S26, 0, &V20903_306_57237, &P20903_305};	/*Property = FileWriter*/
const Value V20903_307_57240 = {S0, NULL};	/*Value = False*/
const Property P20903_307 = {S27, 0, &V20903_307_57240, &P20903_306};	/*Property = FormData*/
const Value V20903_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20903_309 = {S28, 0, &V20903_309_57246, &P20903_307};	/*Property = Fullscreen*/
const Value V20903_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20903_310 = {S29, 0, &V20903_310_57248, &P20903_309};	/*Property = GeoLocation*/
const Value V20903_312_57255 = {S0, NULL};	/*Value = False*/
const Property P20903_312 = {S30, 0, &V20903_312_57255, &P20903_310};	/*Property = History*/
const Value V20903_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20903_314 = {S31, 0, &V20903_314_57261, &P20903_312};	/*Property = Html5*/
const Value V20903_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20903_313 = {S32, 0, &V20903_313_57258, &P20903_314};	/*Property = Html-Media-Capture*/
const Value V20903_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20903_315 = {S33, 0, &V20903_315_57264, &P20903_313};	/*Property = Iframe*/
const Value V20903_316_57267 = {S0, NULL};	/*Value = False*/
const Property P20903_316 = {S34, 0, &V20903_316_57267, &P20903_315};	/*Property = IndexedDB*/
const Value V20903_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20903_317 = {S35, 0, &V20903_317_57269, &P20903_316};	/*Property = Json*/
const Value V20903_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P20903_15 = {S37, 0, &V20903_15_30, &P20903_317};	/*Property = LayoutEngine*/
const Value V20903_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20903_344 = {S38, 0, &V20903_344_57272, &P20903_15};	/*Property = Masking*/
const Value V20903_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20903_320 = {S39, 0, &V20903_320_57290, &P20903_344};	/*Property = PostMessage*/
const Value V20903_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20903_321 = {S40, 0, &V20903_321_57293, &P20903_320};	/*Property = Progress*/
const Value V20903_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20903_322 = {S41, 0, &V20903_322_57296, &P20903_321};	/*Property = Prompts*/
const Value V20903_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20903_323 = {S42, 0, &V20903_323_57299, &P20903_322};	/*Property = Selector*/
const Value V20903_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20903_324 = {S43, 0, &V20903_324_57306, &P20903_323};	/*Property = Svg*/
const Value V20903_327_57315 = {S0, NULL};	/*Value = False*/
const Property P20903_327 = {S44, 0, &V20903_327_57315, &P20903_324};	/*Property = TouchEvents*/
const Value V20903_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20903_328 = {S45, 0, &V20903_328_57318, &P20903_327};	/*Property = Track*/
const Value V20903_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20903_329 = {S46, 0, &V20903_329_57323, &P20903_328};	/*Property = Video*/
const Value V20903_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20903_330 = {S47, 0, &V20903_330_57326, &P20903_329};	/*Property = Viewport*/
const Value V20903_332_57339 = {S0, NULL};	/*Value = False*/
const Property P20903_332 = {S48, 0, &V20903_332_57339, &P20903_330};	/*Property = WebWorkers*/
const Value V20903_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20903_333 = {S49, 0, &V20903_333_57342, &P20903_332};	/*Property = Xhr2*/
const Profile P20903 = {"20903", &P20903_333};	/*Profile ID - 20903*/
const Value V22516_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22516_279 = {S1, 0, &V22516_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22516_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22516_282 = {S3, 0, &V22516_282_57154, &P22516_279};	/*Property = BlobBuilder*/
const Value V22516_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22516_350 = {S4, 0, &V22516_350_57444, &P22516_282};	/*Property = Canvas*/
const Value V22516_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22516_283 = {S5, 0, &V22516_283_57159, &P22516_350};	/*Property = CssBackground*/
const Value V22516_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22516_284 = {S6, 0, &V22516_284_57163, &P22516_283};	/*Property = CssBorderImage*/
const Value V22516_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22516_285 = {S7, 0, &V22516_285_57166, &P22516_284};	/*Property = CssCanvas*/
const Value V22516_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22516_286 = {S8, 0, &V22516_286_57168, &P22516_285};	/*Property = CssColor*/
const Value V22516_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22516_237 = {S9, 0, &V22516_237_55784, &P22516_286};	/*Property = CssColumn*/
const Value V22516_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22516_287 = {S10, 0, &V22516_287_57172, &P22516_237};	/*Property = CssFlexbox*/
const Value V22516_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22516_288 = {S11, 0, &V22516_288_57174, &P22516_287};	/*Property = CssFont*/
const Value V22516_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22516_289 = {S12, 0, &V22516_289_57759, &P22516_288};	/*Property = CssImages*/
const Value V22516_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22516_291 = {S13, 0, &V22516_291_57180, &P22516_289};	/*Property = CssMediaQueries*/
const Value V22516_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22516_292 = {S14, 0, &V22516_292_57183, &P22516_291};	/*Property = CssMinMax*/
const Value V22516_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22516_293 = {S15, 0, &V22516_293_57187, &P22516_292};	/*Property = CssOverflow*/
const Value V22516_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22516_294 = {S16, 0, &V22516_294_57189, &P22516_293};	/*Property = CssPosition*/
const Value V22516_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22516_295 = {S17, 0, &V22516_295_57192, &P22516_294};	/*Property = CssText*/
const Value V22516_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22516_297 = {S18, 0, &V22516_297_57195, &P22516_295};	/*Property = CssTransforms*/
const Value V22516_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22516_241 = {S19, 0, &V22516_241_55799, &P22516_297};	/*Property = CssTransitions*/
const Value V22516_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22516_298 = {S20, 0, &V22516_298_57206, &P22516_241};	/*Property = CssUI*/
const Value V22516_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22516_300 = {S21, 0, &V22516_300_57215, &P22516_298};	/*Property = DataSet*/
const Value V22516_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22516_301 = {S22, 0, &V22516_301_57218, &P22516_300};	/*Property = DataUrl*/
const Value V22516_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22516_302 = {S23, 0, &V22516_302_57221, &P22516_301};	/*Property = DeviceOrientation*/
const Value V22516_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22516_304 = {S24, 0, &V22516_304_57227, &P22516_302};	/*Property = FileReader*/
const Value V22516_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22516_305 = {S25, 0, &V22516_305_57231, &P22516_304};	/*Property = FileSaver*/
const Value V22516_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22516_306 = {S26, 0, &V22516_306_57237, &P22516_305};	/*Property = FileWriter*/
const Value V22516_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22516_307 = {S27, 0, &V22516_307_57239, &P22516_306};	/*Property = FormData*/
const Value V22516_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22516_309 = {S28, 0, &V22516_309_57246, &P22516_307};	/*Property = Fullscreen*/
const Value V22516_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22516_310 = {S29, 0, &V22516_310_57248, &P22516_309};	/*Property = GeoLocation*/
const Value V22516_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22516_312 = {S30, 0, &V22516_312_57254, &P22516_310};	/*Property = History*/
const Value V22516_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22516_314 = {S31, 0, &V22516_314_57261, &P22516_312};	/*Property = Html5*/
const Value V22516_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22516_313 = {S32, 0, &V22516_313_57258, &P22516_314};	/*Property = Html-Media-Capture*/
const Value V22516_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22516_315 = {S33, 0, &V22516_315_57264, &P22516_313};	/*Property = Iframe*/
const Value V22516_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22516_316 = {S34, 0, &V22516_316_57266, &P22516_315};	/*Property = IndexedDB*/
const Value V22516_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22516_317 = {S35, 0, &V22516_317_57269, &P22516_316};	/*Property = Json*/
const Value V22516_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22516_15 = {S37, 0, &V22516_15_29, &P22516_317};	/*Property = LayoutEngine*/
const Value V22516_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22516_344 = {S38, 0, &V22516_344_57272, &P22516_15};	/*Property = Masking*/
const Value V22516_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22516_320 = {S39, 0, &V22516_320_57290, &P22516_344};	/*Property = PostMessage*/
const Value V22516_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22516_321 = {S40, 0, &V22516_321_57293, &P22516_320};	/*Property = Progress*/
const Value V22516_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22516_322 = {S41, 0, &V22516_322_57296, &P22516_321};	/*Property = Prompts*/
const Value V22516_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22516_323 = {S42, 0, &V22516_323_57299, &P22516_322};	/*Property = Selector*/
const Value V22516_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22516_324 = {S43, 0, &V22516_324_57305, &P22516_323};	/*Property = Svg*/
const Value V22516_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22516_327 = {S44, 0, &V22516_327_57315, &P22516_324};	/*Property = TouchEvents*/
const Value V22516_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22516_328 = {S45, 0, &V22516_328_57318, &P22516_327};	/*Property = Track*/
const Value V22516_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22516_329 = {S46, 0, &V22516_329_57323, &P22516_328};	/*Property = Video*/
const Value V22516_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22516_330 = {S47, 0, &V22516_330_57326, &P22516_329};	/*Property = Viewport*/
const Value V22516_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22516_332 = {S48, 0, &V22516_332_57338, &P22516_330};	/*Property = WebWorkers*/
const Value V22516_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22516_333 = {S49, 0, &V22516_333_57341, &P22516_332};	/*Property = Xhr2*/
const Profile P22516 = {"22516", &P22516_333};	/*Profile ID - 22516*/
const Profile P22418 = {"22418", NULL};	/*Profile ID - 22418*/
const Profile P22203 = {"22203", NULL};	/*Profile ID - 22203*/
const Profile P18343 = {"18343", NULL};	/*Profile ID - 18343*/
const Value V21960_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21960_279 = {S1, 0, &V21960_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21960_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21960_282 = {S3, 0, &V21960_282_57154, &P21960_279};	/*Property = BlobBuilder*/
const Value V21960_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21960_350 = {S4, 0, &V21960_350_57444, &P21960_282};	/*Property = Canvas*/
const Value V21960_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21960_283 = {S5, 0, &V21960_283_57159, &P21960_350};	/*Property = CssBackground*/
const Value V21960_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21960_284 = {S6, 0, &V21960_284_57163, &P21960_283};	/*Property = CssBorderImage*/
const Value V21960_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21960_285 = {S7, 0, &V21960_285_57166, &P21960_284};	/*Property = CssCanvas*/
const Value V21960_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21960_286 = {S8, 0, &V21960_286_57168, &P21960_285};	/*Property = CssColor*/
const Value V21960_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21960_237 = {S9, 0, &V21960_237_55784, &P21960_286};	/*Property = CssColumn*/
const Value V21960_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21960_287 = {S10, 0, &V21960_287_57172, &P21960_237};	/*Property = CssFlexbox*/
const Value V21960_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21960_288 = {S11, 0, &V21960_288_57175, &P21960_287};	/*Property = CssFont*/
const Value V21960_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21960_289 = {S12, 0, &V21960_289_57759, &P21960_288};	/*Property = CssImages*/
const Value V21960_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21960_291 = {S13, 0, &V21960_291_57181, &P21960_289};	/*Property = CssMediaQueries*/
const Value V21960_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21960_292 = {S14, 0, &V21960_292_57183, &P21960_291};	/*Property = CssMinMax*/
const Value V21960_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21960_293 = {S15, 0, &V21960_293_57187, &P21960_292};	/*Property = CssOverflow*/
const Value V21960_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21960_294 = {S16, 0, &V21960_294_57190, &P21960_293};	/*Property = CssPosition*/
const Value V21960_295_57193 = {S0, NULL};	/*Value = False*/
const Property P21960_295 = {S17, 0, &V21960_295_57193, &P21960_294};	/*Property = CssText*/
const Value V21960_297_57196 = {S0, NULL};	/*Value = False*/
const Property P21960_297 = {S18, 0, &V21960_297_57196, &P21960_295};	/*Property = CssTransforms*/
const Value V21960_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21960_241 = {S19, 0, &V21960_241_55799, &P21960_297};	/*Property = CssTransitions*/
const Value V21960_298_57207 = {S0, NULL};	/*Value = False*/
const Property P21960_298 = {S20, 0, &V21960_298_57207, &P21960_241};	/*Property = CssUI*/
const Value V21960_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21960_300 = {S21, 0, &V21960_300_57216, &P21960_298};	/*Property = DataSet*/
const Value V21960_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21960_301 = {S22, 0, &V21960_301_57218, &P21960_300};	/*Property = DataUrl*/
const Value V21960_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21960_302 = {S23, 0, &V21960_302_57222, &P21960_301};	/*Property = DeviceOrientation*/
const Value V21960_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21960_304 = {S24, 0, &V21960_304_57228, &P21960_302};	/*Property = FileReader*/
const Value V21960_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21960_305 = {S25, 0, &V21960_305_57231, &P21960_304};	/*Property = FileSaver*/
const Value V21960_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21960_306 = {S26, 0, &V21960_306_57237, &P21960_305};	/*Property = FileWriter*/
const Value V21960_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21960_307 = {S27, 0, &V21960_307_57240, &P21960_306};	/*Property = FormData*/
const Value V21960_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21960_309 = {S28, 0, &V21960_309_57246, &P21960_307};	/*Property = Fullscreen*/
const Value V21960_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21960_310 = {S29, 0, &V21960_310_57248, &P21960_309};	/*Property = GeoLocation*/
const Value V21960_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21960_312 = {S30, 0, &V21960_312_57255, &P21960_310};	/*Property = History*/
const Value V21960_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21960_314 = {S31, 0, &V21960_314_57261, &P21960_312};	/*Property = Html5*/
const Value V21960_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21960_313 = {S32, 0, &V21960_313_57258, &P21960_314};	/*Property = Html-Media-Capture*/
const Value V21960_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21960_315 = {S33, 0, &V21960_315_57264, &P21960_313};	/*Property = Iframe*/
const Value V21960_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21960_316 = {S34, 0, &V21960_316_57267, &P21960_315};	/*Property = IndexedDB*/
const Value V21960_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21960_317 = {S35, 0, &V21960_317_57269, &P21960_316};	/*Property = Json*/
const Value V21960_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P21960_15 = {S37, 0, &V21960_15_29, &P21960_317};	/*Property = LayoutEngine*/
const Value V21960_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21960_344 = {S38, 0, &V21960_344_57272, &P21960_15};	/*Property = Masking*/
const Value V21960_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21960_320 = {S39, 0, &V21960_320_57290, &P21960_344};	/*Property = PostMessage*/
const Value V21960_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21960_321 = {S40, 0, &V21960_321_57293, &P21960_320};	/*Property = Progress*/
const Value V21960_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21960_322 = {S41, 0, &V21960_322_57296, &P21960_321};	/*Property = Prompts*/
const Value V21960_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21960_323 = {S42, 0, &V21960_323_57299, &P21960_322};	/*Property = Selector*/
const Value V21960_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21960_324 = {S43, 0, &V21960_324_57306, &P21960_323};	/*Property = Svg*/
const Value V21960_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21960_327 = {S44, 0, &V21960_327_57315, &P21960_324};	/*Property = TouchEvents*/
const Value V21960_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21960_328 = {S45, 0, &V21960_328_57318, &P21960_327};	/*Property = Track*/
const Value V21960_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21960_329 = {S46, 0, &V21960_329_57323, &P21960_328};	/*Property = Video*/
const Value V21960_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21960_330 = {S47, 0, &V21960_330_57326, &P21960_329};	/*Property = Viewport*/
const Value V21960_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21960_332 = {S48, 0, &V21960_332_57339, &P21960_330};	/*Property = WebWorkers*/
const Value V21960_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21960_333 = {S49, 0, &V21960_333_57342, &P21960_332};	/*Property = Xhr2*/
const Profile P21960 = {"21960", &P21960_333};	/*Profile ID - 21960*/
const Value V22511_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_279 = {S1, 0, &V22511_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22511_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_282 = {S3, 0, &V22511_282_57155, &P22511_279};	/*Property = BlobBuilder*/
const Value V22511_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_350 = {S4, 0, &V22511_350_57446, &P22511_282};	/*Property = Canvas*/
const Value V22511_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_283 = {S5, 0, &V22511_283_57161, &P22511_350};	/*Property = CssBackground*/
const Value V22511_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_284 = {S6, 0, &V22511_284_57164, &P22511_283};	/*Property = CssBorderImage*/
const Value V22511_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_285 = {S7, 0, &V22511_285_57167, &P22511_284};	/*Property = CssCanvas*/
const Value V22511_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_286 = {S8, 0, &V22511_286_57170, &P22511_285};	/*Property = CssColor*/
const Value V22511_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_237 = {S9, 0, &V22511_237_55786, &P22511_286};	/*Property = CssColumn*/
const Value V22511_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_287 = {S10, 0, &V22511_287_57173, &P22511_237};	/*Property = CssFlexbox*/
const Value V22511_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_288 = {S11, 0, &V22511_288_57176, &P22511_287};	/*Property = CssFont*/
const Value V22511_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_289 = {S12, 0, &V22511_289_86899, &P22511_288};	/*Property = CssImages*/
const Value V22511_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_291 = {S13, 0, &V22511_291_57182, &P22511_289};	/*Property = CssMediaQueries*/
const Value V22511_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_292 = {S14, 0, &V22511_292_57185, &P22511_291};	/*Property = CssMinMax*/
const Value V22511_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_293 = {S15, 0, &V22511_293_57188, &P22511_292};	/*Property = CssOverflow*/
const Value V22511_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_294 = {S16, 0, &V22511_294_57191, &P22511_293};	/*Property = CssPosition*/
const Value V22511_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_295 = {S17, 0, &V22511_295_57194, &P22511_294};	/*Property = CssText*/
const Value V22511_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_297 = {S18, 0, &V22511_297_57197, &P22511_295};	/*Property = CssTransforms*/
const Value V22511_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_241 = {S19, 0, &V22511_241_55801, &P22511_297};	/*Property = CssTransitions*/
const Value V22511_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_298 = {S20, 0, &V22511_298_57208, &P22511_241};	/*Property = CssUI*/
const Value V22511_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_300 = {S21, 0, &V22511_300_57217, &P22511_298};	/*Property = DataSet*/
const Value V22511_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_301 = {S22, 0, &V22511_301_57220, &P22511_300};	/*Property = DataUrl*/
const Value V22511_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_302 = {S23, 0, &V22511_302_57223, &P22511_301};	/*Property = DeviceOrientation*/
const Value V22511_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_304 = {S24, 0, &V22511_304_57229, &P22511_302};	/*Property = FileReader*/
const Value V22511_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_305 = {S25, 0, &V22511_305_57232, &P22511_304};	/*Property = FileSaver*/
const Value V22511_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_306 = {S26, 0, &V22511_306_57238, &P22511_305};	/*Property = FileWriter*/
const Value V22511_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_307 = {S27, 0, &V22511_307_57241, &P22511_306};	/*Property = FormData*/
const Value V22511_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_309 = {S28, 0, &V22511_309_57247, &P22511_307};	/*Property = Fullscreen*/
const Value V22511_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_310 = {S29, 0, &V22511_310_57250, &P22511_309};	/*Property = GeoLocation*/
const Value V22511_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_312 = {S30, 0, &V22511_312_57256, &P22511_310};	/*Property = History*/
const Value V22511_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_314 = {S31, 0, &V22511_314_57262, &P22511_312};	/*Property = Html5*/
const Value V22511_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_313 = {S32, 0, &V22511_313_57259, &P22511_314};	/*Property = Html-Media-Capture*/
const Value V22511_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_315 = {S33, 0, &V22511_315_57265, &P22511_313};	/*Property = Iframe*/
const Value V22511_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_316 = {S34, 0, &V22511_316_57268, &P22511_315};	/*Property = IndexedDB*/
const Value V22511_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_317 = {S35, 0, &V22511_317_57271, &P22511_316};	/*Property = Json*/
const Value V22511_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22511_15 = {S37, 0, &V22511_15_29, &P22511_317};	/*Property = LayoutEngine*/
const Value V22511_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_344 = {S38, 0, &V22511_344_57274, &P22511_15};	/*Property = Masking*/
const Value V22511_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_320 = {S39, 0, &V22511_320_57292, &P22511_344};	/*Property = PostMessage*/
const Value V22511_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_321 = {S40, 0, &V22511_321_57295, &P22511_320};	/*Property = Progress*/
const Value V22511_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_322 = {S41, 0, &V22511_322_57298, &P22511_321};	/*Property = Prompts*/
const Value V22511_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_323 = {S42, 0, &V22511_323_57301, &P22511_322};	/*Property = Selector*/
const Value V22511_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_324 = {S43, 0, &V22511_324_57307, &P22511_323};	/*Property = Svg*/
const Value V22511_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_327 = {S44, 0, &V22511_327_57316, &P22511_324};	/*Property = TouchEvents*/
const Value V22511_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_328 = {S45, 0, &V22511_328_57319, &P22511_327};	/*Property = Track*/
const Value V22511_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_329 = {S46, 0, &V22511_329_57325, &P22511_328};	/*Property = Video*/
const Value V22511_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_330 = {S47, 0, &V22511_330_57328, &P22511_329};	/*Property = Viewport*/
const Value V22511_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_332 = {S48, 0, &V22511_332_57340, &P22511_330};	/*Property = WebWorkers*/
const Value V22511_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22511_333 = {S49, 0, &V22511_333_57343, &P22511_332};	/*Property = Xhr2*/
const Profile P22511 = {"22511", &P22511_333};	/*Profile ID - 22511*/
const Profile P18134 = {"18134", NULL};	/*Profile ID - 18134*/
const Value V22498_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_279 = {S1, 0, &V22498_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22498_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_282 = {S3, 0, &V22498_282_57155, &P22498_279};	/*Property = BlobBuilder*/
const Value V22498_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_350 = {S4, 0, &V22498_350_57446, &P22498_282};	/*Property = Canvas*/
const Value V22498_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_283 = {S5, 0, &V22498_283_57161, &P22498_350};	/*Property = CssBackground*/
const Value V22498_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_284 = {S6, 0, &V22498_284_57164, &P22498_283};	/*Property = CssBorderImage*/
const Value V22498_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_285 = {S7, 0, &V22498_285_57167, &P22498_284};	/*Property = CssCanvas*/
const Value V22498_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_286 = {S8, 0, &V22498_286_57170, &P22498_285};	/*Property = CssColor*/
const Value V22498_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_237 = {S9, 0, &V22498_237_55786, &P22498_286};	/*Property = CssColumn*/
const Value V22498_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_287 = {S10, 0, &V22498_287_57173, &P22498_237};	/*Property = CssFlexbox*/
const Value V22498_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_288 = {S11, 0, &V22498_288_57176, &P22498_287};	/*Property = CssFont*/
const Value V22498_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_289 = {S12, 0, &V22498_289_86899, &P22498_288};	/*Property = CssImages*/
const Value V22498_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_291 = {S13, 0, &V22498_291_57182, &P22498_289};	/*Property = CssMediaQueries*/
const Value V22498_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_292 = {S14, 0, &V22498_292_57185, &P22498_291};	/*Property = CssMinMax*/
const Value V22498_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_293 = {S15, 0, &V22498_293_57188, &P22498_292};	/*Property = CssOverflow*/
const Value V22498_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_294 = {S16, 0, &V22498_294_57191, &P22498_293};	/*Property = CssPosition*/
const Value V22498_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_295 = {S17, 0, &V22498_295_57194, &P22498_294};	/*Property = CssText*/
const Value V22498_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_297 = {S18, 0, &V22498_297_57197, &P22498_295};	/*Property = CssTransforms*/
const Value V22498_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_241 = {S19, 0, &V22498_241_55801, &P22498_297};	/*Property = CssTransitions*/
const Value V22498_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_298 = {S20, 0, &V22498_298_57208, &P22498_241};	/*Property = CssUI*/
const Value V22498_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_300 = {S21, 0, &V22498_300_57217, &P22498_298};	/*Property = DataSet*/
const Value V22498_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_301 = {S22, 0, &V22498_301_57220, &P22498_300};	/*Property = DataUrl*/
const Value V22498_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_302 = {S23, 0, &V22498_302_57223, &P22498_301};	/*Property = DeviceOrientation*/
const Value V22498_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_304 = {S24, 0, &V22498_304_57229, &P22498_302};	/*Property = FileReader*/
const Value V22498_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_305 = {S25, 0, &V22498_305_57232, &P22498_304};	/*Property = FileSaver*/
const Value V22498_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_306 = {S26, 0, &V22498_306_57238, &P22498_305};	/*Property = FileWriter*/
const Value V22498_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_307 = {S27, 0, &V22498_307_57241, &P22498_306};	/*Property = FormData*/
const Value V22498_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_309 = {S28, 0, &V22498_309_57247, &P22498_307};	/*Property = Fullscreen*/
const Value V22498_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_310 = {S29, 0, &V22498_310_57250, &P22498_309};	/*Property = GeoLocation*/
const Value V22498_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_312 = {S30, 0, &V22498_312_57256, &P22498_310};	/*Property = History*/
const Value V22498_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_314 = {S31, 0, &V22498_314_57262, &P22498_312};	/*Property = Html5*/
const Value V22498_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_313 = {S32, 0, &V22498_313_57259, &P22498_314};	/*Property = Html-Media-Capture*/
const Value V22498_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_315 = {S33, 0, &V22498_315_57265, &P22498_313};	/*Property = Iframe*/
const Value V22498_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_316 = {S34, 0, &V22498_316_57268, &P22498_315};	/*Property = IndexedDB*/
const Value V22498_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_317 = {S35, 0, &V22498_317_57271, &P22498_316};	/*Property = Json*/
const Value V22498_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22498_15 = {S37, 0, &V22498_15_29, &P22498_317};	/*Property = LayoutEngine*/
const Value V22498_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_344 = {S38, 0, &V22498_344_57274, &P22498_15};	/*Property = Masking*/
const Value V22498_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_320 = {S39, 0, &V22498_320_57292, &P22498_344};	/*Property = PostMessage*/
const Value V22498_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_321 = {S40, 0, &V22498_321_57295, &P22498_320};	/*Property = Progress*/
const Value V22498_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_322 = {S41, 0, &V22498_322_57298, &P22498_321};	/*Property = Prompts*/
const Value V22498_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_323 = {S42, 0, &V22498_323_57301, &P22498_322};	/*Property = Selector*/
const Value V22498_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_324 = {S43, 0, &V22498_324_57307, &P22498_323};	/*Property = Svg*/
const Value V22498_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_327 = {S44, 0, &V22498_327_57316, &P22498_324};	/*Property = TouchEvents*/
const Value V22498_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_328 = {S45, 0, &V22498_328_57319, &P22498_327};	/*Property = Track*/
const Value V22498_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_329 = {S46, 0, &V22498_329_57325, &P22498_328};	/*Property = Video*/
const Value V22498_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_330 = {S47, 0, &V22498_330_57328, &P22498_329};	/*Property = Viewport*/
const Value V22498_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_332 = {S48, 0, &V22498_332_57340, &P22498_330};	/*Property = WebWorkers*/
const Value V22498_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22498_333 = {S49, 0, &V22498_333_57343, &P22498_332};	/*Property = Xhr2*/
const Profile P22498 = {"22498", &P22498_333};	/*Profile ID - 22498*/
const Value V22896_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_279 = {S1, 0, &V22896_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22896_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_282 = {S3, 0, &V22896_282_57155, &P22896_279};	/*Property = BlobBuilder*/
const Value V22896_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_350 = {S4, 0, &V22896_350_57446, &P22896_282};	/*Property = Canvas*/
const Value V22896_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_283 = {S5, 0, &V22896_283_57161, &P22896_350};	/*Property = CssBackground*/
const Value V22896_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_284 = {S6, 0, &V22896_284_57164, &P22896_283};	/*Property = CssBorderImage*/
const Value V22896_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_285 = {S7, 0, &V22896_285_57167, &P22896_284};	/*Property = CssCanvas*/
const Value V22896_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_286 = {S8, 0, &V22896_286_57170, &P22896_285};	/*Property = CssColor*/
const Value V22896_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_237 = {S9, 0, &V22896_237_55786, &P22896_286};	/*Property = CssColumn*/
const Value V22896_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_287 = {S10, 0, &V22896_287_57173, &P22896_237};	/*Property = CssFlexbox*/
const Value V22896_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_288 = {S11, 0, &V22896_288_57176, &P22896_287};	/*Property = CssFont*/
const Value V22896_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_289 = {S12, 0, &V22896_289_86899, &P22896_288};	/*Property = CssImages*/
const Value V22896_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_291 = {S13, 0, &V22896_291_57182, &P22896_289};	/*Property = CssMediaQueries*/
const Value V22896_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_292 = {S14, 0, &V22896_292_57185, &P22896_291};	/*Property = CssMinMax*/
const Value V22896_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_293 = {S15, 0, &V22896_293_57188, &P22896_292};	/*Property = CssOverflow*/
const Value V22896_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_294 = {S16, 0, &V22896_294_57191, &P22896_293};	/*Property = CssPosition*/
const Value V22896_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_295 = {S17, 0, &V22896_295_57194, &P22896_294};	/*Property = CssText*/
const Value V22896_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_297 = {S18, 0, &V22896_297_57197, &P22896_295};	/*Property = CssTransforms*/
const Value V22896_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_241 = {S19, 0, &V22896_241_55801, &P22896_297};	/*Property = CssTransitions*/
const Value V22896_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_298 = {S20, 0, &V22896_298_57208, &P22896_241};	/*Property = CssUI*/
const Value V22896_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_300 = {S21, 0, &V22896_300_57217, &P22896_298};	/*Property = DataSet*/
const Value V22896_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_301 = {S22, 0, &V22896_301_57220, &P22896_300};	/*Property = DataUrl*/
const Value V22896_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_302 = {S23, 0, &V22896_302_57223, &P22896_301};	/*Property = DeviceOrientation*/
const Value V22896_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_304 = {S24, 0, &V22896_304_57229, &P22896_302};	/*Property = FileReader*/
const Value V22896_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_305 = {S25, 0, &V22896_305_57232, &P22896_304};	/*Property = FileSaver*/
const Value V22896_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_306 = {S26, 0, &V22896_306_57238, &P22896_305};	/*Property = FileWriter*/
const Value V22896_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_307 = {S27, 0, &V22896_307_57241, &P22896_306};	/*Property = FormData*/
const Value V22896_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_309 = {S28, 0, &V22896_309_57247, &P22896_307};	/*Property = Fullscreen*/
const Value V22896_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_310 = {S29, 0, &V22896_310_57250, &P22896_309};	/*Property = GeoLocation*/
const Value V22896_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_312 = {S30, 0, &V22896_312_57256, &P22896_310};	/*Property = History*/
const Value V22896_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_314 = {S31, 0, &V22896_314_57262, &P22896_312};	/*Property = Html5*/
const Value V22896_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_313 = {S32, 0, &V22896_313_57259, &P22896_314};	/*Property = Html-Media-Capture*/
const Value V22896_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_315 = {S33, 0, &V22896_315_57265, &P22896_313};	/*Property = Iframe*/
const Value V22896_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_316 = {S34, 0, &V22896_316_57268, &P22896_315};	/*Property = IndexedDB*/
const Value V22896_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_317 = {S35, 0, &V22896_317_57271, &P22896_316};	/*Property = Json*/
const Value V22896_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_15 = {S37, 0, &V22896_15_72, &P22896_317};	/*Property = LayoutEngine*/
const Value V22896_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_344 = {S38, 0, &V22896_344_57274, &P22896_15};	/*Property = Masking*/
const Value V22896_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_320 = {S39, 0, &V22896_320_57292, &P22896_344};	/*Property = PostMessage*/
const Value V22896_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_321 = {S40, 0, &V22896_321_57295, &P22896_320};	/*Property = Progress*/
const Value V22896_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_322 = {S41, 0, &V22896_322_57298, &P22896_321};	/*Property = Prompts*/
const Value V22896_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_323 = {S42, 0, &V22896_323_57301, &P22896_322};	/*Property = Selector*/
const Value V22896_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_324 = {S43, 0, &V22896_324_57307, &P22896_323};	/*Property = Svg*/
const Value V22896_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_327 = {S44, 0, &V22896_327_57316, &P22896_324};	/*Property = TouchEvents*/
const Value V22896_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_328 = {S45, 0, &V22896_328_57319, &P22896_327};	/*Property = Track*/
const Value V22896_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_329 = {S46, 0, &V22896_329_57325, &P22896_328};	/*Property = Video*/
const Value V22896_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_330 = {S47, 0, &V22896_330_57328, &P22896_329};	/*Property = Viewport*/
const Value V22896_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_332 = {S48, 0, &V22896_332_57340, &P22896_330};	/*Property = WebWorkers*/
const Value V22896_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22896_333 = {S49, 0, &V22896_333_57343, &P22896_332};	/*Property = Xhr2*/
const Profile P22896 = {"22896", &P22896_333};	/*Profile ID - 22896*/
const Value V21959_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21959_279 = {S1, 0, &V21959_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21959_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21959_282 = {S3, 0, &V21959_282_57154, &P21959_279};	/*Property = BlobBuilder*/
const Value V21959_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21959_350 = {S4, 0, &V21959_350_57444, &P21959_282};	/*Property = Canvas*/
const Value V21959_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21959_283 = {S5, 0, &V21959_283_57159, &P21959_350};	/*Property = CssBackground*/
const Value V21959_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21959_284 = {S6, 0, &V21959_284_57163, &P21959_283};	/*Property = CssBorderImage*/
const Value V21959_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21959_285 = {S7, 0, &V21959_285_57166, &P21959_284};	/*Property = CssCanvas*/
const Value V21959_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21959_286 = {S8, 0, &V21959_286_57168, &P21959_285};	/*Property = CssColor*/
const Value V21959_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21959_237 = {S9, 0, &V21959_237_55784, &P21959_286};	/*Property = CssColumn*/
const Value V21959_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21959_287 = {S10, 0, &V21959_287_57172, &P21959_237};	/*Property = CssFlexbox*/
const Value V21959_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21959_288 = {S11, 0, &V21959_288_57175, &P21959_287};	/*Property = CssFont*/
const Value V21959_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21959_289 = {S12, 0, &V21959_289_57759, &P21959_288};	/*Property = CssImages*/
const Value V21959_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21959_291 = {S13, 0, &V21959_291_57181, &P21959_289};	/*Property = CssMediaQueries*/
const Value V21959_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21959_292 = {S14, 0, &V21959_292_57183, &P21959_291};	/*Property = CssMinMax*/
const Value V21959_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21959_293 = {S15, 0, &V21959_293_57187, &P21959_292};	/*Property = CssOverflow*/
const Value V21959_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21959_294 = {S16, 0, &V21959_294_57190, &P21959_293};	/*Property = CssPosition*/
const Value V21959_295_57193 = {S0, NULL};	/*Value = False*/
const Property P21959_295 = {S17, 0, &V21959_295_57193, &P21959_294};	/*Property = CssText*/
const Value V21959_297_57196 = {S0, NULL};	/*Value = False*/
const Property P21959_297 = {S18, 0, &V21959_297_57196, &P21959_295};	/*Property = CssTransforms*/
const Value V21959_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21959_241 = {S19, 0, &V21959_241_55799, &P21959_297};	/*Property = CssTransitions*/
const Value V21959_298_57207 = {S0, NULL};	/*Value = False*/
const Property P21959_298 = {S20, 0, &V21959_298_57207, &P21959_241};	/*Property = CssUI*/
const Value V21959_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21959_300 = {S21, 0, &V21959_300_57216, &P21959_298};	/*Property = DataSet*/
const Value V21959_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21959_301 = {S22, 0, &V21959_301_57218, &P21959_300};	/*Property = DataUrl*/
const Value V21959_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21959_302 = {S23, 0, &V21959_302_57222, &P21959_301};	/*Property = DeviceOrientation*/
const Value V21959_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21959_304 = {S24, 0, &V21959_304_57228, &P21959_302};	/*Property = FileReader*/
const Value V21959_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21959_305 = {S25, 0, &V21959_305_57231, &P21959_304};	/*Property = FileSaver*/
const Value V21959_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21959_306 = {S26, 0, &V21959_306_57237, &P21959_305};	/*Property = FileWriter*/
const Value V21959_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21959_307 = {S27, 0, &V21959_307_57240, &P21959_306};	/*Property = FormData*/
const Value V21959_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21959_309 = {S28, 0, &V21959_309_57246, &P21959_307};	/*Property = Fullscreen*/
const Value V21959_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21959_310 = {S29, 0, &V21959_310_57248, &P21959_309};	/*Property = GeoLocation*/
const Value V21959_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21959_312 = {S30, 0, &V21959_312_57255, &P21959_310};	/*Property = History*/
const Value V21959_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21959_314 = {S31, 0, &V21959_314_57261, &P21959_312};	/*Property = Html5*/
const Value V21959_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21959_313 = {S32, 0, &V21959_313_57258, &P21959_314};	/*Property = Html-Media-Capture*/
const Value V21959_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21959_315 = {S33, 0, &V21959_315_57264, &P21959_313};	/*Property = Iframe*/
const Value V21959_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21959_316 = {S34, 0, &V21959_316_57267, &P21959_315};	/*Property = IndexedDB*/
const Value V21959_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21959_317 = {S35, 0, &V21959_317_57269, &P21959_316};	/*Property = Json*/
const Value V21959_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P21959_15 = {S37, 0, &V21959_15_29, &P21959_317};	/*Property = LayoutEngine*/
const Value V21959_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21959_344 = {S38, 0, &V21959_344_57272, &P21959_15};	/*Property = Masking*/
const Value V21959_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21959_320 = {S39, 0, &V21959_320_57290, &P21959_344};	/*Property = PostMessage*/
const Value V21959_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21959_321 = {S40, 0, &V21959_321_57293, &P21959_320};	/*Property = Progress*/
const Value V21959_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21959_322 = {S41, 0, &V21959_322_57296, &P21959_321};	/*Property = Prompts*/
const Value V21959_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21959_323 = {S42, 0, &V21959_323_57299, &P21959_322};	/*Property = Selector*/
const Value V21959_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21959_324 = {S43, 0, &V21959_324_57306, &P21959_323};	/*Property = Svg*/
const Value V21959_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21959_327 = {S44, 0, &V21959_327_57315, &P21959_324};	/*Property = TouchEvents*/
const Value V21959_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21959_328 = {S45, 0, &V21959_328_57318, &P21959_327};	/*Property = Track*/
const Value V21959_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21959_329 = {S46, 0, &V21959_329_57323, &P21959_328};	/*Property = Video*/
const Value V21959_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21959_330 = {S47, 0, &V21959_330_57326, &P21959_329};	/*Property = Viewport*/
const Value V21959_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21959_332 = {S48, 0, &V21959_332_57339, &P21959_330};	/*Property = WebWorkers*/
const Value V21959_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21959_333 = {S49, 0, &V21959_333_57342, &P21959_332};	/*Property = Xhr2*/
const Profile P21959 = {"21959", &P21959_333};	/*Profile ID - 21959*/
const Value V17349_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_279 = {S1, 0, &V17349_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17349_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_282 = {S3, 0, &V17349_282_57155, &P17349_279};	/*Property = BlobBuilder*/
const Value V17349_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_350 = {S4, 0, &V17349_350_57446, &P17349_282};	/*Property = Canvas*/
const Value V17349_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_283 = {S5, 0, &V17349_283_57161, &P17349_350};	/*Property = CssBackground*/
const Value V17349_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_284 = {S6, 0, &V17349_284_57164, &P17349_283};	/*Property = CssBorderImage*/
const Value V17349_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_285 = {S7, 0, &V17349_285_57167, &P17349_284};	/*Property = CssCanvas*/
const Value V17349_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_286 = {S8, 0, &V17349_286_57170, &P17349_285};	/*Property = CssColor*/
const Value V17349_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_237 = {S9, 0, &V17349_237_55786, &P17349_286};	/*Property = CssColumn*/
const Value V17349_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_287 = {S10, 0, &V17349_287_57173, &P17349_237};	/*Property = CssFlexbox*/
const Value V17349_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_288 = {S11, 0, &V17349_288_57176, &P17349_287};	/*Property = CssFont*/
const Value V17349_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_289 = {S12, 0, &V17349_289_86899, &P17349_288};	/*Property = CssImages*/
const Value V17349_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_291 = {S13, 0, &V17349_291_57182, &P17349_289};	/*Property = CssMediaQueries*/
const Value V17349_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_292 = {S14, 0, &V17349_292_57185, &P17349_291};	/*Property = CssMinMax*/
const Value V17349_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_293 = {S15, 0, &V17349_293_57188, &P17349_292};	/*Property = CssOverflow*/
const Value V17349_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_294 = {S16, 0, &V17349_294_57191, &P17349_293};	/*Property = CssPosition*/
const Value V17349_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_295 = {S17, 0, &V17349_295_57194, &P17349_294};	/*Property = CssText*/
const Value V17349_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_297 = {S18, 0, &V17349_297_57197, &P17349_295};	/*Property = CssTransforms*/
const Value V17349_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_241 = {S19, 0, &V17349_241_55801, &P17349_297};	/*Property = CssTransitions*/
const Value V17349_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_298 = {S20, 0, &V17349_298_57208, &P17349_241};	/*Property = CssUI*/
const Value V17349_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_300 = {S21, 0, &V17349_300_57217, &P17349_298};	/*Property = DataSet*/
const Value V17349_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_301 = {S22, 0, &V17349_301_57220, &P17349_300};	/*Property = DataUrl*/
const Value V17349_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_302 = {S23, 0, &V17349_302_57223, &P17349_301};	/*Property = DeviceOrientation*/
const Value V17349_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_304 = {S24, 0, &V17349_304_57229, &P17349_302};	/*Property = FileReader*/
const Value V17349_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_305 = {S25, 0, &V17349_305_57232, &P17349_304};	/*Property = FileSaver*/
const Value V17349_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_306 = {S26, 0, &V17349_306_57238, &P17349_305};	/*Property = FileWriter*/
const Value V17349_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_307 = {S27, 0, &V17349_307_57241, &P17349_306};	/*Property = FormData*/
const Value V17349_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_309 = {S28, 0, &V17349_309_57247, &P17349_307};	/*Property = Fullscreen*/
const Value V17349_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_310 = {S29, 0, &V17349_310_57250, &P17349_309};	/*Property = GeoLocation*/
const Value V17349_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_312 = {S30, 0, &V17349_312_57256, &P17349_310};	/*Property = History*/
const Value V17349_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_314 = {S31, 0, &V17349_314_57262, &P17349_312};	/*Property = Html5*/
const Value V17349_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_313 = {S32, 0, &V17349_313_57259, &P17349_314};	/*Property = Html-Media-Capture*/
const Value V17349_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_315 = {S33, 0, &V17349_315_57265, &P17349_313};	/*Property = Iframe*/
const Value V17349_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_316 = {S34, 0, &V17349_316_57268, &P17349_315};	/*Property = IndexedDB*/
const Value V17349_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_317 = {S35, 0, &V17349_317_57271, &P17349_316};	/*Property = Json*/
const Value V17349_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P17349_15 = {S37, 0, &V17349_15_90837, &P17349_317};	/*Property = LayoutEngine*/
const Value V17349_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_344 = {S38, 0, &V17349_344_57274, &P17349_15};	/*Property = Masking*/
const Value V17349_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_320 = {S39, 0, &V17349_320_57292, &P17349_344};	/*Property = PostMessage*/
const Value V17349_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_321 = {S40, 0, &V17349_321_57295, &P17349_320};	/*Property = Progress*/
const Value V17349_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_322 = {S41, 0, &V17349_322_57298, &P17349_321};	/*Property = Prompts*/
const Value V17349_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_323 = {S42, 0, &V17349_323_57301, &P17349_322};	/*Property = Selector*/
const Value V17349_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_324 = {S43, 0, &V17349_324_57307, &P17349_323};	/*Property = Svg*/
const Value V17349_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_327 = {S44, 0, &V17349_327_57316, &P17349_324};	/*Property = TouchEvents*/
const Value V17349_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_328 = {S45, 0, &V17349_328_57319, &P17349_327};	/*Property = Track*/
const Value V17349_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_329 = {S46, 0, &V17349_329_57325, &P17349_328};	/*Property = Video*/
const Value V17349_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_330 = {S47, 0, &V17349_330_57328, &P17349_329};	/*Property = Viewport*/
const Value V17349_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_332 = {S48, 0, &V17349_332_57340, &P17349_330};	/*Property = WebWorkers*/
const Value V17349_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17349_333 = {S49, 0, &V17349_333_57343, &P17349_332};	/*Property = Xhr2*/
const Profile P17349 = {"17349", &P17349_333};	/*Profile ID - 17349*/
const Value V22532_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22532_279 = {S1, 0, &V22532_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22532_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22532_282 = {S3, 0, &V22532_282_57154, &P22532_279};	/*Property = BlobBuilder*/
const Value V22532_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22532_350 = {S4, 0, &V22532_350_57444, &P22532_282};	/*Property = Canvas*/
const Value V22532_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22532_283 = {S5, 0, &V22532_283_57159, &P22532_350};	/*Property = CssBackground*/
const Value V22532_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22532_284 = {S6, 0, &V22532_284_57163, &P22532_283};	/*Property = CssBorderImage*/
const Value V22532_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22532_285 = {S7, 0, &V22532_285_57166, &P22532_284};	/*Property = CssCanvas*/
const Value V22532_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22532_286 = {S8, 0, &V22532_286_57168, &P22532_285};	/*Property = CssColor*/
const Value V22532_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22532_237 = {S9, 0, &V22532_237_55786, &P22532_286};	/*Property = CssColumn*/
const Value V22532_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22532_287 = {S10, 0, &V22532_287_57172, &P22532_237};	/*Property = CssFlexbox*/
const Value V22532_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22532_288 = {S11, 0, &V22532_288_57175, &P22532_287};	/*Property = CssFont*/
const Value V22532_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22532_289 = {S12, 0, &V22532_289_57759, &P22532_288};	/*Property = CssImages*/
const Value V22532_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22532_291 = {S13, 0, &V22532_291_57181, &P22532_289};	/*Property = CssMediaQueries*/
const Value V22532_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22532_292 = {S14, 0, &V22532_292_57183, &P22532_291};	/*Property = CssMinMax*/
const Value V22532_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22532_293 = {S15, 0, &V22532_293_57187, &P22532_292};	/*Property = CssOverflow*/
const Value V22532_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22532_294 = {S16, 0, &V22532_294_57190, &P22532_293};	/*Property = CssPosition*/
const Value V22532_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22532_295 = {S17, 0, &V22532_295_57193, &P22532_294};	/*Property = CssText*/
const Value V22532_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22532_297 = {S18, 0, &V22532_297_57195, &P22532_295};	/*Property = CssTransforms*/
const Value V22532_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22532_241 = {S19, 0, &V22532_241_55799, &P22532_297};	/*Property = CssTransitions*/
const Value V22532_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22532_298 = {S20, 0, &V22532_298_57206, &P22532_241};	/*Property = CssUI*/
const Value V22532_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22532_300 = {S21, 0, &V22532_300_57216, &P22532_298};	/*Property = DataSet*/
const Value V22532_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22532_301 = {S22, 0, &V22532_301_57218, &P22532_300};	/*Property = DataUrl*/
const Value V22532_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22532_302 = {S23, 0, &V22532_302_57222, &P22532_301};	/*Property = DeviceOrientation*/
const Value V22532_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22532_304 = {S24, 0, &V22532_304_57228, &P22532_302};	/*Property = FileReader*/
const Value V22532_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22532_305 = {S25, 0, &V22532_305_57231, &P22532_304};	/*Property = FileSaver*/
const Value V22532_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22532_306 = {S26, 0, &V22532_306_57237, &P22532_305};	/*Property = FileWriter*/
const Value V22532_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22532_307 = {S27, 0, &V22532_307_57240, &P22532_306};	/*Property = FormData*/
const Value V22532_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22532_309 = {S28, 0, &V22532_309_57246, &P22532_307};	/*Property = Fullscreen*/
const Value V22532_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22532_310 = {S29, 0, &V22532_310_57248, &P22532_309};	/*Property = GeoLocation*/
const Value V22532_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22532_312 = {S30, 0, &V22532_312_57255, &P22532_310};	/*Property = History*/
const Value V22532_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22532_314 = {S31, 0, &V22532_314_57261, &P22532_312};	/*Property = Html5*/
const Value V22532_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22532_313 = {S32, 0, &V22532_313_57258, &P22532_314};	/*Property = Html-Media-Capture*/
const Value V22532_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22532_315 = {S33, 0, &V22532_315_57264, &P22532_313};	/*Property = Iframe*/
const Value V22532_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22532_316 = {S34, 0, &V22532_316_57267, &P22532_315};	/*Property = IndexedDB*/
const Value V22532_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22532_317 = {S35, 0, &V22532_317_57269, &P22532_316};	/*Property = Json*/
const Value V22532_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22532_15 = {S37, 0, &V22532_15_29, &P22532_317};	/*Property = LayoutEngine*/
const Value V22532_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22532_344 = {S38, 0, &V22532_344_57272, &P22532_15};	/*Property = Masking*/
const Value V22532_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22532_320 = {S39, 0, &V22532_320_57290, &P22532_344};	/*Property = PostMessage*/
const Value V22532_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22532_321 = {S40, 0, &V22532_321_57293, &P22532_320};	/*Property = Progress*/
const Value V22532_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22532_322 = {S41, 0, &V22532_322_57296, &P22532_321};	/*Property = Prompts*/
const Value V22532_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22532_323 = {S42, 0, &V22532_323_57299, &P22532_322};	/*Property = Selector*/
const Value V22532_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22532_324 = {S43, 0, &V22532_324_57306, &P22532_323};	/*Property = Svg*/
const Value V22532_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22532_327 = {S44, 0, &V22532_327_57315, &P22532_324};	/*Property = TouchEvents*/
const Value V22532_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22532_328 = {S45, 0, &V22532_328_57318, &P22532_327};	/*Property = Track*/
const Value V22532_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22532_329 = {S46, 0, &V22532_329_57323, &P22532_328};	/*Property = Video*/
const Value V22532_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22532_330 = {S47, 0, &V22532_330_57326, &P22532_329};	/*Property = Viewport*/
const Value V22532_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22532_332 = {S48, 0, &V22532_332_57339, &P22532_330};	/*Property = WebWorkers*/
const Value V22532_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22532_333 = {S49, 0, &V22532_333_57342, &P22532_332};	/*Property = Xhr2*/
const Profile P22532 = {"22532", &P22532_333};	/*Profile ID - 22532*/
const Value V22482_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_279 = {S1, 0, &V22482_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22482_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_282 = {S3, 0, &V22482_282_57155, &P22482_279};	/*Property = BlobBuilder*/
const Value V22482_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_350 = {S4, 0, &V22482_350_57446, &P22482_282};	/*Property = Canvas*/
const Value V22482_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_283 = {S5, 0, &V22482_283_57161, &P22482_350};	/*Property = CssBackground*/
const Value V22482_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_284 = {S6, 0, &V22482_284_57164, &P22482_283};	/*Property = CssBorderImage*/
const Value V22482_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_285 = {S7, 0, &V22482_285_57167, &P22482_284};	/*Property = CssCanvas*/
const Value V22482_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_286 = {S8, 0, &V22482_286_57170, &P22482_285};	/*Property = CssColor*/
const Value V22482_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_237 = {S9, 0, &V22482_237_55786, &P22482_286};	/*Property = CssColumn*/
const Value V22482_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_287 = {S10, 0, &V22482_287_57173, &P22482_237};	/*Property = CssFlexbox*/
const Value V22482_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_288 = {S11, 0, &V22482_288_57176, &P22482_287};	/*Property = CssFont*/
const Value V22482_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_289 = {S12, 0, &V22482_289_86899, &P22482_288};	/*Property = CssImages*/
const Value V22482_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_291 = {S13, 0, &V22482_291_57182, &P22482_289};	/*Property = CssMediaQueries*/
const Value V22482_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_292 = {S14, 0, &V22482_292_57185, &P22482_291};	/*Property = CssMinMax*/
const Value V22482_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_293 = {S15, 0, &V22482_293_57188, &P22482_292};	/*Property = CssOverflow*/
const Value V22482_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_294 = {S16, 0, &V22482_294_57191, &P22482_293};	/*Property = CssPosition*/
const Value V22482_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_295 = {S17, 0, &V22482_295_57194, &P22482_294};	/*Property = CssText*/
const Value V22482_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_297 = {S18, 0, &V22482_297_57197, &P22482_295};	/*Property = CssTransforms*/
const Value V22482_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_241 = {S19, 0, &V22482_241_55801, &P22482_297};	/*Property = CssTransitions*/
const Value V22482_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_298 = {S20, 0, &V22482_298_57208, &P22482_241};	/*Property = CssUI*/
const Value V22482_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_300 = {S21, 0, &V22482_300_57217, &P22482_298};	/*Property = DataSet*/
const Value V22482_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_301 = {S22, 0, &V22482_301_57220, &P22482_300};	/*Property = DataUrl*/
const Value V22482_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_302 = {S23, 0, &V22482_302_57223, &P22482_301};	/*Property = DeviceOrientation*/
const Value V22482_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_304 = {S24, 0, &V22482_304_57229, &P22482_302};	/*Property = FileReader*/
const Value V22482_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_305 = {S25, 0, &V22482_305_57232, &P22482_304};	/*Property = FileSaver*/
const Value V22482_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_306 = {S26, 0, &V22482_306_57238, &P22482_305};	/*Property = FileWriter*/
const Value V22482_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_307 = {S27, 0, &V22482_307_57241, &P22482_306};	/*Property = FormData*/
const Value V22482_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_309 = {S28, 0, &V22482_309_57247, &P22482_307};	/*Property = Fullscreen*/
const Value V22482_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_310 = {S29, 0, &V22482_310_57250, &P22482_309};	/*Property = GeoLocation*/
const Value V22482_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_312 = {S30, 0, &V22482_312_57256, &P22482_310};	/*Property = History*/
const Value V22482_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_314 = {S31, 0, &V22482_314_57262, &P22482_312};	/*Property = Html5*/
const Value V22482_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_313 = {S32, 0, &V22482_313_57259, &P22482_314};	/*Property = Html-Media-Capture*/
const Value V22482_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_315 = {S33, 0, &V22482_315_57265, &P22482_313};	/*Property = Iframe*/
const Value V22482_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_316 = {S34, 0, &V22482_316_57268, &P22482_315};	/*Property = IndexedDB*/
const Value V22482_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_317 = {S35, 0, &V22482_317_57271, &P22482_316};	/*Property = Json*/
const Value V22482_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22482_15 = {S37, 0, &V22482_15_29, &P22482_317};	/*Property = LayoutEngine*/
const Value V22482_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_344 = {S38, 0, &V22482_344_57274, &P22482_15};	/*Property = Masking*/
const Value V22482_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_320 = {S39, 0, &V22482_320_57292, &P22482_344};	/*Property = PostMessage*/
const Value V22482_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_321 = {S40, 0, &V22482_321_57295, &P22482_320};	/*Property = Progress*/
const Value V22482_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_322 = {S41, 0, &V22482_322_57298, &P22482_321};	/*Property = Prompts*/
const Value V22482_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_323 = {S42, 0, &V22482_323_57301, &P22482_322};	/*Property = Selector*/
const Value V22482_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_324 = {S43, 0, &V22482_324_57307, &P22482_323};	/*Property = Svg*/
const Value V22482_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_327 = {S44, 0, &V22482_327_57316, &P22482_324};	/*Property = TouchEvents*/
const Value V22482_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_328 = {S45, 0, &V22482_328_57319, &P22482_327};	/*Property = Track*/
const Value V22482_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_329 = {S46, 0, &V22482_329_57325, &P22482_328};	/*Property = Video*/
const Value V22482_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_330 = {S47, 0, &V22482_330_57328, &P22482_329};	/*Property = Viewport*/
const Value V22482_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_332 = {S48, 0, &V22482_332_57340, &P22482_330};	/*Property = WebWorkers*/
const Value V22482_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22482_333 = {S49, 0, &V22482_333_57343, &P22482_332};	/*Property = Xhr2*/
const Profile P22482 = {"22482", &P22482_333};	/*Profile ID - 22482*/
const Value V22743_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_279 = {S1, 0, &V22743_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22743_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_282 = {S3, 0, &V22743_282_57155, &P22743_279};	/*Property = BlobBuilder*/
const Value V22743_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_350 = {S4, 0, &V22743_350_57446, &P22743_282};	/*Property = Canvas*/
const Value V22743_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_283 = {S5, 0, &V22743_283_57161, &P22743_350};	/*Property = CssBackground*/
const Value V22743_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_284 = {S6, 0, &V22743_284_57164, &P22743_283};	/*Property = CssBorderImage*/
const Value V22743_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_285 = {S7, 0, &V22743_285_57167, &P22743_284};	/*Property = CssCanvas*/
const Value V22743_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_286 = {S8, 0, &V22743_286_57170, &P22743_285};	/*Property = CssColor*/
const Value V22743_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_237 = {S9, 0, &V22743_237_55786, &P22743_286};	/*Property = CssColumn*/
const Value V22743_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_287 = {S10, 0, &V22743_287_57173, &P22743_237};	/*Property = CssFlexbox*/
const Value V22743_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_288 = {S11, 0, &V22743_288_57176, &P22743_287};	/*Property = CssFont*/
const Value V22743_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_289 = {S12, 0, &V22743_289_86899, &P22743_288};	/*Property = CssImages*/
const Value V22743_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_291 = {S13, 0, &V22743_291_57182, &P22743_289};	/*Property = CssMediaQueries*/
const Value V22743_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_292 = {S14, 0, &V22743_292_57185, &P22743_291};	/*Property = CssMinMax*/
const Value V22743_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_293 = {S15, 0, &V22743_293_57188, &P22743_292};	/*Property = CssOverflow*/
const Value V22743_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_294 = {S16, 0, &V22743_294_57191, &P22743_293};	/*Property = CssPosition*/
const Value V22743_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_295 = {S17, 0, &V22743_295_57194, &P22743_294};	/*Property = CssText*/
const Value V22743_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_297 = {S18, 0, &V22743_297_57197, &P22743_295};	/*Property = CssTransforms*/
const Value V22743_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_241 = {S19, 0, &V22743_241_55801, &P22743_297};	/*Property = CssTransitions*/
const Value V22743_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_298 = {S20, 0, &V22743_298_57208, &P22743_241};	/*Property = CssUI*/
const Value V22743_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_300 = {S21, 0, &V22743_300_57217, &P22743_298};	/*Property = DataSet*/
const Value V22743_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_301 = {S22, 0, &V22743_301_57220, &P22743_300};	/*Property = DataUrl*/
const Value V22743_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_302 = {S23, 0, &V22743_302_57223, &P22743_301};	/*Property = DeviceOrientation*/
const Value V22743_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_304 = {S24, 0, &V22743_304_57229, &P22743_302};	/*Property = FileReader*/
const Value V22743_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_305 = {S25, 0, &V22743_305_57232, &P22743_304};	/*Property = FileSaver*/
const Value V22743_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_306 = {S26, 0, &V22743_306_57238, &P22743_305};	/*Property = FileWriter*/
const Value V22743_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_307 = {S27, 0, &V22743_307_57241, &P22743_306};	/*Property = FormData*/
const Value V22743_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_309 = {S28, 0, &V22743_309_57247, &P22743_307};	/*Property = Fullscreen*/
const Value V22743_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_310 = {S29, 0, &V22743_310_57250, &P22743_309};	/*Property = GeoLocation*/
const Value V22743_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_312 = {S30, 0, &V22743_312_57256, &P22743_310};	/*Property = History*/
const Value V22743_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_314 = {S31, 0, &V22743_314_57262, &P22743_312};	/*Property = Html5*/
const Value V22743_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_313 = {S32, 0, &V22743_313_57259, &P22743_314};	/*Property = Html-Media-Capture*/
const Value V22743_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_315 = {S33, 0, &V22743_315_57265, &P22743_313};	/*Property = Iframe*/
const Value V22743_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_316 = {S34, 0, &V22743_316_57268, &P22743_315};	/*Property = IndexedDB*/
const Value V22743_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_317 = {S35, 0, &V22743_317_57271, &P22743_316};	/*Property = Json*/
const Value V22743_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22743_15 = {S37, 0, &V22743_15_29, &P22743_317};	/*Property = LayoutEngine*/
const Value V22743_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_344 = {S38, 0, &V22743_344_57274, &P22743_15};	/*Property = Masking*/
const Value V22743_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_320 = {S39, 0, &V22743_320_57292, &P22743_344};	/*Property = PostMessage*/
const Value V22743_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_321 = {S40, 0, &V22743_321_57295, &P22743_320};	/*Property = Progress*/
const Value V22743_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_322 = {S41, 0, &V22743_322_57298, &P22743_321};	/*Property = Prompts*/
const Value V22743_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_323 = {S42, 0, &V22743_323_57301, &P22743_322};	/*Property = Selector*/
const Value V22743_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_324 = {S43, 0, &V22743_324_57307, &P22743_323};	/*Property = Svg*/
const Value V22743_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_327 = {S44, 0, &V22743_327_57316, &P22743_324};	/*Property = TouchEvents*/
const Value V22743_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_328 = {S45, 0, &V22743_328_57319, &P22743_327};	/*Property = Track*/
const Value V22743_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_329 = {S46, 0, &V22743_329_57325, &P22743_328};	/*Property = Video*/
const Value V22743_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_330 = {S47, 0, &V22743_330_57328, &P22743_329};	/*Property = Viewport*/
const Value V22743_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_332 = {S48, 0, &V22743_332_57340, &P22743_330};	/*Property = WebWorkers*/
const Value V22743_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22743_333 = {S49, 0, &V22743_333_57343, &P22743_332};	/*Property = Xhr2*/
const Profile P22743 = {"22743", &P22743_333};	/*Profile ID - 22743*/
const Value V22510_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_279 = {S1, 0, &V22510_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22510_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_282 = {S3, 0, &V22510_282_57155, &P22510_279};	/*Property = BlobBuilder*/
const Value V22510_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_350 = {S4, 0, &V22510_350_57446, &P22510_282};	/*Property = Canvas*/
const Value V22510_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_283 = {S5, 0, &V22510_283_57161, &P22510_350};	/*Property = CssBackground*/
const Value V22510_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_284 = {S6, 0, &V22510_284_57164, &P22510_283};	/*Property = CssBorderImage*/
const Value V22510_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_285 = {S7, 0, &V22510_285_57167, &P22510_284};	/*Property = CssCanvas*/
const Value V22510_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_286 = {S8, 0, &V22510_286_57170, &P22510_285};	/*Property = CssColor*/
const Value V22510_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_237 = {S9, 0, &V22510_237_55786, &P22510_286};	/*Property = CssColumn*/
const Value V22510_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_287 = {S10, 0, &V22510_287_57173, &P22510_237};	/*Property = CssFlexbox*/
const Value V22510_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_288 = {S11, 0, &V22510_288_57176, &P22510_287};	/*Property = CssFont*/
const Value V22510_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_289 = {S12, 0, &V22510_289_86899, &P22510_288};	/*Property = CssImages*/
const Value V22510_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_291 = {S13, 0, &V22510_291_57182, &P22510_289};	/*Property = CssMediaQueries*/
const Value V22510_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_292 = {S14, 0, &V22510_292_57185, &P22510_291};	/*Property = CssMinMax*/
const Value V22510_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_293 = {S15, 0, &V22510_293_57188, &P22510_292};	/*Property = CssOverflow*/
const Value V22510_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_294 = {S16, 0, &V22510_294_57191, &P22510_293};	/*Property = CssPosition*/
const Value V22510_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_295 = {S17, 0, &V22510_295_57194, &P22510_294};	/*Property = CssText*/
const Value V22510_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_297 = {S18, 0, &V22510_297_57197, &P22510_295};	/*Property = CssTransforms*/
const Value V22510_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_241 = {S19, 0, &V22510_241_55801, &P22510_297};	/*Property = CssTransitions*/
const Value V22510_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_298 = {S20, 0, &V22510_298_57208, &P22510_241};	/*Property = CssUI*/
const Value V22510_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_300 = {S21, 0, &V22510_300_57217, &P22510_298};	/*Property = DataSet*/
const Value V22510_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_301 = {S22, 0, &V22510_301_57220, &P22510_300};	/*Property = DataUrl*/
const Value V22510_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_302 = {S23, 0, &V22510_302_57223, &P22510_301};	/*Property = DeviceOrientation*/
const Value V22510_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_304 = {S24, 0, &V22510_304_57229, &P22510_302};	/*Property = FileReader*/
const Value V22510_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_305 = {S25, 0, &V22510_305_57232, &P22510_304};	/*Property = FileSaver*/
const Value V22510_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_306 = {S26, 0, &V22510_306_57238, &P22510_305};	/*Property = FileWriter*/
const Value V22510_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_307 = {S27, 0, &V22510_307_57241, &P22510_306};	/*Property = FormData*/
const Value V22510_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_309 = {S28, 0, &V22510_309_57247, &P22510_307};	/*Property = Fullscreen*/
const Value V22510_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_310 = {S29, 0, &V22510_310_57250, &P22510_309};	/*Property = GeoLocation*/
const Value V22510_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_312 = {S30, 0, &V22510_312_57256, &P22510_310};	/*Property = History*/
const Value V22510_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_314 = {S31, 0, &V22510_314_57262, &P22510_312};	/*Property = Html5*/
const Value V22510_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_313 = {S32, 0, &V22510_313_57259, &P22510_314};	/*Property = Html-Media-Capture*/
const Value V22510_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_315 = {S33, 0, &V22510_315_57265, &P22510_313};	/*Property = Iframe*/
const Value V22510_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_316 = {S34, 0, &V22510_316_57268, &P22510_315};	/*Property = IndexedDB*/
const Value V22510_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_317 = {S35, 0, &V22510_317_57271, &P22510_316};	/*Property = Json*/
const Value V22510_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22510_15 = {S37, 0, &V22510_15_29, &P22510_317};	/*Property = LayoutEngine*/
const Value V22510_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_344 = {S38, 0, &V22510_344_57274, &P22510_15};	/*Property = Masking*/
const Value V22510_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_320 = {S39, 0, &V22510_320_57292, &P22510_344};	/*Property = PostMessage*/
const Value V22510_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_321 = {S40, 0, &V22510_321_57295, &P22510_320};	/*Property = Progress*/
const Value V22510_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_322 = {S41, 0, &V22510_322_57298, &P22510_321};	/*Property = Prompts*/
const Value V22510_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_323 = {S42, 0, &V22510_323_57301, &P22510_322};	/*Property = Selector*/
const Value V22510_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_324 = {S43, 0, &V22510_324_57307, &P22510_323};	/*Property = Svg*/
const Value V22510_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_327 = {S44, 0, &V22510_327_57316, &P22510_324};	/*Property = TouchEvents*/
const Value V22510_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_328 = {S45, 0, &V22510_328_57319, &P22510_327};	/*Property = Track*/
const Value V22510_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_329 = {S46, 0, &V22510_329_57325, &P22510_328};	/*Property = Video*/
const Value V22510_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_330 = {S47, 0, &V22510_330_57328, &P22510_329};	/*Property = Viewport*/
const Value V22510_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_332 = {S48, 0, &V22510_332_57340, &P22510_330};	/*Property = WebWorkers*/
const Value V22510_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22510_333 = {S49, 0, &V22510_333_57343, &P22510_332};	/*Property = Xhr2*/
const Profile P22510 = {"22510", &P22510_333};	/*Profile ID - 22510*/
const Value V23487_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_279 = {S1, 0, &V23487_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23487_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_282 = {S3, 0, &V23487_282_57155, &P23487_279};	/*Property = BlobBuilder*/
const Value V23487_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_350 = {S4, 0, &V23487_350_57446, &P23487_282};	/*Property = Canvas*/
const Value V23487_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_283 = {S5, 0, &V23487_283_57161, &P23487_350};	/*Property = CssBackground*/
const Value V23487_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_284 = {S6, 0, &V23487_284_57164, &P23487_283};	/*Property = CssBorderImage*/
const Value V23487_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_285 = {S7, 0, &V23487_285_57167, &P23487_284};	/*Property = CssCanvas*/
const Value V23487_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_286 = {S8, 0, &V23487_286_57170, &P23487_285};	/*Property = CssColor*/
const Value V23487_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_237 = {S9, 0, &V23487_237_55786, &P23487_286};	/*Property = CssColumn*/
const Value V23487_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_287 = {S10, 0, &V23487_287_57173, &P23487_237};	/*Property = CssFlexbox*/
const Value V23487_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_288 = {S11, 0, &V23487_288_57176, &P23487_287};	/*Property = CssFont*/
const Value V23487_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_289 = {S12, 0, &V23487_289_86899, &P23487_288};	/*Property = CssImages*/
const Value V23487_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_291 = {S13, 0, &V23487_291_57182, &P23487_289};	/*Property = CssMediaQueries*/
const Value V23487_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_292 = {S14, 0, &V23487_292_57185, &P23487_291};	/*Property = CssMinMax*/
const Value V23487_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_293 = {S15, 0, &V23487_293_57188, &P23487_292};	/*Property = CssOverflow*/
const Value V23487_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_294 = {S16, 0, &V23487_294_57191, &P23487_293};	/*Property = CssPosition*/
const Value V23487_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_295 = {S17, 0, &V23487_295_57194, &P23487_294};	/*Property = CssText*/
const Value V23487_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_297 = {S18, 0, &V23487_297_57197, &P23487_295};	/*Property = CssTransforms*/
const Value V23487_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_241 = {S19, 0, &V23487_241_55801, &P23487_297};	/*Property = CssTransitions*/
const Value V23487_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_298 = {S20, 0, &V23487_298_57208, &P23487_241};	/*Property = CssUI*/
const Value V23487_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_300 = {S21, 0, &V23487_300_57217, &P23487_298};	/*Property = DataSet*/
const Value V23487_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_301 = {S22, 0, &V23487_301_57220, &P23487_300};	/*Property = DataUrl*/
const Value V23487_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_302 = {S23, 0, &V23487_302_57223, &P23487_301};	/*Property = DeviceOrientation*/
const Value V23487_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_304 = {S24, 0, &V23487_304_57229, &P23487_302};	/*Property = FileReader*/
const Value V23487_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_305 = {S25, 0, &V23487_305_57232, &P23487_304};	/*Property = FileSaver*/
const Value V23487_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_306 = {S26, 0, &V23487_306_57238, &P23487_305};	/*Property = FileWriter*/
const Value V23487_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_307 = {S27, 0, &V23487_307_57241, &P23487_306};	/*Property = FormData*/
const Value V23487_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_309 = {S28, 0, &V23487_309_57247, &P23487_307};	/*Property = Fullscreen*/
const Value V23487_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_310 = {S29, 0, &V23487_310_57250, &P23487_309};	/*Property = GeoLocation*/
const Value V23487_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_312 = {S30, 0, &V23487_312_57256, &P23487_310};	/*Property = History*/
const Value V23487_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_314 = {S31, 0, &V23487_314_57262, &P23487_312};	/*Property = Html5*/
const Value V23487_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_313 = {S32, 0, &V23487_313_57259, &P23487_314};	/*Property = Html-Media-Capture*/
const Value V23487_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_315 = {S33, 0, &V23487_315_57265, &P23487_313};	/*Property = Iframe*/
const Value V23487_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_316 = {S34, 0, &V23487_316_57268, &P23487_315};	/*Property = IndexedDB*/
const Value V23487_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_317 = {S35, 0, &V23487_317_57271, &P23487_316};	/*Property = Json*/
const Value V23487_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P23487_15 = {S37, 0, &V23487_15_29, &P23487_317};	/*Property = LayoutEngine*/
const Value V23487_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_344 = {S38, 0, &V23487_344_57274, &P23487_15};	/*Property = Masking*/
const Value V23487_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_320 = {S39, 0, &V23487_320_57292, &P23487_344};	/*Property = PostMessage*/
const Value V23487_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_321 = {S40, 0, &V23487_321_57295, &P23487_320};	/*Property = Progress*/
const Value V23487_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_322 = {S41, 0, &V23487_322_57298, &P23487_321};	/*Property = Prompts*/
const Value V23487_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_323 = {S42, 0, &V23487_323_57301, &P23487_322};	/*Property = Selector*/
const Value V23487_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_324 = {S43, 0, &V23487_324_57307, &P23487_323};	/*Property = Svg*/
const Value V23487_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_327 = {S44, 0, &V23487_327_57316, &P23487_324};	/*Property = TouchEvents*/
const Value V23487_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_328 = {S45, 0, &V23487_328_57319, &P23487_327};	/*Property = Track*/
const Value V23487_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_329 = {S46, 0, &V23487_329_57325, &P23487_328};	/*Property = Video*/
const Value V23487_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_330 = {S47, 0, &V23487_330_57328, &P23487_329};	/*Property = Viewport*/
const Value V23487_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_332 = {S48, 0, &V23487_332_57340, &P23487_330};	/*Property = WebWorkers*/
const Value V23487_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23487_333 = {S49, 0, &V23487_333_57343, &P23487_332};	/*Property = Xhr2*/
const Profile P23487 = {"23487", &P23487_333};	/*Profile ID - 23487*/
const Value V22333_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_279 = {S1, 0, &V22333_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22333_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_282 = {S3, 0, &V22333_282_57155, &P22333_279};	/*Property = BlobBuilder*/
const Value V22333_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_350 = {S4, 0, &V22333_350_57446, &P22333_282};	/*Property = Canvas*/
const Value V22333_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_283 = {S5, 0, &V22333_283_57161, &P22333_350};	/*Property = CssBackground*/
const Value V22333_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_284 = {S6, 0, &V22333_284_57164, &P22333_283};	/*Property = CssBorderImage*/
const Value V22333_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_285 = {S7, 0, &V22333_285_57167, &P22333_284};	/*Property = CssCanvas*/
const Value V22333_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_286 = {S8, 0, &V22333_286_57170, &P22333_285};	/*Property = CssColor*/
const Value V22333_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_237 = {S9, 0, &V22333_237_55786, &P22333_286};	/*Property = CssColumn*/
const Value V22333_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_287 = {S10, 0, &V22333_287_57173, &P22333_237};	/*Property = CssFlexbox*/
const Value V22333_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_288 = {S11, 0, &V22333_288_57176, &P22333_287};	/*Property = CssFont*/
const Value V22333_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_289 = {S12, 0, &V22333_289_86899, &P22333_288};	/*Property = CssImages*/
const Value V22333_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_291 = {S13, 0, &V22333_291_57182, &P22333_289};	/*Property = CssMediaQueries*/
const Value V22333_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_292 = {S14, 0, &V22333_292_57185, &P22333_291};	/*Property = CssMinMax*/
const Value V22333_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_293 = {S15, 0, &V22333_293_57188, &P22333_292};	/*Property = CssOverflow*/
const Value V22333_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_294 = {S16, 0, &V22333_294_57191, &P22333_293};	/*Property = CssPosition*/
const Value V22333_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_295 = {S17, 0, &V22333_295_57194, &P22333_294};	/*Property = CssText*/
const Value V22333_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_297 = {S18, 0, &V22333_297_57197, &P22333_295};	/*Property = CssTransforms*/
const Value V22333_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_241 = {S19, 0, &V22333_241_55801, &P22333_297};	/*Property = CssTransitions*/
const Value V22333_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_298 = {S20, 0, &V22333_298_57208, &P22333_241};	/*Property = CssUI*/
const Value V22333_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_300 = {S21, 0, &V22333_300_57217, &P22333_298};	/*Property = DataSet*/
const Value V22333_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_301 = {S22, 0, &V22333_301_57220, &P22333_300};	/*Property = DataUrl*/
const Value V22333_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_302 = {S23, 0, &V22333_302_57223, &P22333_301};	/*Property = DeviceOrientation*/
const Value V22333_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_304 = {S24, 0, &V22333_304_57229, &P22333_302};	/*Property = FileReader*/
const Value V22333_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_305 = {S25, 0, &V22333_305_57232, &P22333_304};	/*Property = FileSaver*/
const Value V22333_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_306 = {S26, 0, &V22333_306_57238, &P22333_305};	/*Property = FileWriter*/
const Value V22333_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_307 = {S27, 0, &V22333_307_57241, &P22333_306};	/*Property = FormData*/
const Value V22333_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_309 = {S28, 0, &V22333_309_57247, &P22333_307};	/*Property = Fullscreen*/
const Value V22333_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_310 = {S29, 0, &V22333_310_57250, &P22333_309};	/*Property = GeoLocation*/
const Value V22333_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_312 = {S30, 0, &V22333_312_57256, &P22333_310};	/*Property = History*/
const Value V22333_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_314 = {S31, 0, &V22333_314_57262, &P22333_312};	/*Property = Html5*/
const Value V22333_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_313 = {S32, 0, &V22333_313_57259, &P22333_314};	/*Property = Html-Media-Capture*/
const Value V22333_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_315 = {S33, 0, &V22333_315_57265, &P22333_313};	/*Property = Iframe*/
const Value V22333_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_316 = {S34, 0, &V22333_316_57268, &P22333_315};	/*Property = IndexedDB*/
const Value V22333_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_317 = {S35, 0, &V22333_317_57271, &P22333_316};	/*Property = Json*/
const Value V22333_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22333_15 = {S37, 0, &V22333_15_29, &P22333_317};	/*Property = LayoutEngine*/
const Value V22333_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_344 = {S38, 0, &V22333_344_57274, &P22333_15};	/*Property = Masking*/
const Value V22333_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_320 = {S39, 0, &V22333_320_57292, &P22333_344};	/*Property = PostMessage*/
const Value V22333_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_321 = {S40, 0, &V22333_321_57295, &P22333_320};	/*Property = Progress*/
const Value V22333_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_322 = {S41, 0, &V22333_322_57298, &P22333_321};	/*Property = Prompts*/
const Value V22333_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_323 = {S42, 0, &V22333_323_57301, &P22333_322};	/*Property = Selector*/
const Value V22333_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_324 = {S43, 0, &V22333_324_57307, &P22333_323};	/*Property = Svg*/
const Value V22333_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_327 = {S44, 0, &V22333_327_57316, &P22333_324};	/*Property = TouchEvents*/
const Value V22333_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_328 = {S45, 0, &V22333_328_57319, &P22333_327};	/*Property = Track*/
const Value V22333_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_329 = {S46, 0, &V22333_329_57325, &P22333_328};	/*Property = Video*/
const Value V22333_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_330 = {S47, 0, &V22333_330_57328, &P22333_329};	/*Property = Viewport*/
const Value V22333_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_332 = {S48, 0, &V22333_332_57340, &P22333_330};	/*Property = WebWorkers*/
const Value V22333_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22333_333 = {S49, 0, &V22333_333_57343, &P22333_332};	/*Property = Xhr2*/
const Profile P22333 = {"22333", &P22333_333};	/*Profile ID - 22333*/
const Value V16885_1_1 = {S2, NULL};	/*Value = True*/
const Property P16885_1 = {S50, 0, &V16885_1_1, NULL};	/*Property = IsMobile*/
const Value V16885_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P16885_5 = {S52, 0, &V16885_5_1304, &P16885_1};	/*Property = ScreenPixelsHeight*/
const Value V16885_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16885_4 = {S54, 0, &V16885_4_1224, &P16885_5};	/*Property = ScreenPixelsWidth*/
const Profile P16885 = {"16885", &P16885_4};	/*Profile ID - 16885*/
const Value V22304_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_279 = {S1, 0, &V22304_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22304_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_282 = {S3, 0, &V22304_282_57155, &P22304_279};	/*Property = BlobBuilder*/
const Value V22304_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_350 = {S4, 0, &V22304_350_57446, &P22304_282};	/*Property = Canvas*/
const Value V22304_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_283 = {S5, 0, &V22304_283_57161, &P22304_350};	/*Property = CssBackground*/
const Value V22304_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_284 = {S6, 0, &V22304_284_57164, &P22304_283};	/*Property = CssBorderImage*/
const Value V22304_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_285 = {S7, 0, &V22304_285_57167, &P22304_284};	/*Property = CssCanvas*/
const Value V22304_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_286 = {S8, 0, &V22304_286_57170, &P22304_285};	/*Property = CssColor*/
const Value V22304_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_237 = {S9, 0, &V22304_237_55786, &P22304_286};	/*Property = CssColumn*/
const Value V22304_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_287 = {S10, 0, &V22304_287_57173, &P22304_237};	/*Property = CssFlexbox*/
const Value V22304_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_288 = {S11, 0, &V22304_288_57176, &P22304_287};	/*Property = CssFont*/
const Value V22304_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_289 = {S12, 0, &V22304_289_86899, &P22304_288};	/*Property = CssImages*/
const Value V22304_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_291 = {S13, 0, &V22304_291_57182, &P22304_289};	/*Property = CssMediaQueries*/
const Value V22304_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_292 = {S14, 0, &V22304_292_57185, &P22304_291};	/*Property = CssMinMax*/
const Value V22304_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_293 = {S15, 0, &V22304_293_57188, &P22304_292};	/*Property = CssOverflow*/
const Value V22304_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_294 = {S16, 0, &V22304_294_57191, &P22304_293};	/*Property = CssPosition*/
const Value V22304_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_295 = {S17, 0, &V22304_295_57194, &P22304_294};	/*Property = CssText*/
const Value V22304_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_297 = {S18, 0, &V22304_297_57197, &P22304_295};	/*Property = CssTransforms*/
const Value V22304_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_241 = {S19, 0, &V22304_241_55801, &P22304_297};	/*Property = CssTransitions*/
const Value V22304_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_298 = {S20, 0, &V22304_298_57208, &P22304_241};	/*Property = CssUI*/
const Value V22304_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_300 = {S21, 0, &V22304_300_57217, &P22304_298};	/*Property = DataSet*/
const Value V22304_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_301 = {S22, 0, &V22304_301_57220, &P22304_300};	/*Property = DataUrl*/
const Value V22304_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_302 = {S23, 0, &V22304_302_57223, &P22304_301};	/*Property = DeviceOrientation*/
const Value V22304_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_304 = {S24, 0, &V22304_304_57229, &P22304_302};	/*Property = FileReader*/
const Value V22304_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_305 = {S25, 0, &V22304_305_57232, &P22304_304};	/*Property = FileSaver*/
const Value V22304_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_306 = {S26, 0, &V22304_306_57238, &P22304_305};	/*Property = FileWriter*/
const Value V22304_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_307 = {S27, 0, &V22304_307_57241, &P22304_306};	/*Property = FormData*/
const Value V22304_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_309 = {S28, 0, &V22304_309_57247, &P22304_307};	/*Property = Fullscreen*/
const Value V22304_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_310 = {S29, 0, &V22304_310_57250, &P22304_309};	/*Property = GeoLocation*/
const Value V22304_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_312 = {S30, 0, &V22304_312_57256, &P22304_310};	/*Property = History*/
const Value V22304_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_314 = {S31, 0, &V22304_314_57262, &P22304_312};	/*Property = Html5*/
const Value V22304_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_313 = {S32, 0, &V22304_313_57259, &P22304_314};	/*Property = Html-Media-Capture*/
const Value V22304_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_315 = {S33, 0, &V22304_315_57265, &P22304_313};	/*Property = Iframe*/
const Value V22304_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_316 = {S34, 0, &V22304_316_57268, &P22304_315};	/*Property = IndexedDB*/
const Value V22304_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_317 = {S35, 0, &V22304_317_57271, &P22304_316};	/*Property = Json*/
const Value V22304_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22304_15 = {S37, 0, &V22304_15_29, &P22304_317};	/*Property = LayoutEngine*/
const Value V22304_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_344 = {S38, 0, &V22304_344_57274, &P22304_15};	/*Property = Masking*/
const Value V22304_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_320 = {S39, 0, &V22304_320_57292, &P22304_344};	/*Property = PostMessage*/
const Value V22304_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_321 = {S40, 0, &V22304_321_57295, &P22304_320};	/*Property = Progress*/
const Value V22304_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_322 = {S41, 0, &V22304_322_57298, &P22304_321};	/*Property = Prompts*/
const Value V22304_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_323 = {S42, 0, &V22304_323_57301, &P22304_322};	/*Property = Selector*/
const Value V22304_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_324 = {S43, 0, &V22304_324_57307, &P22304_323};	/*Property = Svg*/
const Value V22304_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_327 = {S44, 0, &V22304_327_57316, &P22304_324};	/*Property = TouchEvents*/
const Value V22304_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_328 = {S45, 0, &V22304_328_57319, &P22304_327};	/*Property = Track*/
const Value V22304_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_329 = {S46, 0, &V22304_329_57325, &P22304_328};	/*Property = Video*/
const Value V22304_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_330 = {S47, 0, &V22304_330_57328, &P22304_329};	/*Property = Viewport*/
const Value V22304_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_332 = {S48, 0, &V22304_332_57340, &P22304_330};	/*Property = WebWorkers*/
const Value V22304_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22304_333 = {S49, 0, &V22304_333_57343, &P22304_332};	/*Property = Xhr2*/
const Profile P22304 = {"22304", &P22304_333};	/*Profile ID - 22304*/
const Value V4719_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_279 = {S1, 0, &V4719_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4719_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_282 = {S3, 0, &V4719_282_57155, &P4719_279};	/*Property = BlobBuilder*/
const Value V4719_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_350 = {S4, 0, &V4719_350_57446, &P4719_282};	/*Property = Canvas*/
const Value V4719_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_283 = {S5, 0, &V4719_283_57161, &P4719_350};	/*Property = CssBackground*/
const Value V4719_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_284 = {S6, 0, &V4719_284_57164, &P4719_283};	/*Property = CssBorderImage*/
const Value V4719_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_285 = {S7, 0, &V4719_285_57167, &P4719_284};	/*Property = CssCanvas*/
const Value V4719_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_286 = {S8, 0, &V4719_286_57170, &P4719_285};	/*Property = CssColor*/
const Value V4719_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_237 = {S9, 0, &V4719_237_55786, &P4719_286};	/*Property = CssColumn*/
const Value V4719_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_287 = {S10, 0, &V4719_287_57173, &P4719_237};	/*Property = CssFlexbox*/
const Value V4719_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_288 = {S11, 0, &V4719_288_57176, &P4719_287};	/*Property = CssFont*/
const Value V4719_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_289 = {S12, 0, &V4719_289_86899, &P4719_288};	/*Property = CssImages*/
const Value V4719_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_291 = {S13, 0, &V4719_291_57182, &P4719_289};	/*Property = CssMediaQueries*/
const Value V4719_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_292 = {S14, 0, &V4719_292_57185, &P4719_291};	/*Property = CssMinMax*/
const Value V4719_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_293 = {S15, 0, &V4719_293_57188, &P4719_292};	/*Property = CssOverflow*/
const Value V4719_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_294 = {S16, 0, &V4719_294_57191, &P4719_293};	/*Property = CssPosition*/
const Value V4719_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_295 = {S17, 0, &V4719_295_57194, &P4719_294};	/*Property = CssText*/
const Value V4719_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_297 = {S18, 0, &V4719_297_57197, &P4719_295};	/*Property = CssTransforms*/
const Value V4719_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_241 = {S19, 0, &V4719_241_55801, &P4719_297};	/*Property = CssTransitions*/
const Value V4719_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_298 = {S20, 0, &V4719_298_57208, &P4719_241};	/*Property = CssUI*/
const Value V4719_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_300 = {S21, 0, &V4719_300_57217, &P4719_298};	/*Property = DataSet*/
const Value V4719_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_301 = {S22, 0, &V4719_301_57220, &P4719_300};	/*Property = DataUrl*/
const Value V4719_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_302 = {S23, 0, &V4719_302_57223, &P4719_301};	/*Property = DeviceOrientation*/
const Value V4719_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_304 = {S24, 0, &V4719_304_57229, &P4719_302};	/*Property = FileReader*/
const Value V4719_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_305 = {S25, 0, &V4719_305_57232, &P4719_304};	/*Property = FileSaver*/
const Value V4719_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_306 = {S26, 0, &V4719_306_57238, &P4719_305};	/*Property = FileWriter*/
const Value V4719_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_307 = {S27, 0, &V4719_307_57241, &P4719_306};	/*Property = FormData*/
const Value V4719_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_309 = {S28, 0, &V4719_309_57247, &P4719_307};	/*Property = Fullscreen*/
const Value V4719_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_310 = {S29, 0, &V4719_310_57250, &P4719_309};	/*Property = GeoLocation*/
const Value V4719_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_312 = {S30, 0, &V4719_312_57256, &P4719_310};	/*Property = History*/
const Value V4719_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_314 = {S31, 0, &V4719_314_57262, &P4719_312};	/*Property = Html5*/
const Value V4719_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_313 = {S32, 0, &V4719_313_57259, &P4719_314};	/*Property = Html-Media-Capture*/
const Value V4719_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_315 = {S33, 0, &V4719_315_57265, &P4719_313};	/*Property = Iframe*/
const Value V4719_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_316 = {S34, 0, &V4719_316_57268, &P4719_315};	/*Property = IndexedDB*/
const Value V4719_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_317 = {S35, 0, &V4719_317_57271, &P4719_316};	/*Property = Json*/
const Value V4719_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P4719_15 = {S37, 0, &V4719_15_31, &P4719_317};	/*Property = LayoutEngine*/
const Value V4719_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_344 = {S38, 0, &V4719_344_57274, &P4719_15};	/*Property = Masking*/
const Value V4719_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_320 = {S39, 0, &V4719_320_57292, &P4719_344};	/*Property = PostMessage*/
const Value V4719_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_321 = {S40, 0, &V4719_321_57295, &P4719_320};	/*Property = Progress*/
const Value V4719_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_322 = {S41, 0, &V4719_322_57298, &P4719_321};	/*Property = Prompts*/
const Value V4719_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_323 = {S42, 0, &V4719_323_57301, &P4719_322};	/*Property = Selector*/
const Value V4719_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_324 = {S43, 0, &V4719_324_57307, &P4719_323};	/*Property = Svg*/
const Value V4719_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_327 = {S44, 0, &V4719_327_57316, &P4719_324};	/*Property = TouchEvents*/
const Value V4719_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_328 = {S45, 0, &V4719_328_57319, &P4719_327};	/*Property = Track*/
const Value V4719_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_329 = {S46, 0, &V4719_329_57325, &P4719_328};	/*Property = Video*/
const Value V4719_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_330 = {S47, 0, &V4719_330_57328, &P4719_329};	/*Property = Viewport*/
const Value V4719_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_332 = {S48, 0, &V4719_332_57340, &P4719_330};	/*Property = WebWorkers*/
const Value V4719_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4719_333 = {S49, 0, &V4719_333_57343, &P4719_332};	/*Property = Xhr2*/
const Profile P4719 = {"4719", &P4719_333};	/*Profile ID - 4719*/
const Value V22332_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_279 = {S1, 0, &V22332_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22332_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_282 = {S3, 0, &V22332_282_57155, &P22332_279};	/*Property = BlobBuilder*/
const Value V22332_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_350 = {S4, 0, &V22332_350_57446, &P22332_282};	/*Property = Canvas*/
const Value V22332_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_283 = {S5, 0, &V22332_283_57161, &P22332_350};	/*Property = CssBackground*/
const Value V22332_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_284 = {S6, 0, &V22332_284_57164, &P22332_283};	/*Property = CssBorderImage*/
const Value V22332_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_285 = {S7, 0, &V22332_285_57167, &P22332_284};	/*Property = CssCanvas*/
const Value V22332_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_286 = {S8, 0, &V22332_286_57170, &P22332_285};	/*Property = CssColor*/
const Value V22332_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_237 = {S9, 0, &V22332_237_55786, &P22332_286};	/*Property = CssColumn*/
const Value V22332_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_287 = {S10, 0, &V22332_287_57173, &P22332_237};	/*Property = CssFlexbox*/
const Value V22332_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_288 = {S11, 0, &V22332_288_57176, &P22332_287};	/*Property = CssFont*/
const Value V22332_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_289 = {S12, 0, &V22332_289_86899, &P22332_288};	/*Property = CssImages*/
const Value V22332_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_291 = {S13, 0, &V22332_291_57182, &P22332_289};	/*Property = CssMediaQueries*/
const Value V22332_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_292 = {S14, 0, &V22332_292_57185, &P22332_291};	/*Property = CssMinMax*/
const Value V22332_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_293 = {S15, 0, &V22332_293_57188, &P22332_292};	/*Property = CssOverflow*/
const Value V22332_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_294 = {S16, 0, &V22332_294_57191, &P22332_293};	/*Property = CssPosition*/
const Value V22332_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_295 = {S17, 0, &V22332_295_57194, &P22332_294};	/*Property = CssText*/
const Value V22332_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_297 = {S18, 0, &V22332_297_57197, &P22332_295};	/*Property = CssTransforms*/
const Value V22332_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_241 = {S19, 0, &V22332_241_55801, &P22332_297};	/*Property = CssTransitions*/
const Value V22332_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_298 = {S20, 0, &V22332_298_57208, &P22332_241};	/*Property = CssUI*/
const Value V22332_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_300 = {S21, 0, &V22332_300_57217, &P22332_298};	/*Property = DataSet*/
const Value V22332_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_301 = {S22, 0, &V22332_301_57220, &P22332_300};	/*Property = DataUrl*/
const Value V22332_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_302 = {S23, 0, &V22332_302_57223, &P22332_301};	/*Property = DeviceOrientation*/
const Value V22332_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_304 = {S24, 0, &V22332_304_57229, &P22332_302};	/*Property = FileReader*/
const Value V22332_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_305 = {S25, 0, &V22332_305_57232, &P22332_304};	/*Property = FileSaver*/
const Value V22332_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_306 = {S26, 0, &V22332_306_57238, &P22332_305};	/*Property = FileWriter*/
const Value V22332_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_307 = {S27, 0, &V22332_307_57241, &P22332_306};	/*Property = FormData*/
const Value V22332_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_309 = {S28, 0, &V22332_309_57247, &P22332_307};	/*Property = Fullscreen*/
const Value V22332_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_310 = {S29, 0, &V22332_310_57250, &P22332_309};	/*Property = GeoLocation*/
const Value V22332_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_312 = {S30, 0, &V22332_312_57256, &P22332_310};	/*Property = History*/
const Value V22332_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_314 = {S31, 0, &V22332_314_57262, &P22332_312};	/*Property = Html5*/
const Value V22332_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_313 = {S32, 0, &V22332_313_57259, &P22332_314};	/*Property = Html-Media-Capture*/
const Value V22332_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_315 = {S33, 0, &V22332_315_57265, &P22332_313};	/*Property = Iframe*/
const Value V22332_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_316 = {S34, 0, &V22332_316_57268, &P22332_315};	/*Property = IndexedDB*/
const Value V22332_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_317 = {S35, 0, &V22332_317_57271, &P22332_316};	/*Property = Json*/
const Value V22332_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22332_15 = {S37, 0, &V22332_15_29, &P22332_317};	/*Property = LayoutEngine*/
const Value V22332_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_344 = {S38, 0, &V22332_344_57274, &P22332_15};	/*Property = Masking*/
const Value V22332_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_320 = {S39, 0, &V22332_320_57292, &P22332_344};	/*Property = PostMessage*/
const Value V22332_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_321 = {S40, 0, &V22332_321_57295, &P22332_320};	/*Property = Progress*/
const Value V22332_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_322 = {S41, 0, &V22332_322_57298, &P22332_321};	/*Property = Prompts*/
const Value V22332_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_323 = {S42, 0, &V22332_323_57301, &P22332_322};	/*Property = Selector*/
const Value V22332_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_324 = {S43, 0, &V22332_324_57307, &P22332_323};	/*Property = Svg*/
const Value V22332_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_327 = {S44, 0, &V22332_327_57316, &P22332_324};	/*Property = TouchEvents*/
const Value V22332_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_328 = {S45, 0, &V22332_328_57319, &P22332_327};	/*Property = Track*/
const Value V22332_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_329 = {S46, 0, &V22332_329_57325, &P22332_328};	/*Property = Video*/
const Value V22332_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_330 = {S47, 0, &V22332_330_57328, &P22332_329};	/*Property = Viewport*/
const Value V22332_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_332 = {S48, 0, &V22332_332_57340, &P22332_330};	/*Property = WebWorkers*/
const Value V22332_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22332_333 = {S49, 0, &V22332_333_57343, &P22332_332};	/*Property = Xhr2*/
const Profile P22332 = {"22332", &P22332_333};	/*Profile ID - 22332*/
const Value V22512_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22512_279 = {S1, 0, &V22512_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22512_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22512_282 = {S3, 0, &V22512_282_57153, &P22512_279};	/*Property = BlobBuilder*/
const Value V22512_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22512_350 = {S4, 0, &V22512_350_57444, &P22512_282};	/*Property = Canvas*/
const Value V22512_283_57160 = {S0, NULL};	/*Value = False*/
const Property P22512_283 = {S5, 0, &V22512_283_57160, &P22512_350};	/*Property = CssBackground*/
const Value V22512_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22512_284 = {S6, 0, &V22512_284_57163, &P22512_283};	/*Property = CssBorderImage*/
const Value V22512_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22512_285 = {S7, 0, &V22512_285_57166, &P22512_284};	/*Property = CssCanvas*/
const Value V22512_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22512_286 = {S8, 0, &V22512_286_57168, &P22512_285};	/*Property = CssColor*/
const Value V22512_237_55785 = {S0, NULL};	/*Value = False*/
const Property P22512_237 = {S9, 0, &V22512_237_55785, &P22512_286};	/*Property = CssColumn*/
const Value V22512_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22512_287 = {S10, 0, &V22512_287_57172, &P22512_237};	/*Property = CssFlexbox*/
const Value V22512_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22512_288 = {S11, 0, &V22512_288_57175, &P22512_287};	/*Property = CssFont*/
const Value V22512_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22512_289 = {S12, 0, &V22512_289_86899, &P22512_288};	/*Property = CssImages*/
const Value V22512_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22512_291 = {S13, 0, &V22512_291_57181, &P22512_289};	/*Property = CssMediaQueries*/
const Value V22512_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22512_292 = {S14, 0, &V22512_292_57183, &P22512_291};	/*Property = CssMinMax*/
const Value V22512_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22512_293 = {S15, 0, &V22512_293_57187, &P22512_292};	/*Property = CssOverflow*/
const Value V22512_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22512_294 = {S16, 0, &V22512_294_57189, &P22512_293};	/*Property = CssPosition*/
const Value V22512_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22512_295 = {S17, 0, &V22512_295_57193, &P22512_294};	/*Property = CssText*/
const Value V22512_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22512_297 = {S18, 0, &V22512_297_57195, &P22512_295};	/*Property = CssTransforms*/
const Value V22512_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22512_241 = {S19, 0, &V22512_241_55799, &P22512_297};	/*Property = CssTransitions*/
const Value V22512_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22512_298 = {S20, 0, &V22512_298_57206, &P22512_241};	/*Property = CssUI*/
const Value V22512_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22512_300 = {S21, 0, &V22512_300_57215, &P22512_298};	/*Property = DataSet*/
const Value V22512_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22512_301 = {S22, 0, &V22512_301_57218, &P22512_300};	/*Property = DataUrl*/
const Value V22512_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22512_302 = {S23, 0, &V22512_302_57221, &P22512_301};	/*Property = DeviceOrientation*/
const Value V22512_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22512_304 = {S24, 0, &V22512_304_57227, &P22512_302};	/*Property = FileReader*/
const Value V22512_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22512_305 = {S25, 0, &V22512_305_57231, &P22512_304};	/*Property = FileSaver*/
const Value V22512_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22512_306 = {S26, 0, &V22512_306_57237, &P22512_305};	/*Property = FileWriter*/
const Value V22512_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22512_307 = {S27, 0, &V22512_307_57239, &P22512_306};	/*Property = FormData*/
const Value V22512_309_57245 = {S2, NULL};	/*Value = True*/
const Property P22512_309 = {S28, 0, &V22512_309_57245, &P22512_307};	/*Property = Fullscreen*/
const Value V22512_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22512_310 = {S29, 0, &V22512_310_57248, &P22512_309};	/*Property = GeoLocation*/
const Value V22512_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22512_312 = {S30, 0, &V22512_312_57254, &P22512_310};	/*Property = History*/
const Value V22512_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22512_314 = {S31, 0, &V22512_314_57261, &P22512_312};	/*Property = Html5*/
const Value V22512_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22512_313 = {S32, 0, &V22512_313_57258, &P22512_314};	/*Property = Html-Media-Capture*/
const Value V22512_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22512_315 = {S33, 0, &V22512_315_57264, &P22512_313};	/*Property = Iframe*/
const Value V22512_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22512_316 = {S34, 0, &V22512_316_57266, &P22512_315};	/*Property = IndexedDB*/
const Value V22512_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22512_317 = {S35, 0, &V22512_317_57269, &P22512_316};	/*Property = Json*/
const Value V22512_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22512_15 = {S37, 0, &V22512_15_29, &P22512_317};	/*Property = LayoutEngine*/
const Value V22512_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22512_344 = {S38, 0, &V22512_344_57272, &P22512_15};	/*Property = Masking*/
const Value V22512_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22512_320 = {S39, 0, &V22512_320_57290, &P22512_344};	/*Property = PostMessage*/
const Value V22512_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22512_321 = {S40, 0, &V22512_321_57293, &P22512_320};	/*Property = Progress*/
const Value V22512_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22512_322 = {S41, 0, &V22512_322_57296, &P22512_321};	/*Property = Prompts*/
const Value V22512_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22512_323 = {S42, 0, &V22512_323_57299, &P22512_322};	/*Property = Selector*/
const Value V22512_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22512_324 = {S43, 0, &V22512_324_57305, &P22512_323};	/*Property = Svg*/
const Value V22512_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22512_327 = {S44, 0, &V22512_327_57315, &P22512_324};	/*Property = TouchEvents*/
const Value V22512_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22512_328 = {S45, 0, &V22512_328_57318, &P22512_327};	/*Property = Track*/
const Value V22512_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22512_329 = {S46, 0, &V22512_329_57323, &P22512_328};	/*Property = Video*/
const Value V22512_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22512_330 = {S47, 0, &V22512_330_57326, &P22512_329};	/*Property = Viewport*/
const Value V22512_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22512_332 = {S48, 0, &V22512_332_57339, &P22512_330};	/*Property = WebWorkers*/
const Value V22512_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22512_333 = {S49, 0, &V22512_333_57342, &P22512_332};	/*Property = Xhr2*/
const Profile P22512 = {"22512", &P22512_333};	/*Profile ID - 22512*/
const Value V22502_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22502_279 = {S1, 0, &V22502_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22502_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22502_282 = {S3, 0, &V22502_282_57154, &P22502_279};	/*Property = BlobBuilder*/
const Value V22502_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22502_350 = {S4, 0, &V22502_350_57444, &P22502_282};	/*Property = Canvas*/
const Value V22502_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22502_283 = {S5, 0, &V22502_283_57159, &P22502_350};	/*Property = CssBackground*/
const Value V22502_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22502_284 = {S6, 0, &V22502_284_57163, &P22502_283};	/*Property = CssBorderImage*/
const Value V22502_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22502_285 = {S7, 0, &V22502_285_57166, &P22502_284};	/*Property = CssCanvas*/
const Value V22502_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22502_286 = {S8, 0, &V22502_286_57168, &P22502_285};	/*Property = CssColor*/
const Value V22502_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22502_237 = {S9, 0, &V22502_237_55784, &P22502_286};	/*Property = CssColumn*/
const Value V22502_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22502_287 = {S10, 0, &V22502_287_57172, &P22502_237};	/*Property = CssFlexbox*/
const Value V22502_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22502_288 = {S11, 0, &V22502_288_57174, &P22502_287};	/*Property = CssFont*/
const Value V22502_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22502_289 = {S12, 0, &V22502_289_57759, &P22502_288};	/*Property = CssImages*/
const Value V22502_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22502_291 = {S13, 0, &V22502_291_57180, &P22502_289};	/*Property = CssMediaQueries*/
const Value V22502_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22502_292 = {S14, 0, &V22502_292_57183, &P22502_291};	/*Property = CssMinMax*/
const Value V22502_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22502_293 = {S15, 0, &V22502_293_57187, &P22502_292};	/*Property = CssOverflow*/
const Value V22502_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22502_294 = {S16, 0, &V22502_294_57189, &P22502_293};	/*Property = CssPosition*/
const Value V22502_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22502_295 = {S17, 0, &V22502_295_57192, &P22502_294};	/*Property = CssText*/
const Value V22502_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22502_297 = {S18, 0, &V22502_297_57195, &P22502_295};	/*Property = CssTransforms*/
const Value V22502_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22502_241 = {S19, 0, &V22502_241_55799, &P22502_297};	/*Property = CssTransitions*/
const Value V22502_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22502_298 = {S20, 0, &V22502_298_57206, &P22502_241};	/*Property = CssUI*/
const Value V22502_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22502_300 = {S21, 0, &V22502_300_57215, &P22502_298};	/*Property = DataSet*/
const Value V22502_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22502_301 = {S22, 0, &V22502_301_57218, &P22502_300};	/*Property = DataUrl*/
const Value V22502_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22502_302 = {S23, 0, &V22502_302_57221, &P22502_301};	/*Property = DeviceOrientation*/
const Value V22502_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22502_304 = {S24, 0, &V22502_304_57227, &P22502_302};	/*Property = FileReader*/
const Value V22502_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22502_305 = {S25, 0, &V22502_305_57231, &P22502_304};	/*Property = FileSaver*/
const Value V22502_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22502_306 = {S26, 0, &V22502_306_57237, &P22502_305};	/*Property = FileWriter*/
const Value V22502_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22502_307 = {S27, 0, &V22502_307_57239, &P22502_306};	/*Property = FormData*/
const Value V22502_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22502_309 = {S28, 0, &V22502_309_57246, &P22502_307};	/*Property = Fullscreen*/
const Value V22502_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22502_310 = {S29, 0, &V22502_310_57248, &P22502_309};	/*Property = GeoLocation*/
const Value V22502_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22502_312 = {S30, 0, &V22502_312_57254, &P22502_310};	/*Property = History*/
const Value V22502_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22502_314 = {S31, 0, &V22502_314_57261, &P22502_312};	/*Property = Html5*/
const Value V22502_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22502_313 = {S32, 0, &V22502_313_57258, &P22502_314};	/*Property = Html-Media-Capture*/
const Value V22502_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22502_315 = {S33, 0, &V22502_315_57264, &P22502_313};	/*Property = Iframe*/
const Value V22502_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22502_316 = {S34, 0, &V22502_316_57266, &P22502_315};	/*Property = IndexedDB*/
const Value V22502_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22502_317 = {S35, 0, &V22502_317_57269, &P22502_316};	/*Property = Json*/
const Value V22502_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22502_15 = {S37, 0, &V22502_15_29, &P22502_317};	/*Property = LayoutEngine*/
const Value V22502_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22502_344 = {S38, 0, &V22502_344_57272, &P22502_15};	/*Property = Masking*/
const Value V22502_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22502_320 = {S39, 0, &V22502_320_57290, &P22502_344};	/*Property = PostMessage*/
const Value V22502_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22502_321 = {S40, 0, &V22502_321_57293, &P22502_320};	/*Property = Progress*/
const Value V22502_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22502_322 = {S41, 0, &V22502_322_57296, &P22502_321};	/*Property = Prompts*/
const Value V22502_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22502_323 = {S42, 0, &V22502_323_57299, &P22502_322};	/*Property = Selector*/
const Value V22502_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22502_324 = {S43, 0, &V22502_324_57305, &P22502_323};	/*Property = Svg*/
const Value V22502_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22502_327 = {S44, 0, &V22502_327_57315, &P22502_324};	/*Property = TouchEvents*/
const Value V22502_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22502_328 = {S45, 0, &V22502_328_57318, &P22502_327};	/*Property = Track*/
const Value V22502_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22502_329 = {S46, 0, &V22502_329_57323, &P22502_328};	/*Property = Video*/
const Value V22502_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22502_330 = {S47, 0, &V22502_330_57326, &P22502_329};	/*Property = Viewport*/
const Value V22502_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22502_332 = {S48, 0, &V22502_332_57338, &P22502_330};	/*Property = WebWorkers*/
const Value V22502_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22502_333 = {S49, 0, &V22502_333_57341, &P22502_332};	/*Property = Xhr2*/
const Profile P22502 = {"22502", &P22502_333};	/*Profile ID - 22502*/
const Profile P23101 = {"23101", NULL};	/*Profile ID - 23101*/
const Value V17583_1_1 = {S2, NULL};	/*Value = True*/
const Property P17583_1 = {S50, 0, &V17583_1_1, NULL};	/*Property = IsMobile*/
const Value V17583_5_4286 = {S73, NULL};	/*Value = 432*/
const Property P17583_5 = {S52, 0, &V17583_5_4286, &P17583_1};	/*Property = ScreenPixelsHeight*/
const Value V17583_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17583_4 = {S54, 0, &V17583_4_1224, &P17583_5};	/*Property = ScreenPixelsWidth*/
const Profile P17583 = {"17583", &P17583_4};	/*Profile ID - 17583*/
const Value V20907_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20907_279 = {S1, 0, &V20907_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20907_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20907_282 = {S3, 0, &V20907_282_57154, &P20907_279};	/*Property = BlobBuilder*/
const Value V20907_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20907_350 = {S4, 0, &V20907_350_57444, &P20907_282};	/*Property = Canvas*/
const Value V20907_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20907_283 = {S5, 0, &V20907_283_57159, &P20907_350};	/*Property = CssBackground*/
const Value V20907_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20907_284 = {S6, 0, &V20907_284_57163, &P20907_283};	/*Property = CssBorderImage*/
const Value V20907_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20907_285 = {S7, 0, &V20907_285_57166, &P20907_284};	/*Property = CssCanvas*/
const Value V20907_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20907_286 = {S8, 0, &V20907_286_57168, &P20907_285};	/*Property = CssColor*/
const Value V20907_237_55785 = {S0, NULL};	/*Value = False*/
const Property P20907_237 = {S9, 0, &V20907_237_55785, &P20907_286};	/*Property = CssColumn*/
const Value V20907_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20907_287 = {S10, 0, &V20907_287_57172, &P20907_237};	/*Property = CssFlexbox*/
const Value V20907_288_57175 = {S0, NULL};	/*Value = False*/
const Property P20907_288 = {S11, 0, &V20907_288_57175, &P20907_287};	/*Property = CssFont*/
const Value V20907_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20907_289 = {S12, 0, &V20907_289_57759, &P20907_288};	/*Property = CssImages*/
const Value V20907_291_57181 = {S0, NULL};	/*Value = False*/
const Property P20907_291 = {S13, 0, &V20907_291_57181, &P20907_289};	/*Property = CssMediaQueries*/
const Value V20907_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20907_292 = {S14, 0, &V20907_292_57183, &P20907_291};	/*Property = CssMinMax*/
const Value V20907_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20907_293 = {S15, 0, &V20907_293_57187, &P20907_292};	/*Property = CssOverflow*/
const Value V20907_294_57190 = {S0, NULL};	/*Value = False*/
const Property P20907_294 = {S16, 0, &V20907_294_57190, &P20907_293};	/*Property = CssPosition*/
const Value V20907_295_57192 = {S2, NULL};	/*Value = True*/
const Property P20907_295 = {S17, 0, &V20907_295_57192, &P20907_294};	/*Property = CssText*/
const Value V20907_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20907_297 = {S18, 0, &V20907_297_57195, &P20907_295};	/*Property = CssTransforms*/
const Value V20907_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20907_241 = {S19, 0, &V20907_241_55799, &P20907_297};	/*Property = CssTransitions*/
const Value V20907_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20907_298 = {S20, 0, &V20907_298_57206, &P20907_241};	/*Property = CssUI*/
const Value V20907_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20907_300 = {S21, 0, &V20907_300_57216, &P20907_298};	/*Property = DataSet*/
const Value V20907_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20907_301 = {S22, 0, &V20907_301_57218, &P20907_300};	/*Property = DataUrl*/
const Value V20907_302_57222 = {S0, NULL};	/*Value = False*/
const Property P20907_302 = {S23, 0, &V20907_302_57222, &P20907_301};	/*Property = DeviceOrientation*/
const Value V20907_304_57228 = {S0, NULL};	/*Value = False*/
const Property P20907_304 = {S24, 0, &V20907_304_57228, &P20907_302};	/*Property = FileReader*/
const Value V20907_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20907_305 = {S25, 0, &V20907_305_57231, &P20907_304};	/*Property = FileSaver*/
const Value V20907_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20907_306 = {S26, 0, &V20907_306_57237, &P20907_305};	/*Property = FileWriter*/
const Value V20907_307_57240 = {S0, NULL};	/*Value = False*/
const Property P20907_307 = {S27, 0, &V20907_307_57240, &P20907_306};	/*Property = FormData*/
const Value V20907_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20907_309 = {S28, 0, &V20907_309_57246, &P20907_307};	/*Property = Fullscreen*/
const Value V20907_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20907_310 = {S29, 0, &V20907_310_57248, &P20907_309};	/*Property = GeoLocation*/
const Value V20907_312_57255 = {S0, NULL};	/*Value = False*/
const Property P20907_312 = {S30, 0, &V20907_312_57255, &P20907_310};	/*Property = History*/
const Value V20907_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20907_314 = {S31, 0, &V20907_314_57261, &P20907_312};	/*Property = Html5*/
const Value V20907_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20907_313 = {S32, 0, &V20907_313_57258, &P20907_314};	/*Property = Html-Media-Capture*/
const Value V20907_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20907_315 = {S33, 0, &V20907_315_57264, &P20907_313};	/*Property = Iframe*/
const Value V20907_316_57267 = {S0, NULL};	/*Value = False*/
const Property P20907_316 = {S34, 0, &V20907_316_57267, &P20907_315};	/*Property = IndexedDB*/
const Value V20907_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20907_317 = {S35, 0, &V20907_317_57269, &P20907_316};	/*Property = Json*/
const Value V20907_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P20907_15 = {S37, 0, &V20907_15_30, &P20907_317};	/*Property = LayoutEngine*/
const Value V20907_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20907_344 = {S38, 0, &V20907_344_57272, &P20907_15};	/*Property = Masking*/
const Value V20907_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20907_320 = {S39, 0, &V20907_320_57290, &P20907_344};	/*Property = PostMessage*/
const Value V20907_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20907_321 = {S40, 0, &V20907_321_57293, &P20907_320};	/*Property = Progress*/
const Value V20907_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20907_322 = {S41, 0, &V20907_322_57296, &P20907_321};	/*Property = Prompts*/
const Value V20907_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20907_323 = {S42, 0, &V20907_323_57299, &P20907_322};	/*Property = Selector*/
const Value V20907_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20907_324 = {S43, 0, &V20907_324_57306, &P20907_323};	/*Property = Svg*/
const Value V20907_327_57315 = {S0, NULL};	/*Value = False*/
const Property P20907_327 = {S44, 0, &V20907_327_57315, &P20907_324};	/*Property = TouchEvents*/
const Value V20907_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20907_328 = {S45, 0, &V20907_328_57318, &P20907_327};	/*Property = Track*/
const Value V20907_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20907_329 = {S46, 0, &V20907_329_57323, &P20907_328};	/*Property = Video*/
const Value V20907_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20907_330 = {S47, 0, &V20907_330_57326, &P20907_329};	/*Property = Viewport*/
const Value V20907_332_57339 = {S0, NULL};	/*Value = False*/
const Property P20907_332 = {S48, 0, &V20907_332_57339, &P20907_330};	/*Property = WebWorkers*/
const Value V20907_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20907_333 = {S49, 0, &V20907_333_57342, &P20907_332};	/*Property = Xhr2*/
const Profile P20907 = {"20907", &P20907_333};	/*Profile ID - 20907*/
const Value V16146_1_1 = {S2, NULL};	/*Value = True*/
const Property P16146_1 = {S50, 0, &V16146_1_1, NULL};	/*Property = IsMobile*/
const Value V16146_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P16146_5 = {S52, 0, &V16146_5_1269, &P16146_1};	/*Property = ScreenPixelsHeight*/
const Value V16146_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16146_4 = {S54, 0, &V16146_4_1229, &P16146_5};	/*Property = ScreenPixelsWidth*/
const Profile P16146 = {"16146", &P16146_4};	/*Profile ID - 16146*/
const Value V22499_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_279 = {S1, 0, &V22499_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22499_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_282 = {S3, 0, &V22499_282_57155, &P22499_279};	/*Property = BlobBuilder*/
const Value V22499_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_350 = {S4, 0, &V22499_350_57446, &P22499_282};	/*Property = Canvas*/
const Value V22499_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_283 = {S5, 0, &V22499_283_57161, &P22499_350};	/*Property = CssBackground*/
const Value V22499_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_284 = {S6, 0, &V22499_284_57164, &P22499_283};	/*Property = CssBorderImage*/
const Value V22499_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_285 = {S7, 0, &V22499_285_57167, &P22499_284};	/*Property = CssCanvas*/
const Value V22499_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_286 = {S8, 0, &V22499_286_57170, &P22499_285};	/*Property = CssColor*/
const Value V22499_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_237 = {S9, 0, &V22499_237_55786, &P22499_286};	/*Property = CssColumn*/
const Value V22499_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_287 = {S10, 0, &V22499_287_57173, &P22499_237};	/*Property = CssFlexbox*/
const Value V22499_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_288 = {S11, 0, &V22499_288_57176, &P22499_287};	/*Property = CssFont*/
const Value V22499_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_289 = {S12, 0, &V22499_289_86899, &P22499_288};	/*Property = CssImages*/
const Value V22499_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_291 = {S13, 0, &V22499_291_57182, &P22499_289};	/*Property = CssMediaQueries*/
const Value V22499_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_292 = {S14, 0, &V22499_292_57185, &P22499_291};	/*Property = CssMinMax*/
const Value V22499_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_293 = {S15, 0, &V22499_293_57188, &P22499_292};	/*Property = CssOverflow*/
const Value V22499_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_294 = {S16, 0, &V22499_294_57191, &P22499_293};	/*Property = CssPosition*/
const Value V22499_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_295 = {S17, 0, &V22499_295_57194, &P22499_294};	/*Property = CssText*/
const Value V22499_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_297 = {S18, 0, &V22499_297_57197, &P22499_295};	/*Property = CssTransforms*/
const Value V22499_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_241 = {S19, 0, &V22499_241_55801, &P22499_297};	/*Property = CssTransitions*/
const Value V22499_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_298 = {S20, 0, &V22499_298_57208, &P22499_241};	/*Property = CssUI*/
const Value V22499_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_300 = {S21, 0, &V22499_300_57217, &P22499_298};	/*Property = DataSet*/
const Value V22499_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_301 = {S22, 0, &V22499_301_57220, &P22499_300};	/*Property = DataUrl*/
const Value V22499_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_302 = {S23, 0, &V22499_302_57223, &P22499_301};	/*Property = DeviceOrientation*/
const Value V22499_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_304 = {S24, 0, &V22499_304_57229, &P22499_302};	/*Property = FileReader*/
const Value V22499_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_305 = {S25, 0, &V22499_305_57232, &P22499_304};	/*Property = FileSaver*/
const Value V22499_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_306 = {S26, 0, &V22499_306_57238, &P22499_305};	/*Property = FileWriter*/
const Value V22499_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_307 = {S27, 0, &V22499_307_57241, &P22499_306};	/*Property = FormData*/
const Value V22499_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_309 = {S28, 0, &V22499_309_57247, &P22499_307};	/*Property = Fullscreen*/
const Value V22499_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_310 = {S29, 0, &V22499_310_57250, &P22499_309};	/*Property = GeoLocation*/
const Value V22499_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_312 = {S30, 0, &V22499_312_57256, &P22499_310};	/*Property = History*/
const Value V22499_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_314 = {S31, 0, &V22499_314_57262, &P22499_312};	/*Property = Html5*/
const Value V22499_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_313 = {S32, 0, &V22499_313_57259, &P22499_314};	/*Property = Html-Media-Capture*/
const Value V22499_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_315 = {S33, 0, &V22499_315_57265, &P22499_313};	/*Property = Iframe*/
const Value V22499_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_316 = {S34, 0, &V22499_316_57268, &P22499_315};	/*Property = IndexedDB*/
const Value V22499_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_317 = {S35, 0, &V22499_317_57271, &P22499_316};	/*Property = Json*/
const Value V22499_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22499_15 = {S37, 0, &V22499_15_29, &P22499_317};	/*Property = LayoutEngine*/
const Value V22499_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_344 = {S38, 0, &V22499_344_57274, &P22499_15};	/*Property = Masking*/
const Value V22499_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_320 = {S39, 0, &V22499_320_57292, &P22499_344};	/*Property = PostMessage*/
const Value V22499_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_321 = {S40, 0, &V22499_321_57295, &P22499_320};	/*Property = Progress*/
const Value V22499_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_322 = {S41, 0, &V22499_322_57298, &P22499_321};	/*Property = Prompts*/
const Value V22499_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_323 = {S42, 0, &V22499_323_57301, &P22499_322};	/*Property = Selector*/
const Value V22499_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_324 = {S43, 0, &V22499_324_57307, &P22499_323};	/*Property = Svg*/
const Value V22499_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_327 = {S44, 0, &V22499_327_57316, &P22499_324};	/*Property = TouchEvents*/
const Value V22499_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_328 = {S45, 0, &V22499_328_57319, &P22499_327};	/*Property = Track*/
const Value V22499_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_329 = {S46, 0, &V22499_329_57325, &P22499_328};	/*Property = Video*/
const Value V22499_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_330 = {S47, 0, &V22499_330_57328, &P22499_329};	/*Property = Viewport*/
const Value V22499_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_332 = {S48, 0, &V22499_332_57340, &P22499_330};	/*Property = WebWorkers*/
const Value V22499_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22499_333 = {S49, 0, &V22499_333_57343, &P22499_332};	/*Property = Xhr2*/
const Profile P22499 = {"22499", &P22499_333};	/*Profile ID - 22499*/
const Profile P18136 = {"18136", NULL};	/*Profile ID - 18136*/
const Value V20797_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_279 = {S1, 0, &V20797_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20797_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_282 = {S3, 0, &V20797_282_57155, &P20797_279};	/*Property = BlobBuilder*/
const Value V20797_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_350 = {S4, 0, &V20797_350_57446, &P20797_282};	/*Property = Canvas*/
const Value V20797_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_283 = {S5, 0, &V20797_283_57161, &P20797_350};	/*Property = CssBackground*/
const Value V20797_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_284 = {S6, 0, &V20797_284_57164, &P20797_283};	/*Property = CssBorderImage*/
const Value V20797_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_285 = {S7, 0, &V20797_285_57167, &P20797_284};	/*Property = CssCanvas*/
const Value V20797_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_286 = {S8, 0, &V20797_286_57170, &P20797_285};	/*Property = CssColor*/
const Value V20797_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_237 = {S9, 0, &V20797_237_55786, &P20797_286};	/*Property = CssColumn*/
const Value V20797_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_287 = {S10, 0, &V20797_287_57173, &P20797_237};	/*Property = CssFlexbox*/
const Value V20797_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_288 = {S11, 0, &V20797_288_57176, &P20797_287};	/*Property = CssFont*/
const Value V20797_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_289 = {S12, 0, &V20797_289_86899, &P20797_288};	/*Property = CssImages*/
const Value V20797_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_291 = {S13, 0, &V20797_291_57182, &P20797_289};	/*Property = CssMediaQueries*/
const Value V20797_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_292 = {S14, 0, &V20797_292_57185, &P20797_291};	/*Property = CssMinMax*/
const Value V20797_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_293 = {S15, 0, &V20797_293_57188, &P20797_292};	/*Property = CssOverflow*/
const Value V20797_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_294 = {S16, 0, &V20797_294_57191, &P20797_293};	/*Property = CssPosition*/
const Value V20797_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_295 = {S17, 0, &V20797_295_57194, &P20797_294};	/*Property = CssText*/
const Value V20797_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_297 = {S18, 0, &V20797_297_57197, &P20797_295};	/*Property = CssTransforms*/
const Value V20797_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_241 = {S19, 0, &V20797_241_55801, &P20797_297};	/*Property = CssTransitions*/
const Value V20797_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_298 = {S20, 0, &V20797_298_57208, &P20797_241};	/*Property = CssUI*/
const Value V20797_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_300 = {S21, 0, &V20797_300_57217, &P20797_298};	/*Property = DataSet*/
const Value V20797_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_301 = {S22, 0, &V20797_301_57220, &P20797_300};	/*Property = DataUrl*/
const Value V20797_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_302 = {S23, 0, &V20797_302_57223, &P20797_301};	/*Property = DeviceOrientation*/
const Value V20797_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_304 = {S24, 0, &V20797_304_57229, &P20797_302};	/*Property = FileReader*/
const Value V20797_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_305 = {S25, 0, &V20797_305_57232, &P20797_304};	/*Property = FileSaver*/
const Value V20797_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_306 = {S26, 0, &V20797_306_57238, &P20797_305};	/*Property = FileWriter*/
const Value V20797_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_307 = {S27, 0, &V20797_307_57241, &P20797_306};	/*Property = FormData*/
const Value V20797_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_309 = {S28, 0, &V20797_309_57247, &P20797_307};	/*Property = Fullscreen*/
const Value V20797_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_310 = {S29, 0, &V20797_310_57250, &P20797_309};	/*Property = GeoLocation*/
const Value V20797_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_312 = {S30, 0, &V20797_312_57256, &P20797_310};	/*Property = History*/
const Value V20797_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_314 = {S31, 0, &V20797_314_57262, &P20797_312};	/*Property = Html5*/
const Value V20797_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_313 = {S32, 0, &V20797_313_57259, &P20797_314};	/*Property = Html-Media-Capture*/
const Value V20797_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_315 = {S33, 0, &V20797_315_57265, &P20797_313};	/*Property = Iframe*/
const Value V20797_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_316 = {S34, 0, &V20797_316_57268, &P20797_315};	/*Property = IndexedDB*/
const Value V20797_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_317 = {S35, 0, &V20797_317_57271, &P20797_316};	/*Property = Json*/
const Value V20797_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20797_15 = {S37, 0, &V20797_15_29, &P20797_317};	/*Property = LayoutEngine*/
const Value V20797_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_344 = {S38, 0, &V20797_344_57274, &P20797_15};	/*Property = Masking*/
const Value V20797_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_320 = {S39, 0, &V20797_320_57292, &P20797_344};	/*Property = PostMessage*/
const Value V20797_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_321 = {S40, 0, &V20797_321_57295, &P20797_320};	/*Property = Progress*/
const Value V20797_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_322 = {S41, 0, &V20797_322_57298, &P20797_321};	/*Property = Prompts*/
const Value V20797_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_323 = {S42, 0, &V20797_323_57301, &P20797_322};	/*Property = Selector*/
const Value V20797_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_324 = {S43, 0, &V20797_324_57307, &P20797_323};	/*Property = Svg*/
const Value V20797_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_327 = {S44, 0, &V20797_327_57316, &P20797_324};	/*Property = TouchEvents*/
const Value V20797_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_328 = {S45, 0, &V20797_328_57319, &P20797_327};	/*Property = Track*/
const Value V20797_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_329 = {S46, 0, &V20797_329_57325, &P20797_328};	/*Property = Video*/
const Value V20797_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_330 = {S47, 0, &V20797_330_57328, &P20797_329};	/*Property = Viewport*/
const Value V20797_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_332 = {S48, 0, &V20797_332_57340, &P20797_330};	/*Property = WebWorkers*/
const Value V20797_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20797_333 = {S49, 0, &V20797_333_57343, &P20797_332};	/*Property = Xhr2*/
const Profile P20797 = {"20797", &P20797_333};	/*Profile ID - 20797*/
const Value V22536_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22536_279 = {S1, 0, &V22536_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22536_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22536_282 = {S3, 0, &V22536_282_57153, &P22536_279};	/*Property = BlobBuilder*/
const Value V22536_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22536_350 = {S4, 0, &V22536_350_57444, &P22536_282};	/*Property = Canvas*/
const Value V22536_283_57160 = {S0, NULL};	/*Value = False*/
const Property P22536_283 = {S5, 0, &V22536_283_57160, &P22536_350};	/*Property = CssBackground*/
const Value V22536_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22536_284 = {S6, 0, &V22536_284_57163, &P22536_283};	/*Property = CssBorderImage*/
const Value V22536_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22536_285 = {S7, 0, &V22536_285_57166, &P22536_284};	/*Property = CssCanvas*/
const Value V22536_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22536_286 = {S8, 0, &V22536_286_57168, &P22536_285};	/*Property = CssColor*/
const Value V22536_237_55785 = {S0, NULL};	/*Value = False*/
const Property P22536_237 = {S9, 0, &V22536_237_55785, &P22536_286};	/*Property = CssColumn*/
const Value V22536_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22536_287 = {S10, 0, &V22536_287_57172, &P22536_237};	/*Property = CssFlexbox*/
const Value V22536_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22536_288 = {S11, 0, &V22536_288_57175, &P22536_287};	/*Property = CssFont*/
const Value V22536_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22536_289 = {S12, 0, &V22536_289_86899, &P22536_288};	/*Property = CssImages*/
const Value V22536_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22536_291 = {S13, 0, &V22536_291_57181, &P22536_289};	/*Property = CssMediaQueries*/
const Value V22536_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22536_292 = {S14, 0, &V22536_292_57183, &P22536_291};	/*Property = CssMinMax*/
const Value V22536_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22536_293 = {S15, 0, &V22536_293_57187, &P22536_292};	/*Property = CssOverflow*/
const Value V22536_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22536_294 = {S16, 0, &V22536_294_57189, &P22536_293};	/*Property = CssPosition*/
const Value V22536_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22536_295 = {S17, 0, &V22536_295_57193, &P22536_294};	/*Property = CssText*/
const Value V22536_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22536_297 = {S18, 0, &V22536_297_57195, &P22536_295};	/*Property = CssTransforms*/
const Value V22536_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22536_241 = {S19, 0, &V22536_241_55799, &P22536_297};	/*Property = CssTransitions*/
const Value V22536_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22536_298 = {S20, 0, &V22536_298_57206, &P22536_241};	/*Property = CssUI*/
const Value V22536_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22536_300 = {S21, 0, &V22536_300_57215, &P22536_298};	/*Property = DataSet*/
const Value V22536_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22536_301 = {S22, 0, &V22536_301_57218, &P22536_300};	/*Property = DataUrl*/
const Value V22536_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22536_302 = {S23, 0, &V22536_302_57221, &P22536_301};	/*Property = DeviceOrientation*/
const Value V22536_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22536_304 = {S24, 0, &V22536_304_57227, &P22536_302};	/*Property = FileReader*/
const Value V22536_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22536_305 = {S25, 0, &V22536_305_57231, &P22536_304};	/*Property = FileSaver*/
const Value V22536_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22536_306 = {S26, 0, &V22536_306_57237, &P22536_305};	/*Property = FileWriter*/
const Value V22536_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22536_307 = {S27, 0, &V22536_307_57239, &P22536_306};	/*Property = FormData*/
const Value V22536_309_57245 = {S2, NULL};	/*Value = True*/
const Property P22536_309 = {S28, 0, &V22536_309_57245, &P22536_307};	/*Property = Fullscreen*/
const Value V22536_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22536_310 = {S29, 0, &V22536_310_57248, &P22536_309};	/*Property = GeoLocation*/
const Value V22536_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22536_312 = {S30, 0, &V22536_312_57254, &P22536_310};	/*Property = History*/
const Value V22536_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22536_314 = {S31, 0, &V22536_314_57261, &P22536_312};	/*Property = Html5*/
const Value V22536_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22536_313 = {S32, 0, &V22536_313_57258, &P22536_314};	/*Property = Html-Media-Capture*/
const Value V22536_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22536_315 = {S33, 0, &V22536_315_57264, &P22536_313};	/*Property = Iframe*/
const Value V22536_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22536_316 = {S34, 0, &V22536_316_57266, &P22536_315};	/*Property = IndexedDB*/
const Value V22536_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22536_317 = {S35, 0, &V22536_317_57269, &P22536_316};	/*Property = Json*/
const Value V22536_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22536_15 = {S37, 0, &V22536_15_29, &P22536_317};	/*Property = LayoutEngine*/
const Value V22536_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22536_344 = {S38, 0, &V22536_344_57272, &P22536_15};	/*Property = Masking*/
const Value V22536_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22536_320 = {S39, 0, &V22536_320_57290, &P22536_344};	/*Property = PostMessage*/
const Value V22536_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22536_321 = {S40, 0, &V22536_321_57293, &P22536_320};	/*Property = Progress*/
const Value V22536_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22536_322 = {S41, 0, &V22536_322_57296, &P22536_321};	/*Property = Prompts*/
const Value V22536_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22536_323 = {S42, 0, &V22536_323_57299, &P22536_322};	/*Property = Selector*/
const Value V22536_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22536_324 = {S43, 0, &V22536_324_57305, &P22536_323};	/*Property = Svg*/
const Value V22536_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22536_327 = {S44, 0, &V22536_327_57315, &P22536_324};	/*Property = TouchEvents*/
const Value V22536_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22536_328 = {S45, 0, &V22536_328_57318, &P22536_327};	/*Property = Track*/
const Value V22536_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22536_329 = {S46, 0, &V22536_329_57323, &P22536_328};	/*Property = Video*/
const Value V22536_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22536_330 = {S47, 0, &V22536_330_57326, &P22536_329};	/*Property = Viewport*/
const Value V22536_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22536_332 = {S48, 0, &V22536_332_57339, &P22536_330};	/*Property = WebWorkers*/
const Value V22536_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22536_333 = {S49, 0, &V22536_333_57342, &P22536_332};	/*Property = Xhr2*/
const Profile P22536 = {"22536", &P22536_333};	/*Profile ID - 22536*/
const Value V22478_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22478_279 = {S1, 0, &V22478_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22478_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22478_282 = {S3, 0, &V22478_282_57154, &P22478_279};	/*Property = BlobBuilder*/
const Value V22478_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22478_350 = {S4, 0, &V22478_350_57444, &P22478_282};	/*Property = Canvas*/
const Value V22478_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22478_283 = {S5, 0, &V22478_283_57159, &P22478_350};	/*Property = CssBackground*/
const Value V22478_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22478_284 = {S6, 0, &V22478_284_57163, &P22478_283};	/*Property = CssBorderImage*/
const Value V22478_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22478_285 = {S7, 0, &V22478_285_57166, &P22478_284};	/*Property = CssCanvas*/
const Value V22478_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22478_286 = {S8, 0, &V22478_286_57168, &P22478_285};	/*Property = CssColor*/
const Value V22478_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22478_237 = {S9, 0, &V22478_237_55784, &P22478_286};	/*Property = CssColumn*/
const Value V22478_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22478_287 = {S10, 0, &V22478_287_57172, &P22478_237};	/*Property = CssFlexbox*/
const Value V22478_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22478_288 = {S11, 0, &V22478_288_57174, &P22478_287};	/*Property = CssFont*/
const Value V22478_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22478_289 = {S12, 0, &V22478_289_57759, &P22478_288};	/*Property = CssImages*/
const Value V22478_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22478_291 = {S13, 0, &V22478_291_57180, &P22478_289};	/*Property = CssMediaQueries*/
const Value V22478_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22478_292 = {S14, 0, &V22478_292_57183, &P22478_291};	/*Property = CssMinMax*/
const Value V22478_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22478_293 = {S15, 0, &V22478_293_57187, &P22478_292};	/*Property = CssOverflow*/
const Value V22478_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22478_294 = {S16, 0, &V22478_294_57189, &P22478_293};	/*Property = CssPosition*/
const Value V22478_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22478_295 = {S17, 0, &V22478_295_57192, &P22478_294};	/*Property = CssText*/
const Value V22478_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22478_297 = {S18, 0, &V22478_297_57195, &P22478_295};	/*Property = CssTransforms*/
const Value V22478_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22478_241 = {S19, 0, &V22478_241_55799, &P22478_297};	/*Property = CssTransitions*/
const Value V22478_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22478_298 = {S20, 0, &V22478_298_57206, &P22478_241};	/*Property = CssUI*/
const Value V22478_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22478_300 = {S21, 0, &V22478_300_57215, &P22478_298};	/*Property = DataSet*/
const Value V22478_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22478_301 = {S22, 0, &V22478_301_57218, &P22478_300};	/*Property = DataUrl*/
const Value V22478_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22478_302 = {S23, 0, &V22478_302_57221, &P22478_301};	/*Property = DeviceOrientation*/
const Value V22478_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22478_304 = {S24, 0, &V22478_304_57227, &P22478_302};	/*Property = FileReader*/
const Value V22478_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22478_305 = {S25, 0, &V22478_305_57231, &P22478_304};	/*Property = FileSaver*/
const Value V22478_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22478_306 = {S26, 0, &V22478_306_57237, &P22478_305};	/*Property = FileWriter*/
const Value V22478_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22478_307 = {S27, 0, &V22478_307_57239, &P22478_306};	/*Property = FormData*/
const Value V22478_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22478_309 = {S28, 0, &V22478_309_57246, &P22478_307};	/*Property = Fullscreen*/
const Value V22478_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22478_310 = {S29, 0, &V22478_310_57248, &P22478_309};	/*Property = GeoLocation*/
const Value V22478_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22478_312 = {S30, 0, &V22478_312_57254, &P22478_310};	/*Property = History*/
const Value V22478_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22478_314 = {S31, 0, &V22478_314_57261, &P22478_312};	/*Property = Html5*/
const Value V22478_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22478_313 = {S32, 0, &V22478_313_57258, &P22478_314};	/*Property = Html-Media-Capture*/
const Value V22478_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22478_315 = {S33, 0, &V22478_315_57264, &P22478_313};	/*Property = Iframe*/
const Value V22478_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22478_316 = {S34, 0, &V22478_316_57266, &P22478_315};	/*Property = IndexedDB*/
const Value V22478_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22478_317 = {S35, 0, &V22478_317_57269, &P22478_316};	/*Property = Json*/
const Value V22478_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22478_15 = {S37, 0, &V22478_15_29, &P22478_317};	/*Property = LayoutEngine*/
const Value V22478_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22478_344 = {S38, 0, &V22478_344_57272, &P22478_15};	/*Property = Masking*/
const Value V22478_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22478_320 = {S39, 0, &V22478_320_57290, &P22478_344};	/*Property = PostMessage*/
const Value V22478_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22478_321 = {S40, 0, &V22478_321_57293, &P22478_320};	/*Property = Progress*/
const Value V22478_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22478_322 = {S41, 0, &V22478_322_57296, &P22478_321};	/*Property = Prompts*/
const Value V22478_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22478_323 = {S42, 0, &V22478_323_57299, &P22478_322};	/*Property = Selector*/
const Value V22478_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22478_324 = {S43, 0, &V22478_324_57305, &P22478_323};	/*Property = Svg*/
const Value V22478_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22478_327 = {S44, 0, &V22478_327_57315, &P22478_324};	/*Property = TouchEvents*/
const Value V22478_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22478_328 = {S45, 0, &V22478_328_57318, &P22478_327};	/*Property = Track*/
const Value V22478_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22478_329 = {S46, 0, &V22478_329_57323, &P22478_328};	/*Property = Video*/
const Value V22478_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22478_330 = {S47, 0, &V22478_330_57326, &P22478_329};	/*Property = Viewport*/
const Value V22478_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22478_332 = {S48, 0, &V22478_332_57338, &P22478_330};	/*Property = WebWorkers*/
const Value V22478_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22478_333 = {S49, 0, &V22478_333_57341, &P22478_332};	/*Property = Xhr2*/
const Profile P22478 = {"22478", &P22478_333};	/*Profile ID - 22478*/
const Value V16958_1_1 = {S2, NULL};	/*Value = True*/
const Property P16958_1 = {S50, 0, &V16958_1_1, NULL};	/*Property = IsMobile*/
const Value V16958_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16958_5 = {S52, 0, &V16958_5_1222, &P16958_1};	/*Property = ScreenPixelsHeight*/
const Value V16958_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16958_4 = {S54, 0, &V16958_4_1279, &P16958_5};	/*Property = ScreenPixelsWidth*/
const Profile P16958 = {"16958", &P16958_4};	/*Profile ID - 16958*/
const Value V17883_1_1 = {S2, NULL};	/*Value = True*/
const Property P17883_1 = {S50, 0, &V17883_1_1, NULL};	/*Property = IsMobile*/
const Value V17883_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17883_5 = {S52, 0, &V17883_5_1227, &P17883_1};	/*Property = ScreenPixelsHeight*/
const Value V17883_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P17883_4 = {S54, 0, &V17883_4_1943, &P17883_5};	/*Property = ScreenPixelsWidth*/
const Profile P17883 = {"17883", &P17883_4};	/*Profile ID - 17883*/
const Value V21408_1_1 = {S2, NULL};	/*Value = True*/
const Property P21408_1 = {S50, 0, &V21408_1_1, NULL};	/*Property = IsMobile*/
const Value V21408_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21408_5 = {S52, 0, &V21408_5_1269, &P21408_1};	/*Property = ScreenPixelsHeight*/
const Value V21408_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21408_4 = {S54, 0, &V21408_4_1229, &P21408_5};	/*Property = ScreenPixelsWidth*/
const Profile P21408 = {"21408", &P21408_4};	/*Profile ID - 21408*/
const Value V16039_1_1 = {S2, NULL};	/*Value = True*/
const Property P16039_1 = {S50, 0, &V16039_1_1, NULL};	/*Property = IsMobile*/
const Value V16039_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16039_5 = {S52, 0, &V16039_5_1227, &P16039_1};	/*Property = ScreenPixelsHeight*/
const Value V16039_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16039_4 = {S54, 0, &V16039_4_1229, &P16039_5};	/*Property = ScreenPixelsWidth*/
const Profile P16039 = {"16039", &P16039_4};	/*Profile ID - 16039*/
const Value V21372_1_1 = {S2, NULL};	/*Value = True*/
const Property P21372_1 = {S50, 0, &V21372_1_1, NULL};	/*Property = IsMobile*/
const Value V21372_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21372_5 = {S52, 0, &V21372_5_1227, &P21372_1};	/*Property = ScreenPixelsHeight*/
const Value V21372_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21372_4 = {S54, 0, &V21372_4_1229, &P21372_5};	/*Property = ScreenPixelsWidth*/
const Profile P21372 = {"21372", &P21372_4};	/*Profile ID - 21372*/
const Value V23703_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_279 = {S1, 0, &V23703_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23703_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_282 = {S3, 0, &V23703_282_57155, &P23703_279};	/*Property = BlobBuilder*/
const Value V23703_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_350 = {S4, 0, &V23703_350_57446, &P23703_282};	/*Property = Canvas*/
const Value V23703_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_283 = {S5, 0, &V23703_283_57161, &P23703_350};	/*Property = CssBackground*/
const Value V23703_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_284 = {S6, 0, &V23703_284_57164, &P23703_283};	/*Property = CssBorderImage*/
const Value V23703_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_285 = {S7, 0, &V23703_285_57167, &P23703_284};	/*Property = CssCanvas*/
const Value V23703_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_286 = {S8, 0, &V23703_286_57170, &P23703_285};	/*Property = CssColor*/
const Value V23703_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_237 = {S9, 0, &V23703_237_55786, &P23703_286};	/*Property = CssColumn*/
const Value V23703_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_287 = {S10, 0, &V23703_287_57173, &P23703_237};	/*Property = CssFlexbox*/
const Value V23703_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_288 = {S11, 0, &V23703_288_57176, &P23703_287};	/*Property = CssFont*/
const Value V23703_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_289 = {S12, 0, &V23703_289_86899, &P23703_288};	/*Property = CssImages*/
const Value V23703_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_291 = {S13, 0, &V23703_291_57182, &P23703_289};	/*Property = CssMediaQueries*/
const Value V23703_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_292 = {S14, 0, &V23703_292_57185, &P23703_291};	/*Property = CssMinMax*/
const Value V23703_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_293 = {S15, 0, &V23703_293_57188, &P23703_292};	/*Property = CssOverflow*/
const Value V23703_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_294 = {S16, 0, &V23703_294_57191, &P23703_293};	/*Property = CssPosition*/
const Value V23703_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_295 = {S17, 0, &V23703_295_57194, &P23703_294};	/*Property = CssText*/
const Value V23703_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_297 = {S18, 0, &V23703_297_57197, &P23703_295};	/*Property = CssTransforms*/
const Value V23703_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_241 = {S19, 0, &V23703_241_55801, &P23703_297};	/*Property = CssTransitions*/
const Value V23703_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_298 = {S20, 0, &V23703_298_57208, &P23703_241};	/*Property = CssUI*/
const Value V23703_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_300 = {S21, 0, &V23703_300_57217, &P23703_298};	/*Property = DataSet*/
const Value V23703_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_301 = {S22, 0, &V23703_301_57220, &P23703_300};	/*Property = DataUrl*/
const Value V23703_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_302 = {S23, 0, &V23703_302_57223, &P23703_301};	/*Property = DeviceOrientation*/
const Value V23703_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_304 = {S24, 0, &V23703_304_57229, &P23703_302};	/*Property = FileReader*/
const Value V23703_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_305 = {S25, 0, &V23703_305_57232, &P23703_304};	/*Property = FileSaver*/
const Value V23703_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_306 = {S26, 0, &V23703_306_57238, &P23703_305};	/*Property = FileWriter*/
const Value V23703_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_307 = {S27, 0, &V23703_307_57241, &P23703_306};	/*Property = FormData*/
const Value V23703_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_309 = {S28, 0, &V23703_309_57247, &P23703_307};	/*Property = Fullscreen*/
const Value V23703_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_310 = {S29, 0, &V23703_310_57250, &P23703_309};	/*Property = GeoLocation*/
const Value V23703_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_312 = {S30, 0, &V23703_312_57256, &P23703_310};	/*Property = History*/
const Value V23703_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_314 = {S31, 0, &V23703_314_57262, &P23703_312};	/*Property = Html5*/
const Value V23703_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_313 = {S32, 0, &V23703_313_57259, &P23703_314};	/*Property = Html-Media-Capture*/
const Value V23703_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_315 = {S33, 0, &V23703_315_57265, &P23703_313};	/*Property = Iframe*/
const Value V23703_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_316 = {S34, 0, &V23703_316_57268, &P23703_315};	/*Property = IndexedDB*/
const Value V23703_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_317 = {S35, 0, &V23703_317_57271, &P23703_316};	/*Property = Json*/
const Value V23703_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P23703_15 = {S37, 0, &V23703_15_29, &P23703_317};	/*Property = LayoutEngine*/
const Value V23703_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_344 = {S38, 0, &V23703_344_57274, &P23703_15};	/*Property = Masking*/
const Value V23703_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_320 = {S39, 0, &V23703_320_57292, &P23703_344};	/*Property = PostMessage*/
const Value V23703_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_321 = {S40, 0, &V23703_321_57295, &P23703_320};	/*Property = Progress*/
const Value V23703_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_322 = {S41, 0, &V23703_322_57298, &P23703_321};	/*Property = Prompts*/
const Value V23703_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_323 = {S42, 0, &V23703_323_57301, &P23703_322};	/*Property = Selector*/
const Value V23703_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_324 = {S43, 0, &V23703_324_57307, &P23703_323};	/*Property = Svg*/
const Value V23703_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_327 = {S44, 0, &V23703_327_57316, &P23703_324};	/*Property = TouchEvents*/
const Value V23703_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_328 = {S45, 0, &V23703_328_57319, &P23703_327};	/*Property = Track*/
const Value V23703_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_329 = {S46, 0, &V23703_329_57325, &P23703_328};	/*Property = Video*/
const Value V23703_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_330 = {S47, 0, &V23703_330_57328, &P23703_329};	/*Property = Viewport*/
const Value V23703_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_332 = {S48, 0, &V23703_332_57340, &P23703_330};	/*Property = WebWorkers*/
const Value V23703_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23703_333 = {S49, 0, &V23703_333_57343, &P23703_332};	/*Property = Xhr2*/
const Profile P23703 = {"23703", &P23703_333};	/*Profile ID - 23703*/
const Value V22526_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_279 = {S1, 0, &V22526_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22526_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_282 = {S3, 0, &V22526_282_57155, &P22526_279};	/*Property = BlobBuilder*/
const Value V22526_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_350 = {S4, 0, &V22526_350_57446, &P22526_282};	/*Property = Canvas*/
const Value V22526_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_283 = {S5, 0, &V22526_283_57161, &P22526_350};	/*Property = CssBackground*/
const Value V22526_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_284 = {S6, 0, &V22526_284_57164, &P22526_283};	/*Property = CssBorderImage*/
const Value V22526_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_285 = {S7, 0, &V22526_285_57167, &P22526_284};	/*Property = CssCanvas*/
const Value V22526_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_286 = {S8, 0, &V22526_286_57170, &P22526_285};	/*Property = CssColor*/
const Value V22526_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_237 = {S9, 0, &V22526_237_55786, &P22526_286};	/*Property = CssColumn*/
const Value V22526_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_287 = {S10, 0, &V22526_287_57173, &P22526_237};	/*Property = CssFlexbox*/
const Value V22526_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_288 = {S11, 0, &V22526_288_57176, &P22526_287};	/*Property = CssFont*/
const Value V22526_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_289 = {S12, 0, &V22526_289_86899, &P22526_288};	/*Property = CssImages*/
const Value V22526_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_291 = {S13, 0, &V22526_291_57182, &P22526_289};	/*Property = CssMediaQueries*/
const Value V22526_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_292 = {S14, 0, &V22526_292_57185, &P22526_291};	/*Property = CssMinMax*/
const Value V22526_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_293 = {S15, 0, &V22526_293_57188, &P22526_292};	/*Property = CssOverflow*/
const Value V22526_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_294 = {S16, 0, &V22526_294_57191, &P22526_293};	/*Property = CssPosition*/
const Value V22526_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_295 = {S17, 0, &V22526_295_57194, &P22526_294};	/*Property = CssText*/
const Value V22526_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_297 = {S18, 0, &V22526_297_57197, &P22526_295};	/*Property = CssTransforms*/
const Value V22526_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_241 = {S19, 0, &V22526_241_55801, &P22526_297};	/*Property = CssTransitions*/
const Value V22526_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_298 = {S20, 0, &V22526_298_57208, &P22526_241};	/*Property = CssUI*/
const Value V22526_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_300 = {S21, 0, &V22526_300_57217, &P22526_298};	/*Property = DataSet*/
const Value V22526_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_301 = {S22, 0, &V22526_301_57220, &P22526_300};	/*Property = DataUrl*/
const Value V22526_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_302 = {S23, 0, &V22526_302_57223, &P22526_301};	/*Property = DeviceOrientation*/
const Value V22526_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_304 = {S24, 0, &V22526_304_57229, &P22526_302};	/*Property = FileReader*/
const Value V22526_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_305 = {S25, 0, &V22526_305_57232, &P22526_304};	/*Property = FileSaver*/
const Value V22526_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_306 = {S26, 0, &V22526_306_57238, &P22526_305};	/*Property = FileWriter*/
const Value V22526_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_307 = {S27, 0, &V22526_307_57241, &P22526_306};	/*Property = FormData*/
const Value V22526_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_309 = {S28, 0, &V22526_309_57247, &P22526_307};	/*Property = Fullscreen*/
const Value V22526_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_310 = {S29, 0, &V22526_310_57250, &P22526_309};	/*Property = GeoLocation*/
const Value V22526_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_312 = {S30, 0, &V22526_312_57256, &P22526_310};	/*Property = History*/
const Value V22526_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_314 = {S31, 0, &V22526_314_57262, &P22526_312};	/*Property = Html5*/
const Value V22526_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_313 = {S32, 0, &V22526_313_57259, &P22526_314};	/*Property = Html-Media-Capture*/
const Value V22526_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_315 = {S33, 0, &V22526_315_57265, &P22526_313};	/*Property = Iframe*/
const Value V22526_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_316 = {S34, 0, &V22526_316_57268, &P22526_315};	/*Property = IndexedDB*/
const Value V22526_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_317 = {S35, 0, &V22526_317_57271, &P22526_316};	/*Property = Json*/
const Value V22526_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22526_15 = {S37, 0, &V22526_15_28, &P22526_317};	/*Property = LayoutEngine*/
const Value V22526_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_344 = {S38, 0, &V22526_344_57274, &P22526_15};	/*Property = Masking*/
const Value V22526_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_320 = {S39, 0, &V22526_320_57292, &P22526_344};	/*Property = PostMessage*/
const Value V22526_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_321 = {S40, 0, &V22526_321_57295, &P22526_320};	/*Property = Progress*/
const Value V22526_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_322 = {S41, 0, &V22526_322_57298, &P22526_321};	/*Property = Prompts*/
const Value V22526_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_323 = {S42, 0, &V22526_323_57301, &P22526_322};	/*Property = Selector*/
const Value V22526_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_324 = {S43, 0, &V22526_324_57307, &P22526_323};	/*Property = Svg*/
const Value V22526_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_327 = {S44, 0, &V22526_327_57316, &P22526_324};	/*Property = TouchEvents*/
const Value V22526_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_328 = {S45, 0, &V22526_328_57319, &P22526_327};	/*Property = Track*/
const Value V22526_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_329 = {S46, 0, &V22526_329_57325, &P22526_328};	/*Property = Video*/
const Value V22526_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_330 = {S47, 0, &V22526_330_57328, &P22526_329};	/*Property = Viewport*/
const Value V22526_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_332 = {S48, 0, &V22526_332_57340, &P22526_330};	/*Property = WebWorkers*/
const Value V22526_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22526_333 = {S49, 0, &V22526_333_57343, &P22526_332};	/*Property = Xhr2*/
const Profile P22526 = {"22526", &P22526_333};	/*Profile ID - 22526*/
const Value V20501_1_1 = {S2, NULL};	/*Value = True*/
const Property P20501_1 = {S50, 0, &V20501_1_1, NULL};	/*Property = IsMobile*/
const Value V20501_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20501_5 = {S52, 0, &V20501_5_1277, &P20501_1};	/*Property = ScreenPixelsHeight*/
const Value V20501_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20501_4 = {S54, 0, &V20501_4_1279, &P20501_5};	/*Property = ScreenPixelsWidth*/
const Profile P20501 = {"20501", &P20501_4};	/*Profile ID - 20501*/
const Value V20902_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20902_279 = {S1, 0, &V20902_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20902_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20902_282 = {S3, 0, &V20902_282_57154, &P20902_279};	/*Property = BlobBuilder*/
const Value V20902_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20902_350 = {S4, 0, &V20902_350_57444, &P20902_282};	/*Property = Canvas*/
const Value V20902_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20902_283 = {S5, 0, &V20902_283_57159, &P20902_350};	/*Property = CssBackground*/
const Value V20902_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20902_284 = {S6, 0, &V20902_284_57163, &P20902_283};	/*Property = CssBorderImage*/
const Value V20902_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20902_285 = {S7, 0, &V20902_285_57166, &P20902_284};	/*Property = CssCanvas*/
const Value V20902_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20902_286 = {S8, 0, &V20902_286_57168, &P20902_285};	/*Property = CssColor*/
const Value V20902_237_55785 = {S0, NULL};	/*Value = False*/
const Property P20902_237 = {S9, 0, &V20902_237_55785, &P20902_286};	/*Property = CssColumn*/
const Value V20902_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20902_287 = {S10, 0, &V20902_287_57172, &P20902_237};	/*Property = CssFlexbox*/
const Value V20902_288_57175 = {S0, NULL};	/*Value = False*/
const Property P20902_288 = {S11, 0, &V20902_288_57175, &P20902_287};	/*Property = CssFont*/
const Value V20902_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20902_289 = {S12, 0, &V20902_289_57759, &P20902_288};	/*Property = CssImages*/
const Value V20902_291_57181 = {S0, NULL};	/*Value = False*/
const Property P20902_291 = {S13, 0, &V20902_291_57181, &P20902_289};	/*Property = CssMediaQueries*/
const Value V20902_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20902_292 = {S14, 0, &V20902_292_57183, &P20902_291};	/*Property = CssMinMax*/
const Value V20902_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20902_293 = {S15, 0, &V20902_293_57187, &P20902_292};	/*Property = CssOverflow*/
const Value V20902_294_57190 = {S0, NULL};	/*Value = False*/
const Property P20902_294 = {S16, 0, &V20902_294_57190, &P20902_293};	/*Property = CssPosition*/
const Value V20902_295_57192 = {S2, NULL};	/*Value = True*/
const Property P20902_295 = {S17, 0, &V20902_295_57192, &P20902_294};	/*Property = CssText*/
const Value V20902_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20902_297 = {S18, 0, &V20902_297_57195, &P20902_295};	/*Property = CssTransforms*/
const Value V20902_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20902_241 = {S19, 0, &V20902_241_55799, &P20902_297};	/*Property = CssTransitions*/
const Value V20902_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20902_298 = {S20, 0, &V20902_298_57206, &P20902_241};	/*Property = CssUI*/
const Value V20902_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20902_300 = {S21, 0, &V20902_300_57216, &P20902_298};	/*Property = DataSet*/
const Value V20902_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20902_301 = {S22, 0, &V20902_301_57218, &P20902_300};	/*Property = DataUrl*/
const Value V20902_302_57222 = {S0, NULL};	/*Value = False*/
const Property P20902_302 = {S23, 0, &V20902_302_57222, &P20902_301};	/*Property = DeviceOrientation*/
const Value V20902_304_57228 = {S0, NULL};	/*Value = False*/
const Property P20902_304 = {S24, 0, &V20902_304_57228, &P20902_302};	/*Property = FileReader*/
const Value V20902_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20902_305 = {S25, 0, &V20902_305_57231, &P20902_304};	/*Property = FileSaver*/
const Value V20902_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20902_306 = {S26, 0, &V20902_306_57237, &P20902_305};	/*Property = FileWriter*/
const Value V20902_307_57240 = {S0, NULL};	/*Value = False*/
const Property P20902_307 = {S27, 0, &V20902_307_57240, &P20902_306};	/*Property = FormData*/
const Value V20902_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20902_309 = {S28, 0, &V20902_309_57246, &P20902_307};	/*Property = Fullscreen*/
const Value V20902_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20902_310 = {S29, 0, &V20902_310_57248, &P20902_309};	/*Property = GeoLocation*/
const Value V20902_312_57255 = {S0, NULL};	/*Value = False*/
const Property P20902_312 = {S30, 0, &V20902_312_57255, &P20902_310};	/*Property = History*/
const Value V20902_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20902_314 = {S31, 0, &V20902_314_57261, &P20902_312};	/*Property = Html5*/
const Value V20902_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20902_313 = {S32, 0, &V20902_313_57258, &P20902_314};	/*Property = Html-Media-Capture*/
const Value V20902_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20902_315 = {S33, 0, &V20902_315_57264, &P20902_313};	/*Property = Iframe*/
const Value V20902_316_57267 = {S0, NULL};	/*Value = False*/
const Property P20902_316 = {S34, 0, &V20902_316_57267, &P20902_315};	/*Property = IndexedDB*/
const Value V20902_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20902_317 = {S35, 0, &V20902_317_57269, &P20902_316};	/*Property = Json*/
const Value V20902_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P20902_15 = {S37, 0, &V20902_15_30, &P20902_317};	/*Property = LayoutEngine*/
const Value V20902_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20902_344 = {S38, 0, &V20902_344_57272, &P20902_15};	/*Property = Masking*/
const Value V20902_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20902_320 = {S39, 0, &V20902_320_57290, &P20902_344};	/*Property = PostMessage*/
const Value V20902_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20902_321 = {S40, 0, &V20902_321_57293, &P20902_320};	/*Property = Progress*/
const Value V20902_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20902_322 = {S41, 0, &V20902_322_57296, &P20902_321};	/*Property = Prompts*/
const Value V20902_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20902_323 = {S42, 0, &V20902_323_57299, &P20902_322};	/*Property = Selector*/
const Value V20902_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20902_324 = {S43, 0, &V20902_324_57306, &P20902_323};	/*Property = Svg*/
const Value V20902_327_57315 = {S0, NULL};	/*Value = False*/
const Property P20902_327 = {S44, 0, &V20902_327_57315, &P20902_324};	/*Property = TouchEvents*/
const Value V20902_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20902_328 = {S45, 0, &V20902_328_57318, &P20902_327};	/*Property = Track*/
const Value V20902_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20902_329 = {S46, 0, &V20902_329_57323, &P20902_328};	/*Property = Video*/
const Value V20902_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20902_330 = {S47, 0, &V20902_330_57326, &P20902_329};	/*Property = Viewport*/
const Value V20902_332_57339 = {S0, NULL};	/*Value = False*/
const Property P20902_332 = {S48, 0, &V20902_332_57339, &P20902_330};	/*Property = WebWorkers*/
const Value V20902_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20902_333 = {S49, 0, &V20902_333_57342, &P20902_332};	/*Property = Xhr2*/
const Profile P20902 = {"20902", &P20902_333};	/*Profile ID - 20902*/
const Profile P5567 = {"5567", NULL};	/*Profile ID - 5567*/
const Profile P18132 = {"18132", NULL};	/*Profile ID - 18132*/
const Value V22531_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22531_279 = {S1, 0, &V22531_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22531_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22531_282 = {S3, 0, &V22531_282_57153, &P22531_279};	/*Property = BlobBuilder*/
const Value V22531_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22531_350 = {S4, 0, &V22531_350_57444, &P22531_282};	/*Property = Canvas*/
const Value V22531_283_57160 = {S0, NULL};	/*Value = False*/
const Property P22531_283 = {S5, 0, &V22531_283_57160, &P22531_350};	/*Property = CssBackground*/
const Value V22531_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22531_284 = {S6, 0, &V22531_284_57163, &P22531_283};	/*Property = CssBorderImage*/
const Value V22531_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22531_285 = {S7, 0, &V22531_285_57166, &P22531_284};	/*Property = CssCanvas*/
const Value V22531_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22531_286 = {S8, 0, &V22531_286_57168, &P22531_285};	/*Property = CssColor*/
const Value V22531_237_55785 = {S0, NULL};	/*Value = False*/
const Property P22531_237 = {S9, 0, &V22531_237_55785, &P22531_286};	/*Property = CssColumn*/
const Value V22531_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22531_287 = {S10, 0, &V22531_287_57172, &P22531_237};	/*Property = CssFlexbox*/
const Value V22531_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22531_288 = {S11, 0, &V22531_288_57175, &P22531_287};	/*Property = CssFont*/
const Value V22531_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22531_289 = {S12, 0, &V22531_289_86899, &P22531_288};	/*Property = CssImages*/
const Value V22531_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22531_291 = {S13, 0, &V22531_291_57181, &P22531_289};	/*Property = CssMediaQueries*/
const Value V22531_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22531_292 = {S14, 0, &V22531_292_57183, &P22531_291};	/*Property = CssMinMax*/
const Value V22531_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22531_293 = {S15, 0, &V22531_293_57187, &P22531_292};	/*Property = CssOverflow*/
const Value V22531_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22531_294 = {S16, 0, &V22531_294_57189, &P22531_293};	/*Property = CssPosition*/
const Value V22531_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22531_295 = {S17, 0, &V22531_295_57193, &P22531_294};	/*Property = CssText*/
const Value V22531_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22531_297 = {S18, 0, &V22531_297_57195, &P22531_295};	/*Property = CssTransforms*/
const Value V22531_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22531_241 = {S19, 0, &V22531_241_55799, &P22531_297};	/*Property = CssTransitions*/
const Value V22531_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22531_298 = {S20, 0, &V22531_298_57206, &P22531_241};	/*Property = CssUI*/
const Value V22531_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22531_300 = {S21, 0, &V22531_300_57215, &P22531_298};	/*Property = DataSet*/
const Value V22531_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22531_301 = {S22, 0, &V22531_301_57218, &P22531_300};	/*Property = DataUrl*/
const Value V22531_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22531_302 = {S23, 0, &V22531_302_57221, &P22531_301};	/*Property = DeviceOrientation*/
const Value V22531_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22531_304 = {S24, 0, &V22531_304_57227, &P22531_302};	/*Property = FileReader*/
const Value V22531_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22531_305 = {S25, 0, &V22531_305_57231, &P22531_304};	/*Property = FileSaver*/
const Value V22531_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22531_306 = {S26, 0, &V22531_306_57237, &P22531_305};	/*Property = FileWriter*/
const Value V22531_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22531_307 = {S27, 0, &V22531_307_57239, &P22531_306};	/*Property = FormData*/
const Value V22531_309_57245 = {S2, NULL};	/*Value = True*/
const Property P22531_309 = {S28, 0, &V22531_309_57245, &P22531_307};	/*Property = Fullscreen*/
const Value V22531_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22531_310 = {S29, 0, &V22531_310_57248, &P22531_309};	/*Property = GeoLocation*/
const Value V22531_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22531_312 = {S30, 0, &V22531_312_57254, &P22531_310};	/*Property = History*/
const Value V22531_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22531_314 = {S31, 0, &V22531_314_57261, &P22531_312};	/*Property = Html5*/
const Value V22531_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22531_313 = {S32, 0, &V22531_313_57258, &P22531_314};	/*Property = Html-Media-Capture*/
const Value V22531_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22531_315 = {S33, 0, &V22531_315_57264, &P22531_313};	/*Property = Iframe*/
const Value V22531_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22531_316 = {S34, 0, &V22531_316_57266, &P22531_315};	/*Property = IndexedDB*/
const Value V22531_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22531_317 = {S35, 0, &V22531_317_57269, &P22531_316};	/*Property = Json*/
const Value V22531_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22531_15 = {S37, 0, &V22531_15_29, &P22531_317};	/*Property = LayoutEngine*/
const Value V22531_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22531_344 = {S38, 0, &V22531_344_57272, &P22531_15};	/*Property = Masking*/
const Value V22531_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22531_320 = {S39, 0, &V22531_320_57290, &P22531_344};	/*Property = PostMessage*/
const Value V22531_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22531_321 = {S40, 0, &V22531_321_57293, &P22531_320};	/*Property = Progress*/
const Value V22531_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22531_322 = {S41, 0, &V22531_322_57296, &P22531_321};	/*Property = Prompts*/
const Value V22531_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22531_323 = {S42, 0, &V22531_323_57299, &P22531_322};	/*Property = Selector*/
const Value V22531_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22531_324 = {S43, 0, &V22531_324_57305, &P22531_323};	/*Property = Svg*/
const Value V22531_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22531_327 = {S44, 0, &V22531_327_57315, &P22531_324};	/*Property = TouchEvents*/
const Value V22531_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22531_328 = {S45, 0, &V22531_328_57318, &P22531_327};	/*Property = Track*/
const Value V22531_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22531_329 = {S46, 0, &V22531_329_57323, &P22531_328};	/*Property = Video*/
const Value V22531_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22531_330 = {S47, 0, &V22531_330_57326, &P22531_329};	/*Property = Viewport*/
const Value V22531_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22531_332 = {S48, 0, &V22531_332_57339, &P22531_330};	/*Property = WebWorkers*/
const Value V22531_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22531_333 = {S49, 0, &V22531_333_57342, &P22531_332};	/*Property = Xhr2*/
const Profile P22531 = {"22531", &P22531_333};	/*Profile ID - 22531*/
const Value V21803_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_279 = {S1, 0, &V21803_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21803_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_282 = {S3, 0, &V21803_282_57155, &P21803_279};	/*Property = BlobBuilder*/
const Value V21803_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_350 = {S4, 0, &V21803_350_57446, &P21803_282};	/*Property = Canvas*/
const Value V21803_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_283 = {S5, 0, &V21803_283_57161, &P21803_350};	/*Property = CssBackground*/
const Value V21803_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_284 = {S6, 0, &V21803_284_57164, &P21803_283};	/*Property = CssBorderImage*/
const Value V21803_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_285 = {S7, 0, &V21803_285_57167, &P21803_284};	/*Property = CssCanvas*/
const Value V21803_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_286 = {S8, 0, &V21803_286_57170, &P21803_285};	/*Property = CssColor*/
const Value V21803_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_237 = {S9, 0, &V21803_237_55786, &P21803_286};	/*Property = CssColumn*/
const Value V21803_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_287 = {S10, 0, &V21803_287_57173, &P21803_237};	/*Property = CssFlexbox*/
const Value V21803_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_288 = {S11, 0, &V21803_288_57176, &P21803_287};	/*Property = CssFont*/
const Value V21803_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_289 = {S12, 0, &V21803_289_86899, &P21803_288};	/*Property = CssImages*/
const Value V21803_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_291 = {S13, 0, &V21803_291_57182, &P21803_289};	/*Property = CssMediaQueries*/
const Value V21803_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_292 = {S14, 0, &V21803_292_57185, &P21803_291};	/*Property = CssMinMax*/
const Value V21803_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_293 = {S15, 0, &V21803_293_57188, &P21803_292};	/*Property = CssOverflow*/
const Value V21803_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_294 = {S16, 0, &V21803_294_57191, &P21803_293};	/*Property = CssPosition*/
const Value V21803_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_295 = {S17, 0, &V21803_295_57194, &P21803_294};	/*Property = CssText*/
const Value V21803_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_297 = {S18, 0, &V21803_297_57197, &P21803_295};	/*Property = CssTransforms*/
const Value V21803_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_241 = {S19, 0, &V21803_241_55801, &P21803_297};	/*Property = CssTransitions*/
const Value V21803_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_298 = {S20, 0, &V21803_298_57208, &P21803_241};	/*Property = CssUI*/
const Value V21803_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_300 = {S21, 0, &V21803_300_57217, &P21803_298};	/*Property = DataSet*/
const Value V21803_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_301 = {S22, 0, &V21803_301_57220, &P21803_300};	/*Property = DataUrl*/
const Value V21803_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_302 = {S23, 0, &V21803_302_57223, &P21803_301};	/*Property = DeviceOrientation*/
const Value V21803_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_304 = {S24, 0, &V21803_304_57229, &P21803_302};	/*Property = FileReader*/
const Value V21803_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_305 = {S25, 0, &V21803_305_57232, &P21803_304};	/*Property = FileSaver*/
const Value V21803_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_306 = {S26, 0, &V21803_306_57238, &P21803_305};	/*Property = FileWriter*/
const Value V21803_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_307 = {S27, 0, &V21803_307_57241, &P21803_306};	/*Property = FormData*/
const Value V21803_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_309 = {S28, 0, &V21803_309_57247, &P21803_307};	/*Property = Fullscreen*/
const Value V21803_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_310 = {S29, 0, &V21803_310_57250, &P21803_309};	/*Property = GeoLocation*/
const Value V21803_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_312 = {S30, 0, &V21803_312_57256, &P21803_310};	/*Property = History*/
const Value V21803_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_314 = {S31, 0, &V21803_314_57262, &P21803_312};	/*Property = Html5*/
const Value V21803_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_313 = {S32, 0, &V21803_313_57259, &P21803_314};	/*Property = Html-Media-Capture*/
const Value V21803_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_315 = {S33, 0, &V21803_315_57265, &P21803_313};	/*Property = Iframe*/
const Value V21803_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_316 = {S34, 0, &V21803_316_57268, &P21803_315};	/*Property = IndexedDB*/
const Value V21803_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_317 = {S35, 0, &V21803_317_57271, &P21803_316};	/*Property = Json*/
const Value V21803_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P21803_15 = {S37, 0, &V21803_15_29, &P21803_317};	/*Property = LayoutEngine*/
const Value V21803_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_344 = {S38, 0, &V21803_344_57274, &P21803_15};	/*Property = Masking*/
const Value V21803_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_320 = {S39, 0, &V21803_320_57292, &P21803_344};	/*Property = PostMessage*/
const Value V21803_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_321 = {S40, 0, &V21803_321_57295, &P21803_320};	/*Property = Progress*/
const Value V21803_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_322 = {S41, 0, &V21803_322_57298, &P21803_321};	/*Property = Prompts*/
const Value V21803_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_323 = {S42, 0, &V21803_323_57301, &P21803_322};	/*Property = Selector*/
const Value V21803_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_324 = {S43, 0, &V21803_324_57307, &P21803_323};	/*Property = Svg*/
const Value V21803_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_327 = {S44, 0, &V21803_327_57316, &P21803_324};	/*Property = TouchEvents*/
const Value V21803_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_328 = {S45, 0, &V21803_328_57319, &P21803_327};	/*Property = Track*/
const Value V21803_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_329 = {S46, 0, &V21803_329_57325, &P21803_328};	/*Property = Video*/
const Value V21803_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_330 = {S47, 0, &V21803_330_57328, &P21803_329};	/*Property = Viewport*/
const Value V21803_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_332 = {S48, 0, &V21803_332_57340, &P21803_330};	/*Property = WebWorkers*/
const Value V21803_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21803_333 = {S49, 0, &V21803_333_57343, &P21803_332};	/*Property = Xhr2*/
const Profile P21803 = {"21803", &P21803_333};	/*Profile ID - 21803*/
const Profile P22474 = {"22474", NULL};	/*Profile ID - 22474*/
const Value V21801_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_279 = {S1, 0, &V21801_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21801_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_282 = {S3, 0, &V21801_282_57155, &P21801_279};	/*Property = BlobBuilder*/
const Value V21801_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_350 = {S4, 0, &V21801_350_57446, &P21801_282};	/*Property = Canvas*/
const Value V21801_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_283 = {S5, 0, &V21801_283_57161, &P21801_350};	/*Property = CssBackground*/
const Value V21801_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_284 = {S6, 0, &V21801_284_57164, &P21801_283};	/*Property = CssBorderImage*/
const Value V21801_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_285 = {S7, 0, &V21801_285_57167, &P21801_284};	/*Property = CssCanvas*/
const Value V21801_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_286 = {S8, 0, &V21801_286_57170, &P21801_285};	/*Property = CssColor*/
const Value V21801_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_237 = {S9, 0, &V21801_237_55786, &P21801_286};	/*Property = CssColumn*/
const Value V21801_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_287 = {S10, 0, &V21801_287_57173, &P21801_237};	/*Property = CssFlexbox*/
const Value V21801_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_288 = {S11, 0, &V21801_288_57176, &P21801_287};	/*Property = CssFont*/
const Value V21801_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_289 = {S12, 0, &V21801_289_86899, &P21801_288};	/*Property = CssImages*/
const Value V21801_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_291 = {S13, 0, &V21801_291_57182, &P21801_289};	/*Property = CssMediaQueries*/
const Value V21801_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_292 = {S14, 0, &V21801_292_57185, &P21801_291};	/*Property = CssMinMax*/
const Value V21801_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_293 = {S15, 0, &V21801_293_57188, &P21801_292};	/*Property = CssOverflow*/
const Value V21801_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_294 = {S16, 0, &V21801_294_57191, &P21801_293};	/*Property = CssPosition*/
const Value V21801_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_295 = {S17, 0, &V21801_295_57194, &P21801_294};	/*Property = CssText*/
const Value V21801_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_297 = {S18, 0, &V21801_297_57197, &P21801_295};	/*Property = CssTransforms*/
const Value V21801_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_241 = {S19, 0, &V21801_241_55801, &P21801_297};	/*Property = CssTransitions*/
const Value V21801_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_298 = {S20, 0, &V21801_298_57208, &P21801_241};	/*Property = CssUI*/
const Value V21801_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_300 = {S21, 0, &V21801_300_57217, &P21801_298};	/*Property = DataSet*/
const Value V21801_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_301 = {S22, 0, &V21801_301_57220, &P21801_300};	/*Property = DataUrl*/
const Value V21801_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_302 = {S23, 0, &V21801_302_57223, &P21801_301};	/*Property = DeviceOrientation*/
const Value V21801_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_304 = {S24, 0, &V21801_304_57229, &P21801_302};	/*Property = FileReader*/
const Value V21801_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_305 = {S25, 0, &V21801_305_57232, &P21801_304};	/*Property = FileSaver*/
const Value V21801_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_306 = {S26, 0, &V21801_306_57238, &P21801_305};	/*Property = FileWriter*/
const Value V21801_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_307 = {S27, 0, &V21801_307_57241, &P21801_306};	/*Property = FormData*/
const Value V21801_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_309 = {S28, 0, &V21801_309_57247, &P21801_307};	/*Property = Fullscreen*/
const Value V21801_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_310 = {S29, 0, &V21801_310_57250, &P21801_309};	/*Property = GeoLocation*/
const Value V21801_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_312 = {S30, 0, &V21801_312_57256, &P21801_310};	/*Property = History*/
const Value V21801_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_314 = {S31, 0, &V21801_314_57262, &P21801_312};	/*Property = Html5*/
const Value V21801_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_313 = {S32, 0, &V21801_313_57259, &P21801_314};	/*Property = Html-Media-Capture*/
const Value V21801_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_315 = {S33, 0, &V21801_315_57265, &P21801_313};	/*Property = Iframe*/
const Value V21801_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_316 = {S34, 0, &V21801_316_57268, &P21801_315};	/*Property = IndexedDB*/
const Value V21801_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_317 = {S35, 0, &V21801_317_57271, &P21801_316};	/*Property = Json*/
const Value V21801_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21801_15 = {S37, 0, &V21801_15_54167, &P21801_317};	/*Property = LayoutEngine*/
const Value V21801_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_344 = {S38, 0, &V21801_344_57274, &P21801_15};	/*Property = Masking*/
const Value V21801_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_320 = {S39, 0, &V21801_320_57292, &P21801_344};	/*Property = PostMessage*/
const Value V21801_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_321 = {S40, 0, &V21801_321_57295, &P21801_320};	/*Property = Progress*/
const Value V21801_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_322 = {S41, 0, &V21801_322_57298, &P21801_321};	/*Property = Prompts*/
const Value V21801_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_323 = {S42, 0, &V21801_323_57301, &P21801_322};	/*Property = Selector*/
const Value V21801_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_324 = {S43, 0, &V21801_324_57307, &P21801_323};	/*Property = Svg*/
const Value V21801_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_327 = {S44, 0, &V21801_327_57316, &P21801_324};	/*Property = TouchEvents*/
const Value V21801_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_328 = {S45, 0, &V21801_328_57319, &P21801_327};	/*Property = Track*/
const Value V21801_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_329 = {S46, 0, &V21801_329_57325, &P21801_328};	/*Property = Video*/
const Value V21801_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_330 = {S47, 0, &V21801_330_57328, &P21801_329};	/*Property = Viewport*/
const Value V21801_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_332 = {S48, 0, &V21801_332_57340, &P21801_330};	/*Property = WebWorkers*/
const Value V21801_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21801_333 = {S49, 0, &V21801_333_57343, &P21801_332};	/*Property = Xhr2*/
const Profile P21801 = {"21801", &P21801_333};	/*Profile ID - 21801*/
const Value V17032_1_1 = {S2, NULL};	/*Value = True*/
const Property P17032_1 = {S50, 0, &V17032_1_1, NULL};	/*Property = IsMobile*/
const Value V17032_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17032_5 = {S52, 0, &V17032_5_1269, &P17032_1};	/*Property = ScreenPixelsHeight*/
const Value V17032_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17032_4 = {S54, 0, &V17032_4_1229, &P17032_5};	/*Property = ScreenPixelsWidth*/
const Profile P17032 = {"17032", &P17032_4};	/*Profile ID - 17032*/
const Value V23785_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_279 = {S1, 0, &V23785_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23785_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_282 = {S3, 0, &V23785_282_57155, &P23785_279};	/*Property = BlobBuilder*/
const Value V23785_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_350 = {S4, 0, &V23785_350_57446, &P23785_282};	/*Property = Canvas*/
const Value V23785_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_283 = {S5, 0, &V23785_283_57161, &P23785_350};	/*Property = CssBackground*/
const Value V23785_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_284 = {S6, 0, &V23785_284_57164, &P23785_283};	/*Property = CssBorderImage*/
const Value V23785_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_285 = {S7, 0, &V23785_285_57167, &P23785_284};	/*Property = CssCanvas*/
const Value V23785_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_286 = {S8, 0, &V23785_286_57170, &P23785_285};	/*Property = CssColor*/
const Value V23785_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_237 = {S9, 0, &V23785_237_55786, &P23785_286};	/*Property = CssColumn*/
const Value V23785_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_287 = {S10, 0, &V23785_287_57173, &P23785_237};	/*Property = CssFlexbox*/
const Value V23785_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_288 = {S11, 0, &V23785_288_57176, &P23785_287};	/*Property = CssFont*/
const Value V23785_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_289 = {S12, 0, &V23785_289_86899, &P23785_288};	/*Property = CssImages*/
const Value V23785_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_291 = {S13, 0, &V23785_291_57182, &P23785_289};	/*Property = CssMediaQueries*/
const Value V23785_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_292 = {S14, 0, &V23785_292_57185, &P23785_291};	/*Property = CssMinMax*/
const Value V23785_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_293 = {S15, 0, &V23785_293_57188, &P23785_292};	/*Property = CssOverflow*/
const Value V23785_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_294 = {S16, 0, &V23785_294_57191, &P23785_293};	/*Property = CssPosition*/
const Value V23785_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_295 = {S17, 0, &V23785_295_57194, &P23785_294};	/*Property = CssText*/
const Value V23785_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_297 = {S18, 0, &V23785_297_57197, &P23785_295};	/*Property = CssTransforms*/
const Value V23785_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_241 = {S19, 0, &V23785_241_55801, &P23785_297};	/*Property = CssTransitions*/
const Value V23785_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_298 = {S20, 0, &V23785_298_57208, &P23785_241};	/*Property = CssUI*/
const Value V23785_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_300 = {S21, 0, &V23785_300_57217, &P23785_298};	/*Property = DataSet*/
const Value V23785_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_301 = {S22, 0, &V23785_301_57220, &P23785_300};	/*Property = DataUrl*/
const Value V23785_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_302 = {S23, 0, &V23785_302_57223, &P23785_301};	/*Property = DeviceOrientation*/
const Value V23785_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_304 = {S24, 0, &V23785_304_57229, &P23785_302};	/*Property = FileReader*/
const Value V23785_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_305 = {S25, 0, &V23785_305_57232, &P23785_304};	/*Property = FileSaver*/
const Value V23785_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_306 = {S26, 0, &V23785_306_57238, &P23785_305};	/*Property = FileWriter*/
const Value V23785_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_307 = {S27, 0, &V23785_307_57241, &P23785_306};	/*Property = FormData*/
const Value V23785_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_309 = {S28, 0, &V23785_309_57247, &P23785_307};	/*Property = Fullscreen*/
const Value V23785_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_310 = {S29, 0, &V23785_310_57250, &P23785_309};	/*Property = GeoLocation*/
const Value V23785_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_312 = {S30, 0, &V23785_312_57256, &P23785_310};	/*Property = History*/
const Value V23785_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_314 = {S31, 0, &V23785_314_57262, &P23785_312};	/*Property = Html5*/
const Value V23785_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_313 = {S32, 0, &V23785_313_57259, &P23785_314};	/*Property = Html-Media-Capture*/
const Value V23785_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_315 = {S33, 0, &V23785_315_57265, &P23785_313};	/*Property = Iframe*/
const Value V23785_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_316 = {S34, 0, &V23785_316_57268, &P23785_315};	/*Property = IndexedDB*/
const Value V23785_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_317 = {S35, 0, &V23785_317_57271, &P23785_316};	/*Property = Json*/
const Value V23785_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_15 = {S37, 0, &V23785_15_72, &P23785_317};	/*Property = LayoutEngine*/
const Value V23785_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_344 = {S38, 0, &V23785_344_57274, &P23785_15};	/*Property = Masking*/
const Value V23785_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_320 = {S39, 0, &V23785_320_57292, &P23785_344};	/*Property = PostMessage*/
const Value V23785_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_321 = {S40, 0, &V23785_321_57295, &P23785_320};	/*Property = Progress*/
const Value V23785_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_322 = {S41, 0, &V23785_322_57298, &P23785_321};	/*Property = Prompts*/
const Value V23785_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_323 = {S42, 0, &V23785_323_57301, &P23785_322};	/*Property = Selector*/
const Value V23785_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_324 = {S43, 0, &V23785_324_57307, &P23785_323};	/*Property = Svg*/
const Value V23785_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_327 = {S44, 0, &V23785_327_57316, &P23785_324};	/*Property = TouchEvents*/
const Value V23785_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_328 = {S45, 0, &V23785_328_57319, &P23785_327};	/*Property = Track*/
const Value V23785_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_329 = {S46, 0, &V23785_329_57325, &P23785_328};	/*Property = Video*/
const Value V23785_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_330 = {S47, 0, &V23785_330_57328, &P23785_329};	/*Property = Viewport*/
const Value V23785_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_332 = {S48, 0, &V23785_332_57340, &P23785_330};	/*Property = WebWorkers*/
const Value V23785_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23785_333 = {S49, 0, &V23785_333_57343, &P23785_332};	/*Property = Xhr2*/
const Profile P23785 = {"23785", &P23785_333};	/*Profile ID - 23785*/
const Value V23038_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_279 = {S1, 0, &V23038_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23038_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_282 = {S3, 0, &V23038_282_57155, &P23038_279};	/*Property = BlobBuilder*/
const Value V23038_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_350 = {S4, 0, &V23038_350_57446, &P23038_282};	/*Property = Canvas*/
const Value V23038_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_283 = {S5, 0, &V23038_283_57161, &P23038_350};	/*Property = CssBackground*/
const Value V23038_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_284 = {S6, 0, &V23038_284_57164, &P23038_283};	/*Property = CssBorderImage*/
const Value V23038_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_285 = {S7, 0, &V23038_285_57167, &P23038_284};	/*Property = CssCanvas*/
const Value V23038_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_286 = {S8, 0, &V23038_286_57170, &P23038_285};	/*Property = CssColor*/
const Value V23038_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_237 = {S9, 0, &V23038_237_55786, &P23038_286};	/*Property = CssColumn*/
const Value V23038_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_287 = {S10, 0, &V23038_287_57173, &P23038_237};	/*Property = CssFlexbox*/
const Value V23038_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_288 = {S11, 0, &V23038_288_57176, &P23038_287};	/*Property = CssFont*/
const Value V23038_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_289 = {S12, 0, &V23038_289_86899, &P23038_288};	/*Property = CssImages*/
const Value V23038_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_291 = {S13, 0, &V23038_291_57182, &P23038_289};	/*Property = CssMediaQueries*/
const Value V23038_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_292 = {S14, 0, &V23038_292_57185, &P23038_291};	/*Property = CssMinMax*/
const Value V23038_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_293 = {S15, 0, &V23038_293_57188, &P23038_292};	/*Property = CssOverflow*/
const Value V23038_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_294 = {S16, 0, &V23038_294_57191, &P23038_293};	/*Property = CssPosition*/
const Value V23038_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_295 = {S17, 0, &V23038_295_57194, &P23038_294};	/*Property = CssText*/
const Value V23038_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_297 = {S18, 0, &V23038_297_57197, &P23038_295};	/*Property = CssTransforms*/
const Value V23038_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_241 = {S19, 0, &V23038_241_55801, &P23038_297};	/*Property = CssTransitions*/
const Value V23038_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_298 = {S20, 0, &V23038_298_57208, &P23038_241};	/*Property = CssUI*/
const Value V23038_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_300 = {S21, 0, &V23038_300_57217, &P23038_298};	/*Property = DataSet*/
const Value V23038_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_301 = {S22, 0, &V23038_301_57220, &P23038_300};	/*Property = DataUrl*/
const Value V23038_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_302 = {S23, 0, &V23038_302_57223, &P23038_301};	/*Property = DeviceOrientation*/
const Value V23038_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_304 = {S24, 0, &V23038_304_57229, &P23038_302};	/*Property = FileReader*/
const Value V23038_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_305 = {S25, 0, &V23038_305_57232, &P23038_304};	/*Property = FileSaver*/
const Value V23038_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_306 = {S26, 0, &V23038_306_57238, &P23038_305};	/*Property = FileWriter*/
const Value V23038_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_307 = {S27, 0, &V23038_307_57241, &P23038_306};	/*Property = FormData*/
const Value V23038_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_309 = {S28, 0, &V23038_309_57247, &P23038_307};	/*Property = Fullscreen*/
const Value V23038_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_310 = {S29, 0, &V23038_310_57250, &P23038_309};	/*Property = GeoLocation*/
const Value V23038_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_312 = {S30, 0, &V23038_312_57256, &P23038_310};	/*Property = History*/
const Value V23038_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_314 = {S31, 0, &V23038_314_57262, &P23038_312};	/*Property = Html5*/
const Value V23038_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_313 = {S32, 0, &V23038_313_57259, &P23038_314};	/*Property = Html-Media-Capture*/
const Value V23038_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_315 = {S33, 0, &V23038_315_57265, &P23038_313};	/*Property = Iframe*/
const Value V23038_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_316 = {S34, 0, &V23038_316_57268, &P23038_315};	/*Property = IndexedDB*/
const Value V23038_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_317 = {S35, 0, &V23038_317_57271, &P23038_316};	/*Property = Json*/
const Value V23038_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_15 = {S37, 0, &V23038_15_72, &P23038_317};	/*Property = LayoutEngine*/
const Value V23038_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_344 = {S38, 0, &V23038_344_57274, &P23038_15};	/*Property = Masking*/
const Value V23038_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_320 = {S39, 0, &V23038_320_57292, &P23038_344};	/*Property = PostMessage*/
const Value V23038_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_321 = {S40, 0, &V23038_321_57295, &P23038_320};	/*Property = Progress*/
const Value V23038_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_322 = {S41, 0, &V23038_322_57298, &P23038_321};	/*Property = Prompts*/
const Value V23038_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_323 = {S42, 0, &V23038_323_57301, &P23038_322};	/*Property = Selector*/
const Value V23038_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_324 = {S43, 0, &V23038_324_57307, &P23038_323};	/*Property = Svg*/
const Value V23038_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_327 = {S44, 0, &V23038_327_57316, &P23038_324};	/*Property = TouchEvents*/
const Value V23038_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_328 = {S45, 0, &V23038_328_57319, &P23038_327};	/*Property = Track*/
const Value V23038_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_329 = {S46, 0, &V23038_329_57325, &P23038_328};	/*Property = Video*/
const Value V23038_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_330 = {S47, 0, &V23038_330_57328, &P23038_329};	/*Property = Viewport*/
const Value V23038_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_332 = {S48, 0, &V23038_332_57340, &P23038_330};	/*Property = WebWorkers*/
const Value V23038_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23038_333 = {S49, 0, &V23038_333_57343, &P23038_332};	/*Property = Xhr2*/
const Profile P23038 = {"23038", &P23038_333};	/*Profile ID - 23038*/
const Profile P23780 = {"23780", NULL};	/*Profile ID - 23780*/
const Value V20201_1_1 = {S2, NULL};	/*Value = True*/
const Property P20201_1 = {S50, 0, &V20201_1_1, NULL};	/*Property = IsMobile*/
const Value V20201_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P20201_5 = {S52, 0, &V20201_5_1304, &P20201_1};	/*Property = ScreenPixelsHeight*/
const Value V20201_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20201_4 = {S54, 0, &V20201_4_1224, &P20201_5};	/*Property = ScreenPixelsWidth*/
const Profile P20201 = {"20201", &P20201_4};	/*Profile ID - 20201*/
const Value V16762_1_1 = {S2, NULL};	/*Value = True*/
const Property P16762_1 = {S50, 0, &V16762_1_1, NULL};	/*Property = IsMobile*/
const Value V16762_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16762_5 = {S52, 0, &V16762_5_1277, &P16762_1};	/*Property = ScreenPixelsHeight*/
const Value V16762_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16762_4 = {S54, 0, &V16762_4_1279, &P16762_5};	/*Property = ScreenPixelsWidth*/
const Profile P16762 = {"16762", &P16762_4};	/*Profile ID - 16762*/
const Value V22345_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22345_279 = {S1, 0, &V22345_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22345_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22345_282 = {S3, 0, &V22345_282_57154, &P22345_279};	/*Property = BlobBuilder*/
const Value V22345_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22345_350 = {S4, 0, &V22345_350_57444, &P22345_282};	/*Property = Canvas*/
const Value V22345_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22345_283 = {S5, 0, &V22345_283_57159, &P22345_350};	/*Property = CssBackground*/
const Value V22345_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22345_284 = {S6, 0, &V22345_284_57163, &P22345_283};	/*Property = CssBorderImage*/
const Value V22345_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22345_285 = {S7, 0, &V22345_285_57166, &P22345_284};	/*Property = CssCanvas*/
const Value V22345_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22345_286 = {S8, 0, &V22345_286_57168, &P22345_285};	/*Property = CssColor*/
const Value V22345_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22345_237 = {S9, 0, &V22345_237_55784, &P22345_286};	/*Property = CssColumn*/
const Value V22345_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22345_287 = {S10, 0, &V22345_287_57172, &P22345_237};	/*Property = CssFlexbox*/
const Value V22345_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22345_288 = {S11, 0, &V22345_288_57175, &P22345_287};	/*Property = CssFont*/
const Value V22345_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22345_289 = {S12, 0, &V22345_289_57759, &P22345_288};	/*Property = CssImages*/
const Value V22345_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22345_291 = {S13, 0, &V22345_291_57181, &P22345_289};	/*Property = CssMediaQueries*/
const Value V22345_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22345_292 = {S14, 0, &V22345_292_57183, &P22345_291};	/*Property = CssMinMax*/
const Value V22345_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22345_293 = {S15, 0, &V22345_293_57187, &P22345_292};	/*Property = CssOverflow*/
const Value V22345_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22345_294 = {S16, 0, &V22345_294_57190, &P22345_293};	/*Property = CssPosition*/
const Value V22345_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22345_295 = {S17, 0, &V22345_295_57193, &P22345_294};	/*Property = CssText*/
const Value V22345_297_57196 = {S0, NULL};	/*Value = False*/
const Property P22345_297 = {S18, 0, &V22345_297_57196, &P22345_295};	/*Property = CssTransforms*/
const Value V22345_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22345_241 = {S19, 0, &V22345_241_55799, &P22345_297};	/*Property = CssTransitions*/
const Value V22345_298_57207 = {S0, NULL};	/*Value = False*/
const Property P22345_298 = {S20, 0, &V22345_298_57207, &P22345_241};	/*Property = CssUI*/
const Value V22345_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22345_300 = {S21, 0, &V22345_300_57216, &P22345_298};	/*Property = DataSet*/
const Value V22345_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22345_301 = {S22, 0, &V22345_301_57218, &P22345_300};	/*Property = DataUrl*/
const Value V22345_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22345_302 = {S23, 0, &V22345_302_57222, &P22345_301};	/*Property = DeviceOrientation*/
const Value V22345_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22345_304 = {S24, 0, &V22345_304_57228, &P22345_302};	/*Property = FileReader*/
const Value V22345_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22345_305 = {S25, 0, &V22345_305_57231, &P22345_304};	/*Property = FileSaver*/
const Value V22345_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22345_306 = {S26, 0, &V22345_306_57237, &P22345_305};	/*Property = FileWriter*/
const Value V22345_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22345_307 = {S27, 0, &V22345_307_57240, &P22345_306};	/*Property = FormData*/
const Value V22345_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22345_309 = {S28, 0, &V22345_309_57246, &P22345_307};	/*Property = Fullscreen*/
const Value V22345_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22345_310 = {S29, 0, &V22345_310_57248, &P22345_309};	/*Property = GeoLocation*/
const Value V22345_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22345_312 = {S30, 0, &V22345_312_57255, &P22345_310};	/*Property = History*/
const Value V22345_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22345_314 = {S31, 0, &V22345_314_57261, &P22345_312};	/*Property = Html5*/
const Value V22345_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22345_313 = {S32, 0, &V22345_313_57258, &P22345_314};	/*Property = Html-Media-Capture*/
const Value V22345_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22345_315 = {S33, 0, &V22345_315_57264, &P22345_313};	/*Property = Iframe*/
const Value V22345_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22345_316 = {S34, 0, &V22345_316_57267, &P22345_315};	/*Property = IndexedDB*/
const Value V22345_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22345_317 = {S35, 0, &V22345_317_57269, &P22345_316};	/*Property = Json*/
const Value V22345_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22345_15 = {S37, 0, &V22345_15_29, &P22345_317};	/*Property = LayoutEngine*/
const Value V22345_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22345_344 = {S38, 0, &V22345_344_57272, &P22345_15};	/*Property = Masking*/
const Value V22345_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22345_320 = {S39, 0, &V22345_320_57290, &P22345_344};	/*Property = PostMessage*/
const Value V22345_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22345_321 = {S40, 0, &V22345_321_57293, &P22345_320};	/*Property = Progress*/
const Value V22345_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22345_322 = {S41, 0, &V22345_322_57296, &P22345_321};	/*Property = Prompts*/
const Value V22345_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22345_323 = {S42, 0, &V22345_323_57299, &P22345_322};	/*Property = Selector*/
const Value V22345_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22345_324 = {S43, 0, &V22345_324_57306, &P22345_323};	/*Property = Svg*/
const Value V22345_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22345_327 = {S44, 0, &V22345_327_57315, &P22345_324};	/*Property = TouchEvents*/
const Value V22345_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22345_328 = {S45, 0, &V22345_328_57318, &P22345_327};	/*Property = Track*/
const Value V22345_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22345_329 = {S46, 0, &V22345_329_57323, &P22345_328};	/*Property = Video*/
const Value V22345_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22345_330 = {S47, 0, &V22345_330_57326, &P22345_329};	/*Property = Viewport*/
const Value V22345_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22345_332 = {S48, 0, &V22345_332_57339, &P22345_330};	/*Property = WebWorkers*/
const Value V22345_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22345_333 = {S49, 0, &V22345_333_57342, &P22345_332};	/*Property = Xhr2*/
const Profile P22345 = {"22345", &P22345_333};	/*Profile ID - 22345*/
const Value V15484_1_1 = {S2, NULL};	/*Value = True*/
const Property P15484_1 = {S50, 0, &V15484_1_1, NULL};	/*Property = IsMobile*/
const Value V15484_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P15484_5 = {S52, 0, &V15484_5_1233, &P15484_1};	/*Property = ScreenPixelsHeight*/
const Value V15484_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P15484_4 = {S54, 0, &V15484_4_1235, &P15484_5};	/*Property = ScreenPixelsWidth*/
const Profile P15484 = {"15484", &P15484_4};	/*Profile ID - 15484*/
const Value V22937_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_279 = {S1, 0, &V22937_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22937_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_282 = {S3, 0, &V22937_282_57155, &P22937_279};	/*Property = BlobBuilder*/
const Value V22937_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_350 = {S4, 0, &V22937_350_57446, &P22937_282};	/*Property = Canvas*/
const Value V22937_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_283 = {S5, 0, &V22937_283_57161, &P22937_350};	/*Property = CssBackground*/
const Value V22937_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_284 = {S6, 0, &V22937_284_57164, &P22937_283};	/*Property = CssBorderImage*/
const Value V22937_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_285 = {S7, 0, &V22937_285_57167, &P22937_284};	/*Property = CssCanvas*/
const Value V22937_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_286 = {S8, 0, &V22937_286_57170, &P22937_285};	/*Property = CssColor*/
const Value V22937_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_237 = {S9, 0, &V22937_237_55786, &P22937_286};	/*Property = CssColumn*/
const Value V22937_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_287 = {S10, 0, &V22937_287_57173, &P22937_237};	/*Property = CssFlexbox*/
const Value V22937_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_288 = {S11, 0, &V22937_288_57176, &P22937_287};	/*Property = CssFont*/
const Value V22937_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_289 = {S12, 0, &V22937_289_86899, &P22937_288};	/*Property = CssImages*/
const Value V22937_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_291 = {S13, 0, &V22937_291_57182, &P22937_289};	/*Property = CssMediaQueries*/
const Value V22937_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_292 = {S14, 0, &V22937_292_57185, &P22937_291};	/*Property = CssMinMax*/
const Value V22937_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_293 = {S15, 0, &V22937_293_57188, &P22937_292};	/*Property = CssOverflow*/
const Value V22937_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_294 = {S16, 0, &V22937_294_57191, &P22937_293};	/*Property = CssPosition*/
const Value V22937_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_295 = {S17, 0, &V22937_295_57194, &P22937_294};	/*Property = CssText*/
const Value V22937_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_297 = {S18, 0, &V22937_297_57197, &P22937_295};	/*Property = CssTransforms*/
const Value V22937_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_241 = {S19, 0, &V22937_241_55801, &P22937_297};	/*Property = CssTransitions*/
const Value V22937_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_298 = {S20, 0, &V22937_298_57208, &P22937_241};	/*Property = CssUI*/
const Value V22937_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_300 = {S21, 0, &V22937_300_57217, &P22937_298};	/*Property = DataSet*/
const Value V22937_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_301 = {S22, 0, &V22937_301_57220, &P22937_300};	/*Property = DataUrl*/
const Value V22937_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_302 = {S23, 0, &V22937_302_57223, &P22937_301};	/*Property = DeviceOrientation*/
const Value V22937_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_304 = {S24, 0, &V22937_304_57229, &P22937_302};	/*Property = FileReader*/
const Value V22937_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_305 = {S25, 0, &V22937_305_57232, &P22937_304};	/*Property = FileSaver*/
const Value V22937_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_306 = {S26, 0, &V22937_306_57238, &P22937_305};	/*Property = FileWriter*/
const Value V22937_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_307 = {S27, 0, &V22937_307_57241, &P22937_306};	/*Property = FormData*/
const Value V22937_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_309 = {S28, 0, &V22937_309_57247, &P22937_307};	/*Property = Fullscreen*/
const Value V22937_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_310 = {S29, 0, &V22937_310_57250, &P22937_309};	/*Property = GeoLocation*/
const Value V22937_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_312 = {S30, 0, &V22937_312_57256, &P22937_310};	/*Property = History*/
const Value V22937_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_314 = {S31, 0, &V22937_314_57262, &P22937_312};	/*Property = Html5*/
const Value V22937_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_313 = {S32, 0, &V22937_313_57259, &P22937_314};	/*Property = Html-Media-Capture*/
const Value V22937_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_315 = {S33, 0, &V22937_315_57265, &P22937_313};	/*Property = Iframe*/
const Value V22937_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_316 = {S34, 0, &V22937_316_57268, &P22937_315};	/*Property = IndexedDB*/
const Value V22937_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_317 = {S35, 0, &V22937_317_57271, &P22937_316};	/*Property = Json*/
const Value V22937_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_15 = {S37, 0, &V22937_15_72, &P22937_317};	/*Property = LayoutEngine*/
const Value V22937_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_344 = {S38, 0, &V22937_344_57274, &P22937_15};	/*Property = Masking*/
const Value V22937_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_320 = {S39, 0, &V22937_320_57292, &P22937_344};	/*Property = PostMessage*/
const Value V22937_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_321 = {S40, 0, &V22937_321_57295, &P22937_320};	/*Property = Progress*/
const Value V22937_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_322 = {S41, 0, &V22937_322_57298, &P22937_321};	/*Property = Prompts*/
const Value V22937_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_323 = {S42, 0, &V22937_323_57301, &P22937_322};	/*Property = Selector*/
const Value V22937_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_324 = {S43, 0, &V22937_324_57307, &P22937_323};	/*Property = Svg*/
const Value V22937_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_327 = {S44, 0, &V22937_327_57316, &P22937_324};	/*Property = TouchEvents*/
const Value V22937_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_328 = {S45, 0, &V22937_328_57319, &P22937_327};	/*Property = Track*/
const Value V22937_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_329 = {S46, 0, &V22937_329_57325, &P22937_328};	/*Property = Video*/
const Value V22937_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_330 = {S47, 0, &V22937_330_57328, &P22937_329};	/*Property = Viewport*/
const Value V22937_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_332 = {S48, 0, &V22937_332_57340, &P22937_330};	/*Property = WebWorkers*/
const Value V22937_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22937_333 = {S49, 0, &V22937_333_57343, &P22937_332};	/*Property = Xhr2*/
const Profile P22937 = {"22937", &P22937_333};	/*Profile ID - 22937*/
const Profile P22202 = {"22202", NULL};	/*Profile ID - 22202*/
const Value V24188_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_279 = {S1, 0, &V24188_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24188_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_282 = {S3, 0, &V24188_282_57155, &P24188_279};	/*Property = BlobBuilder*/
const Value V24188_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_350 = {S4, 0, &V24188_350_57446, &P24188_282};	/*Property = Canvas*/
const Value V24188_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_283 = {S5, 0, &V24188_283_57161, &P24188_350};	/*Property = CssBackground*/
const Value V24188_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_284 = {S6, 0, &V24188_284_57164, &P24188_283};	/*Property = CssBorderImage*/
const Value V24188_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_285 = {S7, 0, &V24188_285_57167, &P24188_284};	/*Property = CssCanvas*/
const Value V24188_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_286 = {S8, 0, &V24188_286_57170, &P24188_285};	/*Property = CssColor*/
const Value V24188_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_237 = {S9, 0, &V24188_237_55786, &P24188_286};	/*Property = CssColumn*/
const Value V24188_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_287 = {S10, 0, &V24188_287_57173, &P24188_237};	/*Property = CssFlexbox*/
const Value V24188_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_288 = {S11, 0, &V24188_288_57176, &P24188_287};	/*Property = CssFont*/
const Value V24188_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_289 = {S12, 0, &V24188_289_86899, &P24188_288};	/*Property = CssImages*/
const Value V24188_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_291 = {S13, 0, &V24188_291_57182, &P24188_289};	/*Property = CssMediaQueries*/
const Value V24188_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_292 = {S14, 0, &V24188_292_57185, &P24188_291};	/*Property = CssMinMax*/
const Value V24188_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_293 = {S15, 0, &V24188_293_57188, &P24188_292};	/*Property = CssOverflow*/
const Value V24188_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_294 = {S16, 0, &V24188_294_57191, &P24188_293};	/*Property = CssPosition*/
const Value V24188_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_295 = {S17, 0, &V24188_295_57194, &P24188_294};	/*Property = CssText*/
const Value V24188_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_297 = {S18, 0, &V24188_297_57197, &P24188_295};	/*Property = CssTransforms*/
const Value V24188_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_241 = {S19, 0, &V24188_241_55801, &P24188_297};	/*Property = CssTransitions*/
const Value V24188_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_298 = {S20, 0, &V24188_298_57208, &P24188_241};	/*Property = CssUI*/
const Value V24188_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_300 = {S21, 0, &V24188_300_57217, &P24188_298};	/*Property = DataSet*/
const Value V24188_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_301 = {S22, 0, &V24188_301_57220, &P24188_300};	/*Property = DataUrl*/
const Value V24188_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_302 = {S23, 0, &V24188_302_57223, &P24188_301};	/*Property = DeviceOrientation*/
const Value V24188_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_304 = {S24, 0, &V24188_304_57229, &P24188_302};	/*Property = FileReader*/
const Value V24188_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_305 = {S25, 0, &V24188_305_57232, &P24188_304};	/*Property = FileSaver*/
const Value V24188_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_306 = {S26, 0, &V24188_306_57238, &P24188_305};	/*Property = FileWriter*/
const Value V24188_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_307 = {S27, 0, &V24188_307_57241, &P24188_306};	/*Property = FormData*/
const Value V24188_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_309 = {S28, 0, &V24188_309_57247, &P24188_307};	/*Property = Fullscreen*/
const Value V24188_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_310 = {S29, 0, &V24188_310_57250, &P24188_309};	/*Property = GeoLocation*/
const Value V24188_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_312 = {S30, 0, &V24188_312_57256, &P24188_310};	/*Property = History*/
const Value V24188_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_314 = {S31, 0, &V24188_314_57262, &P24188_312};	/*Property = Html5*/
const Value V24188_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_313 = {S32, 0, &V24188_313_57259, &P24188_314};	/*Property = Html-Media-Capture*/
const Value V24188_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_315 = {S33, 0, &V24188_315_57265, &P24188_313};	/*Property = Iframe*/
const Value V24188_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_316 = {S34, 0, &V24188_316_57268, &P24188_315};	/*Property = IndexedDB*/
const Value V24188_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_317 = {S35, 0, &V24188_317_57271, &P24188_316};	/*Property = Json*/
const Value V24188_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P24188_15 = {S37, 0, &V24188_15_29, &P24188_317};	/*Property = LayoutEngine*/
const Value V24188_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_344 = {S38, 0, &V24188_344_57274, &P24188_15};	/*Property = Masking*/
const Value V24188_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_320 = {S39, 0, &V24188_320_57292, &P24188_344};	/*Property = PostMessage*/
const Value V24188_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_321 = {S40, 0, &V24188_321_57295, &P24188_320};	/*Property = Progress*/
const Value V24188_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_322 = {S41, 0, &V24188_322_57298, &P24188_321};	/*Property = Prompts*/
const Value V24188_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_323 = {S42, 0, &V24188_323_57301, &P24188_322};	/*Property = Selector*/
const Value V24188_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_324 = {S43, 0, &V24188_324_57307, &P24188_323};	/*Property = Svg*/
const Value V24188_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_327 = {S44, 0, &V24188_327_57316, &P24188_324};	/*Property = TouchEvents*/
const Value V24188_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_328 = {S45, 0, &V24188_328_57319, &P24188_327};	/*Property = Track*/
const Value V24188_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_329 = {S46, 0, &V24188_329_57325, &P24188_328};	/*Property = Video*/
const Value V24188_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_330 = {S47, 0, &V24188_330_57328, &P24188_329};	/*Property = Viewport*/
const Value V24188_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_332 = {S48, 0, &V24188_332_57340, &P24188_330};	/*Property = WebWorkers*/
const Value V24188_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24188_333 = {S49, 0, &V24188_333_57343, &P24188_332};	/*Property = Xhr2*/
const Profile P24188 = {"24188", &P24188_333};	/*Profile ID - 24188*/
const Value V22334_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22334_279 = {S1, 0, &V22334_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22334_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22334_282 = {S3, 0, &V22334_282_57154, &P22334_279};	/*Property = BlobBuilder*/
const Value V22334_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22334_350 = {S4, 0, &V22334_350_57444, &P22334_282};	/*Property = Canvas*/
const Value V22334_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22334_283 = {S5, 0, &V22334_283_57159, &P22334_350};	/*Property = CssBackground*/
const Value V22334_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22334_284 = {S6, 0, &V22334_284_57163, &P22334_283};	/*Property = CssBorderImage*/
const Value V22334_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22334_285 = {S7, 0, &V22334_285_57166, &P22334_284};	/*Property = CssCanvas*/
const Value V22334_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22334_286 = {S8, 0, &V22334_286_57168, &P22334_285};	/*Property = CssColor*/
const Value V22334_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22334_237 = {S9, 0, &V22334_237_55784, &P22334_286};	/*Property = CssColumn*/
const Value V22334_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22334_287 = {S10, 0, &V22334_287_57172, &P22334_237};	/*Property = CssFlexbox*/
const Value V22334_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22334_288 = {S11, 0, &V22334_288_57175, &P22334_287};	/*Property = CssFont*/
const Value V22334_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22334_289 = {S12, 0, &V22334_289_57759, &P22334_288};	/*Property = CssImages*/
const Value V22334_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22334_291 = {S13, 0, &V22334_291_57181, &P22334_289};	/*Property = CssMediaQueries*/
const Value V22334_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22334_292 = {S14, 0, &V22334_292_57183, &P22334_291};	/*Property = CssMinMax*/
const Value V22334_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22334_293 = {S15, 0, &V22334_293_57187, &P22334_292};	/*Property = CssOverflow*/
const Value V22334_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22334_294 = {S16, 0, &V22334_294_57190, &P22334_293};	/*Property = CssPosition*/
const Value V22334_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22334_295 = {S17, 0, &V22334_295_57193, &P22334_294};	/*Property = CssText*/
const Value V22334_297_57196 = {S0, NULL};	/*Value = False*/
const Property P22334_297 = {S18, 0, &V22334_297_57196, &P22334_295};	/*Property = CssTransforms*/
const Value V22334_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22334_241 = {S19, 0, &V22334_241_55799, &P22334_297};	/*Property = CssTransitions*/
const Value V22334_298_57207 = {S0, NULL};	/*Value = False*/
const Property P22334_298 = {S20, 0, &V22334_298_57207, &P22334_241};	/*Property = CssUI*/
const Value V22334_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22334_300 = {S21, 0, &V22334_300_57216, &P22334_298};	/*Property = DataSet*/
const Value V22334_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22334_301 = {S22, 0, &V22334_301_57218, &P22334_300};	/*Property = DataUrl*/
const Value V22334_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22334_302 = {S23, 0, &V22334_302_57222, &P22334_301};	/*Property = DeviceOrientation*/
const Value V22334_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22334_304 = {S24, 0, &V22334_304_57228, &P22334_302};	/*Property = FileReader*/
const Value V22334_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22334_305 = {S25, 0, &V22334_305_57231, &P22334_304};	/*Property = FileSaver*/
const Value V22334_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22334_306 = {S26, 0, &V22334_306_57237, &P22334_305};	/*Property = FileWriter*/
const Value V22334_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22334_307 = {S27, 0, &V22334_307_57240, &P22334_306};	/*Property = FormData*/
const Value V22334_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22334_309 = {S28, 0, &V22334_309_57246, &P22334_307};	/*Property = Fullscreen*/
const Value V22334_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22334_310 = {S29, 0, &V22334_310_57248, &P22334_309};	/*Property = GeoLocation*/
const Value V22334_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22334_312 = {S30, 0, &V22334_312_57255, &P22334_310};	/*Property = History*/
const Value V22334_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22334_314 = {S31, 0, &V22334_314_57261, &P22334_312};	/*Property = Html5*/
const Value V22334_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22334_313 = {S32, 0, &V22334_313_57258, &P22334_314};	/*Property = Html-Media-Capture*/
const Value V22334_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22334_315 = {S33, 0, &V22334_315_57264, &P22334_313};	/*Property = Iframe*/
const Value V22334_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22334_316 = {S34, 0, &V22334_316_57267, &P22334_315};	/*Property = IndexedDB*/
const Value V22334_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22334_317 = {S35, 0, &V22334_317_57269, &P22334_316};	/*Property = Json*/
const Value V22334_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22334_15 = {S37, 0, &V22334_15_29, &P22334_317};	/*Property = LayoutEngine*/
const Value V22334_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22334_344 = {S38, 0, &V22334_344_57272, &P22334_15};	/*Property = Masking*/
const Value V22334_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22334_320 = {S39, 0, &V22334_320_57290, &P22334_344};	/*Property = PostMessage*/
const Value V22334_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22334_321 = {S40, 0, &V22334_321_57293, &P22334_320};	/*Property = Progress*/
const Value V22334_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22334_322 = {S41, 0, &V22334_322_57296, &P22334_321};	/*Property = Prompts*/
const Value V22334_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22334_323 = {S42, 0, &V22334_323_57299, &P22334_322};	/*Property = Selector*/
const Value V22334_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22334_324 = {S43, 0, &V22334_324_57306, &P22334_323};	/*Property = Svg*/
const Value V22334_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22334_327 = {S44, 0, &V22334_327_57315, &P22334_324};	/*Property = TouchEvents*/
const Value V22334_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22334_328 = {S45, 0, &V22334_328_57318, &P22334_327};	/*Property = Track*/
const Value V22334_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22334_329 = {S46, 0, &V22334_329_57323, &P22334_328};	/*Property = Video*/
const Value V22334_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22334_330 = {S47, 0, &V22334_330_57326, &P22334_329};	/*Property = Viewport*/
const Value V22334_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22334_332 = {S48, 0, &V22334_332_57339, &P22334_330};	/*Property = WebWorkers*/
const Value V22334_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22334_333 = {S49, 0, &V22334_333_57342, &P22334_332};	/*Property = Xhr2*/
const Profile P22334 = {"22334", &P22334_333};	/*Profile ID - 22334*/
const Value V20317_1_1 = {S2, NULL};	/*Value = True*/
const Property P20317_1 = {S50, 0, &V20317_1_1, NULL};	/*Property = IsMobile*/
const Value V20317_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P20317_5 = {S52, 0, &V20317_5_1293, &P20317_1};	/*Property = ScreenPixelsHeight*/
const Value V20317_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P20317_4 = {S54, 0, &V20317_4_1296, &P20317_5};	/*Property = ScreenPixelsWidth*/
const Profile P20317 = {"20317", &P20317_4};	/*Profile ID - 20317*/
const Profile P22205 = {"22205", NULL};	/*Profile ID - 22205*/
const Profile P23114 = {"23114", NULL};	/*Profile ID - 23114*/
const Value V21521_1_1 = {S2, NULL};	/*Value = True*/
const Property P21521_1 = {S50, 0, &V21521_1_1, NULL};	/*Property = IsMobile*/
const Value V21521_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21521_5 = {S52, 0, &V21521_5_1277, &P21521_1};	/*Property = ScreenPixelsHeight*/
const Value V21521_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21521_4 = {S54, 0, &V21521_4_1279, &P21521_5};	/*Property = ScreenPixelsWidth*/
const Profile P21521 = {"21521", &P21521_4};	/*Profile ID - 21521*/
const Value V20288_1_1 = {S2, NULL};	/*Value = True*/
const Property P20288_1 = {S50, 0, &V20288_1_1, NULL};	/*Property = IsMobile*/
const Value V20288_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20288_5 = {S52, 0, &V20288_5_9981, &P20288_1};	/*Property = ScreenPixelsHeight*/
const Value V20288_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20288_4 = {S54, 0, &V20288_4_39098, &P20288_5};	/*Property = ScreenPixelsWidth*/
const Profile P20288 = {"20288", &P20288_4};	/*Profile ID - 20288*/
const Value V21464_1_1 = {S2, NULL};	/*Value = True*/
const Property P21464_1 = {S50, 0, &V21464_1_1, NULL};	/*Property = IsMobile*/
const Value V21464_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21464_5 = {S52, 0, &V21464_5_1277, &P21464_1};	/*Property = ScreenPixelsHeight*/
const Value V21464_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21464_4 = {S54, 0, &V21464_4_2336, &P21464_5};	/*Property = ScreenPixelsWidth*/
const Profile P21464 = {"21464", &P21464_4};	/*Profile ID - 21464*/
const Profile P22791 = {"22791", NULL};	/*Profile ID - 22791*/
const Value V21174_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_279 = {S1, 0, &V21174_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21174_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_282 = {S3, 0, &V21174_282_57155, &P21174_279};	/*Property = BlobBuilder*/
const Value V21174_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_350 = {S4, 0, &V21174_350_57446, &P21174_282};	/*Property = Canvas*/
const Value V21174_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_283 = {S5, 0, &V21174_283_57161, &P21174_350};	/*Property = CssBackground*/
const Value V21174_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_284 = {S6, 0, &V21174_284_57164, &P21174_283};	/*Property = CssBorderImage*/
const Value V21174_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_285 = {S7, 0, &V21174_285_57167, &P21174_284};	/*Property = CssCanvas*/
const Value V21174_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_286 = {S8, 0, &V21174_286_57170, &P21174_285};	/*Property = CssColor*/
const Value V21174_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_237 = {S9, 0, &V21174_237_55786, &P21174_286};	/*Property = CssColumn*/
const Value V21174_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_287 = {S10, 0, &V21174_287_57173, &P21174_237};	/*Property = CssFlexbox*/
const Value V21174_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_288 = {S11, 0, &V21174_288_57176, &P21174_287};	/*Property = CssFont*/
const Value V21174_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_289 = {S12, 0, &V21174_289_86899, &P21174_288};	/*Property = CssImages*/
const Value V21174_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_291 = {S13, 0, &V21174_291_57182, &P21174_289};	/*Property = CssMediaQueries*/
const Value V21174_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_292 = {S14, 0, &V21174_292_57185, &P21174_291};	/*Property = CssMinMax*/
const Value V21174_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_293 = {S15, 0, &V21174_293_57188, &P21174_292};	/*Property = CssOverflow*/
const Value V21174_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_294 = {S16, 0, &V21174_294_57191, &P21174_293};	/*Property = CssPosition*/
const Value V21174_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_295 = {S17, 0, &V21174_295_57194, &P21174_294};	/*Property = CssText*/
const Value V21174_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_297 = {S18, 0, &V21174_297_57197, &P21174_295};	/*Property = CssTransforms*/
const Value V21174_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_241 = {S19, 0, &V21174_241_55801, &P21174_297};	/*Property = CssTransitions*/
const Value V21174_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_298 = {S20, 0, &V21174_298_57208, &P21174_241};	/*Property = CssUI*/
const Value V21174_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_300 = {S21, 0, &V21174_300_57217, &P21174_298};	/*Property = DataSet*/
const Value V21174_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_301 = {S22, 0, &V21174_301_57220, &P21174_300};	/*Property = DataUrl*/
const Value V21174_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_302 = {S23, 0, &V21174_302_57223, &P21174_301};	/*Property = DeviceOrientation*/
const Value V21174_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_304 = {S24, 0, &V21174_304_57229, &P21174_302};	/*Property = FileReader*/
const Value V21174_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_305 = {S25, 0, &V21174_305_57232, &P21174_304};	/*Property = FileSaver*/
const Value V21174_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_306 = {S26, 0, &V21174_306_57238, &P21174_305};	/*Property = FileWriter*/
const Value V21174_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_307 = {S27, 0, &V21174_307_57241, &P21174_306};	/*Property = FormData*/
const Value V21174_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_309 = {S28, 0, &V21174_309_57247, &P21174_307};	/*Property = Fullscreen*/
const Value V21174_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_310 = {S29, 0, &V21174_310_57250, &P21174_309};	/*Property = GeoLocation*/
const Value V21174_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_312 = {S30, 0, &V21174_312_57256, &P21174_310};	/*Property = History*/
const Value V21174_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_314 = {S31, 0, &V21174_314_57262, &P21174_312};	/*Property = Html5*/
const Value V21174_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_313 = {S32, 0, &V21174_313_57259, &P21174_314};	/*Property = Html-Media-Capture*/
const Value V21174_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_315 = {S33, 0, &V21174_315_57265, &P21174_313};	/*Property = Iframe*/
const Value V21174_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_316 = {S34, 0, &V21174_316_57268, &P21174_315};	/*Property = IndexedDB*/
const Value V21174_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_317 = {S35, 0, &V21174_317_57271, &P21174_316};	/*Property = Json*/
const Value V21174_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P21174_15 = {S37, 0, &V21174_15_31, &P21174_317};	/*Property = LayoutEngine*/
const Value V21174_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_344 = {S38, 0, &V21174_344_57274, &P21174_15};	/*Property = Masking*/
const Value V21174_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_320 = {S39, 0, &V21174_320_57292, &P21174_344};	/*Property = PostMessage*/
const Value V21174_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_321 = {S40, 0, &V21174_321_57295, &P21174_320};	/*Property = Progress*/
const Value V21174_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_322 = {S41, 0, &V21174_322_57298, &P21174_321};	/*Property = Prompts*/
const Value V21174_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_323 = {S42, 0, &V21174_323_57301, &P21174_322};	/*Property = Selector*/
const Value V21174_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_324 = {S43, 0, &V21174_324_57307, &P21174_323};	/*Property = Svg*/
const Value V21174_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_327 = {S44, 0, &V21174_327_57316, &P21174_324};	/*Property = TouchEvents*/
const Value V21174_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_328 = {S45, 0, &V21174_328_57319, &P21174_327};	/*Property = Track*/
const Value V21174_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_329 = {S46, 0, &V21174_329_57325, &P21174_328};	/*Property = Video*/
const Value V21174_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_330 = {S47, 0, &V21174_330_57328, &P21174_329};	/*Property = Viewport*/
const Value V21174_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_332 = {S48, 0, &V21174_332_57340, &P21174_330};	/*Property = WebWorkers*/
const Value V21174_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21174_333 = {S49, 0, &V21174_333_57343, &P21174_332};	/*Property = Xhr2*/
const Profile P21174 = {"21174", &P21174_333};	/*Profile ID - 21174*/
const Value V22181_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22181_279 = {S1, 0, &V22181_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22181_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22181_282 = {S3, 0, &V22181_282_57154, &P22181_279};	/*Property = BlobBuilder*/
const Value V22181_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22181_350 = {S4, 0, &V22181_350_57444, &P22181_282};	/*Property = Canvas*/
const Value V22181_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22181_283 = {S5, 0, &V22181_283_57159, &P22181_350};	/*Property = CssBackground*/
const Value V22181_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22181_284 = {S6, 0, &V22181_284_57163, &P22181_283};	/*Property = CssBorderImage*/
const Value V22181_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22181_285 = {S7, 0, &V22181_285_57166, &P22181_284};	/*Property = CssCanvas*/
const Value V22181_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22181_286 = {S8, 0, &V22181_286_57168, &P22181_285};	/*Property = CssColor*/
const Value V22181_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22181_237 = {S9, 0, &V22181_237_55784, &P22181_286};	/*Property = CssColumn*/
const Value V22181_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22181_287 = {S10, 0, &V22181_287_57172, &P22181_237};	/*Property = CssFlexbox*/
const Value V22181_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22181_288 = {S11, 0, &V22181_288_57175, &P22181_287};	/*Property = CssFont*/
const Value V22181_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22181_289 = {S12, 0, &V22181_289_57759, &P22181_288};	/*Property = CssImages*/
const Value V22181_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22181_291 = {S13, 0, &V22181_291_57181, &P22181_289};	/*Property = CssMediaQueries*/
const Value V22181_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22181_292 = {S14, 0, &V22181_292_57183, &P22181_291};	/*Property = CssMinMax*/
const Value V22181_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22181_293 = {S15, 0, &V22181_293_57187, &P22181_292};	/*Property = CssOverflow*/
const Value V22181_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22181_294 = {S16, 0, &V22181_294_57190, &P22181_293};	/*Property = CssPosition*/
const Value V22181_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22181_295 = {S17, 0, &V22181_295_57193, &P22181_294};	/*Property = CssText*/
const Value V22181_297_57196 = {S0, NULL};	/*Value = False*/
const Property P22181_297 = {S18, 0, &V22181_297_57196, &P22181_295};	/*Property = CssTransforms*/
const Value V22181_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22181_241 = {S19, 0, &V22181_241_55799, &P22181_297};	/*Property = CssTransitions*/
const Value V22181_298_57207 = {S0, NULL};	/*Value = False*/
const Property P22181_298 = {S20, 0, &V22181_298_57207, &P22181_241};	/*Property = CssUI*/
const Value V22181_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22181_300 = {S21, 0, &V22181_300_57216, &P22181_298};	/*Property = DataSet*/
const Value V22181_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22181_301 = {S22, 0, &V22181_301_57218, &P22181_300};	/*Property = DataUrl*/
const Value V22181_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22181_302 = {S23, 0, &V22181_302_57222, &P22181_301};	/*Property = DeviceOrientation*/
const Value V22181_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22181_304 = {S24, 0, &V22181_304_57228, &P22181_302};	/*Property = FileReader*/
const Value V22181_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22181_305 = {S25, 0, &V22181_305_57231, &P22181_304};	/*Property = FileSaver*/
const Value V22181_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22181_306 = {S26, 0, &V22181_306_57237, &P22181_305};	/*Property = FileWriter*/
const Value V22181_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22181_307 = {S27, 0, &V22181_307_57240, &P22181_306};	/*Property = FormData*/
const Value V22181_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22181_309 = {S28, 0, &V22181_309_57246, &P22181_307};	/*Property = Fullscreen*/
const Value V22181_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22181_310 = {S29, 0, &V22181_310_57248, &P22181_309};	/*Property = GeoLocation*/
const Value V22181_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22181_312 = {S30, 0, &V22181_312_57255, &P22181_310};	/*Property = History*/
const Value V22181_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22181_314 = {S31, 0, &V22181_314_57261, &P22181_312};	/*Property = Html5*/
const Value V22181_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22181_313 = {S32, 0, &V22181_313_57258, &P22181_314};	/*Property = Html-Media-Capture*/
const Value V22181_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22181_315 = {S33, 0, &V22181_315_57264, &P22181_313};	/*Property = Iframe*/
const Value V22181_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22181_316 = {S34, 0, &V22181_316_57267, &P22181_315};	/*Property = IndexedDB*/
const Value V22181_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22181_317 = {S35, 0, &V22181_317_57269, &P22181_316};	/*Property = Json*/
const Value V22181_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22181_15 = {S37, 0, &V22181_15_29, &P22181_317};	/*Property = LayoutEngine*/
const Value V22181_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22181_344 = {S38, 0, &V22181_344_57272, &P22181_15};	/*Property = Masking*/
const Value V22181_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22181_320 = {S39, 0, &V22181_320_57290, &P22181_344};	/*Property = PostMessage*/
const Value V22181_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22181_321 = {S40, 0, &V22181_321_57293, &P22181_320};	/*Property = Progress*/
const Value V22181_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22181_322 = {S41, 0, &V22181_322_57296, &P22181_321};	/*Property = Prompts*/
const Value V22181_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22181_323 = {S42, 0, &V22181_323_57299, &P22181_322};	/*Property = Selector*/
const Value V22181_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22181_324 = {S43, 0, &V22181_324_57306, &P22181_323};	/*Property = Svg*/
const Value V22181_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22181_327 = {S44, 0, &V22181_327_57315, &P22181_324};	/*Property = TouchEvents*/
const Value V22181_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22181_328 = {S45, 0, &V22181_328_57318, &P22181_327};	/*Property = Track*/
const Value V22181_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22181_329 = {S46, 0, &V22181_329_57323, &P22181_328};	/*Property = Video*/
const Value V22181_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22181_330 = {S47, 0, &V22181_330_57326, &P22181_329};	/*Property = Viewport*/
const Value V22181_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22181_332 = {S48, 0, &V22181_332_57339, &P22181_330};	/*Property = WebWorkers*/
const Value V22181_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22181_333 = {S49, 0, &V22181_333_57342, &P22181_332};	/*Property = Xhr2*/
const Profile P22181 = {"22181", &P22181_333};	/*Profile ID - 22181*/
const Profile P22201 = {"22201", NULL};	/*Profile ID - 22201*/
const Value V18038_1_1 = {S2, NULL};	/*Value = True*/
const Property P18038_1 = {S50, 0, &V18038_1_1, NULL};	/*Property = IsMobile*/
const Value V18038_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P18038_5 = {S52, 0, &V18038_5_1269, &P18038_1};	/*Property = ScreenPixelsHeight*/
const Value V18038_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18038_4 = {S54, 0, &V18038_4_1229, &P18038_5};	/*Property = ScreenPixelsWidth*/
const Profile P18038 = {"18038", &P18038_4};	/*Profile ID - 18038*/
const Value V21585_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_279 = {S1, 0, &V21585_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21585_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_282 = {S3, 0, &V21585_282_57155, &P21585_279};	/*Property = BlobBuilder*/
const Value V21585_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_350 = {S4, 0, &V21585_350_57446, &P21585_282};	/*Property = Canvas*/
const Value V21585_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_283 = {S5, 0, &V21585_283_57161, &P21585_350};	/*Property = CssBackground*/
const Value V21585_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_284 = {S6, 0, &V21585_284_57164, &P21585_283};	/*Property = CssBorderImage*/
const Value V21585_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_285 = {S7, 0, &V21585_285_57167, &P21585_284};	/*Property = CssCanvas*/
const Value V21585_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_286 = {S8, 0, &V21585_286_57170, &P21585_285};	/*Property = CssColor*/
const Value V21585_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_237 = {S9, 0, &V21585_237_55786, &P21585_286};	/*Property = CssColumn*/
const Value V21585_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_287 = {S10, 0, &V21585_287_57173, &P21585_237};	/*Property = CssFlexbox*/
const Value V21585_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_288 = {S11, 0, &V21585_288_57176, &P21585_287};	/*Property = CssFont*/
const Value V21585_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_289 = {S12, 0, &V21585_289_86899, &P21585_288};	/*Property = CssImages*/
const Value V21585_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_291 = {S13, 0, &V21585_291_57182, &P21585_289};	/*Property = CssMediaQueries*/
const Value V21585_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_292 = {S14, 0, &V21585_292_57185, &P21585_291};	/*Property = CssMinMax*/
const Value V21585_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_293 = {S15, 0, &V21585_293_57188, &P21585_292};	/*Property = CssOverflow*/
const Value V21585_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_294 = {S16, 0, &V21585_294_57191, &P21585_293};	/*Property = CssPosition*/
const Value V21585_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_295 = {S17, 0, &V21585_295_57194, &P21585_294};	/*Property = CssText*/
const Value V21585_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_297 = {S18, 0, &V21585_297_57197, &P21585_295};	/*Property = CssTransforms*/
const Value V21585_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_241 = {S19, 0, &V21585_241_55801, &P21585_297};	/*Property = CssTransitions*/
const Value V21585_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_298 = {S20, 0, &V21585_298_57208, &P21585_241};	/*Property = CssUI*/
const Value V21585_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_300 = {S21, 0, &V21585_300_57217, &P21585_298};	/*Property = DataSet*/
const Value V21585_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_301 = {S22, 0, &V21585_301_57220, &P21585_300};	/*Property = DataUrl*/
const Value V21585_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_302 = {S23, 0, &V21585_302_57223, &P21585_301};	/*Property = DeviceOrientation*/
const Value V21585_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_304 = {S24, 0, &V21585_304_57229, &P21585_302};	/*Property = FileReader*/
const Value V21585_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_305 = {S25, 0, &V21585_305_57232, &P21585_304};	/*Property = FileSaver*/
const Value V21585_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_306 = {S26, 0, &V21585_306_57238, &P21585_305};	/*Property = FileWriter*/
const Value V21585_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_307 = {S27, 0, &V21585_307_57241, &P21585_306};	/*Property = FormData*/
const Value V21585_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_309 = {S28, 0, &V21585_309_57247, &P21585_307};	/*Property = Fullscreen*/
const Value V21585_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_310 = {S29, 0, &V21585_310_57250, &P21585_309};	/*Property = GeoLocation*/
const Value V21585_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_312 = {S30, 0, &V21585_312_57256, &P21585_310};	/*Property = History*/
const Value V21585_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_314 = {S31, 0, &V21585_314_57262, &P21585_312};	/*Property = Html5*/
const Value V21585_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_313 = {S32, 0, &V21585_313_57259, &P21585_314};	/*Property = Html-Media-Capture*/
const Value V21585_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_315 = {S33, 0, &V21585_315_57265, &P21585_313};	/*Property = Iframe*/
const Value V21585_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_316 = {S34, 0, &V21585_316_57268, &P21585_315};	/*Property = IndexedDB*/
const Value V21585_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_317 = {S35, 0, &V21585_317_57271, &P21585_316};	/*Property = Json*/
const Value V21585_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_15 = {S37, 0, &V21585_15_72, &P21585_317};	/*Property = LayoutEngine*/
const Value V21585_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_344 = {S38, 0, &V21585_344_57274, &P21585_15};	/*Property = Masking*/
const Value V21585_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_320 = {S39, 0, &V21585_320_57292, &P21585_344};	/*Property = PostMessage*/
const Value V21585_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_321 = {S40, 0, &V21585_321_57295, &P21585_320};	/*Property = Progress*/
const Value V21585_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_322 = {S41, 0, &V21585_322_57298, &P21585_321};	/*Property = Prompts*/
const Value V21585_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_323 = {S42, 0, &V21585_323_57301, &P21585_322};	/*Property = Selector*/
const Value V21585_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_324 = {S43, 0, &V21585_324_57307, &P21585_323};	/*Property = Svg*/
const Value V21585_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_327 = {S44, 0, &V21585_327_57316, &P21585_324};	/*Property = TouchEvents*/
const Value V21585_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_328 = {S45, 0, &V21585_328_57319, &P21585_327};	/*Property = Track*/
const Value V21585_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_329 = {S46, 0, &V21585_329_57325, &P21585_328};	/*Property = Video*/
const Value V21585_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_330 = {S47, 0, &V21585_330_57328, &P21585_329};	/*Property = Viewport*/
const Value V21585_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_332 = {S48, 0, &V21585_332_57340, &P21585_330};	/*Property = WebWorkers*/
const Value V21585_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21585_333 = {S49, 0, &V21585_333_57343, &P21585_332};	/*Property = Xhr2*/
const Profile P21585 = {"21585", &P21585_333};	/*Profile ID - 21585*/
const Value V21553_1_1 = {S2, NULL};	/*Value = True*/
const Property P21553_1 = {S50, 0, &V21553_1_1, NULL};	/*Property = IsMobile*/
const Value V21553_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21553_5 = {S52, 0, &V21553_5_37, &P21553_1};	/*Property = ScreenPixelsHeight*/
const Value V21553_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21553_4 = {S54, 0, &V21553_4_36, &P21553_5};	/*Property = ScreenPixelsWidth*/
const Profile P21553 = {"21553", &P21553_4};	/*Profile ID - 21553*/
const Value V19970_1_1 = {S2, NULL};	/*Value = True*/
const Property P19970_1 = {S50, 0, &V19970_1_1, NULL};	/*Property = IsMobile*/
const Value V19970_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P19970_5 = {S52, 0, &V19970_5_1222, &P19970_1};	/*Property = ScreenPixelsHeight*/
const Value V19970_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P19970_4 = {S54, 0, &V19970_4_1224, &P19970_5};	/*Property = ScreenPixelsWidth*/
const Profile P19970 = {"19970", &P19970_4};	/*Profile ID - 19970*/
const Value V22309_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_279 = {S1, 0, &V22309_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22309_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_282 = {S3, 0, &V22309_282_57155, &P22309_279};	/*Property = BlobBuilder*/
const Value V22309_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_350 = {S4, 0, &V22309_350_57446, &P22309_282};	/*Property = Canvas*/
const Value V22309_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_283 = {S5, 0, &V22309_283_57161, &P22309_350};	/*Property = CssBackground*/
const Value V22309_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_284 = {S6, 0, &V22309_284_57164, &P22309_283};	/*Property = CssBorderImage*/
const Value V22309_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_285 = {S7, 0, &V22309_285_57167, &P22309_284};	/*Property = CssCanvas*/
const Value V22309_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_286 = {S8, 0, &V22309_286_57170, &P22309_285};	/*Property = CssColor*/
const Value V22309_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_237 = {S9, 0, &V22309_237_55786, &P22309_286};	/*Property = CssColumn*/
const Value V22309_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_287 = {S10, 0, &V22309_287_57173, &P22309_237};	/*Property = CssFlexbox*/
const Value V22309_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_288 = {S11, 0, &V22309_288_57176, &P22309_287};	/*Property = CssFont*/
const Value V22309_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_289 = {S12, 0, &V22309_289_86899, &P22309_288};	/*Property = CssImages*/
const Value V22309_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_291 = {S13, 0, &V22309_291_57182, &P22309_289};	/*Property = CssMediaQueries*/
const Value V22309_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_292 = {S14, 0, &V22309_292_57185, &P22309_291};	/*Property = CssMinMax*/
const Value V22309_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_293 = {S15, 0, &V22309_293_57188, &P22309_292};	/*Property = CssOverflow*/
const Value V22309_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_294 = {S16, 0, &V22309_294_57191, &P22309_293};	/*Property = CssPosition*/
const Value V22309_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_295 = {S17, 0, &V22309_295_57194, &P22309_294};	/*Property = CssText*/
const Value V22309_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_297 = {S18, 0, &V22309_297_57197, &P22309_295};	/*Property = CssTransforms*/
const Value V22309_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_241 = {S19, 0, &V22309_241_55801, &P22309_297};	/*Property = CssTransitions*/
const Value V22309_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_298 = {S20, 0, &V22309_298_57208, &P22309_241};	/*Property = CssUI*/
const Value V22309_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_300 = {S21, 0, &V22309_300_57217, &P22309_298};	/*Property = DataSet*/
const Value V22309_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_301 = {S22, 0, &V22309_301_57220, &P22309_300};	/*Property = DataUrl*/
const Value V22309_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_302 = {S23, 0, &V22309_302_57223, &P22309_301};	/*Property = DeviceOrientation*/
const Value V22309_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_304 = {S24, 0, &V22309_304_57229, &P22309_302};	/*Property = FileReader*/
const Value V22309_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_305 = {S25, 0, &V22309_305_57232, &P22309_304};	/*Property = FileSaver*/
const Value V22309_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_306 = {S26, 0, &V22309_306_57238, &P22309_305};	/*Property = FileWriter*/
const Value V22309_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_307 = {S27, 0, &V22309_307_57241, &P22309_306};	/*Property = FormData*/
const Value V22309_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_309 = {S28, 0, &V22309_309_57247, &P22309_307};	/*Property = Fullscreen*/
const Value V22309_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_310 = {S29, 0, &V22309_310_57250, &P22309_309};	/*Property = GeoLocation*/
const Value V22309_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_312 = {S30, 0, &V22309_312_57256, &P22309_310};	/*Property = History*/
const Value V22309_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_314 = {S31, 0, &V22309_314_57262, &P22309_312};	/*Property = Html5*/
const Value V22309_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_313 = {S32, 0, &V22309_313_57259, &P22309_314};	/*Property = Html-Media-Capture*/
const Value V22309_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_315 = {S33, 0, &V22309_315_57265, &P22309_313};	/*Property = Iframe*/
const Value V22309_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_316 = {S34, 0, &V22309_316_57268, &P22309_315};	/*Property = IndexedDB*/
const Value V22309_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_317 = {S35, 0, &V22309_317_57271, &P22309_316};	/*Property = Json*/
const Value V22309_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22309_15 = {S37, 0, &V22309_15_29, &P22309_317};	/*Property = LayoutEngine*/
const Value V22309_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_344 = {S38, 0, &V22309_344_57274, &P22309_15};	/*Property = Masking*/
const Value V22309_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_320 = {S39, 0, &V22309_320_57292, &P22309_344};	/*Property = PostMessage*/
const Value V22309_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_321 = {S40, 0, &V22309_321_57295, &P22309_320};	/*Property = Progress*/
const Value V22309_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_322 = {S41, 0, &V22309_322_57298, &P22309_321};	/*Property = Prompts*/
const Value V22309_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_323 = {S42, 0, &V22309_323_57301, &P22309_322};	/*Property = Selector*/
const Value V22309_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_324 = {S43, 0, &V22309_324_57307, &P22309_323};	/*Property = Svg*/
const Value V22309_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_327 = {S44, 0, &V22309_327_57316, &P22309_324};	/*Property = TouchEvents*/
const Value V22309_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_328 = {S45, 0, &V22309_328_57319, &P22309_327};	/*Property = Track*/
const Value V22309_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_329 = {S46, 0, &V22309_329_57325, &P22309_328};	/*Property = Video*/
const Value V22309_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_330 = {S47, 0, &V22309_330_57328, &P22309_329};	/*Property = Viewport*/
const Value V22309_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_332 = {S48, 0, &V22309_332_57340, &P22309_330};	/*Property = WebWorkers*/
const Value V22309_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22309_333 = {S49, 0, &V22309_333_57343, &P22309_332};	/*Property = Xhr2*/
const Profile P22309 = {"22309", &P22309_333};	/*Profile ID - 22309*/
const Value V4393_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_279 = {S1, 0, &V4393_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4393_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_282 = {S3, 0, &V4393_282_57155, &P4393_279};	/*Property = BlobBuilder*/
const Value V4393_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_350 = {S4, 0, &V4393_350_57446, &P4393_282};	/*Property = Canvas*/
const Value V4393_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_283 = {S5, 0, &V4393_283_57161, &P4393_350};	/*Property = CssBackground*/
const Value V4393_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_284 = {S6, 0, &V4393_284_57164, &P4393_283};	/*Property = CssBorderImage*/
const Value V4393_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_285 = {S7, 0, &V4393_285_57167, &P4393_284};	/*Property = CssCanvas*/
const Value V4393_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_286 = {S8, 0, &V4393_286_57170, &P4393_285};	/*Property = CssColor*/
const Value V4393_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_237 = {S9, 0, &V4393_237_55786, &P4393_286};	/*Property = CssColumn*/
const Value V4393_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_287 = {S10, 0, &V4393_287_57173, &P4393_237};	/*Property = CssFlexbox*/
const Value V4393_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_288 = {S11, 0, &V4393_288_57176, &P4393_287};	/*Property = CssFont*/
const Value V4393_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_289 = {S12, 0, &V4393_289_86899, &P4393_288};	/*Property = CssImages*/
const Value V4393_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_291 = {S13, 0, &V4393_291_57182, &P4393_289};	/*Property = CssMediaQueries*/
const Value V4393_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_292 = {S14, 0, &V4393_292_57185, &P4393_291};	/*Property = CssMinMax*/
const Value V4393_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_293 = {S15, 0, &V4393_293_57188, &P4393_292};	/*Property = CssOverflow*/
const Value V4393_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_294 = {S16, 0, &V4393_294_57191, &P4393_293};	/*Property = CssPosition*/
const Value V4393_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_295 = {S17, 0, &V4393_295_57194, &P4393_294};	/*Property = CssText*/
const Value V4393_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_297 = {S18, 0, &V4393_297_57197, &P4393_295};	/*Property = CssTransforms*/
const Value V4393_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_241 = {S19, 0, &V4393_241_55801, &P4393_297};	/*Property = CssTransitions*/
const Value V4393_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_298 = {S20, 0, &V4393_298_57208, &P4393_241};	/*Property = CssUI*/
const Value V4393_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_300 = {S21, 0, &V4393_300_57217, &P4393_298};	/*Property = DataSet*/
const Value V4393_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_301 = {S22, 0, &V4393_301_57220, &P4393_300};	/*Property = DataUrl*/
const Value V4393_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_302 = {S23, 0, &V4393_302_57223, &P4393_301};	/*Property = DeviceOrientation*/
const Value V4393_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_304 = {S24, 0, &V4393_304_57229, &P4393_302};	/*Property = FileReader*/
const Value V4393_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_305 = {S25, 0, &V4393_305_57232, &P4393_304};	/*Property = FileSaver*/
const Value V4393_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_306 = {S26, 0, &V4393_306_57238, &P4393_305};	/*Property = FileWriter*/
const Value V4393_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_307 = {S27, 0, &V4393_307_57241, &P4393_306};	/*Property = FormData*/
const Value V4393_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_309 = {S28, 0, &V4393_309_57247, &P4393_307};	/*Property = Fullscreen*/
const Value V4393_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_310 = {S29, 0, &V4393_310_57250, &P4393_309};	/*Property = GeoLocation*/
const Value V4393_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_312 = {S30, 0, &V4393_312_57256, &P4393_310};	/*Property = History*/
const Value V4393_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_314 = {S31, 0, &V4393_314_57262, &P4393_312};	/*Property = Html5*/
const Value V4393_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_313 = {S32, 0, &V4393_313_57259, &P4393_314};	/*Property = Html-Media-Capture*/
const Value V4393_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_315 = {S33, 0, &V4393_315_57265, &P4393_313};	/*Property = Iframe*/
const Value V4393_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_316 = {S34, 0, &V4393_316_57268, &P4393_315};	/*Property = IndexedDB*/
const Value V4393_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_317 = {S35, 0, &V4393_317_57271, &P4393_316};	/*Property = Json*/
const Value V4393_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P4393_15 = {S37, 0, &V4393_15_31, &P4393_317};	/*Property = LayoutEngine*/
const Value V4393_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_344 = {S38, 0, &V4393_344_57274, &P4393_15};	/*Property = Masking*/
const Value V4393_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_320 = {S39, 0, &V4393_320_57292, &P4393_344};	/*Property = PostMessage*/
const Value V4393_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_321 = {S40, 0, &V4393_321_57295, &P4393_320};	/*Property = Progress*/
const Value V4393_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_322 = {S41, 0, &V4393_322_57298, &P4393_321};	/*Property = Prompts*/
const Value V4393_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_323 = {S42, 0, &V4393_323_57301, &P4393_322};	/*Property = Selector*/
const Value V4393_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_324 = {S43, 0, &V4393_324_57307, &P4393_323};	/*Property = Svg*/
const Value V4393_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_327 = {S44, 0, &V4393_327_57316, &P4393_324};	/*Property = TouchEvents*/
const Value V4393_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_328 = {S45, 0, &V4393_328_57319, &P4393_327};	/*Property = Track*/
const Value V4393_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_329 = {S46, 0, &V4393_329_57325, &P4393_328};	/*Property = Video*/
const Value V4393_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_330 = {S47, 0, &V4393_330_57328, &P4393_329};	/*Property = Viewport*/
const Value V4393_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_332 = {S48, 0, &V4393_332_57340, &P4393_330};	/*Property = WebWorkers*/
const Value V4393_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4393_333 = {S49, 0, &V4393_333_57343, &P4393_332};	/*Property = Xhr2*/
const Profile P4393 = {"4393", &P4393_333};	/*Profile ID - 4393*/
const Value V22506_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22506_279 = {S1, 0, &V22506_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22506_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22506_282 = {S3, 0, &V22506_282_57154, &P22506_279};	/*Property = BlobBuilder*/
const Value V22506_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22506_350 = {S4, 0, &V22506_350_57444, &P22506_282};	/*Property = Canvas*/
const Value V22506_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22506_283 = {S5, 0, &V22506_283_57159, &P22506_350};	/*Property = CssBackground*/
const Value V22506_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22506_284 = {S6, 0, &V22506_284_57163, &P22506_283};	/*Property = CssBorderImage*/
const Value V22506_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22506_285 = {S7, 0, &V22506_285_57166, &P22506_284};	/*Property = CssCanvas*/
const Value V22506_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22506_286 = {S8, 0, &V22506_286_57168, &P22506_285};	/*Property = CssColor*/
const Value V22506_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22506_237 = {S9, 0, &V22506_237_55784, &P22506_286};	/*Property = CssColumn*/
const Value V22506_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22506_287 = {S10, 0, &V22506_287_57172, &P22506_237};	/*Property = CssFlexbox*/
const Value V22506_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22506_288 = {S11, 0, &V22506_288_57175, &P22506_287};	/*Property = CssFont*/
const Value V22506_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22506_289 = {S12, 0, &V22506_289_57759, &P22506_288};	/*Property = CssImages*/
const Value V22506_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22506_291 = {S13, 0, &V22506_291_57181, &P22506_289};	/*Property = CssMediaQueries*/
const Value V22506_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22506_292 = {S14, 0, &V22506_292_57183, &P22506_291};	/*Property = CssMinMax*/
const Value V22506_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22506_293 = {S15, 0, &V22506_293_57187, &P22506_292};	/*Property = CssOverflow*/
const Value V22506_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22506_294 = {S16, 0, &V22506_294_57190, &P22506_293};	/*Property = CssPosition*/
const Value V22506_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22506_295 = {S17, 0, &V22506_295_57193, &P22506_294};	/*Property = CssText*/
const Value V22506_297_57196 = {S0, NULL};	/*Value = False*/
const Property P22506_297 = {S18, 0, &V22506_297_57196, &P22506_295};	/*Property = CssTransforms*/
const Value V22506_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22506_241 = {S19, 0, &V22506_241_55799, &P22506_297};	/*Property = CssTransitions*/
const Value V22506_298_57207 = {S0, NULL};	/*Value = False*/
const Property P22506_298 = {S20, 0, &V22506_298_57207, &P22506_241};	/*Property = CssUI*/
const Value V22506_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22506_300 = {S21, 0, &V22506_300_57216, &P22506_298};	/*Property = DataSet*/
const Value V22506_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22506_301 = {S22, 0, &V22506_301_57218, &P22506_300};	/*Property = DataUrl*/
const Value V22506_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22506_302 = {S23, 0, &V22506_302_57222, &P22506_301};	/*Property = DeviceOrientation*/
const Value V22506_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22506_304 = {S24, 0, &V22506_304_57228, &P22506_302};	/*Property = FileReader*/
const Value V22506_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22506_305 = {S25, 0, &V22506_305_57231, &P22506_304};	/*Property = FileSaver*/
const Value V22506_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22506_306 = {S26, 0, &V22506_306_57237, &P22506_305};	/*Property = FileWriter*/
const Value V22506_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22506_307 = {S27, 0, &V22506_307_57240, &P22506_306};	/*Property = FormData*/
const Value V22506_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22506_309 = {S28, 0, &V22506_309_57246, &P22506_307};	/*Property = Fullscreen*/
const Value V22506_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22506_310 = {S29, 0, &V22506_310_57248, &P22506_309};	/*Property = GeoLocation*/
const Value V22506_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22506_312 = {S30, 0, &V22506_312_57255, &P22506_310};	/*Property = History*/
const Value V22506_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22506_314 = {S31, 0, &V22506_314_57261, &P22506_312};	/*Property = Html5*/
const Value V22506_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22506_313 = {S32, 0, &V22506_313_57258, &P22506_314};	/*Property = Html-Media-Capture*/
const Value V22506_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22506_315 = {S33, 0, &V22506_315_57264, &P22506_313};	/*Property = Iframe*/
const Value V22506_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22506_316 = {S34, 0, &V22506_316_57267, &P22506_315};	/*Property = IndexedDB*/
const Value V22506_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22506_317 = {S35, 0, &V22506_317_57269, &P22506_316};	/*Property = Json*/
const Value V22506_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22506_15 = {S37, 0, &V22506_15_29, &P22506_317};	/*Property = LayoutEngine*/
const Value V22506_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22506_344 = {S38, 0, &V22506_344_57272, &P22506_15};	/*Property = Masking*/
const Value V22506_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22506_320 = {S39, 0, &V22506_320_57290, &P22506_344};	/*Property = PostMessage*/
const Value V22506_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22506_321 = {S40, 0, &V22506_321_57293, &P22506_320};	/*Property = Progress*/
const Value V22506_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22506_322 = {S41, 0, &V22506_322_57296, &P22506_321};	/*Property = Prompts*/
const Value V22506_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22506_323 = {S42, 0, &V22506_323_57299, &P22506_322};	/*Property = Selector*/
const Value V22506_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22506_324 = {S43, 0, &V22506_324_57306, &P22506_323};	/*Property = Svg*/
const Value V22506_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22506_327 = {S44, 0, &V22506_327_57315, &P22506_324};	/*Property = TouchEvents*/
const Value V22506_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22506_328 = {S45, 0, &V22506_328_57318, &P22506_327};	/*Property = Track*/
const Value V22506_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22506_329 = {S46, 0, &V22506_329_57323, &P22506_328};	/*Property = Video*/
const Value V22506_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22506_330 = {S47, 0, &V22506_330_57326, &P22506_329};	/*Property = Viewport*/
const Value V22506_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22506_332 = {S48, 0, &V22506_332_57339, &P22506_330};	/*Property = WebWorkers*/
const Value V22506_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22506_333 = {S49, 0, &V22506_333_57342, &P22506_332};	/*Property = Xhr2*/
const Profile P22506 = {"22506", &P22506_333};	/*Profile ID - 22506*/
const Value V18196_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_279 = {S1, 0, &V18196_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18196_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_282 = {S3, 0, &V18196_282_57155, &P18196_279};	/*Property = BlobBuilder*/
const Value V18196_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_350 = {S4, 0, &V18196_350_57446, &P18196_282};	/*Property = Canvas*/
const Value V18196_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_283 = {S5, 0, &V18196_283_57161, &P18196_350};	/*Property = CssBackground*/
const Value V18196_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_284 = {S6, 0, &V18196_284_57164, &P18196_283};	/*Property = CssBorderImage*/
const Value V18196_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_285 = {S7, 0, &V18196_285_57167, &P18196_284};	/*Property = CssCanvas*/
const Value V18196_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_286 = {S8, 0, &V18196_286_57170, &P18196_285};	/*Property = CssColor*/
const Value V18196_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_237 = {S9, 0, &V18196_237_55786, &P18196_286};	/*Property = CssColumn*/
const Value V18196_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_287 = {S10, 0, &V18196_287_57173, &P18196_237};	/*Property = CssFlexbox*/
const Value V18196_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_288 = {S11, 0, &V18196_288_57176, &P18196_287};	/*Property = CssFont*/
const Value V18196_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_289 = {S12, 0, &V18196_289_86899, &P18196_288};	/*Property = CssImages*/
const Value V18196_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_291 = {S13, 0, &V18196_291_57182, &P18196_289};	/*Property = CssMediaQueries*/
const Value V18196_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_292 = {S14, 0, &V18196_292_57185, &P18196_291};	/*Property = CssMinMax*/
const Value V18196_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_293 = {S15, 0, &V18196_293_57188, &P18196_292};	/*Property = CssOverflow*/
const Value V18196_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_294 = {S16, 0, &V18196_294_57191, &P18196_293};	/*Property = CssPosition*/
const Value V18196_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_295 = {S17, 0, &V18196_295_57194, &P18196_294};	/*Property = CssText*/
const Value V18196_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_297 = {S18, 0, &V18196_297_57197, &P18196_295};	/*Property = CssTransforms*/
const Value V18196_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_241 = {S19, 0, &V18196_241_55801, &P18196_297};	/*Property = CssTransitions*/
const Value V18196_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_298 = {S20, 0, &V18196_298_57208, &P18196_241};	/*Property = CssUI*/
const Value V18196_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_300 = {S21, 0, &V18196_300_57217, &P18196_298};	/*Property = DataSet*/
const Value V18196_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_301 = {S22, 0, &V18196_301_57220, &P18196_300};	/*Property = DataUrl*/
const Value V18196_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_302 = {S23, 0, &V18196_302_57223, &P18196_301};	/*Property = DeviceOrientation*/
const Value V18196_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_304 = {S24, 0, &V18196_304_57229, &P18196_302};	/*Property = FileReader*/
const Value V18196_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_305 = {S25, 0, &V18196_305_57232, &P18196_304};	/*Property = FileSaver*/
const Value V18196_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_306 = {S26, 0, &V18196_306_57238, &P18196_305};	/*Property = FileWriter*/
const Value V18196_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_307 = {S27, 0, &V18196_307_57241, &P18196_306};	/*Property = FormData*/
const Value V18196_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_309 = {S28, 0, &V18196_309_57247, &P18196_307};	/*Property = Fullscreen*/
const Value V18196_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_310 = {S29, 0, &V18196_310_57250, &P18196_309};	/*Property = GeoLocation*/
const Value V18196_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_312 = {S30, 0, &V18196_312_57256, &P18196_310};	/*Property = History*/
const Value V18196_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_314 = {S31, 0, &V18196_314_57262, &P18196_312};	/*Property = Html5*/
const Value V18196_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_313 = {S32, 0, &V18196_313_57259, &P18196_314};	/*Property = Html-Media-Capture*/
const Value V18196_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_315 = {S33, 0, &V18196_315_57265, &P18196_313};	/*Property = Iframe*/
const Value V18196_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_316 = {S34, 0, &V18196_316_57268, &P18196_315};	/*Property = IndexedDB*/
const Value V18196_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_317 = {S35, 0, &V18196_317_57271, &P18196_316};	/*Property = Json*/
const Value V18196_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18196_15 = {S37, 0, &V18196_15_29, &P18196_317};	/*Property = LayoutEngine*/
const Value V18196_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_344 = {S38, 0, &V18196_344_57274, &P18196_15};	/*Property = Masking*/
const Value V18196_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_320 = {S39, 0, &V18196_320_57292, &P18196_344};	/*Property = PostMessage*/
const Value V18196_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_321 = {S40, 0, &V18196_321_57295, &P18196_320};	/*Property = Progress*/
const Value V18196_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_322 = {S41, 0, &V18196_322_57298, &P18196_321};	/*Property = Prompts*/
const Value V18196_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_323 = {S42, 0, &V18196_323_57301, &P18196_322};	/*Property = Selector*/
const Value V18196_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_324 = {S43, 0, &V18196_324_57307, &P18196_323};	/*Property = Svg*/
const Value V18196_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_327 = {S44, 0, &V18196_327_57316, &P18196_324};	/*Property = TouchEvents*/
const Value V18196_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_328 = {S45, 0, &V18196_328_57319, &P18196_327};	/*Property = Track*/
const Value V18196_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_329 = {S46, 0, &V18196_329_57325, &P18196_328};	/*Property = Video*/
const Value V18196_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_330 = {S47, 0, &V18196_330_57328, &P18196_329};	/*Property = Viewport*/
const Value V18196_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_332 = {S48, 0, &V18196_332_57340, &P18196_330};	/*Property = WebWorkers*/
const Value V18196_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18196_333 = {S49, 0, &V18196_333_57343, &P18196_332};	/*Property = Xhr2*/
const Profile P18196 = {"18196", &P18196_333};	/*Profile ID - 18196*/
const Value V22346_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22346_279 = {S1, 0, &V22346_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22346_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22346_282 = {S3, 0, &V22346_282_57153, &P22346_279};	/*Property = BlobBuilder*/
const Value V22346_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22346_350 = {S4, 0, &V22346_350_57444, &P22346_282};	/*Property = Canvas*/
const Value V22346_283_57160 = {S0, NULL};	/*Value = False*/
const Property P22346_283 = {S5, 0, &V22346_283_57160, &P22346_350};	/*Property = CssBackground*/
const Value V22346_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22346_284 = {S6, 0, &V22346_284_57163, &P22346_283};	/*Property = CssBorderImage*/
const Value V22346_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22346_285 = {S7, 0, &V22346_285_57166, &P22346_284};	/*Property = CssCanvas*/
const Value V22346_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22346_286 = {S8, 0, &V22346_286_57168, &P22346_285};	/*Property = CssColor*/
const Value V22346_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22346_237 = {S9, 0, &V22346_237_55784, &P22346_286};	/*Property = CssColumn*/
const Value V22346_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22346_287 = {S10, 0, &V22346_287_57172, &P22346_237};	/*Property = CssFlexbox*/
const Value V22346_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22346_288 = {S11, 0, &V22346_288_57175, &P22346_287};	/*Property = CssFont*/
const Value V22346_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22346_289 = {S12, 0, &V22346_289_86899, &P22346_288};	/*Property = CssImages*/
const Value V22346_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22346_291 = {S13, 0, &V22346_291_57181, &P22346_289};	/*Property = CssMediaQueries*/
const Value V22346_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22346_292 = {S14, 0, &V22346_292_57183, &P22346_291};	/*Property = CssMinMax*/
const Value V22346_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22346_293 = {S15, 0, &V22346_293_57187, &P22346_292};	/*Property = CssOverflow*/
const Value V22346_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22346_294 = {S16, 0, &V22346_294_57189, &P22346_293};	/*Property = CssPosition*/
const Value V22346_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22346_295 = {S17, 0, &V22346_295_57193, &P22346_294};	/*Property = CssText*/
const Value V22346_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22346_297 = {S18, 0, &V22346_297_57195, &P22346_295};	/*Property = CssTransforms*/
const Value V22346_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22346_241 = {S19, 0, &V22346_241_55799, &P22346_297};	/*Property = CssTransitions*/
const Value V22346_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22346_298 = {S20, 0, &V22346_298_57206, &P22346_241};	/*Property = CssUI*/
const Value V22346_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22346_300 = {S21, 0, &V22346_300_57215, &P22346_298};	/*Property = DataSet*/
const Value V22346_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22346_301 = {S22, 0, &V22346_301_57218, &P22346_300};	/*Property = DataUrl*/
const Value V22346_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22346_302 = {S23, 0, &V22346_302_57221, &P22346_301};	/*Property = DeviceOrientation*/
const Value V22346_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22346_304 = {S24, 0, &V22346_304_57227, &P22346_302};	/*Property = FileReader*/
const Value V22346_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22346_305 = {S25, 0, &V22346_305_57231, &P22346_304};	/*Property = FileSaver*/
const Value V22346_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22346_306 = {S26, 0, &V22346_306_57237, &P22346_305};	/*Property = FileWriter*/
const Value V22346_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22346_307 = {S27, 0, &V22346_307_57239, &P22346_306};	/*Property = FormData*/
const Value V22346_309_57245 = {S2, NULL};	/*Value = True*/
const Property P22346_309 = {S28, 0, &V22346_309_57245, &P22346_307};	/*Property = Fullscreen*/
const Value V22346_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22346_310 = {S29, 0, &V22346_310_57248, &P22346_309};	/*Property = GeoLocation*/
const Value V22346_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22346_312 = {S30, 0, &V22346_312_57254, &P22346_310};	/*Property = History*/
const Value V22346_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22346_314 = {S31, 0, &V22346_314_57261, &P22346_312};	/*Property = Html5*/
const Value V22346_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22346_313 = {S32, 0, &V22346_313_57258, &P22346_314};	/*Property = Html-Media-Capture*/
const Value V22346_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22346_315 = {S33, 0, &V22346_315_57264, &P22346_313};	/*Property = Iframe*/
const Value V22346_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22346_316 = {S34, 0, &V22346_316_57266, &P22346_315};	/*Property = IndexedDB*/
const Value V22346_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22346_317 = {S35, 0, &V22346_317_57269, &P22346_316};	/*Property = Json*/
const Value V22346_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22346_15 = {S37, 0, &V22346_15_29, &P22346_317};	/*Property = LayoutEngine*/
const Value V22346_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22346_344 = {S38, 0, &V22346_344_57272, &P22346_15};	/*Property = Masking*/
const Value V22346_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22346_320 = {S39, 0, &V22346_320_57290, &P22346_344};	/*Property = PostMessage*/
const Value V22346_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22346_321 = {S40, 0, &V22346_321_57293, &P22346_320};	/*Property = Progress*/
const Value V22346_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22346_322 = {S41, 0, &V22346_322_57296, &P22346_321};	/*Property = Prompts*/
const Value V22346_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22346_323 = {S42, 0, &V22346_323_57299, &P22346_322};	/*Property = Selector*/
const Value V22346_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22346_324 = {S43, 0, &V22346_324_57305, &P22346_323};	/*Property = Svg*/
const Value V22346_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22346_327 = {S44, 0, &V22346_327_57315, &P22346_324};	/*Property = TouchEvents*/
const Value V22346_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22346_328 = {S45, 0, &V22346_328_57318, &P22346_327};	/*Property = Track*/
const Value V22346_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22346_329 = {S46, 0, &V22346_329_57323, &P22346_328};	/*Property = Video*/
const Value V22346_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22346_330 = {S47, 0, &V22346_330_57326, &P22346_329};	/*Property = Viewport*/
const Value V22346_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22346_332 = {S48, 0, &V22346_332_57339, &P22346_330};	/*Property = WebWorkers*/
const Value V22346_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22346_333 = {S49, 0, &V22346_333_57342, &P22346_332};	/*Property = Xhr2*/
const Profile P22346 = {"22346", &P22346_333};	/*Profile ID - 22346*/
const Value V22507_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_279 = {S1, 0, &V22507_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22507_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_282 = {S3, 0, &V22507_282_57155, &P22507_279};	/*Property = BlobBuilder*/
const Value V22507_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_350 = {S4, 0, &V22507_350_57446, &P22507_282};	/*Property = Canvas*/
const Value V22507_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_283 = {S5, 0, &V22507_283_57161, &P22507_350};	/*Property = CssBackground*/
const Value V22507_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_284 = {S6, 0, &V22507_284_57164, &P22507_283};	/*Property = CssBorderImage*/
const Value V22507_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_285 = {S7, 0, &V22507_285_57167, &P22507_284};	/*Property = CssCanvas*/
const Value V22507_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_286 = {S8, 0, &V22507_286_57170, &P22507_285};	/*Property = CssColor*/
const Value V22507_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_237 = {S9, 0, &V22507_237_55786, &P22507_286};	/*Property = CssColumn*/
const Value V22507_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_287 = {S10, 0, &V22507_287_57173, &P22507_237};	/*Property = CssFlexbox*/
const Value V22507_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_288 = {S11, 0, &V22507_288_57176, &P22507_287};	/*Property = CssFont*/
const Value V22507_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_289 = {S12, 0, &V22507_289_86899, &P22507_288};	/*Property = CssImages*/
const Value V22507_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_291 = {S13, 0, &V22507_291_57182, &P22507_289};	/*Property = CssMediaQueries*/
const Value V22507_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_292 = {S14, 0, &V22507_292_57185, &P22507_291};	/*Property = CssMinMax*/
const Value V22507_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_293 = {S15, 0, &V22507_293_57188, &P22507_292};	/*Property = CssOverflow*/
const Value V22507_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_294 = {S16, 0, &V22507_294_57191, &P22507_293};	/*Property = CssPosition*/
const Value V22507_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_295 = {S17, 0, &V22507_295_57194, &P22507_294};	/*Property = CssText*/
const Value V22507_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_297 = {S18, 0, &V22507_297_57197, &P22507_295};	/*Property = CssTransforms*/
const Value V22507_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_241 = {S19, 0, &V22507_241_55801, &P22507_297};	/*Property = CssTransitions*/
const Value V22507_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_298 = {S20, 0, &V22507_298_57208, &P22507_241};	/*Property = CssUI*/
const Value V22507_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_300 = {S21, 0, &V22507_300_57217, &P22507_298};	/*Property = DataSet*/
const Value V22507_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_301 = {S22, 0, &V22507_301_57220, &P22507_300};	/*Property = DataUrl*/
const Value V22507_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_302 = {S23, 0, &V22507_302_57223, &P22507_301};	/*Property = DeviceOrientation*/
const Value V22507_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_304 = {S24, 0, &V22507_304_57229, &P22507_302};	/*Property = FileReader*/
const Value V22507_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_305 = {S25, 0, &V22507_305_57232, &P22507_304};	/*Property = FileSaver*/
const Value V22507_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_306 = {S26, 0, &V22507_306_57238, &P22507_305};	/*Property = FileWriter*/
const Value V22507_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_307 = {S27, 0, &V22507_307_57241, &P22507_306};	/*Property = FormData*/
const Value V22507_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_309 = {S28, 0, &V22507_309_57247, &P22507_307};	/*Property = Fullscreen*/
const Value V22507_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_310 = {S29, 0, &V22507_310_57250, &P22507_309};	/*Property = GeoLocation*/
const Value V22507_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_312 = {S30, 0, &V22507_312_57256, &P22507_310};	/*Property = History*/
const Value V22507_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_314 = {S31, 0, &V22507_314_57262, &P22507_312};	/*Property = Html5*/
const Value V22507_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_313 = {S32, 0, &V22507_313_57259, &P22507_314};	/*Property = Html-Media-Capture*/
const Value V22507_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_315 = {S33, 0, &V22507_315_57265, &P22507_313};	/*Property = Iframe*/
const Value V22507_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_316 = {S34, 0, &V22507_316_57268, &P22507_315};	/*Property = IndexedDB*/
const Value V22507_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_317 = {S35, 0, &V22507_317_57271, &P22507_316};	/*Property = Json*/
const Value V22507_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22507_15 = {S37, 0, &V22507_15_29, &P22507_317};	/*Property = LayoutEngine*/
const Value V22507_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_344 = {S38, 0, &V22507_344_57274, &P22507_15};	/*Property = Masking*/
const Value V22507_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_320 = {S39, 0, &V22507_320_57292, &P22507_344};	/*Property = PostMessage*/
const Value V22507_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_321 = {S40, 0, &V22507_321_57295, &P22507_320};	/*Property = Progress*/
const Value V22507_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_322 = {S41, 0, &V22507_322_57298, &P22507_321};	/*Property = Prompts*/
const Value V22507_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_323 = {S42, 0, &V22507_323_57301, &P22507_322};	/*Property = Selector*/
const Value V22507_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_324 = {S43, 0, &V22507_324_57307, &P22507_323};	/*Property = Svg*/
const Value V22507_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_327 = {S44, 0, &V22507_327_57316, &P22507_324};	/*Property = TouchEvents*/
const Value V22507_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_328 = {S45, 0, &V22507_328_57319, &P22507_327};	/*Property = Track*/
const Value V22507_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_329 = {S46, 0, &V22507_329_57325, &P22507_328};	/*Property = Video*/
const Value V22507_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_330 = {S47, 0, &V22507_330_57328, &P22507_329};	/*Property = Viewport*/
const Value V22507_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_332 = {S48, 0, &V22507_332_57340, &P22507_330};	/*Property = WebWorkers*/
const Value V22507_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22507_333 = {S49, 0, &V22507_333_57343, &P22507_332};	/*Property = Xhr2*/
const Profile P22507 = {"22507", &P22507_333};	/*Profile ID - 22507*/
const Value V18379_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_279 = {S1, 0, &V18379_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18379_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_282 = {S3, 0, &V18379_282_57155, &P18379_279};	/*Property = BlobBuilder*/
const Value V18379_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_350 = {S4, 0, &V18379_350_57446, &P18379_282};	/*Property = Canvas*/
const Value V18379_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_283 = {S5, 0, &V18379_283_57161, &P18379_350};	/*Property = CssBackground*/
const Value V18379_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_284 = {S6, 0, &V18379_284_57164, &P18379_283};	/*Property = CssBorderImage*/
const Value V18379_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_285 = {S7, 0, &V18379_285_57167, &P18379_284};	/*Property = CssCanvas*/
const Value V18379_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_286 = {S8, 0, &V18379_286_57170, &P18379_285};	/*Property = CssColor*/
const Value V18379_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_237 = {S9, 0, &V18379_237_55786, &P18379_286};	/*Property = CssColumn*/
const Value V18379_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_287 = {S10, 0, &V18379_287_57173, &P18379_237};	/*Property = CssFlexbox*/
const Value V18379_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_288 = {S11, 0, &V18379_288_57176, &P18379_287};	/*Property = CssFont*/
const Value V18379_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_289 = {S12, 0, &V18379_289_86899, &P18379_288};	/*Property = CssImages*/
const Value V18379_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_291 = {S13, 0, &V18379_291_57182, &P18379_289};	/*Property = CssMediaQueries*/
const Value V18379_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_292 = {S14, 0, &V18379_292_57185, &P18379_291};	/*Property = CssMinMax*/
const Value V18379_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_293 = {S15, 0, &V18379_293_57188, &P18379_292};	/*Property = CssOverflow*/
const Value V18379_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_294 = {S16, 0, &V18379_294_57191, &P18379_293};	/*Property = CssPosition*/
const Value V18379_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_295 = {S17, 0, &V18379_295_57194, &P18379_294};	/*Property = CssText*/
const Value V18379_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_297 = {S18, 0, &V18379_297_57197, &P18379_295};	/*Property = CssTransforms*/
const Value V18379_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_241 = {S19, 0, &V18379_241_55801, &P18379_297};	/*Property = CssTransitions*/
const Value V18379_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_298 = {S20, 0, &V18379_298_57208, &P18379_241};	/*Property = CssUI*/
const Value V18379_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_300 = {S21, 0, &V18379_300_57217, &P18379_298};	/*Property = DataSet*/
const Value V18379_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_301 = {S22, 0, &V18379_301_57220, &P18379_300};	/*Property = DataUrl*/
const Value V18379_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_302 = {S23, 0, &V18379_302_57223, &P18379_301};	/*Property = DeviceOrientation*/
const Value V18379_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_304 = {S24, 0, &V18379_304_57229, &P18379_302};	/*Property = FileReader*/
const Value V18379_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_305 = {S25, 0, &V18379_305_57232, &P18379_304};	/*Property = FileSaver*/
const Value V18379_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_306 = {S26, 0, &V18379_306_57238, &P18379_305};	/*Property = FileWriter*/
const Value V18379_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_307 = {S27, 0, &V18379_307_57241, &P18379_306};	/*Property = FormData*/
const Value V18379_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_309 = {S28, 0, &V18379_309_57247, &P18379_307};	/*Property = Fullscreen*/
const Value V18379_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_310 = {S29, 0, &V18379_310_57250, &P18379_309};	/*Property = GeoLocation*/
const Value V18379_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_312 = {S30, 0, &V18379_312_57256, &P18379_310};	/*Property = History*/
const Value V18379_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_314 = {S31, 0, &V18379_314_57262, &P18379_312};	/*Property = Html5*/
const Value V18379_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_313 = {S32, 0, &V18379_313_57259, &P18379_314};	/*Property = Html-Media-Capture*/
const Value V18379_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_315 = {S33, 0, &V18379_315_57265, &P18379_313};	/*Property = Iframe*/
const Value V18379_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_316 = {S34, 0, &V18379_316_57268, &P18379_315};	/*Property = IndexedDB*/
const Value V18379_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_317 = {S35, 0, &V18379_317_57271, &P18379_316};	/*Property = Json*/
const Value V18379_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_15 = {S37, 0, &V18379_15_72, &P18379_317};	/*Property = LayoutEngine*/
const Value V18379_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_344 = {S38, 0, &V18379_344_57274, &P18379_15};	/*Property = Masking*/
const Value V18379_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_320 = {S39, 0, &V18379_320_57292, &P18379_344};	/*Property = PostMessage*/
const Value V18379_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_321 = {S40, 0, &V18379_321_57295, &P18379_320};	/*Property = Progress*/
const Value V18379_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_322 = {S41, 0, &V18379_322_57298, &P18379_321};	/*Property = Prompts*/
const Value V18379_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_323 = {S42, 0, &V18379_323_57301, &P18379_322};	/*Property = Selector*/
const Value V18379_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_324 = {S43, 0, &V18379_324_57307, &P18379_323};	/*Property = Svg*/
const Value V18379_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_327 = {S44, 0, &V18379_327_57316, &P18379_324};	/*Property = TouchEvents*/
const Value V18379_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_328 = {S45, 0, &V18379_328_57319, &P18379_327};	/*Property = Track*/
const Value V18379_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_329 = {S46, 0, &V18379_329_57325, &P18379_328};	/*Property = Video*/
const Value V18379_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_330 = {S47, 0, &V18379_330_57328, &P18379_329};	/*Property = Viewport*/
const Value V18379_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_332 = {S48, 0, &V18379_332_57340, &P18379_330};	/*Property = WebWorkers*/
const Value V18379_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18379_333 = {S49, 0, &V18379_333_57343, &P18379_332};	/*Property = Xhr2*/
const Profile P18379 = {"18379", &P18379_333};	/*Profile ID - 18379*/
const Value V16545_1_1 = {S2, NULL};	/*Value = True*/
const Property P16545_1 = {S50, 0, &V16545_1_1, NULL};	/*Property = IsMobile*/
const Value V16545_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P16545_5 = {S52, 0, &V16545_5_1380, &P16545_1};	/*Property = ScreenPixelsHeight*/
const Value V16545_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16545_4 = {S54, 0, &V16545_4_1279, &P16545_5};	/*Property = ScreenPixelsWidth*/
const Profile P16545 = {"16545", &P16545_4};	/*Profile ID - 16545*/
const Value V22341_1_1 = {S2, NULL};	/*Value = True*/
const Property P22341_1 = {S50, 0, &V22341_1_1, NULL};	/*Property = IsMobile*/
const Value V22341_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P22341_5 = {S52, 0, &V22341_5_1269, &P22341_1};	/*Property = ScreenPixelsHeight*/
const Value V22341_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22341_4 = {S54, 0, &V22341_4_1229, &P22341_5};	/*Property = ScreenPixelsWidth*/
const Profile P22341 = {"22341", &P22341_4};	/*Profile ID - 22341*/
const Value V22392_1_1 = {S2, NULL};	/*Value = True*/
const Property P22392_1 = {S50, 0, &V22392_1_1, NULL};	/*Property = IsMobile*/
const Value V22392_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22392_5 = {S52, 0, &V22392_5_1277, &P22392_1};	/*Property = ScreenPixelsHeight*/
const Value V22392_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22392_4 = {S54, 0, &V22392_4_2336, &P22392_5};	/*Property = ScreenPixelsWidth*/
const Profile P22392 = {"22392", &P22392_4};	/*Profile ID - 22392*/
const Value V21477_1_1 = {S2, NULL};	/*Value = True*/
const Property P21477_1 = {S50, 0, &V21477_1_1, NULL};	/*Property = IsMobile*/
const Value V21477_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21477_5 = {S52, 0, &V21477_5_1227, &P21477_1};	/*Property = ScreenPixelsHeight*/
const Value V21477_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21477_4 = {S54, 0, &V21477_4_1229, &P21477_5};	/*Property = ScreenPixelsWidth*/
const Profile P21477 = {"21477", &P21477_4};	/*Profile ID - 21477*/
const Value V21570_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_279 = {S1, 0, &V21570_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21570_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_282 = {S3, 0, &V21570_282_57155, &P21570_279};	/*Property = BlobBuilder*/
const Value V21570_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_350 = {S4, 0, &V21570_350_57446, &P21570_282};	/*Property = Canvas*/
const Value V21570_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_283 = {S5, 0, &V21570_283_57161, &P21570_350};	/*Property = CssBackground*/
const Value V21570_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_284 = {S6, 0, &V21570_284_57164, &P21570_283};	/*Property = CssBorderImage*/
const Value V21570_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_285 = {S7, 0, &V21570_285_57167, &P21570_284};	/*Property = CssCanvas*/
const Value V21570_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_286 = {S8, 0, &V21570_286_57170, &P21570_285};	/*Property = CssColor*/
const Value V21570_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_237 = {S9, 0, &V21570_237_55786, &P21570_286};	/*Property = CssColumn*/
const Value V21570_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_287 = {S10, 0, &V21570_287_57173, &P21570_237};	/*Property = CssFlexbox*/
const Value V21570_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_288 = {S11, 0, &V21570_288_57176, &P21570_287};	/*Property = CssFont*/
const Value V21570_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_289 = {S12, 0, &V21570_289_86899, &P21570_288};	/*Property = CssImages*/
const Value V21570_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_291 = {S13, 0, &V21570_291_57182, &P21570_289};	/*Property = CssMediaQueries*/
const Value V21570_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_292 = {S14, 0, &V21570_292_57185, &P21570_291};	/*Property = CssMinMax*/
const Value V21570_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_293 = {S15, 0, &V21570_293_57188, &P21570_292};	/*Property = CssOverflow*/
const Value V21570_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_294 = {S16, 0, &V21570_294_57191, &P21570_293};	/*Property = CssPosition*/
const Value V21570_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_295 = {S17, 0, &V21570_295_57194, &P21570_294};	/*Property = CssText*/
const Value V21570_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_297 = {S18, 0, &V21570_297_57197, &P21570_295};	/*Property = CssTransforms*/
const Value V21570_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_241 = {S19, 0, &V21570_241_55801, &P21570_297};	/*Property = CssTransitions*/
const Value V21570_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_298 = {S20, 0, &V21570_298_57208, &P21570_241};	/*Property = CssUI*/
const Value V21570_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_300 = {S21, 0, &V21570_300_57217, &P21570_298};	/*Property = DataSet*/
const Value V21570_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_301 = {S22, 0, &V21570_301_57220, &P21570_300};	/*Property = DataUrl*/
const Value V21570_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_302 = {S23, 0, &V21570_302_57223, &P21570_301};	/*Property = DeviceOrientation*/
const Value V21570_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_304 = {S24, 0, &V21570_304_57229, &P21570_302};	/*Property = FileReader*/
const Value V21570_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_305 = {S25, 0, &V21570_305_57232, &P21570_304};	/*Property = FileSaver*/
const Value V21570_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_306 = {S26, 0, &V21570_306_57238, &P21570_305};	/*Property = FileWriter*/
const Value V21570_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_307 = {S27, 0, &V21570_307_57241, &P21570_306};	/*Property = FormData*/
const Value V21570_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_309 = {S28, 0, &V21570_309_57247, &P21570_307};	/*Property = Fullscreen*/
const Value V21570_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_310 = {S29, 0, &V21570_310_57250, &P21570_309};	/*Property = GeoLocation*/
const Value V21570_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_312 = {S30, 0, &V21570_312_57256, &P21570_310};	/*Property = History*/
const Value V21570_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_314 = {S31, 0, &V21570_314_57262, &P21570_312};	/*Property = Html5*/
const Value V21570_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_313 = {S32, 0, &V21570_313_57259, &P21570_314};	/*Property = Html-Media-Capture*/
const Value V21570_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_315 = {S33, 0, &V21570_315_57265, &P21570_313};	/*Property = Iframe*/
const Value V21570_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_316 = {S34, 0, &V21570_316_57268, &P21570_315};	/*Property = IndexedDB*/
const Value V21570_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_317 = {S35, 0, &V21570_317_57271, &P21570_316};	/*Property = Json*/
const Value V21570_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21570_15 = {S37, 0, &V21570_15_28, &P21570_317};	/*Property = LayoutEngine*/
const Value V21570_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_344 = {S38, 0, &V21570_344_57274, &P21570_15};	/*Property = Masking*/
const Value V21570_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_320 = {S39, 0, &V21570_320_57292, &P21570_344};	/*Property = PostMessage*/
const Value V21570_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_321 = {S40, 0, &V21570_321_57295, &P21570_320};	/*Property = Progress*/
const Value V21570_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_322 = {S41, 0, &V21570_322_57298, &P21570_321};	/*Property = Prompts*/
const Value V21570_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_323 = {S42, 0, &V21570_323_57301, &P21570_322};	/*Property = Selector*/
const Value V21570_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_324 = {S43, 0, &V21570_324_57307, &P21570_323};	/*Property = Svg*/
const Value V21570_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_327 = {S44, 0, &V21570_327_57316, &P21570_324};	/*Property = TouchEvents*/
const Value V21570_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_328 = {S45, 0, &V21570_328_57319, &P21570_327};	/*Property = Track*/
const Value V21570_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_329 = {S46, 0, &V21570_329_57325, &P21570_328};	/*Property = Video*/
const Value V21570_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_330 = {S47, 0, &V21570_330_57328, &P21570_329};	/*Property = Viewport*/
const Value V21570_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_332 = {S48, 0, &V21570_332_57340, &P21570_330};	/*Property = WebWorkers*/
const Value V21570_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21570_333 = {S49, 0, &V21570_333_57343, &P21570_332};	/*Property = Xhr2*/
const Profile P21570 = {"21570", &P21570_333};	/*Profile ID - 21570*/
const Value V22311_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_279 = {S1, 0, &V22311_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22311_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_282 = {S3, 0, &V22311_282_57155, &P22311_279};	/*Property = BlobBuilder*/
const Value V22311_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_350 = {S4, 0, &V22311_350_57446, &P22311_282};	/*Property = Canvas*/
const Value V22311_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_283 = {S5, 0, &V22311_283_57161, &P22311_350};	/*Property = CssBackground*/
const Value V22311_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_284 = {S6, 0, &V22311_284_57164, &P22311_283};	/*Property = CssBorderImage*/
const Value V22311_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_285 = {S7, 0, &V22311_285_57167, &P22311_284};	/*Property = CssCanvas*/
const Value V22311_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_286 = {S8, 0, &V22311_286_57170, &P22311_285};	/*Property = CssColor*/
const Value V22311_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_237 = {S9, 0, &V22311_237_55786, &P22311_286};	/*Property = CssColumn*/
const Value V22311_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_287 = {S10, 0, &V22311_287_57173, &P22311_237};	/*Property = CssFlexbox*/
const Value V22311_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_288 = {S11, 0, &V22311_288_57176, &P22311_287};	/*Property = CssFont*/
const Value V22311_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_289 = {S12, 0, &V22311_289_86899, &P22311_288};	/*Property = CssImages*/
const Value V22311_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_291 = {S13, 0, &V22311_291_57182, &P22311_289};	/*Property = CssMediaQueries*/
const Value V22311_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_292 = {S14, 0, &V22311_292_57185, &P22311_291};	/*Property = CssMinMax*/
const Value V22311_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_293 = {S15, 0, &V22311_293_57188, &P22311_292};	/*Property = CssOverflow*/
const Value V22311_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_294 = {S16, 0, &V22311_294_57191, &P22311_293};	/*Property = CssPosition*/
const Value V22311_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_295 = {S17, 0, &V22311_295_57194, &P22311_294};	/*Property = CssText*/
const Value V22311_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_297 = {S18, 0, &V22311_297_57197, &P22311_295};	/*Property = CssTransforms*/
const Value V22311_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_241 = {S19, 0, &V22311_241_55801, &P22311_297};	/*Property = CssTransitions*/
const Value V22311_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_298 = {S20, 0, &V22311_298_57208, &P22311_241};	/*Property = CssUI*/
const Value V22311_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_300 = {S21, 0, &V22311_300_57217, &P22311_298};	/*Property = DataSet*/
const Value V22311_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_301 = {S22, 0, &V22311_301_57220, &P22311_300};	/*Property = DataUrl*/
const Value V22311_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_302 = {S23, 0, &V22311_302_57223, &P22311_301};	/*Property = DeviceOrientation*/
const Value V22311_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_304 = {S24, 0, &V22311_304_57229, &P22311_302};	/*Property = FileReader*/
const Value V22311_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_305 = {S25, 0, &V22311_305_57232, &P22311_304};	/*Property = FileSaver*/
const Value V22311_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_306 = {S26, 0, &V22311_306_57238, &P22311_305};	/*Property = FileWriter*/
const Value V22311_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_307 = {S27, 0, &V22311_307_57241, &P22311_306};	/*Property = FormData*/
const Value V22311_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_309 = {S28, 0, &V22311_309_57247, &P22311_307};	/*Property = Fullscreen*/
const Value V22311_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_310 = {S29, 0, &V22311_310_57250, &P22311_309};	/*Property = GeoLocation*/
const Value V22311_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_312 = {S30, 0, &V22311_312_57256, &P22311_310};	/*Property = History*/
const Value V22311_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_314 = {S31, 0, &V22311_314_57262, &P22311_312};	/*Property = Html5*/
const Value V22311_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_313 = {S32, 0, &V22311_313_57259, &P22311_314};	/*Property = Html-Media-Capture*/
const Value V22311_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_315 = {S33, 0, &V22311_315_57265, &P22311_313};	/*Property = Iframe*/
const Value V22311_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_316 = {S34, 0, &V22311_316_57268, &P22311_315};	/*Property = IndexedDB*/
const Value V22311_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_317 = {S35, 0, &V22311_317_57271, &P22311_316};	/*Property = Json*/
const Value V22311_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22311_15 = {S37, 0, &V22311_15_29, &P22311_317};	/*Property = LayoutEngine*/
const Value V22311_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_344 = {S38, 0, &V22311_344_57274, &P22311_15};	/*Property = Masking*/
const Value V22311_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_320 = {S39, 0, &V22311_320_57292, &P22311_344};	/*Property = PostMessage*/
const Value V22311_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_321 = {S40, 0, &V22311_321_57295, &P22311_320};	/*Property = Progress*/
const Value V22311_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_322 = {S41, 0, &V22311_322_57298, &P22311_321};	/*Property = Prompts*/
const Value V22311_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_323 = {S42, 0, &V22311_323_57301, &P22311_322};	/*Property = Selector*/
const Value V22311_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_324 = {S43, 0, &V22311_324_57307, &P22311_323};	/*Property = Svg*/
const Value V22311_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_327 = {S44, 0, &V22311_327_57316, &P22311_324};	/*Property = TouchEvents*/
const Value V22311_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_328 = {S45, 0, &V22311_328_57319, &P22311_327};	/*Property = Track*/
const Value V22311_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_329 = {S46, 0, &V22311_329_57325, &P22311_328};	/*Property = Video*/
const Value V22311_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_330 = {S47, 0, &V22311_330_57328, &P22311_329};	/*Property = Viewport*/
const Value V22311_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_332 = {S48, 0, &V22311_332_57340, &P22311_330};	/*Property = WebWorkers*/
const Value V22311_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22311_333 = {S49, 0, &V22311_333_57343, &P22311_332};	/*Property = Xhr2*/
const Profile P22311 = {"22311", &P22311_333};	/*Profile ID - 22311*/
const Value V21807_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_279 = {S1, 0, &V21807_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21807_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_282 = {S3, 0, &V21807_282_57155, &P21807_279};	/*Property = BlobBuilder*/
const Value V21807_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_350 = {S4, 0, &V21807_350_57446, &P21807_282};	/*Property = Canvas*/
const Value V21807_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_283 = {S5, 0, &V21807_283_57161, &P21807_350};	/*Property = CssBackground*/
const Value V21807_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_284 = {S6, 0, &V21807_284_57164, &P21807_283};	/*Property = CssBorderImage*/
const Value V21807_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_285 = {S7, 0, &V21807_285_57167, &P21807_284};	/*Property = CssCanvas*/
const Value V21807_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_286 = {S8, 0, &V21807_286_57170, &P21807_285};	/*Property = CssColor*/
const Value V21807_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_237 = {S9, 0, &V21807_237_55786, &P21807_286};	/*Property = CssColumn*/
const Value V21807_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_287 = {S10, 0, &V21807_287_57173, &P21807_237};	/*Property = CssFlexbox*/
const Value V21807_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_288 = {S11, 0, &V21807_288_57176, &P21807_287};	/*Property = CssFont*/
const Value V21807_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_289 = {S12, 0, &V21807_289_86899, &P21807_288};	/*Property = CssImages*/
const Value V21807_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_291 = {S13, 0, &V21807_291_57182, &P21807_289};	/*Property = CssMediaQueries*/
const Value V21807_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_292 = {S14, 0, &V21807_292_57185, &P21807_291};	/*Property = CssMinMax*/
const Value V21807_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_293 = {S15, 0, &V21807_293_57188, &P21807_292};	/*Property = CssOverflow*/
const Value V21807_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_294 = {S16, 0, &V21807_294_57191, &P21807_293};	/*Property = CssPosition*/
const Value V21807_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_295 = {S17, 0, &V21807_295_57194, &P21807_294};	/*Property = CssText*/
const Value V21807_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_297 = {S18, 0, &V21807_297_57197, &P21807_295};	/*Property = CssTransforms*/
const Value V21807_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_241 = {S19, 0, &V21807_241_55801, &P21807_297};	/*Property = CssTransitions*/
const Value V21807_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_298 = {S20, 0, &V21807_298_57208, &P21807_241};	/*Property = CssUI*/
const Value V21807_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_300 = {S21, 0, &V21807_300_57217, &P21807_298};	/*Property = DataSet*/
const Value V21807_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_301 = {S22, 0, &V21807_301_57220, &P21807_300};	/*Property = DataUrl*/
const Value V21807_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_302 = {S23, 0, &V21807_302_57223, &P21807_301};	/*Property = DeviceOrientation*/
const Value V21807_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_304 = {S24, 0, &V21807_304_57229, &P21807_302};	/*Property = FileReader*/
const Value V21807_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_305 = {S25, 0, &V21807_305_57232, &P21807_304};	/*Property = FileSaver*/
const Value V21807_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_306 = {S26, 0, &V21807_306_57238, &P21807_305};	/*Property = FileWriter*/
const Value V21807_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_307 = {S27, 0, &V21807_307_57241, &P21807_306};	/*Property = FormData*/
const Value V21807_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_309 = {S28, 0, &V21807_309_57247, &P21807_307};	/*Property = Fullscreen*/
const Value V21807_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_310 = {S29, 0, &V21807_310_57250, &P21807_309};	/*Property = GeoLocation*/
const Value V21807_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_312 = {S30, 0, &V21807_312_57256, &P21807_310};	/*Property = History*/
const Value V21807_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_314 = {S31, 0, &V21807_314_57262, &P21807_312};	/*Property = Html5*/
const Value V21807_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_313 = {S32, 0, &V21807_313_57259, &P21807_314};	/*Property = Html-Media-Capture*/
const Value V21807_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_315 = {S33, 0, &V21807_315_57265, &P21807_313};	/*Property = Iframe*/
const Value V21807_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_316 = {S34, 0, &V21807_316_57268, &P21807_315};	/*Property = IndexedDB*/
const Value V21807_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_317 = {S35, 0, &V21807_317_57271, &P21807_316};	/*Property = Json*/
const Value V21807_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P21807_15 = {S37, 0, &V21807_15_29, &P21807_317};	/*Property = LayoutEngine*/
const Value V21807_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_344 = {S38, 0, &V21807_344_57274, &P21807_15};	/*Property = Masking*/
const Value V21807_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_320 = {S39, 0, &V21807_320_57292, &P21807_344};	/*Property = PostMessage*/
const Value V21807_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_321 = {S40, 0, &V21807_321_57295, &P21807_320};	/*Property = Progress*/
const Value V21807_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_322 = {S41, 0, &V21807_322_57298, &P21807_321};	/*Property = Prompts*/
const Value V21807_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_323 = {S42, 0, &V21807_323_57301, &P21807_322};	/*Property = Selector*/
const Value V21807_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_324 = {S43, 0, &V21807_324_57307, &P21807_323};	/*Property = Svg*/
const Value V21807_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_327 = {S44, 0, &V21807_327_57316, &P21807_324};	/*Property = TouchEvents*/
const Value V21807_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_328 = {S45, 0, &V21807_328_57319, &P21807_327};	/*Property = Track*/
const Value V21807_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_329 = {S46, 0, &V21807_329_57325, &P21807_328};	/*Property = Video*/
const Value V21807_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_330 = {S47, 0, &V21807_330_57328, &P21807_329};	/*Property = Viewport*/
const Value V21807_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_332 = {S48, 0, &V21807_332_57340, &P21807_330};	/*Property = WebWorkers*/
const Value V21807_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21807_333 = {S49, 0, &V21807_333_57343, &P21807_332};	/*Property = Xhr2*/
const Profile P21807 = {"21807", &P21807_333};	/*Profile ID - 21807*/
const Value V16725_1_1 = {S2, NULL};	/*Value = True*/
const Property P16725_1 = {S50, 0, &V16725_1_1, NULL};	/*Property = IsMobile*/
const Value V16725_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P16725_5 = {S52, 0, &V16725_5_1304, &P16725_1};	/*Property = ScreenPixelsHeight*/
const Value V16725_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16725_4 = {S54, 0, &V16725_4_1224, &P16725_5};	/*Property = ScreenPixelsWidth*/
const Profile P16725 = {"16725", &P16725_4};	/*Profile ID - 16725*/
const Value V23984_1_1 = {S2, NULL};	/*Value = True*/
const Property P23984_1 = {S50, 0, &V23984_1_1, NULL};	/*Property = IsMobile*/
const Value V23984_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23984_5 = {S52, 0, &V23984_5_1458, &P23984_1};	/*Property = ScreenPixelsHeight*/
const Value V23984_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23984_4 = {S54, 0, &V23984_4_1460, &P23984_5};	/*Property = ScreenPixelsWidth*/
const Profile P23984 = {"23984", &P23984_4};	/*Profile ID - 23984*/
const Value V12599_1_1 = {S2, NULL};	/*Value = True*/
const Property P12599_1 = {S50, 0, &V12599_1_1, NULL};	/*Property = IsMobile*/
const Value V12599_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12599_5 = {S52, 0, &V12599_5_1222, &P12599_1};	/*Property = ScreenPixelsHeight*/
const Value V12599_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12599_4 = {S54, 0, &V12599_4_1224, &P12599_5};	/*Property = ScreenPixelsWidth*/
const Profile P12599 = {"12599", &P12599_4};	/*Profile ID - 12599*/
const Value V21388_1_1 = {S2, NULL};	/*Value = True*/
const Property P21388_1 = {S50, 0, &V21388_1_1, NULL};	/*Property = IsMobile*/
const Value V21388_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21388_5 = {S52, 0, &V21388_5_1222, &P21388_1};	/*Property = ScreenPixelsHeight*/
const Value V21388_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21388_4 = {S54, 0, &V21388_4_1224, &P21388_5};	/*Property = ScreenPixelsWidth*/
const Profile P21388 = {"21388", &P21388_4};	/*Profile ID - 21388*/
const Value V23857_1_1 = {S2, NULL};	/*Value = True*/
const Property P23857_1 = {S50, 0, &V23857_1_1, NULL};	/*Property = IsMobile*/
const Value V23857_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23857_5 = {S52, 0, &V23857_5_1277, &P23857_1};	/*Property = ScreenPixelsHeight*/
const Value V23857_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23857_4 = {S54, 0, &V23857_4_2336, &P23857_5};	/*Property = ScreenPixelsWidth*/
const Profile P23857 = {"23857", &P23857_4};	/*Profile ID - 23857*/
const Value V21204_1_1 = {S2, NULL};	/*Value = True*/
const Property P21204_1 = {S50, 0, &V21204_1_1, NULL};	/*Property = IsMobile*/
const Value V21204_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21204_5 = {S52, 0, &V21204_5_37, &P21204_1};	/*Property = ScreenPixelsHeight*/
const Value V21204_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21204_4 = {S54, 0, &V21204_4_36, &P21204_5};	/*Property = ScreenPixelsWidth*/
const Profile P21204 = {"21204", &P21204_4};	/*Profile ID - 21204*/
const Value V15936_1_1 = {S2, NULL};	/*Value = True*/
const Property P15936_1 = {S50, 0, &V15936_1_1, NULL};	/*Property = IsMobile*/
const Value V15936_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15936_5 = {S52, 0, &V15936_5_1222, &P15936_1};	/*Property = ScreenPixelsHeight*/
const Value V15936_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15936_4 = {S54, 0, &V15936_4_1224, &P15936_5};	/*Property = ScreenPixelsWidth*/
const Profile P15936 = {"15936", &P15936_4};	/*Profile ID - 15936*/
const Value V21581_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_279 = {S1, 0, &V21581_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21581_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_282 = {S3, 0, &V21581_282_57155, &P21581_279};	/*Property = BlobBuilder*/
const Value V21581_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_350 = {S4, 0, &V21581_350_57446, &P21581_282};	/*Property = Canvas*/
const Value V21581_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_283 = {S5, 0, &V21581_283_57161, &P21581_350};	/*Property = CssBackground*/
const Value V21581_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_284 = {S6, 0, &V21581_284_57164, &P21581_283};	/*Property = CssBorderImage*/
const Value V21581_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_285 = {S7, 0, &V21581_285_57167, &P21581_284};	/*Property = CssCanvas*/
const Value V21581_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_286 = {S8, 0, &V21581_286_57170, &P21581_285};	/*Property = CssColor*/
const Value V21581_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_237 = {S9, 0, &V21581_237_55786, &P21581_286};	/*Property = CssColumn*/
const Value V21581_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_287 = {S10, 0, &V21581_287_57173, &P21581_237};	/*Property = CssFlexbox*/
const Value V21581_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_288 = {S11, 0, &V21581_288_57176, &P21581_287};	/*Property = CssFont*/
const Value V21581_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_289 = {S12, 0, &V21581_289_86899, &P21581_288};	/*Property = CssImages*/
const Value V21581_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_291 = {S13, 0, &V21581_291_57182, &P21581_289};	/*Property = CssMediaQueries*/
const Value V21581_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_292 = {S14, 0, &V21581_292_57185, &P21581_291};	/*Property = CssMinMax*/
const Value V21581_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_293 = {S15, 0, &V21581_293_57188, &P21581_292};	/*Property = CssOverflow*/
const Value V21581_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_294 = {S16, 0, &V21581_294_57191, &P21581_293};	/*Property = CssPosition*/
const Value V21581_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_295 = {S17, 0, &V21581_295_57194, &P21581_294};	/*Property = CssText*/
const Value V21581_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_297 = {S18, 0, &V21581_297_57197, &P21581_295};	/*Property = CssTransforms*/
const Value V21581_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_241 = {S19, 0, &V21581_241_55801, &P21581_297};	/*Property = CssTransitions*/
const Value V21581_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_298 = {S20, 0, &V21581_298_57208, &P21581_241};	/*Property = CssUI*/
const Value V21581_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_300 = {S21, 0, &V21581_300_57217, &P21581_298};	/*Property = DataSet*/
const Value V21581_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_301 = {S22, 0, &V21581_301_57220, &P21581_300};	/*Property = DataUrl*/
const Value V21581_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_302 = {S23, 0, &V21581_302_57223, &P21581_301};	/*Property = DeviceOrientation*/
const Value V21581_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_304 = {S24, 0, &V21581_304_57229, &P21581_302};	/*Property = FileReader*/
const Value V21581_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_305 = {S25, 0, &V21581_305_57232, &P21581_304};	/*Property = FileSaver*/
const Value V21581_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_306 = {S26, 0, &V21581_306_57238, &P21581_305};	/*Property = FileWriter*/
const Value V21581_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_307 = {S27, 0, &V21581_307_57241, &P21581_306};	/*Property = FormData*/
const Value V21581_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_309 = {S28, 0, &V21581_309_57247, &P21581_307};	/*Property = Fullscreen*/
const Value V21581_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_310 = {S29, 0, &V21581_310_57250, &P21581_309};	/*Property = GeoLocation*/
const Value V21581_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_312 = {S30, 0, &V21581_312_57256, &P21581_310};	/*Property = History*/
const Value V21581_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_314 = {S31, 0, &V21581_314_57262, &P21581_312};	/*Property = Html5*/
const Value V21581_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_313 = {S32, 0, &V21581_313_57259, &P21581_314};	/*Property = Html-Media-Capture*/
const Value V21581_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_315 = {S33, 0, &V21581_315_57265, &P21581_313};	/*Property = Iframe*/
const Value V21581_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_316 = {S34, 0, &V21581_316_57268, &P21581_315};	/*Property = IndexedDB*/
const Value V21581_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_317 = {S35, 0, &V21581_317_57271, &P21581_316};	/*Property = Json*/
const Value V21581_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_15 = {S37, 0, &V21581_15_72, &P21581_317};	/*Property = LayoutEngine*/
const Value V21581_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_344 = {S38, 0, &V21581_344_57274, &P21581_15};	/*Property = Masking*/
const Value V21581_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_320 = {S39, 0, &V21581_320_57292, &P21581_344};	/*Property = PostMessage*/
const Value V21581_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_321 = {S40, 0, &V21581_321_57295, &P21581_320};	/*Property = Progress*/
const Value V21581_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_322 = {S41, 0, &V21581_322_57298, &P21581_321};	/*Property = Prompts*/
const Value V21581_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_323 = {S42, 0, &V21581_323_57301, &P21581_322};	/*Property = Selector*/
const Value V21581_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_324 = {S43, 0, &V21581_324_57307, &P21581_323};	/*Property = Svg*/
const Value V21581_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_327 = {S44, 0, &V21581_327_57316, &P21581_324};	/*Property = TouchEvents*/
const Value V21581_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_328 = {S45, 0, &V21581_328_57319, &P21581_327};	/*Property = Track*/
const Value V21581_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_329 = {S46, 0, &V21581_329_57325, &P21581_328};	/*Property = Video*/
const Value V21581_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_330 = {S47, 0, &V21581_330_57328, &P21581_329};	/*Property = Viewport*/
const Value V21581_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_332 = {S48, 0, &V21581_332_57340, &P21581_330};	/*Property = WebWorkers*/
const Value V21581_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21581_333 = {S49, 0, &V21581_333_57343, &P21581_332};	/*Property = Xhr2*/
const Profile P21581 = {"21581", &P21581_333};	/*Profile ID - 21581*/
const Value V21573_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_279 = {S1, 0, &V21573_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21573_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_282 = {S3, 0, &V21573_282_57155, &P21573_279};	/*Property = BlobBuilder*/
const Value V21573_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_350 = {S4, 0, &V21573_350_57446, &P21573_282};	/*Property = Canvas*/
const Value V21573_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_283 = {S5, 0, &V21573_283_57161, &P21573_350};	/*Property = CssBackground*/
const Value V21573_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_284 = {S6, 0, &V21573_284_57164, &P21573_283};	/*Property = CssBorderImage*/
const Value V21573_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_285 = {S7, 0, &V21573_285_57167, &P21573_284};	/*Property = CssCanvas*/
const Value V21573_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_286 = {S8, 0, &V21573_286_57170, &P21573_285};	/*Property = CssColor*/
const Value V21573_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_237 = {S9, 0, &V21573_237_55786, &P21573_286};	/*Property = CssColumn*/
const Value V21573_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_287 = {S10, 0, &V21573_287_57173, &P21573_237};	/*Property = CssFlexbox*/
const Value V21573_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_288 = {S11, 0, &V21573_288_57176, &P21573_287};	/*Property = CssFont*/
const Value V21573_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_289 = {S12, 0, &V21573_289_86899, &P21573_288};	/*Property = CssImages*/
const Value V21573_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_291 = {S13, 0, &V21573_291_57182, &P21573_289};	/*Property = CssMediaQueries*/
const Value V21573_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_292 = {S14, 0, &V21573_292_57185, &P21573_291};	/*Property = CssMinMax*/
const Value V21573_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_293 = {S15, 0, &V21573_293_57188, &P21573_292};	/*Property = CssOverflow*/
const Value V21573_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_294 = {S16, 0, &V21573_294_57191, &P21573_293};	/*Property = CssPosition*/
const Value V21573_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_295 = {S17, 0, &V21573_295_57194, &P21573_294};	/*Property = CssText*/
const Value V21573_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_297 = {S18, 0, &V21573_297_57197, &P21573_295};	/*Property = CssTransforms*/
const Value V21573_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_241 = {S19, 0, &V21573_241_55801, &P21573_297};	/*Property = CssTransitions*/
const Value V21573_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_298 = {S20, 0, &V21573_298_57208, &P21573_241};	/*Property = CssUI*/
const Value V21573_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_300 = {S21, 0, &V21573_300_57217, &P21573_298};	/*Property = DataSet*/
const Value V21573_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_301 = {S22, 0, &V21573_301_57220, &P21573_300};	/*Property = DataUrl*/
const Value V21573_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_302 = {S23, 0, &V21573_302_57223, &P21573_301};	/*Property = DeviceOrientation*/
const Value V21573_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_304 = {S24, 0, &V21573_304_57229, &P21573_302};	/*Property = FileReader*/
const Value V21573_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_305 = {S25, 0, &V21573_305_57232, &P21573_304};	/*Property = FileSaver*/
const Value V21573_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_306 = {S26, 0, &V21573_306_57238, &P21573_305};	/*Property = FileWriter*/
const Value V21573_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_307 = {S27, 0, &V21573_307_57241, &P21573_306};	/*Property = FormData*/
const Value V21573_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_309 = {S28, 0, &V21573_309_57247, &P21573_307};	/*Property = Fullscreen*/
const Value V21573_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_310 = {S29, 0, &V21573_310_57250, &P21573_309};	/*Property = GeoLocation*/
const Value V21573_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_312 = {S30, 0, &V21573_312_57256, &P21573_310};	/*Property = History*/
const Value V21573_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_314 = {S31, 0, &V21573_314_57262, &P21573_312};	/*Property = Html5*/
const Value V21573_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_313 = {S32, 0, &V21573_313_57259, &P21573_314};	/*Property = Html-Media-Capture*/
const Value V21573_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_315 = {S33, 0, &V21573_315_57265, &P21573_313};	/*Property = Iframe*/
const Value V21573_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_316 = {S34, 0, &V21573_316_57268, &P21573_315};	/*Property = IndexedDB*/
const Value V21573_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_317 = {S35, 0, &V21573_317_57271, &P21573_316};	/*Property = Json*/
const Value V21573_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21573_15 = {S37, 0, &V21573_15_28, &P21573_317};	/*Property = LayoutEngine*/
const Value V21573_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_344 = {S38, 0, &V21573_344_57274, &P21573_15};	/*Property = Masking*/
const Value V21573_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_320 = {S39, 0, &V21573_320_57292, &P21573_344};	/*Property = PostMessage*/
const Value V21573_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_321 = {S40, 0, &V21573_321_57295, &P21573_320};	/*Property = Progress*/
const Value V21573_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_322 = {S41, 0, &V21573_322_57298, &P21573_321};	/*Property = Prompts*/
const Value V21573_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_323 = {S42, 0, &V21573_323_57301, &P21573_322};	/*Property = Selector*/
const Value V21573_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_324 = {S43, 0, &V21573_324_57307, &P21573_323};	/*Property = Svg*/
const Value V21573_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_327 = {S44, 0, &V21573_327_57316, &P21573_324};	/*Property = TouchEvents*/
const Value V21573_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_328 = {S45, 0, &V21573_328_57319, &P21573_327};	/*Property = Track*/
const Value V21573_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_329 = {S46, 0, &V21573_329_57325, &P21573_328};	/*Property = Video*/
const Value V21573_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_330 = {S47, 0, &V21573_330_57328, &P21573_329};	/*Property = Viewport*/
const Value V21573_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_332 = {S48, 0, &V21573_332_57340, &P21573_330};	/*Property = WebWorkers*/
const Value V21573_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21573_333 = {S49, 0, &V21573_333_57343, &P21573_332};	/*Property = Xhr2*/
const Profile P21573 = {"21573", &P21573_333};	/*Profile ID - 21573*/
const Value V21316_1_1 = {S2, NULL};	/*Value = True*/
const Property P21316_1 = {S50, 0, &V21316_1_1, NULL};	/*Property = IsMobile*/
const Value V21316_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21316_5 = {S52, 0, &V21316_5_1227, &P21316_1};	/*Property = ScreenPixelsHeight*/
const Value V21316_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21316_4 = {S54, 0, &V21316_4_1229, &P21316_5};	/*Property = ScreenPixelsWidth*/
const Profile P21316 = {"21316", &P21316_4};	/*Profile ID - 21316*/
const Value V20931_1_1 = {S2, NULL};	/*Value = True*/
const Property P20931_1 = {S50, 0, &V20931_1_1, NULL};	/*Property = IsMobile*/
const Value V20931_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20931_5 = {S52, 0, &V20931_5_1458, &P20931_1};	/*Property = ScreenPixelsHeight*/
const Value V20931_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20931_4 = {S54, 0, &V20931_4_1460, &P20931_5};	/*Property = ScreenPixelsWidth*/
const Profile P20931 = {"20931", &P20931_4};	/*Profile ID - 20931*/
const Value V16246_1_1 = {S2, NULL};	/*Value = True*/
const Property P16246_1 = {S50, 0, &V16246_1_1, NULL};	/*Property = IsMobile*/
const Value V16246_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16246_5 = {S52, 0, &V16246_5_1227, &P16246_1};	/*Property = ScreenPixelsHeight*/
const Value V16246_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16246_4 = {S54, 0, &V16246_4_1229, &P16246_5};	/*Property = ScreenPixelsWidth*/
const Profile P16246 = {"16246", &P16246_4};	/*Profile ID - 16246*/
const Value V21798_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_279 = {S1, 0, &V21798_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21798_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_282 = {S3, 0, &V21798_282_57155, &P21798_279};	/*Property = BlobBuilder*/
const Value V21798_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_350 = {S4, 0, &V21798_350_57446, &P21798_282};	/*Property = Canvas*/
const Value V21798_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_283 = {S5, 0, &V21798_283_57161, &P21798_350};	/*Property = CssBackground*/
const Value V21798_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_284 = {S6, 0, &V21798_284_57164, &P21798_283};	/*Property = CssBorderImage*/
const Value V21798_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_285 = {S7, 0, &V21798_285_57167, &P21798_284};	/*Property = CssCanvas*/
const Value V21798_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_286 = {S8, 0, &V21798_286_57170, &P21798_285};	/*Property = CssColor*/
const Value V21798_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_237 = {S9, 0, &V21798_237_55786, &P21798_286};	/*Property = CssColumn*/
const Value V21798_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_287 = {S10, 0, &V21798_287_57173, &P21798_237};	/*Property = CssFlexbox*/
const Value V21798_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_288 = {S11, 0, &V21798_288_57176, &P21798_287};	/*Property = CssFont*/
const Value V21798_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_289 = {S12, 0, &V21798_289_86899, &P21798_288};	/*Property = CssImages*/
const Value V21798_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_291 = {S13, 0, &V21798_291_57182, &P21798_289};	/*Property = CssMediaQueries*/
const Value V21798_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_292 = {S14, 0, &V21798_292_57185, &P21798_291};	/*Property = CssMinMax*/
const Value V21798_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_293 = {S15, 0, &V21798_293_57188, &P21798_292};	/*Property = CssOverflow*/
const Value V21798_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_294 = {S16, 0, &V21798_294_57191, &P21798_293};	/*Property = CssPosition*/
const Value V21798_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_295 = {S17, 0, &V21798_295_57194, &P21798_294};	/*Property = CssText*/
const Value V21798_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_297 = {S18, 0, &V21798_297_57197, &P21798_295};	/*Property = CssTransforms*/
const Value V21798_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_241 = {S19, 0, &V21798_241_55801, &P21798_297};	/*Property = CssTransitions*/
const Value V21798_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_298 = {S20, 0, &V21798_298_57208, &P21798_241};	/*Property = CssUI*/
const Value V21798_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_300 = {S21, 0, &V21798_300_57217, &P21798_298};	/*Property = DataSet*/
const Value V21798_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_301 = {S22, 0, &V21798_301_57220, &P21798_300};	/*Property = DataUrl*/
const Value V21798_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_302 = {S23, 0, &V21798_302_57223, &P21798_301};	/*Property = DeviceOrientation*/
const Value V21798_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_304 = {S24, 0, &V21798_304_57229, &P21798_302};	/*Property = FileReader*/
const Value V21798_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_305 = {S25, 0, &V21798_305_57232, &P21798_304};	/*Property = FileSaver*/
const Value V21798_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_306 = {S26, 0, &V21798_306_57238, &P21798_305};	/*Property = FileWriter*/
const Value V21798_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_307 = {S27, 0, &V21798_307_57241, &P21798_306};	/*Property = FormData*/
const Value V21798_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_309 = {S28, 0, &V21798_309_57247, &P21798_307};	/*Property = Fullscreen*/
const Value V21798_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_310 = {S29, 0, &V21798_310_57250, &P21798_309};	/*Property = GeoLocation*/
const Value V21798_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_312 = {S30, 0, &V21798_312_57256, &P21798_310};	/*Property = History*/
const Value V21798_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_314 = {S31, 0, &V21798_314_57262, &P21798_312};	/*Property = Html5*/
const Value V21798_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_313 = {S32, 0, &V21798_313_57259, &P21798_314};	/*Property = Html-Media-Capture*/
const Value V21798_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_315 = {S33, 0, &V21798_315_57265, &P21798_313};	/*Property = Iframe*/
const Value V21798_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_316 = {S34, 0, &V21798_316_57268, &P21798_315};	/*Property = IndexedDB*/
const Value V21798_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_317 = {S35, 0, &V21798_317_57271, &P21798_316};	/*Property = Json*/
const Value V21798_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21798_15 = {S37, 0, &V21798_15_54167, &P21798_317};	/*Property = LayoutEngine*/
const Value V21798_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_344 = {S38, 0, &V21798_344_57274, &P21798_15};	/*Property = Masking*/
const Value V21798_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_320 = {S39, 0, &V21798_320_57292, &P21798_344};	/*Property = PostMessage*/
const Value V21798_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_321 = {S40, 0, &V21798_321_57295, &P21798_320};	/*Property = Progress*/
const Value V21798_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_322 = {S41, 0, &V21798_322_57298, &P21798_321};	/*Property = Prompts*/
const Value V21798_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_323 = {S42, 0, &V21798_323_57301, &P21798_322};	/*Property = Selector*/
const Value V21798_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_324 = {S43, 0, &V21798_324_57307, &P21798_323};	/*Property = Svg*/
const Value V21798_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_327 = {S44, 0, &V21798_327_57316, &P21798_324};	/*Property = TouchEvents*/
const Value V21798_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_328 = {S45, 0, &V21798_328_57319, &P21798_327};	/*Property = Track*/
const Value V21798_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_329 = {S46, 0, &V21798_329_57325, &P21798_328};	/*Property = Video*/
const Value V21798_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_330 = {S47, 0, &V21798_330_57328, &P21798_329};	/*Property = Viewport*/
const Value V21798_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_332 = {S48, 0, &V21798_332_57340, &P21798_330};	/*Property = WebWorkers*/
const Value V21798_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21798_333 = {S49, 0, &V21798_333_57343, &P21798_332};	/*Property = Xhr2*/
const Profile P21798 = {"21798", &P21798_333};	/*Profile ID - 21798*/
const Value V18421_1_1 = {S2, NULL};	/*Value = True*/
const Property P18421_1 = {S50, 0, &V18421_1_1, NULL};	/*Property = IsMobile*/
const Value V18421_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18421_5 = {S52, 0, &V18421_5_1277, &P18421_1};	/*Property = ScreenPixelsHeight*/
const Value V18421_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18421_4 = {S54, 0, &V18421_4_2336, &P18421_5};	/*Property = ScreenPixelsWidth*/
const Profile P18421 = {"18421", &P18421_4};	/*Profile ID - 18421*/
const Value V20720_1_1 = {S2, NULL};	/*Value = True*/
const Property P20720_1 = {S50, 0, &V20720_1_1, NULL};	/*Property = IsMobile*/
const Value V20720_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20720_5 = {S52, 0, &V20720_5_1227, &P20720_1};	/*Property = ScreenPixelsHeight*/
const Value V20720_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20720_4 = {S54, 0, &V20720_4_1229, &P20720_5};	/*Property = ScreenPixelsWidth*/
const Profile P20720 = {"20720", &P20720_4};	/*Profile ID - 20720*/
const Value V17437_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_279 = {S1, 0, &V17437_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17437_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_282 = {S3, 0, &V17437_282_57155, &P17437_279};	/*Property = BlobBuilder*/
const Value V17437_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_350 = {S4, 0, &V17437_350_57446, &P17437_282};	/*Property = Canvas*/
const Value V17437_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_283 = {S5, 0, &V17437_283_57161, &P17437_350};	/*Property = CssBackground*/
const Value V17437_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_284 = {S6, 0, &V17437_284_57164, &P17437_283};	/*Property = CssBorderImage*/
const Value V17437_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_285 = {S7, 0, &V17437_285_57167, &P17437_284};	/*Property = CssCanvas*/
const Value V17437_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_286 = {S8, 0, &V17437_286_57170, &P17437_285};	/*Property = CssColor*/
const Value V17437_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_237 = {S9, 0, &V17437_237_55786, &P17437_286};	/*Property = CssColumn*/
const Value V17437_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_287 = {S10, 0, &V17437_287_57173, &P17437_237};	/*Property = CssFlexbox*/
const Value V17437_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_288 = {S11, 0, &V17437_288_57176, &P17437_287};	/*Property = CssFont*/
const Value V17437_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_289 = {S12, 0, &V17437_289_86899, &P17437_288};	/*Property = CssImages*/
const Value V17437_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_291 = {S13, 0, &V17437_291_57182, &P17437_289};	/*Property = CssMediaQueries*/
const Value V17437_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_292 = {S14, 0, &V17437_292_57185, &P17437_291};	/*Property = CssMinMax*/
const Value V17437_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_293 = {S15, 0, &V17437_293_57188, &P17437_292};	/*Property = CssOverflow*/
const Value V17437_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_294 = {S16, 0, &V17437_294_57191, &P17437_293};	/*Property = CssPosition*/
const Value V17437_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_295 = {S17, 0, &V17437_295_57194, &P17437_294};	/*Property = CssText*/
const Value V17437_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_297 = {S18, 0, &V17437_297_57197, &P17437_295};	/*Property = CssTransforms*/
const Value V17437_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_241 = {S19, 0, &V17437_241_55801, &P17437_297};	/*Property = CssTransitions*/
const Value V17437_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_298 = {S20, 0, &V17437_298_57208, &P17437_241};	/*Property = CssUI*/
const Value V17437_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_300 = {S21, 0, &V17437_300_57217, &P17437_298};	/*Property = DataSet*/
const Value V17437_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_301 = {S22, 0, &V17437_301_57220, &P17437_300};	/*Property = DataUrl*/
const Value V17437_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_302 = {S23, 0, &V17437_302_57223, &P17437_301};	/*Property = DeviceOrientation*/
const Value V17437_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_304 = {S24, 0, &V17437_304_57229, &P17437_302};	/*Property = FileReader*/
const Value V17437_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_305 = {S25, 0, &V17437_305_57232, &P17437_304};	/*Property = FileSaver*/
const Value V17437_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_306 = {S26, 0, &V17437_306_57238, &P17437_305};	/*Property = FileWriter*/
const Value V17437_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_307 = {S27, 0, &V17437_307_57241, &P17437_306};	/*Property = FormData*/
const Value V17437_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_309 = {S28, 0, &V17437_309_57247, &P17437_307};	/*Property = Fullscreen*/
const Value V17437_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_310 = {S29, 0, &V17437_310_57250, &P17437_309};	/*Property = GeoLocation*/
const Value V17437_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_312 = {S30, 0, &V17437_312_57256, &P17437_310};	/*Property = History*/
const Value V17437_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_314 = {S31, 0, &V17437_314_57262, &P17437_312};	/*Property = Html5*/
const Value V17437_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_313 = {S32, 0, &V17437_313_57259, &P17437_314};	/*Property = Html-Media-Capture*/
const Value V17437_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_315 = {S33, 0, &V17437_315_57265, &P17437_313};	/*Property = Iframe*/
const Value V17437_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_316 = {S34, 0, &V17437_316_57268, &P17437_315};	/*Property = IndexedDB*/
const Value V17437_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_317 = {S35, 0, &V17437_317_57271, &P17437_316};	/*Property = Json*/
const Value V17437_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P17437_15 = {S37, 0, &V17437_15_29, &P17437_317};	/*Property = LayoutEngine*/
const Value V17437_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_344 = {S38, 0, &V17437_344_57274, &P17437_15};	/*Property = Masking*/
const Value V17437_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_320 = {S39, 0, &V17437_320_57292, &P17437_344};	/*Property = PostMessage*/
const Value V17437_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_321 = {S40, 0, &V17437_321_57295, &P17437_320};	/*Property = Progress*/
const Value V17437_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_322 = {S41, 0, &V17437_322_57298, &P17437_321};	/*Property = Prompts*/
const Value V17437_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_323 = {S42, 0, &V17437_323_57301, &P17437_322};	/*Property = Selector*/
const Value V17437_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_324 = {S43, 0, &V17437_324_57307, &P17437_323};	/*Property = Svg*/
const Value V17437_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_327 = {S44, 0, &V17437_327_57316, &P17437_324};	/*Property = TouchEvents*/
const Value V17437_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_328 = {S45, 0, &V17437_328_57319, &P17437_327};	/*Property = Track*/
const Value V17437_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_329 = {S46, 0, &V17437_329_57325, &P17437_328};	/*Property = Video*/
const Value V17437_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_330 = {S47, 0, &V17437_330_57328, &P17437_329};	/*Property = Viewport*/
const Value V17437_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_332 = {S48, 0, &V17437_332_57340, &P17437_330};	/*Property = WebWorkers*/
const Value V17437_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17437_333 = {S49, 0, &V17437_333_57343, &P17437_332};	/*Property = Xhr2*/
const Profile P17437 = {"17437", &P17437_333};	/*Profile ID - 17437*/
const Value V18193_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_279 = {S1, 0, &V18193_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18193_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_282 = {S3, 0, &V18193_282_57155, &P18193_279};	/*Property = BlobBuilder*/
const Value V18193_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_350 = {S4, 0, &V18193_350_57446, &P18193_282};	/*Property = Canvas*/
const Value V18193_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_283 = {S5, 0, &V18193_283_57161, &P18193_350};	/*Property = CssBackground*/
const Value V18193_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_284 = {S6, 0, &V18193_284_57164, &P18193_283};	/*Property = CssBorderImage*/
const Value V18193_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_285 = {S7, 0, &V18193_285_57167, &P18193_284};	/*Property = CssCanvas*/
const Value V18193_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_286 = {S8, 0, &V18193_286_57170, &P18193_285};	/*Property = CssColor*/
const Value V18193_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_237 = {S9, 0, &V18193_237_55786, &P18193_286};	/*Property = CssColumn*/
const Value V18193_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_287 = {S10, 0, &V18193_287_57173, &P18193_237};	/*Property = CssFlexbox*/
const Value V18193_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_288 = {S11, 0, &V18193_288_57176, &P18193_287};	/*Property = CssFont*/
const Value V18193_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_289 = {S12, 0, &V18193_289_86899, &P18193_288};	/*Property = CssImages*/
const Value V18193_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_291 = {S13, 0, &V18193_291_57182, &P18193_289};	/*Property = CssMediaQueries*/
const Value V18193_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_292 = {S14, 0, &V18193_292_57185, &P18193_291};	/*Property = CssMinMax*/
const Value V18193_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_293 = {S15, 0, &V18193_293_57188, &P18193_292};	/*Property = CssOverflow*/
const Value V18193_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_294 = {S16, 0, &V18193_294_57191, &P18193_293};	/*Property = CssPosition*/
const Value V18193_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_295 = {S17, 0, &V18193_295_57194, &P18193_294};	/*Property = CssText*/
const Value V18193_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_297 = {S18, 0, &V18193_297_57197, &P18193_295};	/*Property = CssTransforms*/
const Value V18193_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_241 = {S19, 0, &V18193_241_55801, &P18193_297};	/*Property = CssTransitions*/
const Value V18193_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_298 = {S20, 0, &V18193_298_57208, &P18193_241};	/*Property = CssUI*/
const Value V18193_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_300 = {S21, 0, &V18193_300_57217, &P18193_298};	/*Property = DataSet*/
const Value V18193_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_301 = {S22, 0, &V18193_301_57220, &P18193_300};	/*Property = DataUrl*/
const Value V18193_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_302 = {S23, 0, &V18193_302_57223, &P18193_301};	/*Property = DeviceOrientation*/
const Value V18193_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_304 = {S24, 0, &V18193_304_57229, &P18193_302};	/*Property = FileReader*/
const Value V18193_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_305 = {S25, 0, &V18193_305_57232, &P18193_304};	/*Property = FileSaver*/
const Value V18193_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_306 = {S26, 0, &V18193_306_57238, &P18193_305};	/*Property = FileWriter*/
const Value V18193_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_307 = {S27, 0, &V18193_307_57241, &P18193_306};	/*Property = FormData*/
const Value V18193_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_309 = {S28, 0, &V18193_309_57247, &P18193_307};	/*Property = Fullscreen*/
const Value V18193_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_310 = {S29, 0, &V18193_310_57250, &P18193_309};	/*Property = GeoLocation*/
const Value V18193_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_312 = {S30, 0, &V18193_312_57256, &P18193_310};	/*Property = History*/
const Value V18193_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_314 = {S31, 0, &V18193_314_57262, &P18193_312};	/*Property = Html5*/
const Value V18193_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_313 = {S32, 0, &V18193_313_57259, &P18193_314};	/*Property = Html-Media-Capture*/
const Value V18193_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_315 = {S33, 0, &V18193_315_57265, &P18193_313};	/*Property = Iframe*/
const Value V18193_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_316 = {S34, 0, &V18193_316_57268, &P18193_315};	/*Property = IndexedDB*/
const Value V18193_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_317 = {S35, 0, &V18193_317_57271, &P18193_316};	/*Property = Json*/
const Value V18193_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18193_15 = {S37, 0, &V18193_15_29, &P18193_317};	/*Property = LayoutEngine*/
const Value V18193_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_344 = {S38, 0, &V18193_344_57274, &P18193_15};	/*Property = Masking*/
const Value V18193_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_320 = {S39, 0, &V18193_320_57292, &P18193_344};	/*Property = PostMessage*/
const Value V18193_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_321 = {S40, 0, &V18193_321_57295, &P18193_320};	/*Property = Progress*/
const Value V18193_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_322 = {S41, 0, &V18193_322_57298, &P18193_321};	/*Property = Prompts*/
const Value V18193_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_323 = {S42, 0, &V18193_323_57301, &P18193_322};	/*Property = Selector*/
const Value V18193_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_324 = {S43, 0, &V18193_324_57307, &P18193_323};	/*Property = Svg*/
const Value V18193_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_327 = {S44, 0, &V18193_327_57316, &P18193_324};	/*Property = TouchEvents*/
const Value V18193_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_328 = {S45, 0, &V18193_328_57319, &P18193_327};	/*Property = Track*/
const Value V18193_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_329 = {S46, 0, &V18193_329_57325, &P18193_328};	/*Property = Video*/
const Value V18193_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_330 = {S47, 0, &V18193_330_57328, &P18193_329};	/*Property = Viewport*/
const Value V18193_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_332 = {S48, 0, &V18193_332_57340, &P18193_330};	/*Property = WebWorkers*/
const Value V18193_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18193_333 = {S49, 0, &V18193_333_57343, &P18193_332};	/*Property = Xhr2*/
const Profile P18193 = {"18193", &P18193_333};	/*Profile ID - 18193*/
const Value V24686_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P24686_15 = {S37, 0, &V24686_15_29, NULL};	/*Property = LayoutEngine*/
const Profile P24686 = {"24686", &P24686_15};	/*Profile ID - 24686*/
const Value V18731_1_1 = {S2, NULL};	/*Value = True*/
const Property P18731_1 = {S50, 0, &V18731_1_1, NULL};	/*Property = IsMobile*/
const Value V18731_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P18731_5 = {S52, 0, &V18731_5_37, &P18731_1};	/*Property = ScreenPixelsHeight*/
const Value V18731_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P18731_4 = {S54, 0, &V18731_4_36, &P18731_5};	/*Property = ScreenPixelsWidth*/
const Profile P18731 = {"18731", &P18731_4};	/*Profile ID - 18731*/
const Value V22537_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22537_279 = {S1, 0, &V22537_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22537_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22537_282 = {S3, 0, &V22537_282_57153, &P22537_279};	/*Property = BlobBuilder*/
const Value V22537_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22537_350 = {S4, 0, &V22537_350_57444, &P22537_282};	/*Property = Canvas*/
const Value V22537_283_57160 = {S0, NULL};	/*Value = False*/
const Property P22537_283 = {S5, 0, &V22537_283_57160, &P22537_350};	/*Property = CssBackground*/
const Value V22537_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22537_284 = {S6, 0, &V22537_284_57163, &P22537_283};	/*Property = CssBorderImage*/
const Value V22537_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22537_285 = {S7, 0, &V22537_285_57166, &P22537_284};	/*Property = CssCanvas*/
const Value V22537_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22537_286 = {S8, 0, &V22537_286_57168, &P22537_285};	/*Property = CssColor*/
const Value V22537_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22537_237 = {S9, 0, &V22537_237_55784, &P22537_286};	/*Property = CssColumn*/
const Value V22537_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22537_287 = {S10, 0, &V22537_287_57172, &P22537_237};	/*Property = CssFlexbox*/
const Value V22537_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22537_288 = {S11, 0, &V22537_288_57175, &P22537_287};	/*Property = CssFont*/
const Value V22537_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22537_289 = {S12, 0, &V22537_289_86899, &P22537_288};	/*Property = CssImages*/
const Value V22537_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22537_291 = {S13, 0, &V22537_291_57181, &P22537_289};	/*Property = CssMediaQueries*/
const Value V22537_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22537_292 = {S14, 0, &V22537_292_57183, &P22537_291};	/*Property = CssMinMax*/
const Value V22537_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22537_293 = {S15, 0, &V22537_293_57187, &P22537_292};	/*Property = CssOverflow*/
const Value V22537_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22537_294 = {S16, 0, &V22537_294_57189, &P22537_293};	/*Property = CssPosition*/
const Value V22537_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22537_295 = {S17, 0, &V22537_295_57193, &P22537_294};	/*Property = CssText*/
const Value V22537_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22537_297 = {S18, 0, &V22537_297_57195, &P22537_295};	/*Property = CssTransforms*/
const Value V22537_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22537_241 = {S19, 0, &V22537_241_55799, &P22537_297};	/*Property = CssTransitions*/
const Value V22537_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22537_298 = {S20, 0, &V22537_298_57206, &P22537_241};	/*Property = CssUI*/
const Value V22537_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22537_300 = {S21, 0, &V22537_300_57215, &P22537_298};	/*Property = DataSet*/
const Value V22537_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22537_301 = {S22, 0, &V22537_301_57218, &P22537_300};	/*Property = DataUrl*/
const Value V22537_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22537_302 = {S23, 0, &V22537_302_57221, &P22537_301};	/*Property = DeviceOrientation*/
const Value V22537_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22537_304 = {S24, 0, &V22537_304_57227, &P22537_302};	/*Property = FileReader*/
const Value V22537_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22537_305 = {S25, 0, &V22537_305_57231, &P22537_304};	/*Property = FileSaver*/
const Value V22537_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22537_306 = {S26, 0, &V22537_306_57237, &P22537_305};	/*Property = FileWriter*/
const Value V22537_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22537_307 = {S27, 0, &V22537_307_57239, &P22537_306};	/*Property = FormData*/
const Value V22537_309_57245 = {S2, NULL};	/*Value = True*/
const Property P22537_309 = {S28, 0, &V22537_309_57245, &P22537_307};	/*Property = Fullscreen*/
const Value V22537_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22537_310 = {S29, 0, &V22537_310_57248, &P22537_309};	/*Property = GeoLocation*/
const Value V22537_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22537_312 = {S30, 0, &V22537_312_57254, &P22537_310};	/*Property = History*/
const Value V22537_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22537_314 = {S31, 0, &V22537_314_57261, &P22537_312};	/*Property = Html5*/
const Value V22537_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22537_313 = {S32, 0, &V22537_313_57258, &P22537_314};	/*Property = Html-Media-Capture*/
const Value V22537_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22537_315 = {S33, 0, &V22537_315_57264, &P22537_313};	/*Property = Iframe*/
const Value V22537_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22537_316 = {S34, 0, &V22537_316_57266, &P22537_315};	/*Property = IndexedDB*/
const Value V22537_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22537_317 = {S35, 0, &V22537_317_57269, &P22537_316};	/*Property = Json*/
const Value V22537_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22537_15 = {S37, 0, &V22537_15_29, &P22537_317};	/*Property = LayoutEngine*/
const Value V22537_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22537_344 = {S38, 0, &V22537_344_57272, &P22537_15};	/*Property = Masking*/
const Value V22537_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22537_320 = {S39, 0, &V22537_320_57290, &P22537_344};	/*Property = PostMessage*/
const Value V22537_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22537_321 = {S40, 0, &V22537_321_57293, &P22537_320};	/*Property = Progress*/
const Value V22537_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22537_322 = {S41, 0, &V22537_322_57296, &P22537_321};	/*Property = Prompts*/
const Value V22537_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22537_323 = {S42, 0, &V22537_323_57299, &P22537_322};	/*Property = Selector*/
const Value V22537_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22537_324 = {S43, 0, &V22537_324_57305, &P22537_323};	/*Property = Svg*/
const Value V22537_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22537_327 = {S44, 0, &V22537_327_57315, &P22537_324};	/*Property = TouchEvents*/
const Value V22537_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22537_328 = {S45, 0, &V22537_328_57318, &P22537_327};	/*Property = Track*/
const Value V22537_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22537_329 = {S46, 0, &V22537_329_57323, &P22537_328};	/*Property = Video*/
const Value V22537_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22537_330 = {S47, 0, &V22537_330_57326, &P22537_329};	/*Property = Viewport*/
const Value V22537_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22537_332 = {S48, 0, &V22537_332_57339, &P22537_330};	/*Property = WebWorkers*/
const Value V22537_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22537_333 = {S49, 0, &V22537_333_57342, &P22537_332};	/*Property = Xhr2*/
const Profile P22537 = {"22537", &P22537_333};	/*Profile ID - 22537*/
const Profile P22328 = {"22328", NULL};	/*Profile ID - 22328*/
const Value V16290_1_1 = {S2, NULL};	/*Value = True*/
const Property P16290_1 = {S50, 0, &V16290_1_1, NULL};	/*Property = IsMobile*/
const Value V16290_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16290_5 = {S52, 0, &V16290_5_1277, &P16290_1};	/*Property = ScreenPixelsHeight*/
const Value V16290_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16290_4 = {S54, 0, &V16290_4_1279, &P16290_5};	/*Property = ScreenPixelsWidth*/
const Profile P16290 = {"16290", &P16290_4};	/*Profile ID - 16290*/
const Value V22766_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22766_279 = {S1, 0, &V22766_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22766_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22766_282 = {S3, 0, &V22766_282_57154, &P22766_279};	/*Property = BlobBuilder*/
const Value V22766_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22766_350 = {S4, 0, &V22766_350_57444, &P22766_282};	/*Property = Canvas*/
const Value V22766_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22766_283 = {S5, 0, &V22766_283_57159, &P22766_350};	/*Property = CssBackground*/
const Value V22766_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22766_284 = {S6, 0, &V22766_284_57163, &P22766_283};	/*Property = CssBorderImage*/
const Value V22766_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22766_285 = {S7, 0, &V22766_285_57166, &P22766_284};	/*Property = CssCanvas*/
const Value V22766_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22766_286 = {S8, 0, &V22766_286_57168, &P22766_285};	/*Property = CssColor*/
const Value V22766_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22766_237 = {S9, 0, &V22766_237_55786, &P22766_286};	/*Property = CssColumn*/
const Value V22766_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22766_287 = {S10, 0, &V22766_287_57172, &P22766_237};	/*Property = CssFlexbox*/
const Value V22766_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22766_288 = {S11, 0, &V22766_288_57176, &P22766_287};	/*Property = CssFont*/
const Value V22766_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22766_289 = {S12, 0, &V22766_289_57759, &P22766_288};	/*Property = CssImages*/
const Value V22766_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22766_291 = {S13, 0, &V22766_291_57180, &P22766_289};	/*Property = CssMediaQueries*/
const Value V22766_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22766_292 = {S14, 0, &V22766_292_57183, &P22766_291};	/*Property = CssMinMax*/
const Value V22766_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22766_293 = {S15, 0, &V22766_293_57187, &P22766_292};	/*Property = CssOverflow*/
const Value V22766_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22766_294 = {S16, 0, &V22766_294_57189, &P22766_293};	/*Property = CssPosition*/
const Value V22766_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22766_295 = {S17, 0, &V22766_295_57192, &P22766_294};	/*Property = CssText*/
const Value V22766_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22766_297 = {S18, 0, &V22766_297_57195, &P22766_295};	/*Property = CssTransforms*/
const Value V22766_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22766_241 = {S19, 0, &V22766_241_55799, &P22766_297};	/*Property = CssTransitions*/
const Value V22766_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22766_298 = {S20, 0, &V22766_298_57206, &P22766_241};	/*Property = CssUI*/
const Value V22766_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22766_300 = {S21, 0, &V22766_300_57216, &P22766_298};	/*Property = DataSet*/
const Value V22766_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22766_301 = {S22, 0, &V22766_301_57218, &P22766_300};	/*Property = DataUrl*/
const Value V22766_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22766_302 = {S23, 0, &V22766_302_57222, &P22766_301};	/*Property = DeviceOrientation*/
const Value V22766_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22766_304 = {S24, 0, &V22766_304_57227, &P22766_302};	/*Property = FileReader*/
const Value V22766_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22766_305 = {S25, 0, &V22766_305_57231, &P22766_304};	/*Property = FileSaver*/
const Value V22766_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22766_306 = {S26, 0, &V22766_306_57237, &P22766_305};	/*Property = FileWriter*/
const Value V22766_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22766_307 = {S27, 0, &V22766_307_57239, &P22766_306};	/*Property = FormData*/
const Value V22766_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22766_309 = {S28, 0, &V22766_309_57246, &P22766_307};	/*Property = Fullscreen*/
const Value V22766_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22766_310 = {S29, 0, &V22766_310_57248, &P22766_309};	/*Property = GeoLocation*/
const Value V22766_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22766_312 = {S30, 0, &V22766_312_57254, &P22766_310};	/*Property = History*/
const Value V22766_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22766_314 = {S31, 0, &V22766_314_57261, &P22766_312};	/*Property = Html5*/
const Value V22766_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22766_313 = {S32, 0, &V22766_313_57258, &P22766_314};	/*Property = Html-Media-Capture*/
const Value V22766_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22766_315 = {S33, 0, &V22766_315_57264, &P22766_313};	/*Property = Iframe*/
const Value V22766_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22766_316 = {S34, 0, &V22766_316_57266, &P22766_315};	/*Property = IndexedDB*/
const Value V22766_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22766_317 = {S35, 0, &V22766_317_57269, &P22766_316};	/*Property = Json*/
const Value V22766_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22766_15 = {S37, 0, &V22766_15_72, &P22766_317};	/*Property = LayoutEngine*/
const Value V22766_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22766_344 = {S38, 0, &V22766_344_57272, &P22766_15};	/*Property = Masking*/
const Value V22766_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22766_320 = {S39, 0, &V22766_320_57290, &P22766_344};	/*Property = PostMessage*/
const Value V22766_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22766_321 = {S40, 0, &V22766_321_57293, &P22766_320};	/*Property = Progress*/
const Value V22766_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22766_322 = {S41, 0, &V22766_322_57296, &P22766_321};	/*Property = Prompts*/
const Value V22766_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22766_323 = {S42, 0, &V22766_323_57299, &P22766_322};	/*Property = Selector*/
const Value V22766_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22766_324 = {S43, 0, &V22766_324_57306, &P22766_323};	/*Property = Svg*/
const Value V22766_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22766_327 = {S44, 0, &V22766_327_57315, &P22766_324};	/*Property = TouchEvents*/
const Value V22766_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22766_328 = {S45, 0, &V22766_328_57318, &P22766_327};	/*Property = Track*/
const Value V22766_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22766_329 = {S46, 0, &V22766_329_57323, &P22766_328};	/*Property = Video*/
const Value V22766_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22766_330 = {S47, 0, &V22766_330_57326, &P22766_329};	/*Property = Viewport*/
const Value V22766_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22766_332 = {S48, 0, &V22766_332_57340, &P22766_330};	/*Property = WebWorkers*/
const Value V22766_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22766_333 = {S49, 0, &V22766_333_57343, &P22766_332};	/*Property = Xhr2*/
const Profile P22766 = {"22766", &P22766_333};	/*Profile ID - 22766*/
const Value V20321_1_2 = {S0, NULL};	/*Value = False*/
const Property P20321_1 = {S50, 0, &V20321_1_2, NULL};	/*Property = IsMobile*/
const Value V20321_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20321_5 = {S52, 0, &V20321_5_37, &P20321_1};	/*Property = ScreenPixelsHeight*/
const Value V20321_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20321_4 = {S54, 0, &V20321_4_36, &P20321_5};	/*Property = ScreenPixelsWidth*/
const Profile P20321 = {"20321", &P20321_4};	/*Profile ID - 20321*/
const Value V18160_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18160_279 = {S1, 0, &V18160_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18160_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18160_282 = {S3, 0, &V18160_282_57154, &P18160_279};	/*Property = BlobBuilder*/
const Value V18160_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18160_350 = {S4, 0, &V18160_350_57444, &P18160_282};	/*Property = Canvas*/
const Value V18160_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18160_283 = {S5, 0, &V18160_283_57159, &P18160_350};	/*Property = CssBackground*/
const Value V18160_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18160_284 = {S6, 0, &V18160_284_57163, &P18160_283};	/*Property = CssBorderImage*/
const Value V18160_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18160_285 = {S7, 0, &V18160_285_57166, &P18160_284};	/*Property = CssCanvas*/
const Value V18160_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18160_286 = {S8, 0, &V18160_286_57168, &P18160_285};	/*Property = CssColor*/
const Value V18160_237_55784 = {S2, NULL};	/*Value = True*/
const Property P18160_237 = {S9, 0, &V18160_237_55784, &P18160_286};	/*Property = CssColumn*/
const Value V18160_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18160_287 = {S10, 0, &V18160_287_57172, &P18160_237};	/*Property = CssFlexbox*/
const Value V18160_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18160_288 = {S11, 0, &V18160_288_57175, &P18160_287};	/*Property = CssFont*/
const Value V18160_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18160_289 = {S12, 0, &V18160_289_57759, &P18160_288};	/*Property = CssImages*/
const Value V18160_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18160_291 = {S13, 0, &V18160_291_57181, &P18160_289};	/*Property = CssMediaQueries*/
const Value V18160_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18160_292 = {S14, 0, &V18160_292_57183, &P18160_291};	/*Property = CssMinMax*/
const Value V18160_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18160_293 = {S15, 0, &V18160_293_57187, &P18160_292};	/*Property = CssOverflow*/
const Value V18160_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18160_294 = {S16, 0, &V18160_294_57190, &P18160_293};	/*Property = CssPosition*/
const Value V18160_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18160_295 = {S17, 0, &V18160_295_57193, &P18160_294};	/*Property = CssText*/
const Value V18160_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18160_297 = {S18, 0, &V18160_297_57195, &P18160_295};	/*Property = CssTransforms*/
const Value V18160_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18160_241 = {S19, 0, &V18160_241_55799, &P18160_297};	/*Property = CssTransitions*/
const Value V18160_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18160_298 = {S20, 0, &V18160_298_57206, &P18160_241};	/*Property = CssUI*/
const Value V18160_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18160_300 = {S21, 0, &V18160_300_57216, &P18160_298};	/*Property = DataSet*/
const Value V18160_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18160_301 = {S22, 0, &V18160_301_57218, &P18160_300};	/*Property = DataUrl*/
const Value V18160_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18160_302 = {S23, 0, &V18160_302_57222, &P18160_301};	/*Property = DeviceOrientation*/
const Value V18160_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18160_304 = {S24, 0, &V18160_304_57228, &P18160_302};	/*Property = FileReader*/
const Value V18160_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18160_305 = {S25, 0, &V18160_305_57231, &P18160_304};	/*Property = FileSaver*/
const Value V18160_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18160_306 = {S26, 0, &V18160_306_57237, &P18160_305};	/*Property = FileWriter*/
const Value V18160_307_57240 = {S0, NULL};	/*Value = False*/
const Property P18160_307 = {S27, 0, &V18160_307_57240, &P18160_306};	/*Property = FormData*/
const Value V18160_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18160_309 = {S28, 0, &V18160_309_57246, &P18160_307};	/*Property = Fullscreen*/
const Value V18160_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18160_310 = {S29, 0, &V18160_310_57248, &P18160_309};	/*Property = GeoLocation*/
const Value V18160_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18160_312 = {S30, 0, &V18160_312_57255, &P18160_310};	/*Property = History*/
const Value V18160_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18160_314 = {S31, 0, &V18160_314_57261, &P18160_312};	/*Property = Html5*/
const Value V18160_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18160_313 = {S32, 0, &V18160_313_57258, &P18160_314};	/*Property = Html-Media-Capture*/
const Value V18160_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18160_315 = {S33, 0, &V18160_315_57264, &P18160_313};	/*Property = Iframe*/
const Value V18160_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18160_316 = {S34, 0, &V18160_316_57267, &P18160_315};	/*Property = IndexedDB*/
const Value V18160_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18160_317 = {S35, 0, &V18160_317_57269, &P18160_316};	/*Property = Json*/
const Value V18160_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18160_15 = {S37, 0, &V18160_15_29, &P18160_317};	/*Property = LayoutEngine*/
const Value V18160_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18160_344 = {S38, 0, &V18160_344_57272, &P18160_15};	/*Property = Masking*/
const Value V18160_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18160_320 = {S39, 0, &V18160_320_57290, &P18160_344};	/*Property = PostMessage*/
const Value V18160_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18160_321 = {S40, 0, &V18160_321_57293, &P18160_320};	/*Property = Progress*/
const Value V18160_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18160_322 = {S41, 0, &V18160_322_57296, &P18160_321};	/*Property = Prompts*/
const Value V18160_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18160_323 = {S42, 0, &V18160_323_57299, &P18160_322};	/*Property = Selector*/
const Value V18160_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18160_324 = {S43, 0, &V18160_324_57306, &P18160_323};	/*Property = Svg*/
const Value V18160_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18160_327 = {S44, 0, &V18160_327_57315, &P18160_324};	/*Property = TouchEvents*/
const Value V18160_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18160_328 = {S45, 0, &V18160_328_57318, &P18160_327};	/*Property = Track*/
const Value V18160_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18160_329 = {S46, 0, &V18160_329_57323, &P18160_328};	/*Property = Video*/
const Value V18160_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18160_330 = {S47, 0, &V18160_330_57326, &P18160_329};	/*Property = Viewport*/
const Value V18160_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18160_332 = {S48, 0, &V18160_332_57339, &P18160_330};	/*Property = WebWorkers*/
const Value V18160_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18160_333 = {S49, 0, &V18160_333_57342, &P18160_332};	/*Property = Xhr2*/
const Profile P18160 = {"18160", &P18160_333};	/*Profile ID - 18160*/
const Value V18368_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_279 = {S1, 0, &V18368_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18368_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_282 = {S3, 0, &V18368_282_57155, &P18368_279};	/*Property = BlobBuilder*/
const Value V18368_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_350 = {S4, 0, &V18368_350_57446, &P18368_282};	/*Property = Canvas*/
const Value V18368_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_283 = {S5, 0, &V18368_283_57161, &P18368_350};	/*Property = CssBackground*/
const Value V18368_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_284 = {S6, 0, &V18368_284_57164, &P18368_283};	/*Property = CssBorderImage*/
const Value V18368_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_285 = {S7, 0, &V18368_285_57167, &P18368_284};	/*Property = CssCanvas*/
const Value V18368_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_286 = {S8, 0, &V18368_286_57170, &P18368_285};	/*Property = CssColor*/
const Value V18368_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_237 = {S9, 0, &V18368_237_55786, &P18368_286};	/*Property = CssColumn*/
const Value V18368_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_287 = {S10, 0, &V18368_287_57173, &P18368_237};	/*Property = CssFlexbox*/
const Value V18368_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_288 = {S11, 0, &V18368_288_57176, &P18368_287};	/*Property = CssFont*/
const Value V18368_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_289 = {S12, 0, &V18368_289_86899, &P18368_288};	/*Property = CssImages*/
const Value V18368_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_291 = {S13, 0, &V18368_291_57182, &P18368_289};	/*Property = CssMediaQueries*/
const Value V18368_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_292 = {S14, 0, &V18368_292_57185, &P18368_291};	/*Property = CssMinMax*/
const Value V18368_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_293 = {S15, 0, &V18368_293_57188, &P18368_292};	/*Property = CssOverflow*/
const Value V18368_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_294 = {S16, 0, &V18368_294_57191, &P18368_293};	/*Property = CssPosition*/
const Value V18368_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_295 = {S17, 0, &V18368_295_57194, &P18368_294};	/*Property = CssText*/
const Value V18368_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_297 = {S18, 0, &V18368_297_57197, &P18368_295};	/*Property = CssTransforms*/
const Value V18368_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_241 = {S19, 0, &V18368_241_55801, &P18368_297};	/*Property = CssTransitions*/
const Value V18368_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_298 = {S20, 0, &V18368_298_57208, &P18368_241};	/*Property = CssUI*/
const Value V18368_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_300 = {S21, 0, &V18368_300_57217, &P18368_298};	/*Property = DataSet*/
const Value V18368_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_301 = {S22, 0, &V18368_301_57220, &P18368_300};	/*Property = DataUrl*/
const Value V18368_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_302 = {S23, 0, &V18368_302_57223, &P18368_301};	/*Property = DeviceOrientation*/
const Value V18368_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_304 = {S24, 0, &V18368_304_57229, &P18368_302};	/*Property = FileReader*/
const Value V18368_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_305 = {S25, 0, &V18368_305_57232, &P18368_304};	/*Property = FileSaver*/
const Value V18368_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_306 = {S26, 0, &V18368_306_57238, &P18368_305};	/*Property = FileWriter*/
const Value V18368_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_307 = {S27, 0, &V18368_307_57241, &P18368_306};	/*Property = FormData*/
const Value V18368_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_309 = {S28, 0, &V18368_309_57247, &P18368_307};	/*Property = Fullscreen*/
const Value V18368_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_310 = {S29, 0, &V18368_310_57250, &P18368_309};	/*Property = GeoLocation*/
const Value V18368_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_312 = {S30, 0, &V18368_312_57256, &P18368_310};	/*Property = History*/
const Value V18368_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_314 = {S31, 0, &V18368_314_57262, &P18368_312};	/*Property = Html5*/
const Value V18368_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_313 = {S32, 0, &V18368_313_57259, &P18368_314};	/*Property = Html-Media-Capture*/
const Value V18368_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_315 = {S33, 0, &V18368_315_57265, &P18368_313};	/*Property = Iframe*/
const Value V18368_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_316 = {S34, 0, &V18368_316_57268, &P18368_315};	/*Property = IndexedDB*/
const Value V18368_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_317 = {S35, 0, &V18368_317_57271, &P18368_316};	/*Property = Json*/
const Value V18368_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18368_15 = {S37, 0, &V18368_15_29, &P18368_317};	/*Property = LayoutEngine*/
const Value V18368_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_344 = {S38, 0, &V18368_344_57274, &P18368_15};	/*Property = Masking*/
const Value V18368_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_320 = {S39, 0, &V18368_320_57292, &P18368_344};	/*Property = PostMessage*/
const Value V18368_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_321 = {S40, 0, &V18368_321_57295, &P18368_320};	/*Property = Progress*/
const Value V18368_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_322 = {S41, 0, &V18368_322_57298, &P18368_321};	/*Property = Prompts*/
const Value V18368_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_323 = {S42, 0, &V18368_323_57301, &P18368_322};	/*Property = Selector*/
const Value V18368_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_324 = {S43, 0, &V18368_324_57307, &P18368_323};	/*Property = Svg*/
const Value V18368_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_327 = {S44, 0, &V18368_327_57316, &P18368_324};	/*Property = TouchEvents*/
const Value V18368_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_328 = {S45, 0, &V18368_328_57319, &P18368_327};	/*Property = Track*/
const Value V18368_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_329 = {S46, 0, &V18368_329_57325, &P18368_328};	/*Property = Video*/
const Value V18368_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_330 = {S47, 0, &V18368_330_57328, &P18368_329};	/*Property = Viewport*/
const Value V18368_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_332 = {S48, 0, &V18368_332_57340, &P18368_330};	/*Property = WebWorkers*/
const Value V18368_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18368_333 = {S49, 0, &V18368_333_57343, &P18368_332};	/*Property = Xhr2*/
const Profile P18368 = {"18368", &P18368_333};	/*Profile ID - 18368*/
const Value V24574_1_1 = {S2, NULL};	/*Value = True*/
const Property P24574_1 = {S50, 0, &V24574_1_1, NULL};	/*Property = IsMobile*/
const Value V24574_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24574_5 = {S52, 0, &V24574_5_1277, &P24574_1};	/*Property = ScreenPixelsHeight*/
const Value V24574_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24574_4 = {S54, 0, &V24574_4_1279, &P24574_5};	/*Property = ScreenPixelsWidth*/
const Profile P24574 = {"24574", &P24574_4};	/*Profile ID - 24574*/
const Value V18494_1_1 = {S2, NULL};	/*Value = True*/
const Property P18494_1 = {S50, 0, &V18494_1_1, NULL};	/*Property = IsMobile*/
const Value V18494_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18494_5 = {S52, 0, &V18494_5_1227, &P18494_1};	/*Property = ScreenPixelsHeight*/
const Value V18494_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18494_4 = {S54, 0, &V18494_4_1229, &P18494_5};	/*Property = ScreenPixelsWidth*/
const Profile P18494 = {"18494", &P18494_4};	/*Profile ID - 18494*/
const Value V15786_1_1 = {S2, NULL};	/*Value = True*/
const Property P15786_1 = {S50, 0, &V15786_1_1, NULL};	/*Property = IsMobile*/
const Value V15786_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15786_5 = {S52, 0, &V15786_5_1222, &P15786_1};	/*Property = ScreenPixelsHeight*/
const Value V15786_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15786_4 = {S54, 0, &V15786_4_1224, &P15786_5};	/*Property = ScreenPixelsWidth*/
const Profile P15786 = {"15786", &P15786_4};	/*Profile ID - 15786*/
const Value V20459_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_279 = {S1, 0, &V20459_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20459_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_282 = {S3, 0, &V20459_282_57155, &P20459_279};	/*Property = BlobBuilder*/
const Value V20459_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_350 = {S4, 0, &V20459_350_57446, &P20459_282};	/*Property = Canvas*/
const Value V20459_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_283 = {S5, 0, &V20459_283_57161, &P20459_350};	/*Property = CssBackground*/
const Value V20459_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_284 = {S6, 0, &V20459_284_57164, &P20459_283};	/*Property = CssBorderImage*/
const Value V20459_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_285 = {S7, 0, &V20459_285_57167, &P20459_284};	/*Property = CssCanvas*/
const Value V20459_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_286 = {S8, 0, &V20459_286_57170, &P20459_285};	/*Property = CssColor*/
const Value V20459_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_237 = {S9, 0, &V20459_237_55786, &P20459_286};	/*Property = CssColumn*/
const Value V20459_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_287 = {S10, 0, &V20459_287_57173, &P20459_237};	/*Property = CssFlexbox*/
const Value V20459_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_288 = {S11, 0, &V20459_288_57176, &P20459_287};	/*Property = CssFont*/
const Value V20459_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_289 = {S12, 0, &V20459_289_86899, &P20459_288};	/*Property = CssImages*/
const Value V20459_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_291 = {S13, 0, &V20459_291_57182, &P20459_289};	/*Property = CssMediaQueries*/
const Value V20459_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_292 = {S14, 0, &V20459_292_57185, &P20459_291};	/*Property = CssMinMax*/
const Value V20459_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_293 = {S15, 0, &V20459_293_57188, &P20459_292};	/*Property = CssOverflow*/
const Value V20459_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_294 = {S16, 0, &V20459_294_57191, &P20459_293};	/*Property = CssPosition*/
const Value V20459_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_295 = {S17, 0, &V20459_295_57194, &P20459_294};	/*Property = CssText*/
const Value V20459_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_297 = {S18, 0, &V20459_297_57197, &P20459_295};	/*Property = CssTransforms*/
const Value V20459_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_241 = {S19, 0, &V20459_241_55801, &P20459_297};	/*Property = CssTransitions*/
const Value V20459_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_298 = {S20, 0, &V20459_298_57208, &P20459_241};	/*Property = CssUI*/
const Value V20459_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_300 = {S21, 0, &V20459_300_57217, &P20459_298};	/*Property = DataSet*/
const Value V20459_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_301 = {S22, 0, &V20459_301_57220, &P20459_300};	/*Property = DataUrl*/
const Value V20459_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_302 = {S23, 0, &V20459_302_57223, &P20459_301};	/*Property = DeviceOrientation*/
const Value V20459_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_304 = {S24, 0, &V20459_304_57229, &P20459_302};	/*Property = FileReader*/
const Value V20459_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_305 = {S25, 0, &V20459_305_57232, &P20459_304};	/*Property = FileSaver*/
const Value V20459_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_306 = {S26, 0, &V20459_306_57238, &P20459_305};	/*Property = FileWriter*/
const Value V20459_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_307 = {S27, 0, &V20459_307_57241, &P20459_306};	/*Property = FormData*/
const Value V20459_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_309 = {S28, 0, &V20459_309_57247, &P20459_307};	/*Property = Fullscreen*/
const Value V20459_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_310 = {S29, 0, &V20459_310_57250, &P20459_309};	/*Property = GeoLocation*/
const Value V20459_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_312 = {S30, 0, &V20459_312_57256, &P20459_310};	/*Property = History*/
const Value V20459_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_314 = {S31, 0, &V20459_314_57262, &P20459_312};	/*Property = Html5*/
const Value V20459_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_313 = {S32, 0, &V20459_313_57259, &P20459_314};	/*Property = Html-Media-Capture*/
const Value V20459_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_315 = {S33, 0, &V20459_315_57265, &P20459_313};	/*Property = Iframe*/
const Value V20459_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_316 = {S34, 0, &V20459_316_57268, &P20459_315};	/*Property = IndexedDB*/
const Value V20459_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_317 = {S35, 0, &V20459_317_57271, &P20459_316};	/*Property = Json*/
const Value V20459_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20459_15 = {S37, 0, &V20459_15_29, &P20459_317};	/*Property = LayoutEngine*/
const Value V20459_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_344 = {S38, 0, &V20459_344_57274, &P20459_15};	/*Property = Masking*/
const Value V20459_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_320 = {S39, 0, &V20459_320_57292, &P20459_344};	/*Property = PostMessage*/
const Value V20459_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_321 = {S40, 0, &V20459_321_57295, &P20459_320};	/*Property = Progress*/
const Value V20459_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_322 = {S41, 0, &V20459_322_57298, &P20459_321};	/*Property = Prompts*/
const Value V20459_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_323 = {S42, 0, &V20459_323_57301, &P20459_322};	/*Property = Selector*/
const Value V20459_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_324 = {S43, 0, &V20459_324_57307, &P20459_323};	/*Property = Svg*/
const Value V20459_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_327 = {S44, 0, &V20459_327_57316, &P20459_324};	/*Property = TouchEvents*/
const Value V20459_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_328 = {S45, 0, &V20459_328_57319, &P20459_327};	/*Property = Track*/
const Value V20459_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_329 = {S46, 0, &V20459_329_57325, &P20459_328};	/*Property = Video*/
const Value V20459_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_330 = {S47, 0, &V20459_330_57328, &P20459_329};	/*Property = Viewport*/
const Value V20459_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_332 = {S48, 0, &V20459_332_57340, &P20459_330};	/*Property = WebWorkers*/
const Value V20459_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20459_333 = {S49, 0, &V20459_333_57343, &P20459_332};	/*Property = Xhr2*/
const Profile P20459 = {"20459", &P20459_333};	/*Profile ID - 20459*/
const Value V18192_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_279 = {S1, 0, &V18192_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18192_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_282 = {S3, 0, &V18192_282_57155, &P18192_279};	/*Property = BlobBuilder*/
const Value V18192_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_350 = {S4, 0, &V18192_350_57446, &P18192_282};	/*Property = Canvas*/
const Value V18192_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_283 = {S5, 0, &V18192_283_57161, &P18192_350};	/*Property = CssBackground*/
const Value V18192_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_284 = {S6, 0, &V18192_284_57164, &P18192_283};	/*Property = CssBorderImage*/
const Value V18192_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_285 = {S7, 0, &V18192_285_57167, &P18192_284};	/*Property = CssCanvas*/
const Value V18192_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_286 = {S8, 0, &V18192_286_57170, &P18192_285};	/*Property = CssColor*/
const Value V18192_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_237 = {S9, 0, &V18192_237_55786, &P18192_286};	/*Property = CssColumn*/
const Value V18192_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_287 = {S10, 0, &V18192_287_57173, &P18192_237};	/*Property = CssFlexbox*/
const Value V18192_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_288 = {S11, 0, &V18192_288_57176, &P18192_287};	/*Property = CssFont*/
const Value V18192_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_289 = {S12, 0, &V18192_289_86899, &P18192_288};	/*Property = CssImages*/
const Value V18192_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_291 = {S13, 0, &V18192_291_57182, &P18192_289};	/*Property = CssMediaQueries*/
const Value V18192_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_292 = {S14, 0, &V18192_292_57185, &P18192_291};	/*Property = CssMinMax*/
const Value V18192_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_293 = {S15, 0, &V18192_293_57188, &P18192_292};	/*Property = CssOverflow*/
const Value V18192_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_294 = {S16, 0, &V18192_294_57191, &P18192_293};	/*Property = CssPosition*/
const Value V18192_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_295 = {S17, 0, &V18192_295_57194, &P18192_294};	/*Property = CssText*/
const Value V18192_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_297 = {S18, 0, &V18192_297_57197, &P18192_295};	/*Property = CssTransforms*/
const Value V18192_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_241 = {S19, 0, &V18192_241_55801, &P18192_297};	/*Property = CssTransitions*/
const Value V18192_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_298 = {S20, 0, &V18192_298_57208, &P18192_241};	/*Property = CssUI*/
const Value V18192_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_300 = {S21, 0, &V18192_300_57217, &P18192_298};	/*Property = DataSet*/
const Value V18192_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_301 = {S22, 0, &V18192_301_57220, &P18192_300};	/*Property = DataUrl*/
const Value V18192_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_302 = {S23, 0, &V18192_302_57223, &P18192_301};	/*Property = DeviceOrientation*/
const Value V18192_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_304 = {S24, 0, &V18192_304_57229, &P18192_302};	/*Property = FileReader*/
const Value V18192_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_305 = {S25, 0, &V18192_305_57232, &P18192_304};	/*Property = FileSaver*/
const Value V18192_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_306 = {S26, 0, &V18192_306_57238, &P18192_305};	/*Property = FileWriter*/
const Value V18192_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_307 = {S27, 0, &V18192_307_57241, &P18192_306};	/*Property = FormData*/
const Value V18192_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_309 = {S28, 0, &V18192_309_57247, &P18192_307};	/*Property = Fullscreen*/
const Value V18192_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_310 = {S29, 0, &V18192_310_57250, &P18192_309};	/*Property = GeoLocation*/
const Value V18192_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_312 = {S30, 0, &V18192_312_57256, &P18192_310};	/*Property = History*/
const Value V18192_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_314 = {S31, 0, &V18192_314_57262, &P18192_312};	/*Property = Html5*/
const Value V18192_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_313 = {S32, 0, &V18192_313_57259, &P18192_314};	/*Property = Html-Media-Capture*/
const Value V18192_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_315 = {S33, 0, &V18192_315_57265, &P18192_313};	/*Property = Iframe*/
const Value V18192_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_316 = {S34, 0, &V18192_316_57268, &P18192_315};	/*Property = IndexedDB*/
const Value V18192_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_317 = {S35, 0, &V18192_317_57271, &P18192_316};	/*Property = Json*/
const Value V18192_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18192_15 = {S37, 0, &V18192_15_29, &P18192_317};	/*Property = LayoutEngine*/
const Value V18192_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_344 = {S38, 0, &V18192_344_57274, &P18192_15};	/*Property = Masking*/
const Value V18192_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_320 = {S39, 0, &V18192_320_57292, &P18192_344};	/*Property = PostMessage*/
const Value V18192_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_321 = {S40, 0, &V18192_321_57295, &P18192_320};	/*Property = Progress*/
const Value V18192_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_322 = {S41, 0, &V18192_322_57298, &P18192_321};	/*Property = Prompts*/
const Value V18192_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_323 = {S42, 0, &V18192_323_57301, &P18192_322};	/*Property = Selector*/
const Value V18192_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_324 = {S43, 0, &V18192_324_57307, &P18192_323};	/*Property = Svg*/
const Value V18192_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_327 = {S44, 0, &V18192_327_57316, &P18192_324};	/*Property = TouchEvents*/
const Value V18192_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_328 = {S45, 0, &V18192_328_57319, &P18192_327};	/*Property = Track*/
const Value V18192_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_329 = {S46, 0, &V18192_329_57325, &P18192_328};	/*Property = Video*/
const Value V18192_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_330 = {S47, 0, &V18192_330_57328, &P18192_329};	/*Property = Viewport*/
const Value V18192_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_332 = {S48, 0, &V18192_332_57340, &P18192_330};	/*Property = WebWorkers*/
const Value V18192_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18192_333 = {S49, 0, &V18192_333_57343, &P18192_332};	/*Property = Xhr2*/
const Profile P18192 = {"18192", &P18192_333};	/*Profile ID - 18192*/
const Value V20000_1_1 = {S2, NULL};	/*Value = True*/
const Property P20000_1 = {S50, 0, &V20000_1_1, NULL};	/*Property = IsMobile*/
const Value V20000_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20000_5 = {S52, 0, &V20000_5_9981, &P20000_1};	/*Property = ScreenPixelsHeight*/
const Value V20000_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20000_4 = {S54, 0, &V20000_4_2336, &P20000_5};	/*Property = ScreenPixelsWidth*/
const Profile P20000 = {"20000", &P20000_4};	/*Profile ID - 20000*/
const Value V19930_1_1 = {S2, NULL};	/*Value = True*/
const Property P19930_1 = {S50, 0, &V19930_1_1, NULL};	/*Property = IsMobile*/
const Value V19930_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19930_5 = {S52, 0, &V19930_5_1227, &P19930_1};	/*Property = ScreenPixelsHeight*/
const Value V19930_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19930_4 = {S54, 0, &V19930_4_1229, &P19930_5};	/*Property = ScreenPixelsWidth*/
const Profile P19930 = {"19930", &P19930_4};	/*Profile ID - 19930*/
const Value V22252_1_1 = {S2, NULL};	/*Value = True*/
const Property P22252_1 = {S50, 0, &V22252_1_1, NULL};	/*Property = IsMobile*/
const Value V22252_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22252_5 = {S52, 0, &V22252_5_1277, &P22252_1};	/*Property = ScreenPixelsHeight*/
const Value V22252_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22252_4 = {S54, 0, &V22252_4_2336, &P22252_5};	/*Property = ScreenPixelsWidth*/
const Profile P22252 = {"22252", &P22252_4};	/*Profile ID - 22252*/
const Value V21582_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_279 = {S1, 0, &V21582_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21582_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_282 = {S3, 0, &V21582_282_57155, &P21582_279};	/*Property = BlobBuilder*/
const Value V21582_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_350 = {S4, 0, &V21582_350_57446, &P21582_282};	/*Property = Canvas*/
const Value V21582_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_283 = {S5, 0, &V21582_283_57161, &P21582_350};	/*Property = CssBackground*/
const Value V21582_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_284 = {S6, 0, &V21582_284_57164, &P21582_283};	/*Property = CssBorderImage*/
const Value V21582_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_285 = {S7, 0, &V21582_285_57167, &P21582_284};	/*Property = CssCanvas*/
const Value V21582_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_286 = {S8, 0, &V21582_286_57170, &P21582_285};	/*Property = CssColor*/
const Value V21582_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_237 = {S9, 0, &V21582_237_55786, &P21582_286};	/*Property = CssColumn*/
const Value V21582_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_287 = {S10, 0, &V21582_287_57173, &P21582_237};	/*Property = CssFlexbox*/
const Value V21582_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_288 = {S11, 0, &V21582_288_57176, &P21582_287};	/*Property = CssFont*/
const Value V21582_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_289 = {S12, 0, &V21582_289_86899, &P21582_288};	/*Property = CssImages*/
const Value V21582_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_291 = {S13, 0, &V21582_291_57182, &P21582_289};	/*Property = CssMediaQueries*/
const Value V21582_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_292 = {S14, 0, &V21582_292_57185, &P21582_291};	/*Property = CssMinMax*/
const Value V21582_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_293 = {S15, 0, &V21582_293_57188, &P21582_292};	/*Property = CssOverflow*/
const Value V21582_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_294 = {S16, 0, &V21582_294_57191, &P21582_293};	/*Property = CssPosition*/
const Value V21582_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_295 = {S17, 0, &V21582_295_57194, &P21582_294};	/*Property = CssText*/
const Value V21582_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_297 = {S18, 0, &V21582_297_57197, &P21582_295};	/*Property = CssTransforms*/
const Value V21582_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_241 = {S19, 0, &V21582_241_55801, &P21582_297};	/*Property = CssTransitions*/
const Value V21582_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_298 = {S20, 0, &V21582_298_57208, &P21582_241};	/*Property = CssUI*/
const Value V21582_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_300 = {S21, 0, &V21582_300_57217, &P21582_298};	/*Property = DataSet*/
const Value V21582_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_301 = {S22, 0, &V21582_301_57220, &P21582_300};	/*Property = DataUrl*/
const Value V21582_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_302 = {S23, 0, &V21582_302_57223, &P21582_301};	/*Property = DeviceOrientation*/
const Value V21582_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_304 = {S24, 0, &V21582_304_57229, &P21582_302};	/*Property = FileReader*/
const Value V21582_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_305 = {S25, 0, &V21582_305_57232, &P21582_304};	/*Property = FileSaver*/
const Value V21582_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_306 = {S26, 0, &V21582_306_57238, &P21582_305};	/*Property = FileWriter*/
const Value V21582_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_307 = {S27, 0, &V21582_307_57241, &P21582_306};	/*Property = FormData*/
const Value V21582_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_309 = {S28, 0, &V21582_309_57247, &P21582_307};	/*Property = Fullscreen*/
const Value V21582_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_310 = {S29, 0, &V21582_310_57250, &P21582_309};	/*Property = GeoLocation*/
const Value V21582_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_312 = {S30, 0, &V21582_312_57256, &P21582_310};	/*Property = History*/
const Value V21582_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_314 = {S31, 0, &V21582_314_57262, &P21582_312};	/*Property = Html5*/
const Value V21582_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_313 = {S32, 0, &V21582_313_57259, &P21582_314};	/*Property = Html-Media-Capture*/
const Value V21582_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_315 = {S33, 0, &V21582_315_57265, &P21582_313};	/*Property = Iframe*/
const Value V21582_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_316 = {S34, 0, &V21582_316_57268, &P21582_315};	/*Property = IndexedDB*/
const Value V21582_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_317 = {S35, 0, &V21582_317_57271, &P21582_316};	/*Property = Json*/
const Value V21582_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_15 = {S37, 0, &V21582_15_72, &P21582_317};	/*Property = LayoutEngine*/
const Value V21582_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_344 = {S38, 0, &V21582_344_57274, &P21582_15};	/*Property = Masking*/
const Value V21582_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_320 = {S39, 0, &V21582_320_57292, &P21582_344};	/*Property = PostMessage*/
const Value V21582_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_321 = {S40, 0, &V21582_321_57295, &P21582_320};	/*Property = Progress*/
const Value V21582_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_322 = {S41, 0, &V21582_322_57298, &P21582_321};	/*Property = Prompts*/
const Value V21582_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_323 = {S42, 0, &V21582_323_57301, &P21582_322};	/*Property = Selector*/
const Value V21582_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_324 = {S43, 0, &V21582_324_57307, &P21582_323};	/*Property = Svg*/
const Value V21582_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_327 = {S44, 0, &V21582_327_57316, &P21582_324};	/*Property = TouchEvents*/
const Value V21582_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_328 = {S45, 0, &V21582_328_57319, &P21582_327};	/*Property = Track*/
const Value V21582_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_329 = {S46, 0, &V21582_329_57325, &P21582_328};	/*Property = Video*/
const Value V21582_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_330 = {S47, 0, &V21582_330_57328, &P21582_329};	/*Property = Viewport*/
const Value V21582_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_332 = {S48, 0, &V21582_332_57340, &P21582_330};	/*Property = WebWorkers*/
const Value V21582_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21582_333 = {S49, 0, &V21582_333_57343, &P21582_332};	/*Property = Xhr2*/
const Profile P21582 = {"21582", &P21582_333};	/*Profile ID - 21582*/
const Value V18684_1_1 = {S2, NULL};	/*Value = True*/
const Property P18684_1 = {S50, 0, &V18684_1_1, NULL};	/*Property = IsMobile*/
const Value V18684_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18684_5 = {S52, 0, &V18684_5_1403, &P18684_1};	/*Property = ScreenPixelsHeight*/
const Value V18684_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18684_4 = {S54, 0, &V18684_4_1404, &P18684_5};	/*Property = ScreenPixelsWidth*/
const Profile P18684 = {"18684", &P18684_4};	/*Profile ID - 18684*/
const Value V21200_1_1 = {S2, NULL};	/*Value = True*/
const Property P21200_1 = {S50, 0, &V21200_1_1, NULL};	/*Property = IsMobile*/
const Value V21200_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21200_5 = {S52, 0, &V21200_5_1458, &P21200_1};	/*Property = ScreenPixelsHeight*/
const Value V21200_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21200_4 = {S54, 0, &V21200_4_1460, &P21200_5};	/*Property = ScreenPixelsWidth*/
const Profile P21200 = {"21200", &P21200_4};	/*Profile ID - 21200*/
const Value V22339_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22339_279 = {S1, 0, &V22339_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22339_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22339_282 = {S3, 0, &V22339_282_57153, &P22339_279};	/*Property = BlobBuilder*/
const Value V22339_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22339_350 = {S4, 0, &V22339_350_57444, &P22339_282};	/*Property = Canvas*/
const Value V22339_283_57160 = {S0, NULL};	/*Value = False*/
const Property P22339_283 = {S5, 0, &V22339_283_57160, &P22339_350};	/*Property = CssBackground*/
const Value V22339_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22339_284 = {S6, 0, &V22339_284_57163, &P22339_283};	/*Property = CssBorderImage*/
const Value V22339_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22339_285 = {S7, 0, &V22339_285_57166, &P22339_284};	/*Property = CssCanvas*/
const Value V22339_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22339_286 = {S8, 0, &V22339_286_57168, &P22339_285};	/*Property = CssColor*/
const Value V22339_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22339_237 = {S9, 0, &V22339_237_55784, &P22339_286};	/*Property = CssColumn*/
const Value V22339_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22339_287 = {S10, 0, &V22339_287_57172, &P22339_237};	/*Property = CssFlexbox*/
const Value V22339_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22339_288 = {S11, 0, &V22339_288_57175, &P22339_287};	/*Property = CssFont*/
const Value V22339_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22339_289 = {S12, 0, &V22339_289_86899, &P22339_288};	/*Property = CssImages*/
const Value V22339_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22339_291 = {S13, 0, &V22339_291_57181, &P22339_289};	/*Property = CssMediaQueries*/
const Value V22339_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22339_292 = {S14, 0, &V22339_292_57183, &P22339_291};	/*Property = CssMinMax*/
const Value V22339_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22339_293 = {S15, 0, &V22339_293_57187, &P22339_292};	/*Property = CssOverflow*/
const Value V22339_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22339_294 = {S16, 0, &V22339_294_57189, &P22339_293};	/*Property = CssPosition*/
const Value V22339_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22339_295 = {S17, 0, &V22339_295_57193, &P22339_294};	/*Property = CssText*/
const Value V22339_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22339_297 = {S18, 0, &V22339_297_57195, &P22339_295};	/*Property = CssTransforms*/
const Value V22339_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22339_241 = {S19, 0, &V22339_241_55799, &P22339_297};	/*Property = CssTransitions*/
const Value V22339_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22339_298 = {S20, 0, &V22339_298_57206, &P22339_241};	/*Property = CssUI*/
const Value V22339_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22339_300 = {S21, 0, &V22339_300_57215, &P22339_298};	/*Property = DataSet*/
const Value V22339_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22339_301 = {S22, 0, &V22339_301_57218, &P22339_300};	/*Property = DataUrl*/
const Value V22339_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22339_302 = {S23, 0, &V22339_302_57221, &P22339_301};	/*Property = DeviceOrientation*/
const Value V22339_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22339_304 = {S24, 0, &V22339_304_57227, &P22339_302};	/*Property = FileReader*/
const Value V22339_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22339_305 = {S25, 0, &V22339_305_57231, &P22339_304};	/*Property = FileSaver*/
const Value V22339_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22339_306 = {S26, 0, &V22339_306_57237, &P22339_305};	/*Property = FileWriter*/
const Value V22339_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22339_307 = {S27, 0, &V22339_307_57239, &P22339_306};	/*Property = FormData*/
const Value V22339_309_57245 = {S2, NULL};	/*Value = True*/
const Property P22339_309 = {S28, 0, &V22339_309_57245, &P22339_307};	/*Property = Fullscreen*/
const Value V22339_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22339_310 = {S29, 0, &V22339_310_57248, &P22339_309};	/*Property = GeoLocation*/
const Value V22339_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22339_312 = {S30, 0, &V22339_312_57254, &P22339_310};	/*Property = History*/
const Value V22339_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22339_314 = {S31, 0, &V22339_314_57261, &P22339_312};	/*Property = Html5*/
const Value V22339_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22339_313 = {S32, 0, &V22339_313_57258, &P22339_314};	/*Property = Html-Media-Capture*/
const Value V22339_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22339_315 = {S33, 0, &V22339_315_57264, &P22339_313};	/*Property = Iframe*/
const Value V22339_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22339_316 = {S34, 0, &V22339_316_57266, &P22339_315};	/*Property = IndexedDB*/
const Value V22339_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22339_317 = {S35, 0, &V22339_317_57269, &P22339_316};	/*Property = Json*/
const Value V22339_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22339_15 = {S37, 0, &V22339_15_29, &P22339_317};	/*Property = LayoutEngine*/
const Value V22339_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22339_344 = {S38, 0, &V22339_344_57272, &P22339_15};	/*Property = Masking*/
const Value V22339_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22339_320 = {S39, 0, &V22339_320_57290, &P22339_344};	/*Property = PostMessage*/
const Value V22339_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22339_321 = {S40, 0, &V22339_321_57293, &P22339_320};	/*Property = Progress*/
const Value V22339_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22339_322 = {S41, 0, &V22339_322_57296, &P22339_321};	/*Property = Prompts*/
const Value V22339_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22339_323 = {S42, 0, &V22339_323_57299, &P22339_322};	/*Property = Selector*/
const Value V22339_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22339_324 = {S43, 0, &V22339_324_57305, &P22339_323};	/*Property = Svg*/
const Value V22339_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22339_327 = {S44, 0, &V22339_327_57315, &P22339_324};	/*Property = TouchEvents*/
const Value V22339_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22339_328 = {S45, 0, &V22339_328_57318, &P22339_327};	/*Property = Track*/
const Value V22339_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22339_329 = {S46, 0, &V22339_329_57323, &P22339_328};	/*Property = Video*/
const Value V22339_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22339_330 = {S47, 0, &V22339_330_57326, &P22339_329};	/*Property = Viewport*/
const Value V22339_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22339_332 = {S48, 0, &V22339_332_57339, &P22339_330};	/*Property = WebWorkers*/
const Value V22339_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22339_333 = {S49, 0, &V22339_333_57342, &P22339_332};	/*Property = Xhr2*/
const Profile P22339 = {"22339", &P22339_333};	/*Profile ID - 22339*/
const Value V22353_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22353_279 = {S1, 0, &V22353_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22353_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22353_282 = {S3, 0, &V22353_282_57153, &P22353_279};	/*Property = BlobBuilder*/
const Value V22353_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22353_350 = {S4, 0, &V22353_350_57444, &P22353_282};	/*Property = Canvas*/
const Value V22353_283_57160 = {S0, NULL};	/*Value = False*/
const Property P22353_283 = {S5, 0, &V22353_283_57160, &P22353_350};	/*Property = CssBackground*/
const Value V22353_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22353_284 = {S6, 0, &V22353_284_57163, &P22353_283};	/*Property = CssBorderImage*/
const Value V22353_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22353_285 = {S7, 0, &V22353_285_57166, &P22353_284};	/*Property = CssCanvas*/
const Value V22353_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22353_286 = {S8, 0, &V22353_286_57168, &P22353_285};	/*Property = CssColor*/
const Value V22353_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22353_237 = {S9, 0, &V22353_237_55784, &P22353_286};	/*Property = CssColumn*/
const Value V22353_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22353_287 = {S10, 0, &V22353_287_57172, &P22353_237};	/*Property = CssFlexbox*/
const Value V22353_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22353_288 = {S11, 0, &V22353_288_57175, &P22353_287};	/*Property = CssFont*/
const Value V22353_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22353_289 = {S12, 0, &V22353_289_86899, &P22353_288};	/*Property = CssImages*/
const Value V22353_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22353_291 = {S13, 0, &V22353_291_57181, &P22353_289};	/*Property = CssMediaQueries*/
const Value V22353_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22353_292 = {S14, 0, &V22353_292_57183, &P22353_291};	/*Property = CssMinMax*/
const Value V22353_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22353_293 = {S15, 0, &V22353_293_57187, &P22353_292};	/*Property = CssOverflow*/
const Value V22353_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22353_294 = {S16, 0, &V22353_294_57189, &P22353_293};	/*Property = CssPosition*/
const Value V22353_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22353_295 = {S17, 0, &V22353_295_57193, &P22353_294};	/*Property = CssText*/
const Value V22353_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22353_297 = {S18, 0, &V22353_297_57195, &P22353_295};	/*Property = CssTransforms*/
const Value V22353_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22353_241 = {S19, 0, &V22353_241_55799, &P22353_297};	/*Property = CssTransitions*/
const Value V22353_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22353_298 = {S20, 0, &V22353_298_57206, &P22353_241};	/*Property = CssUI*/
const Value V22353_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22353_300 = {S21, 0, &V22353_300_57215, &P22353_298};	/*Property = DataSet*/
const Value V22353_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22353_301 = {S22, 0, &V22353_301_57218, &P22353_300};	/*Property = DataUrl*/
const Value V22353_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22353_302 = {S23, 0, &V22353_302_57221, &P22353_301};	/*Property = DeviceOrientation*/
const Value V22353_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22353_304 = {S24, 0, &V22353_304_57227, &P22353_302};	/*Property = FileReader*/
const Value V22353_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22353_305 = {S25, 0, &V22353_305_57231, &P22353_304};	/*Property = FileSaver*/
const Value V22353_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22353_306 = {S26, 0, &V22353_306_57237, &P22353_305};	/*Property = FileWriter*/
const Value V22353_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22353_307 = {S27, 0, &V22353_307_57239, &P22353_306};	/*Property = FormData*/
const Value V22353_309_57245 = {S2, NULL};	/*Value = True*/
const Property P22353_309 = {S28, 0, &V22353_309_57245, &P22353_307};	/*Property = Fullscreen*/
const Value V22353_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22353_310 = {S29, 0, &V22353_310_57248, &P22353_309};	/*Property = GeoLocation*/
const Value V22353_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22353_312 = {S30, 0, &V22353_312_57254, &P22353_310};	/*Property = History*/
const Value V22353_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22353_314 = {S31, 0, &V22353_314_57261, &P22353_312};	/*Property = Html5*/
const Value V22353_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22353_313 = {S32, 0, &V22353_313_57258, &P22353_314};	/*Property = Html-Media-Capture*/
const Value V22353_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22353_315 = {S33, 0, &V22353_315_57264, &P22353_313};	/*Property = Iframe*/
const Value V22353_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22353_316 = {S34, 0, &V22353_316_57266, &P22353_315};	/*Property = IndexedDB*/
const Value V22353_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22353_317 = {S35, 0, &V22353_317_57269, &P22353_316};	/*Property = Json*/
const Value V22353_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22353_15 = {S37, 0, &V22353_15_29, &P22353_317};	/*Property = LayoutEngine*/
const Value V22353_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22353_344 = {S38, 0, &V22353_344_57272, &P22353_15};	/*Property = Masking*/
const Value V22353_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22353_320 = {S39, 0, &V22353_320_57290, &P22353_344};	/*Property = PostMessage*/
const Value V22353_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22353_321 = {S40, 0, &V22353_321_57293, &P22353_320};	/*Property = Progress*/
const Value V22353_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22353_322 = {S41, 0, &V22353_322_57296, &P22353_321};	/*Property = Prompts*/
const Value V22353_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22353_323 = {S42, 0, &V22353_323_57299, &P22353_322};	/*Property = Selector*/
const Value V22353_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22353_324 = {S43, 0, &V22353_324_57305, &P22353_323};	/*Property = Svg*/
const Value V22353_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22353_327 = {S44, 0, &V22353_327_57315, &P22353_324};	/*Property = TouchEvents*/
const Value V22353_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22353_328 = {S45, 0, &V22353_328_57318, &P22353_327};	/*Property = Track*/
const Value V22353_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22353_329 = {S46, 0, &V22353_329_57323, &P22353_328};	/*Property = Video*/
const Value V22353_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22353_330 = {S47, 0, &V22353_330_57326, &P22353_329};	/*Property = Viewport*/
const Value V22353_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22353_332 = {S48, 0, &V22353_332_57339, &P22353_330};	/*Property = WebWorkers*/
const Value V22353_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22353_333 = {S49, 0, &V22353_333_57342, &P22353_332};	/*Property = Xhr2*/
const Profile P22353 = {"22353", &P22353_333};	/*Profile ID - 22353*/
const Value V20183_1_1 = {S2, NULL};	/*Value = True*/
const Property P20183_1 = {S50, 0, &V20183_1_1, NULL};	/*Property = IsMobile*/
const Value V20183_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P20183_5 = {S52, 0, &V20183_5_1269, &P20183_1};	/*Property = ScreenPixelsHeight*/
const Value V20183_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20183_4 = {S54, 0, &V20183_4_1229, &P20183_5};	/*Property = ScreenPixelsWidth*/
const Profile P20183 = {"20183", &P20183_4};	/*Profile ID - 20183*/
const Value V20593_1_1 = {S2, NULL};	/*Value = True*/
const Property P20593_1 = {S50, 0, &V20593_1_1, NULL};	/*Property = IsMobile*/
const Value V20593_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20593_5 = {S52, 0, &V20593_5_1227, &P20593_1};	/*Property = ScreenPixelsHeight*/
const Value V20593_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20593_4 = {S54, 0, &V20593_4_1229, &P20593_5};	/*Property = ScreenPixelsWidth*/
const Profile P20593 = {"20593", &P20593_4};	/*Profile ID - 20593*/
const Value V20460_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_279 = {S1, 0, &V20460_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20460_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_282 = {S3, 0, &V20460_282_57155, &P20460_279};	/*Property = BlobBuilder*/
const Value V20460_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_350 = {S4, 0, &V20460_350_57446, &P20460_282};	/*Property = Canvas*/
const Value V20460_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_283 = {S5, 0, &V20460_283_57161, &P20460_350};	/*Property = CssBackground*/
const Value V20460_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_284 = {S6, 0, &V20460_284_57164, &P20460_283};	/*Property = CssBorderImage*/
const Value V20460_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_285 = {S7, 0, &V20460_285_57167, &P20460_284};	/*Property = CssCanvas*/
const Value V20460_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_286 = {S8, 0, &V20460_286_57170, &P20460_285};	/*Property = CssColor*/
const Value V20460_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_237 = {S9, 0, &V20460_237_55786, &P20460_286};	/*Property = CssColumn*/
const Value V20460_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_287 = {S10, 0, &V20460_287_57173, &P20460_237};	/*Property = CssFlexbox*/
const Value V20460_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_288 = {S11, 0, &V20460_288_57176, &P20460_287};	/*Property = CssFont*/
const Value V20460_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_289 = {S12, 0, &V20460_289_86899, &P20460_288};	/*Property = CssImages*/
const Value V20460_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_291 = {S13, 0, &V20460_291_57182, &P20460_289};	/*Property = CssMediaQueries*/
const Value V20460_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_292 = {S14, 0, &V20460_292_57185, &P20460_291};	/*Property = CssMinMax*/
const Value V20460_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_293 = {S15, 0, &V20460_293_57188, &P20460_292};	/*Property = CssOverflow*/
const Value V20460_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_294 = {S16, 0, &V20460_294_57191, &P20460_293};	/*Property = CssPosition*/
const Value V20460_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_295 = {S17, 0, &V20460_295_57194, &P20460_294};	/*Property = CssText*/
const Value V20460_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_297 = {S18, 0, &V20460_297_57197, &P20460_295};	/*Property = CssTransforms*/
const Value V20460_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_241 = {S19, 0, &V20460_241_55801, &P20460_297};	/*Property = CssTransitions*/
const Value V20460_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_298 = {S20, 0, &V20460_298_57208, &P20460_241};	/*Property = CssUI*/
const Value V20460_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_300 = {S21, 0, &V20460_300_57217, &P20460_298};	/*Property = DataSet*/
const Value V20460_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_301 = {S22, 0, &V20460_301_57220, &P20460_300};	/*Property = DataUrl*/
const Value V20460_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_302 = {S23, 0, &V20460_302_57223, &P20460_301};	/*Property = DeviceOrientation*/
const Value V20460_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_304 = {S24, 0, &V20460_304_57229, &P20460_302};	/*Property = FileReader*/
const Value V20460_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_305 = {S25, 0, &V20460_305_57232, &P20460_304};	/*Property = FileSaver*/
const Value V20460_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_306 = {S26, 0, &V20460_306_57238, &P20460_305};	/*Property = FileWriter*/
const Value V20460_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_307 = {S27, 0, &V20460_307_57241, &P20460_306};	/*Property = FormData*/
const Value V20460_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_309 = {S28, 0, &V20460_309_57247, &P20460_307};	/*Property = Fullscreen*/
const Value V20460_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_310 = {S29, 0, &V20460_310_57250, &P20460_309};	/*Property = GeoLocation*/
const Value V20460_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_312 = {S30, 0, &V20460_312_57256, &P20460_310};	/*Property = History*/
const Value V20460_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_314 = {S31, 0, &V20460_314_57262, &P20460_312};	/*Property = Html5*/
const Value V20460_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_313 = {S32, 0, &V20460_313_57259, &P20460_314};	/*Property = Html-Media-Capture*/
const Value V20460_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_315 = {S33, 0, &V20460_315_57265, &P20460_313};	/*Property = Iframe*/
const Value V20460_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_316 = {S34, 0, &V20460_316_57268, &P20460_315};	/*Property = IndexedDB*/
const Value V20460_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_317 = {S35, 0, &V20460_317_57271, &P20460_316};	/*Property = Json*/
const Value V20460_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20460_15 = {S37, 0, &V20460_15_29, &P20460_317};	/*Property = LayoutEngine*/
const Value V20460_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_344 = {S38, 0, &V20460_344_57274, &P20460_15};	/*Property = Masking*/
const Value V20460_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_320 = {S39, 0, &V20460_320_57292, &P20460_344};	/*Property = PostMessage*/
const Value V20460_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_321 = {S40, 0, &V20460_321_57295, &P20460_320};	/*Property = Progress*/
const Value V20460_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_322 = {S41, 0, &V20460_322_57298, &P20460_321};	/*Property = Prompts*/
const Value V20460_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_323 = {S42, 0, &V20460_323_57301, &P20460_322};	/*Property = Selector*/
const Value V20460_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_324 = {S43, 0, &V20460_324_57307, &P20460_323};	/*Property = Svg*/
const Value V20460_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_327 = {S44, 0, &V20460_327_57316, &P20460_324};	/*Property = TouchEvents*/
const Value V20460_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_328 = {S45, 0, &V20460_328_57319, &P20460_327};	/*Property = Track*/
const Value V20460_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_329 = {S46, 0, &V20460_329_57325, &P20460_328};	/*Property = Video*/
const Value V20460_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_330 = {S47, 0, &V20460_330_57328, &P20460_329};	/*Property = Viewport*/
const Value V20460_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_332 = {S48, 0, &V20460_332_57340, &P20460_330};	/*Property = WebWorkers*/
const Value V20460_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20460_333 = {S49, 0, &V20460_333_57343, &P20460_332};	/*Property = Xhr2*/
const Profile P20460 = {"20460", &P20460_333};	/*Profile ID - 20460*/
const Value V18566_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18566_279 = {S1, 0, &V18566_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18566_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18566_282 = {S3, 0, &V18566_282_57154, &P18566_279};	/*Property = BlobBuilder*/
const Value V18566_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18566_350 = {S4, 0, &V18566_350_57444, &P18566_282};	/*Property = Canvas*/
const Value V18566_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18566_283 = {S5, 0, &V18566_283_57159, &P18566_350};	/*Property = CssBackground*/
const Value V18566_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18566_284 = {S6, 0, &V18566_284_57163, &P18566_283};	/*Property = CssBorderImage*/
const Value V18566_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18566_285 = {S7, 0, &V18566_285_57166, &P18566_284};	/*Property = CssCanvas*/
const Value V18566_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18566_286 = {S8, 0, &V18566_286_57168, &P18566_285};	/*Property = CssColor*/
const Value V18566_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18566_237 = {S9, 0, &V18566_237_55786, &P18566_286};	/*Property = CssColumn*/
const Value V18566_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18566_287 = {S10, 0, &V18566_287_57172, &P18566_237};	/*Property = CssFlexbox*/
const Value V18566_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18566_288 = {S11, 0, &V18566_288_57175, &P18566_287};	/*Property = CssFont*/
const Value V18566_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18566_289 = {S12, 0, &V18566_289_57759, &P18566_288};	/*Property = CssImages*/
const Value V18566_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18566_291 = {S13, 0, &V18566_291_57181, &P18566_289};	/*Property = CssMediaQueries*/
const Value V18566_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18566_292 = {S14, 0, &V18566_292_57183, &P18566_291};	/*Property = CssMinMax*/
const Value V18566_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18566_293 = {S15, 0, &V18566_293_57187, &P18566_292};	/*Property = CssOverflow*/
const Value V18566_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18566_294 = {S16, 0, &V18566_294_57190, &P18566_293};	/*Property = CssPosition*/
const Value V18566_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18566_295 = {S17, 0, &V18566_295_57193, &P18566_294};	/*Property = CssText*/
const Value V18566_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18566_297 = {S18, 0, &V18566_297_57195, &P18566_295};	/*Property = CssTransforms*/
const Value V18566_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18566_241 = {S19, 0, &V18566_241_55799, &P18566_297};	/*Property = CssTransitions*/
const Value V18566_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18566_298 = {S20, 0, &V18566_298_57206, &P18566_241};	/*Property = CssUI*/
const Value V18566_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18566_300 = {S21, 0, &V18566_300_57216, &P18566_298};	/*Property = DataSet*/
const Value V18566_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18566_301 = {S22, 0, &V18566_301_57218, &P18566_300};	/*Property = DataUrl*/
const Value V18566_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18566_302 = {S23, 0, &V18566_302_57222, &P18566_301};	/*Property = DeviceOrientation*/
const Value V18566_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18566_304 = {S24, 0, &V18566_304_57228, &P18566_302};	/*Property = FileReader*/
const Value V18566_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18566_305 = {S25, 0, &V18566_305_57231, &P18566_304};	/*Property = FileSaver*/
const Value V18566_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18566_306 = {S26, 0, &V18566_306_57237, &P18566_305};	/*Property = FileWriter*/
const Value V18566_307_57240 = {S0, NULL};	/*Value = False*/
const Property P18566_307 = {S27, 0, &V18566_307_57240, &P18566_306};	/*Property = FormData*/
const Value V18566_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18566_309 = {S28, 0, &V18566_309_57246, &P18566_307};	/*Property = Fullscreen*/
const Value V18566_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18566_310 = {S29, 0, &V18566_310_57248, &P18566_309};	/*Property = GeoLocation*/
const Value V18566_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18566_312 = {S30, 0, &V18566_312_57255, &P18566_310};	/*Property = History*/
const Value V18566_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18566_314 = {S31, 0, &V18566_314_57261, &P18566_312};	/*Property = Html5*/
const Value V18566_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18566_313 = {S32, 0, &V18566_313_57258, &P18566_314};	/*Property = Html-Media-Capture*/
const Value V18566_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18566_315 = {S33, 0, &V18566_315_57264, &P18566_313};	/*Property = Iframe*/
const Value V18566_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18566_316 = {S34, 0, &V18566_316_57267, &P18566_315};	/*Property = IndexedDB*/
const Value V18566_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18566_317 = {S35, 0, &V18566_317_57269, &P18566_316};	/*Property = Json*/
const Value V18566_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18566_15 = {S37, 0, &V18566_15_29, &P18566_317};	/*Property = LayoutEngine*/
const Value V18566_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18566_344 = {S38, 0, &V18566_344_57272, &P18566_15};	/*Property = Masking*/
const Value V18566_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18566_320 = {S39, 0, &V18566_320_57290, &P18566_344};	/*Property = PostMessage*/
const Value V18566_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18566_321 = {S40, 0, &V18566_321_57293, &P18566_320};	/*Property = Progress*/
const Value V18566_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18566_322 = {S41, 0, &V18566_322_57296, &P18566_321};	/*Property = Prompts*/
const Value V18566_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18566_323 = {S42, 0, &V18566_323_57299, &P18566_322};	/*Property = Selector*/
const Value V18566_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18566_324 = {S43, 0, &V18566_324_57306, &P18566_323};	/*Property = Svg*/
const Value V18566_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18566_327 = {S44, 0, &V18566_327_57315, &P18566_324};	/*Property = TouchEvents*/
const Value V18566_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18566_328 = {S45, 0, &V18566_328_57318, &P18566_327};	/*Property = Track*/
const Value V18566_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18566_329 = {S46, 0, &V18566_329_57323, &P18566_328};	/*Property = Video*/
const Value V18566_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18566_330 = {S47, 0, &V18566_330_57326, &P18566_329};	/*Property = Viewport*/
const Value V18566_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18566_332 = {S48, 0, &V18566_332_57339, &P18566_330};	/*Property = WebWorkers*/
const Value V18566_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18566_333 = {S49, 0, &V18566_333_57342, &P18566_332};	/*Property = Xhr2*/
const Profile P18566 = {"18566", &P18566_333};	/*Profile ID - 18566*/
const Value V18057_1_1 = {S2, NULL};	/*Value = True*/
const Property P18057_1 = {S50, 0, &V18057_1_1, NULL};	/*Property = IsMobile*/
const Value V18057_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P18057_5 = {S52, 0, &V18057_5_1304, &P18057_1};	/*Property = ScreenPixelsHeight*/
const Value V18057_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18057_4 = {S54, 0, &V18057_4_1224, &P18057_5};	/*Property = ScreenPixelsWidth*/
const Profile P18057 = {"18057", &P18057_4};	/*Profile ID - 18057*/
const Value V20781_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_279 = {S1, 0, &V20781_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20781_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_282 = {S3, 0, &V20781_282_57155, &P20781_279};	/*Property = BlobBuilder*/
const Value V20781_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_350 = {S4, 0, &V20781_350_57446, &P20781_282};	/*Property = Canvas*/
const Value V20781_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_283 = {S5, 0, &V20781_283_57161, &P20781_350};	/*Property = CssBackground*/
const Value V20781_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_284 = {S6, 0, &V20781_284_57164, &P20781_283};	/*Property = CssBorderImage*/
const Value V20781_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_285 = {S7, 0, &V20781_285_57167, &P20781_284};	/*Property = CssCanvas*/
const Value V20781_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_286 = {S8, 0, &V20781_286_57170, &P20781_285};	/*Property = CssColor*/
const Value V20781_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_237 = {S9, 0, &V20781_237_55786, &P20781_286};	/*Property = CssColumn*/
const Value V20781_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_287 = {S10, 0, &V20781_287_57173, &P20781_237};	/*Property = CssFlexbox*/
const Value V20781_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_288 = {S11, 0, &V20781_288_57176, &P20781_287};	/*Property = CssFont*/
const Value V20781_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_289 = {S12, 0, &V20781_289_86899, &P20781_288};	/*Property = CssImages*/
const Value V20781_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_291 = {S13, 0, &V20781_291_57182, &P20781_289};	/*Property = CssMediaQueries*/
const Value V20781_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_292 = {S14, 0, &V20781_292_57185, &P20781_291};	/*Property = CssMinMax*/
const Value V20781_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_293 = {S15, 0, &V20781_293_57188, &P20781_292};	/*Property = CssOverflow*/
const Value V20781_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_294 = {S16, 0, &V20781_294_57191, &P20781_293};	/*Property = CssPosition*/
const Value V20781_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_295 = {S17, 0, &V20781_295_57194, &P20781_294};	/*Property = CssText*/
const Value V20781_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_297 = {S18, 0, &V20781_297_57197, &P20781_295};	/*Property = CssTransforms*/
const Value V20781_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_241 = {S19, 0, &V20781_241_55801, &P20781_297};	/*Property = CssTransitions*/
const Value V20781_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_298 = {S20, 0, &V20781_298_57208, &P20781_241};	/*Property = CssUI*/
const Value V20781_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_300 = {S21, 0, &V20781_300_57217, &P20781_298};	/*Property = DataSet*/
const Value V20781_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_301 = {S22, 0, &V20781_301_57220, &P20781_300};	/*Property = DataUrl*/
const Value V20781_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_302 = {S23, 0, &V20781_302_57223, &P20781_301};	/*Property = DeviceOrientation*/
const Value V20781_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_304 = {S24, 0, &V20781_304_57229, &P20781_302};	/*Property = FileReader*/
const Value V20781_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_305 = {S25, 0, &V20781_305_57232, &P20781_304};	/*Property = FileSaver*/
const Value V20781_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_306 = {S26, 0, &V20781_306_57238, &P20781_305};	/*Property = FileWriter*/
const Value V20781_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_307 = {S27, 0, &V20781_307_57241, &P20781_306};	/*Property = FormData*/
const Value V20781_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_309 = {S28, 0, &V20781_309_57247, &P20781_307};	/*Property = Fullscreen*/
const Value V20781_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_310 = {S29, 0, &V20781_310_57250, &P20781_309};	/*Property = GeoLocation*/
const Value V20781_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_312 = {S30, 0, &V20781_312_57256, &P20781_310};	/*Property = History*/
const Value V20781_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_314 = {S31, 0, &V20781_314_57262, &P20781_312};	/*Property = Html5*/
const Value V20781_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_313 = {S32, 0, &V20781_313_57259, &P20781_314};	/*Property = Html-Media-Capture*/
const Value V20781_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_315 = {S33, 0, &V20781_315_57265, &P20781_313};	/*Property = Iframe*/
const Value V20781_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_316 = {S34, 0, &V20781_316_57268, &P20781_315};	/*Property = IndexedDB*/
const Value V20781_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_317 = {S35, 0, &V20781_317_57271, &P20781_316};	/*Property = Json*/
const Value V20781_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P20781_15 = {S37, 0, &V20781_15_54167, &P20781_317};	/*Property = LayoutEngine*/
const Value V20781_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_344 = {S38, 0, &V20781_344_57274, &P20781_15};	/*Property = Masking*/
const Value V20781_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_320 = {S39, 0, &V20781_320_57292, &P20781_344};	/*Property = PostMessage*/
const Value V20781_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_321 = {S40, 0, &V20781_321_57295, &P20781_320};	/*Property = Progress*/
const Value V20781_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_322 = {S41, 0, &V20781_322_57298, &P20781_321};	/*Property = Prompts*/
const Value V20781_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_323 = {S42, 0, &V20781_323_57301, &P20781_322};	/*Property = Selector*/
const Value V20781_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_324 = {S43, 0, &V20781_324_57307, &P20781_323};	/*Property = Svg*/
const Value V20781_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_327 = {S44, 0, &V20781_327_57316, &P20781_324};	/*Property = TouchEvents*/
const Value V20781_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_328 = {S45, 0, &V20781_328_57319, &P20781_327};	/*Property = Track*/
const Value V20781_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_329 = {S46, 0, &V20781_329_57325, &P20781_328};	/*Property = Video*/
const Value V20781_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_330 = {S47, 0, &V20781_330_57328, &P20781_329};	/*Property = Viewport*/
const Value V20781_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_332 = {S48, 0, &V20781_332_57340, &P20781_330};	/*Property = WebWorkers*/
const Value V20781_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20781_333 = {S49, 0, &V20781_333_57343, &P20781_332};	/*Property = Xhr2*/
const Profile P20781 = {"20781", &P20781_333};	/*Profile ID - 20781*/
const Value V24447_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_279 = {S1, 0, &V24447_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24447_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_282 = {S3, 0, &V24447_282_57155, &P24447_279};	/*Property = BlobBuilder*/
const Value V24447_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_350 = {S4, 0, &V24447_350_57446, &P24447_282};	/*Property = Canvas*/
const Value V24447_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_283 = {S5, 0, &V24447_283_57161, &P24447_350};	/*Property = CssBackground*/
const Value V24447_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_284 = {S6, 0, &V24447_284_57164, &P24447_283};	/*Property = CssBorderImage*/
const Value V24447_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_285 = {S7, 0, &V24447_285_57167, &P24447_284};	/*Property = CssCanvas*/
const Value V24447_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_286 = {S8, 0, &V24447_286_57170, &P24447_285};	/*Property = CssColor*/
const Value V24447_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_237 = {S9, 0, &V24447_237_55786, &P24447_286};	/*Property = CssColumn*/
const Value V24447_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_287 = {S10, 0, &V24447_287_57173, &P24447_237};	/*Property = CssFlexbox*/
const Value V24447_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_288 = {S11, 0, &V24447_288_57176, &P24447_287};	/*Property = CssFont*/
const Value V24447_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_289 = {S12, 0, &V24447_289_86899, &P24447_288};	/*Property = CssImages*/
const Value V24447_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_291 = {S13, 0, &V24447_291_57182, &P24447_289};	/*Property = CssMediaQueries*/
const Value V24447_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_292 = {S14, 0, &V24447_292_57185, &P24447_291};	/*Property = CssMinMax*/
const Value V24447_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_293 = {S15, 0, &V24447_293_57188, &P24447_292};	/*Property = CssOverflow*/
const Value V24447_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_294 = {S16, 0, &V24447_294_57191, &P24447_293};	/*Property = CssPosition*/
const Value V24447_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_295 = {S17, 0, &V24447_295_57194, &P24447_294};	/*Property = CssText*/
const Value V24447_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_297 = {S18, 0, &V24447_297_57197, &P24447_295};	/*Property = CssTransforms*/
const Value V24447_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_241 = {S19, 0, &V24447_241_55801, &P24447_297};	/*Property = CssTransitions*/
const Value V24447_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_298 = {S20, 0, &V24447_298_57208, &P24447_241};	/*Property = CssUI*/
const Value V24447_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_300 = {S21, 0, &V24447_300_57217, &P24447_298};	/*Property = DataSet*/
const Value V24447_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_301 = {S22, 0, &V24447_301_57220, &P24447_300};	/*Property = DataUrl*/
const Value V24447_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_302 = {S23, 0, &V24447_302_57223, &P24447_301};	/*Property = DeviceOrientation*/
const Value V24447_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_304 = {S24, 0, &V24447_304_57229, &P24447_302};	/*Property = FileReader*/
const Value V24447_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_305 = {S25, 0, &V24447_305_57232, &P24447_304};	/*Property = FileSaver*/
const Value V24447_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_306 = {S26, 0, &V24447_306_57238, &P24447_305};	/*Property = FileWriter*/
const Value V24447_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_307 = {S27, 0, &V24447_307_57241, &P24447_306};	/*Property = FormData*/
const Value V24447_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_309 = {S28, 0, &V24447_309_57247, &P24447_307};	/*Property = Fullscreen*/
const Value V24447_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_310 = {S29, 0, &V24447_310_57250, &P24447_309};	/*Property = GeoLocation*/
const Value V24447_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_312 = {S30, 0, &V24447_312_57256, &P24447_310};	/*Property = History*/
const Value V24447_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_314 = {S31, 0, &V24447_314_57262, &P24447_312};	/*Property = Html5*/
const Value V24447_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_313 = {S32, 0, &V24447_313_57259, &P24447_314};	/*Property = Html-Media-Capture*/
const Value V24447_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_315 = {S33, 0, &V24447_315_57265, &P24447_313};	/*Property = Iframe*/
const Value V24447_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_316 = {S34, 0, &V24447_316_57268, &P24447_315};	/*Property = IndexedDB*/
const Value V24447_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_317 = {S35, 0, &V24447_317_57271, &P24447_316};	/*Property = Json*/
const Value V24447_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24447_15 = {S37, 0, &V24447_15_28, &P24447_317};	/*Property = LayoutEngine*/
const Value V24447_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_344 = {S38, 0, &V24447_344_57274, &P24447_15};	/*Property = Masking*/
const Value V24447_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_320 = {S39, 0, &V24447_320_57292, &P24447_344};	/*Property = PostMessage*/
const Value V24447_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_321 = {S40, 0, &V24447_321_57295, &P24447_320};	/*Property = Progress*/
const Value V24447_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_322 = {S41, 0, &V24447_322_57298, &P24447_321};	/*Property = Prompts*/
const Value V24447_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_323 = {S42, 0, &V24447_323_57301, &P24447_322};	/*Property = Selector*/
const Value V24447_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_324 = {S43, 0, &V24447_324_57307, &P24447_323};	/*Property = Svg*/
const Value V24447_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_327 = {S44, 0, &V24447_327_57316, &P24447_324};	/*Property = TouchEvents*/
const Value V24447_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_328 = {S45, 0, &V24447_328_57319, &P24447_327};	/*Property = Track*/
const Value V24447_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_329 = {S46, 0, &V24447_329_57325, &P24447_328};	/*Property = Video*/
const Value V24447_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_330 = {S47, 0, &V24447_330_57328, &P24447_329};	/*Property = Viewport*/
const Value V24447_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_332 = {S48, 0, &V24447_332_57340, &P24447_330};	/*Property = WebWorkers*/
const Value V24447_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24447_333 = {S49, 0, &V24447_333_57343, &P24447_332};	/*Property = Xhr2*/
const Profile P24447 = {"24447", &P24447_333};	/*Profile ID - 24447*/
const Value V18212_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_279 = {S1, 0, &V18212_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18212_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_282 = {S3, 0, &V18212_282_57155, &P18212_279};	/*Property = BlobBuilder*/
const Value V18212_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_350 = {S4, 0, &V18212_350_57446, &P18212_282};	/*Property = Canvas*/
const Value V18212_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_283 = {S5, 0, &V18212_283_57161, &P18212_350};	/*Property = CssBackground*/
const Value V18212_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_284 = {S6, 0, &V18212_284_57164, &P18212_283};	/*Property = CssBorderImage*/
const Value V18212_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_285 = {S7, 0, &V18212_285_57167, &P18212_284};	/*Property = CssCanvas*/
const Value V18212_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_286 = {S8, 0, &V18212_286_57170, &P18212_285};	/*Property = CssColor*/
const Value V18212_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_237 = {S9, 0, &V18212_237_55786, &P18212_286};	/*Property = CssColumn*/
const Value V18212_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_287 = {S10, 0, &V18212_287_57173, &P18212_237};	/*Property = CssFlexbox*/
const Value V18212_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_288 = {S11, 0, &V18212_288_57176, &P18212_287};	/*Property = CssFont*/
const Value V18212_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_289 = {S12, 0, &V18212_289_86899, &P18212_288};	/*Property = CssImages*/
const Value V18212_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_291 = {S13, 0, &V18212_291_57182, &P18212_289};	/*Property = CssMediaQueries*/
const Value V18212_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_292 = {S14, 0, &V18212_292_57185, &P18212_291};	/*Property = CssMinMax*/
const Value V18212_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_293 = {S15, 0, &V18212_293_57188, &P18212_292};	/*Property = CssOverflow*/
const Value V18212_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_294 = {S16, 0, &V18212_294_57191, &P18212_293};	/*Property = CssPosition*/
const Value V18212_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_295 = {S17, 0, &V18212_295_57194, &P18212_294};	/*Property = CssText*/
const Value V18212_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_297 = {S18, 0, &V18212_297_57197, &P18212_295};	/*Property = CssTransforms*/
const Value V18212_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_241 = {S19, 0, &V18212_241_55801, &P18212_297};	/*Property = CssTransitions*/
const Value V18212_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_298 = {S20, 0, &V18212_298_57208, &P18212_241};	/*Property = CssUI*/
const Value V18212_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_300 = {S21, 0, &V18212_300_57217, &P18212_298};	/*Property = DataSet*/
const Value V18212_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_301 = {S22, 0, &V18212_301_57220, &P18212_300};	/*Property = DataUrl*/
const Value V18212_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_302 = {S23, 0, &V18212_302_57223, &P18212_301};	/*Property = DeviceOrientation*/
const Value V18212_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_304 = {S24, 0, &V18212_304_57229, &P18212_302};	/*Property = FileReader*/
const Value V18212_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_305 = {S25, 0, &V18212_305_57232, &P18212_304};	/*Property = FileSaver*/
const Value V18212_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_306 = {S26, 0, &V18212_306_57238, &P18212_305};	/*Property = FileWriter*/
const Value V18212_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_307 = {S27, 0, &V18212_307_57241, &P18212_306};	/*Property = FormData*/
const Value V18212_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_309 = {S28, 0, &V18212_309_57247, &P18212_307};	/*Property = Fullscreen*/
const Value V18212_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_310 = {S29, 0, &V18212_310_57250, &P18212_309};	/*Property = GeoLocation*/
const Value V18212_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_312 = {S30, 0, &V18212_312_57256, &P18212_310};	/*Property = History*/
const Value V18212_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_314 = {S31, 0, &V18212_314_57262, &P18212_312};	/*Property = Html5*/
const Value V18212_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_313 = {S32, 0, &V18212_313_57259, &P18212_314};	/*Property = Html-Media-Capture*/
const Value V18212_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_315 = {S33, 0, &V18212_315_57265, &P18212_313};	/*Property = Iframe*/
const Value V18212_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_316 = {S34, 0, &V18212_316_57268, &P18212_315};	/*Property = IndexedDB*/
const Value V18212_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_317 = {S35, 0, &V18212_317_57271, &P18212_316};	/*Property = Json*/
const Value V18212_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18212_15 = {S37, 0, &V18212_15_29, &P18212_317};	/*Property = LayoutEngine*/
const Value V18212_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_344 = {S38, 0, &V18212_344_57274, &P18212_15};	/*Property = Masking*/
const Value V18212_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_320 = {S39, 0, &V18212_320_57292, &P18212_344};	/*Property = PostMessage*/
const Value V18212_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_321 = {S40, 0, &V18212_321_57295, &P18212_320};	/*Property = Progress*/
const Value V18212_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_322 = {S41, 0, &V18212_322_57298, &P18212_321};	/*Property = Prompts*/
const Value V18212_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_323 = {S42, 0, &V18212_323_57301, &P18212_322};	/*Property = Selector*/
const Value V18212_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_324 = {S43, 0, &V18212_324_57307, &P18212_323};	/*Property = Svg*/
const Value V18212_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_327 = {S44, 0, &V18212_327_57316, &P18212_324};	/*Property = TouchEvents*/
const Value V18212_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_328 = {S45, 0, &V18212_328_57319, &P18212_327};	/*Property = Track*/
const Value V18212_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_329 = {S46, 0, &V18212_329_57325, &P18212_328};	/*Property = Video*/
const Value V18212_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_330 = {S47, 0, &V18212_330_57328, &P18212_329};	/*Property = Viewport*/
const Value V18212_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_332 = {S48, 0, &V18212_332_57340, &P18212_330};	/*Property = WebWorkers*/
const Value V18212_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18212_333 = {S49, 0, &V18212_333_57343, &P18212_332};	/*Property = Xhr2*/
const Profile P18212 = {"18212", &P18212_333};	/*Profile ID - 18212*/
const Value V21557_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_279 = {S1, 0, &V21557_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21557_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_282 = {S3, 0, &V21557_282_57155, &P21557_279};	/*Property = BlobBuilder*/
const Value V21557_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_350 = {S4, 0, &V21557_350_57446, &P21557_282};	/*Property = Canvas*/
const Value V21557_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_283 = {S5, 0, &V21557_283_57161, &P21557_350};	/*Property = CssBackground*/
const Value V21557_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_284 = {S6, 0, &V21557_284_57164, &P21557_283};	/*Property = CssBorderImage*/
const Value V21557_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_285 = {S7, 0, &V21557_285_57167, &P21557_284};	/*Property = CssCanvas*/
const Value V21557_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_286 = {S8, 0, &V21557_286_57170, &P21557_285};	/*Property = CssColor*/
const Value V21557_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_237 = {S9, 0, &V21557_237_55786, &P21557_286};	/*Property = CssColumn*/
const Value V21557_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_287 = {S10, 0, &V21557_287_57173, &P21557_237};	/*Property = CssFlexbox*/
const Value V21557_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_288 = {S11, 0, &V21557_288_57176, &P21557_287};	/*Property = CssFont*/
const Value V21557_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_289 = {S12, 0, &V21557_289_86899, &P21557_288};	/*Property = CssImages*/
const Value V21557_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_291 = {S13, 0, &V21557_291_57182, &P21557_289};	/*Property = CssMediaQueries*/
const Value V21557_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_292 = {S14, 0, &V21557_292_57185, &P21557_291};	/*Property = CssMinMax*/
const Value V21557_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_293 = {S15, 0, &V21557_293_57188, &P21557_292};	/*Property = CssOverflow*/
const Value V21557_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_294 = {S16, 0, &V21557_294_57191, &P21557_293};	/*Property = CssPosition*/
const Value V21557_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_295 = {S17, 0, &V21557_295_57194, &P21557_294};	/*Property = CssText*/
const Value V21557_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_297 = {S18, 0, &V21557_297_57197, &P21557_295};	/*Property = CssTransforms*/
const Value V21557_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_241 = {S19, 0, &V21557_241_55801, &P21557_297};	/*Property = CssTransitions*/
const Value V21557_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_298 = {S20, 0, &V21557_298_57208, &P21557_241};	/*Property = CssUI*/
const Value V21557_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_300 = {S21, 0, &V21557_300_57217, &P21557_298};	/*Property = DataSet*/
const Value V21557_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_301 = {S22, 0, &V21557_301_57220, &P21557_300};	/*Property = DataUrl*/
const Value V21557_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_302 = {S23, 0, &V21557_302_57223, &P21557_301};	/*Property = DeviceOrientation*/
const Value V21557_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_304 = {S24, 0, &V21557_304_57229, &P21557_302};	/*Property = FileReader*/
const Value V21557_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_305 = {S25, 0, &V21557_305_57232, &P21557_304};	/*Property = FileSaver*/
const Value V21557_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_306 = {S26, 0, &V21557_306_57238, &P21557_305};	/*Property = FileWriter*/
const Value V21557_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_307 = {S27, 0, &V21557_307_57241, &P21557_306};	/*Property = FormData*/
const Value V21557_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_309 = {S28, 0, &V21557_309_57247, &P21557_307};	/*Property = Fullscreen*/
const Value V21557_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_310 = {S29, 0, &V21557_310_57250, &P21557_309};	/*Property = GeoLocation*/
const Value V21557_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_312 = {S30, 0, &V21557_312_57256, &P21557_310};	/*Property = History*/
const Value V21557_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_314 = {S31, 0, &V21557_314_57262, &P21557_312};	/*Property = Html5*/
const Value V21557_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_313 = {S32, 0, &V21557_313_57259, &P21557_314};	/*Property = Html-Media-Capture*/
const Value V21557_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_315 = {S33, 0, &V21557_315_57265, &P21557_313};	/*Property = Iframe*/
const Value V21557_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_316 = {S34, 0, &V21557_316_57268, &P21557_315};	/*Property = IndexedDB*/
const Value V21557_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_317 = {S35, 0, &V21557_317_57271, &P21557_316};	/*Property = Json*/
const Value V21557_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21557_15 = {S37, 0, &V21557_15_28, &P21557_317};	/*Property = LayoutEngine*/
const Value V21557_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_344 = {S38, 0, &V21557_344_57274, &P21557_15};	/*Property = Masking*/
const Value V21557_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_320 = {S39, 0, &V21557_320_57292, &P21557_344};	/*Property = PostMessage*/
const Value V21557_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_321 = {S40, 0, &V21557_321_57295, &P21557_320};	/*Property = Progress*/
const Value V21557_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_322 = {S41, 0, &V21557_322_57298, &P21557_321};	/*Property = Prompts*/
const Value V21557_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_323 = {S42, 0, &V21557_323_57301, &P21557_322};	/*Property = Selector*/
const Value V21557_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_324 = {S43, 0, &V21557_324_57307, &P21557_323};	/*Property = Svg*/
const Value V21557_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_327 = {S44, 0, &V21557_327_57316, &P21557_324};	/*Property = TouchEvents*/
const Value V21557_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_328 = {S45, 0, &V21557_328_57319, &P21557_327};	/*Property = Track*/
const Value V21557_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_329 = {S46, 0, &V21557_329_57325, &P21557_328};	/*Property = Video*/
const Value V21557_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_330 = {S47, 0, &V21557_330_57328, &P21557_329};	/*Property = Viewport*/
const Value V21557_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_332 = {S48, 0, &V21557_332_57340, &P21557_330};	/*Property = WebWorkers*/
const Value V21557_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21557_333 = {S49, 0, &V21557_333_57343, &P21557_332};	/*Property = Xhr2*/
const Profile P21557 = {"21557", &P21557_333};	/*Profile ID - 21557*/
const Value V21779_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_279 = {S1, 0, &V21779_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21779_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_282 = {S3, 0, &V21779_282_57155, &P21779_279};	/*Property = BlobBuilder*/
const Value V21779_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_350 = {S4, 0, &V21779_350_57446, &P21779_282};	/*Property = Canvas*/
const Value V21779_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_283 = {S5, 0, &V21779_283_57161, &P21779_350};	/*Property = CssBackground*/
const Value V21779_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_284 = {S6, 0, &V21779_284_57164, &P21779_283};	/*Property = CssBorderImage*/
const Value V21779_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_285 = {S7, 0, &V21779_285_57167, &P21779_284};	/*Property = CssCanvas*/
const Value V21779_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_286 = {S8, 0, &V21779_286_57170, &P21779_285};	/*Property = CssColor*/
const Value V21779_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_237 = {S9, 0, &V21779_237_55786, &P21779_286};	/*Property = CssColumn*/
const Value V21779_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_287 = {S10, 0, &V21779_287_57173, &P21779_237};	/*Property = CssFlexbox*/
const Value V21779_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_288 = {S11, 0, &V21779_288_57176, &P21779_287};	/*Property = CssFont*/
const Value V21779_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_289 = {S12, 0, &V21779_289_86899, &P21779_288};	/*Property = CssImages*/
const Value V21779_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_291 = {S13, 0, &V21779_291_57182, &P21779_289};	/*Property = CssMediaQueries*/
const Value V21779_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_292 = {S14, 0, &V21779_292_57185, &P21779_291};	/*Property = CssMinMax*/
const Value V21779_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_293 = {S15, 0, &V21779_293_57188, &P21779_292};	/*Property = CssOverflow*/
const Value V21779_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_294 = {S16, 0, &V21779_294_57191, &P21779_293};	/*Property = CssPosition*/
const Value V21779_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_295 = {S17, 0, &V21779_295_57194, &P21779_294};	/*Property = CssText*/
const Value V21779_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_297 = {S18, 0, &V21779_297_57197, &P21779_295};	/*Property = CssTransforms*/
const Value V21779_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_241 = {S19, 0, &V21779_241_55801, &P21779_297};	/*Property = CssTransitions*/
const Value V21779_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_298 = {S20, 0, &V21779_298_57208, &P21779_241};	/*Property = CssUI*/
const Value V21779_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_300 = {S21, 0, &V21779_300_57217, &P21779_298};	/*Property = DataSet*/
const Value V21779_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_301 = {S22, 0, &V21779_301_57220, &P21779_300};	/*Property = DataUrl*/
const Value V21779_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_302 = {S23, 0, &V21779_302_57223, &P21779_301};	/*Property = DeviceOrientation*/
const Value V21779_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_304 = {S24, 0, &V21779_304_57229, &P21779_302};	/*Property = FileReader*/
const Value V21779_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_305 = {S25, 0, &V21779_305_57232, &P21779_304};	/*Property = FileSaver*/
const Value V21779_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_306 = {S26, 0, &V21779_306_57238, &P21779_305};	/*Property = FileWriter*/
const Value V21779_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_307 = {S27, 0, &V21779_307_57241, &P21779_306};	/*Property = FormData*/
const Value V21779_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_309 = {S28, 0, &V21779_309_57247, &P21779_307};	/*Property = Fullscreen*/
const Value V21779_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_310 = {S29, 0, &V21779_310_57250, &P21779_309};	/*Property = GeoLocation*/
const Value V21779_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_312 = {S30, 0, &V21779_312_57256, &P21779_310};	/*Property = History*/
const Value V21779_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_314 = {S31, 0, &V21779_314_57262, &P21779_312};	/*Property = Html5*/
const Value V21779_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_313 = {S32, 0, &V21779_313_57259, &P21779_314};	/*Property = Html-Media-Capture*/
const Value V21779_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_315 = {S33, 0, &V21779_315_57265, &P21779_313};	/*Property = Iframe*/
const Value V21779_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_316 = {S34, 0, &V21779_316_57268, &P21779_315};	/*Property = IndexedDB*/
const Value V21779_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_317 = {S35, 0, &V21779_317_57271, &P21779_316};	/*Property = Json*/
const Value V21779_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21779_15 = {S37, 0, &V21779_15_54167, &P21779_317};	/*Property = LayoutEngine*/
const Value V21779_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_344 = {S38, 0, &V21779_344_57274, &P21779_15};	/*Property = Masking*/
const Value V21779_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_320 = {S39, 0, &V21779_320_57292, &P21779_344};	/*Property = PostMessage*/
const Value V21779_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_321 = {S40, 0, &V21779_321_57295, &P21779_320};	/*Property = Progress*/
const Value V21779_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_322 = {S41, 0, &V21779_322_57298, &P21779_321};	/*Property = Prompts*/
const Value V21779_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_323 = {S42, 0, &V21779_323_57301, &P21779_322};	/*Property = Selector*/
const Value V21779_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_324 = {S43, 0, &V21779_324_57307, &P21779_323};	/*Property = Svg*/
const Value V21779_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_327 = {S44, 0, &V21779_327_57316, &P21779_324};	/*Property = TouchEvents*/
const Value V21779_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_328 = {S45, 0, &V21779_328_57319, &P21779_327};	/*Property = Track*/
const Value V21779_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_329 = {S46, 0, &V21779_329_57325, &P21779_328};	/*Property = Video*/
const Value V21779_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_330 = {S47, 0, &V21779_330_57328, &P21779_329};	/*Property = Viewport*/
const Value V21779_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_332 = {S48, 0, &V21779_332_57340, &P21779_330};	/*Property = WebWorkers*/
const Value V21779_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21779_333 = {S49, 0, &V21779_333_57343, &P21779_332};	/*Property = Xhr2*/
const Profile P21779 = {"21779", &P21779_333};	/*Profile ID - 21779*/
const Value V22354_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22354_279 = {S1, 0, &V22354_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22354_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22354_282 = {S3, 0, &V22354_282_57153, &P22354_279};	/*Property = BlobBuilder*/
const Value V22354_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22354_350 = {S4, 0, &V22354_350_57444, &P22354_282};	/*Property = Canvas*/
const Value V22354_283_57160 = {S0, NULL};	/*Value = False*/
const Property P22354_283 = {S5, 0, &V22354_283_57160, &P22354_350};	/*Property = CssBackground*/
const Value V22354_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22354_284 = {S6, 0, &V22354_284_57163, &P22354_283};	/*Property = CssBorderImage*/
const Value V22354_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22354_285 = {S7, 0, &V22354_285_57166, &P22354_284};	/*Property = CssCanvas*/
const Value V22354_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22354_286 = {S8, 0, &V22354_286_57168, &P22354_285};	/*Property = CssColor*/
const Value V22354_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22354_237 = {S9, 0, &V22354_237_55784, &P22354_286};	/*Property = CssColumn*/
const Value V22354_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22354_287 = {S10, 0, &V22354_287_57172, &P22354_237};	/*Property = CssFlexbox*/
const Value V22354_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22354_288 = {S11, 0, &V22354_288_57175, &P22354_287};	/*Property = CssFont*/
const Value V22354_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22354_289 = {S12, 0, &V22354_289_86899, &P22354_288};	/*Property = CssImages*/
const Value V22354_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22354_291 = {S13, 0, &V22354_291_57181, &P22354_289};	/*Property = CssMediaQueries*/
const Value V22354_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22354_292 = {S14, 0, &V22354_292_57183, &P22354_291};	/*Property = CssMinMax*/
const Value V22354_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22354_293 = {S15, 0, &V22354_293_57187, &P22354_292};	/*Property = CssOverflow*/
const Value V22354_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22354_294 = {S16, 0, &V22354_294_57189, &P22354_293};	/*Property = CssPosition*/
const Value V22354_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22354_295 = {S17, 0, &V22354_295_57193, &P22354_294};	/*Property = CssText*/
const Value V22354_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22354_297 = {S18, 0, &V22354_297_57195, &P22354_295};	/*Property = CssTransforms*/
const Value V22354_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22354_241 = {S19, 0, &V22354_241_55799, &P22354_297};	/*Property = CssTransitions*/
const Value V22354_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22354_298 = {S20, 0, &V22354_298_57206, &P22354_241};	/*Property = CssUI*/
const Value V22354_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22354_300 = {S21, 0, &V22354_300_57215, &P22354_298};	/*Property = DataSet*/
const Value V22354_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22354_301 = {S22, 0, &V22354_301_57218, &P22354_300};	/*Property = DataUrl*/
const Value V22354_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22354_302 = {S23, 0, &V22354_302_57221, &P22354_301};	/*Property = DeviceOrientation*/
const Value V22354_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22354_304 = {S24, 0, &V22354_304_57227, &P22354_302};	/*Property = FileReader*/
const Value V22354_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22354_305 = {S25, 0, &V22354_305_57231, &P22354_304};	/*Property = FileSaver*/
const Value V22354_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22354_306 = {S26, 0, &V22354_306_57237, &P22354_305};	/*Property = FileWriter*/
const Value V22354_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22354_307 = {S27, 0, &V22354_307_57239, &P22354_306};	/*Property = FormData*/
const Value V22354_309_57245 = {S2, NULL};	/*Value = True*/
const Property P22354_309 = {S28, 0, &V22354_309_57245, &P22354_307};	/*Property = Fullscreen*/
const Value V22354_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22354_310 = {S29, 0, &V22354_310_57248, &P22354_309};	/*Property = GeoLocation*/
const Value V22354_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22354_312 = {S30, 0, &V22354_312_57254, &P22354_310};	/*Property = History*/
const Value V22354_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22354_314 = {S31, 0, &V22354_314_57261, &P22354_312};	/*Property = Html5*/
const Value V22354_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22354_313 = {S32, 0, &V22354_313_57258, &P22354_314};	/*Property = Html-Media-Capture*/
const Value V22354_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22354_315 = {S33, 0, &V22354_315_57264, &P22354_313};	/*Property = Iframe*/
const Value V22354_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22354_316 = {S34, 0, &V22354_316_57266, &P22354_315};	/*Property = IndexedDB*/
const Value V22354_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22354_317 = {S35, 0, &V22354_317_57269, &P22354_316};	/*Property = Json*/
const Value V22354_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22354_15 = {S37, 0, &V22354_15_29, &P22354_317};	/*Property = LayoutEngine*/
const Value V22354_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22354_344 = {S38, 0, &V22354_344_57272, &P22354_15};	/*Property = Masking*/
const Value V22354_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22354_320 = {S39, 0, &V22354_320_57290, &P22354_344};	/*Property = PostMessage*/
const Value V22354_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22354_321 = {S40, 0, &V22354_321_57293, &P22354_320};	/*Property = Progress*/
const Value V22354_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22354_322 = {S41, 0, &V22354_322_57296, &P22354_321};	/*Property = Prompts*/
const Value V22354_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22354_323 = {S42, 0, &V22354_323_57299, &P22354_322};	/*Property = Selector*/
const Value V22354_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22354_324 = {S43, 0, &V22354_324_57305, &P22354_323};	/*Property = Svg*/
const Value V22354_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22354_327 = {S44, 0, &V22354_327_57315, &P22354_324};	/*Property = TouchEvents*/
const Value V22354_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22354_328 = {S45, 0, &V22354_328_57318, &P22354_327};	/*Property = Track*/
const Value V22354_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22354_329 = {S46, 0, &V22354_329_57323, &P22354_328};	/*Property = Video*/
const Value V22354_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22354_330 = {S47, 0, &V22354_330_57326, &P22354_329};	/*Property = Viewport*/
const Value V22354_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22354_332 = {S48, 0, &V22354_332_57339, &P22354_330};	/*Property = WebWorkers*/
const Value V22354_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22354_333 = {S49, 0, &V22354_333_57342, &P22354_332};	/*Property = Xhr2*/
const Profile P22354 = {"22354", &P22354_333};	/*Profile ID - 22354*/
const Value V18700_1_1 = {S2, NULL};	/*Value = True*/
const Property P18700_1 = {S50, 0, &V18700_1_1, NULL};	/*Property = IsMobile*/
const Value V18700_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18700_5 = {S52, 0, &V18700_5_1222, &P18700_1};	/*Property = ScreenPixelsHeight*/
const Value V18700_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18700_4 = {S54, 0, &V18700_4_1224, &P18700_5};	/*Property = ScreenPixelsWidth*/
const Profile P18700 = {"18700", &P18700_4};	/*Profile ID - 18700*/
const Value V18221_1_1 = {S2, NULL};	/*Value = True*/
const Property P18221_1 = {S50, 0, &V18221_1_1, NULL};	/*Property = IsMobile*/
const Value V18221_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P18221_5 = {S52, 0, &V18221_5_1380, &P18221_1};	/*Property = ScreenPixelsHeight*/
const Value V18221_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18221_4 = {S54, 0, &V18221_4_1279, &P18221_5};	/*Property = ScreenPixelsWidth*/
const Profile P18221 = {"18221", &P18221_4};	/*Profile ID - 18221*/
const Value V18200_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_279 = {S1, 0, &V18200_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18200_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_282 = {S3, 0, &V18200_282_57155, &P18200_279};	/*Property = BlobBuilder*/
const Value V18200_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_350 = {S4, 0, &V18200_350_57446, &P18200_282};	/*Property = Canvas*/
const Value V18200_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_283 = {S5, 0, &V18200_283_57161, &P18200_350};	/*Property = CssBackground*/
const Value V18200_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_284 = {S6, 0, &V18200_284_57164, &P18200_283};	/*Property = CssBorderImage*/
const Value V18200_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_285 = {S7, 0, &V18200_285_57167, &P18200_284};	/*Property = CssCanvas*/
const Value V18200_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_286 = {S8, 0, &V18200_286_57170, &P18200_285};	/*Property = CssColor*/
const Value V18200_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_237 = {S9, 0, &V18200_237_55786, &P18200_286};	/*Property = CssColumn*/
const Value V18200_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_287 = {S10, 0, &V18200_287_57173, &P18200_237};	/*Property = CssFlexbox*/
const Value V18200_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_288 = {S11, 0, &V18200_288_57176, &P18200_287};	/*Property = CssFont*/
const Value V18200_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_289 = {S12, 0, &V18200_289_86899, &P18200_288};	/*Property = CssImages*/
const Value V18200_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_291 = {S13, 0, &V18200_291_57182, &P18200_289};	/*Property = CssMediaQueries*/
const Value V18200_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_292 = {S14, 0, &V18200_292_57185, &P18200_291};	/*Property = CssMinMax*/
const Value V18200_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_293 = {S15, 0, &V18200_293_57188, &P18200_292};	/*Property = CssOverflow*/
const Value V18200_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_294 = {S16, 0, &V18200_294_57191, &P18200_293};	/*Property = CssPosition*/
const Value V18200_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_295 = {S17, 0, &V18200_295_57194, &P18200_294};	/*Property = CssText*/
const Value V18200_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_297 = {S18, 0, &V18200_297_57197, &P18200_295};	/*Property = CssTransforms*/
const Value V18200_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_241 = {S19, 0, &V18200_241_55801, &P18200_297};	/*Property = CssTransitions*/
const Value V18200_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_298 = {S20, 0, &V18200_298_57208, &P18200_241};	/*Property = CssUI*/
const Value V18200_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_300 = {S21, 0, &V18200_300_57217, &P18200_298};	/*Property = DataSet*/
const Value V18200_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_301 = {S22, 0, &V18200_301_57220, &P18200_300};	/*Property = DataUrl*/
const Value V18200_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_302 = {S23, 0, &V18200_302_57223, &P18200_301};	/*Property = DeviceOrientation*/
const Value V18200_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_304 = {S24, 0, &V18200_304_57229, &P18200_302};	/*Property = FileReader*/
const Value V18200_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_305 = {S25, 0, &V18200_305_57232, &P18200_304};	/*Property = FileSaver*/
const Value V18200_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_306 = {S26, 0, &V18200_306_57238, &P18200_305};	/*Property = FileWriter*/
const Value V18200_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_307 = {S27, 0, &V18200_307_57241, &P18200_306};	/*Property = FormData*/
const Value V18200_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_309 = {S28, 0, &V18200_309_57247, &P18200_307};	/*Property = Fullscreen*/
const Value V18200_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_310 = {S29, 0, &V18200_310_57250, &P18200_309};	/*Property = GeoLocation*/
const Value V18200_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_312 = {S30, 0, &V18200_312_57256, &P18200_310};	/*Property = History*/
const Value V18200_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_314 = {S31, 0, &V18200_314_57262, &P18200_312};	/*Property = Html5*/
const Value V18200_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_313 = {S32, 0, &V18200_313_57259, &P18200_314};	/*Property = Html-Media-Capture*/
const Value V18200_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_315 = {S33, 0, &V18200_315_57265, &P18200_313};	/*Property = Iframe*/
const Value V18200_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_316 = {S34, 0, &V18200_316_57268, &P18200_315};	/*Property = IndexedDB*/
const Value V18200_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_317 = {S35, 0, &V18200_317_57271, &P18200_316};	/*Property = Json*/
const Value V18200_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18200_15 = {S37, 0, &V18200_15_29, &P18200_317};	/*Property = LayoutEngine*/
const Value V18200_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_344 = {S38, 0, &V18200_344_57274, &P18200_15};	/*Property = Masking*/
const Value V18200_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_320 = {S39, 0, &V18200_320_57292, &P18200_344};	/*Property = PostMessage*/
const Value V18200_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_321 = {S40, 0, &V18200_321_57295, &P18200_320};	/*Property = Progress*/
const Value V18200_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_322 = {S41, 0, &V18200_322_57298, &P18200_321};	/*Property = Prompts*/
const Value V18200_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_323 = {S42, 0, &V18200_323_57301, &P18200_322};	/*Property = Selector*/
const Value V18200_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_324 = {S43, 0, &V18200_324_57307, &P18200_323};	/*Property = Svg*/
const Value V18200_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_327 = {S44, 0, &V18200_327_57316, &P18200_324};	/*Property = TouchEvents*/
const Value V18200_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_328 = {S45, 0, &V18200_328_57319, &P18200_327};	/*Property = Track*/
const Value V18200_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_329 = {S46, 0, &V18200_329_57325, &P18200_328};	/*Property = Video*/
const Value V18200_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_330 = {S47, 0, &V18200_330_57328, &P18200_329};	/*Property = Viewport*/
const Value V18200_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_332 = {S48, 0, &V18200_332_57340, &P18200_330};	/*Property = WebWorkers*/
const Value V18200_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18200_333 = {S49, 0, &V18200_333_57343, &P18200_332};	/*Property = Xhr2*/
const Profile P18200 = {"18200", &P18200_333};	/*Profile ID - 18200*/
const Value V18376_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_279 = {S1, 0, &V18376_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18376_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_282 = {S3, 0, &V18376_282_57155, &P18376_279};	/*Property = BlobBuilder*/
const Value V18376_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_350 = {S4, 0, &V18376_350_57446, &P18376_282};	/*Property = Canvas*/
const Value V18376_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_283 = {S5, 0, &V18376_283_57161, &P18376_350};	/*Property = CssBackground*/
const Value V18376_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_284 = {S6, 0, &V18376_284_57164, &P18376_283};	/*Property = CssBorderImage*/
const Value V18376_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_285 = {S7, 0, &V18376_285_57167, &P18376_284};	/*Property = CssCanvas*/
const Value V18376_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_286 = {S8, 0, &V18376_286_57170, &P18376_285};	/*Property = CssColor*/
const Value V18376_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_237 = {S9, 0, &V18376_237_55786, &P18376_286};	/*Property = CssColumn*/
const Value V18376_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_287 = {S10, 0, &V18376_287_57173, &P18376_237};	/*Property = CssFlexbox*/
const Value V18376_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_288 = {S11, 0, &V18376_288_57176, &P18376_287};	/*Property = CssFont*/
const Value V18376_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_289 = {S12, 0, &V18376_289_86899, &P18376_288};	/*Property = CssImages*/
const Value V18376_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_291 = {S13, 0, &V18376_291_57182, &P18376_289};	/*Property = CssMediaQueries*/
const Value V18376_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_292 = {S14, 0, &V18376_292_57185, &P18376_291};	/*Property = CssMinMax*/
const Value V18376_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_293 = {S15, 0, &V18376_293_57188, &P18376_292};	/*Property = CssOverflow*/
const Value V18376_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_294 = {S16, 0, &V18376_294_57191, &P18376_293};	/*Property = CssPosition*/
const Value V18376_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_295 = {S17, 0, &V18376_295_57194, &P18376_294};	/*Property = CssText*/
const Value V18376_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_297 = {S18, 0, &V18376_297_57197, &P18376_295};	/*Property = CssTransforms*/
const Value V18376_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_241 = {S19, 0, &V18376_241_55801, &P18376_297};	/*Property = CssTransitions*/
const Value V18376_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_298 = {S20, 0, &V18376_298_57208, &P18376_241};	/*Property = CssUI*/
const Value V18376_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_300 = {S21, 0, &V18376_300_57217, &P18376_298};	/*Property = DataSet*/
const Value V18376_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_301 = {S22, 0, &V18376_301_57220, &P18376_300};	/*Property = DataUrl*/
const Value V18376_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_302 = {S23, 0, &V18376_302_57223, &P18376_301};	/*Property = DeviceOrientation*/
const Value V18376_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_304 = {S24, 0, &V18376_304_57229, &P18376_302};	/*Property = FileReader*/
const Value V18376_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_305 = {S25, 0, &V18376_305_57232, &P18376_304};	/*Property = FileSaver*/
const Value V18376_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_306 = {S26, 0, &V18376_306_57238, &P18376_305};	/*Property = FileWriter*/
const Value V18376_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_307 = {S27, 0, &V18376_307_57241, &P18376_306};	/*Property = FormData*/
const Value V18376_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_309 = {S28, 0, &V18376_309_57247, &P18376_307};	/*Property = Fullscreen*/
const Value V18376_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_310 = {S29, 0, &V18376_310_57250, &P18376_309};	/*Property = GeoLocation*/
const Value V18376_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_312 = {S30, 0, &V18376_312_57256, &P18376_310};	/*Property = History*/
const Value V18376_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_314 = {S31, 0, &V18376_314_57262, &P18376_312};	/*Property = Html5*/
const Value V18376_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_313 = {S32, 0, &V18376_313_57259, &P18376_314};	/*Property = Html-Media-Capture*/
const Value V18376_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_315 = {S33, 0, &V18376_315_57265, &P18376_313};	/*Property = Iframe*/
const Value V18376_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_316 = {S34, 0, &V18376_316_57268, &P18376_315};	/*Property = IndexedDB*/
const Value V18376_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_317 = {S35, 0, &V18376_317_57271, &P18376_316};	/*Property = Json*/
const Value V18376_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_15 = {S37, 0, &V18376_15_72, &P18376_317};	/*Property = LayoutEngine*/
const Value V18376_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_344 = {S38, 0, &V18376_344_57274, &P18376_15};	/*Property = Masking*/
const Value V18376_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_320 = {S39, 0, &V18376_320_57292, &P18376_344};	/*Property = PostMessage*/
const Value V18376_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_321 = {S40, 0, &V18376_321_57295, &P18376_320};	/*Property = Progress*/
const Value V18376_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_322 = {S41, 0, &V18376_322_57298, &P18376_321};	/*Property = Prompts*/
const Value V18376_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_323 = {S42, 0, &V18376_323_57301, &P18376_322};	/*Property = Selector*/
const Value V18376_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_324 = {S43, 0, &V18376_324_57307, &P18376_323};	/*Property = Svg*/
const Value V18376_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_327 = {S44, 0, &V18376_327_57316, &P18376_324};	/*Property = TouchEvents*/
const Value V18376_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_328 = {S45, 0, &V18376_328_57319, &P18376_327};	/*Property = Track*/
const Value V18376_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_329 = {S46, 0, &V18376_329_57325, &P18376_328};	/*Property = Video*/
const Value V18376_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_330 = {S47, 0, &V18376_330_57328, &P18376_329};	/*Property = Viewport*/
const Value V18376_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_332 = {S48, 0, &V18376_332_57340, &P18376_330};	/*Property = WebWorkers*/
const Value V18376_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18376_333 = {S49, 0, &V18376_333_57343, &P18376_332};	/*Property = Xhr2*/
const Profile P18376 = {"18376", &P18376_333};	/*Profile ID - 18376*/
const Value V18424_1_1 = {S2, NULL};	/*Value = True*/
const Property P18424_1 = {S50, 0, &V18424_1_1, NULL};	/*Property = IsMobile*/
const Value V18424_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18424_5 = {S52, 0, &V18424_5_1277, &P18424_1};	/*Property = ScreenPixelsHeight*/
const Value V18424_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18424_4 = {S54, 0, &V18424_4_1279, &P18424_5};	/*Property = ScreenPixelsWidth*/
const Profile P18424 = {"18424", &P18424_4};	/*Profile ID - 18424*/
const Value V18677_1_1 = {S2, NULL};	/*Value = True*/
const Property P18677_1 = {S50, 0, &V18677_1_1, NULL};	/*Property = IsMobile*/
const Value V18677_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18677_5 = {S52, 0, &V18677_5_9981, &P18677_1};	/*Property = ScreenPixelsHeight*/
const Value V18677_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18677_4 = {S54, 0, &V18677_4_39098, &P18677_5};	/*Property = ScreenPixelsWidth*/
const Profile P18677 = {"18677", &P18677_4};	/*Profile ID - 18677*/
const Value V24431_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_279 = {S1, 0, &V24431_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24431_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_282 = {S3, 0, &V24431_282_57155, &P24431_279};	/*Property = BlobBuilder*/
const Value V24431_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_350 = {S4, 0, &V24431_350_57446, &P24431_282};	/*Property = Canvas*/
const Value V24431_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_283 = {S5, 0, &V24431_283_57161, &P24431_350};	/*Property = CssBackground*/
const Value V24431_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_284 = {S6, 0, &V24431_284_57164, &P24431_283};	/*Property = CssBorderImage*/
const Value V24431_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_285 = {S7, 0, &V24431_285_57167, &P24431_284};	/*Property = CssCanvas*/
const Value V24431_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_286 = {S8, 0, &V24431_286_57170, &P24431_285};	/*Property = CssColor*/
const Value V24431_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_237 = {S9, 0, &V24431_237_55786, &P24431_286};	/*Property = CssColumn*/
const Value V24431_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_287 = {S10, 0, &V24431_287_57173, &P24431_237};	/*Property = CssFlexbox*/
const Value V24431_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_288 = {S11, 0, &V24431_288_57176, &P24431_287};	/*Property = CssFont*/
const Value V24431_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_289 = {S12, 0, &V24431_289_86899, &P24431_288};	/*Property = CssImages*/
const Value V24431_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_291 = {S13, 0, &V24431_291_57182, &P24431_289};	/*Property = CssMediaQueries*/
const Value V24431_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_292 = {S14, 0, &V24431_292_57185, &P24431_291};	/*Property = CssMinMax*/
const Value V24431_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_293 = {S15, 0, &V24431_293_57188, &P24431_292};	/*Property = CssOverflow*/
const Value V24431_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_294 = {S16, 0, &V24431_294_57191, &P24431_293};	/*Property = CssPosition*/
const Value V24431_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_295 = {S17, 0, &V24431_295_57194, &P24431_294};	/*Property = CssText*/
const Value V24431_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_297 = {S18, 0, &V24431_297_57197, &P24431_295};	/*Property = CssTransforms*/
const Value V24431_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_241 = {S19, 0, &V24431_241_55801, &P24431_297};	/*Property = CssTransitions*/
const Value V24431_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_298 = {S20, 0, &V24431_298_57208, &P24431_241};	/*Property = CssUI*/
const Value V24431_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_300 = {S21, 0, &V24431_300_57217, &P24431_298};	/*Property = DataSet*/
const Value V24431_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_301 = {S22, 0, &V24431_301_57220, &P24431_300};	/*Property = DataUrl*/
const Value V24431_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_302 = {S23, 0, &V24431_302_57223, &P24431_301};	/*Property = DeviceOrientation*/
const Value V24431_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_304 = {S24, 0, &V24431_304_57229, &P24431_302};	/*Property = FileReader*/
const Value V24431_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_305 = {S25, 0, &V24431_305_57232, &P24431_304};	/*Property = FileSaver*/
const Value V24431_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_306 = {S26, 0, &V24431_306_57238, &P24431_305};	/*Property = FileWriter*/
const Value V24431_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_307 = {S27, 0, &V24431_307_57241, &P24431_306};	/*Property = FormData*/
const Value V24431_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_309 = {S28, 0, &V24431_309_57247, &P24431_307};	/*Property = Fullscreen*/
const Value V24431_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_310 = {S29, 0, &V24431_310_57250, &P24431_309};	/*Property = GeoLocation*/
const Value V24431_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_312 = {S30, 0, &V24431_312_57256, &P24431_310};	/*Property = History*/
const Value V24431_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_314 = {S31, 0, &V24431_314_57262, &P24431_312};	/*Property = Html5*/
const Value V24431_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_313 = {S32, 0, &V24431_313_57259, &P24431_314};	/*Property = Html-Media-Capture*/
const Value V24431_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_315 = {S33, 0, &V24431_315_57265, &P24431_313};	/*Property = Iframe*/
const Value V24431_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_316 = {S34, 0, &V24431_316_57268, &P24431_315};	/*Property = IndexedDB*/
const Value V24431_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_317 = {S35, 0, &V24431_317_57271, &P24431_316};	/*Property = Json*/
const Value V24431_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24431_15 = {S37, 0, &V24431_15_28, &P24431_317};	/*Property = LayoutEngine*/
const Value V24431_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_344 = {S38, 0, &V24431_344_57274, &P24431_15};	/*Property = Masking*/
const Value V24431_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_320 = {S39, 0, &V24431_320_57292, &P24431_344};	/*Property = PostMessage*/
const Value V24431_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_321 = {S40, 0, &V24431_321_57295, &P24431_320};	/*Property = Progress*/
const Value V24431_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_322 = {S41, 0, &V24431_322_57298, &P24431_321};	/*Property = Prompts*/
const Value V24431_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_323 = {S42, 0, &V24431_323_57301, &P24431_322};	/*Property = Selector*/
const Value V24431_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_324 = {S43, 0, &V24431_324_57307, &P24431_323};	/*Property = Svg*/
const Value V24431_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_327 = {S44, 0, &V24431_327_57316, &P24431_324};	/*Property = TouchEvents*/
const Value V24431_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_328 = {S45, 0, &V24431_328_57319, &P24431_327};	/*Property = Track*/
const Value V24431_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_329 = {S46, 0, &V24431_329_57325, &P24431_328};	/*Property = Video*/
const Value V24431_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_330 = {S47, 0, &V24431_330_57328, &P24431_329};	/*Property = Viewport*/
const Value V24431_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_332 = {S48, 0, &V24431_332_57340, &P24431_330};	/*Property = WebWorkers*/
const Value V24431_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24431_333 = {S49, 0, &V24431_333_57343, &P24431_332};	/*Property = Xhr2*/
const Profile P24431 = {"24431", &P24431_333};	/*Profile ID - 24431*/
const Value V18367_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_279 = {S1, 0, &V18367_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18367_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_282 = {S3, 0, &V18367_282_57155, &P18367_279};	/*Property = BlobBuilder*/
const Value V18367_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_350 = {S4, 0, &V18367_350_57446, &P18367_282};	/*Property = Canvas*/
const Value V18367_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_283 = {S5, 0, &V18367_283_57161, &P18367_350};	/*Property = CssBackground*/
const Value V18367_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_284 = {S6, 0, &V18367_284_57164, &P18367_283};	/*Property = CssBorderImage*/
const Value V18367_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_285 = {S7, 0, &V18367_285_57167, &P18367_284};	/*Property = CssCanvas*/
const Value V18367_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_286 = {S8, 0, &V18367_286_57170, &P18367_285};	/*Property = CssColor*/
const Value V18367_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_237 = {S9, 0, &V18367_237_55786, &P18367_286};	/*Property = CssColumn*/
const Value V18367_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_287 = {S10, 0, &V18367_287_57173, &P18367_237};	/*Property = CssFlexbox*/
const Value V18367_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_288 = {S11, 0, &V18367_288_57176, &P18367_287};	/*Property = CssFont*/
const Value V18367_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_289 = {S12, 0, &V18367_289_86899, &P18367_288};	/*Property = CssImages*/
const Value V18367_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_291 = {S13, 0, &V18367_291_57182, &P18367_289};	/*Property = CssMediaQueries*/
const Value V18367_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_292 = {S14, 0, &V18367_292_57185, &P18367_291};	/*Property = CssMinMax*/
const Value V18367_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_293 = {S15, 0, &V18367_293_57188, &P18367_292};	/*Property = CssOverflow*/
const Value V18367_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_294 = {S16, 0, &V18367_294_57191, &P18367_293};	/*Property = CssPosition*/
const Value V18367_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_295 = {S17, 0, &V18367_295_57194, &P18367_294};	/*Property = CssText*/
const Value V18367_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_297 = {S18, 0, &V18367_297_57197, &P18367_295};	/*Property = CssTransforms*/
const Value V18367_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_241 = {S19, 0, &V18367_241_55801, &P18367_297};	/*Property = CssTransitions*/
const Value V18367_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_298 = {S20, 0, &V18367_298_57208, &P18367_241};	/*Property = CssUI*/
const Value V18367_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_300 = {S21, 0, &V18367_300_57217, &P18367_298};	/*Property = DataSet*/
const Value V18367_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_301 = {S22, 0, &V18367_301_57220, &P18367_300};	/*Property = DataUrl*/
const Value V18367_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_302 = {S23, 0, &V18367_302_57223, &P18367_301};	/*Property = DeviceOrientation*/
const Value V18367_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_304 = {S24, 0, &V18367_304_57229, &P18367_302};	/*Property = FileReader*/
const Value V18367_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_305 = {S25, 0, &V18367_305_57232, &P18367_304};	/*Property = FileSaver*/
const Value V18367_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_306 = {S26, 0, &V18367_306_57238, &P18367_305};	/*Property = FileWriter*/
const Value V18367_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_307 = {S27, 0, &V18367_307_57241, &P18367_306};	/*Property = FormData*/
const Value V18367_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_309 = {S28, 0, &V18367_309_57247, &P18367_307};	/*Property = Fullscreen*/
const Value V18367_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_310 = {S29, 0, &V18367_310_57250, &P18367_309};	/*Property = GeoLocation*/
const Value V18367_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_312 = {S30, 0, &V18367_312_57256, &P18367_310};	/*Property = History*/
const Value V18367_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_314 = {S31, 0, &V18367_314_57262, &P18367_312};	/*Property = Html5*/
const Value V18367_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_313 = {S32, 0, &V18367_313_57259, &P18367_314};	/*Property = Html-Media-Capture*/
const Value V18367_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_315 = {S33, 0, &V18367_315_57265, &P18367_313};	/*Property = Iframe*/
const Value V18367_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_316 = {S34, 0, &V18367_316_57268, &P18367_315};	/*Property = IndexedDB*/
const Value V18367_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_317 = {S35, 0, &V18367_317_57271, &P18367_316};	/*Property = Json*/
const Value V18367_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18367_15 = {S37, 0, &V18367_15_29, &P18367_317};	/*Property = LayoutEngine*/
const Value V18367_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_344 = {S38, 0, &V18367_344_57274, &P18367_15};	/*Property = Masking*/
const Value V18367_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_320 = {S39, 0, &V18367_320_57292, &P18367_344};	/*Property = PostMessage*/
const Value V18367_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_321 = {S40, 0, &V18367_321_57295, &P18367_320};	/*Property = Progress*/
const Value V18367_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_322 = {S41, 0, &V18367_322_57298, &P18367_321};	/*Property = Prompts*/
const Value V18367_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_323 = {S42, 0, &V18367_323_57301, &P18367_322};	/*Property = Selector*/
const Value V18367_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_324 = {S43, 0, &V18367_324_57307, &P18367_323};	/*Property = Svg*/
const Value V18367_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_327 = {S44, 0, &V18367_327_57316, &P18367_324};	/*Property = TouchEvents*/
const Value V18367_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_328 = {S45, 0, &V18367_328_57319, &P18367_327};	/*Property = Track*/
const Value V18367_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_329 = {S46, 0, &V18367_329_57325, &P18367_328};	/*Property = Video*/
const Value V18367_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_330 = {S47, 0, &V18367_330_57328, &P18367_329};	/*Property = Viewport*/
const Value V18367_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_332 = {S48, 0, &V18367_332_57340, &P18367_330};	/*Property = WebWorkers*/
const Value V18367_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18367_333 = {S49, 0, &V18367_333_57343, &P18367_332};	/*Property = Xhr2*/
const Profile P18367 = {"18367", &P18367_333};	/*Profile ID - 18367*/
const Value V20962_1_1 = {S2, NULL};	/*Value = True*/
const Property P20962_1 = {S50, 0, &V20962_1_1, NULL};	/*Property = IsMobile*/
const Value V20962_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20962_5 = {S52, 0, &V20962_5_1227, &P20962_1};	/*Property = ScreenPixelsHeight*/
const Value V20962_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20962_4 = {S54, 0, &V20962_4_1229, &P20962_5};	/*Property = ScreenPixelsWidth*/
const Profile P20962 = {"20962", &P20962_4};	/*Profile ID - 20962*/
const Value V22571_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_279 = {S1, 0, &V22571_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22571_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_282 = {S3, 0, &V22571_282_57155, &P22571_279};	/*Property = BlobBuilder*/
const Value V22571_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_350 = {S4, 0, &V22571_350_57446, &P22571_282};	/*Property = Canvas*/
const Value V22571_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_283 = {S5, 0, &V22571_283_57161, &P22571_350};	/*Property = CssBackground*/
const Value V22571_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_284 = {S6, 0, &V22571_284_57164, &P22571_283};	/*Property = CssBorderImage*/
const Value V22571_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_285 = {S7, 0, &V22571_285_57167, &P22571_284};	/*Property = CssCanvas*/
const Value V22571_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_286 = {S8, 0, &V22571_286_57170, &P22571_285};	/*Property = CssColor*/
const Value V22571_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_237 = {S9, 0, &V22571_237_55786, &P22571_286};	/*Property = CssColumn*/
const Value V22571_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_287 = {S10, 0, &V22571_287_57173, &P22571_237};	/*Property = CssFlexbox*/
const Value V22571_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_288 = {S11, 0, &V22571_288_57176, &P22571_287};	/*Property = CssFont*/
const Value V22571_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_289 = {S12, 0, &V22571_289_86899, &P22571_288};	/*Property = CssImages*/
const Value V22571_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_291 = {S13, 0, &V22571_291_57182, &P22571_289};	/*Property = CssMediaQueries*/
const Value V22571_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_292 = {S14, 0, &V22571_292_57185, &P22571_291};	/*Property = CssMinMax*/
const Value V22571_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_293 = {S15, 0, &V22571_293_57188, &P22571_292};	/*Property = CssOverflow*/
const Value V22571_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_294 = {S16, 0, &V22571_294_57191, &P22571_293};	/*Property = CssPosition*/
const Value V22571_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_295 = {S17, 0, &V22571_295_57194, &P22571_294};	/*Property = CssText*/
const Value V22571_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_297 = {S18, 0, &V22571_297_57197, &P22571_295};	/*Property = CssTransforms*/
const Value V22571_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_241 = {S19, 0, &V22571_241_55801, &P22571_297};	/*Property = CssTransitions*/
const Value V22571_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_298 = {S20, 0, &V22571_298_57208, &P22571_241};	/*Property = CssUI*/
const Value V22571_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_300 = {S21, 0, &V22571_300_57217, &P22571_298};	/*Property = DataSet*/
const Value V22571_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_301 = {S22, 0, &V22571_301_57220, &P22571_300};	/*Property = DataUrl*/
const Value V22571_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_302 = {S23, 0, &V22571_302_57223, &P22571_301};	/*Property = DeviceOrientation*/
const Value V22571_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_304 = {S24, 0, &V22571_304_57229, &P22571_302};	/*Property = FileReader*/
const Value V22571_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_305 = {S25, 0, &V22571_305_57232, &P22571_304};	/*Property = FileSaver*/
const Value V22571_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_306 = {S26, 0, &V22571_306_57238, &P22571_305};	/*Property = FileWriter*/
const Value V22571_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_307 = {S27, 0, &V22571_307_57241, &P22571_306};	/*Property = FormData*/
const Value V22571_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_309 = {S28, 0, &V22571_309_57247, &P22571_307};	/*Property = Fullscreen*/
const Value V22571_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_310 = {S29, 0, &V22571_310_57250, &P22571_309};	/*Property = GeoLocation*/
const Value V22571_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_312 = {S30, 0, &V22571_312_57256, &P22571_310};	/*Property = History*/
const Value V22571_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_314 = {S31, 0, &V22571_314_57262, &P22571_312};	/*Property = Html5*/
const Value V22571_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_313 = {S32, 0, &V22571_313_57259, &P22571_314};	/*Property = Html-Media-Capture*/
const Value V22571_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_315 = {S33, 0, &V22571_315_57265, &P22571_313};	/*Property = Iframe*/
const Value V22571_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_316 = {S34, 0, &V22571_316_57268, &P22571_315};	/*Property = IndexedDB*/
const Value V22571_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_317 = {S35, 0, &V22571_317_57271, &P22571_316};	/*Property = Json*/
const Value V22571_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P22571_15 = {S37, 0, &V22571_15_54167, &P22571_317};	/*Property = LayoutEngine*/
const Value V22571_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_344 = {S38, 0, &V22571_344_57274, &P22571_15};	/*Property = Masking*/
const Value V22571_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_320 = {S39, 0, &V22571_320_57292, &P22571_344};	/*Property = PostMessage*/
const Value V22571_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_321 = {S40, 0, &V22571_321_57295, &P22571_320};	/*Property = Progress*/
const Value V22571_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_322 = {S41, 0, &V22571_322_57298, &P22571_321};	/*Property = Prompts*/
const Value V22571_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_323 = {S42, 0, &V22571_323_57301, &P22571_322};	/*Property = Selector*/
const Value V22571_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_324 = {S43, 0, &V22571_324_57307, &P22571_323};	/*Property = Svg*/
const Value V22571_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_327 = {S44, 0, &V22571_327_57316, &P22571_324};	/*Property = TouchEvents*/
const Value V22571_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_328 = {S45, 0, &V22571_328_57319, &P22571_327};	/*Property = Track*/
const Value V22571_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_329 = {S46, 0, &V22571_329_57325, &P22571_328};	/*Property = Video*/
const Value V22571_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_330 = {S47, 0, &V22571_330_57328, &P22571_329};	/*Property = Viewport*/
const Value V22571_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_332 = {S48, 0, &V22571_332_57340, &P22571_330};	/*Property = WebWorkers*/
const Value V22571_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22571_333 = {S49, 0, &V22571_333_57343, &P22571_332};	/*Property = Xhr2*/
const Profile P22571 = {"22571", &P22571_333};	/*Profile ID - 22571*/
const Profile P22196 = {"22196", NULL};	/*Profile ID - 22196*/
const Profile P18344 = {"18344", NULL};	/*Profile ID - 18344*/
const Value V22457_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22457_279 = {S1, 0, &V22457_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22457_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22457_282 = {S3, 0, &V22457_282_57153, &P22457_279};	/*Property = BlobBuilder*/
const Value V22457_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22457_350 = {S4, 0, &V22457_350_57444, &P22457_282};	/*Property = Canvas*/
const Value V22457_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22457_283 = {S5, 0, &V22457_283_57159, &P22457_350};	/*Property = CssBackground*/
const Value V22457_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22457_284 = {S6, 0, &V22457_284_57163, &P22457_283};	/*Property = CssBorderImage*/
const Value V22457_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22457_285 = {S7, 0, &V22457_285_57166, &P22457_284};	/*Property = CssCanvas*/
const Value V22457_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22457_286 = {S8, 0, &V22457_286_57168, &P22457_285};	/*Property = CssColor*/
const Value V22457_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22457_237 = {S9, 0, &V22457_237_55784, &P22457_286};	/*Property = CssColumn*/
const Value V22457_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22457_287 = {S10, 0, &V22457_287_57172, &P22457_237};	/*Property = CssFlexbox*/
const Value V22457_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22457_288 = {S11, 0, &V22457_288_57175, &P22457_287};	/*Property = CssFont*/
const Value V22457_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22457_289 = {S12, 0, &V22457_289_57759, &P22457_288};	/*Property = CssImages*/
const Value V22457_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22457_291 = {S13, 0, &V22457_291_57180, &P22457_289};	/*Property = CssMediaQueries*/
const Value V22457_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22457_292 = {S14, 0, &V22457_292_57183, &P22457_291};	/*Property = CssMinMax*/
const Value V22457_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22457_293 = {S15, 0, &V22457_293_57187, &P22457_292};	/*Property = CssOverflow*/
const Value V22457_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22457_294 = {S16, 0, &V22457_294_57189, &P22457_293};	/*Property = CssPosition*/
const Value V22457_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22457_295 = {S17, 0, &V22457_295_57192, &P22457_294};	/*Property = CssText*/
const Value V22457_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22457_297 = {S18, 0, &V22457_297_57195, &P22457_295};	/*Property = CssTransforms*/
const Value V22457_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22457_241 = {S19, 0, &V22457_241_55799, &P22457_297};	/*Property = CssTransitions*/
const Value V22457_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22457_298 = {S20, 0, &V22457_298_57206, &P22457_241};	/*Property = CssUI*/
const Value V22457_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22457_300 = {S21, 0, &V22457_300_57216, &P22457_298};	/*Property = DataSet*/
const Value V22457_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22457_301 = {S22, 0, &V22457_301_57218, &P22457_300};	/*Property = DataUrl*/
const Value V22457_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22457_302 = {S23, 0, &V22457_302_57222, &P22457_301};	/*Property = DeviceOrientation*/
const Value V22457_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22457_304 = {S24, 0, &V22457_304_57227, &P22457_302};	/*Property = FileReader*/
const Value V22457_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22457_305 = {S25, 0, &V22457_305_57231, &P22457_304};	/*Property = FileSaver*/
const Value V22457_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22457_306 = {S26, 0, &V22457_306_57237, &P22457_305};	/*Property = FileWriter*/
const Value V22457_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22457_307 = {S27, 0, &V22457_307_57239, &P22457_306};	/*Property = FormData*/
const Value V22457_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22457_309 = {S28, 0, &V22457_309_57246, &P22457_307};	/*Property = Fullscreen*/
const Value V22457_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22457_310 = {S29, 0, &V22457_310_57248, &P22457_309};	/*Property = GeoLocation*/
const Value V22457_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22457_312 = {S30, 0, &V22457_312_57254, &P22457_310};	/*Property = History*/
const Value V22457_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22457_314 = {S31, 0, &V22457_314_57261, &P22457_312};	/*Property = Html5*/
const Value V22457_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22457_313 = {S32, 0, &V22457_313_57258, &P22457_314};	/*Property = Html-Media-Capture*/
const Value V22457_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22457_315 = {S33, 0, &V22457_315_57264, &P22457_313};	/*Property = Iframe*/
const Value V22457_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22457_316 = {S34, 0, &V22457_316_57266, &P22457_315};	/*Property = IndexedDB*/
const Value V22457_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22457_317 = {S35, 0, &V22457_317_57269, &P22457_316};	/*Property = Json*/
const Value V22457_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22457_15 = {S37, 0, &V22457_15_28, &P22457_317};	/*Property = LayoutEngine*/
const Value V22457_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22457_344 = {S38, 0, &V22457_344_57272, &P22457_15};	/*Property = Masking*/
const Value V22457_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22457_320 = {S39, 0, &V22457_320_57290, &P22457_344};	/*Property = PostMessage*/
const Value V22457_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22457_321 = {S40, 0, &V22457_321_57293, &P22457_320};	/*Property = Progress*/
const Value V22457_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22457_322 = {S41, 0, &V22457_322_57296, &P22457_321};	/*Property = Prompts*/
const Value V22457_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22457_323 = {S42, 0, &V22457_323_57299, &P22457_322};	/*Property = Selector*/
const Value V22457_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22457_324 = {S43, 0, &V22457_324_57306, &P22457_323};	/*Property = Svg*/
const Value V22457_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22457_327 = {S44, 0, &V22457_327_57315, &P22457_324};	/*Property = TouchEvents*/
const Value V22457_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22457_328 = {S45, 0, &V22457_328_57318, &P22457_327};	/*Property = Track*/
const Value V22457_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22457_329 = {S46, 0, &V22457_329_57323, &P22457_328};	/*Property = Video*/
const Value V22457_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22457_330 = {S47, 0, &V22457_330_57326, &P22457_329};	/*Property = Viewport*/
const Value V22457_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22457_332 = {S48, 0, &V22457_332_57338, &P22457_330};	/*Property = WebWorkers*/
const Value V22457_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22457_333 = {S49, 0, &V22457_333_57342, &P22457_332};	/*Property = Xhr2*/
const Profile P22457 = {"22457", &P22457_333};	/*Profile ID - 22457*/
const Value V20783_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_279 = {S1, 0, &V20783_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20783_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_282 = {S3, 0, &V20783_282_57155, &P20783_279};	/*Property = BlobBuilder*/
const Value V20783_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_350 = {S4, 0, &V20783_350_57446, &P20783_282};	/*Property = Canvas*/
const Value V20783_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_283 = {S5, 0, &V20783_283_57161, &P20783_350};	/*Property = CssBackground*/
const Value V20783_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_284 = {S6, 0, &V20783_284_57164, &P20783_283};	/*Property = CssBorderImage*/
const Value V20783_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_285 = {S7, 0, &V20783_285_57167, &P20783_284};	/*Property = CssCanvas*/
const Value V20783_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_286 = {S8, 0, &V20783_286_57170, &P20783_285};	/*Property = CssColor*/
const Value V20783_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_237 = {S9, 0, &V20783_237_55786, &P20783_286};	/*Property = CssColumn*/
const Value V20783_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_287 = {S10, 0, &V20783_287_57173, &P20783_237};	/*Property = CssFlexbox*/
const Value V20783_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_288 = {S11, 0, &V20783_288_57176, &P20783_287};	/*Property = CssFont*/
const Value V20783_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_289 = {S12, 0, &V20783_289_86899, &P20783_288};	/*Property = CssImages*/
const Value V20783_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_291 = {S13, 0, &V20783_291_57182, &P20783_289};	/*Property = CssMediaQueries*/
const Value V20783_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_292 = {S14, 0, &V20783_292_57185, &P20783_291};	/*Property = CssMinMax*/
const Value V20783_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_293 = {S15, 0, &V20783_293_57188, &P20783_292};	/*Property = CssOverflow*/
const Value V20783_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_294 = {S16, 0, &V20783_294_57191, &P20783_293};	/*Property = CssPosition*/
const Value V20783_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_295 = {S17, 0, &V20783_295_57194, &P20783_294};	/*Property = CssText*/
const Value V20783_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_297 = {S18, 0, &V20783_297_57197, &P20783_295};	/*Property = CssTransforms*/
const Value V20783_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_241 = {S19, 0, &V20783_241_55801, &P20783_297};	/*Property = CssTransitions*/
const Value V20783_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_298 = {S20, 0, &V20783_298_57208, &P20783_241};	/*Property = CssUI*/
const Value V20783_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_300 = {S21, 0, &V20783_300_57217, &P20783_298};	/*Property = DataSet*/
const Value V20783_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_301 = {S22, 0, &V20783_301_57220, &P20783_300};	/*Property = DataUrl*/
const Value V20783_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_302 = {S23, 0, &V20783_302_57223, &P20783_301};	/*Property = DeviceOrientation*/
const Value V20783_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_304 = {S24, 0, &V20783_304_57229, &P20783_302};	/*Property = FileReader*/
const Value V20783_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_305 = {S25, 0, &V20783_305_57232, &P20783_304};	/*Property = FileSaver*/
const Value V20783_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_306 = {S26, 0, &V20783_306_57238, &P20783_305};	/*Property = FileWriter*/
const Value V20783_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_307 = {S27, 0, &V20783_307_57241, &P20783_306};	/*Property = FormData*/
const Value V20783_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_309 = {S28, 0, &V20783_309_57247, &P20783_307};	/*Property = Fullscreen*/
const Value V20783_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_310 = {S29, 0, &V20783_310_57250, &P20783_309};	/*Property = GeoLocation*/
const Value V20783_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_312 = {S30, 0, &V20783_312_57256, &P20783_310};	/*Property = History*/
const Value V20783_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_314 = {S31, 0, &V20783_314_57262, &P20783_312};	/*Property = Html5*/
const Value V20783_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_313 = {S32, 0, &V20783_313_57259, &P20783_314};	/*Property = Html-Media-Capture*/
const Value V20783_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_315 = {S33, 0, &V20783_315_57265, &P20783_313};	/*Property = Iframe*/
const Value V20783_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_316 = {S34, 0, &V20783_316_57268, &P20783_315};	/*Property = IndexedDB*/
const Value V20783_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_317 = {S35, 0, &V20783_317_57271, &P20783_316};	/*Property = Json*/
const Value V20783_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P20783_15 = {S37, 0, &V20783_15_54167, &P20783_317};	/*Property = LayoutEngine*/
const Value V20783_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_344 = {S38, 0, &V20783_344_57274, &P20783_15};	/*Property = Masking*/
const Value V20783_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_320 = {S39, 0, &V20783_320_57292, &P20783_344};	/*Property = PostMessage*/
const Value V20783_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_321 = {S40, 0, &V20783_321_57295, &P20783_320};	/*Property = Progress*/
const Value V20783_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_322 = {S41, 0, &V20783_322_57298, &P20783_321};	/*Property = Prompts*/
const Value V20783_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_323 = {S42, 0, &V20783_323_57301, &P20783_322};	/*Property = Selector*/
const Value V20783_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_324 = {S43, 0, &V20783_324_57307, &P20783_323};	/*Property = Svg*/
const Value V20783_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_327 = {S44, 0, &V20783_327_57316, &P20783_324};	/*Property = TouchEvents*/
const Value V20783_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_328 = {S45, 0, &V20783_328_57319, &P20783_327};	/*Property = Track*/
const Value V20783_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_329 = {S46, 0, &V20783_329_57325, &P20783_328};	/*Property = Video*/
const Value V20783_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_330 = {S47, 0, &V20783_330_57328, &P20783_329};	/*Property = Viewport*/
const Value V20783_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_332 = {S48, 0, &V20783_332_57340, &P20783_330};	/*Property = WebWorkers*/
const Value V20783_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20783_333 = {S49, 0, &V20783_333_57343, &P20783_332};	/*Property = Xhr2*/
const Profile P20783 = {"20783", &P20783_333};	/*Profile ID - 20783*/
const Value V22340_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22340_279 = {S1, 0, &V22340_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22340_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22340_282 = {S3, 0, &V22340_282_57153, &P22340_279};	/*Property = BlobBuilder*/
const Value V22340_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22340_350 = {S4, 0, &V22340_350_57444, &P22340_282};	/*Property = Canvas*/
const Value V22340_283_57160 = {S0, NULL};	/*Value = False*/
const Property P22340_283 = {S5, 0, &V22340_283_57160, &P22340_350};	/*Property = CssBackground*/
const Value V22340_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22340_284 = {S6, 0, &V22340_284_57163, &P22340_283};	/*Property = CssBorderImage*/
const Value V22340_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22340_285 = {S7, 0, &V22340_285_57166, &P22340_284};	/*Property = CssCanvas*/
const Value V22340_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22340_286 = {S8, 0, &V22340_286_57168, &P22340_285};	/*Property = CssColor*/
const Value V22340_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22340_237 = {S9, 0, &V22340_237_55784, &P22340_286};	/*Property = CssColumn*/
const Value V22340_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22340_287 = {S10, 0, &V22340_287_57172, &P22340_237};	/*Property = CssFlexbox*/
const Value V22340_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22340_288 = {S11, 0, &V22340_288_57175, &P22340_287};	/*Property = CssFont*/
const Value V22340_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22340_289 = {S12, 0, &V22340_289_86899, &P22340_288};	/*Property = CssImages*/
const Value V22340_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22340_291 = {S13, 0, &V22340_291_57181, &P22340_289};	/*Property = CssMediaQueries*/
const Value V22340_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22340_292 = {S14, 0, &V22340_292_57183, &P22340_291};	/*Property = CssMinMax*/
const Value V22340_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22340_293 = {S15, 0, &V22340_293_57187, &P22340_292};	/*Property = CssOverflow*/
const Value V22340_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22340_294 = {S16, 0, &V22340_294_57189, &P22340_293};	/*Property = CssPosition*/
const Value V22340_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22340_295 = {S17, 0, &V22340_295_57193, &P22340_294};	/*Property = CssText*/
const Value V22340_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22340_297 = {S18, 0, &V22340_297_57195, &P22340_295};	/*Property = CssTransforms*/
const Value V22340_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22340_241 = {S19, 0, &V22340_241_55799, &P22340_297};	/*Property = CssTransitions*/
const Value V22340_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22340_298 = {S20, 0, &V22340_298_57206, &P22340_241};	/*Property = CssUI*/
const Value V22340_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22340_300 = {S21, 0, &V22340_300_57215, &P22340_298};	/*Property = DataSet*/
const Value V22340_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22340_301 = {S22, 0, &V22340_301_57218, &P22340_300};	/*Property = DataUrl*/
const Value V22340_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22340_302 = {S23, 0, &V22340_302_57221, &P22340_301};	/*Property = DeviceOrientation*/
const Value V22340_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22340_304 = {S24, 0, &V22340_304_57227, &P22340_302};	/*Property = FileReader*/
const Value V22340_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22340_305 = {S25, 0, &V22340_305_57231, &P22340_304};	/*Property = FileSaver*/
const Value V22340_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22340_306 = {S26, 0, &V22340_306_57237, &P22340_305};	/*Property = FileWriter*/
const Value V22340_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22340_307 = {S27, 0, &V22340_307_57239, &P22340_306};	/*Property = FormData*/
const Value V22340_309_57245 = {S2, NULL};	/*Value = True*/
const Property P22340_309 = {S28, 0, &V22340_309_57245, &P22340_307};	/*Property = Fullscreen*/
const Value V22340_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22340_310 = {S29, 0, &V22340_310_57248, &P22340_309};	/*Property = GeoLocation*/
const Value V22340_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22340_312 = {S30, 0, &V22340_312_57254, &P22340_310};	/*Property = History*/
const Value V22340_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22340_314 = {S31, 0, &V22340_314_57261, &P22340_312};	/*Property = Html5*/
const Value V22340_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22340_313 = {S32, 0, &V22340_313_57258, &P22340_314};	/*Property = Html-Media-Capture*/
const Value V22340_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22340_315 = {S33, 0, &V22340_315_57264, &P22340_313};	/*Property = Iframe*/
const Value V22340_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22340_316 = {S34, 0, &V22340_316_57266, &P22340_315};	/*Property = IndexedDB*/
const Value V22340_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22340_317 = {S35, 0, &V22340_317_57269, &P22340_316};	/*Property = Json*/
const Value V22340_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22340_15 = {S37, 0, &V22340_15_29, &P22340_317};	/*Property = LayoutEngine*/
const Value V22340_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22340_344 = {S38, 0, &V22340_344_57272, &P22340_15};	/*Property = Masking*/
const Value V22340_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22340_320 = {S39, 0, &V22340_320_57290, &P22340_344};	/*Property = PostMessage*/
const Value V22340_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22340_321 = {S40, 0, &V22340_321_57293, &P22340_320};	/*Property = Progress*/
const Value V22340_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22340_322 = {S41, 0, &V22340_322_57296, &P22340_321};	/*Property = Prompts*/
const Value V22340_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22340_323 = {S42, 0, &V22340_323_57299, &P22340_322};	/*Property = Selector*/
const Value V22340_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22340_324 = {S43, 0, &V22340_324_57305, &P22340_323};	/*Property = Svg*/
const Value V22340_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22340_327 = {S44, 0, &V22340_327_57315, &P22340_324};	/*Property = TouchEvents*/
const Value V22340_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22340_328 = {S45, 0, &V22340_328_57318, &P22340_327};	/*Property = Track*/
const Value V22340_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22340_329 = {S46, 0, &V22340_329_57323, &P22340_328};	/*Property = Video*/
const Value V22340_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22340_330 = {S47, 0, &V22340_330_57326, &P22340_329};	/*Property = Viewport*/
const Value V22340_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22340_332 = {S48, 0, &V22340_332_57339, &P22340_330};	/*Property = WebWorkers*/
const Value V22340_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22340_333 = {S49, 0, &V22340_333_57341, &P22340_332};	/*Property = Xhr2*/
const Profile P22340 = {"22340", &P22340_333};	/*Profile ID - 22340*/
const Profile P22220 = {"22220", NULL};	/*Profile ID - 22220*/
const Value V18241_1_1 = {S2, NULL};	/*Value = True*/
const Property P18241_1 = {S50, 0, &V18241_1_1, NULL};	/*Property = IsMobile*/
const Value V18241_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18241_5 = {S52, 0, &V18241_5_1227, &P18241_1};	/*Property = ScreenPixelsHeight*/
const Value V18241_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18241_4 = {S54, 0, &V18241_4_1229, &P18241_5};	/*Property = ScreenPixelsWidth*/
const Profile P18241 = {"18241", &P18241_4};	/*Profile ID - 18241*/
const Value V21760_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_279 = {S1, 0, &V21760_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21760_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_282 = {S3, 0, &V21760_282_57155, &P21760_279};	/*Property = BlobBuilder*/
const Value V21760_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_350 = {S4, 0, &V21760_350_57446, &P21760_282};	/*Property = Canvas*/
const Value V21760_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_283 = {S5, 0, &V21760_283_57161, &P21760_350};	/*Property = CssBackground*/
const Value V21760_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_284 = {S6, 0, &V21760_284_57164, &P21760_283};	/*Property = CssBorderImage*/
const Value V21760_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_285 = {S7, 0, &V21760_285_57167, &P21760_284};	/*Property = CssCanvas*/
const Value V21760_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_286 = {S8, 0, &V21760_286_57170, &P21760_285};	/*Property = CssColor*/
const Value V21760_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_237 = {S9, 0, &V21760_237_55786, &P21760_286};	/*Property = CssColumn*/
const Value V21760_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_287 = {S10, 0, &V21760_287_57173, &P21760_237};	/*Property = CssFlexbox*/
const Value V21760_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_288 = {S11, 0, &V21760_288_57176, &P21760_287};	/*Property = CssFont*/
const Value V21760_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_289 = {S12, 0, &V21760_289_86899, &P21760_288};	/*Property = CssImages*/
const Value V21760_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_291 = {S13, 0, &V21760_291_57182, &P21760_289};	/*Property = CssMediaQueries*/
const Value V21760_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_292 = {S14, 0, &V21760_292_57185, &P21760_291};	/*Property = CssMinMax*/
const Value V21760_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_293 = {S15, 0, &V21760_293_57188, &P21760_292};	/*Property = CssOverflow*/
const Value V21760_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_294 = {S16, 0, &V21760_294_57191, &P21760_293};	/*Property = CssPosition*/
const Value V21760_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_295 = {S17, 0, &V21760_295_57194, &P21760_294};	/*Property = CssText*/
const Value V21760_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_297 = {S18, 0, &V21760_297_57197, &P21760_295};	/*Property = CssTransforms*/
const Value V21760_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_241 = {S19, 0, &V21760_241_55801, &P21760_297};	/*Property = CssTransitions*/
const Value V21760_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_298 = {S20, 0, &V21760_298_57208, &P21760_241};	/*Property = CssUI*/
const Value V21760_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_300 = {S21, 0, &V21760_300_57217, &P21760_298};	/*Property = DataSet*/
const Value V21760_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_301 = {S22, 0, &V21760_301_57220, &P21760_300};	/*Property = DataUrl*/
const Value V21760_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_302 = {S23, 0, &V21760_302_57223, &P21760_301};	/*Property = DeviceOrientation*/
const Value V21760_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_304 = {S24, 0, &V21760_304_57229, &P21760_302};	/*Property = FileReader*/
const Value V21760_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_305 = {S25, 0, &V21760_305_57232, &P21760_304};	/*Property = FileSaver*/
const Value V21760_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_306 = {S26, 0, &V21760_306_57238, &P21760_305};	/*Property = FileWriter*/
const Value V21760_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_307 = {S27, 0, &V21760_307_57241, &P21760_306};	/*Property = FormData*/
const Value V21760_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_309 = {S28, 0, &V21760_309_57247, &P21760_307};	/*Property = Fullscreen*/
const Value V21760_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_310 = {S29, 0, &V21760_310_57250, &P21760_309};	/*Property = GeoLocation*/
const Value V21760_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_312 = {S30, 0, &V21760_312_57256, &P21760_310};	/*Property = History*/
const Value V21760_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_314 = {S31, 0, &V21760_314_57262, &P21760_312};	/*Property = Html5*/
const Value V21760_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_313 = {S32, 0, &V21760_313_57259, &P21760_314};	/*Property = Html-Media-Capture*/
const Value V21760_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_315 = {S33, 0, &V21760_315_57265, &P21760_313};	/*Property = Iframe*/
const Value V21760_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_316 = {S34, 0, &V21760_316_57268, &P21760_315};	/*Property = IndexedDB*/
const Value V21760_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_317 = {S35, 0, &V21760_317_57271, &P21760_316};	/*Property = Json*/
const Value V21760_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21760_15 = {S37, 0, &V21760_15_54167, &P21760_317};	/*Property = LayoutEngine*/
const Value V21760_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_344 = {S38, 0, &V21760_344_57274, &P21760_15};	/*Property = Masking*/
const Value V21760_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_320 = {S39, 0, &V21760_320_57292, &P21760_344};	/*Property = PostMessage*/
const Value V21760_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_321 = {S40, 0, &V21760_321_57295, &P21760_320};	/*Property = Progress*/
const Value V21760_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_322 = {S41, 0, &V21760_322_57298, &P21760_321};	/*Property = Prompts*/
const Value V21760_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_323 = {S42, 0, &V21760_323_57301, &P21760_322};	/*Property = Selector*/
const Value V21760_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_324 = {S43, 0, &V21760_324_57307, &P21760_323};	/*Property = Svg*/
const Value V21760_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_327 = {S44, 0, &V21760_327_57316, &P21760_324};	/*Property = TouchEvents*/
const Value V21760_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_328 = {S45, 0, &V21760_328_57319, &P21760_327};	/*Property = Track*/
const Value V21760_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_329 = {S46, 0, &V21760_329_57325, &P21760_328};	/*Property = Video*/
const Value V21760_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_330 = {S47, 0, &V21760_330_57328, &P21760_329};	/*Property = Viewport*/
const Value V21760_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_332 = {S48, 0, &V21760_332_57340, &P21760_330};	/*Property = WebWorkers*/
const Value V21760_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21760_333 = {S49, 0, &V21760_333_57343, &P21760_332};	/*Property = Xhr2*/
const Profile P21760 = {"21760", &P21760_333};	/*Profile ID - 21760*/
const Value V18384_1_1 = {S2, NULL};	/*Value = True*/
const Property P18384_1 = {S50, 0, &V18384_1_1, NULL};	/*Property = IsMobile*/
const Value V18384_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P18384_5 = {S52, 0, &V18384_5_1269, &P18384_1};	/*Property = ScreenPixelsHeight*/
const Value V18384_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18384_4 = {S54, 0, &V18384_4_1229, &P18384_5};	/*Property = ScreenPixelsWidth*/
const Profile P18384 = {"18384", &P18384_4};	/*Profile ID - 18384*/
const Value V18214_1_1 = {S2, NULL};	/*Value = True*/
const Property P18214_1 = {S50, 0, &V18214_1_1, NULL};	/*Property = IsMobile*/
const Value V18214_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18214_5 = {S52, 0, &V18214_5_1277, &P18214_1};	/*Property = ScreenPixelsHeight*/
const Value V18214_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18214_4 = {S54, 0, &V18214_4_1279, &P18214_5};	/*Property = ScreenPixelsWidth*/
const Profile P18214 = {"18214", &P18214_4};	/*Profile ID - 18214*/
const Value V20966_1_1 = {S2, NULL};	/*Value = True*/
const Property P20966_1 = {S50, 0, &V20966_1_1, NULL};	/*Property = IsMobile*/
const Value V20966_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20966_5 = {S52, 0, &V20966_5_9981, &P20966_1};	/*Property = ScreenPixelsHeight*/
const Value V20966_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20966_4 = {S54, 0, &V20966_4_39098, &P20966_5};	/*Property = ScreenPixelsWidth*/
const Profile P20966 = {"20966", &P20966_4};	/*Profile ID - 20966*/
const Value V18393_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_279 = {S1, 0, &V18393_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18393_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_282 = {S3, 0, &V18393_282_57155, &P18393_279};	/*Property = BlobBuilder*/
const Value V18393_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_350 = {S4, 0, &V18393_350_57446, &P18393_282};	/*Property = Canvas*/
const Value V18393_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_283 = {S5, 0, &V18393_283_57161, &P18393_350};	/*Property = CssBackground*/
const Value V18393_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_284 = {S6, 0, &V18393_284_57164, &P18393_283};	/*Property = CssBorderImage*/
const Value V18393_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_285 = {S7, 0, &V18393_285_57167, &P18393_284};	/*Property = CssCanvas*/
const Value V18393_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_286 = {S8, 0, &V18393_286_57170, &P18393_285};	/*Property = CssColor*/
const Value V18393_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_237 = {S9, 0, &V18393_237_55786, &P18393_286};	/*Property = CssColumn*/
const Value V18393_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_287 = {S10, 0, &V18393_287_57173, &P18393_237};	/*Property = CssFlexbox*/
const Value V18393_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_288 = {S11, 0, &V18393_288_57176, &P18393_287};	/*Property = CssFont*/
const Value V18393_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_289 = {S12, 0, &V18393_289_86899, &P18393_288};	/*Property = CssImages*/
const Value V18393_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_291 = {S13, 0, &V18393_291_57182, &P18393_289};	/*Property = CssMediaQueries*/
const Value V18393_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_292 = {S14, 0, &V18393_292_57185, &P18393_291};	/*Property = CssMinMax*/
const Value V18393_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_293 = {S15, 0, &V18393_293_57188, &P18393_292};	/*Property = CssOverflow*/
const Value V18393_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_294 = {S16, 0, &V18393_294_57191, &P18393_293};	/*Property = CssPosition*/
const Value V18393_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_295 = {S17, 0, &V18393_295_57194, &P18393_294};	/*Property = CssText*/
const Value V18393_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_297 = {S18, 0, &V18393_297_57197, &P18393_295};	/*Property = CssTransforms*/
const Value V18393_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_241 = {S19, 0, &V18393_241_55801, &P18393_297};	/*Property = CssTransitions*/
const Value V18393_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_298 = {S20, 0, &V18393_298_57208, &P18393_241};	/*Property = CssUI*/
const Value V18393_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_300 = {S21, 0, &V18393_300_57217, &P18393_298};	/*Property = DataSet*/
const Value V18393_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_301 = {S22, 0, &V18393_301_57220, &P18393_300};	/*Property = DataUrl*/
const Value V18393_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_302 = {S23, 0, &V18393_302_57223, &P18393_301};	/*Property = DeviceOrientation*/
const Value V18393_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_304 = {S24, 0, &V18393_304_57229, &P18393_302};	/*Property = FileReader*/
const Value V18393_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_305 = {S25, 0, &V18393_305_57232, &P18393_304};	/*Property = FileSaver*/
const Value V18393_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_306 = {S26, 0, &V18393_306_57238, &P18393_305};	/*Property = FileWriter*/
const Value V18393_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_307 = {S27, 0, &V18393_307_57241, &P18393_306};	/*Property = FormData*/
const Value V18393_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_309 = {S28, 0, &V18393_309_57247, &P18393_307};	/*Property = Fullscreen*/
const Value V18393_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_310 = {S29, 0, &V18393_310_57250, &P18393_309};	/*Property = GeoLocation*/
const Value V18393_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_312 = {S30, 0, &V18393_312_57256, &P18393_310};	/*Property = History*/
const Value V18393_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_314 = {S31, 0, &V18393_314_57262, &P18393_312};	/*Property = Html5*/
const Value V18393_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_313 = {S32, 0, &V18393_313_57259, &P18393_314};	/*Property = Html-Media-Capture*/
const Value V18393_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_315 = {S33, 0, &V18393_315_57265, &P18393_313};	/*Property = Iframe*/
const Value V18393_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_316 = {S34, 0, &V18393_316_57268, &P18393_315};	/*Property = IndexedDB*/
const Value V18393_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_317 = {S35, 0, &V18393_317_57271, &P18393_316};	/*Property = Json*/
const Value V18393_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18393_15 = {S37, 0, &V18393_15_28, &P18393_317};	/*Property = LayoutEngine*/
const Value V18393_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_344 = {S38, 0, &V18393_344_57274, &P18393_15};	/*Property = Masking*/
const Value V18393_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_320 = {S39, 0, &V18393_320_57292, &P18393_344};	/*Property = PostMessage*/
const Value V18393_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_321 = {S40, 0, &V18393_321_57295, &P18393_320};	/*Property = Progress*/
const Value V18393_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_322 = {S41, 0, &V18393_322_57298, &P18393_321};	/*Property = Prompts*/
const Value V18393_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_323 = {S42, 0, &V18393_323_57301, &P18393_322};	/*Property = Selector*/
const Value V18393_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_324 = {S43, 0, &V18393_324_57307, &P18393_323};	/*Property = Svg*/
const Value V18393_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_327 = {S44, 0, &V18393_327_57316, &P18393_324};	/*Property = TouchEvents*/
const Value V18393_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_328 = {S45, 0, &V18393_328_57319, &P18393_327};	/*Property = Track*/
const Value V18393_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_329 = {S46, 0, &V18393_329_57325, &P18393_328};	/*Property = Video*/
const Value V18393_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_330 = {S47, 0, &V18393_330_57328, &P18393_329};	/*Property = Viewport*/
const Value V18393_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_332 = {S48, 0, &V18393_332_57340, &P18393_330};	/*Property = WebWorkers*/
const Value V18393_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18393_333 = {S49, 0, &V18393_333_57343, &P18393_332};	/*Property = Xhr2*/
const Profile P18393 = {"18393", &P18393_333};	/*Profile ID - 18393*/
const Value V18364_1_1 = {S2, NULL};	/*Value = True*/
const Property P18364_1 = {S50, 0, &V18364_1_1, NULL};	/*Property = IsMobile*/
const Value V18364_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P18364_5 = {S52, 0, &V18364_5_1275, &P18364_1};	/*Property = ScreenPixelsHeight*/
const Value V18364_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P18364_4 = {S54, 0, &V18364_4_1278, &P18364_5};	/*Property = ScreenPixelsWidth*/
const Profile P18364 = {"18364", &P18364_4};	/*Profile ID - 18364*/
const Value V19850_1_1 = {S2, NULL};	/*Value = True*/
const Property P19850_1 = {S50, 0, &V19850_1_1, NULL};	/*Property = IsMobile*/
const Value V19850_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19850_5 = {S52, 0, &V19850_5_9981, &P19850_1};	/*Property = ScreenPixelsHeight*/
const Value V19850_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P19850_4 = {S54, 0, &V19850_4_39098, &P19850_5};	/*Property = ScreenPixelsWidth*/
const Profile P19850 = {"19850", &P19850_4};	/*Profile ID - 19850*/
const Value V22427_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22427_279 = {S1, 0, &V22427_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22427_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22427_282 = {S3, 0, &V22427_282_57153, &P22427_279};	/*Property = BlobBuilder*/
const Value V22427_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22427_350 = {S4, 0, &V22427_350_57444, &P22427_282};	/*Property = Canvas*/
const Value V22427_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22427_283 = {S5, 0, &V22427_283_57159, &P22427_350};	/*Property = CssBackground*/
const Value V22427_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22427_284 = {S6, 0, &V22427_284_57163, &P22427_283};	/*Property = CssBorderImage*/
const Value V22427_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22427_285 = {S7, 0, &V22427_285_57166, &P22427_284};	/*Property = CssCanvas*/
const Value V22427_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22427_286 = {S8, 0, &V22427_286_57168, &P22427_285};	/*Property = CssColor*/
const Value V22427_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22427_237 = {S9, 0, &V22427_237_55784, &P22427_286};	/*Property = CssColumn*/
const Value V22427_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22427_287 = {S10, 0, &V22427_287_57172, &P22427_237};	/*Property = CssFlexbox*/
const Value V22427_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22427_288 = {S11, 0, &V22427_288_57175, &P22427_287};	/*Property = CssFont*/
const Value V22427_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22427_289 = {S12, 0, &V22427_289_57759, &P22427_288};	/*Property = CssImages*/
const Value V22427_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22427_291 = {S13, 0, &V22427_291_57180, &P22427_289};	/*Property = CssMediaQueries*/
const Value V22427_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22427_292 = {S14, 0, &V22427_292_57183, &P22427_291};	/*Property = CssMinMax*/
const Value V22427_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22427_293 = {S15, 0, &V22427_293_57187, &P22427_292};	/*Property = CssOverflow*/
const Value V22427_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22427_294 = {S16, 0, &V22427_294_57189, &P22427_293};	/*Property = CssPosition*/
const Value V22427_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22427_295 = {S17, 0, &V22427_295_57192, &P22427_294};	/*Property = CssText*/
const Value V22427_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22427_297 = {S18, 0, &V22427_297_57195, &P22427_295};	/*Property = CssTransforms*/
const Value V22427_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22427_241 = {S19, 0, &V22427_241_55799, &P22427_297};	/*Property = CssTransitions*/
const Value V22427_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22427_298 = {S20, 0, &V22427_298_57206, &P22427_241};	/*Property = CssUI*/
const Value V22427_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22427_300 = {S21, 0, &V22427_300_57216, &P22427_298};	/*Property = DataSet*/
const Value V22427_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22427_301 = {S22, 0, &V22427_301_57218, &P22427_300};	/*Property = DataUrl*/
const Value V22427_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22427_302 = {S23, 0, &V22427_302_57222, &P22427_301};	/*Property = DeviceOrientation*/
const Value V22427_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22427_304 = {S24, 0, &V22427_304_57227, &P22427_302};	/*Property = FileReader*/
const Value V22427_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22427_305 = {S25, 0, &V22427_305_57231, &P22427_304};	/*Property = FileSaver*/
const Value V22427_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22427_306 = {S26, 0, &V22427_306_57237, &P22427_305};	/*Property = FileWriter*/
const Value V22427_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22427_307 = {S27, 0, &V22427_307_57239, &P22427_306};	/*Property = FormData*/
const Value V22427_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22427_309 = {S28, 0, &V22427_309_57246, &P22427_307};	/*Property = Fullscreen*/
const Value V22427_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22427_310 = {S29, 0, &V22427_310_57248, &P22427_309};	/*Property = GeoLocation*/
const Value V22427_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22427_312 = {S30, 0, &V22427_312_57254, &P22427_310};	/*Property = History*/
const Value V22427_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22427_314 = {S31, 0, &V22427_314_57261, &P22427_312};	/*Property = Html5*/
const Value V22427_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22427_313 = {S32, 0, &V22427_313_57258, &P22427_314};	/*Property = Html-Media-Capture*/
const Value V22427_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22427_315 = {S33, 0, &V22427_315_57264, &P22427_313};	/*Property = Iframe*/
const Value V22427_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22427_316 = {S34, 0, &V22427_316_57266, &P22427_315};	/*Property = IndexedDB*/
const Value V22427_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22427_317 = {S35, 0, &V22427_317_57269, &P22427_316};	/*Property = Json*/
const Value V22427_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22427_15 = {S37, 0, &V22427_15_28, &P22427_317};	/*Property = LayoutEngine*/
const Value V22427_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22427_344 = {S38, 0, &V22427_344_57272, &P22427_15};	/*Property = Masking*/
const Value V22427_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22427_320 = {S39, 0, &V22427_320_57290, &P22427_344};	/*Property = PostMessage*/
const Value V22427_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22427_321 = {S40, 0, &V22427_321_57293, &P22427_320};	/*Property = Progress*/
const Value V22427_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22427_322 = {S41, 0, &V22427_322_57296, &P22427_321};	/*Property = Prompts*/
const Value V22427_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22427_323 = {S42, 0, &V22427_323_57299, &P22427_322};	/*Property = Selector*/
const Value V22427_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22427_324 = {S43, 0, &V22427_324_57306, &P22427_323};	/*Property = Svg*/
const Value V22427_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22427_327 = {S44, 0, &V22427_327_57315, &P22427_324};	/*Property = TouchEvents*/
const Value V22427_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22427_328 = {S45, 0, &V22427_328_57318, &P22427_327};	/*Property = Track*/
const Value V22427_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22427_329 = {S46, 0, &V22427_329_57323, &P22427_328};	/*Property = Video*/
const Value V22427_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22427_330 = {S47, 0, &V22427_330_57326, &P22427_329};	/*Property = Viewport*/
const Value V22427_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22427_332 = {S48, 0, &V22427_332_57338, &P22427_330};	/*Property = WebWorkers*/
const Value V22427_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22427_333 = {S49, 0, &V22427_333_57342, &P22427_332};	/*Property = Xhr2*/
const Profile P22427 = {"22427", &P22427_333};	/*Profile ID - 22427*/
const Value V19881_1_1 = {S2, NULL};	/*Value = True*/
const Property P19881_1 = {S50, 0, &V19881_1_1, NULL};	/*Property = IsMobile*/
const Value V19881_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P19881_5 = {S52, 0, &V19881_5_1222, &P19881_1};	/*Property = ScreenPixelsHeight*/
const Value V19881_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P19881_4 = {S54, 0, &V19881_4_1224, &P19881_5};	/*Property = ScreenPixelsWidth*/
const Profile P19881 = {"19881", &P19881_4};	/*Profile ID - 19881*/
const Value V18538_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_279 = {S1, 0, &V18538_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18538_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_282 = {S3, 0, &V18538_282_57155, &P18538_279};	/*Property = BlobBuilder*/
const Value V18538_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_350 = {S4, 0, &V18538_350_57446, &P18538_282};	/*Property = Canvas*/
const Value V18538_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_283 = {S5, 0, &V18538_283_57161, &P18538_350};	/*Property = CssBackground*/
const Value V18538_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_284 = {S6, 0, &V18538_284_57164, &P18538_283};	/*Property = CssBorderImage*/
const Value V18538_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_285 = {S7, 0, &V18538_285_57167, &P18538_284};	/*Property = CssCanvas*/
const Value V18538_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_286 = {S8, 0, &V18538_286_57170, &P18538_285};	/*Property = CssColor*/
const Value V18538_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_237 = {S9, 0, &V18538_237_55786, &P18538_286};	/*Property = CssColumn*/
const Value V18538_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_287 = {S10, 0, &V18538_287_57173, &P18538_237};	/*Property = CssFlexbox*/
const Value V18538_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_288 = {S11, 0, &V18538_288_57176, &P18538_287};	/*Property = CssFont*/
const Value V18538_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_289 = {S12, 0, &V18538_289_86899, &P18538_288};	/*Property = CssImages*/
const Value V18538_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_291 = {S13, 0, &V18538_291_57182, &P18538_289};	/*Property = CssMediaQueries*/
const Value V18538_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_292 = {S14, 0, &V18538_292_57185, &P18538_291};	/*Property = CssMinMax*/
const Value V18538_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_293 = {S15, 0, &V18538_293_57188, &P18538_292};	/*Property = CssOverflow*/
const Value V18538_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_294 = {S16, 0, &V18538_294_57191, &P18538_293};	/*Property = CssPosition*/
const Value V18538_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_295 = {S17, 0, &V18538_295_57194, &P18538_294};	/*Property = CssText*/
const Value V18538_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_297 = {S18, 0, &V18538_297_57197, &P18538_295};	/*Property = CssTransforms*/
const Value V18538_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_241 = {S19, 0, &V18538_241_55801, &P18538_297};	/*Property = CssTransitions*/
const Value V18538_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_298 = {S20, 0, &V18538_298_57208, &P18538_241};	/*Property = CssUI*/
const Value V18538_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_300 = {S21, 0, &V18538_300_57217, &P18538_298};	/*Property = DataSet*/
const Value V18538_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_301 = {S22, 0, &V18538_301_57220, &P18538_300};	/*Property = DataUrl*/
const Value V18538_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_302 = {S23, 0, &V18538_302_57223, &P18538_301};	/*Property = DeviceOrientation*/
const Value V18538_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_304 = {S24, 0, &V18538_304_57229, &P18538_302};	/*Property = FileReader*/
const Value V18538_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_305 = {S25, 0, &V18538_305_57232, &P18538_304};	/*Property = FileSaver*/
const Value V18538_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_306 = {S26, 0, &V18538_306_57238, &P18538_305};	/*Property = FileWriter*/
const Value V18538_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_307 = {S27, 0, &V18538_307_57241, &P18538_306};	/*Property = FormData*/
const Value V18538_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_309 = {S28, 0, &V18538_309_57247, &P18538_307};	/*Property = Fullscreen*/
const Value V18538_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_310 = {S29, 0, &V18538_310_57250, &P18538_309};	/*Property = GeoLocation*/
const Value V18538_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_312 = {S30, 0, &V18538_312_57256, &P18538_310};	/*Property = History*/
const Value V18538_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_314 = {S31, 0, &V18538_314_57262, &P18538_312};	/*Property = Html5*/
const Value V18538_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_313 = {S32, 0, &V18538_313_57259, &P18538_314};	/*Property = Html-Media-Capture*/
const Value V18538_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_315 = {S33, 0, &V18538_315_57265, &P18538_313};	/*Property = Iframe*/
const Value V18538_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_316 = {S34, 0, &V18538_316_57268, &P18538_315};	/*Property = IndexedDB*/
const Value V18538_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_317 = {S35, 0, &V18538_317_57271, &P18538_316};	/*Property = Json*/
const Value V18538_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18538_15 = {S37, 0, &V18538_15_28, &P18538_317};	/*Property = LayoutEngine*/
const Value V18538_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_344 = {S38, 0, &V18538_344_57274, &P18538_15};	/*Property = Masking*/
const Value V18538_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_320 = {S39, 0, &V18538_320_57292, &P18538_344};	/*Property = PostMessage*/
const Value V18538_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_321 = {S40, 0, &V18538_321_57295, &P18538_320};	/*Property = Progress*/
const Value V18538_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_322 = {S41, 0, &V18538_322_57298, &P18538_321};	/*Property = Prompts*/
const Value V18538_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_323 = {S42, 0, &V18538_323_57301, &P18538_322};	/*Property = Selector*/
const Value V18538_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_324 = {S43, 0, &V18538_324_57307, &P18538_323};	/*Property = Svg*/
const Value V18538_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_327 = {S44, 0, &V18538_327_57316, &P18538_324};	/*Property = TouchEvents*/
const Value V18538_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_328 = {S45, 0, &V18538_328_57319, &P18538_327};	/*Property = Track*/
const Value V18538_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_329 = {S46, 0, &V18538_329_57325, &P18538_328};	/*Property = Video*/
const Value V18538_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_330 = {S47, 0, &V18538_330_57328, &P18538_329};	/*Property = Viewport*/
const Value V18538_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_332 = {S48, 0, &V18538_332_57340, &P18538_330};	/*Property = WebWorkers*/
const Value V18538_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18538_333 = {S49, 0, &V18538_333_57343, &P18538_332};	/*Property = Xhr2*/
const Profile P18538 = {"18538", &P18538_333};	/*Profile ID - 18538*/
const Value V21584_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_279 = {S1, 0, &V21584_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21584_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_282 = {S3, 0, &V21584_282_57155, &P21584_279};	/*Property = BlobBuilder*/
const Value V21584_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_350 = {S4, 0, &V21584_350_57446, &P21584_282};	/*Property = Canvas*/
const Value V21584_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_283 = {S5, 0, &V21584_283_57161, &P21584_350};	/*Property = CssBackground*/
const Value V21584_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_284 = {S6, 0, &V21584_284_57164, &P21584_283};	/*Property = CssBorderImage*/
const Value V21584_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_285 = {S7, 0, &V21584_285_57167, &P21584_284};	/*Property = CssCanvas*/
const Value V21584_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_286 = {S8, 0, &V21584_286_57170, &P21584_285};	/*Property = CssColor*/
const Value V21584_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_237 = {S9, 0, &V21584_237_55786, &P21584_286};	/*Property = CssColumn*/
const Value V21584_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_287 = {S10, 0, &V21584_287_57173, &P21584_237};	/*Property = CssFlexbox*/
const Value V21584_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_288 = {S11, 0, &V21584_288_57176, &P21584_287};	/*Property = CssFont*/
const Value V21584_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_289 = {S12, 0, &V21584_289_86899, &P21584_288};	/*Property = CssImages*/
const Value V21584_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_291 = {S13, 0, &V21584_291_57182, &P21584_289};	/*Property = CssMediaQueries*/
const Value V21584_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_292 = {S14, 0, &V21584_292_57185, &P21584_291};	/*Property = CssMinMax*/
const Value V21584_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_293 = {S15, 0, &V21584_293_57188, &P21584_292};	/*Property = CssOverflow*/
const Value V21584_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_294 = {S16, 0, &V21584_294_57191, &P21584_293};	/*Property = CssPosition*/
const Value V21584_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_295 = {S17, 0, &V21584_295_57194, &P21584_294};	/*Property = CssText*/
const Value V21584_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_297 = {S18, 0, &V21584_297_57197, &P21584_295};	/*Property = CssTransforms*/
const Value V21584_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_241 = {S19, 0, &V21584_241_55801, &P21584_297};	/*Property = CssTransitions*/
const Value V21584_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_298 = {S20, 0, &V21584_298_57208, &P21584_241};	/*Property = CssUI*/
const Value V21584_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_300 = {S21, 0, &V21584_300_57217, &P21584_298};	/*Property = DataSet*/
const Value V21584_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_301 = {S22, 0, &V21584_301_57220, &P21584_300};	/*Property = DataUrl*/
const Value V21584_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_302 = {S23, 0, &V21584_302_57223, &P21584_301};	/*Property = DeviceOrientation*/
const Value V21584_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_304 = {S24, 0, &V21584_304_57229, &P21584_302};	/*Property = FileReader*/
const Value V21584_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_305 = {S25, 0, &V21584_305_57232, &P21584_304};	/*Property = FileSaver*/
const Value V21584_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_306 = {S26, 0, &V21584_306_57238, &P21584_305};	/*Property = FileWriter*/
const Value V21584_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_307 = {S27, 0, &V21584_307_57241, &P21584_306};	/*Property = FormData*/
const Value V21584_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_309 = {S28, 0, &V21584_309_57247, &P21584_307};	/*Property = Fullscreen*/
const Value V21584_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_310 = {S29, 0, &V21584_310_57250, &P21584_309};	/*Property = GeoLocation*/
const Value V21584_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_312 = {S30, 0, &V21584_312_57256, &P21584_310};	/*Property = History*/
const Value V21584_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_314 = {S31, 0, &V21584_314_57262, &P21584_312};	/*Property = Html5*/
const Value V21584_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_313 = {S32, 0, &V21584_313_57259, &P21584_314};	/*Property = Html-Media-Capture*/
const Value V21584_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_315 = {S33, 0, &V21584_315_57265, &P21584_313};	/*Property = Iframe*/
const Value V21584_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_316 = {S34, 0, &V21584_316_57268, &P21584_315};	/*Property = IndexedDB*/
const Value V21584_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_317 = {S35, 0, &V21584_317_57271, &P21584_316};	/*Property = Json*/
const Value V21584_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_15 = {S37, 0, &V21584_15_72, &P21584_317};	/*Property = LayoutEngine*/
const Value V21584_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_344 = {S38, 0, &V21584_344_57274, &P21584_15};	/*Property = Masking*/
const Value V21584_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_320 = {S39, 0, &V21584_320_57292, &P21584_344};	/*Property = PostMessage*/
const Value V21584_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_321 = {S40, 0, &V21584_321_57295, &P21584_320};	/*Property = Progress*/
const Value V21584_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_322 = {S41, 0, &V21584_322_57298, &P21584_321};	/*Property = Prompts*/
const Value V21584_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_323 = {S42, 0, &V21584_323_57301, &P21584_322};	/*Property = Selector*/
const Value V21584_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_324 = {S43, 0, &V21584_324_57307, &P21584_323};	/*Property = Svg*/
const Value V21584_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_327 = {S44, 0, &V21584_327_57316, &P21584_324};	/*Property = TouchEvents*/
const Value V21584_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_328 = {S45, 0, &V21584_328_57319, &P21584_327};	/*Property = Track*/
const Value V21584_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_329 = {S46, 0, &V21584_329_57325, &P21584_328};	/*Property = Video*/
const Value V21584_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_330 = {S47, 0, &V21584_330_57328, &P21584_329};	/*Property = Viewport*/
const Value V21584_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_332 = {S48, 0, &V21584_332_57340, &P21584_330};	/*Property = WebWorkers*/
const Value V21584_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21584_333 = {S49, 0, &V21584_333_57343, &P21584_332};	/*Property = Xhr2*/
const Profile P21584 = {"21584", &P21584_333};	/*Profile ID - 21584*/
const Value V21115_1_1 = {S2, NULL};	/*Value = True*/
const Property P21115_1 = {S50, 0, &V21115_1_1, NULL};	/*Property = IsMobile*/
const Value V21115_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P21115_5 = {S52, 0, &V21115_5_1380, &P21115_1};	/*Property = ScreenPixelsHeight*/
const Value V21115_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21115_4 = {S54, 0, &V21115_4_1279, &P21115_5};	/*Property = ScreenPixelsWidth*/
const Profile P21115 = {"21115", &P21115_4};	/*Profile ID - 21115*/
const Value V22037_1_1 = {S2, NULL};	/*Value = True*/
const Property P22037_1 = {S50, 0, &V22037_1_1, NULL};	/*Property = IsMobile*/
const Value V22037_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22037_5 = {S52, 0, &V22037_5_1458, &P22037_1};	/*Property = ScreenPixelsHeight*/
const Value V22037_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22037_4 = {S54, 0, &V22037_4_1460, &P22037_5};	/*Property = ScreenPixelsWidth*/
const Profile P22037 = {"22037", &P22037_4};	/*Profile ID - 22037*/
const Value V18233_1_1 = {S2, NULL};	/*Value = True*/
const Property P18233_1 = {S50, 0, &V18233_1_1, NULL};	/*Property = IsMobile*/
const Value V18233_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18233_5 = {S52, 0, &V18233_5_1227, &P18233_1};	/*Property = ScreenPixelsHeight*/
const Value V18233_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18233_4 = {S54, 0, &V18233_4_1229, &P18233_5};	/*Property = ScreenPixelsWidth*/
const Profile P18233 = {"18233", &P18233_4};	/*Profile ID - 18233*/
const Value V21308_1_1 = {S2, NULL};	/*Value = True*/
const Property P21308_1 = {S50, 0, &V21308_1_1, NULL};	/*Property = IsMobile*/
const Value V21308_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21308_5 = {S52, 0, &V21308_5_1227, &P21308_1};	/*Property = ScreenPixelsHeight*/
const Value V21308_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21308_4 = {S54, 0, &V21308_4_1229, &P21308_5};	/*Property = ScreenPixelsWidth*/
const Profile P21308 = {"21308", &P21308_4};	/*Profile ID - 21308*/
const Value V21066_1_1 = {S2, NULL};	/*Value = True*/
const Property P21066_1 = {S50, 0, &V21066_1_1, NULL};	/*Property = IsMobile*/
const Value V21066_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21066_5 = {S52, 0, &V21066_5_1227, &P21066_1};	/*Property = ScreenPixelsHeight*/
const Value V21066_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21066_4 = {S54, 0, &V21066_4_1229, &P21066_5};	/*Property = ScreenPixelsWidth*/
const Profile P21066 = {"21066", &P21066_4};	/*Profile ID - 21066*/
const Value V18394_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18394_279 = {S1, 0, &V18394_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18394_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18394_282 = {S3, 0, &V18394_282_57154, &P18394_279};	/*Property = BlobBuilder*/
const Value V18394_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18394_350 = {S4, 0, &V18394_350_57444, &P18394_282};	/*Property = Canvas*/
const Value V18394_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18394_283 = {S5, 0, &V18394_283_57159, &P18394_350};	/*Property = CssBackground*/
const Value V18394_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18394_284 = {S6, 0, &V18394_284_57163, &P18394_283};	/*Property = CssBorderImage*/
const Value V18394_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18394_285 = {S7, 0, &V18394_285_57166, &P18394_284};	/*Property = CssCanvas*/
const Value V18394_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18394_286 = {S8, 0, &V18394_286_57168, &P18394_285};	/*Property = CssColor*/
const Value V18394_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18394_237 = {S9, 0, &V18394_237_55786, &P18394_286};	/*Property = CssColumn*/
const Value V18394_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18394_287 = {S10, 0, &V18394_287_57172, &P18394_237};	/*Property = CssFlexbox*/
const Value V18394_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18394_288 = {S11, 0, &V18394_288_57175, &P18394_287};	/*Property = CssFont*/
const Value V18394_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18394_289 = {S12, 0, &V18394_289_57759, &P18394_288};	/*Property = CssImages*/
const Value V18394_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18394_291 = {S13, 0, &V18394_291_57181, &P18394_289};	/*Property = CssMediaQueries*/
const Value V18394_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18394_292 = {S14, 0, &V18394_292_57183, &P18394_291};	/*Property = CssMinMax*/
const Value V18394_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18394_293 = {S15, 0, &V18394_293_57187, &P18394_292};	/*Property = CssOverflow*/
const Value V18394_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18394_294 = {S16, 0, &V18394_294_57190, &P18394_293};	/*Property = CssPosition*/
const Value V18394_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18394_295 = {S17, 0, &V18394_295_57193, &P18394_294};	/*Property = CssText*/
const Value V18394_297_57196 = {S0, NULL};	/*Value = False*/
const Property P18394_297 = {S18, 0, &V18394_297_57196, &P18394_295};	/*Property = CssTransforms*/
const Value V18394_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18394_241 = {S19, 0, &V18394_241_55799, &P18394_297};	/*Property = CssTransitions*/
const Value V18394_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18394_298 = {S20, 0, &V18394_298_57206, &P18394_241};	/*Property = CssUI*/
const Value V18394_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18394_300 = {S21, 0, &V18394_300_57216, &P18394_298};	/*Property = DataSet*/
const Value V18394_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18394_301 = {S22, 0, &V18394_301_57218, &P18394_300};	/*Property = DataUrl*/
const Value V18394_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18394_302 = {S23, 0, &V18394_302_57222, &P18394_301};	/*Property = DeviceOrientation*/
const Value V18394_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18394_304 = {S24, 0, &V18394_304_57228, &P18394_302};	/*Property = FileReader*/
const Value V18394_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18394_305 = {S25, 0, &V18394_305_57231, &P18394_304};	/*Property = FileSaver*/
const Value V18394_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18394_306 = {S26, 0, &V18394_306_57237, &P18394_305};	/*Property = FileWriter*/
const Value V18394_307_57240 = {S0, NULL};	/*Value = False*/
const Property P18394_307 = {S27, 0, &V18394_307_57240, &P18394_306};	/*Property = FormData*/
const Value V18394_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18394_309 = {S28, 0, &V18394_309_57246, &P18394_307};	/*Property = Fullscreen*/
const Value V18394_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18394_310 = {S29, 0, &V18394_310_57248, &P18394_309};	/*Property = GeoLocation*/
const Value V18394_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18394_312 = {S30, 0, &V18394_312_57255, &P18394_310};	/*Property = History*/
const Value V18394_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18394_314 = {S31, 0, &V18394_314_57261, &P18394_312};	/*Property = Html5*/
const Value V18394_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18394_313 = {S32, 0, &V18394_313_57258, &P18394_314};	/*Property = Html-Media-Capture*/
const Value V18394_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18394_315 = {S33, 0, &V18394_315_57264, &P18394_313};	/*Property = Iframe*/
const Value V18394_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18394_316 = {S34, 0, &V18394_316_57267, &P18394_315};	/*Property = IndexedDB*/
const Value V18394_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18394_317 = {S35, 0, &V18394_317_57269, &P18394_316};	/*Property = Json*/
const Value V18394_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18394_15 = {S37, 0, &V18394_15_31, &P18394_317};	/*Property = LayoutEngine*/
const Value V18394_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18394_344 = {S38, 0, &V18394_344_57272, &P18394_15};	/*Property = Masking*/
const Value V18394_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18394_320 = {S39, 0, &V18394_320_57290, &P18394_344};	/*Property = PostMessage*/
const Value V18394_321_57294 = {S0, NULL};	/*Value = False*/
const Property P18394_321 = {S40, 0, &V18394_321_57294, &P18394_320};	/*Property = Progress*/
const Value V18394_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18394_322 = {S41, 0, &V18394_322_57296, &P18394_321};	/*Property = Prompts*/
const Value V18394_323_57300 = {S0, NULL};	/*Value = False*/
const Property P18394_323 = {S42, 0, &V18394_323_57300, &P18394_322};	/*Property = Selector*/
const Value V18394_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18394_324 = {S43, 0, &V18394_324_57306, &P18394_323};	/*Property = Svg*/
const Value V18394_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18394_327 = {S44, 0, &V18394_327_57315, &P18394_324};	/*Property = TouchEvents*/
const Value V18394_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18394_328 = {S45, 0, &V18394_328_57318, &P18394_327};	/*Property = Track*/
const Value V18394_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18394_329 = {S46, 0, &V18394_329_57323, &P18394_328};	/*Property = Video*/
const Value V18394_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18394_330 = {S47, 0, &V18394_330_57326, &P18394_329};	/*Property = Viewport*/
const Value V18394_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18394_332 = {S48, 0, &V18394_332_57339, &P18394_330};	/*Property = WebWorkers*/
const Value V18394_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18394_333 = {S49, 0, &V18394_333_57342, &P18394_332};	/*Property = Xhr2*/
const Profile P18394 = {"18394", &P18394_333};	/*Profile ID - 18394*/
const Value V19786_1_1 = {S2, NULL};	/*Value = True*/
const Property P19786_1 = {S50, 0, &V19786_1_1, NULL};	/*Property = IsMobile*/
const Value V19786_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P19786_5 = {S52, 0, &V19786_5_37, &P19786_1};	/*Property = ScreenPixelsHeight*/
const Value V19786_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P19786_4 = {S54, 0, &V19786_4_36, &P19786_5};	/*Property = ScreenPixelsWidth*/
const Profile P19786 = {"19786", &P19786_4};	/*Profile ID - 19786*/
const Value V18345_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18345_279 = {S1, 0, &V18345_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18345_282_57153 = {S2, NULL};	/*Value = True*/
const Property P18345_282 = {S3, 0, &V18345_282_57153, &P18345_279};	/*Property = BlobBuilder*/
const Value V18345_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18345_350 = {S4, 0, &V18345_350_57444, &P18345_282};	/*Property = Canvas*/
const Value V18345_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18345_283 = {S5, 0, &V18345_283_57159, &P18345_350};	/*Property = CssBackground*/
const Value V18345_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18345_284 = {S6, 0, &V18345_284_57163, &P18345_283};	/*Property = CssBorderImage*/
const Value V18345_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18345_285 = {S7, 0, &V18345_285_57166, &P18345_284};	/*Property = CssCanvas*/
const Value V18345_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18345_286 = {S8, 0, &V18345_286_57168, &P18345_285};	/*Property = CssColor*/
const Value V18345_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18345_237 = {S9, 0, &V18345_237_55786, &P18345_286};	/*Property = CssColumn*/
const Value V18345_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18345_287 = {S10, 0, &V18345_287_57172, &P18345_237};	/*Property = CssFlexbox*/
const Value V18345_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18345_288 = {S11, 0, &V18345_288_57175, &P18345_287};	/*Property = CssFont*/
const Value V18345_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18345_289 = {S12, 0, &V18345_289_57759, &P18345_288};	/*Property = CssImages*/
const Value V18345_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18345_291 = {S13, 0, &V18345_291_57181, &P18345_289};	/*Property = CssMediaQueries*/
const Value V18345_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18345_292 = {S14, 0, &V18345_292_57183, &P18345_291};	/*Property = CssMinMax*/
const Value V18345_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18345_293 = {S15, 0, &V18345_293_57187, &P18345_292};	/*Property = CssOverflow*/
const Value V18345_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18345_294 = {S16, 0, &V18345_294_57189, &P18345_293};	/*Property = CssPosition*/
const Value V18345_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18345_295 = {S17, 0, &V18345_295_57192, &P18345_294};	/*Property = CssText*/
const Value V18345_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18345_297 = {S18, 0, &V18345_297_57195, &P18345_295};	/*Property = CssTransforms*/
const Value V18345_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18345_241 = {S19, 0, &V18345_241_55799, &P18345_297};	/*Property = CssTransitions*/
const Value V18345_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18345_298 = {S20, 0, &V18345_298_57206, &P18345_241};	/*Property = CssUI*/
const Value V18345_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18345_300 = {S21, 0, &V18345_300_57216, &P18345_298};	/*Property = DataSet*/
const Value V18345_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18345_301 = {S22, 0, &V18345_301_57218, &P18345_300};	/*Property = DataUrl*/
const Value V18345_302_57221 = {S2, NULL};	/*Value = True*/
const Property P18345_302 = {S23, 0, &V18345_302_57221, &P18345_301};	/*Property = DeviceOrientation*/
const Value V18345_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18345_304 = {S24, 0, &V18345_304_57227, &P18345_302};	/*Property = FileReader*/
const Value V18345_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18345_305 = {S25, 0, &V18345_305_57231, &P18345_304};	/*Property = FileSaver*/
const Value V18345_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18345_306 = {S26, 0, &V18345_306_57237, &P18345_305};	/*Property = FileWriter*/
const Value V18345_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18345_307 = {S27, 0, &V18345_307_57239, &P18345_306};	/*Property = FormData*/
const Value V18345_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18345_309 = {S28, 0, &V18345_309_57246, &P18345_307};	/*Property = Fullscreen*/
const Value V18345_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18345_310 = {S29, 0, &V18345_310_57248, &P18345_309};	/*Property = GeoLocation*/
const Value V18345_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18345_312 = {S30, 0, &V18345_312_57254, &P18345_310};	/*Property = History*/
const Value V18345_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18345_314 = {S31, 0, &V18345_314_57261, &P18345_312};	/*Property = Html5*/
const Value V18345_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18345_313 = {S32, 0, &V18345_313_57258, &P18345_314};	/*Property = Html-Media-Capture*/
const Value V18345_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18345_315 = {S33, 0, &V18345_315_57264, &P18345_313};	/*Property = Iframe*/
const Value V18345_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18345_316 = {S34, 0, &V18345_316_57266, &P18345_315};	/*Property = IndexedDB*/
const Value V18345_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18345_317 = {S35, 0, &V18345_317_57269, &P18345_316};	/*Property = Json*/
const Value V18345_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18345_15 = {S37, 0, &V18345_15_29, &P18345_317};	/*Property = LayoutEngine*/
const Value V18345_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18345_344 = {S38, 0, &V18345_344_57272, &P18345_15};	/*Property = Masking*/
const Value V18345_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18345_320 = {S39, 0, &V18345_320_57290, &P18345_344};	/*Property = PostMessage*/
const Value V18345_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18345_321 = {S40, 0, &V18345_321_57293, &P18345_320};	/*Property = Progress*/
const Value V18345_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18345_322 = {S41, 0, &V18345_322_57296, &P18345_321};	/*Property = Prompts*/
const Value V18345_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18345_323 = {S42, 0, &V18345_323_57299, &P18345_322};	/*Property = Selector*/
const Value V18345_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18345_324 = {S43, 0, &V18345_324_57306, &P18345_323};	/*Property = Svg*/
const Value V18345_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18345_327 = {S44, 0, &V18345_327_57315, &P18345_324};	/*Property = TouchEvents*/
const Value V18345_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18345_328 = {S45, 0, &V18345_328_57318, &P18345_327};	/*Property = Track*/
const Value V18345_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18345_329 = {S46, 0, &V18345_329_57323, &P18345_328};	/*Property = Video*/
const Value V18345_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18345_330 = {S47, 0, &V18345_330_57326, &P18345_329};	/*Property = Viewport*/
const Value V18345_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18345_332 = {S48, 0, &V18345_332_57339, &P18345_330};	/*Property = WebWorkers*/
const Value V18345_333_57341 = {S2, NULL};	/*Value = True*/
const Property P18345_333 = {S49, 0, &V18345_333_57341, &P18345_332};	/*Property = Xhr2*/
const Profile P18345 = {"18345", &P18345_333};	/*Profile ID - 18345*/
const Value V20435_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20435_279 = {S1, 0, &V20435_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20435_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20435_282 = {S3, 0, &V20435_282_57154, &P20435_279};	/*Property = BlobBuilder*/
const Value V20435_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20435_350 = {S4, 0, &V20435_350_57444, &P20435_282};	/*Property = Canvas*/
const Value V20435_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20435_283 = {S5, 0, &V20435_283_57159, &P20435_350};	/*Property = CssBackground*/
const Value V20435_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20435_284 = {S6, 0, &V20435_284_57163, &P20435_283};	/*Property = CssBorderImage*/
const Value V20435_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20435_285 = {S7, 0, &V20435_285_57166, &P20435_284};	/*Property = CssCanvas*/
const Value V20435_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20435_286 = {S8, 0, &V20435_286_57168, &P20435_285};	/*Property = CssColor*/
const Value V20435_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20435_237 = {S9, 0, &V20435_237_55786, &P20435_286};	/*Property = CssColumn*/
const Value V20435_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20435_287 = {S10, 0, &V20435_287_57172, &P20435_237};	/*Property = CssFlexbox*/
const Value V20435_288_57175 = {S0, NULL};	/*Value = False*/
const Property P20435_288 = {S11, 0, &V20435_288_57175, &P20435_287};	/*Property = CssFont*/
const Value V20435_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20435_289 = {S12, 0, &V20435_289_57759, &P20435_288};	/*Property = CssImages*/
const Value V20435_291_57181 = {S0, NULL};	/*Value = False*/
const Property P20435_291 = {S13, 0, &V20435_291_57181, &P20435_289};	/*Property = CssMediaQueries*/
const Value V20435_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20435_292 = {S14, 0, &V20435_292_57183, &P20435_291};	/*Property = CssMinMax*/
const Value V20435_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20435_293 = {S15, 0, &V20435_293_57187, &P20435_292};	/*Property = CssOverflow*/
const Value V20435_294_57189 = {S2, NULL};	/*Value = True*/
const Property P20435_294 = {S16, 0, &V20435_294_57189, &P20435_293};	/*Property = CssPosition*/
const Value V20435_295_57192 = {S2, NULL};	/*Value = True*/
const Property P20435_295 = {S17, 0, &V20435_295_57192, &P20435_294};	/*Property = CssText*/
const Value V20435_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20435_297 = {S18, 0, &V20435_297_57195, &P20435_295};	/*Property = CssTransforms*/
const Value V20435_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20435_241 = {S19, 0, &V20435_241_55799, &P20435_297};	/*Property = CssTransitions*/
const Value V20435_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20435_298 = {S20, 0, &V20435_298_57206, &P20435_241};	/*Property = CssUI*/
const Value V20435_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20435_300 = {S21, 0, &V20435_300_57216, &P20435_298};	/*Property = DataSet*/
const Value V20435_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20435_301 = {S22, 0, &V20435_301_57218, &P20435_300};	/*Property = DataUrl*/
const Value V20435_302_57221 = {S2, NULL};	/*Value = True*/
const Property P20435_302 = {S23, 0, &V20435_302_57221, &P20435_301};	/*Property = DeviceOrientation*/
const Value V20435_304_57227 = {S2, NULL};	/*Value = True*/
const Property P20435_304 = {S24, 0, &V20435_304_57227, &P20435_302};	/*Property = FileReader*/
const Value V20435_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20435_305 = {S25, 0, &V20435_305_57231, &P20435_304};	/*Property = FileSaver*/
const Value V20435_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20435_306 = {S26, 0, &V20435_306_57237, &P20435_305};	/*Property = FileWriter*/
const Value V20435_307_57239 = {S2, NULL};	/*Value = True*/
const Property P20435_307 = {S27, 0, &V20435_307_57239, &P20435_306};	/*Property = FormData*/
const Value V20435_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20435_309 = {S28, 0, &V20435_309_57246, &P20435_307};	/*Property = Fullscreen*/
const Value V20435_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20435_310 = {S29, 0, &V20435_310_57248, &P20435_309};	/*Property = GeoLocation*/
const Value V20435_312_57254 = {S2, NULL};	/*Value = True*/
const Property P20435_312 = {S30, 0, &V20435_312_57254, &P20435_310};	/*Property = History*/
const Value V20435_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20435_314 = {S31, 0, &V20435_314_57261, &P20435_312};	/*Property = Html5*/
const Value V20435_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20435_313 = {S32, 0, &V20435_313_57258, &P20435_314};	/*Property = Html-Media-Capture*/
const Value V20435_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20435_315 = {S33, 0, &V20435_315_57264, &P20435_313};	/*Property = Iframe*/
const Value V20435_316_57266 = {S2, NULL};	/*Value = True*/
const Property P20435_316 = {S34, 0, &V20435_316_57266, &P20435_315};	/*Property = IndexedDB*/
const Value V20435_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20435_317 = {S35, 0, &V20435_317_57269, &P20435_316};	/*Property = Json*/
const Value V20435_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20435_15 = {S37, 0, &V20435_15_29, &P20435_317};	/*Property = LayoutEngine*/
const Value V20435_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20435_344 = {S38, 0, &V20435_344_57272, &P20435_15};	/*Property = Masking*/
const Value V20435_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20435_320 = {S39, 0, &V20435_320_57290, &P20435_344};	/*Property = PostMessage*/
const Value V20435_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20435_321 = {S40, 0, &V20435_321_57293, &P20435_320};	/*Property = Progress*/
const Value V20435_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20435_322 = {S41, 0, &V20435_322_57296, &P20435_321};	/*Property = Prompts*/
const Value V20435_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20435_323 = {S42, 0, &V20435_323_57299, &P20435_322};	/*Property = Selector*/
const Value V20435_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20435_324 = {S43, 0, &V20435_324_57306, &P20435_323};	/*Property = Svg*/
const Value V20435_327_57314 = {S2, NULL};	/*Value = True*/
const Property P20435_327 = {S44, 0, &V20435_327_57314, &P20435_324};	/*Property = TouchEvents*/
const Value V20435_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20435_328 = {S45, 0, &V20435_328_57318, &P20435_327};	/*Property = Track*/
const Value V20435_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20435_329 = {S46, 0, &V20435_329_57323, &P20435_328};	/*Property = Video*/
const Value V20435_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20435_330 = {S47, 0, &V20435_330_57326, &P20435_329};	/*Property = Viewport*/
const Value V20435_332_57339 = {S0, NULL};	/*Value = False*/
const Property P20435_332 = {S48, 0, &V20435_332_57339, &P20435_330};	/*Property = WebWorkers*/
const Value V20435_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20435_333 = {S49, 0, &V20435_333_57342, &P20435_332};	/*Property = Xhr2*/
const Profile P20435 = {"20435", &P20435_333};	/*Profile ID - 20435*/
const Value V21910_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21910_279 = {S1, 0, &V21910_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21910_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21910_282 = {S3, 0, &V21910_282_57154, &P21910_279};	/*Property = BlobBuilder*/
const Value V21910_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21910_350 = {S4, 0, &V21910_350_57444, &P21910_282};	/*Property = Canvas*/
const Value V21910_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21910_283 = {S5, 0, &V21910_283_57159, &P21910_350};	/*Property = CssBackground*/
const Value V21910_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21910_284 = {S6, 0, &V21910_284_57163, &P21910_283};	/*Property = CssBorderImage*/
const Value V21910_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21910_285 = {S7, 0, &V21910_285_57166, &P21910_284};	/*Property = CssCanvas*/
const Value V21910_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21910_286 = {S8, 0, &V21910_286_57168, &P21910_285};	/*Property = CssColor*/
const Value V21910_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21910_237 = {S9, 0, &V21910_237_55784, &P21910_286};	/*Property = CssColumn*/
const Value V21910_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21910_287 = {S10, 0, &V21910_287_57172, &P21910_237};	/*Property = CssFlexbox*/
const Value V21910_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21910_288 = {S11, 0, &V21910_288_57175, &P21910_287};	/*Property = CssFont*/
const Value V21910_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21910_289 = {S12, 0, &V21910_289_57759, &P21910_288};	/*Property = CssImages*/
const Value V21910_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21910_291 = {S13, 0, &V21910_291_57181, &P21910_289};	/*Property = CssMediaQueries*/
const Value V21910_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21910_292 = {S14, 0, &V21910_292_57183, &P21910_291};	/*Property = CssMinMax*/
const Value V21910_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21910_293 = {S15, 0, &V21910_293_57187, &P21910_292};	/*Property = CssOverflow*/
const Value V21910_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21910_294 = {S16, 0, &V21910_294_57190, &P21910_293};	/*Property = CssPosition*/
const Value V21910_295_57193 = {S0, NULL};	/*Value = False*/
const Property P21910_295 = {S17, 0, &V21910_295_57193, &P21910_294};	/*Property = CssText*/
const Value V21910_297_57196 = {S0, NULL};	/*Value = False*/
const Property P21910_297 = {S18, 0, &V21910_297_57196, &P21910_295};	/*Property = CssTransforms*/
const Value V21910_241_55800 = {S0, NULL};	/*Value = False*/
const Property P21910_241 = {S19, 0, &V21910_241_55800, &P21910_297};	/*Property = CssTransitions*/
const Value V21910_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21910_298 = {S20, 0, &V21910_298_57206, &P21910_241};	/*Property = CssUI*/
const Value V21910_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21910_300 = {S21, 0, &V21910_300_57216, &P21910_298};	/*Property = DataSet*/
const Value V21910_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21910_301 = {S22, 0, &V21910_301_57218, &P21910_300};	/*Property = DataUrl*/
const Value V21910_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21910_302 = {S23, 0, &V21910_302_57222, &P21910_301};	/*Property = DeviceOrientation*/
const Value V21910_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21910_304 = {S24, 0, &V21910_304_57228, &P21910_302};	/*Property = FileReader*/
const Value V21910_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21910_305 = {S25, 0, &V21910_305_57231, &P21910_304};	/*Property = FileSaver*/
const Value V21910_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21910_306 = {S26, 0, &V21910_306_57237, &P21910_305};	/*Property = FileWriter*/
const Value V21910_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21910_307 = {S27, 0, &V21910_307_57240, &P21910_306};	/*Property = FormData*/
const Value V21910_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21910_309 = {S28, 0, &V21910_309_57246, &P21910_307};	/*Property = Fullscreen*/
const Value V21910_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21910_310 = {S29, 0, &V21910_310_57248, &P21910_309};	/*Property = GeoLocation*/
const Value V21910_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21910_312 = {S30, 0, &V21910_312_57255, &P21910_310};	/*Property = History*/
const Value V21910_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21910_314 = {S31, 0, &V21910_314_57261, &P21910_312};	/*Property = Html5*/
const Value V21910_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21910_313 = {S32, 0, &V21910_313_57258, &P21910_314};	/*Property = Html-Media-Capture*/
const Value V21910_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21910_315 = {S33, 0, &V21910_315_57264, &P21910_313};	/*Property = Iframe*/
const Value V21910_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21910_316 = {S34, 0, &V21910_316_57267, &P21910_315};	/*Property = IndexedDB*/
const Value V21910_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21910_317 = {S35, 0, &V21910_317_57269, &P21910_316};	/*Property = Json*/
const Value V21910_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21910_15 = {S37, 0, &V21910_15_28, &P21910_317};	/*Property = LayoutEngine*/
const Value V21910_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21910_344 = {S38, 0, &V21910_344_57272, &P21910_15};	/*Property = Masking*/
const Value V21910_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21910_320 = {S39, 0, &V21910_320_57290, &P21910_344};	/*Property = PostMessage*/
const Value V21910_321_57294 = {S0, NULL};	/*Value = False*/
const Property P21910_321 = {S40, 0, &V21910_321_57294, &P21910_320};	/*Property = Progress*/
const Value V21910_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21910_322 = {S41, 0, &V21910_322_57296, &P21910_321};	/*Property = Prompts*/
const Value V21910_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21910_323 = {S42, 0, &V21910_323_57299, &P21910_322};	/*Property = Selector*/
const Value V21910_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21910_324 = {S43, 0, &V21910_324_57306, &P21910_323};	/*Property = Svg*/
const Value V21910_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21910_327 = {S44, 0, &V21910_327_57315, &P21910_324};	/*Property = TouchEvents*/
const Value V21910_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21910_328 = {S45, 0, &V21910_328_57318, &P21910_327};	/*Property = Track*/
const Value V21910_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21910_329 = {S46, 0, &V21910_329_57323, &P21910_328};	/*Property = Video*/
const Value V21910_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21910_330 = {S47, 0, &V21910_330_57326, &P21910_329};	/*Property = Viewport*/
const Value V21910_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21910_332 = {S48, 0, &V21910_332_57339, &P21910_330};	/*Property = WebWorkers*/
const Value V21910_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21910_333 = {S49, 0, &V21910_333_57342, &P21910_332};	/*Property = Xhr2*/
const Profile P21910 = {"21910", &P21910_333};	/*Profile ID - 21910*/
const Value V21713_1_1 = {S2, NULL};	/*Value = True*/
const Property P21713_1 = {S50, 0, &V21713_1_1, NULL};	/*Property = IsMobile*/
const Value V21713_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21713_5 = {S52, 0, &V21713_5_1227, &P21713_1};	/*Property = ScreenPixelsHeight*/
const Value V21713_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P21713_4 = {S54, 0, &V21713_4_1943, &P21713_5};	/*Property = ScreenPixelsWidth*/
const Profile P21713 = {"21713", &P21713_4};	/*Profile ID - 21713*/
const Value V20929_1_1 = {S2, NULL};	/*Value = True*/
const Property P20929_1 = {S50, 0, &V20929_1_1, NULL};	/*Property = IsMobile*/
const Value V20929_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20929_5 = {S52, 0, &V20929_5_1227, &P20929_1};	/*Property = ScreenPixelsHeight*/
const Value V20929_4_54464 = {S119, NULL};	/*Value = 280*/
const Property P20929_4 = {S54, 0, &V20929_4_54464, &P20929_5};	/*Property = ScreenPixelsWidth*/
const Profile P20929 = {"20929", &P20929_4};	/*Profile ID - 20929*/
const Value V18356_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_279 = {S1, 0, &V18356_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18356_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_282 = {S3, 0, &V18356_282_57155, &P18356_279};	/*Property = BlobBuilder*/
const Value V18356_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_350 = {S4, 0, &V18356_350_57446, &P18356_282};	/*Property = Canvas*/
const Value V18356_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_283 = {S5, 0, &V18356_283_57161, &P18356_350};	/*Property = CssBackground*/
const Value V18356_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_284 = {S6, 0, &V18356_284_57164, &P18356_283};	/*Property = CssBorderImage*/
const Value V18356_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_285 = {S7, 0, &V18356_285_57167, &P18356_284};	/*Property = CssCanvas*/
const Value V18356_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_286 = {S8, 0, &V18356_286_57170, &P18356_285};	/*Property = CssColor*/
const Value V18356_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_237 = {S9, 0, &V18356_237_55786, &P18356_286};	/*Property = CssColumn*/
const Value V18356_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_287 = {S10, 0, &V18356_287_57173, &P18356_237};	/*Property = CssFlexbox*/
const Value V18356_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_288 = {S11, 0, &V18356_288_57176, &P18356_287};	/*Property = CssFont*/
const Value V18356_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_289 = {S12, 0, &V18356_289_86899, &P18356_288};	/*Property = CssImages*/
const Value V18356_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_291 = {S13, 0, &V18356_291_57182, &P18356_289};	/*Property = CssMediaQueries*/
const Value V18356_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_292 = {S14, 0, &V18356_292_57185, &P18356_291};	/*Property = CssMinMax*/
const Value V18356_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_293 = {S15, 0, &V18356_293_57188, &P18356_292};	/*Property = CssOverflow*/
const Value V18356_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_294 = {S16, 0, &V18356_294_57191, &P18356_293};	/*Property = CssPosition*/
const Value V18356_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_295 = {S17, 0, &V18356_295_57194, &P18356_294};	/*Property = CssText*/
const Value V18356_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_297 = {S18, 0, &V18356_297_57197, &P18356_295};	/*Property = CssTransforms*/
const Value V18356_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_241 = {S19, 0, &V18356_241_55801, &P18356_297};	/*Property = CssTransitions*/
const Value V18356_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_298 = {S20, 0, &V18356_298_57208, &P18356_241};	/*Property = CssUI*/
const Value V18356_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_300 = {S21, 0, &V18356_300_57217, &P18356_298};	/*Property = DataSet*/
const Value V18356_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_301 = {S22, 0, &V18356_301_57220, &P18356_300};	/*Property = DataUrl*/
const Value V18356_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_302 = {S23, 0, &V18356_302_57223, &P18356_301};	/*Property = DeviceOrientation*/
const Value V18356_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_304 = {S24, 0, &V18356_304_57229, &P18356_302};	/*Property = FileReader*/
const Value V18356_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_305 = {S25, 0, &V18356_305_57232, &P18356_304};	/*Property = FileSaver*/
const Value V18356_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_306 = {S26, 0, &V18356_306_57238, &P18356_305};	/*Property = FileWriter*/
const Value V18356_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_307 = {S27, 0, &V18356_307_57241, &P18356_306};	/*Property = FormData*/
const Value V18356_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_309 = {S28, 0, &V18356_309_57247, &P18356_307};	/*Property = Fullscreen*/
const Value V18356_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_310 = {S29, 0, &V18356_310_57250, &P18356_309};	/*Property = GeoLocation*/
const Value V18356_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_312 = {S30, 0, &V18356_312_57256, &P18356_310};	/*Property = History*/
const Value V18356_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_314 = {S31, 0, &V18356_314_57262, &P18356_312};	/*Property = Html5*/
const Value V18356_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_313 = {S32, 0, &V18356_313_57259, &P18356_314};	/*Property = Html-Media-Capture*/
const Value V18356_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_315 = {S33, 0, &V18356_315_57265, &P18356_313};	/*Property = Iframe*/
const Value V18356_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_316 = {S34, 0, &V18356_316_57268, &P18356_315};	/*Property = IndexedDB*/
const Value V18356_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_317 = {S35, 0, &V18356_317_57271, &P18356_316};	/*Property = Json*/
const Value V18356_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18356_15 = {S37, 0, &V18356_15_29, &P18356_317};	/*Property = LayoutEngine*/
const Value V18356_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_344 = {S38, 0, &V18356_344_57274, &P18356_15};	/*Property = Masking*/
const Value V18356_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_320 = {S39, 0, &V18356_320_57292, &P18356_344};	/*Property = PostMessage*/
const Value V18356_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_321 = {S40, 0, &V18356_321_57295, &P18356_320};	/*Property = Progress*/
const Value V18356_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_322 = {S41, 0, &V18356_322_57298, &P18356_321};	/*Property = Prompts*/
const Value V18356_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_323 = {S42, 0, &V18356_323_57301, &P18356_322};	/*Property = Selector*/
const Value V18356_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_324 = {S43, 0, &V18356_324_57307, &P18356_323};	/*Property = Svg*/
const Value V18356_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_327 = {S44, 0, &V18356_327_57316, &P18356_324};	/*Property = TouchEvents*/
const Value V18356_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_328 = {S45, 0, &V18356_328_57319, &P18356_327};	/*Property = Track*/
const Value V18356_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_329 = {S46, 0, &V18356_329_57325, &P18356_328};	/*Property = Video*/
const Value V18356_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_330 = {S47, 0, &V18356_330_57328, &P18356_329};	/*Property = Viewport*/
const Value V18356_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_332 = {S48, 0, &V18356_332_57340, &P18356_330};	/*Property = WebWorkers*/
const Value V18356_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18356_333 = {S49, 0, &V18356_333_57343, &P18356_332};	/*Property = Xhr2*/
const Profile P18356 = {"18356", &P18356_333};	/*Profile ID - 18356*/
const Value V24199_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24199_279 = {S1, 0, &V24199_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24199_282_57153 = {S2, NULL};	/*Value = True*/
const Property P24199_282 = {S3, 0, &V24199_282_57153, &P24199_279};	/*Property = BlobBuilder*/
const Value V24199_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24199_350 = {S4, 0, &V24199_350_57444, &P24199_282};	/*Property = Canvas*/
const Value V24199_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24199_283 = {S5, 0, &V24199_283_57159, &P24199_350};	/*Property = CssBackground*/
const Value V24199_284_57163 = {S0, NULL};	/*Value = False*/
const Property P24199_284 = {S6, 0, &V24199_284_57163, &P24199_283};	/*Property = CssBorderImage*/
const Value V24199_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24199_285 = {S7, 0, &V24199_285_57166, &P24199_284};	/*Property = CssCanvas*/
const Value V24199_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24199_286 = {S8, 0, &V24199_286_57168, &P24199_285};	/*Property = CssColor*/
const Value V24199_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24199_237 = {S9, 0, &V24199_237_55786, &P24199_286};	/*Property = CssColumn*/
const Value V24199_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24199_287 = {S10, 0, &V24199_287_57172, &P24199_237};	/*Property = CssFlexbox*/
const Value V24199_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24199_288 = {S11, 0, &V24199_288_57175, &P24199_287};	/*Property = CssFont*/
const Value V24199_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24199_289 = {S12, 0, &V24199_289_57759, &P24199_288};	/*Property = CssImages*/
const Value V24199_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24199_291 = {S13, 0, &V24199_291_57180, &P24199_289};	/*Property = CssMediaQueries*/
const Value V24199_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24199_292 = {S14, 0, &V24199_292_57183, &P24199_291};	/*Property = CssMinMax*/
const Value V24199_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24199_293 = {S15, 0, &V24199_293_57187, &P24199_292};	/*Property = CssOverflow*/
const Value V24199_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24199_294 = {S16, 0, &V24199_294_57189, &P24199_293};	/*Property = CssPosition*/
const Value V24199_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24199_295 = {S17, 0, &V24199_295_57192, &P24199_294};	/*Property = CssText*/
const Value V24199_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24199_297 = {S18, 0, &V24199_297_57195, &P24199_295};	/*Property = CssTransforms*/
const Value V24199_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24199_241 = {S19, 0, &V24199_241_55799, &P24199_297};	/*Property = CssTransitions*/
const Value V24199_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24199_298 = {S20, 0, &V24199_298_57206, &P24199_241};	/*Property = CssUI*/
const Value V24199_300_57216 = {S0, NULL};	/*Value = False*/
const Property P24199_300 = {S21, 0, &V24199_300_57216, &P24199_298};	/*Property = DataSet*/
const Value V24199_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24199_301 = {S22, 0, &V24199_301_57218, &P24199_300};	/*Property = DataUrl*/
const Value V24199_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24199_302 = {S23, 0, &V24199_302_57222, &P24199_301};	/*Property = DeviceOrientation*/
const Value V24199_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24199_304 = {S24, 0, &V24199_304_57227, &P24199_302};	/*Property = FileReader*/
const Value V24199_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24199_305 = {S25, 0, &V24199_305_57231, &P24199_304};	/*Property = FileSaver*/
const Value V24199_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24199_306 = {S26, 0, &V24199_306_57237, &P24199_305};	/*Property = FileWriter*/
const Value V24199_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24199_307 = {S27, 0, &V24199_307_57239, &P24199_306};	/*Property = FormData*/
const Value V24199_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24199_309 = {S28, 0, &V24199_309_57246, &P24199_307};	/*Property = Fullscreen*/
const Value V24199_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24199_310 = {S29, 0, &V24199_310_57248, &P24199_309};	/*Property = GeoLocation*/
const Value V24199_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24199_312 = {S30, 0, &V24199_312_57254, &P24199_310};	/*Property = History*/
const Value V24199_314_57261 = {S0, NULL};	/*Value = False*/
const Property P24199_314 = {S31, 0, &V24199_314_57261, &P24199_312};	/*Property = Html5*/
const Value V24199_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24199_313 = {S32, 0, &V24199_313_57258, &P24199_314};	/*Property = Html-Media-Capture*/
const Value V24199_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24199_315 = {S33, 0, &V24199_315_57264, &P24199_313};	/*Property = Iframe*/
const Value V24199_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24199_316 = {S34, 0, &V24199_316_57266, &P24199_315};	/*Property = IndexedDB*/
const Value V24199_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24199_317 = {S35, 0, &V24199_317_57269, &P24199_316};	/*Property = Json*/
const Value V24199_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24199_15 = {S37, 0, &V24199_15_54167, &P24199_317};	/*Property = LayoutEngine*/
const Value V24199_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24199_344 = {S38, 0, &V24199_344_57272, &P24199_15};	/*Property = Masking*/
const Value V24199_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24199_320 = {S39, 0, &V24199_320_57290, &P24199_344};	/*Property = PostMessage*/
const Value V24199_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24199_321 = {S40, 0, &V24199_321_57293, &P24199_320};	/*Property = Progress*/
const Value V24199_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24199_322 = {S41, 0, &V24199_322_57296, &P24199_321};	/*Property = Prompts*/
const Value V24199_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24199_323 = {S42, 0, &V24199_323_57299, &P24199_322};	/*Property = Selector*/
const Value V24199_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24199_324 = {S43, 0, &V24199_324_57306, &P24199_323};	/*Property = Svg*/
const Value V24199_327_57315 = {S0, NULL};	/*Value = False*/
const Property P24199_327 = {S44, 0, &V24199_327_57315, &P24199_324};	/*Property = TouchEvents*/
const Value V24199_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24199_328 = {S45, 0, &V24199_328_57318, &P24199_327};	/*Property = Track*/
const Value V24199_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24199_329 = {S46, 0, &V24199_329_57323, &P24199_328};	/*Property = Video*/
const Value V24199_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24199_330 = {S47, 0, &V24199_330_57326, &P24199_329};	/*Property = Viewport*/
const Value V24199_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24199_332 = {S48, 0, &V24199_332_57338, &P24199_330};	/*Property = WebWorkers*/
const Value V24199_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24199_333 = {S49, 0, &V24199_333_57342, &P24199_332};	/*Property = Xhr2*/
const Profile P24199 = {"24199", &P24199_333};	/*Profile ID - 24199*/
const Value V18628_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18628_279 = {S1, 0, &V18628_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18628_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18628_282 = {S3, 0, &V18628_282_57154, &P18628_279};	/*Property = BlobBuilder*/
const Value V18628_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18628_350 = {S4, 0, &V18628_350_57444, &P18628_282};	/*Property = Canvas*/
const Value V18628_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18628_283 = {S5, 0, &V18628_283_57159, &P18628_350};	/*Property = CssBackground*/
const Value V18628_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18628_284 = {S6, 0, &V18628_284_57163, &P18628_283};	/*Property = CssBorderImage*/
const Value V18628_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18628_285 = {S7, 0, &V18628_285_57166, &P18628_284};	/*Property = CssCanvas*/
const Value V18628_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18628_286 = {S8, 0, &V18628_286_57168, &P18628_285};	/*Property = CssColor*/
const Value V18628_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18628_237 = {S9, 0, &V18628_237_55786, &P18628_286};	/*Property = CssColumn*/
const Value V18628_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18628_287 = {S10, 0, &V18628_287_57172, &P18628_237};	/*Property = CssFlexbox*/
const Value V18628_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18628_288 = {S11, 0, &V18628_288_57175, &P18628_287};	/*Property = CssFont*/
const Value V18628_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18628_289 = {S12, 0, &V18628_289_57759, &P18628_288};	/*Property = CssImages*/
const Value V18628_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18628_291 = {S13, 0, &V18628_291_57181, &P18628_289};	/*Property = CssMediaQueries*/
const Value V18628_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18628_292 = {S14, 0, &V18628_292_57183, &P18628_291};	/*Property = CssMinMax*/
const Value V18628_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18628_293 = {S15, 0, &V18628_293_57187, &P18628_292};	/*Property = CssOverflow*/
const Value V18628_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18628_294 = {S16, 0, &V18628_294_57190, &P18628_293};	/*Property = CssPosition*/
const Value V18628_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18628_295 = {S17, 0, &V18628_295_57193, &P18628_294};	/*Property = CssText*/
const Value V18628_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18628_297 = {S18, 0, &V18628_297_57195, &P18628_295};	/*Property = CssTransforms*/
const Value V18628_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18628_241 = {S19, 0, &V18628_241_55799, &P18628_297};	/*Property = CssTransitions*/
const Value V18628_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18628_298 = {S20, 0, &V18628_298_57206, &P18628_241};	/*Property = CssUI*/
const Value V18628_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18628_300 = {S21, 0, &V18628_300_57216, &P18628_298};	/*Property = DataSet*/
const Value V18628_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18628_301 = {S22, 0, &V18628_301_57218, &P18628_300};	/*Property = DataUrl*/
const Value V18628_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18628_302 = {S23, 0, &V18628_302_57222, &P18628_301};	/*Property = DeviceOrientation*/
const Value V18628_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18628_304 = {S24, 0, &V18628_304_57228, &P18628_302};	/*Property = FileReader*/
const Value V18628_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18628_305 = {S25, 0, &V18628_305_57231, &P18628_304};	/*Property = FileSaver*/
const Value V18628_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18628_306 = {S26, 0, &V18628_306_57237, &P18628_305};	/*Property = FileWriter*/
const Value V18628_307_57240 = {S0, NULL};	/*Value = False*/
const Property P18628_307 = {S27, 0, &V18628_307_57240, &P18628_306};	/*Property = FormData*/
const Value V18628_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18628_309 = {S28, 0, &V18628_309_57246, &P18628_307};	/*Property = Fullscreen*/
const Value V18628_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18628_310 = {S29, 0, &V18628_310_57248, &P18628_309};	/*Property = GeoLocation*/
const Value V18628_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18628_312 = {S30, 0, &V18628_312_57255, &P18628_310};	/*Property = History*/
const Value V18628_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18628_314 = {S31, 0, &V18628_314_57261, &P18628_312};	/*Property = Html5*/
const Value V18628_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18628_313 = {S32, 0, &V18628_313_57258, &P18628_314};	/*Property = Html-Media-Capture*/
const Value V18628_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18628_315 = {S33, 0, &V18628_315_57264, &P18628_313};	/*Property = Iframe*/
const Value V18628_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18628_316 = {S34, 0, &V18628_316_57267, &P18628_315};	/*Property = IndexedDB*/
const Value V18628_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18628_317 = {S35, 0, &V18628_317_57269, &P18628_316};	/*Property = Json*/
const Value V18628_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18628_15 = {S37, 0, &V18628_15_29, &P18628_317};	/*Property = LayoutEngine*/
const Value V18628_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18628_344 = {S38, 0, &V18628_344_57272, &P18628_15};	/*Property = Masking*/
const Value V18628_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18628_320 = {S39, 0, &V18628_320_57290, &P18628_344};	/*Property = PostMessage*/
const Value V18628_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18628_321 = {S40, 0, &V18628_321_57293, &P18628_320};	/*Property = Progress*/
const Value V18628_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18628_322 = {S41, 0, &V18628_322_57296, &P18628_321};	/*Property = Prompts*/
const Value V18628_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18628_323 = {S42, 0, &V18628_323_57299, &P18628_322};	/*Property = Selector*/
const Value V18628_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18628_324 = {S43, 0, &V18628_324_57306, &P18628_323};	/*Property = Svg*/
const Value V18628_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18628_327 = {S44, 0, &V18628_327_57315, &P18628_324};	/*Property = TouchEvents*/
const Value V18628_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18628_328 = {S45, 0, &V18628_328_57318, &P18628_327};	/*Property = Track*/
const Value V18628_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18628_329 = {S46, 0, &V18628_329_57323, &P18628_328};	/*Property = Video*/
const Value V18628_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18628_330 = {S47, 0, &V18628_330_57326, &P18628_329};	/*Property = Viewport*/
const Value V18628_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18628_332 = {S48, 0, &V18628_332_57339, &P18628_330};	/*Property = WebWorkers*/
const Value V18628_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18628_333 = {S49, 0, &V18628_333_57342, &P18628_332};	/*Property = Xhr2*/
const Profile P18628 = {"18628", &P18628_333};	/*Profile ID - 18628*/
const Value V24670_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_279 = {S1, 0, &V24670_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24670_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_282 = {S3, 0, &V24670_282_57155, &P24670_279};	/*Property = BlobBuilder*/
const Value V24670_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_350 = {S4, 0, &V24670_350_57446, &P24670_282};	/*Property = Canvas*/
const Value V24670_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_283 = {S5, 0, &V24670_283_57161, &P24670_350};	/*Property = CssBackground*/
const Value V24670_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_284 = {S6, 0, &V24670_284_57164, &P24670_283};	/*Property = CssBorderImage*/
const Value V24670_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_285 = {S7, 0, &V24670_285_57167, &P24670_284};	/*Property = CssCanvas*/
const Value V24670_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_286 = {S8, 0, &V24670_286_57170, &P24670_285};	/*Property = CssColor*/
const Value V24670_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_237 = {S9, 0, &V24670_237_55786, &P24670_286};	/*Property = CssColumn*/
const Value V24670_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_287 = {S10, 0, &V24670_287_57173, &P24670_237};	/*Property = CssFlexbox*/
const Value V24670_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_288 = {S11, 0, &V24670_288_57176, &P24670_287};	/*Property = CssFont*/
const Value V24670_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_289 = {S12, 0, &V24670_289_86899, &P24670_288};	/*Property = CssImages*/
const Value V24670_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_291 = {S13, 0, &V24670_291_57182, &P24670_289};	/*Property = CssMediaQueries*/
const Value V24670_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_292 = {S14, 0, &V24670_292_57185, &P24670_291};	/*Property = CssMinMax*/
const Value V24670_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_293 = {S15, 0, &V24670_293_57188, &P24670_292};	/*Property = CssOverflow*/
const Value V24670_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_294 = {S16, 0, &V24670_294_57191, &P24670_293};	/*Property = CssPosition*/
const Value V24670_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_295 = {S17, 0, &V24670_295_57194, &P24670_294};	/*Property = CssText*/
const Value V24670_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_297 = {S18, 0, &V24670_297_57197, &P24670_295};	/*Property = CssTransforms*/
const Value V24670_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_241 = {S19, 0, &V24670_241_55801, &P24670_297};	/*Property = CssTransitions*/
const Value V24670_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_298 = {S20, 0, &V24670_298_57208, &P24670_241};	/*Property = CssUI*/
const Value V24670_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_300 = {S21, 0, &V24670_300_57217, &P24670_298};	/*Property = DataSet*/
const Value V24670_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_301 = {S22, 0, &V24670_301_57220, &P24670_300};	/*Property = DataUrl*/
const Value V24670_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_302 = {S23, 0, &V24670_302_57223, &P24670_301};	/*Property = DeviceOrientation*/
const Value V24670_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_304 = {S24, 0, &V24670_304_57229, &P24670_302};	/*Property = FileReader*/
const Value V24670_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_305 = {S25, 0, &V24670_305_57232, &P24670_304};	/*Property = FileSaver*/
const Value V24670_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_306 = {S26, 0, &V24670_306_57238, &P24670_305};	/*Property = FileWriter*/
const Value V24670_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_307 = {S27, 0, &V24670_307_57241, &P24670_306};	/*Property = FormData*/
const Value V24670_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_309 = {S28, 0, &V24670_309_57247, &P24670_307};	/*Property = Fullscreen*/
const Value V24670_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_310 = {S29, 0, &V24670_310_57250, &P24670_309};	/*Property = GeoLocation*/
const Value V24670_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_312 = {S30, 0, &V24670_312_57256, &P24670_310};	/*Property = History*/
const Value V24670_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_314 = {S31, 0, &V24670_314_57262, &P24670_312};	/*Property = Html5*/
const Value V24670_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_313 = {S32, 0, &V24670_313_57259, &P24670_314};	/*Property = Html-Media-Capture*/
const Value V24670_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_315 = {S33, 0, &V24670_315_57265, &P24670_313};	/*Property = Iframe*/
const Value V24670_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_316 = {S34, 0, &V24670_316_57268, &P24670_315};	/*Property = IndexedDB*/
const Value V24670_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_317 = {S35, 0, &V24670_317_57271, &P24670_316};	/*Property = Json*/
const Value V24670_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24670_15 = {S37, 0, &V24670_15_90837, &P24670_317};	/*Property = LayoutEngine*/
const Value V24670_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_344 = {S38, 0, &V24670_344_57274, &P24670_15};	/*Property = Masking*/
const Value V24670_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_320 = {S39, 0, &V24670_320_57292, &P24670_344};	/*Property = PostMessage*/
const Value V24670_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_321 = {S40, 0, &V24670_321_57295, &P24670_320};	/*Property = Progress*/
const Value V24670_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_322 = {S41, 0, &V24670_322_57298, &P24670_321};	/*Property = Prompts*/
const Value V24670_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_323 = {S42, 0, &V24670_323_57301, &P24670_322};	/*Property = Selector*/
const Value V24670_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_324 = {S43, 0, &V24670_324_57307, &P24670_323};	/*Property = Svg*/
const Value V24670_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_327 = {S44, 0, &V24670_327_57316, &P24670_324};	/*Property = TouchEvents*/
const Value V24670_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_328 = {S45, 0, &V24670_328_57319, &P24670_327};	/*Property = Track*/
const Value V24670_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_329 = {S46, 0, &V24670_329_57325, &P24670_328};	/*Property = Video*/
const Value V24670_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_330 = {S47, 0, &V24670_330_57328, &P24670_329};	/*Property = Viewport*/
const Value V24670_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_332 = {S48, 0, &V24670_332_57340, &P24670_330};	/*Property = WebWorkers*/
const Value V24670_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24670_333 = {S49, 0, &V24670_333_57343, &P24670_332};	/*Property = Xhr2*/
const Profile P24670 = {"24670", &P24670_333};	/*Profile ID - 24670*/
const Value V22357_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22357_279 = {S1, 0, &V22357_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22357_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22357_282 = {S3, 0, &V22357_282_57154, &P22357_279};	/*Property = BlobBuilder*/
const Value V22357_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22357_350 = {S4, 0, &V22357_350_57444, &P22357_282};	/*Property = Canvas*/
const Value V22357_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22357_283 = {S5, 0, &V22357_283_57159, &P22357_350};	/*Property = CssBackground*/
const Value V22357_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22357_284 = {S6, 0, &V22357_284_57163, &P22357_283};	/*Property = CssBorderImage*/
const Value V22357_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22357_285 = {S7, 0, &V22357_285_57166, &P22357_284};	/*Property = CssCanvas*/
const Value V22357_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22357_286 = {S8, 0, &V22357_286_57168, &P22357_285};	/*Property = CssColor*/
const Value V22357_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22357_237 = {S9, 0, &V22357_237_55784, &P22357_286};	/*Property = CssColumn*/
const Value V22357_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22357_287 = {S10, 0, &V22357_287_57172, &P22357_237};	/*Property = CssFlexbox*/
const Value V22357_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22357_288 = {S11, 0, &V22357_288_57175, &P22357_287};	/*Property = CssFont*/
const Value V22357_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22357_289 = {S12, 0, &V22357_289_57759, &P22357_288};	/*Property = CssImages*/
const Value V22357_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22357_291 = {S13, 0, &V22357_291_57181, &P22357_289};	/*Property = CssMediaQueries*/
const Value V22357_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22357_292 = {S14, 0, &V22357_292_57183, &P22357_291};	/*Property = CssMinMax*/
const Value V22357_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22357_293 = {S15, 0, &V22357_293_57187, &P22357_292};	/*Property = CssOverflow*/
const Value V22357_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22357_294 = {S16, 0, &V22357_294_57190, &P22357_293};	/*Property = CssPosition*/
const Value V22357_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22357_295 = {S17, 0, &V22357_295_57193, &P22357_294};	/*Property = CssText*/
const Value V22357_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22357_297 = {S18, 0, &V22357_297_57195, &P22357_295};	/*Property = CssTransforms*/
const Value V22357_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22357_241 = {S19, 0, &V22357_241_55799, &P22357_297};	/*Property = CssTransitions*/
const Value V22357_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22357_298 = {S20, 0, &V22357_298_57206, &P22357_241};	/*Property = CssUI*/
const Value V22357_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22357_300 = {S21, 0, &V22357_300_57216, &P22357_298};	/*Property = DataSet*/
const Value V22357_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22357_301 = {S22, 0, &V22357_301_57218, &P22357_300};	/*Property = DataUrl*/
const Value V22357_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22357_302 = {S23, 0, &V22357_302_57222, &P22357_301};	/*Property = DeviceOrientation*/
const Value V22357_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22357_304 = {S24, 0, &V22357_304_57228, &P22357_302};	/*Property = FileReader*/
const Value V22357_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22357_305 = {S25, 0, &V22357_305_57231, &P22357_304};	/*Property = FileSaver*/
const Value V22357_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22357_306 = {S26, 0, &V22357_306_57237, &P22357_305};	/*Property = FileWriter*/
const Value V22357_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22357_307 = {S27, 0, &V22357_307_57240, &P22357_306};	/*Property = FormData*/
const Value V22357_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22357_309 = {S28, 0, &V22357_309_57246, &P22357_307};	/*Property = Fullscreen*/
const Value V22357_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22357_310 = {S29, 0, &V22357_310_57248, &P22357_309};	/*Property = GeoLocation*/
const Value V22357_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22357_312 = {S30, 0, &V22357_312_57255, &P22357_310};	/*Property = History*/
const Value V22357_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22357_314 = {S31, 0, &V22357_314_57261, &P22357_312};	/*Property = Html5*/
const Value V22357_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22357_313 = {S32, 0, &V22357_313_57258, &P22357_314};	/*Property = Html-Media-Capture*/
const Value V22357_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22357_315 = {S33, 0, &V22357_315_57264, &P22357_313};	/*Property = Iframe*/
const Value V22357_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22357_316 = {S34, 0, &V22357_316_57267, &P22357_315};	/*Property = IndexedDB*/
const Value V22357_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22357_317 = {S35, 0, &V22357_317_57269, &P22357_316};	/*Property = Json*/
const Value V22357_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22357_15 = {S37, 0, &V22357_15_29, &P22357_317};	/*Property = LayoutEngine*/
const Value V22357_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22357_344 = {S38, 0, &V22357_344_57272, &P22357_15};	/*Property = Masking*/
const Value V22357_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22357_320 = {S39, 0, &V22357_320_57290, &P22357_344};	/*Property = PostMessage*/
const Value V22357_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22357_321 = {S40, 0, &V22357_321_57293, &P22357_320};	/*Property = Progress*/
const Value V22357_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22357_322 = {S41, 0, &V22357_322_57296, &P22357_321};	/*Property = Prompts*/
const Value V22357_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22357_323 = {S42, 0, &V22357_323_57299, &P22357_322};	/*Property = Selector*/
const Value V22357_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22357_324 = {S43, 0, &V22357_324_57306, &P22357_323};	/*Property = Svg*/
const Value V22357_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22357_327 = {S44, 0, &V22357_327_57315, &P22357_324};	/*Property = TouchEvents*/
const Value V22357_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22357_328 = {S45, 0, &V22357_328_57318, &P22357_327};	/*Property = Track*/
const Value V22357_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22357_329 = {S46, 0, &V22357_329_57323, &P22357_328};	/*Property = Video*/
const Value V22357_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22357_330 = {S47, 0, &V22357_330_57326, &P22357_329};	/*Property = Viewport*/
const Value V22357_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22357_332 = {S48, 0, &V22357_332_57339, &P22357_330};	/*Property = WebWorkers*/
const Value V22357_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22357_333 = {S49, 0, &V22357_333_57342, &P22357_332};	/*Property = Xhr2*/
const Profile P22357 = {"22357", &P22357_333};	/*Profile ID - 22357*/
const Value V22428_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22428_279 = {S1, 0, &V22428_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22428_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22428_282 = {S3, 0, &V22428_282_57153, &P22428_279};	/*Property = BlobBuilder*/
const Value V22428_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22428_350 = {S4, 0, &V22428_350_57444, &P22428_282};	/*Property = Canvas*/
const Value V22428_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22428_283 = {S5, 0, &V22428_283_57159, &P22428_350};	/*Property = CssBackground*/
const Value V22428_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22428_284 = {S6, 0, &V22428_284_57163, &P22428_283};	/*Property = CssBorderImage*/
const Value V22428_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22428_285 = {S7, 0, &V22428_285_57166, &P22428_284};	/*Property = CssCanvas*/
const Value V22428_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22428_286 = {S8, 0, &V22428_286_57168, &P22428_285};	/*Property = CssColor*/
const Value V22428_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22428_237 = {S9, 0, &V22428_237_55784, &P22428_286};	/*Property = CssColumn*/
const Value V22428_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22428_287 = {S10, 0, &V22428_287_57172, &P22428_237};	/*Property = CssFlexbox*/
const Value V22428_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22428_288 = {S11, 0, &V22428_288_57175, &P22428_287};	/*Property = CssFont*/
const Value V22428_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22428_289 = {S12, 0, &V22428_289_57759, &P22428_288};	/*Property = CssImages*/
const Value V22428_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22428_291 = {S13, 0, &V22428_291_57180, &P22428_289};	/*Property = CssMediaQueries*/
const Value V22428_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22428_292 = {S14, 0, &V22428_292_57183, &P22428_291};	/*Property = CssMinMax*/
const Value V22428_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22428_293 = {S15, 0, &V22428_293_57187, &P22428_292};	/*Property = CssOverflow*/
const Value V22428_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22428_294 = {S16, 0, &V22428_294_57189, &P22428_293};	/*Property = CssPosition*/
const Value V22428_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22428_295 = {S17, 0, &V22428_295_57192, &P22428_294};	/*Property = CssText*/
const Value V22428_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22428_297 = {S18, 0, &V22428_297_57195, &P22428_295};	/*Property = CssTransforms*/
const Value V22428_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22428_241 = {S19, 0, &V22428_241_55799, &P22428_297};	/*Property = CssTransitions*/
const Value V22428_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22428_298 = {S20, 0, &V22428_298_57206, &P22428_241};	/*Property = CssUI*/
const Value V22428_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22428_300 = {S21, 0, &V22428_300_57216, &P22428_298};	/*Property = DataSet*/
const Value V22428_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22428_301 = {S22, 0, &V22428_301_57218, &P22428_300};	/*Property = DataUrl*/
const Value V22428_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22428_302 = {S23, 0, &V22428_302_57222, &P22428_301};	/*Property = DeviceOrientation*/
const Value V22428_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22428_304 = {S24, 0, &V22428_304_57227, &P22428_302};	/*Property = FileReader*/
const Value V22428_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22428_305 = {S25, 0, &V22428_305_57231, &P22428_304};	/*Property = FileSaver*/
const Value V22428_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22428_306 = {S26, 0, &V22428_306_57237, &P22428_305};	/*Property = FileWriter*/
const Value V22428_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22428_307 = {S27, 0, &V22428_307_57239, &P22428_306};	/*Property = FormData*/
const Value V22428_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22428_309 = {S28, 0, &V22428_309_57246, &P22428_307};	/*Property = Fullscreen*/
const Value V22428_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22428_310 = {S29, 0, &V22428_310_57248, &P22428_309};	/*Property = GeoLocation*/
const Value V22428_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22428_312 = {S30, 0, &V22428_312_57254, &P22428_310};	/*Property = History*/
const Value V22428_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22428_314 = {S31, 0, &V22428_314_57261, &P22428_312};	/*Property = Html5*/
const Value V22428_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22428_313 = {S32, 0, &V22428_313_57258, &P22428_314};	/*Property = Html-Media-Capture*/
const Value V22428_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22428_315 = {S33, 0, &V22428_315_57264, &P22428_313};	/*Property = Iframe*/
const Value V22428_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22428_316 = {S34, 0, &V22428_316_57266, &P22428_315};	/*Property = IndexedDB*/
const Value V22428_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22428_317 = {S35, 0, &V22428_317_57269, &P22428_316};	/*Property = Json*/
const Value V22428_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22428_15 = {S37, 0, &V22428_15_28, &P22428_317};	/*Property = LayoutEngine*/
const Value V22428_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22428_344 = {S38, 0, &V22428_344_57272, &P22428_15};	/*Property = Masking*/
const Value V22428_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22428_320 = {S39, 0, &V22428_320_57290, &P22428_344};	/*Property = PostMessage*/
const Value V22428_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22428_321 = {S40, 0, &V22428_321_57293, &P22428_320};	/*Property = Progress*/
const Value V22428_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22428_322 = {S41, 0, &V22428_322_57296, &P22428_321};	/*Property = Prompts*/
const Value V22428_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22428_323 = {S42, 0, &V22428_323_57299, &P22428_322};	/*Property = Selector*/
const Value V22428_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22428_324 = {S43, 0, &V22428_324_57306, &P22428_323};	/*Property = Svg*/
const Value V22428_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22428_327 = {S44, 0, &V22428_327_57315, &P22428_324};	/*Property = TouchEvents*/
const Value V22428_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22428_328 = {S45, 0, &V22428_328_57318, &P22428_327};	/*Property = Track*/
const Value V22428_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22428_329 = {S46, 0, &V22428_329_57323, &P22428_328};	/*Property = Video*/
const Value V22428_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22428_330 = {S47, 0, &V22428_330_57326, &P22428_329};	/*Property = Viewport*/
const Value V22428_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22428_332 = {S48, 0, &V22428_332_57338, &P22428_330};	/*Property = WebWorkers*/
const Value V22428_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22428_333 = {S49, 0, &V22428_333_57342, &P22428_332};	/*Property = Xhr2*/
const Profile P22428 = {"22428", &P22428_333};	/*Profile ID - 22428*/
const Value V20784_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_279 = {S1, 0, &V20784_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20784_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_282 = {S3, 0, &V20784_282_57155, &P20784_279};	/*Property = BlobBuilder*/
const Value V20784_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_350 = {S4, 0, &V20784_350_57446, &P20784_282};	/*Property = Canvas*/
const Value V20784_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_283 = {S5, 0, &V20784_283_57161, &P20784_350};	/*Property = CssBackground*/
const Value V20784_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_284 = {S6, 0, &V20784_284_57164, &P20784_283};	/*Property = CssBorderImage*/
const Value V20784_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_285 = {S7, 0, &V20784_285_57167, &P20784_284};	/*Property = CssCanvas*/
const Value V20784_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_286 = {S8, 0, &V20784_286_57170, &P20784_285};	/*Property = CssColor*/
const Value V20784_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_237 = {S9, 0, &V20784_237_55786, &P20784_286};	/*Property = CssColumn*/
const Value V20784_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_287 = {S10, 0, &V20784_287_57173, &P20784_237};	/*Property = CssFlexbox*/
const Value V20784_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_288 = {S11, 0, &V20784_288_57176, &P20784_287};	/*Property = CssFont*/
const Value V20784_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_289 = {S12, 0, &V20784_289_86899, &P20784_288};	/*Property = CssImages*/
const Value V20784_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_291 = {S13, 0, &V20784_291_57182, &P20784_289};	/*Property = CssMediaQueries*/
const Value V20784_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_292 = {S14, 0, &V20784_292_57185, &P20784_291};	/*Property = CssMinMax*/
const Value V20784_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_293 = {S15, 0, &V20784_293_57188, &P20784_292};	/*Property = CssOverflow*/
const Value V20784_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_294 = {S16, 0, &V20784_294_57191, &P20784_293};	/*Property = CssPosition*/
const Value V20784_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_295 = {S17, 0, &V20784_295_57194, &P20784_294};	/*Property = CssText*/
const Value V20784_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_297 = {S18, 0, &V20784_297_57197, &P20784_295};	/*Property = CssTransforms*/
const Value V20784_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_241 = {S19, 0, &V20784_241_55801, &P20784_297};	/*Property = CssTransitions*/
const Value V20784_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_298 = {S20, 0, &V20784_298_57208, &P20784_241};	/*Property = CssUI*/
const Value V20784_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_300 = {S21, 0, &V20784_300_57217, &P20784_298};	/*Property = DataSet*/
const Value V20784_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_301 = {S22, 0, &V20784_301_57220, &P20784_300};	/*Property = DataUrl*/
const Value V20784_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_302 = {S23, 0, &V20784_302_57223, &P20784_301};	/*Property = DeviceOrientation*/
const Value V20784_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_304 = {S24, 0, &V20784_304_57229, &P20784_302};	/*Property = FileReader*/
const Value V20784_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_305 = {S25, 0, &V20784_305_57232, &P20784_304};	/*Property = FileSaver*/
const Value V20784_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_306 = {S26, 0, &V20784_306_57238, &P20784_305};	/*Property = FileWriter*/
const Value V20784_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_307 = {S27, 0, &V20784_307_57241, &P20784_306};	/*Property = FormData*/
const Value V20784_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_309 = {S28, 0, &V20784_309_57247, &P20784_307};	/*Property = Fullscreen*/
const Value V20784_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_310 = {S29, 0, &V20784_310_57250, &P20784_309};	/*Property = GeoLocation*/
const Value V20784_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_312 = {S30, 0, &V20784_312_57256, &P20784_310};	/*Property = History*/
const Value V20784_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_314 = {S31, 0, &V20784_314_57262, &P20784_312};	/*Property = Html5*/
const Value V20784_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_313 = {S32, 0, &V20784_313_57259, &P20784_314};	/*Property = Html-Media-Capture*/
const Value V20784_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_315 = {S33, 0, &V20784_315_57265, &P20784_313};	/*Property = Iframe*/
const Value V20784_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_316 = {S34, 0, &V20784_316_57268, &P20784_315};	/*Property = IndexedDB*/
const Value V20784_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_317 = {S35, 0, &V20784_317_57271, &P20784_316};	/*Property = Json*/
const Value V20784_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P20784_15 = {S37, 0, &V20784_15_54167, &P20784_317};	/*Property = LayoutEngine*/
const Value V20784_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_344 = {S38, 0, &V20784_344_57274, &P20784_15};	/*Property = Masking*/
const Value V20784_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_320 = {S39, 0, &V20784_320_57292, &P20784_344};	/*Property = PostMessage*/
const Value V20784_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_321 = {S40, 0, &V20784_321_57295, &P20784_320};	/*Property = Progress*/
const Value V20784_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_322 = {S41, 0, &V20784_322_57298, &P20784_321};	/*Property = Prompts*/
const Value V20784_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_323 = {S42, 0, &V20784_323_57301, &P20784_322};	/*Property = Selector*/
const Value V20784_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_324 = {S43, 0, &V20784_324_57307, &P20784_323};	/*Property = Svg*/
const Value V20784_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_327 = {S44, 0, &V20784_327_57316, &P20784_324};	/*Property = TouchEvents*/
const Value V20784_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_328 = {S45, 0, &V20784_328_57319, &P20784_327};	/*Property = Track*/
const Value V20784_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_329 = {S46, 0, &V20784_329_57325, &P20784_328};	/*Property = Video*/
const Value V20784_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_330 = {S47, 0, &V20784_330_57328, &P20784_329};	/*Property = Viewport*/
const Value V20784_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_332 = {S48, 0, &V20784_332_57340, &P20784_330};	/*Property = WebWorkers*/
const Value V20784_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20784_333 = {S49, 0, &V20784_333_57343, &P20784_332};	/*Property = Xhr2*/
const Profile P20784 = {"20784", &P20784_333};	/*Profile ID - 20784*/
const Value V18386_1_1 = {S2, NULL};	/*Value = True*/
const Property P18386_1 = {S50, 0, &V18386_1_1, NULL};	/*Property = IsMobile*/
const Value V18386_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18386_5 = {S52, 0, &V18386_5_1227, &P18386_1};	/*Property = ScreenPixelsHeight*/
const Value V18386_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18386_4 = {S54, 0, &V18386_4_1229, &P18386_5};	/*Property = ScreenPixelsWidth*/
const Profile P18386 = {"18386", &P18386_4};	/*Profile ID - 18386*/
const Value V22043_1_1 = {S2, NULL};	/*Value = True*/
const Property P22043_1 = {S50, 0, &V22043_1_1, NULL};	/*Property = IsMobile*/
const Value V22043_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22043_5 = {S52, 0, &V22043_5_9981, &P22043_1};	/*Property = ScreenPixelsHeight*/
const Value V22043_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22043_4 = {S54, 0, &V22043_4_2336, &P22043_5};	/*Property = ScreenPixelsWidth*/
const Profile P22043 = {"22043", &P22043_4};	/*Profile ID - 22043*/
const Value V22250_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22250_279 = {S1, 0, &V22250_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22250_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22250_282 = {S3, 0, &V22250_282_57154, &P22250_279};	/*Property = BlobBuilder*/
const Value V22250_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22250_350 = {S4, 0, &V22250_350_57444, &P22250_282};	/*Property = Canvas*/
const Value V22250_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22250_283 = {S5, 0, &V22250_283_57159, &P22250_350};	/*Property = CssBackground*/
const Value V22250_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22250_284 = {S6, 0, &V22250_284_57163, &P22250_283};	/*Property = CssBorderImage*/
const Value V22250_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22250_285 = {S7, 0, &V22250_285_57166, &P22250_284};	/*Property = CssCanvas*/
const Value V22250_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22250_286 = {S8, 0, &V22250_286_57168, &P22250_285};	/*Property = CssColor*/
const Value V22250_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22250_237 = {S9, 0, &V22250_237_55786, &P22250_286};	/*Property = CssColumn*/
const Value V22250_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22250_287 = {S10, 0, &V22250_287_57172, &P22250_237};	/*Property = CssFlexbox*/
const Value V22250_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22250_288 = {S11, 0, &V22250_288_57175, &P22250_287};	/*Property = CssFont*/
const Value V22250_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22250_289 = {S12, 0, &V22250_289_57759, &P22250_288};	/*Property = CssImages*/
const Value V22250_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22250_291 = {S13, 0, &V22250_291_57181, &P22250_289};	/*Property = CssMediaQueries*/
const Value V22250_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22250_292 = {S14, 0, &V22250_292_57183, &P22250_291};	/*Property = CssMinMax*/
const Value V22250_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22250_293 = {S15, 0, &V22250_293_57187, &P22250_292};	/*Property = CssOverflow*/
const Value V22250_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22250_294 = {S16, 0, &V22250_294_57190, &P22250_293};	/*Property = CssPosition*/
const Value V22250_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22250_295 = {S17, 0, &V22250_295_57192, &P22250_294};	/*Property = CssText*/
const Value V22250_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22250_297 = {S18, 0, &V22250_297_57195, &P22250_295};	/*Property = CssTransforms*/
const Value V22250_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22250_241 = {S19, 0, &V22250_241_55799, &P22250_297};	/*Property = CssTransitions*/
const Value V22250_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22250_298 = {S20, 0, &V22250_298_57206, &P22250_241};	/*Property = CssUI*/
const Value V22250_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22250_300 = {S21, 0, &V22250_300_57216, &P22250_298};	/*Property = DataSet*/
const Value V22250_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22250_301 = {S22, 0, &V22250_301_57218, &P22250_300};	/*Property = DataUrl*/
const Value V22250_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22250_302 = {S23, 0, &V22250_302_57222, &P22250_301};	/*Property = DeviceOrientation*/
const Value V22250_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22250_304 = {S24, 0, &V22250_304_57228, &P22250_302};	/*Property = FileReader*/
const Value V22250_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22250_305 = {S25, 0, &V22250_305_57231, &P22250_304};	/*Property = FileSaver*/
const Value V22250_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22250_306 = {S26, 0, &V22250_306_57237, &P22250_305};	/*Property = FileWriter*/
const Value V22250_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22250_307 = {S27, 0, &V22250_307_57240, &P22250_306};	/*Property = FormData*/
const Value V22250_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22250_309 = {S28, 0, &V22250_309_57246, &P22250_307};	/*Property = Fullscreen*/
const Value V22250_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22250_310 = {S29, 0, &V22250_310_57248, &P22250_309};	/*Property = GeoLocation*/
const Value V22250_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22250_312 = {S30, 0, &V22250_312_57255, &P22250_310};	/*Property = History*/
const Value V22250_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22250_314 = {S31, 0, &V22250_314_57261, &P22250_312};	/*Property = Html5*/
const Value V22250_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22250_313 = {S32, 0, &V22250_313_57258, &P22250_314};	/*Property = Html-Media-Capture*/
const Value V22250_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22250_315 = {S33, 0, &V22250_315_57264, &P22250_313};	/*Property = Iframe*/
const Value V22250_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22250_316 = {S34, 0, &V22250_316_57267, &P22250_315};	/*Property = IndexedDB*/
const Value V22250_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22250_317 = {S35, 0, &V22250_317_57269, &P22250_316};	/*Property = Json*/
const Value V22250_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22250_15 = {S37, 0, &V22250_15_72, &P22250_317};	/*Property = LayoutEngine*/
const Value V22250_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22250_344 = {S38, 0, &V22250_344_57272, &P22250_15};	/*Property = Masking*/
const Value V22250_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22250_320 = {S39, 0, &V22250_320_57290, &P22250_344};	/*Property = PostMessage*/
const Value V22250_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22250_321 = {S40, 0, &V22250_321_57293, &P22250_320};	/*Property = Progress*/
const Value V22250_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22250_322 = {S41, 0, &V22250_322_57296, &P22250_321};	/*Property = Prompts*/
const Value V22250_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22250_323 = {S42, 0, &V22250_323_57299, &P22250_322};	/*Property = Selector*/
const Value V22250_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22250_324 = {S43, 0, &V22250_324_57306, &P22250_323};	/*Property = Svg*/
const Value V22250_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22250_327 = {S44, 0, &V22250_327_57315, &P22250_324};	/*Property = TouchEvents*/
const Value V22250_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22250_328 = {S45, 0, &V22250_328_57318, &P22250_327};	/*Property = Track*/
const Value V22250_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22250_329 = {S46, 0, &V22250_329_57323, &P22250_328};	/*Property = Video*/
const Value V22250_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22250_330 = {S47, 0, &V22250_330_57326, &P22250_329};	/*Property = Viewport*/
const Value V22250_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22250_332 = {S48, 0, &V22250_332_57339, &P22250_330};	/*Property = WebWorkers*/
const Value V22250_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22250_333 = {S49, 0, &V22250_333_57342, &P22250_332};	/*Property = Xhr2*/
const Profile P22250 = {"22250", &P22250_333};	/*Profile ID - 22250*/
const Value V18347_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18347_279 = {S1, 0, &V18347_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18347_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18347_282 = {S3, 0, &V18347_282_57154, &P18347_279};	/*Property = BlobBuilder*/
const Value V18347_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18347_350 = {S4, 0, &V18347_350_57444, &P18347_282};	/*Property = Canvas*/
const Value V18347_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18347_283 = {S5, 0, &V18347_283_57159, &P18347_350};	/*Property = CssBackground*/
const Value V18347_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18347_284 = {S6, 0, &V18347_284_57163, &P18347_283};	/*Property = CssBorderImage*/
const Value V18347_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18347_285 = {S7, 0, &V18347_285_57166, &P18347_284};	/*Property = CssCanvas*/
const Value V18347_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18347_286 = {S8, 0, &V18347_286_57168, &P18347_285};	/*Property = CssColor*/
const Value V18347_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18347_237 = {S9, 0, &V18347_237_55786, &P18347_286};	/*Property = CssColumn*/
const Value V18347_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18347_287 = {S10, 0, &V18347_287_57172, &P18347_237};	/*Property = CssFlexbox*/
const Value V18347_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18347_288 = {S11, 0, &V18347_288_57175, &P18347_287};	/*Property = CssFont*/
const Value V18347_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18347_289 = {S12, 0, &V18347_289_57759, &P18347_288};	/*Property = CssImages*/
const Value V18347_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18347_291 = {S13, 0, &V18347_291_57181, &P18347_289};	/*Property = CssMediaQueries*/
const Value V18347_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18347_292 = {S14, 0, &V18347_292_57183, &P18347_291};	/*Property = CssMinMax*/
const Value V18347_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18347_293 = {S15, 0, &V18347_293_57187, &P18347_292};	/*Property = CssOverflow*/
const Value V18347_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18347_294 = {S16, 0, &V18347_294_57190, &P18347_293};	/*Property = CssPosition*/
const Value V18347_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18347_295 = {S17, 0, &V18347_295_57193, &P18347_294};	/*Property = CssText*/
const Value V18347_297_57196 = {S0, NULL};	/*Value = False*/
const Property P18347_297 = {S18, 0, &V18347_297_57196, &P18347_295};	/*Property = CssTransforms*/
const Value V18347_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18347_241 = {S19, 0, &V18347_241_55799, &P18347_297};	/*Property = CssTransitions*/
const Value V18347_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18347_298 = {S20, 0, &V18347_298_57206, &P18347_241};	/*Property = CssUI*/
const Value V18347_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18347_300 = {S21, 0, &V18347_300_57216, &P18347_298};	/*Property = DataSet*/
const Value V18347_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18347_301 = {S22, 0, &V18347_301_57218, &P18347_300};	/*Property = DataUrl*/
const Value V18347_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18347_302 = {S23, 0, &V18347_302_57222, &P18347_301};	/*Property = DeviceOrientation*/
const Value V18347_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18347_304 = {S24, 0, &V18347_304_57228, &P18347_302};	/*Property = FileReader*/
const Value V18347_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18347_305 = {S25, 0, &V18347_305_57231, &P18347_304};	/*Property = FileSaver*/
const Value V18347_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18347_306 = {S26, 0, &V18347_306_57237, &P18347_305};	/*Property = FileWriter*/
const Value V18347_307_57240 = {S0, NULL};	/*Value = False*/
const Property P18347_307 = {S27, 0, &V18347_307_57240, &P18347_306};	/*Property = FormData*/
const Value V18347_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18347_309 = {S28, 0, &V18347_309_57246, &P18347_307};	/*Property = Fullscreen*/
const Value V18347_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18347_310 = {S29, 0, &V18347_310_57248, &P18347_309};	/*Property = GeoLocation*/
const Value V18347_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18347_312 = {S30, 0, &V18347_312_57255, &P18347_310};	/*Property = History*/
const Value V18347_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18347_314 = {S31, 0, &V18347_314_57261, &P18347_312};	/*Property = Html5*/
const Value V18347_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18347_313 = {S32, 0, &V18347_313_57258, &P18347_314};	/*Property = Html-Media-Capture*/
const Value V18347_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18347_315 = {S33, 0, &V18347_315_57264, &P18347_313};	/*Property = Iframe*/
const Value V18347_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18347_316 = {S34, 0, &V18347_316_57267, &P18347_315};	/*Property = IndexedDB*/
const Value V18347_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18347_317 = {S35, 0, &V18347_317_57269, &P18347_316};	/*Property = Json*/
const Value V18347_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18347_15 = {S37, 0, &V18347_15_31, &P18347_317};	/*Property = LayoutEngine*/
const Value V18347_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18347_344 = {S38, 0, &V18347_344_57272, &P18347_15};	/*Property = Masking*/
const Value V18347_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18347_320 = {S39, 0, &V18347_320_57290, &P18347_344};	/*Property = PostMessage*/
const Value V18347_321_57294 = {S0, NULL};	/*Value = False*/
const Property P18347_321 = {S40, 0, &V18347_321_57294, &P18347_320};	/*Property = Progress*/
const Value V18347_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18347_322 = {S41, 0, &V18347_322_57296, &P18347_321};	/*Property = Prompts*/
const Value V18347_323_57300 = {S0, NULL};	/*Value = False*/
const Property P18347_323 = {S42, 0, &V18347_323_57300, &P18347_322};	/*Property = Selector*/
const Value V18347_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18347_324 = {S43, 0, &V18347_324_57306, &P18347_323};	/*Property = Svg*/
const Value V18347_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18347_327 = {S44, 0, &V18347_327_57315, &P18347_324};	/*Property = TouchEvents*/
const Value V18347_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18347_328 = {S45, 0, &V18347_328_57318, &P18347_327};	/*Property = Track*/
const Value V18347_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18347_329 = {S46, 0, &V18347_329_57323, &P18347_328};	/*Property = Video*/
const Value V18347_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18347_330 = {S47, 0, &V18347_330_57326, &P18347_329};	/*Property = Viewport*/
const Value V18347_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18347_332 = {S48, 0, &V18347_332_57339, &P18347_330};	/*Property = WebWorkers*/
const Value V18347_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18347_333 = {S49, 0, &V18347_333_57342, &P18347_332};	/*Property = Xhr2*/
const Profile P18347 = {"18347", &P18347_333};	/*Profile ID - 18347*/
const Value V18366_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18366_279 = {S1, 0, &V18366_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18366_282_57153 = {S2, NULL};	/*Value = True*/
const Property P18366_282 = {S3, 0, &V18366_282_57153, &P18366_279};	/*Property = BlobBuilder*/
const Value V18366_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18366_350 = {S4, 0, &V18366_350_57444, &P18366_282};	/*Property = Canvas*/
const Value V18366_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18366_283 = {S5, 0, &V18366_283_57159, &P18366_350};	/*Property = CssBackground*/
const Value V18366_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18366_284 = {S6, 0, &V18366_284_57163, &P18366_283};	/*Property = CssBorderImage*/
const Value V18366_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18366_285 = {S7, 0, &V18366_285_57166, &P18366_284};	/*Property = CssCanvas*/
const Value V18366_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18366_286 = {S8, 0, &V18366_286_57168, &P18366_285};	/*Property = CssColor*/
const Value V18366_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18366_237 = {S9, 0, &V18366_237_55786, &P18366_286};	/*Property = CssColumn*/
const Value V18366_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18366_287 = {S10, 0, &V18366_287_57172, &P18366_237};	/*Property = CssFlexbox*/
const Value V18366_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18366_288 = {S11, 0, &V18366_288_57176, &P18366_287};	/*Property = CssFont*/
const Value V18366_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18366_289 = {S12, 0, &V18366_289_57759, &P18366_288};	/*Property = CssImages*/
const Value V18366_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18366_291 = {S13, 0, &V18366_291_57180, &P18366_289};	/*Property = CssMediaQueries*/
const Value V18366_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18366_292 = {S14, 0, &V18366_292_57183, &P18366_291};	/*Property = CssMinMax*/
const Value V18366_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18366_293 = {S15, 0, &V18366_293_57187, &P18366_292};	/*Property = CssOverflow*/
const Value V18366_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18366_294 = {S16, 0, &V18366_294_57189, &P18366_293};	/*Property = CssPosition*/
const Value V18366_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18366_295 = {S17, 0, &V18366_295_57192, &P18366_294};	/*Property = CssText*/
const Value V18366_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18366_297 = {S18, 0, &V18366_297_57195, &P18366_295};	/*Property = CssTransforms*/
const Value V18366_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18366_241 = {S19, 0, &V18366_241_55799, &P18366_297};	/*Property = CssTransitions*/
const Value V18366_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18366_298 = {S20, 0, &V18366_298_57206, &P18366_241};	/*Property = CssUI*/
const Value V18366_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18366_300 = {S21, 0, &V18366_300_57216, &P18366_298};	/*Property = DataSet*/
const Value V18366_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18366_301 = {S22, 0, &V18366_301_57218, &P18366_300};	/*Property = DataUrl*/
const Value V18366_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18366_302 = {S23, 0, &V18366_302_57222, &P18366_301};	/*Property = DeviceOrientation*/
const Value V18366_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18366_304 = {S24, 0, &V18366_304_57227, &P18366_302};	/*Property = FileReader*/
const Value V18366_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18366_305 = {S25, 0, &V18366_305_57231, &P18366_304};	/*Property = FileSaver*/
const Value V18366_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18366_306 = {S26, 0, &V18366_306_57237, &P18366_305};	/*Property = FileWriter*/
const Value V18366_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18366_307 = {S27, 0, &V18366_307_57239, &P18366_306};	/*Property = FormData*/
const Value V18366_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18366_309 = {S28, 0, &V18366_309_57246, &P18366_307};	/*Property = Fullscreen*/
const Value V18366_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18366_310 = {S29, 0, &V18366_310_57248, &P18366_309};	/*Property = GeoLocation*/
const Value V18366_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18366_312 = {S30, 0, &V18366_312_57254, &P18366_310};	/*Property = History*/
const Value V18366_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18366_314 = {S31, 0, &V18366_314_57261, &P18366_312};	/*Property = Html5*/
const Value V18366_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18366_313 = {S32, 0, &V18366_313_57258, &P18366_314};	/*Property = Html-Media-Capture*/
const Value V18366_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18366_315 = {S33, 0, &V18366_315_57264, &P18366_313};	/*Property = Iframe*/
const Value V18366_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18366_316 = {S34, 0, &V18366_316_57266, &P18366_315};	/*Property = IndexedDB*/
const Value V18366_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18366_317 = {S35, 0, &V18366_317_57269, &P18366_316};	/*Property = Json*/
const Value V18366_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18366_15 = {S37, 0, &V18366_15_28, &P18366_317};	/*Property = LayoutEngine*/
const Value V18366_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18366_344 = {S38, 0, &V18366_344_57272, &P18366_15};	/*Property = Masking*/
const Value V18366_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18366_320 = {S39, 0, &V18366_320_57290, &P18366_344};	/*Property = PostMessage*/
const Value V18366_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18366_321 = {S40, 0, &V18366_321_57293, &P18366_320};	/*Property = Progress*/
const Value V18366_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18366_322 = {S41, 0, &V18366_322_57296, &P18366_321};	/*Property = Prompts*/
const Value V18366_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18366_323 = {S42, 0, &V18366_323_57299, &P18366_322};	/*Property = Selector*/
const Value V18366_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18366_324 = {S43, 0, &V18366_324_57306, &P18366_323};	/*Property = Svg*/
const Value V18366_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18366_327 = {S44, 0, &V18366_327_57315, &P18366_324};	/*Property = TouchEvents*/
const Value V18366_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18366_328 = {S45, 0, &V18366_328_57318, &P18366_327};	/*Property = Track*/
const Value V18366_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18366_329 = {S46, 0, &V18366_329_57323, &P18366_328};	/*Property = Video*/
const Value V18366_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18366_330 = {S47, 0, &V18366_330_57326, &P18366_329};	/*Property = Viewport*/
const Value V18366_332_57338 = {S2, NULL};	/*Value = True*/
const Property P18366_332 = {S48, 0, &V18366_332_57338, &P18366_330};	/*Property = WebWorkers*/
const Value V18366_333_57341 = {S2, NULL};	/*Value = True*/
const Property P18366_333 = {S49, 0, &V18366_333_57341, &P18366_332};	/*Property = Xhr2*/
const Profile P18366 = {"18366", &P18366_333};	/*Profile ID - 18366*/
const Value V24196_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_279 = {S1, 0, &V24196_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24196_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_282 = {S3, 0, &V24196_282_57155, &P24196_279};	/*Property = BlobBuilder*/
const Value V24196_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_350 = {S4, 0, &V24196_350_57446, &P24196_282};	/*Property = Canvas*/
const Value V24196_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_283 = {S5, 0, &V24196_283_57161, &P24196_350};	/*Property = CssBackground*/
const Value V24196_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_284 = {S6, 0, &V24196_284_57164, &P24196_283};	/*Property = CssBorderImage*/
const Value V24196_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_285 = {S7, 0, &V24196_285_57167, &P24196_284};	/*Property = CssCanvas*/
const Value V24196_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_286 = {S8, 0, &V24196_286_57170, &P24196_285};	/*Property = CssColor*/
const Value V24196_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_237 = {S9, 0, &V24196_237_55786, &P24196_286};	/*Property = CssColumn*/
const Value V24196_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_287 = {S10, 0, &V24196_287_57173, &P24196_237};	/*Property = CssFlexbox*/
const Value V24196_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_288 = {S11, 0, &V24196_288_57176, &P24196_287};	/*Property = CssFont*/
const Value V24196_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_289 = {S12, 0, &V24196_289_86899, &P24196_288};	/*Property = CssImages*/
const Value V24196_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_291 = {S13, 0, &V24196_291_57182, &P24196_289};	/*Property = CssMediaQueries*/
const Value V24196_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_292 = {S14, 0, &V24196_292_57185, &P24196_291};	/*Property = CssMinMax*/
const Value V24196_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_293 = {S15, 0, &V24196_293_57188, &P24196_292};	/*Property = CssOverflow*/
const Value V24196_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_294 = {S16, 0, &V24196_294_57191, &P24196_293};	/*Property = CssPosition*/
const Value V24196_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_295 = {S17, 0, &V24196_295_57194, &P24196_294};	/*Property = CssText*/
const Value V24196_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_297 = {S18, 0, &V24196_297_57197, &P24196_295};	/*Property = CssTransforms*/
const Value V24196_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_241 = {S19, 0, &V24196_241_55801, &P24196_297};	/*Property = CssTransitions*/
const Value V24196_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_298 = {S20, 0, &V24196_298_57208, &P24196_241};	/*Property = CssUI*/
const Value V24196_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_300 = {S21, 0, &V24196_300_57217, &P24196_298};	/*Property = DataSet*/
const Value V24196_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_301 = {S22, 0, &V24196_301_57220, &P24196_300};	/*Property = DataUrl*/
const Value V24196_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_302 = {S23, 0, &V24196_302_57223, &P24196_301};	/*Property = DeviceOrientation*/
const Value V24196_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_304 = {S24, 0, &V24196_304_57229, &P24196_302};	/*Property = FileReader*/
const Value V24196_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_305 = {S25, 0, &V24196_305_57232, &P24196_304};	/*Property = FileSaver*/
const Value V24196_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_306 = {S26, 0, &V24196_306_57238, &P24196_305};	/*Property = FileWriter*/
const Value V24196_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_307 = {S27, 0, &V24196_307_57241, &P24196_306};	/*Property = FormData*/
const Value V24196_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_309 = {S28, 0, &V24196_309_57247, &P24196_307};	/*Property = Fullscreen*/
const Value V24196_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_310 = {S29, 0, &V24196_310_57250, &P24196_309};	/*Property = GeoLocation*/
const Value V24196_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_312 = {S30, 0, &V24196_312_57256, &P24196_310};	/*Property = History*/
const Value V24196_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_314 = {S31, 0, &V24196_314_57262, &P24196_312};	/*Property = Html5*/
const Value V24196_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_313 = {S32, 0, &V24196_313_57259, &P24196_314};	/*Property = Html-Media-Capture*/
const Value V24196_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_315 = {S33, 0, &V24196_315_57265, &P24196_313};	/*Property = Iframe*/
const Value V24196_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_316 = {S34, 0, &V24196_316_57268, &P24196_315};	/*Property = IndexedDB*/
const Value V24196_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_317 = {S35, 0, &V24196_317_57271, &P24196_316};	/*Property = Json*/
const Value V24196_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24196_15 = {S37, 0, &V24196_15_54167, &P24196_317};	/*Property = LayoutEngine*/
const Value V24196_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_344 = {S38, 0, &V24196_344_57274, &P24196_15};	/*Property = Masking*/
const Value V24196_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_320 = {S39, 0, &V24196_320_57292, &P24196_344};	/*Property = PostMessage*/
const Value V24196_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_321 = {S40, 0, &V24196_321_57295, &P24196_320};	/*Property = Progress*/
const Value V24196_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_322 = {S41, 0, &V24196_322_57298, &P24196_321};	/*Property = Prompts*/
const Value V24196_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_323 = {S42, 0, &V24196_323_57301, &P24196_322};	/*Property = Selector*/
const Value V24196_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_324 = {S43, 0, &V24196_324_57307, &P24196_323};	/*Property = Svg*/
const Value V24196_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_327 = {S44, 0, &V24196_327_57316, &P24196_324};	/*Property = TouchEvents*/
const Value V24196_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_328 = {S45, 0, &V24196_328_57319, &P24196_327};	/*Property = Track*/
const Value V24196_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_329 = {S46, 0, &V24196_329_57325, &P24196_328};	/*Property = Video*/
const Value V24196_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_330 = {S47, 0, &V24196_330_57328, &P24196_329};	/*Property = Viewport*/
const Value V24196_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_332 = {S48, 0, &V24196_332_57340, &P24196_330};	/*Property = WebWorkers*/
const Value V24196_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24196_333 = {S49, 0, &V24196_333_57343, &P24196_332};	/*Property = Xhr2*/
const Profile P24196 = {"24196", &P24196_333};	/*Profile ID - 24196*/
const Value V21690_1_1 = {S2, NULL};	/*Value = True*/
const Property P21690_1 = {S50, 0, &V21690_1_1, NULL};	/*Property = IsMobile*/
const Value V21690_5_1333 = {S68, NULL};	/*Value = 176*/
const Property P21690_5 = {S52, 0, &V21690_5_1333, &P21690_1};	/*Property = ScreenPixelsHeight*/
const Value V21690_4_1335 = {S67, NULL};	/*Value = 220*/
const Property P21690_4 = {S54, 0, &V21690_4_1335, &P21690_5};	/*Property = ScreenPixelsWidth*/
const Profile P21690 = {"21690", &P21690_4};	/*Profile ID - 21690*/
const Value V18369_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18369_279 = {S1, 0, &V18369_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18369_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18369_282 = {S3, 0, &V18369_282_57154, &P18369_279};	/*Property = BlobBuilder*/
const Value V18369_350_57445 = {S0, NULL};	/*Value = False*/
const Property P18369_350 = {S4, 0, &V18369_350_57445, &P18369_282};	/*Property = Canvas*/
const Value V18369_283_57160 = {S0, NULL};	/*Value = False*/
const Property P18369_283 = {S5, 0, &V18369_283_57160, &P18369_350};	/*Property = CssBackground*/
const Value V18369_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18369_284 = {S6, 0, &V18369_284_57163, &P18369_283};	/*Property = CssBorderImage*/
const Value V18369_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18369_285 = {S7, 0, &V18369_285_57166, &P18369_284};	/*Property = CssCanvas*/
const Value V18369_286_57169 = {S0, NULL};	/*Value = False*/
const Property P18369_286 = {S8, 0, &V18369_286_57169, &P18369_285};	/*Property = CssColor*/
const Value V18369_237_55785 = {S0, NULL};	/*Value = False*/
const Property P18369_237 = {S9, 0, &V18369_237_55785, &P18369_286};	/*Property = CssColumn*/
const Value V18369_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18369_287 = {S10, 0, &V18369_287_57172, &P18369_237};	/*Property = CssFlexbox*/
const Value V18369_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18369_288 = {S11, 0, &V18369_288_57175, &P18369_287};	/*Property = CssFont*/
const Value V18369_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_289 = {S12, 0, &V18369_289_86899, &P18369_288};	/*Property = CssImages*/
const Value V18369_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18369_291 = {S13, 0, &V18369_291_57181, &P18369_289};	/*Property = CssMediaQueries*/
const Value V18369_292_57184 = {S0, NULL};	/*Value = False*/
const Property P18369_292 = {S14, 0, &V18369_292_57184, &P18369_291};	/*Property = CssMinMax*/
const Value V18369_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18369_293 = {S15, 0, &V18369_293_57187, &P18369_292};	/*Property = CssOverflow*/
const Value V18369_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18369_294 = {S16, 0, &V18369_294_57190, &P18369_293};	/*Property = CssPosition*/
const Value V18369_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18369_295 = {S17, 0, &V18369_295_57193, &P18369_294};	/*Property = CssText*/
const Value V18369_297_57196 = {S0, NULL};	/*Value = False*/
const Property P18369_297 = {S18, 0, &V18369_297_57196, &P18369_295};	/*Property = CssTransforms*/
const Value V18369_241_55800 = {S0, NULL};	/*Value = False*/
const Property P18369_241 = {S19, 0, &V18369_241_55800, &P18369_297};	/*Property = CssTransitions*/
const Value V18369_298_57207 = {S0, NULL};	/*Value = False*/
const Property P18369_298 = {S20, 0, &V18369_298_57207, &P18369_241};	/*Property = CssUI*/
const Value V18369_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_300 = {S21, 0, &V18369_300_57217, &P18369_298};	/*Property = DataSet*/
const Value V18369_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_301 = {S22, 0, &V18369_301_57220, &P18369_300};	/*Property = DataUrl*/
const Value V18369_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_302 = {S23, 0, &V18369_302_57223, &P18369_301};	/*Property = DeviceOrientation*/
const Value V18369_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_304 = {S24, 0, &V18369_304_57229, &P18369_302};	/*Property = FileReader*/
const Value V18369_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_305 = {S25, 0, &V18369_305_57232, &P18369_304};	/*Property = FileSaver*/
const Value V18369_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_306 = {S26, 0, &V18369_306_57238, &P18369_305};	/*Property = FileWriter*/
const Value V18369_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_307 = {S27, 0, &V18369_307_57241, &P18369_306};	/*Property = FormData*/
const Value V18369_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_309 = {S28, 0, &V18369_309_57247, &P18369_307};	/*Property = Fullscreen*/
const Value V18369_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_310 = {S29, 0, &V18369_310_57250, &P18369_309};	/*Property = GeoLocation*/
const Value V18369_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_312 = {S30, 0, &V18369_312_57256, &P18369_310};	/*Property = History*/
const Value V18369_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_314 = {S31, 0, &V18369_314_57262, &P18369_312};	/*Property = Html5*/
const Value V18369_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_313 = {S32, 0, &V18369_313_57259, &P18369_314};	/*Property = Html-Media-Capture*/
const Value V18369_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_315 = {S33, 0, &V18369_315_57265, &P18369_313};	/*Property = Iframe*/
const Value V18369_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_316 = {S34, 0, &V18369_316_57268, &P18369_315};	/*Property = IndexedDB*/
const Value V18369_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_317 = {S35, 0, &V18369_317_57271, &P18369_316};	/*Property = Json*/
const Value V18369_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18369_15 = {S37, 0, &V18369_15_31, &P18369_317};	/*Property = LayoutEngine*/
const Value V18369_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_344 = {S38, 0, &V18369_344_57274, &P18369_15};	/*Property = Masking*/
const Value V18369_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_320 = {S39, 0, &V18369_320_57292, &P18369_344};	/*Property = PostMessage*/
const Value V18369_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_321 = {S40, 0, &V18369_321_57295, &P18369_320};	/*Property = Progress*/
const Value V18369_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_322 = {S41, 0, &V18369_322_57298, &P18369_321};	/*Property = Prompts*/
const Value V18369_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_323 = {S42, 0, &V18369_323_57301, &P18369_322};	/*Property = Selector*/
const Value V18369_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_324 = {S43, 0, &V18369_324_57307, &P18369_323};	/*Property = Svg*/
const Value V18369_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_327 = {S44, 0, &V18369_327_57316, &P18369_324};	/*Property = TouchEvents*/
const Value V18369_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_328 = {S45, 0, &V18369_328_57319, &P18369_327};	/*Property = Track*/
const Value V18369_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_329 = {S46, 0, &V18369_329_57325, &P18369_328};	/*Property = Video*/
const Value V18369_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_330 = {S47, 0, &V18369_330_57328, &P18369_329};	/*Property = Viewport*/
const Value V18369_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_332 = {S48, 0, &V18369_332_57340, &P18369_330};	/*Property = WebWorkers*/
const Value V18369_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18369_333 = {S49, 0, &V18369_333_57343, &P18369_332};	/*Property = Xhr2*/
const Profile P18369 = {"18369", &P18369_333};	/*Profile ID - 18369*/
const Value V18385_1_1 = {S2, NULL};	/*Value = True*/
const Property P18385_1 = {S50, 0, &V18385_1_1, NULL};	/*Property = IsMobile*/
const Value V18385_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18385_5 = {S52, 0, &V18385_5_9981, &P18385_1};	/*Property = ScreenPixelsHeight*/
const Value V18385_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18385_4 = {S54, 0, &V18385_4_2336, &P18385_5};	/*Property = ScreenPixelsWidth*/
const Profile P18385 = {"18385", &P18385_4};	/*Profile ID - 18385*/
const Profile P18432 = {"18432", NULL};	/*Profile ID - 18432*/
const Value V22575_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_279 = {S1, 0, &V22575_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22575_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_282 = {S3, 0, &V22575_282_57155, &P22575_279};	/*Property = BlobBuilder*/
const Value V22575_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_350 = {S4, 0, &V22575_350_57446, &P22575_282};	/*Property = Canvas*/
const Value V22575_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_283 = {S5, 0, &V22575_283_57161, &P22575_350};	/*Property = CssBackground*/
const Value V22575_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_284 = {S6, 0, &V22575_284_57164, &P22575_283};	/*Property = CssBorderImage*/
const Value V22575_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_285 = {S7, 0, &V22575_285_57167, &P22575_284};	/*Property = CssCanvas*/
const Value V22575_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_286 = {S8, 0, &V22575_286_57170, &P22575_285};	/*Property = CssColor*/
const Value V22575_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_237 = {S9, 0, &V22575_237_55786, &P22575_286};	/*Property = CssColumn*/
const Value V22575_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_287 = {S10, 0, &V22575_287_57173, &P22575_237};	/*Property = CssFlexbox*/
const Value V22575_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_288 = {S11, 0, &V22575_288_57176, &P22575_287};	/*Property = CssFont*/
const Value V22575_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_289 = {S12, 0, &V22575_289_86899, &P22575_288};	/*Property = CssImages*/
const Value V22575_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_291 = {S13, 0, &V22575_291_57182, &P22575_289};	/*Property = CssMediaQueries*/
const Value V22575_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_292 = {S14, 0, &V22575_292_57185, &P22575_291};	/*Property = CssMinMax*/
const Value V22575_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_293 = {S15, 0, &V22575_293_57188, &P22575_292};	/*Property = CssOverflow*/
const Value V22575_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_294 = {S16, 0, &V22575_294_57191, &P22575_293};	/*Property = CssPosition*/
const Value V22575_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_295 = {S17, 0, &V22575_295_57194, &P22575_294};	/*Property = CssText*/
const Value V22575_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_297 = {S18, 0, &V22575_297_57197, &P22575_295};	/*Property = CssTransforms*/
const Value V22575_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_241 = {S19, 0, &V22575_241_55801, &P22575_297};	/*Property = CssTransitions*/
const Value V22575_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_298 = {S20, 0, &V22575_298_57208, &P22575_241};	/*Property = CssUI*/
const Value V22575_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_300 = {S21, 0, &V22575_300_57217, &P22575_298};	/*Property = DataSet*/
const Value V22575_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_301 = {S22, 0, &V22575_301_57220, &P22575_300};	/*Property = DataUrl*/
const Value V22575_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_302 = {S23, 0, &V22575_302_57223, &P22575_301};	/*Property = DeviceOrientation*/
const Value V22575_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_304 = {S24, 0, &V22575_304_57229, &P22575_302};	/*Property = FileReader*/
const Value V22575_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_305 = {S25, 0, &V22575_305_57232, &P22575_304};	/*Property = FileSaver*/
const Value V22575_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_306 = {S26, 0, &V22575_306_57238, &P22575_305};	/*Property = FileWriter*/
const Value V22575_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_307 = {S27, 0, &V22575_307_57241, &P22575_306};	/*Property = FormData*/
const Value V22575_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_309 = {S28, 0, &V22575_309_57247, &P22575_307};	/*Property = Fullscreen*/
const Value V22575_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_310 = {S29, 0, &V22575_310_57250, &P22575_309};	/*Property = GeoLocation*/
const Value V22575_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_312 = {S30, 0, &V22575_312_57256, &P22575_310};	/*Property = History*/
const Value V22575_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_314 = {S31, 0, &V22575_314_57262, &P22575_312};	/*Property = Html5*/
const Value V22575_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_313 = {S32, 0, &V22575_313_57259, &P22575_314};	/*Property = Html-Media-Capture*/
const Value V22575_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_315 = {S33, 0, &V22575_315_57265, &P22575_313};	/*Property = Iframe*/
const Value V22575_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_316 = {S34, 0, &V22575_316_57268, &P22575_315};	/*Property = IndexedDB*/
const Value V22575_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_317 = {S35, 0, &V22575_317_57271, &P22575_316};	/*Property = Json*/
const Value V22575_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P22575_15 = {S37, 0, &V22575_15_31, &P22575_317};	/*Property = LayoutEngine*/
const Value V22575_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_344 = {S38, 0, &V22575_344_57274, &P22575_15};	/*Property = Masking*/
const Value V22575_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_320 = {S39, 0, &V22575_320_57292, &P22575_344};	/*Property = PostMessage*/
const Value V22575_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_321 = {S40, 0, &V22575_321_57295, &P22575_320};	/*Property = Progress*/
const Value V22575_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_322 = {S41, 0, &V22575_322_57298, &P22575_321};	/*Property = Prompts*/
const Value V22575_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_323 = {S42, 0, &V22575_323_57301, &P22575_322};	/*Property = Selector*/
const Value V22575_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_324 = {S43, 0, &V22575_324_57307, &P22575_323};	/*Property = Svg*/
const Value V22575_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_327 = {S44, 0, &V22575_327_57316, &P22575_324};	/*Property = TouchEvents*/
const Value V22575_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_328 = {S45, 0, &V22575_328_57319, &P22575_327};	/*Property = Track*/
const Value V22575_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_329 = {S46, 0, &V22575_329_57325, &P22575_328};	/*Property = Video*/
const Value V22575_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_330 = {S47, 0, &V22575_330_57328, &P22575_329};	/*Property = Viewport*/
const Value V22575_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_332 = {S48, 0, &V22575_332_57340, &P22575_330};	/*Property = WebWorkers*/
const Value V22575_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22575_333 = {S49, 0, &V22575_333_57343, &P22575_332};	/*Property = Xhr2*/
const Profile P22575 = {"22575", &P22575_333};	/*Profile ID - 22575*/
const Value V18388_1_1 = {S2, NULL};	/*Value = True*/
const Property P18388_1 = {S50, 0, &V18388_1_1, NULL};	/*Property = IsMobile*/
const Value V18388_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18388_5 = {S52, 0, &V18388_5_1227, &P18388_1};	/*Property = ScreenPixelsHeight*/
const Value V18388_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18388_4 = {S54, 0, &V18388_4_1229, &P18388_5};	/*Property = ScreenPixelsWidth*/
const Profile P18388 = {"18388", &P18388_4};	/*Profile ID - 18388*/
const Value V15541_1_1 = {S2, NULL};	/*Value = True*/
const Property P15541_1 = {S50, 0, &V15541_1_1, NULL};	/*Property = IsMobile*/
const Value V15541_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P15541_5 = {S52, 0, &V15541_5_1269, &P15541_1};	/*Property = ScreenPixelsHeight*/
const Value V15541_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15541_4 = {S54, 0, &V15541_4_1229, &P15541_5};	/*Property = ScreenPixelsWidth*/
const Profile P15541 = {"15541", &P15541_4};	/*Profile ID - 15541*/
const Value V24300_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_279 = {S1, 0, &V24300_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24300_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_282 = {S3, 0, &V24300_282_57155, &P24300_279};	/*Property = BlobBuilder*/
const Value V24300_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_350 = {S4, 0, &V24300_350_57446, &P24300_282};	/*Property = Canvas*/
const Value V24300_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_283 = {S5, 0, &V24300_283_57161, &P24300_350};	/*Property = CssBackground*/
const Value V24300_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_284 = {S6, 0, &V24300_284_57164, &P24300_283};	/*Property = CssBorderImage*/
const Value V24300_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_285 = {S7, 0, &V24300_285_57167, &P24300_284};	/*Property = CssCanvas*/
const Value V24300_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_286 = {S8, 0, &V24300_286_57170, &P24300_285};	/*Property = CssColor*/
const Value V24300_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_237 = {S9, 0, &V24300_237_55786, &P24300_286};	/*Property = CssColumn*/
const Value V24300_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_287 = {S10, 0, &V24300_287_57173, &P24300_237};	/*Property = CssFlexbox*/
const Value V24300_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_288 = {S11, 0, &V24300_288_57176, &P24300_287};	/*Property = CssFont*/
const Value V24300_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_289 = {S12, 0, &V24300_289_86899, &P24300_288};	/*Property = CssImages*/
const Value V24300_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_291 = {S13, 0, &V24300_291_57182, &P24300_289};	/*Property = CssMediaQueries*/
const Value V24300_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_292 = {S14, 0, &V24300_292_57185, &P24300_291};	/*Property = CssMinMax*/
const Value V24300_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_293 = {S15, 0, &V24300_293_57188, &P24300_292};	/*Property = CssOverflow*/
const Value V24300_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_294 = {S16, 0, &V24300_294_57191, &P24300_293};	/*Property = CssPosition*/
const Value V24300_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_295 = {S17, 0, &V24300_295_57194, &P24300_294};	/*Property = CssText*/
const Value V24300_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_297 = {S18, 0, &V24300_297_57197, &P24300_295};	/*Property = CssTransforms*/
const Value V24300_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_241 = {S19, 0, &V24300_241_55801, &P24300_297};	/*Property = CssTransitions*/
const Value V24300_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_298 = {S20, 0, &V24300_298_57208, &P24300_241};	/*Property = CssUI*/
const Value V24300_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_300 = {S21, 0, &V24300_300_57217, &P24300_298};	/*Property = DataSet*/
const Value V24300_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_301 = {S22, 0, &V24300_301_57220, &P24300_300};	/*Property = DataUrl*/
const Value V24300_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_302 = {S23, 0, &V24300_302_57223, &P24300_301};	/*Property = DeviceOrientation*/
const Value V24300_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_304 = {S24, 0, &V24300_304_57229, &P24300_302};	/*Property = FileReader*/
const Value V24300_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_305 = {S25, 0, &V24300_305_57232, &P24300_304};	/*Property = FileSaver*/
const Value V24300_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_306 = {S26, 0, &V24300_306_57238, &P24300_305};	/*Property = FileWriter*/
const Value V24300_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_307 = {S27, 0, &V24300_307_57241, &P24300_306};	/*Property = FormData*/
const Value V24300_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_309 = {S28, 0, &V24300_309_57247, &P24300_307};	/*Property = Fullscreen*/
const Value V24300_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_310 = {S29, 0, &V24300_310_57250, &P24300_309};	/*Property = GeoLocation*/
const Value V24300_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_312 = {S30, 0, &V24300_312_57256, &P24300_310};	/*Property = History*/
const Value V24300_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_314 = {S31, 0, &V24300_314_57262, &P24300_312};	/*Property = Html5*/
const Value V24300_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_313 = {S32, 0, &V24300_313_57259, &P24300_314};	/*Property = Html-Media-Capture*/
const Value V24300_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_315 = {S33, 0, &V24300_315_57265, &P24300_313};	/*Property = Iframe*/
const Value V24300_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_316 = {S34, 0, &V24300_316_57268, &P24300_315};	/*Property = IndexedDB*/
const Value V24300_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_317 = {S35, 0, &V24300_317_57271, &P24300_316};	/*Property = Json*/
const Value V24300_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24300_15 = {S37, 0, &V24300_15_54167, &P24300_317};	/*Property = LayoutEngine*/
const Value V24300_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_344 = {S38, 0, &V24300_344_57274, &P24300_15};	/*Property = Masking*/
const Value V24300_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_320 = {S39, 0, &V24300_320_57292, &P24300_344};	/*Property = PostMessage*/
const Value V24300_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_321 = {S40, 0, &V24300_321_57295, &P24300_320};	/*Property = Progress*/
const Value V24300_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_322 = {S41, 0, &V24300_322_57298, &P24300_321};	/*Property = Prompts*/
const Value V24300_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_323 = {S42, 0, &V24300_323_57301, &P24300_322};	/*Property = Selector*/
const Value V24300_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_324 = {S43, 0, &V24300_324_57307, &P24300_323};	/*Property = Svg*/
const Value V24300_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_327 = {S44, 0, &V24300_327_57316, &P24300_324};	/*Property = TouchEvents*/
const Value V24300_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_328 = {S45, 0, &V24300_328_57319, &P24300_327};	/*Property = Track*/
const Value V24300_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_329 = {S46, 0, &V24300_329_57325, &P24300_328};	/*Property = Video*/
const Value V24300_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_330 = {S47, 0, &V24300_330_57328, &P24300_329};	/*Property = Viewport*/
const Value V24300_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_332 = {S48, 0, &V24300_332_57340, &P24300_330};	/*Property = WebWorkers*/
const Value V24300_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24300_333 = {S49, 0, &V24300_333_57343, &P24300_332};	/*Property = Xhr2*/
const Profile P24300 = {"24300", &P24300_333};	/*Profile ID - 24300*/
const Value V24223_1_2 = {S0, NULL};	/*Value = False*/
const Property P24223_1 = {S50, 0, &V24223_1_2, NULL};	/*Property = IsMobile*/
const Value V24223_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24223_5 = {S52, 0, &V24223_5_60417, &P24223_1};	/*Property = ScreenPixelsHeight*/
const Value V24223_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24223_4 = {S54, 0, &V24223_4_57095, &P24223_5};	/*Property = ScreenPixelsWidth*/
const Profile P24223 = {"24223", &P24223_4};	/*Profile ID - 24223*/
const Value V24219_1_2 = {S0, NULL};	/*Value = False*/
const Property P24219_1 = {S50, 0, &V24219_1_2, NULL};	/*Property = IsMobile*/
const Value V24219_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24219_5 = {S52, 0, &V24219_5_60417, &P24219_1};	/*Property = ScreenPixelsHeight*/
const Value V24219_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24219_4 = {S54, 0, &V24219_4_57095, &P24219_5};	/*Property = ScreenPixelsWidth*/
const Profile P24219 = {"24219", &P24219_4};	/*Profile ID - 24219*/
const Value V24225_1_2 = {S0, NULL};	/*Value = False*/
const Property P24225_1 = {S50, 0, &V24225_1_2, NULL};	/*Property = IsMobile*/
const Value V24225_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24225_5 = {S52, 0, &V24225_5_60417, &P24225_1};	/*Property = ScreenPixelsHeight*/
const Value V24225_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24225_4 = {S54, 0, &V24225_4_57095, &P24225_5};	/*Property = ScreenPixelsWidth*/
const Profile P24225 = {"24225", &P24225_4};	/*Profile ID - 24225*/
const Value V24299_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_279 = {S1, 0, &V24299_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24299_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_282 = {S3, 0, &V24299_282_57155, &P24299_279};	/*Property = BlobBuilder*/
const Value V24299_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_350 = {S4, 0, &V24299_350_57446, &P24299_282};	/*Property = Canvas*/
const Value V24299_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_283 = {S5, 0, &V24299_283_57161, &P24299_350};	/*Property = CssBackground*/
const Value V24299_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_284 = {S6, 0, &V24299_284_57164, &P24299_283};	/*Property = CssBorderImage*/
const Value V24299_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_285 = {S7, 0, &V24299_285_57167, &P24299_284};	/*Property = CssCanvas*/
const Value V24299_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_286 = {S8, 0, &V24299_286_57170, &P24299_285};	/*Property = CssColor*/
const Value V24299_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_237 = {S9, 0, &V24299_237_55786, &P24299_286};	/*Property = CssColumn*/
const Value V24299_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_287 = {S10, 0, &V24299_287_57173, &P24299_237};	/*Property = CssFlexbox*/
const Value V24299_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_288 = {S11, 0, &V24299_288_57176, &P24299_287};	/*Property = CssFont*/
const Value V24299_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_289 = {S12, 0, &V24299_289_86899, &P24299_288};	/*Property = CssImages*/
const Value V24299_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_291 = {S13, 0, &V24299_291_57182, &P24299_289};	/*Property = CssMediaQueries*/
const Value V24299_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_292 = {S14, 0, &V24299_292_57185, &P24299_291};	/*Property = CssMinMax*/
const Value V24299_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_293 = {S15, 0, &V24299_293_57188, &P24299_292};	/*Property = CssOverflow*/
const Value V24299_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_294 = {S16, 0, &V24299_294_57191, &P24299_293};	/*Property = CssPosition*/
const Value V24299_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_295 = {S17, 0, &V24299_295_57194, &P24299_294};	/*Property = CssText*/
const Value V24299_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_297 = {S18, 0, &V24299_297_57197, &P24299_295};	/*Property = CssTransforms*/
const Value V24299_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_241 = {S19, 0, &V24299_241_55801, &P24299_297};	/*Property = CssTransitions*/
const Value V24299_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_298 = {S20, 0, &V24299_298_57208, &P24299_241};	/*Property = CssUI*/
const Value V24299_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_300 = {S21, 0, &V24299_300_57217, &P24299_298};	/*Property = DataSet*/
const Value V24299_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_301 = {S22, 0, &V24299_301_57220, &P24299_300};	/*Property = DataUrl*/
const Value V24299_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_302 = {S23, 0, &V24299_302_57223, &P24299_301};	/*Property = DeviceOrientation*/
const Value V24299_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_304 = {S24, 0, &V24299_304_57229, &P24299_302};	/*Property = FileReader*/
const Value V24299_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_305 = {S25, 0, &V24299_305_57232, &P24299_304};	/*Property = FileSaver*/
const Value V24299_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_306 = {S26, 0, &V24299_306_57238, &P24299_305};	/*Property = FileWriter*/
const Value V24299_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_307 = {S27, 0, &V24299_307_57241, &P24299_306};	/*Property = FormData*/
const Value V24299_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_309 = {S28, 0, &V24299_309_57247, &P24299_307};	/*Property = Fullscreen*/
const Value V24299_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_310 = {S29, 0, &V24299_310_57250, &P24299_309};	/*Property = GeoLocation*/
const Value V24299_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_312 = {S30, 0, &V24299_312_57256, &P24299_310};	/*Property = History*/
const Value V24299_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_314 = {S31, 0, &V24299_314_57262, &P24299_312};	/*Property = Html5*/
const Value V24299_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_313 = {S32, 0, &V24299_313_57259, &P24299_314};	/*Property = Html-Media-Capture*/
const Value V24299_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_315 = {S33, 0, &V24299_315_57265, &P24299_313};	/*Property = Iframe*/
const Value V24299_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_316 = {S34, 0, &V24299_316_57268, &P24299_315};	/*Property = IndexedDB*/
const Value V24299_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_317 = {S35, 0, &V24299_317_57271, &P24299_316};	/*Property = Json*/
const Value V24299_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24299_15 = {S37, 0, &V24299_15_54167, &P24299_317};	/*Property = LayoutEngine*/
const Value V24299_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_344 = {S38, 0, &V24299_344_57274, &P24299_15};	/*Property = Masking*/
const Value V24299_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_320 = {S39, 0, &V24299_320_57292, &P24299_344};	/*Property = PostMessage*/
const Value V24299_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_321 = {S40, 0, &V24299_321_57295, &P24299_320};	/*Property = Progress*/
const Value V24299_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_322 = {S41, 0, &V24299_322_57298, &P24299_321};	/*Property = Prompts*/
const Value V24299_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_323 = {S42, 0, &V24299_323_57301, &P24299_322};	/*Property = Selector*/
const Value V24299_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_324 = {S43, 0, &V24299_324_57307, &P24299_323};	/*Property = Svg*/
const Value V24299_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_327 = {S44, 0, &V24299_327_57316, &P24299_324};	/*Property = TouchEvents*/
const Value V24299_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_328 = {S45, 0, &V24299_328_57319, &P24299_327};	/*Property = Track*/
const Value V24299_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_329 = {S46, 0, &V24299_329_57325, &P24299_328};	/*Property = Video*/
const Value V24299_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_330 = {S47, 0, &V24299_330_57328, &P24299_329};	/*Property = Viewport*/
const Value V24299_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_332 = {S48, 0, &V24299_332_57340, &P24299_330};	/*Property = WebWorkers*/
const Value V24299_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24299_333 = {S49, 0, &V24299_333_57343, &P24299_332};	/*Property = Xhr2*/
const Profile P24299 = {"24299", &P24299_333};	/*Profile ID - 24299*/
const Value V24224_1_2 = {S0, NULL};	/*Value = False*/
const Property P24224_1 = {S50, 0, &V24224_1_2, NULL};	/*Property = IsMobile*/
const Value V24224_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24224_5 = {S52, 0, &V24224_5_60417, &P24224_1};	/*Property = ScreenPixelsHeight*/
const Value V24224_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24224_4 = {S54, 0, &V24224_4_57095, &P24224_5};	/*Property = ScreenPixelsWidth*/
const Profile P24224 = {"24224", &P24224_4};	/*Profile ID - 24224*/
const Value V24222_1_2 = {S0, NULL};	/*Value = False*/
const Property P24222_1 = {S50, 0, &V24222_1_2, NULL};	/*Property = IsMobile*/
const Value V24222_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24222_5 = {S52, 0, &V24222_5_60417, &P24222_1};	/*Property = ScreenPixelsHeight*/
const Value V24222_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24222_4 = {S54, 0, &V24222_4_57095, &P24222_5};	/*Property = ScreenPixelsWidth*/
const Profile P24222 = {"24222", &P24222_4};	/*Profile ID - 24222*/
const Value V24220_1_2 = {S0, NULL};	/*Value = False*/
const Property P24220_1 = {S50, 0, &V24220_1_2, NULL};	/*Property = IsMobile*/
const Value V24220_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24220_5 = {S52, 0, &V24220_5_60417, &P24220_1};	/*Property = ScreenPixelsHeight*/
const Value V24220_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24220_4 = {S54, 0, &V24220_4_57095, &P24220_5};	/*Property = ScreenPixelsWidth*/
const Profile P24220 = {"24220", &P24220_4};	/*Profile ID - 24220*/
const Value V24297_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_279 = {S1, 0, &V24297_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24297_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_282 = {S3, 0, &V24297_282_57155, &P24297_279};	/*Property = BlobBuilder*/
const Value V24297_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_350 = {S4, 0, &V24297_350_57446, &P24297_282};	/*Property = Canvas*/
const Value V24297_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_283 = {S5, 0, &V24297_283_57161, &P24297_350};	/*Property = CssBackground*/
const Value V24297_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_284 = {S6, 0, &V24297_284_57164, &P24297_283};	/*Property = CssBorderImage*/
const Value V24297_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_285 = {S7, 0, &V24297_285_57167, &P24297_284};	/*Property = CssCanvas*/
const Value V24297_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_286 = {S8, 0, &V24297_286_57170, &P24297_285};	/*Property = CssColor*/
const Value V24297_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_237 = {S9, 0, &V24297_237_55786, &P24297_286};	/*Property = CssColumn*/
const Value V24297_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_287 = {S10, 0, &V24297_287_57173, &P24297_237};	/*Property = CssFlexbox*/
const Value V24297_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_288 = {S11, 0, &V24297_288_57176, &P24297_287};	/*Property = CssFont*/
const Value V24297_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_289 = {S12, 0, &V24297_289_86899, &P24297_288};	/*Property = CssImages*/
const Value V24297_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_291 = {S13, 0, &V24297_291_57182, &P24297_289};	/*Property = CssMediaQueries*/
const Value V24297_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_292 = {S14, 0, &V24297_292_57185, &P24297_291};	/*Property = CssMinMax*/
const Value V24297_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_293 = {S15, 0, &V24297_293_57188, &P24297_292};	/*Property = CssOverflow*/
const Value V24297_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_294 = {S16, 0, &V24297_294_57191, &P24297_293};	/*Property = CssPosition*/
const Value V24297_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_295 = {S17, 0, &V24297_295_57194, &P24297_294};	/*Property = CssText*/
const Value V24297_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_297 = {S18, 0, &V24297_297_57197, &P24297_295};	/*Property = CssTransforms*/
const Value V24297_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_241 = {S19, 0, &V24297_241_55801, &P24297_297};	/*Property = CssTransitions*/
const Value V24297_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_298 = {S20, 0, &V24297_298_57208, &P24297_241};	/*Property = CssUI*/
const Value V24297_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_300 = {S21, 0, &V24297_300_57217, &P24297_298};	/*Property = DataSet*/
const Value V24297_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_301 = {S22, 0, &V24297_301_57220, &P24297_300};	/*Property = DataUrl*/
const Value V24297_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_302 = {S23, 0, &V24297_302_57223, &P24297_301};	/*Property = DeviceOrientation*/
const Value V24297_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_304 = {S24, 0, &V24297_304_57229, &P24297_302};	/*Property = FileReader*/
const Value V24297_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_305 = {S25, 0, &V24297_305_57232, &P24297_304};	/*Property = FileSaver*/
const Value V24297_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_306 = {S26, 0, &V24297_306_57238, &P24297_305};	/*Property = FileWriter*/
const Value V24297_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_307 = {S27, 0, &V24297_307_57241, &P24297_306};	/*Property = FormData*/
const Value V24297_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_309 = {S28, 0, &V24297_309_57247, &P24297_307};	/*Property = Fullscreen*/
const Value V24297_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_310 = {S29, 0, &V24297_310_57250, &P24297_309};	/*Property = GeoLocation*/
const Value V24297_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_312 = {S30, 0, &V24297_312_57256, &P24297_310};	/*Property = History*/
const Value V24297_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_314 = {S31, 0, &V24297_314_57262, &P24297_312};	/*Property = Html5*/
const Value V24297_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_313 = {S32, 0, &V24297_313_57259, &P24297_314};	/*Property = Html-Media-Capture*/
const Value V24297_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_315 = {S33, 0, &V24297_315_57265, &P24297_313};	/*Property = Iframe*/
const Value V24297_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_316 = {S34, 0, &V24297_316_57268, &P24297_315};	/*Property = IndexedDB*/
const Value V24297_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_317 = {S35, 0, &V24297_317_57271, &P24297_316};	/*Property = Json*/
const Value V24297_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24297_15 = {S37, 0, &V24297_15_54167, &P24297_317};	/*Property = LayoutEngine*/
const Value V24297_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_344 = {S38, 0, &V24297_344_57274, &P24297_15};	/*Property = Masking*/
const Value V24297_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_320 = {S39, 0, &V24297_320_57292, &P24297_344};	/*Property = PostMessage*/
const Value V24297_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_321 = {S40, 0, &V24297_321_57295, &P24297_320};	/*Property = Progress*/
const Value V24297_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_322 = {S41, 0, &V24297_322_57298, &P24297_321};	/*Property = Prompts*/
const Value V24297_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_323 = {S42, 0, &V24297_323_57301, &P24297_322};	/*Property = Selector*/
const Value V24297_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_324 = {S43, 0, &V24297_324_57307, &P24297_323};	/*Property = Svg*/
const Value V24297_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_327 = {S44, 0, &V24297_327_57316, &P24297_324};	/*Property = TouchEvents*/
const Value V24297_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_328 = {S45, 0, &V24297_328_57319, &P24297_327};	/*Property = Track*/
const Value V24297_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_329 = {S46, 0, &V24297_329_57325, &P24297_328};	/*Property = Video*/
const Value V24297_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_330 = {S47, 0, &V24297_330_57328, &P24297_329};	/*Property = Viewport*/
const Value V24297_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_332 = {S48, 0, &V24297_332_57340, &P24297_330};	/*Property = WebWorkers*/
const Value V24297_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24297_333 = {S49, 0, &V24297_333_57343, &P24297_332};	/*Property = Xhr2*/
const Profile P24297 = {"24297", &P24297_333};	/*Profile ID - 24297*/
const Value V18263_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18263_279 = {S1, 0, &V18263_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18263_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18263_282 = {S3, 0, &V18263_282_57154, &P18263_279};	/*Property = BlobBuilder*/
const Value V18263_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18263_350 = {S4, 0, &V18263_350_57444, &P18263_282};	/*Property = Canvas*/
const Value V18263_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18263_283 = {S5, 0, &V18263_283_57159, &P18263_350};	/*Property = CssBackground*/
const Value V18263_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18263_284 = {S6, 0, &V18263_284_57163, &P18263_283};	/*Property = CssBorderImage*/
const Value V18263_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18263_285 = {S7, 0, &V18263_285_57166, &P18263_284};	/*Property = CssCanvas*/
const Value V18263_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18263_286 = {S8, 0, &V18263_286_57168, &P18263_285};	/*Property = CssColor*/
const Value V18263_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18263_237 = {S9, 0, &V18263_237_55786, &P18263_286};	/*Property = CssColumn*/
const Value V18263_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18263_287 = {S10, 0, &V18263_287_57172, &P18263_237};	/*Property = CssFlexbox*/
const Value V18263_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18263_288 = {S11, 0, &V18263_288_57175, &P18263_287};	/*Property = CssFont*/
const Value V18263_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18263_289 = {S12, 0, &V18263_289_57759, &P18263_288};	/*Property = CssImages*/
const Value V18263_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18263_291 = {S13, 0, &V18263_291_57181, &P18263_289};	/*Property = CssMediaQueries*/
const Value V18263_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18263_292 = {S14, 0, &V18263_292_57183, &P18263_291};	/*Property = CssMinMax*/
const Value V18263_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18263_293 = {S15, 0, &V18263_293_57187, &P18263_292};	/*Property = CssOverflow*/
const Value V18263_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18263_294 = {S16, 0, &V18263_294_57190, &P18263_293};	/*Property = CssPosition*/
const Value V18263_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18263_295 = {S17, 0, &V18263_295_57192, &P18263_294};	/*Property = CssText*/
const Value V18263_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18263_297 = {S18, 0, &V18263_297_57195, &P18263_295};	/*Property = CssTransforms*/
const Value V18263_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18263_241 = {S19, 0, &V18263_241_55799, &P18263_297};	/*Property = CssTransitions*/
const Value V18263_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18263_298 = {S20, 0, &V18263_298_57206, &P18263_241};	/*Property = CssUI*/
const Value V18263_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18263_300 = {S21, 0, &V18263_300_57216, &P18263_298};	/*Property = DataSet*/
const Value V18263_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18263_301 = {S22, 0, &V18263_301_57218, &P18263_300};	/*Property = DataUrl*/
const Value V18263_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18263_302 = {S23, 0, &V18263_302_57222, &P18263_301};	/*Property = DeviceOrientation*/
const Value V18263_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18263_304 = {S24, 0, &V18263_304_57228, &P18263_302};	/*Property = FileReader*/
const Value V18263_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18263_305 = {S25, 0, &V18263_305_57231, &P18263_304};	/*Property = FileSaver*/
const Value V18263_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18263_306 = {S26, 0, &V18263_306_57237, &P18263_305};	/*Property = FileWriter*/
const Value V18263_307_57240 = {S0, NULL};	/*Value = False*/
const Property P18263_307 = {S27, 0, &V18263_307_57240, &P18263_306};	/*Property = FormData*/
const Value V18263_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18263_309 = {S28, 0, &V18263_309_57246, &P18263_307};	/*Property = Fullscreen*/
const Value V18263_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18263_310 = {S29, 0, &V18263_310_57248, &P18263_309};	/*Property = GeoLocation*/
const Value V18263_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18263_312 = {S30, 0, &V18263_312_57255, &P18263_310};	/*Property = History*/
const Value V18263_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18263_314 = {S31, 0, &V18263_314_57261, &P18263_312};	/*Property = Html5*/
const Value V18263_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18263_313 = {S32, 0, &V18263_313_57258, &P18263_314};	/*Property = Html-Media-Capture*/
const Value V18263_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18263_315 = {S33, 0, &V18263_315_57264, &P18263_313};	/*Property = Iframe*/
const Value V18263_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18263_316 = {S34, 0, &V18263_316_57267, &P18263_315};	/*Property = IndexedDB*/
const Value V18263_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18263_317 = {S35, 0, &V18263_317_57269, &P18263_316};	/*Property = Json*/
const Value V18263_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P18263_15 = {S37, 0, &V18263_15_30, &P18263_317};	/*Property = LayoutEngine*/
const Value V18263_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18263_344 = {S38, 0, &V18263_344_57272, &P18263_15};	/*Property = Masking*/
const Value V18263_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18263_320 = {S39, 0, &V18263_320_57290, &P18263_344};	/*Property = PostMessage*/
const Value V18263_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18263_321 = {S40, 0, &V18263_321_57293, &P18263_320};	/*Property = Progress*/
const Value V18263_322_57297 = {S0, NULL};	/*Value = False*/
const Property P18263_322 = {S41, 0, &V18263_322_57297, &P18263_321};	/*Property = Prompts*/
const Value V18263_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18263_323 = {S42, 0, &V18263_323_57299, &P18263_322};	/*Property = Selector*/
const Value V18263_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18263_324 = {S43, 0, &V18263_324_57306, &P18263_323};	/*Property = Svg*/
const Value V18263_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18263_327 = {S44, 0, &V18263_327_57315, &P18263_324};	/*Property = TouchEvents*/
const Value V18263_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18263_328 = {S45, 0, &V18263_328_57318, &P18263_327};	/*Property = Track*/
const Value V18263_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18263_329 = {S46, 0, &V18263_329_57323, &P18263_328};	/*Property = Video*/
const Value V18263_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18263_330 = {S47, 0, &V18263_330_57326, &P18263_329};	/*Property = Viewport*/
const Value V18263_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18263_332 = {S48, 0, &V18263_332_57339, &P18263_330};	/*Property = WebWorkers*/
const Value V18263_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18263_333 = {S49, 0, &V18263_333_57342, &P18263_332};	/*Property = Xhr2*/
const Profile P18263 = {"18263", &P18263_333};	/*Profile ID - 18263*/
const Profile P18148 = {"18148", NULL};	/*Profile ID - 18148*/
const Value V24221_1_2 = {S0, NULL};	/*Value = False*/
const Property P24221_1 = {S50, 0, &V24221_1_2, NULL};	/*Property = IsMobile*/
const Value V24221_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24221_5 = {S52, 0, &V24221_5_60417, &P24221_1};	/*Property = ScreenPixelsHeight*/
const Value V24221_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24221_4 = {S54, 0, &V24221_4_57095, &P24221_5};	/*Property = ScreenPixelsWidth*/
const Profile P24221 = {"24221", &P24221_4};	/*Profile ID - 24221*/
const Value V24019_1_1 = {S2, NULL};	/*Value = True*/
const Property P24019_1 = {S50, 0, &V24019_1_1, NULL};	/*Property = IsMobile*/
const Value V24019_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P24019_5 = {S52, 0, &V24019_5_1458, &P24019_1};	/*Property = ScreenPixelsHeight*/
const Value V24019_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P24019_4 = {S54, 0, &V24019_4_1460, &P24019_5};	/*Property = ScreenPixelsWidth*/
const Profile P24019 = {"24019", &P24019_4};	/*Profile ID - 24019*/
const Value V18699_1_1 = {S2, NULL};	/*Value = True*/
const Property P18699_1 = {S50, 0, &V18699_1_1, NULL};	/*Property = IsMobile*/
const Value V18699_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18699_5 = {S52, 0, &V18699_5_1227, &P18699_1};	/*Property = ScreenPixelsHeight*/
const Value V18699_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P18699_4 = {S54, 0, &V18699_4_1460, &P18699_5};	/*Property = ScreenPixelsWidth*/
const Profile P18699 = {"18699", &P18699_4};	/*Profile ID - 18699*/
const Value V22830_1_1 = {S2, NULL};	/*Value = True*/
const Property P22830_1 = {S50, 0, &V22830_1_1, NULL};	/*Property = IsMobile*/
const Value V22830_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22830_5 = {S52, 0, &V22830_5_37569, &P22830_1};	/*Property = ScreenPixelsHeight*/
const Value V22830_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22830_4 = {S54, 0, &V22830_4_5645, &P22830_5};	/*Property = ScreenPixelsWidth*/
const Profile P22830 = {"22830", &P22830_4};	/*Profile ID - 22830*/
const Value V16976_1_1 = {S2, NULL};	/*Value = True*/
const Property P16976_1 = {S50, 0, &V16976_1_1, NULL};	/*Property = IsMobile*/
const Value V16976_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16976_5 = {S52, 0, &V16976_5_1227, &P16976_1};	/*Property = ScreenPixelsHeight*/
const Value V16976_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P16976_4 = {S54, 0, &V16976_4_1477, &P16976_5};	/*Property = ScreenPixelsWidth*/
const Profile P16976 = {"16976", &P16976_4};	/*Profile ID - 16976*/
const Value V18391_1_1 = {S2, NULL};	/*Value = True*/
const Property P18391_1 = {S50, 0, &V18391_1_1, NULL};	/*Property = IsMobile*/
const Value V18391_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P18391_5 = {S52, 0, &V18391_5_37, &P18391_1};	/*Property = ScreenPixelsHeight*/
const Value V18391_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P18391_4 = {S54, 0, &V18391_4_36, &P18391_5};	/*Property = ScreenPixelsWidth*/
const Profile P18391 = {"18391", &P18391_4};	/*Profile ID - 18391*/
const Value V18525_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_279 = {S1, 0, &V18525_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18525_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_282 = {S3, 0, &V18525_282_57155, &P18525_279};	/*Property = BlobBuilder*/
const Value V18525_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_350 = {S4, 0, &V18525_350_57446, &P18525_282};	/*Property = Canvas*/
const Value V18525_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_283 = {S5, 0, &V18525_283_57161, &P18525_350};	/*Property = CssBackground*/
const Value V18525_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_284 = {S6, 0, &V18525_284_57164, &P18525_283};	/*Property = CssBorderImage*/
const Value V18525_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_285 = {S7, 0, &V18525_285_57167, &P18525_284};	/*Property = CssCanvas*/
const Value V18525_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_286 = {S8, 0, &V18525_286_57170, &P18525_285};	/*Property = CssColor*/
const Value V18525_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_237 = {S9, 0, &V18525_237_55786, &P18525_286};	/*Property = CssColumn*/
const Value V18525_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_287 = {S10, 0, &V18525_287_57173, &P18525_237};	/*Property = CssFlexbox*/
const Value V18525_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_288 = {S11, 0, &V18525_288_57176, &P18525_287};	/*Property = CssFont*/
const Value V18525_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_289 = {S12, 0, &V18525_289_86899, &P18525_288};	/*Property = CssImages*/
const Value V18525_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_291 = {S13, 0, &V18525_291_57182, &P18525_289};	/*Property = CssMediaQueries*/
const Value V18525_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_292 = {S14, 0, &V18525_292_57185, &P18525_291};	/*Property = CssMinMax*/
const Value V18525_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_293 = {S15, 0, &V18525_293_57188, &P18525_292};	/*Property = CssOverflow*/
const Value V18525_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_294 = {S16, 0, &V18525_294_57191, &P18525_293};	/*Property = CssPosition*/
const Value V18525_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_295 = {S17, 0, &V18525_295_57194, &P18525_294};	/*Property = CssText*/
const Value V18525_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_297 = {S18, 0, &V18525_297_57197, &P18525_295};	/*Property = CssTransforms*/
const Value V18525_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_241 = {S19, 0, &V18525_241_55801, &P18525_297};	/*Property = CssTransitions*/
const Value V18525_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_298 = {S20, 0, &V18525_298_57208, &P18525_241};	/*Property = CssUI*/
const Value V18525_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_300 = {S21, 0, &V18525_300_57217, &P18525_298};	/*Property = DataSet*/
const Value V18525_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_301 = {S22, 0, &V18525_301_57220, &P18525_300};	/*Property = DataUrl*/
const Value V18525_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_302 = {S23, 0, &V18525_302_57223, &P18525_301};	/*Property = DeviceOrientation*/
const Value V18525_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_304 = {S24, 0, &V18525_304_57229, &P18525_302};	/*Property = FileReader*/
const Value V18525_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_305 = {S25, 0, &V18525_305_57232, &P18525_304};	/*Property = FileSaver*/
const Value V18525_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_306 = {S26, 0, &V18525_306_57238, &P18525_305};	/*Property = FileWriter*/
const Value V18525_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_307 = {S27, 0, &V18525_307_57241, &P18525_306};	/*Property = FormData*/
const Value V18525_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_309 = {S28, 0, &V18525_309_57247, &P18525_307};	/*Property = Fullscreen*/
const Value V18525_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_310 = {S29, 0, &V18525_310_57250, &P18525_309};	/*Property = GeoLocation*/
const Value V18525_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_312 = {S30, 0, &V18525_312_57256, &P18525_310};	/*Property = History*/
const Value V18525_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_314 = {S31, 0, &V18525_314_57262, &P18525_312};	/*Property = Html5*/
const Value V18525_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_313 = {S32, 0, &V18525_313_57259, &P18525_314};	/*Property = Html-Media-Capture*/
const Value V18525_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_315 = {S33, 0, &V18525_315_57265, &P18525_313};	/*Property = Iframe*/
const Value V18525_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_316 = {S34, 0, &V18525_316_57268, &P18525_315};	/*Property = IndexedDB*/
const Value V18525_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_317 = {S35, 0, &V18525_317_57271, &P18525_316};	/*Property = Json*/
const Value V18525_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18525_15 = {S37, 0, &V18525_15_31, &P18525_317};	/*Property = LayoutEngine*/
const Value V18525_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_344 = {S38, 0, &V18525_344_57274, &P18525_15};	/*Property = Masking*/
const Value V18525_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_320 = {S39, 0, &V18525_320_57292, &P18525_344};	/*Property = PostMessage*/
const Value V18525_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_321 = {S40, 0, &V18525_321_57295, &P18525_320};	/*Property = Progress*/
const Value V18525_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_322 = {S41, 0, &V18525_322_57298, &P18525_321};	/*Property = Prompts*/
const Value V18525_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_323 = {S42, 0, &V18525_323_57301, &P18525_322};	/*Property = Selector*/
const Value V18525_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_324 = {S43, 0, &V18525_324_57307, &P18525_323};	/*Property = Svg*/
const Value V18525_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_327 = {S44, 0, &V18525_327_57316, &P18525_324};	/*Property = TouchEvents*/
const Value V18525_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_328 = {S45, 0, &V18525_328_57319, &P18525_327};	/*Property = Track*/
const Value V18525_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_329 = {S46, 0, &V18525_329_57325, &P18525_328};	/*Property = Video*/
const Value V18525_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_330 = {S47, 0, &V18525_330_57328, &P18525_329};	/*Property = Viewport*/
const Value V18525_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_332 = {S48, 0, &V18525_332_57340, &P18525_330};	/*Property = WebWorkers*/
const Value V18525_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18525_333 = {S49, 0, &V18525_333_57343, &P18525_332};	/*Property = Xhr2*/
const Profile P18525 = {"18525", &P18525_333};	/*Profile ID - 18525*/
const Value V24422_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_279 = {S1, 0, &V24422_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24422_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_282 = {S3, 0, &V24422_282_57155, &P24422_279};	/*Property = BlobBuilder*/
const Value V24422_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_350 = {S4, 0, &V24422_350_57446, &P24422_282};	/*Property = Canvas*/
const Value V24422_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_283 = {S5, 0, &V24422_283_57161, &P24422_350};	/*Property = CssBackground*/
const Value V24422_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_284 = {S6, 0, &V24422_284_57164, &P24422_283};	/*Property = CssBorderImage*/
const Value V24422_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_285 = {S7, 0, &V24422_285_57167, &P24422_284};	/*Property = CssCanvas*/
const Value V24422_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_286 = {S8, 0, &V24422_286_57170, &P24422_285};	/*Property = CssColor*/
const Value V24422_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_237 = {S9, 0, &V24422_237_55786, &P24422_286};	/*Property = CssColumn*/
const Value V24422_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_287 = {S10, 0, &V24422_287_57173, &P24422_237};	/*Property = CssFlexbox*/
const Value V24422_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_288 = {S11, 0, &V24422_288_57176, &P24422_287};	/*Property = CssFont*/
const Value V24422_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_289 = {S12, 0, &V24422_289_86899, &P24422_288};	/*Property = CssImages*/
const Value V24422_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_291 = {S13, 0, &V24422_291_57182, &P24422_289};	/*Property = CssMediaQueries*/
const Value V24422_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_292 = {S14, 0, &V24422_292_57185, &P24422_291};	/*Property = CssMinMax*/
const Value V24422_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_293 = {S15, 0, &V24422_293_57188, &P24422_292};	/*Property = CssOverflow*/
const Value V24422_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_294 = {S16, 0, &V24422_294_57191, &P24422_293};	/*Property = CssPosition*/
const Value V24422_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_295 = {S17, 0, &V24422_295_57194, &P24422_294};	/*Property = CssText*/
const Value V24422_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_297 = {S18, 0, &V24422_297_57197, &P24422_295};	/*Property = CssTransforms*/
const Value V24422_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_241 = {S19, 0, &V24422_241_55801, &P24422_297};	/*Property = CssTransitions*/
const Value V24422_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_298 = {S20, 0, &V24422_298_57208, &P24422_241};	/*Property = CssUI*/
const Value V24422_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_300 = {S21, 0, &V24422_300_57217, &P24422_298};	/*Property = DataSet*/
const Value V24422_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_301 = {S22, 0, &V24422_301_57220, &P24422_300};	/*Property = DataUrl*/
const Value V24422_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_302 = {S23, 0, &V24422_302_57223, &P24422_301};	/*Property = DeviceOrientation*/
const Value V24422_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_304 = {S24, 0, &V24422_304_57229, &P24422_302};	/*Property = FileReader*/
const Value V24422_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_305 = {S25, 0, &V24422_305_57232, &P24422_304};	/*Property = FileSaver*/
const Value V24422_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_306 = {S26, 0, &V24422_306_57238, &P24422_305};	/*Property = FileWriter*/
const Value V24422_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_307 = {S27, 0, &V24422_307_57241, &P24422_306};	/*Property = FormData*/
const Value V24422_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_309 = {S28, 0, &V24422_309_57247, &P24422_307};	/*Property = Fullscreen*/
const Value V24422_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_310 = {S29, 0, &V24422_310_57250, &P24422_309};	/*Property = GeoLocation*/
const Value V24422_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_312 = {S30, 0, &V24422_312_57256, &P24422_310};	/*Property = History*/
const Value V24422_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_314 = {S31, 0, &V24422_314_57262, &P24422_312};	/*Property = Html5*/
const Value V24422_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_313 = {S32, 0, &V24422_313_57259, &P24422_314};	/*Property = Html-Media-Capture*/
const Value V24422_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_315 = {S33, 0, &V24422_315_57265, &P24422_313};	/*Property = Iframe*/
const Value V24422_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_316 = {S34, 0, &V24422_316_57268, &P24422_315};	/*Property = IndexedDB*/
const Value V24422_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_317 = {S35, 0, &V24422_317_57271, &P24422_316};	/*Property = Json*/
const Value V24422_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24422_15 = {S37, 0, &V24422_15_28, &P24422_317};	/*Property = LayoutEngine*/
const Value V24422_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_344 = {S38, 0, &V24422_344_57274, &P24422_15};	/*Property = Masking*/
const Value V24422_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_320 = {S39, 0, &V24422_320_57292, &P24422_344};	/*Property = PostMessage*/
const Value V24422_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_321 = {S40, 0, &V24422_321_57295, &P24422_320};	/*Property = Progress*/
const Value V24422_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_322 = {S41, 0, &V24422_322_57298, &P24422_321};	/*Property = Prompts*/
const Value V24422_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_323 = {S42, 0, &V24422_323_57301, &P24422_322};	/*Property = Selector*/
const Value V24422_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_324 = {S43, 0, &V24422_324_57307, &P24422_323};	/*Property = Svg*/
const Value V24422_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_327 = {S44, 0, &V24422_327_57316, &P24422_324};	/*Property = TouchEvents*/
const Value V24422_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_328 = {S45, 0, &V24422_328_57319, &P24422_327};	/*Property = Track*/
const Value V24422_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_329 = {S46, 0, &V24422_329_57325, &P24422_328};	/*Property = Video*/
const Value V24422_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_330 = {S47, 0, &V24422_330_57328, &P24422_329};	/*Property = Viewport*/
const Value V24422_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_332 = {S48, 0, &V24422_332_57340, &P24422_330};	/*Property = WebWorkers*/
const Value V24422_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24422_333 = {S49, 0, &V24422_333_57343, &P24422_332};	/*Property = Xhr2*/
const Profile P24422 = {"24422", &P24422_333};	/*Profile ID - 24422*/
const Value V18542_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_279 = {S1, 0, &V18542_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18542_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_282 = {S3, 0, &V18542_282_57155, &P18542_279};	/*Property = BlobBuilder*/
const Value V18542_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_350 = {S4, 0, &V18542_350_57446, &P18542_282};	/*Property = Canvas*/
const Value V18542_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_283 = {S5, 0, &V18542_283_57161, &P18542_350};	/*Property = CssBackground*/
const Value V18542_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_284 = {S6, 0, &V18542_284_57164, &P18542_283};	/*Property = CssBorderImage*/
const Value V18542_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_285 = {S7, 0, &V18542_285_57167, &P18542_284};	/*Property = CssCanvas*/
const Value V18542_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_286 = {S8, 0, &V18542_286_57170, &P18542_285};	/*Property = CssColor*/
const Value V18542_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_237 = {S9, 0, &V18542_237_55786, &P18542_286};	/*Property = CssColumn*/
const Value V18542_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_287 = {S10, 0, &V18542_287_57173, &P18542_237};	/*Property = CssFlexbox*/
const Value V18542_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_288 = {S11, 0, &V18542_288_57176, &P18542_287};	/*Property = CssFont*/
const Value V18542_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_289 = {S12, 0, &V18542_289_86899, &P18542_288};	/*Property = CssImages*/
const Value V18542_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_291 = {S13, 0, &V18542_291_57182, &P18542_289};	/*Property = CssMediaQueries*/
const Value V18542_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_292 = {S14, 0, &V18542_292_57185, &P18542_291};	/*Property = CssMinMax*/
const Value V18542_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_293 = {S15, 0, &V18542_293_57188, &P18542_292};	/*Property = CssOverflow*/
const Value V18542_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_294 = {S16, 0, &V18542_294_57191, &P18542_293};	/*Property = CssPosition*/
const Value V18542_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_295 = {S17, 0, &V18542_295_57194, &P18542_294};	/*Property = CssText*/
const Value V18542_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_297 = {S18, 0, &V18542_297_57197, &P18542_295};	/*Property = CssTransforms*/
const Value V18542_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_241 = {S19, 0, &V18542_241_55801, &P18542_297};	/*Property = CssTransitions*/
const Value V18542_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_298 = {S20, 0, &V18542_298_57208, &P18542_241};	/*Property = CssUI*/
const Value V18542_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_300 = {S21, 0, &V18542_300_57217, &P18542_298};	/*Property = DataSet*/
const Value V18542_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_301 = {S22, 0, &V18542_301_57220, &P18542_300};	/*Property = DataUrl*/
const Value V18542_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_302 = {S23, 0, &V18542_302_57223, &P18542_301};	/*Property = DeviceOrientation*/
const Value V18542_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_304 = {S24, 0, &V18542_304_57229, &P18542_302};	/*Property = FileReader*/
const Value V18542_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_305 = {S25, 0, &V18542_305_57232, &P18542_304};	/*Property = FileSaver*/
const Value V18542_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_306 = {S26, 0, &V18542_306_57238, &P18542_305};	/*Property = FileWriter*/
const Value V18542_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_307 = {S27, 0, &V18542_307_57241, &P18542_306};	/*Property = FormData*/
const Value V18542_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_309 = {S28, 0, &V18542_309_57247, &P18542_307};	/*Property = Fullscreen*/
const Value V18542_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_310 = {S29, 0, &V18542_310_57250, &P18542_309};	/*Property = GeoLocation*/
const Value V18542_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_312 = {S30, 0, &V18542_312_57256, &P18542_310};	/*Property = History*/
const Value V18542_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_314 = {S31, 0, &V18542_314_57262, &P18542_312};	/*Property = Html5*/
const Value V18542_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_313 = {S32, 0, &V18542_313_57259, &P18542_314};	/*Property = Html-Media-Capture*/
const Value V18542_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_315 = {S33, 0, &V18542_315_57265, &P18542_313};	/*Property = Iframe*/
const Value V18542_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_316 = {S34, 0, &V18542_316_57268, &P18542_315};	/*Property = IndexedDB*/
const Value V18542_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_317 = {S35, 0, &V18542_317_57271, &P18542_316};	/*Property = Json*/
const Value V18542_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18542_15 = {S37, 0, &V18542_15_28, &P18542_317};	/*Property = LayoutEngine*/
const Value V18542_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_344 = {S38, 0, &V18542_344_57274, &P18542_15};	/*Property = Masking*/
const Value V18542_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_320 = {S39, 0, &V18542_320_57292, &P18542_344};	/*Property = PostMessage*/
const Value V18542_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_321 = {S40, 0, &V18542_321_57295, &P18542_320};	/*Property = Progress*/
const Value V18542_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_322 = {S41, 0, &V18542_322_57298, &P18542_321};	/*Property = Prompts*/
const Value V18542_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_323 = {S42, 0, &V18542_323_57301, &P18542_322};	/*Property = Selector*/
const Value V18542_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_324 = {S43, 0, &V18542_324_57307, &P18542_323};	/*Property = Svg*/
const Value V18542_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_327 = {S44, 0, &V18542_327_57316, &P18542_324};	/*Property = TouchEvents*/
const Value V18542_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_328 = {S45, 0, &V18542_328_57319, &P18542_327};	/*Property = Track*/
const Value V18542_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_329 = {S46, 0, &V18542_329_57325, &P18542_328};	/*Property = Video*/
const Value V18542_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_330 = {S47, 0, &V18542_330_57328, &P18542_329};	/*Property = Viewport*/
const Value V18542_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_332 = {S48, 0, &V18542_332_57340, &P18542_330};	/*Property = WebWorkers*/
const Value V18542_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18542_333 = {S49, 0, &V18542_333_57343, &P18542_332};	/*Property = Xhr2*/
const Profile P18542 = {"18542", &P18542_333};	/*Profile ID - 18542*/
const Value V24433_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_279 = {S1, 0, &V24433_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24433_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_282 = {S3, 0, &V24433_282_57155, &P24433_279};	/*Property = BlobBuilder*/
const Value V24433_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_350 = {S4, 0, &V24433_350_57446, &P24433_282};	/*Property = Canvas*/
const Value V24433_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_283 = {S5, 0, &V24433_283_57161, &P24433_350};	/*Property = CssBackground*/
const Value V24433_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_284 = {S6, 0, &V24433_284_57164, &P24433_283};	/*Property = CssBorderImage*/
const Value V24433_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_285 = {S7, 0, &V24433_285_57167, &P24433_284};	/*Property = CssCanvas*/
const Value V24433_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_286 = {S8, 0, &V24433_286_57170, &P24433_285};	/*Property = CssColor*/
const Value V24433_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_237 = {S9, 0, &V24433_237_55786, &P24433_286};	/*Property = CssColumn*/
const Value V24433_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_287 = {S10, 0, &V24433_287_57173, &P24433_237};	/*Property = CssFlexbox*/
const Value V24433_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_288 = {S11, 0, &V24433_288_57176, &P24433_287};	/*Property = CssFont*/
const Value V24433_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_289 = {S12, 0, &V24433_289_86899, &P24433_288};	/*Property = CssImages*/
const Value V24433_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_291 = {S13, 0, &V24433_291_57182, &P24433_289};	/*Property = CssMediaQueries*/
const Value V24433_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_292 = {S14, 0, &V24433_292_57185, &P24433_291};	/*Property = CssMinMax*/
const Value V24433_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_293 = {S15, 0, &V24433_293_57188, &P24433_292};	/*Property = CssOverflow*/
const Value V24433_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_294 = {S16, 0, &V24433_294_57191, &P24433_293};	/*Property = CssPosition*/
const Value V24433_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_295 = {S17, 0, &V24433_295_57194, &P24433_294};	/*Property = CssText*/
const Value V24433_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_297 = {S18, 0, &V24433_297_57197, &P24433_295};	/*Property = CssTransforms*/
const Value V24433_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_241 = {S19, 0, &V24433_241_55801, &P24433_297};	/*Property = CssTransitions*/
const Value V24433_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_298 = {S20, 0, &V24433_298_57208, &P24433_241};	/*Property = CssUI*/
const Value V24433_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_300 = {S21, 0, &V24433_300_57217, &P24433_298};	/*Property = DataSet*/
const Value V24433_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_301 = {S22, 0, &V24433_301_57220, &P24433_300};	/*Property = DataUrl*/
const Value V24433_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_302 = {S23, 0, &V24433_302_57223, &P24433_301};	/*Property = DeviceOrientation*/
const Value V24433_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_304 = {S24, 0, &V24433_304_57229, &P24433_302};	/*Property = FileReader*/
const Value V24433_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_305 = {S25, 0, &V24433_305_57232, &P24433_304};	/*Property = FileSaver*/
const Value V24433_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_306 = {S26, 0, &V24433_306_57238, &P24433_305};	/*Property = FileWriter*/
const Value V24433_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_307 = {S27, 0, &V24433_307_57241, &P24433_306};	/*Property = FormData*/
const Value V24433_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_309 = {S28, 0, &V24433_309_57247, &P24433_307};	/*Property = Fullscreen*/
const Value V24433_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_310 = {S29, 0, &V24433_310_57250, &P24433_309};	/*Property = GeoLocation*/
const Value V24433_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_312 = {S30, 0, &V24433_312_57256, &P24433_310};	/*Property = History*/
const Value V24433_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_314 = {S31, 0, &V24433_314_57262, &P24433_312};	/*Property = Html5*/
const Value V24433_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_313 = {S32, 0, &V24433_313_57259, &P24433_314};	/*Property = Html-Media-Capture*/
const Value V24433_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_315 = {S33, 0, &V24433_315_57265, &P24433_313};	/*Property = Iframe*/
const Value V24433_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_316 = {S34, 0, &V24433_316_57268, &P24433_315};	/*Property = IndexedDB*/
const Value V24433_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_317 = {S35, 0, &V24433_317_57271, &P24433_316};	/*Property = Json*/
const Value V24433_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24433_15 = {S37, 0, &V24433_15_28, &P24433_317};	/*Property = LayoutEngine*/
const Value V24433_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_344 = {S38, 0, &V24433_344_57274, &P24433_15};	/*Property = Masking*/
const Value V24433_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_320 = {S39, 0, &V24433_320_57292, &P24433_344};	/*Property = PostMessage*/
const Value V24433_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_321 = {S40, 0, &V24433_321_57295, &P24433_320};	/*Property = Progress*/
const Value V24433_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_322 = {S41, 0, &V24433_322_57298, &P24433_321};	/*Property = Prompts*/
const Value V24433_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_323 = {S42, 0, &V24433_323_57301, &P24433_322};	/*Property = Selector*/
const Value V24433_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_324 = {S43, 0, &V24433_324_57307, &P24433_323};	/*Property = Svg*/
const Value V24433_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_327 = {S44, 0, &V24433_327_57316, &P24433_324};	/*Property = TouchEvents*/
const Value V24433_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_328 = {S45, 0, &V24433_328_57319, &P24433_327};	/*Property = Track*/
const Value V24433_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_329 = {S46, 0, &V24433_329_57325, &P24433_328};	/*Property = Video*/
const Value V24433_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_330 = {S47, 0, &V24433_330_57328, &P24433_329};	/*Property = Viewport*/
const Value V24433_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_332 = {S48, 0, &V24433_332_57340, &P24433_330};	/*Property = WebWorkers*/
const Value V24433_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24433_333 = {S49, 0, &V24433_333_57343, &P24433_332};	/*Property = Xhr2*/
const Profile P24433 = {"24433", &P24433_333};	/*Profile ID - 24433*/
const Value V24432_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_279 = {S1, 0, &V24432_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24432_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_282 = {S3, 0, &V24432_282_57155, &P24432_279};	/*Property = BlobBuilder*/
const Value V24432_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_350 = {S4, 0, &V24432_350_57446, &P24432_282};	/*Property = Canvas*/
const Value V24432_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_283 = {S5, 0, &V24432_283_57161, &P24432_350};	/*Property = CssBackground*/
const Value V24432_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_284 = {S6, 0, &V24432_284_57164, &P24432_283};	/*Property = CssBorderImage*/
const Value V24432_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_285 = {S7, 0, &V24432_285_57167, &P24432_284};	/*Property = CssCanvas*/
const Value V24432_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_286 = {S8, 0, &V24432_286_57170, &P24432_285};	/*Property = CssColor*/
const Value V24432_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_237 = {S9, 0, &V24432_237_55786, &P24432_286};	/*Property = CssColumn*/
const Value V24432_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_287 = {S10, 0, &V24432_287_57173, &P24432_237};	/*Property = CssFlexbox*/
const Value V24432_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_288 = {S11, 0, &V24432_288_57176, &P24432_287};	/*Property = CssFont*/
const Value V24432_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_289 = {S12, 0, &V24432_289_86899, &P24432_288};	/*Property = CssImages*/
const Value V24432_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_291 = {S13, 0, &V24432_291_57182, &P24432_289};	/*Property = CssMediaQueries*/
const Value V24432_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_292 = {S14, 0, &V24432_292_57185, &P24432_291};	/*Property = CssMinMax*/
const Value V24432_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_293 = {S15, 0, &V24432_293_57188, &P24432_292};	/*Property = CssOverflow*/
const Value V24432_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_294 = {S16, 0, &V24432_294_57191, &P24432_293};	/*Property = CssPosition*/
const Value V24432_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_295 = {S17, 0, &V24432_295_57194, &P24432_294};	/*Property = CssText*/
const Value V24432_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_297 = {S18, 0, &V24432_297_57197, &P24432_295};	/*Property = CssTransforms*/
const Value V24432_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_241 = {S19, 0, &V24432_241_55801, &P24432_297};	/*Property = CssTransitions*/
const Value V24432_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_298 = {S20, 0, &V24432_298_57208, &P24432_241};	/*Property = CssUI*/
const Value V24432_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_300 = {S21, 0, &V24432_300_57217, &P24432_298};	/*Property = DataSet*/
const Value V24432_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_301 = {S22, 0, &V24432_301_57220, &P24432_300};	/*Property = DataUrl*/
const Value V24432_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_302 = {S23, 0, &V24432_302_57223, &P24432_301};	/*Property = DeviceOrientation*/
const Value V24432_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_304 = {S24, 0, &V24432_304_57229, &P24432_302};	/*Property = FileReader*/
const Value V24432_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_305 = {S25, 0, &V24432_305_57232, &P24432_304};	/*Property = FileSaver*/
const Value V24432_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_306 = {S26, 0, &V24432_306_57238, &P24432_305};	/*Property = FileWriter*/
const Value V24432_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_307 = {S27, 0, &V24432_307_57241, &P24432_306};	/*Property = FormData*/
const Value V24432_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_309 = {S28, 0, &V24432_309_57247, &P24432_307};	/*Property = Fullscreen*/
const Value V24432_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_310 = {S29, 0, &V24432_310_57250, &P24432_309};	/*Property = GeoLocation*/
const Value V24432_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_312 = {S30, 0, &V24432_312_57256, &P24432_310};	/*Property = History*/
const Value V24432_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_314 = {S31, 0, &V24432_314_57262, &P24432_312};	/*Property = Html5*/
const Value V24432_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_313 = {S32, 0, &V24432_313_57259, &P24432_314};	/*Property = Html-Media-Capture*/
const Value V24432_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_315 = {S33, 0, &V24432_315_57265, &P24432_313};	/*Property = Iframe*/
const Value V24432_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_316 = {S34, 0, &V24432_316_57268, &P24432_315};	/*Property = IndexedDB*/
const Value V24432_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_317 = {S35, 0, &V24432_317_57271, &P24432_316};	/*Property = Json*/
const Value V24432_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24432_15 = {S37, 0, &V24432_15_28, &P24432_317};	/*Property = LayoutEngine*/
const Value V24432_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_344 = {S38, 0, &V24432_344_57274, &P24432_15};	/*Property = Masking*/
const Value V24432_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_320 = {S39, 0, &V24432_320_57292, &P24432_344};	/*Property = PostMessage*/
const Value V24432_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_321 = {S40, 0, &V24432_321_57295, &P24432_320};	/*Property = Progress*/
const Value V24432_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_322 = {S41, 0, &V24432_322_57298, &P24432_321};	/*Property = Prompts*/
const Value V24432_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_323 = {S42, 0, &V24432_323_57301, &P24432_322};	/*Property = Selector*/
const Value V24432_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_324 = {S43, 0, &V24432_324_57307, &P24432_323};	/*Property = Svg*/
const Value V24432_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_327 = {S44, 0, &V24432_327_57316, &P24432_324};	/*Property = TouchEvents*/
const Value V24432_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_328 = {S45, 0, &V24432_328_57319, &P24432_327};	/*Property = Track*/
const Value V24432_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_329 = {S46, 0, &V24432_329_57325, &P24432_328};	/*Property = Video*/
const Value V24432_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_330 = {S47, 0, &V24432_330_57328, &P24432_329};	/*Property = Viewport*/
const Value V24432_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_332 = {S48, 0, &V24432_332_57340, &P24432_330};	/*Property = WebWorkers*/
const Value V24432_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24432_333 = {S49, 0, &V24432_333_57343, &P24432_332};	/*Property = Xhr2*/
const Profile P24432 = {"24432", &P24432_333};	/*Profile ID - 24432*/
const Value V18551_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_279 = {S1, 0, &V18551_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18551_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_282 = {S3, 0, &V18551_282_57155, &P18551_279};	/*Property = BlobBuilder*/
const Value V18551_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_350 = {S4, 0, &V18551_350_57446, &P18551_282};	/*Property = Canvas*/
const Value V18551_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_283 = {S5, 0, &V18551_283_57161, &P18551_350};	/*Property = CssBackground*/
const Value V18551_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_284 = {S6, 0, &V18551_284_57164, &P18551_283};	/*Property = CssBorderImage*/
const Value V18551_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_285 = {S7, 0, &V18551_285_57167, &P18551_284};	/*Property = CssCanvas*/
const Value V18551_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_286 = {S8, 0, &V18551_286_57170, &P18551_285};	/*Property = CssColor*/
const Value V18551_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_237 = {S9, 0, &V18551_237_55786, &P18551_286};	/*Property = CssColumn*/
const Value V18551_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_287 = {S10, 0, &V18551_287_57173, &P18551_237};	/*Property = CssFlexbox*/
const Value V18551_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_288 = {S11, 0, &V18551_288_57176, &P18551_287};	/*Property = CssFont*/
const Value V18551_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_289 = {S12, 0, &V18551_289_86899, &P18551_288};	/*Property = CssImages*/
const Value V18551_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_291 = {S13, 0, &V18551_291_57182, &P18551_289};	/*Property = CssMediaQueries*/
const Value V18551_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_292 = {S14, 0, &V18551_292_57185, &P18551_291};	/*Property = CssMinMax*/
const Value V18551_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_293 = {S15, 0, &V18551_293_57188, &P18551_292};	/*Property = CssOverflow*/
const Value V18551_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_294 = {S16, 0, &V18551_294_57191, &P18551_293};	/*Property = CssPosition*/
const Value V18551_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_295 = {S17, 0, &V18551_295_57194, &P18551_294};	/*Property = CssText*/
const Value V18551_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_297 = {S18, 0, &V18551_297_57197, &P18551_295};	/*Property = CssTransforms*/
const Value V18551_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_241 = {S19, 0, &V18551_241_55801, &P18551_297};	/*Property = CssTransitions*/
const Value V18551_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_298 = {S20, 0, &V18551_298_57208, &P18551_241};	/*Property = CssUI*/
const Value V18551_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_300 = {S21, 0, &V18551_300_57217, &P18551_298};	/*Property = DataSet*/
const Value V18551_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_301 = {S22, 0, &V18551_301_57220, &P18551_300};	/*Property = DataUrl*/
const Value V18551_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_302 = {S23, 0, &V18551_302_57223, &P18551_301};	/*Property = DeviceOrientation*/
const Value V18551_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_304 = {S24, 0, &V18551_304_57229, &P18551_302};	/*Property = FileReader*/
const Value V18551_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_305 = {S25, 0, &V18551_305_57232, &P18551_304};	/*Property = FileSaver*/
const Value V18551_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_306 = {S26, 0, &V18551_306_57238, &P18551_305};	/*Property = FileWriter*/
const Value V18551_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_307 = {S27, 0, &V18551_307_57241, &P18551_306};	/*Property = FormData*/
const Value V18551_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_309 = {S28, 0, &V18551_309_57247, &P18551_307};	/*Property = Fullscreen*/
const Value V18551_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_310 = {S29, 0, &V18551_310_57250, &P18551_309};	/*Property = GeoLocation*/
const Value V18551_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_312 = {S30, 0, &V18551_312_57256, &P18551_310};	/*Property = History*/
const Value V18551_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_314 = {S31, 0, &V18551_314_57262, &P18551_312};	/*Property = Html5*/
const Value V18551_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_313 = {S32, 0, &V18551_313_57259, &P18551_314};	/*Property = Html-Media-Capture*/
const Value V18551_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_315 = {S33, 0, &V18551_315_57265, &P18551_313};	/*Property = Iframe*/
const Value V18551_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_316 = {S34, 0, &V18551_316_57268, &P18551_315};	/*Property = IndexedDB*/
const Value V18551_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_317 = {S35, 0, &V18551_317_57271, &P18551_316};	/*Property = Json*/
const Value V18551_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18551_15 = {S37, 0, &V18551_15_28, &P18551_317};	/*Property = LayoutEngine*/
const Value V18551_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_344 = {S38, 0, &V18551_344_57274, &P18551_15};	/*Property = Masking*/
const Value V18551_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_320 = {S39, 0, &V18551_320_57292, &P18551_344};	/*Property = PostMessage*/
const Value V18551_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_321 = {S40, 0, &V18551_321_57295, &P18551_320};	/*Property = Progress*/
const Value V18551_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_322 = {S41, 0, &V18551_322_57298, &P18551_321};	/*Property = Prompts*/
const Value V18551_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_323 = {S42, 0, &V18551_323_57301, &P18551_322};	/*Property = Selector*/
const Value V18551_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_324 = {S43, 0, &V18551_324_57307, &P18551_323};	/*Property = Svg*/
const Value V18551_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_327 = {S44, 0, &V18551_327_57316, &P18551_324};	/*Property = TouchEvents*/
const Value V18551_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_328 = {S45, 0, &V18551_328_57319, &P18551_327};	/*Property = Track*/
const Value V18551_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_329 = {S46, 0, &V18551_329_57325, &P18551_328};	/*Property = Video*/
const Value V18551_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_330 = {S47, 0, &V18551_330_57328, &P18551_329};	/*Property = Viewport*/
const Value V18551_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_332 = {S48, 0, &V18551_332_57340, &P18551_330};	/*Property = WebWorkers*/
const Value V18551_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18551_333 = {S49, 0, &V18551_333_57343, &P18551_332};	/*Property = Xhr2*/
const Profile P18551 = {"18551", &P18551_333};	/*Profile ID - 18551*/
const Value V18540_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_279 = {S1, 0, &V18540_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18540_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_282 = {S3, 0, &V18540_282_57155, &P18540_279};	/*Property = BlobBuilder*/
const Value V18540_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_350 = {S4, 0, &V18540_350_57446, &P18540_282};	/*Property = Canvas*/
const Value V18540_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_283 = {S5, 0, &V18540_283_57161, &P18540_350};	/*Property = CssBackground*/
const Value V18540_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_284 = {S6, 0, &V18540_284_57164, &P18540_283};	/*Property = CssBorderImage*/
const Value V18540_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_285 = {S7, 0, &V18540_285_57167, &P18540_284};	/*Property = CssCanvas*/
const Value V18540_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_286 = {S8, 0, &V18540_286_57170, &P18540_285};	/*Property = CssColor*/
const Value V18540_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_237 = {S9, 0, &V18540_237_55786, &P18540_286};	/*Property = CssColumn*/
const Value V18540_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_287 = {S10, 0, &V18540_287_57173, &P18540_237};	/*Property = CssFlexbox*/
const Value V18540_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_288 = {S11, 0, &V18540_288_57176, &P18540_287};	/*Property = CssFont*/
const Value V18540_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_289 = {S12, 0, &V18540_289_86899, &P18540_288};	/*Property = CssImages*/
const Value V18540_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_291 = {S13, 0, &V18540_291_57182, &P18540_289};	/*Property = CssMediaQueries*/
const Value V18540_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_292 = {S14, 0, &V18540_292_57185, &P18540_291};	/*Property = CssMinMax*/
const Value V18540_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_293 = {S15, 0, &V18540_293_57188, &P18540_292};	/*Property = CssOverflow*/
const Value V18540_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_294 = {S16, 0, &V18540_294_57191, &P18540_293};	/*Property = CssPosition*/
const Value V18540_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_295 = {S17, 0, &V18540_295_57194, &P18540_294};	/*Property = CssText*/
const Value V18540_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_297 = {S18, 0, &V18540_297_57197, &P18540_295};	/*Property = CssTransforms*/
const Value V18540_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_241 = {S19, 0, &V18540_241_55801, &P18540_297};	/*Property = CssTransitions*/
const Value V18540_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_298 = {S20, 0, &V18540_298_57208, &P18540_241};	/*Property = CssUI*/
const Value V18540_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_300 = {S21, 0, &V18540_300_57217, &P18540_298};	/*Property = DataSet*/
const Value V18540_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_301 = {S22, 0, &V18540_301_57220, &P18540_300};	/*Property = DataUrl*/
const Value V18540_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_302 = {S23, 0, &V18540_302_57223, &P18540_301};	/*Property = DeviceOrientation*/
const Value V18540_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_304 = {S24, 0, &V18540_304_57229, &P18540_302};	/*Property = FileReader*/
const Value V18540_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_305 = {S25, 0, &V18540_305_57232, &P18540_304};	/*Property = FileSaver*/
const Value V18540_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_306 = {S26, 0, &V18540_306_57238, &P18540_305};	/*Property = FileWriter*/
const Value V18540_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_307 = {S27, 0, &V18540_307_57241, &P18540_306};	/*Property = FormData*/
const Value V18540_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_309 = {S28, 0, &V18540_309_57247, &P18540_307};	/*Property = Fullscreen*/
const Value V18540_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_310 = {S29, 0, &V18540_310_57250, &P18540_309};	/*Property = GeoLocation*/
const Value V18540_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_312 = {S30, 0, &V18540_312_57256, &P18540_310};	/*Property = History*/
const Value V18540_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_314 = {S31, 0, &V18540_314_57262, &P18540_312};	/*Property = Html5*/
const Value V18540_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_313 = {S32, 0, &V18540_313_57259, &P18540_314};	/*Property = Html-Media-Capture*/
const Value V18540_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_315 = {S33, 0, &V18540_315_57265, &P18540_313};	/*Property = Iframe*/
const Value V18540_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_316 = {S34, 0, &V18540_316_57268, &P18540_315};	/*Property = IndexedDB*/
const Value V18540_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_317 = {S35, 0, &V18540_317_57271, &P18540_316};	/*Property = Json*/
const Value V18540_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18540_15 = {S37, 0, &V18540_15_28, &P18540_317};	/*Property = LayoutEngine*/
const Value V18540_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_344 = {S38, 0, &V18540_344_57274, &P18540_15};	/*Property = Masking*/
const Value V18540_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_320 = {S39, 0, &V18540_320_57292, &P18540_344};	/*Property = PostMessage*/
const Value V18540_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_321 = {S40, 0, &V18540_321_57295, &P18540_320};	/*Property = Progress*/
const Value V18540_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_322 = {S41, 0, &V18540_322_57298, &P18540_321};	/*Property = Prompts*/
const Value V18540_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_323 = {S42, 0, &V18540_323_57301, &P18540_322};	/*Property = Selector*/
const Value V18540_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_324 = {S43, 0, &V18540_324_57307, &P18540_323};	/*Property = Svg*/
const Value V18540_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_327 = {S44, 0, &V18540_327_57316, &P18540_324};	/*Property = TouchEvents*/
const Value V18540_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_328 = {S45, 0, &V18540_328_57319, &P18540_327};	/*Property = Track*/
const Value V18540_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_329 = {S46, 0, &V18540_329_57325, &P18540_328};	/*Property = Video*/
const Value V18540_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_330 = {S47, 0, &V18540_330_57328, &P18540_329};	/*Property = Viewport*/
const Value V18540_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_332 = {S48, 0, &V18540_332_57340, &P18540_330};	/*Property = WebWorkers*/
const Value V18540_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18540_333 = {S49, 0, &V18540_333_57343, &P18540_332};	/*Property = Xhr2*/
const Profile P18540 = {"18540", &P18540_333};	/*Profile ID - 18540*/
const Value V21938_1_1 = {S2, NULL};	/*Value = True*/
const Property P21938_1 = {S50, 0, &V21938_1_1, NULL};	/*Property = IsMobile*/
const Value V21938_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21938_5 = {S52, 0, &V21938_5_1277, &P21938_1};	/*Property = ScreenPixelsHeight*/
const Value V21938_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21938_4 = {S54, 0, &V21938_4_1279, &P21938_5};	/*Property = ScreenPixelsWidth*/
const Profile P21938 = {"21938", &P21938_4};	/*Profile ID - 21938*/
const Value V24434_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_279 = {S1, 0, &V24434_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24434_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_282 = {S3, 0, &V24434_282_57155, &P24434_279};	/*Property = BlobBuilder*/
const Value V24434_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_350 = {S4, 0, &V24434_350_57446, &P24434_282};	/*Property = Canvas*/
const Value V24434_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_283 = {S5, 0, &V24434_283_57161, &P24434_350};	/*Property = CssBackground*/
const Value V24434_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_284 = {S6, 0, &V24434_284_57164, &P24434_283};	/*Property = CssBorderImage*/
const Value V24434_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_285 = {S7, 0, &V24434_285_57167, &P24434_284};	/*Property = CssCanvas*/
const Value V24434_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_286 = {S8, 0, &V24434_286_57170, &P24434_285};	/*Property = CssColor*/
const Value V24434_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_237 = {S9, 0, &V24434_237_55786, &P24434_286};	/*Property = CssColumn*/
const Value V24434_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_287 = {S10, 0, &V24434_287_57173, &P24434_237};	/*Property = CssFlexbox*/
const Value V24434_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_288 = {S11, 0, &V24434_288_57176, &P24434_287};	/*Property = CssFont*/
const Value V24434_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_289 = {S12, 0, &V24434_289_86899, &P24434_288};	/*Property = CssImages*/
const Value V24434_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_291 = {S13, 0, &V24434_291_57182, &P24434_289};	/*Property = CssMediaQueries*/
const Value V24434_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_292 = {S14, 0, &V24434_292_57185, &P24434_291};	/*Property = CssMinMax*/
const Value V24434_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_293 = {S15, 0, &V24434_293_57188, &P24434_292};	/*Property = CssOverflow*/
const Value V24434_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_294 = {S16, 0, &V24434_294_57191, &P24434_293};	/*Property = CssPosition*/
const Value V24434_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_295 = {S17, 0, &V24434_295_57194, &P24434_294};	/*Property = CssText*/
const Value V24434_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_297 = {S18, 0, &V24434_297_57197, &P24434_295};	/*Property = CssTransforms*/
const Value V24434_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_241 = {S19, 0, &V24434_241_55801, &P24434_297};	/*Property = CssTransitions*/
const Value V24434_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_298 = {S20, 0, &V24434_298_57208, &P24434_241};	/*Property = CssUI*/
const Value V24434_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_300 = {S21, 0, &V24434_300_57217, &P24434_298};	/*Property = DataSet*/
const Value V24434_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_301 = {S22, 0, &V24434_301_57220, &P24434_300};	/*Property = DataUrl*/
const Value V24434_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_302 = {S23, 0, &V24434_302_57223, &P24434_301};	/*Property = DeviceOrientation*/
const Value V24434_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_304 = {S24, 0, &V24434_304_57229, &P24434_302};	/*Property = FileReader*/
const Value V24434_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_305 = {S25, 0, &V24434_305_57232, &P24434_304};	/*Property = FileSaver*/
const Value V24434_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_306 = {S26, 0, &V24434_306_57238, &P24434_305};	/*Property = FileWriter*/
const Value V24434_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_307 = {S27, 0, &V24434_307_57241, &P24434_306};	/*Property = FormData*/
const Value V24434_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_309 = {S28, 0, &V24434_309_57247, &P24434_307};	/*Property = Fullscreen*/
const Value V24434_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_310 = {S29, 0, &V24434_310_57250, &P24434_309};	/*Property = GeoLocation*/
const Value V24434_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_312 = {S30, 0, &V24434_312_57256, &P24434_310};	/*Property = History*/
const Value V24434_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_314 = {S31, 0, &V24434_314_57262, &P24434_312};	/*Property = Html5*/
const Value V24434_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_313 = {S32, 0, &V24434_313_57259, &P24434_314};	/*Property = Html-Media-Capture*/
const Value V24434_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_315 = {S33, 0, &V24434_315_57265, &P24434_313};	/*Property = Iframe*/
const Value V24434_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_316 = {S34, 0, &V24434_316_57268, &P24434_315};	/*Property = IndexedDB*/
const Value V24434_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_317 = {S35, 0, &V24434_317_57271, &P24434_316};	/*Property = Json*/
const Value V24434_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24434_15 = {S37, 0, &V24434_15_28, &P24434_317};	/*Property = LayoutEngine*/
const Value V24434_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_344 = {S38, 0, &V24434_344_57274, &P24434_15};	/*Property = Masking*/
const Value V24434_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_320 = {S39, 0, &V24434_320_57292, &P24434_344};	/*Property = PostMessage*/
const Value V24434_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_321 = {S40, 0, &V24434_321_57295, &P24434_320};	/*Property = Progress*/
const Value V24434_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_322 = {S41, 0, &V24434_322_57298, &P24434_321};	/*Property = Prompts*/
const Value V24434_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_323 = {S42, 0, &V24434_323_57301, &P24434_322};	/*Property = Selector*/
const Value V24434_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_324 = {S43, 0, &V24434_324_57307, &P24434_323};	/*Property = Svg*/
const Value V24434_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_327 = {S44, 0, &V24434_327_57316, &P24434_324};	/*Property = TouchEvents*/
const Value V24434_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_328 = {S45, 0, &V24434_328_57319, &P24434_327};	/*Property = Track*/
const Value V24434_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_329 = {S46, 0, &V24434_329_57325, &P24434_328};	/*Property = Video*/
const Value V24434_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_330 = {S47, 0, &V24434_330_57328, &P24434_329};	/*Property = Viewport*/
const Value V24434_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_332 = {S48, 0, &V24434_332_57340, &P24434_330};	/*Property = WebWorkers*/
const Value V24434_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24434_333 = {S49, 0, &V24434_333_57343, &P24434_332};	/*Property = Xhr2*/
const Profile P24434 = {"24434", &P24434_333};	/*Profile ID - 24434*/
const Value V24448_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_279 = {S1, 0, &V24448_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24448_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_282 = {S3, 0, &V24448_282_57155, &P24448_279};	/*Property = BlobBuilder*/
const Value V24448_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_350 = {S4, 0, &V24448_350_57446, &P24448_282};	/*Property = Canvas*/
const Value V24448_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_283 = {S5, 0, &V24448_283_57161, &P24448_350};	/*Property = CssBackground*/
const Value V24448_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_284 = {S6, 0, &V24448_284_57164, &P24448_283};	/*Property = CssBorderImage*/
const Value V24448_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_285 = {S7, 0, &V24448_285_57167, &P24448_284};	/*Property = CssCanvas*/
const Value V24448_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_286 = {S8, 0, &V24448_286_57170, &P24448_285};	/*Property = CssColor*/
const Value V24448_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_237 = {S9, 0, &V24448_237_55786, &P24448_286};	/*Property = CssColumn*/
const Value V24448_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_287 = {S10, 0, &V24448_287_57173, &P24448_237};	/*Property = CssFlexbox*/
const Value V24448_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_288 = {S11, 0, &V24448_288_57176, &P24448_287};	/*Property = CssFont*/
const Value V24448_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_289 = {S12, 0, &V24448_289_86899, &P24448_288};	/*Property = CssImages*/
const Value V24448_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_291 = {S13, 0, &V24448_291_57182, &P24448_289};	/*Property = CssMediaQueries*/
const Value V24448_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_292 = {S14, 0, &V24448_292_57185, &P24448_291};	/*Property = CssMinMax*/
const Value V24448_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_293 = {S15, 0, &V24448_293_57188, &P24448_292};	/*Property = CssOverflow*/
const Value V24448_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_294 = {S16, 0, &V24448_294_57191, &P24448_293};	/*Property = CssPosition*/
const Value V24448_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_295 = {S17, 0, &V24448_295_57194, &P24448_294};	/*Property = CssText*/
const Value V24448_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_297 = {S18, 0, &V24448_297_57197, &P24448_295};	/*Property = CssTransforms*/
const Value V24448_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_241 = {S19, 0, &V24448_241_55801, &P24448_297};	/*Property = CssTransitions*/
const Value V24448_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_298 = {S20, 0, &V24448_298_57208, &P24448_241};	/*Property = CssUI*/
const Value V24448_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_300 = {S21, 0, &V24448_300_57217, &P24448_298};	/*Property = DataSet*/
const Value V24448_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_301 = {S22, 0, &V24448_301_57220, &P24448_300};	/*Property = DataUrl*/
const Value V24448_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_302 = {S23, 0, &V24448_302_57223, &P24448_301};	/*Property = DeviceOrientation*/
const Value V24448_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_304 = {S24, 0, &V24448_304_57229, &P24448_302};	/*Property = FileReader*/
const Value V24448_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_305 = {S25, 0, &V24448_305_57232, &P24448_304};	/*Property = FileSaver*/
const Value V24448_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_306 = {S26, 0, &V24448_306_57238, &P24448_305};	/*Property = FileWriter*/
const Value V24448_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_307 = {S27, 0, &V24448_307_57241, &P24448_306};	/*Property = FormData*/
const Value V24448_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_309 = {S28, 0, &V24448_309_57247, &P24448_307};	/*Property = Fullscreen*/
const Value V24448_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_310 = {S29, 0, &V24448_310_57250, &P24448_309};	/*Property = GeoLocation*/
const Value V24448_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_312 = {S30, 0, &V24448_312_57256, &P24448_310};	/*Property = History*/
const Value V24448_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_314 = {S31, 0, &V24448_314_57262, &P24448_312};	/*Property = Html5*/
const Value V24448_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_313 = {S32, 0, &V24448_313_57259, &P24448_314};	/*Property = Html-Media-Capture*/
const Value V24448_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_315 = {S33, 0, &V24448_315_57265, &P24448_313};	/*Property = Iframe*/
const Value V24448_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_316 = {S34, 0, &V24448_316_57268, &P24448_315};	/*Property = IndexedDB*/
const Value V24448_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_317 = {S35, 0, &V24448_317_57271, &P24448_316};	/*Property = Json*/
const Value V24448_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24448_15 = {S37, 0, &V24448_15_28, &P24448_317};	/*Property = LayoutEngine*/
const Value V24448_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_344 = {S38, 0, &V24448_344_57274, &P24448_15};	/*Property = Masking*/
const Value V24448_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_320 = {S39, 0, &V24448_320_57292, &P24448_344};	/*Property = PostMessage*/
const Value V24448_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_321 = {S40, 0, &V24448_321_57295, &P24448_320};	/*Property = Progress*/
const Value V24448_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_322 = {S41, 0, &V24448_322_57298, &P24448_321};	/*Property = Prompts*/
const Value V24448_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_323 = {S42, 0, &V24448_323_57301, &P24448_322};	/*Property = Selector*/
const Value V24448_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_324 = {S43, 0, &V24448_324_57307, &P24448_323};	/*Property = Svg*/
const Value V24448_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_327 = {S44, 0, &V24448_327_57316, &P24448_324};	/*Property = TouchEvents*/
const Value V24448_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_328 = {S45, 0, &V24448_328_57319, &P24448_327};	/*Property = Track*/
const Value V24448_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_329 = {S46, 0, &V24448_329_57325, &P24448_328};	/*Property = Video*/
const Value V24448_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_330 = {S47, 0, &V24448_330_57328, &P24448_329};	/*Property = Viewport*/
const Value V24448_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_332 = {S48, 0, &V24448_332_57340, &P24448_330};	/*Property = WebWorkers*/
const Value V24448_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24448_333 = {S49, 0, &V24448_333_57343, &P24448_332};	/*Property = Xhr2*/
const Profile P24448 = {"24448", &P24448_333};	/*Profile ID - 24448*/
const Value V20972_1_1 = {S2, NULL};	/*Value = True*/
const Property P20972_1 = {S50, 0, &V20972_1_1, NULL};	/*Property = IsMobile*/
const Value V20972_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P20972_5 = {S52, 0, &V20972_5_1269, &P20972_1};	/*Property = ScreenPixelsHeight*/
const Value V20972_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20972_4 = {S54, 0, &V20972_4_1229, &P20972_5};	/*Property = ScreenPixelsWidth*/
const Profile P20972 = {"20972", &P20972_4};	/*Profile ID - 20972*/
const Value V18544_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_279 = {S1, 0, &V18544_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18544_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_282 = {S3, 0, &V18544_282_57155, &P18544_279};	/*Property = BlobBuilder*/
const Value V18544_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_350 = {S4, 0, &V18544_350_57446, &P18544_282};	/*Property = Canvas*/
const Value V18544_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_283 = {S5, 0, &V18544_283_57161, &P18544_350};	/*Property = CssBackground*/
const Value V18544_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_284 = {S6, 0, &V18544_284_57164, &P18544_283};	/*Property = CssBorderImage*/
const Value V18544_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_285 = {S7, 0, &V18544_285_57167, &P18544_284};	/*Property = CssCanvas*/
const Value V18544_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_286 = {S8, 0, &V18544_286_57170, &P18544_285};	/*Property = CssColor*/
const Value V18544_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_237 = {S9, 0, &V18544_237_55786, &P18544_286};	/*Property = CssColumn*/
const Value V18544_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_287 = {S10, 0, &V18544_287_57173, &P18544_237};	/*Property = CssFlexbox*/
const Value V18544_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_288 = {S11, 0, &V18544_288_57176, &P18544_287};	/*Property = CssFont*/
const Value V18544_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_289 = {S12, 0, &V18544_289_86899, &P18544_288};	/*Property = CssImages*/
const Value V18544_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_291 = {S13, 0, &V18544_291_57182, &P18544_289};	/*Property = CssMediaQueries*/
const Value V18544_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_292 = {S14, 0, &V18544_292_57185, &P18544_291};	/*Property = CssMinMax*/
const Value V18544_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_293 = {S15, 0, &V18544_293_57188, &P18544_292};	/*Property = CssOverflow*/
const Value V18544_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_294 = {S16, 0, &V18544_294_57191, &P18544_293};	/*Property = CssPosition*/
const Value V18544_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_295 = {S17, 0, &V18544_295_57194, &P18544_294};	/*Property = CssText*/
const Value V18544_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_297 = {S18, 0, &V18544_297_57197, &P18544_295};	/*Property = CssTransforms*/
const Value V18544_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_241 = {S19, 0, &V18544_241_55801, &P18544_297};	/*Property = CssTransitions*/
const Value V18544_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_298 = {S20, 0, &V18544_298_57208, &P18544_241};	/*Property = CssUI*/
const Value V18544_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_300 = {S21, 0, &V18544_300_57217, &P18544_298};	/*Property = DataSet*/
const Value V18544_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_301 = {S22, 0, &V18544_301_57220, &P18544_300};	/*Property = DataUrl*/
const Value V18544_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_302 = {S23, 0, &V18544_302_57223, &P18544_301};	/*Property = DeviceOrientation*/
const Value V18544_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_304 = {S24, 0, &V18544_304_57229, &P18544_302};	/*Property = FileReader*/
const Value V18544_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_305 = {S25, 0, &V18544_305_57232, &P18544_304};	/*Property = FileSaver*/
const Value V18544_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_306 = {S26, 0, &V18544_306_57238, &P18544_305};	/*Property = FileWriter*/
const Value V18544_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_307 = {S27, 0, &V18544_307_57241, &P18544_306};	/*Property = FormData*/
const Value V18544_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_309 = {S28, 0, &V18544_309_57247, &P18544_307};	/*Property = Fullscreen*/
const Value V18544_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_310 = {S29, 0, &V18544_310_57250, &P18544_309};	/*Property = GeoLocation*/
const Value V18544_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_312 = {S30, 0, &V18544_312_57256, &P18544_310};	/*Property = History*/
const Value V18544_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_314 = {S31, 0, &V18544_314_57262, &P18544_312};	/*Property = Html5*/
const Value V18544_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_313 = {S32, 0, &V18544_313_57259, &P18544_314};	/*Property = Html-Media-Capture*/
const Value V18544_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_315 = {S33, 0, &V18544_315_57265, &P18544_313};	/*Property = Iframe*/
const Value V18544_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_316 = {S34, 0, &V18544_316_57268, &P18544_315};	/*Property = IndexedDB*/
const Value V18544_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_317 = {S35, 0, &V18544_317_57271, &P18544_316};	/*Property = Json*/
const Value V18544_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18544_15 = {S37, 0, &V18544_15_28, &P18544_317};	/*Property = LayoutEngine*/
const Value V18544_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_344 = {S38, 0, &V18544_344_57274, &P18544_15};	/*Property = Masking*/
const Value V18544_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_320 = {S39, 0, &V18544_320_57292, &P18544_344};	/*Property = PostMessage*/
const Value V18544_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_321 = {S40, 0, &V18544_321_57295, &P18544_320};	/*Property = Progress*/
const Value V18544_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_322 = {S41, 0, &V18544_322_57298, &P18544_321};	/*Property = Prompts*/
const Value V18544_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_323 = {S42, 0, &V18544_323_57301, &P18544_322};	/*Property = Selector*/
const Value V18544_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_324 = {S43, 0, &V18544_324_57307, &P18544_323};	/*Property = Svg*/
const Value V18544_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_327 = {S44, 0, &V18544_327_57316, &P18544_324};	/*Property = TouchEvents*/
const Value V18544_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_328 = {S45, 0, &V18544_328_57319, &P18544_327};	/*Property = Track*/
const Value V18544_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_329 = {S46, 0, &V18544_329_57325, &P18544_328};	/*Property = Video*/
const Value V18544_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_330 = {S47, 0, &V18544_330_57328, &P18544_329};	/*Property = Viewport*/
const Value V18544_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_332 = {S48, 0, &V18544_332_57340, &P18544_330};	/*Property = WebWorkers*/
const Value V18544_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18544_333 = {S49, 0, &V18544_333_57343, &P18544_332};	/*Property = Xhr2*/
const Profile P18544 = {"18544", &P18544_333};	/*Profile ID - 18544*/
const Value V18539_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_279 = {S1, 0, &V18539_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18539_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_282 = {S3, 0, &V18539_282_57155, &P18539_279};	/*Property = BlobBuilder*/
const Value V18539_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_350 = {S4, 0, &V18539_350_57446, &P18539_282};	/*Property = Canvas*/
const Value V18539_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_283 = {S5, 0, &V18539_283_57161, &P18539_350};	/*Property = CssBackground*/
const Value V18539_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_284 = {S6, 0, &V18539_284_57164, &P18539_283};	/*Property = CssBorderImage*/
const Value V18539_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_285 = {S7, 0, &V18539_285_57167, &P18539_284};	/*Property = CssCanvas*/
const Value V18539_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_286 = {S8, 0, &V18539_286_57170, &P18539_285};	/*Property = CssColor*/
const Value V18539_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_237 = {S9, 0, &V18539_237_55786, &P18539_286};	/*Property = CssColumn*/
const Value V18539_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_287 = {S10, 0, &V18539_287_57173, &P18539_237};	/*Property = CssFlexbox*/
const Value V18539_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_288 = {S11, 0, &V18539_288_57176, &P18539_287};	/*Property = CssFont*/
const Value V18539_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_289 = {S12, 0, &V18539_289_86899, &P18539_288};	/*Property = CssImages*/
const Value V18539_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_291 = {S13, 0, &V18539_291_57182, &P18539_289};	/*Property = CssMediaQueries*/
const Value V18539_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_292 = {S14, 0, &V18539_292_57185, &P18539_291};	/*Property = CssMinMax*/
const Value V18539_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_293 = {S15, 0, &V18539_293_57188, &P18539_292};	/*Property = CssOverflow*/
const Value V18539_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_294 = {S16, 0, &V18539_294_57191, &P18539_293};	/*Property = CssPosition*/
const Value V18539_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_295 = {S17, 0, &V18539_295_57194, &P18539_294};	/*Property = CssText*/
const Value V18539_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_297 = {S18, 0, &V18539_297_57197, &P18539_295};	/*Property = CssTransforms*/
const Value V18539_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_241 = {S19, 0, &V18539_241_55801, &P18539_297};	/*Property = CssTransitions*/
const Value V18539_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_298 = {S20, 0, &V18539_298_57208, &P18539_241};	/*Property = CssUI*/
const Value V18539_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_300 = {S21, 0, &V18539_300_57217, &P18539_298};	/*Property = DataSet*/
const Value V18539_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_301 = {S22, 0, &V18539_301_57220, &P18539_300};	/*Property = DataUrl*/
const Value V18539_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_302 = {S23, 0, &V18539_302_57223, &P18539_301};	/*Property = DeviceOrientation*/
const Value V18539_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_304 = {S24, 0, &V18539_304_57229, &P18539_302};	/*Property = FileReader*/
const Value V18539_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_305 = {S25, 0, &V18539_305_57232, &P18539_304};	/*Property = FileSaver*/
const Value V18539_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_306 = {S26, 0, &V18539_306_57238, &P18539_305};	/*Property = FileWriter*/
const Value V18539_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_307 = {S27, 0, &V18539_307_57241, &P18539_306};	/*Property = FormData*/
const Value V18539_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_309 = {S28, 0, &V18539_309_57247, &P18539_307};	/*Property = Fullscreen*/
const Value V18539_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_310 = {S29, 0, &V18539_310_57250, &P18539_309};	/*Property = GeoLocation*/
const Value V18539_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_312 = {S30, 0, &V18539_312_57256, &P18539_310};	/*Property = History*/
const Value V18539_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_314 = {S31, 0, &V18539_314_57262, &P18539_312};	/*Property = Html5*/
const Value V18539_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_313 = {S32, 0, &V18539_313_57259, &P18539_314};	/*Property = Html-Media-Capture*/
const Value V18539_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_315 = {S33, 0, &V18539_315_57265, &P18539_313};	/*Property = Iframe*/
const Value V18539_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_316 = {S34, 0, &V18539_316_57268, &P18539_315};	/*Property = IndexedDB*/
const Value V18539_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_317 = {S35, 0, &V18539_317_57271, &P18539_316};	/*Property = Json*/
const Value V18539_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18539_15 = {S37, 0, &V18539_15_28, &P18539_317};	/*Property = LayoutEngine*/
const Value V18539_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_344 = {S38, 0, &V18539_344_57274, &P18539_15};	/*Property = Masking*/
const Value V18539_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_320 = {S39, 0, &V18539_320_57292, &P18539_344};	/*Property = PostMessage*/
const Value V18539_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_321 = {S40, 0, &V18539_321_57295, &P18539_320};	/*Property = Progress*/
const Value V18539_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_322 = {S41, 0, &V18539_322_57298, &P18539_321};	/*Property = Prompts*/
const Value V18539_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_323 = {S42, 0, &V18539_323_57301, &P18539_322};	/*Property = Selector*/
const Value V18539_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_324 = {S43, 0, &V18539_324_57307, &P18539_323};	/*Property = Svg*/
const Value V18539_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_327 = {S44, 0, &V18539_327_57316, &P18539_324};	/*Property = TouchEvents*/
const Value V18539_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_328 = {S45, 0, &V18539_328_57319, &P18539_327};	/*Property = Track*/
const Value V18539_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_329 = {S46, 0, &V18539_329_57325, &P18539_328};	/*Property = Video*/
const Value V18539_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_330 = {S47, 0, &V18539_330_57328, &P18539_329};	/*Property = Viewport*/
const Value V18539_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_332 = {S48, 0, &V18539_332_57340, &P18539_330};	/*Property = WebWorkers*/
const Value V18539_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18539_333 = {S49, 0, &V18539_333_57343, &P18539_332};	/*Property = Xhr2*/
const Profile P18539 = {"18539", &P18539_333};	/*Profile ID - 18539*/
const Value V24451_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_279 = {S1, 0, &V24451_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24451_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_282 = {S3, 0, &V24451_282_57155, &P24451_279};	/*Property = BlobBuilder*/
const Value V24451_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_350 = {S4, 0, &V24451_350_57446, &P24451_282};	/*Property = Canvas*/
const Value V24451_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_283 = {S5, 0, &V24451_283_57161, &P24451_350};	/*Property = CssBackground*/
const Value V24451_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_284 = {S6, 0, &V24451_284_57164, &P24451_283};	/*Property = CssBorderImage*/
const Value V24451_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_285 = {S7, 0, &V24451_285_57167, &P24451_284};	/*Property = CssCanvas*/
const Value V24451_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_286 = {S8, 0, &V24451_286_57170, &P24451_285};	/*Property = CssColor*/
const Value V24451_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_237 = {S9, 0, &V24451_237_55786, &P24451_286};	/*Property = CssColumn*/
const Value V24451_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_287 = {S10, 0, &V24451_287_57173, &P24451_237};	/*Property = CssFlexbox*/
const Value V24451_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_288 = {S11, 0, &V24451_288_57176, &P24451_287};	/*Property = CssFont*/
const Value V24451_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_289 = {S12, 0, &V24451_289_86899, &P24451_288};	/*Property = CssImages*/
const Value V24451_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_291 = {S13, 0, &V24451_291_57182, &P24451_289};	/*Property = CssMediaQueries*/
const Value V24451_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_292 = {S14, 0, &V24451_292_57185, &P24451_291};	/*Property = CssMinMax*/
const Value V24451_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_293 = {S15, 0, &V24451_293_57188, &P24451_292};	/*Property = CssOverflow*/
const Value V24451_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_294 = {S16, 0, &V24451_294_57191, &P24451_293};	/*Property = CssPosition*/
const Value V24451_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_295 = {S17, 0, &V24451_295_57194, &P24451_294};	/*Property = CssText*/
const Value V24451_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_297 = {S18, 0, &V24451_297_57197, &P24451_295};	/*Property = CssTransforms*/
const Value V24451_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_241 = {S19, 0, &V24451_241_55801, &P24451_297};	/*Property = CssTransitions*/
const Value V24451_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_298 = {S20, 0, &V24451_298_57208, &P24451_241};	/*Property = CssUI*/
const Value V24451_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_300 = {S21, 0, &V24451_300_57217, &P24451_298};	/*Property = DataSet*/
const Value V24451_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_301 = {S22, 0, &V24451_301_57220, &P24451_300};	/*Property = DataUrl*/
const Value V24451_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_302 = {S23, 0, &V24451_302_57223, &P24451_301};	/*Property = DeviceOrientation*/
const Value V24451_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_304 = {S24, 0, &V24451_304_57229, &P24451_302};	/*Property = FileReader*/
const Value V24451_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_305 = {S25, 0, &V24451_305_57232, &P24451_304};	/*Property = FileSaver*/
const Value V24451_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_306 = {S26, 0, &V24451_306_57238, &P24451_305};	/*Property = FileWriter*/
const Value V24451_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_307 = {S27, 0, &V24451_307_57241, &P24451_306};	/*Property = FormData*/
const Value V24451_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_309 = {S28, 0, &V24451_309_57247, &P24451_307};	/*Property = Fullscreen*/
const Value V24451_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_310 = {S29, 0, &V24451_310_57250, &P24451_309};	/*Property = GeoLocation*/
const Value V24451_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_312 = {S30, 0, &V24451_312_57256, &P24451_310};	/*Property = History*/
const Value V24451_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_314 = {S31, 0, &V24451_314_57262, &P24451_312};	/*Property = Html5*/
const Value V24451_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_313 = {S32, 0, &V24451_313_57259, &P24451_314};	/*Property = Html-Media-Capture*/
const Value V24451_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_315 = {S33, 0, &V24451_315_57265, &P24451_313};	/*Property = Iframe*/
const Value V24451_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_316 = {S34, 0, &V24451_316_57268, &P24451_315};	/*Property = IndexedDB*/
const Value V24451_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_317 = {S35, 0, &V24451_317_57271, &P24451_316};	/*Property = Json*/
const Value V24451_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24451_15 = {S37, 0, &V24451_15_28, &P24451_317};	/*Property = LayoutEngine*/
const Value V24451_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_344 = {S38, 0, &V24451_344_57274, &P24451_15};	/*Property = Masking*/
const Value V24451_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_320 = {S39, 0, &V24451_320_57292, &P24451_344};	/*Property = PostMessage*/
const Value V24451_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_321 = {S40, 0, &V24451_321_57295, &P24451_320};	/*Property = Progress*/
const Value V24451_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_322 = {S41, 0, &V24451_322_57298, &P24451_321};	/*Property = Prompts*/
const Value V24451_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_323 = {S42, 0, &V24451_323_57301, &P24451_322};	/*Property = Selector*/
const Value V24451_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_324 = {S43, 0, &V24451_324_57307, &P24451_323};	/*Property = Svg*/
const Value V24451_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_327 = {S44, 0, &V24451_327_57316, &P24451_324};	/*Property = TouchEvents*/
const Value V24451_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_328 = {S45, 0, &V24451_328_57319, &P24451_327};	/*Property = Track*/
const Value V24451_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_329 = {S46, 0, &V24451_329_57325, &P24451_328};	/*Property = Video*/
const Value V24451_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_330 = {S47, 0, &V24451_330_57328, &P24451_329};	/*Property = Viewport*/
const Value V24451_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_332 = {S48, 0, &V24451_332_57340, &P24451_330};	/*Property = WebWorkers*/
const Value V24451_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24451_333 = {S49, 0, &V24451_333_57343, &P24451_332};	/*Property = Xhr2*/
const Profile P24451 = {"24451", &P24451_333};	/*Profile ID - 24451*/
const Profile P24691 = {"24691", NULL};	/*Profile ID - 24691*/
const Value V24452_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_279 = {S1, 0, &V24452_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24452_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_282 = {S3, 0, &V24452_282_57155, &P24452_279};	/*Property = BlobBuilder*/
const Value V24452_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_350 = {S4, 0, &V24452_350_57446, &P24452_282};	/*Property = Canvas*/
const Value V24452_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_283 = {S5, 0, &V24452_283_57161, &P24452_350};	/*Property = CssBackground*/
const Value V24452_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_284 = {S6, 0, &V24452_284_57164, &P24452_283};	/*Property = CssBorderImage*/
const Value V24452_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_285 = {S7, 0, &V24452_285_57167, &P24452_284};	/*Property = CssCanvas*/
const Value V24452_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_286 = {S8, 0, &V24452_286_57170, &P24452_285};	/*Property = CssColor*/
const Value V24452_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_237 = {S9, 0, &V24452_237_55786, &P24452_286};	/*Property = CssColumn*/
const Value V24452_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_287 = {S10, 0, &V24452_287_57173, &P24452_237};	/*Property = CssFlexbox*/
const Value V24452_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_288 = {S11, 0, &V24452_288_57176, &P24452_287};	/*Property = CssFont*/
const Value V24452_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_289 = {S12, 0, &V24452_289_86899, &P24452_288};	/*Property = CssImages*/
const Value V24452_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_291 = {S13, 0, &V24452_291_57182, &P24452_289};	/*Property = CssMediaQueries*/
const Value V24452_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_292 = {S14, 0, &V24452_292_57185, &P24452_291};	/*Property = CssMinMax*/
const Value V24452_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_293 = {S15, 0, &V24452_293_57188, &P24452_292};	/*Property = CssOverflow*/
const Value V24452_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_294 = {S16, 0, &V24452_294_57191, &P24452_293};	/*Property = CssPosition*/
const Value V24452_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_295 = {S17, 0, &V24452_295_57194, &P24452_294};	/*Property = CssText*/
const Value V24452_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_297 = {S18, 0, &V24452_297_57197, &P24452_295};	/*Property = CssTransforms*/
const Value V24452_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_241 = {S19, 0, &V24452_241_55801, &P24452_297};	/*Property = CssTransitions*/
const Value V24452_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_298 = {S20, 0, &V24452_298_57208, &P24452_241};	/*Property = CssUI*/
const Value V24452_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_300 = {S21, 0, &V24452_300_57217, &P24452_298};	/*Property = DataSet*/
const Value V24452_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_301 = {S22, 0, &V24452_301_57220, &P24452_300};	/*Property = DataUrl*/
const Value V24452_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_302 = {S23, 0, &V24452_302_57223, &P24452_301};	/*Property = DeviceOrientation*/
const Value V24452_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_304 = {S24, 0, &V24452_304_57229, &P24452_302};	/*Property = FileReader*/
const Value V24452_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_305 = {S25, 0, &V24452_305_57232, &P24452_304};	/*Property = FileSaver*/
const Value V24452_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_306 = {S26, 0, &V24452_306_57238, &P24452_305};	/*Property = FileWriter*/
const Value V24452_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_307 = {S27, 0, &V24452_307_57241, &P24452_306};	/*Property = FormData*/
const Value V24452_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_309 = {S28, 0, &V24452_309_57247, &P24452_307};	/*Property = Fullscreen*/
const Value V24452_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_310 = {S29, 0, &V24452_310_57250, &P24452_309};	/*Property = GeoLocation*/
const Value V24452_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_312 = {S30, 0, &V24452_312_57256, &P24452_310};	/*Property = History*/
const Value V24452_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_314 = {S31, 0, &V24452_314_57262, &P24452_312};	/*Property = Html5*/
const Value V24452_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_313 = {S32, 0, &V24452_313_57259, &P24452_314};	/*Property = Html-Media-Capture*/
const Value V24452_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_315 = {S33, 0, &V24452_315_57265, &P24452_313};	/*Property = Iframe*/
const Value V24452_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_316 = {S34, 0, &V24452_316_57268, &P24452_315};	/*Property = IndexedDB*/
const Value V24452_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_317 = {S35, 0, &V24452_317_57271, &P24452_316};	/*Property = Json*/
const Value V24452_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24452_15 = {S37, 0, &V24452_15_28, &P24452_317};	/*Property = LayoutEngine*/
const Value V24452_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_344 = {S38, 0, &V24452_344_57274, &P24452_15};	/*Property = Masking*/
const Value V24452_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_320 = {S39, 0, &V24452_320_57292, &P24452_344};	/*Property = PostMessage*/
const Value V24452_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_321 = {S40, 0, &V24452_321_57295, &P24452_320};	/*Property = Progress*/
const Value V24452_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_322 = {S41, 0, &V24452_322_57298, &P24452_321};	/*Property = Prompts*/
const Value V24452_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_323 = {S42, 0, &V24452_323_57301, &P24452_322};	/*Property = Selector*/
const Value V24452_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_324 = {S43, 0, &V24452_324_57307, &P24452_323};	/*Property = Svg*/
const Value V24452_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_327 = {S44, 0, &V24452_327_57316, &P24452_324};	/*Property = TouchEvents*/
const Value V24452_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_328 = {S45, 0, &V24452_328_57319, &P24452_327};	/*Property = Track*/
const Value V24452_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_329 = {S46, 0, &V24452_329_57325, &P24452_328};	/*Property = Video*/
const Value V24452_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_330 = {S47, 0, &V24452_330_57328, &P24452_329};	/*Property = Viewport*/
const Value V24452_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_332 = {S48, 0, &V24452_332_57340, &P24452_330};	/*Property = WebWorkers*/
const Value V24452_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24452_333 = {S49, 0, &V24452_333_57343, &P24452_332};	/*Property = Xhr2*/
const Profile P24452 = {"24452", &P24452_333};	/*Profile ID - 24452*/
const Value V18425_1_1 = {S2, NULL};	/*Value = True*/
const Property P18425_1 = {S50, 0, &V18425_1_1, NULL};	/*Property = IsMobile*/
const Value V18425_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18425_5 = {S52, 0, &V18425_5_1222, &P18425_1};	/*Property = ScreenPixelsHeight*/
const Value V18425_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18425_4 = {S54, 0, &V18425_4_1224, &P18425_5};	/*Property = ScreenPixelsWidth*/
const Profile P18425 = {"18425", &P18425_4};	/*Profile ID - 18425*/
const Value V21215_1_1 = {S2, NULL};	/*Value = True*/
const Property P21215_1 = {S50, 0, &V21215_1_1, NULL};	/*Property = IsMobile*/
const Value V21215_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21215_5 = {S52, 0, &V21215_5_1403, &P21215_1};	/*Property = ScreenPixelsHeight*/
const Value V21215_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21215_4 = {S54, 0, &V21215_4_1404, &P21215_5};	/*Property = ScreenPixelsWidth*/
const Profile P21215 = {"21215", &P21215_4};	/*Profile ID - 21215*/
const Value V19756_1_1 = {S2, NULL};	/*Value = True*/
const Property P19756_1 = {S50, 0, &V19756_1_1, NULL};	/*Property = IsMobile*/
const Value V19756_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P19756_5 = {S52, 0, &V19756_5_1458, &P19756_1};	/*Property = ScreenPixelsHeight*/
const Value V19756_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P19756_4 = {S54, 0, &V19756_4_1460, &P19756_5};	/*Property = ScreenPixelsWidth*/
const Profile P19756 = {"19756", &P19756_4};	/*Profile ID - 19756*/
const Value V23784_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_279 = {S1, 0, &V23784_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23784_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_282 = {S3, 0, &V23784_282_57155, &P23784_279};	/*Property = BlobBuilder*/
const Value V23784_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_350 = {S4, 0, &V23784_350_57446, &P23784_282};	/*Property = Canvas*/
const Value V23784_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_283 = {S5, 0, &V23784_283_57161, &P23784_350};	/*Property = CssBackground*/
const Value V23784_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_284 = {S6, 0, &V23784_284_57164, &P23784_283};	/*Property = CssBorderImage*/
const Value V23784_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_285 = {S7, 0, &V23784_285_57167, &P23784_284};	/*Property = CssCanvas*/
const Value V23784_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_286 = {S8, 0, &V23784_286_57170, &P23784_285};	/*Property = CssColor*/
const Value V23784_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_237 = {S9, 0, &V23784_237_55786, &P23784_286};	/*Property = CssColumn*/
const Value V23784_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_287 = {S10, 0, &V23784_287_57173, &P23784_237};	/*Property = CssFlexbox*/
const Value V23784_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_288 = {S11, 0, &V23784_288_57176, &P23784_287};	/*Property = CssFont*/
const Value V23784_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_289 = {S12, 0, &V23784_289_86899, &P23784_288};	/*Property = CssImages*/
const Value V23784_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_291 = {S13, 0, &V23784_291_57182, &P23784_289};	/*Property = CssMediaQueries*/
const Value V23784_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_292 = {S14, 0, &V23784_292_57185, &P23784_291};	/*Property = CssMinMax*/
const Value V23784_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_293 = {S15, 0, &V23784_293_57188, &P23784_292};	/*Property = CssOverflow*/
const Value V23784_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_294 = {S16, 0, &V23784_294_57191, &P23784_293};	/*Property = CssPosition*/
const Value V23784_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_295 = {S17, 0, &V23784_295_57194, &P23784_294};	/*Property = CssText*/
const Value V23784_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_297 = {S18, 0, &V23784_297_57197, &P23784_295};	/*Property = CssTransforms*/
const Value V23784_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_241 = {S19, 0, &V23784_241_55801, &P23784_297};	/*Property = CssTransitions*/
const Value V23784_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_298 = {S20, 0, &V23784_298_57208, &P23784_241};	/*Property = CssUI*/
const Value V23784_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_300 = {S21, 0, &V23784_300_57217, &P23784_298};	/*Property = DataSet*/
const Value V23784_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_301 = {S22, 0, &V23784_301_57220, &P23784_300};	/*Property = DataUrl*/
const Value V23784_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_302 = {S23, 0, &V23784_302_57223, &P23784_301};	/*Property = DeviceOrientation*/
const Value V23784_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_304 = {S24, 0, &V23784_304_57229, &P23784_302};	/*Property = FileReader*/
const Value V23784_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_305 = {S25, 0, &V23784_305_57232, &P23784_304};	/*Property = FileSaver*/
const Value V23784_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_306 = {S26, 0, &V23784_306_57238, &P23784_305};	/*Property = FileWriter*/
const Value V23784_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_307 = {S27, 0, &V23784_307_57241, &P23784_306};	/*Property = FormData*/
const Value V23784_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_309 = {S28, 0, &V23784_309_57247, &P23784_307};	/*Property = Fullscreen*/
const Value V23784_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_310 = {S29, 0, &V23784_310_57250, &P23784_309};	/*Property = GeoLocation*/
const Value V23784_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_312 = {S30, 0, &V23784_312_57256, &P23784_310};	/*Property = History*/
const Value V23784_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_314 = {S31, 0, &V23784_314_57262, &P23784_312};	/*Property = Html5*/
const Value V23784_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_313 = {S32, 0, &V23784_313_57259, &P23784_314};	/*Property = Html-Media-Capture*/
const Value V23784_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_315 = {S33, 0, &V23784_315_57265, &P23784_313};	/*Property = Iframe*/
const Value V23784_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_316 = {S34, 0, &V23784_316_57268, &P23784_315};	/*Property = IndexedDB*/
const Value V23784_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_317 = {S35, 0, &V23784_317_57271, &P23784_316};	/*Property = Json*/
const Value V23784_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_15 = {S37, 0, &V23784_15_72, &P23784_317};	/*Property = LayoutEngine*/
const Value V23784_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_344 = {S38, 0, &V23784_344_57274, &P23784_15};	/*Property = Masking*/
const Value V23784_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_320 = {S39, 0, &V23784_320_57292, &P23784_344};	/*Property = PostMessage*/
const Value V23784_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_321 = {S40, 0, &V23784_321_57295, &P23784_320};	/*Property = Progress*/
const Value V23784_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_322 = {S41, 0, &V23784_322_57298, &P23784_321};	/*Property = Prompts*/
const Value V23784_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_323 = {S42, 0, &V23784_323_57301, &P23784_322};	/*Property = Selector*/
const Value V23784_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_324 = {S43, 0, &V23784_324_57307, &P23784_323};	/*Property = Svg*/
const Value V23784_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_327 = {S44, 0, &V23784_327_57316, &P23784_324};	/*Property = TouchEvents*/
const Value V23784_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_328 = {S45, 0, &V23784_328_57319, &P23784_327};	/*Property = Track*/
const Value V23784_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_329 = {S46, 0, &V23784_329_57325, &P23784_328};	/*Property = Video*/
const Value V23784_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_330 = {S47, 0, &V23784_330_57328, &P23784_329};	/*Property = Viewport*/
const Value V23784_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_332 = {S48, 0, &V23784_332_57340, &P23784_330};	/*Property = WebWorkers*/
const Value V23784_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23784_333 = {S49, 0, &V23784_333_57343, &P23784_332};	/*Property = Xhr2*/
const Profile P23784 = {"23784", &P23784_333};	/*Profile ID - 23784*/
const Value V19757_1_1 = {S2, NULL};	/*Value = True*/
const Property P19757_1 = {S50, 0, &V19757_1_1, NULL};	/*Property = IsMobile*/
const Value V19757_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19757_5 = {S52, 0, &V19757_5_1227, &P19757_1};	/*Property = ScreenPixelsHeight*/
const Value V19757_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P19757_4 = {S54, 0, &V19757_4_1460, &P19757_5};	/*Property = ScreenPixelsWidth*/
const Profile P19757 = {"19757", &P19757_4};	/*Profile ID - 19757*/
const Value V22762_1_1 = {S2, NULL};	/*Value = True*/
const Property P22762_1 = {S50, 0, &V22762_1_1, NULL};	/*Property = IsMobile*/
const Value V22762_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22762_5 = {S52, 0, &V22762_5_9981, &P22762_1};	/*Property = ScreenPixelsHeight*/
const Value V22762_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22762_4 = {S54, 0, &V22762_4_39098, &P22762_5};	/*Property = ScreenPixelsWidth*/
const Profile P22762 = {"22762", &P22762_4};	/*Profile ID - 22762*/
const Value V18300_1_1 = {S2, NULL};	/*Value = True*/
const Property P18300_1 = {S50, 0, &V18300_1_1, NULL};	/*Property = IsMobile*/
const Value V18300_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18300_5 = {S52, 0, &V18300_5_1227, &P18300_1};	/*Property = ScreenPixelsHeight*/
const Value V18300_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18300_4 = {S54, 0, &V18300_4_1229, &P18300_5};	/*Property = ScreenPixelsWidth*/
const Profile P18300 = {"18300", &P18300_4};	/*Profile ID - 18300*/
const Value V21699_1_1 = {S2, NULL};	/*Value = True*/
const Property P21699_1 = {S50, 0, &V21699_1_1, NULL};	/*Property = IsMobile*/
const Value V21699_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21699_5 = {S52, 0, &V21699_5_1277, &P21699_1};	/*Property = ScreenPixelsHeight*/
const Value V21699_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21699_4 = {S54, 0, &V21699_4_1279, &P21699_5};	/*Property = ScreenPixelsWidth*/
const Profile P21699 = {"21699", &P21699_4};	/*Profile ID - 21699*/
const Value V20938_1_1 = {S2, NULL};	/*Value = True*/
const Property P20938_1 = {S50, 0, &V20938_1_1, NULL};	/*Property = IsMobile*/
const Value V20938_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P20938_5 = {S52, 0, &V20938_5_1304, &P20938_1};	/*Property = ScreenPixelsHeight*/
const Value V20938_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20938_4 = {S54, 0, &V20938_4_1224, &P20938_5};	/*Property = ScreenPixelsWidth*/
const Profile P20938 = {"20938", &P20938_4};	/*Profile ID - 20938*/
const Value V15903_1_1 = {S2, NULL};	/*Value = True*/
const Property P15903_1 = {S50, 0, &V15903_1_1, NULL};	/*Property = IsMobile*/
const Value V15903_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P15903_5 = {S52, 0, &V15903_5_37, &P15903_1};	/*Property = ScreenPixelsHeight*/
const Value V15903_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P15903_4 = {S54, 0, &V15903_4_36, &P15903_5};	/*Property = ScreenPixelsWidth*/
const Profile P15903 = {"15903", &P15903_4};	/*Profile ID - 15903*/
const Value V18285_1_1 = {S2, NULL};	/*Value = True*/
const Property P18285_1 = {S50, 0, &V18285_1_1, NULL};	/*Property = IsMobile*/
const Value V18285_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P18285_5 = {S52, 0, &V18285_5_1380, &P18285_1};	/*Property = ScreenPixelsHeight*/
const Value V18285_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18285_4 = {S54, 0, &V18285_4_1279, &P18285_5};	/*Property = ScreenPixelsWidth*/
const Profile P18285 = {"18285", &P18285_4};	/*Profile ID - 18285*/
const Value V15785_1_1 = {S2, NULL};	/*Value = True*/
const Property P15785_1 = {S50, 0, &V15785_1_1, NULL};	/*Property = IsMobile*/
const Value V15785_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15785_5 = {S52, 0, &V15785_5_1380, &P15785_1};	/*Property = ScreenPixelsHeight*/
const Value V15785_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15785_4 = {S54, 0, &V15785_4_1279, &P15785_5};	/*Property = ScreenPixelsWidth*/
const Profile P15785 = {"15785", &P15785_4};	/*Profile ID - 15785*/
const Value V20458_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_279 = {S1, 0, &V20458_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20458_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_282 = {S3, 0, &V20458_282_57155, &P20458_279};	/*Property = BlobBuilder*/
const Value V20458_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_350 = {S4, 0, &V20458_350_57446, &P20458_282};	/*Property = Canvas*/
const Value V20458_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_283 = {S5, 0, &V20458_283_57161, &P20458_350};	/*Property = CssBackground*/
const Value V20458_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_284 = {S6, 0, &V20458_284_57164, &P20458_283};	/*Property = CssBorderImage*/
const Value V20458_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_285 = {S7, 0, &V20458_285_57167, &P20458_284};	/*Property = CssCanvas*/
const Value V20458_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_286 = {S8, 0, &V20458_286_57170, &P20458_285};	/*Property = CssColor*/
const Value V20458_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_237 = {S9, 0, &V20458_237_55786, &P20458_286};	/*Property = CssColumn*/
const Value V20458_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_287 = {S10, 0, &V20458_287_57173, &P20458_237};	/*Property = CssFlexbox*/
const Value V20458_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_288 = {S11, 0, &V20458_288_57176, &P20458_287};	/*Property = CssFont*/
const Value V20458_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_289 = {S12, 0, &V20458_289_86899, &P20458_288};	/*Property = CssImages*/
const Value V20458_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_291 = {S13, 0, &V20458_291_57182, &P20458_289};	/*Property = CssMediaQueries*/
const Value V20458_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_292 = {S14, 0, &V20458_292_57185, &P20458_291};	/*Property = CssMinMax*/
const Value V20458_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_293 = {S15, 0, &V20458_293_57188, &P20458_292};	/*Property = CssOverflow*/
const Value V20458_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_294 = {S16, 0, &V20458_294_57191, &P20458_293};	/*Property = CssPosition*/
const Value V20458_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_295 = {S17, 0, &V20458_295_57194, &P20458_294};	/*Property = CssText*/
const Value V20458_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_297 = {S18, 0, &V20458_297_57197, &P20458_295};	/*Property = CssTransforms*/
const Value V20458_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_241 = {S19, 0, &V20458_241_55801, &P20458_297};	/*Property = CssTransitions*/
const Value V20458_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_298 = {S20, 0, &V20458_298_57208, &P20458_241};	/*Property = CssUI*/
const Value V20458_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_300 = {S21, 0, &V20458_300_57217, &P20458_298};	/*Property = DataSet*/
const Value V20458_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_301 = {S22, 0, &V20458_301_57220, &P20458_300};	/*Property = DataUrl*/
const Value V20458_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_302 = {S23, 0, &V20458_302_57223, &P20458_301};	/*Property = DeviceOrientation*/
const Value V20458_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_304 = {S24, 0, &V20458_304_57229, &P20458_302};	/*Property = FileReader*/
const Value V20458_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_305 = {S25, 0, &V20458_305_57232, &P20458_304};	/*Property = FileSaver*/
const Value V20458_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_306 = {S26, 0, &V20458_306_57238, &P20458_305};	/*Property = FileWriter*/
const Value V20458_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_307 = {S27, 0, &V20458_307_57241, &P20458_306};	/*Property = FormData*/
const Value V20458_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_309 = {S28, 0, &V20458_309_57247, &P20458_307};	/*Property = Fullscreen*/
const Value V20458_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_310 = {S29, 0, &V20458_310_57250, &P20458_309};	/*Property = GeoLocation*/
const Value V20458_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_312 = {S30, 0, &V20458_312_57256, &P20458_310};	/*Property = History*/
const Value V20458_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_314 = {S31, 0, &V20458_314_57262, &P20458_312};	/*Property = Html5*/
const Value V20458_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_313 = {S32, 0, &V20458_313_57259, &P20458_314};	/*Property = Html-Media-Capture*/
const Value V20458_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_315 = {S33, 0, &V20458_315_57265, &P20458_313};	/*Property = Iframe*/
const Value V20458_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_316 = {S34, 0, &V20458_316_57268, &P20458_315};	/*Property = IndexedDB*/
const Value V20458_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_317 = {S35, 0, &V20458_317_57271, &P20458_316};	/*Property = Json*/
const Value V20458_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20458_15 = {S37, 0, &V20458_15_29, &P20458_317};	/*Property = LayoutEngine*/
const Value V20458_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_344 = {S38, 0, &V20458_344_57274, &P20458_15};	/*Property = Masking*/
const Value V20458_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_320 = {S39, 0, &V20458_320_57292, &P20458_344};	/*Property = PostMessage*/
const Value V20458_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_321 = {S40, 0, &V20458_321_57295, &P20458_320};	/*Property = Progress*/
const Value V20458_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_322 = {S41, 0, &V20458_322_57298, &P20458_321};	/*Property = Prompts*/
const Value V20458_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_323 = {S42, 0, &V20458_323_57301, &P20458_322};	/*Property = Selector*/
const Value V20458_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_324 = {S43, 0, &V20458_324_57307, &P20458_323};	/*Property = Svg*/
const Value V20458_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_327 = {S44, 0, &V20458_327_57316, &P20458_324};	/*Property = TouchEvents*/
const Value V20458_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_328 = {S45, 0, &V20458_328_57319, &P20458_327};	/*Property = Track*/
const Value V20458_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_329 = {S46, 0, &V20458_329_57325, &P20458_328};	/*Property = Video*/
const Value V20458_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_330 = {S47, 0, &V20458_330_57328, &P20458_329};	/*Property = Viewport*/
const Value V20458_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_332 = {S48, 0, &V20458_332_57340, &P20458_330};	/*Property = WebWorkers*/
const Value V20458_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20458_333 = {S49, 0, &V20458_333_57343, &P20458_332};	/*Property = Xhr2*/
const Profile P20458 = {"20458", &P20458_333};	/*Profile ID - 20458*/
const Profile P18578 = {"18578", NULL};	/*Profile ID - 18578*/
const Value V15845_1_1 = {S2, NULL};	/*Value = True*/
const Property P15845_1 = {S50, 0, &V15845_1_1, NULL};	/*Property = IsMobile*/
const Value V15845_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P15845_5 = {S52, 0, &V15845_5_1275, &P15845_1};	/*Property = ScreenPixelsHeight*/
const Value V15845_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P15845_4 = {S54, 0, &V15845_4_1278, &P15845_5};	/*Property = ScreenPixelsWidth*/
const Profile P15845 = {"15845", &P15845_4};	/*Profile ID - 15845*/
const Value V20464_1_1 = {S2, NULL};	/*Value = True*/
const Property P20464_1 = {S50, 0, &V20464_1_1, NULL};	/*Property = IsMobile*/
const Value V20464_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20464_5 = {S52, 0, &V20464_5_9981, &P20464_1};	/*Property = ScreenPixelsHeight*/
const Value V20464_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20464_4 = {S54, 0, &V20464_4_37617, &P20464_5};	/*Property = ScreenPixelsWidth*/
const Profile P20464 = {"20464", &P20464_4};	/*Profile ID - 20464*/
const Value V19909_1_1 = {S2, NULL};	/*Value = True*/
const Property P19909_1 = {S50, 0, &V19909_1_1, NULL};	/*Property = IsMobile*/
const Value V19909_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19909_5 = {S52, 0, &V19909_5_9981, &P19909_1};	/*Property = ScreenPixelsHeight*/
const Value V19909_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P19909_4 = {S54, 0, &V19909_4_39098, &P19909_5};	/*Property = ScreenPixelsWidth*/
const Profile P19909 = {"19909", &P19909_4};	/*Profile ID - 19909*/
const Value V22184_1_1 = {S2, NULL};	/*Value = True*/
const Property P22184_1 = {S50, 0, &V22184_1_1, NULL};	/*Property = IsMobile*/
const Value V22184_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22184_5 = {S52, 0, &V22184_5_1227, &P22184_1};	/*Property = ScreenPixelsHeight*/
const Value V22184_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22184_4 = {S54, 0, &V22184_4_1460, &P22184_5};	/*Property = ScreenPixelsWidth*/
const Profile P22184 = {"22184", &P22184_4};	/*Profile ID - 22184*/
const Value V20861_1_1 = {S2, NULL};	/*Value = True*/
const Property P20861_1 = {S50, 0, &V20861_1_1, NULL};	/*Property = IsMobile*/
const Value V20861_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20861_5 = {S52, 0, &V20861_5_1227, &P20861_1};	/*Property = ScreenPixelsHeight*/
const Value V20861_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20861_4 = {S54, 0, &V20861_4_1229, &P20861_5};	/*Property = ScreenPixelsWidth*/
const Profile P20861 = {"20861", &P20861_4};	/*Profile ID - 20861*/
const Profile P22308 = {"22308", NULL};	/*Profile ID - 22308*/
const Value V18474_1_1 = {S2, NULL};	/*Value = True*/
const Property P18474_1 = {S50, 0, &V18474_1_1, NULL};	/*Property = IsMobile*/
const Value V18474_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18474_5 = {S52, 0, &V18474_5_1227, &P18474_1};	/*Property = ScreenPixelsHeight*/
const Value V18474_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18474_4 = {S54, 0, &V18474_4_1229, &P18474_5};	/*Property = ScreenPixelsWidth*/
const Profile P18474 = {"18474", &P18474_4};	/*Profile ID - 18474*/
const Value V18704_1_1 = {S2, NULL};	/*Value = True*/
const Property P18704_1 = {S50, 0, &V18704_1_1, NULL};	/*Property = IsMobile*/
const Value V18704_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18704_5 = {S52, 0, &V18704_5_9981, &P18704_1};	/*Property = ScreenPixelsHeight*/
const Value V18704_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18704_4 = {S54, 0, &V18704_4_2336, &P18704_5};	/*Property = ScreenPixelsWidth*/
const Profile P18704 = {"18704", &P18704_4};	/*Profile ID - 18704*/
const Value V18622_1_1 = {S2, NULL};	/*Value = True*/
const Property P18622_1 = {S50, 0, &V18622_1_1, NULL};	/*Property = IsMobile*/
const Value V18622_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18622_5 = {S52, 0, &V18622_5_9981, &P18622_1};	/*Property = ScreenPixelsHeight*/
const Value V18622_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18622_4 = {S54, 0, &V18622_4_2336, &P18622_5};	/*Property = ScreenPixelsWidth*/
const Profile P18622 = {"18622", &P18622_4};	/*Profile ID - 18622*/
const Value V18536_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_279 = {S1, 0, &V18536_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18536_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_282 = {S3, 0, &V18536_282_57155, &P18536_279};	/*Property = BlobBuilder*/
const Value V18536_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_350 = {S4, 0, &V18536_350_57446, &P18536_282};	/*Property = Canvas*/
const Value V18536_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_283 = {S5, 0, &V18536_283_57161, &P18536_350};	/*Property = CssBackground*/
const Value V18536_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_284 = {S6, 0, &V18536_284_57164, &P18536_283};	/*Property = CssBorderImage*/
const Value V18536_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_285 = {S7, 0, &V18536_285_57167, &P18536_284};	/*Property = CssCanvas*/
const Value V18536_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_286 = {S8, 0, &V18536_286_57170, &P18536_285};	/*Property = CssColor*/
const Value V18536_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_237 = {S9, 0, &V18536_237_55786, &P18536_286};	/*Property = CssColumn*/
const Value V18536_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_287 = {S10, 0, &V18536_287_57173, &P18536_237};	/*Property = CssFlexbox*/
const Value V18536_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_288 = {S11, 0, &V18536_288_57176, &P18536_287};	/*Property = CssFont*/
const Value V18536_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_289 = {S12, 0, &V18536_289_86899, &P18536_288};	/*Property = CssImages*/
const Value V18536_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_291 = {S13, 0, &V18536_291_57182, &P18536_289};	/*Property = CssMediaQueries*/
const Value V18536_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_292 = {S14, 0, &V18536_292_57185, &P18536_291};	/*Property = CssMinMax*/
const Value V18536_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_293 = {S15, 0, &V18536_293_57188, &P18536_292};	/*Property = CssOverflow*/
const Value V18536_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_294 = {S16, 0, &V18536_294_57191, &P18536_293};	/*Property = CssPosition*/
const Value V18536_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_295 = {S17, 0, &V18536_295_57194, &P18536_294};	/*Property = CssText*/
const Value V18536_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_297 = {S18, 0, &V18536_297_57197, &P18536_295};	/*Property = CssTransforms*/
const Value V18536_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_241 = {S19, 0, &V18536_241_55801, &P18536_297};	/*Property = CssTransitions*/
const Value V18536_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_298 = {S20, 0, &V18536_298_57208, &P18536_241};	/*Property = CssUI*/
const Value V18536_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_300 = {S21, 0, &V18536_300_57217, &P18536_298};	/*Property = DataSet*/
const Value V18536_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_301 = {S22, 0, &V18536_301_57220, &P18536_300};	/*Property = DataUrl*/
const Value V18536_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_302 = {S23, 0, &V18536_302_57223, &P18536_301};	/*Property = DeviceOrientation*/
const Value V18536_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_304 = {S24, 0, &V18536_304_57229, &P18536_302};	/*Property = FileReader*/
const Value V18536_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_305 = {S25, 0, &V18536_305_57232, &P18536_304};	/*Property = FileSaver*/
const Value V18536_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_306 = {S26, 0, &V18536_306_57238, &P18536_305};	/*Property = FileWriter*/
const Value V18536_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_307 = {S27, 0, &V18536_307_57241, &P18536_306};	/*Property = FormData*/
const Value V18536_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_309 = {S28, 0, &V18536_309_57247, &P18536_307};	/*Property = Fullscreen*/
const Value V18536_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_310 = {S29, 0, &V18536_310_57250, &P18536_309};	/*Property = GeoLocation*/
const Value V18536_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_312 = {S30, 0, &V18536_312_57256, &P18536_310};	/*Property = History*/
const Value V18536_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_314 = {S31, 0, &V18536_314_57262, &P18536_312};	/*Property = Html5*/
const Value V18536_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_313 = {S32, 0, &V18536_313_57259, &P18536_314};	/*Property = Html-Media-Capture*/
const Value V18536_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_315 = {S33, 0, &V18536_315_57265, &P18536_313};	/*Property = Iframe*/
const Value V18536_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_316 = {S34, 0, &V18536_316_57268, &P18536_315};	/*Property = IndexedDB*/
const Value V18536_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_317 = {S35, 0, &V18536_317_57271, &P18536_316};	/*Property = Json*/
const Value V18536_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18536_15 = {S37, 0, &V18536_15_28, &P18536_317};	/*Property = LayoutEngine*/
const Value V18536_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_344 = {S38, 0, &V18536_344_57274, &P18536_15};	/*Property = Masking*/
const Value V18536_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_320 = {S39, 0, &V18536_320_57292, &P18536_344};	/*Property = PostMessage*/
const Value V18536_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_321 = {S40, 0, &V18536_321_57295, &P18536_320};	/*Property = Progress*/
const Value V18536_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_322 = {S41, 0, &V18536_322_57298, &P18536_321};	/*Property = Prompts*/
const Value V18536_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_323 = {S42, 0, &V18536_323_57301, &P18536_322};	/*Property = Selector*/
const Value V18536_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_324 = {S43, 0, &V18536_324_57307, &P18536_323};	/*Property = Svg*/
const Value V18536_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_327 = {S44, 0, &V18536_327_57316, &P18536_324};	/*Property = TouchEvents*/
const Value V18536_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_328 = {S45, 0, &V18536_328_57319, &P18536_327};	/*Property = Track*/
const Value V18536_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_329 = {S46, 0, &V18536_329_57325, &P18536_328};	/*Property = Video*/
const Value V18536_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_330 = {S47, 0, &V18536_330_57328, &P18536_329};	/*Property = Viewport*/
const Value V18536_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_332 = {S48, 0, &V18536_332_57340, &P18536_330};	/*Property = WebWorkers*/
const Value V18536_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18536_333 = {S49, 0, &V18536_333_57343, &P18536_332};	/*Property = Xhr2*/
const Profile P18536 = {"18536", &P18536_333};	/*Profile ID - 18536*/
const Value V16345_1_1 = {S2, NULL};	/*Value = True*/
const Property P16345_1 = {S50, 0, &V16345_1_1, NULL};	/*Property = IsMobile*/
const Value V16345_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16345_5 = {S52, 0, &V16345_5_1222, &P16345_1};	/*Property = ScreenPixelsHeight*/
const Value V16345_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16345_4 = {S54, 0, &V16345_4_1224, &P16345_5};	/*Property = ScreenPixelsWidth*/
const Profile P16345 = {"16345", &P16345_4};	/*Profile ID - 16345*/
const Value V20957_1_1 = {S2, NULL};	/*Value = True*/
const Property P20957_1 = {S50, 0, &V20957_1_1, NULL};	/*Property = IsMobile*/
const Value V20957_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20957_5 = {S52, 0, &V20957_5_1227, &P20957_1};	/*Property = ScreenPixelsHeight*/
const Value V20957_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20957_4 = {S54, 0, &V20957_4_1229, &P20957_5};	/*Property = ScreenPixelsWidth*/
const Profile P20957 = {"20957", &P20957_4};	/*Profile ID - 20957*/
const Value V21453_1_1 = {S2, NULL};	/*Value = True*/
const Property P21453_1 = {S50, 0, &V21453_1_1, NULL};	/*Property = IsMobile*/
const Value V21453_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21453_5 = {S52, 0, &V21453_5_1458, &P21453_1};	/*Property = ScreenPixelsHeight*/
const Value V21453_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21453_4 = {S54, 0, &V21453_4_1460, &P21453_5};	/*Property = ScreenPixelsWidth*/
const Profile P21453 = {"21453", &P21453_4};	/*Profile ID - 21453*/
const Value V20936_1_1 = {S2, NULL};	/*Value = True*/
const Property P20936_1 = {S50, 0, &V20936_1_1, NULL};	/*Property = IsMobile*/
const Value V20936_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20936_5 = {S52, 0, &V20936_5_1222, &P20936_1};	/*Property = ScreenPixelsHeight*/
const Value V20936_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20936_4 = {S54, 0, &V20936_4_1224, &P20936_5};	/*Property = ScreenPixelsWidth*/
const Profile P20936 = {"20936", &P20936_4};	/*Profile ID - 20936*/
const Value V18500_1_1 = {S2, NULL};	/*Value = True*/
const Property P18500_1 = {S50, 0, &V18500_1_1, NULL};	/*Property = IsMobile*/
const Value V18500_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18500_5 = {S52, 0, &V18500_5_1277, &P18500_1};	/*Property = ScreenPixelsHeight*/
const Value V18500_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18500_4 = {S54, 0, &V18500_4_1279, &P18500_5};	/*Property = ScreenPixelsWidth*/
const Profile P18500 = {"18500", &P18500_4};	/*Profile ID - 18500*/
const Value V21244_1_1 = {S2, NULL};	/*Value = True*/
const Property P21244_1 = {S50, 0, &V21244_1_1, NULL};	/*Property = IsMobile*/
const Value V21244_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21244_5 = {S52, 0, &V21244_5_9981, &P21244_1};	/*Property = ScreenPixelsHeight*/
const Value V21244_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21244_4 = {S54, 0, &V21244_4_2336, &P21244_5};	/*Property = ScreenPixelsWidth*/
const Profile P21244 = {"21244", &P21244_4};	/*Profile ID - 21244*/
const Value V21025_1_1 = {S2, NULL};	/*Value = True*/
const Property P21025_1 = {S50, 0, &V21025_1_1, NULL};	/*Property = IsMobile*/
const Value V21025_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21025_5 = {S52, 0, &V21025_5_1227, &P21025_1};	/*Property = ScreenPixelsHeight*/
const Value V21025_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21025_4 = {S54, 0, &V21025_4_1229, &P21025_5};	/*Property = ScreenPixelsWidth*/
const Profile P21025 = {"21025", &P21025_4};	/*Profile ID - 21025*/
const Value V20066_1_1 = {S2, NULL};	/*Value = True*/
const Property P20066_1 = {S50, 0, &V20066_1_1, NULL};	/*Property = IsMobile*/
const Value V20066_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20066_5 = {S52, 0, &V20066_5_1222, &P20066_1};	/*Property = ScreenPixelsHeight*/
const Value V20066_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20066_4 = {S54, 0, &V20066_4_1224, &P20066_5};	/*Property = ScreenPixelsWidth*/
const Profile P20066 = {"20066", &P20066_4};	/*Profile ID - 20066*/
const Value V21528_1_1 = {S2, NULL};	/*Value = True*/
const Property P21528_1 = {S50, 0, &V21528_1_1, NULL};	/*Property = IsMobile*/
const Value V21528_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21528_5 = {S52, 0, &V21528_5_1277, &P21528_1};	/*Property = ScreenPixelsHeight*/
const Value V21528_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21528_4 = {S54, 0, &V21528_4_1279, &P21528_5};	/*Property = ScreenPixelsWidth*/
const Profile P21528 = {"21528", &P21528_4};	/*Profile ID - 21528*/
const Value V18475_1_1 = {S2, NULL};	/*Value = True*/
const Property P18475_1 = {S50, 0, &V18475_1_1, NULL};	/*Property = IsMobile*/
const Value V18475_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18475_5 = {S52, 0, &V18475_5_9981, &P18475_1};	/*Property = ScreenPixelsHeight*/
const Value V18475_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18475_4 = {S54, 0, &V18475_4_2336, &P18475_5};	/*Property = ScreenPixelsWidth*/
const Profile P18475 = {"18475", &P18475_4};	/*Profile ID - 18475*/
const Value V20946_1_1 = {S2, NULL};	/*Value = True*/
const Property P20946_1 = {S50, 0, &V20946_1_1, NULL};	/*Property = IsMobile*/
const Value V20946_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20946_5 = {S52, 0, &V20946_5_1403, &P20946_1};	/*Property = ScreenPixelsHeight*/
const Value V20946_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P20946_4 = {S54, 0, &V20946_4_8554, &P20946_5};	/*Property = ScreenPixelsWidth*/
const Profile P20946 = {"20946", &P20946_4};	/*Profile ID - 20946*/
const Value V22064_1_1 = {S2, NULL};	/*Value = True*/
const Property P22064_1 = {S50, 0, &V22064_1_1, NULL};	/*Property = IsMobile*/
const Value V22064_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22064_5 = {S52, 0, &V22064_5_9981, &P22064_1};	/*Property = ScreenPixelsHeight*/
const Value V22064_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22064_4 = {S54, 0, &V22064_4_2336, &P22064_5};	/*Property = ScreenPixelsWidth*/
const Profile P22064 = {"22064", &P22064_4};	/*Profile ID - 22064*/
const Value V19854_1_1 = {S2, NULL};	/*Value = True*/
const Property P19854_1 = {S50, 0, &V19854_1_1, NULL};	/*Property = IsMobile*/
const Value V19854_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19854_5 = {S52, 0, &V19854_5_1227, &P19854_1};	/*Property = ScreenPixelsHeight*/
const Value V19854_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19854_4 = {S54, 0, &V19854_4_1229, &P19854_5};	/*Property = ScreenPixelsWidth*/
const Profile P19854 = {"19854", &P19854_4};	/*Profile ID - 19854*/
const Value V21780_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21780_279 = {S1, 0, &V21780_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21780_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21780_282 = {S3, 0, &V21780_282_57154, &P21780_279};	/*Property = BlobBuilder*/
const Value V21780_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21780_350 = {S4, 0, &V21780_350_57444, &P21780_282};	/*Property = Canvas*/
const Value V21780_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21780_283 = {S5, 0, &V21780_283_57159, &P21780_350};	/*Property = CssBackground*/
const Value V21780_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21780_284 = {S6, 0, &V21780_284_57163, &P21780_283};	/*Property = CssBorderImage*/
const Value V21780_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21780_285 = {S7, 0, &V21780_285_57166, &P21780_284};	/*Property = CssCanvas*/
const Value V21780_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21780_286 = {S8, 0, &V21780_286_57168, &P21780_285};	/*Property = CssColor*/
const Value V21780_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21780_237 = {S9, 0, &V21780_237_55786, &P21780_286};	/*Property = CssColumn*/
const Value V21780_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21780_287 = {S10, 0, &V21780_287_57172, &P21780_237};	/*Property = CssFlexbox*/
const Value V21780_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21780_288 = {S11, 0, &V21780_288_57175, &P21780_287};	/*Property = CssFont*/
const Value V21780_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21780_289 = {S12, 0, &V21780_289_57759, &P21780_288};	/*Property = CssImages*/
const Value V21780_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21780_291 = {S13, 0, &V21780_291_57181, &P21780_289};	/*Property = CssMediaQueries*/
const Value V21780_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21780_292 = {S14, 0, &V21780_292_57183, &P21780_291};	/*Property = CssMinMax*/
const Value V21780_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21780_293 = {S15, 0, &V21780_293_57187, &P21780_292};	/*Property = CssOverflow*/
const Value V21780_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21780_294 = {S16, 0, &V21780_294_57190, &P21780_293};	/*Property = CssPosition*/
const Value V21780_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21780_295 = {S17, 0, &V21780_295_57192, &P21780_294};	/*Property = CssText*/
const Value V21780_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21780_297 = {S18, 0, &V21780_297_57195, &P21780_295};	/*Property = CssTransforms*/
const Value V21780_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21780_241 = {S19, 0, &V21780_241_55799, &P21780_297};	/*Property = CssTransitions*/
const Value V21780_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21780_298 = {S20, 0, &V21780_298_57206, &P21780_241};	/*Property = CssUI*/
const Value V21780_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21780_300 = {S21, 0, &V21780_300_57216, &P21780_298};	/*Property = DataSet*/
const Value V21780_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21780_301 = {S22, 0, &V21780_301_57218, &P21780_300};	/*Property = DataUrl*/
const Value V21780_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21780_302 = {S23, 0, &V21780_302_57222, &P21780_301};	/*Property = DeviceOrientation*/
const Value V21780_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21780_304 = {S24, 0, &V21780_304_57228, &P21780_302};	/*Property = FileReader*/
const Value V21780_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21780_305 = {S25, 0, &V21780_305_57231, &P21780_304};	/*Property = FileSaver*/
const Value V21780_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21780_306 = {S26, 0, &V21780_306_57237, &P21780_305};	/*Property = FileWriter*/
const Value V21780_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21780_307 = {S27, 0, &V21780_307_57240, &P21780_306};	/*Property = FormData*/
const Value V21780_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21780_309 = {S28, 0, &V21780_309_57246, &P21780_307};	/*Property = Fullscreen*/
const Value V21780_310_57249 = {S0, NULL};	/*Value = False*/
const Property P21780_310 = {S29, 0, &V21780_310_57249, &P21780_309};	/*Property = GeoLocation*/
const Value V21780_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21780_312 = {S30, 0, &V21780_312_57255, &P21780_310};	/*Property = History*/
const Value V21780_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21780_314 = {S31, 0, &V21780_314_57261, &P21780_312};	/*Property = Html5*/
const Value V21780_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21780_313 = {S32, 0, &V21780_313_57258, &P21780_314};	/*Property = Html-Media-Capture*/
const Value V21780_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21780_315 = {S33, 0, &V21780_315_57264, &P21780_313};	/*Property = Iframe*/
const Value V21780_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21780_316 = {S34, 0, &V21780_316_57267, &P21780_315};	/*Property = IndexedDB*/
const Value V21780_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21780_317 = {S35, 0, &V21780_317_57269, &P21780_316};	/*Property = Json*/
const Value V21780_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21780_15 = {S37, 0, &V21780_15_54167, &P21780_317};	/*Property = LayoutEngine*/
const Value V21780_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21780_344 = {S38, 0, &V21780_344_57272, &P21780_15};	/*Property = Masking*/
const Value V21780_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21780_320 = {S39, 0, &V21780_320_57290, &P21780_344};	/*Property = PostMessage*/
const Value V21780_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21780_321 = {S40, 0, &V21780_321_57293, &P21780_320};	/*Property = Progress*/
const Value V21780_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21780_322 = {S41, 0, &V21780_322_57296, &P21780_321};	/*Property = Prompts*/
const Value V21780_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21780_323 = {S42, 0, &V21780_323_57299, &P21780_322};	/*Property = Selector*/
const Value V21780_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21780_324 = {S43, 0, &V21780_324_57306, &P21780_323};	/*Property = Svg*/
const Value V21780_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21780_327 = {S44, 0, &V21780_327_57315, &P21780_324};	/*Property = TouchEvents*/
const Value V21780_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21780_328 = {S45, 0, &V21780_328_57318, &P21780_327};	/*Property = Track*/
const Value V21780_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21780_329 = {S46, 0, &V21780_329_57325, &P21780_328};	/*Property = Video*/
const Value V21780_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21780_330 = {S47, 0, &V21780_330_57326, &P21780_329};	/*Property = Viewport*/
const Value V21780_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21780_332 = {S48, 0, &V21780_332_57339, &P21780_330};	/*Property = WebWorkers*/
const Value V21780_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21780_333 = {S49, 0, &V21780_333_57342, &P21780_332};	/*Property = Xhr2*/
const Profile P21780 = {"21780", &P21780_333};	/*Profile ID - 21780*/
const Value V20948_1_1 = {S2, NULL};	/*Value = True*/
const Property P20948_1 = {S50, 0, &V20948_1_1, NULL};	/*Property = IsMobile*/
const Value V20948_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20948_5 = {S52, 0, &V20948_5_1458, &P20948_1};	/*Property = ScreenPixelsHeight*/
const Value V20948_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20948_4 = {S54, 0, &V20948_4_37617, &P20948_5};	/*Property = ScreenPixelsWidth*/
const Profile P20948 = {"20948", &P20948_4};	/*Profile ID - 20948*/
const Value V18493_1_1 = {S2, NULL};	/*Value = True*/
const Property P18493_1 = {S50, 0, &V18493_1_1, NULL};	/*Property = IsMobile*/
const Value V18493_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18493_5 = {S52, 0, &V18493_5_1227, &P18493_1};	/*Property = ScreenPixelsHeight*/
const Value V18493_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18493_4 = {S54, 0, &V18493_4_1229, &P18493_5};	/*Property = ScreenPixelsWidth*/
const Profile P18493 = {"18493", &P18493_4};	/*Profile ID - 18493*/
const Value V21321_1_1 = {S2, NULL};	/*Value = True*/
const Property P21321_1 = {S50, 0, &V21321_1_1, NULL};	/*Property = IsMobile*/
const Value V21321_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21321_5 = {S52, 0, &V21321_5_9981, &P21321_1};	/*Property = ScreenPixelsHeight*/
const Value V21321_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21321_4 = {S54, 0, &V21321_4_2336, &P21321_5};	/*Property = ScreenPixelsWidth*/
const Profile P21321 = {"21321", &P21321_4};	/*Profile ID - 21321*/
const Value V21056_1_1 = {S2, NULL};	/*Value = True*/
const Property P21056_1 = {S50, 0, &V21056_1_1, NULL};	/*Property = IsMobile*/
const Value V21056_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21056_5 = {S52, 0, &V21056_5_1403, &P21056_1};	/*Property = ScreenPixelsHeight*/
const Value V21056_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21056_4 = {S54, 0, &V21056_4_1404, &P21056_5};	/*Property = ScreenPixelsWidth*/
const Profile P21056 = {"21056", &P21056_4};	/*Profile ID - 21056*/
const Value V21819_1_1 = {S2, NULL};	/*Value = True*/
const Property P21819_1 = {S50, 0, &V21819_1_1, NULL};	/*Property = IsMobile*/
const Value V21819_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21819_5 = {S52, 0, &V21819_5_1227, &P21819_1};	/*Property = ScreenPixelsHeight*/
const Value V21819_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21819_4 = {S54, 0, &V21819_4_1229, &P21819_5};	/*Property = ScreenPixelsWidth*/
const Profile P21819 = {"21819", &P21819_4};	/*Profile ID - 21819*/
const Value V20345_1_1 = {S2, NULL};	/*Value = True*/
const Property P20345_1 = {S50, 0, &V20345_1_1, NULL};	/*Property = IsMobile*/
const Value V20345_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20345_5 = {S52, 0, &V20345_5_37, &P20345_1};	/*Property = ScreenPixelsHeight*/
const Value V20345_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20345_4 = {S54, 0, &V20345_4_36, &P20345_5};	/*Property = ScreenPixelsWidth*/
const Profile P20345 = {"20345", &P20345_4};	/*Profile ID - 20345*/
const Value V23761_1_1 = {S2, NULL};	/*Value = True*/
const Property P23761_1 = {S50, 0, &V23761_1_1, NULL};	/*Property = IsMobile*/
const Value V23761_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23761_5 = {S52, 0, &V23761_5_1269, &P23761_1};	/*Property = ScreenPixelsHeight*/
const Value V23761_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23761_4 = {S54, 0, &V23761_4_1229, &P23761_5};	/*Property = ScreenPixelsWidth*/
const Profile P23761 = {"23761", &P23761_4};	/*Profile ID - 23761*/
const Value V21182_1_1 = {S2, NULL};	/*Value = True*/
const Property P21182_1 = {S50, 0, &V21182_1_1, NULL};	/*Property = IsMobile*/
const Value V21182_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21182_5 = {S52, 0, &V21182_5_1227, &P21182_1};	/*Property = ScreenPixelsHeight*/
const Value V21182_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21182_4 = {S54, 0, &V21182_4_1229, &P21182_5};	/*Property = ScreenPixelsWidth*/
const Profile P21182 = {"21182", &P21182_4};	/*Profile ID - 21182*/
const Value V18707_1_1 = {S2, NULL};	/*Value = True*/
const Property P18707_1 = {S50, 0, &V18707_1_1, NULL};	/*Property = IsMobile*/
const Value V18707_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18707_5 = {S52, 0, &V18707_5_9981, &P18707_1};	/*Property = ScreenPixelsHeight*/
const Value V18707_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18707_4 = {S54, 0, &V18707_4_2336, &P18707_5};	/*Property = ScreenPixelsWidth*/
const Profile P18707 = {"18707", &P18707_4};	/*Profile ID - 18707*/
const Value V19872_1_1 = {S2, NULL};	/*Value = True*/
const Property P19872_1 = {S50, 0, &V19872_1_1, NULL};	/*Property = IsMobile*/
const Value V19872_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19872_5 = {S52, 0, &V19872_5_1227, &P19872_1};	/*Property = ScreenPixelsHeight*/
const Value V19872_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19872_4 = {S54, 0, &V19872_4_1229, &P19872_5};	/*Property = ScreenPixelsWidth*/
const Profile P19872 = {"19872", &P19872_4};	/*Profile ID - 19872*/
const Value V22591_1_1 = {S2, NULL};	/*Value = True*/
const Property P22591_1 = {S50, 0, &V22591_1_1, NULL};	/*Property = IsMobile*/
const Value V22591_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22591_5 = {S52, 0, &V22591_5_9981, &P22591_1};	/*Property = ScreenPixelsHeight*/
const Value V22591_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22591_4 = {S54, 0, &V22591_4_2336, &P22591_5};	/*Property = ScreenPixelsWidth*/
const Profile P22591 = {"22591", &P22591_4};	/*Profile ID - 22591*/
const Value V18480_1_1 = {S2, NULL};	/*Value = True*/
const Property P18480_1 = {S50, 0, &V18480_1_1, NULL};	/*Property = IsMobile*/
const Value V18480_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18480_5 = {S52, 0, &V18480_5_1277, &P18480_1};	/*Property = ScreenPixelsHeight*/
const Value V18480_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18480_4 = {S54, 0, &V18480_4_1279, &P18480_5};	/*Property = ScreenPixelsWidth*/
const Profile P18480 = {"18480", &P18480_4};	/*Profile ID - 18480*/
const Value V21183_1_1 = {S2, NULL};	/*Value = True*/
const Property P21183_1 = {S50, 0, &V21183_1_1, NULL};	/*Property = IsMobile*/
const Value V21183_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21183_5 = {S52, 0, &V21183_5_1227, &P21183_1};	/*Property = ScreenPixelsHeight*/
const Value V21183_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21183_4 = {S54, 0, &V21183_4_1229, &P21183_5};	/*Property = ScreenPixelsWidth*/
const Profile P21183 = {"21183", &P21183_4};	/*Profile ID - 21183*/
const Value V18685_1_1 = {S2, NULL};	/*Value = True*/
const Property P18685_1 = {S50, 0, &V18685_1_1, NULL};	/*Property = IsMobile*/
const Value V18685_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18685_5 = {S52, 0, &V18685_5_1277, &P18685_1};	/*Property = ScreenPixelsHeight*/
const Value V18685_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18685_4 = {S54, 0, &V18685_4_1279, &P18685_5};	/*Property = ScreenPixelsWidth*/
const Profile P18685 = {"18685", &P18685_4};	/*Profile ID - 18685*/
const Value V18510_1_1 = {S2, NULL};	/*Value = True*/
const Property P18510_1 = {S50, 0, &V18510_1_1, NULL};	/*Property = IsMobile*/
const Value V18510_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18510_5 = {S52, 0, &V18510_5_1277, &P18510_1};	/*Property = ScreenPixelsHeight*/
const Value V18510_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18510_4 = {S54, 0, &V18510_4_1279, &P18510_5};	/*Property = ScreenPixelsWidth*/
const Profile P18510 = {"18510", &P18510_4};	/*Profile ID - 18510*/
const Value V18299_1_1 = {S2, NULL};	/*Value = True*/
const Property P18299_1 = {S50, 0, &V18299_1_1, NULL};	/*Property = IsMobile*/
const Value V18299_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P18299_5 = {S52, 0, &V18299_5_1458, &P18299_1};	/*Property = ScreenPixelsHeight*/
const Value V18299_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P18299_4 = {S54, 0, &V18299_4_37617, &P18299_5};	/*Property = ScreenPixelsWidth*/
const Profile P18299 = {"18299", &P18299_4};	/*Profile ID - 18299*/
const Value V21222_1_1 = {S2, NULL};	/*Value = True*/
const Property P21222_1 = {S50, 0, &V21222_1_1, NULL};	/*Property = IsMobile*/
const Value V21222_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21222_5 = {S52, 0, &V21222_5_9981, &P21222_1};	/*Property = ScreenPixelsHeight*/
const Value V21222_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21222_4 = {S54, 0, &V21222_4_2336, &P21222_5};	/*Property = ScreenPixelsWidth*/
const Profile P21222 = {"21222", &P21222_4};	/*Profile ID - 21222*/
const Value V18252_1_1 = {S2, NULL};	/*Value = True*/
const Property P18252_1 = {S50, 0, &V18252_1_1, NULL};	/*Property = IsMobile*/
const Value V18252_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P18252_5 = {S52, 0, &V18252_5_1269, &P18252_1};	/*Property = ScreenPixelsHeight*/
const Value V18252_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18252_4 = {S54, 0, &V18252_4_1229, &P18252_5};	/*Property = ScreenPixelsWidth*/
const Profile P18252 = {"18252", &P18252_4};	/*Profile ID - 18252*/
const Value V21307_1_1 = {S2, NULL};	/*Value = True*/
const Property P21307_1 = {S50, 0, &V21307_1_1, NULL};	/*Property = IsMobile*/
const Value V21307_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21307_5 = {S52, 0, &V21307_5_1277, &P21307_1};	/*Property = ScreenPixelsHeight*/
const Value V21307_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21307_4 = {S54, 0, &V21307_4_1279, &P21307_5};	/*Property = ScreenPixelsWidth*/
const Profile P21307 = {"21307", &P21307_4};	/*Profile ID - 21307*/
const Value V21203_1_1 = {S2, NULL};	/*Value = True*/
const Property P21203_1 = {S50, 0, &V21203_1_1, NULL};	/*Property = IsMobile*/
const Value V21203_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21203_5 = {S52, 0, &V21203_5_1227, &P21203_1};	/*Property = ScreenPixelsHeight*/
const Value V21203_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21203_4 = {S54, 0, &V21203_4_1229, &P21203_5};	/*Property = ScreenPixelsWidth*/
const Profile P21203 = {"21203", &P21203_4};	/*Profile ID - 21203*/
const Value V22938_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_279 = {S1, 0, &V22938_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22938_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_282 = {S3, 0, &V22938_282_57155, &P22938_279};	/*Property = BlobBuilder*/
const Value V22938_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_350 = {S4, 0, &V22938_350_57446, &P22938_282};	/*Property = Canvas*/
const Value V22938_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_283 = {S5, 0, &V22938_283_57161, &P22938_350};	/*Property = CssBackground*/
const Value V22938_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_284 = {S6, 0, &V22938_284_57164, &P22938_283};	/*Property = CssBorderImage*/
const Value V22938_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_285 = {S7, 0, &V22938_285_57167, &P22938_284};	/*Property = CssCanvas*/
const Value V22938_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_286 = {S8, 0, &V22938_286_57170, &P22938_285};	/*Property = CssColor*/
const Value V22938_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_237 = {S9, 0, &V22938_237_55786, &P22938_286};	/*Property = CssColumn*/
const Value V22938_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_287 = {S10, 0, &V22938_287_57173, &P22938_237};	/*Property = CssFlexbox*/
const Value V22938_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_288 = {S11, 0, &V22938_288_57176, &P22938_287};	/*Property = CssFont*/
const Value V22938_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_289 = {S12, 0, &V22938_289_86899, &P22938_288};	/*Property = CssImages*/
const Value V22938_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_291 = {S13, 0, &V22938_291_57182, &P22938_289};	/*Property = CssMediaQueries*/
const Value V22938_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_292 = {S14, 0, &V22938_292_57185, &P22938_291};	/*Property = CssMinMax*/
const Value V22938_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_293 = {S15, 0, &V22938_293_57188, &P22938_292};	/*Property = CssOverflow*/
const Value V22938_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_294 = {S16, 0, &V22938_294_57191, &P22938_293};	/*Property = CssPosition*/
const Value V22938_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_295 = {S17, 0, &V22938_295_57194, &P22938_294};	/*Property = CssText*/
const Value V22938_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_297 = {S18, 0, &V22938_297_57197, &P22938_295};	/*Property = CssTransforms*/
const Value V22938_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_241 = {S19, 0, &V22938_241_55801, &P22938_297};	/*Property = CssTransitions*/
const Value V22938_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_298 = {S20, 0, &V22938_298_57208, &P22938_241};	/*Property = CssUI*/
const Value V22938_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_300 = {S21, 0, &V22938_300_57217, &P22938_298};	/*Property = DataSet*/
const Value V22938_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_301 = {S22, 0, &V22938_301_57220, &P22938_300};	/*Property = DataUrl*/
const Value V22938_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_302 = {S23, 0, &V22938_302_57223, &P22938_301};	/*Property = DeviceOrientation*/
const Value V22938_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_304 = {S24, 0, &V22938_304_57229, &P22938_302};	/*Property = FileReader*/
const Value V22938_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_305 = {S25, 0, &V22938_305_57232, &P22938_304};	/*Property = FileSaver*/
const Value V22938_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_306 = {S26, 0, &V22938_306_57238, &P22938_305};	/*Property = FileWriter*/
const Value V22938_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_307 = {S27, 0, &V22938_307_57241, &P22938_306};	/*Property = FormData*/
const Value V22938_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_309 = {S28, 0, &V22938_309_57247, &P22938_307};	/*Property = Fullscreen*/
const Value V22938_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_310 = {S29, 0, &V22938_310_57250, &P22938_309};	/*Property = GeoLocation*/
const Value V22938_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_312 = {S30, 0, &V22938_312_57256, &P22938_310};	/*Property = History*/
const Value V22938_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_314 = {S31, 0, &V22938_314_57262, &P22938_312};	/*Property = Html5*/
const Value V22938_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_313 = {S32, 0, &V22938_313_57259, &P22938_314};	/*Property = Html-Media-Capture*/
const Value V22938_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_315 = {S33, 0, &V22938_315_57265, &P22938_313};	/*Property = Iframe*/
const Value V22938_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_316 = {S34, 0, &V22938_316_57268, &P22938_315};	/*Property = IndexedDB*/
const Value V22938_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_317 = {S35, 0, &V22938_317_57271, &P22938_316};	/*Property = Json*/
const Value V22938_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_15 = {S37, 0, &V22938_15_72, &P22938_317};	/*Property = LayoutEngine*/
const Value V22938_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_344 = {S38, 0, &V22938_344_57274, &P22938_15};	/*Property = Masking*/
const Value V22938_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_320 = {S39, 0, &V22938_320_57292, &P22938_344};	/*Property = PostMessage*/
const Value V22938_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_321 = {S40, 0, &V22938_321_57295, &P22938_320};	/*Property = Progress*/
const Value V22938_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_322 = {S41, 0, &V22938_322_57298, &P22938_321};	/*Property = Prompts*/
const Value V22938_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_323 = {S42, 0, &V22938_323_57301, &P22938_322};	/*Property = Selector*/
const Value V22938_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_324 = {S43, 0, &V22938_324_57307, &P22938_323};	/*Property = Svg*/
const Value V22938_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_327 = {S44, 0, &V22938_327_57316, &P22938_324};	/*Property = TouchEvents*/
const Value V22938_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_328 = {S45, 0, &V22938_328_57319, &P22938_327};	/*Property = Track*/
const Value V22938_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_329 = {S46, 0, &V22938_329_57325, &P22938_328};	/*Property = Video*/
const Value V22938_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_330 = {S47, 0, &V22938_330_57328, &P22938_329};	/*Property = Viewport*/
const Value V22938_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_332 = {S48, 0, &V22938_332_57340, &P22938_330};	/*Property = WebWorkers*/
const Value V22938_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22938_333 = {S49, 0, &V22938_333_57343, &P22938_332};	/*Property = Xhr2*/
const Profile P22938 = {"22938", &P22938_333};	/*Profile ID - 22938*/
const Value V21291_1_1 = {S2, NULL};	/*Value = True*/
const Property P21291_1 = {S50, 0, &V21291_1_1, NULL};	/*Property = IsMobile*/
const Value V21291_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P21291_5 = {S52, 0, &V21291_5_1275, &P21291_1};	/*Property = ScreenPixelsHeight*/
const Value V21291_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21291_4 = {S54, 0, &V21291_4_1229, &P21291_5};	/*Property = ScreenPixelsWidth*/
const Profile P21291 = {"21291", &P21291_4};	/*Profile ID - 21291*/
const Value V19758_1_1 = {S2, NULL};	/*Value = True*/
const Property P19758_1 = {S50, 0, &V19758_1_1, NULL};	/*Property = IsMobile*/
const Value V19758_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19758_5 = {S52, 0, &V19758_5_1227, &P19758_1};	/*Property = ScreenPixelsHeight*/
const Value V19758_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P19758_4 = {S54, 0, &V19758_4_1460, &P19758_5};	/*Property = ScreenPixelsWidth*/
const Profile P19758 = {"19758", &P19758_4};	/*Profile ID - 19758*/
const Value V18481_1_1 = {S2, NULL};	/*Value = True*/
const Property P18481_1 = {S50, 0, &V18481_1_1, NULL};	/*Property = IsMobile*/
const Value V18481_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18481_5 = {S52, 0, &V18481_5_1222, &P18481_1};	/*Property = ScreenPixelsHeight*/
const Value V18481_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18481_4 = {S54, 0, &V18481_4_1224, &P18481_5};	/*Property = ScreenPixelsWidth*/
const Profile P18481 = {"18481", &P18481_4};	/*Profile ID - 18481*/
const Value V21797_1_1 = {S2, NULL};	/*Value = True*/
const Property P21797_1 = {S50, 0, &V21797_1_1, NULL};	/*Property = IsMobile*/
const Value V21797_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21797_5 = {S52, 0, &V21797_5_37, &P21797_1};	/*Property = ScreenPixelsHeight*/
const Value V21797_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21797_4 = {S54, 0, &V21797_4_36, &P21797_5};	/*Property = ScreenPixelsWidth*/
const Profile P21797 = {"21797", &P21797_4};	/*Profile ID - 21797*/
const Value V18319_1_1 = {S2, NULL};	/*Value = True*/
const Property P18319_1 = {S50, 0, &V18319_1_1, NULL};	/*Property = IsMobile*/
const Value V18319_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18319_5 = {S52, 0, &V18319_5_1277, &P18319_1};	/*Property = ScreenPixelsHeight*/
const Value V18319_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18319_4 = {S54, 0, &V18319_4_1279, &P18319_5};	/*Property = ScreenPixelsWidth*/
const Profile P18319 = {"18319", &P18319_4};	/*Profile ID - 18319*/
const Value V22213_1_1 = {S2, NULL};	/*Value = True*/
const Property P22213_1 = {S50, 0, &V22213_1_1, NULL};	/*Property = IsMobile*/
const Value V22213_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22213_5 = {S52, 0, &V22213_5_1277, &P22213_1};	/*Property = ScreenPixelsHeight*/
const Value V22213_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22213_4 = {S54, 0, &V22213_4_2336, &P22213_5};	/*Property = ScreenPixelsWidth*/
const Profile P22213 = {"22213", &P22213_4};	/*Profile ID - 22213*/
const Value V18730_1_1 = {S2, NULL};	/*Value = True*/
const Property P18730_1 = {S50, 0, &V18730_1_1, NULL};	/*Property = IsMobile*/
const Value V18730_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18730_5 = {S52, 0, &V18730_5_1227, &P18730_1};	/*Property = ScreenPixelsHeight*/
const Value V18730_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18730_4 = {S54, 0, &V18730_4_1229, &P18730_5};	/*Property = ScreenPixelsWidth*/
const Profile P18730 = {"18730", &P18730_4};	/*Profile ID - 18730*/
const Value V19877_1_1 = {S2, NULL};	/*Value = True*/
const Property P19877_1 = {S50, 0, &V19877_1_1, NULL};	/*Property = IsMobile*/
const Value V19877_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P19877_5 = {S52, 0, &V19877_5_1458, &P19877_1};	/*Property = ScreenPixelsHeight*/
const Value V19877_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P19877_4 = {S54, 0, &V19877_4_1460, &P19877_5};	/*Property = ScreenPixelsWidth*/
const Profile P19877 = {"19877", &P19877_4};	/*Profile ID - 19877*/
const Value V18173_1_1 = {S2, NULL};	/*Value = True*/
const Property P18173_1 = {S50, 0, &V18173_1_1, NULL};	/*Property = IsMobile*/
const Value V18173_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18173_5 = {S52, 0, &V18173_5_9981, &P18173_1};	/*Property = ScreenPixelsHeight*/
const Value V18173_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18173_4 = {S54, 0, &V18173_4_39098, &P18173_5};	/*Property = ScreenPixelsWidth*/
const Profile P18173 = {"18173", &P18173_4};	/*Profile ID - 18173*/
const Value V20370_1_1 = {S2, NULL};	/*Value = True*/
const Property P20370_1 = {S50, 0, &V20370_1_1, NULL};	/*Property = IsMobile*/
const Value V20370_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20370_5 = {S52, 0, &V20370_5_37, &P20370_1};	/*Property = ScreenPixelsHeight*/
const Value V20370_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20370_4 = {S54, 0, &V20370_4_36, &P20370_5};	/*Property = ScreenPixelsWidth*/
const Profile P20370 = {"20370", &P20370_4};	/*Profile ID - 20370*/
const Value V21122_1_1 = {S2, NULL};	/*Value = True*/
const Property P21122_1 = {S50, 0, &V21122_1_1, NULL};	/*Property = IsMobile*/
const Value V21122_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21122_5 = {S52, 0, &V21122_5_1403, &P21122_1};	/*Property = ScreenPixelsHeight*/
const Value V21122_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21122_4 = {S54, 0, &V21122_4_1404, &P21122_5};	/*Property = ScreenPixelsWidth*/
const Profile P21122 = {"21122", &P21122_4};	/*Profile ID - 21122*/
const Value V23940_1_1 = {S2, NULL};	/*Value = True*/
const Property P23940_1 = {S50, 0, &V23940_1_1, NULL};	/*Property = IsMobile*/
const Value V23940_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23940_5 = {S52, 0, &V23940_5_1227, &P23940_1};	/*Property = ScreenPixelsHeight*/
const Value V23940_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23940_4 = {S54, 0, &V23940_4_1229, &P23940_5};	/*Property = ScreenPixelsWidth*/
const Profile P23940 = {"23940", &P23940_4};	/*Profile ID - 23940*/
const Value V18219_1_1 = {S2, NULL};	/*Value = True*/
const Property P18219_1 = {S50, 0, &V18219_1_1, NULL};	/*Property = IsMobile*/
const Value V18219_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18219_5 = {S52, 0, &V18219_5_1222, &P18219_1};	/*Property = ScreenPixelsHeight*/
const Value V18219_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18219_4 = {S54, 0, &V18219_4_1224, &P18219_5};	/*Property = ScreenPixelsWidth*/
const Profile P18219 = {"18219", &P18219_4};	/*Profile ID - 18219*/
const Profile P24689 = {"24689", NULL};	/*Profile ID - 24689*/
const Value V21013_1_1 = {S2, NULL};	/*Value = True*/
const Property P21013_1 = {S50, 0, &V21013_1_1, NULL};	/*Property = IsMobile*/
const Value V21013_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21013_5 = {S52, 0, &V21013_5_1269, &P21013_1};	/*Property = ScreenPixelsHeight*/
const Value V21013_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21013_4 = {S54, 0, &V21013_4_1229, &P21013_5};	/*Property = ScreenPixelsWidth*/
const Profile P21013 = {"21013", &P21013_4};	/*Profile ID - 21013*/
const Value V22592_1_1 = {S2, NULL};	/*Value = True*/
const Property P22592_1 = {S50, 0, &V22592_1_1, NULL};	/*Property = IsMobile*/
const Value V22592_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22592_5 = {S52, 0, &V22592_5_9981, &P22592_1};	/*Property = ScreenPixelsHeight*/
const Value V22592_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22592_4 = {S54, 0, &V22592_4_2336, &P22592_5};	/*Property = ScreenPixelsWidth*/
const Profile P22592 = {"22592", &P22592_4};	/*Profile ID - 22592*/
const Value V22347_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22347_279 = {S1, 0, &V22347_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22347_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22347_282 = {S3, 0, &V22347_282_57153, &P22347_279};	/*Property = BlobBuilder*/
const Value V22347_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22347_350 = {S4, 0, &V22347_350_57444, &P22347_282};	/*Property = Canvas*/
const Value V22347_283_57160 = {S0, NULL};	/*Value = False*/
const Property P22347_283 = {S5, 0, &V22347_283_57160, &P22347_350};	/*Property = CssBackground*/
const Value V22347_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22347_284 = {S6, 0, &V22347_284_57163, &P22347_283};	/*Property = CssBorderImage*/
const Value V22347_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22347_285 = {S7, 0, &V22347_285_57166, &P22347_284};	/*Property = CssCanvas*/
const Value V22347_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22347_286 = {S8, 0, &V22347_286_57168, &P22347_285};	/*Property = CssColor*/
const Value V22347_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22347_237 = {S9, 0, &V22347_237_55784, &P22347_286};	/*Property = CssColumn*/
const Value V22347_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22347_287 = {S10, 0, &V22347_287_57172, &P22347_237};	/*Property = CssFlexbox*/
const Value V22347_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22347_288 = {S11, 0, &V22347_288_57175, &P22347_287};	/*Property = CssFont*/
const Value V22347_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22347_289 = {S12, 0, &V22347_289_86899, &P22347_288};	/*Property = CssImages*/
const Value V22347_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22347_291 = {S13, 0, &V22347_291_57181, &P22347_289};	/*Property = CssMediaQueries*/
const Value V22347_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22347_292 = {S14, 0, &V22347_292_57183, &P22347_291};	/*Property = CssMinMax*/
const Value V22347_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22347_293 = {S15, 0, &V22347_293_57187, &P22347_292};	/*Property = CssOverflow*/
const Value V22347_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22347_294 = {S16, 0, &V22347_294_57189, &P22347_293};	/*Property = CssPosition*/
const Value V22347_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22347_295 = {S17, 0, &V22347_295_57193, &P22347_294};	/*Property = CssText*/
const Value V22347_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22347_297 = {S18, 0, &V22347_297_57195, &P22347_295};	/*Property = CssTransforms*/
const Value V22347_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22347_241 = {S19, 0, &V22347_241_55799, &P22347_297};	/*Property = CssTransitions*/
const Value V22347_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22347_298 = {S20, 0, &V22347_298_57206, &P22347_241};	/*Property = CssUI*/
const Value V22347_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22347_300 = {S21, 0, &V22347_300_57215, &P22347_298};	/*Property = DataSet*/
const Value V22347_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22347_301 = {S22, 0, &V22347_301_57218, &P22347_300};	/*Property = DataUrl*/
const Value V22347_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22347_302 = {S23, 0, &V22347_302_57221, &P22347_301};	/*Property = DeviceOrientation*/
const Value V22347_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22347_304 = {S24, 0, &V22347_304_57227, &P22347_302};	/*Property = FileReader*/
const Value V22347_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22347_305 = {S25, 0, &V22347_305_57231, &P22347_304};	/*Property = FileSaver*/
const Value V22347_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22347_306 = {S26, 0, &V22347_306_57237, &P22347_305};	/*Property = FileWriter*/
const Value V22347_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22347_307 = {S27, 0, &V22347_307_57240, &P22347_306};	/*Property = FormData*/
const Value V22347_309_57245 = {S2, NULL};	/*Value = True*/
const Property P22347_309 = {S28, 0, &V22347_309_57245, &P22347_307};	/*Property = Fullscreen*/
const Value V22347_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22347_310 = {S29, 0, &V22347_310_57248, &P22347_309};	/*Property = GeoLocation*/
const Value V22347_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22347_312 = {S30, 0, &V22347_312_57254, &P22347_310};	/*Property = History*/
const Value V22347_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22347_314 = {S31, 0, &V22347_314_57261, &P22347_312};	/*Property = Html5*/
const Value V22347_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22347_313 = {S32, 0, &V22347_313_57258, &P22347_314};	/*Property = Html-Media-Capture*/
const Value V22347_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22347_315 = {S33, 0, &V22347_315_57264, &P22347_313};	/*Property = Iframe*/
const Value V22347_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22347_316 = {S34, 0, &V22347_316_57266, &P22347_315};	/*Property = IndexedDB*/
const Value V22347_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22347_317 = {S35, 0, &V22347_317_57269, &P22347_316};	/*Property = Json*/
const Value V22347_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22347_15 = {S37, 0, &V22347_15_29, &P22347_317};	/*Property = LayoutEngine*/
const Value V22347_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22347_344 = {S38, 0, &V22347_344_57272, &P22347_15};	/*Property = Masking*/
const Value V22347_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22347_320 = {S39, 0, &V22347_320_57290, &P22347_344};	/*Property = PostMessage*/
const Value V22347_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22347_321 = {S40, 0, &V22347_321_57293, &P22347_320};	/*Property = Progress*/
const Value V22347_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22347_322 = {S41, 0, &V22347_322_57296, &P22347_321};	/*Property = Prompts*/
const Value V22347_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22347_323 = {S42, 0, &V22347_323_57299, &P22347_322};	/*Property = Selector*/
const Value V22347_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22347_324 = {S43, 0, &V22347_324_57305, &P22347_323};	/*Property = Svg*/
const Value V22347_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22347_327 = {S44, 0, &V22347_327_57315, &P22347_324};	/*Property = TouchEvents*/
const Value V22347_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22347_328 = {S45, 0, &V22347_328_57318, &P22347_327};	/*Property = Track*/
const Value V22347_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22347_329 = {S46, 0, &V22347_329_57323, &P22347_328};	/*Property = Video*/
const Value V22347_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22347_330 = {S47, 0, &V22347_330_57326, &P22347_329};	/*Property = Viewport*/
const Value V22347_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22347_332 = {S48, 0, &V22347_332_57339, &P22347_330};	/*Property = WebWorkers*/
const Value V22347_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22347_333 = {S49, 0, &V22347_333_57341, &P22347_332};	/*Property = Xhr2*/
const Profile P22347 = {"22347", &P22347_333};	/*Profile ID - 22347*/
const Value V17586_1_1 = {S2, NULL};	/*Value = True*/
const Property P17586_1 = {S50, 0, &V17586_1_1, NULL};	/*Property = IsMobile*/
const Value V17586_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17586_5 = {S52, 0, &V17586_5_1227, &P17586_1};	/*Property = ScreenPixelsHeight*/
const Value V17586_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P17586_4 = {S54, 0, &V17586_4_1477, &P17586_5};	/*Property = ScreenPixelsWidth*/
const Profile P17586 = {"17586", &P17586_4};	/*Profile ID - 17586*/
const Value V19908_1_1 = {S2, NULL};	/*Value = True*/
const Property P19908_1 = {S50, 0, &V19908_1_1, NULL};	/*Property = IsMobile*/
const Value V19908_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19908_5 = {S52, 0, &V19908_5_1227, &P19908_1};	/*Property = ScreenPixelsHeight*/
const Value V19908_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19908_4 = {S54, 0, &V19908_4_1229, &P19908_5};	/*Property = ScreenPixelsWidth*/
const Profile P19908 = {"19908", &P19908_4};	/*Profile ID - 19908*/
const Value V20465_1_1 = {S2, NULL};	/*Value = True*/
const Property P20465_1 = {S50, 0, &V20465_1_1, NULL};	/*Property = IsMobile*/
const Value V20465_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20465_5 = {S52, 0, &V20465_5_1227, &P20465_1};	/*Property = ScreenPixelsHeight*/
const Value V20465_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20465_4 = {S54, 0, &V20465_4_1229, &P20465_5};	/*Property = ScreenPixelsWidth*/
const Profile P20465 = {"20465", &P20465_4};	/*Profile ID - 20465*/
const Value V18670_1_1 = {S2, NULL};	/*Value = True*/
const Property P18670_1 = {S50, 0, &V18670_1_1, NULL};	/*Property = IsMobile*/
const Value V18670_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18670_5 = {S52, 0, &V18670_5_1222, &P18670_1};	/*Property = ScreenPixelsHeight*/
const Value V18670_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18670_4 = {S54, 0, &V18670_4_1224, &P18670_5};	/*Property = ScreenPixelsWidth*/
const Profile P18670 = {"18670", &P18670_4};	/*Profile ID - 18670*/
const Value V21067_1_1 = {S2, NULL};	/*Value = True*/
const Property P21067_1 = {S50, 0, &V21067_1_1, NULL};	/*Property = IsMobile*/
const Value V21067_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P21067_5 = {S52, 0, &V21067_5_37569, &P21067_1};	/*Property = ScreenPixelsHeight*/
const Value V21067_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21067_4 = {S54, 0, &V21067_4_5645, &P21067_5};	/*Property = ScreenPixelsWidth*/
const Profile P21067 = {"21067", &P21067_4};	/*Profile ID - 21067*/
const Value V24378_1_1 = {S2, NULL};	/*Value = True*/
const Property P24378_1 = {S50, 0, &V24378_1_1, NULL};	/*Property = IsMobile*/
const Value V24378_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24378_5 = {S52, 0, &V24378_5_9981, &P24378_1};	/*Property = ScreenPixelsHeight*/
const Value V24378_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24378_4 = {S54, 0, &V24378_4_39098, &P24378_5};	/*Property = ScreenPixelsWidth*/
const Profile P24378 = {"24378", &P24378_4};	/*Profile ID - 24378*/
const Value V20726_1_1 = {S2, NULL};	/*Value = True*/
const Property P20726_1 = {S50, 0, &V20726_1_1, NULL};	/*Property = IsMobile*/
const Value V20726_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20726_5 = {S52, 0, &V20726_5_1458, &P20726_1};	/*Property = ScreenPixelsHeight*/
const Value V20726_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20726_4 = {S54, 0, &V20726_4_37617, &P20726_5};	/*Property = ScreenPixelsWidth*/
const Profile P20726 = {"20726", &P20726_4};	/*Profile ID - 20726*/
const Value V22736_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22736_279 = {S1, 0, &V22736_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22736_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22736_282 = {S3, 0, &V22736_282_57154, &P22736_279};	/*Property = BlobBuilder*/
const Value V22736_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22736_350 = {S4, 0, &V22736_350_57444, &P22736_282};	/*Property = Canvas*/
const Value V22736_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22736_283 = {S5, 0, &V22736_283_57159, &P22736_350};	/*Property = CssBackground*/
const Value V22736_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22736_284 = {S6, 0, &V22736_284_57163, &P22736_283};	/*Property = CssBorderImage*/
const Value V22736_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22736_285 = {S7, 0, &V22736_285_57166, &P22736_284};	/*Property = CssCanvas*/
const Value V22736_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22736_286 = {S8, 0, &V22736_286_57168, &P22736_285};	/*Property = CssColor*/
const Value V22736_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22736_237 = {S9, 0, &V22736_237_55786, &P22736_286};	/*Property = CssColumn*/
const Value V22736_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22736_287 = {S10, 0, &V22736_287_57172, &P22736_237};	/*Property = CssFlexbox*/
const Value V22736_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22736_288 = {S11, 0, &V22736_288_57174, &P22736_287};	/*Property = CssFont*/
const Value V22736_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22736_289 = {S12, 0, &V22736_289_57759, &P22736_288};	/*Property = CssImages*/
const Value V22736_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22736_291 = {S13, 0, &V22736_291_57180, &P22736_289};	/*Property = CssMediaQueries*/
const Value V22736_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22736_292 = {S14, 0, &V22736_292_57183, &P22736_291};	/*Property = CssMinMax*/
const Value V22736_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22736_293 = {S15, 0, &V22736_293_57187, &P22736_292};	/*Property = CssOverflow*/
const Value V22736_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22736_294 = {S16, 0, &V22736_294_57189, &P22736_293};	/*Property = CssPosition*/
const Value V22736_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22736_295 = {S17, 0, &V22736_295_57192, &P22736_294};	/*Property = CssText*/
const Value V22736_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22736_297 = {S18, 0, &V22736_297_57195, &P22736_295};	/*Property = CssTransforms*/
const Value V22736_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22736_241 = {S19, 0, &V22736_241_55799, &P22736_297};	/*Property = CssTransitions*/
const Value V22736_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22736_298 = {S20, 0, &V22736_298_57206, &P22736_241};	/*Property = CssUI*/
const Value V22736_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22736_300 = {S21, 0, &V22736_300_57216, &P22736_298};	/*Property = DataSet*/
const Value V22736_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22736_301 = {S22, 0, &V22736_301_57218, &P22736_300};	/*Property = DataUrl*/
const Value V22736_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22736_302 = {S23, 0, &V22736_302_57222, &P22736_301};	/*Property = DeviceOrientation*/
const Value V22736_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22736_304 = {S24, 0, &V22736_304_57227, &P22736_302};	/*Property = FileReader*/
const Value V22736_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22736_305 = {S25, 0, &V22736_305_57231, &P22736_304};	/*Property = FileSaver*/
const Value V22736_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22736_306 = {S26, 0, &V22736_306_57237, &P22736_305};	/*Property = FileWriter*/
const Value V22736_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22736_307 = {S27, 0, &V22736_307_57239, &P22736_306};	/*Property = FormData*/
const Value V22736_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22736_309 = {S28, 0, &V22736_309_57246, &P22736_307};	/*Property = Fullscreen*/
const Value V22736_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22736_310 = {S29, 0, &V22736_310_57248, &P22736_309};	/*Property = GeoLocation*/
const Value V22736_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22736_312 = {S30, 0, &V22736_312_57254, &P22736_310};	/*Property = History*/
const Value V22736_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22736_314 = {S31, 0, &V22736_314_57261, &P22736_312};	/*Property = Html5*/
const Value V22736_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22736_313 = {S32, 0, &V22736_313_57258, &P22736_314};	/*Property = Html-Media-Capture*/
const Value V22736_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22736_315 = {S33, 0, &V22736_315_57264, &P22736_313};	/*Property = Iframe*/
const Value V22736_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22736_316 = {S34, 0, &V22736_316_57266, &P22736_315};	/*Property = IndexedDB*/
const Value V22736_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22736_317 = {S35, 0, &V22736_317_57269, &P22736_316};	/*Property = Json*/
const Value V22736_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22736_15 = {S37, 0, &V22736_15_28, &P22736_317};	/*Property = LayoutEngine*/
const Value V22736_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22736_344 = {S38, 0, &V22736_344_57272, &P22736_15};	/*Property = Masking*/
const Value V22736_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22736_320 = {S39, 0, &V22736_320_57290, &P22736_344};	/*Property = PostMessage*/
const Value V22736_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22736_321 = {S40, 0, &V22736_321_57293, &P22736_320};	/*Property = Progress*/
const Value V22736_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22736_322 = {S41, 0, &V22736_322_57296, &P22736_321};	/*Property = Prompts*/
const Value V22736_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22736_323 = {S42, 0, &V22736_323_57299, &P22736_322};	/*Property = Selector*/
const Value V22736_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22736_324 = {S43, 0, &V22736_324_57306, &P22736_323};	/*Property = Svg*/
const Value V22736_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22736_327 = {S44, 0, &V22736_327_57315, &P22736_324};	/*Property = TouchEvents*/
const Value V22736_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22736_328 = {S45, 0, &V22736_328_57318, &P22736_327};	/*Property = Track*/
const Value V22736_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22736_329 = {S46, 0, &V22736_329_57323, &P22736_328};	/*Property = Video*/
const Value V22736_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22736_330 = {S47, 0, &V22736_330_57326, &P22736_329};	/*Property = Viewport*/
const Value V22736_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22736_332 = {S48, 0, &V22736_332_57338, &P22736_330};	/*Property = WebWorkers*/
const Value V22736_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22736_333 = {S49, 0, &V22736_333_57341, &P22736_332};	/*Property = Xhr2*/
const Profile P22736 = {"22736", &P22736_333};	/*Profile ID - 22736*/
const Value V20967_1_1 = {S2, NULL};	/*Value = True*/
const Property P20967_1 = {S50, 0, &V20967_1_1, NULL};	/*Property = IsMobile*/
const Value V20967_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20967_5 = {S52, 0, &V20967_5_1227, &P20967_1};	/*Property = ScreenPixelsHeight*/
const Value V20967_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20967_4 = {S54, 0, &V20967_4_1229, &P20967_5};	/*Property = ScreenPixelsWidth*/
const Profile P20967 = {"20967", &P20967_4};	/*Profile ID - 20967*/
const Value V19935_1_1 = {S2, NULL};	/*Value = True*/
const Property P19935_1 = {S50, 0, &V19935_1_1, NULL};	/*Property = IsMobile*/
const Value V19935_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P19935_5 = {S52, 0, &V19935_5_1277, &P19935_1};	/*Property = ScreenPixelsHeight*/
const Value V19935_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P19935_4 = {S54, 0, &V19935_4_1279, &P19935_5};	/*Property = ScreenPixelsWidth*/
const Profile P19935 = {"19935", &P19935_4};	/*Profile ID - 19935*/
const Value V21451_1_1 = {S2, NULL};	/*Value = True*/
const Property P21451_1 = {S50, 0, &V21451_1_1, NULL};	/*Property = IsMobile*/
const Value V21451_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21451_5 = {S52, 0, &V21451_5_1227, &P21451_1};	/*Property = ScreenPixelsHeight*/
const Value V21451_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21451_4 = {S54, 0, &V21451_4_1229, &P21451_5};	/*Property = ScreenPixelsWidth*/
const Profile P21451 = {"21451", &P21451_4};	/*Profile ID - 21451*/
const Value V15973_1_1 = {S2, NULL};	/*Value = True*/
const Property P15973_1 = {S50, 0, &V15973_1_1, NULL};	/*Property = IsMobile*/
const Value V15973_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15973_5 = {S52, 0, &V15973_5_1227, &P15973_1};	/*Property = ScreenPixelsHeight*/
const Value V15973_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15973_4 = {S54, 0, &V15973_4_1229, &P15973_5};	/*Property = ScreenPixelsWidth*/
const Profile P15973 = {"15973", &P15973_4};	/*Profile ID - 15973*/
const Value V21530_1_1 = {S2, NULL};	/*Value = True*/
const Property P21530_1 = {S50, 0, &V21530_1_1, NULL};	/*Property = IsMobile*/
const Value V21530_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21530_5 = {S52, 0, &V21530_5_1277, &P21530_1};	/*Property = ScreenPixelsHeight*/
const Value V21530_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21530_4 = {S54, 0, &V21530_4_1279, &P21530_5};	/*Property = ScreenPixelsWidth*/
const Profile P21530 = {"21530", &P21530_4};	/*Profile ID - 21530*/
const Value V20058_1_1 = {S2, NULL};	/*Value = True*/
const Property P20058_1 = {S50, 0, &V20058_1_1, NULL};	/*Property = IsMobile*/
const Value V20058_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20058_5 = {S52, 0, &V20058_5_1277, &P20058_1};	/*Property = ScreenPixelsHeight*/
const Value V20058_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20058_4 = {S54, 0, &V20058_4_1279, &P20058_5};	/*Property = ScreenPixelsWidth*/
const Profile P20058 = {"20058", &P20058_4};	/*Profile ID - 20058*/
const Value V21069_1_1 = {S2, NULL};	/*Value = True*/
const Property P21069_1 = {S50, 0, &V21069_1_1, NULL};	/*Property = IsMobile*/
const Value V21069_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21069_5 = {S52, 0, &V21069_5_1403, &P21069_1};	/*Property = ScreenPixelsHeight*/
const Value V21069_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21069_4 = {S54, 0, &V21069_4_1404, &P21069_5};	/*Property = ScreenPixelsWidth*/
const Profile P21069 = {"21069", &P21069_4};	/*Profile ID - 21069*/
const Value V21591_1_1 = {S2, NULL};	/*Value = True*/
const Property P21591_1 = {S50, 0, &V21591_1_1, NULL};	/*Property = IsMobile*/
const Value V21591_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21591_5 = {S52, 0, &V21591_5_1403, &P21591_1};	/*Property = ScreenPixelsHeight*/
const Value V21591_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21591_4 = {S54, 0, &V21591_4_1404, &P21591_5};	/*Property = ScreenPixelsWidth*/
const Profile P21591 = {"21591", &P21591_4};	/*Profile ID - 21591*/
const Profile P22219 = {"22219", NULL};	/*Profile ID - 22219*/
const Value V18240_1_1 = {S2, NULL};	/*Value = True*/
const Property P18240_1 = {S50, 0, &V18240_1_1, NULL};	/*Property = IsMobile*/
const Value V18240_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18240_5 = {S52, 0, &V18240_5_1277, &P18240_1};	/*Property = ScreenPixelsHeight*/
const Value V18240_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18240_4 = {S54, 0, &V18240_4_1279, &P18240_5};	/*Property = ScreenPixelsWidth*/
const Profile P18240 = {"18240", &P18240_4};	/*Profile ID - 18240*/
const Value V22488_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22488_279 = {S1, 0, &V22488_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22488_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22488_282 = {S3, 0, &V22488_282_57154, &P22488_279};	/*Property = BlobBuilder*/
const Value V22488_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22488_350 = {S4, 0, &V22488_350_57444, &P22488_282};	/*Property = Canvas*/
const Value V22488_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22488_283 = {S5, 0, &V22488_283_57159, &P22488_350};	/*Property = CssBackground*/
const Value V22488_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22488_284 = {S6, 0, &V22488_284_57163, &P22488_283};	/*Property = CssBorderImage*/
const Value V22488_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22488_285 = {S7, 0, &V22488_285_57166, &P22488_284};	/*Property = CssCanvas*/
const Value V22488_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22488_286 = {S8, 0, &V22488_286_57168, &P22488_285};	/*Property = CssColor*/
const Value V22488_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22488_237 = {S9, 0, &V22488_237_55784, &P22488_286};	/*Property = CssColumn*/
const Value V22488_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22488_287 = {S10, 0, &V22488_287_57172, &P22488_237};	/*Property = CssFlexbox*/
const Value V22488_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22488_288 = {S11, 0, &V22488_288_57174, &P22488_287};	/*Property = CssFont*/
const Value V22488_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22488_289 = {S12, 0, &V22488_289_57759, &P22488_288};	/*Property = CssImages*/
const Value V22488_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22488_291 = {S13, 0, &V22488_291_57180, &P22488_289};	/*Property = CssMediaQueries*/
const Value V22488_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22488_292 = {S14, 0, &V22488_292_57183, &P22488_291};	/*Property = CssMinMax*/
const Value V22488_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22488_293 = {S15, 0, &V22488_293_57187, &P22488_292};	/*Property = CssOverflow*/
const Value V22488_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22488_294 = {S16, 0, &V22488_294_57189, &P22488_293};	/*Property = CssPosition*/
const Value V22488_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22488_295 = {S17, 0, &V22488_295_57192, &P22488_294};	/*Property = CssText*/
const Value V22488_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22488_297 = {S18, 0, &V22488_297_57195, &P22488_295};	/*Property = CssTransforms*/
const Value V22488_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22488_241 = {S19, 0, &V22488_241_55799, &P22488_297};	/*Property = CssTransitions*/
const Value V22488_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22488_298 = {S20, 0, &V22488_298_57206, &P22488_241};	/*Property = CssUI*/
const Value V22488_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22488_300 = {S21, 0, &V22488_300_57215, &P22488_298};	/*Property = DataSet*/
const Value V22488_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22488_301 = {S22, 0, &V22488_301_57218, &P22488_300};	/*Property = DataUrl*/
const Value V22488_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22488_302 = {S23, 0, &V22488_302_57221, &P22488_301};	/*Property = DeviceOrientation*/
const Value V22488_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22488_304 = {S24, 0, &V22488_304_57227, &P22488_302};	/*Property = FileReader*/
const Value V22488_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22488_305 = {S25, 0, &V22488_305_57231, &P22488_304};	/*Property = FileSaver*/
const Value V22488_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22488_306 = {S26, 0, &V22488_306_57237, &P22488_305};	/*Property = FileWriter*/
const Value V22488_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22488_307 = {S27, 0, &V22488_307_57239, &P22488_306};	/*Property = FormData*/
const Value V22488_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22488_309 = {S28, 0, &V22488_309_57246, &P22488_307};	/*Property = Fullscreen*/
const Value V22488_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22488_310 = {S29, 0, &V22488_310_57248, &P22488_309};	/*Property = GeoLocation*/
const Value V22488_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22488_312 = {S30, 0, &V22488_312_57254, &P22488_310};	/*Property = History*/
const Value V22488_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22488_314 = {S31, 0, &V22488_314_57261, &P22488_312};	/*Property = Html5*/
const Value V22488_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22488_313 = {S32, 0, &V22488_313_57258, &P22488_314};	/*Property = Html-Media-Capture*/
const Value V22488_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22488_315 = {S33, 0, &V22488_315_57264, &P22488_313};	/*Property = Iframe*/
const Value V22488_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22488_316 = {S34, 0, &V22488_316_57266, &P22488_315};	/*Property = IndexedDB*/
const Value V22488_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22488_317 = {S35, 0, &V22488_317_57269, &P22488_316};	/*Property = Json*/
const Value V22488_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22488_15 = {S37, 0, &V22488_15_29, &P22488_317};	/*Property = LayoutEngine*/
const Value V22488_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22488_344 = {S38, 0, &V22488_344_57272, &P22488_15};	/*Property = Masking*/
const Value V22488_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22488_320 = {S39, 0, &V22488_320_57290, &P22488_344};	/*Property = PostMessage*/
const Value V22488_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22488_321 = {S40, 0, &V22488_321_57293, &P22488_320};	/*Property = Progress*/
const Value V22488_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22488_322 = {S41, 0, &V22488_322_57296, &P22488_321};	/*Property = Prompts*/
const Value V22488_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22488_323 = {S42, 0, &V22488_323_57299, &P22488_322};	/*Property = Selector*/
const Value V22488_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22488_324 = {S43, 0, &V22488_324_57305, &P22488_323};	/*Property = Svg*/
const Value V22488_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22488_327 = {S44, 0, &V22488_327_57315, &P22488_324};	/*Property = TouchEvents*/
const Value V22488_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22488_328 = {S45, 0, &V22488_328_57318, &P22488_327};	/*Property = Track*/
const Value V22488_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22488_329 = {S46, 0, &V22488_329_57323, &P22488_328};	/*Property = Video*/
const Value V22488_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22488_330 = {S47, 0, &V22488_330_57326, &P22488_329};	/*Property = Viewport*/
const Value V22488_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22488_332 = {S48, 0, &V22488_332_57338, &P22488_330};	/*Property = WebWorkers*/
const Value V22488_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22488_333 = {S49, 0, &V22488_333_57341, &P22488_332};	/*Property = Xhr2*/
const Profile P22488 = {"22488", &P22488_333};	/*Profile ID - 22488*/
const Value V17625_1_1 = {S2, NULL};	/*Value = True*/
const Property P17625_1 = {S50, 0, &V17625_1_1, NULL};	/*Property = IsMobile*/
const Value V17625_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P17625_5 = {S52, 0, &V17625_5_1269, &P17625_1};	/*Property = ScreenPixelsHeight*/
const Value V17625_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17625_4 = {S54, 0, &V17625_4_1229, &P17625_5};	/*Property = ScreenPixelsWidth*/
const Profile P17625 = {"17625", &P17625_4};	/*Profile ID - 17625*/
const Value V21997_1_1 = {S2, NULL};	/*Value = True*/
const Property P21997_1 = {S50, 0, &V21997_1_1, NULL};	/*Property = IsMobile*/
const Value V21997_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21997_5 = {S52, 0, &V21997_5_1222, &P21997_1};	/*Property = ScreenPixelsHeight*/
const Value V21997_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21997_4 = {S54, 0, &V21997_4_1224, &P21997_5};	/*Property = ScreenPixelsWidth*/
const Profile P21997 = {"21997", &P21997_4};	/*Profile ID - 21997*/
const Value V20597_1_1 = {S2, NULL};	/*Value = True*/
const Property P20597_1 = {S50, 0, &V20597_1_1, NULL};	/*Property = IsMobile*/
const Value V20597_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20597_5 = {S52, 0, &V20597_5_1458, &P20597_1};	/*Property = ScreenPixelsHeight*/
const Value V20597_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20597_4 = {S54, 0, &V20597_4_37617, &P20597_5};	/*Property = ScreenPixelsWidth*/
const Profile P20597 = {"20597", &P20597_4};	/*Profile ID - 20597*/
const Value V22721_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_279 = {S1, 0, &V22721_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22721_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_282 = {S3, 0, &V22721_282_57155, &P22721_279};	/*Property = BlobBuilder*/
const Value V22721_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_350 = {S4, 0, &V22721_350_57446, &P22721_282};	/*Property = Canvas*/
const Value V22721_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_283 = {S5, 0, &V22721_283_57161, &P22721_350};	/*Property = CssBackground*/
const Value V22721_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_284 = {S6, 0, &V22721_284_57164, &P22721_283};	/*Property = CssBorderImage*/
const Value V22721_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_285 = {S7, 0, &V22721_285_57167, &P22721_284};	/*Property = CssCanvas*/
const Value V22721_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_286 = {S8, 0, &V22721_286_57170, &P22721_285};	/*Property = CssColor*/
const Value V22721_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_237 = {S9, 0, &V22721_237_55786, &P22721_286};	/*Property = CssColumn*/
const Value V22721_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_287 = {S10, 0, &V22721_287_57173, &P22721_237};	/*Property = CssFlexbox*/
const Value V22721_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_288 = {S11, 0, &V22721_288_57176, &P22721_287};	/*Property = CssFont*/
const Value V22721_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_289 = {S12, 0, &V22721_289_86899, &P22721_288};	/*Property = CssImages*/
const Value V22721_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_291 = {S13, 0, &V22721_291_57182, &P22721_289};	/*Property = CssMediaQueries*/
const Value V22721_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_292 = {S14, 0, &V22721_292_57185, &P22721_291};	/*Property = CssMinMax*/
const Value V22721_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_293 = {S15, 0, &V22721_293_57188, &P22721_292};	/*Property = CssOverflow*/
const Value V22721_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_294 = {S16, 0, &V22721_294_57191, &P22721_293};	/*Property = CssPosition*/
const Value V22721_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_295 = {S17, 0, &V22721_295_57194, &P22721_294};	/*Property = CssText*/
const Value V22721_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_297 = {S18, 0, &V22721_297_57197, &P22721_295};	/*Property = CssTransforms*/
const Value V22721_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_241 = {S19, 0, &V22721_241_55801, &P22721_297};	/*Property = CssTransitions*/
const Value V22721_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_298 = {S20, 0, &V22721_298_57208, &P22721_241};	/*Property = CssUI*/
const Value V22721_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_300 = {S21, 0, &V22721_300_57217, &P22721_298};	/*Property = DataSet*/
const Value V22721_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_301 = {S22, 0, &V22721_301_57220, &P22721_300};	/*Property = DataUrl*/
const Value V22721_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_302 = {S23, 0, &V22721_302_57223, &P22721_301};	/*Property = DeviceOrientation*/
const Value V22721_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_304 = {S24, 0, &V22721_304_57229, &P22721_302};	/*Property = FileReader*/
const Value V22721_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_305 = {S25, 0, &V22721_305_57232, &P22721_304};	/*Property = FileSaver*/
const Value V22721_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_306 = {S26, 0, &V22721_306_57238, &P22721_305};	/*Property = FileWriter*/
const Value V22721_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_307 = {S27, 0, &V22721_307_57241, &P22721_306};	/*Property = FormData*/
const Value V22721_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_309 = {S28, 0, &V22721_309_57247, &P22721_307};	/*Property = Fullscreen*/
const Value V22721_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_310 = {S29, 0, &V22721_310_57250, &P22721_309};	/*Property = GeoLocation*/
const Value V22721_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_312 = {S30, 0, &V22721_312_57256, &P22721_310};	/*Property = History*/
const Value V22721_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_314 = {S31, 0, &V22721_314_57262, &P22721_312};	/*Property = Html5*/
const Value V22721_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_313 = {S32, 0, &V22721_313_57259, &P22721_314};	/*Property = Html-Media-Capture*/
const Value V22721_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_315 = {S33, 0, &V22721_315_57265, &P22721_313};	/*Property = Iframe*/
const Value V22721_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_316 = {S34, 0, &V22721_316_57268, &P22721_315};	/*Property = IndexedDB*/
const Value V22721_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_317 = {S35, 0, &V22721_317_57271, &P22721_316};	/*Property = Json*/
const Value V22721_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22721_15 = {S37, 0, &V22721_15_29, &P22721_317};	/*Property = LayoutEngine*/
const Value V22721_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_344 = {S38, 0, &V22721_344_57274, &P22721_15};	/*Property = Masking*/
const Value V22721_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_320 = {S39, 0, &V22721_320_57292, &P22721_344};	/*Property = PostMessage*/
const Value V22721_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_321 = {S40, 0, &V22721_321_57295, &P22721_320};	/*Property = Progress*/
const Value V22721_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_322 = {S41, 0, &V22721_322_57298, &P22721_321};	/*Property = Prompts*/
const Value V22721_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_323 = {S42, 0, &V22721_323_57301, &P22721_322};	/*Property = Selector*/
const Value V22721_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_324 = {S43, 0, &V22721_324_57307, &P22721_323};	/*Property = Svg*/
const Value V22721_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_327 = {S44, 0, &V22721_327_57316, &P22721_324};	/*Property = TouchEvents*/
const Value V22721_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_328 = {S45, 0, &V22721_328_57319, &P22721_327};	/*Property = Track*/
const Value V22721_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_329 = {S46, 0, &V22721_329_57325, &P22721_328};	/*Property = Video*/
const Value V22721_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_330 = {S47, 0, &V22721_330_57328, &P22721_329};	/*Property = Viewport*/
const Value V22721_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_332 = {S48, 0, &V22721_332_57340, &P22721_330};	/*Property = WebWorkers*/
const Value V22721_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22721_333 = {S49, 0, &V22721_333_57343, &P22721_332};	/*Property = Xhr2*/
const Profile P22721 = {"22721", &P22721_333};	/*Profile ID - 22721*/
const Value V20199_1_1 = {S2, NULL};	/*Value = True*/
const Property P20199_1 = {S50, 0, &V20199_1_1, NULL};	/*Property = IsMobile*/
const Value V20199_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P20199_5 = {S52, 0, &V20199_5_1304, &P20199_1};	/*Property = ScreenPixelsHeight*/
const Value V20199_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20199_4 = {S54, 0, &V20199_4_1224, &P20199_5};	/*Property = ScreenPixelsWidth*/
const Profile P20199 = {"20199", &P20199_4};	/*Profile ID - 20199*/
const Value V20319_1_2 = {S0, NULL};	/*Value = False*/
const Property P20319_1 = {S50, 0, &V20319_1_2, NULL};	/*Property = IsMobile*/
const Value V20319_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20319_5 = {S52, 0, &V20319_5_37, &P20319_1};	/*Property = ScreenPixelsHeight*/
const Value V20319_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20319_4 = {S54, 0, &V20319_4_36, &P20319_5};	/*Property = ScreenPixelsWidth*/
const Profile P20319 = {"20319", &P20319_4};	/*Profile ID - 20319*/
const Value V22024_1_1 = {S2, NULL};	/*Value = True*/
const Property P22024_1 = {S50, 0, &V22024_1_1, NULL};	/*Property = IsMobile*/
const Value V22024_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22024_5 = {S52, 0, &V22024_5_1403, &P22024_1};	/*Property = ScreenPixelsHeight*/
const Value V22024_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22024_4 = {S54, 0, &V22024_4_1404, &P22024_5};	/*Property = ScreenPixelsWidth*/
const Profile P22024 = {"22024", &P22024_4};	/*Profile ID - 22024*/
const Value V22496_1_1 = {S2, NULL};	/*Value = True*/
const Property P22496_1 = {S50, 0, &V22496_1_1, NULL};	/*Property = IsMobile*/
const Value V22496_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22496_5 = {S52, 0, &V22496_5_37569, &P22496_1};	/*Property = ScreenPixelsHeight*/
const Value V22496_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22496_4 = {S54, 0, &V22496_4_5645, &P22496_5};	/*Property = ScreenPixelsWidth*/
const Profile P22496 = {"22496", &P22496_4};	/*Profile ID - 22496*/
const Value V21799_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_279 = {S1, 0, &V21799_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21799_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_282 = {S3, 0, &V21799_282_57155, &P21799_279};	/*Property = BlobBuilder*/
const Value V21799_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_350 = {S4, 0, &V21799_350_57446, &P21799_282};	/*Property = Canvas*/
const Value V21799_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_283 = {S5, 0, &V21799_283_57161, &P21799_350};	/*Property = CssBackground*/
const Value V21799_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_284 = {S6, 0, &V21799_284_57164, &P21799_283};	/*Property = CssBorderImage*/
const Value V21799_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_285 = {S7, 0, &V21799_285_57167, &P21799_284};	/*Property = CssCanvas*/
const Value V21799_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_286 = {S8, 0, &V21799_286_57170, &P21799_285};	/*Property = CssColor*/
const Value V21799_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_237 = {S9, 0, &V21799_237_55786, &P21799_286};	/*Property = CssColumn*/
const Value V21799_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_287 = {S10, 0, &V21799_287_57173, &P21799_237};	/*Property = CssFlexbox*/
const Value V21799_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_288 = {S11, 0, &V21799_288_57176, &P21799_287};	/*Property = CssFont*/
const Value V21799_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_289 = {S12, 0, &V21799_289_86899, &P21799_288};	/*Property = CssImages*/
const Value V21799_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_291 = {S13, 0, &V21799_291_57182, &P21799_289};	/*Property = CssMediaQueries*/
const Value V21799_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_292 = {S14, 0, &V21799_292_57185, &P21799_291};	/*Property = CssMinMax*/
const Value V21799_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_293 = {S15, 0, &V21799_293_57188, &P21799_292};	/*Property = CssOverflow*/
const Value V21799_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_294 = {S16, 0, &V21799_294_57191, &P21799_293};	/*Property = CssPosition*/
const Value V21799_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_295 = {S17, 0, &V21799_295_57194, &P21799_294};	/*Property = CssText*/
const Value V21799_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_297 = {S18, 0, &V21799_297_57197, &P21799_295};	/*Property = CssTransforms*/
const Value V21799_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_241 = {S19, 0, &V21799_241_55801, &P21799_297};	/*Property = CssTransitions*/
const Value V21799_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_298 = {S20, 0, &V21799_298_57208, &P21799_241};	/*Property = CssUI*/
const Value V21799_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_300 = {S21, 0, &V21799_300_57217, &P21799_298};	/*Property = DataSet*/
const Value V21799_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_301 = {S22, 0, &V21799_301_57220, &P21799_300};	/*Property = DataUrl*/
const Value V21799_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_302 = {S23, 0, &V21799_302_57223, &P21799_301};	/*Property = DeviceOrientation*/
const Value V21799_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_304 = {S24, 0, &V21799_304_57229, &P21799_302};	/*Property = FileReader*/
const Value V21799_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_305 = {S25, 0, &V21799_305_57232, &P21799_304};	/*Property = FileSaver*/
const Value V21799_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_306 = {S26, 0, &V21799_306_57238, &P21799_305};	/*Property = FileWriter*/
const Value V21799_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_307 = {S27, 0, &V21799_307_57241, &P21799_306};	/*Property = FormData*/
const Value V21799_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_309 = {S28, 0, &V21799_309_57247, &P21799_307};	/*Property = Fullscreen*/
const Value V21799_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_310 = {S29, 0, &V21799_310_57250, &P21799_309};	/*Property = GeoLocation*/
const Value V21799_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_312 = {S30, 0, &V21799_312_57256, &P21799_310};	/*Property = History*/
const Value V21799_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_314 = {S31, 0, &V21799_314_57262, &P21799_312};	/*Property = Html5*/
const Value V21799_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_313 = {S32, 0, &V21799_313_57259, &P21799_314};	/*Property = Html-Media-Capture*/
const Value V21799_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_315 = {S33, 0, &V21799_315_57265, &P21799_313};	/*Property = Iframe*/
const Value V21799_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_316 = {S34, 0, &V21799_316_57268, &P21799_315};	/*Property = IndexedDB*/
const Value V21799_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_317 = {S35, 0, &V21799_317_57271, &P21799_316};	/*Property = Json*/
const Value V21799_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21799_15 = {S37, 0, &V21799_15_54167, &P21799_317};	/*Property = LayoutEngine*/
const Value V21799_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_344 = {S38, 0, &V21799_344_57274, &P21799_15};	/*Property = Masking*/
const Value V21799_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_320 = {S39, 0, &V21799_320_57292, &P21799_344};	/*Property = PostMessage*/
const Value V21799_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_321 = {S40, 0, &V21799_321_57295, &P21799_320};	/*Property = Progress*/
const Value V21799_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_322 = {S41, 0, &V21799_322_57298, &P21799_321};	/*Property = Prompts*/
const Value V21799_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_323 = {S42, 0, &V21799_323_57301, &P21799_322};	/*Property = Selector*/
const Value V21799_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_324 = {S43, 0, &V21799_324_57307, &P21799_323};	/*Property = Svg*/
const Value V21799_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_327 = {S44, 0, &V21799_327_57316, &P21799_324};	/*Property = TouchEvents*/
const Value V21799_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_328 = {S45, 0, &V21799_328_57319, &P21799_327};	/*Property = Track*/
const Value V21799_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_329 = {S46, 0, &V21799_329_57325, &P21799_328};	/*Property = Video*/
const Value V21799_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_330 = {S47, 0, &V21799_330_57328, &P21799_329};	/*Property = Viewport*/
const Value V21799_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_332 = {S48, 0, &V21799_332_57340, &P21799_330};	/*Property = WebWorkers*/
const Value V21799_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21799_333 = {S49, 0, &V21799_333_57343, &P21799_332};	/*Property = Xhr2*/
const Profile P21799 = {"21799", &P21799_333};	/*Profile ID - 21799*/
const Value V18482_1_1 = {S2, NULL};	/*Value = True*/
const Property P18482_1 = {S50, 0, &V18482_1_1, NULL};	/*Property = IsMobile*/
const Value V18482_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18482_5 = {S52, 0, &V18482_5_1277, &P18482_1};	/*Property = ScreenPixelsHeight*/
const Value V18482_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18482_4 = {S54, 0, &V18482_4_1279, &P18482_5};	/*Property = ScreenPixelsWidth*/
const Profile P18482 = {"18482", &P18482_4};	/*Profile ID - 18482*/
const Value V23681_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_279 = {S1, 0, &V23681_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23681_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_282 = {S3, 0, &V23681_282_57155, &P23681_279};	/*Property = BlobBuilder*/
const Value V23681_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_350 = {S4, 0, &V23681_350_57446, &P23681_282};	/*Property = Canvas*/
const Value V23681_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_283 = {S5, 0, &V23681_283_57161, &P23681_350};	/*Property = CssBackground*/
const Value V23681_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_284 = {S6, 0, &V23681_284_57164, &P23681_283};	/*Property = CssBorderImage*/
const Value V23681_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_285 = {S7, 0, &V23681_285_57167, &P23681_284};	/*Property = CssCanvas*/
const Value V23681_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_286 = {S8, 0, &V23681_286_57170, &P23681_285};	/*Property = CssColor*/
const Value V23681_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_237 = {S9, 0, &V23681_237_55786, &P23681_286};	/*Property = CssColumn*/
const Value V23681_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_287 = {S10, 0, &V23681_287_57173, &P23681_237};	/*Property = CssFlexbox*/
const Value V23681_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_288 = {S11, 0, &V23681_288_57176, &P23681_287};	/*Property = CssFont*/
const Value V23681_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_289 = {S12, 0, &V23681_289_86899, &P23681_288};	/*Property = CssImages*/
const Value V23681_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_291 = {S13, 0, &V23681_291_57182, &P23681_289};	/*Property = CssMediaQueries*/
const Value V23681_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_292 = {S14, 0, &V23681_292_57185, &P23681_291};	/*Property = CssMinMax*/
const Value V23681_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_293 = {S15, 0, &V23681_293_57188, &P23681_292};	/*Property = CssOverflow*/
const Value V23681_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_294 = {S16, 0, &V23681_294_57191, &P23681_293};	/*Property = CssPosition*/
const Value V23681_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_295 = {S17, 0, &V23681_295_57194, &P23681_294};	/*Property = CssText*/
const Value V23681_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_297 = {S18, 0, &V23681_297_57197, &P23681_295};	/*Property = CssTransforms*/
const Value V23681_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_241 = {S19, 0, &V23681_241_55801, &P23681_297};	/*Property = CssTransitions*/
const Value V23681_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_298 = {S20, 0, &V23681_298_57208, &P23681_241};	/*Property = CssUI*/
const Value V23681_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_300 = {S21, 0, &V23681_300_57217, &P23681_298};	/*Property = DataSet*/
const Value V23681_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_301 = {S22, 0, &V23681_301_57220, &P23681_300};	/*Property = DataUrl*/
const Value V23681_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_302 = {S23, 0, &V23681_302_57223, &P23681_301};	/*Property = DeviceOrientation*/
const Value V23681_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_304 = {S24, 0, &V23681_304_57229, &P23681_302};	/*Property = FileReader*/
const Value V23681_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_305 = {S25, 0, &V23681_305_57232, &P23681_304};	/*Property = FileSaver*/
const Value V23681_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_306 = {S26, 0, &V23681_306_57238, &P23681_305};	/*Property = FileWriter*/
const Value V23681_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_307 = {S27, 0, &V23681_307_57241, &P23681_306};	/*Property = FormData*/
const Value V23681_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_309 = {S28, 0, &V23681_309_57247, &P23681_307};	/*Property = Fullscreen*/
const Value V23681_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_310 = {S29, 0, &V23681_310_57250, &P23681_309};	/*Property = GeoLocation*/
const Value V23681_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_312 = {S30, 0, &V23681_312_57256, &P23681_310};	/*Property = History*/
const Value V23681_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_314 = {S31, 0, &V23681_314_57262, &P23681_312};	/*Property = Html5*/
const Value V23681_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_313 = {S32, 0, &V23681_313_57259, &P23681_314};	/*Property = Html-Media-Capture*/
const Value V23681_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_315 = {S33, 0, &V23681_315_57265, &P23681_313};	/*Property = Iframe*/
const Value V23681_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_316 = {S34, 0, &V23681_316_57268, &P23681_315};	/*Property = IndexedDB*/
const Value V23681_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_317 = {S35, 0, &V23681_317_57271, &P23681_316};	/*Property = Json*/
const Value V23681_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P23681_15 = {S37, 0, &V23681_15_29, &P23681_317};	/*Property = LayoutEngine*/
const Value V23681_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_344 = {S38, 0, &V23681_344_57274, &P23681_15};	/*Property = Masking*/
const Value V23681_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_320 = {S39, 0, &V23681_320_57292, &P23681_344};	/*Property = PostMessage*/
const Value V23681_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_321 = {S40, 0, &V23681_321_57295, &P23681_320};	/*Property = Progress*/
const Value V23681_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_322 = {S41, 0, &V23681_322_57298, &P23681_321};	/*Property = Prompts*/
const Value V23681_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_323 = {S42, 0, &V23681_323_57301, &P23681_322};	/*Property = Selector*/
const Value V23681_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_324 = {S43, 0, &V23681_324_57307, &P23681_323};	/*Property = Svg*/
const Value V23681_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_327 = {S44, 0, &V23681_327_57316, &P23681_324};	/*Property = TouchEvents*/
const Value V23681_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_328 = {S45, 0, &V23681_328_57319, &P23681_327};	/*Property = Track*/
const Value V23681_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_329 = {S46, 0, &V23681_329_57325, &P23681_328};	/*Property = Video*/
const Value V23681_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_330 = {S47, 0, &V23681_330_57328, &P23681_329};	/*Property = Viewport*/
const Value V23681_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_332 = {S48, 0, &V23681_332_57340, &P23681_330};	/*Property = WebWorkers*/
const Value V23681_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23681_333 = {S49, 0, &V23681_333_57343, &P23681_332};	/*Property = Xhr2*/
const Profile P23681 = {"23681", &P23681_333};	/*Profile ID - 23681*/
const Value V22350_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_279 = {S1, 0, &V22350_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22350_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_282 = {S3, 0, &V22350_282_57155, &P22350_279};	/*Property = BlobBuilder*/
const Value V22350_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_350 = {S4, 0, &V22350_350_57446, &P22350_282};	/*Property = Canvas*/
const Value V22350_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_283 = {S5, 0, &V22350_283_57161, &P22350_350};	/*Property = CssBackground*/
const Value V22350_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_284 = {S6, 0, &V22350_284_57164, &P22350_283};	/*Property = CssBorderImage*/
const Value V22350_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_285 = {S7, 0, &V22350_285_57167, &P22350_284};	/*Property = CssCanvas*/
const Value V22350_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_286 = {S8, 0, &V22350_286_57170, &P22350_285};	/*Property = CssColor*/
const Value V22350_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_237 = {S9, 0, &V22350_237_55786, &P22350_286};	/*Property = CssColumn*/
const Value V22350_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_287 = {S10, 0, &V22350_287_57173, &P22350_237};	/*Property = CssFlexbox*/
const Value V22350_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_288 = {S11, 0, &V22350_288_57176, &P22350_287};	/*Property = CssFont*/
const Value V22350_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_289 = {S12, 0, &V22350_289_86899, &P22350_288};	/*Property = CssImages*/
const Value V22350_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_291 = {S13, 0, &V22350_291_57182, &P22350_289};	/*Property = CssMediaQueries*/
const Value V22350_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_292 = {S14, 0, &V22350_292_57185, &P22350_291};	/*Property = CssMinMax*/
const Value V22350_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_293 = {S15, 0, &V22350_293_57188, &P22350_292};	/*Property = CssOverflow*/
const Value V22350_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_294 = {S16, 0, &V22350_294_57191, &P22350_293};	/*Property = CssPosition*/
const Value V22350_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_295 = {S17, 0, &V22350_295_57194, &P22350_294};	/*Property = CssText*/
const Value V22350_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_297 = {S18, 0, &V22350_297_57197, &P22350_295};	/*Property = CssTransforms*/
const Value V22350_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_241 = {S19, 0, &V22350_241_55801, &P22350_297};	/*Property = CssTransitions*/
const Value V22350_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_298 = {S20, 0, &V22350_298_57208, &P22350_241};	/*Property = CssUI*/
const Value V22350_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_300 = {S21, 0, &V22350_300_57217, &P22350_298};	/*Property = DataSet*/
const Value V22350_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_301 = {S22, 0, &V22350_301_57220, &P22350_300};	/*Property = DataUrl*/
const Value V22350_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_302 = {S23, 0, &V22350_302_57223, &P22350_301};	/*Property = DeviceOrientation*/
const Value V22350_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_304 = {S24, 0, &V22350_304_57229, &P22350_302};	/*Property = FileReader*/
const Value V22350_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_305 = {S25, 0, &V22350_305_57232, &P22350_304};	/*Property = FileSaver*/
const Value V22350_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_306 = {S26, 0, &V22350_306_57238, &P22350_305};	/*Property = FileWriter*/
const Value V22350_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_307 = {S27, 0, &V22350_307_57241, &P22350_306};	/*Property = FormData*/
const Value V22350_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_309 = {S28, 0, &V22350_309_57247, &P22350_307};	/*Property = Fullscreen*/
const Value V22350_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_310 = {S29, 0, &V22350_310_57250, &P22350_309};	/*Property = GeoLocation*/
const Value V22350_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_312 = {S30, 0, &V22350_312_57256, &P22350_310};	/*Property = History*/
const Value V22350_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_314 = {S31, 0, &V22350_314_57262, &P22350_312};	/*Property = Html5*/
const Value V22350_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_313 = {S32, 0, &V22350_313_57259, &P22350_314};	/*Property = Html-Media-Capture*/
const Value V22350_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_315 = {S33, 0, &V22350_315_57265, &P22350_313};	/*Property = Iframe*/
const Value V22350_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_316 = {S34, 0, &V22350_316_57268, &P22350_315};	/*Property = IndexedDB*/
const Value V22350_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_317 = {S35, 0, &V22350_317_57271, &P22350_316};	/*Property = Json*/
const Value V22350_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22350_15 = {S37, 0, &V22350_15_29, &P22350_317};	/*Property = LayoutEngine*/
const Value V22350_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_344 = {S38, 0, &V22350_344_57274, &P22350_15};	/*Property = Masking*/
const Value V22350_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_320 = {S39, 0, &V22350_320_57292, &P22350_344};	/*Property = PostMessage*/
const Value V22350_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_321 = {S40, 0, &V22350_321_57295, &P22350_320};	/*Property = Progress*/
const Value V22350_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_322 = {S41, 0, &V22350_322_57298, &P22350_321};	/*Property = Prompts*/
const Value V22350_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_323 = {S42, 0, &V22350_323_57301, &P22350_322};	/*Property = Selector*/
const Value V22350_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_324 = {S43, 0, &V22350_324_57307, &P22350_323};	/*Property = Svg*/
const Value V22350_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_327 = {S44, 0, &V22350_327_57316, &P22350_324};	/*Property = TouchEvents*/
const Value V22350_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_328 = {S45, 0, &V22350_328_57319, &P22350_327};	/*Property = Track*/
const Value V22350_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_329 = {S46, 0, &V22350_329_57325, &P22350_328};	/*Property = Video*/
const Value V22350_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_330 = {S47, 0, &V22350_330_57328, &P22350_329};	/*Property = Viewport*/
const Value V22350_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_332 = {S48, 0, &V22350_332_57340, &P22350_330};	/*Property = WebWorkers*/
const Value V22350_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22350_333 = {S49, 0, &V22350_333_57343, &P22350_332};	/*Property = Xhr2*/
const Profile P22350 = {"22350", &P22350_333};	/*Profile ID - 22350*/
const Value V18478_1_1 = {S2, NULL};	/*Value = True*/
const Property P18478_1 = {S50, 0, &V18478_1_1, NULL};	/*Property = IsMobile*/
const Value V18478_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P18478_5 = {S52, 0, &V18478_5_1458, &P18478_1};	/*Property = ScreenPixelsHeight*/
const Value V18478_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P18478_4 = {S54, 0, &V18478_4_1460, &P18478_5};	/*Property = ScreenPixelsWidth*/
const Profile P18478 = {"18478", &P18478_4};	/*Profile ID - 18478*/
const Value V20039_1_1 = {S2, NULL};	/*Value = True*/
const Property P20039_1 = {S50, 0, &V20039_1_1, NULL};	/*Property = IsMobile*/
const Value V20039_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20039_5 = {S52, 0, &V20039_5_1222, &P20039_1};	/*Property = ScreenPixelsHeight*/
const Value V20039_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20039_4 = {S54, 0, &V20039_4_1224, &P20039_5};	/*Property = ScreenPixelsWidth*/
const Profile P20039 = {"20039", &P20039_4};	/*Profile ID - 20039*/
const Value V23849_1_1 = {S2, NULL};	/*Value = True*/
const Property P23849_1 = {S50, 0, &V23849_1_1, NULL};	/*Property = IsMobile*/
const Value V23849_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23849_5 = {S52, 0, &V23849_5_1222, &P23849_1};	/*Property = ScreenPixelsHeight*/
const Value V23849_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23849_4 = {S54, 0, &V23849_4_1224, &P23849_5};	/*Property = ScreenPixelsWidth*/
const Profile P23849 = {"23849", &P23849_4};	/*Profile ID - 23849*/
const Value V22237_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22237_279 = {S1, 0, &V22237_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22237_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22237_282 = {S3, 0, &V22237_282_57153, &P22237_279};	/*Property = BlobBuilder*/
const Value V22237_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22237_350 = {S4, 0, &V22237_350_57444, &P22237_282};	/*Property = Canvas*/
const Value V22237_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22237_283 = {S5, 0, &V22237_283_57159, &P22237_350};	/*Property = CssBackground*/
const Value V22237_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22237_284 = {S6, 0, &V22237_284_57163, &P22237_283};	/*Property = CssBorderImage*/
const Value V22237_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22237_285 = {S7, 0, &V22237_285_57166, &P22237_284};	/*Property = CssCanvas*/
const Value V22237_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22237_286 = {S8, 0, &V22237_286_57168, &P22237_285};	/*Property = CssColor*/
const Value V22237_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22237_237 = {S9, 0, &V22237_237_55784, &P22237_286};	/*Property = CssColumn*/
const Value V22237_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22237_287 = {S10, 0, &V22237_287_57172, &P22237_237};	/*Property = CssFlexbox*/
const Value V22237_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22237_288 = {S11, 0, &V22237_288_57174, &P22237_287};	/*Property = CssFont*/
const Value V22237_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22237_289 = {S12, 0, &V22237_289_57759, &P22237_288};	/*Property = CssImages*/
const Value V22237_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22237_291 = {S13, 0, &V22237_291_57180, &P22237_289};	/*Property = CssMediaQueries*/
const Value V22237_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22237_292 = {S14, 0, &V22237_292_57183, &P22237_291};	/*Property = CssMinMax*/
const Value V22237_293_57186 = {S2, NULL};	/*Value = True*/
const Property P22237_293 = {S15, 0, &V22237_293_57186, &P22237_292};	/*Property = CssOverflow*/
const Value V22237_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22237_294 = {S16, 0, &V22237_294_57189, &P22237_293};	/*Property = CssPosition*/
const Value V22237_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22237_295 = {S17, 0, &V22237_295_57192, &P22237_294};	/*Property = CssText*/
const Value V22237_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22237_297 = {S18, 0, &V22237_297_57195, &P22237_295};	/*Property = CssTransforms*/
const Value V22237_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22237_241 = {S19, 0, &V22237_241_55799, &P22237_297};	/*Property = CssTransitions*/
const Value V22237_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22237_298 = {S20, 0, &V22237_298_57206, &P22237_241};	/*Property = CssUI*/
const Value V22237_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22237_300 = {S21, 0, &V22237_300_57216, &P22237_298};	/*Property = DataSet*/
const Value V22237_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22237_301 = {S22, 0, &V22237_301_57218, &P22237_300};	/*Property = DataUrl*/
const Value V22237_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22237_302 = {S23, 0, &V22237_302_57222, &P22237_301};	/*Property = DeviceOrientation*/
const Value V22237_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22237_304 = {S24, 0, &V22237_304_57227, &P22237_302};	/*Property = FileReader*/
const Value V22237_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22237_305 = {S25, 0, &V22237_305_57231, &P22237_304};	/*Property = FileSaver*/
const Value V22237_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22237_306 = {S26, 0, &V22237_306_57237, &P22237_305};	/*Property = FileWriter*/
const Value V22237_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22237_307 = {S27, 0, &V22237_307_57239, &P22237_306};	/*Property = FormData*/
const Value V22237_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22237_309 = {S28, 0, &V22237_309_57246, &P22237_307};	/*Property = Fullscreen*/
const Value V22237_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22237_310 = {S29, 0, &V22237_310_57248, &P22237_309};	/*Property = GeoLocation*/
const Value V22237_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22237_312 = {S30, 0, &V22237_312_57254, &P22237_310};	/*Property = History*/
const Value V22237_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22237_314 = {S31, 0, &V22237_314_57261, &P22237_312};	/*Property = Html5*/
const Value V22237_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22237_313 = {S32, 0, &V22237_313_57258, &P22237_314};	/*Property = Html-Media-Capture*/
const Value V22237_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22237_315 = {S33, 0, &V22237_315_57264, &P22237_313};	/*Property = Iframe*/
const Value V22237_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22237_316 = {S34, 0, &V22237_316_57266, &P22237_315};	/*Property = IndexedDB*/
const Value V22237_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22237_317 = {S35, 0, &V22237_317_57269, &P22237_316};	/*Property = Json*/
const Value V22237_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22237_15 = {S37, 0, &V22237_15_28, &P22237_317};	/*Property = LayoutEngine*/
const Value V22237_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22237_344 = {S38, 0, &V22237_344_57272, &P22237_15};	/*Property = Masking*/
const Value V22237_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22237_320 = {S39, 0, &V22237_320_57290, &P22237_344};	/*Property = PostMessage*/
const Value V22237_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22237_321 = {S40, 0, &V22237_321_57293, &P22237_320};	/*Property = Progress*/
const Value V22237_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22237_322 = {S41, 0, &V22237_322_57296, &P22237_321};	/*Property = Prompts*/
const Value V22237_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22237_323 = {S42, 0, &V22237_323_57299, &P22237_322};	/*Property = Selector*/
const Value V22237_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22237_324 = {S43, 0, &V22237_324_57306, &P22237_323};	/*Property = Svg*/
const Value V22237_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22237_327 = {S44, 0, &V22237_327_57315, &P22237_324};	/*Property = TouchEvents*/
const Value V22237_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22237_328 = {S45, 0, &V22237_328_57318, &P22237_327};	/*Property = Track*/
const Value V22237_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22237_329 = {S46, 0, &V22237_329_57323, &P22237_328};	/*Property = Video*/
const Value V22237_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22237_330 = {S47, 0, &V22237_330_57326, &P22237_329};	/*Property = Viewport*/
const Value V22237_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22237_332 = {S48, 0, &V22237_332_57338, &P22237_330};	/*Property = WebWorkers*/
const Value V22237_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22237_333 = {S49, 0, &V22237_333_57342, &P22237_332};	/*Property = Xhr2*/
const Profile P22237 = {"22237", &P22237_333};	/*Profile ID - 22237*/
const Value V22921_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_279 = {S1, 0, &V22921_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22921_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_282 = {S3, 0, &V22921_282_57155, &P22921_279};	/*Property = BlobBuilder*/
const Value V22921_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_350 = {S4, 0, &V22921_350_57446, &P22921_282};	/*Property = Canvas*/
const Value V22921_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_283 = {S5, 0, &V22921_283_57161, &P22921_350};	/*Property = CssBackground*/
const Value V22921_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_284 = {S6, 0, &V22921_284_57164, &P22921_283};	/*Property = CssBorderImage*/
const Value V22921_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_285 = {S7, 0, &V22921_285_57167, &P22921_284};	/*Property = CssCanvas*/
const Value V22921_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_286 = {S8, 0, &V22921_286_57170, &P22921_285};	/*Property = CssColor*/
const Value V22921_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_237 = {S9, 0, &V22921_237_55786, &P22921_286};	/*Property = CssColumn*/
const Value V22921_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_287 = {S10, 0, &V22921_287_57173, &P22921_237};	/*Property = CssFlexbox*/
const Value V22921_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_288 = {S11, 0, &V22921_288_57176, &P22921_287};	/*Property = CssFont*/
const Value V22921_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_289 = {S12, 0, &V22921_289_86899, &P22921_288};	/*Property = CssImages*/
const Value V22921_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_291 = {S13, 0, &V22921_291_57182, &P22921_289};	/*Property = CssMediaQueries*/
const Value V22921_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_292 = {S14, 0, &V22921_292_57185, &P22921_291};	/*Property = CssMinMax*/
const Value V22921_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_293 = {S15, 0, &V22921_293_57188, &P22921_292};	/*Property = CssOverflow*/
const Value V22921_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_294 = {S16, 0, &V22921_294_57191, &P22921_293};	/*Property = CssPosition*/
const Value V22921_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_295 = {S17, 0, &V22921_295_57194, &P22921_294};	/*Property = CssText*/
const Value V22921_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_297 = {S18, 0, &V22921_297_57197, &P22921_295};	/*Property = CssTransforms*/
const Value V22921_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_241 = {S19, 0, &V22921_241_55801, &P22921_297};	/*Property = CssTransitions*/
const Value V22921_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_298 = {S20, 0, &V22921_298_57208, &P22921_241};	/*Property = CssUI*/
const Value V22921_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_300 = {S21, 0, &V22921_300_57217, &P22921_298};	/*Property = DataSet*/
const Value V22921_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_301 = {S22, 0, &V22921_301_57220, &P22921_300};	/*Property = DataUrl*/
const Value V22921_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_302 = {S23, 0, &V22921_302_57223, &P22921_301};	/*Property = DeviceOrientation*/
const Value V22921_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_304 = {S24, 0, &V22921_304_57229, &P22921_302};	/*Property = FileReader*/
const Value V22921_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_305 = {S25, 0, &V22921_305_57232, &P22921_304};	/*Property = FileSaver*/
const Value V22921_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_306 = {S26, 0, &V22921_306_57238, &P22921_305};	/*Property = FileWriter*/
const Value V22921_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_307 = {S27, 0, &V22921_307_57241, &P22921_306};	/*Property = FormData*/
const Value V22921_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_309 = {S28, 0, &V22921_309_57247, &P22921_307};	/*Property = Fullscreen*/
const Value V22921_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_310 = {S29, 0, &V22921_310_57250, &P22921_309};	/*Property = GeoLocation*/
const Value V22921_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_312 = {S30, 0, &V22921_312_57256, &P22921_310};	/*Property = History*/
const Value V22921_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_314 = {S31, 0, &V22921_314_57262, &P22921_312};	/*Property = Html5*/
const Value V22921_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_313 = {S32, 0, &V22921_313_57259, &P22921_314};	/*Property = Html-Media-Capture*/
const Value V22921_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_315 = {S33, 0, &V22921_315_57265, &P22921_313};	/*Property = Iframe*/
const Value V22921_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_316 = {S34, 0, &V22921_316_57268, &P22921_315};	/*Property = IndexedDB*/
const Value V22921_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_317 = {S35, 0, &V22921_317_57271, &P22921_316};	/*Property = Json*/
const Value V22921_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_15 = {S37, 0, &V22921_15_72, &P22921_317};	/*Property = LayoutEngine*/
const Value V22921_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_344 = {S38, 0, &V22921_344_57274, &P22921_15};	/*Property = Masking*/
const Value V22921_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_320 = {S39, 0, &V22921_320_57292, &P22921_344};	/*Property = PostMessage*/
const Value V22921_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_321 = {S40, 0, &V22921_321_57295, &P22921_320};	/*Property = Progress*/
const Value V22921_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_322 = {S41, 0, &V22921_322_57298, &P22921_321};	/*Property = Prompts*/
const Value V22921_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_323 = {S42, 0, &V22921_323_57301, &P22921_322};	/*Property = Selector*/
const Value V22921_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_324 = {S43, 0, &V22921_324_57307, &P22921_323};	/*Property = Svg*/
const Value V22921_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_327 = {S44, 0, &V22921_327_57316, &P22921_324};	/*Property = TouchEvents*/
const Value V22921_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_328 = {S45, 0, &V22921_328_57319, &P22921_327};	/*Property = Track*/
const Value V22921_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_329 = {S46, 0, &V22921_329_57325, &P22921_328};	/*Property = Video*/
const Value V22921_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_330 = {S47, 0, &V22921_330_57328, &P22921_329};	/*Property = Viewport*/
const Value V22921_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_332 = {S48, 0, &V22921_332_57340, &P22921_330};	/*Property = WebWorkers*/
const Value V22921_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22921_333 = {S49, 0, &V22921_333_57343, &P22921_332};	/*Property = Xhr2*/
const Profile P22921 = {"22921", &P22921_333};	/*Profile ID - 22921*/
const Value V21785_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_279 = {S1, 0, &V21785_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21785_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_282 = {S3, 0, &V21785_282_57155, &P21785_279};	/*Property = BlobBuilder*/
const Value V21785_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_350 = {S4, 0, &V21785_350_57446, &P21785_282};	/*Property = Canvas*/
const Value V21785_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_283 = {S5, 0, &V21785_283_57161, &P21785_350};	/*Property = CssBackground*/
const Value V21785_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_284 = {S6, 0, &V21785_284_57164, &P21785_283};	/*Property = CssBorderImage*/
const Value V21785_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_285 = {S7, 0, &V21785_285_57167, &P21785_284};	/*Property = CssCanvas*/
const Value V21785_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_286 = {S8, 0, &V21785_286_57170, &P21785_285};	/*Property = CssColor*/
const Value V21785_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_237 = {S9, 0, &V21785_237_55786, &P21785_286};	/*Property = CssColumn*/
const Value V21785_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_287 = {S10, 0, &V21785_287_57173, &P21785_237};	/*Property = CssFlexbox*/
const Value V21785_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_288 = {S11, 0, &V21785_288_57176, &P21785_287};	/*Property = CssFont*/
const Value V21785_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_289 = {S12, 0, &V21785_289_86899, &P21785_288};	/*Property = CssImages*/
const Value V21785_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_291 = {S13, 0, &V21785_291_57182, &P21785_289};	/*Property = CssMediaQueries*/
const Value V21785_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_292 = {S14, 0, &V21785_292_57185, &P21785_291};	/*Property = CssMinMax*/
const Value V21785_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_293 = {S15, 0, &V21785_293_57188, &P21785_292};	/*Property = CssOverflow*/
const Value V21785_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_294 = {S16, 0, &V21785_294_57191, &P21785_293};	/*Property = CssPosition*/
const Value V21785_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_295 = {S17, 0, &V21785_295_57194, &P21785_294};	/*Property = CssText*/
const Value V21785_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_297 = {S18, 0, &V21785_297_57197, &P21785_295};	/*Property = CssTransforms*/
const Value V21785_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_241 = {S19, 0, &V21785_241_55801, &P21785_297};	/*Property = CssTransitions*/
const Value V21785_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_298 = {S20, 0, &V21785_298_57208, &P21785_241};	/*Property = CssUI*/
const Value V21785_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_300 = {S21, 0, &V21785_300_57217, &P21785_298};	/*Property = DataSet*/
const Value V21785_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_301 = {S22, 0, &V21785_301_57220, &P21785_300};	/*Property = DataUrl*/
const Value V21785_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_302 = {S23, 0, &V21785_302_57223, &P21785_301};	/*Property = DeviceOrientation*/
const Value V21785_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_304 = {S24, 0, &V21785_304_57229, &P21785_302};	/*Property = FileReader*/
const Value V21785_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_305 = {S25, 0, &V21785_305_57232, &P21785_304};	/*Property = FileSaver*/
const Value V21785_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_306 = {S26, 0, &V21785_306_57238, &P21785_305};	/*Property = FileWriter*/
const Value V21785_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_307 = {S27, 0, &V21785_307_57241, &P21785_306};	/*Property = FormData*/
const Value V21785_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_309 = {S28, 0, &V21785_309_57247, &P21785_307};	/*Property = Fullscreen*/
const Value V21785_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_310 = {S29, 0, &V21785_310_57250, &P21785_309};	/*Property = GeoLocation*/
const Value V21785_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_312 = {S30, 0, &V21785_312_57256, &P21785_310};	/*Property = History*/
const Value V21785_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_314 = {S31, 0, &V21785_314_57262, &P21785_312};	/*Property = Html5*/
const Value V21785_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_313 = {S32, 0, &V21785_313_57259, &P21785_314};	/*Property = Html-Media-Capture*/
const Value V21785_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_315 = {S33, 0, &V21785_315_57265, &P21785_313};	/*Property = Iframe*/
const Value V21785_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_316 = {S34, 0, &V21785_316_57268, &P21785_315};	/*Property = IndexedDB*/
const Value V21785_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_317 = {S35, 0, &V21785_317_57271, &P21785_316};	/*Property = Json*/
const Value V21785_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_15 = {S37, 0, &V21785_15_72, &P21785_317};	/*Property = LayoutEngine*/
const Value V21785_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_344 = {S38, 0, &V21785_344_57274, &P21785_15};	/*Property = Masking*/
const Value V21785_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_320 = {S39, 0, &V21785_320_57292, &P21785_344};	/*Property = PostMessage*/
const Value V21785_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_321 = {S40, 0, &V21785_321_57295, &P21785_320};	/*Property = Progress*/
const Value V21785_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_322 = {S41, 0, &V21785_322_57298, &P21785_321};	/*Property = Prompts*/
const Value V21785_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_323 = {S42, 0, &V21785_323_57301, &P21785_322};	/*Property = Selector*/
const Value V21785_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_324 = {S43, 0, &V21785_324_57307, &P21785_323};	/*Property = Svg*/
const Value V21785_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_327 = {S44, 0, &V21785_327_57316, &P21785_324};	/*Property = TouchEvents*/
const Value V21785_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_328 = {S45, 0, &V21785_328_57319, &P21785_327};	/*Property = Track*/
const Value V21785_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_329 = {S46, 0, &V21785_329_57325, &P21785_328};	/*Property = Video*/
const Value V21785_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_330 = {S47, 0, &V21785_330_57328, &P21785_329};	/*Property = Viewport*/
const Value V21785_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_332 = {S48, 0, &V21785_332_57340, &P21785_330};	/*Property = WebWorkers*/
const Value V21785_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21785_333 = {S49, 0, &V21785_333_57343, &P21785_332};	/*Property = Xhr2*/
const Profile P21785 = {"21785", &P21785_333};	/*Profile ID - 21785*/
const Value V23994_1_1 = {S2, NULL};	/*Value = True*/
const Property P23994_1 = {S50, 0, &V23994_1_1, NULL};	/*Property = IsMobile*/
const Value V23994_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23994_5 = {S52, 0, &V23994_5_1277, &P23994_1};	/*Property = ScreenPixelsHeight*/
const Value V23994_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23994_4 = {S54, 0, &V23994_4_1279, &P23994_5};	/*Property = ScreenPixelsWidth*/
const Profile P23994 = {"23994", &P23994_4};	/*Profile ID - 23994*/
const Value V21588_1_1 = {S2, NULL};	/*Value = True*/
const Property P21588_1 = {S50, 0, &V21588_1_1, NULL};	/*Property = IsMobile*/
const Value V21588_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21588_5 = {S52, 0, &V21588_5_1227, &P21588_1};	/*Property = ScreenPixelsHeight*/
const Value V21588_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21588_4 = {S54, 0, &V21588_4_1229, &P21588_5};	/*Property = ScreenPixelsWidth*/
const Profile P21588 = {"21588", &P21588_4};	/*Profile ID - 21588*/
const Value V21102_1_1 = {S2, NULL};	/*Value = True*/
const Property P21102_1 = {S50, 0, &V21102_1_1, NULL};	/*Property = IsMobile*/
const Value V21102_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21102_5 = {S52, 0, &V21102_5_1277, &P21102_1};	/*Property = ScreenPixelsHeight*/
const Value V21102_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21102_4 = {S54, 0, &V21102_4_1279, &P21102_5};	/*Property = ScreenPixelsWidth*/
const Profile P21102 = {"21102", &P21102_4};	/*Profile ID - 21102*/
const Value V20166_1_1 = {S2, NULL};	/*Value = True*/
const Property P20166_1 = {S50, 0, &V20166_1_1, NULL};	/*Property = IsMobile*/
const Value V20166_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20166_5 = {S52, 0, &V20166_5_1222, &P20166_1};	/*Property = ScreenPixelsHeight*/
const Value V20166_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20166_4 = {S54, 0, &V20166_4_1224, &P20166_5};	/*Property = ScreenPixelsWidth*/
const Profile P20166 = {"20166", &P20166_4};	/*Profile ID - 20166*/
const Value V20444_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_279 = {S1, 0, &V20444_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20444_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_282 = {S3, 0, &V20444_282_57155, &P20444_279};	/*Property = BlobBuilder*/
const Value V20444_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_350 = {S4, 0, &V20444_350_57446, &P20444_282};	/*Property = Canvas*/
const Value V20444_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_283 = {S5, 0, &V20444_283_57161, &P20444_350};	/*Property = CssBackground*/
const Value V20444_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_284 = {S6, 0, &V20444_284_57164, &P20444_283};	/*Property = CssBorderImage*/
const Value V20444_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_285 = {S7, 0, &V20444_285_57167, &P20444_284};	/*Property = CssCanvas*/
const Value V20444_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_286 = {S8, 0, &V20444_286_57170, &P20444_285};	/*Property = CssColor*/
const Value V20444_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_237 = {S9, 0, &V20444_237_55786, &P20444_286};	/*Property = CssColumn*/
const Value V20444_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_287 = {S10, 0, &V20444_287_57173, &P20444_237};	/*Property = CssFlexbox*/
const Value V20444_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_288 = {S11, 0, &V20444_288_57176, &P20444_287};	/*Property = CssFont*/
const Value V20444_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_289 = {S12, 0, &V20444_289_86899, &P20444_288};	/*Property = CssImages*/
const Value V20444_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_291 = {S13, 0, &V20444_291_57182, &P20444_289};	/*Property = CssMediaQueries*/
const Value V20444_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_292 = {S14, 0, &V20444_292_57185, &P20444_291};	/*Property = CssMinMax*/
const Value V20444_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_293 = {S15, 0, &V20444_293_57188, &P20444_292};	/*Property = CssOverflow*/
const Value V20444_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_294 = {S16, 0, &V20444_294_57191, &P20444_293};	/*Property = CssPosition*/
const Value V20444_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_295 = {S17, 0, &V20444_295_57194, &P20444_294};	/*Property = CssText*/
const Value V20444_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_297 = {S18, 0, &V20444_297_57197, &P20444_295};	/*Property = CssTransforms*/
const Value V20444_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_241 = {S19, 0, &V20444_241_55801, &P20444_297};	/*Property = CssTransitions*/
const Value V20444_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_298 = {S20, 0, &V20444_298_57208, &P20444_241};	/*Property = CssUI*/
const Value V20444_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_300 = {S21, 0, &V20444_300_57217, &P20444_298};	/*Property = DataSet*/
const Value V20444_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_301 = {S22, 0, &V20444_301_57220, &P20444_300};	/*Property = DataUrl*/
const Value V20444_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_302 = {S23, 0, &V20444_302_57223, &P20444_301};	/*Property = DeviceOrientation*/
const Value V20444_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_304 = {S24, 0, &V20444_304_57229, &P20444_302};	/*Property = FileReader*/
const Value V20444_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_305 = {S25, 0, &V20444_305_57232, &P20444_304};	/*Property = FileSaver*/
const Value V20444_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_306 = {S26, 0, &V20444_306_57238, &P20444_305};	/*Property = FileWriter*/
const Value V20444_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_307 = {S27, 0, &V20444_307_57241, &P20444_306};	/*Property = FormData*/
const Value V20444_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_309 = {S28, 0, &V20444_309_57247, &P20444_307};	/*Property = Fullscreen*/
const Value V20444_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_310 = {S29, 0, &V20444_310_57250, &P20444_309};	/*Property = GeoLocation*/
const Value V20444_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_312 = {S30, 0, &V20444_312_57256, &P20444_310};	/*Property = History*/
const Value V20444_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_314 = {S31, 0, &V20444_314_57262, &P20444_312};	/*Property = Html5*/
const Value V20444_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_313 = {S32, 0, &V20444_313_57259, &P20444_314};	/*Property = Html-Media-Capture*/
const Value V20444_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_315 = {S33, 0, &V20444_315_57265, &P20444_313};	/*Property = Iframe*/
const Value V20444_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_316 = {S34, 0, &V20444_316_57268, &P20444_315};	/*Property = IndexedDB*/
const Value V20444_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_317 = {S35, 0, &V20444_317_57271, &P20444_316};	/*Property = Json*/
const Value V20444_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20444_15 = {S37, 0, &V20444_15_29, &P20444_317};	/*Property = LayoutEngine*/
const Value V20444_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_344 = {S38, 0, &V20444_344_57274, &P20444_15};	/*Property = Masking*/
const Value V20444_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_320 = {S39, 0, &V20444_320_57292, &P20444_344};	/*Property = PostMessage*/
const Value V20444_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_321 = {S40, 0, &V20444_321_57295, &P20444_320};	/*Property = Progress*/
const Value V20444_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_322 = {S41, 0, &V20444_322_57298, &P20444_321};	/*Property = Prompts*/
const Value V20444_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_323 = {S42, 0, &V20444_323_57301, &P20444_322};	/*Property = Selector*/
const Value V20444_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_324 = {S43, 0, &V20444_324_57307, &P20444_323};	/*Property = Svg*/
const Value V20444_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_327 = {S44, 0, &V20444_327_57316, &P20444_324};	/*Property = TouchEvents*/
const Value V20444_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_328 = {S45, 0, &V20444_328_57319, &P20444_327};	/*Property = Track*/
const Value V20444_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_329 = {S46, 0, &V20444_329_57325, &P20444_328};	/*Property = Video*/
const Value V20444_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_330 = {S47, 0, &V20444_330_57328, &P20444_329};	/*Property = Viewport*/
const Value V20444_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_332 = {S48, 0, &V20444_332_57340, &P20444_330};	/*Property = WebWorkers*/
const Value V20444_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20444_333 = {S49, 0, &V20444_333_57343, &P20444_332};	/*Property = Xhr2*/
const Profile P20444 = {"20444", &P20444_333};	/*Profile ID - 20444*/
const Value V18324_1_1 = {S2, NULL};	/*Value = True*/
const Property P18324_1 = {S50, 0, &V18324_1_1, NULL};	/*Property = IsMobile*/
const Value V18324_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18324_5 = {S52, 0, &V18324_5_1277, &P18324_1};	/*Property = ScreenPixelsHeight*/
const Value V18324_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18324_4 = {S54, 0, &V18324_4_1279, &P18324_5};	/*Property = ScreenPixelsWidth*/
const Profile P18324 = {"18324", &P18324_4};	/*Profile ID - 18324*/
const Value V24515_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_279 = {S1, 0, &V24515_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24515_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_282 = {S3, 0, &V24515_282_57155, &P24515_279};	/*Property = BlobBuilder*/
const Value V24515_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_350 = {S4, 0, &V24515_350_57446, &P24515_282};	/*Property = Canvas*/
const Value V24515_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_283 = {S5, 0, &V24515_283_57161, &P24515_350};	/*Property = CssBackground*/
const Value V24515_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_284 = {S6, 0, &V24515_284_57164, &P24515_283};	/*Property = CssBorderImage*/
const Value V24515_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_285 = {S7, 0, &V24515_285_57167, &P24515_284};	/*Property = CssCanvas*/
const Value V24515_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_286 = {S8, 0, &V24515_286_57170, &P24515_285};	/*Property = CssColor*/
const Value V24515_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_237 = {S9, 0, &V24515_237_55786, &P24515_286};	/*Property = CssColumn*/
const Value V24515_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_287 = {S10, 0, &V24515_287_57173, &P24515_237};	/*Property = CssFlexbox*/
const Value V24515_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_288 = {S11, 0, &V24515_288_57176, &P24515_287};	/*Property = CssFont*/
const Value V24515_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_289 = {S12, 0, &V24515_289_86899, &P24515_288};	/*Property = CssImages*/
const Value V24515_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_291 = {S13, 0, &V24515_291_57182, &P24515_289};	/*Property = CssMediaQueries*/
const Value V24515_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_292 = {S14, 0, &V24515_292_57185, &P24515_291};	/*Property = CssMinMax*/
const Value V24515_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_293 = {S15, 0, &V24515_293_57188, &P24515_292};	/*Property = CssOverflow*/
const Value V24515_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_294 = {S16, 0, &V24515_294_57191, &P24515_293};	/*Property = CssPosition*/
const Value V24515_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_295 = {S17, 0, &V24515_295_57194, &P24515_294};	/*Property = CssText*/
const Value V24515_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_297 = {S18, 0, &V24515_297_57197, &P24515_295};	/*Property = CssTransforms*/
const Value V24515_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_241 = {S19, 0, &V24515_241_55801, &P24515_297};	/*Property = CssTransitions*/
const Value V24515_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_298 = {S20, 0, &V24515_298_57208, &P24515_241};	/*Property = CssUI*/
const Value V24515_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_300 = {S21, 0, &V24515_300_57217, &P24515_298};	/*Property = DataSet*/
const Value V24515_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_301 = {S22, 0, &V24515_301_57220, &P24515_300};	/*Property = DataUrl*/
const Value V24515_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_302 = {S23, 0, &V24515_302_57223, &P24515_301};	/*Property = DeviceOrientation*/
const Value V24515_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_304 = {S24, 0, &V24515_304_57229, &P24515_302};	/*Property = FileReader*/
const Value V24515_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_305 = {S25, 0, &V24515_305_57232, &P24515_304};	/*Property = FileSaver*/
const Value V24515_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_306 = {S26, 0, &V24515_306_57238, &P24515_305};	/*Property = FileWriter*/
const Value V24515_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_307 = {S27, 0, &V24515_307_57241, &P24515_306};	/*Property = FormData*/
const Value V24515_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_309 = {S28, 0, &V24515_309_57247, &P24515_307};	/*Property = Fullscreen*/
const Value V24515_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_310 = {S29, 0, &V24515_310_57250, &P24515_309};	/*Property = GeoLocation*/
const Value V24515_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_312 = {S30, 0, &V24515_312_57256, &P24515_310};	/*Property = History*/
const Value V24515_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_314 = {S31, 0, &V24515_314_57262, &P24515_312};	/*Property = Html5*/
const Value V24515_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_313 = {S32, 0, &V24515_313_57259, &P24515_314};	/*Property = Html-Media-Capture*/
const Value V24515_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_315 = {S33, 0, &V24515_315_57265, &P24515_313};	/*Property = Iframe*/
const Value V24515_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_316 = {S34, 0, &V24515_316_57268, &P24515_315};	/*Property = IndexedDB*/
const Value V24515_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_317 = {S35, 0, &V24515_317_57271, &P24515_316};	/*Property = Json*/
const Value V24515_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P24515_15 = {S37, 0, &V24515_15_31, &P24515_317};	/*Property = LayoutEngine*/
const Value V24515_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_344 = {S38, 0, &V24515_344_57274, &P24515_15};	/*Property = Masking*/
const Value V24515_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_320 = {S39, 0, &V24515_320_57292, &P24515_344};	/*Property = PostMessage*/
const Value V24515_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_321 = {S40, 0, &V24515_321_57295, &P24515_320};	/*Property = Progress*/
const Value V24515_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_322 = {S41, 0, &V24515_322_57298, &P24515_321};	/*Property = Prompts*/
const Value V24515_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_323 = {S42, 0, &V24515_323_57301, &P24515_322};	/*Property = Selector*/
const Value V24515_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_324 = {S43, 0, &V24515_324_57307, &P24515_323};	/*Property = Svg*/
const Value V24515_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_327 = {S44, 0, &V24515_327_57316, &P24515_324};	/*Property = TouchEvents*/
const Value V24515_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_328 = {S45, 0, &V24515_328_57319, &P24515_327};	/*Property = Track*/
const Value V24515_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_329 = {S46, 0, &V24515_329_57325, &P24515_328};	/*Property = Video*/
const Value V24515_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_330 = {S47, 0, &V24515_330_57328, &P24515_329};	/*Property = Viewport*/
const Value V24515_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_332 = {S48, 0, &V24515_332_57340, &P24515_330};	/*Property = WebWorkers*/
const Value V24515_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24515_333 = {S49, 0, &V24515_333_57343, &P24515_332};	/*Property = Xhr2*/
const Profile P24515 = {"24515", &P24515_333};	/*Profile ID - 24515*/
const Value V22236_1_1 = {S2, NULL};	/*Value = True*/
const Property P22236_1 = {S50, 0, &V22236_1_1, NULL};	/*Property = IsMobile*/
const Value V22236_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22236_5 = {S52, 0, &V22236_5_1222, &P22236_1};	/*Property = ScreenPixelsHeight*/
const Value V22236_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22236_4 = {S54, 0, &V22236_4_1229, &P22236_5};	/*Property = ScreenPixelsWidth*/
const Profile P22236 = {"22236", &P22236_4};	/*Profile ID - 22236*/
const Profile P22473 = {"22473", NULL};	/*Profile ID - 22473*/
const Value V21218_1_1 = {S2, NULL};	/*Value = True*/
const Property P21218_1 = {S50, 0, &V21218_1_1, NULL};	/*Property = IsMobile*/
const Value V21218_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21218_5 = {S52, 0, &V21218_5_1222, &P21218_1};	/*Property = ScreenPixelsHeight*/
const Value V21218_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21218_4 = {S54, 0, &V21218_4_1224, &P21218_5};	/*Property = ScreenPixelsWidth*/
const Profile P21218 = {"21218", &P21218_4};	/*Profile ID - 21218*/
const Value V20276_1_1 = {S2, NULL};	/*Value = True*/
const Property P20276_1 = {S50, 0, &V20276_1_1, NULL};	/*Property = IsMobile*/
const Value V20276_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20276_5 = {S52, 0, &V20276_5_1227, &P20276_1};	/*Property = ScreenPixelsHeight*/
const Value V20276_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20276_4 = {S54, 0, &V20276_4_1229, &P20276_5};	/*Property = ScreenPixelsWidth*/
const Profile P20276 = {"20276", &P20276_4};	/*Profile ID - 20276*/
const Value V20445_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_279 = {S1, 0, &V20445_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20445_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_282 = {S3, 0, &V20445_282_57155, &P20445_279};	/*Property = BlobBuilder*/
const Value V20445_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_350 = {S4, 0, &V20445_350_57446, &P20445_282};	/*Property = Canvas*/
const Value V20445_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_283 = {S5, 0, &V20445_283_57161, &P20445_350};	/*Property = CssBackground*/
const Value V20445_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_284 = {S6, 0, &V20445_284_57164, &P20445_283};	/*Property = CssBorderImage*/
const Value V20445_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_285 = {S7, 0, &V20445_285_57167, &P20445_284};	/*Property = CssCanvas*/
const Value V20445_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_286 = {S8, 0, &V20445_286_57170, &P20445_285};	/*Property = CssColor*/
const Value V20445_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_237 = {S9, 0, &V20445_237_55786, &P20445_286};	/*Property = CssColumn*/
const Value V20445_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_287 = {S10, 0, &V20445_287_57173, &P20445_237};	/*Property = CssFlexbox*/
const Value V20445_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_288 = {S11, 0, &V20445_288_57176, &P20445_287};	/*Property = CssFont*/
const Value V20445_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_289 = {S12, 0, &V20445_289_86899, &P20445_288};	/*Property = CssImages*/
const Value V20445_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_291 = {S13, 0, &V20445_291_57182, &P20445_289};	/*Property = CssMediaQueries*/
const Value V20445_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_292 = {S14, 0, &V20445_292_57185, &P20445_291};	/*Property = CssMinMax*/
const Value V20445_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_293 = {S15, 0, &V20445_293_57188, &P20445_292};	/*Property = CssOverflow*/
const Value V20445_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_294 = {S16, 0, &V20445_294_57191, &P20445_293};	/*Property = CssPosition*/
const Value V20445_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_295 = {S17, 0, &V20445_295_57194, &P20445_294};	/*Property = CssText*/
const Value V20445_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_297 = {S18, 0, &V20445_297_57197, &P20445_295};	/*Property = CssTransforms*/
const Value V20445_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_241 = {S19, 0, &V20445_241_55801, &P20445_297};	/*Property = CssTransitions*/
const Value V20445_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_298 = {S20, 0, &V20445_298_57208, &P20445_241};	/*Property = CssUI*/
const Value V20445_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_300 = {S21, 0, &V20445_300_57217, &P20445_298};	/*Property = DataSet*/
const Value V20445_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_301 = {S22, 0, &V20445_301_57220, &P20445_300};	/*Property = DataUrl*/
const Value V20445_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_302 = {S23, 0, &V20445_302_57223, &P20445_301};	/*Property = DeviceOrientation*/
const Value V20445_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_304 = {S24, 0, &V20445_304_57229, &P20445_302};	/*Property = FileReader*/
const Value V20445_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_305 = {S25, 0, &V20445_305_57232, &P20445_304};	/*Property = FileSaver*/
const Value V20445_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_306 = {S26, 0, &V20445_306_57238, &P20445_305};	/*Property = FileWriter*/
const Value V20445_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_307 = {S27, 0, &V20445_307_57241, &P20445_306};	/*Property = FormData*/
const Value V20445_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_309 = {S28, 0, &V20445_309_57247, &P20445_307};	/*Property = Fullscreen*/
const Value V20445_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_310 = {S29, 0, &V20445_310_57250, &P20445_309};	/*Property = GeoLocation*/
const Value V20445_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_312 = {S30, 0, &V20445_312_57256, &P20445_310};	/*Property = History*/
const Value V20445_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_314 = {S31, 0, &V20445_314_57262, &P20445_312};	/*Property = Html5*/
const Value V20445_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_313 = {S32, 0, &V20445_313_57259, &P20445_314};	/*Property = Html-Media-Capture*/
const Value V20445_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_315 = {S33, 0, &V20445_315_57265, &P20445_313};	/*Property = Iframe*/
const Value V20445_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_316 = {S34, 0, &V20445_316_57268, &P20445_315};	/*Property = IndexedDB*/
const Value V20445_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_317 = {S35, 0, &V20445_317_57271, &P20445_316};	/*Property = Json*/
const Value V20445_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20445_15 = {S37, 0, &V20445_15_29, &P20445_317};	/*Property = LayoutEngine*/
const Value V20445_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_344 = {S38, 0, &V20445_344_57274, &P20445_15};	/*Property = Masking*/
const Value V20445_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_320 = {S39, 0, &V20445_320_57292, &P20445_344};	/*Property = PostMessage*/
const Value V20445_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_321 = {S40, 0, &V20445_321_57295, &P20445_320};	/*Property = Progress*/
const Value V20445_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_322 = {S41, 0, &V20445_322_57298, &P20445_321};	/*Property = Prompts*/
const Value V20445_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_323 = {S42, 0, &V20445_323_57301, &P20445_322};	/*Property = Selector*/
const Value V20445_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_324 = {S43, 0, &V20445_324_57307, &P20445_323};	/*Property = Svg*/
const Value V20445_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_327 = {S44, 0, &V20445_327_57316, &P20445_324};	/*Property = TouchEvents*/
const Value V20445_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_328 = {S45, 0, &V20445_328_57319, &P20445_327};	/*Property = Track*/
const Value V20445_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_329 = {S46, 0, &V20445_329_57325, &P20445_328};	/*Property = Video*/
const Value V20445_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_330 = {S47, 0, &V20445_330_57328, &P20445_329};	/*Property = Viewport*/
const Value V20445_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_332 = {S48, 0, &V20445_332_57340, &P20445_330};	/*Property = WebWorkers*/
const Value V20445_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20445_333 = {S49, 0, &V20445_333_57343, &P20445_332};	/*Property = Xhr2*/
const Profile P20445 = {"20445", &P20445_333};	/*Profile ID - 20445*/
const Value V23668_279_57148 = {S0, NULL};	/*Value = False*/
const Property P23668_279 = {S1, 0, &V23668_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V23668_282_57153 = {S2, NULL};	/*Value = True*/
const Property P23668_282 = {S3, 0, &V23668_282_57153, &P23668_279};	/*Property = BlobBuilder*/
const Value V23668_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23668_350 = {S4, 0, &V23668_350_57444, &P23668_282};	/*Property = Canvas*/
const Value V23668_283_57160 = {S0, NULL};	/*Value = False*/
const Property P23668_283 = {S5, 0, &V23668_283_57160, &P23668_350};	/*Property = CssBackground*/
const Value V23668_284_57163 = {S0, NULL};	/*Value = False*/
const Property P23668_284 = {S6, 0, &V23668_284_57163, &P23668_283};	/*Property = CssBorderImage*/
const Value V23668_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23668_285 = {S7, 0, &V23668_285_57166, &P23668_284};	/*Property = CssCanvas*/
const Value V23668_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23668_286 = {S8, 0, &V23668_286_57168, &P23668_285};	/*Property = CssColor*/
const Value V23668_237_55785 = {S0, NULL};	/*Value = False*/
const Property P23668_237 = {S9, 0, &V23668_237_55785, &P23668_286};	/*Property = CssColumn*/
const Value V23668_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23668_287 = {S10, 0, &V23668_287_57172, &P23668_237};	/*Property = CssFlexbox*/
const Value V23668_288_57175 = {S0, NULL};	/*Value = False*/
const Property P23668_288 = {S11, 0, &V23668_288_57175, &P23668_287};	/*Property = CssFont*/
const Value V23668_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23668_289 = {S12, 0, &V23668_289_86899, &P23668_288};	/*Property = CssImages*/
const Value V23668_291_57181 = {S0, NULL};	/*Value = False*/
const Property P23668_291 = {S13, 0, &V23668_291_57181, &P23668_289};	/*Property = CssMediaQueries*/
const Value V23668_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23668_292 = {S14, 0, &V23668_292_57183, &P23668_291};	/*Property = CssMinMax*/
const Value V23668_293_57187 = {S0, NULL};	/*Value = False*/
const Property P23668_293 = {S15, 0, &V23668_293_57187, &P23668_292};	/*Property = CssOverflow*/
const Value V23668_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23668_294 = {S16, 0, &V23668_294_57189, &P23668_293};	/*Property = CssPosition*/
const Value V23668_295_57193 = {S0, NULL};	/*Value = False*/
const Property P23668_295 = {S17, 0, &V23668_295_57193, &P23668_294};	/*Property = CssText*/
const Value V23668_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23668_297 = {S18, 0, &V23668_297_57195, &P23668_295};	/*Property = CssTransforms*/
const Value V23668_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23668_241 = {S19, 0, &V23668_241_55799, &P23668_297};	/*Property = CssTransitions*/
const Value V23668_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23668_298 = {S20, 0, &V23668_298_57206, &P23668_241};	/*Property = CssUI*/
const Value V23668_300_57215 = {S2, NULL};	/*Value = True*/
const Property P23668_300 = {S21, 0, &V23668_300_57215, &P23668_298};	/*Property = DataSet*/
const Value V23668_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23668_301 = {S22, 0, &V23668_301_57218, &P23668_300};	/*Property = DataUrl*/
const Value V23668_302_57221 = {S2, NULL};	/*Value = True*/
const Property P23668_302 = {S23, 0, &V23668_302_57221, &P23668_301};	/*Property = DeviceOrientation*/
const Value V23668_304_57227 = {S2, NULL};	/*Value = True*/
const Property P23668_304 = {S24, 0, &V23668_304_57227, &P23668_302};	/*Property = FileReader*/
const Value V23668_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23668_305 = {S25, 0, &V23668_305_57231, &P23668_304};	/*Property = FileSaver*/
const Value V23668_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23668_306 = {S26, 0, &V23668_306_57237, &P23668_305};	/*Property = FileWriter*/
const Value V23668_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23668_307 = {S27, 0, &V23668_307_57239, &P23668_306};	/*Property = FormData*/
const Value V23668_309_57245 = {S2, NULL};	/*Value = True*/
const Property P23668_309 = {S28, 0, &V23668_309_57245, &P23668_307};	/*Property = Fullscreen*/
const Value V23668_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23668_310 = {S29, 0, &V23668_310_57248, &P23668_309};	/*Property = GeoLocation*/
const Value V23668_312_57254 = {S2, NULL};	/*Value = True*/
const Property P23668_312 = {S30, 0, &V23668_312_57254, &P23668_310};	/*Property = History*/
const Value V23668_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23668_314 = {S31, 0, &V23668_314_57261, &P23668_312};	/*Property = Html5*/
const Value V23668_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23668_313 = {S32, 0, &V23668_313_57258, &P23668_314};	/*Property = Html-Media-Capture*/
const Value V23668_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23668_315 = {S33, 0, &V23668_315_57264, &P23668_313};	/*Property = Iframe*/
const Value V23668_316_57266 = {S2, NULL};	/*Value = True*/
const Property P23668_316 = {S34, 0, &V23668_316_57266, &P23668_315};	/*Property = IndexedDB*/
const Value V23668_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23668_317 = {S35, 0, &V23668_317_57269, &P23668_316};	/*Property = Json*/
const Value V23668_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P23668_15 = {S37, 0, &V23668_15_29, &P23668_317};	/*Property = LayoutEngine*/
const Value V23668_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23668_344 = {S38, 0, &V23668_344_57272, &P23668_15};	/*Property = Masking*/
const Value V23668_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23668_320 = {S39, 0, &V23668_320_57290, &P23668_344};	/*Property = PostMessage*/
const Value V23668_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23668_321 = {S40, 0, &V23668_321_57293, &P23668_320};	/*Property = Progress*/
const Value V23668_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23668_322 = {S41, 0, &V23668_322_57296, &P23668_321};	/*Property = Prompts*/
const Value V23668_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23668_323 = {S42, 0, &V23668_323_57299, &P23668_322};	/*Property = Selector*/
const Value V23668_324_57305 = {S2, NULL};	/*Value = True*/
const Property P23668_324 = {S43, 0, &V23668_324_57305, &P23668_323};	/*Property = Svg*/
const Value V23668_327_57315 = {S0, NULL};	/*Value = False*/
const Property P23668_327 = {S44, 0, &V23668_327_57315, &P23668_324};	/*Property = TouchEvents*/
const Value V23668_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23668_328 = {S45, 0, &V23668_328_57318, &P23668_327};	/*Property = Track*/
const Value V23668_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23668_329 = {S46, 0, &V23668_329_57323, &P23668_328};	/*Property = Video*/
const Value V23668_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23668_330 = {S47, 0, &V23668_330_57326, &P23668_329};	/*Property = Viewport*/
const Value V23668_332_57339 = {S0, NULL};	/*Value = False*/
const Property P23668_332 = {S48, 0, &V23668_332_57339, &P23668_330};	/*Property = WebWorkers*/
const Value V23668_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23668_333 = {S49, 0, &V23668_333_57342, &P23668_332};	/*Property = Xhr2*/
const Profile P23668 = {"23668", &P23668_333};	/*Profile ID - 23668*/
const Profile P23111 = {"23111", NULL};	/*Profile ID - 23111*/
const Value V21212_1_1 = {S2, NULL};	/*Value = True*/
const Property P21212_1 = {S50, 0, &V21212_1_1, NULL};	/*Property = IsMobile*/
const Value V21212_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21212_5 = {S52, 0, &V21212_5_1458, &P21212_1};	/*Property = ScreenPixelsHeight*/
const Value V21212_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21212_4 = {S54, 0, &V21212_4_37617, &P21212_5};	/*Property = ScreenPixelsWidth*/
const Profile P21212 = {"21212", &P21212_4};	/*Profile ID - 21212*/
const Value V20164_1_1 = {S2, NULL};	/*Value = True*/
const Property P20164_1 = {S50, 0, &V20164_1_1, NULL};	/*Property = IsMobile*/
const Value V20164_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20164_5 = {S52, 0, &V20164_5_1222, &P20164_1};	/*Property = ScreenPixelsHeight*/
const Value V20164_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20164_4 = {S54, 0, &V20164_4_1224, &P20164_5};	/*Property = ScreenPixelsWidth*/
const Profile P20164 = {"20164", &P20164_4};	/*Profile ID - 20164*/
const Value V16322_1_1 = {S2, NULL};	/*Value = True*/
const Property P16322_1 = {S50, 0, &V16322_1_1, NULL};	/*Property = IsMobile*/
const Value V16322_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16322_5 = {S52, 0, &V16322_5_1222, &P16322_1};	/*Property = ScreenPixelsHeight*/
const Value V16322_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16322_4 = {S54, 0, &V16322_4_1224, &P16322_5};	/*Property = ScreenPixelsWidth*/
const Profile P16322 = {"16322", &P16322_4};	/*Profile ID - 16322*/
const Value V21070_1_1 = {S2, NULL};	/*Value = True*/
const Property P21070_1 = {S50, 0, &V21070_1_1, NULL};	/*Property = IsMobile*/
const Value V21070_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P21070_5 = {S52, 0, &V21070_5_37569, &P21070_1};	/*Property = ScreenPixelsHeight*/
const Value V21070_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21070_4 = {S54, 0, &V21070_4_5645, &P21070_5};	/*Property = ScreenPixelsWidth*/
const Profile P21070 = {"21070", &P21070_4};	/*Profile ID - 21070*/
const Value V18514_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_279 = {S1, 0, &V18514_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18514_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_282 = {S3, 0, &V18514_282_57155, &P18514_279};	/*Property = BlobBuilder*/
const Value V18514_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_350 = {S4, 0, &V18514_350_57446, &P18514_282};	/*Property = Canvas*/
const Value V18514_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_283 = {S5, 0, &V18514_283_57161, &P18514_350};	/*Property = CssBackground*/
const Value V18514_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_284 = {S6, 0, &V18514_284_57164, &P18514_283};	/*Property = CssBorderImage*/
const Value V18514_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_285 = {S7, 0, &V18514_285_57167, &P18514_284};	/*Property = CssCanvas*/
const Value V18514_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_286 = {S8, 0, &V18514_286_57170, &P18514_285};	/*Property = CssColor*/
const Value V18514_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_237 = {S9, 0, &V18514_237_55786, &P18514_286};	/*Property = CssColumn*/
const Value V18514_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_287 = {S10, 0, &V18514_287_57173, &P18514_237};	/*Property = CssFlexbox*/
const Value V18514_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_288 = {S11, 0, &V18514_288_57176, &P18514_287};	/*Property = CssFont*/
const Value V18514_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_289 = {S12, 0, &V18514_289_86899, &P18514_288};	/*Property = CssImages*/
const Value V18514_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_291 = {S13, 0, &V18514_291_57182, &P18514_289};	/*Property = CssMediaQueries*/
const Value V18514_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_292 = {S14, 0, &V18514_292_57185, &P18514_291};	/*Property = CssMinMax*/
const Value V18514_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_293 = {S15, 0, &V18514_293_57188, &P18514_292};	/*Property = CssOverflow*/
const Value V18514_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_294 = {S16, 0, &V18514_294_57191, &P18514_293};	/*Property = CssPosition*/
const Value V18514_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_295 = {S17, 0, &V18514_295_57194, &P18514_294};	/*Property = CssText*/
const Value V18514_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_297 = {S18, 0, &V18514_297_57197, &P18514_295};	/*Property = CssTransforms*/
const Value V18514_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_241 = {S19, 0, &V18514_241_55801, &P18514_297};	/*Property = CssTransitions*/
const Value V18514_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_298 = {S20, 0, &V18514_298_57208, &P18514_241};	/*Property = CssUI*/
const Value V18514_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_300 = {S21, 0, &V18514_300_57217, &P18514_298};	/*Property = DataSet*/
const Value V18514_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_301 = {S22, 0, &V18514_301_57220, &P18514_300};	/*Property = DataUrl*/
const Value V18514_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_302 = {S23, 0, &V18514_302_57223, &P18514_301};	/*Property = DeviceOrientation*/
const Value V18514_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_304 = {S24, 0, &V18514_304_57229, &P18514_302};	/*Property = FileReader*/
const Value V18514_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_305 = {S25, 0, &V18514_305_57232, &P18514_304};	/*Property = FileSaver*/
const Value V18514_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_306 = {S26, 0, &V18514_306_57238, &P18514_305};	/*Property = FileWriter*/
const Value V18514_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_307 = {S27, 0, &V18514_307_57241, &P18514_306};	/*Property = FormData*/
const Value V18514_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_309 = {S28, 0, &V18514_309_57247, &P18514_307};	/*Property = Fullscreen*/
const Value V18514_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_310 = {S29, 0, &V18514_310_57250, &P18514_309};	/*Property = GeoLocation*/
const Value V18514_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_312 = {S30, 0, &V18514_312_57256, &P18514_310};	/*Property = History*/
const Value V18514_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_314 = {S31, 0, &V18514_314_57262, &P18514_312};	/*Property = Html5*/
const Value V18514_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_313 = {S32, 0, &V18514_313_57259, &P18514_314};	/*Property = Html-Media-Capture*/
const Value V18514_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_315 = {S33, 0, &V18514_315_57265, &P18514_313};	/*Property = Iframe*/
const Value V18514_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_316 = {S34, 0, &V18514_316_57268, &P18514_315};	/*Property = IndexedDB*/
const Value V18514_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_317 = {S35, 0, &V18514_317_57271, &P18514_316};	/*Property = Json*/
const Value V18514_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18514_15 = {S37, 0, &V18514_15_29, &P18514_317};	/*Property = LayoutEngine*/
const Value V18514_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_344 = {S38, 0, &V18514_344_57274, &P18514_15};	/*Property = Masking*/
const Value V18514_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_320 = {S39, 0, &V18514_320_57292, &P18514_344};	/*Property = PostMessage*/
const Value V18514_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_321 = {S40, 0, &V18514_321_57295, &P18514_320};	/*Property = Progress*/
const Value V18514_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_322 = {S41, 0, &V18514_322_57298, &P18514_321};	/*Property = Prompts*/
const Value V18514_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_323 = {S42, 0, &V18514_323_57301, &P18514_322};	/*Property = Selector*/
const Value V18514_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_324 = {S43, 0, &V18514_324_57307, &P18514_323};	/*Property = Svg*/
const Value V18514_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_327 = {S44, 0, &V18514_327_57316, &P18514_324};	/*Property = TouchEvents*/
const Value V18514_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_328 = {S45, 0, &V18514_328_57319, &P18514_327};	/*Property = Track*/
const Value V18514_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_329 = {S46, 0, &V18514_329_57325, &P18514_328};	/*Property = Video*/
const Value V18514_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_330 = {S47, 0, &V18514_330_57328, &P18514_329};	/*Property = Viewport*/
const Value V18514_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_332 = {S48, 0, &V18514_332_57340, &P18514_330};	/*Property = WebWorkers*/
const Value V18514_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18514_333 = {S49, 0, &V18514_333_57343, &P18514_332};	/*Property = Xhr2*/
const Profile P18514 = {"18514", &P18514_333};	/*Profile ID - 18514*/
const Value V19958_1_1 = {S2, NULL};	/*Value = True*/
const Property P19958_1 = {S50, 0, &V19958_1_1, NULL};	/*Property = IsMobile*/
const Value V19958_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P19958_5 = {S52, 0, &V19958_5_1222, &P19958_1};	/*Property = ScreenPixelsHeight*/
const Value V19958_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P19958_4 = {S54, 0, &V19958_4_1224, &P19958_5};	/*Property = ScreenPixelsWidth*/
const Profile P19958 = {"19958", &P19958_4};	/*Profile ID - 19958*/
const Value V21786_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_279 = {S1, 0, &V21786_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21786_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_282 = {S3, 0, &V21786_282_57155, &P21786_279};	/*Property = BlobBuilder*/
const Value V21786_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_350 = {S4, 0, &V21786_350_57446, &P21786_282};	/*Property = Canvas*/
const Value V21786_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_283 = {S5, 0, &V21786_283_57161, &P21786_350};	/*Property = CssBackground*/
const Value V21786_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_284 = {S6, 0, &V21786_284_57164, &P21786_283};	/*Property = CssBorderImage*/
const Value V21786_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_285 = {S7, 0, &V21786_285_57167, &P21786_284};	/*Property = CssCanvas*/
const Value V21786_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_286 = {S8, 0, &V21786_286_57170, &P21786_285};	/*Property = CssColor*/
const Value V21786_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_237 = {S9, 0, &V21786_237_55786, &P21786_286};	/*Property = CssColumn*/
const Value V21786_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_287 = {S10, 0, &V21786_287_57173, &P21786_237};	/*Property = CssFlexbox*/
const Value V21786_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_288 = {S11, 0, &V21786_288_57176, &P21786_287};	/*Property = CssFont*/
const Value V21786_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_289 = {S12, 0, &V21786_289_86899, &P21786_288};	/*Property = CssImages*/
const Value V21786_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_291 = {S13, 0, &V21786_291_57182, &P21786_289};	/*Property = CssMediaQueries*/
const Value V21786_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_292 = {S14, 0, &V21786_292_57185, &P21786_291};	/*Property = CssMinMax*/
const Value V21786_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_293 = {S15, 0, &V21786_293_57188, &P21786_292};	/*Property = CssOverflow*/
const Value V21786_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_294 = {S16, 0, &V21786_294_57191, &P21786_293};	/*Property = CssPosition*/
const Value V21786_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_295 = {S17, 0, &V21786_295_57194, &P21786_294};	/*Property = CssText*/
const Value V21786_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_297 = {S18, 0, &V21786_297_57197, &P21786_295};	/*Property = CssTransforms*/
const Value V21786_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_241 = {S19, 0, &V21786_241_55801, &P21786_297};	/*Property = CssTransitions*/
const Value V21786_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_298 = {S20, 0, &V21786_298_57208, &P21786_241};	/*Property = CssUI*/
const Value V21786_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_300 = {S21, 0, &V21786_300_57217, &P21786_298};	/*Property = DataSet*/
const Value V21786_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_301 = {S22, 0, &V21786_301_57220, &P21786_300};	/*Property = DataUrl*/
const Value V21786_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_302 = {S23, 0, &V21786_302_57223, &P21786_301};	/*Property = DeviceOrientation*/
const Value V21786_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_304 = {S24, 0, &V21786_304_57229, &P21786_302};	/*Property = FileReader*/
const Value V21786_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_305 = {S25, 0, &V21786_305_57232, &P21786_304};	/*Property = FileSaver*/
const Value V21786_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_306 = {S26, 0, &V21786_306_57238, &P21786_305};	/*Property = FileWriter*/
const Value V21786_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_307 = {S27, 0, &V21786_307_57241, &P21786_306};	/*Property = FormData*/
const Value V21786_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_309 = {S28, 0, &V21786_309_57247, &P21786_307};	/*Property = Fullscreen*/
const Value V21786_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_310 = {S29, 0, &V21786_310_57250, &P21786_309};	/*Property = GeoLocation*/
const Value V21786_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_312 = {S30, 0, &V21786_312_57256, &P21786_310};	/*Property = History*/
const Value V21786_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_314 = {S31, 0, &V21786_314_57262, &P21786_312};	/*Property = Html5*/
const Value V21786_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_313 = {S32, 0, &V21786_313_57259, &P21786_314};	/*Property = Html-Media-Capture*/
const Value V21786_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_315 = {S33, 0, &V21786_315_57265, &P21786_313};	/*Property = Iframe*/
const Value V21786_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_316 = {S34, 0, &V21786_316_57268, &P21786_315};	/*Property = IndexedDB*/
const Value V21786_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_317 = {S35, 0, &V21786_317_57271, &P21786_316};	/*Property = Json*/
const Value V21786_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_15 = {S37, 0, &V21786_15_72, &P21786_317};	/*Property = LayoutEngine*/
const Value V21786_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_344 = {S38, 0, &V21786_344_57274, &P21786_15};	/*Property = Masking*/
const Value V21786_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_320 = {S39, 0, &V21786_320_57292, &P21786_344};	/*Property = PostMessage*/
const Value V21786_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_321 = {S40, 0, &V21786_321_57295, &P21786_320};	/*Property = Progress*/
const Value V21786_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_322 = {S41, 0, &V21786_322_57298, &P21786_321};	/*Property = Prompts*/
const Value V21786_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_323 = {S42, 0, &V21786_323_57301, &P21786_322};	/*Property = Selector*/
const Value V21786_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_324 = {S43, 0, &V21786_324_57307, &P21786_323};	/*Property = Svg*/
const Value V21786_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_327 = {S44, 0, &V21786_327_57316, &P21786_324};	/*Property = TouchEvents*/
const Value V21786_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_328 = {S45, 0, &V21786_328_57319, &P21786_327};	/*Property = Track*/
const Value V21786_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_329 = {S46, 0, &V21786_329_57325, &P21786_328};	/*Property = Video*/
const Value V21786_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_330 = {S47, 0, &V21786_330_57328, &P21786_329};	/*Property = Viewport*/
const Value V21786_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_332 = {S48, 0, &V21786_332_57340, &P21786_330};	/*Property = WebWorkers*/
const Value V21786_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21786_333 = {S49, 0, &V21786_333_57343, &P21786_332};	/*Property = Xhr2*/
const Profile P21786 = {"21786", &P21786_333};	/*Profile ID - 21786*/
const Value V22051_1_1 = {S2, NULL};	/*Value = True*/
const Property P22051_1 = {S50, 0, &V22051_1_1, NULL};	/*Property = IsMobile*/
const Value V22051_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22051_5 = {S52, 0, &V22051_5_1832, &P22051_1};	/*Property = ScreenPixelsHeight*/
const Value V22051_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22051_4 = {S54, 0, &V22051_4_5645, &P22051_5};	/*Property = ScreenPixelsWidth*/
const Profile P22051 = {"22051", &P22051_4};	/*Profile ID - 22051*/
const Value V20203_1_1 = {S2, NULL};	/*Value = True*/
const Property P20203_1 = {S50, 0, &V20203_1_1, NULL};	/*Property = IsMobile*/
const Value V20203_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20203_5 = {S52, 0, &V20203_5_1227, &P20203_1};	/*Property = ScreenPixelsHeight*/
const Value V20203_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20203_4 = {S54, 0, &V20203_4_1229, &P20203_5};	/*Property = ScreenPixelsWidth*/
const Profile P20203 = {"20203", &P20203_4};	/*Profile ID - 20203*/
const Value V23839_1_1 = {S2, NULL};	/*Value = True*/
const Property P23839_1 = {S50, 0, &V23839_1_1, NULL};	/*Property = IsMobile*/
const Value V23839_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23839_5 = {S52, 0, &V23839_5_1227, &P23839_1};	/*Property = ScreenPixelsHeight*/
const Value V23839_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23839_4 = {S54, 0, &V23839_4_1229, &P23839_5};	/*Property = ScreenPixelsWidth*/
const Profile P23839 = {"23839", &P23839_4};	/*Profile ID - 23839*/
const Value V21068_1_1 = {S2, NULL};	/*Value = True*/
const Property P21068_1 = {S50, 0, &V21068_1_1, NULL};	/*Property = IsMobile*/
const Value V21068_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21068_5 = {S52, 0, &V21068_5_1269, &P21068_1};	/*Property = ScreenPixelsHeight*/
const Value V21068_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21068_4 = {S54, 0, &V21068_4_1229, &P21068_5};	/*Property = ScreenPixelsWidth*/
const Profile P21068 = {"21068", &P21068_4};	/*Profile ID - 21068*/
const Value V21989_1_1 = {S2, NULL};	/*Value = True*/
const Property P21989_1 = {S50, 0, &V21989_1_1, NULL};	/*Property = IsMobile*/
const Value V21989_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P21989_5 = {S52, 0, &V21989_5_1832, &P21989_1};	/*Property = ScreenPixelsHeight*/
const Value V21989_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21989_4 = {S54, 0, &V21989_4_2336, &P21989_5};	/*Property = ScreenPixelsWidth*/
const Profile P21989 = {"21989", &P21989_4};	/*Profile ID - 21989*/
const Value V18504_1_1 = {S2, NULL};	/*Value = True*/
const Property P18504_1 = {S50, 0, &V18504_1_1, NULL};	/*Property = IsMobile*/
const Value V18504_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18504_5 = {S52, 0, &V18504_5_1227, &P18504_1};	/*Property = ScreenPixelsHeight*/
const Value V18504_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18504_4 = {S54, 0, &V18504_4_1229, &P18504_5};	/*Property = ScreenPixelsWidth*/
const Profile P18504 = {"18504", &P18504_4};	/*Profile ID - 18504*/
const Value V21949_1_1 = {S2, NULL};	/*Value = True*/
const Property P21949_1 = {S50, 0, &V21949_1_1, NULL};	/*Property = IsMobile*/
const Value V21949_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21949_5 = {S52, 0, &V21949_5_1458, &P21949_1};	/*Property = ScreenPixelsHeight*/
const Value V21949_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21949_4 = {S54, 0, &V21949_4_37617, &P21949_5};	/*Property = ScreenPixelsWidth*/
const Profile P21949 = {"21949", &P21949_4};	/*Profile ID - 21949*/
const Value V20592_1_1 = {S2, NULL};	/*Value = True*/
const Property P20592_1 = {S50, 0, &V20592_1_1, NULL};	/*Property = IsMobile*/
const Value V20592_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20592_5 = {S52, 0, &V20592_5_1227, &P20592_1};	/*Property = ScreenPixelsHeight*/
const Value V20592_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20592_4 = {S54, 0, &V20592_4_1229, &P20592_5};	/*Property = ScreenPixelsWidth*/
const Profile P20592 = {"20592", &P20592_4};	/*Profile ID - 20592*/
const Value V15910_1_1 = {S2, NULL};	/*Value = True*/
const Property P15910_1 = {S50, 0, &V15910_1_1, NULL};	/*Property = IsMobile*/
const Value V15910_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15910_5 = {S52, 0, &V15910_5_1222, &P15910_1};	/*Property = ScreenPixelsHeight*/
const Value V15910_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15910_4 = {S54, 0, &V15910_4_1224, &P15910_5};	/*Property = ScreenPixelsWidth*/
const Profile P15910 = {"15910", &P15910_4};	/*Profile ID - 15910*/
const Value V21108_1_1 = {S2, NULL};	/*Value = True*/
const Property P21108_1 = {S50, 0, &V21108_1_1, NULL};	/*Property = IsMobile*/
const Value V21108_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21108_5 = {S52, 0, &V21108_5_1403, &P21108_1};	/*Property = ScreenPixelsHeight*/
const Value V21108_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21108_4 = {S54, 0, &V21108_4_1404, &P21108_5};	/*Property = ScreenPixelsWidth*/
const Profile P21108 = {"21108", &P21108_4};	/*Profile ID - 21108*/
const Value V17307_1_1 = {S2, NULL};	/*Value = True*/
const Property P17307_1 = {S50, 0, &V17307_1_1, NULL};	/*Property = IsMobile*/
const Value V17307_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17307_5 = {S52, 0, &V17307_5_1222, &P17307_1};	/*Property = ScreenPixelsHeight*/
const Value V17307_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17307_4 = {S54, 0, &V17307_4_1224, &P17307_5};	/*Property = ScreenPixelsWidth*/
const Profile P17307 = {"17307", &P17307_4};	/*Profile ID - 17307*/
const Value V18303_1_1 = {S2, NULL};	/*Value = True*/
const Property P18303_1 = {S50, 0, &V18303_1_1, NULL};	/*Property = IsMobile*/
const Value V18303_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P18303_5 = {S52, 0, &V18303_5_1269, &P18303_1};	/*Property = ScreenPixelsHeight*/
const Value V18303_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18303_4 = {S54, 0, &V18303_4_1229, &P18303_5};	/*Property = ScreenPixelsWidth*/
const Profile P18303 = {"18303", &P18303_4};	/*Profile ID - 18303*/
const Value V21436_1_1 = {S2, NULL};	/*Value = True*/
const Property P21436_1 = {S50, 0, &V21436_1_1, NULL};	/*Property = IsMobile*/
const Value V21436_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21436_5 = {S52, 0, &V21436_5_1277, &P21436_1};	/*Property = ScreenPixelsHeight*/
const Value V21436_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21436_4 = {S54, 0, &V21436_4_1279, &P21436_5};	/*Property = ScreenPixelsWidth*/
const Profile P21436 = {"21436", &P21436_4};	/*Profile ID - 21436*/
const Value V21111_1_1 = {S2, NULL};	/*Value = True*/
const Property P21111_1 = {S50, 0, &V21111_1_1, NULL};	/*Property = IsMobile*/
const Value V21111_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21111_5 = {S52, 0, &V21111_5_9981, &P21111_1};	/*Property = ScreenPixelsHeight*/
const Value V21111_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21111_4 = {S54, 0, &V21111_4_39098, &P21111_5};	/*Property = ScreenPixelsWidth*/
const Profile P21111 = {"21111", &P21111_4};	/*Profile ID - 21111*/
const Value V21821_1_1 = {S2, NULL};	/*Value = True*/
const Property P21821_1 = {S50, 0, &V21821_1_1, NULL};	/*Property = IsMobile*/
const Value V21821_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P21821_5 = {S52, 0, &V21821_5_1832, &P21821_1};	/*Property = ScreenPixelsHeight*/
const Value V21821_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21821_4 = {S54, 0, &V21821_4_5645, &P21821_5};	/*Property = ScreenPixelsWidth*/
const Profile P21821 = {"21821", &P21821_4};	/*Profile ID - 21821*/
const Value V21107_1_1 = {S2, NULL};	/*Value = True*/
const Property P21107_1 = {S50, 0, &V21107_1_1, NULL};	/*Property = IsMobile*/
const Value V21107_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21107_5 = {S52, 0, &V21107_5_1269, &P21107_1};	/*Property = ScreenPixelsHeight*/
const Value V21107_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21107_4 = {S54, 0, &V21107_4_1229, &P21107_5};	/*Property = ScreenPixelsWidth*/
const Profile P21107 = {"21107", &P21107_4};	/*Profile ID - 21107*/
const Value V20959_1_1 = {S2, NULL};	/*Value = True*/
const Property P20959_1 = {S50, 0, &V20959_1_1, NULL};	/*Property = IsMobile*/
const Value V20959_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20959_5 = {S52, 0, &V20959_5_1277, &P20959_1};	/*Property = ScreenPixelsHeight*/
const Value V20959_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20959_4 = {S54, 0, &V20959_4_1279, &P20959_5};	/*Property = ScreenPixelsWidth*/
const Profile P20959 = {"20959", &P20959_4};	/*Profile ID - 20959*/
const Profile P18342 = {"18342", NULL};	/*Profile ID - 18342*/
const Value V20149_1_1 = {S2, NULL};	/*Value = True*/
const Property P20149_1 = {S50, 0, &V20149_1_1, NULL};	/*Property = IsMobile*/
const Value V20149_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20149_5 = {S52, 0, &V20149_5_1458, &P20149_1};	/*Property = ScreenPixelsHeight*/
const Value V20149_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20149_4 = {S54, 0, &V20149_4_1460, &P20149_5};	/*Property = ScreenPixelsWidth*/
const Profile P20149 = {"20149", &P20149_4};	/*Profile ID - 20149*/
const Value V20376_1_1 = {S2, NULL};	/*Value = True*/
const Property P20376_1 = {S50, 0, &V20376_1_1, NULL};	/*Property = IsMobile*/
const Value V20376_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20376_5 = {S52, 0, &V20376_5_1458, &P20376_1};	/*Property = ScreenPixelsHeight*/
const Value V20376_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20376_4 = {S54, 0, &V20376_4_1460, &P20376_5};	/*Property = ScreenPixelsWidth*/
const Profile P20376 = {"20376", &P20376_4};	/*Profile ID - 20376*/
const Value V23898_1_1 = {S2, NULL};	/*Value = True*/
const Property P23898_1 = {S50, 0, &V23898_1_1, NULL};	/*Property = IsMobile*/
const Value V23898_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23898_5 = {S52, 0, &V23898_5_1227, &P23898_1};	/*Property = ScreenPixelsHeight*/
const Value V23898_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23898_4 = {S54, 0, &V23898_4_1229, &P23898_5};	/*Property = ScreenPixelsWidth*/
const Profile P23898 = {"23898", &P23898_4};	/*Profile ID - 23898*/
const Value V21282_1_1 = {S2, NULL};	/*Value = True*/
const Property P21282_1 = {S50, 0, &V21282_1_1, NULL};	/*Property = IsMobile*/
const Value V21282_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21282_5 = {S52, 0, &V21282_5_1277, &P21282_1};	/*Property = ScreenPixelsHeight*/
const Value V21282_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21282_4 = {S54, 0, &V21282_4_2336, &P21282_5};	/*Property = ScreenPixelsWidth*/
const Profile P21282 = {"21282", &P21282_4};	/*Profile ID - 21282*/
const Value V21334_1_1 = {S2, NULL};	/*Value = True*/
const Property P21334_1 = {S50, 0, &V21334_1_1, NULL};	/*Property = IsMobile*/
const Value V21334_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21334_5 = {S52, 0, &V21334_5_1277, &P21334_1};	/*Property = ScreenPixelsHeight*/
const Value V21334_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21334_4 = {S54, 0, &V21334_4_1279, &P21334_5};	/*Property = ScreenPixelsWidth*/
const Profile P21334 = {"21334", &P21334_4};	/*Profile ID - 21334*/
const Value V22949_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_279 = {S1, 0, &V22949_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22949_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_282 = {S3, 0, &V22949_282_57155, &P22949_279};	/*Property = BlobBuilder*/
const Value V22949_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_350 = {S4, 0, &V22949_350_57446, &P22949_282};	/*Property = Canvas*/
const Value V22949_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_283 = {S5, 0, &V22949_283_57161, &P22949_350};	/*Property = CssBackground*/
const Value V22949_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_284 = {S6, 0, &V22949_284_57164, &P22949_283};	/*Property = CssBorderImage*/
const Value V22949_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_285 = {S7, 0, &V22949_285_57167, &P22949_284};	/*Property = CssCanvas*/
const Value V22949_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_286 = {S8, 0, &V22949_286_57170, &P22949_285};	/*Property = CssColor*/
const Value V22949_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_237 = {S9, 0, &V22949_237_55786, &P22949_286};	/*Property = CssColumn*/
const Value V22949_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_287 = {S10, 0, &V22949_287_57173, &P22949_237};	/*Property = CssFlexbox*/
const Value V22949_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_288 = {S11, 0, &V22949_288_57176, &P22949_287};	/*Property = CssFont*/
const Value V22949_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_289 = {S12, 0, &V22949_289_86899, &P22949_288};	/*Property = CssImages*/
const Value V22949_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_291 = {S13, 0, &V22949_291_57182, &P22949_289};	/*Property = CssMediaQueries*/
const Value V22949_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_292 = {S14, 0, &V22949_292_57185, &P22949_291};	/*Property = CssMinMax*/
const Value V22949_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_293 = {S15, 0, &V22949_293_57188, &P22949_292};	/*Property = CssOverflow*/
const Value V22949_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_294 = {S16, 0, &V22949_294_57191, &P22949_293};	/*Property = CssPosition*/
const Value V22949_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_295 = {S17, 0, &V22949_295_57194, &P22949_294};	/*Property = CssText*/
const Value V22949_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_297 = {S18, 0, &V22949_297_57197, &P22949_295};	/*Property = CssTransforms*/
const Value V22949_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_241 = {S19, 0, &V22949_241_55801, &P22949_297};	/*Property = CssTransitions*/
const Value V22949_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_298 = {S20, 0, &V22949_298_57208, &P22949_241};	/*Property = CssUI*/
const Value V22949_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_300 = {S21, 0, &V22949_300_57217, &P22949_298};	/*Property = DataSet*/
const Value V22949_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_301 = {S22, 0, &V22949_301_57220, &P22949_300};	/*Property = DataUrl*/
const Value V22949_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_302 = {S23, 0, &V22949_302_57223, &P22949_301};	/*Property = DeviceOrientation*/
const Value V22949_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_304 = {S24, 0, &V22949_304_57229, &P22949_302};	/*Property = FileReader*/
const Value V22949_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_305 = {S25, 0, &V22949_305_57232, &P22949_304};	/*Property = FileSaver*/
const Value V22949_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_306 = {S26, 0, &V22949_306_57238, &P22949_305};	/*Property = FileWriter*/
const Value V22949_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_307 = {S27, 0, &V22949_307_57241, &P22949_306};	/*Property = FormData*/
const Value V22949_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_309 = {S28, 0, &V22949_309_57247, &P22949_307};	/*Property = Fullscreen*/
const Value V22949_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_310 = {S29, 0, &V22949_310_57250, &P22949_309};	/*Property = GeoLocation*/
const Value V22949_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_312 = {S30, 0, &V22949_312_57256, &P22949_310};	/*Property = History*/
const Value V22949_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_314 = {S31, 0, &V22949_314_57262, &P22949_312};	/*Property = Html5*/
const Value V22949_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_313 = {S32, 0, &V22949_313_57259, &P22949_314};	/*Property = Html-Media-Capture*/
const Value V22949_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_315 = {S33, 0, &V22949_315_57265, &P22949_313};	/*Property = Iframe*/
const Value V22949_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_316 = {S34, 0, &V22949_316_57268, &P22949_315};	/*Property = IndexedDB*/
const Value V22949_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_317 = {S35, 0, &V22949_317_57271, &P22949_316};	/*Property = Json*/
const Value V22949_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_15 = {S37, 0, &V22949_15_72, &P22949_317};	/*Property = LayoutEngine*/
const Value V22949_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_344 = {S38, 0, &V22949_344_57274, &P22949_15};	/*Property = Masking*/
const Value V22949_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_320 = {S39, 0, &V22949_320_57292, &P22949_344};	/*Property = PostMessage*/
const Value V22949_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_321 = {S40, 0, &V22949_321_57295, &P22949_320};	/*Property = Progress*/
const Value V22949_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_322 = {S41, 0, &V22949_322_57298, &P22949_321};	/*Property = Prompts*/
const Value V22949_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_323 = {S42, 0, &V22949_323_57301, &P22949_322};	/*Property = Selector*/
const Value V22949_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_324 = {S43, 0, &V22949_324_57307, &P22949_323};	/*Property = Svg*/
const Value V22949_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_327 = {S44, 0, &V22949_327_57316, &P22949_324};	/*Property = TouchEvents*/
const Value V22949_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_328 = {S45, 0, &V22949_328_57319, &P22949_327};	/*Property = Track*/
const Value V22949_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_329 = {S46, 0, &V22949_329_57325, &P22949_328};	/*Property = Video*/
const Value V22949_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_330 = {S47, 0, &V22949_330_57328, &P22949_329};	/*Property = Viewport*/
const Value V22949_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_332 = {S48, 0, &V22949_332_57340, &P22949_330};	/*Property = WebWorkers*/
const Value V22949_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22949_333 = {S49, 0, &V22949_333_57343, &P22949_332};	/*Property = Xhr2*/
const Profile P22949 = {"22949", &P22949_333};	/*Profile ID - 22949*/
const Value V22821_1_1 = {S2, NULL};	/*Value = True*/
const Property P22821_1 = {S50, 0, &V22821_1_1, NULL};	/*Property = IsMobile*/
const Value V22821_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22821_5 = {S52, 0, &V22821_5_1227, &P22821_1};	/*Property = ScreenPixelsHeight*/
const Value V22821_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22821_4 = {S54, 0, &V22821_4_1229, &P22821_5};	/*Property = ScreenPixelsWidth*/
const Profile P22821 = {"22821", &P22821_4};	/*Profile ID - 22821*/
const Value V19937_1_1 = {S2, NULL};	/*Value = True*/
const Property P19937_1 = {S50, 0, &V19937_1_1, NULL};	/*Property = IsMobile*/
const Value V19937_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P19937_5 = {S52, 0, &V19937_5_1277, &P19937_1};	/*Property = ScreenPixelsHeight*/
const Value V19937_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P19937_4 = {S54, 0, &V19937_4_1279, &P19937_5};	/*Property = ScreenPixelsWidth*/
const Profile P19937 = {"19937", &P19937_4};	/*Profile ID - 19937*/
const Value V16752_1_1 = {S2, NULL};	/*Value = True*/
const Property P16752_1 = {S50, 0, &V16752_1_1, NULL};	/*Property = IsMobile*/
const Value V16752_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16752_5 = {S52, 0, &V16752_5_1227, &P16752_1};	/*Property = ScreenPixelsHeight*/
const Value V16752_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16752_4 = {S54, 0, &V16752_4_1229, &P16752_5};	/*Property = ScreenPixelsWidth*/
const Profile P16752 = {"16752", &P16752_4};	/*Profile ID - 16752*/
const Value V20010_1_1 = {S2, NULL};	/*Value = True*/
const Property P20010_1 = {S50, 0, &V20010_1_1, NULL};	/*Property = IsMobile*/
const Value V20010_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20010_5 = {S52, 0, &V20010_5_1227, &P20010_1};	/*Property = ScreenPixelsHeight*/
const Value V20010_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20010_4 = {S54, 0, &V20010_4_1229, &P20010_5};	/*Property = ScreenPixelsWidth*/
const Profile P20010 = {"20010", &P20010_4};	/*Profile ID - 20010*/
const Value V21199_1_1 = {S2, NULL};	/*Value = True*/
const Property P21199_1 = {S50, 0, &V21199_1_1, NULL};	/*Property = IsMobile*/
const Value V21199_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21199_5 = {S52, 0, &V21199_5_1222, &P21199_1};	/*Property = ScreenPixelsHeight*/
const Value V21199_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21199_4 = {S54, 0, &V21199_4_1224, &P21199_5};	/*Property = ScreenPixelsWidth*/
const Profile P21199 = {"21199", &P21199_4};	/*Profile ID - 21199*/
const Value V23861_1_1 = {S2, NULL};	/*Value = True*/
const Property P23861_1 = {S50, 0, &V23861_1_1, NULL};	/*Property = IsMobile*/
const Value V23861_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23861_5 = {S52, 0, &V23861_5_1227, &P23861_1};	/*Property = ScreenPixelsHeight*/
const Value V23861_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23861_4 = {S54, 0, &V23861_4_1229, &P23861_5};	/*Property = ScreenPixelsWidth*/
const Profile P23861 = {"23861", &P23861_4};	/*Profile ID - 23861*/
const Value V21363_1_1 = {S2, NULL};	/*Value = True*/
const Property P21363_1 = {S50, 0, &V21363_1_1, NULL};	/*Property = IsMobile*/
const Value V21363_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21363_5 = {S52, 0, &V21363_5_1227, &P21363_1};	/*Property = ScreenPixelsHeight*/
const Value V21363_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21363_4 = {S54, 0, &V21363_4_1229, &P21363_5};	/*Property = ScreenPixelsWidth*/
const Profile P21363 = {"21363", &P21363_4};	/*Profile ID - 21363*/
const Value V18249_1_1 = {S2, NULL};	/*Value = True*/
const Property P18249_1 = {S50, 0, &V18249_1_1, NULL};	/*Property = IsMobile*/
const Value V18249_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18249_5 = {S52, 0, &V18249_5_1227, &P18249_1};	/*Property = ScreenPixelsHeight*/
const Value V18249_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18249_4 = {S54, 0, &V18249_4_1229, &P18249_5};	/*Property = ScreenPixelsWidth*/
const Profile P18249 = {"18249", &P18249_4};	/*Profile ID - 18249*/
const Value V22456_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22456_279 = {S1, 0, &V22456_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22456_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22456_282 = {S3, 0, &V22456_282_57154, &P22456_279};	/*Property = BlobBuilder*/
const Value V22456_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22456_350 = {S4, 0, &V22456_350_57444, &P22456_282};	/*Property = Canvas*/
const Value V22456_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22456_283 = {S5, 0, &V22456_283_57159, &P22456_350};	/*Property = CssBackground*/
const Value V22456_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22456_284 = {S6, 0, &V22456_284_57163, &P22456_283};	/*Property = CssBorderImage*/
const Value V22456_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22456_285 = {S7, 0, &V22456_285_57166, &P22456_284};	/*Property = CssCanvas*/
const Value V22456_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22456_286 = {S8, 0, &V22456_286_57168, &P22456_285};	/*Property = CssColor*/
const Value V22456_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22456_237 = {S9, 0, &V22456_237_55784, &P22456_286};	/*Property = CssColumn*/
const Value V22456_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22456_287 = {S10, 0, &V22456_287_57172, &P22456_237};	/*Property = CssFlexbox*/
const Value V22456_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22456_288 = {S11, 0, &V22456_288_57174, &P22456_287};	/*Property = CssFont*/
const Value V22456_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22456_289 = {S12, 0, &V22456_289_57759, &P22456_288};	/*Property = CssImages*/
const Value V22456_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22456_291 = {S13, 0, &V22456_291_57181, &P22456_289};	/*Property = CssMediaQueries*/
const Value V22456_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22456_292 = {S14, 0, &V22456_292_57183, &P22456_291};	/*Property = CssMinMax*/
const Value V22456_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22456_293 = {S15, 0, &V22456_293_57187, &P22456_292};	/*Property = CssOverflow*/
const Value V22456_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22456_294 = {S16, 0, &V22456_294_57189, &P22456_293};	/*Property = CssPosition*/
const Value V22456_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22456_295 = {S17, 0, &V22456_295_57192, &P22456_294};	/*Property = CssText*/
const Value V22456_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22456_297 = {S18, 0, &V22456_297_57195, &P22456_295};	/*Property = CssTransforms*/
const Value V22456_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22456_241 = {S19, 0, &V22456_241_55799, &P22456_297};	/*Property = CssTransitions*/
const Value V22456_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22456_298 = {S20, 0, &V22456_298_57206, &P22456_241};	/*Property = CssUI*/
const Value V22456_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22456_300 = {S21, 0, &V22456_300_57216, &P22456_298};	/*Property = DataSet*/
const Value V22456_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22456_301 = {S22, 0, &V22456_301_57218, &P22456_300};	/*Property = DataUrl*/
const Value V22456_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22456_302 = {S23, 0, &V22456_302_57222, &P22456_301};	/*Property = DeviceOrientation*/
const Value V22456_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22456_304 = {S24, 0, &V22456_304_57228, &P22456_302};	/*Property = FileReader*/
const Value V22456_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22456_305 = {S25, 0, &V22456_305_57231, &P22456_304};	/*Property = FileSaver*/
const Value V22456_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22456_306 = {S26, 0, &V22456_306_57237, &P22456_305};	/*Property = FileWriter*/
const Value V22456_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22456_307 = {S27, 0, &V22456_307_57240, &P22456_306};	/*Property = FormData*/
const Value V22456_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22456_309 = {S28, 0, &V22456_309_57246, &P22456_307};	/*Property = Fullscreen*/
const Value V22456_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22456_310 = {S29, 0, &V22456_310_57248, &P22456_309};	/*Property = GeoLocation*/
const Value V22456_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22456_312 = {S30, 0, &V22456_312_57255, &P22456_310};	/*Property = History*/
const Value V22456_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22456_314 = {S31, 0, &V22456_314_57261, &P22456_312};	/*Property = Html5*/
const Value V22456_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22456_313 = {S32, 0, &V22456_313_57258, &P22456_314};	/*Property = Html-Media-Capture*/
const Value V22456_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22456_315 = {S33, 0, &V22456_315_57264, &P22456_313};	/*Property = Iframe*/
const Value V22456_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22456_316 = {S34, 0, &V22456_316_57267, &P22456_315};	/*Property = IndexedDB*/
const Value V22456_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22456_317 = {S35, 0, &V22456_317_57269, &P22456_316};	/*Property = Json*/
const Value V22456_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22456_15 = {S37, 0, &V22456_15_28, &P22456_317};	/*Property = LayoutEngine*/
const Value V22456_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22456_344 = {S38, 0, &V22456_344_57272, &P22456_15};	/*Property = Masking*/
const Value V22456_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22456_320 = {S39, 0, &V22456_320_57290, &P22456_344};	/*Property = PostMessage*/
const Value V22456_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22456_321 = {S40, 0, &V22456_321_57293, &P22456_320};	/*Property = Progress*/
const Value V22456_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22456_322 = {S41, 0, &V22456_322_57296, &P22456_321};	/*Property = Prompts*/
const Value V22456_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22456_323 = {S42, 0, &V22456_323_57299, &P22456_322};	/*Property = Selector*/
const Value V22456_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22456_324 = {S43, 0, &V22456_324_57306, &P22456_323};	/*Property = Svg*/
const Value V22456_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22456_327 = {S44, 0, &V22456_327_57315, &P22456_324};	/*Property = TouchEvents*/
const Value V22456_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22456_328 = {S45, 0, &V22456_328_57318, &P22456_327};	/*Property = Track*/
const Value V22456_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22456_329 = {S46, 0, &V22456_329_57323, &P22456_328};	/*Property = Video*/
const Value V22456_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22456_330 = {S47, 0, &V22456_330_57326, &P22456_329};	/*Property = Viewport*/
const Value V22456_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22456_332 = {S48, 0, &V22456_332_57339, &P22456_330};	/*Property = WebWorkers*/
const Value V22456_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22456_333 = {S49, 0, &V22456_333_57342, &P22456_332};	/*Property = Xhr2*/
const Profile P22456 = {"22456", &P22456_333};	/*Profile ID - 22456*/
const Value V19998_1_1 = {S2, NULL};	/*Value = True*/
const Property P19998_1 = {S50, 0, &V19998_1_1, NULL};	/*Property = IsMobile*/
const Value V19998_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19998_5 = {S52, 0, &V19998_5_1227, &P19998_1};	/*Property = ScreenPixelsHeight*/
const Value V19998_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19998_4 = {S54, 0, &V19998_4_1229, &P19998_5};	/*Property = ScreenPixelsWidth*/
const Profile P19998 = {"19998", &P19998_4};	/*Profile ID - 19998*/
const Value V16404_1_1 = {S2, NULL};	/*Value = True*/
const Property P16404_1 = {S50, 0, &V16404_1_1, NULL};	/*Property = IsMobile*/
const Value V16404_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P16404_5 = {S52, 0, &V16404_5_1380, &P16404_1};	/*Property = ScreenPixelsHeight*/
const Value V16404_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16404_4 = {S54, 0, &V16404_4_1279, &P16404_5};	/*Property = ScreenPixelsWidth*/
const Profile P16404 = {"16404", &P16404_4};	/*Profile ID - 16404*/
const Value V21092_1_1 = {S2, NULL};	/*Value = True*/
const Property P21092_1 = {S50, 0, &V21092_1_1, NULL};	/*Property = IsMobile*/
const Value V21092_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21092_5 = {S52, 0, &V21092_5_37, &P21092_1};	/*Property = ScreenPixelsHeight*/
const Value V21092_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21092_4 = {S54, 0, &V21092_4_36, &P21092_5};	/*Property = ScreenPixelsWidth*/
const Profile P21092 = {"21092", &P21092_4};	/*Profile ID - 21092*/
const Value V19901_1_1 = {S2, NULL};	/*Value = True*/
const Property P19901_1 = {S50, 0, &V19901_1_1, NULL};	/*Property = IsMobile*/
const Value V19901_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P19901_5 = {S52, 0, &V19901_5_1233, &P19901_1};	/*Property = ScreenPixelsHeight*/
const Value V19901_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P19901_4 = {S54, 0, &V19901_4_1235, &P19901_5};	/*Property = ScreenPixelsWidth*/
const Profile P19901 = {"19901", &P19901_4};	/*Profile ID - 19901*/
const Value V17459_1_1 = {S2, NULL};	/*Value = True*/
const Property P17459_1 = {S50, 0, &V17459_1_1, NULL};	/*Property = IsMobile*/
const Value V17459_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17459_5 = {S52, 0, &V17459_5_1227, &P17459_1};	/*Property = ScreenPixelsHeight*/
const Value V17459_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17459_4 = {S54, 0, &V17459_4_1229, &P17459_5};	/*Property = ScreenPixelsWidth*/
const Profile P17459 = {"17459", &P17459_4};	/*Profile ID - 17459*/
const Value V21095_1_1 = {S2, NULL};	/*Value = True*/
const Property P21095_1 = {S50, 0, &V21095_1_1, NULL};	/*Property = IsMobile*/
const Value V21095_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21095_5 = {S52, 0, &V21095_5_37, &P21095_1};	/*Property = ScreenPixelsHeight*/
const Value V21095_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21095_4 = {S54, 0, &V21095_4_36, &P21095_5};	/*Property = ScreenPixelsWidth*/
const Profile P21095 = {"21095", &P21095_4};	/*Profile ID - 21095*/
const Value V21146_1_1 = {S2, NULL};	/*Value = True*/
const Property P21146_1 = {S50, 0, &V21146_1_1, NULL};	/*Property = IsMobile*/
const Value V21146_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21146_5 = {S52, 0, &V21146_5_1277, &P21146_1};	/*Property = ScreenPixelsHeight*/
const Value V21146_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21146_4 = {S54, 0, &V21146_4_1279, &P21146_5};	/*Property = ScreenPixelsWidth*/
const Profile P21146 = {"21146", &P21146_4};	/*Profile ID - 21146*/
const Value V20027_1_1 = {S2, NULL};	/*Value = True*/
const Property P20027_1 = {S50, 0, &V20027_1_1, NULL};	/*Property = IsMobile*/
const Value V20027_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P20027_5 = {S52, 0, &V20027_5_1304, &P20027_1};	/*Property = ScreenPixelsHeight*/
const Value V20027_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20027_4 = {S54, 0, &V20027_4_1224, &P20027_5};	/*Property = ScreenPixelsWidth*/
const Profile P20027 = {"20027", &P20027_4};	/*Profile ID - 20027*/
const Value V15778_1_1 = {S2, NULL};	/*Value = True*/
const Property P15778_1 = {S50, 0, &V15778_1_1, NULL};	/*Property = IsMobile*/
const Value V15778_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P15778_5 = {S52, 0, &V15778_5_1380, &P15778_1};	/*Property = ScreenPixelsHeight*/
const Value V15778_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15778_4 = {S54, 0, &V15778_4_1279, &P15778_5};	/*Property = ScreenPixelsWidth*/
const Profile P15778 = {"15778", &P15778_4};	/*Profile ID - 15778*/
const Value V21047_1_1 = {S2, NULL};	/*Value = True*/
const Property P21047_1 = {S50, 0, &V21047_1_1, NULL};	/*Property = IsMobile*/
const Value V21047_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21047_5 = {S52, 0, &V21047_5_1227, &P21047_1};	/*Property = ScreenPixelsHeight*/
const Value V21047_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21047_4 = {S54, 0, &V21047_4_1229, &P21047_5};	/*Property = ScreenPixelsWidth*/
const Profile P21047 = {"21047", &P21047_4};	/*Profile ID - 21047*/
const Value V24555_1_1 = {S2, NULL};	/*Value = True*/
const Property P24555_1 = {S50, 0, &V24555_1_1, NULL};	/*Property = IsMobile*/
const Value V24555_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P24555_5 = {S52, 0, &V24555_5_1222, &P24555_1};	/*Property = ScreenPixelsHeight*/
const Value V24555_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P24555_4 = {S54, 0, &V24555_4_1224, &P24555_5};	/*Property = ScreenPixelsWidth*/
const Profile P24555 = {"24555", &P24555_4};	/*Profile ID - 24555*/
const Value V18687_1_1 = {S2, NULL};	/*Value = True*/
const Property P18687_1 = {S50, 0, &V18687_1_1, NULL};	/*Property = IsMobile*/
const Value V18687_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18687_5 = {S52, 0, &V18687_5_1227, &P18687_1};	/*Property = ScreenPixelsHeight*/
const Value V18687_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18687_4 = {S54, 0, &V18687_4_1229, &P18687_5};	/*Property = ScreenPixelsWidth*/
const Profile P18687 = {"18687", &P18687_4};	/*Profile ID - 18687*/
const Value V21109_1_1 = {S2, NULL};	/*Value = True*/
const Property P21109_1 = {S50, 0, &V21109_1_1, NULL};	/*Property = IsMobile*/
const Value V21109_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21109_5 = {S52, 0, &V21109_5_1227, &P21109_1};	/*Property = ScreenPixelsHeight*/
const Value V21109_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21109_4 = {S54, 0, &V21109_4_1229, &P21109_5};	/*Property = ScreenPixelsWidth*/
const Profile P21109 = {"21109", &P21109_4};	/*Profile ID - 21109*/
const Value V18720_1_1 = {S2, NULL};	/*Value = True*/
const Property P18720_1 = {S50, 0, &V18720_1_1, NULL};	/*Property = IsMobile*/
const Value V18720_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18720_5 = {S52, 0, &V18720_5_1222, &P18720_1};	/*Property = ScreenPixelsHeight*/
const Value V18720_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18720_4 = {S54, 0, &V18720_4_1224, &P18720_5};	/*Property = ScreenPixelsWidth*/
const Profile P18720 = {"18720", &P18720_4};	/*Profile ID - 18720*/
const Value V20362_1_1 = {S2, NULL};	/*Value = True*/
const Property P20362_1 = {S50, 0, &V20362_1_1, NULL};	/*Property = IsMobile*/
const Value V20362_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20362_5 = {S52, 0, &V20362_5_1458, &P20362_1};	/*Property = ScreenPixelsHeight*/
const Value V20362_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20362_4 = {S54, 0, &V20362_4_37617, &P20362_5};	/*Property = ScreenPixelsWidth*/
const Profile P20362 = {"20362", &P20362_4};	/*Profile ID - 20362*/
const Value V21328_1_1 = {S2, NULL};	/*Value = True*/
const Property P21328_1 = {S50, 0, &V21328_1_1, NULL};	/*Property = IsMobile*/
const Value V21328_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21328_5 = {S52, 0, &V21328_5_1222, &P21328_1};	/*Property = ScreenPixelsHeight*/
const Value V21328_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21328_4 = {S54, 0, &V21328_4_1224, &P21328_5};	/*Property = ScreenPixelsWidth*/
const Profile P21328 = {"21328", &P21328_4};	/*Profile ID - 21328*/
const Value V22210_1_1 = {S2, NULL};	/*Value = True*/
const Property P22210_1 = {S50, 0, &V22210_1_1, NULL};	/*Property = IsMobile*/
const Value V22210_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22210_5 = {S52, 0, &V22210_5_1277, &P22210_1};	/*Property = ScreenPixelsHeight*/
const Value V22210_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P22210_4 = {S54, 0, &V22210_4_1278, &P22210_5};	/*Property = ScreenPixelsWidth*/
const Profile P22210 = {"22210", &P22210_4};	/*Profile ID - 22210*/
const Value V21396_1_1 = {S2, NULL};	/*Value = True*/
const Property P21396_1 = {S50, 0, &V21396_1_1, NULL};	/*Property = IsMobile*/
const Value V21396_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21396_5 = {S52, 0, &V21396_5_1277, &P21396_1};	/*Property = ScreenPixelsHeight*/
const Value V21396_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21396_4 = {S54, 0, &V21396_4_1279, &P21396_5};	/*Property = ScreenPixelsWidth*/
const Profile P21396 = {"21396", &P21396_4};	/*Profile ID - 21396*/
const Value V21033_1_1 = {S2, NULL};	/*Value = True*/
const Property P21033_1 = {S50, 0, &V21033_1_1, NULL};	/*Property = IsMobile*/
const Value V21033_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21033_5 = {S52, 0, &V21033_5_1227, &P21033_1};	/*Property = ScreenPixelsHeight*/
const Value V21033_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21033_4 = {S54, 0, &V21033_4_1229, &P21033_5};	/*Property = ScreenPixelsWidth*/
const Profile P21033 = {"21033", &P21033_4};	/*Profile ID - 21033*/
const Value V20588_1_1 = {S2, NULL};	/*Value = True*/
const Property P20588_1 = {S50, 0, &V20588_1_1, NULL};	/*Property = IsMobile*/
const Value V20588_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20588_5 = {S52, 0, &V20588_5_1458, &P20588_1};	/*Property = ScreenPixelsHeight*/
const Value V20588_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20588_4 = {S54, 0, &V20588_4_37617, &P20588_5};	/*Property = ScreenPixelsWidth*/
const Profile P20588 = {"20588", &P20588_4};	/*Profile ID - 20588*/
const Value V21399_1_1 = {S2, NULL};	/*Value = True*/
const Property P21399_1 = {S50, 0, &V21399_1_1, NULL};	/*Property = IsMobile*/
const Value V21399_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21399_5 = {S52, 0, &V21399_5_1277, &P21399_1};	/*Property = ScreenPixelsHeight*/
const Value V21399_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21399_4 = {S54, 0, &V21399_4_1279, &P21399_5};	/*Property = ScreenPixelsWidth*/
const Profile P21399 = {"21399", &P21399_4};	/*Profile ID - 21399*/
const Value V22814_1_1 = {S2, NULL};	/*Value = True*/
const Property P22814_1 = {S50, 0, &V22814_1_1, NULL};	/*Property = IsMobile*/
const Value V22814_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22814_5 = {S52, 0, &V22814_5_1832, &P22814_1};	/*Property = ScreenPixelsHeight*/
const Value V22814_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22814_4 = {S54, 0, &V22814_4_5645, &P22814_5};	/*Property = ScreenPixelsWidth*/
const Profile P22814 = {"22814", &P22814_4};	/*Profile ID - 22814*/
const Value V21439_1_1 = {S2, NULL};	/*Value = True*/
const Property P21439_1 = {S50, 0, &V21439_1_1, NULL};	/*Property = IsMobile*/
const Value V21439_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21439_5 = {S52, 0, &V21439_5_1222, &P21439_1};	/*Property = ScreenPixelsHeight*/
const Value V21439_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21439_4 = {S54, 0, &V21439_4_1224, &P21439_5};	/*Property = ScreenPixelsWidth*/
const Profile P21439 = {"21439", &P21439_4};	/*Profile ID - 21439*/
const Value V21298_1_1 = {S2, NULL};	/*Value = True*/
const Property P21298_1 = {S50, 0, &V21298_1_1, NULL};	/*Property = IsMobile*/
const Value V21298_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P21298_5 = {S52, 0, &V21298_5_60417, &P21298_1};	/*Property = ScreenPixelsHeight*/
const Value V21298_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21298_4 = {S54, 0, &V21298_4_39098, &P21298_5};	/*Property = ScreenPixelsWidth*/
const Profile P21298 = {"21298", &P21298_4};	/*Profile ID - 21298*/
const Profile P22195 = {"22195", NULL};	/*Profile ID - 22195*/
const Value V18243_1_1 = {S2, NULL};	/*Value = True*/
const Property P18243_1 = {S50, 0, &V18243_1_1, NULL};	/*Property = IsMobile*/
const Value V18243_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18243_5 = {S52, 0, &V18243_5_9981, &P18243_1};	/*Property = ScreenPixelsHeight*/
const Value V18243_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18243_4 = {S54, 0, &V18243_4_39098, &P18243_5};	/*Property = ScreenPixelsWidth*/
const Profile P18243 = {"18243", &P18243_4};	/*Profile ID - 18243*/
const Value V21040_1_1 = {S2, NULL};	/*Value = True*/
const Property P21040_1 = {S50, 0, &V21040_1_1, NULL};	/*Property = IsMobile*/
const Value V21040_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21040_5 = {S52, 0, &V21040_5_1227, &P21040_1};	/*Property = ScreenPixelsHeight*/
const Value V21040_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21040_4 = {S54, 0, &V21040_4_1229, &P21040_5};	/*Property = ScreenPixelsWidth*/
const Profile P21040 = {"21040", &P21040_4};	/*Profile ID - 21040*/
const Value V17972_1_1 = {S2, NULL};	/*Value = True*/
const Property P17972_1 = {S50, 0, &V17972_1_1, NULL};	/*Property = IsMobile*/
const Value V17972_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17972_5 = {S52, 0, &V17972_5_1227, &P17972_1};	/*Property = ScreenPixelsHeight*/
const Value V17972_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17972_4 = {S54, 0, &V17972_4_1229, &P17972_5};	/*Property = ScreenPixelsWidth*/
const Profile P17972 = {"17972", &P17972_4};	/*Profile ID - 17972*/
const Value V21828_1_1 = {S2, NULL};	/*Value = True*/
const Property P21828_1 = {S50, 0, &V21828_1_1, NULL};	/*Property = IsMobile*/
const Value V21828_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21828_5 = {S52, 0, &V21828_5_1277, &P21828_1};	/*Property = ScreenPixelsHeight*/
const Value V21828_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21828_4 = {S54, 0, &V21828_4_1279, &P21828_5};	/*Property = ScreenPixelsWidth*/
const Profile P21828 = {"21828", &P21828_4};	/*Profile ID - 21828*/
const Profile P18131 = {"18131", NULL};	/*Profile ID - 18131*/
const Value V21513_1_1 = {S2, NULL};	/*Value = True*/
const Property P21513_1 = {S50, 0, &V21513_1_1, NULL};	/*Property = IsMobile*/
const Value V21513_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21513_5 = {S52, 0, &V21513_5_1277, &P21513_1};	/*Property = ScreenPixelsHeight*/
const Value V21513_4_37618 = {S83, NULL};	/*Value = 272*/
const Property P21513_4 = {S54, 0, &V21513_4_37618, &P21513_5};	/*Property = ScreenPixelsWidth*/
const Profile P21513 = {"21513", &P21513_4};	/*Profile ID - 21513*/
const Value V21444_1_1 = {S2, NULL};	/*Value = True*/
const Property P21444_1 = {S50, 0, &V21444_1_1, NULL};	/*Property = IsMobile*/
const Value V21444_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21444_5 = {S52, 0, &V21444_5_37, &P21444_1};	/*Property = ScreenPixelsHeight*/
const Value V21444_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21444_4 = {S54, 0, &V21444_4_36, &P21444_5};	/*Property = ScreenPixelsWidth*/
const Profile P21444 = {"21444", &P21444_4};	/*Profile ID - 21444*/
const Value V19990_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19990_279 = {S1, 0, &V19990_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19990_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19990_282 = {S3, 0, &V19990_282_57154, &P19990_279};	/*Property = BlobBuilder*/
const Value V19990_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19990_350 = {S4, 0, &V19990_350_57444, &P19990_282};	/*Property = Canvas*/
const Value V19990_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19990_283 = {S5, 0, &V19990_283_57159, &P19990_350};	/*Property = CssBackground*/
const Value V19990_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19990_284 = {S6, 0, &V19990_284_57163, &P19990_283};	/*Property = CssBorderImage*/
const Value V19990_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19990_285 = {S7, 0, &V19990_285_57166, &P19990_284};	/*Property = CssCanvas*/
const Value V19990_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19990_286 = {S8, 0, &V19990_286_57168, &P19990_285};	/*Property = CssColor*/
const Value V19990_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19990_237 = {S9, 0, &V19990_237_55786, &P19990_286};	/*Property = CssColumn*/
const Value V19990_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19990_287 = {S10, 0, &V19990_287_57172, &P19990_237};	/*Property = CssFlexbox*/
const Value V19990_288_57175 = {S0, NULL};	/*Value = False*/
const Property P19990_288 = {S11, 0, &V19990_288_57175, &P19990_287};	/*Property = CssFont*/
const Value V19990_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19990_289 = {S12, 0, &V19990_289_57759, &P19990_288};	/*Property = CssImages*/
const Value V19990_291_57180 = {S2, NULL};	/*Value = True*/
const Property P19990_291 = {S13, 0, &V19990_291_57180, &P19990_289};	/*Property = CssMediaQueries*/
const Value V19990_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19990_292 = {S14, 0, &V19990_292_57183, &P19990_291};	/*Property = CssMinMax*/
const Value V19990_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19990_293 = {S15, 0, &V19990_293_57187, &P19990_292};	/*Property = CssOverflow*/
const Value V19990_294_57189 = {S2, NULL};	/*Value = True*/
const Property P19990_294 = {S16, 0, &V19990_294_57189, &P19990_293};	/*Property = CssPosition*/
const Value V19990_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19990_295 = {S17, 0, &V19990_295_57192, &P19990_294};	/*Property = CssText*/
const Value V19990_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19990_297 = {S18, 0, &V19990_297_57195, &P19990_295};	/*Property = CssTransforms*/
const Value V19990_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19990_241 = {S19, 0, &V19990_241_55799, &P19990_297};	/*Property = CssTransitions*/
const Value V19990_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19990_298 = {S20, 0, &V19990_298_57206, &P19990_241};	/*Property = CssUI*/
const Value V19990_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19990_300 = {S21, 0, &V19990_300_57216, &P19990_298};	/*Property = DataSet*/
const Value V19990_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19990_301 = {S22, 0, &V19990_301_57218, &P19990_300};	/*Property = DataUrl*/
const Value V19990_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19990_302 = {S23, 0, &V19990_302_57222, &P19990_301};	/*Property = DeviceOrientation*/
const Value V19990_304_57228 = {S0, NULL};	/*Value = False*/
const Property P19990_304 = {S24, 0, &V19990_304_57228, &P19990_302};	/*Property = FileReader*/
const Value V19990_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19990_305 = {S25, 0, &V19990_305_57231, &P19990_304};	/*Property = FileSaver*/
const Value V19990_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19990_306 = {S26, 0, &V19990_306_57237, &P19990_305};	/*Property = FileWriter*/
const Value V19990_307_57239 = {S2, NULL};	/*Value = True*/
const Property P19990_307 = {S27, 0, &V19990_307_57239, &P19990_306};	/*Property = FormData*/
const Value V19990_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19990_309 = {S28, 0, &V19990_309_57246, &P19990_307};	/*Property = Fullscreen*/
const Value V19990_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19990_310 = {S29, 0, &V19990_310_57248, &P19990_309};	/*Property = GeoLocation*/
const Value V19990_312_57255 = {S0, NULL};	/*Value = False*/
const Property P19990_312 = {S30, 0, &V19990_312_57255, &P19990_310};	/*Property = History*/
const Value V19990_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19990_314 = {S31, 0, &V19990_314_57261, &P19990_312};	/*Property = Html5*/
const Value V19990_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19990_313 = {S32, 0, &V19990_313_57258, &P19990_314};	/*Property = Html-Media-Capture*/
const Value V19990_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19990_315 = {S33, 0, &V19990_315_57264, &P19990_313};	/*Property = Iframe*/
const Value V19990_316_57267 = {S0, NULL};	/*Value = False*/
const Property P19990_316 = {S34, 0, &V19990_316_57267, &P19990_315};	/*Property = IndexedDB*/
const Value V19990_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19990_317 = {S35, 0, &V19990_317_57269, &P19990_316};	/*Property = Json*/
const Value V19990_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P19990_15 = {S37, 0, &V19990_15_30, &P19990_317};	/*Property = LayoutEngine*/
const Value V19990_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19990_344 = {S38, 0, &V19990_344_57272, &P19990_15};	/*Property = Masking*/
const Value V19990_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19990_320 = {S39, 0, &V19990_320_57290, &P19990_344};	/*Property = PostMessage*/
const Value V19990_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19990_321 = {S40, 0, &V19990_321_57293, &P19990_320};	/*Property = Progress*/
const Value V19990_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19990_322 = {S41, 0, &V19990_322_57296, &P19990_321};	/*Property = Prompts*/
const Value V19990_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19990_323 = {S42, 0, &V19990_323_57299, &P19990_322};	/*Property = Selector*/
const Value V19990_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19990_324 = {S43, 0, &V19990_324_57306, &P19990_323};	/*Property = Svg*/
const Value V19990_327_57315 = {S0, NULL};	/*Value = False*/
const Property P19990_327 = {S44, 0, &V19990_327_57315, &P19990_324};	/*Property = TouchEvents*/
const Value V19990_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19990_328 = {S45, 0, &V19990_328_57318, &P19990_327};	/*Property = Track*/
const Value V19990_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19990_329 = {S46, 0, &V19990_329_57323, &P19990_328};	/*Property = Video*/
const Value V19990_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19990_330 = {S47, 0, &V19990_330_57326, &P19990_329};	/*Property = Viewport*/
const Value V19990_332_57339 = {S0, NULL};	/*Value = False*/
const Property P19990_332 = {S48, 0, &V19990_332_57339, &P19990_330};	/*Property = WebWorkers*/
const Value V19990_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19990_333 = {S49, 0, &V19990_333_57342, &P19990_332};	/*Property = Xhr2*/
const Profile P19990 = {"19990", &P19990_333};	/*Profile ID - 19990*/
const Value V19752_1_1 = {S2, NULL};	/*Value = True*/
const Property P19752_1 = {S50, 0, &V19752_1_1, NULL};	/*Property = IsMobile*/
const Value V19752_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19752_5 = {S52, 0, &V19752_5_1227, &P19752_1};	/*Property = ScreenPixelsHeight*/
const Value V19752_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19752_4 = {S54, 0, &V19752_4_1229, &P19752_5};	/*Property = ScreenPixelsWidth*/
const Profile P19752 = {"19752", &P19752_4};	/*Profile ID - 19752*/
const Value V18581_1_1 = {S2, NULL};	/*Value = True*/
const Property P18581_1 = {S50, 0, &V18581_1_1, NULL};	/*Property = IsMobile*/
const Value V18581_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18581_5 = {S52, 0, &V18581_5_1277, &P18581_1};	/*Property = ScreenPixelsHeight*/
const Value V18581_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18581_4 = {S54, 0, &V18581_4_1279, &P18581_5};	/*Property = ScreenPixelsWidth*/
const Profile P18581 = {"18581", &P18581_4};	/*Profile ID - 18581*/
const Value V21195_1_1 = {S2, NULL};	/*Value = True*/
const Property P21195_1 = {S50, 0, &V21195_1_1, NULL};	/*Property = IsMobile*/
const Value V21195_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21195_5 = {S52, 0, &V21195_5_9981, &P21195_1};	/*Property = ScreenPixelsHeight*/
const Value V21195_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21195_4 = {S54, 0, &V21195_4_2336, &P21195_5};	/*Property = ScreenPixelsWidth*/
const Profile P21195 = {"21195", &P21195_4};	/*Profile ID - 21195*/
const Value V21818_1_1 = {S2, NULL};	/*Value = True*/
const Property P21818_1 = {S50, 0, &V21818_1_1, NULL};	/*Property = IsMobile*/
const Value V21818_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P21818_5 = {S52, 0, &V21818_5_1832, &P21818_1};	/*Property = ScreenPixelsHeight*/
const Value V21818_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21818_4 = {S54, 0, &V21818_4_2336, &P21818_5};	/*Property = ScreenPixelsWidth*/
const Profile P21818 = {"21818", &P21818_4};	/*Profile ID - 21818*/
const Value V21253_1_1 = {S2, NULL};	/*Value = True*/
const Property P21253_1 = {S50, 0, &V21253_1_1, NULL};	/*Property = IsMobile*/
const Value V21253_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21253_5 = {S52, 0, &V21253_5_1277, &P21253_1};	/*Property = ScreenPixelsHeight*/
const Value V21253_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21253_4 = {S54, 0, &V21253_4_1279, &P21253_5};	/*Property = ScreenPixelsWidth*/
const Profile P21253 = {"21253", &P21253_4};	/*Profile ID - 21253*/
const Value V22490_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_279 = {S1, 0, &V22490_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22490_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_282 = {S3, 0, &V22490_282_57155, &P22490_279};	/*Property = BlobBuilder*/
const Value V22490_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_350 = {S4, 0, &V22490_350_57446, &P22490_282};	/*Property = Canvas*/
const Value V22490_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_283 = {S5, 0, &V22490_283_57161, &P22490_350};	/*Property = CssBackground*/
const Value V22490_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_284 = {S6, 0, &V22490_284_57164, &P22490_283};	/*Property = CssBorderImage*/
const Value V22490_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_285 = {S7, 0, &V22490_285_57167, &P22490_284};	/*Property = CssCanvas*/
const Value V22490_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_286 = {S8, 0, &V22490_286_57170, &P22490_285};	/*Property = CssColor*/
const Value V22490_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_237 = {S9, 0, &V22490_237_55786, &P22490_286};	/*Property = CssColumn*/
const Value V22490_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_287 = {S10, 0, &V22490_287_57173, &P22490_237};	/*Property = CssFlexbox*/
const Value V22490_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_288 = {S11, 0, &V22490_288_57176, &P22490_287};	/*Property = CssFont*/
const Value V22490_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_289 = {S12, 0, &V22490_289_86899, &P22490_288};	/*Property = CssImages*/
const Value V22490_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_291 = {S13, 0, &V22490_291_57182, &P22490_289};	/*Property = CssMediaQueries*/
const Value V22490_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_292 = {S14, 0, &V22490_292_57185, &P22490_291};	/*Property = CssMinMax*/
const Value V22490_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_293 = {S15, 0, &V22490_293_57188, &P22490_292};	/*Property = CssOverflow*/
const Value V22490_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_294 = {S16, 0, &V22490_294_57191, &P22490_293};	/*Property = CssPosition*/
const Value V22490_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_295 = {S17, 0, &V22490_295_57194, &P22490_294};	/*Property = CssText*/
const Value V22490_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_297 = {S18, 0, &V22490_297_57197, &P22490_295};	/*Property = CssTransforms*/
const Value V22490_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_241 = {S19, 0, &V22490_241_55801, &P22490_297};	/*Property = CssTransitions*/
const Value V22490_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_298 = {S20, 0, &V22490_298_57208, &P22490_241};	/*Property = CssUI*/
const Value V22490_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_300 = {S21, 0, &V22490_300_57217, &P22490_298};	/*Property = DataSet*/
const Value V22490_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_301 = {S22, 0, &V22490_301_57220, &P22490_300};	/*Property = DataUrl*/
const Value V22490_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_302 = {S23, 0, &V22490_302_57223, &P22490_301};	/*Property = DeviceOrientation*/
const Value V22490_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_304 = {S24, 0, &V22490_304_57229, &P22490_302};	/*Property = FileReader*/
const Value V22490_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_305 = {S25, 0, &V22490_305_57232, &P22490_304};	/*Property = FileSaver*/
const Value V22490_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_306 = {S26, 0, &V22490_306_57238, &P22490_305};	/*Property = FileWriter*/
const Value V22490_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_307 = {S27, 0, &V22490_307_57241, &P22490_306};	/*Property = FormData*/
const Value V22490_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_309 = {S28, 0, &V22490_309_57247, &P22490_307};	/*Property = Fullscreen*/
const Value V22490_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_310 = {S29, 0, &V22490_310_57250, &P22490_309};	/*Property = GeoLocation*/
const Value V22490_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_312 = {S30, 0, &V22490_312_57256, &P22490_310};	/*Property = History*/
const Value V22490_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_314 = {S31, 0, &V22490_314_57262, &P22490_312};	/*Property = Html5*/
const Value V22490_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_313 = {S32, 0, &V22490_313_57259, &P22490_314};	/*Property = Html-Media-Capture*/
const Value V22490_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_315 = {S33, 0, &V22490_315_57265, &P22490_313};	/*Property = Iframe*/
const Value V22490_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_316 = {S34, 0, &V22490_316_57268, &P22490_315};	/*Property = IndexedDB*/
const Value V22490_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_317 = {S35, 0, &V22490_317_57271, &P22490_316};	/*Property = Json*/
const Value V22490_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22490_15 = {S37, 0, &V22490_15_29, &P22490_317};	/*Property = LayoutEngine*/
const Value V22490_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_344 = {S38, 0, &V22490_344_57274, &P22490_15};	/*Property = Masking*/
const Value V22490_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_320 = {S39, 0, &V22490_320_57292, &P22490_344};	/*Property = PostMessage*/
const Value V22490_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_321 = {S40, 0, &V22490_321_57295, &P22490_320};	/*Property = Progress*/
const Value V22490_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_322 = {S41, 0, &V22490_322_57298, &P22490_321};	/*Property = Prompts*/
const Value V22490_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_323 = {S42, 0, &V22490_323_57301, &P22490_322};	/*Property = Selector*/
const Value V22490_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_324 = {S43, 0, &V22490_324_57307, &P22490_323};	/*Property = Svg*/
const Value V22490_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_327 = {S44, 0, &V22490_327_57316, &P22490_324};	/*Property = TouchEvents*/
const Value V22490_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_328 = {S45, 0, &V22490_328_57319, &P22490_327};	/*Property = Track*/
const Value V22490_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_329 = {S46, 0, &V22490_329_57325, &P22490_328};	/*Property = Video*/
const Value V22490_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_330 = {S47, 0, &V22490_330_57328, &P22490_329};	/*Property = Viewport*/
const Value V22490_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_332 = {S48, 0, &V22490_332_57340, &P22490_330};	/*Property = WebWorkers*/
const Value V22490_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22490_333 = {S49, 0, &V22490_333_57343, &P22490_332};	/*Property = Xhr2*/
const Profile P22490 = {"22490", &P22490_333};	/*Profile ID - 22490*/
const Value V22240_1_1 = {S2, NULL};	/*Value = True*/
const Property P22240_1 = {S50, 0, &V22240_1_1, NULL};	/*Property = IsMobile*/
const Value V22240_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22240_5 = {S52, 0, &V22240_5_1277, &P22240_1};	/*Property = ScreenPixelsHeight*/
const Value V22240_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22240_4 = {S54, 0, &V22240_4_2336, &P22240_5};	/*Property = ScreenPixelsWidth*/
const Profile P22240 = {"22240", &P22240_4};	/*Profile ID - 22240*/
const Value V23603_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_279 = {S1, 0, &V23603_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23603_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_282 = {S3, 0, &V23603_282_57155, &P23603_279};	/*Property = BlobBuilder*/
const Value V23603_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_350 = {S4, 0, &V23603_350_57446, &P23603_282};	/*Property = Canvas*/
const Value V23603_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_283 = {S5, 0, &V23603_283_57161, &P23603_350};	/*Property = CssBackground*/
const Value V23603_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_284 = {S6, 0, &V23603_284_57164, &P23603_283};	/*Property = CssBorderImage*/
const Value V23603_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_285 = {S7, 0, &V23603_285_57167, &P23603_284};	/*Property = CssCanvas*/
const Value V23603_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_286 = {S8, 0, &V23603_286_57170, &P23603_285};	/*Property = CssColor*/
const Value V23603_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_237 = {S9, 0, &V23603_237_55786, &P23603_286};	/*Property = CssColumn*/
const Value V23603_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_287 = {S10, 0, &V23603_287_57173, &P23603_237};	/*Property = CssFlexbox*/
const Value V23603_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_288 = {S11, 0, &V23603_288_57176, &P23603_287};	/*Property = CssFont*/
const Value V23603_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_289 = {S12, 0, &V23603_289_86899, &P23603_288};	/*Property = CssImages*/
const Value V23603_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_291 = {S13, 0, &V23603_291_57182, &P23603_289};	/*Property = CssMediaQueries*/
const Value V23603_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_292 = {S14, 0, &V23603_292_57185, &P23603_291};	/*Property = CssMinMax*/
const Value V23603_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_293 = {S15, 0, &V23603_293_57188, &P23603_292};	/*Property = CssOverflow*/
const Value V23603_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_294 = {S16, 0, &V23603_294_57191, &P23603_293};	/*Property = CssPosition*/
const Value V23603_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_295 = {S17, 0, &V23603_295_57194, &P23603_294};	/*Property = CssText*/
const Value V23603_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_297 = {S18, 0, &V23603_297_57197, &P23603_295};	/*Property = CssTransforms*/
const Value V23603_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_241 = {S19, 0, &V23603_241_55801, &P23603_297};	/*Property = CssTransitions*/
const Value V23603_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_298 = {S20, 0, &V23603_298_57208, &P23603_241};	/*Property = CssUI*/
const Value V23603_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_300 = {S21, 0, &V23603_300_57217, &P23603_298};	/*Property = DataSet*/
const Value V23603_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_301 = {S22, 0, &V23603_301_57220, &P23603_300};	/*Property = DataUrl*/
const Value V23603_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_302 = {S23, 0, &V23603_302_57223, &P23603_301};	/*Property = DeviceOrientation*/
const Value V23603_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_304 = {S24, 0, &V23603_304_57229, &P23603_302};	/*Property = FileReader*/
const Value V23603_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_305 = {S25, 0, &V23603_305_57232, &P23603_304};	/*Property = FileSaver*/
const Value V23603_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_306 = {S26, 0, &V23603_306_57238, &P23603_305};	/*Property = FileWriter*/
const Value V23603_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_307 = {S27, 0, &V23603_307_57241, &P23603_306};	/*Property = FormData*/
const Value V23603_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_309 = {S28, 0, &V23603_309_57247, &P23603_307};	/*Property = Fullscreen*/
const Value V23603_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_310 = {S29, 0, &V23603_310_57250, &P23603_309};	/*Property = GeoLocation*/
const Value V23603_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_312 = {S30, 0, &V23603_312_57256, &P23603_310};	/*Property = History*/
const Value V23603_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_314 = {S31, 0, &V23603_314_57262, &P23603_312};	/*Property = Html5*/
const Value V23603_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_313 = {S32, 0, &V23603_313_57259, &P23603_314};	/*Property = Html-Media-Capture*/
const Value V23603_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_315 = {S33, 0, &V23603_315_57265, &P23603_313};	/*Property = Iframe*/
const Value V23603_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_316 = {S34, 0, &V23603_316_57268, &P23603_315};	/*Property = IndexedDB*/
const Value V23603_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_317 = {S35, 0, &V23603_317_57271, &P23603_316};	/*Property = Json*/
const Value V23603_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P23603_15 = {S37, 0, &V23603_15_54167, &P23603_317};	/*Property = LayoutEngine*/
const Value V23603_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_344 = {S38, 0, &V23603_344_57274, &P23603_15};	/*Property = Masking*/
const Value V23603_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_320 = {S39, 0, &V23603_320_57292, &P23603_344};	/*Property = PostMessage*/
const Value V23603_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_321 = {S40, 0, &V23603_321_57295, &P23603_320};	/*Property = Progress*/
const Value V23603_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_322 = {S41, 0, &V23603_322_57298, &P23603_321};	/*Property = Prompts*/
const Value V23603_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_323 = {S42, 0, &V23603_323_57301, &P23603_322};	/*Property = Selector*/
const Value V23603_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_324 = {S43, 0, &V23603_324_57307, &P23603_323};	/*Property = Svg*/
const Value V23603_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_327 = {S44, 0, &V23603_327_57316, &P23603_324};	/*Property = TouchEvents*/
const Value V23603_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_328 = {S45, 0, &V23603_328_57319, &P23603_327};	/*Property = Track*/
const Value V23603_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_329 = {S46, 0, &V23603_329_57325, &P23603_328};	/*Property = Video*/
const Value V23603_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_330 = {S47, 0, &V23603_330_57328, &P23603_329};	/*Property = Viewport*/
const Value V23603_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_332 = {S48, 0, &V23603_332_57340, &P23603_330};	/*Property = WebWorkers*/
const Value V23603_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23603_333 = {S49, 0, &V23603_333_57343, &P23603_332};	/*Property = Xhr2*/
const Profile P23603 = {"23603", &P23603_333};	/*Profile ID - 23603*/
const Value V21424_1_1 = {S2, NULL};	/*Value = True*/
const Property P21424_1 = {S50, 0, &V21424_1_1, NULL};	/*Property = IsMobile*/
const Value V21424_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21424_5 = {S52, 0, &V21424_5_1227, &P21424_1};	/*Property = ScreenPixelsHeight*/
const Value V21424_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21424_4 = {S54, 0, &V21424_4_1229, &P21424_5};	/*Property = ScreenPixelsWidth*/
const Profile P21424 = {"21424", &P21424_4};	/*Profile ID - 21424*/
const Value V21331_1_1 = {S2, NULL};	/*Value = True*/
const Property P21331_1 = {S50, 0, &V21331_1_1, NULL};	/*Property = IsMobile*/
const Value V21331_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21331_5 = {S52, 0, &V21331_5_1227, &P21331_1};	/*Property = ScreenPixelsHeight*/
const Value V21331_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21331_4 = {S54, 0, &V21331_4_1460, &P21331_5};	/*Property = ScreenPixelsWidth*/
const Profile P21331 = {"21331", &P21331_4};	/*Profile ID - 21331*/
const Value V22467_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_279 = {S1, 0, &V22467_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22467_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_282 = {S3, 0, &V22467_282_57155, &P22467_279};	/*Property = BlobBuilder*/
const Value V22467_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_350 = {S4, 0, &V22467_350_57446, &P22467_282};	/*Property = Canvas*/
const Value V22467_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_283 = {S5, 0, &V22467_283_57161, &P22467_350};	/*Property = CssBackground*/
const Value V22467_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_284 = {S6, 0, &V22467_284_57164, &P22467_283};	/*Property = CssBorderImage*/
const Value V22467_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_285 = {S7, 0, &V22467_285_57167, &P22467_284};	/*Property = CssCanvas*/
const Value V22467_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_286 = {S8, 0, &V22467_286_57170, &P22467_285};	/*Property = CssColor*/
const Value V22467_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_237 = {S9, 0, &V22467_237_55786, &P22467_286};	/*Property = CssColumn*/
const Value V22467_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_287 = {S10, 0, &V22467_287_57173, &P22467_237};	/*Property = CssFlexbox*/
const Value V22467_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_288 = {S11, 0, &V22467_288_57176, &P22467_287};	/*Property = CssFont*/
const Value V22467_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_289 = {S12, 0, &V22467_289_86899, &P22467_288};	/*Property = CssImages*/
const Value V22467_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_291 = {S13, 0, &V22467_291_57182, &P22467_289};	/*Property = CssMediaQueries*/
const Value V22467_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_292 = {S14, 0, &V22467_292_57185, &P22467_291};	/*Property = CssMinMax*/
const Value V22467_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_293 = {S15, 0, &V22467_293_57188, &P22467_292};	/*Property = CssOverflow*/
const Value V22467_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_294 = {S16, 0, &V22467_294_57191, &P22467_293};	/*Property = CssPosition*/
const Value V22467_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_295 = {S17, 0, &V22467_295_57194, &P22467_294};	/*Property = CssText*/
const Value V22467_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_297 = {S18, 0, &V22467_297_57197, &P22467_295};	/*Property = CssTransforms*/
const Value V22467_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_241 = {S19, 0, &V22467_241_55801, &P22467_297};	/*Property = CssTransitions*/
const Value V22467_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_298 = {S20, 0, &V22467_298_57208, &P22467_241};	/*Property = CssUI*/
const Value V22467_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_300 = {S21, 0, &V22467_300_57217, &P22467_298};	/*Property = DataSet*/
const Value V22467_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_301 = {S22, 0, &V22467_301_57220, &P22467_300};	/*Property = DataUrl*/
const Value V22467_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_302 = {S23, 0, &V22467_302_57223, &P22467_301};	/*Property = DeviceOrientation*/
const Value V22467_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_304 = {S24, 0, &V22467_304_57229, &P22467_302};	/*Property = FileReader*/
const Value V22467_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_305 = {S25, 0, &V22467_305_57232, &P22467_304};	/*Property = FileSaver*/
const Value V22467_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_306 = {S26, 0, &V22467_306_57238, &P22467_305};	/*Property = FileWriter*/
const Value V22467_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_307 = {S27, 0, &V22467_307_57241, &P22467_306};	/*Property = FormData*/
const Value V22467_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_309 = {S28, 0, &V22467_309_57247, &P22467_307};	/*Property = Fullscreen*/
const Value V22467_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_310 = {S29, 0, &V22467_310_57250, &P22467_309};	/*Property = GeoLocation*/
const Value V22467_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_312 = {S30, 0, &V22467_312_57256, &P22467_310};	/*Property = History*/
const Value V22467_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_314 = {S31, 0, &V22467_314_57262, &P22467_312};	/*Property = Html5*/
const Value V22467_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_313 = {S32, 0, &V22467_313_57259, &P22467_314};	/*Property = Html-Media-Capture*/
const Value V22467_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_315 = {S33, 0, &V22467_315_57265, &P22467_313};	/*Property = Iframe*/
const Value V22467_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_316 = {S34, 0, &V22467_316_57268, &P22467_315};	/*Property = IndexedDB*/
const Value V22467_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_317 = {S35, 0, &V22467_317_57271, &P22467_316};	/*Property = Json*/
const Value V22467_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22467_15 = {S37, 0, &V22467_15_28, &P22467_317};	/*Property = LayoutEngine*/
const Value V22467_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_344 = {S38, 0, &V22467_344_57274, &P22467_15};	/*Property = Masking*/
const Value V22467_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_320 = {S39, 0, &V22467_320_57292, &P22467_344};	/*Property = PostMessage*/
const Value V22467_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_321 = {S40, 0, &V22467_321_57295, &P22467_320};	/*Property = Progress*/
const Value V22467_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_322 = {S41, 0, &V22467_322_57298, &P22467_321};	/*Property = Prompts*/
const Value V22467_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_323 = {S42, 0, &V22467_323_57301, &P22467_322};	/*Property = Selector*/
const Value V22467_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_324 = {S43, 0, &V22467_324_57307, &P22467_323};	/*Property = Svg*/
const Value V22467_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_327 = {S44, 0, &V22467_327_57316, &P22467_324};	/*Property = TouchEvents*/
const Value V22467_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_328 = {S45, 0, &V22467_328_57319, &P22467_327};	/*Property = Track*/
const Value V22467_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_329 = {S46, 0, &V22467_329_57325, &P22467_328};	/*Property = Video*/
const Value V22467_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_330 = {S47, 0, &V22467_330_57328, &P22467_329};	/*Property = Viewport*/
const Value V22467_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_332 = {S48, 0, &V22467_332_57340, &P22467_330};	/*Property = WebWorkers*/
const Value V22467_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22467_333 = {S49, 0, &V22467_333_57343, &P22467_332};	/*Property = Xhr2*/
const Profile P22467 = {"22467", &P22467_333};	/*Profile ID - 22467*/
const Value V22315_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_279 = {S1, 0, &V22315_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22315_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_282 = {S3, 0, &V22315_282_57155, &P22315_279};	/*Property = BlobBuilder*/
const Value V22315_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_350 = {S4, 0, &V22315_350_57446, &P22315_282};	/*Property = Canvas*/
const Value V22315_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_283 = {S5, 0, &V22315_283_57161, &P22315_350};	/*Property = CssBackground*/
const Value V22315_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_284 = {S6, 0, &V22315_284_57164, &P22315_283};	/*Property = CssBorderImage*/
const Value V22315_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_285 = {S7, 0, &V22315_285_57167, &P22315_284};	/*Property = CssCanvas*/
const Value V22315_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_286 = {S8, 0, &V22315_286_57170, &P22315_285};	/*Property = CssColor*/
const Value V22315_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_237 = {S9, 0, &V22315_237_55786, &P22315_286};	/*Property = CssColumn*/
const Value V22315_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_287 = {S10, 0, &V22315_287_57173, &P22315_237};	/*Property = CssFlexbox*/
const Value V22315_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_288 = {S11, 0, &V22315_288_57176, &P22315_287};	/*Property = CssFont*/
const Value V22315_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_289 = {S12, 0, &V22315_289_86899, &P22315_288};	/*Property = CssImages*/
const Value V22315_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_291 = {S13, 0, &V22315_291_57182, &P22315_289};	/*Property = CssMediaQueries*/
const Value V22315_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_292 = {S14, 0, &V22315_292_57185, &P22315_291};	/*Property = CssMinMax*/
const Value V22315_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_293 = {S15, 0, &V22315_293_57188, &P22315_292};	/*Property = CssOverflow*/
const Value V22315_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_294 = {S16, 0, &V22315_294_57191, &P22315_293};	/*Property = CssPosition*/
const Value V22315_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_295 = {S17, 0, &V22315_295_57194, &P22315_294};	/*Property = CssText*/
const Value V22315_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_297 = {S18, 0, &V22315_297_57197, &P22315_295};	/*Property = CssTransforms*/
const Value V22315_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_241 = {S19, 0, &V22315_241_55801, &P22315_297};	/*Property = CssTransitions*/
const Value V22315_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_298 = {S20, 0, &V22315_298_57208, &P22315_241};	/*Property = CssUI*/
const Value V22315_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_300 = {S21, 0, &V22315_300_57217, &P22315_298};	/*Property = DataSet*/
const Value V22315_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_301 = {S22, 0, &V22315_301_57220, &P22315_300};	/*Property = DataUrl*/
const Value V22315_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_302 = {S23, 0, &V22315_302_57223, &P22315_301};	/*Property = DeviceOrientation*/
const Value V22315_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_304 = {S24, 0, &V22315_304_57229, &P22315_302};	/*Property = FileReader*/
const Value V22315_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_305 = {S25, 0, &V22315_305_57232, &P22315_304};	/*Property = FileSaver*/
const Value V22315_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_306 = {S26, 0, &V22315_306_57238, &P22315_305};	/*Property = FileWriter*/
const Value V22315_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_307 = {S27, 0, &V22315_307_57241, &P22315_306};	/*Property = FormData*/
const Value V22315_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_309 = {S28, 0, &V22315_309_57247, &P22315_307};	/*Property = Fullscreen*/
const Value V22315_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_310 = {S29, 0, &V22315_310_57250, &P22315_309};	/*Property = GeoLocation*/
const Value V22315_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_312 = {S30, 0, &V22315_312_57256, &P22315_310};	/*Property = History*/
const Value V22315_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_314 = {S31, 0, &V22315_314_57262, &P22315_312};	/*Property = Html5*/
const Value V22315_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_313 = {S32, 0, &V22315_313_57259, &P22315_314};	/*Property = Html-Media-Capture*/
const Value V22315_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_315 = {S33, 0, &V22315_315_57265, &P22315_313};	/*Property = Iframe*/
const Value V22315_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_316 = {S34, 0, &V22315_316_57268, &P22315_315};	/*Property = IndexedDB*/
const Value V22315_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_317 = {S35, 0, &V22315_317_57271, &P22315_316};	/*Property = Json*/
const Value V22315_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22315_15 = {S37, 0, &V22315_15_29, &P22315_317};	/*Property = LayoutEngine*/
const Value V22315_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_344 = {S38, 0, &V22315_344_57274, &P22315_15};	/*Property = Masking*/
const Value V22315_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_320 = {S39, 0, &V22315_320_57292, &P22315_344};	/*Property = PostMessage*/
const Value V22315_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_321 = {S40, 0, &V22315_321_57295, &P22315_320};	/*Property = Progress*/
const Value V22315_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_322 = {S41, 0, &V22315_322_57298, &P22315_321};	/*Property = Prompts*/
const Value V22315_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_323 = {S42, 0, &V22315_323_57301, &P22315_322};	/*Property = Selector*/
const Value V22315_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_324 = {S43, 0, &V22315_324_57307, &P22315_323};	/*Property = Svg*/
const Value V22315_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_327 = {S44, 0, &V22315_327_57316, &P22315_324};	/*Property = TouchEvents*/
const Value V22315_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_328 = {S45, 0, &V22315_328_57319, &P22315_327};	/*Property = Track*/
const Value V22315_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_329 = {S46, 0, &V22315_329_57325, &P22315_328};	/*Property = Video*/
const Value V22315_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_330 = {S47, 0, &V22315_330_57328, &P22315_329};	/*Property = Viewport*/
const Value V22315_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_332 = {S48, 0, &V22315_332_57340, &P22315_330};	/*Property = WebWorkers*/
const Value V22315_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22315_333 = {S49, 0, &V22315_333_57343, &P22315_332};	/*Property = Xhr2*/
const Profile P22315 = {"22315", &P22315_333};	/*Profile ID - 22315*/
const Value V21073_1_1 = {S2, NULL};	/*Value = True*/
const Property P21073_1 = {S50, 0, &V21073_1_1, NULL};	/*Property = IsMobile*/
const Value V21073_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P21073_5 = {S52, 0, &V21073_5_37569, &P21073_1};	/*Property = ScreenPixelsHeight*/
const Value V21073_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21073_4 = {S54, 0, &V21073_4_5645, &P21073_5};	/*Property = ScreenPixelsWidth*/
const Profile P21073 = {"21073", &P21073_4};	/*Profile ID - 21073*/
const Value V21450_1_1 = {S2, NULL};	/*Value = True*/
const Property P21450_1 = {S50, 0, &V21450_1_1, NULL};	/*Property = IsMobile*/
const Value V21450_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21450_5 = {S52, 0, &V21450_5_1227, &P21450_1};	/*Property = ScreenPixelsHeight*/
const Value V21450_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21450_4 = {S54, 0, &V21450_4_1229, &P21450_5};	/*Property = ScreenPixelsWidth*/
const Profile P21450 = {"21450", &P21450_4};	/*Profile ID - 21450*/
const Value V21219_1_1 = {S2, NULL};	/*Value = True*/
const Property P21219_1 = {S50, 0, &V21219_1_1, NULL};	/*Property = IsMobile*/
const Value V21219_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21219_5 = {S52, 0, &V21219_5_37, &P21219_1};	/*Property = ScreenPixelsHeight*/
const Value V21219_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21219_4 = {S54, 0, &V21219_4_36, &P21219_5};	/*Property = ScreenPixelsWidth*/
const Profile P21219 = {"21219", &P21219_4};	/*Profile ID - 21219*/
const Value V24006_1_1 = {S2, NULL};	/*Value = True*/
const Property P24006_1 = {S50, 0, &V24006_1_1, NULL};	/*Property = IsMobile*/
const Value V24006_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P24006_5 = {S52, 0, &V24006_5_1832, &P24006_1};	/*Property = ScreenPixelsHeight*/
const Value V24006_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P24006_4 = {S54, 0, &V24006_4_5645, &P24006_5};	/*Property = ScreenPixelsWidth*/
const Profile P24006 = {"24006", &P24006_4};	/*Profile ID - 24006*/
const Value V21572_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_279 = {S1, 0, &V21572_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21572_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_282 = {S3, 0, &V21572_282_57155, &P21572_279};	/*Property = BlobBuilder*/
const Value V21572_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_350 = {S4, 0, &V21572_350_57446, &P21572_282};	/*Property = Canvas*/
const Value V21572_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_283 = {S5, 0, &V21572_283_57161, &P21572_350};	/*Property = CssBackground*/
const Value V21572_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_284 = {S6, 0, &V21572_284_57164, &P21572_283};	/*Property = CssBorderImage*/
const Value V21572_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_285 = {S7, 0, &V21572_285_57167, &P21572_284};	/*Property = CssCanvas*/
const Value V21572_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_286 = {S8, 0, &V21572_286_57170, &P21572_285};	/*Property = CssColor*/
const Value V21572_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_237 = {S9, 0, &V21572_237_55786, &P21572_286};	/*Property = CssColumn*/
const Value V21572_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_287 = {S10, 0, &V21572_287_57173, &P21572_237};	/*Property = CssFlexbox*/
const Value V21572_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_288 = {S11, 0, &V21572_288_57176, &P21572_287};	/*Property = CssFont*/
const Value V21572_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_289 = {S12, 0, &V21572_289_86899, &P21572_288};	/*Property = CssImages*/
const Value V21572_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_291 = {S13, 0, &V21572_291_57182, &P21572_289};	/*Property = CssMediaQueries*/
const Value V21572_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_292 = {S14, 0, &V21572_292_57185, &P21572_291};	/*Property = CssMinMax*/
const Value V21572_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_293 = {S15, 0, &V21572_293_57188, &P21572_292};	/*Property = CssOverflow*/
const Value V21572_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_294 = {S16, 0, &V21572_294_57191, &P21572_293};	/*Property = CssPosition*/
const Value V21572_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_295 = {S17, 0, &V21572_295_57194, &P21572_294};	/*Property = CssText*/
const Value V21572_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_297 = {S18, 0, &V21572_297_57197, &P21572_295};	/*Property = CssTransforms*/
const Value V21572_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_241 = {S19, 0, &V21572_241_55801, &P21572_297};	/*Property = CssTransitions*/
const Value V21572_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_298 = {S20, 0, &V21572_298_57208, &P21572_241};	/*Property = CssUI*/
const Value V21572_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_300 = {S21, 0, &V21572_300_57217, &P21572_298};	/*Property = DataSet*/
const Value V21572_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_301 = {S22, 0, &V21572_301_57220, &P21572_300};	/*Property = DataUrl*/
const Value V21572_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_302 = {S23, 0, &V21572_302_57223, &P21572_301};	/*Property = DeviceOrientation*/
const Value V21572_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_304 = {S24, 0, &V21572_304_57229, &P21572_302};	/*Property = FileReader*/
const Value V21572_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_305 = {S25, 0, &V21572_305_57232, &P21572_304};	/*Property = FileSaver*/
const Value V21572_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_306 = {S26, 0, &V21572_306_57238, &P21572_305};	/*Property = FileWriter*/
const Value V21572_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_307 = {S27, 0, &V21572_307_57241, &P21572_306};	/*Property = FormData*/
const Value V21572_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_309 = {S28, 0, &V21572_309_57247, &P21572_307};	/*Property = Fullscreen*/
const Value V21572_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_310 = {S29, 0, &V21572_310_57250, &P21572_309};	/*Property = GeoLocation*/
const Value V21572_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_312 = {S30, 0, &V21572_312_57256, &P21572_310};	/*Property = History*/
const Value V21572_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_314 = {S31, 0, &V21572_314_57262, &P21572_312};	/*Property = Html5*/
const Value V21572_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_313 = {S32, 0, &V21572_313_57259, &P21572_314};	/*Property = Html-Media-Capture*/
const Value V21572_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_315 = {S33, 0, &V21572_315_57265, &P21572_313};	/*Property = Iframe*/
const Value V21572_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_316 = {S34, 0, &V21572_316_57268, &P21572_315};	/*Property = IndexedDB*/
const Value V21572_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_317 = {S35, 0, &V21572_317_57271, &P21572_316};	/*Property = Json*/
const Value V21572_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21572_15 = {S37, 0, &V21572_15_28, &P21572_317};	/*Property = LayoutEngine*/
const Value V21572_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_344 = {S38, 0, &V21572_344_57274, &P21572_15};	/*Property = Masking*/
const Value V21572_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_320 = {S39, 0, &V21572_320_57292, &P21572_344};	/*Property = PostMessage*/
const Value V21572_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_321 = {S40, 0, &V21572_321_57295, &P21572_320};	/*Property = Progress*/
const Value V21572_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_322 = {S41, 0, &V21572_322_57298, &P21572_321};	/*Property = Prompts*/
const Value V21572_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_323 = {S42, 0, &V21572_323_57301, &P21572_322};	/*Property = Selector*/
const Value V21572_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_324 = {S43, 0, &V21572_324_57307, &P21572_323};	/*Property = Svg*/
const Value V21572_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_327 = {S44, 0, &V21572_327_57316, &P21572_324};	/*Property = TouchEvents*/
const Value V21572_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_328 = {S45, 0, &V21572_328_57319, &P21572_327};	/*Property = Track*/
const Value V21572_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_329 = {S46, 0, &V21572_329_57325, &P21572_328};	/*Property = Video*/
const Value V21572_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_330 = {S47, 0, &V21572_330_57328, &P21572_329};	/*Property = Viewport*/
const Value V21572_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_332 = {S48, 0, &V21572_332_57340, &P21572_330};	/*Property = WebWorkers*/
const Value V21572_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21572_333 = {S49, 0, &V21572_333_57343, &P21572_332};	/*Property = Xhr2*/
const Profile P21572 = {"21572", &P21572_333};	/*Profile ID - 21572*/
const Value V21197_1_1 = {S2, NULL};	/*Value = True*/
const Property P21197_1 = {S50, 0, &V21197_1_1, NULL};	/*Property = IsMobile*/
const Value V21197_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21197_5 = {S52, 0, &V21197_5_1222, &P21197_1};	/*Property = ScreenPixelsHeight*/
const Value V21197_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21197_4 = {S54, 0, &V21197_4_1224, &P21197_5};	/*Property = ScreenPixelsWidth*/
const Profile P21197 = {"21197", &P21197_4};	/*Profile ID - 21197*/
const Value V18435_1_1 = {S2, NULL};	/*Value = True*/
const Property P18435_1 = {S50, 0, &V18435_1_1, NULL};	/*Property = IsMobile*/
const Value V18435_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P18435_5 = {S52, 0, &V18435_5_37, &P18435_1};	/*Property = ScreenPixelsHeight*/
const Value V18435_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P18435_4 = {S54, 0, &V18435_4_36, &P18435_5};	/*Property = ScreenPixelsWidth*/
const Profile P18435 = {"18435", &P18435_4};	/*Profile ID - 18435*/
const Profile P22217 = {"22217", NULL};	/*Profile ID - 22217*/
const Value V20215_1_1 = {S2, NULL};	/*Value = True*/
const Property P20215_1 = {S50, 0, &V20215_1_1, NULL};	/*Property = IsMobile*/
const Value V20215_5_37452 = {S86, NULL};	/*Value = 48*/
const Property P20215_5 = {S52, 0, &V20215_5_37452, &P20215_1};	/*Property = ScreenPixelsHeight*/
const Value V20215_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20215_4 = {S54, 0, &V20215_4_1279, &P20215_5};	/*Property = ScreenPixelsWidth*/
const Profile P20215 = {"20215", &P20215_4};	/*Profile ID - 20215*/
const Profile P22218 = {"22218", NULL};	/*Profile ID - 22218*/
const Value V20011_1_1 = {S2, NULL};	/*Value = True*/
const Property P20011_1 = {S50, 0, &V20011_1_1, NULL};	/*Property = IsMobile*/
const Value V20011_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20011_5 = {S52, 0, &V20011_5_1227, &P20011_1};	/*Property = ScreenPixelsHeight*/
const Value V20011_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20011_4 = {S54, 0, &V20011_4_1229, &P20011_5};	/*Property = ScreenPixelsWidth*/
const Profile P20011 = {"20011", &P20011_4};	/*Profile ID - 20011*/
const Value V18495_1_1 = {S2, NULL};	/*Value = True*/
const Property P18495_1 = {S50, 0, &V18495_1_1, NULL};	/*Property = IsMobile*/
const Value V18495_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18495_5 = {S52, 0, &V18495_5_1222, &P18495_1};	/*Property = ScreenPixelsHeight*/
const Value V18495_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18495_4 = {S54, 0, &V18495_4_1224, &P18495_5};	/*Property = ScreenPixelsWidth*/
const Profile P18495 = {"18495", &P18495_4};	/*Profile ID - 18495*/
const Value V21370_1_1 = {S2, NULL};	/*Value = True*/
const Property P21370_1 = {S50, 0, &V21370_1_1, NULL};	/*Property = IsMobile*/
const Value V21370_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21370_5 = {S52, 0, &V21370_5_9981, &P21370_1};	/*Property = ScreenPixelsHeight*/
const Value V21370_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21370_4 = {S54, 0, &V21370_4_2336, &P21370_5};	/*Property = ScreenPixelsWidth*/
const Profile P21370 = {"21370", &P21370_4};	/*Profile ID - 21370*/
const Value V22438_1_1 = {S2, NULL};	/*Value = True*/
const Property P22438_1 = {S50, 0, &V22438_1_1, NULL};	/*Property = IsMobile*/
const Value V22438_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22438_5 = {S52, 0, &V22438_5_1832, &P22438_1};	/*Property = ScreenPixelsHeight*/
const Value V22438_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22438_4 = {S54, 0, &V22438_4_5645, &P22438_5};	/*Property = ScreenPixelsWidth*/
const Profile P22438 = {"22438", &P22438_4};	/*Profile ID - 22438*/
const Profile P22215 = {"22215", NULL};	/*Profile ID - 22215*/
const Value V20095_1_1 = {S2, NULL};	/*Value = True*/
const Property P20095_1 = {S50, 0, &V20095_1_1, NULL};	/*Property = IsMobile*/
const Value V20095_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20095_5 = {S52, 0, &V20095_5_1403, &P20095_1};	/*Property = ScreenPixelsHeight*/
const Value V20095_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20095_4 = {S54, 0, &V20095_4_1404, &P20095_5};	/*Property = ScreenPixelsWidth*/
const Profile P20095 = {"20095", &P20095_4};	/*Profile ID - 20095*/
const Value V21678_1_1 = {S2, NULL};	/*Value = True*/
const Property P21678_1 = {S50, 0, &V21678_1_1, NULL};	/*Property = IsMobile*/
const Value V21678_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21678_5 = {S52, 0, &V21678_5_1227, &P21678_1};	/*Property = ScreenPixelsHeight*/
const Value V21678_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21678_4 = {S54, 0, &V21678_4_1229, &P21678_5};	/*Property = ScreenPixelsWidth*/
const Profile P21678 = {"21678", &P21678_4};	/*Profile ID - 21678*/
const Value V22563_1_1 = {S2, NULL};	/*Value = True*/
const Property P22563_1 = {S50, 0, &V22563_1_1, NULL};	/*Property = IsMobile*/
const Value V22563_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22563_5 = {S52, 0, &V22563_5_1832, &P22563_1};	/*Property = ScreenPixelsHeight*/
const Value V22563_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22563_4 = {S54, 0, &V22563_4_5645, &P22563_5};	/*Property = ScreenPixelsWidth*/
const Profile P22563 = {"22563", &P22563_4};	/*Profile ID - 22563*/
const Value V23900_1_1 = {S2, NULL};	/*Value = True*/
const Property P23900_1 = {S50, 0, &V23900_1_1, NULL};	/*Property = IsMobile*/
const Value V23900_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23900_5 = {S52, 0, &V23900_5_37, &P23900_1};	/*Property = ScreenPixelsHeight*/
const Value V23900_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23900_4 = {S54, 0, &V23900_4_36, &P23900_5};	/*Property = ScreenPixelsWidth*/
const Profile P23900 = {"23900", &P23900_4};	/*Profile ID - 23900*/
const Value V23823_1_1 = {S2, NULL};	/*Value = True*/
const Property P23823_1 = {S50, 0, &V23823_1_1, NULL};	/*Property = IsMobile*/
const Value V23823_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23823_5 = {S52, 0, &V23823_5_1227, &P23823_1};	/*Property = ScreenPixelsHeight*/
const Value V23823_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P23823_4 = {S54, 0, &V23823_4_1477, &P23823_5};	/*Property = ScreenPixelsWidth*/
const Profile P23823 = {"23823", &P23823_4};	/*Profile ID - 23823*/
const Value V23730_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_279 = {S1, 0, &V23730_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23730_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_282 = {S3, 0, &V23730_282_57155, &P23730_279};	/*Property = BlobBuilder*/
const Value V23730_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_350 = {S4, 0, &V23730_350_57446, &P23730_282};	/*Property = Canvas*/
const Value V23730_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_283 = {S5, 0, &V23730_283_57161, &P23730_350};	/*Property = CssBackground*/
const Value V23730_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_284 = {S6, 0, &V23730_284_57164, &P23730_283};	/*Property = CssBorderImage*/
const Value V23730_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_285 = {S7, 0, &V23730_285_57167, &P23730_284};	/*Property = CssCanvas*/
const Value V23730_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_286 = {S8, 0, &V23730_286_57170, &P23730_285};	/*Property = CssColor*/
const Value V23730_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_237 = {S9, 0, &V23730_237_55786, &P23730_286};	/*Property = CssColumn*/
const Value V23730_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_287 = {S10, 0, &V23730_287_57173, &P23730_237};	/*Property = CssFlexbox*/
const Value V23730_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_288 = {S11, 0, &V23730_288_57176, &P23730_287};	/*Property = CssFont*/
const Value V23730_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_289 = {S12, 0, &V23730_289_86899, &P23730_288};	/*Property = CssImages*/
const Value V23730_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_291 = {S13, 0, &V23730_291_57182, &P23730_289};	/*Property = CssMediaQueries*/
const Value V23730_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_292 = {S14, 0, &V23730_292_57185, &P23730_291};	/*Property = CssMinMax*/
const Value V23730_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_293 = {S15, 0, &V23730_293_57188, &P23730_292};	/*Property = CssOverflow*/
const Value V23730_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_294 = {S16, 0, &V23730_294_57191, &P23730_293};	/*Property = CssPosition*/
const Value V23730_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_295 = {S17, 0, &V23730_295_57194, &P23730_294};	/*Property = CssText*/
const Value V23730_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_297 = {S18, 0, &V23730_297_57197, &P23730_295};	/*Property = CssTransforms*/
const Value V23730_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_241 = {S19, 0, &V23730_241_55801, &P23730_297};	/*Property = CssTransitions*/
const Value V23730_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_298 = {S20, 0, &V23730_298_57208, &P23730_241};	/*Property = CssUI*/
const Value V23730_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_300 = {S21, 0, &V23730_300_57217, &P23730_298};	/*Property = DataSet*/
const Value V23730_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_301 = {S22, 0, &V23730_301_57220, &P23730_300};	/*Property = DataUrl*/
const Value V23730_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_302 = {S23, 0, &V23730_302_57223, &P23730_301};	/*Property = DeviceOrientation*/
const Value V23730_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_304 = {S24, 0, &V23730_304_57229, &P23730_302};	/*Property = FileReader*/
const Value V23730_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_305 = {S25, 0, &V23730_305_57232, &P23730_304};	/*Property = FileSaver*/
const Value V23730_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_306 = {S26, 0, &V23730_306_57238, &P23730_305};	/*Property = FileWriter*/
const Value V23730_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_307 = {S27, 0, &V23730_307_57241, &P23730_306};	/*Property = FormData*/
const Value V23730_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_309 = {S28, 0, &V23730_309_57247, &P23730_307};	/*Property = Fullscreen*/
const Value V23730_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_310 = {S29, 0, &V23730_310_57250, &P23730_309};	/*Property = GeoLocation*/
const Value V23730_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_312 = {S30, 0, &V23730_312_57256, &P23730_310};	/*Property = History*/
const Value V23730_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_314 = {S31, 0, &V23730_314_57262, &P23730_312};	/*Property = Html5*/
const Value V23730_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_313 = {S32, 0, &V23730_313_57259, &P23730_314};	/*Property = Html-Media-Capture*/
const Value V23730_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_315 = {S33, 0, &V23730_315_57265, &P23730_313};	/*Property = Iframe*/
const Value V23730_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_316 = {S34, 0, &V23730_316_57268, &P23730_315};	/*Property = IndexedDB*/
const Value V23730_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_317 = {S35, 0, &V23730_317_57271, &P23730_316};	/*Property = Json*/
const Value V23730_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P23730_15 = {S37, 0, &V23730_15_28, &P23730_317};	/*Property = LayoutEngine*/
const Value V23730_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_344 = {S38, 0, &V23730_344_57274, &P23730_15};	/*Property = Masking*/
const Value V23730_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_320 = {S39, 0, &V23730_320_57292, &P23730_344};	/*Property = PostMessage*/
const Value V23730_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_321 = {S40, 0, &V23730_321_57295, &P23730_320};	/*Property = Progress*/
const Value V23730_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_322 = {S41, 0, &V23730_322_57298, &P23730_321};	/*Property = Prompts*/
const Value V23730_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_323 = {S42, 0, &V23730_323_57301, &P23730_322};	/*Property = Selector*/
const Value V23730_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_324 = {S43, 0, &V23730_324_57307, &P23730_323};	/*Property = Svg*/
const Value V23730_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_327 = {S44, 0, &V23730_327_57316, &P23730_324};	/*Property = TouchEvents*/
const Value V23730_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_328 = {S45, 0, &V23730_328_57319, &P23730_327};	/*Property = Track*/
const Value V23730_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_329 = {S46, 0, &V23730_329_57325, &P23730_328};	/*Property = Video*/
const Value V23730_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_330 = {S47, 0, &V23730_330_57328, &P23730_329};	/*Property = Viewport*/
const Value V23730_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_332 = {S48, 0, &V23730_332_57340, &P23730_330};	/*Property = WebWorkers*/
const Value V23730_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23730_333 = {S49, 0, &V23730_333_57343, &P23730_332};	/*Property = Xhr2*/
const Profile P23730 = {"23730", &P23730_333};	/*Profile ID - 23730*/
const Value V22733_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22733_279 = {S1, 0, &V22733_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22733_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22733_282 = {S3, 0, &V22733_282_57154, &P22733_279};	/*Property = BlobBuilder*/
const Value V22733_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22733_350 = {S4, 0, &V22733_350_57444, &P22733_282};	/*Property = Canvas*/
const Value V22733_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22733_283 = {S5, 0, &V22733_283_57159, &P22733_350};	/*Property = CssBackground*/
const Value V22733_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22733_284 = {S6, 0, &V22733_284_57163, &P22733_283};	/*Property = CssBorderImage*/
const Value V22733_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22733_285 = {S7, 0, &V22733_285_57166, &P22733_284};	/*Property = CssCanvas*/
const Value V22733_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22733_286 = {S8, 0, &V22733_286_57168, &P22733_285};	/*Property = CssColor*/
const Value V22733_237_55785 = {S0, NULL};	/*Value = False*/
const Property P22733_237 = {S9, 0, &V22733_237_55785, &P22733_286};	/*Property = CssColumn*/
const Value V22733_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22733_287 = {S10, 0, &V22733_287_57172, &P22733_237};	/*Property = CssFlexbox*/
const Value V22733_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22733_288 = {S11, 0, &V22733_288_57174, &P22733_287};	/*Property = CssFont*/
const Value V22733_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22733_289 = {S12, 0, &V22733_289_57759, &P22733_288};	/*Property = CssImages*/
const Value V22733_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22733_291 = {S13, 0, &V22733_291_57181, &P22733_289};	/*Property = CssMediaQueries*/
const Value V22733_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22733_292 = {S14, 0, &V22733_292_57183, &P22733_291};	/*Property = CssMinMax*/
const Value V22733_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22733_293 = {S15, 0, &V22733_293_57187, &P22733_292};	/*Property = CssOverflow*/
const Value V22733_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22733_294 = {S16, 0, &V22733_294_57189, &P22733_293};	/*Property = CssPosition*/
const Value V22733_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22733_295 = {S17, 0, &V22733_295_57192, &P22733_294};	/*Property = CssText*/
const Value V22733_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22733_297 = {S18, 0, &V22733_297_57195, &P22733_295};	/*Property = CssTransforms*/
const Value V22733_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22733_241 = {S19, 0, &V22733_241_55799, &P22733_297};	/*Property = CssTransitions*/
const Value V22733_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22733_298 = {S20, 0, &V22733_298_57206, &P22733_241};	/*Property = CssUI*/
const Value V22733_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22733_300 = {S21, 0, &V22733_300_57216, &P22733_298};	/*Property = DataSet*/
const Value V22733_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22733_301 = {S22, 0, &V22733_301_57218, &P22733_300};	/*Property = DataUrl*/
const Value V22733_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22733_302 = {S23, 0, &V22733_302_57222, &P22733_301};	/*Property = DeviceOrientation*/
const Value V22733_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22733_304 = {S24, 0, &V22733_304_57227, &P22733_302};	/*Property = FileReader*/
const Value V22733_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22733_305 = {S25, 0, &V22733_305_57231, &P22733_304};	/*Property = FileSaver*/
const Value V22733_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22733_306 = {S26, 0, &V22733_306_57237, &P22733_305};	/*Property = FileWriter*/
const Value V22733_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22733_307 = {S27, 0, &V22733_307_57239, &P22733_306};	/*Property = FormData*/
const Value V22733_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22733_309 = {S28, 0, &V22733_309_57246, &P22733_307};	/*Property = Fullscreen*/
const Value V22733_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22733_310 = {S29, 0, &V22733_310_57248, &P22733_309};	/*Property = GeoLocation*/
const Value V22733_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22733_312 = {S30, 0, &V22733_312_57254, &P22733_310};	/*Property = History*/
const Value V22733_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22733_314 = {S31, 0, &V22733_314_57261, &P22733_312};	/*Property = Html5*/
const Value V22733_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22733_313 = {S32, 0, &V22733_313_57258, &P22733_314};	/*Property = Html-Media-Capture*/
const Value V22733_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22733_315 = {S33, 0, &V22733_315_57264, &P22733_313};	/*Property = Iframe*/
const Value V22733_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22733_316 = {S34, 0, &V22733_316_57266, &P22733_315};	/*Property = IndexedDB*/
const Value V22733_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22733_317 = {S35, 0, &V22733_317_57269, &P22733_316};	/*Property = Json*/
const Value V22733_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P22733_15 = {S37, 0, &V22733_15_30, &P22733_317};	/*Property = LayoutEngine*/
const Value V22733_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22733_344 = {S38, 0, &V22733_344_57272, &P22733_15};	/*Property = Masking*/
const Value V22733_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22733_320 = {S39, 0, &V22733_320_57290, &P22733_344};	/*Property = PostMessage*/
const Value V22733_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22733_321 = {S40, 0, &V22733_321_57293, &P22733_320};	/*Property = Progress*/
const Value V22733_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22733_322 = {S41, 0, &V22733_322_57296, &P22733_321};	/*Property = Prompts*/
const Value V22733_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22733_323 = {S42, 0, &V22733_323_57299, &P22733_322};	/*Property = Selector*/
const Value V22733_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22733_324 = {S43, 0, &V22733_324_57306, &P22733_323};	/*Property = Svg*/
const Value V22733_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22733_327 = {S44, 0, &V22733_327_57315, &P22733_324};	/*Property = TouchEvents*/
const Value V22733_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22733_328 = {S45, 0, &V22733_328_57318, &P22733_327};	/*Property = Track*/
const Value V22733_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22733_329 = {S46, 0, &V22733_329_57323, &P22733_328};	/*Property = Video*/
const Value V22733_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22733_330 = {S47, 0, &V22733_330_57326, &P22733_329};	/*Property = Viewport*/
const Value V22733_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22733_332 = {S48, 0, &V22733_332_57339, &P22733_330};	/*Property = WebWorkers*/
const Value V22733_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22733_333 = {S49, 0, &V22733_333_57342, &P22733_332};	/*Property = Xhr2*/
const Profile P22733 = {"22733", &P22733_333};	/*Profile ID - 22733*/
const Value V23996_1_1 = {S2, NULL};	/*Value = True*/
const Property P23996_1 = {S50, 0, &V23996_1_1, NULL};	/*Property = IsMobile*/
const Value V23996_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23996_5 = {S52, 0, &V23996_5_1277, &P23996_1};	/*Property = ScreenPixelsHeight*/
const Value V23996_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23996_4 = {S54, 0, &V23996_4_1279, &P23996_5};	/*Property = ScreenPixelsWidth*/
const Profile P23996 = {"23996", &P23996_4};	/*Profile ID - 23996*/
const Value V15139_1_1 = {S2, NULL};	/*Value = True*/
const Property P15139_1 = {S50, 0, &V15139_1_1, NULL};	/*Property = IsMobile*/
const Value V15139_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P15139_5 = {S52, 0, &V15139_5_1227, &P15139_1};	/*Property = ScreenPixelsHeight*/
const Value V15139_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15139_4 = {S54, 0, &V15139_4_1229, &P15139_5};	/*Property = ScreenPixelsWidth*/
const Profile P15139 = {"15139", &P15139_4};	/*Profile ID - 15139*/
const Value V21113_1_1 = {S2, NULL};	/*Value = True*/
const Property P21113_1 = {S50, 0, &V21113_1_1, NULL};	/*Property = IsMobile*/
const Value V21113_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21113_5 = {S52, 0, &V21113_5_1277, &P21113_1};	/*Property = ScreenPixelsHeight*/
const Value V21113_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21113_4 = {S54, 0, &V21113_4_1279, &P21113_5};	/*Property = ScreenPixelsWidth*/
const Profile P21113 = {"21113", &P21113_4};	/*Profile ID - 21113*/
const Value V20142_1_1 = {S2, NULL};	/*Value = True*/
const Property P20142_1 = {S50, 0, &V20142_1_1, NULL};	/*Property = IsMobile*/
const Value V20142_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20142_5 = {S52, 0, &V20142_5_9981, &P20142_1};	/*Property = ScreenPixelsHeight*/
const Value V20142_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20142_4 = {S54, 0, &V20142_4_39098, &P20142_5};	/*Property = ScreenPixelsWidth*/
const Profile P20142 = {"20142", &P20142_4};	/*Profile ID - 20142*/
const Value V21210_1_1 = {S2, NULL};	/*Value = True*/
const Property P21210_1 = {S50, 0, &V21210_1_1, NULL};	/*Property = IsMobile*/
const Value V21210_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21210_5 = {S52, 0, &V21210_5_1227, &P21210_1};	/*Property = ScreenPixelsHeight*/
const Value V21210_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21210_4 = {S54, 0, &V21210_4_1460, &P21210_5};	/*Property = ScreenPixelsWidth*/
const Profile P21210 = {"21210", &P21210_4};	/*Profile ID - 21210*/
const Value V18476_1_1 = {S2, NULL};	/*Value = True*/
const Property P18476_1 = {S50, 0, &V18476_1_1, NULL};	/*Property = IsMobile*/
const Value V18476_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18476_5 = {S52, 0, &V18476_5_1227, &P18476_1};	/*Property = ScreenPixelsHeight*/
const Value V18476_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18476_4 = {S54, 0, &V18476_4_1229, &P18476_5};	/*Property = ScreenPixelsWidth*/
const Profile P18476 = {"18476", &P18476_4};	/*Profile ID - 18476*/
const Value V21846_1_1 = {S2, NULL};	/*Value = True*/
const Property P21846_1 = {S50, 0, &V21846_1_1, NULL};	/*Property = IsMobile*/
const Value V21846_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21846_5 = {S52, 0, &V21846_5_1227, &P21846_1};	/*Property = ScreenPixelsHeight*/
const Value V21846_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21846_4 = {S54, 0, &V21846_4_1229, &P21846_5};	/*Property = ScreenPixelsWidth*/
const Profile P21846 = {"21846", &P21846_4};	/*Profile ID - 21846*/
const Value V19873_1_1 = {S2, NULL};	/*Value = True*/
const Property P19873_1 = {S50, 0, &V19873_1_1, NULL};	/*Property = IsMobile*/
const Value V19873_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P19873_5 = {S52, 0, &V19873_5_1277, &P19873_1};	/*Property = ScreenPixelsHeight*/
const Value V19873_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P19873_4 = {S54, 0, &V19873_4_1279, &P19873_5};	/*Property = ScreenPixelsWidth*/
const Profile P19873 = {"19873", &P19873_4};	/*Profile ID - 19873*/
const Value V22694_1_1 = {S2, NULL};	/*Value = True*/
const Property P22694_1 = {S50, 0, &V22694_1_1, NULL};	/*Property = IsMobile*/
const Value V22694_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22694_5 = {S52, 0, &V22694_5_1222, &P22694_1};	/*Property = ScreenPixelsHeight*/
const Value V22694_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22694_4 = {S54, 0, &V22694_4_1224, &P22694_5};	/*Property = ScreenPixelsWidth*/
const Profile P22694 = {"22694", &P22694_4};	/*Profile ID - 22694*/
const Value V22123_1_1 = {S2, NULL};	/*Value = True*/
const Property P22123_1 = {S50, 0, &V22123_1_1, NULL};	/*Property = IsMobile*/
const Value V22123_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22123_5 = {S52, 0, &V22123_5_37569, &P22123_1};	/*Property = ScreenPixelsHeight*/
const Value V22123_4_53077 = {S129, NULL};	/*Value = 1366*/
const Property P22123_4 = {S54, 0, &V22123_4_53077, &P22123_5};	/*Property = ScreenPixelsWidth*/
const Profile P22123 = {"22123", &P22123_4};	/*Profile ID - 22123*/
const Value V18669_1_1 = {S2, NULL};	/*Value = True*/
const Property P18669_1 = {S50, 0, &V18669_1_1, NULL};	/*Property = IsMobile*/
const Value V18669_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P18669_5 = {S52, 0, &V18669_5_1380, &P18669_1};	/*Property = ScreenPixelsHeight*/
const Value V18669_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18669_4 = {S54, 0, &V18669_4_1279, &P18669_5};	/*Property = ScreenPixelsWidth*/
const Profile P18669 = {"18669", &P18669_4};	/*Profile ID - 18669*/
const Value V21147_1_1 = {S2, NULL};	/*Value = True*/
const Property P21147_1 = {S50, 0, &V21147_1_1, NULL};	/*Property = IsMobile*/
const Value V21147_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21147_5 = {S52, 0, &V21147_5_1458, &P21147_1};	/*Property = ScreenPixelsHeight*/
const Value V21147_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21147_4 = {S54, 0, &V21147_4_37617, &P21147_5};	/*Property = ScreenPixelsWidth*/
const Profile P21147 = {"21147", &P21147_4};	/*Profile ID - 21147*/
const Value V21841_1_1 = {S2, NULL};	/*Value = True*/
const Property P21841_1 = {S50, 0, &V21841_1_1, NULL};	/*Property = IsMobile*/
const Value V21841_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21841_5 = {S52, 0, &V21841_5_1458, &P21841_1};	/*Property = ScreenPixelsHeight*/
const Value V21841_4_80836 = {S130, NULL};	/*Value = 700*/
const Property P21841_4 = {S54, 0, &V21841_4_80836, &P21841_5};	/*Property = ScreenPixelsWidth*/
const Profile P21841 = {"21841", &P21841_4};	/*Profile ID - 21841*/
const Value V21184_1_1 = {S2, NULL};	/*Value = True*/
const Property P21184_1 = {S50, 0, &V21184_1_1, NULL};	/*Property = IsMobile*/
const Value V21184_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21184_5 = {S52, 0, &V21184_5_1227, &P21184_1};	/*Property = ScreenPixelsHeight*/
const Value V21184_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21184_4 = {S54, 0, &V21184_4_1229, &P21184_5};	/*Property = ScreenPixelsWidth*/
const Profile P21184 = {"21184", &P21184_4};	/*Profile ID - 21184*/
const Value V20796_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_279 = {S1, 0, &V20796_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20796_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_282 = {S3, 0, &V20796_282_57155, &P20796_279};	/*Property = BlobBuilder*/
const Value V20796_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_350 = {S4, 0, &V20796_350_57446, &P20796_282};	/*Property = Canvas*/
const Value V20796_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_283 = {S5, 0, &V20796_283_57161, &P20796_350};	/*Property = CssBackground*/
const Value V20796_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_284 = {S6, 0, &V20796_284_57164, &P20796_283};	/*Property = CssBorderImage*/
const Value V20796_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_285 = {S7, 0, &V20796_285_57167, &P20796_284};	/*Property = CssCanvas*/
const Value V20796_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_286 = {S8, 0, &V20796_286_57170, &P20796_285};	/*Property = CssColor*/
const Value V20796_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_237 = {S9, 0, &V20796_237_55786, &P20796_286};	/*Property = CssColumn*/
const Value V20796_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_287 = {S10, 0, &V20796_287_57173, &P20796_237};	/*Property = CssFlexbox*/
const Value V20796_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_288 = {S11, 0, &V20796_288_57176, &P20796_287};	/*Property = CssFont*/
const Value V20796_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_289 = {S12, 0, &V20796_289_86899, &P20796_288};	/*Property = CssImages*/
const Value V20796_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_291 = {S13, 0, &V20796_291_57182, &P20796_289};	/*Property = CssMediaQueries*/
const Value V20796_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_292 = {S14, 0, &V20796_292_57185, &P20796_291};	/*Property = CssMinMax*/
const Value V20796_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_293 = {S15, 0, &V20796_293_57188, &P20796_292};	/*Property = CssOverflow*/
const Value V20796_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_294 = {S16, 0, &V20796_294_57191, &P20796_293};	/*Property = CssPosition*/
const Value V20796_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_295 = {S17, 0, &V20796_295_57194, &P20796_294};	/*Property = CssText*/
const Value V20796_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_297 = {S18, 0, &V20796_297_57197, &P20796_295};	/*Property = CssTransforms*/
const Value V20796_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_241 = {S19, 0, &V20796_241_55801, &P20796_297};	/*Property = CssTransitions*/
const Value V20796_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_298 = {S20, 0, &V20796_298_57208, &P20796_241};	/*Property = CssUI*/
const Value V20796_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_300 = {S21, 0, &V20796_300_57217, &P20796_298};	/*Property = DataSet*/
const Value V20796_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_301 = {S22, 0, &V20796_301_57220, &P20796_300};	/*Property = DataUrl*/
const Value V20796_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_302 = {S23, 0, &V20796_302_57223, &P20796_301};	/*Property = DeviceOrientation*/
const Value V20796_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_304 = {S24, 0, &V20796_304_57229, &P20796_302};	/*Property = FileReader*/
const Value V20796_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_305 = {S25, 0, &V20796_305_57232, &P20796_304};	/*Property = FileSaver*/
const Value V20796_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_306 = {S26, 0, &V20796_306_57238, &P20796_305};	/*Property = FileWriter*/
const Value V20796_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_307 = {S27, 0, &V20796_307_57241, &P20796_306};	/*Property = FormData*/
const Value V20796_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_309 = {S28, 0, &V20796_309_57247, &P20796_307};	/*Property = Fullscreen*/
const Value V20796_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_310 = {S29, 0, &V20796_310_57250, &P20796_309};	/*Property = GeoLocation*/
const Value V20796_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_312 = {S30, 0, &V20796_312_57256, &P20796_310};	/*Property = History*/
const Value V20796_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_314 = {S31, 0, &V20796_314_57262, &P20796_312};	/*Property = Html5*/
const Value V20796_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_313 = {S32, 0, &V20796_313_57259, &P20796_314};	/*Property = Html-Media-Capture*/
const Value V20796_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_315 = {S33, 0, &V20796_315_57265, &P20796_313};	/*Property = Iframe*/
const Value V20796_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_316 = {S34, 0, &V20796_316_57268, &P20796_315};	/*Property = IndexedDB*/
const Value V20796_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_317 = {S35, 0, &V20796_317_57271, &P20796_316};	/*Property = Json*/
const Value V20796_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20796_15 = {S37, 0, &V20796_15_29, &P20796_317};	/*Property = LayoutEngine*/
const Value V20796_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_344 = {S38, 0, &V20796_344_57274, &P20796_15};	/*Property = Masking*/
const Value V20796_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_320 = {S39, 0, &V20796_320_57292, &P20796_344};	/*Property = PostMessage*/
const Value V20796_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_321 = {S40, 0, &V20796_321_57295, &P20796_320};	/*Property = Progress*/
const Value V20796_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_322 = {S41, 0, &V20796_322_57298, &P20796_321};	/*Property = Prompts*/
const Value V20796_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_323 = {S42, 0, &V20796_323_57301, &P20796_322};	/*Property = Selector*/
const Value V20796_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_324 = {S43, 0, &V20796_324_57307, &P20796_323};	/*Property = Svg*/
const Value V20796_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_327 = {S44, 0, &V20796_327_57316, &P20796_324};	/*Property = TouchEvents*/
const Value V20796_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_328 = {S45, 0, &V20796_328_57319, &P20796_327};	/*Property = Track*/
const Value V20796_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_329 = {S46, 0, &V20796_329_57325, &P20796_328};	/*Property = Video*/
const Value V20796_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_330 = {S47, 0, &V20796_330_57328, &P20796_329};	/*Property = Viewport*/
const Value V20796_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_332 = {S48, 0, &V20796_332_57340, &P20796_330};	/*Property = WebWorkers*/
const Value V20796_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20796_333 = {S49, 0, &V20796_333_57343, &P20796_332};	/*Property = Xhr2*/
const Profile P20796 = {"20796", &P20796_333};	/*Profile ID - 20796*/
const Value V19980_1_1 = {S2, NULL};	/*Value = True*/
const Property P19980_1 = {S50, 0, &V19980_1_1, NULL};	/*Property = IsMobile*/
const Value V19980_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19980_5 = {S52, 0, &V19980_5_1227, &P19980_1};	/*Property = ScreenPixelsHeight*/
const Value V19980_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19980_4 = {S54, 0, &V19980_4_1229, &P19980_5};	/*Property = ScreenPixelsWidth*/
const Profile P19980 = {"19980", &P19980_4};	/*Profile ID - 19980*/
const Value V21410_1_1 = {S2, NULL};	/*Value = True*/
const Property P21410_1 = {S50, 0, &V21410_1_1, NULL};	/*Property = IsMobile*/
const Value V21410_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21410_5 = {S52, 0, &V21410_5_1227, &P21410_1};	/*Property = ScreenPixelsHeight*/
const Value V21410_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21410_4 = {S54, 0, &V21410_4_1229, &P21410_5};	/*Property = ScreenPixelsWidth*/
const Profile P21410 = {"21410", &P21410_4};	/*Profile ID - 21410*/
const Value V23808_1_1 = {S2, NULL};	/*Value = True*/
const Property P23808_1 = {S50, 0, &V23808_1_1, NULL};	/*Property = IsMobile*/
const Value V23808_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23808_5 = {S52, 0, &V23808_5_1458, &P23808_1};	/*Property = ScreenPixelsHeight*/
const Value V23808_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23808_4 = {S54, 0, &V23808_4_1460, &P23808_5};	/*Property = ScreenPixelsWidth*/
const Profile P23808 = {"23808", &P23808_4};	/*Profile ID - 23808*/
const Value V21438_1_1 = {S2, NULL};	/*Value = True*/
const Property P21438_1 = {S50, 0, &V21438_1_1, NULL};	/*Property = IsMobile*/
const Value V21438_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21438_5 = {S52, 0, &V21438_5_1227, &P21438_1};	/*Property = ScreenPixelsHeight*/
const Value V21438_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21438_4 = {S54, 0, &V21438_4_1229, &P21438_5};	/*Property = ScreenPixelsWidth*/
const Profile P21438 = {"21438", &P21438_4};	/*Profile ID - 21438*/
const Value V16287_1_1 = {S2, NULL};	/*Value = True*/
const Property P16287_1 = {S50, 0, &V16287_1_1, NULL};	/*Property = IsMobile*/
const Value V16287_5_37458 = {S131, NULL};	/*Value = 60*/
const Property P16287_5 = {S52, 0, &V16287_5_37458, &P16287_1};	/*Property = ScreenPixelsHeight*/
const Value V16287_4_31956 = {S92, NULL};	/*Value = 96*/
const Property P16287_4 = {S54, 0, &V16287_4_31956, &P16287_5};	/*Property = ScreenPixelsWidth*/
const Profile P16287 = {"16287", &P16287_4};	/*Profile ID - 16287*/
const Value V21012_1_1 = {S2, NULL};	/*Value = True*/
const Property P21012_1 = {S50, 0, &V21012_1_1, NULL};	/*Property = IsMobile*/
const Value V21012_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21012_5 = {S52, 0, &V21012_5_9981, &P21012_1};	/*Property = ScreenPixelsHeight*/
const Value V21012_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21012_4 = {S54, 0, &V21012_4_2336, &P21012_5};	/*Property = ScreenPixelsWidth*/
const Profile P21012 = {"21012", &P21012_4};	/*Profile ID - 21012*/
const Value V18734_1_1 = {S2, NULL};	/*Value = True*/
const Property P18734_1 = {S50, 0, &V18734_1_1, NULL};	/*Property = IsMobile*/
const Value V18734_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P18734_5 = {S52, 0, &V18734_5_1458, &P18734_1};	/*Property = ScreenPixelsHeight*/
const Value V18734_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P18734_4 = {S54, 0, &V18734_4_1460, &P18734_5};	/*Property = ScreenPixelsWidth*/
const Profile P18734 = {"18734", &P18734_4};	/*Profile ID - 18734*/
const Profile P24181 = {"24181", NULL};	/*Profile ID - 24181*/
const Value V24417_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24417_279 = {S1, 0, &V24417_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24417_282_57153 = {S2, NULL};	/*Value = True*/
const Property P24417_282 = {S3, 0, &V24417_282_57153, &P24417_279};	/*Property = BlobBuilder*/
const Value V24417_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24417_350 = {S4, 0, &V24417_350_57444, &P24417_282};	/*Property = Canvas*/
const Value V24417_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24417_283 = {S5, 0, &V24417_283_57159, &P24417_350};	/*Property = CssBackground*/
const Value V24417_284_57163 = {S0, NULL};	/*Value = False*/
const Property P24417_284 = {S6, 0, &V24417_284_57163, &P24417_283};	/*Property = CssBorderImage*/
const Value V24417_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24417_285 = {S7, 0, &V24417_285_57166, &P24417_284};	/*Property = CssCanvas*/
const Value V24417_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24417_286 = {S8, 0, &V24417_286_57168, &P24417_285};	/*Property = CssColor*/
const Value V24417_237_55784 = {S2, NULL};	/*Value = True*/
const Property P24417_237 = {S9, 0, &V24417_237_55784, &P24417_286};	/*Property = CssColumn*/
const Value V24417_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24417_287 = {S10, 0, &V24417_287_57172, &P24417_237};	/*Property = CssFlexbox*/
const Value V24417_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24417_288 = {S11, 0, &V24417_288_57174, &P24417_287};	/*Property = CssFont*/
const Value V24417_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24417_289 = {S12, 0, &V24417_289_57759, &P24417_288};	/*Property = CssImages*/
const Value V24417_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24417_291 = {S13, 0, &V24417_291_57180, &P24417_289};	/*Property = CssMediaQueries*/
const Value V24417_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24417_292 = {S14, 0, &V24417_292_57183, &P24417_291};	/*Property = CssMinMax*/
const Value V24417_293_57186 = {S2, NULL};	/*Value = True*/
const Property P24417_293 = {S15, 0, &V24417_293_57186, &P24417_292};	/*Property = CssOverflow*/
const Value V24417_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24417_294 = {S16, 0, &V24417_294_57189, &P24417_293};	/*Property = CssPosition*/
const Value V24417_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24417_295 = {S17, 0, &V24417_295_57192, &P24417_294};	/*Property = CssText*/
const Value V24417_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24417_297 = {S18, 0, &V24417_297_57195, &P24417_295};	/*Property = CssTransforms*/
const Value V24417_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24417_241 = {S19, 0, &V24417_241_55799, &P24417_297};	/*Property = CssTransitions*/
const Value V24417_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24417_298 = {S20, 0, &V24417_298_57206, &P24417_241};	/*Property = CssUI*/
const Value V24417_300_57216 = {S0, NULL};	/*Value = False*/
const Property P24417_300 = {S21, 0, &V24417_300_57216, &P24417_298};	/*Property = DataSet*/
const Value V24417_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24417_301 = {S22, 0, &V24417_301_57218, &P24417_300};	/*Property = DataUrl*/
const Value V24417_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24417_302 = {S23, 0, &V24417_302_57221, &P24417_301};	/*Property = DeviceOrientation*/
const Value V24417_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24417_304 = {S24, 0, &V24417_304_57227, &P24417_302};	/*Property = FileReader*/
const Value V24417_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24417_305 = {S25, 0, &V24417_305_57231, &P24417_304};	/*Property = FileSaver*/
const Value V24417_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24417_306 = {S26, 0, &V24417_306_57237, &P24417_305};	/*Property = FileWriter*/
const Value V24417_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24417_307 = {S27, 0, &V24417_307_57239, &P24417_306};	/*Property = FormData*/
const Value V24417_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24417_309 = {S28, 0, &V24417_309_57246, &P24417_307};	/*Property = Fullscreen*/
const Value V24417_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24417_310 = {S29, 0, &V24417_310_57248, &P24417_309};	/*Property = GeoLocation*/
const Value V24417_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24417_312 = {S30, 0, &V24417_312_57254, &P24417_310};	/*Property = History*/
const Value V24417_314_57261 = {S0, NULL};	/*Value = False*/
const Property P24417_314 = {S31, 0, &V24417_314_57261, &P24417_312};	/*Property = Html5*/
const Value V24417_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24417_313 = {S32, 0, &V24417_313_57258, &P24417_314};	/*Property = Html-Media-Capture*/
const Value V24417_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24417_315 = {S33, 0, &V24417_315_57264, &P24417_313};	/*Property = Iframe*/
const Value V24417_316_57267 = {S0, NULL};	/*Value = False*/
const Property P24417_316 = {S34, 0, &V24417_316_57267, &P24417_315};	/*Property = IndexedDB*/
const Value V24417_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24417_317 = {S35, 0, &V24417_317_57269, &P24417_316};	/*Property = Json*/
const Value V24417_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24417_15 = {S37, 0, &V24417_15_28, &P24417_317};	/*Property = LayoutEngine*/
const Value V24417_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24417_344 = {S38, 0, &V24417_344_57272, &P24417_15};	/*Property = Masking*/
const Value V24417_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24417_320 = {S39, 0, &V24417_320_57290, &P24417_344};	/*Property = PostMessage*/
const Value V24417_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24417_321 = {S40, 0, &V24417_321_57293, &P24417_320};	/*Property = Progress*/
const Value V24417_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24417_322 = {S41, 0, &V24417_322_57296, &P24417_321};	/*Property = Prompts*/
const Value V24417_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24417_323 = {S42, 0, &V24417_323_57299, &P24417_322};	/*Property = Selector*/
const Value V24417_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24417_324 = {S43, 0, &V24417_324_57306, &P24417_323};	/*Property = Svg*/
const Value V24417_327_57315 = {S0, NULL};	/*Value = False*/
const Property P24417_327 = {S44, 0, &V24417_327_57315, &P24417_324};	/*Property = TouchEvents*/
const Value V24417_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24417_328 = {S45, 0, &V24417_328_57318, &P24417_327};	/*Property = Track*/
const Value V24417_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24417_329 = {S46, 0, &V24417_329_57323, &P24417_328};	/*Property = Video*/
const Value V24417_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24417_330 = {S47, 0, &V24417_330_57326, &P24417_329};	/*Property = Viewport*/
const Value V24417_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24417_332 = {S48, 0, &V24417_332_57338, &P24417_330};	/*Property = WebWorkers*/
const Value V24417_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24417_333 = {S49, 0, &V24417_333_57342, &P24417_332};	/*Property = Xhr2*/
const Profile P24417 = {"24417", &P24417_333};	/*Profile ID - 24417*/
const Value V16452_1_1 = {S2, NULL};	/*Value = True*/
const Property P16452_1 = {S50, 0, &V16452_1_1, NULL};	/*Property = IsMobile*/
const Value V16452_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16452_5 = {S52, 0, &V16452_5_1277, &P16452_1};	/*Property = ScreenPixelsHeight*/
const Value V16452_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16452_4 = {S54, 0, &V16452_4_1279, &P16452_5};	/*Property = ScreenPixelsWidth*/
const Profile P16452 = {"16452", &P16452_4};	/*Profile ID - 16452*/
const Value V21080_1_1 = {S2, NULL};	/*Value = True*/
const Property P21080_1 = {S50, 0, &V21080_1_1, NULL};	/*Property = IsMobile*/
const Value V21080_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21080_5 = {S52, 0, &V21080_5_1227, &P21080_1};	/*Property = ScreenPixelsHeight*/
const Value V21080_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21080_4 = {S54, 0, &V21080_4_1229, &P21080_5};	/*Property = ScreenPixelsWidth*/
const Profile P21080 = {"21080", &P21080_4};	/*Profile ID - 21080*/
const Value V21414_1_1 = {S2, NULL};	/*Value = True*/
const Property P21414_1 = {S50, 0, &V21414_1_1, NULL};	/*Property = IsMobile*/
const Value V21414_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21414_5 = {S52, 0, &V21414_5_1227, &P21414_1};	/*Property = ScreenPixelsHeight*/
const Value V21414_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21414_4 = {S54, 0, &V21414_4_1460, &P21414_5};	/*Property = ScreenPixelsWidth*/
const Profile P21414 = {"21414", &P21414_4};	/*Profile ID - 21414*/
const Value V23611_1_2 = {S0, NULL};	/*Value = False*/
const Property P23611_1 = {S50, 0, &V23611_1_2, NULL};	/*Property = IsMobile*/
const Value V23611_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23611_5 = {S52, 0, &V23611_5_60417, &P23611_1};	/*Property = ScreenPixelsHeight*/
const Value V23611_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23611_4 = {S54, 0, &V23611_4_57095, &P23611_5};	/*Property = ScreenPixelsWidth*/
const Profile P23611 = {"23611", &P23611_4};	/*Profile ID - 23611*/
const Value V22133_1_1 = {S2, NULL};	/*Value = True*/
const Property P22133_1 = {S50, 0, &V22133_1_1, NULL};	/*Property = IsMobile*/
const Value V22133_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22133_5 = {S52, 0, &V22133_5_1403, &P22133_1};	/*Property = ScreenPixelsHeight*/
const Value V22133_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22133_4 = {S54, 0, &V22133_4_1404, &P22133_5};	/*Property = ScreenPixelsWidth*/
const Profile P22133 = {"22133", &P22133_4};	/*Profile ID - 22133*/
const Value V21138_1_1 = {S2, NULL};	/*Value = True*/
const Property P21138_1 = {S50, 0, &V21138_1_1, NULL};	/*Property = IsMobile*/
const Value V21138_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21138_5 = {S52, 0, &V21138_5_9981, &P21138_1};	/*Property = ScreenPixelsHeight*/
const Value V21138_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21138_4 = {S54, 0, &V21138_4_2336, &P21138_5};	/*Property = ScreenPixelsWidth*/
const Profile P21138 = {"21138", &P21138_4};	/*Profile ID - 21138*/
const Value V20365_1_1 = {S2, NULL};	/*Value = True*/
const Property P20365_1 = {S50, 0, &V20365_1_1, NULL};	/*Property = IsMobile*/
const Value V20365_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20365_5 = {S52, 0, &V20365_5_1458, &P20365_1};	/*Property = ScreenPixelsHeight*/
const Value V20365_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20365_4 = {S54, 0, &V20365_4_37617, &P20365_5};	/*Property = ScreenPixelsWidth*/
const Profile P20365 = {"20365", &P20365_4};	/*Profile ID - 20365*/
const Value V21430_1_1 = {S2, NULL};	/*Value = True*/
const Property P21430_1 = {S50, 0, &V21430_1_1, NULL};	/*Property = IsMobile*/
const Value V21430_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21430_5 = {S52, 0, &V21430_5_1277, &P21430_1};	/*Property = ScreenPixelsHeight*/
const Value V21430_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21430_4 = {S54, 0, &V21430_4_1279, &P21430_5};	/*Property = ScreenPixelsWidth*/
const Profile P21430 = {"21430", &P21430_4};	/*Profile ID - 21430*/
const Value V23618_1_2 = {S0, NULL};	/*Value = False*/
const Property P23618_1 = {S50, 0, &V23618_1_2, NULL};	/*Property = IsMobile*/
const Value V23618_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23618_5 = {S52, 0, &V23618_5_60417, &P23618_1};	/*Property = ScreenPixelsHeight*/
const Value V23618_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23618_4 = {S54, 0, &V23618_4_57095, &P23618_5};	/*Property = ScreenPixelsWidth*/
const Profile P23618 = {"23618", &P23618_4};	/*Profile ID - 23618*/
const Value V20935_1_1 = {S2, NULL};	/*Value = True*/
const Property P20935_1 = {S50, 0, &V20935_1_1, NULL};	/*Property = IsMobile*/
const Value V20935_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20935_5 = {S52, 0, &V20935_5_1277, &P20935_1};	/*Property = ScreenPixelsHeight*/
const Value V20935_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20935_4 = {S54, 0, &V20935_4_1279, &P20935_5};	/*Property = ScreenPixelsWidth*/
const Profile P20935 = {"20935", &P20935_4};	/*Profile ID - 20935*/
const Value V23645_1_2 = {S0, NULL};	/*Value = False*/
const Property P23645_1 = {S50, 0, &V23645_1_2, NULL};	/*Property = IsMobile*/
const Value V23645_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23645_5 = {S52, 0, &V23645_5_60417, &P23645_1};	/*Property = ScreenPixelsHeight*/
const Value V23645_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23645_4 = {S54, 0, &V23645_4_57095, &P23645_5};	/*Property = ScreenPixelsWidth*/
const Profile P23645 = {"23645", &P23645_4};	/*Profile ID - 23645*/
const Value V19938_1_1 = {S2, NULL};	/*Value = True*/
const Property P19938_1 = {S50, 0, &V19938_1_1, NULL};	/*Property = IsMobile*/
const Value V19938_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P19938_5 = {S52, 0, &V19938_5_1277, &P19938_1};	/*Property = ScreenPixelsHeight*/
const Value V19938_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P19938_4 = {S54, 0, &V19938_4_1279, &P19938_5};	/*Property = ScreenPixelsWidth*/
const Profile P19938 = {"19938", &P19938_4};	/*Profile ID - 19938*/
const Value V22007_1_1 = {S2, NULL};	/*Value = True*/
const Property P22007_1 = {S50, 0, &V22007_1_1, NULL};	/*Property = IsMobile*/
const Value V22007_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22007_5 = {S52, 0, &V22007_5_1277, &P22007_1};	/*Property = ScreenPixelsHeight*/
const Value V22007_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22007_4 = {S54, 0, &V22007_4_1279, &P22007_5};	/*Property = ScreenPixelsWidth*/
const Profile P22007 = {"22007", &P22007_4};	/*Profile ID - 22007*/
const Value V24002_1_1 = {S2, NULL};	/*Value = True*/
const Property P24002_1 = {S50, 0, &V24002_1_1, NULL};	/*Property = IsMobile*/
const Value V24002_5_37570 = {S132, NULL};	/*Value = 340*/
const Property P24002_5 = {S52, 0, &V24002_5_37570, &P24002_1};	/*Property = ScreenPixelsHeight*/
const Value V24002_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P24002_4 = {S54, 0, &V24002_4_1224, &P24002_5};	/*Property = ScreenPixelsWidth*/
const Profile P24002 = {"24002", &P24002_4};	/*Profile ID - 24002*/
const Value V20206_1_1 = {S2, NULL};	/*Value = True*/
const Property P20206_1 = {S50, 0, &V20206_1_1, NULL};	/*Property = IsMobile*/
const Value V20206_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20206_5 = {S52, 0, &V20206_5_1227, &P20206_1};	/*Property = ScreenPixelsHeight*/
const Value V20206_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20206_4 = {S54, 0, &V20206_4_1229, &P20206_5};	/*Property = ScreenPixelsWidth*/
const Profile P20206 = {"20206", &P20206_4};	/*Profile ID - 20206*/
const Value V20204_1_1 = {S2, NULL};	/*Value = True*/
const Property P20204_1 = {S50, 0, &V20204_1_1, NULL};	/*Property = IsMobile*/
const Value V20204_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P20204_5 = {S52, 0, &V20204_5_1304, &P20204_1};	/*Property = ScreenPixelsHeight*/
const Value V20204_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20204_4 = {S54, 0, &V20204_4_1224, &P20204_5};	/*Property = ScreenPixelsWidth*/
const Profile P20204 = {"20204", &P20204_4};	/*Profile ID - 20204*/
const Value V12266_1_1 = {S2, NULL};	/*Value = True*/
const Property P12266_1 = {S50, 0, &V12266_1_1, NULL};	/*Property = IsMobile*/
const Value V12266_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P12266_5 = {S52, 0, &V12266_5_1293, &P12266_1};	/*Property = ScreenPixelsHeight*/
const Value V12266_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P12266_4 = {S54, 0, &V12266_4_1296, &P12266_5};	/*Property = ScreenPixelsWidth*/
const Profile P12266 = {"12266", &P12266_4};	/*Profile ID - 12266*/
const Value V21497_1_1 = {S2, NULL};	/*Value = True*/
const Property P21497_1 = {S50, 0, &V21497_1_1, NULL};	/*Property = IsMobile*/
const Value V21497_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P21497_5 = {S52, 0, &V21497_5_1275, &P21497_1};	/*Property = ScreenPixelsHeight*/
const Value V21497_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21497_4 = {S54, 0, &V21497_4_1229, &P21497_5};	/*Property = ScreenPixelsWidth*/
const Profile P21497 = {"21497", &P21497_4};	/*Profile ID - 21497*/
const Value V21479_1_1 = {S2, NULL};	/*Value = True*/
const Property P21479_1 = {S50, 0, &V21479_1_1, NULL};	/*Property = IsMobile*/
const Value V21479_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21479_5 = {S52, 0, &V21479_5_1277, &P21479_1};	/*Property = ScreenPixelsHeight*/
const Value V21479_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P21479_4 = {S54, 0, &V21479_4_1943, &P21479_5};	/*Property = ScreenPixelsWidth*/
const Profile P21479 = {"21479", &P21479_4};	/*Profile ID - 21479*/
const Value V23624_1_2 = {S0, NULL};	/*Value = False*/
const Property P23624_1 = {S50, 0, &V23624_1_2, NULL};	/*Property = IsMobile*/
const Value V23624_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23624_5 = {S52, 0, &V23624_5_60417, &P23624_1};	/*Property = ScreenPixelsHeight*/
const Value V23624_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23624_4 = {S54, 0, &V23624_4_57095, &P23624_5};	/*Property = ScreenPixelsWidth*/
const Profile P23624 = {"23624", &P23624_4};	/*Profile ID - 23624*/
const Value V17103_1_1 = {S2, NULL};	/*Value = True*/
const Property P17103_1 = {S50, 0, &V17103_1_1, NULL};	/*Property = IsMobile*/
const Value V17103_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17103_5 = {S52, 0, &V17103_5_1222, &P17103_1};	/*Property = ScreenPixelsHeight*/
const Value V17103_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17103_4 = {S54, 0, &V17103_4_1224, &P17103_5};	/*Property = ScreenPixelsWidth*/
const Profile P17103 = {"17103", &P17103_4};	/*Profile ID - 17103*/
const Value V22251_1_1 = {S2, NULL};	/*Value = True*/
const Property P22251_1 = {S50, 0, &V22251_1_1, NULL};	/*Property = IsMobile*/
const Value V22251_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22251_5 = {S52, 0, &V22251_5_1227, &P22251_1};	/*Property = ScreenPixelsHeight*/
const Value V22251_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22251_4 = {S54, 0, &V22251_4_1460, &P22251_5};	/*Property = ScreenPixelsWidth*/
const Profile P22251 = {"22251", &P22251_4};	/*Profile ID - 22251*/
const Value V23678_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_279 = {S1, 0, &V23678_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23678_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_282 = {S3, 0, &V23678_282_57155, &P23678_279};	/*Property = BlobBuilder*/
const Value V23678_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_350 = {S4, 0, &V23678_350_57446, &P23678_282};	/*Property = Canvas*/
const Value V23678_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_283 = {S5, 0, &V23678_283_57161, &P23678_350};	/*Property = CssBackground*/
const Value V23678_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_284 = {S6, 0, &V23678_284_57164, &P23678_283};	/*Property = CssBorderImage*/
const Value V23678_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_285 = {S7, 0, &V23678_285_57167, &P23678_284};	/*Property = CssCanvas*/
const Value V23678_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_286 = {S8, 0, &V23678_286_57170, &P23678_285};	/*Property = CssColor*/
const Value V23678_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_237 = {S9, 0, &V23678_237_55786, &P23678_286};	/*Property = CssColumn*/
const Value V23678_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_287 = {S10, 0, &V23678_287_57173, &P23678_237};	/*Property = CssFlexbox*/
const Value V23678_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_288 = {S11, 0, &V23678_288_57176, &P23678_287};	/*Property = CssFont*/
const Value V23678_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_289 = {S12, 0, &V23678_289_86899, &P23678_288};	/*Property = CssImages*/
const Value V23678_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_291 = {S13, 0, &V23678_291_57182, &P23678_289};	/*Property = CssMediaQueries*/
const Value V23678_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_292 = {S14, 0, &V23678_292_57185, &P23678_291};	/*Property = CssMinMax*/
const Value V23678_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_293 = {S15, 0, &V23678_293_57188, &P23678_292};	/*Property = CssOverflow*/
const Value V23678_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_294 = {S16, 0, &V23678_294_57191, &P23678_293};	/*Property = CssPosition*/
const Value V23678_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_295 = {S17, 0, &V23678_295_57194, &P23678_294};	/*Property = CssText*/
const Value V23678_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_297 = {S18, 0, &V23678_297_57197, &P23678_295};	/*Property = CssTransforms*/
const Value V23678_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_241 = {S19, 0, &V23678_241_55801, &P23678_297};	/*Property = CssTransitions*/
const Value V23678_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_298 = {S20, 0, &V23678_298_57208, &P23678_241};	/*Property = CssUI*/
const Value V23678_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_300 = {S21, 0, &V23678_300_57217, &P23678_298};	/*Property = DataSet*/
const Value V23678_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_301 = {S22, 0, &V23678_301_57220, &P23678_300};	/*Property = DataUrl*/
const Value V23678_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_302 = {S23, 0, &V23678_302_57223, &P23678_301};	/*Property = DeviceOrientation*/
const Value V23678_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_304 = {S24, 0, &V23678_304_57229, &P23678_302};	/*Property = FileReader*/
const Value V23678_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_305 = {S25, 0, &V23678_305_57232, &P23678_304};	/*Property = FileSaver*/
const Value V23678_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_306 = {S26, 0, &V23678_306_57238, &P23678_305};	/*Property = FileWriter*/
const Value V23678_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_307 = {S27, 0, &V23678_307_57241, &P23678_306};	/*Property = FormData*/
const Value V23678_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_309 = {S28, 0, &V23678_309_57247, &P23678_307};	/*Property = Fullscreen*/
const Value V23678_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_310 = {S29, 0, &V23678_310_57250, &P23678_309};	/*Property = GeoLocation*/
const Value V23678_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_312 = {S30, 0, &V23678_312_57256, &P23678_310};	/*Property = History*/
const Value V23678_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_314 = {S31, 0, &V23678_314_57262, &P23678_312};	/*Property = Html5*/
const Value V23678_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_313 = {S32, 0, &V23678_313_57259, &P23678_314};	/*Property = Html-Media-Capture*/
const Value V23678_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_315 = {S33, 0, &V23678_315_57265, &P23678_313};	/*Property = Iframe*/
const Value V23678_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_316 = {S34, 0, &V23678_316_57268, &P23678_315};	/*Property = IndexedDB*/
const Value V23678_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_317 = {S35, 0, &V23678_317_57271, &P23678_316};	/*Property = Json*/
const Value V23678_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_15 = {S37, 0, &V23678_15_72, &P23678_317};	/*Property = LayoutEngine*/
const Value V23678_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_344 = {S38, 0, &V23678_344_57274, &P23678_15};	/*Property = Masking*/
const Value V23678_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_320 = {S39, 0, &V23678_320_57292, &P23678_344};	/*Property = PostMessage*/
const Value V23678_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_321 = {S40, 0, &V23678_321_57295, &P23678_320};	/*Property = Progress*/
const Value V23678_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_322 = {S41, 0, &V23678_322_57298, &P23678_321};	/*Property = Prompts*/
const Value V23678_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_323 = {S42, 0, &V23678_323_57301, &P23678_322};	/*Property = Selector*/
const Value V23678_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_324 = {S43, 0, &V23678_324_57307, &P23678_323};	/*Property = Svg*/
const Value V23678_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_327 = {S44, 0, &V23678_327_57316, &P23678_324};	/*Property = TouchEvents*/
const Value V23678_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_328 = {S45, 0, &V23678_328_57319, &P23678_327};	/*Property = Track*/
const Value V23678_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_329 = {S46, 0, &V23678_329_57325, &P23678_328};	/*Property = Video*/
const Value V23678_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_330 = {S47, 0, &V23678_330_57328, &P23678_329};	/*Property = Viewport*/
const Value V23678_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_332 = {S48, 0, &V23678_332_57340, &P23678_330};	/*Property = WebWorkers*/
const Value V23678_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23678_333 = {S49, 0, &V23678_333_57343, &P23678_332};	/*Property = Xhr2*/
const Profile P23678 = {"23678", &P23678_333};	/*Profile ID - 23678*/
const Value V20386_1_1 = {S2, NULL};	/*Value = True*/
const Property P20386_1 = {S50, 0, &V20386_1_1, NULL};	/*Property = IsMobile*/
const Value V20386_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P20386_5 = {S52, 0, &V20386_5_1832, &P20386_1};	/*Property = ScreenPixelsHeight*/
const Value V20386_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P20386_4 = {S54, 0, &V20386_4_5645, &P20386_5};	/*Property = ScreenPixelsWidth*/
const Profile P20386 = {"20386", &P20386_4};	/*Profile ID - 20386*/
const Value V23998_1_1 = {S2, NULL};	/*Value = True*/
const Property P23998_1 = {S50, 0, &V23998_1_1, NULL};	/*Property = IsMobile*/
const Value V23998_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23998_5 = {S52, 0, &V23998_5_37, &P23998_1};	/*Property = ScreenPixelsHeight*/
const Value V23998_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23998_4 = {S54, 0, &V23998_4_36, &P23998_5};	/*Property = ScreenPixelsWidth*/
const Profile P23998 = {"23998", &P23998_4};	/*Profile ID - 23998*/
const Value V21380_1_1 = {S2, NULL};	/*Value = True*/
const Property P21380_1 = {S50, 0, &V21380_1_1, NULL};	/*Property = IsMobile*/
const Value V21380_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21380_5 = {S52, 0, &V21380_5_1277, &P21380_1};	/*Property = ScreenPixelsHeight*/
const Value V21380_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21380_4 = {S54, 0, &V21380_4_1279, &P21380_5};	/*Property = ScreenPixelsWidth*/
const Profile P21380 = {"21380", &P21380_4};	/*Profile ID - 21380*/
const Value V20238_1_2 = {S0, NULL};	/*Value = False*/
const Property P20238_1 = {S50, 0, &V20238_1_2, NULL};	/*Property = IsMobile*/
const Value V20238_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20238_5 = {S52, 0, &V20238_5_37, &P20238_1};	/*Property = ScreenPixelsHeight*/
const Value V20238_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20238_4 = {S54, 0, &V20238_4_36, &P20238_5};	/*Property = ScreenPixelsWidth*/
const Profile P20238 = {"20238", &P20238_4};	/*Profile ID - 20238*/
const Value V15879_1_1 = {S2, NULL};	/*Value = True*/
const Property P15879_1 = {S50, 0, &V15879_1_1, NULL};	/*Property = IsMobile*/
const Value V15879_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P15879_5 = {S52, 0, &V15879_5_1269, &P15879_1};	/*Property = ScreenPixelsHeight*/
const Value V15879_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P15879_4 = {S54, 0, &V15879_4_1229, &P15879_5};	/*Property = ScreenPixelsWidth*/
const Profile P15879 = {"15879", &P15879_4};	/*Profile ID - 15879*/
const Value V21589_1_1 = {S2, NULL};	/*Value = True*/
const Property P21589_1 = {S50, 0, &V21589_1_1, NULL};	/*Property = IsMobile*/
const Value V21589_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21589_5 = {S52, 0, &V21589_5_1403, &P21589_1};	/*Property = ScreenPixelsHeight*/
const Value V21589_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21589_4 = {S54, 0, &V21589_4_1404, &P21589_5};	/*Property = ScreenPixelsWidth*/
const Profile P21589 = {"21589", &P21589_4};	/*Profile ID - 21589*/
const Value V20182_1_1 = {S2, NULL};	/*Value = True*/
const Property P20182_1 = {S50, 0, &V20182_1_1, NULL};	/*Property = IsMobile*/
const Value V20182_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20182_5 = {S52, 0, &V20182_5_1458, &P20182_1};	/*Property = ScreenPixelsHeight*/
const Value V20182_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20182_4 = {S54, 0, &V20182_4_1460, &P20182_5};	/*Property = ScreenPixelsWidth*/
const Profile P20182 = {"20182", &P20182_4};	/*Profile ID - 20182*/
const Value V24005_1_1 = {S2, NULL};	/*Value = True*/
const Property P24005_1 = {S50, 0, &V24005_1_1, NULL};	/*Property = IsMobile*/
const Value V24005_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P24005_5 = {S52, 0, &V24005_5_37, &P24005_1};	/*Property = ScreenPixelsHeight*/
const Value V24005_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P24005_4 = {S54, 0, &V24005_4_36, &P24005_5};	/*Property = ScreenPixelsWidth*/
const Profile P24005 = {"24005", &P24005_4};	/*Profile ID - 24005*/
const Value V20766_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_279 = {S1, 0, &V20766_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20766_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_282 = {S3, 0, &V20766_282_57155, &P20766_279};	/*Property = BlobBuilder*/
const Value V20766_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_350 = {S4, 0, &V20766_350_57446, &P20766_282};	/*Property = Canvas*/
const Value V20766_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_283 = {S5, 0, &V20766_283_57161, &P20766_350};	/*Property = CssBackground*/
const Value V20766_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_284 = {S6, 0, &V20766_284_57164, &P20766_283};	/*Property = CssBorderImage*/
const Value V20766_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_285 = {S7, 0, &V20766_285_57167, &P20766_284};	/*Property = CssCanvas*/
const Value V20766_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_286 = {S8, 0, &V20766_286_57170, &P20766_285};	/*Property = CssColor*/
const Value V20766_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_237 = {S9, 0, &V20766_237_55786, &P20766_286};	/*Property = CssColumn*/
const Value V20766_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_287 = {S10, 0, &V20766_287_57173, &P20766_237};	/*Property = CssFlexbox*/
const Value V20766_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_288 = {S11, 0, &V20766_288_57176, &P20766_287};	/*Property = CssFont*/
const Value V20766_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_289 = {S12, 0, &V20766_289_86899, &P20766_288};	/*Property = CssImages*/
const Value V20766_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_291 = {S13, 0, &V20766_291_57182, &P20766_289};	/*Property = CssMediaQueries*/
const Value V20766_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_292 = {S14, 0, &V20766_292_57185, &P20766_291};	/*Property = CssMinMax*/
const Value V20766_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_293 = {S15, 0, &V20766_293_57188, &P20766_292};	/*Property = CssOverflow*/
const Value V20766_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_294 = {S16, 0, &V20766_294_57191, &P20766_293};	/*Property = CssPosition*/
const Value V20766_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_295 = {S17, 0, &V20766_295_57194, &P20766_294};	/*Property = CssText*/
const Value V20766_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_297 = {S18, 0, &V20766_297_57197, &P20766_295};	/*Property = CssTransforms*/
const Value V20766_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_241 = {S19, 0, &V20766_241_55801, &P20766_297};	/*Property = CssTransitions*/
const Value V20766_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_298 = {S20, 0, &V20766_298_57208, &P20766_241};	/*Property = CssUI*/
const Value V20766_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_300 = {S21, 0, &V20766_300_57217, &P20766_298};	/*Property = DataSet*/
const Value V20766_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_301 = {S22, 0, &V20766_301_57220, &P20766_300};	/*Property = DataUrl*/
const Value V20766_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_302 = {S23, 0, &V20766_302_57223, &P20766_301};	/*Property = DeviceOrientation*/
const Value V20766_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_304 = {S24, 0, &V20766_304_57229, &P20766_302};	/*Property = FileReader*/
const Value V20766_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_305 = {S25, 0, &V20766_305_57232, &P20766_304};	/*Property = FileSaver*/
const Value V20766_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_306 = {S26, 0, &V20766_306_57238, &P20766_305};	/*Property = FileWriter*/
const Value V20766_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_307 = {S27, 0, &V20766_307_57241, &P20766_306};	/*Property = FormData*/
const Value V20766_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_309 = {S28, 0, &V20766_309_57247, &P20766_307};	/*Property = Fullscreen*/
const Value V20766_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_310 = {S29, 0, &V20766_310_57250, &P20766_309};	/*Property = GeoLocation*/
const Value V20766_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_312 = {S30, 0, &V20766_312_57256, &P20766_310};	/*Property = History*/
const Value V20766_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_314 = {S31, 0, &V20766_314_57262, &P20766_312};	/*Property = Html5*/
const Value V20766_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_313 = {S32, 0, &V20766_313_57259, &P20766_314};	/*Property = Html-Media-Capture*/
const Value V20766_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_315 = {S33, 0, &V20766_315_57265, &P20766_313};	/*Property = Iframe*/
const Value V20766_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_316 = {S34, 0, &V20766_316_57268, &P20766_315};	/*Property = IndexedDB*/
const Value V20766_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_317 = {S35, 0, &V20766_317_57271, &P20766_316};	/*Property = Json*/
const Value V20766_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P20766_15 = {S37, 0, &V20766_15_90837, &P20766_317};	/*Property = LayoutEngine*/
const Value V20766_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_344 = {S38, 0, &V20766_344_57274, &P20766_15};	/*Property = Masking*/
const Value V20766_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_320 = {S39, 0, &V20766_320_57292, &P20766_344};	/*Property = PostMessage*/
const Value V20766_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_321 = {S40, 0, &V20766_321_57295, &P20766_320};	/*Property = Progress*/
const Value V20766_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_322 = {S41, 0, &V20766_322_57298, &P20766_321};	/*Property = Prompts*/
const Value V20766_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_323 = {S42, 0, &V20766_323_57301, &P20766_322};	/*Property = Selector*/
const Value V20766_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_324 = {S43, 0, &V20766_324_57307, &P20766_323};	/*Property = Svg*/
const Value V20766_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_327 = {S44, 0, &V20766_327_57316, &P20766_324};	/*Property = TouchEvents*/
const Value V20766_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_328 = {S45, 0, &V20766_328_57319, &P20766_327};	/*Property = Track*/
const Value V20766_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_329 = {S46, 0, &V20766_329_57325, &P20766_328};	/*Property = Video*/
const Value V20766_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_330 = {S47, 0, &V20766_330_57328, &P20766_329};	/*Property = Viewport*/
const Value V20766_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_332 = {S48, 0, &V20766_332_57340, &P20766_330};	/*Property = WebWorkers*/
const Value V20766_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20766_333 = {S49, 0, &V20766_333_57343, &P20766_332};	/*Property = Xhr2*/
const Profile P20766 = {"20766", &P20766_333};	/*Profile ID - 20766*/
const Value V22519_1_1 = {S2, NULL};	/*Value = True*/
const Property P22519_1 = {S50, 0, &V22519_1_1, NULL};	/*Property = IsMobile*/
const Value V22519_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22519_5 = {S52, 0, &V22519_5_1832, &P22519_1};	/*Property = ScreenPixelsHeight*/
const Value V22519_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22519_4 = {S54, 0, &V22519_4_2336, &P22519_5};	/*Property = ScreenPixelsWidth*/
const Profile P22519 = {"22519", &P22519_4};	/*Profile ID - 22519*/
const Value V21228_1_1 = {S2, NULL};	/*Value = True*/
const Property P21228_1 = {S50, 0, &V21228_1_1, NULL};	/*Property = IsMobile*/
const Value V21228_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21228_5 = {S52, 0, &V21228_5_1227, &P21228_1};	/*Property = ScreenPixelsHeight*/
const Value V21228_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21228_4 = {S54, 0, &V21228_4_1229, &P21228_5};	/*Property = ScreenPixelsWidth*/
const Profile P21228 = {"21228", &P21228_4};	/*Profile ID - 21228*/
const Value V21220_1_1 = {S2, NULL};	/*Value = True*/
const Property P21220_1 = {S50, 0, &V21220_1_1, NULL};	/*Property = IsMobile*/
const Value V21220_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21220_5 = {S52, 0, &V21220_5_1403, &P21220_1};	/*Property = ScreenPixelsHeight*/
const Value V21220_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21220_4 = {S54, 0, &V21220_4_1404, &P21220_5};	/*Property = ScreenPixelsWidth*/
const Profile P21220 = {"21220", &P21220_4};	/*Profile ID - 21220*/
const Value V22817_1_1 = {S2, NULL};	/*Value = True*/
const Property P22817_1 = {S50, 0, &V22817_1_1, NULL};	/*Property = IsMobile*/
const Value V22817_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22817_5 = {S52, 0, &V22817_5_1277, &P22817_1};	/*Property = ScreenPixelsHeight*/
const Value V22817_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22817_4 = {S54, 0, &V22817_4_2336, &P22817_5};	/*Property = ScreenPixelsWidth*/
const Profile P22817 = {"22817", &P22817_4};	/*Profile ID - 22817*/
const Value V21527_1_1 = {S2, NULL};	/*Value = True*/
const Property P21527_1 = {S50, 0, &V21527_1_1, NULL};	/*Property = IsMobile*/
const Value V21527_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21527_5 = {S52, 0, &V21527_5_1227, &P21527_1};	/*Property = ScreenPixelsHeight*/
const Value V21527_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21527_4 = {S54, 0, &V21527_4_1229, &P21527_5};	/*Property = ScreenPixelsWidth*/
const Profile P21527 = {"21527", &P21527_4};	/*Profile ID - 21527*/
const Value V18717_1_1 = {S2, NULL};	/*Value = True*/
const Property P18717_1 = {S50, 0, &V18717_1_1, NULL};	/*Property = IsMobile*/
const Value V18717_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P18717_5 = {S52, 0, &V18717_5_1458, &P18717_1};	/*Property = ScreenPixelsHeight*/
const Value V18717_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P18717_4 = {S54, 0, &V18717_4_37617, &P18717_5};	/*Property = ScreenPixelsWidth*/
const Profile P18717 = {"18717", &P18717_4};	/*Profile ID - 18717*/
const Value V22262_1_1 = {S2, NULL};	/*Value = True*/
const Property P22262_1 = {S50, 0, &V22262_1_1, NULL};	/*Property = IsMobile*/
const Value V22262_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22262_5 = {S52, 0, &V22262_5_1227, &P22262_1};	/*Property = ScreenPixelsHeight*/
const Value V22262_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22262_4 = {S54, 0, &V22262_4_1229, &P22262_5};	/*Property = ScreenPixelsWidth*/
const Profile P22262 = {"22262", &P22262_4};	/*Profile ID - 22262*/
const Value V20944_1_1 = {S2, NULL};	/*Value = True*/
const Property P20944_1 = {S50, 0, &V20944_1_1, NULL};	/*Property = IsMobile*/
const Value V20944_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20944_5 = {S52, 0, &V20944_5_1458, &P20944_1};	/*Property = ScreenPixelsHeight*/
const Value V20944_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20944_4 = {S54, 0, &V20944_4_1460, &P20944_5};	/*Property = ScreenPixelsWidth*/
const Profile P20944 = {"20944", &P20944_4};	/*Profile ID - 20944*/
const Value V20391_1_2 = {S0, NULL};	/*Value = False*/
const Property P20391_1 = {S50, 0, &V20391_1_2, NULL};	/*Property = IsMobile*/
const Value V20391_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20391_5 = {S52, 0, &V20391_5_37, &P20391_1};	/*Property = ScreenPixelsHeight*/
const Value V20391_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20391_4 = {S54, 0, &V20391_4_36, &P20391_5};	/*Property = ScreenPixelsWidth*/
const Profile P20391 = {"20391", &P20391_4};	/*Profile ID - 20391*/
const Value V19787_1_1 = {S2, NULL};	/*Value = True*/
const Property P19787_1 = {S50, 0, &V19787_1_1, NULL};	/*Property = IsMobile*/
const Value V19787_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19787_5 = {S52, 0, &V19787_5_1227, &P19787_1};	/*Property = ScreenPixelsHeight*/
const Value V19787_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19787_4 = {S54, 0, &V19787_4_1229, &P19787_5};	/*Property = ScreenPixelsWidth*/
const Profile P19787 = {"19787", &P19787_4};	/*Profile ID - 19787*/
const Value V19859_1_1 = {S2, NULL};	/*Value = True*/
const Property P19859_1 = {S50, 0, &V19859_1_1, NULL};	/*Property = IsMobile*/
const Value V19859_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19859_5 = {S52, 0, &V19859_5_1227, &P19859_1};	/*Property = ScreenPixelsHeight*/
const Value V19859_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19859_4 = {S54, 0, &V19859_4_1229, &P19859_5};	/*Property = ScreenPixelsWidth*/
const Profile P19859 = {"19859", &P19859_4};	/*Profile ID - 19859*/
const Value V21129_1_1 = {S2, NULL};	/*Value = True*/
const Property P21129_1 = {S50, 0, &V21129_1_1, NULL};	/*Property = IsMobile*/
const Value V21129_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21129_5 = {S52, 0, &V21129_5_1227, &P21129_1};	/*Property = ScreenPixelsHeight*/
const Value V21129_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21129_4 = {S54, 0, &V21129_4_1229, &P21129_5};	/*Property = ScreenPixelsWidth*/
const Profile P21129 = {"21129", &P21129_4};	/*Profile ID - 21129*/
const Value V18496_1_1 = {S2, NULL};	/*Value = True*/
const Property P18496_1 = {S50, 0, &V18496_1_1, NULL};	/*Property = IsMobile*/
const Value V18496_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18496_5 = {S52, 0, &V18496_5_1403, &P18496_1};	/*Property = ScreenPixelsHeight*/
const Value V18496_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18496_4 = {S54, 0, &V18496_4_1404, &P18496_5};	/*Property = ScreenPixelsWidth*/
const Profile P18496 = {"18496", &P18496_4};	/*Profile ID - 18496*/
const Value V21347_1_1 = {S2, NULL};	/*Value = True*/
const Property P21347_1 = {S50, 0, &V21347_1_1, NULL};	/*Property = IsMobile*/
const Value V21347_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21347_5 = {S52, 0, &V21347_5_1277, &P21347_1};	/*Property = ScreenPixelsHeight*/
const Value V21347_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21347_4 = {S54, 0, &V21347_4_1279, &P21347_5};	/*Property = ScreenPixelsWidth*/
const Profile P21347 = {"21347", &P21347_4};	/*Profile ID - 21347*/
const Value V22349_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_279 = {S1, 0, &V22349_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22349_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_282 = {S3, 0, &V22349_282_57155, &P22349_279};	/*Property = BlobBuilder*/
const Value V22349_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_350 = {S4, 0, &V22349_350_57446, &P22349_282};	/*Property = Canvas*/
const Value V22349_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_283 = {S5, 0, &V22349_283_57161, &P22349_350};	/*Property = CssBackground*/
const Value V22349_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_284 = {S6, 0, &V22349_284_57164, &P22349_283};	/*Property = CssBorderImage*/
const Value V22349_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_285 = {S7, 0, &V22349_285_57167, &P22349_284};	/*Property = CssCanvas*/
const Value V22349_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_286 = {S8, 0, &V22349_286_57170, &P22349_285};	/*Property = CssColor*/
const Value V22349_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_237 = {S9, 0, &V22349_237_55786, &P22349_286};	/*Property = CssColumn*/
const Value V22349_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_287 = {S10, 0, &V22349_287_57173, &P22349_237};	/*Property = CssFlexbox*/
const Value V22349_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_288 = {S11, 0, &V22349_288_57176, &P22349_287};	/*Property = CssFont*/
const Value V22349_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_289 = {S12, 0, &V22349_289_86899, &P22349_288};	/*Property = CssImages*/
const Value V22349_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_291 = {S13, 0, &V22349_291_57182, &P22349_289};	/*Property = CssMediaQueries*/
const Value V22349_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_292 = {S14, 0, &V22349_292_57185, &P22349_291};	/*Property = CssMinMax*/
const Value V22349_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_293 = {S15, 0, &V22349_293_57188, &P22349_292};	/*Property = CssOverflow*/
const Value V22349_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_294 = {S16, 0, &V22349_294_57191, &P22349_293};	/*Property = CssPosition*/
const Value V22349_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_295 = {S17, 0, &V22349_295_57194, &P22349_294};	/*Property = CssText*/
const Value V22349_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_297 = {S18, 0, &V22349_297_57197, &P22349_295};	/*Property = CssTransforms*/
const Value V22349_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_241 = {S19, 0, &V22349_241_55801, &P22349_297};	/*Property = CssTransitions*/
const Value V22349_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_298 = {S20, 0, &V22349_298_57208, &P22349_241};	/*Property = CssUI*/
const Value V22349_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_300 = {S21, 0, &V22349_300_57217, &P22349_298};	/*Property = DataSet*/
const Value V22349_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_301 = {S22, 0, &V22349_301_57220, &P22349_300};	/*Property = DataUrl*/
const Value V22349_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_302 = {S23, 0, &V22349_302_57223, &P22349_301};	/*Property = DeviceOrientation*/
const Value V22349_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_304 = {S24, 0, &V22349_304_57229, &P22349_302};	/*Property = FileReader*/
const Value V22349_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_305 = {S25, 0, &V22349_305_57232, &P22349_304};	/*Property = FileSaver*/
const Value V22349_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_306 = {S26, 0, &V22349_306_57238, &P22349_305};	/*Property = FileWriter*/
const Value V22349_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_307 = {S27, 0, &V22349_307_57241, &P22349_306};	/*Property = FormData*/
const Value V22349_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_309 = {S28, 0, &V22349_309_57247, &P22349_307};	/*Property = Fullscreen*/
const Value V22349_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_310 = {S29, 0, &V22349_310_57250, &P22349_309};	/*Property = GeoLocation*/
const Value V22349_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_312 = {S30, 0, &V22349_312_57256, &P22349_310};	/*Property = History*/
const Value V22349_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_314 = {S31, 0, &V22349_314_57262, &P22349_312};	/*Property = Html5*/
const Value V22349_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_313 = {S32, 0, &V22349_313_57259, &P22349_314};	/*Property = Html-Media-Capture*/
const Value V22349_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_315 = {S33, 0, &V22349_315_57265, &P22349_313};	/*Property = Iframe*/
const Value V22349_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_316 = {S34, 0, &V22349_316_57268, &P22349_315};	/*Property = IndexedDB*/
const Value V22349_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_317 = {S35, 0, &V22349_317_57271, &P22349_316};	/*Property = Json*/
const Value V22349_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22349_15 = {S37, 0, &V22349_15_29, &P22349_317};	/*Property = LayoutEngine*/
const Value V22349_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_344 = {S38, 0, &V22349_344_57274, &P22349_15};	/*Property = Masking*/
const Value V22349_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_320 = {S39, 0, &V22349_320_57292, &P22349_344};	/*Property = PostMessage*/
const Value V22349_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_321 = {S40, 0, &V22349_321_57295, &P22349_320};	/*Property = Progress*/
const Value V22349_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_322 = {S41, 0, &V22349_322_57298, &P22349_321};	/*Property = Prompts*/
const Value V22349_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_323 = {S42, 0, &V22349_323_57301, &P22349_322};	/*Property = Selector*/
const Value V22349_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_324 = {S43, 0, &V22349_324_57307, &P22349_323};	/*Property = Svg*/
const Value V22349_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_327 = {S44, 0, &V22349_327_57316, &P22349_324};	/*Property = TouchEvents*/
const Value V22349_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_328 = {S45, 0, &V22349_328_57319, &P22349_327};	/*Property = Track*/
const Value V22349_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_329 = {S46, 0, &V22349_329_57325, &P22349_328};	/*Property = Video*/
const Value V22349_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_330 = {S47, 0, &V22349_330_57328, &P22349_329};	/*Property = Viewport*/
const Value V22349_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_332 = {S48, 0, &V22349_332_57340, &P22349_330};	/*Property = WebWorkers*/
const Value V22349_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22349_333 = {S49, 0, &V22349_333_57343, &P22349_332};	/*Property = Xhr2*/
const Profile P22349 = {"22349", &P22349_333};	/*Profile ID - 22349*/
const Value V21170_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_279 = {S1, 0, &V21170_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21170_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_282 = {S3, 0, &V21170_282_57155, &P21170_279};	/*Property = BlobBuilder*/
const Value V21170_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_350 = {S4, 0, &V21170_350_57446, &P21170_282};	/*Property = Canvas*/
const Value V21170_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_283 = {S5, 0, &V21170_283_57161, &P21170_350};	/*Property = CssBackground*/
const Value V21170_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_284 = {S6, 0, &V21170_284_57164, &P21170_283};	/*Property = CssBorderImage*/
const Value V21170_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_285 = {S7, 0, &V21170_285_57167, &P21170_284};	/*Property = CssCanvas*/
const Value V21170_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_286 = {S8, 0, &V21170_286_57170, &P21170_285};	/*Property = CssColor*/
const Value V21170_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_237 = {S9, 0, &V21170_237_55786, &P21170_286};	/*Property = CssColumn*/
const Value V21170_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_287 = {S10, 0, &V21170_287_57173, &P21170_237};	/*Property = CssFlexbox*/
const Value V21170_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_288 = {S11, 0, &V21170_288_57176, &P21170_287};	/*Property = CssFont*/
const Value V21170_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_289 = {S12, 0, &V21170_289_86899, &P21170_288};	/*Property = CssImages*/
const Value V21170_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_291 = {S13, 0, &V21170_291_57182, &P21170_289};	/*Property = CssMediaQueries*/
const Value V21170_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_292 = {S14, 0, &V21170_292_57185, &P21170_291};	/*Property = CssMinMax*/
const Value V21170_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_293 = {S15, 0, &V21170_293_57188, &P21170_292};	/*Property = CssOverflow*/
const Value V21170_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_294 = {S16, 0, &V21170_294_57191, &P21170_293};	/*Property = CssPosition*/
const Value V21170_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_295 = {S17, 0, &V21170_295_57194, &P21170_294};	/*Property = CssText*/
const Value V21170_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_297 = {S18, 0, &V21170_297_57197, &P21170_295};	/*Property = CssTransforms*/
const Value V21170_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_241 = {S19, 0, &V21170_241_55801, &P21170_297};	/*Property = CssTransitions*/
const Value V21170_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_298 = {S20, 0, &V21170_298_57208, &P21170_241};	/*Property = CssUI*/
const Value V21170_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_300 = {S21, 0, &V21170_300_57217, &P21170_298};	/*Property = DataSet*/
const Value V21170_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_301 = {S22, 0, &V21170_301_57220, &P21170_300};	/*Property = DataUrl*/
const Value V21170_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_302 = {S23, 0, &V21170_302_57223, &P21170_301};	/*Property = DeviceOrientation*/
const Value V21170_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_304 = {S24, 0, &V21170_304_57229, &P21170_302};	/*Property = FileReader*/
const Value V21170_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_305 = {S25, 0, &V21170_305_57232, &P21170_304};	/*Property = FileSaver*/
const Value V21170_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_306 = {S26, 0, &V21170_306_57238, &P21170_305};	/*Property = FileWriter*/
const Value V21170_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_307 = {S27, 0, &V21170_307_57241, &P21170_306};	/*Property = FormData*/
const Value V21170_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_309 = {S28, 0, &V21170_309_57247, &P21170_307};	/*Property = Fullscreen*/
const Value V21170_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_310 = {S29, 0, &V21170_310_57250, &P21170_309};	/*Property = GeoLocation*/
const Value V21170_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_312 = {S30, 0, &V21170_312_57256, &P21170_310};	/*Property = History*/
const Value V21170_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_314 = {S31, 0, &V21170_314_57262, &P21170_312};	/*Property = Html5*/
const Value V21170_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_313 = {S32, 0, &V21170_313_57259, &P21170_314};	/*Property = Html-Media-Capture*/
const Value V21170_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_315 = {S33, 0, &V21170_315_57265, &P21170_313};	/*Property = Iframe*/
const Value V21170_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_316 = {S34, 0, &V21170_316_57268, &P21170_315};	/*Property = IndexedDB*/
const Value V21170_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_317 = {S35, 0, &V21170_317_57271, &P21170_316};	/*Property = Json*/
const Value V21170_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P21170_15 = {S37, 0, &V21170_15_29, &P21170_317};	/*Property = LayoutEngine*/
const Value V21170_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_344 = {S38, 0, &V21170_344_57274, &P21170_15};	/*Property = Masking*/
const Value V21170_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_320 = {S39, 0, &V21170_320_57292, &P21170_344};	/*Property = PostMessage*/
const Value V21170_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_321 = {S40, 0, &V21170_321_57295, &P21170_320};	/*Property = Progress*/
const Value V21170_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_322 = {S41, 0, &V21170_322_57298, &P21170_321};	/*Property = Prompts*/
const Value V21170_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_323 = {S42, 0, &V21170_323_57301, &P21170_322};	/*Property = Selector*/
const Value V21170_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_324 = {S43, 0, &V21170_324_57307, &P21170_323};	/*Property = Svg*/
const Value V21170_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_327 = {S44, 0, &V21170_327_57316, &P21170_324};	/*Property = TouchEvents*/
const Value V21170_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_328 = {S45, 0, &V21170_328_57319, &P21170_327};	/*Property = Track*/
const Value V21170_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_329 = {S46, 0, &V21170_329_57325, &P21170_328};	/*Property = Video*/
const Value V21170_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_330 = {S47, 0, &V21170_330_57328, &P21170_329};	/*Property = Viewport*/
const Value V21170_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_332 = {S48, 0, &V21170_332_57340, &P21170_330};	/*Property = WebWorkers*/
const Value V21170_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21170_333 = {S49, 0, &V21170_333_57343, &P21170_332};	/*Property = Xhr2*/
const Profile P21170 = {"21170", &P21170_333};	/*Profile ID - 21170*/
const Value V21673_1_1 = {S2, NULL};	/*Value = True*/
const Property P21673_1 = {S50, 0, &V21673_1_1, NULL};	/*Property = IsMobile*/
const Value V21673_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21673_5 = {S52, 0, &V21673_5_1227, &P21673_1};	/*Property = ScreenPixelsHeight*/
const Value V21673_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21673_4 = {S54, 0, &V21673_4_1229, &P21673_5};	/*Property = ScreenPixelsWidth*/
const Profile P21673 = {"21673", &P21673_4};	/*Profile ID - 21673*/
const Value V24030_1_1 = {S2, NULL};	/*Value = True*/
const Property P24030_1 = {S50, 0, &V24030_1_1, NULL};	/*Property = IsMobile*/
const Value V24030_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24030_5 = {S52, 0, &V24030_5_1227, &P24030_1};	/*Property = ScreenPixelsHeight*/
const Value V24030_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24030_4 = {S54, 0, &V24030_4_1229, &P24030_5};	/*Property = ScreenPixelsWidth*/
const Profile P24030 = {"24030", &P24030_4};	/*Profile ID - 24030*/
const Value V21702_1_1 = {S2, NULL};	/*Value = True*/
const Property P21702_1 = {S50, 0, &V21702_1_1, NULL};	/*Property = IsMobile*/
const Value V21702_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21702_5 = {S52, 0, &V21702_5_1277, &P21702_1};	/*Property = ScreenPixelsHeight*/
const Value V21702_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21702_4 = {S54, 0, &V21702_4_2336, &P21702_5};	/*Property = ScreenPixelsWidth*/
const Profile P21702 = {"21702", &P21702_4};	/*Profile ID - 21702*/
const Value V22106_1_1 = {S2, NULL};	/*Value = True*/
const Property P22106_1 = {S50, 0, &V22106_1_1, NULL};	/*Property = IsMobile*/
const Value V22106_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22106_5 = {S52, 0, &V22106_5_1227, &P22106_1};	/*Property = ScreenPixelsHeight*/
const Value V22106_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22106_4 = {S54, 0, &V22106_4_1229, &P22106_5};	/*Property = ScreenPixelsWidth*/
const Profile P22106 = {"22106", &P22106_4};	/*Profile ID - 22106*/
const Value V20158_1_1 = {S2, NULL};	/*Value = True*/
const Property P20158_1 = {S50, 0, &V20158_1_1, NULL};	/*Property = IsMobile*/
const Value V20158_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20158_5 = {S52, 0, &V20158_5_1277, &P20158_1};	/*Property = ScreenPixelsHeight*/
const Value V20158_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20158_4 = {S54, 0, &V20158_4_1279, &P20158_5};	/*Property = ScreenPixelsWidth*/
const Profile P20158 = {"20158", &P20158_4};	/*Profile ID - 20158*/
const Value V20904_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20904_279 = {S1, 0, &V20904_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20904_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20904_282 = {S3, 0, &V20904_282_57154, &P20904_279};	/*Property = BlobBuilder*/
const Value V20904_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20904_350 = {S4, 0, &V20904_350_57444, &P20904_282};	/*Property = Canvas*/
const Value V20904_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20904_283 = {S5, 0, &V20904_283_57159, &P20904_350};	/*Property = CssBackground*/
const Value V20904_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20904_284 = {S6, 0, &V20904_284_57163, &P20904_283};	/*Property = CssBorderImage*/
const Value V20904_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20904_285 = {S7, 0, &V20904_285_57166, &P20904_284};	/*Property = CssCanvas*/
const Value V20904_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20904_286 = {S8, 0, &V20904_286_57168, &P20904_285};	/*Property = CssColor*/
const Value V20904_237_55785 = {S0, NULL};	/*Value = False*/
const Property P20904_237 = {S9, 0, &V20904_237_55785, &P20904_286};	/*Property = CssColumn*/
const Value V20904_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20904_287 = {S10, 0, &V20904_287_57172, &P20904_237};	/*Property = CssFlexbox*/
const Value V20904_288_57175 = {S0, NULL};	/*Value = False*/
const Property P20904_288 = {S11, 0, &V20904_288_57175, &P20904_287};	/*Property = CssFont*/
const Value V20904_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20904_289 = {S12, 0, &V20904_289_57759, &P20904_288};	/*Property = CssImages*/
const Value V20904_291_57181 = {S0, NULL};	/*Value = False*/
const Property P20904_291 = {S13, 0, &V20904_291_57181, &P20904_289};	/*Property = CssMediaQueries*/
const Value V20904_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20904_292 = {S14, 0, &V20904_292_57183, &P20904_291};	/*Property = CssMinMax*/
const Value V20904_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20904_293 = {S15, 0, &V20904_293_57187, &P20904_292};	/*Property = CssOverflow*/
const Value V20904_294_57190 = {S0, NULL};	/*Value = False*/
const Property P20904_294 = {S16, 0, &V20904_294_57190, &P20904_293};	/*Property = CssPosition*/
const Value V20904_295_57192 = {S2, NULL};	/*Value = True*/
const Property P20904_295 = {S17, 0, &V20904_295_57192, &P20904_294};	/*Property = CssText*/
const Value V20904_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20904_297 = {S18, 0, &V20904_297_57195, &P20904_295};	/*Property = CssTransforms*/
const Value V20904_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20904_241 = {S19, 0, &V20904_241_55799, &P20904_297};	/*Property = CssTransitions*/
const Value V20904_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20904_298 = {S20, 0, &V20904_298_57206, &P20904_241};	/*Property = CssUI*/
const Value V20904_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20904_300 = {S21, 0, &V20904_300_57216, &P20904_298};	/*Property = DataSet*/
const Value V20904_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20904_301 = {S22, 0, &V20904_301_57218, &P20904_300};	/*Property = DataUrl*/
const Value V20904_302_57222 = {S0, NULL};	/*Value = False*/
const Property P20904_302 = {S23, 0, &V20904_302_57222, &P20904_301};	/*Property = DeviceOrientation*/
const Value V20904_304_57228 = {S0, NULL};	/*Value = False*/
const Property P20904_304 = {S24, 0, &V20904_304_57228, &P20904_302};	/*Property = FileReader*/
const Value V20904_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20904_305 = {S25, 0, &V20904_305_57231, &P20904_304};	/*Property = FileSaver*/
const Value V20904_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20904_306 = {S26, 0, &V20904_306_57237, &P20904_305};	/*Property = FileWriter*/
const Value V20904_307_57240 = {S0, NULL};	/*Value = False*/
const Property P20904_307 = {S27, 0, &V20904_307_57240, &P20904_306};	/*Property = FormData*/
const Value V20904_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20904_309 = {S28, 0, &V20904_309_57246, &P20904_307};	/*Property = Fullscreen*/
const Value V20904_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20904_310 = {S29, 0, &V20904_310_57248, &P20904_309};	/*Property = GeoLocation*/
const Value V20904_312_57255 = {S0, NULL};	/*Value = False*/
const Property P20904_312 = {S30, 0, &V20904_312_57255, &P20904_310};	/*Property = History*/
const Value V20904_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20904_314 = {S31, 0, &V20904_314_57261, &P20904_312};	/*Property = Html5*/
const Value V20904_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20904_313 = {S32, 0, &V20904_313_57258, &P20904_314};	/*Property = Html-Media-Capture*/
const Value V20904_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20904_315 = {S33, 0, &V20904_315_57264, &P20904_313};	/*Property = Iframe*/
const Value V20904_316_57267 = {S0, NULL};	/*Value = False*/
const Property P20904_316 = {S34, 0, &V20904_316_57267, &P20904_315};	/*Property = IndexedDB*/
const Value V20904_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20904_317 = {S35, 0, &V20904_317_57269, &P20904_316};	/*Property = Json*/
const Value V20904_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P20904_15 = {S37, 0, &V20904_15_30, &P20904_317};	/*Property = LayoutEngine*/
const Value V20904_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20904_344 = {S38, 0, &V20904_344_57272, &P20904_15};	/*Property = Masking*/
const Value V20904_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20904_320 = {S39, 0, &V20904_320_57290, &P20904_344};	/*Property = PostMessage*/
const Value V20904_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20904_321 = {S40, 0, &V20904_321_57293, &P20904_320};	/*Property = Progress*/
const Value V20904_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20904_322 = {S41, 0, &V20904_322_57296, &P20904_321};	/*Property = Prompts*/
const Value V20904_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20904_323 = {S42, 0, &V20904_323_57299, &P20904_322};	/*Property = Selector*/
const Value V20904_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20904_324 = {S43, 0, &V20904_324_57306, &P20904_323};	/*Property = Svg*/
const Value V20904_327_57315 = {S0, NULL};	/*Value = False*/
const Property P20904_327 = {S44, 0, &V20904_327_57315, &P20904_324};	/*Property = TouchEvents*/
const Value V20904_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20904_328 = {S45, 0, &V20904_328_57318, &P20904_327};	/*Property = Track*/
const Value V20904_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20904_329 = {S46, 0, &V20904_329_57323, &P20904_328};	/*Property = Video*/
const Value V20904_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20904_330 = {S47, 0, &V20904_330_57326, &P20904_329};	/*Property = Viewport*/
const Value V20904_332_57339 = {S0, NULL};	/*Value = False*/
const Property P20904_332 = {S48, 0, &V20904_332_57339, &P20904_330};	/*Property = WebWorkers*/
const Value V20904_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20904_333 = {S49, 0, &V20904_333_57342, &P20904_332};	/*Property = Xhr2*/
const Profile P20904 = {"20904", &P20904_333};	/*Profile ID - 20904*/
const Value V15984_1_1 = {S2, NULL};	/*Value = True*/
const Property P15984_1 = {S50, 0, &V15984_1_1, NULL};	/*Property = IsMobile*/
const Value V15984_5_37567 = {S95, NULL};	/*Value = 300*/
const Property P15984_5 = {S52, 0, &V15984_5_37567, &P15984_1};	/*Property = ScreenPixelsHeight*/
const Value V15984_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15984_4 = {S54, 0, &V15984_4_1224, &P15984_5};	/*Property = ScreenPixelsWidth*/
const Profile P15984 = {"15984", &P15984_4};	/*Profile ID - 15984*/
const Value V22530_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22530_279 = {S1, 0, &V22530_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22530_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22530_282 = {S3, 0, &V22530_282_57153, &P22530_279};	/*Property = BlobBuilder*/
const Value V22530_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22530_350 = {S4, 0, &V22530_350_57444, &P22530_282};	/*Property = Canvas*/
const Value V22530_283_57160 = {S0, NULL};	/*Value = False*/
const Property P22530_283 = {S5, 0, &V22530_283_57160, &P22530_350};	/*Property = CssBackground*/
const Value V22530_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22530_284 = {S6, 0, &V22530_284_57163, &P22530_283};	/*Property = CssBorderImage*/
const Value V22530_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22530_285 = {S7, 0, &V22530_285_57166, &P22530_284};	/*Property = CssCanvas*/
const Value V22530_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22530_286 = {S8, 0, &V22530_286_57168, &P22530_285};	/*Property = CssColor*/
const Value V22530_237_55785 = {S0, NULL};	/*Value = False*/
const Property P22530_237 = {S9, 0, &V22530_237_55785, &P22530_286};	/*Property = CssColumn*/
const Value V22530_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22530_287 = {S10, 0, &V22530_287_57172, &P22530_237};	/*Property = CssFlexbox*/
const Value V22530_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22530_288 = {S11, 0, &V22530_288_57175, &P22530_287};	/*Property = CssFont*/
const Value V22530_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22530_289 = {S12, 0, &V22530_289_86899, &P22530_288};	/*Property = CssImages*/
const Value V22530_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22530_291 = {S13, 0, &V22530_291_57181, &P22530_289};	/*Property = CssMediaQueries*/
const Value V22530_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22530_292 = {S14, 0, &V22530_292_57183, &P22530_291};	/*Property = CssMinMax*/
const Value V22530_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22530_293 = {S15, 0, &V22530_293_57187, &P22530_292};	/*Property = CssOverflow*/
const Value V22530_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22530_294 = {S16, 0, &V22530_294_57189, &P22530_293};	/*Property = CssPosition*/
const Value V22530_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22530_295 = {S17, 0, &V22530_295_57193, &P22530_294};	/*Property = CssText*/
const Value V22530_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22530_297 = {S18, 0, &V22530_297_57195, &P22530_295};	/*Property = CssTransforms*/
const Value V22530_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22530_241 = {S19, 0, &V22530_241_55799, &P22530_297};	/*Property = CssTransitions*/
const Value V22530_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22530_298 = {S20, 0, &V22530_298_57206, &P22530_241};	/*Property = CssUI*/
const Value V22530_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22530_300 = {S21, 0, &V22530_300_57215, &P22530_298};	/*Property = DataSet*/
const Value V22530_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22530_301 = {S22, 0, &V22530_301_57218, &P22530_300};	/*Property = DataUrl*/
const Value V22530_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22530_302 = {S23, 0, &V22530_302_57221, &P22530_301};	/*Property = DeviceOrientation*/
const Value V22530_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22530_304 = {S24, 0, &V22530_304_57227, &P22530_302};	/*Property = FileReader*/
const Value V22530_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22530_305 = {S25, 0, &V22530_305_57231, &P22530_304};	/*Property = FileSaver*/
const Value V22530_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22530_306 = {S26, 0, &V22530_306_57237, &P22530_305};	/*Property = FileWriter*/
const Value V22530_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22530_307 = {S27, 0, &V22530_307_57239, &P22530_306};	/*Property = FormData*/
const Value V22530_309_57245 = {S2, NULL};	/*Value = True*/
const Property P22530_309 = {S28, 0, &V22530_309_57245, &P22530_307};	/*Property = Fullscreen*/
const Value V22530_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22530_310 = {S29, 0, &V22530_310_57248, &P22530_309};	/*Property = GeoLocation*/
const Value V22530_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22530_312 = {S30, 0, &V22530_312_57254, &P22530_310};	/*Property = History*/
const Value V22530_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22530_314 = {S31, 0, &V22530_314_57261, &P22530_312};	/*Property = Html5*/
const Value V22530_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22530_313 = {S32, 0, &V22530_313_57258, &P22530_314};	/*Property = Html-Media-Capture*/
const Value V22530_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22530_315 = {S33, 0, &V22530_315_57264, &P22530_313};	/*Property = Iframe*/
const Value V22530_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22530_316 = {S34, 0, &V22530_316_57266, &P22530_315};	/*Property = IndexedDB*/
const Value V22530_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22530_317 = {S35, 0, &V22530_317_57269, &P22530_316};	/*Property = Json*/
const Value V22530_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22530_15 = {S37, 0, &V22530_15_29, &P22530_317};	/*Property = LayoutEngine*/
const Value V22530_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22530_344 = {S38, 0, &V22530_344_57272, &P22530_15};	/*Property = Masking*/
const Value V22530_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22530_320 = {S39, 0, &V22530_320_57290, &P22530_344};	/*Property = PostMessage*/
const Value V22530_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22530_321 = {S40, 0, &V22530_321_57293, &P22530_320};	/*Property = Progress*/
const Value V22530_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22530_322 = {S41, 0, &V22530_322_57296, &P22530_321};	/*Property = Prompts*/
const Value V22530_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22530_323 = {S42, 0, &V22530_323_57299, &P22530_322};	/*Property = Selector*/
const Value V22530_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22530_324 = {S43, 0, &V22530_324_57305, &P22530_323};	/*Property = Svg*/
const Value V22530_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22530_327 = {S44, 0, &V22530_327_57315, &P22530_324};	/*Property = TouchEvents*/
const Value V22530_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22530_328 = {S45, 0, &V22530_328_57318, &P22530_327};	/*Property = Track*/
const Value V22530_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22530_329 = {S46, 0, &V22530_329_57323, &P22530_328};	/*Property = Video*/
const Value V22530_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22530_330 = {S47, 0, &V22530_330_57326, &P22530_329};	/*Property = Viewport*/
const Value V22530_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22530_332 = {S48, 0, &V22530_332_57339, &P22530_330};	/*Property = WebWorkers*/
const Value V22530_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22530_333 = {S49, 0, &V22530_333_57342, &P22530_332};	/*Property = Xhr2*/
const Profile P22530 = {"22530", &P22530_333};	/*Profile ID - 22530*/
const Value V18436_1_2 = {S0, NULL};	/*Value = False*/
const Property P18436_1 = {S50, 0, &V18436_1_2, NULL};	/*Property = IsMobile*/
const Value V18436_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P18436_5 = {S52, 0, &V18436_5_37, &P18436_1};	/*Property = ScreenPixelsHeight*/
const Value V18436_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P18436_4 = {S54, 0, &V18436_4_36, &P18436_5};	/*Property = ScreenPixelsWidth*/
const Profile P18436 = {"18436", &P18436_4};	/*Profile ID - 18436*/
const Value V21401_1_1 = {S2, NULL};	/*Value = True*/
const Property P21401_1 = {S50, 0, &V21401_1_1, NULL};	/*Property = IsMobile*/
const Value V21401_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21401_5 = {S52, 0, &V21401_5_1277, &P21401_1};	/*Property = ScreenPixelsHeight*/
const Value V21401_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21401_4 = {S54, 0, &V21401_4_1279, &P21401_5};	/*Property = ScreenPixelsWidth*/
const Profile P21401 = {"21401", &P21401_4};	/*Profile ID - 21401*/
const Value V18645_1_1 = {S2, NULL};	/*Value = True*/
const Property P18645_1 = {S50, 0, &V18645_1_1, NULL};	/*Property = IsMobile*/
const Value V18645_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18645_5 = {S52, 0, &V18645_5_9981, &P18645_1};	/*Property = ScreenPixelsHeight*/
const Value V18645_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18645_4 = {S54, 0, &V18645_4_39098, &P18645_5};	/*Property = ScreenPixelsWidth*/
const Profile P18645 = {"18645", &P18645_4};	/*Profile ID - 18645*/
const Value V19952_1_1 = {S2, NULL};	/*Value = True*/
const Property P19952_1 = {S50, 0, &V19952_1_1, NULL};	/*Property = IsMobile*/
const Value V19952_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P19952_5 = {S52, 0, &V19952_5_1222, &P19952_1};	/*Property = ScreenPixelsHeight*/
const Value V19952_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P19952_4 = {S54, 0, &V19952_4_1224, &P19952_5};	/*Property = ScreenPixelsWidth*/
const Profile P19952 = {"19952", &P19952_4};	/*Profile ID - 19952*/
const Value V18727_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_279 = {S1, 0, &V18727_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18727_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_282 = {S3, 0, &V18727_282_57155, &P18727_279};	/*Property = BlobBuilder*/
const Value V18727_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_350 = {S4, 0, &V18727_350_57446, &P18727_282};	/*Property = Canvas*/
const Value V18727_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_283 = {S5, 0, &V18727_283_57161, &P18727_350};	/*Property = CssBackground*/
const Value V18727_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_284 = {S6, 0, &V18727_284_57164, &P18727_283};	/*Property = CssBorderImage*/
const Value V18727_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_285 = {S7, 0, &V18727_285_57167, &P18727_284};	/*Property = CssCanvas*/
const Value V18727_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_286 = {S8, 0, &V18727_286_57170, &P18727_285};	/*Property = CssColor*/
const Value V18727_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_237 = {S9, 0, &V18727_237_55786, &P18727_286};	/*Property = CssColumn*/
const Value V18727_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_287 = {S10, 0, &V18727_287_57173, &P18727_237};	/*Property = CssFlexbox*/
const Value V18727_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_288 = {S11, 0, &V18727_288_57176, &P18727_287};	/*Property = CssFont*/
const Value V18727_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_289 = {S12, 0, &V18727_289_86899, &P18727_288};	/*Property = CssImages*/
const Value V18727_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_291 = {S13, 0, &V18727_291_57182, &P18727_289};	/*Property = CssMediaQueries*/
const Value V18727_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_292 = {S14, 0, &V18727_292_57185, &P18727_291};	/*Property = CssMinMax*/
const Value V18727_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_293 = {S15, 0, &V18727_293_57188, &P18727_292};	/*Property = CssOverflow*/
const Value V18727_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_294 = {S16, 0, &V18727_294_57191, &P18727_293};	/*Property = CssPosition*/
const Value V18727_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_295 = {S17, 0, &V18727_295_57194, &P18727_294};	/*Property = CssText*/
const Value V18727_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_297 = {S18, 0, &V18727_297_57197, &P18727_295};	/*Property = CssTransforms*/
const Value V18727_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_241 = {S19, 0, &V18727_241_55801, &P18727_297};	/*Property = CssTransitions*/
const Value V18727_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_298 = {S20, 0, &V18727_298_57208, &P18727_241};	/*Property = CssUI*/
const Value V18727_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_300 = {S21, 0, &V18727_300_57217, &P18727_298};	/*Property = DataSet*/
const Value V18727_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_301 = {S22, 0, &V18727_301_57220, &P18727_300};	/*Property = DataUrl*/
const Value V18727_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_302 = {S23, 0, &V18727_302_57223, &P18727_301};	/*Property = DeviceOrientation*/
const Value V18727_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_304 = {S24, 0, &V18727_304_57229, &P18727_302};	/*Property = FileReader*/
const Value V18727_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_305 = {S25, 0, &V18727_305_57232, &P18727_304};	/*Property = FileSaver*/
const Value V18727_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_306 = {S26, 0, &V18727_306_57238, &P18727_305};	/*Property = FileWriter*/
const Value V18727_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_307 = {S27, 0, &V18727_307_57241, &P18727_306};	/*Property = FormData*/
const Value V18727_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_309 = {S28, 0, &V18727_309_57247, &P18727_307};	/*Property = Fullscreen*/
const Value V18727_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_310 = {S29, 0, &V18727_310_57250, &P18727_309};	/*Property = GeoLocation*/
const Value V18727_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_312 = {S30, 0, &V18727_312_57256, &P18727_310};	/*Property = History*/
const Value V18727_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_314 = {S31, 0, &V18727_314_57262, &P18727_312};	/*Property = Html5*/
const Value V18727_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_313 = {S32, 0, &V18727_313_57259, &P18727_314};	/*Property = Html-Media-Capture*/
const Value V18727_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_315 = {S33, 0, &V18727_315_57265, &P18727_313};	/*Property = Iframe*/
const Value V18727_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_316 = {S34, 0, &V18727_316_57268, &P18727_315};	/*Property = IndexedDB*/
const Value V18727_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_317 = {S35, 0, &V18727_317_57271, &P18727_316};	/*Property = Json*/
const Value V18727_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18727_15 = {S37, 0, &V18727_15_29, &P18727_317};	/*Property = LayoutEngine*/
const Value V18727_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_344 = {S38, 0, &V18727_344_57274, &P18727_15};	/*Property = Masking*/
const Value V18727_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_320 = {S39, 0, &V18727_320_57292, &P18727_344};	/*Property = PostMessage*/
const Value V18727_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_321 = {S40, 0, &V18727_321_57295, &P18727_320};	/*Property = Progress*/
const Value V18727_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_322 = {S41, 0, &V18727_322_57298, &P18727_321};	/*Property = Prompts*/
const Value V18727_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_323 = {S42, 0, &V18727_323_57301, &P18727_322};	/*Property = Selector*/
const Value V18727_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_324 = {S43, 0, &V18727_324_57307, &P18727_323};	/*Property = Svg*/
const Value V18727_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_327 = {S44, 0, &V18727_327_57316, &P18727_324};	/*Property = TouchEvents*/
const Value V18727_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_328 = {S45, 0, &V18727_328_57319, &P18727_327};	/*Property = Track*/
const Value V18727_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_329 = {S46, 0, &V18727_329_57325, &P18727_328};	/*Property = Video*/
const Value V18727_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_330 = {S47, 0, &V18727_330_57328, &P18727_329};	/*Property = Viewport*/
const Value V18727_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_332 = {S48, 0, &V18727_332_57340, &P18727_330};	/*Property = WebWorkers*/
const Value V18727_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18727_333 = {S49, 0, &V18727_333_57343, &P18727_332};	/*Property = Xhr2*/
const Profile P18727 = {"18727", &P18727_333};	/*Profile ID - 18727*/
const Value V23886_1_1 = {S2, NULL};	/*Value = True*/
const Property P23886_1 = {S50, 0, &V23886_1_1, NULL};	/*Property = IsMobile*/
const Value V23886_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23886_5 = {S52, 0, &V23886_5_1458, &P23886_1};	/*Property = ScreenPixelsHeight*/
const Value V23886_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23886_4 = {S54, 0, &V23886_4_1460, &P23886_5};	/*Property = ScreenPixelsWidth*/
const Profile P23886 = {"23886", &P23886_4};	/*Profile ID - 23886*/
const Value V18437_1_1 = {S2, NULL};	/*Value = True*/
const Property P18437_1 = {S50, 0, &V18437_1_1, NULL};	/*Property = IsMobile*/
const Value V18437_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18437_5 = {S52, 0, &V18437_5_9981, &P18437_1};	/*Property = ScreenPixelsHeight*/
const Value V18437_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18437_4 = {S54, 0, &V18437_4_2336, &P18437_5};	/*Property = ScreenPixelsWidth*/
const Profile P18437 = {"18437", &P18437_4};	/*Profile ID - 18437*/
const Value V21315_1_1 = {S2, NULL};	/*Value = True*/
const Property P21315_1 = {S50, 0, &V21315_1_1, NULL};	/*Property = IsMobile*/
const Value V21315_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21315_5 = {S52, 0, &V21315_5_1458, &P21315_1};	/*Property = ScreenPixelsHeight*/
const Value V21315_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21315_4 = {S54, 0, &V21315_4_37617, &P21315_5};	/*Property = ScreenPixelsWidth*/
const Profile P21315 = {"21315", &P21315_4};	/*Profile ID - 21315*/
const Value V21255_1_1 = {S2, NULL};	/*Value = True*/
const Property P21255_1 = {S50, 0, &V21255_1_1, NULL};	/*Property = IsMobile*/
const Value V21255_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21255_5 = {S52, 0, &V21255_5_1227, &P21255_1};	/*Property = ScreenPixelsHeight*/
const Value V21255_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21255_4 = {S54, 0, &V21255_4_1229, &P21255_5};	/*Property = ScreenPixelsWidth*/
const Profile P21255 = {"21255", &P21255_4};	/*Profile ID - 21255*/
const Value V18709_1_1 = {S2, NULL};	/*Value = True*/
const Property P18709_1 = {S50, 0, &V18709_1_1, NULL};	/*Property = IsMobile*/
const Value V18709_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18709_5 = {S52, 0, &V18709_5_1227, &P18709_1};	/*Property = ScreenPixelsHeight*/
const Value V18709_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18709_4 = {S54, 0, &V18709_4_1229, &P18709_5};	/*Property = ScreenPixelsWidth*/
const Profile P18709 = {"18709", &P18709_4};	/*Profile ID - 18709*/
const Value V20590_1_1 = {S2, NULL};	/*Value = True*/
const Property P20590_1 = {S50, 0, &V20590_1_1, NULL};	/*Property = IsMobile*/
const Value V20590_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20590_5 = {S52, 0, &V20590_5_1458, &P20590_1};	/*Property = ScreenPixelsHeight*/
const Value V20590_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20590_4 = {S54, 0, &V20590_4_1460, &P20590_5};	/*Property = ScreenPixelsWidth*/
const Profile P20590 = {"20590", &P20590_4};	/*Profile ID - 20590*/
const Value V20086_1_1 = {S2, NULL};	/*Value = True*/
const Property P20086_1 = {S50, 0, &V20086_1_1, NULL};	/*Property = IsMobile*/
const Value V20086_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20086_5 = {S52, 0, &V20086_5_1277, &P20086_1};	/*Property = ScreenPixelsHeight*/
const Value V20086_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20086_4 = {S54, 0, &V20086_4_1279, &P20086_5};	/*Property = ScreenPixelsWidth*/
const Profile P20086 = {"20086", &P20086_4};	/*Profile ID - 20086*/
const Value V15972_1_1 = {S2, NULL};	/*Value = True*/
const Property P15972_1 = {S50, 0, &V15972_1_1, NULL};	/*Property = IsMobile*/
const Value V15972_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P15972_5 = {S52, 0, &V15972_5_9981, &P15972_1};	/*Property = ScreenPixelsHeight*/
const Value V15972_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P15972_4 = {S54, 0, &V15972_4_39098, &P15972_5};	/*Property = ScreenPixelsWidth*/
const Profile P15972 = {"15972", &P15972_4};	/*Profile ID - 15972*/
const Value V18382_1_1 = {S2, NULL};	/*Value = True*/
const Property P18382_1 = {S50, 0, &V18382_1_1, NULL};	/*Property = IsMobile*/
const Value V18382_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18382_5 = {S52, 0, &V18382_5_1277, &P18382_1};	/*Property = ScreenPixelsHeight*/
const Value V18382_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18382_4 = {S54, 0, &V18382_4_1279, &P18382_5};	/*Property = ScreenPixelsWidth*/
const Profile P18382 = {"18382", &P18382_4};	/*Profile ID - 18382*/
const Value V20369_1_1 = {S2, NULL};	/*Value = True*/
const Property P20369_1 = {S50, 0, &V20369_1_1, NULL};	/*Property = IsMobile*/
const Value V20369_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20369_5 = {S52, 0, &V20369_5_1227, &P20369_1};	/*Property = ScreenPixelsHeight*/
const Value V20369_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20369_4 = {S54, 0, &V20369_4_1229, &P20369_5};	/*Property = ScreenPixelsWidth*/
const Profile P20369 = {"20369", &P20369_4};	/*Profile ID - 20369*/
const Value V19800_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_279 = {S1, 0, &V19800_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V19800_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_282 = {S3, 0, &V19800_282_57155, &P19800_279};	/*Property = BlobBuilder*/
const Value V19800_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_350 = {S4, 0, &V19800_350_57446, &P19800_282};	/*Property = Canvas*/
const Value V19800_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_283 = {S5, 0, &V19800_283_57161, &P19800_350};	/*Property = CssBackground*/
const Value V19800_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_284 = {S6, 0, &V19800_284_57164, &P19800_283};	/*Property = CssBorderImage*/
const Value V19800_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_285 = {S7, 0, &V19800_285_57167, &P19800_284};	/*Property = CssCanvas*/
const Value V19800_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_286 = {S8, 0, &V19800_286_57170, &P19800_285};	/*Property = CssColor*/
const Value V19800_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_237 = {S9, 0, &V19800_237_55786, &P19800_286};	/*Property = CssColumn*/
const Value V19800_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_287 = {S10, 0, &V19800_287_57173, &P19800_237};	/*Property = CssFlexbox*/
const Value V19800_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_288 = {S11, 0, &V19800_288_57176, &P19800_287};	/*Property = CssFont*/
const Value V19800_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_289 = {S12, 0, &V19800_289_86899, &P19800_288};	/*Property = CssImages*/
const Value V19800_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_291 = {S13, 0, &V19800_291_57182, &P19800_289};	/*Property = CssMediaQueries*/
const Value V19800_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_292 = {S14, 0, &V19800_292_57185, &P19800_291};	/*Property = CssMinMax*/
const Value V19800_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_293 = {S15, 0, &V19800_293_57188, &P19800_292};	/*Property = CssOverflow*/
const Value V19800_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_294 = {S16, 0, &V19800_294_57191, &P19800_293};	/*Property = CssPosition*/
const Value V19800_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_295 = {S17, 0, &V19800_295_57194, &P19800_294};	/*Property = CssText*/
const Value V19800_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_297 = {S18, 0, &V19800_297_57197, &P19800_295};	/*Property = CssTransforms*/
const Value V19800_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_241 = {S19, 0, &V19800_241_55801, &P19800_297};	/*Property = CssTransitions*/
const Value V19800_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_298 = {S20, 0, &V19800_298_57208, &P19800_241};	/*Property = CssUI*/
const Value V19800_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_300 = {S21, 0, &V19800_300_57217, &P19800_298};	/*Property = DataSet*/
const Value V19800_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_301 = {S22, 0, &V19800_301_57220, &P19800_300};	/*Property = DataUrl*/
const Value V19800_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_302 = {S23, 0, &V19800_302_57223, &P19800_301};	/*Property = DeviceOrientation*/
const Value V19800_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_304 = {S24, 0, &V19800_304_57229, &P19800_302};	/*Property = FileReader*/
const Value V19800_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_305 = {S25, 0, &V19800_305_57232, &P19800_304};	/*Property = FileSaver*/
const Value V19800_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_306 = {S26, 0, &V19800_306_57238, &P19800_305};	/*Property = FileWriter*/
const Value V19800_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_307 = {S27, 0, &V19800_307_57241, &P19800_306};	/*Property = FormData*/
const Value V19800_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_309 = {S28, 0, &V19800_309_57247, &P19800_307};	/*Property = Fullscreen*/
const Value V19800_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_310 = {S29, 0, &V19800_310_57250, &P19800_309};	/*Property = GeoLocation*/
const Value V19800_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_312 = {S30, 0, &V19800_312_57256, &P19800_310};	/*Property = History*/
const Value V19800_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_314 = {S31, 0, &V19800_314_57262, &P19800_312};	/*Property = Html5*/
const Value V19800_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_313 = {S32, 0, &V19800_313_57259, &P19800_314};	/*Property = Html-Media-Capture*/
const Value V19800_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_315 = {S33, 0, &V19800_315_57265, &P19800_313};	/*Property = Iframe*/
const Value V19800_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_316 = {S34, 0, &V19800_316_57268, &P19800_315};	/*Property = IndexedDB*/
const Value V19800_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_317 = {S35, 0, &V19800_317_57271, &P19800_316};	/*Property = Json*/
const Value V19800_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19800_15 = {S37, 0, &V19800_15_28, &P19800_317};	/*Property = LayoutEngine*/
const Value V19800_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_344 = {S38, 0, &V19800_344_57274, &P19800_15};	/*Property = Masking*/
const Value V19800_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_320 = {S39, 0, &V19800_320_57292, &P19800_344};	/*Property = PostMessage*/
const Value V19800_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_321 = {S40, 0, &V19800_321_57295, &P19800_320};	/*Property = Progress*/
const Value V19800_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_322 = {S41, 0, &V19800_322_57298, &P19800_321};	/*Property = Prompts*/
const Value V19800_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_323 = {S42, 0, &V19800_323_57301, &P19800_322};	/*Property = Selector*/
const Value V19800_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_324 = {S43, 0, &V19800_324_57307, &P19800_323};	/*Property = Svg*/
const Value V19800_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_327 = {S44, 0, &V19800_327_57316, &P19800_324};	/*Property = TouchEvents*/
const Value V19800_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_328 = {S45, 0, &V19800_328_57319, &P19800_327};	/*Property = Track*/
const Value V19800_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_329 = {S46, 0, &V19800_329_57325, &P19800_328};	/*Property = Video*/
const Value V19800_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_330 = {S47, 0, &V19800_330_57328, &P19800_329};	/*Property = Viewport*/
const Value V19800_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_332 = {S48, 0, &V19800_332_57340, &P19800_330};	/*Property = WebWorkers*/
const Value V19800_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P19800_333 = {S49, 0, &V19800_333_57343, &P19800_332};	/*Property = Xhr2*/
const Profile P19800 = {"19800", &P19800_333};	/*Profile ID - 19800*/
const Value V19799_1_2 = {S0, NULL};	/*Value = False*/
const Property P19799_1 = {S50, 0, &V19799_1_2, NULL};	/*Property = IsMobile*/
const Value V19799_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P19799_5 = {S52, 0, &V19799_5_37, &P19799_1};	/*Property = ScreenPixelsHeight*/
const Value V19799_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P19799_4 = {S54, 0, &V19799_4_36, &P19799_5};	/*Property = ScreenPixelsWidth*/
const Profile P19799 = {"19799", &P19799_4};	/*Profile ID - 19799*/
const Value V24726_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24726_15 = {S37, 0, &V24726_15_28, NULL};	/*Property = LayoutEngine*/
const Profile P24726 = {"24726", &P24726_15};	/*Profile ID - 24726*/
const Value V18469_1_1 = {S2, NULL};	/*Value = True*/
const Property P18469_1 = {S50, 0, &V18469_1_1, NULL};	/*Property = IsMobile*/
const Value V18469_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P18469_5 = {S52, 0, &V18469_5_1458, &P18469_1};	/*Property = ScreenPixelsHeight*/
const Value V18469_4_59156 = {S133, NULL};	/*Value = 758*/
const Property P18469_4 = {S54, 0, &V18469_4_59156, &P18469_5};	/*Property = ScreenPixelsWidth*/
const Profile P18469 = {"18469", &P18469_4};	/*Profile ID - 18469*/
const Value V20055_1_1 = {S2, NULL};	/*Value = True*/
const Property P20055_1 = {S50, 0, &V20055_1_1, NULL};	/*Property = IsMobile*/
const Value V20055_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20055_5 = {S52, 0, &V20055_5_1222, &P20055_1};	/*Property = ScreenPixelsHeight*/
const Value V20055_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20055_4 = {S54, 0, &V20055_4_1224, &P20055_5};	/*Property = ScreenPixelsWidth*/
const Profile P20055 = {"20055", &P20055_4};	/*Profile ID - 20055*/
const Value V23696_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_279 = {S1, 0, &V23696_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23696_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_282 = {S3, 0, &V23696_282_57155, &P23696_279};	/*Property = BlobBuilder*/
const Value V23696_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_350 = {S4, 0, &V23696_350_57446, &P23696_282};	/*Property = Canvas*/
const Value V23696_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_283 = {S5, 0, &V23696_283_57161, &P23696_350};	/*Property = CssBackground*/
const Value V23696_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_284 = {S6, 0, &V23696_284_57164, &P23696_283};	/*Property = CssBorderImage*/
const Value V23696_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_285 = {S7, 0, &V23696_285_57167, &P23696_284};	/*Property = CssCanvas*/
const Value V23696_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_286 = {S8, 0, &V23696_286_57170, &P23696_285};	/*Property = CssColor*/
const Value V23696_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_237 = {S9, 0, &V23696_237_55786, &P23696_286};	/*Property = CssColumn*/
const Value V23696_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_287 = {S10, 0, &V23696_287_57173, &P23696_237};	/*Property = CssFlexbox*/
const Value V23696_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_288 = {S11, 0, &V23696_288_57176, &P23696_287};	/*Property = CssFont*/
const Value V23696_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_289 = {S12, 0, &V23696_289_86899, &P23696_288};	/*Property = CssImages*/
const Value V23696_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_291 = {S13, 0, &V23696_291_57182, &P23696_289};	/*Property = CssMediaQueries*/
const Value V23696_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_292 = {S14, 0, &V23696_292_57185, &P23696_291};	/*Property = CssMinMax*/
const Value V23696_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_293 = {S15, 0, &V23696_293_57188, &P23696_292};	/*Property = CssOverflow*/
const Value V23696_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_294 = {S16, 0, &V23696_294_57191, &P23696_293};	/*Property = CssPosition*/
const Value V23696_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_295 = {S17, 0, &V23696_295_57194, &P23696_294};	/*Property = CssText*/
const Value V23696_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_297 = {S18, 0, &V23696_297_57197, &P23696_295};	/*Property = CssTransforms*/
const Value V23696_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_241 = {S19, 0, &V23696_241_55801, &P23696_297};	/*Property = CssTransitions*/
const Value V23696_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_298 = {S20, 0, &V23696_298_57208, &P23696_241};	/*Property = CssUI*/
const Value V23696_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_300 = {S21, 0, &V23696_300_57217, &P23696_298};	/*Property = DataSet*/
const Value V23696_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_301 = {S22, 0, &V23696_301_57220, &P23696_300};	/*Property = DataUrl*/
const Value V23696_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_302 = {S23, 0, &V23696_302_57223, &P23696_301};	/*Property = DeviceOrientation*/
const Value V23696_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_304 = {S24, 0, &V23696_304_57229, &P23696_302};	/*Property = FileReader*/
const Value V23696_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_305 = {S25, 0, &V23696_305_57232, &P23696_304};	/*Property = FileSaver*/
const Value V23696_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_306 = {S26, 0, &V23696_306_57238, &P23696_305};	/*Property = FileWriter*/
const Value V23696_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_307 = {S27, 0, &V23696_307_57241, &P23696_306};	/*Property = FormData*/
const Value V23696_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_309 = {S28, 0, &V23696_309_57247, &P23696_307};	/*Property = Fullscreen*/
const Value V23696_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_310 = {S29, 0, &V23696_310_57250, &P23696_309};	/*Property = GeoLocation*/
const Value V23696_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_312 = {S30, 0, &V23696_312_57256, &P23696_310};	/*Property = History*/
const Value V23696_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_314 = {S31, 0, &V23696_314_57262, &P23696_312};	/*Property = Html5*/
const Value V23696_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_313 = {S32, 0, &V23696_313_57259, &P23696_314};	/*Property = Html-Media-Capture*/
const Value V23696_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_315 = {S33, 0, &V23696_315_57265, &P23696_313};	/*Property = Iframe*/
const Value V23696_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_316 = {S34, 0, &V23696_316_57268, &P23696_315};	/*Property = IndexedDB*/
const Value V23696_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_317 = {S35, 0, &V23696_317_57271, &P23696_316};	/*Property = Json*/
const Value V23696_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P23696_15 = {S37, 0, &V23696_15_29, &P23696_317};	/*Property = LayoutEngine*/
const Value V23696_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_344 = {S38, 0, &V23696_344_57274, &P23696_15};	/*Property = Masking*/
const Value V23696_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_320 = {S39, 0, &V23696_320_57292, &P23696_344};	/*Property = PostMessage*/
const Value V23696_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_321 = {S40, 0, &V23696_321_57295, &P23696_320};	/*Property = Progress*/
const Value V23696_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_322 = {S41, 0, &V23696_322_57298, &P23696_321};	/*Property = Prompts*/
const Value V23696_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_323 = {S42, 0, &V23696_323_57301, &P23696_322};	/*Property = Selector*/
const Value V23696_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_324 = {S43, 0, &V23696_324_57307, &P23696_323};	/*Property = Svg*/
const Value V23696_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_327 = {S44, 0, &V23696_327_57316, &P23696_324};	/*Property = TouchEvents*/
const Value V23696_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_328 = {S45, 0, &V23696_328_57319, &P23696_327};	/*Property = Track*/
const Value V23696_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_329 = {S46, 0, &V23696_329_57325, &P23696_328};	/*Property = Video*/
const Value V23696_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_330 = {S47, 0, &V23696_330_57328, &P23696_329};	/*Property = Viewport*/
const Value V23696_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_332 = {S48, 0, &V23696_332_57340, &P23696_330};	/*Property = WebWorkers*/
const Value V23696_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23696_333 = {S49, 0, &V23696_333_57343, &P23696_332};	/*Property = Xhr2*/
const Profile P23696 = {"23696", &P23696_333};	/*Profile ID - 23696*/
const Value V19875_1_1 = {S2, NULL};	/*Value = True*/
const Property P19875_1 = {S50, 0, &V19875_1_1, NULL};	/*Property = IsMobile*/
const Value V19875_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P19875_5 = {S52, 0, &V19875_5_1277, &P19875_1};	/*Property = ScreenPixelsHeight*/
const Value V19875_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P19875_4 = {S54, 0, &V19875_4_1279, &P19875_5};	/*Property = ScreenPixelsWidth*/
const Profile P19875 = {"19875", &P19875_4};	/*Profile ID - 19875*/
const Value V18507_1_1 = {S2, NULL};	/*Value = True*/
const Property P18507_1 = {S50, 0, &V18507_1_1, NULL};	/*Property = IsMobile*/
const Value V18507_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18507_5 = {S52, 0, &V18507_5_1227, &P18507_1};	/*Property = ScreenPixelsHeight*/
const Value V18507_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18507_4 = {S54, 0, &V18507_4_1229, &P18507_5};	/*Property = ScreenPixelsWidth*/
const Profile P18507 = {"18507", &P18507_4};	/*Profile ID - 18507*/
const Value V17519_1_1 = {S2, NULL};	/*Value = True*/
const Property P17519_1 = {S50, 0, &V17519_1_1, NULL};	/*Property = IsMobile*/
const Value V17519_5_37540 = {S105, NULL};	/*Value = 427*/
const Property P17519_5 = {S52, 0, &V17519_5_37540, &P17519_1};	/*Property = ScreenPixelsHeight*/
const Value V17519_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17519_4 = {S54, 0, &V17519_4_1224, &P17519_5};	/*Property = ScreenPixelsWidth*/
const Profile P17519 = {"17519", &P17519_4};	/*Profile ID - 17519*/
const Value V18563_1_1 = {S2, NULL};	/*Value = True*/
const Property P18563_1 = {S50, 0, &V18563_1_1, NULL};	/*Property = IsMobile*/
const Value V18563_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P18563_5 = {S52, 0, &V18563_5_1275, &P18563_1};	/*Property = ScreenPixelsHeight*/
const Value V18563_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18563_4 = {S54, 0, &V18563_4_1229, &P18563_5};	/*Property = ScreenPixelsWidth*/
const Profile P18563 = {"18563", &P18563_4};	/*Profile ID - 18563*/
const Profile P22198 = {"22198", NULL};	/*Profile ID - 22198*/
const Value V21151_1_1 = {S2, NULL};	/*Value = True*/
const Property P21151_1 = {S50, 0, &V21151_1_1, NULL};	/*Property = IsMobile*/
const Value V21151_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21151_5 = {S52, 0, &V21151_5_1227, &P21151_1};	/*Property = ScreenPixelsHeight*/
const Value V21151_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21151_4 = {S54, 0, &V21151_4_1229, &P21151_5};	/*Property = ScreenPixelsWidth*/
const Profile P21151 = {"21151", &P21151_4};	/*Profile ID - 21151*/
const Value V18402_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_279 = {S1, 0, &V18402_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18402_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_282 = {S3, 0, &V18402_282_57155, &P18402_279};	/*Property = BlobBuilder*/
const Value V18402_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_350 = {S4, 0, &V18402_350_57446, &P18402_282};	/*Property = Canvas*/
const Value V18402_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_283 = {S5, 0, &V18402_283_57161, &P18402_350};	/*Property = CssBackground*/
const Value V18402_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_284 = {S6, 0, &V18402_284_57164, &P18402_283};	/*Property = CssBorderImage*/
const Value V18402_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_285 = {S7, 0, &V18402_285_57167, &P18402_284};	/*Property = CssCanvas*/
const Value V18402_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_286 = {S8, 0, &V18402_286_57170, &P18402_285};	/*Property = CssColor*/
const Value V18402_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_237 = {S9, 0, &V18402_237_55786, &P18402_286};	/*Property = CssColumn*/
const Value V18402_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_287 = {S10, 0, &V18402_287_57173, &P18402_237};	/*Property = CssFlexbox*/
const Value V18402_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_288 = {S11, 0, &V18402_288_57176, &P18402_287};	/*Property = CssFont*/
const Value V18402_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_289 = {S12, 0, &V18402_289_86899, &P18402_288};	/*Property = CssImages*/
const Value V18402_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_291 = {S13, 0, &V18402_291_57182, &P18402_289};	/*Property = CssMediaQueries*/
const Value V18402_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_292 = {S14, 0, &V18402_292_57185, &P18402_291};	/*Property = CssMinMax*/
const Value V18402_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_293 = {S15, 0, &V18402_293_57188, &P18402_292};	/*Property = CssOverflow*/
const Value V18402_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_294 = {S16, 0, &V18402_294_57191, &P18402_293};	/*Property = CssPosition*/
const Value V18402_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_295 = {S17, 0, &V18402_295_57194, &P18402_294};	/*Property = CssText*/
const Value V18402_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_297 = {S18, 0, &V18402_297_57197, &P18402_295};	/*Property = CssTransforms*/
const Value V18402_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_241 = {S19, 0, &V18402_241_55801, &P18402_297};	/*Property = CssTransitions*/
const Value V18402_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_298 = {S20, 0, &V18402_298_57208, &P18402_241};	/*Property = CssUI*/
const Value V18402_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_300 = {S21, 0, &V18402_300_57217, &P18402_298};	/*Property = DataSet*/
const Value V18402_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_301 = {S22, 0, &V18402_301_57220, &P18402_300};	/*Property = DataUrl*/
const Value V18402_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_302 = {S23, 0, &V18402_302_57223, &P18402_301};	/*Property = DeviceOrientation*/
const Value V18402_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_304 = {S24, 0, &V18402_304_57229, &P18402_302};	/*Property = FileReader*/
const Value V18402_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_305 = {S25, 0, &V18402_305_57232, &P18402_304};	/*Property = FileSaver*/
const Value V18402_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_306 = {S26, 0, &V18402_306_57238, &P18402_305};	/*Property = FileWriter*/
const Value V18402_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_307 = {S27, 0, &V18402_307_57241, &P18402_306};	/*Property = FormData*/
const Value V18402_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_309 = {S28, 0, &V18402_309_57247, &P18402_307};	/*Property = Fullscreen*/
const Value V18402_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_310 = {S29, 0, &V18402_310_57250, &P18402_309};	/*Property = GeoLocation*/
const Value V18402_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_312 = {S30, 0, &V18402_312_57256, &P18402_310};	/*Property = History*/
const Value V18402_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_314 = {S31, 0, &V18402_314_57262, &P18402_312};	/*Property = Html5*/
const Value V18402_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_313 = {S32, 0, &V18402_313_57259, &P18402_314};	/*Property = Html-Media-Capture*/
const Value V18402_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_315 = {S33, 0, &V18402_315_57265, &P18402_313};	/*Property = Iframe*/
const Value V18402_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_316 = {S34, 0, &V18402_316_57268, &P18402_315};	/*Property = IndexedDB*/
const Value V18402_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_317 = {S35, 0, &V18402_317_57271, &P18402_316};	/*Property = Json*/
const Value V18402_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18402_15 = {S37, 0, &V18402_15_31, &P18402_317};	/*Property = LayoutEngine*/
const Value V18402_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_344 = {S38, 0, &V18402_344_57274, &P18402_15};	/*Property = Masking*/
const Value V18402_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_320 = {S39, 0, &V18402_320_57292, &P18402_344};	/*Property = PostMessage*/
const Value V18402_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_321 = {S40, 0, &V18402_321_57295, &P18402_320};	/*Property = Progress*/
const Value V18402_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_322 = {S41, 0, &V18402_322_57298, &P18402_321};	/*Property = Prompts*/
const Value V18402_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_323 = {S42, 0, &V18402_323_57301, &P18402_322};	/*Property = Selector*/
const Value V18402_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_324 = {S43, 0, &V18402_324_57307, &P18402_323};	/*Property = Svg*/
const Value V18402_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_327 = {S44, 0, &V18402_327_57316, &P18402_324};	/*Property = TouchEvents*/
const Value V18402_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_328 = {S45, 0, &V18402_328_57319, &P18402_327};	/*Property = Track*/
const Value V18402_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_329 = {S46, 0, &V18402_329_57325, &P18402_328};	/*Property = Video*/
const Value V18402_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_330 = {S47, 0, &V18402_330_57328, &P18402_329};	/*Property = Viewport*/
const Value V18402_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_332 = {S48, 0, &V18402_332_57340, &P18402_330};	/*Property = WebWorkers*/
const Value V18402_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18402_333 = {S49, 0, &V18402_333_57343, &P18402_332};	/*Property = Xhr2*/
const Profile P18402 = {"18402", &P18402_333};	/*Profile ID - 18402*/
const Value V18714_1_1 = {S2, NULL};	/*Value = True*/
const Property P18714_1 = {S50, 0, &V18714_1_1, NULL};	/*Property = IsMobile*/
const Value V18714_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18714_5 = {S52, 0, &V18714_5_1227, &P18714_1};	/*Property = ScreenPixelsHeight*/
const Value V18714_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18714_4 = {S54, 0, &V18714_4_1229, &P18714_5};	/*Property = ScreenPixelsWidth*/
const Profile P18714 = {"18714", &P18714_4};	/*Profile ID - 18714*/
const Value V20219_1_1 = {S2, NULL};	/*Value = True*/
const Property P20219_1 = {S50, 0, &V20219_1_1, NULL};	/*Property = IsMobile*/
const Value V20219_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20219_5 = {S52, 0, &V20219_5_1458, &P20219_1};	/*Property = ScreenPixelsHeight*/
const Value V20219_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20219_4 = {S54, 0, &V20219_4_37617, &P20219_5};	/*Property = ScreenPixelsWidth*/
const Profile P20219 = {"20219", &P20219_4};	/*Profile ID - 20219*/
const Value V23992_1_1 = {S2, NULL};	/*Value = True*/
const Property P23992_1 = {S50, 0, &V23992_1_1, NULL};	/*Property = IsMobile*/
const Value V23992_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23992_5 = {S52, 0, &V23992_5_1277, &P23992_1};	/*Property = ScreenPixelsHeight*/
const Value V23992_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23992_4 = {S54, 0, &V23992_4_2336, &P23992_5};	/*Property = ScreenPixelsWidth*/
const Profile P23992 = {"23992", &P23992_4};	/*Profile ID - 23992*/
const Value V22010_1_1 = {S2, NULL};	/*Value = True*/
const Property P22010_1 = {S50, 0, &V22010_1_1, NULL};	/*Property = IsMobile*/
const Value V22010_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22010_5 = {S52, 0, &V22010_5_1277, &P22010_1};	/*Property = ScreenPixelsHeight*/
const Value V22010_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22010_4 = {S54, 0, &V22010_4_2336, &P22010_5};	/*Property = ScreenPixelsWidth*/
const Profile P22010 = {"22010", &P22010_4};	/*Profile ID - 22010*/
const Value V21376_1_1 = {S2, NULL};	/*Value = True*/
const Property P21376_1 = {S50, 0, &V21376_1_1, NULL};	/*Property = IsMobile*/
const Value V21376_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21376_5 = {S52, 0, &V21376_5_1227, &P21376_1};	/*Property = ScreenPixelsHeight*/
const Value V21376_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21376_4 = {S54, 0, &V21376_4_1229, &P21376_5};	/*Property = ScreenPixelsWidth*/
const Profile P21376 = {"21376", &P21376_4};	/*Profile ID - 21376*/
const Value V14628_1_1 = {S2, NULL};	/*Value = True*/
const Property P14628_1 = {S50, 0, &V14628_1_1, NULL};	/*Property = IsMobile*/
const Value V14628_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P14628_5 = {S52, 0, &V14628_5_1304, &P14628_1};	/*Property = ScreenPixelsHeight*/
const Value V14628_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P14628_4 = {S54, 0, &V14628_4_1224, &P14628_5};	/*Property = ScreenPixelsWidth*/
const Profile P14628 = {"14628", &P14628_4};	/*Profile ID - 14628*/
const Value V17599_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_279 = {S1, 0, &V17599_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17599_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_282 = {S3, 0, &V17599_282_57155, &P17599_279};	/*Property = BlobBuilder*/
const Value V17599_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_350 = {S4, 0, &V17599_350_57446, &P17599_282};	/*Property = Canvas*/
const Value V17599_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_283 = {S5, 0, &V17599_283_57161, &P17599_350};	/*Property = CssBackground*/
const Value V17599_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_284 = {S6, 0, &V17599_284_57164, &P17599_283};	/*Property = CssBorderImage*/
const Value V17599_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_285 = {S7, 0, &V17599_285_57167, &P17599_284};	/*Property = CssCanvas*/
const Value V17599_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_286 = {S8, 0, &V17599_286_57170, &P17599_285};	/*Property = CssColor*/
const Value V17599_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_237 = {S9, 0, &V17599_237_55786, &P17599_286};	/*Property = CssColumn*/
const Value V17599_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_287 = {S10, 0, &V17599_287_57173, &P17599_237};	/*Property = CssFlexbox*/
const Value V17599_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_288 = {S11, 0, &V17599_288_57176, &P17599_287};	/*Property = CssFont*/
const Value V17599_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_289 = {S12, 0, &V17599_289_86899, &P17599_288};	/*Property = CssImages*/
const Value V17599_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_291 = {S13, 0, &V17599_291_57182, &P17599_289};	/*Property = CssMediaQueries*/
const Value V17599_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_292 = {S14, 0, &V17599_292_57185, &P17599_291};	/*Property = CssMinMax*/
const Value V17599_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_293 = {S15, 0, &V17599_293_57188, &P17599_292};	/*Property = CssOverflow*/
const Value V17599_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_294 = {S16, 0, &V17599_294_57191, &P17599_293};	/*Property = CssPosition*/
const Value V17599_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_295 = {S17, 0, &V17599_295_57194, &P17599_294};	/*Property = CssText*/
const Value V17599_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_297 = {S18, 0, &V17599_297_57197, &P17599_295};	/*Property = CssTransforms*/
const Value V17599_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_241 = {S19, 0, &V17599_241_55801, &P17599_297};	/*Property = CssTransitions*/
const Value V17599_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_298 = {S20, 0, &V17599_298_57208, &P17599_241};	/*Property = CssUI*/
const Value V17599_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_300 = {S21, 0, &V17599_300_57217, &P17599_298};	/*Property = DataSet*/
const Value V17599_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_301 = {S22, 0, &V17599_301_57220, &P17599_300};	/*Property = DataUrl*/
const Value V17599_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_302 = {S23, 0, &V17599_302_57223, &P17599_301};	/*Property = DeviceOrientation*/
const Value V17599_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_304 = {S24, 0, &V17599_304_57229, &P17599_302};	/*Property = FileReader*/
const Value V17599_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_305 = {S25, 0, &V17599_305_57232, &P17599_304};	/*Property = FileSaver*/
const Value V17599_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_306 = {S26, 0, &V17599_306_57238, &P17599_305};	/*Property = FileWriter*/
const Value V17599_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_307 = {S27, 0, &V17599_307_57241, &P17599_306};	/*Property = FormData*/
const Value V17599_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_309 = {S28, 0, &V17599_309_57247, &P17599_307};	/*Property = Fullscreen*/
const Value V17599_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_310 = {S29, 0, &V17599_310_57250, &P17599_309};	/*Property = GeoLocation*/
const Value V17599_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_312 = {S30, 0, &V17599_312_57256, &P17599_310};	/*Property = History*/
const Value V17599_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_314 = {S31, 0, &V17599_314_57262, &P17599_312};	/*Property = Html5*/
const Value V17599_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_313 = {S32, 0, &V17599_313_57259, &P17599_314};	/*Property = Html-Media-Capture*/
const Value V17599_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_315 = {S33, 0, &V17599_315_57265, &P17599_313};	/*Property = Iframe*/
const Value V17599_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_316 = {S34, 0, &V17599_316_57268, &P17599_315};	/*Property = IndexedDB*/
const Value V17599_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_317 = {S35, 0, &V17599_317_57271, &P17599_316};	/*Property = Json*/
const Value V17599_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P17599_15 = {S37, 0, &V17599_15_29, &P17599_317};	/*Property = LayoutEngine*/
const Value V17599_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_344 = {S38, 0, &V17599_344_57274, &P17599_15};	/*Property = Masking*/
const Value V17599_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_320 = {S39, 0, &V17599_320_57292, &P17599_344};	/*Property = PostMessage*/
const Value V17599_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_321 = {S40, 0, &V17599_321_57295, &P17599_320};	/*Property = Progress*/
const Value V17599_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_322 = {S41, 0, &V17599_322_57298, &P17599_321};	/*Property = Prompts*/
const Value V17599_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_323 = {S42, 0, &V17599_323_57301, &P17599_322};	/*Property = Selector*/
const Value V17599_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_324 = {S43, 0, &V17599_324_57307, &P17599_323};	/*Property = Svg*/
const Value V17599_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_327 = {S44, 0, &V17599_327_57316, &P17599_324};	/*Property = TouchEvents*/
const Value V17599_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_328 = {S45, 0, &V17599_328_57319, &P17599_327};	/*Property = Track*/
const Value V17599_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_329 = {S46, 0, &V17599_329_57325, &P17599_328};	/*Property = Video*/
const Value V17599_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_330 = {S47, 0, &V17599_330_57328, &P17599_329};	/*Property = Viewport*/
const Value V17599_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_332 = {S48, 0, &V17599_332_57340, &P17599_330};	/*Property = WebWorkers*/
const Value V17599_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17599_333 = {S49, 0, &V17599_333_57343, &P17599_332};	/*Property = Xhr2*/
const Profile P17599 = {"17599", &P17599_333};	/*Profile ID - 17599*/
const Value V18729_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_279 = {S1, 0, &V18729_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18729_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_282 = {S3, 0, &V18729_282_57155, &P18729_279};	/*Property = BlobBuilder*/
const Value V18729_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_350 = {S4, 0, &V18729_350_57446, &P18729_282};	/*Property = Canvas*/
const Value V18729_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_283 = {S5, 0, &V18729_283_57161, &P18729_350};	/*Property = CssBackground*/
const Value V18729_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_284 = {S6, 0, &V18729_284_57164, &P18729_283};	/*Property = CssBorderImage*/
const Value V18729_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_285 = {S7, 0, &V18729_285_57167, &P18729_284};	/*Property = CssCanvas*/
const Value V18729_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_286 = {S8, 0, &V18729_286_57170, &P18729_285};	/*Property = CssColor*/
const Value V18729_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_237 = {S9, 0, &V18729_237_55786, &P18729_286};	/*Property = CssColumn*/
const Value V18729_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_287 = {S10, 0, &V18729_287_57173, &P18729_237};	/*Property = CssFlexbox*/
const Value V18729_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_288 = {S11, 0, &V18729_288_57176, &P18729_287};	/*Property = CssFont*/
const Value V18729_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_289 = {S12, 0, &V18729_289_86899, &P18729_288};	/*Property = CssImages*/
const Value V18729_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_291 = {S13, 0, &V18729_291_57182, &P18729_289};	/*Property = CssMediaQueries*/
const Value V18729_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_292 = {S14, 0, &V18729_292_57185, &P18729_291};	/*Property = CssMinMax*/
const Value V18729_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_293 = {S15, 0, &V18729_293_57188, &P18729_292};	/*Property = CssOverflow*/
const Value V18729_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_294 = {S16, 0, &V18729_294_57191, &P18729_293};	/*Property = CssPosition*/
const Value V18729_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_295 = {S17, 0, &V18729_295_57194, &P18729_294};	/*Property = CssText*/
const Value V18729_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_297 = {S18, 0, &V18729_297_57197, &P18729_295};	/*Property = CssTransforms*/
const Value V18729_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_241 = {S19, 0, &V18729_241_55801, &P18729_297};	/*Property = CssTransitions*/
const Value V18729_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_298 = {S20, 0, &V18729_298_57208, &P18729_241};	/*Property = CssUI*/
const Value V18729_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_300 = {S21, 0, &V18729_300_57217, &P18729_298};	/*Property = DataSet*/
const Value V18729_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_301 = {S22, 0, &V18729_301_57220, &P18729_300};	/*Property = DataUrl*/
const Value V18729_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_302 = {S23, 0, &V18729_302_57223, &P18729_301};	/*Property = DeviceOrientation*/
const Value V18729_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_304 = {S24, 0, &V18729_304_57229, &P18729_302};	/*Property = FileReader*/
const Value V18729_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_305 = {S25, 0, &V18729_305_57232, &P18729_304};	/*Property = FileSaver*/
const Value V18729_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_306 = {S26, 0, &V18729_306_57238, &P18729_305};	/*Property = FileWriter*/
const Value V18729_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_307 = {S27, 0, &V18729_307_57241, &P18729_306};	/*Property = FormData*/
const Value V18729_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_309 = {S28, 0, &V18729_309_57247, &P18729_307};	/*Property = Fullscreen*/
const Value V18729_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_310 = {S29, 0, &V18729_310_57250, &P18729_309};	/*Property = GeoLocation*/
const Value V18729_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_312 = {S30, 0, &V18729_312_57256, &P18729_310};	/*Property = History*/
const Value V18729_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_314 = {S31, 0, &V18729_314_57262, &P18729_312};	/*Property = Html5*/
const Value V18729_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_313 = {S32, 0, &V18729_313_57259, &P18729_314};	/*Property = Html-Media-Capture*/
const Value V18729_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_315 = {S33, 0, &V18729_315_57265, &P18729_313};	/*Property = Iframe*/
const Value V18729_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_316 = {S34, 0, &V18729_316_57268, &P18729_315};	/*Property = IndexedDB*/
const Value V18729_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_317 = {S35, 0, &V18729_317_57271, &P18729_316};	/*Property = Json*/
const Value V18729_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18729_15 = {S37, 0, &V18729_15_29, &P18729_317};	/*Property = LayoutEngine*/
const Value V18729_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_344 = {S38, 0, &V18729_344_57274, &P18729_15};	/*Property = Masking*/
const Value V18729_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_320 = {S39, 0, &V18729_320_57292, &P18729_344};	/*Property = PostMessage*/
const Value V18729_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_321 = {S40, 0, &V18729_321_57295, &P18729_320};	/*Property = Progress*/
const Value V18729_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_322 = {S41, 0, &V18729_322_57298, &P18729_321};	/*Property = Prompts*/
const Value V18729_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_323 = {S42, 0, &V18729_323_57301, &P18729_322};	/*Property = Selector*/
const Value V18729_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_324 = {S43, 0, &V18729_324_57307, &P18729_323};	/*Property = Svg*/
const Value V18729_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_327 = {S44, 0, &V18729_327_57316, &P18729_324};	/*Property = TouchEvents*/
const Value V18729_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_328 = {S45, 0, &V18729_328_57319, &P18729_327};	/*Property = Track*/
const Value V18729_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_329 = {S46, 0, &V18729_329_57325, &P18729_328};	/*Property = Video*/
const Value V18729_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_330 = {S47, 0, &V18729_330_57328, &P18729_329};	/*Property = Viewport*/
const Value V18729_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_332 = {S48, 0, &V18729_332_57340, &P18729_330};	/*Property = WebWorkers*/
const Value V18729_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18729_333 = {S49, 0, &V18729_333_57343, &P18729_332};	/*Property = Xhr2*/
const Profile P18729 = {"18729", &P18729_333};	/*Profile ID - 18729*/
const Value V19899_1_1 = {S2, NULL};	/*Value = True*/
const Property P19899_1 = {S50, 0, &V19899_1_1, NULL};	/*Property = IsMobile*/
const Value V19899_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P19899_5 = {S52, 0, &V19899_5_1222, &P19899_1};	/*Property = ScreenPixelsHeight*/
const Value V19899_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P19899_4 = {S54, 0, &V19899_4_1224, &P19899_5};	/*Property = ScreenPixelsWidth*/
const Profile P19899 = {"19899", &P19899_4};	/*Profile ID - 19899*/
const Value V18582_1_1 = {S2, NULL};	/*Value = True*/
const Property P18582_1 = {S50, 0, &V18582_1_1, NULL};	/*Property = IsMobile*/
const Value V18582_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18582_5 = {S52, 0, &V18582_5_9981, &P18582_1};	/*Property = ScreenPixelsHeight*/
const Value V18582_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18582_4 = {S54, 0, &V18582_4_39098, &P18582_5};	/*Property = ScreenPixelsWidth*/
const Profile P18582 = {"18582", &P18582_4};	/*Profile ID - 18582*/
const Value V17372_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_279 = {S1, 0, &V17372_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17372_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_282 = {S3, 0, &V17372_282_57155, &P17372_279};	/*Property = BlobBuilder*/
const Value V17372_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_350 = {S4, 0, &V17372_350_57446, &P17372_282};	/*Property = Canvas*/
const Value V17372_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_283 = {S5, 0, &V17372_283_57161, &P17372_350};	/*Property = CssBackground*/
const Value V17372_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_284 = {S6, 0, &V17372_284_57164, &P17372_283};	/*Property = CssBorderImage*/
const Value V17372_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_285 = {S7, 0, &V17372_285_57167, &P17372_284};	/*Property = CssCanvas*/
const Value V17372_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_286 = {S8, 0, &V17372_286_57170, &P17372_285};	/*Property = CssColor*/
const Value V17372_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_237 = {S9, 0, &V17372_237_55786, &P17372_286};	/*Property = CssColumn*/
const Value V17372_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_287 = {S10, 0, &V17372_287_57173, &P17372_237};	/*Property = CssFlexbox*/
const Value V17372_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_288 = {S11, 0, &V17372_288_57176, &P17372_287};	/*Property = CssFont*/
const Value V17372_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_289 = {S12, 0, &V17372_289_86899, &P17372_288};	/*Property = CssImages*/
const Value V17372_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_291 = {S13, 0, &V17372_291_57182, &P17372_289};	/*Property = CssMediaQueries*/
const Value V17372_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_292 = {S14, 0, &V17372_292_57185, &P17372_291};	/*Property = CssMinMax*/
const Value V17372_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_293 = {S15, 0, &V17372_293_57188, &P17372_292};	/*Property = CssOverflow*/
const Value V17372_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_294 = {S16, 0, &V17372_294_57191, &P17372_293};	/*Property = CssPosition*/
const Value V17372_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_295 = {S17, 0, &V17372_295_57194, &P17372_294};	/*Property = CssText*/
const Value V17372_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_297 = {S18, 0, &V17372_297_57197, &P17372_295};	/*Property = CssTransforms*/
const Value V17372_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_241 = {S19, 0, &V17372_241_55801, &P17372_297};	/*Property = CssTransitions*/
const Value V17372_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_298 = {S20, 0, &V17372_298_57208, &P17372_241};	/*Property = CssUI*/
const Value V17372_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_300 = {S21, 0, &V17372_300_57217, &P17372_298};	/*Property = DataSet*/
const Value V17372_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_301 = {S22, 0, &V17372_301_57220, &P17372_300};	/*Property = DataUrl*/
const Value V17372_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_302 = {S23, 0, &V17372_302_57223, &P17372_301};	/*Property = DeviceOrientation*/
const Value V17372_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_304 = {S24, 0, &V17372_304_57229, &P17372_302};	/*Property = FileReader*/
const Value V17372_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_305 = {S25, 0, &V17372_305_57232, &P17372_304};	/*Property = FileSaver*/
const Value V17372_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_306 = {S26, 0, &V17372_306_57238, &P17372_305};	/*Property = FileWriter*/
const Value V17372_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_307 = {S27, 0, &V17372_307_57241, &P17372_306};	/*Property = FormData*/
const Value V17372_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_309 = {S28, 0, &V17372_309_57247, &P17372_307};	/*Property = Fullscreen*/
const Value V17372_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_310 = {S29, 0, &V17372_310_57250, &P17372_309};	/*Property = GeoLocation*/
const Value V17372_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_312 = {S30, 0, &V17372_312_57256, &P17372_310};	/*Property = History*/
const Value V17372_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_314 = {S31, 0, &V17372_314_57262, &P17372_312};	/*Property = Html5*/
const Value V17372_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_313 = {S32, 0, &V17372_313_57259, &P17372_314};	/*Property = Html-Media-Capture*/
const Value V17372_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_315 = {S33, 0, &V17372_315_57265, &P17372_313};	/*Property = Iframe*/
const Value V17372_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_316 = {S34, 0, &V17372_316_57268, &P17372_315};	/*Property = IndexedDB*/
const Value V17372_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_317 = {S35, 0, &V17372_317_57271, &P17372_316};	/*Property = Json*/
const Value V17372_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P17372_15 = {S37, 0, &V17372_15_31, &P17372_317};	/*Property = LayoutEngine*/
const Value V17372_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_344 = {S38, 0, &V17372_344_57274, &P17372_15};	/*Property = Masking*/
const Value V17372_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_320 = {S39, 0, &V17372_320_57292, &P17372_344};	/*Property = PostMessage*/
const Value V17372_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_321 = {S40, 0, &V17372_321_57295, &P17372_320};	/*Property = Progress*/
const Value V17372_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_322 = {S41, 0, &V17372_322_57298, &P17372_321};	/*Property = Prompts*/
const Value V17372_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_323 = {S42, 0, &V17372_323_57301, &P17372_322};	/*Property = Selector*/
const Value V17372_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_324 = {S43, 0, &V17372_324_57307, &P17372_323};	/*Property = Svg*/
const Value V17372_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_327 = {S44, 0, &V17372_327_57316, &P17372_324};	/*Property = TouchEvents*/
const Value V17372_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_328 = {S45, 0, &V17372_328_57319, &P17372_327};	/*Property = Track*/
const Value V17372_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_329 = {S46, 0, &V17372_329_57325, &P17372_328};	/*Property = Video*/
const Value V17372_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_330 = {S47, 0, &V17372_330_57328, &P17372_329};	/*Property = Viewport*/
const Value V17372_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_332 = {S48, 0, &V17372_332_57340, &P17372_330};	/*Property = WebWorkers*/
const Value V17372_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17372_333 = {S49, 0, &V17372_333_57343, &P17372_332};	/*Property = Xhr2*/
const Profile P17372 = {"17372", &P17372_333};	/*Profile ID - 17372*/
const Value V21568_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_279 = {S1, 0, &V21568_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21568_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_282 = {S3, 0, &V21568_282_57155, &P21568_279};	/*Property = BlobBuilder*/
const Value V21568_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_350 = {S4, 0, &V21568_350_57446, &P21568_282};	/*Property = Canvas*/
const Value V21568_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_283 = {S5, 0, &V21568_283_57161, &P21568_350};	/*Property = CssBackground*/
const Value V21568_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_284 = {S6, 0, &V21568_284_57164, &P21568_283};	/*Property = CssBorderImage*/
const Value V21568_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_285 = {S7, 0, &V21568_285_57167, &P21568_284};	/*Property = CssCanvas*/
const Value V21568_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_286 = {S8, 0, &V21568_286_57170, &P21568_285};	/*Property = CssColor*/
const Value V21568_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_237 = {S9, 0, &V21568_237_55786, &P21568_286};	/*Property = CssColumn*/
const Value V21568_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_287 = {S10, 0, &V21568_287_57173, &P21568_237};	/*Property = CssFlexbox*/
const Value V21568_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_288 = {S11, 0, &V21568_288_57176, &P21568_287};	/*Property = CssFont*/
const Value V21568_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_289 = {S12, 0, &V21568_289_86899, &P21568_288};	/*Property = CssImages*/
const Value V21568_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_291 = {S13, 0, &V21568_291_57182, &P21568_289};	/*Property = CssMediaQueries*/
const Value V21568_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_292 = {S14, 0, &V21568_292_57185, &P21568_291};	/*Property = CssMinMax*/
const Value V21568_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_293 = {S15, 0, &V21568_293_57188, &P21568_292};	/*Property = CssOverflow*/
const Value V21568_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_294 = {S16, 0, &V21568_294_57191, &P21568_293};	/*Property = CssPosition*/
const Value V21568_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_295 = {S17, 0, &V21568_295_57194, &P21568_294};	/*Property = CssText*/
const Value V21568_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_297 = {S18, 0, &V21568_297_57197, &P21568_295};	/*Property = CssTransforms*/
const Value V21568_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_241 = {S19, 0, &V21568_241_55801, &P21568_297};	/*Property = CssTransitions*/
const Value V21568_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_298 = {S20, 0, &V21568_298_57208, &P21568_241};	/*Property = CssUI*/
const Value V21568_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_300 = {S21, 0, &V21568_300_57217, &P21568_298};	/*Property = DataSet*/
const Value V21568_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_301 = {S22, 0, &V21568_301_57220, &P21568_300};	/*Property = DataUrl*/
const Value V21568_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_302 = {S23, 0, &V21568_302_57223, &P21568_301};	/*Property = DeviceOrientation*/
const Value V21568_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_304 = {S24, 0, &V21568_304_57229, &P21568_302};	/*Property = FileReader*/
const Value V21568_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_305 = {S25, 0, &V21568_305_57232, &P21568_304};	/*Property = FileSaver*/
const Value V21568_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_306 = {S26, 0, &V21568_306_57238, &P21568_305};	/*Property = FileWriter*/
const Value V21568_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_307 = {S27, 0, &V21568_307_57241, &P21568_306};	/*Property = FormData*/
const Value V21568_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_309 = {S28, 0, &V21568_309_57247, &P21568_307};	/*Property = Fullscreen*/
const Value V21568_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_310 = {S29, 0, &V21568_310_57250, &P21568_309};	/*Property = GeoLocation*/
const Value V21568_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_312 = {S30, 0, &V21568_312_57256, &P21568_310};	/*Property = History*/
const Value V21568_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_314 = {S31, 0, &V21568_314_57262, &P21568_312};	/*Property = Html5*/
const Value V21568_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_313 = {S32, 0, &V21568_313_57259, &P21568_314};	/*Property = Html-Media-Capture*/
const Value V21568_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_315 = {S33, 0, &V21568_315_57265, &P21568_313};	/*Property = Iframe*/
const Value V21568_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_316 = {S34, 0, &V21568_316_57268, &P21568_315};	/*Property = IndexedDB*/
const Value V21568_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_317 = {S35, 0, &V21568_317_57271, &P21568_316};	/*Property = Json*/
const Value V21568_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_15 = {S37, 0, &V21568_15_72, &P21568_317};	/*Property = LayoutEngine*/
const Value V21568_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_344 = {S38, 0, &V21568_344_57274, &P21568_15};	/*Property = Masking*/
const Value V21568_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_320 = {S39, 0, &V21568_320_57292, &P21568_344};	/*Property = PostMessage*/
const Value V21568_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_321 = {S40, 0, &V21568_321_57295, &P21568_320};	/*Property = Progress*/
const Value V21568_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_322 = {S41, 0, &V21568_322_57298, &P21568_321};	/*Property = Prompts*/
const Value V21568_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_323 = {S42, 0, &V21568_323_57301, &P21568_322};	/*Property = Selector*/
const Value V21568_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_324 = {S43, 0, &V21568_324_57307, &P21568_323};	/*Property = Svg*/
const Value V21568_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_327 = {S44, 0, &V21568_327_57316, &P21568_324};	/*Property = TouchEvents*/
const Value V21568_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_328 = {S45, 0, &V21568_328_57319, &P21568_327};	/*Property = Track*/
const Value V21568_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_329 = {S46, 0, &V21568_329_57325, &P21568_328};	/*Property = Video*/
const Value V21568_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_330 = {S47, 0, &V21568_330_57328, &P21568_329};	/*Property = Viewport*/
const Value V21568_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_332 = {S48, 0, &V21568_332_57340, &P21568_330};	/*Property = WebWorkers*/
const Value V21568_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21568_333 = {S49, 0, &V21568_333_57343, &P21568_332};	/*Property = Xhr2*/
const Profile P21568 = {"21568", &P21568_333};	/*Profile ID - 21568*/
const Value V18662_1_1 = {S2, NULL};	/*Value = True*/
const Property P18662_1 = {S50, 0, &V18662_1_1, NULL};	/*Property = IsMobile*/
const Value V18662_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18662_5 = {S52, 0, &V18662_5_1403, &P18662_1};	/*Property = ScreenPixelsHeight*/
const Value V18662_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18662_4 = {S54, 0, &V18662_4_1404, &P18662_5};	/*Property = ScreenPixelsWidth*/
const Profile P18662 = {"18662", &P18662_4};	/*Profile ID - 18662*/
const Value V22528_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22528_279 = {S1, 0, &V22528_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22528_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22528_282 = {S3, 0, &V22528_282_57154, &P22528_279};	/*Property = BlobBuilder*/
const Value V22528_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22528_350 = {S4, 0, &V22528_350_57444, &P22528_282};	/*Property = Canvas*/
const Value V22528_283_57160 = {S0, NULL};	/*Value = False*/
const Property P22528_283 = {S5, 0, &V22528_283_57160, &P22528_350};	/*Property = CssBackground*/
const Value V22528_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22528_284 = {S6, 0, &V22528_284_57163, &P22528_283};	/*Property = CssBorderImage*/
const Value V22528_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22528_285 = {S7, 0, &V22528_285_57166, &P22528_284};	/*Property = CssCanvas*/
const Value V22528_286_57169 = {S0, NULL};	/*Value = False*/
const Property P22528_286 = {S8, 0, &V22528_286_57169, &P22528_285};	/*Property = CssColor*/
const Value V22528_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22528_237 = {S9, 0, &V22528_237_55786, &P22528_286};	/*Property = CssColumn*/
const Value V22528_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22528_287 = {S10, 0, &V22528_287_57172, &P22528_237};	/*Property = CssFlexbox*/
const Value V22528_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22528_288 = {S11, 0, &V22528_288_57175, &P22528_287};	/*Property = CssFont*/
const Value V22528_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22528_289 = {S12, 0, &V22528_289_57759, &P22528_288};	/*Property = CssImages*/
const Value V22528_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22528_291 = {S13, 0, &V22528_291_57181, &P22528_289};	/*Property = CssMediaQueries*/
const Value V22528_292_57184 = {S0, NULL};	/*Value = False*/
const Property P22528_292 = {S14, 0, &V22528_292_57184, &P22528_291};	/*Property = CssMinMax*/
const Value V22528_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22528_293 = {S15, 0, &V22528_293_57187, &P22528_292};	/*Property = CssOverflow*/
const Value V22528_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22528_294 = {S16, 0, &V22528_294_57190, &P22528_293};	/*Property = CssPosition*/
const Value V22528_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22528_295 = {S17, 0, &V22528_295_57193, &P22528_294};	/*Property = CssText*/
const Value V22528_297_57196 = {S0, NULL};	/*Value = False*/
const Property P22528_297 = {S18, 0, &V22528_297_57196, &P22528_295};	/*Property = CssTransforms*/
const Value V22528_241_55800 = {S0, NULL};	/*Value = False*/
const Property P22528_241 = {S19, 0, &V22528_241_55800, &P22528_297};	/*Property = CssTransitions*/
const Value V22528_298_57207 = {S0, NULL};	/*Value = False*/
const Property P22528_298 = {S20, 0, &V22528_298_57207, &P22528_241};	/*Property = CssUI*/
const Value V22528_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22528_300 = {S21, 0, &V22528_300_57216, &P22528_298};	/*Property = DataSet*/
const Value V22528_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22528_301 = {S22, 0, &V22528_301_57218, &P22528_300};	/*Property = DataUrl*/
const Value V22528_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22528_302 = {S23, 0, &V22528_302_57222, &P22528_301};	/*Property = DeviceOrientation*/
const Value V22528_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22528_304 = {S24, 0, &V22528_304_57228, &P22528_302};	/*Property = FileReader*/
const Value V22528_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22528_305 = {S25, 0, &V22528_305_57231, &P22528_304};	/*Property = FileSaver*/
const Value V22528_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22528_306 = {S26, 0, &V22528_306_57237, &P22528_305};	/*Property = FileWriter*/
const Value V22528_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22528_307 = {S27, 0, &V22528_307_57240, &P22528_306};	/*Property = FormData*/
const Value V22528_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22528_309 = {S28, 0, &V22528_309_57246, &P22528_307};	/*Property = Fullscreen*/
const Value V22528_310_57249 = {S0, NULL};	/*Value = False*/
const Property P22528_310 = {S29, 0, &V22528_310_57249, &P22528_309};	/*Property = GeoLocation*/
const Value V22528_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22528_312 = {S30, 0, &V22528_312_57255, &P22528_310};	/*Property = History*/
const Value V22528_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22528_314 = {S31, 0, &V22528_314_57261, &P22528_312};	/*Property = Html5*/
const Value V22528_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22528_313 = {S32, 0, &V22528_313_57258, &P22528_314};	/*Property = Html-Media-Capture*/
const Value V22528_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22528_315 = {S33, 0, &V22528_315_57264, &P22528_313};	/*Property = Iframe*/
const Value V22528_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22528_316 = {S34, 0, &V22528_316_57267, &P22528_315};	/*Property = IndexedDB*/
const Value V22528_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22528_317 = {S35, 0, &V22528_317_57269, &P22528_316};	/*Property = Json*/
const Value V22528_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22528_15 = {S37, 0, &V22528_15_28, &P22528_317};	/*Property = LayoutEngine*/
const Value V22528_344_57273 = {S0, NULL};	/*Value = False*/
const Property P22528_344 = {S38, 0, &V22528_344_57273, &P22528_15};	/*Property = Masking*/
const Value V22528_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22528_320 = {S39, 0, &V22528_320_57290, &P22528_344};	/*Property = PostMessage*/
const Value V22528_321_57294 = {S0, NULL};	/*Value = False*/
const Property P22528_321 = {S40, 0, &V22528_321_57294, &P22528_320};	/*Property = Progress*/
const Value V22528_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22528_322 = {S41, 0, &V22528_322_57296, &P22528_321};	/*Property = Prompts*/
const Value V22528_323_57300 = {S0, NULL};	/*Value = False*/
const Property P22528_323 = {S42, 0, &V22528_323_57300, &P22528_322};	/*Property = Selector*/
const Value V22528_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22528_324 = {S43, 0, &V22528_324_57306, &P22528_323};	/*Property = Svg*/
const Value V22528_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22528_327 = {S44, 0, &V22528_327_57315, &P22528_324};	/*Property = TouchEvents*/
const Value V22528_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22528_328 = {S45, 0, &V22528_328_57318, &P22528_327};	/*Property = Track*/
const Value V22528_329_57324 = {S0, NULL};	/*Value = False*/
const Property P22528_329 = {S46, 0, &V22528_329_57324, &P22528_328};	/*Property = Video*/
const Value V22528_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22528_330 = {S47, 0, &V22528_330_57326, &P22528_329};	/*Property = Viewport*/
const Value V22528_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22528_332 = {S48, 0, &V22528_332_57339, &P22528_330};	/*Property = WebWorkers*/
const Value V22528_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22528_333 = {S49, 0, &V22528_333_57342, &P22528_332};	/*Property = Xhr2*/
const Profile P22528 = {"22528", &P22528_333};	/*Profile ID - 22528*/
const Value V21866_1_1 = {S2, NULL};	/*Value = True*/
const Property P21866_1 = {S50, 0, &V21866_1_1, NULL};	/*Property = IsMobile*/
const Value V21866_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21866_5 = {S52, 0, &V21866_5_1458, &P21866_1};	/*Property = ScreenPixelsHeight*/
const Value V21866_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21866_4 = {S54, 0, &V21866_4_1460, &P21866_5};	/*Property = ScreenPixelsWidth*/
const Profile P21866 = {"21866", &P21866_4};	/*Profile ID - 21866*/
const Value V21223_1_1 = {S2, NULL};	/*Value = True*/
const Property P21223_1 = {S50, 0, &V21223_1_1, NULL};	/*Property = IsMobile*/
const Value V21223_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21223_5 = {S52, 0, &V21223_5_1227, &P21223_1};	/*Property = ScreenPixelsHeight*/
const Value V21223_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21223_4 = {S54, 0, &V21223_4_1229, &P21223_5};	/*Property = ScreenPixelsWidth*/
const Profile P21223 = {"21223", &P21223_4};	/*Profile ID - 21223*/
const Value V21990_1_1 = {S2, NULL};	/*Value = True*/
const Property P21990_1 = {S50, 0, &V21990_1_1, NULL};	/*Property = IsMobile*/
const Value V21990_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21990_5 = {S52, 0, &V21990_5_1227, &P21990_1};	/*Property = ScreenPixelsHeight*/
const Value V21990_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P21990_4 = {S54, 0, &V21990_4_1477, &P21990_5};	/*Property = ScreenPixelsWidth*/
const Profile P21990 = {"21990", &P21990_4};	/*Profile ID - 21990*/
const Value V22086_1_1 = {S2, NULL};	/*Value = True*/
const Property P22086_1 = {S50, 0, &V22086_1_1, NULL};	/*Property = IsMobile*/
const Value V22086_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22086_5 = {S52, 0, &V22086_5_37569, &P22086_1};	/*Property = ScreenPixelsHeight*/
const Value V22086_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22086_4 = {S54, 0, &V22086_4_5645, &P22086_5};	/*Property = ScreenPixelsWidth*/
const Profile P22086 = {"22086", &P22086_4};	/*Profile ID - 22086*/
const Value V19977_1_1 = {S2, NULL};	/*Value = True*/
const Property P19977_1 = {S50, 0, &V19977_1_1, NULL};	/*Property = IsMobile*/
const Value V19977_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19977_5 = {S52, 0, &V19977_5_1227, &P19977_1};	/*Property = ScreenPixelsHeight*/
const Value V19977_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19977_4 = {S54, 0, &V19977_4_1229, &P19977_5};	/*Property = ScreenPixelsWidth*/
const Profile P19977 = {"19977", &P19977_4};	/*Profile ID - 19977*/
const Value V23556_1_1 = {S2, NULL};	/*Value = True*/
const Property P23556_1 = {S50, 0, &V23556_1_1, NULL};	/*Property = IsMobile*/
const Value V23556_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23556_5 = {S52, 0, &V23556_5_37, &P23556_1};	/*Property = ScreenPixelsHeight*/
const Value V23556_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23556_4 = {S54, 0, &V23556_4_36, &P23556_5};	/*Property = ScreenPixelsWidth*/
const Profile P23556 = {"23556", &P23556_4};	/*Profile ID - 23556*/
const Value V19874_1_1 = {S2, NULL};	/*Value = True*/
const Property P19874_1 = {S50, 0, &V19874_1_1, NULL};	/*Property = IsMobile*/
const Value V19874_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P19874_5 = {S52, 0, &V19874_5_1458, &P19874_1};	/*Property = ScreenPixelsHeight*/
const Value V19874_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P19874_4 = {S54, 0, &V19874_4_1460, &P19874_5};	/*Property = ScreenPixelsWidth*/
const Profile P19874 = {"19874", &P19874_4};	/*Profile ID - 19874*/
const Value V20197_1_1 = {S2, NULL};	/*Value = True*/
const Property P20197_1 = {S50, 0, &V20197_1_1, NULL};	/*Property = IsMobile*/
const Value V20197_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20197_5 = {S52, 0, &V20197_5_1227, &P20197_1};	/*Property = ScreenPixelsHeight*/
const Value V20197_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20197_4 = {S54, 0, &V20197_4_1229, &P20197_5};	/*Property = ScreenPixelsWidth*/
const Profile P20197 = {"20197", &P20197_4};	/*Profile ID - 20197*/
const Value V20853_1_1 = {S2, NULL};	/*Value = True*/
const Property P20853_1 = {S50, 0, &V20853_1_1, NULL};	/*Property = IsMobile*/
const Value V20853_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20853_5 = {S52, 0, &V20853_5_1458, &P20853_1};	/*Property = ScreenPixelsHeight*/
const Value V20853_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20853_4 = {S54, 0, &V20853_4_37617, &P20853_5};	/*Property = ScreenPixelsWidth*/
const Profile P20853 = {"20853", &P20853_4};	/*Profile ID - 20853*/
const Value V23792_1_1 = {S2, NULL};	/*Value = True*/
const Property P23792_1 = {S50, 0, &V23792_1_1, NULL};	/*Property = IsMobile*/
const Value V23792_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23792_5 = {S52, 0, &V23792_5_1222, &P23792_1};	/*Property = ScreenPixelsHeight*/
const Value V23792_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23792_4 = {S54, 0, &V23792_4_1224, &P23792_5};	/*Property = ScreenPixelsWidth*/
const Profile P23792 = {"23792", &P23792_4};	/*Profile ID - 23792*/
const Value V22535_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22535_279 = {S1, 0, &V22535_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22535_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22535_282 = {S3, 0, &V22535_282_57153, &P22535_279};	/*Property = BlobBuilder*/
const Value V22535_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22535_350 = {S4, 0, &V22535_350_57444, &P22535_282};	/*Property = Canvas*/
const Value V22535_283_57160 = {S0, NULL};	/*Value = False*/
const Property P22535_283 = {S5, 0, &V22535_283_57160, &P22535_350};	/*Property = CssBackground*/
const Value V22535_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22535_284 = {S6, 0, &V22535_284_57163, &P22535_283};	/*Property = CssBorderImage*/
const Value V22535_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22535_285 = {S7, 0, &V22535_285_57166, &P22535_284};	/*Property = CssCanvas*/
const Value V22535_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22535_286 = {S8, 0, &V22535_286_57168, &P22535_285};	/*Property = CssColor*/
const Value V22535_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22535_237 = {S9, 0, &V22535_237_55784, &P22535_286};	/*Property = CssColumn*/
const Value V22535_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22535_287 = {S10, 0, &V22535_287_57172, &P22535_237};	/*Property = CssFlexbox*/
const Value V22535_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22535_288 = {S11, 0, &V22535_288_57175, &P22535_287};	/*Property = CssFont*/
const Value V22535_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22535_289 = {S12, 0, &V22535_289_86899, &P22535_288};	/*Property = CssImages*/
const Value V22535_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22535_291 = {S13, 0, &V22535_291_57181, &P22535_289};	/*Property = CssMediaQueries*/
const Value V22535_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22535_292 = {S14, 0, &V22535_292_57183, &P22535_291};	/*Property = CssMinMax*/
const Value V22535_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22535_293 = {S15, 0, &V22535_293_57187, &P22535_292};	/*Property = CssOverflow*/
const Value V22535_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22535_294 = {S16, 0, &V22535_294_57189, &P22535_293};	/*Property = CssPosition*/
const Value V22535_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22535_295 = {S17, 0, &V22535_295_57193, &P22535_294};	/*Property = CssText*/
const Value V22535_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22535_297 = {S18, 0, &V22535_297_57195, &P22535_295};	/*Property = CssTransforms*/
const Value V22535_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22535_241 = {S19, 0, &V22535_241_55799, &P22535_297};	/*Property = CssTransitions*/
const Value V22535_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22535_298 = {S20, 0, &V22535_298_57206, &P22535_241};	/*Property = CssUI*/
const Value V22535_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22535_300 = {S21, 0, &V22535_300_57215, &P22535_298};	/*Property = DataSet*/
const Value V22535_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22535_301 = {S22, 0, &V22535_301_57218, &P22535_300};	/*Property = DataUrl*/
const Value V22535_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22535_302 = {S23, 0, &V22535_302_57221, &P22535_301};	/*Property = DeviceOrientation*/
const Value V22535_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22535_304 = {S24, 0, &V22535_304_57227, &P22535_302};	/*Property = FileReader*/
const Value V22535_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22535_305 = {S25, 0, &V22535_305_57231, &P22535_304};	/*Property = FileSaver*/
const Value V22535_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22535_306 = {S26, 0, &V22535_306_57237, &P22535_305};	/*Property = FileWriter*/
const Value V22535_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22535_307 = {S27, 0, &V22535_307_57239, &P22535_306};	/*Property = FormData*/
const Value V22535_309_57245 = {S2, NULL};	/*Value = True*/
const Property P22535_309 = {S28, 0, &V22535_309_57245, &P22535_307};	/*Property = Fullscreen*/
const Value V22535_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22535_310 = {S29, 0, &V22535_310_57248, &P22535_309};	/*Property = GeoLocation*/
const Value V22535_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22535_312 = {S30, 0, &V22535_312_57254, &P22535_310};	/*Property = History*/
const Value V22535_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22535_314 = {S31, 0, &V22535_314_57261, &P22535_312};	/*Property = Html5*/
const Value V22535_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22535_313 = {S32, 0, &V22535_313_57258, &P22535_314};	/*Property = Html-Media-Capture*/
const Value V22535_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22535_315 = {S33, 0, &V22535_315_57264, &P22535_313};	/*Property = Iframe*/
const Value V22535_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22535_316 = {S34, 0, &V22535_316_57266, &P22535_315};	/*Property = IndexedDB*/
const Value V22535_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22535_317 = {S35, 0, &V22535_317_57269, &P22535_316};	/*Property = Json*/
const Value V22535_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22535_15 = {S37, 0, &V22535_15_29, &P22535_317};	/*Property = LayoutEngine*/
const Value V22535_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22535_344 = {S38, 0, &V22535_344_57272, &P22535_15};	/*Property = Masking*/
const Value V22535_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22535_320 = {S39, 0, &V22535_320_57290, &P22535_344};	/*Property = PostMessage*/
const Value V22535_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22535_321 = {S40, 0, &V22535_321_57293, &P22535_320};	/*Property = Progress*/
const Value V22535_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22535_322 = {S41, 0, &V22535_322_57296, &P22535_321};	/*Property = Prompts*/
const Value V22535_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22535_323 = {S42, 0, &V22535_323_57299, &P22535_322};	/*Property = Selector*/
const Value V22535_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22535_324 = {S43, 0, &V22535_324_57305, &P22535_323};	/*Property = Svg*/
const Value V22535_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22535_327 = {S44, 0, &V22535_327_57315, &P22535_324};	/*Property = TouchEvents*/
const Value V22535_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22535_328 = {S45, 0, &V22535_328_57318, &P22535_327};	/*Property = Track*/
const Value V22535_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22535_329 = {S46, 0, &V22535_329_57323, &P22535_328};	/*Property = Video*/
const Value V22535_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22535_330 = {S47, 0, &V22535_330_57326, &P22535_329};	/*Property = Viewport*/
const Value V22535_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22535_332 = {S48, 0, &V22535_332_57339, &P22535_330};	/*Property = WebWorkers*/
const Value V22535_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22535_333 = {S49, 0, &V22535_333_57342, &P22535_332};	/*Property = Xhr2*/
const Profile P22535 = {"22535", &P22535_333};	/*Profile ID - 22535*/
const Value V21119_1_1 = {S2, NULL};	/*Value = True*/
const Property P21119_1 = {S50, 0, &V21119_1_1, NULL};	/*Property = IsMobile*/
const Value V21119_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P21119_5 = {S52, 0, &V21119_5_1832, &P21119_1};	/*Property = ScreenPixelsHeight*/
const Value V21119_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21119_4 = {S54, 0, &V21119_4_2336, &P21119_5};	/*Property = ScreenPixelsWidth*/
const Profile P21119 = {"21119", &P21119_4};	/*Profile ID - 21119*/
const Value V21055_1_1 = {S2, NULL};	/*Value = True*/
const Property P21055_1 = {S50, 0, &V21055_1_1, NULL};	/*Property = IsMobile*/
const Value V21055_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21055_5 = {S52, 0, &V21055_5_1227, &P21055_1};	/*Property = ScreenPixelsHeight*/
const Value V21055_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21055_4 = {S54, 0, &V21055_4_1229, &P21055_5};	/*Property = ScreenPixelsWidth*/
const Profile P21055 = {"21055", &P21055_4};	/*Profile ID - 21055*/
const Value V22157_1_1 = {S2, NULL};	/*Value = True*/
const Property P22157_1 = {S50, 0, &V22157_1_1, NULL};	/*Property = IsMobile*/
const Value V22157_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22157_5 = {S52, 0, &V22157_5_1277, &P22157_1};	/*Property = ScreenPixelsHeight*/
const Value V22157_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22157_4 = {S54, 0, &V22157_4_2336, &P22157_5};	/*Property = ScreenPixelsWidth*/
const Profile P22157 = {"22157", &P22157_4};	/*Profile ID - 22157*/
const Value V24204_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24204_279 = {S1, 0, &V24204_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24204_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24204_282 = {S3, 0, &V24204_282_57154, &P24204_279};	/*Property = BlobBuilder*/
const Value V24204_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24204_350 = {S4, 0, &V24204_350_57444, &P24204_282};	/*Property = Canvas*/
const Value V24204_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24204_283 = {S5, 0, &V24204_283_57159, &P24204_350};	/*Property = CssBackground*/
const Value V24204_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24204_284 = {S6, 0, &V24204_284_57162, &P24204_283};	/*Property = CssBorderImage*/
const Value V24204_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24204_285 = {S7, 0, &V24204_285_57166, &P24204_284};	/*Property = CssCanvas*/
const Value V24204_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24204_286 = {S8, 0, &V24204_286_57168, &P24204_285};	/*Property = CssColor*/
const Value V24204_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24204_237 = {S9, 0, &V24204_237_55786, &P24204_286};	/*Property = CssColumn*/
const Value V24204_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24204_287 = {S10, 0, &V24204_287_57172, &P24204_237};	/*Property = CssFlexbox*/
const Value V24204_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24204_288 = {S11, 0, &V24204_288_57175, &P24204_287};	/*Property = CssFont*/
const Value V24204_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24204_289 = {S12, 0, &V24204_289_57759, &P24204_288};	/*Property = CssImages*/
const Value V24204_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24204_291 = {S13, 0, &V24204_291_57180, &P24204_289};	/*Property = CssMediaQueries*/
const Value V24204_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24204_292 = {S14, 0, &V24204_292_57183, &P24204_291};	/*Property = CssMinMax*/
const Value V24204_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24204_293 = {S15, 0, &V24204_293_57187, &P24204_292};	/*Property = CssOverflow*/
const Value V24204_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24204_294 = {S16, 0, &V24204_294_57189, &P24204_293};	/*Property = CssPosition*/
const Value V24204_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24204_295 = {S17, 0, &V24204_295_57192, &P24204_294};	/*Property = CssText*/
const Value V24204_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24204_297 = {S18, 0, &V24204_297_57195, &P24204_295};	/*Property = CssTransforms*/
const Value V24204_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24204_241 = {S19, 0, &V24204_241_55799, &P24204_297};	/*Property = CssTransitions*/
const Value V24204_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24204_298 = {S20, 0, &V24204_298_57206, &P24204_241};	/*Property = CssUI*/
const Value V24204_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24204_300 = {S21, 0, &V24204_300_57215, &P24204_298};	/*Property = DataSet*/
const Value V24204_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24204_301 = {S22, 0, &V24204_301_57218, &P24204_300};	/*Property = DataUrl*/
const Value V24204_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24204_302 = {S23, 0, &V24204_302_57221, &P24204_301};	/*Property = DeviceOrientation*/
const Value V24204_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24204_304 = {S24, 0, &V24204_304_57227, &P24204_302};	/*Property = FileReader*/
const Value V24204_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24204_305 = {S25, 0, &V24204_305_57231, &P24204_304};	/*Property = FileSaver*/
const Value V24204_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24204_306 = {S26, 0, &V24204_306_57237, &P24204_305};	/*Property = FileWriter*/
const Value V24204_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24204_307 = {S27, 0, &V24204_307_57239, &P24204_306};	/*Property = FormData*/
const Value V24204_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24204_309 = {S28, 0, &V24204_309_57246, &P24204_307};	/*Property = Fullscreen*/
const Value V24204_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24204_310 = {S29, 0, &V24204_310_57248, &P24204_309};	/*Property = GeoLocation*/
const Value V24204_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24204_312 = {S30, 0, &V24204_312_57254, &P24204_310};	/*Property = History*/
const Value V24204_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24204_314 = {S31, 0, &V24204_314_57260, &P24204_312};	/*Property = Html5*/
const Value V24204_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24204_313 = {S32, 0, &V24204_313_57257, &P24204_314};	/*Property = Html-Media-Capture*/
const Value V24204_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24204_315 = {S33, 0, &V24204_315_57264, &P24204_313};	/*Property = Iframe*/
const Value V24204_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24204_316 = {S34, 0, &V24204_316_57266, &P24204_315};	/*Property = IndexedDB*/
const Value V24204_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24204_317 = {S35, 0, &V24204_317_57269, &P24204_316};	/*Property = Json*/
const Value V24204_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24204_15 = {S37, 0, &V24204_15_54167, &P24204_317};	/*Property = LayoutEngine*/
const Value V24204_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24204_344 = {S38, 0, &V24204_344_57272, &P24204_15};	/*Property = Masking*/
const Value V24204_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24204_320 = {S39, 0, &V24204_320_57290, &P24204_344};	/*Property = PostMessage*/
const Value V24204_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24204_321 = {S40, 0, &V24204_321_57293, &P24204_320};	/*Property = Progress*/
const Value V24204_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24204_322 = {S41, 0, &V24204_322_57296, &P24204_321};	/*Property = Prompts*/
const Value V24204_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24204_323 = {S42, 0, &V24204_323_57299, &P24204_322};	/*Property = Selector*/
const Value V24204_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24204_324 = {S43, 0, &V24204_324_57306, &P24204_323};	/*Property = Svg*/
const Value V24204_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24204_327 = {S44, 0, &V24204_327_57314, &P24204_324};	/*Property = TouchEvents*/
const Value V24204_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24204_328 = {S45, 0, &V24204_328_57318, &P24204_327};	/*Property = Track*/
const Value V24204_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24204_329 = {S46, 0, &V24204_329_57323, &P24204_328};	/*Property = Video*/
const Value V24204_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24204_330 = {S47, 0, &V24204_330_57326, &P24204_329};	/*Property = Viewport*/
const Value V24204_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24204_332 = {S48, 0, &V24204_332_57339, &P24204_330};	/*Property = WebWorkers*/
const Value V24204_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24204_333 = {S49, 0, &V24204_333_57342, &P24204_332};	/*Property = Xhr2*/
const Profile P24204 = {"24204", &P24204_333};	/*Profile ID - 24204*/
const Value V18332_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_279 = {S1, 0, &V18332_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18332_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_282 = {S3, 0, &V18332_282_57155, &P18332_279};	/*Property = BlobBuilder*/
const Value V18332_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_350 = {S4, 0, &V18332_350_57446, &P18332_282};	/*Property = Canvas*/
const Value V18332_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_283 = {S5, 0, &V18332_283_57161, &P18332_350};	/*Property = CssBackground*/
const Value V18332_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_284 = {S6, 0, &V18332_284_57164, &P18332_283};	/*Property = CssBorderImage*/
const Value V18332_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_285 = {S7, 0, &V18332_285_57167, &P18332_284};	/*Property = CssCanvas*/
const Value V18332_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_286 = {S8, 0, &V18332_286_57170, &P18332_285};	/*Property = CssColor*/
const Value V18332_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_237 = {S9, 0, &V18332_237_55786, &P18332_286};	/*Property = CssColumn*/
const Value V18332_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_287 = {S10, 0, &V18332_287_57173, &P18332_237};	/*Property = CssFlexbox*/
const Value V18332_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_288 = {S11, 0, &V18332_288_57176, &P18332_287};	/*Property = CssFont*/
const Value V18332_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_289 = {S12, 0, &V18332_289_86899, &P18332_288};	/*Property = CssImages*/
const Value V18332_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_291 = {S13, 0, &V18332_291_57182, &P18332_289};	/*Property = CssMediaQueries*/
const Value V18332_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_292 = {S14, 0, &V18332_292_57185, &P18332_291};	/*Property = CssMinMax*/
const Value V18332_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_293 = {S15, 0, &V18332_293_57188, &P18332_292};	/*Property = CssOverflow*/
const Value V18332_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_294 = {S16, 0, &V18332_294_57191, &P18332_293};	/*Property = CssPosition*/
const Value V18332_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_295 = {S17, 0, &V18332_295_57194, &P18332_294};	/*Property = CssText*/
const Value V18332_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_297 = {S18, 0, &V18332_297_57197, &P18332_295};	/*Property = CssTransforms*/
const Value V18332_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_241 = {S19, 0, &V18332_241_55801, &P18332_297};	/*Property = CssTransitions*/
const Value V18332_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_298 = {S20, 0, &V18332_298_57208, &P18332_241};	/*Property = CssUI*/
const Value V18332_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_300 = {S21, 0, &V18332_300_57217, &P18332_298};	/*Property = DataSet*/
const Value V18332_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_301 = {S22, 0, &V18332_301_57220, &P18332_300};	/*Property = DataUrl*/
const Value V18332_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_302 = {S23, 0, &V18332_302_57223, &P18332_301};	/*Property = DeviceOrientation*/
const Value V18332_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_304 = {S24, 0, &V18332_304_57229, &P18332_302};	/*Property = FileReader*/
const Value V18332_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_305 = {S25, 0, &V18332_305_57232, &P18332_304};	/*Property = FileSaver*/
const Value V18332_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_306 = {S26, 0, &V18332_306_57238, &P18332_305};	/*Property = FileWriter*/
const Value V18332_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_307 = {S27, 0, &V18332_307_57241, &P18332_306};	/*Property = FormData*/
const Value V18332_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_309 = {S28, 0, &V18332_309_57247, &P18332_307};	/*Property = Fullscreen*/
const Value V18332_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_310 = {S29, 0, &V18332_310_57250, &P18332_309};	/*Property = GeoLocation*/
const Value V18332_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_312 = {S30, 0, &V18332_312_57256, &P18332_310};	/*Property = History*/
const Value V18332_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_314 = {S31, 0, &V18332_314_57262, &P18332_312};	/*Property = Html5*/
const Value V18332_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_313 = {S32, 0, &V18332_313_57259, &P18332_314};	/*Property = Html-Media-Capture*/
const Value V18332_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_315 = {S33, 0, &V18332_315_57265, &P18332_313};	/*Property = Iframe*/
const Value V18332_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_316 = {S34, 0, &V18332_316_57268, &P18332_315};	/*Property = IndexedDB*/
const Value V18332_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_317 = {S35, 0, &V18332_317_57271, &P18332_316};	/*Property = Json*/
const Value V18332_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18332_15 = {S37, 0, &V18332_15_31, &P18332_317};	/*Property = LayoutEngine*/
const Value V18332_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_344 = {S38, 0, &V18332_344_57274, &P18332_15};	/*Property = Masking*/
const Value V18332_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_320 = {S39, 0, &V18332_320_57292, &P18332_344};	/*Property = PostMessage*/
const Value V18332_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_321 = {S40, 0, &V18332_321_57295, &P18332_320};	/*Property = Progress*/
const Value V18332_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_322 = {S41, 0, &V18332_322_57298, &P18332_321};	/*Property = Prompts*/
const Value V18332_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_323 = {S42, 0, &V18332_323_57301, &P18332_322};	/*Property = Selector*/
const Value V18332_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_324 = {S43, 0, &V18332_324_57307, &P18332_323};	/*Property = Svg*/
const Value V18332_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_327 = {S44, 0, &V18332_327_57316, &P18332_324};	/*Property = TouchEvents*/
const Value V18332_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_328 = {S45, 0, &V18332_328_57319, &P18332_327};	/*Property = Track*/
const Value V18332_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_329 = {S46, 0, &V18332_329_57325, &P18332_328};	/*Property = Video*/
const Value V18332_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_330 = {S47, 0, &V18332_330_57328, &P18332_329};	/*Property = Viewport*/
const Value V18332_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_332 = {S48, 0, &V18332_332_57340, &P18332_330};	/*Property = WebWorkers*/
const Value V18332_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18332_333 = {S49, 0, &V18332_333_57343, &P18332_332};	/*Property = Xhr2*/
const Profile P18332 = {"18332", &P18332_333};	/*Profile ID - 18332*/
const Value V18632_1_1 = {S2, NULL};	/*Value = True*/
const Property P18632_1 = {S50, 0, &V18632_1_1, NULL};	/*Property = IsMobile*/
const Value V18632_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P18632_5 = {S52, 0, &V18632_5_1458, &P18632_1};	/*Property = ScreenPixelsHeight*/
const Value V18632_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P18632_4 = {S54, 0, &V18632_4_1460, &P18632_5};	/*Property = ScreenPixelsWidth*/
const Profile P18632 = {"18632", &P18632_4};	/*Profile ID - 18632*/
const Value V22489_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_279 = {S1, 0, &V22489_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22489_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_282 = {S3, 0, &V22489_282_57155, &P22489_279};	/*Property = BlobBuilder*/
const Value V22489_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_350 = {S4, 0, &V22489_350_57446, &P22489_282};	/*Property = Canvas*/
const Value V22489_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_283 = {S5, 0, &V22489_283_57161, &P22489_350};	/*Property = CssBackground*/
const Value V22489_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_284 = {S6, 0, &V22489_284_57164, &P22489_283};	/*Property = CssBorderImage*/
const Value V22489_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_285 = {S7, 0, &V22489_285_57167, &P22489_284};	/*Property = CssCanvas*/
const Value V22489_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_286 = {S8, 0, &V22489_286_57170, &P22489_285};	/*Property = CssColor*/
const Value V22489_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_237 = {S9, 0, &V22489_237_55786, &P22489_286};	/*Property = CssColumn*/
const Value V22489_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_287 = {S10, 0, &V22489_287_57173, &P22489_237};	/*Property = CssFlexbox*/
const Value V22489_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_288 = {S11, 0, &V22489_288_57176, &P22489_287};	/*Property = CssFont*/
const Value V22489_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_289 = {S12, 0, &V22489_289_86899, &P22489_288};	/*Property = CssImages*/
const Value V22489_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_291 = {S13, 0, &V22489_291_57182, &P22489_289};	/*Property = CssMediaQueries*/
const Value V22489_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_292 = {S14, 0, &V22489_292_57185, &P22489_291};	/*Property = CssMinMax*/
const Value V22489_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_293 = {S15, 0, &V22489_293_57188, &P22489_292};	/*Property = CssOverflow*/
const Value V22489_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_294 = {S16, 0, &V22489_294_57191, &P22489_293};	/*Property = CssPosition*/
const Value V22489_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_295 = {S17, 0, &V22489_295_57194, &P22489_294};	/*Property = CssText*/
const Value V22489_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_297 = {S18, 0, &V22489_297_57197, &P22489_295};	/*Property = CssTransforms*/
const Value V22489_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_241 = {S19, 0, &V22489_241_55801, &P22489_297};	/*Property = CssTransitions*/
const Value V22489_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_298 = {S20, 0, &V22489_298_57208, &P22489_241};	/*Property = CssUI*/
const Value V22489_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_300 = {S21, 0, &V22489_300_57217, &P22489_298};	/*Property = DataSet*/
const Value V22489_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_301 = {S22, 0, &V22489_301_57220, &P22489_300};	/*Property = DataUrl*/
const Value V22489_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_302 = {S23, 0, &V22489_302_57223, &P22489_301};	/*Property = DeviceOrientation*/
const Value V22489_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_304 = {S24, 0, &V22489_304_57229, &P22489_302};	/*Property = FileReader*/
const Value V22489_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_305 = {S25, 0, &V22489_305_57232, &P22489_304};	/*Property = FileSaver*/
const Value V22489_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_306 = {S26, 0, &V22489_306_57238, &P22489_305};	/*Property = FileWriter*/
const Value V22489_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_307 = {S27, 0, &V22489_307_57241, &P22489_306};	/*Property = FormData*/
const Value V22489_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_309 = {S28, 0, &V22489_309_57247, &P22489_307};	/*Property = Fullscreen*/
const Value V22489_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_310 = {S29, 0, &V22489_310_57250, &P22489_309};	/*Property = GeoLocation*/
const Value V22489_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_312 = {S30, 0, &V22489_312_57256, &P22489_310};	/*Property = History*/
const Value V22489_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_314 = {S31, 0, &V22489_314_57262, &P22489_312};	/*Property = Html5*/
const Value V22489_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_313 = {S32, 0, &V22489_313_57259, &P22489_314};	/*Property = Html-Media-Capture*/
const Value V22489_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_315 = {S33, 0, &V22489_315_57265, &P22489_313};	/*Property = Iframe*/
const Value V22489_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_316 = {S34, 0, &V22489_316_57268, &P22489_315};	/*Property = IndexedDB*/
const Value V22489_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_317 = {S35, 0, &V22489_317_57271, &P22489_316};	/*Property = Json*/
const Value V22489_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22489_15 = {S37, 0, &V22489_15_29, &P22489_317};	/*Property = LayoutEngine*/
const Value V22489_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_344 = {S38, 0, &V22489_344_57274, &P22489_15};	/*Property = Masking*/
const Value V22489_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_320 = {S39, 0, &V22489_320_57292, &P22489_344};	/*Property = PostMessage*/
const Value V22489_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_321 = {S40, 0, &V22489_321_57295, &P22489_320};	/*Property = Progress*/
const Value V22489_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_322 = {S41, 0, &V22489_322_57298, &P22489_321};	/*Property = Prompts*/
const Value V22489_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_323 = {S42, 0, &V22489_323_57301, &P22489_322};	/*Property = Selector*/
const Value V22489_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_324 = {S43, 0, &V22489_324_57307, &P22489_323};	/*Property = Svg*/
const Value V22489_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_327 = {S44, 0, &V22489_327_57316, &P22489_324};	/*Property = TouchEvents*/
const Value V22489_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_328 = {S45, 0, &V22489_328_57319, &P22489_327};	/*Property = Track*/
const Value V22489_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_329 = {S46, 0, &V22489_329_57325, &P22489_328};	/*Property = Video*/
const Value V22489_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_330 = {S47, 0, &V22489_330_57328, &P22489_329};	/*Property = Viewport*/
const Value V22489_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_332 = {S48, 0, &V22489_332_57340, &P22489_330};	/*Property = WebWorkers*/
const Value V22489_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22489_333 = {S49, 0, &V22489_333_57343, &P22489_332};	/*Property = Xhr2*/
const Profile P22489 = {"22489", &P22489_333};	/*Profile ID - 22489*/
const Value V22483_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_279 = {S1, 0, &V22483_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22483_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_282 = {S3, 0, &V22483_282_57155, &P22483_279};	/*Property = BlobBuilder*/
const Value V22483_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_350 = {S4, 0, &V22483_350_57446, &P22483_282};	/*Property = Canvas*/
const Value V22483_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_283 = {S5, 0, &V22483_283_57161, &P22483_350};	/*Property = CssBackground*/
const Value V22483_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_284 = {S6, 0, &V22483_284_57164, &P22483_283};	/*Property = CssBorderImage*/
const Value V22483_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_285 = {S7, 0, &V22483_285_57167, &P22483_284};	/*Property = CssCanvas*/
const Value V22483_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_286 = {S8, 0, &V22483_286_57170, &P22483_285};	/*Property = CssColor*/
const Value V22483_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_237 = {S9, 0, &V22483_237_55786, &P22483_286};	/*Property = CssColumn*/
const Value V22483_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_287 = {S10, 0, &V22483_287_57173, &P22483_237};	/*Property = CssFlexbox*/
const Value V22483_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_288 = {S11, 0, &V22483_288_57176, &P22483_287};	/*Property = CssFont*/
const Value V22483_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_289 = {S12, 0, &V22483_289_86899, &P22483_288};	/*Property = CssImages*/
const Value V22483_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_291 = {S13, 0, &V22483_291_57182, &P22483_289};	/*Property = CssMediaQueries*/
const Value V22483_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_292 = {S14, 0, &V22483_292_57185, &P22483_291};	/*Property = CssMinMax*/
const Value V22483_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_293 = {S15, 0, &V22483_293_57188, &P22483_292};	/*Property = CssOverflow*/
const Value V22483_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_294 = {S16, 0, &V22483_294_57191, &P22483_293};	/*Property = CssPosition*/
const Value V22483_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_295 = {S17, 0, &V22483_295_57194, &P22483_294};	/*Property = CssText*/
const Value V22483_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_297 = {S18, 0, &V22483_297_57197, &P22483_295};	/*Property = CssTransforms*/
const Value V22483_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_241 = {S19, 0, &V22483_241_55801, &P22483_297};	/*Property = CssTransitions*/
const Value V22483_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_298 = {S20, 0, &V22483_298_57208, &P22483_241};	/*Property = CssUI*/
const Value V22483_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_300 = {S21, 0, &V22483_300_57217, &P22483_298};	/*Property = DataSet*/
const Value V22483_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_301 = {S22, 0, &V22483_301_57220, &P22483_300};	/*Property = DataUrl*/
const Value V22483_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_302 = {S23, 0, &V22483_302_57223, &P22483_301};	/*Property = DeviceOrientation*/
const Value V22483_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_304 = {S24, 0, &V22483_304_57229, &P22483_302};	/*Property = FileReader*/
const Value V22483_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_305 = {S25, 0, &V22483_305_57232, &P22483_304};	/*Property = FileSaver*/
const Value V22483_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_306 = {S26, 0, &V22483_306_57238, &P22483_305};	/*Property = FileWriter*/
const Value V22483_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_307 = {S27, 0, &V22483_307_57241, &P22483_306};	/*Property = FormData*/
const Value V22483_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_309 = {S28, 0, &V22483_309_57247, &P22483_307};	/*Property = Fullscreen*/
const Value V22483_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_310 = {S29, 0, &V22483_310_57250, &P22483_309};	/*Property = GeoLocation*/
const Value V22483_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_312 = {S30, 0, &V22483_312_57256, &P22483_310};	/*Property = History*/
const Value V22483_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_314 = {S31, 0, &V22483_314_57262, &P22483_312};	/*Property = Html5*/
const Value V22483_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_313 = {S32, 0, &V22483_313_57259, &P22483_314};	/*Property = Html-Media-Capture*/
const Value V22483_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_315 = {S33, 0, &V22483_315_57265, &P22483_313};	/*Property = Iframe*/
const Value V22483_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_316 = {S34, 0, &V22483_316_57268, &P22483_315};	/*Property = IndexedDB*/
const Value V22483_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_317 = {S35, 0, &V22483_317_57271, &P22483_316};	/*Property = Json*/
const Value V22483_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22483_15 = {S37, 0, &V22483_15_29, &P22483_317};	/*Property = LayoutEngine*/
const Value V22483_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_344 = {S38, 0, &V22483_344_57274, &P22483_15};	/*Property = Masking*/
const Value V22483_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_320 = {S39, 0, &V22483_320_57292, &P22483_344};	/*Property = PostMessage*/
const Value V22483_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_321 = {S40, 0, &V22483_321_57295, &P22483_320};	/*Property = Progress*/
const Value V22483_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_322 = {S41, 0, &V22483_322_57298, &P22483_321};	/*Property = Prompts*/
const Value V22483_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_323 = {S42, 0, &V22483_323_57301, &P22483_322};	/*Property = Selector*/
const Value V22483_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_324 = {S43, 0, &V22483_324_57307, &P22483_323};	/*Property = Svg*/
const Value V22483_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_327 = {S44, 0, &V22483_327_57316, &P22483_324};	/*Property = TouchEvents*/
const Value V22483_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_328 = {S45, 0, &V22483_328_57319, &P22483_327};	/*Property = Track*/
const Value V22483_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_329 = {S46, 0, &V22483_329_57325, &P22483_328};	/*Property = Video*/
const Value V22483_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_330 = {S47, 0, &V22483_330_57328, &P22483_329};	/*Property = Viewport*/
const Value V22483_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_332 = {S48, 0, &V22483_332_57340, &P22483_330};	/*Property = WebWorkers*/
const Value V22483_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22483_333 = {S49, 0, &V22483_333_57343, &P22483_332};	/*Property = Xhr2*/
const Profile P22483 = {"22483", &P22483_333};	/*Profile ID - 22483*/
const Value V18316_1_1 = {S2, NULL};	/*Value = True*/
const Property P18316_1 = {S50, 0, &V18316_1_1, NULL};	/*Property = IsMobile*/
const Value V18316_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18316_5 = {S52, 0, &V18316_5_9981, &P18316_1};	/*Property = ScreenPixelsHeight*/
const Value V18316_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18316_4 = {S54, 0, &V18316_4_39098, &P18316_5};	/*Property = ScreenPixelsWidth*/
const Profile P18316 = {"18316", &P18316_4};	/*Profile ID - 18316*/
const Value V20127_1_1 = {S2, NULL};	/*Value = True*/
const Property P20127_1 = {S50, 0, &V20127_1_1, NULL};	/*Property = IsMobile*/
const Value V20127_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20127_5 = {S52, 0, &V20127_5_9981, &P20127_1};	/*Property = ScreenPixelsHeight*/
const Value V20127_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20127_4 = {S54, 0, &V20127_4_2336, &P20127_5};	/*Property = ScreenPixelsWidth*/
const Profile P20127 = {"20127", &P20127_4};	/*Profile ID - 20127*/
const Value V19946_1_1 = {S2, NULL};	/*Value = True*/
const Property P19946_1 = {S50, 0, &V19946_1_1, NULL};	/*Property = IsMobile*/
const Value V19946_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P19946_5 = {S52, 0, &V19946_5_1222, &P19946_1};	/*Property = ScreenPixelsHeight*/
const Value V19946_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P19946_4 = {S54, 0, &V19946_4_1224, &P19946_5};	/*Property = ScreenPixelsWidth*/
const Profile P19946 = {"19946", &P19946_4};	/*Profile ID - 19946*/
const Value V20267_1_1 = {S2, NULL};	/*Value = True*/
const Property P20267_1 = {S50, 0, &V20267_1_1, NULL};	/*Property = IsMobile*/
const Value V20267_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20267_5 = {S52, 0, &V20267_5_1227, &P20267_1};	/*Property = ScreenPixelsHeight*/
const Value V20267_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20267_4 = {S54, 0, &V20267_4_1229, &P20267_5};	/*Property = ScreenPixelsWidth*/
const Profile P20267 = {"20267", &P20267_4};	/*Profile ID - 20267*/
const Value V22437_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_279 = {S1, 0, &V22437_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22437_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_282 = {S3, 0, &V22437_282_57155, &P22437_279};	/*Property = BlobBuilder*/
const Value V22437_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_350 = {S4, 0, &V22437_350_57446, &P22437_282};	/*Property = Canvas*/
const Value V22437_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_283 = {S5, 0, &V22437_283_57161, &P22437_350};	/*Property = CssBackground*/
const Value V22437_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_284 = {S6, 0, &V22437_284_57164, &P22437_283};	/*Property = CssBorderImage*/
const Value V22437_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_285 = {S7, 0, &V22437_285_57167, &P22437_284};	/*Property = CssCanvas*/
const Value V22437_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_286 = {S8, 0, &V22437_286_57170, &P22437_285};	/*Property = CssColor*/
const Value V22437_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_237 = {S9, 0, &V22437_237_55786, &P22437_286};	/*Property = CssColumn*/
const Value V22437_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_287 = {S10, 0, &V22437_287_57173, &P22437_237};	/*Property = CssFlexbox*/
const Value V22437_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_288 = {S11, 0, &V22437_288_57176, &P22437_287};	/*Property = CssFont*/
const Value V22437_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_289 = {S12, 0, &V22437_289_86899, &P22437_288};	/*Property = CssImages*/
const Value V22437_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_291 = {S13, 0, &V22437_291_57182, &P22437_289};	/*Property = CssMediaQueries*/
const Value V22437_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_292 = {S14, 0, &V22437_292_57185, &P22437_291};	/*Property = CssMinMax*/
const Value V22437_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_293 = {S15, 0, &V22437_293_57188, &P22437_292};	/*Property = CssOverflow*/
const Value V22437_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_294 = {S16, 0, &V22437_294_57191, &P22437_293};	/*Property = CssPosition*/
const Value V22437_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_295 = {S17, 0, &V22437_295_57194, &P22437_294};	/*Property = CssText*/
const Value V22437_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_297 = {S18, 0, &V22437_297_57197, &P22437_295};	/*Property = CssTransforms*/
const Value V22437_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_241 = {S19, 0, &V22437_241_55801, &P22437_297};	/*Property = CssTransitions*/
const Value V22437_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_298 = {S20, 0, &V22437_298_57208, &P22437_241};	/*Property = CssUI*/
const Value V22437_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_300 = {S21, 0, &V22437_300_57217, &P22437_298};	/*Property = DataSet*/
const Value V22437_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_301 = {S22, 0, &V22437_301_57220, &P22437_300};	/*Property = DataUrl*/
const Value V22437_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_302 = {S23, 0, &V22437_302_57223, &P22437_301};	/*Property = DeviceOrientation*/
const Value V22437_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_304 = {S24, 0, &V22437_304_57229, &P22437_302};	/*Property = FileReader*/
const Value V22437_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_305 = {S25, 0, &V22437_305_57232, &P22437_304};	/*Property = FileSaver*/
const Value V22437_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_306 = {S26, 0, &V22437_306_57238, &P22437_305};	/*Property = FileWriter*/
const Value V22437_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_307 = {S27, 0, &V22437_307_57241, &P22437_306};	/*Property = FormData*/
const Value V22437_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_309 = {S28, 0, &V22437_309_57247, &P22437_307};	/*Property = Fullscreen*/
const Value V22437_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_310 = {S29, 0, &V22437_310_57250, &P22437_309};	/*Property = GeoLocation*/
const Value V22437_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_312 = {S30, 0, &V22437_312_57256, &P22437_310};	/*Property = History*/
const Value V22437_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_314 = {S31, 0, &V22437_314_57262, &P22437_312};	/*Property = Html5*/
const Value V22437_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_313 = {S32, 0, &V22437_313_57259, &P22437_314};	/*Property = Html-Media-Capture*/
const Value V22437_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_315 = {S33, 0, &V22437_315_57265, &P22437_313};	/*Property = Iframe*/
const Value V22437_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_316 = {S34, 0, &V22437_316_57268, &P22437_315};	/*Property = IndexedDB*/
const Value V22437_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_317 = {S35, 0, &V22437_317_57271, &P22437_316};	/*Property = Json*/
const Value V22437_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22437_15 = {S37, 0, &V22437_15_28, &P22437_317};	/*Property = LayoutEngine*/
const Value V22437_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_344 = {S38, 0, &V22437_344_57274, &P22437_15};	/*Property = Masking*/
const Value V22437_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_320 = {S39, 0, &V22437_320_57292, &P22437_344};	/*Property = PostMessage*/
const Value V22437_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_321 = {S40, 0, &V22437_321_57295, &P22437_320};	/*Property = Progress*/
const Value V22437_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_322 = {S41, 0, &V22437_322_57298, &P22437_321};	/*Property = Prompts*/
const Value V22437_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_323 = {S42, 0, &V22437_323_57301, &P22437_322};	/*Property = Selector*/
const Value V22437_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_324 = {S43, 0, &V22437_324_57307, &P22437_323};	/*Property = Svg*/
const Value V22437_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_327 = {S44, 0, &V22437_327_57316, &P22437_324};	/*Property = TouchEvents*/
const Value V22437_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_328 = {S45, 0, &V22437_328_57319, &P22437_327};	/*Property = Track*/
const Value V22437_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_329 = {S46, 0, &V22437_329_57325, &P22437_328};	/*Property = Video*/
const Value V22437_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_330 = {S47, 0, &V22437_330_57328, &P22437_329};	/*Property = Viewport*/
const Value V22437_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_332 = {S48, 0, &V22437_332_57340, &P22437_330};	/*Property = WebWorkers*/
const Value V22437_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22437_333 = {S49, 0, &V22437_333_57343, &P22437_332};	/*Property = Xhr2*/
const Profile P22437 = {"22437", &P22437_333};	/*Profile ID - 22437*/
const Profile P22216 = {"22216", NULL};	/*Profile ID - 22216*/
const Value V18076_1_1 = {S2, NULL};	/*Value = True*/
const Property P18076_1 = {S50, 0, &V18076_1_1, NULL};	/*Property = IsMobile*/
const Value V18076_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18076_5 = {S52, 0, &V18076_5_1227, &P18076_1};	/*Property = ScreenPixelsHeight*/
const Value V18076_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18076_4 = {S54, 0, &V18076_4_1229, &P18076_5};	/*Property = ScreenPixelsWidth*/
const Profile P18076 = {"18076", &P18076_4};	/*Profile ID - 18076*/
const Value V22061_1_1 = {S2, NULL};	/*Value = True*/
const Property P22061_1 = {S50, 0, &V22061_1_1, NULL};	/*Property = IsMobile*/
const Value V22061_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22061_5 = {S52, 0, &V22061_5_1227, &P22061_1};	/*Property = ScreenPixelsHeight*/
const Value V22061_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22061_4 = {S54, 0, &V22061_4_1229, &P22061_5};	/*Property = ScreenPixelsWidth*/
const Profile P22061 = {"22061", &P22061_4};	/*Profile ID - 22061*/
const Value V18706_1_1 = {S2, NULL};	/*Value = True*/
const Property P18706_1 = {S50, 0, &V18706_1_1, NULL};	/*Property = IsMobile*/
const Value V18706_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P18706_5 = {S52, 0, &V18706_5_37, &P18706_1};	/*Property = ScreenPixelsHeight*/
const Value V18706_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P18706_4 = {S54, 0, &V18706_4_36, &P18706_5};	/*Property = ScreenPixelsWidth*/
const Profile P18706 = {"18706", &P18706_4};	/*Profile ID - 18706*/
const Value V22243_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22243_279 = {S1, 0, &V22243_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22243_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22243_282 = {S3, 0, &V22243_282_57154, &P22243_279};	/*Property = BlobBuilder*/
const Value V22243_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22243_350 = {S4, 0, &V22243_350_57444, &P22243_282};	/*Property = Canvas*/
const Value V22243_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22243_283 = {S5, 0, &V22243_283_57159, &P22243_350};	/*Property = CssBackground*/
const Value V22243_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22243_284 = {S6, 0, &V22243_284_57163, &P22243_283};	/*Property = CssBorderImage*/
const Value V22243_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22243_285 = {S7, 0, &V22243_285_57166, &P22243_284};	/*Property = CssCanvas*/
const Value V22243_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22243_286 = {S8, 0, &V22243_286_57168, &P22243_285};	/*Property = CssColor*/
const Value V22243_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22243_237 = {S9, 0, &V22243_237_55786, &P22243_286};	/*Property = CssColumn*/
const Value V22243_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22243_287 = {S10, 0, &V22243_287_57172, &P22243_237};	/*Property = CssFlexbox*/
const Value V22243_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22243_288 = {S11, 0, &V22243_288_57175, &P22243_287};	/*Property = CssFont*/
const Value V22243_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22243_289 = {S12, 0, &V22243_289_57759, &P22243_288};	/*Property = CssImages*/
const Value V22243_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22243_291 = {S13, 0, &V22243_291_57181, &P22243_289};	/*Property = CssMediaQueries*/
const Value V22243_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22243_292 = {S14, 0, &V22243_292_57183, &P22243_291};	/*Property = CssMinMax*/
const Value V22243_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22243_293 = {S15, 0, &V22243_293_57187, &P22243_292};	/*Property = CssOverflow*/
const Value V22243_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22243_294 = {S16, 0, &V22243_294_57190, &P22243_293};	/*Property = CssPosition*/
const Value V22243_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22243_295 = {S17, 0, &V22243_295_57192, &P22243_294};	/*Property = CssText*/
const Value V22243_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22243_297 = {S18, 0, &V22243_297_57195, &P22243_295};	/*Property = CssTransforms*/
const Value V22243_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22243_241 = {S19, 0, &V22243_241_55799, &P22243_297};	/*Property = CssTransitions*/
const Value V22243_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22243_298 = {S20, 0, &V22243_298_57206, &P22243_241};	/*Property = CssUI*/
const Value V22243_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22243_300 = {S21, 0, &V22243_300_57216, &P22243_298};	/*Property = DataSet*/
const Value V22243_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22243_301 = {S22, 0, &V22243_301_57218, &P22243_300};	/*Property = DataUrl*/
const Value V22243_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22243_302 = {S23, 0, &V22243_302_57222, &P22243_301};	/*Property = DeviceOrientation*/
const Value V22243_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22243_304 = {S24, 0, &V22243_304_57228, &P22243_302};	/*Property = FileReader*/
const Value V22243_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22243_305 = {S25, 0, &V22243_305_57231, &P22243_304};	/*Property = FileSaver*/
const Value V22243_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22243_306 = {S26, 0, &V22243_306_57237, &P22243_305};	/*Property = FileWriter*/
const Value V22243_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22243_307 = {S27, 0, &V22243_307_57240, &P22243_306};	/*Property = FormData*/
const Value V22243_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22243_309 = {S28, 0, &V22243_309_57246, &P22243_307};	/*Property = Fullscreen*/
const Value V22243_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22243_310 = {S29, 0, &V22243_310_57248, &P22243_309};	/*Property = GeoLocation*/
const Value V22243_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22243_312 = {S30, 0, &V22243_312_57255, &P22243_310};	/*Property = History*/
const Value V22243_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22243_314 = {S31, 0, &V22243_314_57261, &P22243_312};	/*Property = Html5*/
const Value V22243_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22243_313 = {S32, 0, &V22243_313_57258, &P22243_314};	/*Property = Html-Media-Capture*/
const Value V22243_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22243_315 = {S33, 0, &V22243_315_57264, &P22243_313};	/*Property = Iframe*/
const Value V22243_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22243_316 = {S34, 0, &V22243_316_57267, &P22243_315};	/*Property = IndexedDB*/
const Value V22243_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22243_317 = {S35, 0, &V22243_317_57269, &P22243_316};	/*Property = Json*/
const Value V22243_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22243_15 = {S37, 0, &V22243_15_72, &P22243_317};	/*Property = LayoutEngine*/
const Value V22243_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22243_344 = {S38, 0, &V22243_344_57272, &P22243_15};	/*Property = Masking*/
const Value V22243_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22243_320 = {S39, 0, &V22243_320_57290, &P22243_344};	/*Property = PostMessage*/
const Value V22243_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22243_321 = {S40, 0, &V22243_321_57293, &P22243_320};	/*Property = Progress*/
const Value V22243_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22243_322 = {S41, 0, &V22243_322_57296, &P22243_321};	/*Property = Prompts*/
const Value V22243_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22243_323 = {S42, 0, &V22243_323_57299, &P22243_322};	/*Property = Selector*/
const Value V22243_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22243_324 = {S43, 0, &V22243_324_57306, &P22243_323};	/*Property = Svg*/
const Value V22243_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22243_327 = {S44, 0, &V22243_327_57315, &P22243_324};	/*Property = TouchEvents*/
const Value V22243_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22243_328 = {S45, 0, &V22243_328_57318, &P22243_327};	/*Property = Track*/
const Value V22243_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22243_329 = {S46, 0, &V22243_329_57323, &P22243_328};	/*Property = Video*/
const Value V22243_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22243_330 = {S47, 0, &V22243_330_57326, &P22243_329};	/*Property = Viewport*/
const Value V22243_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22243_332 = {S48, 0, &V22243_332_57339, &P22243_330};	/*Property = WebWorkers*/
const Value V22243_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22243_333 = {S49, 0, &V22243_333_57342, &P22243_332};	/*Property = Xhr2*/
const Profile P22243 = {"22243", &P22243_333};	/*Profile ID - 22243*/
const Value V18516_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18516_279 = {S1, 0, &V18516_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18516_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18516_282 = {S3, 0, &V18516_282_57154, &P18516_279};	/*Property = BlobBuilder*/
const Value V18516_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18516_350 = {S4, 0, &V18516_350_57444, &P18516_282};	/*Property = Canvas*/
const Value V18516_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18516_283 = {S5, 0, &V18516_283_57159, &P18516_350};	/*Property = CssBackground*/
const Value V18516_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18516_284 = {S6, 0, &V18516_284_57163, &P18516_283};	/*Property = CssBorderImage*/
const Value V18516_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18516_285 = {S7, 0, &V18516_285_57166, &P18516_284};	/*Property = CssCanvas*/
const Value V18516_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18516_286 = {S8, 0, &V18516_286_57168, &P18516_285};	/*Property = CssColor*/
const Value V18516_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18516_237 = {S9, 0, &V18516_237_55786, &P18516_286};	/*Property = CssColumn*/
const Value V18516_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18516_287 = {S10, 0, &V18516_287_57172, &P18516_237};	/*Property = CssFlexbox*/
const Value V18516_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18516_288 = {S11, 0, &V18516_288_57175, &P18516_287};	/*Property = CssFont*/
const Value V18516_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18516_289 = {S12, 0, &V18516_289_57759, &P18516_288};	/*Property = CssImages*/
const Value V18516_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18516_291 = {S13, 0, &V18516_291_57180, &P18516_289};	/*Property = CssMediaQueries*/
const Value V18516_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18516_292 = {S14, 0, &V18516_292_57183, &P18516_291};	/*Property = CssMinMax*/
const Value V18516_293_57186 = {S2, NULL};	/*Value = True*/
const Property P18516_293 = {S15, 0, &V18516_293_57186, &P18516_292};	/*Property = CssOverflow*/
const Value V18516_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18516_294 = {S16, 0, &V18516_294_57189, &P18516_293};	/*Property = CssPosition*/
const Value V18516_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18516_295 = {S17, 0, &V18516_295_57192, &P18516_294};	/*Property = CssText*/
const Value V18516_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18516_297 = {S18, 0, &V18516_297_57195, &P18516_295};	/*Property = CssTransforms*/
const Value V18516_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18516_241 = {S19, 0, &V18516_241_55799, &P18516_297};	/*Property = CssTransitions*/
const Value V18516_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18516_298 = {S20, 0, &V18516_298_57206, &P18516_241};	/*Property = CssUI*/
const Value V18516_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18516_300 = {S21, 0, &V18516_300_57216, &P18516_298};	/*Property = DataSet*/
const Value V18516_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18516_301 = {S22, 0, &V18516_301_57218, &P18516_300};	/*Property = DataUrl*/
const Value V18516_302_57221 = {S2, NULL};	/*Value = True*/
const Property P18516_302 = {S23, 0, &V18516_302_57221, &P18516_301};	/*Property = DeviceOrientation*/
const Value V18516_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18516_304 = {S24, 0, &V18516_304_57228, &P18516_302};	/*Property = FileReader*/
const Value V18516_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18516_305 = {S25, 0, &V18516_305_57231, &P18516_304};	/*Property = FileSaver*/
const Value V18516_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18516_306 = {S26, 0, &V18516_306_57237, &P18516_305};	/*Property = FileWriter*/
const Value V18516_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18516_307 = {S27, 0, &V18516_307_57239, &P18516_306};	/*Property = FormData*/
const Value V18516_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18516_309 = {S28, 0, &V18516_309_57246, &P18516_307};	/*Property = Fullscreen*/
const Value V18516_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18516_310 = {S29, 0, &V18516_310_57248, &P18516_309};	/*Property = GeoLocation*/
const Value V18516_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18516_312 = {S30, 0, &V18516_312_57255, &P18516_310};	/*Property = History*/
const Value V18516_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18516_314 = {S31, 0, &V18516_314_57261, &P18516_312};	/*Property = Html5*/
const Value V18516_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18516_313 = {S32, 0, &V18516_313_57258, &P18516_314};	/*Property = Html-Media-Capture*/
const Value V18516_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18516_315 = {S33, 0, &V18516_315_57264, &P18516_313};	/*Property = Iframe*/
const Value V18516_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18516_316 = {S34, 0, &V18516_316_57267, &P18516_315};	/*Property = IndexedDB*/
const Value V18516_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18516_317 = {S35, 0, &V18516_317_57269, &P18516_316};	/*Property = Json*/
const Value V18516_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18516_15 = {S37, 0, &V18516_15_28, &P18516_317};	/*Property = LayoutEngine*/
const Value V18516_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18516_344 = {S38, 0, &V18516_344_57272, &P18516_15};	/*Property = Masking*/
const Value V18516_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18516_320 = {S39, 0, &V18516_320_57290, &P18516_344};	/*Property = PostMessage*/
const Value V18516_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18516_321 = {S40, 0, &V18516_321_57293, &P18516_320};	/*Property = Progress*/
const Value V18516_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18516_322 = {S41, 0, &V18516_322_57296, &P18516_321};	/*Property = Prompts*/
const Value V18516_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18516_323 = {S42, 0, &V18516_323_57299, &P18516_322};	/*Property = Selector*/
const Value V18516_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18516_324 = {S43, 0, &V18516_324_57306, &P18516_323};	/*Property = Svg*/
const Value V18516_327_57314 = {S2, NULL};	/*Value = True*/
const Property P18516_327 = {S44, 0, &V18516_327_57314, &P18516_324};	/*Property = TouchEvents*/
const Value V18516_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18516_328 = {S45, 0, &V18516_328_57318, &P18516_327};	/*Property = Track*/
const Value V18516_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18516_329 = {S46, 0, &V18516_329_57323, &P18516_328};	/*Property = Video*/
const Value V18516_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18516_330 = {S47, 0, &V18516_330_57326, &P18516_329};	/*Property = Viewport*/
const Value V18516_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18516_332 = {S48, 0, &V18516_332_57339, &P18516_330};	/*Property = WebWorkers*/
const Value V18516_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18516_333 = {S49, 0, &V18516_333_57342, &P18516_332};	/*Property = Xhr2*/
const Profile P18516 = {"18516", &P18516_333};	/*Profile ID - 18516*/
const Value V23682_279_57148 = {S0, NULL};	/*Value = False*/
const Property P23682_279 = {S1, 0, &V23682_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V23682_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23682_282 = {S3, 0, &V23682_282_57154, &P23682_279};	/*Property = BlobBuilder*/
const Value V23682_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23682_350 = {S4, 0, &V23682_350_57444, &P23682_282};	/*Property = Canvas*/
const Value V23682_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23682_283 = {S5, 0, &V23682_283_57159, &P23682_350};	/*Property = CssBackground*/
const Value V23682_284_57163 = {S0, NULL};	/*Value = False*/
const Property P23682_284 = {S6, 0, &V23682_284_57163, &P23682_283};	/*Property = CssBorderImage*/
const Value V23682_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23682_285 = {S7, 0, &V23682_285_57166, &P23682_284};	/*Property = CssCanvas*/
const Value V23682_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23682_286 = {S8, 0, &V23682_286_57168, &P23682_285};	/*Property = CssColor*/
const Value V23682_237_55784 = {S2, NULL};	/*Value = True*/
const Property P23682_237 = {S9, 0, &V23682_237_55784, &P23682_286};	/*Property = CssColumn*/
const Value V23682_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23682_287 = {S10, 0, &V23682_287_57172, &P23682_237};	/*Property = CssFlexbox*/
const Value V23682_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23682_288 = {S11, 0, &V23682_288_57174, &P23682_287};	/*Property = CssFont*/
const Value V23682_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23682_289 = {S12, 0, &V23682_289_57759, &P23682_288};	/*Property = CssImages*/
const Value V23682_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23682_291 = {S13, 0, &V23682_291_57180, &P23682_289};	/*Property = CssMediaQueries*/
const Value V23682_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23682_292 = {S14, 0, &V23682_292_57183, &P23682_291};	/*Property = CssMinMax*/
const Value V23682_293_57187 = {S0, NULL};	/*Value = False*/
const Property P23682_293 = {S15, 0, &V23682_293_57187, &P23682_292};	/*Property = CssOverflow*/
const Value V23682_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23682_294 = {S16, 0, &V23682_294_57189, &P23682_293};	/*Property = CssPosition*/
const Value V23682_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23682_295 = {S17, 0, &V23682_295_57192, &P23682_294};	/*Property = CssText*/
const Value V23682_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23682_297 = {S18, 0, &V23682_297_57195, &P23682_295};	/*Property = CssTransforms*/
const Value V23682_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23682_241 = {S19, 0, &V23682_241_55799, &P23682_297};	/*Property = CssTransitions*/
const Value V23682_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23682_298 = {S20, 0, &V23682_298_57206, &P23682_241};	/*Property = CssUI*/
const Value V23682_300_57215 = {S2, NULL};	/*Value = True*/
const Property P23682_300 = {S21, 0, &V23682_300_57215, &P23682_298};	/*Property = DataSet*/
const Value V23682_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23682_301 = {S22, 0, &V23682_301_57218, &P23682_300};	/*Property = DataUrl*/
const Value V23682_302_57221 = {S2, NULL};	/*Value = True*/
const Property P23682_302 = {S23, 0, &V23682_302_57221, &P23682_301};	/*Property = DeviceOrientation*/
const Value V23682_304_57227 = {S2, NULL};	/*Value = True*/
const Property P23682_304 = {S24, 0, &V23682_304_57227, &P23682_302};	/*Property = FileReader*/
const Value V23682_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23682_305 = {S25, 0, &V23682_305_57231, &P23682_304};	/*Property = FileSaver*/
const Value V23682_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23682_306 = {S26, 0, &V23682_306_57237, &P23682_305};	/*Property = FileWriter*/
const Value V23682_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23682_307 = {S27, 0, &V23682_307_57239, &P23682_306};	/*Property = FormData*/
const Value V23682_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23682_309 = {S28, 0, &V23682_309_57246, &P23682_307};	/*Property = Fullscreen*/
const Value V23682_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23682_310 = {S29, 0, &V23682_310_57248, &P23682_309};	/*Property = GeoLocation*/
const Value V23682_312_57254 = {S2, NULL};	/*Value = True*/
const Property P23682_312 = {S30, 0, &V23682_312_57254, &P23682_310};	/*Property = History*/
const Value V23682_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23682_314 = {S31, 0, &V23682_314_57261, &P23682_312};	/*Property = Html5*/
const Value V23682_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23682_313 = {S32, 0, &V23682_313_57258, &P23682_314};	/*Property = Html-Media-Capture*/
const Value V23682_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23682_315 = {S33, 0, &V23682_315_57264, &P23682_313};	/*Property = Iframe*/
const Value V23682_316_57266 = {S2, NULL};	/*Value = True*/
const Property P23682_316 = {S34, 0, &V23682_316_57266, &P23682_315};	/*Property = IndexedDB*/
const Value V23682_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23682_317 = {S35, 0, &V23682_317_57269, &P23682_316};	/*Property = Json*/
const Value V23682_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P23682_15 = {S37, 0, &V23682_15_29, &P23682_317};	/*Property = LayoutEngine*/
const Value V23682_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23682_344 = {S38, 0, &V23682_344_57272, &P23682_15};	/*Property = Masking*/
const Value V23682_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23682_320 = {S39, 0, &V23682_320_57290, &P23682_344};	/*Property = PostMessage*/
const Value V23682_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23682_321 = {S40, 0, &V23682_321_57293, &P23682_320};	/*Property = Progress*/
const Value V23682_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23682_322 = {S41, 0, &V23682_322_57296, &P23682_321};	/*Property = Prompts*/
const Value V23682_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23682_323 = {S42, 0, &V23682_323_57299, &P23682_322};	/*Property = Selector*/
const Value V23682_324_57305 = {S2, NULL};	/*Value = True*/
const Property P23682_324 = {S43, 0, &V23682_324_57305, &P23682_323};	/*Property = Svg*/
const Value V23682_327_57315 = {S0, NULL};	/*Value = False*/
const Property P23682_327 = {S44, 0, &V23682_327_57315, &P23682_324};	/*Property = TouchEvents*/
const Value V23682_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23682_328 = {S45, 0, &V23682_328_57318, &P23682_327};	/*Property = Track*/
const Value V23682_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23682_329 = {S46, 0, &V23682_329_57323, &P23682_328};	/*Property = Video*/
const Value V23682_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23682_330 = {S47, 0, &V23682_330_57326, &P23682_329};	/*Property = Viewport*/
const Value V23682_332_57338 = {S2, NULL};	/*Value = True*/
const Property P23682_332 = {S48, 0, &V23682_332_57338, &P23682_330};	/*Property = WebWorkers*/
const Value V23682_333_57341 = {S2, NULL};	/*Value = True*/
const Property P23682_333 = {S49, 0, &V23682_333_57341, &P23682_332};	/*Property = Xhr2*/
const Profile P23682 = {"23682", &P23682_333};	/*Profile ID - 23682*/
const Value V18728_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_279 = {S1, 0, &V18728_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18728_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_282 = {S3, 0, &V18728_282_57155, &P18728_279};	/*Property = BlobBuilder*/
const Value V18728_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_350 = {S4, 0, &V18728_350_57446, &P18728_282};	/*Property = Canvas*/
const Value V18728_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_283 = {S5, 0, &V18728_283_57161, &P18728_350};	/*Property = CssBackground*/
const Value V18728_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_284 = {S6, 0, &V18728_284_57164, &P18728_283};	/*Property = CssBorderImage*/
const Value V18728_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_285 = {S7, 0, &V18728_285_57167, &P18728_284};	/*Property = CssCanvas*/
const Value V18728_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_286 = {S8, 0, &V18728_286_57170, &P18728_285};	/*Property = CssColor*/
const Value V18728_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_237 = {S9, 0, &V18728_237_55786, &P18728_286};	/*Property = CssColumn*/
const Value V18728_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_287 = {S10, 0, &V18728_287_57173, &P18728_237};	/*Property = CssFlexbox*/
const Value V18728_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_288 = {S11, 0, &V18728_288_57176, &P18728_287};	/*Property = CssFont*/
const Value V18728_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_289 = {S12, 0, &V18728_289_86899, &P18728_288};	/*Property = CssImages*/
const Value V18728_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_291 = {S13, 0, &V18728_291_57182, &P18728_289};	/*Property = CssMediaQueries*/
const Value V18728_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_292 = {S14, 0, &V18728_292_57185, &P18728_291};	/*Property = CssMinMax*/
const Value V18728_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_293 = {S15, 0, &V18728_293_57188, &P18728_292};	/*Property = CssOverflow*/
const Value V18728_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_294 = {S16, 0, &V18728_294_57191, &P18728_293};	/*Property = CssPosition*/
const Value V18728_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_295 = {S17, 0, &V18728_295_57194, &P18728_294};	/*Property = CssText*/
const Value V18728_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_297 = {S18, 0, &V18728_297_57197, &P18728_295};	/*Property = CssTransforms*/
const Value V18728_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_241 = {S19, 0, &V18728_241_55801, &P18728_297};	/*Property = CssTransitions*/
const Value V18728_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_298 = {S20, 0, &V18728_298_57208, &P18728_241};	/*Property = CssUI*/
const Value V18728_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_300 = {S21, 0, &V18728_300_57217, &P18728_298};	/*Property = DataSet*/
const Value V18728_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_301 = {S22, 0, &V18728_301_57220, &P18728_300};	/*Property = DataUrl*/
const Value V18728_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_302 = {S23, 0, &V18728_302_57223, &P18728_301};	/*Property = DeviceOrientation*/
const Value V18728_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_304 = {S24, 0, &V18728_304_57229, &P18728_302};	/*Property = FileReader*/
const Value V18728_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_305 = {S25, 0, &V18728_305_57232, &P18728_304};	/*Property = FileSaver*/
const Value V18728_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_306 = {S26, 0, &V18728_306_57238, &P18728_305};	/*Property = FileWriter*/
const Value V18728_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_307 = {S27, 0, &V18728_307_57241, &P18728_306};	/*Property = FormData*/
const Value V18728_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_309 = {S28, 0, &V18728_309_57247, &P18728_307};	/*Property = Fullscreen*/
const Value V18728_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_310 = {S29, 0, &V18728_310_57250, &P18728_309};	/*Property = GeoLocation*/
const Value V18728_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_312 = {S30, 0, &V18728_312_57256, &P18728_310};	/*Property = History*/
const Value V18728_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_314 = {S31, 0, &V18728_314_57262, &P18728_312};	/*Property = Html5*/
const Value V18728_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_313 = {S32, 0, &V18728_313_57259, &P18728_314};	/*Property = Html-Media-Capture*/
const Value V18728_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_315 = {S33, 0, &V18728_315_57265, &P18728_313};	/*Property = Iframe*/
const Value V18728_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_316 = {S34, 0, &V18728_316_57268, &P18728_315};	/*Property = IndexedDB*/
const Value V18728_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_317 = {S35, 0, &V18728_317_57271, &P18728_316};	/*Property = Json*/
const Value V18728_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18728_15 = {S37, 0, &V18728_15_29, &P18728_317};	/*Property = LayoutEngine*/
const Value V18728_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_344 = {S38, 0, &V18728_344_57274, &P18728_15};	/*Property = Masking*/
const Value V18728_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_320 = {S39, 0, &V18728_320_57292, &P18728_344};	/*Property = PostMessage*/
const Value V18728_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_321 = {S40, 0, &V18728_321_57295, &P18728_320};	/*Property = Progress*/
const Value V18728_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_322 = {S41, 0, &V18728_322_57298, &P18728_321};	/*Property = Prompts*/
const Value V18728_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_323 = {S42, 0, &V18728_323_57301, &P18728_322};	/*Property = Selector*/
const Value V18728_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_324 = {S43, 0, &V18728_324_57307, &P18728_323};	/*Property = Svg*/
const Value V18728_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_327 = {S44, 0, &V18728_327_57316, &P18728_324};	/*Property = TouchEvents*/
const Value V18728_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_328 = {S45, 0, &V18728_328_57319, &P18728_327};	/*Property = Track*/
const Value V18728_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_329 = {S46, 0, &V18728_329_57325, &P18728_328};	/*Property = Video*/
const Value V18728_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_330 = {S47, 0, &V18728_330_57328, &P18728_329};	/*Property = Viewport*/
const Value V18728_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_332 = {S48, 0, &V18728_332_57340, &P18728_330};	/*Property = WebWorkers*/
const Value V18728_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18728_333 = {S49, 0, &V18728_333_57343, &P18728_332};	/*Property = Xhr2*/
const Profile P18728 = {"18728", &P18728_333};	/*Profile ID - 18728*/
const Value V21216_1_1 = {S2, NULL};	/*Value = True*/
const Property P21216_1 = {S50, 0, &V21216_1_1, NULL};	/*Property = IsMobile*/
const Value V21216_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21216_5 = {S52, 0, &V21216_5_1277, &P21216_1};	/*Property = ScreenPixelsHeight*/
const Value V21216_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P21216_4 = {S54, 0, &V21216_4_1278, &P21216_5};	/*Property = ScreenPixelsWidth*/
const Profile P21216 = {"21216", &P21216_4};	/*Profile ID - 21216*/
const Value V22429_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22429_279 = {S1, 0, &V22429_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22429_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22429_282 = {S3, 0, &V22429_282_57153, &P22429_279};	/*Property = BlobBuilder*/
const Value V22429_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22429_350 = {S4, 0, &V22429_350_57444, &P22429_282};	/*Property = Canvas*/
const Value V22429_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22429_283 = {S5, 0, &V22429_283_57159, &P22429_350};	/*Property = CssBackground*/
const Value V22429_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22429_284 = {S6, 0, &V22429_284_57163, &P22429_283};	/*Property = CssBorderImage*/
const Value V22429_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22429_285 = {S7, 0, &V22429_285_57166, &P22429_284};	/*Property = CssCanvas*/
const Value V22429_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22429_286 = {S8, 0, &V22429_286_57168, &P22429_285};	/*Property = CssColor*/
const Value V22429_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22429_237 = {S9, 0, &V22429_237_55784, &P22429_286};	/*Property = CssColumn*/
const Value V22429_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22429_287 = {S10, 0, &V22429_287_57172, &P22429_237};	/*Property = CssFlexbox*/
const Value V22429_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22429_288 = {S11, 0, &V22429_288_57175, &P22429_287};	/*Property = CssFont*/
const Value V22429_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22429_289 = {S12, 0, &V22429_289_57759, &P22429_288};	/*Property = CssImages*/
const Value V22429_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22429_291 = {S13, 0, &V22429_291_57180, &P22429_289};	/*Property = CssMediaQueries*/
const Value V22429_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22429_292 = {S14, 0, &V22429_292_57183, &P22429_291};	/*Property = CssMinMax*/
const Value V22429_293_57186 = {S2, NULL};	/*Value = True*/
const Property P22429_293 = {S15, 0, &V22429_293_57186, &P22429_292};	/*Property = CssOverflow*/
const Value V22429_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22429_294 = {S16, 0, &V22429_294_57189, &P22429_293};	/*Property = CssPosition*/
const Value V22429_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22429_295 = {S17, 0, &V22429_295_57192, &P22429_294};	/*Property = CssText*/
const Value V22429_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22429_297 = {S18, 0, &V22429_297_57195, &P22429_295};	/*Property = CssTransforms*/
const Value V22429_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22429_241 = {S19, 0, &V22429_241_55799, &P22429_297};	/*Property = CssTransitions*/
const Value V22429_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22429_298 = {S20, 0, &V22429_298_57206, &P22429_241};	/*Property = CssUI*/
const Value V22429_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22429_300 = {S21, 0, &V22429_300_57216, &P22429_298};	/*Property = DataSet*/
const Value V22429_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22429_301 = {S22, 0, &V22429_301_57218, &P22429_300};	/*Property = DataUrl*/
const Value V22429_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22429_302 = {S23, 0, &V22429_302_57222, &P22429_301};	/*Property = DeviceOrientation*/
const Value V22429_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22429_304 = {S24, 0, &V22429_304_57227, &P22429_302};	/*Property = FileReader*/
const Value V22429_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22429_305 = {S25, 0, &V22429_305_57231, &P22429_304};	/*Property = FileSaver*/
const Value V22429_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22429_306 = {S26, 0, &V22429_306_57237, &P22429_305};	/*Property = FileWriter*/
const Value V22429_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22429_307 = {S27, 0, &V22429_307_57239, &P22429_306};	/*Property = FormData*/
const Value V22429_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22429_309 = {S28, 0, &V22429_309_57246, &P22429_307};	/*Property = Fullscreen*/
const Value V22429_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22429_310 = {S29, 0, &V22429_310_57248, &P22429_309};	/*Property = GeoLocation*/
const Value V22429_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22429_312 = {S30, 0, &V22429_312_57254, &P22429_310};	/*Property = History*/
const Value V22429_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22429_314 = {S31, 0, &V22429_314_57261, &P22429_312};	/*Property = Html5*/
const Value V22429_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22429_313 = {S32, 0, &V22429_313_57258, &P22429_314};	/*Property = Html-Media-Capture*/
const Value V22429_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22429_315 = {S33, 0, &V22429_315_57264, &P22429_313};	/*Property = Iframe*/
const Value V22429_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22429_316 = {S34, 0, &V22429_316_57266, &P22429_315};	/*Property = IndexedDB*/
const Value V22429_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22429_317 = {S35, 0, &V22429_317_57269, &P22429_316};	/*Property = Json*/
const Value V22429_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22429_15 = {S37, 0, &V22429_15_28, &P22429_317};	/*Property = LayoutEngine*/
const Value V22429_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22429_344 = {S38, 0, &V22429_344_57272, &P22429_15};	/*Property = Masking*/
const Value V22429_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22429_320 = {S39, 0, &V22429_320_57290, &P22429_344};	/*Property = PostMessage*/
const Value V22429_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22429_321 = {S40, 0, &V22429_321_57293, &P22429_320};	/*Property = Progress*/
const Value V22429_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22429_322 = {S41, 0, &V22429_322_57296, &P22429_321};	/*Property = Prompts*/
const Value V22429_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22429_323 = {S42, 0, &V22429_323_57299, &P22429_322};	/*Property = Selector*/
const Value V22429_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22429_324 = {S43, 0, &V22429_324_57306, &P22429_323};	/*Property = Svg*/
const Value V22429_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22429_327 = {S44, 0, &V22429_327_57315, &P22429_324};	/*Property = TouchEvents*/
const Value V22429_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22429_328 = {S45, 0, &V22429_328_57318, &P22429_327};	/*Property = Track*/
const Value V22429_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22429_329 = {S46, 0, &V22429_329_57323, &P22429_328};	/*Property = Video*/
const Value V22429_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22429_330 = {S47, 0, &V22429_330_57326, &P22429_329};	/*Property = Viewport*/
const Value V22429_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22429_332 = {S48, 0, &V22429_332_57338, &P22429_330};	/*Property = WebWorkers*/
const Value V22429_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22429_333 = {S49, 0, &V22429_333_57341, &P22429_332};	/*Property = Xhr2*/
const Profile P22429 = {"22429", &P22429_333};	/*Profile ID - 22429*/
const Value V18722_1_1 = {S2, NULL};	/*Value = True*/
const Property P18722_1 = {S50, 0, &V18722_1_1, NULL};	/*Property = IsMobile*/
const Value V18722_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18722_5 = {S52, 0, &V18722_5_9981, &P18722_1};	/*Property = ScreenPixelsHeight*/
const Value V18722_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18722_4 = {S54, 0, &V18722_4_39098, &P18722_5};	/*Property = ScreenPixelsWidth*/
const Profile P18722 = {"18722", &P18722_4};	/*Profile ID - 18722*/
const Value V18636_1_1 = {S2, NULL};	/*Value = True*/
const Property P18636_1 = {S50, 0, &V18636_1_1, NULL};	/*Property = IsMobile*/
const Value V18636_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18636_5 = {S52, 0, &V18636_5_9981, &P18636_1};	/*Property = ScreenPixelsHeight*/
const Value V18636_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18636_4 = {S54, 0, &V18636_4_39098, &P18636_5};	/*Property = ScreenPixelsWidth*/
const Profile P18636 = {"18636", &P18636_4};	/*Profile ID - 18636*/
const Value V22359_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22359_279 = {S1, 0, &V22359_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22359_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22359_282 = {S3, 0, &V22359_282_57154, &P22359_279};	/*Property = BlobBuilder*/
const Value V22359_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22359_350 = {S4, 0, &V22359_350_57444, &P22359_282};	/*Property = Canvas*/
const Value V22359_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22359_283 = {S5, 0, &V22359_283_57159, &P22359_350};	/*Property = CssBackground*/
const Value V22359_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22359_284 = {S6, 0, &V22359_284_57163, &P22359_283};	/*Property = CssBorderImage*/
const Value V22359_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22359_285 = {S7, 0, &V22359_285_57166, &P22359_284};	/*Property = CssCanvas*/
const Value V22359_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22359_286 = {S8, 0, &V22359_286_57168, &P22359_285};	/*Property = CssColor*/
const Value V22359_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22359_237 = {S9, 0, &V22359_237_55784, &P22359_286};	/*Property = CssColumn*/
const Value V22359_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22359_287 = {S10, 0, &V22359_287_57172, &P22359_237};	/*Property = CssFlexbox*/
const Value V22359_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22359_288 = {S11, 0, &V22359_288_57175, &P22359_287};	/*Property = CssFont*/
const Value V22359_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22359_289 = {S12, 0, &V22359_289_57759, &P22359_288};	/*Property = CssImages*/
const Value V22359_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22359_291 = {S13, 0, &V22359_291_57181, &P22359_289};	/*Property = CssMediaQueries*/
const Value V22359_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22359_292 = {S14, 0, &V22359_292_57183, &P22359_291};	/*Property = CssMinMax*/
const Value V22359_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22359_293 = {S15, 0, &V22359_293_57187, &P22359_292};	/*Property = CssOverflow*/
const Value V22359_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22359_294 = {S16, 0, &V22359_294_57190, &P22359_293};	/*Property = CssPosition*/
const Value V22359_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22359_295 = {S17, 0, &V22359_295_57193, &P22359_294};	/*Property = CssText*/
const Value V22359_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22359_297 = {S18, 0, &V22359_297_57195, &P22359_295};	/*Property = CssTransforms*/
const Value V22359_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22359_241 = {S19, 0, &V22359_241_55799, &P22359_297};	/*Property = CssTransitions*/
const Value V22359_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22359_298 = {S20, 0, &V22359_298_57206, &P22359_241};	/*Property = CssUI*/
const Value V22359_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22359_300 = {S21, 0, &V22359_300_57216, &P22359_298};	/*Property = DataSet*/
const Value V22359_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22359_301 = {S22, 0, &V22359_301_57218, &P22359_300};	/*Property = DataUrl*/
const Value V22359_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22359_302 = {S23, 0, &V22359_302_57222, &P22359_301};	/*Property = DeviceOrientation*/
const Value V22359_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22359_304 = {S24, 0, &V22359_304_57228, &P22359_302};	/*Property = FileReader*/
const Value V22359_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22359_305 = {S25, 0, &V22359_305_57231, &P22359_304};	/*Property = FileSaver*/
const Value V22359_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22359_306 = {S26, 0, &V22359_306_57237, &P22359_305};	/*Property = FileWriter*/
const Value V22359_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22359_307 = {S27, 0, &V22359_307_57240, &P22359_306};	/*Property = FormData*/
const Value V22359_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22359_309 = {S28, 0, &V22359_309_57246, &P22359_307};	/*Property = Fullscreen*/
const Value V22359_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22359_310 = {S29, 0, &V22359_310_57248, &P22359_309};	/*Property = GeoLocation*/
const Value V22359_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22359_312 = {S30, 0, &V22359_312_57255, &P22359_310};	/*Property = History*/
const Value V22359_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22359_314 = {S31, 0, &V22359_314_57261, &P22359_312};	/*Property = Html5*/
const Value V22359_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22359_313 = {S32, 0, &V22359_313_57258, &P22359_314};	/*Property = Html-Media-Capture*/
const Value V22359_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22359_315 = {S33, 0, &V22359_315_57264, &P22359_313};	/*Property = Iframe*/
const Value V22359_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22359_316 = {S34, 0, &V22359_316_57267, &P22359_315};	/*Property = IndexedDB*/
const Value V22359_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22359_317 = {S35, 0, &V22359_317_57269, &P22359_316};	/*Property = Json*/
const Value V22359_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22359_15 = {S37, 0, &V22359_15_29, &P22359_317};	/*Property = LayoutEngine*/
const Value V22359_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22359_344 = {S38, 0, &V22359_344_57272, &P22359_15};	/*Property = Masking*/
const Value V22359_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22359_320 = {S39, 0, &V22359_320_57290, &P22359_344};	/*Property = PostMessage*/
const Value V22359_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22359_321 = {S40, 0, &V22359_321_57293, &P22359_320};	/*Property = Progress*/
const Value V22359_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22359_322 = {S41, 0, &V22359_322_57296, &P22359_321};	/*Property = Prompts*/
const Value V22359_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22359_323 = {S42, 0, &V22359_323_57299, &P22359_322};	/*Property = Selector*/
const Value V22359_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22359_324 = {S43, 0, &V22359_324_57306, &P22359_323};	/*Property = Svg*/
const Value V22359_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22359_327 = {S44, 0, &V22359_327_57315, &P22359_324};	/*Property = TouchEvents*/
const Value V22359_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22359_328 = {S45, 0, &V22359_328_57318, &P22359_327};	/*Property = Track*/
const Value V22359_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22359_329 = {S46, 0, &V22359_329_57323, &P22359_328};	/*Property = Video*/
const Value V22359_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22359_330 = {S47, 0, &V22359_330_57326, &P22359_329};	/*Property = Viewport*/
const Value V22359_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22359_332 = {S48, 0, &V22359_332_57339, &P22359_330};	/*Property = WebWorkers*/
const Value V22359_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22359_333 = {S49, 0, &V22359_333_57342, &P22359_332};	/*Property = Xhr2*/
const Profile P22359 = {"22359", &P22359_333};	/*Profile ID - 22359*/
const Value V22028_1_1 = {S2, NULL};	/*Value = True*/
const Property P22028_1 = {S50, 0, &V22028_1_1, NULL};	/*Property = IsMobile*/
const Value V22028_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22028_5 = {S52, 0, &V22028_5_1277, &P22028_1};	/*Property = ScreenPixelsHeight*/
const Value V22028_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22028_4 = {S54, 0, &V22028_4_2336, &P22028_5};	/*Property = ScreenPixelsWidth*/
const Profile P22028 = {"22028", &P22028_4};	/*Profile ID - 22028*/
const Value V20934_1_1 = {S2, NULL};	/*Value = True*/
const Property P20934_1 = {S50, 0, &V20934_1_1, NULL};	/*Property = IsMobile*/
const Value V20934_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20934_5 = {S52, 0, &V20934_5_1227, &P20934_1};	/*Property = ScreenPixelsHeight*/
const Value V20934_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20934_4 = {S54, 0, &V20934_4_1229, &P20934_5};	/*Property = ScreenPixelsWidth*/
const Profile P20934 = {"20934", &P20934_4};	/*Profile ID - 20934*/
const Value V21469_1_1 = {S2, NULL};	/*Value = True*/
const Property P21469_1 = {S50, 0, &V21469_1_1, NULL};	/*Property = IsMobile*/
const Value V21469_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21469_5 = {S52, 0, &V21469_5_1458, &P21469_1};	/*Property = ScreenPixelsHeight*/
const Value V21469_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21469_4 = {S54, 0, &V21469_4_1460, &P21469_5};	/*Property = ScreenPixelsWidth*/
const Profile P21469 = {"21469", &P21469_4};	/*Profile ID - 21469*/
const Value V22026_1_1 = {S2, NULL};	/*Value = True*/
const Property P22026_1 = {S50, 0, &V22026_1_1, NULL};	/*Property = IsMobile*/
const Value V22026_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22026_5 = {S52, 0, &V22026_5_1277, &P22026_1};	/*Property = ScreenPixelsHeight*/
const Value V22026_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22026_4 = {S54, 0, &V22026_4_2336, &P22026_5};	/*Property = ScreenPixelsWidth*/
const Profile P22026 = {"22026", &P22026_4};	/*Profile ID - 22026*/
const Value V21140_1_1 = {S2, NULL};	/*Value = True*/
const Property P21140_1 = {S50, 0, &V21140_1_1, NULL};	/*Property = IsMobile*/
const Value V21140_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21140_5 = {S52, 0, &V21140_5_1227, &P21140_1};	/*Property = ScreenPixelsHeight*/
const Value V21140_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21140_4 = {S54, 0, &V21140_4_1229, &P21140_5};	/*Property = ScreenPixelsWidth*/
const Profile P21140 = {"21140", &P21140_4};	/*Profile ID - 21140*/
const Value V21052_1_1 = {S2, NULL};	/*Value = True*/
const Property P21052_1 = {S50, 0, &V21052_1_1, NULL};	/*Property = IsMobile*/
const Value V21052_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21052_5 = {S52, 0, &V21052_5_1227, &P21052_1};	/*Property = ScreenPixelsHeight*/
const Value V21052_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21052_4 = {S54, 0, &V21052_4_1229, &P21052_5};	/*Property = ScreenPixelsWidth*/
const Profile P21052 = {"21052", &P21052_4};	/*Profile ID - 21052*/
const Value V20930_1_1 = {S2, NULL};	/*Value = True*/
const Property P20930_1 = {S50, 0, &V20930_1_1, NULL};	/*Property = IsMobile*/
const Value V20930_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20930_5 = {S52, 0, &V20930_5_1222, &P20930_1};	/*Property = ScreenPixelsHeight*/
const Value V20930_4_54464 = {S119, NULL};	/*Value = 280*/
const Property P20930_4 = {S54, 0, &V20930_4_54464, &P20930_5};	/*Property = ScreenPixelsWidth*/
const Profile P20930 = {"20930", &P20930_4};	/*Profile ID - 20930*/
const Value V21877_1_1 = {S2, NULL};	/*Value = True*/
const Property P21877_1 = {S50, 0, &V21877_1_1, NULL};	/*Property = IsMobile*/
const Value V21877_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21877_5 = {S52, 0, &V21877_5_1227, &P21877_1};	/*Property = ScreenPixelsHeight*/
const Value V21877_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21877_4 = {S54, 0, &V21877_4_1460, &P21877_5};	/*Property = ScreenPixelsWidth*/
const Profile P21877 = {"21877", &P21877_4};	/*Profile ID - 21877*/
const Value V23403_279_57148 = {S0, NULL};	/*Value = False*/
const Property P23403_279 = {S1, 0, &V23403_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V23403_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23403_282 = {S3, 0, &V23403_282_57154, &P23403_279};	/*Property = BlobBuilder*/
const Value V23403_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23403_350 = {S4, 0, &V23403_350_57444, &P23403_282};	/*Property = Canvas*/
const Value V23403_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23403_283 = {S5, 0, &V23403_283_57159, &P23403_350};	/*Property = CssBackground*/
const Value V23403_284_57162 = {S2, NULL};	/*Value = True*/
const Property P23403_284 = {S6, 0, &V23403_284_57162, &P23403_283};	/*Property = CssBorderImage*/
const Value V23403_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23403_285 = {S7, 0, &V23403_285_57166, &P23403_284};	/*Property = CssCanvas*/
const Value V23403_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23403_286 = {S8, 0, &V23403_286_57168, &P23403_285};	/*Property = CssColor*/
const Value V23403_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23403_237 = {S9, 0, &V23403_237_55786, &P23403_286};	/*Property = CssColumn*/
const Value V23403_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23403_287 = {S10, 0, &V23403_287_57172, &P23403_237};	/*Property = CssFlexbox*/
const Value V23403_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23403_288 = {S11, 0, &V23403_288_57174, &P23403_287};	/*Property = CssFont*/
const Value V23403_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23403_289 = {S12, 0, &V23403_289_57759, &P23403_288};	/*Property = CssImages*/
const Value V23403_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23403_291 = {S13, 0, &V23403_291_57180, &P23403_289};	/*Property = CssMediaQueries*/
const Value V23403_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23403_292 = {S14, 0, &V23403_292_57183, &P23403_291};	/*Property = CssMinMax*/
const Value V23403_293_57186 = {S2, NULL};	/*Value = True*/
const Property P23403_293 = {S15, 0, &V23403_293_57186, &P23403_292};	/*Property = CssOverflow*/
const Value V23403_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23403_294 = {S16, 0, &V23403_294_57189, &P23403_293};	/*Property = CssPosition*/
const Value V23403_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23403_295 = {S17, 0, &V23403_295_57192, &P23403_294};	/*Property = CssText*/
const Value V23403_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23403_297 = {S18, 0, &V23403_297_57195, &P23403_295};	/*Property = CssTransforms*/
const Value V23403_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23403_241 = {S19, 0, &V23403_241_55799, &P23403_297};	/*Property = CssTransitions*/
const Value V23403_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23403_298 = {S20, 0, &V23403_298_57206, &P23403_241};	/*Property = CssUI*/
const Value V23403_300_57215 = {S2, NULL};	/*Value = True*/
const Property P23403_300 = {S21, 0, &V23403_300_57215, &P23403_298};	/*Property = DataSet*/
const Value V23403_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23403_301 = {S22, 0, &V23403_301_57218, &P23403_300};	/*Property = DataUrl*/
const Value V23403_302_57221 = {S2, NULL};	/*Value = True*/
const Property P23403_302 = {S23, 0, &V23403_302_57221, &P23403_301};	/*Property = DeviceOrientation*/
const Value V23403_304_57228 = {S0, NULL};	/*Value = False*/
const Property P23403_304 = {S24, 0, &V23403_304_57228, &P23403_302};	/*Property = FileReader*/
const Value V23403_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23403_305 = {S25, 0, &V23403_305_57231, &P23403_304};	/*Property = FileSaver*/
const Value V23403_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23403_306 = {S26, 0, &V23403_306_57237, &P23403_305};	/*Property = FileWriter*/
const Value V23403_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23403_307 = {S27, 0, &V23403_307_57239, &P23403_306};	/*Property = FormData*/
const Value V23403_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23403_309 = {S28, 0, &V23403_309_57246, &P23403_307};	/*Property = Fullscreen*/
const Value V23403_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23403_310 = {S29, 0, &V23403_310_57248, &P23403_309};	/*Property = GeoLocation*/
const Value V23403_312_57255 = {S0, NULL};	/*Value = False*/
const Property P23403_312 = {S30, 0, &V23403_312_57255, &P23403_310};	/*Property = History*/
const Value V23403_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23403_314 = {S31, 0, &V23403_314_57261, &P23403_312};	/*Property = Html5*/
const Value V23403_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23403_313 = {S32, 0, &V23403_313_57258, &P23403_314};	/*Property = Html-Media-Capture*/
const Value V23403_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23403_315 = {S33, 0, &V23403_315_57264, &P23403_313};	/*Property = Iframe*/
const Value V23403_316_57267 = {S0, NULL};	/*Value = False*/
const Property P23403_316 = {S34, 0, &V23403_316_57267, &P23403_315};	/*Property = IndexedDB*/
const Value V23403_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23403_317 = {S35, 0, &V23403_317_57269, &P23403_316};	/*Property = Json*/
const Value V23403_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P23403_15 = {S37, 0, &V23403_15_54167, &P23403_317};	/*Property = LayoutEngine*/
const Value V23403_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23403_344 = {S38, 0, &V23403_344_57272, &P23403_15};	/*Property = Masking*/
const Value V23403_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23403_320 = {S39, 0, &V23403_320_57290, &P23403_344};	/*Property = PostMessage*/
const Value V23403_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23403_321 = {S40, 0, &V23403_321_57293, &P23403_320};	/*Property = Progress*/
const Value V23403_322_57297 = {S0, NULL};	/*Value = False*/
const Property P23403_322 = {S41, 0, &V23403_322_57297, &P23403_321};	/*Property = Prompts*/
const Value V23403_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23403_323 = {S42, 0, &V23403_323_57299, &P23403_322};	/*Property = Selector*/
const Value V23403_324_57306 = {S0, NULL};	/*Value = False*/
const Property P23403_324 = {S43, 0, &V23403_324_57306, &P23403_323};	/*Property = Svg*/
const Value V23403_327_57314 = {S2, NULL};	/*Value = True*/
const Property P23403_327 = {S44, 0, &V23403_327_57314, &P23403_324};	/*Property = TouchEvents*/
const Value V23403_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23403_328 = {S45, 0, &V23403_328_57318, &P23403_327};	/*Property = Track*/
const Value V23403_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23403_329 = {S46, 0, &V23403_329_57323, &P23403_328};	/*Property = Video*/
const Value V23403_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23403_330 = {S47, 0, &V23403_330_57326, &P23403_329};	/*Property = Viewport*/
const Value V23403_332_57339 = {S0, NULL};	/*Value = False*/
const Property P23403_332 = {S48, 0, &V23403_332_57339, &P23403_330};	/*Property = WebWorkers*/
const Value V23403_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23403_333 = {S49, 0, &V23403_333_57342, &P23403_332};	/*Property = Xhr2*/
const Profile P23403 = {"23403", &P23403_333};	/*Profile ID - 23403*/
const Value V20767_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_279 = {S1, 0, &V20767_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20767_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_282 = {S3, 0, &V20767_282_57155, &P20767_279};	/*Property = BlobBuilder*/
const Value V20767_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_350 = {S4, 0, &V20767_350_57446, &P20767_282};	/*Property = Canvas*/
const Value V20767_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_283 = {S5, 0, &V20767_283_57161, &P20767_350};	/*Property = CssBackground*/
const Value V20767_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_284 = {S6, 0, &V20767_284_57164, &P20767_283};	/*Property = CssBorderImage*/
const Value V20767_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_285 = {S7, 0, &V20767_285_57167, &P20767_284};	/*Property = CssCanvas*/
const Value V20767_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_286 = {S8, 0, &V20767_286_57170, &P20767_285};	/*Property = CssColor*/
const Value V20767_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_237 = {S9, 0, &V20767_237_55786, &P20767_286};	/*Property = CssColumn*/
const Value V20767_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_287 = {S10, 0, &V20767_287_57173, &P20767_237};	/*Property = CssFlexbox*/
const Value V20767_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_288 = {S11, 0, &V20767_288_57176, &P20767_287};	/*Property = CssFont*/
const Value V20767_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_289 = {S12, 0, &V20767_289_86899, &P20767_288};	/*Property = CssImages*/
const Value V20767_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_291 = {S13, 0, &V20767_291_57182, &P20767_289};	/*Property = CssMediaQueries*/
const Value V20767_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_292 = {S14, 0, &V20767_292_57185, &P20767_291};	/*Property = CssMinMax*/
const Value V20767_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_293 = {S15, 0, &V20767_293_57188, &P20767_292};	/*Property = CssOverflow*/
const Value V20767_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_294 = {S16, 0, &V20767_294_57191, &P20767_293};	/*Property = CssPosition*/
const Value V20767_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_295 = {S17, 0, &V20767_295_57194, &P20767_294};	/*Property = CssText*/
const Value V20767_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_297 = {S18, 0, &V20767_297_57197, &P20767_295};	/*Property = CssTransforms*/
const Value V20767_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_241 = {S19, 0, &V20767_241_55801, &P20767_297};	/*Property = CssTransitions*/
const Value V20767_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_298 = {S20, 0, &V20767_298_57208, &P20767_241};	/*Property = CssUI*/
const Value V20767_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_300 = {S21, 0, &V20767_300_57217, &P20767_298};	/*Property = DataSet*/
const Value V20767_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_301 = {S22, 0, &V20767_301_57220, &P20767_300};	/*Property = DataUrl*/
const Value V20767_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_302 = {S23, 0, &V20767_302_57223, &P20767_301};	/*Property = DeviceOrientation*/
const Value V20767_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_304 = {S24, 0, &V20767_304_57229, &P20767_302};	/*Property = FileReader*/
const Value V20767_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_305 = {S25, 0, &V20767_305_57232, &P20767_304};	/*Property = FileSaver*/
const Value V20767_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_306 = {S26, 0, &V20767_306_57238, &P20767_305};	/*Property = FileWriter*/
const Value V20767_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_307 = {S27, 0, &V20767_307_57241, &P20767_306};	/*Property = FormData*/
const Value V20767_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_309 = {S28, 0, &V20767_309_57247, &P20767_307};	/*Property = Fullscreen*/
const Value V20767_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_310 = {S29, 0, &V20767_310_57250, &P20767_309};	/*Property = GeoLocation*/
const Value V20767_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_312 = {S30, 0, &V20767_312_57256, &P20767_310};	/*Property = History*/
const Value V20767_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_314 = {S31, 0, &V20767_314_57262, &P20767_312};	/*Property = Html5*/
const Value V20767_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_313 = {S32, 0, &V20767_313_57259, &P20767_314};	/*Property = Html-Media-Capture*/
const Value V20767_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_315 = {S33, 0, &V20767_315_57265, &P20767_313};	/*Property = Iframe*/
const Value V20767_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_316 = {S34, 0, &V20767_316_57268, &P20767_315};	/*Property = IndexedDB*/
const Value V20767_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_317 = {S35, 0, &V20767_317_57271, &P20767_316};	/*Property = Json*/
const Value V20767_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P20767_15 = {S37, 0, &V20767_15_90837, &P20767_317};	/*Property = LayoutEngine*/
const Value V20767_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_344 = {S38, 0, &V20767_344_57274, &P20767_15};	/*Property = Masking*/
const Value V20767_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_320 = {S39, 0, &V20767_320_57292, &P20767_344};	/*Property = PostMessage*/
const Value V20767_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_321 = {S40, 0, &V20767_321_57295, &P20767_320};	/*Property = Progress*/
const Value V20767_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_322 = {S41, 0, &V20767_322_57298, &P20767_321};	/*Property = Prompts*/
const Value V20767_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_323 = {S42, 0, &V20767_323_57301, &P20767_322};	/*Property = Selector*/
const Value V20767_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_324 = {S43, 0, &V20767_324_57307, &P20767_323};	/*Property = Svg*/
const Value V20767_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_327 = {S44, 0, &V20767_327_57316, &P20767_324};	/*Property = TouchEvents*/
const Value V20767_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_328 = {S45, 0, &V20767_328_57319, &P20767_327};	/*Property = Track*/
const Value V20767_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_329 = {S46, 0, &V20767_329_57325, &P20767_328};	/*Property = Video*/
const Value V20767_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_330 = {S47, 0, &V20767_330_57328, &P20767_329};	/*Property = Viewport*/
const Value V20767_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_332 = {S48, 0, &V20767_332_57340, &P20767_330};	/*Property = WebWorkers*/
const Value V20767_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20767_333 = {S49, 0, &V20767_333_57343, &P20767_332};	/*Property = Xhr2*/
const Profile P20767 = {"20767", &P20767_333};	/*Profile ID - 20767*/
const Value V18671_1_1 = {S2, NULL};	/*Value = True*/
const Property P18671_1 = {S50, 0, &V18671_1_1, NULL};	/*Property = IsMobile*/
const Value V18671_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P18671_5 = {S52, 0, &V18671_5_1293, &P18671_1};	/*Property = ScreenPixelsHeight*/
const Value V18671_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P18671_4 = {S54, 0, &V18671_4_1296, &P18671_5};	/*Property = ScreenPixelsWidth*/
const Profile P18671 = {"18671", &P18671_4};	/*Profile ID - 18671*/
const Value V22538_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22538_279 = {S1, 0, &V22538_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22538_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22538_282 = {S3, 0, &V22538_282_57154, &P22538_279};	/*Property = BlobBuilder*/
const Value V22538_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22538_350 = {S4, 0, &V22538_350_57444, &P22538_282};	/*Property = Canvas*/
const Value V22538_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22538_283 = {S5, 0, &V22538_283_57159, &P22538_350};	/*Property = CssBackground*/
const Value V22538_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22538_284 = {S6, 0, &V22538_284_57163, &P22538_283};	/*Property = CssBorderImage*/
const Value V22538_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22538_285 = {S7, 0, &V22538_285_57166, &P22538_284};	/*Property = CssCanvas*/
const Value V22538_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22538_286 = {S8, 0, &V22538_286_57168, &P22538_285};	/*Property = CssColor*/
const Value V22538_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22538_237 = {S9, 0, &V22538_237_55784, &P22538_286};	/*Property = CssColumn*/
const Value V22538_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22538_287 = {S10, 0, &V22538_287_57172, &P22538_237};	/*Property = CssFlexbox*/
const Value V22538_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22538_288 = {S11, 0, &V22538_288_57175, &P22538_287};	/*Property = CssFont*/
const Value V22538_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22538_289 = {S12, 0, &V22538_289_57759, &P22538_288};	/*Property = CssImages*/
const Value V22538_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22538_291 = {S13, 0, &V22538_291_57181, &P22538_289};	/*Property = CssMediaQueries*/
const Value V22538_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22538_292 = {S14, 0, &V22538_292_57183, &P22538_291};	/*Property = CssMinMax*/
const Value V22538_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22538_293 = {S15, 0, &V22538_293_57187, &P22538_292};	/*Property = CssOverflow*/
const Value V22538_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22538_294 = {S16, 0, &V22538_294_57190, &P22538_293};	/*Property = CssPosition*/
const Value V22538_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22538_295 = {S17, 0, &V22538_295_57193, &P22538_294};	/*Property = CssText*/
const Value V22538_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22538_297 = {S18, 0, &V22538_297_57195, &P22538_295};	/*Property = CssTransforms*/
const Value V22538_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22538_241 = {S19, 0, &V22538_241_55799, &P22538_297};	/*Property = CssTransitions*/
const Value V22538_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22538_298 = {S20, 0, &V22538_298_57206, &P22538_241};	/*Property = CssUI*/
const Value V22538_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22538_300 = {S21, 0, &V22538_300_57216, &P22538_298};	/*Property = DataSet*/
const Value V22538_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22538_301 = {S22, 0, &V22538_301_57218, &P22538_300};	/*Property = DataUrl*/
const Value V22538_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22538_302 = {S23, 0, &V22538_302_57222, &P22538_301};	/*Property = DeviceOrientation*/
const Value V22538_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22538_304 = {S24, 0, &V22538_304_57228, &P22538_302};	/*Property = FileReader*/
const Value V22538_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22538_305 = {S25, 0, &V22538_305_57231, &P22538_304};	/*Property = FileSaver*/
const Value V22538_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22538_306 = {S26, 0, &V22538_306_57237, &P22538_305};	/*Property = FileWriter*/
const Value V22538_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22538_307 = {S27, 0, &V22538_307_57240, &P22538_306};	/*Property = FormData*/
const Value V22538_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22538_309 = {S28, 0, &V22538_309_57246, &P22538_307};	/*Property = Fullscreen*/
const Value V22538_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22538_310 = {S29, 0, &V22538_310_57248, &P22538_309};	/*Property = GeoLocation*/
const Value V22538_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22538_312 = {S30, 0, &V22538_312_57255, &P22538_310};	/*Property = History*/
const Value V22538_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22538_314 = {S31, 0, &V22538_314_57261, &P22538_312};	/*Property = Html5*/
const Value V22538_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22538_313 = {S32, 0, &V22538_313_57258, &P22538_314};	/*Property = Html-Media-Capture*/
const Value V22538_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22538_315 = {S33, 0, &V22538_315_57264, &P22538_313};	/*Property = Iframe*/
const Value V22538_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22538_316 = {S34, 0, &V22538_316_57267, &P22538_315};	/*Property = IndexedDB*/
const Value V22538_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22538_317 = {S35, 0, &V22538_317_57269, &P22538_316};	/*Property = Json*/
const Value V22538_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22538_15 = {S37, 0, &V22538_15_29, &P22538_317};	/*Property = LayoutEngine*/
const Value V22538_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22538_344 = {S38, 0, &V22538_344_57272, &P22538_15};	/*Property = Masking*/
const Value V22538_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22538_320 = {S39, 0, &V22538_320_57290, &P22538_344};	/*Property = PostMessage*/
const Value V22538_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22538_321 = {S40, 0, &V22538_321_57293, &P22538_320};	/*Property = Progress*/
const Value V22538_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22538_322 = {S41, 0, &V22538_322_57296, &P22538_321};	/*Property = Prompts*/
const Value V22538_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22538_323 = {S42, 0, &V22538_323_57299, &P22538_322};	/*Property = Selector*/
const Value V22538_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22538_324 = {S43, 0, &V22538_324_57306, &P22538_323};	/*Property = Svg*/
const Value V22538_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22538_327 = {S44, 0, &V22538_327_57315, &P22538_324};	/*Property = TouchEvents*/
const Value V22538_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22538_328 = {S45, 0, &V22538_328_57318, &P22538_327};	/*Property = Track*/
const Value V22538_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22538_329 = {S46, 0, &V22538_329_57323, &P22538_328};	/*Property = Video*/
const Value V22538_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22538_330 = {S47, 0, &V22538_330_57326, &P22538_329};	/*Property = Viewport*/
const Value V22538_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22538_332 = {S48, 0, &V22538_332_57339, &P22538_330};	/*Property = WebWorkers*/
const Value V22538_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22538_333 = {S49, 0, &V22538_333_57342, &P22538_332};	/*Property = Xhr2*/
const Profile P22538 = {"22538", &P22538_333};	/*Profile ID - 22538*/
const Value V18513_1_1 = {S2, NULL};	/*Value = True*/
const Property P18513_1 = {S50, 0, &V18513_1_1, NULL};	/*Property = IsMobile*/
const Value V18513_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18513_5 = {S52, 0, &V18513_5_1277, &P18513_1};	/*Property = ScreenPixelsHeight*/
const Value V18513_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18513_4 = {S54, 0, &V18513_4_1279, &P18513_5};	/*Property = ScreenPixelsWidth*/
const Profile P18513 = {"18513", &P18513_4};	/*Profile ID - 18513*/
const Value V18511_1_1 = {S2, NULL};	/*Value = True*/
const Property P18511_1 = {S50, 0, &V18511_1_1, NULL};	/*Property = IsMobile*/
const Value V18511_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18511_5 = {S52, 0, &V18511_5_1227, &P18511_1};	/*Property = ScreenPixelsHeight*/
const Value V18511_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18511_4 = {S54, 0, &V18511_4_1229, &P18511_5};	/*Property = ScreenPixelsWidth*/
const Profile P18511 = {"18511", &P18511_4};	/*Profile ID - 18511*/
const Value V21162_1_1 = {S2, NULL};	/*Value = True*/
const Property P21162_1 = {S50, 0, &V21162_1_1, NULL};	/*Property = IsMobile*/
const Value V21162_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21162_5 = {S52, 0, &V21162_5_37, &P21162_1};	/*Property = ScreenPixelsHeight*/
const Value V21162_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21162_4 = {S54, 0, &V21162_4_36, &P21162_5};	/*Property = ScreenPixelsWidth*/
const Profile P21162 = {"21162", &P21162_4};	/*Profile ID - 21162*/
const Value V18430_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18430_279 = {S1, 0, &V18430_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18430_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18430_282 = {S3, 0, &V18430_282_57154, &P18430_279};	/*Property = BlobBuilder*/
const Value V18430_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18430_350 = {S4, 0, &V18430_350_57444, &P18430_282};	/*Property = Canvas*/
const Value V18430_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18430_283 = {S5, 0, &V18430_283_57159, &P18430_350};	/*Property = CssBackground*/
const Value V18430_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18430_284 = {S6, 0, &V18430_284_57163, &P18430_283};	/*Property = CssBorderImage*/
const Value V18430_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18430_285 = {S7, 0, &V18430_285_57166, &P18430_284};	/*Property = CssCanvas*/
const Value V18430_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18430_286 = {S8, 0, &V18430_286_57168, &P18430_285};	/*Property = CssColor*/
const Value V18430_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18430_237 = {S9, 0, &V18430_237_55786, &P18430_286};	/*Property = CssColumn*/
const Value V18430_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18430_287 = {S10, 0, &V18430_287_57172, &P18430_237};	/*Property = CssFlexbox*/
const Value V18430_288_57174 = {S2, NULL};	/*Value = True*/
const Property P18430_288 = {S11, 0, &V18430_288_57174, &P18430_287};	/*Property = CssFont*/
const Value V18430_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18430_289 = {S12, 0, &V18430_289_57759, &P18430_288};	/*Property = CssImages*/
const Value V18430_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18430_291 = {S13, 0, &V18430_291_57180, &P18430_289};	/*Property = CssMediaQueries*/
const Value V18430_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18430_292 = {S14, 0, &V18430_292_57183, &P18430_291};	/*Property = CssMinMax*/
const Value V18430_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18430_293 = {S15, 0, &V18430_293_57187, &P18430_292};	/*Property = CssOverflow*/
const Value V18430_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18430_294 = {S16, 0, &V18430_294_57189, &P18430_293};	/*Property = CssPosition*/
const Value V18430_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18430_295 = {S17, 0, &V18430_295_57192, &P18430_294};	/*Property = CssText*/
const Value V18430_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18430_297 = {S18, 0, &V18430_297_57195, &P18430_295};	/*Property = CssTransforms*/
const Value V18430_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18430_241 = {S19, 0, &V18430_241_55799, &P18430_297};	/*Property = CssTransitions*/
const Value V18430_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18430_298 = {S20, 0, &V18430_298_57206, &P18430_241};	/*Property = CssUI*/
const Value V18430_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18430_300 = {S21, 0, &V18430_300_57216, &P18430_298};	/*Property = DataSet*/
const Value V18430_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18430_301 = {S22, 0, &V18430_301_57218, &P18430_300};	/*Property = DataUrl*/
const Value V18430_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18430_302 = {S23, 0, &V18430_302_57222, &P18430_301};	/*Property = DeviceOrientation*/
const Value V18430_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18430_304 = {S24, 0, &V18430_304_57227, &P18430_302};	/*Property = FileReader*/
const Value V18430_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18430_305 = {S25, 0, &V18430_305_57231, &P18430_304};	/*Property = FileSaver*/
const Value V18430_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18430_306 = {S26, 0, &V18430_306_57237, &P18430_305};	/*Property = FileWriter*/
const Value V18430_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18430_307 = {S27, 0, &V18430_307_57239, &P18430_306};	/*Property = FormData*/
const Value V18430_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18430_309 = {S28, 0, &V18430_309_57246, &P18430_307};	/*Property = Fullscreen*/
const Value V18430_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18430_310 = {S29, 0, &V18430_310_57248, &P18430_309};	/*Property = GeoLocation*/
const Value V18430_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18430_312 = {S30, 0, &V18430_312_57254, &P18430_310};	/*Property = History*/
const Value V18430_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18430_314 = {S31, 0, &V18430_314_57261, &P18430_312};	/*Property = Html5*/
const Value V18430_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18430_313 = {S32, 0, &V18430_313_57258, &P18430_314};	/*Property = Html-Media-Capture*/
const Value V18430_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18430_315 = {S33, 0, &V18430_315_57264, &P18430_313};	/*Property = Iframe*/
const Value V18430_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18430_316 = {S34, 0, &V18430_316_57267, &P18430_315};	/*Property = IndexedDB*/
const Value V18430_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18430_317 = {S35, 0, &V18430_317_57269, &P18430_316};	/*Property = Json*/
const Value V18430_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18430_15 = {S37, 0, &V18430_15_28, &P18430_317};	/*Property = LayoutEngine*/
const Value V18430_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18430_344 = {S38, 0, &V18430_344_57272, &P18430_15};	/*Property = Masking*/
const Value V18430_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18430_320 = {S39, 0, &V18430_320_57290, &P18430_344};	/*Property = PostMessage*/
const Value V18430_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18430_321 = {S40, 0, &V18430_321_57293, &P18430_320};	/*Property = Progress*/
const Value V18430_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18430_322 = {S41, 0, &V18430_322_57296, &P18430_321};	/*Property = Prompts*/
const Value V18430_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18430_323 = {S42, 0, &V18430_323_57299, &P18430_322};	/*Property = Selector*/
const Value V18430_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18430_324 = {S43, 0, &V18430_324_57306, &P18430_323};	/*Property = Svg*/
const Value V18430_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18430_327 = {S44, 0, &V18430_327_57315, &P18430_324};	/*Property = TouchEvents*/
const Value V18430_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18430_328 = {S45, 0, &V18430_328_57318, &P18430_327};	/*Property = Track*/
const Value V18430_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18430_329 = {S46, 0, &V18430_329_57323, &P18430_328};	/*Property = Video*/
const Value V18430_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18430_330 = {S47, 0, &V18430_330_57326, &P18430_329};	/*Property = Viewport*/
const Value V18430_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18430_332 = {S48, 0, &V18430_332_57339, &P18430_330};	/*Property = WebWorkers*/
const Value V18430_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18430_333 = {S49, 0, &V18430_333_57342, &P18430_332};	/*Property = Xhr2*/
const Profile P18430 = {"18430", &P18430_333};	/*Profile ID - 18430*/
const Value V21018_1_1 = {S2, NULL};	/*Value = True*/
const Property P21018_1 = {S50, 0, &V21018_1_1, NULL};	/*Property = IsMobile*/
const Value V21018_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P21018_5 = {S52, 0, &V21018_5_59138, &P21018_1};	/*Property = ScreenPixelsHeight*/
const Value V21018_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P21018_4 = {S54, 0, &V21018_4_59970, &P21018_5};	/*Property = ScreenPixelsWidth*/
const Profile P21018 = {"21018", &P21018_4};	/*Profile ID - 21018*/
const Profile P18375 = {"18375", NULL};	/*Profile ID - 18375*/
const Value V23752_1_1 = {S2, NULL};	/*Value = True*/
const Property P23752_1 = {S50, 0, &V23752_1_1, NULL};	/*Property = IsMobile*/
const Value V23752_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23752_5 = {S52, 0, &V23752_5_1277, &P23752_1};	/*Property = ScreenPixelsHeight*/
const Value V23752_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23752_4 = {S54, 0, &V23752_4_2336, &P23752_5};	/*Property = ScreenPixelsWidth*/
const Profile P23752 = {"23752", &P23752_4};	/*Profile ID - 23752*/
const Value V18711_1_1 = {S2, NULL};	/*Value = True*/
const Property P18711_1 = {S50, 0, &V18711_1_1, NULL};	/*Property = IsMobile*/
const Value V18711_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P18711_5 = {S52, 0, &V18711_5_1293, &P18711_1};	/*Property = ScreenPixelsHeight*/
const Value V18711_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P18711_4 = {S54, 0, &V18711_4_1296, &P18711_5};	/*Property = ScreenPixelsWidth*/
const Profile P18711 = {"18711", &P18711_4};	/*Profile ID - 18711*/
const Value V19848_1_1 = {S2, NULL};	/*Value = True*/
const Property P19848_1 = {S50, 0, &V19848_1_1, NULL};	/*Property = IsMobile*/
const Value V19848_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19848_5 = {S52, 0, &V19848_5_9981, &P19848_1};	/*Property = ScreenPixelsHeight*/
const Value V19848_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P19848_4 = {S54, 0, &V19848_4_39098, &P19848_5};	/*Property = ScreenPixelsWidth*/
const Profile P19848 = {"19848", &P19848_4};	/*Profile ID - 19848*/
const Value V18723_1_1 = {S2, NULL};	/*Value = True*/
const Property P18723_1 = {S50, 0, &V18723_1_1, NULL};	/*Property = IsMobile*/
const Value V18723_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18723_5 = {S52, 0, &V18723_5_1227, &P18723_1};	/*Property = ScreenPixelsHeight*/
const Value V18723_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18723_4 = {S54, 0, &V18723_4_1229, &P18723_5};	/*Property = ScreenPixelsWidth*/
const Profile P18723 = {"18723", &P18723_4};	/*Profile ID - 18723*/
const Value V21669_1_1 = {S2, NULL};	/*Value = True*/
const Property P21669_1 = {S50, 0, &V21669_1_1, NULL};	/*Property = IsMobile*/
const Value V21669_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21669_5 = {S52, 0, &V21669_5_1458, &P21669_1};	/*Property = ScreenPixelsHeight*/
const Value V21669_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21669_4 = {S54, 0, &V21669_4_1460, &P21669_5};	/*Property = ScreenPixelsWidth*/
const Profile P21669 = {"21669", &P21669_4};	/*Profile ID - 21669*/
const Value V19750_1_1 = {S2, NULL};	/*Value = True*/
const Property P19750_1 = {S50, 0, &V19750_1_1, NULL};	/*Property = IsMobile*/
const Value V19750_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P19750_5 = {S52, 0, &V19750_5_1222, &P19750_1};	/*Property = ScreenPixelsHeight*/
const Value V19750_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P19750_4 = {S54, 0, &V19750_4_1224, &P19750_5};	/*Property = ScreenPixelsWidth*/
const Profile P19750 = {"19750", &P19750_4};	/*Profile ID - 19750*/
const Value V24194_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24194_279 = {S1, 0, &V24194_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24194_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24194_282 = {S3, 0, &V24194_282_57154, &P24194_279};	/*Property = BlobBuilder*/
const Value V24194_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24194_350 = {S4, 0, &V24194_350_57444, &P24194_282};	/*Property = Canvas*/
const Value V24194_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24194_283 = {S5, 0, &V24194_283_57159, &P24194_350};	/*Property = CssBackground*/
const Value V24194_284_57163 = {S0, NULL};	/*Value = False*/
const Property P24194_284 = {S6, 0, &V24194_284_57163, &P24194_283};	/*Property = CssBorderImage*/
const Value V24194_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24194_285 = {S7, 0, &V24194_285_57166, &P24194_284};	/*Property = CssCanvas*/
const Value V24194_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24194_286 = {S8, 0, &V24194_286_57168, &P24194_285};	/*Property = CssColor*/
const Value V24194_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24194_237 = {S9, 0, &V24194_237_55786, &P24194_286};	/*Property = CssColumn*/
const Value V24194_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24194_287 = {S10, 0, &V24194_287_57172, &P24194_237};	/*Property = CssFlexbox*/
const Value V24194_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24194_288 = {S11, 0, &V24194_288_57175, &P24194_287};	/*Property = CssFont*/
const Value V24194_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24194_289 = {S12, 0, &V24194_289_57759, &P24194_288};	/*Property = CssImages*/
const Value V24194_291_57181 = {S0, NULL};	/*Value = False*/
const Property P24194_291 = {S13, 0, &V24194_291_57181, &P24194_289};	/*Property = CssMediaQueries*/
const Value V24194_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24194_292 = {S14, 0, &V24194_292_57183, &P24194_291};	/*Property = CssMinMax*/
const Value V24194_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24194_293 = {S15, 0, &V24194_293_57187, &P24194_292};	/*Property = CssOverflow*/
const Value V24194_294_57190 = {S0, NULL};	/*Value = False*/
const Property P24194_294 = {S16, 0, &V24194_294_57190, &P24194_293};	/*Property = CssPosition*/
const Value V24194_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24194_295 = {S17, 0, &V24194_295_57192, &P24194_294};	/*Property = CssText*/
const Value V24194_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24194_297 = {S18, 0, &V24194_297_57195, &P24194_295};	/*Property = CssTransforms*/
const Value V24194_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24194_241 = {S19, 0, &V24194_241_55799, &P24194_297};	/*Property = CssTransitions*/
const Value V24194_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24194_298 = {S20, 0, &V24194_298_57206, &P24194_241};	/*Property = CssUI*/
const Value V24194_300_57216 = {S0, NULL};	/*Value = False*/
const Property P24194_300 = {S21, 0, &V24194_300_57216, &P24194_298};	/*Property = DataSet*/
const Value V24194_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24194_301 = {S22, 0, &V24194_301_57218, &P24194_300};	/*Property = DataUrl*/
const Value V24194_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24194_302 = {S23, 0, &V24194_302_57222, &P24194_301};	/*Property = DeviceOrientation*/
const Value V24194_304_57228 = {S0, NULL};	/*Value = False*/
const Property P24194_304 = {S24, 0, &V24194_304_57228, &P24194_302};	/*Property = FileReader*/
const Value V24194_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24194_305 = {S25, 0, &V24194_305_57231, &P24194_304};	/*Property = FileSaver*/
const Value V24194_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24194_306 = {S26, 0, &V24194_306_57237, &P24194_305};	/*Property = FileWriter*/
const Value V24194_307_57240 = {S0, NULL};	/*Value = False*/
const Property P24194_307 = {S27, 0, &V24194_307_57240, &P24194_306};	/*Property = FormData*/
const Value V24194_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24194_309 = {S28, 0, &V24194_309_57246, &P24194_307};	/*Property = Fullscreen*/
const Value V24194_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24194_310 = {S29, 0, &V24194_310_57248, &P24194_309};	/*Property = GeoLocation*/
const Value V24194_312_57255 = {S0, NULL};	/*Value = False*/
const Property P24194_312 = {S30, 0, &V24194_312_57255, &P24194_310};	/*Property = History*/
const Value V24194_314_57261 = {S0, NULL};	/*Value = False*/
const Property P24194_314 = {S31, 0, &V24194_314_57261, &P24194_312};	/*Property = Html5*/
const Value V24194_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24194_313 = {S32, 0, &V24194_313_57258, &P24194_314};	/*Property = Html-Media-Capture*/
const Value V24194_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24194_315 = {S33, 0, &V24194_315_57264, &P24194_313};	/*Property = Iframe*/
const Value V24194_316_57267 = {S0, NULL};	/*Value = False*/
const Property P24194_316 = {S34, 0, &V24194_316_57267, &P24194_315};	/*Property = IndexedDB*/
const Value V24194_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24194_317 = {S35, 0, &V24194_317_57269, &P24194_316};	/*Property = Json*/
const Value V24194_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24194_15 = {S37, 0, &V24194_15_54167, &P24194_317};	/*Property = LayoutEngine*/
const Value V24194_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24194_344 = {S38, 0, &V24194_344_57272, &P24194_15};	/*Property = Masking*/
const Value V24194_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24194_320 = {S39, 0, &V24194_320_57290, &P24194_344};	/*Property = PostMessage*/
const Value V24194_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24194_321 = {S40, 0, &V24194_321_57293, &P24194_320};	/*Property = Progress*/
const Value V24194_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24194_322 = {S41, 0, &V24194_322_57296, &P24194_321};	/*Property = Prompts*/
const Value V24194_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24194_323 = {S42, 0, &V24194_323_57299, &P24194_322};	/*Property = Selector*/
const Value V24194_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24194_324 = {S43, 0, &V24194_324_57306, &P24194_323};	/*Property = Svg*/
const Value V24194_327_57315 = {S0, NULL};	/*Value = False*/
const Property P24194_327 = {S44, 0, &V24194_327_57315, &P24194_324};	/*Property = TouchEvents*/
const Value V24194_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24194_328 = {S45, 0, &V24194_328_57318, &P24194_327};	/*Property = Track*/
const Value V24194_329_57324 = {S0, NULL};	/*Value = False*/
const Property P24194_329 = {S46, 0, &V24194_329_57324, &P24194_328};	/*Property = Video*/
const Value V24194_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24194_330 = {S47, 0, &V24194_330_57326, &P24194_329};	/*Property = Viewport*/
const Value V24194_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24194_332 = {S48, 0, &V24194_332_57339, &P24194_330};	/*Property = WebWorkers*/
const Value V24194_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24194_333 = {S49, 0, &V24194_333_57342, &P24194_332};	/*Property = Xhr2*/
const Profile P24194 = {"24194", &P24194_333};	/*Profile ID - 24194*/
const Value V18508_1_1 = {S2, NULL};	/*Value = True*/
const Property P18508_1 = {S50, 0, &V18508_1_1, NULL};	/*Property = IsMobile*/
const Value V18508_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18508_5 = {S52, 0, &V18508_5_1277, &P18508_1};	/*Property = ScreenPixelsHeight*/
const Value V18508_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18508_4 = {S54, 0, &V18508_4_1279, &P18508_5};	/*Property = ScreenPixelsWidth*/
const Profile P18508 = {"18508", &P18508_4};	/*Profile ID - 18508*/
const Value V22458_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22458_279 = {S1, 0, &V22458_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22458_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22458_282 = {S3, 0, &V22458_282_57153, &P22458_279};	/*Property = BlobBuilder*/
const Value V22458_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22458_350 = {S4, 0, &V22458_350_57444, &P22458_282};	/*Property = Canvas*/
const Value V22458_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22458_283 = {S5, 0, &V22458_283_57159, &P22458_350};	/*Property = CssBackground*/
const Value V22458_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22458_284 = {S6, 0, &V22458_284_57163, &P22458_283};	/*Property = CssBorderImage*/
const Value V22458_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22458_285 = {S7, 0, &V22458_285_57166, &P22458_284};	/*Property = CssCanvas*/
const Value V22458_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22458_286 = {S8, 0, &V22458_286_57168, &P22458_285};	/*Property = CssColor*/
const Value V22458_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22458_237 = {S9, 0, &V22458_237_55784, &P22458_286};	/*Property = CssColumn*/
const Value V22458_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22458_287 = {S10, 0, &V22458_287_57172, &P22458_237};	/*Property = CssFlexbox*/
const Value V22458_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22458_288 = {S11, 0, &V22458_288_57175, &P22458_287};	/*Property = CssFont*/
const Value V22458_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22458_289 = {S12, 0, &V22458_289_57759, &P22458_288};	/*Property = CssImages*/
const Value V22458_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22458_291 = {S13, 0, &V22458_291_57180, &P22458_289};	/*Property = CssMediaQueries*/
const Value V22458_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22458_292 = {S14, 0, &V22458_292_57183, &P22458_291};	/*Property = CssMinMax*/
const Value V22458_293_57186 = {S2, NULL};	/*Value = True*/
const Property P22458_293 = {S15, 0, &V22458_293_57186, &P22458_292};	/*Property = CssOverflow*/
const Value V22458_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22458_294 = {S16, 0, &V22458_294_57189, &P22458_293};	/*Property = CssPosition*/
const Value V22458_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22458_295 = {S17, 0, &V22458_295_57192, &P22458_294};	/*Property = CssText*/
const Value V22458_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22458_297 = {S18, 0, &V22458_297_57195, &P22458_295};	/*Property = CssTransforms*/
const Value V22458_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22458_241 = {S19, 0, &V22458_241_55799, &P22458_297};	/*Property = CssTransitions*/
const Value V22458_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22458_298 = {S20, 0, &V22458_298_57206, &P22458_241};	/*Property = CssUI*/
const Value V22458_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22458_300 = {S21, 0, &V22458_300_57216, &P22458_298};	/*Property = DataSet*/
const Value V22458_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22458_301 = {S22, 0, &V22458_301_57218, &P22458_300};	/*Property = DataUrl*/
const Value V22458_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22458_302 = {S23, 0, &V22458_302_57222, &P22458_301};	/*Property = DeviceOrientation*/
const Value V22458_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22458_304 = {S24, 0, &V22458_304_57227, &P22458_302};	/*Property = FileReader*/
const Value V22458_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22458_305 = {S25, 0, &V22458_305_57231, &P22458_304};	/*Property = FileSaver*/
const Value V22458_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22458_306 = {S26, 0, &V22458_306_57237, &P22458_305};	/*Property = FileWriter*/
const Value V22458_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22458_307 = {S27, 0, &V22458_307_57239, &P22458_306};	/*Property = FormData*/
const Value V22458_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22458_309 = {S28, 0, &V22458_309_57246, &P22458_307};	/*Property = Fullscreen*/
const Value V22458_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22458_310 = {S29, 0, &V22458_310_57248, &P22458_309};	/*Property = GeoLocation*/
const Value V22458_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22458_312 = {S30, 0, &V22458_312_57254, &P22458_310};	/*Property = History*/
const Value V22458_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22458_314 = {S31, 0, &V22458_314_57261, &P22458_312};	/*Property = Html5*/
const Value V22458_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22458_313 = {S32, 0, &V22458_313_57258, &P22458_314};	/*Property = Html-Media-Capture*/
const Value V22458_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22458_315 = {S33, 0, &V22458_315_57264, &P22458_313};	/*Property = Iframe*/
const Value V22458_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22458_316 = {S34, 0, &V22458_316_57266, &P22458_315};	/*Property = IndexedDB*/
const Value V22458_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22458_317 = {S35, 0, &V22458_317_57269, &P22458_316};	/*Property = Json*/
const Value V22458_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22458_15 = {S37, 0, &V22458_15_28, &P22458_317};	/*Property = LayoutEngine*/
const Value V22458_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22458_344 = {S38, 0, &V22458_344_57272, &P22458_15};	/*Property = Masking*/
const Value V22458_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22458_320 = {S39, 0, &V22458_320_57290, &P22458_344};	/*Property = PostMessage*/
const Value V22458_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22458_321 = {S40, 0, &V22458_321_57293, &P22458_320};	/*Property = Progress*/
const Value V22458_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22458_322 = {S41, 0, &V22458_322_57296, &P22458_321};	/*Property = Prompts*/
const Value V22458_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22458_323 = {S42, 0, &V22458_323_57299, &P22458_322};	/*Property = Selector*/
const Value V22458_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22458_324 = {S43, 0, &V22458_324_57306, &P22458_323};	/*Property = Svg*/
const Value V22458_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22458_327 = {S44, 0, &V22458_327_57315, &P22458_324};	/*Property = TouchEvents*/
const Value V22458_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22458_328 = {S45, 0, &V22458_328_57318, &P22458_327};	/*Property = Track*/
const Value V22458_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22458_329 = {S46, 0, &V22458_329_57323, &P22458_328};	/*Property = Video*/
const Value V22458_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22458_330 = {S47, 0, &V22458_330_57326, &P22458_329};	/*Property = Viewport*/
const Value V22458_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22458_332 = {S48, 0, &V22458_332_57338, &P22458_330};	/*Property = WebWorkers*/
const Value V22458_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22458_333 = {S49, 0, &V22458_333_57342, &P22458_332};	/*Property = Xhr2*/
const Profile P22458 = {"22458", &P22458_333};	/*Profile ID - 22458*/
const Value V18456_1_1 = {S2, NULL};	/*Value = True*/
const Property P18456_1 = {S50, 0, &V18456_1_1, NULL};	/*Property = IsMobile*/
const Value V18456_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18456_5 = {S52, 0, &V18456_5_1227, &P18456_1};	/*Property = ScreenPixelsHeight*/
const Value V18456_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18456_4 = {S54, 0, &V18456_4_1229, &P18456_5};	/*Property = ScreenPixelsWidth*/
const Profile P18456 = {"18456", &P18456_4};	/*Profile ID - 18456*/
const Value V21027_1_1 = {S2, NULL};	/*Value = True*/
const Property P21027_1 = {S50, 0, &V21027_1_1, NULL};	/*Property = IsMobile*/
const Value V21027_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21027_5 = {S52, 0, &V21027_5_1277, &P21027_1};	/*Property = ScreenPixelsHeight*/
const Value V21027_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21027_4 = {S54, 0, &V21027_4_1279, &P21027_5};	/*Property = ScreenPixelsWidth*/
const Profile P21027 = {"21027", &P21027_4};	/*Profile ID - 21027*/
const Value V21753_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_279 = {S1, 0, &V21753_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21753_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_282 = {S3, 0, &V21753_282_57155, &P21753_279};	/*Property = BlobBuilder*/
const Value V21753_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_350 = {S4, 0, &V21753_350_57446, &P21753_282};	/*Property = Canvas*/
const Value V21753_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_283 = {S5, 0, &V21753_283_57161, &P21753_350};	/*Property = CssBackground*/
const Value V21753_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_284 = {S6, 0, &V21753_284_57164, &P21753_283};	/*Property = CssBorderImage*/
const Value V21753_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_285 = {S7, 0, &V21753_285_57167, &P21753_284};	/*Property = CssCanvas*/
const Value V21753_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_286 = {S8, 0, &V21753_286_57170, &P21753_285};	/*Property = CssColor*/
const Value V21753_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_237 = {S9, 0, &V21753_237_55786, &P21753_286};	/*Property = CssColumn*/
const Value V21753_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_287 = {S10, 0, &V21753_287_57173, &P21753_237};	/*Property = CssFlexbox*/
const Value V21753_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_288 = {S11, 0, &V21753_288_57176, &P21753_287};	/*Property = CssFont*/
const Value V21753_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_289 = {S12, 0, &V21753_289_86899, &P21753_288};	/*Property = CssImages*/
const Value V21753_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_291 = {S13, 0, &V21753_291_57182, &P21753_289};	/*Property = CssMediaQueries*/
const Value V21753_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_292 = {S14, 0, &V21753_292_57185, &P21753_291};	/*Property = CssMinMax*/
const Value V21753_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_293 = {S15, 0, &V21753_293_57188, &P21753_292};	/*Property = CssOverflow*/
const Value V21753_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_294 = {S16, 0, &V21753_294_57191, &P21753_293};	/*Property = CssPosition*/
const Value V21753_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_295 = {S17, 0, &V21753_295_57194, &P21753_294};	/*Property = CssText*/
const Value V21753_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_297 = {S18, 0, &V21753_297_57197, &P21753_295};	/*Property = CssTransforms*/
const Value V21753_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_241 = {S19, 0, &V21753_241_55801, &P21753_297};	/*Property = CssTransitions*/
const Value V21753_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_298 = {S20, 0, &V21753_298_57208, &P21753_241};	/*Property = CssUI*/
const Value V21753_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_300 = {S21, 0, &V21753_300_57217, &P21753_298};	/*Property = DataSet*/
const Value V21753_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_301 = {S22, 0, &V21753_301_57220, &P21753_300};	/*Property = DataUrl*/
const Value V21753_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_302 = {S23, 0, &V21753_302_57223, &P21753_301};	/*Property = DeviceOrientation*/
const Value V21753_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_304 = {S24, 0, &V21753_304_57229, &P21753_302};	/*Property = FileReader*/
const Value V21753_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_305 = {S25, 0, &V21753_305_57232, &P21753_304};	/*Property = FileSaver*/
const Value V21753_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_306 = {S26, 0, &V21753_306_57238, &P21753_305};	/*Property = FileWriter*/
const Value V21753_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_307 = {S27, 0, &V21753_307_57241, &P21753_306};	/*Property = FormData*/
const Value V21753_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_309 = {S28, 0, &V21753_309_57247, &P21753_307};	/*Property = Fullscreen*/
const Value V21753_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_310 = {S29, 0, &V21753_310_57250, &P21753_309};	/*Property = GeoLocation*/
const Value V21753_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_312 = {S30, 0, &V21753_312_57256, &P21753_310};	/*Property = History*/
const Value V21753_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_314 = {S31, 0, &V21753_314_57262, &P21753_312};	/*Property = Html5*/
const Value V21753_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_313 = {S32, 0, &V21753_313_57259, &P21753_314};	/*Property = Html-Media-Capture*/
const Value V21753_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_315 = {S33, 0, &V21753_315_57265, &P21753_313};	/*Property = Iframe*/
const Value V21753_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_316 = {S34, 0, &V21753_316_57268, &P21753_315};	/*Property = IndexedDB*/
const Value V21753_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_317 = {S35, 0, &V21753_317_57271, &P21753_316};	/*Property = Json*/
const Value V21753_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21753_15 = {S37, 0, &V21753_15_54167, &P21753_317};	/*Property = LayoutEngine*/
const Value V21753_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_344 = {S38, 0, &V21753_344_57274, &P21753_15};	/*Property = Masking*/
const Value V21753_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_320 = {S39, 0, &V21753_320_57292, &P21753_344};	/*Property = PostMessage*/
const Value V21753_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_321 = {S40, 0, &V21753_321_57295, &P21753_320};	/*Property = Progress*/
const Value V21753_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_322 = {S41, 0, &V21753_322_57298, &P21753_321};	/*Property = Prompts*/
const Value V21753_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_323 = {S42, 0, &V21753_323_57301, &P21753_322};	/*Property = Selector*/
const Value V21753_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_324 = {S43, 0, &V21753_324_57307, &P21753_323};	/*Property = Svg*/
const Value V21753_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_327 = {S44, 0, &V21753_327_57316, &P21753_324};	/*Property = TouchEvents*/
const Value V21753_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_328 = {S45, 0, &V21753_328_57319, &P21753_327};	/*Property = Track*/
const Value V21753_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_329 = {S46, 0, &V21753_329_57325, &P21753_328};	/*Property = Video*/
const Value V21753_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_330 = {S47, 0, &V21753_330_57328, &P21753_329};	/*Property = Viewport*/
const Value V21753_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_332 = {S48, 0, &V21753_332_57340, &P21753_330};	/*Property = WebWorkers*/
const Value V21753_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21753_333 = {S49, 0, &V21753_333_57343, &P21753_332};	/*Property = Xhr2*/
const Profile P21753 = {"21753", &P21753_333};	/*Profile ID - 21753*/
const Value V21050_1_1 = {S2, NULL};	/*Value = True*/
const Property P21050_1 = {S50, 0, &V21050_1_1, NULL};	/*Property = IsMobile*/
const Value V21050_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21050_5 = {S52, 0, &V21050_5_1458, &P21050_1};	/*Property = ScreenPixelsHeight*/
const Value V21050_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21050_4 = {S54, 0, &V21050_4_37617, &P21050_5};	/*Property = ScreenPixelsWidth*/
const Profile P21050 = {"21050", &P21050_4};	/*Profile ID - 21050*/
const Value V21329_1_1 = {S2, NULL};	/*Value = True*/
const Property P21329_1 = {S50, 0, &V21329_1_1, NULL};	/*Property = IsMobile*/
const Value V21329_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21329_5 = {S52, 0, &V21329_5_1227, &P21329_1};	/*Property = ScreenPixelsHeight*/
const Value V21329_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21329_4 = {S54, 0, &V21329_4_1229, &P21329_5};	/*Property = ScreenPixelsWidth*/
const Profile P21329 = {"21329", &P21329_4};	/*Profile ID - 21329*/
const Value V21427_1_1 = {S2, NULL};	/*Value = True*/
const Property P21427_1 = {S50, 0, &V21427_1_1, NULL};	/*Property = IsMobile*/
const Value V21427_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21427_5 = {S52, 0, &V21427_5_1227, &P21427_1};	/*Property = ScreenPixelsHeight*/
const Value V21427_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21427_4 = {S54, 0, &V21427_4_1229, &P21427_5};	/*Property = ScreenPixelsWidth*/
const Profile P21427 = {"21427", &P21427_4};	/*Profile ID - 21427*/
const Value V23078_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_279 = {S1, 0, &V23078_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23078_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_282 = {S3, 0, &V23078_282_57155, &P23078_279};	/*Property = BlobBuilder*/
const Value V23078_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_350 = {S4, 0, &V23078_350_57446, &P23078_282};	/*Property = Canvas*/
const Value V23078_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_283 = {S5, 0, &V23078_283_57161, &P23078_350};	/*Property = CssBackground*/
const Value V23078_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_284 = {S6, 0, &V23078_284_57164, &P23078_283};	/*Property = CssBorderImage*/
const Value V23078_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_285 = {S7, 0, &V23078_285_57167, &P23078_284};	/*Property = CssCanvas*/
const Value V23078_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_286 = {S8, 0, &V23078_286_57170, &P23078_285};	/*Property = CssColor*/
const Value V23078_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_237 = {S9, 0, &V23078_237_55786, &P23078_286};	/*Property = CssColumn*/
const Value V23078_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_287 = {S10, 0, &V23078_287_57173, &P23078_237};	/*Property = CssFlexbox*/
const Value V23078_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_288 = {S11, 0, &V23078_288_57176, &P23078_287};	/*Property = CssFont*/
const Value V23078_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_289 = {S12, 0, &V23078_289_86899, &P23078_288};	/*Property = CssImages*/
const Value V23078_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_291 = {S13, 0, &V23078_291_57182, &P23078_289};	/*Property = CssMediaQueries*/
const Value V23078_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_292 = {S14, 0, &V23078_292_57185, &P23078_291};	/*Property = CssMinMax*/
const Value V23078_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_293 = {S15, 0, &V23078_293_57188, &P23078_292};	/*Property = CssOverflow*/
const Value V23078_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_294 = {S16, 0, &V23078_294_57191, &P23078_293};	/*Property = CssPosition*/
const Value V23078_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_295 = {S17, 0, &V23078_295_57194, &P23078_294};	/*Property = CssText*/
const Value V23078_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_297 = {S18, 0, &V23078_297_57197, &P23078_295};	/*Property = CssTransforms*/
const Value V23078_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_241 = {S19, 0, &V23078_241_55801, &P23078_297};	/*Property = CssTransitions*/
const Value V23078_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_298 = {S20, 0, &V23078_298_57208, &P23078_241};	/*Property = CssUI*/
const Value V23078_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_300 = {S21, 0, &V23078_300_57217, &P23078_298};	/*Property = DataSet*/
const Value V23078_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_301 = {S22, 0, &V23078_301_57220, &P23078_300};	/*Property = DataUrl*/
const Value V23078_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_302 = {S23, 0, &V23078_302_57223, &P23078_301};	/*Property = DeviceOrientation*/
const Value V23078_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_304 = {S24, 0, &V23078_304_57229, &P23078_302};	/*Property = FileReader*/
const Value V23078_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_305 = {S25, 0, &V23078_305_57232, &P23078_304};	/*Property = FileSaver*/
const Value V23078_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_306 = {S26, 0, &V23078_306_57238, &P23078_305};	/*Property = FileWriter*/
const Value V23078_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_307 = {S27, 0, &V23078_307_57241, &P23078_306};	/*Property = FormData*/
const Value V23078_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_309 = {S28, 0, &V23078_309_57247, &P23078_307};	/*Property = Fullscreen*/
const Value V23078_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_310 = {S29, 0, &V23078_310_57250, &P23078_309};	/*Property = GeoLocation*/
const Value V23078_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_312 = {S30, 0, &V23078_312_57256, &P23078_310};	/*Property = History*/
const Value V23078_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_314 = {S31, 0, &V23078_314_57262, &P23078_312};	/*Property = Html5*/
const Value V23078_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_313 = {S32, 0, &V23078_313_57259, &P23078_314};	/*Property = Html-Media-Capture*/
const Value V23078_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_315 = {S33, 0, &V23078_315_57265, &P23078_313};	/*Property = Iframe*/
const Value V23078_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_316 = {S34, 0, &V23078_316_57268, &P23078_315};	/*Property = IndexedDB*/
const Value V23078_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_317 = {S35, 0, &V23078_317_57271, &P23078_316};	/*Property = Json*/
const Value V23078_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_15 = {S37, 0, &V23078_15_72, &P23078_317};	/*Property = LayoutEngine*/
const Value V23078_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_344 = {S38, 0, &V23078_344_57274, &P23078_15};	/*Property = Masking*/
const Value V23078_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_320 = {S39, 0, &V23078_320_57292, &P23078_344};	/*Property = PostMessage*/
const Value V23078_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_321 = {S40, 0, &V23078_321_57295, &P23078_320};	/*Property = Progress*/
const Value V23078_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_322 = {S41, 0, &V23078_322_57298, &P23078_321};	/*Property = Prompts*/
const Value V23078_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_323 = {S42, 0, &V23078_323_57301, &P23078_322};	/*Property = Selector*/
const Value V23078_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_324 = {S43, 0, &V23078_324_57307, &P23078_323};	/*Property = Svg*/
const Value V23078_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_327 = {S44, 0, &V23078_327_57316, &P23078_324};	/*Property = TouchEvents*/
const Value V23078_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_328 = {S45, 0, &V23078_328_57319, &P23078_327};	/*Property = Track*/
const Value V23078_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_329 = {S46, 0, &V23078_329_57325, &P23078_328};	/*Property = Video*/
const Value V23078_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_330 = {S47, 0, &V23078_330_57328, &P23078_329};	/*Property = Viewport*/
const Value V23078_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_332 = {S48, 0, &V23078_332_57340, &P23078_330};	/*Property = WebWorkers*/
const Value V23078_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23078_333 = {S49, 0, &V23078_333_57343, &P23078_332};	/*Property = Xhr2*/
const Profile P23078 = {"23078", &P23078_333};	/*Profile ID - 23078*/
const Value V18472_1_1 = {S2, NULL};	/*Value = True*/
const Property P18472_1 = {S50, 0, &V18472_1_1, NULL};	/*Property = IsMobile*/
const Value V18472_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18472_5 = {S52, 0, &V18472_5_9981, &P18472_1};	/*Property = ScreenPixelsHeight*/
const Value V18472_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18472_4 = {S54, 0, &V18472_4_39098, &P18472_5};	/*Property = ScreenPixelsWidth*/
const Profile P18472 = {"18472", &P18472_4};	/*Profile ID - 18472*/
const Value V20283_1_1 = {S2, NULL};	/*Value = True*/
const Property P20283_1 = {S50, 0, &V20283_1_1, NULL};	/*Property = IsMobile*/
const Value V20283_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20283_5 = {S52, 0, &V20283_5_1458, &P20283_1};	/*Property = ScreenPixelsHeight*/
const Value V20283_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20283_4 = {S54, 0, &V20283_4_1460, &P20283_5};	/*Property = ScreenPixelsWidth*/
const Profile P20283 = {"20283", &P20283_4};	/*Profile ID - 20283*/
const Value V20947_1_1 = {S2, NULL};	/*Value = True*/
const Property P20947_1 = {S50, 0, &V20947_1_1, NULL};	/*Property = IsMobile*/
const Value V20947_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20947_5 = {S52, 0, &V20947_5_1403, &P20947_1};	/*Property = ScreenPixelsHeight*/
const Value V20947_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20947_4 = {S54, 0, &V20947_4_1404, &P20947_5};	/*Property = ScreenPixelsWidth*/
const Profile P20947 = {"20947", &P20947_4};	/*Profile ID - 20947*/
const Value V18583_1_1 = {S2, NULL};	/*Value = True*/
const Property P18583_1 = {S50, 0, &V18583_1_1, NULL};	/*Property = IsMobile*/
const Value V18583_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P18583_5 = {S52, 0, &V18583_5_1293, &P18583_1};	/*Property = ScreenPixelsHeight*/
const Value V18583_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P18583_4 = {S54, 0, &V18583_4_1296, &P18583_5};	/*Property = ScreenPixelsWidth*/
const Profile P18583 = {"18583", &P18583_4};	/*Profile ID - 18583*/
const Value V21179_1_1 = {S2, NULL};	/*Value = True*/
const Property P21179_1 = {S50, 0, &V21179_1_1, NULL};	/*Property = IsMobile*/
const Value V21179_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21179_5 = {S52, 0, &V21179_5_1458, &P21179_1};	/*Property = ScreenPixelsHeight*/
const Value V21179_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21179_4 = {S54, 0, &V21179_4_37617, &P21179_5};	/*Property = ScreenPixelsWidth*/
const Profile P21179 = {"21179", &P21179_4};	/*Profile ID - 21179*/
const Value V21118_1_1 = {S2, NULL};	/*Value = True*/
const Property P21118_1 = {S50, 0, &V21118_1_1, NULL};	/*Property = IsMobile*/
const Value V21118_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P21118_5 = {S52, 0, &V21118_5_1832, &P21118_1};	/*Property = ScreenPixelsHeight*/
const Value V21118_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21118_4 = {S54, 0, &V21118_4_5645, &P21118_5};	/*Property = ScreenPixelsWidth*/
const Profile P21118 = {"21118", &P21118_4};	/*Profile ID - 21118*/
const Value V18220_1_1 = {S2, NULL};	/*Value = True*/
const Property P18220_1 = {S50, 0, &V18220_1_1, NULL};	/*Property = IsMobile*/
const Value V18220_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P18220_5 = {S52, 0, &V18220_5_1275, &P18220_1};	/*Property = ScreenPixelsHeight*/
const Value V18220_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P18220_4 = {S54, 0, &V18220_4_1278, &P18220_5};	/*Property = ScreenPixelsWidth*/
const Profile P18220 = {"18220", &P18220_4};	/*Profile ID - 18220*/
const Value V22430_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22430_279 = {S1, 0, &V22430_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22430_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22430_282 = {S3, 0, &V22430_282_57153, &P22430_279};	/*Property = BlobBuilder*/
const Value V22430_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22430_350 = {S4, 0, &V22430_350_57444, &P22430_282};	/*Property = Canvas*/
const Value V22430_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22430_283 = {S5, 0, &V22430_283_57159, &P22430_350};	/*Property = CssBackground*/
const Value V22430_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22430_284 = {S6, 0, &V22430_284_57163, &P22430_283};	/*Property = CssBorderImage*/
const Value V22430_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22430_285 = {S7, 0, &V22430_285_57166, &P22430_284};	/*Property = CssCanvas*/
const Value V22430_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22430_286 = {S8, 0, &V22430_286_57168, &P22430_285};	/*Property = CssColor*/
const Value V22430_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22430_237 = {S9, 0, &V22430_237_55784, &P22430_286};	/*Property = CssColumn*/
const Value V22430_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22430_287 = {S10, 0, &V22430_287_57172, &P22430_237};	/*Property = CssFlexbox*/
const Value V22430_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22430_288 = {S11, 0, &V22430_288_57175, &P22430_287};	/*Property = CssFont*/
const Value V22430_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22430_289 = {S12, 0, &V22430_289_57759, &P22430_288};	/*Property = CssImages*/
const Value V22430_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22430_291 = {S13, 0, &V22430_291_57180, &P22430_289};	/*Property = CssMediaQueries*/
const Value V22430_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22430_292 = {S14, 0, &V22430_292_57183, &P22430_291};	/*Property = CssMinMax*/
const Value V22430_293_57186 = {S2, NULL};	/*Value = True*/
const Property P22430_293 = {S15, 0, &V22430_293_57186, &P22430_292};	/*Property = CssOverflow*/
const Value V22430_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22430_294 = {S16, 0, &V22430_294_57189, &P22430_293};	/*Property = CssPosition*/
const Value V22430_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22430_295 = {S17, 0, &V22430_295_57192, &P22430_294};	/*Property = CssText*/
const Value V22430_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22430_297 = {S18, 0, &V22430_297_57195, &P22430_295};	/*Property = CssTransforms*/
const Value V22430_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22430_241 = {S19, 0, &V22430_241_55799, &P22430_297};	/*Property = CssTransitions*/
const Value V22430_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22430_298 = {S20, 0, &V22430_298_57206, &P22430_241};	/*Property = CssUI*/
const Value V22430_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22430_300 = {S21, 0, &V22430_300_57216, &P22430_298};	/*Property = DataSet*/
const Value V22430_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22430_301 = {S22, 0, &V22430_301_57218, &P22430_300};	/*Property = DataUrl*/
const Value V22430_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22430_302 = {S23, 0, &V22430_302_57222, &P22430_301};	/*Property = DeviceOrientation*/
const Value V22430_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22430_304 = {S24, 0, &V22430_304_57227, &P22430_302};	/*Property = FileReader*/
const Value V22430_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22430_305 = {S25, 0, &V22430_305_57231, &P22430_304};	/*Property = FileSaver*/
const Value V22430_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22430_306 = {S26, 0, &V22430_306_57237, &P22430_305};	/*Property = FileWriter*/
const Value V22430_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22430_307 = {S27, 0, &V22430_307_57239, &P22430_306};	/*Property = FormData*/
const Value V22430_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22430_309 = {S28, 0, &V22430_309_57246, &P22430_307};	/*Property = Fullscreen*/
const Value V22430_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22430_310 = {S29, 0, &V22430_310_57248, &P22430_309};	/*Property = GeoLocation*/
const Value V22430_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22430_312 = {S30, 0, &V22430_312_57254, &P22430_310};	/*Property = History*/
const Value V22430_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22430_314 = {S31, 0, &V22430_314_57261, &P22430_312};	/*Property = Html5*/
const Value V22430_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22430_313 = {S32, 0, &V22430_313_57258, &P22430_314};	/*Property = Html-Media-Capture*/
const Value V22430_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22430_315 = {S33, 0, &V22430_315_57264, &P22430_313};	/*Property = Iframe*/
const Value V22430_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22430_316 = {S34, 0, &V22430_316_57266, &P22430_315};	/*Property = IndexedDB*/
const Value V22430_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22430_317 = {S35, 0, &V22430_317_57269, &P22430_316};	/*Property = Json*/
const Value V22430_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22430_15 = {S37, 0, &V22430_15_28, &P22430_317};	/*Property = LayoutEngine*/
const Value V22430_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22430_344 = {S38, 0, &V22430_344_57272, &P22430_15};	/*Property = Masking*/
const Value V22430_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22430_320 = {S39, 0, &V22430_320_57290, &P22430_344};	/*Property = PostMessage*/
const Value V22430_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22430_321 = {S40, 0, &V22430_321_57293, &P22430_320};	/*Property = Progress*/
const Value V22430_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22430_322 = {S41, 0, &V22430_322_57296, &P22430_321};	/*Property = Prompts*/
const Value V22430_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22430_323 = {S42, 0, &V22430_323_57299, &P22430_322};	/*Property = Selector*/
const Value V22430_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22430_324 = {S43, 0, &V22430_324_57306, &P22430_323};	/*Property = Svg*/
const Value V22430_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22430_327 = {S44, 0, &V22430_327_57315, &P22430_324};	/*Property = TouchEvents*/
const Value V22430_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22430_328 = {S45, 0, &V22430_328_57318, &P22430_327};	/*Property = Track*/
const Value V22430_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22430_329 = {S46, 0, &V22430_329_57323, &P22430_328};	/*Property = Video*/
const Value V22430_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22430_330 = {S47, 0, &V22430_330_57326, &P22430_329};	/*Property = Viewport*/
const Value V22430_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22430_332 = {S48, 0, &V22430_332_57338, &P22430_330};	/*Property = WebWorkers*/
const Value V22430_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22430_333 = {S49, 0, &V22430_333_57342, &P22430_332};	/*Property = Xhr2*/
const Profile P22430 = {"22430", &P22430_333};	/*Profile ID - 22430*/
const Value V21159_1_1 = {S2, NULL};	/*Value = True*/
const Property P21159_1 = {S50, 0, &V21159_1_1, NULL};	/*Property = IsMobile*/
const Value V21159_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21159_5 = {S52, 0, &V21159_5_37, &P21159_1};	/*Property = ScreenPixelsHeight*/
const Value V21159_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21159_4 = {S54, 0, &V21159_4_36, &P21159_5};	/*Property = ScreenPixelsWidth*/
const Profile P21159 = {"21159", &P21159_4};	/*Profile ID - 21159*/
const Value V18703_1_1 = {S2, NULL};	/*Value = True*/
const Property P18703_1 = {S50, 0, &V18703_1_1, NULL};	/*Property = IsMobile*/
const Value V18703_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18703_5 = {S52, 0, &V18703_5_1277, &P18703_1};	/*Property = ScreenPixelsHeight*/
const Value V18703_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18703_4 = {S54, 0, &V18703_4_1279, &P18703_5};	/*Property = ScreenPixelsWidth*/
const Profile P18703 = {"18703", &P18703_4};	/*Profile ID - 18703*/
const Value V22796_1_1 = {S2, NULL};	/*Value = True*/
const Property P22796_1 = {S50, 0, &V22796_1_1, NULL};	/*Property = IsMobile*/
const Value V22796_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P22796_5 = {S52, 0, &V22796_5_1380, &P22796_1};	/*Property = ScreenPixelsHeight*/
const Value V22796_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22796_4 = {S54, 0, &V22796_4_1279, &P22796_5};	/*Property = ScreenPixelsWidth*/
const Profile P22796 = {"22796", &P22796_4};	/*Profile ID - 22796*/
const Value V23864_1_1 = {S2, NULL};	/*Value = True*/
const Property P23864_1 = {S50, 0, &V23864_1_1, NULL};	/*Property = IsMobile*/
const Value V23864_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23864_5 = {S52, 0, &V23864_5_37, &P23864_1};	/*Property = ScreenPixelsHeight*/
const Value V23864_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23864_4 = {S54, 0, &V23864_4_36, &P23864_5};	/*Property = ScreenPixelsWidth*/
const Profile P23864 = {"23864", &P23864_4};	/*Profile ID - 23864*/
const Value V23381_1_1 = {S2, NULL};	/*Value = True*/
const Property P23381_1 = {S50, 0, &V23381_1_1, NULL};	/*Property = IsMobile*/
const Value V23381_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23381_5 = {S52, 0, &V23381_5_1227, &P23381_1};	/*Property = ScreenPixelsHeight*/
const Value V23381_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23381_4 = {S54, 0, &V23381_4_1229, &P23381_5};	/*Property = ScreenPixelsWidth*/
const Profile P23381 = {"23381", &P23381_4};	/*Profile ID - 23381*/
const Value V18509_1_1 = {S2, NULL};	/*Value = True*/
const Property P18509_1 = {S50, 0, &V18509_1_1, NULL};	/*Property = IsMobile*/
const Value V18509_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18509_5 = {S52, 0, &V18509_5_1277, &P18509_1};	/*Property = ScreenPixelsHeight*/
const Value V18509_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18509_4 = {S54, 0, &V18509_4_1279, &P18509_5};	/*Property = ScreenPixelsWidth*/
const Profile P18509 = {"18509", &P18509_4};	/*Profile ID - 18509*/
const Value V18428_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18428_279 = {S1, 0, &V18428_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18428_282_57153 = {S2, NULL};	/*Value = True*/
const Property P18428_282 = {S3, 0, &V18428_282_57153, &P18428_279};	/*Property = BlobBuilder*/
const Value V18428_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18428_350 = {S4, 0, &V18428_350_57444, &P18428_282};	/*Property = Canvas*/
const Value V18428_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18428_283 = {S5, 0, &V18428_283_57159, &P18428_350};	/*Property = CssBackground*/
const Value V18428_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18428_284 = {S6, 0, &V18428_284_57163, &P18428_283};	/*Property = CssBorderImage*/
const Value V18428_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18428_285 = {S7, 0, &V18428_285_57166, &P18428_284};	/*Property = CssCanvas*/
const Value V18428_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18428_286 = {S8, 0, &V18428_286_57168, &P18428_285};	/*Property = CssColor*/
const Value V18428_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18428_237 = {S9, 0, &V18428_237_55786, &P18428_286};	/*Property = CssColumn*/
const Value V18428_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18428_287 = {S10, 0, &V18428_287_57172, &P18428_237};	/*Property = CssFlexbox*/
const Value V18428_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18428_288 = {S11, 0, &V18428_288_57175, &P18428_287};	/*Property = CssFont*/
const Value V18428_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18428_289 = {S12, 0, &V18428_289_57759, &P18428_288};	/*Property = CssImages*/
const Value V18428_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18428_291 = {S13, 0, &V18428_291_57181, &P18428_289};	/*Property = CssMediaQueries*/
const Value V18428_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18428_292 = {S14, 0, &V18428_292_57183, &P18428_291};	/*Property = CssMinMax*/
const Value V18428_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18428_293 = {S15, 0, &V18428_293_57187, &P18428_292};	/*Property = CssOverflow*/
const Value V18428_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18428_294 = {S16, 0, &V18428_294_57189, &P18428_293};	/*Property = CssPosition*/
const Value V18428_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18428_295 = {S17, 0, &V18428_295_57192, &P18428_294};	/*Property = CssText*/
const Value V18428_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18428_297 = {S18, 0, &V18428_297_57195, &P18428_295};	/*Property = CssTransforms*/
const Value V18428_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18428_241 = {S19, 0, &V18428_241_55799, &P18428_297};	/*Property = CssTransitions*/
const Value V18428_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18428_298 = {S20, 0, &V18428_298_57206, &P18428_241};	/*Property = CssUI*/
const Value V18428_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18428_300 = {S21, 0, &V18428_300_57216, &P18428_298};	/*Property = DataSet*/
const Value V18428_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18428_301 = {S22, 0, &V18428_301_57218, &P18428_300};	/*Property = DataUrl*/
const Value V18428_302_57221 = {S2, NULL};	/*Value = True*/
const Property P18428_302 = {S23, 0, &V18428_302_57221, &P18428_301};	/*Property = DeviceOrientation*/
const Value V18428_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18428_304 = {S24, 0, &V18428_304_57227, &P18428_302};	/*Property = FileReader*/
const Value V18428_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18428_305 = {S25, 0, &V18428_305_57231, &P18428_304};	/*Property = FileSaver*/
const Value V18428_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18428_306 = {S26, 0, &V18428_306_57237, &P18428_305};	/*Property = FileWriter*/
const Value V18428_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18428_307 = {S27, 0, &V18428_307_57239, &P18428_306};	/*Property = FormData*/
const Value V18428_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18428_309 = {S28, 0, &V18428_309_57246, &P18428_307};	/*Property = Fullscreen*/
const Value V18428_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18428_310 = {S29, 0, &V18428_310_57248, &P18428_309};	/*Property = GeoLocation*/
const Value V18428_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18428_312 = {S30, 0, &V18428_312_57254, &P18428_310};	/*Property = History*/
const Value V18428_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18428_314 = {S31, 0, &V18428_314_57261, &P18428_312};	/*Property = Html5*/
const Value V18428_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18428_313 = {S32, 0, &V18428_313_57258, &P18428_314};	/*Property = Html-Media-Capture*/
const Value V18428_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18428_315 = {S33, 0, &V18428_315_57264, &P18428_313};	/*Property = Iframe*/
const Value V18428_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18428_316 = {S34, 0, &V18428_316_57266, &P18428_315};	/*Property = IndexedDB*/
const Value V18428_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18428_317 = {S35, 0, &V18428_317_57269, &P18428_316};	/*Property = Json*/
const Value V18428_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18428_15 = {S37, 0, &V18428_15_29, &P18428_317};	/*Property = LayoutEngine*/
const Value V18428_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18428_344 = {S38, 0, &V18428_344_57272, &P18428_15};	/*Property = Masking*/
const Value V18428_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18428_320 = {S39, 0, &V18428_320_57290, &P18428_344};	/*Property = PostMessage*/
const Value V18428_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18428_321 = {S40, 0, &V18428_321_57293, &P18428_320};	/*Property = Progress*/
const Value V18428_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18428_322 = {S41, 0, &V18428_322_57296, &P18428_321};	/*Property = Prompts*/
const Value V18428_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18428_323 = {S42, 0, &V18428_323_57299, &P18428_322};	/*Property = Selector*/
const Value V18428_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18428_324 = {S43, 0, &V18428_324_57306, &P18428_323};	/*Property = Svg*/
const Value V18428_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18428_327 = {S44, 0, &V18428_327_57315, &P18428_324};	/*Property = TouchEvents*/
const Value V18428_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18428_328 = {S45, 0, &V18428_328_57318, &P18428_327};	/*Property = Track*/
const Value V18428_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18428_329 = {S46, 0, &V18428_329_57323, &P18428_328};	/*Property = Video*/
const Value V18428_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18428_330 = {S47, 0, &V18428_330_57326, &P18428_329};	/*Property = Viewport*/
const Value V18428_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18428_332 = {S48, 0, &V18428_332_57339, &P18428_330};	/*Property = WebWorkers*/
const Value V18428_333_57341 = {S2, NULL};	/*Value = True*/
const Property P18428_333 = {S49, 0, &V18428_333_57341, &P18428_332};	/*Property = Xhr2*/
const Profile P18428 = {"18428", &P18428_333};	/*Profile ID - 18428*/
const Value V20004_1_1 = {S2, NULL};	/*Value = True*/
const Property P20004_1 = {S50, 0, &V20004_1_1, NULL};	/*Property = IsMobile*/
const Value V20004_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20004_5 = {S52, 0, &V20004_5_1227, &P20004_1};	/*Property = ScreenPixelsHeight*/
const Value V20004_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20004_4 = {S54, 0, &V20004_4_1229, &P20004_5};	/*Property = ScreenPixelsWidth*/
const Profile P20004 = {"20004", &P20004_4};	/*Profile ID - 20004*/
const Value V22360_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22360_279 = {S1, 0, &V22360_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22360_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22360_282 = {S3, 0, &V22360_282_57154, &P22360_279};	/*Property = BlobBuilder*/
const Value V22360_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22360_350 = {S4, 0, &V22360_350_57444, &P22360_282};	/*Property = Canvas*/
const Value V22360_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22360_283 = {S5, 0, &V22360_283_57159, &P22360_350};	/*Property = CssBackground*/
const Value V22360_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22360_284 = {S6, 0, &V22360_284_57163, &P22360_283};	/*Property = CssBorderImage*/
const Value V22360_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22360_285 = {S7, 0, &V22360_285_57166, &P22360_284};	/*Property = CssCanvas*/
const Value V22360_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22360_286 = {S8, 0, &V22360_286_57168, &P22360_285};	/*Property = CssColor*/
const Value V22360_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22360_237 = {S9, 0, &V22360_237_55784, &P22360_286};	/*Property = CssColumn*/
const Value V22360_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22360_287 = {S10, 0, &V22360_287_57172, &P22360_237};	/*Property = CssFlexbox*/
const Value V22360_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22360_288 = {S11, 0, &V22360_288_57175, &P22360_287};	/*Property = CssFont*/
const Value V22360_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22360_289 = {S12, 0, &V22360_289_57759, &P22360_288};	/*Property = CssImages*/
const Value V22360_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22360_291 = {S13, 0, &V22360_291_57181, &P22360_289};	/*Property = CssMediaQueries*/
const Value V22360_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22360_292 = {S14, 0, &V22360_292_57183, &P22360_291};	/*Property = CssMinMax*/
const Value V22360_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22360_293 = {S15, 0, &V22360_293_57187, &P22360_292};	/*Property = CssOverflow*/
const Value V22360_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22360_294 = {S16, 0, &V22360_294_57190, &P22360_293};	/*Property = CssPosition*/
const Value V22360_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22360_295 = {S17, 0, &V22360_295_57193, &P22360_294};	/*Property = CssText*/
const Value V22360_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22360_297 = {S18, 0, &V22360_297_57195, &P22360_295};	/*Property = CssTransforms*/
const Value V22360_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22360_241 = {S19, 0, &V22360_241_55799, &P22360_297};	/*Property = CssTransitions*/
const Value V22360_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22360_298 = {S20, 0, &V22360_298_57206, &P22360_241};	/*Property = CssUI*/
const Value V22360_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22360_300 = {S21, 0, &V22360_300_57216, &P22360_298};	/*Property = DataSet*/
const Value V22360_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22360_301 = {S22, 0, &V22360_301_57218, &P22360_300};	/*Property = DataUrl*/
const Value V22360_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22360_302 = {S23, 0, &V22360_302_57222, &P22360_301};	/*Property = DeviceOrientation*/
const Value V22360_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22360_304 = {S24, 0, &V22360_304_57228, &P22360_302};	/*Property = FileReader*/
const Value V22360_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22360_305 = {S25, 0, &V22360_305_57231, &P22360_304};	/*Property = FileSaver*/
const Value V22360_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22360_306 = {S26, 0, &V22360_306_57237, &P22360_305};	/*Property = FileWriter*/
const Value V22360_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22360_307 = {S27, 0, &V22360_307_57240, &P22360_306};	/*Property = FormData*/
const Value V22360_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22360_309 = {S28, 0, &V22360_309_57246, &P22360_307};	/*Property = Fullscreen*/
const Value V22360_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22360_310 = {S29, 0, &V22360_310_57248, &P22360_309};	/*Property = GeoLocation*/
const Value V22360_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22360_312 = {S30, 0, &V22360_312_57255, &P22360_310};	/*Property = History*/
const Value V22360_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22360_314 = {S31, 0, &V22360_314_57261, &P22360_312};	/*Property = Html5*/
const Value V22360_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22360_313 = {S32, 0, &V22360_313_57258, &P22360_314};	/*Property = Html-Media-Capture*/
const Value V22360_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22360_315 = {S33, 0, &V22360_315_57264, &P22360_313};	/*Property = Iframe*/
const Value V22360_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22360_316 = {S34, 0, &V22360_316_57267, &P22360_315};	/*Property = IndexedDB*/
const Value V22360_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22360_317 = {S35, 0, &V22360_317_57269, &P22360_316};	/*Property = Json*/
const Value V22360_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22360_15 = {S37, 0, &V22360_15_29, &P22360_317};	/*Property = LayoutEngine*/
const Value V22360_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22360_344 = {S38, 0, &V22360_344_57272, &P22360_15};	/*Property = Masking*/
const Value V22360_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22360_320 = {S39, 0, &V22360_320_57290, &P22360_344};	/*Property = PostMessage*/
const Value V22360_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22360_321 = {S40, 0, &V22360_321_57293, &P22360_320};	/*Property = Progress*/
const Value V22360_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22360_322 = {S41, 0, &V22360_322_57296, &P22360_321};	/*Property = Prompts*/
const Value V22360_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22360_323 = {S42, 0, &V22360_323_57299, &P22360_322};	/*Property = Selector*/
const Value V22360_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22360_324 = {S43, 0, &V22360_324_57306, &P22360_323};	/*Property = Svg*/
const Value V22360_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22360_327 = {S44, 0, &V22360_327_57315, &P22360_324};	/*Property = TouchEvents*/
const Value V22360_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22360_328 = {S45, 0, &V22360_328_57318, &P22360_327};	/*Property = Track*/
const Value V22360_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22360_329 = {S46, 0, &V22360_329_57323, &P22360_328};	/*Property = Video*/
const Value V22360_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22360_330 = {S47, 0, &V22360_330_57326, &P22360_329};	/*Property = Viewport*/
const Value V22360_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22360_332 = {S48, 0, &V22360_332_57339, &P22360_330};	/*Property = WebWorkers*/
const Value V22360_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22360_333 = {S49, 0, &V22360_333_57342, &P22360_332};	/*Property = Xhr2*/
const Profile P22360 = {"22360", &P22360_333};	/*Profile ID - 22360*/
const Value V20120_1_1 = {S2, NULL};	/*Value = True*/
const Property P20120_1 = {S50, 0, &V20120_1_1, NULL};	/*Property = IsMobile*/
const Value V20120_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20120_5 = {S52, 0, &V20120_5_1277, &P20120_1};	/*Property = ScreenPixelsHeight*/
const Value V20120_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20120_4 = {S54, 0, &V20120_4_1279, &P20120_5};	/*Property = ScreenPixelsWidth*/
const Profile P20120 = {"20120", &P20120_4};	/*Profile ID - 20120*/
const Value V21037_1_1 = {S2, NULL};	/*Value = True*/
const Property P21037_1 = {S50, 0, &V21037_1_1, NULL};	/*Property = IsMobile*/
const Value V21037_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21037_5 = {S52, 0, &V21037_5_1458, &P21037_1};	/*Property = ScreenPixelsHeight*/
const Value V21037_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21037_4 = {S54, 0, &V21037_4_37617, &P21037_5};	/*Property = ScreenPixelsWidth*/
const Profile P21037 = {"21037", &P21037_4};	/*Profile ID - 21037*/
const Value V22316_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_279 = {S1, 0, &V22316_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22316_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_282 = {S3, 0, &V22316_282_57155, &P22316_279};	/*Property = BlobBuilder*/
const Value V22316_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_350 = {S4, 0, &V22316_350_57446, &P22316_282};	/*Property = Canvas*/
const Value V22316_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_283 = {S5, 0, &V22316_283_57161, &P22316_350};	/*Property = CssBackground*/
const Value V22316_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_284 = {S6, 0, &V22316_284_57164, &P22316_283};	/*Property = CssBorderImage*/
const Value V22316_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_285 = {S7, 0, &V22316_285_57167, &P22316_284};	/*Property = CssCanvas*/
const Value V22316_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_286 = {S8, 0, &V22316_286_57170, &P22316_285};	/*Property = CssColor*/
const Value V22316_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_237 = {S9, 0, &V22316_237_55786, &P22316_286};	/*Property = CssColumn*/
const Value V22316_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_287 = {S10, 0, &V22316_287_57173, &P22316_237};	/*Property = CssFlexbox*/
const Value V22316_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_288 = {S11, 0, &V22316_288_57176, &P22316_287};	/*Property = CssFont*/
const Value V22316_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_289 = {S12, 0, &V22316_289_86899, &P22316_288};	/*Property = CssImages*/
const Value V22316_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_291 = {S13, 0, &V22316_291_57182, &P22316_289};	/*Property = CssMediaQueries*/
const Value V22316_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_292 = {S14, 0, &V22316_292_57185, &P22316_291};	/*Property = CssMinMax*/
const Value V22316_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_293 = {S15, 0, &V22316_293_57188, &P22316_292};	/*Property = CssOverflow*/
const Value V22316_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_294 = {S16, 0, &V22316_294_57191, &P22316_293};	/*Property = CssPosition*/
const Value V22316_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_295 = {S17, 0, &V22316_295_57194, &P22316_294};	/*Property = CssText*/
const Value V22316_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_297 = {S18, 0, &V22316_297_57197, &P22316_295};	/*Property = CssTransforms*/
const Value V22316_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_241 = {S19, 0, &V22316_241_55801, &P22316_297};	/*Property = CssTransitions*/
const Value V22316_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_298 = {S20, 0, &V22316_298_57208, &P22316_241};	/*Property = CssUI*/
const Value V22316_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_300 = {S21, 0, &V22316_300_57217, &P22316_298};	/*Property = DataSet*/
const Value V22316_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_301 = {S22, 0, &V22316_301_57220, &P22316_300};	/*Property = DataUrl*/
const Value V22316_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_302 = {S23, 0, &V22316_302_57223, &P22316_301};	/*Property = DeviceOrientation*/
const Value V22316_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_304 = {S24, 0, &V22316_304_57229, &P22316_302};	/*Property = FileReader*/
const Value V22316_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_305 = {S25, 0, &V22316_305_57232, &P22316_304};	/*Property = FileSaver*/
const Value V22316_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_306 = {S26, 0, &V22316_306_57238, &P22316_305};	/*Property = FileWriter*/
const Value V22316_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_307 = {S27, 0, &V22316_307_57241, &P22316_306};	/*Property = FormData*/
const Value V22316_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_309 = {S28, 0, &V22316_309_57247, &P22316_307};	/*Property = Fullscreen*/
const Value V22316_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_310 = {S29, 0, &V22316_310_57250, &P22316_309};	/*Property = GeoLocation*/
const Value V22316_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_312 = {S30, 0, &V22316_312_57256, &P22316_310};	/*Property = History*/
const Value V22316_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_314 = {S31, 0, &V22316_314_57262, &P22316_312};	/*Property = Html5*/
const Value V22316_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_313 = {S32, 0, &V22316_313_57259, &P22316_314};	/*Property = Html-Media-Capture*/
const Value V22316_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_315 = {S33, 0, &V22316_315_57265, &P22316_313};	/*Property = Iframe*/
const Value V22316_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_316 = {S34, 0, &V22316_316_57268, &P22316_315};	/*Property = IndexedDB*/
const Value V22316_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_317 = {S35, 0, &V22316_317_57271, &P22316_316};	/*Property = Json*/
const Value V22316_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22316_15 = {S37, 0, &V22316_15_29, &P22316_317};	/*Property = LayoutEngine*/
const Value V22316_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_344 = {S38, 0, &V22316_344_57274, &P22316_15};	/*Property = Masking*/
const Value V22316_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_320 = {S39, 0, &V22316_320_57292, &P22316_344};	/*Property = PostMessage*/
const Value V22316_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_321 = {S40, 0, &V22316_321_57295, &P22316_320};	/*Property = Progress*/
const Value V22316_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_322 = {S41, 0, &V22316_322_57298, &P22316_321};	/*Property = Prompts*/
const Value V22316_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_323 = {S42, 0, &V22316_323_57301, &P22316_322};	/*Property = Selector*/
const Value V22316_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_324 = {S43, 0, &V22316_324_57307, &P22316_323};	/*Property = Svg*/
const Value V22316_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_327 = {S44, 0, &V22316_327_57316, &P22316_324};	/*Property = TouchEvents*/
const Value V22316_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_328 = {S45, 0, &V22316_328_57319, &P22316_327};	/*Property = Track*/
const Value V22316_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_329 = {S46, 0, &V22316_329_57325, &P22316_328};	/*Property = Video*/
const Value V22316_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_330 = {S47, 0, &V22316_330_57328, &P22316_329};	/*Property = Viewport*/
const Value V22316_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_332 = {S48, 0, &V22316_332_57340, &P22316_330};	/*Property = WebWorkers*/
const Value V22316_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22316_333 = {S49, 0, &V22316_333_57343, &P22316_332};	/*Property = Xhr2*/
const Profile P22316 = {"22316", &P22316_333};	/*Profile ID - 22316*/
const Value V20753_1_1 = {S2, NULL};	/*Value = True*/
const Property P20753_1 = {S50, 0, &V20753_1_1, NULL};	/*Property = IsMobile*/
const Value V20753_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20753_5 = {S52, 0, &V20753_5_1227, &P20753_1};	/*Property = ScreenPixelsHeight*/
const Value V20753_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20753_4 = {S54, 0, &V20753_4_1229, &P20753_5};	/*Property = ScreenPixelsWidth*/
const Profile P20753 = {"20753", &P20753_4};	/*Profile ID - 20753*/
const Value V24193_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24193_279 = {S1, 0, &V24193_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24193_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24193_282 = {S3, 0, &V24193_282_57154, &P24193_279};	/*Property = BlobBuilder*/
const Value V24193_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24193_350 = {S4, 0, &V24193_350_57444, &P24193_282};	/*Property = Canvas*/
const Value V24193_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24193_283 = {S5, 0, &V24193_283_57159, &P24193_350};	/*Property = CssBackground*/
const Value V24193_284_57163 = {S0, NULL};	/*Value = False*/
const Property P24193_284 = {S6, 0, &V24193_284_57163, &P24193_283};	/*Property = CssBorderImage*/
const Value V24193_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24193_285 = {S7, 0, &V24193_285_57166, &P24193_284};	/*Property = CssCanvas*/
const Value V24193_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24193_286 = {S8, 0, &V24193_286_57168, &P24193_285};	/*Property = CssColor*/
const Value V24193_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24193_237 = {S9, 0, &V24193_237_55786, &P24193_286};	/*Property = CssColumn*/
const Value V24193_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24193_287 = {S10, 0, &V24193_287_57172, &P24193_237};	/*Property = CssFlexbox*/
const Value V24193_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24193_288 = {S11, 0, &V24193_288_57175, &P24193_287};	/*Property = CssFont*/
const Value V24193_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24193_289 = {S12, 0, &V24193_289_57759, &P24193_288};	/*Property = CssImages*/
const Value V24193_291_57181 = {S0, NULL};	/*Value = False*/
const Property P24193_291 = {S13, 0, &V24193_291_57181, &P24193_289};	/*Property = CssMediaQueries*/
const Value V24193_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24193_292 = {S14, 0, &V24193_292_57183, &P24193_291};	/*Property = CssMinMax*/
const Value V24193_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24193_293 = {S15, 0, &V24193_293_57187, &P24193_292};	/*Property = CssOverflow*/
const Value V24193_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24193_294 = {S16, 0, &V24193_294_57189, &P24193_293};	/*Property = CssPosition*/
const Value V24193_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24193_295 = {S17, 0, &V24193_295_57192, &P24193_294};	/*Property = CssText*/
const Value V24193_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24193_297 = {S18, 0, &V24193_297_57195, &P24193_295};	/*Property = CssTransforms*/
const Value V24193_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24193_241 = {S19, 0, &V24193_241_55799, &P24193_297};	/*Property = CssTransitions*/
const Value V24193_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24193_298 = {S20, 0, &V24193_298_57206, &P24193_241};	/*Property = CssUI*/
const Value V24193_300_57216 = {S0, NULL};	/*Value = False*/
const Property P24193_300 = {S21, 0, &V24193_300_57216, &P24193_298};	/*Property = DataSet*/
const Value V24193_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24193_301 = {S22, 0, &V24193_301_57218, &P24193_300};	/*Property = DataUrl*/
const Value V24193_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24193_302 = {S23, 0, &V24193_302_57221, &P24193_301};	/*Property = DeviceOrientation*/
const Value V24193_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24193_304 = {S24, 0, &V24193_304_57227, &P24193_302};	/*Property = FileReader*/
const Value V24193_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24193_305 = {S25, 0, &V24193_305_57231, &P24193_304};	/*Property = FileSaver*/
const Value V24193_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24193_306 = {S26, 0, &V24193_306_57237, &P24193_305};	/*Property = FileWriter*/
const Value V24193_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24193_307 = {S27, 0, &V24193_307_57239, &P24193_306};	/*Property = FormData*/
const Value V24193_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24193_309 = {S28, 0, &V24193_309_57246, &P24193_307};	/*Property = Fullscreen*/
const Value V24193_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24193_310 = {S29, 0, &V24193_310_57248, &P24193_309};	/*Property = GeoLocation*/
const Value V24193_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24193_312 = {S30, 0, &V24193_312_57254, &P24193_310};	/*Property = History*/
const Value V24193_314_57261 = {S0, NULL};	/*Value = False*/
const Property P24193_314 = {S31, 0, &V24193_314_57261, &P24193_312};	/*Property = Html5*/
const Value V24193_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24193_313 = {S32, 0, &V24193_313_57258, &P24193_314};	/*Property = Html-Media-Capture*/
const Value V24193_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24193_315 = {S33, 0, &V24193_315_57264, &P24193_313};	/*Property = Iframe*/
const Value V24193_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24193_316 = {S34, 0, &V24193_316_57266, &P24193_315};	/*Property = IndexedDB*/
const Value V24193_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24193_317 = {S35, 0, &V24193_317_57269, &P24193_316};	/*Property = Json*/
const Value V24193_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P24193_15 = {S37, 0, &V24193_15_29, &P24193_317};	/*Property = LayoutEngine*/
const Value V24193_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24193_344 = {S38, 0, &V24193_344_57272, &P24193_15};	/*Property = Masking*/
const Value V24193_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24193_320 = {S39, 0, &V24193_320_57290, &P24193_344};	/*Property = PostMessage*/
const Value V24193_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24193_321 = {S40, 0, &V24193_321_57293, &P24193_320};	/*Property = Progress*/
const Value V24193_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24193_322 = {S41, 0, &V24193_322_57296, &P24193_321};	/*Property = Prompts*/
const Value V24193_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24193_323 = {S42, 0, &V24193_323_57299, &P24193_322};	/*Property = Selector*/
const Value V24193_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24193_324 = {S43, 0, &V24193_324_57306, &P24193_323};	/*Property = Svg*/
const Value V24193_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24193_327 = {S44, 0, &V24193_327_57314, &P24193_324};	/*Property = TouchEvents*/
const Value V24193_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24193_328 = {S45, 0, &V24193_328_57318, &P24193_327};	/*Property = Track*/
const Value V24193_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24193_329 = {S46, 0, &V24193_329_57323, &P24193_328};	/*Property = Video*/
const Value V24193_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24193_330 = {S47, 0, &V24193_330_57326, &P24193_329};	/*Property = Viewport*/
const Value V24193_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24193_332 = {S48, 0, &V24193_332_57339, &P24193_330};	/*Property = WebWorkers*/
const Value V24193_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24193_333 = {S49, 0, &V24193_333_57342, &P24193_332};	/*Property = Xhr2*/
const Profile P24193 = {"24193", &P24193_333};	/*Profile ID - 24193*/
const Value V22431_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22431_279 = {S1, 0, &V22431_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22431_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22431_282 = {S3, 0, &V22431_282_57153, &P22431_279};	/*Property = BlobBuilder*/
const Value V22431_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22431_350 = {S4, 0, &V22431_350_57444, &P22431_282};	/*Property = Canvas*/
const Value V22431_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22431_283 = {S5, 0, &V22431_283_57159, &P22431_350};	/*Property = CssBackground*/
const Value V22431_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22431_284 = {S6, 0, &V22431_284_57163, &P22431_283};	/*Property = CssBorderImage*/
const Value V22431_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22431_285 = {S7, 0, &V22431_285_57166, &P22431_284};	/*Property = CssCanvas*/
const Value V22431_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22431_286 = {S8, 0, &V22431_286_57168, &P22431_285};	/*Property = CssColor*/
const Value V22431_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22431_237 = {S9, 0, &V22431_237_55784, &P22431_286};	/*Property = CssColumn*/
const Value V22431_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22431_287 = {S10, 0, &V22431_287_57172, &P22431_237};	/*Property = CssFlexbox*/
const Value V22431_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22431_288 = {S11, 0, &V22431_288_57175, &P22431_287};	/*Property = CssFont*/
const Value V22431_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22431_289 = {S12, 0, &V22431_289_57759, &P22431_288};	/*Property = CssImages*/
const Value V22431_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22431_291 = {S13, 0, &V22431_291_57180, &P22431_289};	/*Property = CssMediaQueries*/
const Value V22431_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22431_292 = {S14, 0, &V22431_292_57183, &P22431_291};	/*Property = CssMinMax*/
const Value V22431_293_57186 = {S2, NULL};	/*Value = True*/
const Property P22431_293 = {S15, 0, &V22431_293_57186, &P22431_292};	/*Property = CssOverflow*/
const Value V22431_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22431_294 = {S16, 0, &V22431_294_57189, &P22431_293};	/*Property = CssPosition*/
const Value V22431_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22431_295 = {S17, 0, &V22431_295_57192, &P22431_294};	/*Property = CssText*/
const Value V22431_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22431_297 = {S18, 0, &V22431_297_57195, &P22431_295};	/*Property = CssTransforms*/
const Value V22431_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22431_241 = {S19, 0, &V22431_241_55799, &P22431_297};	/*Property = CssTransitions*/
const Value V22431_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22431_298 = {S20, 0, &V22431_298_57206, &P22431_241};	/*Property = CssUI*/
const Value V22431_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22431_300 = {S21, 0, &V22431_300_57216, &P22431_298};	/*Property = DataSet*/
const Value V22431_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22431_301 = {S22, 0, &V22431_301_57218, &P22431_300};	/*Property = DataUrl*/
const Value V22431_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22431_302 = {S23, 0, &V22431_302_57222, &P22431_301};	/*Property = DeviceOrientation*/
const Value V22431_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22431_304 = {S24, 0, &V22431_304_57227, &P22431_302};	/*Property = FileReader*/
const Value V22431_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22431_305 = {S25, 0, &V22431_305_57231, &P22431_304};	/*Property = FileSaver*/
const Value V22431_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22431_306 = {S26, 0, &V22431_306_57237, &P22431_305};	/*Property = FileWriter*/
const Value V22431_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22431_307 = {S27, 0, &V22431_307_57239, &P22431_306};	/*Property = FormData*/
const Value V22431_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22431_309 = {S28, 0, &V22431_309_57246, &P22431_307};	/*Property = Fullscreen*/
const Value V22431_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22431_310 = {S29, 0, &V22431_310_57248, &P22431_309};	/*Property = GeoLocation*/
const Value V22431_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22431_312 = {S30, 0, &V22431_312_57254, &P22431_310};	/*Property = History*/
const Value V22431_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22431_314 = {S31, 0, &V22431_314_57261, &P22431_312};	/*Property = Html5*/
const Value V22431_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22431_313 = {S32, 0, &V22431_313_57258, &P22431_314};	/*Property = Html-Media-Capture*/
const Value V22431_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22431_315 = {S33, 0, &V22431_315_57264, &P22431_313};	/*Property = Iframe*/
const Value V22431_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22431_316 = {S34, 0, &V22431_316_57266, &P22431_315};	/*Property = IndexedDB*/
const Value V22431_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22431_317 = {S35, 0, &V22431_317_57269, &P22431_316};	/*Property = Json*/
const Value V22431_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22431_15 = {S37, 0, &V22431_15_28, &P22431_317};	/*Property = LayoutEngine*/
const Value V22431_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22431_344 = {S38, 0, &V22431_344_57272, &P22431_15};	/*Property = Masking*/
const Value V22431_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22431_320 = {S39, 0, &V22431_320_57290, &P22431_344};	/*Property = PostMessage*/
const Value V22431_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22431_321 = {S40, 0, &V22431_321_57293, &P22431_320};	/*Property = Progress*/
const Value V22431_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22431_322 = {S41, 0, &V22431_322_57296, &P22431_321};	/*Property = Prompts*/
const Value V22431_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22431_323 = {S42, 0, &V22431_323_57299, &P22431_322};	/*Property = Selector*/
const Value V22431_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22431_324 = {S43, 0, &V22431_324_57306, &P22431_323};	/*Property = Svg*/
const Value V22431_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22431_327 = {S44, 0, &V22431_327_57315, &P22431_324};	/*Property = TouchEvents*/
const Value V22431_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22431_328 = {S45, 0, &V22431_328_57318, &P22431_327};	/*Property = Track*/
const Value V22431_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22431_329 = {S46, 0, &V22431_329_57323, &P22431_328};	/*Property = Video*/
const Value V22431_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22431_330 = {S47, 0, &V22431_330_57326, &P22431_329};	/*Property = Viewport*/
const Value V22431_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22431_332 = {S48, 0, &V22431_332_57338, &P22431_330};	/*Property = WebWorkers*/
const Value V22431_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22431_333 = {S49, 0, &V22431_333_57341, &P22431_332};	/*Property = Xhr2*/
const Profile P22431 = {"22431", &P22431_333};	/*Profile ID - 22431*/
const Value V23079_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_279 = {S1, 0, &V23079_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23079_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_282 = {S3, 0, &V23079_282_57155, &P23079_279};	/*Property = BlobBuilder*/
const Value V23079_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_350 = {S4, 0, &V23079_350_57446, &P23079_282};	/*Property = Canvas*/
const Value V23079_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_283 = {S5, 0, &V23079_283_57161, &P23079_350};	/*Property = CssBackground*/
const Value V23079_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_284 = {S6, 0, &V23079_284_57164, &P23079_283};	/*Property = CssBorderImage*/
const Value V23079_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_285 = {S7, 0, &V23079_285_57167, &P23079_284};	/*Property = CssCanvas*/
const Value V23079_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_286 = {S8, 0, &V23079_286_57170, &P23079_285};	/*Property = CssColor*/
const Value V23079_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_237 = {S9, 0, &V23079_237_55786, &P23079_286};	/*Property = CssColumn*/
const Value V23079_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_287 = {S10, 0, &V23079_287_57173, &P23079_237};	/*Property = CssFlexbox*/
const Value V23079_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_288 = {S11, 0, &V23079_288_57176, &P23079_287};	/*Property = CssFont*/
const Value V23079_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_289 = {S12, 0, &V23079_289_86899, &P23079_288};	/*Property = CssImages*/
const Value V23079_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_291 = {S13, 0, &V23079_291_57182, &P23079_289};	/*Property = CssMediaQueries*/
const Value V23079_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_292 = {S14, 0, &V23079_292_57185, &P23079_291};	/*Property = CssMinMax*/
const Value V23079_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_293 = {S15, 0, &V23079_293_57188, &P23079_292};	/*Property = CssOverflow*/
const Value V23079_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_294 = {S16, 0, &V23079_294_57191, &P23079_293};	/*Property = CssPosition*/
const Value V23079_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_295 = {S17, 0, &V23079_295_57194, &P23079_294};	/*Property = CssText*/
const Value V23079_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_297 = {S18, 0, &V23079_297_57197, &P23079_295};	/*Property = CssTransforms*/
const Value V23079_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_241 = {S19, 0, &V23079_241_55801, &P23079_297};	/*Property = CssTransitions*/
const Value V23079_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_298 = {S20, 0, &V23079_298_57208, &P23079_241};	/*Property = CssUI*/
const Value V23079_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_300 = {S21, 0, &V23079_300_57217, &P23079_298};	/*Property = DataSet*/
const Value V23079_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_301 = {S22, 0, &V23079_301_57220, &P23079_300};	/*Property = DataUrl*/
const Value V23079_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_302 = {S23, 0, &V23079_302_57223, &P23079_301};	/*Property = DeviceOrientation*/
const Value V23079_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_304 = {S24, 0, &V23079_304_57229, &P23079_302};	/*Property = FileReader*/
const Value V23079_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_305 = {S25, 0, &V23079_305_57232, &P23079_304};	/*Property = FileSaver*/
const Value V23079_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_306 = {S26, 0, &V23079_306_57238, &P23079_305};	/*Property = FileWriter*/
const Value V23079_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_307 = {S27, 0, &V23079_307_57241, &P23079_306};	/*Property = FormData*/
const Value V23079_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_309 = {S28, 0, &V23079_309_57247, &P23079_307};	/*Property = Fullscreen*/
const Value V23079_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_310 = {S29, 0, &V23079_310_57250, &P23079_309};	/*Property = GeoLocation*/
const Value V23079_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_312 = {S30, 0, &V23079_312_57256, &P23079_310};	/*Property = History*/
const Value V23079_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_314 = {S31, 0, &V23079_314_57262, &P23079_312};	/*Property = Html5*/
const Value V23079_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_313 = {S32, 0, &V23079_313_57259, &P23079_314};	/*Property = Html-Media-Capture*/
const Value V23079_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_315 = {S33, 0, &V23079_315_57265, &P23079_313};	/*Property = Iframe*/
const Value V23079_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_316 = {S34, 0, &V23079_316_57268, &P23079_315};	/*Property = IndexedDB*/
const Value V23079_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_317 = {S35, 0, &V23079_317_57271, &P23079_316};	/*Property = Json*/
const Value V23079_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_15 = {S37, 0, &V23079_15_72, &P23079_317};	/*Property = LayoutEngine*/
const Value V23079_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_344 = {S38, 0, &V23079_344_57274, &P23079_15};	/*Property = Masking*/
const Value V23079_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_320 = {S39, 0, &V23079_320_57292, &P23079_344};	/*Property = PostMessage*/
const Value V23079_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_321 = {S40, 0, &V23079_321_57295, &P23079_320};	/*Property = Progress*/
const Value V23079_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_322 = {S41, 0, &V23079_322_57298, &P23079_321};	/*Property = Prompts*/
const Value V23079_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_323 = {S42, 0, &V23079_323_57301, &P23079_322};	/*Property = Selector*/
const Value V23079_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_324 = {S43, 0, &V23079_324_57307, &P23079_323};	/*Property = Svg*/
const Value V23079_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_327 = {S44, 0, &V23079_327_57316, &P23079_324};	/*Property = TouchEvents*/
const Value V23079_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_328 = {S45, 0, &V23079_328_57319, &P23079_327};	/*Property = Track*/
const Value V23079_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_329 = {S46, 0, &V23079_329_57325, &P23079_328};	/*Property = Video*/
const Value V23079_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_330 = {S47, 0, &V23079_330_57328, &P23079_329};	/*Property = Viewport*/
const Value V23079_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_332 = {S48, 0, &V23079_332_57340, &P23079_330};	/*Property = WebWorkers*/
const Value V23079_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23079_333 = {S49, 0, &V23079_333_57343, &P23079_332};	/*Property = Xhr2*/
const Profile P23079 = {"23079", &P23079_333};	/*Profile ID - 23079*/
const Value V23081_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_279 = {S1, 0, &V23081_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23081_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_282 = {S3, 0, &V23081_282_57155, &P23081_279};	/*Property = BlobBuilder*/
const Value V23081_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_350 = {S4, 0, &V23081_350_57446, &P23081_282};	/*Property = Canvas*/
const Value V23081_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_283 = {S5, 0, &V23081_283_57161, &P23081_350};	/*Property = CssBackground*/
const Value V23081_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_284 = {S6, 0, &V23081_284_57164, &P23081_283};	/*Property = CssBorderImage*/
const Value V23081_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_285 = {S7, 0, &V23081_285_57167, &P23081_284};	/*Property = CssCanvas*/
const Value V23081_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_286 = {S8, 0, &V23081_286_57170, &P23081_285};	/*Property = CssColor*/
const Value V23081_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_237 = {S9, 0, &V23081_237_55786, &P23081_286};	/*Property = CssColumn*/
const Value V23081_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_287 = {S10, 0, &V23081_287_57173, &P23081_237};	/*Property = CssFlexbox*/
const Value V23081_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_288 = {S11, 0, &V23081_288_57176, &P23081_287};	/*Property = CssFont*/
const Value V23081_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_289 = {S12, 0, &V23081_289_86899, &P23081_288};	/*Property = CssImages*/
const Value V23081_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_291 = {S13, 0, &V23081_291_57182, &P23081_289};	/*Property = CssMediaQueries*/
const Value V23081_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_292 = {S14, 0, &V23081_292_57185, &P23081_291};	/*Property = CssMinMax*/
const Value V23081_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_293 = {S15, 0, &V23081_293_57188, &P23081_292};	/*Property = CssOverflow*/
const Value V23081_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_294 = {S16, 0, &V23081_294_57191, &P23081_293};	/*Property = CssPosition*/
const Value V23081_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_295 = {S17, 0, &V23081_295_57194, &P23081_294};	/*Property = CssText*/
const Value V23081_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_297 = {S18, 0, &V23081_297_57197, &P23081_295};	/*Property = CssTransforms*/
const Value V23081_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_241 = {S19, 0, &V23081_241_55801, &P23081_297};	/*Property = CssTransitions*/
const Value V23081_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_298 = {S20, 0, &V23081_298_57208, &P23081_241};	/*Property = CssUI*/
const Value V23081_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_300 = {S21, 0, &V23081_300_57217, &P23081_298};	/*Property = DataSet*/
const Value V23081_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_301 = {S22, 0, &V23081_301_57220, &P23081_300};	/*Property = DataUrl*/
const Value V23081_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_302 = {S23, 0, &V23081_302_57223, &P23081_301};	/*Property = DeviceOrientation*/
const Value V23081_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_304 = {S24, 0, &V23081_304_57229, &P23081_302};	/*Property = FileReader*/
const Value V23081_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_305 = {S25, 0, &V23081_305_57232, &P23081_304};	/*Property = FileSaver*/
const Value V23081_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_306 = {S26, 0, &V23081_306_57238, &P23081_305};	/*Property = FileWriter*/
const Value V23081_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_307 = {S27, 0, &V23081_307_57241, &P23081_306};	/*Property = FormData*/
const Value V23081_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_309 = {S28, 0, &V23081_309_57247, &P23081_307};	/*Property = Fullscreen*/
const Value V23081_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_310 = {S29, 0, &V23081_310_57250, &P23081_309};	/*Property = GeoLocation*/
const Value V23081_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_312 = {S30, 0, &V23081_312_57256, &P23081_310};	/*Property = History*/
const Value V23081_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_314 = {S31, 0, &V23081_314_57262, &P23081_312};	/*Property = Html5*/
const Value V23081_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_313 = {S32, 0, &V23081_313_57259, &P23081_314};	/*Property = Html-Media-Capture*/
const Value V23081_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_315 = {S33, 0, &V23081_315_57265, &P23081_313};	/*Property = Iframe*/
const Value V23081_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_316 = {S34, 0, &V23081_316_57268, &P23081_315};	/*Property = IndexedDB*/
const Value V23081_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_317 = {S35, 0, &V23081_317_57271, &P23081_316};	/*Property = Json*/
const Value V23081_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_15 = {S37, 0, &V23081_15_72, &P23081_317};	/*Property = LayoutEngine*/
const Value V23081_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_344 = {S38, 0, &V23081_344_57274, &P23081_15};	/*Property = Masking*/
const Value V23081_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_320 = {S39, 0, &V23081_320_57292, &P23081_344};	/*Property = PostMessage*/
const Value V23081_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_321 = {S40, 0, &V23081_321_57295, &P23081_320};	/*Property = Progress*/
const Value V23081_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_322 = {S41, 0, &V23081_322_57298, &P23081_321};	/*Property = Prompts*/
const Value V23081_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_323 = {S42, 0, &V23081_323_57301, &P23081_322};	/*Property = Selector*/
const Value V23081_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_324 = {S43, 0, &V23081_324_57307, &P23081_323};	/*Property = Svg*/
const Value V23081_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_327 = {S44, 0, &V23081_327_57316, &P23081_324};	/*Property = TouchEvents*/
const Value V23081_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_328 = {S45, 0, &V23081_328_57319, &P23081_327};	/*Property = Track*/
const Value V23081_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_329 = {S46, 0, &V23081_329_57325, &P23081_328};	/*Property = Video*/
const Value V23081_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_330 = {S47, 0, &V23081_330_57328, &P23081_329};	/*Property = Viewport*/
const Value V23081_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_332 = {S48, 0, &V23081_332_57340, &P23081_330};	/*Property = WebWorkers*/
const Value V23081_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23081_333 = {S49, 0, &V23081_333_57343, &P23081_332};	/*Property = Xhr2*/
const Profile P23081 = {"23081", &P23081_333};	/*Profile ID - 23081*/
const Profile P18571 = {"18571", NULL};	/*Profile ID - 18571*/
const Value V24630_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24630_279 = {S1, 0, &V24630_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24630_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24630_282 = {S3, 0, &V24630_282_57154, &P24630_279};	/*Property = BlobBuilder*/
const Value V24630_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24630_350 = {S4, 0, &V24630_350_57444, &P24630_282};	/*Property = Canvas*/
const Value V24630_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24630_283 = {S5, 0, &V24630_283_57159, &P24630_350};	/*Property = CssBackground*/
const Value V24630_284_57163 = {S0, NULL};	/*Value = False*/
const Property P24630_284 = {S6, 0, &V24630_284_57163, &P24630_283};	/*Property = CssBorderImage*/
const Value V24630_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24630_285 = {S7, 0, &V24630_285_57166, &P24630_284};	/*Property = CssCanvas*/
const Value V24630_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24630_286 = {S8, 0, &V24630_286_57168, &P24630_285};	/*Property = CssColor*/
const Value V24630_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24630_237 = {S9, 0, &V24630_237_55786, &P24630_286};	/*Property = CssColumn*/
const Value V24630_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24630_287 = {S10, 0, &V24630_287_57172, &P24630_237};	/*Property = CssFlexbox*/
const Value V24630_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24630_288 = {S11, 0, &V24630_288_57176, &P24630_287};	/*Property = CssFont*/
const Value V24630_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24630_289 = {S12, 0, &V24630_289_57759, &P24630_288};	/*Property = CssImages*/
const Value V24630_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24630_291 = {S13, 0, &V24630_291_57182, &P24630_289};	/*Property = CssMediaQueries*/
const Value V24630_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24630_292 = {S14, 0, &V24630_292_57183, &P24630_291};	/*Property = CssMinMax*/
const Value V24630_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24630_293 = {S15, 0, &V24630_293_57188, &P24630_292};	/*Property = CssOverflow*/
const Value V24630_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24630_294 = {S16, 0, &V24630_294_57191, &P24630_293};	/*Property = CssPosition*/
const Value V24630_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24630_295 = {S17, 0, &V24630_295_57192, &P24630_294};	/*Property = CssText*/
const Value V24630_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24630_297 = {S18, 0, &V24630_297_57195, &P24630_295};	/*Property = CssTransforms*/
const Value V24630_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24630_241 = {S19, 0, &V24630_241_55799, &P24630_297};	/*Property = CssTransitions*/
const Value V24630_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24630_298 = {S20, 0, &V24630_298_57206, &P24630_241};	/*Property = CssUI*/
const Value V24630_300_57216 = {S0, NULL};	/*Value = False*/
const Property P24630_300 = {S21, 0, &V24630_300_57216, &P24630_298};	/*Property = DataSet*/
const Value V24630_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24630_301 = {S22, 0, &V24630_301_57218, &P24630_300};	/*Property = DataUrl*/
const Value V24630_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24630_302 = {S23, 0, &V24630_302_57223, &P24630_301};	/*Property = DeviceOrientation*/
const Value V24630_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24630_304 = {S24, 0, &V24630_304_57229, &P24630_302};	/*Property = FileReader*/
const Value V24630_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24630_305 = {S25, 0, &V24630_305_57231, &P24630_304};	/*Property = FileSaver*/
const Value V24630_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24630_306 = {S26, 0, &V24630_306_57237, &P24630_305};	/*Property = FileWriter*/
const Value V24630_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24630_307 = {S27, 0, &V24630_307_57241, &P24630_306};	/*Property = FormData*/
const Value V24630_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24630_309 = {S28, 0, &V24630_309_57246, &P24630_307};	/*Property = Fullscreen*/
const Value V24630_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24630_310 = {S29, 0, &V24630_310_57248, &P24630_309};	/*Property = GeoLocation*/
const Value V24630_312_57255 = {S0, NULL};	/*Value = False*/
const Property P24630_312 = {S30, 0, &V24630_312_57255, &P24630_310};	/*Property = History*/
const Value V24630_314_57261 = {S0, NULL};	/*Value = False*/
const Property P24630_314 = {S31, 0, &V24630_314_57261, &P24630_312};	/*Property = Html5*/
const Value V24630_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24630_313 = {S32, 0, &V24630_313_57258, &P24630_314};	/*Property = Html-Media-Capture*/
const Value V24630_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24630_315 = {S33, 0, &V24630_315_57264, &P24630_313};	/*Property = Iframe*/
const Value V24630_316_57267 = {S0, NULL};	/*Value = False*/
const Property P24630_316 = {S34, 0, &V24630_316_57267, &P24630_315};	/*Property = IndexedDB*/
const Value V24630_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24630_317 = {S35, 0, &V24630_317_57269, &P24630_316};	/*Property = Json*/
const Value V24630_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24630_15 = {S37, 0, &V24630_15_90837, &P24630_317};	/*Property = LayoutEngine*/
const Value V24630_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24630_344 = {S38, 0, &V24630_344_57272, &P24630_15};	/*Property = Masking*/
const Value V24630_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24630_320 = {S39, 0, &V24630_320_57290, &P24630_344};	/*Property = PostMessage*/
const Value V24630_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24630_321 = {S40, 0, &V24630_321_57293, &P24630_320};	/*Property = Progress*/
const Value V24630_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24630_322 = {S41, 0, &V24630_322_57296, &P24630_321};	/*Property = Prompts*/
const Value V24630_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24630_323 = {S42, 0, &V24630_323_57299, &P24630_322};	/*Property = Selector*/
const Value V24630_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24630_324 = {S43, 0, &V24630_324_57306, &P24630_323};	/*Property = Svg*/
const Value V24630_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24630_327 = {S44, 0, &V24630_327_57316, &P24630_324};	/*Property = TouchEvents*/
const Value V24630_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24630_328 = {S45, 0, &V24630_328_57318, &P24630_327};	/*Property = Track*/
const Value V24630_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24630_329 = {S46, 0, &V24630_329_57323, &P24630_328};	/*Property = Video*/
const Value V24630_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24630_330 = {S47, 0, &V24630_330_57326, &P24630_329};	/*Property = Viewport*/
const Value V24630_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24630_332 = {S48, 0, &V24630_332_57339, &P24630_330};	/*Property = WebWorkers*/
const Value V24630_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24630_333 = {S49, 0, &V24630_333_57342, &P24630_332};	/*Property = Xhr2*/
const Profile P24630 = {"24630", &P24630_333};	/*Profile ID - 24630*/
const Value V21135_1_1 = {S2, NULL};	/*Value = True*/
const Property P21135_1 = {S50, 0, &V21135_1_1, NULL};	/*Property = IsMobile*/
const Value V21135_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21135_5 = {S52, 0, &V21135_5_1227, &P21135_1};	/*Property = ScreenPixelsHeight*/
const Value V21135_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21135_4 = {S54, 0, &V21135_4_1229, &P21135_5};	/*Property = ScreenPixelsWidth*/
const Profile P21135 = {"21135", &P21135_4};	/*Profile ID - 21135*/
const Value V22375_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22375_279 = {S1, 0, &V22375_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22375_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22375_282 = {S3, 0, &V22375_282_57155, &P22375_279};	/*Property = BlobBuilder*/
const Value V22375_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22375_350 = {S4, 0, &V22375_350_57444, &P22375_282};	/*Property = Canvas*/
const Value V22375_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22375_283 = {S5, 0, &V22375_283_57159, &P22375_350};	/*Property = CssBackground*/
const Value V22375_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22375_284 = {S6, 0, &V22375_284_57163, &P22375_283};	/*Property = CssBorderImage*/
const Value V22375_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22375_285 = {S7, 0, &V22375_285_57166, &P22375_284};	/*Property = CssCanvas*/
const Value V22375_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22375_286 = {S8, 0, &V22375_286_57168, &P22375_285};	/*Property = CssColor*/
const Value V22375_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22375_237 = {S9, 0, &V22375_237_55786, &P22375_286};	/*Property = CssColumn*/
const Value V22375_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22375_287 = {S10, 0, &V22375_287_57172, &P22375_237};	/*Property = CssFlexbox*/
const Value V22375_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22375_288 = {S11, 0, &V22375_288_57176, &P22375_287};	/*Property = CssFont*/
const Value V22375_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22375_289 = {S12, 0, &V22375_289_57759, &P22375_288};	/*Property = CssImages*/
const Value V22375_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22375_291 = {S13, 0, &V22375_291_57180, &P22375_289};	/*Property = CssMediaQueries*/
const Value V22375_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22375_292 = {S14, 0, &V22375_292_57183, &P22375_291};	/*Property = CssMinMax*/
const Value V22375_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22375_293 = {S15, 0, &V22375_293_57187, &P22375_292};	/*Property = CssOverflow*/
const Value V22375_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22375_294 = {S16, 0, &V22375_294_57189, &P22375_293};	/*Property = CssPosition*/
const Value V22375_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22375_295 = {S17, 0, &V22375_295_57192, &P22375_294};	/*Property = CssText*/
const Value V22375_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22375_297 = {S18, 0, &V22375_297_57195, &P22375_295};	/*Property = CssTransforms*/
const Value V22375_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22375_241 = {S19, 0, &V22375_241_55799, &P22375_297};	/*Property = CssTransitions*/
const Value V22375_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22375_298 = {S20, 0, &V22375_298_57206, &P22375_241};	/*Property = CssUI*/
const Value V22375_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22375_300 = {S21, 0, &V22375_300_57216, &P22375_298};	/*Property = DataSet*/
const Value V22375_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22375_301 = {S22, 0, &V22375_301_57218, &P22375_300};	/*Property = DataUrl*/
const Value V22375_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22375_302 = {S23, 0, &V22375_302_57222, &P22375_301};	/*Property = DeviceOrientation*/
const Value V22375_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22375_304 = {S24, 0, &V22375_304_57227, &P22375_302};	/*Property = FileReader*/
const Value V22375_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22375_305 = {S25, 0, &V22375_305_57231, &P22375_304};	/*Property = FileSaver*/
const Value V22375_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22375_306 = {S26, 0, &V22375_306_57237, &P22375_305};	/*Property = FileWriter*/
const Value V22375_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22375_307 = {S27, 0, &V22375_307_57239, &P22375_306};	/*Property = FormData*/
const Value V22375_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22375_309 = {S28, 0, &V22375_309_57246, &P22375_307};	/*Property = Fullscreen*/
const Value V22375_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22375_310 = {S29, 0, &V22375_310_57248, &P22375_309};	/*Property = GeoLocation*/
const Value V22375_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22375_312 = {S30, 0, &V22375_312_57254, &P22375_310};	/*Property = History*/
const Value V22375_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22375_314 = {S31, 0, &V22375_314_57261, &P22375_312};	/*Property = Html5*/
const Value V22375_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22375_313 = {S32, 0, &V22375_313_57258, &P22375_314};	/*Property = Html-Media-Capture*/
const Value V22375_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22375_315 = {S33, 0, &V22375_315_57264, &P22375_313};	/*Property = Iframe*/
const Value V22375_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22375_316 = {S34, 0, &V22375_316_57266, &P22375_315};	/*Property = IndexedDB*/
const Value V22375_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22375_317 = {S35, 0, &V22375_317_57269, &P22375_316};	/*Property = Json*/
const Value V22375_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22375_15 = {S37, 0, &V22375_15_28, &P22375_317};	/*Property = LayoutEngine*/
const Value V22375_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22375_344 = {S38, 0, &V22375_344_57272, &P22375_15};	/*Property = Masking*/
const Value V22375_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22375_320 = {S39, 0, &V22375_320_57290, &P22375_344};	/*Property = PostMessage*/
const Value V22375_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22375_321 = {S40, 0, &V22375_321_57293, &P22375_320};	/*Property = Progress*/
const Value V22375_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22375_322 = {S41, 0, &V22375_322_57296, &P22375_321};	/*Property = Prompts*/
const Value V22375_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22375_323 = {S42, 0, &V22375_323_57299, &P22375_322};	/*Property = Selector*/
const Value V22375_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22375_324 = {S43, 0, &V22375_324_57306, &P22375_323};	/*Property = Svg*/
const Value V22375_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22375_327 = {S44, 0, &V22375_327_57315, &P22375_324};	/*Property = TouchEvents*/
const Value V22375_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22375_328 = {S45, 0, &V22375_328_57318, &P22375_327};	/*Property = Track*/
const Value V22375_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22375_329 = {S46, 0, &V22375_329_57323, &P22375_328};	/*Property = Video*/
const Value V22375_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22375_330 = {S47, 0, &V22375_330_57326, &P22375_329};	/*Property = Viewport*/
const Value V22375_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22375_332 = {S48, 0, &V22375_332_57340, &P22375_330};	/*Property = WebWorkers*/
const Value V22375_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22375_333 = {S49, 0, &V22375_333_57343, &P22375_332};	/*Property = Xhr2*/
const Profile P22375 = {"22375", &P22375_333};	/*Profile ID - 22375*/
const Value V24671_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_279 = {S1, 0, &V24671_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24671_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_282 = {S3, 0, &V24671_282_57155, &P24671_279};	/*Property = BlobBuilder*/
const Value V24671_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_350 = {S4, 0, &V24671_350_57446, &P24671_282};	/*Property = Canvas*/
const Value V24671_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_283 = {S5, 0, &V24671_283_57161, &P24671_350};	/*Property = CssBackground*/
const Value V24671_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_284 = {S6, 0, &V24671_284_57164, &P24671_283};	/*Property = CssBorderImage*/
const Value V24671_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_285 = {S7, 0, &V24671_285_57167, &P24671_284};	/*Property = CssCanvas*/
const Value V24671_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_286 = {S8, 0, &V24671_286_57170, &P24671_285};	/*Property = CssColor*/
const Value V24671_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_237 = {S9, 0, &V24671_237_55786, &P24671_286};	/*Property = CssColumn*/
const Value V24671_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_287 = {S10, 0, &V24671_287_57173, &P24671_237};	/*Property = CssFlexbox*/
const Value V24671_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_288 = {S11, 0, &V24671_288_57176, &P24671_287};	/*Property = CssFont*/
const Value V24671_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_289 = {S12, 0, &V24671_289_86899, &P24671_288};	/*Property = CssImages*/
const Value V24671_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_291 = {S13, 0, &V24671_291_57182, &P24671_289};	/*Property = CssMediaQueries*/
const Value V24671_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_292 = {S14, 0, &V24671_292_57185, &P24671_291};	/*Property = CssMinMax*/
const Value V24671_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_293 = {S15, 0, &V24671_293_57188, &P24671_292};	/*Property = CssOverflow*/
const Value V24671_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_294 = {S16, 0, &V24671_294_57191, &P24671_293};	/*Property = CssPosition*/
const Value V24671_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_295 = {S17, 0, &V24671_295_57194, &P24671_294};	/*Property = CssText*/
const Value V24671_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_297 = {S18, 0, &V24671_297_57197, &P24671_295};	/*Property = CssTransforms*/
const Value V24671_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_241 = {S19, 0, &V24671_241_55801, &P24671_297};	/*Property = CssTransitions*/
const Value V24671_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_298 = {S20, 0, &V24671_298_57208, &P24671_241};	/*Property = CssUI*/
const Value V24671_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_300 = {S21, 0, &V24671_300_57217, &P24671_298};	/*Property = DataSet*/
const Value V24671_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_301 = {S22, 0, &V24671_301_57220, &P24671_300};	/*Property = DataUrl*/
const Value V24671_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_302 = {S23, 0, &V24671_302_57223, &P24671_301};	/*Property = DeviceOrientation*/
const Value V24671_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_304 = {S24, 0, &V24671_304_57229, &P24671_302};	/*Property = FileReader*/
const Value V24671_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_305 = {S25, 0, &V24671_305_57232, &P24671_304};	/*Property = FileSaver*/
const Value V24671_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_306 = {S26, 0, &V24671_306_57238, &P24671_305};	/*Property = FileWriter*/
const Value V24671_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_307 = {S27, 0, &V24671_307_57241, &P24671_306};	/*Property = FormData*/
const Value V24671_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_309 = {S28, 0, &V24671_309_57247, &P24671_307};	/*Property = Fullscreen*/
const Value V24671_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_310 = {S29, 0, &V24671_310_57250, &P24671_309};	/*Property = GeoLocation*/
const Value V24671_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_312 = {S30, 0, &V24671_312_57256, &P24671_310};	/*Property = History*/
const Value V24671_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_314 = {S31, 0, &V24671_314_57262, &P24671_312};	/*Property = Html5*/
const Value V24671_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_313 = {S32, 0, &V24671_313_57259, &P24671_314};	/*Property = Html-Media-Capture*/
const Value V24671_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_315 = {S33, 0, &V24671_315_57265, &P24671_313};	/*Property = Iframe*/
const Value V24671_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_316 = {S34, 0, &V24671_316_57268, &P24671_315};	/*Property = IndexedDB*/
const Value V24671_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_317 = {S35, 0, &V24671_317_57271, &P24671_316};	/*Property = Json*/
const Value V24671_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24671_15 = {S37, 0, &V24671_15_90837, &P24671_317};	/*Property = LayoutEngine*/
const Value V24671_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_344 = {S38, 0, &V24671_344_57274, &P24671_15};	/*Property = Masking*/
const Value V24671_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_320 = {S39, 0, &V24671_320_57292, &P24671_344};	/*Property = PostMessage*/
const Value V24671_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_321 = {S40, 0, &V24671_321_57295, &P24671_320};	/*Property = Progress*/
const Value V24671_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_322 = {S41, 0, &V24671_322_57298, &P24671_321};	/*Property = Prompts*/
const Value V24671_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_323 = {S42, 0, &V24671_323_57301, &P24671_322};	/*Property = Selector*/
const Value V24671_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_324 = {S43, 0, &V24671_324_57307, &P24671_323};	/*Property = Svg*/
const Value V24671_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_327 = {S44, 0, &V24671_327_57316, &P24671_324};	/*Property = TouchEvents*/
const Value V24671_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_328 = {S45, 0, &V24671_328_57319, &P24671_327};	/*Property = Track*/
const Value V24671_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_329 = {S46, 0, &V24671_329_57325, &P24671_328};	/*Property = Video*/
const Value V24671_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_330 = {S47, 0, &V24671_330_57328, &P24671_329};	/*Property = Viewport*/
const Value V24671_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_332 = {S48, 0, &V24671_332_57340, &P24671_330};	/*Property = WebWorkers*/
const Value V24671_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24671_333 = {S49, 0, &V24671_333_57343, &P24671_332};	/*Property = Xhr2*/
const Profile P24671 = {"24671", &P24671_333};	/*Profile ID - 24671*/
const Value V20709_1_1 = {S2, NULL};	/*Value = True*/
const Property P20709_1 = {S50, 0, &V20709_1_1, NULL};	/*Property = IsMobile*/
const Value V20709_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20709_5 = {S52, 0, &V20709_5_1458, &P20709_1};	/*Property = ScreenPixelsHeight*/
const Value V20709_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20709_4 = {S54, 0, &V20709_4_1460, &P20709_5};	/*Property = ScreenPixelsWidth*/
const Profile P20709 = {"20709", &P20709_4};	/*Profile ID - 20709*/
const Value V21036_1_1 = {S2, NULL};	/*Value = True*/
const Property P21036_1 = {S50, 0, &V21036_1_1, NULL};	/*Property = IsMobile*/
const Value V21036_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21036_5 = {S52, 0, &V21036_5_1227, &P21036_1};	/*Property = ScreenPixelsHeight*/
const Value V21036_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21036_4 = {S54, 0, &V21036_4_1229, &P21036_5};	/*Property = ScreenPixelsWidth*/
const Profile P21036 = {"21036", &P21036_4};	/*Profile ID - 21036*/
const Value V21227_1_1 = {S2, NULL};	/*Value = True*/
const Property P21227_1 = {S50, 0, &V21227_1_1, NULL};	/*Property = IsMobile*/
const Value V21227_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21227_5 = {S52, 0, &V21227_5_37, &P21227_1};	/*Property = ScreenPixelsHeight*/
const Value V21227_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21227_4 = {S54, 0, &V21227_4_36, &P21227_5};	/*Property = ScreenPixelsWidth*/
const Profile P21227 = {"21227", &P21227_4};	/*Profile ID - 21227*/
const Value V21120_1_1 = {S2, NULL};	/*Value = True*/
const Property P21120_1 = {S50, 0, &V21120_1_1, NULL};	/*Property = IsMobile*/
const Value V21120_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21120_5 = {S52, 0, &V21120_5_1277, &P21120_1};	/*Property = ScreenPixelsHeight*/
const Value V21120_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21120_4 = {S54, 0, &V21120_4_2336, &P21120_5};	/*Property = ScreenPixelsWidth*/
const Profile P21120 = {"21120", &P21120_4};	/*Profile ID - 21120*/
const Value V23051_1_2 = {S0, NULL};	/*Value = False*/
const Property P23051_1 = {S50, 0, &V23051_1_2, NULL};	/*Property = IsMobile*/
const Value V23051_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23051_5 = {S52, 0, &V23051_5_1222, &P23051_1};	/*Property = ScreenPixelsHeight*/
const Value V23051_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23051_4 = {S54, 0, &V23051_4_1224, &P23051_5};	/*Property = ScreenPixelsWidth*/
const Profile P23051 = {"23051", &P23051_4};	/*Profile ID - 23051*/
const Value V18535_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_279 = {S1, 0, &V18535_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18535_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_282 = {S3, 0, &V18535_282_57155, &P18535_279};	/*Property = BlobBuilder*/
const Value V18535_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_350 = {S4, 0, &V18535_350_57446, &P18535_282};	/*Property = Canvas*/
const Value V18535_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_283 = {S5, 0, &V18535_283_57161, &P18535_350};	/*Property = CssBackground*/
const Value V18535_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_284 = {S6, 0, &V18535_284_57164, &P18535_283};	/*Property = CssBorderImage*/
const Value V18535_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_285 = {S7, 0, &V18535_285_57167, &P18535_284};	/*Property = CssCanvas*/
const Value V18535_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_286 = {S8, 0, &V18535_286_57170, &P18535_285};	/*Property = CssColor*/
const Value V18535_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_237 = {S9, 0, &V18535_237_55786, &P18535_286};	/*Property = CssColumn*/
const Value V18535_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_287 = {S10, 0, &V18535_287_57173, &P18535_237};	/*Property = CssFlexbox*/
const Value V18535_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_288 = {S11, 0, &V18535_288_57176, &P18535_287};	/*Property = CssFont*/
const Value V18535_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_289 = {S12, 0, &V18535_289_86899, &P18535_288};	/*Property = CssImages*/
const Value V18535_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_291 = {S13, 0, &V18535_291_57182, &P18535_289};	/*Property = CssMediaQueries*/
const Value V18535_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_292 = {S14, 0, &V18535_292_57185, &P18535_291};	/*Property = CssMinMax*/
const Value V18535_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_293 = {S15, 0, &V18535_293_57188, &P18535_292};	/*Property = CssOverflow*/
const Value V18535_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_294 = {S16, 0, &V18535_294_57191, &P18535_293};	/*Property = CssPosition*/
const Value V18535_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_295 = {S17, 0, &V18535_295_57194, &P18535_294};	/*Property = CssText*/
const Value V18535_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_297 = {S18, 0, &V18535_297_57197, &P18535_295};	/*Property = CssTransforms*/
const Value V18535_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_241 = {S19, 0, &V18535_241_55801, &P18535_297};	/*Property = CssTransitions*/
const Value V18535_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_298 = {S20, 0, &V18535_298_57208, &P18535_241};	/*Property = CssUI*/
const Value V18535_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_300 = {S21, 0, &V18535_300_57217, &P18535_298};	/*Property = DataSet*/
const Value V18535_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_301 = {S22, 0, &V18535_301_57220, &P18535_300};	/*Property = DataUrl*/
const Value V18535_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_302 = {S23, 0, &V18535_302_57223, &P18535_301};	/*Property = DeviceOrientation*/
const Value V18535_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_304 = {S24, 0, &V18535_304_57229, &P18535_302};	/*Property = FileReader*/
const Value V18535_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_305 = {S25, 0, &V18535_305_57232, &P18535_304};	/*Property = FileSaver*/
const Value V18535_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_306 = {S26, 0, &V18535_306_57238, &P18535_305};	/*Property = FileWriter*/
const Value V18535_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_307 = {S27, 0, &V18535_307_57241, &P18535_306};	/*Property = FormData*/
const Value V18535_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_309 = {S28, 0, &V18535_309_57247, &P18535_307};	/*Property = Fullscreen*/
const Value V18535_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_310 = {S29, 0, &V18535_310_57250, &P18535_309};	/*Property = GeoLocation*/
const Value V18535_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_312 = {S30, 0, &V18535_312_57256, &P18535_310};	/*Property = History*/
const Value V18535_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_314 = {S31, 0, &V18535_314_57262, &P18535_312};	/*Property = Html5*/
const Value V18535_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_313 = {S32, 0, &V18535_313_57259, &P18535_314};	/*Property = Html-Media-Capture*/
const Value V18535_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_315 = {S33, 0, &V18535_315_57265, &P18535_313};	/*Property = Iframe*/
const Value V18535_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_316 = {S34, 0, &V18535_316_57268, &P18535_315};	/*Property = IndexedDB*/
const Value V18535_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_317 = {S35, 0, &V18535_317_57271, &P18535_316};	/*Property = Json*/
const Value V18535_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18535_15 = {S37, 0, &V18535_15_28, &P18535_317};	/*Property = LayoutEngine*/
const Value V18535_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_344 = {S38, 0, &V18535_344_57274, &P18535_15};	/*Property = Masking*/
const Value V18535_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_320 = {S39, 0, &V18535_320_57292, &P18535_344};	/*Property = PostMessage*/
const Value V18535_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_321 = {S40, 0, &V18535_321_57295, &P18535_320};	/*Property = Progress*/
const Value V18535_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_322 = {S41, 0, &V18535_322_57298, &P18535_321};	/*Property = Prompts*/
const Value V18535_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_323 = {S42, 0, &V18535_323_57301, &P18535_322};	/*Property = Selector*/
const Value V18535_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_324 = {S43, 0, &V18535_324_57307, &P18535_323};	/*Property = Svg*/
const Value V18535_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_327 = {S44, 0, &V18535_327_57316, &P18535_324};	/*Property = TouchEvents*/
const Value V18535_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_328 = {S45, 0, &V18535_328_57319, &P18535_327};	/*Property = Track*/
const Value V18535_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_329 = {S46, 0, &V18535_329_57325, &P18535_328};	/*Property = Video*/
const Value V18535_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_330 = {S47, 0, &V18535_330_57328, &P18535_329};	/*Property = Viewport*/
const Value V18535_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_332 = {S48, 0, &V18535_332_57340, &P18535_330};	/*Property = WebWorkers*/
const Value V18535_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18535_333 = {S49, 0, &V18535_333_57343, &P18535_332};	/*Property = Xhr2*/
const Profile P18535 = {"18535", &P18535_333};	/*Profile ID - 18535*/
const Value V20785_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_279 = {S1, 0, &V20785_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20785_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_282 = {S3, 0, &V20785_282_57155, &P20785_279};	/*Property = BlobBuilder*/
const Value V20785_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_350 = {S4, 0, &V20785_350_57446, &P20785_282};	/*Property = Canvas*/
const Value V20785_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_283 = {S5, 0, &V20785_283_57161, &P20785_350};	/*Property = CssBackground*/
const Value V20785_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_284 = {S6, 0, &V20785_284_57164, &P20785_283};	/*Property = CssBorderImage*/
const Value V20785_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_285 = {S7, 0, &V20785_285_57167, &P20785_284};	/*Property = CssCanvas*/
const Value V20785_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_286 = {S8, 0, &V20785_286_57170, &P20785_285};	/*Property = CssColor*/
const Value V20785_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_237 = {S9, 0, &V20785_237_55786, &P20785_286};	/*Property = CssColumn*/
const Value V20785_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_287 = {S10, 0, &V20785_287_57173, &P20785_237};	/*Property = CssFlexbox*/
const Value V20785_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_288 = {S11, 0, &V20785_288_57176, &P20785_287};	/*Property = CssFont*/
const Value V20785_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_289 = {S12, 0, &V20785_289_86899, &P20785_288};	/*Property = CssImages*/
const Value V20785_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_291 = {S13, 0, &V20785_291_57182, &P20785_289};	/*Property = CssMediaQueries*/
const Value V20785_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_292 = {S14, 0, &V20785_292_57185, &P20785_291};	/*Property = CssMinMax*/
const Value V20785_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_293 = {S15, 0, &V20785_293_57188, &P20785_292};	/*Property = CssOverflow*/
const Value V20785_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_294 = {S16, 0, &V20785_294_57191, &P20785_293};	/*Property = CssPosition*/
const Value V20785_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_295 = {S17, 0, &V20785_295_57194, &P20785_294};	/*Property = CssText*/
const Value V20785_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_297 = {S18, 0, &V20785_297_57197, &P20785_295};	/*Property = CssTransforms*/
const Value V20785_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_241 = {S19, 0, &V20785_241_55801, &P20785_297};	/*Property = CssTransitions*/
const Value V20785_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_298 = {S20, 0, &V20785_298_57208, &P20785_241};	/*Property = CssUI*/
const Value V20785_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_300 = {S21, 0, &V20785_300_57217, &P20785_298};	/*Property = DataSet*/
const Value V20785_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_301 = {S22, 0, &V20785_301_57220, &P20785_300};	/*Property = DataUrl*/
const Value V20785_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_302 = {S23, 0, &V20785_302_57223, &P20785_301};	/*Property = DeviceOrientation*/
const Value V20785_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_304 = {S24, 0, &V20785_304_57229, &P20785_302};	/*Property = FileReader*/
const Value V20785_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_305 = {S25, 0, &V20785_305_57232, &P20785_304};	/*Property = FileSaver*/
const Value V20785_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_306 = {S26, 0, &V20785_306_57238, &P20785_305};	/*Property = FileWriter*/
const Value V20785_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_307 = {S27, 0, &V20785_307_57241, &P20785_306};	/*Property = FormData*/
const Value V20785_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_309 = {S28, 0, &V20785_309_57247, &P20785_307};	/*Property = Fullscreen*/
const Value V20785_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_310 = {S29, 0, &V20785_310_57250, &P20785_309};	/*Property = GeoLocation*/
const Value V20785_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_312 = {S30, 0, &V20785_312_57256, &P20785_310};	/*Property = History*/
const Value V20785_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_314 = {S31, 0, &V20785_314_57262, &P20785_312};	/*Property = Html5*/
const Value V20785_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_313 = {S32, 0, &V20785_313_57259, &P20785_314};	/*Property = Html-Media-Capture*/
const Value V20785_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_315 = {S33, 0, &V20785_315_57265, &P20785_313};	/*Property = Iframe*/
const Value V20785_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_316 = {S34, 0, &V20785_316_57268, &P20785_315};	/*Property = IndexedDB*/
const Value V20785_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_317 = {S35, 0, &V20785_317_57271, &P20785_316};	/*Property = Json*/
const Value V20785_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P20785_15 = {S37, 0, &V20785_15_54167, &P20785_317};	/*Property = LayoutEngine*/
const Value V20785_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_344 = {S38, 0, &V20785_344_57274, &P20785_15};	/*Property = Masking*/
const Value V20785_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_320 = {S39, 0, &V20785_320_57292, &P20785_344};	/*Property = PostMessage*/
const Value V20785_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_321 = {S40, 0, &V20785_321_57295, &P20785_320};	/*Property = Progress*/
const Value V20785_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_322 = {S41, 0, &V20785_322_57298, &P20785_321};	/*Property = Prompts*/
const Value V20785_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_323 = {S42, 0, &V20785_323_57301, &P20785_322};	/*Property = Selector*/
const Value V20785_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_324 = {S43, 0, &V20785_324_57307, &P20785_323};	/*Property = Svg*/
const Value V20785_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_327 = {S44, 0, &V20785_327_57316, &P20785_324};	/*Property = TouchEvents*/
const Value V20785_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_328 = {S45, 0, &V20785_328_57319, &P20785_327};	/*Property = Track*/
const Value V20785_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_329 = {S46, 0, &V20785_329_57325, &P20785_328};	/*Property = Video*/
const Value V20785_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_330 = {S47, 0, &V20785_330_57328, &P20785_329};	/*Property = Viewport*/
const Value V20785_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_332 = {S48, 0, &V20785_332_57340, &P20785_330};	/*Property = WebWorkers*/
const Value V20785_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20785_333 = {S49, 0, &V20785_333_57343, &P20785_332};	/*Property = Xhr2*/
const Profile P20785 = {"20785", &P20785_333};	/*Profile ID - 20785*/
const Value V18621_1_1 = {S2, NULL};	/*Value = True*/
const Property P18621_1 = {S50, 0, &V18621_1_1, NULL};	/*Property = IsMobile*/
const Value V18621_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18621_5 = {S52, 0, &V18621_5_1277, &P18621_1};	/*Property = ScreenPixelsHeight*/
const Value V18621_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18621_4 = {S54, 0, &V18621_4_1279, &P18621_5};	/*Property = ScreenPixelsWidth*/
const Profile P18621 = {"18621", &P18621_4};	/*Profile ID - 18621*/
const Value V18515_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_279 = {S1, 0, &V18515_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18515_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_282 = {S3, 0, &V18515_282_57155, &P18515_279};	/*Property = BlobBuilder*/
const Value V18515_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_350 = {S4, 0, &V18515_350_57446, &P18515_282};	/*Property = Canvas*/
const Value V18515_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_283 = {S5, 0, &V18515_283_57161, &P18515_350};	/*Property = CssBackground*/
const Value V18515_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_284 = {S6, 0, &V18515_284_57164, &P18515_283};	/*Property = CssBorderImage*/
const Value V18515_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_285 = {S7, 0, &V18515_285_57167, &P18515_284};	/*Property = CssCanvas*/
const Value V18515_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_286 = {S8, 0, &V18515_286_57170, &P18515_285};	/*Property = CssColor*/
const Value V18515_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_237 = {S9, 0, &V18515_237_55786, &P18515_286};	/*Property = CssColumn*/
const Value V18515_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_287 = {S10, 0, &V18515_287_57173, &P18515_237};	/*Property = CssFlexbox*/
const Value V18515_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_288 = {S11, 0, &V18515_288_57176, &P18515_287};	/*Property = CssFont*/
const Value V18515_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_289 = {S12, 0, &V18515_289_86899, &P18515_288};	/*Property = CssImages*/
const Value V18515_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_291 = {S13, 0, &V18515_291_57182, &P18515_289};	/*Property = CssMediaQueries*/
const Value V18515_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_292 = {S14, 0, &V18515_292_57185, &P18515_291};	/*Property = CssMinMax*/
const Value V18515_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_293 = {S15, 0, &V18515_293_57188, &P18515_292};	/*Property = CssOverflow*/
const Value V18515_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_294 = {S16, 0, &V18515_294_57191, &P18515_293};	/*Property = CssPosition*/
const Value V18515_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_295 = {S17, 0, &V18515_295_57194, &P18515_294};	/*Property = CssText*/
const Value V18515_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_297 = {S18, 0, &V18515_297_57197, &P18515_295};	/*Property = CssTransforms*/
const Value V18515_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_241 = {S19, 0, &V18515_241_55801, &P18515_297};	/*Property = CssTransitions*/
const Value V18515_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_298 = {S20, 0, &V18515_298_57208, &P18515_241};	/*Property = CssUI*/
const Value V18515_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_300 = {S21, 0, &V18515_300_57217, &P18515_298};	/*Property = DataSet*/
const Value V18515_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_301 = {S22, 0, &V18515_301_57220, &P18515_300};	/*Property = DataUrl*/
const Value V18515_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_302 = {S23, 0, &V18515_302_57223, &P18515_301};	/*Property = DeviceOrientation*/
const Value V18515_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_304 = {S24, 0, &V18515_304_57229, &P18515_302};	/*Property = FileReader*/
const Value V18515_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_305 = {S25, 0, &V18515_305_57232, &P18515_304};	/*Property = FileSaver*/
const Value V18515_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_306 = {S26, 0, &V18515_306_57238, &P18515_305};	/*Property = FileWriter*/
const Value V18515_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_307 = {S27, 0, &V18515_307_57241, &P18515_306};	/*Property = FormData*/
const Value V18515_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_309 = {S28, 0, &V18515_309_57247, &P18515_307};	/*Property = Fullscreen*/
const Value V18515_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_310 = {S29, 0, &V18515_310_57250, &P18515_309};	/*Property = GeoLocation*/
const Value V18515_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_312 = {S30, 0, &V18515_312_57256, &P18515_310};	/*Property = History*/
const Value V18515_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_314 = {S31, 0, &V18515_314_57262, &P18515_312};	/*Property = Html5*/
const Value V18515_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_313 = {S32, 0, &V18515_313_57259, &P18515_314};	/*Property = Html-Media-Capture*/
const Value V18515_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_315 = {S33, 0, &V18515_315_57265, &P18515_313};	/*Property = Iframe*/
const Value V18515_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_316 = {S34, 0, &V18515_316_57268, &P18515_315};	/*Property = IndexedDB*/
const Value V18515_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_317 = {S35, 0, &V18515_317_57271, &P18515_316};	/*Property = Json*/
const Value V18515_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18515_15 = {S37, 0, &V18515_15_29, &P18515_317};	/*Property = LayoutEngine*/
const Value V18515_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_344 = {S38, 0, &V18515_344_57274, &P18515_15};	/*Property = Masking*/
const Value V18515_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_320 = {S39, 0, &V18515_320_57292, &P18515_344};	/*Property = PostMessage*/
const Value V18515_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_321 = {S40, 0, &V18515_321_57295, &P18515_320};	/*Property = Progress*/
const Value V18515_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_322 = {S41, 0, &V18515_322_57298, &P18515_321};	/*Property = Prompts*/
const Value V18515_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_323 = {S42, 0, &V18515_323_57301, &P18515_322};	/*Property = Selector*/
const Value V18515_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_324 = {S43, 0, &V18515_324_57307, &P18515_323};	/*Property = Svg*/
const Value V18515_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_327 = {S44, 0, &V18515_327_57316, &P18515_324};	/*Property = TouchEvents*/
const Value V18515_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_328 = {S45, 0, &V18515_328_57319, &P18515_327};	/*Property = Track*/
const Value V18515_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_329 = {S46, 0, &V18515_329_57325, &P18515_328};	/*Property = Video*/
const Value V18515_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_330 = {S47, 0, &V18515_330_57328, &P18515_329};	/*Property = Viewport*/
const Value V18515_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_332 = {S48, 0, &V18515_332_57340, &P18515_330};	/*Property = WebWorkers*/
const Value V18515_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18515_333 = {S49, 0, &V18515_333_57343, &P18515_332};	/*Property = Xhr2*/
const Profile P18515 = {"18515", &P18515_333};	/*Profile ID - 18515*/
const Value V21023_1_1 = {S2, NULL};	/*Value = True*/
const Property P21023_1 = {S50, 0, &V21023_1_1, NULL};	/*Property = IsMobile*/
const Value V21023_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21023_5 = {S52, 0, &V21023_5_1222, &P21023_1};	/*Property = ScreenPixelsHeight*/
const Value V21023_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21023_4 = {S54, 0, &V21023_4_1224, &P21023_5};	/*Property = ScreenPixelsWidth*/
const Profile P21023 = {"21023", &P21023_4};	/*Profile ID - 21023*/
const Value V18427_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18427_279 = {S1, 0, &V18427_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18427_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18427_282 = {S3, 0, &V18427_282_57154, &P18427_279};	/*Property = BlobBuilder*/
const Value V18427_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18427_350 = {S4, 0, &V18427_350_57444, &P18427_282};	/*Property = Canvas*/
const Value V18427_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18427_283 = {S5, 0, &V18427_283_57159, &P18427_350};	/*Property = CssBackground*/
const Value V18427_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18427_284 = {S6, 0, &V18427_284_57163, &P18427_283};	/*Property = CssBorderImage*/
const Value V18427_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18427_285 = {S7, 0, &V18427_285_57166, &P18427_284};	/*Property = CssCanvas*/
const Value V18427_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18427_286 = {S8, 0, &V18427_286_57168, &P18427_285};	/*Property = CssColor*/
const Value V18427_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18427_237 = {S9, 0, &V18427_237_55786, &P18427_286};	/*Property = CssColumn*/
const Value V18427_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18427_287 = {S10, 0, &V18427_287_57172, &P18427_237};	/*Property = CssFlexbox*/
const Value V18427_288_57174 = {S2, NULL};	/*Value = True*/
const Property P18427_288 = {S11, 0, &V18427_288_57174, &P18427_287};	/*Property = CssFont*/
const Value V18427_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18427_289 = {S12, 0, &V18427_289_57759, &P18427_288};	/*Property = CssImages*/
const Value V18427_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18427_291 = {S13, 0, &V18427_291_57180, &P18427_289};	/*Property = CssMediaQueries*/
const Value V18427_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18427_292 = {S14, 0, &V18427_292_57183, &P18427_291};	/*Property = CssMinMax*/
const Value V18427_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18427_293 = {S15, 0, &V18427_293_57187, &P18427_292};	/*Property = CssOverflow*/
const Value V18427_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18427_294 = {S16, 0, &V18427_294_57189, &P18427_293};	/*Property = CssPosition*/
const Value V18427_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18427_295 = {S17, 0, &V18427_295_57192, &P18427_294};	/*Property = CssText*/
const Value V18427_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18427_297 = {S18, 0, &V18427_297_57195, &P18427_295};	/*Property = CssTransforms*/
const Value V18427_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18427_241 = {S19, 0, &V18427_241_55799, &P18427_297};	/*Property = CssTransitions*/
const Value V18427_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18427_298 = {S20, 0, &V18427_298_57206, &P18427_241};	/*Property = CssUI*/
const Value V18427_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18427_300 = {S21, 0, &V18427_300_57216, &P18427_298};	/*Property = DataSet*/
const Value V18427_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18427_301 = {S22, 0, &V18427_301_57218, &P18427_300};	/*Property = DataUrl*/
const Value V18427_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18427_302 = {S23, 0, &V18427_302_57222, &P18427_301};	/*Property = DeviceOrientation*/
const Value V18427_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18427_304 = {S24, 0, &V18427_304_57227, &P18427_302};	/*Property = FileReader*/
const Value V18427_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18427_305 = {S25, 0, &V18427_305_57231, &P18427_304};	/*Property = FileSaver*/
const Value V18427_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18427_306 = {S26, 0, &V18427_306_57237, &P18427_305};	/*Property = FileWriter*/
const Value V18427_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18427_307 = {S27, 0, &V18427_307_57239, &P18427_306};	/*Property = FormData*/
const Value V18427_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18427_309 = {S28, 0, &V18427_309_57246, &P18427_307};	/*Property = Fullscreen*/
const Value V18427_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18427_310 = {S29, 0, &V18427_310_57248, &P18427_309};	/*Property = GeoLocation*/
const Value V18427_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18427_312 = {S30, 0, &V18427_312_57254, &P18427_310};	/*Property = History*/
const Value V18427_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18427_314 = {S31, 0, &V18427_314_57261, &P18427_312};	/*Property = Html5*/
const Value V18427_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18427_313 = {S32, 0, &V18427_313_57258, &P18427_314};	/*Property = Html-Media-Capture*/
const Value V18427_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18427_315 = {S33, 0, &V18427_315_57264, &P18427_313};	/*Property = Iframe*/
const Value V18427_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18427_316 = {S34, 0, &V18427_316_57266, &P18427_315};	/*Property = IndexedDB*/
const Value V18427_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18427_317 = {S35, 0, &V18427_317_57269, &P18427_316};	/*Property = Json*/
const Value V18427_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18427_15 = {S37, 0, &V18427_15_28, &P18427_317};	/*Property = LayoutEngine*/
const Value V18427_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18427_344 = {S38, 0, &V18427_344_57272, &P18427_15};	/*Property = Masking*/
const Value V18427_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18427_320 = {S39, 0, &V18427_320_57290, &P18427_344};	/*Property = PostMessage*/
const Value V18427_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18427_321 = {S40, 0, &V18427_321_57293, &P18427_320};	/*Property = Progress*/
const Value V18427_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18427_322 = {S41, 0, &V18427_322_57296, &P18427_321};	/*Property = Prompts*/
const Value V18427_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18427_323 = {S42, 0, &V18427_323_57299, &P18427_322};	/*Property = Selector*/
const Value V18427_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18427_324 = {S43, 0, &V18427_324_57306, &P18427_323};	/*Property = Svg*/
const Value V18427_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18427_327 = {S44, 0, &V18427_327_57315, &P18427_324};	/*Property = TouchEvents*/
const Value V18427_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18427_328 = {S45, 0, &V18427_328_57318, &P18427_327};	/*Property = Track*/
const Value V18427_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18427_329 = {S46, 0, &V18427_329_57323, &P18427_328};	/*Property = Video*/
const Value V18427_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18427_330 = {S47, 0, &V18427_330_57326, &P18427_329};	/*Property = Viewport*/
const Value V18427_332_57338 = {S2, NULL};	/*Value = True*/
const Property P18427_332 = {S48, 0, &V18427_332_57338, &P18427_330};	/*Property = WebWorkers*/
const Value V18427_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18427_333 = {S49, 0, &V18427_333_57343, &P18427_332};	/*Property = Xhr2*/
const Profile P18427 = {"18427", &P18427_333};	/*Profile ID - 18427*/
const Value V18654_1_1 = {S2, NULL};	/*Value = True*/
const Property P18654_1 = {S50, 0, &V18654_1_1, NULL};	/*Property = IsMobile*/
const Value V18654_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18654_5 = {S52, 0, &V18654_5_1227, &P18654_1};	/*Property = ScreenPixelsHeight*/
const Value V18654_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18654_4 = {S54, 0, &V18654_4_1229, &P18654_5};	/*Property = ScreenPixelsWidth*/
const Profile P18654 = {"18654", &P18654_4};	/*Profile ID - 18654*/
const Value V21566_1_1 = {S2, NULL};	/*Value = True*/
const Property P21566_1 = {S50, 0, &V21566_1_1, NULL};	/*Property = IsMobile*/
const Value V21566_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21566_5 = {S52, 0, &V21566_5_1458, &P21566_1};	/*Property = ScreenPixelsHeight*/
const Value V21566_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21566_4 = {S54, 0, &V21566_4_37617, &P21566_5};	/*Property = ScreenPixelsWidth*/
const Profile P21566 = {"21566", &P21566_4};	/*Profile ID - 21566*/
const Value V23887_1_1 = {S2, NULL};	/*Value = True*/
const Property P23887_1 = {S50, 0, &V23887_1_1, NULL};	/*Property = IsMobile*/
const Value V23887_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23887_5 = {S52, 0, &V23887_5_1458, &P23887_1};	/*Property = ScreenPixelsHeight*/
const Value V23887_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23887_4 = {S54, 0, &V23887_4_1460, &P23887_5};	/*Property = ScreenPixelsWidth*/
const Profile P23887 = {"23887", &P23887_4};	/*Profile ID - 23887*/
const Value V20301_1_1 = {S2, NULL};	/*Value = True*/
const Property P20301_1 = {S50, 0, &V20301_1_1, NULL};	/*Property = IsMobile*/
const Value V20301_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20301_5 = {S52, 0, &V20301_5_1227, &P20301_1};	/*Property = ScreenPixelsHeight*/
const Value V20301_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20301_4 = {S54, 0, &V20301_4_1229, &P20301_5};	/*Property = ScreenPixelsWidth*/
const Profile P20301 = {"20301", &P20301_4};	/*Profile ID - 20301*/
const Value V21062_1_1 = {S2, NULL};	/*Value = True*/
const Property P21062_1 = {S50, 0, &V21062_1_1, NULL};	/*Property = IsMobile*/
const Value V21062_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21062_5 = {S52, 0, &V21062_5_1227, &P21062_1};	/*Property = ScreenPixelsHeight*/
const Value V21062_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21062_4 = {S54, 0, &V21062_4_1229, &P21062_5};	/*Property = ScreenPixelsWidth*/
const Profile P21062 = {"21062", &P21062_4};	/*Profile ID - 21062*/
const Value V18486_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18486_279 = {S1, 0, &V18486_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18486_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18486_282 = {S3, 0, &V18486_282_57154, &P18486_279};	/*Property = BlobBuilder*/
const Value V18486_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18486_350 = {S4, 0, &V18486_350_57444, &P18486_282};	/*Property = Canvas*/
const Value V18486_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18486_283 = {S5, 0, &V18486_283_57159, &P18486_350};	/*Property = CssBackground*/
const Value V18486_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18486_284 = {S6, 0, &V18486_284_57163, &P18486_283};	/*Property = CssBorderImage*/
const Value V18486_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18486_285 = {S7, 0, &V18486_285_57166, &P18486_284};	/*Property = CssCanvas*/
const Value V18486_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18486_286 = {S8, 0, &V18486_286_57168, &P18486_285};	/*Property = CssColor*/
const Value V18486_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18486_237 = {S9, 0, &V18486_237_55786, &P18486_286};	/*Property = CssColumn*/
const Value V18486_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18486_287 = {S10, 0, &V18486_287_57172, &P18486_237};	/*Property = CssFlexbox*/
const Value V18486_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18486_288 = {S11, 0, &V18486_288_57175, &P18486_287};	/*Property = CssFont*/
const Value V18486_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18486_289 = {S12, 0, &V18486_289_57759, &P18486_288};	/*Property = CssImages*/
const Value V18486_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18486_291 = {S13, 0, &V18486_291_57180, &P18486_289};	/*Property = CssMediaQueries*/
const Value V18486_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18486_292 = {S14, 0, &V18486_292_57183, &P18486_291};	/*Property = CssMinMax*/
const Value V18486_293_57186 = {S2, NULL};	/*Value = True*/
const Property P18486_293 = {S15, 0, &V18486_293_57186, &P18486_292};	/*Property = CssOverflow*/
const Value V18486_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18486_294 = {S16, 0, &V18486_294_57189, &P18486_293};	/*Property = CssPosition*/
const Value V18486_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18486_295 = {S17, 0, &V18486_295_57192, &P18486_294};	/*Property = CssText*/
const Value V18486_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18486_297 = {S18, 0, &V18486_297_57195, &P18486_295};	/*Property = CssTransforms*/
const Value V18486_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18486_241 = {S19, 0, &V18486_241_55799, &P18486_297};	/*Property = CssTransitions*/
const Value V18486_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18486_298 = {S20, 0, &V18486_298_57206, &P18486_241};	/*Property = CssUI*/
const Value V18486_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18486_300 = {S21, 0, &V18486_300_57216, &P18486_298};	/*Property = DataSet*/
const Value V18486_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18486_301 = {S22, 0, &V18486_301_57218, &P18486_300};	/*Property = DataUrl*/
const Value V18486_302_57221 = {S2, NULL};	/*Value = True*/
const Property P18486_302 = {S23, 0, &V18486_302_57221, &P18486_301};	/*Property = DeviceOrientation*/
const Value V18486_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18486_304 = {S24, 0, &V18486_304_57228, &P18486_302};	/*Property = FileReader*/
const Value V18486_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18486_305 = {S25, 0, &V18486_305_57231, &P18486_304};	/*Property = FileSaver*/
const Value V18486_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18486_306 = {S26, 0, &V18486_306_57237, &P18486_305};	/*Property = FileWriter*/
const Value V18486_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18486_307 = {S27, 0, &V18486_307_57239, &P18486_306};	/*Property = FormData*/
const Value V18486_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18486_309 = {S28, 0, &V18486_309_57246, &P18486_307};	/*Property = Fullscreen*/
const Value V18486_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18486_310 = {S29, 0, &V18486_310_57248, &P18486_309};	/*Property = GeoLocation*/
const Value V18486_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18486_312 = {S30, 0, &V18486_312_57255, &P18486_310};	/*Property = History*/
const Value V18486_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18486_314 = {S31, 0, &V18486_314_57261, &P18486_312};	/*Property = Html5*/
const Value V18486_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18486_313 = {S32, 0, &V18486_313_57258, &P18486_314};	/*Property = Html-Media-Capture*/
const Value V18486_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18486_315 = {S33, 0, &V18486_315_57264, &P18486_313};	/*Property = Iframe*/
const Value V18486_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18486_316 = {S34, 0, &V18486_316_57267, &P18486_315};	/*Property = IndexedDB*/
const Value V18486_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18486_317 = {S35, 0, &V18486_317_57269, &P18486_316};	/*Property = Json*/
const Value V18486_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18486_15 = {S37, 0, &V18486_15_28, &P18486_317};	/*Property = LayoutEngine*/
const Value V18486_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18486_344 = {S38, 0, &V18486_344_57272, &P18486_15};	/*Property = Masking*/
const Value V18486_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18486_320 = {S39, 0, &V18486_320_57290, &P18486_344};	/*Property = PostMessage*/
const Value V18486_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18486_321 = {S40, 0, &V18486_321_57293, &P18486_320};	/*Property = Progress*/
const Value V18486_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18486_322 = {S41, 0, &V18486_322_57296, &P18486_321};	/*Property = Prompts*/
const Value V18486_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18486_323 = {S42, 0, &V18486_323_57299, &P18486_322};	/*Property = Selector*/
const Value V18486_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18486_324 = {S43, 0, &V18486_324_57306, &P18486_323};	/*Property = Svg*/
const Value V18486_327_57314 = {S2, NULL};	/*Value = True*/
const Property P18486_327 = {S44, 0, &V18486_327_57314, &P18486_324};	/*Property = TouchEvents*/
const Value V18486_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18486_328 = {S45, 0, &V18486_328_57318, &P18486_327};	/*Property = Track*/
const Value V18486_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18486_329 = {S46, 0, &V18486_329_57323, &P18486_328};	/*Property = Video*/
const Value V18486_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18486_330 = {S47, 0, &V18486_330_57326, &P18486_329};	/*Property = Viewport*/
const Value V18486_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18486_332 = {S48, 0, &V18486_332_57339, &P18486_330};	/*Property = WebWorkers*/
const Value V18486_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18486_333 = {S49, 0, &V18486_333_57342, &P18486_332};	/*Property = Xhr2*/
const Profile P18486 = {"18486", &P18486_333};	/*Profile ID - 18486*/
const Value V20185_1_1 = {S2, NULL};	/*Value = True*/
const Property P20185_1 = {S50, 0, &V20185_1_1, NULL};	/*Property = IsMobile*/
const Value V20185_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20185_5 = {S52, 0, &V20185_5_1227, &P20185_1};	/*Property = ScreenPixelsHeight*/
const Value V20185_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20185_4 = {S54, 0, &V20185_4_1229, &P20185_5};	/*Property = ScreenPixelsWidth*/
const Profile P20185 = {"20185", &P20185_4};	/*Profile ID - 20185*/
const Value V20583_1_1 = {S2, NULL};	/*Value = True*/
const Property P20583_1 = {S50, 0, &V20583_1_1, NULL};	/*Property = IsMobile*/
const Value V20583_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20583_5 = {S52, 0, &V20583_5_1227, &P20583_1};	/*Property = ScreenPixelsHeight*/
const Value V20583_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20583_4 = {S54, 0, &V20583_4_1229, &P20583_5};	/*Property = ScreenPixelsWidth*/
const Profile P20583 = {"20583", &P20583_4};	/*Profile ID - 20583*/
const Profile P18443 = {"18443", NULL};	/*Profile ID - 18443*/
const Value V18725_1_1 = {S2, NULL};	/*Value = True*/
const Property P18725_1 = {S50, 0, &V18725_1_1, NULL};	/*Property = IsMobile*/
const Value V18725_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18725_5 = {S52, 0, &V18725_5_1227, &P18725_1};	/*Property = ScreenPixelsHeight*/
const Value V18725_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P18725_4 = {S54, 0, &V18725_4_1460, &P18725_5};	/*Property = ScreenPixelsWidth*/
const Profile P18725 = {"18725", &P18725_4};	/*Profile ID - 18725*/
const Value V20265_1_1 = {S2, NULL};	/*Value = True*/
const Property P20265_1 = {S50, 0, &V20265_1_1, NULL};	/*Property = IsMobile*/
const Value V20265_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20265_5 = {S52, 0, &V20265_5_1222, &P20265_1};	/*Property = ScreenPixelsHeight*/
const Value V20265_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20265_4 = {S54, 0, &V20265_4_1224, &P20265_5};	/*Property = ScreenPixelsWidth*/
const Profile P20265 = {"20265", &P20265_4};	/*Profile ID - 20265*/
const Value V18633_1_1 = {S2, NULL};	/*Value = True*/
const Property P18633_1 = {S50, 0, &V18633_1_1, NULL};	/*Property = IsMobile*/
const Value V18633_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18633_5 = {S52, 0, &V18633_5_1222, &P18633_1};	/*Property = ScreenPixelsHeight*/
const Value V18633_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18633_4 = {S54, 0, &V18633_4_1224, &P18633_5};	/*Property = ScreenPixelsWidth*/
const Profile P18633 = {"18633", &P18633_4};	/*Profile ID - 18633*/
const Value V23921_1_1 = {S2, NULL};	/*Value = True*/
const Property P23921_1 = {S50, 0, &V23921_1_1, NULL};	/*Property = IsMobile*/
const Value V23921_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23921_5 = {S52, 0, &V23921_5_37, &P23921_1};	/*Property = ScreenPixelsHeight*/
const Value V23921_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23921_4 = {S54, 0, &V23921_4_36, &P23921_5};	/*Property = ScreenPixelsWidth*/
const Profile P23921 = {"23921", &P23921_4};	/*Profile ID - 23921*/
const Value V21014_1_1 = {S2, NULL};	/*Value = True*/
const Property P21014_1 = {S50, 0, &V21014_1_1, NULL};	/*Property = IsMobile*/
const Value V21014_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21014_5 = {S52, 0, &V21014_5_1277, &P21014_1};	/*Property = ScreenPixelsHeight*/
const Value V21014_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21014_4 = {S54, 0, &V21014_4_1279, &P21014_5};	/*Property = ScreenPixelsWidth*/
const Profile P21014 = {"21014", &P21014_4};	/*Profile ID - 21014*/
const Value V21072_1_1 = {S2, NULL};	/*Value = True*/
const Property P21072_1 = {S50, 0, &V21072_1_1, NULL};	/*Property = IsMobile*/
const Value V21072_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21072_5 = {S52, 0, &V21072_5_1458, &P21072_1};	/*Property = ScreenPixelsHeight*/
const Value V21072_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21072_4 = {S54, 0, &V21072_4_37617, &P21072_5};	/*Property = ScreenPixelsWidth*/
const Profile P21072 = {"21072", &P21072_4};	/*Profile ID - 21072*/
const Value V18658_1_1 = {S2, NULL};	/*Value = True*/
const Property P18658_1 = {S50, 0, &V18658_1_1, NULL};	/*Property = IsMobile*/
const Value V18658_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18658_5 = {S52, 0, &V18658_5_9981, &P18658_1};	/*Property = ScreenPixelsHeight*/
const Value V18658_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18658_4 = {S54, 0, &V18658_4_39098, &P18658_5};	/*Property = ScreenPixelsWidth*/
const Profile P18658 = {"18658", &P18658_4};	/*Profile ID - 18658*/
const Value V21912_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21912_279 = {S1, 0, &V21912_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21912_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21912_282 = {S3, 0, &V21912_282_57154, &P21912_279};	/*Property = BlobBuilder*/
const Value V21912_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21912_350 = {S4, 0, &V21912_350_57444, &P21912_282};	/*Property = Canvas*/
const Value V21912_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21912_283 = {S5, 0, &V21912_283_57159, &P21912_350};	/*Property = CssBackground*/
const Value V21912_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21912_284 = {S6, 0, &V21912_284_57163, &P21912_283};	/*Property = CssBorderImage*/
const Value V21912_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21912_285 = {S7, 0, &V21912_285_57166, &P21912_284};	/*Property = CssCanvas*/
const Value V21912_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21912_286 = {S8, 0, &V21912_286_57168, &P21912_285};	/*Property = CssColor*/
const Value V21912_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21912_237 = {S9, 0, &V21912_237_55786, &P21912_286};	/*Property = CssColumn*/
const Value V21912_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21912_287 = {S10, 0, &V21912_287_57172, &P21912_237};	/*Property = CssFlexbox*/
const Value V21912_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21912_288 = {S11, 0, &V21912_288_57176, &P21912_287};	/*Property = CssFont*/
const Value V21912_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21912_289 = {S12, 0, &V21912_289_57759, &P21912_288};	/*Property = CssImages*/
const Value V21912_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21912_291 = {S13, 0, &V21912_291_57180, &P21912_289};	/*Property = CssMediaQueries*/
const Value V21912_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21912_292 = {S14, 0, &V21912_292_57183, &P21912_291};	/*Property = CssMinMax*/
const Value V21912_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21912_293 = {S15, 0, &V21912_293_57187, &P21912_292};	/*Property = CssOverflow*/
const Value V21912_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21912_294 = {S16, 0, &V21912_294_57189, &P21912_293};	/*Property = CssPosition*/
const Value V21912_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21912_295 = {S17, 0, &V21912_295_57192, &P21912_294};	/*Property = CssText*/
const Value V21912_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21912_297 = {S18, 0, &V21912_297_57195, &P21912_295};	/*Property = CssTransforms*/
const Value V21912_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21912_241 = {S19, 0, &V21912_241_55799, &P21912_297};	/*Property = CssTransitions*/
const Value V21912_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21912_298 = {S20, 0, &V21912_298_57206, &P21912_241};	/*Property = CssUI*/
const Value V21912_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21912_300 = {S21, 0, &V21912_300_57216, &P21912_298};	/*Property = DataSet*/
const Value V21912_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21912_301 = {S22, 0, &V21912_301_57218, &P21912_300};	/*Property = DataUrl*/
const Value V21912_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21912_302 = {S23, 0, &V21912_302_57222, &P21912_301};	/*Property = DeviceOrientation*/
const Value V21912_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21912_304 = {S24, 0, &V21912_304_57227, &P21912_302};	/*Property = FileReader*/
const Value V21912_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21912_305 = {S25, 0, &V21912_305_57231, &P21912_304};	/*Property = FileSaver*/
const Value V21912_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21912_306 = {S26, 0, &V21912_306_57237, &P21912_305};	/*Property = FileWriter*/
const Value V21912_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21912_307 = {S27, 0, &V21912_307_57239, &P21912_306};	/*Property = FormData*/
const Value V21912_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21912_309 = {S28, 0, &V21912_309_57246, &P21912_307};	/*Property = Fullscreen*/
const Value V21912_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21912_310 = {S29, 0, &V21912_310_57248, &P21912_309};	/*Property = GeoLocation*/
const Value V21912_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21912_312 = {S30, 0, &V21912_312_57254, &P21912_310};	/*Property = History*/
const Value V21912_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21912_314 = {S31, 0, &V21912_314_57261, &P21912_312};	/*Property = Html5*/
const Value V21912_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21912_313 = {S32, 0, &V21912_313_57258, &P21912_314};	/*Property = Html-Media-Capture*/
const Value V21912_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21912_315 = {S33, 0, &V21912_315_57264, &P21912_313};	/*Property = Iframe*/
const Value V21912_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21912_316 = {S34, 0, &V21912_316_57266, &P21912_315};	/*Property = IndexedDB*/
const Value V21912_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21912_317 = {S35, 0, &V21912_317_57269, &P21912_316};	/*Property = Json*/
const Value V21912_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21912_15 = {S37, 0, &V21912_15_28, &P21912_317};	/*Property = LayoutEngine*/
const Value V21912_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21912_344 = {S38, 0, &V21912_344_57272, &P21912_15};	/*Property = Masking*/
const Value V21912_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21912_320 = {S39, 0, &V21912_320_57290, &P21912_344};	/*Property = PostMessage*/
const Value V21912_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21912_321 = {S40, 0, &V21912_321_57293, &P21912_320};	/*Property = Progress*/
const Value V21912_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21912_322 = {S41, 0, &V21912_322_57296, &P21912_321};	/*Property = Prompts*/
const Value V21912_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21912_323 = {S42, 0, &V21912_323_57299, &P21912_322};	/*Property = Selector*/
const Value V21912_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21912_324 = {S43, 0, &V21912_324_57306, &P21912_323};	/*Property = Svg*/
const Value V21912_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21912_327 = {S44, 0, &V21912_327_57315, &P21912_324};	/*Property = TouchEvents*/
const Value V21912_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21912_328 = {S45, 0, &V21912_328_57318, &P21912_327};	/*Property = Track*/
const Value V21912_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21912_329 = {S46, 0, &V21912_329_57323, &P21912_328};	/*Property = Video*/
const Value V21912_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21912_330 = {S47, 0, &V21912_330_57326, &P21912_329};	/*Property = Viewport*/
const Value V21912_332_57338 = {S2, NULL};	/*Value = True*/
const Property P21912_332 = {S48, 0, &V21912_332_57338, &P21912_330};	/*Property = WebWorkers*/
const Value V21912_333_57341 = {S2, NULL};	/*Value = True*/
const Property P21912_333 = {S49, 0, &V21912_333_57341, &P21912_332};	/*Property = Xhr2*/
const Profile P21912 = {"21912", &P21912_333};	/*Profile ID - 21912*/
const Value V21457_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21457_279 = {S1, 0, &V21457_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21457_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21457_282 = {S3, 0, &V21457_282_57154, &P21457_279};	/*Property = BlobBuilder*/
const Value V21457_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21457_350 = {S4, 0, &V21457_350_57444, &P21457_282};	/*Property = Canvas*/
const Value V21457_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21457_283 = {S5, 0, &V21457_283_57159, &P21457_350};	/*Property = CssBackground*/
const Value V21457_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21457_284 = {S6, 0, &V21457_284_57163, &P21457_283};	/*Property = CssBorderImage*/
const Value V21457_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21457_285 = {S7, 0, &V21457_285_57166, &P21457_284};	/*Property = CssCanvas*/
const Value V21457_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21457_286 = {S8, 0, &V21457_286_57168, &P21457_285};	/*Property = CssColor*/
const Value V21457_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21457_237 = {S9, 0, &V21457_237_55786, &P21457_286};	/*Property = CssColumn*/
const Value V21457_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21457_287 = {S10, 0, &V21457_287_57172, &P21457_237};	/*Property = CssFlexbox*/
const Value V21457_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21457_288 = {S11, 0, &V21457_288_57175, &P21457_287};	/*Property = CssFont*/
const Value V21457_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21457_289 = {S12, 0, &V21457_289_57759, &P21457_288};	/*Property = CssImages*/
const Value V21457_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21457_291 = {S13, 0, &V21457_291_57181, &P21457_289};	/*Property = CssMediaQueries*/
const Value V21457_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21457_292 = {S14, 0, &V21457_292_57183, &P21457_291};	/*Property = CssMinMax*/
const Value V21457_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21457_293 = {S15, 0, &V21457_293_57187, &P21457_292};	/*Property = CssOverflow*/
const Value V21457_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21457_294 = {S16, 0, &V21457_294_57190, &P21457_293};	/*Property = CssPosition*/
const Value V21457_295_57193 = {S0, NULL};	/*Value = False*/
const Property P21457_295 = {S17, 0, &V21457_295_57193, &P21457_294};	/*Property = CssText*/
const Value V21457_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21457_297 = {S18, 0, &V21457_297_57195, &P21457_295};	/*Property = CssTransforms*/
const Value V21457_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21457_241 = {S19, 0, &V21457_241_55799, &P21457_297};	/*Property = CssTransitions*/
const Value V21457_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21457_298 = {S20, 0, &V21457_298_57206, &P21457_241};	/*Property = CssUI*/
const Value V21457_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21457_300 = {S21, 0, &V21457_300_57216, &P21457_298};	/*Property = DataSet*/
const Value V21457_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21457_301 = {S22, 0, &V21457_301_57218, &P21457_300};	/*Property = DataUrl*/
const Value V21457_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21457_302 = {S23, 0, &V21457_302_57222, &P21457_301};	/*Property = DeviceOrientation*/
const Value V21457_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21457_304 = {S24, 0, &V21457_304_57228, &P21457_302};	/*Property = FileReader*/
const Value V21457_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21457_305 = {S25, 0, &V21457_305_57231, &P21457_304};	/*Property = FileSaver*/
const Value V21457_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21457_306 = {S26, 0, &V21457_306_57237, &P21457_305};	/*Property = FileWriter*/
const Value V21457_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21457_307 = {S27, 0, &V21457_307_57240, &P21457_306};	/*Property = FormData*/
const Value V21457_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21457_309 = {S28, 0, &V21457_309_57246, &P21457_307};	/*Property = Fullscreen*/
const Value V21457_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21457_310 = {S29, 0, &V21457_310_57248, &P21457_309};	/*Property = GeoLocation*/
const Value V21457_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21457_312 = {S30, 0, &V21457_312_57255, &P21457_310};	/*Property = History*/
const Value V21457_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21457_314 = {S31, 0, &V21457_314_57261, &P21457_312};	/*Property = Html5*/
const Value V21457_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21457_313 = {S32, 0, &V21457_313_57258, &P21457_314};	/*Property = Html-Media-Capture*/
const Value V21457_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21457_315 = {S33, 0, &V21457_315_57264, &P21457_313};	/*Property = Iframe*/
const Value V21457_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21457_316 = {S34, 0, &V21457_316_57267, &P21457_315};	/*Property = IndexedDB*/
const Value V21457_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21457_317 = {S35, 0, &V21457_317_57269, &P21457_316};	/*Property = Json*/
const Value V21457_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P21457_15 = {S37, 0, &V21457_15_30, &P21457_317};	/*Property = LayoutEngine*/
const Value V21457_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21457_344 = {S38, 0, &V21457_344_57272, &P21457_15};	/*Property = Masking*/
const Value V21457_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21457_320 = {S39, 0, &V21457_320_57290, &P21457_344};	/*Property = PostMessage*/
const Value V21457_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21457_321 = {S40, 0, &V21457_321_57293, &P21457_320};	/*Property = Progress*/
const Value V21457_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21457_322 = {S41, 0, &V21457_322_57296, &P21457_321};	/*Property = Prompts*/
const Value V21457_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21457_323 = {S42, 0, &V21457_323_57299, &P21457_322};	/*Property = Selector*/
const Value V21457_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21457_324 = {S43, 0, &V21457_324_57306, &P21457_323};	/*Property = Svg*/
const Value V21457_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21457_327 = {S44, 0, &V21457_327_57315, &P21457_324};	/*Property = TouchEvents*/
const Value V21457_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21457_328 = {S45, 0, &V21457_328_57318, &P21457_327};	/*Property = Track*/
const Value V21457_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21457_329 = {S46, 0, &V21457_329_57323, &P21457_328};	/*Property = Video*/
const Value V21457_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21457_330 = {S47, 0, &V21457_330_57326, &P21457_329};	/*Property = Viewport*/
const Value V21457_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21457_332 = {S48, 0, &V21457_332_57339, &P21457_330};	/*Property = WebWorkers*/
const Value V21457_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21457_333 = {S49, 0, &V21457_333_57342, &P21457_332};	/*Property = Xhr2*/
const Profile P21457 = {"21457", &P21457_333};	/*Profile ID - 21457*/
const Value V21245_1_1 = {S2, NULL};	/*Value = True*/
const Property P21245_1 = {S50, 0, &V21245_1_1, NULL};	/*Property = IsMobile*/
const Value V21245_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21245_5 = {S52, 0, &V21245_5_1277, &P21245_1};	/*Property = ScreenPixelsHeight*/
const Value V21245_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21245_4 = {S54, 0, &V21245_4_2336, &P21245_5};	/*Property = ScreenPixelsWidth*/
const Profile P21245 = {"21245", &P21245_4};	/*Profile ID - 21245*/
const Value V18655_1_1 = {S2, NULL};	/*Value = True*/
const Property P18655_1 = {S50, 0, &V18655_1_1, NULL};	/*Property = IsMobile*/
const Value V18655_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18655_5 = {S52, 0, &V18655_5_9981, &P18655_1};	/*Property = ScreenPixelsHeight*/
const Value V18655_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18655_4 = {S54, 0, &V18655_4_39098, &P18655_5};	/*Property = ScreenPixelsWidth*/
const Profile P18655 = {"18655", &P18655_4};	/*Profile ID - 18655*/
const Value V18519_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_279 = {S1, 0, &V18519_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18519_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_282 = {S3, 0, &V18519_282_57155, &P18519_279};	/*Property = BlobBuilder*/
const Value V18519_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_350 = {S4, 0, &V18519_350_57446, &P18519_282};	/*Property = Canvas*/
const Value V18519_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_283 = {S5, 0, &V18519_283_57161, &P18519_350};	/*Property = CssBackground*/
const Value V18519_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_284 = {S6, 0, &V18519_284_57164, &P18519_283};	/*Property = CssBorderImage*/
const Value V18519_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_285 = {S7, 0, &V18519_285_57167, &P18519_284};	/*Property = CssCanvas*/
const Value V18519_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_286 = {S8, 0, &V18519_286_57170, &P18519_285};	/*Property = CssColor*/
const Value V18519_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_237 = {S9, 0, &V18519_237_55786, &P18519_286};	/*Property = CssColumn*/
const Value V18519_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_287 = {S10, 0, &V18519_287_57173, &P18519_237};	/*Property = CssFlexbox*/
const Value V18519_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_288 = {S11, 0, &V18519_288_57176, &P18519_287};	/*Property = CssFont*/
const Value V18519_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_289 = {S12, 0, &V18519_289_86899, &P18519_288};	/*Property = CssImages*/
const Value V18519_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_291 = {S13, 0, &V18519_291_57182, &P18519_289};	/*Property = CssMediaQueries*/
const Value V18519_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_292 = {S14, 0, &V18519_292_57185, &P18519_291};	/*Property = CssMinMax*/
const Value V18519_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_293 = {S15, 0, &V18519_293_57188, &P18519_292};	/*Property = CssOverflow*/
const Value V18519_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_294 = {S16, 0, &V18519_294_57191, &P18519_293};	/*Property = CssPosition*/
const Value V18519_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_295 = {S17, 0, &V18519_295_57194, &P18519_294};	/*Property = CssText*/
const Value V18519_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_297 = {S18, 0, &V18519_297_57197, &P18519_295};	/*Property = CssTransforms*/
const Value V18519_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_241 = {S19, 0, &V18519_241_55801, &P18519_297};	/*Property = CssTransitions*/
const Value V18519_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_298 = {S20, 0, &V18519_298_57208, &P18519_241};	/*Property = CssUI*/
const Value V18519_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_300 = {S21, 0, &V18519_300_57217, &P18519_298};	/*Property = DataSet*/
const Value V18519_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_301 = {S22, 0, &V18519_301_57220, &P18519_300};	/*Property = DataUrl*/
const Value V18519_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_302 = {S23, 0, &V18519_302_57223, &P18519_301};	/*Property = DeviceOrientation*/
const Value V18519_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_304 = {S24, 0, &V18519_304_57229, &P18519_302};	/*Property = FileReader*/
const Value V18519_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_305 = {S25, 0, &V18519_305_57232, &P18519_304};	/*Property = FileSaver*/
const Value V18519_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_306 = {S26, 0, &V18519_306_57238, &P18519_305};	/*Property = FileWriter*/
const Value V18519_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_307 = {S27, 0, &V18519_307_57241, &P18519_306};	/*Property = FormData*/
const Value V18519_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_309 = {S28, 0, &V18519_309_57247, &P18519_307};	/*Property = Fullscreen*/
const Value V18519_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_310 = {S29, 0, &V18519_310_57250, &P18519_309};	/*Property = GeoLocation*/
const Value V18519_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_312 = {S30, 0, &V18519_312_57256, &P18519_310};	/*Property = History*/
const Value V18519_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_314 = {S31, 0, &V18519_314_57262, &P18519_312};	/*Property = Html5*/
const Value V18519_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_313 = {S32, 0, &V18519_313_57259, &P18519_314};	/*Property = Html-Media-Capture*/
const Value V18519_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_315 = {S33, 0, &V18519_315_57265, &P18519_313};	/*Property = Iframe*/
const Value V18519_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_316 = {S34, 0, &V18519_316_57268, &P18519_315};	/*Property = IndexedDB*/
const Value V18519_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_317 = {S35, 0, &V18519_317_57271, &P18519_316};	/*Property = Json*/
const Value V18519_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18519_15 = {S37, 0, &V18519_15_28, &P18519_317};	/*Property = LayoutEngine*/
const Value V18519_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_344 = {S38, 0, &V18519_344_57274, &P18519_15};	/*Property = Masking*/
const Value V18519_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_320 = {S39, 0, &V18519_320_57292, &P18519_344};	/*Property = PostMessage*/
const Value V18519_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_321 = {S40, 0, &V18519_321_57295, &P18519_320};	/*Property = Progress*/
const Value V18519_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_322 = {S41, 0, &V18519_322_57298, &P18519_321};	/*Property = Prompts*/
const Value V18519_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_323 = {S42, 0, &V18519_323_57301, &P18519_322};	/*Property = Selector*/
const Value V18519_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_324 = {S43, 0, &V18519_324_57307, &P18519_323};	/*Property = Svg*/
const Value V18519_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_327 = {S44, 0, &V18519_327_57316, &P18519_324};	/*Property = TouchEvents*/
const Value V18519_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_328 = {S45, 0, &V18519_328_57319, &P18519_327};	/*Property = Track*/
const Value V18519_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_329 = {S46, 0, &V18519_329_57325, &P18519_328};	/*Property = Video*/
const Value V18519_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_330 = {S47, 0, &V18519_330_57328, &P18519_329};	/*Property = Viewport*/
const Value V18519_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_332 = {S48, 0, &V18519_332_57340, &P18519_330};	/*Property = WebWorkers*/
const Value V18519_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18519_333 = {S49, 0, &V18519_333_57343, &P18519_332};	/*Property = Xhr2*/
const Profile P18519 = {"18519", &P18519_333};	/*Profile ID - 18519*/
const Value V18528_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_279 = {S1, 0, &V18528_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18528_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_282 = {S3, 0, &V18528_282_57155, &P18528_279};	/*Property = BlobBuilder*/
const Value V18528_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_350 = {S4, 0, &V18528_350_57446, &P18528_282};	/*Property = Canvas*/
const Value V18528_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_283 = {S5, 0, &V18528_283_57161, &P18528_350};	/*Property = CssBackground*/
const Value V18528_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_284 = {S6, 0, &V18528_284_57164, &P18528_283};	/*Property = CssBorderImage*/
const Value V18528_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_285 = {S7, 0, &V18528_285_57167, &P18528_284};	/*Property = CssCanvas*/
const Value V18528_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_286 = {S8, 0, &V18528_286_57170, &P18528_285};	/*Property = CssColor*/
const Value V18528_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_237 = {S9, 0, &V18528_237_55786, &P18528_286};	/*Property = CssColumn*/
const Value V18528_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_287 = {S10, 0, &V18528_287_57173, &P18528_237};	/*Property = CssFlexbox*/
const Value V18528_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_288 = {S11, 0, &V18528_288_57176, &P18528_287};	/*Property = CssFont*/
const Value V18528_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_289 = {S12, 0, &V18528_289_86899, &P18528_288};	/*Property = CssImages*/
const Value V18528_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_291 = {S13, 0, &V18528_291_57182, &P18528_289};	/*Property = CssMediaQueries*/
const Value V18528_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_292 = {S14, 0, &V18528_292_57185, &P18528_291};	/*Property = CssMinMax*/
const Value V18528_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_293 = {S15, 0, &V18528_293_57188, &P18528_292};	/*Property = CssOverflow*/
const Value V18528_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_294 = {S16, 0, &V18528_294_57191, &P18528_293};	/*Property = CssPosition*/
const Value V18528_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_295 = {S17, 0, &V18528_295_57194, &P18528_294};	/*Property = CssText*/
const Value V18528_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_297 = {S18, 0, &V18528_297_57197, &P18528_295};	/*Property = CssTransforms*/
const Value V18528_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_241 = {S19, 0, &V18528_241_55801, &P18528_297};	/*Property = CssTransitions*/
const Value V18528_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_298 = {S20, 0, &V18528_298_57208, &P18528_241};	/*Property = CssUI*/
const Value V18528_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_300 = {S21, 0, &V18528_300_57217, &P18528_298};	/*Property = DataSet*/
const Value V18528_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_301 = {S22, 0, &V18528_301_57220, &P18528_300};	/*Property = DataUrl*/
const Value V18528_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_302 = {S23, 0, &V18528_302_57223, &P18528_301};	/*Property = DeviceOrientation*/
const Value V18528_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_304 = {S24, 0, &V18528_304_57229, &P18528_302};	/*Property = FileReader*/
const Value V18528_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_305 = {S25, 0, &V18528_305_57232, &P18528_304};	/*Property = FileSaver*/
const Value V18528_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_306 = {S26, 0, &V18528_306_57238, &P18528_305};	/*Property = FileWriter*/
const Value V18528_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_307 = {S27, 0, &V18528_307_57241, &P18528_306};	/*Property = FormData*/
const Value V18528_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_309 = {S28, 0, &V18528_309_57247, &P18528_307};	/*Property = Fullscreen*/
const Value V18528_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_310 = {S29, 0, &V18528_310_57250, &P18528_309};	/*Property = GeoLocation*/
const Value V18528_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_312 = {S30, 0, &V18528_312_57256, &P18528_310};	/*Property = History*/
const Value V18528_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_314 = {S31, 0, &V18528_314_57262, &P18528_312};	/*Property = Html5*/
const Value V18528_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_313 = {S32, 0, &V18528_313_57259, &P18528_314};	/*Property = Html-Media-Capture*/
const Value V18528_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_315 = {S33, 0, &V18528_315_57265, &P18528_313};	/*Property = Iframe*/
const Value V18528_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_316 = {S34, 0, &V18528_316_57268, &P18528_315};	/*Property = IndexedDB*/
const Value V18528_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_317 = {S35, 0, &V18528_317_57271, &P18528_316};	/*Property = Json*/
const Value V18528_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P18528_15 = {S37, 0, &V18528_15_30, &P18528_317};	/*Property = LayoutEngine*/
const Value V18528_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_344 = {S38, 0, &V18528_344_57274, &P18528_15};	/*Property = Masking*/
const Value V18528_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_320 = {S39, 0, &V18528_320_57292, &P18528_344};	/*Property = PostMessage*/
const Value V18528_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_321 = {S40, 0, &V18528_321_57295, &P18528_320};	/*Property = Progress*/
const Value V18528_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_322 = {S41, 0, &V18528_322_57298, &P18528_321};	/*Property = Prompts*/
const Value V18528_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_323 = {S42, 0, &V18528_323_57301, &P18528_322};	/*Property = Selector*/
const Value V18528_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_324 = {S43, 0, &V18528_324_57307, &P18528_323};	/*Property = Svg*/
const Value V18528_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_327 = {S44, 0, &V18528_327_57316, &P18528_324};	/*Property = TouchEvents*/
const Value V18528_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_328 = {S45, 0, &V18528_328_57319, &P18528_327};	/*Property = Track*/
const Value V18528_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_329 = {S46, 0, &V18528_329_57325, &P18528_328};	/*Property = Video*/
const Value V18528_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_330 = {S47, 0, &V18528_330_57328, &P18528_329};	/*Property = Viewport*/
const Value V18528_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_332 = {S48, 0, &V18528_332_57340, &P18528_330};	/*Property = WebWorkers*/
const Value V18528_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18528_333 = {S49, 0, &V18528_333_57343, &P18528_332};	/*Property = Xhr2*/
const Profile P18528 = {"18528", &P18528_333};	/*Profile ID - 18528*/
const Value V18505_1_1 = {S2, NULL};	/*Value = True*/
const Property P18505_1 = {S50, 0, &V18505_1_1, NULL};	/*Property = IsMobile*/
const Value V18505_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18505_5 = {S52, 0, &V18505_5_9981, &P18505_1};	/*Property = ScreenPixelsHeight*/
const Value V18505_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18505_4 = {S54, 0, &V18505_4_39098, &P18505_5};	/*Property = ScreenPixelsWidth*/
const Profile P18505 = {"18505", &P18505_4};	/*Profile ID - 18505*/
const Value V24435_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_279 = {S1, 0, &V24435_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24435_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_282 = {S3, 0, &V24435_282_57155, &P24435_279};	/*Property = BlobBuilder*/
const Value V24435_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_350 = {S4, 0, &V24435_350_57446, &P24435_282};	/*Property = Canvas*/
const Value V24435_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_283 = {S5, 0, &V24435_283_57161, &P24435_350};	/*Property = CssBackground*/
const Value V24435_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_284 = {S6, 0, &V24435_284_57164, &P24435_283};	/*Property = CssBorderImage*/
const Value V24435_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_285 = {S7, 0, &V24435_285_57167, &P24435_284};	/*Property = CssCanvas*/
const Value V24435_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_286 = {S8, 0, &V24435_286_57170, &P24435_285};	/*Property = CssColor*/
const Value V24435_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_237 = {S9, 0, &V24435_237_55786, &P24435_286};	/*Property = CssColumn*/
const Value V24435_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_287 = {S10, 0, &V24435_287_57173, &P24435_237};	/*Property = CssFlexbox*/
const Value V24435_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_288 = {S11, 0, &V24435_288_57176, &P24435_287};	/*Property = CssFont*/
const Value V24435_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_289 = {S12, 0, &V24435_289_86899, &P24435_288};	/*Property = CssImages*/
const Value V24435_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_291 = {S13, 0, &V24435_291_57182, &P24435_289};	/*Property = CssMediaQueries*/
const Value V24435_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_292 = {S14, 0, &V24435_292_57185, &P24435_291};	/*Property = CssMinMax*/
const Value V24435_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_293 = {S15, 0, &V24435_293_57188, &P24435_292};	/*Property = CssOverflow*/
const Value V24435_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_294 = {S16, 0, &V24435_294_57191, &P24435_293};	/*Property = CssPosition*/
const Value V24435_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_295 = {S17, 0, &V24435_295_57194, &P24435_294};	/*Property = CssText*/
const Value V24435_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_297 = {S18, 0, &V24435_297_57197, &P24435_295};	/*Property = CssTransforms*/
const Value V24435_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_241 = {S19, 0, &V24435_241_55801, &P24435_297};	/*Property = CssTransitions*/
const Value V24435_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_298 = {S20, 0, &V24435_298_57208, &P24435_241};	/*Property = CssUI*/
const Value V24435_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_300 = {S21, 0, &V24435_300_57217, &P24435_298};	/*Property = DataSet*/
const Value V24435_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_301 = {S22, 0, &V24435_301_57220, &P24435_300};	/*Property = DataUrl*/
const Value V24435_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_302 = {S23, 0, &V24435_302_57223, &P24435_301};	/*Property = DeviceOrientation*/
const Value V24435_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_304 = {S24, 0, &V24435_304_57229, &P24435_302};	/*Property = FileReader*/
const Value V24435_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_305 = {S25, 0, &V24435_305_57232, &P24435_304};	/*Property = FileSaver*/
const Value V24435_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_306 = {S26, 0, &V24435_306_57238, &P24435_305};	/*Property = FileWriter*/
const Value V24435_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_307 = {S27, 0, &V24435_307_57241, &P24435_306};	/*Property = FormData*/
const Value V24435_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_309 = {S28, 0, &V24435_309_57247, &P24435_307};	/*Property = Fullscreen*/
const Value V24435_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_310 = {S29, 0, &V24435_310_57250, &P24435_309};	/*Property = GeoLocation*/
const Value V24435_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_312 = {S30, 0, &V24435_312_57256, &P24435_310};	/*Property = History*/
const Value V24435_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_314 = {S31, 0, &V24435_314_57262, &P24435_312};	/*Property = Html5*/
const Value V24435_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_313 = {S32, 0, &V24435_313_57259, &P24435_314};	/*Property = Html-Media-Capture*/
const Value V24435_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_315 = {S33, 0, &V24435_315_57265, &P24435_313};	/*Property = Iframe*/
const Value V24435_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_316 = {S34, 0, &V24435_316_57268, &P24435_315};	/*Property = IndexedDB*/
const Value V24435_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_317 = {S35, 0, &V24435_317_57271, &P24435_316};	/*Property = Json*/
const Value V24435_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24435_15 = {S37, 0, &V24435_15_28, &P24435_317};	/*Property = LayoutEngine*/
const Value V24435_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_344 = {S38, 0, &V24435_344_57274, &P24435_15};	/*Property = Masking*/
const Value V24435_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_320 = {S39, 0, &V24435_320_57292, &P24435_344};	/*Property = PostMessage*/
const Value V24435_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_321 = {S40, 0, &V24435_321_57295, &P24435_320};	/*Property = Progress*/
const Value V24435_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_322 = {S41, 0, &V24435_322_57298, &P24435_321};	/*Property = Prompts*/
const Value V24435_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_323 = {S42, 0, &V24435_323_57301, &P24435_322};	/*Property = Selector*/
const Value V24435_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_324 = {S43, 0, &V24435_324_57307, &P24435_323};	/*Property = Svg*/
const Value V24435_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_327 = {S44, 0, &V24435_327_57316, &P24435_324};	/*Property = TouchEvents*/
const Value V24435_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_328 = {S45, 0, &V24435_328_57319, &P24435_327};	/*Property = Track*/
const Value V24435_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_329 = {S46, 0, &V24435_329_57325, &P24435_328};	/*Property = Video*/
const Value V24435_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_330 = {S47, 0, &V24435_330_57328, &P24435_329};	/*Property = Viewport*/
const Value V24435_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_332 = {S48, 0, &V24435_332_57340, &P24435_330};	/*Property = WebWorkers*/
const Value V24435_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24435_333 = {S49, 0, &V24435_333_57343, &P24435_332};	/*Property = Xhr2*/
const Profile P24435 = {"24435", &P24435_333};	/*Profile ID - 24435*/
const Value V21006_1_1 = {S2, NULL};	/*Value = True*/
const Property P21006_1 = {S50, 0, &V21006_1_1, NULL};	/*Property = IsMobile*/
const Value V21006_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21006_5 = {S52, 0, &V21006_5_1227, &P21006_1};	/*Property = ScreenPixelsHeight*/
const Value V21006_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21006_4 = {S54, 0, &V21006_4_1229, &P21006_5};	/*Property = ScreenPixelsWidth*/
const Profile P21006 = {"21006", &P21006_4};	/*Profile ID - 21006*/
const Value V21049_1_1 = {S2, NULL};	/*Value = True*/
const Property P21049_1 = {S50, 0, &V21049_1_1, NULL};	/*Property = IsMobile*/
const Value V21049_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21049_5 = {S52, 0, &V21049_5_1227, &P21049_1};	/*Property = ScreenPixelsHeight*/
const Value V21049_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21049_4 = {S54, 0, &V21049_4_1229, &P21049_5};	/*Property = ScreenPixelsWidth*/
const Profile P21049 = {"21049", &P21049_4};	/*Profile ID - 21049*/
const Value V22432_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22432_279 = {S1, 0, &V22432_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22432_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22432_282 = {S3, 0, &V22432_282_57153, &P22432_279};	/*Property = BlobBuilder*/
const Value V22432_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22432_350 = {S4, 0, &V22432_350_57444, &P22432_282};	/*Property = Canvas*/
const Value V22432_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22432_283 = {S5, 0, &V22432_283_57159, &P22432_350};	/*Property = CssBackground*/
const Value V22432_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22432_284 = {S6, 0, &V22432_284_57163, &P22432_283};	/*Property = CssBorderImage*/
const Value V22432_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22432_285 = {S7, 0, &V22432_285_57166, &P22432_284};	/*Property = CssCanvas*/
const Value V22432_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22432_286 = {S8, 0, &V22432_286_57168, &P22432_285};	/*Property = CssColor*/
const Value V22432_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22432_237 = {S9, 0, &V22432_237_55784, &P22432_286};	/*Property = CssColumn*/
const Value V22432_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22432_287 = {S10, 0, &V22432_287_57172, &P22432_237};	/*Property = CssFlexbox*/
const Value V22432_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22432_288 = {S11, 0, &V22432_288_57175, &P22432_287};	/*Property = CssFont*/
const Value V22432_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22432_289 = {S12, 0, &V22432_289_57759, &P22432_288};	/*Property = CssImages*/
const Value V22432_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22432_291 = {S13, 0, &V22432_291_57180, &P22432_289};	/*Property = CssMediaQueries*/
const Value V22432_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22432_292 = {S14, 0, &V22432_292_57183, &P22432_291};	/*Property = CssMinMax*/
const Value V22432_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22432_293 = {S15, 0, &V22432_293_57187, &P22432_292};	/*Property = CssOverflow*/
const Value V22432_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22432_294 = {S16, 0, &V22432_294_57189, &P22432_293};	/*Property = CssPosition*/
const Value V22432_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22432_295 = {S17, 0, &V22432_295_57192, &P22432_294};	/*Property = CssText*/
const Value V22432_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22432_297 = {S18, 0, &V22432_297_57195, &P22432_295};	/*Property = CssTransforms*/
const Value V22432_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22432_241 = {S19, 0, &V22432_241_55799, &P22432_297};	/*Property = CssTransitions*/
const Value V22432_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22432_298 = {S20, 0, &V22432_298_57206, &P22432_241};	/*Property = CssUI*/
const Value V22432_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22432_300 = {S21, 0, &V22432_300_57216, &P22432_298};	/*Property = DataSet*/
const Value V22432_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22432_301 = {S22, 0, &V22432_301_57218, &P22432_300};	/*Property = DataUrl*/
const Value V22432_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22432_302 = {S23, 0, &V22432_302_57222, &P22432_301};	/*Property = DeviceOrientation*/
const Value V22432_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22432_304 = {S24, 0, &V22432_304_57227, &P22432_302};	/*Property = FileReader*/
const Value V22432_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22432_305 = {S25, 0, &V22432_305_57231, &P22432_304};	/*Property = FileSaver*/
const Value V22432_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22432_306 = {S26, 0, &V22432_306_57237, &P22432_305};	/*Property = FileWriter*/
const Value V22432_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22432_307 = {S27, 0, &V22432_307_57239, &P22432_306};	/*Property = FormData*/
const Value V22432_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22432_309 = {S28, 0, &V22432_309_57246, &P22432_307};	/*Property = Fullscreen*/
const Value V22432_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22432_310 = {S29, 0, &V22432_310_57248, &P22432_309};	/*Property = GeoLocation*/
const Value V22432_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22432_312 = {S30, 0, &V22432_312_57254, &P22432_310};	/*Property = History*/
const Value V22432_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22432_314 = {S31, 0, &V22432_314_57261, &P22432_312};	/*Property = Html5*/
const Value V22432_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22432_313 = {S32, 0, &V22432_313_57258, &P22432_314};	/*Property = Html-Media-Capture*/
const Value V22432_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22432_315 = {S33, 0, &V22432_315_57264, &P22432_313};	/*Property = Iframe*/
const Value V22432_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22432_316 = {S34, 0, &V22432_316_57266, &P22432_315};	/*Property = IndexedDB*/
const Value V22432_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22432_317 = {S35, 0, &V22432_317_57269, &P22432_316};	/*Property = Json*/
const Value V22432_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22432_15 = {S37, 0, &V22432_15_28, &P22432_317};	/*Property = LayoutEngine*/
const Value V22432_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22432_344 = {S38, 0, &V22432_344_57272, &P22432_15};	/*Property = Masking*/
const Value V22432_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22432_320 = {S39, 0, &V22432_320_57290, &P22432_344};	/*Property = PostMessage*/
const Value V22432_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22432_321 = {S40, 0, &V22432_321_57293, &P22432_320};	/*Property = Progress*/
const Value V22432_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22432_322 = {S41, 0, &V22432_322_57296, &P22432_321};	/*Property = Prompts*/
const Value V22432_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22432_323 = {S42, 0, &V22432_323_57299, &P22432_322};	/*Property = Selector*/
const Value V22432_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22432_324 = {S43, 0, &V22432_324_57306, &P22432_323};	/*Property = Svg*/
const Value V22432_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22432_327 = {S44, 0, &V22432_327_57315, &P22432_324};	/*Property = TouchEvents*/
const Value V22432_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22432_328 = {S45, 0, &V22432_328_57318, &P22432_327};	/*Property = Track*/
const Value V22432_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22432_329 = {S46, 0, &V22432_329_57323, &P22432_328};	/*Property = Video*/
const Value V22432_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22432_330 = {S47, 0, &V22432_330_57326, &P22432_329};	/*Property = Viewport*/
const Value V22432_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22432_332 = {S48, 0, &V22432_332_57338, &P22432_330};	/*Property = WebWorkers*/
const Value V22432_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22432_333 = {S49, 0, &V22432_333_57341, &P22432_332};	/*Property = Xhr2*/
const Profile P22432 = {"22432", &P22432_333};	/*Profile ID - 22432*/
const Profile P18533 = {"18533", NULL};	/*Profile ID - 18533*/
const Value V18361_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_279 = {S1, 0, &V18361_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18361_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_282 = {S3, 0, &V18361_282_57155, &P18361_279};	/*Property = BlobBuilder*/
const Value V18361_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_350 = {S4, 0, &V18361_350_57446, &P18361_282};	/*Property = Canvas*/
const Value V18361_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_283 = {S5, 0, &V18361_283_57161, &P18361_350};	/*Property = CssBackground*/
const Value V18361_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_284 = {S6, 0, &V18361_284_57164, &P18361_283};	/*Property = CssBorderImage*/
const Value V18361_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_285 = {S7, 0, &V18361_285_57167, &P18361_284};	/*Property = CssCanvas*/
const Value V18361_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_286 = {S8, 0, &V18361_286_57170, &P18361_285};	/*Property = CssColor*/
const Value V18361_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_237 = {S9, 0, &V18361_237_55786, &P18361_286};	/*Property = CssColumn*/
const Value V18361_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_287 = {S10, 0, &V18361_287_57173, &P18361_237};	/*Property = CssFlexbox*/
const Value V18361_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_288 = {S11, 0, &V18361_288_57176, &P18361_287};	/*Property = CssFont*/
const Value V18361_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_289 = {S12, 0, &V18361_289_86899, &P18361_288};	/*Property = CssImages*/
const Value V18361_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_291 = {S13, 0, &V18361_291_57182, &P18361_289};	/*Property = CssMediaQueries*/
const Value V18361_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_292 = {S14, 0, &V18361_292_57185, &P18361_291};	/*Property = CssMinMax*/
const Value V18361_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_293 = {S15, 0, &V18361_293_57188, &P18361_292};	/*Property = CssOverflow*/
const Value V18361_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_294 = {S16, 0, &V18361_294_57191, &P18361_293};	/*Property = CssPosition*/
const Value V18361_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_295 = {S17, 0, &V18361_295_57194, &P18361_294};	/*Property = CssText*/
const Value V18361_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_297 = {S18, 0, &V18361_297_57197, &P18361_295};	/*Property = CssTransforms*/
const Value V18361_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_241 = {S19, 0, &V18361_241_55801, &P18361_297};	/*Property = CssTransitions*/
const Value V18361_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_298 = {S20, 0, &V18361_298_57208, &P18361_241};	/*Property = CssUI*/
const Value V18361_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_300 = {S21, 0, &V18361_300_57217, &P18361_298};	/*Property = DataSet*/
const Value V18361_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_301 = {S22, 0, &V18361_301_57220, &P18361_300};	/*Property = DataUrl*/
const Value V18361_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_302 = {S23, 0, &V18361_302_57223, &P18361_301};	/*Property = DeviceOrientation*/
const Value V18361_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_304 = {S24, 0, &V18361_304_57229, &P18361_302};	/*Property = FileReader*/
const Value V18361_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_305 = {S25, 0, &V18361_305_57232, &P18361_304};	/*Property = FileSaver*/
const Value V18361_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_306 = {S26, 0, &V18361_306_57238, &P18361_305};	/*Property = FileWriter*/
const Value V18361_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_307 = {S27, 0, &V18361_307_57241, &P18361_306};	/*Property = FormData*/
const Value V18361_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_309 = {S28, 0, &V18361_309_57247, &P18361_307};	/*Property = Fullscreen*/
const Value V18361_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_310 = {S29, 0, &V18361_310_57250, &P18361_309};	/*Property = GeoLocation*/
const Value V18361_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_312 = {S30, 0, &V18361_312_57256, &P18361_310};	/*Property = History*/
const Value V18361_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_314 = {S31, 0, &V18361_314_57262, &P18361_312};	/*Property = Html5*/
const Value V18361_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_313 = {S32, 0, &V18361_313_57259, &P18361_314};	/*Property = Html-Media-Capture*/
const Value V18361_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_315 = {S33, 0, &V18361_315_57265, &P18361_313};	/*Property = Iframe*/
const Value V18361_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_316 = {S34, 0, &V18361_316_57268, &P18361_315};	/*Property = IndexedDB*/
const Value V18361_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_317 = {S35, 0, &V18361_317_57271, &P18361_316};	/*Property = Json*/
const Value V18361_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18361_15 = {S37, 0, &V18361_15_29, &P18361_317};	/*Property = LayoutEngine*/
const Value V18361_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_344 = {S38, 0, &V18361_344_57274, &P18361_15};	/*Property = Masking*/
const Value V18361_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_320 = {S39, 0, &V18361_320_57292, &P18361_344};	/*Property = PostMessage*/
const Value V18361_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_321 = {S40, 0, &V18361_321_57295, &P18361_320};	/*Property = Progress*/
const Value V18361_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_322 = {S41, 0, &V18361_322_57298, &P18361_321};	/*Property = Prompts*/
const Value V18361_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_323 = {S42, 0, &V18361_323_57301, &P18361_322};	/*Property = Selector*/
const Value V18361_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_324 = {S43, 0, &V18361_324_57307, &P18361_323};	/*Property = Svg*/
const Value V18361_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_327 = {S44, 0, &V18361_327_57316, &P18361_324};	/*Property = TouchEvents*/
const Value V18361_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_328 = {S45, 0, &V18361_328_57319, &P18361_327};	/*Property = Track*/
const Value V18361_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_329 = {S46, 0, &V18361_329_57325, &P18361_328};	/*Property = Video*/
const Value V18361_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_330 = {S47, 0, &V18361_330_57328, &P18361_329};	/*Property = Viewport*/
const Value V18361_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_332 = {S48, 0, &V18361_332_57340, &P18361_330};	/*Property = WebWorkers*/
const Value V18361_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18361_333 = {S49, 0, &V18361_333_57343, &P18361_332};	/*Property = Xhr2*/
const Profile P18361 = {"18361", &P18361_333};	/*Profile ID - 18361*/
const Value V22578_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22578_279 = {S1, 0, &V22578_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22578_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22578_282 = {S3, 0, &V22578_282_57154, &P22578_279};	/*Property = BlobBuilder*/
const Value V22578_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22578_350 = {S4, 0, &V22578_350_57444, &P22578_282};	/*Property = Canvas*/
const Value V22578_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22578_283 = {S5, 0, &V22578_283_57159, &P22578_350};	/*Property = CssBackground*/
const Value V22578_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22578_284 = {S6, 0, &V22578_284_57163, &P22578_283};	/*Property = CssBorderImage*/
const Value V22578_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22578_285 = {S7, 0, &V22578_285_57166, &P22578_284};	/*Property = CssCanvas*/
const Value V22578_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22578_286 = {S8, 0, &V22578_286_57168, &P22578_285};	/*Property = CssColor*/
const Value V22578_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22578_237 = {S9, 0, &V22578_237_55784, &P22578_286};	/*Property = CssColumn*/
const Value V22578_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22578_287 = {S10, 0, &V22578_287_57172, &P22578_237};	/*Property = CssFlexbox*/
const Value V22578_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22578_288 = {S11, 0, &V22578_288_57175, &P22578_287};	/*Property = CssFont*/
const Value V22578_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22578_289 = {S12, 0, &V22578_289_57759, &P22578_288};	/*Property = CssImages*/
const Value V22578_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22578_291 = {S13, 0, &V22578_291_57181, &P22578_289};	/*Property = CssMediaQueries*/
const Value V22578_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22578_292 = {S14, 0, &V22578_292_57183, &P22578_291};	/*Property = CssMinMax*/
const Value V22578_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22578_293 = {S15, 0, &V22578_293_57187, &P22578_292};	/*Property = CssOverflow*/
const Value V22578_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22578_294 = {S16, 0, &V22578_294_57190, &P22578_293};	/*Property = CssPosition*/
const Value V22578_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22578_295 = {S17, 0, &V22578_295_57193, &P22578_294};	/*Property = CssText*/
const Value V22578_297_57196 = {S0, NULL};	/*Value = False*/
const Property P22578_297 = {S18, 0, &V22578_297_57196, &P22578_295};	/*Property = CssTransforms*/
const Value V22578_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22578_241 = {S19, 0, &V22578_241_55799, &P22578_297};	/*Property = CssTransitions*/
const Value V22578_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22578_298 = {S20, 0, &V22578_298_57206, &P22578_241};	/*Property = CssUI*/
const Value V22578_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22578_300 = {S21, 0, &V22578_300_57216, &P22578_298};	/*Property = DataSet*/
const Value V22578_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22578_301 = {S22, 0, &V22578_301_57218, &P22578_300};	/*Property = DataUrl*/
const Value V22578_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22578_302 = {S23, 0, &V22578_302_57222, &P22578_301};	/*Property = DeviceOrientation*/
const Value V22578_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22578_304 = {S24, 0, &V22578_304_57228, &P22578_302};	/*Property = FileReader*/
const Value V22578_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22578_305 = {S25, 0, &V22578_305_57231, &P22578_304};	/*Property = FileSaver*/
const Value V22578_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22578_306 = {S26, 0, &V22578_306_57237, &P22578_305};	/*Property = FileWriter*/
const Value V22578_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22578_307 = {S27, 0, &V22578_307_57240, &P22578_306};	/*Property = FormData*/
const Value V22578_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22578_309 = {S28, 0, &V22578_309_57246, &P22578_307};	/*Property = Fullscreen*/
const Value V22578_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22578_310 = {S29, 0, &V22578_310_57248, &P22578_309};	/*Property = GeoLocation*/
const Value V22578_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22578_312 = {S30, 0, &V22578_312_57255, &P22578_310};	/*Property = History*/
const Value V22578_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22578_314 = {S31, 0, &V22578_314_57261, &P22578_312};	/*Property = Html5*/
const Value V22578_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22578_313 = {S32, 0, &V22578_313_57258, &P22578_314};	/*Property = Html-Media-Capture*/
const Value V22578_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22578_315 = {S33, 0, &V22578_315_57264, &P22578_313};	/*Property = Iframe*/
const Value V22578_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22578_316 = {S34, 0, &V22578_316_57267, &P22578_315};	/*Property = IndexedDB*/
const Value V22578_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22578_317 = {S35, 0, &V22578_317_57269, &P22578_316};	/*Property = Json*/
const Value V22578_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P22578_15 = {S37, 0, &V22578_15_31, &P22578_317};	/*Property = LayoutEngine*/
const Value V22578_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22578_344 = {S38, 0, &V22578_344_57272, &P22578_15};	/*Property = Masking*/
const Value V22578_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22578_320 = {S39, 0, &V22578_320_57290, &P22578_344};	/*Property = PostMessage*/
const Value V22578_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22578_321 = {S40, 0, &V22578_321_57293, &P22578_320};	/*Property = Progress*/
const Value V22578_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22578_322 = {S41, 0, &V22578_322_57296, &P22578_321};	/*Property = Prompts*/
const Value V22578_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22578_323 = {S42, 0, &V22578_323_57299, &P22578_322};	/*Property = Selector*/
const Value V22578_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22578_324 = {S43, 0, &V22578_324_57306, &P22578_323};	/*Property = Svg*/
const Value V22578_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22578_327 = {S44, 0, &V22578_327_57315, &P22578_324};	/*Property = TouchEvents*/
const Value V22578_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22578_328 = {S45, 0, &V22578_328_57318, &P22578_327};	/*Property = Track*/
const Value V22578_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22578_329 = {S46, 0, &V22578_329_57323, &P22578_328};	/*Property = Video*/
const Value V22578_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22578_330 = {S47, 0, &V22578_330_57326, &P22578_329};	/*Property = Viewport*/
const Value V22578_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22578_332 = {S48, 0, &V22578_332_57339, &P22578_330};	/*Property = WebWorkers*/
const Value V22578_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22578_333 = {S49, 0, &V22578_333_57342, &P22578_332};	/*Property = Xhr2*/
const Profile P22578 = {"22578", &P22578_333};	/*Profile ID - 22578*/
const Value V18506_1_1 = {S2, NULL};	/*Value = True*/
const Property P18506_1 = {S50, 0, &V18506_1_1, NULL};	/*Property = IsMobile*/
const Value V18506_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P18506_5 = {S52, 0, &V18506_5_1304, &P18506_1};	/*Property = ScreenPixelsHeight*/
const Value V18506_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18506_4 = {S54, 0, &V18506_4_1224, &P18506_5};	/*Property = ScreenPixelsWidth*/
const Profile P18506 = {"18506", &P18506_4};	/*Profile ID - 18506*/
const Value V22460_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22460_279 = {S1, 0, &V22460_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22460_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22460_282 = {S3, 0, &V22460_282_57153, &P22460_279};	/*Property = BlobBuilder*/
const Value V22460_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22460_350 = {S4, 0, &V22460_350_57444, &P22460_282};	/*Property = Canvas*/
const Value V22460_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22460_283 = {S5, 0, &V22460_283_57159, &P22460_350};	/*Property = CssBackground*/
const Value V22460_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22460_284 = {S6, 0, &V22460_284_57163, &P22460_283};	/*Property = CssBorderImage*/
const Value V22460_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22460_285 = {S7, 0, &V22460_285_57166, &P22460_284};	/*Property = CssCanvas*/
const Value V22460_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22460_286 = {S8, 0, &V22460_286_57168, &P22460_285};	/*Property = CssColor*/
const Value V22460_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22460_237 = {S9, 0, &V22460_237_55784, &P22460_286};	/*Property = CssColumn*/
const Value V22460_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22460_287 = {S10, 0, &V22460_287_57172, &P22460_237};	/*Property = CssFlexbox*/
const Value V22460_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22460_288 = {S11, 0, &V22460_288_57175, &P22460_287};	/*Property = CssFont*/
const Value V22460_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22460_289 = {S12, 0, &V22460_289_57759, &P22460_288};	/*Property = CssImages*/
const Value V22460_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22460_291 = {S13, 0, &V22460_291_57180, &P22460_289};	/*Property = CssMediaQueries*/
const Value V22460_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22460_292 = {S14, 0, &V22460_292_57183, &P22460_291};	/*Property = CssMinMax*/
const Value V22460_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22460_293 = {S15, 0, &V22460_293_57187, &P22460_292};	/*Property = CssOverflow*/
const Value V22460_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22460_294 = {S16, 0, &V22460_294_57189, &P22460_293};	/*Property = CssPosition*/
const Value V22460_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22460_295 = {S17, 0, &V22460_295_57192, &P22460_294};	/*Property = CssText*/
const Value V22460_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22460_297 = {S18, 0, &V22460_297_57195, &P22460_295};	/*Property = CssTransforms*/
const Value V22460_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22460_241 = {S19, 0, &V22460_241_55799, &P22460_297};	/*Property = CssTransitions*/
const Value V22460_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22460_298 = {S20, 0, &V22460_298_57206, &P22460_241};	/*Property = CssUI*/
const Value V22460_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22460_300 = {S21, 0, &V22460_300_57216, &P22460_298};	/*Property = DataSet*/
const Value V22460_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22460_301 = {S22, 0, &V22460_301_57218, &P22460_300};	/*Property = DataUrl*/
const Value V22460_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22460_302 = {S23, 0, &V22460_302_57222, &P22460_301};	/*Property = DeviceOrientation*/
const Value V22460_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22460_304 = {S24, 0, &V22460_304_57227, &P22460_302};	/*Property = FileReader*/
const Value V22460_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22460_305 = {S25, 0, &V22460_305_57231, &P22460_304};	/*Property = FileSaver*/
const Value V22460_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22460_306 = {S26, 0, &V22460_306_57237, &P22460_305};	/*Property = FileWriter*/
const Value V22460_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22460_307 = {S27, 0, &V22460_307_57239, &P22460_306};	/*Property = FormData*/
const Value V22460_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22460_309 = {S28, 0, &V22460_309_57246, &P22460_307};	/*Property = Fullscreen*/
const Value V22460_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22460_310 = {S29, 0, &V22460_310_57248, &P22460_309};	/*Property = GeoLocation*/
const Value V22460_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22460_312 = {S30, 0, &V22460_312_57254, &P22460_310};	/*Property = History*/
const Value V22460_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22460_314 = {S31, 0, &V22460_314_57261, &P22460_312};	/*Property = Html5*/
const Value V22460_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22460_313 = {S32, 0, &V22460_313_57258, &P22460_314};	/*Property = Html-Media-Capture*/
const Value V22460_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22460_315 = {S33, 0, &V22460_315_57264, &P22460_313};	/*Property = Iframe*/
const Value V22460_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22460_316 = {S34, 0, &V22460_316_57266, &P22460_315};	/*Property = IndexedDB*/
const Value V22460_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22460_317 = {S35, 0, &V22460_317_57269, &P22460_316};	/*Property = Json*/
const Value V22460_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22460_15 = {S37, 0, &V22460_15_28, &P22460_317};	/*Property = LayoutEngine*/
const Value V22460_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22460_344 = {S38, 0, &V22460_344_57272, &P22460_15};	/*Property = Masking*/
const Value V22460_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22460_320 = {S39, 0, &V22460_320_57290, &P22460_344};	/*Property = PostMessage*/
const Value V22460_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22460_321 = {S40, 0, &V22460_321_57293, &P22460_320};	/*Property = Progress*/
const Value V22460_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22460_322 = {S41, 0, &V22460_322_57296, &P22460_321};	/*Property = Prompts*/
const Value V22460_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22460_323 = {S42, 0, &V22460_323_57299, &P22460_322};	/*Property = Selector*/
const Value V22460_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22460_324 = {S43, 0, &V22460_324_57306, &P22460_323};	/*Property = Svg*/
const Value V22460_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22460_327 = {S44, 0, &V22460_327_57315, &P22460_324};	/*Property = TouchEvents*/
const Value V22460_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22460_328 = {S45, 0, &V22460_328_57318, &P22460_327};	/*Property = Track*/
const Value V22460_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22460_329 = {S46, 0, &V22460_329_57323, &P22460_328};	/*Property = Video*/
const Value V22460_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22460_330 = {S47, 0, &V22460_330_57326, &P22460_329};	/*Property = Viewport*/
const Value V22460_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22460_332 = {S48, 0, &V22460_332_57338, &P22460_330};	/*Property = WebWorkers*/
const Value V22460_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22460_333 = {S49, 0, &V22460_333_57341, &P22460_332};	/*Property = Xhr2*/
const Profile P22460 = {"22460", &P22460_333};	/*Profile ID - 22460*/
const Value V21913_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21913_279 = {S1, 0, &V21913_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21913_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21913_282 = {S3, 0, &V21913_282_57154, &P21913_279};	/*Property = BlobBuilder*/
const Value V21913_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21913_350 = {S4, 0, &V21913_350_57444, &P21913_282};	/*Property = Canvas*/
const Value V21913_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21913_283 = {S5, 0, &V21913_283_57159, &P21913_350};	/*Property = CssBackground*/
const Value V21913_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21913_284 = {S6, 0, &V21913_284_57163, &P21913_283};	/*Property = CssBorderImage*/
const Value V21913_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21913_285 = {S7, 0, &V21913_285_57166, &P21913_284};	/*Property = CssCanvas*/
const Value V21913_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21913_286 = {S8, 0, &V21913_286_57168, &P21913_285};	/*Property = CssColor*/
const Value V21913_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21913_237 = {S9, 0, &V21913_237_55786, &P21913_286};	/*Property = CssColumn*/
const Value V21913_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21913_287 = {S10, 0, &V21913_287_57172, &P21913_237};	/*Property = CssFlexbox*/
const Value V21913_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21913_288 = {S11, 0, &V21913_288_57175, &P21913_287};	/*Property = CssFont*/
const Value V21913_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21913_289 = {S12, 0, &V21913_289_57759, &P21913_288};	/*Property = CssImages*/
const Value V21913_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21913_291 = {S13, 0, &V21913_291_57181, &P21913_289};	/*Property = CssMediaQueries*/
const Value V21913_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21913_292 = {S14, 0, &V21913_292_57183, &P21913_291};	/*Property = CssMinMax*/
const Value V21913_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21913_293 = {S15, 0, &V21913_293_57187, &P21913_292};	/*Property = CssOverflow*/
const Value V21913_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21913_294 = {S16, 0, &V21913_294_57190, &P21913_293};	/*Property = CssPosition*/
const Value V21913_295_57193 = {S0, NULL};	/*Value = False*/
const Property P21913_295 = {S17, 0, &V21913_295_57193, &P21913_294};	/*Property = CssText*/
const Value V21913_297_57196 = {S0, NULL};	/*Value = False*/
const Property P21913_297 = {S18, 0, &V21913_297_57196, &P21913_295};	/*Property = CssTransforms*/
const Value V21913_241_55800 = {S0, NULL};	/*Value = False*/
const Property P21913_241 = {S19, 0, &V21913_241_55800, &P21913_297};	/*Property = CssTransitions*/
const Value V21913_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21913_298 = {S20, 0, &V21913_298_57206, &P21913_241};	/*Property = CssUI*/
const Value V21913_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21913_300 = {S21, 0, &V21913_300_57216, &P21913_298};	/*Property = DataSet*/
const Value V21913_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21913_301 = {S22, 0, &V21913_301_57218, &P21913_300};	/*Property = DataUrl*/
const Value V21913_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21913_302 = {S23, 0, &V21913_302_57222, &P21913_301};	/*Property = DeviceOrientation*/
const Value V21913_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21913_304 = {S24, 0, &V21913_304_57228, &P21913_302};	/*Property = FileReader*/
const Value V21913_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21913_305 = {S25, 0, &V21913_305_57231, &P21913_304};	/*Property = FileSaver*/
const Value V21913_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21913_306 = {S26, 0, &V21913_306_57237, &P21913_305};	/*Property = FileWriter*/
const Value V21913_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21913_307 = {S27, 0, &V21913_307_57240, &P21913_306};	/*Property = FormData*/
const Value V21913_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21913_309 = {S28, 0, &V21913_309_57246, &P21913_307};	/*Property = Fullscreen*/
const Value V21913_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21913_310 = {S29, 0, &V21913_310_57248, &P21913_309};	/*Property = GeoLocation*/
const Value V21913_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21913_312 = {S30, 0, &V21913_312_57255, &P21913_310};	/*Property = History*/
const Value V21913_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21913_314 = {S31, 0, &V21913_314_57261, &P21913_312};	/*Property = Html5*/
const Value V21913_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21913_313 = {S32, 0, &V21913_313_57258, &P21913_314};	/*Property = Html-Media-Capture*/
const Value V21913_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21913_315 = {S33, 0, &V21913_315_57264, &P21913_313};	/*Property = Iframe*/
const Value V21913_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21913_316 = {S34, 0, &V21913_316_57267, &P21913_315};	/*Property = IndexedDB*/
const Value V21913_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21913_317 = {S35, 0, &V21913_317_57269, &P21913_316};	/*Property = Json*/
const Value V21913_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21913_15 = {S37, 0, &V21913_15_28, &P21913_317};	/*Property = LayoutEngine*/
const Value V21913_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21913_344 = {S38, 0, &V21913_344_57272, &P21913_15};	/*Property = Masking*/
const Value V21913_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21913_320 = {S39, 0, &V21913_320_57290, &P21913_344};	/*Property = PostMessage*/
const Value V21913_321_57294 = {S0, NULL};	/*Value = False*/
const Property P21913_321 = {S40, 0, &V21913_321_57294, &P21913_320};	/*Property = Progress*/
const Value V21913_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21913_322 = {S41, 0, &V21913_322_57296, &P21913_321};	/*Property = Prompts*/
const Value V21913_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21913_323 = {S42, 0, &V21913_323_57299, &P21913_322};	/*Property = Selector*/
const Value V21913_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21913_324 = {S43, 0, &V21913_324_57306, &P21913_323};	/*Property = Svg*/
const Value V21913_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21913_327 = {S44, 0, &V21913_327_57315, &P21913_324};	/*Property = TouchEvents*/
const Value V21913_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21913_328 = {S45, 0, &V21913_328_57318, &P21913_327};	/*Property = Track*/
const Value V21913_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21913_329 = {S46, 0, &V21913_329_57323, &P21913_328};	/*Property = Video*/
const Value V21913_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21913_330 = {S47, 0, &V21913_330_57326, &P21913_329};	/*Property = Viewport*/
const Value V21913_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21913_332 = {S48, 0, &V21913_332_57339, &P21913_330};	/*Property = WebWorkers*/
const Value V21913_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21913_333 = {S49, 0, &V21913_333_57342, &P21913_332};	/*Property = Xhr2*/
const Profile P21913 = {"21913", &P21913_333};	/*Profile ID - 21913*/
const Value V22433_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22433_279 = {S1, 0, &V22433_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22433_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22433_282 = {S3, 0, &V22433_282_57154, &P22433_279};	/*Property = BlobBuilder*/
const Value V22433_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22433_350 = {S4, 0, &V22433_350_57444, &P22433_282};	/*Property = Canvas*/
const Value V22433_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22433_283 = {S5, 0, &V22433_283_57159, &P22433_350};	/*Property = CssBackground*/
const Value V22433_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22433_284 = {S6, 0, &V22433_284_57163, &P22433_283};	/*Property = CssBorderImage*/
const Value V22433_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22433_285 = {S7, 0, &V22433_285_57166, &P22433_284};	/*Property = CssCanvas*/
const Value V22433_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22433_286 = {S8, 0, &V22433_286_57168, &P22433_285};	/*Property = CssColor*/
const Value V22433_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22433_237 = {S9, 0, &V22433_237_55784, &P22433_286};	/*Property = CssColumn*/
const Value V22433_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22433_287 = {S10, 0, &V22433_287_57172, &P22433_237};	/*Property = CssFlexbox*/
const Value V22433_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22433_288 = {S11, 0, &V22433_288_57174, &P22433_287};	/*Property = CssFont*/
const Value V22433_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22433_289 = {S12, 0, &V22433_289_57759, &P22433_288};	/*Property = CssImages*/
const Value V22433_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22433_291 = {S13, 0, &V22433_291_57180, &P22433_289};	/*Property = CssMediaQueries*/
const Value V22433_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22433_292 = {S14, 0, &V22433_292_57183, &P22433_291};	/*Property = CssMinMax*/
const Value V22433_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22433_293 = {S15, 0, &V22433_293_57187, &P22433_292};	/*Property = CssOverflow*/
const Value V22433_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22433_294 = {S16, 0, &V22433_294_57189, &P22433_293};	/*Property = CssPosition*/
const Value V22433_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22433_295 = {S17, 0, &V22433_295_57192, &P22433_294};	/*Property = CssText*/
const Value V22433_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22433_297 = {S18, 0, &V22433_297_57195, &P22433_295};	/*Property = CssTransforms*/
const Value V22433_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22433_241 = {S19, 0, &V22433_241_55799, &P22433_297};	/*Property = CssTransitions*/
const Value V22433_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22433_298 = {S20, 0, &V22433_298_57206, &P22433_241};	/*Property = CssUI*/
const Value V22433_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22433_300 = {S21, 0, &V22433_300_57216, &P22433_298};	/*Property = DataSet*/
const Value V22433_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22433_301 = {S22, 0, &V22433_301_57218, &P22433_300};	/*Property = DataUrl*/
const Value V22433_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22433_302 = {S23, 0, &V22433_302_57222, &P22433_301};	/*Property = DeviceOrientation*/
const Value V22433_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22433_304 = {S24, 0, &V22433_304_57227, &P22433_302};	/*Property = FileReader*/
const Value V22433_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22433_305 = {S25, 0, &V22433_305_57231, &P22433_304};	/*Property = FileSaver*/
const Value V22433_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22433_306 = {S26, 0, &V22433_306_57237, &P22433_305};	/*Property = FileWriter*/
const Value V22433_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22433_307 = {S27, 0, &V22433_307_57239, &P22433_306};	/*Property = FormData*/
const Value V22433_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22433_309 = {S28, 0, &V22433_309_57246, &P22433_307};	/*Property = Fullscreen*/
const Value V22433_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22433_310 = {S29, 0, &V22433_310_57248, &P22433_309};	/*Property = GeoLocation*/
const Value V22433_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22433_312 = {S30, 0, &V22433_312_57254, &P22433_310};	/*Property = History*/
const Value V22433_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22433_314 = {S31, 0, &V22433_314_57261, &P22433_312};	/*Property = Html5*/
const Value V22433_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22433_313 = {S32, 0, &V22433_313_57258, &P22433_314};	/*Property = Html-Media-Capture*/
const Value V22433_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22433_315 = {S33, 0, &V22433_315_57264, &P22433_313};	/*Property = Iframe*/
const Value V22433_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22433_316 = {S34, 0, &V22433_316_57266, &P22433_315};	/*Property = IndexedDB*/
const Value V22433_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22433_317 = {S35, 0, &V22433_317_57269, &P22433_316};	/*Property = Json*/
const Value V22433_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22433_15 = {S37, 0, &V22433_15_28, &P22433_317};	/*Property = LayoutEngine*/
const Value V22433_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22433_344 = {S38, 0, &V22433_344_57272, &P22433_15};	/*Property = Masking*/
const Value V22433_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22433_320 = {S39, 0, &V22433_320_57290, &P22433_344};	/*Property = PostMessage*/
const Value V22433_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22433_321 = {S40, 0, &V22433_321_57293, &P22433_320};	/*Property = Progress*/
const Value V22433_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22433_322 = {S41, 0, &V22433_322_57296, &P22433_321};	/*Property = Prompts*/
const Value V22433_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22433_323 = {S42, 0, &V22433_323_57299, &P22433_322};	/*Property = Selector*/
const Value V22433_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22433_324 = {S43, 0, &V22433_324_57306, &P22433_323};	/*Property = Svg*/
const Value V22433_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22433_327 = {S44, 0, &V22433_327_57315, &P22433_324};	/*Property = TouchEvents*/
const Value V22433_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22433_328 = {S45, 0, &V22433_328_57318, &P22433_327};	/*Property = Track*/
const Value V22433_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22433_329 = {S46, 0, &V22433_329_57323, &P22433_328};	/*Property = Video*/
const Value V22433_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22433_330 = {S47, 0, &V22433_330_57326, &P22433_329};	/*Property = Viewport*/
const Value V22433_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22433_332 = {S48, 0, &V22433_332_57338, &P22433_330};	/*Property = WebWorkers*/
const Value V22433_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22433_333 = {S49, 0, &V22433_333_57341, &P22433_332};	/*Property = Xhr2*/
const Profile P22433 = {"22433", &P22433_333};	/*Profile ID - 22433*/
const Value V18431_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18431_279 = {S1, 0, &V18431_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18431_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18431_282 = {S3, 0, &V18431_282_57154, &P18431_279};	/*Property = BlobBuilder*/
const Value V18431_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18431_350 = {S4, 0, &V18431_350_57444, &P18431_282};	/*Property = Canvas*/
const Value V18431_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18431_283 = {S5, 0, &V18431_283_57159, &P18431_350};	/*Property = CssBackground*/
const Value V18431_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18431_284 = {S6, 0, &V18431_284_57163, &P18431_283};	/*Property = CssBorderImage*/
const Value V18431_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18431_285 = {S7, 0, &V18431_285_57166, &P18431_284};	/*Property = CssCanvas*/
const Value V18431_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18431_286 = {S8, 0, &V18431_286_57168, &P18431_285};	/*Property = CssColor*/
const Value V18431_237_55784 = {S2, NULL};	/*Value = True*/
const Property P18431_237 = {S9, 0, &V18431_237_55784, &P18431_286};	/*Property = CssColumn*/
const Value V18431_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18431_287 = {S10, 0, &V18431_287_57172, &P18431_237};	/*Property = CssFlexbox*/
const Value V18431_288_57174 = {S2, NULL};	/*Value = True*/
const Property P18431_288 = {S11, 0, &V18431_288_57174, &P18431_287};	/*Property = CssFont*/
const Value V18431_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18431_289 = {S12, 0, &V18431_289_57759, &P18431_288};	/*Property = CssImages*/
const Value V18431_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18431_291 = {S13, 0, &V18431_291_57180, &P18431_289};	/*Property = CssMediaQueries*/
const Value V18431_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18431_292 = {S14, 0, &V18431_292_57183, &P18431_291};	/*Property = CssMinMax*/
const Value V18431_293_57186 = {S2, NULL};	/*Value = True*/
const Property P18431_293 = {S15, 0, &V18431_293_57186, &P18431_292};	/*Property = CssOverflow*/
const Value V18431_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18431_294 = {S16, 0, &V18431_294_57189, &P18431_293};	/*Property = CssPosition*/
const Value V18431_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18431_295 = {S17, 0, &V18431_295_57192, &P18431_294};	/*Property = CssText*/
const Value V18431_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18431_297 = {S18, 0, &V18431_297_57195, &P18431_295};	/*Property = CssTransforms*/
const Value V18431_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18431_241 = {S19, 0, &V18431_241_55799, &P18431_297};	/*Property = CssTransitions*/
const Value V18431_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18431_298 = {S20, 0, &V18431_298_57206, &P18431_241};	/*Property = CssUI*/
const Value V18431_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18431_300 = {S21, 0, &V18431_300_57216, &P18431_298};	/*Property = DataSet*/
const Value V18431_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18431_301 = {S22, 0, &V18431_301_57218, &P18431_300};	/*Property = DataUrl*/
const Value V18431_302_57221 = {S2, NULL};	/*Value = True*/
const Property P18431_302 = {S23, 0, &V18431_302_57221, &P18431_301};	/*Property = DeviceOrientation*/
const Value V18431_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18431_304 = {S24, 0, &V18431_304_57227, &P18431_302};	/*Property = FileReader*/
const Value V18431_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18431_305 = {S25, 0, &V18431_305_57231, &P18431_304};	/*Property = FileSaver*/
const Value V18431_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18431_306 = {S26, 0, &V18431_306_57237, &P18431_305};	/*Property = FileWriter*/
const Value V18431_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18431_307 = {S27, 0, &V18431_307_57239, &P18431_306};	/*Property = FormData*/
const Value V18431_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18431_309 = {S28, 0, &V18431_309_57246, &P18431_307};	/*Property = Fullscreen*/
const Value V18431_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18431_310 = {S29, 0, &V18431_310_57248, &P18431_309};	/*Property = GeoLocation*/
const Value V18431_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18431_312 = {S30, 0, &V18431_312_57254, &P18431_310};	/*Property = History*/
const Value V18431_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18431_314 = {S31, 0, &V18431_314_57261, &P18431_312};	/*Property = Html5*/
const Value V18431_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18431_313 = {S32, 0, &V18431_313_57258, &P18431_314};	/*Property = Html-Media-Capture*/
const Value V18431_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18431_315 = {S33, 0, &V18431_315_57264, &P18431_313};	/*Property = Iframe*/
const Value V18431_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18431_316 = {S34, 0, &V18431_316_57267, &P18431_315};	/*Property = IndexedDB*/
const Value V18431_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18431_317 = {S35, 0, &V18431_317_57269, &P18431_316};	/*Property = Json*/
const Value V18431_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18431_15 = {S37, 0, &V18431_15_28, &P18431_317};	/*Property = LayoutEngine*/
const Value V18431_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18431_344 = {S38, 0, &V18431_344_57272, &P18431_15};	/*Property = Masking*/
const Value V18431_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18431_320 = {S39, 0, &V18431_320_57290, &P18431_344};	/*Property = PostMessage*/
const Value V18431_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18431_321 = {S40, 0, &V18431_321_57293, &P18431_320};	/*Property = Progress*/
const Value V18431_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18431_322 = {S41, 0, &V18431_322_57296, &P18431_321};	/*Property = Prompts*/
const Value V18431_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18431_323 = {S42, 0, &V18431_323_57299, &P18431_322};	/*Property = Selector*/
const Value V18431_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18431_324 = {S43, 0, &V18431_324_57306, &P18431_323};	/*Property = Svg*/
const Value V18431_327_57314 = {S2, NULL};	/*Value = True*/
const Property P18431_327 = {S44, 0, &V18431_327_57314, &P18431_324};	/*Property = TouchEvents*/
const Value V18431_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18431_328 = {S45, 0, &V18431_328_57318, &P18431_327};	/*Property = Track*/
const Value V18431_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18431_329 = {S46, 0, &V18431_329_57323, &P18431_328};	/*Property = Video*/
const Value V18431_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18431_330 = {S47, 0, &V18431_330_57326, &P18431_329};	/*Property = Viewport*/
const Value V18431_332_57338 = {S2, NULL};	/*Value = True*/
const Property P18431_332 = {S48, 0, &V18431_332_57338, &P18431_330};	/*Property = WebWorkers*/
const Value V18431_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18431_333 = {S49, 0, &V18431_333_57342, &P18431_332};	/*Property = Xhr2*/
const Profile P18431 = {"18431", &P18431_333};	/*Profile ID - 18431*/
const Value V18429_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18429_279 = {S1, 0, &V18429_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18429_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18429_282 = {S3, 0, &V18429_282_57154, &P18429_279};	/*Property = BlobBuilder*/
const Value V18429_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18429_350 = {S4, 0, &V18429_350_57444, &P18429_282};	/*Property = Canvas*/
const Value V18429_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18429_283 = {S5, 0, &V18429_283_57159, &P18429_350};	/*Property = CssBackground*/
const Value V18429_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18429_284 = {S6, 0, &V18429_284_57163, &P18429_283};	/*Property = CssBorderImage*/
const Value V18429_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18429_285 = {S7, 0, &V18429_285_57166, &P18429_284};	/*Property = CssCanvas*/
const Value V18429_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18429_286 = {S8, 0, &V18429_286_57168, &P18429_285};	/*Property = CssColor*/
const Value V18429_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18429_237 = {S9, 0, &V18429_237_55786, &P18429_286};	/*Property = CssColumn*/
const Value V18429_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18429_287 = {S10, 0, &V18429_287_57172, &P18429_237};	/*Property = CssFlexbox*/
const Value V18429_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18429_288 = {S11, 0, &V18429_288_57175, &P18429_287};	/*Property = CssFont*/
const Value V18429_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18429_289 = {S12, 0, &V18429_289_57759, &P18429_288};	/*Property = CssImages*/
const Value V18429_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18429_291 = {S13, 0, &V18429_291_57180, &P18429_289};	/*Property = CssMediaQueries*/
const Value V18429_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18429_292 = {S14, 0, &V18429_292_57183, &P18429_291};	/*Property = CssMinMax*/
const Value V18429_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18429_293 = {S15, 0, &V18429_293_57187, &P18429_292};	/*Property = CssOverflow*/
const Value V18429_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18429_294 = {S16, 0, &V18429_294_57189, &P18429_293};	/*Property = CssPosition*/
const Value V18429_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18429_295 = {S17, 0, &V18429_295_57192, &P18429_294};	/*Property = CssText*/
const Value V18429_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18429_297 = {S18, 0, &V18429_297_57195, &P18429_295};	/*Property = CssTransforms*/
const Value V18429_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18429_241 = {S19, 0, &V18429_241_55799, &P18429_297};	/*Property = CssTransitions*/
const Value V18429_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18429_298 = {S20, 0, &V18429_298_57206, &P18429_241};	/*Property = CssUI*/
const Value V18429_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18429_300 = {S21, 0, &V18429_300_57216, &P18429_298};	/*Property = DataSet*/
const Value V18429_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18429_301 = {S22, 0, &V18429_301_57218, &P18429_300};	/*Property = DataUrl*/
const Value V18429_302_57221 = {S2, NULL};	/*Value = True*/
const Property P18429_302 = {S23, 0, &V18429_302_57221, &P18429_301};	/*Property = DeviceOrientation*/
const Value V18429_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18429_304 = {S24, 0, &V18429_304_57227, &P18429_302};	/*Property = FileReader*/
const Value V18429_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18429_305 = {S25, 0, &V18429_305_57231, &P18429_304};	/*Property = FileSaver*/
const Value V18429_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18429_306 = {S26, 0, &V18429_306_57237, &P18429_305};	/*Property = FileWriter*/
const Value V18429_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18429_307 = {S27, 0, &V18429_307_57239, &P18429_306};	/*Property = FormData*/
const Value V18429_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18429_309 = {S28, 0, &V18429_309_57246, &P18429_307};	/*Property = Fullscreen*/
const Value V18429_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18429_310 = {S29, 0, &V18429_310_57248, &P18429_309};	/*Property = GeoLocation*/
const Value V18429_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18429_312 = {S30, 0, &V18429_312_57254, &P18429_310};	/*Property = History*/
const Value V18429_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18429_314 = {S31, 0, &V18429_314_57261, &P18429_312};	/*Property = Html5*/
const Value V18429_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18429_313 = {S32, 0, &V18429_313_57258, &P18429_314};	/*Property = Html-Media-Capture*/
const Value V18429_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18429_315 = {S33, 0, &V18429_315_57264, &P18429_313};	/*Property = Iframe*/
const Value V18429_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18429_316 = {S34, 0, &V18429_316_57266, &P18429_315};	/*Property = IndexedDB*/
const Value V18429_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18429_317 = {S35, 0, &V18429_317_57269, &P18429_316};	/*Property = Json*/
const Value V18429_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18429_15 = {S37, 0, &V18429_15_29, &P18429_317};	/*Property = LayoutEngine*/
const Value V18429_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18429_344 = {S38, 0, &V18429_344_57272, &P18429_15};	/*Property = Masking*/
const Value V18429_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18429_320 = {S39, 0, &V18429_320_57290, &P18429_344};	/*Property = PostMessage*/
const Value V18429_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18429_321 = {S40, 0, &V18429_321_57293, &P18429_320};	/*Property = Progress*/
const Value V18429_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18429_322 = {S41, 0, &V18429_322_57296, &P18429_321};	/*Property = Prompts*/
const Value V18429_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18429_323 = {S42, 0, &V18429_323_57299, &P18429_322};	/*Property = Selector*/
const Value V18429_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18429_324 = {S43, 0, &V18429_324_57306, &P18429_323};	/*Property = Svg*/
const Value V18429_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18429_327 = {S44, 0, &V18429_327_57315, &P18429_324};	/*Property = TouchEvents*/
const Value V18429_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18429_328 = {S45, 0, &V18429_328_57318, &P18429_327};	/*Property = Track*/
const Value V18429_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18429_329 = {S46, 0, &V18429_329_57323, &P18429_328};	/*Property = Video*/
const Value V18429_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18429_330 = {S47, 0, &V18429_330_57326, &P18429_329};	/*Property = Viewport*/
const Value V18429_332_57338 = {S2, NULL};	/*Value = True*/
const Property P18429_332 = {S48, 0, &V18429_332_57338, &P18429_330};	/*Property = WebWorkers*/
const Value V18429_333_57341 = {S2, NULL};	/*Value = True*/
const Property P18429_333 = {S49, 0, &V18429_333_57341, &P18429_332};	/*Property = Xhr2*/
const Profile P18429 = {"18429", &P18429_333};	/*Profile ID - 18429*/
const Profile P19822 = {"19822", NULL};	/*Profile ID - 19822*/
const Value V21816_1_1 = {S2, NULL};	/*Value = True*/
const Property P21816_1 = {S50, 0, &V21816_1_1, NULL};	/*Property = IsMobile*/
const Value V21816_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21816_5 = {S52, 0, &V21816_5_1227, &P21816_1};	/*Property = ScreenPixelsHeight*/
const Value V21816_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21816_4 = {S54, 0, &V21816_4_1229, &P21816_5};	/*Property = ScreenPixelsWidth*/
const Profile P21816 = {"21816", &P21816_4};	/*Profile ID - 21816*/
const Value V20786_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20786_279 = {S1, 0, &V20786_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20786_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20786_282 = {S3, 0, &V20786_282_57154, &P20786_279};	/*Property = BlobBuilder*/
const Value V20786_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20786_350 = {S4, 0, &V20786_350_57444, &P20786_282};	/*Property = Canvas*/
const Value V20786_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20786_283 = {S5, 0, &V20786_283_57159, &P20786_350};	/*Property = CssBackground*/
const Value V20786_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20786_284 = {S6, 0, &V20786_284_57163, &P20786_283};	/*Property = CssBorderImage*/
const Value V20786_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20786_285 = {S7, 0, &V20786_285_57166, &P20786_284};	/*Property = CssCanvas*/
const Value V20786_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20786_286 = {S8, 0, &V20786_286_57168, &P20786_285};	/*Property = CssColor*/
const Value V20786_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20786_237 = {S9, 0, &V20786_237_55786, &P20786_286};	/*Property = CssColumn*/
const Value V20786_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20786_287 = {S10, 0, &V20786_287_57172, &P20786_237};	/*Property = CssFlexbox*/
const Value V20786_288_57174 = {S2, NULL};	/*Value = True*/
const Property P20786_288 = {S11, 0, &V20786_288_57174, &P20786_287};	/*Property = CssFont*/
const Value V20786_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20786_289 = {S12, 0, &V20786_289_57759, &P20786_288};	/*Property = CssImages*/
const Value V20786_291_57180 = {S2, NULL};	/*Value = True*/
const Property P20786_291 = {S13, 0, &V20786_291_57180, &P20786_289};	/*Property = CssMediaQueries*/
const Value V20786_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20786_292 = {S14, 0, &V20786_292_57183, &P20786_291};	/*Property = CssMinMax*/
const Value V20786_293_57186 = {S2, NULL};	/*Value = True*/
const Property P20786_293 = {S15, 0, &V20786_293_57186, &P20786_292};	/*Property = CssOverflow*/
const Value V20786_294_57189 = {S2, NULL};	/*Value = True*/
const Property P20786_294 = {S16, 0, &V20786_294_57189, &P20786_293};	/*Property = CssPosition*/
const Value V20786_295_57192 = {S2, NULL};	/*Value = True*/
const Property P20786_295 = {S17, 0, &V20786_295_57192, &P20786_294};	/*Property = CssText*/
const Value V20786_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20786_297 = {S18, 0, &V20786_297_57195, &P20786_295};	/*Property = CssTransforms*/
const Value V20786_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20786_241 = {S19, 0, &V20786_241_55799, &P20786_297};	/*Property = CssTransitions*/
const Value V20786_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20786_298 = {S20, 0, &V20786_298_57206, &P20786_241};	/*Property = CssUI*/
const Value V20786_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20786_300 = {S21, 0, &V20786_300_57216, &P20786_298};	/*Property = DataSet*/
const Value V20786_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20786_301 = {S22, 0, &V20786_301_57218, &P20786_300};	/*Property = DataUrl*/
const Value V20786_302_57221 = {S2, NULL};	/*Value = True*/
const Property P20786_302 = {S23, 0, &V20786_302_57221, &P20786_301};	/*Property = DeviceOrientation*/
const Value V20786_304_57227 = {S2, NULL};	/*Value = True*/
const Property P20786_304 = {S24, 0, &V20786_304_57227, &P20786_302};	/*Property = FileReader*/
const Value V20786_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20786_305 = {S25, 0, &V20786_305_57231, &P20786_304};	/*Property = FileSaver*/
const Value V20786_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20786_306 = {S26, 0, &V20786_306_57237, &P20786_305};	/*Property = FileWriter*/
const Value V20786_307_57239 = {S2, NULL};	/*Value = True*/
const Property P20786_307 = {S27, 0, &V20786_307_57239, &P20786_306};	/*Property = FormData*/
const Value V20786_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20786_309 = {S28, 0, &V20786_309_57246, &P20786_307};	/*Property = Fullscreen*/
const Value V20786_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20786_310 = {S29, 0, &V20786_310_57248, &P20786_309};	/*Property = GeoLocation*/
const Value V20786_312_57254 = {S2, NULL};	/*Value = True*/
const Property P20786_312 = {S30, 0, &V20786_312_57254, &P20786_310};	/*Property = History*/
const Value V20786_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20786_314 = {S31, 0, &V20786_314_57261, &P20786_312};	/*Property = Html5*/
const Value V20786_313_57257 = {S2, NULL};	/*Value = True*/
const Property P20786_313 = {S32, 0, &V20786_313_57257, &P20786_314};	/*Property = Html-Media-Capture*/
const Value V20786_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20786_315 = {S33, 0, &V20786_315_57264, &P20786_313};	/*Property = Iframe*/
const Value V20786_316_57266 = {S2, NULL};	/*Value = True*/
const Property P20786_316 = {S34, 0, &V20786_316_57266, &P20786_315};	/*Property = IndexedDB*/
const Value V20786_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20786_317 = {S35, 0, &V20786_317_57269, &P20786_316};	/*Property = Json*/
const Value V20786_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P20786_15 = {S37, 0, &V20786_15_54167, &P20786_317};	/*Property = LayoutEngine*/
const Value V20786_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20786_344 = {S38, 0, &V20786_344_57272, &P20786_15};	/*Property = Masking*/
const Value V20786_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20786_320 = {S39, 0, &V20786_320_57290, &P20786_344};	/*Property = PostMessage*/
const Value V20786_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20786_321 = {S40, 0, &V20786_321_57293, &P20786_320};	/*Property = Progress*/
const Value V20786_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20786_322 = {S41, 0, &V20786_322_57296, &P20786_321};	/*Property = Prompts*/
const Value V20786_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20786_323 = {S42, 0, &V20786_323_57299, &P20786_322};	/*Property = Selector*/
const Value V20786_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20786_324 = {S43, 0, &V20786_324_57306, &P20786_323};	/*Property = Svg*/
const Value V20786_327_57314 = {S2, NULL};	/*Value = True*/
const Property P20786_327 = {S44, 0, &V20786_327_57314, &P20786_324};	/*Property = TouchEvents*/
const Value V20786_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20786_328 = {S45, 0, &V20786_328_57318, &P20786_327};	/*Property = Track*/
const Value V20786_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20786_329 = {S46, 0, &V20786_329_57323, &P20786_328};	/*Property = Video*/
const Value V20786_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20786_330 = {S47, 0, &V20786_330_57326, &P20786_329};	/*Property = Viewport*/
const Value V20786_332_57339 = {S0, NULL};	/*Value = False*/
const Property P20786_332 = {S48, 0, &V20786_332_57339, &P20786_330};	/*Property = WebWorkers*/
const Value V20786_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20786_333 = {S49, 0, &V20786_333_57342, &P20786_332};	/*Property = Xhr2*/
const Profile P20786 = {"20786", &P20786_333};	/*Profile ID - 20786*/
const Profile P18579 = {"18579", NULL};	/*Profile ID - 18579*/
const Value V18449_1_1 = {S2, NULL};	/*Value = True*/
const Property P18449_1 = {S50, 0, &V18449_1_1, NULL};	/*Property = IsMobile*/
const Value V18449_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18449_5 = {S52, 0, &V18449_5_1277, &P18449_1};	/*Property = ScreenPixelsHeight*/
const Value V18449_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18449_4 = {S54, 0, &V18449_4_1279, &P18449_5};	/*Property = ScreenPixelsWidth*/
const Profile P18449 = {"18449", &P18449_4};	/*Profile ID - 18449*/
const Value V22667_1_2 = {S0, NULL};	/*Value = False*/
const Property P22667_1 = {S50, 0, &V22667_1_2, NULL};	/*Property = IsMobile*/
const Value V22667_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P22667_5 = {S52, 0, &V22667_5_37, &P22667_1};	/*Property = ScreenPixelsHeight*/
const Value V22667_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P22667_4 = {S54, 0, &V22667_4_36, &P22667_5};	/*Property = ScreenPixelsWidth*/
const Profile P22667 = {"22667", &P22667_4};	/*Profile ID - 22667*/
const Value V18668_1_1 = {S2, NULL};	/*Value = True*/
const Property P18668_1 = {S50, 0, &V18668_1_1, NULL};	/*Property = IsMobile*/
const Value V18668_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P18668_5 = {S52, 0, &V18668_5_1380, &P18668_1};	/*Property = ScreenPixelsHeight*/
const Value V18668_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18668_4 = {S54, 0, &V18668_4_1279, &P18668_5};	/*Property = ScreenPixelsWidth*/
const Profile P18668 = {"18668", &P18668_4};	/*Profile ID - 18668*/
const Value V19910_1_1 = {S2, NULL};	/*Value = True*/
const Property P19910_1 = {S50, 0, &V19910_1_1, NULL};	/*Property = IsMobile*/
const Value V19910_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P19910_5 = {S52, 0, &V19910_5_1277, &P19910_1};	/*Property = ScreenPixelsHeight*/
const Value V19910_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P19910_4 = {S54, 0, &V19910_4_1279, &P19910_5};	/*Property = ScreenPixelsWidth*/
const Profile P19910 = {"19910", &P19910_4};	/*Profile ID - 19910*/
const Value V22551_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22551_279 = {S1, 0, &V22551_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22551_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22551_282 = {S3, 0, &V22551_282_57154, &P22551_279};	/*Property = BlobBuilder*/
const Value V22551_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22551_350 = {S4, 0, &V22551_350_57444, &P22551_282};	/*Property = Canvas*/
const Value V22551_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22551_283 = {S5, 0, &V22551_283_57159, &P22551_350};	/*Property = CssBackground*/
const Value V22551_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22551_284 = {S6, 0, &V22551_284_57163, &P22551_283};	/*Property = CssBorderImage*/
const Value V22551_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22551_285 = {S7, 0, &V22551_285_57166, &P22551_284};	/*Property = CssCanvas*/
const Value V22551_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22551_286 = {S8, 0, &V22551_286_57168, &P22551_285};	/*Property = CssColor*/
const Value V22551_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22551_237 = {S9, 0, &V22551_237_55784, &P22551_286};	/*Property = CssColumn*/
const Value V22551_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22551_287 = {S10, 0, &V22551_287_57172, &P22551_237};	/*Property = CssFlexbox*/
const Value V22551_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22551_288 = {S11, 0, &V22551_288_57175, &P22551_287};	/*Property = CssFont*/
const Value V22551_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22551_289 = {S12, 0, &V22551_289_57759, &P22551_288};	/*Property = CssImages*/
const Value V22551_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22551_291 = {S13, 0, &V22551_291_57181, &P22551_289};	/*Property = CssMediaQueries*/
const Value V22551_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22551_292 = {S14, 0, &V22551_292_57183, &P22551_291};	/*Property = CssMinMax*/
const Value V22551_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22551_293 = {S15, 0, &V22551_293_57187, &P22551_292};	/*Property = CssOverflow*/
const Value V22551_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22551_294 = {S16, 0, &V22551_294_57190, &P22551_293};	/*Property = CssPosition*/
const Value V22551_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22551_295 = {S17, 0, &V22551_295_57193, &P22551_294};	/*Property = CssText*/
const Value V22551_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22551_297 = {S18, 0, &V22551_297_57195, &P22551_295};	/*Property = CssTransforms*/
const Value V22551_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22551_241 = {S19, 0, &V22551_241_55799, &P22551_297};	/*Property = CssTransitions*/
const Value V22551_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22551_298 = {S20, 0, &V22551_298_57206, &P22551_241};	/*Property = CssUI*/
const Value V22551_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22551_300 = {S21, 0, &V22551_300_57216, &P22551_298};	/*Property = DataSet*/
const Value V22551_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22551_301 = {S22, 0, &V22551_301_57218, &P22551_300};	/*Property = DataUrl*/
const Value V22551_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22551_302 = {S23, 0, &V22551_302_57222, &P22551_301};	/*Property = DeviceOrientation*/
const Value V22551_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22551_304 = {S24, 0, &V22551_304_57228, &P22551_302};	/*Property = FileReader*/
const Value V22551_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22551_305 = {S25, 0, &V22551_305_57231, &P22551_304};	/*Property = FileSaver*/
const Value V22551_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22551_306 = {S26, 0, &V22551_306_57237, &P22551_305};	/*Property = FileWriter*/
const Value V22551_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22551_307 = {S27, 0, &V22551_307_57240, &P22551_306};	/*Property = FormData*/
const Value V22551_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22551_309 = {S28, 0, &V22551_309_57246, &P22551_307};	/*Property = Fullscreen*/
const Value V22551_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22551_310 = {S29, 0, &V22551_310_57248, &P22551_309};	/*Property = GeoLocation*/
const Value V22551_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22551_312 = {S30, 0, &V22551_312_57255, &P22551_310};	/*Property = History*/
const Value V22551_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22551_314 = {S31, 0, &V22551_314_57261, &P22551_312};	/*Property = Html5*/
const Value V22551_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22551_313 = {S32, 0, &V22551_313_57258, &P22551_314};	/*Property = Html-Media-Capture*/
const Value V22551_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22551_315 = {S33, 0, &V22551_315_57264, &P22551_313};	/*Property = Iframe*/
const Value V22551_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22551_316 = {S34, 0, &V22551_316_57267, &P22551_315};	/*Property = IndexedDB*/
const Value V22551_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22551_317 = {S35, 0, &V22551_317_57269, &P22551_316};	/*Property = Json*/
const Value V22551_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22551_15 = {S37, 0, &V22551_15_29, &P22551_317};	/*Property = LayoutEngine*/
const Value V22551_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22551_344 = {S38, 0, &V22551_344_57272, &P22551_15};	/*Property = Masking*/
const Value V22551_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22551_320 = {S39, 0, &V22551_320_57290, &P22551_344};	/*Property = PostMessage*/
const Value V22551_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22551_321 = {S40, 0, &V22551_321_57293, &P22551_320};	/*Property = Progress*/
const Value V22551_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22551_322 = {S41, 0, &V22551_322_57296, &P22551_321};	/*Property = Prompts*/
const Value V22551_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22551_323 = {S42, 0, &V22551_323_57299, &P22551_322};	/*Property = Selector*/
const Value V22551_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22551_324 = {S43, 0, &V22551_324_57306, &P22551_323};	/*Property = Svg*/
const Value V22551_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22551_327 = {S44, 0, &V22551_327_57315, &P22551_324};	/*Property = TouchEvents*/
const Value V22551_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22551_328 = {S45, 0, &V22551_328_57318, &P22551_327};	/*Property = Track*/
const Value V22551_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22551_329 = {S46, 0, &V22551_329_57323, &P22551_328};	/*Property = Video*/
const Value V22551_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22551_330 = {S47, 0, &V22551_330_57326, &P22551_329};	/*Property = Viewport*/
const Value V22551_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22551_332 = {S48, 0, &V22551_332_57339, &P22551_330};	/*Property = WebWorkers*/
const Value V22551_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22551_333 = {S49, 0, &V22551_333_57342, &P22551_332};	/*Property = Xhr2*/
const Profile P22551 = {"22551", &P22551_333};	/*Profile ID - 22551*/
const Value V20274_1_1 = {S2, NULL};	/*Value = True*/
const Property P20274_1 = {S50, 0, &V20274_1_1, NULL};	/*Property = IsMobile*/
const Value V20274_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20274_5 = {S52, 0, &V20274_5_1458, &P20274_1};	/*Property = ScreenPixelsHeight*/
const Value V20274_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20274_4 = {S54, 0, &V20274_4_1460, &P20274_5};	/*Property = ScreenPixelsWidth*/
const Profile P20274 = {"20274", &P20274_4};	/*Profile ID - 20274*/
const Value V22361_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22361_279 = {S1, 0, &V22361_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22361_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22361_282 = {S3, 0, &V22361_282_57154, &P22361_279};	/*Property = BlobBuilder*/
const Value V22361_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22361_350 = {S4, 0, &V22361_350_57444, &P22361_282};	/*Property = Canvas*/
const Value V22361_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22361_283 = {S5, 0, &V22361_283_57159, &P22361_350};	/*Property = CssBackground*/
const Value V22361_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22361_284 = {S6, 0, &V22361_284_57163, &P22361_283};	/*Property = CssBorderImage*/
const Value V22361_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22361_285 = {S7, 0, &V22361_285_57166, &P22361_284};	/*Property = CssCanvas*/
const Value V22361_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22361_286 = {S8, 0, &V22361_286_57168, &P22361_285};	/*Property = CssColor*/
const Value V22361_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22361_237 = {S9, 0, &V22361_237_55784, &P22361_286};	/*Property = CssColumn*/
const Value V22361_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22361_287 = {S10, 0, &V22361_287_57172, &P22361_237};	/*Property = CssFlexbox*/
const Value V22361_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22361_288 = {S11, 0, &V22361_288_57175, &P22361_287};	/*Property = CssFont*/
const Value V22361_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22361_289 = {S12, 0, &V22361_289_57759, &P22361_288};	/*Property = CssImages*/
const Value V22361_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22361_291 = {S13, 0, &V22361_291_57181, &P22361_289};	/*Property = CssMediaQueries*/
const Value V22361_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22361_292 = {S14, 0, &V22361_292_57183, &P22361_291};	/*Property = CssMinMax*/
const Value V22361_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22361_293 = {S15, 0, &V22361_293_57187, &P22361_292};	/*Property = CssOverflow*/
const Value V22361_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22361_294 = {S16, 0, &V22361_294_57190, &P22361_293};	/*Property = CssPosition*/
const Value V22361_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22361_295 = {S17, 0, &V22361_295_57193, &P22361_294};	/*Property = CssText*/
const Value V22361_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22361_297 = {S18, 0, &V22361_297_57195, &P22361_295};	/*Property = CssTransforms*/
const Value V22361_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22361_241 = {S19, 0, &V22361_241_55799, &P22361_297};	/*Property = CssTransitions*/
const Value V22361_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22361_298 = {S20, 0, &V22361_298_57206, &P22361_241};	/*Property = CssUI*/
const Value V22361_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22361_300 = {S21, 0, &V22361_300_57216, &P22361_298};	/*Property = DataSet*/
const Value V22361_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22361_301 = {S22, 0, &V22361_301_57218, &P22361_300};	/*Property = DataUrl*/
const Value V22361_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22361_302 = {S23, 0, &V22361_302_57222, &P22361_301};	/*Property = DeviceOrientation*/
const Value V22361_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22361_304 = {S24, 0, &V22361_304_57228, &P22361_302};	/*Property = FileReader*/
const Value V22361_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22361_305 = {S25, 0, &V22361_305_57231, &P22361_304};	/*Property = FileSaver*/
const Value V22361_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22361_306 = {S26, 0, &V22361_306_57237, &P22361_305};	/*Property = FileWriter*/
const Value V22361_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22361_307 = {S27, 0, &V22361_307_57240, &P22361_306};	/*Property = FormData*/
const Value V22361_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22361_309 = {S28, 0, &V22361_309_57246, &P22361_307};	/*Property = Fullscreen*/
const Value V22361_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22361_310 = {S29, 0, &V22361_310_57248, &P22361_309};	/*Property = GeoLocation*/
const Value V22361_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22361_312 = {S30, 0, &V22361_312_57255, &P22361_310};	/*Property = History*/
const Value V22361_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22361_314 = {S31, 0, &V22361_314_57261, &P22361_312};	/*Property = Html5*/
const Value V22361_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22361_313 = {S32, 0, &V22361_313_57258, &P22361_314};	/*Property = Html-Media-Capture*/
const Value V22361_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22361_315 = {S33, 0, &V22361_315_57264, &P22361_313};	/*Property = Iframe*/
const Value V22361_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22361_316 = {S34, 0, &V22361_316_57267, &P22361_315};	/*Property = IndexedDB*/
const Value V22361_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22361_317 = {S35, 0, &V22361_317_57269, &P22361_316};	/*Property = Json*/
const Value V22361_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22361_15 = {S37, 0, &V22361_15_29, &P22361_317};	/*Property = LayoutEngine*/
const Value V22361_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22361_344 = {S38, 0, &V22361_344_57272, &P22361_15};	/*Property = Masking*/
const Value V22361_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22361_320 = {S39, 0, &V22361_320_57290, &P22361_344};	/*Property = PostMessage*/
const Value V22361_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22361_321 = {S40, 0, &V22361_321_57293, &P22361_320};	/*Property = Progress*/
const Value V22361_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22361_322 = {S41, 0, &V22361_322_57296, &P22361_321};	/*Property = Prompts*/
const Value V22361_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22361_323 = {S42, 0, &V22361_323_57299, &P22361_322};	/*Property = Selector*/
const Value V22361_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22361_324 = {S43, 0, &V22361_324_57306, &P22361_323};	/*Property = Svg*/
const Value V22361_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22361_327 = {S44, 0, &V22361_327_57315, &P22361_324};	/*Property = TouchEvents*/
const Value V22361_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22361_328 = {S45, 0, &V22361_328_57318, &P22361_327};	/*Property = Track*/
const Value V22361_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22361_329 = {S46, 0, &V22361_329_57323, &P22361_328};	/*Property = Video*/
const Value V22361_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22361_330 = {S47, 0, &V22361_330_57326, &P22361_329};	/*Property = Viewport*/
const Value V22361_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22361_332 = {S48, 0, &V22361_332_57339, &P22361_330};	/*Property = WebWorkers*/
const Value V22361_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22361_333 = {S49, 0, &V22361_333_57342, &P22361_332};	/*Property = Xhr2*/
const Profile P22361 = {"22361", &P22361_333};	/*Profile ID - 22361*/
const Value V21005_1_1 = {S2, NULL};	/*Value = True*/
const Property P21005_1 = {S50, 0, &V21005_1_1, NULL};	/*Property = IsMobile*/
const Value V21005_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21005_5 = {S52, 0, &V21005_5_9981, &P21005_1};	/*Property = ScreenPixelsHeight*/
const Value V21005_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21005_4 = {S54, 0, &V21005_4_39098, &P21005_5};	/*Property = ScreenPixelsWidth*/
const Profile P21005 = {"21005", &P21005_4};	/*Profile ID - 21005*/
const Value V21086_1_1 = {S2, NULL};	/*Value = True*/
const Property P21086_1 = {S50, 0, &V21086_1_1, NULL};	/*Property = IsMobile*/
const Value V21086_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21086_5 = {S52, 0, &V21086_5_1277, &P21086_1};	/*Property = ScreenPixelsHeight*/
const Value V21086_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21086_4 = {S54, 0, &V21086_4_2336, &P21086_5};	/*Property = ScreenPixelsWidth*/
const Profile P21086 = {"21086", &P21086_4};	/*Profile ID - 21086*/
const Value V18523_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_279 = {S1, 0, &V18523_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18523_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_282 = {S3, 0, &V18523_282_57155, &P18523_279};	/*Property = BlobBuilder*/
const Value V18523_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_350 = {S4, 0, &V18523_350_57446, &P18523_282};	/*Property = Canvas*/
const Value V18523_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_283 = {S5, 0, &V18523_283_57161, &P18523_350};	/*Property = CssBackground*/
const Value V18523_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_284 = {S6, 0, &V18523_284_57164, &P18523_283};	/*Property = CssBorderImage*/
const Value V18523_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_285 = {S7, 0, &V18523_285_57167, &P18523_284};	/*Property = CssCanvas*/
const Value V18523_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_286 = {S8, 0, &V18523_286_57170, &P18523_285};	/*Property = CssColor*/
const Value V18523_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_237 = {S9, 0, &V18523_237_55786, &P18523_286};	/*Property = CssColumn*/
const Value V18523_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_287 = {S10, 0, &V18523_287_57173, &P18523_237};	/*Property = CssFlexbox*/
const Value V18523_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_288 = {S11, 0, &V18523_288_57176, &P18523_287};	/*Property = CssFont*/
const Value V18523_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_289 = {S12, 0, &V18523_289_86899, &P18523_288};	/*Property = CssImages*/
const Value V18523_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_291 = {S13, 0, &V18523_291_57182, &P18523_289};	/*Property = CssMediaQueries*/
const Value V18523_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_292 = {S14, 0, &V18523_292_57185, &P18523_291};	/*Property = CssMinMax*/
const Value V18523_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_293 = {S15, 0, &V18523_293_57188, &P18523_292};	/*Property = CssOverflow*/
const Value V18523_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_294 = {S16, 0, &V18523_294_57191, &P18523_293};	/*Property = CssPosition*/
const Value V18523_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_295 = {S17, 0, &V18523_295_57194, &P18523_294};	/*Property = CssText*/
const Value V18523_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_297 = {S18, 0, &V18523_297_57197, &P18523_295};	/*Property = CssTransforms*/
const Value V18523_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_241 = {S19, 0, &V18523_241_55801, &P18523_297};	/*Property = CssTransitions*/
const Value V18523_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_298 = {S20, 0, &V18523_298_57208, &P18523_241};	/*Property = CssUI*/
const Value V18523_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_300 = {S21, 0, &V18523_300_57217, &P18523_298};	/*Property = DataSet*/
const Value V18523_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_301 = {S22, 0, &V18523_301_57220, &P18523_300};	/*Property = DataUrl*/
const Value V18523_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_302 = {S23, 0, &V18523_302_57223, &P18523_301};	/*Property = DeviceOrientation*/
const Value V18523_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_304 = {S24, 0, &V18523_304_57229, &P18523_302};	/*Property = FileReader*/
const Value V18523_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_305 = {S25, 0, &V18523_305_57232, &P18523_304};	/*Property = FileSaver*/
const Value V18523_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_306 = {S26, 0, &V18523_306_57238, &P18523_305};	/*Property = FileWriter*/
const Value V18523_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_307 = {S27, 0, &V18523_307_57241, &P18523_306};	/*Property = FormData*/
const Value V18523_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_309 = {S28, 0, &V18523_309_57247, &P18523_307};	/*Property = Fullscreen*/
const Value V18523_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_310 = {S29, 0, &V18523_310_57250, &P18523_309};	/*Property = GeoLocation*/
const Value V18523_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_312 = {S30, 0, &V18523_312_57256, &P18523_310};	/*Property = History*/
const Value V18523_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_314 = {S31, 0, &V18523_314_57262, &P18523_312};	/*Property = Html5*/
const Value V18523_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_313 = {S32, 0, &V18523_313_57259, &P18523_314};	/*Property = Html-Media-Capture*/
const Value V18523_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_315 = {S33, 0, &V18523_315_57265, &P18523_313};	/*Property = Iframe*/
const Value V18523_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_316 = {S34, 0, &V18523_316_57268, &P18523_315};	/*Property = IndexedDB*/
const Value V18523_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_317 = {S35, 0, &V18523_317_57271, &P18523_316};	/*Property = Json*/
const Value V18523_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18523_15 = {S37, 0, &V18523_15_29, &P18523_317};	/*Property = LayoutEngine*/
const Value V18523_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_344 = {S38, 0, &V18523_344_57274, &P18523_15};	/*Property = Masking*/
const Value V18523_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_320 = {S39, 0, &V18523_320_57292, &P18523_344};	/*Property = PostMessage*/
const Value V18523_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_321 = {S40, 0, &V18523_321_57295, &P18523_320};	/*Property = Progress*/
const Value V18523_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_322 = {S41, 0, &V18523_322_57298, &P18523_321};	/*Property = Prompts*/
const Value V18523_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_323 = {S42, 0, &V18523_323_57301, &P18523_322};	/*Property = Selector*/
const Value V18523_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_324 = {S43, 0, &V18523_324_57307, &P18523_323};	/*Property = Svg*/
const Value V18523_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_327 = {S44, 0, &V18523_327_57316, &P18523_324};	/*Property = TouchEvents*/
const Value V18523_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_328 = {S45, 0, &V18523_328_57319, &P18523_327};	/*Property = Track*/
const Value V18523_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_329 = {S46, 0, &V18523_329_57325, &P18523_328};	/*Property = Video*/
const Value V18523_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_330 = {S47, 0, &V18523_330_57328, &P18523_329};	/*Property = Viewport*/
const Value V18523_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_332 = {S48, 0, &V18523_332_57340, &P18523_330};	/*Property = WebWorkers*/
const Value V18523_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18523_333 = {S49, 0, &V18523_333_57343, &P18523_332};	/*Property = Xhr2*/
const Profile P18523 = {"18523", &P18523_333};	/*Profile ID - 18523*/
const Value V24436_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_279 = {S1, 0, &V24436_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24436_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_282 = {S3, 0, &V24436_282_57155, &P24436_279};	/*Property = BlobBuilder*/
const Value V24436_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_350 = {S4, 0, &V24436_350_57446, &P24436_282};	/*Property = Canvas*/
const Value V24436_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_283 = {S5, 0, &V24436_283_57161, &P24436_350};	/*Property = CssBackground*/
const Value V24436_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_284 = {S6, 0, &V24436_284_57164, &P24436_283};	/*Property = CssBorderImage*/
const Value V24436_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_285 = {S7, 0, &V24436_285_57167, &P24436_284};	/*Property = CssCanvas*/
const Value V24436_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_286 = {S8, 0, &V24436_286_57170, &P24436_285};	/*Property = CssColor*/
const Value V24436_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_237 = {S9, 0, &V24436_237_55786, &P24436_286};	/*Property = CssColumn*/
const Value V24436_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_287 = {S10, 0, &V24436_287_57173, &P24436_237};	/*Property = CssFlexbox*/
const Value V24436_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_288 = {S11, 0, &V24436_288_57176, &P24436_287};	/*Property = CssFont*/
const Value V24436_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_289 = {S12, 0, &V24436_289_86899, &P24436_288};	/*Property = CssImages*/
const Value V24436_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_291 = {S13, 0, &V24436_291_57182, &P24436_289};	/*Property = CssMediaQueries*/
const Value V24436_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_292 = {S14, 0, &V24436_292_57185, &P24436_291};	/*Property = CssMinMax*/
const Value V24436_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_293 = {S15, 0, &V24436_293_57188, &P24436_292};	/*Property = CssOverflow*/
const Value V24436_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_294 = {S16, 0, &V24436_294_57191, &P24436_293};	/*Property = CssPosition*/
const Value V24436_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_295 = {S17, 0, &V24436_295_57194, &P24436_294};	/*Property = CssText*/
const Value V24436_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_297 = {S18, 0, &V24436_297_57197, &P24436_295};	/*Property = CssTransforms*/
const Value V24436_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_241 = {S19, 0, &V24436_241_55801, &P24436_297};	/*Property = CssTransitions*/
const Value V24436_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_298 = {S20, 0, &V24436_298_57208, &P24436_241};	/*Property = CssUI*/
const Value V24436_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_300 = {S21, 0, &V24436_300_57217, &P24436_298};	/*Property = DataSet*/
const Value V24436_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_301 = {S22, 0, &V24436_301_57220, &P24436_300};	/*Property = DataUrl*/
const Value V24436_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_302 = {S23, 0, &V24436_302_57223, &P24436_301};	/*Property = DeviceOrientation*/
const Value V24436_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_304 = {S24, 0, &V24436_304_57229, &P24436_302};	/*Property = FileReader*/
const Value V24436_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_305 = {S25, 0, &V24436_305_57232, &P24436_304};	/*Property = FileSaver*/
const Value V24436_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_306 = {S26, 0, &V24436_306_57238, &P24436_305};	/*Property = FileWriter*/
const Value V24436_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_307 = {S27, 0, &V24436_307_57241, &P24436_306};	/*Property = FormData*/
const Value V24436_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_309 = {S28, 0, &V24436_309_57247, &P24436_307};	/*Property = Fullscreen*/
const Value V24436_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_310 = {S29, 0, &V24436_310_57250, &P24436_309};	/*Property = GeoLocation*/
const Value V24436_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_312 = {S30, 0, &V24436_312_57256, &P24436_310};	/*Property = History*/
const Value V24436_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_314 = {S31, 0, &V24436_314_57262, &P24436_312};	/*Property = Html5*/
const Value V24436_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_313 = {S32, 0, &V24436_313_57259, &P24436_314};	/*Property = Html-Media-Capture*/
const Value V24436_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_315 = {S33, 0, &V24436_315_57265, &P24436_313};	/*Property = Iframe*/
const Value V24436_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_316 = {S34, 0, &V24436_316_57268, &P24436_315};	/*Property = IndexedDB*/
const Value V24436_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_317 = {S35, 0, &V24436_317_57271, &P24436_316};	/*Property = Json*/
const Value V24436_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24436_15 = {S37, 0, &V24436_15_28, &P24436_317};	/*Property = LayoutEngine*/
const Value V24436_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_344 = {S38, 0, &V24436_344_57274, &P24436_15};	/*Property = Masking*/
const Value V24436_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_320 = {S39, 0, &V24436_320_57292, &P24436_344};	/*Property = PostMessage*/
const Value V24436_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_321 = {S40, 0, &V24436_321_57295, &P24436_320};	/*Property = Progress*/
const Value V24436_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_322 = {S41, 0, &V24436_322_57298, &P24436_321};	/*Property = Prompts*/
const Value V24436_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_323 = {S42, 0, &V24436_323_57301, &P24436_322};	/*Property = Selector*/
const Value V24436_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_324 = {S43, 0, &V24436_324_57307, &P24436_323};	/*Property = Svg*/
const Value V24436_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_327 = {S44, 0, &V24436_327_57316, &P24436_324};	/*Property = TouchEvents*/
const Value V24436_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_328 = {S45, 0, &V24436_328_57319, &P24436_327};	/*Property = Track*/
const Value V24436_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_329 = {S46, 0, &V24436_329_57325, &P24436_328};	/*Property = Video*/
const Value V24436_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_330 = {S47, 0, &V24436_330_57328, &P24436_329};	/*Property = Viewport*/
const Value V24436_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_332 = {S48, 0, &V24436_332_57340, &P24436_330};	/*Property = WebWorkers*/
const Value V24436_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24436_333 = {S49, 0, &V24436_333_57343, &P24436_332};	/*Property = Xhr2*/
const Profile P24436 = {"24436", &P24436_333};	/*Profile ID - 24436*/
const Profile P22212 = {"22212", NULL};	/*Profile ID - 22212*/
const Profile P19829 = {"19829", NULL};	/*Profile ID - 19829*/
const Value V18438_1_1 = {S2, NULL};	/*Value = True*/
const Property P18438_1 = {S50, 0, &V18438_1_1, NULL};	/*Property = IsMobile*/
const Value V18438_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18438_5 = {S52, 0, &V18438_5_9981, &P18438_1};	/*Property = ScreenPixelsHeight*/
const Value V18438_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P18438_4 = {S54, 0, &V18438_4_37617, &P18438_5};	/*Property = ScreenPixelsWidth*/
const Profile P18438 = {"18438", &P18438_4};	/*Profile ID - 18438*/
const Value V18439_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18439_279 = {S1, 0, &V18439_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18439_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18439_282 = {S3, 0, &V18439_282_57154, &P18439_279};	/*Property = BlobBuilder*/
const Value V18439_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18439_350 = {S4, 0, &V18439_350_57444, &P18439_282};	/*Property = Canvas*/
const Value V18439_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18439_283 = {S5, 0, &V18439_283_57159, &P18439_350};	/*Property = CssBackground*/
const Value V18439_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18439_284 = {S6, 0, &V18439_284_57163, &P18439_283};	/*Property = CssBorderImage*/
const Value V18439_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18439_285 = {S7, 0, &V18439_285_57166, &P18439_284};	/*Property = CssCanvas*/
const Value V18439_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18439_286 = {S8, 0, &V18439_286_57168, &P18439_285};	/*Property = CssColor*/
const Value V18439_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18439_237 = {S9, 0, &V18439_237_55786, &P18439_286};	/*Property = CssColumn*/
const Value V18439_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18439_287 = {S10, 0, &V18439_287_57172, &P18439_237};	/*Property = CssFlexbox*/
const Value V18439_288_57174 = {S2, NULL};	/*Value = True*/
const Property P18439_288 = {S11, 0, &V18439_288_57174, &P18439_287};	/*Property = CssFont*/
const Value V18439_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18439_289 = {S12, 0, &V18439_289_57759, &P18439_288};	/*Property = CssImages*/
const Value V18439_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18439_291 = {S13, 0, &V18439_291_57180, &P18439_289};	/*Property = CssMediaQueries*/
const Value V18439_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18439_292 = {S14, 0, &V18439_292_57183, &P18439_291};	/*Property = CssMinMax*/
const Value V18439_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18439_293 = {S15, 0, &V18439_293_57187, &P18439_292};	/*Property = CssOverflow*/
const Value V18439_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18439_294 = {S16, 0, &V18439_294_57189, &P18439_293};	/*Property = CssPosition*/
const Value V18439_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18439_295 = {S17, 0, &V18439_295_57192, &P18439_294};	/*Property = CssText*/
const Value V18439_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18439_297 = {S18, 0, &V18439_297_57195, &P18439_295};	/*Property = CssTransforms*/
const Value V18439_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18439_241 = {S19, 0, &V18439_241_55799, &P18439_297};	/*Property = CssTransitions*/
const Value V18439_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18439_298 = {S20, 0, &V18439_298_57206, &P18439_241};	/*Property = CssUI*/
const Value V18439_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18439_300 = {S21, 0, &V18439_300_57216, &P18439_298};	/*Property = DataSet*/
const Value V18439_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18439_301 = {S22, 0, &V18439_301_57218, &P18439_300};	/*Property = DataUrl*/
const Value V18439_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18439_302 = {S23, 0, &V18439_302_57222, &P18439_301};	/*Property = DeviceOrientation*/
const Value V18439_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18439_304 = {S24, 0, &V18439_304_57227, &P18439_302};	/*Property = FileReader*/
const Value V18439_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18439_305 = {S25, 0, &V18439_305_57231, &P18439_304};	/*Property = FileSaver*/
const Value V18439_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18439_306 = {S26, 0, &V18439_306_57237, &P18439_305};	/*Property = FileWriter*/
const Value V18439_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18439_307 = {S27, 0, &V18439_307_57239, &P18439_306};	/*Property = FormData*/
const Value V18439_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18439_309 = {S28, 0, &V18439_309_57246, &P18439_307};	/*Property = Fullscreen*/
const Value V18439_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18439_310 = {S29, 0, &V18439_310_57248, &P18439_309};	/*Property = GeoLocation*/
const Value V18439_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18439_312 = {S30, 0, &V18439_312_57254, &P18439_310};	/*Property = History*/
const Value V18439_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18439_314 = {S31, 0, &V18439_314_57261, &P18439_312};	/*Property = Html5*/
const Value V18439_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18439_313 = {S32, 0, &V18439_313_57258, &P18439_314};	/*Property = Html-Media-Capture*/
const Value V18439_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18439_315 = {S33, 0, &V18439_315_57264, &P18439_313};	/*Property = Iframe*/
const Value V18439_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18439_316 = {S34, 0, &V18439_316_57266, &P18439_315};	/*Property = IndexedDB*/
const Value V18439_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18439_317 = {S35, 0, &V18439_317_57269, &P18439_316};	/*Property = Json*/
const Value V18439_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18439_15 = {S37, 0, &V18439_15_28, &P18439_317};	/*Property = LayoutEngine*/
const Value V18439_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18439_344 = {S38, 0, &V18439_344_57272, &P18439_15};	/*Property = Masking*/
const Value V18439_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18439_320 = {S39, 0, &V18439_320_57290, &P18439_344};	/*Property = PostMessage*/
const Value V18439_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18439_321 = {S40, 0, &V18439_321_57293, &P18439_320};	/*Property = Progress*/
const Value V18439_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18439_322 = {S41, 0, &V18439_322_57296, &P18439_321};	/*Property = Prompts*/
const Value V18439_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18439_323 = {S42, 0, &V18439_323_57299, &P18439_322};	/*Property = Selector*/
const Value V18439_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18439_324 = {S43, 0, &V18439_324_57306, &P18439_323};	/*Property = Svg*/
const Value V18439_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18439_327 = {S44, 0, &V18439_327_57315, &P18439_324};	/*Property = TouchEvents*/
const Value V18439_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18439_328 = {S45, 0, &V18439_328_57318, &P18439_327};	/*Property = Track*/
const Value V18439_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18439_329 = {S46, 0, &V18439_329_57323, &P18439_328};	/*Property = Video*/
const Value V18439_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18439_330 = {S47, 0, &V18439_330_57326, &P18439_329};	/*Property = Viewport*/
const Value V18439_332_57338 = {S2, NULL};	/*Value = True*/
const Property P18439_332 = {S48, 0, &V18439_332_57338, &P18439_330};	/*Property = WebWorkers*/
const Value V18439_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18439_333 = {S49, 0, &V18439_333_57343, &P18439_332};	/*Property = Xhr2*/
const Profile P18439 = {"18439", &P18439_333};	/*Profile ID - 18439*/
const Value V18442_1_1 = {S2, NULL};	/*Value = True*/
const Property P18442_1 = {S50, 0, &V18442_1_1, NULL};	/*Property = IsMobile*/
const Value V18442_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18442_5 = {S52, 0, &V18442_5_9981, &P18442_1};	/*Property = ScreenPixelsHeight*/
const Value V18442_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18442_4 = {S54, 0, &V18442_4_39098, &P18442_5};	/*Property = ScreenPixelsWidth*/
const Profile P18442 = {"18442", &P18442_4};	/*Profile ID - 18442*/
const Value V16052_1_1 = {S2, NULL};	/*Value = True*/
const Property P16052_1 = {S50, 0, &V16052_1_1, NULL};	/*Property = IsMobile*/
const Value V16052_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16052_5 = {S52, 0, &V16052_5_1227, &P16052_1};	/*Property = ScreenPixelsHeight*/
const Value V16052_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16052_4 = {S54, 0, &V16052_4_1229, &P16052_5};	/*Property = ScreenPixelsWidth*/
const Profile P16052 = {"16052", &P16052_4};	/*Profile ID - 16052*/
const Value V16084_1_1 = {S2, NULL};	/*Value = True*/
const Property P16084_1 = {S50, 0, &V16084_1_1, NULL};	/*Property = IsMobile*/
const Value V16084_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16084_5 = {S52, 0, &V16084_5_1227, &P16084_1};	/*Property = ScreenPixelsHeight*/
const Value V16084_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16084_4 = {S54, 0, &V16084_4_1229, &P16084_5};	/*Property = ScreenPixelsWidth*/
const Profile P16084 = {"16084", &P16084_4};	/*Profile ID - 16084*/
const Value V18560_1_1 = {S2, NULL};	/*Value = True*/
const Property P18560_1 = {S50, 0, &V18560_1_1, NULL};	/*Property = IsMobile*/
const Value V18560_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18560_5 = {S52, 0, &V18560_5_1227, &P18560_1};	/*Property = ScreenPixelsHeight*/
const Value V18560_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18560_4 = {S54, 0, &V18560_4_1229, &P18560_5};	/*Property = ScreenPixelsWidth*/
const Profile P18560 = {"18560", &P18560_4};	/*Profile ID - 18560*/
const Value V18561_1_1 = {S2, NULL};	/*Value = True*/
const Property P18561_1 = {S50, 0, &V18561_1_1, NULL};	/*Property = IsMobile*/
const Value V18561_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18561_5 = {S52, 0, &V18561_5_1227, &P18561_1};	/*Property = ScreenPixelsHeight*/
const Value V18561_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18561_4 = {S54, 0, &V18561_4_1229, &P18561_5};	/*Property = ScreenPixelsWidth*/
const Profile P18561 = {"18561", &P18561_4};	/*Profile ID - 18561*/
const Value V18562_1_1 = {S2, NULL};	/*Value = True*/
const Property P18562_1 = {S50, 0, &V18562_1_1, NULL};	/*Property = IsMobile*/
const Value V18562_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18562_5 = {S52, 0, &V18562_5_1227, &P18562_1};	/*Property = ScreenPixelsHeight*/
const Value V18562_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18562_4 = {S54, 0, &V18562_4_1229, &P18562_5};	/*Property = ScreenPixelsWidth*/
const Profile P18562 = {"18562", &P18562_4};	/*Profile ID - 18562*/
const Value V16567_1_1 = {S2, NULL};	/*Value = True*/
const Property P16567_1 = {S50, 0, &V16567_1_1, NULL};	/*Property = IsMobile*/
const Value V16567_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16567_5 = {S52, 0, &V16567_5_1227, &P16567_1};	/*Property = ScreenPixelsHeight*/
const Value V16567_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16567_4 = {S54, 0, &V16567_4_1229, &P16567_5};	/*Property = ScreenPixelsWidth*/
const Profile P16567 = {"16567", &P16567_4};	/*Profile ID - 16567*/
const Value V23071_1_1 = {S2, NULL};	/*Value = True*/
const Property P23071_1 = {S50, 0, &V23071_1_1, NULL};	/*Property = IsMobile*/
const Value V23071_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23071_5 = {S52, 0, &V23071_5_37569, &P23071_1};	/*Property = ScreenPixelsHeight*/
const Value V23071_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23071_4 = {S54, 0, &V23071_4_5645, &P23071_5};	/*Property = ScreenPixelsWidth*/
const Profile P23071 = {"23071", &P23071_4};	/*Profile ID - 23071*/
const Value V20461_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_279 = {S1, 0, &V20461_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20461_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_282 = {S3, 0, &V20461_282_57155, &P20461_279};	/*Property = BlobBuilder*/
const Value V20461_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_350 = {S4, 0, &V20461_350_57446, &P20461_282};	/*Property = Canvas*/
const Value V20461_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_283 = {S5, 0, &V20461_283_57161, &P20461_350};	/*Property = CssBackground*/
const Value V20461_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_284 = {S6, 0, &V20461_284_57164, &P20461_283};	/*Property = CssBorderImage*/
const Value V20461_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_285 = {S7, 0, &V20461_285_57167, &P20461_284};	/*Property = CssCanvas*/
const Value V20461_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_286 = {S8, 0, &V20461_286_57170, &P20461_285};	/*Property = CssColor*/
const Value V20461_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_237 = {S9, 0, &V20461_237_55786, &P20461_286};	/*Property = CssColumn*/
const Value V20461_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_287 = {S10, 0, &V20461_287_57173, &P20461_237};	/*Property = CssFlexbox*/
const Value V20461_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_288 = {S11, 0, &V20461_288_57176, &P20461_287};	/*Property = CssFont*/
const Value V20461_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_289 = {S12, 0, &V20461_289_86899, &P20461_288};	/*Property = CssImages*/
const Value V20461_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_291 = {S13, 0, &V20461_291_57182, &P20461_289};	/*Property = CssMediaQueries*/
const Value V20461_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_292 = {S14, 0, &V20461_292_57185, &P20461_291};	/*Property = CssMinMax*/
const Value V20461_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_293 = {S15, 0, &V20461_293_57188, &P20461_292};	/*Property = CssOverflow*/
const Value V20461_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_294 = {S16, 0, &V20461_294_57191, &P20461_293};	/*Property = CssPosition*/
const Value V20461_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_295 = {S17, 0, &V20461_295_57194, &P20461_294};	/*Property = CssText*/
const Value V20461_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_297 = {S18, 0, &V20461_297_57197, &P20461_295};	/*Property = CssTransforms*/
const Value V20461_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_241 = {S19, 0, &V20461_241_55801, &P20461_297};	/*Property = CssTransitions*/
const Value V20461_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_298 = {S20, 0, &V20461_298_57208, &P20461_241};	/*Property = CssUI*/
const Value V20461_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_300 = {S21, 0, &V20461_300_57217, &P20461_298};	/*Property = DataSet*/
const Value V20461_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_301 = {S22, 0, &V20461_301_57220, &P20461_300};	/*Property = DataUrl*/
const Value V20461_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_302 = {S23, 0, &V20461_302_57223, &P20461_301};	/*Property = DeviceOrientation*/
const Value V20461_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_304 = {S24, 0, &V20461_304_57229, &P20461_302};	/*Property = FileReader*/
const Value V20461_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_305 = {S25, 0, &V20461_305_57232, &P20461_304};	/*Property = FileSaver*/
const Value V20461_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_306 = {S26, 0, &V20461_306_57238, &P20461_305};	/*Property = FileWriter*/
const Value V20461_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_307 = {S27, 0, &V20461_307_57241, &P20461_306};	/*Property = FormData*/
const Value V20461_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_309 = {S28, 0, &V20461_309_57247, &P20461_307};	/*Property = Fullscreen*/
const Value V20461_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_310 = {S29, 0, &V20461_310_57250, &P20461_309};	/*Property = GeoLocation*/
const Value V20461_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_312 = {S30, 0, &V20461_312_57256, &P20461_310};	/*Property = History*/
const Value V20461_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_314 = {S31, 0, &V20461_314_57262, &P20461_312};	/*Property = Html5*/
const Value V20461_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_313 = {S32, 0, &V20461_313_57259, &P20461_314};	/*Property = Html-Media-Capture*/
const Value V20461_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_315 = {S33, 0, &V20461_315_57265, &P20461_313};	/*Property = Iframe*/
const Value V20461_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_316 = {S34, 0, &V20461_316_57268, &P20461_315};	/*Property = IndexedDB*/
const Value V20461_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_317 = {S35, 0, &V20461_317_57271, &P20461_316};	/*Property = Json*/
const Value V20461_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20461_15 = {S37, 0, &V20461_15_29, &P20461_317};	/*Property = LayoutEngine*/
const Value V20461_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_344 = {S38, 0, &V20461_344_57274, &P20461_15};	/*Property = Masking*/
const Value V20461_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_320 = {S39, 0, &V20461_320_57292, &P20461_344};	/*Property = PostMessage*/
const Value V20461_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_321 = {S40, 0, &V20461_321_57295, &P20461_320};	/*Property = Progress*/
const Value V20461_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_322 = {S41, 0, &V20461_322_57298, &P20461_321};	/*Property = Prompts*/
const Value V20461_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_323 = {S42, 0, &V20461_323_57301, &P20461_322};	/*Property = Selector*/
const Value V20461_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_324 = {S43, 0, &V20461_324_57307, &P20461_323};	/*Property = Svg*/
const Value V20461_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_327 = {S44, 0, &V20461_327_57316, &P20461_324};	/*Property = TouchEvents*/
const Value V20461_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_328 = {S45, 0, &V20461_328_57319, &P20461_327};	/*Property = Track*/
const Value V20461_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_329 = {S46, 0, &V20461_329_57325, &P20461_328};	/*Property = Video*/
const Value V20461_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_330 = {S47, 0, &V20461_330_57328, &P20461_329};	/*Property = Viewport*/
const Value V20461_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_332 = {S48, 0, &V20461_332_57340, &P20461_330};	/*Property = WebWorkers*/
const Value V20461_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20461_333 = {S49, 0, &V20461_333_57343, &P20461_332};	/*Property = Xhr2*/
const Profile P20461 = {"20461", &P20461_333};	/*Profile ID - 20461*/
const Value V23684_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_279 = {S1, 0, &V23684_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23684_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_282 = {S3, 0, &V23684_282_57155, &P23684_279};	/*Property = BlobBuilder*/
const Value V23684_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_350 = {S4, 0, &V23684_350_57446, &P23684_282};	/*Property = Canvas*/
const Value V23684_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_283 = {S5, 0, &V23684_283_57161, &P23684_350};	/*Property = CssBackground*/
const Value V23684_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_284 = {S6, 0, &V23684_284_57164, &P23684_283};	/*Property = CssBorderImage*/
const Value V23684_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_285 = {S7, 0, &V23684_285_57167, &P23684_284};	/*Property = CssCanvas*/
const Value V23684_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_286 = {S8, 0, &V23684_286_57170, &P23684_285};	/*Property = CssColor*/
const Value V23684_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_237 = {S9, 0, &V23684_237_55786, &P23684_286};	/*Property = CssColumn*/
const Value V23684_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_287 = {S10, 0, &V23684_287_57173, &P23684_237};	/*Property = CssFlexbox*/
const Value V23684_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_288 = {S11, 0, &V23684_288_57176, &P23684_287};	/*Property = CssFont*/
const Value V23684_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_289 = {S12, 0, &V23684_289_86899, &P23684_288};	/*Property = CssImages*/
const Value V23684_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_291 = {S13, 0, &V23684_291_57182, &P23684_289};	/*Property = CssMediaQueries*/
const Value V23684_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_292 = {S14, 0, &V23684_292_57185, &P23684_291};	/*Property = CssMinMax*/
const Value V23684_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_293 = {S15, 0, &V23684_293_57188, &P23684_292};	/*Property = CssOverflow*/
const Value V23684_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_294 = {S16, 0, &V23684_294_57191, &P23684_293};	/*Property = CssPosition*/
const Value V23684_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_295 = {S17, 0, &V23684_295_57194, &P23684_294};	/*Property = CssText*/
const Value V23684_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_297 = {S18, 0, &V23684_297_57197, &P23684_295};	/*Property = CssTransforms*/
const Value V23684_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_241 = {S19, 0, &V23684_241_55801, &P23684_297};	/*Property = CssTransitions*/
const Value V23684_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_298 = {S20, 0, &V23684_298_57208, &P23684_241};	/*Property = CssUI*/
const Value V23684_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_300 = {S21, 0, &V23684_300_57217, &P23684_298};	/*Property = DataSet*/
const Value V23684_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_301 = {S22, 0, &V23684_301_57220, &P23684_300};	/*Property = DataUrl*/
const Value V23684_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_302 = {S23, 0, &V23684_302_57223, &P23684_301};	/*Property = DeviceOrientation*/
const Value V23684_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_304 = {S24, 0, &V23684_304_57229, &P23684_302};	/*Property = FileReader*/
const Value V23684_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_305 = {S25, 0, &V23684_305_57232, &P23684_304};	/*Property = FileSaver*/
const Value V23684_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_306 = {S26, 0, &V23684_306_57238, &P23684_305};	/*Property = FileWriter*/
const Value V23684_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_307 = {S27, 0, &V23684_307_57241, &P23684_306};	/*Property = FormData*/
const Value V23684_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_309 = {S28, 0, &V23684_309_57247, &P23684_307};	/*Property = Fullscreen*/
const Value V23684_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_310 = {S29, 0, &V23684_310_57250, &P23684_309};	/*Property = GeoLocation*/
const Value V23684_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_312 = {S30, 0, &V23684_312_57256, &P23684_310};	/*Property = History*/
const Value V23684_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_314 = {S31, 0, &V23684_314_57262, &P23684_312};	/*Property = Html5*/
const Value V23684_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_313 = {S32, 0, &V23684_313_57259, &P23684_314};	/*Property = Html-Media-Capture*/
const Value V23684_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_315 = {S33, 0, &V23684_315_57265, &P23684_313};	/*Property = Iframe*/
const Value V23684_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_316 = {S34, 0, &V23684_316_57268, &P23684_315};	/*Property = IndexedDB*/
const Value V23684_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_317 = {S35, 0, &V23684_317_57271, &P23684_316};	/*Property = Json*/
const Value V23684_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P23684_15 = {S37, 0, &V23684_15_29, &P23684_317};	/*Property = LayoutEngine*/
const Value V23684_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_344 = {S38, 0, &V23684_344_57274, &P23684_15};	/*Property = Masking*/
const Value V23684_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_320 = {S39, 0, &V23684_320_57292, &P23684_344};	/*Property = PostMessage*/
const Value V23684_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_321 = {S40, 0, &V23684_321_57295, &P23684_320};	/*Property = Progress*/
const Value V23684_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_322 = {S41, 0, &V23684_322_57298, &P23684_321};	/*Property = Prompts*/
const Value V23684_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_323 = {S42, 0, &V23684_323_57301, &P23684_322};	/*Property = Selector*/
const Value V23684_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_324 = {S43, 0, &V23684_324_57307, &P23684_323};	/*Property = Svg*/
const Value V23684_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_327 = {S44, 0, &V23684_327_57316, &P23684_324};	/*Property = TouchEvents*/
const Value V23684_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_328 = {S45, 0, &V23684_328_57319, &P23684_327};	/*Property = Track*/
const Value V23684_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_329 = {S46, 0, &V23684_329_57325, &P23684_328};	/*Property = Video*/
const Value V23684_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_330 = {S47, 0, &V23684_330_57328, &P23684_329};	/*Property = Viewport*/
const Value V23684_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_332 = {S48, 0, &V23684_332_57340, &P23684_330};	/*Property = WebWorkers*/
const Value V23684_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23684_333 = {S49, 0, &V23684_333_57343, &P23684_332};	/*Property = Xhr2*/
const Profile P23684 = {"23684", &P23684_333};	/*Profile ID - 23684*/
const Profile P22193 = {"22193", NULL};	/*Profile ID - 22193*/
const Value V20905_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20905_279 = {S1, 0, &V20905_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20905_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20905_282 = {S3, 0, &V20905_282_57154, &P20905_279};	/*Property = BlobBuilder*/
const Value V20905_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20905_350 = {S4, 0, &V20905_350_57444, &P20905_282};	/*Property = Canvas*/
const Value V20905_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20905_283 = {S5, 0, &V20905_283_57159, &P20905_350};	/*Property = CssBackground*/
const Value V20905_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20905_284 = {S6, 0, &V20905_284_57163, &P20905_283};	/*Property = CssBorderImage*/
const Value V20905_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20905_285 = {S7, 0, &V20905_285_57166, &P20905_284};	/*Property = CssCanvas*/
const Value V20905_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20905_286 = {S8, 0, &V20905_286_57168, &P20905_285};	/*Property = CssColor*/
const Value V20905_237_55785 = {S0, NULL};	/*Value = False*/
const Property P20905_237 = {S9, 0, &V20905_237_55785, &P20905_286};	/*Property = CssColumn*/
const Value V20905_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20905_287 = {S10, 0, &V20905_287_57172, &P20905_237};	/*Property = CssFlexbox*/
const Value V20905_288_57175 = {S0, NULL};	/*Value = False*/
const Property P20905_288 = {S11, 0, &V20905_288_57175, &P20905_287};	/*Property = CssFont*/
const Value V20905_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20905_289 = {S12, 0, &V20905_289_57759, &P20905_288};	/*Property = CssImages*/
const Value V20905_291_57181 = {S0, NULL};	/*Value = False*/
const Property P20905_291 = {S13, 0, &V20905_291_57181, &P20905_289};	/*Property = CssMediaQueries*/
const Value V20905_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20905_292 = {S14, 0, &V20905_292_57183, &P20905_291};	/*Property = CssMinMax*/
const Value V20905_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20905_293 = {S15, 0, &V20905_293_57187, &P20905_292};	/*Property = CssOverflow*/
const Value V20905_294_57190 = {S0, NULL};	/*Value = False*/
const Property P20905_294 = {S16, 0, &V20905_294_57190, &P20905_293};	/*Property = CssPosition*/
const Value V20905_295_57192 = {S2, NULL};	/*Value = True*/
const Property P20905_295 = {S17, 0, &V20905_295_57192, &P20905_294};	/*Property = CssText*/
const Value V20905_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20905_297 = {S18, 0, &V20905_297_57195, &P20905_295};	/*Property = CssTransforms*/
const Value V20905_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20905_241 = {S19, 0, &V20905_241_55799, &P20905_297};	/*Property = CssTransitions*/
const Value V20905_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20905_298 = {S20, 0, &V20905_298_57206, &P20905_241};	/*Property = CssUI*/
const Value V20905_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20905_300 = {S21, 0, &V20905_300_57216, &P20905_298};	/*Property = DataSet*/
const Value V20905_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20905_301 = {S22, 0, &V20905_301_57218, &P20905_300};	/*Property = DataUrl*/
const Value V20905_302_57222 = {S0, NULL};	/*Value = False*/
const Property P20905_302 = {S23, 0, &V20905_302_57222, &P20905_301};	/*Property = DeviceOrientation*/
const Value V20905_304_57228 = {S0, NULL};	/*Value = False*/
const Property P20905_304 = {S24, 0, &V20905_304_57228, &P20905_302};	/*Property = FileReader*/
const Value V20905_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20905_305 = {S25, 0, &V20905_305_57231, &P20905_304};	/*Property = FileSaver*/
const Value V20905_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20905_306 = {S26, 0, &V20905_306_57237, &P20905_305};	/*Property = FileWriter*/
const Value V20905_307_57240 = {S0, NULL};	/*Value = False*/
const Property P20905_307 = {S27, 0, &V20905_307_57240, &P20905_306};	/*Property = FormData*/
const Value V20905_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20905_309 = {S28, 0, &V20905_309_57246, &P20905_307};	/*Property = Fullscreen*/
const Value V20905_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20905_310 = {S29, 0, &V20905_310_57248, &P20905_309};	/*Property = GeoLocation*/
const Value V20905_312_57255 = {S0, NULL};	/*Value = False*/
const Property P20905_312 = {S30, 0, &V20905_312_57255, &P20905_310};	/*Property = History*/
const Value V20905_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20905_314 = {S31, 0, &V20905_314_57261, &P20905_312};	/*Property = Html5*/
const Value V20905_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20905_313 = {S32, 0, &V20905_313_57258, &P20905_314};	/*Property = Html-Media-Capture*/
const Value V20905_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20905_315 = {S33, 0, &V20905_315_57264, &P20905_313};	/*Property = Iframe*/
const Value V20905_316_57267 = {S0, NULL};	/*Value = False*/
const Property P20905_316 = {S34, 0, &V20905_316_57267, &P20905_315};	/*Property = IndexedDB*/
const Value V20905_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20905_317 = {S35, 0, &V20905_317_57269, &P20905_316};	/*Property = Json*/
const Value V20905_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P20905_15 = {S37, 0, &V20905_15_30, &P20905_317};	/*Property = LayoutEngine*/
const Value V20905_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20905_344 = {S38, 0, &V20905_344_57272, &P20905_15};	/*Property = Masking*/
const Value V20905_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20905_320 = {S39, 0, &V20905_320_57290, &P20905_344};	/*Property = PostMessage*/
const Value V20905_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20905_321 = {S40, 0, &V20905_321_57293, &P20905_320};	/*Property = Progress*/
const Value V20905_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20905_322 = {S41, 0, &V20905_322_57296, &P20905_321};	/*Property = Prompts*/
const Value V20905_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20905_323 = {S42, 0, &V20905_323_57299, &P20905_322};	/*Property = Selector*/
const Value V20905_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20905_324 = {S43, 0, &V20905_324_57306, &P20905_323};	/*Property = Svg*/
const Value V20905_327_57315 = {S0, NULL};	/*Value = False*/
const Property P20905_327 = {S44, 0, &V20905_327_57315, &P20905_324};	/*Property = TouchEvents*/
const Value V20905_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20905_328 = {S45, 0, &V20905_328_57318, &P20905_327};	/*Property = Track*/
const Value V20905_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20905_329 = {S46, 0, &V20905_329_57323, &P20905_328};	/*Property = Video*/
const Value V20905_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20905_330 = {S47, 0, &V20905_330_57326, &P20905_329};	/*Property = Viewport*/
const Value V20905_332_57339 = {S0, NULL};	/*Value = False*/
const Property P20905_332 = {S48, 0, &V20905_332_57339, &P20905_330};	/*Property = WebWorkers*/
const Value V20905_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20905_333 = {S49, 0, &V20905_333_57342, &P20905_332};	/*Property = Xhr2*/
const Profile P20905 = {"20905", &P20905_333};	/*Profile ID - 20905*/
const Value V24725_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24725_15 = {S37, 0, &V24725_15_28, NULL};	/*Property = LayoutEngine*/
const Profile P24725 = {"24725", &P24725_15};	/*Profile ID - 24725*/
const Value V22241_1_1 = {S2, NULL};	/*Value = True*/
const Property P22241_1 = {S50, 0, &V22241_1_1, NULL};	/*Property = IsMobile*/
const Value V22241_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22241_5 = {S52, 0, &V22241_5_37569, &P22241_1};	/*Property = ScreenPixelsHeight*/
const Value V22241_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22241_4 = {S54, 0, &V22241_4_5645, &P22241_5};	/*Property = ScreenPixelsWidth*/
const Profile P22241 = {"22241", &P22241_4};	/*Profile ID - 22241*/
const Value V18362_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_279 = {S1, 0, &V18362_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18362_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_282 = {S3, 0, &V18362_282_57155, &P18362_279};	/*Property = BlobBuilder*/
const Value V18362_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_350 = {S4, 0, &V18362_350_57446, &P18362_282};	/*Property = Canvas*/
const Value V18362_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_283 = {S5, 0, &V18362_283_57161, &P18362_350};	/*Property = CssBackground*/
const Value V18362_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_284 = {S6, 0, &V18362_284_57164, &P18362_283};	/*Property = CssBorderImage*/
const Value V18362_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_285 = {S7, 0, &V18362_285_57167, &P18362_284};	/*Property = CssCanvas*/
const Value V18362_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_286 = {S8, 0, &V18362_286_57170, &P18362_285};	/*Property = CssColor*/
const Value V18362_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_237 = {S9, 0, &V18362_237_55786, &P18362_286};	/*Property = CssColumn*/
const Value V18362_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_287 = {S10, 0, &V18362_287_57173, &P18362_237};	/*Property = CssFlexbox*/
const Value V18362_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_288 = {S11, 0, &V18362_288_57176, &P18362_287};	/*Property = CssFont*/
const Value V18362_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_289 = {S12, 0, &V18362_289_86899, &P18362_288};	/*Property = CssImages*/
const Value V18362_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_291 = {S13, 0, &V18362_291_57182, &P18362_289};	/*Property = CssMediaQueries*/
const Value V18362_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_292 = {S14, 0, &V18362_292_57185, &P18362_291};	/*Property = CssMinMax*/
const Value V18362_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_293 = {S15, 0, &V18362_293_57188, &P18362_292};	/*Property = CssOverflow*/
const Value V18362_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_294 = {S16, 0, &V18362_294_57191, &P18362_293};	/*Property = CssPosition*/
const Value V18362_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_295 = {S17, 0, &V18362_295_57194, &P18362_294};	/*Property = CssText*/
const Value V18362_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_297 = {S18, 0, &V18362_297_57197, &P18362_295};	/*Property = CssTransforms*/
const Value V18362_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_241 = {S19, 0, &V18362_241_55801, &P18362_297};	/*Property = CssTransitions*/
const Value V18362_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_298 = {S20, 0, &V18362_298_57208, &P18362_241};	/*Property = CssUI*/
const Value V18362_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_300 = {S21, 0, &V18362_300_57217, &P18362_298};	/*Property = DataSet*/
const Value V18362_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_301 = {S22, 0, &V18362_301_57220, &P18362_300};	/*Property = DataUrl*/
const Value V18362_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_302 = {S23, 0, &V18362_302_57223, &P18362_301};	/*Property = DeviceOrientation*/
const Value V18362_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_304 = {S24, 0, &V18362_304_57229, &P18362_302};	/*Property = FileReader*/
const Value V18362_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_305 = {S25, 0, &V18362_305_57232, &P18362_304};	/*Property = FileSaver*/
const Value V18362_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_306 = {S26, 0, &V18362_306_57238, &P18362_305};	/*Property = FileWriter*/
const Value V18362_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_307 = {S27, 0, &V18362_307_57241, &P18362_306};	/*Property = FormData*/
const Value V18362_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_309 = {S28, 0, &V18362_309_57247, &P18362_307};	/*Property = Fullscreen*/
const Value V18362_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_310 = {S29, 0, &V18362_310_57250, &P18362_309};	/*Property = GeoLocation*/
const Value V18362_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_312 = {S30, 0, &V18362_312_57256, &P18362_310};	/*Property = History*/
const Value V18362_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_314 = {S31, 0, &V18362_314_57262, &P18362_312};	/*Property = Html5*/
const Value V18362_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_313 = {S32, 0, &V18362_313_57259, &P18362_314};	/*Property = Html-Media-Capture*/
const Value V18362_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_315 = {S33, 0, &V18362_315_57265, &P18362_313};	/*Property = Iframe*/
const Value V18362_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_316 = {S34, 0, &V18362_316_57268, &P18362_315};	/*Property = IndexedDB*/
const Value V18362_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_317 = {S35, 0, &V18362_317_57271, &P18362_316};	/*Property = Json*/
const Value V18362_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18362_15 = {S37, 0, &V18362_15_29, &P18362_317};	/*Property = LayoutEngine*/
const Value V18362_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_344 = {S38, 0, &V18362_344_57274, &P18362_15};	/*Property = Masking*/
const Value V18362_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_320 = {S39, 0, &V18362_320_57292, &P18362_344};	/*Property = PostMessage*/
const Value V18362_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_321 = {S40, 0, &V18362_321_57295, &P18362_320};	/*Property = Progress*/
const Value V18362_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_322 = {S41, 0, &V18362_322_57298, &P18362_321};	/*Property = Prompts*/
const Value V18362_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_323 = {S42, 0, &V18362_323_57301, &P18362_322};	/*Property = Selector*/
const Value V18362_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_324 = {S43, 0, &V18362_324_57307, &P18362_323};	/*Property = Svg*/
const Value V18362_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_327 = {S44, 0, &V18362_327_57316, &P18362_324};	/*Property = TouchEvents*/
const Value V18362_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_328 = {S45, 0, &V18362_328_57319, &P18362_327};	/*Property = Track*/
const Value V18362_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_329 = {S46, 0, &V18362_329_57325, &P18362_328};	/*Property = Video*/
const Value V18362_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_330 = {S47, 0, &V18362_330_57328, &P18362_329};	/*Property = Viewport*/
const Value V18362_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_332 = {S48, 0, &V18362_332_57340, &P18362_330};	/*Property = WebWorkers*/
const Value V18362_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18362_333 = {S49, 0, &V18362_333_57343, &P18362_332};	/*Property = Xhr2*/
const Profile P18362 = {"18362", &P18362_333};	/*Profile ID - 18362*/
const Value V22509_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22509_279 = {S1, 0, &V22509_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22509_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22509_282 = {S3, 0, &V22509_282_57154, &P22509_279};	/*Property = BlobBuilder*/
const Value V22509_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22509_350 = {S4, 0, &V22509_350_57444, &P22509_282};	/*Property = Canvas*/
const Value V22509_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22509_283 = {S5, 0, &V22509_283_57159, &P22509_350};	/*Property = CssBackground*/
const Value V22509_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22509_284 = {S6, 0, &V22509_284_57163, &P22509_283};	/*Property = CssBorderImage*/
const Value V22509_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22509_285 = {S7, 0, &V22509_285_57166, &P22509_284};	/*Property = CssCanvas*/
const Value V22509_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22509_286 = {S8, 0, &V22509_286_57168, &P22509_285};	/*Property = CssColor*/
const Value V22509_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22509_237 = {S9, 0, &V22509_237_55784, &P22509_286};	/*Property = CssColumn*/
const Value V22509_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22509_287 = {S10, 0, &V22509_287_57172, &P22509_237};	/*Property = CssFlexbox*/
const Value V22509_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22509_288 = {S11, 0, &V22509_288_57175, &P22509_287};	/*Property = CssFont*/
const Value V22509_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22509_289 = {S12, 0, &V22509_289_57759, &P22509_288};	/*Property = CssImages*/
const Value V22509_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22509_291 = {S13, 0, &V22509_291_57181, &P22509_289};	/*Property = CssMediaQueries*/
const Value V22509_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22509_292 = {S14, 0, &V22509_292_57183, &P22509_291};	/*Property = CssMinMax*/
const Value V22509_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22509_293 = {S15, 0, &V22509_293_57187, &P22509_292};	/*Property = CssOverflow*/
const Value V22509_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22509_294 = {S16, 0, &V22509_294_57190, &P22509_293};	/*Property = CssPosition*/
const Value V22509_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22509_295 = {S17, 0, &V22509_295_57193, &P22509_294};	/*Property = CssText*/
const Value V22509_297_57196 = {S0, NULL};	/*Value = False*/
const Property P22509_297 = {S18, 0, &V22509_297_57196, &P22509_295};	/*Property = CssTransforms*/
const Value V22509_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22509_241 = {S19, 0, &V22509_241_55799, &P22509_297};	/*Property = CssTransitions*/
const Value V22509_298_57207 = {S0, NULL};	/*Value = False*/
const Property P22509_298 = {S20, 0, &V22509_298_57207, &P22509_241};	/*Property = CssUI*/
const Value V22509_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22509_300 = {S21, 0, &V22509_300_57216, &P22509_298};	/*Property = DataSet*/
const Value V22509_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22509_301 = {S22, 0, &V22509_301_57218, &P22509_300};	/*Property = DataUrl*/
const Value V22509_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22509_302 = {S23, 0, &V22509_302_57222, &P22509_301};	/*Property = DeviceOrientation*/
const Value V22509_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22509_304 = {S24, 0, &V22509_304_57228, &P22509_302};	/*Property = FileReader*/
const Value V22509_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22509_305 = {S25, 0, &V22509_305_57231, &P22509_304};	/*Property = FileSaver*/
const Value V22509_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22509_306 = {S26, 0, &V22509_306_57237, &P22509_305};	/*Property = FileWriter*/
const Value V22509_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22509_307 = {S27, 0, &V22509_307_57240, &P22509_306};	/*Property = FormData*/
const Value V22509_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22509_309 = {S28, 0, &V22509_309_57246, &P22509_307};	/*Property = Fullscreen*/
const Value V22509_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22509_310 = {S29, 0, &V22509_310_57248, &P22509_309};	/*Property = GeoLocation*/
const Value V22509_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22509_312 = {S30, 0, &V22509_312_57255, &P22509_310};	/*Property = History*/
const Value V22509_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22509_314 = {S31, 0, &V22509_314_57261, &P22509_312};	/*Property = Html5*/
const Value V22509_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22509_313 = {S32, 0, &V22509_313_57258, &P22509_314};	/*Property = Html-Media-Capture*/
const Value V22509_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22509_315 = {S33, 0, &V22509_315_57264, &P22509_313};	/*Property = Iframe*/
const Value V22509_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22509_316 = {S34, 0, &V22509_316_57267, &P22509_315};	/*Property = IndexedDB*/
const Value V22509_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22509_317 = {S35, 0, &V22509_317_57269, &P22509_316};	/*Property = Json*/
const Value V22509_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22509_15 = {S37, 0, &V22509_15_29, &P22509_317};	/*Property = LayoutEngine*/
const Value V22509_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22509_344 = {S38, 0, &V22509_344_57272, &P22509_15};	/*Property = Masking*/
const Value V22509_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22509_320 = {S39, 0, &V22509_320_57290, &P22509_344};	/*Property = PostMessage*/
const Value V22509_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22509_321 = {S40, 0, &V22509_321_57293, &P22509_320};	/*Property = Progress*/
const Value V22509_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22509_322 = {S41, 0, &V22509_322_57296, &P22509_321};	/*Property = Prompts*/
const Value V22509_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22509_323 = {S42, 0, &V22509_323_57299, &P22509_322};	/*Property = Selector*/
const Value V22509_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22509_324 = {S43, 0, &V22509_324_57306, &P22509_323};	/*Property = Svg*/
const Value V22509_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22509_327 = {S44, 0, &V22509_327_57315, &P22509_324};	/*Property = TouchEvents*/
const Value V22509_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22509_328 = {S45, 0, &V22509_328_57318, &P22509_327};	/*Property = Track*/
const Value V22509_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22509_329 = {S46, 0, &V22509_329_57323, &P22509_328};	/*Property = Video*/
const Value V22509_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22509_330 = {S47, 0, &V22509_330_57326, &P22509_329};	/*Property = Viewport*/
const Value V22509_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22509_332 = {S48, 0, &V22509_332_57339, &P22509_330};	/*Property = WebWorkers*/
const Value V22509_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22509_333 = {S49, 0, &V22509_333_57342, &P22509_332};	/*Property = Xhr2*/
const Profile P22509 = {"22509", &P22509_333};	/*Profile ID - 22509*/
const Value V21675_1_1 = {S2, NULL};	/*Value = True*/
const Property P21675_1 = {S50, 0, &V21675_1_1, NULL};	/*Property = IsMobile*/
const Value V21675_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21675_5 = {S52, 0, &V21675_5_1277, &P21675_1};	/*Property = ScreenPixelsHeight*/
const Value V21675_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21675_4 = {S54, 0, &V21675_4_1279, &P21675_5};	/*Property = ScreenPixelsWidth*/
const Profile P21675 = {"21675", &P21675_4};	/*Profile ID - 21675*/
const Value V20430_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_279 = {S1, 0, &V20430_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20430_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_282 = {S3, 0, &V20430_282_57155, &P20430_279};	/*Property = BlobBuilder*/
const Value V20430_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_350 = {S4, 0, &V20430_350_57446, &P20430_282};	/*Property = Canvas*/
const Value V20430_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_283 = {S5, 0, &V20430_283_57161, &P20430_350};	/*Property = CssBackground*/
const Value V20430_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_284 = {S6, 0, &V20430_284_57164, &P20430_283};	/*Property = CssBorderImage*/
const Value V20430_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_285 = {S7, 0, &V20430_285_57167, &P20430_284};	/*Property = CssCanvas*/
const Value V20430_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_286 = {S8, 0, &V20430_286_57170, &P20430_285};	/*Property = CssColor*/
const Value V20430_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_237 = {S9, 0, &V20430_237_55786, &P20430_286};	/*Property = CssColumn*/
const Value V20430_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_287 = {S10, 0, &V20430_287_57173, &P20430_237};	/*Property = CssFlexbox*/
const Value V20430_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_288 = {S11, 0, &V20430_288_57176, &P20430_287};	/*Property = CssFont*/
const Value V20430_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_289 = {S12, 0, &V20430_289_86899, &P20430_288};	/*Property = CssImages*/
const Value V20430_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_291 = {S13, 0, &V20430_291_57182, &P20430_289};	/*Property = CssMediaQueries*/
const Value V20430_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_292 = {S14, 0, &V20430_292_57185, &P20430_291};	/*Property = CssMinMax*/
const Value V20430_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_293 = {S15, 0, &V20430_293_57188, &P20430_292};	/*Property = CssOverflow*/
const Value V20430_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_294 = {S16, 0, &V20430_294_57191, &P20430_293};	/*Property = CssPosition*/
const Value V20430_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_295 = {S17, 0, &V20430_295_57194, &P20430_294};	/*Property = CssText*/
const Value V20430_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_297 = {S18, 0, &V20430_297_57197, &P20430_295};	/*Property = CssTransforms*/
const Value V20430_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_241 = {S19, 0, &V20430_241_55801, &P20430_297};	/*Property = CssTransitions*/
const Value V20430_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_298 = {S20, 0, &V20430_298_57208, &P20430_241};	/*Property = CssUI*/
const Value V20430_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_300 = {S21, 0, &V20430_300_57217, &P20430_298};	/*Property = DataSet*/
const Value V20430_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_301 = {S22, 0, &V20430_301_57220, &P20430_300};	/*Property = DataUrl*/
const Value V20430_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_302 = {S23, 0, &V20430_302_57223, &P20430_301};	/*Property = DeviceOrientation*/
const Value V20430_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_304 = {S24, 0, &V20430_304_57229, &P20430_302};	/*Property = FileReader*/
const Value V20430_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_305 = {S25, 0, &V20430_305_57232, &P20430_304};	/*Property = FileSaver*/
const Value V20430_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_306 = {S26, 0, &V20430_306_57238, &P20430_305};	/*Property = FileWriter*/
const Value V20430_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_307 = {S27, 0, &V20430_307_57241, &P20430_306};	/*Property = FormData*/
const Value V20430_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_309 = {S28, 0, &V20430_309_57247, &P20430_307};	/*Property = Fullscreen*/
const Value V20430_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_310 = {S29, 0, &V20430_310_57250, &P20430_309};	/*Property = GeoLocation*/
const Value V20430_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_312 = {S30, 0, &V20430_312_57256, &P20430_310};	/*Property = History*/
const Value V20430_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_314 = {S31, 0, &V20430_314_57262, &P20430_312};	/*Property = Html5*/
const Value V20430_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_313 = {S32, 0, &V20430_313_57259, &P20430_314};	/*Property = Html-Media-Capture*/
const Value V20430_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_315 = {S33, 0, &V20430_315_57265, &P20430_313};	/*Property = Iframe*/
const Value V20430_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_316 = {S34, 0, &V20430_316_57268, &P20430_315};	/*Property = IndexedDB*/
const Value V20430_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_317 = {S35, 0, &V20430_317_57271, &P20430_316};	/*Property = Json*/
const Value V20430_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20430_15 = {S37, 0, &V20430_15_29, &P20430_317};	/*Property = LayoutEngine*/
const Value V20430_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_344 = {S38, 0, &V20430_344_57274, &P20430_15};	/*Property = Masking*/
const Value V20430_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_320 = {S39, 0, &V20430_320_57292, &P20430_344};	/*Property = PostMessage*/
const Value V20430_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_321 = {S40, 0, &V20430_321_57295, &P20430_320};	/*Property = Progress*/
const Value V20430_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_322 = {S41, 0, &V20430_322_57298, &P20430_321};	/*Property = Prompts*/
const Value V20430_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_323 = {S42, 0, &V20430_323_57301, &P20430_322};	/*Property = Selector*/
const Value V20430_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_324 = {S43, 0, &V20430_324_57307, &P20430_323};	/*Property = Svg*/
const Value V20430_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_327 = {S44, 0, &V20430_327_57316, &P20430_324};	/*Property = TouchEvents*/
const Value V20430_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_328 = {S45, 0, &V20430_328_57319, &P20430_327};	/*Property = Track*/
const Value V20430_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_329 = {S46, 0, &V20430_329_57325, &P20430_328};	/*Property = Video*/
const Value V20430_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_330 = {S47, 0, &V20430_330_57328, &P20430_329};	/*Property = Viewport*/
const Value V20430_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_332 = {S48, 0, &V20430_332_57340, &P20430_330};	/*Property = WebWorkers*/
const Value V20430_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20430_333 = {S49, 0, &V20430_333_57343, &P20430_332};	/*Property = Xhr2*/
const Profile P20430 = {"20430", &P20430_333};	/*Profile ID - 20430*/
const Value V22891_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_279 = {S1, 0, &V22891_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22891_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_282 = {S3, 0, &V22891_282_57155, &P22891_279};	/*Property = BlobBuilder*/
const Value V22891_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_350 = {S4, 0, &V22891_350_57446, &P22891_282};	/*Property = Canvas*/
const Value V22891_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_283 = {S5, 0, &V22891_283_57161, &P22891_350};	/*Property = CssBackground*/
const Value V22891_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_284 = {S6, 0, &V22891_284_57164, &P22891_283};	/*Property = CssBorderImage*/
const Value V22891_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_285 = {S7, 0, &V22891_285_57167, &P22891_284};	/*Property = CssCanvas*/
const Value V22891_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_286 = {S8, 0, &V22891_286_57170, &P22891_285};	/*Property = CssColor*/
const Value V22891_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_237 = {S9, 0, &V22891_237_55786, &P22891_286};	/*Property = CssColumn*/
const Value V22891_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_287 = {S10, 0, &V22891_287_57173, &P22891_237};	/*Property = CssFlexbox*/
const Value V22891_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_288 = {S11, 0, &V22891_288_57176, &P22891_287};	/*Property = CssFont*/
const Value V22891_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_289 = {S12, 0, &V22891_289_86899, &P22891_288};	/*Property = CssImages*/
const Value V22891_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_291 = {S13, 0, &V22891_291_57182, &P22891_289};	/*Property = CssMediaQueries*/
const Value V22891_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_292 = {S14, 0, &V22891_292_57185, &P22891_291};	/*Property = CssMinMax*/
const Value V22891_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_293 = {S15, 0, &V22891_293_57188, &P22891_292};	/*Property = CssOverflow*/
const Value V22891_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_294 = {S16, 0, &V22891_294_57191, &P22891_293};	/*Property = CssPosition*/
const Value V22891_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_295 = {S17, 0, &V22891_295_57194, &P22891_294};	/*Property = CssText*/
const Value V22891_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_297 = {S18, 0, &V22891_297_57197, &P22891_295};	/*Property = CssTransforms*/
const Value V22891_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_241 = {S19, 0, &V22891_241_55801, &P22891_297};	/*Property = CssTransitions*/
const Value V22891_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_298 = {S20, 0, &V22891_298_57208, &P22891_241};	/*Property = CssUI*/
const Value V22891_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_300 = {S21, 0, &V22891_300_57217, &P22891_298};	/*Property = DataSet*/
const Value V22891_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_301 = {S22, 0, &V22891_301_57220, &P22891_300};	/*Property = DataUrl*/
const Value V22891_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_302 = {S23, 0, &V22891_302_57223, &P22891_301};	/*Property = DeviceOrientation*/
const Value V22891_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_304 = {S24, 0, &V22891_304_57229, &P22891_302};	/*Property = FileReader*/
const Value V22891_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_305 = {S25, 0, &V22891_305_57232, &P22891_304};	/*Property = FileSaver*/
const Value V22891_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_306 = {S26, 0, &V22891_306_57238, &P22891_305};	/*Property = FileWriter*/
const Value V22891_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_307 = {S27, 0, &V22891_307_57241, &P22891_306};	/*Property = FormData*/
const Value V22891_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_309 = {S28, 0, &V22891_309_57247, &P22891_307};	/*Property = Fullscreen*/
const Value V22891_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_310 = {S29, 0, &V22891_310_57250, &P22891_309};	/*Property = GeoLocation*/
const Value V22891_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_312 = {S30, 0, &V22891_312_57256, &P22891_310};	/*Property = History*/
const Value V22891_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_314 = {S31, 0, &V22891_314_57262, &P22891_312};	/*Property = Html5*/
const Value V22891_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_313 = {S32, 0, &V22891_313_57259, &P22891_314};	/*Property = Html-Media-Capture*/
const Value V22891_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_315 = {S33, 0, &V22891_315_57265, &P22891_313};	/*Property = Iframe*/
const Value V22891_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_316 = {S34, 0, &V22891_316_57268, &P22891_315};	/*Property = IndexedDB*/
const Value V22891_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_317 = {S35, 0, &V22891_317_57271, &P22891_316};	/*Property = Json*/
const Value V22891_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_15 = {S37, 0, &V22891_15_72, &P22891_317};	/*Property = LayoutEngine*/
const Value V22891_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_344 = {S38, 0, &V22891_344_57274, &P22891_15};	/*Property = Masking*/
const Value V22891_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_320 = {S39, 0, &V22891_320_57292, &P22891_344};	/*Property = PostMessage*/
const Value V22891_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_321 = {S40, 0, &V22891_321_57295, &P22891_320};	/*Property = Progress*/
const Value V22891_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_322 = {S41, 0, &V22891_322_57298, &P22891_321};	/*Property = Prompts*/
const Value V22891_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_323 = {S42, 0, &V22891_323_57301, &P22891_322};	/*Property = Selector*/
const Value V22891_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_324 = {S43, 0, &V22891_324_57307, &P22891_323};	/*Property = Svg*/
const Value V22891_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_327 = {S44, 0, &V22891_327_57316, &P22891_324};	/*Property = TouchEvents*/
const Value V22891_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_328 = {S45, 0, &V22891_328_57319, &P22891_327};	/*Property = Track*/
const Value V22891_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_329 = {S46, 0, &V22891_329_57325, &P22891_328};	/*Property = Video*/
const Value V22891_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_330 = {S47, 0, &V22891_330_57328, &P22891_329};	/*Property = Viewport*/
const Value V22891_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_332 = {S48, 0, &V22891_332_57340, &P22891_330};	/*Property = WebWorkers*/
const Value V22891_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22891_333 = {S49, 0, &V22891_333_57343, &P22891_332};	/*Property = Xhr2*/
const Profile P22891 = {"22891", &P22891_333};	/*Profile ID - 22891*/
const Value V23735_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_279 = {S1, 0, &V23735_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23735_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_282 = {S3, 0, &V23735_282_57155, &P23735_279};	/*Property = BlobBuilder*/
const Value V23735_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_350 = {S4, 0, &V23735_350_57446, &P23735_282};	/*Property = Canvas*/
const Value V23735_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_283 = {S5, 0, &V23735_283_57161, &P23735_350};	/*Property = CssBackground*/
const Value V23735_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_284 = {S6, 0, &V23735_284_57164, &P23735_283};	/*Property = CssBorderImage*/
const Value V23735_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_285 = {S7, 0, &V23735_285_57167, &P23735_284};	/*Property = CssCanvas*/
const Value V23735_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_286 = {S8, 0, &V23735_286_57170, &P23735_285};	/*Property = CssColor*/
const Value V23735_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_237 = {S9, 0, &V23735_237_55786, &P23735_286};	/*Property = CssColumn*/
const Value V23735_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_287 = {S10, 0, &V23735_287_57173, &P23735_237};	/*Property = CssFlexbox*/
const Value V23735_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_288 = {S11, 0, &V23735_288_57176, &P23735_287};	/*Property = CssFont*/
const Value V23735_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_289 = {S12, 0, &V23735_289_86899, &P23735_288};	/*Property = CssImages*/
const Value V23735_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_291 = {S13, 0, &V23735_291_57182, &P23735_289};	/*Property = CssMediaQueries*/
const Value V23735_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_292 = {S14, 0, &V23735_292_57185, &P23735_291};	/*Property = CssMinMax*/
const Value V23735_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_293 = {S15, 0, &V23735_293_57188, &P23735_292};	/*Property = CssOverflow*/
const Value V23735_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_294 = {S16, 0, &V23735_294_57191, &P23735_293};	/*Property = CssPosition*/
const Value V23735_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_295 = {S17, 0, &V23735_295_57194, &P23735_294};	/*Property = CssText*/
const Value V23735_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_297 = {S18, 0, &V23735_297_57197, &P23735_295};	/*Property = CssTransforms*/
const Value V23735_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_241 = {S19, 0, &V23735_241_55801, &P23735_297};	/*Property = CssTransitions*/
const Value V23735_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_298 = {S20, 0, &V23735_298_57208, &P23735_241};	/*Property = CssUI*/
const Value V23735_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_300 = {S21, 0, &V23735_300_57217, &P23735_298};	/*Property = DataSet*/
const Value V23735_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_301 = {S22, 0, &V23735_301_57220, &P23735_300};	/*Property = DataUrl*/
const Value V23735_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_302 = {S23, 0, &V23735_302_57223, &P23735_301};	/*Property = DeviceOrientation*/
const Value V23735_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_304 = {S24, 0, &V23735_304_57229, &P23735_302};	/*Property = FileReader*/
const Value V23735_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_305 = {S25, 0, &V23735_305_57232, &P23735_304};	/*Property = FileSaver*/
const Value V23735_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_306 = {S26, 0, &V23735_306_57238, &P23735_305};	/*Property = FileWriter*/
const Value V23735_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_307 = {S27, 0, &V23735_307_57241, &P23735_306};	/*Property = FormData*/
const Value V23735_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_309 = {S28, 0, &V23735_309_57247, &P23735_307};	/*Property = Fullscreen*/
const Value V23735_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_310 = {S29, 0, &V23735_310_57250, &P23735_309};	/*Property = GeoLocation*/
const Value V23735_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_312 = {S30, 0, &V23735_312_57256, &P23735_310};	/*Property = History*/
const Value V23735_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_314 = {S31, 0, &V23735_314_57262, &P23735_312};	/*Property = Html5*/
const Value V23735_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_313 = {S32, 0, &V23735_313_57259, &P23735_314};	/*Property = Html-Media-Capture*/
const Value V23735_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_315 = {S33, 0, &V23735_315_57265, &P23735_313};	/*Property = Iframe*/
const Value V23735_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_316 = {S34, 0, &V23735_316_57268, &P23735_315};	/*Property = IndexedDB*/
const Value V23735_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_317 = {S35, 0, &V23735_317_57271, &P23735_316};	/*Property = Json*/
const Value V23735_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P23735_15 = {S37, 0, &V23735_15_28, &P23735_317};	/*Property = LayoutEngine*/
const Value V23735_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_344 = {S38, 0, &V23735_344_57274, &P23735_15};	/*Property = Masking*/
const Value V23735_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_320 = {S39, 0, &V23735_320_57292, &P23735_344};	/*Property = PostMessage*/
const Value V23735_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_321 = {S40, 0, &V23735_321_57295, &P23735_320};	/*Property = Progress*/
const Value V23735_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_322 = {S41, 0, &V23735_322_57298, &P23735_321};	/*Property = Prompts*/
const Value V23735_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_323 = {S42, 0, &V23735_323_57301, &P23735_322};	/*Property = Selector*/
const Value V23735_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_324 = {S43, 0, &V23735_324_57307, &P23735_323};	/*Property = Svg*/
const Value V23735_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_327 = {S44, 0, &V23735_327_57316, &P23735_324};	/*Property = TouchEvents*/
const Value V23735_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_328 = {S45, 0, &V23735_328_57319, &P23735_327};	/*Property = Track*/
const Value V23735_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_329 = {S46, 0, &V23735_329_57325, &P23735_328};	/*Property = Video*/
const Value V23735_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_330 = {S47, 0, &V23735_330_57328, &P23735_329};	/*Property = Viewport*/
const Value V23735_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_332 = {S48, 0, &V23735_332_57340, &P23735_330};	/*Property = WebWorkers*/
const Value V23735_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23735_333 = {S49, 0, &V23735_333_57343, &P23735_332};	/*Property = Xhr2*/
const Profile P23735 = {"23735", &P23735_333};	/*Profile ID - 23735*/
const Value V23620_1_2 = {S0, NULL};	/*Value = False*/
const Property P23620_1 = {S50, 0, &V23620_1_2, NULL};	/*Property = IsMobile*/
const Value V23620_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23620_5 = {S52, 0, &V23620_5_60417, &P23620_1};	/*Property = ScreenPixelsHeight*/
const Value V23620_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23620_4 = {S54, 0, &V23620_4_57095, &P23620_5};	/*Property = ScreenPixelsWidth*/
const Profile P23620 = {"23620", &P23620_4};	/*Profile ID - 23620*/
const Value V23613_1_2 = {S0, NULL};	/*Value = False*/
const Property P23613_1 = {S50, 0, &V23613_1_2, NULL};	/*Property = IsMobile*/
const Value V23613_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23613_5 = {S52, 0, &V23613_5_60417, &P23613_1};	/*Property = ScreenPixelsHeight*/
const Value V23613_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23613_4 = {S54, 0, &V23613_4_57095, &P23613_5};	/*Property = ScreenPixelsWidth*/
const Profile P23613 = {"23613", &P23613_4};	/*Profile ID - 23613*/
const Value V18726_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_279 = {S1, 0, &V18726_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18726_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_282 = {S3, 0, &V18726_282_57155, &P18726_279};	/*Property = BlobBuilder*/
const Value V18726_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_350 = {S4, 0, &V18726_350_57446, &P18726_282};	/*Property = Canvas*/
const Value V18726_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_283 = {S5, 0, &V18726_283_57161, &P18726_350};	/*Property = CssBackground*/
const Value V18726_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_284 = {S6, 0, &V18726_284_57164, &P18726_283};	/*Property = CssBorderImage*/
const Value V18726_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_285 = {S7, 0, &V18726_285_57167, &P18726_284};	/*Property = CssCanvas*/
const Value V18726_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_286 = {S8, 0, &V18726_286_57170, &P18726_285};	/*Property = CssColor*/
const Value V18726_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_237 = {S9, 0, &V18726_237_55786, &P18726_286};	/*Property = CssColumn*/
const Value V18726_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_287 = {S10, 0, &V18726_287_57173, &P18726_237};	/*Property = CssFlexbox*/
const Value V18726_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_288 = {S11, 0, &V18726_288_57176, &P18726_287};	/*Property = CssFont*/
const Value V18726_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_289 = {S12, 0, &V18726_289_86899, &P18726_288};	/*Property = CssImages*/
const Value V18726_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_291 = {S13, 0, &V18726_291_57182, &P18726_289};	/*Property = CssMediaQueries*/
const Value V18726_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_292 = {S14, 0, &V18726_292_57185, &P18726_291};	/*Property = CssMinMax*/
const Value V18726_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_293 = {S15, 0, &V18726_293_57188, &P18726_292};	/*Property = CssOverflow*/
const Value V18726_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_294 = {S16, 0, &V18726_294_57191, &P18726_293};	/*Property = CssPosition*/
const Value V18726_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_295 = {S17, 0, &V18726_295_57194, &P18726_294};	/*Property = CssText*/
const Value V18726_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_297 = {S18, 0, &V18726_297_57197, &P18726_295};	/*Property = CssTransforms*/
const Value V18726_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_241 = {S19, 0, &V18726_241_55801, &P18726_297};	/*Property = CssTransitions*/
const Value V18726_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_298 = {S20, 0, &V18726_298_57208, &P18726_241};	/*Property = CssUI*/
const Value V18726_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_300 = {S21, 0, &V18726_300_57217, &P18726_298};	/*Property = DataSet*/
const Value V18726_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_301 = {S22, 0, &V18726_301_57220, &P18726_300};	/*Property = DataUrl*/
const Value V18726_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_302 = {S23, 0, &V18726_302_57223, &P18726_301};	/*Property = DeviceOrientation*/
const Value V18726_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_304 = {S24, 0, &V18726_304_57229, &P18726_302};	/*Property = FileReader*/
const Value V18726_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_305 = {S25, 0, &V18726_305_57232, &P18726_304};	/*Property = FileSaver*/
const Value V18726_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_306 = {S26, 0, &V18726_306_57238, &P18726_305};	/*Property = FileWriter*/
const Value V18726_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_307 = {S27, 0, &V18726_307_57241, &P18726_306};	/*Property = FormData*/
const Value V18726_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_309 = {S28, 0, &V18726_309_57247, &P18726_307};	/*Property = Fullscreen*/
const Value V18726_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_310 = {S29, 0, &V18726_310_57250, &P18726_309};	/*Property = GeoLocation*/
const Value V18726_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_312 = {S30, 0, &V18726_312_57256, &P18726_310};	/*Property = History*/
const Value V18726_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_314 = {S31, 0, &V18726_314_57262, &P18726_312};	/*Property = Html5*/
const Value V18726_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_313 = {S32, 0, &V18726_313_57259, &P18726_314};	/*Property = Html-Media-Capture*/
const Value V18726_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_315 = {S33, 0, &V18726_315_57265, &P18726_313};	/*Property = Iframe*/
const Value V18726_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_316 = {S34, 0, &V18726_316_57268, &P18726_315};	/*Property = IndexedDB*/
const Value V18726_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_317 = {S35, 0, &V18726_317_57271, &P18726_316};	/*Property = Json*/
const Value V18726_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18726_15 = {S37, 0, &V18726_15_29, &P18726_317};	/*Property = LayoutEngine*/
const Value V18726_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_344 = {S38, 0, &V18726_344_57274, &P18726_15};	/*Property = Masking*/
const Value V18726_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_320 = {S39, 0, &V18726_320_57292, &P18726_344};	/*Property = PostMessage*/
const Value V18726_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_321 = {S40, 0, &V18726_321_57295, &P18726_320};	/*Property = Progress*/
const Value V18726_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_322 = {S41, 0, &V18726_322_57298, &P18726_321};	/*Property = Prompts*/
const Value V18726_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_323 = {S42, 0, &V18726_323_57301, &P18726_322};	/*Property = Selector*/
const Value V18726_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_324 = {S43, 0, &V18726_324_57307, &P18726_323};	/*Property = Svg*/
const Value V18726_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_327 = {S44, 0, &V18726_327_57316, &P18726_324};	/*Property = TouchEvents*/
const Value V18726_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_328 = {S45, 0, &V18726_328_57319, &P18726_327};	/*Property = Track*/
const Value V18726_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_329 = {S46, 0, &V18726_329_57325, &P18726_328};	/*Property = Video*/
const Value V18726_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_330 = {S47, 0, &V18726_330_57328, &P18726_329};	/*Property = Viewport*/
const Value V18726_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_332 = {S48, 0, &V18726_332_57340, &P18726_330};	/*Property = WebWorkers*/
const Value V18726_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18726_333 = {S49, 0, &V18726_333_57343, &P18726_332};	/*Property = Xhr2*/
const Profile P18726 = {"18726", &P18726_333};	/*Profile ID - 18726*/
const Value V23626_1_2 = {S0, NULL};	/*Value = False*/
const Property P23626_1 = {S50, 0, &V23626_1_2, NULL};	/*Property = IsMobile*/
const Value V23626_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23626_5 = {S52, 0, &V23626_5_60417, &P23626_1};	/*Property = ScreenPixelsHeight*/
const Value V23626_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23626_4 = {S54, 0, &V23626_4_57095, &P23626_5};	/*Property = ScreenPixelsWidth*/
const Profile P23626 = {"23626", &P23626_4};	/*Profile ID - 23626*/
const Value V23650_1_2 = {S0, NULL};	/*Value = False*/
const Property P23650_1 = {S50, 0, &V23650_1_2, NULL};	/*Property = IsMobile*/
const Value V23650_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23650_5 = {S52, 0, &V23650_5_60417, &P23650_1};	/*Property = ScreenPixelsHeight*/
const Value V23650_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23650_4 = {S54, 0, &V23650_4_57095, &P23650_5};	/*Property = ScreenPixelsWidth*/
const Profile P23650 = {"23650", &P23650_4};	/*Profile ID - 23650*/
const Value V22337_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_279 = {S1, 0, &V22337_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22337_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_282 = {S3, 0, &V22337_282_57155, &P22337_279};	/*Property = BlobBuilder*/
const Value V22337_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_350 = {S4, 0, &V22337_350_57446, &P22337_282};	/*Property = Canvas*/
const Value V22337_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_283 = {S5, 0, &V22337_283_57161, &P22337_350};	/*Property = CssBackground*/
const Value V22337_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_284 = {S6, 0, &V22337_284_57164, &P22337_283};	/*Property = CssBorderImage*/
const Value V22337_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_285 = {S7, 0, &V22337_285_57167, &P22337_284};	/*Property = CssCanvas*/
const Value V22337_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_286 = {S8, 0, &V22337_286_57170, &P22337_285};	/*Property = CssColor*/
const Value V22337_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_237 = {S9, 0, &V22337_237_55786, &P22337_286};	/*Property = CssColumn*/
const Value V22337_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_287 = {S10, 0, &V22337_287_57173, &P22337_237};	/*Property = CssFlexbox*/
const Value V22337_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_288 = {S11, 0, &V22337_288_57176, &P22337_287};	/*Property = CssFont*/
const Value V22337_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_289 = {S12, 0, &V22337_289_86899, &P22337_288};	/*Property = CssImages*/
const Value V22337_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_291 = {S13, 0, &V22337_291_57182, &P22337_289};	/*Property = CssMediaQueries*/
const Value V22337_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_292 = {S14, 0, &V22337_292_57185, &P22337_291};	/*Property = CssMinMax*/
const Value V22337_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_293 = {S15, 0, &V22337_293_57188, &P22337_292};	/*Property = CssOverflow*/
const Value V22337_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_294 = {S16, 0, &V22337_294_57191, &P22337_293};	/*Property = CssPosition*/
const Value V22337_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_295 = {S17, 0, &V22337_295_57194, &P22337_294};	/*Property = CssText*/
const Value V22337_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_297 = {S18, 0, &V22337_297_57197, &P22337_295};	/*Property = CssTransforms*/
const Value V22337_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_241 = {S19, 0, &V22337_241_55801, &P22337_297};	/*Property = CssTransitions*/
const Value V22337_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_298 = {S20, 0, &V22337_298_57208, &P22337_241};	/*Property = CssUI*/
const Value V22337_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_300 = {S21, 0, &V22337_300_57217, &P22337_298};	/*Property = DataSet*/
const Value V22337_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_301 = {S22, 0, &V22337_301_57220, &P22337_300};	/*Property = DataUrl*/
const Value V22337_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_302 = {S23, 0, &V22337_302_57223, &P22337_301};	/*Property = DeviceOrientation*/
const Value V22337_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_304 = {S24, 0, &V22337_304_57229, &P22337_302};	/*Property = FileReader*/
const Value V22337_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_305 = {S25, 0, &V22337_305_57232, &P22337_304};	/*Property = FileSaver*/
const Value V22337_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_306 = {S26, 0, &V22337_306_57238, &P22337_305};	/*Property = FileWriter*/
const Value V22337_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_307 = {S27, 0, &V22337_307_57241, &P22337_306};	/*Property = FormData*/
const Value V22337_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_309 = {S28, 0, &V22337_309_57247, &P22337_307};	/*Property = Fullscreen*/
const Value V22337_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_310 = {S29, 0, &V22337_310_57250, &P22337_309};	/*Property = GeoLocation*/
const Value V22337_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_312 = {S30, 0, &V22337_312_57256, &P22337_310};	/*Property = History*/
const Value V22337_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_314 = {S31, 0, &V22337_314_57262, &P22337_312};	/*Property = Html5*/
const Value V22337_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_313 = {S32, 0, &V22337_313_57259, &P22337_314};	/*Property = Html-Media-Capture*/
const Value V22337_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_315 = {S33, 0, &V22337_315_57265, &P22337_313};	/*Property = Iframe*/
const Value V22337_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_316 = {S34, 0, &V22337_316_57268, &P22337_315};	/*Property = IndexedDB*/
const Value V22337_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_317 = {S35, 0, &V22337_317_57271, &P22337_316};	/*Property = Json*/
const Value V22337_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22337_15 = {S37, 0, &V22337_15_29, &P22337_317};	/*Property = LayoutEngine*/
const Value V22337_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_344 = {S38, 0, &V22337_344_57274, &P22337_15};	/*Property = Masking*/
const Value V22337_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_320 = {S39, 0, &V22337_320_57292, &P22337_344};	/*Property = PostMessage*/
const Value V22337_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_321 = {S40, 0, &V22337_321_57295, &P22337_320};	/*Property = Progress*/
const Value V22337_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_322 = {S41, 0, &V22337_322_57298, &P22337_321};	/*Property = Prompts*/
const Value V22337_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_323 = {S42, 0, &V22337_323_57301, &P22337_322};	/*Property = Selector*/
const Value V22337_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_324 = {S43, 0, &V22337_324_57307, &P22337_323};	/*Property = Svg*/
const Value V22337_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_327 = {S44, 0, &V22337_327_57316, &P22337_324};	/*Property = TouchEvents*/
const Value V22337_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_328 = {S45, 0, &V22337_328_57319, &P22337_327};	/*Property = Track*/
const Value V22337_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_329 = {S46, 0, &V22337_329_57325, &P22337_328};	/*Property = Video*/
const Value V22337_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_330 = {S47, 0, &V22337_330_57328, &P22337_329};	/*Property = Viewport*/
const Value V22337_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_332 = {S48, 0, &V22337_332_57340, &P22337_330};	/*Property = WebWorkers*/
const Value V22337_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22337_333 = {S49, 0, &V22337_333_57343, &P22337_332};	/*Property = Xhr2*/
const Profile P22337 = {"22337", &P22337_333};	/*Profile ID - 22337*/
const Value V21899_1_1 = {S2, NULL};	/*Value = True*/
const Property P21899_1 = {S50, 0, &V21899_1_1, NULL};	/*Property = IsMobile*/
const Value V21899_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21899_5 = {S52, 0, &V21899_5_1403, &P21899_1};	/*Property = ScreenPixelsHeight*/
const Value V21899_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21899_4 = {S54, 0, &V21899_4_1404, &P21899_5};	/*Property = ScreenPixelsWidth*/
const Profile P21899 = {"21899", &P21899_4};	/*Profile ID - 21899*/
const Value V13847_1_1 = {S2, NULL};	/*Value = True*/
const Property P13847_1 = {S50, 0, &V13847_1_1, NULL};	/*Property = IsMobile*/
const Value V13847_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13847_5 = {S52, 0, &V13847_5_1222, &P13847_1};	/*Property = ScreenPixelsHeight*/
const Value V13847_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13847_4 = {S54, 0, &V13847_4_1224, &P13847_5};	/*Property = ScreenPixelsWidth*/
const Profile P13847 = {"13847", &P13847_4};	/*Profile ID - 13847*/
const Value V19978_1_1 = {S2, NULL};	/*Value = True*/
const Property P19978_1 = {S50, 0, &V19978_1_1, NULL};	/*Property = IsMobile*/
const Value V19978_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19978_5 = {S52, 0, &V19978_5_1227, &P19978_1};	/*Property = ScreenPixelsHeight*/
const Value V19978_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19978_4 = {S54, 0, &V19978_4_1229, &P19978_5};	/*Property = ScreenPixelsWidth*/
const Profile P19978 = {"19978", &P19978_4};	/*Profile ID - 19978*/
const Value V21381_1_1 = {S2, NULL};	/*Value = True*/
const Property P21381_1 = {S50, 0, &V21381_1_1, NULL};	/*Property = IsMobile*/
const Value V21381_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21381_5 = {S52, 0, &V21381_5_1227, &P21381_1};	/*Property = ScreenPixelsHeight*/
const Value V21381_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21381_4 = {S54, 0, &V21381_4_1229, &P21381_5};	/*Property = ScreenPixelsWidth*/
const Profile P21381 = {"21381", &P21381_4};	/*Profile ID - 21381*/
const Value V21470_1_1 = {S2, NULL};	/*Value = True*/
const Property P21470_1 = {S50, 0, &V21470_1_1, NULL};	/*Property = IsMobile*/
const Value V21470_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21470_5 = {S52, 0, &V21470_5_1277, &P21470_1};	/*Property = ScreenPixelsHeight*/
const Value V21470_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21470_4 = {S54, 0, &V21470_4_1279, &P21470_5};	/*Property = ScreenPixelsWidth*/
const Profile P21470 = {"21470", &P21470_4};	/*Profile ID - 21470*/
const Value V22974_1_1 = {S2, NULL};	/*Value = True*/
const Property P22974_1 = {S50, 0, &V22974_1_1, NULL};	/*Property = IsMobile*/
const Value V22974_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22974_5 = {S52, 0, &V22974_5_1222, &P22974_1};	/*Property = ScreenPixelsHeight*/
const Value V22974_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22974_4 = {S54, 0, &V22974_4_1224, &P22974_5};	/*Property = ScreenPixelsWidth*/
const Profile P22974 = {"22974", &P22974_4};	/*Profile ID - 22974*/
const Value V20431_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_279 = {S1, 0, &V20431_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20431_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_282 = {S3, 0, &V20431_282_57155, &P20431_279};	/*Property = BlobBuilder*/
const Value V20431_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_350 = {S4, 0, &V20431_350_57446, &P20431_282};	/*Property = Canvas*/
const Value V20431_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_283 = {S5, 0, &V20431_283_57161, &P20431_350};	/*Property = CssBackground*/
const Value V20431_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_284 = {S6, 0, &V20431_284_57164, &P20431_283};	/*Property = CssBorderImage*/
const Value V20431_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_285 = {S7, 0, &V20431_285_57167, &P20431_284};	/*Property = CssCanvas*/
const Value V20431_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_286 = {S8, 0, &V20431_286_57170, &P20431_285};	/*Property = CssColor*/
const Value V20431_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_237 = {S9, 0, &V20431_237_55786, &P20431_286};	/*Property = CssColumn*/
const Value V20431_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_287 = {S10, 0, &V20431_287_57173, &P20431_237};	/*Property = CssFlexbox*/
const Value V20431_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_288 = {S11, 0, &V20431_288_57176, &P20431_287};	/*Property = CssFont*/
const Value V20431_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_289 = {S12, 0, &V20431_289_86899, &P20431_288};	/*Property = CssImages*/
const Value V20431_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_291 = {S13, 0, &V20431_291_57182, &P20431_289};	/*Property = CssMediaQueries*/
const Value V20431_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_292 = {S14, 0, &V20431_292_57185, &P20431_291};	/*Property = CssMinMax*/
const Value V20431_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_293 = {S15, 0, &V20431_293_57188, &P20431_292};	/*Property = CssOverflow*/
const Value V20431_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_294 = {S16, 0, &V20431_294_57191, &P20431_293};	/*Property = CssPosition*/
const Value V20431_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_295 = {S17, 0, &V20431_295_57194, &P20431_294};	/*Property = CssText*/
const Value V20431_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_297 = {S18, 0, &V20431_297_57197, &P20431_295};	/*Property = CssTransforms*/
const Value V20431_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_241 = {S19, 0, &V20431_241_55801, &P20431_297};	/*Property = CssTransitions*/
const Value V20431_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_298 = {S20, 0, &V20431_298_57208, &P20431_241};	/*Property = CssUI*/
const Value V20431_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_300 = {S21, 0, &V20431_300_57217, &P20431_298};	/*Property = DataSet*/
const Value V20431_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_301 = {S22, 0, &V20431_301_57220, &P20431_300};	/*Property = DataUrl*/
const Value V20431_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_302 = {S23, 0, &V20431_302_57223, &P20431_301};	/*Property = DeviceOrientation*/
const Value V20431_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_304 = {S24, 0, &V20431_304_57229, &P20431_302};	/*Property = FileReader*/
const Value V20431_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_305 = {S25, 0, &V20431_305_57232, &P20431_304};	/*Property = FileSaver*/
const Value V20431_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_306 = {S26, 0, &V20431_306_57238, &P20431_305};	/*Property = FileWriter*/
const Value V20431_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_307 = {S27, 0, &V20431_307_57241, &P20431_306};	/*Property = FormData*/
const Value V20431_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_309 = {S28, 0, &V20431_309_57247, &P20431_307};	/*Property = Fullscreen*/
const Value V20431_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_310 = {S29, 0, &V20431_310_57250, &P20431_309};	/*Property = GeoLocation*/
const Value V20431_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_312 = {S30, 0, &V20431_312_57256, &P20431_310};	/*Property = History*/
const Value V20431_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_314 = {S31, 0, &V20431_314_57262, &P20431_312};	/*Property = Html5*/
const Value V20431_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_313 = {S32, 0, &V20431_313_57259, &P20431_314};	/*Property = Html-Media-Capture*/
const Value V20431_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_315 = {S33, 0, &V20431_315_57265, &P20431_313};	/*Property = Iframe*/
const Value V20431_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_316 = {S34, 0, &V20431_316_57268, &P20431_315};	/*Property = IndexedDB*/
const Value V20431_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_317 = {S35, 0, &V20431_317_57271, &P20431_316};	/*Property = Json*/
const Value V20431_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20431_15 = {S37, 0, &V20431_15_29, &P20431_317};	/*Property = LayoutEngine*/
const Value V20431_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_344 = {S38, 0, &V20431_344_57274, &P20431_15};	/*Property = Masking*/
const Value V20431_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_320 = {S39, 0, &V20431_320_57292, &P20431_344};	/*Property = PostMessage*/
const Value V20431_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_321 = {S40, 0, &V20431_321_57295, &P20431_320};	/*Property = Progress*/
const Value V20431_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_322 = {S41, 0, &V20431_322_57298, &P20431_321};	/*Property = Prompts*/
const Value V20431_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_323 = {S42, 0, &V20431_323_57301, &P20431_322};	/*Property = Selector*/
const Value V20431_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_324 = {S43, 0, &V20431_324_57307, &P20431_323};	/*Property = Svg*/
const Value V20431_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_327 = {S44, 0, &V20431_327_57316, &P20431_324};	/*Property = TouchEvents*/
const Value V20431_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_328 = {S45, 0, &V20431_328_57319, &P20431_327};	/*Property = Track*/
const Value V20431_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_329 = {S46, 0, &V20431_329_57325, &P20431_328};	/*Property = Video*/
const Value V20431_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_330 = {S47, 0, &V20431_330_57328, &P20431_329};	/*Property = Viewport*/
const Value V20431_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_332 = {S48, 0, &V20431_332_57340, &P20431_330};	/*Property = WebWorkers*/
const Value V20431_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20431_333 = {S49, 0, &V20431_333_57343, &P20431_332};	/*Property = Xhr2*/
const Profile P20431 = {"20431", &P20431_333};	/*Profile ID - 20431*/
const Value V20503_1_1 = {S2, NULL};	/*Value = True*/
const Property P20503_1 = {S50, 0, &V20503_1_1, NULL};	/*Property = IsMobile*/
const Value V20503_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20503_5 = {S52, 0, &V20503_5_1227, &P20503_1};	/*Property = ScreenPixelsHeight*/
const Value V20503_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20503_4 = {S54, 0, &V20503_4_1229, &P20503_5};	/*Property = ScreenPixelsWidth*/
const Profile P20503 = {"20503", &P20503_4};	/*Profile ID - 20503*/
const Value V21510_1_1 = {S2, NULL};	/*Value = True*/
const Property P21510_1 = {S50, 0, &V21510_1_1, NULL};	/*Property = IsMobile*/
const Value V21510_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21510_5 = {S52, 0, &V21510_5_1227, &P21510_1};	/*Property = ScreenPixelsHeight*/
const Value V21510_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21510_4 = {S54, 0, &V21510_4_1229, &P21510_5};	/*Property = ScreenPixelsWidth*/
const Profile P21510 = {"21510", &P21510_4};	/*Profile ID - 21510*/
const Value V22257_1_1 = {S2, NULL};	/*Value = True*/
const Property P22257_1 = {S50, 0, &V22257_1_1, NULL};	/*Property = IsMobile*/
const Value V22257_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22257_5 = {S52, 0, &V22257_5_1277, &P22257_1};	/*Property = ScreenPixelsHeight*/
const Value V22257_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22257_4 = {S54, 0, &V22257_4_2336, &P22257_5};	/*Property = ScreenPixelsWidth*/
const Profile P22257 = {"22257", &P22257_4};	/*Profile ID - 22257*/
const Value V20295_1_1 = {S2, NULL};	/*Value = True*/
const Property P20295_1 = {S50, 0, &V20295_1_1, NULL};	/*Property = IsMobile*/
const Value V20295_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20295_5 = {S52, 0, &V20295_5_37, &P20295_1};	/*Property = ScreenPixelsHeight*/
const Value V20295_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20295_4 = {S54, 0, &V20295_4_36, &P20295_5};	/*Property = ScreenPixelsWidth*/
const Profile P20295 = {"20295", &P20295_4};	/*Profile ID - 20295*/
const Value V21672_1_1 = {S2, NULL};	/*Value = True*/
const Property P21672_1 = {S50, 0, &V21672_1_1, NULL};	/*Property = IsMobile*/
const Value V21672_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21672_5 = {S52, 0, &V21672_5_1227, &P21672_1};	/*Property = ScreenPixelsHeight*/
const Value V21672_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21672_4 = {S54, 0, &V21672_4_1229, &P21672_5};	/*Property = ScreenPixelsWidth*/
const Profile P21672 = {"21672", &P21672_4};	/*Profile ID - 21672*/
const Value V21658_1_1 = {S2, NULL};	/*Value = True*/
const Property P21658_1 = {S50, 0, &V21658_1_1, NULL};	/*Property = IsMobile*/
const Value V21658_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21658_5 = {S52, 0, &V21658_5_1227, &P21658_1};	/*Property = ScreenPixelsHeight*/
const Value V21658_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21658_4 = {S54, 0, &V21658_4_1229, &P21658_5};	/*Property = ScreenPixelsWidth*/
const Profile P21658 = {"21658", &P21658_4};	/*Profile ID - 21658*/
const Value V22386_1_1 = {S2, NULL};	/*Value = True*/
const Property P22386_1 = {S50, 0, &V22386_1_1, NULL};	/*Property = IsMobile*/
const Value V22386_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22386_5 = {S52, 0, &V22386_5_1227, &P22386_1};	/*Property = ScreenPixelsHeight*/
const Value V22386_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22386_4 = {S54, 0, &V22386_4_1229, &P22386_5};	/*Property = ScreenPixelsWidth*/
const Profile P22386 = {"22386", &P22386_4};	/*Profile ID - 22386*/
const Value V19903_1_1 = {S2, NULL};	/*Value = True*/
const Property P19903_1 = {S50, 0, &V19903_1_1, NULL};	/*Property = IsMobile*/
const Value V19903_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19903_5 = {S52, 0, &V19903_5_1227, &P19903_1};	/*Property = ScreenPixelsHeight*/
const Value V19903_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19903_4 = {S54, 0, &V19903_4_1229, &P19903_5};	/*Property = ScreenPixelsWidth*/
const Profile P19903 = {"19903", &P19903_4};	/*Profile ID - 19903*/
const Value V18477_1_1 = {S2, NULL};	/*Value = True*/
const Property P18477_1 = {S50, 0, &V18477_1_1, NULL};	/*Property = IsMobile*/
const Value V18477_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18477_5 = {S52, 0, &V18477_5_1222, &P18477_1};	/*Property = ScreenPixelsHeight*/
const Value V18477_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18477_4 = {S54, 0, &V18477_4_1224, &P18477_5};	/*Property = ScreenPixelsWidth*/
const Profile P18477 = {"18477", &P18477_4};	/*Profile ID - 18477*/
const Value V22348_1_1 = {S2, NULL};	/*Value = True*/
const Property P22348_1 = {S50, 0, &V22348_1_1, NULL};	/*Property = IsMobile*/
const Value V22348_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22348_5 = {S52, 0, &V22348_5_1227, &P22348_1};	/*Property = ScreenPixelsHeight*/
const Value V22348_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22348_4 = {S54, 0, &V22348_4_1460, &P22348_5};	/*Property = ScreenPixelsWidth*/
const Profile P22348 = {"22348", &P22348_4};	/*Profile ID - 22348*/
const Value V21313_1_1 = {S2, NULL};	/*Value = True*/
const Property P21313_1 = {S50, 0, &V21313_1_1, NULL};	/*Property = IsMobile*/
const Value V21313_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21313_5 = {S52, 0, &V21313_5_1227, &P21313_1};	/*Property = ScreenPixelsHeight*/
const Value V21313_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21313_4 = {S54, 0, &V21313_4_1229, &P21313_5};	/*Property = ScreenPixelsWidth*/
const Profile P21313 = {"21313", &P21313_4};	/*Profile ID - 21313*/
const Value V20320_1_2 = {S0, NULL};	/*Value = False*/
const Property P20320_1 = {S50, 0, &V20320_1_2, NULL};	/*Property = IsMobile*/
const Value V20320_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20320_5 = {S52, 0, &V20320_5_37, &P20320_1};	/*Property = ScreenPixelsHeight*/
const Value V20320_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20320_4 = {S54, 0, &V20320_4_36, &P20320_5};	/*Property = ScreenPixelsWidth*/
const Profile P20320 = {"20320", &P20320_4};	/*Profile ID - 20320*/
const Value V22352_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22352_279 = {S1, 0, &V22352_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22352_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22352_282 = {S3, 0, &V22352_282_57153, &P22352_279};	/*Property = BlobBuilder*/
const Value V22352_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22352_350 = {S4, 0, &V22352_350_57444, &P22352_282};	/*Property = Canvas*/
const Value V22352_283_57160 = {S0, NULL};	/*Value = False*/
const Property P22352_283 = {S5, 0, &V22352_283_57160, &P22352_350};	/*Property = CssBackground*/
const Value V22352_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22352_284 = {S6, 0, &V22352_284_57163, &P22352_283};	/*Property = CssBorderImage*/
const Value V22352_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22352_285 = {S7, 0, &V22352_285_57166, &P22352_284};	/*Property = CssCanvas*/
const Value V22352_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22352_286 = {S8, 0, &V22352_286_57168, &P22352_285};	/*Property = CssColor*/
const Value V22352_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22352_237 = {S9, 0, &V22352_237_55784, &P22352_286};	/*Property = CssColumn*/
const Value V22352_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22352_287 = {S10, 0, &V22352_287_57172, &P22352_237};	/*Property = CssFlexbox*/
const Value V22352_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22352_288 = {S11, 0, &V22352_288_57175, &P22352_287};	/*Property = CssFont*/
const Value V22352_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22352_289 = {S12, 0, &V22352_289_86899, &P22352_288};	/*Property = CssImages*/
const Value V22352_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22352_291 = {S13, 0, &V22352_291_57181, &P22352_289};	/*Property = CssMediaQueries*/
const Value V22352_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22352_292 = {S14, 0, &V22352_292_57183, &P22352_291};	/*Property = CssMinMax*/
const Value V22352_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22352_293 = {S15, 0, &V22352_293_57187, &P22352_292};	/*Property = CssOverflow*/
const Value V22352_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22352_294 = {S16, 0, &V22352_294_57189, &P22352_293};	/*Property = CssPosition*/
const Value V22352_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22352_295 = {S17, 0, &V22352_295_57193, &P22352_294};	/*Property = CssText*/
const Value V22352_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22352_297 = {S18, 0, &V22352_297_57195, &P22352_295};	/*Property = CssTransforms*/
const Value V22352_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22352_241 = {S19, 0, &V22352_241_55799, &P22352_297};	/*Property = CssTransitions*/
const Value V22352_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22352_298 = {S20, 0, &V22352_298_57206, &P22352_241};	/*Property = CssUI*/
const Value V22352_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22352_300 = {S21, 0, &V22352_300_57215, &P22352_298};	/*Property = DataSet*/
const Value V22352_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22352_301 = {S22, 0, &V22352_301_57218, &P22352_300};	/*Property = DataUrl*/
const Value V22352_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22352_302 = {S23, 0, &V22352_302_57221, &P22352_301};	/*Property = DeviceOrientation*/
const Value V22352_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22352_304 = {S24, 0, &V22352_304_57227, &P22352_302};	/*Property = FileReader*/
const Value V22352_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22352_305 = {S25, 0, &V22352_305_57231, &P22352_304};	/*Property = FileSaver*/
const Value V22352_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22352_306 = {S26, 0, &V22352_306_57237, &P22352_305};	/*Property = FileWriter*/
const Value V22352_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22352_307 = {S27, 0, &V22352_307_57239, &P22352_306};	/*Property = FormData*/
const Value V22352_309_57245 = {S2, NULL};	/*Value = True*/
const Property P22352_309 = {S28, 0, &V22352_309_57245, &P22352_307};	/*Property = Fullscreen*/
const Value V22352_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22352_310 = {S29, 0, &V22352_310_57248, &P22352_309};	/*Property = GeoLocation*/
const Value V22352_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22352_312 = {S30, 0, &V22352_312_57254, &P22352_310};	/*Property = History*/
const Value V22352_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22352_314 = {S31, 0, &V22352_314_57261, &P22352_312};	/*Property = Html5*/
const Value V22352_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22352_313 = {S32, 0, &V22352_313_57258, &P22352_314};	/*Property = Html-Media-Capture*/
const Value V22352_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22352_315 = {S33, 0, &V22352_315_57264, &P22352_313};	/*Property = Iframe*/
const Value V22352_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22352_316 = {S34, 0, &V22352_316_57266, &P22352_315};	/*Property = IndexedDB*/
const Value V22352_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22352_317 = {S35, 0, &V22352_317_57269, &P22352_316};	/*Property = Json*/
const Value V22352_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22352_15 = {S37, 0, &V22352_15_29, &P22352_317};	/*Property = LayoutEngine*/
const Value V22352_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22352_344 = {S38, 0, &V22352_344_57272, &P22352_15};	/*Property = Masking*/
const Value V22352_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22352_320 = {S39, 0, &V22352_320_57290, &P22352_344};	/*Property = PostMessage*/
const Value V22352_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22352_321 = {S40, 0, &V22352_321_57293, &P22352_320};	/*Property = Progress*/
const Value V22352_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22352_322 = {S41, 0, &V22352_322_57296, &P22352_321};	/*Property = Prompts*/
const Value V22352_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22352_323 = {S42, 0, &V22352_323_57299, &P22352_322};	/*Property = Selector*/
const Value V22352_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22352_324 = {S43, 0, &V22352_324_57305, &P22352_323};	/*Property = Svg*/
const Value V22352_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22352_327 = {S44, 0, &V22352_327_57315, &P22352_324};	/*Property = TouchEvents*/
const Value V22352_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22352_328 = {S45, 0, &V22352_328_57318, &P22352_327};	/*Property = Track*/
const Value V22352_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22352_329 = {S46, 0, &V22352_329_57323, &P22352_328};	/*Property = Video*/
const Value V22352_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22352_330 = {S47, 0, &V22352_330_57326, &P22352_329};	/*Property = Viewport*/
const Value V22352_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22352_332 = {S48, 0, &V22352_332_57339, &P22352_330};	/*Property = WebWorkers*/
const Value V22352_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22352_333 = {S49, 0, &V22352_333_57342, &P22352_332};	/*Property = Xhr2*/
const Profile P22352 = {"22352", &P22352_333};	/*Profile ID - 22352*/
const Value V21550_1_1 = {S2, NULL};	/*Value = True*/
const Property P21550_1 = {S50, 0, &V21550_1_1, NULL};	/*Property = IsMobile*/
const Value V21550_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21550_5 = {S52, 0, &V21550_5_1277, &P21550_1};	/*Property = ScreenPixelsHeight*/
const Value V21550_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21550_4 = {S54, 0, &V21550_4_1279, &P21550_5};	/*Property = ScreenPixelsWidth*/
const Profile P21550 = {"21550", &P21550_4};	/*Profile ID - 21550*/
const Value V20328_1_2 = {S0, NULL};	/*Value = False*/
const Property P20328_1 = {S50, 0, &V20328_1_2, NULL};	/*Property = IsMobile*/
const Value V20328_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20328_5 = {S52, 0, &V20328_5_37, &P20328_1};	/*Property = ScreenPixelsHeight*/
const Value V20328_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20328_4 = {S54, 0, &V20328_4_36, &P20328_5};	/*Property = ScreenPixelsWidth*/
const Profile P20328 = {"20328", &P20328_4};	/*Profile ID - 20328*/
const Value V18742_1_1 = {S2, NULL};	/*Value = True*/
const Property P18742_1 = {S50, 0, &V18742_1_1, NULL};	/*Property = IsMobile*/
const Value V18742_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18742_5 = {S52, 0, &V18742_5_1227, &P18742_1};	/*Property = ScreenPixelsHeight*/
const Value V18742_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18742_4 = {S54, 0, &V18742_4_1229, &P18742_5};	/*Property = ScreenPixelsWidth*/
const Profile P18742 = {"18742", &P18742_4};	/*Profile ID - 18742*/
const Value V23987_1_1 = {S2, NULL};	/*Value = True*/
const Property P23987_1 = {S50, 0, &V23987_1_1, NULL};	/*Property = IsMobile*/
const Value V23987_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23987_5 = {S52, 0, &V23987_5_1277, &P23987_1};	/*Property = ScreenPixelsHeight*/
const Value V23987_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23987_4 = {S54, 0, &V23987_4_1279, &P23987_5};	/*Property = ScreenPixelsWidth*/
const Profile P23987 = {"23987", &P23987_4};	/*Profile ID - 23987*/
const Value V21754_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_279 = {S1, 0, &V21754_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21754_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_282 = {S3, 0, &V21754_282_57155, &P21754_279};	/*Property = BlobBuilder*/
const Value V21754_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_350 = {S4, 0, &V21754_350_57446, &P21754_282};	/*Property = Canvas*/
const Value V21754_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_283 = {S5, 0, &V21754_283_57161, &P21754_350};	/*Property = CssBackground*/
const Value V21754_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_284 = {S6, 0, &V21754_284_57164, &P21754_283};	/*Property = CssBorderImage*/
const Value V21754_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_285 = {S7, 0, &V21754_285_57167, &P21754_284};	/*Property = CssCanvas*/
const Value V21754_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_286 = {S8, 0, &V21754_286_57170, &P21754_285};	/*Property = CssColor*/
const Value V21754_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_237 = {S9, 0, &V21754_237_55786, &P21754_286};	/*Property = CssColumn*/
const Value V21754_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_287 = {S10, 0, &V21754_287_57173, &P21754_237};	/*Property = CssFlexbox*/
const Value V21754_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_288 = {S11, 0, &V21754_288_57176, &P21754_287};	/*Property = CssFont*/
const Value V21754_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_289 = {S12, 0, &V21754_289_86899, &P21754_288};	/*Property = CssImages*/
const Value V21754_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_291 = {S13, 0, &V21754_291_57182, &P21754_289};	/*Property = CssMediaQueries*/
const Value V21754_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_292 = {S14, 0, &V21754_292_57185, &P21754_291};	/*Property = CssMinMax*/
const Value V21754_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_293 = {S15, 0, &V21754_293_57188, &P21754_292};	/*Property = CssOverflow*/
const Value V21754_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_294 = {S16, 0, &V21754_294_57191, &P21754_293};	/*Property = CssPosition*/
const Value V21754_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_295 = {S17, 0, &V21754_295_57194, &P21754_294};	/*Property = CssText*/
const Value V21754_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_297 = {S18, 0, &V21754_297_57197, &P21754_295};	/*Property = CssTransforms*/
const Value V21754_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_241 = {S19, 0, &V21754_241_55801, &P21754_297};	/*Property = CssTransitions*/
const Value V21754_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_298 = {S20, 0, &V21754_298_57208, &P21754_241};	/*Property = CssUI*/
const Value V21754_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_300 = {S21, 0, &V21754_300_57217, &P21754_298};	/*Property = DataSet*/
const Value V21754_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_301 = {S22, 0, &V21754_301_57220, &P21754_300};	/*Property = DataUrl*/
const Value V21754_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_302 = {S23, 0, &V21754_302_57223, &P21754_301};	/*Property = DeviceOrientation*/
const Value V21754_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_304 = {S24, 0, &V21754_304_57229, &P21754_302};	/*Property = FileReader*/
const Value V21754_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_305 = {S25, 0, &V21754_305_57232, &P21754_304};	/*Property = FileSaver*/
const Value V21754_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_306 = {S26, 0, &V21754_306_57238, &P21754_305};	/*Property = FileWriter*/
const Value V21754_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_307 = {S27, 0, &V21754_307_57241, &P21754_306};	/*Property = FormData*/
const Value V21754_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_309 = {S28, 0, &V21754_309_57247, &P21754_307};	/*Property = Fullscreen*/
const Value V21754_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_310 = {S29, 0, &V21754_310_57250, &P21754_309};	/*Property = GeoLocation*/
const Value V21754_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_312 = {S30, 0, &V21754_312_57256, &P21754_310};	/*Property = History*/
const Value V21754_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_314 = {S31, 0, &V21754_314_57262, &P21754_312};	/*Property = Html5*/
const Value V21754_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_313 = {S32, 0, &V21754_313_57259, &P21754_314};	/*Property = Html-Media-Capture*/
const Value V21754_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_315 = {S33, 0, &V21754_315_57265, &P21754_313};	/*Property = Iframe*/
const Value V21754_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_316 = {S34, 0, &V21754_316_57268, &P21754_315};	/*Property = IndexedDB*/
const Value V21754_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_317 = {S35, 0, &V21754_317_57271, &P21754_316};	/*Property = Json*/
const Value V21754_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21754_15 = {S37, 0, &V21754_15_54167, &P21754_317};	/*Property = LayoutEngine*/
const Value V21754_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_344 = {S38, 0, &V21754_344_57274, &P21754_15};	/*Property = Masking*/
const Value V21754_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_320 = {S39, 0, &V21754_320_57292, &P21754_344};	/*Property = PostMessage*/
const Value V21754_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_321 = {S40, 0, &V21754_321_57295, &P21754_320};	/*Property = Progress*/
const Value V21754_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_322 = {S41, 0, &V21754_322_57298, &P21754_321};	/*Property = Prompts*/
const Value V21754_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_323 = {S42, 0, &V21754_323_57301, &P21754_322};	/*Property = Selector*/
const Value V21754_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_324 = {S43, 0, &V21754_324_57307, &P21754_323};	/*Property = Svg*/
const Value V21754_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_327 = {S44, 0, &V21754_327_57316, &P21754_324};	/*Property = TouchEvents*/
const Value V21754_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_328 = {S45, 0, &V21754_328_57319, &P21754_327};	/*Property = Track*/
const Value V21754_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_329 = {S46, 0, &V21754_329_57325, &P21754_328};	/*Property = Video*/
const Value V21754_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_330 = {S47, 0, &V21754_330_57328, &P21754_329};	/*Property = Viewport*/
const Value V21754_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_332 = {S48, 0, &V21754_332_57340, &P21754_330};	/*Property = WebWorkers*/
const Value V21754_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21754_333 = {S49, 0, &V21754_333_57343, &P21754_332};	/*Property = Xhr2*/
const Profile P21754 = {"21754", &P21754_333};	/*Profile ID - 21754*/
const Value V21476_1_1 = {S2, NULL};	/*Value = True*/
const Property P21476_1 = {S50, 0, &V21476_1_1, NULL};	/*Property = IsMobile*/
const Value V21476_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21476_5 = {S52, 0, &V21476_5_1227, &P21476_1};	/*Property = ScreenPixelsHeight*/
const Value V21476_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21476_4 = {S54, 0, &V21476_4_1229, &P21476_5};	/*Property = ScreenPixelsWidth*/
const Profile P21476 = {"21476", &P21476_4};	/*Profile ID - 21476*/
const Value V19880_1_1 = {S2, NULL};	/*Value = True*/
const Property P19880_1 = {S50, 0, &V19880_1_1, NULL};	/*Property = IsMobile*/
const Value V19880_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19880_5 = {S52, 0, &V19880_5_1227, &P19880_1};	/*Property = ScreenPixelsHeight*/
const Value V19880_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19880_4 = {S54, 0, &V19880_4_1229, &P19880_5};	/*Property = ScreenPixelsWidth*/
const Profile P19880 = {"19880", &P19880_4};	/*Profile ID - 19880*/
const Value V19991_1_1 = {S2, NULL};	/*Value = True*/
const Property P19991_1 = {S50, 0, &V19991_1_1, NULL};	/*Property = IsMobile*/
const Value V19991_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19991_5 = {S52, 0, &V19991_5_9981, &P19991_1};	/*Property = ScreenPixelsHeight*/
const Value V19991_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P19991_4 = {S54, 0, &V19991_4_2336, &P19991_5};	/*Property = ScreenPixelsWidth*/
const Profile P19991 = {"19991", &P19991_4};	/*Profile ID - 19991*/
const Value V21961_1_1 = {S2, NULL};	/*Value = True*/
const Property P21961_1 = {S50, 0, &V21961_1_1, NULL};	/*Property = IsMobile*/
const Value V21961_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21961_5 = {S52, 0, &V21961_5_1277, &P21961_1};	/*Property = ScreenPixelsHeight*/
const Value V21961_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21961_4 = {S54, 0, &V21961_4_1279, &P21961_5};	/*Property = ScreenPixelsWidth*/
const Profile P21961 = {"21961", &P21961_4};	/*Profile ID - 21961*/
const Value V21428_1_1 = {S2, NULL};	/*Value = True*/
const Property P21428_1 = {S50, 0, &V21428_1_1, NULL};	/*Property = IsMobile*/
const Value V21428_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21428_5 = {S52, 0, &V21428_5_1227, &P21428_1};	/*Property = ScreenPixelsHeight*/
const Value V21428_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21428_4 = {S54, 0, &V21428_4_1460, &P21428_5};	/*Property = ScreenPixelsWidth*/
const Profile P21428 = {"21428", &P21428_4};	/*Profile ID - 21428*/
const Value V20380_1_2 = {S0, NULL};	/*Value = False*/
const Property P20380_1 = {S50, 0, &V20380_1_2, NULL};	/*Property = IsMobile*/
const Value V20380_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20380_5 = {S52, 0, &V20380_5_37, &P20380_1};	/*Property = ScreenPixelsHeight*/
const Value V20380_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20380_4 = {S54, 0, &V20380_4_36, &P20380_5};	/*Property = ScreenPixelsWidth*/
const Profile P20380 = {"20380", &P20380_4};	/*Profile ID - 20380*/
const Value V22096_1_1 = {S2, NULL};	/*Value = True*/
const Property P22096_1 = {S50, 0, &V22096_1_1, NULL};	/*Property = IsMobile*/
const Value V22096_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22096_5 = {S52, 0, &V22096_5_1277, &P22096_1};	/*Property = ScreenPixelsHeight*/
const Value V22096_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22096_4 = {S54, 0, &V22096_4_2336, &P22096_5};	/*Property = ScreenPixelsWidth*/
const Profile P22096 = {"22096", &P22096_4};	/*Profile ID - 22096*/
const Value V21209_1_1 = {S2, NULL};	/*Value = True*/
const Property P21209_1 = {S50, 0, &V21209_1_1, NULL};	/*Property = IsMobile*/
const Value V21209_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21209_5 = {S52, 0, &V21209_5_1277, &P21209_1};	/*Property = ScreenPixelsHeight*/
const Value V21209_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21209_4 = {S54, 0, &V21209_4_1279, &P21209_5};	/*Property = ScreenPixelsWidth*/
const Profile P21209 = {"21209", &P21209_4};	/*Profile ID - 21209*/
const Value V18733_1_1 = {S2, NULL};	/*Value = True*/
const Property P18733_1 = {S50, 0, &V18733_1_1, NULL};	/*Property = IsMobile*/
const Value V18733_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18733_5 = {S52, 0, &V18733_5_9981, &P18733_1};	/*Property = ScreenPixelsHeight*/
const Value V18733_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18733_4 = {S54, 0, &V18733_4_2336, &P18733_5};	/*Property = ScreenPixelsWidth*/
const Profile P18733 = {"18733", &P18733_4};	/*Profile ID - 18733*/
const Value V21671_1_1 = {S2, NULL};	/*Value = True*/
const Property P21671_1 = {S50, 0, &V21671_1_1, NULL};	/*Property = IsMobile*/
const Value V21671_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21671_5 = {S52, 0, &V21671_5_1227, &P21671_1};	/*Property = ScreenPixelsHeight*/
const Value V21671_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21671_4 = {S54, 0, &V21671_4_1229, &P21671_5};	/*Property = ScreenPixelsWidth*/
const Profile P21671 = {"21671", &P21671_4};	/*Profile ID - 21671*/
const Value V21531_1_1 = {S2, NULL};	/*Value = True*/
const Property P21531_1 = {S50, 0, &V21531_1_1, NULL};	/*Property = IsMobile*/
const Value V21531_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21531_5 = {S52, 0, &V21531_5_1277, &P21531_1};	/*Property = ScreenPixelsHeight*/
const Value V21531_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21531_4 = {S54, 0, &V21531_4_1279, &P21531_5};	/*Property = ScreenPixelsWidth*/
const Profile P21531 = {"21531", &P21531_4};	/*Profile ID - 21531*/
const Value V21065_1_1 = {S2, NULL};	/*Value = True*/
const Property P21065_1 = {S50, 0, &V21065_1_1, NULL};	/*Property = IsMobile*/
const Value V21065_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21065_5 = {S52, 0, &V21065_5_1277, &P21065_1};	/*Property = ScreenPixelsHeight*/
const Value V21065_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21065_4 = {S54, 0, &V21065_4_1279, &P21065_5};	/*Property = ScreenPixelsWidth*/
const Profile P21065 = {"21065", &P21065_4};	/*Profile ID - 21065*/
const Value V21940_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_279 = {S1, 0, &V21940_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21940_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_282 = {S3, 0, &V21940_282_57155, &P21940_279};	/*Property = BlobBuilder*/
const Value V21940_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_350 = {S4, 0, &V21940_350_57446, &P21940_282};	/*Property = Canvas*/
const Value V21940_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_283 = {S5, 0, &V21940_283_57161, &P21940_350};	/*Property = CssBackground*/
const Value V21940_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_284 = {S6, 0, &V21940_284_57164, &P21940_283};	/*Property = CssBorderImage*/
const Value V21940_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_285 = {S7, 0, &V21940_285_57167, &P21940_284};	/*Property = CssCanvas*/
const Value V21940_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_286 = {S8, 0, &V21940_286_57170, &P21940_285};	/*Property = CssColor*/
const Value V21940_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_237 = {S9, 0, &V21940_237_55786, &P21940_286};	/*Property = CssColumn*/
const Value V21940_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_287 = {S10, 0, &V21940_287_57173, &P21940_237};	/*Property = CssFlexbox*/
const Value V21940_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_288 = {S11, 0, &V21940_288_57176, &P21940_287};	/*Property = CssFont*/
const Value V21940_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_289 = {S12, 0, &V21940_289_86899, &P21940_288};	/*Property = CssImages*/
const Value V21940_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_291 = {S13, 0, &V21940_291_57182, &P21940_289};	/*Property = CssMediaQueries*/
const Value V21940_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_292 = {S14, 0, &V21940_292_57185, &P21940_291};	/*Property = CssMinMax*/
const Value V21940_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_293 = {S15, 0, &V21940_293_57188, &P21940_292};	/*Property = CssOverflow*/
const Value V21940_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_294 = {S16, 0, &V21940_294_57191, &P21940_293};	/*Property = CssPosition*/
const Value V21940_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_295 = {S17, 0, &V21940_295_57194, &P21940_294};	/*Property = CssText*/
const Value V21940_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_297 = {S18, 0, &V21940_297_57197, &P21940_295};	/*Property = CssTransforms*/
const Value V21940_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_241 = {S19, 0, &V21940_241_55801, &P21940_297};	/*Property = CssTransitions*/
const Value V21940_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_298 = {S20, 0, &V21940_298_57208, &P21940_241};	/*Property = CssUI*/
const Value V21940_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_300 = {S21, 0, &V21940_300_57217, &P21940_298};	/*Property = DataSet*/
const Value V21940_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_301 = {S22, 0, &V21940_301_57220, &P21940_300};	/*Property = DataUrl*/
const Value V21940_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_302 = {S23, 0, &V21940_302_57223, &P21940_301};	/*Property = DeviceOrientation*/
const Value V21940_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_304 = {S24, 0, &V21940_304_57229, &P21940_302};	/*Property = FileReader*/
const Value V21940_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_305 = {S25, 0, &V21940_305_57232, &P21940_304};	/*Property = FileSaver*/
const Value V21940_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_306 = {S26, 0, &V21940_306_57238, &P21940_305};	/*Property = FileWriter*/
const Value V21940_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_307 = {S27, 0, &V21940_307_57241, &P21940_306};	/*Property = FormData*/
const Value V21940_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_309 = {S28, 0, &V21940_309_57247, &P21940_307};	/*Property = Fullscreen*/
const Value V21940_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_310 = {S29, 0, &V21940_310_57250, &P21940_309};	/*Property = GeoLocation*/
const Value V21940_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_312 = {S30, 0, &V21940_312_57256, &P21940_310};	/*Property = History*/
const Value V21940_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_314 = {S31, 0, &V21940_314_57262, &P21940_312};	/*Property = Html5*/
const Value V21940_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_313 = {S32, 0, &V21940_313_57259, &P21940_314};	/*Property = Html-Media-Capture*/
const Value V21940_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_315 = {S33, 0, &V21940_315_57265, &P21940_313};	/*Property = Iframe*/
const Value V21940_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_316 = {S34, 0, &V21940_316_57268, &P21940_315};	/*Property = IndexedDB*/
const Value V21940_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_317 = {S35, 0, &V21940_317_57271, &P21940_316};	/*Property = Json*/
const Value V21940_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P21940_15 = {S37, 0, &V21940_15_29, &P21940_317};	/*Property = LayoutEngine*/
const Value V21940_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_344 = {S38, 0, &V21940_344_57274, &P21940_15};	/*Property = Masking*/
const Value V21940_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_320 = {S39, 0, &V21940_320_57292, &P21940_344};	/*Property = PostMessage*/
const Value V21940_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_321 = {S40, 0, &V21940_321_57295, &P21940_320};	/*Property = Progress*/
const Value V21940_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_322 = {S41, 0, &V21940_322_57298, &P21940_321};	/*Property = Prompts*/
const Value V21940_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_323 = {S42, 0, &V21940_323_57301, &P21940_322};	/*Property = Selector*/
const Value V21940_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_324 = {S43, 0, &V21940_324_57307, &P21940_323};	/*Property = Svg*/
const Value V21940_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_327 = {S44, 0, &V21940_327_57316, &P21940_324};	/*Property = TouchEvents*/
const Value V21940_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_328 = {S45, 0, &V21940_328_57319, &P21940_327};	/*Property = Track*/
const Value V21940_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_329 = {S46, 0, &V21940_329_57325, &P21940_328};	/*Property = Video*/
const Value V21940_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_330 = {S47, 0, &V21940_330_57328, &P21940_329};	/*Property = Viewport*/
const Value V21940_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_332 = {S48, 0, &V21940_332_57340, &P21940_330};	/*Property = WebWorkers*/
const Value V21940_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21940_333 = {S49, 0, &V21940_333_57343, &P21940_332};	/*Property = Xhr2*/
const Profile P21940 = {"21940", &P21940_333};	/*Profile ID - 21940*/
const Value V20983_1_1 = {S2, NULL};	/*Value = True*/
const Property P20983_1 = {S50, 0, &V20983_1_1, NULL};	/*Property = IsMobile*/
const Value V20983_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20983_5 = {S52, 0, &V20983_5_9981, &P20983_1};	/*Property = ScreenPixelsHeight*/
const Value V20983_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20983_4 = {S54, 0, &V20983_4_39098, &P20983_5};	/*Property = ScreenPixelsWidth*/
const Profile P20983 = {"20983", &P20983_4};	/*Profile ID - 20983*/
const Value V20076_1_1 = {S2, NULL};	/*Value = True*/
const Property P20076_1 = {S50, 0, &V20076_1_1, NULL};	/*Property = IsMobile*/
const Value V20076_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20076_5 = {S52, 0, &V20076_5_9981, &P20076_1};	/*Property = ScreenPixelsHeight*/
const Value V20076_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20076_4 = {S54, 0, &V20076_4_39098, &P20076_5};	/*Property = ScreenPixelsWidth*/
const Profile P20076 = {"20076", &P20076_4};	/*Profile ID - 20076*/
const Value V22103_1_1 = {S2, NULL};	/*Value = True*/
const Property P22103_1 = {S50, 0, &V22103_1_1, NULL};	/*Property = IsMobile*/
const Value V22103_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22103_5 = {S52, 0, &V22103_5_1227, &P22103_1};	/*Property = ScreenPixelsHeight*/
const Value V22103_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22103_4 = {S54, 0, &V22103_4_1229, &P22103_5};	/*Property = ScreenPixelsWidth*/
const Profile P22103 = {"22103", &P22103_4};	/*Profile ID - 22103*/
const Value V18228_1_1 = {S2, NULL};	/*Value = True*/
const Property P18228_1 = {S50, 0, &V18228_1_1, NULL};	/*Property = IsMobile*/
const Value V18228_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18228_5 = {S52, 0, &V18228_5_1277, &P18228_1};	/*Property = ScreenPixelsHeight*/
const Value V18228_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18228_4 = {S54, 0, &V18228_4_1279, &P18228_5};	/*Property = ScreenPixelsWidth*/
const Profile P18228 = {"18228", &P18228_4};	/*Profile ID - 18228*/
const Value V20382_1_1 = {S2, NULL};	/*Value = True*/
const Property P20382_1 = {S50, 0, &V20382_1_1, NULL};	/*Property = IsMobile*/
const Value V20382_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20382_5 = {S52, 0, &V20382_5_37, &P20382_1};	/*Property = ScreenPixelsHeight*/
const Value V20382_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20382_4 = {S54, 0, &V20382_4_36, &P20382_5};	/*Property = ScreenPixelsWidth*/
const Profile P20382 = {"20382", &P20382_4};	/*Profile ID - 20382*/
const Value V21297_1_1 = {S2, NULL};	/*Value = True*/
const Property P21297_1 = {S50, 0, &V21297_1_1, NULL};	/*Property = IsMobile*/
const Value V21297_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21297_5 = {S52, 0, &V21297_5_1227, &P21297_1};	/*Property = ScreenPixelsHeight*/
const Value V21297_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21297_4 = {S54, 0, &V21297_4_1229, &P21297_5};	/*Property = ScreenPixelsWidth*/
const Profile P21297 = {"21297", &P21297_4};	/*Profile ID - 21297*/
const Value V21722_1_1 = {S2, NULL};	/*Value = True*/
const Property P21722_1 = {S50, 0, &V21722_1_1, NULL};	/*Property = IsMobile*/
const Value V21722_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21722_5 = {S52, 0, &V21722_5_1222, &P21722_1};	/*Property = ScreenPixelsHeight*/
const Value V21722_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21722_4 = {S54, 0, &V21722_4_1224, &P21722_5};	/*Property = ScreenPixelsWidth*/
const Profile P21722 = {"21722", &P21722_4};	/*Profile ID - 21722*/
const Value V22117_1_1 = {S2, NULL};	/*Value = True*/
const Property P22117_1 = {S50, 0, &V22117_1_1, NULL};	/*Property = IsMobile*/
const Value V22117_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22117_5 = {S52, 0, &V22117_5_1277, &P22117_1};	/*Property = ScreenPixelsHeight*/
const Value V22117_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22117_4 = {S54, 0, &V22117_4_2336, &P22117_5};	/*Property = ScreenPixelsWidth*/
const Profile P22117 = {"22117", &P22117_4};	/*Profile ID - 22117*/
const Value V22008_1_1 = {S2, NULL};	/*Value = True*/
const Property P22008_1 = {S50, 0, &V22008_1_1, NULL};	/*Property = IsMobile*/
const Value V22008_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22008_5 = {S52, 0, &V22008_5_1832, &P22008_1};	/*Property = ScreenPixelsHeight*/
const Value V22008_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22008_4 = {S54, 0, &V22008_4_2336, &P22008_5};	/*Property = ScreenPixelsWidth*/
const Profile P22008 = {"22008", &P22008_4};	/*Profile ID - 22008*/
const Value V22160_1_1 = {S2, NULL};	/*Value = True*/
const Property P22160_1 = {S50, 0, &V22160_1_1, NULL};	/*Property = IsMobile*/
const Value V22160_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22160_5 = {S52, 0, &V22160_5_1227, &P22160_1};	/*Property = ScreenPixelsHeight*/
const Value V22160_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P22160_4 = {S54, 0, &V22160_4_1943, &P22160_5};	/*Property = ScreenPixelsWidth*/
const Profile P22160 = {"22160", &P22160_4};	/*Profile ID - 22160*/
const Value V18302_1_1 = {S2, NULL};	/*Value = True*/
const Property P18302_1 = {S50, 0, &V18302_1_1, NULL};	/*Property = IsMobile*/
const Value V18302_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P18302_5 = {S52, 0, &V18302_5_1269, &P18302_1};	/*Property = ScreenPixelsHeight*/
const Value V18302_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18302_4 = {S54, 0, &V18302_4_1229, &P18302_5};	/*Property = ScreenPixelsWidth*/
const Profile P18302 = {"18302", &P18302_4};	/*Profile ID - 18302*/
const Value V21233_1_1 = {S2, NULL};	/*Value = True*/
const Property P21233_1 = {S50, 0, &V21233_1_1, NULL};	/*Property = IsMobile*/
const Value V21233_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21233_5 = {S52, 0, &V21233_5_1277, &P21233_1};	/*Property = ScreenPixelsHeight*/
const Value V21233_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21233_4 = {S54, 0, &V21233_4_1279, &P21233_5};	/*Property = ScreenPixelsWidth*/
const Profile P21233 = {"21233", &P21233_4};	/*Profile ID - 21233*/
const Value V21752_1_1 = {S2, NULL};	/*Value = True*/
const Property P21752_1 = {S50, 0, &V21752_1_1, NULL};	/*Property = IsMobile*/
const Value V21752_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21752_5 = {S52, 0, &V21752_5_1403, &P21752_1};	/*Property = ScreenPixelsHeight*/
const Value V21752_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P21752_4 = {S54, 0, &V21752_4_8554, &P21752_5};	/*Property = ScreenPixelsWidth*/
const Profile P21752 = {"21752", &P21752_4};	/*Profile ID - 21752*/
const Value V22135_1_1 = {S2, NULL};	/*Value = True*/
const Property P22135_1 = {S50, 0, &V22135_1_1, NULL};	/*Property = IsMobile*/
const Value V22135_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22135_5 = {S52, 0, &V22135_5_1227, &P22135_1};	/*Property = ScreenPixelsHeight*/
const Value V22135_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22135_4 = {S54, 0, &V22135_4_1229, &P22135_5};	/*Property = ScreenPixelsWidth*/
const Profile P22135 = {"22135", &P22135_4};	/*Profile ID - 22135*/
const Value V22017_1_1 = {S2, NULL};	/*Value = True*/
const Property P22017_1 = {S50, 0, &V22017_1_1, NULL};	/*Property = IsMobile*/
const Value V22017_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22017_5 = {S52, 0, &V22017_5_37569, &P22017_1};	/*Property = ScreenPixelsHeight*/
const Value V22017_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22017_4 = {S54, 0, &V22017_4_5645, &P22017_5};	/*Property = ScreenPixelsWidth*/
const Profile P22017 = {"22017", &P22017_4};	/*Profile ID - 22017*/
const Value V20734_1_1 = {S2, NULL};	/*Value = True*/
const Property P20734_1 = {S50, 0, &V20734_1_1, NULL};	/*Property = IsMobile*/
const Value V20734_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20734_5 = {S52, 0, &V20734_5_1277, &P20734_1};	/*Property = ScreenPixelsHeight*/
const Value V20734_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20734_4 = {S54, 0, &V20734_4_1279, &P20734_5};	/*Property = ScreenPixelsWidth*/
const Profile P20734 = {"20734", &P20734_4};	/*Profile ID - 20734*/
const Value V21532_1_1 = {S2, NULL};	/*Value = True*/
const Property P21532_1 = {S50, 0, &V21532_1_1, NULL};	/*Property = IsMobile*/
const Value V21532_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21532_5 = {S52, 0, &V21532_5_1277, &P21532_1};	/*Property = ScreenPixelsHeight*/
const Value V21532_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21532_4 = {S54, 0, &V21532_4_1279, &P21532_5};	/*Property = ScreenPixelsWidth*/
const Profile P21532 = {"21532", &P21532_4};	/*Profile ID - 21532*/
const Value V20945_1_1 = {S2, NULL};	/*Value = True*/
const Property P20945_1 = {S50, 0, &V20945_1_1, NULL};	/*Property = IsMobile*/
const Value V20945_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20945_5 = {S52, 0, &V20945_5_1277, &P20945_1};	/*Property = ScreenPixelsHeight*/
const Value V20945_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20945_4 = {S54, 0, &V20945_4_1279, &P20945_5};	/*Property = ScreenPixelsWidth*/
const Profile P20945 = {"20945", &P20945_4};	/*Profile ID - 20945*/
const Value V20251_1_1 = {S2, NULL};	/*Value = True*/
const Property P20251_1 = {S50, 0, &V20251_1_1, NULL};	/*Property = IsMobile*/
const Value V20251_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20251_5 = {S52, 0, &V20251_5_1227, &P20251_1};	/*Property = ScreenPixelsHeight*/
const Value V20251_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20251_4 = {S54, 0, &V20251_4_1229, &P20251_5};	/*Property = ScreenPixelsWidth*/
const Profile P20251 = {"20251", &P20251_4};	/*Profile ID - 20251*/
const Value V23791_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_279 = {S1, 0, &V23791_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23791_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_282 = {S3, 0, &V23791_282_57155, &P23791_279};	/*Property = BlobBuilder*/
const Value V23791_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_350 = {S4, 0, &V23791_350_57446, &P23791_282};	/*Property = Canvas*/
const Value V23791_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_283 = {S5, 0, &V23791_283_57161, &P23791_350};	/*Property = CssBackground*/
const Value V23791_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_284 = {S6, 0, &V23791_284_57164, &P23791_283};	/*Property = CssBorderImage*/
const Value V23791_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_285 = {S7, 0, &V23791_285_57167, &P23791_284};	/*Property = CssCanvas*/
const Value V23791_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_286 = {S8, 0, &V23791_286_57170, &P23791_285};	/*Property = CssColor*/
const Value V23791_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_237 = {S9, 0, &V23791_237_55786, &P23791_286};	/*Property = CssColumn*/
const Value V23791_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_287 = {S10, 0, &V23791_287_57173, &P23791_237};	/*Property = CssFlexbox*/
const Value V23791_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_288 = {S11, 0, &V23791_288_57176, &P23791_287};	/*Property = CssFont*/
const Value V23791_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_289 = {S12, 0, &V23791_289_86899, &P23791_288};	/*Property = CssImages*/
const Value V23791_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_291 = {S13, 0, &V23791_291_57182, &P23791_289};	/*Property = CssMediaQueries*/
const Value V23791_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_292 = {S14, 0, &V23791_292_57185, &P23791_291};	/*Property = CssMinMax*/
const Value V23791_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_293 = {S15, 0, &V23791_293_57188, &P23791_292};	/*Property = CssOverflow*/
const Value V23791_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_294 = {S16, 0, &V23791_294_57191, &P23791_293};	/*Property = CssPosition*/
const Value V23791_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_295 = {S17, 0, &V23791_295_57194, &P23791_294};	/*Property = CssText*/
const Value V23791_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_297 = {S18, 0, &V23791_297_57197, &P23791_295};	/*Property = CssTransforms*/
const Value V23791_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_241 = {S19, 0, &V23791_241_55801, &P23791_297};	/*Property = CssTransitions*/
const Value V23791_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_298 = {S20, 0, &V23791_298_57208, &P23791_241};	/*Property = CssUI*/
const Value V23791_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_300 = {S21, 0, &V23791_300_57217, &P23791_298};	/*Property = DataSet*/
const Value V23791_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_301 = {S22, 0, &V23791_301_57220, &P23791_300};	/*Property = DataUrl*/
const Value V23791_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_302 = {S23, 0, &V23791_302_57223, &P23791_301};	/*Property = DeviceOrientation*/
const Value V23791_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_304 = {S24, 0, &V23791_304_57229, &P23791_302};	/*Property = FileReader*/
const Value V23791_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_305 = {S25, 0, &V23791_305_57232, &P23791_304};	/*Property = FileSaver*/
const Value V23791_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_306 = {S26, 0, &V23791_306_57238, &P23791_305};	/*Property = FileWriter*/
const Value V23791_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_307 = {S27, 0, &V23791_307_57241, &P23791_306};	/*Property = FormData*/
const Value V23791_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_309 = {S28, 0, &V23791_309_57247, &P23791_307};	/*Property = Fullscreen*/
const Value V23791_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_310 = {S29, 0, &V23791_310_57250, &P23791_309};	/*Property = GeoLocation*/
const Value V23791_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_312 = {S30, 0, &V23791_312_57256, &P23791_310};	/*Property = History*/
const Value V23791_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_314 = {S31, 0, &V23791_314_57262, &P23791_312};	/*Property = Html5*/
const Value V23791_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_313 = {S32, 0, &V23791_313_57259, &P23791_314};	/*Property = Html-Media-Capture*/
const Value V23791_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_315 = {S33, 0, &V23791_315_57265, &P23791_313};	/*Property = Iframe*/
const Value V23791_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_316 = {S34, 0, &V23791_316_57268, &P23791_315};	/*Property = IndexedDB*/
const Value V23791_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_317 = {S35, 0, &V23791_317_57271, &P23791_316};	/*Property = Json*/
const Value V23791_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_15 = {S37, 0, &V23791_15_72, &P23791_317};	/*Property = LayoutEngine*/
const Value V23791_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_344 = {S38, 0, &V23791_344_57274, &P23791_15};	/*Property = Masking*/
const Value V23791_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_320 = {S39, 0, &V23791_320_57292, &P23791_344};	/*Property = PostMessage*/
const Value V23791_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_321 = {S40, 0, &V23791_321_57295, &P23791_320};	/*Property = Progress*/
const Value V23791_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_322 = {S41, 0, &V23791_322_57298, &P23791_321};	/*Property = Prompts*/
const Value V23791_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_323 = {S42, 0, &V23791_323_57301, &P23791_322};	/*Property = Selector*/
const Value V23791_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_324 = {S43, 0, &V23791_324_57307, &P23791_323};	/*Property = Svg*/
const Value V23791_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_327 = {S44, 0, &V23791_327_57316, &P23791_324};	/*Property = TouchEvents*/
const Value V23791_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_328 = {S45, 0, &V23791_328_57319, &P23791_327};	/*Property = Track*/
const Value V23791_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_329 = {S46, 0, &V23791_329_57325, &P23791_328};	/*Property = Video*/
const Value V23791_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_330 = {S47, 0, &V23791_330_57328, &P23791_329};	/*Property = Viewport*/
const Value V23791_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_332 = {S48, 0, &V23791_332_57340, &P23791_330};	/*Property = WebWorkers*/
const Value V23791_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23791_333 = {S49, 0, &V23791_333_57343, &P23791_332};	/*Property = Xhr2*/
const Profile P23791 = {"23791", &P23791_333};	/*Profile ID - 23791*/
const Value V18459_1_1 = {S2, NULL};	/*Value = True*/
const Property P18459_1 = {S50, 0, &V18459_1_1, NULL};	/*Property = IsMobile*/
const Value V18459_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P18459_5 = {S52, 0, &V18459_5_1458, &P18459_1};	/*Property = ScreenPixelsHeight*/
const Value V18459_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P18459_4 = {S54, 0, &V18459_4_37617, &P18459_5};	/*Property = ScreenPixelsWidth*/
const Profile P18459 = {"18459", &P18459_4};	/*Profile ID - 18459*/
const Value V21112_1_1 = {S2, NULL};	/*Value = True*/
const Property P21112_1 = {S50, 0, &V21112_1_1, NULL};	/*Property = IsMobile*/
const Value V21112_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21112_5 = {S52, 0, &V21112_5_1227, &P21112_1};	/*Property = ScreenPixelsHeight*/
const Value V21112_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21112_4 = {S54, 0, &V21112_4_1229, &P21112_5};	/*Property = ScreenPixelsWidth*/
const Profile P21112 = {"21112", &P21112_4};	/*Profile ID - 21112*/
const Value V20556_1_1 = {S2, NULL};	/*Value = True*/
const Property P20556_1 = {S50, 0, &V20556_1_1, NULL};	/*Property = IsMobile*/
const Value V20556_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P20556_5 = {S52, 0, &V20556_5_59138, &P20556_1};	/*Property = ScreenPixelsHeight*/
const Value V20556_4_52971 = {S122, NULL};	/*Value = 1200*/
const Property P20556_4 = {S54, 0, &V20556_4_52971, &P20556_5};	/*Property = ScreenPixelsWidth*/
const Profile P20556 = {"20556", &P20556_4};	/*Profile ID - 20556*/
const Value V20454_1_1 = {S2, NULL};	/*Value = True*/
const Property P20454_1 = {S50, 0, &V20454_1_1, NULL};	/*Property = IsMobile*/
const Value V20454_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20454_5 = {S52, 0, &V20454_5_1277, &P20454_1};	/*Property = ScreenPixelsHeight*/
const Value V20454_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20454_4 = {S54, 0, &V20454_4_1279, &P20454_5};	/*Property = ScreenPixelsWidth*/
const Profile P20454 = {"20454", &P20454_4};	/*Profile ID - 20454*/
const Value V21293_1_1 = {S2, NULL};	/*Value = True*/
const Property P21293_1 = {S50, 0, &V21293_1_1, NULL};	/*Property = IsMobile*/
const Value V21293_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21293_5 = {S52, 0, &V21293_5_1277, &P21293_1};	/*Property = ScreenPixelsHeight*/
const Value V21293_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21293_4 = {S54, 0, &V21293_4_1279, &P21293_5};	/*Property = ScreenPixelsWidth*/
const Profile P21293 = {"21293", &P21293_4};	/*Profile ID - 21293*/
const Value V22101_1_1 = {S2, NULL};	/*Value = True*/
const Property P22101_1 = {S50, 0, &V22101_1_1, NULL};	/*Property = IsMobile*/
const Value V22101_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22101_5 = {S52, 0, &V22101_5_37569, &P22101_1};	/*Property = ScreenPixelsHeight*/
const Value V22101_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22101_4 = {S54, 0, &V22101_4_5645, &P22101_5};	/*Property = ScreenPixelsWidth*/
const Profile P22101 = {"22101", &P22101_4};	/*Profile ID - 22101*/
const Value V21716_1_1 = {S2, NULL};	/*Value = True*/
const Property P21716_1 = {S50, 0, &V21716_1_1, NULL};	/*Property = IsMobile*/
const Value V21716_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21716_5 = {S52, 0, &V21716_5_1227, &P21716_1};	/*Property = ScreenPixelsHeight*/
const Value V21716_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21716_4 = {S54, 0, &V21716_4_1229, &P21716_5};	/*Property = ScreenPixelsWidth*/
const Profile P21716 = {"21716", &P21716_4};	/*Profile ID - 21716*/
const Value V20579_1_1 = {S2, NULL};	/*Value = True*/
const Property P20579_1 = {S50, 0, &V20579_1_1, NULL};	/*Property = IsMobile*/
const Value V20579_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20579_5 = {S52, 0, &V20579_5_9981, &P20579_1};	/*Property = ScreenPixelsHeight*/
const Value V20579_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20579_4 = {S54, 0, &V20579_4_2336, &P20579_5};	/*Property = ScreenPixelsWidth*/
const Profile P20579 = {"20579", &P20579_4};	/*Profile ID - 20579*/
const Value V20894_1_1 = {S2, NULL};	/*Value = True*/
const Property P20894_1 = {S50, 0, &V20894_1_1, NULL};	/*Property = IsMobile*/
const Value V20894_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20894_5 = {S52, 0, &V20894_5_1222, &P20894_1};	/*Property = ScreenPixelsHeight*/
const Value V20894_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20894_4 = {S54, 0, &V20894_4_1224, &P20894_5};	/*Property = ScreenPixelsWidth*/
const Profile P20894 = {"20894", &P20894_4};	/*Profile ID - 20894*/
const Value V18524_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18524_279 = {S1, 0, &V18524_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18524_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18524_282 = {S3, 0, &V18524_282_57154, &P18524_279};	/*Property = BlobBuilder*/
const Value V18524_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18524_350 = {S4, 0, &V18524_350_57444, &P18524_282};	/*Property = Canvas*/
const Value V18524_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18524_283 = {S5, 0, &V18524_283_57159, &P18524_350};	/*Property = CssBackground*/
const Value V18524_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18524_284 = {S6, 0, &V18524_284_57163, &P18524_283};	/*Property = CssBorderImage*/
const Value V18524_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18524_285 = {S7, 0, &V18524_285_57166, &P18524_284};	/*Property = CssCanvas*/
const Value V18524_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18524_286 = {S8, 0, &V18524_286_57168, &P18524_285};	/*Property = CssColor*/
const Value V18524_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18524_237 = {S9, 0, &V18524_237_55786, &P18524_286};	/*Property = CssColumn*/
const Value V18524_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18524_287 = {S10, 0, &V18524_287_57172, &P18524_237};	/*Property = CssFlexbox*/
const Value V18524_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18524_288 = {S11, 0, &V18524_288_57175, &P18524_287};	/*Property = CssFont*/
const Value V18524_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18524_289 = {S12, 0, &V18524_289_57759, &P18524_288};	/*Property = CssImages*/
const Value V18524_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18524_291 = {S13, 0, &V18524_291_57181, &P18524_289};	/*Property = CssMediaQueries*/
const Value V18524_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18524_292 = {S14, 0, &V18524_292_57183, &P18524_291};	/*Property = CssMinMax*/
const Value V18524_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18524_293 = {S15, 0, &V18524_293_57187, &P18524_292};	/*Property = CssOverflow*/
const Value V18524_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18524_294 = {S16, 0, &V18524_294_57190, &P18524_293};	/*Property = CssPosition*/
const Value V18524_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18524_295 = {S17, 0, &V18524_295_57192, &P18524_294};	/*Property = CssText*/
const Value V18524_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18524_297 = {S18, 0, &V18524_297_57195, &P18524_295};	/*Property = CssTransforms*/
const Value V18524_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18524_241 = {S19, 0, &V18524_241_55799, &P18524_297};	/*Property = CssTransitions*/
const Value V18524_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18524_298 = {S20, 0, &V18524_298_57206, &P18524_241};	/*Property = CssUI*/
const Value V18524_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18524_300 = {S21, 0, &V18524_300_57216, &P18524_298};	/*Property = DataSet*/
const Value V18524_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18524_301 = {S22, 0, &V18524_301_57218, &P18524_300};	/*Property = DataUrl*/
const Value V18524_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18524_302 = {S23, 0, &V18524_302_57222, &P18524_301};	/*Property = DeviceOrientation*/
const Value V18524_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18524_304 = {S24, 0, &V18524_304_57227, &P18524_302};	/*Property = FileReader*/
const Value V18524_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18524_305 = {S25, 0, &V18524_305_57231, &P18524_304};	/*Property = FileSaver*/
const Value V18524_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18524_306 = {S26, 0, &V18524_306_57237, &P18524_305};	/*Property = FileWriter*/
const Value V18524_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18524_307 = {S27, 0, &V18524_307_57239, &P18524_306};	/*Property = FormData*/
const Value V18524_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18524_309 = {S28, 0, &V18524_309_57246, &P18524_307};	/*Property = Fullscreen*/
const Value V18524_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18524_310 = {S29, 0, &V18524_310_57248, &P18524_309};	/*Property = GeoLocation*/
const Value V18524_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18524_312 = {S30, 0, &V18524_312_57255, &P18524_310};	/*Property = History*/
const Value V18524_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18524_314 = {S31, 0, &V18524_314_57261, &P18524_312};	/*Property = Html5*/
const Value V18524_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18524_313 = {S32, 0, &V18524_313_57258, &P18524_314};	/*Property = Html-Media-Capture*/
const Value V18524_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18524_315 = {S33, 0, &V18524_315_57264, &P18524_313};	/*Property = Iframe*/
const Value V18524_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18524_316 = {S34, 0, &V18524_316_57267, &P18524_315};	/*Property = IndexedDB*/
const Value V18524_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18524_317 = {S35, 0, &V18524_317_57269, &P18524_316};	/*Property = Json*/
const Value V18524_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18524_15 = {S37, 0, &V18524_15_28, &P18524_317};	/*Property = LayoutEngine*/
const Value V18524_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18524_344 = {S38, 0, &V18524_344_57272, &P18524_15};	/*Property = Masking*/
const Value V18524_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18524_320 = {S39, 0, &V18524_320_57290, &P18524_344};	/*Property = PostMessage*/
const Value V18524_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18524_321 = {S40, 0, &V18524_321_57293, &P18524_320};	/*Property = Progress*/
const Value V18524_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18524_322 = {S41, 0, &V18524_322_57296, &P18524_321};	/*Property = Prompts*/
const Value V18524_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18524_323 = {S42, 0, &V18524_323_57299, &P18524_322};	/*Property = Selector*/
const Value V18524_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18524_324 = {S43, 0, &V18524_324_57306, &P18524_323};	/*Property = Svg*/
const Value V18524_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18524_327 = {S44, 0, &V18524_327_57315, &P18524_324};	/*Property = TouchEvents*/
const Value V18524_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18524_328 = {S45, 0, &V18524_328_57318, &P18524_327};	/*Property = Track*/
const Value V18524_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18524_329 = {S46, 0, &V18524_329_57323, &P18524_328};	/*Property = Video*/
const Value V18524_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18524_330 = {S47, 0, &V18524_330_57326, &P18524_329};	/*Property = Viewport*/
const Value V18524_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18524_332 = {S48, 0, &V18524_332_57339, &P18524_330};	/*Property = WebWorkers*/
const Value V18524_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18524_333 = {S49, 0, &V18524_333_57342, &P18524_332};	/*Property = Xhr2*/
const Profile P18524 = {"18524", &P18524_333};	/*Profile ID - 18524*/
const Value V23999_1_1 = {S2, NULL};	/*Value = True*/
const Property P23999_1 = {S50, 0, &V23999_1_1, NULL};	/*Property = IsMobile*/
const Value V23999_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23999_5 = {S52, 0, &V23999_5_1458, &P23999_1};	/*Property = ScreenPixelsHeight*/
const Value V23999_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23999_4 = {S54, 0, &V23999_4_1460, &P23999_5};	/*Property = ScreenPixelsWidth*/
const Profile P23999 = {"23999", &P23999_4};	/*Profile ID - 23999*/
const Value V20135_1_1 = {S2, NULL};	/*Value = True*/
const Property P20135_1 = {S50, 0, &V20135_1_1, NULL};	/*Property = IsMobile*/
const Value V20135_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20135_5 = {S52, 0, &V20135_5_1227, &P20135_1};	/*Property = ScreenPixelsHeight*/
const Value V20135_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20135_4 = {S54, 0, &V20135_4_1229, &P20135_5};	/*Property = ScreenPixelsWidth*/
const Profile P20135 = {"20135", &P20135_4};	/*Profile ID - 20135*/
const Value V24021_1_1 = {S2, NULL};	/*Value = True*/
const Property P24021_1 = {S50, 0, &V24021_1_1, NULL};	/*Property = IsMobile*/
const Value V24021_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24021_5 = {S52, 0, &V24021_5_1277, &P24021_1};	/*Property = ScreenPixelsHeight*/
const Value V24021_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24021_4 = {S54, 0, &V24021_4_1279, &P24021_5};	/*Property = ScreenPixelsWidth*/
const Profile P24021 = {"24021", &P24021_4};	/*Profile ID - 24021*/
const Value V21659_1_1 = {S2, NULL};	/*Value = True*/
const Property P21659_1 = {S50, 0, &V21659_1_1, NULL};	/*Property = IsMobile*/
const Value V21659_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21659_5 = {S52, 0, &V21659_5_1277, &P21659_1};	/*Property = ScreenPixelsHeight*/
const Value V21659_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21659_4 = {S54, 0, &V21659_4_1279, &P21659_5};	/*Property = ScreenPixelsWidth*/
const Profile P21659 = {"21659", &P21659_4};	/*Profile ID - 21659*/
const Value V20287_1_1 = {S2, NULL};	/*Value = True*/
const Property P20287_1 = {S50, 0, &V20287_1_1, NULL};	/*Property = IsMobile*/
const Value V20287_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P20287_5 = {S52, 0, &V20287_5_1269, &P20287_1};	/*Property = ScreenPixelsHeight*/
const Value V20287_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20287_4 = {S54, 0, &V20287_4_1229, &P20287_5};	/*Property = ScreenPixelsWidth*/
const Profile P20287 = {"20287", &P20287_4};	/*Profile ID - 20287*/
const Value V21242_1_1 = {S2, NULL};	/*Value = True*/
const Property P21242_1 = {S50, 0, &V21242_1_1, NULL};	/*Property = IsMobile*/
const Value V21242_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21242_5 = {S52, 0, &V21242_5_1277, &P21242_1};	/*Property = ScreenPixelsHeight*/
const Value V21242_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21242_4 = {S54, 0, &V21242_4_1279, &P21242_5};	/*Property = ScreenPixelsWidth*/
const Profile P21242 = {"21242", &P21242_4};	/*Profile ID - 21242*/
const Value V18708_1_1 = {S2, NULL};	/*Value = True*/
const Property P18708_1 = {S50, 0, &V18708_1_1, NULL};	/*Property = IsMobile*/
const Value V18708_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18708_5 = {S52, 0, &V18708_5_9981, &P18708_1};	/*Property = ScreenPixelsHeight*/
const Value V18708_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18708_4 = {S54, 0, &V18708_4_2336, &P18708_5};	/*Property = ScreenPixelsWidth*/
const Profile P18708 = {"18708", &P18708_4};	/*Profile ID - 18708*/
const Value V22062_1_1 = {S2, NULL};	/*Value = True*/
const Property P22062_1 = {S50, 0, &V22062_1_1, NULL};	/*Property = IsMobile*/
const Value V22062_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22062_5 = {S52, 0, &V22062_5_1458, &P22062_1};	/*Property = ScreenPixelsHeight*/
const Value V22062_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P22062_4 = {S54, 0, &V22062_4_37617, &P22062_5};	/*Property = ScreenPixelsWidth*/
const Profile P22062 = {"22062", &P22062_4};	/*Profile ID - 22062*/
const Value V20067_1_1 = {S2, NULL};	/*Value = True*/
const Property P20067_1 = {S50, 0, &V20067_1_1, NULL};	/*Property = IsMobile*/
const Value V20067_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20067_5 = {S52, 0, &V20067_5_1277, &P20067_1};	/*Property = ScreenPixelsHeight*/
const Value V20067_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P20067_4 = {S54, 0, &V20067_4_1278, &P20067_5};	/*Property = ScreenPixelsWidth*/
const Profile P20067 = {"20067", &P20067_4};	/*Profile ID - 20067*/
const Value V21653_1_1 = {S2, NULL};	/*Value = True*/
const Property P21653_1 = {S50, 0, &V21653_1_1, NULL};	/*Property = IsMobile*/
const Value V21653_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21653_5 = {S52, 0, &V21653_5_37, &P21653_1};	/*Property = ScreenPixelsHeight*/
const Value V21653_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21653_4 = {S54, 0, &V21653_4_36, &P21653_5};	/*Property = ScreenPixelsWidth*/
const Profile P21653 = {"21653", &P21653_4};	/*Profile ID - 21653*/
const Value V24738_1_1 = {S2, NULL};	/*Value = True*/
const Property P24738_1 = {S50, 0, &V24738_1_1, NULL};	/*Property = IsMobile*/
const Value V24738_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24738_5 = {S52, 0, &V24738_5_1277, &P24738_1};	/*Property = ScreenPixelsHeight*/
const Value V24738_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24738_4 = {S54, 0, &V24738_4_1279, &P24738_5};	/*Property = ScreenPixelsWidth*/
const Profile P24738 = {"24738", &P24738_4};	/*Profile ID - 24738*/
const Value V24017_1_1 = {S2, NULL};	/*Value = True*/
const Property P24017_1 = {S50, 0, &V24017_1_1, NULL};	/*Property = IsMobile*/
const Value V24017_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P24017_5 = {S52, 0, &V24017_5_1222, &P24017_1};	/*Property = ScreenPixelsHeight*/
const Value V24017_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P24017_4 = {S54, 0, &V24017_4_1224, &P24017_5};	/*Property = ScreenPixelsWidth*/
const Profile P24017 = {"24017", &P24017_4};	/*Profile ID - 24017*/
const Value V20031_1_1 = {S2, NULL};	/*Value = True*/
const Property P20031_1 = {S50, 0, &V20031_1_1, NULL};	/*Property = IsMobile*/
const Value V20031_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20031_5 = {S52, 0, &V20031_5_1222, &P20031_1};	/*Property = ScreenPixelsHeight*/
const Value V20031_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20031_4 = {S54, 0, &V20031_4_1224, &P20031_5};	/*Property = ScreenPixelsWidth*/
const Profile P20031 = {"20031", &P20031_4};	/*Profile ID - 20031*/
const Value V21795_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_279 = {S1, 0, &V21795_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21795_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_282 = {S3, 0, &V21795_282_57155, &P21795_279};	/*Property = BlobBuilder*/
const Value V21795_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_350 = {S4, 0, &V21795_350_57446, &P21795_282};	/*Property = Canvas*/
const Value V21795_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_283 = {S5, 0, &V21795_283_57161, &P21795_350};	/*Property = CssBackground*/
const Value V21795_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_284 = {S6, 0, &V21795_284_57164, &P21795_283};	/*Property = CssBorderImage*/
const Value V21795_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_285 = {S7, 0, &V21795_285_57167, &P21795_284};	/*Property = CssCanvas*/
const Value V21795_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_286 = {S8, 0, &V21795_286_57170, &P21795_285};	/*Property = CssColor*/
const Value V21795_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_237 = {S9, 0, &V21795_237_55786, &P21795_286};	/*Property = CssColumn*/
const Value V21795_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_287 = {S10, 0, &V21795_287_57173, &P21795_237};	/*Property = CssFlexbox*/
const Value V21795_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_288 = {S11, 0, &V21795_288_57176, &P21795_287};	/*Property = CssFont*/
const Value V21795_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_289 = {S12, 0, &V21795_289_86899, &P21795_288};	/*Property = CssImages*/
const Value V21795_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_291 = {S13, 0, &V21795_291_57182, &P21795_289};	/*Property = CssMediaQueries*/
const Value V21795_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_292 = {S14, 0, &V21795_292_57185, &P21795_291};	/*Property = CssMinMax*/
const Value V21795_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_293 = {S15, 0, &V21795_293_57188, &P21795_292};	/*Property = CssOverflow*/
const Value V21795_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_294 = {S16, 0, &V21795_294_57191, &P21795_293};	/*Property = CssPosition*/
const Value V21795_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_295 = {S17, 0, &V21795_295_57194, &P21795_294};	/*Property = CssText*/
const Value V21795_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_297 = {S18, 0, &V21795_297_57197, &P21795_295};	/*Property = CssTransforms*/
const Value V21795_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_241 = {S19, 0, &V21795_241_55801, &P21795_297};	/*Property = CssTransitions*/
const Value V21795_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_298 = {S20, 0, &V21795_298_57208, &P21795_241};	/*Property = CssUI*/
const Value V21795_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_300 = {S21, 0, &V21795_300_57217, &P21795_298};	/*Property = DataSet*/
const Value V21795_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_301 = {S22, 0, &V21795_301_57220, &P21795_300};	/*Property = DataUrl*/
const Value V21795_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_302 = {S23, 0, &V21795_302_57223, &P21795_301};	/*Property = DeviceOrientation*/
const Value V21795_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_304 = {S24, 0, &V21795_304_57229, &P21795_302};	/*Property = FileReader*/
const Value V21795_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_305 = {S25, 0, &V21795_305_57232, &P21795_304};	/*Property = FileSaver*/
const Value V21795_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_306 = {S26, 0, &V21795_306_57238, &P21795_305};	/*Property = FileWriter*/
const Value V21795_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_307 = {S27, 0, &V21795_307_57241, &P21795_306};	/*Property = FormData*/
const Value V21795_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_309 = {S28, 0, &V21795_309_57247, &P21795_307};	/*Property = Fullscreen*/
const Value V21795_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_310 = {S29, 0, &V21795_310_57250, &P21795_309};	/*Property = GeoLocation*/
const Value V21795_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_312 = {S30, 0, &V21795_312_57256, &P21795_310};	/*Property = History*/
const Value V21795_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_314 = {S31, 0, &V21795_314_57262, &P21795_312};	/*Property = Html5*/
const Value V21795_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_313 = {S32, 0, &V21795_313_57259, &P21795_314};	/*Property = Html-Media-Capture*/
const Value V21795_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_315 = {S33, 0, &V21795_315_57265, &P21795_313};	/*Property = Iframe*/
const Value V21795_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_316 = {S34, 0, &V21795_316_57268, &P21795_315};	/*Property = IndexedDB*/
const Value V21795_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_317 = {S35, 0, &V21795_317_57271, &P21795_316};	/*Property = Json*/
const Value V21795_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_15 = {S37, 0, &V21795_15_72, &P21795_317};	/*Property = LayoutEngine*/
const Value V21795_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_344 = {S38, 0, &V21795_344_57274, &P21795_15};	/*Property = Masking*/
const Value V21795_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_320 = {S39, 0, &V21795_320_57292, &P21795_344};	/*Property = PostMessage*/
const Value V21795_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_321 = {S40, 0, &V21795_321_57295, &P21795_320};	/*Property = Progress*/
const Value V21795_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_322 = {S41, 0, &V21795_322_57298, &P21795_321};	/*Property = Prompts*/
const Value V21795_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_323 = {S42, 0, &V21795_323_57301, &P21795_322};	/*Property = Selector*/
const Value V21795_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_324 = {S43, 0, &V21795_324_57307, &P21795_323};	/*Property = Svg*/
const Value V21795_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_327 = {S44, 0, &V21795_327_57316, &P21795_324};	/*Property = TouchEvents*/
const Value V21795_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_328 = {S45, 0, &V21795_328_57319, &P21795_327};	/*Property = Track*/
const Value V21795_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_329 = {S46, 0, &V21795_329_57325, &P21795_328};	/*Property = Video*/
const Value V21795_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_330 = {S47, 0, &V21795_330_57328, &P21795_329};	/*Property = Viewport*/
const Value V21795_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_332 = {S48, 0, &V21795_332_57340, &P21795_330};	/*Property = WebWorkers*/
const Value V21795_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21795_333 = {S49, 0, &V21795_333_57343, &P21795_332};	/*Property = Xhr2*/
const Profile P21795 = {"21795", &P21795_333};	/*Profile ID - 21795*/
const Value V21021_1_1 = {S2, NULL};	/*Value = True*/
const Property P21021_1 = {S50, 0, &V21021_1_1, NULL};	/*Property = IsMobile*/
const Value V21021_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21021_5 = {S52, 0, &V21021_5_1227, &P21021_1};	/*Property = ScreenPixelsHeight*/
const Value V21021_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21021_4 = {S54, 0, &V21021_4_1229, &P21021_5};	/*Property = ScreenPixelsWidth*/
const Profile P21021 = {"21021", &P21021_4};	/*Profile ID - 21021*/
const Value V18740_1_1 = {S2, NULL};	/*Value = True*/
const Property P18740_1 = {S50, 0, &V18740_1_1, NULL};	/*Property = IsMobile*/
const Value V18740_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18740_5 = {S52, 0, &V18740_5_1227, &P18740_1};	/*Property = ScreenPixelsHeight*/
const Value V18740_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18740_4 = {S54, 0, &V18740_4_1229, &P18740_5};	/*Property = ScreenPixelsWidth*/
const Profile P18740 = {"18740", &P18740_4};	/*Profile ID - 18740*/
const Value V21335_1_1 = {S2, NULL};	/*Value = True*/
const Property P21335_1 = {S50, 0, &V21335_1_1, NULL};	/*Property = IsMobile*/
const Value V21335_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21335_5 = {S52, 0, &V21335_5_1277, &P21335_1};	/*Property = ScreenPixelsHeight*/
const Value V21335_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21335_4 = {S54, 0, &V21335_4_1279, &P21335_5};	/*Property = ScreenPixelsWidth*/
const Profile P21335 = {"21335", &P21335_4};	/*Profile ID - 21335*/
const Value V18242_1_1 = {S2, NULL};	/*Value = True*/
const Property P18242_1 = {S50, 0, &V18242_1_1, NULL};	/*Property = IsMobile*/
const Value V18242_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18242_5 = {S52, 0, &V18242_5_1227, &P18242_1};	/*Property = ScreenPixelsHeight*/
const Value V18242_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18242_4 = {S54, 0, &V18242_4_1229, &P18242_5};	/*Property = ScreenPixelsWidth*/
const Profile P18242 = {"18242", &P18242_4};	/*Profile ID - 18242*/
const Value V19855_1_1 = {S2, NULL};	/*Value = True*/
const Property P19855_1 = {S50, 0, &V19855_1_1, NULL};	/*Property = IsMobile*/
const Value V19855_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P19855_5 = {S52, 0, &V19855_5_1403, &P19855_1};	/*Property = ScreenPixelsHeight*/
const Value V19855_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P19855_4 = {S54, 0, &V19855_4_1404, &P19855_5};	/*Property = ScreenPixelsWidth*/
const Profile P19855 = {"19855", &P19855_4};	/*Profile ID - 19855*/
const Value V21719_1_1 = {S2, NULL};	/*Value = True*/
const Property P21719_1 = {S50, 0, &V21719_1_1, NULL};	/*Property = IsMobile*/
const Value V21719_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21719_5 = {S52, 0, &V21719_5_9981, &P21719_1};	/*Property = ScreenPixelsHeight*/
const Value V21719_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21719_4 = {S54, 0, &V21719_4_39098, &P21719_5};	/*Property = ScreenPixelsWidth*/
const Profile P21719 = {"21719", &P21719_4};	/*Profile ID - 21719*/
const Value V21263_1_1 = {S2, NULL};	/*Value = True*/
const Property P21263_1 = {S50, 0, &V21263_1_1, NULL};	/*Property = IsMobile*/
const Value V21263_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21263_5 = {S52, 0, &V21263_5_1277, &P21263_1};	/*Property = ScreenPixelsHeight*/
const Value V21263_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21263_4 = {S54, 0, &V21263_4_2336, &P21263_5};	/*Property = ScreenPixelsWidth*/
const Profile P21263 = {"21263", &P21263_4};	/*Profile ID - 21263*/
const Value V20864_1_1 = {S2, NULL};	/*Value = True*/
const Property P20864_1 = {S50, 0, &V20864_1_1, NULL};	/*Property = IsMobile*/
const Value V20864_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20864_5 = {S52, 0, &V20864_5_1227, &P20864_1};	/*Property = ScreenPixelsHeight*/
const Value V20864_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20864_4 = {S54, 0, &V20864_4_1229, &P20864_5};	/*Property = ScreenPixelsWidth*/
const Profile P20864 = {"20864", &P20864_4};	/*Profile ID - 20864*/
const Value V22078_1_1 = {S2, NULL};	/*Value = True*/
const Property P22078_1 = {S50, 0, &V22078_1_1, NULL};	/*Property = IsMobile*/
const Value V22078_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22078_5 = {S52, 0, &V22078_5_1227, &P22078_1};	/*Property = ScreenPixelsHeight*/
const Value V22078_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22078_4 = {S54, 0, &V22078_4_1229, &P22078_5};	/*Property = ScreenPixelsWidth*/
const Profile P22078 = {"22078", &P22078_4};	/*Profile ID - 22078*/
const Value V18458_1_1 = {S2, NULL};	/*Value = True*/
const Property P18458_1 = {S50, 0, &V18458_1_1, NULL};	/*Property = IsMobile*/
const Value V18458_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18458_5 = {S52, 0, &V18458_5_1227, &P18458_1};	/*Property = ScreenPixelsHeight*/
const Value V18458_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18458_4 = {S54, 0, &V18458_4_1229, &P18458_5};	/*Property = ScreenPixelsWidth*/
const Profile P18458 = {"18458", &P18458_4};	/*Profile ID - 18458*/
const Value V21338_1_1 = {S2, NULL};	/*Value = True*/
const Property P21338_1 = {S50, 0, &V21338_1_1, NULL};	/*Property = IsMobile*/
const Value V21338_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P21338_5 = {S52, 0, &V21338_5_1304, &P21338_1};	/*Property = ScreenPixelsHeight*/
const Value V21338_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21338_4 = {S54, 0, &V21338_4_1279, &P21338_5};	/*Property = ScreenPixelsWidth*/
const Profile P21338 = {"21338", &P21338_4};	/*Profile ID - 21338*/
const Value V22128_1_1 = {S2, NULL};	/*Value = True*/
const Property P22128_1 = {S50, 0, &V22128_1_1, NULL};	/*Property = IsMobile*/
const Value V22128_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22128_5 = {S52, 0, &V22128_5_1227, &P22128_1};	/*Property = ScreenPixelsHeight*/
const Value V22128_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22128_4 = {S54, 0, &V22128_4_1229, &P22128_5};	/*Property = ScreenPixelsWidth*/
const Profile P22128 = {"22128", &P22128_4};	/*Profile ID - 22128*/
const Value V19868_1_1 = {S2, NULL};	/*Value = True*/
const Property P19868_1 = {S50, 0, &V19868_1_1, NULL};	/*Property = IsMobile*/
const Value V19868_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P19868_5 = {S52, 0, &V19868_5_1277, &P19868_1};	/*Property = ScreenPixelsHeight*/
const Value V19868_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P19868_4 = {S54, 0, &V19868_4_1279, &P19868_5};	/*Property = ScreenPixelsWidth*/
const Profile P19868 = {"19868", &P19868_4};	/*Profile ID - 19868*/
const Value V21943_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_279 = {S1, 0, &V21943_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21943_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_282 = {S3, 0, &V21943_282_57155, &P21943_279};	/*Property = BlobBuilder*/
const Value V21943_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_350 = {S4, 0, &V21943_350_57446, &P21943_282};	/*Property = Canvas*/
const Value V21943_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_283 = {S5, 0, &V21943_283_57161, &P21943_350};	/*Property = CssBackground*/
const Value V21943_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_284 = {S6, 0, &V21943_284_57164, &P21943_283};	/*Property = CssBorderImage*/
const Value V21943_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_285 = {S7, 0, &V21943_285_57167, &P21943_284};	/*Property = CssCanvas*/
const Value V21943_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_286 = {S8, 0, &V21943_286_57170, &P21943_285};	/*Property = CssColor*/
const Value V21943_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_237 = {S9, 0, &V21943_237_55786, &P21943_286};	/*Property = CssColumn*/
const Value V21943_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_287 = {S10, 0, &V21943_287_57173, &P21943_237};	/*Property = CssFlexbox*/
const Value V21943_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_288 = {S11, 0, &V21943_288_57176, &P21943_287};	/*Property = CssFont*/
const Value V21943_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_289 = {S12, 0, &V21943_289_86899, &P21943_288};	/*Property = CssImages*/
const Value V21943_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_291 = {S13, 0, &V21943_291_57182, &P21943_289};	/*Property = CssMediaQueries*/
const Value V21943_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_292 = {S14, 0, &V21943_292_57185, &P21943_291};	/*Property = CssMinMax*/
const Value V21943_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_293 = {S15, 0, &V21943_293_57188, &P21943_292};	/*Property = CssOverflow*/
const Value V21943_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_294 = {S16, 0, &V21943_294_57191, &P21943_293};	/*Property = CssPosition*/
const Value V21943_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_295 = {S17, 0, &V21943_295_57194, &P21943_294};	/*Property = CssText*/
const Value V21943_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_297 = {S18, 0, &V21943_297_57197, &P21943_295};	/*Property = CssTransforms*/
const Value V21943_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_241 = {S19, 0, &V21943_241_55801, &P21943_297};	/*Property = CssTransitions*/
const Value V21943_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_298 = {S20, 0, &V21943_298_57208, &P21943_241};	/*Property = CssUI*/
const Value V21943_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_300 = {S21, 0, &V21943_300_57217, &P21943_298};	/*Property = DataSet*/
const Value V21943_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_301 = {S22, 0, &V21943_301_57220, &P21943_300};	/*Property = DataUrl*/
const Value V21943_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_302 = {S23, 0, &V21943_302_57223, &P21943_301};	/*Property = DeviceOrientation*/
const Value V21943_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_304 = {S24, 0, &V21943_304_57229, &P21943_302};	/*Property = FileReader*/
const Value V21943_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_305 = {S25, 0, &V21943_305_57232, &P21943_304};	/*Property = FileSaver*/
const Value V21943_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_306 = {S26, 0, &V21943_306_57238, &P21943_305};	/*Property = FileWriter*/
const Value V21943_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_307 = {S27, 0, &V21943_307_57241, &P21943_306};	/*Property = FormData*/
const Value V21943_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_309 = {S28, 0, &V21943_309_57247, &P21943_307};	/*Property = Fullscreen*/
const Value V21943_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_310 = {S29, 0, &V21943_310_57250, &P21943_309};	/*Property = GeoLocation*/
const Value V21943_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_312 = {S30, 0, &V21943_312_57256, &P21943_310};	/*Property = History*/
const Value V21943_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_314 = {S31, 0, &V21943_314_57262, &P21943_312};	/*Property = Html5*/
const Value V21943_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_313 = {S32, 0, &V21943_313_57259, &P21943_314};	/*Property = Html-Media-Capture*/
const Value V21943_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_315 = {S33, 0, &V21943_315_57265, &P21943_313};	/*Property = Iframe*/
const Value V21943_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_316 = {S34, 0, &V21943_316_57268, &P21943_315};	/*Property = IndexedDB*/
const Value V21943_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_317 = {S35, 0, &V21943_317_57271, &P21943_316};	/*Property = Json*/
const Value V21943_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P21943_15 = {S37, 0, &V21943_15_29, &P21943_317};	/*Property = LayoutEngine*/
const Value V21943_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_344 = {S38, 0, &V21943_344_57274, &P21943_15};	/*Property = Masking*/
const Value V21943_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_320 = {S39, 0, &V21943_320_57292, &P21943_344};	/*Property = PostMessage*/
const Value V21943_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_321 = {S40, 0, &V21943_321_57295, &P21943_320};	/*Property = Progress*/
const Value V21943_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_322 = {S41, 0, &V21943_322_57298, &P21943_321};	/*Property = Prompts*/
const Value V21943_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_323 = {S42, 0, &V21943_323_57301, &P21943_322};	/*Property = Selector*/
const Value V21943_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_324 = {S43, 0, &V21943_324_57307, &P21943_323};	/*Property = Svg*/
const Value V21943_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_327 = {S44, 0, &V21943_327_57316, &P21943_324};	/*Property = TouchEvents*/
const Value V21943_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_328 = {S45, 0, &V21943_328_57319, &P21943_327};	/*Property = Track*/
const Value V21943_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_329 = {S46, 0, &V21943_329_57325, &P21943_328};	/*Property = Video*/
const Value V21943_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_330 = {S47, 0, &V21943_330_57328, &P21943_329};	/*Property = Viewport*/
const Value V21943_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_332 = {S48, 0, &V21943_332_57340, &P21943_330};	/*Property = WebWorkers*/
const Value V21943_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21943_333 = {S49, 0, &V21943_333_57343, &P21943_332};	/*Property = Xhr2*/
const Profile P21943 = {"21943", &P21943_333};	/*Profile ID - 21943*/
const Value V20433_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_279 = {S1, 0, &V20433_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20433_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_282 = {S3, 0, &V20433_282_57155, &P20433_279};	/*Property = BlobBuilder*/
const Value V20433_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_350 = {S4, 0, &V20433_350_57446, &P20433_282};	/*Property = Canvas*/
const Value V20433_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_283 = {S5, 0, &V20433_283_57161, &P20433_350};	/*Property = CssBackground*/
const Value V20433_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_284 = {S6, 0, &V20433_284_57164, &P20433_283};	/*Property = CssBorderImage*/
const Value V20433_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_285 = {S7, 0, &V20433_285_57167, &P20433_284};	/*Property = CssCanvas*/
const Value V20433_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_286 = {S8, 0, &V20433_286_57170, &P20433_285};	/*Property = CssColor*/
const Value V20433_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_237 = {S9, 0, &V20433_237_55786, &P20433_286};	/*Property = CssColumn*/
const Value V20433_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_287 = {S10, 0, &V20433_287_57173, &P20433_237};	/*Property = CssFlexbox*/
const Value V20433_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_288 = {S11, 0, &V20433_288_57176, &P20433_287};	/*Property = CssFont*/
const Value V20433_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_289 = {S12, 0, &V20433_289_86899, &P20433_288};	/*Property = CssImages*/
const Value V20433_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_291 = {S13, 0, &V20433_291_57182, &P20433_289};	/*Property = CssMediaQueries*/
const Value V20433_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_292 = {S14, 0, &V20433_292_57185, &P20433_291};	/*Property = CssMinMax*/
const Value V20433_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_293 = {S15, 0, &V20433_293_57188, &P20433_292};	/*Property = CssOverflow*/
const Value V20433_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_294 = {S16, 0, &V20433_294_57191, &P20433_293};	/*Property = CssPosition*/
const Value V20433_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_295 = {S17, 0, &V20433_295_57194, &P20433_294};	/*Property = CssText*/
const Value V20433_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_297 = {S18, 0, &V20433_297_57197, &P20433_295};	/*Property = CssTransforms*/
const Value V20433_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_241 = {S19, 0, &V20433_241_55801, &P20433_297};	/*Property = CssTransitions*/
const Value V20433_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_298 = {S20, 0, &V20433_298_57208, &P20433_241};	/*Property = CssUI*/
const Value V20433_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_300 = {S21, 0, &V20433_300_57217, &P20433_298};	/*Property = DataSet*/
const Value V20433_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_301 = {S22, 0, &V20433_301_57220, &P20433_300};	/*Property = DataUrl*/
const Value V20433_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_302 = {S23, 0, &V20433_302_57223, &P20433_301};	/*Property = DeviceOrientation*/
const Value V20433_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_304 = {S24, 0, &V20433_304_57229, &P20433_302};	/*Property = FileReader*/
const Value V20433_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_305 = {S25, 0, &V20433_305_57232, &P20433_304};	/*Property = FileSaver*/
const Value V20433_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_306 = {S26, 0, &V20433_306_57238, &P20433_305};	/*Property = FileWriter*/
const Value V20433_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_307 = {S27, 0, &V20433_307_57241, &P20433_306};	/*Property = FormData*/
const Value V20433_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_309 = {S28, 0, &V20433_309_57247, &P20433_307};	/*Property = Fullscreen*/
const Value V20433_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_310 = {S29, 0, &V20433_310_57250, &P20433_309};	/*Property = GeoLocation*/
const Value V20433_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_312 = {S30, 0, &V20433_312_57256, &P20433_310};	/*Property = History*/
const Value V20433_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_314 = {S31, 0, &V20433_314_57262, &P20433_312};	/*Property = Html5*/
const Value V20433_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_313 = {S32, 0, &V20433_313_57259, &P20433_314};	/*Property = Html-Media-Capture*/
const Value V20433_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_315 = {S33, 0, &V20433_315_57265, &P20433_313};	/*Property = Iframe*/
const Value V20433_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_316 = {S34, 0, &V20433_316_57268, &P20433_315};	/*Property = IndexedDB*/
const Value V20433_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_317 = {S35, 0, &V20433_317_57271, &P20433_316};	/*Property = Json*/
const Value V20433_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20433_15 = {S37, 0, &V20433_15_29, &P20433_317};	/*Property = LayoutEngine*/
const Value V20433_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_344 = {S38, 0, &V20433_344_57274, &P20433_15};	/*Property = Masking*/
const Value V20433_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_320 = {S39, 0, &V20433_320_57292, &P20433_344};	/*Property = PostMessage*/
const Value V20433_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_321 = {S40, 0, &V20433_321_57295, &P20433_320};	/*Property = Progress*/
const Value V20433_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_322 = {S41, 0, &V20433_322_57298, &P20433_321};	/*Property = Prompts*/
const Value V20433_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_323 = {S42, 0, &V20433_323_57301, &P20433_322};	/*Property = Selector*/
const Value V20433_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_324 = {S43, 0, &V20433_324_57307, &P20433_323};	/*Property = Svg*/
const Value V20433_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_327 = {S44, 0, &V20433_327_57316, &P20433_324};	/*Property = TouchEvents*/
const Value V20433_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_328 = {S45, 0, &V20433_328_57319, &P20433_327};	/*Property = Track*/
const Value V20433_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_329 = {S46, 0, &V20433_329_57325, &P20433_328};	/*Property = Video*/
const Value V20433_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_330 = {S47, 0, &V20433_330_57328, &P20433_329};	/*Property = Viewport*/
const Value V20433_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_332 = {S48, 0, &V20433_332_57340, &P20433_330};	/*Property = WebWorkers*/
const Value V20433_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20433_333 = {S49, 0, &V20433_333_57343, &P20433_332};	/*Property = Xhr2*/
const Profile P20433 = {"20433", &P20433_333};	/*Profile ID - 20433*/
const Value V24518_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_279 = {S1, 0, &V24518_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24518_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_282 = {S3, 0, &V24518_282_57155, &P24518_279};	/*Property = BlobBuilder*/
const Value V24518_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_350 = {S4, 0, &V24518_350_57446, &P24518_282};	/*Property = Canvas*/
const Value V24518_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_283 = {S5, 0, &V24518_283_57161, &P24518_350};	/*Property = CssBackground*/
const Value V24518_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_284 = {S6, 0, &V24518_284_57164, &P24518_283};	/*Property = CssBorderImage*/
const Value V24518_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_285 = {S7, 0, &V24518_285_57167, &P24518_284};	/*Property = CssCanvas*/
const Value V24518_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_286 = {S8, 0, &V24518_286_57170, &P24518_285};	/*Property = CssColor*/
const Value V24518_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_237 = {S9, 0, &V24518_237_55786, &P24518_286};	/*Property = CssColumn*/
const Value V24518_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_287 = {S10, 0, &V24518_287_57173, &P24518_237};	/*Property = CssFlexbox*/
const Value V24518_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_288 = {S11, 0, &V24518_288_57176, &P24518_287};	/*Property = CssFont*/
const Value V24518_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_289 = {S12, 0, &V24518_289_86899, &P24518_288};	/*Property = CssImages*/
const Value V24518_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_291 = {S13, 0, &V24518_291_57182, &P24518_289};	/*Property = CssMediaQueries*/
const Value V24518_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_292 = {S14, 0, &V24518_292_57185, &P24518_291};	/*Property = CssMinMax*/
const Value V24518_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_293 = {S15, 0, &V24518_293_57188, &P24518_292};	/*Property = CssOverflow*/
const Value V24518_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_294 = {S16, 0, &V24518_294_57191, &P24518_293};	/*Property = CssPosition*/
const Value V24518_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_295 = {S17, 0, &V24518_295_57194, &P24518_294};	/*Property = CssText*/
const Value V24518_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_297 = {S18, 0, &V24518_297_57197, &P24518_295};	/*Property = CssTransforms*/
const Value V24518_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_241 = {S19, 0, &V24518_241_55801, &P24518_297};	/*Property = CssTransitions*/
const Value V24518_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_298 = {S20, 0, &V24518_298_57208, &P24518_241};	/*Property = CssUI*/
const Value V24518_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_300 = {S21, 0, &V24518_300_57217, &P24518_298};	/*Property = DataSet*/
const Value V24518_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_301 = {S22, 0, &V24518_301_57220, &P24518_300};	/*Property = DataUrl*/
const Value V24518_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_302 = {S23, 0, &V24518_302_57223, &P24518_301};	/*Property = DeviceOrientation*/
const Value V24518_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_304 = {S24, 0, &V24518_304_57229, &P24518_302};	/*Property = FileReader*/
const Value V24518_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_305 = {S25, 0, &V24518_305_57232, &P24518_304};	/*Property = FileSaver*/
const Value V24518_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_306 = {S26, 0, &V24518_306_57238, &P24518_305};	/*Property = FileWriter*/
const Value V24518_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_307 = {S27, 0, &V24518_307_57241, &P24518_306};	/*Property = FormData*/
const Value V24518_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_309 = {S28, 0, &V24518_309_57247, &P24518_307};	/*Property = Fullscreen*/
const Value V24518_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_310 = {S29, 0, &V24518_310_57250, &P24518_309};	/*Property = GeoLocation*/
const Value V24518_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_312 = {S30, 0, &V24518_312_57256, &P24518_310};	/*Property = History*/
const Value V24518_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_314 = {S31, 0, &V24518_314_57262, &P24518_312};	/*Property = Html5*/
const Value V24518_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_313 = {S32, 0, &V24518_313_57259, &P24518_314};	/*Property = Html-Media-Capture*/
const Value V24518_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_315 = {S33, 0, &V24518_315_57265, &P24518_313};	/*Property = Iframe*/
const Value V24518_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_316 = {S34, 0, &V24518_316_57268, &P24518_315};	/*Property = IndexedDB*/
const Value V24518_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_317 = {S35, 0, &V24518_317_57271, &P24518_316};	/*Property = Json*/
const Value V24518_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24518_15 = {S37, 0, &V24518_15_54167, &P24518_317};	/*Property = LayoutEngine*/
const Value V24518_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_344 = {S38, 0, &V24518_344_57274, &P24518_15};	/*Property = Masking*/
const Value V24518_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_320 = {S39, 0, &V24518_320_57292, &P24518_344};	/*Property = PostMessage*/
const Value V24518_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_321 = {S40, 0, &V24518_321_57295, &P24518_320};	/*Property = Progress*/
const Value V24518_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_322 = {S41, 0, &V24518_322_57298, &P24518_321};	/*Property = Prompts*/
const Value V24518_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_323 = {S42, 0, &V24518_323_57301, &P24518_322};	/*Property = Selector*/
const Value V24518_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_324 = {S43, 0, &V24518_324_57307, &P24518_323};	/*Property = Svg*/
const Value V24518_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_327 = {S44, 0, &V24518_327_57316, &P24518_324};	/*Property = TouchEvents*/
const Value V24518_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_328 = {S45, 0, &V24518_328_57319, &P24518_327};	/*Property = Track*/
const Value V24518_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_329 = {S46, 0, &V24518_329_57325, &P24518_328};	/*Property = Video*/
const Value V24518_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_330 = {S47, 0, &V24518_330_57328, &P24518_329};	/*Property = Viewport*/
const Value V24518_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_332 = {S48, 0, &V24518_332_57340, &P24518_330};	/*Property = WebWorkers*/
const Value V24518_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24518_333 = {S49, 0, &V24518_333_57343, &P24518_332};	/*Property = Xhr2*/
const Profile P24518 = {"24518", &P24518_333};	/*Profile ID - 24518*/
const Value V21827_1_1 = {S2, NULL};	/*Value = True*/
const Property P21827_1 = {S50, 0, &V21827_1_1, NULL};	/*Property = IsMobile*/
const Value V21827_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21827_5 = {S52, 0, &V21827_5_1277, &P21827_1};	/*Property = ScreenPixelsHeight*/
const Value V21827_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21827_4 = {S54, 0, &V21827_4_2336, &P21827_5};	/*Property = ScreenPixelsWidth*/
const Profile P21827 = {"21827", &P21827_4};	/*Profile ID - 21827*/
const Value V18697_1_1 = {S2, NULL};	/*Value = True*/
const Property P18697_1 = {S50, 0, &V18697_1_1, NULL};	/*Property = IsMobile*/
const Value V18697_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18697_5 = {S52, 0, &V18697_5_1227, &P18697_1};	/*Property = ScreenPixelsHeight*/
const Value V18697_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18697_4 = {S54, 0, &V18697_4_1229, &P18697_5};	/*Property = ScreenPixelsWidth*/
const Profile P18697 = {"18697", &P18697_4};	/*Profile ID - 18697*/
const Value V21437_1_1 = {S2, NULL};	/*Value = True*/
const Property P21437_1 = {S50, 0, &V21437_1_1, NULL};	/*Property = IsMobile*/
const Value V21437_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21437_5 = {S52, 0, &V21437_5_1227, &P21437_1};	/*Property = ScreenPixelsHeight*/
const Value V21437_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21437_4 = {S54, 0, &V21437_4_1229, &P21437_5};	/*Property = ScreenPixelsWidth*/
const Profile P21437 = {"21437", &P21437_4};	/*Profile ID - 21437*/
const Value V20029_1_1 = {S2, NULL};	/*Value = True*/
const Property P20029_1 = {S50, 0, &V20029_1_1, NULL};	/*Property = IsMobile*/
const Value V20029_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20029_5 = {S52, 0, &V20029_5_1222, &P20029_1};	/*Property = ScreenPixelsHeight*/
const Value V20029_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20029_4 = {S54, 0, &V20029_4_1224, &P20029_5};	/*Property = ScreenPixelsWidth*/
const Profile P20029 = {"20029", &P20029_4};	/*Profile ID - 20029*/
const Value V21773_1_1 = {S2, NULL};	/*Value = True*/
const Property P21773_1 = {S50, 0, &V21773_1_1, NULL};	/*Property = IsMobile*/
const Value V21773_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21773_5 = {S52, 0, &V21773_5_1227, &P21773_1};	/*Property = ScreenPixelsHeight*/
const Value V21773_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21773_4 = {S54, 0, &V21773_4_1229, &P21773_5};	/*Property = ScreenPixelsWidth*/
const Profile P21773 = {"21773", &P21773_4};	/*Profile ID - 21773*/
const Value V22802_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_279 = {S1, 0, &V22802_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22802_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_282 = {S3, 0, &V22802_282_57155, &P22802_279};	/*Property = BlobBuilder*/
const Value V22802_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_350 = {S4, 0, &V22802_350_57446, &P22802_282};	/*Property = Canvas*/
const Value V22802_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_283 = {S5, 0, &V22802_283_57161, &P22802_350};	/*Property = CssBackground*/
const Value V22802_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_284 = {S6, 0, &V22802_284_57164, &P22802_283};	/*Property = CssBorderImage*/
const Value V22802_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_285 = {S7, 0, &V22802_285_57167, &P22802_284};	/*Property = CssCanvas*/
const Value V22802_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_286 = {S8, 0, &V22802_286_57170, &P22802_285};	/*Property = CssColor*/
const Value V22802_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_237 = {S9, 0, &V22802_237_55786, &P22802_286};	/*Property = CssColumn*/
const Value V22802_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_287 = {S10, 0, &V22802_287_57173, &P22802_237};	/*Property = CssFlexbox*/
const Value V22802_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_288 = {S11, 0, &V22802_288_57176, &P22802_287};	/*Property = CssFont*/
const Value V22802_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_289 = {S12, 0, &V22802_289_86899, &P22802_288};	/*Property = CssImages*/
const Value V22802_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_291 = {S13, 0, &V22802_291_57182, &P22802_289};	/*Property = CssMediaQueries*/
const Value V22802_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_292 = {S14, 0, &V22802_292_57185, &P22802_291};	/*Property = CssMinMax*/
const Value V22802_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_293 = {S15, 0, &V22802_293_57188, &P22802_292};	/*Property = CssOverflow*/
const Value V22802_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_294 = {S16, 0, &V22802_294_57191, &P22802_293};	/*Property = CssPosition*/
const Value V22802_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_295 = {S17, 0, &V22802_295_57194, &P22802_294};	/*Property = CssText*/
const Value V22802_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_297 = {S18, 0, &V22802_297_57197, &P22802_295};	/*Property = CssTransforms*/
const Value V22802_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_241 = {S19, 0, &V22802_241_55801, &P22802_297};	/*Property = CssTransitions*/
const Value V22802_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_298 = {S20, 0, &V22802_298_57208, &P22802_241};	/*Property = CssUI*/
const Value V22802_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_300 = {S21, 0, &V22802_300_57217, &P22802_298};	/*Property = DataSet*/
const Value V22802_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_301 = {S22, 0, &V22802_301_57220, &P22802_300};	/*Property = DataUrl*/
const Value V22802_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_302 = {S23, 0, &V22802_302_57223, &P22802_301};	/*Property = DeviceOrientation*/
const Value V22802_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_304 = {S24, 0, &V22802_304_57229, &P22802_302};	/*Property = FileReader*/
const Value V22802_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_305 = {S25, 0, &V22802_305_57232, &P22802_304};	/*Property = FileSaver*/
const Value V22802_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_306 = {S26, 0, &V22802_306_57238, &P22802_305};	/*Property = FileWriter*/
const Value V22802_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_307 = {S27, 0, &V22802_307_57241, &P22802_306};	/*Property = FormData*/
const Value V22802_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_309 = {S28, 0, &V22802_309_57247, &P22802_307};	/*Property = Fullscreen*/
const Value V22802_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_310 = {S29, 0, &V22802_310_57250, &P22802_309};	/*Property = GeoLocation*/
const Value V22802_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_312 = {S30, 0, &V22802_312_57256, &P22802_310};	/*Property = History*/
const Value V22802_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_314 = {S31, 0, &V22802_314_57262, &P22802_312};	/*Property = Html5*/
const Value V22802_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_313 = {S32, 0, &V22802_313_57259, &P22802_314};	/*Property = Html-Media-Capture*/
const Value V22802_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_315 = {S33, 0, &V22802_315_57265, &P22802_313};	/*Property = Iframe*/
const Value V22802_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_316 = {S34, 0, &V22802_316_57268, &P22802_315};	/*Property = IndexedDB*/
const Value V22802_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_317 = {S35, 0, &V22802_317_57271, &P22802_316};	/*Property = Json*/
const Value V22802_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_15 = {S37, 0, &V22802_15_72, &P22802_317};	/*Property = LayoutEngine*/
const Value V22802_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_344 = {S38, 0, &V22802_344_57274, &P22802_15};	/*Property = Masking*/
const Value V22802_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_320 = {S39, 0, &V22802_320_57292, &P22802_344};	/*Property = PostMessage*/
const Value V22802_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_321 = {S40, 0, &V22802_321_57295, &P22802_320};	/*Property = Progress*/
const Value V22802_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_322 = {S41, 0, &V22802_322_57298, &P22802_321};	/*Property = Prompts*/
const Value V22802_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_323 = {S42, 0, &V22802_323_57301, &P22802_322};	/*Property = Selector*/
const Value V22802_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_324 = {S43, 0, &V22802_324_57307, &P22802_323};	/*Property = Svg*/
const Value V22802_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_327 = {S44, 0, &V22802_327_57316, &P22802_324};	/*Property = TouchEvents*/
const Value V22802_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_328 = {S45, 0, &V22802_328_57319, &P22802_327};	/*Property = Track*/
const Value V22802_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_329 = {S46, 0, &V22802_329_57325, &P22802_328};	/*Property = Video*/
const Value V22802_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_330 = {S47, 0, &V22802_330_57328, &P22802_329};	/*Property = Viewport*/
const Value V22802_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_332 = {S48, 0, &V22802_332_57340, &P22802_330};	/*Property = WebWorkers*/
const Value V22802_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22802_333 = {S49, 0, &V22802_333_57343, &P22802_332};	/*Property = Xhr2*/
const Profile P22802 = {"22802", &P22802_333};	/*Profile ID - 22802*/
const Value V21096_1_1 = {S2, NULL};	/*Value = True*/
const Property P21096_1 = {S50, 0, &V21096_1_1, NULL};	/*Property = IsMobile*/
const Value V21096_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21096_5 = {S52, 0, &V21096_5_1227, &P21096_1};	/*Property = ScreenPixelsHeight*/
const Value V21096_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21096_4 = {S54, 0, &V21096_4_1229, &P21096_5};	/*Property = ScreenPixelsWidth*/
const Profile P21096 = {"21096", &P21096_4};	/*Profile ID - 21096*/
const Value V18702_1_1 = {S2, NULL};	/*Value = True*/
const Property P18702_1 = {S50, 0, &V18702_1_1, NULL};	/*Property = IsMobile*/
const Value V18702_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18702_5 = {S52, 0, &V18702_5_9981, &P18702_1};	/*Property = ScreenPixelsHeight*/
const Value V18702_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18702_4 = {S54, 0, &V18702_4_2336, &P18702_5};	/*Property = ScreenPixelsWidth*/
const Profile P18702 = {"18702", &P18702_4};	/*Profile ID - 18702*/
const Value V23873_1_1 = {S2, NULL};	/*Value = True*/
const Property P23873_1 = {S50, 0, &V23873_1_1, NULL};	/*Property = IsMobile*/
const Value V23873_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23873_5 = {S52, 0, &V23873_5_1458, &P23873_1};	/*Property = ScreenPixelsHeight*/
const Value V23873_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23873_4 = {S54, 0, &V23873_4_1460, &P23873_5};	/*Property = ScreenPixelsWidth*/
const Profile P23873 = {"23873", &P23873_4};	/*Profile ID - 23873*/
const Value V20594_1_1 = {S2, NULL};	/*Value = True*/
const Property P20594_1 = {S50, 0, &V20594_1_1, NULL};	/*Property = IsMobile*/
const Value V20594_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20594_5 = {S52, 0, &V20594_5_1458, &P20594_1};	/*Property = ScreenPixelsHeight*/
const Value V20594_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20594_4 = {S54, 0, &V20594_4_37617, &P20594_5};	/*Property = ScreenPixelsWidth*/
const Profile P20594 = {"20594", &P20594_4};	/*Profile ID - 20594*/
const Value V20977_1_1 = {S2, NULL};	/*Value = True*/
const Property P20977_1 = {S50, 0, &V20977_1_1, NULL};	/*Property = IsMobile*/
const Value V20977_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20977_5 = {S52, 0, &V20977_5_1277, &P20977_1};	/*Property = ScreenPixelsHeight*/
const Value V20977_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20977_4 = {S54, 0, &V20977_4_1279, &P20977_5};	/*Property = ScreenPixelsWidth*/
const Profile P20977 = {"20977", &P20977_4};	/*Profile ID - 20977*/
const Value V18743_1_1 = {S2, NULL};	/*Value = True*/
const Property P18743_1 = {S50, 0, &V18743_1_1, NULL};	/*Property = IsMobile*/
const Value V18743_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18743_5 = {S52, 0, &V18743_5_1277, &P18743_1};	/*Property = ScreenPixelsHeight*/
const Value V18743_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18743_4 = {S54, 0, &V18743_4_1279, &P18743_5};	/*Property = ScreenPixelsWidth*/
const Profile P18743 = {"18743", &P18743_4};	/*Profile ID - 18743*/
const Value V18618_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_279 = {S1, 0, &V18618_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18618_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_282 = {S3, 0, &V18618_282_57155, &P18618_279};	/*Property = BlobBuilder*/
const Value V18618_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_350 = {S4, 0, &V18618_350_57446, &P18618_282};	/*Property = Canvas*/
const Value V18618_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_283 = {S5, 0, &V18618_283_57161, &P18618_350};	/*Property = CssBackground*/
const Value V18618_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_284 = {S6, 0, &V18618_284_57164, &P18618_283};	/*Property = CssBorderImage*/
const Value V18618_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_285 = {S7, 0, &V18618_285_57167, &P18618_284};	/*Property = CssCanvas*/
const Value V18618_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_286 = {S8, 0, &V18618_286_57170, &P18618_285};	/*Property = CssColor*/
const Value V18618_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_237 = {S9, 0, &V18618_237_55786, &P18618_286};	/*Property = CssColumn*/
const Value V18618_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_287 = {S10, 0, &V18618_287_57173, &P18618_237};	/*Property = CssFlexbox*/
const Value V18618_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_288 = {S11, 0, &V18618_288_57176, &P18618_287};	/*Property = CssFont*/
const Value V18618_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_289 = {S12, 0, &V18618_289_86899, &P18618_288};	/*Property = CssImages*/
const Value V18618_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_291 = {S13, 0, &V18618_291_57182, &P18618_289};	/*Property = CssMediaQueries*/
const Value V18618_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_292 = {S14, 0, &V18618_292_57185, &P18618_291};	/*Property = CssMinMax*/
const Value V18618_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_293 = {S15, 0, &V18618_293_57188, &P18618_292};	/*Property = CssOverflow*/
const Value V18618_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_294 = {S16, 0, &V18618_294_57191, &P18618_293};	/*Property = CssPosition*/
const Value V18618_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_295 = {S17, 0, &V18618_295_57194, &P18618_294};	/*Property = CssText*/
const Value V18618_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_297 = {S18, 0, &V18618_297_57197, &P18618_295};	/*Property = CssTransforms*/
const Value V18618_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_241 = {S19, 0, &V18618_241_55801, &P18618_297};	/*Property = CssTransitions*/
const Value V18618_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_298 = {S20, 0, &V18618_298_57208, &P18618_241};	/*Property = CssUI*/
const Value V18618_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_300 = {S21, 0, &V18618_300_57217, &P18618_298};	/*Property = DataSet*/
const Value V18618_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_301 = {S22, 0, &V18618_301_57220, &P18618_300};	/*Property = DataUrl*/
const Value V18618_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_302 = {S23, 0, &V18618_302_57223, &P18618_301};	/*Property = DeviceOrientation*/
const Value V18618_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_304 = {S24, 0, &V18618_304_57229, &P18618_302};	/*Property = FileReader*/
const Value V18618_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_305 = {S25, 0, &V18618_305_57232, &P18618_304};	/*Property = FileSaver*/
const Value V18618_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_306 = {S26, 0, &V18618_306_57238, &P18618_305};	/*Property = FileWriter*/
const Value V18618_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_307 = {S27, 0, &V18618_307_57241, &P18618_306};	/*Property = FormData*/
const Value V18618_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_309 = {S28, 0, &V18618_309_57247, &P18618_307};	/*Property = Fullscreen*/
const Value V18618_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_310 = {S29, 0, &V18618_310_57250, &P18618_309};	/*Property = GeoLocation*/
const Value V18618_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_312 = {S30, 0, &V18618_312_57256, &P18618_310};	/*Property = History*/
const Value V18618_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_314 = {S31, 0, &V18618_314_57262, &P18618_312};	/*Property = Html5*/
const Value V18618_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_313 = {S32, 0, &V18618_313_57259, &P18618_314};	/*Property = Html-Media-Capture*/
const Value V18618_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_315 = {S33, 0, &V18618_315_57265, &P18618_313};	/*Property = Iframe*/
const Value V18618_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_316 = {S34, 0, &V18618_316_57268, &P18618_315};	/*Property = IndexedDB*/
const Value V18618_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_317 = {S35, 0, &V18618_317_57271, &P18618_316};	/*Property = Json*/
const Value V18618_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_15 = {S37, 0, &V18618_15_72, &P18618_317};	/*Property = LayoutEngine*/
const Value V18618_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_344 = {S38, 0, &V18618_344_57274, &P18618_15};	/*Property = Masking*/
const Value V18618_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_320 = {S39, 0, &V18618_320_57292, &P18618_344};	/*Property = PostMessage*/
const Value V18618_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_321 = {S40, 0, &V18618_321_57295, &P18618_320};	/*Property = Progress*/
const Value V18618_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_322 = {S41, 0, &V18618_322_57298, &P18618_321};	/*Property = Prompts*/
const Value V18618_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_323 = {S42, 0, &V18618_323_57301, &P18618_322};	/*Property = Selector*/
const Value V18618_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_324 = {S43, 0, &V18618_324_57307, &P18618_323};	/*Property = Svg*/
const Value V18618_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_327 = {S44, 0, &V18618_327_57316, &P18618_324};	/*Property = TouchEvents*/
const Value V18618_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_328 = {S45, 0, &V18618_328_57319, &P18618_327};	/*Property = Track*/
const Value V18618_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_329 = {S46, 0, &V18618_329_57325, &P18618_328};	/*Property = Video*/
const Value V18618_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_330 = {S47, 0, &V18618_330_57328, &P18618_329};	/*Property = Viewport*/
const Value V18618_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_332 = {S48, 0, &V18618_332_57340, &P18618_330};	/*Property = WebWorkers*/
const Value V18618_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18618_333 = {S49, 0, &V18618_333_57343, &P18618_332};	/*Property = Xhr2*/
const Profile P18618 = {"18618", &P18618_333};	/*Profile ID - 18618*/
const Value V21471_1_1 = {S2, NULL};	/*Value = True*/
const Property P21471_1 = {S50, 0, &V21471_1_1, NULL};	/*Property = IsMobile*/
const Value V21471_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21471_5 = {S52, 0, &V21471_5_1227, &P21471_1};	/*Property = ScreenPixelsHeight*/
const Value V21471_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21471_4 = {S54, 0, &V21471_4_1229, &P21471_5};	/*Property = ScreenPixelsWidth*/
const Profile P21471 = {"21471", &P21471_4};	/*Profile ID - 21471*/
const Value V20108_1_1 = {S2, NULL};	/*Value = True*/
const Property P20108_1 = {S50, 0, &V20108_1_1, NULL};	/*Property = IsMobile*/
const Value V20108_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20108_5 = {S52, 0, &V20108_5_1227, &P20108_1};	/*Property = ScreenPixelsHeight*/
const Value V20108_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20108_4 = {S54, 0, &V20108_4_1229, &P20108_5};	/*Property = ScreenPixelsWidth*/
const Profile P20108 = {"20108", &P20108_4};	/*Profile ID - 20108*/
const Value V21235_1_1 = {S2, NULL};	/*Value = True*/
const Property P21235_1 = {S50, 0, &V21235_1_1, NULL};	/*Property = IsMobile*/
const Value V21235_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21235_5 = {S52, 0, &V21235_5_1227, &P21235_1};	/*Property = ScreenPixelsHeight*/
const Value V21235_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21235_4 = {S54, 0, &V21235_4_1229, &P21235_5};	/*Property = ScreenPixelsWidth*/
const Profile P21235 = {"21235", &P21235_4};	/*Profile ID - 21235*/
const Value V21276_1_1 = {S2, NULL};	/*Value = True*/
const Property P21276_1 = {S50, 0, &V21276_1_1, NULL};	/*Property = IsMobile*/
const Value V21276_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21276_5 = {S52, 0, &V21276_5_37, &P21276_1};	/*Property = ScreenPixelsHeight*/
const Value V21276_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21276_4 = {S54, 0, &V21276_4_36, &P21276_5};	/*Property = ScreenPixelsWidth*/
const Profile P21276 = {"21276", &P21276_4};	/*Profile ID - 21276*/
const Value V20941_1_1 = {S2, NULL};	/*Value = True*/
const Property P20941_1 = {S50, 0, &V20941_1_1, NULL};	/*Property = IsMobile*/
const Value V20941_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20941_5 = {S52, 0, &V20941_5_1277, &P20941_1};	/*Property = ScreenPixelsHeight*/
const Value V20941_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20941_4 = {S54, 0, &V20941_4_1279, &P20941_5};	/*Property = ScreenPixelsWidth*/
const Profile P20941 = {"20941", &P20941_4};	/*Profile ID - 20941*/
const Value V20466_1_1 = {S2, NULL};	/*Value = True*/
const Property P20466_1 = {S50, 0, &V20466_1_1, NULL};	/*Property = IsMobile*/
const Value V20466_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20466_5 = {S52, 0, &V20466_5_1403, &P20466_1};	/*Property = ScreenPixelsHeight*/
const Value V20466_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20466_4 = {S54, 0, &V20466_4_1404, &P20466_5};	/*Property = ScreenPixelsWidth*/
const Profile P20466 = {"20466", &P20466_4};	/*Profile ID - 20466*/
const Value V20513_1_1 = {S2, NULL};	/*Value = True*/
const Property P20513_1 = {S50, 0, &V20513_1_1, NULL};	/*Property = IsMobile*/
const Value V20513_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20513_5 = {S52, 0, &V20513_5_1277, &P20513_1};	/*Property = ScreenPixelsHeight*/
const Value V20513_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20513_4 = {S54, 0, &V20513_4_1279, &P20513_5};	/*Property = ScreenPixelsWidth*/
const Profile P20513 = {"20513", &P20513_4};	/*Profile ID - 20513*/
const Value V21252_1_1 = {S2, NULL};	/*Value = True*/
const Property P21252_1 = {S50, 0, &V21252_1_1, NULL};	/*Property = IsMobile*/
const Value V21252_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21252_5 = {S52, 0, &V21252_5_1222, &P21252_1};	/*Property = ScreenPixelsHeight*/
const Value V21252_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21252_4 = {S54, 0, &V21252_4_1224, &P21252_5};	/*Property = ScreenPixelsWidth*/
const Profile P21252 = {"21252", &P21252_4};	/*Profile ID - 21252*/
const Value V22000_1_1 = {S2, NULL};	/*Value = True*/
const Property P22000_1 = {S50, 0, &V22000_1_1, NULL};	/*Property = IsMobile*/
const Value V22000_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22000_5 = {S52, 0, &V22000_5_37569, &P22000_1};	/*Property = ScreenPixelsHeight*/
const Value V22000_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22000_4 = {S54, 0, &V22000_4_5645, &P22000_5};	/*Property = ScreenPixelsWidth*/
const Profile P22000 = {"22000", &P22000_4};	/*Profile ID - 22000*/
const Value V20516_1_1 = {S2, NULL};	/*Value = True*/
const Property P20516_1 = {S50, 0, &V20516_1_1, NULL};	/*Property = IsMobile*/
const Value V20516_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20516_5 = {S52, 0, &V20516_5_1458, &P20516_1};	/*Property = ScreenPixelsHeight*/
const Value V20516_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20516_4 = {S54, 0, &V20516_4_37617, &P20516_5};	/*Property = ScreenPixelsWidth*/
const Profile P20516 = {"20516", &P20516_4};	/*Profile ID - 20516*/
const Value V17971_1_1 = {S2, NULL};	/*Value = True*/
const Property P17971_1 = {S50, 0, &V17971_1_1, NULL};	/*Property = IsMobile*/
const Value V17971_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P17971_5 = {S52, 0, &V17971_5_1227, &P17971_1};	/*Property = ScreenPixelsHeight*/
const Value V17971_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P17971_4 = {S54, 0, &V17971_4_1229, &P17971_5};	/*Property = ScreenPixelsWidth*/
const Profile P17971 = {"17971", &P17971_4};	/*Profile ID - 17971*/
const Value V18517_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_279 = {S1, 0, &V18517_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18517_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_282 = {S3, 0, &V18517_282_57155, &P18517_279};	/*Property = BlobBuilder*/
const Value V18517_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_350 = {S4, 0, &V18517_350_57446, &P18517_282};	/*Property = Canvas*/
const Value V18517_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_283 = {S5, 0, &V18517_283_57161, &P18517_350};	/*Property = CssBackground*/
const Value V18517_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_284 = {S6, 0, &V18517_284_57164, &P18517_283};	/*Property = CssBorderImage*/
const Value V18517_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_285 = {S7, 0, &V18517_285_57167, &P18517_284};	/*Property = CssCanvas*/
const Value V18517_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_286 = {S8, 0, &V18517_286_57170, &P18517_285};	/*Property = CssColor*/
const Value V18517_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_237 = {S9, 0, &V18517_237_55786, &P18517_286};	/*Property = CssColumn*/
const Value V18517_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_287 = {S10, 0, &V18517_287_57173, &P18517_237};	/*Property = CssFlexbox*/
const Value V18517_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_288 = {S11, 0, &V18517_288_57176, &P18517_287};	/*Property = CssFont*/
const Value V18517_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_289 = {S12, 0, &V18517_289_86899, &P18517_288};	/*Property = CssImages*/
const Value V18517_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_291 = {S13, 0, &V18517_291_57182, &P18517_289};	/*Property = CssMediaQueries*/
const Value V18517_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_292 = {S14, 0, &V18517_292_57185, &P18517_291};	/*Property = CssMinMax*/
const Value V18517_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_293 = {S15, 0, &V18517_293_57188, &P18517_292};	/*Property = CssOverflow*/
const Value V18517_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_294 = {S16, 0, &V18517_294_57191, &P18517_293};	/*Property = CssPosition*/
const Value V18517_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_295 = {S17, 0, &V18517_295_57194, &P18517_294};	/*Property = CssText*/
const Value V18517_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_297 = {S18, 0, &V18517_297_57197, &P18517_295};	/*Property = CssTransforms*/
const Value V18517_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_241 = {S19, 0, &V18517_241_55801, &P18517_297};	/*Property = CssTransitions*/
const Value V18517_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_298 = {S20, 0, &V18517_298_57208, &P18517_241};	/*Property = CssUI*/
const Value V18517_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_300 = {S21, 0, &V18517_300_57217, &P18517_298};	/*Property = DataSet*/
const Value V18517_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_301 = {S22, 0, &V18517_301_57220, &P18517_300};	/*Property = DataUrl*/
const Value V18517_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_302 = {S23, 0, &V18517_302_57223, &P18517_301};	/*Property = DeviceOrientation*/
const Value V18517_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_304 = {S24, 0, &V18517_304_57229, &P18517_302};	/*Property = FileReader*/
const Value V18517_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_305 = {S25, 0, &V18517_305_57232, &P18517_304};	/*Property = FileSaver*/
const Value V18517_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_306 = {S26, 0, &V18517_306_57238, &P18517_305};	/*Property = FileWriter*/
const Value V18517_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_307 = {S27, 0, &V18517_307_57241, &P18517_306};	/*Property = FormData*/
const Value V18517_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_309 = {S28, 0, &V18517_309_57247, &P18517_307};	/*Property = Fullscreen*/
const Value V18517_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_310 = {S29, 0, &V18517_310_57250, &P18517_309};	/*Property = GeoLocation*/
const Value V18517_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_312 = {S30, 0, &V18517_312_57256, &P18517_310};	/*Property = History*/
const Value V18517_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_314 = {S31, 0, &V18517_314_57262, &P18517_312};	/*Property = Html5*/
const Value V18517_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_313 = {S32, 0, &V18517_313_57259, &P18517_314};	/*Property = Html-Media-Capture*/
const Value V18517_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_315 = {S33, 0, &V18517_315_57265, &P18517_313};	/*Property = Iframe*/
const Value V18517_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_316 = {S34, 0, &V18517_316_57268, &P18517_315};	/*Property = IndexedDB*/
const Value V18517_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_317 = {S35, 0, &V18517_317_57271, &P18517_316};	/*Property = Json*/
const Value V18517_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18517_15 = {S37, 0, &V18517_15_28, &P18517_317};	/*Property = LayoutEngine*/
const Value V18517_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_344 = {S38, 0, &V18517_344_57274, &P18517_15};	/*Property = Masking*/
const Value V18517_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_320 = {S39, 0, &V18517_320_57292, &P18517_344};	/*Property = PostMessage*/
const Value V18517_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_321 = {S40, 0, &V18517_321_57295, &P18517_320};	/*Property = Progress*/
const Value V18517_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_322 = {S41, 0, &V18517_322_57298, &P18517_321};	/*Property = Prompts*/
const Value V18517_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_323 = {S42, 0, &V18517_323_57301, &P18517_322};	/*Property = Selector*/
const Value V18517_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_324 = {S43, 0, &V18517_324_57307, &P18517_323};	/*Property = Svg*/
const Value V18517_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_327 = {S44, 0, &V18517_327_57316, &P18517_324};	/*Property = TouchEvents*/
const Value V18517_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_328 = {S45, 0, &V18517_328_57319, &P18517_327};	/*Property = Track*/
const Value V18517_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_329 = {S46, 0, &V18517_329_57325, &P18517_328};	/*Property = Video*/
const Value V18517_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_330 = {S47, 0, &V18517_330_57328, &P18517_329};	/*Property = Viewport*/
const Value V18517_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_332 = {S48, 0, &V18517_332_57340, &P18517_330};	/*Property = WebWorkers*/
const Value V18517_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18517_333 = {S49, 0, &V18517_333_57343, &P18517_332};	/*Property = Xhr2*/
const Profile P18517 = {"18517", &P18517_333};	/*Profile ID - 18517*/
const Value V18648_1_1 = {S2, NULL};	/*Value = True*/
const Property P18648_1 = {S50, 0, &V18648_1_1, NULL};	/*Property = IsMobile*/
const Value V18648_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P18648_5 = {S52, 0, &V18648_5_1304, &P18648_1};	/*Property = ScreenPixelsHeight*/
const Value V18648_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18648_4 = {S54, 0, &V18648_4_1224, &P18648_5};	/*Property = ScreenPixelsWidth*/
const Profile P18648 = {"18648", &P18648_4};	/*Profile ID - 18648*/
const Value V21999_1_1 = {S2, NULL};	/*Value = True*/
const Property P21999_1 = {S50, 0, &V21999_1_1, NULL};	/*Property = IsMobile*/
const Value V21999_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21999_5 = {S52, 0, &V21999_5_1277, &P21999_1};	/*Property = ScreenPixelsHeight*/
const Value V21999_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21999_4 = {S54, 0, &V21999_4_1279, &P21999_5};	/*Property = ScreenPixelsWidth*/
const Profile P21999 = {"21999", &P21999_4};	/*Profile ID - 21999*/
const Value V21761_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_279 = {S1, 0, &V21761_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21761_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_282 = {S3, 0, &V21761_282_57155, &P21761_279};	/*Property = BlobBuilder*/
const Value V21761_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_350 = {S4, 0, &V21761_350_57446, &P21761_282};	/*Property = Canvas*/
const Value V21761_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_283 = {S5, 0, &V21761_283_57161, &P21761_350};	/*Property = CssBackground*/
const Value V21761_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_284 = {S6, 0, &V21761_284_57164, &P21761_283};	/*Property = CssBorderImage*/
const Value V21761_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_285 = {S7, 0, &V21761_285_57167, &P21761_284};	/*Property = CssCanvas*/
const Value V21761_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_286 = {S8, 0, &V21761_286_57170, &P21761_285};	/*Property = CssColor*/
const Value V21761_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_237 = {S9, 0, &V21761_237_55786, &P21761_286};	/*Property = CssColumn*/
const Value V21761_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_287 = {S10, 0, &V21761_287_57173, &P21761_237};	/*Property = CssFlexbox*/
const Value V21761_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_288 = {S11, 0, &V21761_288_57176, &P21761_287};	/*Property = CssFont*/
const Value V21761_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_289 = {S12, 0, &V21761_289_86899, &P21761_288};	/*Property = CssImages*/
const Value V21761_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_291 = {S13, 0, &V21761_291_57182, &P21761_289};	/*Property = CssMediaQueries*/
const Value V21761_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_292 = {S14, 0, &V21761_292_57185, &P21761_291};	/*Property = CssMinMax*/
const Value V21761_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_293 = {S15, 0, &V21761_293_57188, &P21761_292};	/*Property = CssOverflow*/
const Value V21761_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_294 = {S16, 0, &V21761_294_57191, &P21761_293};	/*Property = CssPosition*/
const Value V21761_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_295 = {S17, 0, &V21761_295_57194, &P21761_294};	/*Property = CssText*/
const Value V21761_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_297 = {S18, 0, &V21761_297_57197, &P21761_295};	/*Property = CssTransforms*/
const Value V21761_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_241 = {S19, 0, &V21761_241_55801, &P21761_297};	/*Property = CssTransitions*/
const Value V21761_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_298 = {S20, 0, &V21761_298_57208, &P21761_241};	/*Property = CssUI*/
const Value V21761_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_300 = {S21, 0, &V21761_300_57217, &P21761_298};	/*Property = DataSet*/
const Value V21761_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_301 = {S22, 0, &V21761_301_57220, &P21761_300};	/*Property = DataUrl*/
const Value V21761_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_302 = {S23, 0, &V21761_302_57223, &P21761_301};	/*Property = DeviceOrientation*/
const Value V21761_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_304 = {S24, 0, &V21761_304_57229, &P21761_302};	/*Property = FileReader*/
const Value V21761_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_305 = {S25, 0, &V21761_305_57232, &P21761_304};	/*Property = FileSaver*/
const Value V21761_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_306 = {S26, 0, &V21761_306_57238, &P21761_305};	/*Property = FileWriter*/
const Value V21761_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_307 = {S27, 0, &V21761_307_57241, &P21761_306};	/*Property = FormData*/
const Value V21761_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_309 = {S28, 0, &V21761_309_57247, &P21761_307};	/*Property = Fullscreen*/
const Value V21761_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_310 = {S29, 0, &V21761_310_57250, &P21761_309};	/*Property = GeoLocation*/
const Value V21761_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_312 = {S30, 0, &V21761_312_57256, &P21761_310};	/*Property = History*/
const Value V21761_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_314 = {S31, 0, &V21761_314_57262, &P21761_312};	/*Property = Html5*/
const Value V21761_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_313 = {S32, 0, &V21761_313_57259, &P21761_314};	/*Property = Html-Media-Capture*/
const Value V21761_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_315 = {S33, 0, &V21761_315_57265, &P21761_313};	/*Property = Iframe*/
const Value V21761_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_316 = {S34, 0, &V21761_316_57268, &P21761_315};	/*Property = IndexedDB*/
const Value V21761_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_317 = {S35, 0, &V21761_317_57271, &P21761_316};	/*Property = Json*/
const Value V21761_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21761_15 = {S37, 0, &V21761_15_54167, &P21761_317};	/*Property = LayoutEngine*/
const Value V21761_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_344 = {S38, 0, &V21761_344_57274, &P21761_15};	/*Property = Masking*/
const Value V21761_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_320 = {S39, 0, &V21761_320_57292, &P21761_344};	/*Property = PostMessage*/
const Value V21761_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_321 = {S40, 0, &V21761_321_57295, &P21761_320};	/*Property = Progress*/
const Value V21761_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_322 = {S41, 0, &V21761_322_57298, &P21761_321};	/*Property = Prompts*/
const Value V21761_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_323 = {S42, 0, &V21761_323_57301, &P21761_322};	/*Property = Selector*/
const Value V21761_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_324 = {S43, 0, &V21761_324_57307, &P21761_323};	/*Property = Svg*/
const Value V21761_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_327 = {S44, 0, &V21761_327_57316, &P21761_324};	/*Property = TouchEvents*/
const Value V21761_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_328 = {S45, 0, &V21761_328_57319, &P21761_327};	/*Property = Track*/
const Value V21761_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_329 = {S46, 0, &V21761_329_57325, &P21761_328};	/*Property = Video*/
const Value V21761_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_330 = {S47, 0, &V21761_330_57328, &P21761_329};	/*Property = Viewport*/
const Value V21761_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_332 = {S48, 0, &V21761_332_57340, &P21761_330};	/*Property = WebWorkers*/
const Value V21761_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21761_333 = {S49, 0, &V21761_333_57343, &P21761_332};	/*Property = Xhr2*/
const Profile P21761 = {"21761", &P21761_333};	/*Profile ID - 21761*/
const Value V21332_1_1 = {S2, NULL};	/*Value = True*/
const Property P21332_1 = {S50, 0, &V21332_1_1, NULL};	/*Property = IsMobile*/
const Value V21332_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21332_5 = {S52, 0, &V21332_5_1277, &P21332_1};	/*Property = ScreenPixelsHeight*/
const Value V21332_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21332_4 = {S54, 0, &V21332_4_1279, &P21332_5};	/*Property = ScreenPixelsWidth*/
const Profile P21332 = {"21332", &P21332_4};	/*Profile ID - 21332*/
const Value V21734_1_1 = {S2, NULL};	/*Value = True*/
const Property P21734_1 = {S50, 0, &V21734_1_1, NULL};	/*Property = IsMobile*/
const Value V21734_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21734_5 = {S52, 0, &V21734_5_1227, &P21734_1};	/*Property = ScreenPixelsHeight*/
const Value V21734_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21734_4 = {S54, 0, &V21734_4_1229, &P21734_5};	/*Property = ScreenPixelsWidth*/
const Profile P21734 = {"21734", &P21734_4};	/*Profile ID - 21734*/
const Value V18739_1_1 = {S2, NULL};	/*Value = True*/
const Property P18739_1 = {S50, 0, &V18739_1_1, NULL};	/*Property = IsMobile*/
const Value V18739_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18739_5 = {S52, 0, &V18739_5_1277, &P18739_1};	/*Property = ScreenPixelsHeight*/
const Value V18739_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18739_4 = {S54, 0, &V18739_4_1279, &P18739_5};	/*Property = ScreenPixelsWidth*/
const Profile P18739 = {"18739", &P18739_4};	/*Profile ID - 18739*/
const Value V21281_1_1 = {S2, NULL};	/*Value = True*/
const Property P21281_1 = {S50, 0, &V21281_1_1, NULL};	/*Property = IsMobile*/
const Value V21281_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21281_5 = {S52, 0, &V21281_5_1458, &P21281_1};	/*Property = ScreenPixelsHeight*/
const Value V21281_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21281_4 = {S54, 0, &V21281_4_37617, &P21281_5};	/*Property = ScreenPixelsWidth*/
const Profile P21281 = {"21281", &P21281_4};	/*Profile ID - 21281*/
const Value V21008_1_1 = {S2, NULL};	/*Value = True*/
const Property P21008_1 = {S50, 0, &V21008_1_1, NULL};	/*Property = IsMobile*/
const Value V21008_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21008_5 = {S52, 0, &V21008_5_1227, &P21008_1};	/*Property = ScreenPixelsHeight*/
const Value V21008_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21008_4 = {S54, 0, &V21008_4_1229, &P21008_5};	/*Property = ScreenPixelsWidth*/
const Profile P21008 = {"21008", &P21008_4};	/*Profile ID - 21008*/
const Value V20329_1_2 = {S0, NULL};	/*Value = False*/
const Property P20329_1 = {S50, 0, &V20329_1_2, NULL};	/*Property = IsMobile*/
const Value V20329_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20329_5 = {S52, 0, &V20329_5_37, &P20329_1};	/*Property = ScreenPixelsHeight*/
const Value V20329_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20329_4 = {S54, 0, &V20329_4_36, &P20329_5};	/*Property = ScreenPixelsWidth*/
const Profile P20329 = {"20329", &P20329_4};	/*Profile ID - 20329*/
const Value V23404_279_57148 = {S0, NULL};	/*Value = False*/
const Property P23404_279 = {S1, 0, &V23404_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V23404_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23404_282 = {S3, 0, &V23404_282_57154, &P23404_279};	/*Property = BlobBuilder*/
const Value V23404_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23404_350 = {S4, 0, &V23404_350_57444, &P23404_282};	/*Property = Canvas*/
const Value V23404_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23404_283 = {S5, 0, &V23404_283_57159, &P23404_350};	/*Property = CssBackground*/
const Value V23404_284_57162 = {S2, NULL};	/*Value = True*/
const Property P23404_284 = {S6, 0, &V23404_284_57162, &P23404_283};	/*Property = CssBorderImage*/
const Value V23404_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23404_285 = {S7, 0, &V23404_285_57166, &P23404_284};	/*Property = CssCanvas*/
const Value V23404_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23404_286 = {S8, 0, &V23404_286_57168, &P23404_285};	/*Property = CssColor*/
const Value V23404_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23404_237 = {S9, 0, &V23404_237_55786, &P23404_286};	/*Property = CssColumn*/
const Value V23404_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23404_287 = {S10, 0, &V23404_287_57172, &P23404_237};	/*Property = CssFlexbox*/
const Value V23404_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23404_288 = {S11, 0, &V23404_288_57174, &P23404_287};	/*Property = CssFont*/
const Value V23404_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23404_289 = {S12, 0, &V23404_289_57759, &P23404_288};	/*Property = CssImages*/
const Value V23404_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23404_291 = {S13, 0, &V23404_291_57180, &P23404_289};	/*Property = CssMediaQueries*/
const Value V23404_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23404_292 = {S14, 0, &V23404_292_57183, &P23404_291};	/*Property = CssMinMax*/
const Value V23404_293_57186 = {S2, NULL};	/*Value = True*/
const Property P23404_293 = {S15, 0, &V23404_293_57186, &P23404_292};	/*Property = CssOverflow*/
const Value V23404_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23404_294 = {S16, 0, &V23404_294_57189, &P23404_293};	/*Property = CssPosition*/
const Value V23404_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23404_295 = {S17, 0, &V23404_295_57192, &P23404_294};	/*Property = CssText*/
const Value V23404_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23404_297 = {S18, 0, &V23404_297_57195, &P23404_295};	/*Property = CssTransforms*/
const Value V23404_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23404_241 = {S19, 0, &V23404_241_55799, &P23404_297};	/*Property = CssTransitions*/
const Value V23404_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23404_298 = {S20, 0, &V23404_298_57206, &P23404_241};	/*Property = CssUI*/
const Value V23404_300_57215 = {S2, NULL};	/*Value = True*/
const Property P23404_300 = {S21, 0, &V23404_300_57215, &P23404_298};	/*Property = DataSet*/
const Value V23404_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23404_301 = {S22, 0, &V23404_301_57218, &P23404_300};	/*Property = DataUrl*/
const Value V23404_302_57221 = {S2, NULL};	/*Value = True*/
const Property P23404_302 = {S23, 0, &V23404_302_57221, &P23404_301};	/*Property = DeviceOrientation*/
const Value V23404_304_57228 = {S0, NULL};	/*Value = False*/
const Property P23404_304 = {S24, 0, &V23404_304_57228, &P23404_302};	/*Property = FileReader*/
const Value V23404_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23404_305 = {S25, 0, &V23404_305_57231, &P23404_304};	/*Property = FileSaver*/
const Value V23404_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23404_306 = {S26, 0, &V23404_306_57237, &P23404_305};	/*Property = FileWriter*/
const Value V23404_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23404_307 = {S27, 0, &V23404_307_57239, &P23404_306};	/*Property = FormData*/
const Value V23404_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23404_309 = {S28, 0, &V23404_309_57246, &P23404_307};	/*Property = Fullscreen*/
const Value V23404_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23404_310 = {S29, 0, &V23404_310_57248, &P23404_309};	/*Property = GeoLocation*/
const Value V23404_312_57255 = {S0, NULL};	/*Value = False*/
const Property P23404_312 = {S30, 0, &V23404_312_57255, &P23404_310};	/*Property = History*/
const Value V23404_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23404_314 = {S31, 0, &V23404_314_57261, &P23404_312};	/*Property = Html5*/
const Value V23404_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23404_313 = {S32, 0, &V23404_313_57258, &P23404_314};	/*Property = Html-Media-Capture*/
const Value V23404_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23404_315 = {S33, 0, &V23404_315_57264, &P23404_313};	/*Property = Iframe*/
const Value V23404_316_57267 = {S0, NULL};	/*Value = False*/
const Property P23404_316 = {S34, 0, &V23404_316_57267, &P23404_315};	/*Property = IndexedDB*/
const Value V23404_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23404_317 = {S35, 0, &V23404_317_57269, &P23404_316};	/*Property = Json*/
const Value V23404_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P23404_15 = {S37, 0, &V23404_15_54167, &P23404_317};	/*Property = LayoutEngine*/
const Value V23404_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23404_344 = {S38, 0, &V23404_344_57272, &P23404_15};	/*Property = Masking*/
const Value V23404_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23404_320 = {S39, 0, &V23404_320_57290, &P23404_344};	/*Property = PostMessage*/
const Value V23404_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23404_321 = {S40, 0, &V23404_321_57293, &P23404_320};	/*Property = Progress*/
const Value V23404_322_57297 = {S0, NULL};	/*Value = False*/
const Property P23404_322 = {S41, 0, &V23404_322_57297, &P23404_321};	/*Property = Prompts*/
const Value V23404_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23404_323 = {S42, 0, &V23404_323_57299, &P23404_322};	/*Property = Selector*/
const Value V23404_324_57306 = {S0, NULL};	/*Value = False*/
const Property P23404_324 = {S43, 0, &V23404_324_57306, &P23404_323};	/*Property = Svg*/
const Value V23404_327_57314 = {S2, NULL};	/*Value = True*/
const Property P23404_327 = {S44, 0, &V23404_327_57314, &P23404_324};	/*Property = TouchEvents*/
const Value V23404_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23404_328 = {S45, 0, &V23404_328_57318, &P23404_327};	/*Property = Track*/
const Value V23404_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23404_329 = {S46, 0, &V23404_329_57323, &P23404_328};	/*Property = Video*/
const Value V23404_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23404_330 = {S47, 0, &V23404_330_57326, &P23404_329};	/*Property = Viewport*/
const Value V23404_332_57339 = {S0, NULL};	/*Value = False*/
const Property P23404_332 = {S48, 0, &V23404_332_57339, &P23404_330};	/*Property = WebWorkers*/
const Value V23404_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23404_333 = {S49, 0, &V23404_333_57342, &P23404_332};	/*Property = Xhr2*/
const Profile P23404 = {"23404", &P23404_333};	/*Profile ID - 23404*/
const Value V21226_1_1 = {S2, NULL};	/*Value = True*/
const Property P21226_1 = {S50, 0, &V21226_1_1, NULL};	/*Property = IsMobile*/
const Value V21226_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21226_5 = {S52, 0, &V21226_5_1222, &P21226_1};	/*Property = ScreenPixelsHeight*/
const Value V21226_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21226_4 = {S54, 0, &V21226_4_1224, &P21226_5};	/*Property = ScreenPixelsWidth*/
const Profile P21226 = {"21226", &P21226_4};	/*Profile ID - 21226*/
const Value V21954_1_1 = {S2, NULL};	/*Value = True*/
const Property P21954_1 = {S50, 0, &V21954_1_1, NULL};	/*Property = IsMobile*/
const Value V21954_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21954_5 = {S52, 0, &V21954_5_1227, &P21954_1};	/*Property = ScreenPixelsHeight*/
const Value V21954_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21954_4 = {S54, 0, &V21954_4_1229, &P21954_5};	/*Property = ScreenPixelsWidth*/
const Profile P21954 = {"21954", &P21954_4};	/*Profile ID - 21954*/
const Value V18488_1_1 = {S2, NULL};	/*Value = True*/
const Property P18488_1 = {S50, 0, &V18488_1_1, NULL};	/*Property = IsMobile*/
const Value V18488_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18488_5 = {S52, 0, &V18488_5_1227, &P18488_1};	/*Property = ScreenPixelsHeight*/
const Value V18488_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18488_4 = {S54, 0, &V18488_4_1229, &P18488_5};	/*Property = ScreenPixelsWidth*/
const Profile P18488 = {"18488", &P18488_4};	/*Profile ID - 18488*/
const Value V21224_1_1 = {S2, NULL};	/*Value = True*/
const Property P21224_1 = {S50, 0, &V21224_1_1, NULL};	/*Property = IsMobile*/
const Value V21224_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21224_5 = {S52, 0, &V21224_5_9981, &P21224_1};	/*Property = ScreenPixelsHeight*/
const Value V21224_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21224_4 = {S54, 0, &V21224_4_39098, &P21224_5};	/*Property = ScreenPixelsWidth*/
const Profile P21224 = {"21224", &P21224_4};	/*Profile ID - 21224*/
const Value V21237_1_1 = {S2, NULL};	/*Value = True*/
const Property P21237_1 = {S50, 0, &V21237_1_1, NULL};	/*Property = IsMobile*/
const Value V21237_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21237_5 = {S52, 0, &V21237_5_1227, &P21237_1};	/*Property = ScreenPixelsHeight*/
const Value V21237_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21237_4 = {S54, 0, &V21237_4_1229, &P21237_5};	/*Property = ScreenPixelsWidth*/
const Profile P21237 = {"21237", &P21237_4};	/*Profile ID - 21237*/
const Value V20688_1_1 = {S2, NULL};	/*Value = True*/
const Property P20688_1 = {S50, 0, &V20688_1_1, NULL};	/*Property = IsMobile*/
const Value V20688_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20688_5 = {S52, 0, &V20688_5_1227, &P20688_1};	/*Property = ScreenPixelsHeight*/
const Value V20688_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20688_4 = {S54, 0, &V20688_4_1229, &P20688_5};	/*Property = ScreenPixelsWidth*/
const Profile P20688 = {"20688", &P20688_4};	/*Profile ID - 20688*/
const Value V22099_1_1 = {S2, NULL};	/*Value = True*/
const Property P22099_1 = {S50, 0, &V22099_1_1, NULL};	/*Property = IsMobile*/
const Value V22099_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22099_5 = {S52, 0, &V22099_5_37569, &P22099_1};	/*Property = ScreenPixelsHeight*/
const Value V22099_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22099_4 = {S54, 0, &V22099_4_5645, &P22099_5};	/*Property = ScreenPixelsWidth*/
const Profile P22099 = {"22099", &P22099_4};	/*Profile ID - 22099*/
const Value V21097_1_1 = {S2, NULL};	/*Value = True*/
const Property P21097_1 = {S50, 0, &V21097_1_1, NULL};	/*Property = IsMobile*/
const Value V21097_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21097_5 = {S52, 0, &V21097_5_1277, &P21097_1};	/*Property = ScreenPixelsHeight*/
const Value V21097_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21097_4 = {S54, 0, &V21097_4_1279, &P21097_5};	/*Property = ScreenPixelsWidth*/
const Profile P21097 = {"21097", &P21097_4};	/*Profile ID - 21097*/
const Value V23935_1_1 = {S2, NULL};	/*Value = True*/
const Property P23935_1 = {S50, 0, &V23935_1_1, NULL};	/*Property = IsMobile*/
const Value V23935_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23935_5 = {S52, 0, &V23935_5_1832, &P23935_1};	/*Property = ScreenPixelsHeight*/
const Value V23935_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23935_4 = {S54, 0, &V23935_4_2336, &P23935_5};	/*Property = ScreenPixelsWidth*/
const Profile P23935 = {"23935", &P23935_4};	/*Profile ID - 23935*/
const Value V21353_1_1 = {S2, NULL};	/*Value = True*/
const Property P21353_1 = {S50, 0, &V21353_1_1, NULL};	/*Property = IsMobile*/
const Value V21353_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21353_5 = {S52, 0, &V21353_5_1277, &P21353_1};	/*Property = ScreenPixelsHeight*/
const Value V21353_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21353_4 = {S54, 0, &V21353_4_1279, &P21353_5};	/*Property = ScreenPixelsWidth*/
const Profile P21353 = {"21353", &P21353_4};	/*Profile ID - 21353*/
const Value V18676_1_1 = {S2, NULL};	/*Value = True*/
const Property P18676_1 = {S50, 0, &V18676_1_1, NULL};	/*Property = IsMobile*/
const Value V18676_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18676_5 = {S52, 0, &V18676_5_9981, &P18676_1};	/*Property = ScreenPixelsHeight*/
const Value V18676_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18676_4 = {S54, 0, &V18676_4_39098, &P18676_5};	/*Property = ScreenPixelsWidth*/
const Profile P18676 = {"18676", &P18676_4};	/*Profile ID - 18676*/
const Value V24013_1_1 = {S2, NULL};	/*Value = True*/
const Property P24013_1 = {S50, 0, &V24013_1_1, NULL};	/*Property = IsMobile*/
const Value V24013_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P24013_5 = {S52, 0, &V24013_5_1269, &P24013_1};	/*Property = ScreenPixelsHeight*/
const Value V24013_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24013_4 = {S54, 0, &V24013_4_1229, &P24013_5};	/*Property = ScreenPixelsWidth*/
const Profile P24013 = {"24013", &P24013_4};	/*Profile ID - 24013*/
const Value V20170_1_1 = {S2, NULL};	/*Value = True*/
const Property P20170_1 = {S50, 0, &V20170_1_1, NULL};	/*Property = IsMobile*/
const Value V20170_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20170_5 = {S52, 0, &V20170_5_1222, &P20170_1};	/*Property = ScreenPixelsHeight*/
const Value V20170_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20170_4 = {S54, 0, &V20170_4_1224, &P20170_5};	/*Property = ScreenPixelsWidth*/
const Profile P20170 = {"20170", &P20170_4};	/*Profile ID - 20170*/
const Value V20585_1_1 = {S2, NULL};	/*Value = True*/
const Property P20585_1 = {S50, 0, &V20585_1_1, NULL};	/*Property = IsMobile*/
const Value V20585_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20585_5 = {S52, 0, &V20585_5_1227, &P20585_1};	/*Property = ScreenPixelsHeight*/
const Value V20585_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20585_4 = {S54, 0, &V20585_4_1229, &P20585_5};	/*Property = ScreenPixelsWidth*/
const Profile P20585 = {"20585", &P20585_4};	/*Profile ID - 20585*/
const Value V18253_1_1 = {S2, NULL};	/*Value = True*/
const Property P18253_1 = {S50, 0, &V18253_1_1, NULL};	/*Property = IsMobile*/
const Value V18253_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18253_5 = {S52, 0, &V18253_5_1403, &P18253_1};	/*Property = ScreenPixelsHeight*/
const Value V18253_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18253_4 = {S54, 0, &V18253_4_1404, &P18253_5};	/*Property = ScreenPixelsWidth*/
const Profile P18253 = {"18253", &P18253_4};	/*Profile ID - 18253*/
const Value V20563_1_1 = {S2, NULL};	/*Value = True*/
const Property P20563_1 = {S50, 0, &V20563_1_1, NULL};	/*Property = IsMobile*/
const Value V20563_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20563_5 = {S52, 0, &V20563_5_9981, &P20563_1};	/*Property = ScreenPixelsHeight*/
const Value V20563_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20563_4 = {S54, 0, &V20563_4_2336, &P20563_5};	/*Property = ScreenPixelsWidth*/
const Profile P20563 = {"20563", &P20563_4};	/*Profile ID - 20563*/
const Value V21078_1_1 = {S2, NULL};	/*Value = True*/
const Property P21078_1 = {S50, 0, &V21078_1_1, NULL};	/*Property = IsMobile*/
const Value V21078_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21078_5 = {S52, 0, &V21078_5_1227, &P21078_1};	/*Property = ScreenPixelsHeight*/
const Value V21078_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21078_4 = {S54, 0, &V21078_4_1229, &P21078_5};	/*Property = ScreenPixelsWidth*/
const Profile P21078 = {"21078", &P21078_4};	/*Profile ID - 21078*/
const Value V21357_1_1 = {S2, NULL};	/*Value = True*/
const Property P21357_1 = {S50, 0, &V21357_1_1, NULL};	/*Property = IsMobile*/
const Value V21357_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21357_5 = {S52, 0, &V21357_5_1277, &P21357_1};	/*Property = ScreenPixelsHeight*/
const Value V21357_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21357_4 = {S54, 0, &V21357_4_1279, &P21357_5};	/*Property = ScreenPixelsWidth*/
const Profile P21357 = {"21357", &P21357_4};	/*Profile ID - 21357*/
const Value V19907_1_1 = {S2, NULL};	/*Value = True*/
const Property P19907_1 = {S50, 0, &V19907_1_1, NULL};	/*Property = IsMobile*/
const Value V19907_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P19907_5 = {S52, 0, &V19907_5_1277, &P19907_1};	/*Property = ScreenPixelsHeight*/
const Value V19907_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P19907_4 = {S54, 0, &V19907_4_1279, &P19907_5};	/*Property = ScreenPixelsWidth*/
const Profile P19907 = {"19907", &P19907_4};	/*Profile ID - 19907*/
const Value V21275_1_1 = {S2, NULL};	/*Value = True*/
const Property P21275_1 = {S50, 0, &V21275_1_1, NULL};	/*Property = IsMobile*/
const Value V21275_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21275_5 = {S52, 0, &V21275_5_1458, &P21275_1};	/*Property = ScreenPixelsHeight*/
const Value V21275_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21275_4 = {S54, 0, &V21275_4_1460, &P21275_5};	/*Property = ScreenPixelsWidth*/
const Profile P21275 = {"21275", &P21275_4};	/*Profile ID - 21275*/
const Value V21337_1_1 = {S2, NULL};	/*Value = True*/
const Property P21337_1 = {S50, 0, &V21337_1_1, NULL};	/*Property = IsMobile*/
const Value V21337_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21337_5 = {S52, 0, &V21337_5_1227, &P21337_1};	/*Property = ScreenPixelsHeight*/
const Value V21337_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21337_4 = {S54, 0, &V21337_4_1229, &P21337_5};	/*Property = ScreenPixelsWidth*/
const Profile P21337 = {"21337", &P21337_4};	/*Profile ID - 21337*/
const Value V21148_1_1 = {S2, NULL};	/*Value = True*/
const Property P21148_1 = {S50, 0, &V21148_1_1, NULL};	/*Property = IsMobile*/
const Value V21148_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21148_5 = {S52, 0, &V21148_5_1458, &P21148_1};	/*Property = ScreenPixelsHeight*/
const Value V21148_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21148_4 = {S54, 0, &V21148_4_37617, &P21148_5};	/*Property = ScreenPixelsWidth*/
const Profile P21148 = {"21148", &P21148_4};	/*Profile ID - 21148*/
const Value V21091_1_1 = {S2, NULL};	/*Value = True*/
const Property P21091_1 = {S50, 0, &V21091_1_1, NULL};	/*Property = IsMobile*/
const Value V21091_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21091_5 = {S52, 0, &V21091_5_1227, &P21091_1};	/*Property = ScreenPixelsHeight*/
const Value V21091_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21091_4 = {S54, 0, &V21091_4_1229, &P21091_5};	/*Property = ScreenPixelsWidth*/
const Profile P21091 = {"21091", &P21091_4};	/*Profile ID - 21091*/
const Value V24465_1_1 = {S2, NULL};	/*Value = True*/
const Property P24465_1 = {S50, 0, &V24465_1_1, NULL};	/*Property = IsMobile*/
const Value V24465_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P24465_5 = {S52, 0, &V24465_5_1304, &P24465_1};	/*Property = ScreenPixelsHeight*/
const Value V24465_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P24465_4 = {S54, 0, &V24465_4_1224, &P24465_5};	/*Property = ScreenPixelsWidth*/
const Profile P24465 = {"24465", &P24465_4};	/*Profile ID - 24465*/
const Value V23777_1_1 = {S2, NULL};	/*Value = True*/
const Property P23777_1 = {S50, 0, &V23777_1_1, NULL};	/*Property = IsMobile*/
const Value V23777_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23777_5 = {S52, 0, &V23777_5_1227, &P23777_1};	/*Property = ScreenPixelsHeight*/
const Value V23777_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23777_4 = {S54, 0, &V23777_4_1460, &P23777_5};	/*Property = ScreenPixelsWidth*/
const Profile P23777 = {"23777", &P23777_4};	/*Profile ID - 23777*/
const Value V21154_1_1 = {S2, NULL};	/*Value = True*/
const Property P21154_1 = {S50, 0, &V21154_1_1, NULL};	/*Property = IsMobile*/
const Value V21154_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21154_5 = {S52, 0, &V21154_5_1277, &P21154_1};	/*Property = ScreenPixelsHeight*/
const Value V21154_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21154_4 = {S54, 0, &V21154_4_1279, &P21154_5};	/*Property = ScreenPixelsWidth*/
const Profile P21154 = {"21154", &P21154_4};	/*Profile ID - 21154*/
const Value V21173_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21173_279 = {S1, 0, &V21173_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21173_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21173_282 = {S3, 0, &V21173_282_57154, &P21173_279};	/*Property = BlobBuilder*/
const Value V21173_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21173_350 = {S4, 0, &V21173_350_57444, &P21173_282};	/*Property = Canvas*/
const Value V21173_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21173_283 = {S5, 0, &V21173_283_57159, &P21173_350};	/*Property = CssBackground*/
const Value V21173_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21173_284 = {S6, 0, &V21173_284_57163, &P21173_283};	/*Property = CssBorderImage*/
const Value V21173_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21173_285 = {S7, 0, &V21173_285_57166, &P21173_284};	/*Property = CssCanvas*/
const Value V21173_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21173_286 = {S8, 0, &V21173_286_57168, &P21173_285};	/*Property = CssColor*/
const Value V21173_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21173_237 = {S9, 0, &V21173_237_55784, &P21173_286};	/*Property = CssColumn*/
const Value V21173_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21173_287 = {S10, 0, &V21173_287_57172, &P21173_237};	/*Property = CssFlexbox*/
const Value V21173_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21173_288 = {S11, 0, &V21173_288_57175, &P21173_287};	/*Property = CssFont*/
const Value V21173_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21173_289 = {S12, 0, &V21173_289_57759, &P21173_288};	/*Property = CssImages*/
const Value V21173_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21173_291 = {S13, 0, &V21173_291_57181, &P21173_289};	/*Property = CssMediaQueries*/
const Value V21173_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21173_292 = {S14, 0, &V21173_292_57183, &P21173_291};	/*Property = CssMinMax*/
const Value V21173_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21173_293 = {S15, 0, &V21173_293_57187, &P21173_292};	/*Property = CssOverflow*/
const Value V21173_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21173_294 = {S16, 0, &V21173_294_57190, &P21173_293};	/*Property = CssPosition*/
const Value V21173_295_57193 = {S0, NULL};	/*Value = False*/
const Property P21173_295 = {S17, 0, &V21173_295_57193, &P21173_294};	/*Property = CssText*/
const Value V21173_297_57196 = {S0, NULL};	/*Value = False*/
const Property P21173_297 = {S18, 0, &V21173_297_57196, &P21173_295};	/*Property = CssTransforms*/
const Value V21173_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21173_241 = {S19, 0, &V21173_241_55799, &P21173_297};	/*Property = CssTransitions*/
const Value V21173_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21173_298 = {S20, 0, &V21173_298_57206, &P21173_241};	/*Property = CssUI*/
const Value V21173_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21173_300 = {S21, 0, &V21173_300_57216, &P21173_298};	/*Property = DataSet*/
const Value V21173_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21173_301 = {S22, 0, &V21173_301_57218, &P21173_300};	/*Property = DataUrl*/
const Value V21173_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21173_302 = {S23, 0, &V21173_302_57222, &P21173_301};	/*Property = DeviceOrientation*/
const Value V21173_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21173_304 = {S24, 0, &V21173_304_57228, &P21173_302};	/*Property = FileReader*/
const Value V21173_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21173_305 = {S25, 0, &V21173_305_57231, &P21173_304};	/*Property = FileSaver*/
const Value V21173_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21173_306 = {S26, 0, &V21173_306_57237, &P21173_305};	/*Property = FileWriter*/
const Value V21173_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21173_307 = {S27, 0, &V21173_307_57240, &P21173_306};	/*Property = FormData*/
const Value V21173_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21173_309 = {S28, 0, &V21173_309_57246, &P21173_307};	/*Property = Fullscreen*/
const Value V21173_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21173_310 = {S29, 0, &V21173_310_57248, &P21173_309};	/*Property = GeoLocation*/
const Value V21173_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21173_312 = {S30, 0, &V21173_312_57255, &P21173_310};	/*Property = History*/
const Value V21173_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21173_314 = {S31, 0, &V21173_314_57261, &P21173_312};	/*Property = Html5*/
const Value V21173_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21173_313 = {S32, 0, &V21173_313_57258, &P21173_314};	/*Property = Html-Media-Capture*/
const Value V21173_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21173_315 = {S33, 0, &V21173_315_57264, &P21173_313};	/*Property = Iframe*/
const Value V21173_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21173_316 = {S34, 0, &V21173_316_57267, &P21173_315};	/*Property = IndexedDB*/
const Value V21173_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21173_317 = {S35, 0, &V21173_317_57269, &P21173_316};	/*Property = Json*/
const Value V21173_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P21173_15 = {S37, 0, &V21173_15_31, &P21173_317};	/*Property = LayoutEngine*/
const Value V21173_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21173_344 = {S38, 0, &V21173_344_57272, &P21173_15};	/*Property = Masking*/
const Value V21173_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21173_320 = {S39, 0, &V21173_320_57290, &P21173_344};	/*Property = PostMessage*/
const Value V21173_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21173_321 = {S40, 0, &V21173_321_57293, &P21173_320};	/*Property = Progress*/
const Value V21173_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21173_322 = {S41, 0, &V21173_322_57296, &P21173_321};	/*Property = Prompts*/
const Value V21173_323_57300 = {S0, NULL};	/*Value = False*/
const Property P21173_323 = {S42, 0, &V21173_323_57300, &P21173_322};	/*Property = Selector*/
const Value V21173_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21173_324 = {S43, 0, &V21173_324_57306, &P21173_323};	/*Property = Svg*/
const Value V21173_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21173_327 = {S44, 0, &V21173_327_57315, &P21173_324};	/*Property = TouchEvents*/
const Value V21173_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21173_328 = {S45, 0, &V21173_328_57318, &P21173_327};	/*Property = Track*/
const Value V21173_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21173_329 = {S46, 0, &V21173_329_57323, &P21173_328};	/*Property = Video*/
const Value V21173_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21173_330 = {S47, 0, &V21173_330_57326, &P21173_329};	/*Property = Viewport*/
const Value V21173_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21173_332 = {S48, 0, &V21173_332_57339, &P21173_330};	/*Property = WebWorkers*/
const Value V21173_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21173_333 = {S49, 0, &V21173_333_57342, &P21173_332};	/*Property = Xhr2*/
const Profile P21173 = {"21173", &P21173_333};	/*Profile ID - 21173*/
const Value V22011_1_1 = {S2, NULL};	/*Value = True*/
const Property P22011_1 = {S50, 0, &V22011_1_1, NULL};	/*Property = IsMobile*/
const Value V22011_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22011_5 = {S52, 0, &V22011_5_1832, &P22011_1};	/*Property = ScreenPixelsHeight*/
const Value V22011_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22011_4 = {S54, 0, &V22011_4_2336, &P22011_5};	/*Property = ScreenPixelsWidth*/
const Profile P22011 = {"22011", &P22011_4};	/*Profile ID - 22011*/
const Value V23985_1_1 = {S2, NULL};	/*Value = True*/
const Property P23985_1 = {S50, 0, &V23985_1_1, NULL};	/*Property = IsMobile*/
const Value V23985_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23985_5 = {S52, 0, &V23985_5_1277, &P23985_1};	/*Property = ScreenPixelsHeight*/
const Value V23985_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23985_4 = {S54, 0, &V23985_4_1279, &P23985_5};	/*Property = ScreenPixelsWidth*/
const Profile P23985 = {"23985", &P23985_4};	/*Profile ID - 23985*/
const Value V18643_1_1 = {S2, NULL};	/*Value = True*/
const Property P18643_1 = {S50, 0, &V18643_1_1, NULL};	/*Property = IsMobile*/
const Value V18643_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P18643_5 = {S52, 0, &V18643_5_1304, &P18643_1};	/*Property = ScreenPixelsHeight*/
const Value V18643_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18643_4 = {S54, 0, &V18643_4_1224, &P18643_5};	/*Property = ScreenPixelsWidth*/
const Profile P18643 = {"18643", &P18643_4};	/*Profile ID - 18643*/
const Value V23915_1_1 = {S2, NULL};	/*Value = True*/
const Property P23915_1 = {S50, 0, &V23915_1_1, NULL};	/*Property = IsMobile*/
const Value V23915_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23915_5 = {S52, 0, &V23915_5_1227, &P23915_1};	/*Property = ScreenPixelsHeight*/
const Value V23915_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23915_4 = {S54, 0, &V23915_4_1229, &P23915_5};	/*Property = ScreenPixelsWidth*/
const Profile P23915 = {"23915", &P23915_4};	/*Profile ID - 23915*/
const Value V20381_1_1 = {S2, NULL};	/*Value = True*/
const Property P20381_1 = {S50, 0, &V20381_1_1, NULL};	/*Property = IsMobile*/
const Value V20381_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20381_5 = {S52, 0, &V20381_5_37, &P20381_1};	/*Property = ScreenPixelsHeight*/
const Value V20381_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20381_4 = {S54, 0, &V20381_4_36, &P20381_5};	/*Property = ScreenPixelsWidth*/
const Profile P20381 = {"20381", &P20381_4};	/*Profile ID - 20381*/
const Value V23083_1_1 = {S2, NULL};	/*Value = True*/
const Property P23083_1 = {S50, 0, &V23083_1_1, NULL};	/*Property = IsMobile*/
const Value V23083_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23083_5 = {S52, 0, &V23083_5_1222, &P23083_1};	/*Property = ScreenPixelsHeight*/
const Value V23083_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23083_4 = {S54, 0, &V23083_4_1224, &P23083_5};	/*Property = ScreenPixelsWidth*/
const Profile P23083 = {"23083", &P23083_4};	/*Profile ID - 23083*/
const Value V22269_1_1 = {S2, NULL};	/*Value = True*/
const Property P22269_1 = {S50, 0, &V22269_1_1, NULL};	/*Property = IsMobile*/
const Value V22269_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22269_5 = {S52, 0, &V22269_5_37569, &P22269_1};	/*Property = ScreenPixelsHeight*/
const Value V22269_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22269_4 = {S54, 0, &V22269_4_5645, &P22269_5};	/*Property = ScreenPixelsWidth*/
const Profile P22269 = {"22269", &P22269_4};	/*Profile ID - 22269*/
const Value V20937_1_1 = {S2, NULL};	/*Value = True*/
const Property P20937_1 = {S50, 0, &V20937_1_1, NULL};	/*Property = IsMobile*/
const Value V20937_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20937_5 = {S52, 0, &V20937_5_1227, &P20937_1};	/*Property = ScreenPixelsHeight*/
const Value V20937_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20937_4 = {S54, 0, &V20937_4_1229, &P20937_5};	/*Property = ScreenPixelsWidth*/
const Profile P20937 = {"20937", &P20937_4};	/*Profile ID - 20937*/
const Value V21621_1_1 = {S2, NULL};	/*Value = True*/
const Property P21621_1 = {S50, 0, &V21621_1_1, NULL};	/*Property = IsMobile*/
const Value V21621_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21621_5 = {S52, 0, &V21621_5_1227, &P21621_1};	/*Property = ScreenPixelsHeight*/
const Value V21621_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21621_4 = {S54, 0, &V21621_4_1229, &P21621_5};	/*Property = ScreenPixelsWidth*/
const Profile P21621 = {"21621", &P21621_4};	/*Profile ID - 21621*/
const Value V21643_1_1 = {S2, NULL};	/*Value = True*/
const Property P21643_1 = {S50, 0, &V21643_1_1, NULL};	/*Property = IsMobile*/
const Value V21643_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P21643_5 = {S52, 0, &V21643_5_1304, &P21643_1};	/*Property = ScreenPixelsHeight*/
const Value V21643_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21643_4 = {S54, 0, &V21643_4_1224, &P21643_5};	/*Property = ScreenPixelsWidth*/
const Profile P21643 = {"21643", &P21643_4};	/*Profile ID - 21643*/
const Value V21101_1_1 = {S2, NULL};	/*Value = True*/
const Property P21101_1 = {S50, 0, &V21101_1_1, NULL};	/*Property = IsMobile*/
const Value V21101_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21101_5 = {S52, 0, &V21101_5_1403, &P21101_1};	/*Property = ScreenPixelsHeight*/
const Value V21101_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21101_4 = {S54, 0, &V21101_4_1404, &P21101_5};	/*Property = ScreenPixelsWidth*/
const Profile P21101 = {"21101", &P21101_4};	/*Profile ID - 21101*/
const Value V20084_1_2 = {S0, NULL};	/*Value = False*/
const Property P20084_1 = {S50, 0, &V20084_1_2, NULL};	/*Property = IsMobile*/
const Value V20084_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20084_5 = {S52, 0, &V20084_5_37, &P20084_1};	/*Property = ScreenPixelsHeight*/
const Value V20084_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20084_4 = {S54, 0, &V20084_4_36, &P20084_5};	/*Property = ScreenPixelsWidth*/
const Profile P20084 = {"20084", &P20084_4};	/*Profile ID - 20084*/
const Value V23274_1_1 = {S2, NULL};	/*Value = True*/
const Property P23274_1 = {S50, 0, &V23274_1_1, NULL};	/*Property = IsMobile*/
const Value V23274_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23274_5 = {S52, 0, &V23274_5_1458, &P23274_1};	/*Property = ScreenPixelsHeight*/
const Value V23274_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23274_4 = {S54, 0, &V23274_4_37617, &P23274_5};	/*Property = ScreenPixelsWidth*/
const Profile P23274 = {"23274", &P23274_4};	/*Profile ID - 23274*/
const Value V24009_1_1 = {S2, NULL};	/*Value = True*/
const Property P24009_1 = {S50, 0, &V24009_1_1, NULL};	/*Property = IsMobile*/
const Value V24009_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24009_5 = {S52, 0, &V24009_5_1227, &P24009_1};	/*Property = ScreenPixelsHeight*/
const Value V24009_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P24009_4 = {S54, 0, &V24009_4_1460, &P24009_5};	/*Property = ScreenPixelsWidth*/
const Profile P24009 = {"24009", &P24009_4};	/*Profile ID - 24009*/
const Value V18745_1_1 = {S2, NULL};	/*Value = True*/
const Property P18745_1 = {S50, 0, &V18745_1_1, NULL};	/*Property = IsMobile*/
const Value V18745_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18745_5 = {S52, 0, &V18745_5_1227, &P18745_1};	/*Property = ScreenPixelsHeight*/
const Value V18745_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18745_4 = {S54, 0, &V18745_4_1229, &P18745_5};	/*Property = ScreenPixelsWidth*/
const Profile P18745 = {"18745", &P18745_4};	/*Profile ID - 18745*/
const Value V21813_1_1 = {S2, NULL};	/*Value = True*/
const Property P21813_1 = {S50, 0, &V21813_1_1, NULL};	/*Property = IsMobile*/
const Value V21813_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21813_5 = {S52, 0, &V21813_5_1227, &P21813_1};	/*Property = ScreenPixelsHeight*/
const Value V21813_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21813_4 = {S54, 0, &V21813_4_1229, &P21813_5};	/*Property = ScreenPixelsWidth*/
const Profile P21813 = {"21813", &P21813_4};	/*Profile ID - 21813*/
const Value V21429_1_1 = {S2, NULL};	/*Value = True*/
const Property P21429_1 = {S50, 0, &V21429_1_1, NULL};	/*Property = IsMobile*/
const Value V21429_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21429_5 = {S52, 0, &V21429_5_1227, &P21429_1};	/*Property = ScreenPixelsHeight*/
const Value V21429_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21429_4 = {S54, 0, &V21429_4_1229, &P21429_5};	/*Property = ScreenPixelsWidth*/
const Profile P21429 = {"21429", &P21429_4};	/*Profile ID - 21429*/
const Value V18352_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_279 = {S1, 0, &V18352_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18352_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_282 = {S3, 0, &V18352_282_57155, &P18352_279};	/*Property = BlobBuilder*/
const Value V18352_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_350 = {S4, 0, &V18352_350_57446, &P18352_282};	/*Property = Canvas*/
const Value V18352_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_283 = {S5, 0, &V18352_283_57161, &P18352_350};	/*Property = CssBackground*/
const Value V18352_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_284 = {S6, 0, &V18352_284_57164, &P18352_283};	/*Property = CssBorderImage*/
const Value V18352_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_285 = {S7, 0, &V18352_285_57167, &P18352_284};	/*Property = CssCanvas*/
const Value V18352_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_286 = {S8, 0, &V18352_286_57170, &P18352_285};	/*Property = CssColor*/
const Value V18352_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_237 = {S9, 0, &V18352_237_55786, &P18352_286};	/*Property = CssColumn*/
const Value V18352_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_287 = {S10, 0, &V18352_287_57173, &P18352_237};	/*Property = CssFlexbox*/
const Value V18352_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_288 = {S11, 0, &V18352_288_57176, &P18352_287};	/*Property = CssFont*/
const Value V18352_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_289 = {S12, 0, &V18352_289_86899, &P18352_288};	/*Property = CssImages*/
const Value V18352_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_291 = {S13, 0, &V18352_291_57182, &P18352_289};	/*Property = CssMediaQueries*/
const Value V18352_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_292 = {S14, 0, &V18352_292_57185, &P18352_291};	/*Property = CssMinMax*/
const Value V18352_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_293 = {S15, 0, &V18352_293_57188, &P18352_292};	/*Property = CssOverflow*/
const Value V18352_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_294 = {S16, 0, &V18352_294_57191, &P18352_293};	/*Property = CssPosition*/
const Value V18352_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_295 = {S17, 0, &V18352_295_57194, &P18352_294};	/*Property = CssText*/
const Value V18352_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_297 = {S18, 0, &V18352_297_57197, &P18352_295};	/*Property = CssTransforms*/
const Value V18352_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_241 = {S19, 0, &V18352_241_55801, &P18352_297};	/*Property = CssTransitions*/
const Value V18352_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_298 = {S20, 0, &V18352_298_57208, &P18352_241};	/*Property = CssUI*/
const Value V18352_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_300 = {S21, 0, &V18352_300_57217, &P18352_298};	/*Property = DataSet*/
const Value V18352_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_301 = {S22, 0, &V18352_301_57220, &P18352_300};	/*Property = DataUrl*/
const Value V18352_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_302 = {S23, 0, &V18352_302_57223, &P18352_301};	/*Property = DeviceOrientation*/
const Value V18352_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_304 = {S24, 0, &V18352_304_57229, &P18352_302};	/*Property = FileReader*/
const Value V18352_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_305 = {S25, 0, &V18352_305_57232, &P18352_304};	/*Property = FileSaver*/
const Value V18352_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_306 = {S26, 0, &V18352_306_57238, &P18352_305};	/*Property = FileWriter*/
const Value V18352_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_307 = {S27, 0, &V18352_307_57241, &P18352_306};	/*Property = FormData*/
const Value V18352_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_309 = {S28, 0, &V18352_309_57247, &P18352_307};	/*Property = Fullscreen*/
const Value V18352_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_310 = {S29, 0, &V18352_310_57250, &P18352_309};	/*Property = GeoLocation*/
const Value V18352_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_312 = {S30, 0, &V18352_312_57256, &P18352_310};	/*Property = History*/
const Value V18352_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_314 = {S31, 0, &V18352_314_57262, &P18352_312};	/*Property = Html5*/
const Value V18352_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_313 = {S32, 0, &V18352_313_57259, &P18352_314};	/*Property = Html-Media-Capture*/
const Value V18352_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_315 = {S33, 0, &V18352_315_57265, &P18352_313};	/*Property = Iframe*/
const Value V18352_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_316 = {S34, 0, &V18352_316_57268, &P18352_315};	/*Property = IndexedDB*/
const Value V18352_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_317 = {S35, 0, &V18352_317_57271, &P18352_316};	/*Property = Json*/
const Value V18352_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18352_15 = {S37, 0, &V18352_15_29, &P18352_317};	/*Property = LayoutEngine*/
const Value V18352_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_344 = {S38, 0, &V18352_344_57274, &P18352_15};	/*Property = Masking*/
const Value V18352_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_320 = {S39, 0, &V18352_320_57292, &P18352_344};	/*Property = PostMessage*/
const Value V18352_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_321 = {S40, 0, &V18352_321_57295, &P18352_320};	/*Property = Progress*/
const Value V18352_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_322 = {S41, 0, &V18352_322_57298, &P18352_321};	/*Property = Prompts*/
const Value V18352_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_323 = {S42, 0, &V18352_323_57301, &P18352_322};	/*Property = Selector*/
const Value V18352_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_324 = {S43, 0, &V18352_324_57307, &P18352_323};	/*Property = Svg*/
const Value V18352_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_327 = {S44, 0, &V18352_327_57316, &P18352_324};	/*Property = TouchEvents*/
const Value V18352_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_328 = {S45, 0, &V18352_328_57319, &P18352_327};	/*Property = Track*/
const Value V18352_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_329 = {S46, 0, &V18352_329_57325, &P18352_328};	/*Property = Video*/
const Value V18352_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_330 = {S47, 0, &V18352_330_57328, &P18352_329};	/*Property = Viewport*/
const Value V18352_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_332 = {S48, 0, &V18352_332_57340, &P18352_330};	/*Property = WebWorkers*/
const Value V18352_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18352_333 = {S49, 0, &V18352_333_57343, &P18352_332};	/*Property = Xhr2*/
const Profile P18352 = {"18352", &P18352_333};	/*Profile ID - 18352*/
const Value V20964_1_1 = {S2, NULL};	/*Value = True*/
const Property P20964_1 = {S50, 0, &V20964_1_1, NULL};	/*Property = IsMobile*/
const Value V20964_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20964_5 = {S52, 0, &V20964_5_1277, &P20964_1};	/*Property = ScreenPixelsHeight*/
const Value V20964_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20964_4 = {S54, 0, &V20964_4_1279, &P20964_5};	/*Property = ScreenPixelsWidth*/
const Profile P20964 = {"20964", &P20964_4};	/*Profile ID - 20964*/
const Value V20019_1_1 = {S2, NULL};	/*Value = True*/
const Property P20019_1 = {S50, 0, &V20019_1_1, NULL};	/*Property = IsMobile*/
const Value V20019_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20019_5 = {S52, 0, &V20019_5_1222, &P20019_1};	/*Property = ScreenPixelsHeight*/
const Value V20019_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20019_4 = {S54, 0, &V20019_4_1224, &P20019_5};	/*Property = ScreenPixelsWidth*/
const Profile P20019 = {"20019", &P20019_4};	/*Profile ID - 20019*/
const Value V21007_1_1 = {S2, NULL};	/*Value = True*/
const Property P21007_1 = {S50, 0, &V21007_1_1, NULL};	/*Property = IsMobile*/
const Value V21007_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21007_5 = {S52, 0, &V21007_5_9981, &P21007_1};	/*Property = ScreenPixelsHeight*/
const Value V21007_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21007_4 = {S54, 0, &V21007_4_39098, &P21007_5};	/*Property = ScreenPixelsWidth*/
const Profile P21007 = {"21007", &P21007_4};	/*Profile ID - 21007*/
const Value V18747_1_1 = {S2, NULL};	/*Value = True*/
const Property P18747_1 = {S50, 0, &V18747_1_1, NULL};	/*Property = IsMobile*/
const Value V18747_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18747_5 = {S52, 0, &V18747_5_9981, &P18747_1};	/*Property = ScreenPixelsHeight*/
const Value V18747_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18747_4 = {S54, 0, &V18747_4_39098, &P18747_5};	/*Property = ScreenPixelsWidth*/
const Profile P18747 = {"18747", &P18747_4};	/*Profile ID - 18747*/
const Value V21755_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21755_279 = {S1, 0, &V21755_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21755_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21755_282 = {S3, 0, &V21755_282_57154, &P21755_279};	/*Property = BlobBuilder*/
const Value V21755_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21755_350 = {S4, 0, &V21755_350_57444, &P21755_282};	/*Property = Canvas*/
const Value V21755_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21755_283 = {S5, 0, &V21755_283_57159, &P21755_350};	/*Property = CssBackground*/
const Value V21755_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21755_284 = {S6, 0, &V21755_284_57163, &P21755_283};	/*Property = CssBorderImage*/
const Value V21755_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21755_285 = {S7, 0, &V21755_285_57166, &P21755_284};	/*Property = CssCanvas*/
const Value V21755_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21755_286 = {S8, 0, &V21755_286_57168, &P21755_285};	/*Property = CssColor*/
const Value V21755_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21755_237 = {S9, 0, &V21755_237_55786, &P21755_286};	/*Property = CssColumn*/
const Value V21755_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21755_287 = {S10, 0, &V21755_287_57172, &P21755_237};	/*Property = CssFlexbox*/
const Value V21755_288_57174 = {S2, NULL};	/*Value = True*/
const Property P21755_288 = {S11, 0, &V21755_288_57174, &P21755_287};	/*Property = CssFont*/
const Value V21755_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21755_289 = {S12, 0, &V21755_289_57759, &P21755_288};	/*Property = CssImages*/
const Value V21755_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21755_291 = {S13, 0, &V21755_291_57181, &P21755_289};	/*Property = CssMediaQueries*/
const Value V21755_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21755_292 = {S14, 0, &V21755_292_57183, &P21755_291};	/*Property = CssMinMax*/
const Value V21755_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21755_293 = {S15, 0, &V21755_293_57187, &P21755_292};	/*Property = CssOverflow*/
const Value V21755_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21755_294 = {S16, 0, &V21755_294_57190, &P21755_293};	/*Property = CssPosition*/
const Value V21755_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21755_295 = {S17, 0, &V21755_295_57192, &P21755_294};	/*Property = CssText*/
const Value V21755_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21755_297 = {S18, 0, &V21755_297_57195, &P21755_295};	/*Property = CssTransforms*/
const Value V21755_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21755_241 = {S19, 0, &V21755_241_55799, &P21755_297};	/*Property = CssTransitions*/
const Value V21755_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21755_298 = {S20, 0, &V21755_298_57206, &P21755_241};	/*Property = CssUI*/
const Value V21755_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21755_300 = {S21, 0, &V21755_300_57216, &P21755_298};	/*Property = DataSet*/
const Value V21755_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21755_301 = {S22, 0, &V21755_301_57218, &P21755_300};	/*Property = DataUrl*/
const Value V21755_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21755_302 = {S23, 0, &V21755_302_57222, &P21755_301};	/*Property = DeviceOrientation*/
const Value V21755_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21755_304 = {S24, 0, &V21755_304_57228, &P21755_302};	/*Property = FileReader*/
const Value V21755_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21755_305 = {S25, 0, &V21755_305_57231, &P21755_304};	/*Property = FileSaver*/
const Value V21755_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21755_306 = {S26, 0, &V21755_306_57237, &P21755_305};	/*Property = FileWriter*/
const Value V21755_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21755_307 = {S27, 0, &V21755_307_57240, &P21755_306};	/*Property = FormData*/
const Value V21755_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21755_309 = {S28, 0, &V21755_309_57246, &P21755_307};	/*Property = Fullscreen*/
const Value V21755_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21755_310 = {S29, 0, &V21755_310_57248, &P21755_309};	/*Property = GeoLocation*/
const Value V21755_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21755_312 = {S30, 0, &V21755_312_57255, &P21755_310};	/*Property = History*/
const Value V21755_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21755_314 = {S31, 0, &V21755_314_57261, &P21755_312};	/*Property = Html5*/
const Value V21755_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21755_313 = {S32, 0, &V21755_313_57258, &P21755_314};	/*Property = Html-Media-Capture*/
const Value V21755_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21755_315 = {S33, 0, &V21755_315_57264, &P21755_313};	/*Property = Iframe*/
const Value V21755_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21755_316 = {S34, 0, &V21755_316_57267, &P21755_315};	/*Property = IndexedDB*/
const Value V21755_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21755_317 = {S35, 0, &V21755_317_57269, &P21755_316};	/*Property = Json*/
const Value V21755_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21755_15 = {S37, 0, &V21755_15_54167, &P21755_317};	/*Property = LayoutEngine*/
const Value V21755_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21755_344 = {S38, 0, &V21755_344_57272, &P21755_15};	/*Property = Masking*/
const Value V21755_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21755_320 = {S39, 0, &V21755_320_57290, &P21755_344};	/*Property = PostMessage*/
const Value V21755_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21755_321 = {S40, 0, &V21755_321_57293, &P21755_320};	/*Property = Progress*/
const Value V21755_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21755_322 = {S41, 0, &V21755_322_57296, &P21755_321};	/*Property = Prompts*/
const Value V21755_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21755_323 = {S42, 0, &V21755_323_57299, &P21755_322};	/*Property = Selector*/
const Value V21755_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21755_324 = {S43, 0, &V21755_324_57306, &P21755_323};	/*Property = Svg*/
const Value V21755_327_57314 = {S2, NULL};	/*Value = True*/
const Property P21755_327 = {S44, 0, &V21755_327_57314, &P21755_324};	/*Property = TouchEvents*/
const Value V21755_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21755_328 = {S45, 0, &V21755_328_57318, &P21755_327};	/*Property = Track*/
const Value V21755_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21755_329 = {S46, 0, &V21755_329_57323, &P21755_328};	/*Property = Video*/
const Value V21755_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21755_330 = {S47, 0, &V21755_330_57326, &P21755_329};	/*Property = Viewport*/
const Value V21755_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21755_332 = {S48, 0, &V21755_332_57339, &P21755_330};	/*Property = WebWorkers*/
const Value V21755_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21755_333 = {S49, 0, &V21755_333_57342, &P21755_332};	/*Property = Xhr2*/
const Profile P21755 = {"21755", &P21755_333};	/*Profile ID - 21755*/
const Value V21764_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_279 = {S1, 0, &V21764_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21764_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_282 = {S3, 0, &V21764_282_57155, &P21764_279};	/*Property = BlobBuilder*/
const Value V21764_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_350 = {S4, 0, &V21764_350_57446, &P21764_282};	/*Property = Canvas*/
const Value V21764_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_283 = {S5, 0, &V21764_283_57161, &P21764_350};	/*Property = CssBackground*/
const Value V21764_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_284 = {S6, 0, &V21764_284_57164, &P21764_283};	/*Property = CssBorderImage*/
const Value V21764_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_285 = {S7, 0, &V21764_285_57167, &P21764_284};	/*Property = CssCanvas*/
const Value V21764_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_286 = {S8, 0, &V21764_286_57170, &P21764_285};	/*Property = CssColor*/
const Value V21764_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_237 = {S9, 0, &V21764_237_55786, &P21764_286};	/*Property = CssColumn*/
const Value V21764_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_287 = {S10, 0, &V21764_287_57173, &P21764_237};	/*Property = CssFlexbox*/
const Value V21764_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_288 = {S11, 0, &V21764_288_57176, &P21764_287};	/*Property = CssFont*/
const Value V21764_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_289 = {S12, 0, &V21764_289_86899, &P21764_288};	/*Property = CssImages*/
const Value V21764_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_291 = {S13, 0, &V21764_291_57182, &P21764_289};	/*Property = CssMediaQueries*/
const Value V21764_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_292 = {S14, 0, &V21764_292_57185, &P21764_291};	/*Property = CssMinMax*/
const Value V21764_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_293 = {S15, 0, &V21764_293_57188, &P21764_292};	/*Property = CssOverflow*/
const Value V21764_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_294 = {S16, 0, &V21764_294_57191, &P21764_293};	/*Property = CssPosition*/
const Value V21764_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_295 = {S17, 0, &V21764_295_57194, &P21764_294};	/*Property = CssText*/
const Value V21764_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_297 = {S18, 0, &V21764_297_57197, &P21764_295};	/*Property = CssTransforms*/
const Value V21764_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_241 = {S19, 0, &V21764_241_55801, &P21764_297};	/*Property = CssTransitions*/
const Value V21764_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_298 = {S20, 0, &V21764_298_57208, &P21764_241};	/*Property = CssUI*/
const Value V21764_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_300 = {S21, 0, &V21764_300_57217, &P21764_298};	/*Property = DataSet*/
const Value V21764_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_301 = {S22, 0, &V21764_301_57220, &P21764_300};	/*Property = DataUrl*/
const Value V21764_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_302 = {S23, 0, &V21764_302_57223, &P21764_301};	/*Property = DeviceOrientation*/
const Value V21764_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_304 = {S24, 0, &V21764_304_57229, &P21764_302};	/*Property = FileReader*/
const Value V21764_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_305 = {S25, 0, &V21764_305_57232, &P21764_304};	/*Property = FileSaver*/
const Value V21764_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_306 = {S26, 0, &V21764_306_57238, &P21764_305};	/*Property = FileWriter*/
const Value V21764_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_307 = {S27, 0, &V21764_307_57241, &P21764_306};	/*Property = FormData*/
const Value V21764_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_309 = {S28, 0, &V21764_309_57247, &P21764_307};	/*Property = Fullscreen*/
const Value V21764_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_310 = {S29, 0, &V21764_310_57250, &P21764_309};	/*Property = GeoLocation*/
const Value V21764_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_312 = {S30, 0, &V21764_312_57256, &P21764_310};	/*Property = History*/
const Value V21764_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_314 = {S31, 0, &V21764_314_57262, &P21764_312};	/*Property = Html5*/
const Value V21764_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_313 = {S32, 0, &V21764_313_57259, &P21764_314};	/*Property = Html-Media-Capture*/
const Value V21764_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_315 = {S33, 0, &V21764_315_57265, &P21764_313};	/*Property = Iframe*/
const Value V21764_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_316 = {S34, 0, &V21764_316_57268, &P21764_315};	/*Property = IndexedDB*/
const Value V21764_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_317 = {S35, 0, &V21764_317_57271, &P21764_316};	/*Property = Json*/
const Value V21764_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21764_15 = {S37, 0, &V21764_15_54167, &P21764_317};	/*Property = LayoutEngine*/
const Value V21764_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_344 = {S38, 0, &V21764_344_57274, &P21764_15};	/*Property = Masking*/
const Value V21764_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_320 = {S39, 0, &V21764_320_57292, &P21764_344};	/*Property = PostMessage*/
const Value V21764_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_321 = {S40, 0, &V21764_321_57295, &P21764_320};	/*Property = Progress*/
const Value V21764_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_322 = {S41, 0, &V21764_322_57298, &P21764_321};	/*Property = Prompts*/
const Value V21764_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_323 = {S42, 0, &V21764_323_57301, &P21764_322};	/*Property = Selector*/
const Value V21764_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_324 = {S43, 0, &V21764_324_57307, &P21764_323};	/*Property = Svg*/
const Value V21764_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_327 = {S44, 0, &V21764_327_57316, &P21764_324};	/*Property = TouchEvents*/
const Value V21764_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_328 = {S45, 0, &V21764_328_57319, &P21764_327};	/*Property = Track*/
const Value V21764_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_329 = {S46, 0, &V21764_329_57325, &P21764_328};	/*Property = Video*/
const Value V21764_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_330 = {S47, 0, &V21764_330_57328, &P21764_329};	/*Property = Viewport*/
const Value V21764_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_332 = {S48, 0, &V21764_332_57340, &P21764_330};	/*Property = WebWorkers*/
const Value V21764_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21764_333 = {S49, 0, &V21764_333_57343, &P21764_332};	/*Property = Xhr2*/
const Profile P21764 = {"21764", &P21764_333};	/*Profile ID - 21764*/
const Value V22130_1_1 = {S2, NULL};	/*Value = True*/
const Property P22130_1 = {S50, 0, &V22130_1_1, NULL};	/*Property = IsMobile*/
const Value V22130_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22130_5 = {S52, 0, &V22130_5_1227, &P22130_1};	/*Property = ScreenPixelsHeight*/
const Value V22130_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22130_4 = {S54, 0, &V22130_4_1460, &P22130_5};	/*Property = ScreenPixelsWidth*/
const Profile P22130 = {"22130", &P22130_4};	/*Profile ID - 22130*/
const Value V19979_1_1 = {S2, NULL};	/*Value = True*/
const Property P19979_1 = {S50, 0, &V19979_1_1, NULL};	/*Property = IsMobile*/
const Value V19979_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19979_5 = {S52, 0, &V19979_5_1227, &P19979_1};	/*Property = ScreenPixelsHeight*/
const Value V19979_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19979_4 = {S54, 0, &V19979_4_1229, &P19979_5};	/*Property = ScreenPixelsWidth*/
const Profile P19979 = {"19979", &P19979_4};	/*Profile ID - 19979*/
const Value V21704_1_1 = {S2, NULL};	/*Value = True*/
const Property P21704_1 = {S50, 0, &V21704_1_1, NULL};	/*Property = IsMobile*/
const Value V21704_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P21704_5 = {S52, 0, &V21704_5_37569, &P21704_1};	/*Property = ScreenPixelsHeight*/
const Value V21704_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21704_4 = {S54, 0, &V21704_4_5645, &P21704_5};	/*Property = ScreenPixelsWidth*/
const Profile P21704 = {"21704", &P21704_4};	/*Profile ID - 21704*/
const Value V21355_1_1 = {S2, NULL};	/*Value = True*/
const Property P21355_1 = {S50, 0, &V21355_1_1, NULL};	/*Property = IsMobile*/
const Value V21355_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21355_5 = {S52, 0, &V21355_5_9981, &P21355_1};	/*Property = ScreenPixelsHeight*/
const Value V21355_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21355_4 = {S54, 0, &V21355_4_39098, &P21355_5};	/*Property = ScreenPixelsWidth*/
const Profile P21355 = {"21355", &P21355_4};	/*Profile ID - 21355*/
const Value V21310_1_1 = {S2, NULL};	/*Value = True*/
const Property P21310_1 = {S50, 0, &V21310_1_1, NULL};	/*Property = IsMobile*/
const Value V21310_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21310_5 = {S52, 0, &V21310_5_1227, &P21310_1};	/*Property = ScreenPixelsHeight*/
const Value V21310_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21310_4 = {S54, 0, &V21310_4_1229, &P21310_5};	/*Property = ScreenPixelsWidth*/
const Profile P21310 = {"21310", &P21310_4};	/*Profile ID - 21310*/
const Value V24010_1_1 = {S2, NULL};	/*Value = True*/
const Property P24010_1 = {S50, 0, &V24010_1_1, NULL};	/*Property = IsMobile*/
const Value V24010_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P24010_5 = {S52, 0, &V24010_5_1458, &P24010_1};	/*Property = ScreenPixelsHeight*/
const Value V24010_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P24010_4 = {S54, 0, &V24010_4_37617, &P24010_5};	/*Property = ScreenPixelsWidth*/
const Profile P24010 = {"24010", &P24010_4};	/*Profile ID - 24010*/
const Value V21105_1_1 = {S2, NULL};	/*Value = True*/
const Property P21105_1 = {S50, 0, &V21105_1_1, NULL};	/*Property = IsMobile*/
const Value V21105_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21105_5 = {S52, 0, &V21105_5_1403, &P21105_1};	/*Property = ScreenPixelsHeight*/
const Value V21105_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21105_4 = {S54, 0, &V21105_4_1404, &P21105_5};	/*Property = ScreenPixelsWidth*/
const Profile P21105 = {"21105", &P21105_4};	/*Profile ID - 21105*/
const Value V23426_1_1 = {S2, NULL};	/*Value = True*/
const Property P23426_1 = {S50, 0, &V23426_1_1, NULL};	/*Property = IsMobile*/
const Value V23426_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23426_5 = {S52, 0, &V23426_5_1277, &P23426_1};	/*Property = ScreenPixelsHeight*/
const Value V23426_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23426_4 = {S54, 0, &V23426_4_2336, &P23426_5};	/*Property = ScreenPixelsWidth*/
const Profile P23426 = {"23426", &P23426_4};	/*Profile ID - 23426*/
const Value V21491_1_1 = {S2, NULL};	/*Value = True*/
const Property P21491_1 = {S50, 0, &V21491_1_1, NULL};	/*Property = IsMobile*/
const Value V21491_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21491_5 = {S52, 0, &V21491_5_1277, &P21491_1};	/*Property = ScreenPixelsHeight*/
const Value V21491_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21491_4 = {S54, 0, &V21491_4_1279, &P21491_5};	/*Property = ScreenPixelsWidth*/
const Profile P21491 = {"21491", &P21491_4};	/*Profile ID - 21491*/
const Value V21061_1_1 = {S2, NULL};	/*Value = True*/
const Property P21061_1 = {S50, 0, &V21061_1_1, NULL};	/*Property = IsMobile*/
const Value V21061_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21061_5 = {S52, 0, &V21061_5_1269, &P21061_1};	/*Property = ScreenPixelsHeight*/
const Value V21061_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21061_4 = {S54, 0, &V21061_4_1229, &P21061_5};	/*Property = ScreenPixelsWidth*/
const Profile P21061 = {"21061", &P21061_4};	/*Profile ID - 21061*/
const Value V20151_1_1 = {S2, NULL};	/*Value = True*/
const Property P20151_1 = {S50, 0, &V20151_1_1, NULL};	/*Property = IsMobile*/
const Value V20151_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20151_5 = {S52, 0, &V20151_5_1403, &P20151_1};	/*Property = ScreenPixelsHeight*/
const Value V20151_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P20151_4 = {S54, 0, &V20151_4_8554, &P20151_5};	/*Property = ScreenPixelsWidth*/
const Profile P20151 = {"20151", &P20151_4};	/*Profile ID - 20151*/
const Profile P18597 = {"18597", NULL};	/*Profile ID - 18597*/
const Value V21114_1_1 = {S2, NULL};	/*Value = True*/
const Property P21114_1 = {S50, 0, &V21114_1_1, NULL};	/*Property = IsMobile*/
const Value V21114_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21114_5 = {S52, 0, &V21114_5_1403, &P21114_1};	/*Property = ScreenPixelsHeight*/
const Value V21114_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21114_4 = {S54, 0, &V21114_4_1404, &P21114_5};	/*Property = ScreenPixelsWidth*/
const Profile P21114 = {"21114", &P21114_4};	/*Profile ID - 21114*/
const Value V20974_1_1 = {S2, NULL};	/*Value = True*/
const Property P20974_1 = {S50, 0, &V20974_1_1, NULL};	/*Property = IsMobile*/
const Value V20974_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20974_5 = {S52, 0, &V20974_5_1227, &P20974_1};	/*Property = ScreenPixelsHeight*/
const Value V20974_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20974_4 = {S54, 0, &V20974_4_1229, &P20974_5};	/*Property = ScreenPixelsWidth*/
const Profile P20974 = {"20974", &P20974_4};	/*Profile ID - 20974*/
const Value V21130_1_1 = {S2, NULL};	/*Value = True*/
const Property P21130_1 = {S50, 0, &V21130_1_1, NULL};	/*Property = IsMobile*/
const Value V21130_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21130_5 = {S52, 0, &V21130_5_1227, &P21130_1};	/*Property = ScreenPixelsHeight*/
const Value V21130_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21130_4 = {S54, 0, &V21130_4_1229, &P21130_5};	/*Property = ScreenPixelsWidth*/
const Profile P21130 = {"21130", &P21130_4};	/*Profile ID - 21130*/
const Value V18512_1_1 = {S2, NULL};	/*Value = True*/
const Property P18512_1 = {S50, 0, &V18512_1_1, NULL};	/*Property = IsMobile*/
const Value V18512_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18512_5 = {S52, 0, &V18512_5_1403, &P18512_1};	/*Property = ScreenPixelsHeight*/
const Value V18512_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18512_4 = {S54, 0, &V18512_4_1404, &P18512_5};	/*Property = ScreenPixelsWidth*/
const Profile P18512 = {"18512", &P18512_4};	/*Profile ID - 18512*/
const Value V21946_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_279 = {S1, 0, &V21946_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21946_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_282 = {S3, 0, &V21946_282_57155, &P21946_279};	/*Property = BlobBuilder*/
const Value V21946_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_350 = {S4, 0, &V21946_350_57446, &P21946_282};	/*Property = Canvas*/
const Value V21946_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_283 = {S5, 0, &V21946_283_57161, &P21946_350};	/*Property = CssBackground*/
const Value V21946_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_284 = {S6, 0, &V21946_284_57164, &P21946_283};	/*Property = CssBorderImage*/
const Value V21946_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_285 = {S7, 0, &V21946_285_57167, &P21946_284};	/*Property = CssCanvas*/
const Value V21946_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_286 = {S8, 0, &V21946_286_57170, &P21946_285};	/*Property = CssColor*/
const Value V21946_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_237 = {S9, 0, &V21946_237_55786, &P21946_286};	/*Property = CssColumn*/
const Value V21946_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_287 = {S10, 0, &V21946_287_57173, &P21946_237};	/*Property = CssFlexbox*/
const Value V21946_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_288 = {S11, 0, &V21946_288_57176, &P21946_287};	/*Property = CssFont*/
const Value V21946_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_289 = {S12, 0, &V21946_289_86899, &P21946_288};	/*Property = CssImages*/
const Value V21946_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_291 = {S13, 0, &V21946_291_57182, &P21946_289};	/*Property = CssMediaQueries*/
const Value V21946_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_292 = {S14, 0, &V21946_292_57185, &P21946_291};	/*Property = CssMinMax*/
const Value V21946_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_293 = {S15, 0, &V21946_293_57188, &P21946_292};	/*Property = CssOverflow*/
const Value V21946_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_294 = {S16, 0, &V21946_294_57191, &P21946_293};	/*Property = CssPosition*/
const Value V21946_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_295 = {S17, 0, &V21946_295_57194, &P21946_294};	/*Property = CssText*/
const Value V21946_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_297 = {S18, 0, &V21946_297_57197, &P21946_295};	/*Property = CssTransforms*/
const Value V21946_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_241 = {S19, 0, &V21946_241_55801, &P21946_297};	/*Property = CssTransitions*/
const Value V21946_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_298 = {S20, 0, &V21946_298_57208, &P21946_241};	/*Property = CssUI*/
const Value V21946_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_300 = {S21, 0, &V21946_300_57217, &P21946_298};	/*Property = DataSet*/
const Value V21946_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_301 = {S22, 0, &V21946_301_57220, &P21946_300};	/*Property = DataUrl*/
const Value V21946_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_302 = {S23, 0, &V21946_302_57223, &P21946_301};	/*Property = DeviceOrientation*/
const Value V21946_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_304 = {S24, 0, &V21946_304_57229, &P21946_302};	/*Property = FileReader*/
const Value V21946_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_305 = {S25, 0, &V21946_305_57232, &P21946_304};	/*Property = FileSaver*/
const Value V21946_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_306 = {S26, 0, &V21946_306_57238, &P21946_305};	/*Property = FileWriter*/
const Value V21946_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_307 = {S27, 0, &V21946_307_57241, &P21946_306};	/*Property = FormData*/
const Value V21946_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_309 = {S28, 0, &V21946_309_57247, &P21946_307};	/*Property = Fullscreen*/
const Value V21946_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_310 = {S29, 0, &V21946_310_57250, &P21946_309};	/*Property = GeoLocation*/
const Value V21946_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_312 = {S30, 0, &V21946_312_57256, &P21946_310};	/*Property = History*/
const Value V21946_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_314 = {S31, 0, &V21946_314_57262, &P21946_312};	/*Property = Html5*/
const Value V21946_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_313 = {S32, 0, &V21946_313_57259, &P21946_314};	/*Property = Html-Media-Capture*/
const Value V21946_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_315 = {S33, 0, &V21946_315_57265, &P21946_313};	/*Property = Iframe*/
const Value V21946_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_316 = {S34, 0, &V21946_316_57268, &P21946_315};	/*Property = IndexedDB*/
const Value V21946_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_317 = {S35, 0, &V21946_317_57271, &P21946_316};	/*Property = Json*/
const Value V21946_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21946_15 = {S37, 0, &V21946_15_54167, &P21946_317};	/*Property = LayoutEngine*/
const Value V21946_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_344 = {S38, 0, &V21946_344_57274, &P21946_15};	/*Property = Masking*/
const Value V21946_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_320 = {S39, 0, &V21946_320_57292, &P21946_344};	/*Property = PostMessage*/
const Value V21946_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_321 = {S40, 0, &V21946_321_57295, &P21946_320};	/*Property = Progress*/
const Value V21946_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_322 = {S41, 0, &V21946_322_57298, &P21946_321};	/*Property = Prompts*/
const Value V21946_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_323 = {S42, 0, &V21946_323_57301, &P21946_322};	/*Property = Selector*/
const Value V21946_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_324 = {S43, 0, &V21946_324_57307, &P21946_323};	/*Property = Svg*/
const Value V21946_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_327 = {S44, 0, &V21946_327_57316, &P21946_324};	/*Property = TouchEvents*/
const Value V21946_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_328 = {S45, 0, &V21946_328_57319, &P21946_327};	/*Property = Track*/
const Value V21946_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_329 = {S46, 0, &V21946_329_57325, &P21946_328};	/*Property = Video*/
const Value V21946_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_330 = {S47, 0, &V21946_330_57328, &P21946_329};	/*Property = Viewport*/
const Value V21946_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_332 = {S48, 0, &V21946_332_57340, &P21946_330};	/*Property = WebWorkers*/
const Value V21946_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21946_333 = {S49, 0, &V21946_333_57343, &P21946_332};	/*Property = Xhr2*/
const Profile P21946 = {"21946", &P21946_333};	/*Profile ID - 21946*/
const Value V23989_1_1 = {S2, NULL};	/*Value = True*/
const Property P23989_1 = {S50, 0, &V23989_1_1, NULL};	/*Property = IsMobile*/
const Value V23989_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23989_5 = {S52, 0, &V23989_5_1227, &P23989_1};	/*Property = ScreenPixelsHeight*/
const Value V23989_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23989_4 = {S54, 0, &V23989_4_1229, &P23989_5};	/*Property = ScreenPixelsWidth*/
const Profile P23989 = {"23989", &P23989_4};	/*Profile ID - 23989*/
const Value V22102_1_1 = {S2, NULL};	/*Value = True*/
const Property P22102_1 = {S50, 0, &V22102_1_1, NULL};	/*Property = IsMobile*/
const Value V22102_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22102_5 = {S52, 0, &V22102_5_1227, &P22102_1};	/*Property = ScreenPixelsHeight*/
const Value V22102_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22102_4 = {S54, 0, &V22102_4_1229, &P22102_5};	/*Property = ScreenPixelsWidth*/
const Profile P22102 = {"22102", &P22102_4};	/*Profile ID - 22102*/
const Value V20347_1_1 = {S2, NULL};	/*Value = True*/
const Property P20347_1 = {S50, 0, &V20347_1_1, NULL};	/*Property = IsMobile*/
const Value V20347_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20347_5 = {S52, 0, &V20347_5_1227, &P20347_1};	/*Property = ScreenPixelsHeight*/
const Value V20347_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20347_4 = {S54, 0, &V20347_4_1229, &P20347_5};	/*Property = ScreenPixelsWidth*/
const Profile P20347 = {"20347", &P20347_4};	/*Profile ID - 20347*/
const Value V19751_1_1 = {S2, NULL};	/*Value = True*/
const Property P19751_1 = {S50, 0, &V19751_1_1, NULL};	/*Property = IsMobile*/
const Value V19751_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P19751_5 = {S52, 0, &V19751_5_1277, &P19751_1};	/*Property = ScreenPixelsHeight*/
const Value V19751_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P19751_4 = {S54, 0, &V19751_4_2336, &P19751_5};	/*Property = ScreenPixelsWidth*/
const Profile P19751 = {"19751", &P19751_4};	/*Profile ID - 19751*/
const Value V18746_1_1 = {S2, NULL};	/*Value = True*/
const Property P18746_1 = {S50, 0, &V18746_1_1, NULL};	/*Property = IsMobile*/
const Value V18746_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18746_5 = {S52, 0, &V18746_5_1227, &P18746_1};	/*Property = ScreenPixelsHeight*/
const Value V18746_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18746_4 = {S54, 0, &V18746_4_1229, &P18746_5};	/*Property = ScreenPixelsWidth*/
const Profile P18746 = {"18746", &P18746_4};	/*Profile ID - 18746*/
const Value V23773_1_1 = {S2, NULL};	/*Value = True*/
const Property P23773_1 = {S50, 0, &V23773_1_1, NULL};	/*Property = IsMobile*/
const Value V23773_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23773_5 = {S52, 0, &V23773_5_9981, &P23773_1};	/*Property = ScreenPixelsHeight*/
const Value V23773_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23773_4 = {S54, 0, &V23773_4_2336, &P23773_5};	/*Property = ScreenPixelsWidth*/
const Profile P23773 = {"23773", &P23773_4};	/*Profile ID - 23773*/
const Value V21778_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21778_279 = {S1, 0, &V21778_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21778_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21778_282 = {S3, 0, &V21778_282_57154, &P21778_279};	/*Property = BlobBuilder*/
const Value V21778_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21778_350 = {S4, 0, &V21778_350_57444, &P21778_282};	/*Property = Canvas*/
const Value V21778_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21778_283 = {S5, 0, &V21778_283_57159, &P21778_350};	/*Property = CssBackground*/
const Value V21778_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21778_284 = {S6, 0, &V21778_284_57163, &P21778_283};	/*Property = CssBorderImage*/
const Value V21778_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21778_285 = {S7, 0, &V21778_285_57166, &P21778_284};	/*Property = CssCanvas*/
const Value V21778_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21778_286 = {S8, 0, &V21778_286_57168, &P21778_285};	/*Property = CssColor*/
const Value V21778_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21778_237 = {S9, 0, &V21778_237_55786, &P21778_286};	/*Property = CssColumn*/
const Value V21778_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21778_287 = {S10, 0, &V21778_287_57172, &P21778_237};	/*Property = CssFlexbox*/
const Value V21778_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21778_288 = {S11, 0, &V21778_288_57175, &P21778_287};	/*Property = CssFont*/
const Value V21778_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21778_289 = {S12, 0, &V21778_289_57759, &P21778_288};	/*Property = CssImages*/
const Value V21778_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21778_291 = {S13, 0, &V21778_291_57181, &P21778_289};	/*Property = CssMediaQueries*/
const Value V21778_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21778_292 = {S14, 0, &V21778_292_57183, &P21778_291};	/*Property = CssMinMax*/
const Value V21778_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21778_293 = {S15, 0, &V21778_293_57187, &P21778_292};	/*Property = CssOverflow*/
const Value V21778_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21778_294 = {S16, 0, &V21778_294_57190, &P21778_293};	/*Property = CssPosition*/
const Value V21778_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21778_295 = {S17, 0, &V21778_295_57192, &P21778_294};	/*Property = CssText*/
const Value V21778_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21778_297 = {S18, 0, &V21778_297_57195, &P21778_295};	/*Property = CssTransforms*/
const Value V21778_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21778_241 = {S19, 0, &V21778_241_55799, &P21778_297};	/*Property = CssTransitions*/
const Value V21778_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21778_298 = {S20, 0, &V21778_298_57206, &P21778_241};	/*Property = CssUI*/
const Value V21778_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21778_300 = {S21, 0, &V21778_300_57216, &P21778_298};	/*Property = DataSet*/
const Value V21778_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21778_301 = {S22, 0, &V21778_301_57218, &P21778_300};	/*Property = DataUrl*/
const Value V21778_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21778_302 = {S23, 0, &V21778_302_57222, &P21778_301};	/*Property = DeviceOrientation*/
const Value V21778_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21778_304 = {S24, 0, &V21778_304_57228, &P21778_302};	/*Property = FileReader*/
const Value V21778_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21778_305 = {S25, 0, &V21778_305_57231, &P21778_304};	/*Property = FileSaver*/
const Value V21778_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21778_306 = {S26, 0, &V21778_306_57237, &P21778_305};	/*Property = FileWriter*/
const Value V21778_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21778_307 = {S27, 0, &V21778_307_57240, &P21778_306};	/*Property = FormData*/
const Value V21778_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21778_309 = {S28, 0, &V21778_309_57246, &P21778_307};	/*Property = Fullscreen*/
const Value V21778_310_57249 = {S0, NULL};	/*Value = False*/
const Property P21778_310 = {S29, 0, &V21778_310_57249, &P21778_309};	/*Property = GeoLocation*/
const Value V21778_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21778_312 = {S30, 0, &V21778_312_57255, &P21778_310};	/*Property = History*/
const Value V21778_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21778_314 = {S31, 0, &V21778_314_57261, &P21778_312};	/*Property = Html5*/
const Value V21778_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21778_313 = {S32, 0, &V21778_313_57258, &P21778_314};	/*Property = Html-Media-Capture*/
const Value V21778_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21778_315 = {S33, 0, &V21778_315_57264, &P21778_313};	/*Property = Iframe*/
const Value V21778_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21778_316 = {S34, 0, &V21778_316_57267, &P21778_315};	/*Property = IndexedDB*/
const Value V21778_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21778_317 = {S35, 0, &V21778_317_57269, &P21778_316};	/*Property = Json*/
const Value V21778_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21778_15 = {S37, 0, &V21778_15_54167, &P21778_317};	/*Property = LayoutEngine*/
const Value V21778_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21778_344 = {S38, 0, &V21778_344_57272, &P21778_15};	/*Property = Masking*/
const Value V21778_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21778_320 = {S39, 0, &V21778_320_57290, &P21778_344};	/*Property = PostMessage*/
const Value V21778_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21778_321 = {S40, 0, &V21778_321_57293, &P21778_320};	/*Property = Progress*/
const Value V21778_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21778_322 = {S41, 0, &V21778_322_57296, &P21778_321};	/*Property = Prompts*/
const Value V21778_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21778_323 = {S42, 0, &V21778_323_57299, &P21778_322};	/*Property = Selector*/
const Value V21778_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21778_324 = {S43, 0, &V21778_324_57306, &P21778_323};	/*Property = Svg*/
const Value V21778_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21778_327 = {S44, 0, &V21778_327_57315, &P21778_324};	/*Property = TouchEvents*/
const Value V21778_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21778_328 = {S45, 0, &V21778_328_57318, &P21778_327};	/*Property = Track*/
const Value V21778_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21778_329 = {S46, 0, &V21778_329_57325, &P21778_328};	/*Property = Video*/
const Value V21778_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21778_330 = {S47, 0, &V21778_330_57326, &P21778_329};	/*Property = Viewport*/
const Value V21778_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21778_332 = {S48, 0, &V21778_332_57339, &P21778_330};	/*Property = WebWorkers*/
const Value V21778_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21778_333 = {S49, 0, &V21778_333_57342, &P21778_332};	/*Property = Xhr2*/
const Profile P21778 = {"21778", &P21778_333};	/*Profile ID - 21778*/
const Value V24672_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_279 = {S1, 0, &V24672_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24672_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_282 = {S3, 0, &V24672_282_57155, &P24672_279};	/*Property = BlobBuilder*/
const Value V24672_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_350 = {S4, 0, &V24672_350_57446, &P24672_282};	/*Property = Canvas*/
const Value V24672_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_283 = {S5, 0, &V24672_283_57161, &P24672_350};	/*Property = CssBackground*/
const Value V24672_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_284 = {S6, 0, &V24672_284_57164, &P24672_283};	/*Property = CssBorderImage*/
const Value V24672_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_285 = {S7, 0, &V24672_285_57167, &P24672_284};	/*Property = CssCanvas*/
const Value V24672_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_286 = {S8, 0, &V24672_286_57170, &P24672_285};	/*Property = CssColor*/
const Value V24672_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_237 = {S9, 0, &V24672_237_55786, &P24672_286};	/*Property = CssColumn*/
const Value V24672_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_287 = {S10, 0, &V24672_287_57173, &P24672_237};	/*Property = CssFlexbox*/
const Value V24672_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_288 = {S11, 0, &V24672_288_57176, &P24672_287};	/*Property = CssFont*/
const Value V24672_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_289 = {S12, 0, &V24672_289_86899, &P24672_288};	/*Property = CssImages*/
const Value V24672_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_291 = {S13, 0, &V24672_291_57182, &P24672_289};	/*Property = CssMediaQueries*/
const Value V24672_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_292 = {S14, 0, &V24672_292_57185, &P24672_291};	/*Property = CssMinMax*/
const Value V24672_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_293 = {S15, 0, &V24672_293_57188, &P24672_292};	/*Property = CssOverflow*/
const Value V24672_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_294 = {S16, 0, &V24672_294_57191, &P24672_293};	/*Property = CssPosition*/
const Value V24672_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_295 = {S17, 0, &V24672_295_57194, &P24672_294};	/*Property = CssText*/
const Value V24672_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_297 = {S18, 0, &V24672_297_57197, &P24672_295};	/*Property = CssTransforms*/
const Value V24672_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_241 = {S19, 0, &V24672_241_55801, &P24672_297};	/*Property = CssTransitions*/
const Value V24672_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_298 = {S20, 0, &V24672_298_57208, &P24672_241};	/*Property = CssUI*/
const Value V24672_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_300 = {S21, 0, &V24672_300_57217, &P24672_298};	/*Property = DataSet*/
const Value V24672_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_301 = {S22, 0, &V24672_301_57220, &P24672_300};	/*Property = DataUrl*/
const Value V24672_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_302 = {S23, 0, &V24672_302_57223, &P24672_301};	/*Property = DeviceOrientation*/
const Value V24672_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_304 = {S24, 0, &V24672_304_57229, &P24672_302};	/*Property = FileReader*/
const Value V24672_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_305 = {S25, 0, &V24672_305_57232, &P24672_304};	/*Property = FileSaver*/
const Value V24672_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_306 = {S26, 0, &V24672_306_57238, &P24672_305};	/*Property = FileWriter*/
const Value V24672_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_307 = {S27, 0, &V24672_307_57241, &P24672_306};	/*Property = FormData*/
const Value V24672_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_309 = {S28, 0, &V24672_309_57247, &P24672_307};	/*Property = Fullscreen*/
const Value V24672_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_310 = {S29, 0, &V24672_310_57250, &P24672_309};	/*Property = GeoLocation*/
const Value V24672_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_312 = {S30, 0, &V24672_312_57256, &P24672_310};	/*Property = History*/
const Value V24672_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_314 = {S31, 0, &V24672_314_57262, &P24672_312};	/*Property = Html5*/
const Value V24672_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_313 = {S32, 0, &V24672_313_57259, &P24672_314};	/*Property = Html-Media-Capture*/
const Value V24672_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_315 = {S33, 0, &V24672_315_57265, &P24672_313};	/*Property = Iframe*/
const Value V24672_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_316 = {S34, 0, &V24672_316_57268, &P24672_315};	/*Property = IndexedDB*/
const Value V24672_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_317 = {S35, 0, &V24672_317_57271, &P24672_316};	/*Property = Json*/
const Value V24672_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24672_15 = {S37, 0, &V24672_15_90837, &P24672_317};	/*Property = LayoutEngine*/
const Value V24672_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_344 = {S38, 0, &V24672_344_57274, &P24672_15};	/*Property = Masking*/
const Value V24672_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_320 = {S39, 0, &V24672_320_57292, &P24672_344};	/*Property = PostMessage*/
const Value V24672_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_321 = {S40, 0, &V24672_321_57295, &P24672_320};	/*Property = Progress*/
const Value V24672_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_322 = {S41, 0, &V24672_322_57298, &P24672_321};	/*Property = Prompts*/
const Value V24672_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_323 = {S42, 0, &V24672_323_57301, &P24672_322};	/*Property = Selector*/
const Value V24672_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_324 = {S43, 0, &V24672_324_57307, &P24672_323};	/*Property = Svg*/
const Value V24672_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_327 = {S44, 0, &V24672_327_57316, &P24672_324};	/*Property = TouchEvents*/
const Value V24672_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_328 = {S45, 0, &V24672_328_57319, &P24672_327};	/*Property = Track*/
const Value V24672_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_329 = {S46, 0, &V24672_329_57325, &P24672_328};	/*Property = Video*/
const Value V24672_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_330 = {S47, 0, &V24672_330_57328, &P24672_329};	/*Property = Viewport*/
const Value V24672_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_332 = {S48, 0, &V24672_332_57340, &P24672_330};	/*Property = WebWorkers*/
const Value V24672_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24672_333 = {S49, 0, &V24672_333_57343, &P24672_332};	/*Property = Xhr2*/
const Profile P24672 = {"24672", &P24672_333};	/*Profile ID - 24672*/
const Value V21083_1_1 = {S2, NULL};	/*Value = True*/
const Property P21083_1 = {S50, 0, &V21083_1_1, NULL};	/*Property = IsMobile*/
const Value V21083_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21083_5 = {S52, 0, &V21083_5_1269, &P21083_1};	/*Property = ScreenPixelsHeight*/
const Value V21083_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21083_4 = {S54, 0, &V21083_4_1229, &P21083_5};	/*Property = ScreenPixelsWidth*/
const Profile P21083 = {"21083", &P21083_4};	/*Profile ID - 21083*/
const Value V24632_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24632_279 = {S1, 0, &V24632_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24632_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24632_282 = {S3, 0, &V24632_282_57154, &P24632_279};	/*Property = BlobBuilder*/
const Value V24632_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24632_350 = {S4, 0, &V24632_350_57444, &P24632_282};	/*Property = Canvas*/
const Value V24632_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24632_283 = {S5, 0, &V24632_283_57159, &P24632_350};	/*Property = CssBackground*/
const Value V24632_284_57163 = {S0, NULL};	/*Value = False*/
const Property P24632_284 = {S6, 0, &V24632_284_57163, &P24632_283};	/*Property = CssBorderImage*/
const Value V24632_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24632_285 = {S7, 0, &V24632_285_57166, &P24632_284};	/*Property = CssCanvas*/
const Value V24632_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24632_286 = {S8, 0, &V24632_286_57168, &P24632_285};	/*Property = CssColor*/
const Value V24632_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24632_237 = {S9, 0, &V24632_237_55786, &P24632_286};	/*Property = CssColumn*/
const Value V24632_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24632_287 = {S10, 0, &V24632_287_57172, &P24632_237};	/*Property = CssFlexbox*/
const Value V24632_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24632_288 = {S11, 0, &V24632_288_57176, &P24632_287};	/*Property = CssFont*/
const Value V24632_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24632_289 = {S12, 0, &V24632_289_57759, &P24632_288};	/*Property = CssImages*/
const Value V24632_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24632_291 = {S13, 0, &V24632_291_57182, &P24632_289};	/*Property = CssMediaQueries*/
const Value V24632_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24632_292 = {S14, 0, &V24632_292_57183, &P24632_291};	/*Property = CssMinMax*/
const Value V24632_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24632_293 = {S15, 0, &V24632_293_57188, &P24632_292};	/*Property = CssOverflow*/
const Value V24632_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24632_294 = {S16, 0, &V24632_294_57191, &P24632_293};	/*Property = CssPosition*/
const Value V24632_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24632_295 = {S17, 0, &V24632_295_57192, &P24632_294};	/*Property = CssText*/
const Value V24632_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24632_297 = {S18, 0, &V24632_297_57195, &P24632_295};	/*Property = CssTransforms*/
const Value V24632_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24632_241 = {S19, 0, &V24632_241_55799, &P24632_297};	/*Property = CssTransitions*/
const Value V24632_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24632_298 = {S20, 0, &V24632_298_57206, &P24632_241};	/*Property = CssUI*/
const Value V24632_300_57216 = {S0, NULL};	/*Value = False*/
const Property P24632_300 = {S21, 0, &V24632_300_57216, &P24632_298};	/*Property = DataSet*/
const Value V24632_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24632_301 = {S22, 0, &V24632_301_57218, &P24632_300};	/*Property = DataUrl*/
const Value V24632_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24632_302 = {S23, 0, &V24632_302_57223, &P24632_301};	/*Property = DeviceOrientation*/
const Value V24632_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24632_304 = {S24, 0, &V24632_304_57229, &P24632_302};	/*Property = FileReader*/
const Value V24632_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24632_305 = {S25, 0, &V24632_305_57231, &P24632_304};	/*Property = FileSaver*/
const Value V24632_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24632_306 = {S26, 0, &V24632_306_57237, &P24632_305};	/*Property = FileWriter*/
const Value V24632_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24632_307 = {S27, 0, &V24632_307_57241, &P24632_306};	/*Property = FormData*/
const Value V24632_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24632_309 = {S28, 0, &V24632_309_57246, &P24632_307};	/*Property = Fullscreen*/
const Value V24632_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24632_310 = {S29, 0, &V24632_310_57248, &P24632_309};	/*Property = GeoLocation*/
const Value V24632_312_57255 = {S0, NULL};	/*Value = False*/
const Property P24632_312 = {S30, 0, &V24632_312_57255, &P24632_310};	/*Property = History*/
const Value V24632_314_57261 = {S0, NULL};	/*Value = False*/
const Property P24632_314 = {S31, 0, &V24632_314_57261, &P24632_312};	/*Property = Html5*/
const Value V24632_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24632_313 = {S32, 0, &V24632_313_57258, &P24632_314};	/*Property = Html-Media-Capture*/
const Value V24632_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24632_315 = {S33, 0, &V24632_315_57264, &P24632_313};	/*Property = Iframe*/
const Value V24632_316_57267 = {S0, NULL};	/*Value = False*/
const Property P24632_316 = {S34, 0, &V24632_316_57267, &P24632_315};	/*Property = IndexedDB*/
const Value V24632_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24632_317 = {S35, 0, &V24632_317_57269, &P24632_316};	/*Property = Json*/
const Value V24632_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24632_15 = {S37, 0, &V24632_15_90837, &P24632_317};	/*Property = LayoutEngine*/
const Value V24632_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24632_344 = {S38, 0, &V24632_344_57272, &P24632_15};	/*Property = Masking*/
const Value V24632_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24632_320 = {S39, 0, &V24632_320_57290, &P24632_344};	/*Property = PostMessage*/
const Value V24632_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24632_321 = {S40, 0, &V24632_321_57293, &P24632_320};	/*Property = Progress*/
const Value V24632_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24632_322 = {S41, 0, &V24632_322_57296, &P24632_321};	/*Property = Prompts*/
const Value V24632_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24632_323 = {S42, 0, &V24632_323_57299, &P24632_322};	/*Property = Selector*/
const Value V24632_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24632_324 = {S43, 0, &V24632_324_57306, &P24632_323};	/*Property = Svg*/
const Value V24632_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24632_327 = {S44, 0, &V24632_327_57316, &P24632_324};	/*Property = TouchEvents*/
const Value V24632_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24632_328 = {S45, 0, &V24632_328_57318, &P24632_327};	/*Property = Track*/
const Value V24632_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24632_329 = {S46, 0, &V24632_329_57323, &P24632_328};	/*Property = Video*/
const Value V24632_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24632_330 = {S47, 0, &V24632_330_57326, &P24632_329};	/*Property = Viewport*/
const Value V24632_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24632_332 = {S48, 0, &V24632_332_57339, &P24632_330};	/*Property = WebWorkers*/
const Value V24632_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24632_333 = {S49, 0, &V24632_333_57342, &P24632_332};	/*Property = Xhr2*/
const Profile P24632 = {"24632", &P24632_333};	/*Profile ID - 24632*/
const Value V21125_1_1 = {S2, NULL};	/*Value = True*/
const Property P21125_1 = {S50, 0, &V21125_1_1, NULL};	/*Property = IsMobile*/
const Value V21125_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21125_5 = {S52, 0, &V21125_5_1458, &P21125_1};	/*Property = ScreenPixelsHeight*/
const Value V21125_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21125_4 = {S54, 0, &V21125_4_37617, &P21125_5};	/*Property = ScreenPixelsWidth*/
const Profile P21125 = {"21125", &P21125_4};	/*Profile ID - 21125*/
const Value V20124_1_1 = {S2, NULL};	/*Value = True*/
const Property P20124_1 = {S50, 0, &V20124_1_1, NULL};	/*Property = IsMobile*/
const Value V20124_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20124_5 = {S52, 0, &V20124_5_1403, &P20124_1};	/*Property = ScreenPixelsHeight*/
const Value V20124_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20124_4 = {S54, 0, &V20124_4_1404, &P20124_5};	/*Property = ScreenPixelsWidth*/
const Profile P20124 = {"20124", &P20124_4};	/*Profile ID - 20124*/
const Value V18683_1_1 = {S2, NULL};	/*Value = True*/
const Property P18683_1 = {S50, 0, &V18683_1_1, NULL};	/*Property = IsMobile*/
const Value V18683_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18683_5 = {S52, 0, &V18683_5_9981, &P18683_1};	/*Property = ScreenPixelsHeight*/
const Value V18683_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18683_4 = {S54, 0, &V18683_4_39098, &P18683_5};	/*Property = ScreenPixelsWidth*/
const Profile P18683 = {"18683", &P18683_4};	/*Profile ID - 18683*/
const Value V18075_1_1 = {S2, NULL};	/*Value = True*/
const Property P18075_1 = {S50, 0, &V18075_1_1, NULL};	/*Property = IsMobile*/
const Value V18075_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18075_5 = {S52, 0, &V18075_5_1277, &P18075_1};	/*Property = ScreenPixelsHeight*/
const Value V18075_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18075_4 = {S54, 0, &V18075_4_1279, &P18075_5};	/*Property = ScreenPixelsWidth*/
const Profile P18075 = {"18075", &P18075_4};	/*Profile ID - 18075*/
const Value V21805_1_1 = {S2, NULL};	/*Value = True*/
const Property P21805_1 = {S50, 0, &V21805_1_1, NULL};	/*Property = IsMobile*/
const Value V21805_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21805_5 = {S52, 0, &V21805_5_1277, &P21805_1};	/*Property = ScreenPixelsHeight*/
const Value V21805_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21805_4 = {S54, 0, &V21805_4_2336, &P21805_5};	/*Property = ScreenPixelsWidth*/
const Profile P21805 = {"21805", &P21805_4};	/*Profile ID - 21805*/
const Value V18238_1_1 = {S2, NULL};	/*Value = True*/
const Property P18238_1 = {S50, 0, &V18238_1_1, NULL};	/*Property = IsMobile*/
const Value V18238_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18238_5 = {S52, 0, &V18238_5_1227, &P18238_1};	/*Property = ScreenPixelsHeight*/
const Value V18238_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18238_4 = {S54, 0, &V18238_4_1229, &P18238_5};	/*Property = ScreenPixelsWidth*/
const Profile P18238 = {"18238", &P18238_4};	/*Profile ID - 18238*/
const Value V21009_1_1 = {S2, NULL};	/*Value = True*/
const Property P21009_1 = {S50, 0, &V21009_1_1, NULL};	/*Property = IsMobile*/
const Value V21009_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21009_5 = {S52, 0, &V21009_5_9981, &P21009_1};	/*Property = ScreenPixelsHeight*/
const Value V21009_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21009_4 = {S54, 0, &V21009_4_39098, &P21009_5};	/*Property = ScreenPixelsWidth*/
const Profile P21009 = {"21009", &P21009_4};	/*Profile ID - 21009*/
const Value V20910_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_279 = {S1, 0, &V20910_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20910_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_282 = {S3, 0, &V20910_282_57155, &P20910_279};	/*Property = BlobBuilder*/
const Value V20910_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_350 = {S4, 0, &V20910_350_57446, &P20910_282};	/*Property = Canvas*/
const Value V20910_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_283 = {S5, 0, &V20910_283_57161, &P20910_350};	/*Property = CssBackground*/
const Value V20910_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_284 = {S6, 0, &V20910_284_57164, &P20910_283};	/*Property = CssBorderImage*/
const Value V20910_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_285 = {S7, 0, &V20910_285_57167, &P20910_284};	/*Property = CssCanvas*/
const Value V20910_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_286 = {S8, 0, &V20910_286_57170, &P20910_285};	/*Property = CssColor*/
const Value V20910_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_237 = {S9, 0, &V20910_237_55786, &P20910_286};	/*Property = CssColumn*/
const Value V20910_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_287 = {S10, 0, &V20910_287_57173, &P20910_237};	/*Property = CssFlexbox*/
const Value V20910_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_288 = {S11, 0, &V20910_288_57176, &P20910_287};	/*Property = CssFont*/
const Value V20910_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_289 = {S12, 0, &V20910_289_86899, &P20910_288};	/*Property = CssImages*/
const Value V20910_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_291 = {S13, 0, &V20910_291_57182, &P20910_289};	/*Property = CssMediaQueries*/
const Value V20910_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_292 = {S14, 0, &V20910_292_57185, &P20910_291};	/*Property = CssMinMax*/
const Value V20910_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_293 = {S15, 0, &V20910_293_57188, &P20910_292};	/*Property = CssOverflow*/
const Value V20910_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_294 = {S16, 0, &V20910_294_57191, &P20910_293};	/*Property = CssPosition*/
const Value V20910_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_295 = {S17, 0, &V20910_295_57194, &P20910_294};	/*Property = CssText*/
const Value V20910_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_297 = {S18, 0, &V20910_297_57197, &P20910_295};	/*Property = CssTransforms*/
const Value V20910_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_241 = {S19, 0, &V20910_241_55801, &P20910_297};	/*Property = CssTransitions*/
const Value V20910_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_298 = {S20, 0, &V20910_298_57208, &P20910_241};	/*Property = CssUI*/
const Value V20910_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_300 = {S21, 0, &V20910_300_57217, &P20910_298};	/*Property = DataSet*/
const Value V20910_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_301 = {S22, 0, &V20910_301_57220, &P20910_300};	/*Property = DataUrl*/
const Value V20910_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_302 = {S23, 0, &V20910_302_57223, &P20910_301};	/*Property = DeviceOrientation*/
const Value V20910_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_304 = {S24, 0, &V20910_304_57229, &P20910_302};	/*Property = FileReader*/
const Value V20910_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_305 = {S25, 0, &V20910_305_57232, &P20910_304};	/*Property = FileSaver*/
const Value V20910_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_306 = {S26, 0, &V20910_306_57238, &P20910_305};	/*Property = FileWriter*/
const Value V20910_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_307 = {S27, 0, &V20910_307_57241, &P20910_306};	/*Property = FormData*/
const Value V20910_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_309 = {S28, 0, &V20910_309_57247, &P20910_307};	/*Property = Fullscreen*/
const Value V20910_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_310 = {S29, 0, &V20910_310_57250, &P20910_309};	/*Property = GeoLocation*/
const Value V20910_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_312 = {S30, 0, &V20910_312_57256, &P20910_310};	/*Property = History*/
const Value V20910_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_314 = {S31, 0, &V20910_314_57262, &P20910_312};	/*Property = Html5*/
const Value V20910_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_313 = {S32, 0, &V20910_313_57259, &P20910_314};	/*Property = Html-Media-Capture*/
const Value V20910_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_315 = {S33, 0, &V20910_315_57265, &P20910_313};	/*Property = Iframe*/
const Value V20910_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_316 = {S34, 0, &V20910_316_57268, &P20910_315};	/*Property = IndexedDB*/
const Value V20910_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_317 = {S35, 0, &V20910_317_57271, &P20910_316};	/*Property = Json*/
const Value V20910_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P20910_15 = {S37, 0, &V20910_15_28, &P20910_317};	/*Property = LayoutEngine*/
const Value V20910_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_344 = {S38, 0, &V20910_344_57274, &P20910_15};	/*Property = Masking*/
const Value V20910_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_320 = {S39, 0, &V20910_320_57292, &P20910_344};	/*Property = PostMessage*/
const Value V20910_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_321 = {S40, 0, &V20910_321_57295, &P20910_320};	/*Property = Progress*/
const Value V20910_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_322 = {S41, 0, &V20910_322_57298, &P20910_321};	/*Property = Prompts*/
const Value V20910_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_323 = {S42, 0, &V20910_323_57301, &P20910_322};	/*Property = Selector*/
const Value V20910_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_324 = {S43, 0, &V20910_324_57307, &P20910_323};	/*Property = Svg*/
const Value V20910_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_327 = {S44, 0, &V20910_327_57316, &P20910_324};	/*Property = TouchEvents*/
const Value V20910_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_328 = {S45, 0, &V20910_328_57319, &P20910_327};	/*Property = Track*/
const Value V20910_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_329 = {S46, 0, &V20910_329_57325, &P20910_328};	/*Property = Video*/
const Value V20910_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_330 = {S47, 0, &V20910_330_57328, &P20910_329};	/*Property = Viewport*/
const Value V20910_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_332 = {S48, 0, &V20910_332_57340, &P20910_330};	/*Property = WebWorkers*/
const Value V20910_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20910_333 = {S49, 0, &V20910_333_57343, &P20910_332};	/*Property = Xhr2*/
const Profile P20910 = {"20910", &P20910_333};	/*Profile ID - 20910*/
const Value V22029_1_1 = {S2, NULL};	/*Value = True*/
const Property P22029_1 = {S50, 0, &V22029_1_1, NULL};	/*Property = IsMobile*/
const Value V22029_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22029_5 = {S52, 0, &V22029_5_1277, &P22029_1};	/*Property = ScreenPixelsHeight*/
const Value V22029_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22029_4 = {S54, 0, &V22029_4_1279, &P22029_5};	/*Property = ScreenPixelsWidth*/
const Profile P22029 = {"22029", &P22029_4};	/*Profile ID - 22029*/
const Value V20755_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_279 = {S1, 0, &V20755_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20755_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_282 = {S3, 0, &V20755_282_57155, &P20755_279};	/*Property = BlobBuilder*/
const Value V20755_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_350 = {S4, 0, &V20755_350_57446, &P20755_282};	/*Property = Canvas*/
const Value V20755_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_283 = {S5, 0, &V20755_283_57161, &P20755_350};	/*Property = CssBackground*/
const Value V20755_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_284 = {S6, 0, &V20755_284_57164, &P20755_283};	/*Property = CssBorderImage*/
const Value V20755_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_285 = {S7, 0, &V20755_285_57167, &P20755_284};	/*Property = CssCanvas*/
const Value V20755_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_286 = {S8, 0, &V20755_286_57170, &P20755_285};	/*Property = CssColor*/
const Value V20755_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_237 = {S9, 0, &V20755_237_55786, &P20755_286};	/*Property = CssColumn*/
const Value V20755_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_287 = {S10, 0, &V20755_287_57173, &P20755_237};	/*Property = CssFlexbox*/
const Value V20755_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_288 = {S11, 0, &V20755_288_57176, &P20755_287};	/*Property = CssFont*/
const Value V20755_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_289 = {S12, 0, &V20755_289_86899, &P20755_288};	/*Property = CssImages*/
const Value V20755_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_291 = {S13, 0, &V20755_291_57182, &P20755_289};	/*Property = CssMediaQueries*/
const Value V20755_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_292 = {S14, 0, &V20755_292_57185, &P20755_291};	/*Property = CssMinMax*/
const Value V20755_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_293 = {S15, 0, &V20755_293_57188, &P20755_292};	/*Property = CssOverflow*/
const Value V20755_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_294 = {S16, 0, &V20755_294_57191, &P20755_293};	/*Property = CssPosition*/
const Value V20755_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_295 = {S17, 0, &V20755_295_57194, &P20755_294};	/*Property = CssText*/
const Value V20755_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_297 = {S18, 0, &V20755_297_57197, &P20755_295};	/*Property = CssTransforms*/
const Value V20755_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_241 = {S19, 0, &V20755_241_55801, &P20755_297};	/*Property = CssTransitions*/
const Value V20755_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_298 = {S20, 0, &V20755_298_57208, &P20755_241};	/*Property = CssUI*/
const Value V20755_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_300 = {S21, 0, &V20755_300_57217, &P20755_298};	/*Property = DataSet*/
const Value V20755_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_301 = {S22, 0, &V20755_301_57220, &P20755_300};	/*Property = DataUrl*/
const Value V20755_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_302 = {S23, 0, &V20755_302_57223, &P20755_301};	/*Property = DeviceOrientation*/
const Value V20755_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_304 = {S24, 0, &V20755_304_57229, &P20755_302};	/*Property = FileReader*/
const Value V20755_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_305 = {S25, 0, &V20755_305_57232, &P20755_304};	/*Property = FileSaver*/
const Value V20755_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_306 = {S26, 0, &V20755_306_57238, &P20755_305};	/*Property = FileWriter*/
const Value V20755_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_307 = {S27, 0, &V20755_307_57241, &P20755_306};	/*Property = FormData*/
const Value V20755_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_309 = {S28, 0, &V20755_309_57247, &P20755_307};	/*Property = Fullscreen*/
const Value V20755_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_310 = {S29, 0, &V20755_310_57250, &P20755_309};	/*Property = GeoLocation*/
const Value V20755_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_312 = {S30, 0, &V20755_312_57256, &P20755_310};	/*Property = History*/
const Value V20755_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_314 = {S31, 0, &V20755_314_57262, &P20755_312};	/*Property = Html5*/
const Value V20755_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_313 = {S32, 0, &V20755_313_57259, &P20755_314};	/*Property = Html-Media-Capture*/
const Value V20755_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_315 = {S33, 0, &V20755_315_57265, &P20755_313};	/*Property = Iframe*/
const Value V20755_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_316 = {S34, 0, &V20755_316_57268, &P20755_315};	/*Property = IndexedDB*/
const Value V20755_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_317 = {S35, 0, &V20755_317_57271, &P20755_316};	/*Property = Json*/
const Value V20755_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20755_15 = {S37, 0, &V20755_15_29, &P20755_317};	/*Property = LayoutEngine*/
const Value V20755_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_344 = {S38, 0, &V20755_344_57274, &P20755_15};	/*Property = Masking*/
const Value V20755_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_320 = {S39, 0, &V20755_320_57292, &P20755_344};	/*Property = PostMessage*/
const Value V20755_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_321 = {S40, 0, &V20755_321_57295, &P20755_320};	/*Property = Progress*/
const Value V20755_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_322 = {S41, 0, &V20755_322_57298, &P20755_321};	/*Property = Prompts*/
const Value V20755_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_323 = {S42, 0, &V20755_323_57301, &P20755_322};	/*Property = Selector*/
const Value V20755_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_324 = {S43, 0, &V20755_324_57307, &P20755_323};	/*Property = Svg*/
const Value V20755_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_327 = {S44, 0, &V20755_327_57316, &P20755_324};	/*Property = TouchEvents*/
const Value V20755_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_328 = {S45, 0, &V20755_328_57319, &P20755_327};	/*Property = Track*/
const Value V20755_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_329 = {S46, 0, &V20755_329_57325, &P20755_328};	/*Property = Video*/
const Value V20755_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_330 = {S47, 0, &V20755_330_57328, &P20755_329};	/*Property = Viewport*/
const Value V20755_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_332 = {S48, 0, &V20755_332_57340, &P20755_330};	/*Property = WebWorkers*/
const Value V20755_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20755_333 = {S49, 0, &V20755_333_57343, &P20755_332};	/*Property = Xhr2*/
const Profile P20755 = {"20755", &P20755_333};	/*Profile ID - 20755*/
const Value V21093_1_1 = {S2, NULL};	/*Value = True*/
const Property P21093_1 = {S50, 0, &V21093_1_1, NULL};	/*Property = IsMobile*/
const Value V21093_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21093_5 = {S52, 0, &V21093_5_1227, &P21093_1};	/*Property = ScreenPixelsHeight*/
const Value V21093_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21093_4 = {S54, 0, &V21093_4_1229, &P21093_5};	/*Property = ScreenPixelsWidth*/
const Profile P21093 = {"21093", &P21093_4};	/*Profile ID - 21093*/
const Value V23931_1_1 = {S2, NULL};	/*Value = True*/
const Property P23931_1 = {S50, 0, &V23931_1_1, NULL};	/*Property = IsMobile*/
const Value V23931_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P23931_5 = {S52, 0, &V23931_5_1304, &P23931_1};	/*Property = ScreenPixelsHeight*/
const Value V23931_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23931_4 = {S54, 0, &V23931_4_2336, &P23931_5};	/*Property = ScreenPixelsWidth*/
const Profile P23931 = {"23931", &P23931_4};	/*Profile ID - 23931*/
const Value V20408_1_1 = {S2, NULL};	/*Value = True*/
const Property P20408_1 = {S50, 0, &V20408_1_1, NULL};	/*Property = IsMobile*/
const Value V20408_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20408_5 = {S52, 0, &V20408_5_1277, &P20408_1};	/*Property = ScreenPixelsHeight*/
const Value V20408_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20408_4 = {S54, 0, &V20408_4_1279, &P20408_5};	/*Property = ScreenPixelsWidth*/
const Profile P20408 = {"20408", &P20408_4};	/*Profile ID - 20408*/
const Value V20453_1_1 = {S2, NULL};	/*Value = True*/
const Property P20453_1 = {S50, 0, &V20453_1_1, NULL};	/*Property = IsMobile*/
const Value V20453_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20453_5 = {S52, 0, &V20453_5_1458, &P20453_1};	/*Property = ScreenPixelsHeight*/
const Value V20453_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20453_4 = {S54, 0, &V20453_4_1460, &P20453_5};	/*Property = ScreenPixelsWidth*/
const Profile P20453 = {"20453", &P20453_4};	/*Profile ID - 20453*/
const Value V22021_1_1 = {S2, NULL};	/*Value = True*/
const Property P22021_1 = {S50, 0, &V22021_1_1, NULL};	/*Property = IsMobile*/
const Value V22021_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22021_5 = {S52, 0, &V22021_5_1227, &P22021_1};	/*Property = ScreenPixelsHeight*/
const Value V22021_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22021_4 = {S54, 0, &V22021_4_1229, &P22021_5};	/*Property = ScreenPixelsWidth*/
const Profile P22021 = {"22021", &P22021_4};	/*Profile ID - 22021*/
const Value V18372_1_1 = {S2, NULL};	/*Value = True*/
const Property P18372_1 = {S50, 0, &V18372_1_1, NULL};	/*Property = IsMobile*/
const Value V18372_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18372_5 = {S52, 0, &V18372_5_9981, &P18372_1};	/*Property = ScreenPixelsHeight*/
const Value V18372_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18372_4 = {S54, 0, &V18372_4_39098, &P18372_5};	/*Property = ScreenPixelsWidth*/
const Profile P18372 = {"18372", &P18372_4};	/*Profile ID - 18372*/
const Value V21054_1_1 = {S2, NULL};	/*Value = True*/
const Property P21054_1 = {S50, 0, &V21054_1_1, NULL};	/*Property = IsMobile*/
const Value V21054_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21054_5 = {S52, 0, &V21054_5_1277, &P21054_1};	/*Property = ScreenPixelsHeight*/
const Value V21054_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21054_4 = {S54, 0, &V21054_4_1279, &P21054_5};	/*Property = ScreenPixelsWidth*/
const Profile P21054 = {"21054", &P21054_4};	/*Profile ID - 21054*/
const Value V18471_1_1 = {S2, NULL};	/*Value = True*/
const Property P18471_1 = {S50, 0, &V18471_1_1, NULL};	/*Property = IsMobile*/
const Value V18471_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18471_5 = {S52, 0, &V18471_5_1403, &P18471_1};	/*Property = ScreenPixelsHeight*/
const Value V18471_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18471_4 = {S54, 0, &V18471_4_1404, &P18471_5};	/*Property = ScreenPixelsWidth*/
const Profile P18471 = {"18471", &P18471_4};	/*Profile ID - 18471*/
const Value V21992_1_1 = {S2, NULL};	/*Value = True*/
const Property P21992_1 = {S50, 0, &V21992_1_1, NULL};	/*Property = IsMobile*/
const Value V21992_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21992_5 = {S52, 0, &V21992_5_1227, &P21992_1};	/*Property = ScreenPixelsHeight*/
const Value V21992_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21992_4 = {S54, 0, &V21992_4_1229, &P21992_5};	/*Property = ScreenPixelsWidth*/
const Profile P21992 = {"21992", &P21992_4};	/*Profile ID - 21992*/
const Value V19904_1_1 = {S2, NULL};	/*Value = True*/
const Property P19904_1 = {S50, 0, &V19904_1_1, NULL};	/*Property = IsMobile*/
const Value V19904_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P19904_5 = {S52, 0, &V19904_5_1277, &P19904_1};	/*Property = ScreenPixelsHeight*/
const Value V19904_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P19904_4 = {S54, 0, &V19904_4_1279, &P19904_5};	/*Property = ScreenPixelsWidth*/
const Profile P19904 = {"19904", &P19904_4};	/*Profile ID - 19904*/
const Value V21879_1_1 = {S2, NULL};	/*Value = True*/
const Property P21879_1 = {S50, 0, &V21879_1_1, NULL};	/*Property = IsMobile*/
const Value V21879_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21879_5 = {S52, 0, &V21879_5_1227, &P21879_1};	/*Property = ScreenPixelsHeight*/
const Value V21879_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21879_4 = {S54, 0, &V21879_4_1229, &P21879_5};	/*Property = ScreenPixelsWidth*/
const Profile P21879 = {"21879", &P21879_4};	/*Profile ID - 21879*/
const Value V23896_1_1 = {S2, NULL};	/*Value = True*/
const Property P23896_1 = {S50, 0, &V23896_1_1, NULL};	/*Property = IsMobile*/
const Value V23896_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23896_5 = {S52, 0, &V23896_5_9981, &P23896_1};	/*Property = ScreenPixelsHeight*/
const Value V23896_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23896_4 = {S54, 0, &V23896_4_39098, &P23896_5};	/*Property = ScreenPixelsWidth*/
const Profile P23896 = {"23896", &P23896_4};	/*Profile ID - 23896*/
const Value V20586_1_1 = {S2, NULL};	/*Value = True*/
const Property P20586_1 = {S50, 0, &V20586_1_1, NULL};	/*Property = IsMobile*/
const Value V20586_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20586_5 = {S52, 0, &V20586_5_1227, &P20586_1};	/*Property = ScreenPixelsHeight*/
const Value V20586_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20586_4 = {S54, 0, &V20586_4_1460, &P20586_5};	/*Property = ScreenPixelsWidth*/
const Profile P20586 = {"20586", &P20586_4};	/*Profile ID - 20586*/
const Value V21571_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_279 = {S1, 0, &V21571_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21571_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_282 = {S3, 0, &V21571_282_57155, &P21571_279};	/*Property = BlobBuilder*/
const Value V21571_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_350 = {S4, 0, &V21571_350_57446, &P21571_282};	/*Property = Canvas*/
const Value V21571_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_283 = {S5, 0, &V21571_283_57161, &P21571_350};	/*Property = CssBackground*/
const Value V21571_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_284 = {S6, 0, &V21571_284_57164, &P21571_283};	/*Property = CssBorderImage*/
const Value V21571_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_285 = {S7, 0, &V21571_285_57167, &P21571_284};	/*Property = CssCanvas*/
const Value V21571_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_286 = {S8, 0, &V21571_286_57170, &P21571_285};	/*Property = CssColor*/
const Value V21571_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_237 = {S9, 0, &V21571_237_55786, &P21571_286};	/*Property = CssColumn*/
const Value V21571_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_287 = {S10, 0, &V21571_287_57173, &P21571_237};	/*Property = CssFlexbox*/
const Value V21571_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_288 = {S11, 0, &V21571_288_57176, &P21571_287};	/*Property = CssFont*/
const Value V21571_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_289 = {S12, 0, &V21571_289_86899, &P21571_288};	/*Property = CssImages*/
const Value V21571_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_291 = {S13, 0, &V21571_291_57182, &P21571_289};	/*Property = CssMediaQueries*/
const Value V21571_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_292 = {S14, 0, &V21571_292_57185, &P21571_291};	/*Property = CssMinMax*/
const Value V21571_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_293 = {S15, 0, &V21571_293_57188, &P21571_292};	/*Property = CssOverflow*/
const Value V21571_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_294 = {S16, 0, &V21571_294_57191, &P21571_293};	/*Property = CssPosition*/
const Value V21571_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_295 = {S17, 0, &V21571_295_57194, &P21571_294};	/*Property = CssText*/
const Value V21571_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_297 = {S18, 0, &V21571_297_57197, &P21571_295};	/*Property = CssTransforms*/
const Value V21571_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_241 = {S19, 0, &V21571_241_55801, &P21571_297};	/*Property = CssTransitions*/
const Value V21571_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_298 = {S20, 0, &V21571_298_57208, &P21571_241};	/*Property = CssUI*/
const Value V21571_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_300 = {S21, 0, &V21571_300_57217, &P21571_298};	/*Property = DataSet*/
const Value V21571_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_301 = {S22, 0, &V21571_301_57220, &P21571_300};	/*Property = DataUrl*/
const Value V21571_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_302 = {S23, 0, &V21571_302_57223, &P21571_301};	/*Property = DeviceOrientation*/
const Value V21571_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_304 = {S24, 0, &V21571_304_57229, &P21571_302};	/*Property = FileReader*/
const Value V21571_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_305 = {S25, 0, &V21571_305_57232, &P21571_304};	/*Property = FileSaver*/
const Value V21571_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_306 = {S26, 0, &V21571_306_57238, &P21571_305};	/*Property = FileWriter*/
const Value V21571_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_307 = {S27, 0, &V21571_307_57241, &P21571_306};	/*Property = FormData*/
const Value V21571_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_309 = {S28, 0, &V21571_309_57247, &P21571_307};	/*Property = Fullscreen*/
const Value V21571_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_310 = {S29, 0, &V21571_310_57250, &P21571_309};	/*Property = GeoLocation*/
const Value V21571_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_312 = {S30, 0, &V21571_312_57256, &P21571_310};	/*Property = History*/
const Value V21571_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_314 = {S31, 0, &V21571_314_57262, &P21571_312};	/*Property = Html5*/
const Value V21571_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_313 = {S32, 0, &V21571_313_57259, &P21571_314};	/*Property = Html-Media-Capture*/
const Value V21571_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_315 = {S33, 0, &V21571_315_57265, &P21571_313};	/*Property = Iframe*/
const Value V21571_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_316 = {S34, 0, &V21571_316_57268, &P21571_315};	/*Property = IndexedDB*/
const Value V21571_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_317 = {S35, 0, &V21571_317_57271, &P21571_316};	/*Property = Json*/
const Value V21571_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21571_15 = {S37, 0, &V21571_15_28, &P21571_317};	/*Property = LayoutEngine*/
const Value V21571_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_344 = {S38, 0, &V21571_344_57274, &P21571_15};	/*Property = Masking*/
const Value V21571_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_320 = {S39, 0, &V21571_320_57292, &P21571_344};	/*Property = PostMessage*/
const Value V21571_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_321 = {S40, 0, &V21571_321_57295, &P21571_320};	/*Property = Progress*/
const Value V21571_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_322 = {S41, 0, &V21571_322_57298, &P21571_321};	/*Property = Prompts*/
const Value V21571_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_323 = {S42, 0, &V21571_323_57301, &P21571_322};	/*Property = Selector*/
const Value V21571_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_324 = {S43, 0, &V21571_324_57307, &P21571_323};	/*Property = Svg*/
const Value V21571_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_327 = {S44, 0, &V21571_327_57316, &P21571_324};	/*Property = TouchEvents*/
const Value V21571_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_328 = {S45, 0, &V21571_328_57319, &P21571_327};	/*Property = Track*/
const Value V21571_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_329 = {S46, 0, &V21571_329_57325, &P21571_328};	/*Property = Video*/
const Value V21571_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_330 = {S47, 0, &V21571_330_57328, &P21571_329};	/*Property = Viewport*/
const Value V21571_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_332 = {S48, 0, &V21571_332_57340, &P21571_330};	/*Property = WebWorkers*/
const Value V21571_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21571_333 = {S49, 0, &V21571_333_57343, &P21571_332};	/*Property = Xhr2*/
const Profile P21571 = {"21571", &P21571_333};	/*Profile ID - 21571*/
const Value V24727_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24727_15 = {S37, 0, &V24727_15_28, NULL};	/*Property = LayoutEngine*/
const Profile P24727 = {"24727", &P24727_15};	/*Profile ID - 24727*/
const Value V18647_1_1 = {S2, NULL};	/*Value = True*/
const Property P18647_1 = {S50, 0, &V18647_1_1, NULL};	/*Property = IsMobile*/
const Value V18647_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18647_5 = {S52, 0, &V18647_5_9981, &P18647_1};	/*Property = ScreenPixelsHeight*/
const Value V18647_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P18647_4 = {S54, 0, &V18647_4_37617, &P18647_5};	/*Property = ScreenPixelsWidth*/
const Profile P18647 = {"18647", &P18647_4};	/*Profile ID - 18647*/
const Profile P18580 = {"18580", NULL};	/*Profile ID - 18580*/
const Profile P24690 = {"24690", NULL};	/*Profile ID - 24690*/
const Value V19819_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_279 = {S1, 0, &V19819_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V19819_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_282 = {S3, 0, &V19819_282_57155, &P19819_279};	/*Property = BlobBuilder*/
const Value V19819_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_350 = {S4, 0, &V19819_350_57446, &P19819_282};	/*Property = Canvas*/
const Value V19819_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_283 = {S5, 0, &V19819_283_57161, &P19819_350};	/*Property = CssBackground*/
const Value V19819_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_284 = {S6, 0, &V19819_284_57164, &P19819_283};	/*Property = CssBorderImage*/
const Value V19819_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_285 = {S7, 0, &V19819_285_57167, &P19819_284};	/*Property = CssCanvas*/
const Value V19819_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_286 = {S8, 0, &V19819_286_57170, &P19819_285};	/*Property = CssColor*/
const Value V19819_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_237 = {S9, 0, &V19819_237_55786, &P19819_286};	/*Property = CssColumn*/
const Value V19819_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_287 = {S10, 0, &V19819_287_57173, &P19819_237};	/*Property = CssFlexbox*/
const Value V19819_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_288 = {S11, 0, &V19819_288_57176, &P19819_287};	/*Property = CssFont*/
const Value V19819_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_289 = {S12, 0, &V19819_289_86899, &P19819_288};	/*Property = CssImages*/
const Value V19819_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_291 = {S13, 0, &V19819_291_57182, &P19819_289};	/*Property = CssMediaQueries*/
const Value V19819_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_292 = {S14, 0, &V19819_292_57185, &P19819_291};	/*Property = CssMinMax*/
const Value V19819_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_293 = {S15, 0, &V19819_293_57188, &P19819_292};	/*Property = CssOverflow*/
const Value V19819_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_294 = {S16, 0, &V19819_294_57191, &P19819_293};	/*Property = CssPosition*/
const Value V19819_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_295 = {S17, 0, &V19819_295_57194, &P19819_294};	/*Property = CssText*/
const Value V19819_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_297 = {S18, 0, &V19819_297_57197, &P19819_295};	/*Property = CssTransforms*/
const Value V19819_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_241 = {S19, 0, &V19819_241_55801, &P19819_297};	/*Property = CssTransitions*/
const Value V19819_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_298 = {S20, 0, &V19819_298_57208, &P19819_241};	/*Property = CssUI*/
const Value V19819_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_300 = {S21, 0, &V19819_300_57217, &P19819_298};	/*Property = DataSet*/
const Value V19819_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_301 = {S22, 0, &V19819_301_57220, &P19819_300};	/*Property = DataUrl*/
const Value V19819_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_302 = {S23, 0, &V19819_302_57223, &P19819_301};	/*Property = DeviceOrientation*/
const Value V19819_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_304 = {S24, 0, &V19819_304_57229, &P19819_302};	/*Property = FileReader*/
const Value V19819_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_305 = {S25, 0, &V19819_305_57232, &P19819_304};	/*Property = FileSaver*/
const Value V19819_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_306 = {S26, 0, &V19819_306_57238, &P19819_305};	/*Property = FileWriter*/
const Value V19819_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_307 = {S27, 0, &V19819_307_57241, &P19819_306};	/*Property = FormData*/
const Value V19819_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_309 = {S28, 0, &V19819_309_57247, &P19819_307};	/*Property = Fullscreen*/
const Value V19819_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_310 = {S29, 0, &V19819_310_57250, &P19819_309};	/*Property = GeoLocation*/
const Value V19819_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_312 = {S30, 0, &V19819_312_57256, &P19819_310};	/*Property = History*/
const Value V19819_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_314 = {S31, 0, &V19819_314_57262, &P19819_312};	/*Property = Html5*/
const Value V19819_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_313 = {S32, 0, &V19819_313_57259, &P19819_314};	/*Property = Html-Media-Capture*/
const Value V19819_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_315 = {S33, 0, &V19819_315_57265, &P19819_313};	/*Property = Iframe*/
const Value V19819_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_316 = {S34, 0, &V19819_316_57268, &P19819_315};	/*Property = IndexedDB*/
const Value V19819_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_317 = {S35, 0, &V19819_317_57271, &P19819_316};	/*Property = Json*/
const Value V19819_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P19819_15 = {S37, 0, &V19819_15_54167, &P19819_317};	/*Property = LayoutEngine*/
const Value V19819_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_344 = {S38, 0, &V19819_344_57274, &P19819_15};	/*Property = Masking*/
const Value V19819_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_320 = {S39, 0, &V19819_320_57292, &P19819_344};	/*Property = PostMessage*/
const Value V19819_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_321 = {S40, 0, &V19819_321_57295, &P19819_320};	/*Property = Progress*/
const Value V19819_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_322 = {S41, 0, &V19819_322_57298, &P19819_321};	/*Property = Prompts*/
const Value V19819_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_323 = {S42, 0, &V19819_323_57301, &P19819_322};	/*Property = Selector*/
const Value V19819_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_324 = {S43, 0, &V19819_324_57307, &P19819_323};	/*Property = Svg*/
const Value V19819_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_327 = {S44, 0, &V19819_327_57316, &P19819_324};	/*Property = TouchEvents*/
const Value V19819_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_328 = {S45, 0, &V19819_328_57319, &P19819_327};	/*Property = Track*/
const Value V19819_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_329 = {S46, 0, &V19819_329_57325, &P19819_328};	/*Property = Video*/
const Value V19819_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_330 = {S47, 0, &V19819_330_57328, &P19819_329};	/*Property = Viewport*/
const Value V19819_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_332 = {S48, 0, &V19819_332_57340, &P19819_330};	/*Property = WebWorkers*/
const Value V19819_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P19819_333 = {S49, 0, &V19819_333_57343, &P19819_332};	/*Property = Xhr2*/
const Profile P19819 = {"19819", &P19819_333};	/*Profile ID - 19819*/
const Value V21082_1_1 = {S2, NULL};	/*Value = True*/
const Property P21082_1 = {S50, 0, &V21082_1_1, NULL};	/*Property = IsMobile*/
const Value V21082_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21082_5 = {S52, 0, &V21082_5_1222, &P21082_1};	/*Property = ScreenPixelsHeight*/
const Value V21082_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21082_4 = {S54, 0, &V21082_4_1224, &P21082_5};	/*Property = ScreenPixelsWidth*/
const Profile P21082 = {"21082", &P21082_4};	/*Profile ID - 21082*/
const Value V20792_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20792_279 = {S1, 0, &V20792_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20792_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20792_282 = {S3, 0, &V20792_282_57154, &P20792_279};	/*Property = BlobBuilder*/
const Value V20792_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20792_350 = {S4, 0, &V20792_350_57444, &P20792_282};	/*Property = Canvas*/
const Value V20792_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20792_283 = {S5, 0, &V20792_283_57159, &P20792_350};	/*Property = CssBackground*/
const Value V20792_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20792_284 = {S6, 0, &V20792_284_57163, &P20792_283};	/*Property = CssBorderImage*/
const Value V20792_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20792_285 = {S7, 0, &V20792_285_57166, &P20792_284};	/*Property = CssCanvas*/
const Value V20792_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20792_286 = {S8, 0, &V20792_286_57168, &P20792_285};	/*Property = CssColor*/
const Value V20792_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20792_237 = {S9, 0, &V20792_237_55786, &P20792_286};	/*Property = CssColumn*/
const Value V20792_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20792_287 = {S10, 0, &V20792_287_57172, &P20792_237};	/*Property = CssFlexbox*/
const Value V20792_288_57175 = {S0, NULL};	/*Value = False*/
const Property P20792_288 = {S11, 0, &V20792_288_57175, &P20792_287};	/*Property = CssFont*/
const Value V20792_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20792_289 = {S12, 0, &V20792_289_57759, &P20792_288};	/*Property = CssImages*/
const Value V20792_291_57181 = {S0, NULL};	/*Value = False*/
const Property P20792_291 = {S13, 0, &V20792_291_57181, &P20792_289};	/*Property = CssMediaQueries*/
const Value V20792_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20792_292 = {S14, 0, &V20792_292_57183, &P20792_291};	/*Property = CssMinMax*/
const Value V20792_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20792_293 = {S15, 0, &V20792_293_57187, &P20792_292};	/*Property = CssOverflow*/
const Value V20792_294_57190 = {S0, NULL};	/*Value = False*/
const Property P20792_294 = {S16, 0, &V20792_294_57190, &P20792_293};	/*Property = CssPosition*/
const Value V20792_295_57193 = {S0, NULL};	/*Value = False*/
const Property P20792_295 = {S17, 0, &V20792_295_57193, &P20792_294};	/*Property = CssText*/
const Value V20792_297_57196 = {S0, NULL};	/*Value = False*/
const Property P20792_297 = {S18, 0, &V20792_297_57196, &P20792_295};	/*Property = CssTransforms*/
const Value V20792_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20792_241 = {S19, 0, &V20792_241_55799, &P20792_297};	/*Property = CssTransitions*/
const Value V20792_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20792_298 = {S20, 0, &V20792_298_57206, &P20792_241};	/*Property = CssUI*/
const Value V20792_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20792_300 = {S21, 0, &V20792_300_57216, &P20792_298};	/*Property = DataSet*/
const Value V20792_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20792_301 = {S22, 0, &V20792_301_57218, &P20792_300};	/*Property = DataUrl*/
const Value V20792_302_57222 = {S0, NULL};	/*Value = False*/
const Property P20792_302 = {S23, 0, &V20792_302_57222, &P20792_301};	/*Property = DeviceOrientation*/
const Value V20792_304_57228 = {S0, NULL};	/*Value = False*/
const Property P20792_304 = {S24, 0, &V20792_304_57228, &P20792_302};	/*Property = FileReader*/
const Value V20792_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20792_305 = {S25, 0, &V20792_305_57231, &P20792_304};	/*Property = FileSaver*/
const Value V20792_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20792_306 = {S26, 0, &V20792_306_57237, &P20792_305};	/*Property = FileWriter*/
const Value V20792_307_57240 = {S0, NULL};	/*Value = False*/
const Property P20792_307 = {S27, 0, &V20792_307_57240, &P20792_306};	/*Property = FormData*/
const Value V20792_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20792_309 = {S28, 0, &V20792_309_57246, &P20792_307};	/*Property = Fullscreen*/
const Value V20792_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20792_310 = {S29, 0, &V20792_310_57248, &P20792_309};	/*Property = GeoLocation*/
const Value V20792_312_57255 = {S0, NULL};	/*Value = False*/
const Property P20792_312 = {S30, 0, &V20792_312_57255, &P20792_310};	/*Property = History*/
const Value V20792_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20792_314 = {S31, 0, &V20792_314_57261, &P20792_312};	/*Property = Html5*/
const Value V20792_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20792_313 = {S32, 0, &V20792_313_57258, &P20792_314};	/*Property = Html-Media-Capture*/
const Value V20792_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20792_315 = {S33, 0, &V20792_315_57264, &P20792_313};	/*Property = Iframe*/
const Value V20792_316_57267 = {S0, NULL};	/*Value = False*/
const Property P20792_316 = {S34, 0, &V20792_316_57267, &P20792_315};	/*Property = IndexedDB*/
const Value V20792_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20792_317 = {S35, 0, &V20792_317_57269, &P20792_316};	/*Property = Json*/
const Value V20792_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P20792_15 = {S37, 0, &V20792_15_31, &P20792_317};	/*Property = LayoutEngine*/
const Value V20792_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20792_344 = {S38, 0, &V20792_344_57272, &P20792_15};	/*Property = Masking*/
const Value V20792_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20792_320 = {S39, 0, &V20792_320_57290, &P20792_344};	/*Property = PostMessage*/
const Value V20792_321_57294 = {S0, NULL};	/*Value = False*/
const Property P20792_321 = {S40, 0, &V20792_321_57294, &P20792_320};	/*Property = Progress*/
const Value V20792_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20792_322 = {S41, 0, &V20792_322_57296, &P20792_321};	/*Property = Prompts*/
const Value V20792_323_57300 = {S0, NULL};	/*Value = False*/
const Property P20792_323 = {S42, 0, &V20792_323_57300, &P20792_322};	/*Property = Selector*/
const Value V20792_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20792_324 = {S43, 0, &V20792_324_57306, &P20792_323};	/*Property = Svg*/
const Value V20792_327_57315 = {S0, NULL};	/*Value = False*/
const Property P20792_327 = {S44, 0, &V20792_327_57315, &P20792_324};	/*Property = TouchEvents*/
const Value V20792_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20792_328 = {S45, 0, &V20792_328_57318, &P20792_327};	/*Property = Track*/
const Value V20792_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20792_329 = {S46, 0, &V20792_329_57323, &P20792_328};	/*Property = Video*/
const Value V20792_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20792_330 = {S47, 0, &V20792_330_57326, &P20792_329};	/*Property = Viewport*/
const Value V20792_332_57339 = {S0, NULL};	/*Value = False*/
const Property P20792_332 = {S48, 0, &V20792_332_57339, &P20792_330};	/*Property = WebWorkers*/
const Value V20792_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20792_333 = {S49, 0, &V20792_333_57342, &P20792_332};	/*Property = Xhr2*/
const Profile P20792 = {"20792", &P20792_333};	/*Profile ID - 20792*/
const Value V22552_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22552_279 = {S1, 0, &V22552_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22552_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22552_282 = {S3, 0, &V22552_282_57154, &P22552_279};	/*Property = BlobBuilder*/
const Value V22552_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22552_350 = {S4, 0, &V22552_350_57444, &P22552_282};	/*Property = Canvas*/
const Value V22552_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22552_283 = {S5, 0, &V22552_283_57159, &P22552_350};	/*Property = CssBackground*/
const Value V22552_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22552_284 = {S6, 0, &V22552_284_57163, &P22552_283};	/*Property = CssBorderImage*/
const Value V22552_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22552_285 = {S7, 0, &V22552_285_57166, &P22552_284};	/*Property = CssCanvas*/
const Value V22552_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22552_286 = {S8, 0, &V22552_286_57168, &P22552_285};	/*Property = CssColor*/
const Value V22552_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22552_237 = {S9, 0, &V22552_237_55784, &P22552_286};	/*Property = CssColumn*/
const Value V22552_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22552_287 = {S10, 0, &V22552_287_57172, &P22552_237};	/*Property = CssFlexbox*/
const Value V22552_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22552_288 = {S11, 0, &V22552_288_57175, &P22552_287};	/*Property = CssFont*/
const Value V22552_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22552_289 = {S12, 0, &V22552_289_57759, &P22552_288};	/*Property = CssImages*/
const Value V22552_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22552_291 = {S13, 0, &V22552_291_57181, &P22552_289};	/*Property = CssMediaQueries*/
const Value V22552_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22552_292 = {S14, 0, &V22552_292_57183, &P22552_291};	/*Property = CssMinMax*/
const Value V22552_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22552_293 = {S15, 0, &V22552_293_57187, &P22552_292};	/*Property = CssOverflow*/
const Value V22552_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22552_294 = {S16, 0, &V22552_294_57190, &P22552_293};	/*Property = CssPosition*/
const Value V22552_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22552_295 = {S17, 0, &V22552_295_57193, &P22552_294};	/*Property = CssText*/
const Value V22552_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22552_297 = {S18, 0, &V22552_297_57195, &P22552_295};	/*Property = CssTransforms*/
const Value V22552_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22552_241 = {S19, 0, &V22552_241_55799, &P22552_297};	/*Property = CssTransitions*/
const Value V22552_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22552_298 = {S20, 0, &V22552_298_57206, &P22552_241};	/*Property = CssUI*/
const Value V22552_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22552_300 = {S21, 0, &V22552_300_57216, &P22552_298};	/*Property = DataSet*/
const Value V22552_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22552_301 = {S22, 0, &V22552_301_57218, &P22552_300};	/*Property = DataUrl*/
const Value V22552_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22552_302 = {S23, 0, &V22552_302_57222, &P22552_301};	/*Property = DeviceOrientation*/
const Value V22552_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22552_304 = {S24, 0, &V22552_304_57228, &P22552_302};	/*Property = FileReader*/
const Value V22552_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22552_305 = {S25, 0, &V22552_305_57231, &P22552_304};	/*Property = FileSaver*/
const Value V22552_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22552_306 = {S26, 0, &V22552_306_57237, &P22552_305};	/*Property = FileWriter*/
const Value V22552_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22552_307 = {S27, 0, &V22552_307_57240, &P22552_306};	/*Property = FormData*/
const Value V22552_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22552_309 = {S28, 0, &V22552_309_57246, &P22552_307};	/*Property = Fullscreen*/
const Value V22552_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22552_310 = {S29, 0, &V22552_310_57248, &P22552_309};	/*Property = GeoLocation*/
const Value V22552_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22552_312 = {S30, 0, &V22552_312_57255, &P22552_310};	/*Property = History*/
const Value V22552_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22552_314 = {S31, 0, &V22552_314_57261, &P22552_312};	/*Property = Html5*/
const Value V22552_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22552_313 = {S32, 0, &V22552_313_57258, &P22552_314};	/*Property = Html-Media-Capture*/
const Value V22552_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22552_315 = {S33, 0, &V22552_315_57264, &P22552_313};	/*Property = Iframe*/
const Value V22552_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22552_316 = {S34, 0, &V22552_316_57267, &P22552_315};	/*Property = IndexedDB*/
const Value V22552_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22552_317 = {S35, 0, &V22552_317_57269, &P22552_316};	/*Property = Json*/
const Value V22552_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22552_15 = {S37, 0, &V22552_15_29, &P22552_317};	/*Property = LayoutEngine*/
const Value V22552_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22552_344 = {S38, 0, &V22552_344_57272, &P22552_15};	/*Property = Masking*/
const Value V22552_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22552_320 = {S39, 0, &V22552_320_57290, &P22552_344};	/*Property = PostMessage*/
const Value V22552_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22552_321 = {S40, 0, &V22552_321_57293, &P22552_320};	/*Property = Progress*/
const Value V22552_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22552_322 = {S41, 0, &V22552_322_57296, &P22552_321};	/*Property = Prompts*/
const Value V22552_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22552_323 = {S42, 0, &V22552_323_57299, &P22552_322};	/*Property = Selector*/
const Value V22552_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22552_324 = {S43, 0, &V22552_324_57306, &P22552_323};	/*Property = Svg*/
const Value V22552_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22552_327 = {S44, 0, &V22552_327_57315, &P22552_324};	/*Property = TouchEvents*/
const Value V22552_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22552_328 = {S45, 0, &V22552_328_57318, &P22552_327};	/*Property = Track*/
const Value V22552_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22552_329 = {S46, 0, &V22552_329_57323, &P22552_328};	/*Property = Video*/
const Value V22552_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22552_330 = {S47, 0, &V22552_330_57326, &P22552_329};	/*Property = Viewport*/
const Value V22552_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22552_332 = {S48, 0, &V22552_332_57339, &P22552_330};	/*Property = WebWorkers*/
const Value V22552_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22552_333 = {S49, 0, &V22552_333_57342, &P22552_332};	/*Property = Xhr2*/
const Profile P22552 = {"22552", &P22552_333};	/*Profile ID - 22552*/
const Value V21700_1_1 = {S2, NULL};	/*Value = True*/
const Property P21700_1 = {S50, 0, &V21700_1_1, NULL};	/*Property = IsMobile*/
const Value V21700_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P21700_5 = {S52, 0, &V21700_5_1304, &P21700_1};	/*Property = ScreenPixelsHeight*/
const Value V21700_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21700_4 = {S54, 0, &V21700_4_2336, &P21700_5};	/*Property = ScreenPixelsWidth*/
const Profile P21700 = {"21700", &P21700_4};	/*Profile ID - 21700*/
const Value V18532_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18532_279 = {S1, 0, &V18532_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18532_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18532_282 = {S3, 0, &V18532_282_57154, &P18532_279};	/*Property = BlobBuilder*/
const Value V18532_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18532_350 = {S4, 0, &V18532_350_57444, &P18532_282};	/*Property = Canvas*/
const Value V18532_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18532_283 = {S5, 0, &V18532_283_57159, &P18532_350};	/*Property = CssBackground*/
const Value V18532_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18532_284 = {S6, 0, &V18532_284_57163, &P18532_283};	/*Property = CssBorderImage*/
const Value V18532_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18532_285 = {S7, 0, &V18532_285_57166, &P18532_284};	/*Property = CssCanvas*/
const Value V18532_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18532_286 = {S8, 0, &V18532_286_57168, &P18532_285};	/*Property = CssColor*/
const Value V18532_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18532_237 = {S9, 0, &V18532_237_55786, &P18532_286};	/*Property = CssColumn*/
const Value V18532_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18532_287 = {S10, 0, &V18532_287_57172, &P18532_237};	/*Property = CssFlexbox*/
const Value V18532_288_57174 = {S2, NULL};	/*Value = True*/
const Property P18532_288 = {S11, 0, &V18532_288_57174, &P18532_287};	/*Property = CssFont*/
const Value V18532_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18532_289 = {S12, 0, &V18532_289_57759, &P18532_288};	/*Property = CssImages*/
const Value V18532_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18532_291 = {S13, 0, &V18532_291_57180, &P18532_289};	/*Property = CssMediaQueries*/
const Value V18532_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18532_292 = {S14, 0, &V18532_292_57183, &P18532_291};	/*Property = CssMinMax*/
const Value V18532_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18532_293 = {S15, 0, &V18532_293_57187, &P18532_292};	/*Property = CssOverflow*/
const Value V18532_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18532_294 = {S16, 0, &V18532_294_57189, &P18532_293};	/*Property = CssPosition*/
const Value V18532_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18532_295 = {S17, 0, &V18532_295_57192, &P18532_294};	/*Property = CssText*/
const Value V18532_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18532_297 = {S18, 0, &V18532_297_57195, &P18532_295};	/*Property = CssTransforms*/
const Value V18532_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18532_241 = {S19, 0, &V18532_241_55799, &P18532_297};	/*Property = CssTransitions*/
const Value V18532_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18532_298 = {S20, 0, &V18532_298_57206, &P18532_241};	/*Property = CssUI*/
const Value V18532_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18532_300 = {S21, 0, &V18532_300_57216, &P18532_298};	/*Property = DataSet*/
const Value V18532_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18532_301 = {S22, 0, &V18532_301_57218, &P18532_300};	/*Property = DataUrl*/
const Value V18532_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18532_302 = {S23, 0, &V18532_302_57222, &P18532_301};	/*Property = DeviceOrientation*/
const Value V18532_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18532_304 = {S24, 0, &V18532_304_57227, &P18532_302};	/*Property = FileReader*/
const Value V18532_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18532_305 = {S25, 0, &V18532_305_57231, &P18532_304};	/*Property = FileSaver*/
const Value V18532_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18532_306 = {S26, 0, &V18532_306_57237, &P18532_305};	/*Property = FileWriter*/
const Value V18532_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18532_307 = {S27, 0, &V18532_307_57239, &P18532_306};	/*Property = FormData*/
const Value V18532_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18532_309 = {S28, 0, &V18532_309_57246, &P18532_307};	/*Property = Fullscreen*/
const Value V18532_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18532_310 = {S29, 0, &V18532_310_57248, &P18532_309};	/*Property = GeoLocation*/
const Value V18532_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18532_312 = {S30, 0, &V18532_312_57254, &P18532_310};	/*Property = History*/
const Value V18532_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18532_314 = {S31, 0, &V18532_314_57261, &P18532_312};	/*Property = Html5*/
const Value V18532_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18532_313 = {S32, 0, &V18532_313_57258, &P18532_314};	/*Property = Html-Media-Capture*/
const Value V18532_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18532_315 = {S33, 0, &V18532_315_57264, &P18532_313};	/*Property = Iframe*/
const Value V18532_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18532_316 = {S34, 0, &V18532_316_57266, &P18532_315};	/*Property = IndexedDB*/
const Value V18532_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18532_317 = {S35, 0, &V18532_317_57269, &P18532_316};	/*Property = Json*/
const Value V18532_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P18532_15 = {S37, 0, &V18532_15_30, &P18532_317};	/*Property = LayoutEngine*/
const Value V18532_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18532_344 = {S38, 0, &V18532_344_57272, &P18532_15};	/*Property = Masking*/
const Value V18532_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18532_320 = {S39, 0, &V18532_320_57290, &P18532_344};	/*Property = PostMessage*/
const Value V18532_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18532_321 = {S40, 0, &V18532_321_57293, &P18532_320};	/*Property = Progress*/
const Value V18532_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18532_322 = {S41, 0, &V18532_322_57296, &P18532_321};	/*Property = Prompts*/
const Value V18532_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18532_323 = {S42, 0, &V18532_323_57299, &P18532_322};	/*Property = Selector*/
const Value V18532_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18532_324 = {S43, 0, &V18532_324_57306, &P18532_323};	/*Property = Svg*/
const Value V18532_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18532_327 = {S44, 0, &V18532_327_57315, &P18532_324};	/*Property = TouchEvents*/
const Value V18532_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18532_328 = {S45, 0, &V18532_328_57318, &P18532_327};	/*Property = Track*/
const Value V18532_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18532_329 = {S46, 0, &V18532_329_57323, &P18532_328};	/*Property = Video*/
const Value V18532_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18532_330 = {S47, 0, &V18532_330_57326, &P18532_329};	/*Property = Viewport*/
const Value V18532_332_57338 = {S2, NULL};	/*Value = True*/
const Property P18532_332 = {S48, 0, &V18532_332_57338, &P18532_330};	/*Property = WebWorkers*/
const Value V18532_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18532_333 = {S49, 0, &V18532_333_57343, &P18532_332};	/*Property = Xhr2*/
const Profile P18532 = {"18532", &P18532_333};	/*Profile ID - 18532*/
const Value V22461_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22461_279 = {S1, 0, &V22461_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22461_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22461_282 = {S3, 0, &V22461_282_57154, &P22461_279};	/*Property = BlobBuilder*/
const Value V22461_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22461_350 = {S4, 0, &V22461_350_57444, &P22461_282};	/*Property = Canvas*/
const Value V22461_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22461_283 = {S5, 0, &V22461_283_57159, &P22461_350};	/*Property = CssBackground*/
const Value V22461_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22461_284 = {S6, 0, &V22461_284_57163, &P22461_283};	/*Property = CssBorderImage*/
const Value V22461_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22461_285 = {S7, 0, &V22461_285_57166, &P22461_284};	/*Property = CssCanvas*/
const Value V22461_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22461_286 = {S8, 0, &V22461_286_57168, &P22461_285};	/*Property = CssColor*/
const Value V22461_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22461_237 = {S9, 0, &V22461_237_55786, &P22461_286};	/*Property = CssColumn*/
const Value V22461_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22461_287 = {S10, 0, &V22461_287_57172, &P22461_237};	/*Property = CssFlexbox*/
const Value V22461_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22461_288 = {S11, 0, &V22461_288_57174, &P22461_287};	/*Property = CssFont*/
const Value V22461_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22461_289 = {S12, 0, &V22461_289_57759, &P22461_288};	/*Property = CssImages*/
const Value V22461_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22461_291 = {S13, 0, &V22461_291_57180, &P22461_289};	/*Property = CssMediaQueries*/
const Value V22461_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22461_292 = {S14, 0, &V22461_292_57183, &P22461_291};	/*Property = CssMinMax*/
const Value V22461_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22461_293 = {S15, 0, &V22461_293_57187, &P22461_292};	/*Property = CssOverflow*/
const Value V22461_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22461_294 = {S16, 0, &V22461_294_57189, &P22461_293};	/*Property = CssPosition*/
const Value V22461_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22461_295 = {S17, 0, &V22461_295_57192, &P22461_294};	/*Property = CssText*/
const Value V22461_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22461_297 = {S18, 0, &V22461_297_57195, &P22461_295};	/*Property = CssTransforms*/
const Value V22461_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22461_241 = {S19, 0, &V22461_241_55799, &P22461_297};	/*Property = CssTransitions*/
const Value V22461_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22461_298 = {S20, 0, &V22461_298_57206, &P22461_241};	/*Property = CssUI*/
const Value V22461_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22461_300 = {S21, 0, &V22461_300_57216, &P22461_298};	/*Property = DataSet*/
const Value V22461_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22461_301 = {S22, 0, &V22461_301_57218, &P22461_300};	/*Property = DataUrl*/
const Value V22461_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22461_302 = {S23, 0, &V22461_302_57222, &P22461_301};	/*Property = DeviceOrientation*/
const Value V22461_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22461_304 = {S24, 0, &V22461_304_57227, &P22461_302};	/*Property = FileReader*/
const Value V22461_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22461_305 = {S25, 0, &V22461_305_57231, &P22461_304};	/*Property = FileSaver*/
const Value V22461_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22461_306 = {S26, 0, &V22461_306_57237, &P22461_305};	/*Property = FileWriter*/
const Value V22461_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22461_307 = {S27, 0, &V22461_307_57239, &P22461_306};	/*Property = FormData*/
const Value V22461_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22461_309 = {S28, 0, &V22461_309_57246, &P22461_307};	/*Property = Fullscreen*/
const Value V22461_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22461_310 = {S29, 0, &V22461_310_57248, &P22461_309};	/*Property = GeoLocation*/
const Value V22461_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22461_312 = {S30, 0, &V22461_312_57254, &P22461_310};	/*Property = History*/
const Value V22461_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22461_314 = {S31, 0, &V22461_314_57261, &P22461_312};	/*Property = Html5*/
const Value V22461_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22461_313 = {S32, 0, &V22461_313_57258, &P22461_314};	/*Property = Html-Media-Capture*/
const Value V22461_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22461_315 = {S33, 0, &V22461_315_57264, &P22461_313};	/*Property = Iframe*/
const Value V22461_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22461_316 = {S34, 0, &V22461_316_57266, &P22461_315};	/*Property = IndexedDB*/
const Value V22461_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22461_317 = {S35, 0, &V22461_317_57269, &P22461_316};	/*Property = Json*/
const Value V22461_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22461_15 = {S37, 0, &V22461_15_28, &P22461_317};	/*Property = LayoutEngine*/
const Value V22461_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22461_344 = {S38, 0, &V22461_344_57272, &P22461_15};	/*Property = Masking*/
const Value V22461_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22461_320 = {S39, 0, &V22461_320_57290, &P22461_344};	/*Property = PostMessage*/
const Value V22461_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22461_321 = {S40, 0, &V22461_321_57293, &P22461_320};	/*Property = Progress*/
const Value V22461_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22461_322 = {S41, 0, &V22461_322_57296, &P22461_321};	/*Property = Prompts*/
const Value V22461_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22461_323 = {S42, 0, &V22461_323_57299, &P22461_322};	/*Property = Selector*/
const Value V22461_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22461_324 = {S43, 0, &V22461_324_57306, &P22461_323};	/*Property = Svg*/
const Value V22461_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22461_327 = {S44, 0, &V22461_327_57315, &P22461_324};	/*Property = TouchEvents*/
const Value V22461_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22461_328 = {S45, 0, &V22461_328_57318, &P22461_327};	/*Property = Track*/
const Value V22461_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22461_329 = {S46, 0, &V22461_329_57323, &P22461_328};	/*Property = Video*/
const Value V22461_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22461_330 = {S47, 0, &V22461_330_57326, &P22461_329};	/*Property = Viewport*/
const Value V22461_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22461_332 = {S48, 0, &V22461_332_57338, &P22461_330};	/*Property = WebWorkers*/
const Value V22461_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22461_333 = {S49, 0, &V22461_333_57341, &P22461_332};	/*Property = Xhr2*/
const Profile P22461 = {"22461", &P22461_333};	/*Profile ID - 22461*/
const Value V22376_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22376_279 = {S1, 0, &V22376_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22376_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22376_282 = {S3, 0, &V22376_282_57154, &P22376_279};	/*Property = BlobBuilder*/
const Value V22376_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22376_350 = {S4, 0, &V22376_350_57444, &P22376_282};	/*Property = Canvas*/
const Value V22376_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22376_283 = {S5, 0, &V22376_283_57159, &P22376_350};	/*Property = CssBackground*/
const Value V22376_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22376_284 = {S6, 0, &V22376_284_57163, &P22376_283};	/*Property = CssBorderImage*/
const Value V22376_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22376_285 = {S7, 0, &V22376_285_57166, &P22376_284};	/*Property = CssCanvas*/
const Value V22376_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22376_286 = {S8, 0, &V22376_286_57168, &P22376_285};	/*Property = CssColor*/
const Value V22376_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22376_237 = {S9, 0, &V22376_237_55786, &P22376_286};	/*Property = CssColumn*/
const Value V22376_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22376_287 = {S10, 0, &V22376_287_57172, &P22376_237};	/*Property = CssFlexbox*/
const Value V22376_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22376_288 = {S11, 0, &V22376_288_57174, &P22376_287};	/*Property = CssFont*/
const Value V22376_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22376_289 = {S12, 0, &V22376_289_57759, &P22376_288};	/*Property = CssImages*/
const Value V22376_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22376_291 = {S13, 0, &V22376_291_57180, &P22376_289};	/*Property = CssMediaQueries*/
const Value V22376_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22376_292 = {S14, 0, &V22376_292_57183, &P22376_291};	/*Property = CssMinMax*/
const Value V22376_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22376_293 = {S15, 0, &V22376_293_57187, &P22376_292};	/*Property = CssOverflow*/
const Value V22376_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22376_294 = {S16, 0, &V22376_294_57189, &P22376_293};	/*Property = CssPosition*/
const Value V22376_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22376_295 = {S17, 0, &V22376_295_57192, &P22376_294};	/*Property = CssText*/
const Value V22376_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22376_297 = {S18, 0, &V22376_297_57195, &P22376_295};	/*Property = CssTransforms*/
const Value V22376_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22376_241 = {S19, 0, &V22376_241_55799, &P22376_297};	/*Property = CssTransitions*/
const Value V22376_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22376_298 = {S20, 0, &V22376_298_57206, &P22376_241};	/*Property = CssUI*/
const Value V22376_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22376_300 = {S21, 0, &V22376_300_57216, &P22376_298};	/*Property = DataSet*/
const Value V22376_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22376_301 = {S22, 0, &V22376_301_57218, &P22376_300};	/*Property = DataUrl*/
const Value V22376_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22376_302 = {S23, 0, &V22376_302_57222, &P22376_301};	/*Property = DeviceOrientation*/
const Value V22376_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22376_304 = {S24, 0, &V22376_304_57227, &P22376_302};	/*Property = FileReader*/
const Value V22376_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22376_305 = {S25, 0, &V22376_305_57231, &P22376_304};	/*Property = FileSaver*/
const Value V22376_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22376_306 = {S26, 0, &V22376_306_57237, &P22376_305};	/*Property = FileWriter*/
const Value V22376_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22376_307 = {S27, 0, &V22376_307_57239, &P22376_306};	/*Property = FormData*/
const Value V22376_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22376_309 = {S28, 0, &V22376_309_57246, &P22376_307};	/*Property = Fullscreen*/
const Value V22376_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22376_310 = {S29, 0, &V22376_310_57248, &P22376_309};	/*Property = GeoLocation*/
const Value V22376_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22376_312 = {S30, 0, &V22376_312_57254, &P22376_310};	/*Property = History*/
const Value V22376_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22376_314 = {S31, 0, &V22376_314_57261, &P22376_312};	/*Property = Html5*/
const Value V22376_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22376_313 = {S32, 0, &V22376_313_57258, &P22376_314};	/*Property = Html-Media-Capture*/
const Value V22376_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22376_315 = {S33, 0, &V22376_315_57264, &P22376_313};	/*Property = Iframe*/
const Value V22376_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22376_316 = {S34, 0, &V22376_316_57266, &P22376_315};	/*Property = IndexedDB*/
const Value V22376_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22376_317 = {S35, 0, &V22376_317_57269, &P22376_316};	/*Property = Json*/
const Value V22376_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22376_15 = {S37, 0, &V22376_15_28, &P22376_317};	/*Property = LayoutEngine*/
const Value V22376_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22376_344 = {S38, 0, &V22376_344_57272, &P22376_15};	/*Property = Masking*/
const Value V22376_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22376_320 = {S39, 0, &V22376_320_57290, &P22376_344};	/*Property = PostMessage*/
const Value V22376_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22376_321 = {S40, 0, &V22376_321_57293, &P22376_320};	/*Property = Progress*/
const Value V22376_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22376_322 = {S41, 0, &V22376_322_57296, &P22376_321};	/*Property = Prompts*/
const Value V22376_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22376_323 = {S42, 0, &V22376_323_57299, &P22376_322};	/*Property = Selector*/
const Value V22376_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22376_324 = {S43, 0, &V22376_324_57306, &P22376_323};	/*Property = Svg*/
const Value V22376_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22376_327 = {S44, 0, &V22376_327_57315, &P22376_324};	/*Property = TouchEvents*/
const Value V22376_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22376_328 = {S45, 0, &V22376_328_57318, &P22376_327};	/*Property = Track*/
const Value V22376_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22376_329 = {S46, 0, &V22376_329_57323, &P22376_328};	/*Property = Video*/
const Value V22376_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22376_330 = {S47, 0, &V22376_330_57326, &P22376_329};	/*Property = Viewport*/
const Value V22376_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22376_332 = {S48, 0, &V22376_332_57340, &P22376_330};	/*Property = WebWorkers*/
const Value V22376_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22376_333 = {S49, 0, &V22376_333_57343, &P22376_332};	/*Property = Xhr2*/
const Profile P22376 = {"22376", &P22376_333};	/*Profile ID - 22376*/
const Value V19961_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19961_279 = {S1, 0, &V19961_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19961_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19961_282 = {S3, 0, &V19961_282_57154, &P19961_279};	/*Property = BlobBuilder*/
const Value V19961_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19961_350 = {S4, 0, &V19961_350_57444, &P19961_282};	/*Property = Canvas*/
const Value V19961_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19961_283 = {S5, 0, &V19961_283_57159, &P19961_350};	/*Property = CssBackground*/
const Value V19961_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19961_284 = {S6, 0, &V19961_284_57163, &P19961_283};	/*Property = CssBorderImage*/
const Value V19961_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19961_285 = {S7, 0, &V19961_285_57166, &P19961_284};	/*Property = CssCanvas*/
const Value V19961_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19961_286 = {S8, 0, &V19961_286_57168, &P19961_285};	/*Property = CssColor*/
const Value V19961_237_55784 = {S2, NULL};	/*Value = True*/
const Property P19961_237 = {S9, 0, &V19961_237_55784, &P19961_286};	/*Property = CssColumn*/
const Value V19961_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19961_287 = {S10, 0, &V19961_287_57172, &P19961_237};	/*Property = CssFlexbox*/
const Value V19961_288_57175 = {S0, NULL};	/*Value = False*/
const Property P19961_288 = {S11, 0, &V19961_288_57175, &P19961_287};	/*Property = CssFont*/
const Value V19961_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19961_289 = {S12, 0, &V19961_289_57759, &P19961_288};	/*Property = CssImages*/
const Value V19961_291_57181 = {S0, NULL};	/*Value = False*/
const Property P19961_291 = {S13, 0, &V19961_291_57181, &P19961_289};	/*Property = CssMediaQueries*/
const Value V19961_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19961_292 = {S14, 0, &V19961_292_57183, &P19961_291};	/*Property = CssMinMax*/
const Value V19961_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19961_293 = {S15, 0, &V19961_293_57187, &P19961_292};	/*Property = CssOverflow*/
const Value V19961_294_57190 = {S0, NULL};	/*Value = False*/
const Property P19961_294 = {S16, 0, &V19961_294_57190, &P19961_293};	/*Property = CssPosition*/
const Value V19961_295_57193 = {S0, NULL};	/*Value = False*/
const Property P19961_295 = {S17, 0, &V19961_295_57193, &P19961_294};	/*Property = CssText*/
const Value V19961_297_57196 = {S0, NULL};	/*Value = False*/
const Property P19961_297 = {S18, 0, &V19961_297_57196, &P19961_295};	/*Property = CssTransforms*/
const Value V19961_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19961_241 = {S19, 0, &V19961_241_55799, &P19961_297};	/*Property = CssTransitions*/
const Value V19961_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19961_298 = {S20, 0, &V19961_298_57206, &P19961_241};	/*Property = CssUI*/
const Value V19961_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19961_300 = {S21, 0, &V19961_300_57216, &P19961_298};	/*Property = DataSet*/
const Value V19961_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19961_301 = {S22, 0, &V19961_301_57218, &P19961_300};	/*Property = DataUrl*/
const Value V19961_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19961_302 = {S23, 0, &V19961_302_57222, &P19961_301};	/*Property = DeviceOrientation*/
const Value V19961_304_57228 = {S0, NULL};	/*Value = False*/
const Property P19961_304 = {S24, 0, &V19961_304_57228, &P19961_302};	/*Property = FileReader*/
const Value V19961_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19961_305 = {S25, 0, &V19961_305_57231, &P19961_304};	/*Property = FileSaver*/
const Value V19961_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19961_306 = {S26, 0, &V19961_306_57237, &P19961_305};	/*Property = FileWriter*/
const Value V19961_307_57240 = {S0, NULL};	/*Value = False*/
const Property P19961_307 = {S27, 0, &V19961_307_57240, &P19961_306};	/*Property = FormData*/
const Value V19961_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19961_309 = {S28, 0, &V19961_309_57246, &P19961_307};	/*Property = Fullscreen*/
const Value V19961_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19961_310 = {S29, 0, &V19961_310_57248, &P19961_309};	/*Property = GeoLocation*/
const Value V19961_312_57255 = {S0, NULL};	/*Value = False*/
const Property P19961_312 = {S30, 0, &V19961_312_57255, &P19961_310};	/*Property = History*/
const Value V19961_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19961_314 = {S31, 0, &V19961_314_57261, &P19961_312};	/*Property = Html5*/
const Value V19961_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19961_313 = {S32, 0, &V19961_313_57258, &P19961_314};	/*Property = Html-Media-Capture*/
const Value V19961_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19961_315 = {S33, 0, &V19961_315_57264, &P19961_313};	/*Property = Iframe*/
const Value V19961_316_57267 = {S0, NULL};	/*Value = False*/
const Property P19961_316 = {S34, 0, &V19961_316_57267, &P19961_315};	/*Property = IndexedDB*/
const Value V19961_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19961_317 = {S35, 0, &V19961_317_57269, &P19961_316};	/*Property = Json*/
const Value V19961_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P19961_15 = {S37, 0, &V19961_15_31, &P19961_317};	/*Property = LayoutEngine*/
const Value V19961_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19961_344 = {S38, 0, &V19961_344_57272, &P19961_15};	/*Property = Masking*/
const Value V19961_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19961_320 = {S39, 0, &V19961_320_57290, &P19961_344};	/*Property = PostMessage*/
const Value V19961_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19961_321 = {S40, 0, &V19961_321_57293, &P19961_320};	/*Property = Progress*/
const Value V19961_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19961_322 = {S41, 0, &V19961_322_57296, &P19961_321};	/*Property = Prompts*/
const Value V19961_323_57300 = {S0, NULL};	/*Value = False*/
const Property P19961_323 = {S42, 0, &V19961_323_57300, &P19961_322};	/*Property = Selector*/
const Value V19961_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19961_324 = {S43, 0, &V19961_324_57306, &P19961_323};	/*Property = Svg*/
const Value V19961_327_57315 = {S0, NULL};	/*Value = False*/
const Property P19961_327 = {S44, 0, &V19961_327_57315, &P19961_324};	/*Property = TouchEvents*/
const Value V19961_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19961_328 = {S45, 0, &V19961_328_57318, &P19961_327};	/*Property = Track*/
const Value V19961_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19961_329 = {S46, 0, &V19961_329_57323, &P19961_328};	/*Property = Video*/
const Value V19961_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19961_330 = {S47, 0, &V19961_330_57326, &P19961_329};	/*Property = Viewport*/
const Value V19961_332_57339 = {S0, NULL};	/*Value = False*/
const Property P19961_332 = {S48, 0, &V19961_332_57339, &P19961_330};	/*Property = WebWorkers*/
const Value V19961_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19961_333 = {S49, 0, &V19961_333_57342, &P19961_332};	/*Property = Xhr2*/
const Profile P19961 = {"19961", &P19961_333};	/*Profile ID - 19961*/
const Value V15989_1_1 = {S2, NULL};	/*Value = True*/
const Property P15989_1 = {S50, 0, &V15989_1_1, NULL};	/*Property = IsMobile*/
const Value V15989_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P15989_5 = {S52, 0, &V15989_5_1277, &P15989_1};	/*Property = ScreenPixelsHeight*/
const Value V15989_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P15989_4 = {S54, 0, &V15989_4_1279, &P15989_5};	/*Property = ScreenPixelsWidth*/
const Profile P15989 = {"15989", &P15989_4};	/*Profile ID - 15989*/
const Value V18531_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18531_279 = {S1, 0, &V18531_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18531_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18531_282 = {S3, 0, &V18531_282_57154, &P18531_279};	/*Property = BlobBuilder*/
const Value V18531_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18531_350 = {S4, 0, &V18531_350_57444, &P18531_282};	/*Property = Canvas*/
const Value V18531_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18531_283 = {S5, 0, &V18531_283_57159, &P18531_350};	/*Property = CssBackground*/
const Value V18531_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18531_284 = {S6, 0, &V18531_284_57163, &P18531_283};	/*Property = CssBorderImage*/
const Value V18531_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18531_285 = {S7, 0, &V18531_285_57166, &P18531_284};	/*Property = CssCanvas*/
const Value V18531_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18531_286 = {S8, 0, &V18531_286_57168, &P18531_285};	/*Property = CssColor*/
const Value V18531_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18531_237 = {S9, 0, &V18531_237_55786, &P18531_286};	/*Property = CssColumn*/
const Value V18531_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18531_287 = {S10, 0, &V18531_287_57172, &P18531_237};	/*Property = CssFlexbox*/
const Value V18531_288_57174 = {S2, NULL};	/*Value = True*/
const Property P18531_288 = {S11, 0, &V18531_288_57174, &P18531_287};	/*Property = CssFont*/
const Value V18531_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18531_289 = {S12, 0, &V18531_289_57759, &P18531_288};	/*Property = CssImages*/
const Value V18531_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18531_291 = {S13, 0, &V18531_291_57180, &P18531_289};	/*Property = CssMediaQueries*/
const Value V18531_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18531_292 = {S14, 0, &V18531_292_57183, &P18531_291};	/*Property = CssMinMax*/
const Value V18531_293_57186 = {S2, NULL};	/*Value = True*/
const Property P18531_293 = {S15, 0, &V18531_293_57186, &P18531_292};	/*Property = CssOverflow*/
const Value V18531_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18531_294 = {S16, 0, &V18531_294_57189, &P18531_293};	/*Property = CssPosition*/
const Value V18531_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18531_295 = {S17, 0, &V18531_295_57192, &P18531_294};	/*Property = CssText*/
const Value V18531_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18531_297 = {S18, 0, &V18531_297_57195, &P18531_295};	/*Property = CssTransforms*/
const Value V18531_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18531_241 = {S19, 0, &V18531_241_55799, &P18531_297};	/*Property = CssTransitions*/
const Value V18531_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18531_298 = {S20, 0, &V18531_298_57206, &P18531_241};	/*Property = CssUI*/
const Value V18531_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18531_300 = {S21, 0, &V18531_300_57216, &P18531_298};	/*Property = DataSet*/
const Value V18531_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18531_301 = {S22, 0, &V18531_301_57218, &P18531_300};	/*Property = DataUrl*/
const Value V18531_302_57221 = {S2, NULL};	/*Value = True*/
const Property P18531_302 = {S23, 0, &V18531_302_57221, &P18531_301};	/*Property = DeviceOrientation*/
const Value V18531_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18531_304 = {S24, 0, &V18531_304_57228, &P18531_302};	/*Property = FileReader*/
const Value V18531_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18531_305 = {S25, 0, &V18531_305_57231, &P18531_304};	/*Property = FileSaver*/
const Value V18531_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18531_306 = {S26, 0, &V18531_306_57237, &P18531_305};	/*Property = FileWriter*/
const Value V18531_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18531_307 = {S27, 0, &V18531_307_57239, &P18531_306};	/*Property = FormData*/
const Value V18531_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18531_309 = {S28, 0, &V18531_309_57246, &P18531_307};	/*Property = Fullscreen*/
const Value V18531_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18531_310 = {S29, 0, &V18531_310_57248, &P18531_309};	/*Property = GeoLocation*/
const Value V18531_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18531_312 = {S30, 0, &V18531_312_57255, &P18531_310};	/*Property = History*/
const Value V18531_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18531_314 = {S31, 0, &V18531_314_57261, &P18531_312};	/*Property = Html5*/
const Value V18531_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18531_313 = {S32, 0, &V18531_313_57258, &P18531_314};	/*Property = Html-Media-Capture*/
const Value V18531_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18531_315 = {S33, 0, &V18531_315_57264, &P18531_313};	/*Property = Iframe*/
const Value V18531_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18531_316 = {S34, 0, &V18531_316_57267, &P18531_315};	/*Property = IndexedDB*/
const Value V18531_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18531_317 = {S35, 0, &V18531_317_57269, &P18531_316};	/*Property = Json*/
const Value V18531_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18531_15 = {S37, 0, &V18531_15_28, &P18531_317};	/*Property = LayoutEngine*/
const Value V18531_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18531_344 = {S38, 0, &V18531_344_57272, &P18531_15};	/*Property = Masking*/
const Value V18531_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18531_320 = {S39, 0, &V18531_320_57290, &P18531_344};	/*Property = PostMessage*/
const Value V18531_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18531_321 = {S40, 0, &V18531_321_57293, &P18531_320};	/*Property = Progress*/
const Value V18531_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18531_322 = {S41, 0, &V18531_322_57296, &P18531_321};	/*Property = Prompts*/
const Value V18531_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18531_323 = {S42, 0, &V18531_323_57299, &P18531_322};	/*Property = Selector*/
const Value V18531_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18531_324 = {S43, 0, &V18531_324_57306, &P18531_323};	/*Property = Svg*/
const Value V18531_327_57314 = {S2, NULL};	/*Value = True*/
const Property P18531_327 = {S44, 0, &V18531_327_57314, &P18531_324};	/*Property = TouchEvents*/
const Value V18531_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18531_328 = {S45, 0, &V18531_328_57318, &P18531_327};	/*Property = Track*/
const Value V18531_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18531_329 = {S46, 0, &V18531_329_57323, &P18531_328};	/*Property = Video*/
const Value V18531_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18531_330 = {S47, 0, &V18531_330_57326, &P18531_329};	/*Property = Viewport*/
const Value V18531_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18531_332 = {S48, 0, &V18531_332_57339, &P18531_330};	/*Property = WebWorkers*/
const Value V18531_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18531_333 = {S49, 0, &V18531_333_57342, &P18531_332};	/*Property = Xhr2*/
const Profile P18531 = {"18531", &P18531_333};	/*Profile ID - 18531*/
const Value V24437_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_279 = {S1, 0, &V24437_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24437_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_282 = {S3, 0, &V24437_282_57155, &P24437_279};	/*Property = BlobBuilder*/
const Value V24437_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_350 = {S4, 0, &V24437_350_57446, &P24437_282};	/*Property = Canvas*/
const Value V24437_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_283 = {S5, 0, &V24437_283_57161, &P24437_350};	/*Property = CssBackground*/
const Value V24437_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_284 = {S6, 0, &V24437_284_57164, &P24437_283};	/*Property = CssBorderImage*/
const Value V24437_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_285 = {S7, 0, &V24437_285_57167, &P24437_284};	/*Property = CssCanvas*/
const Value V24437_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_286 = {S8, 0, &V24437_286_57170, &P24437_285};	/*Property = CssColor*/
const Value V24437_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_237 = {S9, 0, &V24437_237_55786, &P24437_286};	/*Property = CssColumn*/
const Value V24437_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_287 = {S10, 0, &V24437_287_57173, &P24437_237};	/*Property = CssFlexbox*/
const Value V24437_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_288 = {S11, 0, &V24437_288_57176, &P24437_287};	/*Property = CssFont*/
const Value V24437_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_289 = {S12, 0, &V24437_289_86899, &P24437_288};	/*Property = CssImages*/
const Value V24437_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_291 = {S13, 0, &V24437_291_57182, &P24437_289};	/*Property = CssMediaQueries*/
const Value V24437_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_292 = {S14, 0, &V24437_292_57185, &P24437_291};	/*Property = CssMinMax*/
const Value V24437_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_293 = {S15, 0, &V24437_293_57188, &P24437_292};	/*Property = CssOverflow*/
const Value V24437_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_294 = {S16, 0, &V24437_294_57191, &P24437_293};	/*Property = CssPosition*/
const Value V24437_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_295 = {S17, 0, &V24437_295_57194, &P24437_294};	/*Property = CssText*/
const Value V24437_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_297 = {S18, 0, &V24437_297_57197, &P24437_295};	/*Property = CssTransforms*/
const Value V24437_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_241 = {S19, 0, &V24437_241_55801, &P24437_297};	/*Property = CssTransitions*/
const Value V24437_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_298 = {S20, 0, &V24437_298_57208, &P24437_241};	/*Property = CssUI*/
const Value V24437_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_300 = {S21, 0, &V24437_300_57217, &P24437_298};	/*Property = DataSet*/
const Value V24437_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_301 = {S22, 0, &V24437_301_57220, &P24437_300};	/*Property = DataUrl*/
const Value V24437_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_302 = {S23, 0, &V24437_302_57223, &P24437_301};	/*Property = DeviceOrientation*/
const Value V24437_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_304 = {S24, 0, &V24437_304_57229, &P24437_302};	/*Property = FileReader*/
const Value V24437_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_305 = {S25, 0, &V24437_305_57232, &P24437_304};	/*Property = FileSaver*/
const Value V24437_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_306 = {S26, 0, &V24437_306_57238, &P24437_305};	/*Property = FileWriter*/
const Value V24437_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_307 = {S27, 0, &V24437_307_57241, &P24437_306};	/*Property = FormData*/
const Value V24437_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_309 = {S28, 0, &V24437_309_57247, &P24437_307};	/*Property = Fullscreen*/
const Value V24437_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_310 = {S29, 0, &V24437_310_57250, &P24437_309};	/*Property = GeoLocation*/
const Value V24437_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_312 = {S30, 0, &V24437_312_57256, &P24437_310};	/*Property = History*/
const Value V24437_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_314 = {S31, 0, &V24437_314_57262, &P24437_312};	/*Property = Html5*/
const Value V24437_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_313 = {S32, 0, &V24437_313_57259, &P24437_314};	/*Property = Html-Media-Capture*/
const Value V24437_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_315 = {S33, 0, &V24437_315_57265, &P24437_313};	/*Property = Iframe*/
const Value V24437_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_316 = {S34, 0, &V24437_316_57268, &P24437_315};	/*Property = IndexedDB*/
const Value V24437_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_317 = {S35, 0, &V24437_317_57271, &P24437_316};	/*Property = Json*/
const Value V24437_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24437_15 = {S37, 0, &V24437_15_28, &P24437_317};	/*Property = LayoutEngine*/
const Value V24437_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_344 = {S38, 0, &V24437_344_57274, &P24437_15};	/*Property = Masking*/
const Value V24437_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_320 = {S39, 0, &V24437_320_57292, &P24437_344};	/*Property = PostMessage*/
const Value V24437_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_321 = {S40, 0, &V24437_321_57295, &P24437_320};	/*Property = Progress*/
const Value V24437_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_322 = {S41, 0, &V24437_322_57298, &P24437_321};	/*Property = Prompts*/
const Value V24437_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_323 = {S42, 0, &V24437_323_57301, &P24437_322};	/*Property = Selector*/
const Value V24437_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_324 = {S43, 0, &V24437_324_57307, &P24437_323};	/*Property = Svg*/
const Value V24437_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_327 = {S44, 0, &V24437_327_57316, &P24437_324};	/*Property = TouchEvents*/
const Value V24437_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_328 = {S45, 0, &V24437_328_57319, &P24437_327};	/*Property = Track*/
const Value V24437_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_329 = {S46, 0, &V24437_329_57325, &P24437_328};	/*Property = Video*/
const Value V24437_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_330 = {S47, 0, &V24437_330_57328, &P24437_329};	/*Property = Viewport*/
const Value V24437_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_332 = {S48, 0, &V24437_332_57340, &P24437_330};	/*Property = WebWorkers*/
const Value V24437_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24437_333 = {S49, 0, &V24437_333_57343, &P24437_332};	/*Property = Xhr2*/
const Profile P24437 = {"24437", &P24437_333};	/*Profile ID - 24437*/
const Value V22070_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_279 = {S1, 0, &V22070_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22070_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_282 = {S3, 0, &V22070_282_57155, &P22070_279};	/*Property = BlobBuilder*/
const Value V22070_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_350 = {S4, 0, &V22070_350_57446, &P22070_282};	/*Property = Canvas*/
const Value V22070_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_283 = {S5, 0, &V22070_283_57161, &P22070_350};	/*Property = CssBackground*/
const Value V22070_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_284 = {S6, 0, &V22070_284_57164, &P22070_283};	/*Property = CssBorderImage*/
const Value V22070_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_285 = {S7, 0, &V22070_285_57167, &P22070_284};	/*Property = CssCanvas*/
const Value V22070_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_286 = {S8, 0, &V22070_286_57170, &P22070_285};	/*Property = CssColor*/
const Value V22070_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_237 = {S9, 0, &V22070_237_55786, &P22070_286};	/*Property = CssColumn*/
const Value V22070_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_287 = {S10, 0, &V22070_287_57173, &P22070_237};	/*Property = CssFlexbox*/
const Value V22070_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_288 = {S11, 0, &V22070_288_57176, &P22070_287};	/*Property = CssFont*/
const Value V22070_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_289 = {S12, 0, &V22070_289_86899, &P22070_288};	/*Property = CssImages*/
const Value V22070_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_291 = {S13, 0, &V22070_291_57182, &P22070_289};	/*Property = CssMediaQueries*/
const Value V22070_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_292 = {S14, 0, &V22070_292_57185, &P22070_291};	/*Property = CssMinMax*/
const Value V22070_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_293 = {S15, 0, &V22070_293_57188, &P22070_292};	/*Property = CssOverflow*/
const Value V22070_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_294 = {S16, 0, &V22070_294_57191, &P22070_293};	/*Property = CssPosition*/
const Value V22070_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_295 = {S17, 0, &V22070_295_57194, &P22070_294};	/*Property = CssText*/
const Value V22070_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_297 = {S18, 0, &V22070_297_57197, &P22070_295};	/*Property = CssTransforms*/
const Value V22070_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_241 = {S19, 0, &V22070_241_55801, &P22070_297};	/*Property = CssTransitions*/
const Value V22070_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_298 = {S20, 0, &V22070_298_57208, &P22070_241};	/*Property = CssUI*/
const Value V22070_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_300 = {S21, 0, &V22070_300_57217, &P22070_298};	/*Property = DataSet*/
const Value V22070_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_301 = {S22, 0, &V22070_301_57220, &P22070_300};	/*Property = DataUrl*/
const Value V22070_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_302 = {S23, 0, &V22070_302_57223, &P22070_301};	/*Property = DeviceOrientation*/
const Value V22070_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_304 = {S24, 0, &V22070_304_57229, &P22070_302};	/*Property = FileReader*/
const Value V22070_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_305 = {S25, 0, &V22070_305_57232, &P22070_304};	/*Property = FileSaver*/
const Value V22070_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_306 = {S26, 0, &V22070_306_57238, &P22070_305};	/*Property = FileWriter*/
const Value V22070_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_307 = {S27, 0, &V22070_307_57241, &P22070_306};	/*Property = FormData*/
const Value V22070_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_309 = {S28, 0, &V22070_309_57247, &P22070_307};	/*Property = Fullscreen*/
const Value V22070_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_310 = {S29, 0, &V22070_310_57250, &P22070_309};	/*Property = GeoLocation*/
const Value V22070_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_312 = {S30, 0, &V22070_312_57256, &P22070_310};	/*Property = History*/
const Value V22070_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_314 = {S31, 0, &V22070_314_57262, &P22070_312};	/*Property = Html5*/
const Value V22070_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_313 = {S32, 0, &V22070_313_57259, &P22070_314};	/*Property = Html-Media-Capture*/
const Value V22070_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_315 = {S33, 0, &V22070_315_57265, &P22070_313};	/*Property = Iframe*/
const Value V22070_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_316 = {S34, 0, &V22070_316_57268, &P22070_315};	/*Property = IndexedDB*/
const Value V22070_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_317 = {S35, 0, &V22070_317_57271, &P22070_316};	/*Property = Json*/
const Value V22070_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_15 = {S37, 0, &V22070_15_72, &P22070_317};	/*Property = LayoutEngine*/
const Value V22070_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_344 = {S38, 0, &V22070_344_57274, &P22070_15};	/*Property = Masking*/
const Value V22070_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_320 = {S39, 0, &V22070_320_57292, &P22070_344};	/*Property = PostMessage*/
const Value V22070_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_321 = {S40, 0, &V22070_321_57295, &P22070_320};	/*Property = Progress*/
const Value V22070_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_322 = {S41, 0, &V22070_322_57298, &P22070_321};	/*Property = Prompts*/
const Value V22070_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_323 = {S42, 0, &V22070_323_57301, &P22070_322};	/*Property = Selector*/
const Value V22070_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_324 = {S43, 0, &V22070_324_57307, &P22070_323};	/*Property = Svg*/
const Value V22070_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_327 = {S44, 0, &V22070_327_57316, &P22070_324};	/*Property = TouchEvents*/
const Value V22070_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_328 = {S45, 0, &V22070_328_57319, &P22070_327};	/*Property = Track*/
const Value V22070_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_329 = {S46, 0, &V22070_329_57325, &P22070_328};	/*Property = Video*/
const Value V22070_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_330 = {S47, 0, &V22070_330_57328, &P22070_329};	/*Property = Viewport*/
const Value V22070_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_332 = {S48, 0, &V22070_332_57340, &P22070_330};	/*Property = WebWorkers*/
const Value V22070_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22070_333 = {S49, 0, &V22070_333_57343, &P22070_332};	/*Property = Xhr2*/
const Profile P22070 = {"22070", &P22070_333};	/*Profile ID - 22070*/
const Value V19846_1_1 = {S2, NULL};	/*Value = True*/
const Property P19846_1 = {S50, 0, &V19846_1_1, NULL};	/*Property = IsMobile*/
const Value V19846_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P19846_5 = {S52, 0, &V19846_5_1403, &P19846_1};	/*Property = ScreenPixelsHeight*/
const Value V19846_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P19846_4 = {S54, 0, &V19846_4_1404, &P19846_5};	/*Property = ScreenPixelsWidth*/
const Profile P19846 = {"19846", &P19846_4};	/*Profile ID - 19846*/
const Value V21759_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_279 = {S1, 0, &V21759_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21759_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_282 = {S3, 0, &V21759_282_57155, &P21759_279};	/*Property = BlobBuilder*/
const Value V21759_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_350 = {S4, 0, &V21759_350_57446, &P21759_282};	/*Property = Canvas*/
const Value V21759_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_283 = {S5, 0, &V21759_283_57161, &P21759_350};	/*Property = CssBackground*/
const Value V21759_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_284 = {S6, 0, &V21759_284_57164, &P21759_283};	/*Property = CssBorderImage*/
const Value V21759_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_285 = {S7, 0, &V21759_285_57167, &P21759_284};	/*Property = CssCanvas*/
const Value V21759_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_286 = {S8, 0, &V21759_286_57170, &P21759_285};	/*Property = CssColor*/
const Value V21759_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_237 = {S9, 0, &V21759_237_55786, &P21759_286};	/*Property = CssColumn*/
const Value V21759_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_287 = {S10, 0, &V21759_287_57173, &P21759_237};	/*Property = CssFlexbox*/
const Value V21759_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_288 = {S11, 0, &V21759_288_57176, &P21759_287};	/*Property = CssFont*/
const Value V21759_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_289 = {S12, 0, &V21759_289_86899, &P21759_288};	/*Property = CssImages*/
const Value V21759_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_291 = {S13, 0, &V21759_291_57182, &P21759_289};	/*Property = CssMediaQueries*/
const Value V21759_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_292 = {S14, 0, &V21759_292_57185, &P21759_291};	/*Property = CssMinMax*/
const Value V21759_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_293 = {S15, 0, &V21759_293_57188, &P21759_292};	/*Property = CssOverflow*/
const Value V21759_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_294 = {S16, 0, &V21759_294_57191, &P21759_293};	/*Property = CssPosition*/
const Value V21759_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_295 = {S17, 0, &V21759_295_57194, &P21759_294};	/*Property = CssText*/
const Value V21759_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_297 = {S18, 0, &V21759_297_57197, &P21759_295};	/*Property = CssTransforms*/
const Value V21759_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_241 = {S19, 0, &V21759_241_55801, &P21759_297};	/*Property = CssTransitions*/
const Value V21759_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_298 = {S20, 0, &V21759_298_57208, &P21759_241};	/*Property = CssUI*/
const Value V21759_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_300 = {S21, 0, &V21759_300_57217, &P21759_298};	/*Property = DataSet*/
const Value V21759_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_301 = {S22, 0, &V21759_301_57220, &P21759_300};	/*Property = DataUrl*/
const Value V21759_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_302 = {S23, 0, &V21759_302_57223, &P21759_301};	/*Property = DeviceOrientation*/
const Value V21759_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_304 = {S24, 0, &V21759_304_57229, &P21759_302};	/*Property = FileReader*/
const Value V21759_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_305 = {S25, 0, &V21759_305_57232, &P21759_304};	/*Property = FileSaver*/
const Value V21759_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_306 = {S26, 0, &V21759_306_57238, &P21759_305};	/*Property = FileWriter*/
const Value V21759_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_307 = {S27, 0, &V21759_307_57241, &P21759_306};	/*Property = FormData*/
const Value V21759_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_309 = {S28, 0, &V21759_309_57247, &P21759_307};	/*Property = Fullscreen*/
const Value V21759_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_310 = {S29, 0, &V21759_310_57250, &P21759_309};	/*Property = GeoLocation*/
const Value V21759_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_312 = {S30, 0, &V21759_312_57256, &P21759_310};	/*Property = History*/
const Value V21759_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_314 = {S31, 0, &V21759_314_57262, &P21759_312};	/*Property = Html5*/
const Value V21759_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_313 = {S32, 0, &V21759_313_57259, &P21759_314};	/*Property = Html-Media-Capture*/
const Value V21759_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_315 = {S33, 0, &V21759_315_57265, &P21759_313};	/*Property = Iframe*/
const Value V21759_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_316 = {S34, 0, &V21759_316_57268, &P21759_315};	/*Property = IndexedDB*/
const Value V21759_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_317 = {S35, 0, &V21759_317_57271, &P21759_316};	/*Property = Json*/
const Value V21759_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21759_15 = {S37, 0, &V21759_15_54167, &P21759_317};	/*Property = LayoutEngine*/
const Value V21759_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_344 = {S38, 0, &V21759_344_57274, &P21759_15};	/*Property = Masking*/
const Value V21759_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_320 = {S39, 0, &V21759_320_57292, &P21759_344};	/*Property = PostMessage*/
const Value V21759_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_321 = {S40, 0, &V21759_321_57295, &P21759_320};	/*Property = Progress*/
const Value V21759_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_322 = {S41, 0, &V21759_322_57298, &P21759_321};	/*Property = Prompts*/
const Value V21759_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_323 = {S42, 0, &V21759_323_57301, &P21759_322};	/*Property = Selector*/
const Value V21759_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_324 = {S43, 0, &V21759_324_57307, &P21759_323};	/*Property = Svg*/
const Value V21759_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_327 = {S44, 0, &V21759_327_57316, &P21759_324};	/*Property = TouchEvents*/
const Value V21759_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_328 = {S45, 0, &V21759_328_57319, &P21759_327};	/*Property = Track*/
const Value V21759_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_329 = {S46, 0, &V21759_329_57325, &P21759_328};	/*Property = Video*/
const Value V21759_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_330 = {S47, 0, &V21759_330_57328, &P21759_329};	/*Property = Viewport*/
const Value V21759_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_332 = {S48, 0, &V21759_332_57340, &P21759_330};	/*Property = WebWorkers*/
const Value V21759_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21759_333 = {S49, 0, &V21759_333_57343, &P21759_332};	/*Property = Xhr2*/
const Profile P21759 = {"21759", &P21759_333};	/*Profile ID - 21759*/
const Value V24673_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_279 = {S1, 0, &V24673_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24673_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_282 = {S3, 0, &V24673_282_57155, &P24673_279};	/*Property = BlobBuilder*/
const Value V24673_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_350 = {S4, 0, &V24673_350_57446, &P24673_282};	/*Property = Canvas*/
const Value V24673_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_283 = {S5, 0, &V24673_283_57161, &P24673_350};	/*Property = CssBackground*/
const Value V24673_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_284 = {S6, 0, &V24673_284_57164, &P24673_283};	/*Property = CssBorderImage*/
const Value V24673_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_285 = {S7, 0, &V24673_285_57167, &P24673_284};	/*Property = CssCanvas*/
const Value V24673_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_286 = {S8, 0, &V24673_286_57170, &P24673_285};	/*Property = CssColor*/
const Value V24673_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_237 = {S9, 0, &V24673_237_55786, &P24673_286};	/*Property = CssColumn*/
const Value V24673_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_287 = {S10, 0, &V24673_287_57173, &P24673_237};	/*Property = CssFlexbox*/
const Value V24673_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_288 = {S11, 0, &V24673_288_57176, &P24673_287};	/*Property = CssFont*/
const Value V24673_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_289 = {S12, 0, &V24673_289_86899, &P24673_288};	/*Property = CssImages*/
const Value V24673_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_291 = {S13, 0, &V24673_291_57182, &P24673_289};	/*Property = CssMediaQueries*/
const Value V24673_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_292 = {S14, 0, &V24673_292_57185, &P24673_291};	/*Property = CssMinMax*/
const Value V24673_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_293 = {S15, 0, &V24673_293_57188, &P24673_292};	/*Property = CssOverflow*/
const Value V24673_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_294 = {S16, 0, &V24673_294_57191, &P24673_293};	/*Property = CssPosition*/
const Value V24673_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_295 = {S17, 0, &V24673_295_57194, &P24673_294};	/*Property = CssText*/
const Value V24673_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_297 = {S18, 0, &V24673_297_57197, &P24673_295};	/*Property = CssTransforms*/
const Value V24673_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_241 = {S19, 0, &V24673_241_55801, &P24673_297};	/*Property = CssTransitions*/
const Value V24673_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_298 = {S20, 0, &V24673_298_57208, &P24673_241};	/*Property = CssUI*/
const Value V24673_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_300 = {S21, 0, &V24673_300_57217, &P24673_298};	/*Property = DataSet*/
const Value V24673_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_301 = {S22, 0, &V24673_301_57220, &P24673_300};	/*Property = DataUrl*/
const Value V24673_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_302 = {S23, 0, &V24673_302_57223, &P24673_301};	/*Property = DeviceOrientation*/
const Value V24673_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_304 = {S24, 0, &V24673_304_57229, &P24673_302};	/*Property = FileReader*/
const Value V24673_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_305 = {S25, 0, &V24673_305_57232, &P24673_304};	/*Property = FileSaver*/
const Value V24673_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_306 = {S26, 0, &V24673_306_57238, &P24673_305};	/*Property = FileWriter*/
const Value V24673_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_307 = {S27, 0, &V24673_307_57241, &P24673_306};	/*Property = FormData*/
const Value V24673_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_309 = {S28, 0, &V24673_309_57247, &P24673_307};	/*Property = Fullscreen*/
const Value V24673_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_310 = {S29, 0, &V24673_310_57250, &P24673_309};	/*Property = GeoLocation*/
const Value V24673_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_312 = {S30, 0, &V24673_312_57256, &P24673_310};	/*Property = History*/
const Value V24673_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_314 = {S31, 0, &V24673_314_57262, &P24673_312};	/*Property = Html5*/
const Value V24673_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_313 = {S32, 0, &V24673_313_57259, &P24673_314};	/*Property = Html-Media-Capture*/
const Value V24673_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_315 = {S33, 0, &V24673_315_57265, &P24673_313};	/*Property = Iframe*/
const Value V24673_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_316 = {S34, 0, &V24673_316_57268, &P24673_315};	/*Property = IndexedDB*/
const Value V24673_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_317 = {S35, 0, &V24673_317_57271, &P24673_316};	/*Property = Json*/
const Value V24673_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24673_15 = {S37, 0, &V24673_15_90837, &P24673_317};	/*Property = LayoutEngine*/
const Value V24673_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_344 = {S38, 0, &V24673_344_57274, &P24673_15};	/*Property = Masking*/
const Value V24673_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_320 = {S39, 0, &V24673_320_57292, &P24673_344};	/*Property = PostMessage*/
const Value V24673_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_321 = {S40, 0, &V24673_321_57295, &P24673_320};	/*Property = Progress*/
const Value V24673_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_322 = {S41, 0, &V24673_322_57298, &P24673_321};	/*Property = Prompts*/
const Value V24673_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_323 = {S42, 0, &V24673_323_57301, &P24673_322};	/*Property = Selector*/
const Value V24673_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_324 = {S43, 0, &V24673_324_57307, &P24673_323};	/*Property = Svg*/
const Value V24673_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_327 = {S44, 0, &V24673_327_57316, &P24673_324};	/*Property = TouchEvents*/
const Value V24673_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_328 = {S45, 0, &V24673_328_57319, &P24673_327};	/*Property = Track*/
const Value V24673_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_329 = {S46, 0, &V24673_329_57325, &P24673_328};	/*Property = Video*/
const Value V24673_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_330 = {S47, 0, &V24673_330_57328, &P24673_329};	/*Property = Viewport*/
const Value V24673_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_332 = {S48, 0, &V24673_332_57340, &P24673_330};	/*Property = WebWorkers*/
const Value V24673_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24673_333 = {S49, 0, &V24673_333_57343, &P24673_332};	/*Property = Xhr2*/
const Profile P24673 = {"24673", &P24673_333};	/*Profile ID - 24673*/
const Value V22436_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22436_279 = {S1, 0, &V22436_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22436_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22436_282 = {S3, 0, &V22436_282_57154, &P22436_279};	/*Property = BlobBuilder*/
const Value V22436_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22436_350 = {S4, 0, &V22436_350_57444, &P22436_282};	/*Property = Canvas*/
const Value V22436_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22436_283 = {S5, 0, &V22436_283_57159, &P22436_350};	/*Property = CssBackground*/
const Value V22436_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22436_284 = {S6, 0, &V22436_284_57163, &P22436_283};	/*Property = CssBorderImage*/
const Value V22436_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22436_285 = {S7, 0, &V22436_285_57166, &P22436_284};	/*Property = CssCanvas*/
const Value V22436_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22436_286 = {S8, 0, &V22436_286_57168, &P22436_285};	/*Property = CssColor*/
const Value V22436_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22436_237 = {S9, 0, &V22436_237_55786, &P22436_286};	/*Property = CssColumn*/
const Value V22436_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22436_287 = {S10, 0, &V22436_287_57172, &P22436_237};	/*Property = CssFlexbox*/
const Value V22436_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22436_288 = {S11, 0, &V22436_288_57174, &P22436_287};	/*Property = CssFont*/
const Value V22436_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22436_289 = {S12, 0, &V22436_289_57759, &P22436_288};	/*Property = CssImages*/
const Value V22436_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22436_291 = {S13, 0, &V22436_291_57180, &P22436_289};	/*Property = CssMediaQueries*/
const Value V22436_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22436_292 = {S14, 0, &V22436_292_57183, &P22436_291};	/*Property = CssMinMax*/
const Value V22436_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22436_293 = {S15, 0, &V22436_293_57187, &P22436_292};	/*Property = CssOverflow*/
const Value V22436_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22436_294 = {S16, 0, &V22436_294_57189, &P22436_293};	/*Property = CssPosition*/
const Value V22436_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22436_295 = {S17, 0, &V22436_295_57192, &P22436_294};	/*Property = CssText*/
const Value V22436_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22436_297 = {S18, 0, &V22436_297_57195, &P22436_295};	/*Property = CssTransforms*/
const Value V22436_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22436_241 = {S19, 0, &V22436_241_55799, &P22436_297};	/*Property = CssTransitions*/
const Value V22436_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22436_298 = {S20, 0, &V22436_298_57206, &P22436_241};	/*Property = CssUI*/
const Value V22436_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22436_300 = {S21, 0, &V22436_300_57216, &P22436_298};	/*Property = DataSet*/
const Value V22436_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22436_301 = {S22, 0, &V22436_301_57218, &P22436_300};	/*Property = DataUrl*/
const Value V22436_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22436_302 = {S23, 0, &V22436_302_57222, &P22436_301};	/*Property = DeviceOrientation*/
const Value V22436_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22436_304 = {S24, 0, &V22436_304_57227, &P22436_302};	/*Property = FileReader*/
const Value V22436_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22436_305 = {S25, 0, &V22436_305_57231, &P22436_304};	/*Property = FileSaver*/
const Value V22436_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22436_306 = {S26, 0, &V22436_306_57237, &P22436_305};	/*Property = FileWriter*/
const Value V22436_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22436_307 = {S27, 0, &V22436_307_57239, &P22436_306};	/*Property = FormData*/
const Value V22436_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22436_309 = {S28, 0, &V22436_309_57246, &P22436_307};	/*Property = Fullscreen*/
const Value V22436_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22436_310 = {S29, 0, &V22436_310_57248, &P22436_309};	/*Property = GeoLocation*/
const Value V22436_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22436_312 = {S30, 0, &V22436_312_57254, &P22436_310};	/*Property = History*/
const Value V22436_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22436_314 = {S31, 0, &V22436_314_57261, &P22436_312};	/*Property = Html5*/
const Value V22436_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22436_313 = {S32, 0, &V22436_313_57258, &P22436_314};	/*Property = Html-Media-Capture*/
const Value V22436_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22436_315 = {S33, 0, &V22436_315_57264, &P22436_313};	/*Property = Iframe*/
const Value V22436_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22436_316 = {S34, 0, &V22436_316_57266, &P22436_315};	/*Property = IndexedDB*/
const Value V22436_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22436_317 = {S35, 0, &V22436_317_57269, &P22436_316};	/*Property = Json*/
const Value V22436_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22436_15 = {S37, 0, &V22436_15_28, &P22436_317};	/*Property = LayoutEngine*/
const Value V22436_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22436_344 = {S38, 0, &V22436_344_57272, &P22436_15};	/*Property = Masking*/
const Value V22436_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22436_320 = {S39, 0, &V22436_320_57290, &P22436_344};	/*Property = PostMessage*/
const Value V22436_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22436_321 = {S40, 0, &V22436_321_57293, &P22436_320};	/*Property = Progress*/
const Value V22436_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22436_322 = {S41, 0, &V22436_322_57296, &P22436_321};	/*Property = Prompts*/
const Value V22436_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22436_323 = {S42, 0, &V22436_323_57299, &P22436_322};	/*Property = Selector*/
const Value V22436_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22436_324 = {S43, 0, &V22436_324_57306, &P22436_323};	/*Property = Svg*/
const Value V22436_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22436_327 = {S44, 0, &V22436_327_57315, &P22436_324};	/*Property = TouchEvents*/
const Value V22436_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22436_328 = {S45, 0, &V22436_328_57318, &P22436_327};	/*Property = Track*/
const Value V22436_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22436_329 = {S46, 0, &V22436_329_57323, &P22436_328};	/*Property = Video*/
const Value V22436_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22436_330 = {S47, 0, &V22436_330_57326, &P22436_329};	/*Property = Viewport*/
const Value V22436_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22436_332 = {S48, 0, &V22436_332_57338, &P22436_330};	/*Property = WebWorkers*/
const Value V22436_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22436_333 = {S49, 0, &V22436_333_57341, &P22436_332};	/*Property = Xhr2*/
const Profile P22436 = {"22436", &P22436_333};	/*Profile ID - 22436*/
const Value V20787_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_279 = {S1, 0, &V20787_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20787_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_282 = {S3, 0, &V20787_282_57155, &P20787_279};	/*Property = BlobBuilder*/
const Value V20787_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_350 = {S4, 0, &V20787_350_57446, &P20787_282};	/*Property = Canvas*/
const Value V20787_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_283 = {S5, 0, &V20787_283_57161, &P20787_350};	/*Property = CssBackground*/
const Value V20787_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_284 = {S6, 0, &V20787_284_57164, &P20787_283};	/*Property = CssBorderImage*/
const Value V20787_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_285 = {S7, 0, &V20787_285_57167, &P20787_284};	/*Property = CssCanvas*/
const Value V20787_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_286 = {S8, 0, &V20787_286_57170, &P20787_285};	/*Property = CssColor*/
const Value V20787_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_237 = {S9, 0, &V20787_237_55786, &P20787_286};	/*Property = CssColumn*/
const Value V20787_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_287 = {S10, 0, &V20787_287_57173, &P20787_237};	/*Property = CssFlexbox*/
const Value V20787_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_288 = {S11, 0, &V20787_288_57176, &P20787_287};	/*Property = CssFont*/
const Value V20787_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_289 = {S12, 0, &V20787_289_86899, &P20787_288};	/*Property = CssImages*/
const Value V20787_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_291 = {S13, 0, &V20787_291_57182, &P20787_289};	/*Property = CssMediaQueries*/
const Value V20787_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_292 = {S14, 0, &V20787_292_57185, &P20787_291};	/*Property = CssMinMax*/
const Value V20787_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_293 = {S15, 0, &V20787_293_57188, &P20787_292};	/*Property = CssOverflow*/
const Value V20787_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_294 = {S16, 0, &V20787_294_57191, &P20787_293};	/*Property = CssPosition*/
const Value V20787_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_295 = {S17, 0, &V20787_295_57194, &P20787_294};	/*Property = CssText*/
const Value V20787_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_297 = {S18, 0, &V20787_297_57197, &P20787_295};	/*Property = CssTransforms*/
const Value V20787_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_241 = {S19, 0, &V20787_241_55801, &P20787_297};	/*Property = CssTransitions*/
const Value V20787_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_298 = {S20, 0, &V20787_298_57208, &P20787_241};	/*Property = CssUI*/
const Value V20787_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_300 = {S21, 0, &V20787_300_57217, &P20787_298};	/*Property = DataSet*/
const Value V20787_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_301 = {S22, 0, &V20787_301_57220, &P20787_300};	/*Property = DataUrl*/
const Value V20787_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_302 = {S23, 0, &V20787_302_57223, &P20787_301};	/*Property = DeviceOrientation*/
const Value V20787_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_304 = {S24, 0, &V20787_304_57229, &P20787_302};	/*Property = FileReader*/
const Value V20787_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_305 = {S25, 0, &V20787_305_57232, &P20787_304};	/*Property = FileSaver*/
const Value V20787_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_306 = {S26, 0, &V20787_306_57238, &P20787_305};	/*Property = FileWriter*/
const Value V20787_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_307 = {S27, 0, &V20787_307_57241, &P20787_306};	/*Property = FormData*/
const Value V20787_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_309 = {S28, 0, &V20787_309_57247, &P20787_307};	/*Property = Fullscreen*/
const Value V20787_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_310 = {S29, 0, &V20787_310_57250, &P20787_309};	/*Property = GeoLocation*/
const Value V20787_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_312 = {S30, 0, &V20787_312_57256, &P20787_310};	/*Property = History*/
const Value V20787_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_314 = {S31, 0, &V20787_314_57262, &P20787_312};	/*Property = Html5*/
const Value V20787_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_313 = {S32, 0, &V20787_313_57259, &P20787_314};	/*Property = Html-Media-Capture*/
const Value V20787_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_315 = {S33, 0, &V20787_315_57265, &P20787_313};	/*Property = Iframe*/
const Value V20787_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_316 = {S34, 0, &V20787_316_57268, &P20787_315};	/*Property = IndexedDB*/
const Value V20787_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_317 = {S35, 0, &V20787_317_57271, &P20787_316};	/*Property = Json*/
const Value V20787_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P20787_15 = {S37, 0, &V20787_15_54167, &P20787_317};	/*Property = LayoutEngine*/
const Value V20787_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_344 = {S38, 0, &V20787_344_57274, &P20787_15};	/*Property = Masking*/
const Value V20787_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_320 = {S39, 0, &V20787_320_57292, &P20787_344};	/*Property = PostMessage*/
const Value V20787_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_321 = {S40, 0, &V20787_321_57295, &P20787_320};	/*Property = Progress*/
const Value V20787_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_322 = {S41, 0, &V20787_322_57298, &P20787_321};	/*Property = Prompts*/
const Value V20787_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_323 = {S42, 0, &V20787_323_57301, &P20787_322};	/*Property = Selector*/
const Value V20787_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_324 = {S43, 0, &V20787_324_57307, &P20787_323};	/*Property = Svg*/
const Value V20787_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_327 = {S44, 0, &V20787_327_57316, &P20787_324};	/*Property = TouchEvents*/
const Value V20787_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_328 = {S45, 0, &V20787_328_57319, &P20787_327};	/*Property = Track*/
const Value V20787_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_329 = {S46, 0, &V20787_329_57325, &P20787_328};	/*Property = Video*/
const Value V20787_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_330 = {S47, 0, &V20787_330_57328, &P20787_329};	/*Property = Viewport*/
const Value V20787_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_332 = {S48, 0, &V20787_332_57340, &P20787_330};	/*Property = WebWorkers*/
const Value V20787_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20787_333 = {S49, 0, &V20787_333_57343, &P20787_332};	/*Property = Xhr2*/
const Profile P20787 = {"20787", &P20787_333};	/*Profile ID - 20787*/
const Profile P18164 = {"18164", NULL};	/*Profile ID - 18164*/
const Value V21811_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21811_279 = {S1, 0, &V21811_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21811_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21811_282 = {S3, 0, &V21811_282_57154, &P21811_279};	/*Property = BlobBuilder*/
const Value V21811_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21811_350 = {S4, 0, &V21811_350_57444, &P21811_282};	/*Property = Canvas*/
const Value V21811_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21811_283 = {S5, 0, &V21811_283_57159, &P21811_350};	/*Property = CssBackground*/
const Value V21811_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21811_284 = {S6, 0, &V21811_284_57163, &P21811_283};	/*Property = CssBorderImage*/
const Value V21811_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21811_285 = {S7, 0, &V21811_285_57166, &P21811_284};	/*Property = CssCanvas*/
const Value V21811_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21811_286 = {S8, 0, &V21811_286_57168, &P21811_285};	/*Property = CssColor*/
const Value V21811_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21811_237 = {S9, 0, &V21811_237_55786, &P21811_286};	/*Property = CssColumn*/
const Value V21811_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21811_287 = {S10, 0, &V21811_287_57172, &P21811_237};	/*Property = CssFlexbox*/
const Value V21811_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21811_288 = {S11, 0, &V21811_288_57175, &P21811_287};	/*Property = CssFont*/
const Value V21811_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21811_289 = {S12, 0, &V21811_289_57759, &P21811_288};	/*Property = CssImages*/
const Value V21811_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21811_291 = {S13, 0, &V21811_291_57181, &P21811_289};	/*Property = CssMediaQueries*/
const Value V21811_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21811_292 = {S14, 0, &V21811_292_57183, &P21811_291};	/*Property = CssMinMax*/
const Value V21811_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21811_293 = {S15, 0, &V21811_293_57187, &P21811_292};	/*Property = CssOverflow*/
const Value V21811_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21811_294 = {S16, 0, &V21811_294_57190, &P21811_293};	/*Property = CssPosition*/
const Value V21811_295_57193 = {S0, NULL};	/*Value = False*/
const Property P21811_295 = {S17, 0, &V21811_295_57193, &P21811_294};	/*Property = CssText*/
const Value V21811_297_57196 = {S0, NULL};	/*Value = False*/
const Property P21811_297 = {S18, 0, &V21811_297_57196, &P21811_295};	/*Property = CssTransforms*/
const Value V21811_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21811_241 = {S19, 0, &V21811_241_55799, &P21811_297};	/*Property = CssTransitions*/
const Value V21811_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21811_298 = {S20, 0, &V21811_298_57206, &P21811_241};	/*Property = CssUI*/
const Value V21811_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21811_300 = {S21, 0, &V21811_300_57216, &P21811_298};	/*Property = DataSet*/
const Value V21811_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21811_301 = {S22, 0, &V21811_301_57218, &P21811_300};	/*Property = DataUrl*/
const Value V21811_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21811_302 = {S23, 0, &V21811_302_57222, &P21811_301};	/*Property = DeviceOrientation*/
const Value V21811_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21811_304 = {S24, 0, &V21811_304_57228, &P21811_302};	/*Property = FileReader*/
const Value V21811_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21811_305 = {S25, 0, &V21811_305_57231, &P21811_304};	/*Property = FileSaver*/
const Value V21811_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21811_306 = {S26, 0, &V21811_306_57237, &P21811_305};	/*Property = FileWriter*/
const Value V21811_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21811_307 = {S27, 0, &V21811_307_57240, &P21811_306};	/*Property = FormData*/
const Value V21811_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21811_309 = {S28, 0, &V21811_309_57246, &P21811_307};	/*Property = Fullscreen*/
const Value V21811_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21811_310 = {S29, 0, &V21811_310_57248, &P21811_309};	/*Property = GeoLocation*/
const Value V21811_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21811_312 = {S30, 0, &V21811_312_57255, &P21811_310};	/*Property = History*/
const Value V21811_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21811_314 = {S31, 0, &V21811_314_57261, &P21811_312};	/*Property = Html5*/
const Value V21811_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21811_313 = {S32, 0, &V21811_313_57258, &P21811_314};	/*Property = Html-Media-Capture*/
const Value V21811_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21811_315 = {S33, 0, &V21811_315_57264, &P21811_313};	/*Property = Iframe*/
const Value V21811_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21811_316 = {S34, 0, &V21811_316_57267, &P21811_315};	/*Property = IndexedDB*/
const Value V21811_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21811_317 = {S35, 0, &V21811_317_57269, &P21811_316};	/*Property = Json*/
const Value V21811_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P21811_15 = {S37, 0, &V21811_15_31, &P21811_317};	/*Property = LayoutEngine*/
const Value V21811_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21811_344 = {S38, 0, &V21811_344_57272, &P21811_15};	/*Property = Masking*/
const Value V21811_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21811_320 = {S39, 0, &V21811_320_57290, &P21811_344};	/*Property = PostMessage*/
const Value V21811_321_57294 = {S0, NULL};	/*Value = False*/
const Property P21811_321 = {S40, 0, &V21811_321_57294, &P21811_320};	/*Property = Progress*/
const Value V21811_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21811_322 = {S41, 0, &V21811_322_57296, &P21811_321};	/*Property = Prompts*/
const Value V21811_323_57300 = {S0, NULL};	/*Value = False*/
const Property P21811_323 = {S42, 0, &V21811_323_57300, &P21811_322};	/*Property = Selector*/
const Value V21811_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21811_324 = {S43, 0, &V21811_324_57306, &P21811_323};	/*Property = Svg*/
const Value V21811_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21811_327 = {S44, 0, &V21811_327_57315, &P21811_324};	/*Property = TouchEvents*/
const Value V21811_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21811_328 = {S45, 0, &V21811_328_57318, &P21811_327};	/*Property = Track*/
const Value V21811_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21811_329 = {S46, 0, &V21811_329_57323, &P21811_328};	/*Property = Video*/
const Value V21811_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21811_330 = {S47, 0, &V21811_330_57326, &P21811_329};	/*Property = Viewport*/
const Value V21811_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21811_332 = {S48, 0, &V21811_332_57339, &P21811_330};	/*Property = WebWorkers*/
const Value V21811_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21811_333 = {S49, 0, &V21811_333_57342, &P21811_332};	/*Property = Xhr2*/
const Profile P21811 = {"21811", &P21811_333};	/*Profile ID - 21811*/
const Value V18644_1_1 = {S2, NULL};	/*Value = True*/
const Property P18644_1 = {S50, 0, &V18644_1_1, NULL};	/*Property = IsMobile*/
const Value V18644_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P18644_5 = {S52, 0, &V18644_5_1293, &P18644_1};	/*Property = ScreenPixelsHeight*/
const Value V18644_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P18644_4 = {S54, 0, &V18644_4_1296, &P18644_5};	/*Property = ScreenPixelsWidth*/
const Profile P18644 = {"18644", &P18644_4};	/*Profile ID - 18644*/
const Value V19791_1_1 = {S2, NULL};	/*Value = True*/
const Property P19791_1 = {S50, 0, &V19791_1_1, NULL};	/*Property = IsMobile*/
const Value V19791_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P19791_5 = {S52, 0, &V19791_5_1269, &P19791_1};	/*Property = ScreenPixelsHeight*/
const Value V19791_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19791_4 = {S54, 0, &V19791_4_1229, &P19791_5};	/*Property = ScreenPixelsWidth*/
const Profile P19791 = {"19791", &P19791_4};	/*Profile ID - 19791*/
const Value V20140_1_1 = {S2, NULL};	/*Value = True*/
const Property P20140_1 = {S50, 0, &V20140_1_1, NULL};	/*Property = IsMobile*/
const Value V20140_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20140_5 = {S52, 0, &V20140_5_1458, &P20140_1};	/*Property = ScreenPixelsHeight*/
const Value V20140_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20140_4 = {S54, 0, &V20140_4_37617, &P20140_5};	/*Property = ScreenPixelsWidth*/
const Profile P20140 = {"20140", &P20140_4};	/*Profile ID - 20140*/
const Value V20333_1_2 = {S0, NULL};	/*Value = False*/
const Property P20333_1 = {S50, 0, &V20333_1_2, NULL};	/*Property = IsMobile*/
const Value V20333_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20333_5 = {S52, 0, &V20333_5_37, &P20333_1};	/*Property = ScreenPixelsHeight*/
const Value V20333_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20333_4 = {S54, 0, &V20333_4_36, &P20333_5};	/*Property = ScreenPixelsWidth*/
const Profile P20333 = {"20333", &P20333_4};	/*Profile ID - 20333*/
const Profile P22221 = {"22221", NULL};	/*Profile ID - 22221*/
const Value V18552_1_1 = {S2, NULL};	/*Value = True*/
const Property P18552_1 = {S50, 0, &V18552_1_1, NULL};	/*Property = IsMobile*/
const Value V18552_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18552_5 = {S52, 0, &V18552_5_1403, &P18552_1};	/*Property = ScreenPixelsHeight*/
const Value V18552_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18552_4 = {S54, 0, &V18552_4_1404, &P18552_5};	/*Property = ScreenPixelsWidth*/
const Profile P18552 = {"18552", &P18552_4};	/*Profile ID - 18552*/
const Value V18521_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18521_279 = {S1, 0, &V18521_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18521_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18521_282 = {S3, 0, &V18521_282_57154, &P18521_279};	/*Property = BlobBuilder*/
const Value V18521_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18521_350 = {S4, 0, &V18521_350_57444, &P18521_282};	/*Property = Canvas*/
const Value V18521_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18521_283 = {S5, 0, &V18521_283_57159, &P18521_350};	/*Property = CssBackground*/
const Value V18521_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18521_284 = {S6, 0, &V18521_284_57163, &P18521_283};	/*Property = CssBorderImage*/
const Value V18521_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18521_285 = {S7, 0, &V18521_285_57166, &P18521_284};	/*Property = CssCanvas*/
const Value V18521_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18521_286 = {S8, 0, &V18521_286_57168, &P18521_285};	/*Property = CssColor*/
const Value V18521_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18521_237 = {S9, 0, &V18521_237_55786, &P18521_286};	/*Property = CssColumn*/
const Value V18521_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18521_287 = {S10, 0, &V18521_287_57172, &P18521_237};	/*Property = CssFlexbox*/
const Value V18521_288_57174 = {S2, NULL};	/*Value = True*/
const Property P18521_288 = {S11, 0, &V18521_288_57174, &P18521_287};	/*Property = CssFont*/
const Value V18521_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18521_289 = {S12, 0, &V18521_289_57759, &P18521_288};	/*Property = CssImages*/
const Value V18521_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18521_291 = {S13, 0, &V18521_291_57180, &P18521_289};	/*Property = CssMediaQueries*/
const Value V18521_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18521_292 = {S14, 0, &V18521_292_57183, &P18521_291};	/*Property = CssMinMax*/
const Value V18521_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18521_293 = {S15, 0, &V18521_293_57187, &P18521_292};	/*Property = CssOverflow*/
const Value V18521_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18521_294 = {S16, 0, &V18521_294_57189, &P18521_293};	/*Property = CssPosition*/
const Value V18521_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18521_295 = {S17, 0, &V18521_295_57192, &P18521_294};	/*Property = CssText*/
const Value V18521_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18521_297 = {S18, 0, &V18521_297_57195, &P18521_295};	/*Property = CssTransforms*/
const Value V18521_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18521_241 = {S19, 0, &V18521_241_55799, &P18521_297};	/*Property = CssTransitions*/
const Value V18521_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18521_298 = {S20, 0, &V18521_298_57206, &P18521_241};	/*Property = CssUI*/
const Value V18521_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18521_300 = {S21, 0, &V18521_300_57216, &P18521_298};	/*Property = DataSet*/
const Value V18521_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18521_301 = {S22, 0, &V18521_301_57218, &P18521_300};	/*Property = DataUrl*/
const Value V18521_302_57221 = {S2, NULL};	/*Value = True*/
const Property P18521_302 = {S23, 0, &V18521_302_57221, &P18521_301};	/*Property = DeviceOrientation*/
const Value V18521_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18521_304 = {S24, 0, &V18521_304_57227, &P18521_302};	/*Property = FileReader*/
const Value V18521_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18521_305 = {S25, 0, &V18521_305_57231, &P18521_304};	/*Property = FileSaver*/
const Value V18521_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18521_306 = {S26, 0, &V18521_306_57237, &P18521_305};	/*Property = FileWriter*/
const Value V18521_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18521_307 = {S27, 0, &V18521_307_57239, &P18521_306};	/*Property = FormData*/
const Value V18521_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18521_309 = {S28, 0, &V18521_309_57246, &P18521_307};	/*Property = Fullscreen*/
const Value V18521_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18521_310 = {S29, 0, &V18521_310_57248, &P18521_309};	/*Property = GeoLocation*/
const Value V18521_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18521_312 = {S30, 0, &V18521_312_57254, &P18521_310};	/*Property = History*/
const Value V18521_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18521_314 = {S31, 0, &V18521_314_57261, &P18521_312};	/*Property = Html5*/
const Value V18521_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18521_313 = {S32, 0, &V18521_313_57258, &P18521_314};	/*Property = Html-Media-Capture*/
const Value V18521_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18521_315 = {S33, 0, &V18521_315_57264, &P18521_313};	/*Property = Iframe*/
const Value V18521_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18521_316 = {S34, 0, &V18521_316_57266, &P18521_315};	/*Property = IndexedDB*/
const Value V18521_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18521_317 = {S35, 0, &V18521_317_57269, &P18521_316};	/*Property = Json*/
const Value V18521_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18521_15 = {S37, 0, &V18521_15_29, &P18521_317};	/*Property = LayoutEngine*/
const Value V18521_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18521_344 = {S38, 0, &V18521_344_57272, &P18521_15};	/*Property = Masking*/
const Value V18521_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18521_320 = {S39, 0, &V18521_320_57290, &P18521_344};	/*Property = PostMessage*/
const Value V18521_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18521_321 = {S40, 0, &V18521_321_57293, &P18521_320};	/*Property = Progress*/
const Value V18521_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18521_322 = {S41, 0, &V18521_322_57296, &P18521_321};	/*Property = Prompts*/
const Value V18521_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18521_323 = {S42, 0, &V18521_323_57299, &P18521_322};	/*Property = Selector*/
const Value V18521_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18521_324 = {S43, 0, &V18521_324_57306, &P18521_323};	/*Property = Svg*/
const Value V18521_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18521_327 = {S44, 0, &V18521_327_57315, &P18521_324};	/*Property = TouchEvents*/
const Value V18521_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18521_328 = {S45, 0, &V18521_328_57318, &P18521_327};	/*Property = Track*/
const Value V18521_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18521_329 = {S46, 0, &V18521_329_57323, &P18521_328};	/*Property = Video*/
const Value V18521_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18521_330 = {S47, 0, &V18521_330_57326, &P18521_329};	/*Property = Viewport*/
const Value V18521_332_57338 = {S2, NULL};	/*Value = True*/
const Property P18521_332 = {S48, 0, &V18521_332_57338, &P18521_330};	/*Property = WebWorkers*/
const Value V18521_333_57341 = {S2, NULL};	/*Value = True*/
const Property P18521_333 = {S49, 0, &V18521_333_57341, &P18521_332};	/*Property = Xhr2*/
const Profile P18521 = {"18521", &P18521_333};	/*Profile ID - 18521*/
const Value V19986_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19986_279 = {S1, 0, &V19986_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19986_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19986_282 = {S3, 0, &V19986_282_57154, &P19986_279};	/*Property = BlobBuilder*/
const Value V19986_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19986_350 = {S4, 0, &V19986_350_57444, &P19986_282};	/*Property = Canvas*/
const Value V19986_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19986_283 = {S5, 0, &V19986_283_57159, &P19986_350};	/*Property = CssBackground*/
const Value V19986_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19986_284 = {S6, 0, &V19986_284_57163, &P19986_283};	/*Property = CssBorderImage*/
const Value V19986_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19986_285 = {S7, 0, &V19986_285_57166, &P19986_284};	/*Property = CssCanvas*/
const Value V19986_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19986_286 = {S8, 0, &V19986_286_57168, &P19986_285};	/*Property = CssColor*/
const Value V19986_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19986_237 = {S9, 0, &V19986_237_55786, &P19986_286};	/*Property = CssColumn*/
const Value V19986_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19986_287 = {S10, 0, &V19986_287_57172, &P19986_237};	/*Property = CssFlexbox*/
const Value V19986_288_57174 = {S2, NULL};	/*Value = True*/
const Property P19986_288 = {S11, 0, &V19986_288_57174, &P19986_287};	/*Property = CssFont*/
const Value V19986_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19986_289 = {S12, 0, &V19986_289_57759, &P19986_288};	/*Property = CssImages*/
const Value V19986_291_57180 = {S2, NULL};	/*Value = True*/
const Property P19986_291 = {S13, 0, &V19986_291_57180, &P19986_289};	/*Property = CssMediaQueries*/
const Value V19986_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19986_292 = {S14, 0, &V19986_292_57183, &P19986_291};	/*Property = CssMinMax*/
const Value V19986_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19986_293 = {S15, 0, &V19986_293_57187, &P19986_292};	/*Property = CssOverflow*/
const Value V19986_294_57189 = {S2, NULL};	/*Value = True*/
const Property P19986_294 = {S16, 0, &V19986_294_57189, &P19986_293};	/*Property = CssPosition*/
const Value V19986_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19986_295 = {S17, 0, &V19986_295_57192, &P19986_294};	/*Property = CssText*/
const Value V19986_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19986_297 = {S18, 0, &V19986_297_57195, &P19986_295};	/*Property = CssTransforms*/
const Value V19986_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19986_241 = {S19, 0, &V19986_241_55799, &P19986_297};	/*Property = CssTransitions*/
const Value V19986_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19986_298 = {S20, 0, &V19986_298_57206, &P19986_241};	/*Property = CssUI*/
const Value V19986_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19986_300 = {S21, 0, &V19986_300_57216, &P19986_298};	/*Property = DataSet*/
const Value V19986_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19986_301 = {S22, 0, &V19986_301_57218, &P19986_300};	/*Property = DataUrl*/
const Value V19986_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19986_302 = {S23, 0, &V19986_302_57222, &P19986_301};	/*Property = DeviceOrientation*/
const Value V19986_304_57227 = {S2, NULL};	/*Value = True*/
const Property P19986_304 = {S24, 0, &V19986_304_57227, &P19986_302};	/*Property = FileReader*/
const Value V19986_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19986_305 = {S25, 0, &V19986_305_57231, &P19986_304};	/*Property = FileSaver*/
const Value V19986_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19986_306 = {S26, 0, &V19986_306_57237, &P19986_305};	/*Property = FileWriter*/
const Value V19986_307_57239 = {S2, NULL};	/*Value = True*/
const Property P19986_307 = {S27, 0, &V19986_307_57239, &P19986_306};	/*Property = FormData*/
const Value V19986_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19986_309 = {S28, 0, &V19986_309_57246, &P19986_307};	/*Property = Fullscreen*/
const Value V19986_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19986_310 = {S29, 0, &V19986_310_57248, &P19986_309};	/*Property = GeoLocation*/
const Value V19986_312_57254 = {S2, NULL};	/*Value = True*/
const Property P19986_312 = {S30, 0, &V19986_312_57254, &P19986_310};	/*Property = History*/
const Value V19986_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19986_314 = {S31, 0, &V19986_314_57261, &P19986_312};	/*Property = Html5*/
const Value V19986_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19986_313 = {S32, 0, &V19986_313_57258, &P19986_314};	/*Property = Html-Media-Capture*/
const Value V19986_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19986_315 = {S33, 0, &V19986_315_57264, &P19986_313};	/*Property = Iframe*/
const Value V19986_316_57266 = {S2, NULL};	/*Value = True*/
const Property P19986_316 = {S34, 0, &V19986_316_57266, &P19986_315};	/*Property = IndexedDB*/
const Value V19986_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19986_317 = {S35, 0, &V19986_317_57269, &P19986_316};	/*Property = Json*/
const Value V19986_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P19986_15 = {S37, 0, &V19986_15_30, &P19986_317};	/*Property = LayoutEngine*/
const Value V19986_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19986_344 = {S38, 0, &V19986_344_57272, &P19986_15};	/*Property = Masking*/
const Value V19986_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19986_320 = {S39, 0, &V19986_320_57290, &P19986_344};	/*Property = PostMessage*/
const Value V19986_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19986_321 = {S40, 0, &V19986_321_57293, &P19986_320};	/*Property = Progress*/
const Value V19986_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19986_322 = {S41, 0, &V19986_322_57296, &P19986_321};	/*Property = Prompts*/
const Value V19986_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19986_323 = {S42, 0, &V19986_323_57299, &P19986_322};	/*Property = Selector*/
const Value V19986_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19986_324 = {S43, 0, &V19986_324_57306, &P19986_323};	/*Property = Svg*/
const Value V19986_327_57315 = {S0, NULL};	/*Value = False*/
const Property P19986_327 = {S44, 0, &V19986_327_57315, &P19986_324};	/*Property = TouchEvents*/
const Value V19986_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19986_328 = {S45, 0, &V19986_328_57318, &P19986_327};	/*Property = Track*/
const Value V19986_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19986_329 = {S46, 0, &V19986_329_57323, &P19986_328};	/*Property = Video*/
const Value V19986_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19986_330 = {S47, 0, &V19986_330_57326, &P19986_329};	/*Property = Viewport*/
const Value V19986_332_57338 = {S2, NULL};	/*Value = True*/
const Property P19986_332 = {S48, 0, &V19986_332_57338, &P19986_330};	/*Property = WebWorkers*/
const Value V19986_333_57341 = {S2, NULL};	/*Value = True*/
const Property P19986_333 = {S49, 0, &V19986_333_57341, &P19986_332};	/*Property = Xhr2*/
const Profile P19986 = {"19986", &P19986_333};	/*Profile ID - 19986*/
const Value V20360_1_2 = {S0, NULL};	/*Value = False*/
const Property P20360_1 = {S50, 0, &V20360_1_2, NULL};	/*Property = IsMobile*/
const Value V20360_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20360_5 = {S52, 0, &V20360_5_37, &P20360_1};	/*Property = ScreenPixelsHeight*/
const Value V20360_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20360_4 = {S54, 0, &V20360_4_36, &P20360_5};	/*Property = ScreenPixelsWidth*/
const Profile P20360 = {"20360", &P20360_4};	/*Profile ID - 20360*/
const Value V22363_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22363_279 = {S1, 0, &V22363_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22363_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22363_282 = {S3, 0, &V22363_282_57153, &P22363_279};	/*Property = BlobBuilder*/
const Value V22363_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22363_350 = {S4, 0, &V22363_350_57444, &P22363_282};	/*Property = Canvas*/
const Value V22363_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22363_283 = {S5, 0, &V22363_283_57159, &P22363_350};	/*Property = CssBackground*/
const Value V22363_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22363_284 = {S6, 0, &V22363_284_57163, &P22363_283};	/*Property = CssBorderImage*/
const Value V22363_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22363_285 = {S7, 0, &V22363_285_57166, &P22363_284};	/*Property = CssCanvas*/
const Value V22363_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22363_286 = {S8, 0, &V22363_286_57168, &P22363_285};	/*Property = CssColor*/
const Value V22363_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22363_237 = {S9, 0, &V22363_237_55786, &P22363_286};	/*Property = CssColumn*/
const Value V22363_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22363_287 = {S10, 0, &V22363_287_57172, &P22363_237};	/*Property = CssFlexbox*/
const Value V22363_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22363_288 = {S11, 0, &V22363_288_57175, &P22363_287};	/*Property = CssFont*/
const Value V22363_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22363_289 = {S12, 0, &V22363_289_57759, &P22363_288};	/*Property = CssImages*/
const Value V22363_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22363_291 = {S13, 0, &V22363_291_57181, &P22363_289};	/*Property = CssMediaQueries*/
const Value V22363_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22363_292 = {S14, 0, &V22363_292_57183, &P22363_291};	/*Property = CssMinMax*/
const Value V22363_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22363_293 = {S15, 0, &V22363_293_57187, &P22363_292};	/*Property = CssOverflow*/
const Value V22363_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22363_294 = {S16, 0, &V22363_294_57189, &P22363_293};	/*Property = CssPosition*/
const Value V22363_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22363_295 = {S17, 0, &V22363_295_57192, &P22363_294};	/*Property = CssText*/
const Value V22363_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22363_297 = {S18, 0, &V22363_297_57195, &P22363_295};	/*Property = CssTransforms*/
const Value V22363_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22363_241 = {S19, 0, &V22363_241_55799, &P22363_297};	/*Property = CssTransitions*/
const Value V22363_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22363_298 = {S20, 0, &V22363_298_57206, &P22363_241};	/*Property = CssUI*/
const Value V22363_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22363_300 = {S21, 0, &V22363_300_57216, &P22363_298};	/*Property = DataSet*/
const Value V22363_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22363_301 = {S22, 0, &V22363_301_57218, &P22363_300};	/*Property = DataUrl*/
const Value V22363_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22363_302 = {S23, 0, &V22363_302_57221, &P22363_301};	/*Property = DeviceOrientation*/
const Value V22363_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22363_304 = {S24, 0, &V22363_304_57227, &P22363_302};	/*Property = FileReader*/
const Value V22363_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22363_305 = {S25, 0, &V22363_305_57231, &P22363_304};	/*Property = FileSaver*/
const Value V22363_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22363_306 = {S26, 0, &V22363_306_57237, &P22363_305};	/*Property = FileWriter*/
const Value V22363_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22363_307 = {S27, 0, &V22363_307_57239, &P22363_306};	/*Property = FormData*/
const Value V22363_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22363_309 = {S28, 0, &V22363_309_57246, &P22363_307};	/*Property = Fullscreen*/
const Value V22363_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22363_310 = {S29, 0, &V22363_310_57248, &P22363_309};	/*Property = GeoLocation*/
const Value V22363_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22363_312 = {S30, 0, &V22363_312_57254, &P22363_310};	/*Property = History*/
const Value V22363_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22363_314 = {S31, 0, &V22363_314_57261, &P22363_312};	/*Property = Html5*/
const Value V22363_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22363_313 = {S32, 0, &V22363_313_57258, &P22363_314};	/*Property = Html-Media-Capture*/
const Value V22363_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22363_315 = {S33, 0, &V22363_315_57264, &P22363_313};	/*Property = Iframe*/
const Value V22363_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22363_316 = {S34, 0, &V22363_316_57266, &P22363_315};	/*Property = IndexedDB*/
const Value V22363_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22363_317 = {S35, 0, &V22363_317_57269, &P22363_316};	/*Property = Json*/
const Value V22363_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22363_15 = {S37, 0, &V22363_15_29, &P22363_317};	/*Property = LayoutEngine*/
const Value V22363_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22363_344 = {S38, 0, &V22363_344_57272, &P22363_15};	/*Property = Masking*/
const Value V22363_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22363_320 = {S39, 0, &V22363_320_57290, &P22363_344};	/*Property = PostMessage*/
const Value V22363_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22363_321 = {S40, 0, &V22363_321_57293, &P22363_320};	/*Property = Progress*/
const Value V22363_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22363_322 = {S41, 0, &V22363_322_57296, &P22363_321};	/*Property = Prompts*/
const Value V22363_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22363_323 = {S42, 0, &V22363_323_57299, &P22363_322};	/*Property = Selector*/
const Value V22363_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22363_324 = {S43, 0, &V22363_324_57306, &P22363_323};	/*Property = Svg*/
const Value V22363_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22363_327 = {S44, 0, &V22363_327_57315, &P22363_324};	/*Property = TouchEvents*/
const Value V22363_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22363_328 = {S45, 0, &V22363_328_57318, &P22363_327};	/*Property = Track*/
const Value V22363_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22363_329 = {S46, 0, &V22363_329_57323, &P22363_328};	/*Property = Video*/
const Value V22363_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22363_330 = {S47, 0, &V22363_330_57326, &P22363_329};	/*Property = Viewport*/
const Value V22363_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22363_332 = {S48, 0, &V22363_332_57339, &P22363_330};	/*Property = WebWorkers*/
const Value V22363_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22363_333 = {S49, 0, &V22363_333_57341, &P22363_332};	/*Property = Xhr2*/
const Profile P22363 = {"22363", &P22363_333};	/*Profile ID - 22363*/
const Value V22434_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22434_279 = {S1, 0, &V22434_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22434_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22434_282 = {S3, 0, &V22434_282_57154, &P22434_279};	/*Property = BlobBuilder*/
const Value V22434_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22434_350 = {S4, 0, &V22434_350_57444, &P22434_282};	/*Property = Canvas*/
const Value V22434_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22434_283 = {S5, 0, &V22434_283_57159, &P22434_350};	/*Property = CssBackground*/
const Value V22434_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22434_284 = {S6, 0, &V22434_284_57163, &P22434_283};	/*Property = CssBorderImage*/
const Value V22434_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22434_285 = {S7, 0, &V22434_285_57166, &P22434_284};	/*Property = CssCanvas*/
const Value V22434_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22434_286 = {S8, 0, &V22434_286_57168, &P22434_285};	/*Property = CssColor*/
const Value V22434_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22434_237 = {S9, 0, &V22434_237_55786, &P22434_286};	/*Property = CssColumn*/
const Value V22434_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22434_287 = {S10, 0, &V22434_287_57172, &P22434_237};	/*Property = CssFlexbox*/
const Value V22434_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22434_288 = {S11, 0, &V22434_288_57174, &P22434_287};	/*Property = CssFont*/
const Value V22434_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22434_289 = {S12, 0, &V22434_289_57759, &P22434_288};	/*Property = CssImages*/
const Value V22434_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22434_291 = {S13, 0, &V22434_291_57180, &P22434_289};	/*Property = CssMediaQueries*/
const Value V22434_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22434_292 = {S14, 0, &V22434_292_57183, &P22434_291};	/*Property = CssMinMax*/
const Value V22434_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22434_293 = {S15, 0, &V22434_293_57187, &P22434_292};	/*Property = CssOverflow*/
const Value V22434_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22434_294 = {S16, 0, &V22434_294_57189, &P22434_293};	/*Property = CssPosition*/
const Value V22434_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22434_295 = {S17, 0, &V22434_295_57192, &P22434_294};	/*Property = CssText*/
const Value V22434_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22434_297 = {S18, 0, &V22434_297_57195, &P22434_295};	/*Property = CssTransforms*/
const Value V22434_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22434_241 = {S19, 0, &V22434_241_55799, &P22434_297};	/*Property = CssTransitions*/
const Value V22434_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22434_298 = {S20, 0, &V22434_298_57206, &P22434_241};	/*Property = CssUI*/
const Value V22434_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22434_300 = {S21, 0, &V22434_300_57216, &P22434_298};	/*Property = DataSet*/
const Value V22434_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22434_301 = {S22, 0, &V22434_301_57218, &P22434_300};	/*Property = DataUrl*/
const Value V22434_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22434_302 = {S23, 0, &V22434_302_57222, &P22434_301};	/*Property = DeviceOrientation*/
const Value V22434_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22434_304 = {S24, 0, &V22434_304_57227, &P22434_302};	/*Property = FileReader*/
const Value V22434_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22434_305 = {S25, 0, &V22434_305_57231, &P22434_304};	/*Property = FileSaver*/
const Value V22434_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22434_306 = {S26, 0, &V22434_306_57237, &P22434_305};	/*Property = FileWriter*/
const Value V22434_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22434_307 = {S27, 0, &V22434_307_57239, &P22434_306};	/*Property = FormData*/
const Value V22434_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22434_309 = {S28, 0, &V22434_309_57246, &P22434_307};	/*Property = Fullscreen*/
const Value V22434_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22434_310 = {S29, 0, &V22434_310_57248, &P22434_309};	/*Property = GeoLocation*/
const Value V22434_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22434_312 = {S30, 0, &V22434_312_57254, &P22434_310};	/*Property = History*/
const Value V22434_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22434_314 = {S31, 0, &V22434_314_57261, &P22434_312};	/*Property = Html5*/
const Value V22434_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22434_313 = {S32, 0, &V22434_313_57258, &P22434_314};	/*Property = Html-Media-Capture*/
const Value V22434_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22434_315 = {S33, 0, &V22434_315_57264, &P22434_313};	/*Property = Iframe*/
const Value V22434_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22434_316 = {S34, 0, &V22434_316_57266, &P22434_315};	/*Property = IndexedDB*/
const Value V22434_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22434_317 = {S35, 0, &V22434_317_57269, &P22434_316};	/*Property = Json*/
const Value V22434_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22434_15 = {S37, 0, &V22434_15_28, &P22434_317};	/*Property = LayoutEngine*/
const Value V22434_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22434_344 = {S38, 0, &V22434_344_57272, &P22434_15};	/*Property = Masking*/
const Value V22434_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22434_320 = {S39, 0, &V22434_320_57290, &P22434_344};	/*Property = PostMessage*/
const Value V22434_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22434_321 = {S40, 0, &V22434_321_57293, &P22434_320};	/*Property = Progress*/
const Value V22434_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22434_322 = {S41, 0, &V22434_322_57296, &P22434_321};	/*Property = Prompts*/
const Value V22434_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22434_323 = {S42, 0, &V22434_323_57299, &P22434_322};	/*Property = Selector*/
const Value V22434_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22434_324 = {S43, 0, &V22434_324_57306, &P22434_323};	/*Property = Svg*/
const Value V22434_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22434_327 = {S44, 0, &V22434_327_57315, &P22434_324};	/*Property = TouchEvents*/
const Value V22434_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22434_328 = {S45, 0, &V22434_328_57318, &P22434_327};	/*Property = Track*/
const Value V22434_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22434_329 = {S46, 0, &V22434_329_57323, &P22434_328};	/*Property = Video*/
const Value V22434_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22434_330 = {S47, 0, &V22434_330_57326, &P22434_329};	/*Property = Viewport*/
const Value V22434_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22434_332 = {S48, 0, &V22434_332_57338, &P22434_330};	/*Property = WebWorkers*/
const Value V22434_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22434_333 = {S49, 0, &V22434_333_57341, &P22434_332};	/*Property = Xhr2*/
const Profile P22434 = {"22434", &P22434_333};	/*Profile ID - 22434*/
const Value V21064_1_1 = {S2, NULL};	/*Value = True*/
const Property P21064_1 = {S50, 0, &V21064_1_1, NULL};	/*Property = IsMobile*/
const Value V21064_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21064_5 = {S52, 0, &V21064_5_1222, &P21064_1};	/*Property = ScreenPixelsHeight*/
const Value V21064_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21064_4 = {S54, 0, &V21064_4_1224, &P21064_5};	/*Property = ScreenPixelsWidth*/
const Profile P21064 = {"21064", &P21064_4};	/*Profile ID - 21064*/
const Value V19794_1_1 = {S2, NULL};	/*Value = True*/
const Property P19794_1 = {S50, 0, &V19794_1_1, NULL};	/*Property = IsMobile*/
const Value V19794_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19794_5 = {S52, 0, &V19794_5_9981, &P19794_1};	/*Property = ScreenPixelsHeight*/
const Value V19794_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P19794_4 = {S54, 0, &V19794_4_37617, &P19794_5};	/*Property = ScreenPixelsWidth*/
const Profile P19794 = {"19794", &P19794_4};	/*Profile ID - 19794*/
const Profile P19795 = {"19795", NULL};	/*Profile ID - 19795*/
const Value V19797_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19797_279 = {S1, 0, &V19797_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19797_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19797_282 = {S3, 0, &V19797_282_57154, &P19797_279};	/*Property = BlobBuilder*/
const Value V19797_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19797_350 = {S4, 0, &V19797_350_57444, &P19797_282};	/*Property = Canvas*/
const Value V19797_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19797_283 = {S5, 0, &V19797_283_57159, &P19797_350};	/*Property = CssBackground*/
const Value V19797_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19797_284 = {S6, 0, &V19797_284_57163, &P19797_283};	/*Property = CssBorderImage*/
const Value V19797_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19797_285 = {S7, 0, &V19797_285_57166, &P19797_284};	/*Property = CssCanvas*/
const Value V19797_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19797_286 = {S8, 0, &V19797_286_57168, &P19797_285};	/*Property = CssColor*/
const Value V19797_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19797_237 = {S9, 0, &V19797_237_55786, &P19797_286};	/*Property = CssColumn*/
const Value V19797_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19797_287 = {S10, 0, &V19797_287_57172, &P19797_237};	/*Property = CssFlexbox*/
const Value V19797_288_57174 = {S2, NULL};	/*Value = True*/
const Property P19797_288 = {S11, 0, &V19797_288_57174, &P19797_287};	/*Property = CssFont*/
const Value V19797_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19797_289 = {S12, 0, &V19797_289_57759, &P19797_288};	/*Property = CssImages*/
const Value V19797_291_57180 = {S2, NULL};	/*Value = True*/
const Property P19797_291 = {S13, 0, &V19797_291_57180, &P19797_289};	/*Property = CssMediaQueries*/
const Value V19797_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19797_292 = {S14, 0, &V19797_292_57183, &P19797_291};	/*Property = CssMinMax*/
const Value V19797_293_57186 = {S2, NULL};	/*Value = True*/
const Property P19797_293 = {S15, 0, &V19797_293_57186, &P19797_292};	/*Property = CssOverflow*/
const Value V19797_294_57189 = {S2, NULL};	/*Value = True*/
const Property P19797_294 = {S16, 0, &V19797_294_57189, &P19797_293};	/*Property = CssPosition*/
const Value V19797_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19797_295 = {S17, 0, &V19797_295_57192, &P19797_294};	/*Property = CssText*/
const Value V19797_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19797_297 = {S18, 0, &V19797_297_57195, &P19797_295};	/*Property = CssTransforms*/
const Value V19797_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19797_241 = {S19, 0, &V19797_241_55799, &P19797_297};	/*Property = CssTransitions*/
const Value V19797_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19797_298 = {S20, 0, &V19797_298_57206, &P19797_241};	/*Property = CssUI*/
const Value V19797_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19797_300 = {S21, 0, &V19797_300_57216, &P19797_298};	/*Property = DataSet*/
const Value V19797_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19797_301 = {S22, 0, &V19797_301_57218, &P19797_300};	/*Property = DataUrl*/
const Value V19797_302_57221 = {S2, NULL};	/*Value = True*/
const Property P19797_302 = {S23, 0, &V19797_302_57221, &P19797_301};	/*Property = DeviceOrientation*/
const Value V19797_304_57227 = {S2, NULL};	/*Value = True*/
const Property P19797_304 = {S24, 0, &V19797_304_57227, &P19797_302};	/*Property = FileReader*/
const Value V19797_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19797_305 = {S25, 0, &V19797_305_57231, &P19797_304};	/*Property = FileSaver*/
const Value V19797_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19797_306 = {S26, 0, &V19797_306_57237, &P19797_305};	/*Property = FileWriter*/
const Value V19797_307_57239 = {S2, NULL};	/*Value = True*/
const Property P19797_307 = {S27, 0, &V19797_307_57239, &P19797_306};	/*Property = FormData*/
const Value V19797_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19797_309 = {S28, 0, &V19797_309_57246, &P19797_307};	/*Property = Fullscreen*/
const Value V19797_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19797_310 = {S29, 0, &V19797_310_57248, &P19797_309};	/*Property = GeoLocation*/
const Value V19797_312_57254 = {S2, NULL};	/*Value = True*/
const Property P19797_312 = {S30, 0, &V19797_312_57254, &P19797_310};	/*Property = History*/
const Value V19797_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19797_314 = {S31, 0, &V19797_314_57261, &P19797_312};	/*Property = Html5*/
const Value V19797_313_57257 = {S2, NULL};	/*Value = True*/
const Property P19797_313 = {S32, 0, &V19797_313_57257, &P19797_314};	/*Property = Html-Media-Capture*/
const Value V19797_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19797_315 = {S33, 0, &V19797_315_57264, &P19797_313};	/*Property = Iframe*/
const Value V19797_316_57266 = {S2, NULL};	/*Value = True*/
const Property P19797_316 = {S34, 0, &V19797_316_57266, &P19797_315};	/*Property = IndexedDB*/
const Value V19797_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19797_317 = {S35, 0, &V19797_317_57269, &P19797_316};	/*Property = Json*/
const Value V19797_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19797_15 = {S37, 0, &V19797_15_28, &P19797_317};	/*Property = LayoutEngine*/
const Value V19797_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19797_344 = {S38, 0, &V19797_344_57272, &P19797_15};	/*Property = Masking*/
const Value V19797_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19797_320 = {S39, 0, &V19797_320_57290, &P19797_344};	/*Property = PostMessage*/
const Value V19797_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19797_321 = {S40, 0, &V19797_321_57293, &P19797_320};	/*Property = Progress*/
const Value V19797_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19797_322 = {S41, 0, &V19797_322_57296, &P19797_321};	/*Property = Prompts*/
const Value V19797_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19797_323 = {S42, 0, &V19797_323_57299, &P19797_322};	/*Property = Selector*/
const Value V19797_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19797_324 = {S43, 0, &V19797_324_57306, &P19797_323};	/*Property = Svg*/
const Value V19797_327_57314 = {S2, NULL};	/*Value = True*/
const Property P19797_327 = {S44, 0, &V19797_327_57314, &P19797_324};	/*Property = TouchEvents*/
const Value V19797_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19797_328 = {S45, 0, &V19797_328_57318, &P19797_327};	/*Property = Track*/
const Value V19797_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19797_329 = {S46, 0, &V19797_329_57323, &P19797_328};	/*Property = Video*/
const Value V19797_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19797_330 = {S47, 0, &V19797_330_57326, &P19797_329};	/*Property = Viewport*/
const Value V19797_332_57338 = {S2, NULL};	/*Value = True*/
const Property P19797_332 = {S48, 0, &V19797_332_57338, &P19797_330};	/*Property = WebWorkers*/
const Value V19797_333_57341 = {S2, NULL};	/*Value = True*/
const Property P19797_333 = {S49, 0, &V19797_333_57341, &P19797_332};	/*Property = Xhr2*/
const Profile P19797 = {"19797", &P19797_333};	/*Profile ID - 19797*/
const Value V19754_1_1 = {S2, NULL};	/*Value = True*/
const Property P19754_1 = {S50, 0, &V19754_1_1, NULL};	/*Property = IsMobile*/
const Value V19754_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P19754_5 = {S52, 0, &V19754_5_1458, &P19754_1};	/*Property = ScreenPixelsHeight*/
const Value V19754_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P19754_4 = {S54, 0, &V19754_4_37617, &P19754_5};	/*Property = ScreenPixelsWidth*/
const Profile P19754 = {"19754", &P19754_4};	/*Profile ID - 19754*/
const Value V21039_1_1 = {S2, NULL};	/*Value = True*/
const Property P21039_1 = {S50, 0, &V21039_1_1, NULL};	/*Property = IsMobile*/
const Value V21039_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21039_5 = {S52, 0, &V21039_5_1227, &P21039_1};	/*Property = ScreenPixelsHeight*/
const Value V21039_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21039_4 = {S54, 0, &V21039_4_1229, &P21039_5};	/*Property = ScreenPixelsWidth*/
const Profile P21039 = {"21039", &P21039_4};	/*Profile ID - 21039*/
const Profile P19834 = {"19834", NULL};	/*Profile ID - 19834*/
const Value V19936_1_1 = {S2, NULL};	/*Value = True*/
const Property P19936_1 = {S50, 0, &V19936_1_1, NULL};	/*Property = IsMobile*/
const Value V19936_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P19936_5 = {S52, 0, &V19936_5_1269, &P19936_1};	/*Property = ScreenPixelsHeight*/
const Value V19936_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19936_4 = {S54, 0, &V19936_4_1229, &P19936_5};	/*Property = ScreenPixelsWidth*/
const Profile P19936 = {"19936", &P19936_4};	/*Profile ID - 19936*/
const Value V21117_1_1 = {S2, NULL};	/*Value = True*/
const Property P21117_1 = {S50, 0, &V21117_1_1, NULL};	/*Property = IsMobile*/
const Value V21117_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21117_5 = {S52, 0, &V21117_5_1277, &P21117_1};	/*Property = ScreenPixelsHeight*/
const Value V21117_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21117_4 = {S54, 0, &V21117_4_2336, &P21117_5};	/*Property = ScreenPixelsWidth*/
const Profile P21117 = {"21117", &P21117_4};	/*Profile ID - 21117*/
const Value V21029_1_1 = {S2, NULL};	/*Value = True*/
const Property P21029_1 = {S50, 0, &V21029_1_1, NULL};	/*Property = IsMobile*/
const Value V21029_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21029_5 = {S52, 0, &V21029_5_1458, &P21029_1};	/*Property = ScreenPixelsHeight*/
const Value V21029_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21029_4 = {S54, 0, &V21029_4_37617, &P21029_5};	/*Property = ScreenPixelsWidth*/
const Profile P21029 = {"21029", &P21029_4};	/*Profile ID - 21029*/
const Value V16473_1_1 = {S2, NULL};	/*Value = True*/
const Property P16473_1 = {S50, 0, &V16473_1_1, NULL};	/*Property = IsMobile*/
const Value V16473_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16473_5 = {S52, 0, &V16473_5_1277, &P16473_1};	/*Property = ScreenPixelsHeight*/
const Value V16473_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16473_4 = {S54, 0, &V16473_4_1279, &P16473_5};	/*Property = ScreenPixelsWidth*/
const Profile P16473 = {"16473", &P16473_4};	/*Profile ID - 16473*/
const Value V21688_1_1 = {S2, NULL};	/*Value = True*/
const Property P21688_1 = {S50, 0, &V21688_1_1, NULL};	/*Property = IsMobile*/
const Value V21688_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21688_5 = {S52, 0, &V21688_5_1277, &P21688_1};	/*Property = ScreenPixelsHeight*/
const Value V21688_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21688_4 = {S54, 0, &V21688_4_1279, &P21688_5};	/*Property = ScreenPixelsWidth*/
const Profile P21688 = {"21688", &P21688_4};	/*Profile ID - 21688*/
const Value V21670_1_1 = {S2, NULL};	/*Value = True*/
const Property P21670_1 = {S50, 0, &V21670_1_1, NULL};	/*Property = IsMobile*/
const Value V21670_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21670_5 = {S52, 0, &V21670_5_37, &P21670_1};	/*Property = ScreenPixelsHeight*/
const Value V21670_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21670_4 = {S54, 0, &V21670_4_36, &P21670_5};	/*Property = ScreenPixelsWidth*/
const Profile P21670 = {"21670", &P21670_4};	/*Profile ID - 21670*/
const Value V16034_1_1 = {S2, NULL};	/*Value = True*/
const Property P16034_1 = {S50, 0, &V16034_1_1, NULL};	/*Property = IsMobile*/
const Value V16034_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P16034_5 = {S52, 0, &V16034_5_1269, &P16034_1};	/*Property = ScreenPixelsHeight*/
const Value V16034_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16034_4 = {S54, 0, &V16034_4_1229, &P16034_5};	/*Property = ScreenPixelsWidth*/
const Profile P16034 = {"16034", &P16034_4};	/*Profile ID - 16034*/
const Value V16805_1_1 = {S2, NULL};	/*Value = True*/
const Property P16805_1 = {S50, 0, &V16805_1_1, NULL};	/*Property = IsMobile*/
const Value V16805_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P16805_5 = {S52, 0, &V16805_5_1277, &P16805_1};	/*Property = ScreenPixelsHeight*/
const Value V16805_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16805_4 = {S54, 0, &V16805_4_1279, &P16805_5};	/*Property = ScreenPixelsWidth*/
const Profile P16805 = {"16805", &P16805_4};	/*Profile ID - 16805*/
const Value V21612_1_1 = {S2, NULL};	/*Value = True*/
const Property P21612_1 = {S50, 0, &V21612_1_1, NULL};	/*Property = IsMobile*/
const Value V21612_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21612_5 = {S52, 0, &V21612_5_1269, &P21612_1};	/*Property = ScreenPixelsHeight*/
const Value V21612_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21612_4 = {S54, 0, &V21612_4_1229, &P21612_5};	/*Property = ScreenPixelsWidth*/
const Profile P21612 = {"21612", &P21612_4};	/*Profile ID - 21612*/
const Value V22469_1_1 = {S2, NULL};	/*Value = True*/
const Property P22469_1 = {S50, 0, &V22469_1_1, NULL};	/*Property = IsMobile*/
const Value V22469_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22469_5 = {S52, 0, &V22469_5_1277, &P22469_1};	/*Property = ScreenPixelsHeight*/
const Value V22469_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22469_4 = {S54, 0, &V22469_4_2336, &P22469_5};	/*Property = ScreenPixelsWidth*/
const Profile P22469 = {"22469", &P22469_4};	/*Profile ID - 22469*/
const Value V22567_1_1 = {S2, NULL};	/*Value = True*/
const Property P22567_1 = {S50, 0, &V22567_1_1, NULL};	/*Property = IsMobile*/
const Value V22567_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22567_5 = {S52, 0, &V22567_5_1277, &P22567_1};	/*Property = ScreenPixelsHeight*/
const Value V22567_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22567_4 = {S54, 0, &V22567_4_2336, &P22567_5};	/*Property = ScreenPixelsWidth*/
const Profile P22567 = {"22567", &P22567_4};	/*Profile ID - 22567*/
const Value V22232_1_1 = {S2, NULL};	/*Value = True*/
const Property P22232_1 = {S50, 0, &V22232_1_1, NULL};	/*Property = IsMobile*/
const Value V22232_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22232_5 = {S52, 0, &V22232_5_1832, &P22232_1};	/*Property = ScreenPixelsHeight*/
const Value V22232_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22232_4 = {S54, 0, &V22232_4_5645, &P22232_5};	/*Property = ScreenPixelsWidth*/
const Profile P22232 = {"22232", &P22232_4};	/*Profile ID - 22232*/
const Value V22545_1_1 = {S2, NULL};	/*Value = True*/
const Property P22545_1 = {S50, 0, &V22545_1_1, NULL};	/*Property = IsMobile*/
const Value V22545_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22545_5 = {S52, 0, &V22545_5_1277, &P22545_1};	/*Property = ScreenPixelsHeight*/
const Value V22545_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22545_4 = {S54, 0, &V22545_4_2336, &P22545_5};	/*Property = ScreenPixelsWidth*/
const Profile P22545 = {"22545", &P22545_4};	/*Profile ID - 22545*/
const Value V23829_1_1 = {S2, NULL};	/*Value = True*/
const Property P23829_1 = {S50, 0, &V23829_1_1, NULL};	/*Property = IsMobile*/
const Value V23829_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23829_5 = {S52, 0, &V23829_5_37, &P23829_1};	/*Property = ScreenPixelsHeight*/
const Value V23829_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23829_4 = {S54, 0, &V23829_4_36, &P23829_5};	/*Property = ScreenPixelsWidth*/
const Profile P23829 = {"23829", &P23829_4};	/*Profile ID - 23829*/
const Value V22268_1_1 = {S2, NULL};	/*Value = True*/
const Property P22268_1 = {S50, 0, &V22268_1_1, NULL};	/*Property = IsMobile*/
const Value V22268_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22268_5 = {S52, 0, &V22268_5_1227, &P22268_1};	/*Property = ScreenPixelsHeight*/
const Value V22268_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P22268_4 = {S54, 0, &V22268_4_1477, &P22268_5};	/*Property = ScreenPixelsWidth*/
const Profile P22268 = {"22268", &P22268_4};	/*Profile ID - 22268*/
const Value V21364_1_1 = {S2, NULL};	/*Value = True*/
const Property P21364_1 = {S50, 0, &V21364_1_1, NULL};	/*Property = IsMobile*/
const Value V21364_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21364_5 = {S52, 0, &V21364_5_1227, &P21364_1};	/*Property = ScreenPixelsHeight*/
const Value V21364_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21364_4 = {S54, 0, &V21364_4_1229, &P21364_5};	/*Property = ScreenPixelsWidth*/
const Profile P21364 = {"21364", &P21364_4};	/*Profile ID - 21364*/
const Value V17600_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_279 = {S1, 0, &V17600_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17600_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_282 = {S3, 0, &V17600_282_57155, &P17600_279};	/*Property = BlobBuilder*/
const Value V17600_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_350 = {S4, 0, &V17600_350_57446, &P17600_282};	/*Property = Canvas*/
const Value V17600_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_283 = {S5, 0, &V17600_283_57161, &P17600_350};	/*Property = CssBackground*/
const Value V17600_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_284 = {S6, 0, &V17600_284_57164, &P17600_283};	/*Property = CssBorderImage*/
const Value V17600_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_285 = {S7, 0, &V17600_285_57167, &P17600_284};	/*Property = CssCanvas*/
const Value V17600_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_286 = {S8, 0, &V17600_286_57170, &P17600_285};	/*Property = CssColor*/
const Value V17600_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_237 = {S9, 0, &V17600_237_55786, &P17600_286};	/*Property = CssColumn*/
const Value V17600_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_287 = {S10, 0, &V17600_287_57173, &P17600_237};	/*Property = CssFlexbox*/
const Value V17600_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_288 = {S11, 0, &V17600_288_57176, &P17600_287};	/*Property = CssFont*/
const Value V17600_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_289 = {S12, 0, &V17600_289_86899, &P17600_288};	/*Property = CssImages*/
const Value V17600_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_291 = {S13, 0, &V17600_291_57182, &P17600_289};	/*Property = CssMediaQueries*/
const Value V17600_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_292 = {S14, 0, &V17600_292_57185, &P17600_291};	/*Property = CssMinMax*/
const Value V17600_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_293 = {S15, 0, &V17600_293_57188, &P17600_292};	/*Property = CssOverflow*/
const Value V17600_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_294 = {S16, 0, &V17600_294_57191, &P17600_293};	/*Property = CssPosition*/
const Value V17600_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_295 = {S17, 0, &V17600_295_57194, &P17600_294};	/*Property = CssText*/
const Value V17600_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_297 = {S18, 0, &V17600_297_57197, &P17600_295};	/*Property = CssTransforms*/
const Value V17600_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_241 = {S19, 0, &V17600_241_55801, &P17600_297};	/*Property = CssTransitions*/
const Value V17600_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_298 = {S20, 0, &V17600_298_57208, &P17600_241};	/*Property = CssUI*/
const Value V17600_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_300 = {S21, 0, &V17600_300_57217, &P17600_298};	/*Property = DataSet*/
const Value V17600_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_301 = {S22, 0, &V17600_301_57220, &P17600_300};	/*Property = DataUrl*/
const Value V17600_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_302 = {S23, 0, &V17600_302_57223, &P17600_301};	/*Property = DeviceOrientation*/
const Value V17600_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_304 = {S24, 0, &V17600_304_57229, &P17600_302};	/*Property = FileReader*/
const Value V17600_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_305 = {S25, 0, &V17600_305_57232, &P17600_304};	/*Property = FileSaver*/
const Value V17600_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_306 = {S26, 0, &V17600_306_57238, &P17600_305};	/*Property = FileWriter*/
const Value V17600_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_307 = {S27, 0, &V17600_307_57241, &P17600_306};	/*Property = FormData*/
const Value V17600_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_309 = {S28, 0, &V17600_309_57247, &P17600_307};	/*Property = Fullscreen*/
const Value V17600_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_310 = {S29, 0, &V17600_310_57250, &P17600_309};	/*Property = GeoLocation*/
const Value V17600_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_312 = {S30, 0, &V17600_312_57256, &P17600_310};	/*Property = History*/
const Value V17600_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_314 = {S31, 0, &V17600_314_57262, &P17600_312};	/*Property = Html5*/
const Value V17600_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_313 = {S32, 0, &V17600_313_57259, &P17600_314};	/*Property = Html-Media-Capture*/
const Value V17600_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_315 = {S33, 0, &V17600_315_57265, &P17600_313};	/*Property = Iframe*/
const Value V17600_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_316 = {S34, 0, &V17600_316_57268, &P17600_315};	/*Property = IndexedDB*/
const Value V17600_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_317 = {S35, 0, &V17600_317_57271, &P17600_316};	/*Property = Json*/
const Value V17600_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P17600_15 = {S37, 0, &V17600_15_29, &P17600_317};	/*Property = LayoutEngine*/
const Value V17600_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_344 = {S38, 0, &V17600_344_57274, &P17600_15};	/*Property = Masking*/
const Value V17600_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_320 = {S39, 0, &V17600_320_57292, &P17600_344};	/*Property = PostMessage*/
const Value V17600_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_321 = {S40, 0, &V17600_321_57295, &P17600_320};	/*Property = Progress*/
const Value V17600_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_322 = {S41, 0, &V17600_322_57298, &P17600_321};	/*Property = Prompts*/
const Value V17600_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_323 = {S42, 0, &V17600_323_57301, &P17600_322};	/*Property = Selector*/
const Value V17600_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_324 = {S43, 0, &V17600_324_57307, &P17600_323};	/*Property = Svg*/
const Value V17600_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_327 = {S44, 0, &V17600_327_57316, &P17600_324};	/*Property = TouchEvents*/
const Value V17600_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_328 = {S45, 0, &V17600_328_57319, &P17600_327};	/*Property = Track*/
const Value V17600_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_329 = {S46, 0, &V17600_329_57325, &P17600_328};	/*Property = Video*/
const Value V17600_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_330 = {S47, 0, &V17600_330_57328, &P17600_329};	/*Property = Viewport*/
const Value V17600_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_332 = {S48, 0, &V17600_332_57340, &P17600_330};	/*Property = WebWorkers*/
const Value V17600_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17600_333 = {S49, 0, &V17600_333_57343, &P17600_332};	/*Property = Xhr2*/
const Profile P17600 = {"17600", &P17600_333};	/*Profile ID - 17600*/
const Value V16539_1_1 = {S2, NULL};	/*Value = True*/
const Property P16539_1 = {S50, 0, &V16539_1_1, NULL};	/*Property = IsMobile*/
const Value V16539_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P16539_5 = {S52, 0, &V16539_5_1227, &P16539_1};	/*Property = ScreenPixelsHeight*/
const Value V16539_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16539_4 = {S54, 0, &V16539_4_1229, &P16539_5};	/*Property = ScreenPixelsWidth*/
const Profile P16539 = {"16539", &P16539_4};	/*Profile ID - 16539*/
const Value V22253_1_1 = {S2, NULL};	/*Value = True*/
const Property P22253_1 = {S50, 0, &V22253_1_1, NULL};	/*Property = IsMobile*/
const Value V22253_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22253_5 = {S52, 0, &V22253_5_37569, &P22253_1};	/*Property = ScreenPixelsHeight*/
const Value V22253_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P22253_4 = {S54, 0, &V22253_4_1477, &P22253_5};	/*Property = ScreenPixelsWidth*/
const Profile P22253 = {"22253", &P22253_4};	/*Profile ID - 22253*/
const Value V21862_1_1 = {S2, NULL};	/*Value = True*/
const Property P21862_1 = {S50, 0, &V21862_1_1, NULL};	/*Property = IsMobile*/
const Value V21862_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21862_5 = {S52, 0, &V21862_5_1227, &P21862_1};	/*Property = ScreenPixelsHeight*/
const Value V21862_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21862_4 = {S54, 0, &V21862_4_1229, &P21862_5};	/*Property = ScreenPixelsWidth*/
const Profile P21862 = {"21862", &P21862_4};	/*Profile ID - 21862*/
const Value V21458_1_1 = {S2, NULL};	/*Value = True*/
const Property P21458_1 = {S50, 0, &V21458_1_1, NULL};	/*Property = IsMobile*/
const Value V21458_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21458_5 = {S52, 0, &V21458_5_1458, &P21458_1};	/*Property = ScreenPixelsHeight*/
const Value V21458_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21458_4 = {S54, 0, &V21458_4_1460, &P21458_5};	/*Property = ScreenPixelsWidth*/
const Profile P21458 = {"21458", &P21458_4};	/*Profile ID - 21458*/
const Value V21412_1_1 = {S2, NULL};	/*Value = True*/
const Property P21412_1 = {S50, 0, &V21412_1_1, NULL};	/*Property = IsMobile*/
const Value V21412_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21412_5 = {S52, 0, &V21412_5_1227, &P21412_1};	/*Property = ScreenPixelsHeight*/
const Value V21412_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21412_4 = {S54, 0, &V21412_4_1460, &P21412_5};	/*Property = ScreenPixelsWidth*/
const Profile P21412 = {"21412", &P21412_4};	/*Profile ID - 21412*/
const Value V20497_1_1 = {S2, NULL};	/*Value = True*/
const Property P20497_1 = {S50, 0, &V20497_1_1, NULL};	/*Property = IsMobile*/
const Value V20497_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20497_5 = {S52, 0, &V20497_5_1222, &P20497_1};	/*Property = ScreenPixelsHeight*/
const Value V20497_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20497_4 = {S54, 0, &V20497_4_1224, &P20497_5};	/*Property = ScreenPixelsWidth*/
const Profile P20497 = {"20497", &P20497_4};	/*Profile ID - 20497*/
const Value V17601_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_279 = {S1, 0, &V17601_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17601_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_282 = {S3, 0, &V17601_282_57155, &P17601_279};	/*Property = BlobBuilder*/
const Value V17601_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_350 = {S4, 0, &V17601_350_57446, &P17601_282};	/*Property = Canvas*/
const Value V17601_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_283 = {S5, 0, &V17601_283_57161, &P17601_350};	/*Property = CssBackground*/
const Value V17601_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_284 = {S6, 0, &V17601_284_57164, &P17601_283};	/*Property = CssBorderImage*/
const Value V17601_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_285 = {S7, 0, &V17601_285_57167, &P17601_284};	/*Property = CssCanvas*/
const Value V17601_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_286 = {S8, 0, &V17601_286_57170, &P17601_285};	/*Property = CssColor*/
const Value V17601_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_237 = {S9, 0, &V17601_237_55786, &P17601_286};	/*Property = CssColumn*/
const Value V17601_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_287 = {S10, 0, &V17601_287_57173, &P17601_237};	/*Property = CssFlexbox*/
const Value V17601_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_288 = {S11, 0, &V17601_288_57176, &P17601_287};	/*Property = CssFont*/
const Value V17601_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_289 = {S12, 0, &V17601_289_86899, &P17601_288};	/*Property = CssImages*/
const Value V17601_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_291 = {S13, 0, &V17601_291_57182, &P17601_289};	/*Property = CssMediaQueries*/
const Value V17601_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_292 = {S14, 0, &V17601_292_57185, &P17601_291};	/*Property = CssMinMax*/
const Value V17601_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_293 = {S15, 0, &V17601_293_57188, &P17601_292};	/*Property = CssOverflow*/
const Value V17601_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_294 = {S16, 0, &V17601_294_57191, &P17601_293};	/*Property = CssPosition*/
const Value V17601_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_295 = {S17, 0, &V17601_295_57194, &P17601_294};	/*Property = CssText*/
const Value V17601_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_297 = {S18, 0, &V17601_297_57197, &P17601_295};	/*Property = CssTransforms*/
const Value V17601_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_241 = {S19, 0, &V17601_241_55801, &P17601_297};	/*Property = CssTransitions*/
const Value V17601_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_298 = {S20, 0, &V17601_298_57208, &P17601_241};	/*Property = CssUI*/
const Value V17601_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_300 = {S21, 0, &V17601_300_57217, &P17601_298};	/*Property = DataSet*/
const Value V17601_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_301 = {S22, 0, &V17601_301_57220, &P17601_300};	/*Property = DataUrl*/
const Value V17601_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_302 = {S23, 0, &V17601_302_57223, &P17601_301};	/*Property = DeviceOrientation*/
const Value V17601_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_304 = {S24, 0, &V17601_304_57229, &P17601_302};	/*Property = FileReader*/
const Value V17601_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_305 = {S25, 0, &V17601_305_57232, &P17601_304};	/*Property = FileSaver*/
const Value V17601_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_306 = {S26, 0, &V17601_306_57238, &P17601_305};	/*Property = FileWriter*/
const Value V17601_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_307 = {S27, 0, &V17601_307_57241, &P17601_306};	/*Property = FormData*/
const Value V17601_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_309 = {S28, 0, &V17601_309_57247, &P17601_307};	/*Property = Fullscreen*/
const Value V17601_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_310 = {S29, 0, &V17601_310_57250, &P17601_309};	/*Property = GeoLocation*/
const Value V17601_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_312 = {S30, 0, &V17601_312_57256, &P17601_310};	/*Property = History*/
const Value V17601_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_314 = {S31, 0, &V17601_314_57262, &P17601_312};	/*Property = Html5*/
const Value V17601_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_313 = {S32, 0, &V17601_313_57259, &P17601_314};	/*Property = Html-Media-Capture*/
const Value V17601_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_315 = {S33, 0, &V17601_315_57265, &P17601_313};	/*Property = Iframe*/
const Value V17601_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_316 = {S34, 0, &V17601_316_57268, &P17601_315};	/*Property = IndexedDB*/
const Value V17601_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_317 = {S35, 0, &V17601_317_57271, &P17601_316};	/*Property = Json*/
const Value V17601_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P17601_15 = {S37, 0, &V17601_15_29, &P17601_317};	/*Property = LayoutEngine*/
const Value V17601_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_344 = {S38, 0, &V17601_344_57274, &P17601_15};	/*Property = Masking*/
const Value V17601_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_320 = {S39, 0, &V17601_320_57292, &P17601_344};	/*Property = PostMessage*/
const Value V17601_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_321 = {S40, 0, &V17601_321_57295, &P17601_320};	/*Property = Progress*/
const Value V17601_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_322 = {S41, 0, &V17601_322_57298, &P17601_321};	/*Property = Prompts*/
const Value V17601_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_323 = {S42, 0, &V17601_323_57301, &P17601_322};	/*Property = Selector*/
const Value V17601_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_324 = {S43, 0, &V17601_324_57307, &P17601_323};	/*Property = Svg*/
const Value V17601_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_327 = {S44, 0, &V17601_327_57316, &P17601_324};	/*Property = TouchEvents*/
const Value V17601_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_328 = {S45, 0, &V17601_328_57319, &P17601_327};	/*Property = Track*/
const Value V17601_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_329 = {S46, 0, &V17601_329_57325, &P17601_328};	/*Property = Video*/
const Value V17601_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_330 = {S47, 0, &V17601_330_57328, &P17601_329};	/*Property = Viewport*/
const Value V17601_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_332 = {S48, 0, &V17601_332_57340, &P17601_330};	/*Property = WebWorkers*/
const Value V17601_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17601_333 = {S49, 0, &V17601_333_57343, &P17601_332};	/*Property = Xhr2*/
const Profile P17601 = {"17601", &P17601_333};	/*Profile ID - 17601*/
const Value V22389_1_1 = {S2, NULL};	/*Value = True*/
const Property P22389_1 = {S50, 0, &V22389_1_1, NULL};	/*Property = IsMobile*/
const Value V22389_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22389_5 = {S52, 0, &V22389_5_1832, &P22389_1};	/*Property = ScreenPixelsHeight*/
const Value V22389_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22389_4 = {S54, 0, &V22389_4_5645, &P22389_5};	/*Property = ScreenPixelsWidth*/
const Profile P22389 = {"22389", &P22389_4};	/*Profile ID - 22389*/
const Value V22239_1_1 = {S2, NULL};	/*Value = True*/
const Property P22239_1 = {S50, 0, &V22239_1_1, NULL};	/*Property = IsMobile*/
const Value V22239_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22239_5 = {S52, 0, &V22239_5_1458, &P22239_1};	/*Property = ScreenPixelsHeight*/
const Value V22239_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22239_4 = {S54, 0, &V22239_4_1460, &P22239_5};	/*Property = ScreenPixelsWidth*/
const Profile P22239 = {"22239", &P22239_4};	/*Profile ID - 22239*/
const Value V22383_1_1 = {S2, NULL};	/*Value = True*/
const Property P22383_1 = {S50, 0, &V22383_1_1, NULL};	/*Property = IsMobile*/
const Value V22383_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22383_5 = {S52, 0, &V22383_5_1277, &P22383_1};	/*Property = ScreenPixelsHeight*/
const Value V22383_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22383_4 = {S54, 0, &V22383_4_2336, &P22383_5};	/*Property = ScreenPixelsWidth*/
const Profile P22383 = {"22383", &P22383_4};	/*Profile ID - 22383*/
const Value V22388_1_1 = {S2, NULL};	/*Value = True*/
const Property P22388_1 = {S50, 0, &V22388_1_1, NULL};	/*Property = IsMobile*/
const Value V22388_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22388_5 = {S52, 0, &V22388_5_1227, &P22388_1};	/*Property = ScreenPixelsHeight*/
const Value V22388_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22388_4 = {S54, 0, &V22388_4_1229, &P22388_5};	/*Property = ScreenPixelsWidth*/
const Profile P22388 = {"22388", &P22388_4};	/*Profile ID - 22388*/
const Value V22238_1_1 = {S2, NULL};	/*Value = True*/
const Property P22238_1 = {S50, 0, &V22238_1_1, NULL};	/*Property = IsMobile*/
const Value V22238_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22238_5 = {S52, 0, &V22238_5_1277, &P22238_1};	/*Property = ScreenPixelsHeight*/
const Value V22238_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22238_4 = {S54, 0, &V22238_4_2336, &P22238_5};	/*Property = ScreenPixelsWidth*/
const Profile P22238 = {"22238", &P22238_4};	/*Profile ID - 22238*/
const Value V23354_1_1 = {S2, NULL};	/*Value = True*/
const Property P23354_1 = {S50, 0, &V23354_1_1, NULL};	/*Property = IsMobile*/
const Value V23354_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23354_5 = {S52, 0, &V23354_5_1277, &P23354_1};	/*Property = ScreenPixelsHeight*/
const Value V23354_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23354_4 = {S54, 0, &V23354_4_1279, &P23354_5};	/*Property = ScreenPixelsWidth*/
const Profile P23354 = {"23354", &P23354_4};	/*Profile ID - 23354*/
const Value V22557_1_1 = {S2, NULL};	/*Value = True*/
const Property P22557_1 = {S50, 0, &V22557_1_1, NULL};	/*Property = IsMobile*/
const Value V22557_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22557_5 = {S52, 0, &V22557_5_1277, &P22557_1};	/*Property = ScreenPixelsHeight*/
const Value V22557_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22557_4 = {S54, 0, &V22557_4_1279, &P22557_5};	/*Property = ScreenPixelsWidth*/
const Profile P22557 = {"22557", &P22557_4};	/*Profile ID - 22557*/
const Value V16033_1_1 = {S2, NULL};	/*Value = True*/
const Property P16033_1 = {S50, 0, &V16033_1_1, NULL};	/*Property = IsMobile*/
const Value V16033_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P16033_5 = {S52, 0, &V16033_5_1269, &P16033_1};	/*Property = ScreenPixelsHeight*/
const Value V16033_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P16033_4 = {S54, 0, &V16033_4_1229, &P16033_5};	/*Property = ScreenPixelsWidth*/
const Profile P16033 = {"16033", &P16033_4};	/*Profile ID - 16033*/
const Value V21423_1_1 = {S2, NULL};	/*Value = True*/
const Property P21423_1 = {S50, 0, &V21423_1_1, NULL};	/*Property = IsMobile*/
const Value V21423_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21423_5 = {S52, 0, &V21423_5_1227, &P21423_1};	/*Property = ScreenPixelsHeight*/
const Value V21423_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21423_4 = {S54, 0, &V21423_4_1460, &P21423_5};	/*Property = ScreenPixelsWidth*/
const Profile P21423 = {"21423", &P21423_4};	/*Profile ID - 21423*/
const Value V20474_1_1 = {S2, NULL};	/*Value = True*/
const Property P20474_1 = {S50, 0, &V20474_1_1, NULL};	/*Property = IsMobile*/
const Value V20474_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20474_5 = {S52, 0, &V20474_5_1277, &P20474_1};	/*Property = ScreenPixelsHeight*/
const Value V20474_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20474_4 = {S54, 0, &V20474_4_1279, &P20474_5};	/*Property = ScreenPixelsWidth*/
const Profile P20474 = {"20474", &P20474_4};	/*Profile ID - 20474*/
const Value V18103_1_1 = {S2, NULL};	/*Value = True*/
const Property P18103_1 = {S50, 0, &V18103_1_1, NULL};	/*Property = IsMobile*/
const Value V18103_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P18103_5 = {S52, 0, &V18103_5_1458, &P18103_1};	/*Property = ScreenPixelsHeight*/
const Value V18103_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P18103_4 = {S54, 0, &V18103_4_37617, &P18103_5};	/*Property = ScreenPixelsWidth*/
const Profile P18103 = {"18103", &P18103_4};	/*Profile ID - 18103*/
const Value V21121_1_1 = {S2, NULL};	/*Value = True*/
const Property P21121_1 = {S50, 0, &V21121_1_1, NULL};	/*Property = IsMobile*/
const Value V21121_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21121_5 = {S52, 0, &V21121_5_1277, &P21121_1};	/*Property = ScreenPixelsHeight*/
const Value V21121_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21121_4 = {S54, 0, &V21121_4_2336, &P21121_5};	/*Property = ScreenPixelsWidth*/
const Profile P21121 = {"21121", &P21121_4};	/*Profile ID - 21121*/
const Value V22494_1_1 = {S2, NULL};	/*Value = True*/
const Property P22494_1 = {S50, 0, &V22494_1_1, NULL};	/*Property = IsMobile*/
const Value V22494_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22494_5 = {S52, 0, &V22494_5_1832, &P22494_1};	/*Property = ScreenPixelsHeight*/
const Value V22494_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22494_4 = {S54, 0, &V22494_4_5645, &P22494_5};	/*Property = ScreenPixelsWidth*/
const Profile P22494 = {"22494", &P22494_4};	/*Profile ID - 22494*/
const Value V22561_1_1 = {S2, NULL};	/*Value = True*/
const Property P22561_1 = {S50, 0, &V22561_1_1, NULL};	/*Property = IsMobile*/
const Value V22561_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22561_5 = {S52, 0, &V22561_5_1832, &P22561_1};	/*Property = ScreenPixelsHeight*/
const Value V22561_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22561_4 = {S54, 0, &V22561_4_5645, &P22561_5};	/*Property = ScreenPixelsWidth*/
const Profile P22561 = {"22561", &P22561_4};	/*Profile ID - 22561*/
const Value V21551_1_1 = {S2, NULL};	/*Value = True*/
const Property P21551_1 = {S50, 0, &V21551_1_1, NULL};	/*Property = IsMobile*/
const Value V21551_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21551_5 = {S52, 0, &V21551_5_1227, &P21551_1};	/*Property = ScreenPixelsHeight*/
const Value V21551_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21551_4 = {S54, 0, &V21551_4_1229, &P21551_5};	/*Property = ScreenPixelsWidth*/
const Profile P21551 = {"21551", &P21551_4};	/*Profile ID - 21551*/
const Value V21325_1_1 = {S2, NULL};	/*Value = True*/
const Property P21325_1 = {S50, 0, &V21325_1_1, NULL};	/*Property = IsMobile*/
const Value V21325_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21325_5 = {S52, 0, &V21325_5_1227, &P21325_1};	/*Property = ScreenPixelsHeight*/
const Value V21325_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21325_4 = {S54, 0, &V21325_4_1229, &P21325_5};	/*Property = ScreenPixelsWidth*/
const Profile P21325 = {"21325", &P21325_4};	/*Profile ID - 21325*/
const Value V22384_1_1 = {S2, NULL};	/*Value = True*/
const Property P22384_1 = {S50, 0, &V22384_1_1, NULL};	/*Property = IsMobile*/
const Value V22384_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22384_5 = {S52, 0, &V22384_5_37569, &P22384_1};	/*Property = ScreenPixelsHeight*/
const Value V22384_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22384_4 = {S54, 0, &V22384_4_5645, &P22384_5};	/*Property = ScreenPixelsWidth*/
const Profile P22384 = {"22384", &P22384_4};	/*Profile ID - 22384*/
const Value V22362_1_1 = {S2, NULL};	/*Value = True*/
const Property P22362_1 = {S50, 0, &V22362_1_1, NULL};	/*Property = IsMobile*/
const Value V22362_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22362_5 = {S52, 0, &V22362_5_1458, &P22362_1};	/*Property = ScreenPixelsHeight*/
const Value V22362_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P22362_4 = {S54, 0, &V22362_4_37617, &P22362_5};	/*Property = ScreenPixelsWidth*/
const Profile P22362 = {"22362", &P22362_4};	/*Profile ID - 22362*/
const Value V23923_1_1 = {S2, NULL};	/*Value = True*/
const Property P23923_1 = {S50, 0, &V23923_1_1, NULL};	/*Property = IsMobile*/
const Value V23923_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23923_5 = {S52, 0, &V23923_5_1269, &P23923_1};	/*Property = ScreenPixelsHeight*/
const Value V23923_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23923_4 = {S54, 0, &V23923_4_1229, &P23923_5};	/*Property = ScreenPixelsWidth*/
const Profile P23923 = {"23923", &P23923_4};	/*Profile ID - 23923*/
const Value V21265_1_1 = {S2, NULL};	/*Value = True*/
const Property P21265_1 = {S50, 0, &V21265_1_1, NULL};	/*Property = IsMobile*/
const Value V21265_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21265_5 = {S52, 0, &V21265_5_1277, &P21265_1};	/*Property = ScreenPixelsHeight*/
const Value V21265_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21265_4 = {S54, 0, &V21265_4_2336, &P21265_5};	/*Property = ScreenPixelsWidth*/
const Profile P21265 = {"21265", &P21265_4};	/*Profile ID - 21265*/
const Value V21158_1_1 = {S2, NULL};	/*Value = True*/
const Property P21158_1 = {S50, 0, &V21158_1_1, NULL};	/*Property = IsMobile*/
const Value V21158_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21158_5 = {S52, 0, &V21158_5_1277, &P21158_1};	/*Property = ScreenPixelsHeight*/
const Value V21158_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P21158_4 = {S54, 0, &V21158_4_1278, &P21158_5};	/*Property = ScreenPixelsWidth*/
const Profile P21158 = {"21158", &P21158_4};	/*Profile ID - 21158*/
const Value V19876_1_1 = {S2, NULL};	/*Value = True*/
const Property P19876_1 = {S50, 0, &V19876_1_1, NULL};	/*Property = IsMobile*/
const Value V19876_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19876_5 = {S52, 0, &V19876_5_1227, &P19876_1};	/*Property = ScreenPixelsHeight*/
const Value V19876_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19876_4 = {S54, 0, &V19876_4_1229, &P19876_5};	/*Property = ScreenPixelsWidth*/
const Profile P19876 = {"19876", &P19876_4};	/*Profile ID - 19876*/
const Value V20708_1_1 = {S2, NULL};	/*Value = True*/
const Property P20708_1 = {S50, 0, &V20708_1_1, NULL};	/*Property = IsMobile*/
const Value V20708_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20708_5 = {S52, 0, &V20708_5_9981, &P20708_1};	/*Property = ScreenPixelsHeight*/
const Value V20708_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20708_4 = {S54, 0, &V20708_4_39098, &P20708_5};	/*Property = ScreenPixelsWidth*/
const Profile P20708 = {"20708", &P20708_4};	/*Profile ID - 20708*/
const Value V21368_1_1 = {S2, NULL};	/*Value = True*/
const Property P21368_1 = {S50, 0, &V21368_1_1, NULL};	/*Property = IsMobile*/
const Value V21368_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21368_5 = {S52, 0, &V21368_5_1227, &P21368_1};	/*Property = ScreenPixelsHeight*/
const Value V21368_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21368_4 = {S54, 0, &V21368_4_1460, &P21368_5};	/*Property = ScreenPixelsWidth*/
const Profile P21368 = {"21368", &P21368_4};	/*Profile ID - 21368*/
const Value V22242_1_1 = {S2, NULL};	/*Value = True*/
const Property P22242_1 = {S50, 0, &V22242_1_1, NULL};	/*Property = IsMobile*/
const Value V22242_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22242_5 = {S52, 0, &V22242_5_1227, &P22242_1};	/*Property = ScreenPixelsHeight*/
const Value V22242_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P22242_4 = {S54, 0, &V22242_4_1477, &P22242_5};	/*Property = ScreenPixelsWidth*/
const Profile P22242 = {"22242", &P22242_4};	/*Profile ID - 22242*/
const Value V22105_1_1 = {S2, NULL};	/*Value = True*/
const Property P22105_1 = {S50, 0, &V22105_1_1, NULL};	/*Property = IsMobile*/
const Value V22105_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22105_5 = {S52, 0, &V22105_5_1227, &P22105_1};	/*Property = ScreenPixelsHeight*/
const Value V22105_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22105_4 = {S54, 0, &V22105_4_1229, &P22105_5};	/*Property = ScreenPixelsWidth*/
const Profile P22105 = {"22105", &P22105_4};	/*Profile ID - 22105*/
const Value V22081_1_1 = {S2, NULL};	/*Value = True*/
const Property P22081_1 = {S50, 0, &V22081_1_1, NULL};	/*Property = IsMobile*/
const Value V22081_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22081_5 = {S52, 0, &V22081_5_1227, &P22081_1};	/*Property = ScreenPixelsHeight*/
const Value V22081_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22081_4 = {S54, 0, &V22081_4_1460, &P22081_5};	/*Property = ScreenPixelsWidth*/
const Profile P22081 = {"22081", &P22081_4};	/*Profile ID - 22081*/
const Value V18277_1_1 = {S2, NULL};	/*Value = True*/
const Property P18277_1 = {S50, 0, &V18277_1_1, NULL};	/*Property = IsMobile*/
const Value V18277_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18277_5 = {S52, 0, &V18277_5_9981, &P18277_1};	/*Property = ScreenPixelsHeight*/
const Value V18277_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18277_4 = {S54, 0, &V18277_4_39098, &P18277_5};	/*Property = ScreenPixelsWidth*/
const Profile P18277 = {"18277", &P18277_4};	/*Profile ID - 18277*/
const Value V21563_1_1 = {S2, NULL};	/*Value = True*/
const Property P21563_1 = {S50, 0, &V21563_1_1, NULL};	/*Property = IsMobile*/
const Value V21563_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21563_5 = {S52, 0, &V21563_5_1403, &P21563_1};	/*Property = ScreenPixelsHeight*/
const Value V21563_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21563_4 = {S54, 0, &V21563_4_1404, &P21563_5};	/*Property = ScreenPixelsWidth*/
const Profile P21563 = {"21563", &P21563_4};	/*Profile ID - 21563*/
const Value V21165_1_1 = {S2, NULL};	/*Value = True*/
const Property P21165_1 = {S50, 0, &V21165_1_1, NULL};	/*Property = IsMobile*/
const Value V21165_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21165_5 = {S52, 0, &V21165_5_9981, &P21165_1};	/*Property = ScreenPixelsHeight*/
const Value V21165_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21165_4 = {S54, 0, &V21165_4_39098, &P21165_5};	/*Property = ScreenPixelsWidth*/
const Profile P21165 = {"21165", &P21165_4};	/*Profile ID - 21165*/
const Value V20499_1_1 = {S2, NULL};	/*Value = True*/
const Property P20499_1 = {S50, 0, &V20499_1_1, NULL};	/*Property = IsMobile*/
const Value V20499_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20499_5 = {S52, 0, &V20499_5_1227, &P20499_1};	/*Property = ScreenPixelsHeight*/
const Value V20499_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20499_4 = {S54, 0, &V20499_4_1229, &P20499_5};	/*Property = ScreenPixelsWidth*/
const Profile P20499 = {"20499", &P20499_4};	/*Profile ID - 20499*/
const Value V20604_1_1 = {S2, NULL};	/*Value = True*/
const Property P20604_1 = {S50, 0, &V20604_1_1, NULL};	/*Property = IsMobile*/
const Value V20604_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20604_5 = {S52, 0, &V20604_5_1227, &P20604_1};	/*Property = ScreenPixelsHeight*/
const Value V20604_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20604_4 = {S54, 0, &V20604_4_1229, &P20604_5};	/*Property = ScreenPixelsWidth*/
const Profile P20604 = {"20604", &P20604_4};	/*Profile ID - 20604*/
const Value V20591_1_1 = {S2, NULL};	/*Value = True*/
const Property P20591_1 = {S50, 0, &V20591_1_1, NULL};	/*Property = IsMobile*/
const Value V20591_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20591_5 = {S52, 0, &V20591_5_1227, &P20591_1};	/*Property = ScreenPixelsHeight*/
const Value V20591_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20591_4 = {S54, 0, &V20591_4_1229, &P20591_5};	/*Property = ScreenPixelsWidth*/
const Profile P20591 = {"20591", &P20591_4};	/*Profile ID - 20591*/
const Value V22176_1_1 = {S2, NULL};	/*Value = True*/
const Property P22176_1 = {S50, 0, &V22176_1_1, NULL};	/*Property = IsMobile*/
const Value V22176_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22176_5 = {S52, 0, &V22176_5_1227, &P22176_1};	/*Property = ScreenPixelsHeight*/
const Value V22176_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22176_4 = {S54, 0, &V22176_4_1229, &P22176_5};	/*Property = ScreenPixelsWidth*/
const Profile P22176 = {"22176", &P22176_4};	/*Profile ID - 22176*/
const Value V18653_1_1 = {S2, NULL};	/*Value = True*/
const Property P18653_1 = {S50, 0, &V18653_1_1, NULL};	/*Property = IsMobile*/
const Value V18653_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18653_5 = {S52, 0, &V18653_5_1403, &P18653_1};	/*Property = ScreenPixelsHeight*/
const Value V18653_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18653_4 = {S54, 0, &V18653_4_1404, &P18653_5};	/*Property = ScreenPixelsWidth*/
const Profile P18653 = {"18653", &P18653_4};	/*Profile ID - 18653*/
const Value V23988_1_1 = {S2, NULL};	/*Value = True*/
const Property P23988_1 = {S50, 0, &V23988_1_1, NULL};	/*Property = IsMobile*/
const Value V23988_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P23988_5 = {S52, 0, &V23988_5_1304, &P23988_1};	/*Property = ScreenPixelsHeight*/
const Value V23988_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23988_4 = {S54, 0, &V23988_4_1224, &P23988_5};	/*Property = ScreenPixelsWidth*/
const Profile P23988 = {"23988", &P23988_4};	/*Profile ID - 23988*/
const Value V21341_1_1 = {S2, NULL};	/*Value = True*/
const Property P21341_1 = {S50, 0, &V21341_1_1, NULL};	/*Property = IsMobile*/
const Value V21341_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21341_5 = {S52, 0, &V21341_5_1227, &P21341_1};	/*Property = ScreenPixelsHeight*/
const Value V21341_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21341_4 = {S54, 0, &V21341_4_1229, &P21341_5};	/*Property = ScreenPixelsWidth*/
const Profile P21341 = {"21341", &P21341_4};	/*Profile ID - 21341*/
const Value V23877_1_1 = {S2, NULL};	/*Value = True*/
const Property P23877_1 = {S50, 0, &V23877_1_1, NULL};	/*Property = IsMobile*/
const Value V23877_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23877_5 = {S52, 0, &V23877_5_1227, &P23877_1};	/*Property = ScreenPixelsHeight*/
const Value V23877_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P23877_4 = {S54, 0, &V23877_4_1477, &P23877_5};	/*Property = ScreenPixelsWidth*/
const Profile P23877 = {"23877", &P23877_4};	/*Profile ID - 23877*/
const Value V22091_1_1 = {S2, NULL};	/*Value = True*/
const Property P22091_1 = {S50, 0, &V22091_1_1, NULL};	/*Property = IsMobile*/
const Value V22091_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22091_5 = {S52, 0, &V22091_5_1277, &P22091_1};	/*Property = ScreenPixelsHeight*/
const Value V22091_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22091_4 = {S54, 0, &V22091_4_1279, &P22091_5};	/*Property = ScreenPixelsWidth*/
const Profile P22091 = {"22091", &P22091_4};	/*Profile ID - 22091*/
const Value V21250_1_1 = {S2, NULL};	/*Value = True*/
const Property P21250_1 = {S50, 0, &V21250_1_1, NULL};	/*Property = IsMobile*/
const Value V21250_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21250_5 = {S52, 0, &V21250_5_1277, &P21250_1};	/*Property = ScreenPixelsHeight*/
const Value V21250_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21250_4 = {S54, 0, &V21250_4_1279, &P21250_5};	/*Property = ScreenPixelsWidth*/
const Profile P21250 = {"21250", &P21250_4};	/*Profile ID - 21250*/
const Value V21383_1_1 = {S2, NULL};	/*Value = True*/
const Property P21383_1 = {S50, 0, &V21383_1_1, NULL};	/*Property = IsMobile*/
const Value V21383_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21383_5 = {S52, 0, &V21383_5_1277, &P21383_1};	/*Property = ScreenPixelsHeight*/
const Value V21383_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21383_4 = {S54, 0, &V21383_4_1279, &P21383_5};	/*Property = ScreenPixelsWidth*/
const Profile P21383 = {"21383", &P21383_4};	/*Profile ID - 21383*/
const Value V23888_1_1 = {S2, NULL};	/*Value = True*/
const Property P23888_1 = {S50, 0, &V23888_1_1, NULL};	/*Property = IsMobile*/
const Value V23888_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23888_5 = {S52, 0, &V23888_5_1227, &P23888_1};	/*Property = ScreenPixelsHeight*/
const Value V23888_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23888_4 = {S54, 0, &V23888_4_1229, &P23888_5};	/*Property = ScreenPixelsWidth*/
const Profile P23888 = {"23888", &P23888_4};	/*Profile ID - 23888*/
const Value V24543_1_1 = {S2, NULL};	/*Value = True*/
const Property P24543_1 = {S50, 0, &V24543_1_1, NULL};	/*Property = IsMobile*/
const Value V24543_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P24543_5 = {S52, 0, &V24543_5_1832, &P24543_1};	/*Property = ScreenPixelsHeight*/
const Value V24543_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P24543_4 = {S54, 0, &V24543_4_5645, &P24543_5};	/*Property = ScreenPixelsWidth*/
const Profile P24543 = {"24543", &P24543_4};	/*Profile ID - 24543*/
const Value V21164_1_1 = {S2, NULL};	/*Value = True*/
const Property P21164_1 = {S50, 0, &V21164_1_1, NULL};	/*Property = IsMobile*/
const Value V21164_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21164_5 = {S52, 0, &V21164_5_1403, &P21164_1};	/*Property = ScreenPixelsHeight*/
const Value V21164_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21164_4 = {S54, 0, &V21164_4_1404, &P21164_5};	/*Property = ScreenPixelsWidth*/
const Profile P21164 = {"21164", &P21164_4};	/*Profile ID - 21164*/
const Value V20478_1_1 = {S2, NULL};	/*Value = True*/
const Property P20478_1 = {S50, 0, &V20478_1_1, NULL};	/*Property = IsMobile*/
const Value V20478_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20478_5 = {S52, 0, &V20478_5_1227, &P20478_1};	/*Property = ScreenPixelsHeight*/
const Value V20478_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20478_4 = {S54, 0, &V20478_4_1229, &P20478_5};	/*Property = ScreenPixelsWidth*/
const Profile P20478 = {"20478", &P20478_4};	/*Profile ID - 20478*/
const Value V23990_1_1 = {S2, NULL};	/*Value = True*/
const Property P23990_1 = {S50, 0, &V23990_1_1, NULL};	/*Property = IsMobile*/
const Value V23990_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23990_5 = {S52, 0, &V23990_5_9981, &P23990_1};	/*Property = ScreenPixelsHeight*/
const Value V23990_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23990_4 = {S54, 0, &V23990_4_2336, &P23990_5};	/*Property = ScreenPixelsWidth*/
const Profile P23990 = {"23990", &P23990_4};	/*Profile ID - 23990*/
const Value V24456_1_1 = {S2, NULL};	/*Value = True*/
const Property P24456_1 = {S50, 0, &V24456_1_1, NULL};	/*Property = IsMobile*/
const Value V24456_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24456_5 = {S52, 0, &V24456_5_1227, &P24456_1};	/*Property = ScreenPixelsHeight*/
const Value V24456_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24456_4 = {S54, 0, &V24456_4_1229, &P24456_5};	/*Property = ScreenPixelsWidth*/
const Profile P24456 = {"24456", &P24456_4};	/*Profile ID - 24456*/
const Value V21103_1_1 = {S2, NULL};	/*Value = True*/
const Property P21103_1 = {S50, 0, &V21103_1_1, NULL};	/*Property = IsMobile*/
const Value V21103_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21103_5 = {S52, 0, &V21103_5_1458, &P21103_1};	/*Property = ScreenPixelsHeight*/
const Value V21103_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21103_4 = {S54, 0, &V21103_4_37617, &P21103_5};	/*Property = ScreenPixelsWidth*/
const Profile P21103 = {"21103", &P21103_4};	/*Profile ID - 21103*/
const Value V18484_1_1 = {S2, NULL};	/*Value = True*/
const Property P18484_1 = {S50, 0, &V18484_1_1, NULL};	/*Property = IsMobile*/
const Value V18484_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18484_5 = {S52, 0, &V18484_5_1227, &P18484_1};	/*Property = ScreenPixelsHeight*/
const Value V18484_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18484_4 = {S54, 0, &V18484_4_1229, &P18484_5};	/*Property = ScreenPixelsWidth*/
const Profile P18484 = {"18484", &P18484_4};	/*Profile ID - 18484*/
const Value V23085_1_1 = {S2, NULL};	/*Value = True*/
const Property P23085_1 = {S50, 0, &V23085_1_1, NULL};	/*Property = IsMobile*/
const Value V23085_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23085_5 = {S52, 0, &V23085_5_1269, &P23085_1};	/*Property = ScreenPixelsHeight*/
const Value V23085_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23085_4 = {S54, 0, &V23085_4_1229, &P23085_5};	/*Property = ScreenPixelsWidth*/
const Profile P23085 = {"23085", &P23085_4};	/*Profile ID - 23085*/
const Value V21243_1_1 = {S2, NULL};	/*Value = True*/
const Property P21243_1 = {S50, 0, &V21243_1_1, NULL};	/*Property = IsMobile*/
const Value V21243_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21243_5 = {S52, 0, &V21243_5_1269, &P21243_1};	/*Property = ScreenPixelsHeight*/
const Value V21243_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21243_4 = {S54, 0, &V21243_4_1229, &P21243_5};	/*Property = ScreenPixelsWidth*/
const Profile P21243 = {"21243", &P21243_4};	/*Profile ID - 21243*/
const Value V24510_1_1 = {S2, NULL};	/*Value = True*/
const Property P24510_1 = {S50, 0, &V24510_1_1, NULL};	/*Property = IsMobile*/
const Value V24510_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24510_5 = {S52, 0, &V24510_5_9981, &P24510_1};	/*Property = ScreenPixelsHeight*/
const Value V24510_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P24510_4 = {S54, 0, &V24510_4_2336, &P24510_5};	/*Property = ScreenPixelsWidth*/
const Profile P24510 = {"24510", &P24510_4};	/*Profile ID - 24510*/
const Value V21406_1_1 = {S2, NULL};	/*Value = True*/
const Property P21406_1 = {S50, 0, &V21406_1_1, NULL};	/*Property = IsMobile*/
const Value V21406_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21406_5 = {S52, 0, &V21406_5_1227, &P21406_1};	/*Property = ScreenPixelsHeight*/
const Value V21406_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21406_4 = {S54, 0, &V21406_4_1229, &P21406_5};	/*Property = ScreenPixelsWidth*/
const Profile P21406 = {"21406", &P21406_4};	/*Profile ID - 21406*/
const Value V20496_1_1 = {S2, NULL};	/*Value = True*/
const Property P20496_1 = {S50, 0, &V20496_1_1, NULL};	/*Property = IsMobile*/
const Value V20496_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20496_5 = {S52, 0, &V20496_5_1227, &P20496_1};	/*Property = ScreenPixelsHeight*/
const Value V20496_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20496_4 = {S54, 0, &V20496_4_1229, &P20496_5};	/*Property = ScreenPixelsWidth*/
const Profile P20496 = {"20496", &P20496_4};	/*Profile ID - 20496*/
const Value V21826_1_1 = {S2, NULL};	/*Value = True*/
const Property P21826_1 = {S50, 0, &V21826_1_1, NULL};	/*Property = IsMobile*/
const Value V21826_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21826_5 = {S52, 0, &V21826_5_1277, &P21826_1};	/*Property = ScreenPixelsHeight*/
const Value V21826_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21826_4 = {S54, 0, &V21826_4_1279, &P21826_5};	/*Property = ScreenPixelsWidth*/
const Profile P21826 = {"21826", &P21826_4};	/*Profile ID - 21826*/
const Value V21552_1_1 = {S2, NULL};	/*Value = True*/
const Property P21552_1 = {S50, 0, &V21552_1_1, NULL};	/*Property = IsMobile*/
const Value V21552_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21552_5 = {S52, 0, &V21552_5_1277, &P21552_1};	/*Property = ScreenPixelsHeight*/
const Value V21552_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21552_4 = {S54, 0, &V21552_4_1279, &P21552_5};	/*Property = ScreenPixelsWidth*/
const Profile P21552 = {"21552", &P21552_4};	/*Profile ID - 21552*/
const Value V22985_1_1 = {S2, NULL};	/*Value = True*/
const Property P22985_1 = {S50, 0, &V22985_1_1, NULL};	/*Property = IsMobile*/
const Value V22985_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22985_5 = {S52, 0, &V22985_5_9981, &P22985_1};	/*Property = ScreenPixelsHeight*/
const Value V22985_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22985_4 = {S54, 0, &V22985_4_2336, &P22985_5};	/*Property = ScreenPixelsWidth*/
const Profile P22985 = {"22985", &P22985_4};	/*Profile ID - 22985*/
const Value V21343_1_1 = {S2, NULL};	/*Value = True*/
const Property P21343_1 = {S50, 0, &V21343_1_1, NULL};	/*Property = IsMobile*/
const Value V21343_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21343_5 = {S52, 0, &V21343_5_1227, &P21343_1};	/*Property = ScreenPixelsHeight*/
const Value V21343_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21343_4 = {S54, 0, &V21343_4_1460, &P21343_5};	/*Property = ScreenPixelsWidth*/
const Profile P21343 = {"21343", &P21343_4};	/*Profile ID - 21343*/
const Value V18250_1_1 = {S2, NULL};	/*Value = True*/
const Property P18250_1 = {S50, 0, &V18250_1_1, NULL};	/*Property = IsMobile*/
const Value V18250_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18250_5 = {S52, 0, &V18250_5_1403, &P18250_1};	/*Property = ScreenPixelsHeight*/
const Value V18250_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18250_4 = {S54, 0, &V18250_4_1404, &P18250_5};	/*Property = ScreenPixelsWidth*/
const Profile P18250 = {"18250", &P18250_4};	/*Profile ID - 18250*/
const Value V22495_1_1 = {S2, NULL};	/*Value = True*/
const Property P22495_1 = {S50, 0, &V22495_1_1, NULL};	/*Property = IsMobile*/
const Value V22495_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22495_5 = {S52, 0, &V22495_5_1277, &P22495_1};	/*Property = ScreenPixelsHeight*/
const Value V22495_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22495_4 = {S54, 0, &V22495_4_2336, &P22495_5};	/*Property = ScreenPixelsWidth*/
const Profile P22495 = {"22495", &P22495_4};	/*Profile ID - 22495*/
const Value V20580_1_1 = {S2, NULL};	/*Value = True*/
const Property P20580_1 = {S50, 0, &V20580_1_1, NULL};	/*Property = IsMobile*/
const Value V20580_5_64958 = {S134, NULL};	/*Value = 1600*/
const Property P20580_5 = {S52, 0, &V20580_5_64958, &P20580_1};	/*Property = ScreenPixelsHeight*/
const Value V20580_4_59865 = {S135, NULL};	/*Value = 900*/
const Property P20580_4 = {S54, 0, &V20580_4_59865, &P20580_5};	/*Property = ScreenPixelsWidth*/
const Profile P20580 = {"20580", &P20580_4};	/*Profile ID - 20580*/
const Value V22544_1_1 = {S2, NULL};	/*Value = True*/
const Property P22544_1 = {S50, 0, &V22544_1_1, NULL};	/*Property = IsMobile*/
const Value V22544_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22544_5 = {S52, 0, &V22544_5_37569, &P22544_1};	/*Property = ScreenPixelsHeight*/
const Value V22544_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22544_4 = {S54, 0, &V22544_4_5645, &P22544_5};	/*Property = ScreenPixelsWidth*/
const Profile P22544 = {"22544", &P22544_4};	/*Profile ID - 22544*/
const Value V21509_1_1 = {S2, NULL};	/*Value = True*/
const Property P21509_1 = {S50, 0, &V21509_1_1, NULL};	/*Property = IsMobile*/
const Value V21509_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21509_5 = {S52, 0, &V21509_5_1227, &P21509_1};	/*Property = ScreenPixelsHeight*/
const Value V21509_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21509_4 = {S54, 0, &V21509_4_1229, &P21509_5};	/*Property = ScreenPixelsWidth*/
const Profile P21509 = {"21509", &P21509_4};	/*Profile ID - 21509*/
const Value V23744_1_1 = {S2, NULL};	/*Value = True*/
const Property P23744_1 = {S50, 0, &V23744_1_1, NULL};	/*Property = IsMobile*/
const Value V23744_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23744_5 = {S52, 0, &V23744_5_1277, &P23744_1};	/*Property = ScreenPixelsHeight*/
const Value V23744_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23744_4 = {S54, 0, &V23744_4_1279, &P23744_5};	/*Property = ScreenPixelsWidth*/
const Profile P23744 = {"23744", &P23744_4};	/*Profile ID - 23744*/
const Value V21322_1_1 = {S2, NULL};	/*Value = True*/
const Property P21322_1 = {S50, 0, &V21322_1_1, NULL};	/*Property = IsMobile*/
const Value V21322_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21322_5 = {S52, 0, &V21322_5_1227, &P21322_1};	/*Property = ScreenPixelsHeight*/
const Value V21322_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21322_4 = {S54, 0, &V21322_4_1229, &P21322_5};	/*Property = ScreenPixelsWidth*/
const Profile P21322 = {"21322", &P21322_4};	/*Profile ID - 21322*/
const Value V18626_1_1 = {S2, NULL};	/*Value = True*/
const Property P18626_1 = {S50, 0, &V18626_1_1, NULL};	/*Property = IsMobile*/
const Value V18626_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18626_5 = {S52, 0, &V18626_5_1227, &P18626_1};	/*Property = ScreenPixelsHeight*/
const Value V18626_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18626_4 = {S54, 0, &V18626_4_1229, &P18626_5};	/*Property = ScreenPixelsWidth*/
const Profile P18626 = {"18626", &P18626_4};	/*Profile ID - 18626*/
const Value V21839_1_1 = {S2, NULL};	/*Value = True*/
const Property P21839_1 = {S50, 0, &V21839_1_1, NULL};	/*Property = IsMobile*/
const Value V21839_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21839_5 = {S52, 0, &V21839_5_1227, &P21839_1};	/*Property = ScreenPixelsHeight*/
const Value V21839_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21839_4 = {S54, 0, &V21839_4_1460, &P21839_5};	/*Property = ScreenPixelsWidth*/
const Profile P21839 = {"21839", &P21839_4};	/*Profile ID - 21839*/
const Value V21303_1_1 = {S2, NULL};	/*Value = True*/
const Property P21303_1 = {S50, 0, &V21303_1_1, NULL};	/*Property = IsMobile*/
const Value V21303_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21303_5 = {S52, 0, &V21303_5_1277, &P21303_1};	/*Property = ScreenPixelsHeight*/
const Value V21303_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21303_4 = {S54, 0, &V21303_4_1279, &P21303_5};	/*Property = ScreenPixelsWidth*/
const Profile P21303 = {"21303", &P21303_4};	/*Profile ID - 21303*/
const Value V21394_1_1 = {S2, NULL};	/*Value = True*/
const Property P21394_1 = {S50, 0, &V21394_1_1, NULL};	/*Property = IsMobile*/
const Value V21394_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21394_5 = {S52, 0, &V21394_5_1227, &P21394_1};	/*Property = ScreenPixelsHeight*/
const Value V21394_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21394_4 = {S54, 0, &V21394_4_1229, &P21394_5};	/*Property = ScreenPixelsWidth*/
const Profile P21394 = {"21394", &P21394_4};	/*Profile ID - 21394*/
const Value V21994_1_1 = {S2, NULL};	/*Value = True*/
const Property P21994_1 = {S50, 0, &V21994_1_1, NULL};	/*Property = IsMobile*/
const Value V21994_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P21994_5 = {S52, 0, &V21994_5_37569, &P21994_1};	/*Property = ScreenPixelsHeight*/
const Value V21994_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21994_4 = {S54, 0, &V21994_4_5645, &P21994_5};	/*Property = ScreenPixelsWidth*/
const Profile P21994 = {"21994", &P21994_4};	/*Profile ID - 21994*/
const Value V19897_1_1 = {S2, NULL};	/*Value = True*/
const Property P19897_1 = {S50, 0, &V19897_1_1, NULL};	/*Property = IsMobile*/
const Value V19897_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19897_5 = {S52, 0, &V19897_5_1227, &P19897_1};	/*Property = ScreenPixelsHeight*/
const Value V19897_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19897_4 = {S54, 0, &V19897_4_1229, &P19897_5};	/*Property = ScreenPixelsWidth*/
const Profile P19897 = {"19897", &P19897_4};	/*Profile ID - 19897*/
const Value V22439_1_1 = {S2, NULL};	/*Value = True*/
const Property P22439_1 = {S50, 0, &V22439_1_1, NULL};	/*Property = IsMobile*/
const Value V22439_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22439_5 = {S52, 0, &V22439_5_1277, &P22439_1};	/*Property = ScreenPixelsHeight*/
const Value V22439_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22439_4 = {S54, 0, &V22439_4_2336, &P22439_5};	/*Property = ScreenPixelsWidth*/
const Profile P22439 = {"22439", &P22439_4};	/*Profile ID - 22439*/
const Value V23938_1_1 = {S2, NULL};	/*Value = True*/
const Property P23938_1 = {S50, 0, &V23938_1_1, NULL};	/*Property = IsMobile*/
const Value V23938_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23938_5 = {S52, 0, &V23938_5_1277, &P23938_1};	/*Property = ScreenPixelsHeight*/
const Value V23938_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23938_4 = {S54, 0, &V23938_4_1279, &P23938_5};	/*Property = ScreenPixelsWidth*/
const Profile P23938 = {"23938", &P23938_4};	/*Profile ID - 23938*/
const Value V21664_1_1 = {S2, NULL};	/*Value = True*/
const Property P21664_1 = {S50, 0, &V21664_1_1, NULL};	/*Property = IsMobile*/
const Value V21664_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21664_5 = {S52, 0, &V21664_5_1227, &P21664_1};	/*Property = ScreenPixelsHeight*/
const Value V21664_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21664_4 = {S54, 0, &V21664_4_1229, &P21664_5};	/*Property = ScreenPixelsWidth*/
const Profile P21664 = {"21664", &P21664_4};	/*Profile ID - 21664*/
const Value V21398_1_1 = {S2, NULL};	/*Value = True*/
const Property P21398_1 = {S50, 0, &V21398_1_1, NULL};	/*Property = IsMobile*/
const Value V21398_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21398_5 = {S52, 0, &V21398_5_1227, &P21398_1};	/*Property = ScreenPixelsHeight*/
const Value V21398_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21398_4 = {S54, 0, &V21398_4_1229, &P21398_5};	/*Property = ScreenPixelsWidth*/
const Profile P21398 = {"21398", &P21398_4};	/*Profile ID - 21398*/
const Value V24026_1_1 = {S2, NULL};	/*Value = True*/
const Property P24026_1 = {S50, 0, &V24026_1_1, NULL};	/*Property = IsMobile*/
const Value V24026_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24026_5 = {S52, 0, &V24026_5_1227, &P24026_1};	/*Property = ScreenPixelsHeight*/
const Value V24026_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24026_4 = {S54, 0, &V24026_4_1229, &P24026_5};	/*Property = ScreenPixelsWidth*/
const Profile P24026 = {"24026", &P24026_4};	/*Profile ID - 24026*/
const Value V20452_1_1 = {S2, NULL};	/*Value = True*/
const Property P20452_1 = {S50, 0, &V20452_1_1, NULL};	/*Property = IsMobile*/
const Value V20452_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20452_5 = {S52, 0, &V20452_5_1227, &P20452_1};	/*Property = ScreenPixelsHeight*/
const Value V20452_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20452_4 = {S54, 0, &V20452_4_1229, &P20452_5};	/*Property = ScreenPixelsWidth*/
const Profile P20452 = {"20452", &P20452_4};	/*Profile ID - 20452*/
const Value V18234_1_1 = {S2, NULL};	/*Value = True*/
const Property P18234_1 = {S50, 0, &V18234_1_1, NULL};	/*Property = IsMobile*/
const Value V18234_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P18234_5 = {S52, 0, &V18234_5_1458, &P18234_1};	/*Property = ScreenPixelsHeight*/
const Value V18234_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P18234_4 = {S54, 0, &V18234_4_37617, &P18234_5};	/*Property = ScreenPixelsWidth*/
const Profile P18234 = {"18234", &P18234_4};	/*Profile ID - 18234*/
const Value V20961_1_1 = {S2, NULL};	/*Value = True*/
const Property P20961_1 = {S50, 0, &V20961_1_1, NULL};	/*Property = IsMobile*/
const Value V20961_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20961_5 = {S52, 0, &V20961_5_1227, &P20961_1};	/*Property = ScreenPixelsHeight*/
const Value V20961_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20961_4 = {S54, 0, &V20961_4_1229, &P20961_5};	/*Property = ScreenPixelsWidth*/
const Profile P20961 = {"20961", &P20961_4};	/*Profile ID - 20961*/
const Value V20817_1_1 = {S2, NULL};	/*Value = True*/
const Property P20817_1 = {S50, 0, &V20817_1_1, NULL};	/*Property = IsMobile*/
const Value V20817_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20817_5 = {S52, 0, &V20817_5_1227, &P20817_1};	/*Property = ScreenPixelsHeight*/
const Value V20817_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20817_4 = {S54, 0, &V20817_4_1229, &P20817_5};	/*Property = ScreenPixelsWidth*/
const Profile P20817 = {"20817", &P20817_4};	/*Profile ID - 20817*/
const Value V20595_1_1 = {S2, NULL};	/*Value = True*/
const Property P20595_1 = {S50, 0, &V20595_1_1, NULL};	/*Property = IsMobile*/
const Value V20595_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20595_5 = {S52, 0, &V20595_5_1458, &P20595_1};	/*Property = ScreenPixelsHeight*/
const Value V20595_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20595_4 = {S54, 0, &V20595_4_1460, &P20595_5};	/*Property = ScreenPixelsWidth*/
const Profile P20595 = {"20595", &P20595_4};	/*Profile ID - 20595*/
const Value V18713_1_1 = {S2, NULL};	/*Value = True*/
const Property P18713_1 = {S50, 0, &V18713_1_1, NULL};	/*Property = IsMobile*/
const Value V18713_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18713_5 = {S52, 0, &V18713_5_1403, &P18713_1};	/*Property = ScreenPixelsHeight*/
const Value V18713_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18713_4 = {S54, 0, &V18713_4_1404, &P18713_5};	/*Property = ScreenPixelsWidth*/
const Profile P18713 = {"18713", &P18713_4};	/*Profile ID - 18713*/
const Value V20815_1_1 = {S2, NULL};	/*Value = True*/
const Property P20815_1 = {S50, 0, &V20815_1_1, NULL};	/*Property = IsMobile*/
const Value V20815_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20815_5 = {S52, 0, &V20815_5_1458, &P20815_1};	/*Property = ScreenPixelsHeight*/
const Value V20815_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20815_4 = {S54, 0, &V20815_4_37617, &P20815_5};	/*Property = ScreenPixelsWidth*/
const Profile P20815 = {"20815", &P20815_4};	/*Profile ID - 20815*/
const Value V21236_1_1 = {S2, NULL};	/*Value = True*/
const Property P21236_1 = {S50, 0, &V21236_1_1, NULL};	/*Property = IsMobile*/
const Value V21236_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21236_5 = {S52, 0, &V21236_5_1277, &P21236_1};	/*Property = ScreenPixelsHeight*/
const Value V21236_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21236_4 = {S54, 0, &V21236_4_2336, &P21236_5};	/*Property = ScreenPixelsWidth*/
const Profile P21236 = {"21236", &P21236_4};	/*Profile ID - 21236*/
const Value V21448_1_1 = {S2, NULL};	/*Value = True*/
const Property P21448_1 = {S50, 0, &V21448_1_1, NULL};	/*Property = IsMobile*/
const Value V21448_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21448_5 = {S52, 0, &V21448_5_1227, &P21448_1};	/*Property = ScreenPixelsHeight*/
const Value V21448_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21448_4 = {S54, 0, &V21448_4_1229, &P21448_5};	/*Property = ScreenPixelsWidth*/
const Profile P21448 = {"21448", &P21448_4};	/*Profile ID - 21448*/
const Value V23783_1_1 = {S2, NULL};	/*Value = True*/
const Property P23783_1 = {S50, 0, &V23783_1_1, NULL};	/*Property = IsMobile*/
const Value V23783_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23783_5 = {S52, 0, &V23783_5_1832, &P23783_1};	/*Property = ScreenPixelsHeight*/
const Value V23783_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23783_4 = {S54, 0, &V23783_4_2336, &P23783_5};	/*Property = ScreenPixelsWidth*/
const Profile P23783 = {"23783", &P23783_4};	/*Profile ID - 23783*/
const Value V21094_1_1 = {S2, NULL};	/*Value = True*/
const Property P21094_1 = {S50, 0, &V21094_1_1, NULL};	/*Property = IsMobile*/
const Value V21094_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21094_5 = {S52, 0, &V21094_5_1227, &P21094_1};	/*Property = ScreenPixelsHeight*/
const Value V21094_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21094_4 = {S54, 0, &V21094_4_1229, &P21094_5};	/*Property = ScreenPixelsWidth*/
const Profile P21094 = {"21094", &P21094_4};	/*Profile ID - 21094*/
const Value V22259_1_1 = {S2, NULL};	/*Value = True*/
const Property P22259_1 = {S50, 0, &V22259_1_1, NULL};	/*Property = IsMobile*/
const Value V22259_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22259_5 = {S52, 0, &V22259_5_1277, &P22259_1};	/*Property = ScreenPixelsHeight*/
const Value V22259_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22259_4 = {S54, 0, &V22259_4_1279, &P22259_5};	/*Property = ScreenPixelsWidth*/
const Profile P22259 = {"22259", &P22259_4};	/*Profile ID - 22259*/
const Value V20122_1_1 = {S2, NULL};	/*Value = True*/
const Property P20122_1 = {S50, 0, &V20122_1_1, NULL};	/*Property = IsMobile*/
const Value V20122_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20122_5 = {S52, 0, &V20122_5_1277, &P20122_1};	/*Property = ScreenPixelsHeight*/
const Value V20122_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20122_4 = {S54, 0, &V20122_4_1279, &P20122_5};	/*Property = ScreenPixelsWidth*/
const Profile P20122 = {"20122", &P20122_4};	/*Profile ID - 20122*/
const Value V24025_1_1 = {S2, NULL};	/*Value = True*/
const Property P24025_1 = {S50, 0, &V24025_1_1, NULL};	/*Property = IsMobile*/
const Value V24025_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24025_5 = {S52, 0, &V24025_5_1227, &P24025_1};	/*Property = ScreenPixelsHeight*/
const Value V24025_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24025_4 = {S54, 0, &V24025_4_1229, &P24025_5};	/*Property = ScreenPixelsWidth*/
const Profile P24025 = {"24025", &P24025_4};	/*Profile ID - 24025*/
const Value V22120_1_1 = {S2, NULL};	/*Value = True*/
const Property P22120_1 = {S50, 0, &V22120_1_1, NULL};	/*Property = IsMobile*/
const Value V22120_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22120_5 = {S52, 0, &V22120_5_37569, &P22120_1};	/*Property = ScreenPixelsHeight*/
const Value V22120_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22120_4 = {S54, 0, &V22120_4_5645, &P22120_5};	/*Property = ScreenPixelsWidth*/
const Profile P22120 = {"22120", &P22120_4};	/*Profile ID - 22120*/
const Value V19924_1_1 = {S2, NULL};	/*Value = True*/
const Property P19924_1 = {S50, 0, &V19924_1_1, NULL};	/*Property = IsMobile*/
const Value V19924_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P19924_5 = {S52, 0, &V19924_5_1458, &P19924_1};	/*Property = ScreenPixelsHeight*/
const Value V19924_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P19924_4 = {S54, 0, &V19924_4_37617, &P19924_5};	/*Property = ScreenPixelsWidth*/
const Profile P19924 = {"19924", &P19924_4};	/*Profile ID - 19924*/
const Value V20472_1_1 = {S2, NULL};	/*Value = True*/
const Property P20472_1 = {S50, 0, &V20472_1_1, NULL};	/*Property = IsMobile*/
const Value V20472_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20472_5 = {S52, 0, &V20472_5_1227, &P20472_1};	/*Property = ScreenPixelsHeight*/
const Value V20472_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20472_4 = {S54, 0, &V20472_4_1229, &P20472_5};	/*Property = ScreenPixelsWidth*/
const Profile P20472 = {"20472", &P20472_4};	/*Profile ID - 20472*/
const Value V21161_1_1 = {S2, NULL};	/*Value = True*/
const Property P21161_1 = {S50, 0, &V21161_1_1, NULL};	/*Property = IsMobile*/
const Value V21161_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21161_5 = {S52, 0, &V21161_5_37, &P21161_1};	/*Property = ScreenPixelsHeight*/
const Value V21161_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21161_4 = {S54, 0, &V21161_4_36, &P21161_5};	/*Property = ScreenPixelsWidth*/
const Profile P21161 = {"21161", &P21161_4};	/*Profile ID - 21161*/
const Value V18724_1_1 = {S2, NULL};	/*Value = True*/
const Property P18724_1 = {S50, 0, &V18724_1_1, NULL};	/*Property = IsMobile*/
const Value V18724_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P18724_5 = {S52, 0, &V18724_5_1458, &P18724_1};	/*Property = ScreenPixelsHeight*/
const Value V18724_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P18724_4 = {S54, 0, &V18724_4_1460, &P18724_5};	/*Property = ScreenPixelsWidth*/
const Profile P18724 = {"18724", &P18724_4};	/*Profile ID - 18724*/
const Value V22558_1_1 = {S2, NULL};	/*Value = True*/
const Property P22558_1 = {S50, 0, &V22558_1_1, NULL};	/*Property = IsMobile*/
const Value V22558_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22558_5 = {S52, 0, &V22558_5_1277, &P22558_1};	/*Property = ScreenPixelsHeight*/
const Value V22558_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22558_4 = {S54, 0, &V22558_4_2336, &P22558_5};	/*Property = ScreenPixelsWidth*/
const Profile P22558 = {"22558", &P22558_4};	/*Profile ID - 22558*/
const Value V15992_1_1 = {S2, NULL};	/*Value = True*/
const Property P15992_1 = {S50, 0, &V15992_1_1, NULL};	/*Property = IsMobile*/
const Value V15992_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P15992_5 = {S52, 0, &V15992_5_1403, &P15992_1};	/*Property = ScreenPixelsHeight*/
const Value V15992_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P15992_4 = {S54, 0, &V15992_4_1404, &P15992_5};	/*Property = ScreenPixelsWidth*/
const Profile P15992 = {"15992", &P15992_4};	/*Profile ID - 15992*/
const Value V21536_1_1 = {S2, NULL};	/*Value = True*/
const Property P21536_1 = {S50, 0, &V21536_1_1, NULL};	/*Property = IsMobile*/
const Value V21536_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21536_5 = {S52, 0, &V21536_5_1227, &P21536_1};	/*Property = ScreenPixelsHeight*/
const Value V21536_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21536_4 = {S54, 0, &V21536_4_1229, &P21536_5};	/*Property = ScreenPixelsWidth*/
const Profile P21536 = {"21536", &P21536_4};	/*Profile ID - 21536*/
const Value V21240_1_1 = {S2, NULL};	/*Value = True*/
const Property P21240_1 = {S50, 0, &V21240_1_1, NULL};	/*Property = IsMobile*/
const Value V21240_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21240_5 = {S52, 0, &V21240_5_1227, &P21240_1};	/*Property = ScreenPixelsHeight*/
const Value V21240_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21240_4 = {S54, 0, &V21240_4_1229, &P21240_5};	/*Property = ScreenPixelsWidth*/
const Profile P21240 = {"21240", &P21240_4};	/*Profile ID - 21240*/
const Value V22344_1_1 = {S2, NULL};	/*Value = True*/
const Property P22344_1 = {S50, 0, &V22344_1_1, NULL};	/*Property = IsMobile*/
const Value V22344_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22344_5 = {S52, 0, &V22344_5_37569, &P22344_1};	/*Property = ScreenPixelsHeight*/
const Value V22344_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22344_4 = {S54, 0, &V22344_4_5645, &P22344_5};	/*Property = ScreenPixelsWidth*/
const Profile P22344 = {"22344", &P22344_4};	/*Profile ID - 22344*/
const Value V21301_1_1 = {S2, NULL};	/*Value = True*/
const Property P21301_1 = {S50, 0, &V21301_1_1, NULL};	/*Property = IsMobile*/
const Value V21301_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21301_5 = {S52, 0, &V21301_5_1403, &P21301_1};	/*Property = ScreenPixelsHeight*/
const Value V21301_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P21301_4 = {S54, 0, &V21301_4_8554, &P21301_5};	/*Property = ScreenPixelsWidth*/
const Profile P21301 = {"21301", &P21301_4};	/*Profile ID - 21301*/
const Value V23508_1_1 = {S2, NULL};	/*Value = True*/
const Property P23508_1 = {S50, 0, &V23508_1_1, NULL};	/*Property = IsMobile*/
const Value V23508_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23508_5 = {S52, 0, &V23508_5_37, &P23508_1};	/*Property = ScreenPixelsHeight*/
const Value V23508_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23508_4 = {S54, 0, &V23508_4_36, &P23508_5};	/*Property = ScreenPixelsWidth*/
const Profile P23508 = {"23508", &P23508_4};	/*Profile ID - 23508*/
const Value V21630_1_1 = {S2, NULL};	/*Value = True*/
const Property P21630_1 = {S50, 0, &V21630_1_1, NULL};	/*Property = IsMobile*/
const Value V21630_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21630_5 = {S52, 0, &V21630_5_1227, &P21630_1};	/*Property = ScreenPixelsHeight*/
const Value V21630_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21630_4 = {S54, 0, &V21630_4_1229, &P21630_5};	/*Property = ScreenPixelsWidth*/
const Profile P21630 = {"21630", &P21630_4};	/*Profile ID - 21630*/
const Value V19911_1_1 = {S2, NULL};	/*Value = True*/
const Property P19911_1 = {S50, 0, &V19911_1_1, NULL};	/*Property = IsMobile*/
const Value V19911_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19911_5 = {S52, 0, &V19911_5_9981, &P19911_1};	/*Property = ScreenPixelsHeight*/
const Value V19911_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P19911_4 = {S54, 0, &V19911_4_37617, &P19911_5};	/*Property = ScreenPixelsWidth*/
const Profile P19911 = {"19911", &P19911_4};	/*Profile ID - 19911*/
const Value V21624_1_1 = {S2, NULL};	/*Value = True*/
const Property P21624_1 = {S50, 0, &V21624_1_1, NULL};	/*Property = IsMobile*/
const Value V21624_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21624_5 = {S52, 0, &V21624_5_1227, &P21624_1};	/*Property = ScreenPixelsHeight*/
const Value V21624_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21624_4 = {S54, 0, &V21624_4_1229, &P21624_5};	/*Property = ScreenPixelsWidth*/
const Profile P21624 = {"21624", &P21624_4};	/*Profile ID - 21624*/
const Value V23855_1_1 = {S2, NULL};	/*Value = True*/
const Property P23855_1 = {S50, 0, &V23855_1_1, NULL};	/*Property = IsMobile*/
const Value V23855_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23855_5 = {S52, 0, &V23855_5_1227, &P23855_1};	/*Property = ScreenPixelsHeight*/
const Value V23855_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23855_4 = {S54, 0, &V23855_4_1229, &P23855_5};	/*Property = ScreenPixelsWidth*/
const Profile P23855 = {"23855", &P23855_4};	/*Profile ID - 23855*/
const Value V19856_1_1 = {S2, NULL};	/*Value = True*/
const Property P19856_1 = {S50, 0, &V19856_1_1, NULL};	/*Property = IsMobile*/
const Value V19856_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19856_5 = {S52, 0, &V19856_5_9981, &P19856_1};	/*Property = ScreenPixelsHeight*/
const Value V19856_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P19856_4 = {S54, 0, &V19856_4_37617, &P19856_5};	/*Property = ScreenPixelsWidth*/
const Profile P19856 = {"19856", &P19856_4};	/*Profile ID - 19856*/
const Value V21945_1_1 = {S2, NULL};	/*Value = True*/
const Property P21945_1 = {S50, 0, &V21945_1_1, NULL};	/*Property = IsMobile*/
const Value V21945_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21945_5 = {S52, 0, &V21945_5_9981, &P21945_1};	/*Property = ScreenPixelsHeight*/
const Value V21945_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21945_4 = {S54, 0, &V21945_4_2336, &P21945_5};	/*Property = ScreenPixelsWidth*/
const Profile P21945 = {"21945", &P21945_4};	/*Profile ID - 21945*/
const Value V24024_1_1 = {S2, NULL};	/*Value = True*/
const Property P24024_1 = {S50, 0, &V24024_1_1, NULL};	/*Property = IsMobile*/
const Value V24024_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24024_5 = {S52, 0, &V24024_5_1227, &P24024_1};	/*Property = ScreenPixelsHeight*/
const Value V24024_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24024_4 = {S54, 0, &V24024_4_1229, &P24024_5};	/*Property = ScreenPixelsWidth*/
const Profile P24024 = {"24024", &P24024_4};	/*Profile ID - 24024*/
const Value V19774_1_1 = {S2, NULL};	/*Value = True*/
const Property P19774_1 = {S50, 0, &V19774_1_1, NULL};	/*Property = IsMobile*/
const Value V19774_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19774_5 = {S52, 0, &V19774_5_1227, &P19774_1};	/*Property = ScreenPixelsHeight*/
const Value V19774_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19774_4 = {S54, 0, &V19774_4_1229, &P19774_5};	/*Property = ScreenPixelsWidth*/
const Profile P19774 = {"19774", &P19774_4};	/*Profile ID - 19774*/
const Value V22818_1_1 = {S2, NULL};	/*Value = True*/
const Property P22818_1 = {S50, 0, &V22818_1_1, NULL};	/*Property = IsMobile*/
const Value V22818_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22818_5 = {S52, 0, &V22818_5_37569, &P22818_1};	/*Property = ScreenPixelsHeight*/
const Value V22818_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22818_4 = {S54, 0, &V22818_4_5645, &P22818_5};	/*Property = ScreenPixelsWidth*/
const Profile P22818 = {"22818", &P22818_4};	/*Profile ID - 22818*/
const Value V19778_1_1 = {S2, NULL};	/*Value = True*/
const Property P19778_1 = {S50, 0, &V19778_1_1, NULL};	/*Property = IsMobile*/
const Value V19778_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19778_5 = {S52, 0, &V19778_5_9981, &P19778_1};	/*Property = ScreenPixelsHeight*/
const Value V19778_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P19778_4 = {S54, 0, &V19778_4_39098, &P19778_5};	/*Property = ScreenPixelsWidth*/
const Profile P19778 = {"19778", &P19778_4};	/*Profile ID - 19778*/
const Value V21359_1_1 = {S2, NULL};	/*Value = True*/
const Property P21359_1 = {S50, 0, &V21359_1_1, NULL};	/*Property = IsMobile*/
const Value V21359_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21359_5 = {S52, 0, &V21359_5_1227, &P21359_1};	/*Property = ScreenPixelsHeight*/
const Value V21359_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21359_4 = {S54, 0, &V21359_4_1229, &P21359_5};	/*Property = ScreenPixelsWidth*/
const Profile P21359 = {"21359", &P21359_4};	/*Profile ID - 21359*/
const Value V21319_1_1 = {S2, NULL};	/*Value = True*/
const Property P21319_1 = {S50, 0, &V21319_1_1, NULL};	/*Property = IsMobile*/
const Value V21319_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21319_5 = {S52, 0, &V21319_5_1458, &P21319_1};	/*Property = ScreenPixelsHeight*/
const Value V21319_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21319_4 = {S54, 0, &V21319_4_37617, &P21319_5};	/*Property = ScreenPixelsWidth*/
const Profile P21319 = {"21319", &P21319_4};	/*Profile ID - 21319*/
const Value V23910_1_1 = {S2, NULL};	/*Value = True*/
const Property P23910_1 = {S50, 0, &V23910_1_1, NULL};	/*Property = IsMobile*/
const Value V23910_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23910_5 = {S52, 0, &V23910_5_9981, &P23910_1};	/*Property = ScreenPixelsHeight*/
const Value V23910_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23910_4 = {S54, 0, &V23910_4_39098, &P23910_5};	/*Property = ScreenPixelsWidth*/
const Profile P23910 = {"23910", &P23910_4};	/*Profile ID - 23910*/
const Value V20293_1_1 = {S2, NULL};	/*Value = True*/
const Property P20293_1 = {S50, 0, &V20293_1_1, NULL};	/*Property = IsMobile*/
const Value V20293_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20293_5 = {S52, 0, &V20293_5_1227, &P20293_1};	/*Property = ScreenPixelsHeight*/
const Value V20293_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20293_4 = {S54, 0, &V20293_4_1229, &P20293_5};	/*Property = ScreenPixelsWidth*/
const Profile P20293 = {"20293", &P20293_4};	/*Profile ID - 20293*/
const Value V20378_1_1 = {S2, NULL};	/*Value = True*/
const Property P20378_1 = {S50, 0, &V20378_1_1, NULL};	/*Property = IsMobile*/
const Value V20378_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20378_5 = {S52, 0, &V20378_5_37, &P20378_1};	/*Property = ScreenPixelsHeight*/
const Value V20378_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20378_4 = {S54, 0, &V20378_4_36, &P20378_5};	/*Property = ScreenPixelsWidth*/
const Profile P20378 = {"20378", &P20378_4};	/*Profile ID - 20378*/
const Value V22016_1_1 = {S2, NULL};	/*Value = True*/
const Property P22016_1 = {S50, 0, &V22016_1_1, NULL};	/*Property = IsMobile*/
const Value V22016_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22016_5 = {S52, 0, &V22016_5_1227, &P22016_1};	/*Property = ScreenPixelsHeight*/
const Value V22016_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22016_4 = {S54, 0, &V22016_4_1229, &P22016_5};	/*Property = ScreenPixelsWidth*/
const Profile P22016 = {"22016", &P22016_4};	/*Profile ID - 22016*/
const Value V20943_1_1 = {S2, NULL};	/*Value = True*/
const Property P20943_1 = {S50, 0, &V20943_1_1, NULL};	/*Property = IsMobile*/
const Value V20943_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20943_5 = {S52, 0, &V20943_5_1227, &P20943_1};	/*Property = ScreenPixelsHeight*/
const Value V20943_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20943_4 = {S54, 0, &V20943_4_1229, &P20943_5};	/*Property = ScreenPixelsWidth*/
const Profile P20943 = {"20943", &P20943_4};	/*Profile ID - 20943*/
const Value V20075_1_1 = {S2, NULL};	/*Value = True*/
const Property P20075_1 = {S50, 0, &V20075_1_1, NULL};	/*Property = IsMobile*/
const Value V20075_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20075_5 = {S52, 0, &V20075_5_9981, &P20075_1};	/*Property = ScreenPixelsHeight*/
const Value V20075_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20075_4 = {S54, 0, &V20075_4_39098, &P20075_5};	/*Property = ScreenPixelsWidth*/
const Profile P20075 = {"20075", &P20075_4};	/*Profile ID - 20075*/
const Value V21468_1_1 = {S2, NULL};	/*Value = True*/
const Property P21468_1 = {S50, 0, &V21468_1_1, NULL};	/*Property = IsMobile*/
const Value V21468_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21468_5 = {S52, 0, &V21468_5_1227, &P21468_1};	/*Property = ScreenPixelsHeight*/
const Value V21468_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21468_4 = {S54, 0, &V21468_4_1229, &P21468_5};	/*Property = ScreenPixelsWidth*/
const Profile P21468 = {"21468", &P21468_4};	/*Profile ID - 21468*/
const Value V21608_1_1 = {S2, NULL};	/*Value = True*/
const Property P21608_1 = {S50, 0, &V21608_1_1, NULL};	/*Property = IsMobile*/
const Value V21608_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21608_5 = {S52, 0, &V21608_5_37, &P21608_1};	/*Property = ScreenPixelsHeight*/
const Value V21608_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21608_4 = {S54, 0, &V21608_4_36, &P21608_5};	/*Property = ScreenPixelsWidth*/
const Profile P21608 = {"21608", &P21608_4};	/*Profile ID - 21608*/
const Value V21311_1_1 = {S2, NULL};	/*Value = True*/
const Property P21311_1 = {S50, 0, &V21311_1_1, NULL};	/*Property = IsMobile*/
const Value V21311_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21311_5 = {S52, 0, &V21311_5_1227, &P21311_1};	/*Property = ScreenPixelsHeight*/
const Value V21311_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21311_4 = {S54, 0, &V21311_4_1229, &P21311_5};	/*Property = ScreenPixelsWidth*/
const Profile P21311 = {"21311", &P21311_4};	/*Profile ID - 21311*/
const Value V21354_1_1 = {S2, NULL};	/*Value = True*/
const Property P21354_1 = {S50, 0, &V21354_1_1, NULL};	/*Property = IsMobile*/
const Value V21354_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21354_5 = {S52, 0, &V21354_5_1277, &P21354_1};	/*Property = ScreenPixelsHeight*/
const Value V21354_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21354_4 = {S54, 0, &V21354_4_1279, &P21354_5};	/*Property = ScreenPixelsWidth*/
const Profile P21354 = {"21354", &P21354_4};	/*Profile ID - 21354*/
const Value V24573_1_1 = {S2, NULL};	/*Value = True*/
const Property P24573_1 = {S50, 0, &V24573_1_1, NULL};	/*Property = IsMobile*/
const Value V24573_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P24573_5 = {S52, 0, &V24573_5_1269, &P24573_1};	/*Property = ScreenPixelsHeight*/
const Value V24573_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24573_4 = {S54, 0, &V24573_4_1229, &P24573_5};	/*Property = ScreenPixelsWidth*/
const Profile P24573 = {"24573", &P24573_4};	/*Profile ID - 24573*/
const Value V22084_1_1 = {S2, NULL};	/*Value = True*/
const Property P22084_1 = {S50, 0, &V22084_1_1, NULL};	/*Property = IsMobile*/
const Value V22084_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22084_5 = {S52, 0, &V22084_5_37569, &P22084_1};	/*Property = ScreenPixelsHeight*/
const Value V22084_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22084_4 = {S54, 0, &V22084_4_5645, &P22084_5};	/*Property = ScreenPixelsWidth*/
const Profile P22084 = {"22084", &P22084_4};	/*Profile ID - 22084*/
const Value V22032_1_1 = {S2, NULL};	/*Value = True*/
const Property P22032_1 = {S50, 0, &V22032_1_1, NULL};	/*Property = IsMobile*/
const Value V22032_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22032_5 = {S52, 0, &V22032_5_37569, &P22032_1};	/*Property = ScreenPixelsHeight*/
const Value V22032_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22032_4 = {S54, 0, &V22032_4_5645, &P22032_5};	/*Property = ScreenPixelsWidth*/
const Profile P22032 = {"22032", &P22032_4};	/*Profile ID - 22032*/
const Value V20175_1_1 = {S2, NULL};	/*Value = True*/
const Property P20175_1 = {S50, 0, &V20175_1_1, NULL};	/*Property = IsMobile*/
const Value V20175_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P20175_5 = {S52, 0, &V20175_5_1269, &P20175_1};	/*Property = ScreenPixelsHeight*/
const Value V20175_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20175_4 = {S54, 0, &V20175_4_1229, &P20175_5};	/*Property = ScreenPixelsWidth*/
const Profile P20175 = {"20175", &P20175_4};	/*Profile ID - 20175*/
const Value V21524_1_1 = {S2, NULL};	/*Value = True*/
const Property P21524_1 = {S50, 0, &V21524_1_1, NULL};	/*Property = IsMobile*/
const Value V21524_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21524_5 = {S52, 0, &V21524_5_1277, &P21524_1};	/*Property = ScreenPixelsHeight*/
const Value V21524_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21524_4 = {S54, 0, &V21524_4_1279, &P21524_5};	/*Property = ScreenPixelsWidth*/
const Profile P21524 = {"21524", &P21524_4};	/*Profile ID - 21524*/
const Value V24016_1_1 = {S2, NULL};	/*Value = True*/
const Property P24016_1 = {S50, 0, &V24016_1_1, NULL};	/*Property = IsMobile*/
const Value V24016_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P24016_5 = {S52, 0, &V24016_5_1222, &P24016_1};	/*Property = ScreenPixelsHeight*/
const Value V24016_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P24016_4 = {S54, 0, &V24016_4_1224, &P24016_5};	/*Property = ScreenPixelsWidth*/
const Profile P24016 = {"24016", &P24016_4};	/*Profile ID - 24016*/
const Value V19867_1_1 = {S2, NULL};	/*Value = True*/
const Property P19867_1 = {S50, 0, &V19867_1_1, NULL};	/*Property = IsMobile*/
const Value V19867_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19867_5 = {S52, 0, &V19867_5_1227, &P19867_1};	/*Property = ScreenPixelsHeight*/
const Value V19867_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19867_4 = {S54, 0, &V19867_4_1229, &P19867_5};	/*Property = ScreenPixelsWidth*/
const Profile P19867 = {"19867", &P19867_4};	/*Profile ID - 19867*/
const Value V21031_1_1 = {S2, NULL};	/*Value = True*/
const Property P21031_1 = {S50, 0, &V21031_1_1, NULL};	/*Property = IsMobile*/
const Value V21031_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21031_5 = {S52, 0, &V21031_5_1227, &P21031_1};	/*Property = ScreenPixelsHeight*/
const Value V21031_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21031_4 = {S54, 0, &V21031_4_1229, &P21031_5};	/*Property = ScreenPixelsWidth*/
const Profile P21031 = {"21031", &P21031_4};	/*Profile ID - 21031*/
const Value V20346_1_1 = {S2, NULL};	/*Value = True*/
const Property P20346_1 = {S50, 0, &V20346_1_1, NULL};	/*Property = IsMobile*/
const Value V20346_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20346_5 = {S52, 0, &V20346_5_1227, &P20346_1};	/*Property = ScreenPixelsHeight*/
const Value V20346_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20346_4 = {S54, 0, &V20346_4_1229, &P20346_5};	/*Property = ScreenPixelsWidth*/
const Profile P20346 = {"20346", &P20346_4};	/*Profile ID - 20346*/
const Value V19879_1_1 = {S2, NULL};	/*Value = True*/
const Property P19879_1 = {S50, 0, &V19879_1_1, NULL};	/*Property = IsMobile*/
const Value V19879_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19879_5 = {S52, 0, &V19879_5_9981, &P19879_1};	/*Property = ScreenPixelsHeight*/
const Value V19879_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P19879_4 = {S54, 0, &V19879_4_2336, &P19879_5};	/*Property = ScreenPixelsWidth*/
const Profile P19879 = {"19879", &P19879_4};	/*Profile ID - 19879*/
const Value V18674_1_1 = {S2, NULL};	/*Value = True*/
const Property P18674_1 = {S50, 0, &V18674_1_1, NULL};	/*Property = IsMobile*/
const Value V18674_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18674_5 = {S52, 0, &V18674_5_9981, &P18674_1};	/*Property = ScreenPixelsHeight*/
const Value V18674_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18674_4 = {S54, 0, &V18674_4_2336, &P18674_5};	/*Property = ScreenPixelsWidth*/
const Profile P18674 = {"18674", &P18674_4};	/*Profile ID - 18674*/
const Value V21166_1_1 = {S2, NULL};	/*Value = True*/
const Property P21166_1 = {S50, 0, &V21166_1_1, NULL};	/*Property = IsMobile*/
const Value V21166_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21166_5 = {S52, 0, &V21166_5_1227, &P21166_1};	/*Property = ScreenPixelsHeight*/
const Value V21166_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21166_4 = {S54, 0, &V21166_4_1229, &P21166_5};	/*Property = ScreenPixelsWidth*/
const Profile P21166 = {"21166", &P21166_4};	/*Profile ID - 21166*/
const Value V24101_1_1 = {S2, NULL};	/*Value = True*/
const Property P24101_1 = {S50, 0, &V24101_1_1, NULL};	/*Property = IsMobile*/
const Value V24101_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24101_5 = {S52, 0, &V24101_5_9981, &P24101_1};	/*Property = ScreenPixelsHeight*/
const Value V24101_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24101_4 = {S54, 0, &V24101_4_39098, &P24101_5};	/*Property = ScreenPixelsWidth*/
const Profile P24101 = {"24101", &P24101_4};	/*Profile ID - 24101*/
const Value V21948_1_1 = {S2, NULL};	/*Value = True*/
const Property P21948_1 = {S50, 0, &V21948_1_1, NULL};	/*Property = IsMobile*/
const Value V21948_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21948_5 = {S52, 0, &V21948_5_1277, &P21948_1};	/*Property = ScreenPixelsHeight*/
const Value V21948_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21948_4 = {S54, 0, &V21948_4_2336, &P21948_5};	/*Property = ScreenPixelsWidth*/
const Profile P21948 = {"21948", &P21948_4};	/*Profile ID - 21948*/
const Value V21706_1_1 = {S2, NULL};	/*Value = True*/
const Property P21706_1 = {S50, 0, &V21706_1_1, NULL};	/*Property = IsMobile*/
const Value V21706_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21706_5 = {S52, 0, &V21706_5_1227, &P21706_1};	/*Property = ScreenPixelsHeight*/
const Value V21706_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21706_4 = {S54, 0, &V21706_4_1229, &P21706_5};	/*Property = ScreenPixelsWidth*/
const Profile P21706 = {"21706", &P21706_4};	/*Profile ID - 21706*/
const Value V21397_1_1 = {S2, NULL};	/*Value = True*/
const Property P21397_1 = {S50, 0, &V21397_1_1, NULL};	/*Property = IsMobile*/
const Value V21397_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21397_5 = {S52, 0, &V21397_5_1227, &P21397_1};	/*Property = ScreenPixelsHeight*/
const Value V21397_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21397_4 = {S54, 0, &V21397_4_1229, &P21397_5};	/*Property = ScreenPixelsWidth*/
const Profile P21397 = {"21397", &P21397_4};	/*Profile ID - 21397*/
const Value V20352_1_1 = {S2, NULL};	/*Value = True*/
const Property P20352_1 = {S50, 0, &V20352_1_1, NULL};	/*Property = IsMobile*/
const Value V20352_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20352_5 = {S52, 0, &V20352_5_1227, &P20352_1};	/*Property = ScreenPixelsHeight*/
const Value V20352_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20352_4 = {S54, 0, &V20352_4_1229, &P20352_5};	/*Property = ScreenPixelsWidth*/
const Profile P20352 = {"20352", &P20352_4};	/*Profile ID - 20352*/
const Value V20396_1_2 = {S0, NULL};	/*Value = False*/
const Property P20396_1 = {S50, 0, &V20396_1_2, NULL};	/*Property = IsMobile*/
const Value V20396_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20396_5 = {S52, 0, &V20396_5_37, &P20396_1};	/*Property = ScreenPixelsHeight*/
const Value V20396_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20396_4 = {S54, 0, &V20396_4_36, &P20396_5};	/*Property = ScreenPixelsWidth*/
const Profile P20396 = {"20396", &P20396_4};	/*Profile ID - 20396*/
const Value V24033_1_1 = {S2, NULL};	/*Value = True*/
const Property P24033_1 = {S50, 0, &V24033_1_1, NULL};	/*Property = IsMobile*/
const Value V24033_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24033_5 = {S52, 0, &V24033_5_1227, &P24033_1};	/*Property = ScreenPixelsHeight*/
const Value V24033_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24033_4 = {S54, 0, &V24033_4_1229, &P24033_5};	/*Property = ScreenPixelsWidth*/
const Profile P24033 = {"24033", &P24033_4};	/*Profile ID - 24033*/
const Value V20189_1_1 = {S2, NULL};	/*Value = True*/
const Property P20189_1 = {S50, 0, &V20189_1_1, NULL};	/*Property = IsMobile*/
const Value V20189_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20189_5 = {S52, 0, &V20189_5_1458, &P20189_1};	/*Property = ScreenPixelsHeight*/
const Value V20189_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20189_4 = {S54, 0, &V20189_4_1460, &P20189_5};	/*Property = ScreenPixelsWidth*/
const Profile P20189 = {"20189", &P20189_4};	/*Profile ID - 20189*/
const Value V24020_1_1 = {S2, NULL};	/*Value = True*/
const Property P24020_1 = {S50, 0, &V24020_1_1, NULL};	/*Property = IsMobile*/
const Value V24020_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24020_5 = {S52, 0, &V24020_5_1227, &P24020_1};	/*Property = ScreenPixelsHeight*/
const Value V24020_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24020_4 = {S54, 0, &V24020_4_1229, &P24020_5};	/*Property = ScreenPixelsWidth*/
const Profile P24020 = {"24020", &P24020_4};	/*Profile ID - 24020*/
const Value V20007_1_1 = {S2, NULL};	/*Value = True*/
const Property P20007_1 = {S50, 0, &V20007_1_1, NULL};	/*Property = IsMobile*/
const Value V20007_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20007_5 = {S52, 0, &V20007_5_1403, &P20007_1};	/*Property = ScreenPixelsHeight*/
const Value V20007_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20007_4 = {S54, 0, &V20007_4_1404, &P20007_5};	/*Property = ScreenPixelsWidth*/
const Profile P20007 = {"20007", &P20007_4};	/*Profile ID - 20007*/
const Value V21127_1_1 = {S2, NULL};	/*Value = True*/
const Property P21127_1 = {S50, 0, &V21127_1_1, NULL};	/*Property = IsMobile*/
const Value V21127_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21127_5 = {S52, 0, &V21127_5_1277, &P21127_1};	/*Property = ScreenPixelsHeight*/
const Value V21127_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21127_4 = {S54, 0, &V21127_4_1279, &P21127_5};	/*Property = ScreenPixelsWidth*/
const Profile P21127 = {"21127", &P21127_4};	/*Profile ID - 21127*/
const Value V21539_1_1 = {S2, NULL};	/*Value = True*/
const Property P21539_1 = {S50, 0, &V21539_1_1, NULL};	/*Property = IsMobile*/
const Value V21539_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21539_5 = {S52, 0, &V21539_5_1227, &P21539_1};	/*Property = ScreenPixelsHeight*/
const Value V21539_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21539_4 = {S54, 0, &V21539_4_1229, &P21539_5};	/*Property = ScreenPixelsWidth*/
const Profile P21539 = {"21539", &P21539_4};	/*Profile ID - 21539*/
const Value V22035_1_1 = {S2, NULL};	/*Value = True*/
const Property P22035_1 = {S50, 0, &V22035_1_1, NULL};	/*Property = IsMobile*/
const Value V22035_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22035_5 = {S52, 0, &V22035_5_1277, &P22035_1};	/*Property = ScreenPixelsHeight*/
const Value V22035_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22035_4 = {S54, 0, &V22035_4_2336, &P22035_5};	/*Property = ScreenPixelsWidth*/
const Profile P22035 = {"22035", &P22035_4};	/*Profile ID - 22035*/
const Profile P18598 = {"18598", NULL};	/*Profile ID - 18598*/
const Value V22131_1_1 = {S2, NULL};	/*Value = True*/
const Property P22131_1 = {S50, 0, &V22131_1_1, NULL};	/*Property = IsMobile*/
const Value V22131_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22131_5 = {S52, 0, &V22131_5_1227, &P22131_1};	/*Property = ScreenPixelsHeight*/
const Value V22131_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22131_4 = {S54, 0, &V22131_4_1229, &P22131_5};	/*Property = ScreenPixelsWidth*/
const Profile P22131 = {"22131", &P22131_4};	/*Profile ID - 22131*/
const Value V18468_1_1 = {S2, NULL};	/*Value = True*/
const Property P18468_1 = {S50, 0, &V18468_1_1, NULL};	/*Property = IsMobile*/
const Value V18468_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18468_5 = {S52, 0, &V18468_5_9981, &P18468_1};	/*Property = ScreenPixelsHeight*/
const Value V18468_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18468_4 = {S54, 0, &V18468_4_2336, &P18468_5};	/*Property = ScreenPixelsWidth*/
const Profile P18468 = {"18468", &P18468_4};	/*Profile ID - 18468*/
const Value V23981_1_1 = {S2, NULL};	/*Value = True*/
const Property P23981_1 = {S50, 0, &V23981_1_1, NULL};	/*Property = IsMobile*/
const Value V23981_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23981_5 = {S52, 0, &V23981_5_1403, &P23981_1};	/*Property = ScreenPixelsHeight*/
const Value V23981_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23981_4 = {S54, 0, &V23981_4_1404, &P23981_5};	/*Property = ScreenPixelsWidth*/
const Profile P23981 = {"23981", &P23981_4};	/*Profile ID - 23981*/
const Value V21272_1_1 = {S2, NULL};	/*Value = True*/
const Property P21272_1 = {S50, 0, &V21272_1_1, NULL};	/*Property = IsMobile*/
const Value V21272_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21272_5 = {S52, 0, &V21272_5_1277, &P21272_1};	/*Property = ScreenPixelsHeight*/
const Value V21272_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21272_4 = {S54, 0, &V21272_4_1279, &P21272_5};	/*Property = ScreenPixelsWidth*/
const Profile P21272 = {"21272", &P21272_4};	/*Profile ID - 21272*/
const Value V22993_1_1 = {S2, NULL};	/*Value = True*/
const Property P22993_1 = {S50, 0, &V22993_1_1, NULL};	/*Property = IsMobile*/
const Value V22993_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22993_5 = {S52, 0, &V22993_5_1227, &P22993_1};	/*Property = ScreenPixelsHeight*/
const Value V22993_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22993_4 = {S54, 0, &V22993_4_1229, &P22993_5};	/*Property = ScreenPixelsWidth*/
const Profile P22993 = {"22993", &P22993_4};	/*Profile ID - 22993*/
const Value V21051_1_1 = {S2, NULL};	/*Value = True*/
const Property P21051_1 = {S50, 0, &V21051_1_1, NULL};	/*Property = IsMobile*/
const Value V21051_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21051_5 = {S52, 0, &V21051_5_1227, &P21051_1};	/*Property = ScreenPixelsHeight*/
const Value V21051_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21051_4 = {S54, 0, &V21051_4_1460, &P21051_5};	/*Property = ScreenPixelsWidth*/
const Profile P21051 = {"21051", &P21051_4};	/*Profile ID - 21051*/
const Value V22025_1_1 = {S2, NULL};	/*Value = True*/
const Property P22025_1 = {S50, 0, &V22025_1_1, NULL};	/*Property = IsMobile*/
const Value V22025_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22025_5 = {S52, 0, &V22025_5_1277, &P22025_1};	/*Property = ScreenPixelsHeight*/
const Value V22025_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22025_4 = {S54, 0, &V22025_4_2336, &P22025_5};	/*Property = ScreenPixelsWidth*/
const Profile P22025 = {"22025", &P22025_4};	/*Profile ID - 22025*/
const Value V21248_1_1 = {S2, NULL};	/*Value = True*/
const Property P21248_1 = {S50, 0, &V21248_1_1, NULL};	/*Property = IsMobile*/
const Value V21248_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21248_5 = {S52, 0, &V21248_5_1227, &P21248_1};	/*Property = ScreenPixelsHeight*/
const Value V21248_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21248_4 = {S54, 0, &V21248_4_1229, &P21248_5};	/*Property = ScreenPixelsWidth*/
const Profile P21248 = {"21248", &P21248_4};	/*Profile ID - 21248*/
const Value V18522_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18522_279 = {S1, 0, &V18522_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18522_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18522_282 = {S3, 0, &V18522_282_57154, &P18522_279};	/*Property = BlobBuilder*/
const Value V18522_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18522_350 = {S4, 0, &V18522_350_57444, &P18522_282};	/*Property = Canvas*/
const Value V18522_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18522_283 = {S5, 0, &V18522_283_57159, &P18522_350};	/*Property = CssBackground*/
const Value V18522_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18522_284 = {S6, 0, &V18522_284_57163, &P18522_283};	/*Property = CssBorderImage*/
const Value V18522_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18522_285 = {S7, 0, &V18522_285_57166, &P18522_284};	/*Property = CssCanvas*/
const Value V18522_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18522_286 = {S8, 0, &V18522_286_57168, &P18522_285};	/*Property = CssColor*/
const Value V18522_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18522_237 = {S9, 0, &V18522_237_55786, &P18522_286};	/*Property = CssColumn*/
const Value V18522_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18522_287 = {S10, 0, &V18522_287_57172, &P18522_237};	/*Property = CssFlexbox*/
const Value V18522_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18522_288 = {S11, 0, &V18522_288_57176, &P18522_287};	/*Property = CssFont*/
const Value V18522_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18522_289 = {S12, 0, &V18522_289_57759, &P18522_288};	/*Property = CssImages*/
const Value V18522_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18522_291 = {S13, 0, &V18522_291_57180, &P18522_289};	/*Property = CssMediaQueries*/
const Value V18522_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18522_292 = {S14, 0, &V18522_292_57183, &P18522_291};	/*Property = CssMinMax*/
const Value V18522_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18522_293 = {S15, 0, &V18522_293_57187, &P18522_292};	/*Property = CssOverflow*/
const Value V18522_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18522_294 = {S16, 0, &V18522_294_57189, &P18522_293};	/*Property = CssPosition*/
const Value V18522_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18522_295 = {S17, 0, &V18522_295_57192, &P18522_294};	/*Property = CssText*/
const Value V18522_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18522_297 = {S18, 0, &V18522_297_57195, &P18522_295};	/*Property = CssTransforms*/
const Value V18522_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18522_241 = {S19, 0, &V18522_241_55799, &P18522_297};	/*Property = CssTransitions*/
const Value V18522_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18522_298 = {S20, 0, &V18522_298_57206, &P18522_241};	/*Property = CssUI*/
const Value V18522_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18522_300 = {S21, 0, &V18522_300_57216, &P18522_298};	/*Property = DataSet*/
const Value V18522_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18522_301 = {S22, 0, &V18522_301_57218, &P18522_300};	/*Property = DataUrl*/
const Value V18522_302_57221 = {S2, NULL};	/*Value = True*/
const Property P18522_302 = {S23, 0, &V18522_302_57221, &P18522_301};	/*Property = DeviceOrientation*/
const Value V18522_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18522_304 = {S24, 0, &V18522_304_57227, &P18522_302};	/*Property = FileReader*/
const Value V18522_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18522_305 = {S25, 0, &V18522_305_57231, &P18522_304};	/*Property = FileSaver*/
const Value V18522_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18522_306 = {S26, 0, &V18522_306_57237, &P18522_305};	/*Property = FileWriter*/
const Value V18522_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18522_307 = {S27, 0, &V18522_307_57239, &P18522_306};	/*Property = FormData*/
const Value V18522_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18522_309 = {S28, 0, &V18522_309_57246, &P18522_307};	/*Property = Fullscreen*/
const Value V18522_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18522_310 = {S29, 0, &V18522_310_57248, &P18522_309};	/*Property = GeoLocation*/
const Value V18522_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18522_312 = {S30, 0, &V18522_312_57254, &P18522_310};	/*Property = History*/
const Value V18522_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18522_314 = {S31, 0, &V18522_314_57261, &P18522_312};	/*Property = Html5*/
const Value V18522_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18522_313 = {S32, 0, &V18522_313_57258, &P18522_314};	/*Property = Html-Media-Capture*/
const Value V18522_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18522_315 = {S33, 0, &V18522_315_57264, &P18522_313};	/*Property = Iframe*/
const Value V18522_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18522_316 = {S34, 0, &V18522_316_57266, &P18522_315};	/*Property = IndexedDB*/
const Value V18522_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18522_317 = {S35, 0, &V18522_317_57269, &P18522_316};	/*Property = Json*/
const Value V18522_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18522_15 = {S37, 0, &V18522_15_29, &P18522_317};	/*Property = LayoutEngine*/
const Value V18522_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18522_344 = {S38, 0, &V18522_344_57272, &P18522_15};	/*Property = Masking*/
const Value V18522_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18522_320 = {S39, 0, &V18522_320_57290, &P18522_344};	/*Property = PostMessage*/
const Value V18522_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18522_321 = {S40, 0, &V18522_321_57293, &P18522_320};	/*Property = Progress*/
const Value V18522_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18522_322 = {S41, 0, &V18522_322_57296, &P18522_321};	/*Property = Prompts*/
const Value V18522_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18522_323 = {S42, 0, &V18522_323_57299, &P18522_322};	/*Property = Selector*/
const Value V18522_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18522_324 = {S43, 0, &V18522_324_57306, &P18522_323};	/*Property = Svg*/
const Value V18522_327_57314 = {S2, NULL};	/*Value = True*/
const Property P18522_327 = {S44, 0, &V18522_327_57314, &P18522_324};	/*Property = TouchEvents*/
const Value V18522_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18522_328 = {S45, 0, &V18522_328_57318, &P18522_327};	/*Property = Track*/
const Value V18522_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18522_329 = {S46, 0, &V18522_329_57323, &P18522_328};	/*Property = Video*/
const Value V18522_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18522_330 = {S47, 0, &V18522_330_57326, &P18522_329};	/*Property = Viewport*/
const Value V18522_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18522_332 = {S48, 0, &V18522_332_57340, &P18522_330};	/*Property = WebWorkers*/
const Value V18522_333_57341 = {S2, NULL};	/*Value = True*/
const Property P18522_333 = {S49, 0, &V18522_333_57341, &P18522_332};	/*Property = Xhr2*/
const Profile P18522 = {"18522", &P18522_333};	/*Profile ID - 18522*/
const Value V22177_1_1 = {S2, NULL};	/*Value = True*/
const Property P22177_1 = {S50, 0, &V22177_1_1, NULL};	/*Property = IsMobile*/
const Value V22177_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22177_5 = {S52, 0, &V22177_5_1277, &P22177_1};	/*Property = ScreenPixelsHeight*/
const Value V22177_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22177_4 = {S54, 0, &V22177_4_2336, &P22177_5};	/*Property = ScreenPixelsWidth*/
const Profile P22177 = {"22177", &P22177_4};	/*Profile ID - 22177*/
const Value V18306_1_1 = {S2, NULL};	/*Value = True*/
const Property P18306_1 = {S50, 0, &V18306_1_1, NULL};	/*Property = IsMobile*/
const Value V18306_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18306_5 = {S52, 0, &V18306_5_9981, &P18306_1};	/*Property = ScreenPixelsHeight*/
const Value V18306_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18306_4 = {S54, 0, &V18306_4_39098, &P18306_5};	/*Property = ScreenPixelsWidth*/
const Profile P18306 = {"18306", &P18306_4};	/*Profile ID - 18306*/
const Value V20128_1_1 = {S2, NULL};	/*Value = True*/
const Property P20128_1 = {S50, 0, &V20128_1_1, NULL};	/*Property = IsMobile*/
const Value V20128_5_64188 = {S136, NULL};	/*Value = 533*/
const Property P20128_5 = {S52, 0, &V20128_5_64188, &P20128_1};	/*Property = ScreenPixelsHeight*/
const Value V20128_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20128_4 = {S54, 0, &V20128_4_1279, &P20128_5};	/*Property = ScreenPixelsWidth*/
const Profile P20128 = {"20128", &P20128_4};	/*Profile ID - 20128*/
const Value V18526_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_279 = {S1, 0, &V18526_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18526_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_282 = {S3, 0, &V18526_282_57155, &P18526_279};	/*Property = BlobBuilder*/
const Value V18526_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_350 = {S4, 0, &V18526_350_57446, &P18526_282};	/*Property = Canvas*/
const Value V18526_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_283 = {S5, 0, &V18526_283_57161, &P18526_350};	/*Property = CssBackground*/
const Value V18526_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_284 = {S6, 0, &V18526_284_57164, &P18526_283};	/*Property = CssBorderImage*/
const Value V18526_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_285 = {S7, 0, &V18526_285_57167, &P18526_284};	/*Property = CssCanvas*/
const Value V18526_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_286 = {S8, 0, &V18526_286_57170, &P18526_285};	/*Property = CssColor*/
const Value V18526_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_237 = {S9, 0, &V18526_237_55786, &P18526_286};	/*Property = CssColumn*/
const Value V18526_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_287 = {S10, 0, &V18526_287_57173, &P18526_237};	/*Property = CssFlexbox*/
const Value V18526_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_288 = {S11, 0, &V18526_288_57176, &P18526_287};	/*Property = CssFont*/
const Value V18526_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_289 = {S12, 0, &V18526_289_86899, &P18526_288};	/*Property = CssImages*/
const Value V18526_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_291 = {S13, 0, &V18526_291_57182, &P18526_289};	/*Property = CssMediaQueries*/
const Value V18526_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_292 = {S14, 0, &V18526_292_57185, &P18526_291};	/*Property = CssMinMax*/
const Value V18526_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_293 = {S15, 0, &V18526_293_57188, &P18526_292};	/*Property = CssOverflow*/
const Value V18526_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_294 = {S16, 0, &V18526_294_57191, &P18526_293};	/*Property = CssPosition*/
const Value V18526_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_295 = {S17, 0, &V18526_295_57194, &P18526_294};	/*Property = CssText*/
const Value V18526_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_297 = {S18, 0, &V18526_297_57197, &P18526_295};	/*Property = CssTransforms*/
const Value V18526_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_241 = {S19, 0, &V18526_241_55801, &P18526_297};	/*Property = CssTransitions*/
const Value V18526_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_298 = {S20, 0, &V18526_298_57208, &P18526_241};	/*Property = CssUI*/
const Value V18526_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_300 = {S21, 0, &V18526_300_57217, &P18526_298};	/*Property = DataSet*/
const Value V18526_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_301 = {S22, 0, &V18526_301_57220, &P18526_300};	/*Property = DataUrl*/
const Value V18526_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_302 = {S23, 0, &V18526_302_57223, &P18526_301};	/*Property = DeviceOrientation*/
const Value V18526_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_304 = {S24, 0, &V18526_304_57229, &P18526_302};	/*Property = FileReader*/
const Value V18526_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_305 = {S25, 0, &V18526_305_57232, &P18526_304};	/*Property = FileSaver*/
const Value V18526_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_306 = {S26, 0, &V18526_306_57238, &P18526_305};	/*Property = FileWriter*/
const Value V18526_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_307 = {S27, 0, &V18526_307_57241, &P18526_306};	/*Property = FormData*/
const Value V18526_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_309 = {S28, 0, &V18526_309_57247, &P18526_307};	/*Property = Fullscreen*/
const Value V18526_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_310 = {S29, 0, &V18526_310_57250, &P18526_309};	/*Property = GeoLocation*/
const Value V18526_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_312 = {S30, 0, &V18526_312_57256, &P18526_310};	/*Property = History*/
const Value V18526_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_314 = {S31, 0, &V18526_314_57262, &P18526_312};	/*Property = Html5*/
const Value V18526_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_313 = {S32, 0, &V18526_313_57259, &P18526_314};	/*Property = Html-Media-Capture*/
const Value V18526_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_315 = {S33, 0, &V18526_315_57265, &P18526_313};	/*Property = Iframe*/
const Value V18526_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_316 = {S34, 0, &V18526_316_57268, &P18526_315};	/*Property = IndexedDB*/
const Value V18526_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_317 = {S35, 0, &V18526_317_57271, &P18526_316};	/*Property = Json*/
const Value V18526_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18526_15 = {S37, 0, &V18526_15_31, &P18526_317};	/*Property = LayoutEngine*/
const Value V18526_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_344 = {S38, 0, &V18526_344_57274, &P18526_15};	/*Property = Masking*/
const Value V18526_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_320 = {S39, 0, &V18526_320_57292, &P18526_344};	/*Property = PostMessage*/
const Value V18526_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_321 = {S40, 0, &V18526_321_57295, &P18526_320};	/*Property = Progress*/
const Value V18526_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_322 = {S41, 0, &V18526_322_57298, &P18526_321};	/*Property = Prompts*/
const Value V18526_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_323 = {S42, 0, &V18526_323_57301, &P18526_322};	/*Property = Selector*/
const Value V18526_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_324 = {S43, 0, &V18526_324_57307, &P18526_323};	/*Property = Svg*/
const Value V18526_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_327 = {S44, 0, &V18526_327_57316, &P18526_324};	/*Property = TouchEvents*/
const Value V18526_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_328 = {S45, 0, &V18526_328_57319, &P18526_327};	/*Property = Track*/
const Value V18526_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_329 = {S46, 0, &V18526_329_57325, &P18526_328};	/*Property = Video*/
const Value V18526_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_330 = {S47, 0, &V18526_330_57328, &P18526_329};	/*Property = Viewport*/
const Value V18526_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_332 = {S48, 0, &V18526_332_57340, &P18526_330};	/*Property = WebWorkers*/
const Value V18526_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18526_333 = {S49, 0, &V18526_333_57343, &P18526_332};	/*Property = Xhr2*/
const Profile P18526 = {"18526", &P18526_333};	/*Profile ID - 18526*/
const Value V21010_1_1 = {S2, NULL};	/*Value = True*/
const Property P21010_1 = {S50, 0, &V21010_1_1, NULL};	/*Property = IsMobile*/
const Value V21010_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21010_5 = {S52, 0, &V21010_5_1277, &P21010_1};	/*Property = ScreenPixelsHeight*/
const Value V21010_4_78222 = {S137, NULL};	/*Value = 279*/
const Property P21010_4 = {S54, 0, &V21010_4_78222, &P21010_5};	/*Property = ScreenPixelsWidth*/
const Profile P21010 = {"21010", &P21010_4};	/*Profile ID - 21010*/
const Value V18663_1_1 = {S2, NULL};	/*Value = True*/
const Property P18663_1 = {S50, 0, &V18663_1_1, NULL};	/*Property = IsMobile*/
const Value V18663_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18663_5 = {S52, 0, &V18663_5_9981, &P18663_1};	/*Property = ScreenPixelsHeight*/
const Value V18663_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18663_4 = {S54, 0, &V18663_4_39098, &P18663_5};	/*Property = ScreenPixelsWidth*/
const Profile P18663 = {"18663", &P18663_4};	/*Profile ID - 18663*/
const Value V22151_1_1 = {S2, NULL};	/*Value = True*/
const Property P22151_1 = {S50, 0, &V22151_1_1, NULL};	/*Property = IsMobile*/
const Value V22151_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22151_5 = {S52, 0, &V22151_5_1227, &P22151_1};	/*Property = ScreenPixelsHeight*/
const Value V22151_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22151_4 = {S54, 0, &V22151_4_1229, &P22151_5};	/*Property = ScreenPixelsWidth*/
const Profile P22151 = {"22151", &P22151_4};	/*Profile ID - 22151*/
const Value V19775_1_1 = {S2, NULL};	/*Value = True*/
const Property P19775_1 = {S50, 0, &V19775_1_1, NULL};	/*Property = IsMobile*/
const Value V19775_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19775_5 = {S52, 0, &V19775_5_9981, &P19775_1};	/*Property = ScreenPixelsHeight*/
const Value V19775_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P19775_4 = {S54, 0, &V19775_4_2336, &P19775_5};	/*Property = ScreenPixelsWidth*/
const Profile P19775 = {"19775", &P19775_4};	/*Profile ID - 19775*/
const Value V21382_1_1 = {S2, NULL};	/*Value = True*/
const Property P21382_1 = {S50, 0, &V21382_1_1, NULL};	/*Property = IsMobile*/
const Value V21382_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21382_5 = {S52, 0, &V21382_5_1277, &P21382_1};	/*Property = ScreenPixelsHeight*/
const Value V21382_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21382_4 = {S54, 0, &V21382_4_1279, &P21382_5};	/*Property = ScreenPixelsWidth*/
const Profile P21382 = {"21382", &P21382_4};	/*Profile ID - 21382*/
const Value V21060_1_1 = {S2, NULL};	/*Value = True*/
const Property P21060_1 = {S50, 0, &V21060_1_1, NULL};	/*Property = IsMobile*/
const Value V21060_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21060_5 = {S52, 0, &V21060_5_9981, &P21060_1};	/*Property = ScreenPixelsHeight*/
const Value V21060_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21060_4 = {S54, 0, &V21060_4_2336, &P21060_5};	/*Property = ScreenPixelsWidth*/
const Profile P21060 = {"21060", &P21060_4};	/*Profile ID - 21060*/
const Value V20094_1_1 = {S2, NULL};	/*Value = True*/
const Property P20094_1 = {S50, 0, &V20094_1_1, NULL};	/*Property = IsMobile*/
const Value V20094_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20094_5 = {S52, 0, &V20094_5_1227, &P20094_1};	/*Property = ScreenPixelsHeight*/
const Value V20094_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20094_4 = {S54, 0, &V20094_4_1229, &P20094_5};	/*Property = ScreenPixelsWidth*/
const Profile P20094 = {"20094", &P20094_4};	/*Profile ID - 20094*/
const Value V21822_1_1 = {S2, NULL};	/*Value = True*/
const Property P21822_1 = {S50, 0, &V21822_1_1, NULL};	/*Property = IsMobile*/
const Value V21822_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21822_5 = {S52, 0, &V21822_5_1227, &P21822_1};	/*Property = ScreenPixelsHeight*/
const Value V21822_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21822_4 = {S54, 0, &V21822_4_1229, &P21822_5};	/*Property = ScreenPixelsWidth*/
const Profile P21822 = {"21822", &P21822_4};	/*Profile ID - 21822*/
const Value V21279_1_1 = {S2, NULL};	/*Value = True*/
const Property P21279_1 = {S50, 0, &V21279_1_1, NULL};	/*Property = IsMobile*/
const Value V21279_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21279_5 = {S52, 0, &V21279_5_1458, &P21279_1};	/*Property = ScreenPixelsHeight*/
const Value V21279_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21279_4 = {S54, 0, &V21279_4_1460, &P21279_5};	/*Property = ScreenPixelsWidth*/
const Profile P21279 = {"21279", &P21279_4};	/*Profile ID - 21279*/
const Value V22018_1_1 = {S2, NULL};	/*Value = True*/
const Property P22018_1 = {S50, 0, &V22018_1_1, NULL};	/*Property = IsMobile*/
const Value V22018_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22018_5 = {S52, 0, &V22018_5_37569, &P22018_1};	/*Property = ScreenPixelsHeight*/
const Value V22018_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22018_4 = {S54, 0, &V22018_4_5645, &P22018_5};	/*Property = ScreenPixelsWidth*/
const Profile P22018 = {"22018", &P22018_4};	/*Profile ID - 22018*/
const Value V22092_1_1 = {S2, NULL};	/*Value = True*/
const Property P22092_1 = {S50, 0, &V22092_1_1, NULL};	/*Property = IsMobile*/
const Value V22092_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22092_5 = {S52, 0, &V22092_5_37569, &P22092_1};	/*Property = ScreenPixelsHeight*/
const Value V22092_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22092_4 = {S54, 0, &V22092_4_5645, &P22092_5};	/*Property = ScreenPixelsWidth*/
const Profile P22092 = {"22092", &P22092_4};	/*Profile ID - 22092*/
const Value V22129_1_1 = {S2, NULL};	/*Value = True*/
const Property P22129_1 = {S50, 0, &V22129_1_1, NULL};	/*Property = IsMobile*/
const Value V22129_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22129_5 = {S52, 0, &V22129_5_1222, &P22129_1};	/*Property = ScreenPixelsHeight*/
const Value V22129_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22129_4 = {S54, 0, &V22129_4_1229, &P22129_5};	/*Property = ScreenPixelsWidth*/
const Profile P22129 = {"22129", &P22129_4};	/*Profile ID - 22129*/
const Value V21098_1_1 = {S2, NULL};	/*Value = True*/
const Property P21098_1 = {S50, 0, &V21098_1_1, NULL};	/*Property = IsMobile*/
const Value V21098_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21098_5 = {S52, 0, &V21098_5_1277, &P21098_1};	/*Property = ScreenPixelsHeight*/
const Value V21098_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21098_4 = {S54, 0, &V21098_4_1279, &P21098_5};	/*Property = ScreenPixelsWidth*/
const Profile P21098 = {"21098", &P21098_4};	/*Profile ID - 21098*/
const Value V22435_1_1 = {S2, NULL};	/*Value = True*/
const Property P22435_1 = {S50, 0, &V22435_1_1, NULL};	/*Property = IsMobile*/
const Value V22435_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22435_5 = {S52, 0, &V22435_5_37569, &P22435_1};	/*Property = ScreenPixelsHeight*/
const Value V22435_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22435_4 = {S54, 0, &V22435_4_5645, &P22435_5};	/*Property = ScreenPixelsWidth*/
const Profile P22435 = {"22435", &P22435_4};	/*Profile ID - 22435*/
const Value V22696_1_1 = {S2, NULL};	/*Value = True*/
const Property P22696_1 = {S50, 0, &V22696_1_1, NULL};	/*Property = IsMobile*/
const Value V22696_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22696_5 = {S52, 0, &V22696_5_1227, &P22696_1};	/*Property = ScreenPixelsHeight*/
const Value V22696_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22696_4 = {S54, 0, &V22696_4_1229, &P22696_5};	/*Property = ScreenPixelsWidth*/
const Profile P22696 = {"22696", &P22696_4};	/*Profile ID - 22696*/
const Value V20812_1_1 = {S2, NULL};	/*Value = True*/
const Property P20812_1 = {S50, 0, &V20812_1_1, NULL};	/*Property = IsMobile*/
const Value V20812_5_64958 = {S134, NULL};	/*Value = 1600*/
const Property P20812_5 = {S52, 0, &V20812_5_64958, &P20812_1};	/*Property = ScreenPixelsHeight*/
const Value V20812_4_59865 = {S135, NULL};	/*Value = 900*/
const Property P20812_4 = {S54, 0, &V20812_4_59865, &P20812_5};	/*Property = ScreenPixelsWidth*/
const Profile P20812 = {"20812", &P20812_4};	/*Profile ID - 20812*/
const Value V21157_1_1 = {S2, NULL};	/*Value = True*/
const Property P21157_1 = {S50, 0, &V21157_1_1, NULL};	/*Property = IsMobile*/
const Value V21157_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21157_5 = {S52, 0, &V21157_5_37, &P21157_1};	/*Property = ScreenPixelsHeight*/
const Value V21157_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21157_4 = {S54, 0, &V21157_4_36, &P21157_5};	/*Property = ScreenPixelsWidth*/
const Profile P21157 = {"21157", &P21157_4};	/*Profile ID - 21157*/
const Value V23827_1_1 = {S2, NULL};	/*Value = True*/
const Property P23827_1 = {S50, 0, &V23827_1_1, NULL};	/*Property = IsMobile*/
const Value V23827_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23827_5 = {S52, 0, &V23827_5_1227, &P23827_1};	/*Property = ScreenPixelsHeight*/
const Value V23827_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23827_4 = {S54, 0, &V23827_4_1229, &P23827_5};	/*Property = ScreenPixelsWidth*/
const Profile P23827 = {"23827", &P23827_4};	/*Profile ID - 23827*/
const Value V18237_1_1 = {S2, NULL};	/*Value = True*/
const Property P18237_1 = {S50, 0, &V18237_1_1, NULL};	/*Property = IsMobile*/
const Value V18237_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18237_5 = {S52, 0, &V18237_5_1227, &P18237_1};	/*Property = ScreenPixelsHeight*/
const Value V18237_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P18237_4 = {S54, 0, &V18237_4_1477, &P18237_5};	/*Property = ScreenPixelsWidth*/
const Profile P18237 = {"18237", &P18237_4};	/*Profile ID - 18237*/
const Value V22019_1_1 = {S2, NULL};	/*Value = True*/
const Property P22019_1 = {S50, 0, &V22019_1_1, NULL};	/*Property = IsMobile*/
const Value V22019_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22019_5 = {S52, 0, &V22019_5_1832, &P22019_1};	/*Property = ScreenPixelsHeight*/
const Value V22019_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22019_4 = {S54, 0, &V22019_4_5645, &P22019_5};	/*Property = ScreenPixelsWidth*/
const Profile P22019 = {"22019", &P22019_4};	/*Profile ID - 22019*/
const Value V20602_1_1 = {S2, NULL};	/*Value = True*/
const Property P20602_1 = {S50, 0, &V20602_1_1, NULL};	/*Property = IsMobile*/
const Value V20602_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20602_5 = {S52, 0, &V20602_5_1227, &P20602_1};	/*Property = ScreenPixelsHeight*/
const Value V20602_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P20602_4 = {S54, 0, &V20602_4_1943, &P20602_5};	/*Property = ScreenPixelsWidth*/
const Profile P20602 = {"20602", &P20602_4};	/*Profile ID - 20602*/
const Value V18447_1_1 = {S2, NULL};	/*Value = True*/
const Property P18447_1 = {S50, 0, &V18447_1_1, NULL};	/*Property = IsMobile*/
const Value V18447_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18447_5 = {S52, 0, &V18447_5_9981, &P18447_1};	/*Property = ScreenPixelsHeight*/
const Value V18447_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18447_4 = {S54, 0, &V18447_4_39098, &P18447_5};	/*Property = ScreenPixelsWidth*/
const Profile P18447 = {"18447", &P18447_4};	/*Profile ID - 18447*/
const Value V21554_1_1 = {S2, NULL};	/*Value = True*/
const Property P21554_1 = {S50, 0, &V21554_1_1, NULL};	/*Property = IsMobile*/
const Value V21554_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21554_5 = {S52, 0, &V21554_5_1277, &P21554_1};	/*Property = ScreenPixelsHeight*/
const Value V21554_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21554_4 = {S54, 0, &V21554_4_1279, &P21554_5};	/*Property = ScreenPixelsWidth*/
const Profile P21554 = {"21554", &P21554_4};	/*Profile ID - 21554*/
const Value V21489_1_1 = {S2, NULL};	/*Value = True*/
const Property P21489_1 = {S50, 0, &V21489_1_1, NULL};	/*Property = IsMobile*/
const Value V21489_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21489_5 = {S52, 0, &V21489_5_9981, &P21489_1};	/*Property = ScreenPixelsHeight*/
const Value V21489_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21489_4 = {S54, 0, &V21489_4_39098, &P21489_5};	/*Property = ScreenPixelsWidth*/
const Profile P21489 = {"21489", &P21489_4};	/*Profile ID - 21489*/
const Value V23926_1_1 = {S2, NULL};	/*Value = True*/
const Property P23926_1 = {S50, 0, &V23926_1_1, NULL};	/*Property = IsMobile*/
const Value V23926_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23926_5 = {S52, 0, &V23926_5_1222, &P23926_1};	/*Property = ScreenPixelsHeight*/
const Value V23926_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23926_4 = {S54, 0, &V23926_4_1224, &P23926_5};	/*Property = ScreenPixelsWidth*/
const Profile P23926 = {"23926", &P23926_4};	/*Profile ID - 23926*/
const Value V21345_1_1 = {S2, NULL};	/*Value = True*/
const Property P21345_1 = {S50, 0, &V21345_1_1, NULL};	/*Property = IsMobile*/
const Value V21345_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21345_5 = {S52, 0, &V21345_5_1227, &P21345_1};	/*Property = ScreenPixelsHeight*/
const Value V21345_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21345_4 = {S54, 0, &V21345_4_1229, &P21345_5};	/*Property = ScreenPixelsWidth*/
const Profile P21345 = {"21345", &P21345_4};	/*Profile ID - 21345*/
const Value V21834_1_1 = {S2, NULL};	/*Value = True*/
const Property P21834_1 = {S50, 0, &V21834_1_1, NULL};	/*Property = IsMobile*/
const Value V21834_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P21834_5 = {S52, 0, &V21834_5_1275, &P21834_1};	/*Property = ScreenPixelsHeight*/
const Value V21834_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21834_4 = {S54, 0, &V21834_4_1229, &P21834_5};	/*Property = ScreenPixelsWidth*/
const Profile P21834 = {"21834", &P21834_4};	/*Profile ID - 21834*/
const Value V23889_1_1 = {S2, NULL};	/*Value = True*/
const Property P23889_1 = {S50, 0, &V23889_1_1, NULL};	/*Property = IsMobile*/
const Value V23889_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23889_5 = {S52, 0, &V23889_5_1227, &P23889_1};	/*Property = ScreenPixelsHeight*/
const Value V23889_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23889_4 = {S54, 0, &V23889_4_1229, &P23889_5};	/*Property = ScreenPixelsWidth*/
const Profile P23889 = {"23889", &P23889_4};	/*Profile ID - 23889*/
const Value V20008_1_1 = {S2, NULL};	/*Value = True*/
const Property P20008_1 = {S50, 0, &V20008_1_1, NULL};	/*Property = IsMobile*/
const Value V20008_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P20008_5 = {S52, 0, &V20008_5_1269, &P20008_1};	/*Property = ScreenPixelsHeight*/
const Value V20008_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20008_4 = {S54, 0, &V20008_4_1229, &P20008_5};	/*Property = ScreenPixelsWidth*/
const Profile P20008 = {"20008", &P20008_4};	/*Profile ID - 20008*/
const Value V23982_1_1 = {S2, NULL};	/*Value = True*/
const Property P23982_1 = {S50, 0, &V23982_1_1, NULL};	/*Property = IsMobile*/
const Value V23982_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23982_5 = {S52, 0, &V23982_5_1277, &P23982_1};	/*Property = ScreenPixelsHeight*/
const Value V23982_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23982_4 = {S54, 0, &V23982_4_1279, &P23982_5};	/*Property = ScreenPixelsWidth*/
const Profile P23982 = {"23982", &P23982_4};	/*Profile ID - 23982*/
const Value V22083_1_1 = {S2, NULL};	/*Value = True*/
const Property P22083_1 = {S50, 0, &V22083_1_1, NULL};	/*Property = IsMobile*/
const Value V22083_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22083_5 = {S52, 0, &V22083_5_1227, &P22083_1};	/*Property = ScreenPixelsHeight*/
const Value V22083_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22083_4 = {S54, 0, &V22083_4_1229, &P22083_5};	/*Property = ScreenPixelsWidth*/
const Profile P22083 = {"22083", &P22083_4};	/*Profile ID - 22083*/
const Value V24004_1_1 = {S2, NULL};	/*Value = True*/
const Property P24004_1 = {S50, 0, &V24004_1_1, NULL};	/*Property = IsMobile*/
const Value V24004_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24004_5 = {S52, 0, &V24004_5_1227, &P24004_1};	/*Property = ScreenPixelsHeight*/
const Value V24004_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P24004_4 = {S54, 0, &V24004_4_1460, &P24004_5};	/*Property = ScreenPixelsWidth*/
const Profile P24004 = {"24004", &P24004_4};	/*Profile ID - 24004*/
const Value V18744_1_1 = {S2, NULL};	/*Value = True*/
const Property P18744_1 = {S50, 0, &V18744_1_1, NULL};	/*Property = IsMobile*/
const Value V18744_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18744_5 = {S52, 0, &V18744_5_1277, &P18744_1};	/*Property = ScreenPixelsHeight*/
const Value V18744_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18744_4 = {S54, 0, &V18744_4_1279, &P18744_5};	/*Property = ScreenPixelsWidth*/
const Profile P18744 = {"18744", &P18744_4};	/*Profile ID - 18744*/
const Value V20399_1_1 = {S2, NULL};	/*Value = True*/
const Property P20399_1 = {S50, 0, &V20399_1_1, NULL};	/*Property = IsMobile*/
const Value V20399_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20399_5 = {S52, 0, &V20399_5_1403, &P20399_1};	/*Property = ScreenPixelsHeight*/
const Value V20399_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20399_4 = {S54, 0, &V20399_4_1404, &P20399_5};	/*Property = ScreenPixelsWidth*/
const Profile P20399 = {"20399", &P20399_4};	/*Profile ID - 20399*/
const Value V18454_1_1 = {S2, NULL};	/*Value = True*/
const Property P18454_1 = {S50, 0, &V18454_1_1, NULL};	/*Property = IsMobile*/
const Value V18454_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18454_5 = {S52, 0, &V18454_5_9981, &P18454_1};	/*Property = ScreenPixelsHeight*/
const Value V18454_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18454_4 = {S54, 0, &V18454_4_2336, &P18454_5};	/*Property = ScreenPixelsWidth*/
const Profile P18454 = {"18454", &P18454_4};	/*Profile ID - 18454*/
const Value V21089_1_1 = {S2, NULL};	/*Value = True*/
const Property P21089_1 = {S50, 0, &V21089_1_1, NULL};	/*Property = IsMobile*/
const Value V21089_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21089_5 = {S52, 0, &V21089_5_1227, &P21089_1};	/*Property = ScreenPixelsHeight*/
const Value V21089_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21089_4 = {S54, 0, &V21089_4_1229, &P21089_5};	/*Property = ScreenPixelsWidth*/
const Profile P21089 = {"21089", &P21089_4};	/*Profile ID - 21089*/
const Value V21309_1_2 = {S0, NULL};	/*Value = False*/
const Property P21309_1 = {S50, 0, &V21309_1_2, NULL};	/*Property = IsMobile*/
const Value V21309_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21309_5 = {S52, 0, &V21309_5_9981, &P21309_1};	/*Property = ScreenPixelsHeight*/
const Value V21309_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21309_4 = {S54, 0, &V21309_4_2336, &P21309_5};	/*Property = ScreenPixelsWidth*/
const Profile P21309 = {"21309", &P21309_4};	/*Profile ID - 21309*/
const Value V21876_1_1 = {S2, NULL};	/*Value = True*/
const Property P21876_1 = {S50, 0, &V21876_1_1, NULL};	/*Property = IsMobile*/
const Value V21876_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21876_5 = {S52, 0, &V21876_5_1227, &P21876_1};	/*Property = ScreenPixelsHeight*/
const Value V21876_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21876_4 = {S54, 0, &V21876_4_1229, &P21876_5};	/*Property = ScreenPixelsWidth*/
const Profile P21876 = {"21876", &P21876_4};	/*Profile ID - 21876*/
const Value V23924_1_1 = {S2, NULL};	/*Value = True*/
const Property P23924_1 = {S50, 0, &V23924_1_1, NULL};	/*Property = IsMobile*/
const Value V23924_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23924_5 = {S52, 0, &V23924_5_1227, &P23924_1};	/*Property = ScreenPixelsHeight*/
const Value V23924_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23924_4 = {S54, 0, &V23924_4_1229, &P23924_5};	/*Property = ScreenPixelsWidth*/
const Profile P23924 = {"23924", &P23924_4};	/*Profile ID - 23924*/
const Value V24007_1_1 = {S2, NULL};	/*Value = True*/
const Property P24007_1 = {S50, 0, &V24007_1_1, NULL};	/*Property = IsMobile*/
const Value V24007_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24007_5 = {S52, 0, &V24007_5_1277, &P24007_1};	/*Property = ScreenPixelsHeight*/
const Value V24007_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24007_4 = {S54, 0, &V24007_4_1279, &P24007_5};	/*Property = ScreenPixelsWidth*/
const Profile P24007 = {"24007", &P24007_4};	/*Profile ID - 24007*/
const Value V22079_1_1 = {S2, NULL};	/*Value = True*/
const Property P22079_1 = {S50, 0, &V22079_1_1, NULL};	/*Property = IsMobile*/
const Value V22079_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22079_5 = {S52, 0, &V22079_5_1458, &P22079_1};	/*Property = ScreenPixelsHeight*/
const Value V22079_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22079_4 = {S54, 0, &V22079_4_1460, &P22079_5};	/*Property = ScreenPixelsWidth*/
const Profile P22079 = {"22079", &P22079_4};	/*Profile ID - 22079*/
const Value V21967_1_1 = {S2, NULL};	/*Value = True*/
const Property P21967_1 = {S50, 0, &V21967_1_1, NULL};	/*Property = IsMobile*/
const Value V21967_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P21967_5 = {S52, 0, &V21967_5_37569, &P21967_1};	/*Property = ScreenPixelsHeight*/
const Value V21967_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21967_4 = {S54, 0, &V21967_4_5645, &P21967_5};	/*Property = ScreenPixelsWidth*/
const Profile P21967 = {"21967", &P21967_4};	/*Profile ID - 21967*/
const Value V22260_1_1 = {S2, NULL};	/*Value = True*/
const Property P22260_1 = {S50, 0, &V22260_1_1, NULL};	/*Property = IsMobile*/
const Value V22260_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22260_5 = {S52, 0, &V22260_5_1277, &P22260_1};	/*Property = ScreenPixelsHeight*/
const Value V22260_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22260_4 = {S54, 0, &V22260_4_1279, &P22260_5};	/*Property = ScreenPixelsWidth*/
const Profile P22260 = {"22260", &P22260_4};	/*Profile ID - 22260*/
const Value V22153_1_1 = {S2, NULL};	/*Value = True*/
const Property P22153_1 = {S50, 0, &V22153_1_1, NULL};	/*Property = IsMobile*/
const Value V22153_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P22153_5 = {S52, 0, &V22153_5_1275, &P22153_1};	/*Property = ScreenPixelsHeight*/
const Value V22153_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22153_4 = {S54, 0, &V22153_4_1229, &P22153_5};	/*Property = ScreenPixelsWidth*/
const Profile P22153 = {"22153", &P22153_4};	/*Profile ID - 22153*/
const Value V21534_1_1 = {S2, NULL};	/*Value = True*/
const Property P21534_1 = {S50, 0, &V21534_1_1, NULL};	/*Property = IsMobile*/
const Value V21534_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21534_5 = {S52, 0, &V21534_5_1277, &P21534_1};	/*Property = ScreenPixelsHeight*/
const Value V21534_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21534_4 = {S54, 0, &V21534_4_1279, &P21534_5};	/*Property = ScreenPixelsWidth*/
const Profile P21534 = {"21534", &P21534_4};	/*Profile ID - 21534*/
const Value V21090_1_1 = {S2, NULL};	/*Value = True*/
const Property P21090_1 = {S50, 0, &V21090_1_1, NULL};	/*Property = IsMobile*/
const Value V21090_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21090_5 = {S52, 0, &V21090_5_1277, &P21090_1};	/*Property = ScreenPixelsHeight*/
const Value V21090_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21090_4 = {S54, 0, &V21090_4_1279, &P21090_5};	/*Property = ScreenPixelsWidth*/
const Profile P21090 = {"21090", &P21090_4};	/*Profile ID - 21090*/
const Value V24008_1_1 = {S2, NULL};	/*Value = True*/
const Property P24008_1 = {S50, 0, &V24008_1_1, NULL};	/*Property = IsMobile*/
const Value V24008_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P24008_5 = {S52, 0, &V24008_5_37, &P24008_1};	/*Property = ScreenPixelsHeight*/
const Value V24008_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P24008_4 = {S54, 0, &V24008_4_36, &P24008_5};	/*Property = ScreenPixelsWidth*/
const Profile P24008 = {"24008", &P24008_4};	/*Profile ID - 24008*/
const Value V18462_1_1 = {S2, NULL};	/*Value = True*/
const Property P18462_1 = {S50, 0, &V18462_1_1, NULL};	/*Property = IsMobile*/
const Value V18462_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18462_5 = {S52, 0, &V18462_5_1227, &P18462_1};	/*Property = ScreenPixelsHeight*/
const Value V18462_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18462_4 = {S54, 0, &V18462_4_1229, &P18462_5};	/*Property = ScreenPixelsWidth*/
const Profile P18462 = {"18462", &P18462_4};	/*Profile ID - 18462*/
const Value V18452_1_1 = {S2, NULL};	/*Value = True*/
const Property P18452_1 = {S50, 0, &V18452_1_1, NULL};	/*Property = IsMobile*/
const Value V18452_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18452_5 = {S52, 0, &V18452_5_9981, &P18452_1};	/*Property = ScreenPixelsHeight*/
const Value V18452_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18452_4 = {S54, 0, &V18452_4_39098, &P18452_5};	/*Property = ScreenPixelsWidth*/
const Profile P18452 = {"18452", &P18452_4};	/*Profile ID - 18452*/
const Value V20006_1_1 = {S2, NULL};	/*Value = True*/
const Property P20006_1 = {S50, 0, &V20006_1_1, NULL};	/*Property = IsMobile*/
const Value V20006_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20006_5 = {S52, 0, &V20006_5_9981, &P20006_1};	/*Property = ScreenPixelsHeight*/
const Value V20006_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20006_4 = {S54, 0, &V20006_4_39098, &P20006_5};	/*Property = ScreenPixelsWidth*/
const Profile P20006 = {"20006", &P20006_4};	/*Profile ID - 20006*/
const Value V18453_1_1 = {S2, NULL};	/*Value = True*/
const Property P18453_1 = {S50, 0, &V18453_1_1, NULL};	/*Property = IsMobile*/
const Value V18453_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18453_5 = {S52, 0, &V18453_5_9981, &P18453_1};	/*Property = ScreenPixelsHeight*/
const Value V18453_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18453_4 = {S54, 0, &V18453_4_39098, &P18453_5};	/*Property = ScreenPixelsWidth*/
const Profile P18453 = {"18453", &P18453_4};	/*Profile ID - 18453*/
const Value V18450_1_1 = {S2, NULL};	/*Value = True*/
const Property P18450_1 = {S50, 0, &V18450_1_1, NULL};	/*Property = IsMobile*/
const Value V18450_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18450_5 = {S52, 0, &V18450_5_1277, &P18450_1};	/*Property = ScreenPixelsHeight*/
const Value V18450_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18450_4 = {S54, 0, &V18450_4_1279, &P18450_5};	/*Property = ScreenPixelsWidth*/
const Profile P18450 = {"18450", &P18450_4};	/*Profile ID - 18450*/
const Value V18448_1_1 = {S2, NULL};	/*Value = True*/
const Property P18448_1 = {S50, 0, &V18448_1_1, NULL};	/*Property = IsMobile*/
const Value V18448_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P18448_5 = {S52, 0, &V18448_5_1269, &P18448_1};	/*Property = ScreenPixelsHeight*/
const Value V18448_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18448_4 = {S54, 0, &V18448_4_1229, &P18448_5};	/*Property = ScreenPixelsWidth*/
const Profile P18448 = {"18448", &P18448_4};	/*Profile ID - 18448*/
const Value V18451_1_1 = {S2, NULL};	/*Value = True*/
const Property P18451_1 = {S50, 0, &V18451_1_1, NULL};	/*Property = IsMobile*/
const Value V18451_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18451_5 = {S52, 0, &V18451_5_9981, &P18451_1};	/*Property = ScreenPixelsHeight*/
const Value V18451_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18451_4 = {S54, 0, &V18451_4_39098, &P18451_5};	/*Property = ScreenPixelsWidth*/
const Profile P18451 = {"18451", &P18451_4};	/*Profile ID - 18451*/
const Value V18445_1_1 = {S2, NULL};	/*Value = True*/
const Property P18445_1 = {S50, 0, &V18445_1_1, NULL};	/*Property = IsMobile*/
const Value V18445_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18445_5 = {S52, 0, &V18445_5_9981, &P18445_1};	/*Property = ScreenPixelsHeight*/
const Value V18445_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P18445_4 = {S54, 0, &V18445_4_37617, &P18445_5};	/*Property = ScreenPixelsWidth*/
const Profile P18445 = {"18445", &P18445_4};	/*Profile ID - 18445*/
const Value V18444_1_1 = {S2, NULL};	/*Value = True*/
const Property P18444_1 = {S50, 0, &V18444_1_1, NULL};	/*Property = IsMobile*/
const Value V18444_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18444_5 = {S52, 0, &V18444_5_1227, &P18444_1};	/*Property = ScreenPixelsHeight*/
const Value V18444_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18444_4 = {S54, 0, &V18444_4_1229, &P18444_5};	/*Property = ScreenPixelsWidth*/
const Profile P18444 = {"18444", &P18444_4};	/*Profile ID - 18444*/
const Value V18466_1_1 = {S2, NULL};	/*Value = True*/
const Property P18466_1 = {S50, 0, &V18466_1_1, NULL};	/*Property = IsMobile*/
const Value V18466_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18466_5 = {S52, 0, &V18466_5_9981, &P18466_1};	/*Property = ScreenPixelsHeight*/
const Value V18466_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18466_4 = {S54, 0, &V18466_4_2336, &P18466_5};	/*Property = ScreenPixelsWidth*/
const Profile P18466 = {"18466", &P18466_4};	/*Profile ID - 18466*/
const Value V19989_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_279 = {S1, 0, &V19989_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V19989_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_282 = {S3, 0, &V19989_282_57155, &P19989_279};	/*Property = BlobBuilder*/
const Value V19989_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_350 = {S4, 0, &V19989_350_57446, &P19989_282};	/*Property = Canvas*/
const Value V19989_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_283 = {S5, 0, &V19989_283_57161, &P19989_350};	/*Property = CssBackground*/
const Value V19989_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_284 = {S6, 0, &V19989_284_57164, &P19989_283};	/*Property = CssBorderImage*/
const Value V19989_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_285 = {S7, 0, &V19989_285_57167, &P19989_284};	/*Property = CssCanvas*/
const Value V19989_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_286 = {S8, 0, &V19989_286_57170, &P19989_285};	/*Property = CssColor*/
const Value V19989_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_237 = {S9, 0, &V19989_237_55786, &P19989_286};	/*Property = CssColumn*/
const Value V19989_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_287 = {S10, 0, &V19989_287_57173, &P19989_237};	/*Property = CssFlexbox*/
const Value V19989_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_288 = {S11, 0, &V19989_288_57176, &P19989_287};	/*Property = CssFont*/
const Value V19989_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_289 = {S12, 0, &V19989_289_86899, &P19989_288};	/*Property = CssImages*/
const Value V19989_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_291 = {S13, 0, &V19989_291_57182, &P19989_289};	/*Property = CssMediaQueries*/
const Value V19989_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_292 = {S14, 0, &V19989_292_57185, &P19989_291};	/*Property = CssMinMax*/
const Value V19989_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_293 = {S15, 0, &V19989_293_57188, &P19989_292};	/*Property = CssOverflow*/
const Value V19989_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_294 = {S16, 0, &V19989_294_57191, &P19989_293};	/*Property = CssPosition*/
const Value V19989_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_295 = {S17, 0, &V19989_295_57194, &P19989_294};	/*Property = CssText*/
const Value V19989_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_297 = {S18, 0, &V19989_297_57197, &P19989_295};	/*Property = CssTransforms*/
const Value V19989_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_241 = {S19, 0, &V19989_241_55801, &P19989_297};	/*Property = CssTransitions*/
const Value V19989_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_298 = {S20, 0, &V19989_298_57208, &P19989_241};	/*Property = CssUI*/
const Value V19989_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_300 = {S21, 0, &V19989_300_57217, &P19989_298};	/*Property = DataSet*/
const Value V19989_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_301 = {S22, 0, &V19989_301_57220, &P19989_300};	/*Property = DataUrl*/
const Value V19989_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_302 = {S23, 0, &V19989_302_57223, &P19989_301};	/*Property = DeviceOrientation*/
const Value V19989_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_304 = {S24, 0, &V19989_304_57229, &P19989_302};	/*Property = FileReader*/
const Value V19989_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_305 = {S25, 0, &V19989_305_57232, &P19989_304};	/*Property = FileSaver*/
const Value V19989_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_306 = {S26, 0, &V19989_306_57238, &P19989_305};	/*Property = FileWriter*/
const Value V19989_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_307 = {S27, 0, &V19989_307_57241, &P19989_306};	/*Property = FormData*/
const Value V19989_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_309 = {S28, 0, &V19989_309_57247, &P19989_307};	/*Property = Fullscreen*/
const Value V19989_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_310 = {S29, 0, &V19989_310_57250, &P19989_309};	/*Property = GeoLocation*/
const Value V19989_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_312 = {S30, 0, &V19989_312_57256, &P19989_310};	/*Property = History*/
const Value V19989_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_314 = {S31, 0, &V19989_314_57262, &P19989_312};	/*Property = Html5*/
const Value V19989_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_313 = {S32, 0, &V19989_313_57259, &P19989_314};	/*Property = Html-Media-Capture*/
const Value V19989_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_315 = {S33, 0, &V19989_315_57265, &P19989_313};	/*Property = Iframe*/
const Value V19989_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_316 = {S34, 0, &V19989_316_57268, &P19989_315};	/*Property = IndexedDB*/
const Value V19989_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_317 = {S35, 0, &V19989_317_57271, &P19989_316};	/*Property = Json*/
const Value V19989_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19989_15 = {S37, 0, &V19989_15_28, &P19989_317};	/*Property = LayoutEngine*/
const Value V19989_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_344 = {S38, 0, &V19989_344_57274, &P19989_15};	/*Property = Masking*/
const Value V19989_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_320 = {S39, 0, &V19989_320_57292, &P19989_344};	/*Property = PostMessage*/
const Value V19989_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_321 = {S40, 0, &V19989_321_57295, &P19989_320};	/*Property = Progress*/
const Value V19989_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_322 = {S41, 0, &V19989_322_57298, &P19989_321};	/*Property = Prompts*/
const Value V19989_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_323 = {S42, 0, &V19989_323_57301, &P19989_322};	/*Property = Selector*/
const Value V19989_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_324 = {S43, 0, &V19989_324_57307, &P19989_323};	/*Property = Svg*/
const Value V19989_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_327 = {S44, 0, &V19989_327_57316, &P19989_324};	/*Property = TouchEvents*/
const Value V19989_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_328 = {S45, 0, &V19989_328_57319, &P19989_327};	/*Property = Track*/
const Value V19989_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_329 = {S46, 0, &V19989_329_57325, &P19989_328};	/*Property = Video*/
const Value V19989_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_330 = {S47, 0, &V19989_330_57328, &P19989_329};	/*Property = Viewport*/
const Value V19989_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_332 = {S48, 0, &V19989_332_57340, &P19989_330};	/*Property = WebWorkers*/
const Value V19989_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P19989_333 = {S49, 0, &V19989_333_57343, &P19989_332};	/*Property = Xhr2*/
const Profile P19989 = {"19989", &P19989_333};	/*Profile ID - 19989*/
const Profile P18572 = {"18572", NULL};	/*Profile ID - 18572*/
const Value V18629_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18629_279 = {S1, 0, &V18629_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18629_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18629_282 = {S3, 0, &V18629_282_57154, &P18629_279};	/*Property = BlobBuilder*/
const Value V18629_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18629_350 = {S4, 0, &V18629_350_57444, &P18629_282};	/*Property = Canvas*/
const Value V18629_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18629_283 = {S5, 0, &V18629_283_57159, &P18629_350};	/*Property = CssBackground*/
const Value V18629_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18629_284 = {S6, 0, &V18629_284_57163, &P18629_283};	/*Property = CssBorderImage*/
const Value V18629_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18629_285 = {S7, 0, &V18629_285_57166, &P18629_284};	/*Property = CssCanvas*/
const Value V18629_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18629_286 = {S8, 0, &V18629_286_57168, &P18629_285};	/*Property = CssColor*/
const Value V18629_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18629_237 = {S9, 0, &V18629_237_55786, &P18629_286};	/*Property = CssColumn*/
const Value V18629_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18629_287 = {S10, 0, &V18629_287_57172, &P18629_237};	/*Property = CssFlexbox*/
const Value V18629_288_57174 = {S2, NULL};	/*Value = True*/
const Property P18629_288 = {S11, 0, &V18629_288_57174, &P18629_287};	/*Property = CssFont*/
const Value V18629_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18629_289 = {S12, 0, &V18629_289_57759, &P18629_288};	/*Property = CssImages*/
const Value V18629_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18629_291 = {S13, 0, &V18629_291_57180, &P18629_289};	/*Property = CssMediaQueries*/
const Value V18629_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18629_292 = {S14, 0, &V18629_292_57183, &P18629_291};	/*Property = CssMinMax*/
const Value V18629_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18629_293 = {S15, 0, &V18629_293_57187, &P18629_292};	/*Property = CssOverflow*/
const Value V18629_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18629_294 = {S16, 0, &V18629_294_57189, &P18629_293};	/*Property = CssPosition*/
const Value V18629_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18629_295 = {S17, 0, &V18629_295_57192, &P18629_294};	/*Property = CssText*/
const Value V18629_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18629_297 = {S18, 0, &V18629_297_57195, &P18629_295};	/*Property = CssTransforms*/
const Value V18629_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18629_241 = {S19, 0, &V18629_241_55799, &P18629_297};	/*Property = CssTransitions*/
const Value V18629_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18629_298 = {S20, 0, &V18629_298_57206, &P18629_241};	/*Property = CssUI*/
const Value V18629_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18629_300 = {S21, 0, &V18629_300_57216, &P18629_298};	/*Property = DataSet*/
const Value V18629_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18629_301 = {S22, 0, &V18629_301_57218, &P18629_300};	/*Property = DataUrl*/
const Value V18629_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18629_302 = {S23, 0, &V18629_302_57222, &P18629_301};	/*Property = DeviceOrientation*/
const Value V18629_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18629_304 = {S24, 0, &V18629_304_57227, &P18629_302};	/*Property = FileReader*/
const Value V18629_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18629_305 = {S25, 0, &V18629_305_57231, &P18629_304};	/*Property = FileSaver*/
const Value V18629_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18629_306 = {S26, 0, &V18629_306_57237, &P18629_305};	/*Property = FileWriter*/
const Value V18629_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18629_307 = {S27, 0, &V18629_307_57239, &P18629_306};	/*Property = FormData*/
const Value V18629_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18629_309 = {S28, 0, &V18629_309_57246, &P18629_307};	/*Property = Fullscreen*/
const Value V18629_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18629_310 = {S29, 0, &V18629_310_57248, &P18629_309};	/*Property = GeoLocation*/
const Value V18629_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18629_312 = {S30, 0, &V18629_312_57254, &P18629_310};	/*Property = History*/
const Value V18629_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18629_314 = {S31, 0, &V18629_314_57261, &P18629_312};	/*Property = Html5*/
const Value V18629_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18629_313 = {S32, 0, &V18629_313_57258, &P18629_314};	/*Property = Html-Media-Capture*/
const Value V18629_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18629_315 = {S33, 0, &V18629_315_57264, &P18629_313};	/*Property = Iframe*/
const Value V18629_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18629_316 = {S34, 0, &V18629_316_57266, &P18629_315};	/*Property = IndexedDB*/
const Value V18629_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18629_317 = {S35, 0, &V18629_317_57269, &P18629_316};	/*Property = Json*/
const Value V18629_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18629_15 = {S37, 0, &V18629_15_28, &P18629_317};	/*Property = LayoutEngine*/
const Value V18629_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18629_344 = {S38, 0, &V18629_344_57272, &P18629_15};	/*Property = Masking*/
const Value V18629_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18629_320 = {S39, 0, &V18629_320_57290, &P18629_344};	/*Property = PostMessage*/
const Value V18629_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18629_321 = {S40, 0, &V18629_321_57293, &P18629_320};	/*Property = Progress*/
const Value V18629_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18629_322 = {S41, 0, &V18629_322_57296, &P18629_321};	/*Property = Prompts*/
const Value V18629_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18629_323 = {S42, 0, &V18629_323_57299, &P18629_322};	/*Property = Selector*/
const Value V18629_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18629_324 = {S43, 0, &V18629_324_57306, &P18629_323};	/*Property = Svg*/
const Value V18629_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18629_327 = {S44, 0, &V18629_327_57315, &P18629_324};	/*Property = TouchEvents*/
const Value V18629_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18629_328 = {S45, 0, &V18629_328_57318, &P18629_327};	/*Property = Track*/
const Value V18629_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18629_329 = {S46, 0, &V18629_329_57323, &P18629_328};	/*Property = Video*/
const Value V18629_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18629_330 = {S47, 0, &V18629_330_57326, &P18629_329};	/*Property = Viewport*/
const Value V18629_332_57338 = {S2, NULL};	/*Value = True*/
const Property P18629_332 = {S48, 0, &V18629_332_57338, &P18629_330};	/*Property = WebWorkers*/
const Value V18629_333_57341 = {S2, NULL};	/*Value = True*/
const Property P18629_333 = {S49, 0, &V18629_333_57341, &P18629_332};	/*Property = Xhr2*/
const Profile P18629 = {"18629", &P18629_333};	/*Profile ID - 18629*/
const Value V22553_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22553_279 = {S1, 0, &V22553_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22553_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22553_282 = {S3, 0, &V22553_282_57153, &P22553_279};	/*Property = BlobBuilder*/
const Value V22553_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22553_350 = {S4, 0, &V22553_350_57444, &P22553_282};	/*Property = Canvas*/
const Value V22553_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22553_283 = {S5, 0, &V22553_283_57159, &P22553_350};	/*Property = CssBackground*/
const Value V22553_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22553_284 = {S6, 0, &V22553_284_57163, &P22553_283};	/*Property = CssBorderImage*/
const Value V22553_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22553_285 = {S7, 0, &V22553_285_57166, &P22553_284};	/*Property = CssCanvas*/
const Value V22553_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22553_286 = {S8, 0, &V22553_286_57168, &P22553_285};	/*Property = CssColor*/
const Value V22553_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22553_237 = {S9, 0, &V22553_237_55786, &P22553_286};	/*Property = CssColumn*/
const Value V22553_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22553_287 = {S10, 0, &V22553_287_57172, &P22553_237};	/*Property = CssFlexbox*/
const Value V22553_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22553_288 = {S11, 0, &V22553_288_57175, &P22553_287};	/*Property = CssFont*/
const Value V22553_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22553_289 = {S12, 0, &V22553_289_57759, &P22553_288};	/*Property = CssImages*/
const Value V22553_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22553_291 = {S13, 0, &V22553_291_57181, &P22553_289};	/*Property = CssMediaQueries*/
const Value V22553_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22553_292 = {S14, 0, &V22553_292_57183, &P22553_291};	/*Property = CssMinMax*/
const Value V22553_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22553_293 = {S15, 0, &V22553_293_57187, &P22553_292};	/*Property = CssOverflow*/
const Value V22553_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22553_294 = {S16, 0, &V22553_294_57189, &P22553_293};	/*Property = CssPosition*/
const Value V22553_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22553_295 = {S17, 0, &V22553_295_57192, &P22553_294};	/*Property = CssText*/
const Value V22553_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22553_297 = {S18, 0, &V22553_297_57195, &P22553_295};	/*Property = CssTransforms*/
const Value V22553_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22553_241 = {S19, 0, &V22553_241_55799, &P22553_297};	/*Property = CssTransitions*/
const Value V22553_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22553_298 = {S20, 0, &V22553_298_57206, &P22553_241};	/*Property = CssUI*/
const Value V22553_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22553_300 = {S21, 0, &V22553_300_57216, &P22553_298};	/*Property = DataSet*/
const Value V22553_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22553_301 = {S22, 0, &V22553_301_57218, &P22553_300};	/*Property = DataUrl*/
const Value V22553_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22553_302 = {S23, 0, &V22553_302_57221, &P22553_301};	/*Property = DeviceOrientation*/
const Value V22553_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22553_304 = {S24, 0, &V22553_304_57227, &P22553_302};	/*Property = FileReader*/
const Value V22553_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22553_305 = {S25, 0, &V22553_305_57231, &P22553_304};	/*Property = FileSaver*/
const Value V22553_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22553_306 = {S26, 0, &V22553_306_57237, &P22553_305};	/*Property = FileWriter*/
const Value V22553_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22553_307 = {S27, 0, &V22553_307_57239, &P22553_306};	/*Property = FormData*/
const Value V22553_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22553_309 = {S28, 0, &V22553_309_57246, &P22553_307};	/*Property = Fullscreen*/
const Value V22553_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22553_310 = {S29, 0, &V22553_310_57248, &P22553_309};	/*Property = GeoLocation*/
const Value V22553_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22553_312 = {S30, 0, &V22553_312_57254, &P22553_310};	/*Property = History*/
const Value V22553_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22553_314 = {S31, 0, &V22553_314_57261, &P22553_312};	/*Property = Html5*/
const Value V22553_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22553_313 = {S32, 0, &V22553_313_57258, &P22553_314};	/*Property = Html-Media-Capture*/
const Value V22553_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22553_315 = {S33, 0, &V22553_315_57264, &P22553_313};	/*Property = Iframe*/
const Value V22553_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22553_316 = {S34, 0, &V22553_316_57266, &P22553_315};	/*Property = IndexedDB*/
const Value V22553_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22553_317 = {S35, 0, &V22553_317_57269, &P22553_316};	/*Property = Json*/
const Value V22553_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22553_15 = {S37, 0, &V22553_15_29, &P22553_317};	/*Property = LayoutEngine*/
const Value V22553_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22553_344 = {S38, 0, &V22553_344_57272, &P22553_15};	/*Property = Masking*/
const Value V22553_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22553_320 = {S39, 0, &V22553_320_57290, &P22553_344};	/*Property = PostMessage*/
const Value V22553_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22553_321 = {S40, 0, &V22553_321_57293, &P22553_320};	/*Property = Progress*/
const Value V22553_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22553_322 = {S41, 0, &V22553_322_57296, &P22553_321};	/*Property = Prompts*/
const Value V22553_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22553_323 = {S42, 0, &V22553_323_57299, &P22553_322};	/*Property = Selector*/
const Value V22553_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22553_324 = {S43, 0, &V22553_324_57306, &P22553_323};	/*Property = Svg*/
const Value V22553_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22553_327 = {S44, 0, &V22553_327_57315, &P22553_324};	/*Property = TouchEvents*/
const Value V22553_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22553_328 = {S45, 0, &V22553_328_57318, &P22553_327};	/*Property = Track*/
const Value V22553_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22553_329 = {S46, 0, &V22553_329_57323, &P22553_328};	/*Property = Video*/
const Value V22553_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22553_330 = {S47, 0, &V22553_330_57326, &P22553_329};	/*Property = Viewport*/
const Value V22553_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22553_332 = {S48, 0, &V22553_332_57339, &P22553_330};	/*Property = WebWorkers*/
const Value V22553_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22553_333 = {S49, 0, &V22553_333_57341, &P22553_332};	/*Property = Xhr2*/
const Profile P22553 = {"22553", &P22553_333};	/*Profile ID - 22553*/
const Value V18567_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_279 = {S1, 0, &V18567_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18567_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_282 = {S3, 0, &V18567_282_57155, &P18567_279};	/*Property = BlobBuilder*/
const Value V18567_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_350 = {S4, 0, &V18567_350_57446, &P18567_282};	/*Property = Canvas*/
const Value V18567_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_283 = {S5, 0, &V18567_283_57161, &P18567_350};	/*Property = CssBackground*/
const Value V18567_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_284 = {S6, 0, &V18567_284_57164, &P18567_283};	/*Property = CssBorderImage*/
const Value V18567_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_285 = {S7, 0, &V18567_285_57167, &P18567_284};	/*Property = CssCanvas*/
const Value V18567_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_286 = {S8, 0, &V18567_286_57170, &P18567_285};	/*Property = CssColor*/
const Value V18567_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_237 = {S9, 0, &V18567_237_55786, &P18567_286};	/*Property = CssColumn*/
const Value V18567_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_287 = {S10, 0, &V18567_287_57173, &P18567_237};	/*Property = CssFlexbox*/
const Value V18567_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_288 = {S11, 0, &V18567_288_57176, &P18567_287};	/*Property = CssFont*/
const Value V18567_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_289 = {S12, 0, &V18567_289_86899, &P18567_288};	/*Property = CssImages*/
const Value V18567_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_291 = {S13, 0, &V18567_291_57182, &P18567_289};	/*Property = CssMediaQueries*/
const Value V18567_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_292 = {S14, 0, &V18567_292_57185, &P18567_291};	/*Property = CssMinMax*/
const Value V18567_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_293 = {S15, 0, &V18567_293_57188, &P18567_292};	/*Property = CssOverflow*/
const Value V18567_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_294 = {S16, 0, &V18567_294_57191, &P18567_293};	/*Property = CssPosition*/
const Value V18567_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_295 = {S17, 0, &V18567_295_57194, &P18567_294};	/*Property = CssText*/
const Value V18567_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_297 = {S18, 0, &V18567_297_57197, &P18567_295};	/*Property = CssTransforms*/
const Value V18567_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_241 = {S19, 0, &V18567_241_55801, &P18567_297};	/*Property = CssTransitions*/
const Value V18567_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_298 = {S20, 0, &V18567_298_57208, &P18567_241};	/*Property = CssUI*/
const Value V18567_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_300 = {S21, 0, &V18567_300_57217, &P18567_298};	/*Property = DataSet*/
const Value V18567_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_301 = {S22, 0, &V18567_301_57220, &P18567_300};	/*Property = DataUrl*/
const Value V18567_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_302 = {S23, 0, &V18567_302_57223, &P18567_301};	/*Property = DeviceOrientation*/
const Value V18567_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_304 = {S24, 0, &V18567_304_57229, &P18567_302};	/*Property = FileReader*/
const Value V18567_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_305 = {S25, 0, &V18567_305_57232, &P18567_304};	/*Property = FileSaver*/
const Value V18567_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_306 = {S26, 0, &V18567_306_57238, &P18567_305};	/*Property = FileWriter*/
const Value V18567_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_307 = {S27, 0, &V18567_307_57241, &P18567_306};	/*Property = FormData*/
const Value V18567_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_309 = {S28, 0, &V18567_309_57247, &P18567_307};	/*Property = Fullscreen*/
const Value V18567_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_310 = {S29, 0, &V18567_310_57250, &P18567_309};	/*Property = GeoLocation*/
const Value V18567_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_312 = {S30, 0, &V18567_312_57256, &P18567_310};	/*Property = History*/
const Value V18567_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_314 = {S31, 0, &V18567_314_57262, &P18567_312};	/*Property = Html5*/
const Value V18567_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_313 = {S32, 0, &V18567_313_57259, &P18567_314};	/*Property = Html-Media-Capture*/
const Value V18567_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_315 = {S33, 0, &V18567_315_57265, &P18567_313};	/*Property = Iframe*/
const Value V18567_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_316 = {S34, 0, &V18567_316_57268, &P18567_315};	/*Property = IndexedDB*/
const Value V18567_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_317 = {S35, 0, &V18567_317_57271, &P18567_316};	/*Property = Json*/
const Value V18567_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18567_15 = {S37, 0, &V18567_15_29, &P18567_317};	/*Property = LayoutEngine*/
const Value V18567_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_344 = {S38, 0, &V18567_344_57274, &P18567_15};	/*Property = Masking*/
const Value V18567_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_320 = {S39, 0, &V18567_320_57292, &P18567_344};	/*Property = PostMessage*/
const Value V18567_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_321 = {S40, 0, &V18567_321_57295, &P18567_320};	/*Property = Progress*/
const Value V18567_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_322 = {S41, 0, &V18567_322_57298, &P18567_321};	/*Property = Prompts*/
const Value V18567_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_323 = {S42, 0, &V18567_323_57301, &P18567_322};	/*Property = Selector*/
const Value V18567_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_324 = {S43, 0, &V18567_324_57307, &P18567_323};	/*Property = Svg*/
const Value V18567_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_327 = {S44, 0, &V18567_327_57316, &P18567_324};	/*Property = TouchEvents*/
const Value V18567_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_328 = {S45, 0, &V18567_328_57319, &P18567_327};	/*Property = Track*/
const Value V18567_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_329 = {S46, 0, &V18567_329_57325, &P18567_328};	/*Property = Video*/
const Value V18567_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_330 = {S47, 0, &V18567_330_57328, &P18567_329};	/*Property = Viewport*/
const Value V18567_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_332 = {S48, 0, &V18567_332_57340, &P18567_330};	/*Property = WebWorkers*/
const Value V18567_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18567_333 = {S49, 0, &V18567_333_57343, &P18567_332};	/*Property = Xhr2*/
const Profile P18567 = {"18567", &P18567_333};	/*Profile ID - 18567*/
const Value V22364_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22364_279 = {S1, 0, &V22364_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22364_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22364_282 = {S3, 0, &V22364_282_57153, &P22364_279};	/*Property = BlobBuilder*/
const Value V22364_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22364_350 = {S4, 0, &V22364_350_57444, &P22364_282};	/*Property = Canvas*/
const Value V22364_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22364_283 = {S5, 0, &V22364_283_57159, &P22364_350};	/*Property = CssBackground*/
const Value V22364_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22364_284 = {S6, 0, &V22364_284_57163, &P22364_283};	/*Property = CssBorderImage*/
const Value V22364_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22364_285 = {S7, 0, &V22364_285_57166, &P22364_284};	/*Property = CssCanvas*/
const Value V22364_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22364_286 = {S8, 0, &V22364_286_57168, &P22364_285};	/*Property = CssColor*/
const Value V22364_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22364_237 = {S9, 0, &V22364_237_55786, &P22364_286};	/*Property = CssColumn*/
const Value V22364_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22364_287 = {S10, 0, &V22364_287_57172, &P22364_237};	/*Property = CssFlexbox*/
const Value V22364_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22364_288 = {S11, 0, &V22364_288_57175, &P22364_287};	/*Property = CssFont*/
const Value V22364_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22364_289 = {S12, 0, &V22364_289_57759, &P22364_288};	/*Property = CssImages*/
const Value V22364_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22364_291 = {S13, 0, &V22364_291_57181, &P22364_289};	/*Property = CssMediaQueries*/
const Value V22364_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22364_292 = {S14, 0, &V22364_292_57183, &P22364_291};	/*Property = CssMinMax*/
const Value V22364_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22364_293 = {S15, 0, &V22364_293_57187, &P22364_292};	/*Property = CssOverflow*/
const Value V22364_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22364_294 = {S16, 0, &V22364_294_57189, &P22364_293};	/*Property = CssPosition*/
const Value V22364_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22364_295 = {S17, 0, &V22364_295_57192, &P22364_294};	/*Property = CssText*/
const Value V22364_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22364_297 = {S18, 0, &V22364_297_57195, &P22364_295};	/*Property = CssTransforms*/
const Value V22364_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22364_241 = {S19, 0, &V22364_241_55799, &P22364_297};	/*Property = CssTransitions*/
const Value V22364_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22364_298 = {S20, 0, &V22364_298_57206, &P22364_241};	/*Property = CssUI*/
const Value V22364_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22364_300 = {S21, 0, &V22364_300_57216, &P22364_298};	/*Property = DataSet*/
const Value V22364_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22364_301 = {S22, 0, &V22364_301_57218, &P22364_300};	/*Property = DataUrl*/
const Value V22364_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22364_302 = {S23, 0, &V22364_302_57221, &P22364_301};	/*Property = DeviceOrientation*/
const Value V22364_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22364_304 = {S24, 0, &V22364_304_57227, &P22364_302};	/*Property = FileReader*/
const Value V22364_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22364_305 = {S25, 0, &V22364_305_57231, &P22364_304};	/*Property = FileSaver*/
const Value V22364_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22364_306 = {S26, 0, &V22364_306_57237, &P22364_305};	/*Property = FileWriter*/
const Value V22364_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22364_307 = {S27, 0, &V22364_307_57239, &P22364_306};	/*Property = FormData*/
const Value V22364_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22364_309 = {S28, 0, &V22364_309_57246, &P22364_307};	/*Property = Fullscreen*/
const Value V22364_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22364_310 = {S29, 0, &V22364_310_57248, &P22364_309};	/*Property = GeoLocation*/
const Value V22364_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22364_312 = {S30, 0, &V22364_312_57254, &P22364_310};	/*Property = History*/
const Value V22364_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22364_314 = {S31, 0, &V22364_314_57261, &P22364_312};	/*Property = Html5*/
const Value V22364_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22364_313 = {S32, 0, &V22364_313_57258, &P22364_314};	/*Property = Html-Media-Capture*/
const Value V22364_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22364_315 = {S33, 0, &V22364_315_57264, &P22364_313};	/*Property = Iframe*/
const Value V22364_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22364_316 = {S34, 0, &V22364_316_57266, &P22364_315};	/*Property = IndexedDB*/
const Value V22364_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22364_317 = {S35, 0, &V22364_317_57269, &P22364_316};	/*Property = Json*/
const Value V22364_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22364_15 = {S37, 0, &V22364_15_29, &P22364_317};	/*Property = LayoutEngine*/
const Value V22364_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22364_344 = {S38, 0, &V22364_344_57272, &P22364_15};	/*Property = Masking*/
const Value V22364_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22364_320 = {S39, 0, &V22364_320_57290, &P22364_344};	/*Property = PostMessage*/
const Value V22364_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22364_321 = {S40, 0, &V22364_321_57293, &P22364_320};	/*Property = Progress*/
const Value V22364_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22364_322 = {S41, 0, &V22364_322_57296, &P22364_321};	/*Property = Prompts*/
const Value V22364_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22364_323 = {S42, 0, &V22364_323_57299, &P22364_322};	/*Property = Selector*/
const Value V22364_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22364_324 = {S43, 0, &V22364_324_57306, &P22364_323};	/*Property = Svg*/
const Value V22364_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22364_327 = {S44, 0, &V22364_327_57315, &P22364_324};	/*Property = TouchEvents*/
const Value V22364_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22364_328 = {S45, 0, &V22364_328_57318, &P22364_327};	/*Property = Track*/
const Value V22364_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22364_329 = {S46, 0, &V22364_329_57323, &P22364_328};	/*Property = Video*/
const Value V22364_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22364_330 = {S47, 0, &V22364_330_57326, &P22364_329};	/*Property = Viewport*/
const Value V22364_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22364_332 = {S48, 0, &V22364_332_57339, &P22364_330};	/*Property = WebWorkers*/
const Value V22364_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22364_333 = {S49, 0, &V22364_333_57341, &P22364_332};	/*Property = Xhr2*/
const Profile P22364 = {"22364", &P22364_333};	/*Profile ID - 22364*/
const Value V18617_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18617_279 = {S1, 0, &V18617_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18617_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18617_282 = {S3, 0, &V18617_282_57154, &P18617_279};	/*Property = BlobBuilder*/
const Value V18617_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18617_350 = {S4, 0, &V18617_350_57444, &P18617_282};	/*Property = Canvas*/
const Value V18617_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18617_283 = {S5, 0, &V18617_283_57159, &P18617_350};	/*Property = CssBackground*/
const Value V18617_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18617_284 = {S6, 0, &V18617_284_57163, &P18617_283};	/*Property = CssBorderImage*/
const Value V18617_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18617_285 = {S7, 0, &V18617_285_57166, &P18617_284};	/*Property = CssCanvas*/
const Value V18617_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18617_286 = {S8, 0, &V18617_286_57168, &P18617_285};	/*Property = CssColor*/
const Value V18617_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18617_237 = {S9, 0, &V18617_237_55786, &P18617_286};	/*Property = CssColumn*/
const Value V18617_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18617_287 = {S10, 0, &V18617_287_57172, &P18617_237};	/*Property = CssFlexbox*/
const Value V18617_288_57174 = {S2, NULL};	/*Value = True*/
const Property P18617_288 = {S11, 0, &V18617_288_57174, &P18617_287};	/*Property = CssFont*/
const Value V18617_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18617_289 = {S12, 0, &V18617_289_57759, &P18617_288};	/*Property = CssImages*/
const Value V18617_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18617_291 = {S13, 0, &V18617_291_57180, &P18617_289};	/*Property = CssMediaQueries*/
const Value V18617_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18617_292 = {S14, 0, &V18617_292_57183, &P18617_291};	/*Property = CssMinMax*/
const Value V18617_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18617_293 = {S15, 0, &V18617_293_57187, &P18617_292};	/*Property = CssOverflow*/
const Value V18617_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18617_294 = {S16, 0, &V18617_294_57189, &P18617_293};	/*Property = CssPosition*/
const Value V18617_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18617_295 = {S17, 0, &V18617_295_57192, &P18617_294};	/*Property = CssText*/
const Value V18617_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18617_297 = {S18, 0, &V18617_297_57195, &P18617_295};	/*Property = CssTransforms*/
const Value V18617_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18617_241 = {S19, 0, &V18617_241_55799, &P18617_297};	/*Property = CssTransitions*/
const Value V18617_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18617_298 = {S20, 0, &V18617_298_57206, &P18617_241};	/*Property = CssUI*/
const Value V18617_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18617_300 = {S21, 0, &V18617_300_57216, &P18617_298};	/*Property = DataSet*/
const Value V18617_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18617_301 = {S22, 0, &V18617_301_57218, &P18617_300};	/*Property = DataUrl*/
const Value V18617_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18617_302 = {S23, 0, &V18617_302_57222, &P18617_301};	/*Property = DeviceOrientation*/
const Value V18617_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18617_304 = {S24, 0, &V18617_304_57227, &P18617_302};	/*Property = FileReader*/
const Value V18617_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18617_305 = {S25, 0, &V18617_305_57231, &P18617_304};	/*Property = FileSaver*/
const Value V18617_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18617_306 = {S26, 0, &V18617_306_57237, &P18617_305};	/*Property = FileWriter*/
const Value V18617_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18617_307 = {S27, 0, &V18617_307_57239, &P18617_306};	/*Property = FormData*/
const Value V18617_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18617_309 = {S28, 0, &V18617_309_57246, &P18617_307};	/*Property = Fullscreen*/
const Value V18617_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18617_310 = {S29, 0, &V18617_310_57248, &P18617_309};	/*Property = GeoLocation*/
const Value V18617_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18617_312 = {S30, 0, &V18617_312_57254, &P18617_310};	/*Property = History*/
const Value V18617_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18617_314 = {S31, 0, &V18617_314_57261, &P18617_312};	/*Property = Html5*/
const Value V18617_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18617_313 = {S32, 0, &V18617_313_57258, &P18617_314};	/*Property = Html-Media-Capture*/
const Value V18617_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18617_315 = {S33, 0, &V18617_315_57264, &P18617_313};	/*Property = Iframe*/
const Value V18617_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18617_316 = {S34, 0, &V18617_316_57266, &P18617_315};	/*Property = IndexedDB*/
const Value V18617_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18617_317 = {S35, 0, &V18617_317_57269, &P18617_316};	/*Property = Json*/
const Value V18617_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18617_15 = {S37, 0, &V18617_15_28, &P18617_317};	/*Property = LayoutEngine*/
const Value V18617_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18617_344 = {S38, 0, &V18617_344_57272, &P18617_15};	/*Property = Masking*/
const Value V18617_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18617_320 = {S39, 0, &V18617_320_57290, &P18617_344};	/*Property = PostMessage*/
const Value V18617_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18617_321 = {S40, 0, &V18617_321_57293, &P18617_320};	/*Property = Progress*/
const Value V18617_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18617_322 = {S41, 0, &V18617_322_57296, &P18617_321};	/*Property = Prompts*/
const Value V18617_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18617_323 = {S42, 0, &V18617_323_57299, &P18617_322};	/*Property = Selector*/
const Value V18617_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18617_324 = {S43, 0, &V18617_324_57306, &P18617_323};	/*Property = Svg*/
const Value V18617_327_57314 = {S2, NULL};	/*Value = True*/
const Property P18617_327 = {S44, 0, &V18617_327_57314, &P18617_324};	/*Property = TouchEvents*/
const Value V18617_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18617_328 = {S45, 0, &V18617_328_57318, &P18617_327};	/*Property = Track*/
const Value V18617_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18617_329 = {S46, 0, &V18617_329_57323, &P18617_328};	/*Property = Video*/
const Value V18617_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18617_330 = {S47, 0, &V18617_330_57326, &P18617_329};	/*Property = Viewport*/
const Value V18617_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18617_332 = {S48, 0, &V18617_332_57339, &P18617_330};	/*Property = WebWorkers*/
const Value V18617_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18617_333 = {S49, 0, &V18617_333_57342, &P18617_332};	/*Property = Xhr2*/
const Profile P18617 = {"18617", &P18617_333};	/*Profile ID - 18617*/
const Value V18412_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_279 = {S1, 0, &V18412_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18412_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_282 = {S3, 0, &V18412_282_57155, &P18412_279};	/*Property = BlobBuilder*/
const Value V18412_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_350 = {S4, 0, &V18412_350_57446, &P18412_282};	/*Property = Canvas*/
const Value V18412_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_283 = {S5, 0, &V18412_283_57161, &P18412_350};	/*Property = CssBackground*/
const Value V18412_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_284 = {S6, 0, &V18412_284_57164, &P18412_283};	/*Property = CssBorderImage*/
const Value V18412_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_285 = {S7, 0, &V18412_285_57167, &P18412_284};	/*Property = CssCanvas*/
const Value V18412_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_286 = {S8, 0, &V18412_286_57170, &P18412_285};	/*Property = CssColor*/
const Value V18412_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_237 = {S9, 0, &V18412_237_55786, &P18412_286};	/*Property = CssColumn*/
const Value V18412_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_287 = {S10, 0, &V18412_287_57173, &P18412_237};	/*Property = CssFlexbox*/
const Value V18412_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_288 = {S11, 0, &V18412_288_57176, &P18412_287};	/*Property = CssFont*/
const Value V18412_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_289 = {S12, 0, &V18412_289_86899, &P18412_288};	/*Property = CssImages*/
const Value V18412_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_291 = {S13, 0, &V18412_291_57182, &P18412_289};	/*Property = CssMediaQueries*/
const Value V18412_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_292 = {S14, 0, &V18412_292_57185, &P18412_291};	/*Property = CssMinMax*/
const Value V18412_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_293 = {S15, 0, &V18412_293_57188, &P18412_292};	/*Property = CssOverflow*/
const Value V18412_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_294 = {S16, 0, &V18412_294_57191, &P18412_293};	/*Property = CssPosition*/
const Value V18412_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_295 = {S17, 0, &V18412_295_57194, &P18412_294};	/*Property = CssText*/
const Value V18412_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_297 = {S18, 0, &V18412_297_57197, &P18412_295};	/*Property = CssTransforms*/
const Value V18412_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_241 = {S19, 0, &V18412_241_55801, &P18412_297};	/*Property = CssTransitions*/
const Value V18412_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_298 = {S20, 0, &V18412_298_57208, &P18412_241};	/*Property = CssUI*/
const Value V18412_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_300 = {S21, 0, &V18412_300_57217, &P18412_298};	/*Property = DataSet*/
const Value V18412_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_301 = {S22, 0, &V18412_301_57220, &P18412_300};	/*Property = DataUrl*/
const Value V18412_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_302 = {S23, 0, &V18412_302_57223, &P18412_301};	/*Property = DeviceOrientation*/
const Value V18412_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_304 = {S24, 0, &V18412_304_57229, &P18412_302};	/*Property = FileReader*/
const Value V18412_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_305 = {S25, 0, &V18412_305_57232, &P18412_304};	/*Property = FileSaver*/
const Value V18412_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_306 = {S26, 0, &V18412_306_57238, &P18412_305};	/*Property = FileWriter*/
const Value V18412_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_307 = {S27, 0, &V18412_307_57241, &P18412_306};	/*Property = FormData*/
const Value V18412_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_309 = {S28, 0, &V18412_309_57247, &P18412_307};	/*Property = Fullscreen*/
const Value V18412_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_310 = {S29, 0, &V18412_310_57250, &P18412_309};	/*Property = GeoLocation*/
const Value V18412_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_312 = {S30, 0, &V18412_312_57256, &P18412_310};	/*Property = History*/
const Value V18412_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_314 = {S31, 0, &V18412_314_57262, &P18412_312};	/*Property = Html5*/
const Value V18412_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_313 = {S32, 0, &V18412_313_57259, &P18412_314};	/*Property = Html-Media-Capture*/
const Value V18412_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_315 = {S33, 0, &V18412_315_57265, &P18412_313};	/*Property = Iframe*/
const Value V18412_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_316 = {S34, 0, &V18412_316_57268, &P18412_315};	/*Property = IndexedDB*/
const Value V18412_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_317 = {S35, 0, &V18412_317_57271, &P18412_316};	/*Property = Json*/
const Value V18412_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18412_15 = {S37, 0, &V18412_15_31, &P18412_317};	/*Property = LayoutEngine*/
const Value V18412_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_344 = {S38, 0, &V18412_344_57274, &P18412_15};	/*Property = Masking*/
const Value V18412_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_320 = {S39, 0, &V18412_320_57292, &P18412_344};	/*Property = PostMessage*/
const Value V18412_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_321 = {S40, 0, &V18412_321_57295, &P18412_320};	/*Property = Progress*/
const Value V18412_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_322 = {S41, 0, &V18412_322_57298, &P18412_321};	/*Property = Prompts*/
const Value V18412_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_323 = {S42, 0, &V18412_323_57301, &P18412_322};	/*Property = Selector*/
const Value V18412_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_324 = {S43, 0, &V18412_324_57307, &P18412_323};	/*Property = Svg*/
const Value V18412_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_327 = {S44, 0, &V18412_327_57316, &P18412_324};	/*Property = TouchEvents*/
const Value V18412_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_328 = {S45, 0, &V18412_328_57319, &P18412_327};	/*Property = Track*/
const Value V18412_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_329 = {S46, 0, &V18412_329_57325, &P18412_328};	/*Property = Video*/
const Value V18412_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_330 = {S47, 0, &V18412_330_57328, &P18412_329};	/*Property = Viewport*/
const Value V18412_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_332 = {S48, 0, &V18412_332_57340, &P18412_330};	/*Property = WebWorkers*/
const Value V18412_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18412_333 = {S49, 0, &V18412_333_57343, &P18412_332};	/*Property = Xhr2*/
const Profile P18412 = {"18412", &P18412_333};	/*Profile ID - 18412*/
const Value V23025_1_1 = {S2, NULL};	/*Value = True*/
const Property P23025_1 = {S50, 0, &V23025_1_1, NULL};	/*Property = IsMobile*/
const Value V23025_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23025_5 = {S52, 0, &V23025_5_1222, &P23025_1};	/*Property = ScreenPixelsHeight*/
const Value V23025_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23025_4 = {S54, 0, &V23025_4_1224, &P23025_5};	/*Property = ScreenPixelsWidth*/
const Profile P23025 = {"23025", &P23025_4};	/*Profile ID - 23025*/
const Value V24438_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_279 = {S1, 0, &V24438_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24438_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_282 = {S3, 0, &V24438_282_57155, &P24438_279};	/*Property = BlobBuilder*/
const Value V24438_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_350 = {S4, 0, &V24438_350_57446, &P24438_282};	/*Property = Canvas*/
const Value V24438_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_283 = {S5, 0, &V24438_283_57161, &P24438_350};	/*Property = CssBackground*/
const Value V24438_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_284 = {S6, 0, &V24438_284_57164, &P24438_283};	/*Property = CssBorderImage*/
const Value V24438_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_285 = {S7, 0, &V24438_285_57167, &P24438_284};	/*Property = CssCanvas*/
const Value V24438_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_286 = {S8, 0, &V24438_286_57170, &P24438_285};	/*Property = CssColor*/
const Value V24438_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_237 = {S9, 0, &V24438_237_55786, &P24438_286};	/*Property = CssColumn*/
const Value V24438_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_287 = {S10, 0, &V24438_287_57173, &P24438_237};	/*Property = CssFlexbox*/
const Value V24438_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_288 = {S11, 0, &V24438_288_57176, &P24438_287};	/*Property = CssFont*/
const Value V24438_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_289 = {S12, 0, &V24438_289_86899, &P24438_288};	/*Property = CssImages*/
const Value V24438_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_291 = {S13, 0, &V24438_291_57182, &P24438_289};	/*Property = CssMediaQueries*/
const Value V24438_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_292 = {S14, 0, &V24438_292_57185, &P24438_291};	/*Property = CssMinMax*/
const Value V24438_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_293 = {S15, 0, &V24438_293_57188, &P24438_292};	/*Property = CssOverflow*/
const Value V24438_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_294 = {S16, 0, &V24438_294_57191, &P24438_293};	/*Property = CssPosition*/
const Value V24438_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_295 = {S17, 0, &V24438_295_57194, &P24438_294};	/*Property = CssText*/
const Value V24438_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_297 = {S18, 0, &V24438_297_57197, &P24438_295};	/*Property = CssTransforms*/
const Value V24438_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_241 = {S19, 0, &V24438_241_55801, &P24438_297};	/*Property = CssTransitions*/
const Value V24438_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_298 = {S20, 0, &V24438_298_57208, &P24438_241};	/*Property = CssUI*/
const Value V24438_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_300 = {S21, 0, &V24438_300_57217, &P24438_298};	/*Property = DataSet*/
const Value V24438_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_301 = {S22, 0, &V24438_301_57220, &P24438_300};	/*Property = DataUrl*/
const Value V24438_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_302 = {S23, 0, &V24438_302_57223, &P24438_301};	/*Property = DeviceOrientation*/
const Value V24438_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_304 = {S24, 0, &V24438_304_57229, &P24438_302};	/*Property = FileReader*/
const Value V24438_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_305 = {S25, 0, &V24438_305_57232, &P24438_304};	/*Property = FileSaver*/
const Value V24438_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_306 = {S26, 0, &V24438_306_57238, &P24438_305};	/*Property = FileWriter*/
const Value V24438_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_307 = {S27, 0, &V24438_307_57241, &P24438_306};	/*Property = FormData*/
const Value V24438_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_309 = {S28, 0, &V24438_309_57247, &P24438_307};	/*Property = Fullscreen*/
const Value V24438_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_310 = {S29, 0, &V24438_310_57250, &P24438_309};	/*Property = GeoLocation*/
const Value V24438_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_312 = {S30, 0, &V24438_312_57256, &P24438_310};	/*Property = History*/
const Value V24438_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_314 = {S31, 0, &V24438_314_57262, &P24438_312};	/*Property = Html5*/
const Value V24438_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_313 = {S32, 0, &V24438_313_57259, &P24438_314};	/*Property = Html-Media-Capture*/
const Value V24438_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_315 = {S33, 0, &V24438_315_57265, &P24438_313};	/*Property = Iframe*/
const Value V24438_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_316 = {S34, 0, &V24438_316_57268, &P24438_315};	/*Property = IndexedDB*/
const Value V24438_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_317 = {S35, 0, &V24438_317_57271, &P24438_316};	/*Property = Json*/
const Value V24438_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24438_15 = {S37, 0, &V24438_15_28, &P24438_317};	/*Property = LayoutEngine*/
const Value V24438_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_344 = {S38, 0, &V24438_344_57274, &P24438_15};	/*Property = Masking*/
const Value V24438_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_320 = {S39, 0, &V24438_320_57292, &P24438_344};	/*Property = PostMessage*/
const Value V24438_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_321 = {S40, 0, &V24438_321_57295, &P24438_320};	/*Property = Progress*/
const Value V24438_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_322 = {S41, 0, &V24438_322_57298, &P24438_321};	/*Property = Prompts*/
const Value V24438_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_323 = {S42, 0, &V24438_323_57301, &P24438_322};	/*Property = Selector*/
const Value V24438_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_324 = {S43, 0, &V24438_324_57307, &P24438_323};	/*Property = Svg*/
const Value V24438_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_327 = {S44, 0, &V24438_327_57316, &P24438_324};	/*Property = TouchEvents*/
const Value V24438_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_328 = {S45, 0, &V24438_328_57319, &P24438_327};	/*Property = Track*/
const Value V24438_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_329 = {S46, 0, &V24438_329_57325, &P24438_328};	/*Property = Video*/
const Value V24438_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_330 = {S47, 0, &V24438_330_57328, &P24438_329};	/*Property = Viewport*/
const Value V24438_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_332 = {S48, 0, &V24438_332_57340, &P24438_330};	/*Property = WebWorkers*/
const Value V24438_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24438_333 = {S49, 0, &V24438_333_57343, &P24438_332};	/*Property = Xhr2*/
const Profile P24438 = {"24438", &P24438_333};	/*Profile ID - 24438*/
const Value V20911_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20911_279 = {S1, 0, &V20911_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20911_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20911_282 = {S3, 0, &V20911_282_57154, &P20911_279};	/*Property = BlobBuilder*/
const Value V20911_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20911_350 = {S4, 0, &V20911_350_57444, &P20911_282};	/*Property = Canvas*/
const Value V20911_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20911_283 = {S5, 0, &V20911_283_57159, &P20911_350};	/*Property = CssBackground*/
const Value V20911_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20911_284 = {S6, 0, &V20911_284_57163, &P20911_283};	/*Property = CssBorderImage*/
const Value V20911_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20911_285 = {S7, 0, &V20911_285_57166, &P20911_284};	/*Property = CssCanvas*/
const Value V20911_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20911_286 = {S8, 0, &V20911_286_57168, &P20911_285};	/*Property = CssColor*/
const Value V20911_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20911_237 = {S9, 0, &V20911_237_55786, &P20911_286};	/*Property = CssColumn*/
const Value V20911_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20911_287 = {S10, 0, &V20911_287_57172, &P20911_237};	/*Property = CssFlexbox*/
const Value V20911_288_57175 = {S0, NULL};	/*Value = False*/
const Property P20911_288 = {S11, 0, &V20911_288_57175, &P20911_287};	/*Property = CssFont*/
const Value V20911_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20911_289 = {S12, 0, &V20911_289_57759, &P20911_288};	/*Property = CssImages*/
const Value V20911_291_57181 = {S0, NULL};	/*Value = False*/
const Property P20911_291 = {S13, 0, &V20911_291_57181, &P20911_289};	/*Property = CssMediaQueries*/
const Value V20911_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20911_292 = {S14, 0, &V20911_292_57183, &P20911_291};	/*Property = CssMinMax*/
const Value V20911_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20911_293 = {S15, 0, &V20911_293_57187, &P20911_292};	/*Property = CssOverflow*/
const Value V20911_294_57190 = {S0, NULL};	/*Value = False*/
const Property P20911_294 = {S16, 0, &V20911_294_57190, &P20911_293};	/*Property = CssPosition*/
const Value V20911_295_57192 = {S2, NULL};	/*Value = True*/
const Property P20911_295 = {S17, 0, &V20911_295_57192, &P20911_294};	/*Property = CssText*/
const Value V20911_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20911_297 = {S18, 0, &V20911_297_57195, &P20911_295};	/*Property = CssTransforms*/
const Value V20911_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20911_241 = {S19, 0, &V20911_241_55799, &P20911_297};	/*Property = CssTransitions*/
const Value V20911_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20911_298 = {S20, 0, &V20911_298_57206, &P20911_241};	/*Property = CssUI*/
const Value V20911_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20911_300 = {S21, 0, &V20911_300_57216, &P20911_298};	/*Property = DataSet*/
const Value V20911_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20911_301 = {S22, 0, &V20911_301_57218, &P20911_300};	/*Property = DataUrl*/
const Value V20911_302_57222 = {S0, NULL};	/*Value = False*/
const Property P20911_302 = {S23, 0, &V20911_302_57222, &P20911_301};	/*Property = DeviceOrientation*/
const Value V20911_304_57228 = {S0, NULL};	/*Value = False*/
const Property P20911_304 = {S24, 0, &V20911_304_57228, &P20911_302};	/*Property = FileReader*/
const Value V20911_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20911_305 = {S25, 0, &V20911_305_57231, &P20911_304};	/*Property = FileSaver*/
const Value V20911_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20911_306 = {S26, 0, &V20911_306_57237, &P20911_305};	/*Property = FileWriter*/
const Value V20911_307_57240 = {S0, NULL};	/*Value = False*/
const Property P20911_307 = {S27, 0, &V20911_307_57240, &P20911_306};	/*Property = FormData*/
const Value V20911_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20911_309 = {S28, 0, &V20911_309_57246, &P20911_307};	/*Property = Fullscreen*/
const Value V20911_310_57249 = {S0, NULL};	/*Value = False*/
const Property P20911_310 = {S29, 0, &V20911_310_57249, &P20911_309};	/*Property = GeoLocation*/
const Value V20911_312_57255 = {S0, NULL};	/*Value = False*/
const Property P20911_312 = {S30, 0, &V20911_312_57255, &P20911_310};	/*Property = History*/
const Value V20911_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20911_314 = {S31, 0, &V20911_314_57261, &P20911_312};	/*Property = Html5*/
const Value V20911_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20911_313 = {S32, 0, &V20911_313_57258, &P20911_314};	/*Property = Html-Media-Capture*/
const Value V20911_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20911_315 = {S33, 0, &V20911_315_57264, &P20911_313};	/*Property = Iframe*/
const Value V20911_316_57267 = {S0, NULL};	/*Value = False*/
const Property P20911_316 = {S34, 0, &V20911_316_57267, &P20911_315};	/*Property = IndexedDB*/
const Value V20911_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20911_317 = {S35, 0, &V20911_317_57269, &P20911_316};	/*Property = Json*/
const Value V20911_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P20911_15 = {S37, 0, &V20911_15_28, &P20911_317};	/*Property = LayoutEngine*/
const Value V20911_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20911_344 = {S38, 0, &V20911_344_57272, &P20911_15};	/*Property = Masking*/
const Value V20911_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20911_320 = {S39, 0, &V20911_320_57290, &P20911_344};	/*Property = PostMessage*/
const Value V20911_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20911_321 = {S40, 0, &V20911_321_57293, &P20911_320};	/*Property = Progress*/
const Value V20911_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20911_322 = {S41, 0, &V20911_322_57296, &P20911_321};	/*Property = Prompts*/
const Value V20911_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20911_323 = {S42, 0, &V20911_323_57299, &P20911_322};	/*Property = Selector*/
const Value V20911_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20911_324 = {S43, 0, &V20911_324_57306, &P20911_323};	/*Property = Svg*/
const Value V20911_327_57315 = {S0, NULL};	/*Value = False*/
const Property P20911_327 = {S44, 0, &V20911_327_57315, &P20911_324};	/*Property = TouchEvents*/
const Value V20911_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20911_328 = {S45, 0, &V20911_328_57318, &P20911_327};	/*Property = Track*/
const Value V20911_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20911_329 = {S46, 0, &V20911_329_57323, &P20911_328};	/*Property = Video*/
const Value V20911_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20911_330 = {S47, 0, &V20911_330_57326, &P20911_329};	/*Property = Viewport*/
const Value V20911_332_57339 = {S0, NULL};	/*Value = False*/
const Property P20911_332 = {S48, 0, &V20911_332_57339, &P20911_330};	/*Property = WebWorkers*/
const Value V20911_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20911_333 = {S49, 0, &V20911_333_57342, &P20911_332};	/*Property = Xhr2*/
const Profile P20911 = {"20911", &P20911_333};	/*Profile ID - 20911*/
const Value V21951_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21951_279 = {S1, 0, &V21951_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21951_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21951_282 = {S3, 0, &V21951_282_57154, &P21951_279};	/*Property = BlobBuilder*/
const Value V21951_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21951_350 = {S4, 0, &V21951_350_57444, &P21951_282};	/*Property = Canvas*/
const Value V21951_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21951_283 = {S5, 0, &V21951_283_57159, &P21951_350};	/*Property = CssBackground*/
const Value V21951_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21951_284 = {S6, 0, &V21951_284_57163, &P21951_283};	/*Property = CssBorderImage*/
const Value V21951_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21951_285 = {S7, 0, &V21951_285_57166, &P21951_284};	/*Property = CssCanvas*/
const Value V21951_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21951_286 = {S8, 0, &V21951_286_57168, &P21951_285};	/*Property = CssColor*/
const Value V21951_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21951_237 = {S9, 0, &V21951_237_55784, &P21951_286};	/*Property = CssColumn*/
const Value V21951_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21951_287 = {S10, 0, &V21951_287_57172, &P21951_237};	/*Property = CssFlexbox*/
const Value V21951_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21951_288 = {S11, 0, &V21951_288_57175, &P21951_287};	/*Property = CssFont*/
const Value V21951_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21951_289 = {S12, 0, &V21951_289_57759, &P21951_288};	/*Property = CssImages*/
const Value V21951_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21951_291 = {S13, 0, &V21951_291_57181, &P21951_289};	/*Property = CssMediaQueries*/
const Value V21951_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21951_292 = {S14, 0, &V21951_292_57183, &P21951_291};	/*Property = CssMinMax*/
const Value V21951_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21951_293 = {S15, 0, &V21951_293_57187, &P21951_292};	/*Property = CssOverflow*/
const Value V21951_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21951_294 = {S16, 0, &V21951_294_57190, &P21951_293};	/*Property = CssPosition*/
const Value V21951_295_57193 = {S0, NULL};	/*Value = False*/
const Property P21951_295 = {S17, 0, &V21951_295_57193, &P21951_294};	/*Property = CssText*/
const Value V21951_297_57196 = {S0, NULL};	/*Value = False*/
const Property P21951_297 = {S18, 0, &V21951_297_57196, &P21951_295};	/*Property = CssTransforms*/
const Value V21951_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21951_241 = {S19, 0, &V21951_241_55799, &P21951_297};	/*Property = CssTransitions*/
const Value V21951_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21951_298 = {S20, 0, &V21951_298_57206, &P21951_241};	/*Property = CssUI*/
const Value V21951_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21951_300 = {S21, 0, &V21951_300_57216, &P21951_298};	/*Property = DataSet*/
const Value V21951_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21951_301 = {S22, 0, &V21951_301_57218, &P21951_300};	/*Property = DataUrl*/
const Value V21951_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21951_302 = {S23, 0, &V21951_302_57222, &P21951_301};	/*Property = DeviceOrientation*/
const Value V21951_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21951_304 = {S24, 0, &V21951_304_57228, &P21951_302};	/*Property = FileReader*/
const Value V21951_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21951_305 = {S25, 0, &V21951_305_57231, &P21951_304};	/*Property = FileSaver*/
const Value V21951_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21951_306 = {S26, 0, &V21951_306_57237, &P21951_305};	/*Property = FileWriter*/
const Value V21951_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21951_307 = {S27, 0, &V21951_307_57240, &P21951_306};	/*Property = FormData*/
const Value V21951_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21951_309 = {S28, 0, &V21951_309_57246, &P21951_307};	/*Property = Fullscreen*/
const Value V21951_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21951_310 = {S29, 0, &V21951_310_57248, &P21951_309};	/*Property = GeoLocation*/
const Value V21951_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21951_312 = {S30, 0, &V21951_312_57255, &P21951_310};	/*Property = History*/
const Value V21951_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21951_314 = {S31, 0, &V21951_314_57261, &P21951_312};	/*Property = Html5*/
const Value V21951_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21951_313 = {S32, 0, &V21951_313_57258, &P21951_314};	/*Property = Html-Media-Capture*/
const Value V21951_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21951_315 = {S33, 0, &V21951_315_57264, &P21951_313};	/*Property = Iframe*/
const Value V21951_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21951_316 = {S34, 0, &V21951_316_57267, &P21951_315};	/*Property = IndexedDB*/
const Value V21951_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21951_317 = {S35, 0, &V21951_317_57269, &P21951_316};	/*Property = Json*/
const Value V21951_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P21951_15 = {S37, 0, &V21951_15_31, &P21951_317};	/*Property = LayoutEngine*/
const Value V21951_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21951_344 = {S38, 0, &V21951_344_57272, &P21951_15};	/*Property = Masking*/
const Value V21951_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21951_320 = {S39, 0, &V21951_320_57290, &P21951_344};	/*Property = PostMessage*/
const Value V21951_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21951_321 = {S40, 0, &V21951_321_57293, &P21951_320};	/*Property = Progress*/
const Value V21951_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21951_322 = {S41, 0, &V21951_322_57296, &P21951_321};	/*Property = Prompts*/
const Value V21951_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21951_323 = {S42, 0, &V21951_323_57299, &P21951_322};	/*Property = Selector*/
const Value V21951_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21951_324 = {S43, 0, &V21951_324_57306, &P21951_323};	/*Property = Svg*/
const Value V21951_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21951_327 = {S44, 0, &V21951_327_57315, &P21951_324};	/*Property = TouchEvents*/
const Value V21951_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21951_328 = {S45, 0, &V21951_328_57318, &P21951_327};	/*Property = Track*/
const Value V21951_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21951_329 = {S46, 0, &V21951_329_57323, &P21951_328};	/*Property = Video*/
const Value V21951_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21951_330 = {S47, 0, &V21951_330_57326, &P21951_329};	/*Property = Viewport*/
const Value V21951_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21951_332 = {S48, 0, &V21951_332_57339, &P21951_330};	/*Property = WebWorkers*/
const Value V21951_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21951_333 = {S49, 0, &V21951_333_57342, &P21951_332};	/*Property = Xhr2*/
const Profile P21951 = {"21951", &P21951_333};	/*Profile ID - 21951*/
const Value V20761_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_279 = {S1, 0, &V20761_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20761_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_282 = {S3, 0, &V20761_282_57155, &P20761_279};	/*Property = BlobBuilder*/
const Value V20761_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_350 = {S4, 0, &V20761_350_57446, &P20761_282};	/*Property = Canvas*/
const Value V20761_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_283 = {S5, 0, &V20761_283_57161, &P20761_350};	/*Property = CssBackground*/
const Value V20761_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_284 = {S6, 0, &V20761_284_57164, &P20761_283};	/*Property = CssBorderImage*/
const Value V20761_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_285 = {S7, 0, &V20761_285_57167, &P20761_284};	/*Property = CssCanvas*/
const Value V20761_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_286 = {S8, 0, &V20761_286_57170, &P20761_285};	/*Property = CssColor*/
const Value V20761_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_237 = {S9, 0, &V20761_237_55786, &P20761_286};	/*Property = CssColumn*/
const Value V20761_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_287 = {S10, 0, &V20761_287_57173, &P20761_237};	/*Property = CssFlexbox*/
const Value V20761_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_288 = {S11, 0, &V20761_288_57176, &P20761_287};	/*Property = CssFont*/
const Value V20761_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_289 = {S12, 0, &V20761_289_86899, &P20761_288};	/*Property = CssImages*/
const Value V20761_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_291 = {S13, 0, &V20761_291_57182, &P20761_289};	/*Property = CssMediaQueries*/
const Value V20761_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_292 = {S14, 0, &V20761_292_57185, &P20761_291};	/*Property = CssMinMax*/
const Value V20761_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_293 = {S15, 0, &V20761_293_57188, &P20761_292};	/*Property = CssOverflow*/
const Value V20761_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_294 = {S16, 0, &V20761_294_57191, &P20761_293};	/*Property = CssPosition*/
const Value V20761_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_295 = {S17, 0, &V20761_295_57194, &P20761_294};	/*Property = CssText*/
const Value V20761_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_297 = {S18, 0, &V20761_297_57197, &P20761_295};	/*Property = CssTransforms*/
const Value V20761_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_241 = {S19, 0, &V20761_241_55801, &P20761_297};	/*Property = CssTransitions*/
const Value V20761_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_298 = {S20, 0, &V20761_298_57208, &P20761_241};	/*Property = CssUI*/
const Value V20761_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_300 = {S21, 0, &V20761_300_57217, &P20761_298};	/*Property = DataSet*/
const Value V20761_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_301 = {S22, 0, &V20761_301_57220, &P20761_300};	/*Property = DataUrl*/
const Value V20761_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_302 = {S23, 0, &V20761_302_57223, &P20761_301};	/*Property = DeviceOrientation*/
const Value V20761_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_304 = {S24, 0, &V20761_304_57229, &P20761_302};	/*Property = FileReader*/
const Value V20761_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_305 = {S25, 0, &V20761_305_57232, &P20761_304};	/*Property = FileSaver*/
const Value V20761_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_306 = {S26, 0, &V20761_306_57238, &P20761_305};	/*Property = FileWriter*/
const Value V20761_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_307 = {S27, 0, &V20761_307_57241, &P20761_306};	/*Property = FormData*/
const Value V20761_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_309 = {S28, 0, &V20761_309_57247, &P20761_307};	/*Property = Fullscreen*/
const Value V20761_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_310 = {S29, 0, &V20761_310_57250, &P20761_309};	/*Property = GeoLocation*/
const Value V20761_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_312 = {S30, 0, &V20761_312_57256, &P20761_310};	/*Property = History*/
const Value V20761_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_314 = {S31, 0, &V20761_314_57262, &P20761_312};	/*Property = Html5*/
const Value V20761_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_313 = {S32, 0, &V20761_313_57259, &P20761_314};	/*Property = Html-Media-Capture*/
const Value V20761_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_315 = {S33, 0, &V20761_315_57265, &P20761_313};	/*Property = Iframe*/
const Value V20761_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_316 = {S34, 0, &V20761_316_57268, &P20761_315};	/*Property = IndexedDB*/
const Value V20761_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_317 = {S35, 0, &V20761_317_57271, &P20761_316};	/*Property = Json*/
const Value V20761_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20761_15 = {S37, 0, &V20761_15_29, &P20761_317};	/*Property = LayoutEngine*/
const Value V20761_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_344 = {S38, 0, &V20761_344_57274, &P20761_15};	/*Property = Masking*/
const Value V20761_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_320 = {S39, 0, &V20761_320_57292, &P20761_344};	/*Property = PostMessage*/
const Value V20761_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_321 = {S40, 0, &V20761_321_57295, &P20761_320};	/*Property = Progress*/
const Value V20761_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_322 = {S41, 0, &V20761_322_57298, &P20761_321};	/*Property = Prompts*/
const Value V20761_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_323 = {S42, 0, &V20761_323_57301, &P20761_322};	/*Property = Selector*/
const Value V20761_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_324 = {S43, 0, &V20761_324_57307, &P20761_323};	/*Property = Svg*/
const Value V20761_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_327 = {S44, 0, &V20761_327_57316, &P20761_324};	/*Property = TouchEvents*/
const Value V20761_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_328 = {S45, 0, &V20761_328_57319, &P20761_327};	/*Property = Track*/
const Value V20761_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_329 = {S46, 0, &V20761_329_57325, &P20761_328};	/*Property = Video*/
const Value V20761_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_330 = {S47, 0, &V20761_330_57328, &P20761_329};	/*Property = Viewport*/
const Value V20761_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_332 = {S48, 0, &V20761_332_57340, &P20761_330};	/*Property = WebWorkers*/
const Value V20761_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20761_333 = {S49, 0, &V20761_333_57343, &P20761_332};	/*Property = Xhr2*/
const Profile P20761 = {"20761", &P20761_333};	/*Profile ID - 20761*/
const Value V18691_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_279 = {S1, 0, &V18691_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18691_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_282 = {S3, 0, &V18691_282_57155, &P18691_279};	/*Property = BlobBuilder*/
const Value V18691_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_350 = {S4, 0, &V18691_350_57446, &P18691_282};	/*Property = Canvas*/
const Value V18691_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_283 = {S5, 0, &V18691_283_57161, &P18691_350};	/*Property = CssBackground*/
const Value V18691_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_284 = {S6, 0, &V18691_284_57164, &P18691_283};	/*Property = CssBorderImage*/
const Value V18691_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_285 = {S7, 0, &V18691_285_57167, &P18691_284};	/*Property = CssCanvas*/
const Value V18691_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_286 = {S8, 0, &V18691_286_57170, &P18691_285};	/*Property = CssColor*/
const Value V18691_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_237 = {S9, 0, &V18691_237_55786, &P18691_286};	/*Property = CssColumn*/
const Value V18691_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_287 = {S10, 0, &V18691_287_57173, &P18691_237};	/*Property = CssFlexbox*/
const Value V18691_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_288 = {S11, 0, &V18691_288_57176, &P18691_287};	/*Property = CssFont*/
const Value V18691_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_289 = {S12, 0, &V18691_289_86899, &P18691_288};	/*Property = CssImages*/
const Value V18691_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_291 = {S13, 0, &V18691_291_57182, &P18691_289};	/*Property = CssMediaQueries*/
const Value V18691_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_292 = {S14, 0, &V18691_292_57185, &P18691_291};	/*Property = CssMinMax*/
const Value V18691_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_293 = {S15, 0, &V18691_293_57188, &P18691_292};	/*Property = CssOverflow*/
const Value V18691_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_294 = {S16, 0, &V18691_294_57191, &P18691_293};	/*Property = CssPosition*/
const Value V18691_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_295 = {S17, 0, &V18691_295_57194, &P18691_294};	/*Property = CssText*/
const Value V18691_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_297 = {S18, 0, &V18691_297_57197, &P18691_295};	/*Property = CssTransforms*/
const Value V18691_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_241 = {S19, 0, &V18691_241_55801, &P18691_297};	/*Property = CssTransitions*/
const Value V18691_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_298 = {S20, 0, &V18691_298_57208, &P18691_241};	/*Property = CssUI*/
const Value V18691_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_300 = {S21, 0, &V18691_300_57217, &P18691_298};	/*Property = DataSet*/
const Value V18691_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_301 = {S22, 0, &V18691_301_57220, &P18691_300};	/*Property = DataUrl*/
const Value V18691_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_302 = {S23, 0, &V18691_302_57223, &P18691_301};	/*Property = DeviceOrientation*/
const Value V18691_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_304 = {S24, 0, &V18691_304_57229, &P18691_302};	/*Property = FileReader*/
const Value V18691_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_305 = {S25, 0, &V18691_305_57232, &P18691_304};	/*Property = FileSaver*/
const Value V18691_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_306 = {S26, 0, &V18691_306_57238, &P18691_305};	/*Property = FileWriter*/
const Value V18691_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_307 = {S27, 0, &V18691_307_57241, &P18691_306};	/*Property = FormData*/
const Value V18691_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_309 = {S28, 0, &V18691_309_57247, &P18691_307};	/*Property = Fullscreen*/
const Value V18691_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_310 = {S29, 0, &V18691_310_57250, &P18691_309};	/*Property = GeoLocation*/
const Value V18691_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_312 = {S30, 0, &V18691_312_57256, &P18691_310};	/*Property = History*/
const Value V18691_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_314 = {S31, 0, &V18691_314_57262, &P18691_312};	/*Property = Html5*/
const Value V18691_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_313 = {S32, 0, &V18691_313_57259, &P18691_314};	/*Property = Html-Media-Capture*/
const Value V18691_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_315 = {S33, 0, &V18691_315_57265, &P18691_313};	/*Property = Iframe*/
const Value V18691_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_316 = {S34, 0, &V18691_316_57268, &P18691_315};	/*Property = IndexedDB*/
const Value V18691_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_317 = {S35, 0, &V18691_317_57271, &P18691_316};	/*Property = Json*/
const Value V18691_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18691_15 = {S37, 0, &V18691_15_29, &P18691_317};	/*Property = LayoutEngine*/
const Value V18691_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_344 = {S38, 0, &V18691_344_57274, &P18691_15};	/*Property = Masking*/
const Value V18691_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_320 = {S39, 0, &V18691_320_57292, &P18691_344};	/*Property = PostMessage*/
const Value V18691_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_321 = {S40, 0, &V18691_321_57295, &P18691_320};	/*Property = Progress*/
const Value V18691_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_322 = {S41, 0, &V18691_322_57298, &P18691_321};	/*Property = Prompts*/
const Value V18691_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_323 = {S42, 0, &V18691_323_57301, &P18691_322};	/*Property = Selector*/
const Value V18691_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_324 = {S43, 0, &V18691_324_57307, &P18691_323};	/*Property = Svg*/
const Value V18691_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_327 = {S44, 0, &V18691_327_57316, &P18691_324};	/*Property = TouchEvents*/
const Value V18691_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_328 = {S45, 0, &V18691_328_57319, &P18691_327};	/*Property = Track*/
const Value V18691_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_329 = {S46, 0, &V18691_329_57325, &P18691_328};	/*Property = Video*/
const Value V18691_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_330 = {S47, 0, &V18691_330_57328, &P18691_329};	/*Property = Viewport*/
const Value V18691_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_332 = {S48, 0, &V18691_332_57340, &P18691_330};	/*Property = WebWorkers*/
const Value V18691_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18691_333 = {S49, 0, &V18691_333_57343, &P18691_332};	/*Property = Xhr2*/
const Profile P18691 = {"18691", &P18691_333};	/*Profile ID - 18691*/
const Value V21914_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21914_279 = {S1, 0, &V21914_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21914_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21914_282 = {S3, 0, &V21914_282_57154, &P21914_279};	/*Property = BlobBuilder*/
const Value V21914_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21914_350 = {S4, 0, &V21914_350_57444, &P21914_282};	/*Property = Canvas*/
const Value V21914_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21914_283 = {S5, 0, &V21914_283_57159, &P21914_350};	/*Property = CssBackground*/
const Value V21914_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21914_284 = {S6, 0, &V21914_284_57163, &P21914_283};	/*Property = CssBorderImage*/
const Value V21914_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21914_285 = {S7, 0, &V21914_285_57166, &P21914_284};	/*Property = CssCanvas*/
const Value V21914_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21914_286 = {S8, 0, &V21914_286_57168, &P21914_285};	/*Property = CssColor*/
const Value V21914_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21914_237 = {S9, 0, &V21914_237_55786, &P21914_286};	/*Property = CssColumn*/
const Value V21914_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21914_287 = {S10, 0, &V21914_287_57172, &P21914_237};	/*Property = CssFlexbox*/
const Value V21914_288_57174 = {S2, NULL};	/*Value = True*/
const Property P21914_288 = {S11, 0, &V21914_288_57174, &P21914_287};	/*Property = CssFont*/
const Value V21914_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21914_289 = {S12, 0, &V21914_289_57759, &P21914_288};	/*Property = CssImages*/
const Value V21914_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21914_291 = {S13, 0, &V21914_291_57181, &P21914_289};	/*Property = CssMediaQueries*/
const Value V21914_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21914_292 = {S14, 0, &V21914_292_57183, &P21914_291};	/*Property = CssMinMax*/
const Value V21914_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21914_293 = {S15, 0, &V21914_293_57187, &P21914_292};	/*Property = CssOverflow*/
const Value V21914_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21914_294 = {S16, 0, &V21914_294_57189, &P21914_293};	/*Property = CssPosition*/
const Value V21914_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21914_295 = {S17, 0, &V21914_295_57192, &P21914_294};	/*Property = CssText*/
const Value V21914_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21914_297 = {S18, 0, &V21914_297_57195, &P21914_295};	/*Property = CssTransforms*/
const Value V21914_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21914_241 = {S19, 0, &V21914_241_55799, &P21914_297};	/*Property = CssTransitions*/
const Value V21914_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21914_298 = {S20, 0, &V21914_298_57206, &P21914_241};	/*Property = CssUI*/
const Value V21914_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21914_300 = {S21, 0, &V21914_300_57216, &P21914_298};	/*Property = DataSet*/
const Value V21914_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21914_301 = {S22, 0, &V21914_301_57218, &P21914_300};	/*Property = DataUrl*/
const Value V21914_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21914_302 = {S23, 0, &V21914_302_57222, &P21914_301};	/*Property = DeviceOrientation*/
const Value V21914_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21914_304 = {S24, 0, &V21914_304_57227, &P21914_302};	/*Property = FileReader*/
const Value V21914_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21914_305 = {S25, 0, &V21914_305_57231, &P21914_304};	/*Property = FileSaver*/
const Value V21914_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21914_306 = {S26, 0, &V21914_306_57237, &P21914_305};	/*Property = FileWriter*/
const Value V21914_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21914_307 = {S27, 0, &V21914_307_57239, &P21914_306};	/*Property = FormData*/
const Value V21914_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21914_309 = {S28, 0, &V21914_309_57246, &P21914_307};	/*Property = Fullscreen*/
const Value V21914_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21914_310 = {S29, 0, &V21914_310_57248, &P21914_309};	/*Property = GeoLocation*/
const Value V21914_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21914_312 = {S30, 0, &V21914_312_57254, &P21914_310};	/*Property = History*/
const Value V21914_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21914_314 = {S31, 0, &V21914_314_57261, &P21914_312};	/*Property = Html5*/
const Value V21914_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21914_313 = {S32, 0, &V21914_313_57258, &P21914_314};	/*Property = Html-Media-Capture*/
const Value V21914_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21914_315 = {S33, 0, &V21914_315_57264, &P21914_313};	/*Property = Iframe*/
const Value V21914_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21914_316 = {S34, 0, &V21914_316_57266, &P21914_315};	/*Property = IndexedDB*/
const Value V21914_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21914_317 = {S35, 0, &V21914_317_57269, &P21914_316};	/*Property = Json*/
const Value V21914_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21914_15 = {S37, 0, &V21914_15_28, &P21914_317};	/*Property = LayoutEngine*/
const Value V21914_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21914_344 = {S38, 0, &V21914_344_57272, &P21914_15};	/*Property = Masking*/
const Value V21914_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21914_320 = {S39, 0, &V21914_320_57290, &P21914_344};	/*Property = PostMessage*/
const Value V21914_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21914_321 = {S40, 0, &V21914_321_57293, &P21914_320};	/*Property = Progress*/
const Value V21914_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21914_322 = {S41, 0, &V21914_322_57296, &P21914_321};	/*Property = Prompts*/
const Value V21914_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21914_323 = {S42, 0, &V21914_323_57299, &P21914_322};	/*Property = Selector*/
const Value V21914_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21914_324 = {S43, 0, &V21914_324_57306, &P21914_323};	/*Property = Svg*/
const Value V21914_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21914_327 = {S44, 0, &V21914_327_57315, &P21914_324};	/*Property = TouchEvents*/
const Value V21914_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21914_328 = {S45, 0, &V21914_328_57318, &P21914_327};	/*Property = Track*/
const Value V21914_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21914_329 = {S46, 0, &V21914_329_57323, &P21914_328};	/*Property = Video*/
const Value V21914_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21914_330 = {S47, 0, &V21914_330_57326, &P21914_329};	/*Property = Viewport*/
const Value V21914_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21914_332 = {S48, 0, &V21914_332_57339, &P21914_330};	/*Property = WebWorkers*/
const Value V21914_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21914_333 = {S49, 0, &V21914_333_57343, &P21914_332};	/*Property = Xhr2*/
const Profile P21914 = {"21914", &P21914_333};	/*Profile ID - 21914*/
const Profile P24701 = {"24701", NULL};	/*Profile ID - 24701*/
const Value V22775_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_279 = {S1, 0, &V22775_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22775_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_282 = {S3, 0, &V22775_282_57155, &P22775_279};	/*Property = BlobBuilder*/
const Value V22775_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_350 = {S4, 0, &V22775_350_57446, &P22775_282};	/*Property = Canvas*/
const Value V22775_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_283 = {S5, 0, &V22775_283_57161, &P22775_350};	/*Property = CssBackground*/
const Value V22775_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_284 = {S6, 0, &V22775_284_57164, &P22775_283};	/*Property = CssBorderImage*/
const Value V22775_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_285 = {S7, 0, &V22775_285_57167, &P22775_284};	/*Property = CssCanvas*/
const Value V22775_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_286 = {S8, 0, &V22775_286_57170, &P22775_285};	/*Property = CssColor*/
const Value V22775_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_237 = {S9, 0, &V22775_237_55786, &P22775_286};	/*Property = CssColumn*/
const Value V22775_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_287 = {S10, 0, &V22775_287_57173, &P22775_237};	/*Property = CssFlexbox*/
const Value V22775_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_288 = {S11, 0, &V22775_288_57176, &P22775_287};	/*Property = CssFont*/
const Value V22775_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_289 = {S12, 0, &V22775_289_86899, &P22775_288};	/*Property = CssImages*/
const Value V22775_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_291 = {S13, 0, &V22775_291_57182, &P22775_289};	/*Property = CssMediaQueries*/
const Value V22775_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_292 = {S14, 0, &V22775_292_57185, &P22775_291};	/*Property = CssMinMax*/
const Value V22775_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_293 = {S15, 0, &V22775_293_57188, &P22775_292};	/*Property = CssOverflow*/
const Value V22775_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_294 = {S16, 0, &V22775_294_57191, &P22775_293};	/*Property = CssPosition*/
const Value V22775_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_295 = {S17, 0, &V22775_295_57194, &P22775_294};	/*Property = CssText*/
const Value V22775_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_297 = {S18, 0, &V22775_297_57197, &P22775_295};	/*Property = CssTransforms*/
const Value V22775_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_241 = {S19, 0, &V22775_241_55801, &P22775_297};	/*Property = CssTransitions*/
const Value V22775_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_298 = {S20, 0, &V22775_298_57208, &P22775_241};	/*Property = CssUI*/
const Value V22775_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_300 = {S21, 0, &V22775_300_57217, &P22775_298};	/*Property = DataSet*/
const Value V22775_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_301 = {S22, 0, &V22775_301_57220, &P22775_300};	/*Property = DataUrl*/
const Value V22775_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_302 = {S23, 0, &V22775_302_57223, &P22775_301};	/*Property = DeviceOrientation*/
const Value V22775_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_304 = {S24, 0, &V22775_304_57229, &P22775_302};	/*Property = FileReader*/
const Value V22775_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_305 = {S25, 0, &V22775_305_57232, &P22775_304};	/*Property = FileSaver*/
const Value V22775_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_306 = {S26, 0, &V22775_306_57238, &P22775_305};	/*Property = FileWriter*/
const Value V22775_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_307 = {S27, 0, &V22775_307_57241, &P22775_306};	/*Property = FormData*/
const Value V22775_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_309 = {S28, 0, &V22775_309_57247, &P22775_307};	/*Property = Fullscreen*/
const Value V22775_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_310 = {S29, 0, &V22775_310_57250, &P22775_309};	/*Property = GeoLocation*/
const Value V22775_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_312 = {S30, 0, &V22775_312_57256, &P22775_310};	/*Property = History*/
const Value V22775_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_314 = {S31, 0, &V22775_314_57262, &P22775_312};	/*Property = Html5*/
const Value V22775_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_313 = {S32, 0, &V22775_313_57259, &P22775_314};	/*Property = Html-Media-Capture*/
const Value V22775_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_315 = {S33, 0, &V22775_315_57265, &P22775_313};	/*Property = Iframe*/
const Value V22775_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_316 = {S34, 0, &V22775_316_57268, &P22775_315};	/*Property = IndexedDB*/
const Value V22775_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_317 = {S35, 0, &V22775_317_57271, &P22775_316};	/*Property = Json*/
const Value V22775_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P22775_15 = {S37, 0, &V22775_15_30, &P22775_317};	/*Property = LayoutEngine*/
const Value V22775_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_344 = {S38, 0, &V22775_344_57274, &P22775_15};	/*Property = Masking*/
const Value V22775_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_320 = {S39, 0, &V22775_320_57292, &P22775_344};	/*Property = PostMessage*/
const Value V22775_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_321 = {S40, 0, &V22775_321_57295, &P22775_320};	/*Property = Progress*/
const Value V22775_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_322 = {S41, 0, &V22775_322_57298, &P22775_321};	/*Property = Prompts*/
const Value V22775_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_323 = {S42, 0, &V22775_323_57301, &P22775_322};	/*Property = Selector*/
const Value V22775_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_324 = {S43, 0, &V22775_324_57307, &P22775_323};	/*Property = Svg*/
const Value V22775_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_327 = {S44, 0, &V22775_327_57316, &P22775_324};	/*Property = TouchEvents*/
const Value V22775_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_328 = {S45, 0, &V22775_328_57319, &P22775_327};	/*Property = Track*/
const Value V22775_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_329 = {S46, 0, &V22775_329_57325, &P22775_328};	/*Property = Video*/
const Value V22775_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_330 = {S47, 0, &V22775_330_57328, &P22775_329};	/*Property = Viewport*/
const Value V22775_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_332 = {S48, 0, &V22775_332_57340, &P22775_330};	/*Property = WebWorkers*/
const Value V22775_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22775_333 = {S49, 0, &V22775_333_57343, &P22775_332};	/*Property = Xhr2*/
const Profile P22775 = {"22775", &P22775_333};	/*Profile ID - 22775*/
const Value V18559_1_1 = {S2, NULL};	/*Value = True*/
const Property P18559_1 = {S50, 0, &V18559_1_1, NULL};	/*Property = IsMobile*/
const Value V18559_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18559_5 = {S52, 0, &V18559_5_1227, &P18559_1};	/*Property = ScreenPixelsHeight*/
const Value V18559_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18559_4 = {S54, 0, &V18559_4_1229, &P18559_5};	/*Property = ScreenPixelsWidth*/
const Profile P18559 = {"18559", &P18559_4};	/*Profile ID - 18559*/
const Value V18461_1_1 = {S2, NULL};	/*Value = True*/
const Property P18461_1 = {S50, 0, &V18461_1_1, NULL};	/*Property = IsMobile*/
const Value V18461_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18461_5 = {S52, 0, &V18461_5_1403, &P18461_1};	/*Property = ScreenPixelsHeight*/
const Value V18461_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18461_4 = {S54, 0, &V18461_4_1404, &P18461_5};	/*Property = ScreenPixelsWidth*/
const Profile P18461 = {"18461", &P18461_4};	/*Profile ID - 18461*/
const Value V18502_1_1 = {S2, NULL};	/*Value = True*/
const Property P18502_1 = {S50, 0, &V18502_1_1, NULL};	/*Property = IsMobile*/
const Value V18502_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18502_5 = {S52, 0, &V18502_5_9981, &P18502_1};	/*Property = ScreenPixelsHeight*/
const Value V18502_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18502_4 = {S54, 0, &V18502_4_39098, &P18502_5};	/*Property = ScreenPixelsWidth*/
const Profile P18502 = {"18502", &P18502_4};	/*Profile ID - 18502*/
const Value V18591_1_1 = {S2, NULL};	/*Value = True*/
const Property P18591_1 = {S50, 0, &V18591_1_1, NULL};	/*Property = IsMobile*/
const Value V18591_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18591_5 = {S52, 0, &V18591_5_1227, &P18591_1};	/*Property = ScreenPixelsHeight*/
const Value V18591_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18591_4 = {S54, 0, &V18591_4_1229, &P18591_5};	/*Property = ScreenPixelsWidth*/
const Profile P18591 = {"18591", &P18591_4};	/*Profile ID - 18591*/
const Value V18564_1_1 = {S2, NULL};	/*Value = True*/
const Property P18564_1 = {S50, 0, &V18564_1_1, NULL};	/*Property = IsMobile*/
const Value V18564_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18564_5 = {S52, 0, &V18564_5_9981, &P18564_1};	/*Property = ScreenPixelsHeight*/
const Value V18564_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18564_4 = {S54, 0, &V18564_4_39098, &P18564_5};	/*Property = ScreenPixelsWidth*/
const Profile P18564 = {"18564", &P18564_4};	/*Profile ID - 18564*/
const Profile P22471 = {"22471", NULL};	/*Profile ID - 22471*/
const Value V22892_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_279 = {S1, 0, &V22892_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22892_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_282 = {S3, 0, &V22892_282_57155, &P22892_279};	/*Property = BlobBuilder*/
const Value V22892_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_350 = {S4, 0, &V22892_350_57446, &P22892_282};	/*Property = Canvas*/
const Value V22892_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_283 = {S5, 0, &V22892_283_57161, &P22892_350};	/*Property = CssBackground*/
const Value V22892_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_284 = {S6, 0, &V22892_284_57164, &P22892_283};	/*Property = CssBorderImage*/
const Value V22892_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_285 = {S7, 0, &V22892_285_57167, &P22892_284};	/*Property = CssCanvas*/
const Value V22892_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_286 = {S8, 0, &V22892_286_57170, &P22892_285};	/*Property = CssColor*/
const Value V22892_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_237 = {S9, 0, &V22892_237_55786, &P22892_286};	/*Property = CssColumn*/
const Value V22892_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_287 = {S10, 0, &V22892_287_57173, &P22892_237};	/*Property = CssFlexbox*/
const Value V22892_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_288 = {S11, 0, &V22892_288_57176, &P22892_287};	/*Property = CssFont*/
const Value V22892_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_289 = {S12, 0, &V22892_289_86899, &P22892_288};	/*Property = CssImages*/
const Value V22892_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_291 = {S13, 0, &V22892_291_57182, &P22892_289};	/*Property = CssMediaQueries*/
const Value V22892_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_292 = {S14, 0, &V22892_292_57185, &P22892_291};	/*Property = CssMinMax*/
const Value V22892_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_293 = {S15, 0, &V22892_293_57188, &P22892_292};	/*Property = CssOverflow*/
const Value V22892_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_294 = {S16, 0, &V22892_294_57191, &P22892_293};	/*Property = CssPosition*/
const Value V22892_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_295 = {S17, 0, &V22892_295_57194, &P22892_294};	/*Property = CssText*/
const Value V22892_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_297 = {S18, 0, &V22892_297_57197, &P22892_295};	/*Property = CssTransforms*/
const Value V22892_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_241 = {S19, 0, &V22892_241_55801, &P22892_297};	/*Property = CssTransitions*/
const Value V22892_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_298 = {S20, 0, &V22892_298_57208, &P22892_241};	/*Property = CssUI*/
const Value V22892_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_300 = {S21, 0, &V22892_300_57217, &P22892_298};	/*Property = DataSet*/
const Value V22892_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_301 = {S22, 0, &V22892_301_57220, &P22892_300};	/*Property = DataUrl*/
const Value V22892_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_302 = {S23, 0, &V22892_302_57223, &P22892_301};	/*Property = DeviceOrientation*/
const Value V22892_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_304 = {S24, 0, &V22892_304_57229, &P22892_302};	/*Property = FileReader*/
const Value V22892_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_305 = {S25, 0, &V22892_305_57232, &P22892_304};	/*Property = FileSaver*/
const Value V22892_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_306 = {S26, 0, &V22892_306_57238, &P22892_305};	/*Property = FileWriter*/
const Value V22892_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_307 = {S27, 0, &V22892_307_57241, &P22892_306};	/*Property = FormData*/
const Value V22892_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_309 = {S28, 0, &V22892_309_57247, &P22892_307};	/*Property = Fullscreen*/
const Value V22892_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_310 = {S29, 0, &V22892_310_57250, &P22892_309};	/*Property = GeoLocation*/
const Value V22892_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_312 = {S30, 0, &V22892_312_57256, &P22892_310};	/*Property = History*/
const Value V22892_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_314 = {S31, 0, &V22892_314_57262, &P22892_312};	/*Property = Html5*/
const Value V22892_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_313 = {S32, 0, &V22892_313_57259, &P22892_314};	/*Property = Html-Media-Capture*/
const Value V22892_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_315 = {S33, 0, &V22892_315_57265, &P22892_313};	/*Property = Iframe*/
const Value V22892_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_316 = {S34, 0, &V22892_316_57268, &P22892_315};	/*Property = IndexedDB*/
const Value V22892_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_317 = {S35, 0, &V22892_317_57271, &P22892_316};	/*Property = Json*/
const Value V22892_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_15 = {S37, 0, &V22892_15_72, &P22892_317};	/*Property = LayoutEngine*/
const Value V22892_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_344 = {S38, 0, &V22892_344_57274, &P22892_15};	/*Property = Masking*/
const Value V22892_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_320 = {S39, 0, &V22892_320_57292, &P22892_344};	/*Property = PostMessage*/
const Value V22892_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_321 = {S40, 0, &V22892_321_57295, &P22892_320};	/*Property = Progress*/
const Value V22892_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_322 = {S41, 0, &V22892_322_57298, &P22892_321};	/*Property = Prompts*/
const Value V22892_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_323 = {S42, 0, &V22892_323_57301, &P22892_322};	/*Property = Selector*/
const Value V22892_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_324 = {S43, 0, &V22892_324_57307, &P22892_323};	/*Property = Svg*/
const Value V22892_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_327 = {S44, 0, &V22892_327_57316, &P22892_324};	/*Property = TouchEvents*/
const Value V22892_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_328 = {S45, 0, &V22892_328_57319, &P22892_327};	/*Property = Track*/
const Value V22892_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_329 = {S46, 0, &V22892_329_57325, &P22892_328};	/*Property = Video*/
const Value V22892_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_330 = {S47, 0, &V22892_330_57328, &P22892_329};	/*Property = Viewport*/
const Value V22892_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_332 = {S48, 0, &V22892_332_57340, &P22892_330};	/*Property = WebWorkers*/
const Value V22892_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22892_333 = {S49, 0, &V22892_333_57343, &P22892_332};	/*Property = Xhr2*/
const Profile P22892 = {"22892", &P22892_333};	/*Profile ID - 22892*/
const Value V23738_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_279 = {S1, 0, &V23738_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23738_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_282 = {S3, 0, &V23738_282_57155, &P23738_279};	/*Property = BlobBuilder*/
const Value V23738_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_350 = {S4, 0, &V23738_350_57446, &P23738_282};	/*Property = Canvas*/
const Value V23738_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_283 = {S5, 0, &V23738_283_57161, &P23738_350};	/*Property = CssBackground*/
const Value V23738_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_284 = {S6, 0, &V23738_284_57164, &P23738_283};	/*Property = CssBorderImage*/
const Value V23738_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_285 = {S7, 0, &V23738_285_57167, &P23738_284};	/*Property = CssCanvas*/
const Value V23738_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_286 = {S8, 0, &V23738_286_57170, &P23738_285};	/*Property = CssColor*/
const Value V23738_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_237 = {S9, 0, &V23738_237_55786, &P23738_286};	/*Property = CssColumn*/
const Value V23738_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_287 = {S10, 0, &V23738_287_57173, &P23738_237};	/*Property = CssFlexbox*/
const Value V23738_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_288 = {S11, 0, &V23738_288_57176, &P23738_287};	/*Property = CssFont*/
const Value V23738_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_289 = {S12, 0, &V23738_289_86899, &P23738_288};	/*Property = CssImages*/
const Value V23738_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_291 = {S13, 0, &V23738_291_57182, &P23738_289};	/*Property = CssMediaQueries*/
const Value V23738_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_292 = {S14, 0, &V23738_292_57185, &P23738_291};	/*Property = CssMinMax*/
const Value V23738_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_293 = {S15, 0, &V23738_293_57188, &P23738_292};	/*Property = CssOverflow*/
const Value V23738_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_294 = {S16, 0, &V23738_294_57191, &P23738_293};	/*Property = CssPosition*/
const Value V23738_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_295 = {S17, 0, &V23738_295_57194, &P23738_294};	/*Property = CssText*/
const Value V23738_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_297 = {S18, 0, &V23738_297_57197, &P23738_295};	/*Property = CssTransforms*/
const Value V23738_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_241 = {S19, 0, &V23738_241_55801, &P23738_297};	/*Property = CssTransitions*/
const Value V23738_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_298 = {S20, 0, &V23738_298_57208, &P23738_241};	/*Property = CssUI*/
const Value V23738_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_300 = {S21, 0, &V23738_300_57217, &P23738_298};	/*Property = DataSet*/
const Value V23738_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_301 = {S22, 0, &V23738_301_57220, &P23738_300};	/*Property = DataUrl*/
const Value V23738_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_302 = {S23, 0, &V23738_302_57223, &P23738_301};	/*Property = DeviceOrientation*/
const Value V23738_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_304 = {S24, 0, &V23738_304_57229, &P23738_302};	/*Property = FileReader*/
const Value V23738_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_305 = {S25, 0, &V23738_305_57232, &P23738_304};	/*Property = FileSaver*/
const Value V23738_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_306 = {S26, 0, &V23738_306_57238, &P23738_305};	/*Property = FileWriter*/
const Value V23738_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_307 = {S27, 0, &V23738_307_57241, &P23738_306};	/*Property = FormData*/
const Value V23738_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_309 = {S28, 0, &V23738_309_57247, &P23738_307};	/*Property = Fullscreen*/
const Value V23738_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_310 = {S29, 0, &V23738_310_57250, &P23738_309};	/*Property = GeoLocation*/
const Value V23738_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_312 = {S30, 0, &V23738_312_57256, &P23738_310};	/*Property = History*/
const Value V23738_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_314 = {S31, 0, &V23738_314_57262, &P23738_312};	/*Property = Html5*/
const Value V23738_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_313 = {S32, 0, &V23738_313_57259, &P23738_314};	/*Property = Html-Media-Capture*/
const Value V23738_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_315 = {S33, 0, &V23738_315_57265, &P23738_313};	/*Property = Iframe*/
const Value V23738_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_316 = {S34, 0, &V23738_316_57268, &P23738_315};	/*Property = IndexedDB*/
const Value V23738_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_317 = {S35, 0, &V23738_317_57271, &P23738_316};	/*Property = Json*/
const Value V23738_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P23738_15 = {S37, 0, &V23738_15_28, &P23738_317};	/*Property = LayoutEngine*/
const Value V23738_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_344 = {S38, 0, &V23738_344_57274, &P23738_15};	/*Property = Masking*/
const Value V23738_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_320 = {S39, 0, &V23738_320_57292, &P23738_344};	/*Property = PostMessage*/
const Value V23738_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_321 = {S40, 0, &V23738_321_57295, &P23738_320};	/*Property = Progress*/
const Value V23738_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_322 = {S41, 0, &V23738_322_57298, &P23738_321};	/*Property = Prompts*/
const Value V23738_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_323 = {S42, 0, &V23738_323_57301, &P23738_322};	/*Property = Selector*/
const Value V23738_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_324 = {S43, 0, &V23738_324_57307, &P23738_323};	/*Property = Svg*/
const Value V23738_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_327 = {S44, 0, &V23738_327_57316, &P23738_324};	/*Property = TouchEvents*/
const Value V23738_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_328 = {S45, 0, &V23738_328_57319, &P23738_327};	/*Property = Track*/
const Value V23738_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_329 = {S46, 0, &V23738_329_57325, &P23738_328};	/*Property = Video*/
const Value V23738_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_330 = {S47, 0, &V23738_330_57328, &P23738_329};	/*Property = Viewport*/
const Value V23738_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_332 = {S48, 0, &V23738_332_57340, &P23738_330};	/*Property = WebWorkers*/
const Value V23738_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23738_333 = {S49, 0, &V23738_333_57343, &P23738_332};	/*Property = Xhr2*/
const Profile P23738 = {"23738", &P23738_333};	/*Profile ID - 23738*/
const Value V24722_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24722_15 = {S37, 0, &V24722_15_28, NULL};	/*Property = LayoutEngine*/
const Profile P24722 = {"24722", &P24722_15};	/*Profile ID - 24722*/
const Value V23633_1_2 = {S0, NULL};	/*Value = False*/
const Property P23633_1 = {S50, 0, &V23633_1_2, NULL};	/*Property = IsMobile*/
const Value V23633_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23633_5 = {S52, 0, &V23633_5_37569, &P23633_1};	/*Property = ScreenPixelsHeight*/
const Value V23633_4_53077 = {S129, NULL};	/*Value = 1366*/
const Property P23633_4 = {S54, 0, &V23633_4_53077, &P23633_5};	/*Property = ScreenPixelsWidth*/
const Profile P23633 = {"23633", &P23633_4};	/*Profile ID - 23633*/
const Value V23623_1_2 = {S0, NULL};	/*Value = False*/
const Property P23623_1 = {S50, 0, &V23623_1_2, NULL};	/*Property = IsMobile*/
const Value V23623_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23623_5 = {S52, 0, &V23623_5_60417, &P23623_1};	/*Property = ScreenPixelsHeight*/
const Value V23623_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23623_4 = {S54, 0, &V23623_4_57095, &P23623_5};	/*Property = ScreenPixelsWidth*/
const Profile P23623 = {"23623", &P23623_4};	/*Profile ID - 23623*/
const Value V21238_1_1 = {S2, NULL};	/*Value = True*/
const Property P21238_1 = {S50, 0, &V21238_1_1, NULL};	/*Property = IsMobile*/
const Value V21238_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P21238_5 = {S52, 0, &V21238_5_37569, &P21238_1};	/*Property = ScreenPixelsHeight*/
const Value V21238_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21238_4 = {S54, 0, &V21238_4_5645, &P21238_5};	/*Property = ScreenPixelsWidth*/
const Profile P21238 = {"21238", &P21238_4};	/*Profile ID - 21238*/
const Value V20331_1_2 = {S0, NULL};	/*Value = False*/
const Property P20331_1 = {S50, 0, &V20331_1_2, NULL};	/*Property = IsMobile*/
const Value V20331_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20331_5 = {S52, 0, &V20331_5_37, &P20331_1};	/*Property = ScreenPixelsHeight*/
const Value V20331_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20331_4 = {S54, 0, &V20331_4_36, &P20331_5};	/*Property = ScreenPixelsWidth*/
const Profile P20331 = {"20331", &P20331_4};	/*Profile ID - 20331*/
const Value V20247_1_1 = {S2, NULL};	/*Value = True*/
const Property P20247_1 = {S50, 0, &V20247_1_1, NULL};	/*Property = IsMobile*/
const Value V20247_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20247_5 = {S52, 0, &V20247_5_1222, &P20247_1};	/*Property = ScreenPixelsHeight*/
const Value V20247_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20247_4 = {S54, 0, &V20247_4_1224, &P20247_5};	/*Property = ScreenPixelsWidth*/
const Profile P20247 = {"20247", &P20247_4};	/*Profile ID - 20247*/
const Value V21970_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_279 = {S1, 0, &V21970_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21970_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_282 = {S3, 0, &V21970_282_57155, &P21970_279};	/*Property = BlobBuilder*/
const Value V21970_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_350 = {S4, 0, &V21970_350_57446, &P21970_282};	/*Property = Canvas*/
const Value V21970_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_283 = {S5, 0, &V21970_283_57161, &P21970_350};	/*Property = CssBackground*/
const Value V21970_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_284 = {S6, 0, &V21970_284_57164, &P21970_283};	/*Property = CssBorderImage*/
const Value V21970_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_285 = {S7, 0, &V21970_285_57167, &P21970_284};	/*Property = CssCanvas*/
const Value V21970_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_286 = {S8, 0, &V21970_286_57170, &P21970_285};	/*Property = CssColor*/
const Value V21970_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_237 = {S9, 0, &V21970_237_55786, &P21970_286};	/*Property = CssColumn*/
const Value V21970_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_287 = {S10, 0, &V21970_287_57173, &P21970_237};	/*Property = CssFlexbox*/
const Value V21970_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_288 = {S11, 0, &V21970_288_57176, &P21970_287};	/*Property = CssFont*/
const Value V21970_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_289 = {S12, 0, &V21970_289_86899, &P21970_288};	/*Property = CssImages*/
const Value V21970_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_291 = {S13, 0, &V21970_291_57182, &P21970_289};	/*Property = CssMediaQueries*/
const Value V21970_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_292 = {S14, 0, &V21970_292_57185, &P21970_291};	/*Property = CssMinMax*/
const Value V21970_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_293 = {S15, 0, &V21970_293_57188, &P21970_292};	/*Property = CssOverflow*/
const Value V21970_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_294 = {S16, 0, &V21970_294_57191, &P21970_293};	/*Property = CssPosition*/
const Value V21970_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_295 = {S17, 0, &V21970_295_57194, &P21970_294};	/*Property = CssText*/
const Value V21970_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_297 = {S18, 0, &V21970_297_57197, &P21970_295};	/*Property = CssTransforms*/
const Value V21970_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_241 = {S19, 0, &V21970_241_55801, &P21970_297};	/*Property = CssTransitions*/
const Value V21970_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_298 = {S20, 0, &V21970_298_57208, &P21970_241};	/*Property = CssUI*/
const Value V21970_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_300 = {S21, 0, &V21970_300_57217, &P21970_298};	/*Property = DataSet*/
const Value V21970_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_301 = {S22, 0, &V21970_301_57220, &P21970_300};	/*Property = DataUrl*/
const Value V21970_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_302 = {S23, 0, &V21970_302_57223, &P21970_301};	/*Property = DeviceOrientation*/
const Value V21970_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_304 = {S24, 0, &V21970_304_57229, &P21970_302};	/*Property = FileReader*/
const Value V21970_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_305 = {S25, 0, &V21970_305_57232, &P21970_304};	/*Property = FileSaver*/
const Value V21970_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_306 = {S26, 0, &V21970_306_57238, &P21970_305};	/*Property = FileWriter*/
const Value V21970_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_307 = {S27, 0, &V21970_307_57241, &P21970_306};	/*Property = FormData*/
const Value V21970_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_309 = {S28, 0, &V21970_309_57247, &P21970_307};	/*Property = Fullscreen*/
const Value V21970_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_310 = {S29, 0, &V21970_310_57250, &P21970_309};	/*Property = GeoLocation*/
const Value V21970_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_312 = {S30, 0, &V21970_312_57256, &P21970_310};	/*Property = History*/
const Value V21970_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_314 = {S31, 0, &V21970_314_57262, &P21970_312};	/*Property = Html5*/
const Value V21970_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_313 = {S32, 0, &V21970_313_57259, &P21970_314};	/*Property = Html-Media-Capture*/
const Value V21970_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_315 = {S33, 0, &V21970_315_57265, &P21970_313};	/*Property = Iframe*/
const Value V21970_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_316 = {S34, 0, &V21970_316_57268, &P21970_315};	/*Property = IndexedDB*/
const Value V21970_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_317 = {S35, 0, &V21970_317_57271, &P21970_316};	/*Property = Json*/
const Value V21970_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P21970_15 = {S37, 0, &V21970_15_29, &P21970_317};	/*Property = LayoutEngine*/
const Value V21970_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_344 = {S38, 0, &V21970_344_57274, &P21970_15};	/*Property = Masking*/
const Value V21970_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_320 = {S39, 0, &V21970_320_57292, &P21970_344};	/*Property = PostMessage*/
const Value V21970_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_321 = {S40, 0, &V21970_321_57295, &P21970_320};	/*Property = Progress*/
const Value V21970_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_322 = {S41, 0, &V21970_322_57298, &P21970_321};	/*Property = Prompts*/
const Value V21970_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_323 = {S42, 0, &V21970_323_57301, &P21970_322};	/*Property = Selector*/
const Value V21970_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_324 = {S43, 0, &V21970_324_57307, &P21970_323};	/*Property = Svg*/
const Value V21970_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_327 = {S44, 0, &V21970_327_57316, &P21970_324};	/*Property = TouchEvents*/
const Value V21970_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_328 = {S45, 0, &V21970_328_57319, &P21970_327};	/*Property = Track*/
const Value V21970_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_329 = {S46, 0, &V21970_329_57325, &P21970_328};	/*Property = Video*/
const Value V21970_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_330 = {S47, 0, &V21970_330_57328, &P21970_329};	/*Property = Viewport*/
const Value V21970_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_332 = {S48, 0, &V21970_332_57340, &P21970_330};	/*Property = WebWorkers*/
const Value V21970_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21970_333 = {S49, 0, &V21970_333_57343, &P21970_332};	/*Property = Xhr2*/
const Profile P21970 = {"21970", &P21970_333};	/*Profile ID - 21970*/
const Value V21461_279_57147 = {S2, NULL};	/*Value = True*/
const Property P21461_279 = {S1, 0, &V21461_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V21461_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21461_282 = {S3, 0, &V21461_282_57154, &P21461_279};	/*Property = BlobBuilder*/
const Value V21461_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21461_350 = {S4, 0, &V21461_350_57444, &P21461_282};	/*Property = Canvas*/
const Value V21461_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21461_283 = {S5, 0, &V21461_283_57159, &P21461_350};	/*Property = CssBackground*/
const Value V21461_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21461_284 = {S6, 0, &V21461_284_57163, &P21461_283};	/*Property = CssBorderImage*/
const Value V21461_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21461_285 = {S7, 0, &V21461_285_57166, &P21461_284};	/*Property = CssCanvas*/
const Value V21461_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21461_286 = {S8, 0, &V21461_286_57168, &P21461_285};	/*Property = CssColor*/
const Value V21461_237_55785 = {S0, NULL};	/*Value = False*/
const Property P21461_237 = {S9, 0, &V21461_237_55785, &P21461_286};	/*Property = CssColumn*/
const Value V21461_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21461_287 = {S10, 0, &V21461_287_57172, &P21461_237};	/*Property = CssFlexbox*/
const Value V21461_288_57174 = {S2, NULL};	/*Value = True*/
const Property P21461_288 = {S11, 0, &V21461_288_57174, &P21461_287};	/*Property = CssFont*/
const Value V21461_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21461_289 = {S12, 0, &V21461_289_57759, &P21461_288};	/*Property = CssImages*/
const Value V21461_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21461_291 = {S13, 0, &V21461_291_57181, &P21461_289};	/*Property = CssMediaQueries*/
const Value V21461_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21461_292 = {S14, 0, &V21461_292_57183, &P21461_291};	/*Property = CssMinMax*/
const Value V21461_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21461_293 = {S15, 0, &V21461_293_57187, &P21461_292};	/*Property = CssOverflow*/
const Value V21461_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21461_294 = {S16, 0, &V21461_294_57189, &P21461_293};	/*Property = CssPosition*/
const Value V21461_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21461_295 = {S17, 0, &V21461_295_57192, &P21461_294};	/*Property = CssText*/
const Value V21461_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21461_297 = {S18, 0, &V21461_297_57195, &P21461_295};	/*Property = CssTransforms*/
const Value V21461_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21461_241 = {S19, 0, &V21461_241_55799, &P21461_297};	/*Property = CssTransitions*/
const Value V21461_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21461_298 = {S20, 0, &V21461_298_57206, &P21461_241};	/*Property = CssUI*/
const Value V21461_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21461_300 = {S21, 0, &V21461_300_57216, &P21461_298};	/*Property = DataSet*/
const Value V21461_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21461_301 = {S22, 0, &V21461_301_57218, &P21461_300};	/*Property = DataUrl*/
const Value V21461_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21461_302 = {S23, 0, &V21461_302_57222, &P21461_301};	/*Property = DeviceOrientation*/
const Value V21461_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21461_304 = {S24, 0, &V21461_304_57227, &P21461_302};	/*Property = FileReader*/
const Value V21461_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21461_305 = {S25, 0, &V21461_305_57231, &P21461_304};	/*Property = FileSaver*/
const Value V21461_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21461_306 = {S26, 0, &V21461_306_57237, &P21461_305};	/*Property = FileWriter*/
const Value V21461_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21461_307 = {S27, 0, &V21461_307_57239, &P21461_306};	/*Property = FormData*/
const Value V21461_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21461_309 = {S28, 0, &V21461_309_57246, &P21461_307};	/*Property = Fullscreen*/
const Value V21461_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21461_310 = {S29, 0, &V21461_310_57248, &P21461_309};	/*Property = GeoLocation*/
const Value V21461_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21461_312 = {S30, 0, &V21461_312_57254, &P21461_310};	/*Property = History*/
const Value V21461_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21461_314 = {S31, 0, &V21461_314_57261, &P21461_312};	/*Property = Html5*/
const Value V21461_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21461_313 = {S32, 0, &V21461_313_57258, &P21461_314};	/*Property = Html-Media-Capture*/
const Value V21461_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21461_315 = {S33, 0, &V21461_315_57264, &P21461_313};	/*Property = Iframe*/
const Value V21461_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21461_316 = {S34, 0, &V21461_316_57266, &P21461_315};	/*Property = IndexedDB*/
const Value V21461_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21461_317 = {S35, 0, &V21461_317_57269, &P21461_316};	/*Property = Json*/
const Value V21461_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P21461_15 = {S37, 0, &V21461_15_30, &P21461_317};	/*Property = LayoutEngine*/
const Value V21461_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21461_344 = {S38, 0, &V21461_344_57272, &P21461_15};	/*Property = Masking*/
const Value V21461_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21461_320 = {S39, 0, &V21461_320_57290, &P21461_344};	/*Property = PostMessage*/
const Value V21461_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21461_321 = {S40, 0, &V21461_321_57293, &P21461_320};	/*Property = Progress*/
const Value V21461_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21461_322 = {S41, 0, &V21461_322_57296, &P21461_321};	/*Property = Prompts*/
const Value V21461_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21461_323 = {S42, 0, &V21461_323_57299, &P21461_322};	/*Property = Selector*/
const Value V21461_324_57305 = {S2, NULL};	/*Value = True*/
const Property P21461_324 = {S43, 0, &V21461_324_57305, &P21461_323};	/*Property = Svg*/
const Value V21461_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21461_327 = {S44, 0, &V21461_327_57315, &P21461_324};	/*Property = TouchEvents*/
const Value V21461_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21461_328 = {S45, 0, &V21461_328_57318, &P21461_327};	/*Property = Track*/
const Value V21461_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21461_329 = {S46, 0, &V21461_329_57323, &P21461_328};	/*Property = Video*/
const Value V21461_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21461_330 = {S47, 0, &V21461_330_57326, &P21461_329};	/*Property = Viewport*/
const Value V21461_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21461_332 = {S48, 0, &V21461_332_57339, &P21461_330};	/*Property = WebWorkers*/
const Value V21461_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21461_333 = {S49, 0, &V21461_333_57342, &P21461_332};	/*Property = Xhr2*/
const Profile P21461 = {"21461", &P21461_333};	/*Profile ID - 21461*/
const Value V20033_1_1 = {S2, NULL};	/*Value = True*/
const Property P20033_1 = {S50, 0, &V20033_1_1, NULL};	/*Property = IsMobile*/
const Value V20033_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20033_5 = {S52, 0, &V20033_5_1222, &P20033_1};	/*Property = ScreenPixelsHeight*/
const Value V20033_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20033_4 = {S54, 0, &V20033_4_1224, &P20033_5};	/*Property = ScreenPixelsWidth*/
const Profile P20033 = {"20033", &P20033_4};	/*Profile ID - 20033*/
const Value V18665_1_1 = {S2, NULL};	/*Value = True*/
const Property P18665_1 = {S50, 0, &V18665_1_1, NULL};	/*Property = IsMobile*/
const Value V18665_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18665_5 = {S52, 0, &V18665_5_1403, &P18665_1};	/*Property = ScreenPixelsHeight*/
const Value V18665_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18665_4 = {S54, 0, &V18665_4_1404, &P18665_5};	/*Property = ScreenPixelsWidth*/
const Profile P18665 = {"18665", &P18665_4};	/*Profile ID - 18665*/
const Value V22799_1_1 = {S2, NULL};	/*Value = True*/
const Property P22799_1 = {S50, 0, &V22799_1_1, NULL};	/*Property = IsMobile*/
const Value V22799_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P22799_5 = {S52, 0, &V22799_5_1304, &P22799_1};	/*Property = ScreenPixelsHeight*/
const Value V22799_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22799_4 = {S54, 0, &V22799_4_1224, &P22799_5};	/*Property = ScreenPixelsWidth*/
const Profile P22799 = {"22799", &P22799_4};	/*Profile ID - 22799*/
const Value V20032_1_1 = {S2, NULL};	/*Value = True*/
const Property P20032_1 = {S50, 0, &V20032_1_1, NULL};	/*Property = IsMobile*/
const Value V20032_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20032_5 = {S52, 0, &V20032_5_1222, &P20032_1};	/*Property = ScreenPixelsHeight*/
const Value V20032_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20032_4 = {S54, 0, &V20032_4_1224, &P20032_5};	/*Property = ScreenPixelsWidth*/
const Profile P20032 = {"20032", &P20032_4};	/*Profile ID - 20032*/
const Value V18672_1_1 = {S2, NULL};	/*Value = True*/
const Property P18672_1 = {S50, 0, &V18672_1_1, NULL};	/*Property = IsMobile*/
const Value V18672_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18672_5 = {S52, 0, &V18672_5_1222, &P18672_1};	/*Property = ScreenPixelsHeight*/
const Value V18672_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18672_4 = {S54, 0, &V18672_4_1224, &P18672_5};	/*Property = ScreenPixelsWidth*/
const Profile P18672 = {"18672", &P18672_4};	/*Profile ID - 18672*/
const Value V20028_1_1 = {S2, NULL};	/*Value = True*/
const Property P20028_1 = {S50, 0, &V20028_1_1, NULL};	/*Property = IsMobile*/
const Value V20028_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20028_5 = {S52, 0, &V20028_5_1222, &P20028_1};	/*Property = ScreenPixelsHeight*/
const Value V20028_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20028_4 = {S54, 0, &V20028_4_1224, &P20028_5};	/*Property = ScreenPixelsWidth*/
const Profile P20028 = {"20028", &P20028_4};	/*Profile ID - 20028*/
const Value V19972_1_1 = {S2, NULL};	/*Value = True*/
const Property P19972_1 = {S50, 0, &V19972_1_1, NULL};	/*Property = IsMobile*/
const Value V19972_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P19972_5 = {S52, 0, &V19972_5_1293, &P19972_1};	/*Property = ScreenPixelsHeight*/
const Value V19972_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P19972_4 = {S54, 0, &V19972_4_1296, &P19972_5};	/*Property = ScreenPixelsWidth*/
const Profile P19972 = {"19972", &P19972_4};	/*Profile ID - 19972*/
const Value V23018_1_1 = {S2, NULL};	/*Value = True*/
const Property P23018_1 = {S50, 0, &V23018_1_1, NULL};	/*Property = IsMobile*/
const Value V23018_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P23018_5 = {S52, 0, &V23018_5_1380, &P23018_1};	/*Property = ScreenPixelsHeight*/
const Value V23018_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23018_4 = {S54, 0, &V23018_4_1279, &P23018_5};	/*Property = ScreenPixelsWidth*/
const Profile P23018 = {"23018", &P23018_4};	/*Profile ID - 23018*/
const Value V18673_1_1 = {S2, NULL};	/*Value = True*/
const Property P18673_1 = {S50, 0, &V18673_1_1, NULL};	/*Property = IsMobile*/
const Value V18673_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18673_5 = {S52, 0, &V18673_5_1227, &P18673_1};	/*Property = ScreenPixelsHeight*/
const Value V18673_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18673_4 = {S54, 0, &V18673_4_1229, &P18673_5};	/*Property = ScreenPixelsWidth*/
const Profile P18673 = {"18673", &P18673_4};	/*Profile ID - 18673*/
const Value V21533_1_1 = {S2, NULL};	/*Value = True*/
const Property P21533_1 = {S50, 0, &V21533_1_1, NULL};	/*Property = IsMobile*/
const Value V21533_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21533_5 = {S52, 0, &V21533_5_1277, &P21533_1};	/*Property = ScreenPixelsHeight*/
const Value V21533_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21533_4 = {S54, 0, &V21533_4_1279, &P21533_5};	/*Property = ScreenPixelsWidth*/
const Profile P21533 = {"21533", &P21533_4};	/*Profile ID - 21533*/
const Value V18679_1_1 = {S2, NULL};	/*Value = True*/
const Property P18679_1 = {S50, 0, &V18679_1_1, NULL};	/*Property = IsMobile*/
const Value V18679_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18679_5 = {S52, 0, &V18679_5_1222, &P18679_1};	/*Property = ScreenPixelsHeight*/
const Value V18679_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18679_4 = {S54, 0, &V18679_4_1224, &P18679_5};	/*Property = ScreenPixelsWidth*/
const Profile P18679 = {"18679", &P18679_4};	/*Profile ID - 18679*/
const Value V21059_1_1 = {S2, NULL};	/*Value = True*/
const Property P21059_1 = {S50, 0, &V21059_1_1, NULL};	/*Property = IsMobile*/
const Value V21059_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21059_5 = {S52, 0, &V21059_5_1227, &P21059_1};	/*Property = ScreenPixelsHeight*/
const Value V21059_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21059_4 = {S54, 0, &V21059_4_1229, &P21059_5};	/*Property = ScreenPixelsWidth*/
const Profile P21059 = {"21059", &P21059_4};	/*Profile ID - 21059*/
const Value V20973_1_1 = {S2, NULL};	/*Value = True*/
const Property P20973_1 = {S50, 0, &V20973_1_1, NULL};	/*Property = IsMobile*/
const Value V20973_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20973_5 = {S52, 0, &V20973_5_1277, &P20973_1};	/*Property = ScreenPixelsHeight*/
const Value V20973_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20973_4 = {S54, 0, &V20973_4_1279, &P20973_5};	/*Property = ScreenPixelsWidth*/
const Profile P20973 = {"20973", &P20973_4};	/*Profile ID - 20973*/
const Value V19862_1_1 = {S2, NULL};	/*Value = True*/
const Property P19862_1 = {S50, 0, &V19862_1_1, NULL};	/*Property = IsMobile*/
const Value V19862_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P19862_5 = {S52, 0, &V19862_5_1269, &P19862_1};	/*Property = ScreenPixelsHeight*/
const Value V19862_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19862_4 = {S54, 0, &V19862_4_1229, &P19862_5};	/*Property = ScreenPixelsWidth*/
const Profile P19862 = {"19862", &P19862_4};	/*Profile ID - 19862*/
const Value V21022_1_1 = {S2, NULL};	/*Value = True*/
const Property P21022_1 = {S50, 0, &V21022_1_1, NULL};	/*Property = IsMobile*/
const Value V21022_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21022_5 = {S52, 0, &V21022_5_1277, &P21022_1};	/*Property = ScreenPixelsHeight*/
const Value V21022_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21022_4 = {S54, 0, &V21022_4_1279, &P21022_5};	/*Property = ScreenPixelsWidth*/
const Profile P21022 = {"21022", &P21022_4};	/*Profile ID - 21022*/
const Value V20073_1_1 = {S2, NULL};	/*Value = True*/
const Property P20073_1 = {S50, 0, &V20073_1_1, NULL};	/*Property = IsMobile*/
const Value V20073_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20073_5 = {S52, 0, &V20073_5_1227, &P20073_1};	/*Property = ScreenPixelsHeight*/
const Value V20073_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20073_4 = {S54, 0, &V20073_4_1229, &P20073_5};	/*Property = ScreenPixelsWidth*/
const Profile P20073 = {"20073", &P20073_4};	/*Profile ID - 20073*/
const Profile P22828 = {"22828", NULL};	/*Profile ID - 22828*/
const Value V23026_1_1 = {S2, NULL};	/*Value = True*/
const Property P23026_1 = {S50, 0, &V23026_1_1, NULL};	/*Property = IsMobile*/
const Value V23026_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23026_5 = {S52, 0, &V23026_5_1222, &P23026_1};	/*Property = ScreenPixelsHeight*/
const Value V23026_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23026_4 = {S54, 0, &V23026_4_1224, &P23026_5};	/*Property = ScreenPixelsWidth*/
const Profile P23026 = {"23026", &P23026_4};	/*Profile ID - 23026*/
const Value V20289_1_1 = {S2, NULL};	/*Value = True*/
const Property P20289_1 = {S50, 0, &V20289_1_1, NULL};	/*Property = IsMobile*/
const Value V20289_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P20289_5 = {S52, 0, &V20289_5_59138, &P20289_1};	/*Property = ScreenPixelsHeight*/
const Value V20289_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P20289_4 = {S54, 0, &V20289_4_59970, &P20289_5};	/*Property = ScreenPixelsWidth*/
const Profile P20289 = {"20289", &P20289_4};	/*Profile ID - 20289*/
const Value V20034_1_1 = {S2, NULL};	/*Value = True*/
const Property P20034_1 = {S50, 0, &V20034_1_1, NULL};	/*Property = IsMobile*/
const Value V20034_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20034_5 = {S52, 0, &V20034_5_1222, &P20034_1};	/*Property = ScreenPixelsHeight*/
const Value V20034_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20034_4 = {S54, 0, &V20034_4_1224, &P20034_5};	/*Property = ScreenPixelsWidth*/
const Profile P20034 = {"20034", &P20034_4};	/*Profile ID - 20034*/
const Value V23937_1_1 = {S2, NULL};	/*Value = True*/
const Property P23937_1 = {S50, 0, &V23937_1_1, NULL};	/*Property = IsMobile*/
const Value V23937_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23937_5 = {S52, 0, &V23937_5_1832, &P23937_1};	/*Property = ScreenPixelsHeight*/
const Value V23937_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23937_4 = {S54, 0, &V23937_4_5645, &P23937_5};	/*Property = ScreenPixelsWidth*/
const Profile P23937 = {"23937", &P23937_4};	/*Profile ID - 23937*/
const Value V20030_1_1 = {S2, NULL};	/*Value = True*/
const Property P20030_1 = {S50, 0, &V20030_1_1, NULL};	/*Property = IsMobile*/
const Value V20030_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20030_5 = {S52, 0, &V20030_5_1222, &P20030_1};	/*Property = ScreenPixelsHeight*/
const Value V20030_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20030_4 = {S54, 0, &V20030_4_1224, &P20030_5};	/*Property = ScreenPixelsWidth*/
const Profile P20030 = {"20030", &P20030_4};	/*Profile ID - 20030*/
const Value V23069_1_1 = {S2, NULL};	/*Value = True*/
const Property P23069_1 = {S50, 0, &V23069_1_1, NULL};	/*Property = IsMobile*/
const Value V23069_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23069_5 = {S52, 0, &V23069_5_1222, &P23069_1};	/*Property = ScreenPixelsHeight*/
const Value V23069_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23069_4 = {S54, 0, &V23069_4_1224, &P23069_5};	/*Property = ScreenPixelsWidth*/
const Profile P23069 = {"23069", &P23069_4};	/*Profile ID - 23069*/
const Value V19988_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19988_279 = {S1, 0, &V19988_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19988_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19988_282 = {S3, 0, &V19988_282_57154, &P19988_279};	/*Property = BlobBuilder*/
const Value V19988_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19988_350 = {S4, 0, &V19988_350_57444, &P19988_282};	/*Property = Canvas*/
const Value V19988_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19988_283 = {S5, 0, &V19988_283_57159, &P19988_350};	/*Property = CssBackground*/
const Value V19988_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19988_284 = {S6, 0, &V19988_284_57163, &P19988_283};	/*Property = CssBorderImage*/
const Value V19988_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19988_285 = {S7, 0, &V19988_285_57166, &P19988_284};	/*Property = CssCanvas*/
const Value V19988_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19988_286 = {S8, 0, &V19988_286_57168, &P19988_285};	/*Property = CssColor*/
const Value V19988_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19988_237 = {S9, 0, &V19988_237_55786, &P19988_286};	/*Property = CssColumn*/
const Value V19988_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19988_287 = {S10, 0, &V19988_287_57172, &P19988_237};	/*Property = CssFlexbox*/
const Value V19988_288_57174 = {S2, NULL};	/*Value = True*/
const Property P19988_288 = {S11, 0, &V19988_288_57174, &P19988_287};	/*Property = CssFont*/
const Value V19988_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19988_289 = {S12, 0, &V19988_289_57759, &P19988_288};	/*Property = CssImages*/
const Value V19988_291_57180 = {S2, NULL};	/*Value = True*/
const Property P19988_291 = {S13, 0, &V19988_291_57180, &P19988_289};	/*Property = CssMediaQueries*/
const Value V19988_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19988_292 = {S14, 0, &V19988_292_57183, &P19988_291};	/*Property = CssMinMax*/
const Value V19988_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19988_293 = {S15, 0, &V19988_293_57187, &P19988_292};	/*Property = CssOverflow*/
const Value V19988_294_57189 = {S2, NULL};	/*Value = True*/
const Property P19988_294 = {S16, 0, &V19988_294_57189, &P19988_293};	/*Property = CssPosition*/
const Value V19988_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19988_295 = {S17, 0, &V19988_295_57192, &P19988_294};	/*Property = CssText*/
const Value V19988_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19988_297 = {S18, 0, &V19988_297_57195, &P19988_295};	/*Property = CssTransforms*/
const Value V19988_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19988_241 = {S19, 0, &V19988_241_55799, &P19988_297};	/*Property = CssTransitions*/
const Value V19988_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19988_298 = {S20, 0, &V19988_298_57206, &P19988_241};	/*Property = CssUI*/
const Value V19988_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19988_300 = {S21, 0, &V19988_300_57216, &P19988_298};	/*Property = DataSet*/
const Value V19988_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19988_301 = {S22, 0, &V19988_301_57218, &P19988_300};	/*Property = DataUrl*/
const Value V19988_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19988_302 = {S23, 0, &V19988_302_57222, &P19988_301};	/*Property = DeviceOrientation*/
const Value V19988_304_57227 = {S2, NULL};	/*Value = True*/
const Property P19988_304 = {S24, 0, &V19988_304_57227, &P19988_302};	/*Property = FileReader*/
const Value V19988_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19988_305 = {S25, 0, &V19988_305_57231, &P19988_304};	/*Property = FileSaver*/
const Value V19988_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19988_306 = {S26, 0, &V19988_306_57237, &P19988_305};	/*Property = FileWriter*/
const Value V19988_307_57239 = {S2, NULL};	/*Value = True*/
const Property P19988_307 = {S27, 0, &V19988_307_57239, &P19988_306};	/*Property = FormData*/
const Value V19988_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19988_309 = {S28, 0, &V19988_309_57246, &P19988_307};	/*Property = Fullscreen*/
const Value V19988_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19988_310 = {S29, 0, &V19988_310_57248, &P19988_309};	/*Property = GeoLocation*/
const Value V19988_312_57254 = {S2, NULL};	/*Value = True*/
const Property P19988_312 = {S30, 0, &V19988_312_57254, &P19988_310};	/*Property = History*/
const Value V19988_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19988_314 = {S31, 0, &V19988_314_57261, &P19988_312};	/*Property = Html5*/
const Value V19988_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19988_313 = {S32, 0, &V19988_313_57258, &P19988_314};	/*Property = Html-Media-Capture*/
const Value V19988_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19988_315 = {S33, 0, &V19988_315_57264, &P19988_313};	/*Property = Iframe*/
const Value V19988_316_57266 = {S2, NULL};	/*Value = True*/
const Property P19988_316 = {S34, 0, &V19988_316_57266, &P19988_315};	/*Property = IndexedDB*/
const Value V19988_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19988_317 = {S35, 0, &V19988_317_57269, &P19988_316};	/*Property = Json*/
const Value V19988_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19988_15 = {S37, 0, &V19988_15_28, &P19988_317};	/*Property = LayoutEngine*/
const Value V19988_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19988_344 = {S38, 0, &V19988_344_57272, &P19988_15};	/*Property = Masking*/
const Value V19988_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19988_320 = {S39, 0, &V19988_320_57290, &P19988_344};	/*Property = PostMessage*/
const Value V19988_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19988_321 = {S40, 0, &V19988_321_57293, &P19988_320};	/*Property = Progress*/
const Value V19988_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19988_322 = {S41, 0, &V19988_322_57296, &P19988_321};	/*Property = Prompts*/
const Value V19988_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19988_323 = {S42, 0, &V19988_323_57299, &P19988_322};	/*Property = Selector*/
const Value V19988_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19988_324 = {S43, 0, &V19988_324_57306, &P19988_323};	/*Property = Svg*/
const Value V19988_327_57314 = {S2, NULL};	/*Value = True*/
const Property P19988_327 = {S44, 0, &V19988_327_57314, &P19988_324};	/*Property = TouchEvents*/
const Value V19988_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19988_328 = {S45, 0, &V19988_328_57318, &P19988_327};	/*Property = Track*/
const Value V19988_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19988_329 = {S46, 0, &V19988_329_57323, &P19988_328};	/*Property = Video*/
const Value V19988_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19988_330 = {S47, 0, &V19988_330_57326, &P19988_329};	/*Property = Viewport*/
const Value V19988_332_57339 = {S0, NULL};	/*Value = False*/
const Property P19988_332 = {S48, 0, &V19988_332_57339, &P19988_330};	/*Property = WebWorkers*/
const Value V19988_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19988_333 = {S49, 0, &V19988_333_57342, &P19988_332};	/*Property = Xhr2*/
const Profile P19988 = {"19988", &P19988_333};	/*Profile ID - 19988*/
const Value V19902_1_1 = {S2, NULL};	/*Value = True*/
const Property P19902_1 = {S50, 0, &V19902_1_1, NULL};	/*Property = IsMobile*/
const Value V19902_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19902_5 = {S52, 0, &V19902_5_9981, &P19902_1};	/*Property = ScreenPixelsHeight*/
const Value V19902_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P19902_4 = {S54, 0, &V19902_4_39098, &P19902_5};	/*Property = ScreenPixelsWidth*/
const Profile P19902 = {"19902", &P19902_4};	/*Profile ID - 19902*/
const Value V21234_1_1 = {S2, NULL};	/*Value = True*/
const Property P21234_1 = {S50, 0, &V21234_1_1, NULL};	/*Property = IsMobile*/
const Value V21234_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21234_5 = {S52, 0, &V21234_5_1222, &P21234_1};	/*Property = ScreenPixelsHeight*/
const Value V21234_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21234_4 = {S54, 0, &V21234_4_1229, &P21234_5};	/*Property = ScreenPixelsWidth*/
const Profile P21234 = {"21234", &P21234_4};	/*Profile ID - 21234*/
const Value V22987_1_1 = {S2, NULL};	/*Value = True*/
const Property P22987_1 = {S50, 0, &V22987_1_1, NULL};	/*Property = IsMobile*/
const Value V22987_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22987_5 = {S52, 0, &V22987_5_1222, &P22987_1};	/*Property = ScreenPixelsHeight*/
const Value V22987_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22987_4 = {S54, 0, &V22987_4_1224, &P22987_5};	/*Property = ScreenPixelsWidth*/
const Profile P22987 = {"22987", &P22987_4};	/*Profile ID - 22987*/
const Value V23405_279_57148 = {S0, NULL};	/*Value = False*/
const Property P23405_279 = {S1, 0, &V23405_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V23405_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23405_282 = {S3, 0, &V23405_282_57154, &P23405_279};	/*Property = BlobBuilder*/
const Value V23405_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23405_350 = {S4, 0, &V23405_350_57444, &P23405_282};	/*Property = Canvas*/
const Value V23405_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23405_283 = {S5, 0, &V23405_283_57159, &P23405_350};	/*Property = CssBackground*/
const Value V23405_284_57162 = {S2, NULL};	/*Value = True*/
const Property P23405_284 = {S6, 0, &V23405_284_57162, &P23405_283};	/*Property = CssBorderImage*/
const Value V23405_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23405_285 = {S7, 0, &V23405_285_57166, &P23405_284};	/*Property = CssCanvas*/
const Value V23405_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23405_286 = {S8, 0, &V23405_286_57168, &P23405_285};	/*Property = CssColor*/
const Value V23405_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23405_237 = {S9, 0, &V23405_237_55786, &P23405_286};	/*Property = CssColumn*/
const Value V23405_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23405_287 = {S10, 0, &V23405_287_57172, &P23405_237};	/*Property = CssFlexbox*/
const Value V23405_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23405_288 = {S11, 0, &V23405_288_57174, &P23405_287};	/*Property = CssFont*/
const Value V23405_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23405_289 = {S12, 0, &V23405_289_57759, &P23405_288};	/*Property = CssImages*/
const Value V23405_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23405_291 = {S13, 0, &V23405_291_57180, &P23405_289};	/*Property = CssMediaQueries*/
const Value V23405_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23405_292 = {S14, 0, &V23405_292_57183, &P23405_291};	/*Property = CssMinMax*/
const Value V23405_293_57186 = {S2, NULL};	/*Value = True*/
const Property P23405_293 = {S15, 0, &V23405_293_57186, &P23405_292};	/*Property = CssOverflow*/
const Value V23405_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23405_294 = {S16, 0, &V23405_294_57189, &P23405_293};	/*Property = CssPosition*/
const Value V23405_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23405_295 = {S17, 0, &V23405_295_57192, &P23405_294};	/*Property = CssText*/
const Value V23405_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23405_297 = {S18, 0, &V23405_297_57195, &P23405_295};	/*Property = CssTransforms*/
const Value V23405_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23405_241 = {S19, 0, &V23405_241_55799, &P23405_297};	/*Property = CssTransitions*/
const Value V23405_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23405_298 = {S20, 0, &V23405_298_57206, &P23405_241};	/*Property = CssUI*/
const Value V23405_300_57215 = {S2, NULL};	/*Value = True*/
const Property P23405_300 = {S21, 0, &V23405_300_57215, &P23405_298};	/*Property = DataSet*/
const Value V23405_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23405_301 = {S22, 0, &V23405_301_57218, &P23405_300};	/*Property = DataUrl*/
const Value V23405_302_57221 = {S2, NULL};	/*Value = True*/
const Property P23405_302 = {S23, 0, &V23405_302_57221, &P23405_301};	/*Property = DeviceOrientation*/
const Value V23405_304_57228 = {S0, NULL};	/*Value = False*/
const Property P23405_304 = {S24, 0, &V23405_304_57228, &P23405_302};	/*Property = FileReader*/
const Value V23405_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23405_305 = {S25, 0, &V23405_305_57231, &P23405_304};	/*Property = FileSaver*/
const Value V23405_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23405_306 = {S26, 0, &V23405_306_57237, &P23405_305};	/*Property = FileWriter*/
const Value V23405_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23405_307 = {S27, 0, &V23405_307_57239, &P23405_306};	/*Property = FormData*/
const Value V23405_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23405_309 = {S28, 0, &V23405_309_57246, &P23405_307};	/*Property = Fullscreen*/
const Value V23405_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23405_310 = {S29, 0, &V23405_310_57248, &P23405_309};	/*Property = GeoLocation*/
const Value V23405_312_57255 = {S0, NULL};	/*Value = False*/
const Property P23405_312 = {S30, 0, &V23405_312_57255, &P23405_310};	/*Property = History*/
const Value V23405_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23405_314 = {S31, 0, &V23405_314_57261, &P23405_312};	/*Property = Html5*/
const Value V23405_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23405_313 = {S32, 0, &V23405_313_57258, &P23405_314};	/*Property = Html-Media-Capture*/
const Value V23405_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23405_315 = {S33, 0, &V23405_315_57264, &P23405_313};	/*Property = Iframe*/
const Value V23405_316_57267 = {S0, NULL};	/*Value = False*/
const Property P23405_316 = {S34, 0, &V23405_316_57267, &P23405_315};	/*Property = IndexedDB*/
const Value V23405_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23405_317 = {S35, 0, &V23405_317_57269, &P23405_316};	/*Property = Json*/
const Value V23405_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P23405_15 = {S37, 0, &V23405_15_54167, &P23405_317};	/*Property = LayoutEngine*/
const Value V23405_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23405_344 = {S38, 0, &V23405_344_57272, &P23405_15};	/*Property = Masking*/
const Value V23405_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23405_320 = {S39, 0, &V23405_320_57290, &P23405_344};	/*Property = PostMessage*/
const Value V23405_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23405_321 = {S40, 0, &V23405_321_57293, &P23405_320};	/*Property = Progress*/
const Value V23405_322_57297 = {S0, NULL};	/*Value = False*/
const Property P23405_322 = {S41, 0, &V23405_322_57297, &P23405_321};	/*Property = Prompts*/
const Value V23405_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23405_323 = {S42, 0, &V23405_323_57299, &P23405_322};	/*Property = Selector*/
const Value V23405_324_57306 = {S0, NULL};	/*Value = False*/
const Property P23405_324 = {S43, 0, &V23405_324_57306, &P23405_323};	/*Property = Svg*/
const Value V23405_327_57314 = {S2, NULL};	/*Value = True*/
const Property P23405_327 = {S44, 0, &V23405_327_57314, &P23405_324};	/*Property = TouchEvents*/
const Value V23405_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23405_328 = {S45, 0, &V23405_328_57318, &P23405_327};	/*Property = Track*/
const Value V23405_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23405_329 = {S46, 0, &V23405_329_57323, &P23405_328};	/*Property = Video*/
const Value V23405_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23405_330 = {S47, 0, &V23405_330_57326, &P23405_329};	/*Property = Viewport*/
const Value V23405_332_57339 = {S0, NULL};	/*Value = False*/
const Property P23405_332 = {S48, 0, &V23405_332_57339, &P23405_330};	/*Property = WebWorkers*/
const Value V23405_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23405_333 = {S49, 0, &V23405_333_57342, &P23405_332};	/*Property = Xhr2*/
const Profile P23405 = {"23405", &P23405_333};	/*Profile ID - 23405*/
const Value V18652_1_1 = {S2, NULL};	/*Value = True*/
const Property P18652_1 = {S50, 0, &V18652_1_1, NULL};	/*Property = IsMobile*/
const Value V18652_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18652_5 = {S52, 0, &V18652_5_1227, &P18652_1};	/*Property = ScreenPixelsHeight*/
const Value V18652_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18652_4 = {S54, 0, &V18652_4_1229, &P18652_5};	/*Property = ScreenPixelsWidth*/
const Profile P18652 = {"18652", &P18652_4};	/*Profile ID - 18652*/
const Value V18682_1_1 = {S2, NULL};	/*Value = True*/
const Property P18682_1 = {S50, 0, &V18682_1_1, NULL};	/*Property = IsMobile*/
const Value V18682_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18682_5 = {S52, 0, &V18682_5_1227, &P18682_1};	/*Property = ScreenPixelsHeight*/
const Value V18682_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18682_4 = {S54, 0, &V18682_4_1229, &P18682_5};	/*Property = ScreenPixelsWidth*/
const Profile P18682 = {"18682", &P18682_4};	/*Profile ID - 18682*/
const Value V22693_1_1 = {S2, NULL};	/*Value = True*/
const Property P22693_1 = {S50, 0, &V22693_1_1, NULL};	/*Property = IsMobile*/
const Value V22693_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P22693_5 = {S52, 0, &V22693_5_1293, &P22693_1};	/*Property = ScreenPixelsHeight*/
const Value V22693_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P22693_4 = {S54, 0, &V22693_4_1296, &P22693_5};	/*Property = ScreenPixelsWidth*/
const Profile P22693 = {"22693", &P22693_4};	/*Profile ID - 22693*/
const Value V23892_1_2 = {S0, NULL};	/*Value = False*/
const Property P23892_1 = {S50, 0, &V23892_1_2, NULL};	/*Property = IsMobile*/
const Value V23892_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23892_5 = {S52, 0, &V23892_5_37, &P23892_1};	/*Property = ScreenPixelsHeight*/
const Value V23892_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23892_4 = {S54, 0, &V23892_4_36, &P23892_5};	/*Property = ScreenPixelsWidth*/
const Profile P23892 = {"23892", &P23892_4};	/*Profile ID - 23892*/
const Value V23336_1_1 = {S2, NULL};	/*Value = True*/
const Property P23336_1 = {S50, 0, &V23336_1_1, NULL};	/*Property = IsMobile*/
const Value V23336_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23336_5 = {S52, 0, &V23336_5_1227, &P23336_1};	/*Property = ScreenPixelsHeight*/
const Value V23336_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23336_4 = {S54, 0, &V23336_4_1229, &P23336_5};	/*Property = ScreenPixelsWidth*/
const Profile P23336 = {"23336", &P23336_4};	/*Profile ID - 23336*/
const Value V21144_1_1 = {S2, NULL};	/*Value = True*/
const Property P21144_1 = {S50, 0, &V21144_1_1, NULL};	/*Property = IsMobile*/
const Value V21144_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P21144_5 = {S52, 0, &V21144_5_1304, &P21144_1};	/*Property = ScreenPixelsHeight*/
const Value V21144_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21144_4 = {S54, 0, &V21144_4_1224, &P21144_5};	/*Property = ScreenPixelsWidth*/
const Profile P21144 = {"21144", &P21144_4};	/*Profile ID - 21144*/
const Value V20763_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_279 = {S1, 0, &V20763_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20763_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_282 = {S3, 0, &V20763_282_57155, &P20763_279};	/*Property = BlobBuilder*/
const Value V20763_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_350 = {S4, 0, &V20763_350_57446, &P20763_282};	/*Property = Canvas*/
const Value V20763_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_283 = {S5, 0, &V20763_283_57161, &P20763_350};	/*Property = CssBackground*/
const Value V20763_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_284 = {S6, 0, &V20763_284_57164, &P20763_283};	/*Property = CssBorderImage*/
const Value V20763_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_285 = {S7, 0, &V20763_285_57167, &P20763_284};	/*Property = CssCanvas*/
const Value V20763_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_286 = {S8, 0, &V20763_286_57170, &P20763_285};	/*Property = CssColor*/
const Value V20763_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_237 = {S9, 0, &V20763_237_55786, &P20763_286};	/*Property = CssColumn*/
const Value V20763_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_287 = {S10, 0, &V20763_287_57173, &P20763_237};	/*Property = CssFlexbox*/
const Value V20763_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_288 = {S11, 0, &V20763_288_57176, &P20763_287};	/*Property = CssFont*/
const Value V20763_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_289 = {S12, 0, &V20763_289_86899, &P20763_288};	/*Property = CssImages*/
const Value V20763_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_291 = {S13, 0, &V20763_291_57182, &P20763_289};	/*Property = CssMediaQueries*/
const Value V20763_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_292 = {S14, 0, &V20763_292_57185, &P20763_291};	/*Property = CssMinMax*/
const Value V20763_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_293 = {S15, 0, &V20763_293_57188, &P20763_292};	/*Property = CssOverflow*/
const Value V20763_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_294 = {S16, 0, &V20763_294_57191, &P20763_293};	/*Property = CssPosition*/
const Value V20763_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_295 = {S17, 0, &V20763_295_57194, &P20763_294};	/*Property = CssText*/
const Value V20763_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_297 = {S18, 0, &V20763_297_57197, &P20763_295};	/*Property = CssTransforms*/
const Value V20763_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_241 = {S19, 0, &V20763_241_55801, &P20763_297};	/*Property = CssTransitions*/
const Value V20763_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_298 = {S20, 0, &V20763_298_57208, &P20763_241};	/*Property = CssUI*/
const Value V20763_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_300 = {S21, 0, &V20763_300_57217, &P20763_298};	/*Property = DataSet*/
const Value V20763_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_301 = {S22, 0, &V20763_301_57220, &P20763_300};	/*Property = DataUrl*/
const Value V20763_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_302 = {S23, 0, &V20763_302_57223, &P20763_301};	/*Property = DeviceOrientation*/
const Value V20763_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_304 = {S24, 0, &V20763_304_57229, &P20763_302};	/*Property = FileReader*/
const Value V20763_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_305 = {S25, 0, &V20763_305_57232, &P20763_304};	/*Property = FileSaver*/
const Value V20763_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_306 = {S26, 0, &V20763_306_57238, &P20763_305};	/*Property = FileWriter*/
const Value V20763_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_307 = {S27, 0, &V20763_307_57241, &P20763_306};	/*Property = FormData*/
const Value V20763_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_309 = {S28, 0, &V20763_309_57247, &P20763_307};	/*Property = Fullscreen*/
const Value V20763_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_310 = {S29, 0, &V20763_310_57250, &P20763_309};	/*Property = GeoLocation*/
const Value V20763_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_312 = {S30, 0, &V20763_312_57256, &P20763_310};	/*Property = History*/
const Value V20763_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_314 = {S31, 0, &V20763_314_57262, &P20763_312};	/*Property = Html5*/
const Value V20763_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_313 = {S32, 0, &V20763_313_57259, &P20763_314};	/*Property = Html-Media-Capture*/
const Value V20763_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_315 = {S33, 0, &V20763_315_57265, &P20763_313};	/*Property = Iframe*/
const Value V20763_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_316 = {S34, 0, &V20763_316_57268, &P20763_315};	/*Property = IndexedDB*/
const Value V20763_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_317 = {S35, 0, &V20763_317_57271, &P20763_316};	/*Property = Json*/
const Value V20763_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P20763_15 = {S37, 0, &V20763_15_90837, &P20763_317};	/*Property = LayoutEngine*/
const Value V20763_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_344 = {S38, 0, &V20763_344_57274, &P20763_15};	/*Property = Masking*/
const Value V20763_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_320 = {S39, 0, &V20763_320_57292, &P20763_344};	/*Property = PostMessage*/
const Value V20763_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_321 = {S40, 0, &V20763_321_57295, &P20763_320};	/*Property = Progress*/
const Value V20763_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_322 = {S41, 0, &V20763_322_57298, &P20763_321};	/*Property = Prompts*/
const Value V20763_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_323 = {S42, 0, &V20763_323_57301, &P20763_322};	/*Property = Selector*/
const Value V20763_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_324 = {S43, 0, &V20763_324_57307, &P20763_323};	/*Property = Svg*/
const Value V20763_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_327 = {S44, 0, &V20763_327_57316, &P20763_324};	/*Property = TouchEvents*/
const Value V20763_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_328 = {S45, 0, &V20763_328_57319, &P20763_327};	/*Property = Track*/
const Value V20763_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_329 = {S46, 0, &V20763_329_57325, &P20763_328};	/*Property = Video*/
const Value V20763_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_330 = {S47, 0, &V20763_330_57328, &P20763_329};	/*Property = Viewport*/
const Value V20763_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_332 = {S48, 0, &V20763_332_57340, &P20763_330};	/*Property = WebWorkers*/
const Value V20763_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20763_333 = {S49, 0, &V20763_333_57343, &P20763_332};	/*Property = Xhr2*/
const Profile P20763 = {"20763", &P20763_333};	/*Profile ID - 20763*/
const Value V21330_1_1 = {S2, NULL};	/*Value = True*/
const Property P21330_1 = {S50, 0, &V21330_1_1, NULL};	/*Property = IsMobile*/
const Value V21330_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21330_5 = {S52, 0, &V21330_5_1458, &P21330_1};	/*Property = ScreenPixelsHeight*/
const Value V21330_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21330_4 = {S54, 0, &V21330_4_37617, &P21330_5};	/*Property = ScreenPixelsWidth*/
const Profile P21330 = {"21330", &P21330_4};	/*Profile ID - 21330*/
const Value V20262_1_1 = {S2, NULL};	/*Value = True*/
const Property P20262_1 = {S50, 0, &V20262_1_1, NULL};	/*Property = IsMobile*/
const Value V20262_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20262_5 = {S52, 0, &V20262_5_1227, &P20262_1};	/*Property = ScreenPixelsHeight*/
const Value V20262_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20262_4 = {S54, 0, &V20262_4_1229, &P20262_5};	/*Property = ScreenPixelsWidth*/
const Profile P20262 = {"20262", &P20262_4};	/*Profile ID - 20262*/
const Value V20373_1_2 = {S0, NULL};	/*Value = False*/
const Property P20373_1 = {S50, 0, &V20373_1_2, NULL};	/*Property = IsMobile*/
const Value V20373_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20373_5 = {S52, 0, &V20373_5_37, &P20373_1};	/*Property = ScreenPixelsHeight*/
const Value V20373_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20373_4 = {S54, 0, &V20373_4_36, &P20373_5};	/*Property = ScreenPixelsWidth*/
const Profile P20373 = {"20373", &P20373_4};	/*Profile ID - 20373*/
const Value V20981_1_1 = {S2, NULL};	/*Value = True*/
const Property P20981_1 = {S50, 0, &V20981_1_1, NULL};	/*Property = IsMobile*/
const Value V20981_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20981_5 = {S52, 0, &V20981_5_1227, &P20981_1};	/*Property = ScreenPixelsHeight*/
const Value V20981_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20981_4 = {S54, 0, &V20981_4_1229, &P20981_5};	/*Property = ScreenPixelsWidth*/
const Profile P20981 = {"20981", &P20981_4};	/*Profile ID - 20981*/
const Value V21825_1_1 = {S2, NULL};	/*Value = True*/
const Property P21825_1 = {S50, 0, &V21825_1_1, NULL};	/*Property = IsMobile*/
const Value V21825_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21825_5 = {S52, 0, &V21825_5_37, &P21825_1};	/*Property = ScreenPixelsHeight*/
const Value V21825_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21825_4 = {S54, 0, &V21825_4_36, &P21825_5};	/*Property = ScreenPixelsWidth*/
const Profile P21825 = {"21825", &P21825_4};	/*Profile ID - 21825*/
const Value V21202_1_1 = {S2, NULL};	/*Value = True*/
const Property P21202_1 = {S50, 0, &V21202_1_1, NULL};	/*Property = IsMobile*/
const Value V21202_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21202_5 = {S52, 0, &V21202_5_9981, &P21202_1};	/*Property = ScreenPixelsHeight*/
const Value V21202_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21202_4 = {S54, 0, &V21202_4_2336, &P21202_5};	/*Property = ScreenPixelsWidth*/
const Profile P21202 = {"21202", &P21202_4};	/*Profile ID - 21202*/
const Value V18501_1_1 = {S2, NULL};	/*Value = True*/
const Property P18501_1 = {S50, 0, &V18501_1_1, NULL};	/*Property = IsMobile*/
const Value V18501_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18501_5 = {S52, 0, &V18501_5_1222, &P18501_1};	/*Property = ScreenPixelsHeight*/
const Value V18501_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18501_4 = {S54, 0, &V18501_4_1224, &P18501_5};	/*Property = ScreenPixelsWidth*/
const Profile P18501 = {"18501", &P18501_4};	/*Profile ID - 18501*/
const Value V20187_1_1 = {S2, NULL};	/*Value = True*/
const Property P20187_1 = {S50, 0, &V20187_1_1, NULL};	/*Property = IsMobile*/
const Value V20187_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20187_5 = {S52, 0, &V20187_5_9981, &P20187_1};	/*Property = ScreenPixelsHeight*/
const Value V20187_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20187_4 = {S54, 0, &V20187_4_2336, &P20187_5};	/*Property = ScreenPixelsWidth*/
const Profile P20187 = {"20187", &P20187_4};	/*Profile ID - 20187*/
const Value V21205_1_1 = {S2, NULL};	/*Value = True*/
const Property P21205_1 = {S50, 0, &V21205_1_1, NULL};	/*Property = IsMobile*/
const Value V21205_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21205_5 = {S52, 0, &V21205_5_37, &P21205_1};	/*Property = ScreenPixelsHeight*/
const Value V21205_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21205_4 = {S54, 0, &V21205_4_36, &P21205_5};	/*Property = ScreenPixelsWidth*/
const Profile P21205 = {"21205", &P21205_4};	/*Profile ID - 21205*/
const Value V20844_1_1 = {S2, NULL};	/*Value = True*/
const Property P20844_1 = {S50, 0, &V20844_1_1, NULL};	/*Property = IsMobile*/
const Value V20844_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20844_5 = {S52, 0, &V20844_5_9981, &P20844_1};	/*Property = ScreenPixelsHeight*/
const Value V20844_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20844_4 = {S54, 0, &V20844_4_2336, &P20844_5};	/*Property = ScreenPixelsWidth*/
const Profile P20844 = {"20844", &P20844_4};	/*Profile ID - 20844*/
const Value V22030_1_1 = {S2, NULL};	/*Value = True*/
const Property P22030_1 = {S50, 0, &V22030_1_1, NULL};	/*Property = IsMobile*/
const Value V22030_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22030_5 = {S52, 0, &V22030_5_1277, &P22030_1};	/*Property = ScreenPixelsHeight*/
const Value V22030_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22030_4 = {S54, 0, &V22030_4_1279, &P22030_5};	/*Property = ScreenPixelsWidth*/
const Profile P22030 = {"22030", &P22030_4};	/*Profile ID - 22030*/
const Value V18616_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18616_279 = {S1, 0, &V18616_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18616_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18616_282 = {S3, 0, &V18616_282_57154, &P18616_279};	/*Property = BlobBuilder*/
const Value V18616_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18616_350 = {S4, 0, &V18616_350_57444, &P18616_282};	/*Property = Canvas*/
const Value V18616_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18616_283 = {S5, 0, &V18616_283_57159, &P18616_350};	/*Property = CssBackground*/
const Value V18616_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18616_284 = {S6, 0, &V18616_284_57163, &P18616_283};	/*Property = CssBorderImage*/
const Value V18616_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18616_285 = {S7, 0, &V18616_285_57166, &P18616_284};	/*Property = CssCanvas*/
const Value V18616_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18616_286 = {S8, 0, &V18616_286_57168, &P18616_285};	/*Property = CssColor*/
const Value V18616_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18616_237 = {S9, 0, &V18616_237_55786, &P18616_286};	/*Property = CssColumn*/
const Value V18616_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18616_287 = {S10, 0, &V18616_287_57172, &P18616_237};	/*Property = CssFlexbox*/
const Value V18616_288_57174 = {S2, NULL};	/*Value = True*/
const Property P18616_288 = {S11, 0, &V18616_288_57174, &P18616_287};	/*Property = CssFont*/
const Value V18616_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18616_289 = {S12, 0, &V18616_289_57759, &P18616_288};	/*Property = CssImages*/
const Value V18616_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18616_291 = {S13, 0, &V18616_291_57180, &P18616_289};	/*Property = CssMediaQueries*/
const Value V18616_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18616_292 = {S14, 0, &V18616_292_57183, &P18616_291};	/*Property = CssMinMax*/
const Value V18616_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18616_293 = {S15, 0, &V18616_293_57187, &P18616_292};	/*Property = CssOverflow*/
const Value V18616_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18616_294 = {S16, 0, &V18616_294_57189, &P18616_293};	/*Property = CssPosition*/
const Value V18616_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18616_295 = {S17, 0, &V18616_295_57192, &P18616_294};	/*Property = CssText*/
const Value V18616_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18616_297 = {S18, 0, &V18616_297_57195, &P18616_295};	/*Property = CssTransforms*/
const Value V18616_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18616_241 = {S19, 0, &V18616_241_55799, &P18616_297};	/*Property = CssTransitions*/
const Value V18616_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18616_298 = {S20, 0, &V18616_298_57206, &P18616_241};	/*Property = CssUI*/
const Value V18616_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18616_300 = {S21, 0, &V18616_300_57216, &P18616_298};	/*Property = DataSet*/
const Value V18616_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18616_301 = {S22, 0, &V18616_301_57218, &P18616_300};	/*Property = DataUrl*/
const Value V18616_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18616_302 = {S23, 0, &V18616_302_57222, &P18616_301};	/*Property = DeviceOrientation*/
const Value V18616_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18616_304 = {S24, 0, &V18616_304_57227, &P18616_302};	/*Property = FileReader*/
const Value V18616_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18616_305 = {S25, 0, &V18616_305_57231, &P18616_304};	/*Property = FileSaver*/
const Value V18616_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18616_306 = {S26, 0, &V18616_306_57237, &P18616_305};	/*Property = FileWriter*/
const Value V18616_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18616_307 = {S27, 0, &V18616_307_57239, &P18616_306};	/*Property = FormData*/
const Value V18616_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18616_309 = {S28, 0, &V18616_309_57246, &P18616_307};	/*Property = Fullscreen*/
const Value V18616_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18616_310 = {S29, 0, &V18616_310_57248, &P18616_309};	/*Property = GeoLocation*/
const Value V18616_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18616_312 = {S30, 0, &V18616_312_57254, &P18616_310};	/*Property = History*/
const Value V18616_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18616_314 = {S31, 0, &V18616_314_57261, &P18616_312};	/*Property = Html5*/
const Value V18616_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18616_313 = {S32, 0, &V18616_313_57258, &P18616_314};	/*Property = Html-Media-Capture*/
const Value V18616_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18616_315 = {S33, 0, &V18616_315_57264, &P18616_313};	/*Property = Iframe*/
const Value V18616_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18616_316 = {S34, 0, &V18616_316_57266, &P18616_315};	/*Property = IndexedDB*/
const Value V18616_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18616_317 = {S35, 0, &V18616_317_57269, &P18616_316};	/*Property = Json*/
const Value V18616_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18616_15 = {S37, 0, &V18616_15_28, &P18616_317};	/*Property = LayoutEngine*/
const Value V18616_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18616_344 = {S38, 0, &V18616_344_57272, &P18616_15};	/*Property = Masking*/
const Value V18616_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18616_320 = {S39, 0, &V18616_320_57290, &P18616_344};	/*Property = PostMessage*/
const Value V18616_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18616_321 = {S40, 0, &V18616_321_57293, &P18616_320};	/*Property = Progress*/
const Value V18616_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18616_322 = {S41, 0, &V18616_322_57296, &P18616_321};	/*Property = Prompts*/
const Value V18616_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18616_323 = {S42, 0, &V18616_323_57299, &P18616_322};	/*Property = Selector*/
const Value V18616_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18616_324 = {S43, 0, &V18616_324_57306, &P18616_323};	/*Property = Svg*/
const Value V18616_327_57314 = {S2, NULL};	/*Value = True*/
const Property P18616_327 = {S44, 0, &V18616_327_57314, &P18616_324};	/*Property = TouchEvents*/
const Value V18616_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18616_328 = {S45, 0, &V18616_328_57318, &P18616_327};	/*Property = Track*/
const Value V18616_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18616_329 = {S46, 0, &V18616_329_57323, &P18616_328};	/*Property = Video*/
const Value V18616_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18616_330 = {S47, 0, &V18616_330_57326, &P18616_329};	/*Property = Viewport*/
const Value V18616_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18616_332 = {S48, 0, &V18616_332_57339, &P18616_330};	/*Property = WebWorkers*/
const Value V18616_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18616_333 = {S49, 0, &V18616_333_57342, &P18616_332};	/*Property = Xhr2*/
const Profile P18616 = {"18616", &P18616_333};	/*Profile ID - 18616*/
const Value V18520_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18520_279 = {S1, 0, &V18520_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18520_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18520_282 = {S3, 0, &V18520_282_57154, &P18520_279};	/*Property = BlobBuilder*/
const Value V18520_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18520_350 = {S4, 0, &V18520_350_57444, &P18520_282};	/*Property = Canvas*/
const Value V18520_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18520_283 = {S5, 0, &V18520_283_57159, &P18520_350};	/*Property = CssBackground*/
const Value V18520_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18520_284 = {S6, 0, &V18520_284_57163, &P18520_283};	/*Property = CssBorderImage*/
const Value V18520_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18520_285 = {S7, 0, &V18520_285_57166, &P18520_284};	/*Property = CssCanvas*/
const Value V18520_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18520_286 = {S8, 0, &V18520_286_57168, &P18520_285};	/*Property = CssColor*/
const Value V18520_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18520_237 = {S9, 0, &V18520_237_55786, &P18520_286};	/*Property = CssColumn*/
const Value V18520_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18520_287 = {S10, 0, &V18520_287_57172, &P18520_237};	/*Property = CssFlexbox*/
const Value V18520_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18520_288 = {S11, 0, &V18520_288_57175, &P18520_287};	/*Property = CssFont*/
const Value V18520_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18520_289 = {S12, 0, &V18520_289_57759, &P18520_288};	/*Property = CssImages*/
const Value V18520_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18520_291 = {S13, 0, &V18520_291_57181, &P18520_289};	/*Property = CssMediaQueries*/
const Value V18520_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18520_292 = {S14, 0, &V18520_292_57183, &P18520_291};	/*Property = CssMinMax*/
const Value V18520_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18520_293 = {S15, 0, &V18520_293_57187, &P18520_292};	/*Property = CssOverflow*/
const Value V18520_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18520_294 = {S16, 0, &V18520_294_57190, &P18520_293};	/*Property = CssPosition*/
const Value V18520_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18520_295 = {S17, 0, &V18520_295_57192, &P18520_294};	/*Property = CssText*/
const Value V18520_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18520_297 = {S18, 0, &V18520_297_57195, &P18520_295};	/*Property = CssTransforms*/
const Value V18520_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18520_241 = {S19, 0, &V18520_241_55799, &P18520_297};	/*Property = CssTransitions*/
const Value V18520_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18520_298 = {S20, 0, &V18520_298_57206, &P18520_241};	/*Property = CssUI*/
const Value V18520_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18520_300 = {S21, 0, &V18520_300_57216, &P18520_298};	/*Property = DataSet*/
const Value V18520_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18520_301 = {S22, 0, &V18520_301_57218, &P18520_300};	/*Property = DataUrl*/
const Value V18520_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18520_302 = {S23, 0, &V18520_302_57222, &P18520_301};	/*Property = DeviceOrientation*/
const Value V18520_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18520_304 = {S24, 0, &V18520_304_57227, &P18520_302};	/*Property = FileReader*/
const Value V18520_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18520_305 = {S25, 0, &V18520_305_57231, &P18520_304};	/*Property = FileSaver*/
const Value V18520_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18520_306 = {S26, 0, &V18520_306_57237, &P18520_305};	/*Property = FileWriter*/
const Value V18520_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18520_307 = {S27, 0, &V18520_307_57239, &P18520_306};	/*Property = FormData*/
const Value V18520_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18520_309 = {S28, 0, &V18520_309_57246, &P18520_307};	/*Property = Fullscreen*/
const Value V18520_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18520_310 = {S29, 0, &V18520_310_57248, &P18520_309};	/*Property = GeoLocation*/
const Value V18520_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18520_312 = {S30, 0, &V18520_312_57255, &P18520_310};	/*Property = History*/
const Value V18520_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18520_314 = {S31, 0, &V18520_314_57261, &P18520_312};	/*Property = Html5*/
const Value V18520_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18520_313 = {S32, 0, &V18520_313_57258, &P18520_314};	/*Property = Html-Media-Capture*/
const Value V18520_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18520_315 = {S33, 0, &V18520_315_57264, &P18520_313};	/*Property = Iframe*/
const Value V18520_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18520_316 = {S34, 0, &V18520_316_57267, &P18520_315};	/*Property = IndexedDB*/
const Value V18520_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18520_317 = {S35, 0, &V18520_317_57269, &P18520_316};	/*Property = Json*/
const Value V18520_15_90838 = {S70, NULL};	/*Value = U3*/
const Property P18520_15 = {S37, 0, &V18520_15_90838, &P18520_317};	/*Property = LayoutEngine*/
const Value V18520_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18520_344 = {S38, 0, &V18520_344_57272, &P18520_15};	/*Property = Masking*/
const Value V18520_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18520_320 = {S39, 0, &V18520_320_57290, &P18520_344};	/*Property = PostMessage*/
const Value V18520_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18520_321 = {S40, 0, &V18520_321_57293, &P18520_320};	/*Property = Progress*/
const Value V18520_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18520_322 = {S41, 0, &V18520_322_57296, &P18520_321};	/*Property = Prompts*/
const Value V18520_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18520_323 = {S42, 0, &V18520_323_57299, &P18520_322};	/*Property = Selector*/
const Value V18520_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18520_324 = {S43, 0, &V18520_324_57306, &P18520_323};	/*Property = Svg*/
const Value V18520_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18520_327 = {S44, 0, &V18520_327_57315, &P18520_324};	/*Property = TouchEvents*/
const Value V18520_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18520_328 = {S45, 0, &V18520_328_57318, &P18520_327};	/*Property = Track*/
const Value V18520_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18520_329 = {S46, 0, &V18520_329_57323, &P18520_328};	/*Property = Video*/
const Value V18520_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18520_330 = {S47, 0, &V18520_330_57326, &P18520_329};	/*Property = Viewport*/
const Value V18520_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18520_332 = {S48, 0, &V18520_332_57339, &P18520_330};	/*Property = WebWorkers*/
const Value V18520_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18520_333 = {S49, 0, &V18520_333_57342, &P18520_332};	/*Property = Xhr2*/
const Profile P18520 = {"18520", &P18520_333};	/*Profile ID - 18520*/
const Value V18649_1_1 = {S2, NULL};	/*Value = True*/
const Property P18649_1 = {S50, 0, &V18649_1_1, NULL};	/*Property = IsMobile*/
const Value V18649_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18649_5 = {S52, 0, &V18649_5_1277, &P18649_1};	/*Property = ScreenPixelsHeight*/
const Value V18649_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18649_4 = {S54, 0, &V18649_4_1279, &P18649_5};	/*Property = ScreenPixelsWidth*/
const Profile P18649 = {"18649", &P18649_4};	/*Profile ID - 18649*/
const Value V18641_1_1 = {S2, NULL};	/*Value = True*/
const Property P18641_1 = {S50, 0, &V18641_1_1, NULL};	/*Property = IsMobile*/
const Value V18641_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18641_5 = {S52, 0, &V18641_5_1222, &P18641_1};	/*Property = ScreenPixelsHeight*/
const Value V18641_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18641_4 = {S54, 0, &V18641_4_1224, &P18641_5};	/*Property = ScreenPixelsWidth*/
const Profile P18641 = {"18641", &P18641_4};	/*Profile ID - 18641*/
const Value V21131_1_1 = {S2, NULL};	/*Value = True*/
const Property P21131_1 = {S50, 0, &V21131_1_1, NULL};	/*Property = IsMobile*/
const Value V21131_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21131_5 = {S52, 0, &V21131_5_1277, &P21131_1};	/*Property = ScreenPixelsHeight*/
const Value V21131_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21131_4 = {S54, 0, &V21131_4_1279, &P21131_5};	/*Property = ScreenPixelsWidth*/
const Profile P21131 = {"21131", &P21131_4};	/*Profile ID - 21131*/
const Value V21128_1_1 = {S2, NULL};	/*Value = True*/
const Property P21128_1 = {S50, 0, &V21128_1_1, NULL};	/*Property = IsMobile*/
const Value V21128_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21128_5 = {S52, 0, &V21128_5_1277, &P21128_1};	/*Property = ScreenPixelsHeight*/
const Value V21128_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21128_4 = {S54, 0, &V21128_4_1279, &P21128_5};	/*Property = ScreenPixelsWidth*/
const Profile P21128 = {"21128", &P21128_4};	/*Profile ID - 21128*/
const Profile P19835 = {"19835", NULL};	/*Profile ID - 19835*/
const Value V18664_1_1 = {S2, NULL};	/*Value = True*/
const Property P18664_1 = {S50, 0, &V18664_1_1, NULL};	/*Property = IsMobile*/
const Value V18664_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18664_5 = {S52, 0, &V18664_5_1277, &P18664_1};	/*Property = ScreenPixelsHeight*/
const Value V18664_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18664_4 = {S54, 0, &V18664_4_1279, &P18664_5};	/*Property = ScreenPixelsWidth*/
const Profile P18664 = {"18664", &P18664_4};	/*Profile ID - 18664*/
const Value V19788_1_1 = {S2, NULL};	/*Value = True*/
const Property P19788_1 = {S50, 0, &V19788_1_1, NULL};	/*Property = IsMobile*/
const Value V19788_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P19788_5 = {S52, 0, &V19788_5_1293, &P19788_1};	/*Property = ScreenPixelsHeight*/
const Value V19788_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P19788_4 = {S54, 0, &V19788_4_1296, &P19788_5};	/*Property = ScreenPixelsWidth*/
const Profile P19788 = {"19788", &P19788_4};	/*Profile ID - 19788*/
const Value V21288_1_1 = {S2, NULL};	/*Value = True*/
const Property P21288_1 = {S50, 0, &V21288_1_1, NULL};	/*Property = IsMobile*/
const Value V21288_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21288_5 = {S52, 0, &V21288_5_1227, &P21288_1};	/*Property = ScreenPixelsHeight*/
const Value V21288_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21288_4 = {S54, 0, &V21288_4_1229, &P21288_5};	/*Property = ScreenPixelsWidth*/
const Profile P21288 = {"21288", &P21288_4};	/*Profile ID - 21288*/
const Value V18657_1_1 = {S2, NULL};	/*Value = True*/
const Property P18657_1 = {S50, 0, &V18657_1_1, NULL};	/*Property = IsMobile*/
const Value V18657_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18657_5 = {S52, 0, &V18657_5_1222, &P18657_1};	/*Property = ScreenPixelsHeight*/
const Value V18657_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18657_4 = {S54, 0, &V18657_4_1224, &P18657_5};	/*Property = ScreenPixelsWidth*/
const Profile P18657 = {"18657", &P18657_4};	/*Profile ID - 18657*/
const Profile P18550 = {"18550", NULL};	/*Profile ID - 18550*/
const Value V22266_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22266_279 = {S1, 0, &V22266_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22266_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22266_282 = {S3, 0, &V22266_282_57154, &P22266_279};	/*Property = BlobBuilder*/
const Value V22266_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22266_350 = {S4, 0, &V22266_350_57444, &P22266_282};	/*Property = Canvas*/
const Value V22266_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22266_283 = {S5, 0, &V22266_283_57159, &P22266_350};	/*Property = CssBackground*/
const Value V22266_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22266_284 = {S6, 0, &V22266_284_57162, &P22266_283};	/*Property = CssBorderImage*/
const Value V22266_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22266_285 = {S7, 0, &V22266_285_57166, &P22266_284};	/*Property = CssCanvas*/
const Value V22266_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22266_286 = {S8, 0, &V22266_286_57168, &P22266_285};	/*Property = CssColor*/
const Value V22266_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22266_237 = {S9, 0, &V22266_237_55786, &P22266_286};	/*Property = CssColumn*/
const Value V22266_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22266_287 = {S10, 0, &V22266_287_57172, &P22266_237};	/*Property = CssFlexbox*/
const Value V22266_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22266_288 = {S11, 0, &V22266_288_57175, &P22266_287};	/*Property = CssFont*/
const Value V22266_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22266_289 = {S12, 0, &V22266_289_57759, &P22266_288};	/*Property = CssImages*/
const Value V22266_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22266_291 = {S13, 0, &V22266_291_57180, &P22266_289};	/*Property = CssMediaQueries*/
const Value V22266_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22266_292 = {S14, 0, &V22266_292_57183, &P22266_291};	/*Property = CssMinMax*/
const Value V22266_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22266_293 = {S15, 0, &V22266_293_57187, &P22266_292};	/*Property = CssOverflow*/
const Value V22266_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22266_294 = {S16, 0, &V22266_294_57189, &P22266_293};	/*Property = CssPosition*/
const Value V22266_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22266_295 = {S17, 0, &V22266_295_57192, &P22266_294};	/*Property = CssText*/
const Value V22266_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22266_297 = {S18, 0, &V22266_297_57195, &P22266_295};	/*Property = CssTransforms*/
const Value V22266_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22266_241 = {S19, 0, &V22266_241_55799, &P22266_297};	/*Property = CssTransitions*/
const Value V22266_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22266_298 = {S20, 0, &V22266_298_57206, &P22266_241};	/*Property = CssUI*/
const Value V22266_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22266_300 = {S21, 0, &V22266_300_57215, &P22266_298};	/*Property = DataSet*/
const Value V22266_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22266_301 = {S22, 0, &V22266_301_57218, &P22266_300};	/*Property = DataUrl*/
const Value V22266_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22266_302 = {S23, 0, &V22266_302_57222, &P22266_301};	/*Property = DeviceOrientation*/
const Value V22266_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22266_304 = {S24, 0, &V22266_304_57227, &P22266_302};	/*Property = FileReader*/
const Value V22266_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22266_305 = {S25, 0, &V22266_305_57231, &P22266_304};	/*Property = FileSaver*/
const Value V22266_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22266_306 = {S26, 0, &V22266_306_57237, &P22266_305};	/*Property = FileWriter*/
const Value V22266_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22266_307 = {S27, 0, &V22266_307_57239, &P22266_306};	/*Property = FormData*/
const Value V22266_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22266_309 = {S28, 0, &V22266_309_57246, &P22266_307};	/*Property = Fullscreen*/
const Value V22266_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22266_310 = {S29, 0, &V22266_310_57248, &P22266_309};	/*Property = GeoLocation*/
const Value V22266_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22266_312 = {S30, 0, &V22266_312_57254, &P22266_310};	/*Property = History*/
const Value V22266_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22266_314 = {S31, 0, &V22266_314_57260, &P22266_312};	/*Property = Html5*/
const Value V22266_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22266_313 = {S32, 0, &V22266_313_57258, &P22266_314};	/*Property = Html-Media-Capture*/
const Value V22266_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22266_315 = {S33, 0, &V22266_315_57264, &P22266_313};	/*Property = Iframe*/
const Value V22266_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22266_316 = {S34, 0, &V22266_316_57267, &P22266_315};	/*Property = IndexedDB*/
const Value V22266_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22266_317 = {S35, 0, &V22266_317_57269, &P22266_316};	/*Property = Json*/
const Value V22266_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22266_15 = {S37, 0, &V22266_15_28, &P22266_317};	/*Property = LayoutEngine*/
const Value V22266_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22266_344 = {S38, 0, &V22266_344_57272, &P22266_15};	/*Property = Masking*/
const Value V22266_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22266_320 = {S39, 0, &V22266_320_57290, &P22266_344};	/*Property = PostMessage*/
const Value V22266_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22266_321 = {S40, 0, &V22266_321_57293, &P22266_320};	/*Property = Progress*/
const Value V22266_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22266_322 = {S41, 0, &V22266_322_57296, &P22266_321};	/*Property = Prompts*/
const Value V22266_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22266_323 = {S42, 0, &V22266_323_57299, &P22266_322};	/*Property = Selector*/
const Value V22266_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22266_324 = {S43, 0, &V22266_324_57305, &P22266_323};	/*Property = Svg*/
const Value V22266_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22266_327 = {S44, 0, &V22266_327_57315, &P22266_324};	/*Property = TouchEvents*/
const Value V22266_328_57317 = {S2, NULL};	/*Value = True*/
const Property P22266_328 = {S45, 0, &V22266_328_57317, &P22266_327};	/*Property = Track*/
const Value V22266_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22266_329 = {S46, 0, &V22266_329_57323, &P22266_328};	/*Property = Video*/
const Value V22266_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22266_330 = {S47, 0, &V22266_330_57326, &P22266_329};	/*Property = Viewport*/
const Value V22266_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22266_332 = {S48, 0, &V22266_332_57338, &P22266_330};	/*Property = WebWorkers*/
const Value V22266_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22266_333 = {S49, 0, &V22266_333_57342, &P22266_332};	/*Property = Xhr2*/
const Profile P22266 = {"22266", &P22266_333};	/*Profile ID - 22266*/
const Value V22798_1_1 = {S2, NULL};	/*Value = True*/
const Property P22798_1 = {S50, 0, &V22798_1_1, NULL};	/*Property = IsMobile*/
const Value V22798_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P22798_5 = {S52, 0, &V22798_5_1380, &P22798_1};	/*Property = ScreenPixelsHeight*/
const Value V22798_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22798_4 = {S54, 0, &V22798_4_1279, &P22798_5};	/*Property = ScreenPixelsWidth*/
const Profile P22798 = {"22798", &P22798_4};	/*Profile ID - 22798*/
const Value V21229_1_1 = {S2, NULL};	/*Value = True*/
const Property P21229_1 = {S50, 0, &V21229_1_1, NULL};	/*Property = IsMobile*/
const Value V21229_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21229_5 = {S52, 0, &V21229_5_37, &P21229_1};	/*Property = ScreenPixelsHeight*/
const Value V21229_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21229_4 = {S54, 0, &V21229_4_36, &P21229_5};	/*Property = ScreenPixelsWidth*/
const Profile P21229 = {"21229", &P21229_4};	/*Profile ID - 21229*/
const Value V21639_1_1 = {S2, NULL};	/*Value = True*/
const Property P21639_1 = {S50, 0, &V21639_1_1, NULL};	/*Property = IsMobile*/
const Value V21639_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P21639_5 = {S52, 0, &V21639_5_37569, &P21639_1};	/*Property = ScreenPixelsHeight*/
const Value V21639_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21639_4 = {S54, 0, &V21639_4_5645, &P21639_5};	/*Property = ScreenPixelsWidth*/
const Profile P21639 = {"21639", &P21639_4};	/*Profile ID - 21639*/
const Value V22094_1_1 = {S2, NULL};	/*Value = True*/
const Property P22094_1 = {S50, 0, &V22094_1_1, NULL};	/*Property = IsMobile*/
const Value V22094_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22094_5 = {S52, 0, &V22094_5_1277, &P22094_1};	/*Property = ScreenPixelsHeight*/
const Value V22094_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22094_4 = {S54, 0, &V22094_4_2336, &P22094_5};	/*Property = ScreenPixelsWidth*/
const Profile P22094 = {"22094", &P22094_4};	/*Profile ID - 22094*/
const Value V19964_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_279 = {S1, 0, &V19964_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V19964_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_282 = {S3, 0, &V19964_282_57155, &P19964_279};	/*Property = BlobBuilder*/
const Value V19964_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_350 = {S4, 0, &V19964_350_57446, &P19964_282};	/*Property = Canvas*/
const Value V19964_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_283 = {S5, 0, &V19964_283_57161, &P19964_350};	/*Property = CssBackground*/
const Value V19964_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_284 = {S6, 0, &V19964_284_57164, &P19964_283};	/*Property = CssBorderImage*/
const Value V19964_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_285 = {S7, 0, &V19964_285_57167, &P19964_284};	/*Property = CssCanvas*/
const Value V19964_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_286 = {S8, 0, &V19964_286_57170, &P19964_285};	/*Property = CssColor*/
const Value V19964_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_237 = {S9, 0, &V19964_237_55786, &P19964_286};	/*Property = CssColumn*/
const Value V19964_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_287 = {S10, 0, &V19964_287_57173, &P19964_237};	/*Property = CssFlexbox*/
const Value V19964_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_288 = {S11, 0, &V19964_288_57176, &P19964_287};	/*Property = CssFont*/
const Value V19964_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_289 = {S12, 0, &V19964_289_86899, &P19964_288};	/*Property = CssImages*/
const Value V19964_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_291 = {S13, 0, &V19964_291_57182, &P19964_289};	/*Property = CssMediaQueries*/
const Value V19964_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_292 = {S14, 0, &V19964_292_57185, &P19964_291};	/*Property = CssMinMax*/
const Value V19964_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_293 = {S15, 0, &V19964_293_57188, &P19964_292};	/*Property = CssOverflow*/
const Value V19964_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_294 = {S16, 0, &V19964_294_57191, &P19964_293};	/*Property = CssPosition*/
const Value V19964_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_295 = {S17, 0, &V19964_295_57194, &P19964_294};	/*Property = CssText*/
const Value V19964_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_297 = {S18, 0, &V19964_297_57197, &P19964_295};	/*Property = CssTransforms*/
const Value V19964_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_241 = {S19, 0, &V19964_241_55801, &P19964_297};	/*Property = CssTransitions*/
const Value V19964_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_298 = {S20, 0, &V19964_298_57208, &P19964_241};	/*Property = CssUI*/
const Value V19964_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_300 = {S21, 0, &V19964_300_57217, &P19964_298};	/*Property = DataSet*/
const Value V19964_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_301 = {S22, 0, &V19964_301_57220, &P19964_300};	/*Property = DataUrl*/
const Value V19964_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_302 = {S23, 0, &V19964_302_57223, &P19964_301};	/*Property = DeviceOrientation*/
const Value V19964_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_304 = {S24, 0, &V19964_304_57229, &P19964_302};	/*Property = FileReader*/
const Value V19964_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_305 = {S25, 0, &V19964_305_57232, &P19964_304};	/*Property = FileSaver*/
const Value V19964_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_306 = {S26, 0, &V19964_306_57238, &P19964_305};	/*Property = FileWriter*/
const Value V19964_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_307 = {S27, 0, &V19964_307_57241, &P19964_306};	/*Property = FormData*/
const Value V19964_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_309 = {S28, 0, &V19964_309_57247, &P19964_307};	/*Property = Fullscreen*/
const Value V19964_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_310 = {S29, 0, &V19964_310_57250, &P19964_309};	/*Property = GeoLocation*/
const Value V19964_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_312 = {S30, 0, &V19964_312_57256, &P19964_310};	/*Property = History*/
const Value V19964_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_314 = {S31, 0, &V19964_314_57262, &P19964_312};	/*Property = Html5*/
const Value V19964_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_313 = {S32, 0, &V19964_313_57259, &P19964_314};	/*Property = Html-Media-Capture*/
const Value V19964_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_315 = {S33, 0, &V19964_315_57265, &P19964_313};	/*Property = Iframe*/
const Value V19964_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_316 = {S34, 0, &V19964_316_57268, &P19964_315};	/*Property = IndexedDB*/
const Value V19964_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_317 = {S35, 0, &V19964_317_57271, &P19964_316};	/*Property = Json*/
const Value V19964_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P19964_15 = {S37, 0, &V19964_15_29, &P19964_317};	/*Property = LayoutEngine*/
const Value V19964_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_344 = {S38, 0, &V19964_344_57274, &P19964_15};	/*Property = Masking*/
const Value V19964_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_320 = {S39, 0, &V19964_320_57292, &P19964_344};	/*Property = PostMessage*/
const Value V19964_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_321 = {S40, 0, &V19964_321_57295, &P19964_320};	/*Property = Progress*/
const Value V19964_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_322 = {S41, 0, &V19964_322_57298, &P19964_321};	/*Property = Prompts*/
const Value V19964_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_323 = {S42, 0, &V19964_323_57301, &P19964_322};	/*Property = Selector*/
const Value V19964_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_324 = {S43, 0, &V19964_324_57307, &P19964_323};	/*Property = Svg*/
const Value V19964_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_327 = {S44, 0, &V19964_327_57316, &P19964_324};	/*Property = TouchEvents*/
const Value V19964_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_328 = {S45, 0, &V19964_328_57319, &P19964_327};	/*Property = Track*/
const Value V19964_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_329 = {S46, 0, &V19964_329_57325, &P19964_328};	/*Property = Video*/
const Value V19964_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_330 = {S47, 0, &V19964_330_57328, &P19964_329};	/*Property = Viewport*/
const Value V19964_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_332 = {S48, 0, &V19964_332_57340, &P19964_330};	/*Property = WebWorkers*/
const Value V19964_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P19964_333 = {S49, 0, &V19964_333_57343, &P19964_332};	/*Property = Xhr2*/
const Profile P19964 = {"19964", &P19964_333};	/*Profile ID - 19964*/
const Value V19861_1_1 = {S2, NULL};	/*Value = True*/
const Property P19861_1 = {S50, 0, &V19861_1_1, NULL};	/*Property = IsMobile*/
const Value V19861_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19861_5 = {S52, 0, &V19861_5_9981, &P19861_1};	/*Property = ScreenPixelsHeight*/
const Value V19861_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P19861_4 = {S54, 0, &V19861_4_39098, &P19861_5};	/*Property = ScreenPixelsWidth*/
const Profile P19861 = {"19861", &P19861_4};	/*Profile ID - 19861*/
const Value V18274_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_279 = {S1, 0, &V18274_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18274_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_282 = {S3, 0, &V18274_282_57155, &P18274_279};	/*Property = BlobBuilder*/
const Value V18274_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_350 = {S4, 0, &V18274_350_57446, &P18274_282};	/*Property = Canvas*/
const Value V18274_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_283 = {S5, 0, &V18274_283_57161, &P18274_350};	/*Property = CssBackground*/
const Value V18274_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_284 = {S6, 0, &V18274_284_57164, &P18274_283};	/*Property = CssBorderImage*/
const Value V18274_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_285 = {S7, 0, &V18274_285_57167, &P18274_284};	/*Property = CssCanvas*/
const Value V18274_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_286 = {S8, 0, &V18274_286_57170, &P18274_285};	/*Property = CssColor*/
const Value V18274_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_237 = {S9, 0, &V18274_237_55786, &P18274_286};	/*Property = CssColumn*/
const Value V18274_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_287 = {S10, 0, &V18274_287_57173, &P18274_237};	/*Property = CssFlexbox*/
const Value V18274_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_288 = {S11, 0, &V18274_288_57176, &P18274_287};	/*Property = CssFont*/
const Value V18274_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_289 = {S12, 0, &V18274_289_86899, &P18274_288};	/*Property = CssImages*/
const Value V18274_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_291 = {S13, 0, &V18274_291_57182, &P18274_289};	/*Property = CssMediaQueries*/
const Value V18274_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_292 = {S14, 0, &V18274_292_57185, &P18274_291};	/*Property = CssMinMax*/
const Value V18274_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_293 = {S15, 0, &V18274_293_57188, &P18274_292};	/*Property = CssOverflow*/
const Value V18274_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_294 = {S16, 0, &V18274_294_57191, &P18274_293};	/*Property = CssPosition*/
const Value V18274_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_295 = {S17, 0, &V18274_295_57194, &P18274_294};	/*Property = CssText*/
const Value V18274_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_297 = {S18, 0, &V18274_297_57197, &P18274_295};	/*Property = CssTransforms*/
const Value V18274_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_241 = {S19, 0, &V18274_241_55801, &P18274_297};	/*Property = CssTransitions*/
const Value V18274_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_298 = {S20, 0, &V18274_298_57208, &P18274_241};	/*Property = CssUI*/
const Value V18274_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_300 = {S21, 0, &V18274_300_57217, &P18274_298};	/*Property = DataSet*/
const Value V18274_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_301 = {S22, 0, &V18274_301_57220, &P18274_300};	/*Property = DataUrl*/
const Value V18274_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_302 = {S23, 0, &V18274_302_57223, &P18274_301};	/*Property = DeviceOrientation*/
const Value V18274_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_304 = {S24, 0, &V18274_304_57229, &P18274_302};	/*Property = FileReader*/
const Value V18274_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_305 = {S25, 0, &V18274_305_57232, &P18274_304};	/*Property = FileSaver*/
const Value V18274_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_306 = {S26, 0, &V18274_306_57238, &P18274_305};	/*Property = FileWriter*/
const Value V18274_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_307 = {S27, 0, &V18274_307_57241, &P18274_306};	/*Property = FormData*/
const Value V18274_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_309 = {S28, 0, &V18274_309_57247, &P18274_307};	/*Property = Fullscreen*/
const Value V18274_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_310 = {S29, 0, &V18274_310_57250, &P18274_309};	/*Property = GeoLocation*/
const Value V18274_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_312 = {S30, 0, &V18274_312_57256, &P18274_310};	/*Property = History*/
const Value V18274_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_314 = {S31, 0, &V18274_314_57262, &P18274_312};	/*Property = Html5*/
const Value V18274_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_313 = {S32, 0, &V18274_313_57259, &P18274_314};	/*Property = Html-Media-Capture*/
const Value V18274_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_315 = {S33, 0, &V18274_315_57265, &P18274_313};	/*Property = Iframe*/
const Value V18274_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_316 = {S34, 0, &V18274_316_57268, &P18274_315};	/*Property = IndexedDB*/
const Value V18274_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_317 = {S35, 0, &V18274_317_57271, &P18274_316};	/*Property = Json*/
const Value V18274_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18274_15 = {S37, 0, &V18274_15_31, &P18274_317};	/*Property = LayoutEngine*/
const Value V18274_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_344 = {S38, 0, &V18274_344_57274, &P18274_15};	/*Property = Masking*/
const Value V18274_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_320 = {S39, 0, &V18274_320_57292, &P18274_344};	/*Property = PostMessage*/
const Value V18274_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_321 = {S40, 0, &V18274_321_57295, &P18274_320};	/*Property = Progress*/
const Value V18274_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_322 = {S41, 0, &V18274_322_57298, &P18274_321};	/*Property = Prompts*/
const Value V18274_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_323 = {S42, 0, &V18274_323_57301, &P18274_322};	/*Property = Selector*/
const Value V18274_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_324 = {S43, 0, &V18274_324_57307, &P18274_323};	/*Property = Svg*/
const Value V18274_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_327 = {S44, 0, &V18274_327_57316, &P18274_324};	/*Property = TouchEvents*/
const Value V18274_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_328 = {S45, 0, &V18274_328_57319, &P18274_327};	/*Property = Track*/
const Value V18274_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_329 = {S46, 0, &V18274_329_57325, &P18274_328};	/*Property = Video*/
const Value V18274_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_330 = {S47, 0, &V18274_330_57328, &P18274_329};	/*Property = Viewport*/
const Value V18274_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_332 = {S48, 0, &V18274_332_57340, &P18274_330};	/*Property = WebWorkers*/
const Value V18274_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18274_333 = {S49, 0, &V18274_333_57343, &P18274_332};	/*Property = Xhr2*/
const Profile P18274 = {"18274", &P18274_333};	/*Profile ID - 18274*/
const Value V22898_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22898_279 = {S1, 0, &V22898_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22898_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22898_282 = {S3, 0, &V22898_282_57154, &P22898_279};	/*Property = BlobBuilder*/
const Value V22898_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22898_350 = {S4, 0, &V22898_350_57444, &P22898_282};	/*Property = Canvas*/
const Value V22898_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22898_283 = {S5, 0, &V22898_283_57159, &P22898_350};	/*Property = CssBackground*/
const Value V22898_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22898_284 = {S6, 0, &V22898_284_57163, &P22898_283};	/*Property = CssBorderImage*/
const Value V22898_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22898_285 = {S7, 0, &V22898_285_57166, &P22898_284};	/*Property = CssCanvas*/
const Value V22898_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22898_286 = {S8, 0, &V22898_286_57168, &P22898_285};	/*Property = CssColor*/
const Value V22898_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22898_237 = {S9, 0, &V22898_237_55786, &P22898_286};	/*Property = CssColumn*/
const Value V22898_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22898_287 = {S10, 0, &V22898_287_57172, &P22898_237};	/*Property = CssFlexbox*/
const Value V22898_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22898_288 = {S11, 0, &V22898_288_57174, &P22898_287};	/*Property = CssFont*/
const Value V22898_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22898_289 = {S12, 0, &V22898_289_57759, &P22898_288};	/*Property = CssImages*/
const Value V22898_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22898_291 = {S13, 0, &V22898_291_57180, &P22898_289};	/*Property = CssMediaQueries*/
const Value V22898_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22898_292 = {S14, 0, &V22898_292_57183, &P22898_291};	/*Property = CssMinMax*/
const Value V22898_293_57186 = {S2, NULL};	/*Value = True*/
const Property P22898_293 = {S15, 0, &V22898_293_57186, &P22898_292};	/*Property = CssOverflow*/
const Value V22898_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22898_294 = {S16, 0, &V22898_294_57189, &P22898_293};	/*Property = CssPosition*/
const Value V22898_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22898_295 = {S17, 0, &V22898_295_57192, &P22898_294};	/*Property = CssText*/
const Value V22898_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22898_297 = {S18, 0, &V22898_297_57195, &P22898_295};	/*Property = CssTransforms*/
const Value V22898_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22898_241 = {S19, 0, &V22898_241_55799, &P22898_297};	/*Property = CssTransitions*/
const Value V22898_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22898_298 = {S20, 0, &V22898_298_57206, &P22898_241};	/*Property = CssUI*/
const Value V22898_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22898_300 = {S21, 0, &V22898_300_57216, &P22898_298};	/*Property = DataSet*/
const Value V22898_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22898_301 = {S22, 0, &V22898_301_57218, &P22898_300};	/*Property = DataUrl*/
const Value V22898_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22898_302 = {S23, 0, &V22898_302_57221, &P22898_301};	/*Property = DeviceOrientation*/
const Value V22898_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22898_304 = {S24, 0, &V22898_304_57227, &P22898_302};	/*Property = FileReader*/
const Value V22898_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22898_305 = {S25, 0, &V22898_305_57231, &P22898_304};	/*Property = FileSaver*/
const Value V22898_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22898_306 = {S26, 0, &V22898_306_57237, &P22898_305};	/*Property = FileWriter*/
const Value V22898_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22898_307 = {S27, 0, &V22898_307_57239, &P22898_306};	/*Property = FormData*/
const Value V22898_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22898_309 = {S28, 0, &V22898_309_57246, &P22898_307};	/*Property = Fullscreen*/
const Value V22898_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22898_310 = {S29, 0, &V22898_310_57248, &P22898_309};	/*Property = GeoLocation*/
const Value V22898_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22898_312 = {S30, 0, &V22898_312_57254, &P22898_310};	/*Property = History*/
const Value V22898_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22898_314 = {S31, 0, &V22898_314_57261, &P22898_312};	/*Property = Html5*/
const Value V22898_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22898_313 = {S32, 0, &V22898_313_57258, &P22898_314};	/*Property = Html-Media-Capture*/
const Value V22898_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22898_315 = {S33, 0, &V22898_315_57264, &P22898_313};	/*Property = Iframe*/
const Value V22898_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22898_316 = {S34, 0, &V22898_316_57267, &P22898_315};	/*Property = IndexedDB*/
const Value V22898_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22898_317 = {S35, 0, &V22898_317_57269, &P22898_316};	/*Property = Json*/
const Value V22898_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22898_15 = {S37, 0, &V22898_15_28, &P22898_317};	/*Property = LayoutEngine*/
const Value V22898_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22898_344 = {S38, 0, &V22898_344_57272, &P22898_15};	/*Property = Masking*/
const Value V22898_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22898_320 = {S39, 0, &V22898_320_57290, &P22898_344};	/*Property = PostMessage*/
const Value V22898_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22898_321 = {S40, 0, &V22898_321_57293, &P22898_320};	/*Property = Progress*/
const Value V22898_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22898_322 = {S41, 0, &V22898_322_57296, &P22898_321};	/*Property = Prompts*/
const Value V22898_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22898_323 = {S42, 0, &V22898_323_57299, &P22898_322};	/*Property = Selector*/
const Value V22898_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22898_324 = {S43, 0, &V22898_324_57306, &P22898_323};	/*Property = Svg*/
const Value V22898_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22898_327 = {S44, 0, &V22898_327_57314, &P22898_324};	/*Property = TouchEvents*/
const Value V22898_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22898_328 = {S45, 0, &V22898_328_57318, &P22898_327};	/*Property = Track*/
const Value V22898_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22898_329 = {S46, 0, &V22898_329_57323, &P22898_328};	/*Property = Video*/
const Value V22898_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22898_330 = {S47, 0, &V22898_330_57326, &P22898_329};	/*Property = Viewport*/
const Value V22898_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22898_332 = {S48, 0, &V22898_332_57338, &P22898_330};	/*Property = WebWorkers*/
const Value V22898_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22898_333 = {S49, 0, &V22898_333_57342, &P22898_332};	/*Property = Xhr2*/
const Profile P22898 = {"22898", &P22898_333};	/*Profile ID - 22898*/
const Value V21575_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_279 = {S1, 0, &V21575_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21575_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_282 = {S3, 0, &V21575_282_57155, &P21575_279};	/*Property = BlobBuilder*/
const Value V21575_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_350 = {S4, 0, &V21575_350_57446, &P21575_282};	/*Property = Canvas*/
const Value V21575_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_283 = {S5, 0, &V21575_283_57161, &P21575_350};	/*Property = CssBackground*/
const Value V21575_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_284 = {S6, 0, &V21575_284_57164, &P21575_283};	/*Property = CssBorderImage*/
const Value V21575_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_285 = {S7, 0, &V21575_285_57167, &P21575_284};	/*Property = CssCanvas*/
const Value V21575_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_286 = {S8, 0, &V21575_286_57170, &P21575_285};	/*Property = CssColor*/
const Value V21575_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_237 = {S9, 0, &V21575_237_55786, &P21575_286};	/*Property = CssColumn*/
const Value V21575_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_287 = {S10, 0, &V21575_287_57173, &P21575_237};	/*Property = CssFlexbox*/
const Value V21575_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_288 = {S11, 0, &V21575_288_57176, &P21575_287};	/*Property = CssFont*/
const Value V21575_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_289 = {S12, 0, &V21575_289_86899, &P21575_288};	/*Property = CssImages*/
const Value V21575_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_291 = {S13, 0, &V21575_291_57182, &P21575_289};	/*Property = CssMediaQueries*/
const Value V21575_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_292 = {S14, 0, &V21575_292_57185, &P21575_291};	/*Property = CssMinMax*/
const Value V21575_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_293 = {S15, 0, &V21575_293_57188, &P21575_292};	/*Property = CssOverflow*/
const Value V21575_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_294 = {S16, 0, &V21575_294_57191, &P21575_293};	/*Property = CssPosition*/
const Value V21575_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_295 = {S17, 0, &V21575_295_57194, &P21575_294};	/*Property = CssText*/
const Value V21575_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_297 = {S18, 0, &V21575_297_57197, &P21575_295};	/*Property = CssTransforms*/
const Value V21575_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_241 = {S19, 0, &V21575_241_55801, &P21575_297};	/*Property = CssTransitions*/
const Value V21575_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_298 = {S20, 0, &V21575_298_57208, &P21575_241};	/*Property = CssUI*/
const Value V21575_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_300 = {S21, 0, &V21575_300_57217, &P21575_298};	/*Property = DataSet*/
const Value V21575_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_301 = {S22, 0, &V21575_301_57220, &P21575_300};	/*Property = DataUrl*/
const Value V21575_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_302 = {S23, 0, &V21575_302_57223, &P21575_301};	/*Property = DeviceOrientation*/
const Value V21575_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_304 = {S24, 0, &V21575_304_57229, &P21575_302};	/*Property = FileReader*/
const Value V21575_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_305 = {S25, 0, &V21575_305_57232, &P21575_304};	/*Property = FileSaver*/
const Value V21575_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_306 = {S26, 0, &V21575_306_57238, &P21575_305};	/*Property = FileWriter*/
const Value V21575_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_307 = {S27, 0, &V21575_307_57241, &P21575_306};	/*Property = FormData*/
const Value V21575_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_309 = {S28, 0, &V21575_309_57247, &P21575_307};	/*Property = Fullscreen*/
const Value V21575_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_310 = {S29, 0, &V21575_310_57250, &P21575_309};	/*Property = GeoLocation*/
const Value V21575_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_312 = {S30, 0, &V21575_312_57256, &P21575_310};	/*Property = History*/
const Value V21575_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_314 = {S31, 0, &V21575_314_57262, &P21575_312};	/*Property = Html5*/
const Value V21575_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_313 = {S32, 0, &V21575_313_57259, &P21575_314};	/*Property = Html-Media-Capture*/
const Value V21575_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_315 = {S33, 0, &V21575_315_57265, &P21575_313};	/*Property = Iframe*/
const Value V21575_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_316 = {S34, 0, &V21575_316_57268, &P21575_315};	/*Property = IndexedDB*/
const Value V21575_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_317 = {S35, 0, &V21575_317_57271, &P21575_316};	/*Property = Json*/
const Value V21575_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21575_15 = {S37, 0, &V21575_15_28, &P21575_317};	/*Property = LayoutEngine*/
const Value V21575_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_344 = {S38, 0, &V21575_344_57274, &P21575_15};	/*Property = Masking*/
const Value V21575_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_320 = {S39, 0, &V21575_320_57292, &P21575_344};	/*Property = PostMessage*/
const Value V21575_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_321 = {S40, 0, &V21575_321_57295, &P21575_320};	/*Property = Progress*/
const Value V21575_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_322 = {S41, 0, &V21575_322_57298, &P21575_321};	/*Property = Prompts*/
const Value V21575_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_323 = {S42, 0, &V21575_323_57301, &P21575_322};	/*Property = Selector*/
const Value V21575_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_324 = {S43, 0, &V21575_324_57307, &P21575_323};	/*Property = Svg*/
const Value V21575_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_327 = {S44, 0, &V21575_327_57316, &P21575_324};	/*Property = TouchEvents*/
const Value V21575_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_328 = {S45, 0, &V21575_328_57319, &P21575_327};	/*Property = Track*/
const Value V21575_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_329 = {S46, 0, &V21575_329_57325, &P21575_328};	/*Property = Video*/
const Value V21575_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_330 = {S47, 0, &V21575_330_57328, &P21575_329};	/*Property = Viewport*/
const Value V21575_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_332 = {S48, 0, &V21575_332_57340, &P21575_330};	/*Property = WebWorkers*/
const Value V21575_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21575_333 = {S49, 0, &V21575_333_57343, &P21575_332};	/*Property = Xhr2*/
const Profile P21575 = {"21575", &P21575_333};	/*Profile ID - 21575*/
const Value V19857_1_1 = {S2, NULL};	/*Value = True*/
const Property P19857_1 = {S50, 0, &V19857_1_1, NULL};	/*Property = IsMobile*/
const Value V19857_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P19857_5 = {S52, 0, &V19857_5_1222, &P19857_1};	/*Property = ScreenPixelsHeight*/
const Value V19857_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P19857_4 = {S54, 0, &V19857_4_1224, &P19857_5};	/*Property = ScreenPixelsWidth*/
const Profile P19857 = {"19857", &P19857_4};	/*Profile ID - 19857*/
const Profile P19830 = {"19830", NULL};	/*Profile ID - 19830*/
const Value V18666_1_1 = {S2, NULL};	/*Value = True*/
const Property P18666_1 = {S50, 0, &V18666_1_1, NULL};	/*Property = IsMobile*/
const Value V18666_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18666_5 = {S52, 0, &V18666_5_1222, &P18666_1};	/*Property = ScreenPixelsHeight*/
const Value V18666_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18666_4 = {S54, 0, &V18666_4_1224, &P18666_5};	/*Property = ScreenPixelsWidth*/
const Profile P18666 = {"18666", &P18666_4};	/*Profile ID - 18666*/
const Value V23479_279_57147 = {S2, NULL};	/*Value = True*/
const Property P23479_279 = {S1, 0, &V23479_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V23479_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23479_282 = {S3, 0, &V23479_282_57154, &P23479_279};	/*Property = BlobBuilder*/
const Value V23479_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23479_350 = {S4, 0, &V23479_350_57444, &P23479_282};	/*Property = Canvas*/
const Value V23479_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23479_283 = {S5, 0, &V23479_283_57159, &P23479_350};	/*Property = CssBackground*/
const Value V23479_284_57162 = {S2, NULL};	/*Value = True*/
const Property P23479_284 = {S6, 0, &V23479_284_57162, &P23479_283};	/*Property = CssBorderImage*/
const Value V23479_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23479_285 = {S7, 0, &V23479_285_57166, &P23479_284};	/*Property = CssCanvas*/
const Value V23479_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23479_286 = {S8, 0, &V23479_286_57168, &P23479_285};	/*Property = CssColor*/
const Value V23479_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23479_237 = {S9, 0, &V23479_237_55786, &P23479_286};	/*Property = CssColumn*/
const Value V23479_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23479_287 = {S10, 0, &V23479_287_57172, &P23479_237};	/*Property = CssFlexbox*/
const Value V23479_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23479_288 = {S11, 0, &V23479_288_57174, &P23479_287};	/*Property = CssFont*/
const Value V23479_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23479_289 = {S12, 0, &V23479_289_57759, &P23479_288};	/*Property = CssImages*/
const Value V23479_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23479_291 = {S13, 0, &V23479_291_57180, &P23479_289};	/*Property = CssMediaQueries*/
const Value V23479_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23479_292 = {S14, 0, &V23479_292_57183, &P23479_291};	/*Property = CssMinMax*/
const Value V23479_293_57187 = {S0, NULL};	/*Value = False*/
const Property P23479_293 = {S15, 0, &V23479_293_57187, &P23479_292};	/*Property = CssOverflow*/
const Value V23479_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23479_294 = {S16, 0, &V23479_294_57189, &P23479_293};	/*Property = CssPosition*/
const Value V23479_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23479_295 = {S17, 0, &V23479_295_57192, &P23479_294};	/*Property = CssText*/
const Value V23479_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23479_297 = {S18, 0, &V23479_297_57195, &P23479_295};	/*Property = CssTransforms*/
const Value V23479_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23479_241 = {S19, 0, &V23479_241_55799, &P23479_297};	/*Property = CssTransitions*/
const Value V23479_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23479_298 = {S20, 0, &V23479_298_57206, &P23479_241};	/*Property = CssUI*/
const Value V23479_300_57215 = {S2, NULL};	/*Value = True*/
const Property P23479_300 = {S21, 0, &V23479_300_57215, &P23479_298};	/*Property = DataSet*/
const Value V23479_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23479_301 = {S22, 0, &V23479_301_57218, &P23479_300};	/*Property = DataUrl*/
const Value V23479_302_57221 = {S2, NULL};	/*Value = True*/
const Property P23479_302 = {S23, 0, &V23479_302_57221, &P23479_301};	/*Property = DeviceOrientation*/
const Value V23479_304_57227 = {S2, NULL};	/*Value = True*/
const Property P23479_304 = {S24, 0, &V23479_304_57227, &P23479_302};	/*Property = FileReader*/
const Value V23479_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23479_305 = {S25, 0, &V23479_305_57231, &P23479_304};	/*Property = FileSaver*/
const Value V23479_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23479_306 = {S26, 0, &V23479_306_57237, &P23479_305};	/*Property = FileWriter*/
const Value V23479_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23479_307 = {S27, 0, &V23479_307_57239, &P23479_306};	/*Property = FormData*/
const Value V23479_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23479_309 = {S28, 0, &V23479_309_57246, &P23479_307};	/*Property = Fullscreen*/
const Value V23479_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23479_310 = {S29, 0, &V23479_310_57248, &P23479_309};	/*Property = GeoLocation*/
const Value V23479_312_57254 = {S2, NULL};	/*Value = True*/
const Property P23479_312 = {S30, 0, &V23479_312_57254, &P23479_310};	/*Property = History*/
const Value V23479_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23479_314 = {S31, 0, &V23479_314_57261, &P23479_312};	/*Property = Html5*/
const Value V23479_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23479_313 = {S32, 0, &V23479_313_57258, &P23479_314};	/*Property = Html-Media-Capture*/
const Value V23479_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23479_315 = {S33, 0, &V23479_315_57264, &P23479_313};	/*Property = Iframe*/
const Value V23479_316_57266 = {S2, NULL};	/*Value = True*/
const Property P23479_316 = {S34, 0, &V23479_316_57266, &P23479_315};	/*Property = IndexedDB*/
const Value V23479_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23479_317 = {S35, 0, &V23479_317_57269, &P23479_316};	/*Property = Json*/
const Value V23479_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P23479_15 = {S37, 0, &V23479_15_29, &P23479_317};	/*Property = LayoutEngine*/
const Value V23479_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23479_344 = {S38, 0, &V23479_344_57272, &P23479_15};	/*Property = Masking*/
const Value V23479_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23479_320 = {S39, 0, &V23479_320_57290, &P23479_344};	/*Property = PostMessage*/
const Value V23479_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23479_321 = {S40, 0, &V23479_321_57293, &P23479_320};	/*Property = Progress*/
const Value V23479_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23479_322 = {S41, 0, &V23479_322_57296, &P23479_321};	/*Property = Prompts*/
const Value V23479_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23479_323 = {S42, 0, &V23479_323_57299, &P23479_322};	/*Property = Selector*/
const Value V23479_324_57305 = {S2, NULL};	/*Value = True*/
const Property P23479_324 = {S43, 0, &V23479_324_57305, &P23479_323};	/*Property = Svg*/
const Value V23479_327_57314 = {S2, NULL};	/*Value = True*/
const Property P23479_327 = {S44, 0, &V23479_327_57314, &P23479_324};	/*Property = TouchEvents*/
const Value V23479_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23479_328 = {S45, 0, &V23479_328_57318, &P23479_327};	/*Property = Track*/
const Value V23479_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23479_329 = {S46, 0, &V23479_329_57323, &P23479_328};	/*Property = Video*/
const Value V23479_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23479_330 = {S47, 0, &V23479_330_57326, &P23479_329};	/*Property = Viewport*/
const Value V23479_332_57338 = {S2, NULL};	/*Value = True*/
const Property P23479_332 = {S48, 0, &V23479_332_57338, &P23479_330};	/*Property = WebWorkers*/
const Value V23479_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23479_333 = {S49, 0, &V23479_333_57342, &P23479_332};	/*Property = Xhr2*/
const Profile P23479 = {"23479", &P23479_333};	/*Profile ID - 23479*/
const Value V20389_1_2 = {S0, NULL};	/*Value = False*/
const Property P20389_1 = {S50, 0, &V20389_1_2, NULL};	/*Property = IsMobile*/
const Value V20389_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20389_5 = {S52, 0, &V20389_5_37, &P20389_1};	/*Property = ScreenPixelsHeight*/
const Value V20389_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20389_4 = {S54, 0, &V20389_4_36, &P20389_5};	/*Property = ScreenPixelsWidth*/
const Profile P20389 = {"20389", &P20389_4};	/*Profile ID - 20389*/
const Value V19836_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19836_279 = {S1, 0, &V19836_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19836_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19836_282 = {S3, 0, &V19836_282_57154, &P19836_279};	/*Property = BlobBuilder*/
const Value V19836_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19836_350 = {S4, 0, &V19836_350_57444, &P19836_282};	/*Property = Canvas*/
const Value V19836_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19836_283 = {S5, 0, &V19836_283_57159, &P19836_350};	/*Property = CssBackground*/
const Value V19836_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19836_284 = {S6, 0, &V19836_284_57163, &P19836_283};	/*Property = CssBorderImage*/
const Value V19836_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19836_285 = {S7, 0, &V19836_285_57166, &P19836_284};	/*Property = CssCanvas*/
const Value V19836_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19836_286 = {S8, 0, &V19836_286_57168, &P19836_285};	/*Property = CssColor*/
const Value V19836_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19836_237 = {S9, 0, &V19836_237_55786, &P19836_286};	/*Property = CssColumn*/
const Value V19836_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19836_287 = {S10, 0, &V19836_287_57172, &P19836_237};	/*Property = CssFlexbox*/
const Value V19836_288_57175 = {S0, NULL};	/*Value = False*/
const Property P19836_288 = {S11, 0, &V19836_288_57175, &P19836_287};	/*Property = CssFont*/
const Value V19836_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19836_289 = {S12, 0, &V19836_289_57759, &P19836_288};	/*Property = CssImages*/
const Value V19836_291_57181 = {S0, NULL};	/*Value = False*/
const Property P19836_291 = {S13, 0, &V19836_291_57181, &P19836_289};	/*Property = CssMediaQueries*/
const Value V19836_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19836_292 = {S14, 0, &V19836_292_57183, &P19836_291};	/*Property = CssMinMax*/
const Value V19836_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19836_293 = {S15, 0, &V19836_293_57187, &P19836_292};	/*Property = CssOverflow*/
const Value V19836_294_57190 = {S0, NULL};	/*Value = False*/
const Property P19836_294 = {S16, 0, &V19836_294_57190, &P19836_293};	/*Property = CssPosition*/
const Value V19836_295_57193 = {S0, NULL};	/*Value = False*/
const Property P19836_295 = {S17, 0, &V19836_295_57193, &P19836_294};	/*Property = CssText*/
const Value V19836_297_57196 = {S0, NULL};	/*Value = False*/
const Property P19836_297 = {S18, 0, &V19836_297_57196, &P19836_295};	/*Property = CssTransforms*/
const Value V19836_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19836_241 = {S19, 0, &V19836_241_55799, &P19836_297};	/*Property = CssTransitions*/
const Value V19836_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19836_298 = {S20, 0, &V19836_298_57206, &P19836_241};	/*Property = CssUI*/
const Value V19836_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19836_300 = {S21, 0, &V19836_300_57216, &P19836_298};	/*Property = DataSet*/
const Value V19836_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19836_301 = {S22, 0, &V19836_301_57218, &P19836_300};	/*Property = DataUrl*/
const Value V19836_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19836_302 = {S23, 0, &V19836_302_57222, &P19836_301};	/*Property = DeviceOrientation*/
const Value V19836_304_57228 = {S0, NULL};	/*Value = False*/
const Property P19836_304 = {S24, 0, &V19836_304_57228, &P19836_302};	/*Property = FileReader*/
const Value V19836_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19836_305 = {S25, 0, &V19836_305_57231, &P19836_304};	/*Property = FileSaver*/
const Value V19836_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19836_306 = {S26, 0, &V19836_306_57237, &P19836_305};	/*Property = FileWriter*/
const Value V19836_307_57240 = {S0, NULL};	/*Value = False*/
const Property P19836_307 = {S27, 0, &V19836_307_57240, &P19836_306};	/*Property = FormData*/
const Value V19836_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19836_309 = {S28, 0, &V19836_309_57246, &P19836_307};	/*Property = Fullscreen*/
const Value V19836_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19836_310 = {S29, 0, &V19836_310_57248, &P19836_309};	/*Property = GeoLocation*/
const Value V19836_312_57255 = {S0, NULL};	/*Value = False*/
const Property P19836_312 = {S30, 0, &V19836_312_57255, &P19836_310};	/*Property = History*/
const Value V19836_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19836_314 = {S31, 0, &V19836_314_57261, &P19836_312};	/*Property = Html5*/
const Value V19836_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19836_313 = {S32, 0, &V19836_313_57258, &P19836_314};	/*Property = Html-Media-Capture*/
const Value V19836_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19836_315 = {S33, 0, &V19836_315_57264, &P19836_313};	/*Property = Iframe*/
const Value V19836_316_57267 = {S0, NULL};	/*Value = False*/
const Property P19836_316 = {S34, 0, &V19836_316_57267, &P19836_315};	/*Property = IndexedDB*/
const Value V19836_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19836_317 = {S35, 0, &V19836_317_57269, &P19836_316};	/*Property = Json*/
const Value V19836_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P19836_15 = {S37, 0, &V19836_15_31, &P19836_317};	/*Property = LayoutEngine*/
const Value V19836_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19836_344 = {S38, 0, &V19836_344_57272, &P19836_15};	/*Property = Masking*/
const Value V19836_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19836_320 = {S39, 0, &V19836_320_57290, &P19836_344};	/*Property = PostMessage*/
const Value V19836_321_57294 = {S0, NULL};	/*Value = False*/
const Property P19836_321 = {S40, 0, &V19836_321_57294, &P19836_320};	/*Property = Progress*/
const Value V19836_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19836_322 = {S41, 0, &V19836_322_57296, &P19836_321};	/*Property = Prompts*/
const Value V19836_323_57300 = {S0, NULL};	/*Value = False*/
const Property P19836_323 = {S42, 0, &V19836_323_57300, &P19836_322};	/*Property = Selector*/
const Value V19836_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19836_324 = {S43, 0, &V19836_324_57306, &P19836_323};	/*Property = Svg*/
const Value V19836_327_57315 = {S0, NULL};	/*Value = False*/
const Property P19836_327 = {S44, 0, &V19836_327_57315, &P19836_324};	/*Property = TouchEvents*/
const Value V19836_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19836_328 = {S45, 0, &V19836_328_57318, &P19836_327};	/*Property = Track*/
const Value V19836_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19836_329 = {S46, 0, &V19836_329_57323, &P19836_328};	/*Property = Video*/
const Value V19836_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19836_330 = {S47, 0, &V19836_330_57326, &P19836_329};	/*Property = Viewport*/
const Value V19836_332_57339 = {S0, NULL};	/*Value = False*/
const Property P19836_332 = {S48, 0, &V19836_332_57339, &P19836_330};	/*Property = WebWorkers*/
const Value V19836_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19836_333 = {S49, 0, &V19836_333_57342, &P19836_332};	/*Property = Xhr2*/
const Profile P19836 = {"19836", &P19836_333};	/*Profile ID - 19836*/
const Value V22462_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22462_279 = {S1, 0, &V22462_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22462_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22462_282 = {S3, 0, &V22462_282_57154, &P22462_279};	/*Property = BlobBuilder*/
const Value V22462_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22462_350 = {S4, 0, &V22462_350_57444, &P22462_282};	/*Property = Canvas*/
const Value V22462_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22462_283 = {S5, 0, &V22462_283_57159, &P22462_350};	/*Property = CssBackground*/
const Value V22462_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22462_284 = {S6, 0, &V22462_284_57163, &P22462_283};	/*Property = CssBorderImage*/
const Value V22462_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22462_285 = {S7, 0, &V22462_285_57166, &P22462_284};	/*Property = CssCanvas*/
const Value V22462_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22462_286 = {S8, 0, &V22462_286_57168, &P22462_285};	/*Property = CssColor*/
const Value V22462_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22462_237 = {S9, 0, &V22462_237_55786, &P22462_286};	/*Property = CssColumn*/
const Value V22462_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22462_287 = {S10, 0, &V22462_287_57172, &P22462_237};	/*Property = CssFlexbox*/
const Value V22462_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22462_288 = {S11, 0, &V22462_288_57174, &P22462_287};	/*Property = CssFont*/
const Value V22462_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22462_289 = {S12, 0, &V22462_289_57759, &P22462_288};	/*Property = CssImages*/
const Value V22462_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22462_291 = {S13, 0, &V22462_291_57180, &P22462_289};	/*Property = CssMediaQueries*/
const Value V22462_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22462_292 = {S14, 0, &V22462_292_57183, &P22462_291};	/*Property = CssMinMax*/
const Value V22462_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22462_293 = {S15, 0, &V22462_293_57187, &P22462_292};	/*Property = CssOverflow*/
const Value V22462_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22462_294 = {S16, 0, &V22462_294_57189, &P22462_293};	/*Property = CssPosition*/
const Value V22462_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22462_295 = {S17, 0, &V22462_295_57192, &P22462_294};	/*Property = CssText*/
const Value V22462_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22462_297 = {S18, 0, &V22462_297_57195, &P22462_295};	/*Property = CssTransforms*/
const Value V22462_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22462_241 = {S19, 0, &V22462_241_55799, &P22462_297};	/*Property = CssTransitions*/
const Value V22462_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22462_298 = {S20, 0, &V22462_298_57206, &P22462_241};	/*Property = CssUI*/
const Value V22462_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22462_300 = {S21, 0, &V22462_300_57216, &P22462_298};	/*Property = DataSet*/
const Value V22462_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22462_301 = {S22, 0, &V22462_301_57218, &P22462_300};	/*Property = DataUrl*/
const Value V22462_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22462_302 = {S23, 0, &V22462_302_57222, &P22462_301};	/*Property = DeviceOrientation*/
const Value V22462_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22462_304 = {S24, 0, &V22462_304_57227, &P22462_302};	/*Property = FileReader*/
const Value V22462_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22462_305 = {S25, 0, &V22462_305_57231, &P22462_304};	/*Property = FileSaver*/
const Value V22462_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22462_306 = {S26, 0, &V22462_306_57237, &P22462_305};	/*Property = FileWriter*/
const Value V22462_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22462_307 = {S27, 0, &V22462_307_57239, &P22462_306};	/*Property = FormData*/
const Value V22462_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22462_309 = {S28, 0, &V22462_309_57246, &P22462_307};	/*Property = Fullscreen*/
const Value V22462_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22462_310 = {S29, 0, &V22462_310_57248, &P22462_309};	/*Property = GeoLocation*/
const Value V22462_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22462_312 = {S30, 0, &V22462_312_57254, &P22462_310};	/*Property = History*/
const Value V22462_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22462_314 = {S31, 0, &V22462_314_57261, &P22462_312};	/*Property = Html5*/
const Value V22462_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22462_313 = {S32, 0, &V22462_313_57258, &P22462_314};	/*Property = Html-Media-Capture*/
const Value V22462_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22462_315 = {S33, 0, &V22462_315_57264, &P22462_313};	/*Property = Iframe*/
const Value V22462_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22462_316 = {S34, 0, &V22462_316_57266, &P22462_315};	/*Property = IndexedDB*/
const Value V22462_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22462_317 = {S35, 0, &V22462_317_57269, &P22462_316};	/*Property = Json*/
const Value V22462_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22462_15 = {S37, 0, &V22462_15_28, &P22462_317};	/*Property = LayoutEngine*/
const Value V22462_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22462_344 = {S38, 0, &V22462_344_57272, &P22462_15};	/*Property = Masking*/
const Value V22462_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22462_320 = {S39, 0, &V22462_320_57290, &P22462_344};	/*Property = PostMessage*/
const Value V22462_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22462_321 = {S40, 0, &V22462_321_57293, &P22462_320};	/*Property = Progress*/
const Value V22462_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22462_322 = {S41, 0, &V22462_322_57296, &P22462_321};	/*Property = Prompts*/
const Value V22462_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22462_323 = {S42, 0, &V22462_323_57299, &P22462_322};	/*Property = Selector*/
const Value V22462_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22462_324 = {S43, 0, &V22462_324_57306, &P22462_323};	/*Property = Svg*/
const Value V22462_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22462_327 = {S44, 0, &V22462_327_57315, &P22462_324};	/*Property = TouchEvents*/
const Value V22462_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22462_328 = {S45, 0, &V22462_328_57318, &P22462_327};	/*Property = Track*/
const Value V22462_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22462_329 = {S46, 0, &V22462_329_57323, &P22462_328};	/*Property = Video*/
const Value V22462_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22462_330 = {S47, 0, &V22462_330_57326, &P22462_329};	/*Property = Viewport*/
const Value V22462_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22462_332 = {S48, 0, &V22462_332_57338, &P22462_330};	/*Property = WebWorkers*/
const Value V22462_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22462_333 = {S49, 0, &V22462_333_57341, &P22462_332};	/*Property = Xhr2*/
const Profile P22462 = {"22462", &P22462_333};	/*Profile ID - 22462*/
const Value V23766_1_1 = {S2, NULL};	/*Value = True*/
const Property P23766_1 = {S50, 0, &V23766_1_1, NULL};	/*Property = IsMobile*/
const Value V23766_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23766_5 = {S52, 0, &V23766_5_1277, &P23766_1};	/*Property = ScreenPixelsHeight*/
const Value V23766_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23766_4 = {S54, 0, &V23766_4_2336, &P23766_5};	/*Property = ScreenPixelsWidth*/
const Profile P23766 = {"23766", &P23766_4};	/*Profile ID - 23766*/
const Value V22071_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_279 = {S1, 0, &V22071_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22071_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_282 = {S3, 0, &V22071_282_57155, &P22071_279};	/*Property = BlobBuilder*/
const Value V22071_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_350 = {S4, 0, &V22071_350_57446, &P22071_282};	/*Property = Canvas*/
const Value V22071_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_283 = {S5, 0, &V22071_283_57161, &P22071_350};	/*Property = CssBackground*/
const Value V22071_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_284 = {S6, 0, &V22071_284_57164, &P22071_283};	/*Property = CssBorderImage*/
const Value V22071_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_285 = {S7, 0, &V22071_285_57167, &P22071_284};	/*Property = CssCanvas*/
const Value V22071_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_286 = {S8, 0, &V22071_286_57170, &P22071_285};	/*Property = CssColor*/
const Value V22071_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_237 = {S9, 0, &V22071_237_55786, &P22071_286};	/*Property = CssColumn*/
const Value V22071_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_287 = {S10, 0, &V22071_287_57173, &P22071_237};	/*Property = CssFlexbox*/
const Value V22071_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_288 = {S11, 0, &V22071_288_57176, &P22071_287};	/*Property = CssFont*/
const Value V22071_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_289 = {S12, 0, &V22071_289_86899, &P22071_288};	/*Property = CssImages*/
const Value V22071_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_291 = {S13, 0, &V22071_291_57182, &P22071_289};	/*Property = CssMediaQueries*/
const Value V22071_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_292 = {S14, 0, &V22071_292_57185, &P22071_291};	/*Property = CssMinMax*/
const Value V22071_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_293 = {S15, 0, &V22071_293_57188, &P22071_292};	/*Property = CssOverflow*/
const Value V22071_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_294 = {S16, 0, &V22071_294_57191, &P22071_293};	/*Property = CssPosition*/
const Value V22071_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_295 = {S17, 0, &V22071_295_57194, &P22071_294};	/*Property = CssText*/
const Value V22071_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_297 = {S18, 0, &V22071_297_57197, &P22071_295};	/*Property = CssTransforms*/
const Value V22071_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_241 = {S19, 0, &V22071_241_55801, &P22071_297};	/*Property = CssTransitions*/
const Value V22071_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_298 = {S20, 0, &V22071_298_57208, &P22071_241};	/*Property = CssUI*/
const Value V22071_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_300 = {S21, 0, &V22071_300_57217, &P22071_298};	/*Property = DataSet*/
const Value V22071_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_301 = {S22, 0, &V22071_301_57220, &P22071_300};	/*Property = DataUrl*/
const Value V22071_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_302 = {S23, 0, &V22071_302_57223, &P22071_301};	/*Property = DeviceOrientation*/
const Value V22071_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_304 = {S24, 0, &V22071_304_57229, &P22071_302};	/*Property = FileReader*/
const Value V22071_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_305 = {S25, 0, &V22071_305_57232, &P22071_304};	/*Property = FileSaver*/
const Value V22071_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_306 = {S26, 0, &V22071_306_57238, &P22071_305};	/*Property = FileWriter*/
const Value V22071_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_307 = {S27, 0, &V22071_307_57241, &P22071_306};	/*Property = FormData*/
const Value V22071_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_309 = {S28, 0, &V22071_309_57247, &P22071_307};	/*Property = Fullscreen*/
const Value V22071_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_310 = {S29, 0, &V22071_310_57250, &P22071_309};	/*Property = GeoLocation*/
const Value V22071_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_312 = {S30, 0, &V22071_312_57256, &P22071_310};	/*Property = History*/
const Value V22071_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_314 = {S31, 0, &V22071_314_57262, &P22071_312};	/*Property = Html5*/
const Value V22071_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_313 = {S32, 0, &V22071_313_57259, &P22071_314};	/*Property = Html-Media-Capture*/
const Value V22071_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_315 = {S33, 0, &V22071_315_57265, &P22071_313};	/*Property = Iframe*/
const Value V22071_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_316 = {S34, 0, &V22071_316_57268, &P22071_315};	/*Property = IndexedDB*/
const Value V22071_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_317 = {S35, 0, &V22071_317_57271, &P22071_316};	/*Property = Json*/
const Value V22071_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_15 = {S37, 0, &V22071_15_72, &P22071_317};	/*Property = LayoutEngine*/
const Value V22071_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_344 = {S38, 0, &V22071_344_57274, &P22071_15};	/*Property = Masking*/
const Value V22071_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_320 = {S39, 0, &V22071_320_57292, &P22071_344};	/*Property = PostMessage*/
const Value V22071_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_321 = {S40, 0, &V22071_321_57295, &P22071_320};	/*Property = Progress*/
const Value V22071_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_322 = {S41, 0, &V22071_322_57298, &P22071_321};	/*Property = Prompts*/
const Value V22071_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_323 = {S42, 0, &V22071_323_57301, &P22071_322};	/*Property = Selector*/
const Value V22071_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_324 = {S43, 0, &V22071_324_57307, &P22071_323};	/*Property = Svg*/
const Value V22071_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_327 = {S44, 0, &V22071_327_57316, &P22071_324};	/*Property = TouchEvents*/
const Value V22071_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_328 = {S45, 0, &V22071_328_57319, &P22071_327};	/*Property = Track*/
const Value V22071_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_329 = {S46, 0, &V22071_329_57325, &P22071_328};	/*Property = Video*/
const Value V22071_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_330 = {S47, 0, &V22071_330_57328, &P22071_329};	/*Property = Viewport*/
const Value V22071_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_332 = {S48, 0, &V22071_332_57340, &P22071_330};	/*Property = WebWorkers*/
const Value V22071_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22071_333 = {S49, 0, &V22071_333_57343, &P22071_332};	/*Property = Xhr2*/
const Profile P22071 = {"22071", &P22071_333};	/*Profile ID - 22071*/
const Value V18614_1_1 = {S2, NULL};	/*Value = True*/
const Property P18614_1 = {S50, 0, &V18614_1_1, NULL};	/*Property = IsMobile*/
const Value V18614_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18614_5 = {S52, 0, &V18614_5_1403, &P18614_1};	/*Property = ScreenPixelsHeight*/
const Value V18614_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18614_4 = {S54, 0, &V18614_4_1404, &P18614_5};	/*Property = ScreenPixelsWidth*/
const Profile P18614 = {"18614", &P18614_4};	/*Profile ID - 18614*/
const Value V18642_1_1 = {S2, NULL};	/*Value = True*/
const Property P18642_1 = {S50, 0, &V18642_1_1, NULL};	/*Property = IsMobile*/
const Value V18642_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18642_5 = {S52, 0, &V18642_5_1277, &P18642_1};	/*Property = ScreenPixelsHeight*/
const Value V18642_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18642_4 = {S54, 0, &V18642_4_1279, &P18642_5};	/*Property = ScreenPixelsWidth*/
const Profile P18642 = {"18642", &P18642_4};	/*Profile ID - 18642*/
const Value V21075_1_1 = {S2, NULL};	/*Value = True*/
const Property P21075_1 = {S50, 0, &V21075_1_1, NULL};	/*Property = IsMobile*/
const Value V21075_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21075_5 = {S52, 0, &V21075_5_1222, &P21075_1};	/*Property = ScreenPixelsHeight*/
const Value V21075_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21075_4 = {S54, 0, &V21075_4_1224, &P21075_5};	/*Property = ScreenPixelsWidth*/
const Profile P21075 = {"21075", &P21075_4};	/*Profile ID - 21075*/
const Value V19922_1_1 = {S2, NULL};	/*Value = True*/
const Property P19922_1 = {S50, 0, &V19922_1_1, NULL};	/*Property = IsMobile*/
const Value V19922_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19922_5 = {S52, 0, &V19922_5_9981, &P19922_1};	/*Property = ScreenPixelsHeight*/
const Value V19922_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P19922_4 = {S54, 0, &V19922_4_39098, &P19922_5};	/*Property = ScreenPixelsWidth*/
const Profile P19922 = {"19922", &P19922_4};	/*Profile ID - 19922*/
const Value V22734_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22734_279 = {S1, 0, &V22734_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22734_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22734_282 = {S3, 0, &V22734_282_57154, &P22734_279};	/*Property = BlobBuilder*/
const Value V22734_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22734_350 = {S4, 0, &V22734_350_57444, &P22734_282};	/*Property = Canvas*/
const Value V22734_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22734_283 = {S5, 0, &V22734_283_57159, &P22734_350};	/*Property = CssBackground*/
const Value V22734_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22734_284 = {S6, 0, &V22734_284_57163, &P22734_283};	/*Property = CssBorderImage*/
const Value V22734_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22734_285 = {S7, 0, &V22734_285_57166, &P22734_284};	/*Property = CssCanvas*/
const Value V22734_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22734_286 = {S8, 0, &V22734_286_57168, &P22734_285};	/*Property = CssColor*/
const Value V22734_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22734_237 = {S9, 0, &V22734_237_55786, &P22734_286};	/*Property = CssColumn*/
const Value V22734_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22734_287 = {S10, 0, &V22734_287_57172, &P22734_237};	/*Property = CssFlexbox*/
const Value V22734_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22734_288 = {S11, 0, &V22734_288_57174, &P22734_287};	/*Property = CssFont*/
const Value V22734_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22734_289 = {S12, 0, &V22734_289_57759, &P22734_288};	/*Property = CssImages*/
const Value V22734_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22734_291 = {S13, 0, &V22734_291_57180, &P22734_289};	/*Property = CssMediaQueries*/
const Value V22734_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22734_292 = {S14, 0, &V22734_292_57183, &P22734_291};	/*Property = CssMinMax*/
const Value V22734_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22734_293 = {S15, 0, &V22734_293_57187, &P22734_292};	/*Property = CssOverflow*/
const Value V22734_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22734_294 = {S16, 0, &V22734_294_57189, &P22734_293};	/*Property = CssPosition*/
const Value V22734_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22734_295 = {S17, 0, &V22734_295_57192, &P22734_294};	/*Property = CssText*/
const Value V22734_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22734_297 = {S18, 0, &V22734_297_57195, &P22734_295};	/*Property = CssTransforms*/
const Value V22734_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22734_241 = {S19, 0, &V22734_241_55799, &P22734_297};	/*Property = CssTransitions*/
const Value V22734_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22734_298 = {S20, 0, &V22734_298_57206, &P22734_241};	/*Property = CssUI*/
const Value V22734_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22734_300 = {S21, 0, &V22734_300_57216, &P22734_298};	/*Property = DataSet*/
const Value V22734_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22734_301 = {S22, 0, &V22734_301_57218, &P22734_300};	/*Property = DataUrl*/
const Value V22734_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22734_302 = {S23, 0, &V22734_302_57221, &P22734_301};	/*Property = DeviceOrientation*/
const Value V22734_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22734_304 = {S24, 0, &V22734_304_57227, &P22734_302};	/*Property = FileReader*/
const Value V22734_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22734_305 = {S25, 0, &V22734_305_57231, &P22734_304};	/*Property = FileSaver*/
const Value V22734_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22734_306 = {S26, 0, &V22734_306_57237, &P22734_305};	/*Property = FileWriter*/
const Value V22734_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22734_307 = {S27, 0, &V22734_307_57239, &P22734_306};	/*Property = FormData*/
const Value V22734_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22734_309 = {S28, 0, &V22734_309_57246, &P22734_307};	/*Property = Fullscreen*/
const Value V22734_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22734_310 = {S29, 0, &V22734_310_57248, &P22734_309};	/*Property = GeoLocation*/
const Value V22734_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22734_312 = {S30, 0, &V22734_312_57254, &P22734_310};	/*Property = History*/
const Value V22734_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22734_314 = {S31, 0, &V22734_314_57261, &P22734_312};	/*Property = Html5*/
const Value V22734_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22734_313 = {S32, 0, &V22734_313_57258, &P22734_314};	/*Property = Html-Media-Capture*/
const Value V22734_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22734_315 = {S33, 0, &V22734_315_57264, &P22734_313};	/*Property = Iframe*/
const Value V22734_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22734_316 = {S34, 0, &V22734_316_57266, &P22734_315};	/*Property = IndexedDB*/
const Value V22734_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22734_317 = {S35, 0, &V22734_317_57269, &P22734_316};	/*Property = Json*/
const Value V22734_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22734_15 = {S37, 0, &V22734_15_29, &P22734_317};	/*Property = LayoutEngine*/
const Value V22734_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22734_344 = {S38, 0, &V22734_344_57272, &P22734_15};	/*Property = Masking*/
const Value V22734_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22734_320 = {S39, 0, &V22734_320_57290, &P22734_344};	/*Property = PostMessage*/
const Value V22734_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22734_321 = {S40, 0, &V22734_321_57293, &P22734_320};	/*Property = Progress*/
const Value V22734_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22734_322 = {S41, 0, &V22734_322_57296, &P22734_321};	/*Property = Prompts*/
const Value V22734_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22734_323 = {S42, 0, &V22734_323_57299, &P22734_322};	/*Property = Selector*/
const Value V22734_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22734_324 = {S43, 0, &V22734_324_57306, &P22734_323};	/*Property = Svg*/
const Value V22734_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22734_327 = {S44, 0, &V22734_327_57315, &P22734_324};	/*Property = TouchEvents*/
const Value V22734_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22734_328 = {S45, 0, &V22734_328_57318, &P22734_327};	/*Property = Track*/
const Value V22734_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22734_329 = {S46, 0, &V22734_329_57323, &P22734_328};	/*Property = Video*/
const Value V22734_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22734_330 = {S47, 0, &V22734_330_57326, &P22734_329};	/*Property = Viewport*/
const Value V22734_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22734_332 = {S48, 0, &V22734_332_57338, &P22734_330};	/*Property = WebWorkers*/
const Value V22734_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22734_333 = {S49, 0, &V22734_333_57341, &P22734_332};	/*Property = Xhr2*/
const Profile P22734 = {"22734", &P22734_333};	/*Profile ID - 22734*/
const Value V21574_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_279 = {S1, 0, &V21574_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21574_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_282 = {S3, 0, &V21574_282_57155, &P21574_279};	/*Property = BlobBuilder*/
const Value V21574_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_350 = {S4, 0, &V21574_350_57446, &P21574_282};	/*Property = Canvas*/
const Value V21574_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_283 = {S5, 0, &V21574_283_57161, &P21574_350};	/*Property = CssBackground*/
const Value V21574_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_284 = {S6, 0, &V21574_284_57164, &P21574_283};	/*Property = CssBorderImage*/
const Value V21574_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_285 = {S7, 0, &V21574_285_57167, &P21574_284};	/*Property = CssCanvas*/
const Value V21574_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_286 = {S8, 0, &V21574_286_57170, &P21574_285};	/*Property = CssColor*/
const Value V21574_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_237 = {S9, 0, &V21574_237_55786, &P21574_286};	/*Property = CssColumn*/
const Value V21574_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_287 = {S10, 0, &V21574_287_57173, &P21574_237};	/*Property = CssFlexbox*/
const Value V21574_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_288 = {S11, 0, &V21574_288_57176, &P21574_287};	/*Property = CssFont*/
const Value V21574_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_289 = {S12, 0, &V21574_289_86899, &P21574_288};	/*Property = CssImages*/
const Value V21574_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_291 = {S13, 0, &V21574_291_57182, &P21574_289};	/*Property = CssMediaQueries*/
const Value V21574_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_292 = {S14, 0, &V21574_292_57185, &P21574_291};	/*Property = CssMinMax*/
const Value V21574_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_293 = {S15, 0, &V21574_293_57188, &P21574_292};	/*Property = CssOverflow*/
const Value V21574_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_294 = {S16, 0, &V21574_294_57191, &P21574_293};	/*Property = CssPosition*/
const Value V21574_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_295 = {S17, 0, &V21574_295_57194, &P21574_294};	/*Property = CssText*/
const Value V21574_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_297 = {S18, 0, &V21574_297_57197, &P21574_295};	/*Property = CssTransforms*/
const Value V21574_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_241 = {S19, 0, &V21574_241_55801, &P21574_297};	/*Property = CssTransitions*/
const Value V21574_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_298 = {S20, 0, &V21574_298_57208, &P21574_241};	/*Property = CssUI*/
const Value V21574_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_300 = {S21, 0, &V21574_300_57217, &P21574_298};	/*Property = DataSet*/
const Value V21574_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_301 = {S22, 0, &V21574_301_57220, &P21574_300};	/*Property = DataUrl*/
const Value V21574_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_302 = {S23, 0, &V21574_302_57223, &P21574_301};	/*Property = DeviceOrientation*/
const Value V21574_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_304 = {S24, 0, &V21574_304_57229, &P21574_302};	/*Property = FileReader*/
const Value V21574_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_305 = {S25, 0, &V21574_305_57232, &P21574_304};	/*Property = FileSaver*/
const Value V21574_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_306 = {S26, 0, &V21574_306_57238, &P21574_305};	/*Property = FileWriter*/
const Value V21574_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_307 = {S27, 0, &V21574_307_57241, &P21574_306};	/*Property = FormData*/
const Value V21574_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_309 = {S28, 0, &V21574_309_57247, &P21574_307};	/*Property = Fullscreen*/
const Value V21574_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_310 = {S29, 0, &V21574_310_57250, &P21574_309};	/*Property = GeoLocation*/
const Value V21574_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_312 = {S30, 0, &V21574_312_57256, &P21574_310};	/*Property = History*/
const Value V21574_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_314 = {S31, 0, &V21574_314_57262, &P21574_312};	/*Property = Html5*/
const Value V21574_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_313 = {S32, 0, &V21574_313_57259, &P21574_314};	/*Property = Html-Media-Capture*/
const Value V21574_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_315 = {S33, 0, &V21574_315_57265, &P21574_313};	/*Property = Iframe*/
const Value V21574_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_316 = {S34, 0, &V21574_316_57268, &P21574_315};	/*Property = IndexedDB*/
const Value V21574_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_317 = {S35, 0, &V21574_317_57271, &P21574_316};	/*Property = Json*/
const Value V21574_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21574_15 = {S37, 0, &V21574_15_28, &P21574_317};	/*Property = LayoutEngine*/
const Value V21574_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_344 = {S38, 0, &V21574_344_57274, &P21574_15};	/*Property = Masking*/
const Value V21574_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_320 = {S39, 0, &V21574_320_57292, &P21574_344};	/*Property = PostMessage*/
const Value V21574_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_321 = {S40, 0, &V21574_321_57295, &P21574_320};	/*Property = Progress*/
const Value V21574_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_322 = {S41, 0, &V21574_322_57298, &P21574_321};	/*Property = Prompts*/
const Value V21574_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_323 = {S42, 0, &V21574_323_57301, &P21574_322};	/*Property = Selector*/
const Value V21574_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_324 = {S43, 0, &V21574_324_57307, &P21574_323};	/*Property = Svg*/
const Value V21574_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_327 = {S44, 0, &V21574_327_57316, &P21574_324};	/*Property = TouchEvents*/
const Value V21574_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_328 = {S45, 0, &V21574_328_57319, &P21574_327};	/*Property = Track*/
const Value V21574_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_329 = {S46, 0, &V21574_329_57325, &P21574_328};	/*Property = Video*/
const Value V21574_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_330 = {S47, 0, &V21574_330_57328, &P21574_329};	/*Property = Viewport*/
const Value V21574_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_332 = {S48, 0, &V21574_332_57340, &P21574_330};	/*Property = WebWorkers*/
const Value V21574_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21574_333 = {S49, 0, &V21574_333_57343, &P21574_332};	/*Property = Xhr2*/
const Profile P21574 = {"21574", &P21574_333};	/*Profile ID - 21574*/
const Value V22365_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22365_279 = {S1, 0, &V22365_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22365_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22365_282 = {S3, 0, &V22365_282_57153, &P22365_279};	/*Property = BlobBuilder*/
const Value V22365_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22365_350 = {S4, 0, &V22365_350_57444, &P22365_282};	/*Property = Canvas*/
const Value V22365_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22365_283 = {S5, 0, &V22365_283_57159, &P22365_350};	/*Property = CssBackground*/
const Value V22365_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22365_284 = {S6, 0, &V22365_284_57163, &P22365_283};	/*Property = CssBorderImage*/
const Value V22365_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22365_285 = {S7, 0, &V22365_285_57166, &P22365_284};	/*Property = CssCanvas*/
const Value V22365_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22365_286 = {S8, 0, &V22365_286_57168, &P22365_285};	/*Property = CssColor*/
const Value V22365_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22365_237 = {S9, 0, &V22365_237_55786, &P22365_286};	/*Property = CssColumn*/
const Value V22365_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22365_287 = {S10, 0, &V22365_287_57172, &P22365_237};	/*Property = CssFlexbox*/
const Value V22365_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22365_288 = {S11, 0, &V22365_288_57175, &P22365_287};	/*Property = CssFont*/
const Value V22365_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22365_289 = {S12, 0, &V22365_289_57759, &P22365_288};	/*Property = CssImages*/
const Value V22365_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22365_291 = {S13, 0, &V22365_291_57181, &P22365_289};	/*Property = CssMediaQueries*/
const Value V22365_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22365_292 = {S14, 0, &V22365_292_57183, &P22365_291};	/*Property = CssMinMax*/
const Value V22365_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22365_293 = {S15, 0, &V22365_293_57187, &P22365_292};	/*Property = CssOverflow*/
const Value V22365_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22365_294 = {S16, 0, &V22365_294_57189, &P22365_293};	/*Property = CssPosition*/
const Value V22365_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22365_295 = {S17, 0, &V22365_295_57192, &P22365_294};	/*Property = CssText*/
const Value V22365_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22365_297 = {S18, 0, &V22365_297_57195, &P22365_295};	/*Property = CssTransforms*/
const Value V22365_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22365_241 = {S19, 0, &V22365_241_55799, &P22365_297};	/*Property = CssTransitions*/
const Value V22365_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22365_298 = {S20, 0, &V22365_298_57206, &P22365_241};	/*Property = CssUI*/
const Value V22365_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22365_300 = {S21, 0, &V22365_300_57216, &P22365_298};	/*Property = DataSet*/
const Value V22365_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22365_301 = {S22, 0, &V22365_301_57218, &P22365_300};	/*Property = DataUrl*/
const Value V22365_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22365_302 = {S23, 0, &V22365_302_57221, &P22365_301};	/*Property = DeviceOrientation*/
const Value V22365_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22365_304 = {S24, 0, &V22365_304_57227, &P22365_302};	/*Property = FileReader*/
const Value V22365_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22365_305 = {S25, 0, &V22365_305_57231, &P22365_304};	/*Property = FileSaver*/
const Value V22365_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22365_306 = {S26, 0, &V22365_306_57237, &P22365_305};	/*Property = FileWriter*/
const Value V22365_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22365_307 = {S27, 0, &V22365_307_57239, &P22365_306};	/*Property = FormData*/
const Value V22365_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22365_309 = {S28, 0, &V22365_309_57246, &P22365_307};	/*Property = Fullscreen*/
const Value V22365_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22365_310 = {S29, 0, &V22365_310_57248, &P22365_309};	/*Property = GeoLocation*/
const Value V22365_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22365_312 = {S30, 0, &V22365_312_57254, &P22365_310};	/*Property = History*/
const Value V22365_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22365_314 = {S31, 0, &V22365_314_57261, &P22365_312};	/*Property = Html5*/
const Value V22365_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22365_313 = {S32, 0, &V22365_313_57258, &P22365_314};	/*Property = Html-Media-Capture*/
const Value V22365_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22365_315 = {S33, 0, &V22365_315_57264, &P22365_313};	/*Property = Iframe*/
const Value V22365_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22365_316 = {S34, 0, &V22365_316_57266, &P22365_315};	/*Property = IndexedDB*/
const Value V22365_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22365_317 = {S35, 0, &V22365_317_57269, &P22365_316};	/*Property = Json*/
const Value V22365_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22365_15 = {S37, 0, &V22365_15_29, &P22365_317};	/*Property = LayoutEngine*/
const Value V22365_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22365_344 = {S38, 0, &V22365_344_57272, &P22365_15};	/*Property = Masking*/
const Value V22365_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22365_320 = {S39, 0, &V22365_320_57290, &P22365_344};	/*Property = PostMessage*/
const Value V22365_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22365_321 = {S40, 0, &V22365_321_57293, &P22365_320};	/*Property = Progress*/
const Value V22365_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22365_322 = {S41, 0, &V22365_322_57296, &P22365_321};	/*Property = Prompts*/
const Value V22365_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22365_323 = {S42, 0, &V22365_323_57299, &P22365_322};	/*Property = Selector*/
const Value V22365_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22365_324 = {S43, 0, &V22365_324_57306, &P22365_323};	/*Property = Svg*/
const Value V22365_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22365_327 = {S44, 0, &V22365_327_57315, &P22365_324};	/*Property = TouchEvents*/
const Value V22365_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22365_328 = {S45, 0, &V22365_328_57318, &P22365_327};	/*Property = Track*/
const Value V22365_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22365_329 = {S46, 0, &V22365_329_57323, &P22365_328};	/*Property = Video*/
const Value V22365_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22365_330 = {S47, 0, &V22365_330_57326, &P22365_329};	/*Property = Viewport*/
const Value V22365_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22365_332 = {S48, 0, &V22365_332_57339, &P22365_330};	/*Property = WebWorkers*/
const Value V22365_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22365_333 = {S49, 0, &V22365_333_57341, &P22365_332};	/*Property = Xhr2*/
const Profile P22365 = {"22365", &P22365_333};	/*Profile ID - 22365*/
const Value V21421_1_1 = {S2, NULL};	/*Value = True*/
const Property P21421_1 = {S50, 0, &V21421_1_1, NULL};	/*Property = IsMobile*/
const Value V21421_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21421_5 = {S52, 0, &V21421_5_1458, &P21421_1};	/*Property = ScreenPixelsHeight*/
const Value V21421_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21421_4 = {S54, 0, &V21421_4_37617, &P21421_5};	/*Property = ScreenPixelsWidth*/
const Profile P21421 = {"21421", &P21421_4};	/*Profile ID - 21421*/
const Value V20608_1_1 = {S2, NULL};	/*Value = True*/
const Property P20608_1 = {S50, 0, &V20608_1_1, NULL};	/*Property = IsMobile*/
const Value V20608_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20608_5 = {S52, 0, &V20608_5_9981, &P20608_1};	/*Property = ScreenPixelsHeight*/
const Value V20608_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20608_4 = {S54, 0, &V20608_4_2336, &P20608_5};	/*Property = ScreenPixelsWidth*/
const Profile P20608 = {"20608", &P20608_4};	/*Profile ID - 20608*/
const Value V21249_1_1 = {S2, NULL};	/*Value = True*/
const Property P21249_1 = {S50, 0, &V21249_1_1, NULL};	/*Property = IsMobile*/
const Value V21249_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21249_5 = {S52, 0, &V21249_5_1222, &P21249_1};	/*Property = ScreenPixelsHeight*/
const Value V21249_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21249_4 = {S54, 0, &V21249_4_1224, &P21249_5};	/*Property = ScreenPixelsWidth*/
const Profile P21249 = {"21249", &P21249_4};	/*Profile ID - 21249*/
const Value V21723_1_1 = {S2, NULL};	/*Value = True*/
const Property P21723_1 = {S50, 0, &V21723_1_1, NULL};	/*Property = IsMobile*/
const Value V21723_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21723_5 = {S52, 0, &V21723_5_1227, &P21723_1};	/*Property = ScreenPixelsHeight*/
const Value V21723_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21723_4 = {S54, 0, &V21723_4_1460, &P21723_5};	/*Property = ScreenPixelsWidth*/
const Profile P21723 = {"21723", &P21723_4};	/*Profile ID - 21723*/
const Value V19962_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19962_279 = {S1, 0, &V19962_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19962_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19962_282 = {S3, 0, &V19962_282_57154, &P19962_279};	/*Property = BlobBuilder*/
const Value V19962_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19962_350 = {S4, 0, &V19962_350_57444, &P19962_282};	/*Property = Canvas*/
const Value V19962_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19962_283 = {S5, 0, &V19962_283_57159, &P19962_350};	/*Property = CssBackground*/
const Value V19962_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19962_284 = {S6, 0, &V19962_284_57163, &P19962_283};	/*Property = CssBorderImage*/
const Value V19962_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19962_285 = {S7, 0, &V19962_285_57166, &P19962_284};	/*Property = CssCanvas*/
const Value V19962_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19962_286 = {S8, 0, &V19962_286_57168, &P19962_285};	/*Property = CssColor*/
const Value V19962_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19962_237 = {S9, 0, &V19962_237_55786, &P19962_286};	/*Property = CssColumn*/
const Value V19962_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19962_287 = {S10, 0, &V19962_287_57172, &P19962_237};	/*Property = CssFlexbox*/
const Value V19962_288_57175 = {S0, NULL};	/*Value = False*/
const Property P19962_288 = {S11, 0, &V19962_288_57175, &P19962_287};	/*Property = CssFont*/
const Value V19962_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19962_289 = {S12, 0, &V19962_289_57759, &P19962_288};	/*Property = CssImages*/
const Value V19962_291_57181 = {S0, NULL};	/*Value = False*/
const Property P19962_291 = {S13, 0, &V19962_291_57181, &P19962_289};	/*Property = CssMediaQueries*/
const Value V19962_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19962_292 = {S14, 0, &V19962_292_57183, &P19962_291};	/*Property = CssMinMax*/
const Value V19962_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19962_293 = {S15, 0, &V19962_293_57187, &P19962_292};	/*Property = CssOverflow*/
const Value V19962_294_57190 = {S0, NULL};	/*Value = False*/
const Property P19962_294 = {S16, 0, &V19962_294_57190, &P19962_293};	/*Property = CssPosition*/
const Value V19962_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19962_295 = {S17, 0, &V19962_295_57192, &P19962_294};	/*Property = CssText*/
const Value V19962_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19962_297 = {S18, 0, &V19962_297_57195, &P19962_295};	/*Property = CssTransforms*/
const Value V19962_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19962_241 = {S19, 0, &V19962_241_55799, &P19962_297};	/*Property = CssTransitions*/
const Value V19962_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19962_298 = {S20, 0, &V19962_298_57206, &P19962_241};	/*Property = CssUI*/
const Value V19962_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19962_300 = {S21, 0, &V19962_300_57216, &P19962_298};	/*Property = DataSet*/
const Value V19962_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19962_301 = {S22, 0, &V19962_301_57218, &P19962_300};	/*Property = DataUrl*/
const Value V19962_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19962_302 = {S23, 0, &V19962_302_57222, &P19962_301};	/*Property = DeviceOrientation*/
const Value V19962_304_57228 = {S0, NULL};	/*Value = False*/
const Property P19962_304 = {S24, 0, &V19962_304_57228, &P19962_302};	/*Property = FileReader*/
const Value V19962_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19962_305 = {S25, 0, &V19962_305_57231, &P19962_304};	/*Property = FileSaver*/
const Value V19962_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19962_306 = {S26, 0, &V19962_306_57237, &P19962_305};	/*Property = FileWriter*/
const Value V19962_307_57240 = {S0, NULL};	/*Value = False*/
const Property P19962_307 = {S27, 0, &V19962_307_57240, &P19962_306};	/*Property = FormData*/
const Value V19962_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19962_309 = {S28, 0, &V19962_309_57246, &P19962_307};	/*Property = Fullscreen*/
const Value V19962_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19962_310 = {S29, 0, &V19962_310_57248, &P19962_309};	/*Property = GeoLocation*/
const Value V19962_312_57255 = {S0, NULL};	/*Value = False*/
const Property P19962_312 = {S30, 0, &V19962_312_57255, &P19962_310};	/*Property = History*/
const Value V19962_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19962_314 = {S31, 0, &V19962_314_57261, &P19962_312};	/*Property = Html5*/
const Value V19962_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19962_313 = {S32, 0, &V19962_313_57258, &P19962_314};	/*Property = Html-Media-Capture*/
const Value V19962_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19962_315 = {S33, 0, &V19962_315_57264, &P19962_313};	/*Property = Iframe*/
const Value V19962_316_57267 = {S0, NULL};	/*Value = False*/
const Property P19962_316 = {S34, 0, &V19962_316_57267, &P19962_315};	/*Property = IndexedDB*/
const Value V19962_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19962_317 = {S35, 0, &V19962_317_57269, &P19962_316};	/*Property = Json*/
const Value V19962_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P19962_15 = {S37, 0, &V19962_15_29, &P19962_317};	/*Property = LayoutEngine*/
const Value V19962_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19962_344 = {S38, 0, &V19962_344_57272, &P19962_15};	/*Property = Masking*/
const Value V19962_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19962_320 = {S39, 0, &V19962_320_57290, &P19962_344};	/*Property = PostMessage*/
const Value V19962_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19962_321 = {S40, 0, &V19962_321_57293, &P19962_320};	/*Property = Progress*/
const Value V19962_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19962_322 = {S41, 0, &V19962_322_57296, &P19962_321};	/*Property = Prompts*/
const Value V19962_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19962_323 = {S42, 0, &V19962_323_57299, &P19962_322};	/*Property = Selector*/
const Value V19962_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19962_324 = {S43, 0, &V19962_324_57306, &P19962_323};	/*Property = Svg*/
const Value V19962_327_57315 = {S0, NULL};	/*Value = False*/
const Property P19962_327 = {S44, 0, &V19962_327_57315, &P19962_324};	/*Property = TouchEvents*/
const Value V19962_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19962_328 = {S45, 0, &V19962_328_57318, &P19962_327};	/*Property = Track*/
const Value V19962_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19962_329 = {S46, 0, &V19962_329_57323, &P19962_328};	/*Property = Video*/
const Value V19962_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19962_330 = {S47, 0, &V19962_330_57326, &P19962_329};	/*Property = Viewport*/
const Value V19962_332_57339 = {S0, NULL};	/*Value = False*/
const Property P19962_332 = {S48, 0, &V19962_332_57339, &P19962_330};	/*Property = WebWorkers*/
const Value V19962_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19962_333 = {S49, 0, &V19962_333_57342, &P19962_332};	/*Property = Xhr2*/
const Profile P19962 = {"19962", &P19962_333};	/*Profile ID - 19962*/
const Value V20350_1_1 = {S2, NULL};	/*Value = True*/
const Property P20350_1 = {S50, 0, &V20350_1_1, NULL};	/*Property = IsMobile*/
const Value V20350_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20350_5 = {S52, 0, &V20350_5_1227, &P20350_1};	/*Property = ScreenPixelsHeight*/
const Value V20350_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20350_4 = {S54, 0, &V20350_4_1229, &P20350_5};	/*Property = ScreenPixelsWidth*/
const Profile P20350 = {"20350", &P20350_4};	/*Profile ID - 20350*/
const Value V18692_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18692_279 = {S1, 0, &V18692_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18692_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18692_282 = {S3, 0, &V18692_282_57154, &P18692_279};	/*Property = BlobBuilder*/
const Value V18692_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18692_350 = {S4, 0, &V18692_350_57444, &P18692_282};	/*Property = Canvas*/
const Value V18692_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18692_283 = {S5, 0, &V18692_283_57159, &P18692_350};	/*Property = CssBackground*/
const Value V18692_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18692_284 = {S6, 0, &V18692_284_57163, &P18692_283};	/*Property = CssBorderImage*/
const Value V18692_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18692_285 = {S7, 0, &V18692_285_57166, &P18692_284};	/*Property = CssCanvas*/
const Value V18692_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18692_286 = {S8, 0, &V18692_286_57168, &P18692_285};	/*Property = CssColor*/
const Value V18692_237_55784 = {S2, NULL};	/*Value = True*/
const Property P18692_237 = {S9, 0, &V18692_237_55784, &P18692_286};	/*Property = CssColumn*/
const Value V18692_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18692_287 = {S10, 0, &V18692_287_57172, &P18692_237};	/*Property = CssFlexbox*/
const Value V18692_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18692_288 = {S11, 0, &V18692_288_57175, &P18692_287};	/*Property = CssFont*/
const Value V18692_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18692_289 = {S12, 0, &V18692_289_57759, &P18692_288};	/*Property = CssImages*/
const Value V18692_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18692_291 = {S13, 0, &V18692_291_57181, &P18692_289};	/*Property = CssMediaQueries*/
const Value V18692_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18692_292 = {S14, 0, &V18692_292_57183, &P18692_291};	/*Property = CssMinMax*/
const Value V18692_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18692_293 = {S15, 0, &V18692_293_57187, &P18692_292};	/*Property = CssOverflow*/
const Value V18692_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18692_294 = {S16, 0, &V18692_294_57190, &P18692_293};	/*Property = CssPosition*/
const Value V18692_295_57193 = {S0, NULL};	/*Value = False*/
const Property P18692_295 = {S17, 0, &V18692_295_57193, &P18692_294};	/*Property = CssText*/
const Value V18692_297_57196 = {S0, NULL};	/*Value = False*/
const Property P18692_297 = {S18, 0, &V18692_297_57196, &P18692_295};	/*Property = CssTransforms*/
const Value V18692_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18692_241 = {S19, 0, &V18692_241_55799, &P18692_297};	/*Property = CssTransitions*/
const Value V18692_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18692_298 = {S20, 0, &V18692_298_57206, &P18692_241};	/*Property = CssUI*/
const Value V18692_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18692_300 = {S21, 0, &V18692_300_57216, &P18692_298};	/*Property = DataSet*/
const Value V18692_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18692_301 = {S22, 0, &V18692_301_57218, &P18692_300};	/*Property = DataUrl*/
const Value V18692_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18692_302 = {S23, 0, &V18692_302_57222, &P18692_301};	/*Property = DeviceOrientation*/
const Value V18692_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18692_304 = {S24, 0, &V18692_304_57228, &P18692_302};	/*Property = FileReader*/
const Value V18692_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18692_305 = {S25, 0, &V18692_305_57231, &P18692_304};	/*Property = FileSaver*/
const Value V18692_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18692_306 = {S26, 0, &V18692_306_57237, &P18692_305};	/*Property = FileWriter*/
const Value V18692_307_57240 = {S0, NULL};	/*Value = False*/
const Property P18692_307 = {S27, 0, &V18692_307_57240, &P18692_306};	/*Property = FormData*/
const Value V18692_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18692_309 = {S28, 0, &V18692_309_57246, &P18692_307};	/*Property = Fullscreen*/
const Value V18692_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18692_310 = {S29, 0, &V18692_310_57248, &P18692_309};	/*Property = GeoLocation*/
const Value V18692_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18692_312 = {S30, 0, &V18692_312_57255, &P18692_310};	/*Property = History*/
const Value V18692_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18692_314 = {S31, 0, &V18692_314_57261, &P18692_312};	/*Property = Html5*/
const Value V18692_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18692_313 = {S32, 0, &V18692_313_57258, &P18692_314};	/*Property = Html-Media-Capture*/
const Value V18692_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18692_315 = {S33, 0, &V18692_315_57264, &P18692_313};	/*Property = Iframe*/
const Value V18692_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18692_316 = {S34, 0, &V18692_316_57267, &P18692_315};	/*Property = IndexedDB*/
const Value V18692_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18692_317 = {S35, 0, &V18692_317_57269, &P18692_316};	/*Property = Json*/
const Value V18692_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18692_15 = {S37, 0, &V18692_15_31, &P18692_317};	/*Property = LayoutEngine*/
const Value V18692_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18692_344 = {S38, 0, &V18692_344_57272, &P18692_15};	/*Property = Masking*/
const Value V18692_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18692_320 = {S39, 0, &V18692_320_57290, &P18692_344};	/*Property = PostMessage*/
const Value V18692_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18692_321 = {S40, 0, &V18692_321_57293, &P18692_320};	/*Property = Progress*/
const Value V18692_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18692_322 = {S41, 0, &V18692_322_57296, &P18692_321};	/*Property = Prompts*/
const Value V18692_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18692_323 = {S42, 0, &V18692_323_57299, &P18692_322};	/*Property = Selector*/
const Value V18692_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18692_324 = {S43, 0, &V18692_324_57306, &P18692_323};	/*Property = Svg*/
const Value V18692_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18692_327 = {S44, 0, &V18692_327_57315, &P18692_324};	/*Property = TouchEvents*/
const Value V18692_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18692_328 = {S45, 0, &V18692_328_57318, &P18692_327};	/*Property = Track*/
const Value V18692_329_57324 = {S0, NULL};	/*Value = False*/
const Property P18692_329 = {S46, 0, &V18692_329_57324, &P18692_328};	/*Property = Video*/
const Value V18692_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18692_330 = {S47, 0, &V18692_330_57326, &P18692_329};	/*Property = Viewport*/
const Value V18692_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18692_332 = {S48, 0, &V18692_332_57339, &P18692_330};	/*Property = WebWorkers*/
const Value V18692_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18692_333 = {S49, 0, &V18692_333_57342, &P18692_332};	/*Property = Xhr2*/
const Profile P18692 = {"18692", &P18692_333};	/*Profile ID - 18692*/
const Value V20788_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_279 = {S1, 0, &V20788_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20788_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_282 = {S3, 0, &V20788_282_57155, &P20788_279};	/*Property = BlobBuilder*/
const Value V20788_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_350 = {S4, 0, &V20788_350_57446, &P20788_282};	/*Property = Canvas*/
const Value V20788_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_283 = {S5, 0, &V20788_283_57161, &P20788_350};	/*Property = CssBackground*/
const Value V20788_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_284 = {S6, 0, &V20788_284_57164, &P20788_283};	/*Property = CssBorderImage*/
const Value V20788_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_285 = {S7, 0, &V20788_285_57167, &P20788_284};	/*Property = CssCanvas*/
const Value V20788_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_286 = {S8, 0, &V20788_286_57170, &P20788_285};	/*Property = CssColor*/
const Value V20788_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_237 = {S9, 0, &V20788_237_55786, &P20788_286};	/*Property = CssColumn*/
const Value V20788_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_287 = {S10, 0, &V20788_287_57173, &P20788_237};	/*Property = CssFlexbox*/
const Value V20788_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_288 = {S11, 0, &V20788_288_57176, &P20788_287};	/*Property = CssFont*/
const Value V20788_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_289 = {S12, 0, &V20788_289_86899, &P20788_288};	/*Property = CssImages*/
const Value V20788_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_291 = {S13, 0, &V20788_291_57182, &P20788_289};	/*Property = CssMediaQueries*/
const Value V20788_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_292 = {S14, 0, &V20788_292_57185, &P20788_291};	/*Property = CssMinMax*/
const Value V20788_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_293 = {S15, 0, &V20788_293_57188, &P20788_292};	/*Property = CssOverflow*/
const Value V20788_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_294 = {S16, 0, &V20788_294_57191, &P20788_293};	/*Property = CssPosition*/
const Value V20788_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_295 = {S17, 0, &V20788_295_57194, &P20788_294};	/*Property = CssText*/
const Value V20788_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_297 = {S18, 0, &V20788_297_57197, &P20788_295};	/*Property = CssTransforms*/
const Value V20788_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_241 = {S19, 0, &V20788_241_55801, &P20788_297};	/*Property = CssTransitions*/
const Value V20788_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_298 = {S20, 0, &V20788_298_57208, &P20788_241};	/*Property = CssUI*/
const Value V20788_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_300 = {S21, 0, &V20788_300_57217, &P20788_298};	/*Property = DataSet*/
const Value V20788_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_301 = {S22, 0, &V20788_301_57220, &P20788_300};	/*Property = DataUrl*/
const Value V20788_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_302 = {S23, 0, &V20788_302_57223, &P20788_301};	/*Property = DeviceOrientation*/
const Value V20788_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_304 = {S24, 0, &V20788_304_57229, &P20788_302};	/*Property = FileReader*/
const Value V20788_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_305 = {S25, 0, &V20788_305_57232, &P20788_304};	/*Property = FileSaver*/
const Value V20788_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_306 = {S26, 0, &V20788_306_57238, &P20788_305};	/*Property = FileWriter*/
const Value V20788_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_307 = {S27, 0, &V20788_307_57241, &P20788_306};	/*Property = FormData*/
const Value V20788_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_309 = {S28, 0, &V20788_309_57247, &P20788_307};	/*Property = Fullscreen*/
const Value V20788_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_310 = {S29, 0, &V20788_310_57250, &P20788_309};	/*Property = GeoLocation*/
const Value V20788_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_312 = {S30, 0, &V20788_312_57256, &P20788_310};	/*Property = History*/
const Value V20788_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_314 = {S31, 0, &V20788_314_57262, &P20788_312};	/*Property = Html5*/
const Value V20788_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_313 = {S32, 0, &V20788_313_57259, &P20788_314};	/*Property = Html-Media-Capture*/
const Value V20788_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_315 = {S33, 0, &V20788_315_57265, &P20788_313};	/*Property = Iframe*/
const Value V20788_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_316 = {S34, 0, &V20788_316_57268, &P20788_315};	/*Property = IndexedDB*/
const Value V20788_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_317 = {S35, 0, &V20788_317_57271, &P20788_316};	/*Property = Json*/
const Value V20788_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P20788_15 = {S37, 0, &V20788_15_54167, &P20788_317};	/*Property = LayoutEngine*/
const Value V20788_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_344 = {S38, 0, &V20788_344_57274, &P20788_15};	/*Property = Masking*/
const Value V20788_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_320 = {S39, 0, &V20788_320_57292, &P20788_344};	/*Property = PostMessage*/
const Value V20788_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_321 = {S40, 0, &V20788_321_57295, &P20788_320};	/*Property = Progress*/
const Value V20788_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_322 = {S41, 0, &V20788_322_57298, &P20788_321};	/*Property = Prompts*/
const Value V20788_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_323 = {S42, 0, &V20788_323_57301, &P20788_322};	/*Property = Selector*/
const Value V20788_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_324 = {S43, 0, &V20788_324_57307, &P20788_323};	/*Property = Svg*/
const Value V20788_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_327 = {S44, 0, &V20788_327_57316, &P20788_324};	/*Property = TouchEvents*/
const Value V20788_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_328 = {S45, 0, &V20788_328_57319, &P20788_327};	/*Property = Track*/
const Value V20788_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_329 = {S46, 0, &V20788_329_57325, &P20788_328};	/*Property = Video*/
const Value V20788_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_330 = {S47, 0, &V20788_330_57328, &P20788_329};	/*Property = Viewport*/
const Value V20788_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_332 = {S48, 0, &V20788_332_57340, &P20788_330};	/*Property = WebWorkers*/
const Value V20788_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20788_333 = {S49, 0, &V20788_333_57343, &P20788_332};	/*Property = Xhr2*/
const Profile P20788 = {"20788", &P20788_333};	/*Profile ID - 20788*/
const Value V18634_1_1 = {S2, NULL};	/*Value = True*/
const Property P18634_1 = {S50, 0, &V18634_1_1, NULL};	/*Property = IsMobile*/
const Value V18634_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18634_5 = {S52, 0, &V18634_5_1227, &P18634_1};	/*Property = ScreenPixelsHeight*/
const Value V18634_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18634_4 = {S54, 0, &V18634_4_1229, &P18634_5};	/*Property = ScreenPixelsWidth*/
const Profile P18634 = {"18634", &P18634_4};	/*Profile ID - 18634*/
const Value V21808_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_279 = {S1, 0, &V21808_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21808_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_282 = {S3, 0, &V21808_282_57155, &P21808_279};	/*Property = BlobBuilder*/
const Value V21808_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_350 = {S4, 0, &V21808_350_57446, &P21808_282};	/*Property = Canvas*/
const Value V21808_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_283 = {S5, 0, &V21808_283_57161, &P21808_350};	/*Property = CssBackground*/
const Value V21808_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_284 = {S6, 0, &V21808_284_57164, &P21808_283};	/*Property = CssBorderImage*/
const Value V21808_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_285 = {S7, 0, &V21808_285_57167, &P21808_284};	/*Property = CssCanvas*/
const Value V21808_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_286 = {S8, 0, &V21808_286_57170, &P21808_285};	/*Property = CssColor*/
const Value V21808_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_237 = {S9, 0, &V21808_237_55786, &P21808_286};	/*Property = CssColumn*/
const Value V21808_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_287 = {S10, 0, &V21808_287_57173, &P21808_237};	/*Property = CssFlexbox*/
const Value V21808_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_288 = {S11, 0, &V21808_288_57176, &P21808_287};	/*Property = CssFont*/
const Value V21808_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_289 = {S12, 0, &V21808_289_86899, &P21808_288};	/*Property = CssImages*/
const Value V21808_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_291 = {S13, 0, &V21808_291_57182, &P21808_289};	/*Property = CssMediaQueries*/
const Value V21808_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_292 = {S14, 0, &V21808_292_57185, &P21808_291};	/*Property = CssMinMax*/
const Value V21808_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_293 = {S15, 0, &V21808_293_57188, &P21808_292};	/*Property = CssOverflow*/
const Value V21808_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_294 = {S16, 0, &V21808_294_57191, &P21808_293};	/*Property = CssPosition*/
const Value V21808_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_295 = {S17, 0, &V21808_295_57194, &P21808_294};	/*Property = CssText*/
const Value V21808_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_297 = {S18, 0, &V21808_297_57197, &P21808_295};	/*Property = CssTransforms*/
const Value V21808_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_241 = {S19, 0, &V21808_241_55801, &P21808_297};	/*Property = CssTransitions*/
const Value V21808_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_298 = {S20, 0, &V21808_298_57208, &P21808_241};	/*Property = CssUI*/
const Value V21808_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_300 = {S21, 0, &V21808_300_57217, &P21808_298};	/*Property = DataSet*/
const Value V21808_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_301 = {S22, 0, &V21808_301_57220, &P21808_300};	/*Property = DataUrl*/
const Value V21808_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_302 = {S23, 0, &V21808_302_57223, &P21808_301};	/*Property = DeviceOrientation*/
const Value V21808_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_304 = {S24, 0, &V21808_304_57229, &P21808_302};	/*Property = FileReader*/
const Value V21808_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_305 = {S25, 0, &V21808_305_57232, &P21808_304};	/*Property = FileSaver*/
const Value V21808_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_306 = {S26, 0, &V21808_306_57238, &P21808_305};	/*Property = FileWriter*/
const Value V21808_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_307 = {S27, 0, &V21808_307_57241, &P21808_306};	/*Property = FormData*/
const Value V21808_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_309 = {S28, 0, &V21808_309_57247, &P21808_307};	/*Property = Fullscreen*/
const Value V21808_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_310 = {S29, 0, &V21808_310_57250, &P21808_309};	/*Property = GeoLocation*/
const Value V21808_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_312 = {S30, 0, &V21808_312_57256, &P21808_310};	/*Property = History*/
const Value V21808_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_314 = {S31, 0, &V21808_314_57262, &P21808_312};	/*Property = Html5*/
const Value V21808_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_313 = {S32, 0, &V21808_313_57259, &P21808_314};	/*Property = Html-Media-Capture*/
const Value V21808_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_315 = {S33, 0, &V21808_315_57265, &P21808_313};	/*Property = Iframe*/
const Value V21808_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_316 = {S34, 0, &V21808_316_57268, &P21808_315};	/*Property = IndexedDB*/
const Value V21808_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_317 = {S35, 0, &V21808_317_57271, &P21808_316};	/*Property = Json*/
const Value V21808_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P21808_15 = {S37, 0, &V21808_15_29, &P21808_317};	/*Property = LayoutEngine*/
const Value V21808_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_344 = {S38, 0, &V21808_344_57274, &P21808_15};	/*Property = Masking*/
const Value V21808_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_320 = {S39, 0, &V21808_320_57292, &P21808_344};	/*Property = PostMessage*/
const Value V21808_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_321 = {S40, 0, &V21808_321_57295, &P21808_320};	/*Property = Progress*/
const Value V21808_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_322 = {S41, 0, &V21808_322_57298, &P21808_321};	/*Property = Prompts*/
const Value V21808_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_323 = {S42, 0, &V21808_323_57301, &P21808_322};	/*Property = Selector*/
const Value V21808_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_324 = {S43, 0, &V21808_324_57307, &P21808_323};	/*Property = Svg*/
const Value V21808_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_327 = {S44, 0, &V21808_327_57316, &P21808_324};	/*Property = TouchEvents*/
const Value V21808_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_328 = {S45, 0, &V21808_328_57319, &P21808_327};	/*Property = Track*/
const Value V21808_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_329 = {S46, 0, &V21808_329_57325, &P21808_328};	/*Property = Video*/
const Value V21808_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_330 = {S47, 0, &V21808_330_57328, &P21808_329};	/*Property = Viewport*/
const Value V21808_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_332 = {S48, 0, &V21808_332_57340, &P21808_330};	/*Property = WebWorkers*/
const Value V21808_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21808_333 = {S49, 0, &V21808_333_57343, &P21808_332};	/*Property = Xhr2*/
const Profile P21808 = {"21808", &P21808_333};	/*Profile ID - 21808*/
const Value V20196_1_1 = {S2, NULL};	/*Value = True*/
const Property P20196_1 = {S50, 0, &V20196_1_1, NULL};	/*Property = IsMobile*/
const Value V20196_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20196_5 = {S52, 0, &V20196_5_1227, &P20196_1};	/*Property = ScreenPixelsHeight*/
const Value V20196_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20196_4 = {S54, 0, &V20196_4_1229, &P20196_5};	/*Property = ScreenPixelsWidth*/
const Profile P20196 = {"20196", &P20196_4};	/*Profile ID - 20196*/
const Value V19966_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_279 = {S1, 0, &V19966_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V19966_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_282 = {S3, 0, &V19966_282_57155, &P19966_279};	/*Property = BlobBuilder*/
const Value V19966_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_350 = {S4, 0, &V19966_350_57446, &P19966_282};	/*Property = Canvas*/
const Value V19966_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_283 = {S5, 0, &V19966_283_57161, &P19966_350};	/*Property = CssBackground*/
const Value V19966_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_284 = {S6, 0, &V19966_284_57164, &P19966_283};	/*Property = CssBorderImage*/
const Value V19966_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_285 = {S7, 0, &V19966_285_57167, &P19966_284};	/*Property = CssCanvas*/
const Value V19966_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_286 = {S8, 0, &V19966_286_57170, &P19966_285};	/*Property = CssColor*/
const Value V19966_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_237 = {S9, 0, &V19966_237_55786, &P19966_286};	/*Property = CssColumn*/
const Value V19966_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_287 = {S10, 0, &V19966_287_57173, &P19966_237};	/*Property = CssFlexbox*/
const Value V19966_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_288 = {S11, 0, &V19966_288_57176, &P19966_287};	/*Property = CssFont*/
const Value V19966_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_289 = {S12, 0, &V19966_289_86899, &P19966_288};	/*Property = CssImages*/
const Value V19966_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_291 = {S13, 0, &V19966_291_57182, &P19966_289};	/*Property = CssMediaQueries*/
const Value V19966_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_292 = {S14, 0, &V19966_292_57185, &P19966_291};	/*Property = CssMinMax*/
const Value V19966_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_293 = {S15, 0, &V19966_293_57188, &P19966_292};	/*Property = CssOverflow*/
const Value V19966_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_294 = {S16, 0, &V19966_294_57191, &P19966_293};	/*Property = CssPosition*/
const Value V19966_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_295 = {S17, 0, &V19966_295_57194, &P19966_294};	/*Property = CssText*/
const Value V19966_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_297 = {S18, 0, &V19966_297_57197, &P19966_295};	/*Property = CssTransforms*/
const Value V19966_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_241 = {S19, 0, &V19966_241_55801, &P19966_297};	/*Property = CssTransitions*/
const Value V19966_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_298 = {S20, 0, &V19966_298_57208, &P19966_241};	/*Property = CssUI*/
const Value V19966_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_300 = {S21, 0, &V19966_300_57217, &P19966_298};	/*Property = DataSet*/
const Value V19966_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_301 = {S22, 0, &V19966_301_57220, &P19966_300};	/*Property = DataUrl*/
const Value V19966_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_302 = {S23, 0, &V19966_302_57223, &P19966_301};	/*Property = DeviceOrientation*/
const Value V19966_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_304 = {S24, 0, &V19966_304_57229, &P19966_302};	/*Property = FileReader*/
const Value V19966_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_305 = {S25, 0, &V19966_305_57232, &P19966_304};	/*Property = FileSaver*/
const Value V19966_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_306 = {S26, 0, &V19966_306_57238, &P19966_305};	/*Property = FileWriter*/
const Value V19966_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_307 = {S27, 0, &V19966_307_57241, &P19966_306};	/*Property = FormData*/
const Value V19966_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_309 = {S28, 0, &V19966_309_57247, &P19966_307};	/*Property = Fullscreen*/
const Value V19966_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_310 = {S29, 0, &V19966_310_57250, &P19966_309};	/*Property = GeoLocation*/
const Value V19966_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_312 = {S30, 0, &V19966_312_57256, &P19966_310};	/*Property = History*/
const Value V19966_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_314 = {S31, 0, &V19966_314_57262, &P19966_312};	/*Property = Html5*/
const Value V19966_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_313 = {S32, 0, &V19966_313_57259, &P19966_314};	/*Property = Html-Media-Capture*/
const Value V19966_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_315 = {S33, 0, &V19966_315_57265, &P19966_313};	/*Property = Iframe*/
const Value V19966_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_316 = {S34, 0, &V19966_316_57268, &P19966_315};	/*Property = IndexedDB*/
const Value V19966_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_317 = {S35, 0, &V19966_317_57271, &P19966_316};	/*Property = Json*/
const Value V19966_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P19966_15 = {S37, 0, &V19966_15_29, &P19966_317};	/*Property = LayoutEngine*/
const Value V19966_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_344 = {S38, 0, &V19966_344_57274, &P19966_15};	/*Property = Masking*/
const Value V19966_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_320 = {S39, 0, &V19966_320_57292, &P19966_344};	/*Property = PostMessage*/
const Value V19966_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_321 = {S40, 0, &V19966_321_57295, &P19966_320};	/*Property = Progress*/
const Value V19966_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_322 = {S41, 0, &V19966_322_57298, &P19966_321};	/*Property = Prompts*/
const Value V19966_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_323 = {S42, 0, &V19966_323_57301, &P19966_322};	/*Property = Selector*/
const Value V19966_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_324 = {S43, 0, &V19966_324_57307, &P19966_323};	/*Property = Svg*/
const Value V19966_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_327 = {S44, 0, &V19966_327_57316, &P19966_324};	/*Property = TouchEvents*/
const Value V19966_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_328 = {S45, 0, &V19966_328_57319, &P19966_327};	/*Property = Track*/
const Value V19966_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_329 = {S46, 0, &V19966_329_57325, &P19966_328};	/*Property = Video*/
const Value V19966_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_330 = {S47, 0, &V19966_330_57328, &P19966_329};	/*Property = Viewport*/
const Value V19966_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_332 = {S48, 0, &V19966_332_57340, &P19966_330};	/*Property = WebWorkers*/
const Value V19966_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P19966_333 = {S49, 0, &V19966_333_57343, &P19966_332};	/*Property = Xhr2*/
const Profile P19966 = {"19966", &P19966_333};	/*Profile ID - 19966*/
const Value V18689_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18689_279 = {S1, 0, &V18689_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18689_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18689_282 = {S3, 0, &V18689_282_57154, &P18689_279};	/*Property = BlobBuilder*/
const Value V18689_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18689_350 = {S4, 0, &V18689_350_57444, &P18689_282};	/*Property = Canvas*/
const Value V18689_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18689_283 = {S5, 0, &V18689_283_57159, &P18689_350};	/*Property = CssBackground*/
const Value V18689_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18689_284 = {S6, 0, &V18689_284_57163, &P18689_283};	/*Property = CssBorderImage*/
const Value V18689_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18689_285 = {S7, 0, &V18689_285_57166, &P18689_284};	/*Property = CssCanvas*/
const Value V18689_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18689_286 = {S8, 0, &V18689_286_57168, &P18689_285};	/*Property = CssColor*/
const Value V18689_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18689_237 = {S9, 0, &V18689_237_55786, &P18689_286};	/*Property = CssColumn*/
const Value V18689_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18689_287 = {S10, 0, &V18689_287_57172, &P18689_237};	/*Property = CssFlexbox*/
const Value V18689_288_57174 = {S2, NULL};	/*Value = True*/
const Property P18689_288 = {S11, 0, &V18689_288_57174, &P18689_287};	/*Property = CssFont*/
const Value V18689_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18689_289 = {S12, 0, &V18689_289_57759, &P18689_288};	/*Property = CssImages*/
const Value V18689_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18689_291 = {S13, 0, &V18689_291_57180, &P18689_289};	/*Property = CssMediaQueries*/
const Value V18689_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18689_292 = {S14, 0, &V18689_292_57183, &P18689_291};	/*Property = CssMinMax*/
const Value V18689_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18689_293 = {S15, 0, &V18689_293_57187, &P18689_292};	/*Property = CssOverflow*/
const Value V18689_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18689_294 = {S16, 0, &V18689_294_57189, &P18689_293};	/*Property = CssPosition*/
const Value V18689_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18689_295 = {S17, 0, &V18689_295_57192, &P18689_294};	/*Property = CssText*/
const Value V18689_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18689_297 = {S18, 0, &V18689_297_57195, &P18689_295};	/*Property = CssTransforms*/
const Value V18689_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18689_241 = {S19, 0, &V18689_241_55799, &P18689_297};	/*Property = CssTransitions*/
const Value V18689_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18689_298 = {S20, 0, &V18689_298_57206, &P18689_241};	/*Property = CssUI*/
const Value V18689_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18689_300 = {S21, 0, &V18689_300_57216, &P18689_298};	/*Property = DataSet*/
const Value V18689_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18689_301 = {S22, 0, &V18689_301_57218, &P18689_300};	/*Property = DataUrl*/
const Value V18689_302_57221 = {S2, NULL};	/*Value = True*/
const Property P18689_302 = {S23, 0, &V18689_302_57221, &P18689_301};	/*Property = DeviceOrientation*/
const Value V18689_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18689_304 = {S24, 0, &V18689_304_57227, &P18689_302};	/*Property = FileReader*/
const Value V18689_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18689_305 = {S25, 0, &V18689_305_57231, &P18689_304};	/*Property = FileSaver*/
const Value V18689_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18689_306 = {S26, 0, &V18689_306_57237, &P18689_305};	/*Property = FileWriter*/
const Value V18689_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18689_307 = {S27, 0, &V18689_307_57239, &P18689_306};	/*Property = FormData*/
const Value V18689_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18689_309 = {S28, 0, &V18689_309_57246, &P18689_307};	/*Property = Fullscreen*/
const Value V18689_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18689_310 = {S29, 0, &V18689_310_57248, &P18689_309};	/*Property = GeoLocation*/
const Value V18689_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18689_312 = {S30, 0, &V18689_312_57254, &P18689_310};	/*Property = History*/
const Value V18689_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18689_314 = {S31, 0, &V18689_314_57261, &P18689_312};	/*Property = Html5*/
const Value V18689_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18689_313 = {S32, 0, &V18689_313_57258, &P18689_314};	/*Property = Html-Media-Capture*/
const Value V18689_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18689_315 = {S33, 0, &V18689_315_57264, &P18689_313};	/*Property = Iframe*/
const Value V18689_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18689_316 = {S34, 0, &V18689_316_57266, &P18689_315};	/*Property = IndexedDB*/
const Value V18689_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18689_317 = {S35, 0, &V18689_317_57269, &P18689_316};	/*Property = Json*/
const Value V18689_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P18689_15 = {S37, 0, &V18689_15_29, &P18689_317};	/*Property = LayoutEngine*/
const Value V18689_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18689_344 = {S38, 0, &V18689_344_57272, &P18689_15};	/*Property = Masking*/
const Value V18689_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18689_320 = {S39, 0, &V18689_320_57290, &P18689_344};	/*Property = PostMessage*/
const Value V18689_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18689_321 = {S40, 0, &V18689_321_57293, &P18689_320};	/*Property = Progress*/
const Value V18689_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18689_322 = {S41, 0, &V18689_322_57296, &P18689_321};	/*Property = Prompts*/
const Value V18689_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18689_323 = {S42, 0, &V18689_323_57299, &P18689_322};	/*Property = Selector*/
const Value V18689_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18689_324 = {S43, 0, &V18689_324_57306, &P18689_323};	/*Property = Svg*/
const Value V18689_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18689_327 = {S44, 0, &V18689_327_57315, &P18689_324};	/*Property = TouchEvents*/
const Value V18689_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18689_328 = {S45, 0, &V18689_328_57318, &P18689_327};	/*Property = Track*/
const Value V18689_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18689_329 = {S46, 0, &V18689_329_57323, &P18689_328};	/*Property = Video*/
const Value V18689_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18689_330 = {S47, 0, &V18689_330_57326, &P18689_329};	/*Property = Viewport*/
const Value V18689_332_57338 = {S2, NULL};	/*Value = True*/
const Property P18689_332 = {S48, 0, &V18689_332_57338, &P18689_330};	/*Property = WebWorkers*/
const Value V18689_333_57341 = {S2, NULL};	/*Value = True*/
const Property P18689_333 = {S49, 0, &V18689_333_57341, &P18689_332};	/*Property = Xhr2*/
const Profile P18689 = {"18689", &P18689_333};	/*Profile ID - 18689*/
const Value V18527_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18527_279 = {S1, 0, &V18527_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18527_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18527_282 = {S3, 0, &V18527_282_57154, &P18527_279};	/*Property = BlobBuilder*/
const Value V18527_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18527_350 = {S4, 0, &V18527_350_57444, &P18527_282};	/*Property = Canvas*/
const Value V18527_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18527_283 = {S5, 0, &V18527_283_57159, &P18527_350};	/*Property = CssBackground*/
const Value V18527_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18527_284 = {S6, 0, &V18527_284_57163, &P18527_283};	/*Property = CssBorderImage*/
const Value V18527_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18527_285 = {S7, 0, &V18527_285_57166, &P18527_284};	/*Property = CssCanvas*/
const Value V18527_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18527_286 = {S8, 0, &V18527_286_57168, &P18527_285};	/*Property = CssColor*/
const Value V18527_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18527_237 = {S9, 0, &V18527_237_55786, &P18527_286};	/*Property = CssColumn*/
const Value V18527_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18527_287 = {S10, 0, &V18527_287_57172, &P18527_237};	/*Property = CssFlexbox*/
const Value V18527_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18527_288 = {S11, 0, &V18527_288_57175, &P18527_287};	/*Property = CssFont*/
const Value V18527_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18527_289 = {S12, 0, &V18527_289_57759, &P18527_288};	/*Property = CssImages*/
const Value V18527_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18527_291 = {S13, 0, &V18527_291_57181, &P18527_289};	/*Property = CssMediaQueries*/
const Value V18527_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18527_292 = {S14, 0, &V18527_292_57183, &P18527_291};	/*Property = CssMinMax*/
const Value V18527_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18527_293 = {S15, 0, &V18527_293_57187, &P18527_292};	/*Property = CssOverflow*/
const Value V18527_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18527_294 = {S16, 0, &V18527_294_57189, &P18527_293};	/*Property = CssPosition*/
const Value V18527_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18527_295 = {S17, 0, &V18527_295_57192, &P18527_294};	/*Property = CssText*/
const Value V18527_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18527_297 = {S18, 0, &V18527_297_57195, &P18527_295};	/*Property = CssTransforms*/
const Value V18527_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18527_241 = {S19, 0, &V18527_241_55799, &P18527_297};	/*Property = CssTransitions*/
const Value V18527_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18527_298 = {S20, 0, &V18527_298_57206, &P18527_241};	/*Property = CssUI*/
const Value V18527_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18527_300 = {S21, 0, &V18527_300_57216, &P18527_298};	/*Property = DataSet*/
const Value V18527_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18527_301 = {S22, 0, &V18527_301_57218, &P18527_300};	/*Property = DataUrl*/
const Value V18527_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18527_302 = {S23, 0, &V18527_302_57222, &P18527_301};	/*Property = DeviceOrientation*/
const Value V18527_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18527_304 = {S24, 0, &V18527_304_57227, &P18527_302};	/*Property = FileReader*/
const Value V18527_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18527_305 = {S25, 0, &V18527_305_57231, &P18527_304};	/*Property = FileSaver*/
const Value V18527_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18527_306 = {S26, 0, &V18527_306_57237, &P18527_305};	/*Property = FileWriter*/
const Value V18527_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18527_307 = {S27, 0, &V18527_307_57239, &P18527_306};	/*Property = FormData*/
const Value V18527_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18527_309 = {S28, 0, &V18527_309_57246, &P18527_307};	/*Property = Fullscreen*/
const Value V18527_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18527_310 = {S29, 0, &V18527_310_57248, &P18527_309};	/*Property = GeoLocation*/
const Value V18527_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18527_312 = {S30, 0, &V18527_312_57255, &P18527_310};	/*Property = History*/
const Value V18527_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18527_314 = {S31, 0, &V18527_314_57261, &P18527_312};	/*Property = Html5*/
const Value V18527_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18527_313 = {S32, 0, &V18527_313_57258, &P18527_314};	/*Property = Html-Media-Capture*/
const Value V18527_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18527_315 = {S33, 0, &V18527_315_57264, &P18527_313};	/*Property = Iframe*/
const Value V18527_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18527_316 = {S34, 0, &V18527_316_57267, &P18527_315};	/*Property = IndexedDB*/
const Value V18527_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18527_317 = {S35, 0, &V18527_317_57269, &P18527_316};	/*Property = Json*/
const Value V18527_15_90838 = {S70, NULL};	/*Value = U3*/
const Property P18527_15 = {S37, 0, &V18527_15_90838, &P18527_317};	/*Property = LayoutEngine*/
const Value V18527_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18527_344 = {S38, 0, &V18527_344_57272, &P18527_15};	/*Property = Masking*/
const Value V18527_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18527_320 = {S39, 0, &V18527_320_57290, &P18527_344};	/*Property = PostMessage*/
const Value V18527_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18527_321 = {S40, 0, &V18527_321_57293, &P18527_320};	/*Property = Progress*/
const Value V18527_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18527_322 = {S41, 0, &V18527_322_57296, &P18527_321};	/*Property = Prompts*/
const Value V18527_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18527_323 = {S42, 0, &V18527_323_57299, &P18527_322};	/*Property = Selector*/
const Value V18527_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18527_324 = {S43, 0, &V18527_324_57306, &P18527_323};	/*Property = Svg*/
const Value V18527_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18527_327 = {S44, 0, &V18527_327_57315, &P18527_324};	/*Property = TouchEvents*/
const Value V18527_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18527_328 = {S45, 0, &V18527_328_57318, &P18527_327};	/*Property = Track*/
const Value V18527_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18527_329 = {S46, 0, &V18527_329_57323, &P18527_328};	/*Property = Video*/
const Value V18527_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18527_330 = {S47, 0, &V18527_330_57326, &P18527_329};	/*Property = Viewport*/
const Value V18527_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18527_332 = {S48, 0, &V18527_332_57339, &P18527_330};	/*Property = WebWorkers*/
const Value V18527_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18527_333 = {S49, 0, &V18527_333_57342, &P18527_332};	/*Property = Xhr2*/
const Profile P18527 = {"18527", &P18527_333};	/*Profile ID - 18527*/
const Value V21915_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21915_279 = {S1, 0, &V21915_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21915_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21915_282 = {S3, 0, &V21915_282_57154, &P21915_279};	/*Property = BlobBuilder*/
const Value V21915_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21915_350 = {S4, 0, &V21915_350_57444, &P21915_282};	/*Property = Canvas*/
const Value V21915_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21915_283 = {S5, 0, &V21915_283_57159, &P21915_350};	/*Property = CssBackground*/
const Value V21915_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21915_284 = {S6, 0, &V21915_284_57163, &P21915_283};	/*Property = CssBorderImage*/
const Value V21915_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21915_285 = {S7, 0, &V21915_285_57166, &P21915_284};	/*Property = CssCanvas*/
const Value V21915_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21915_286 = {S8, 0, &V21915_286_57168, &P21915_285};	/*Property = CssColor*/
const Value V21915_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21915_237 = {S9, 0, &V21915_237_55786, &P21915_286};	/*Property = CssColumn*/
const Value V21915_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21915_287 = {S10, 0, &V21915_287_57172, &P21915_237};	/*Property = CssFlexbox*/
const Value V21915_288_57174 = {S2, NULL};	/*Value = True*/
const Property P21915_288 = {S11, 0, &V21915_288_57174, &P21915_287};	/*Property = CssFont*/
const Value V21915_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21915_289 = {S12, 0, &V21915_289_57759, &P21915_288};	/*Property = CssImages*/
const Value V21915_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21915_291 = {S13, 0, &V21915_291_57181, &P21915_289};	/*Property = CssMediaQueries*/
const Value V21915_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21915_292 = {S14, 0, &V21915_292_57183, &P21915_291};	/*Property = CssMinMax*/
const Value V21915_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21915_293 = {S15, 0, &V21915_293_57187, &P21915_292};	/*Property = CssOverflow*/
const Value V21915_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21915_294 = {S16, 0, &V21915_294_57189, &P21915_293};	/*Property = CssPosition*/
const Value V21915_295_57193 = {S0, NULL};	/*Value = False*/
const Property P21915_295 = {S17, 0, &V21915_295_57193, &P21915_294};	/*Property = CssText*/
const Value V21915_297_57196 = {S0, NULL};	/*Value = False*/
const Property P21915_297 = {S18, 0, &V21915_297_57196, &P21915_295};	/*Property = CssTransforms*/
const Value V21915_241_55800 = {S0, NULL};	/*Value = False*/
const Property P21915_241 = {S19, 0, &V21915_241_55800, &P21915_297};	/*Property = CssTransitions*/
const Value V21915_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21915_298 = {S20, 0, &V21915_298_57206, &P21915_241};	/*Property = CssUI*/
const Value V21915_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21915_300 = {S21, 0, &V21915_300_57216, &P21915_298};	/*Property = DataSet*/
const Value V21915_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21915_301 = {S22, 0, &V21915_301_57218, &P21915_300};	/*Property = DataUrl*/
const Value V21915_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21915_302 = {S23, 0, &V21915_302_57222, &P21915_301};	/*Property = DeviceOrientation*/
const Value V21915_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21915_304 = {S24, 0, &V21915_304_57228, &P21915_302};	/*Property = FileReader*/
const Value V21915_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21915_305 = {S25, 0, &V21915_305_57231, &P21915_304};	/*Property = FileSaver*/
const Value V21915_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21915_306 = {S26, 0, &V21915_306_57237, &P21915_305};	/*Property = FileWriter*/
const Value V21915_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21915_307 = {S27, 0, &V21915_307_57240, &P21915_306};	/*Property = FormData*/
const Value V21915_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21915_309 = {S28, 0, &V21915_309_57246, &P21915_307};	/*Property = Fullscreen*/
const Value V21915_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21915_310 = {S29, 0, &V21915_310_57248, &P21915_309};	/*Property = GeoLocation*/
const Value V21915_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21915_312 = {S30, 0, &V21915_312_57255, &P21915_310};	/*Property = History*/
const Value V21915_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21915_314 = {S31, 0, &V21915_314_57261, &P21915_312};	/*Property = Html5*/
const Value V21915_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21915_313 = {S32, 0, &V21915_313_57258, &P21915_314};	/*Property = Html-Media-Capture*/
const Value V21915_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21915_315 = {S33, 0, &V21915_315_57264, &P21915_313};	/*Property = Iframe*/
const Value V21915_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21915_316 = {S34, 0, &V21915_316_57267, &P21915_315};	/*Property = IndexedDB*/
const Value V21915_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21915_317 = {S35, 0, &V21915_317_57269, &P21915_316};	/*Property = Json*/
const Value V21915_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21915_15 = {S37, 0, &V21915_15_28, &P21915_317};	/*Property = LayoutEngine*/
const Value V21915_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21915_344 = {S38, 0, &V21915_344_57272, &P21915_15};	/*Property = Masking*/
const Value V21915_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21915_320 = {S39, 0, &V21915_320_57290, &P21915_344};	/*Property = PostMessage*/
const Value V21915_321_57294 = {S0, NULL};	/*Value = False*/
const Property P21915_321 = {S40, 0, &V21915_321_57294, &P21915_320};	/*Property = Progress*/
const Value V21915_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21915_322 = {S41, 0, &V21915_322_57296, &P21915_321};	/*Property = Prompts*/
const Value V21915_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21915_323 = {S42, 0, &V21915_323_57299, &P21915_322};	/*Property = Selector*/
const Value V21915_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21915_324 = {S43, 0, &V21915_324_57306, &P21915_323};	/*Property = Svg*/
const Value V21915_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21915_327 = {S44, 0, &V21915_327_57315, &P21915_324};	/*Property = TouchEvents*/
const Value V21915_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21915_328 = {S45, 0, &V21915_328_57318, &P21915_327};	/*Property = Track*/
const Value V21915_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21915_329 = {S46, 0, &V21915_329_57323, &P21915_328};	/*Property = Video*/
const Value V21915_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21915_330 = {S47, 0, &V21915_330_57326, &P21915_329};	/*Property = Viewport*/
const Value V21915_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21915_332 = {S48, 0, &V21915_332_57339, &P21915_330};	/*Property = WebWorkers*/
const Value V21915_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21915_333 = {S49, 0, &V21915_333_57342, &P21915_332};	/*Property = Xhr2*/
const Profile P21915 = {"21915", &P21915_333};	/*Profile ID - 21915*/
const Value V21800_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_279 = {S1, 0, &V21800_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21800_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_282 = {S3, 0, &V21800_282_57155, &P21800_279};	/*Property = BlobBuilder*/
const Value V21800_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_350 = {S4, 0, &V21800_350_57446, &P21800_282};	/*Property = Canvas*/
const Value V21800_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_283 = {S5, 0, &V21800_283_57161, &P21800_350};	/*Property = CssBackground*/
const Value V21800_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_284 = {S6, 0, &V21800_284_57164, &P21800_283};	/*Property = CssBorderImage*/
const Value V21800_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_285 = {S7, 0, &V21800_285_57167, &P21800_284};	/*Property = CssCanvas*/
const Value V21800_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_286 = {S8, 0, &V21800_286_57170, &P21800_285};	/*Property = CssColor*/
const Value V21800_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_237 = {S9, 0, &V21800_237_55786, &P21800_286};	/*Property = CssColumn*/
const Value V21800_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_287 = {S10, 0, &V21800_287_57173, &P21800_237};	/*Property = CssFlexbox*/
const Value V21800_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_288 = {S11, 0, &V21800_288_57176, &P21800_287};	/*Property = CssFont*/
const Value V21800_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_289 = {S12, 0, &V21800_289_86899, &P21800_288};	/*Property = CssImages*/
const Value V21800_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_291 = {S13, 0, &V21800_291_57182, &P21800_289};	/*Property = CssMediaQueries*/
const Value V21800_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_292 = {S14, 0, &V21800_292_57185, &P21800_291};	/*Property = CssMinMax*/
const Value V21800_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_293 = {S15, 0, &V21800_293_57188, &P21800_292};	/*Property = CssOverflow*/
const Value V21800_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_294 = {S16, 0, &V21800_294_57191, &P21800_293};	/*Property = CssPosition*/
const Value V21800_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_295 = {S17, 0, &V21800_295_57194, &P21800_294};	/*Property = CssText*/
const Value V21800_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_297 = {S18, 0, &V21800_297_57197, &P21800_295};	/*Property = CssTransforms*/
const Value V21800_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_241 = {S19, 0, &V21800_241_55801, &P21800_297};	/*Property = CssTransitions*/
const Value V21800_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_298 = {S20, 0, &V21800_298_57208, &P21800_241};	/*Property = CssUI*/
const Value V21800_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_300 = {S21, 0, &V21800_300_57217, &P21800_298};	/*Property = DataSet*/
const Value V21800_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_301 = {S22, 0, &V21800_301_57220, &P21800_300};	/*Property = DataUrl*/
const Value V21800_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_302 = {S23, 0, &V21800_302_57223, &P21800_301};	/*Property = DeviceOrientation*/
const Value V21800_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_304 = {S24, 0, &V21800_304_57229, &P21800_302};	/*Property = FileReader*/
const Value V21800_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_305 = {S25, 0, &V21800_305_57232, &P21800_304};	/*Property = FileSaver*/
const Value V21800_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_306 = {S26, 0, &V21800_306_57238, &P21800_305};	/*Property = FileWriter*/
const Value V21800_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_307 = {S27, 0, &V21800_307_57241, &P21800_306};	/*Property = FormData*/
const Value V21800_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_309 = {S28, 0, &V21800_309_57247, &P21800_307};	/*Property = Fullscreen*/
const Value V21800_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_310 = {S29, 0, &V21800_310_57250, &P21800_309};	/*Property = GeoLocation*/
const Value V21800_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_312 = {S30, 0, &V21800_312_57256, &P21800_310};	/*Property = History*/
const Value V21800_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_314 = {S31, 0, &V21800_314_57262, &P21800_312};	/*Property = Html5*/
const Value V21800_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_313 = {S32, 0, &V21800_313_57259, &P21800_314};	/*Property = Html-Media-Capture*/
const Value V21800_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_315 = {S33, 0, &V21800_315_57265, &P21800_313};	/*Property = Iframe*/
const Value V21800_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_316 = {S34, 0, &V21800_316_57268, &P21800_315};	/*Property = IndexedDB*/
const Value V21800_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_317 = {S35, 0, &V21800_317_57271, &P21800_316};	/*Property = Json*/
const Value V21800_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21800_15 = {S37, 0, &V21800_15_54167, &P21800_317};	/*Property = LayoutEngine*/
const Value V21800_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_344 = {S38, 0, &V21800_344_57274, &P21800_15};	/*Property = Masking*/
const Value V21800_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_320 = {S39, 0, &V21800_320_57292, &P21800_344};	/*Property = PostMessage*/
const Value V21800_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_321 = {S40, 0, &V21800_321_57295, &P21800_320};	/*Property = Progress*/
const Value V21800_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_322 = {S41, 0, &V21800_322_57298, &P21800_321};	/*Property = Prompts*/
const Value V21800_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_323 = {S42, 0, &V21800_323_57301, &P21800_322};	/*Property = Selector*/
const Value V21800_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_324 = {S43, 0, &V21800_324_57307, &P21800_323};	/*Property = Svg*/
const Value V21800_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_327 = {S44, 0, &V21800_327_57316, &P21800_324};	/*Property = TouchEvents*/
const Value V21800_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_328 = {S45, 0, &V21800_328_57319, &P21800_327};	/*Property = Track*/
const Value V21800_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_329 = {S46, 0, &V21800_329_57325, &P21800_328};	/*Property = Video*/
const Value V21800_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_330 = {S47, 0, &V21800_330_57328, &P21800_329};	/*Property = Viewport*/
const Value V21800_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_332 = {S48, 0, &V21800_332_57340, &P21800_330};	/*Property = WebWorkers*/
const Value V21800_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21800_333 = {S49, 0, &V21800_333_57343, &P21800_332};	/*Property = Xhr2*/
const Profile P21800 = {"21800", &P21800_333};	/*Profile ID - 21800*/
const Value V20968_1_1 = {S2, NULL};	/*Value = True*/
const Property P20968_1 = {S50, 0, &V20968_1_1, NULL};	/*Property = IsMobile*/
const Value V20968_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20968_5 = {S52, 0, &V20968_5_1458, &P20968_1};	/*Property = ScreenPixelsHeight*/
const Value V20968_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20968_4 = {S54, 0, &V20968_4_37617, &P20968_5};	/*Property = ScreenPixelsWidth*/
const Profile P20968 = {"20968", &P20968_4};	/*Profile ID - 20968*/
const Value V18549_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_279 = {S1, 0, &V18549_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18549_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_282 = {S3, 0, &V18549_282_57155, &P18549_279};	/*Property = BlobBuilder*/
const Value V18549_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_350 = {S4, 0, &V18549_350_57446, &P18549_282};	/*Property = Canvas*/
const Value V18549_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_283 = {S5, 0, &V18549_283_57161, &P18549_350};	/*Property = CssBackground*/
const Value V18549_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_284 = {S6, 0, &V18549_284_57164, &P18549_283};	/*Property = CssBorderImage*/
const Value V18549_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_285 = {S7, 0, &V18549_285_57167, &P18549_284};	/*Property = CssCanvas*/
const Value V18549_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_286 = {S8, 0, &V18549_286_57170, &P18549_285};	/*Property = CssColor*/
const Value V18549_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_237 = {S9, 0, &V18549_237_55786, &P18549_286};	/*Property = CssColumn*/
const Value V18549_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_287 = {S10, 0, &V18549_287_57173, &P18549_237};	/*Property = CssFlexbox*/
const Value V18549_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_288 = {S11, 0, &V18549_288_57176, &P18549_287};	/*Property = CssFont*/
const Value V18549_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_289 = {S12, 0, &V18549_289_86899, &P18549_288};	/*Property = CssImages*/
const Value V18549_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_291 = {S13, 0, &V18549_291_57182, &P18549_289};	/*Property = CssMediaQueries*/
const Value V18549_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_292 = {S14, 0, &V18549_292_57185, &P18549_291};	/*Property = CssMinMax*/
const Value V18549_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_293 = {S15, 0, &V18549_293_57188, &P18549_292};	/*Property = CssOverflow*/
const Value V18549_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_294 = {S16, 0, &V18549_294_57191, &P18549_293};	/*Property = CssPosition*/
const Value V18549_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_295 = {S17, 0, &V18549_295_57194, &P18549_294};	/*Property = CssText*/
const Value V18549_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_297 = {S18, 0, &V18549_297_57197, &P18549_295};	/*Property = CssTransforms*/
const Value V18549_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_241 = {S19, 0, &V18549_241_55801, &P18549_297};	/*Property = CssTransitions*/
const Value V18549_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_298 = {S20, 0, &V18549_298_57208, &P18549_241};	/*Property = CssUI*/
const Value V18549_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_300 = {S21, 0, &V18549_300_57217, &P18549_298};	/*Property = DataSet*/
const Value V18549_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_301 = {S22, 0, &V18549_301_57220, &P18549_300};	/*Property = DataUrl*/
const Value V18549_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_302 = {S23, 0, &V18549_302_57223, &P18549_301};	/*Property = DeviceOrientation*/
const Value V18549_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_304 = {S24, 0, &V18549_304_57229, &P18549_302};	/*Property = FileReader*/
const Value V18549_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_305 = {S25, 0, &V18549_305_57232, &P18549_304};	/*Property = FileSaver*/
const Value V18549_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_306 = {S26, 0, &V18549_306_57238, &P18549_305};	/*Property = FileWriter*/
const Value V18549_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_307 = {S27, 0, &V18549_307_57241, &P18549_306};	/*Property = FormData*/
const Value V18549_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_309 = {S28, 0, &V18549_309_57247, &P18549_307};	/*Property = Fullscreen*/
const Value V18549_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_310 = {S29, 0, &V18549_310_57250, &P18549_309};	/*Property = GeoLocation*/
const Value V18549_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_312 = {S30, 0, &V18549_312_57256, &P18549_310};	/*Property = History*/
const Value V18549_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_314 = {S31, 0, &V18549_314_57262, &P18549_312};	/*Property = Html5*/
const Value V18549_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_313 = {S32, 0, &V18549_313_57259, &P18549_314};	/*Property = Html-Media-Capture*/
const Value V18549_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_315 = {S33, 0, &V18549_315_57265, &P18549_313};	/*Property = Iframe*/
const Value V18549_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_316 = {S34, 0, &V18549_316_57268, &P18549_315};	/*Property = IndexedDB*/
const Value V18549_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_317 = {S35, 0, &V18549_317_57271, &P18549_316};	/*Property = Json*/
const Value V18549_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18549_15 = {S37, 0, &V18549_15_31, &P18549_317};	/*Property = LayoutEngine*/
const Value V18549_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_344 = {S38, 0, &V18549_344_57274, &P18549_15};	/*Property = Masking*/
const Value V18549_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_320 = {S39, 0, &V18549_320_57292, &P18549_344};	/*Property = PostMessage*/
const Value V18549_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_321 = {S40, 0, &V18549_321_57295, &P18549_320};	/*Property = Progress*/
const Value V18549_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_322 = {S41, 0, &V18549_322_57298, &P18549_321};	/*Property = Prompts*/
const Value V18549_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_323 = {S42, 0, &V18549_323_57301, &P18549_322};	/*Property = Selector*/
const Value V18549_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_324 = {S43, 0, &V18549_324_57307, &P18549_323};	/*Property = Svg*/
const Value V18549_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_327 = {S44, 0, &V18549_327_57316, &P18549_324};	/*Property = TouchEvents*/
const Value V18549_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_328 = {S45, 0, &V18549_328_57319, &P18549_327};	/*Property = Track*/
const Value V18549_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_329 = {S46, 0, &V18549_329_57325, &P18549_328};	/*Property = Video*/
const Value V18549_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_330 = {S47, 0, &V18549_330_57328, &P18549_329};	/*Property = Viewport*/
const Value V18549_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_332 = {S48, 0, &V18549_332_57340, &P18549_330};	/*Property = WebWorkers*/
const Value V18549_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18549_333 = {S49, 0, &V18549_333_57343, &P18549_332};	/*Property = Xhr2*/
const Profile P18549 = {"18549", &P18549_333};	/*Profile ID - 18549*/
const Value V21123_1_1 = {S2, NULL};	/*Value = True*/
const Property P21123_1 = {S50, 0, &V21123_1_1, NULL};	/*Property = IsMobile*/
const Value V21123_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21123_5 = {S52, 0, &V21123_5_1277, &P21123_1};	/*Property = ScreenPixelsHeight*/
const Value V21123_4_37618 = {S83, NULL};	/*Value = 272*/
const Property P21123_4 = {S54, 0, &V21123_4_37618, &P21123_5};	/*Property = ScreenPixelsWidth*/
const Profile P21123 = {"21123", &P21123_4};	/*Profile ID - 21123*/
const Value V23929_1_1 = {S2, NULL};	/*Value = True*/
const Property P23929_1 = {S50, 0, &V23929_1_1, NULL};	/*Property = IsMobile*/
const Value V23929_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23929_5 = {S52, 0, &V23929_5_1227, &P23929_1};	/*Property = ScreenPixelsHeight*/
const Value V23929_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23929_4 = {S54, 0, &V23929_4_1229, &P23929_5};	/*Property = ScreenPixelsWidth*/
const Profile P23929 = {"23929", &P23929_4};	/*Profile ID - 23929*/
const Value V20235_1_1 = {S2, NULL};	/*Value = True*/
const Property P20235_1 = {S50, 0, &V20235_1_1, NULL};	/*Property = IsMobile*/
const Value V20235_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20235_5 = {S52, 0, &V20235_5_1227, &P20235_1};	/*Property = ScreenPixelsHeight*/
const Value V20235_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20235_4 = {S54, 0, &V20235_4_1229, &P20235_5};	/*Property = ScreenPixelsWidth*/
const Profile P20235 = {"20235", &P20235_4};	/*Profile ID - 20235*/
const Profile P22781 = {"22781", NULL};	/*Profile ID - 22781*/
const Value V18395_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_279 = {S1, 0, &V18395_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18395_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_282 = {S3, 0, &V18395_282_57155, &P18395_279};	/*Property = BlobBuilder*/
const Value V18395_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_350 = {S4, 0, &V18395_350_57446, &P18395_282};	/*Property = Canvas*/
const Value V18395_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_283 = {S5, 0, &V18395_283_57161, &P18395_350};	/*Property = CssBackground*/
const Value V18395_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_284 = {S6, 0, &V18395_284_57164, &P18395_283};	/*Property = CssBorderImage*/
const Value V18395_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_285 = {S7, 0, &V18395_285_57167, &P18395_284};	/*Property = CssCanvas*/
const Value V18395_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_286 = {S8, 0, &V18395_286_57170, &P18395_285};	/*Property = CssColor*/
const Value V18395_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_237 = {S9, 0, &V18395_237_55786, &P18395_286};	/*Property = CssColumn*/
const Value V18395_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_287 = {S10, 0, &V18395_287_57173, &P18395_237};	/*Property = CssFlexbox*/
const Value V18395_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_288 = {S11, 0, &V18395_288_57176, &P18395_287};	/*Property = CssFont*/
const Value V18395_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_289 = {S12, 0, &V18395_289_86899, &P18395_288};	/*Property = CssImages*/
const Value V18395_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_291 = {S13, 0, &V18395_291_57182, &P18395_289};	/*Property = CssMediaQueries*/
const Value V18395_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_292 = {S14, 0, &V18395_292_57185, &P18395_291};	/*Property = CssMinMax*/
const Value V18395_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_293 = {S15, 0, &V18395_293_57188, &P18395_292};	/*Property = CssOverflow*/
const Value V18395_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_294 = {S16, 0, &V18395_294_57191, &P18395_293};	/*Property = CssPosition*/
const Value V18395_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_295 = {S17, 0, &V18395_295_57194, &P18395_294};	/*Property = CssText*/
const Value V18395_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_297 = {S18, 0, &V18395_297_57197, &P18395_295};	/*Property = CssTransforms*/
const Value V18395_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_241 = {S19, 0, &V18395_241_55801, &P18395_297};	/*Property = CssTransitions*/
const Value V18395_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_298 = {S20, 0, &V18395_298_57208, &P18395_241};	/*Property = CssUI*/
const Value V18395_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_300 = {S21, 0, &V18395_300_57217, &P18395_298};	/*Property = DataSet*/
const Value V18395_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_301 = {S22, 0, &V18395_301_57220, &P18395_300};	/*Property = DataUrl*/
const Value V18395_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_302 = {S23, 0, &V18395_302_57223, &P18395_301};	/*Property = DeviceOrientation*/
const Value V18395_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_304 = {S24, 0, &V18395_304_57229, &P18395_302};	/*Property = FileReader*/
const Value V18395_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_305 = {S25, 0, &V18395_305_57232, &P18395_304};	/*Property = FileSaver*/
const Value V18395_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_306 = {S26, 0, &V18395_306_57238, &P18395_305};	/*Property = FileWriter*/
const Value V18395_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_307 = {S27, 0, &V18395_307_57241, &P18395_306};	/*Property = FormData*/
const Value V18395_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_309 = {S28, 0, &V18395_309_57247, &P18395_307};	/*Property = Fullscreen*/
const Value V18395_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_310 = {S29, 0, &V18395_310_57250, &P18395_309};	/*Property = GeoLocation*/
const Value V18395_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_312 = {S30, 0, &V18395_312_57256, &P18395_310};	/*Property = History*/
const Value V18395_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_314 = {S31, 0, &V18395_314_57262, &P18395_312};	/*Property = Html5*/
const Value V18395_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_313 = {S32, 0, &V18395_313_57259, &P18395_314};	/*Property = Html-Media-Capture*/
const Value V18395_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_315 = {S33, 0, &V18395_315_57265, &P18395_313};	/*Property = Iframe*/
const Value V18395_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_316 = {S34, 0, &V18395_316_57268, &P18395_315};	/*Property = IndexedDB*/
const Value V18395_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_317 = {S35, 0, &V18395_317_57271, &P18395_316};	/*Property = Json*/
const Value V18395_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18395_15 = {S37, 0, &V18395_15_31, &P18395_317};	/*Property = LayoutEngine*/
const Value V18395_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_344 = {S38, 0, &V18395_344_57274, &P18395_15};	/*Property = Masking*/
const Value V18395_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_320 = {S39, 0, &V18395_320_57292, &P18395_344};	/*Property = PostMessage*/
const Value V18395_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_321 = {S40, 0, &V18395_321_57295, &P18395_320};	/*Property = Progress*/
const Value V18395_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_322 = {S41, 0, &V18395_322_57298, &P18395_321};	/*Property = Prompts*/
const Value V18395_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_323 = {S42, 0, &V18395_323_57301, &P18395_322};	/*Property = Selector*/
const Value V18395_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_324 = {S43, 0, &V18395_324_57307, &P18395_323};	/*Property = Svg*/
const Value V18395_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_327 = {S44, 0, &V18395_327_57316, &P18395_324};	/*Property = TouchEvents*/
const Value V18395_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_328 = {S45, 0, &V18395_328_57319, &P18395_327};	/*Property = Track*/
const Value V18395_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_329 = {S46, 0, &V18395_329_57325, &P18395_328};	/*Property = Video*/
const Value V18395_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_330 = {S47, 0, &V18395_330_57328, &P18395_329};	/*Property = Viewport*/
const Value V18395_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_332 = {S48, 0, &V18395_332_57340, &P18395_330};	/*Property = WebWorkers*/
const Value V18395_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18395_333 = {S49, 0, &V18395_333_57343, &P18395_332};	/*Property = Xhr2*/
const Profile P18395 = {"18395", &P18395_333};	/*Profile ID - 18395*/
const Value V19813_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_279 = {S1, 0, &V19813_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V19813_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_282 = {S3, 0, &V19813_282_57155, &P19813_279};	/*Property = BlobBuilder*/
const Value V19813_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_350 = {S4, 0, &V19813_350_57446, &P19813_282};	/*Property = Canvas*/
const Value V19813_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_283 = {S5, 0, &V19813_283_57161, &P19813_350};	/*Property = CssBackground*/
const Value V19813_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_284 = {S6, 0, &V19813_284_57164, &P19813_283};	/*Property = CssBorderImage*/
const Value V19813_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_285 = {S7, 0, &V19813_285_57167, &P19813_284};	/*Property = CssCanvas*/
const Value V19813_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_286 = {S8, 0, &V19813_286_57170, &P19813_285};	/*Property = CssColor*/
const Value V19813_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_237 = {S9, 0, &V19813_237_55786, &P19813_286};	/*Property = CssColumn*/
const Value V19813_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_287 = {S10, 0, &V19813_287_57173, &P19813_237};	/*Property = CssFlexbox*/
const Value V19813_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_288 = {S11, 0, &V19813_288_57176, &P19813_287};	/*Property = CssFont*/
const Value V19813_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_289 = {S12, 0, &V19813_289_86899, &P19813_288};	/*Property = CssImages*/
const Value V19813_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_291 = {S13, 0, &V19813_291_57182, &P19813_289};	/*Property = CssMediaQueries*/
const Value V19813_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_292 = {S14, 0, &V19813_292_57185, &P19813_291};	/*Property = CssMinMax*/
const Value V19813_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_293 = {S15, 0, &V19813_293_57188, &P19813_292};	/*Property = CssOverflow*/
const Value V19813_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_294 = {S16, 0, &V19813_294_57191, &P19813_293};	/*Property = CssPosition*/
const Value V19813_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_295 = {S17, 0, &V19813_295_57194, &P19813_294};	/*Property = CssText*/
const Value V19813_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_297 = {S18, 0, &V19813_297_57197, &P19813_295};	/*Property = CssTransforms*/
const Value V19813_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_241 = {S19, 0, &V19813_241_55801, &P19813_297};	/*Property = CssTransitions*/
const Value V19813_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_298 = {S20, 0, &V19813_298_57208, &P19813_241};	/*Property = CssUI*/
const Value V19813_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_300 = {S21, 0, &V19813_300_57217, &P19813_298};	/*Property = DataSet*/
const Value V19813_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_301 = {S22, 0, &V19813_301_57220, &P19813_300};	/*Property = DataUrl*/
const Value V19813_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_302 = {S23, 0, &V19813_302_57223, &P19813_301};	/*Property = DeviceOrientation*/
const Value V19813_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_304 = {S24, 0, &V19813_304_57229, &P19813_302};	/*Property = FileReader*/
const Value V19813_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_305 = {S25, 0, &V19813_305_57232, &P19813_304};	/*Property = FileSaver*/
const Value V19813_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_306 = {S26, 0, &V19813_306_57238, &P19813_305};	/*Property = FileWriter*/
const Value V19813_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_307 = {S27, 0, &V19813_307_57241, &P19813_306};	/*Property = FormData*/
const Value V19813_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_309 = {S28, 0, &V19813_309_57247, &P19813_307};	/*Property = Fullscreen*/
const Value V19813_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_310 = {S29, 0, &V19813_310_57250, &P19813_309};	/*Property = GeoLocation*/
const Value V19813_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_312 = {S30, 0, &V19813_312_57256, &P19813_310};	/*Property = History*/
const Value V19813_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_314 = {S31, 0, &V19813_314_57262, &P19813_312};	/*Property = Html5*/
const Value V19813_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_313 = {S32, 0, &V19813_313_57259, &P19813_314};	/*Property = Html-Media-Capture*/
const Value V19813_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_315 = {S33, 0, &V19813_315_57265, &P19813_313};	/*Property = Iframe*/
const Value V19813_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_316 = {S34, 0, &V19813_316_57268, &P19813_315};	/*Property = IndexedDB*/
const Value V19813_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_317 = {S35, 0, &V19813_317_57271, &P19813_316};	/*Property = Json*/
const Value V19813_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19813_15 = {S37, 0, &V19813_15_28, &P19813_317};	/*Property = LayoutEngine*/
const Value V19813_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_344 = {S38, 0, &V19813_344_57274, &P19813_15};	/*Property = Masking*/
const Value V19813_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_320 = {S39, 0, &V19813_320_57292, &P19813_344};	/*Property = PostMessage*/
const Value V19813_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_321 = {S40, 0, &V19813_321_57295, &P19813_320};	/*Property = Progress*/
const Value V19813_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_322 = {S41, 0, &V19813_322_57298, &P19813_321};	/*Property = Prompts*/
const Value V19813_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_323 = {S42, 0, &V19813_323_57301, &P19813_322};	/*Property = Selector*/
const Value V19813_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_324 = {S43, 0, &V19813_324_57307, &P19813_323};	/*Property = Svg*/
const Value V19813_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_327 = {S44, 0, &V19813_327_57316, &P19813_324};	/*Property = TouchEvents*/
const Value V19813_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_328 = {S45, 0, &V19813_328_57319, &P19813_327};	/*Property = Track*/
const Value V19813_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_329 = {S46, 0, &V19813_329_57325, &P19813_328};	/*Property = Video*/
const Value V19813_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_330 = {S47, 0, &V19813_330_57328, &P19813_329};	/*Property = Viewport*/
const Value V19813_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_332 = {S48, 0, &V19813_332_57340, &P19813_330};	/*Property = WebWorkers*/
const Value V19813_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P19813_333 = {S49, 0, &V19813_333_57343, &P19813_332};	/*Property = Xhr2*/
const Profile P19813 = {"19813", &P19813_333};	/*Profile ID - 19813*/
const Value V21787_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_279 = {S1, 0, &V21787_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21787_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_282 = {S3, 0, &V21787_282_57155, &P21787_279};	/*Property = BlobBuilder*/
const Value V21787_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_350 = {S4, 0, &V21787_350_57446, &P21787_282};	/*Property = Canvas*/
const Value V21787_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_283 = {S5, 0, &V21787_283_57161, &P21787_350};	/*Property = CssBackground*/
const Value V21787_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_284 = {S6, 0, &V21787_284_57164, &P21787_283};	/*Property = CssBorderImage*/
const Value V21787_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_285 = {S7, 0, &V21787_285_57167, &P21787_284};	/*Property = CssCanvas*/
const Value V21787_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_286 = {S8, 0, &V21787_286_57170, &P21787_285};	/*Property = CssColor*/
const Value V21787_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_237 = {S9, 0, &V21787_237_55786, &P21787_286};	/*Property = CssColumn*/
const Value V21787_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_287 = {S10, 0, &V21787_287_57173, &P21787_237};	/*Property = CssFlexbox*/
const Value V21787_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_288 = {S11, 0, &V21787_288_57176, &P21787_287};	/*Property = CssFont*/
const Value V21787_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_289 = {S12, 0, &V21787_289_86899, &P21787_288};	/*Property = CssImages*/
const Value V21787_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_291 = {S13, 0, &V21787_291_57182, &P21787_289};	/*Property = CssMediaQueries*/
const Value V21787_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_292 = {S14, 0, &V21787_292_57185, &P21787_291};	/*Property = CssMinMax*/
const Value V21787_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_293 = {S15, 0, &V21787_293_57188, &P21787_292};	/*Property = CssOverflow*/
const Value V21787_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_294 = {S16, 0, &V21787_294_57191, &P21787_293};	/*Property = CssPosition*/
const Value V21787_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_295 = {S17, 0, &V21787_295_57194, &P21787_294};	/*Property = CssText*/
const Value V21787_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_297 = {S18, 0, &V21787_297_57197, &P21787_295};	/*Property = CssTransforms*/
const Value V21787_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_241 = {S19, 0, &V21787_241_55801, &P21787_297};	/*Property = CssTransitions*/
const Value V21787_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_298 = {S20, 0, &V21787_298_57208, &P21787_241};	/*Property = CssUI*/
const Value V21787_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_300 = {S21, 0, &V21787_300_57217, &P21787_298};	/*Property = DataSet*/
const Value V21787_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_301 = {S22, 0, &V21787_301_57220, &P21787_300};	/*Property = DataUrl*/
const Value V21787_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_302 = {S23, 0, &V21787_302_57223, &P21787_301};	/*Property = DeviceOrientation*/
const Value V21787_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_304 = {S24, 0, &V21787_304_57229, &P21787_302};	/*Property = FileReader*/
const Value V21787_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_305 = {S25, 0, &V21787_305_57232, &P21787_304};	/*Property = FileSaver*/
const Value V21787_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_306 = {S26, 0, &V21787_306_57238, &P21787_305};	/*Property = FileWriter*/
const Value V21787_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_307 = {S27, 0, &V21787_307_57241, &P21787_306};	/*Property = FormData*/
const Value V21787_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_309 = {S28, 0, &V21787_309_57247, &P21787_307};	/*Property = Fullscreen*/
const Value V21787_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_310 = {S29, 0, &V21787_310_57250, &P21787_309};	/*Property = GeoLocation*/
const Value V21787_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_312 = {S30, 0, &V21787_312_57256, &P21787_310};	/*Property = History*/
const Value V21787_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_314 = {S31, 0, &V21787_314_57262, &P21787_312};	/*Property = Html5*/
const Value V21787_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_313 = {S32, 0, &V21787_313_57259, &P21787_314};	/*Property = Html-Media-Capture*/
const Value V21787_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_315 = {S33, 0, &V21787_315_57265, &P21787_313};	/*Property = Iframe*/
const Value V21787_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_316 = {S34, 0, &V21787_316_57268, &P21787_315};	/*Property = IndexedDB*/
const Value V21787_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_317 = {S35, 0, &V21787_317_57271, &P21787_316};	/*Property = Json*/
const Value V21787_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_15 = {S37, 0, &V21787_15_72, &P21787_317};	/*Property = LayoutEngine*/
const Value V21787_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_344 = {S38, 0, &V21787_344_57274, &P21787_15};	/*Property = Masking*/
const Value V21787_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_320 = {S39, 0, &V21787_320_57292, &P21787_344};	/*Property = PostMessage*/
const Value V21787_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_321 = {S40, 0, &V21787_321_57295, &P21787_320};	/*Property = Progress*/
const Value V21787_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_322 = {S41, 0, &V21787_322_57298, &P21787_321};	/*Property = Prompts*/
const Value V21787_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_323 = {S42, 0, &V21787_323_57301, &P21787_322};	/*Property = Selector*/
const Value V21787_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_324 = {S43, 0, &V21787_324_57307, &P21787_323};	/*Property = Svg*/
const Value V21787_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_327 = {S44, 0, &V21787_327_57316, &P21787_324};	/*Property = TouchEvents*/
const Value V21787_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_328 = {S45, 0, &V21787_328_57319, &P21787_327};	/*Property = Track*/
const Value V21787_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_329 = {S46, 0, &V21787_329_57325, &P21787_328};	/*Property = Video*/
const Value V21787_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_330 = {S47, 0, &V21787_330_57328, &P21787_329};	/*Property = Viewport*/
const Value V21787_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_332 = {S48, 0, &V21787_332_57340, &P21787_330};	/*Property = WebWorkers*/
const Value V21787_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21787_333 = {S49, 0, &V21787_333_57343, &P21787_332};	/*Property = Xhr2*/
const Profile P21787 = {"21787", &P21787_333};	/*Profile ID - 21787*/
const Value V17098_1_1 = {S2, NULL};	/*Value = True*/
const Property P17098_1 = {S50, 0, &V17098_1_1, NULL};	/*Property = IsMobile*/
const Value V17098_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17098_5 = {S52, 0, &V17098_5_1222, &P17098_1};	/*Property = ScreenPixelsHeight*/
const Value V17098_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17098_4 = {S54, 0, &V17098_4_1224, &P17098_5};	/*Property = ScreenPixelsWidth*/
const Profile P17098 = {"17098", &P17098_4};	/*Profile ID - 17098*/
const Value V19886_1_1 = {S2, NULL};	/*Value = True*/
const Property P19886_1 = {S50, 0, &V19886_1_1, NULL};	/*Property = IsMobile*/
const Value V19886_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P19886_5 = {S52, 0, &V19886_5_1222, &P19886_1};	/*Property = ScreenPixelsHeight*/
const Value V19886_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P19886_4 = {S54, 0, &V19886_4_1224, &P19886_5};	/*Property = ScreenPixelsWidth*/
const Profile P19886 = {"19886", &P19886_4};	/*Profile ID - 19886*/
const Value V20377_1_2 = {S0, NULL};	/*Value = False*/
const Property P20377_1 = {S50, 0, &V20377_1_2, NULL};	/*Property = IsMobile*/
const Value V20377_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20377_5 = {S52, 0, &V20377_5_37, &P20377_1};	/*Property = ScreenPixelsHeight*/
const Value V20377_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20377_4 = {S54, 0, &V20377_4_36, &P20377_5};	/*Property = ScreenPixelsWidth*/
const Profile P20377 = {"20377", &P20377_4};	/*Profile ID - 20377*/
const Value V16519_1_1 = {S2, NULL};	/*Value = True*/
const Property P16519_1 = {S50, 0, &V16519_1_1, NULL};	/*Property = IsMobile*/
const Value V16519_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16519_5 = {S52, 0, &V16519_5_1222, &P16519_1};	/*Property = ScreenPixelsHeight*/
const Value V16519_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16519_4 = {S54, 0, &V16519_4_1279, &P16519_5};	/*Property = ScreenPixelsWidth*/
const Profile P16519 = {"16519", &P16519_4};	/*Profile ID - 16519*/
const Value V18518_1_1 = {S2, NULL};	/*Value = True*/
const Property P18518_1 = {S50, 0, &V18518_1_1, NULL};	/*Property = IsMobile*/
const Value V18518_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18518_5 = {S52, 0, &V18518_5_1222, &P18518_1};	/*Property = ScreenPixelsHeight*/
const Value V18518_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18518_4 = {S54, 0, &V18518_4_1224, &P18518_5};	/*Property = ScreenPixelsWidth*/
const Profile P18518 = {"18518", &P18518_4};	/*Profile ID - 18518*/
const Value V16488_1_1 = {S2, NULL};	/*Value = True*/
const Property P16488_1 = {S50, 0, &V16488_1_1, NULL};	/*Property = IsMobile*/
const Value V16488_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P16488_5 = {S52, 0, &V16488_5_1304, &P16488_1};	/*Property = ScreenPixelsHeight*/
const Value V16488_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16488_4 = {S54, 0, &V16488_4_1224, &P16488_5};	/*Property = ScreenPixelsWidth*/
const Profile P16488 = {"16488", &P16488_4};	/*Profile ID - 16488*/
const Value V19870_1_1 = {S2, NULL};	/*Value = True*/
const Property P19870_1 = {S50, 0, &V19870_1_1, NULL};	/*Property = IsMobile*/
const Value V19870_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19870_5 = {S52, 0, &V19870_5_9981, &P19870_1};	/*Property = ScreenPixelsHeight*/
const Value V19870_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P19870_4 = {S54, 0, &V19870_4_39098, &P19870_5};	/*Property = ScreenPixelsWidth*/
const Profile P19870 = {"19870", &P19870_4};	/*Profile ID - 19870*/
const Value V20745_1_1 = {S2, NULL};	/*Value = True*/
const Property P20745_1 = {S50, 0, &V20745_1_1, NULL};	/*Property = IsMobile*/
const Value V20745_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20745_5 = {S52, 0, &V20745_5_1227, &P20745_1};	/*Property = ScreenPixelsHeight*/
const Value V20745_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20745_4 = {S54, 0, &V20745_4_1229, &P20745_5};	/*Property = ScreenPixelsWidth*/
const Profile P20745 = {"20745", &P20745_4};	/*Profile ID - 20745*/
const Value V18675_1_1 = {S2, NULL};	/*Value = True*/
const Property P18675_1 = {S50, 0, &V18675_1_1, NULL};	/*Property = IsMobile*/
const Value V18675_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18675_5 = {S52, 0, &V18675_5_1403, &P18675_1};	/*Property = ScreenPixelsHeight*/
const Value V18675_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18675_4 = {S54, 0, &V18675_4_1404, &P18675_5};	/*Property = ScreenPixelsWidth*/
const Profile P18675 = {"18675", &P18675_4};	/*Profile ID - 18675*/
const Value V18606_1_1 = {S2, NULL};	/*Value = True*/
const Property P18606_1 = {S50, 0, &V18606_1_1, NULL};	/*Property = IsMobile*/
const Value V18606_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18606_5 = {S52, 0, &V18606_5_9981, &P18606_1};	/*Property = ScreenPixelsHeight*/
const Value V18606_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18606_4 = {S54, 0, &V18606_4_39098, &P18606_5};	/*Property = ScreenPixelsWidth*/
const Profile P18606 = {"18606", &P18606_4};	/*Profile ID - 18606*/
const Value V18605_1_1 = {S2, NULL};	/*Value = True*/
const Property P18605_1 = {S50, 0, &V18605_1_1, NULL};	/*Property = IsMobile*/
const Value V18605_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P18605_5 = {S52, 0, &V18605_5_59138, &P18605_1};	/*Property = ScreenPixelsHeight*/
const Value V18605_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P18605_4 = {S54, 0, &V18605_4_59970, &P18605_5};	/*Property = ScreenPixelsWidth*/
const Profile P18605 = {"18605", &P18605_4};	/*Profile ID - 18605*/
const Value V18623_1_1 = {S2, NULL};	/*Value = True*/
const Property P18623_1 = {S50, 0, &V18623_1_1, NULL};	/*Property = IsMobile*/
const Value V18623_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18623_5 = {S52, 0, &V18623_5_1227, &P18623_1};	/*Property = ScreenPixelsHeight*/
const Value V18623_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18623_4 = {S54, 0, &V18623_4_1229, &P18623_5};	/*Property = ScreenPixelsWidth*/
const Profile P18623 = {"18623", &P18623_4};	/*Profile ID - 18623*/
const Value V22486_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22486_279 = {S1, 0, &V22486_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22486_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22486_282 = {S3, 0, &V22486_282_57154, &P22486_279};	/*Property = BlobBuilder*/
const Value V22486_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22486_350 = {S4, 0, &V22486_350_57444, &P22486_282};	/*Property = Canvas*/
const Value V22486_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22486_283 = {S5, 0, &V22486_283_57159, &P22486_350};	/*Property = CssBackground*/
const Value V22486_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22486_284 = {S6, 0, &V22486_284_57163, &P22486_283};	/*Property = CssBorderImage*/
const Value V22486_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22486_285 = {S7, 0, &V22486_285_57166, &P22486_284};	/*Property = CssCanvas*/
const Value V22486_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22486_286 = {S8, 0, &V22486_286_57168, &P22486_285};	/*Property = CssColor*/
const Value V22486_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22486_237 = {S9, 0, &V22486_237_55784, &P22486_286};	/*Property = CssColumn*/
const Value V22486_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22486_287 = {S10, 0, &V22486_287_57172, &P22486_237};	/*Property = CssFlexbox*/
const Value V22486_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22486_288 = {S11, 0, &V22486_288_57174, &P22486_287};	/*Property = CssFont*/
const Value V22486_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22486_289 = {S12, 0, &V22486_289_57759, &P22486_288};	/*Property = CssImages*/
const Value V22486_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22486_291 = {S13, 0, &V22486_291_57180, &P22486_289};	/*Property = CssMediaQueries*/
const Value V22486_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22486_292 = {S14, 0, &V22486_292_57183, &P22486_291};	/*Property = CssMinMax*/
const Value V22486_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22486_293 = {S15, 0, &V22486_293_57187, &P22486_292};	/*Property = CssOverflow*/
const Value V22486_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22486_294 = {S16, 0, &V22486_294_57189, &P22486_293};	/*Property = CssPosition*/
const Value V22486_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22486_295 = {S17, 0, &V22486_295_57192, &P22486_294};	/*Property = CssText*/
const Value V22486_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22486_297 = {S18, 0, &V22486_297_57195, &P22486_295};	/*Property = CssTransforms*/
const Value V22486_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22486_241 = {S19, 0, &V22486_241_55799, &P22486_297};	/*Property = CssTransitions*/
const Value V22486_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22486_298 = {S20, 0, &V22486_298_57206, &P22486_241};	/*Property = CssUI*/
const Value V22486_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22486_300 = {S21, 0, &V22486_300_57215, &P22486_298};	/*Property = DataSet*/
const Value V22486_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22486_301 = {S22, 0, &V22486_301_57218, &P22486_300};	/*Property = DataUrl*/
const Value V22486_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22486_302 = {S23, 0, &V22486_302_57221, &P22486_301};	/*Property = DeviceOrientation*/
const Value V22486_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22486_304 = {S24, 0, &V22486_304_57227, &P22486_302};	/*Property = FileReader*/
const Value V22486_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22486_305 = {S25, 0, &V22486_305_57231, &P22486_304};	/*Property = FileSaver*/
const Value V22486_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22486_306 = {S26, 0, &V22486_306_57237, &P22486_305};	/*Property = FileWriter*/
const Value V22486_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22486_307 = {S27, 0, &V22486_307_57239, &P22486_306};	/*Property = FormData*/
const Value V22486_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22486_309 = {S28, 0, &V22486_309_57246, &P22486_307};	/*Property = Fullscreen*/
const Value V22486_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22486_310 = {S29, 0, &V22486_310_57248, &P22486_309};	/*Property = GeoLocation*/
const Value V22486_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22486_312 = {S30, 0, &V22486_312_57254, &P22486_310};	/*Property = History*/
const Value V22486_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22486_314 = {S31, 0, &V22486_314_57261, &P22486_312};	/*Property = Html5*/
const Value V22486_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22486_313 = {S32, 0, &V22486_313_57258, &P22486_314};	/*Property = Html-Media-Capture*/
const Value V22486_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22486_315 = {S33, 0, &V22486_315_57264, &P22486_313};	/*Property = Iframe*/
const Value V22486_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22486_316 = {S34, 0, &V22486_316_57266, &P22486_315};	/*Property = IndexedDB*/
const Value V22486_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22486_317 = {S35, 0, &V22486_317_57269, &P22486_316};	/*Property = Json*/
const Value V22486_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22486_15 = {S37, 0, &V22486_15_29, &P22486_317};	/*Property = LayoutEngine*/
const Value V22486_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22486_344 = {S38, 0, &V22486_344_57272, &P22486_15};	/*Property = Masking*/
const Value V22486_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22486_320 = {S39, 0, &V22486_320_57290, &P22486_344};	/*Property = PostMessage*/
const Value V22486_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22486_321 = {S40, 0, &V22486_321_57293, &P22486_320};	/*Property = Progress*/
const Value V22486_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22486_322 = {S41, 0, &V22486_322_57296, &P22486_321};	/*Property = Prompts*/
const Value V22486_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22486_323 = {S42, 0, &V22486_323_57299, &P22486_322};	/*Property = Selector*/
const Value V22486_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22486_324 = {S43, 0, &V22486_324_57305, &P22486_323};	/*Property = Svg*/
const Value V22486_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22486_327 = {S44, 0, &V22486_327_57315, &P22486_324};	/*Property = TouchEvents*/
const Value V22486_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22486_328 = {S45, 0, &V22486_328_57318, &P22486_327};	/*Property = Track*/
const Value V22486_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22486_329 = {S46, 0, &V22486_329_57323, &P22486_328};	/*Property = Video*/
const Value V22486_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22486_330 = {S47, 0, &V22486_330_57326, &P22486_329};	/*Property = Viewport*/
const Value V22486_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22486_332 = {S48, 0, &V22486_332_57338, &P22486_330};	/*Property = WebWorkers*/
const Value V22486_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22486_333 = {S49, 0, &V22486_333_57341, &P22486_332};	/*Property = Xhr2*/
const Profile P22486 = {"22486", &P22486_333};	/*Profile ID - 22486*/
const Value V16569_1_1 = {S2, NULL};	/*Value = True*/
const Property P16569_1 = {S50, 0, &V16569_1_1, NULL};	/*Property = IsMobile*/
const Value V16569_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16569_5 = {S52, 0, &V16569_5_1222, &P16569_1};	/*Property = ScreenPixelsHeight*/
const Value V16569_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16569_4 = {S54, 0, &V16569_4_1224, &P16569_5};	/*Property = ScreenPixelsWidth*/
const Profile P16569 = {"16569", &P16569_4};	/*Profile ID - 16569*/
const Value V23649_1_2 = {S0, NULL};	/*Value = False*/
const Property P23649_1 = {S50, 0, &V23649_1_2, NULL};	/*Property = IsMobile*/
const Value V23649_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23649_5 = {S52, 0, &V23649_5_60417, &P23649_1};	/*Property = ScreenPixelsHeight*/
const Value V23649_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23649_4 = {S54, 0, &V23649_4_57095, &P23649_5};	/*Property = ScreenPixelsWidth*/
const Profile P23649 = {"23649", &P23649_4};	/*Profile ID - 23649*/
const Value V22335_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22335_279 = {S1, 0, &V22335_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22335_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22335_282 = {S3, 0, &V22335_282_57153, &P22335_279};	/*Property = BlobBuilder*/
const Value V22335_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22335_350 = {S4, 0, &V22335_350_57444, &P22335_282};	/*Property = Canvas*/
const Value V22335_283_57160 = {S0, NULL};	/*Value = False*/
const Property P22335_283 = {S5, 0, &V22335_283_57160, &P22335_350};	/*Property = CssBackground*/
const Value V22335_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22335_284 = {S6, 0, &V22335_284_57163, &P22335_283};	/*Property = CssBorderImage*/
const Value V22335_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22335_285 = {S7, 0, &V22335_285_57166, &P22335_284};	/*Property = CssCanvas*/
const Value V22335_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22335_286 = {S8, 0, &V22335_286_57168, &P22335_285};	/*Property = CssColor*/
const Value V22335_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22335_237 = {S9, 0, &V22335_237_55784, &P22335_286};	/*Property = CssColumn*/
const Value V22335_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22335_287 = {S10, 0, &V22335_287_57172, &P22335_237};	/*Property = CssFlexbox*/
const Value V22335_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22335_288 = {S11, 0, &V22335_288_57175, &P22335_287};	/*Property = CssFont*/
const Value V22335_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22335_289 = {S12, 0, &V22335_289_86899, &P22335_288};	/*Property = CssImages*/
const Value V22335_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22335_291 = {S13, 0, &V22335_291_57181, &P22335_289};	/*Property = CssMediaQueries*/
const Value V22335_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22335_292 = {S14, 0, &V22335_292_57183, &P22335_291};	/*Property = CssMinMax*/
const Value V22335_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22335_293 = {S15, 0, &V22335_293_57187, &P22335_292};	/*Property = CssOverflow*/
const Value V22335_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22335_294 = {S16, 0, &V22335_294_57189, &P22335_293};	/*Property = CssPosition*/
const Value V22335_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22335_295 = {S17, 0, &V22335_295_57193, &P22335_294};	/*Property = CssText*/
const Value V22335_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22335_297 = {S18, 0, &V22335_297_57195, &P22335_295};	/*Property = CssTransforms*/
const Value V22335_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22335_241 = {S19, 0, &V22335_241_55799, &P22335_297};	/*Property = CssTransitions*/
const Value V22335_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22335_298 = {S20, 0, &V22335_298_57206, &P22335_241};	/*Property = CssUI*/
const Value V22335_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22335_300 = {S21, 0, &V22335_300_57215, &P22335_298};	/*Property = DataSet*/
const Value V22335_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22335_301 = {S22, 0, &V22335_301_57218, &P22335_300};	/*Property = DataUrl*/
const Value V22335_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22335_302 = {S23, 0, &V22335_302_57221, &P22335_301};	/*Property = DeviceOrientation*/
const Value V22335_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22335_304 = {S24, 0, &V22335_304_57227, &P22335_302};	/*Property = FileReader*/
const Value V22335_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22335_305 = {S25, 0, &V22335_305_57231, &P22335_304};	/*Property = FileSaver*/
const Value V22335_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22335_306 = {S26, 0, &V22335_306_57237, &P22335_305};	/*Property = FileWriter*/
const Value V22335_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22335_307 = {S27, 0, &V22335_307_57239, &P22335_306};	/*Property = FormData*/
const Value V22335_309_57245 = {S2, NULL};	/*Value = True*/
const Property P22335_309 = {S28, 0, &V22335_309_57245, &P22335_307};	/*Property = Fullscreen*/
const Value V22335_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22335_310 = {S29, 0, &V22335_310_57248, &P22335_309};	/*Property = GeoLocation*/
const Value V22335_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22335_312 = {S30, 0, &V22335_312_57254, &P22335_310};	/*Property = History*/
const Value V22335_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22335_314 = {S31, 0, &V22335_314_57261, &P22335_312};	/*Property = Html5*/
const Value V22335_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22335_313 = {S32, 0, &V22335_313_57258, &P22335_314};	/*Property = Html-Media-Capture*/
const Value V22335_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22335_315 = {S33, 0, &V22335_315_57264, &P22335_313};	/*Property = Iframe*/
const Value V22335_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22335_316 = {S34, 0, &V22335_316_57266, &P22335_315};	/*Property = IndexedDB*/
const Value V22335_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22335_317 = {S35, 0, &V22335_317_57269, &P22335_316};	/*Property = Json*/
const Value V22335_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22335_15 = {S37, 0, &V22335_15_29, &P22335_317};	/*Property = LayoutEngine*/
const Value V22335_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22335_344 = {S38, 0, &V22335_344_57272, &P22335_15};	/*Property = Masking*/
const Value V22335_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22335_320 = {S39, 0, &V22335_320_57290, &P22335_344};	/*Property = PostMessage*/
const Value V22335_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22335_321 = {S40, 0, &V22335_321_57293, &P22335_320};	/*Property = Progress*/
const Value V22335_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22335_322 = {S41, 0, &V22335_322_57296, &P22335_321};	/*Property = Prompts*/
const Value V22335_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22335_323 = {S42, 0, &V22335_323_57299, &P22335_322};	/*Property = Selector*/
const Value V22335_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22335_324 = {S43, 0, &V22335_324_57305, &P22335_323};	/*Property = Svg*/
const Value V22335_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22335_327 = {S44, 0, &V22335_327_57315, &P22335_324};	/*Property = TouchEvents*/
const Value V22335_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22335_328 = {S45, 0, &V22335_328_57318, &P22335_327};	/*Property = Track*/
const Value V22335_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22335_329 = {S46, 0, &V22335_329_57323, &P22335_328};	/*Property = Video*/
const Value V22335_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22335_330 = {S47, 0, &V22335_330_57326, &P22335_329};	/*Property = Viewport*/
const Value V22335_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22335_332 = {S48, 0, &V22335_332_57339, &P22335_330};	/*Property = WebWorkers*/
const Value V22335_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22335_333 = {S49, 0, &V22335_333_57342, &P22335_332};	/*Property = Xhr2*/
const Profile P22335 = {"22335", &P22335_333};	/*Profile ID - 22335*/
const Value V23651_1_2 = {S0, NULL};	/*Value = False*/
const Property P23651_1 = {S50, 0, &V23651_1_2, NULL};	/*Property = IsMobile*/
const Value V23651_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23651_5 = {S52, 0, &V23651_5_37569, &P23651_1};	/*Property = ScreenPixelsHeight*/
const Value V23651_4_53077 = {S129, NULL};	/*Value = 1366*/
const Property P23651_4 = {S54, 0, &V23651_4_53077, &P23651_5};	/*Property = ScreenPixelsWidth*/
const Profile P23651 = {"23651", &P23651_4};	/*Profile ID - 23651*/
const Value V23629_1_2 = {S0, NULL};	/*Value = False*/
const Property P23629_1 = {S50, 0, &V23629_1_2, NULL};	/*Property = IsMobile*/
const Value V23629_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23629_5 = {S52, 0, &V23629_5_60417, &P23629_1};	/*Property = ScreenPixelsHeight*/
const Value V23629_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23629_4 = {S54, 0, &V23629_4_57095, &P23629_5};	/*Property = ScreenPixelsWidth*/
const Profile P23629 = {"23629", &P23629_4};	/*Profile ID - 23629*/
const Value V21956_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21956_279 = {S1, 0, &V21956_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21956_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21956_282 = {S3, 0, &V21956_282_57154, &P21956_279};	/*Property = BlobBuilder*/
const Value V21956_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21956_350 = {S4, 0, &V21956_350_57444, &P21956_282};	/*Property = Canvas*/
const Value V21956_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21956_283 = {S5, 0, &V21956_283_57159, &P21956_350};	/*Property = CssBackground*/
const Value V21956_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21956_284 = {S6, 0, &V21956_284_57163, &P21956_283};	/*Property = CssBorderImage*/
const Value V21956_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21956_285 = {S7, 0, &V21956_285_57166, &P21956_284};	/*Property = CssCanvas*/
const Value V21956_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21956_286 = {S8, 0, &V21956_286_57168, &P21956_285};	/*Property = CssColor*/
const Value V21956_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21956_237 = {S9, 0, &V21956_237_55784, &P21956_286};	/*Property = CssColumn*/
const Value V21956_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21956_287 = {S10, 0, &V21956_287_57172, &P21956_237};	/*Property = CssFlexbox*/
const Value V21956_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21956_288 = {S11, 0, &V21956_288_57175, &P21956_287};	/*Property = CssFont*/
const Value V21956_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21956_289 = {S12, 0, &V21956_289_57759, &P21956_288};	/*Property = CssImages*/
const Value V21956_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21956_291 = {S13, 0, &V21956_291_57181, &P21956_289};	/*Property = CssMediaQueries*/
const Value V21956_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21956_292 = {S14, 0, &V21956_292_57183, &P21956_291};	/*Property = CssMinMax*/
const Value V21956_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21956_293 = {S15, 0, &V21956_293_57187, &P21956_292};	/*Property = CssOverflow*/
const Value V21956_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21956_294 = {S16, 0, &V21956_294_57190, &P21956_293};	/*Property = CssPosition*/
const Value V21956_295_57193 = {S0, NULL};	/*Value = False*/
const Property P21956_295 = {S17, 0, &V21956_295_57193, &P21956_294};	/*Property = CssText*/
const Value V21956_297_57196 = {S0, NULL};	/*Value = False*/
const Property P21956_297 = {S18, 0, &V21956_297_57196, &P21956_295};	/*Property = CssTransforms*/
const Value V21956_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21956_241 = {S19, 0, &V21956_241_55799, &P21956_297};	/*Property = CssTransitions*/
const Value V21956_298_57207 = {S0, NULL};	/*Value = False*/
const Property P21956_298 = {S20, 0, &V21956_298_57207, &P21956_241};	/*Property = CssUI*/
const Value V21956_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21956_300 = {S21, 0, &V21956_300_57216, &P21956_298};	/*Property = DataSet*/
const Value V21956_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21956_301 = {S22, 0, &V21956_301_57218, &P21956_300};	/*Property = DataUrl*/
const Value V21956_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21956_302 = {S23, 0, &V21956_302_57222, &P21956_301};	/*Property = DeviceOrientation*/
const Value V21956_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21956_304 = {S24, 0, &V21956_304_57228, &P21956_302};	/*Property = FileReader*/
const Value V21956_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21956_305 = {S25, 0, &V21956_305_57231, &P21956_304};	/*Property = FileSaver*/
const Value V21956_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21956_306 = {S26, 0, &V21956_306_57237, &P21956_305};	/*Property = FileWriter*/
const Value V21956_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21956_307 = {S27, 0, &V21956_307_57240, &P21956_306};	/*Property = FormData*/
const Value V21956_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21956_309 = {S28, 0, &V21956_309_57246, &P21956_307};	/*Property = Fullscreen*/
const Value V21956_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21956_310 = {S29, 0, &V21956_310_57248, &P21956_309};	/*Property = GeoLocation*/
const Value V21956_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21956_312 = {S30, 0, &V21956_312_57255, &P21956_310};	/*Property = History*/
const Value V21956_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21956_314 = {S31, 0, &V21956_314_57261, &P21956_312};	/*Property = Html5*/
const Value V21956_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21956_313 = {S32, 0, &V21956_313_57258, &P21956_314};	/*Property = Html-Media-Capture*/
const Value V21956_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21956_315 = {S33, 0, &V21956_315_57264, &P21956_313};	/*Property = Iframe*/
const Value V21956_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21956_316 = {S34, 0, &V21956_316_57267, &P21956_315};	/*Property = IndexedDB*/
const Value V21956_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21956_317 = {S35, 0, &V21956_317_57269, &P21956_316};	/*Property = Json*/
const Value V21956_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P21956_15 = {S37, 0, &V21956_15_29, &P21956_317};	/*Property = LayoutEngine*/
const Value V21956_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21956_344 = {S38, 0, &V21956_344_57272, &P21956_15};	/*Property = Masking*/
const Value V21956_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21956_320 = {S39, 0, &V21956_320_57290, &P21956_344};	/*Property = PostMessage*/
const Value V21956_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21956_321 = {S40, 0, &V21956_321_57293, &P21956_320};	/*Property = Progress*/
const Value V21956_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21956_322 = {S41, 0, &V21956_322_57296, &P21956_321};	/*Property = Prompts*/
const Value V21956_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21956_323 = {S42, 0, &V21956_323_57299, &P21956_322};	/*Property = Selector*/
const Value V21956_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21956_324 = {S43, 0, &V21956_324_57306, &P21956_323};	/*Property = Svg*/
const Value V21956_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21956_327 = {S44, 0, &V21956_327_57315, &P21956_324};	/*Property = TouchEvents*/
const Value V21956_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21956_328 = {S45, 0, &V21956_328_57318, &P21956_327};	/*Property = Track*/
const Value V21956_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21956_329 = {S46, 0, &V21956_329_57323, &P21956_328};	/*Property = Video*/
const Value V21956_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21956_330 = {S47, 0, &V21956_330_57326, &P21956_329};	/*Property = Viewport*/
const Value V21956_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21956_332 = {S48, 0, &V21956_332_57339, &P21956_330};	/*Property = WebWorkers*/
const Value V21956_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21956_333 = {S49, 0, &V21956_333_57342, &P21956_332};	/*Property = Xhr2*/
const Profile P21956 = {"21956", &P21956_333};	/*Profile ID - 21956*/
const Value V20156_1_1 = {S2, NULL};	/*Value = True*/
const Property P20156_1 = {S50, 0, &V20156_1_1, NULL};	/*Property = IsMobile*/
const Value V20156_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20156_5 = {S52, 0, &V20156_5_9981, &P20156_1};	/*Property = ScreenPixelsHeight*/
const Value V20156_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20156_4 = {S54, 0, &V20156_4_39098, &P20156_5};	/*Property = ScreenPixelsWidth*/
const Profile P20156 = {"20156", &P20156_4};	/*Profile ID - 20156*/
const Value V21395_1_1 = {S2, NULL};	/*Value = True*/
const Property P21395_1 = {S50, 0, &V21395_1_1, NULL};	/*Property = IsMobile*/
const Value V21395_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21395_5 = {S52, 0, &V21395_5_1277, &P21395_1};	/*Property = ScreenPixelsHeight*/
const Value V21395_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21395_4 = {S54, 0, &V21395_4_1279, &P21395_5};	/*Property = ScreenPixelsWidth*/
const Profile P21395 = {"21395", &P21395_4};	/*Profile ID - 21395*/
const Value V23671_1_2 = {S0, NULL};	/*Value = False*/
const Property P23671_1 = {S50, 0, &V23671_1_2, NULL};	/*Property = IsMobile*/
const Value V23671_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23671_5 = {S52, 0, &V23671_5_60417, &P23671_1};	/*Property = ScreenPixelsHeight*/
const Value V23671_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23671_4 = {S54, 0, &V23671_4_57095, &P23671_5};	/*Property = ScreenPixelsWidth*/
const Profile P23671 = {"23671", &P23671_4};	/*Profile ID - 23671*/
const Value V19885_1_1 = {S2, NULL};	/*Value = True*/
const Property P19885_1 = {S50, 0, &V19885_1_1, NULL};	/*Property = IsMobile*/
const Value V19885_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P19885_5 = {S52, 0, &V19885_5_1293, &P19885_1};	/*Property = ScreenPixelsHeight*/
const Value V19885_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P19885_4 = {S54, 0, &V19885_4_1296, &P19885_5};	/*Property = ScreenPixelsWidth*/
const Profile P19885 = {"19885", &P19885_4};	/*Profile ID - 19885*/
const Value V20036_1_1 = {S2, NULL};	/*Value = True*/
const Property P20036_1 = {S50, 0, &V20036_1_1, NULL};	/*Property = IsMobile*/
const Value V20036_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20036_5 = {S52, 0, &V20036_5_1222, &P20036_1};	/*Property = ScreenPixelsHeight*/
const Value V20036_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20036_4 = {S54, 0, &V20036_4_1224, &P20036_5};	/*Property = ScreenPixelsWidth*/
const Profile P20036 = {"20036", &P20036_4};	/*Profile ID - 20036*/
const Value V21211_1_1 = {S2, NULL};	/*Value = True*/
const Property P21211_1 = {S50, 0, &V21211_1_1, NULL};	/*Property = IsMobile*/
const Value V21211_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21211_5 = {S52, 0, &V21211_5_1227, &P21211_1};	/*Property = ScreenPixelsHeight*/
const Value V21211_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21211_4 = {S54, 0, &V21211_4_1229, &P21211_5};	/*Property = ScreenPixelsWidth*/
const Profile P21211 = {"21211", &P21211_4};	/*Profile ID - 21211*/
const Value V20793_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_279 = {S1, 0, &V20793_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20793_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_282 = {S3, 0, &V20793_282_57155, &P20793_279};	/*Property = BlobBuilder*/
const Value V20793_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_350 = {S4, 0, &V20793_350_57446, &P20793_282};	/*Property = Canvas*/
const Value V20793_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_283 = {S5, 0, &V20793_283_57161, &P20793_350};	/*Property = CssBackground*/
const Value V20793_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_284 = {S6, 0, &V20793_284_57164, &P20793_283};	/*Property = CssBorderImage*/
const Value V20793_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_285 = {S7, 0, &V20793_285_57167, &P20793_284};	/*Property = CssCanvas*/
const Value V20793_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_286 = {S8, 0, &V20793_286_57170, &P20793_285};	/*Property = CssColor*/
const Value V20793_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_237 = {S9, 0, &V20793_237_55786, &P20793_286};	/*Property = CssColumn*/
const Value V20793_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_287 = {S10, 0, &V20793_287_57173, &P20793_237};	/*Property = CssFlexbox*/
const Value V20793_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_288 = {S11, 0, &V20793_288_57176, &P20793_287};	/*Property = CssFont*/
const Value V20793_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_289 = {S12, 0, &V20793_289_86899, &P20793_288};	/*Property = CssImages*/
const Value V20793_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_291 = {S13, 0, &V20793_291_57182, &P20793_289};	/*Property = CssMediaQueries*/
const Value V20793_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_292 = {S14, 0, &V20793_292_57185, &P20793_291};	/*Property = CssMinMax*/
const Value V20793_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_293 = {S15, 0, &V20793_293_57188, &P20793_292};	/*Property = CssOverflow*/
const Value V20793_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_294 = {S16, 0, &V20793_294_57191, &P20793_293};	/*Property = CssPosition*/
const Value V20793_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_295 = {S17, 0, &V20793_295_57194, &P20793_294};	/*Property = CssText*/
const Value V20793_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_297 = {S18, 0, &V20793_297_57197, &P20793_295};	/*Property = CssTransforms*/
const Value V20793_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_241 = {S19, 0, &V20793_241_55801, &P20793_297};	/*Property = CssTransitions*/
const Value V20793_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_298 = {S20, 0, &V20793_298_57208, &P20793_241};	/*Property = CssUI*/
const Value V20793_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_300 = {S21, 0, &V20793_300_57217, &P20793_298};	/*Property = DataSet*/
const Value V20793_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_301 = {S22, 0, &V20793_301_57220, &P20793_300};	/*Property = DataUrl*/
const Value V20793_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_302 = {S23, 0, &V20793_302_57223, &P20793_301};	/*Property = DeviceOrientation*/
const Value V20793_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_304 = {S24, 0, &V20793_304_57229, &P20793_302};	/*Property = FileReader*/
const Value V20793_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_305 = {S25, 0, &V20793_305_57232, &P20793_304};	/*Property = FileSaver*/
const Value V20793_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_306 = {S26, 0, &V20793_306_57238, &P20793_305};	/*Property = FileWriter*/
const Value V20793_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_307 = {S27, 0, &V20793_307_57241, &P20793_306};	/*Property = FormData*/
const Value V20793_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_309 = {S28, 0, &V20793_309_57247, &P20793_307};	/*Property = Fullscreen*/
const Value V20793_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_310 = {S29, 0, &V20793_310_57250, &P20793_309};	/*Property = GeoLocation*/
const Value V20793_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_312 = {S30, 0, &V20793_312_57256, &P20793_310};	/*Property = History*/
const Value V20793_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_314 = {S31, 0, &V20793_314_57262, &P20793_312};	/*Property = Html5*/
const Value V20793_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_313 = {S32, 0, &V20793_313_57259, &P20793_314};	/*Property = Html-Media-Capture*/
const Value V20793_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_315 = {S33, 0, &V20793_315_57265, &P20793_313};	/*Property = Iframe*/
const Value V20793_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_316 = {S34, 0, &V20793_316_57268, &P20793_315};	/*Property = IndexedDB*/
const Value V20793_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_317 = {S35, 0, &V20793_317_57271, &P20793_316};	/*Property = Json*/
const Value V20793_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P20793_15 = {S37, 0, &V20793_15_90837, &P20793_317};	/*Property = LayoutEngine*/
const Value V20793_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_344 = {S38, 0, &V20793_344_57274, &P20793_15};	/*Property = Masking*/
const Value V20793_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_320 = {S39, 0, &V20793_320_57292, &P20793_344};	/*Property = PostMessage*/
const Value V20793_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_321 = {S40, 0, &V20793_321_57295, &P20793_320};	/*Property = Progress*/
const Value V20793_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_322 = {S41, 0, &V20793_322_57298, &P20793_321};	/*Property = Prompts*/
const Value V20793_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_323 = {S42, 0, &V20793_323_57301, &P20793_322};	/*Property = Selector*/
const Value V20793_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_324 = {S43, 0, &V20793_324_57307, &P20793_323};	/*Property = Svg*/
const Value V20793_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_327 = {S44, 0, &V20793_327_57316, &P20793_324};	/*Property = TouchEvents*/
const Value V20793_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_328 = {S45, 0, &V20793_328_57319, &P20793_327};	/*Property = Track*/
const Value V20793_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_329 = {S46, 0, &V20793_329_57325, &P20793_328};	/*Property = Video*/
const Value V20793_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_330 = {S47, 0, &V20793_330_57328, &P20793_329};	/*Property = Viewport*/
const Value V20793_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_332 = {S48, 0, &V20793_332_57340, &P20793_330};	/*Property = WebWorkers*/
const Value V20793_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20793_333 = {S49, 0, &V20793_333_57343, &P20793_332};	/*Property = Xhr2*/
const Profile P20793 = {"20793", &P20793_333};	/*Profile ID - 20793*/
const Value V21756_1_1 = {S2, NULL};	/*Value = True*/
const Property P21756_1 = {S50, 0, &V21756_1_1, NULL};	/*Property = IsMobile*/
const Value V21756_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21756_5 = {S52, 0, &V21756_5_1227, &P21756_1};	/*Property = ScreenPixelsHeight*/
const Value V21756_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21756_4 = {S54, 0, &V21756_4_1229, &P21756_5};	/*Property = ScreenPixelsWidth*/
const Profile P21756 = {"21756", &P21756_4};	/*Profile ID - 21756*/
const Value V21032_1_1 = {S2, NULL};	/*Value = True*/
const Property P21032_1 = {S50, 0, &V21032_1_1, NULL};	/*Property = IsMobile*/
const Value V21032_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21032_5 = {S52, 0, &V21032_5_37, &P21032_1};	/*Property = ScreenPixelsHeight*/
const Value V21032_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21032_4 = {S54, 0, &V21032_4_36, &P21032_5};	/*Property = ScreenPixelsWidth*/
const Profile P21032 = {"21032", &P21032_4};	/*Profile ID - 21032*/
const Profile P18615 = {"18615", NULL};	/*Profile ID - 18615*/
const Value V23805_1_1 = {S2, NULL};	/*Value = True*/
const Property P23805_1 = {S50, 0, &V23805_1_1, NULL};	/*Property = IsMobile*/
const Value V23805_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23805_5 = {S52, 0, &V23805_5_1222, &P23805_1};	/*Property = ScreenPixelsHeight*/
const Value V23805_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23805_4 = {S54, 0, &V23805_4_1224, &P23805_5};	/*Property = ScreenPixelsWidth*/
const Profile P23805 = {"23805", &P23805_4};	/*Profile ID - 23805*/
const Value V20042_1_1 = {S2, NULL};	/*Value = True*/
const Property P20042_1 = {S50, 0, &V20042_1_1, NULL};	/*Property = IsMobile*/
const Value V20042_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20042_5 = {S52, 0, &V20042_5_1222, &P20042_1};	/*Property = ScreenPixelsHeight*/
const Value V20042_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20042_4 = {S54, 0, &V20042_4_1224, &P20042_5};	/*Property = ScreenPixelsWidth*/
const Profile P20042 = {"20042", &P20042_4};	/*Profile ID - 20042*/
const Value V21312_1_1 = {S2, NULL};	/*Value = True*/
const Property P21312_1 = {S50, 0, &V21312_1_1, NULL};	/*Property = IsMobile*/
const Value V21312_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21312_5 = {S52, 0, &V21312_5_1227, &P21312_1};	/*Property = ScreenPixelsHeight*/
const Value V21312_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21312_4 = {S54, 0, &V21312_4_1229, &P21312_5};	/*Property = ScreenPixelsWidth*/
const Profile P21312 = {"21312", &P21312_4};	/*Profile ID - 21312*/
const Value V21149_1_1 = {S2, NULL};	/*Value = True*/
const Property P21149_1 = {S50, 0, &V21149_1_1, NULL};	/*Property = IsMobile*/
const Value V21149_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21149_5 = {S52, 0, &V21149_5_1227, &P21149_1};	/*Property = ScreenPixelsHeight*/
const Value V21149_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21149_4 = {S54, 0, &V21149_4_1229, &P21149_5};	/*Property = ScreenPixelsWidth*/
const Profile P21149 = {"21149", &P21149_4};	/*Profile ID - 21149*/
const Value V21535_1_1 = {S2, NULL};	/*Value = True*/
const Property P21535_1 = {S50, 0, &V21535_1_1, NULL};	/*Property = IsMobile*/
const Value V21535_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21535_5 = {S52, 0, &V21535_5_1227, &P21535_1};	/*Property = ScreenPixelsHeight*/
const Value V21535_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21535_4 = {S54, 0, &V21535_4_1229, &P21535_5};	/*Property = ScreenPixelsWidth*/
const Profile P21535 = {"21535", &P21535_4};	/*Profile ID - 21535*/
const Value V21725_1_1 = {S2, NULL};	/*Value = True*/
const Property P21725_1 = {S50, 0, &V21725_1_1, NULL};	/*Property = IsMobile*/
const Value V21725_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P21725_5 = {S52, 0, &V21725_5_37569, &P21725_1};	/*Property = ScreenPixelsHeight*/
const Value V21725_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21725_4 = {S54, 0, &V21725_4_5645, &P21725_5};	/*Property = ScreenPixelsWidth*/
const Profile P21725 = {"21725", &P21725_4};	/*Profile ID - 21725*/
const Value V20383_1_2 = {S0, NULL};	/*Value = False*/
const Property P20383_1 = {S50, 0, &V20383_1_2, NULL};	/*Property = IsMobile*/
const Value V20383_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20383_5 = {S52, 0, &V20383_5_37, &P20383_1};	/*Property = ScreenPixelsHeight*/
const Value V20383_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20383_4 = {S54, 0, &V20383_4_36, &P20383_5};	/*Property = ScreenPixelsWidth*/
const Profile P20383 = {"20383", &P20383_4};	/*Profile ID - 20383*/
const Value V22097_1_1 = {S2, NULL};	/*Value = True*/
const Property P22097_1 = {S50, 0, &V22097_1_1, NULL};	/*Property = IsMobile*/
const Value V22097_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22097_5 = {S52, 0, &V22097_5_1227, &P22097_1};	/*Property = ScreenPixelsHeight*/
const Value V22097_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P22097_4 = {S54, 0, &V22097_4_1477, &P22097_5};	/*Property = ScreenPixelsWidth*/
const Profile P22097 = {"22097", &P22097_4};	/*Profile ID - 22097*/
const Value V20013_1_1 = {S2, NULL};	/*Value = True*/
const Property P20013_1 = {S50, 0, &V20013_1_1, NULL};	/*Property = IsMobile*/
const Value V20013_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20013_5 = {S52, 0, &V20013_5_9981, &P20013_1};	/*Property = ScreenPixelsHeight*/
const Value V20013_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20013_4 = {S54, 0, &V20013_4_37617, &P20013_5};	/*Property = ScreenPixelsWidth*/
const Profile P20013 = {"20013", &P20013_4};	/*Profile ID - 20013*/
const Value V20284_1_1 = {S2, NULL};	/*Value = True*/
const Property P20284_1 = {S50, 0, &V20284_1_1, NULL};	/*Property = IsMobile*/
const Value V20284_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20284_5 = {S52, 0, &V20284_5_1458, &P20284_1};	/*Property = ScreenPixelsHeight*/
const Value V20284_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20284_4 = {S54, 0, &V20284_4_37617, &P20284_5};	/*Property = ScreenPixelsWidth*/
const Profile P20284 = {"20284", &P20284_4};	/*Profile ID - 20284*/
const Value V20273_1_1 = {S2, NULL};	/*Value = True*/
const Property P20273_1 = {S50, 0, &V20273_1_1, NULL};	/*Property = IsMobile*/
const Value V20273_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20273_5 = {S52, 0, &V20273_5_9981, &P20273_1};	/*Property = ScreenPixelsHeight*/
const Value V20273_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20273_4 = {S54, 0, &V20273_4_39098, &P20273_5};	/*Property = ScreenPixelsWidth*/
const Profile P20273 = {"20273", &P20273_4};	/*Profile ID - 20273*/
const Value V19759_1_1 = {S2, NULL};	/*Value = True*/
const Property P19759_1 = {S50, 0, &V19759_1_1, NULL};	/*Property = IsMobile*/
const Value V19759_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19759_5 = {S52, 0, &V19759_5_9981, &P19759_1};	/*Property = ScreenPixelsHeight*/
const Value V19759_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P19759_4 = {S54, 0, &V19759_4_39098, &P19759_5};	/*Property = ScreenPixelsWidth*/
const Profile P19759 = {"19759", &P19759_4};	/*Profile ID - 19759*/
const Value V23894_1_1 = {S2, NULL};	/*Value = True*/
const Property P23894_1 = {S50, 0, &V23894_1_1, NULL};	/*Property = IsMobile*/
const Value V23894_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23894_5 = {S52, 0, &V23894_5_1403, &P23894_1};	/*Property = ScreenPixelsHeight*/
const Value V23894_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23894_4 = {S54, 0, &V23894_4_1404, &P23894_5};	/*Property = ScreenPixelsWidth*/
const Profile P23894 = {"23894", &P23894_4};	/*Profile ID - 23894*/
const Value V20980_1_1 = {S2, NULL};	/*Value = True*/
const Property P20980_1 = {S50, 0, &V20980_1_1, NULL};	/*Property = IsMobile*/
const Value V20980_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20980_5 = {S52, 0, &V20980_5_1227, &P20980_1};	/*Property = ScreenPixelsHeight*/
const Value V20980_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20980_4 = {S54, 0, &V20980_4_1229, &P20980_5};	/*Property = ScreenPixelsWidth*/
const Profile P20980 = {"20980", &P20980_4};	/*Profile ID - 20980*/
const Value V21698_1_1 = {S2, NULL};	/*Value = True*/
const Property P21698_1 = {S50, 0, &V21698_1_1, NULL};	/*Property = IsMobile*/
const Value V21698_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P21698_5 = {S52, 0, &V21698_5_1304, &P21698_1};	/*Property = ScreenPixelsHeight*/
const Value V21698_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21698_4 = {S54, 0, &V21698_4_2336, &P21698_5};	/*Property = ScreenPixelsWidth*/
const Profile P21698 = {"21698", &P21698_4};	/*Profile ID - 21698*/
const Value V19974_1_1 = {S2, NULL};	/*Value = True*/
const Property P19974_1 = {S50, 0, &V19974_1_1, NULL};	/*Property = IsMobile*/
const Value V19974_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P19974_5 = {S52, 0, &V19974_5_1293, &P19974_1};	/*Property = ScreenPixelsHeight*/
const Value V19974_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P19974_4 = {S54, 0, &V19974_4_1296, &P19974_5};	/*Property = ScreenPixelsWidth*/
const Profile P19974 = {"19974", &P19974_4};	/*Profile ID - 19974*/
const Value V18656_1_1 = {S2, NULL};	/*Value = True*/
const Property P18656_1 = {S50, 0, &V18656_1_1, NULL};	/*Property = IsMobile*/
const Value V18656_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18656_5 = {S52, 0, &V18656_5_1277, &P18656_1};	/*Property = ScreenPixelsHeight*/
const Value V18656_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18656_4 = {S54, 0, &V18656_4_1279, &P18656_5};	/*Property = ScreenPixelsWidth*/
const Profile P18656 = {"18656", &P18656_4};	/*Profile ID - 18656*/
const Value V20822_1_1 = {S2, NULL};	/*Value = True*/
const Property P20822_1 = {S50, 0, &V20822_1_1, NULL};	/*Property = IsMobile*/
const Value V20822_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20822_5 = {S52, 0, &V20822_5_9981, &P20822_1};	/*Property = ScreenPixelsHeight*/
const Value V20822_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20822_4 = {S54, 0, &V20822_4_2336, &P20822_5};	/*Property = ScreenPixelsWidth*/
const Profile P20822 = {"20822", &P20822_4};	/*Profile ID - 20822*/
const Value V24031_1_1 = {S2, NULL};	/*Value = True*/
const Property P24031_1 = {S50, 0, &V24031_1_1, NULL};	/*Property = IsMobile*/
const Value V24031_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24031_5 = {S52, 0, &V24031_5_1277, &P24031_1};	/*Property = ScreenPixelsHeight*/
const Value V24031_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P24031_4 = {S54, 0, &V24031_4_2336, &P24031_5};	/*Property = ScreenPixelsWidth*/
const Profile P24031 = {"24031", &P24031_4};	/*Profile ID - 24031*/
const Value V20358_1_1 = {S2, NULL};	/*Value = True*/
const Property P20358_1 = {S50, 0, &V20358_1_1, NULL};	/*Property = IsMobile*/
const Value V20358_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20358_5 = {S52, 0, &V20358_5_1227, &P20358_1};	/*Property = ScreenPixelsHeight*/
const Value V20358_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20358_4 = {S54, 0, &V20358_4_1229, &P20358_5};	/*Property = ScreenPixelsWidth*/
const Profile P20358 = {"20358", &P20358_4};	/*Profile ID - 20358*/
const Value V20832_1_1 = {S2, NULL};	/*Value = True*/
const Property P20832_1 = {S50, 0, &V20832_1_1, NULL};	/*Property = IsMobile*/
const Value V20832_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20832_5 = {S52, 0, &V20832_5_37, &P20832_1};	/*Property = ScreenPixelsHeight*/
const Value V20832_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20832_4 = {S54, 0, &V20832_4_36, &P20832_5};	/*Property = ScreenPixelsWidth*/
const Profile P20832 = {"20832", &P20832_4};	/*Profile ID - 20832*/
const Value V20332_1_1 = {S2, NULL};	/*Value = True*/
const Property P20332_1 = {S50, 0, &V20332_1_1, NULL};	/*Property = IsMobile*/
const Value V20332_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20332_5 = {S52, 0, &V20332_5_37, &P20332_1};	/*Property = ScreenPixelsHeight*/
const Value V20332_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20332_4 = {S54, 0, &V20332_4_36, &P20332_5};	/*Property = ScreenPixelsWidth*/
const Profile P20332 = {"20332", &P20332_4};	/*Profile ID - 20332*/
const Value V22163_1_1 = {S2, NULL};	/*Value = True*/
const Property P22163_1 = {S50, 0, &V22163_1_1, NULL};	/*Property = IsMobile*/
const Value V22163_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22163_5 = {S52, 0, &V22163_5_1227, &P22163_1};	/*Property = ScreenPixelsHeight*/
const Value V22163_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22163_4 = {S54, 0, &V22163_4_1229, &P22163_5};	/*Property = ScreenPixelsWidth*/
const Profile P22163 = {"22163", &P22163_4};	/*Profile ID - 22163*/
const Value V20018_1_1 = {S2, NULL};	/*Value = True*/
const Property P20018_1 = {S50, 0, &V20018_1_1, NULL};	/*Property = IsMobile*/
const Value V20018_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20018_5 = {S52, 0, &V20018_5_1227, &P20018_1};	/*Property = ScreenPixelsHeight*/
const Value V20018_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20018_4 = {S54, 0, &V20018_4_1229, &P20018_5};	/*Property = ScreenPixelsWidth*/
const Profile P20018 = {"20018", &P20018_4};	/*Profile ID - 20018*/
const Value V21577_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_279 = {S1, 0, &V21577_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21577_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_282 = {S3, 0, &V21577_282_57155, &P21577_279};	/*Property = BlobBuilder*/
const Value V21577_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_350 = {S4, 0, &V21577_350_57446, &P21577_282};	/*Property = Canvas*/
const Value V21577_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_283 = {S5, 0, &V21577_283_57161, &P21577_350};	/*Property = CssBackground*/
const Value V21577_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_284 = {S6, 0, &V21577_284_57164, &P21577_283};	/*Property = CssBorderImage*/
const Value V21577_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_285 = {S7, 0, &V21577_285_57167, &P21577_284};	/*Property = CssCanvas*/
const Value V21577_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_286 = {S8, 0, &V21577_286_57170, &P21577_285};	/*Property = CssColor*/
const Value V21577_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_237 = {S9, 0, &V21577_237_55786, &P21577_286};	/*Property = CssColumn*/
const Value V21577_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_287 = {S10, 0, &V21577_287_57173, &P21577_237};	/*Property = CssFlexbox*/
const Value V21577_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_288 = {S11, 0, &V21577_288_57176, &P21577_287};	/*Property = CssFont*/
const Value V21577_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_289 = {S12, 0, &V21577_289_86899, &P21577_288};	/*Property = CssImages*/
const Value V21577_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_291 = {S13, 0, &V21577_291_57182, &P21577_289};	/*Property = CssMediaQueries*/
const Value V21577_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_292 = {S14, 0, &V21577_292_57185, &P21577_291};	/*Property = CssMinMax*/
const Value V21577_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_293 = {S15, 0, &V21577_293_57188, &P21577_292};	/*Property = CssOverflow*/
const Value V21577_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_294 = {S16, 0, &V21577_294_57191, &P21577_293};	/*Property = CssPosition*/
const Value V21577_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_295 = {S17, 0, &V21577_295_57194, &P21577_294};	/*Property = CssText*/
const Value V21577_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_297 = {S18, 0, &V21577_297_57197, &P21577_295};	/*Property = CssTransforms*/
const Value V21577_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_241 = {S19, 0, &V21577_241_55801, &P21577_297};	/*Property = CssTransitions*/
const Value V21577_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_298 = {S20, 0, &V21577_298_57208, &P21577_241};	/*Property = CssUI*/
const Value V21577_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_300 = {S21, 0, &V21577_300_57217, &P21577_298};	/*Property = DataSet*/
const Value V21577_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_301 = {S22, 0, &V21577_301_57220, &P21577_300};	/*Property = DataUrl*/
const Value V21577_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_302 = {S23, 0, &V21577_302_57223, &P21577_301};	/*Property = DeviceOrientation*/
const Value V21577_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_304 = {S24, 0, &V21577_304_57229, &P21577_302};	/*Property = FileReader*/
const Value V21577_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_305 = {S25, 0, &V21577_305_57232, &P21577_304};	/*Property = FileSaver*/
const Value V21577_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_306 = {S26, 0, &V21577_306_57238, &P21577_305};	/*Property = FileWriter*/
const Value V21577_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_307 = {S27, 0, &V21577_307_57241, &P21577_306};	/*Property = FormData*/
const Value V21577_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_309 = {S28, 0, &V21577_309_57247, &P21577_307};	/*Property = Fullscreen*/
const Value V21577_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_310 = {S29, 0, &V21577_310_57250, &P21577_309};	/*Property = GeoLocation*/
const Value V21577_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_312 = {S30, 0, &V21577_312_57256, &P21577_310};	/*Property = History*/
const Value V21577_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_314 = {S31, 0, &V21577_314_57262, &P21577_312};	/*Property = Html5*/
const Value V21577_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_313 = {S32, 0, &V21577_313_57259, &P21577_314};	/*Property = Html-Media-Capture*/
const Value V21577_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_315 = {S33, 0, &V21577_315_57265, &P21577_313};	/*Property = Iframe*/
const Value V21577_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_316 = {S34, 0, &V21577_316_57268, &P21577_315};	/*Property = IndexedDB*/
const Value V21577_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_317 = {S35, 0, &V21577_317_57271, &P21577_316};	/*Property = Json*/
const Value V21577_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21577_15 = {S37, 0, &V21577_15_28, &P21577_317};	/*Property = LayoutEngine*/
const Value V21577_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_344 = {S38, 0, &V21577_344_57274, &P21577_15};	/*Property = Masking*/
const Value V21577_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_320 = {S39, 0, &V21577_320_57292, &P21577_344};	/*Property = PostMessage*/
const Value V21577_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_321 = {S40, 0, &V21577_321_57295, &P21577_320};	/*Property = Progress*/
const Value V21577_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_322 = {S41, 0, &V21577_322_57298, &P21577_321};	/*Property = Prompts*/
const Value V21577_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_323 = {S42, 0, &V21577_323_57301, &P21577_322};	/*Property = Selector*/
const Value V21577_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_324 = {S43, 0, &V21577_324_57307, &P21577_323};	/*Property = Svg*/
const Value V21577_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_327 = {S44, 0, &V21577_327_57316, &P21577_324};	/*Property = TouchEvents*/
const Value V21577_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_328 = {S45, 0, &V21577_328_57319, &P21577_327};	/*Property = Track*/
const Value V21577_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_329 = {S46, 0, &V21577_329_57325, &P21577_328};	/*Property = Video*/
const Value V21577_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_330 = {S47, 0, &V21577_330_57328, &P21577_329};	/*Property = Viewport*/
const Value V21577_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_332 = {S48, 0, &V21577_332_57340, &P21577_330};	/*Property = WebWorkers*/
const Value V21577_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21577_333 = {S49, 0, &V21577_333_57343, &P21577_332};	/*Property = Xhr2*/
const Profile P21577 = {"21577", &P21577_333};	/*Profile ID - 21577*/
const Value V21030_1_1 = {S2, NULL};	/*Value = True*/
const Property P21030_1 = {S50, 0, &V21030_1_1, NULL};	/*Property = IsMobile*/
const Value V21030_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21030_5 = {S52, 0, &V21030_5_1458, &P21030_1};	/*Property = ScreenPixelsHeight*/
const Value V21030_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21030_4 = {S54, 0, &V21030_4_37617, &P21030_5};	/*Property = ScreenPixelsWidth*/
const Profile P21030 = {"21030", &P21030_4};	/*Profile ID - 21030*/
const Value V19809_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19809_279 = {S1, 0, &V19809_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19809_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19809_282 = {S3, 0, &V19809_282_57154, &P19809_279};	/*Property = BlobBuilder*/
const Value V19809_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19809_350 = {S4, 0, &V19809_350_57444, &P19809_282};	/*Property = Canvas*/
const Value V19809_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19809_283 = {S5, 0, &V19809_283_57159, &P19809_350};	/*Property = CssBackground*/
const Value V19809_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19809_284 = {S6, 0, &V19809_284_57163, &P19809_283};	/*Property = CssBorderImage*/
const Value V19809_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19809_285 = {S7, 0, &V19809_285_57166, &P19809_284};	/*Property = CssCanvas*/
const Value V19809_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19809_286 = {S8, 0, &V19809_286_57168, &P19809_285};	/*Property = CssColor*/
const Value V19809_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19809_237 = {S9, 0, &V19809_237_55786, &P19809_286};	/*Property = CssColumn*/
const Value V19809_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19809_287 = {S10, 0, &V19809_287_57172, &P19809_237};	/*Property = CssFlexbox*/
const Value V19809_288_57174 = {S2, NULL};	/*Value = True*/
const Property P19809_288 = {S11, 0, &V19809_288_57174, &P19809_287};	/*Property = CssFont*/
const Value V19809_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19809_289 = {S12, 0, &V19809_289_57759, &P19809_288};	/*Property = CssImages*/
const Value V19809_291_57180 = {S2, NULL};	/*Value = True*/
const Property P19809_291 = {S13, 0, &V19809_291_57180, &P19809_289};	/*Property = CssMediaQueries*/
const Value V19809_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19809_292 = {S14, 0, &V19809_292_57183, &P19809_291};	/*Property = CssMinMax*/
const Value V19809_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19809_293 = {S15, 0, &V19809_293_57187, &P19809_292};	/*Property = CssOverflow*/
const Value V19809_294_57189 = {S2, NULL};	/*Value = True*/
const Property P19809_294 = {S16, 0, &V19809_294_57189, &P19809_293};	/*Property = CssPosition*/
const Value V19809_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19809_295 = {S17, 0, &V19809_295_57192, &P19809_294};	/*Property = CssText*/
const Value V19809_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19809_297 = {S18, 0, &V19809_297_57195, &P19809_295};	/*Property = CssTransforms*/
const Value V19809_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19809_241 = {S19, 0, &V19809_241_55799, &P19809_297};	/*Property = CssTransitions*/
const Value V19809_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19809_298 = {S20, 0, &V19809_298_57206, &P19809_241};	/*Property = CssUI*/
const Value V19809_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19809_300 = {S21, 0, &V19809_300_57216, &P19809_298};	/*Property = DataSet*/
const Value V19809_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19809_301 = {S22, 0, &V19809_301_57218, &P19809_300};	/*Property = DataUrl*/
const Value V19809_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19809_302 = {S23, 0, &V19809_302_57222, &P19809_301};	/*Property = DeviceOrientation*/
const Value V19809_304_57227 = {S2, NULL};	/*Value = True*/
const Property P19809_304 = {S24, 0, &V19809_304_57227, &P19809_302};	/*Property = FileReader*/
const Value V19809_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19809_305 = {S25, 0, &V19809_305_57231, &P19809_304};	/*Property = FileSaver*/
const Value V19809_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19809_306 = {S26, 0, &V19809_306_57237, &P19809_305};	/*Property = FileWriter*/
const Value V19809_307_57239 = {S2, NULL};	/*Value = True*/
const Property P19809_307 = {S27, 0, &V19809_307_57239, &P19809_306};	/*Property = FormData*/
const Value V19809_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19809_309 = {S28, 0, &V19809_309_57246, &P19809_307};	/*Property = Fullscreen*/
const Value V19809_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19809_310 = {S29, 0, &V19809_310_57248, &P19809_309};	/*Property = GeoLocation*/
const Value V19809_312_57254 = {S2, NULL};	/*Value = True*/
const Property P19809_312 = {S30, 0, &V19809_312_57254, &P19809_310};	/*Property = History*/
const Value V19809_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19809_314 = {S31, 0, &V19809_314_57261, &P19809_312};	/*Property = Html5*/
const Value V19809_313_57257 = {S2, NULL};	/*Value = True*/
const Property P19809_313 = {S32, 0, &V19809_313_57257, &P19809_314};	/*Property = Html-Media-Capture*/
const Value V19809_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19809_315 = {S33, 0, &V19809_315_57264, &P19809_313};	/*Property = Iframe*/
const Value V19809_316_57266 = {S2, NULL};	/*Value = True*/
const Property P19809_316 = {S34, 0, &V19809_316_57266, &P19809_315};	/*Property = IndexedDB*/
const Value V19809_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19809_317 = {S35, 0, &V19809_317_57269, &P19809_316};	/*Property = Json*/
const Value V19809_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19809_15 = {S37, 0, &V19809_15_28, &P19809_317};	/*Property = LayoutEngine*/
const Value V19809_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19809_344 = {S38, 0, &V19809_344_57272, &P19809_15};	/*Property = Masking*/
const Value V19809_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19809_320 = {S39, 0, &V19809_320_57290, &P19809_344};	/*Property = PostMessage*/
const Value V19809_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19809_321 = {S40, 0, &V19809_321_57293, &P19809_320};	/*Property = Progress*/
const Value V19809_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19809_322 = {S41, 0, &V19809_322_57296, &P19809_321};	/*Property = Prompts*/
const Value V19809_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19809_323 = {S42, 0, &V19809_323_57299, &P19809_322};	/*Property = Selector*/
const Value V19809_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19809_324 = {S43, 0, &V19809_324_57306, &P19809_323};	/*Property = Svg*/
const Value V19809_327_57314 = {S2, NULL};	/*Value = True*/
const Property P19809_327 = {S44, 0, &V19809_327_57314, &P19809_324};	/*Property = TouchEvents*/
const Value V19809_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19809_328 = {S45, 0, &V19809_328_57318, &P19809_327};	/*Property = Track*/
const Value V19809_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19809_329 = {S46, 0, &V19809_329_57323, &P19809_328};	/*Property = Video*/
const Value V19809_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19809_330 = {S47, 0, &V19809_330_57326, &P19809_329};	/*Property = Viewport*/
const Value V19809_332_57338 = {S2, NULL};	/*Value = True*/
const Property P19809_332 = {S48, 0, &V19809_332_57338, &P19809_330};	/*Property = WebWorkers*/
const Value V19809_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19809_333 = {S49, 0, &V19809_333_57342, &P19809_332};	/*Property = Xhr2*/
const Profile P19809 = {"19809", &P19809_333};	/*Profile ID - 19809*/
const Value V20088_1_1 = {S2, NULL};	/*Value = True*/
const Property P20088_1 = {S50, 0, &V20088_1_1, NULL};	/*Property = IsMobile*/
const Value V20088_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20088_5 = {S52, 0, &V20088_5_1277, &P20088_1};	/*Property = ScreenPixelsHeight*/
const Value V20088_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20088_4 = {S54, 0, &V20088_4_1279, &P20088_5};	/*Property = ScreenPixelsWidth*/
const Profile P20088 = {"20088", &P20088_4};	/*Profile ID - 20088*/
const Value V18690_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18690_279 = {S1, 0, &V18690_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18690_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18690_282 = {S3, 0, &V18690_282_57154, &P18690_279};	/*Property = BlobBuilder*/
const Value V18690_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18690_350 = {S4, 0, &V18690_350_57444, &P18690_282};	/*Property = Canvas*/
const Value V18690_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18690_283 = {S5, 0, &V18690_283_57159, &P18690_350};	/*Property = CssBackground*/
const Value V18690_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18690_284 = {S6, 0, &V18690_284_57163, &P18690_283};	/*Property = CssBorderImage*/
const Value V18690_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18690_285 = {S7, 0, &V18690_285_57166, &P18690_284};	/*Property = CssCanvas*/
const Value V18690_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18690_286 = {S8, 0, &V18690_286_57168, &P18690_285};	/*Property = CssColor*/
const Value V18690_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18690_237 = {S9, 0, &V18690_237_55786, &P18690_286};	/*Property = CssColumn*/
const Value V18690_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18690_287 = {S10, 0, &V18690_287_57172, &P18690_237};	/*Property = CssFlexbox*/
const Value V18690_288_57174 = {S2, NULL};	/*Value = True*/
const Property P18690_288 = {S11, 0, &V18690_288_57174, &P18690_287};	/*Property = CssFont*/
const Value V18690_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18690_289 = {S12, 0, &V18690_289_57759, &P18690_288};	/*Property = CssImages*/
const Value V18690_291_57180 = {S2, NULL};	/*Value = True*/
const Property P18690_291 = {S13, 0, &V18690_291_57180, &P18690_289};	/*Property = CssMediaQueries*/
const Value V18690_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18690_292 = {S14, 0, &V18690_292_57183, &P18690_291};	/*Property = CssMinMax*/
const Value V18690_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18690_293 = {S15, 0, &V18690_293_57187, &P18690_292};	/*Property = CssOverflow*/
const Value V18690_294_57189 = {S2, NULL};	/*Value = True*/
const Property P18690_294 = {S16, 0, &V18690_294_57189, &P18690_293};	/*Property = CssPosition*/
const Value V18690_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18690_295 = {S17, 0, &V18690_295_57192, &P18690_294};	/*Property = CssText*/
const Value V18690_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18690_297 = {S18, 0, &V18690_297_57195, &P18690_295};	/*Property = CssTransforms*/
const Value V18690_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18690_241 = {S19, 0, &V18690_241_55799, &P18690_297};	/*Property = CssTransitions*/
const Value V18690_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18690_298 = {S20, 0, &V18690_298_57206, &P18690_241};	/*Property = CssUI*/
const Value V18690_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18690_300 = {S21, 0, &V18690_300_57216, &P18690_298};	/*Property = DataSet*/
const Value V18690_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18690_301 = {S22, 0, &V18690_301_57218, &P18690_300};	/*Property = DataUrl*/
const Value V18690_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18690_302 = {S23, 0, &V18690_302_57222, &P18690_301};	/*Property = DeviceOrientation*/
const Value V18690_304_57227 = {S2, NULL};	/*Value = True*/
const Property P18690_304 = {S24, 0, &V18690_304_57227, &P18690_302};	/*Property = FileReader*/
const Value V18690_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18690_305 = {S25, 0, &V18690_305_57231, &P18690_304};	/*Property = FileSaver*/
const Value V18690_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18690_306 = {S26, 0, &V18690_306_57237, &P18690_305};	/*Property = FileWriter*/
const Value V18690_307_57239 = {S2, NULL};	/*Value = True*/
const Property P18690_307 = {S27, 0, &V18690_307_57239, &P18690_306};	/*Property = FormData*/
const Value V18690_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18690_309 = {S28, 0, &V18690_309_57246, &P18690_307};	/*Property = Fullscreen*/
const Value V18690_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18690_310 = {S29, 0, &V18690_310_57248, &P18690_309};	/*Property = GeoLocation*/
const Value V18690_312_57254 = {S2, NULL};	/*Value = True*/
const Property P18690_312 = {S30, 0, &V18690_312_57254, &P18690_310};	/*Property = History*/
const Value V18690_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18690_314 = {S31, 0, &V18690_314_57261, &P18690_312};	/*Property = Html5*/
const Value V18690_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18690_313 = {S32, 0, &V18690_313_57258, &P18690_314};	/*Property = Html-Media-Capture*/
const Value V18690_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18690_315 = {S33, 0, &V18690_315_57264, &P18690_313};	/*Property = Iframe*/
const Value V18690_316_57266 = {S2, NULL};	/*Value = True*/
const Property P18690_316 = {S34, 0, &V18690_316_57266, &P18690_315};	/*Property = IndexedDB*/
const Value V18690_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18690_317 = {S35, 0, &V18690_317_57269, &P18690_316};	/*Property = Json*/
const Value V18690_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18690_15 = {S37, 0, &V18690_15_28, &P18690_317};	/*Property = LayoutEngine*/
const Value V18690_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18690_344 = {S38, 0, &V18690_344_57272, &P18690_15};	/*Property = Masking*/
const Value V18690_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18690_320 = {S39, 0, &V18690_320_57290, &P18690_344};	/*Property = PostMessage*/
const Value V18690_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18690_321 = {S40, 0, &V18690_321_57293, &P18690_320};	/*Property = Progress*/
const Value V18690_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18690_322 = {S41, 0, &V18690_322_57296, &P18690_321};	/*Property = Prompts*/
const Value V18690_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18690_323 = {S42, 0, &V18690_323_57299, &P18690_322};	/*Property = Selector*/
const Value V18690_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18690_324 = {S43, 0, &V18690_324_57306, &P18690_323};	/*Property = Svg*/
const Value V18690_327_57314 = {S2, NULL};	/*Value = True*/
const Property P18690_327 = {S44, 0, &V18690_327_57314, &P18690_324};	/*Property = TouchEvents*/
const Value V18690_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18690_328 = {S45, 0, &V18690_328_57318, &P18690_327};	/*Property = Track*/
const Value V18690_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18690_329 = {S46, 0, &V18690_329_57323, &P18690_328};	/*Property = Video*/
const Value V18690_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18690_330 = {S47, 0, &V18690_330_57326, &P18690_329};	/*Property = Viewport*/
const Value V18690_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18690_332 = {S48, 0, &V18690_332_57339, &P18690_330};	/*Property = WebWorkers*/
const Value V18690_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18690_333 = {S49, 0, &V18690_333_57342, &P18690_332};	/*Property = Xhr2*/
const Profile P18690 = {"18690", &P18690_333};	/*Profile ID - 18690*/
const Value V20012_1_1 = {S2, NULL};	/*Value = True*/
const Property P20012_1 = {S50, 0, &V20012_1_1, NULL};	/*Property = IsMobile*/
const Value V20012_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P20012_5 = {S52, 0, &V20012_5_1275, &P20012_1};	/*Property = ScreenPixelsHeight*/
const Value V20012_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20012_4 = {S54, 0, &V20012_4_1229, &P20012_5};	/*Property = ScreenPixelsWidth*/
const Profile P20012 = {"20012", &P20012_4};	/*Profile ID - 20012*/
const Value V21246_1_1 = {S2, NULL};	/*Value = True*/
const Property P21246_1 = {S50, 0, &V21246_1_1, NULL};	/*Property = IsMobile*/
const Value V21246_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21246_5 = {S52, 0, &V21246_5_1277, &P21246_1};	/*Property = ScreenPixelsHeight*/
const Value V21246_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21246_4 = {S54, 0, &V21246_4_1279, &P21246_5};	/*Property = ScreenPixelsWidth*/
const Profile P21246 = {"21246", &P21246_4};	/*Profile ID - 21246*/
const Value V21306_1_1 = {S2, NULL};	/*Value = True*/
const Property P21306_1 = {S50, 0, &V21306_1_1, NULL};	/*Property = IsMobile*/
const Value V21306_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21306_5 = {S52, 0, &V21306_5_1458, &P21306_1};	/*Property = ScreenPixelsHeight*/
const Value V21306_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21306_4 = {S54, 0, &V21306_4_37617, &P21306_5};	/*Property = ScreenPixelsWidth*/
const Profile P21306 = {"21306", &P21306_4};	/*Profile ID - 21306*/
const Value V20037_1_1 = {S2, NULL};	/*Value = True*/
const Property P20037_1 = {S50, 0, &V20037_1_1, NULL};	/*Property = IsMobile*/
const Value V20037_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20037_5 = {S52, 0, &V20037_5_1222, &P20037_1};	/*Property = ScreenPixelsHeight*/
const Value V20037_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20037_4 = {S54, 0, &V20037_4_1224, &P20037_5};	/*Property = ScreenPixelsWidth*/
const Profile P20037 = {"20037", &P20037_4};	/*Profile ID - 20037*/
const Value V19779_1_1 = {S2, NULL};	/*Value = True*/
const Property P19779_1 = {S50, 0, &V19779_1_1, NULL};	/*Property = IsMobile*/
const Value V19779_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P19779_5 = {S52, 0, &V19779_5_1277, &P19779_1};	/*Property = ScreenPixelsHeight*/
const Value V19779_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P19779_4 = {S54, 0, &V19779_4_1279, &P19779_5};	/*Property = ScreenPixelsWidth*/
const Profile P19779 = {"19779", &P19779_4};	/*Profile ID - 19779*/
const Value V21273_1_1 = {S2, NULL};	/*Value = True*/
const Property P21273_1 = {S50, 0, &V21273_1_1, NULL};	/*Property = IsMobile*/
const Value V21273_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21273_5 = {S52, 0, &V21273_5_1277, &P21273_1};	/*Property = ScreenPixelsHeight*/
const Value V21273_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21273_4 = {S54, 0, &V21273_4_1279, &P21273_5};	/*Property = ScreenPixelsWidth*/
const Profile P21273 = {"21273", &P21273_4};	/*Profile ID - 21273*/
const Value V20168_1_1 = {S2, NULL};	/*Value = True*/
const Property P20168_1 = {S50, 0, &V20168_1_1, NULL};	/*Property = IsMobile*/
const Value V20168_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20168_5 = {S52, 0, &V20168_5_1458, &P20168_1};	/*Property = ScreenPixelsHeight*/
const Value V20168_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20168_4 = {S54, 0, &V20168_4_1460, &P20168_5};	/*Property = ScreenPixelsWidth*/
const Profile P20168 = {"20168", &P20168_4};	/*Profile ID - 20168*/
const Value V21465_1_1 = {S2, NULL};	/*Value = True*/
const Property P21465_1 = {S50, 0, &V21465_1_1, NULL};	/*Property = IsMobile*/
const Value V21465_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21465_5 = {S52, 0, &V21465_5_1277, &P21465_1};	/*Property = ScreenPixelsHeight*/
const Value V21465_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21465_4 = {S54, 0, &V21465_4_1279, &P21465_5};	/*Property = ScreenPixelsWidth*/
const Profile P21465 = {"21465", &P21465_4};	/*Profile ID - 21465*/
const Value V19814_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19814_279 = {S1, 0, &V19814_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19814_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19814_282 = {S3, 0, &V19814_282_57154, &P19814_279};	/*Property = BlobBuilder*/
const Value V19814_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19814_350 = {S4, 0, &V19814_350_57444, &P19814_282};	/*Property = Canvas*/
const Value V19814_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19814_283 = {S5, 0, &V19814_283_57159, &P19814_350};	/*Property = CssBackground*/
const Value V19814_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19814_284 = {S6, 0, &V19814_284_57163, &P19814_283};	/*Property = CssBorderImage*/
const Value V19814_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19814_285 = {S7, 0, &V19814_285_57166, &P19814_284};	/*Property = CssCanvas*/
const Value V19814_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19814_286 = {S8, 0, &V19814_286_57168, &P19814_285};	/*Property = CssColor*/
const Value V19814_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19814_237 = {S9, 0, &V19814_237_55786, &P19814_286};	/*Property = CssColumn*/
const Value V19814_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19814_287 = {S10, 0, &V19814_287_57172, &P19814_237};	/*Property = CssFlexbox*/
const Value V19814_288_57174 = {S2, NULL};	/*Value = True*/
const Property P19814_288 = {S11, 0, &V19814_288_57174, &P19814_287};	/*Property = CssFont*/
const Value V19814_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19814_289 = {S12, 0, &V19814_289_57759, &P19814_288};	/*Property = CssImages*/
const Value V19814_291_57180 = {S2, NULL};	/*Value = True*/
const Property P19814_291 = {S13, 0, &V19814_291_57180, &P19814_289};	/*Property = CssMediaQueries*/
const Value V19814_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19814_292 = {S14, 0, &V19814_292_57183, &P19814_291};	/*Property = CssMinMax*/
const Value V19814_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19814_293 = {S15, 0, &V19814_293_57187, &P19814_292};	/*Property = CssOverflow*/
const Value V19814_294_57189 = {S2, NULL};	/*Value = True*/
const Property P19814_294 = {S16, 0, &V19814_294_57189, &P19814_293};	/*Property = CssPosition*/
const Value V19814_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19814_295 = {S17, 0, &V19814_295_57192, &P19814_294};	/*Property = CssText*/
const Value V19814_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19814_297 = {S18, 0, &V19814_297_57195, &P19814_295};	/*Property = CssTransforms*/
const Value V19814_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19814_241 = {S19, 0, &V19814_241_55799, &P19814_297};	/*Property = CssTransitions*/
const Value V19814_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19814_298 = {S20, 0, &V19814_298_57206, &P19814_241};	/*Property = CssUI*/
const Value V19814_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19814_300 = {S21, 0, &V19814_300_57216, &P19814_298};	/*Property = DataSet*/
const Value V19814_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19814_301 = {S22, 0, &V19814_301_57218, &P19814_300};	/*Property = DataUrl*/
const Value V19814_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19814_302 = {S23, 0, &V19814_302_57222, &P19814_301};	/*Property = DeviceOrientation*/
const Value V19814_304_57227 = {S2, NULL};	/*Value = True*/
const Property P19814_304 = {S24, 0, &V19814_304_57227, &P19814_302};	/*Property = FileReader*/
const Value V19814_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19814_305 = {S25, 0, &V19814_305_57231, &P19814_304};	/*Property = FileSaver*/
const Value V19814_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19814_306 = {S26, 0, &V19814_306_57237, &P19814_305};	/*Property = FileWriter*/
const Value V19814_307_57239 = {S2, NULL};	/*Value = True*/
const Property P19814_307 = {S27, 0, &V19814_307_57239, &P19814_306};	/*Property = FormData*/
const Value V19814_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19814_309 = {S28, 0, &V19814_309_57246, &P19814_307};	/*Property = Fullscreen*/
const Value V19814_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19814_310 = {S29, 0, &V19814_310_57248, &P19814_309};	/*Property = GeoLocation*/
const Value V19814_312_57254 = {S2, NULL};	/*Value = True*/
const Property P19814_312 = {S30, 0, &V19814_312_57254, &P19814_310};	/*Property = History*/
const Value V19814_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19814_314 = {S31, 0, &V19814_314_57261, &P19814_312};	/*Property = Html5*/
const Value V19814_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19814_313 = {S32, 0, &V19814_313_57258, &P19814_314};	/*Property = Html-Media-Capture*/
const Value V19814_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19814_315 = {S33, 0, &V19814_315_57264, &P19814_313};	/*Property = Iframe*/
const Value V19814_316_57266 = {S2, NULL};	/*Value = True*/
const Property P19814_316 = {S34, 0, &V19814_316_57266, &P19814_315};	/*Property = IndexedDB*/
const Value V19814_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19814_317 = {S35, 0, &V19814_317_57269, &P19814_316};	/*Property = Json*/
const Value V19814_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19814_15 = {S37, 0, &V19814_15_28, &P19814_317};	/*Property = LayoutEngine*/
const Value V19814_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19814_344 = {S38, 0, &V19814_344_57272, &P19814_15};	/*Property = Masking*/
const Value V19814_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19814_320 = {S39, 0, &V19814_320_57290, &P19814_344};	/*Property = PostMessage*/
const Value V19814_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19814_321 = {S40, 0, &V19814_321_57293, &P19814_320};	/*Property = Progress*/
const Value V19814_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19814_322 = {S41, 0, &V19814_322_57296, &P19814_321};	/*Property = Prompts*/
const Value V19814_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19814_323 = {S42, 0, &V19814_323_57299, &P19814_322};	/*Property = Selector*/
const Value V19814_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19814_324 = {S43, 0, &V19814_324_57306, &P19814_323};	/*Property = Svg*/
const Value V19814_327_57314 = {S2, NULL};	/*Value = True*/
const Property P19814_327 = {S44, 0, &V19814_327_57314, &P19814_324};	/*Property = TouchEvents*/
const Value V19814_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19814_328 = {S45, 0, &V19814_328_57318, &P19814_327};	/*Property = Track*/
const Value V19814_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19814_329 = {S46, 0, &V19814_329_57323, &P19814_328};	/*Property = Video*/
const Value V19814_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19814_330 = {S47, 0, &V19814_330_57326, &P19814_329};	/*Property = Viewport*/
const Value V19814_332_57339 = {S0, NULL};	/*Value = False*/
const Property P19814_332 = {S48, 0, &V19814_332_57339, &P19814_330};	/*Property = WebWorkers*/
const Value V19814_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19814_333 = {S49, 0, &V19814_333_57342, &P19814_332};	/*Property = Xhr2*/
const Profile P19814 = {"19814", &P19814_333};	/*Profile ID - 19814*/
const Value V22322_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22322_279 = {S1, 0, &V22322_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22322_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22322_282 = {S3, 0, &V22322_282_57153, &P22322_279};	/*Property = BlobBuilder*/
const Value V22322_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22322_350 = {S4, 0, &V22322_350_57444, &P22322_282};	/*Property = Canvas*/
const Value V22322_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22322_283 = {S5, 0, &V22322_283_57159, &P22322_350};	/*Property = CssBackground*/
const Value V22322_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22322_284 = {S6, 0, &V22322_284_57163, &P22322_283};	/*Property = CssBorderImage*/
const Value V22322_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22322_285 = {S7, 0, &V22322_285_57166, &P22322_284};	/*Property = CssCanvas*/
const Value V22322_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22322_286 = {S8, 0, &V22322_286_57168, &P22322_285};	/*Property = CssColor*/
const Value V22322_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22322_237 = {S9, 0, &V22322_237_55784, &P22322_286};	/*Property = CssColumn*/
const Value V22322_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22322_287 = {S10, 0, &V22322_287_57172, &P22322_237};	/*Property = CssFlexbox*/
const Value V22322_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22322_288 = {S11, 0, &V22322_288_57175, &P22322_287};	/*Property = CssFont*/
const Value V22322_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22322_289 = {S12, 0, &V22322_289_57759, &P22322_288};	/*Property = CssImages*/
const Value V22322_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22322_291 = {S13, 0, &V22322_291_57180, &P22322_289};	/*Property = CssMediaQueries*/
const Value V22322_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22322_292 = {S14, 0, &V22322_292_57183, &P22322_291};	/*Property = CssMinMax*/
const Value V22322_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22322_293 = {S15, 0, &V22322_293_57187, &P22322_292};	/*Property = CssOverflow*/
const Value V22322_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22322_294 = {S16, 0, &V22322_294_57189, &P22322_293};	/*Property = CssPosition*/
const Value V22322_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22322_295 = {S17, 0, &V22322_295_57192, &P22322_294};	/*Property = CssText*/
const Value V22322_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22322_297 = {S18, 0, &V22322_297_57195, &P22322_295};	/*Property = CssTransforms*/
const Value V22322_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22322_241 = {S19, 0, &V22322_241_55799, &P22322_297};	/*Property = CssTransitions*/
const Value V22322_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22322_298 = {S20, 0, &V22322_298_57206, &P22322_241};	/*Property = CssUI*/
const Value V22322_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22322_300 = {S21, 0, &V22322_300_57216, &P22322_298};	/*Property = DataSet*/
const Value V22322_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22322_301 = {S22, 0, &V22322_301_57218, &P22322_300};	/*Property = DataUrl*/
const Value V22322_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22322_302 = {S23, 0, &V22322_302_57222, &P22322_301};	/*Property = DeviceOrientation*/
const Value V22322_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22322_304 = {S24, 0, &V22322_304_57227, &P22322_302};	/*Property = FileReader*/
const Value V22322_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22322_305 = {S25, 0, &V22322_305_57231, &P22322_304};	/*Property = FileSaver*/
const Value V22322_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22322_306 = {S26, 0, &V22322_306_57237, &P22322_305};	/*Property = FileWriter*/
const Value V22322_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22322_307 = {S27, 0, &V22322_307_57239, &P22322_306};	/*Property = FormData*/
const Value V22322_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22322_309 = {S28, 0, &V22322_309_57246, &P22322_307};	/*Property = Fullscreen*/
const Value V22322_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22322_310 = {S29, 0, &V22322_310_57248, &P22322_309};	/*Property = GeoLocation*/
const Value V22322_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22322_312 = {S30, 0, &V22322_312_57255, &P22322_310};	/*Property = History*/
const Value V22322_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22322_314 = {S31, 0, &V22322_314_57261, &P22322_312};	/*Property = Html5*/
const Value V22322_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22322_313 = {S32, 0, &V22322_313_57258, &P22322_314};	/*Property = Html-Media-Capture*/
const Value V22322_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22322_315 = {S33, 0, &V22322_315_57264, &P22322_313};	/*Property = Iframe*/
const Value V22322_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22322_316 = {S34, 0, &V22322_316_57266, &P22322_315};	/*Property = IndexedDB*/
const Value V22322_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22322_317 = {S35, 0, &V22322_317_57269, &P22322_316};	/*Property = Json*/
const Value V22322_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22322_15 = {S37, 0, &V22322_15_28, &P22322_317};	/*Property = LayoutEngine*/
const Value V22322_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22322_344 = {S38, 0, &V22322_344_57272, &P22322_15};	/*Property = Masking*/
const Value V22322_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22322_320 = {S39, 0, &V22322_320_57290, &P22322_344};	/*Property = PostMessage*/
const Value V22322_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22322_321 = {S40, 0, &V22322_321_57293, &P22322_320};	/*Property = Progress*/
const Value V22322_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22322_322 = {S41, 0, &V22322_322_57296, &P22322_321};	/*Property = Prompts*/
const Value V22322_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22322_323 = {S42, 0, &V22322_323_57299, &P22322_322};	/*Property = Selector*/
const Value V22322_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22322_324 = {S43, 0, &V22322_324_57306, &P22322_323};	/*Property = Svg*/
const Value V22322_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22322_327 = {S44, 0, &V22322_327_57315, &P22322_324};	/*Property = TouchEvents*/
const Value V22322_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22322_328 = {S45, 0, &V22322_328_57318, &P22322_327};	/*Property = Track*/
const Value V22322_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22322_329 = {S46, 0, &V22322_329_57323, &P22322_328};	/*Property = Video*/
const Value V22322_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22322_330 = {S47, 0, &V22322_330_57326, &P22322_329};	/*Property = Viewport*/
const Value V22322_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22322_332 = {S48, 0, &V22322_332_57338, &P22322_330};	/*Property = WebWorkers*/
const Value V22322_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22322_333 = {S49, 0, &V22322_333_57342, &P22322_332};	/*Property = Xhr2*/
const Profile P22322 = {"22322", &P22322_333};	/*Profile ID - 22322*/
const Value V22136_1_1 = {S2, NULL};	/*Value = True*/
const Property P22136_1 = {S50, 0, &V22136_1_1, NULL};	/*Property = IsMobile*/
const Value V22136_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22136_5 = {S52, 0, &V22136_5_1832, &P22136_1};	/*Property = ScreenPixelsHeight*/
const Value V22136_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22136_4 = {S54, 0, &V22136_4_5645, &P22136_5};	/*Property = ScreenPixelsWidth*/
const Profile P22136 = {"22136", &P22136_4};	/*Profile ID - 22136*/
const Value V21057_1_1 = {S2, NULL};	/*Value = True*/
const Property P21057_1 = {S50, 0, &V21057_1_1, NULL};	/*Property = IsMobile*/
const Value V21057_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21057_5 = {S52, 0, &V21057_5_1277, &P21057_1};	/*Property = ScreenPixelsHeight*/
const Value V21057_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21057_4 = {S54, 0, &V21057_4_1279, &P21057_5};	/*Property = ScreenPixelsWidth*/
const Profile P21057 = {"21057", &P21057_4};	/*Profile ID - 21057*/
const Profile P22710 = {"22710", NULL};	/*Profile ID - 22710*/
const Value V19939_1_1 = {S2, NULL};	/*Value = True*/
const Property P19939_1 = {S50, 0, &V19939_1_1, NULL};	/*Property = IsMobile*/
const Value V19939_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P19939_5 = {S52, 0, &V19939_5_1277, &P19939_1};	/*Property = ScreenPixelsHeight*/
const Value V19939_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P19939_4 = {S54, 0, &V19939_4_1279, &P19939_5};	/*Property = ScreenPixelsWidth*/
const Profile P19939 = {"19939", &P19939_4};	/*Profile ID - 19939*/
const Value V20762_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_279 = {S1, 0, &V20762_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20762_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_282 = {S3, 0, &V20762_282_57155, &P20762_279};	/*Property = BlobBuilder*/
const Value V20762_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_350 = {S4, 0, &V20762_350_57446, &P20762_282};	/*Property = Canvas*/
const Value V20762_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_283 = {S5, 0, &V20762_283_57161, &P20762_350};	/*Property = CssBackground*/
const Value V20762_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_284 = {S6, 0, &V20762_284_57164, &P20762_283};	/*Property = CssBorderImage*/
const Value V20762_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_285 = {S7, 0, &V20762_285_57167, &P20762_284};	/*Property = CssCanvas*/
const Value V20762_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_286 = {S8, 0, &V20762_286_57170, &P20762_285};	/*Property = CssColor*/
const Value V20762_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_237 = {S9, 0, &V20762_237_55786, &P20762_286};	/*Property = CssColumn*/
const Value V20762_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_287 = {S10, 0, &V20762_287_57173, &P20762_237};	/*Property = CssFlexbox*/
const Value V20762_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_288 = {S11, 0, &V20762_288_57176, &P20762_287};	/*Property = CssFont*/
const Value V20762_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_289 = {S12, 0, &V20762_289_86899, &P20762_288};	/*Property = CssImages*/
const Value V20762_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_291 = {S13, 0, &V20762_291_57182, &P20762_289};	/*Property = CssMediaQueries*/
const Value V20762_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_292 = {S14, 0, &V20762_292_57185, &P20762_291};	/*Property = CssMinMax*/
const Value V20762_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_293 = {S15, 0, &V20762_293_57188, &P20762_292};	/*Property = CssOverflow*/
const Value V20762_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_294 = {S16, 0, &V20762_294_57191, &P20762_293};	/*Property = CssPosition*/
const Value V20762_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_295 = {S17, 0, &V20762_295_57194, &P20762_294};	/*Property = CssText*/
const Value V20762_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_297 = {S18, 0, &V20762_297_57197, &P20762_295};	/*Property = CssTransforms*/
const Value V20762_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_241 = {S19, 0, &V20762_241_55801, &P20762_297};	/*Property = CssTransitions*/
const Value V20762_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_298 = {S20, 0, &V20762_298_57208, &P20762_241};	/*Property = CssUI*/
const Value V20762_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_300 = {S21, 0, &V20762_300_57217, &P20762_298};	/*Property = DataSet*/
const Value V20762_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_301 = {S22, 0, &V20762_301_57220, &P20762_300};	/*Property = DataUrl*/
const Value V20762_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_302 = {S23, 0, &V20762_302_57223, &P20762_301};	/*Property = DeviceOrientation*/
const Value V20762_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_304 = {S24, 0, &V20762_304_57229, &P20762_302};	/*Property = FileReader*/
const Value V20762_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_305 = {S25, 0, &V20762_305_57232, &P20762_304};	/*Property = FileSaver*/
const Value V20762_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_306 = {S26, 0, &V20762_306_57238, &P20762_305};	/*Property = FileWriter*/
const Value V20762_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_307 = {S27, 0, &V20762_307_57241, &P20762_306};	/*Property = FormData*/
const Value V20762_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_309 = {S28, 0, &V20762_309_57247, &P20762_307};	/*Property = Fullscreen*/
const Value V20762_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_310 = {S29, 0, &V20762_310_57250, &P20762_309};	/*Property = GeoLocation*/
const Value V20762_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_312 = {S30, 0, &V20762_312_57256, &P20762_310};	/*Property = History*/
const Value V20762_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_314 = {S31, 0, &V20762_314_57262, &P20762_312};	/*Property = Html5*/
const Value V20762_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_313 = {S32, 0, &V20762_313_57259, &P20762_314};	/*Property = Html-Media-Capture*/
const Value V20762_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_315 = {S33, 0, &V20762_315_57265, &P20762_313};	/*Property = Iframe*/
const Value V20762_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_316 = {S34, 0, &V20762_316_57268, &P20762_315};	/*Property = IndexedDB*/
const Value V20762_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_317 = {S35, 0, &V20762_317_57271, &P20762_316};	/*Property = Json*/
const Value V20762_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P20762_15 = {S37, 0, &V20762_15_90837, &P20762_317};	/*Property = LayoutEngine*/
const Value V20762_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_344 = {S38, 0, &V20762_344_57274, &P20762_15};	/*Property = Masking*/
const Value V20762_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_320 = {S39, 0, &V20762_320_57292, &P20762_344};	/*Property = PostMessage*/
const Value V20762_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_321 = {S40, 0, &V20762_321_57295, &P20762_320};	/*Property = Progress*/
const Value V20762_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_322 = {S41, 0, &V20762_322_57298, &P20762_321};	/*Property = Prompts*/
const Value V20762_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_323 = {S42, 0, &V20762_323_57301, &P20762_322};	/*Property = Selector*/
const Value V20762_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_324 = {S43, 0, &V20762_324_57307, &P20762_323};	/*Property = Svg*/
const Value V20762_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_327 = {S44, 0, &V20762_327_57316, &P20762_324};	/*Property = TouchEvents*/
const Value V20762_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_328 = {S45, 0, &V20762_328_57319, &P20762_327};	/*Property = Track*/
const Value V20762_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_329 = {S46, 0, &V20762_329_57325, &P20762_328};	/*Property = Video*/
const Value V20762_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_330 = {S47, 0, &V20762_330_57328, &P20762_329};	/*Property = Viewport*/
const Value V20762_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_332 = {S48, 0, &V20762_332_57340, &P20762_330};	/*Property = WebWorkers*/
const Value V20762_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20762_333 = {S49, 0, &V20762_333_57343, &P20762_332};	/*Property = Xhr2*/
const Profile P20762 = {"20762", &P20762_333};	/*Profile ID - 20762*/
const Value V19987_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19987_279 = {S1, 0, &V19987_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19987_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19987_282 = {S3, 0, &V19987_282_57154, &P19987_279};	/*Property = BlobBuilder*/
const Value V19987_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19987_350 = {S4, 0, &V19987_350_57444, &P19987_282};	/*Property = Canvas*/
const Value V19987_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19987_283 = {S5, 0, &V19987_283_57159, &P19987_350};	/*Property = CssBackground*/
const Value V19987_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19987_284 = {S6, 0, &V19987_284_57163, &P19987_283};	/*Property = CssBorderImage*/
const Value V19987_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19987_285 = {S7, 0, &V19987_285_57166, &P19987_284};	/*Property = CssCanvas*/
const Value V19987_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19987_286 = {S8, 0, &V19987_286_57168, &P19987_285};	/*Property = CssColor*/
const Value V19987_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19987_237 = {S9, 0, &V19987_237_55786, &P19987_286};	/*Property = CssColumn*/
const Value V19987_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19987_287 = {S10, 0, &V19987_287_57172, &P19987_237};	/*Property = CssFlexbox*/
const Value V19987_288_57175 = {S0, NULL};	/*Value = False*/
const Property P19987_288 = {S11, 0, &V19987_288_57175, &P19987_287};	/*Property = CssFont*/
const Value V19987_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19987_289 = {S12, 0, &V19987_289_57759, &P19987_288};	/*Property = CssImages*/
const Value V19987_291_57181 = {S0, NULL};	/*Value = False*/
const Property P19987_291 = {S13, 0, &V19987_291_57181, &P19987_289};	/*Property = CssMediaQueries*/
const Value V19987_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19987_292 = {S14, 0, &V19987_292_57183, &P19987_291};	/*Property = CssMinMax*/
const Value V19987_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19987_293 = {S15, 0, &V19987_293_57187, &P19987_292};	/*Property = CssOverflow*/
const Value V19987_294_57190 = {S0, NULL};	/*Value = False*/
const Property P19987_294 = {S16, 0, &V19987_294_57190, &P19987_293};	/*Property = CssPosition*/
const Value V19987_295_57193 = {S0, NULL};	/*Value = False*/
const Property P19987_295 = {S17, 0, &V19987_295_57193, &P19987_294};	/*Property = CssText*/
const Value V19987_297_57196 = {S0, NULL};	/*Value = False*/
const Property P19987_297 = {S18, 0, &V19987_297_57196, &P19987_295};	/*Property = CssTransforms*/
const Value V19987_241_55800 = {S0, NULL};	/*Value = False*/
const Property P19987_241 = {S19, 0, &V19987_241_55800, &P19987_297};	/*Property = CssTransitions*/
const Value V19987_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19987_298 = {S20, 0, &V19987_298_57206, &P19987_241};	/*Property = CssUI*/
const Value V19987_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19987_300 = {S21, 0, &V19987_300_57216, &P19987_298};	/*Property = DataSet*/
const Value V19987_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19987_301 = {S22, 0, &V19987_301_57218, &P19987_300};	/*Property = DataUrl*/
const Value V19987_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19987_302 = {S23, 0, &V19987_302_57222, &P19987_301};	/*Property = DeviceOrientation*/
const Value V19987_304_57228 = {S0, NULL};	/*Value = False*/
const Property P19987_304 = {S24, 0, &V19987_304_57228, &P19987_302};	/*Property = FileReader*/
const Value V19987_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19987_305 = {S25, 0, &V19987_305_57231, &P19987_304};	/*Property = FileSaver*/
const Value V19987_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19987_306 = {S26, 0, &V19987_306_57237, &P19987_305};	/*Property = FileWriter*/
const Value V19987_307_57240 = {S0, NULL};	/*Value = False*/
const Property P19987_307 = {S27, 0, &V19987_307_57240, &P19987_306};	/*Property = FormData*/
const Value V19987_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19987_309 = {S28, 0, &V19987_309_57246, &P19987_307};	/*Property = Fullscreen*/
const Value V19987_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19987_310 = {S29, 0, &V19987_310_57248, &P19987_309};	/*Property = GeoLocation*/
const Value V19987_312_57255 = {S0, NULL};	/*Value = False*/
const Property P19987_312 = {S30, 0, &V19987_312_57255, &P19987_310};	/*Property = History*/
const Value V19987_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19987_314 = {S31, 0, &V19987_314_57261, &P19987_312};	/*Property = Html5*/
const Value V19987_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19987_313 = {S32, 0, &V19987_313_57258, &P19987_314};	/*Property = Html-Media-Capture*/
const Value V19987_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19987_315 = {S33, 0, &V19987_315_57264, &P19987_313};	/*Property = Iframe*/
const Value V19987_316_57267 = {S0, NULL};	/*Value = False*/
const Property P19987_316 = {S34, 0, &V19987_316_57267, &P19987_315};	/*Property = IndexedDB*/
const Value V19987_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19987_317 = {S35, 0, &V19987_317_57269, &P19987_316};	/*Property = Json*/
const Value V19987_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P19987_15 = {S37, 0, &V19987_15_30, &P19987_317};	/*Property = LayoutEngine*/
const Value V19987_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19987_344 = {S38, 0, &V19987_344_57272, &P19987_15};	/*Property = Masking*/
const Value V19987_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19987_320 = {S39, 0, &V19987_320_57290, &P19987_344};	/*Property = PostMessage*/
const Value V19987_321_57294 = {S0, NULL};	/*Value = False*/
const Property P19987_321 = {S40, 0, &V19987_321_57294, &P19987_320};	/*Property = Progress*/
const Value V19987_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19987_322 = {S41, 0, &V19987_322_57296, &P19987_321};	/*Property = Prompts*/
const Value V19987_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19987_323 = {S42, 0, &V19987_323_57299, &P19987_322};	/*Property = Selector*/
const Value V19987_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19987_324 = {S43, 0, &V19987_324_57306, &P19987_323};	/*Property = Svg*/
const Value V19987_327_57315 = {S0, NULL};	/*Value = False*/
const Property P19987_327 = {S44, 0, &V19987_327_57315, &P19987_324};	/*Property = TouchEvents*/
const Value V19987_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19987_328 = {S45, 0, &V19987_328_57318, &P19987_327};	/*Property = Track*/
const Value V19987_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19987_329 = {S46, 0, &V19987_329_57323, &P19987_328};	/*Property = Video*/
const Value V19987_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19987_330 = {S47, 0, &V19987_330_57326, &P19987_329};	/*Property = Viewport*/
const Value V19987_332_57339 = {S0, NULL};	/*Value = False*/
const Property P19987_332 = {S48, 0, &V19987_332_57339, &P19987_330};	/*Property = WebWorkers*/
const Value V19987_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19987_333 = {S49, 0, &V19987_333_57342, &P19987_332};	/*Property = Xhr2*/
const Profile P19987 = {"19987", &P19987_333};	/*Profile ID - 19987*/
const Value V21587_1_1 = {S2, NULL};	/*Value = True*/
const Property P21587_1 = {S50, 0, &V21587_1_1, NULL};	/*Property = IsMobile*/
const Value V21587_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21587_5 = {S52, 0, &V21587_5_1227, &P21587_1};	/*Property = ScreenPixelsHeight*/
const Value V21587_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21587_4 = {S54, 0, &V21587_4_1229, &P21587_5};	/*Property = ScreenPixelsWidth*/
const Profile P21587 = {"21587", &P21587_4};	/*Profile ID - 21587*/
const Value V23028_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_279 = {S1, 0, &V23028_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23028_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_282 = {S3, 0, &V23028_282_57155, &P23028_279};	/*Property = BlobBuilder*/
const Value V23028_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_350 = {S4, 0, &V23028_350_57446, &P23028_282};	/*Property = Canvas*/
const Value V23028_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_283 = {S5, 0, &V23028_283_57161, &P23028_350};	/*Property = CssBackground*/
const Value V23028_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_284 = {S6, 0, &V23028_284_57164, &P23028_283};	/*Property = CssBorderImage*/
const Value V23028_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_285 = {S7, 0, &V23028_285_57167, &P23028_284};	/*Property = CssCanvas*/
const Value V23028_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_286 = {S8, 0, &V23028_286_57170, &P23028_285};	/*Property = CssColor*/
const Value V23028_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_237 = {S9, 0, &V23028_237_55786, &P23028_286};	/*Property = CssColumn*/
const Value V23028_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_287 = {S10, 0, &V23028_287_57173, &P23028_237};	/*Property = CssFlexbox*/
const Value V23028_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_288 = {S11, 0, &V23028_288_57176, &P23028_287};	/*Property = CssFont*/
const Value V23028_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_289 = {S12, 0, &V23028_289_86899, &P23028_288};	/*Property = CssImages*/
const Value V23028_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_291 = {S13, 0, &V23028_291_57182, &P23028_289};	/*Property = CssMediaQueries*/
const Value V23028_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_292 = {S14, 0, &V23028_292_57185, &P23028_291};	/*Property = CssMinMax*/
const Value V23028_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_293 = {S15, 0, &V23028_293_57188, &P23028_292};	/*Property = CssOverflow*/
const Value V23028_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_294 = {S16, 0, &V23028_294_57191, &P23028_293};	/*Property = CssPosition*/
const Value V23028_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_295 = {S17, 0, &V23028_295_57194, &P23028_294};	/*Property = CssText*/
const Value V23028_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_297 = {S18, 0, &V23028_297_57197, &P23028_295};	/*Property = CssTransforms*/
const Value V23028_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_241 = {S19, 0, &V23028_241_55801, &P23028_297};	/*Property = CssTransitions*/
const Value V23028_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_298 = {S20, 0, &V23028_298_57208, &P23028_241};	/*Property = CssUI*/
const Value V23028_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_300 = {S21, 0, &V23028_300_57217, &P23028_298};	/*Property = DataSet*/
const Value V23028_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_301 = {S22, 0, &V23028_301_57220, &P23028_300};	/*Property = DataUrl*/
const Value V23028_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_302 = {S23, 0, &V23028_302_57223, &P23028_301};	/*Property = DeviceOrientation*/
const Value V23028_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_304 = {S24, 0, &V23028_304_57229, &P23028_302};	/*Property = FileReader*/
const Value V23028_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_305 = {S25, 0, &V23028_305_57232, &P23028_304};	/*Property = FileSaver*/
const Value V23028_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_306 = {S26, 0, &V23028_306_57238, &P23028_305};	/*Property = FileWriter*/
const Value V23028_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_307 = {S27, 0, &V23028_307_57241, &P23028_306};	/*Property = FormData*/
const Value V23028_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_309 = {S28, 0, &V23028_309_57247, &P23028_307};	/*Property = Fullscreen*/
const Value V23028_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_310 = {S29, 0, &V23028_310_57250, &P23028_309};	/*Property = GeoLocation*/
const Value V23028_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_312 = {S30, 0, &V23028_312_57256, &P23028_310};	/*Property = History*/
const Value V23028_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_314 = {S31, 0, &V23028_314_57262, &P23028_312};	/*Property = Html5*/
const Value V23028_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_313 = {S32, 0, &V23028_313_57259, &P23028_314};	/*Property = Html-Media-Capture*/
const Value V23028_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_315 = {S33, 0, &V23028_315_57265, &P23028_313};	/*Property = Iframe*/
const Value V23028_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_316 = {S34, 0, &V23028_316_57268, &P23028_315};	/*Property = IndexedDB*/
const Value V23028_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_317 = {S35, 0, &V23028_317_57271, &P23028_316};	/*Property = Json*/
const Value V23028_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P23028_15 = {S37, 0, &V23028_15_28, &P23028_317};	/*Property = LayoutEngine*/
const Value V23028_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_344 = {S38, 0, &V23028_344_57274, &P23028_15};	/*Property = Masking*/
const Value V23028_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_320 = {S39, 0, &V23028_320_57292, &P23028_344};	/*Property = PostMessage*/
const Value V23028_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_321 = {S40, 0, &V23028_321_57295, &P23028_320};	/*Property = Progress*/
const Value V23028_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_322 = {S41, 0, &V23028_322_57298, &P23028_321};	/*Property = Prompts*/
const Value V23028_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_323 = {S42, 0, &V23028_323_57301, &P23028_322};	/*Property = Selector*/
const Value V23028_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_324 = {S43, 0, &V23028_324_57307, &P23028_323};	/*Property = Svg*/
const Value V23028_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_327 = {S44, 0, &V23028_327_57316, &P23028_324};	/*Property = TouchEvents*/
const Value V23028_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_328 = {S45, 0, &V23028_328_57319, &P23028_327};	/*Property = Track*/
const Value V23028_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_329 = {S46, 0, &V23028_329_57325, &P23028_328};	/*Property = Video*/
const Value V23028_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_330 = {S47, 0, &V23028_330_57328, &P23028_329};	/*Property = Viewport*/
const Value V23028_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_332 = {S48, 0, &V23028_332_57340, &P23028_330};	/*Property = WebWorkers*/
const Value V23028_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23028_333 = {S49, 0, &V23028_333_57343, &P23028_332};	/*Property = Xhr2*/
const Profile P23028 = {"23028", &P23028_333};	/*Profile ID - 23028*/
const Value V22002_1_1 = {S2, NULL};	/*Value = True*/
const Property P22002_1 = {S50, 0, &V22002_1_1, NULL};	/*Property = IsMobile*/
const Value V22002_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22002_5 = {S52, 0, &V22002_5_1277, &P22002_1};	/*Property = ScreenPixelsHeight*/
const Value V22002_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22002_4 = {S54, 0, &V22002_4_1279, &P22002_5};	/*Property = ScreenPixelsWidth*/
const Profile P22002 = {"22002", &P22002_4};	/*Profile ID - 22002*/
const Profile P22637 = {"22637", NULL};	/*Profile ID - 22637*/
const Value V21835_1_1 = {S2, NULL};	/*Value = True*/
const Property P21835_1 = {S50, 0, &V21835_1_1, NULL};	/*Property = IsMobile*/
const Value V21835_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P21835_5 = {S52, 0, &V21835_5_1832, &P21835_1};	/*Property = ScreenPixelsHeight*/
const Value V21835_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21835_4 = {S54, 0, &V21835_4_2336, &P21835_5};	/*Property = ScreenPixelsWidth*/
const Profile P21835 = {"21835", &P21835_4};	/*Profile ID - 21835*/
const Value V20231_1_1 = {S2, NULL};	/*Value = True*/
const Property P20231_1 = {S50, 0, &V20231_1_1, NULL};	/*Property = IsMobile*/
const Value V20231_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20231_5 = {S52, 0, &V20231_5_1227, &P20231_1};	/*Property = ScreenPixelsHeight*/
const Value V20231_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20231_4 = {S54, 0, &V20231_4_1229, &P20231_5};	/*Property = ScreenPixelsWidth*/
const Profile P20231 = {"20231", &P20231_4};	/*Profile ID - 20231*/
const Value V20132_1_1 = {S2, NULL};	/*Value = True*/
const Property P20132_1 = {S50, 0, &V20132_1_1, NULL};	/*Property = IsMobile*/
const Value V20132_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20132_5 = {S52, 0, &V20132_5_1227, &P20132_1};	/*Property = ScreenPixelsHeight*/
const Value V20132_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P20132_4 = {S54, 0, &V20132_4_1943, &P20132_5};	/*Property = ScreenPixelsWidth*/
const Profile P20132 = {"20132", &P20132_4};	/*Profile ID - 20132*/
const Value V20912_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_279 = {S1, 0, &V20912_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20912_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_282 = {S3, 0, &V20912_282_57155, &P20912_279};	/*Property = BlobBuilder*/
const Value V20912_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_350 = {S4, 0, &V20912_350_57446, &P20912_282};	/*Property = Canvas*/
const Value V20912_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_283 = {S5, 0, &V20912_283_57161, &P20912_350};	/*Property = CssBackground*/
const Value V20912_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_284 = {S6, 0, &V20912_284_57164, &P20912_283};	/*Property = CssBorderImage*/
const Value V20912_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_285 = {S7, 0, &V20912_285_57167, &P20912_284};	/*Property = CssCanvas*/
const Value V20912_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_286 = {S8, 0, &V20912_286_57170, &P20912_285};	/*Property = CssColor*/
const Value V20912_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_237 = {S9, 0, &V20912_237_55786, &P20912_286};	/*Property = CssColumn*/
const Value V20912_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_287 = {S10, 0, &V20912_287_57173, &P20912_237};	/*Property = CssFlexbox*/
const Value V20912_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_288 = {S11, 0, &V20912_288_57176, &P20912_287};	/*Property = CssFont*/
const Value V20912_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_289 = {S12, 0, &V20912_289_86899, &P20912_288};	/*Property = CssImages*/
const Value V20912_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_291 = {S13, 0, &V20912_291_57182, &P20912_289};	/*Property = CssMediaQueries*/
const Value V20912_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_292 = {S14, 0, &V20912_292_57185, &P20912_291};	/*Property = CssMinMax*/
const Value V20912_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_293 = {S15, 0, &V20912_293_57188, &P20912_292};	/*Property = CssOverflow*/
const Value V20912_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_294 = {S16, 0, &V20912_294_57191, &P20912_293};	/*Property = CssPosition*/
const Value V20912_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_295 = {S17, 0, &V20912_295_57194, &P20912_294};	/*Property = CssText*/
const Value V20912_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_297 = {S18, 0, &V20912_297_57197, &P20912_295};	/*Property = CssTransforms*/
const Value V20912_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_241 = {S19, 0, &V20912_241_55801, &P20912_297};	/*Property = CssTransitions*/
const Value V20912_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_298 = {S20, 0, &V20912_298_57208, &P20912_241};	/*Property = CssUI*/
const Value V20912_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_300 = {S21, 0, &V20912_300_57217, &P20912_298};	/*Property = DataSet*/
const Value V20912_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_301 = {S22, 0, &V20912_301_57220, &P20912_300};	/*Property = DataUrl*/
const Value V20912_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_302 = {S23, 0, &V20912_302_57223, &P20912_301};	/*Property = DeviceOrientation*/
const Value V20912_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_304 = {S24, 0, &V20912_304_57229, &P20912_302};	/*Property = FileReader*/
const Value V20912_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_305 = {S25, 0, &V20912_305_57232, &P20912_304};	/*Property = FileSaver*/
const Value V20912_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_306 = {S26, 0, &V20912_306_57238, &P20912_305};	/*Property = FileWriter*/
const Value V20912_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_307 = {S27, 0, &V20912_307_57241, &P20912_306};	/*Property = FormData*/
const Value V20912_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_309 = {S28, 0, &V20912_309_57247, &P20912_307};	/*Property = Fullscreen*/
const Value V20912_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_310 = {S29, 0, &V20912_310_57250, &P20912_309};	/*Property = GeoLocation*/
const Value V20912_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_312 = {S30, 0, &V20912_312_57256, &P20912_310};	/*Property = History*/
const Value V20912_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_314 = {S31, 0, &V20912_314_57262, &P20912_312};	/*Property = Html5*/
const Value V20912_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_313 = {S32, 0, &V20912_313_57259, &P20912_314};	/*Property = Html-Media-Capture*/
const Value V20912_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_315 = {S33, 0, &V20912_315_57265, &P20912_313};	/*Property = Iframe*/
const Value V20912_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_316 = {S34, 0, &V20912_316_57268, &P20912_315};	/*Property = IndexedDB*/
const Value V20912_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_317 = {S35, 0, &V20912_317_57271, &P20912_316};	/*Property = Json*/
const Value V20912_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P20912_15 = {S37, 0, &V20912_15_28, &P20912_317};	/*Property = LayoutEngine*/
const Value V20912_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_344 = {S38, 0, &V20912_344_57274, &P20912_15};	/*Property = Masking*/
const Value V20912_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_320 = {S39, 0, &V20912_320_57292, &P20912_344};	/*Property = PostMessage*/
const Value V20912_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_321 = {S40, 0, &V20912_321_57295, &P20912_320};	/*Property = Progress*/
const Value V20912_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_322 = {S41, 0, &V20912_322_57298, &P20912_321};	/*Property = Prompts*/
const Value V20912_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_323 = {S42, 0, &V20912_323_57301, &P20912_322};	/*Property = Selector*/
const Value V20912_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_324 = {S43, 0, &V20912_324_57307, &P20912_323};	/*Property = Svg*/
const Value V20912_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_327 = {S44, 0, &V20912_327_57316, &P20912_324};	/*Property = TouchEvents*/
const Value V20912_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_328 = {S45, 0, &V20912_328_57319, &P20912_327};	/*Property = Track*/
const Value V20912_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_329 = {S46, 0, &V20912_329_57325, &P20912_328};	/*Property = Video*/
const Value V20912_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_330 = {S47, 0, &V20912_330_57328, &P20912_329};	/*Property = Viewport*/
const Value V20912_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_332 = {S48, 0, &V20912_332_57340, &P20912_330};	/*Property = WebWorkers*/
const Value V20912_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20912_333 = {S49, 0, &V20912_333_57343, &P20912_332};	/*Property = Xhr2*/
const Profile P20912 = {"20912", &P20912_333};	/*Profile ID - 20912*/
const Value V18694_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_279 = {S1, 0, &V18694_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18694_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_282 = {S3, 0, &V18694_282_57155, &P18694_279};	/*Property = BlobBuilder*/
const Value V18694_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_350 = {S4, 0, &V18694_350_57446, &P18694_282};	/*Property = Canvas*/
const Value V18694_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_283 = {S5, 0, &V18694_283_57161, &P18694_350};	/*Property = CssBackground*/
const Value V18694_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_284 = {S6, 0, &V18694_284_57164, &P18694_283};	/*Property = CssBorderImage*/
const Value V18694_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_285 = {S7, 0, &V18694_285_57167, &P18694_284};	/*Property = CssCanvas*/
const Value V18694_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_286 = {S8, 0, &V18694_286_57170, &P18694_285};	/*Property = CssColor*/
const Value V18694_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_237 = {S9, 0, &V18694_237_55786, &P18694_286};	/*Property = CssColumn*/
const Value V18694_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_287 = {S10, 0, &V18694_287_57173, &P18694_237};	/*Property = CssFlexbox*/
const Value V18694_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_288 = {S11, 0, &V18694_288_57176, &P18694_287};	/*Property = CssFont*/
const Value V18694_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_289 = {S12, 0, &V18694_289_86899, &P18694_288};	/*Property = CssImages*/
const Value V18694_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_291 = {S13, 0, &V18694_291_57182, &P18694_289};	/*Property = CssMediaQueries*/
const Value V18694_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_292 = {S14, 0, &V18694_292_57185, &P18694_291};	/*Property = CssMinMax*/
const Value V18694_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_293 = {S15, 0, &V18694_293_57188, &P18694_292};	/*Property = CssOverflow*/
const Value V18694_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_294 = {S16, 0, &V18694_294_57191, &P18694_293};	/*Property = CssPosition*/
const Value V18694_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_295 = {S17, 0, &V18694_295_57194, &P18694_294};	/*Property = CssText*/
const Value V18694_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_297 = {S18, 0, &V18694_297_57197, &P18694_295};	/*Property = CssTransforms*/
const Value V18694_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_241 = {S19, 0, &V18694_241_55801, &P18694_297};	/*Property = CssTransitions*/
const Value V18694_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_298 = {S20, 0, &V18694_298_57208, &P18694_241};	/*Property = CssUI*/
const Value V18694_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_300 = {S21, 0, &V18694_300_57217, &P18694_298};	/*Property = DataSet*/
const Value V18694_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_301 = {S22, 0, &V18694_301_57220, &P18694_300};	/*Property = DataUrl*/
const Value V18694_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_302 = {S23, 0, &V18694_302_57223, &P18694_301};	/*Property = DeviceOrientation*/
const Value V18694_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_304 = {S24, 0, &V18694_304_57229, &P18694_302};	/*Property = FileReader*/
const Value V18694_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_305 = {S25, 0, &V18694_305_57232, &P18694_304};	/*Property = FileSaver*/
const Value V18694_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_306 = {S26, 0, &V18694_306_57238, &P18694_305};	/*Property = FileWriter*/
const Value V18694_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_307 = {S27, 0, &V18694_307_57241, &P18694_306};	/*Property = FormData*/
const Value V18694_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_309 = {S28, 0, &V18694_309_57247, &P18694_307};	/*Property = Fullscreen*/
const Value V18694_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_310 = {S29, 0, &V18694_310_57250, &P18694_309};	/*Property = GeoLocation*/
const Value V18694_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_312 = {S30, 0, &V18694_312_57256, &P18694_310};	/*Property = History*/
const Value V18694_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_314 = {S31, 0, &V18694_314_57262, &P18694_312};	/*Property = Html5*/
const Value V18694_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_313 = {S32, 0, &V18694_313_57259, &P18694_314};	/*Property = Html-Media-Capture*/
const Value V18694_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_315 = {S33, 0, &V18694_315_57265, &P18694_313};	/*Property = Iframe*/
const Value V18694_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_316 = {S34, 0, &V18694_316_57268, &P18694_315};	/*Property = IndexedDB*/
const Value V18694_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_317 = {S35, 0, &V18694_317_57271, &P18694_316};	/*Property = Json*/
const Value V18694_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18694_15 = {S37, 0, &V18694_15_28, &P18694_317};	/*Property = LayoutEngine*/
const Value V18694_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_344 = {S38, 0, &V18694_344_57274, &P18694_15};	/*Property = Masking*/
const Value V18694_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_320 = {S39, 0, &V18694_320_57292, &P18694_344};	/*Property = PostMessage*/
const Value V18694_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_321 = {S40, 0, &V18694_321_57295, &P18694_320};	/*Property = Progress*/
const Value V18694_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_322 = {S41, 0, &V18694_322_57298, &P18694_321};	/*Property = Prompts*/
const Value V18694_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_323 = {S42, 0, &V18694_323_57301, &P18694_322};	/*Property = Selector*/
const Value V18694_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_324 = {S43, 0, &V18694_324_57307, &P18694_323};	/*Property = Svg*/
const Value V18694_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_327 = {S44, 0, &V18694_327_57316, &P18694_324};	/*Property = TouchEvents*/
const Value V18694_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_328 = {S45, 0, &V18694_328_57319, &P18694_327};	/*Property = Track*/
const Value V18694_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_329 = {S46, 0, &V18694_329_57325, &P18694_328};	/*Property = Video*/
const Value V18694_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_330 = {S47, 0, &V18694_330_57328, &P18694_329};	/*Property = Viewport*/
const Value V18694_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_332 = {S48, 0, &V18694_332_57340, &P18694_330};	/*Property = WebWorkers*/
const Value V18694_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18694_333 = {S49, 0, &V18694_333_57343, &P18694_332};	/*Property = Xhr2*/
const Profile P18694 = {"18694", &P18694_333};	/*Profile ID - 18694*/
const Value V20859_1_1 = {S2, NULL};	/*Value = True*/
const Property P20859_1 = {S50, 0, &V20859_1_1, NULL};	/*Property = IsMobile*/
const Value V20859_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20859_5 = {S52, 0, &V20859_5_1227, &P20859_1};	/*Property = ScreenPixelsHeight*/
const Value V20859_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20859_4 = {S54, 0, &V20859_4_1229, &P20859_5};	/*Property = ScreenPixelsWidth*/
const Profile P20859 = {"20859", &P20859_4};	/*Profile ID - 20859*/
const Value V19806_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_279 = {S1, 0, &V19806_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V19806_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_282 = {S3, 0, &V19806_282_57155, &P19806_279};	/*Property = BlobBuilder*/
const Value V19806_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_350 = {S4, 0, &V19806_350_57446, &P19806_282};	/*Property = Canvas*/
const Value V19806_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_283 = {S5, 0, &V19806_283_57161, &P19806_350};	/*Property = CssBackground*/
const Value V19806_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_284 = {S6, 0, &V19806_284_57164, &P19806_283};	/*Property = CssBorderImage*/
const Value V19806_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_285 = {S7, 0, &V19806_285_57167, &P19806_284};	/*Property = CssCanvas*/
const Value V19806_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_286 = {S8, 0, &V19806_286_57170, &P19806_285};	/*Property = CssColor*/
const Value V19806_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_237 = {S9, 0, &V19806_237_55786, &P19806_286};	/*Property = CssColumn*/
const Value V19806_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_287 = {S10, 0, &V19806_287_57173, &P19806_237};	/*Property = CssFlexbox*/
const Value V19806_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_288 = {S11, 0, &V19806_288_57176, &P19806_287};	/*Property = CssFont*/
const Value V19806_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_289 = {S12, 0, &V19806_289_86899, &P19806_288};	/*Property = CssImages*/
const Value V19806_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_291 = {S13, 0, &V19806_291_57182, &P19806_289};	/*Property = CssMediaQueries*/
const Value V19806_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_292 = {S14, 0, &V19806_292_57185, &P19806_291};	/*Property = CssMinMax*/
const Value V19806_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_293 = {S15, 0, &V19806_293_57188, &P19806_292};	/*Property = CssOverflow*/
const Value V19806_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_294 = {S16, 0, &V19806_294_57191, &P19806_293};	/*Property = CssPosition*/
const Value V19806_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_295 = {S17, 0, &V19806_295_57194, &P19806_294};	/*Property = CssText*/
const Value V19806_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_297 = {S18, 0, &V19806_297_57197, &P19806_295};	/*Property = CssTransforms*/
const Value V19806_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_241 = {S19, 0, &V19806_241_55801, &P19806_297};	/*Property = CssTransitions*/
const Value V19806_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_298 = {S20, 0, &V19806_298_57208, &P19806_241};	/*Property = CssUI*/
const Value V19806_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_300 = {S21, 0, &V19806_300_57217, &P19806_298};	/*Property = DataSet*/
const Value V19806_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_301 = {S22, 0, &V19806_301_57220, &P19806_300};	/*Property = DataUrl*/
const Value V19806_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_302 = {S23, 0, &V19806_302_57223, &P19806_301};	/*Property = DeviceOrientation*/
const Value V19806_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_304 = {S24, 0, &V19806_304_57229, &P19806_302};	/*Property = FileReader*/
const Value V19806_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_305 = {S25, 0, &V19806_305_57232, &P19806_304};	/*Property = FileSaver*/
const Value V19806_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_306 = {S26, 0, &V19806_306_57238, &P19806_305};	/*Property = FileWriter*/
const Value V19806_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_307 = {S27, 0, &V19806_307_57241, &P19806_306};	/*Property = FormData*/
const Value V19806_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_309 = {S28, 0, &V19806_309_57247, &P19806_307};	/*Property = Fullscreen*/
const Value V19806_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_310 = {S29, 0, &V19806_310_57250, &P19806_309};	/*Property = GeoLocation*/
const Value V19806_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_312 = {S30, 0, &V19806_312_57256, &P19806_310};	/*Property = History*/
const Value V19806_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_314 = {S31, 0, &V19806_314_57262, &P19806_312};	/*Property = Html5*/
const Value V19806_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_313 = {S32, 0, &V19806_313_57259, &P19806_314};	/*Property = Html-Media-Capture*/
const Value V19806_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_315 = {S33, 0, &V19806_315_57265, &P19806_313};	/*Property = Iframe*/
const Value V19806_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_316 = {S34, 0, &V19806_316_57268, &P19806_315};	/*Property = IndexedDB*/
const Value V19806_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_317 = {S35, 0, &V19806_317_57271, &P19806_316};	/*Property = Json*/
const Value V19806_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P19806_15 = {S37, 0, &V19806_15_29, &P19806_317};	/*Property = LayoutEngine*/
const Value V19806_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_344 = {S38, 0, &V19806_344_57274, &P19806_15};	/*Property = Masking*/
const Value V19806_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_320 = {S39, 0, &V19806_320_57292, &P19806_344};	/*Property = PostMessage*/
const Value V19806_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_321 = {S40, 0, &V19806_321_57295, &P19806_320};	/*Property = Progress*/
const Value V19806_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_322 = {S41, 0, &V19806_322_57298, &P19806_321};	/*Property = Prompts*/
const Value V19806_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_323 = {S42, 0, &V19806_323_57301, &P19806_322};	/*Property = Selector*/
const Value V19806_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_324 = {S43, 0, &V19806_324_57307, &P19806_323};	/*Property = Svg*/
const Value V19806_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_327 = {S44, 0, &V19806_327_57316, &P19806_324};	/*Property = TouchEvents*/
const Value V19806_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_328 = {S45, 0, &V19806_328_57319, &P19806_327};	/*Property = Track*/
const Value V19806_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_329 = {S46, 0, &V19806_329_57325, &P19806_328};	/*Property = Video*/
const Value V19806_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_330 = {S47, 0, &V19806_330_57328, &P19806_329};	/*Property = Viewport*/
const Value V19806_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_332 = {S48, 0, &V19806_332_57340, &P19806_330};	/*Property = WebWorkers*/
const Value V19806_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P19806_333 = {S49, 0, &V19806_333_57343, &P19806_332};	/*Property = Xhr2*/
const Profile P19806 = {"19806", &P19806_333};	/*Profile ID - 19806*/
const Value V21247_1_1 = {S2, NULL};	/*Value = True*/
const Property P21247_1 = {S50, 0, &V21247_1_1, NULL};	/*Property = IsMobile*/
const Value V21247_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21247_5 = {S52, 0, &V21247_5_1277, &P21247_1};	/*Property = ScreenPixelsHeight*/
const Value V21247_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21247_4 = {S54, 0, &V21247_4_1279, &P21247_5};	/*Property = ScreenPixelsWidth*/
const Profile P21247 = {"21247", &P21247_4};	/*Profile ID - 21247*/
const Value V21596_1_1 = {S2, NULL};	/*Value = True*/
const Property P21596_1 = {S50, 0, &V21596_1_1, NULL};	/*Property = IsMobile*/
const Value V21596_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21596_5 = {S52, 0, &V21596_5_1227, &P21596_1};	/*Property = ScreenPixelsHeight*/
const Value V21596_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21596_4 = {S54, 0, &V21596_4_1229, &P21596_5};	/*Property = ScreenPixelsWidth*/
const Profile P21596 = {"21596", &P21596_4};	/*Profile ID - 21596*/
const Value V19746_1_1 = {S2, NULL};	/*Value = True*/
const Property P19746_1 = {S50, 0, &V19746_1_1, NULL};	/*Property = IsMobile*/
const Value V19746_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19746_5 = {S52, 0, &V19746_5_1227, &P19746_1};	/*Property = ScreenPixelsHeight*/
const Value V19746_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19746_4 = {S54, 0, &V19746_4_1229, &P19746_5};	/*Property = ScreenPixelsWidth*/
const Profile P19746 = {"19746", &P19746_4};	/*Profile ID - 19746*/
const Value V21230_1_1 = {S2, NULL};	/*Value = True*/
const Property P21230_1 = {S50, 0, &V21230_1_1, NULL};	/*Property = IsMobile*/
const Value V21230_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21230_5 = {S52, 0, &V21230_5_1277, &P21230_1};	/*Property = ScreenPixelsHeight*/
const Value V21230_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21230_4 = {S54, 0, &V21230_4_1279, &P21230_5};	/*Property = ScreenPixelsWidth*/
const Profile P21230 = {"21230", &P21230_4};	/*Profile ID - 21230*/
const Value V22124_1_1 = {S2, NULL};	/*Value = True*/
const Property P22124_1 = {S50, 0, &V22124_1_1, NULL};	/*Property = IsMobile*/
const Value V22124_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22124_5 = {S52, 0, &V22124_5_1832, &P22124_1};	/*Property = ScreenPixelsHeight*/
const Value V22124_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22124_4 = {S54, 0, &V22124_4_2336, &P22124_5};	/*Property = ScreenPixelsWidth*/
const Profile P22124 = {"22124", &P22124_4};	/*Profile ID - 22124*/
const Value V20330_1_1 = {S2, NULL};	/*Value = True*/
const Property P20330_1 = {S50, 0, &V20330_1_1, NULL};	/*Property = IsMobile*/
const Value V20330_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20330_5 = {S52, 0, &V20330_5_37, &P20330_1};	/*Property = ScreenPixelsHeight*/
const Value V20330_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20330_4 = {S54, 0, &V20330_4_36, &P20330_5};	/*Property = ScreenPixelsWidth*/
const Profile P20330 = {"20330", &P20330_4};	/*Profile ID - 20330*/
const Profile P17290 = {"17290", NULL};	/*Profile ID - 17290*/
const Value V21011_1_1 = {S2, NULL};	/*Value = True*/
const Property P21011_1 = {S50, 0, &V21011_1_1, NULL};	/*Property = IsMobile*/
const Value V21011_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21011_5 = {S52, 0, &V21011_5_9981, &P21011_1};	/*Property = ScreenPixelsHeight*/
const Value V21011_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21011_4 = {S54, 0, &V21011_4_39098, &P21011_5};	/*Property = ScreenPixelsWidth*/
const Profile P21011 = {"21011", &P21011_4};	/*Profile ID - 21011*/
const Profile P18646 = {"18646", NULL};	/*Profile ID - 18646*/
const Value V21422_1_1 = {S2, NULL};	/*Value = True*/
const Property P21422_1 = {S50, 0, &V21422_1_1, NULL};	/*Property = IsMobile*/
const Value V21422_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21422_5 = {S52, 0, &V21422_5_1458, &P21422_1};	/*Property = ScreenPixelsHeight*/
const Value V21422_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21422_4 = {S54, 0, &V21422_4_37617, &P21422_5};	/*Property = ScreenPixelsWidth*/
const Profile P21422 = {"21422", &P21422_4};	/*Profile ID - 21422*/
const Value V22815_1_1 = {S2, NULL};	/*Value = True*/
const Property P22815_1 = {S50, 0, &V22815_1_1, NULL};	/*Property = IsMobile*/
const Value V22815_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22815_5 = {S52, 0, &V22815_5_1277, &P22815_1};	/*Property = ScreenPixelsHeight*/
const Value V22815_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22815_4 = {S54, 0, &V22815_4_1279, &P22815_5};	/*Property = ScreenPixelsWidth*/
const Profile P22815 = {"22815", &P22815_4};	/*Profile ID - 22815*/
const Value V20349_1_1 = {S2, NULL};	/*Value = True*/
const Property P20349_1 = {S50, 0, &V20349_1_1, NULL};	/*Property = IsMobile*/
const Value V20349_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20349_5 = {S52, 0, &V20349_5_9981, &P20349_1};	/*Property = ScreenPixelsHeight*/
const Value V20349_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20349_4 = {S54, 0, &V20349_4_2336, &P20349_5};	/*Property = ScreenPixelsWidth*/
const Profile P20349 = {"20349", &P20349_4};	/*Profile ID - 20349*/
const Value V22058_1_1 = {S2, NULL};	/*Value = True*/
const Property P22058_1 = {S50, 0, &V22058_1_1, NULL};	/*Property = IsMobile*/
const Value V22058_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22058_5 = {S52, 0, &V22058_5_1227, &P22058_1};	/*Property = ScreenPixelsHeight*/
const Value V22058_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22058_4 = {S54, 0, &V22058_4_1229, &P22058_5};	/*Property = ScreenPixelsWidth*/
const Profile P22058 = {"22058", &P22058_4};	/*Profile ID - 22058*/
const Value V21616_1_1 = {S2, NULL};	/*Value = True*/
const Property P21616_1 = {S50, 0, &V21616_1_1, NULL};	/*Property = IsMobile*/
const Value V21616_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21616_5 = {S52, 0, &V21616_5_1227, &P21616_1};	/*Property = ScreenPixelsHeight*/
const Value V21616_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21616_4 = {S54, 0, &V21616_4_1229, &P21616_5};	/*Property = ScreenPixelsWidth*/
const Profile P21616 = {"21616", &P21616_4};	/*Profile ID - 21616*/
const Value V21728_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_279 = {S1, 0, &V21728_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21728_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_282 = {S3, 0, &V21728_282_57155, &P21728_279};	/*Property = BlobBuilder*/
const Value V21728_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_350 = {S4, 0, &V21728_350_57446, &P21728_282};	/*Property = Canvas*/
const Value V21728_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_283 = {S5, 0, &V21728_283_57161, &P21728_350};	/*Property = CssBackground*/
const Value V21728_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_284 = {S6, 0, &V21728_284_57164, &P21728_283};	/*Property = CssBorderImage*/
const Value V21728_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_285 = {S7, 0, &V21728_285_57167, &P21728_284};	/*Property = CssCanvas*/
const Value V21728_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_286 = {S8, 0, &V21728_286_57170, &P21728_285};	/*Property = CssColor*/
const Value V21728_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_237 = {S9, 0, &V21728_237_55786, &P21728_286};	/*Property = CssColumn*/
const Value V21728_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_287 = {S10, 0, &V21728_287_57173, &P21728_237};	/*Property = CssFlexbox*/
const Value V21728_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_288 = {S11, 0, &V21728_288_57176, &P21728_287};	/*Property = CssFont*/
const Value V21728_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_289 = {S12, 0, &V21728_289_86899, &P21728_288};	/*Property = CssImages*/
const Value V21728_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_291 = {S13, 0, &V21728_291_57182, &P21728_289};	/*Property = CssMediaQueries*/
const Value V21728_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_292 = {S14, 0, &V21728_292_57185, &P21728_291};	/*Property = CssMinMax*/
const Value V21728_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_293 = {S15, 0, &V21728_293_57188, &P21728_292};	/*Property = CssOverflow*/
const Value V21728_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_294 = {S16, 0, &V21728_294_57191, &P21728_293};	/*Property = CssPosition*/
const Value V21728_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_295 = {S17, 0, &V21728_295_57194, &P21728_294};	/*Property = CssText*/
const Value V21728_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_297 = {S18, 0, &V21728_297_57197, &P21728_295};	/*Property = CssTransforms*/
const Value V21728_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_241 = {S19, 0, &V21728_241_55801, &P21728_297};	/*Property = CssTransitions*/
const Value V21728_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_298 = {S20, 0, &V21728_298_57208, &P21728_241};	/*Property = CssUI*/
const Value V21728_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_300 = {S21, 0, &V21728_300_57217, &P21728_298};	/*Property = DataSet*/
const Value V21728_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_301 = {S22, 0, &V21728_301_57220, &P21728_300};	/*Property = DataUrl*/
const Value V21728_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_302 = {S23, 0, &V21728_302_57223, &P21728_301};	/*Property = DeviceOrientation*/
const Value V21728_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_304 = {S24, 0, &V21728_304_57229, &P21728_302};	/*Property = FileReader*/
const Value V21728_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_305 = {S25, 0, &V21728_305_57232, &P21728_304};	/*Property = FileSaver*/
const Value V21728_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_306 = {S26, 0, &V21728_306_57238, &P21728_305};	/*Property = FileWriter*/
const Value V21728_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_307 = {S27, 0, &V21728_307_57241, &P21728_306};	/*Property = FormData*/
const Value V21728_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_309 = {S28, 0, &V21728_309_57247, &P21728_307};	/*Property = Fullscreen*/
const Value V21728_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_310 = {S29, 0, &V21728_310_57250, &P21728_309};	/*Property = GeoLocation*/
const Value V21728_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_312 = {S30, 0, &V21728_312_57256, &P21728_310};	/*Property = History*/
const Value V21728_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_314 = {S31, 0, &V21728_314_57262, &P21728_312};	/*Property = Html5*/
const Value V21728_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_313 = {S32, 0, &V21728_313_57259, &P21728_314};	/*Property = Html-Media-Capture*/
const Value V21728_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_315 = {S33, 0, &V21728_315_57265, &P21728_313};	/*Property = Iframe*/
const Value V21728_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_316 = {S34, 0, &V21728_316_57268, &P21728_315};	/*Property = IndexedDB*/
const Value V21728_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_317 = {S35, 0, &V21728_317_57271, &P21728_316};	/*Property = Json*/
const Value V21728_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21728_15 = {S37, 0, &V21728_15_54167, &P21728_317};	/*Property = LayoutEngine*/
const Value V21728_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_344 = {S38, 0, &V21728_344_57274, &P21728_15};	/*Property = Masking*/
const Value V21728_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_320 = {S39, 0, &V21728_320_57292, &P21728_344};	/*Property = PostMessage*/
const Value V21728_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_321 = {S40, 0, &V21728_321_57295, &P21728_320};	/*Property = Progress*/
const Value V21728_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_322 = {S41, 0, &V21728_322_57298, &P21728_321};	/*Property = Prompts*/
const Value V21728_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_323 = {S42, 0, &V21728_323_57301, &P21728_322};	/*Property = Selector*/
const Value V21728_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_324 = {S43, 0, &V21728_324_57307, &P21728_323};	/*Property = Svg*/
const Value V21728_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_327 = {S44, 0, &V21728_327_57316, &P21728_324};	/*Property = TouchEvents*/
const Value V21728_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_328 = {S45, 0, &V21728_328_57319, &P21728_327};	/*Property = Track*/
const Value V21728_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_329 = {S46, 0, &V21728_329_57325, &P21728_328};	/*Property = Video*/
const Value V21728_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_330 = {S47, 0, &V21728_330_57328, &P21728_329};	/*Property = Viewport*/
const Value V21728_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_332 = {S48, 0, &V21728_332_57340, &P21728_330};	/*Property = WebWorkers*/
const Value V21728_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21728_333 = {S49, 0, &V21728_333_57343, &P21728_332};	/*Property = Xhr2*/
const Profile P21728 = {"21728", &P21728_333};	/*Profile ID - 21728*/
const Value V23216_279_57148 = {S0, NULL};	/*Value = False*/
const Property P23216_279 = {S1, 0, &V23216_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V23216_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23216_282 = {S3, 0, &V23216_282_57154, &P23216_279};	/*Property = BlobBuilder*/
const Value V23216_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23216_350 = {S4, 0, &V23216_350_57444, &P23216_282};	/*Property = Canvas*/
const Value V23216_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23216_283 = {S5, 0, &V23216_283_57159, &P23216_350};	/*Property = CssBackground*/
const Value V23216_284_57163 = {S0, NULL};	/*Value = False*/
const Property P23216_284 = {S6, 0, &V23216_284_57163, &P23216_283};	/*Property = CssBorderImage*/
const Value V23216_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23216_285 = {S7, 0, &V23216_285_57166, &P23216_284};	/*Property = CssCanvas*/
const Value V23216_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23216_286 = {S8, 0, &V23216_286_57168, &P23216_285};	/*Property = CssColor*/
const Value V23216_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23216_237 = {S9, 0, &V23216_237_55786, &P23216_286};	/*Property = CssColumn*/
const Value V23216_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23216_287 = {S10, 0, &V23216_287_57172, &P23216_237};	/*Property = CssFlexbox*/
const Value V23216_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23216_288 = {S11, 0, &V23216_288_57174, &P23216_287};	/*Property = CssFont*/
const Value V23216_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23216_289 = {S12, 0, &V23216_289_57759, &P23216_288};	/*Property = CssImages*/
const Value V23216_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23216_291 = {S13, 0, &V23216_291_57180, &P23216_289};	/*Property = CssMediaQueries*/
const Value V23216_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23216_292 = {S14, 0, &V23216_292_57183, &P23216_291};	/*Property = CssMinMax*/
const Value V23216_293_57187 = {S0, NULL};	/*Value = False*/
const Property P23216_293 = {S15, 0, &V23216_293_57187, &P23216_292};	/*Property = CssOverflow*/
const Value V23216_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23216_294 = {S16, 0, &V23216_294_57189, &P23216_293};	/*Property = CssPosition*/
const Value V23216_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23216_295 = {S17, 0, &V23216_295_57192, &P23216_294};	/*Property = CssText*/
const Value V23216_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23216_297 = {S18, 0, &V23216_297_57195, &P23216_295};	/*Property = CssTransforms*/
const Value V23216_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23216_241 = {S19, 0, &V23216_241_55799, &P23216_297};	/*Property = CssTransitions*/
const Value V23216_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23216_298 = {S20, 0, &V23216_298_57206, &P23216_241};	/*Property = CssUI*/
const Value V23216_300_57216 = {S0, NULL};	/*Value = False*/
const Property P23216_300 = {S21, 0, &V23216_300_57216, &P23216_298};	/*Property = DataSet*/
const Value V23216_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23216_301 = {S22, 0, &V23216_301_57218, &P23216_300};	/*Property = DataUrl*/
const Value V23216_302_57221 = {S2, NULL};	/*Value = True*/
const Property P23216_302 = {S23, 0, &V23216_302_57221, &P23216_301};	/*Property = DeviceOrientation*/
const Value V23216_304_57227 = {S2, NULL};	/*Value = True*/
const Property P23216_304 = {S24, 0, &V23216_304_57227, &P23216_302};	/*Property = FileReader*/
const Value V23216_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23216_305 = {S25, 0, &V23216_305_57231, &P23216_304};	/*Property = FileSaver*/
const Value V23216_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23216_306 = {S26, 0, &V23216_306_57237, &P23216_305};	/*Property = FileWriter*/
const Value V23216_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23216_307 = {S27, 0, &V23216_307_57239, &P23216_306};	/*Property = FormData*/
const Value V23216_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23216_309 = {S28, 0, &V23216_309_57246, &P23216_307};	/*Property = Fullscreen*/
const Value V23216_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23216_310 = {S29, 0, &V23216_310_57248, &P23216_309};	/*Property = GeoLocation*/
const Value V23216_312_57254 = {S2, NULL};	/*Value = True*/
const Property P23216_312 = {S30, 0, &V23216_312_57254, &P23216_310};	/*Property = History*/
const Value V23216_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23216_314 = {S31, 0, &V23216_314_57261, &P23216_312};	/*Property = Html5*/
const Value V23216_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23216_313 = {S32, 0, &V23216_313_57258, &P23216_314};	/*Property = Html-Media-Capture*/
const Value V23216_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23216_315 = {S33, 0, &V23216_315_57264, &P23216_313};	/*Property = Iframe*/
const Value V23216_316_57266 = {S2, NULL};	/*Value = True*/
const Property P23216_316 = {S34, 0, &V23216_316_57266, &P23216_315};	/*Property = IndexedDB*/
const Value V23216_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23216_317 = {S35, 0, &V23216_317_57269, &P23216_316};	/*Property = Json*/
const Value V23216_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P23216_15 = {S37, 0, &V23216_15_29, &P23216_317};	/*Property = LayoutEngine*/
const Value V23216_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23216_344 = {S38, 0, &V23216_344_57272, &P23216_15};	/*Property = Masking*/
const Value V23216_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23216_320 = {S39, 0, &V23216_320_57290, &P23216_344};	/*Property = PostMessage*/
const Value V23216_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23216_321 = {S40, 0, &V23216_321_57293, &P23216_320};	/*Property = Progress*/
const Value V23216_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23216_322 = {S41, 0, &V23216_322_57296, &P23216_321};	/*Property = Prompts*/
const Value V23216_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23216_323 = {S42, 0, &V23216_323_57299, &P23216_322};	/*Property = Selector*/
const Value V23216_324_57306 = {S0, NULL};	/*Value = False*/
const Property P23216_324 = {S43, 0, &V23216_324_57306, &P23216_323};	/*Property = Svg*/
const Value V23216_327_57314 = {S2, NULL};	/*Value = True*/
const Property P23216_327 = {S44, 0, &V23216_327_57314, &P23216_324};	/*Property = TouchEvents*/
const Value V23216_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23216_328 = {S45, 0, &V23216_328_57318, &P23216_327};	/*Property = Track*/
const Value V23216_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23216_329 = {S46, 0, &V23216_329_57323, &P23216_328};	/*Property = Video*/
const Value V23216_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23216_330 = {S47, 0, &V23216_330_57326, &P23216_329};	/*Property = Viewport*/
const Value V23216_332_57338 = {S2, NULL};	/*Value = True*/
const Property P23216_332 = {S48, 0, &V23216_332_57338, &P23216_330};	/*Property = WebWorkers*/
const Value V23216_333_57341 = {S2, NULL};	/*Value = True*/
const Property P23216_333 = {S49, 0, &V23216_333_57341, &P23216_332};	/*Property = Xhr2*/
const Profile P23216 = {"23216", &P23216_333};	/*Profile ID - 23216*/
const Value V20799_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20799_279 = {S1, 0, &V20799_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20799_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20799_282 = {S3, 0, &V20799_282_57154, &P20799_279};	/*Property = BlobBuilder*/
const Value V20799_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20799_350 = {S4, 0, &V20799_350_57444, &P20799_282};	/*Property = Canvas*/
const Value V20799_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20799_283 = {S5, 0, &V20799_283_57159, &P20799_350};	/*Property = CssBackground*/
const Value V20799_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20799_284 = {S6, 0, &V20799_284_57163, &P20799_283};	/*Property = CssBorderImage*/
const Value V20799_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20799_285 = {S7, 0, &V20799_285_57166, &P20799_284};	/*Property = CssCanvas*/
const Value V20799_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20799_286 = {S8, 0, &V20799_286_57168, &P20799_285};	/*Property = CssColor*/
const Value V20799_237_55784 = {S2, NULL};	/*Value = True*/
const Property P20799_237 = {S9, 0, &V20799_237_55784, &P20799_286};	/*Property = CssColumn*/
const Value V20799_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20799_287 = {S10, 0, &V20799_287_57172, &P20799_237};	/*Property = CssFlexbox*/
const Value V20799_288_57174 = {S2, NULL};	/*Value = True*/
const Property P20799_288 = {S11, 0, &V20799_288_57174, &P20799_287};	/*Property = CssFont*/
const Value V20799_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20799_289 = {S12, 0, &V20799_289_57759, &P20799_288};	/*Property = CssImages*/
const Value V20799_291_57180 = {S2, NULL};	/*Value = True*/
const Property P20799_291 = {S13, 0, &V20799_291_57180, &P20799_289};	/*Property = CssMediaQueries*/
const Value V20799_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20799_292 = {S14, 0, &V20799_292_57183, &P20799_291};	/*Property = CssMinMax*/
const Value V20799_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20799_293 = {S15, 0, &V20799_293_57187, &P20799_292};	/*Property = CssOverflow*/
const Value V20799_294_57189 = {S2, NULL};	/*Value = True*/
const Property P20799_294 = {S16, 0, &V20799_294_57189, &P20799_293};	/*Property = CssPosition*/
const Value V20799_295_57192 = {S2, NULL};	/*Value = True*/
const Property P20799_295 = {S17, 0, &V20799_295_57192, &P20799_294};	/*Property = CssText*/
const Value V20799_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20799_297 = {S18, 0, &V20799_297_57195, &P20799_295};	/*Property = CssTransforms*/
const Value V20799_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20799_241 = {S19, 0, &V20799_241_55799, &P20799_297};	/*Property = CssTransitions*/
const Value V20799_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20799_298 = {S20, 0, &V20799_298_57206, &P20799_241};	/*Property = CssUI*/
const Value V20799_300_57215 = {S2, NULL};	/*Value = True*/
const Property P20799_300 = {S21, 0, &V20799_300_57215, &P20799_298};	/*Property = DataSet*/
const Value V20799_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20799_301 = {S22, 0, &V20799_301_57218, &P20799_300};	/*Property = DataUrl*/
const Value V20799_302_57221 = {S2, NULL};	/*Value = True*/
const Property P20799_302 = {S23, 0, &V20799_302_57221, &P20799_301};	/*Property = DeviceOrientation*/
const Value V20799_304_57227 = {S2, NULL};	/*Value = True*/
const Property P20799_304 = {S24, 0, &V20799_304_57227, &P20799_302};	/*Property = FileReader*/
const Value V20799_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20799_305 = {S25, 0, &V20799_305_57231, &P20799_304};	/*Property = FileSaver*/
const Value V20799_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20799_306 = {S26, 0, &V20799_306_57237, &P20799_305};	/*Property = FileWriter*/
const Value V20799_307_57239 = {S2, NULL};	/*Value = True*/
const Property P20799_307 = {S27, 0, &V20799_307_57239, &P20799_306};	/*Property = FormData*/
const Value V20799_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20799_309 = {S28, 0, &V20799_309_57246, &P20799_307};	/*Property = Fullscreen*/
const Value V20799_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20799_310 = {S29, 0, &V20799_310_57248, &P20799_309};	/*Property = GeoLocation*/
const Value V20799_312_57254 = {S2, NULL};	/*Value = True*/
const Property P20799_312 = {S30, 0, &V20799_312_57254, &P20799_310};	/*Property = History*/
const Value V20799_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20799_314 = {S31, 0, &V20799_314_57261, &P20799_312};	/*Property = Html5*/
const Value V20799_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20799_313 = {S32, 0, &V20799_313_57258, &P20799_314};	/*Property = Html-Media-Capture*/
const Value V20799_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20799_315 = {S33, 0, &V20799_315_57264, &P20799_313};	/*Property = Iframe*/
const Value V20799_316_57266 = {S2, NULL};	/*Value = True*/
const Property P20799_316 = {S34, 0, &V20799_316_57266, &P20799_315};	/*Property = IndexedDB*/
const Value V20799_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20799_317 = {S35, 0, &V20799_317_57269, &P20799_316};	/*Property = Json*/
const Value V20799_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20799_15 = {S37, 0, &V20799_15_29, &P20799_317};	/*Property = LayoutEngine*/
const Value V20799_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20799_344 = {S38, 0, &V20799_344_57272, &P20799_15};	/*Property = Masking*/
const Value V20799_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20799_320 = {S39, 0, &V20799_320_57290, &P20799_344};	/*Property = PostMessage*/
const Value V20799_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20799_321 = {S40, 0, &V20799_321_57293, &P20799_320};	/*Property = Progress*/
const Value V20799_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20799_322 = {S41, 0, &V20799_322_57296, &P20799_321};	/*Property = Prompts*/
const Value V20799_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20799_323 = {S42, 0, &V20799_323_57299, &P20799_322};	/*Property = Selector*/
const Value V20799_324_57305 = {S2, NULL};	/*Value = True*/
const Property P20799_324 = {S43, 0, &V20799_324_57305, &P20799_323};	/*Property = Svg*/
const Value V20799_327_57315 = {S0, NULL};	/*Value = False*/
const Property P20799_327 = {S44, 0, &V20799_327_57315, &P20799_324};	/*Property = TouchEvents*/
const Value V20799_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20799_328 = {S45, 0, &V20799_328_57318, &P20799_327};	/*Property = Track*/
const Value V20799_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20799_329 = {S46, 0, &V20799_329_57323, &P20799_328};	/*Property = Video*/
const Value V20799_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20799_330 = {S47, 0, &V20799_330_57326, &P20799_329};	/*Property = Viewport*/
const Value V20799_332_57338 = {S2, NULL};	/*Value = True*/
const Property P20799_332 = {S48, 0, &V20799_332_57338, &P20799_330};	/*Property = WebWorkers*/
const Value V20799_333_57341 = {S2, NULL};	/*Value = True*/
const Property P20799_333 = {S49, 0, &V20799_333_57341, &P20799_332};	/*Property = Xhr2*/
const Profile P20799 = {"20799", &P20799_333};	/*Profile ID - 20799*/
const Value V22816_1_1 = {S2, NULL};	/*Value = True*/
const Property P22816_1 = {S50, 0, &V22816_1_1, NULL};	/*Property = IsMobile*/
const Value V22816_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22816_5 = {S52, 0, &V22816_5_1227, &P22816_1};	/*Property = ScreenPixelsHeight*/
const Value V22816_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22816_4 = {S54, 0, &V22816_4_1229, &P22816_5};	/*Property = ScreenPixelsWidth*/
const Profile P22816 = {"22816", &P22816_4};	/*Profile ID - 22816*/
const Value V21831_1_1 = {S2, NULL};	/*Value = True*/
const Property P21831_1 = {S50, 0, &V21831_1_1, NULL};	/*Property = IsMobile*/
const Value V21831_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21831_5 = {S52, 0, &V21831_5_1277, &P21831_1};	/*Property = ScreenPixelsHeight*/
const Value V21831_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21831_4 = {S54, 0, &V21831_4_2336, &P21831_5};	/*Property = ScreenPixelsWidth*/
const Profile P21831 = {"21831", &P21831_4};	/*Profile ID - 21831*/
const Value V20869_1_1 = {S2, NULL};	/*Value = True*/
const Property P20869_1 = {S50, 0, &V20869_1_1, NULL};	/*Property = IsMobile*/
const Value V20869_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20869_5 = {S52, 0, &V20869_5_1227, &P20869_1};	/*Property = ScreenPixelsHeight*/
const Value V20869_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20869_4 = {S54, 0, &V20869_4_1229, &P20869_5};	/*Property = ScreenPixelsWidth*/
const Profile P20869 = {"20869", &P20869_4};	/*Profile ID - 20869*/
const Value V22554_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22554_279 = {S1, 0, &V22554_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22554_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22554_282 = {S3, 0, &V22554_282_57153, &P22554_279};	/*Property = BlobBuilder*/
const Value V22554_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22554_350 = {S4, 0, &V22554_350_57444, &P22554_282};	/*Property = Canvas*/
const Value V22554_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22554_283 = {S5, 0, &V22554_283_57159, &P22554_350};	/*Property = CssBackground*/
const Value V22554_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22554_284 = {S6, 0, &V22554_284_57163, &P22554_283};	/*Property = CssBorderImage*/
const Value V22554_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22554_285 = {S7, 0, &V22554_285_57166, &P22554_284};	/*Property = CssCanvas*/
const Value V22554_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22554_286 = {S8, 0, &V22554_286_57168, &P22554_285};	/*Property = CssColor*/
const Value V22554_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22554_237 = {S9, 0, &V22554_237_55786, &P22554_286};	/*Property = CssColumn*/
const Value V22554_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22554_287 = {S10, 0, &V22554_287_57172, &P22554_237};	/*Property = CssFlexbox*/
const Value V22554_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22554_288 = {S11, 0, &V22554_288_57175, &P22554_287};	/*Property = CssFont*/
const Value V22554_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22554_289 = {S12, 0, &V22554_289_57759, &P22554_288};	/*Property = CssImages*/
const Value V22554_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22554_291 = {S13, 0, &V22554_291_57181, &P22554_289};	/*Property = CssMediaQueries*/
const Value V22554_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22554_292 = {S14, 0, &V22554_292_57183, &P22554_291};	/*Property = CssMinMax*/
const Value V22554_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22554_293 = {S15, 0, &V22554_293_57187, &P22554_292};	/*Property = CssOverflow*/
const Value V22554_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22554_294 = {S16, 0, &V22554_294_57189, &P22554_293};	/*Property = CssPosition*/
const Value V22554_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22554_295 = {S17, 0, &V22554_295_57192, &P22554_294};	/*Property = CssText*/
const Value V22554_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22554_297 = {S18, 0, &V22554_297_57195, &P22554_295};	/*Property = CssTransforms*/
const Value V22554_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22554_241 = {S19, 0, &V22554_241_55799, &P22554_297};	/*Property = CssTransitions*/
const Value V22554_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22554_298 = {S20, 0, &V22554_298_57206, &P22554_241};	/*Property = CssUI*/
const Value V22554_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22554_300 = {S21, 0, &V22554_300_57216, &P22554_298};	/*Property = DataSet*/
const Value V22554_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22554_301 = {S22, 0, &V22554_301_57218, &P22554_300};	/*Property = DataUrl*/
const Value V22554_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22554_302 = {S23, 0, &V22554_302_57221, &P22554_301};	/*Property = DeviceOrientation*/
const Value V22554_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22554_304 = {S24, 0, &V22554_304_57227, &P22554_302};	/*Property = FileReader*/
const Value V22554_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22554_305 = {S25, 0, &V22554_305_57231, &P22554_304};	/*Property = FileSaver*/
const Value V22554_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22554_306 = {S26, 0, &V22554_306_57237, &P22554_305};	/*Property = FileWriter*/
const Value V22554_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22554_307 = {S27, 0, &V22554_307_57239, &P22554_306};	/*Property = FormData*/
const Value V22554_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22554_309 = {S28, 0, &V22554_309_57246, &P22554_307};	/*Property = Fullscreen*/
const Value V22554_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22554_310 = {S29, 0, &V22554_310_57248, &P22554_309};	/*Property = GeoLocation*/
const Value V22554_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22554_312 = {S30, 0, &V22554_312_57254, &P22554_310};	/*Property = History*/
const Value V22554_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22554_314 = {S31, 0, &V22554_314_57261, &P22554_312};	/*Property = Html5*/
const Value V22554_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22554_313 = {S32, 0, &V22554_313_57258, &P22554_314};	/*Property = Html-Media-Capture*/
const Value V22554_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22554_315 = {S33, 0, &V22554_315_57264, &P22554_313};	/*Property = Iframe*/
const Value V22554_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22554_316 = {S34, 0, &V22554_316_57266, &P22554_315};	/*Property = IndexedDB*/
const Value V22554_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22554_317 = {S35, 0, &V22554_317_57269, &P22554_316};	/*Property = Json*/
const Value V22554_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22554_15 = {S37, 0, &V22554_15_29, &P22554_317};	/*Property = LayoutEngine*/
const Value V22554_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22554_344 = {S38, 0, &V22554_344_57272, &P22554_15};	/*Property = Masking*/
const Value V22554_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22554_320 = {S39, 0, &V22554_320_57290, &P22554_344};	/*Property = PostMessage*/
const Value V22554_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22554_321 = {S40, 0, &V22554_321_57293, &P22554_320};	/*Property = Progress*/
const Value V22554_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22554_322 = {S41, 0, &V22554_322_57296, &P22554_321};	/*Property = Prompts*/
const Value V22554_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22554_323 = {S42, 0, &V22554_323_57299, &P22554_322};	/*Property = Selector*/
const Value V22554_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22554_324 = {S43, 0, &V22554_324_57306, &P22554_323};	/*Property = Svg*/
const Value V22554_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22554_327 = {S44, 0, &V22554_327_57315, &P22554_324};	/*Property = TouchEvents*/
const Value V22554_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22554_328 = {S45, 0, &V22554_328_57318, &P22554_327};	/*Property = Track*/
const Value V22554_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22554_329 = {S46, 0, &V22554_329_57323, &P22554_328};	/*Property = Video*/
const Value V22554_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22554_330 = {S47, 0, &V22554_330_57326, &P22554_329};	/*Property = Viewport*/
const Value V22554_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22554_332 = {S48, 0, &V22554_332_57339, &P22554_330};	/*Property = WebWorkers*/
const Value V22554_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22554_333 = {S49, 0, &V22554_333_57341, &P22554_332};	/*Property = Xhr2*/
const Profile P22554 = {"22554", &P22554_333};	/*Profile ID - 22554*/
const Value V18595_1_1 = {S2, NULL};	/*Value = True*/
const Property P18595_1 = {S50, 0, &V18595_1_1, NULL};	/*Property = IsMobile*/
const Value V18595_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18595_5 = {S52, 0, &V18595_5_1227, &P18595_1};	/*Property = ScreenPixelsHeight*/
const Value V18595_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18595_4 = {S54, 0, &V18595_4_1229, &P18595_5};	/*Property = ScreenPixelsWidth*/
const Profile P18595 = {"18595", &P18595_4};	/*Profile ID - 18595*/
const Value V23771_1_1 = {S2, NULL};	/*Value = True*/
const Property P23771_1 = {S50, 0, &V23771_1_1, NULL};	/*Property = IsMobile*/
const Value V23771_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23771_5 = {S52, 0, &V23771_5_1227, &P23771_1};	/*Property = ScreenPixelsHeight*/
const Value V23771_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23771_4 = {S54, 0, &V23771_4_1229, &P23771_5};	/*Property = ScreenPixelsWidth*/
const Profile P23771 = {"23771", &P23771_4};	/*Profile ID - 23771*/
const Value V18635_1_1 = {S2, NULL};	/*Value = True*/
const Property P18635_1 = {S50, 0, &V18635_1_1, NULL};	/*Property = IsMobile*/
const Value V18635_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18635_5 = {S52, 0, &V18635_5_9981, &P18635_1};	/*Property = ScreenPixelsHeight*/
const Value V18635_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P18635_4 = {S54, 0, &V18635_4_37617, &P18635_5};	/*Property = ScreenPixelsWidth*/
const Profile P18635 = {"18635", &P18635_4};	/*Profile ID - 18635*/
const Value V19802_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19802_279 = {S1, 0, &V19802_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19802_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19802_282 = {S3, 0, &V19802_282_57154, &P19802_279};	/*Property = BlobBuilder*/
const Value V19802_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19802_350 = {S4, 0, &V19802_350_57444, &P19802_282};	/*Property = Canvas*/
const Value V19802_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19802_283 = {S5, 0, &V19802_283_57159, &P19802_350};	/*Property = CssBackground*/
const Value V19802_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19802_284 = {S6, 0, &V19802_284_57163, &P19802_283};	/*Property = CssBorderImage*/
const Value V19802_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19802_285 = {S7, 0, &V19802_285_57166, &P19802_284};	/*Property = CssCanvas*/
const Value V19802_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19802_286 = {S8, 0, &V19802_286_57168, &P19802_285};	/*Property = CssColor*/
const Value V19802_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19802_237 = {S9, 0, &V19802_237_55786, &P19802_286};	/*Property = CssColumn*/
const Value V19802_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19802_287 = {S10, 0, &V19802_287_57172, &P19802_237};	/*Property = CssFlexbox*/
const Value V19802_288_57174 = {S2, NULL};	/*Value = True*/
const Property P19802_288 = {S11, 0, &V19802_288_57174, &P19802_287};	/*Property = CssFont*/
const Value V19802_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19802_289 = {S12, 0, &V19802_289_57759, &P19802_288};	/*Property = CssImages*/
const Value V19802_291_57180 = {S2, NULL};	/*Value = True*/
const Property P19802_291 = {S13, 0, &V19802_291_57180, &P19802_289};	/*Property = CssMediaQueries*/
const Value V19802_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19802_292 = {S14, 0, &V19802_292_57183, &P19802_291};	/*Property = CssMinMax*/
const Value V19802_293_57186 = {S2, NULL};	/*Value = True*/
const Property P19802_293 = {S15, 0, &V19802_293_57186, &P19802_292};	/*Property = CssOverflow*/
const Value V19802_294_57189 = {S2, NULL};	/*Value = True*/
const Property P19802_294 = {S16, 0, &V19802_294_57189, &P19802_293};	/*Property = CssPosition*/
const Value V19802_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19802_295 = {S17, 0, &V19802_295_57192, &P19802_294};	/*Property = CssText*/
const Value V19802_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19802_297 = {S18, 0, &V19802_297_57195, &P19802_295};	/*Property = CssTransforms*/
const Value V19802_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19802_241 = {S19, 0, &V19802_241_55799, &P19802_297};	/*Property = CssTransitions*/
const Value V19802_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19802_298 = {S20, 0, &V19802_298_57206, &P19802_241};	/*Property = CssUI*/
const Value V19802_300_57215 = {S2, NULL};	/*Value = True*/
const Property P19802_300 = {S21, 0, &V19802_300_57215, &P19802_298};	/*Property = DataSet*/
const Value V19802_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19802_301 = {S22, 0, &V19802_301_57218, &P19802_300};	/*Property = DataUrl*/
const Value V19802_302_57221 = {S2, NULL};	/*Value = True*/
const Property P19802_302 = {S23, 0, &V19802_302_57221, &P19802_301};	/*Property = DeviceOrientation*/
const Value V19802_304_57227 = {S2, NULL};	/*Value = True*/
const Property P19802_304 = {S24, 0, &V19802_304_57227, &P19802_302};	/*Property = FileReader*/
const Value V19802_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19802_305 = {S25, 0, &V19802_305_57231, &P19802_304};	/*Property = FileSaver*/
const Value V19802_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19802_306 = {S26, 0, &V19802_306_57237, &P19802_305};	/*Property = FileWriter*/
const Value V19802_307_57239 = {S2, NULL};	/*Value = True*/
const Property P19802_307 = {S27, 0, &V19802_307_57239, &P19802_306};	/*Property = FormData*/
const Value V19802_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19802_309 = {S28, 0, &V19802_309_57246, &P19802_307};	/*Property = Fullscreen*/
const Value V19802_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19802_310 = {S29, 0, &V19802_310_57248, &P19802_309};	/*Property = GeoLocation*/
const Value V19802_312_57255 = {S0, NULL};	/*Value = False*/
const Property P19802_312 = {S30, 0, &V19802_312_57255, &P19802_310};	/*Property = History*/
const Value V19802_314_57260 = {S2, NULL};	/*Value = True*/
const Property P19802_314 = {S31, 0, &V19802_314_57260, &P19802_312};	/*Property = Html5*/
const Value V19802_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19802_313 = {S32, 0, &V19802_313_57258, &P19802_314};	/*Property = Html-Media-Capture*/
const Value V19802_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19802_315 = {S33, 0, &V19802_315_57264, &P19802_313};	/*Property = Iframe*/
const Value V19802_316_57267 = {S0, NULL};	/*Value = False*/
const Property P19802_316 = {S34, 0, &V19802_316_57267, &P19802_315};	/*Property = IndexedDB*/
const Value V19802_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19802_317 = {S35, 0, &V19802_317_57269, &P19802_316};	/*Property = Json*/
const Value V19802_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19802_15 = {S37, 0, &V19802_15_28, &P19802_317};	/*Property = LayoutEngine*/
const Value V19802_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19802_344 = {S38, 0, &V19802_344_57272, &P19802_15};	/*Property = Masking*/
const Value V19802_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19802_320 = {S39, 0, &V19802_320_57290, &P19802_344};	/*Property = PostMessage*/
const Value V19802_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19802_321 = {S40, 0, &V19802_321_57293, &P19802_320};	/*Property = Progress*/
const Value V19802_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19802_322 = {S41, 0, &V19802_322_57296, &P19802_321};	/*Property = Prompts*/
const Value V19802_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19802_323 = {S42, 0, &V19802_323_57299, &P19802_322};	/*Property = Selector*/
const Value V19802_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19802_324 = {S43, 0, &V19802_324_57306, &P19802_323};	/*Property = Svg*/
const Value V19802_327_57314 = {S2, NULL};	/*Value = True*/
const Property P19802_327 = {S44, 0, &V19802_327_57314, &P19802_324};	/*Property = TouchEvents*/
const Value V19802_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19802_328 = {S45, 0, &V19802_328_57318, &P19802_327};	/*Property = Track*/
const Value V19802_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19802_329 = {S46, 0, &V19802_329_57323, &P19802_328};	/*Property = Video*/
const Value V19802_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19802_330 = {S47, 0, &V19802_330_57326, &P19802_329};	/*Property = Viewport*/
const Value V19802_332_57339 = {S0, NULL};	/*Value = False*/
const Property P19802_332 = {S48, 0, &V19802_332_57339, &P19802_330};	/*Property = WebWorkers*/
const Value V19802_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19802_333 = {S49, 0, &V19802_333_57342, &P19802_332};	/*Property = Xhr2*/
const Profile P19802 = {"19802", &P19802_333};	/*Profile ID - 19802*/
const Profile P24693 = {"24693", NULL};	/*Profile ID - 24693*/
const Value V22072_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_279 = {S1, 0, &V22072_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22072_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_282 = {S3, 0, &V22072_282_57155, &P22072_279};	/*Property = BlobBuilder*/
const Value V22072_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_350 = {S4, 0, &V22072_350_57446, &P22072_282};	/*Property = Canvas*/
const Value V22072_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_283 = {S5, 0, &V22072_283_57161, &P22072_350};	/*Property = CssBackground*/
const Value V22072_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_284 = {S6, 0, &V22072_284_57164, &P22072_283};	/*Property = CssBorderImage*/
const Value V22072_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_285 = {S7, 0, &V22072_285_57167, &P22072_284};	/*Property = CssCanvas*/
const Value V22072_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_286 = {S8, 0, &V22072_286_57170, &P22072_285};	/*Property = CssColor*/
const Value V22072_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_237 = {S9, 0, &V22072_237_55786, &P22072_286};	/*Property = CssColumn*/
const Value V22072_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_287 = {S10, 0, &V22072_287_57173, &P22072_237};	/*Property = CssFlexbox*/
const Value V22072_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_288 = {S11, 0, &V22072_288_57176, &P22072_287};	/*Property = CssFont*/
const Value V22072_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_289 = {S12, 0, &V22072_289_86899, &P22072_288};	/*Property = CssImages*/
const Value V22072_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_291 = {S13, 0, &V22072_291_57182, &P22072_289};	/*Property = CssMediaQueries*/
const Value V22072_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_292 = {S14, 0, &V22072_292_57185, &P22072_291};	/*Property = CssMinMax*/
const Value V22072_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_293 = {S15, 0, &V22072_293_57188, &P22072_292};	/*Property = CssOverflow*/
const Value V22072_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_294 = {S16, 0, &V22072_294_57191, &P22072_293};	/*Property = CssPosition*/
const Value V22072_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_295 = {S17, 0, &V22072_295_57194, &P22072_294};	/*Property = CssText*/
const Value V22072_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_297 = {S18, 0, &V22072_297_57197, &P22072_295};	/*Property = CssTransforms*/
const Value V22072_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_241 = {S19, 0, &V22072_241_55801, &P22072_297};	/*Property = CssTransitions*/
const Value V22072_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_298 = {S20, 0, &V22072_298_57208, &P22072_241};	/*Property = CssUI*/
const Value V22072_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_300 = {S21, 0, &V22072_300_57217, &P22072_298};	/*Property = DataSet*/
const Value V22072_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_301 = {S22, 0, &V22072_301_57220, &P22072_300};	/*Property = DataUrl*/
const Value V22072_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_302 = {S23, 0, &V22072_302_57223, &P22072_301};	/*Property = DeviceOrientation*/
const Value V22072_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_304 = {S24, 0, &V22072_304_57229, &P22072_302};	/*Property = FileReader*/
const Value V22072_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_305 = {S25, 0, &V22072_305_57232, &P22072_304};	/*Property = FileSaver*/
const Value V22072_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_306 = {S26, 0, &V22072_306_57238, &P22072_305};	/*Property = FileWriter*/
const Value V22072_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_307 = {S27, 0, &V22072_307_57241, &P22072_306};	/*Property = FormData*/
const Value V22072_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_309 = {S28, 0, &V22072_309_57247, &P22072_307};	/*Property = Fullscreen*/
const Value V22072_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_310 = {S29, 0, &V22072_310_57250, &P22072_309};	/*Property = GeoLocation*/
const Value V22072_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_312 = {S30, 0, &V22072_312_57256, &P22072_310};	/*Property = History*/
const Value V22072_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_314 = {S31, 0, &V22072_314_57262, &P22072_312};	/*Property = Html5*/
const Value V22072_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_313 = {S32, 0, &V22072_313_57259, &P22072_314};	/*Property = Html-Media-Capture*/
const Value V22072_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_315 = {S33, 0, &V22072_315_57265, &P22072_313};	/*Property = Iframe*/
const Value V22072_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_316 = {S34, 0, &V22072_316_57268, &P22072_315};	/*Property = IndexedDB*/
const Value V22072_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_317 = {S35, 0, &V22072_317_57271, &P22072_316};	/*Property = Json*/
const Value V22072_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_15 = {S37, 0, &V22072_15_72, &P22072_317};	/*Property = LayoutEngine*/
const Value V22072_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_344 = {S38, 0, &V22072_344_57274, &P22072_15};	/*Property = Masking*/
const Value V22072_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_320 = {S39, 0, &V22072_320_57292, &P22072_344};	/*Property = PostMessage*/
const Value V22072_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_321 = {S40, 0, &V22072_321_57295, &P22072_320};	/*Property = Progress*/
const Value V22072_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_322 = {S41, 0, &V22072_322_57298, &P22072_321};	/*Property = Prompts*/
const Value V22072_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_323 = {S42, 0, &V22072_323_57301, &P22072_322};	/*Property = Selector*/
const Value V22072_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_324 = {S43, 0, &V22072_324_57307, &P22072_323};	/*Property = Svg*/
const Value V22072_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_327 = {S44, 0, &V22072_327_57316, &P22072_324};	/*Property = TouchEvents*/
const Value V22072_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_328 = {S45, 0, &V22072_328_57319, &P22072_327};	/*Property = Track*/
const Value V22072_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_329 = {S46, 0, &V22072_329_57325, &P22072_328};	/*Property = Video*/
const Value V22072_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_330 = {S47, 0, &V22072_330_57328, &P22072_329};	/*Property = Viewport*/
const Value V22072_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_332 = {S48, 0, &V22072_332_57340, &P22072_330};	/*Property = WebWorkers*/
const Value V22072_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22072_333 = {S49, 0, &V22072_333_57343, &P22072_332};	/*Property = Xhr2*/
const Profile P22072 = {"22072", &P22072_333};	/*Profile ID - 22072*/
const Value V20429_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_279 = {S1, 0, &V20429_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20429_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_282 = {S3, 0, &V20429_282_57155, &P20429_279};	/*Property = BlobBuilder*/
const Value V20429_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_350 = {S4, 0, &V20429_350_57446, &P20429_282};	/*Property = Canvas*/
const Value V20429_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_283 = {S5, 0, &V20429_283_57161, &P20429_350};	/*Property = CssBackground*/
const Value V20429_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_284 = {S6, 0, &V20429_284_57164, &P20429_283};	/*Property = CssBorderImage*/
const Value V20429_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_285 = {S7, 0, &V20429_285_57167, &P20429_284};	/*Property = CssCanvas*/
const Value V20429_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_286 = {S8, 0, &V20429_286_57170, &P20429_285};	/*Property = CssColor*/
const Value V20429_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_237 = {S9, 0, &V20429_237_55786, &P20429_286};	/*Property = CssColumn*/
const Value V20429_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_287 = {S10, 0, &V20429_287_57173, &P20429_237};	/*Property = CssFlexbox*/
const Value V20429_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_288 = {S11, 0, &V20429_288_57176, &P20429_287};	/*Property = CssFont*/
const Value V20429_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_289 = {S12, 0, &V20429_289_86899, &P20429_288};	/*Property = CssImages*/
const Value V20429_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_291 = {S13, 0, &V20429_291_57182, &P20429_289};	/*Property = CssMediaQueries*/
const Value V20429_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_292 = {S14, 0, &V20429_292_57185, &P20429_291};	/*Property = CssMinMax*/
const Value V20429_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_293 = {S15, 0, &V20429_293_57188, &P20429_292};	/*Property = CssOverflow*/
const Value V20429_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_294 = {S16, 0, &V20429_294_57191, &P20429_293};	/*Property = CssPosition*/
const Value V20429_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_295 = {S17, 0, &V20429_295_57194, &P20429_294};	/*Property = CssText*/
const Value V20429_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_297 = {S18, 0, &V20429_297_57197, &P20429_295};	/*Property = CssTransforms*/
const Value V20429_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_241 = {S19, 0, &V20429_241_55801, &P20429_297};	/*Property = CssTransitions*/
const Value V20429_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_298 = {S20, 0, &V20429_298_57208, &P20429_241};	/*Property = CssUI*/
const Value V20429_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_300 = {S21, 0, &V20429_300_57217, &P20429_298};	/*Property = DataSet*/
const Value V20429_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_301 = {S22, 0, &V20429_301_57220, &P20429_300};	/*Property = DataUrl*/
const Value V20429_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_302 = {S23, 0, &V20429_302_57223, &P20429_301};	/*Property = DeviceOrientation*/
const Value V20429_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_304 = {S24, 0, &V20429_304_57229, &P20429_302};	/*Property = FileReader*/
const Value V20429_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_305 = {S25, 0, &V20429_305_57232, &P20429_304};	/*Property = FileSaver*/
const Value V20429_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_306 = {S26, 0, &V20429_306_57238, &P20429_305};	/*Property = FileWriter*/
const Value V20429_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_307 = {S27, 0, &V20429_307_57241, &P20429_306};	/*Property = FormData*/
const Value V20429_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_309 = {S28, 0, &V20429_309_57247, &P20429_307};	/*Property = Fullscreen*/
const Value V20429_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_310 = {S29, 0, &V20429_310_57250, &P20429_309};	/*Property = GeoLocation*/
const Value V20429_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_312 = {S30, 0, &V20429_312_57256, &P20429_310};	/*Property = History*/
const Value V20429_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_314 = {S31, 0, &V20429_314_57262, &P20429_312};	/*Property = Html5*/
const Value V20429_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_313 = {S32, 0, &V20429_313_57259, &P20429_314};	/*Property = Html-Media-Capture*/
const Value V20429_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_315 = {S33, 0, &V20429_315_57265, &P20429_313};	/*Property = Iframe*/
const Value V20429_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_316 = {S34, 0, &V20429_316_57268, &P20429_315};	/*Property = IndexedDB*/
const Value V20429_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_317 = {S35, 0, &V20429_317_57271, &P20429_316};	/*Property = Json*/
const Value V20429_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20429_15 = {S37, 0, &V20429_15_29, &P20429_317};	/*Property = LayoutEngine*/
const Value V20429_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_344 = {S38, 0, &V20429_344_57274, &P20429_15};	/*Property = Masking*/
const Value V20429_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_320 = {S39, 0, &V20429_320_57292, &P20429_344};	/*Property = PostMessage*/
const Value V20429_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_321 = {S40, 0, &V20429_321_57295, &P20429_320};	/*Property = Progress*/
const Value V20429_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_322 = {S41, 0, &V20429_322_57298, &P20429_321};	/*Property = Prompts*/
const Value V20429_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_323 = {S42, 0, &V20429_323_57301, &P20429_322};	/*Property = Selector*/
const Value V20429_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_324 = {S43, 0, &V20429_324_57307, &P20429_323};	/*Property = Svg*/
const Value V20429_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_327 = {S44, 0, &V20429_327_57316, &P20429_324};	/*Property = TouchEvents*/
const Value V20429_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_328 = {S45, 0, &V20429_328_57319, &P20429_327};	/*Property = Track*/
const Value V20429_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_329 = {S46, 0, &V20429_329_57325, &P20429_328};	/*Property = Video*/
const Value V20429_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_330 = {S47, 0, &V20429_330_57328, &P20429_329};	/*Property = Viewport*/
const Value V20429_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_332 = {S48, 0, &V20429_332_57340, &P20429_330};	/*Property = WebWorkers*/
const Value V20429_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20429_333 = {S49, 0, &V20429_333_57343, &P20429_332};	/*Property = Xhr2*/
const Profile P20429 = {"20429", &P20429_333};	/*Profile ID - 20429*/
const Value V20791_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20791_279 = {S1, 0, &V20791_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20791_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20791_282 = {S3, 0, &V20791_282_57154, &P20791_279};	/*Property = BlobBuilder*/
const Value V20791_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20791_350 = {S4, 0, &V20791_350_57444, &P20791_282};	/*Property = Canvas*/
const Value V20791_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20791_283 = {S5, 0, &V20791_283_57159, &P20791_350};	/*Property = CssBackground*/
const Value V20791_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20791_284 = {S6, 0, &V20791_284_57163, &P20791_283};	/*Property = CssBorderImage*/
const Value V20791_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20791_285 = {S7, 0, &V20791_285_57166, &P20791_284};	/*Property = CssCanvas*/
const Value V20791_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20791_286 = {S8, 0, &V20791_286_57168, &P20791_285};	/*Property = CssColor*/
const Value V20791_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20791_237 = {S9, 0, &V20791_237_55786, &P20791_286};	/*Property = CssColumn*/
const Value V20791_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20791_287 = {S10, 0, &V20791_287_57172, &P20791_237};	/*Property = CssFlexbox*/
const Value V20791_288_57174 = {S2, NULL};	/*Value = True*/
const Property P20791_288 = {S11, 0, &V20791_288_57174, &P20791_287};	/*Property = CssFont*/
const Value V20791_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20791_289 = {S12, 0, &V20791_289_57759, &P20791_288};	/*Property = CssImages*/
const Value V20791_291_57180 = {S2, NULL};	/*Value = True*/
const Property P20791_291 = {S13, 0, &V20791_291_57180, &P20791_289};	/*Property = CssMediaQueries*/
const Value V20791_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20791_292 = {S14, 0, &V20791_292_57183, &P20791_291};	/*Property = CssMinMax*/
const Value V20791_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20791_293 = {S15, 0, &V20791_293_57187, &P20791_292};	/*Property = CssOverflow*/
const Value V20791_294_57189 = {S2, NULL};	/*Value = True*/
const Property P20791_294 = {S16, 0, &V20791_294_57189, &P20791_293};	/*Property = CssPosition*/
const Value V20791_295_57192 = {S2, NULL};	/*Value = True*/
const Property P20791_295 = {S17, 0, &V20791_295_57192, &P20791_294};	/*Property = CssText*/
const Value V20791_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20791_297 = {S18, 0, &V20791_297_57195, &P20791_295};	/*Property = CssTransforms*/
const Value V20791_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20791_241 = {S19, 0, &V20791_241_55799, &P20791_297};	/*Property = CssTransitions*/
const Value V20791_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20791_298 = {S20, 0, &V20791_298_57206, &P20791_241};	/*Property = CssUI*/
const Value V20791_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20791_300 = {S21, 0, &V20791_300_57216, &P20791_298};	/*Property = DataSet*/
const Value V20791_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20791_301 = {S22, 0, &V20791_301_57218, &P20791_300};	/*Property = DataUrl*/
const Value V20791_302_57221 = {S2, NULL};	/*Value = True*/
const Property P20791_302 = {S23, 0, &V20791_302_57221, &P20791_301};	/*Property = DeviceOrientation*/
const Value V20791_304_57227 = {S2, NULL};	/*Value = True*/
const Property P20791_304 = {S24, 0, &V20791_304_57227, &P20791_302};	/*Property = FileReader*/
const Value V20791_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20791_305 = {S25, 0, &V20791_305_57231, &P20791_304};	/*Property = FileSaver*/
const Value V20791_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20791_306 = {S26, 0, &V20791_306_57237, &P20791_305};	/*Property = FileWriter*/
const Value V20791_307_57239 = {S2, NULL};	/*Value = True*/
const Property P20791_307 = {S27, 0, &V20791_307_57239, &P20791_306};	/*Property = FormData*/
const Value V20791_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20791_309 = {S28, 0, &V20791_309_57246, &P20791_307};	/*Property = Fullscreen*/
const Value V20791_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20791_310 = {S29, 0, &V20791_310_57248, &P20791_309};	/*Property = GeoLocation*/
const Value V20791_312_57254 = {S2, NULL};	/*Value = True*/
const Property P20791_312 = {S30, 0, &V20791_312_57254, &P20791_310};	/*Property = History*/
const Value V20791_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20791_314 = {S31, 0, &V20791_314_57261, &P20791_312};	/*Property = Html5*/
const Value V20791_313_57257 = {S2, NULL};	/*Value = True*/
const Property P20791_313 = {S32, 0, &V20791_313_57257, &P20791_314};	/*Property = Html-Media-Capture*/
const Value V20791_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20791_315 = {S33, 0, &V20791_315_57264, &P20791_313};	/*Property = Iframe*/
const Value V20791_316_57266 = {S2, NULL};	/*Value = True*/
const Property P20791_316 = {S34, 0, &V20791_316_57266, &P20791_315};	/*Property = IndexedDB*/
const Value V20791_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20791_317 = {S35, 0, &V20791_317_57269, &P20791_316};	/*Property = Json*/
const Value V20791_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P20791_15 = {S37, 0, &V20791_15_54167, &P20791_317};	/*Property = LayoutEngine*/
const Value V20791_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20791_344 = {S38, 0, &V20791_344_57272, &P20791_15};	/*Property = Masking*/
const Value V20791_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20791_320 = {S39, 0, &V20791_320_57290, &P20791_344};	/*Property = PostMessage*/
const Value V20791_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20791_321 = {S40, 0, &V20791_321_57293, &P20791_320};	/*Property = Progress*/
const Value V20791_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20791_322 = {S41, 0, &V20791_322_57296, &P20791_321};	/*Property = Prompts*/
const Value V20791_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20791_323 = {S42, 0, &V20791_323_57299, &P20791_322};	/*Property = Selector*/
const Value V20791_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20791_324 = {S43, 0, &V20791_324_57306, &P20791_323};	/*Property = Svg*/
const Value V20791_327_57314 = {S2, NULL};	/*Value = True*/
const Property P20791_327 = {S44, 0, &V20791_327_57314, &P20791_324};	/*Property = TouchEvents*/
const Value V20791_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20791_328 = {S45, 0, &V20791_328_57318, &P20791_327};	/*Property = Track*/
const Value V20791_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20791_329 = {S46, 0, &V20791_329_57323, &P20791_328};	/*Property = Video*/
const Value V20791_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20791_330 = {S47, 0, &V20791_330_57326, &P20791_329};	/*Property = Viewport*/
const Value V20791_332_57339 = {S0, NULL};	/*Value = False*/
const Property P20791_332 = {S48, 0, &V20791_332_57339, &P20791_330};	/*Property = WebWorkers*/
const Value V20791_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20791_333 = {S49, 0, &V20791_333_57342, &P20791_332};	/*Property = Xhr2*/
const Profile P20791 = {"20791", &P20791_333};	/*Profile ID - 20791*/
const Value V18592_1_1 = {S2, NULL};	/*Value = True*/
const Property P18592_1 = {S50, 0, &V18592_1_1, NULL};	/*Property = IsMobile*/
const Value V18592_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18592_5 = {S52, 0, &V18592_5_1403, &P18592_1};	/*Property = ScreenPixelsHeight*/
const Value V18592_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18592_4 = {S54, 0, &V18592_4_1404, &P18592_5};	/*Property = ScreenPixelsWidth*/
const Profile P18592 = {"18592", &P18592_4};	/*Profile ID - 18592*/
const Value V20925_1_1 = {S2, NULL};	/*Value = True*/
const Property P20925_1 = {S50, 0, &V20925_1_1, NULL};	/*Property = IsMobile*/
const Value V20925_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20925_5 = {S52, 0, &V20925_5_1403, &P20925_1};	/*Property = ScreenPixelsHeight*/
const Value V20925_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20925_4 = {S54, 0, &V20925_4_1404, &P20925_5};	/*Property = ScreenPixelsWidth*/
const Profile P20925 = {"20925", &P20925_4};	/*Profile ID - 20925*/
const Value V23920_1_1 = {S2, NULL};	/*Value = True*/
const Property P23920_1 = {S50, 0, &V23920_1_1, NULL};	/*Property = IsMobile*/
const Value V23920_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23920_5 = {S52, 0, &V23920_5_37, &P23920_1};	/*Property = ScreenPixelsHeight*/
const Value V23920_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23920_4 = {S54, 0, &V23920_4_36, &P23920_5};	/*Property = ScreenPixelsWidth*/
const Profile P23920 = {"23920", &P23920_4};	/*Profile ID - 23920*/
const Value V21809_1_1 = {S2, NULL};	/*Value = True*/
const Property P21809_1 = {S50, 0, &V21809_1_1, NULL};	/*Property = IsMobile*/
const Value V21809_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21809_5 = {S52, 0, &V21809_5_1277, &P21809_1};	/*Property = ScreenPixelsHeight*/
const Value V21809_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21809_4 = {S54, 0, &V21809_4_1279, &P21809_5};	/*Property = ScreenPixelsWidth*/
const Profile P21809 = {"21809", &P21809_4};	/*Profile ID - 21809*/
const Value V20225_1_1 = {S2, NULL};	/*Value = True*/
const Property P20225_1 = {S50, 0, &V20225_1_1, NULL};	/*Property = IsMobile*/
const Value V20225_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P20225_5 = {S52, 0, &V20225_5_1293, &P20225_1};	/*Property = ScreenPixelsHeight*/
const Value V20225_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P20225_4 = {S54, 0, &V20225_4_1296, &P20225_5};	/*Property = ScreenPixelsWidth*/
const Profile P20225 = {"20225", &P20225_4};	/*Profile ID - 20225*/
const Value V24675_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_279 = {S1, 0, &V24675_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24675_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_282 = {S3, 0, &V24675_282_57155, &P24675_279};	/*Property = BlobBuilder*/
const Value V24675_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_350 = {S4, 0, &V24675_350_57446, &P24675_282};	/*Property = Canvas*/
const Value V24675_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_283 = {S5, 0, &V24675_283_57161, &P24675_350};	/*Property = CssBackground*/
const Value V24675_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_284 = {S6, 0, &V24675_284_57164, &P24675_283};	/*Property = CssBorderImage*/
const Value V24675_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_285 = {S7, 0, &V24675_285_57167, &P24675_284};	/*Property = CssCanvas*/
const Value V24675_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_286 = {S8, 0, &V24675_286_57170, &P24675_285};	/*Property = CssColor*/
const Value V24675_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_237 = {S9, 0, &V24675_237_55786, &P24675_286};	/*Property = CssColumn*/
const Value V24675_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_287 = {S10, 0, &V24675_287_57173, &P24675_237};	/*Property = CssFlexbox*/
const Value V24675_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_288 = {S11, 0, &V24675_288_57176, &P24675_287};	/*Property = CssFont*/
const Value V24675_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_289 = {S12, 0, &V24675_289_86899, &P24675_288};	/*Property = CssImages*/
const Value V24675_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_291 = {S13, 0, &V24675_291_57182, &P24675_289};	/*Property = CssMediaQueries*/
const Value V24675_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_292 = {S14, 0, &V24675_292_57185, &P24675_291};	/*Property = CssMinMax*/
const Value V24675_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_293 = {S15, 0, &V24675_293_57188, &P24675_292};	/*Property = CssOverflow*/
const Value V24675_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_294 = {S16, 0, &V24675_294_57191, &P24675_293};	/*Property = CssPosition*/
const Value V24675_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_295 = {S17, 0, &V24675_295_57194, &P24675_294};	/*Property = CssText*/
const Value V24675_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_297 = {S18, 0, &V24675_297_57197, &P24675_295};	/*Property = CssTransforms*/
const Value V24675_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_241 = {S19, 0, &V24675_241_55801, &P24675_297};	/*Property = CssTransitions*/
const Value V24675_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_298 = {S20, 0, &V24675_298_57208, &P24675_241};	/*Property = CssUI*/
const Value V24675_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_300 = {S21, 0, &V24675_300_57217, &P24675_298};	/*Property = DataSet*/
const Value V24675_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_301 = {S22, 0, &V24675_301_57220, &P24675_300};	/*Property = DataUrl*/
const Value V24675_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_302 = {S23, 0, &V24675_302_57223, &P24675_301};	/*Property = DeviceOrientation*/
const Value V24675_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_304 = {S24, 0, &V24675_304_57229, &P24675_302};	/*Property = FileReader*/
const Value V24675_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_305 = {S25, 0, &V24675_305_57232, &P24675_304};	/*Property = FileSaver*/
const Value V24675_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_306 = {S26, 0, &V24675_306_57238, &P24675_305};	/*Property = FileWriter*/
const Value V24675_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_307 = {S27, 0, &V24675_307_57241, &P24675_306};	/*Property = FormData*/
const Value V24675_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_309 = {S28, 0, &V24675_309_57247, &P24675_307};	/*Property = Fullscreen*/
const Value V24675_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_310 = {S29, 0, &V24675_310_57250, &P24675_309};	/*Property = GeoLocation*/
const Value V24675_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_312 = {S30, 0, &V24675_312_57256, &P24675_310};	/*Property = History*/
const Value V24675_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_314 = {S31, 0, &V24675_314_57262, &P24675_312};	/*Property = Html5*/
const Value V24675_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_313 = {S32, 0, &V24675_313_57259, &P24675_314};	/*Property = Html-Media-Capture*/
const Value V24675_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_315 = {S33, 0, &V24675_315_57265, &P24675_313};	/*Property = Iframe*/
const Value V24675_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_316 = {S34, 0, &V24675_316_57268, &P24675_315};	/*Property = IndexedDB*/
const Value V24675_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_317 = {S35, 0, &V24675_317_57271, &P24675_316};	/*Property = Json*/
const Value V24675_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24675_15 = {S37, 0, &V24675_15_90837, &P24675_317};	/*Property = LayoutEngine*/
const Value V24675_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_344 = {S38, 0, &V24675_344_57274, &P24675_15};	/*Property = Masking*/
const Value V24675_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_320 = {S39, 0, &V24675_320_57292, &P24675_344};	/*Property = PostMessage*/
const Value V24675_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_321 = {S40, 0, &V24675_321_57295, &P24675_320};	/*Property = Progress*/
const Value V24675_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_322 = {S41, 0, &V24675_322_57298, &P24675_321};	/*Property = Prompts*/
const Value V24675_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_323 = {S42, 0, &V24675_323_57301, &P24675_322};	/*Property = Selector*/
const Value V24675_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_324 = {S43, 0, &V24675_324_57307, &P24675_323};	/*Property = Svg*/
const Value V24675_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_327 = {S44, 0, &V24675_327_57316, &P24675_324};	/*Property = TouchEvents*/
const Value V24675_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_328 = {S45, 0, &V24675_328_57319, &P24675_327};	/*Property = Track*/
const Value V24675_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_329 = {S46, 0, &V24675_329_57325, &P24675_328};	/*Property = Video*/
const Value V24675_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_330 = {S47, 0, &V24675_330_57328, &P24675_329};	/*Property = Viewport*/
const Value V24675_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_332 = {S48, 0, &V24675_332_57340, &P24675_330};	/*Property = WebWorkers*/
const Value V24675_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24675_333 = {S49, 0, &V24675_333_57343, &P24675_332};	/*Property = Xhr2*/
const Profile P24675 = {"24675", &P24675_333};	/*Profile ID - 24675*/
const Value V22108_1_1 = {S2, NULL};	/*Value = True*/
const Property P22108_1 = {S50, 0, &V22108_1_1, NULL};	/*Property = IsMobile*/
const Value V22108_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22108_5 = {S52, 0, &V22108_5_1227, &P22108_1};	/*Property = ScreenPixelsHeight*/
const Value V22108_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22108_4 = {S54, 0, &V22108_4_1229, &P22108_5};	/*Property = ScreenPixelsWidth*/
const Profile P22108 = {"22108", &P22108_4};	/*Profile ID - 22108*/
const Value V21578_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_279 = {S1, 0, &V21578_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21578_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_282 = {S3, 0, &V21578_282_57155, &P21578_279};	/*Property = BlobBuilder*/
const Value V21578_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_350 = {S4, 0, &V21578_350_57446, &P21578_282};	/*Property = Canvas*/
const Value V21578_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_283 = {S5, 0, &V21578_283_57161, &P21578_350};	/*Property = CssBackground*/
const Value V21578_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_284 = {S6, 0, &V21578_284_57164, &P21578_283};	/*Property = CssBorderImage*/
const Value V21578_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_285 = {S7, 0, &V21578_285_57167, &P21578_284};	/*Property = CssCanvas*/
const Value V21578_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_286 = {S8, 0, &V21578_286_57170, &P21578_285};	/*Property = CssColor*/
const Value V21578_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_237 = {S9, 0, &V21578_237_55786, &P21578_286};	/*Property = CssColumn*/
const Value V21578_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_287 = {S10, 0, &V21578_287_57173, &P21578_237};	/*Property = CssFlexbox*/
const Value V21578_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_288 = {S11, 0, &V21578_288_57176, &P21578_287};	/*Property = CssFont*/
const Value V21578_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_289 = {S12, 0, &V21578_289_86899, &P21578_288};	/*Property = CssImages*/
const Value V21578_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_291 = {S13, 0, &V21578_291_57182, &P21578_289};	/*Property = CssMediaQueries*/
const Value V21578_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_292 = {S14, 0, &V21578_292_57185, &P21578_291};	/*Property = CssMinMax*/
const Value V21578_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_293 = {S15, 0, &V21578_293_57188, &P21578_292};	/*Property = CssOverflow*/
const Value V21578_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_294 = {S16, 0, &V21578_294_57191, &P21578_293};	/*Property = CssPosition*/
const Value V21578_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_295 = {S17, 0, &V21578_295_57194, &P21578_294};	/*Property = CssText*/
const Value V21578_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_297 = {S18, 0, &V21578_297_57197, &P21578_295};	/*Property = CssTransforms*/
const Value V21578_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_241 = {S19, 0, &V21578_241_55801, &P21578_297};	/*Property = CssTransitions*/
const Value V21578_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_298 = {S20, 0, &V21578_298_57208, &P21578_241};	/*Property = CssUI*/
const Value V21578_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_300 = {S21, 0, &V21578_300_57217, &P21578_298};	/*Property = DataSet*/
const Value V21578_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_301 = {S22, 0, &V21578_301_57220, &P21578_300};	/*Property = DataUrl*/
const Value V21578_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_302 = {S23, 0, &V21578_302_57223, &P21578_301};	/*Property = DeviceOrientation*/
const Value V21578_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_304 = {S24, 0, &V21578_304_57229, &P21578_302};	/*Property = FileReader*/
const Value V21578_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_305 = {S25, 0, &V21578_305_57232, &P21578_304};	/*Property = FileSaver*/
const Value V21578_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_306 = {S26, 0, &V21578_306_57238, &P21578_305};	/*Property = FileWriter*/
const Value V21578_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_307 = {S27, 0, &V21578_307_57241, &P21578_306};	/*Property = FormData*/
const Value V21578_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_309 = {S28, 0, &V21578_309_57247, &P21578_307};	/*Property = Fullscreen*/
const Value V21578_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_310 = {S29, 0, &V21578_310_57250, &P21578_309};	/*Property = GeoLocation*/
const Value V21578_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_312 = {S30, 0, &V21578_312_57256, &P21578_310};	/*Property = History*/
const Value V21578_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_314 = {S31, 0, &V21578_314_57262, &P21578_312};	/*Property = Html5*/
const Value V21578_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_313 = {S32, 0, &V21578_313_57259, &P21578_314};	/*Property = Html-Media-Capture*/
const Value V21578_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_315 = {S33, 0, &V21578_315_57265, &P21578_313};	/*Property = Iframe*/
const Value V21578_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_316 = {S34, 0, &V21578_316_57268, &P21578_315};	/*Property = IndexedDB*/
const Value V21578_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_317 = {S35, 0, &V21578_317_57271, &P21578_316};	/*Property = Json*/
const Value V21578_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21578_15 = {S37, 0, &V21578_15_28, &P21578_317};	/*Property = LayoutEngine*/
const Value V21578_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_344 = {S38, 0, &V21578_344_57274, &P21578_15};	/*Property = Masking*/
const Value V21578_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_320 = {S39, 0, &V21578_320_57292, &P21578_344};	/*Property = PostMessage*/
const Value V21578_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_321 = {S40, 0, &V21578_321_57295, &P21578_320};	/*Property = Progress*/
const Value V21578_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_322 = {S41, 0, &V21578_322_57298, &P21578_321};	/*Property = Prompts*/
const Value V21578_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_323 = {S42, 0, &V21578_323_57301, &P21578_322};	/*Property = Selector*/
const Value V21578_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_324 = {S43, 0, &V21578_324_57307, &P21578_323};	/*Property = Svg*/
const Value V21578_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_327 = {S44, 0, &V21578_327_57316, &P21578_324};	/*Property = TouchEvents*/
const Value V21578_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_328 = {S45, 0, &V21578_328_57319, &P21578_327};	/*Property = Track*/
const Value V21578_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_329 = {S46, 0, &V21578_329_57325, &P21578_328};	/*Property = Video*/
const Value V21578_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_330 = {S47, 0, &V21578_330_57328, &P21578_329};	/*Property = Viewport*/
const Value V21578_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_332 = {S48, 0, &V21578_332_57340, &P21578_330};	/*Property = WebWorkers*/
const Value V21578_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21578_333 = {S49, 0, &V21578_333_57343, &P21578_332};	/*Property = Xhr2*/
const Profile P21578 = {"21578", &P21578_333};	/*Profile ID - 21578*/
const Value V20038_1_1 = {S2, NULL};	/*Value = True*/
const Property P20038_1 = {S50, 0, &V20038_1_1, NULL};	/*Property = IsMobile*/
const Value V20038_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P20038_5 = {S52, 0, &V20038_5_1233, &P20038_1};	/*Property = ScreenPixelsHeight*/
const Value V20038_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P20038_4 = {S54, 0, &V20038_4_1235, &P20038_5};	/*Property = ScreenPixelsWidth*/
const Profile P20038 = {"20038", &P20038_4};	/*Profile ID - 20038*/
const Value V21100_1_1 = {S2, NULL};	/*Value = True*/
const Property P21100_1 = {S50, 0, &V21100_1_1, NULL};	/*Property = IsMobile*/
const Value V21100_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21100_5 = {S52, 0, &V21100_5_1277, &P21100_1};	/*Property = ScreenPixelsHeight*/
const Value V21100_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21100_4 = {S54, 0, &V21100_4_1279, &P21100_5};	/*Property = ScreenPixelsWidth*/
const Profile P21100 = {"21100", &P21100_4};	/*Profile ID - 21100*/
const Value V21024_1_1 = {S2, NULL};	/*Value = True*/
const Property P21024_1 = {S50, 0, &V21024_1_1, NULL};	/*Property = IsMobile*/
const Value V21024_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21024_5 = {S52, 0, &V21024_5_1277, &P21024_1};	/*Property = ScreenPixelsHeight*/
const Value V21024_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21024_4 = {S54, 0, &V21024_4_1279, &P21024_5};	/*Property = ScreenPixelsWidth*/
const Profile P21024 = {"21024", &P21024_4};	/*Profile ID - 21024*/
const Value V20976_1_1 = {S2, NULL};	/*Value = True*/
const Property P20976_1 = {S50, 0, &V20976_1_1, NULL};	/*Property = IsMobile*/
const Value V20976_5_77419 = {S138, NULL};	/*Value = 728*/
const Property P20976_5 = {S52, 0, &V20976_5_77419, &P20976_1};	/*Property = ScreenPixelsHeight*/
const Value V20976_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20976_4 = {S54, 0, &V20976_4_1229, &P20976_5};	/*Property = ScreenPixelsWidth*/
const Profile P20976 = {"20976", &P20976_4};	/*Profile ID - 20976*/
const Value V23082_1_1 = {S2, NULL};	/*Value = True*/
const Property P23082_1 = {S50, 0, &V23082_1_1, NULL};	/*Property = IsMobile*/
const Value V23082_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23082_5 = {S52, 0, &V23082_5_1277, &P23082_1};	/*Property = ScreenPixelsHeight*/
const Value V23082_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23082_4 = {S54, 0, &V23082_4_1279, &P23082_5};	/*Property = ScreenPixelsWidth*/
const Profile P23082 = {"23082", &P23082_4};	/*Profile ID - 23082*/
const Value V23939_1_1 = {S2, NULL};	/*Value = True*/
const Property P23939_1 = {S50, 0, &V23939_1_1, NULL};	/*Property = IsMobile*/
const Value V23939_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23939_5 = {S52, 0, &V23939_5_1277, &P23939_1};	/*Property = ScreenPixelsHeight*/
const Value V23939_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23939_4 = {S54, 0, &V23939_4_1279, &P23939_5};	/*Property = ScreenPixelsWidth*/
const Profile P23939 = {"23939", &P23939_4};	/*Profile ID - 23939*/
const Value V22182_1_1 = {S2, NULL};	/*Value = True*/
const Property P22182_1 = {S50, 0, &V22182_1_1, NULL};	/*Property = IsMobile*/
const Value V22182_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22182_5 = {S52, 0, &V22182_5_1227, &P22182_1};	/*Property = ScreenPixelsHeight*/
const Value V22182_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P22182_4 = {S54, 0, &V22182_4_1943, &P22182_5};	/*Property = ScreenPixelsWidth*/
const Profile P22182 = {"22182", &P22182_4};	/*Profile ID - 22182*/
const Value V21063_1_1 = {S2, NULL};	/*Value = True*/
const Property P21063_1 = {S50, 0, &V21063_1_1, NULL};	/*Property = IsMobile*/
const Value V21063_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21063_5 = {S52, 0, &V21063_5_1227, &P21063_1};	/*Property = ScreenPixelsHeight*/
const Value V21063_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21063_4 = {S54, 0, &V21063_4_1229, &P21063_5};	/*Property = ScreenPixelsWidth*/
const Profile P21063 = {"21063", &P21063_4};	/*Profile ID - 21063*/
const Value V21543_1_1 = {S2, NULL};	/*Value = True*/
const Property P21543_1 = {S50, 0, &V21543_1_1, NULL};	/*Property = IsMobile*/
const Value V21543_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21543_5 = {S52, 0, &V21543_5_1227, &P21543_1};	/*Property = ScreenPixelsHeight*/
const Value V21543_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21543_4 = {S54, 0, &V21543_4_1229, &P21543_5};	/*Property = ScreenPixelsWidth*/
const Profile P21543 = {"21543", &P21543_4};	/*Profile ID - 21543*/
const Value V23059_1_1 = {S2, NULL};	/*Value = True*/
const Property P23059_1 = {S50, 0, &V23059_1_1, NULL};	/*Property = IsMobile*/
const Value V23059_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23059_5 = {S52, 0, &V23059_5_1277, &P23059_1};	/*Property = ScreenPixelsHeight*/
const Value V23059_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23059_4 = {S54, 0, &V23059_4_1279, &P23059_5};	/*Property = ScreenPixelsWidth*/
const Profile P23059 = {"23059", &P23059_4};	/*Profile ID - 23059*/
const Value V21035_1_1 = {S2, NULL};	/*Value = True*/
const Property P21035_1 = {S50, 0, &V21035_1_1, NULL};	/*Property = IsMobile*/
const Value V21035_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21035_5 = {S52, 0, &V21035_5_1277, &P21035_1};	/*Property = ScreenPixelsHeight*/
const Value V21035_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21035_4 = {S54, 0, &V21035_4_1279, &P21035_5};	/*Property = ScreenPixelsWidth*/
const Profile P21035 = {"21035", &P21035_4};	/*Profile ID - 21035*/
const Value V23075_1_1 = {S2, NULL};	/*Value = True*/
const Property P23075_1 = {S50, 0, &V23075_1_1, NULL};	/*Property = IsMobile*/
const Value V23075_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23075_5 = {S52, 0, &V23075_5_1277, &P23075_1};	/*Property = ScreenPixelsHeight*/
const Value V23075_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23075_4 = {S54, 0, &V23075_4_1279, &P23075_5};	/*Property = ScreenPixelsWidth*/
const Profile P23075 = {"23075", &P23075_4};	/*Profile ID - 23075*/
const Value V20887_1_1 = {S2, NULL};	/*Value = True*/
const Property P20887_1 = {S50, 0, &V20887_1_1, NULL};	/*Property = IsMobile*/
const Value V20887_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20887_5 = {S52, 0, &V20887_5_1227, &P20887_1};	/*Property = ScreenPixelsHeight*/
const Value V20887_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20887_4 = {S54, 0, &V20887_4_1229, &P20887_5};	/*Property = ScreenPixelsWidth*/
const Profile P20887 = {"20887", &P20887_4};	/*Profile ID - 20887*/
const Value V23303_1_1 = {S2, NULL};	/*Value = True*/
const Property P23303_1 = {S50, 0, &V23303_1_1, NULL};	/*Property = IsMobile*/
const Value V23303_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23303_5 = {S52, 0, &V23303_5_1227, &P23303_1};	/*Property = ScreenPixelsHeight*/
const Value V23303_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23303_4 = {S54, 0, &V23303_4_1229, &P23303_5};	/*Property = ScreenPixelsWidth*/
const Profile P23303 = {"23303", &P23303_4};	/*Profile ID - 23303*/
const Value V18491_1_1 = {S2, NULL};	/*Value = True*/
const Property P18491_1 = {S50, 0, &V18491_1_1, NULL};	/*Property = IsMobile*/
const Value V18491_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18491_5 = {S52, 0, &V18491_5_1227, &P18491_1};	/*Property = ScreenPixelsHeight*/
const Value V18491_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18491_4 = {S54, 0, &V18491_4_1229, &P18491_5};	/*Property = ScreenPixelsWidth*/
const Profile P18491 = {"18491", &P18491_4};	/*Profile ID - 18491*/
const Value V21593_1_1 = {S2, NULL};	/*Value = True*/
const Property P21593_1 = {S50, 0, &V21593_1_1, NULL};	/*Property = IsMobile*/
const Value V21593_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P21593_5 = {S52, 0, &V21593_5_1832, &P21593_1};	/*Property = ScreenPixelsHeight*/
const Value V21593_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21593_4 = {S54, 0, &V21593_4_5645, &P21593_5};	/*Property = ScreenPixelsWidth*/
const Profile P21593 = {"21593", &P21593_4};	/*Profile ID - 21593*/
const Value V21333_1_1 = {S2, NULL};	/*Value = True*/
const Property P21333_1 = {S50, 0, &V21333_1_1, NULL};	/*Property = IsMobile*/
const Value V21333_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21333_5 = {S52, 0, &V21333_5_1227, &P21333_1};	/*Property = ScreenPixelsHeight*/
const Value V21333_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21333_4 = {S54, 0, &V21333_4_1229, &P21333_5};	/*Property = ScreenPixelsWidth*/
const Profile P21333 = {"21333", &P21333_4};	/*Profile ID - 21333*/
const Value V20209_1_1 = {S2, NULL};	/*Value = True*/
const Property P20209_1 = {S50, 0, &V20209_1_1, NULL};	/*Property = IsMobile*/
const Value V20209_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20209_5 = {S52, 0, &V20209_5_1227, &P20209_1};	/*Property = ScreenPixelsHeight*/
const Value V20209_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20209_4 = {S54, 0, &V20209_4_1229, &P20209_5};	/*Property = ScreenPixelsWidth*/
const Profile P20209 = {"20209", &P20209_4};	/*Profile ID - 20209*/
const Value V21048_1_1 = {S2, NULL};	/*Value = True*/
const Property P21048_1 = {S50, 0, &V21048_1_1, NULL};	/*Property = IsMobile*/
const Value V21048_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21048_5 = {S52, 0, &V21048_5_1227, &P21048_1};	/*Property = ScreenPixelsHeight*/
const Value V21048_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21048_4 = {S54, 0, &V21048_4_1229, &P21048_5};	/*Property = ScreenPixelsWidth*/
const Profile P21048 = {"21048", &P21048_4};	/*Profile ID - 21048*/
const Value V21386_1_1 = {S2, NULL};	/*Value = True*/
const Property P21386_1 = {S50, 0, &V21386_1_1, NULL};	/*Property = IsMobile*/
const Value V21386_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21386_5 = {S52, 0, &V21386_5_1227, &P21386_1};	/*Property = ScreenPixelsHeight*/
const Value V21386_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21386_4 = {S54, 0, &V21386_4_1229, &P21386_5};	/*Property = ScreenPixelsWidth*/
const Profile P21386 = {"21386", &P21386_4};	/*Profile ID - 21386*/
const Value V22142_1_1 = {S2, NULL};	/*Value = True*/
const Property P22142_1 = {S50, 0, &V22142_1_1, NULL};	/*Property = IsMobile*/
const Value V22142_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22142_5 = {S52, 0, &V22142_5_1832, &P22142_1};	/*Property = ScreenPixelsHeight*/
const Value V22142_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22142_4 = {S54, 0, &V22142_4_5645, &P22142_5};	/*Property = ScreenPixelsWidth*/
const Profile P22142 = {"22142", &P22142_4};	/*Profile ID - 22142*/
const Value V20860_1_1 = {S2, NULL};	/*Value = True*/
const Property P20860_1 = {S50, 0, &V20860_1_1, NULL};	/*Property = IsMobile*/
const Value V20860_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20860_5 = {S52, 0, &V20860_5_1227, &P20860_1};	/*Property = ScreenPixelsHeight*/
const Value V20860_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20860_4 = {S54, 0, &V20860_4_1229, &P20860_5};	/*Property = ScreenPixelsWidth*/
const Profile P20860 = {"20860", &P20860_4};	/*Profile ID - 20860*/
const Value V22139_1_1 = {S2, NULL};	/*Value = True*/
const Property P22139_1 = {S50, 0, &V22139_1_1, NULL};	/*Property = IsMobile*/
const Value V22139_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22139_5 = {S52, 0, &V22139_5_37569, &P22139_1};	/*Property = ScreenPixelsHeight*/
const Value V22139_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22139_4 = {S54, 0, &V22139_4_5645, &P22139_5};	/*Property = ScreenPixelsWidth*/
const Profile P22139 = {"22139", &P22139_4};	/*Profile ID - 22139*/
const Value V23130_1_1 = {S2, NULL};	/*Value = True*/
const Property P23130_1 = {S50, 0, &V23130_1_1, NULL};	/*Property = IsMobile*/
const Value V23130_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23130_5 = {S52, 0, &V23130_5_1269, &P23130_1};	/*Property = ScreenPixelsHeight*/
const Value V23130_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23130_4 = {S54, 0, &V23130_4_1229, &P23130_5};	/*Property = ScreenPixelsWidth*/
const Profile P23130 = {"23130", &P23130_4};	/*Profile ID - 23130*/
const Value V21133_1_1 = {S2, NULL};	/*Value = True*/
const Property P21133_1 = {S50, 0, &V21133_1_1, NULL};	/*Property = IsMobile*/
const Value V21133_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21133_5 = {S52, 0, &V21133_5_1227, &P21133_1};	/*Property = ScreenPixelsHeight*/
const Value V21133_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21133_4 = {S54, 0, &V21133_4_1229, &P21133_5};	/*Property = ScreenPixelsWidth*/
const Profile P21133 = {"21133", &P21133_4};	/*Profile ID - 21133*/
const Value V20960_1_1 = {S2, NULL};	/*Value = True*/
const Property P20960_1 = {S50, 0, &V20960_1_1, NULL};	/*Property = IsMobile*/
const Value V20960_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20960_5 = {S52, 0, &V20960_5_1277, &P20960_1};	/*Property = ScreenPixelsHeight*/
const Value V20960_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20960_4 = {S54, 0, &V20960_4_1279, &P20960_5};	/*Property = ScreenPixelsWidth*/
const Profile P20960 = {"20960", &P20960_4};	/*Profile ID - 20960*/
const Value V23911_1_1 = {S2, NULL};	/*Value = True*/
const Property P23911_1 = {S50, 0, &V23911_1_1, NULL};	/*Property = IsMobile*/
const Value V23911_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23911_5 = {S52, 0, &V23911_5_1227, &P23911_1};	/*Property = ScreenPixelsHeight*/
const Value V23911_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23911_4 = {S54, 0, &V23911_4_1229, &P23911_5};	/*Property = ScreenPixelsWidth*/
const Profile P23911 = {"23911", &P23911_4};	/*Profile ID - 23911*/
const Value V21026_1_1 = {S2, NULL};	/*Value = True*/
const Property P21026_1 = {S50, 0, &V21026_1_1, NULL};	/*Property = IsMobile*/
const Value V21026_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21026_5 = {S52, 0, &V21026_5_1403, &P21026_1};	/*Property = ScreenPixelsHeight*/
const Value V21026_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21026_4 = {S54, 0, &V21026_4_1404, &P21026_5};	/*Property = ScreenPixelsWidth*/
const Profile P21026 = {"21026", &P21026_4};	/*Profile ID - 21026*/
const Value V22033_1_1 = {S2, NULL};	/*Value = True*/
const Property P22033_1 = {S50, 0, &V22033_1_1, NULL};	/*Property = IsMobile*/
const Value V22033_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22033_5 = {S52, 0, &V22033_5_1403, &P22033_1};	/*Property = ScreenPixelsHeight*/
const Value V22033_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22033_4 = {S54, 0, &V22033_4_1404, &P22033_5};	/*Property = ScreenPixelsWidth*/
const Profile P22033 = {"22033", &P22033_4};	/*Profile ID - 22033*/
const Value V20410_1_1 = {S2, NULL};	/*Value = True*/
const Property P20410_1 = {S50, 0, &V20410_1_1, NULL};	/*Property = IsMobile*/
const Value V20410_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20410_5 = {S52, 0, &V20410_5_1458, &P20410_1};	/*Property = ScreenPixelsHeight*/
const Value V20410_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20410_4 = {S54, 0, &V20410_4_37617, &P20410_5};	/*Property = ScreenPixelsWidth*/
const Profile P20410 = {"20410", &P20410_4};	/*Profile ID - 20410*/
const Value V23860_1_1 = {S2, NULL};	/*Value = True*/
const Property P23860_1 = {S50, 0, &V23860_1_1, NULL};	/*Property = IsMobile*/
const Value V23860_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23860_5 = {S52, 0, &V23860_5_1277, &P23860_1};	/*Property = ScreenPixelsHeight*/
const Value V23860_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23860_4 = {S54, 0, &V23860_4_1279, &P23860_5};	/*Property = ScreenPixelsWidth*/
const Profile P23860 = {"23860", &P23860_4};	/*Profile ID - 23860*/
const Value V22065_1_1 = {S2, NULL};	/*Value = True*/
const Property P22065_1 = {S50, 0, &V22065_1_1, NULL};	/*Property = IsMobile*/
const Value V22065_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22065_5 = {S52, 0, &V22065_5_1227, &P22065_1};	/*Property = ScreenPixelsHeight*/
const Value V22065_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22065_4 = {S54, 0, &V22065_4_1229, &P22065_5};	/*Property = ScreenPixelsWidth*/
const Profile P22065 = {"22065", &P22065_4};	/*Profile ID - 22065*/
const Value V21642_1_1 = {S2, NULL};	/*Value = True*/
const Property P21642_1 = {S50, 0, &V21642_1_1, NULL};	/*Property = IsMobile*/
const Value V21642_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21642_5 = {S52, 0, &V21642_5_1227, &P21642_1};	/*Property = ScreenPixelsHeight*/
const Value V21642_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21642_4 = {S54, 0, &V21642_4_1460, &P21642_5};	/*Property = ScreenPixelsWidth*/
const Profile P21642 = {"21642", &P21642_4};	/*Profile ID - 21642*/
const Value V23897_1_1 = {S2, NULL};	/*Value = True*/
const Property P23897_1 = {S50, 0, &V23897_1_1, NULL};	/*Property = IsMobile*/
const Value V23897_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23897_5 = {S52, 0, &V23897_5_37569, &P23897_1};	/*Property = ScreenPixelsHeight*/
const Value V23897_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23897_4 = {S54, 0, &V23897_4_5645, &P23897_5};	/*Property = ScreenPixelsWidth*/
const Profile P23897 = {"23897", &P23897_4};	/*Profile ID - 23897*/
const Value V20988_1_1 = {S2, NULL};	/*Value = True*/
const Property P20988_1 = {S50, 0, &V20988_1_1, NULL};	/*Property = IsMobile*/
const Value V20988_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20988_5 = {S52, 0, &V20988_5_1227, &P20988_1};	/*Property = ScreenPixelsHeight*/
const Value V20988_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20988_4 = {S54, 0, &V20988_4_1229, &P20988_5};	/*Property = ScreenPixelsWidth*/
const Profile P20988 = {"20988", &P20988_4};	/*Profile ID - 20988*/
const Value V21284_1_1 = {S2, NULL};	/*Value = True*/
const Property P21284_1 = {S50, 0, &V21284_1_1, NULL};	/*Property = IsMobile*/
const Value V21284_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21284_5 = {S52, 0, &V21284_5_1458, &P21284_1};	/*Property = ScreenPixelsHeight*/
const Value V21284_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21284_4 = {S54, 0, &V21284_4_1460, &P21284_5};	/*Property = ScreenPixelsWidth*/
const Profile P21284 = {"21284", &P21284_4};	/*Profile ID - 21284*/
const Value V20940_1_1 = {S2, NULL};	/*Value = True*/
const Property P20940_1 = {S50, 0, &V20940_1_1, NULL};	/*Property = IsMobile*/
const Value V20940_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20940_5 = {S52, 0, &V20940_5_1227, &P20940_1};	/*Property = ScreenPixelsHeight*/
const Value V20940_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20940_4 = {S54, 0, &V20940_4_1229, &P20940_5};	/*Property = ScreenPixelsWidth*/
const Profile P20940 = {"20940", &P20940_4};	/*Profile ID - 20940*/
const Value V20730_1_1 = {S2, NULL};	/*Value = True*/
const Property P20730_1 = {S50, 0, &V20730_1_1, NULL};	/*Property = IsMobile*/
const Value V20730_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20730_5 = {S52, 0, &V20730_5_1227, &P20730_1};	/*Property = ScreenPixelsHeight*/
const Value V20730_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20730_4 = {S54, 0, &V20730_4_1229, &P20730_5};	/*Property = ScreenPixelsWidth*/
const Profile P20730 = {"20730", &P20730_4};	/*Profile ID - 20730*/
const Value V17174_1_1 = {S2, NULL};	/*Value = True*/
const Property P17174_1 = {S50, 0, &V17174_1_1, NULL};	/*Property = IsMobile*/
const Value V17174_5_37526 = {S122, NULL};	/*Value = 1200*/
const Property P17174_5 = {S52, 0, &V17174_5_37526, &P17174_1};	/*Property = ScreenPixelsHeight*/
const Value V17174_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P17174_4 = {S54, 0, &V17174_4_57095, &P17174_5};	/*Property = ScreenPixelsWidth*/
const Profile P17174 = {"17174", &P17174_4};	/*Profile ID - 17174*/
const Value V20939_1_1 = {S2, NULL};	/*Value = True*/
const Property P20939_1 = {S50, 0, &V20939_1_1, NULL};	/*Property = IsMobile*/
const Value V20939_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20939_5 = {S52, 0, &V20939_5_1227, &P20939_1};	/*Property = ScreenPixelsHeight*/
const Value V20939_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20939_4 = {S54, 0, &V20939_4_1229, &P20939_5};	/*Property = ScreenPixelsWidth*/
const Profile P20939 = {"20939", &P20939_4};	/*Profile ID - 20939*/
const Value V19878_1_1 = {S2, NULL};	/*Value = True*/
const Property P19878_1 = {S50, 0, &V19878_1_1, NULL};	/*Property = IsMobile*/
const Value V19878_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19878_5 = {S52, 0, &V19878_5_9981, &P19878_1};	/*Property = ScreenPixelsHeight*/
const Value V19878_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P19878_4 = {S54, 0, &V19878_4_2336, &P19878_5};	/*Property = ScreenPixelsWidth*/
const Profile P19878 = {"19878", &P19878_4};	/*Profile ID - 19878*/
const Value V23158_1_1 = {S2, NULL};	/*Value = True*/
const Property P23158_1 = {S50, 0, &V23158_1_1, NULL};	/*Property = IsMobile*/
const Value V23158_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23158_5 = {S52, 0, &V23158_5_1227, &P23158_1};	/*Property = ScreenPixelsHeight*/
const Value V23158_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23158_4 = {S54, 0, &V23158_4_1229, &P23158_5};	/*Property = ScreenPixelsWidth*/
const Profile P23158 = {"23158", &P23158_4};	/*Profile ID - 23158*/
const Value V20481_1_1 = {S2, NULL};	/*Value = True*/
const Property P20481_1 = {S50, 0, &V20481_1_1, NULL};	/*Property = IsMobile*/
const Value V20481_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20481_5 = {S52, 0, &V20481_5_1227, &P20481_1};	/*Property = ScreenPixelsHeight*/
const Value V20481_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20481_4 = {S54, 0, &V20481_4_1229, &P20481_5};	/*Property = ScreenPixelsWidth*/
const Profile P20481 = {"20481", &P20481_4};	/*Profile ID - 20481*/
const Value V18556_1_1 = {S2, NULL};	/*Value = True*/
const Property P18556_1 = {S50, 0, &V18556_1_1, NULL};	/*Property = IsMobile*/
const Value V18556_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P18556_5 = {S52, 0, &V18556_5_59138, &P18556_1};	/*Property = ScreenPixelsHeight*/
const Value V18556_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P18556_4 = {S54, 0, &V18556_4_1477, &P18556_5};	/*Property = ScreenPixelsWidth*/
const Profile P18556 = {"18556", &P18556_4};	/*Profile ID - 18556*/
const Value V20816_1_1 = {S2, NULL};	/*Value = True*/
const Property P20816_1 = {S50, 0, &V20816_1_1, NULL};	/*Property = IsMobile*/
const Value V20816_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20816_5 = {S52, 0, &V20816_5_1227, &P20816_1};	/*Property = ScreenPixelsHeight*/
const Value V20816_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20816_4 = {S54, 0, &V20816_4_1229, &P20816_5};	/*Property = ScreenPixelsWidth*/
const Profile P20816 = {"20816", &P20816_4};	/*Profile ID - 20816*/
const Value V18470_1_1 = {S2, NULL};	/*Value = True*/
const Property P18470_1 = {S50, 0, &V18470_1_1, NULL};	/*Property = IsMobile*/
const Value V18470_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18470_5 = {S52, 0, &V18470_5_9981, &P18470_1};	/*Property = ScreenPixelsHeight*/
const Value V18470_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P18470_4 = {S54, 0, &V18470_4_2336, &P18470_5};	/*Property = ScreenPixelsWidth*/
const Profile P18470 = {"18470", &P18470_4};	/*Profile ID - 18470*/
const Value V21028_1_1 = {S2, NULL};	/*Value = True*/
const Property P21028_1 = {S50, 0, &V21028_1_1, NULL};	/*Property = IsMobile*/
const Value V21028_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21028_5 = {S52, 0, &V21028_5_1458, &P21028_1};	/*Property = ScreenPixelsHeight*/
const Value V21028_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21028_4 = {S54, 0, &V21028_4_1460, &P21028_5};	/*Property = ScreenPixelsWidth*/
const Profile P21028 = {"21028", &P21028_4};	/*Profile ID - 21028*/
const Value V19865_1_1 = {S2, NULL};	/*Value = True*/
const Property P19865_1 = {S50, 0, &V19865_1_1, NULL};	/*Property = IsMobile*/
const Value V19865_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19865_5 = {S52, 0, &V19865_5_9981, &P19865_1};	/*Property = ScreenPixelsHeight*/
const Value V19865_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P19865_4 = {S54, 0, &V19865_4_39098, &P19865_5};	/*Property = ScreenPixelsWidth*/
const Profile P19865 = {"19865", &P19865_4};	/*Profile ID - 19865*/
const Value V22165_1_1 = {S2, NULL};	/*Value = True*/
const Property P22165_1 = {S50, 0, &V22165_1_1, NULL};	/*Property = IsMobile*/
const Value V22165_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22165_5 = {S52, 0, &V22165_5_1277, &P22165_1};	/*Property = ScreenPixelsHeight*/
const Value V22165_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22165_4 = {S54, 0, &V22165_4_2336, &P22165_5};	/*Property = ScreenPixelsWidth*/
const Profile P22165 = {"22165", &P22165_4};	/*Profile ID - 22165*/
const Value V20252_1_1 = {S2, NULL};	/*Value = True*/
const Property P20252_1 = {S50, 0, &V20252_1_1, NULL};	/*Property = IsMobile*/
const Value V20252_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20252_5 = {S52, 0, &V20252_5_9981, &P20252_1};	/*Property = ScreenPixelsHeight*/
const Value V20252_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20252_4 = {S54, 0, &V20252_4_2336, &P20252_5};	/*Property = ScreenPixelsWidth*/
const Profile P20252 = {"20252", &P20252_4};	/*Profile ID - 20252*/
const Value V20290_1_1 = {S2, NULL};	/*Value = True*/
const Property P20290_1 = {S50, 0, &V20290_1_1, NULL};	/*Property = IsMobile*/
const Value V20290_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20290_5 = {S52, 0, &V20290_5_1227, &P20290_1};	/*Property = ScreenPixelsHeight*/
const Value V20290_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20290_4 = {S54, 0, &V20290_4_1229, &P20290_5};	/*Property = ScreenPixelsWidth*/
const Profile P20290 = {"20290", &P20290_4};	/*Profile ID - 20290*/
const Value V21859_1_1 = {S2, NULL};	/*Value = True*/
const Property P21859_1 = {S50, 0, &V21859_1_1, NULL};	/*Property = IsMobile*/
const Value V21859_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21859_5 = {S52, 0, &V21859_5_1227, &P21859_1};	/*Property = ScreenPixelsHeight*/
const Value V21859_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21859_4 = {S54, 0, &V21859_4_1229, &P21859_5};	/*Property = ScreenPixelsWidth*/
const Profile P21859 = {"21859", &P21859_4};	/*Profile ID - 21859*/
const Value V24735_1_1 = {S2, NULL};	/*Value = True*/
const Property P24735_1 = {S50, 0, &V24735_1_1, NULL};	/*Property = IsMobile*/
const Value V24735_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24735_5 = {S52, 0, &V24735_5_1277, &P24735_1};	/*Property = ScreenPixelsHeight*/
const Value V24735_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24735_4 = {S54, 0, &V24735_4_1279, &P24735_5};	/*Property = ScreenPixelsWidth*/
const Profile P24735 = {"24735", &P24735_4};	/*Profile ID - 24735*/
const Value V21352_1_1 = {S2, NULL};	/*Value = True*/
const Property P21352_1 = {S50, 0, &V21352_1_1, NULL};	/*Property = IsMobile*/
const Value V21352_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21352_5 = {S52, 0, &V21352_5_1458, &P21352_1};	/*Property = ScreenPixelsHeight*/
const Value V21352_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21352_4 = {S54, 0, &V21352_4_37617, &P21352_5};	/*Property = ScreenPixelsWidth*/
const Profile P21352 = {"21352", &P21352_4};	/*Profile ID - 21352*/
const Value V18463_1_1 = {S2, NULL};	/*Value = True*/
const Property P18463_1 = {S50, 0, &V18463_1_1, NULL};	/*Property = IsMobile*/
const Value V18463_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18463_5 = {S52, 0, &V18463_5_9981, &P18463_1};	/*Property = ScreenPixelsHeight*/
const Value V18463_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18463_4 = {S54, 0, &V18463_4_39098, &P18463_5};	/*Property = ScreenPixelsWidth*/
const Profile P18463 = {"18463", &P18463_4};	/*Profile ID - 18463*/
const Value V23906_1_1 = {S2, NULL};	/*Value = True*/
const Property P23906_1 = {S50, 0, &V23906_1_1, NULL};	/*Property = IsMobile*/
const Value V23906_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23906_5 = {S52, 0, &V23906_5_1458, &P23906_1};	/*Property = ScreenPixelsHeight*/
const Value V23906_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23906_4 = {S54, 0, &V23906_4_37617, &P23906_5};	/*Property = ScreenPixelsWidth*/
const Profile P23906 = {"23906", &P23906_4};	/*Profile ID - 23906*/
const Value V19811_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19811_279 = {S1, 0, &V19811_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19811_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19811_282 = {S3, 0, &V19811_282_57154, &P19811_279};	/*Property = BlobBuilder*/
const Value V19811_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19811_350 = {S4, 0, &V19811_350_57444, &P19811_282};	/*Property = Canvas*/
const Value V19811_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19811_283 = {S5, 0, &V19811_283_57159, &P19811_350};	/*Property = CssBackground*/
const Value V19811_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19811_284 = {S6, 0, &V19811_284_57163, &P19811_283};	/*Property = CssBorderImage*/
const Value V19811_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19811_285 = {S7, 0, &V19811_285_57166, &P19811_284};	/*Property = CssCanvas*/
const Value V19811_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19811_286 = {S8, 0, &V19811_286_57168, &P19811_285};	/*Property = CssColor*/
const Value V19811_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19811_237 = {S9, 0, &V19811_237_55786, &P19811_286};	/*Property = CssColumn*/
const Value V19811_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19811_287 = {S10, 0, &V19811_287_57172, &P19811_237};	/*Property = CssFlexbox*/
const Value V19811_288_57174 = {S2, NULL};	/*Value = True*/
const Property P19811_288 = {S11, 0, &V19811_288_57174, &P19811_287};	/*Property = CssFont*/
const Value V19811_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19811_289 = {S12, 0, &V19811_289_57759, &P19811_288};	/*Property = CssImages*/
const Value V19811_291_57180 = {S2, NULL};	/*Value = True*/
const Property P19811_291 = {S13, 0, &V19811_291_57180, &P19811_289};	/*Property = CssMediaQueries*/
const Value V19811_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19811_292 = {S14, 0, &V19811_292_57183, &P19811_291};	/*Property = CssMinMax*/
const Value V19811_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19811_293 = {S15, 0, &V19811_293_57187, &P19811_292};	/*Property = CssOverflow*/
const Value V19811_294_57189 = {S2, NULL};	/*Value = True*/
const Property P19811_294 = {S16, 0, &V19811_294_57189, &P19811_293};	/*Property = CssPosition*/
const Value V19811_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19811_295 = {S17, 0, &V19811_295_57192, &P19811_294};	/*Property = CssText*/
const Value V19811_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19811_297 = {S18, 0, &V19811_297_57195, &P19811_295};	/*Property = CssTransforms*/
const Value V19811_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19811_241 = {S19, 0, &V19811_241_55799, &P19811_297};	/*Property = CssTransitions*/
const Value V19811_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19811_298 = {S20, 0, &V19811_298_57206, &P19811_241};	/*Property = CssUI*/
const Value V19811_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19811_300 = {S21, 0, &V19811_300_57216, &P19811_298};	/*Property = DataSet*/
const Value V19811_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19811_301 = {S22, 0, &V19811_301_57218, &P19811_300};	/*Property = DataUrl*/
const Value V19811_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19811_302 = {S23, 0, &V19811_302_57222, &P19811_301};	/*Property = DeviceOrientation*/
const Value V19811_304_57227 = {S2, NULL};	/*Value = True*/
const Property P19811_304 = {S24, 0, &V19811_304_57227, &P19811_302};	/*Property = FileReader*/
const Value V19811_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19811_305 = {S25, 0, &V19811_305_57231, &P19811_304};	/*Property = FileSaver*/
const Value V19811_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19811_306 = {S26, 0, &V19811_306_57237, &P19811_305};	/*Property = FileWriter*/
const Value V19811_307_57239 = {S2, NULL};	/*Value = True*/
const Property P19811_307 = {S27, 0, &V19811_307_57239, &P19811_306};	/*Property = FormData*/
const Value V19811_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19811_309 = {S28, 0, &V19811_309_57246, &P19811_307};	/*Property = Fullscreen*/
const Value V19811_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19811_310 = {S29, 0, &V19811_310_57248, &P19811_309};	/*Property = GeoLocation*/
const Value V19811_312_57254 = {S2, NULL};	/*Value = True*/
const Property P19811_312 = {S30, 0, &V19811_312_57254, &P19811_310};	/*Property = History*/
const Value V19811_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19811_314 = {S31, 0, &V19811_314_57261, &P19811_312};	/*Property = Html5*/
const Value V19811_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19811_313 = {S32, 0, &V19811_313_57258, &P19811_314};	/*Property = Html-Media-Capture*/
const Value V19811_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19811_315 = {S33, 0, &V19811_315_57264, &P19811_313};	/*Property = Iframe*/
const Value V19811_316_57266 = {S2, NULL};	/*Value = True*/
const Property P19811_316 = {S34, 0, &V19811_316_57266, &P19811_315};	/*Property = IndexedDB*/
const Value V19811_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19811_317 = {S35, 0, &V19811_317_57269, &P19811_316};	/*Property = Json*/
const Value V19811_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19811_15 = {S37, 0, &V19811_15_28, &P19811_317};	/*Property = LayoutEngine*/
const Value V19811_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19811_344 = {S38, 0, &V19811_344_57272, &P19811_15};	/*Property = Masking*/
const Value V19811_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19811_320 = {S39, 0, &V19811_320_57290, &P19811_344};	/*Property = PostMessage*/
const Value V19811_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19811_321 = {S40, 0, &V19811_321_57293, &P19811_320};	/*Property = Progress*/
const Value V19811_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19811_322 = {S41, 0, &V19811_322_57296, &P19811_321};	/*Property = Prompts*/
const Value V19811_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19811_323 = {S42, 0, &V19811_323_57299, &P19811_322};	/*Property = Selector*/
const Value V19811_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19811_324 = {S43, 0, &V19811_324_57306, &P19811_323};	/*Property = Svg*/
const Value V19811_327_57315 = {S0, NULL};	/*Value = False*/
const Property P19811_327 = {S44, 0, &V19811_327_57315, &P19811_324};	/*Property = TouchEvents*/
const Value V19811_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19811_328 = {S45, 0, &V19811_328_57318, &P19811_327};	/*Property = Track*/
const Value V19811_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19811_329 = {S46, 0, &V19811_329_57323, &P19811_328};	/*Property = Video*/
const Value V19811_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19811_330 = {S47, 0, &V19811_330_57326, &P19811_329};	/*Property = Viewport*/
const Value V19811_332_57338 = {S2, NULL};	/*Value = True*/
const Property P19811_332 = {S48, 0, &V19811_332_57338, &P19811_330};	/*Property = WebWorkers*/
const Value V19811_333_57341 = {S2, NULL};	/*Value = True*/
const Property P19811_333 = {S49, 0, &V19811_333_57341, &P19811_332};	/*Property = Xhr2*/
const Profile P19811 = {"19811", &P19811_333};	/*Profile ID - 19811*/
const Profile P20706 = {"20706", NULL};	/*Profile ID - 20706*/
const Value V18557_1_1 = {S2, NULL};	/*Value = True*/
const Property P18557_1 = {S50, 0, &V18557_1_1, NULL};	/*Property = IsMobile*/
const Value V18557_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P18557_5 = {S52, 0, &V18557_5_1304, &P18557_1};	/*Property = ScreenPixelsHeight*/
const Value V18557_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18557_4 = {S54, 0, &V18557_4_1224, &P18557_5};	/*Property = ScreenPixelsWidth*/
const Profile P18557 = {"18557", &P18557_4};	/*Profile ID - 18557*/
const Profile P18667 = {"18667", NULL};	/*Profile ID - 18667*/
const Value V22404_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22404_279 = {S1, 0, &V22404_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22404_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22404_282 = {S3, 0, &V22404_282_57153, &P22404_279};	/*Property = BlobBuilder*/
const Value V22404_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22404_350 = {S4, 0, &V22404_350_57444, &P22404_282};	/*Property = Canvas*/
const Value V22404_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22404_283 = {S5, 0, &V22404_283_57159, &P22404_350};	/*Property = CssBackground*/
const Value V22404_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22404_284 = {S6, 0, &V22404_284_57163, &P22404_283};	/*Property = CssBorderImage*/
const Value V22404_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22404_285 = {S7, 0, &V22404_285_57166, &P22404_284};	/*Property = CssCanvas*/
const Value V22404_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22404_286 = {S8, 0, &V22404_286_57168, &P22404_285};	/*Property = CssColor*/
const Value V22404_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22404_237 = {S9, 0, &V22404_237_55784, &P22404_286};	/*Property = CssColumn*/
const Value V22404_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22404_287 = {S10, 0, &V22404_287_57172, &P22404_237};	/*Property = CssFlexbox*/
const Value V22404_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22404_288 = {S11, 0, &V22404_288_57175, &P22404_287};	/*Property = CssFont*/
const Value V22404_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22404_289 = {S12, 0, &V22404_289_57759, &P22404_288};	/*Property = CssImages*/
const Value V22404_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22404_291 = {S13, 0, &V22404_291_57180, &P22404_289};	/*Property = CssMediaQueries*/
const Value V22404_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22404_292 = {S14, 0, &V22404_292_57183, &P22404_291};	/*Property = CssMinMax*/
const Value V22404_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22404_293 = {S15, 0, &V22404_293_57187, &P22404_292};	/*Property = CssOverflow*/
const Value V22404_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22404_294 = {S16, 0, &V22404_294_57189, &P22404_293};	/*Property = CssPosition*/
const Value V22404_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22404_295 = {S17, 0, &V22404_295_57192, &P22404_294};	/*Property = CssText*/
const Value V22404_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22404_297 = {S18, 0, &V22404_297_57195, &P22404_295};	/*Property = CssTransforms*/
const Value V22404_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22404_241 = {S19, 0, &V22404_241_55799, &P22404_297};	/*Property = CssTransitions*/
const Value V22404_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22404_298 = {S20, 0, &V22404_298_57206, &P22404_241};	/*Property = CssUI*/
const Value V22404_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22404_300 = {S21, 0, &V22404_300_57216, &P22404_298};	/*Property = DataSet*/
const Value V22404_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22404_301 = {S22, 0, &V22404_301_57218, &P22404_300};	/*Property = DataUrl*/
const Value V22404_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22404_302 = {S23, 0, &V22404_302_57222, &P22404_301};	/*Property = DeviceOrientation*/
const Value V22404_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22404_304 = {S24, 0, &V22404_304_57227, &P22404_302};	/*Property = FileReader*/
const Value V22404_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22404_305 = {S25, 0, &V22404_305_57231, &P22404_304};	/*Property = FileSaver*/
const Value V22404_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22404_306 = {S26, 0, &V22404_306_57237, &P22404_305};	/*Property = FileWriter*/
const Value V22404_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22404_307 = {S27, 0, &V22404_307_57239, &P22404_306};	/*Property = FormData*/
const Value V22404_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22404_309 = {S28, 0, &V22404_309_57246, &P22404_307};	/*Property = Fullscreen*/
const Value V22404_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22404_310 = {S29, 0, &V22404_310_57248, &P22404_309};	/*Property = GeoLocation*/
const Value V22404_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22404_312 = {S30, 0, &V22404_312_57255, &P22404_310};	/*Property = History*/
const Value V22404_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22404_314 = {S31, 0, &V22404_314_57261, &P22404_312};	/*Property = Html5*/
const Value V22404_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22404_313 = {S32, 0, &V22404_313_57258, &P22404_314};	/*Property = Html-Media-Capture*/
const Value V22404_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22404_315 = {S33, 0, &V22404_315_57264, &P22404_313};	/*Property = Iframe*/
const Value V22404_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22404_316 = {S34, 0, &V22404_316_57266, &P22404_315};	/*Property = IndexedDB*/
const Value V22404_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22404_317 = {S35, 0, &V22404_317_57269, &P22404_316};	/*Property = Json*/
const Value V22404_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22404_15 = {S37, 0, &V22404_15_28, &P22404_317};	/*Property = LayoutEngine*/
const Value V22404_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22404_344 = {S38, 0, &V22404_344_57272, &P22404_15};	/*Property = Masking*/
const Value V22404_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22404_320 = {S39, 0, &V22404_320_57290, &P22404_344};	/*Property = PostMessage*/
const Value V22404_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22404_321 = {S40, 0, &V22404_321_57293, &P22404_320};	/*Property = Progress*/
const Value V22404_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22404_322 = {S41, 0, &V22404_322_57296, &P22404_321};	/*Property = Prompts*/
const Value V22404_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22404_323 = {S42, 0, &V22404_323_57299, &P22404_322};	/*Property = Selector*/
const Value V22404_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22404_324 = {S43, 0, &V22404_324_57306, &P22404_323};	/*Property = Svg*/
const Value V22404_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22404_327 = {S44, 0, &V22404_327_57315, &P22404_324};	/*Property = TouchEvents*/
const Value V22404_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22404_328 = {S45, 0, &V22404_328_57318, &P22404_327};	/*Property = Track*/
const Value V22404_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22404_329 = {S46, 0, &V22404_329_57323, &P22404_328};	/*Property = Video*/
const Value V22404_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22404_330 = {S47, 0, &V22404_330_57326, &P22404_329};	/*Property = Viewport*/
const Value V22404_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22404_332 = {S48, 0, &V22404_332_57338, &P22404_330};	/*Property = WebWorkers*/
const Value V22404_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22404_333 = {S49, 0, &V22404_333_57342, &P22404_332};	/*Property = Xhr2*/
const Profile P22404 = {"22404", &P22404_333};	/*Profile ID - 22404*/
const Value V20457_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20457_279 = {S1, 0, &V20457_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20457_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20457_282 = {S3, 0, &V20457_282_57154, &P20457_279};	/*Property = BlobBuilder*/
const Value V20457_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20457_350 = {S4, 0, &V20457_350_57444, &P20457_282};	/*Property = Canvas*/
const Value V20457_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20457_283 = {S5, 0, &V20457_283_57159, &P20457_350};	/*Property = CssBackground*/
const Value V20457_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20457_284 = {S6, 0, &V20457_284_57163, &P20457_283};	/*Property = CssBorderImage*/
const Value V20457_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20457_285 = {S7, 0, &V20457_285_57166, &P20457_284};	/*Property = CssCanvas*/
const Value V20457_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20457_286 = {S8, 0, &V20457_286_57168, &P20457_285};	/*Property = CssColor*/
const Value V20457_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20457_237 = {S9, 0, &V20457_237_55786, &P20457_286};	/*Property = CssColumn*/
const Value V20457_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20457_287 = {S10, 0, &V20457_287_57172, &P20457_237};	/*Property = CssFlexbox*/
const Value V20457_288_57174 = {S2, NULL};	/*Value = True*/
const Property P20457_288 = {S11, 0, &V20457_288_57174, &P20457_287};	/*Property = CssFont*/
const Value V20457_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20457_289 = {S12, 0, &V20457_289_57759, &P20457_288};	/*Property = CssImages*/
const Value V20457_291_57180 = {S2, NULL};	/*Value = True*/
const Property P20457_291 = {S13, 0, &V20457_291_57180, &P20457_289};	/*Property = CssMediaQueries*/
const Value V20457_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20457_292 = {S14, 0, &V20457_292_57183, &P20457_291};	/*Property = CssMinMax*/
const Value V20457_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20457_293 = {S15, 0, &V20457_293_57187, &P20457_292};	/*Property = CssOverflow*/
const Value V20457_294_57189 = {S2, NULL};	/*Value = True*/
const Property P20457_294 = {S16, 0, &V20457_294_57189, &P20457_293};	/*Property = CssPosition*/
const Value V20457_295_57192 = {S2, NULL};	/*Value = True*/
const Property P20457_295 = {S17, 0, &V20457_295_57192, &P20457_294};	/*Property = CssText*/
const Value V20457_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20457_297 = {S18, 0, &V20457_297_57195, &P20457_295};	/*Property = CssTransforms*/
const Value V20457_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20457_241 = {S19, 0, &V20457_241_55799, &P20457_297};	/*Property = CssTransitions*/
const Value V20457_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20457_298 = {S20, 0, &V20457_298_57206, &P20457_241};	/*Property = CssUI*/
const Value V20457_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20457_300 = {S21, 0, &V20457_300_57216, &P20457_298};	/*Property = DataSet*/
const Value V20457_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20457_301 = {S22, 0, &V20457_301_57218, &P20457_300};	/*Property = DataUrl*/
const Value V20457_302_57221 = {S2, NULL};	/*Value = True*/
const Property P20457_302 = {S23, 0, &V20457_302_57221, &P20457_301};	/*Property = DeviceOrientation*/
const Value V20457_304_57227 = {S2, NULL};	/*Value = True*/
const Property P20457_304 = {S24, 0, &V20457_304_57227, &P20457_302};	/*Property = FileReader*/
const Value V20457_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20457_305 = {S25, 0, &V20457_305_57231, &P20457_304};	/*Property = FileSaver*/
const Value V20457_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20457_306 = {S26, 0, &V20457_306_57237, &P20457_305};	/*Property = FileWriter*/
const Value V20457_307_57239 = {S2, NULL};	/*Value = True*/
const Property P20457_307 = {S27, 0, &V20457_307_57239, &P20457_306};	/*Property = FormData*/
const Value V20457_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20457_309 = {S28, 0, &V20457_309_57246, &P20457_307};	/*Property = Fullscreen*/
const Value V20457_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20457_310 = {S29, 0, &V20457_310_57248, &P20457_309};	/*Property = GeoLocation*/
const Value V20457_312_57254 = {S2, NULL};	/*Value = True*/
const Property P20457_312 = {S30, 0, &V20457_312_57254, &P20457_310};	/*Property = History*/
const Value V20457_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20457_314 = {S31, 0, &V20457_314_57261, &P20457_312};	/*Property = Html5*/
const Value V20457_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20457_313 = {S32, 0, &V20457_313_57258, &P20457_314};	/*Property = Html-Media-Capture*/
const Value V20457_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20457_315 = {S33, 0, &V20457_315_57264, &P20457_313};	/*Property = Iframe*/
const Value V20457_316_57266 = {S2, NULL};	/*Value = True*/
const Property P20457_316 = {S34, 0, &V20457_316_57266, &P20457_315};	/*Property = IndexedDB*/
const Value V20457_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20457_317 = {S35, 0, &V20457_317_57269, &P20457_316};	/*Property = Json*/
const Value V20457_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20457_15 = {S37, 0, &V20457_15_29, &P20457_317};	/*Property = LayoutEngine*/
const Value V20457_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20457_344 = {S38, 0, &V20457_344_57272, &P20457_15};	/*Property = Masking*/
const Value V20457_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20457_320 = {S39, 0, &V20457_320_57290, &P20457_344};	/*Property = PostMessage*/
const Value V20457_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20457_321 = {S40, 0, &V20457_321_57293, &P20457_320};	/*Property = Progress*/
const Value V20457_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20457_322 = {S41, 0, &V20457_322_57296, &P20457_321};	/*Property = Prompts*/
const Value V20457_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20457_323 = {S42, 0, &V20457_323_57299, &P20457_322};	/*Property = Selector*/
const Value V20457_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20457_324 = {S43, 0, &V20457_324_57306, &P20457_323};	/*Property = Svg*/
const Value V20457_327_57315 = {S0, NULL};	/*Value = False*/
const Property P20457_327 = {S44, 0, &V20457_327_57315, &P20457_324};	/*Property = TouchEvents*/
const Value V20457_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20457_328 = {S45, 0, &V20457_328_57318, &P20457_327};	/*Property = Track*/
const Value V20457_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20457_329 = {S46, 0, &V20457_329_57323, &P20457_328};	/*Property = Video*/
const Value V20457_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20457_330 = {S47, 0, &V20457_330_57326, &P20457_329};	/*Property = Viewport*/
const Value V20457_332_57338 = {S2, NULL};	/*Value = True*/
const Property P20457_332 = {S48, 0, &V20457_332_57338, &P20457_330};	/*Property = WebWorkers*/
const Value V20457_333_57341 = {S2, NULL};	/*Value = True*/
const Property P20457_333 = {S49, 0, &V20457_333_57341, &P20457_332};	/*Property = Xhr2*/
const Profile P20457 = {"20457", &P20457_333};	/*Profile ID - 20457*/
const Profile P22189 = {"22189", NULL};	/*Profile ID - 22189*/
const Value V21942_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_279 = {S1, 0, &V21942_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21942_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_282 = {S3, 0, &V21942_282_57155, &P21942_279};	/*Property = BlobBuilder*/
const Value V21942_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_350 = {S4, 0, &V21942_350_57446, &P21942_282};	/*Property = Canvas*/
const Value V21942_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_283 = {S5, 0, &V21942_283_57161, &P21942_350};	/*Property = CssBackground*/
const Value V21942_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_284 = {S6, 0, &V21942_284_57164, &P21942_283};	/*Property = CssBorderImage*/
const Value V21942_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_285 = {S7, 0, &V21942_285_57167, &P21942_284};	/*Property = CssCanvas*/
const Value V21942_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_286 = {S8, 0, &V21942_286_57170, &P21942_285};	/*Property = CssColor*/
const Value V21942_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_237 = {S9, 0, &V21942_237_55786, &P21942_286};	/*Property = CssColumn*/
const Value V21942_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_287 = {S10, 0, &V21942_287_57173, &P21942_237};	/*Property = CssFlexbox*/
const Value V21942_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_288 = {S11, 0, &V21942_288_57176, &P21942_287};	/*Property = CssFont*/
const Value V21942_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_289 = {S12, 0, &V21942_289_86899, &P21942_288};	/*Property = CssImages*/
const Value V21942_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_291 = {S13, 0, &V21942_291_57182, &P21942_289};	/*Property = CssMediaQueries*/
const Value V21942_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_292 = {S14, 0, &V21942_292_57185, &P21942_291};	/*Property = CssMinMax*/
const Value V21942_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_293 = {S15, 0, &V21942_293_57188, &P21942_292};	/*Property = CssOverflow*/
const Value V21942_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_294 = {S16, 0, &V21942_294_57191, &P21942_293};	/*Property = CssPosition*/
const Value V21942_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_295 = {S17, 0, &V21942_295_57194, &P21942_294};	/*Property = CssText*/
const Value V21942_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_297 = {S18, 0, &V21942_297_57197, &P21942_295};	/*Property = CssTransforms*/
const Value V21942_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_241 = {S19, 0, &V21942_241_55801, &P21942_297};	/*Property = CssTransitions*/
const Value V21942_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_298 = {S20, 0, &V21942_298_57208, &P21942_241};	/*Property = CssUI*/
const Value V21942_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_300 = {S21, 0, &V21942_300_57217, &P21942_298};	/*Property = DataSet*/
const Value V21942_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_301 = {S22, 0, &V21942_301_57220, &P21942_300};	/*Property = DataUrl*/
const Value V21942_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_302 = {S23, 0, &V21942_302_57223, &P21942_301};	/*Property = DeviceOrientation*/
const Value V21942_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_304 = {S24, 0, &V21942_304_57229, &P21942_302};	/*Property = FileReader*/
const Value V21942_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_305 = {S25, 0, &V21942_305_57232, &P21942_304};	/*Property = FileSaver*/
const Value V21942_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_306 = {S26, 0, &V21942_306_57238, &P21942_305};	/*Property = FileWriter*/
const Value V21942_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_307 = {S27, 0, &V21942_307_57241, &P21942_306};	/*Property = FormData*/
const Value V21942_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_309 = {S28, 0, &V21942_309_57247, &P21942_307};	/*Property = Fullscreen*/
const Value V21942_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_310 = {S29, 0, &V21942_310_57250, &P21942_309};	/*Property = GeoLocation*/
const Value V21942_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_312 = {S30, 0, &V21942_312_57256, &P21942_310};	/*Property = History*/
const Value V21942_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_314 = {S31, 0, &V21942_314_57262, &P21942_312};	/*Property = Html5*/
const Value V21942_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_313 = {S32, 0, &V21942_313_57259, &P21942_314};	/*Property = Html-Media-Capture*/
const Value V21942_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_315 = {S33, 0, &V21942_315_57265, &P21942_313};	/*Property = Iframe*/
const Value V21942_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_316 = {S34, 0, &V21942_316_57268, &P21942_315};	/*Property = IndexedDB*/
const Value V21942_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_317 = {S35, 0, &V21942_317_57271, &P21942_316};	/*Property = Json*/
const Value V21942_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P21942_15 = {S37, 0, &V21942_15_29, &P21942_317};	/*Property = LayoutEngine*/
const Value V21942_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_344 = {S38, 0, &V21942_344_57274, &P21942_15};	/*Property = Masking*/
const Value V21942_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_320 = {S39, 0, &V21942_320_57292, &P21942_344};	/*Property = PostMessage*/
const Value V21942_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_321 = {S40, 0, &V21942_321_57295, &P21942_320};	/*Property = Progress*/
const Value V21942_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_322 = {S41, 0, &V21942_322_57298, &P21942_321};	/*Property = Prompts*/
const Value V21942_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_323 = {S42, 0, &V21942_323_57301, &P21942_322};	/*Property = Selector*/
const Value V21942_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_324 = {S43, 0, &V21942_324_57307, &P21942_323};	/*Property = Svg*/
const Value V21942_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_327 = {S44, 0, &V21942_327_57316, &P21942_324};	/*Property = TouchEvents*/
const Value V21942_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_328 = {S45, 0, &V21942_328_57319, &P21942_327};	/*Property = Track*/
const Value V21942_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_329 = {S46, 0, &V21942_329_57325, &P21942_328};	/*Property = Video*/
const Value V21942_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_330 = {S47, 0, &V21942_330_57328, &P21942_329};	/*Property = Viewport*/
const Value V21942_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_332 = {S48, 0, &V21942_332_57340, &P21942_330};	/*Property = WebWorkers*/
const Value V21942_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21942_333 = {S49, 0, &V21942_333_57343, &P21942_332};	/*Property = Xhr2*/
const Profile P21942 = {"21942", &P21942_333};	/*Profile ID - 21942*/
const Value V23351_1_1 = {S2, NULL};	/*Value = True*/
const Property P23351_1 = {S50, 0, &V23351_1_1, NULL};	/*Property = IsMobile*/
const Value V23351_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23351_5 = {S52, 0, &V23351_5_37, &P23351_1};	/*Property = ScreenPixelsHeight*/
const Value V23351_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23351_4 = {S54, 0, &V23351_4_36, &P23351_5};	/*Property = ScreenPixelsWidth*/
const Profile P23351 = {"23351", &P23351_4};	/*Profile ID - 23351*/
const Value V23978_1_1 = {S2, NULL};	/*Value = True*/
const Property P23978_1 = {S50, 0, &V23978_1_1, NULL};	/*Property = IsMobile*/
const Value V23978_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23978_5 = {S52, 0, &V23978_5_1227, &P23978_1};	/*Property = ScreenPixelsHeight*/
const Value V23978_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23978_4 = {S54, 0, &V23978_4_1229, &P23978_5};	/*Property = ScreenPixelsWidth*/
const Profile P23978 = {"23978", &P23978_4};	/*Profile ID - 23978*/
const Value V21724_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_279 = {S1, 0, &V21724_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21724_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_282 = {S3, 0, &V21724_282_57155, &P21724_279};	/*Property = BlobBuilder*/
const Value V21724_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_350 = {S4, 0, &V21724_350_57446, &P21724_282};	/*Property = Canvas*/
const Value V21724_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_283 = {S5, 0, &V21724_283_57161, &P21724_350};	/*Property = CssBackground*/
const Value V21724_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_284 = {S6, 0, &V21724_284_57164, &P21724_283};	/*Property = CssBorderImage*/
const Value V21724_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_285 = {S7, 0, &V21724_285_57167, &P21724_284};	/*Property = CssCanvas*/
const Value V21724_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_286 = {S8, 0, &V21724_286_57170, &P21724_285};	/*Property = CssColor*/
const Value V21724_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_237 = {S9, 0, &V21724_237_55786, &P21724_286};	/*Property = CssColumn*/
const Value V21724_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_287 = {S10, 0, &V21724_287_57173, &P21724_237};	/*Property = CssFlexbox*/
const Value V21724_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_288 = {S11, 0, &V21724_288_57176, &P21724_287};	/*Property = CssFont*/
const Value V21724_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_289 = {S12, 0, &V21724_289_86899, &P21724_288};	/*Property = CssImages*/
const Value V21724_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_291 = {S13, 0, &V21724_291_57182, &P21724_289};	/*Property = CssMediaQueries*/
const Value V21724_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_292 = {S14, 0, &V21724_292_57185, &P21724_291};	/*Property = CssMinMax*/
const Value V21724_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_293 = {S15, 0, &V21724_293_57188, &P21724_292};	/*Property = CssOverflow*/
const Value V21724_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_294 = {S16, 0, &V21724_294_57191, &P21724_293};	/*Property = CssPosition*/
const Value V21724_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_295 = {S17, 0, &V21724_295_57194, &P21724_294};	/*Property = CssText*/
const Value V21724_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_297 = {S18, 0, &V21724_297_57197, &P21724_295};	/*Property = CssTransforms*/
const Value V21724_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_241 = {S19, 0, &V21724_241_55801, &P21724_297};	/*Property = CssTransitions*/
const Value V21724_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_298 = {S20, 0, &V21724_298_57208, &P21724_241};	/*Property = CssUI*/
const Value V21724_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_300 = {S21, 0, &V21724_300_57217, &P21724_298};	/*Property = DataSet*/
const Value V21724_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_301 = {S22, 0, &V21724_301_57220, &P21724_300};	/*Property = DataUrl*/
const Value V21724_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_302 = {S23, 0, &V21724_302_57223, &P21724_301};	/*Property = DeviceOrientation*/
const Value V21724_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_304 = {S24, 0, &V21724_304_57229, &P21724_302};	/*Property = FileReader*/
const Value V21724_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_305 = {S25, 0, &V21724_305_57232, &P21724_304};	/*Property = FileSaver*/
const Value V21724_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_306 = {S26, 0, &V21724_306_57238, &P21724_305};	/*Property = FileWriter*/
const Value V21724_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_307 = {S27, 0, &V21724_307_57241, &P21724_306};	/*Property = FormData*/
const Value V21724_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_309 = {S28, 0, &V21724_309_57247, &P21724_307};	/*Property = Fullscreen*/
const Value V21724_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_310 = {S29, 0, &V21724_310_57250, &P21724_309};	/*Property = GeoLocation*/
const Value V21724_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_312 = {S30, 0, &V21724_312_57256, &P21724_310};	/*Property = History*/
const Value V21724_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_314 = {S31, 0, &V21724_314_57262, &P21724_312};	/*Property = Html5*/
const Value V21724_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_313 = {S32, 0, &V21724_313_57259, &P21724_314};	/*Property = Html-Media-Capture*/
const Value V21724_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_315 = {S33, 0, &V21724_315_57265, &P21724_313};	/*Property = Iframe*/
const Value V21724_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_316 = {S34, 0, &V21724_316_57268, &P21724_315};	/*Property = IndexedDB*/
const Value V21724_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_317 = {S35, 0, &V21724_317_57271, &P21724_316};	/*Property = Json*/
const Value V21724_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21724_15 = {S37, 0, &V21724_15_54167, &P21724_317};	/*Property = LayoutEngine*/
const Value V21724_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_344 = {S38, 0, &V21724_344_57274, &P21724_15};	/*Property = Masking*/
const Value V21724_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_320 = {S39, 0, &V21724_320_57292, &P21724_344};	/*Property = PostMessage*/
const Value V21724_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_321 = {S40, 0, &V21724_321_57295, &P21724_320};	/*Property = Progress*/
const Value V21724_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_322 = {S41, 0, &V21724_322_57298, &P21724_321};	/*Property = Prompts*/
const Value V21724_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_323 = {S42, 0, &V21724_323_57301, &P21724_322};	/*Property = Selector*/
const Value V21724_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_324 = {S43, 0, &V21724_324_57307, &P21724_323};	/*Property = Svg*/
const Value V21724_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_327 = {S44, 0, &V21724_327_57316, &P21724_324};	/*Property = TouchEvents*/
const Value V21724_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_328 = {S45, 0, &V21724_328_57319, &P21724_327};	/*Property = Track*/
const Value V21724_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_329 = {S46, 0, &V21724_329_57325, &P21724_328};	/*Property = Video*/
const Value V21724_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_330 = {S47, 0, &V21724_330_57328, &P21724_329};	/*Property = Viewport*/
const Value V21724_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_332 = {S48, 0, &V21724_332_57340, &P21724_330};	/*Property = WebWorkers*/
const Value V21724_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21724_333 = {S49, 0, &V21724_333_57343, &P21724_332};	/*Property = Xhr2*/
const Profile P21724 = {"21724", &P21724_333};	/*Profile ID - 21724*/
const Value V20403_1_1 = {S2, NULL};	/*Value = True*/
const Property P20403_1 = {S50, 0, &V20403_1_1, NULL};	/*Property = IsMobile*/
const Value V20403_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20403_5 = {S52, 0, &V20403_5_1222, &P20403_1};	/*Property = ScreenPixelsHeight*/
const Value V20403_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20403_4 = {S54, 0, &V20403_4_1224, &P20403_5};	/*Property = ScreenPixelsWidth*/
const Profile P20403 = {"20403", &P20403_4};	/*Profile ID - 20403*/
const Value V20082_1_2 = {S0, NULL};	/*Value = False*/
const Property P20082_1 = {S50, 0, &V20082_1_2, NULL};	/*Property = IsMobile*/
const Value V20082_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20082_5 = {S52, 0, &V20082_5_37, &P20082_1};	/*Property = ScreenPixelsHeight*/
const Value V20082_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20082_4 = {S54, 0, &V20082_4_36, &P20082_5};	/*Property = ScreenPixelsWidth*/
const Profile P20082 = {"20082", &P20082_4};	/*Profile ID - 20082*/
const Value V21958_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21958_279 = {S1, 0, &V21958_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21958_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21958_282 = {S3, 0, &V21958_282_57154, &P21958_279};	/*Property = BlobBuilder*/
const Value V21958_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21958_350 = {S4, 0, &V21958_350_57444, &P21958_282};	/*Property = Canvas*/
const Value V21958_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21958_283 = {S5, 0, &V21958_283_57159, &P21958_350};	/*Property = CssBackground*/
const Value V21958_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21958_284 = {S6, 0, &V21958_284_57163, &P21958_283};	/*Property = CssBorderImage*/
const Value V21958_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21958_285 = {S7, 0, &V21958_285_57166, &P21958_284};	/*Property = CssCanvas*/
const Value V21958_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21958_286 = {S8, 0, &V21958_286_57168, &P21958_285};	/*Property = CssColor*/
const Value V21958_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21958_237 = {S9, 0, &V21958_237_55784, &P21958_286};	/*Property = CssColumn*/
const Value V21958_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21958_287 = {S10, 0, &V21958_287_57172, &P21958_237};	/*Property = CssFlexbox*/
const Value V21958_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21958_288 = {S11, 0, &V21958_288_57175, &P21958_287};	/*Property = CssFont*/
const Value V21958_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21958_289 = {S12, 0, &V21958_289_57759, &P21958_288};	/*Property = CssImages*/
const Value V21958_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21958_291 = {S13, 0, &V21958_291_57181, &P21958_289};	/*Property = CssMediaQueries*/
const Value V21958_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21958_292 = {S14, 0, &V21958_292_57183, &P21958_291};	/*Property = CssMinMax*/
const Value V21958_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21958_293 = {S15, 0, &V21958_293_57187, &P21958_292};	/*Property = CssOverflow*/
const Value V21958_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21958_294 = {S16, 0, &V21958_294_57190, &P21958_293};	/*Property = CssPosition*/
const Value V21958_295_57193 = {S0, NULL};	/*Value = False*/
const Property P21958_295 = {S17, 0, &V21958_295_57193, &P21958_294};	/*Property = CssText*/
const Value V21958_297_57196 = {S0, NULL};	/*Value = False*/
const Property P21958_297 = {S18, 0, &V21958_297_57196, &P21958_295};	/*Property = CssTransforms*/
const Value V21958_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21958_241 = {S19, 0, &V21958_241_55799, &P21958_297};	/*Property = CssTransitions*/
const Value V21958_298_57207 = {S0, NULL};	/*Value = False*/
const Property P21958_298 = {S20, 0, &V21958_298_57207, &P21958_241};	/*Property = CssUI*/
const Value V21958_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21958_300 = {S21, 0, &V21958_300_57216, &P21958_298};	/*Property = DataSet*/
const Value V21958_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21958_301 = {S22, 0, &V21958_301_57218, &P21958_300};	/*Property = DataUrl*/
const Value V21958_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21958_302 = {S23, 0, &V21958_302_57222, &P21958_301};	/*Property = DeviceOrientation*/
const Value V21958_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21958_304 = {S24, 0, &V21958_304_57228, &P21958_302};	/*Property = FileReader*/
const Value V21958_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21958_305 = {S25, 0, &V21958_305_57231, &P21958_304};	/*Property = FileSaver*/
const Value V21958_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21958_306 = {S26, 0, &V21958_306_57237, &P21958_305};	/*Property = FileWriter*/
const Value V21958_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21958_307 = {S27, 0, &V21958_307_57240, &P21958_306};	/*Property = FormData*/
const Value V21958_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21958_309 = {S28, 0, &V21958_309_57246, &P21958_307};	/*Property = Fullscreen*/
const Value V21958_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21958_310 = {S29, 0, &V21958_310_57248, &P21958_309};	/*Property = GeoLocation*/
const Value V21958_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21958_312 = {S30, 0, &V21958_312_57255, &P21958_310};	/*Property = History*/
const Value V21958_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21958_314 = {S31, 0, &V21958_314_57261, &P21958_312};	/*Property = Html5*/
const Value V21958_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21958_313 = {S32, 0, &V21958_313_57258, &P21958_314};	/*Property = Html-Media-Capture*/
const Value V21958_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21958_315 = {S33, 0, &V21958_315_57264, &P21958_313};	/*Property = Iframe*/
const Value V21958_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21958_316 = {S34, 0, &V21958_316_57267, &P21958_315};	/*Property = IndexedDB*/
const Value V21958_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21958_317 = {S35, 0, &V21958_317_57269, &P21958_316};	/*Property = Json*/
const Value V21958_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P21958_15 = {S37, 0, &V21958_15_29, &P21958_317};	/*Property = LayoutEngine*/
const Value V21958_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21958_344 = {S38, 0, &V21958_344_57272, &P21958_15};	/*Property = Masking*/
const Value V21958_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21958_320 = {S39, 0, &V21958_320_57290, &P21958_344};	/*Property = PostMessage*/
const Value V21958_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21958_321 = {S40, 0, &V21958_321_57293, &P21958_320};	/*Property = Progress*/
const Value V21958_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21958_322 = {S41, 0, &V21958_322_57296, &P21958_321};	/*Property = Prompts*/
const Value V21958_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21958_323 = {S42, 0, &V21958_323_57299, &P21958_322};	/*Property = Selector*/
const Value V21958_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21958_324 = {S43, 0, &V21958_324_57306, &P21958_323};	/*Property = Svg*/
const Value V21958_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21958_327 = {S44, 0, &V21958_327_57315, &P21958_324};	/*Property = TouchEvents*/
const Value V21958_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21958_328 = {S45, 0, &V21958_328_57318, &P21958_327};	/*Property = Track*/
const Value V21958_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21958_329 = {S46, 0, &V21958_329_57323, &P21958_328};	/*Property = Video*/
const Value V21958_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21958_330 = {S47, 0, &V21958_330_57326, &P21958_329};	/*Property = Viewport*/
const Value V21958_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21958_332 = {S48, 0, &V21958_332_57339, &P21958_330};	/*Property = WebWorkers*/
const Value V21958_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21958_333 = {S49, 0, &V21958_333_57342, &P21958_332};	/*Property = Xhr2*/
const Profile P21958 = {"21958", &P21958_333};	/*Profile ID - 21958*/
const Profile P22204 = {"22204", NULL};	/*Profile ID - 22204*/
const Value V21957_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21957_279 = {S1, 0, &V21957_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21957_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21957_282 = {S3, 0, &V21957_282_57154, &P21957_279};	/*Property = BlobBuilder*/
const Value V21957_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21957_350 = {S4, 0, &V21957_350_57444, &P21957_282};	/*Property = Canvas*/
const Value V21957_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21957_283 = {S5, 0, &V21957_283_57159, &P21957_350};	/*Property = CssBackground*/
const Value V21957_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21957_284 = {S6, 0, &V21957_284_57163, &P21957_283};	/*Property = CssBorderImage*/
const Value V21957_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21957_285 = {S7, 0, &V21957_285_57166, &P21957_284};	/*Property = CssCanvas*/
const Value V21957_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21957_286 = {S8, 0, &V21957_286_57168, &P21957_285};	/*Property = CssColor*/
const Value V21957_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21957_237 = {S9, 0, &V21957_237_55784, &P21957_286};	/*Property = CssColumn*/
const Value V21957_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21957_287 = {S10, 0, &V21957_287_57172, &P21957_237};	/*Property = CssFlexbox*/
const Value V21957_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21957_288 = {S11, 0, &V21957_288_57175, &P21957_287};	/*Property = CssFont*/
const Value V21957_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21957_289 = {S12, 0, &V21957_289_57759, &P21957_288};	/*Property = CssImages*/
const Value V21957_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21957_291 = {S13, 0, &V21957_291_57181, &P21957_289};	/*Property = CssMediaQueries*/
const Value V21957_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21957_292 = {S14, 0, &V21957_292_57183, &P21957_291};	/*Property = CssMinMax*/
const Value V21957_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21957_293 = {S15, 0, &V21957_293_57187, &P21957_292};	/*Property = CssOverflow*/
const Value V21957_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21957_294 = {S16, 0, &V21957_294_57190, &P21957_293};	/*Property = CssPosition*/
const Value V21957_295_57193 = {S0, NULL};	/*Value = False*/
const Property P21957_295 = {S17, 0, &V21957_295_57193, &P21957_294};	/*Property = CssText*/
const Value V21957_297_57196 = {S0, NULL};	/*Value = False*/
const Property P21957_297 = {S18, 0, &V21957_297_57196, &P21957_295};	/*Property = CssTransforms*/
const Value V21957_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21957_241 = {S19, 0, &V21957_241_55799, &P21957_297};	/*Property = CssTransitions*/
const Value V21957_298_57207 = {S0, NULL};	/*Value = False*/
const Property P21957_298 = {S20, 0, &V21957_298_57207, &P21957_241};	/*Property = CssUI*/
const Value V21957_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21957_300 = {S21, 0, &V21957_300_57216, &P21957_298};	/*Property = DataSet*/
const Value V21957_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21957_301 = {S22, 0, &V21957_301_57218, &P21957_300};	/*Property = DataUrl*/
const Value V21957_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21957_302 = {S23, 0, &V21957_302_57222, &P21957_301};	/*Property = DeviceOrientation*/
const Value V21957_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21957_304 = {S24, 0, &V21957_304_57228, &P21957_302};	/*Property = FileReader*/
const Value V21957_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21957_305 = {S25, 0, &V21957_305_57231, &P21957_304};	/*Property = FileSaver*/
const Value V21957_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21957_306 = {S26, 0, &V21957_306_57237, &P21957_305};	/*Property = FileWriter*/
const Value V21957_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21957_307 = {S27, 0, &V21957_307_57240, &P21957_306};	/*Property = FormData*/
const Value V21957_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21957_309 = {S28, 0, &V21957_309_57246, &P21957_307};	/*Property = Fullscreen*/
const Value V21957_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21957_310 = {S29, 0, &V21957_310_57248, &P21957_309};	/*Property = GeoLocation*/
const Value V21957_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21957_312 = {S30, 0, &V21957_312_57255, &P21957_310};	/*Property = History*/
const Value V21957_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21957_314 = {S31, 0, &V21957_314_57261, &P21957_312};	/*Property = Html5*/
const Value V21957_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21957_313 = {S32, 0, &V21957_313_57258, &P21957_314};	/*Property = Html-Media-Capture*/
const Value V21957_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21957_315 = {S33, 0, &V21957_315_57264, &P21957_313};	/*Property = Iframe*/
const Value V21957_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21957_316 = {S34, 0, &V21957_316_57267, &P21957_315};	/*Property = IndexedDB*/
const Value V21957_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21957_317 = {S35, 0, &V21957_317_57269, &P21957_316};	/*Property = Json*/
const Value V21957_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P21957_15 = {S37, 0, &V21957_15_29, &P21957_317};	/*Property = LayoutEngine*/
const Value V21957_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21957_344 = {S38, 0, &V21957_344_57272, &P21957_15};	/*Property = Masking*/
const Value V21957_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21957_320 = {S39, 0, &V21957_320_57290, &P21957_344};	/*Property = PostMessage*/
const Value V21957_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21957_321 = {S40, 0, &V21957_321_57293, &P21957_320};	/*Property = Progress*/
const Value V21957_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21957_322 = {S41, 0, &V21957_322_57296, &P21957_321};	/*Property = Prompts*/
const Value V21957_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21957_323 = {S42, 0, &V21957_323_57299, &P21957_322};	/*Property = Selector*/
const Value V21957_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21957_324 = {S43, 0, &V21957_324_57306, &P21957_323};	/*Property = Svg*/
const Value V21957_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21957_327 = {S44, 0, &V21957_327_57315, &P21957_324};	/*Property = TouchEvents*/
const Value V21957_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21957_328 = {S45, 0, &V21957_328_57318, &P21957_327};	/*Property = Track*/
const Value V21957_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21957_329 = {S46, 0, &V21957_329_57323, &P21957_328};	/*Property = Video*/
const Value V21957_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21957_330 = {S47, 0, &V21957_330_57326, &P21957_329};	/*Property = Viewport*/
const Value V21957_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21957_332 = {S48, 0, &V21957_332_57339, &P21957_330};	/*Property = WebWorkers*/
const Value V21957_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21957_333 = {S49, 0, &V21957_333_57342, &P21957_332};	/*Property = Xhr2*/
const Profile P21957 = {"21957", &P21957_333};	/*Profile ID - 21957*/
const Value V20148_1_1 = {S2, NULL};	/*Value = True*/
const Property P20148_1 = {S50, 0, &V20148_1_1, NULL};	/*Property = IsMobile*/
const Value V20148_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20148_5 = {S52, 0, &V20148_5_1227, &P20148_1};	/*Property = ScreenPixelsHeight*/
const Value V20148_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20148_4 = {S54, 0, &V20148_4_1229, &P20148_5};	/*Property = ScreenPixelsWidth*/
const Profile P20148 = {"20148", &P20148_4};	/*Profile ID - 20148*/
const Value V22508_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_279 = {S1, 0, &V22508_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22508_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_282 = {S3, 0, &V22508_282_57155, &P22508_279};	/*Property = BlobBuilder*/
const Value V22508_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_350 = {S4, 0, &V22508_350_57446, &P22508_282};	/*Property = Canvas*/
const Value V22508_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_283 = {S5, 0, &V22508_283_57161, &P22508_350};	/*Property = CssBackground*/
const Value V22508_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_284 = {S6, 0, &V22508_284_57164, &P22508_283};	/*Property = CssBorderImage*/
const Value V22508_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_285 = {S7, 0, &V22508_285_57167, &P22508_284};	/*Property = CssCanvas*/
const Value V22508_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_286 = {S8, 0, &V22508_286_57170, &P22508_285};	/*Property = CssColor*/
const Value V22508_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_237 = {S9, 0, &V22508_237_55786, &P22508_286};	/*Property = CssColumn*/
const Value V22508_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_287 = {S10, 0, &V22508_287_57173, &P22508_237};	/*Property = CssFlexbox*/
const Value V22508_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_288 = {S11, 0, &V22508_288_57176, &P22508_287};	/*Property = CssFont*/
const Value V22508_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_289 = {S12, 0, &V22508_289_86899, &P22508_288};	/*Property = CssImages*/
const Value V22508_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_291 = {S13, 0, &V22508_291_57182, &P22508_289};	/*Property = CssMediaQueries*/
const Value V22508_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_292 = {S14, 0, &V22508_292_57185, &P22508_291};	/*Property = CssMinMax*/
const Value V22508_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_293 = {S15, 0, &V22508_293_57188, &P22508_292};	/*Property = CssOverflow*/
const Value V22508_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_294 = {S16, 0, &V22508_294_57191, &P22508_293};	/*Property = CssPosition*/
const Value V22508_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_295 = {S17, 0, &V22508_295_57194, &P22508_294};	/*Property = CssText*/
const Value V22508_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_297 = {S18, 0, &V22508_297_57197, &P22508_295};	/*Property = CssTransforms*/
const Value V22508_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_241 = {S19, 0, &V22508_241_55801, &P22508_297};	/*Property = CssTransitions*/
const Value V22508_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_298 = {S20, 0, &V22508_298_57208, &P22508_241};	/*Property = CssUI*/
const Value V22508_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_300 = {S21, 0, &V22508_300_57217, &P22508_298};	/*Property = DataSet*/
const Value V22508_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_301 = {S22, 0, &V22508_301_57220, &P22508_300};	/*Property = DataUrl*/
const Value V22508_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_302 = {S23, 0, &V22508_302_57223, &P22508_301};	/*Property = DeviceOrientation*/
const Value V22508_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_304 = {S24, 0, &V22508_304_57229, &P22508_302};	/*Property = FileReader*/
const Value V22508_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_305 = {S25, 0, &V22508_305_57232, &P22508_304};	/*Property = FileSaver*/
const Value V22508_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_306 = {S26, 0, &V22508_306_57238, &P22508_305};	/*Property = FileWriter*/
const Value V22508_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_307 = {S27, 0, &V22508_307_57241, &P22508_306};	/*Property = FormData*/
const Value V22508_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_309 = {S28, 0, &V22508_309_57247, &P22508_307};	/*Property = Fullscreen*/
const Value V22508_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_310 = {S29, 0, &V22508_310_57250, &P22508_309};	/*Property = GeoLocation*/
const Value V22508_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_312 = {S30, 0, &V22508_312_57256, &P22508_310};	/*Property = History*/
const Value V22508_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_314 = {S31, 0, &V22508_314_57262, &P22508_312};	/*Property = Html5*/
const Value V22508_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_313 = {S32, 0, &V22508_313_57259, &P22508_314};	/*Property = Html-Media-Capture*/
const Value V22508_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_315 = {S33, 0, &V22508_315_57265, &P22508_313};	/*Property = Iframe*/
const Value V22508_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_316 = {S34, 0, &V22508_316_57268, &P22508_315};	/*Property = IndexedDB*/
const Value V22508_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_317 = {S35, 0, &V22508_317_57271, &P22508_316};	/*Property = Json*/
const Value V22508_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22508_15 = {S37, 0, &V22508_15_29, &P22508_317};	/*Property = LayoutEngine*/
const Value V22508_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_344 = {S38, 0, &V22508_344_57274, &P22508_15};	/*Property = Masking*/
const Value V22508_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_320 = {S39, 0, &V22508_320_57292, &P22508_344};	/*Property = PostMessage*/
const Value V22508_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_321 = {S40, 0, &V22508_321_57295, &P22508_320};	/*Property = Progress*/
const Value V22508_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_322 = {S41, 0, &V22508_322_57298, &P22508_321};	/*Property = Prompts*/
const Value V22508_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_323 = {S42, 0, &V22508_323_57301, &P22508_322};	/*Property = Selector*/
const Value V22508_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_324 = {S43, 0, &V22508_324_57307, &P22508_323};	/*Property = Svg*/
const Value V22508_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_327 = {S44, 0, &V22508_327_57316, &P22508_324};	/*Property = TouchEvents*/
const Value V22508_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_328 = {S45, 0, &V22508_328_57319, &P22508_327};	/*Property = Track*/
const Value V22508_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_329 = {S46, 0, &V22508_329_57325, &P22508_328};	/*Property = Video*/
const Value V22508_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_330 = {S47, 0, &V22508_330_57328, &P22508_329};	/*Property = Viewport*/
const Value V22508_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_332 = {S48, 0, &V22508_332_57340, &P22508_330};	/*Property = WebWorkers*/
const Value V22508_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22508_333 = {S49, 0, &V22508_333_57343, &P22508_332};	/*Property = Xhr2*/
const Profile P22508 = {"22508", &P22508_333};	/*Profile ID - 22508*/
const Value V24749_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24749_15 = {S37, 0, &V24749_15_28, NULL};	/*Property = LayoutEngine*/
const Profile P24749 = {"24749", &P24749_15};	/*Profile ID - 24749*/
const Value V24748_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24748_15 = {S37, 0, &V24748_15_28, NULL};	/*Property = LayoutEngine*/
const Profile P24748 = {"24748", &P24748_15};	/*Profile ID - 24748*/
const Value V23734_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_279 = {S1, 0, &V23734_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23734_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_282 = {S3, 0, &V23734_282_57155, &P23734_279};	/*Property = BlobBuilder*/
const Value V23734_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_350 = {S4, 0, &V23734_350_57446, &P23734_282};	/*Property = Canvas*/
const Value V23734_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_283 = {S5, 0, &V23734_283_57161, &P23734_350};	/*Property = CssBackground*/
const Value V23734_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_284 = {S6, 0, &V23734_284_57164, &P23734_283};	/*Property = CssBorderImage*/
const Value V23734_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_285 = {S7, 0, &V23734_285_57167, &P23734_284};	/*Property = CssCanvas*/
const Value V23734_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_286 = {S8, 0, &V23734_286_57170, &P23734_285};	/*Property = CssColor*/
const Value V23734_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_237 = {S9, 0, &V23734_237_55786, &P23734_286};	/*Property = CssColumn*/
const Value V23734_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_287 = {S10, 0, &V23734_287_57173, &P23734_237};	/*Property = CssFlexbox*/
const Value V23734_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_288 = {S11, 0, &V23734_288_57176, &P23734_287};	/*Property = CssFont*/
const Value V23734_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_289 = {S12, 0, &V23734_289_86899, &P23734_288};	/*Property = CssImages*/
const Value V23734_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_291 = {S13, 0, &V23734_291_57182, &P23734_289};	/*Property = CssMediaQueries*/
const Value V23734_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_292 = {S14, 0, &V23734_292_57185, &P23734_291};	/*Property = CssMinMax*/
const Value V23734_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_293 = {S15, 0, &V23734_293_57188, &P23734_292};	/*Property = CssOverflow*/
const Value V23734_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_294 = {S16, 0, &V23734_294_57191, &P23734_293};	/*Property = CssPosition*/
const Value V23734_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_295 = {S17, 0, &V23734_295_57194, &P23734_294};	/*Property = CssText*/
const Value V23734_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_297 = {S18, 0, &V23734_297_57197, &P23734_295};	/*Property = CssTransforms*/
const Value V23734_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_241 = {S19, 0, &V23734_241_55801, &P23734_297};	/*Property = CssTransitions*/
const Value V23734_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_298 = {S20, 0, &V23734_298_57208, &P23734_241};	/*Property = CssUI*/
const Value V23734_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_300 = {S21, 0, &V23734_300_57217, &P23734_298};	/*Property = DataSet*/
const Value V23734_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_301 = {S22, 0, &V23734_301_57220, &P23734_300};	/*Property = DataUrl*/
const Value V23734_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_302 = {S23, 0, &V23734_302_57223, &P23734_301};	/*Property = DeviceOrientation*/
const Value V23734_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_304 = {S24, 0, &V23734_304_57229, &P23734_302};	/*Property = FileReader*/
const Value V23734_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_305 = {S25, 0, &V23734_305_57232, &P23734_304};	/*Property = FileSaver*/
const Value V23734_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_306 = {S26, 0, &V23734_306_57238, &P23734_305};	/*Property = FileWriter*/
const Value V23734_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_307 = {S27, 0, &V23734_307_57241, &P23734_306};	/*Property = FormData*/
const Value V23734_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_309 = {S28, 0, &V23734_309_57247, &P23734_307};	/*Property = Fullscreen*/
const Value V23734_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_310 = {S29, 0, &V23734_310_57250, &P23734_309};	/*Property = GeoLocation*/
const Value V23734_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_312 = {S30, 0, &V23734_312_57256, &P23734_310};	/*Property = History*/
const Value V23734_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_314 = {S31, 0, &V23734_314_57262, &P23734_312};	/*Property = Html5*/
const Value V23734_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_313 = {S32, 0, &V23734_313_57259, &P23734_314};	/*Property = Html-Media-Capture*/
const Value V23734_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_315 = {S33, 0, &V23734_315_57265, &P23734_313};	/*Property = Iframe*/
const Value V23734_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_316 = {S34, 0, &V23734_316_57268, &P23734_315};	/*Property = IndexedDB*/
const Value V23734_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_317 = {S35, 0, &V23734_317_57271, &P23734_316};	/*Property = Json*/
const Value V23734_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P23734_15 = {S37, 0, &V23734_15_28, &P23734_317};	/*Property = LayoutEngine*/
const Value V23734_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_344 = {S38, 0, &V23734_344_57274, &P23734_15};	/*Property = Masking*/
const Value V23734_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_320 = {S39, 0, &V23734_320_57292, &P23734_344};	/*Property = PostMessage*/
const Value V23734_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_321 = {S40, 0, &V23734_321_57295, &P23734_320};	/*Property = Progress*/
const Value V23734_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_322 = {S41, 0, &V23734_322_57298, &P23734_321};	/*Property = Prompts*/
const Value V23734_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_323 = {S42, 0, &V23734_323_57301, &P23734_322};	/*Property = Selector*/
const Value V23734_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_324 = {S43, 0, &V23734_324_57307, &P23734_323};	/*Property = Svg*/
const Value V23734_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_327 = {S44, 0, &V23734_327_57316, &P23734_324};	/*Property = TouchEvents*/
const Value V23734_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_328 = {S45, 0, &V23734_328_57319, &P23734_327};	/*Property = Track*/
const Value V23734_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_329 = {S46, 0, &V23734_329_57325, &P23734_328};	/*Property = Video*/
const Value V23734_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_330 = {S47, 0, &V23734_330_57328, &P23734_329};	/*Property = Viewport*/
const Value V23734_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_332 = {S48, 0, &V23734_332_57340, &P23734_330};	/*Property = WebWorkers*/
const Value V23734_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23734_333 = {S49, 0, &V23734_333_57343, &P23734_332};	/*Property = Xhr2*/
const Profile P23734 = {"23734", &P23734_333};	/*Profile ID - 23734*/
const Value V19984_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_279 = {S1, 0, &V19984_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V19984_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_282 = {S3, 0, &V19984_282_57155, &P19984_279};	/*Property = BlobBuilder*/
const Value V19984_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_350 = {S4, 0, &V19984_350_57446, &P19984_282};	/*Property = Canvas*/
const Value V19984_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_283 = {S5, 0, &V19984_283_57161, &P19984_350};	/*Property = CssBackground*/
const Value V19984_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_284 = {S6, 0, &V19984_284_57164, &P19984_283};	/*Property = CssBorderImage*/
const Value V19984_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_285 = {S7, 0, &V19984_285_57167, &P19984_284};	/*Property = CssCanvas*/
const Value V19984_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_286 = {S8, 0, &V19984_286_57170, &P19984_285};	/*Property = CssColor*/
const Value V19984_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_237 = {S9, 0, &V19984_237_55786, &P19984_286};	/*Property = CssColumn*/
const Value V19984_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_287 = {S10, 0, &V19984_287_57173, &P19984_237};	/*Property = CssFlexbox*/
const Value V19984_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_288 = {S11, 0, &V19984_288_57176, &P19984_287};	/*Property = CssFont*/
const Value V19984_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_289 = {S12, 0, &V19984_289_86899, &P19984_288};	/*Property = CssImages*/
const Value V19984_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_291 = {S13, 0, &V19984_291_57182, &P19984_289};	/*Property = CssMediaQueries*/
const Value V19984_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_292 = {S14, 0, &V19984_292_57185, &P19984_291};	/*Property = CssMinMax*/
const Value V19984_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_293 = {S15, 0, &V19984_293_57188, &P19984_292};	/*Property = CssOverflow*/
const Value V19984_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_294 = {S16, 0, &V19984_294_57191, &P19984_293};	/*Property = CssPosition*/
const Value V19984_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_295 = {S17, 0, &V19984_295_57194, &P19984_294};	/*Property = CssText*/
const Value V19984_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_297 = {S18, 0, &V19984_297_57197, &P19984_295};	/*Property = CssTransforms*/
const Value V19984_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_241 = {S19, 0, &V19984_241_55801, &P19984_297};	/*Property = CssTransitions*/
const Value V19984_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_298 = {S20, 0, &V19984_298_57208, &P19984_241};	/*Property = CssUI*/
const Value V19984_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_300 = {S21, 0, &V19984_300_57217, &P19984_298};	/*Property = DataSet*/
const Value V19984_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_301 = {S22, 0, &V19984_301_57220, &P19984_300};	/*Property = DataUrl*/
const Value V19984_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_302 = {S23, 0, &V19984_302_57223, &P19984_301};	/*Property = DeviceOrientation*/
const Value V19984_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_304 = {S24, 0, &V19984_304_57229, &P19984_302};	/*Property = FileReader*/
const Value V19984_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_305 = {S25, 0, &V19984_305_57232, &P19984_304};	/*Property = FileSaver*/
const Value V19984_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_306 = {S26, 0, &V19984_306_57238, &P19984_305};	/*Property = FileWriter*/
const Value V19984_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_307 = {S27, 0, &V19984_307_57241, &P19984_306};	/*Property = FormData*/
const Value V19984_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_309 = {S28, 0, &V19984_309_57247, &P19984_307};	/*Property = Fullscreen*/
const Value V19984_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_310 = {S29, 0, &V19984_310_57250, &P19984_309};	/*Property = GeoLocation*/
const Value V19984_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_312 = {S30, 0, &V19984_312_57256, &P19984_310};	/*Property = History*/
const Value V19984_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_314 = {S31, 0, &V19984_314_57262, &P19984_312};	/*Property = Html5*/
const Value V19984_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_313 = {S32, 0, &V19984_313_57259, &P19984_314};	/*Property = Html-Media-Capture*/
const Value V19984_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_315 = {S33, 0, &V19984_315_57265, &P19984_313};	/*Property = Iframe*/
const Value V19984_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_316 = {S34, 0, &V19984_316_57268, &P19984_315};	/*Property = IndexedDB*/
const Value V19984_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_317 = {S35, 0, &V19984_317_57271, &P19984_316};	/*Property = Json*/
const Value V19984_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_15 = {S37, 0, &V19984_15_72, &P19984_317};	/*Property = LayoutEngine*/
const Value V19984_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_344 = {S38, 0, &V19984_344_57274, &P19984_15};	/*Property = Masking*/
const Value V19984_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_320 = {S39, 0, &V19984_320_57292, &P19984_344};	/*Property = PostMessage*/
const Value V19984_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_321 = {S40, 0, &V19984_321_57295, &P19984_320};	/*Property = Progress*/
const Value V19984_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_322 = {S41, 0, &V19984_322_57298, &P19984_321};	/*Property = Prompts*/
const Value V19984_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_323 = {S42, 0, &V19984_323_57301, &P19984_322};	/*Property = Selector*/
const Value V19984_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_324 = {S43, 0, &V19984_324_57307, &P19984_323};	/*Property = Svg*/
const Value V19984_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_327 = {S44, 0, &V19984_327_57316, &P19984_324};	/*Property = TouchEvents*/
const Value V19984_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_328 = {S45, 0, &V19984_328_57319, &P19984_327};	/*Property = Track*/
const Value V19984_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_329 = {S46, 0, &V19984_329_57325, &P19984_328};	/*Property = Video*/
const Value V19984_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_330 = {S47, 0, &V19984_330_57328, &P19984_329};	/*Property = Viewport*/
const Value V19984_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_332 = {S48, 0, &V19984_332_57340, &P19984_330};	/*Property = WebWorkers*/
const Value V19984_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P19984_333 = {S49, 0, &V19984_333_57343, &P19984_332};	/*Property = Xhr2*/
const Profile P19984 = {"19984", &P19984_333};	/*Profile ID - 19984*/
const Value V23663_1_2 = {S0, NULL};	/*Value = False*/
const Property P23663_1 = {S50, 0, &V23663_1_2, NULL};	/*Property = IsMobile*/
const Value V23663_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23663_5 = {S52, 0, &V23663_5_60417, &P23663_1};	/*Property = ScreenPixelsHeight*/
const Value V23663_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23663_4 = {S54, 0, &V23663_4_57095, &P23663_5};	/*Property = ScreenPixelsWidth*/
const Profile P23663 = {"23663", &P23663_4};	/*Profile ID - 23663*/
const Profile P23098 = {"23098", NULL};	/*Profile ID - 23098*/
const Value V23679_1_2 = {S0, NULL};	/*Value = False*/
const Property P23679_1 = {S50, 0, &V23679_1_2, NULL};	/*Property = IsMobile*/
const Value V23679_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23679_5 = {S52, 0, &V23679_5_60417, &P23679_1};	/*Property = ScreenPixelsHeight*/
const Value V23679_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23679_4 = {S54, 0, &V23679_4_57095, &P23679_5};	/*Property = ScreenPixelsWidth*/
const Profile P23679 = {"23679", &P23679_4};	/*Profile ID - 23679*/
const Profile P18138 = {"18138", NULL};	/*Profile ID - 18138*/
const Value V23732_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_279 = {S1, 0, &V23732_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23732_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_282 = {S3, 0, &V23732_282_57155, &P23732_279};	/*Property = BlobBuilder*/
const Value V23732_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_350 = {S4, 0, &V23732_350_57446, &P23732_282};	/*Property = Canvas*/
const Value V23732_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_283 = {S5, 0, &V23732_283_57161, &P23732_350};	/*Property = CssBackground*/
const Value V23732_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_284 = {S6, 0, &V23732_284_57164, &P23732_283};	/*Property = CssBorderImage*/
const Value V23732_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_285 = {S7, 0, &V23732_285_57167, &P23732_284};	/*Property = CssCanvas*/
const Value V23732_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_286 = {S8, 0, &V23732_286_57170, &P23732_285};	/*Property = CssColor*/
const Value V23732_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_237 = {S9, 0, &V23732_237_55786, &P23732_286};	/*Property = CssColumn*/
const Value V23732_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_287 = {S10, 0, &V23732_287_57173, &P23732_237};	/*Property = CssFlexbox*/
const Value V23732_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_288 = {S11, 0, &V23732_288_57176, &P23732_287};	/*Property = CssFont*/
const Value V23732_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_289 = {S12, 0, &V23732_289_86899, &P23732_288};	/*Property = CssImages*/
const Value V23732_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_291 = {S13, 0, &V23732_291_57182, &P23732_289};	/*Property = CssMediaQueries*/
const Value V23732_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_292 = {S14, 0, &V23732_292_57185, &P23732_291};	/*Property = CssMinMax*/
const Value V23732_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_293 = {S15, 0, &V23732_293_57188, &P23732_292};	/*Property = CssOverflow*/
const Value V23732_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_294 = {S16, 0, &V23732_294_57191, &P23732_293};	/*Property = CssPosition*/
const Value V23732_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_295 = {S17, 0, &V23732_295_57194, &P23732_294};	/*Property = CssText*/
const Value V23732_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_297 = {S18, 0, &V23732_297_57197, &P23732_295};	/*Property = CssTransforms*/
const Value V23732_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_241 = {S19, 0, &V23732_241_55801, &P23732_297};	/*Property = CssTransitions*/
const Value V23732_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_298 = {S20, 0, &V23732_298_57208, &P23732_241};	/*Property = CssUI*/
const Value V23732_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_300 = {S21, 0, &V23732_300_57217, &P23732_298};	/*Property = DataSet*/
const Value V23732_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_301 = {S22, 0, &V23732_301_57220, &P23732_300};	/*Property = DataUrl*/
const Value V23732_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_302 = {S23, 0, &V23732_302_57223, &P23732_301};	/*Property = DeviceOrientation*/
const Value V23732_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_304 = {S24, 0, &V23732_304_57229, &P23732_302};	/*Property = FileReader*/
const Value V23732_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_305 = {S25, 0, &V23732_305_57232, &P23732_304};	/*Property = FileSaver*/
const Value V23732_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_306 = {S26, 0, &V23732_306_57238, &P23732_305};	/*Property = FileWriter*/
const Value V23732_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_307 = {S27, 0, &V23732_307_57241, &P23732_306};	/*Property = FormData*/
const Value V23732_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_309 = {S28, 0, &V23732_309_57247, &P23732_307};	/*Property = Fullscreen*/
const Value V23732_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_310 = {S29, 0, &V23732_310_57250, &P23732_309};	/*Property = GeoLocation*/
const Value V23732_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_312 = {S30, 0, &V23732_312_57256, &P23732_310};	/*Property = History*/
const Value V23732_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_314 = {S31, 0, &V23732_314_57262, &P23732_312};	/*Property = Html5*/
const Value V23732_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_313 = {S32, 0, &V23732_313_57259, &P23732_314};	/*Property = Html-Media-Capture*/
const Value V23732_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_315 = {S33, 0, &V23732_315_57265, &P23732_313};	/*Property = Iframe*/
const Value V23732_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_316 = {S34, 0, &V23732_316_57268, &P23732_315};	/*Property = IndexedDB*/
const Value V23732_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_317 = {S35, 0, &V23732_317_57271, &P23732_316};	/*Property = Json*/
const Value V23732_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P23732_15 = {S37, 0, &V23732_15_28, &P23732_317};	/*Property = LayoutEngine*/
const Value V23732_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_344 = {S38, 0, &V23732_344_57274, &P23732_15};	/*Property = Masking*/
const Value V23732_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_320 = {S39, 0, &V23732_320_57292, &P23732_344};	/*Property = PostMessage*/
const Value V23732_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_321 = {S40, 0, &V23732_321_57295, &P23732_320};	/*Property = Progress*/
const Value V23732_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_322 = {S41, 0, &V23732_322_57298, &P23732_321};	/*Property = Prompts*/
const Value V23732_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_323 = {S42, 0, &V23732_323_57301, &P23732_322};	/*Property = Selector*/
const Value V23732_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_324 = {S43, 0, &V23732_324_57307, &P23732_323};	/*Property = Svg*/
const Value V23732_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_327 = {S44, 0, &V23732_327_57316, &P23732_324};	/*Property = TouchEvents*/
const Value V23732_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_328 = {S45, 0, &V23732_328_57319, &P23732_327};	/*Property = Track*/
const Value V23732_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_329 = {S46, 0, &V23732_329_57325, &P23732_328};	/*Property = Video*/
const Value V23732_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_330 = {S47, 0, &V23732_330_57328, &P23732_329};	/*Property = Viewport*/
const Value V23732_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_332 = {S48, 0, &V23732_332_57340, &P23732_330};	/*Property = WebWorkers*/
const Value V23732_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23732_333 = {S49, 0, &V23732_333_57343, &P23732_332};	/*Property = Xhr2*/
const Profile P23732 = {"23732", &P23732_333};	/*Profile ID - 23732*/
const Value V23652_1_2 = {S0, NULL};	/*Value = False*/
const Property P23652_1 = {S50, 0, &V23652_1_2, NULL};	/*Property = IsMobile*/
const Value V23652_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23652_5 = {S52, 0, &V23652_5_60417, &P23652_1};	/*Property = ScreenPixelsHeight*/
const Value V23652_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23652_4 = {S54, 0, &V23652_4_57095, &P23652_5};	/*Property = ScreenPixelsWidth*/
const Profile P23652 = {"23652", &P23652_4};	/*Profile ID - 23652*/
const Value V20975_1_1 = {S2, NULL};	/*Value = True*/
const Property P20975_1 = {S50, 0, &V20975_1_1, NULL};	/*Property = IsMobile*/
const Value V20975_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20975_5 = {S52, 0, &V20975_5_1227, &P20975_1};	/*Property = ScreenPixelsHeight*/
const Value V20975_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20975_4 = {S54, 0, &V20975_4_1229, &P20975_5};	/*Property = ScreenPixelsWidth*/
const Profile P20975 = {"20975", &P20975_4};	/*Profile ID - 20975*/
const Value V18735_1_1 = {S2, NULL};	/*Value = True*/
const Property P18735_1 = {S50, 0, &V18735_1_1, NULL};	/*Property = IsMobile*/
const Value V18735_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18735_5 = {S52, 0, &V18735_5_1277, &P18735_1};	/*Property = ScreenPixelsHeight*/
const Value V18735_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18735_4 = {S54, 0, &V18735_4_1279, &P18735_5};	/*Property = ScreenPixelsWidth*/
const Profile P18735 = {"18735", &P18735_4};	/*Profile ID - 18735*/
const Value V22801_1_1 = {S2, NULL};	/*Value = True*/
const Property P22801_1 = {S50, 0, &V22801_1_1, NULL};	/*Property = IsMobile*/
const Value V22801_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22801_5 = {S52, 0, &V22801_5_1222, &P22801_1};	/*Property = ScreenPixelsHeight*/
const Value V22801_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22801_4 = {S54, 0, &V22801_4_1224, &P22801_5};	/*Property = ScreenPixelsWidth*/
const Profile P22801 = {"22801", &P22801_4};	/*Profile ID - 22801*/
const Value V22977_1_1 = {S2, NULL};	/*Value = True*/
const Property P22977_1 = {S50, 0, &V22977_1_1, NULL};	/*Property = IsMobile*/
const Value V22977_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22977_5 = {S52, 0, &V22977_5_1227, &P22977_1};	/*Property = ScreenPixelsHeight*/
const Value V22977_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22977_4 = {S54, 0, &V22977_4_1229, &P22977_5};	/*Property = ScreenPixelsWidth*/
const Profile P22977 = {"22977", &P22977_4};	/*Profile ID - 22977*/
const Value V21620_1_1 = {S2, NULL};	/*Value = True*/
const Property P21620_1 = {S50, 0, &V21620_1_1, NULL};	/*Property = IsMobile*/
const Value V21620_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21620_5 = {S52, 0, &V21620_5_1277, &P21620_1};	/*Property = ScreenPixelsHeight*/
const Value V21620_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21620_4 = {S54, 0, &V21620_4_1279, &P21620_5};	/*Property = ScreenPixelsWidth*/
const Profile P21620 = {"21620", &P21620_4};	/*Profile ID - 21620*/
const Value V18594_1_1 = {S2, NULL};	/*Value = True*/
const Property P18594_1 = {S50, 0, &V18594_1_1, NULL};	/*Property = IsMobile*/
const Value V18594_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18594_5 = {S52, 0, &V18594_5_1227, &P18594_1};	/*Property = ScreenPixelsHeight*/
const Value V18594_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18594_4 = {S54, 0, &V18594_4_1229, &P18594_5};	/*Property = ScreenPixelsWidth*/
const Profile P18594 = {"18594", &P18594_4};	/*Profile ID - 18594*/
const Value V24548_1_1 = {S2, NULL};	/*Value = True*/
const Property P24548_1 = {S50, 0, &V24548_1_1, NULL};	/*Property = IsMobile*/
const Value V24548_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P24548_5 = {S52, 0, &V24548_5_37569, &P24548_1};	/*Property = ScreenPixelsHeight*/
const Value V24548_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P24548_4 = {S54, 0, &V24548_4_5645, &P24548_5};	/*Property = ScreenPixelsWidth*/
const Profile P24548 = {"24548", &P24548_4};	/*Profile ID - 24548*/
const Value V24015_1_1 = {S2, NULL};	/*Value = True*/
const Property P24015_1 = {S50, 0, &V24015_1_1, NULL};	/*Property = IsMobile*/
const Value V24015_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24015_5 = {S52, 0, &V24015_5_1277, &P24015_1};	/*Property = ScreenPixelsHeight*/
const Value V24015_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24015_4 = {S54, 0, &V24015_4_1279, &P24015_5};	/*Property = ScreenPixelsWidth*/
const Profile P24015 = {"24015", &P24015_4};	/*Profile ID - 24015*/
const Value V21387_1_1 = {S2, NULL};	/*Value = True*/
const Property P21387_1 = {S50, 0, &V21387_1_1, NULL};	/*Property = IsMobile*/
const Value V21387_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21387_5 = {S52, 0, &V21387_5_1277, &P21387_1};	/*Property = ScreenPixelsHeight*/
const Value V21387_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21387_4 = {S54, 0, &V21387_4_1279, &P21387_5};	/*Property = ScreenPixelsWidth*/
const Profile P21387 = {"21387", &P21387_4};	/*Profile ID - 21387*/
const Value V23430_1_1 = {S2, NULL};	/*Value = True*/
const Property P23430_1 = {S50, 0, &V23430_1_1, NULL};	/*Property = IsMobile*/
const Value V23430_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23430_5 = {S52, 0, &V23430_5_1277, &P23430_1};	/*Property = ScreenPixelsHeight*/
const Value V23430_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23430_4 = {S54, 0, &V23430_4_2336, &P23430_5};	/*Property = ScreenPixelsWidth*/
const Profile P23430 = {"23430", &P23430_4};	/*Profile ID - 23430*/
const Value V21344_1_1 = {S2, NULL};	/*Value = True*/
const Property P21344_1 = {S50, 0, &V21344_1_1, NULL};	/*Property = IsMobile*/
const Value V21344_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21344_5 = {S52, 0, &V21344_5_1458, &P21344_1};	/*Property = ScreenPixelsHeight*/
const Value V21344_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21344_4 = {S54, 0, &V21344_4_1460, &P21344_5};	/*Property = ScreenPixelsWidth*/
const Profile P21344 = {"21344", &P21344_4};	/*Profile ID - 21344*/
const Value V23936_1_1 = {S2, NULL};	/*Value = True*/
const Property P23936_1 = {S50, 0, &V23936_1_1, NULL};	/*Property = IsMobile*/
const Value V23936_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23936_5 = {S52, 0, &V23936_5_1227, &P23936_1};	/*Property = ScreenPixelsHeight*/
const Value V23936_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23936_4 = {S54, 0, &V23936_4_1229, &P23936_5};	/*Property = ScreenPixelsWidth*/
const Profile P23936 = {"23936", &P23936_4};	/*Profile ID - 23936*/
const Value V20121_1_1 = {S2, NULL};	/*Value = True*/
const Property P20121_1 = {S50, 0, &V20121_1_1, NULL};	/*Property = IsMobile*/
const Value V20121_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20121_5 = {S52, 0, &V20121_5_1227, &P20121_1};	/*Property = ScreenPixelsHeight*/
const Value V20121_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20121_4 = {S54, 0, &V20121_4_1229, &P20121_5};	/*Property = ScreenPixelsWidth*/
const Profile P20121 = {"20121", &P20121_4};	/*Profile ID - 20121*/
const Value V24022_1_1 = {S2, NULL};	/*Value = True*/
const Property P24022_1 = {S50, 0, &V24022_1_1, NULL};	/*Property = IsMobile*/
const Value V24022_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P24022_5 = {S52, 0, &V24022_5_1222, &P24022_1};	/*Property = ScreenPixelsHeight*/
const Value V24022_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P24022_4 = {S54, 0, &V24022_4_1224, &P24022_5};	/*Property = ScreenPixelsWidth*/
const Profile P24022 = {"24022", &P24022_4};	/*Profile ID - 24022*/
const Value V20924_1_1 = {S2, NULL};	/*Value = True*/
const Property P20924_1 = {S50, 0, &V20924_1_1, NULL};	/*Property = IsMobile*/
const Value V20924_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20924_5 = {S52, 0, &V20924_5_1227, &P20924_1};	/*Property = ScreenPixelsHeight*/
const Value V20924_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20924_4 = {S54, 0, &V20924_4_1229, &P20924_5};	/*Property = ScreenPixelsWidth*/
const Profile P20924 = {"20924", &P20924_4};	/*Profile ID - 20924*/
const Value V24023_1_1 = {S2, NULL};	/*Value = True*/
const Property P24023_1 = {S50, 0, &V24023_1_1, NULL};	/*Property = IsMobile*/
const Value V24023_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24023_5 = {S52, 0, &V24023_5_1277, &P24023_1};	/*Property = ScreenPixelsHeight*/
const Value V24023_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24023_4 = {S54, 0, &V24023_4_1279, &P24023_5};	/*Property = ScreenPixelsWidth*/
const Profile P24023 = {"24023", &P24023_4};	/*Profile ID - 24023*/
const Value V21336_1_1 = {S2, NULL};	/*Value = True*/
const Property P21336_1 = {S50, 0, &V21336_1_1, NULL};	/*Property = IsMobile*/
const Value V21336_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21336_5 = {S52, 0, &V21336_5_1227, &P21336_1};	/*Property = ScreenPixelsHeight*/
const Value V21336_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21336_4 = {S54, 0, &V21336_4_1229, &P21336_5};	/*Property = ScreenPixelsWidth*/
const Profile P21336 = {"21336", &P21336_4};	/*Profile ID - 21336*/
const Value V21435_1_1 = {S2, NULL};	/*Value = True*/
const Property P21435_1 = {S50, 0, &V21435_1_1, NULL};	/*Property = IsMobile*/
const Value V21435_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21435_5 = {S52, 0, &V21435_5_1277, &P21435_1};	/*Property = ScreenPixelsHeight*/
const Value V21435_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21435_4 = {S54, 0, &V21435_4_1279, &P21435_5};	/*Property = ScreenPixelsWidth*/
const Profile P21435 = {"21435", &P21435_4};	/*Profile ID - 21435*/
const Value V21765_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_279 = {S1, 0, &V21765_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21765_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_282 = {S3, 0, &V21765_282_57155, &P21765_279};	/*Property = BlobBuilder*/
const Value V21765_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_350 = {S4, 0, &V21765_350_57446, &P21765_282};	/*Property = Canvas*/
const Value V21765_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_283 = {S5, 0, &V21765_283_57161, &P21765_350};	/*Property = CssBackground*/
const Value V21765_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_284 = {S6, 0, &V21765_284_57164, &P21765_283};	/*Property = CssBorderImage*/
const Value V21765_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_285 = {S7, 0, &V21765_285_57167, &P21765_284};	/*Property = CssCanvas*/
const Value V21765_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_286 = {S8, 0, &V21765_286_57170, &P21765_285};	/*Property = CssColor*/
const Value V21765_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_237 = {S9, 0, &V21765_237_55786, &P21765_286};	/*Property = CssColumn*/
const Value V21765_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_287 = {S10, 0, &V21765_287_57173, &P21765_237};	/*Property = CssFlexbox*/
const Value V21765_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_288 = {S11, 0, &V21765_288_57176, &P21765_287};	/*Property = CssFont*/
const Value V21765_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_289 = {S12, 0, &V21765_289_86899, &P21765_288};	/*Property = CssImages*/
const Value V21765_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_291 = {S13, 0, &V21765_291_57182, &P21765_289};	/*Property = CssMediaQueries*/
const Value V21765_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_292 = {S14, 0, &V21765_292_57185, &P21765_291};	/*Property = CssMinMax*/
const Value V21765_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_293 = {S15, 0, &V21765_293_57188, &P21765_292};	/*Property = CssOverflow*/
const Value V21765_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_294 = {S16, 0, &V21765_294_57191, &P21765_293};	/*Property = CssPosition*/
const Value V21765_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_295 = {S17, 0, &V21765_295_57194, &P21765_294};	/*Property = CssText*/
const Value V21765_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_297 = {S18, 0, &V21765_297_57197, &P21765_295};	/*Property = CssTransforms*/
const Value V21765_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_241 = {S19, 0, &V21765_241_55801, &P21765_297};	/*Property = CssTransitions*/
const Value V21765_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_298 = {S20, 0, &V21765_298_57208, &P21765_241};	/*Property = CssUI*/
const Value V21765_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_300 = {S21, 0, &V21765_300_57217, &P21765_298};	/*Property = DataSet*/
const Value V21765_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_301 = {S22, 0, &V21765_301_57220, &P21765_300};	/*Property = DataUrl*/
const Value V21765_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_302 = {S23, 0, &V21765_302_57223, &P21765_301};	/*Property = DeviceOrientation*/
const Value V21765_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_304 = {S24, 0, &V21765_304_57229, &P21765_302};	/*Property = FileReader*/
const Value V21765_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_305 = {S25, 0, &V21765_305_57232, &P21765_304};	/*Property = FileSaver*/
const Value V21765_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_306 = {S26, 0, &V21765_306_57238, &P21765_305};	/*Property = FileWriter*/
const Value V21765_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_307 = {S27, 0, &V21765_307_57241, &P21765_306};	/*Property = FormData*/
const Value V21765_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_309 = {S28, 0, &V21765_309_57247, &P21765_307};	/*Property = Fullscreen*/
const Value V21765_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_310 = {S29, 0, &V21765_310_57250, &P21765_309};	/*Property = GeoLocation*/
const Value V21765_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_312 = {S30, 0, &V21765_312_57256, &P21765_310};	/*Property = History*/
const Value V21765_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_314 = {S31, 0, &V21765_314_57262, &P21765_312};	/*Property = Html5*/
const Value V21765_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_313 = {S32, 0, &V21765_313_57259, &P21765_314};	/*Property = Html-Media-Capture*/
const Value V21765_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_315 = {S33, 0, &V21765_315_57265, &P21765_313};	/*Property = Iframe*/
const Value V21765_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_316 = {S34, 0, &V21765_316_57268, &P21765_315};	/*Property = IndexedDB*/
const Value V21765_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_317 = {S35, 0, &V21765_317_57271, &P21765_316};	/*Property = Json*/
const Value V21765_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21765_15 = {S37, 0, &V21765_15_54167, &P21765_317};	/*Property = LayoutEngine*/
const Value V21765_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_344 = {S38, 0, &V21765_344_57274, &P21765_15};	/*Property = Masking*/
const Value V21765_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_320 = {S39, 0, &V21765_320_57292, &P21765_344};	/*Property = PostMessage*/
const Value V21765_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_321 = {S40, 0, &V21765_321_57295, &P21765_320};	/*Property = Progress*/
const Value V21765_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_322 = {S41, 0, &V21765_322_57298, &P21765_321};	/*Property = Prompts*/
const Value V21765_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_323 = {S42, 0, &V21765_323_57301, &P21765_322};	/*Property = Selector*/
const Value V21765_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_324 = {S43, 0, &V21765_324_57307, &P21765_323};	/*Property = Svg*/
const Value V21765_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_327 = {S44, 0, &V21765_327_57316, &P21765_324};	/*Property = TouchEvents*/
const Value V21765_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_328 = {S45, 0, &V21765_328_57319, &P21765_327};	/*Property = Track*/
const Value V21765_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_329 = {S46, 0, &V21765_329_57325, &P21765_328};	/*Property = Video*/
const Value V21765_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_330 = {S47, 0, &V21765_330_57328, &P21765_329};	/*Property = Viewport*/
const Value V21765_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_332 = {S48, 0, &V21765_332_57340, &P21765_330};	/*Property = WebWorkers*/
const Value V21765_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21765_333 = {S49, 0, &V21765_333_57343, &P21765_332};	/*Property = Xhr2*/
const Profile P21765 = {"21765", &P21765_333};	/*Profile ID - 21765*/
const Value V23983_1_1 = {S2, NULL};	/*Value = True*/
const Property P23983_1 = {S50, 0, &V23983_1_1, NULL};	/*Property = IsMobile*/
const Value V23983_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23983_5 = {S52, 0, &V23983_5_1277, &P23983_1};	/*Property = ScreenPixelsHeight*/
const Value V23983_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23983_4 = {S54, 0, &V23983_4_1279, &P23983_5};	/*Property = ScreenPixelsWidth*/
const Profile P23983 = {"23983", &P23983_4};	/*Profile ID - 23983*/
const Value V22501_1_1 = {S2, NULL};	/*Value = True*/
const Property P22501_1 = {S50, 0, &V22501_1_1, NULL};	/*Property = IsMobile*/
const Value V22501_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22501_5 = {S52, 0, &V22501_5_1277, &P22501_1};	/*Property = ScreenPixelsHeight*/
const Value V22501_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22501_4 = {S54, 0, &V22501_4_2336, &P22501_5};	/*Property = ScreenPixelsWidth*/
const Profile P22501 = {"22501", &P22501_4};	/*Profile ID - 22501*/
const Value V21016_1_1 = {S2, NULL};	/*Value = True*/
const Property P21016_1 = {S50, 0, &V21016_1_1, NULL};	/*Property = IsMobile*/
const Value V21016_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21016_5 = {S52, 0, &V21016_5_9981, &P21016_1};	/*Property = ScreenPixelsHeight*/
const Value V21016_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21016_4 = {S54, 0, &V21016_4_39098, &P21016_5};	/*Property = ScreenPixelsWidth*/
const Profile P21016 = {"21016", &P21016_4};	/*Profile ID - 21016*/
const Value V21480_1_1 = {S2, NULL};	/*Value = True*/
const Property P21480_1 = {S50, 0, &V21480_1_1, NULL};	/*Property = IsMobile*/
const Value V21480_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21480_5 = {S52, 0, &V21480_5_1227, &P21480_1};	/*Property = ScreenPixelsHeight*/
const Value V21480_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21480_4 = {S54, 0, &V21480_4_1229, &P21480_5};	/*Property = ScreenPixelsWidth*/
const Profile P21480 = {"21480", &P21480_4};	/*Profile ID - 21480*/
const Value V21463_1_1 = {S2, NULL};	/*Value = True*/
const Property P21463_1 = {S50, 0, &V21463_1_1, NULL};	/*Property = IsMobile*/
const Value V21463_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21463_5 = {S52, 0, &V21463_5_1277, &P21463_1};	/*Property = ScreenPixelsHeight*/
const Value V21463_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21463_4 = {S54, 0, &V21463_4_2336, &P21463_5};	/*Property = ScreenPixelsWidth*/
const Profile P21463 = {"21463", &P21463_4};	/*Profile ID - 21463*/
const Value V23974_1_1 = {S2, NULL};	/*Value = True*/
const Property P23974_1 = {S50, 0, &V23974_1_1, NULL};	/*Property = IsMobile*/
const Value V23974_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23974_5 = {S52, 0, &V23974_5_1227, &P23974_1};	/*Property = ScreenPixelsHeight*/
const Value V23974_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23974_4 = {S54, 0, &V23974_4_1229, &P23974_5};	/*Property = ScreenPixelsWidth*/
const Profile P23974 = {"23974", &P23974_4};	/*Profile ID - 23974*/
const Value V23943_1_1 = {S2, NULL};	/*Value = True*/
const Property P23943_1 = {S50, 0, &V23943_1_1, NULL};	/*Property = IsMobile*/
const Value V23943_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23943_5 = {S52, 0, &V23943_5_1277, &P23943_1};	/*Property = ScreenPixelsHeight*/
const Value V23943_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23943_4 = {S54, 0, &V23943_4_1279, &P23943_5};	/*Property = ScreenPixelsWidth*/
const Profile P23943 = {"23943", &P23943_4};	/*Profile ID - 23943*/
const Value V22141_1_1 = {S2, NULL};	/*Value = True*/
const Property P22141_1 = {S50, 0, &V22141_1_1, NULL};	/*Property = IsMobile*/
const Value V22141_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22141_5 = {S52, 0, &V22141_5_1227, &P22141_1};	/*Property = ScreenPixelsHeight*/
const Value V22141_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22141_4 = {S54, 0, &V22141_4_1229, &P22141_5};	/*Property = ScreenPixelsWidth*/
const Profile P22141 = {"22141", &P22141_4};	/*Profile ID - 22141*/
const Value V21286_1_1 = {S2, NULL};	/*Value = True*/
const Property P21286_1 = {S50, 0, &V21286_1_1, NULL};	/*Property = IsMobile*/
const Value V21286_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21286_5 = {S52, 0, &V21286_5_1458, &P21286_1};	/*Property = ScreenPixelsHeight*/
const Value V21286_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21286_4 = {S54, 0, &V21286_4_1460, &P21286_5};	/*Property = ScreenPixelsWidth*/
const Profile P21286 = {"21286", &P21286_4};	/*Profile ID - 21286*/
const Value V23942_1_1 = {S2, NULL};	/*Value = True*/
const Property P23942_1 = {S50, 0, &V23942_1_1, NULL};	/*Property = IsMobile*/
const Value V23942_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23942_5 = {S52, 0, &V23942_5_37569, &P23942_1};	/*Property = ScreenPixelsHeight*/
const Value V23942_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23942_4 = {S54, 0, &V23942_4_5645, &P23942_5};	/*Property = ScreenPixelsWidth*/
const Profile P23942 = {"23942", &P23942_4};	/*Profile ID - 23942*/
const Value V20379_1_1 = {S2, NULL};	/*Value = True*/
const Property P20379_1 = {S50, 0, &V20379_1_1, NULL};	/*Property = IsMobile*/
const Value V20379_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P20379_5 = {S52, 0, &V20379_5_37569, &P20379_1};	/*Property = ScreenPixelsHeight*/
const Value V20379_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P20379_4 = {S54, 0, &V20379_4_5645, &P20379_5};	/*Property = ScreenPixelsWidth*/
const Profile P20379 = {"20379", &P20379_4};	/*Profile ID - 20379*/
const Value V23930_1_1 = {S2, NULL};	/*Value = True*/
const Property P23930_1 = {S50, 0, &V23930_1_1, NULL};	/*Property = IsMobile*/
const Value V23930_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23930_5 = {S52, 0, &V23930_5_1222, &P23930_1};	/*Property = ScreenPixelsHeight*/
const Value V23930_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23930_4 = {S54, 0, &V23930_4_1224, &P23930_5};	/*Property = ScreenPixelsWidth*/
const Profile P23930 = {"23930", &P23930_4};	/*Profile ID - 23930*/
const Value V21619_1_1 = {S2, NULL};	/*Value = True*/
const Property P21619_1 = {S50, 0, &V21619_1_1, NULL};	/*Property = IsMobile*/
const Value V21619_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21619_5 = {S52, 0, &V21619_5_1277, &P21619_1};	/*Property = ScreenPixelsHeight*/
const Value V21619_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21619_4 = {S54, 0, &V21619_4_1279, &P21619_5};	/*Property = ScreenPixelsWidth*/
const Profile P21619 = {"21619", &P21619_4};	/*Profile ID - 21619*/
const Value V23917_1_1 = {S2, NULL};	/*Value = True*/
const Property P23917_1 = {S50, 0, &V23917_1_1, NULL};	/*Property = IsMobile*/
const Value V23917_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23917_5 = {S52, 0, &V23917_5_1227, &P23917_1};	/*Property = ScreenPixelsHeight*/
const Value V23917_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23917_4 = {S54, 0, &V23917_4_1229, &P23917_5};	/*Property = ScreenPixelsWidth*/
const Profile P23917 = {"23917", &P23917_4};	/*Profile ID - 23917*/
const Value V20017_1_1 = {S2, NULL};	/*Value = True*/
const Property P20017_1 = {S50, 0, &V20017_1_1, NULL};	/*Property = IsMobile*/
const Value V20017_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20017_5 = {S52, 0, &V20017_5_1227, &P20017_1};	/*Property = ScreenPixelsHeight*/
const Value V20017_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20017_4 = {S54, 0, &V20017_4_1229, &P20017_5};	/*Property = ScreenPixelsWidth*/
const Profile P20017 = {"20017", &P20017_4};	/*Profile ID - 20017*/
const Value V20343_1_1 = {S2, NULL};	/*Value = True*/
const Property P20343_1 = {S50, 0, &V20343_1_1, NULL};	/*Property = IsMobile*/
const Value V20343_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20343_5 = {S52, 0, &V20343_5_1403, &P20343_1};	/*Property = ScreenPixelsHeight*/
const Value V20343_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20343_4 = {S54, 0, &V20343_4_1404, &P20343_5};	/*Property = ScreenPixelsWidth*/
const Profile P20343 = {"20343", &P20343_4};	/*Profile ID - 20343*/
const Value V19898_1_1 = {S2, NULL};	/*Value = True*/
const Property P19898_1 = {S50, 0, &V19898_1_1, NULL};	/*Property = IsMobile*/
const Value V19898_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19898_5 = {S52, 0, &V19898_5_1227, &P19898_1};	/*Property = ScreenPixelsHeight*/
const Value V19898_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19898_4 = {S54, 0, &V19898_4_1229, &P19898_5};	/*Property = ScreenPixelsWidth*/
const Profile P19898 = {"19898", &P19898_4};	/*Profile ID - 19898*/
const Value V21598_1_1 = {S2, NULL};	/*Value = True*/
const Property P21598_1 = {S50, 0, &V21598_1_1, NULL};	/*Property = IsMobile*/
const Value V21598_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21598_5 = {S52, 0, &V21598_5_1403, &P21598_1};	/*Property = ScreenPixelsHeight*/
const Value V21598_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21598_4 = {S54, 0, &V21598_4_1404, &P21598_5};	/*Property = ScreenPixelsWidth*/
const Profile P21598 = {"21598", &P21598_4};	/*Profile ID - 21598*/
const Value V21732_1_1 = {S2, NULL};	/*Value = True*/
const Property P21732_1 = {S50, 0, &V21732_1_1, NULL};	/*Property = IsMobile*/
const Value V21732_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21732_5 = {S52, 0, &V21732_5_1277, &P21732_1};	/*Property = ScreenPixelsHeight*/
const Value V21732_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21732_4 = {S54, 0, &V21732_4_1279, &P21732_5};	/*Property = ScreenPixelsWidth*/
const Profile P21732 = {"21732", &P21732_4};	/*Profile ID - 21732*/
const Value V22063_1_1 = {S2, NULL};	/*Value = True*/
const Property P22063_1 = {S50, 0, &V22063_1_1, NULL};	/*Property = IsMobile*/
const Value V22063_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P22063_5 = {S52, 0, &V22063_5_37, &P22063_1};	/*Property = ScreenPixelsHeight*/
const Value V22063_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P22063_4 = {S54, 0, &V22063_4_36, &P22063_5};	/*Property = ScreenPixelsWidth*/
const Profile P22063 = {"22063", &P22063_4};	/*Profile ID - 22063*/
const Value V23087_1_1 = {S2, NULL};	/*Value = True*/
const Property P23087_1 = {S50, 0, &V23087_1_1, NULL};	/*Property = IsMobile*/
const Value V23087_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23087_5 = {S52, 0, &V23087_5_1277, &P23087_1};	/*Property = ScreenPixelsHeight*/
const Value V23087_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23087_4 = {S54, 0, &V23087_4_1279, &P23087_5};	/*Property = ScreenPixelsWidth*/
const Profile P23087 = {"23087", &P23087_4};	/*Profile ID - 23087*/
const Value V22858_1_1 = {S2, NULL};	/*Value = True*/
const Property P22858_1 = {S50, 0, &V22858_1_1, NULL};	/*Property = IsMobile*/
const Value V22858_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22858_5 = {S52, 0, &V22858_5_1403, &P22858_1};	/*Property = ScreenPixelsHeight*/
const Value V22858_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22858_4 = {S54, 0, &V22858_4_1404, &P22858_5};	/*Property = ScreenPixelsWidth*/
const Profile P22858 = {"22858", &P22858_4};	/*Profile ID - 22858*/
const Value V20014_1_1 = {S2, NULL};	/*Value = True*/
const Property P20014_1 = {S50, 0, &V20014_1_1, NULL};	/*Property = IsMobile*/
const Value V20014_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20014_5 = {S52, 0, &V20014_5_9981, &P20014_1};	/*Property = ScreenPixelsHeight*/
const Value V20014_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20014_4 = {S54, 0, &V20014_4_2336, &P20014_5};	/*Property = ScreenPixelsWidth*/
const Profile P20014 = {"20014", &P20014_4};	/*Profile ID - 20014*/
const Value V23907_1_1 = {S2, NULL};	/*Value = True*/
const Property P23907_1 = {S50, 0, &V23907_1_1, NULL};	/*Property = IsMobile*/
const Value V23907_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23907_5 = {S52, 0, &V23907_5_1227, &P23907_1};	/*Property = ScreenPixelsHeight*/
const Value V23907_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23907_4 = {S54, 0, &V23907_4_1229, &P23907_5};	/*Property = ScreenPixelsWidth*/
const Profile P23907 = {"23907", &P23907_4};	/*Profile ID - 23907*/
const Value V21294_1_1 = {S2, NULL};	/*Value = True*/
const Property P21294_1 = {S50, 0, &V21294_1_1, NULL};	/*Property = IsMobile*/
const Value V21294_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P21294_5 = {S52, 0, &V21294_5_37569, &P21294_1};	/*Property = ScreenPixelsHeight*/
const Value V21294_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21294_4 = {S54, 0, &V21294_4_5645, &P21294_5};	/*Property = ScreenPixelsWidth*/
const Profile P21294 = {"21294", &P21294_4};	/*Profile ID - 21294*/
const Value V21962_1_1 = {S2, NULL};	/*Value = True*/
const Property P21962_1 = {S50, 0, &V21962_1_1, NULL};	/*Property = IsMobile*/
const Value V21962_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21962_5 = {S52, 0, &V21962_5_1277, &P21962_1};	/*Property = ScreenPixelsHeight*/
const Value V21962_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21962_4 = {S54, 0, &V21962_4_2336, &P21962_5};	/*Property = ScreenPixelsWidth*/
const Profile P21962 = {"21962", &P21962_4};	/*Profile ID - 21962*/
const Value V21087_1_1 = {S2, NULL};	/*Value = True*/
const Property P21087_1 = {S50, 0, &V21087_1_1, NULL};	/*Property = IsMobile*/
const Value V21087_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21087_5 = {S52, 0, &V21087_5_1227, &P21087_1};	/*Property = ScreenPixelsHeight*/
const Value V21087_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21087_4 = {S54, 0, &V21087_4_1229, &P21087_5};	/*Property = ScreenPixelsWidth*/
const Profile P21087 = {"21087", &P21087_4};	/*Profile ID - 21087*/
const Value V23867_1_1 = {S2, NULL};	/*Value = True*/
const Property P23867_1 = {S50, 0, &V23867_1_1, NULL};	/*Property = IsMobile*/
const Value V23867_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23867_5 = {S52, 0, &V23867_5_1277, &P23867_1};	/*Property = ScreenPixelsHeight*/
const Value V23867_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23867_4 = {S54, 0, &V23867_4_1279, &P23867_5};	/*Property = ScreenPixelsWidth*/
const Profile P23867 = {"23867", &P23867_4};	/*Profile ID - 23867*/
const Value V20173_1_1 = {S2, NULL};	/*Value = True*/
const Property P20173_1 = {S50, 0, &V20173_1_1, NULL};	/*Property = IsMobile*/
const Value V20173_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20173_5 = {S52, 0, &V20173_5_1227, &P20173_1};	/*Property = ScreenPixelsHeight*/
const Value V20173_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20173_4 = {S54, 0, &V20173_4_1229, &P20173_5};	/*Property = ScreenPixelsWidth*/
const Profile P20173 = {"20173", &P20173_4};	/*Profile ID - 20173*/
const Value V18554_1_1 = {S2, NULL};	/*Value = True*/
const Property P18554_1 = {S50, 0, &V18554_1_1, NULL};	/*Property = IsMobile*/
const Value V18554_5_59864 = {S139, NULL};	/*Value = 1440*/
const Property P18554_5 = {S52, 0, &V18554_5_59864, &P18554_1};	/*Property = ScreenPixelsHeight*/
const Value V18554_4_59865 = {S135, NULL};	/*Value = 900*/
const Property P18554_4 = {S54, 0, &V18554_4_59865, &P18554_5};	/*Property = ScreenPixelsWidth*/
const Profile P18554 = {"18554", &P18554_4};	/*Profile ID - 18554*/
const Value V24028_1_1 = {S2, NULL};	/*Value = True*/
const Property P24028_1 = {S50, 0, &V24028_1_1, NULL};	/*Property = IsMobile*/
const Value V24028_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24028_5 = {S52, 0, &V24028_5_1277, &P24028_1};	/*Property = ScreenPixelsHeight*/
const Value V24028_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24028_4 = {S54, 0, &V24028_4_1279, &P24028_5};	/*Property = ScreenPixelsWidth*/
const Profile P24028 = {"24028", &P24028_4};	/*Profile ID - 24028*/
const Value V21634_1_1 = {S2, NULL};	/*Value = True*/
const Property P21634_1 = {S50, 0, &V21634_1_1, NULL};	/*Property = IsMobile*/
const Value V21634_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P21634_5 = {S52, 0, &V21634_5_37569, &P21634_1};	/*Property = ScreenPixelsHeight*/
const Value V21634_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21634_4 = {S54, 0, &V21634_4_5645, &P21634_5};	/*Property = ScreenPixelsWidth*/
const Profile P21634 = {"21634", &P21634_4};	/*Profile ID - 21634*/
const Value V21302_1_1 = {S2, NULL};	/*Value = True*/
const Property P21302_1 = {S50, 0, &V21302_1_1, NULL};	/*Property = IsMobile*/
const Value V21302_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21302_5 = {S52, 0, &V21302_5_1227, &P21302_1};	/*Property = ScreenPixelsHeight*/
const Value V21302_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21302_4 = {S54, 0, &V21302_4_1229, &P21302_5};	/*Property = ScreenPixelsWidth*/
const Profile P21302 = {"21302", &P21302_4};	/*Profile ID - 21302*/
const Value V20361_1_1 = {S2, NULL};	/*Value = True*/
const Property P20361_1 = {S50, 0, &V20361_1_1, NULL};	/*Property = IsMobile*/
const Value V20361_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20361_5 = {S52, 0, &V20361_5_1227, &P20361_1};	/*Property = ScreenPixelsHeight*/
const Value V20361_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20361_4 = {S54, 0, &V20361_4_1229, &P20361_5};	/*Property = ScreenPixelsWidth*/
const Profile P20361 = {"20361", &P20361_4};	/*Profile ID - 20361*/
const Value V21326_1_1 = {S2, NULL};	/*Value = True*/
const Property P21326_1 = {S50, 0, &V21326_1_1, NULL};	/*Property = IsMobile*/
const Value V21326_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21326_5 = {S52, 0, &V21326_5_1458, &P21326_1};	/*Property = ScreenPixelsHeight*/
const Value V21326_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21326_4 = {S54, 0, &V21326_4_37617, &P21326_5};	/*Property = ScreenPixelsWidth*/
const Profile P21326 = {"21326", &P21326_4};	/*Profile ID - 21326*/
const Value V21791_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_279 = {S1, 0, &V21791_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21791_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_282 = {S3, 0, &V21791_282_57155, &P21791_279};	/*Property = BlobBuilder*/
const Value V21791_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_350 = {S4, 0, &V21791_350_57446, &P21791_282};	/*Property = Canvas*/
const Value V21791_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_283 = {S5, 0, &V21791_283_57161, &P21791_350};	/*Property = CssBackground*/
const Value V21791_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_284 = {S6, 0, &V21791_284_57164, &P21791_283};	/*Property = CssBorderImage*/
const Value V21791_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_285 = {S7, 0, &V21791_285_57167, &P21791_284};	/*Property = CssCanvas*/
const Value V21791_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_286 = {S8, 0, &V21791_286_57170, &P21791_285};	/*Property = CssColor*/
const Value V21791_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_237 = {S9, 0, &V21791_237_55786, &P21791_286};	/*Property = CssColumn*/
const Value V21791_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_287 = {S10, 0, &V21791_287_57173, &P21791_237};	/*Property = CssFlexbox*/
const Value V21791_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_288 = {S11, 0, &V21791_288_57176, &P21791_287};	/*Property = CssFont*/
const Value V21791_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_289 = {S12, 0, &V21791_289_86899, &P21791_288};	/*Property = CssImages*/
const Value V21791_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_291 = {S13, 0, &V21791_291_57182, &P21791_289};	/*Property = CssMediaQueries*/
const Value V21791_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_292 = {S14, 0, &V21791_292_57185, &P21791_291};	/*Property = CssMinMax*/
const Value V21791_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_293 = {S15, 0, &V21791_293_57188, &P21791_292};	/*Property = CssOverflow*/
const Value V21791_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_294 = {S16, 0, &V21791_294_57191, &P21791_293};	/*Property = CssPosition*/
const Value V21791_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_295 = {S17, 0, &V21791_295_57194, &P21791_294};	/*Property = CssText*/
const Value V21791_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_297 = {S18, 0, &V21791_297_57197, &P21791_295};	/*Property = CssTransforms*/
const Value V21791_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_241 = {S19, 0, &V21791_241_55801, &P21791_297};	/*Property = CssTransitions*/
const Value V21791_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_298 = {S20, 0, &V21791_298_57208, &P21791_241};	/*Property = CssUI*/
const Value V21791_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_300 = {S21, 0, &V21791_300_57217, &P21791_298};	/*Property = DataSet*/
const Value V21791_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_301 = {S22, 0, &V21791_301_57220, &P21791_300};	/*Property = DataUrl*/
const Value V21791_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_302 = {S23, 0, &V21791_302_57223, &P21791_301};	/*Property = DeviceOrientation*/
const Value V21791_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_304 = {S24, 0, &V21791_304_57229, &P21791_302};	/*Property = FileReader*/
const Value V21791_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_305 = {S25, 0, &V21791_305_57232, &P21791_304};	/*Property = FileSaver*/
const Value V21791_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_306 = {S26, 0, &V21791_306_57238, &P21791_305};	/*Property = FileWriter*/
const Value V21791_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_307 = {S27, 0, &V21791_307_57241, &P21791_306};	/*Property = FormData*/
const Value V21791_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_309 = {S28, 0, &V21791_309_57247, &P21791_307};	/*Property = Fullscreen*/
const Value V21791_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_310 = {S29, 0, &V21791_310_57250, &P21791_309};	/*Property = GeoLocation*/
const Value V21791_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_312 = {S30, 0, &V21791_312_57256, &P21791_310};	/*Property = History*/
const Value V21791_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_314 = {S31, 0, &V21791_314_57262, &P21791_312};	/*Property = Html5*/
const Value V21791_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_313 = {S32, 0, &V21791_313_57259, &P21791_314};	/*Property = Html-Media-Capture*/
const Value V21791_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_315 = {S33, 0, &V21791_315_57265, &P21791_313};	/*Property = Iframe*/
const Value V21791_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_316 = {S34, 0, &V21791_316_57268, &P21791_315};	/*Property = IndexedDB*/
const Value V21791_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_317 = {S35, 0, &V21791_317_57271, &P21791_316};	/*Property = Json*/
const Value V21791_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P21791_15 = {S37, 0, &V21791_15_29, &P21791_317};	/*Property = LayoutEngine*/
const Value V21791_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_344 = {S38, 0, &V21791_344_57274, &P21791_15};	/*Property = Masking*/
const Value V21791_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_320 = {S39, 0, &V21791_320_57292, &P21791_344};	/*Property = PostMessage*/
const Value V21791_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_321 = {S40, 0, &V21791_321_57295, &P21791_320};	/*Property = Progress*/
const Value V21791_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_322 = {S41, 0, &V21791_322_57298, &P21791_321};	/*Property = Prompts*/
const Value V21791_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_323 = {S42, 0, &V21791_323_57301, &P21791_322};	/*Property = Selector*/
const Value V21791_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_324 = {S43, 0, &V21791_324_57307, &P21791_323};	/*Property = Svg*/
const Value V21791_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_327 = {S44, 0, &V21791_327_57316, &P21791_324};	/*Property = TouchEvents*/
const Value V21791_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_328 = {S45, 0, &V21791_328_57319, &P21791_327};	/*Property = Track*/
const Value V21791_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_329 = {S46, 0, &V21791_329_57325, &P21791_328};	/*Property = Video*/
const Value V21791_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_330 = {S47, 0, &V21791_330_57328, &P21791_329};	/*Property = Viewport*/
const Value V21791_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_332 = {S48, 0, &V21791_332_57340, &P21791_330};	/*Property = WebWorkers*/
const Value V21791_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21791_333 = {S49, 0, &V21791_333_57343, &P21791_332};	/*Property = Xhr2*/
const Profile P21791 = {"21791", &P21791_333};	/*Profile ID - 21791*/
const Value V21046_1_1 = {S2, NULL};	/*Value = True*/
const Property P21046_1 = {S50, 0, &V21046_1_1, NULL};	/*Property = IsMobile*/
const Value V21046_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21046_5 = {S52, 0, &V21046_5_1277, &P21046_1};	/*Property = ScreenPixelsHeight*/
const Value V21046_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21046_4 = {S54, 0, &V21046_4_1279, &P21046_5};	/*Property = ScreenPixelsWidth*/
const Profile P21046 = {"21046", &P21046_4};	/*Profile ID - 21046*/
const Value V23664_1_1 = {S2, NULL};	/*Value = True*/
const Property P23664_1 = {S50, 0, &V23664_1_1, NULL};	/*Property = IsMobile*/
const Value V23664_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23664_5 = {S52, 0, &V23664_5_1227, &P23664_1};	/*Property = ScreenPixelsHeight*/
const Value V23664_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23664_4 = {S54, 0, &V23664_4_1229, &P23664_5};	/*Property = ScreenPixelsWidth*/
const Profile P23664 = {"23664", &P23664_4};	/*Profile ID - 23664*/
const Value V23979_1_1 = {S2, NULL};	/*Value = True*/
const Property P23979_1 = {S50, 0, &V23979_1_1, NULL};	/*Property = IsMobile*/
const Value V23979_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23979_5 = {S52, 0, &V23979_5_1277, &P23979_1};	/*Property = ScreenPixelsHeight*/
const Value V23979_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23979_4 = {S54, 0, &V23979_4_1279, &P23979_5};	/*Property = ScreenPixelsWidth*/
const Profile P23979 = {"23979", &P23979_4};	/*Profile ID - 23979*/
const Value V21592_1_1 = {S2, NULL};	/*Value = True*/
const Property P21592_1 = {S50, 0, &V21592_1_1, NULL};	/*Property = IsMobile*/
const Value V21592_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21592_5 = {S52, 0, &V21592_5_1227, &P21592_1};	/*Property = ScreenPixelsHeight*/
const Value V21592_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21592_4 = {S54, 0, &V21592_4_1229, &P21592_5};	/*Property = ScreenPixelsWidth*/
const Profile P21592 = {"21592", &P21592_4};	/*Profile ID - 21592*/
const Value V21649_1_1 = {S2, NULL};	/*Value = True*/
const Property P21649_1 = {S50, 0, &V21649_1_1, NULL};	/*Property = IsMobile*/
const Value V21649_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21649_5 = {S52, 0, &V21649_5_9981, &P21649_1};	/*Property = ScreenPixelsHeight*/
const Value V21649_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21649_4 = {S54, 0, &V21649_4_2336, &P21649_5};	/*Property = ScreenPixelsWidth*/
const Profile P21649 = {"21649", &P21649_4};	/*Profile ID - 21649*/
const Value V20269_1_1 = {S2, NULL};	/*Value = True*/
const Property P20269_1 = {S50, 0, &V20269_1_1, NULL};	/*Property = IsMobile*/
const Value V20269_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20269_5 = {S52, 0, &V20269_5_9981, &P20269_1};	/*Property = ScreenPixelsHeight*/
const Value V20269_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20269_4 = {S54, 0, &V20269_4_2336, &P20269_5};	/*Property = ScreenPixelsWidth*/
const Profile P20269 = {"20269", &P20269_4};	/*Profile ID - 20269*/
const Value V22164_1_1 = {S2, NULL};	/*Value = True*/
const Property P22164_1 = {S50, 0, &V22164_1_1, NULL};	/*Property = IsMobile*/
const Value V22164_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22164_5 = {S52, 0, &V22164_5_37569, &P22164_1};	/*Property = ScreenPixelsHeight*/
const Value V22164_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22164_4 = {S54, 0, &V22164_4_5645, &P22164_5};	/*Property = ScreenPixelsWidth*/
const Profile P22164 = {"22164", &P22164_4};	/*Profile ID - 22164*/
const Value V22692_1_1 = {S2, NULL};	/*Value = True*/
const Property P22692_1 = {S50, 0, &V22692_1_1, NULL};	/*Property = IsMobile*/
const Value V22692_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P22692_5 = {S52, 0, &V22692_5_37, &P22692_1};	/*Property = ScreenPixelsHeight*/
const Value V22692_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P22692_4 = {S54, 0, &V22692_4_36, &P22692_5};	/*Property = ScreenPixelsWidth*/
const Profile P22692 = {"22692", &P22692_4};	/*Profile ID - 22692*/
const Value V22066_1_1 = {S2, NULL};	/*Value = True*/
const Property P22066_1 = {S50, 0, &V22066_1_1, NULL};	/*Property = IsMobile*/
const Value V22066_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22066_5 = {S52, 0, &V22066_5_1227, &P22066_1};	/*Property = ScreenPixelsHeight*/
const Value V22066_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22066_4 = {S54, 0, &V22066_4_1229, &P22066_5};	/*Property = ScreenPixelsWidth*/
const Profile P22066 = {"22066", &P22066_4};	/*Profile ID - 22066*/
const Value V20439_1_1 = {S2, NULL};	/*Value = True*/
const Property P20439_1 = {S50, 0, &V20439_1_1, NULL};	/*Property = IsMobile*/
const Value V20439_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20439_5 = {S52, 0, &V20439_5_9981, &P20439_1};	/*Property = ScreenPixelsHeight*/
const Value V20439_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20439_4 = {S54, 0, &V20439_4_39098, &P20439_5};	/*Property = ScreenPixelsWidth*/
const Profile P20439 = {"20439", &P20439_4};	/*Profile ID - 20439*/
const Value V21766_1_1 = {S2, NULL};	/*Value = True*/
const Property P21766_1 = {S50, 0, &V21766_1_1, NULL};	/*Property = IsMobile*/
const Value V21766_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21766_5 = {S52, 0, &V21766_5_1277, &P21766_1};	/*Property = ScreenPixelsHeight*/
const Value V21766_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21766_4 = {S54, 0, &V21766_4_1279, &P21766_5};	/*Property = ScreenPixelsWidth*/
const Profile P21766 = {"21766", &P21766_4};	/*Profile ID - 21766*/
const Value V21207_1_1 = {S2, NULL};	/*Value = True*/
const Property P21207_1 = {S50, 0, &V21207_1_1, NULL};	/*Property = IsMobile*/
const Value V21207_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21207_5 = {S52, 0, &V21207_5_1277, &P21207_1};	/*Property = ScreenPixelsHeight*/
const Value V21207_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21207_4 = {S54, 0, &V21207_4_1279, &P21207_5};	/*Property = ScreenPixelsWidth*/
const Profile P21207 = {"21207", &P21207_4};	/*Profile ID - 21207*/
const Value V21181_1_1 = {S2, NULL};	/*Value = True*/
const Property P21181_1 = {S50, 0, &V21181_1_1, NULL};	/*Property = IsMobile*/
const Value V21181_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21181_5 = {S52, 0, &V21181_5_1227, &P21181_1};	/*Property = ScreenPixelsHeight*/
const Value V21181_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21181_4 = {S54, 0, &V21181_4_1229, &P21181_5};	/*Property = ScreenPixelsWidth*/
const Profile P21181 = {"21181", &P21181_4};	/*Profile ID - 21181*/
const Value V21088_1_1 = {S2, NULL};	/*Value = True*/
const Property P21088_1 = {S50, 0, &V21088_1_1, NULL};	/*Property = IsMobile*/
const Value V21088_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21088_5 = {S52, 0, &V21088_5_1227, &P21088_1};	/*Property = ScreenPixelsHeight*/
const Value V21088_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21088_4 = {S54, 0, &V21088_4_1229, &P21088_5};	/*Property = ScreenPixelsWidth*/
const Profile P21088 = {"21088", &P21088_4};	/*Profile ID - 21088*/
const Value V19803_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19803_279 = {S1, 0, &V19803_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19803_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19803_282 = {S3, 0, &V19803_282_57154, &P19803_279};	/*Property = BlobBuilder*/
const Value V19803_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19803_350 = {S4, 0, &V19803_350_57444, &P19803_282};	/*Property = Canvas*/
const Value V19803_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19803_283 = {S5, 0, &V19803_283_57159, &P19803_350};	/*Property = CssBackground*/
const Value V19803_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19803_284 = {S6, 0, &V19803_284_57163, &P19803_283};	/*Property = CssBorderImage*/
const Value V19803_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19803_285 = {S7, 0, &V19803_285_57166, &P19803_284};	/*Property = CssCanvas*/
const Value V19803_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19803_286 = {S8, 0, &V19803_286_57168, &P19803_285};	/*Property = CssColor*/
const Value V19803_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19803_237 = {S9, 0, &V19803_237_55786, &P19803_286};	/*Property = CssColumn*/
const Value V19803_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19803_287 = {S10, 0, &V19803_287_57172, &P19803_237};	/*Property = CssFlexbox*/
const Value V19803_288_57174 = {S2, NULL};	/*Value = True*/
const Property P19803_288 = {S11, 0, &V19803_288_57174, &P19803_287};	/*Property = CssFont*/
const Value V19803_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19803_289 = {S12, 0, &V19803_289_57759, &P19803_288};	/*Property = CssImages*/
const Value V19803_291_57180 = {S2, NULL};	/*Value = True*/
const Property P19803_291 = {S13, 0, &V19803_291_57180, &P19803_289};	/*Property = CssMediaQueries*/
const Value V19803_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19803_292 = {S14, 0, &V19803_292_57183, &P19803_291};	/*Property = CssMinMax*/
const Value V19803_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19803_293 = {S15, 0, &V19803_293_57187, &P19803_292};	/*Property = CssOverflow*/
const Value V19803_294_57189 = {S2, NULL};	/*Value = True*/
const Property P19803_294 = {S16, 0, &V19803_294_57189, &P19803_293};	/*Property = CssPosition*/
const Value V19803_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19803_295 = {S17, 0, &V19803_295_57192, &P19803_294};	/*Property = CssText*/
const Value V19803_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19803_297 = {S18, 0, &V19803_297_57195, &P19803_295};	/*Property = CssTransforms*/
const Value V19803_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19803_241 = {S19, 0, &V19803_241_55799, &P19803_297};	/*Property = CssTransitions*/
const Value V19803_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19803_298 = {S20, 0, &V19803_298_57206, &P19803_241};	/*Property = CssUI*/
const Value V19803_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19803_300 = {S21, 0, &V19803_300_57216, &P19803_298};	/*Property = DataSet*/
const Value V19803_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19803_301 = {S22, 0, &V19803_301_57218, &P19803_300};	/*Property = DataUrl*/
const Value V19803_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19803_302 = {S23, 0, &V19803_302_57222, &P19803_301};	/*Property = DeviceOrientation*/
const Value V19803_304_57227 = {S2, NULL};	/*Value = True*/
const Property P19803_304 = {S24, 0, &V19803_304_57227, &P19803_302};	/*Property = FileReader*/
const Value V19803_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19803_305 = {S25, 0, &V19803_305_57231, &P19803_304};	/*Property = FileSaver*/
const Value V19803_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19803_306 = {S26, 0, &V19803_306_57237, &P19803_305};	/*Property = FileWriter*/
const Value V19803_307_57239 = {S2, NULL};	/*Value = True*/
const Property P19803_307 = {S27, 0, &V19803_307_57239, &P19803_306};	/*Property = FormData*/
const Value V19803_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19803_309 = {S28, 0, &V19803_309_57246, &P19803_307};	/*Property = Fullscreen*/
const Value V19803_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19803_310 = {S29, 0, &V19803_310_57248, &P19803_309};	/*Property = GeoLocation*/
const Value V19803_312_57254 = {S2, NULL};	/*Value = True*/
const Property P19803_312 = {S30, 0, &V19803_312_57254, &P19803_310};	/*Property = History*/
const Value V19803_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19803_314 = {S31, 0, &V19803_314_57261, &P19803_312};	/*Property = Html5*/
const Value V19803_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19803_313 = {S32, 0, &V19803_313_57258, &P19803_314};	/*Property = Html-Media-Capture*/
const Value V19803_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19803_315 = {S33, 0, &V19803_315_57264, &P19803_313};	/*Property = Iframe*/
const Value V19803_316_57266 = {S2, NULL};	/*Value = True*/
const Property P19803_316 = {S34, 0, &V19803_316_57266, &P19803_315};	/*Property = IndexedDB*/
const Value V19803_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19803_317 = {S35, 0, &V19803_317_57269, &P19803_316};	/*Property = Json*/
const Value V19803_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19803_15 = {S37, 0, &V19803_15_28, &P19803_317};	/*Property = LayoutEngine*/
const Value V19803_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19803_344 = {S38, 0, &V19803_344_57272, &P19803_15};	/*Property = Masking*/
const Value V19803_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19803_320 = {S39, 0, &V19803_320_57290, &P19803_344};	/*Property = PostMessage*/
const Value V19803_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19803_321 = {S40, 0, &V19803_321_57293, &P19803_320};	/*Property = Progress*/
const Value V19803_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19803_322 = {S41, 0, &V19803_322_57296, &P19803_321};	/*Property = Prompts*/
const Value V19803_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19803_323 = {S42, 0, &V19803_323_57299, &P19803_322};	/*Property = Selector*/
const Value V19803_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19803_324 = {S43, 0, &V19803_324_57306, &P19803_323};	/*Property = Svg*/
const Value V19803_327_57314 = {S2, NULL};	/*Value = True*/
const Property P19803_327 = {S44, 0, &V19803_327_57314, &P19803_324};	/*Property = TouchEvents*/
const Value V19803_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19803_328 = {S45, 0, &V19803_328_57318, &P19803_327};	/*Property = Track*/
const Value V19803_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19803_329 = {S46, 0, &V19803_329_57323, &P19803_328};	/*Property = Video*/
const Value V19803_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19803_330 = {S47, 0, &V19803_330_57326, &P19803_329};	/*Property = Viewport*/
const Value V19803_332_57339 = {S0, NULL};	/*Value = False*/
const Property P19803_332 = {S48, 0, &V19803_332_57339, &P19803_330};	/*Property = WebWorkers*/
const Value V19803_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19803_333 = {S49, 0, &V19803_333_57342, &P19803_332};	/*Property = Xhr2*/
const Profile P19803 = {"19803", &P19803_333};	/*Profile ID - 19803*/
const Value V23054_1_1 = {S2, NULL};	/*Value = True*/
const Property P23054_1 = {S50, 0, &V23054_1_1, NULL};	/*Property = IsMobile*/
const Value V23054_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23054_5 = {S52, 0, &V23054_5_1222, &P23054_1};	/*Property = ScreenPixelsHeight*/
const Value V23054_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23054_4 = {S54, 0, &V23054_4_1224, &P23054_5};	/*Property = ScreenPixelsWidth*/
const Profile P23054 = {"23054", &P23054_4};	/*Profile ID - 23054*/
const Value V21106_1_1 = {S2, NULL};	/*Value = True*/
const Property P21106_1 = {S50, 0, &V21106_1_1, NULL};	/*Property = IsMobile*/
const Value V21106_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21106_5 = {S52, 0, &V21106_5_9981, &P21106_1};	/*Property = ScreenPixelsHeight*/
const Value V21106_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21106_4 = {S54, 0, &V21106_4_2336, &P21106_5};	/*Property = ScreenPixelsWidth*/
const Profile P21106 = {"21106", &P21106_4};	/*Profile ID - 21106*/
const Value V20841_1_1 = {S2, NULL};	/*Value = True*/
const Property P20841_1 = {S50, 0, &V20841_1_1, NULL};	/*Property = IsMobile*/
const Value V20841_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20841_5 = {S52, 0, &V20841_5_9981, &P20841_1};	/*Property = ScreenPixelsHeight*/
const Value V20841_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20841_4 = {S54, 0, &V20841_4_2336, &P20841_5};	/*Property = ScreenPixelsWidth*/
const Profile P20841 = {"20841", &P20841_4};	/*Profile ID - 20841*/
const Value V18558_1_1 = {S2, NULL};	/*Value = True*/
const Property P18558_1 = {S50, 0, &V18558_1_1, NULL};	/*Property = IsMobile*/
const Value V18558_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P18558_5 = {S52, 0, &V18558_5_1304, &P18558_1};	/*Property = ScreenPixelsHeight*/
const Value V18558_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18558_4 = {S54, 0, &V18558_4_1224, &P18558_5};	/*Property = ScreenPixelsWidth*/
const Profile P18558 = {"18558", &P18558_4};	/*Profile ID - 18558*/
const Value V21356_1_1 = {S2, NULL};	/*Value = True*/
const Property P21356_1 = {S50, 0, &V21356_1_1, NULL};	/*Property = IsMobile*/
const Value V21356_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21356_5 = {S52, 0, &V21356_5_1277, &P21356_1};	/*Property = ScreenPixelsHeight*/
const Value V21356_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21356_4 = {S54, 0, &V21356_4_1279, &P21356_5};	/*Property = ScreenPixelsWidth*/
const Profile P21356 = {"21356", &P21356_4};	/*Profile ID - 21356*/
const Value V20750_1_1 = {S2, NULL};	/*Value = True*/
const Property P20750_1 = {S50, 0, &V20750_1_1, NULL};	/*Property = IsMobile*/
const Value V20750_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20750_5 = {S52, 0, &V20750_5_1458, &P20750_1};	/*Property = ScreenPixelsHeight*/
const Value V20750_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20750_4 = {S54, 0, &V20750_4_1460, &P20750_5};	/*Property = ScreenPixelsWidth*/
const Profile P20750 = {"20750", &P20750_4};	/*Profile ID - 20750*/
const Value V21768_1_1 = {S2, NULL};	/*Value = True*/
const Property P21768_1 = {S50, 0, &V21768_1_1, NULL};	/*Property = IsMobile*/
const Value V21768_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21768_5 = {S52, 0, &V21768_5_1277, &P21768_1};	/*Property = ScreenPixelsHeight*/
const Value V21768_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21768_4 = {S54, 0, &V21768_4_1279, &P21768_5};	/*Property = ScreenPixelsWidth*/
const Profile P21768 = {"21768", &P21768_4};	/*Profile ID - 21768*/
const Value V21526_1_1 = {S2, NULL};	/*Value = True*/
const Property P21526_1 = {S50, 0, &V21526_1_1, NULL};	/*Property = IsMobile*/
const Value V21526_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21526_5 = {S52, 0, &V21526_5_1227, &P21526_1};	/*Property = ScreenPixelsHeight*/
const Value V21526_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21526_4 = {S54, 0, &V21526_4_1229, &P21526_5};	/*Property = ScreenPixelsWidth*/
const Profile P21526 = {"21526", &P21526_4};	/*Profile ID - 21526*/
const Value V18639_1_1 = {S2, NULL};	/*Value = True*/
const Property P18639_1 = {S50, 0, &V18639_1_1, NULL};	/*Property = IsMobile*/
const Value V18639_5_60078 = {S140, NULL};	/*Value = 2560*/
const Property P18639_5 = {S52, 0, &V18639_5_60078, &P18639_1};	/*Property = ScreenPixelsHeight*/
const Value V18639_4_37607 = {S134, NULL};	/*Value = 1600*/
const Property P18639_4 = {S54, 0, &V18639_4_37607, &P18639_5};	/*Property = ScreenPixelsWidth*/
const Profile P18639 = {"18639", &P18639_4};	/*Profile ID - 18639*/
const Value V18640_1_1 = {S2, NULL};	/*Value = True*/
const Property P18640_1 = {S50, 0, &V18640_1_1, NULL};	/*Property = IsMobile*/
const Value V18640_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18640_5 = {S52, 0, &V18640_5_1403, &P18640_1};	/*Property = ScreenPixelsHeight*/
const Value V18640_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18640_4 = {S54, 0, &V18640_4_1404, &P18640_5};	/*Property = ScreenPixelsWidth*/
const Profile P18640 = {"18640", &P18640_4};	/*Profile ID - 18640*/
const Value V20913_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_279 = {S1, 0, &V20913_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20913_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_282 = {S3, 0, &V20913_282_57155, &P20913_279};	/*Property = BlobBuilder*/
const Value V20913_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_350 = {S4, 0, &V20913_350_57446, &P20913_282};	/*Property = Canvas*/
const Value V20913_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_283 = {S5, 0, &V20913_283_57161, &P20913_350};	/*Property = CssBackground*/
const Value V20913_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_284 = {S6, 0, &V20913_284_57164, &P20913_283};	/*Property = CssBorderImage*/
const Value V20913_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_285 = {S7, 0, &V20913_285_57167, &P20913_284};	/*Property = CssCanvas*/
const Value V20913_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_286 = {S8, 0, &V20913_286_57170, &P20913_285};	/*Property = CssColor*/
const Value V20913_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_237 = {S9, 0, &V20913_237_55786, &P20913_286};	/*Property = CssColumn*/
const Value V20913_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_287 = {S10, 0, &V20913_287_57173, &P20913_237};	/*Property = CssFlexbox*/
const Value V20913_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_288 = {S11, 0, &V20913_288_57176, &P20913_287};	/*Property = CssFont*/
const Value V20913_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_289 = {S12, 0, &V20913_289_86899, &P20913_288};	/*Property = CssImages*/
const Value V20913_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_291 = {S13, 0, &V20913_291_57182, &P20913_289};	/*Property = CssMediaQueries*/
const Value V20913_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_292 = {S14, 0, &V20913_292_57185, &P20913_291};	/*Property = CssMinMax*/
const Value V20913_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_293 = {S15, 0, &V20913_293_57188, &P20913_292};	/*Property = CssOverflow*/
const Value V20913_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_294 = {S16, 0, &V20913_294_57191, &P20913_293};	/*Property = CssPosition*/
const Value V20913_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_295 = {S17, 0, &V20913_295_57194, &P20913_294};	/*Property = CssText*/
const Value V20913_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_297 = {S18, 0, &V20913_297_57197, &P20913_295};	/*Property = CssTransforms*/
const Value V20913_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_241 = {S19, 0, &V20913_241_55801, &P20913_297};	/*Property = CssTransitions*/
const Value V20913_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_298 = {S20, 0, &V20913_298_57208, &P20913_241};	/*Property = CssUI*/
const Value V20913_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_300 = {S21, 0, &V20913_300_57217, &P20913_298};	/*Property = DataSet*/
const Value V20913_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_301 = {S22, 0, &V20913_301_57220, &P20913_300};	/*Property = DataUrl*/
const Value V20913_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_302 = {S23, 0, &V20913_302_57223, &P20913_301};	/*Property = DeviceOrientation*/
const Value V20913_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_304 = {S24, 0, &V20913_304_57229, &P20913_302};	/*Property = FileReader*/
const Value V20913_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_305 = {S25, 0, &V20913_305_57232, &P20913_304};	/*Property = FileSaver*/
const Value V20913_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_306 = {S26, 0, &V20913_306_57238, &P20913_305};	/*Property = FileWriter*/
const Value V20913_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_307 = {S27, 0, &V20913_307_57241, &P20913_306};	/*Property = FormData*/
const Value V20913_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_309 = {S28, 0, &V20913_309_57247, &P20913_307};	/*Property = Fullscreen*/
const Value V20913_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_310 = {S29, 0, &V20913_310_57250, &P20913_309};	/*Property = GeoLocation*/
const Value V20913_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_312 = {S30, 0, &V20913_312_57256, &P20913_310};	/*Property = History*/
const Value V20913_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_314 = {S31, 0, &V20913_314_57262, &P20913_312};	/*Property = Html5*/
const Value V20913_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_313 = {S32, 0, &V20913_313_57259, &P20913_314};	/*Property = Html-Media-Capture*/
const Value V20913_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_315 = {S33, 0, &V20913_315_57265, &P20913_313};	/*Property = Iframe*/
const Value V20913_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_316 = {S34, 0, &V20913_316_57268, &P20913_315};	/*Property = IndexedDB*/
const Value V20913_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_317 = {S35, 0, &V20913_317_57271, &P20913_316};	/*Property = Json*/
const Value V20913_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P20913_15 = {S37, 0, &V20913_15_28, &P20913_317};	/*Property = LayoutEngine*/
const Value V20913_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_344 = {S38, 0, &V20913_344_57274, &P20913_15};	/*Property = Masking*/
const Value V20913_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_320 = {S39, 0, &V20913_320_57292, &P20913_344};	/*Property = PostMessage*/
const Value V20913_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_321 = {S40, 0, &V20913_321_57295, &P20913_320};	/*Property = Progress*/
const Value V20913_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_322 = {S41, 0, &V20913_322_57298, &P20913_321};	/*Property = Prompts*/
const Value V20913_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_323 = {S42, 0, &V20913_323_57301, &P20913_322};	/*Property = Selector*/
const Value V20913_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_324 = {S43, 0, &V20913_324_57307, &P20913_323};	/*Property = Svg*/
const Value V20913_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_327 = {S44, 0, &V20913_327_57316, &P20913_324};	/*Property = TouchEvents*/
const Value V20913_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_328 = {S45, 0, &V20913_328_57319, &P20913_327};	/*Property = Track*/
const Value V20913_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_329 = {S46, 0, &V20913_329_57325, &P20913_328};	/*Property = Video*/
const Value V20913_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_330 = {S47, 0, &V20913_330_57328, &P20913_329};	/*Property = Viewport*/
const Value V20913_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_332 = {S48, 0, &V20913_332_57340, &P20913_330};	/*Property = WebWorkers*/
const Value V20913_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20913_333 = {S49, 0, &V20913_333_57343, &P20913_332};	/*Property = Xhr2*/
const Profile P20913 = {"20913", &P20913_333};	/*Profile ID - 20913*/
const Value V19798_1_1 = {S2, NULL};	/*Value = True*/
const Property P19798_1 = {S50, 0, &V19798_1_1, NULL};	/*Property = IsMobile*/
const Value V19798_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P19798_5 = {S52, 0, &V19798_5_1277, &P19798_1};	/*Property = ScreenPixelsHeight*/
const Value V19798_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P19798_4 = {S54, 0, &V19798_4_1279, &P19798_5};	/*Property = ScreenPixelsWidth*/
const Profile P19798 = {"19798", &P19798_4};	/*Profile ID - 19798*/
const Value V20427_1_1 = {S2, NULL};	/*Value = True*/
const Property P20427_1 = {S50, 0, &V20427_1_1, NULL};	/*Property = IsMobile*/
const Value V20427_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20427_5 = {S52, 0, &V20427_5_9981, &P20427_1};	/*Property = ScreenPixelsHeight*/
const Value V20427_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20427_4 = {S54, 0, &V20427_4_39098, &P20427_5};	/*Property = ScreenPixelsWidth*/
const Profile P20427 = {"20427", &P20427_4};	/*Profile ID - 20427*/
const Profile P19826 = {"19826", NULL};	/*Profile ID - 19826*/
const Value V20764_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_279 = {S1, 0, &V20764_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20764_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_282 = {S3, 0, &V20764_282_57155, &P20764_279};	/*Property = BlobBuilder*/
const Value V20764_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_350 = {S4, 0, &V20764_350_57446, &P20764_282};	/*Property = Canvas*/
const Value V20764_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_283 = {S5, 0, &V20764_283_57161, &P20764_350};	/*Property = CssBackground*/
const Value V20764_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_284 = {S6, 0, &V20764_284_57164, &P20764_283};	/*Property = CssBorderImage*/
const Value V20764_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_285 = {S7, 0, &V20764_285_57167, &P20764_284};	/*Property = CssCanvas*/
const Value V20764_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_286 = {S8, 0, &V20764_286_57170, &P20764_285};	/*Property = CssColor*/
const Value V20764_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_237 = {S9, 0, &V20764_237_55786, &P20764_286};	/*Property = CssColumn*/
const Value V20764_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_287 = {S10, 0, &V20764_287_57173, &P20764_237};	/*Property = CssFlexbox*/
const Value V20764_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_288 = {S11, 0, &V20764_288_57176, &P20764_287};	/*Property = CssFont*/
const Value V20764_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_289 = {S12, 0, &V20764_289_86899, &P20764_288};	/*Property = CssImages*/
const Value V20764_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_291 = {S13, 0, &V20764_291_57182, &P20764_289};	/*Property = CssMediaQueries*/
const Value V20764_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_292 = {S14, 0, &V20764_292_57185, &P20764_291};	/*Property = CssMinMax*/
const Value V20764_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_293 = {S15, 0, &V20764_293_57188, &P20764_292};	/*Property = CssOverflow*/
const Value V20764_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_294 = {S16, 0, &V20764_294_57191, &P20764_293};	/*Property = CssPosition*/
const Value V20764_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_295 = {S17, 0, &V20764_295_57194, &P20764_294};	/*Property = CssText*/
const Value V20764_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_297 = {S18, 0, &V20764_297_57197, &P20764_295};	/*Property = CssTransforms*/
const Value V20764_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_241 = {S19, 0, &V20764_241_55801, &P20764_297};	/*Property = CssTransitions*/
const Value V20764_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_298 = {S20, 0, &V20764_298_57208, &P20764_241};	/*Property = CssUI*/
const Value V20764_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_300 = {S21, 0, &V20764_300_57217, &P20764_298};	/*Property = DataSet*/
const Value V20764_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_301 = {S22, 0, &V20764_301_57220, &P20764_300};	/*Property = DataUrl*/
const Value V20764_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_302 = {S23, 0, &V20764_302_57223, &P20764_301};	/*Property = DeviceOrientation*/
const Value V20764_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_304 = {S24, 0, &V20764_304_57229, &P20764_302};	/*Property = FileReader*/
const Value V20764_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_305 = {S25, 0, &V20764_305_57232, &P20764_304};	/*Property = FileSaver*/
const Value V20764_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_306 = {S26, 0, &V20764_306_57238, &P20764_305};	/*Property = FileWriter*/
const Value V20764_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_307 = {S27, 0, &V20764_307_57241, &P20764_306};	/*Property = FormData*/
const Value V20764_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_309 = {S28, 0, &V20764_309_57247, &P20764_307};	/*Property = Fullscreen*/
const Value V20764_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_310 = {S29, 0, &V20764_310_57250, &P20764_309};	/*Property = GeoLocation*/
const Value V20764_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_312 = {S30, 0, &V20764_312_57256, &P20764_310};	/*Property = History*/
const Value V20764_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_314 = {S31, 0, &V20764_314_57262, &P20764_312};	/*Property = Html5*/
const Value V20764_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_313 = {S32, 0, &V20764_313_57259, &P20764_314};	/*Property = Html-Media-Capture*/
const Value V20764_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_315 = {S33, 0, &V20764_315_57265, &P20764_313};	/*Property = Iframe*/
const Value V20764_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_316 = {S34, 0, &V20764_316_57268, &P20764_315};	/*Property = IndexedDB*/
const Value V20764_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_317 = {S35, 0, &V20764_317_57271, &P20764_316};	/*Property = Json*/
const Value V20764_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P20764_15 = {S37, 0, &V20764_15_90837, &P20764_317};	/*Property = LayoutEngine*/
const Value V20764_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_344 = {S38, 0, &V20764_344_57274, &P20764_15};	/*Property = Masking*/
const Value V20764_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_320 = {S39, 0, &V20764_320_57292, &P20764_344};	/*Property = PostMessage*/
const Value V20764_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_321 = {S40, 0, &V20764_321_57295, &P20764_320};	/*Property = Progress*/
const Value V20764_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_322 = {S41, 0, &V20764_322_57298, &P20764_321};	/*Property = Prompts*/
const Value V20764_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_323 = {S42, 0, &V20764_323_57301, &P20764_322};	/*Property = Selector*/
const Value V20764_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_324 = {S43, 0, &V20764_324_57307, &P20764_323};	/*Property = Svg*/
const Value V20764_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_327 = {S44, 0, &V20764_327_57316, &P20764_324};	/*Property = TouchEvents*/
const Value V20764_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_328 = {S45, 0, &V20764_328_57319, &P20764_327};	/*Property = Track*/
const Value V20764_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_329 = {S46, 0, &V20764_329_57325, &P20764_328};	/*Property = Video*/
const Value V20764_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_330 = {S47, 0, &V20764_330_57328, &P20764_329};	/*Property = Viewport*/
const Value V20764_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_332 = {S48, 0, &V20764_332_57340, &P20764_330};	/*Property = WebWorkers*/
const Value V20764_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20764_333 = {S49, 0, &V20764_333_57343, &P20764_332};	/*Property = Xhr2*/
const Profile P20764 = {"20764", &P20764_333};	/*Profile ID - 20764*/
const Value V24514_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_279 = {S1, 0, &V24514_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24514_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_282 = {S3, 0, &V24514_282_57155, &P24514_279};	/*Property = BlobBuilder*/
const Value V24514_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_350 = {S4, 0, &V24514_350_57446, &P24514_282};	/*Property = Canvas*/
const Value V24514_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_283 = {S5, 0, &V24514_283_57161, &P24514_350};	/*Property = CssBackground*/
const Value V24514_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_284 = {S6, 0, &V24514_284_57164, &P24514_283};	/*Property = CssBorderImage*/
const Value V24514_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_285 = {S7, 0, &V24514_285_57167, &P24514_284};	/*Property = CssCanvas*/
const Value V24514_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_286 = {S8, 0, &V24514_286_57170, &P24514_285};	/*Property = CssColor*/
const Value V24514_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_237 = {S9, 0, &V24514_237_55786, &P24514_286};	/*Property = CssColumn*/
const Value V24514_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_287 = {S10, 0, &V24514_287_57173, &P24514_237};	/*Property = CssFlexbox*/
const Value V24514_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_288 = {S11, 0, &V24514_288_57176, &P24514_287};	/*Property = CssFont*/
const Value V24514_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_289 = {S12, 0, &V24514_289_86899, &P24514_288};	/*Property = CssImages*/
const Value V24514_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_291 = {S13, 0, &V24514_291_57182, &P24514_289};	/*Property = CssMediaQueries*/
const Value V24514_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_292 = {S14, 0, &V24514_292_57185, &P24514_291};	/*Property = CssMinMax*/
const Value V24514_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_293 = {S15, 0, &V24514_293_57188, &P24514_292};	/*Property = CssOverflow*/
const Value V24514_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_294 = {S16, 0, &V24514_294_57191, &P24514_293};	/*Property = CssPosition*/
const Value V24514_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_295 = {S17, 0, &V24514_295_57194, &P24514_294};	/*Property = CssText*/
const Value V24514_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_297 = {S18, 0, &V24514_297_57197, &P24514_295};	/*Property = CssTransforms*/
const Value V24514_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_241 = {S19, 0, &V24514_241_55801, &P24514_297};	/*Property = CssTransitions*/
const Value V24514_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_298 = {S20, 0, &V24514_298_57208, &P24514_241};	/*Property = CssUI*/
const Value V24514_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_300 = {S21, 0, &V24514_300_57217, &P24514_298};	/*Property = DataSet*/
const Value V24514_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_301 = {S22, 0, &V24514_301_57220, &P24514_300};	/*Property = DataUrl*/
const Value V24514_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_302 = {S23, 0, &V24514_302_57223, &P24514_301};	/*Property = DeviceOrientation*/
const Value V24514_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_304 = {S24, 0, &V24514_304_57229, &P24514_302};	/*Property = FileReader*/
const Value V24514_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_305 = {S25, 0, &V24514_305_57232, &P24514_304};	/*Property = FileSaver*/
const Value V24514_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_306 = {S26, 0, &V24514_306_57238, &P24514_305};	/*Property = FileWriter*/
const Value V24514_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_307 = {S27, 0, &V24514_307_57241, &P24514_306};	/*Property = FormData*/
const Value V24514_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_309 = {S28, 0, &V24514_309_57247, &P24514_307};	/*Property = Fullscreen*/
const Value V24514_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_310 = {S29, 0, &V24514_310_57250, &P24514_309};	/*Property = GeoLocation*/
const Value V24514_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_312 = {S30, 0, &V24514_312_57256, &P24514_310};	/*Property = History*/
const Value V24514_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_314 = {S31, 0, &V24514_314_57262, &P24514_312};	/*Property = Html5*/
const Value V24514_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_313 = {S32, 0, &V24514_313_57259, &P24514_314};	/*Property = Html-Media-Capture*/
const Value V24514_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_315 = {S33, 0, &V24514_315_57265, &P24514_313};	/*Property = Iframe*/
const Value V24514_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_316 = {S34, 0, &V24514_316_57268, &P24514_315};	/*Property = IndexedDB*/
const Value V24514_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_317 = {S35, 0, &V24514_317_57271, &P24514_316};	/*Property = Json*/
const Value V24514_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P24514_15 = {S37, 0, &V24514_15_31, &P24514_317};	/*Property = LayoutEngine*/
const Value V24514_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_344 = {S38, 0, &V24514_344_57274, &P24514_15};	/*Property = Masking*/
const Value V24514_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_320 = {S39, 0, &V24514_320_57292, &P24514_344};	/*Property = PostMessage*/
const Value V24514_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_321 = {S40, 0, &V24514_321_57295, &P24514_320};	/*Property = Progress*/
const Value V24514_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_322 = {S41, 0, &V24514_322_57298, &P24514_321};	/*Property = Prompts*/
const Value V24514_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_323 = {S42, 0, &V24514_323_57301, &P24514_322};	/*Property = Selector*/
const Value V24514_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_324 = {S43, 0, &V24514_324_57307, &P24514_323};	/*Property = Svg*/
const Value V24514_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_327 = {S44, 0, &V24514_327_57316, &P24514_324};	/*Property = TouchEvents*/
const Value V24514_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_328 = {S45, 0, &V24514_328_57319, &P24514_327};	/*Property = Track*/
const Value V24514_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_329 = {S46, 0, &V24514_329_57325, &P24514_328};	/*Property = Video*/
const Value V24514_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_330 = {S47, 0, &V24514_330_57328, &P24514_329};	/*Property = Viewport*/
const Value V24514_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_332 = {S48, 0, &V24514_332_57340, &P24514_330};	/*Property = WebWorkers*/
const Value V24514_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24514_333 = {S49, 0, &V24514_333_57343, &P24514_332};	/*Property = Xhr2*/
const Profile P24514 = {"24514", &P24514_333};	/*Profile ID - 24514*/
const Value V21814_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21814_279 = {S1, 0, &V21814_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21814_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21814_282 = {S3, 0, &V21814_282_57154, &P21814_279};	/*Property = BlobBuilder*/
const Value V21814_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21814_350 = {S4, 0, &V21814_350_57444, &P21814_282};	/*Property = Canvas*/
const Value V21814_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21814_283 = {S5, 0, &V21814_283_57159, &P21814_350};	/*Property = CssBackground*/
const Value V21814_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21814_284 = {S6, 0, &V21814_284_57163, &P21814_283};	/*Property = CssBorderImage*/
const Value V21814_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21814_285 = {S7, 0, &V21814_285_57166, &P21814_284};	/*Property = CssCanvas*/
const Value V21814_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21814_286 = {S8, 0, &V21814_286_57168, &P21814_285};	/*Property = CssColor*/
const Value V21814_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21814_237 = {S9, 0, &V21814_237_55784, &P21814_286};	/*Property = CssColumn*/
const Value V21814_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21814_287 = {S10, 0, &V21814_287_57172, &P21814_237};	/*Property = CssFlexbox*/
const Value V21814_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21814_288 = {S11, 0, &V21814_288_57175, &P21814_287};	/*Property = CssFont*/
const Value V21814_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21814_289 = {S12, 0, &V21814_289_57759, &P21814_288};	/*Property = CssImages*/
const Value V21814_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21814_291 = {S13, 0, &V21814_291_57181, &P21814_289};	/*Property = CssMediaQueries*/
const Value V21814_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21814_292 = {S14, 0, &V21814_292_57183, &P21814_291};	/*Property = CssMinMax*/
const Value V21814_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21814_293 = {S15, 0, &V21814_293_57187, &P21814_292};	/*Property = CssOverflow*/
const Value V21814_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21814_294 = {S16, 0, &V21814_294_57190, &P21814_293};	/*Property = CssPosition*/
const Value V21814_295_57193 = {S0, NULL};	/*Value = False*/
const Property P21814_295 = {S17, 0, &V21814_295_57193, &P21814_294};	/*Property = CssText*/
const Value V21814_297_57196 = {S0, NULL};	/*Value = False*/
const Property P21814_297 = {S18, 0, &V21814_297_57196, &P21814_295};	/*Property = CssTransforms*/
const Value V21814_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21814_241 = {S19, 0, &V21814_241_55799, &P21814_297};	/*Property = CssTransitions*/
const Value V21814_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21814_298 = {S20, 0, &V21814_298_57206, &P21814_241};	/*Property = CssUI*/
const Value V21814_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21814_300 = {S21, 0, &V21814_300_57216, &P21814_298};	/*Property = DataSet*/
const Value V21814_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21814_301 = {S22, 0, &V21814_301_57218, &P21814_300};	/*Property = DataUrl*/
const Value V21814_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21814_302 = {S23, 0, &V21814_302_57222, &P21814_301};	/*Property = DeviceOrientation*/
const Value V21814_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21814_304 = {S24, 0, &V21814_304_57228, &P21814_302};	/*Property = FileReader*/
const Value V21814_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21814_305 = {S25, 0, &V21814_305_57231, &P21814_304};	/*Property = FileSaver*/
const Value V21814_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21814_306 = {S26, 0, &V21814_306_57237, &P21814_305};	/*Property = FileWriter*/
const Value V21814_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21814_307 = {S27, 0, &V21814_307_57240, &P21814_306};	/*Property = FormData*/
const Value V21814_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21814_309 = {S28, 0, &V21814_309_57246, &P21814_307};	/*Property = Fullscreen*/
const Value V21814_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21814_310 = {S29, 0, &V21814_310_57248, &P21814_309};	/*Property = GeoLocation*/
const Value V21814_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21814_312 = {S30, 0, &V21814_312_57255, &P21814_310};	/*Property = History*/
const Value V21814_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21814_314 = {S31, 0, &V21814_314_57261, &P21814_312};	/*Property = Html5*/
const Value V21814_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21814_313 = {S32, 0, &V21814_313_57258, &P21814_314};	/*Property = Html-Media-Capture*/
const Value V21814_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21814_315 = {S33, 0, &V21814_315_57264, &P21814_313};	/*Property = Iframe*/
const Value V21814_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21814_316 = {S34, 0, &V21814_316_57267, &P21814_315};	/*Property = IndexedDB*/
const Value V21814_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21814_317 = {S35, 0, &V21814_317_57269, &P21814_316};	/*Property = Json*/
const Value V21814_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P21814_15 = {S37, 0, &V21814_15_31, &P21814_317};	/*Property = LayoutEngine*/
const Value V21814_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21814_344 = {S38, 0, &V21814_344_57272, &P21814_15};	/*Property = Masking*/
const Value V21814_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21814_320 = {S39, 0, &V21814_320_57290, &P21814_344};	/*Property = PostMessage*/
const Value V21814_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21814_321 = {S40, 0, &V21814_321_57293, &P21814_320};	/*Property = Progress*/
const Value V21814_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21814_322 = {S41, 0, &V21814_322_57296, &P21814_321};	/*Property = Prompts*/
const Value V21814_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21814_323 = {S42, 0, &V21814_323_57299, &P21814_322};	/*Property = Selector*/
const Value V21814_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21814_324 = {S43, 0, &V21814_324_57306, &P21814_323};	/*Property = Svg*/
const Value V21814_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21814_327 = {S44, 0, &V21814_327_57315, &P21814_324};	/*Property = TouchEvents*/
const Value V21814_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21814_328 = {S45, 0, &V21814_328_57318, &P21814_327};	/*Property = Track*/
const Value V21814_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21814_329 = {S46, 0, &V21814_329_57323, &P21814_328};	/*Property = Video*/
const Value V21814_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21814_330 = {S47, 0, &V21814_330_57326, &P21814_329};	/*Property = Viewport*/
const Value V21814_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21814_332 = {S48, 0, &V21814_332_57339, &P21814_330};	/*Property = WebWorkers*/
const Value V21814_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21814_333 = {S49, 0, &V21814_333_57342, &P21814_332};	/*Property = Xhr2*/
const Profile P21814 = {"21814", &P21814_333};	/*Profile ID - 21814*/
const Value V22367_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22367_279 = {S1, 0, &V22367_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22367_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22367_282 = {S3, 0, &V22367_282_57153, &P22367_279};	/*Property = BlobBuilder*/
const Value V22367_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22367_350 = {S4, 0, &V22367_350_57444, &P22367_282};	/*Property = Canvas*/
const Value V22367_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22367_283 = {S5, 0, &V22367_283_57159, &P22367_350};	/*Property = CssBackground*/
const Value V22367_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22367_284 = {S6, 0, &V22367_284_57163, &P22367_283};	/*Property = CssBorderImage*/
const Value V22367_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22367_285 = {S7, 0, &V22367_285_57166, &P22367_284};	/*Property = CssCanvas*/
const Value V22367_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22367_286 = {S8, 0, &V22367_286_57168, &P22367_285};	/*Property = CssColor*/
const Value V22367_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22367_237 = {S9, 0, &V22367_237_55786, &P22367_286};	/*Property = CssColumn*/
const Value V22367_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22367_287 = {S10, 0, &V22367_287_57172, &P22367_237};	/*Property = CssFlexbox*/
const Value V22367_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22367_288 = {S11, 0, &V22367_288_57175, &P22367_287};	/*Property = CssFont*/
const Value V22367_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22367_289 = {S12, 0, &V22367_289_57759, &P22367_288};	/*Property = CssImages*/
const Value V22367_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22367_291 = {S13, 0, &V22367_291_57181, &P22367_289};	/*Property = CssMediaQueries*/
const Value V22367_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22367_292 = {S14, 0, &V22367_292_57183, &P22367_291};	/*Property = CssMinMax*/
const Value V22367_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22367_293 = {S15, 0, &V22367_293_57187, &P22367_292};	/*Property = CssOverflow*/
const Value V22367_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22367_294 = {S16, 0, &V22367_294_57189, &P22367_293};	/*Property = CssPosition*/
const Value V22367_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22367_295 = {S17, 0, &V22367_295_57192, &P22367_294};	/*Property = CssText*/
const Value V22367_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22367_297 = {S18, 0, &V22367_297_57195, &P22367_295};	/*Property = CssTransforms*/
const Value V22367_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22367_241 = {S19, 0, &V22367_241_55799, &P22367_297};	/*Property = CssTransitions*/
const Value V22367_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22367_298 = {S20, 0, &V22367_298_57206, &P22367_241};	/*Property = CssUI*/
const Value V22367_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22367_300 = {S21, 0, &V22367_300_57216, &P22367_298};	/*Property = DataSet*/
const Value V22367_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22367_301 = {S22, 0, &V22367_301_57218, &P22367_300};	/*Property = DataUrl*/
const Value V22367_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22367_302 = {S23, 0, &V22367_302_57221, &P22367_301};	/*Property = DeviceOrientation*/
const Value V22367_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22367_304 = {S24, 0, &V22367_304_57227, &P22367_302};	/*Property = FileReader*/
const Value V22367_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22367_305 = {S25, 0, &V22367_305_57231, &P22367_304};	/*Property = FileSaver*/
const Value V22367_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22367_306 = {S26, 0, &V22367_306_57237, &P22367_305};	/*Property = FileWriter*/
const Value V22367_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22367_307 = {S27, 0, &V22367_307_57239, &P22367_306};	/*Property = FormData*/
const Value V22367_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22367_309 = {S28, 0, &V22367_309_57246, &P22367_307};	/*Property = Fullscreen*/
const Value V22367_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22367_310 = {S29, 0, &V22367_310_57248, &P22367_309};	/*Property = GeoLocation*/
const Value V22367_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22367_312 = {S30, 0, &V22367_312_57254, &P22367_310};	/*Property = History*/
const Value V22367_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22367_314 = {S31, 0, &V22367_314_57261, &P22367_312};	/*Property = Html5*/
const Value V22367_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22367_313 = {S32, 0, &V22367_313_57258, &P22367_314};	/*Property = Html-Media-Capture*/
const Value V22367_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22367_315 = {S33, 0, &V22367_315_57264, &P22367_313};	/*Property = Iframe*/
const Value V22367_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22367_316 = {S34, 0, &V22367_316_57266, &P22367_315};	/*Property = IndexedDB*/
const Value V22367_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22367_317 = {S35, 0, &V22367_317_57269, &P22367_316};	/*Property = Json*/
const Value V22367_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22367_15 = {S37, 0, &V22367_15_29, &P22367_317};	/*Property = LayoutEngine*/
const Value V22367_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22367_344 = {S38, 0, &V22367_344_57272, &P22367_15};	/*Property = Masking*/
const Value V22367_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22367_320 = {S39, 0, &V22367_320_57290, &P22367_344};	/*Property = PostMessage*/
const Value V22367_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22367_321 = {S40, 0, &V22367_321_57293, &P22367_320};	/*Property = Progress*/
const Value V22367_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22367_322 = {S41, 0, &V22367_322_57296, &P22367_321};	/*Property = Prompts*/
const Value V22367_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22367_323 = {S42, 0, &V22367_323_57299, &P22367_322};	/*Property = Selector*/
const Value V22367_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22367_324 = {S43, 0, &V22367_324_57306, &P22367_323};	/*Property = Svg*/
const Value V22367_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22367_327 = {S44, 0, &V22367_327_57315, &P22367_324};	/*Property = TouchEvents*/
const Value V22367_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22367_328 = {S45, 0, &V22367_328_57318, &P22367_327};	/*Property = Track*/
const Value V22367_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22367_329 = {S46, 0, &V22367_329_57323, &P22367_328};	/*Property = Video*/
const Value V22367_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22367_330 = {S47, 0, &V22367_330_57326, &P22367_329};	/*Property = Viewport*/
const Value V22367_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22367_332 = {S48, 0, &V22367_332_57339, &P22367_330};	/*Property = WebWorkers*/
const Value V22367_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22367_333 = {S49, 0, &V22367_333_57341, &P22367_332};	/*Property = Xhr2*/
const Profile P22367 = {"22367", &P22367_333};	/*Profile ID - 22367*/
const Value V21918_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21918_279 = {S1, 0, &V21918_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21918_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21918_282 = {S3, 0, &V21918_282_57154, &P21918_279};	/*Property = BlobBuilder*/
const Value V21918_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21918_350 = {S4, 0, &V21918_350_57444, &P21918_282};	/*Property = Canvas*/
const Value V21918_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21918_283 = {S5, 0, &V21918_283_57159, &P21918_350};	/*Property = CssBackground*/
const Value V21918_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21918_284 = {S6, 0, &V21918_284_57163, &P21918_283};	/*Property = CssBorderImage*/
const Value V21918_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21918_285 = {S7, 0, &V21918_285_57166, &P21918_284};	/*Property = CssCanvas*/
const Value V21918_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21918_286 = {S8, 0, &V21918_286_57168, &P21918_285};	/*Property = CssColor*/
const Value V21918_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21918_237 = {S9, 0, &V21918_237_55786, &P21918_286};	/*Property = CssColumn*/
const Value V21918_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21918_287 = {S10, 0, &V21918_287_57172, &P21918_237};	/*Property = CssFlexbox*/
const Value V21918_288_57174 = {S2, NULL};	/*Value = True*/
const Property P21918_288 = {S11, 0, &V21918_288_57174, &P21918_287};	/*Property = CssFont*/
const Value V21918_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21918_289 = {S12, 0, &V21918_289_57759, &P21918_288};	/*Property = CssImages*/
const Value V21918_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21918_291 = {S13, 0, &V21918_291_57181, &P21918_289};	/*Property = CssMediaQueries*/
const Value V21918_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21918_292 = {S14, 0, &V21918_292_57183, &P21918_291};	/*Property = CssMinMax*/
const Value V21918_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21918_293 = {S15, 0, &V21918_293_57187, &P21918_292};	/*Property = CssOverflow*/
const Value V21918_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21918_294 = {S16, 0, &V21918_294_57189, &P21918_293};	/*Property = CssPosition*/
const Value V21918_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21918_295 = {S17, 0, &V21918_295_57192, &P21918_294};	/*Property = CssText*/
const Value V21918_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21918_297 = {S18, 0, &V21918_297_57195, &P21918_295};	/*Property = CssTransforms*/
const Value V21918_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21918_241 = {S19, 0, &V21918_241_55799, &P21918_297};	/*Property = CssTransitions*/
const Value V21918_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21918_298 = {S20, 0, &V21918_298_57206, &P21918_241};	/*Property = CssUI*/
const Value V21918_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21918_300 = {S21, 0, &V21918_300_57216, &P21918_298};	/*Property = DataSet*/
const Value V21918_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21918_301 = {S22, 0, &V21918_301_57218, &P21918_300};	/*Property = DataUrl*/
const Value V21918_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21918_302 = {S23, 0, &V21918_302_57222, &P21918_301};	/*Property = DeviceOrientation*/
const Value V21918_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21918_304 = {S24, 0, &V21918_304_57227, &P21918_302};	/*Property = FileReader*/
const Value V21918_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21918_305 = {S25, 0, &V21918_305_57231, &P21918_304};	/*Property = FileSaver*/
const Value V21918_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21918_306 = {S26, 0, &V21918_306_57237, &P21918_305};	/*Property = FileWriter*/
const Value V21918_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21918_307 = {S27, 0, &V21918_307_57239, &P21918_306};	/*Property = FormData*/
const Value V21918_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21918_309 = {S28, 0, &V21918_309_57246, &P21918_307};	/*Property = Fullscreen*/
const Value V21918_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21918_310 = {S29, 0, &V21918_310_57248, &P21918_309};	/*Property = GeoLocation*/
const Value V21918_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21918_312 = {S30, 0, &V21918_312_57254, &P21918_310};	/*Property = History*/
const Value V21918_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21918_314 = {S31, 0, &V21918_314_57261, &P21918_312};	/*Property = Html5*/
const Value V21918_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21918_313 = {S32, 0, &V21918_313_57258, &P21918_314};	/*Property = Html-Media-Capture*/
const Value V21918_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21918_315 = {S33, 0, &V21918_315_57264, &P21918_313};	/*Property = Iframe*/
const Value V21918_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21918_316 = {S34, 0, &V21918_316_57266, &P21918_315};	/*Property = IndexedDB*/
const Value V21918_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21918_317 = {S35, 0, &V21918_317_57269, &P21918_316};	/*Property = Json*/
const Value V21918_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21918_15 = {S37, 0, &V21918_15_28, &P21918_317};	/*Property = LayoutEngine*/
const Value V21918_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21918_344 = {S38, 0, &V21918_344_57272, &P21918_15};	/*Property = Masking*/
const Value V21918_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21918_320 = {S39, 0, &V21918_320_57290, &P21918_344};	/*Property = PostMessage*/
const Value V21918_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21918_321 = {S40, 0, &V21918_321_57293, &P21918_320};	/*Property = Progress*/
const Value V21918_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21918_322 = {S41, 0, &V21918_322_57296, &P21918_321};	/*Property = Prompts*/
const Value V21918_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21918_323 = {S42, 0, &V21918_323_57299, &P21918_322};	/*Property = Selector*/
const Value V21918_324_57305 = {S2, NULL};	/*Value = True*/
const Property P21918_324 = {S43, 0, &V21918_324_57305, &P21918_323};	/*Property = Svg*/
const Value V21918_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21918_327 = {S44, 0, &V21918_327_57315, &P21918_324};	/*Property = TouchEvents*/
const Value V21918_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21918_328 = {S45, 0, &V21918_328_57318, &P21918_327};	/*Property = Track*/
const Value V21918_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21918_329 = {S46, 0, &V21918_329_57323, &P21918_328};	/*Property = Video*/
const Value V21918_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21918_330 = {S47, 0, &V21918_330_57326, &P21918_329};	/*Property = Viewport*/
const Value V21918_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21918_332 = {S48, 0, &V21918_332_57339, &P21918_330};	/*Property = WebWorkers*/
const Value V21918_333_57341 = {S2, NULL};	/*Value = True*/
const Property P21918_333 = {S49, 0, &V21918_333_57341, &P21918_332};	/*Property = Xhr2*/
const Profile P21918 = {"21918", &P21918_333};	/*Profile ID - 21918*/
const Value V23190_1_1 = {S2, NULL};	/*Value = True*/
const Property P23190_1 = {S50, 0, &V23190_1_1, NULL};	/*Property = IsMobile*/
const Value V23190_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23190_5 = {S52, 0, &V23190_5_37569, &P23190_1};	/*Property = ScreenPixelsHeight*/
const Value V23190_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23190_4 = {S54, 0, &V23190_4_5645, &P23190_5};	/*Property = ScreenPixelsWidth*/
const Profile P23190 = {"23190", &P23190_4};	/*Profile ID - 23190*/
const Value V22463_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22463_279 = {S1, 0, &V22463_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22463_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22463_282 = {S3, 0, &V22463_282_57154, &P22463_279};	/*Property = BlobBuilder*/
const Value V22463_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22463_350 = {S4, 0, &V22463_350_57444, &P22463_282};	/*Property = Canvas*/
const Value V22463_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22463_283 = {S5, 0, &V22463_283_57159, &P22463_350};	/*Property = CssBackground*/
const Value V22463_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22463_284 = {S6, 0, &V22463_284_57163, &P22463_283};	/*Property = CssBorderImage*/
const Value V22463_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22463_285 = {S7, 0, &V22463_285_57166, &P22463_284};	/*Property = CssCanvas*/
const Value V22463_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22463_286 = {S8, 0, &V22463_286_57168, &P22463_285};	/*Property = CssColor*/
const Value V22463_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22463_237 = {S9, 0, &V22463_237_55786, &P22463_286};	/*Property = CssColumn*/
const Value V22463_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22463_287 = {S10, 0, &V22463_287_57172, &P22463_237};	/*Property = CssFlexbox*/
const Value V22463_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22463_288 = {S11, 0, &V22463_288_57174, &P22463_287};	/*Property = CssFont*/
const Value V22463_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22463_289 = {S12, 0, &V22463_289_57759, &P22463_288};	/*Property = CssImages*/
const Value V22463_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22463_291 = {S13, 0, &V22463_291_57180, &P22463_289};	/*Property = CssMediaQueries*/
const Value V22463_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22463_292 = {S14, 0, &V22463_292_57183, &P22463_291};	/*Property = CssMinMax*/
const Value V22463_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22463_293 = {S15, 0, &V22463_293_57187, &P22463_292};	/*Property = CssOverflow*/
const Value V22463_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22463_294 = {S16, 0, &V22463_294_57189, &P22463_293};	/*Property = CssPosition*/
const Value V22463_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22463_295 = {S17, 0, &V22463_295_57192, &P22463_294};	/*Property = CssText*/
const Value V22463_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22463_297 = {S18, 0, &V22463_297_57195, &P22463_295};	/*Property = CssTransforms*/
const Value V22463_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22463_241 = {S19, 0, &V22463_241_55799, &P22463_297};	/*Property = CssTransitions*/
const Value V22463_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22463_298 = {S20, 0, &V22463_298_57206, &P22463_241};	/*Property = CssUI*/
const Value V22463_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22463_300 = {S21, 0, &V22463_300_57216, &P22463_298};	/*Property = DataSet*/
const Value V22463_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22463_301 = {S22, 0, &V22463_301_57218, &P22463_300};	/*Property = DataUrl*/
const Value V22463_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22463_302 = {S23, 0, &V22463_302_57222, &P22463_301};	/*Property = DeviceOrientation*/
const Value V22463_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22463_304 = {S24, 0, &V22463_304_57227, &P22463_302};	/*Property = FileReader*/
const Value V22463_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22463_305 = {S25, 0, &V22463_305_57231, &P22463_304};	/*Property = FileSaver*/
const Value V22463_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22463_306 = {S26, 0, &V22463_306_57237, &P22463_305};	/*Property = FileWriter*/
const Value V22463_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22463_307 = {S27, 0, &V22463_307_57239, &P22463_306};	/*Property = FormData*/
const Value V22463_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22463_309 = {S28, 0, &V22463_309_57246, &P22463_307};	/*Property = Fullscreen*/
const Value V22463_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22463_310 = {S29, 0, &V22463_310_57248, &P22463_309};	/*Property = GeoLocation*/
const Value V22463_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22463_312 = {S30, 0, &V22463_312_57254, &P22463_310};	/*Property = History*/
const Value V22463_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22463_314 = {S31, 0, &V22463_314_57261, &P22463_312};	/*Property = Html5*/
const Value V22463_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22463_313 = {S32, 0, &V22463_313_57258, &P22463_314};	/*Property = Html-Media-Capture*/
const Value V22463_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22463_315 = {S33, 0, &V22463_315_57264, &P22463_313};	/*Property = Iframe*/
const Value V22463_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22463_316 = {S34, 0, &V22463_316_57266, &P22463_315};	/*Property = IndexedDB*/
const Value V22463_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22463_317 = {S35, 0, &V22463_317_57269, &P22463_316};	/*Property = Json*/
const Value V22463_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22463_15 = {S37, 0, &V22463_15_28, &P22463_317};	/*Property = LayoutEngine*/
const Value V22463_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22463_344 = {S38, 0, &V22463_344_57272, &P22463_15};	/*Property = Masking*/
const Value V22463_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22463_320 = {S39, 0, &V22463_320_57290, &P22463_344};	/*Property = PostMessage*/
const Value V22463_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22463_321 = {S40, 0, &V22463_321_57293, &P22463_320};	/*Property = Progress*/
const Value V22463_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22463_322 = {S41, 0, &V22463_322_57296, &P22463_321};	/*Property = Prompts*/
const Value V22463_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22463_323 = {S42, 0, &V22463_323_57299, &P22463_322};	/*Property = Selector*/
const Value V22463_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22463_324 = {S43, 0, &V22463_324_57306, &P22463_323};	/*Property = Svg*/
const Value V22463_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22463_327 = {S44, 0, &V22463_327_57315, &P22463_324};	/*Property = TouchEvents*/
const Value V22463_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22463_328 = {S45, 0, &V22463_328_57318, &P22463_327};	/*Property = Track*/
const Value V22463_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22463_329 = {S46, 0, &V22463_329_57323, &P22463_328};	/*Property = Video*/
const Value V22463_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22463_330 = {S47, 0, &V22463_330_57326, &P22463_329};	/*Property = Viewport*/
const Value V22463_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22463_332 = {S48, 0, &V22463_332_57339, &P22463_330};	/*Property = WebWorkers*/
const Value V22463_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22463_333 = {S49, 0, &V22463_333_57342, &P22463_332};	/*Property = Xhr2*/
const Profile P22463 = {"22463", &P22463_333};	/*Profile ID - 22463*/
const Value V23546_1_1 = {S2, NULL};	/*Value = True*/
const Property P23546_1 = {S50, 0, &V23546_1_1, NULL};	/*Property = IsMobile*/
const Value V23546_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23546_5 = {S52, 0, &V23546_5_1227, &P23546_1};	/*Property = ScreenPixelsHeight*/
const Value V23546_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23546_4 = {S54, 0, &V23546_4_1229, &P23546_5};	/*Property = ScreenPixelsWidth*/
const Profile P23546 = {"23546", &P23546_4};	/*Profile ID - 23546*/
const Value V23473_1_1 = {S2, NULL};	/*Value = True*/
const Property P23473_1 = {S50, 0, &V23473_1_1, NULL};	/*Property = IsMobile*/
const Value V23473_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23473_5 = {S52, 0, &V23473_5_37569, &P23473_1};	/*Property = ScreenPixelsHeight*/
const Value V23473_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23473_4 = {S54, 0, &V23473_4_5645, &P23473_5};	/*Property = ScreenPixelsWidth*/
const Profile P23473 = {"23473", &P23473_4};	/*Profile ID - 23473*/
const Value V19860_1_1 = {S2, NULL};	/*Value = True*/
const Property P19860_1 = {S50, 0, &V19860_1_1, NULL};	/*Property = IsMobile*/
const Value V19860_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19860_5 = {S52, 0, &V19860_5_1227, &P19860_1};	/*Property = ScreenPixelsHeight*/
const Value V19860_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19860_4 = {S54, 0, &V19860_4_1229, &P19860_5};	/*Property = ScreenPixelsWidth*/
const Profile P19860 = {"19860", &P19860_4};	/*Profile ID - 19860*/
const Value V22167_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22167_279 = {S1, 0, &V22167_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22167_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22167_282 = {S3, 0, &V22167_282_57154, &P22167_279};	/*Property = BlobBuilder*/
const Value V22167_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22167_350 = {S4, 0, &V22167_350_57444, &P22167_282};	/*Property = Canvas*/
const Value V22167_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22167_283 = {S5, 0, &V22167_283_57159, &P22167_350};	/*Property = CssBackground*/
const Value V22167_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22167_284 = {S6, 0, &V22167_284_57163, &P22167_283};	/*Property = CssBorderImage*/
const Value V22167_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22167_285 = {S7, 0, &V22167_285_57166, &P22167_284};	/*Property = CssCanvas*/
const Value V22167_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22167_286 = {S8, 0, &V22167_286_57168, &P22167_285};	/*Property = CssColor*/
const Value V22167_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22167_237 = {S9, 0, &V22167_237_55786, &P22167_286};	/*Property = CssColumn*/
const Value V22167_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22167_287 = {S10, 0, &V22167_287_57172, &P22167_237};	/*Property = CssFlexbox*/
const Value V22167_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22167_288 = {S11, 0, &V22167_288_57174, &P22167_287};	/*Property = CssFont*/
const Value V22167_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22167_289 = {S12, 0, &V22167_289_57759, &P22167_288};	/*Property = CssImages*/
const Value V22167_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22167_291 = {S13, 0, &V22167_291_57180, &P22167_289};	/*Property = CssMediaQueries*/
const Value V22167_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22167_292 = {S14, 0, &V22167_292_57183, &P22167_291};	/*Property = CssMinMax*/
const Value V22167_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22167_293 = {S15, 0, &V22167_293_57187, &P22167_292};	/*Property = CssOverflow*/
const Value V22167_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22167_294 = {S16, 0, &V22167_294_57189, &P22167_293};	/*Property = CssPosition*/
const Value V22167_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22167_295 = {S17, 0, &V22167_295_57192, &P22167_294};	/*Property = CssText*/
const Value V22167_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22167_297 = {S18, 0, &V22167_297_57195, &P22167_295};	/*Property = CssTransforms*/
const Value V22167_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22167_241 = {S19, 0, &V22167_241_55799, &P22167_297};	/*Property = CssTransitions*/
const Value V22167_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22167_298 = {S20, 0, &V22167_298_57206, &P22167_241};	/*Property = CssUI*/
const Value V22167_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22167_300 = {S21, 0, &V22167_300_57216, &P22167_298};	/*Property = DataSet*/
const Value V22167_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22167_301 = {S22, 0, &V22167_301_57218, &P22167_300};	/*Property = DataUrl*/
const Value V22167_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22167_302 = {S23, 0, &V22167_302_57221, &P22167_301};	/*Property = DeviceOrientation*/
const Value V22167_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22167_304 = {S24, 0, &V22167_304_57227, &P22167_302};	/*Property = FileReader*/
const Value V22167_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22167_305 = {S25, 0, &V22167_305_57231, &P22167_304};	/*Property = FileSaver*/
const Value V22167_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22167_306 = {S26, 0, &V22167_306_57237, &P22167_305};	/*Property = FileWriter*/
const Value V22167_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22167_307 = {S27, 0, &V22167_307_57239, &P22167_306};	/*Property = FormData*/
const Value V22167_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22167_309 = {S28, 0, &V22167_309_57246, &P22167_307};	/*Property = Fullscreen*/
const Value V22167_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22167_310 = {S29, 0, &V22167_310_57248, &P22167_309};	/*Property = GeoLocation*/
const Value V22167_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22167_312 = {S30, 0, &V22167_312_57254, &P22167_310};	/*Property = History*/
const Value V22167_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22167_314 = {S31, 0, &V22167_314_57261, &P22167_312};	/*Property = Html5*/
const Value V22167_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22167_313 = {S32, 0, &V22167_313_57258, &P22167_314};	/*Property = Html-Media-Capture*/
const Value V22167_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22167_315 = {S33, 0, &V22167_315_57264, &P22167_313};	/*Property = Iframe*/
const Value V22167_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22167_316 = {S34, 0, &V22167_316_57266, &P22167_315};	/*Property = IndexedDB*/
const Value V22167_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22167_317 = {S35, 0, &V22167_317_57269, &P22167_316};	/*Property = Json*/
const Value V22167_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22167_15 = {S37, 0, &V22167_15_29, &P22167_317};	/*Property = LayoutEngine*/
const Value V22167_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22167_344 = {S38, 0, &V22167_344_57272, &P22167_15};	/*Property = Masking*/
const Value V22167_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22167_320 = {S39, 0, &V22167_320_57290, &P22167_344};	/*Property = PostMessage*/
const Value V22167_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22167_321 = {S40, 0, &V22167_321_57293, &P22167_320};	/*Property = Progress*/
const Value V22167_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22167_322 = {S41, 0, &V22167_322_57296, &P22167_321};	/*Property = Prompts*/
const Value V22167_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22167_323 = {S42, 0, &V22167_323_57299, &P22167_322};	/*Property = Selector*/
const Value V22167_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22167_324 = {S43, 0, &V22167_324_57306, &P22167_323};	/*Property = Svg*/
const Value V22167_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22167_327 = {S44, 0, &V22167_327_57315, &P22167_324};	/*Property = TouchEvents*/
const Value V22167_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22167_328 = {S45, 0, &V22167_328_57318, &P22167_327};	/*Property = Track*/
const Value V22167_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22167_329 = {S46, 0, &V22167_329_57323, &P22167_328};	/*Property = Video*/
const Value V22167_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22167_330 = {S47, 0, &V22167_330_57326, &P22167_329};	/*Property = Viewport*/
const Value V22167_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22167_332 = {S48, 0, &V22167_332_57338, &P22167_330};	/*Property = WebWorkers*/
const Value V22167_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22167_333 = {S49, 0, &V22167_333_57341, &P22167_332};	/*Property = Xhr2*/
const Profile P22167 = {"22167", &P22167_333};	/*Profile ID - 22167*/
const Value V19810_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19810_279 = {S1, 0, &V19810_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19810_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19810_282 = {S3, 0, &V19810_282_57154, &P19810_279};	/*Property = BlobBuilder*/
const Value V19810_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19810_350 = {S4, 0, &V19810_350_57444, &P19810_282};	/*Property = Canvas*/
const Value V19810_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19810_283 = {S5, 0, &V19810_283_57159, &P19810_350};	/*Property = CssBackground*/
const Value V19810_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19810_284 = {S6, 0, &V19810_284_57163, &P19810_283};	/*Property = CssBorderImage*/
const Value V19810_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19810_285 = {S7, 0, &V19810_285_57166, &P19810_284};	/*Property = CssCanvas*/
const Value V19810_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19810_286 = {S8, 0, &V19810_286_57168, &P19810_285};	/*Property = CssColor*/
const Value V19810_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19810_237 = {S9, 0, &V19810_237_55786, &P19810_286};	/*Property = CssColumn*/
const Value V19810_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19810_287 = {S10, 0, &V19810_287_57172, &P19810_237};	/*Property = CssFlexbox*/
const Value V19810_288_57175 = {S0, NULL};	/*Value = False*/
const Property P19810_288 = {S11, 0, &V19810_288_57175, &P19810_287};	/*Property = CssFont*/
const Value V19810_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19810_289 = {S12, 0, &V19810_289_57759, &P19810_288};	/*Property = CssImages*/
const Value V19810_291_57181 = {S0, NULL};	/*Value = False*/
const Property P19810_291 = {S13, 0, &V19810_291_57181, &P19810_289};	/*Property = CssMediaQueries*/
const Value V19810_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19810_292 = {S14, 0, &V19810_292_57183, &P19810_291};	/*Property = CssMinMax*/
const Value V19810_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19810_293 = {S15, 0, &V19810_293_57187, &P19810_292};	/*Property = CssOverflow*/
const Value V19810_294_57190 = {S0, NULL};	/*Value = False*/
const Property P19810_294 = {S16, 0, &V19810_294_57190, &P19810_293};	/*Property = CssPosition*/
const Value V19810_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19810_295 = {S17, 0, &V19810_295_57192, &P19810_294};	/*Property = CssText*/
const Value V19810_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19810_297 = {S18, 0, &V19810_297_57195, &P19810_295};	/*Property = CssTransforms*/
const Value V19810_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19810_241 = {S19, 0, &V19810_241_55799, &P19810_297};	/*Property = CssTransitions*/
const Value V19810_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19810_298 = {S20, 0, &V19810_298_57206, &P19810_241};	/*Property = CssUI*/
const Value V19810_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19810_300 = {S21, 0, &V19810_300_57216, &P19810_298};	/*Property = DataSet*/
const Value V19810_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19810_301 = {S22, 0, &V19810_301_57218, &P19810_300};	/*Property = DataUrl*/
const Value V19810_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19810_302 = {S23, 0, &V19810_302_57222, &P19810_301};	/*Property = DeviceOrientation*/
const Value V19810_304_57228 = {S0, NULL};	/*Value = False*/
const Property P19810_304 = {S24, 0, &V19810_304_57228, &P19810_302};	/*Property = FileReader*/
const Value V19810_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19810_305 = {S25, 0, &V19810_305_57231, &P19810_304};	/*Property = FileSaver*/
const Value V19810_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19810_306 = {S26, 0, &V19810_306_57237, &P19810_305};	/*Property = FileWriter*/
const Value V19810_307_57240 = {S0, NULL};	/*Value = False*/
const Property P19810_307 = {S27, 0, &V19810_307_57240, &P19810_306};	/*Property = FormData*/
const Value V19810_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19810_309 = {S28, 0, &V19810_309_57246, &P19810_307};	/*Property = Fullscreen*/
const Value V19810_310_57249 = {S0, NULL};	/*Value = False*/
const Property P19810_310 = {S29, 0, &V19810_310_57249, &P19810_309};	/*Property = GeoLocation*/
const Value V19810_312_57255 = {S0, NULL};	/*Value = False*/
const Property P19810_312 = {S30, 0, &V19810_312_57255, &P19810_310};	/*Property = History*/
const Value V19810_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19810_314 = {S31, 0, &V19810_314_57261, &P19810_312};	/*Property = Html5*/
const Value V19810_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19810_313 = {S32, 0, &V19810_313_57258, &P19810_314};	/*Property = Html-Media-Capture*/
const Value V19810_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19810_315 = {S33, 0, &V19810_315_57264, &P19810_313};	/*Property = Iframe*/
const Value V19810_316_57267 = {S0, NULL};	/*Value = False*/
const Property P19810_316 = {S34, 0, &V19810_316_57267, &P19810_315};	/*Property = IndexedDB*/
const Value V19810_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19810_317 = {S35, 0, &V19810_317_57269, &P19810_316};	/*Property = Json*/
const Value V19810_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19810_15 = {S37, 0, &V19810_15_28, &P19810_317};	/*Property = LayoutEngine*/
const Value V19810_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19810_344 = {S38, 0, &V19810_344_57272, &P19810_15};	/*Property = Masking*/
const Value V19810_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19810_320 = {S39, 0, &V19810_320_57290, &P19810_344};	/*Property = PostMessage*/
const Value V19810_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19810_321 = {S40, 0, &V19810_321_57293, &P19810_320};	/*Property = Progress*/
const Value V19810_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19810_322 = {S41, 0, &V19810_322_57296, &P19810_321};	/*Property = Prompts*/
const Value V19810_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19810_323 = {S42, 0, &V19810_323_57299, &P19810_322};	/*Property = Selector*/
const Value V19810_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19810_324 = {S43, 0, &V19810_324_57306, &P19810_323};	/*Property = Svg*/
const Value V19810_327_57315 = {S0, NULL};	/*Value = False*/
const Property P19810_327 = {S44, 0, &V19810_327_57315, &P19810_324};	/*Property = TouchEvents*/
const Value V19810_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19810_328 = {S45, 0, &V19810_328_57318, &P19810_327};	/*Property = Track*/
const Value V19810_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19810_329 = {S46, 0, &V19810_329_57323, &P19810_328};	/*Property = Video*/
const Value V19810_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19810_330 = {S47, 0, &V19810_330_57326, &P19810_329};	/*Property = Viewport*/
const Value V19810_332_57339 = {S0, NULL};	/*Value = False*/
const Property P19810_332 = {S48, 0, &V19810_332_57339, &P19810_330};	/*Property = WebWorkers*/
const Value V19810_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19810_333 = {S49, 0, &V19810_333_57342, &P19810_332};	/*Property = Xhr2*/
const Profile P19810 = {"19810", &P19810_333};	/*Profile ID - 19810*/
const Value V23589_1_2 = {S0, NULL};	/*Value = False*/
const Property P23589_1 = {S50, 0, &V23589_1_2, NULL};	/*Property = IsMobile*/
const Value V23589_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23589_5 = {S52, 0, &V23589_5_37, &P23589_1};	/*Property = ScreenPixelsHeight*/
const Value V23589_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23589_4 = {S54, 0, &V23589_4_36, &P23589_5};	/*Property = ScreenPixelsWidth*/
const Profile P23589 = {"23589", &P23589_4};	/*Profile ID - 23589*/
const Value V20414_1_1 = {S2, NULL};	/*Value = True*/
const Property P20414_1 = {S50, 0, &V20414_1_1, NULL};	/*Property = IsMobile*/
const Value V20414_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20414_5 = {S52, 0, &V20414_5_9981, &P20414_1};	/*Property = ScreenPixelsHeight*/
const Value V20414_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20414_4 = {S54, 0, &V20414_4_2336, &P20414_5};	/*Property = ScreenPixelsWidth*/
const Profile P20414 = {"20414", &P20414_4};	/*Profile ID - 20414*/
const Value V21269_1_1 = {S2, NULL};	/*Value = True*/
const Property P21269_1 = {S50, 0, &V21269_1_1, NULL};	/*Property = IsMobile*/
const Value V21269_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21269_5 = {S52, 0, &V21269_5_1222, &P21269_1};	/*Property = ScreenPixelsHeight*/
const Value V21269_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21269_4 = {S54, 0, &V21269_4_1224, &P21269_5};	/*Property = ScreenPixelsWidth*/
const Profile P21269 = {"21269", &P21269_4};	/*Profile ID - 21269*/
const Value V20268_1_1 = {S2, NULL};	/*Value = True*/
const Property P20268_1 = {S50, 0, &V20268_1_1, NULL};	/*Property = IsMobile*/
const Value V20268_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20268_5 = {S52, 0, &V20268_5_1227, &P20268_1};	/*Property = ScreenPixelsHeight*/
const Value V20268_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20268_4 = {S54, 0, &V20268_4_1229, &P20268_5};	/*Property = ScreenPixelsWidth*/
const Profile P20268 = {"20268", &P20268_4};	/*Profile ID - 20268*/
const Profile P19823 = {"19823", NULL};	/*Profile ID - 19823*/
const Value V18680_1_1 = {S2, NULL};	/*Value = True*/
const Property P18680_1 = {S50, 0, &V18680_1_1, NULL};	/*Property = IsMobile*/
const Value V18680_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P18680_5 = {S52, 0, &V18680_5_59138, &P18680_1};	/*Property = ScreenPixelsHeight*/
const Value V18680_4_52971 = {S122, NULL};	/*Value = 1200*/
const Property P18680_4 = {S54, 0, &V18680_4_52971, &P18680_5};	/*Property = ScreenPixelsWidth*/
const Profile P18680 = {"18680", &P18680_4};	/*Profile ID - 18680*/
const Value V19815_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19815_279 = {S1, 0, &V19815_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19815_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19815_282 = {S3, 0, &V19815_282_57154, &P19815_279};	/*Property = BlobBuilder*/
const Value V19815_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19815_350 = {S4, 0, &V19815_350_57444, &P19815_282};	/*Property = Canvas*/
const Value V19815_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19815_283 = {S5, 0, &V19815_283_57159, &P19815_350};	/*Property = CssBackground*/
const Value V19815_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19815_284 = {S6, 0, &V19815_284_57163, &P19815_283};	/*Property = CssBorderImage*/
const Value V19815_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19815_285 = {S7, 0, &V19815_285_57166, &P19815_284};	/*Property = CssCanvas*/
const Value V19815_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19815_286 = {S8, 0, &V19815_286_57168, &P19815_285};	/*Property = CssColor*/
const Value V19815_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19815_237 = {S9, 0, &V19815_237_55786, &P19815_286};	/*Property = CssColumn*/
const Value V19815_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19815_287 = {S10, 0, &V19815_287_57172, &P19815_237};	/*Property = CssFlexbox*/
const Value V19815_288_57174 = {S2, NULL};	/*Value = True*/
const Property P19815_288 = {S11, 0, &V19815_288_57174, &P19815_287};	/*Property = CssFont*/
const Value V19815_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19815_289 = {S12, 0, &V19815_289_57759, &P19815_288};	/*Property = CssImages*/
const Value V19815_291_57180 = {S2, NULL};	/*Value = True*/
const Property P19815_291 = {S13, 0, &V19815_291_57180, &P19815_289};	/*Property = CssMediaQueries*/
const Value V19815_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19815_292 = {S14, 0, &V19815_292_57183, &P19815_291};	/*Property = CssMinMax*/
const Value V19815_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19815_293 = {S15, 0, &V19815_293_57187, &P19815_292};	/*Property = CssOverflow*/
const Value V19815_294_57189 = {S2, NULL};	/*Value = True*/
const Property P19815_294 = {S16, 0, &V19815_294_57189, &P19815_293};	/*Property = CssPosition*/
const Value V19815_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19815_295 = {S17, 0, &V19815_295_57192, &P19815_294};	/*Property = CssText*/
const Value V19815_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19815_297 = {S18, 0, &V19815_297_57195, &P19815_295};	/*Property = CssTransforms*/
const Value V19815_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19815_241 = {S19, 0, &V19815_241_55799, &P19815_297};	/*Property = CssTransitions*/
const Value V19815_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19815_298 = {S20, 0, &V19815_298_57206, &P19815_241};	/*Property = CssUI*/
const Value V19815_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19815_300 = {S21, 0, &V19815_300_57216, &P19815_298};	/*Property = DataSet*/
const Value V19815_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19815_301 = {S22, 0, &V19815_301_57218, &P19815_300};	/*Property = DataUrl*/
const Value V19815_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19815_302 = {S23, 0, &V19815_302_57222, &P19815_301};	/*Property = DeviceOrientation*/
const Value V19815_304_57227 = {S2, NULL};	/*Value = True*/
const Property P19815_304 = {S24, 0, &V19815_304_57227, &P19815_302};	/*Property = FileReader*/
const Value V19815_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19815_305 = {S25, 0, &V19815_305_57231, &P19815_304};	/*Property = FileSaver*/
const Value V19815_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19815_306 = {S26, 0, &V19815_306_57237, &P19815_305};	/*Property = FileWriter*/
const Value V19815_307_57239 = {S2, NULL};	/*Value = True*/
const Property P19815_307 = {S27, 0, &V19815_307_57239, &P19815_306};	/*Property = FormData*/
const Value V19815_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19815_309 = {S28, 0, &V19815_309_57246, &P19815_307};	/*Property = Fullscreen*/
const Value V19815_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19815_310 = {S29, 0, &V19815_310_57248, &P19815_309};	/*Property = GeoLocation*/
const Value V19815_312_57254 = {S2, NULL};	/*Value = True*/
const Property P19815_312 = {S30, 0, &V19815_312_57254, &P19815_310};	/*Property = History*/
const Value V19815_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19815_314 = {S31, 0, &V19815_314_57261, &P19815_312};	/*Property = Html5*/
const Value V19815_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19815_313 = {S32, 0, &V19815_313_57258, &P19815_314};	/*Property = Html-Media-Capture*/
const Value V19815_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19815_315 = {S33, 0, &V19815_315_57264, &P19815_313};	/*Property = Iframe*/
const Value V19815_316_57266 = {S2, NULL};	/*Value = True*/
const Property P19815_316 = {S34, 0, &V19815_316_57266, &P19815_315};	/*Property = IndexedDB*/
const Value V19815_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19815_317 = {S35, 0, &V19815_317_57269, &P19815_316};	/*Property = Json*/
const Value V19815_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19815_15 = {S37, 0, &V19815_15_28, &P19815_317};	/*Property = LayoutEngine*/
const Value V19815_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19815_344 = {S38, 0, &V19815_344_57272, &P19815_15};	/*Property = Masking*/
const Value V19815_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19815_320 = {S39, 0, &V19815_320_57290, &P19815_344};	/*Property = PostMessage*/
const Value V19815_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19815_321 = {S40, 0, &V19815_321_57293, &P19815_320};	/*Property = Progress*/
const Value V19815_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19815_322 = {S41, 0, &V19815_322_57296, &P19815_321};	/*Property = Prompts*/
const Value V19815_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19815_323 = {S42, 0, &V19815_323_57299, &P19815_322};	/*Property = Selector*/
const Value V19815_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19815_324 = {S43, 0, &V19815_324_57306, &P19815_323};	/*Property = Svg*/
const Value V19815_327_57314 = {S2, NULL};	/*Value = True*/
const Property P19815_327 = {S44, 0, &V19815_327_57314, &P19815_324};	/*Property = TouchEvents*/
const Value V19815_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19815_328 = {S45, 0, &V19815_328_57318, &P19815_327};	/*Property = Track*/
const Value V19815_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19815_329 = {S46, 0, &V19815_329_57323, &P19815_328};	/*Property = Video*/
const Value V19815_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19815_330 = {S47, 0, &V19815_330_57326, &P19815_329};	/*Property = Viewport*/
const Value V19815_332_57339 = {S0, NULL};	/*Value = False*/
const Property P19815_332 = {S48, 0, &V19815_332_57339, &P19815_330};	/*Property = WebWorkers*/
const Value V19815_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19815_333 = {S49, 0, &V19815_333_57342, &P19815_332};	/*Property = Xhr2*/
const Profile P19815 = {"19815", &P19815_333};	/*Profile ID - 19815*/
const Value V23535_1_1 = {S2, NULL};	/*Value = True*/
const Property P23535_1 = {S50, 0, &V23535_1_1, NULL};	/*Property = IsMobile*/
const Value V23535_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23535_5 = {S52, 0, &V23535_5_1403, &P23535_1};	/*Property = ScreenPixelsHeight*/
const Value V23535_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23535_4 = {S54, 0, &V23535_4_1404, &P23535_5};	/*Property = ScreenPixelsWidth*/
const Profile P23535 = {"23535", &P23535_4};	/*Profile ID - 23535*/
const Value V21865_1_1 = {S2, NULL};	/*Value = True*/
const Property P21865_1 = {S50, 0, &V21865_1_1, NULL};	/*Property = IsMobile*/
const Value V21865_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21865_5 = {S52, 0, &V21865_5_1227, &P21865_1};	/*Property = ScreenPixelsHeight*/
const Value V21865_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21865_4 = {S54, 0, &V21865_4_1229, &P21865_5};	/*Property = ScreenPixelsWidth*/
const Profile P21865 = {"21865", &P21865_4};	/*Profile ID - 21865*/
const Value V20898_1_1 = {S2, NULL};	/*Value = True*/
const Property P20898_1 = {S50, 0, &V20898_1_1, NULL};	/*Property = IsMobile*/
const Value V20898_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20898_5 = {S52, 0, &V20898_5_1277, &P20898_1};	/*Property = ScreenPixelsHeight*/
const Value V20898_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20898_4 = {S54, 0, &V20898_4_1279, &P20898_5};	/*Property = ScreenPixelsWidth*/
const Profile P20898 = {"20898", &P20898_4};	/*Profile ID - 20898*/
const Value V20025_1_1 = {S2, NULL};	/*Value = True*/
const Property P20025_1 = {S50, 0, &V20025_1_1, NULL};	/*Property = IsMobile*/
const Value V20025_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20025_5 = {S52, 0, &V20025_5_1222, &P20025_1};	/*Property = ScreenPixelsHeight*/
const Value V20025_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20025_4 = {S54, 0, &V20025_4_1224, &P20025_5};	/*Property = ScreenPixelsWidth*/
const Profile P20025 = {"20025", &P20025_4};	/*Profile ID - 20025*/
const Profile P22849 = {"22849", NULL};	/*Profile ID - 22849*/
const Value V23102_1_1 = {S2, NULL};	/*Value = True*/
const Property P23102_1 = {S50, 0, &V23102_1_1, NULL};	/*Property = IsMobile*/
const Value V23102_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23102_5 = {S52, 0, &V23102_5_1227, &P23102_1};	/*Property = ScreenPixelsHeight*/
const Value V23102_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23102_4 = {S54, 0, &V23102_4_1229, &P23102_5};	/*Property = ScreenPixelsWidth*/
const Profile P23102 = {"23102", &P23102_4};	/*Profile ID - 23102*/
const Value V20834_1_1 = {S2, NULL};	/*Value = True*/
const Property P20834_1 = {S50, 0, &V20834_1_1, NULL};	/*Property = IsMobile*/
const Value V20834_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20834_5 = {S52, 0, &V20834_5_1458, &P20834_1};	/*Property = ScreenPixelsHeight*/
const Value V20834_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20834_4 = {S54, 0, &V20834_4_1460, &P20834_5};	/*Property = ScreenPixelsWidth*/
const Profile P20834 = {"20834", &P20834_4};	/*Profile ID - 20834*/
const Value V20715_1_1 = {S2, NULL};	/*Value = True*/
const Property P20715_1 = {S50, 0, &V20715_1_1, NULL};	/*Property = IsMobile*/
const Value V20715_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20715_5 = {S52, 0, &V20715_5_1458, &P20715_1};	/*Property = ScreenPixelsHeight*/
const Value V20715_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20715_4 = {S54, 0, &V20715_4_37617, &P20715_5};	/*Property = ScreenPixelsWidth*/
const Profile P20715 = {"20715", &P20715_4};	/*Profile ID - 20715*/
const Value V20540_1_1 = {S2, NULL};	/*Value = True*/
const Property P20540_1 = {S50, 0, &V20540_1_1, NULL};	/*Property = IsMobile*/
const Value V20540_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20540_5 = {S52, 0, &V20540_5_1227, &P20540_1};	/*Property = ScreenPixelsHeight*/
const Value V20540_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20540_4 = {S54, 0, &V20540_4_1229, &P20540_5};	/*Property = ScreenPixelsWidth*/
const Profile P20540 = {"20540", &P20540_4};	/*Profile ID - 20540*/
const Value V24536_1_1 = {S2, NULL};	/*Value = True*/
const Property P24536_1 = {S50, 0, &V24536_1_1, NULL};	/*Property = IsMobile*/
const Value V24536_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P24536_5 = {S52, 0, &V24536_5_1832, &P24536_1};	/*Property = ScreenPixelsHeight*/
const Value V24536_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P24536_4 = {S54, 0, &V24536_4_5645, &P24536_5};	/*Property = ScreenPixelsWidth*/
const Profile P24536 = {"24536", &P24536_4};	/*Profile ID - 24536*/
const Value V20256_1_1 = {S2, NULL};	/*Value = True*/
const Property P20256_1 = {S50, 0, &V20256_1_1, NULL};	/*Property = IsMobile*/
const Value V20256_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20256_5 = {S52, 0, &V20256_5_1458, &P20256_1};	/*Property = ScreenPixelsHeight*/
const Value V20256_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20256_4 = {S54, 0, &V20256_4_1460, &P20256_5};	/*Property = ScreenPixelsWidth*/
const Profile P20256 = {"20256", &P20256_4};	/*Profile ID - 20256*/
const Value V19957_1_1 = {S2, NULL};	/*Value = True*/
const Property P19957_1 = {S50, 0, &V19957_1_1, NULL};	/*Property = IsMobile*/
const Value V19957_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P19957_5 = {S52, 0, &V19957_5_1293, &P19957_1};	/*Property = ScreenPixelsHeight*/
const Value V19957_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P19957_4 = {S54, 0, &V19957_4_1296, &P19957_5};	/*Property = ScreenPixelsWidth*/
const Profile P19957 = {"19957", &P19957_4};	/*Profile ID - 19957*/
const Value V23488_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_279 = {S1, 0, &V23488_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23488_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_282 = {S3, 0, &V23488_282_57155, &P23488_279};	/*Property = BlobBuilder*/
const Value V23488_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_350 = {S4, 0, &V23488_350_57446, &P23488_282};	/*Property = Canvas*/
const Value V23488_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_283 = {S5, 0, &V23488_283_57161, &P23488_350};	/*Property = CssBackground*/
const Value V23488_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_284 = {S6, 0, &V23488_284_57164, &P23488_283};	/*Property = CssBorderImage*/
const Value V23488_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_285 = {S7, 0, &V23488_285_57167, &P23488_284};	/*Property = CssCanvas*/
const Value V23488_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_286 = {S8, 0, &V23488_286_57170, &P23488_285};	/*Property = CssColor*/
const Value V23488_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_237 = {S9, 0, &V23488_237_55786, &P23488_286};	/*Property = CssColumn*/
const Value V23488_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_287 = {S10, 0, &V23488_287_57173, &P23488_237};	/*Property = CssFlexbox*/
const Value V23488_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_288 = {S11, 0, &V23488_288_57176, &P23488_287};	/*Property = CssFont*/
const Value V23488_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_289 = {S12, 0, &V23488_289_86899, &P23488_288};	/*Property = CssImages*/
const Value V23488_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_291 = {S13, 0, &V23488_291_57182, &P23488_289};	/*Property = CssMediaQueries*/
const Value V23488_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_292 = {S14, 0, &V23488_292_57185, &P23488_291};	/*Property = CssMinMax*/
const Value V23488_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_293 = {S15, 0, &V23488_293_57188, &P23488_292};	/*Property = CssOverflow*/
const Value V23488_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_294 = {S16, 0, &V23488_294_57191, &P23488_293};	/*Property = CssPosition*/
const Value V23488_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_295 = {S17, 0, &V23488_295_57194, &P23488_294};	/*Property = CssText*/
const Value V23488_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_297 = {S18, 0, &V23488_297_57197, &P23488_295};	/*Property = CssTransforms*/
const Value V23488_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_241 = {S19, 0, &V23488_241_55801, &P23488_297};	/*Property = CssTransitions*/
const Value V23488_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_298 = {S20, 0, &V23488_298_57208, &P23488_241};	/*Property = CssUI*/
const Value V23488_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_300 = {S21, 0, &V23488_300_57217, &P23488_298};	/*Property = DataSet*/
const Value V23488_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_301 = {S22, 0, &V23488_301_57220, &P23488_300};	/*Property = DataUrl*/
const Value V23488_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_302 = {S23, 0, &V23488_302_57223, &P23488_301};	/*Property = DeviceOrientation*/
const Value V23488_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_304 = {S24, 0, &V23488_304_57229, &P23488_302};	/*Property = FileReader*/
const Value V23488_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_305 = {S25, 0, &V23488_305_57232, &P23488_304};	/*Property = FileSaver*/
const Value V23488_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_306 = {S26, 0, &V23488_306_57238, &P23488_305};	/*Property = FileWriter*/
const Value V23488_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_307 = {S27, 0, &V23488_307_57241, &P23488_306};	/*Property = FormData*/
const Value V23488_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_309 = {S28, 0, &V23488_309_57247, &P23488_307};	/*Property = Fullscreen*/
const Value V23488_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_310 = {S29, 0, &V23488_310_57250, &P23488_309};	/*Property = GeoLocation*/
const Value V23488_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_312 = {S30, 0, &V23488_312_57256, &P23488_310};	/*Property = History*/
const Value V23488_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_314 = {S31, 0, &V23488_314_57262, &P23488_312};	/*Property = Html5*/
const Value V23488_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_313 = {S32, 0, &V23488_313_57259, &P23488_314};	/*Property = Html-Media-Capture*/
const Value V23488_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_315 = {S33, 0, &V23488_315_57265, &P23488_313};	/*Property = Iframe*/
const Value V23488_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_316 = {S34, 0, &V23488_316_57268, &P23488_315};	/*Property = IndexedDB*/
const Value V23488_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_317 = {S35, 0, &V23488_317_57271, &P23488_316};	/*Property = Json*/
const Value V23488_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P23488_15 = {S37, 0, &V23488_15_29, &P23488_317};	/*Property = LayoutEngine*/
const Value V23488_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_344 = {S38, 0, &V23488_344_57274, &P23488_15};	/*Property = Masking*/
const Value V23488_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_320 = {S39, 0, &V23488_320_57292, &P23488_344};	/*Property = PostMessage*/
const Value V23488_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_321 = {S40, 0, &V23488_321_57295, &P23488_320};	/*Property = Progress*/
const Value V23488_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_322 = {S41, 0, &V23488_322_57298, &P23488_321};	/*Property = Prompts*/
const Value V23488_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_323 = {S42, 0, &V23488_323_57301, &P23488_322};	/*Property = Selector*/
const Value V23488_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_324 = {S43, 0, &V23488_324_57307, &P23488_323};	/*Property = Svg*/
const Value V23488_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_327 = {S44, 0, &V23488_327_57316, &P23488_324};	/*Property = TouchEvents*/
const Value V23488_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_328 = {S45, 0, &V23488_328_57319, &P23488_327};	/*Property = Track*/
const Value V23488_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_329 = {S46, 0, &V23488_329_57325, &P23488_328};	/*Property = Video*/
const Value V23488_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_330 = {S47, 0, &V23488_330_57328, &P23488_329};	/*Property = Viewport*/
const Value V23488_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_332 = {S48, 0, &V23488_332_57340, &P23488_330};	/*Property = WebWorkers*/
const Value V23488_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23488_333 = {S49, 0, &V23488_333_57343, &P23488_332};	/*Property = Xhr2*/
const Profile P23488 = {"23488", &P23488_333};	/*Profile ID - 23488*/
const Value V21540_1_1 = {S2, NULL};	/*Value = True*/
const Property P21540_1 = {S50, 0, &V21540_1_1, NULL};	/*Property = IsMobile*/
const Value V21540_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21540_5 = {S52, 0, &V21540_5_1227, &P21540_1};	/*Property = ScreenPixelsHeight*/
const Value V21540_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21540_4 = {S54, 0, &V21540_4_1229, &P21540_5};	/*Property = ScreenPixelsWidth*/
const Profile P21540 = {"21540", &P21540_4};	/*Profile ID - 21540*/
const Value V20103_1_1 = {S2, NULL};	/*Value = True*/
const Property P20103_1 = {S50, 0, &V20103_1_1, NULL};	/*Property = IsMobile*/
const Value V20103_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20103_5 = {S52, 0, &V20103_5_9981, &P20103_1};	/*Property = ScreenPixelsHeight*/
const Value V20103_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20103_4 = {S54, 0, &V20103_4_39098, &P20103_5};	/*Property = ScreenPixelsWidth*/
const Profile P20103 = {"20103", &P20103_4};	/*Profile ID - 20103*/
const Value V20240_1_1 = {S2, NULL};	/*Value = True*/
const Property P20240_1 = {S50, 0, &V20240_1_1, NULL};	/*Property = IsMobile*/
const Value V20240_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20240_5 = {S52, 0, &V20240_5_9981, &P20240_1};	/*Property = ScreenPixelsHeight*/
const Value V20240_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20240_4 = {S54, 0, &V20240_4_37617, &P20240_5};	/*Property = ScreenPixelsWidth*/
const Profile P20240 = {"20240", &P20240_4};	/*Profile ID - 20240*/
const Value V20882_1_1 = {S2, NULL};	/*Value = True*/
const Property P20882_1 = {S50, 0, &V20882_1_1, NULL};	/*Property = IsMobile*/
const Value V20882_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20882_5 = {S52, 0, &V20882_5_1227, &P20882_1};	/*Property = ScreenPixelsHeight*/
const Value V20882_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20882_4 = {S54, 0, &V20882_4_1229, &P20882_5};	/*Property = ScreenPixelsWidth*/
const Profile P20882 = {"20882", &P20882_4};	/*Profile ID - 20882*/
const Value V23160_1_1 = {S2, NULL};	/*Value = True*/
const Property P23160_1 = {S50, 0, &V23160_1_1, NULL};	/*Property = IsMobile*/
const Value V23160_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23160_5 = {S52, 0, &V23160_5_1277, &P23160_1};	/*Property = ScreenPixelsHeight*/
const Value V23160_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23160_4 = {S54, 0, &V23160_4_2336, &P23160_5};	/*Property = ScreenPixelsWidth*/
const Profile P23160 = {"23160", &P23160_4};	/*Profile ID - 23160*/
const Value V18227_1_1 = {S2, NULL};	/*Value = True*/
const Property P18227_1 = {S50, 0, &V18227_1_1, NULL};	/*Property = IsMobile*/
const Value V18227_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18227_5 = {S52, 0, &V18227_5_9981, &P18227_1};	/*Property = ScreenPixelsHeight*/
const Value V18227_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18227_4 = {S54, 0, &V18227_4_39098, &P18227_5};	/*Property = ScreenPixelsWidth*/
const Profile P18227 = {"18227", &P18227_4};	/*Profile ID - 18227*/
const Value V20145_1_1 = {S2, NULL};	/*Value = True*/
const Property P20145_1 = {S50, 0, &V20145_1_1, NULL};	/*Property = IsMobile*/
const Value V20145_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20145_5 = {S52, 0, &V20145_5_1458, &P20145_1};	/*Property = ScreenPixelsHeight*/
const Value V20145_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20145_4 = {S54, 0, &V20145_4_37617, &P20145_5};	/*Property = ScreenPixelsWidth*/
const Profile P20145 = {"20145", &P20145_4};	/*Profile ID - 20145*/
const Value V23949_1_1 = {S2, NULL};	/*Value = True*/
const Property P23949_1 = {S50, 0, &V23949_1_1, NULL};	/*Property = IsMobile*/
const Value V23949_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23949_5 = {S52, 0, &V23949_5_9981, &P23949_1};	/*Property = ScreenPixelsHeight*/
const Value V23949_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23949_4 = {S54, 0, &V23949_4_39098, &P23949_5};	/*Property = ScreenPixelsWidth*/
const Profile P23949 = {"23949", &P23949_4};	/*Profile ID - 23949*/
const Value V18721_1_1 = {S2, NULL};	/*Value = True*/
const Property P18721_1 = {S50, 0, &V18721_1_1, NULL};	/*Property = IsMobile*/
const Value V18721_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18721_5 = {S52, 0, &V18721_5_1227, &P18721_1};	/*Property = ScreenPixelsHeight*/
const Value V18721_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18721_4 = {S54, 0, &V18721_4_1229, &P18721_5};	/*Property = ScreenPixelsWidth*/
const Profile P18721 = {"18721", &P18721_4};	/*Profile ID - 18721*/
const Value V22093_1_1 = {S2, NULL};	/*Value = True*/
const Property P22093_1 = {S50, 0, &V22093_1_1, NULL};	/*Property = IsMobile*/
const Value V22093_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22093_5 = {S52, 0, &V22093_5_1227, &P22093_1};	/*Property = ScreenPixelsHeight*/
const Value V22093_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22093_4 = {S54, 0, &V22093_4_1229, &P22093_5};	/*Property = ScreenPixelsWidth*/
const Profile P22093 = {"22093", &P22093_4};	/*Profile ID - 22093*/
const Value V19765_1_1 = {S2, NULL};	/*Value = True*/
const Property P19765_1 = {S50, 0, &V19765_1_1, NULL};	/*Property = IsMobile*/
const Value V19765_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19765_5 = {S52, 0, &V19765_5_1227, &P19765_1};	/*Property = ScreenPixelsHeight*/
const Value V19765_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19765_4 = {S54, 0, &V19765_4_1229, &P19765_5};	/*Property = ScreenPixelsWidth*/
const Profile P19765 = {"19765", &P19765_4};	/*Profile ID - 19765*/
const Value V23459_1_1 = {S2, NULL};	/*Value = True*/
const Property P23459_1 = {S50, 0, &V23459_1_1, NULL};	/*Property = IsMobile*/
const Value V23459_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23459_5 = {S52, 0, &V23459_5_1458, &P23459_1};	/*Property = ScreenPixelsHeight*/
const Value V23459_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23459_4 = {S54, 0, &V23459_4_37617, &P23459_5};	/*Property = ScreenPixelsWidth*/
const Profile P23459 = {"23459", &P23459_4};	/*Profile ID - 23459*/
const Value V23699_1_1 = {S2, NULL};	/*Value = True*/
const Property P23699_1 = {S50, 0, &V23699_1_1, NULL};	/*Property = IsMobile*/
const Value V23699_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23699_5 = {S52, 0, &V23699_5_1277, &P23699_1};	/*Property = ScreenPixelsHeight*/
const Value V23699_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23699_4 = {S54, 0, &V23699_4_2336, &P23699_5};	/*Property = ScreenPixelsWidth*/
const Profile P23699 = {"23699", &P23699_4};	/*Profile ID - 23699*/
const Value V20525_1_1 = {S2, NULL};	/*Value = True*/
const Property P20525_1 = {S50, 0, &V20525_1_1, NULL};	/*Property = IsMobile*/
const Value V20525_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20525_5 = {S52, 0, &V20525_5_1227, &P20525_1};	/*Property = ScreenPixelsHeight*/
const Value V20525_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20525_4 = {S54, 0, &V20525_4_1229, &P20525_5};	/*Property = ScreenPixelsWidth*/
const Profile P20525 = {"20525", &P20525_4};	/*Profile ID - 20525*/
const Value V24154_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24154_279 = {S1, 0, &V24154_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24154_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24154_282 = {S3, 0, &V24154_282_57154, &P24154_279};	/*Property = BlobBuilder*/
const Value V24154_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24154_350 = {S4, 0, &V24154_350_57444, &P24154_282};	/*Property = Canvas*/
const Value V24154_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24154_283 = {S5, 0, &V24154_283_57159, &P24154_350};	/*Property = CssBackground*/
const Value V24154_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24154_284 = {S6, 0, &V24154_284_57162, &P24154_283};	/*Property = CssBorderImage*/
const Value V24154_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24154_285 = {S7, 0, &V24154_285_57166, &P24154_284};	/*Property = CssCanvas*/
const Value V24154_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24154_286 = {S8, 0, &V24154_286_57168, &P24154_285};	/*Property = CssColor*/
const Value V24154_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24154_237 = {S9, 0, &V24154_237_55786, &P24154_286};	/*Property = CssColumn*/
const Value V24154_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24154_287 = {S10, 0, &V24154_287_57172, &P24154_237};	/*Property = CssFlexbox*/
const Value V24154_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24154_288 = {S11, 0, &V24154_288_57174, &P24154_287};	/*Property = CssFont*/
const Value V24154_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24154_289 = {S12, 0, &V24154_289_57759, &P24154_288};	/*Property = CssImages*/
const Value V24154_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24154_291 = {S13, 0, &V24154_291_57180, &P24154_289};	/*Property = CssMediaQueries*/
const Value V24154_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24154_292 = {S14, 0, &V24154_292_57183, &P24154_291};	/*Property = CssMinMax*/
const Value V24154_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24154_293 = {S15, 0, &V24154_293_57187, &P24154_292};	/*Property = CssOverflow*/
const Value V24154_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24154_294 = {S16, 0, &V24154_294_57189, &P24154_293};	/*Property = CssPosition*/
const Value V24154_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24154_295 = {S17, 0, &V24154_295_57192, &P24154_294};	/*Property = CssText*/
const Value V24154_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24154_297 = {S18, 0, &V24154_297_57195, &P24154_295};	/*Property = CssTransforms*/
const Value V24154_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24154_241 = {S19, 0, &V24154_241_55799, &P24154_297};	/*Property = CssTransitions*/
const Value V24154_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24154_298 = {S20, 0, &V24154_298_57206, &P24154_241};	/*Property = CssUI*/
const Value V24154_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24154_300 = {S21, 0, &V24154_300_57215, &P24154_298};	/*Property = DataSet*/
const Value V24154_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24154_301 = {S22, 0, &V24154_301_57218, &P24154_300};	/*Property = DataUrl*/
const Value V24154_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24154_302 = {S23, 0, &V24154_302_57221, &P24154_301};	/*Property = DeviceOrientation*/
const Value V24154_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24154_304 = {S24, 0, &V24154_304_57227, &P24154_302};	/*Property = FileReader*/
const Value V24154_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24154_305 = {S25, 0, &V24154_305_57231, &P24154_304};	/*Property = FileSaver*/
const Value V24154_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24154_306 = {S26, 0, &V24154_306_57237, &P24154_305};	/*Property = FileWriter*/
const Value V24154_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24154_307 = {S27, 0, &V24154_307_57239, &P24154_306};	/*Property = FormData*/
const Value V24154_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24154_309 = {S28, 0, &V24154_309_57246, &P24154_307};	/*Property = Fullscreen*/
const Value V24154_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24154_310 = {S29, 0, &V24154_310_57248, &P24154_309};	/*Property = GeoLocation*/
const Value V24154_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24154_312 = {S30, 0, &V24154_312_57254, &P24154_310};	/*Property = History*/
const Value V24154_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24154_314 = {S31, 0, &V24154_314_57260, &P24154_312};	/*Property = Html5*/
const Value V24154_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24154_313 = {S32, 0, &V24154_313_57257, &P24154_314};	/*Property = Html-Media-Capture*/
const Value V24154_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24154_315 = {S33, 0, &V24154_315_57264, &P24154_313};	/*Property = Iframe*/
const Value V24154_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24154_316 = {S34, 0, &V24154_316_57266, &P24154_315};	/*Property = IndexedDB*/
const Value V24154_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24154_317 = {S35, 0, &V24154_317_57269, &P24154_316};	/*Property = Json*/
const Value V24154_15_81502 = {S141, NULL};	/*Value = Blink*/
const Property P24154_15 = {S37, 0, &V24154_15_81502, &P24154_317};	/*Property = LayoutEngine*/
const Value V24154_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24154_344 = {S38, 0, &V24154_344_57272, &P24154_15};	/*Property = Masking*/
const Value V24154_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24154_320 = {S39, 0, &V24154_320_57290, &P24154_344};	/*Property = PostMessage*/
const Value V24154_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24154_321 = {S40, 0, &V24154_321_57293, &P24154_320};	/*Property = Progress*/
const Value V24154_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24154_322 = {S41, 0, &V24154_322_57296, &P24154_321};	/*Property = Prompts*/
const Value V24154_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24154_323 = {S42, 0, &V24154_323_57299, &P24154_322};	/*Property = Selector*/
const Value V24154_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24154_324 = {S43, 0, &V24154_324_57305, &P24154_323};	/*Property = Svg*/
const Value V24154_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24154_327 = {S44, 0, &V24154_327_57314, &P24154_324};	/*Property = TouchEvents*/
const Value V24154_328_57317 = {S2, NULL};	/*Value = True*/
const Property P24154_328 = {S45, 0, &V24154_328_57317, &P24154_327};	/*Property = Track*/
const Value V24154_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24154_329 = {S46, 0, &V24154_329_57323, &P24154_328};	/*Property = Video*/
const Value V24154_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24154_330 = {S47, 0, &V24154_330_57326, &P24154_329};	/*Property = Viewport*/
const Value V24154_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24154_332 = {S48, 0, &V24154_332_57338, &P24154_330};	/*Property = WebWorkers*/
const Value V24154_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24154_333 = {S49, 0, &V24154_333_57342, &P24154_332};	/*Property = Xhr2*/
const Profile P24154 = {"24154", &P24154_333};	/*Profile ID - 24154*/
const Value V22899_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_279 = {S1, 0, &V22899_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22899_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_282 = {S3, 0, &V22899_282_57155, &P22899_279};	/*Property = BlobBuilder*/
const Value V22899_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_350 = {S4, 0, &V22899_350_57446, &P22899_282};	/*Property = Canvas*/
const Value V22899_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_283 = {S5, 0, &V22899_283_57161, &P22899_350};	/*Property = CssBackground*/
const Value V22899_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_284 = {S6, 0, &V22899_284_57164, &P22899_283};	/*Property = CssBorderImage*/
const Value V22899_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_285 = {S7, 0, &V22899_285_57167, &P22899_284};	/*Property = CssCanvas*/
const Value V22899_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_286 = {S8, 0, &V22899_286_57170, &P22899_285};	/*Property = CssColor*/
const Value V22899_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_237 = {S9, 0, &V22899_237_55786, &P22899_286};	/*Property = CssColumn*/
const Value V22899_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_287 = {S10, 0, &V22899_287_57173, &P22899_237};	/*Property = CssFlexbox*/
const Value V22899_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_288 = {S11, 0, &V22899_288_57176, &P22899_287};	/*Property = CssFont*/
const Value V22899_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_289 = {S12, 0, &V22899_289_86899, &P22899_288};	/*Property = CssImages*/
const Value V22899_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_291 = {S13, 0, &V22899_291_57182, &P22899_289};	/*Property = CssMediaQueries*/
const Value V22899_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_292 = {S14, 0, &V22899_292_57185, &P22899_291};	/*Property = CssMinMax*/
const Value V22899_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_293 = {S15, 0, &V22899_293_57188, &P22899_292};	/*Property = CssOverflow*/
const Value V22899_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_294 = {S16, 0, &V22899_294_57191, &P22899_293};	/*Property = CssPosition*/
const Value V22899_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_295 = {S17, 0, &V22899_295_57194, &P22899_294};	/*Property = CssText*/
const Value V22899_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_297 = {S18, 0, &V22899_297_57197, &P22899_295};	/*Property = CssTransforms*/
const Value V22899_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_241 = {S19, 0, &V22899_241_55801, &P22899_297};	/*Property = CssTransitions*/
const Value V22899_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_298 = {S20, 0, &V22899_298_57208, &P22899_241};	/*Property = CssUI*/
const Value V22899_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_300 = {S21, 0, &V22899_300_57217, &P22899_298};	/*Property = DataSet*/
const Value V22899_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_301 = {S22, 0, &V22899_301_57220, &P22899_300};	/*Property = DataUrl*/
const Value V22899_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_302 = {S23, 0, &V22899_302_57223, &P22899_301};	/*Property = DeviceOrientation*/
const Value V22899_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_304 = {S24, 0, &V22899_304_57229, &P22899_302};	/*Property = FileReader*/
const Value V22899_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_305 = {S25, 0, &V22899_305_57232, &P22899_304};	/*Property = FileSaver*/
const Value V22899_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_306 = {S26, 0, &V22899_306_57238, &P22899_305};	/*Property = FileWriter*/
const Value V22899_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_307 = {S27, 0, &V22899_307_57241, &P22899_306};	/*Property = FormData*/
const Value V22899_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_309 = {S28, 0, &V22899_309_57247, &P22899_307};	/*Property = Fullscreen*/
const Value V22899_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_310 = {S29, 0, &V22899_310_57250, &P22899_309};	/*Property = GeoLocation*/
const Value V22899_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_312 = {S30, 0, &V22899_312_57256, &P22899_310};	/*Property = History*/
const Value V22899_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_314 = {S31, 0, &V22899_314_57262, &P22899_312};	/*Property = Html5*/
const Value V22899_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_313 = {S32, 0, &V22899_313_57259, &P22899_314};	/*Property = Html-Media-Capture*/
const Value V22899_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_315 = {S33, 0, &V22899_315_57265, &P22899_313};	/*Property = Iframe*/
const Value V22899_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_316 = {S34, 0, &V22899_316_57268, &P22899_315};	/*Property = IndexedDB*/
const Value V22899_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_317 = {S35, 0, &V22899_317_57271, &P22899_316};	/*Property = Json*/
const Value V22899_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22899_15 = {S37, 0, &V22899_15_28, &P22899_317};	/*Property = LayoutEngine*/
const Value V22899_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_344 = {S38, 0, &V22899_344_57274, &P22899_15};	/*Property = Masking*/
const Value V22899_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_320 = {S39, 0, &V22899_320_57292, &P22899_344};	/*Property = PostMessage*/
const Value V22899_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_321 = {S40, 0, &V22899_321_57295, &P22899_320};	/*Property = Progress*/
const Value V22899_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_322 = {S41, 0, &V22899_322_57298, &P22899_321};	/*Property = Prompts*/
const Value V22899_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_323 = {S42, 0, &V22899_323_57301, &P22899_322};	/*Property = Selector*/
const Value V22899_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_324 = {S43, 0, &V22899_324_57307, &P22899_323};	/*Property = Svg*/
const Value V22899_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_327 = {S44, 0, &V22899_327_57316, &P22899_324};	/*Property = TouchEvents*/
const Value V22899_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_328 = {S45, 0, &V22899_328_57319, &P22899_327};	/*Property = Track*/
const Value V22899_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_329 = {S46, 0, &V22899_329_57325, &P22899_328};	/*Property = Video*/
const Value V22899_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_330 = {S47, 0, &V22899_330_57328, &P22899_329};	/*Property = Viewport*/
const Value V22899_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_332 = {S48, 0, &V22899_332_57340, &P22899_330};	/*Property = WebWorkers*/
const Value V22899_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22899_333 = {S49, 0, &V22899_333_57343, &P22899_332};	/*Property = Xhr2*/
const Profile P22899 = {"22899", &P22899_333};	/*Profile ID - 22899*/
const Value V20035_1_1 = {S2, NULL};	/*Value = True*/
const Property P20035_1 = {S50, 0, &V20035_1_1, NULL};	/*Property = IsMobile*/
const Value V20035_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20035_5 = {S52, 0, &V20035_5_9981, &P20035_1};	/*Property = ScreenPixelsHeight*/
const Value V20035_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20035_4 = {S54, 0, &V20035_4_2336, &P20035_5};	/*Property = ScreenPixelsWidth*/
const Profile P20035 = {"20035", &P20035_4};	/*Profile ID - 20035*/
const Value V18716_1_1 = {S2, NULL};	/*Value = True*/
const Property P18716_1 = {S50, 0, &V18716_1_1, NULL};	/*Property = IsMobile*/
const Value V18716_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18716_5 = {S52, 0, &V18716_5_1277, &P18716_1};	/*Property = ScreenPixelsHeight*/
const Value V18716_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18716_4 = {S54, 0, &V18716_4_1279, &P18716_5};	/*Property = ScreenPixelsWidth*/
const Profile P18716 = {"18716", &P18716_4};	/*Profile ID - 18716*/
const Value V20477_1_1 = {S2, NULL};	/*Value = True*/
const Property P20477_1 = {S50, 0, &V20477_1_1, NULL};	/*Property = IsMobile*/
const Value V20477_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20477_5 = {S52, 0, &V20477_5_1458, &P20477_1};	/*Property = ScreenPixelsHeight*/
const Value V20477_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20477_4 = {S54, 0, &V20477_4_37617, &P20477_5};	/*Property = ScreenPixelsWidth*/
const Profile P20477 = {"20477", &P20477_4};	/*Profile ID - 20477*/
const Value V20104_1_1 = {S2, NULL};	/*Value = True*/
const Property P20104_1 = {S50, 0, &V20104_1_1, NULL};	/*Property = IsMobile*/
const Value V20104_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20104_5 = {S52, 0, &V20104_5_1403, &P20104_1};	/*Property = ScreenPixelsHeight*/
const Value V20104_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20104_4 = {S54, 0, &V20104_4_1404, &P20104_5};	/*Property = ScreenPixelsWidth*/
const Profile P20104 = {"20104", &P20104_4};	/*Profile ID - 20104*/
const Value V18460_1_1 = {S2, NULL};	/*Value = True*/
const Property P18460_1 = {S50, 0, &V18460_1_1, NULL};	/*Property = IsMobile*/
const Value V18460_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P18460_5 = {S52, 0, &V18460_5_1403, &P18460_1};	/*Property = ScreenPixelsHeight*/
const Value V18460_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P18460_4 = {S54, 0, &V18460_4_1404, &P18460_5};	/*Property = ScreenPixelsWidth*/
const Profile P18460 = {"18460", &P18460_4};	/*Profile ID - 18460*/
const Value V20263_1_1 = {S2, NULL};	/*Value = True*/
const Property P20263_1 = {S50, 0, &V20263_1_1, NULL};	/*Property = IsMobile*/
const Value V20263_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20263_5 = {S52, 0, &V20263_5_1227, &P20263_1};	/*Property = ScreenPixelsHeight*/
const Value V20263_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20263_4 = {S54, 0, &V20263_4_1229, &P20263_5};	/*Property = ScreenPixelsWidth*/
const Profile P20263 = {"20263", &P20263_4};	/*Profile ID - 20263*/
const Value V23039_1_1 = {S2, NULL};	/*Value = True*/
const Property P23039_1 = {S50, 0, &V23039_1_1, NULL};	/*Property = IsMobile*/
const Value V23039_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23039_5 = {S52, 0, &V23039_5_1227, &P23039_1};	/*Property = ScreenPixelsHeight*/
const Value V23039_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P23039_4 = {S54, 0, &V23039_4_1477, &P23039_5};	/*Property = ScreenPixelsWidth*/
const Profile P23039 = {"23039", &P23039_4};	/*Profile ID - 23039*/
const Value V20789_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_279 = {S1, 0, &V20789_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20789_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_282 = {S3, 0, &V20789_282_57155, &P20789_279};	/*Property = BlobBuilder*/
const Value V20789_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_350 = {S4, 0, &V20789_350_57446, &P20789_282};	/*Property = Canvas*/
const Value V20789_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_283 = {S5, 0, &V20789_283_57161, &P20789_350};	/*Property = CssBackground*/
const Value V20789_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_284 = {S6, 0, &V20789_284_57164, &P20789_283};	/*Property = CssBorderImage*/
const Value V20789_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_285 = {S7, 0, &V20789_285_57167, &P20789_284};	/*Property = CssCanvas*/
const Value V20789_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_286 = {S8, 0, &V20789_286_57170, &P20789_285};	/*Property = CssColor*/
const Value V20789_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_237 = {S9, 0, &V20789_237_55786, &P20789_286};	/*Property = CssColumn*/
const Value V20789_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_287 = {S10, 0, &V20789_287_57173, &P20789_237};	/*Property = CssFlexbox*/
const Value V20789_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_288 = {S11, 0, &V20789_288_57176, &P20789_287};	/*Property = CssFont*/
const Value V20789_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_289 = {S12, 0, &V20789_289_86899, &P20789_288};	/*Property = CssImages*/
const Value V20789_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_291 = {S13, 0, &V20789_291_57182, &P20789_289};	/*Property = CssMediaQueries*/
const Value V20789_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_292 = {S14, 0, &V20789_292_57185, &P20789_291};	/*Property = CssMinMax*/
const Value V20789_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_293 = {S15, 0, &V20789_293_57188, &P20789_292};	/*Property = CssOverflow*/
const Value V20789_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_294 = {S16, 0, &V20789_294_57191, &P20789_293};	/*Property = CssPosition*/
const Value V20789_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_295 = {S17, 0, &V20789_295_57194, &P20789_294};	/*Property = CssText*/
const Value V20789_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_297 = {S18, 0, &V20789_297_57197, &P20789_295};	/*Property = CssTransforms*/
const Value V20789_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_241 = {S19, 0, &V20789_241_55801, &P20789_297};	/*Property = CssTransitions*/
const Value V20789_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_298 = {S20, 0, &V20789_298_57208, &P20789_241};	/*Property = CssUI*/
const Value V20789_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_300 = {S21, 0, &V20789_300_57217, &P20789_298};	/*Property = DataSet*/
const Value V20789_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_301 = {S22, 0, &V20789_301_57220, &P20789_300};	/*Property = DataUrl*/
const Value V20789_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_302 = {S23, 0, &V20789_302_57223, &P20789_301};	/*Property = DeviceOrientation*/
const Value V20789_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_304 = {S24, 0, &V20789_304_57229, &P20789_302};	/*Property = FileReader*/
const Value V20789_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_305 = {S25, 0, &V20789_305_57232, &P20789_304};	/*Property = FileSaver*/
const Value V20789_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_306 = {S26, 0, &V20789_306_57238, &P20789_305};	/*Property = FileWriter*/
const Value V20789_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_307 = {S27, 0, &V20789_307_57241, &P20789_306};	/*Property = FormData*/
const Value V20789_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_309 = {S28, 0, &V20789_309_57247, &P20789_307};	/*Property = Fullscreen*/
const Value V20789_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_310 = {S29, 0, &V20789_310_57250, &P20789_309};	/*Property = GeoLocation*/
const Value V20789_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_312 = {S30, 0, &V20789_312_57256, &P20789_310};	/*Property = History*/
const Value V20789_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_314 = {S31, 0, &V20789_314_57262, &P20789_312};	/*Property = Html5*/
const Value V20789_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_313 = {S32, 0, &V20789_313_57259, &P20789_314};	/*Property = Html-Media-Capture*/
const Value V20789_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_315 = {S33, 0, &V20789_315_57265, &P20789_313};	/*Property = Iframe*/
const Value V20789_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_316 = {S34, 0, &V20789_316_57268, &P20789_315};	/*Property = IndexedDB*/
const Value V20789_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_317 = {S35, 0, &V20789_317_57271, &P20789_316};	/*Property = Json*/
const Value V20789_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P20789_15 = {S37, 0, &V20789_15_54167, &P20789_317};	/*Property = LayoutEngine*/
const Value V20789_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_344 = {S38, 0, &V20789_344_57274, &P20789_15};	/*Property = Masking*/
const Value V20789_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_320 = {S39, 0, &V20789_320_57292, &P20789_344};	/*Property = PostMessage*/
const Value V20789_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_321 = {S40, 0, &V20789_321_57295, &P20789_320};	/*Property = Progress*/
const Value V20789_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_322 = {S41, 0, &V20789_322_57298, &P20789_321};	/*Property = Prompts*/
const Value V20789_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_323 = {S42, 0, &V20789_323_57301, &P20789_322};	/*Property = Selector*/
const Value V20789_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_324 = {S43, 0, &V20789_324_57307, &P20789_323};	/*Property = Svg*/
const Value V20789_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_327 = {S44, 0, &V20789_327_57316, &P20789_324};	/*Property = TouchEvents*/
const Value V20789_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_328 = {S45, 0, &V20789_328_57319, &P20789_327};	/*Property = Track*/
const Value V20789_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_329 = {S46, 0, &V20789_329_57325, &P20789_328};	/*Property = Video*/
const Value V20789_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_330 = {S47, 0, &V20789_330_57328, &P20789_329};	/*Property = Viewport*/
const Value V20789_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_332 = {S48, 0, &V20789_332_57340, &P20789_330};	/*Property = WebWorkers*/
const Value V20789_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20789_333 = {S49, 0, &V20789_333_57343, &P20789_332};	/*Property = Xhr2*/
const Profile P20789 = {"20789", &P20789_333};	/*Profile ID - 20789*/
const Value V23695_1_1 = {S2, NULL};	/*Value = True*/
const Property P23695_1 = {S50, 0, &V23695_1_1, NULL};	/*Property = IsMobile*/
const Value V23695_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23695_5 = {S52, 0, &V23695_5_37569, &P23695_1};	/*Property = ScreenPixelsHeight*/
const Value V23695_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23695_4 = {S54, 0, &V23695_4_5645, &P23695_5};	/*Property = ScreenPixelsWidth*/
const Profile P23695 = {"23695", &P23695_4};	/*Profile ID - 23695*/
const Value V20177_1_1 = {S2, NULL};	/*Value = True*/
const Property P20177_1 = {S50, 0, &V20177_1_1, NULL};	/*Property = IsMobile*/
const Value V20177_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20177_5 = {S52, 0, &V20177_5_1227, &P20177_1};	/*Property = ScreenPixelsHeight*/
const Value V20177_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20177_4 = {S54, 0, &V20177_4_1229, &P20177_5};	/*Property = ScreenPixelsWidth*/
const Profile P20177 = {"20177", &P20177_4};	/*Profile ID - 20177*/
const Value V21907_1_1 = {S2, NULL};	/*Value = True*/
const Property P21907_1 = {S50, 0, &V21907_1_1, NULL};	/*Property = IsMobile*/
const Value V21907_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21907_5 = {S52, 0, &V21907_5_1227, &P21907_1};	/*Property = ScreenPixelsHeight*/
const Value V21907_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21907_4 = {S54, 0, &V21907_4_1460, &P21907_5};	/*Property = ScreenPixelsWidth*/
const Profile P21907 = {"21907", &P21907_4};	/*Profile ID - 21907*/
const Value V19866_1_1 = {S2, NULL};	/*Value = True*/
const Property P19866_1 = {S50, 0, &V19866_1_1, NULL};	/*Property = IsMobile*/
const Value V19866_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P19866_5 = {S52, 0, &V19866_5_1277, &P19866_1};	/*Property = ScreenPixelsHeight*/
const Value V19866_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P19866_4 = {S54, 0, &V19866_4_1279, &P19866_5};	/*Property = ScreenPixelsWidth*/
const Profile P19866 = {"19866", &P19866_4};	/*Profile ID - 19866*/
const Value V20107_1_1 = {S2, NULL};	/*Value = True*/
const Property P20107_1 = {S50, 0, &V20107_1_1, NULL};	/*Property = IsMobile*/
const Value V20107_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20107_5 = {S52, 0, &V20107_5_1458, &P20107_1};	/*Property = ScreenPixelsHeight*/
const Value V20107_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20107_4 = {S54, 0, &V20107_4_37617, &P20107_5};	/*Property = ScreenPixelsWidth*/
const Profile P20107 = {"20107", &P20107_4};	/*Profile ID - 20107*/
const Value V20527_1_1 = {S2, NULL};	/*Value = True*/
const Property P20527_1 = {S50, 0, &V20527_1_1, NULL};	/*Property = IsMobile*/
const Value V20527_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20527_5 = {S52, 0, &V20527_5_1227, &P20527_1};	/*Property = ScreenPixelsHeight*/
const Value V20527_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20527_4 = {S54, 0, &V20527_4_1229, &P20527_5};	/*Property = ScreenPixelsWidth*/
const Profile P20527 = {"20527", &P20527_4};	/*Profile ID - 20527*/
const Value V22265_1_1 = {S2, NULL};	/*Value = True*/
const Property P22265_1 = {S50, 0, &V22265_1_1, NULL};	/*Property = IsMobile*/
const Value V22265_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22265_5 = {S52, 0, &V22265_5_1277, &P22265_1};	/*Property = ScreenPixelsHeight*/
const Value V22265_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22265_4 = {S54, 0, &V22265_4_2336, &P22265_5};	/*Property = ScreenPixelsWidth*/
const Profile P22265 = {"22265", &P22265_4};	/*Profile ID - 22265*/
const Value V20404_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20404_279 = {S1, 0, &V20404_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20404_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20404_282 = {S3, 0, &V20404_282_57154, &P20404_279};	/*Property = BlobBuilder*/
const Value V20404_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20404_350 = {S4, 0, &V20404_350_57444, &P20404_282};	/*Property = Canvas*/
const Value V20404_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20404_283 = {S5, 0, &V20404_283_57159, &P20404_350};	/*Property = CssBackground*/
const Value V20404_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20404_284 = {S6, 0, &V20404_284_57163, &P20404_283};	/*Property = CssBorderImage*/
const Value V20404_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20404_285 = {S7, 0, &V20404_285_57166, &P20404_284};	/*Property = CssCanvas*/
const Value V20404_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20404_286 = {S8, 0, &V20404_286_57168, &P20404_285};	/*Property = CssColor*/
const Value V20404_237_55784 = {S2, NULL};	/*Value = True*/
const Property P20404_237 = {S9, 0, &V20404_237_55784, &P20404_286};	/*Property = CssColumn*/
const Value V20404_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20404_287 = {S10, 0, &V20404_287_57172, &P20404_237};	/*Property = CssFlexbox*/
const Value V20404_288_57175 = {S0, NULL};	/*Value = False*/
const Property P20404_288 = {S11, 0, &V20404_288_57175, &P20404_287};	/*Property = CssFont*/
const Value V20404_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20404_289 = {S12, 0, &V20404_289_57759, &P20404_288};	/*Property = CssImages*/
const Value V20404_291_57181 = {S0, NULL};	/*Value = False*/
const Property P20404_291 = {S13, 0, &V20404_291_57181, &P20404_289};	/*Property = CssMediaQueries*/
const Value V20404_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20404_292 = {S14, 0, &V20404_292_57183, &P20404_291};	/*Property = CssMinMax*/
const Value V20404_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20404_293 = {S15, 0, &V20404_293_57187, &P20404_292};	/*Property = CssOverflow*/
const Value V20404_294_57190 = {S0, NULL};	/*Value = False*/
const Property P20404_294 = {S16, 0, &V20404_294_57190, &P20404_293};	/*Property = CssPosition*/
const Value V20404_295_57193 = {S0, NULL};	/*Value = False*/
const Property P20404_295 = {S17, 0, &V20404_295_57193, &P20404_294};	/*Property = CssText*/
const Value V20404_297_57196 = {S0, NULL};	/*Value = False*/
const Property P20404_297 = {S18, 0, &V20404_297_57196, &P20404_295};	/*Property = CssTransforms*/
const Value V20404_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20404_241 = {S19, 0, &V20404_241_55799, &P20404_297};	/*Property = CssTransitions*/
const Value V20404_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20404_298 = {S20, 0, &V20404_298_57206, &P20404_241};	/*Property = CssUI*/
const Value V20404_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20404_300 = {S21, 0, &V20404_300_57216, &P20404_298};	/*Property = DataSet*/
const Value V20404_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20404_301 = {S22, 0, &V20404_301_57218, &P20404_300};	/*Property = DataUrl*/
const Value V20404_302_57222 = {S0, NULL};	/*Value = False*/
const Property P20404_302 = {S23, 0, &V20404_302_57222, &P20404_301};	/*Property = DeviceOrientation*/
const Value V20404_304_57228 = {S0, NULL};	/*Value = False*/
const Property P20404_304 = {S24, 0, &V20404_304_57228, &P20404_302};	/*Property = FileReader*/
const Value V20404_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20404_305 = {S25, 0, &V20404_305_57231, &P20404_304};	/*Property = FileSaver*/
const Value V20404_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20404_306 = {S26, 0, &V20404_306_57237, &P20404_305};	/*Property = FileWriter*/
const Value V20404_307_57240 = {S0, NULL};	/*Value = False*/
const Property P20404_307 = {S27, 0, &V20404_307_57240, &P20404_306};	/*Property = FormData*/
const Value V20404_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20404_309 = {S28, 0, &V20404_309_57246, &P20404_307};	/*Property = Fullscreen*/
const Value V20404_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20404_310 = {S29, 0, &V20404_310_57248, &P20404_309};	/*Property = GeoLocation*/
const Value V20404_312_57255 = {S0, NULL};	/*Value = False*/
const Property P20404_312 = {S30, 0, &V20404_312_57255, &P20404_310};	/*Property = History*/
const Value V20404_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20404_314 = {S31, 0, &V20404_314_57261, &P20404_312};	/*Property = Html5*/
const Value V20404_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20404_313 = {S32, 0, &V20404_313_57258, &P20404_314};	/*Property = Html-Media-Capture*/
const Value V20404_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20404_315 = {S33, 0, &V20404_315_57264, &P20404_313};	/*Property = Iframe*/
const Value V20404_316_57267 = {S0, NULL};	/*Value = False*/
const Property P20404_316 = {S34, 0, &V20404_316_57267, &P20404_315};	/*Property = IndexedDB*/
const Value V20404_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20404_317 = {S35, 0, &V20404_317_57269, &P20404_316};	/*Property = Json*/
const Value V20404_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P20404_15 = {S37, 0, &V20404_15_31, &P20404_317};	/*Property = LayoutEngine*/
const Value V20404_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20404_344 = {S38, 0, &V20404_344_57272, &P20404_15};	/*Property = Masking*/
const Value V20404_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20404_320 = {S39, 0, &V20404_320_57290, &P20404_344};	/*Property = PostMessage*/
const Value V20404_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20404_321 = {S40, 0, &V20404_321_57293, &P20404_320};	/*Property = Progress*/
const Value V20404_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20404_322 = {S41, 0, &V20404_322_57296, &P20404_321};	/*Property = Prompts*/
const Value V20404_323_57300 = {S0, NULL};	/*Value = False*/
const Property P20404_323 = {S42, 0, &V20404_323_57300, &P20404_322};	/*Property = Selector*/
const Value V20404_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20404_324 = {S43, 0, &V20404_324_57306, &P20404_323};	/*Property = Svg*/
const Value V20404_327_57315 = {S0, NULL};	/*Value = False*/
const Property P20404_327 = {S44, 0, &V20404_327_57315, &P20404_324};	/*Property = TouchEvents*/
const Value V20404_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20404_328 = {S45, 0, &V20404_328_57318, &P20404_327};	/*Property = Track*/
const Value V20404_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20404_329 = {S46, 0, &V20404_329_57323, &P20404_328};	/*Property = Video*/
const Value V20404_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20404_330 = {S47, 0, &V20404_330_57326, &P20404_329};	/*Property = Viewport*/
const Value V20404_332_57339 = {S0, NULL};	/*Value = False*/
const Property P20404_332 = {S48, 0, &V20404_332_57339, &P20404_330};	/*Property = WebWorkers*/
const Value V20404_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20404_333 = {S49, 0, &V20404_333_57342, &P20404_332};	/*Property = Xhr2*/
const Profile P20404 = {"20404", &P20404_333};	/*Profile ID - 20404*/
const Value V18715_1_1 = {S2, NULL};	/*Value = True*/
const Property P18715_1 = {S50, 0, &V18715_1_1, NULL};	/*Property = IsMobile*/
const Value V18715_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18715_5 = {S52, 0, &V18715_5_1227, &P18715_1};	/*Property = ScreenPixelsHeight*/
const Value V18715_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P18715_4 = {S54, 0, &V18715_4_1460, &P18715_5};	/*Property = ScreenPixelsWidth*/
const Profile P18715 = {"18715", &P18715_4};	/*Profile ID - 18715*/
const Value V18464_1_1 = {S2, NULL};	/*Value = True*/
const Property P18464_1 = {S50, 0, &V18464_1_1, NULL};	/*Property = IsMobile*/
const Value V18464_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P18464_5 = {S52, 0, &V18464_5_59138, &P18464_1};	/*Property = ScreenPixelsHeight*/
const Value V18464_4_52971 = {S122, NULL};	/*Value = 1200*/
const Property P18464_4 = {S54, 0, &V18464_4_52971, &P18464_5};	/*Property = ScreenPixelsWidth*/
const Profile P18464 = {"18464", &P18464_4};	/*Profile ID - 18464*/
const Value V20089_1_1 = {S2, NULL};	/*Value = True*/
const Property P20089_1 = {S50, 0, &V20089_1_1, NULL};	/*Property = IsMobile*/
const Value V20089_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20089_5 = {S52, 0, &V20089_5_9981, &P20089_1};	/*Property = ScreenPixelsHeight*/
const Value V20089_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20089_4 = {S54, 0, &V20089_4_39098, &P20089_5};	/*Property = ScreenPixelsWidth*/
const Profile P20089 = {"20089", &P20089_4};	/*Profile ID - 20089*/
const Value V22254_1_1 = {S2, NULL};	/*Value = True*/
const Property P22254_1 = {S50, 0, &V22254_1_1, NULL};	/*Property = IsMobile*/
const Value V22254_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22254_5 = {S52, 0, &V22254_5_1227, &P22254_1};	/*Property = ScreenPixelsHeight*/
const Value V22254_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P22254_4 = {S54, 0, &V22254_4_1477, &P22254_5};	/*Property = ScreenPixelsWidth*/
const Profile P22254 = {"22254", &P22254_4};	/*Profile ID - 22254*/
const Value V23662_1_2 = {S0, NULL};	/*Value = False*/
const Property P23662_1 = {S50, 0, &V23662_1_2, NULL};	/*Property = IsMobile*/
const Value V23662_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23662_5 = {S52, 0, &V23662_5_60417, &P23662_1};	/*Property = ScreenPixelsHeight*/
const Value V23662_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23662_4 = {S54, 0, &V23662_4_57095, &P23662_5};	/*Property = ScreenPixelsWidth*/
const Profile P23662 = {"23662", &P23662_4};	/*Profile ID - 23662*/
const Value V20202_1_1 = {S2, NULL};	/*Value = True*/
const Property P20202_1 = {S50, 0, &V20202_1_1, NULL};	/*Property = IsMobile*/
const Value V20202_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20202_5 = {S52, 0, &V20202_5_1222, &P20202_1};	/*Property = ScreenPixelsHeight*/
const Value V20202_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20202_4 = {S54, 0, &V20202_4_1224, &P20202_5};	/*Property = ScreenPixelsWidth*/
const Profile P20202 = {"20202", &P20202_4};	/*Profile ID - 20202*/
const Value V20443_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_279 = {S1, 0, &V20443_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20443_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_282 = {S3, 0, &V20443_282_57155, &P20443_279};	/*Property = BlobBuilder*/
const Value V20443_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_350 = {S4, 0, &V20443_350_57446, &P20443_282};	/*Property = Canvas*/
const Value V20443_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_283 = {S5, 0, &V20443_283_57161, &P20443_350};	/*Property = CssBackground*/
const Value V20443_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_284 = {S6, 0, &V20443_284_57164, &P20443_283};	/*Property = CssBorderImage*/
const Value V20443_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_285 = {S7, 0, &V20443_285_57167, &P20443_284};	/*Property = CssCanvas*/
const Value V20443_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_286 = {S8, 0, &V20443_286_57170, &P20443_285};	/*Property = CssColor*/
const Value V20443_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_237 = {S9, 0, &V20443_237_55786, &P20443_286};	/*Property = CssColumn*/
const Value V20443_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_287 = {S10, 0, &V20443_287_57173, &P20443_237};	/*Property = CssFlexbox*/
const Value V20443_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_288 = {S11, 0, &V20443_288_57176, &P20443_287};	/*Property = CssFont*/
const Value V20443_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_289 = {S12, 0, &V20443_289_86899, &P20443_288};	/*Property = CssImages*/
const Value V20443_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_291 = {S13, 0, &V20443_291_57182, &P20443_289};	/*Property = CssMediaQueries*/
const Value V20443_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_292 = {S14, 0, &V20443_292_57185, &P20443_291};	/*Property = CssMinMax*/
const Value V20443_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_293 = {S15, 0, &V20443_293_57188, &P20443_292};	/*Property = CssOverflow*/
const Value V20443_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_294 = {S16, 0, &V20443_294_57191, &P20443_293};	/*Property = CssPosition*/
const Value V20443_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_295 = {S17, 0, &V20443_295_57194, &P20443_294};	/*Property = CssText*/
const Value V20443_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_297 = {S18, 0, &V20443_297_57197, &P20443_295};	/*Property = CssTransforms*/
const Value V20443_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_241 = {S19, 0, &V20443_241_55801, &P20443_297};	/*Property = CssTransitions*/
const Value V20443_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_298 = {S20, 0, &V20443_298_57208, &P20443_241};	/*Property = CssUI*/
const Value V20443_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_300 = {S21, 0, &V20443_300_57217, &P20443_298};	/*Property = DataSet*/
const Value V20443_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_301 = {S22, 0, &V20443_301_57220, &P20443_300};	/*Property = DataUrl*/
const Value V20443_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_302 = {S23, 0, &V20443_302_57223, &P20443_301};	/*Property = DeviceOrientation*/
const Value V20443_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_304 = {S24, 0, &V20443_304_57229, &P20443_302};	/*Property = FileReader*/
const Value V20443_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_305 = {S25, 0, &V20443_305_57232, &P20443_304};	/*Property = FileSaver*/
const Value V20443_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_306 = {S26, 0, &V20443_306_57238, &P20443_305};	/*Property = FileWriter*/
const Value V20443_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_307 = {S27, 0, &V20443_307_57241, &P20443_306};	/*Property = FormData*/
const Value V20443_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_309 = {S28, 0, &V20443_309_57247, &P20443_307};	/*Property = Fullscreen*/
const Value V20443_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_310 = {S29, 0, &V20443_310_57250, &P20443_309};	/*Property = GeoLocation*/
const Value V20443_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_312 = {S30, 0, &V20443_312_57256, &P20443_310};	/*Property = History*/
const Value V20443_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_314 = {S31, 0, &V20443_314_57262, &P20443_312};	/*Property = Html5*/
const Value V20443_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_313 = {S32, 0, &V20443_313_57259, &P20443_314};	/*Property = Html-Media-Capture*/
const Value V20443_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_315 = {S33, 0, &V20443_315_57265, &P20443_313};	/*Property = Iframe*/
const Value V20443_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_316 = {S34, 0, &V20443_316_57268, &P20443_315};	/*Property = IndexedDB*/
const Value V20443_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_317 = {S35, 0, &V20443_317_57271, &P20443_316};	/*Property = Json*/
const Value V20443_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20443_15 = {S37, 0, &V20443_15_29, &P20443_317};	/*Property = LayoutEngine*/
const Value V20443_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_344 = {S38, 0, &V20443_344_57274, &P20443_15};	/*Property = Masking*/
const Value V20443_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_320 = {S39, 0, &V20443_320_57292, &P20443_344};	/*Property = PostMessage*/
const Value V20443_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_321 = {S40, 0, &V20443_321_57295, &P20443_320};	/*Property = Progress*/
const Value V20443_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_322 = {S41, 0, &V20443_322_57298, &P20443_321};	/*Property = Prompts*/
const Value V20443_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_323 = {S42, 0, &V20443_323_57301, &P20443_322};	/*Property = Selector*/
const Value V20443_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_324 = {S43, 0, &V20443_324_57307, &P20443_323};	/*Property = Svg*/
const Value V20443_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_327 = {S44, 0, &V20443_327_57316, &P20443_324};	/*Property = TouchEvents*/
const Value V20443_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_328 = {S45, 0, &V20443_328_57319, &P20443_327};	/*Property = Track*/
const Value V20443_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_329 = {S46, 0, &V20443_329_57325, &P20443_328};	/*Property = Video*/
const Value V20443_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_330 = {S47, 0, &V20443_330_57328, &P20443_329};	/*Property = Viewport*/
const Value V20443_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_332 = {S48, 0, &V20443_332_57340, &P20443_330};	/*Property = WebWorkers*/
const Value V20443_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20443_333 = {S49, 0, &V20443_333_57343, &P20443_332};	/*Property = Xhr2*/
const Profile P20443 = {"20443", &P20443_333};	/*Profile ID - 20443*/
const Value V20188_1_1 = {S2, NULL};	/*Value = True*/
const Property P20188_1 = {S50, 0, &V20188_1_1, NULL};	/*Property = IsMobile*/
const Value V20188_5_64708 = {S142, NULL};	/*Value = 1028*/
const Property P20188_5 = {S52, 0, &V20188_5_64708, &P20188_1};	/*Property = ScreenPixelsHeight*/
const Value V20188_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20188_4 = {S54, 0, &V20188_4_37617, &P20188_5};	/*Property = ScreenPixelsWidth*/
const Profile P20188 = {"20188", &P20188_4};	/*Profile ID - 20188*/
const Value V23601_1_2 = {S0, NULL};	/*Value = False*/
const Property P23601_1 = {S50, 0, &V23601_1_2, NULL};	/*Property = IsMobile*/
const Value V23601_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23601_5 = {S52, 0, &V23601_5_60417, &P23601_1};	/*Property = ScreenPixelsHeight*/
const Value V23601_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23601_4 = {S54, 0, &V23601_4_57095, &P23601_5};	/*Property = ScreenPixelsWidth*/
const Profile P23601 = {"23601", &P23601_4};	/*Profile ID - 23601*/
const Value V23604_1_2 = {S0, NULL};	/*Value = False*/
const Property P23604_1 = {S50, 0, &V23604_1_2, NULL};	/*Property = IsMobile*/
const Value V23604_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23604_5 = {S52, 0, &V23604_5_60417, &P23604_1};	/*Property = ScreenPixelsHeight*/
const Value V23604_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23604_4 = {S54, 0, &V23604_4_57095, &P23604_5};	/*Property = ScreenPixelsWidth*/
const Profile P23604 = {"23604", &P23604_4};	/*Profile ID - 23604*/
const Profile P22222 = {"22222", NULL};	/*Profile ID - 22222*/
const Value V20494_1_1 = {S2, NULL};	/*Value = True*/
const Property P20494_1 = {S50, 0, &V20494_1_1, NULL};	/*Property = IsMobile*/
const Value V20494_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20494_5 = {S52, 0, &V20494_5_1227, &P20494_1};	/*Property = ScreenPixelsHeight*/
const Value V20494_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20494_4 = {S54, 0, &V20494_4_1229, &P20494_5};	/*Property = ScreenPixelsWidth*/
const Profile P20494 = {"20494", &P20494_4};	/*Profile ID - 20494*/
const Value V22976_1_1 = {S2, NULL};	/*Value = True*/
const Property P22976_1 = {S50, 0, &V22976_1_1, NULL};	/*Property = IsMobile*/
const Value V22976_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22976_5 = {S52, 0, &V22976_5_1277, &P22976_1};	/*Property = ScreenPixelsHeight*/
const Value V22976_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22976_4 = {S54, 0, &V22976_4_1279, &P22976_5};	/*Property = ScreenPixelsWidth*/
const Profile P22976 = {"22976", &P22976_4};	/*Profile ID - 22976*/
const Profile P22223 = {"22223", NULL};	/*Profile ID - 22223*/
const Value V20227_1_1 = {S2, NULL};	/*Value = True*/
const Property P20227_1 = {S50, 0, &V20227_1_1, NULL};	/*Property = IsMobile*/
const Value V20227_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20227_5 = {S52, 0, &V20227_5_37, &P20227_1};	/*Property = ScreenPixelsHeight*/
const Value V20227_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20227_4 = {S54, 0, &V20227_4_36, &P20227_5};	/*Property = ScreenPixelsWidth*/
const Profile P20227 = {"20227", &P20227_4};	/*Profile ID - 20227*/
const Value V23022_1_1 = {S2, NULL};	/*Value = True*/
const Property P23022_1 = {S50, 0, &V23022_1_1, NULL};	/*Property = IsMobile*/
const Value V23022_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23022_5 = {S52, 0, &V23022_5_1222, &P23022_1};	/*Property = ScreenPixelsHeight*/
const Value V23022_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23022_4 = {S54, 0, &V23022_4_1224, &P23022_5};	/*Property = ScreenPixelsWidth*/
const Profile P23022 = {"23022", &P23022_4};	/*Profile ID - 23022*/
const Value V22161_1_1 = {S2, NULL};	/*Value = True*/
const Property P22161_1 = {S50, 0, &V22161_1_1, NULL};	/*Property = IsMobile*/
const Value V22161_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22161_5 = {S52, 0, &V22161_5_1227, &P22161_1};	/*Property = ScreenPixelsHeight*/
const Value V22161_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P22161_4 = {S54, 0, &V22161_4_1943, &P22161_5};	/*Property = ScreenPixelsWidth*/
const Profile P22161 = {"22161", &P22161_4};	/*Profile ID - 22161*/
const Value V21705_1_1 = {S2, NULL};	/*Value = True*/
const Property P21705_1 = {S50, 0, &V21705_1_1, NULL};	/*Property = IsMobile*/
const Value V21705_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21705_5 = {S52, 0, &V21705_5_37, &P21705_1};	/*Property = ScreenPixelsHeight*/
const Value V21705_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21705_4 = {S54, 0, &V21705_4_36, &P21705_5};	/*Property = ScreenPixelsWidth*/
const Profile P21705 = {"21705", &P21705_4};	/*Profile ID - 21705*/
const Value V21137_1_1 = {S2, NULL};	/*Value = True*/
const Property P21137_1 = {S50, 0, &V21137_1_1, NULL};	/*Property = IsMobile*/
const Value V21137_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21137_5 = {S52, 0, &V21137_5_1458, &P21137_1};	/*Property = ScreenPixelsHeight*/
const Value V21137_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21137_4 = {S54, 0, &V21137_4_1460, &P21137_5};	/*Property = ScreenPixelsWidth*/
const Profile P21137 = {"21137", &P21137_4};	/*Profile ID - 21137*/
const Value V23193_1_1 = {S2, NULL};	/*Value = True*/
const Property P23193_1 = {S50, 0, &V23193_1_1, NULL};	/*Property = IsMobile*/
const Value V23193_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23193_5 = {S52, 0, &V23193_5_9981, &P23193_1};	/*Property = ScreenPixelsHeight*/
const Value V23193_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23193_4 = {S54, 0, &V23193_4_39098, &P23193_5};	/*Property = ScreenPixelsWidth*/
const Profile P23193 = {"23193", &P23193_4};	/*Profile ID - 23193*/
const Value V18738_1_1 = {S2, NULL};	/*Value = True*/
const Property P18738_1 = {S50, 0, &V18738_1_1, NULL};	/*Property = IsMobile*/
const Value V18738_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18738_5 = {S52, 0, &V18738_5_1227, &P18738_1};	/*Property = ScreenPixelsHeight*/
const Value V18738_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18738_4 = {S54, 0, &V18738_4_1229, &P18738_5};	/*Property = ScreenPixelsWidth*/
const Profile P18738 = {"18738", &P18738_4};	/*Profile ID - 18738*/
const Value V22459_1_1 = {S2, NULL};	/*Value = True*/
const Property P22459_1 = {S50, 0, &V22459_1_1, NULL};	/*Property = IsMobile*/
const Value V22459_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22459_5 = {S52, 0, &V22459_5_1227, &P22459_1};	/*Property = ScreenPixelsHeight*/
const Value V22459_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22459_4 = {S54, 0, &V22459_4_1229, &P22459_5};	/*Property = ScreenPixelsWidth*/
const Profile P22459 = {"22459", &P22459_4};	/*Profile ID - 22459*/
const Value V23270_1_1 = {S2, NULL};	/*Value = True*/
const Property P23270_1 = {S50, 0, &V23270_1_1, NULL};	/*Property = IsMobile*/
const Value V23270_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P23270_5 = {S52, 0, &V23270_5_1304, &P23270_1};	/*Property = ScreenPixelsHeight*/
const Value V23270_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23270_4 = {S54, 0, &V23270_4_2336, &P23270_5};	/*Property = ScreenPixelsWidth*/
const Profile P23270 = {"23270", &P23270_4};	/*Profile ID - 23270*/
const Value V21632_1_1 = {S2, NULL};	/*Value = True*/
const Property P21632_1 = {S50, 0, &V21632_1_1, NULL};	/*Property = IsMobile*/
const Value V21632_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21632_5 = {S52, 0, &V21632_5_1277, &P21632_1};	/*Property = ScreenPixelsHeight*/
const Value V21632_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21632_4 = {S54, 0, &V21632_4_1279, &P21632_5};	/*Property = ScreenPixelsWidth*/
const Profile P21632 = {"21632", &P21632_4};	/*Profile ID - 21632*/
const Value V19894_1_1 = {S2, NULL};	/*Value = True*/
const Property P19894_1 = {S50, 0, &V19894_1_1, NULL};	/*Property = IsMobile*/
const Value V19894_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19894_5 = {S52, 0, &V19894_5_1227, &P19894_1};	/*Property = ScreenPixelsHeight*/
const Value V19894_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19894_4 = {S54, 0, &V19894_4_1229, &P19894_5};	/*Property = ScreenPixelsWidth*/
const Profile P19894 = {"19894", &P19894_4};	/*Profile ID - 19894*/
const Value V23053_1_1 = {S2, NULL};	/*Value = True*/
const Property P23053_1 = {S50, 0, &V23053_1_1, NULL};	/*Property = IsMobile*/
const Value V23053_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23053_5 = {S52, 0, &V23053_5_1227, &P23053_1};	/*Property = ScreenPixelsHeight*/
const Value V23053_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23053_4 = {S54, 0, &V23053_4_1229, &P23053_5};	/*Property = ScreenPixelsWidth*/
const Profile P23053 = {"23053", &P23053_4};	/*Profile ID - 23053*/
const Value V20359_1_1 = {S2, NULL};	/*Value = True*/
const Property P20359_1 = {S50, 0, &V20359_1_1, NULL};	/*Property = IsMobile*/
const Value V20359_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20359_5 = {S52, 0, &V20359_5_9981, &P20359_1};	/*Property = ScreenPixelsHeight*/
const Value V20359_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20359_4 = {S54, 0, &V20359_4_39098, &P20359_5};	/*Property = ScreenPixelsWidth*/
const Profile P20359 = {"20359", &P20359_4};	/*Profile ID - 20359*/
const Value V22513_1_1 = {S2, NULL};	/*Value = True*/
const Property P22513_1 = {S50, 0, &V22513_1_1, NULL};	/*Property = IsMobile*/
const Value V22513_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22513_5 = {S52, 0, &V22513_5_1277, &P22513_1};	/*Property = ScreenPixelsHeight*/
const Value V22513_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22513_4 = {S54, 0, &V22513_4_1279, &P22513_5};	/*Property = ScreenPixelsWidth*/
const Profile P22513 = {"22513", &P22513_4};	/*Profile ID - 22513*/
const Value V21440_1_1 = {S2, NULL};	/*Value = True*/
const Property P21440_1 = {S50, 0, &V21440_1_1, NULL};	/*Property = IsMobile*/
const Value V21440_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21440_5 = {S52, 0, &V21440_5_1227, &P21440_1};	/*Property = ScreenPixelsHeight*/
const Value V21440_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21440_4 = {S54, 0, &V21440_4_1229, &P21440_5};	/*Property = ScreenPixelsWidth*/
const Profile P21440 = {"21440", &P21440_4};	/*Profile ID - 21440*/
const Value V21871_1_1 = {S2, NULL};	/*Value = True*/
const Property P21871_1 = {S50, 0, &V21871_1_1, NULL};	/*Property = IsMobile*/
const Value V21871_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21871_5 = {S52, 0, &V21871_5_1227, &P21871_1};	/*Property = ScreenPixelsHeight*/
const Value V21871_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21871_4 = {S54, 0, &V21871_4_1229, &P21871_5};	/*Property = ScreenPixelsWidth*/
const Profile P21871 = {"21871", &P21871_4};	/*Profile ID - 21871*/
const Value V20051_1_1 = {S2, NULL};	/*Value = True*/
const Property P20051_1 = {S50, 0, &V20051_1_1, NULL};	/*Property = IsMobile*/
const Value V20051_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20051_5 = {S52, 0, &V20051_5_1222, &P20051_1};	/*Property = ScreenPixelsHeight*/
const Value V20051_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20051_4 = {S54, 0, &V20051_4_1224, &P20051_5};	/*Property = ScreenPixelsWidth*/
const Profile P20051 = {"20051", &P20051_4};	/*Profile ID - 20051*/
const Value V22355_1_1 = {S2, NULL};	/*Value = True*/
const Property P22355_1 = {S50, 0, &V22355_1_1, NULL};	/*Property = IsMobile*/
const Value V22355_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22355_5 = {S52, 0, &V22355_5_1832, &P22355_1};	/*Property = ScreenPixelsHeight*/
const Value V22355_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22355_4 = {S54, 0, &V22355_4_5645, &P22355_5};	/*Property = ScreenPixelsWidth*/
const Profile P22355 = {"22355", &P22355_4};	/*Profile ID - 22355*/
const Value V21278_1_1 = {S2, NULL};	/*Value = True*/
const Property P21278_1 = {S50, 0, &V21278_1_1, NULL};	/*Property = IsMobile*/
const Value V21278_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21278_5 = {S52, 0, &V21278_5_37, &P21278_1};	/*Property = ScreenPixelsHeight*/
const Value V21278_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21278_4 = {S54, 0, &V21278_4_36, &P21278_5};	/*Property = ScreenPixelsWidth*/
const Profile P21278 = {"21278", &P21278_4};	/*Profile ID - 21278*/
const Value V20239_1_1 = {S2, NULL};	/*Value = True*/
const Property P20239_1 = {S50, 0, &V20239_1_1, NULL};	/*Property = IsMobile*/
const Value V20239_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20239_5 = {S52, 0, &V20239_5_1458, &P20239_1};	/*Property = ScreenPixelsHeight*/
const Value V20239_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20239_4 = {S54, 0, &V20239_4_37617, &P20239_5};	/*Property = ScreenPixelsWidth*/
const Profile P20239 = {"20239", &P20239_4};	/*Profile ID - 20239*/
const Value V21613_1_1 = {S2, NULL};	/*Value = True*/
const Property P21613_1 = {S50, 0, &V21613_1_1, NULL};	/*Property = IsMobile*/
const Value V21613_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21613_5 = {S52, 0, &V21613_5_1458, &P21613_1};	/*Property = ScreenPixelsHeight*/
const Value V21613_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21613_4 = {S54, 0, &V21613_4_1460, &P21613_5};	/*Property = ScreenPixelsWidth*/
const Profile P21613 = {"21613", &P21613_4};	/*Profile ID - 21613*/
const Value V23343_1_1 = {S2, NULL};	/*Value = True*/
const Property P23343_1 = {S50, 0, &V23343_1_1, NULL};	/*Property = IsMobile*/
const Value V23343_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23343_5 = {S52, 0, &V23343_5_1277, &P23343_1};	/*Property = ScreenPixelsHeight*/
const Value V23343_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23343_4 = {S54, 0, &V23343_4_1279, &P23343_5};	/*Property = ScreenPixelsWidth*/
const Profile P23343 = {"23343", &P23343_4};	/*Profile ID - 23343*/
const Value V23746_1_1 = {S2, NULL};	/*Value = True*/
const Property P23746_1 = {S50, 0, &V23746_1_1, NULL};	/*Property = IsMobile*/
const Value V23746_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23746_5 = {S52, 0, &V23746_5_1832, &P23746_1};	/*Property = ScreenPixelsHeight*/
const Value V23746_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23746_4 = {S54, 0, &V23746_4_5645, &P23746_5};	/*Property = ScreenPixelsWidth*/
const Profile P23746 = {"23746", &P23746_4};	/*Profile ID - 23746*/
const Value V22140_1_1 = {S2, NULL};	/*Value = True*/
const Property P22140_1 = {S50, 0, &V22140_1_1, NULL};	/*Property = IsMobile*/
const Value V22140_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22140_5 = {S52, 0, &V22140_5_1458, &P22140_1};	/*Property = ScreenPixelsHeight*/
const Value V22140_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P22140_4 = {S54, 0, &V22140_4_37617, &P22140_5};	/*Property = ScreenPixelsWidth*/
const Profile P22140 = {"22140", &P22140_4};	/*Profile ID - 22140*/
const Value V20179_1_1 = {S2, NULL};	/*Value = True*/
const Property P20179_1 = {S50, 0, &V20179_1_1, NULL};	/*Property = IsMobile*/
const Value V20179_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20179_5 = {S52, 0, &V20179_5_1227, &P20179_1};	/*Property = ScreenPixelsHeight*/
const Value V20179_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20179_4 = {S54, 0, &V20179_4_1229, &P20179_5};	/*Property = ScreenPixelsWidth*/
const Profile P20179 = {"20179", &P20179_4};	/*Profile ID - 20179*/
const Value V23318_1_1 = {S2, NULL};	/*Value = True*/
const Property P23318_1 = {S50, 0, &V23318_1_1, NULL};	/*Property = IsMobile*/
const Value V23318_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23318_5 = {S52, 0, &V23318_5_1277, &P23318_1};	/*Property = ScreenPixelsHeight*/
const Value V23318_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23318_4 = {S54, 0, &V23318_4_2336, &P23318_5};	/*Property = ScreenPixelsWidth*/
const Profile P23318 = {"23318", &P23318_4};	/*Profile ID - 23318*/
const Value V21972_1_1 = {S2, NULL};	/*Value = True*/
const Property P21972_1 = {S50, 0, &V21972_1_1, NULL};	/*Property = IsMobile*/
const Value V21972_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21972_5 = {S52, 0, &V21972_5_1277, &P21972_1};	/*Property = ScreenPixelsHeight*/
const Value V21972_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21972_4 = {S54, 0, &V21972_4_2336, &P21972_5};	/*Property = ScreenPixelsWidth*/
const Profile P21972 = {"21972", &P21972_4};	/*Profile ID - 21972*/
const Value V23952_1_1 = {S2, NULL};	/*Value = True*/
const Property P23952_1 = {S50, 0, &V23952_1_1, NULL};	/*Property = IsMobile*/
const Value V23952_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23952_5 = {S52, 0, &V23952_5_37, &P23952_1};	/*Property = ScreenPixelsHeight*/
const Value V23952_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23952_4 = {S54, 0, &V23952_4_36, &P23952_5};	/*Property = ScreenPixelsWidth*/
const Profile P23952 = {"23952", &P23952_4};	/*Profile ID - 23952*/
const Value V21952_1_1 = {S2, NULL};	/*Value = True*/
const Property P21952_1 = {S50, 0, &V21952_1_1, NULL};	/*Property = IsMobile*/
const Value V21952_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21952_5 = {S52, 0, &V21952_5_1458, &P21952_1};	/*Property = ScreenPixelsHeight*/
const Value V21952_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21952_4 = {S54, 0, &V21952_4_37617, &P21952_5};	/*Property = ScreenPixelsWidth*/
const Profile P21952 = {"21952", &P21952_4};	/*Profile ID - 21952*/
const Value V20534_1_1 = {S2, NULL};	/*Value = True*/
const Property P20534_1 = {S50, 0, &V20534_1_1, NULL};	/*Property = IsMobile*/
const Value V20534_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20534_5 = {S52, 0, &V20534_5_1227, &P20534_1};	/*Property = ScreenPixelsHeight*/
const Value V20534_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20534_4 = {S54, 0, &V20534_4_1229, &P20534_5};	/*Property = ScreenPixelsWidth*/
const Profile P20534 = {"20534", &P20534_4};	/*Profile ID - 20534*/
const Value V21731_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_279 = {S1, 0, &V21731_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21731_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_282 = {S3, 0, &V21731_282_57155, &P21731_279};	/*Property = BlobBuilder*/
const Value V21731_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_350 = {S4, 0, &V21731_350_57446, &P21731_282};	/*Property = Canvas*/
const Value V21731_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_283 = {S5, 0, &V21731_283_57161, &P21731_350};	/*Property = CssBackground*/
const Value V21731_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_284 = {S6, 0, &V21731_284_57164, &P21731_283};	/*Property = CssBorderImage*/
const Value V21731_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_285 = {S7, 0, &V21731_285_57167, &P21731_284};	/*Property = CssCanvas*/
const Value V21731_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_286 = {S8, 0, &V21731_286_57170, &P21731_285};	/*Property = CssColor*/
const Value V21731_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_237 = {S9, 0, &V21731_237_55786, &P21731_286};	/*Property = CssColumn*/
const Value V21731_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_287 = {S10, 0, &V21731_287_57173, &P21731_237};	/*Property = CssFlexbox*/
const Value V21731_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_288 = {S11, 0, &V21731_288_57176, &P21731_287};	/*Property = CssFont*/
const Value V21731_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_289 = {S12, 0, &V21731_289_86899, &P21731_288};	/*Property = CssImages*/
const Value V21731_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_291 = {S13, 0, &V21731_291_57182, &P21731_289};	/*Property = CssMediaQueries*/
const Value V21731_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_292 = {S14, 0, &V21731_292_57185, &P21731_291};	/*Property = CssMinMax*/
const Value V21731_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_293 = {S15, 0, &V21731_293_57188, &P21731_292};	/*Property = CssOverflow*/
const Value V21731_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_294 = {S16, 0, &V21731_294_57191, &P21731_293};	/*Property = CssPosition*/
const Value V21731_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_295 = {S17, 0, &V21731_295_57194, &P21731_294};	/*Property = CssText*/
const Value V21731_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_297 = {S18, 0, &V21731_297_57197, &P21731_295};	/*Property = CssTransforms*/
const Value V21731_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_241 = {S19, 0, &V21731_241_55801, &P21731_297};	/*Property = CssTransitions*/
const Value V21731_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_298 = {S20, 0, &V21731_298_57208, &P21731_241};	/*Property = CssUI*/
const Value V21731_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_300 = {S21, 0, &V21731_300_57217, &P21731_298};	/*Property = DataSet*/
const Value V21731_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_301 = {S22, 0, &V21731_301_57220, &P21731_300};	/*Property = DataUrl*/
const Value V21731_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_302 = {S23, 0, &V21731_302_57223, &P21731_301};	/*Property = DeviceOrientation*/
const Value V21731_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_304 = {S24, 0, &V21731_304_57229, &P21731_302};	/*Property = FileReader*/
const Value V21731_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_305 = {S25, 0, &V21731_305_57232, &P21731_304};	/*Property = FileSaver*/
const Value V21731_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_306 = {S26, 0, &V21731_306_57238, &P21731_305};	/*Property = FileWriter*/
const Value V21731_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_307 = {S27, 0, &V21731_307_57241, &P21731_306};	/*Property = FormData*/
const Value V21731_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_309 = {S28, 0, &V21731_309_57247, &P21731_307};	/*Property = Fullscreen*/
const Value V21731_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_310 = {S29, 0, &V21731_310_57250, &P21731_309};	/*Property = GeoLocation*/
const Value V21731_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_312 = {S30, 0, &V21731_312_57256, &P21731_310};	/*Property = History*/
const Value V21731_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_314 = {S31, 0, &V21731_314_57262, &P21731_312};	/*Property = Html5*/
const Value V21731_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_313 = {S32, 0, &V21731_313_57259, &P21731_314};	/*Property = Html-Media-Capture*/
const Value V21731_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_315 = {S33, 0, &V21731_315_57265, &P21731_313};	/*Property = Iframe*/
const Value V21731_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_316 = {S34, 0, &V21731_316_57268, &P21731_315};	/*Property = IndexedDB*/
const Value V21731_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_317 = {S35, 0, &V21731_317_57271, &P21731_316};	/*Property = Json*/
const Value V21731_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21731_15 = {S37, 0, &V21731_15_54167, &P21731_317};	/*Property = LayoutEngine*/
const Value V21731_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_344 = {S38, 0, &V21731_344_57274, &P21731_15};	/*Property = Masking*/
const Value V21731_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_320 = {S39, 0, &V21731_320_57292, &P21731_344};	/*Property = PostMessage*/
const Value V21731_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_321 = {S40, 0, &V21731_321_57295, &P21731_320};	/*Property = Progress*/
const Value V21731_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_322 = {S41, 0, &V21731_322_57298, &P21731_321};	/*Property = Prompts*/
const Value V21731_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_323 = {S42, 0, &V21731_323_57301, &P21731_322};	/*Property = Selector*/
const Value V21731_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_324 = {S43, 0, &V21731_324_57307, &P21731_323};	/*Property = Svg*/
const Value V21731_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_327 = {S44, 0, &V21731_327_57316, &P21731_324};	/*Property = TouchEvents*/
const Value V21731_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_328 = {S45, 0, &V21731_328_57319, &P21731_327};	/*Property = Track*/
const Value V21731_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_329 = {S46, 0, &V21731_329_57325, &P21731_328};	/*Property = Video*/
const Value V21731_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_330 = {S47, 0, &V21731_330_57328, &P21731_329};	/*Property = Viewport*/
const Value V21731_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_332 = {S48, 0, &V21731_332_57340, &P21731_330};	/*Property = WebWorkers*/
const Value V21731_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21731_333 = {S49, 0, &V21731_333_57343, &P21731_332};	/*Property = Xhr2*/
const Profile P21731 = {"21731", &P21731_333};	/*Profile ID - 21731*/
const Value V20535_1_1 = {S2, NULL};	/*Value = True*/
const Property P20535_1 = {S50, 0, &V20535_1_1, NULL};	/*Property = IsMobile*/
const Value V20535_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20535_5 = {S52, 0, &V20535_5_1277, &P20535_1};	/*Property = ScreenPixelsHeight*/
const Value V20535_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20535_4 = {S54, 0, &V20535_4_1279, &P20535_5};	/*Property = ScreenPixelsWidth*/
const Profile P20535 = {"20535", &P20535_4};	/*Profile ID - 20535*/
const Value V21415_1_1 = {S2, NULL};	/*Value = True*/
const Property P21415_1 = {S50, 0, &V21415_1_1, NULL};	/*Property = IsMobile*/
const Value V21415_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21415_5 = {S52, 0, &V21415_5_1227, &P21415_1};	/*Property = ScreenPixelsHeight*/
const Value V21415_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21415_4 = {S54, 0, &V21415_4_1229, &P21415_5};	/*Property = ScreenPixelsWidth*/
const Profile P21415 = {"21415", &P21415_4};	/*Profile ID - 21415*/
const Value V20596_1_1 = {S2, NULL};	/*Value = True*/
const Property P20596_1 = {S50, 0, &V20596_1_1, NULL};	/*Property = IsMobile*/
const Value V20596_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20596_5 = {S52, 0, &V20596_5_1227, &P20596_1};	/*Property = ScreenPixelsHeight*/
const Value V20596_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20596_4 = {S54, 0, &V20596_4_1229, &P20596_5};	/*Property = ScreenPixelsWidth*/
const Profile P20596 = {"20596", &P20596_4};	/*Profile ID - 20596*/
const Value V21474_1_1 = {S2, NULL};	/*Value = True*/
const Property P21474_1 = {S50, 0, &V21474_1_1, NULL};	/*Property = IsMobile*/
const Value V21474_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21474_5 = {S52, 0, &V21474_5_1277, &P21474_1};	/*Property = ScreenPixelsHeight*/
const Value V21474_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21474_4 = {S54, 0, &V21474_4_1279, &P21474_5};	/*Property = ScreenPixelsWidth*/
const Profile P21474 = {"21474", &P21474_4};	/*Profile ID - 21474*/
const Value V21730_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_279 = {S1, 0, &V21730_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21730_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_282 = {S3, 0, &V21730_282_57155, &P21730_279};	/*Property = BlobBuilder*/
const Value V21730_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_350 = {S4, 0, &V21730_350_57446, &P21730_282};	/*Property = Canvas*/
const Value V21730_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_283 = {S5, 0, &V21730_283_57161, &P21730_350};	/*Property = CssBackground*/
const Value V21730_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_284 = {S6, 0, &V21730_284_57164, &P21730_283};	/*Property = CssBorderImage*/
const Value V21730_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_285 = {S7, 0, &V21730_285_57167, &P21730_284};	/*Property = CssCanvas*/
const Value V21730_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_286 = {S8, 0, &V21730_286_57170, &P21730_285};	/*Property = CssColor*/
const Value V21730_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_237 = {S9, 0, &V21730_237_55786, &P21730_286};	/*Property = CssColumn*/
const Value V21730_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_287 = {S10, 0, &V21730_287_57173, &P21730_237};	/*Property = CssFlexbox*/
const Value V21730_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_288 = {S11, 0, &V21730_288_57176, &P21730_287};	/*Property = CssFont*/
const Value V21730_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_289 = {S12, 0, &V21730_289_86899, &P21730_288};	/*Property = CssImages*/
const Value V21730_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_291 = {S13, 0, &V21730_291_57182, &P21730_289};	/*Property = CssMediaQueries*/
const Value V21730_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_292 = {S14, 0, &V21730_292_57185, &P21730_291};	/*Property = CssMinMax*/
const Value V21730_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_293 = {S15, 0, &V21730_293_57188, &P21730_292};	/*Property = CssOverflow*/
const Value V21730_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_294 = {S16, 0, &V21730_294_57191, &P21730_293};	/*Property = CssPosition*/
const Value V21730_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_295 = {S17, 0, &V21730_295_57194, &P21730_294};	/*Property = CssText*/
const Value V21730_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_297 = {S18, 0, &V21730_297_57197, &P21730_295};	/*Property = CssTransforms*/
const Value V21730_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_241 = {S19, 0, &V21730_241_55801, &P21730_297};	/*Property = CssTransitions*/
const Value V21730_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_298 = {S20, 0, &V21730_298_57208, &P21730_241};	/*Property = CssUI*/
const Value V21730_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_300 = {S21, 0, &V21730_300_57217, &P21730_298};	/*Property = DataSet*/
const Value V21730_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_301 = {S22, 0, &V21730_301_57220, &P21730_300};	/*Property = DataUrl*/
const Value V21730_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_302 = {S23, 0, &V21730_302_57223, &P21730_301};	/*Property = DeviceOrientation*/
const Value V21730_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_304 = {S24, 0, &V21730_304_57229, &P21730_302};	/*Property = FileReader*/
const Value V21730_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_305 = {S25, 0, &V21730_305_57232, &P21730_304};	/*Property = FileSaver*/
const Value V21730_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_306 = {S26, 0, &V21730_306_57238, &P21730_305};	/*Property = FileWriter*/
const Value V21730_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_307 = {S27, 0, &V21730_307_57241, &P21730_306};	/*Property = FormData*/
const Value V21730_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_309 = {S28, 0, &V21730_309_57247, &P21730_307};	/*Property = Fullscreen*/
const Value V21730_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_310 = {S29, 0, &V21730_310_57250, &P21730_309};	/*Property = GeoLocation*/
const Value V21730_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_312 = {S30, 0, &V21730_312_57256, &P21730_310};	/*Property = History*/
const Value V21730_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_314 = {S31, 0, &V21730_314_57262, &P21730_312};	/*Property = Html5*/
const Value V21730_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_313 = {S32, 0, &V21730_313_57259, &P21730_314};	/*Property = Html-Media-Capture*/
const Value V21730_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_315 = {S33, 0, &V21730_315_57265, &P21730_313};	/*Property = Iframe*/
const Value V21730_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_316 = {S34, 0, &V21730_316_57268, &P21730_315};	/*Property = IndexedDB*/
const Value V21730_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_317 = {S35, 0, &V21730_317_57271, &P21730_316};	/*Property = Json*/
const Value V21730_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21730_15 = {S37, 0, &V21730_15_54167, &P21730_317};	/*Property = LayoutEngine*/
const Value V21730_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_344 = {S38, 0, &V21730_344_57274, &P21730_15};	/*Property = Masking*/
const Value V21730_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_320 = {S39, 0, &V21730_320_57292, &P21730_344};	/*Property = PostMessage*/
const Value V21730_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_321 = {S40, 0, &V21730_321_57295, &P21730_320};	/*Property = Progress*/
const Value V21730_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_322 = {S41, 0, &V21730_322_57298, &P21730_321};	/*Property = Prompts*/
const Value V21730_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_323 = {S42, 0, &V21730_323_57301, &P21730_322};	/*Property = Selector*/
const Value V21730_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_324 = {S43, 0, &V21730_324_57307, &P21730_323};	/*Property = Svg*/
const Value V21730_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_327 = {S44, 0, &V21730_327_57316, &P21730_324};	/*Property = TouchEvents*/
const Value V21730_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_328 = {S45, 0, &V21730_328_57319, &P21730_327};	/*Property = Track*/
const Value V21730_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_329 = {S46, 0, &V21730_329_57325, &P21730_328};	/*Property = Video*/
const Value V21730_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_330 = {S47, 0, &V21730_330_57328, &P21730_329};	/*Property = Viewport*/
const Value V21730_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_332 = {S48, 0, &V21730_332_57340, &P21730_330};	/*Property = WebWorkers*/
const Value V21730_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21730_333 = {S49, 0, &V21730_333_57343, &P21730_332};	/*Property = Xhr2*/
const Profile P21730 = {"21730", &P21730_333};	/*Profile ID - 21730*/
const Value V20807_1_1 = {S2, NULL};	/*Value = True*/
const Property P20807_1 = {S50, 0, &V20807_1_1, NULL};	/*Property = IsMobile*/
const Value V20807_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20807_5 = {S52, 0, &V20807_5_9981, &P20807_1};	/*Property = ScreenPixelsHeight*/
const Value V20807_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20807_4 = {S54, 0, &V20807_4_39098, &P20807_5};	/*Property = ScreenPixelsWidth*/
const Profile P20807 = {"20807", &P20807_4};	/*Profile ID - 20807*/
const Value V23056_1_1 = {S2, NULL};	/*Value = True*/
const Property P23056_1 = {S50, 0, &V23056_1_1, NULL};	/*Property = IsMobile*/
const Value V23056_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P23056_5 = {S52, 0, &V23056_5_1304, &P23056_1};	/*Property = ScreenPixelsHeight*/
const Value V23056_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23056_4 = {S54, 0, &V23056_4_1224, &P23056_5};	/*Property = ScreenPixelsWidth*/
const Profile P23056 = {"23056", &P23056_4};	/*Profile ID - 23056*/
const Value V24100_1_1 = {S2, NULL};	/*Value = True*/
const Property P24100_1 = {S50, 0, &V24100_1_1, NULL};	/*Property = IsMobile*/
const Value V24100_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24100_5 = {S52, 0, &V24100_5_1227, &P24100_1};	/*Property = ScreenPixelsHeight*/
const Value V24100_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24100_4 = {S54, 0, &V24100_4_1229, &P24100_5};	/*Property = ScreenPixelsWidth*/
const Profile P24100 = {"24100", &P24100_4};	/*Profile ID - 24100*/
const Value V24152_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24152_279 = {S1, 0, &V24152_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24152_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24152_282 = {S3, 0, &V24152_282_57154, &P24152_279};	/*Property = BlobBuilder*/
const Value V24152_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24152_350 = {S4, 0, &V24152_350_57444, &P24152_282};	/*Property = Canvas*/
const Value V24152_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24152_283 = {S5, 0, &V24152_283_57159, &P24152_350};	/*Property = CssBackground*/
const Value V24152_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24152_284 = {S6, 0, &V24152_284_57162, &P24152_283};	/*Property = CssBorderImage*/
const Value V24152_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24152_285 = {S7, 0, &V24152_285_57166, &P24152_284};	/*Property = CssCanvas*/
const Value V24152_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24152_286 = {S8, 0, &V24152_286_57168, &P24152_285};	/*Property = CssColor*/
const Value V24152_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24152_237 = {S9, 0, &V24152_237_55786, &P24152_286};	/*Property = CssColumn*/
const Value V24152_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24152_287 = {S10, 0, &V24152_287_57172, &P24152_237};	/*Property = CssFlexbox*/
const Value V24152_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24152_288 = {S11, 0, &V24152_288_57174, &P24152_287};	/*Property = CssFont*/
const Value V24152_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24152_289 = {S12, 0, &V24152_289_57759, &P24152_288};	/*Property = CssImages*/
const Value V24152_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24152_291 = {S13, 0, &V24152_291_57180, &P24152_289};	/*Property = CssMediaQueries*/
const Value V24152_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24152_292 = {S14, 0, &V24152_292_57183, &P24152_291};	/*Property = CssMinMax*/
const Value V24152_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24152_293 = {S15, 0, &V24152_293_57187, &P24152_292};	/*Property = CssOverflow*/
const Value V24152_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24152_294 = {S16, 0, &V24152_294_57189, &P24152_293};	/*Property = CssPosition*/
const Value V24152_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24152_295 = {S17, 0, &V24152_295_57192, &P24152_294};	/*Property = CssText*/
const Value V24152_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24152_297 = {S18, 0, &V24152_297_57195, &P24152_295};	/*Property = CssTransforms*/
const Value V24152_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24152_241 = {S19, 0, &V24152_241_55799, &P24152_297};	/*Property = CssTransitions*/
const Value V24152_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24152_298 = {S20, 0, &V24152_298_57206, &P24152_241};	/*Property = CssUI*/
const Value V24152_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24152_300 = {S21, 0, &V24152_300_57215, &P24152_298};	/*Property = DataSet*/
const Value V24152_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24152_301 = {S22, 0, &V24152_301_57218, &P24152_300};	/*Property = DataUrl*/
const Value V24152_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24152_302 = {S23, 0, &V24152_302_57221, &P24152_301};	/*Property = DeviceOrientation*/
const Value V24152_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24152_304 = {S24, 0, &V24152_304_57227, &P24152_302};	/*Property = FileReader*/
const Value V24152_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24152_305 = {S25, 0, &V24152_305_57231, &P24152_304};	/*Property = FileSaver*/
const Value V24152_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24152_306 = {S26, 0, &V24152_306_57237, &P24152_305};	/*Property = FileWriter*/
const Value V24152_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24152_307 = {S27, 0, &V24152_307_57239, &P24152_306};	/*Property = FormData*/
const Value V24152_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24152_309 = {S28, 0, &V24152_309_57246, &P24152_307};	/*Property = Fullscreen*/
const Value V24152_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24152_310 = {S29, 0, &V24152_310_57248, &P24152_309};	/*Property = GeoLocation*/
const Value V24152_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24152_312 = {S30, 0, &V24152_312_57254, &P24152_310};	/*Property = History*/
const Value V24152_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24152_314 = {S31, 0, &V24152_314_57260, &P24152_312};	/*Property = Html5*/
const Value V24152_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24152_313 = {S32, 0, &V24152_313_57258, &P24152_314};	/*Property = Html-Media-Capture*/
const Value V24152_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24152_315 = {S33, 0, &V24152_315_57264, &P24152_313};	/*Property = Iframe*/
const Value V24152_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24152_316 = {S34, 0, &V24152_316_57266, &P24152_315};	/*Property = IndexedDB*/
const Value V24152_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24152_317 = {S35, 0, &V24152_317_57269, &P24152_316};	/*Property = Json*/
const Value V24152_15_81502 = {S141, NULL};	/*Value = Blink*/
const Property P24152_15 = {S37, 0, &V24152_15_81502, &P24152_317};	/*Property = LayoutEngine*/
const Value V24152_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24152_344 = {S38, 0, &V24152_344_57272, &P24152_15};	/*Property = Masking*/
const Value V24152_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24152_320 = {S39, 0, &V24152_320_57290, &P24152_344};	/*Property = PostMessage*/
const Value V24152_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24152_321 = {S40, 0, &V24152_321_57293, &P24152_320};	/*Property = Progress*/
const Value V24152_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24152_322 = {S41, 0, &V24152_322_57296, &P24152_321};	/*Property = Prompts*/
const Value V24152_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24152_323 = {S42, 0, &V24152_323_57299, &P24152_322};	/*Property = Selector*/
const Value V24152_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24152_324 = {S43, 0, &V24152_324_57305, &P24152_323};	/*Property = Svg*/
const Value V24152_327_57315 = {S0, NULL};	/*Value = False*/
const Property P24152_327 = {S44, 0, &V24152_327_57315, &P24152_324};	/*Property = TouchEvents*/
const Value V24152_328_57317 = {S2, NULL};	/*Value = True*/
const Property P24152_328 = {S45, 0, &V24152_328_57317, &P24152_327};	/*Property = Track*/
const Value V24152_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24152_329 = {S46, 0, &V24152_329_57323, &P24152_328};	/*Property = Video*/
const Value V24152_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24152_330 = {S47, 0, &V24152_330_57326, &P24152_329};	/*Property = Viewport*/
const Value V24152_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24152_332 = {S48, 0, &V24152_332_57338, &P24152_330};	/*Property = WebWorkers*/
const Value V24152_333_57341 = {S2, NULL};	/*Value = True*/
const Property P24152_333 = {S49, 0, &V24152_333_57341, &P24152_332};	/*Property = Xhr2*/
const Profile P24152 = {"24152", &P24152_333};	/*Profile ID - 24152*/
const Value V23515_1_1 = {S2, NULL};	/*Value = True*/
const Property P23515_1 = {S50, 0, &V23515_1_1, NULL};	/*Property = IsMobile*/
const Value V23515_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23515_5 = {S52, 0, &V23515_5_1458, &P23515_1};	/*Property = ScreenPixelsHeight*/
const Value V23515_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23515_4 = {S54, 0, &V23515_4_1460, &P23515_5};	/*Property = ScreenPixelsWidth*/
const Profile P23515 = {"23515", &P23515_4};	/*Profile ID - 23515*/
const Value V21258_1_1 = {S2, NULL};	/*Value = True*/
const Property P21258_1 = {S50, 0, &V21258_1_1, NULL};	/*Property = IsMobile*/
const Value V21258_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21258_5 = {S52, 0, &V21258_5_1458, &P21258_1};	/*Property = ScreenPixelsHeight*/
const Value V21258_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21258_4 = {S54, 0, &V21258_4_1460, &P21258_5};	/*Property = ScreenPixelsWidth*/
const Profile P21258 = {"21258", &P21258_4};	/*Profile ID - 21258*/
const Value V20344_1_1 = {S2, NULL};	/*Value = True*/
const Property P20344_1 = {S50, 0, &V20344_1_1, NULL};	/*Property = IsMobile*/
const Value V20344_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20344_5 = {S52, 0, &V20344_5_1458, &P20344_1};	/*Property = ScreenPixelsHeight*/
const Value V20344_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20344_4 = {S54, 0, &V20344_4_37617, &P20344_5};	/*Property = ScreenPixelsWidth*/
const Profile P20344 = {"20344", &P20344_4};	/*Profile ID - 20344*/
const Value V23507_1_1 = {S2, NULL};	/*Value = True*/
const Property P23507_1 = {S50, 0, &V23507_1_1, NULL};	/*Property = IsMobile*/
const Value V23507_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23507_5 = {S52, 0, &V23507_5_1227, &P23507_1};	/*Property = ScreenPixelsHeight*/
const Value V23507_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23507_4 = {S54, 0, &V23507_4_1229, &P23507_5};	/*Property = ScreenPixelsWidth*/
const Profile P23507 = {"23507", &P23507_4};	/*Profile ID - 23507*/
const Value V20357_1_1 = {S2, NULL};	/*Value = True*/
const Property P20357_1 = {S50, 0, &V20357_1_1, NULL};	/*Property = IsMobile*/
const Value V20357_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20357_5 = {S52, 0, &V20357_5_1458, &P20357_1};	/*Property = ScreenPixelsHeight*/
const Value V20357_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20357_4 = {S54, 0, &V20357_4_1460, &P20357_5};	/*Property = ScreenPixelsWidth*/
const Profile P20357 = {"20357", &P20357_4};	/*Profile ID - 20357*/
const Value V19801_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19801_279 = {S1, 0, &V19801_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19801_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19801_282 = {S3, 0, &V19801_282_57154, &P19801_279};	/*Property = BlobBuilder*/
const Value V19801_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19801_350 = {S4, 0, &V19801_350_57444, &P19801_282};	/*Property = Canvas*/
const Value V19801_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19801_283 = {S5, 0, &V19801_283_57159, &P19801_350};	/*Property = CssBackground*/
const Value V19801_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19801_284 = {S6, 0, &V19801_284_57163, &P19801_283};	/*Property = CssBorderImage*/
const Value V19801_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19801_285 = {S7, 0, &V19801_285_57166, &P19801_284};	/*Property = CssCanvas*/
const Value V19801_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19801_286 = {S8, 0, &V19801_286_57168, &P19801_285};	/*Property = CssColor*/
const Value V19801_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19801_237 = {S9, 0, &V19801_237_55786, &P19801_286};	/*Property = CssColumn*/
const Value V19801_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19801_287 = {S10, 0, &V19801_287_57172, &P19801_237};	/*Property = CssFlexbox*/
const Value V19801_288_57174 = {S2, NULL};	/*Value = True*/
const Property P19801_288 = {S11, 0, &V19801_288_57174, &P19801_287};	/*Property = CssFont*/
const Value V19801_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19801_289 = {S12, 0, &V19801_289_57759, &P19801_288};	/*Property = CssImages*/
const Value V19801_291_57180 = {S2, NULL};	/*Value = True*/
const Property P19801_291 = {S13, 0, &V19801_291_57180, &P19801_289};	/*Property = CssMediaQueries*/
const Value V19801_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19801_292 = {S14, 0, &V19801_292_57183, &P19801_291};	/*Property = CssMinMax*/
const Value V19801_293_57186 = {S2, NULL};	/*Value = True*/
const Property P19801_293 = {S15, 0, &V19801_293_57186, &P19801_292};	/*Property = CssOverflow*/
const Value V19801_294_57189 = {S2, NULL};	/*Value = True*/
const Property P19801_294 = {S16, 0, &V19801_294_57189, &P19801_293};	/*Property = CssPosition*/
const Value V19801_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19801_295 = {S17, 0, &V19801_295_57192, &P19801_294};	/*Property = CssText*/
const Value V19801_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19801_297 = {S18, 0, &V19801_297_57195, &P19801_295};	/*Property = CssTransforms*/
const Value V19801_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19801_241 = {S19, 0, &V19801_241_55799, &P19801_297};	/*Property = CssTransitions*/
const Value V19801_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19801_298 = {S20, 0, &V19801_298_57206, &P19801_241};	/*Property = CssUI*/
const Value V19801_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19801_300 = {S21, 0, &V19801_300_57216, &P19801_298};	/*Property = DataSet*/
const Value V19801_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19801_301 = {S22, 0, &V19801_301_57218, &P19801_300};	/*Property = DataUrl*/
const Value V19801_302_57221 = {S2, NULL};	/*Value = True*/
const Property P19801_302 = {S23, 0, &V19801_302_57221, &P19801_301};	/*Property = DeviceOrientation*/
const Value V19801_304_57227 = {S2, NULL};	/*Value = True*/
const Property P19801_304 = {S24, 0, &V19801_304_57227, &P19801_302};	/*Property = FileReader*/
const Value V19801_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19801_305 = {S25, 0, &V19801_305_57231, &P19801_304};	/*Property = FileSaver*/
const Value V19801_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19801_306 = {S26, 0, &V19801_306_57237, &P19801_305};	/*Property = FileWriter*/
const Value V19801_307_57239 = {S2, NULL};	/*Value = True*/
const Property P19801_307 = {S27, 0, &V19801_307_57239, &P19801_306};	/*Property = FormData*/
const Value V19801_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19801_309 = {S28, 0, &V19801_309_57246, &P19801_307};	/*Property = Fullscreen*/
const Value V19801_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19801_310 = {S29, 0, &V19801_310_57248, &P19801_309};	/*Property = GeoLocation*/
const Value V19801_312_57254 = {S2, NULL};	/*Value = True*/
const Property P19801_312 = {S30, 0, &V19801_312_57254, &P19801_310};	/*Property = History*/
const Value V19801_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19801_314 = {S31, 0, &V19801_314_57261, &P19801_312};	/*Property = Html5*/
const Value V19801_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19801_313 = {S32, 0, &V19801_313_57258, &P19801_314};	/*Property = Html-Media-Capture*/
const Value V19801_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19801_315 = {S33, 0, &V19801_315_57264, &P19801_313};	/*Property = Iframe*/
const Value V19801_316_57267 = {S0, NULL};	/*Value = False*/
const Property P19801_316 = {S34, 0, &V19801_316_57267, &P19801_315};	/*Property = IndexedDB*/
const Value V19801_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19801_317 = {S35, 0, &V19801_317_57269, &P19801_316};	/*Property = Json*/
const Value V19801_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19801_15 = {S37, 0, &V19801_15_28, &P19801_317};	/*Property = LayoutEngine*/
const Value V19801_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19801_344 = {S38, 0, &V19801_344_57272, &P19801_15};	/*Property = Masking*/
const Value V19801_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19801_320 = {S39, 0, &V19801_320_57290, &P19801_344};	/*Property = PostMessage*/
const Value V19801_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19801_321 = {S40, 0, &V19801_321_57293, &P19801_320};	/*Property = Progress*/
const Value V19801_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19801_322 = {S41, 0, &V19801_322_57296, &P19801_321};	/*Property = Prompts*/
const Value V19801_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19801_323 = {S42, 0, &V19801_323_57299, &P19801_322};	/*Property = Selector*/
const Value V19801_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19801_324 = {S43, 0, &V19801_324_57306, &P19801_323};	/*Property = Svg*/
const Value V19801_327_57314 = {S2, NULL};	/*Value = True*/
const Property P19801_327 = {S44, 0, &V19801_327_57314, &P19801_324};	/*Property = TouchEvents*/
const Value V19801_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19801_328 = {S45, 0, &V19801_328_57318, &P19801_327};	/*Property = Track*/
const Value V19801_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19801_329 = {S46, 0, &V19801_329_57323, &P19801_328};	/*Property = Video*/
const Value V19801_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19801_330 = {S47, 0, &V19801_330_57326, &P19801_329};	/*Property = Viewport*/
const Value V19801_332_57339 = {S0, NULL};	/*Value = False*/
const Property P19801_332 = {S48, 0, &V19801_332_57339, &P19801_330};	/*Property = WebWorkers*/
const Value V19801_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19801_333 = {S49, 0, &V19801_333_57342, &P19801_332};	/*Property = Xhr2*/
const Profile P19801 = {"19801", &P19801_333};	/*Profile ID - 19801*/
const Value V20412_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20412_279 = {S1, 0, &V20412_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20412_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20412_282 = {S3, 0, &V20412_282_57154, &P20412_279};	/*Property = BlobBuilder*/
const Value V20412_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20412_350 = {S4, 0, &V20412_350_57444, &P20412_282};	/*Property = Canvas*/
const Value V20412_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20412_283 = {S5, 0, &V20412_283_57159, &P20412_350};	/*Property = CssBackground*/
const Value V20412_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20412_284 = {S6, 0, &V20412_284_57163, &P20412_283};	/*Property = CssBorderImage*/
const Value V20412_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20412_285 = {S7, 0, &V20412_285_57166, &P20412_284};	/*Property = CssCanvas*/
const Value V20412_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20412_286 = {S8, 0, &V20412_286_57168, &P20412_285};	/*Property = CssColor*/
const Value V20412_237_55784 = {S2, NULL};	/*Value = True*/
const Property P20412_237 = {S9, 0, &V20412_237_55784, &P20412_286};	/*Property = CssColumn*/
const Value V20412_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20412_287 = {S10, 0, &V20412_287_57172, &P20412_237};	/*Property = CssFlexbox*/
const Value V20412_288_57175 = {S0, NULL};	/*Value = False*/
const Property P20412_288 = {S11, 0, &V20412_288_57175, &P20412_287};	/*Property = CssFont*/
const Value V20412_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20412_289 = {S12, 0, &V20412_289_57759, &P20412_288};	/*Property = CssImages*/
const Value V20412_291_57181 = {S0, NULL};	/*Value = False*/
const Property P20412_291 = {S13, 0, &V20412_291_57181, &P20412_289};	/*Property = CssMediaQueries*/
const Value V20412_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20412_292 = {S14, 0, &V20412_292_57183, &P20412_291};	/*Property = CssMinMax*/
const Value V20412_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20412_293 = {S15, 0, &V20412_293_57187, &P20412_292};	/*Property = CssOverflow*/
const Value V20412_294_57190 = {S0, NULL};	/*Value = False*/
const Property P20412_294 = {S16, 0, &V20412_294_57190, &P20412_293};	/*Property = CssPosition*/
const Value V20412_295_57193 = {S0, NULL};	/*Value = False*/
const Property P20412_295 = {S17, 0, &V20412_295_57193, &P20412_294};	/*Property = CssText*/
const Value V20412_297_57196 = {S0, NULL};	/*Value = False*/
const Property P20412_297 = {S18, 0, &V20412_297_57196, &P20412_295};	/*Property = CssTransforms*/
const Value V20412_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20412_241 = {S19, 0, &V20412_241_55799, &P20412_297};	/*Property = CssTransitions*/
const Value V20412_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20412_298 = {S20, 0, &V20412_298_57206, &P20412_241};	/*Property = CssUI*/
const Value V20412_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20412_300 = {S21, 0, &V20412_300_57216, &P20412_298};	/*Property = DataSet*/
const Value V20412_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20412_301 = {S22, 0, &V20412_301_57218, &P20412_300};	/*Property = DataUrl*/
const Value V20412_302_57222 = {S0, NULL};	/*Value = False*/
const Property P20412_302 = {S23, 0, &V20412_302_57222, &P20412_301};	/*Property = DeviceOrientation*/
const Value V20412_304_57228 = {S0, NULL};	/*Value = False*/
const Property P20412_304 = {S24, 0, &V20412_304_57228, &P20412_302};	/*Property = FileReader*/
const Value V20412_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20412_305 = {S25, 0, &V20412_305_57231, &P20412_304};	/*Property = FileSaver*/
const Value V20412_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20412_306 = {S26, 0, &V20412_306_57237, &P20412_305};	/*Property = FileWriter*/
const Value V20412_307_57240 = {S0, NULL};	/*Value = False*/
const Property P20412_307 = {S27, 0, &V20412_307_57240, &P20412_306};	/*Property = FormData*/
const Value V20412_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20412_309 = {S28, 0, &V20412_309_57246, &P20412_307};	/*Property = Fullscreen*/
const Value V20412_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20412_310 = {S29, 0, &V20412_310_57248, &P20412_309};	/*Property = GeoLocation*/
const Value V20412_312_57255 = {S0, NULL};	/*Value = False*/
const Property P20412_312 = {S30, 0, &V20412_312_57255, &P20412_310};	/*Property = History*/
const Value V20412_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20412_314 = {S31, 0, &V20412_314_57261, &P20412_312};	/*Property = Html5*/
const Value V20412_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20412_313 = {S32, 0, &V20412_313_57258, &P20412_314};	/*Property = Html-Media-Capture*/
const Value V20412_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20412_315 = {S33, 0, &V20412_315_57264, &P20412_313};	/*Property = Iframe*/
const Value V20412_316_57267 = {S0, NULL};	/*Value = False*/
const Property P20412_316 = {S34, 0, &V20412_316_57267, &P20412_315};	/*Property = IndexedDB*/
const Value V20412_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20412_317 = {S35, 0, &V20412_317_57269, &P20412_316};	/*Property = Json*/
const Value V20412_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P20412_15 = {S37, 0, &V20412_15_31, &P20412_317};	/*Property = LayoutEngine*/
const Value V20412_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20412_344 = {S38, 0, &V20412_344_57272, &P20412_15};	/*Property = Masking*/
const Value V20412_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20412_320 = {S39, 0, &V20412_320_57290, &P20412_344};	/*Property = PostMessage*/
const Value V20412_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20412_321 = {S40, 0, &V20412_321_57293, &P20412_320};	/*Property = Progress*/
const Value V20412_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20412_322 = {S41, 0, &V20412_322_57296, &P20412_321};	/*Property = Prompts*/
const Value V20412_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20412_323 = {S42, 0, &V20412_323_57299, &P20412_322};	/*Property = Selector*/
const Value V20412_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20412_324 = {S43, 0, &V20412_324_57306, &P20412_323};	/*Property = Svg*/
const Value V20412_327_57315 = {S0, NULL};	/*Value = False*/
const Property P20412_327 = {S44, 0, &V20412_327_57315, &P20412_324};	/*Property = TouchEvents*/
const Value V20412_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20412_328 = {S45, 0, &V20412_328_57318, &P20412_327};	/*Property = Track*/
const Value V20412_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20412_329 = {S46, 0, &V20412_329_57323, &P20412_328};	/*Property = Video*/
const Value V20412_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20412_330 = {S47, 0, &V20412_330_57326, &P20412_329};	/*Property = Viewport*/
const Value V20412_332_57339 = {S0, NULL};	/*Value = False*/
const Property P20412_332 = {S48, 0, &V20412_332_57339, &P20412_330};	/*Property = WebWorkers*/
const Value V20412_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20412_333 = {S49, 0, &V20412_333_57342, &P20412_332};	/*Property = Xhr2*/
const Profile P20412 = {"20412", &P20412_333};	/*Profile ID - 20412*/
const Value V19985_1_1 = {S2, NULL};	/*Value = True*/
const Property P19985_1 = {S50, 0, &V19985_1_1, NULL};	/*Property = IsMobile*/
const Value V19985_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19985_5 = {S52, 0, &V19985_5_9981, &P19985_1};	/*Property = ScreenPixelsHeight*/
const Value V19985_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P19985_4 = {S54, 0, &V19985_4_39098, &P19985_5};	/*Property = ScreenPixelsWidth*/
const Profile P19985 = {"19985", &P19985_4};	/*Profile ID - 19985*/
const Profile P19824 = {"19824", NULL};	/*Profile ID - 19824*/
const Value V20259_1_1 = {S2, NULL};	/*Value = True*/
const Property P20259_1 = {S50, 0, &V20259_1_1, NULL};	/*Property = IsMobile*/
const Value V20259_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20259_5 = {S52, 0, &V20259_5_1227, &P20259_1};	/*Property = ScreenPixelsHeight*/
const Value V20259_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20259_4 = {S54, 0, &V20259_4_1229, &P20259_5};	/*Property = ScreenPixelsWidth*/
const Profile P20259 = {"20259", &P20259_4};	/*Profile ID - 20259*/
const Value V20456_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20456_279 = {S1, 0, &V20456_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20456_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20456_282 = {S3, 0, &V20456_282_57154, &P20456_279};	/*Property = BlobBuilder*/
const Value V20456_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20456_350 = {S4, 0, &V20456_350_57444, &P20456_282};	/*Property = Canvas*/
const Value V20456_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20456_283 = {S5, 0, &V20456_283_57159, &P20456_350};	/*Property = CssBackground*/
const Value V20456_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20456_284 = {S6, 0, &V20456_284_57163, &P20456_283};	/*Property = CssBorderImage*/
const Value V20456_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20456_285 = {S7, 0, &V20456_285_57166, &P20456_284};	/*Property = CssCanvas*/
const Value V20456_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20456_286 = {S8, 0, &V20456_286_57168, &P20456_285};	/*Property = CssColor*/
const Value V20456_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20456_237 = {S9, 0, &V20456_237_55786, &P20456_286};	/*Property = CssColumn*/
const Value V20456_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20456_287 = {S10, 0, &V20456_287_57172, &P20456_237};	/*Property = CssFlexbox*/
const Value V20456_288_57174 = {S2, NULL};	/*Value = True*/
const Property P20456_288 = {S11, 0, &V20456_288_57174, &P20456_287};	/*Property = CssFont*/
const Value V20456_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20456_289 = {S12, 0, &V20456_289_57759, &P20456_288};	/*Property = CssImages*/
const Value V20456_291_57180 = {S2, NULL};	/*Value = True*/
const Property P20456_291 = {S13, 0, &V20456_291_57180, &P20456_289};	/*Property = CssMediaQueries*/
const Value V20456_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20456_292 = {S14, 0, &V20456_292_57183, &P20456_291};	/*Property = CssMinMax*/
const Value V20456_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20456_293 = {S15, 0, &V20456_293_57187, &P20456_292};	/*Property = CssOverflow*/
const Value V20456_294_57189 = {S2, NULL};	/*Value = True*/
const Property P20456_294 = {S16, 0, &V20456_294_57189, &P20456_293};	/*Property = CssPosition*/
const Value V20456_295_57192 = {S2, NULL};	/*Value = True*/
const Property P20456_295 = {S17, 0, &V20456_295_57192, &P20456_294};	/*Property = CssText*/
const Value V20456_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20456_297 = {S18, 0, &V20456_297_57195, &P20456_295};	/*Property = CssTransforms*/
const Value V20456_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20456_241 = {S19, 0, &V20456_241_55799, &P20456_297};	/*Property = CssTransitions*/
const Value V20456_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20456_298 = {S20, 0, &V20456_298_57206, &P20456_241};	/*Property = CssUI*/
const Value V20456_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20456_300 = {S21, 0, &V20456_300_57216, &P20456_298};	/*Property = DataSet*/
const Value V20456_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20456_301 = {S22, 0, &V20456_301_57218, &P20456_300};	/*Property = DataUrl*/
const Value V20456_302_57221 = {S2, NULL};	/*Value = True*/
const Property P20456_302 = {S23, 0, &V20456_302_57221, &P20456_301};	/*Property = DeviceOrientation*/
const Value V20456_304_57227 = {S2, NULL};	/*Value = True*/
const Property P20456_304 = {S24, 0, &V20456_304_57227, &P20456_302};	/*Property = FileReader*/
const Value V20456_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20456_305 = {S25, 0, &V20456_305_57231, &P20456_304};	/*Property = FileSaver*/
const Value V20456_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20456_306 = {S26, 0, &V20456_306_57237, &P20456_305};	/*Property = FileWriter*/
const Value V20456_307_57239 = {S2, NULL};	/*Value = True*/
const Property P20456_307 = {S27, 0, &V20456_307_57239, &P20456_306};	/*Property = FormData*/
const Value V20456_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20456_309 = {S28, 0, &V20456_309_57246, &P20456_307};	/*Property = Fullscreen*/
const Value V20456_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20456_310 = {S29, 0, &V20456_310_57248, &P20456_309};	/*Property = GeoLocation*/
const Value V20456_312_57254 = {S2, NULL};	/*Value = True*/
const Property P20456_312 = {S30, 0, &V20456_312_57254, &P20456_310};	/*Property = History*/
const Value V20456_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20456_314 = {S31, 0, &V20456_314_57261, &P20456_312};	/*Property = Html5*/
const Value V20456_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20456_313 = {S32, 0, &V20456_313_57258, &P20456_314};	/*Property = Html-Media-Capture*/
const Value V20456_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20456_315 = {S33, 0, &V20456_315_57264, &P20456_313};	/*Property = Iframe*/
const Value V20456_316_57266 = {S2, NULL};	/*Value = True*/
const Property P20456_316 = {S34, 0, &V20456_316_57266, &P20456_315};	/*Property = IndexedDB*/
const Value V20456_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20456_317 = {S35, 0, &V20456_317_57269, &P20456_316};	/*Property = Json*/
const Value V20456_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P20456_15 = {S37, 0, &V20456_15_29, &P20456_317};	/*Property = LayoutEngine*/
const Value V20456_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20456_344 = {S38, 0, &V20456_344_57272, &P20456_15};	/*Property = Masking*/
const Value V20456_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20456_320 = {S39, 0, &V20456_320_57290, &P20456_344};	/*Property = PostMessage*/
const Value V20456_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20456_321 = {S40, 0, &V20456_321_57293, &P20456_320};	/*Property = Progress*/
const Value V20456_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20456_322 = {S41, 0, &V20456_322_57296, &P20456_321};	/*Property = Prompts*/
const Value V20456_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20456_323 = {S42, 0, &V20456_323_57299, &P20456_322};	/*Property = Selector*/
const Value V20456_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20456_324 = {S43, 0, &V20456_324_57306, &P20456_323};	/*Property = Svg*/
const Value V20456_327_57315 = {S0, NULL};	/*Value = False*/
const Property P20456_327 = {S44, 0, &V20456_327_57315, &P20456_324};	/*Property = TouchEvents*/
const Value V20456_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20456_328 = {S45, 0, &V20456_328_57318, &P20456_327};	/*Property = Track*/
const Value V20456_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20456_329 = {S46, 0, &V20456_329_57323, &P20456_328};	/*Property = Video*/
const Value V20456_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20456_330 = {S47, 0, &V20456_330_57326, &P20456_329};	/*Property = Viewport*/
const Value V20456_332_57338 = {S2, NULL};	/*Value = True*/
const Property P20456_332 = {S48, 0, &V20456_332_57338, &P20456_330};	/*Property = WebWorkers*/
const Value V20456_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20456_333 = {S49, 0, &V20456_333_57342, &P20456_332};	/*Property = Xhr2*/
const Profile P20456 = {"20456", &P20456_333};	/*Profile ID - 20456*/
const Value V22555_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22555_279 = {S1, 0, &V22555_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22555_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22555_282 = {S3, 0, &V22555_282_57154, &P22555_279};	/*Property = BlobBuilder*/
const Value V22555_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22555_350 = {S4, 0, &V22555_350_57444, &P22555_282};	/*Property = Canvas*/
const Value V22555_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22555_283 = {S5, 0, &V22555_283_57159, &P22555_350};	/*Property = CssBackground*/
const Value V22555_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22555_284 = {S6, 0, &V22555_284_57163, &P22555_283};	/*Property = CssBorderImage*/
const Value V22555_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22555_285 = {S7, 0, &V22555_285_57166, &P22555_284};	/*Property = CssCanvas*/
const Value V22555_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22555_286 = {S8, 0, &V22555_286_57168, &P22555_285};	/*Property = CssColor*/
const Value V22555_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22555_237 = {S9, 0, &V22555_237_55786, &P22555_286};	/*Property = CssColumn*/
const Value V22555_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22555_287 = {S10, 0, &V22555_287_57172, &P22555_237};	/*Property = CssFlexbox*/
const Value V22555_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22555_288 = {S11, 0, &V22555_288_57174, &P22555_287};	/*Property = CssFont*/
const Value V22555_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22555_289 = {S12, 0, &V22555_289_57759, &P22555_288};	/*Property = CssImages*/
const Value V22555_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22555_291 = {S13, 0, &V22555_291_57180, &P22555_289};	/*Property = CssMediaQueries*/
const Value V22555_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22555_292 = {S14, 0, &V22555_292_57183, &P22555_291};	/*Property = CssMinMax*/
const Value V22555_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22555_293 = {S15, 0, &V22555_293_57187, &P22555_292};	/*Property = CssOverflow*/
const Value V22555_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22555_294 = {S16, 0, &V22555_294_57189, &P22555_293};	/*Property = CssPosition*/
const Value V22555_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22555_295 = {S17, 0, &V22555_295_57192, &P22555_294};	/*Property = CssText*/
const Value V22555_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22555_297 = {S18, 0, &V22555_297_57195, &P22555_295};	/*Property = CssTransforms*/
const Value V22555_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22555_241 = {S19, 0, &V22555_241_55799, &P22555_297};	/*Property = CssTransitions*/
const Value V22555_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22555_298 = {S20, 0, &V22555_298_57206, &P22555_241};	/*Property = CssUI*/
const Value V22555_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22555_300 = {S21, 0, &V22555_300_57216, &P22555_298};	/*Property = DataSet*/
const Value V22555_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22555_301 = {S22, 0, &V22555_301_57218, &P22555_300};	/*Property = DataUrl*/
const Value V22555_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22555_302 = {S23, 0, &V22555_302_57221, &P22555_301};	/*Property = DeviceOrientation*/
const Value V22555_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22555_304 = {S24, 0, &V22555_304_57227, &P22555_302};	/*Property = FileReader*/
const Value V22555_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22555_305 = {S25, 0, &V22555_305_57231, &P22555_304};	/*Property = FileSaver*/
const Value V22555_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22555_306 = {S26, 0, &V22555_306_57237, &P22555_305};	/*Property = FileWriter*/
const Value V22555_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22555_307 = {S27, 0, &V22555_307_57239, &P22555_306};	/*Property = FormData*/
const Value V22555_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22555_309 = {S28, 0, &V22555_309_57246, &P22555_307};	/*Property = Fullscreen*/
const Value V22555_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22555_310 = {S29, 0, &V22555_310_57248, &P22555_309};	/*Property = GeoLocation*/
const Value V22555_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22555_312 = {S30, 0, &V22555_312_57254, &P22555_310};	/*Property = History*/
const Value V22555_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22555_314 = {S31, 0, &V22555_314_57261, &P22555_312};	/*Property = Html5*/
const Value V22555_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22555_313 = {S32, 0, &V22555_313_57258, &P22555_314};	/*Property = Html-Media-Capture*/
const Value V22555_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22555_315 = {S33, 0, &V22555_315_57264, &P22555_313};	/*Property = Iframe*/
const Value V22555_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22555_316 = {S34, 0, &V22555_316_57266, &P22555_315};	/*Property = IndexedDB*/
const Value V22555_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22555_317 = {S35, 0, &V22555_317_57269, &P22555_316};	/*Property = Json*/
const Value V22555_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22555_15 = {S37, 0, &V22555_15_29, &P22555_317};	/*Property = LayoutEngine*/
const Value V22555_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22555_344 = {S38, 0, &V22555_344_57272, &P22555_15};	/*Property = Masking*/
const Value V22555_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22555_320 = {S39, 0, &V22555_320_57290, &P22555_344};	/*Property = PostMessage*/
const Value V22555_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22555_321 = {S40, 0, &V22555_321_57293, &P22555_320};	/*Property = Progress*/
const Value V22555_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22555_322 = {S41, 0, &V22555_322_57296, &P22555_321};	/*Property = Prompts*/
const Value V22555_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22555_323 = {S42, 0, &V22555_323_57299, &P22555_322};	/*Property = Selector*/
const Value V22555_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22555_324 = {S43, 0, &V22555_324_57306, &P22555_323};	/*Property = Svg*/
const Value V22555_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22555_327 = {S44, 0, &V22555_327_57315, &P22555_324};	/*Property = TouchEvents*/
const Value V22555_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22555_328 = {S45, 0, &V22555_328_57318, &P22555_327};	/*Property = Track*/
const Value V22555_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22555_329 = {S46, 0, &V22555_329_57323, &P22555_328};	/*Property = Video*/
const Value V22555_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22555_330 = {S47, 0, &V22555_330_57326, &P22555_329};	/*Property = Viewport*/
const Value V22555_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22555_332 = {S48, 0, &V22555_332_57338, &P22555_330};	/*Property = WebWorkers*/
const Value V22555_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22555_333 = {S49, 0, &V22555_333_57341, &P22555_332};	/*Property = Xhr2*/
const Profile P22555 = {"22555", &P22555_333};	/*Profile ID - 22555*/
const Value V20416_1_1 = {S2, NULL};	/*Value = True*/
const Property P20416_1 = {S50, 0, &V20416_1_1, NULL};	/*Property = IsMobile*/
const Value V20416_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20416_5 = {S52, 0, &V20416_5_9981, &P20416_1};	/*Property = ScreenPixelsHeight*/
const Value V20416_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20416_4 = {S54, 0, &V20416_4_39098, &P20416_5};	/*Property = ScreenPixelsWidth*/
const Profile P20416 = {"20416", &P20416_4};	/*Profile ID - 20416*/
const Value V20091_1_1 = {S2, NULL};	/*Value = True*/
const Property P20091_1 = {S50, 0, &V20091_1_1, NULL};	/*Property = IsMobile*/
const Value V20091_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20091_5 = {S52, 0, &V20091_5_1458, &P20091_1};	/*Property = ScreenPixelsHeight*/
const Value V20091_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20091_4 = {S54, 0, &V20091_4_37617, &P20091_5};	/*Property = ScreenPixelsWidth*/
const Profile P20091 = {"20091", &P20091_4};	/*Profile ID - 20091*/
const Value V19849_1_1 = {S2, NULL};	/*Value = True*/
const Property P19849_1 = {S50, 0, &V19849_1_1, NULL};	/*Property = IsMobile*/
const Value V19849_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19849_5 = {S52, 0, &V19849_5_1227, &P19849_1};	/*Property = ScreenPixelsHeight*/
const Value V19849_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19849_4 = {S54, 0, &V19849_4_1229, &P19849_5};	/*Property = ScreenPixelsWidth*/
const Profile P19849 = {"19849", &P19849_4};	/*Profile ID - 19849*/
const Value V20508_1_1 = {S2, NULL};	/*Value = True*/
const Property P20508_1 = {S50, 0, &V20508_1_1, NULL};	/*Property = IsMobile*/
const Value V20508_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20508_5 = {S52, 0, &V20508_5_1227, &P20508_1};	/*Property = ScreenPixelsHeight*/
const Value V20508_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20508_4 = {S54, 0, &V20508_4_1229, &P20508_5};	/*Property = ScreenPixelsWidth*/
const Profile P20508 = {"20508", &P20508_4};	/*Profile ID - 20508*/
const Value V20831_1_1 = {S2, NULL};	/*Value = True*/
const Property P20831_1 = {S50, 0, &V20831_1_1, NULL};	/*Property = IsMobile*/
const Value V20831_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20831_5 = {S52, 0, &V20831_5_1227, &P20831_1};	/*Property = ScreenPixelsHeight*/
const Value V20831_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20831_4 = {S54, 0, &V20831_4_1229, &P20831_5};	/*Property = ScreenPixelsWidth*/
const Profile P20831 = {"20831", &P20831_4};	/*Profile ID - 20831*/
const Value V19821_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_279 = {S1, 0, &V19821_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V19821_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_282 = {S3, 0, &V19821_282_57155, &P19821_279};	/*Property = BlobBuilder*/
const Value V19821_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_350 = {S4, 0, &V19821_350_57446, &P19821_282};	/*Property = Canvas*/
const Value V19821_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_283 = {S5, 0, &V19821_283_57161, &P19821_350};	/*Property = CssBackground*/
const Value V19821_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_284 = {S6, 0, &V19821_284_57164, &P19821_283};	/*Property = CssBorderImage*/
const Value V19821_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_285 = {S7, 0, &V19821_285_57167, &P19821_284};	/*Property = CssCanvas*/
const Value V19821_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_286 = {S8, 0, &V19821_286_57170, &P19821_285};	/*Property = CssColor*/
const Value V19821_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_237 = {S9, 0, &V19821_237_55786, &P19821_286};	/*Property = CssColumn*/
const Value V19821_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_287 = {S10, 0, &V19821_287_57173, &P19821_237};	/*Property = CssFlexbox*/
const Value V19821_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_288 = {S11, 0, &V19821_288_57176, &P19821_287};	/*Property = CssFont*/
const Value V19821_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_289 = {S12, 0, &V19821_289_86899, &P19821_288};	/*Property = CssImages*/
const Value V19821_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_291 = {S13, 0, &V19821_291_57182, &P19821_289};	/*Property = CssMediaQueries*/
const Value V19821_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_292 = {S14, 0, &V19821_292_57185, &P19821_291};	/*Property = CssMinMax*/
const Value V19821_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_293 = {S15, 0, &V19821_293_57188, &P19821_292};	/*Property = CssOverflow*/
const Value V19821_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_294 = {S16, 0, &V19821_294_57191, &P19821_293};	/*Property = CssPosition*/
const Value V19821_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_295 = {S17, 0, &V19821_295_57194, &P19821_294};	/*Property = CssText*/
const Value V19821_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_297 = {S18, 0, &V19821_297_57197, &P19821_295};	/*Property = CssTransforms*/
const Value V19821_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_241 = {S19, 0, &V19821_241_55801, &P19821_297};	/*Property = CssTransitions*/
const Value V19821_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_298 = {S20, 0, &V19821_298_57208, &P19821_241};	/*Property = CssUI*/
const Value V19821_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_300 = {S21, 0, &V19821_300_57217, &P19821_298};	/*Property = DataSet*/
const Value V19821_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_301 = {S22, 0, &V19821_301_57220, &P19821_300};	/*Property = DataUrl*/
const Value V19821_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_302 = {S23, 0, &V19821_302_57223, &P19821_301};	/*Property = DeviceOrientation*/
const Value V19821_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_304 = {S24, 0, &V19821_304_57229, &P19821_302};	/*Property = FileReader*/
const Value V19821_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_305 = {S25, 0, &V19821_305_57232, &P19821_304};	/*Property = FileSaver*/
const Value V19821_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_306 = {S26, 0, &V19821_306_57238, &P19821_305};	/*Property = FileWriter*/
const Value V19821_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_307 = {S27, 0, &V19821_307_57241, &P19821_306};	/*Property = FormData*/
const Value V19821_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_309 = {S28, 0, &V19821_309_57247, &P19821_307};	/*Property = Fullscreen*/
const Value V19821_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_310 = {S29, 0, &V19821_310_57250, &P19821_309};	/*Property = GeoLocation*/
const Value V19821_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_312 = {S30, 0, &V19821_312_57256, &P19821_310};	/*Property = History*/
const Value V19821_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_314 = {S31, 0, &V19821_314_57262, &P19821_312};	/*Property = Html5*/
const Value V19821_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_313 = {S32, 0, &V19821_313_57259, &P19821_314};	/*Property = Html-Media-Capture*/
const Value V19821_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_315 = {S33, 0, &V19821_315_57265, &P19821_313};	/*Property = Iframe*/
const Value V19821_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_316 = {S34, 0, &V19821_316_57268, &P19821_315};	/*Property = IndexedDB*/
const Value V19821_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_317 = {S35, 0, &V19821_317_57271, &P19821_316};	/*Property = Json*/
const Value V19821_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P19821_15 = {S37, 0, &V19821_15_29, &P19821_317};	/*Property = LayoutEngine*/
const Value V19821_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_344 = {S38, 0, &V19821_344_57274, &P19821_15};	/*Property = Masking*/
const Value V19821_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_320 = {S39, 0, &V19821_320_57292, &P19821_344};	/*Property = PostMessage*/
const Value V19821_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_321 = {S40, 0, &V19821_321_57295, &P19821_320};	/*Property = Progress*/
const Value V19821_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_322 = {S41, 0, &V19821_322_57298, &P19821_321};	/*Property = Prompts*/
const Value V19821_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_323 = {S42, 0, &V19821_323_57301, &P19821_322};	/*Property = Selector*/
const Value V19821_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_324 = {S43, 0, &V19821_324_57307, &P19821_323};	/*Property = Svg*/
const Value V19821_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_327 = {S44, 0, &V19821_327_57316, &P19821_324};	/*Property = TouchEvents*/
const Value V19821_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_328 = {S45, 0, &V19821_328_57319, &P19821_327};	/*Property = Track*/
const Value V19821_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_329 = {S46, 0, &V19821_329_57325, &P19821_328};	/*Property = Video*/
const Value V19821_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_330 = {S47, 0, &V19821_330_57328, &P19821_329};	/*Property = Viewport*/
const Value V19821_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_332 = {S48, 0, &V19821_332_57340, &P19821_330};	/*Property = WebWorkers*/
const Value V19821_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P19821_333 = {S49, 0, &V19821_333_57343, &P19821_332};	/*Property = Xhr2*/
const Profile P19821 = {"19821", &P19821_333};	/*Profile ID - 19821*/
const Value V23631_1_2 = {S0, NULL};	/*Value = False*/
const Property P23631_1 = {S50, 0, &V23631_1_2, NULL};	/*Property = IsMobile*/
const Value V23631_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23631_5 = {S52, 0, &V23631_5_60417, &P23631_1};	/*Property = ScreenPixelsHeight*/
const Value V23631_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23631_4 = {S54, 0, &V23631_4_57095, &P23631_5};	/*Property = ScreenPixelsWidth*/
const Profile P23631 = {"23631", &P23631_4};	/*Profile ID - 23631*/
const Value V15779_1_1 = {S2, NULL};	/*Value = True*/
const Property P15779_1 = {S50, 0, &V15779_1_1, NULL};	/*Property = IsMobile*/
const Value V15779_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15779_5 = {S52, 0, &V15779_5_1222, &P15779_1};	/*Property = ScreenPixelsHeight*/
const Value V15779_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15779_4 = {S54, 0, &V15779_4_1224, &P15779_5};	/*Property = ScreenPixelsWidth*/
const Profile P15779 = {"15779", &P15779_4};	/*Profile ID - 15779*/
const Value V20213_1_1 = {S2, NULL};	/*Value = True*/
const Property P20213_1 = {S50, 0, &V20213_1_1, NULL};	/*Property = IsMobile*/
const Value V20213_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20213_5 = {S52, 0, &V20213_5_9981, &P20213_1};	/*Property = ScreenPixelsHeight*/
const Value V20213_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20213_4 = {S54, 0, &V20213_4_39098, &P20213_5};	/*Property = ScreenPixelsWidth*/
const Profile P20213 = {"20213", &P20213_4};	/*Profile ID - 20213*/
const Value V21432_1_1 = {S2, NULL};	/*Value = True*/
const Property P21432_1 = {S50, 0, &V21432_1_1, NULL};	/*Property = IsMobile*/
const Value V21432_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21432_5 = {S52, 0, &V21432_5_1277, &P21432_1};	/*Property = ScreenPixelsHeight*/
const Value V21432_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21432_4 = {S54, 0, &V21432_4_1279, &P21432_5};	/*Property = ScreenPixelsWidth*/
const Profile P21432 = {"21432", &P21432_4};	/*Profile ID - 21432*/
const Value V22159_1_1 = {S2, NULL};	/*Value = True*/
const Property P22159_1 = {S50, 0, &V22159_1_1, NULL};	/*Property = IsMobile*/
const Value V22159_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22159_5 = {S52, 0, &V22159_5_1227, &P22159_1};	/*Property = ScreenPixelsHeight*/
const Value V22159_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22159_4 = {S54, 0, &V22159_4_1229, &P22159_5};	/*Property = ScreenPixelsWidth*/
const Profile P22159 = {"22159", &P22159_4};	/*Profile ID - 22159*/
const Value V20083_1_2 = {S0, NULL};	/*Value = False*/
const Property P20083_1 = {S50, 0, &V20083_1_2, NULL};	/*Property = IsMobile*/
const Value V20083_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20083_5 = {S52, 0, &V20083_5_37, &P20083_1};	/*Property = ScreenPixelsHeight*/
const Value V20083_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20083_4 = {S54, 0, &V20083_4_36, &P20083_5};	/*Property = ScreenPixelsWidth*/
const Profile P20083 = {"20083", &P20083_4};	/*Profile ID - 20083*/
const Value V21867_1_1 = {S2, NULL};	/*Value = True*/
const Property P21867_1 = {S50, 0, &V21867_1_1, NULL};	/*Property = IsMobile*/
const Value V21867_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21867_5 = {S52, 0, &V21867_5_1227, &P21867_1};	/*Property = ScreenPixelsHeight*/
const Value V21867_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21867_4 = {S54, 0, &V21867_4_1229, &P21867_5};	/*Property = ScreenPixelsWidth*/
const Profile P21867 = {"21867", &P21867_4};	/*Profile ID - 21867*/
const Value V20324_1_2 = {S0, NULL};	/*Value = False*/
const Property P20324_1 = {S50, 0, &V20324_1_2, NULL};	/*Property = IsMobile*/
const Value V20324_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20324_5 = {S52, 0, &V20324_5_37, &P20324_1};	/*Property = ScreenPixelsHeight*/
const Value V20324_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20324_4 = {S54, 0, &V20324_4_36, &P20324_5};	/*Property = ScreenPixelsWidth*/
const Profile P20324 = {"20324", &P20324_4};	/*Profile ID - 20324*/
const Value V20366_1_1 = {S2, NULL};	/*Value = True*/
const Property P20366_1 = {S50, 0, &V20366_1_1, NULL};	/*Property = IsMobile*/
const Value V20366_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20366_5 = {S52, 0, &V20366_5_1403, &P20366_1};	/*Property = ScreenPixelsHeight*/
const Value V20366_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20366_4 = {S54, 0, &V20366_4_1404, &P20366_5};	/*Property = ScreenPixelsWidth*/
const Profile P20366 = {"20366", &P20366_4};	/*Profile ID - 20366*/
const Value V23550_1_1 = {S2, NULL};	/*Value = True*/
const Property P23550_1 = {S50, 0, &V23550_1_1, NULL};	/*Property = IsMobile*/
const Value V23550_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23550_5 = {S52, 0, &V23550_5_1403, &P23550_1};	/*Property = ScreenPixelsHeight*/
const Value V23550_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23550_4 = {S54, 0, &V23550_4_1404, &P23550_5};	/*Property = ScreenPixelsWidth*/
const Profile P23550 = {"23550", &P23550_4};	/*Profile ID - 23550*/
const Value V22677_1_1 = {S2, NULL};	/*Value = True*/
const Property P22677_1 = {S50, 0, &V22677_1_1, NULL};	/*Property = IsMobile*/
const Value V22677_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22677_5 = {S52, 0, &V22677_5_9981, &P22677_1};	/*Property = ScreenPixelsHeight*/
const Value V22677_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22677_4 = {S54, 0, &V22677_4_39098, &P22677_5};	/*Property = ScreenPixelsWidth*/
const Profile P22677 = {"22677", &P22677_4};	/*Profile ID - 22677*/
const Value V20119_1_1 = {S2, NULL};	/*Value = True*/
const Property P20119_1 = {S50, 0, &V20119_1_1, NULL};	/*Property = IsMobile*/
const Value V20119_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20119_5 = {S52, 0, &V20119_5_9981, &P20119_1};	/*Property = ScreenPixelsHeight*/
const Value V20119_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20119_4 = {S54, 0, &V20119_4_2336, &P20119_5};	/*Property = ScreenPixelsWidth*/
const Profile P20119 = {"20119", &P20119_4};	/*Profile ID - 20119*/
const Value V21870_1_1 = {S2, NULL};	/*Value = True*/
const Property P21870_1 = {S50, 0, &V21870_1_1, NULL};	/*Property = IsMobile*/
const Value V21870_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21870_5 = {S52, 0, &V21870_5_1227, &P21870_1};	/*Property = ScreenPixelsHeight*/
const Value V21870_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21870_4 = {S54, 0, &V21870_4_1229, &P21870_5};	/*Property = ScreenPixelsWidth*/
const Profile P21870 = {"21870", &P21870_4};	/*Profile ID - 21870*/
const Value V20294_1_2 = {S0, NULL};	/*Value = False*/
const Property P20294_1 = {S50, 0, &V20294_1_2, NULL};	/*Property = IsMobile*/
const Value V20294_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20294_5 = {S52, 0, &V20294_5_37, &P20294_1};	/*Property = ScreenPixelsHeight*/
const Value V20294_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20294_4 = {S54, 0, &V20294_4_36, &P20294_5};	/*Property = ScreenPixelsWidth*/
const Profile P20294 = {"20294", &P20294_4};	/*Profile ID - 20294*/
const Value V21501_1_1 = {S2, NULL};	/*Value = True*/
const Property P21501_1 = {S50, 0, &V21501_1_1, NULL};	/*Property = IsMobile*/
const Value V21501_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21501_5 = {S52, 0, &V21501_5_1277, &P21501_1};	/*Property = ScreenPixelsHeight*/
const Value V21501_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21501_4 = {S54, 0, &V21501_4_1279, &P21501_5};	/*Property = ScreenPixelsWidth*/
const Profile P21501 = {"21501", &P21501_4};	/*Profile ID - 21501*/
const Value V23463_1_1 = {S2, NULL};	/*Value = True*/
const Property P23463_1 = {S50, 0, &V23463_1_1, NULL};	/*Property = IsMobile*/
const Value V23463_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23463_5 = {S52, 0, &V23463_5_1222, &P23463_1};	/*Property = ScreenPixelsHeight*/
const Value V23463_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23463_4 = {S54, 0, &V23463_4_1224, &P23463_5};	/*Property = ScreenPixelsWidth*/
const Profile P23463 = {"23463", &P23463_4};	/*Profile ID - 23463*/
const Value V20123_1_1 = {S2, NULL};	/*Value = True*/
const Property P20123_1 = {S50, 0, &V20123_1_1, NULL};	/*Property = IsMobile*/
const Value V20123_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20123_5 = {S52, 0, &V20123_5_1227, &P20123_1};	/*Property = ScreenPixelsHeight*/
const Value V20123_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20123_4 = {S54, 0, &V20123_4_1229, &P20123_5};	/*Property = ScreenPixelsWidth*/
const Profile P20123 = {"20123", &P20123_4};	/*Profile ID - 20123*/
const Value V23379_1_1 = {S2, NULL};	/*Value = True*/
const Property P23379_1 = {S50, 0, &V23379_1_1, NULL};	/*Property = IsMobile*/
const Value V23379_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23379_5 = {S52, 0, &V23379_5_1458, &P23379_1};	/*Property = ScreenPixelsHeight*/
const Value V23379_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23379_4 = {S54, 0, &V23379_4_37617, &P23379_5};	/*Property = ScreenPixelsWidth*/
const Profile P23379 = {"23379", &P23379_4};	/*Profile ID - 23379*/
const Value V23448_1_1 = {S2, NULL};	/*Value = True*/
const Property P23448_1 = {S50, 0, &V23448_1_1, NULL};	/*Property = IsMobile*/
const Value V23448_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23448_5 = {S52, 0, &V23448_5_1277, &P23448_1};	/*Property = ScreenPixelsHeight*/
const Value V23448_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23448_4 = {S54, 0, &V23448_4_1279, &P23448_5};	/*Property = ScreenPixelsWidth*/
const Profile P23448 = {"23448", &P23448_4};	/*Profile ID - 23448*/
const Value V20413_1_1 = {S2, NULL};	/*Value = True*/
const Property P20413_1 = {S50, 0, &V20413_1_1, NULL};	/*Property = IsMobile*/
const Value V20413_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20413_5 = {S52, 0, &V20413_5_1227, &P20413_1};	/*Property = ScreenPixelsHeight*/
const Value V20413_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20413_4 = {S54, 0, &V20413_4_1229, &P20413_5};	/*Property = ScreenPixelsWidth*/
const Profile P20413 = {"20413", &P20413_4};	/*Profile ID - 20413*/
const Value V20001_1_1 = {S2, NULL};	/*Value = True*/
const Property P20001_1 = {S50, 0, &V20001_1_1, NULL};	/*Property = IsMobile*/
const Value V20001_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20001_5 = {S52, 0, &V20001_5_1403, &P20001_1};	/*Property = ScreenPixelsHeight*/
const Value V20001_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20001_4 = {S54, 0, &V20001_4_1404, &P20001_5};	/*Property = ScreenPixelsWidth*/
const Profile P20001 = {"20001", &P20001_4};	/*Profile ID - 20001*/
const Value V24129_1_1 = {S2, NULL};	/*Value = True*/
const Property P24129_1 = {S50, 0, &V24129_1_1, NULL};	/*Property = IsMobile*/
const Value V24129_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24129_5 = {S52, 0, &V24129_5_9981, &P24129_1};	/*Property = ScreenPixelsHeight*/
const Value V24129_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24129_4 = {S54, 0, &V24129_4_39098, &P24129_5};	/*Property = ScreenPixelsWidth*/
const Profile P24129 = {"24129", &P24129_4};	/*Profile ID - 24129*/
const Value V20093_1_2 = {S0, NULL};	/*Value = False*/
const Property P20093_1 = {S50, 0, &V20093_1_2, NULL};	/*Property = IsMobile*/
const Value V20093_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20093_5 = {S52, 0, &V20093_5_37, &P20093_1};	/*Property = ScreenPixelsHeight*/
const Value V20093_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20093_4 = {S54, 0, &V20093_4_36, &P20093_5};	/*Property = ScreenPixelsWidth*/
const Profile P20093 = {"20093", &P20093_4};	/*Profile ID - 20093*/
const Value V13006_1_1 = {S2, NULL};	/*Value = True*/
const Property P13006_1 = {S50, 0, &V13006_1_1, NULL};	/*Property = IsMobile*/
const Value V13006_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P13006_5 = {S52, 0, &V13006_5_1832, &P13006_1};	/*Property = ScreenPixelsHeight*/
const Value V13006_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P13006_4 = {S54, 0, &V13006_4_5645, &P13006_5};	/*Property = ScreenPixelsWidth*/
const Profile P13006 = {"13006", &P13006_4};	/*Profile ID - 13006*/
const Value V22305_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22305_279 = {S1, 0, &V22305_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22305_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22305_282 = {S3, 0, &V22305_282_57154, &P22305_279};	/*Property = BlobBuilder*/
const Value V22305_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22305_350 = {S4, 0, &V22305_350_57444, &P22305_282};	/*Property = Canvas*/
const Value V22305_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22305_283 = {S5, 0, &V22305_283_57159, &P22305_350};	/*Property = CssBackground*/
const Value V22305_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22305_284 = {S6, 0, &V22305_284_57162, &P22305_283};	/*Property = CssBorderImage*/
const Value V22305_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22305_285 = {S7, 0, &V22305_285_57166, &P22305_284};	/*Property = CssCanvas*/
const Value V22305_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22305_286 = {S8, 0, &V22305_286_57168, &P22305_285};	/*Property = CssColor*/
const Value V22305_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22305_237 = {S9, 0, &V22305_237_55786, &P22305_286};	/*Property = CssColumn*/
const Value V22305_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22305_287 = {S10, 0, &V22305_287_57172, &P22305_237};	/*Property = CssFlexbox*/
const Value V22305_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22305_288 = {S11, 0, &V22305_288_57174, &P22305_287};	/*Property = CssFont*/
const Value V22305_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22305_289 = {S12, 0, &V22305_289_57759, &P22305_288};	/*Property = CssImages*/
const Value V22305_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22305_291 = {S13, 0, &V22305_291_57180, &P22305_289};	/*Property = CssMediaQueries*/
const Value V22305_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22305_292 = {S14, 0, &V22305_292_57183, &P22305_291};	/*Property = CssMinMax*/
const Value V22305_293_57186 = {S2, NULL};	/*Value = True*/
const Property P22305_293 = {S15, 0, &V22305_293_57186, &P22305_292};	/*Property = CssOverflow*/
const Value V22305_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22305_294 = {S16, 0, &V22305_294_57189, &P22305_293};	/*Property = CssPosition*/
const Value V22305_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22305_295 = {S17, 0, &V22305_295_57192, &P22305_294};	/*Property = CssText*/
const Value V22305_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22305_297 = {S18, 0, &V22305_297_57195, &P22305_295};	/*Property = CssTransforms*/
const Value V22305_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22305_241 = {S19, 0, &V22305_241_55799, &P22305_297};	/*Property = CssTransitions*/
const Value V22305_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22305_298 = {S20, 0, &V22305_298_57206, &P22305_241};	/*Property = CssUI*/
const Value V22305_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22305_300 = {S21, 0, &V22305_300_57215, &P22305_298};	/*Property = DataSet*/
const Value V22305_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22305_301 = {S22, 0, &V22305_301_57218, &P22305_300};	/*Property = DataUrl*/
const Value V22305_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22305_302 = {S23, 0, &V22305_302_57221, &P22305_301};	/*Property = DeviceOrientation*/
const Value V22305_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22305_304 = {S24, 0, &V22305_304_57227, &P22305_302};	/*Property = FileReader*/
const Value V22305_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22305_305 = {S25, 0, &V22305_305_57231, &P22305_304};	/*Property = FileSaver*/
const Value V22305_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22305_306 = {S26, 0, &V22305_306_57237, &P22305_305};	/*Property = FileWriter*/
const Value V22305_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22305_307 = {S27, 0, &V22305_307_57239, &P22305_306};	/*Property = FormData*/
const Value V22305_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22305_309 = {S28, 0, &V22305_309_57246, &P22305_307};	/*Property = Fullscreen*/
const Value V22305_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22305_310 = {S29, 0, &V22305_310_57248, &P22305_309};	/*Property = GeoLocation*/
const Value V22305_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22305_312 = {S30, 0, &V22305_312_57254, &P22305_310};	/*Property = History*/
const Value V22305_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22305_314 = {S31, 0, &V22305_314_57260, &P22305_312};	/*Property = Html5*/
const Value V22305_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22305_313 = {S32, 0, &V22305_313_57258, &P22305_314};	/*Property = Html-Media-Capture*/
const Value V22305_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22305_315 = {S33, 0, &V22305_315_57264, &P22305_313};	/*Property = Iframe*/
const Value V22305_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22305_316 = {S34, 0, &V22305_316_57267, &P22305_315};	/*Property = IndexedDB*/
const Value V22305_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22305_317 = {S35, 0, &V22305_317_57269, &P22305_316};	/*Property = Json*/
const Value V22305_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22305_15 = {S37, 0, &V22305_15_28, &P22305_317};	/*Property = LayoutEngine*/
const Value V22305_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22305_344 = {S38, 0, &V22305_344_57272, &P22305_15};	/*Property = Masking*/
const Value V22305_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22305_320 = {S39, 0, &V22305_320_57290, &P22305_344};	/*Property = PostMessage*/
const Value V22305_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22305_321 = {S40, 0, &V22305_321_57293, &P22305_320};	/*Property = Progress*/
const Value V22305_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22305_322 = {S41, 0, &V22305_322_57296, &P22305_321};	/*Property = Prompts*/
const Value V22305_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22305_323 = {S42, 0, &V22305_323_57299, &P22305_322};	/*Property = Selector*/
const Value V22305_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22305_324 = {S43, 0, &V22305_324_57305, &P22305_323};	/*Property = Svg*/
const Value V22305_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22305_327 = {S44, 0, &V22305_327_57314, &P22305_324};	/*Property = TouchEvents*/
const Value V22305_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22305_328 = {S45, 0, &V22305_328_57318, &P22305_327};	/*Property = Track*/
const Value V22305_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22305_329 = {S46, 0, &V22305_329_57323, &P22305_328};	/*Property = Video*/
const Value V22305_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22305_330 = {S47, 0, &V22305_330_57326, &P22305_329};	/*Property = Viewport*/
const Value V22305_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22305_332 = {S48, 0, &V22305_332_57338, &P22305_330};	/*Property = WebWorkers*/
const Value V22305_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22305_333 = {S49, 0, &V22305_333_57342, &P22305_332};	/*Property = Xhr2*/
const Profile P22305 = {"22305", &P22305_333};	/*Profile ID - 22305*/
const Value V20480_1_1 = {S2, NULL};	/*Value = True*/
const Property P20480_1 = {S50, 0, &V20480_1_1, NULL};	/*Property = IsMobile*/
const Value V20480_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20480_5 = {S52, 0, &V20480_5_9981, &P20480_1};	/*Property = ScreenPixelsHeight*/
const Value V20480_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20480_4 = {S54, 0, &V20480_4_39098, &P20480_5};	/*Property = ScreenPixelsWidth*/
const Profile P20480 = {"20480", &P20480_4};	/*Profile ID - 20480*/
const Value V20056_1_1 = {S2, NULL};	/*Value = True*/
const Property P20056_1 = {S50, 0, &V20056_1_1, NULL};	/*Property = IsMobile*/
const Value V20056_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20056_5 = {S52, 0, &V20056_5_1222, &P20056_1};	/*Property = ScreenPixelsHeight*/
const Value V20056_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20056_4 = {S54, 0, &V20056_4_1224, &P20056_5};	/*Property = ScreenPixelsWidth*/
const Profile P20056 = {"20056", &P20056_4};	/*Profile ID - 20056*/
const Value V21520_1_1 = {S2, NULL};	/*Value = True*/
const Property P21520_1 = {S50, 0, &V21520_1_1, NULL};	/*Property = IsMobile*/
const Value V21520_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21520_5 = {S52, 0, &V21520_5_1227, &P21520_1};	/*Property = ScreenPixelsHeight*/
const Value V21520_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21520_4 = {S54, 0, &V21520_4_1229, &P21520_5};	/*Property = ScreenPixelsWidth*/
const Profile P21520 = {"21520", &P21520_4};	/*Profile ID - 21520*/
const Value V20886_1_1 = {S2, NULL};	/*Value = True*/
const Property P20886_1 = {S50, 0, &V20886_1_1, NULL};	/*Property = IsMobile*/
const Value V20886_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20886_5 = {S52, 0, &V20886_5_1458, &P20886_1};	/*Property = ScreenPixelsHeight*/
const Value V20886_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20886_4 = {S54, 0, &V20886_4_37617, &P20886_5};	/*Property = ScreenPixelsWidth*/
const Profile P20886 = {"20886", &P20886_4};	/*Profile ID - 20886*/
const Value V19753_1_1 = {S2, NULL};	/*Value = True*/
const Property P19753_1 = {S50, 0, &V19753_1_1, NULL};	/*Property = IsMobile*/
const Value V19753_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P19753_5 = {S52, 0, &V19753_5_1277, &P19753_1};	/*Property = ScreenPixelsHeight*/
const Value V19753_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P19753_4 = {S54, 0, &V19753_4_1279, &P19753_5};	/*Property = ScreenPixelsWidth*/
const Profile P19753 = {"19753", &P19753_4};	/*Profile ID - 19753*/
const Value V19804_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19804_279 = {S1, 0, &V19804_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19804_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19804_282 = {S3, 0, &V19804_282_57154, &P19804_279};	/*Property = BlobBuilder*/
const Value V19804_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19804_350 = {S4, 0, &V19804_350_57444, &P19804_282};	/*Property = Canvas*/
const Value V19804_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19804_283 = {S5, 0, &V19804_283_57159, &P19804_350};	/*Property = CssBackground*/
const Value V19804_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19804_284 = {S6, 0, &V19804_284_57163, &P19804_283};	/*Property = CssBorderImage*/
const Value V19804_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19804_285 = {S7, 0, &V19804_285_57166, &P19804_284};	/*Property = CssCanvas*/
const Value V19804_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19804_286 = {S8, 0, &V19804_286_57168, &P19804_285};	/*Property = CssColor*/
const Value V19804_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19804_237 = {S9, 0, &V19804_237_55786, &P19804_286};	/*Property = CssColumn*/
const Value V19804_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19804_287 = {S10, 0, &V19804_287_57172, &P19804_237};	/*Property = CssFlexbox*/
const Value V19804_288_57174 = {S2, NULL};	/*Value = True*/
const Property P19804_288 = {S11, 0, &V19804_288_57174, &P19804_287};	/*Property = CssFont*/
const Value V19804_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19804_289 = {S12, 0, &V19804_289_57759, &P19804_288};	/*Property = CssImages*/
const Value V19804_291_57180 = {S2, NULL};	/*Value = True*/
const Property P19804_291 = {S13, 0, &V19804_291_57180, &P19804_289};	/*Property = CssMediaQueries*/
const Value V19804_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19804_292 = {S14, 0, &V19804_292_57183, &P19804_291};	/*Property = CssMinMax*/
const Value V19804_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19804_293 = {S15, 0, &V19804_293_57187, &P19804_292};	/*Property = CssOverflow*/
const Value V19804_294_57189 = {S2, NULL};	/*Value = True*/
const Property P19804_294 = {S16, 0, &V19804_294_57189, &P19804_293};	/*Property = CssPosition*/
const Value V19804_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19804_295 = {S17, 0, &V19804_295_57192, &P19804_294};	/*Property = CssText*/
const Value V19804_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19804_297 = {S18, 0, &V19804_297_57195, &P19804_295};	/*Property = CssTransforms*/
const Value V19804_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19804_241 = {S19, 0, &V19804_241_55799, &P19804_297};	/*Property = CssTransitions*/
const Value V19804_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19804_298 = {S20, 0, &V19804_298_57206, &P19804_241};	/*Property = CssUI*/
const Value V19804_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19804_300 = {S21, 0, &V19804_300_57216, &P19804_298};	/*Property = DataSet*/
const Value V19804_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19804_301 = {S22, 0, &V19804_301_57218, &P19804_300};	/*Property = DataUrl*/
const Value V19804_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19804_302 = {S23, 0, &V19804_302_57222, &P19804_301};	/*Property = DeviceOrientation*/
const Value V19804_304_57227 = {S2, NULL};	/*Value = True*/
const Property P19804_304 = {S24, 0, &V19804_304_57227, &P19804_302};	/*Property = FileReader*/
const Value V19804_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19804_305 = {S25, 0, &V19804_305_57231, &P19804_304};	/*Property = FileSaver*/
const Value V19804_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19804_306 = {S26, 0, &V19804_306_57237, &P19804_305};	/*Property = FileWriter*/
const Value V19804_307_57239 = {S2, NULL};	/*Value = True*/
const Property P19804_307 = {S27, 0, &V19804_307_57239, &P19804_306};	/*Property = FormData*/
const Value V19804_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19804_309 = {S28, 0, &V19804_309_57246, &P19804_307};	/*Property = Fullscreen*/
const Value V19804_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19804_310 = {S29, 0, &V19804_310_57248, &P19804_309};	/*Property = GeoLocation*/
const Value V19804_312_57254 = {S2, NULL};	/*Value = True*/
const Property P19804_312 = {S30, 0, &V19804_312_57254, &P19804_310};	/*Property = History*/
const Value V19804_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19804_314 = {S31, 0, &V19804_314_57261, &P19804_312};	/*Property = Html5*/
const Value V19804_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19804_313 = {S32, 0, &V19804_313_57258, &P19804_314};	/*Property = Html-Media-Capture*/
const Value V19804_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19804_315 = {S33, 0, &V19804_315_57264, &P19804_313};	/*Property = Iframe*/
const Value V19804_316_57266 = {S2, NULL};	/*Value = True*/
const Property P19804_316 = {S34, 0, &V19804_316_57266, &P19804_315};	/*Property = IndexedDB*/
const Value V19804_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19804_317 = {S35, 0, &V19804_317_57269, &P19804_316};	/*Property = Json*/
const Value V19804_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19804_15 = {S37, 0, &V19804_15_28, &P19804_317};	/*Property = LayoutEngine*/
const Value V19804_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19804_344 = {S38, 0, &V19804_344_57272, &P19804_15};	/*Property = Masking*/
const Value V19804_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19804_320 = {S39, 0, &V19804_320_57290, &P19804_344};	/*Property = PostMessage*/
const Value V19804_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19804_321 = {S40, 0, &V19804_321_57293, &P19804_320};	/*Property = Progress*/
const Value V19804_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19804_322 = {S41, 0, &V19804_322_57296, &P19804_321};	/*Property = Prompts*/
const Value V19804_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19804_323 = {S42, 0, &V19804_323_57299, &P19804_322};	/*Property = Selector*/
const Value V19804_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19804_324 = {S43, 0, &V19804_324_57306, &P19804_323};	/*Property = Svg*/
const Value V19804_327_57314 = {S2, NULL};	/*Value = True*/
const Property P19804_327 = {S44, 0, &V19804_327_57314, &P19804_324};	/*Property = TouchEvents*/
const Value V19804_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19804_328 = {S45, 0, &V19804_328_57318, &P19804_327};	/*Property = Track*/
const Value V19804_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19804_329 = {S46, 0, &V19804_329_57323, &P19804_328};	/*Property = Video*/
const Value V19804_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19804_330 = {S47, 0, &V19804_330_57326, &P19804_329};	/*Property = Viewport*/
const Value V19804_332_57339 = {S0, NULL};	/*Value = False*/
const Property P19804_332 = {S48, 0, &V19804_332_57339, &P19804_330};	/*Property = WebWorkers*/
const Value V19804_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19804_333 = {S49, 0, &V19804_333_57342, &P19804_332};	/*Property = Xhr2*/
const Profile P19804 = {"19804", &P19804_333};	/*Profile ID - 19804*/
const Value V22695_1_1 = {S2, NULL};	/*Value = True*/
const Property P22695_1 = {S50, 0, &V22695_1_1, NULL};	/*Property = IsMobile*/
const Value V22695_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22695_5 = {S52, 0, &V22695_5_1222, &P22695_1};	/*Property = ScreenPixelsHeight*/
const Value V22695_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22695_4 = {S54, 0, &V22695_4_1224, &P22695_5};	/*Property = ScreenPixelsWidth*/
const Profile P22695 = {"22695", &P22695_4};	/*Profile ID - 22695*/
const Value V23523_1_1 = {S2, NULL};	/*Value = True*/
const Property P23523_1 = {S50, 0, &V23523_1_1, NULL};	/*Property = IsMobile*/
const Value V23523_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23523_5 = {S52, 0, &V23523_5_1227, &P23523_1};	/*Property = ScreenPixelsHeight*/
const Value V23523_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23523_4 = {S54, 0, &V23523_4_1229, &P23523_5};	/*Property = ScreenPixelsWidth*/
const Profile P23523 = {"23523", &P23523_4};	/*Profile ID - 23523*/
const Value V22098_1_1 = {S2, NULL};	/*Value = True*/
const Property P22098_1 = {S50, 0, &V22098_1_1, NULL};	/*Property = IsMobile*/
const Value V22098_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22098_5 = {S52, 0, &V22098_5_1222, &P22098_1};	/*Property = ScreenPixelsHeight*/
const Value V22098_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22098_4 = {S54, 0, &V22098_4_1224, &P22098_5};	/*Property = ScreenPixelsWidth*/
const Profile P22098 = {"22098", &P22098_4};	/*Profile ID - 22098*/
const Value V19917_1_1 = {S2, NULL};	/*Value = True*/
const Property P19917_1 = {S50, 0, &V19917_1_1, NULL};	/*Property = IsMobile*/
const Value V19917_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P19917_5 = {S52, 0, &V19917_5_1403, &P19917_1};	/*Property = ScreenPixelsHeight*/
const Value V19917_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P19917_4 = {S54, 0, &V19917_4_1404, &P19917_5};	/*Property = ScreenPixelsWidth*/
const Profile P19917 = {"19917", &P19917_4};	/*Profile ID - 19917*/
const Value V20015_1_1 = {S2, NULL};	/*Value = True*/
const Property P20015_1 = {S50, 0, &V20015_1_1, NULL};	/*Property = IsMobile*/
const Value V20015_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20015_5 = {S52, 0, &V20015_5_9981, &P20015_1};	/*Property = ScreenPixelsHeight*/
const Value V20015_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20015_4 = {S54, 0, &V20015_4_39098, &P20015_5};	/*Property = ScreenPixelsWidth*/
const Profile P20015 = {"20015", &P20015_4};	/*Profile ID - 20015*/
const Value V22156_1_1 = {S2, NULL};	/*Value = True*/
const Property P22156_1 = {S50, 0, &V22156_1_1, NULL};	/*Property = IsMobile*/
const Value V22156_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22156_5 = {S52, 0, &V22156_5_1458, &P22156_1};	/*Property = ScreenPixelsHeight*/
const Value V22156_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P22156_4 = {S54, 0, &V22156_4_37617, &P22156_5};	/*Property = ScreenPixelsWidth*/
const Profile P22156 = {"22156", &P22156_4};	/*Profile ID - 22156*/
const Value V21941_1_1 = {S2, NULL};	/*Value = True*/
const Property P21941_1 = {S50, 0, &V21941_1_1, NULL};	/*Property = IsMobile*/
const Value V21941_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21941_5 = {S52, 0, &V21941_5_1458, &P21941_1};	/*Property = ScreenPixelsHeight*/
const Value V21941_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21941_4 = {S54, 0, &V21941_4_37617, &P21941_5};	/*Property = ScreenPixelsWidth*/
const Profile P21941 = {"21941", &P21941_4};	/*Profile ID - 21941*/
const Value V19816_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19816_279 = {S1, 0, &V19816_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19816_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19816_282 = {S3, 0, &V19816_282_57154, &P19816_279};	/*Property = BlobBuilder*/
const Value V19816_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19816_350 = {S4, 0, &V19816_350_57444, &P19816_282};	/*Property = Canvas*/
const Value V19816_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19816_283 = {S5, 0, &V19816_283_57159, &P19816_350};	/*Property = CssBackground*/
const Value V19816_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19816_284 = {S6, 0, &V19816_284_57163, &P19816_283};	/*Property = CssBorderImage*/
const Value V19816_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19816_285 = {S7, 0, &V19816_285_57166, &P19816_284};	/*Property = CssCanvas*/
const Value V19816_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19816_286 = {S8, 0, &V19816_286_57168, &P19816_285};	/*Property = CssColor*/
const Value V19816_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19816_237 = {S9, 0, &V19816_237_55786, &P19816_286};	/*Property = CssColumn*/
const Value V19816_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19816_287 = {S10, 0, &V19816_287_57172, &P19816_237};	/*Property = CssFlexbox*/
const Value V19816_288_57174 = {S2, NULL};	/*Value = True*/
const Property P19816_288 = {S11, 0, &V19816_288_57174, &P19816_287};	/*Property = CssFont*/
const Value V19816_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19816_289 = {S12, 0, &V19816_289_57759, &P19816_288};	/*Property = CssImages*/
const Value V19816_291_57180 = {S2, NULL};	/*Value = True*/
const Property P19816_291 = {S13, 0, &V19816_291_57180, &P19816_289};	/*Property = CssMediaQueries*/
const Value V19816_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19816_292 = {S14, 0, &V19816_292_57183, &P19816_291};	/*Property = CssMinMax*/
const Value V19816_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19816_293 = {S15, 0, &V19816_293_57187, &P19816_292};	/*Property = CssOverflow*/
const Value V19816_294_57189 = {S2, NULL};	/*Value = True*/
const Property P19816_294 = {S16, 0, &V19816_294_57189, &P19816_293};	/*Property = CssPosition*/
const Value V19816_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19816_295 = {S17, 0, &V19816_295_57192, &P19816_294};	/*Property = CssText*/
const Value V19816_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19816_297 = {S18, 0, &V19816_297_57195, &P19816_295};	/*Property = CssTransforms*/
const Value V19816_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19816_241 = {S19, 0, &V19816_241_55799, &P19816_297};	/*Property = CssTransitions*/
const Value V19816_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19816_298 = {S20, 0, &V19816_298_57206, &P19816_241};	/*Property = CssUI*/
const Value V19816_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19816_300 = {S21, 0, &V19816_300_57216, &P19816_298};	/*Property = DataSet*/
const Value V19816_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19816_301 = {S22, 0, &V19816_301_57218, &P19816_300};	/*Property = DataUrl*/
const Value V19816_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19816_302 = {S23, 0, &V19816_302_57222, &P19816_301};	/*Property = DeviceOrientation*/
const Value V19816_304_57227 = {S2, NULL};	/*Value = True*/
const Property P19816_304 = {S24, 0, &V19816_304_57227, &P19816_302};	/*Property = FileReader*/
const Value V19816_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19816_305 = {S25, 0, &V19816_305_57231, &P19816_304};	/*Property = FileSaver*/
const Value V19816_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19816_306 = {S26, 0, &V19816_306_57237, &P19816_305};	/*Property = FileWriter*/
const Value V19816_307_57239 = {S2, NULL};	/*Value = True*/
const Property P19816_307 = {S27, 0, &V19816_307_57239, &P19816_306};	/*Property = FormData*/
const Value V19816_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19816_309 = {S28, 0, &V19816_309_57246, &P19816_307};	/*Property = Fullscreen*/
const Value V19816_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19816_310 = {S29, 0, &V19816_310_57248, &P19816_309};	/*Property = GeoLocation*/
const Value V19816_312_57254 = {S2, NULL};	/*Value = True*/
const Property P19816_312 = {S30, 0, &V19816_312_57254, &P19816_310};	/*Property = History*/
const Value V19816_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19816_314 = {S31, 0, &V19816_314_57261, &P19816_312};	/*Property = Html5*/
const Value V19816_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19816_313 = {S32, 0, &V19816_313_57258, &P19816_314};	/*Property = Html-Media-Capture*/
const Value V19816_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19816_315 = {S33, 0, &V19816_315_57264, &P19816_313};	/*Property = Iframe*/
const Value V19816_316_57266 = {S2, NULL};	/*Value = True*/
const Property P19816_316 = {S34, 0, &V19816_316_57266, &P19816_315};	/*Property = IndexedDB*/
const Value V19816_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19816_317 = {S35, 0, &V19816_317_57269, &P19816_316};	/*Property = Json*/
const Value V19816_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19816_15 = {S37, 0, &V19816_15_28, &P19816_317};	/*Property = LayoutEngine*/
const Value V19816_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19816_344 = {S38, 0, &V19816_344_57272, &P19816_15};	/*Property = Masking*/
const Value V19816_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19816_320 = {S39, 0, &V19816_320_57290, &P19816_344};	/*Property = PostMessage*/
const Value V19816_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19816_321 = {S40, 0, &V19816_321_57293, &P19816_320};	/*Property = Progress*/
const Value V19816_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19816_322 = {S41, 0, &V19816_322_57296, &P19816_321};	/*Property = Prompts*/
const Value V19816_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19816_323 = {S42, 0, &V19816_323_57299, &P19816_322};	/*Property = Selector*/
const Value V19816_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19816_324 = {S43, 0, &V19816_324_57306, &P19816_323};	/*Property = Svg*/
const Value V19816_327_57314 = {S2, NULL};	/*Value = True*/
const Property P19816_327 = {S44, 0, &V19816_327_57314, &P19816_324};	/*Property = TouchEvents*/
const Value V19816_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19816_328 = {S45, 0, &V19816_328_57318, &P19816_327};	/*Property = Track*/
const Value V19816_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19816_329 = {S46, 0, &V19816_329_57323, &P19816_328};	/*Property = Video*/
const Value V19816_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19816_330 = {S47, 0, &V19816_330_57326, &P19816_329};	/*Property = Viewport*/
const Value V19816_332_57339 = {S0, NULL};	/*Value = False*/
const Property P19816_332 = {S48, 0, &V19816_332_57339, &P19816_330};	/*Property = WebWorkers*/
const Value V19816_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19816_333 = {S49, 0, &V19816_333_57342, &P19816_332};	/*Property = Xhr2*/
const Profile P19816 = {"19816", &P19816_333};	/*Profile ID - 19816*/
const Value V19871_1_1 = {S2, NULL};	/*Value = True*/
const Property P19871_1 = {S50, 0, &V19871_1_1, NULL};	/*Property = IsMobile*/
const Value V19871_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P19871_5 = {S52, 0, &V19871_5_1277, &P19871_1};	/*Property = ScreenPixelsHeight*/
const Value V19871_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P19871_4 = {S54, 0, &V19871_4_1279, &P19871_5};	/*Property = ScreenPixelsWidth*/
const Profile P19871 = {"19871", &P19871_4};	/*Profile ID - 19871*/
const Value V23532_1_1 = {S2, NULL};	/*Value = True*/
const Property P23532_1 = {S50, 0, &V23532_1_1, NULL};	/*Property = IsMobile*/
const Value V23532_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P23532_5 = {S52, 0, &V23532_5_1233, &P23532_1};	/*Property = ScreenPixelsHeight*/
const Value V23532_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P23532_4 = {S54, 0, &V23532_4_1235, &P23532_5};	/*Property = ScreenPixelsWidth*/
const Profile P23532 = {"23532", &P23532_4};	/*Profile ID - 23532*/
const Value V23414_1_1 = {S2, NULL};	/*Value = True*/
const Property P23414_1 = {S50, 0, &V23414_1_1, NULL};	/*Property = IsMobile*/
const Value V23414_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23414_5 = {S52, 0, &V23414_5_1277, &P23414_1};	/*Property = ScreenPixelsHeight*/
const Value V23414_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23414_4 = {S54, 0, &V23414_4_1279, &P23414_5};	/*Property = ScreenPixelsWidth*/
const Profile P23414 = {"23414", &P23414_4};	/*Profile ID - 23414*/
const Value V22965_1_1 = {S2, NULL};	/*Value = True*/
const Property P22965_1 = {S50, 0, &V22965_1_1, NULL};	/*Property = IsMobile*/
const Value V22965_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P22965_5 = {S52, 0, &V22965_5_1293, &P22965_1};	/*Property = ScreenPixelsHeight*/
const Value V22965_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P22965_4 = {S54, 0, &V22965_4_1296, &P22965_5};	/*Property = ScreenPixelsWidth*/
const Profile P22965 = {"22965", &P22965_4};	/*Profile ID - 22965*/
const Value V20105_1_1 = {S2, NULL};	/*Value = True*/
const Property P20105_1 = {S50, 0, &V20105_1_1, NULL};	/*Property = IsMobile*/
const Value V20105_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P20105_5 = {S52, 0, &V20105_5_1269, &P20105_1};	/*Property = ScreenPixelsHeight*/
const Value V20105_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20105_4 = {S54, 0, &V20105_4_1229, &P20105_5};	/*Property = ScreenPixelsWidth*/
const Profile P20105 = {"20105", &P20105_4};	/*Profile ID - 20105*/
const Value V20322_1_1 = {S2, NULL};	/*Value = True*/
const Property P20322_1 = {S50, 0, &V20322_1_1, NULL};	/*Property = IsMobile*/
const Value V20322_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20322_5 = {S52, 0, &V20322_5_1277, &P20322_1};	/*Property = ScreenPixelsHeight*/
const Value V20322_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20322_4 = {S54, 0, &V20322_4_1279, &P20322_5};	/*Property = ScreenPixelsWidth*/
const Profile P20322 = {"20322", &P20322_4};	/*Profile ID - 20322*/
const Value V20154_1_1 = {S2, NULL};	/*Value = True*/
const Property P20154_1 = {S50, 0, &V20154_1_1, NULL};	/*Property = IsMobile*/
const Value V20154_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20154_5 = {S52, 0, &V20154_5_1458, &P20154_1};	/*Property = ScreenPixelsHeight*/
const Value V20154_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20154_4 = {S54, 0, &V20154_4_1460, &P20154_5};	/*Property = ScreenPixelsWidth*/
const Profile P20154 = {"20154", &P20154_4};	/*Profile ID - 20154*/
const Value V21626_1_1 = {S2, NULL};	/*Value = True*/
const Property P21626_1 = {S50, 0, &V21626_1_1, NULL};	/*Property = IsMobile*/
const Value V21626_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21626_5 = {S52, 0, &V21626_5_1227, &P21626_1};	/*Property = ScreenPixelsHeight*/
const Value V21626_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21626_4 = {S54, 0, &V21626_4_1229, &P21626_5};	/*Property = ScreenPixelsWidth*/
const Profile P21626 = {"21626", &P21626_4};	/*Profile ID - 21626*/
const Value V20129_1_1 = {S2, NULL};	/*Value = True*/
const Property P20129_1 = {S50, 0, &V20129_1_1, NULL};	/*Property = IsMobile*/
const Value V20129_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20129_5 = {S52, 0, &V20129_5_9981, &P20129_1};	/*Property = ScreenPixelsHeight*/
const Value V20129_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20129_4 = {S54, 0, &V20129_4_39098, &P20129_5};	/*Property = ScreenPixelsWidth*/
const Profile P20129 = {"20129", &P20129_4};	/*Profile ID - 20129*/
const Value V20143_1_1 = {S2, NULL};	/*Value = True*/
const Property P20143_1 = {S50, 0, &V20143_1_1, NULL};	/*Property = IsMobile*/
const Value V20143_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20143_5 = {S52, 0, &V20143_5_9981, &P20143_1};	/*Property = ScreenPixelsHeight*/
const Value V20143_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20143_4 = {S54, 0, &V20143_4_2336, &P20143_5};	/*Property = ScreenPixelsWidth*/
const Profile P20143 = {"20143", &P20143_4};	/*Profile ID - 20143*/
const Value V20827_1_1 = {S2, NULL};	/*Value = True*/
const Property P20827_1 = {S50, 0, &V20827_1_1, NULL};	/*Property = IsMobile*/
const Value V20827_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20827_5 = {S52, 0, &V20827_5_1277, &P20827_1};	/*Property = ScreenPixelsHeight*/
const Value V20827_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20827_4 = {S54, 0, &V20827_4_1279, &P20827_5};	/*Property = ScreenPixelsWidth*/
const Profile P20827 = {"20827", &P20827_4};	/*Profile ID - 20827*/
const Value V18593_1_1 = {S2, NULL};	/*Value = True*/
const Property P18593_1 = {S50, 0, &V18593_1_1, NULL};	/*Property = IsMobile*/
const Value V18593_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P18593_5 = {S52, 0, &V18593_5_1222, &P18593_1};	/*Property = ScreenPixelsHeight*/
const Value V18593_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P18593_4 = {S54, 0, &V18593_4_1224, &P18593_5};	/*Property = ScreenPixelsWidth*/
const Profile P18593 = {"18593", &P18593_4};	/*Profile ID - 18593*/
const Value V20437_1_1 = {S2, NULL};	/*Value = True*/
const Property P20437_1 = {S50, 0, &V20437_1_1, NULL};	/*Property = IsMobile*/
const Value V20437_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20437_5 = {S52, 0, &V20437_5_9981, &P20437_1};	/*Property = ScreenPixelsHeight*/
const Value V20437_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20437_4 = {S54, 0, &V20437_4_39098, &P20437_5};	/*Property = ScreenPixelsWidth*/
const Profile P20437 = {"20437", &P20437_4};	/*Profile ID - 20437*/
const Value V20436_1_1 = {S2, NULL};	/*Value = True*/
const Property P20436_1 = {S50, 0, &V20436_1_1, NULL};	/*Property = IsMobile*/
const Value V20436_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20436_5 = {S52, 0, &V20436_5_1227, &P20436_1};	/*Property = ScreenPixelsHeight*/
const Value V20436_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20436_4 = {S54, 0, &V20436_4_1229, &P20436_5};	/*Property = ScreenPixelsWidth*/
const Profile P20436 = {"20436", &P20436_4};	/*Profile ID - 20436*/
const Value V24354_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24354_279 = {S1, 0, &V24354_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24354_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24354_282 = {S3, 0, &V24354_282_57154, &P24354_279};	/*Property = BlobBuilder*/
const Value V24354_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24354_350 = {S4, 0, &V24354_350_57444, &P24354_282};	/*Property = Canvas*/
const Value V24354_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24354_283 = {S5, 0, &V24354_283_57159, &P24354_350};	/*Property = CssBackground*/
const Value V24354_284_57163 = {S0, NULL};	/*Value = False*/
const Property P24354_284 = {S6, 0, &V24354_284_57163, &P24354_283};	/*Property = CssBorderImage*/
const Value V24354_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24354_285 = {S7, 0, &V24354_285_57166, &P24354_284};	/*Property = CssCanvas*/
const Value V24354_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24354_286 = {S8, 0, &V24354_286_57168, &P24354_285};	/*Property = CssColor*/
const Value V24354_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24354_237 = {S9, 0, &V24354_237_55786, &P24354_286};	/*Property = CssColumn*/
const Value V24354_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24354_287 = {S10, 0, &V24354_287_57172, &P24354_237};	/*Property = CssFlexbox*/
const Value V24354_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24354_288 = {S11, 0, &V24354_288_57175, &P24354_287};	/*Property = CssFont*/
const Value V24354_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24354_289 = {S12, 0, &V24354_289_57759, &P24354_288};	/*Property = CssImages*/
const Value V24354_291_57181 = {S0, NULL};	/*Value = False*/
const Property P24354_291 = {S13, 0, &V24354_291_57181, &P24354_289};	/*Property = CssMediaQueries*/
const Value V24354_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24354_292 = {S14, 0, &V24354_292_57183, &P24354_291};	/*Property = CssMinMax*/
const Value V24354_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24354_293 = {S15, 0, &V24354_293_57187, &P24354_292};	/*Property = CssOverflow*/
const Value V24354_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24354_294 = {S16, 0, &V24354_294_57189, &P24354_293};	/*Property = CssPosition*/
const Value V24354_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24354_295 = {S17, 0, &V24354_295_57192, &P24354_294};	/*Property = CssText*/
const Value V24354_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24354_297 = {S18, 0, &V24354_297_57195, &P24354_295};	/*Property = CssTransforms*/
const Value V24354_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24354_241 = {S19, 0, &V24354_241_55799, &P24354_297};	/*Property = CssTransitions*/
const Value V24354_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24354_298 = {S20, 0, &V24354_298_57206, &P24354_241};	/*Property = CssUI*/
const Value V24354_300_57216 = {S0, NULL};	/*Value = False*/
const Property P24354_300 = {S21, 0, &V24354_300_57216, &P24354_298};	/*Property = DataSet*/
const Value V24354_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24354_301 = {S22, 0, &V24354_301_57218, &P24354_300};	/*Property = DataUrl*/
const Value V24354_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24354_302 = {S23, 0, &V24354_302_57222, &P24354_301};	/*Property = DeviceOrientation*/
const Value V24354_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24354_304 = {S24, 0, &V24354_304_57227, &P24354_302};	/*Property = FileReader*/
const Value V24354_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24354_305 = {S25, 0, &V24354_305_57231, &P24354_304};	/*Property = FileSaver*/
const Value V24354_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24354_306 = {S26, 0, &V24354_306_57237, &P24354_305};	/*Property = FileWriter*/
const Value V24354_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24354_307 = {S27, 0, &V24354_307_57239, &P24354_306};	/*Property = FormData*/
const Value V24354_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24354_309 = {S28, 0, &V24354_309_57246, &P24354_307};	/*Property = Fullscreen*/
const Value V24354_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24354_310 = {S29, 0, &V24354_310_57248, &P24354_309};	/*Property = GeoLocation*/
const Value V24354_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24354_312 = {S30, 0, &V24354_312_57254, &P24354_310};	/*Property = History*/
const Value V24354_314_57261 = {S0, NULL};	/*Value = False*/
const Property P24354_314 = {S31, 0, &V24354_314_57261, &P24354_312};	/*Property = Html5*/
const Value V24354_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24354_313 = {S32, 0, &V24354_313_57258, &P24354_314};	/*Property = Html-Media-Capture*/
const Value V24354_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24354_315 = {S33, 0, &V24354_315_57264, &P24354_313};	/*Property = Iframe*/
const Value V24354_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24354_316 = {S34, 0, &V24354_316_57266, &P24354_315};	/*Property = IndexedDB*/
const Value V24354_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24354_317 = {S35, 0, &V24354_317_57269, &P24354_316};	/*Property = Json*/
const Value V24354_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24354_15 = {S37, 0, &V24354_15_54167, &P24354_317};	/*Property = LayoutEngine*/
const Value V24354_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24354_344 = {S38, 0, &V24354_344_57272, &P24354_15};	/*Property = Masking*/
const Value V24354_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24354_320 = {S39, 0, &V24354_320_57290, &P24354_344};	/*Property = PostMessage*/
const Value V24354_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24354_321 = {S40, 0, &V24354_321_57293, &P24354_320};	/*Property = Progress*/
const Value V24354_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24354_322 = {S41, 0, &V24354_322_57296, &P24354_321};	/*Property = Prompts*/
const Value V24354_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24354_323 = {S42, 0, &V24354_323_57299, &P24354_322};	/*Property = Selector*/
const Value V24354_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24354_324 = {S43, 0, &V24354_324_57306, &P24354_323};	/*Property = Svg*/
const Value V24354_327_57315 = {S0, NULL};	/*Value = False*/
const Property P24354_327 = {S44, 0, &V24354_327_57315, &P24354_324};	/*Property = TouchEvents*/
const Value V24354_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24354_328 = {S45, 0, &V24354_328_57318, &P24354_327};	/*Property = Track*/
const Value V24354_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24354_329 = {S46, 0, &V24354_329_57323, &P24354_328};	/*Property = Video*/
const Value V24354_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24354_330 = {S47, 0, &V24354_330_57326, &P24354_329};	/*Property = Viewport*/
const Value V24354_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24354_332 = {S48, 0, &V24354_332_57339, &P24354_330};	/*Property = WebWorkers*/
const Value V24354_333_57341 = {S2, NULL};	/*Value = True*/
const Property P24354_333 = {S49, 0, &V24354_333_57341, &P24354_332};	/*Property = Xhr2*/
const Profile P24354 = {"24354", &P24354_333};	/*Profile ID - 24354*/
const Value V18503_1_1 = {S2, NULL};	/*Value = True*/
const Property P18503_1 = {S50, 0, &V18503_1_1, NULL};	/*Property = IsMobile*/
const Value V18503_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18503_5 = {S52, 0, &V18503_5_1227, &P18503_1};	/*Property = ScreenPixelsHeight*/
const Value V18503_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18503_4 = {S54, 0, &V18503_4_1229, &P18503_5};	/*Property = ScreenPixelsWidth*/
const Profile P18503 = {"18503", &P18503_4};	/*Profile ID - 18503*/
const Value V18158_279_57148 = {S0, NULL};	/*Value = False*/
const Property P18158_279 = {S1, 0, &V18158_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V18158_282_57154 = {S0, NULL};	/*Value = False*/
const Property P18158_282 = {S3, 0, &V18158_282_57154, &P18158_279};	/*Property = BlobBuilder*/
const Value V18158_350_57444 = {S2, NULL};	/*Value = True*/
const Property P18158_350 = {S4, 0, &V18158_350_57444, &P18158_282};	/*Property = Canvas*/
const Value V18158_283_57159 = {S2, NULL};	/*Value = True*/
const Property P18158_283 = {S5, 0, &V18158_283_57159, &P18158_350};	/*Property = CssBackground*/
const Value V18158_284_57163 = {S0, NULL};	/*Value = False*/
const Property P18158_284 = {S6, 0, &V18158_284_57163, &P18158_283};	/*Property = CssBorderImage*/
const Value V18158_285_57166 = {S0, NULL};	/*Value = False*/
const Property P18158_285 = {S7, 0, &V18158_285_57166, &P18158_284};	/*Property = CssCanvas*/
const Value V18158_286_57168 = {S2, NULL};	/*Value = True*/
const Property P18158_286 = {S8, 0, &V18158_286_57168, &P18158_285};	/*Property = CssColor*/
const Value V18158_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18158_237 = {S9, 0, &V18158_237_55786, &P18158_286};	/*Property = CssColumn*/
const Value V18158_287_57172 = {S0, NULL};	/*Value = False*/
const Property P18158_287 = {S10, 0, &V18158_287_57172, &P18158_237};	/*Property = CssFlexbox*/
const Value V18158_288_57175 = {S0, NULL};	/*Value = False*/
const Property P18158_288 = {S11, 0, &V18158_288_57175, &P18158_287};	/*Property = CssFont*/
const Value V18158_289_57759 = {S0, NULL};	/*Value = False*/
const Property P18158_289 = {S12, 0, &V18158_289_57759, &P18158_288};	/*Property = CssImages*/
const Value V18158_291_57181 = {S0, NULL};	/*Value = False*/
const Property P18158_291 = {S13, 0, &V18158_291_57181, &P18158_289};	/*Property = CssMediaQueries*/
const Value V18158_292_57183 = {S2, NULL};	/*Value = True*/
const Property P18158_292 = {S14, 0, &V18158_292_57183, &P18158_291};	/*Property = CssMinMax*/
const Value V18158_293_57187 = {S0, NULL};	/*Value = False*/
const Property P18158_293 = {S15, 0, &V18158_293_57187, &P18158_292};	/*Property = CssOverflow*/
const Value V18158_294_57190 = {S0, NULL};	/*Value = False*/
const Property P18158_294 = {S16, 0, &V18158_294_57190, &P18158_293};	/*Property = CssPosition*/
const Value V18158_295_57192 = {S2, NULL};	/*Value = True*/
const Property P18158_295 = {S17, 0, &V18158_295_57192, &P18158_294};	/*Property = CssText*/
const Value V18158_297_57195 = {S2, NULL};	/*Value = True*/
const Property P18158_297 = {S18, 0, &V18158_297_57195, &P18158_295};	/*Property = CssTransforms*/
const Value V18158_241_55799 = {S2, NULL};	/*Value = True*/
const Property P18158_241 = {S19, 0, &V18158_241_55799, &P18158_297};	/*Property = CssTransitions*/
const Value V18158_298_57206 = {S2, NULL};	/*Value = True*/
const Property P18158_298 = {S20, 0, &V18158_298_57206, &P18158_241};	/*Property = CssUI*/
const Value V18158_300_57216 = {S0, NULL};	/*Value = False*/
const Property P18158_300 = {S21, 0, &V18158_300_57216, &P18158_298};	/*Property = DataSet*/
const Value V18158_301_57218 = {S2, NULL};	/*Value = True*/
const Property P18158_301 = {S22, 0, &V18158_301_57218, &P18158_300};	/*Property = DataUrl*/
const Value V18158_302_57222 = {S0, NULL};	/*Value = False*/
const Property P18158_302 = {S23, 0, &V18158_302_57222, &P18158_301};	/*Property = DeviceOrientation*/
const Value V18158_304_57228 = {S0, NULL};	/*Value = False*/
const Property P18158_304 = {S24, 0, &V18158_304_57228, &P18158_302};	/*Property = FileReader*/
const Value V18158_305_57231 = {S0, NULL};	/*Value = False*/
const Property P18158_305 = {S25, 0, &V18158_305_57231, &P18158_304};	/*Property = FileSaver*/
const Value V18158_306_57237 = {S0, NULL};	/*Value = False*/
const Property P18158_306 = {S26, 0, &V18158_306_57237, &P18158_305};	/*Property = FileWriter*/
const Value V18158_307_57240 = {S0, NULL};	/*Value = False*/
const Property P18158_307 = {S27, 0, &V18158_307_57240, &P18158_306};	/*Property = FormData*/
const Value V18158_309_57246 = {S0, NULL};	/*Value = False*/
const Property P18158_309 = {S28, 0, &V18158_309_57246, &P18158_307};	/*Property = Fullscreen*/
const Value V18158_310_57248 = {S2, NULL};	/*Value = True*/
const Property P18158_310 = {S29, 0, &V18158_310_57248, &P18158_309};	/*Property = GeoLocation*/
const Value V18158_312_57255 = {S0, NULL};	/*Value = False*/
const Property P18158_312 = {S30, 0, &V18158_312_57255, &P18158_310};	/*Property = History*/
const Value V18158_314_57261 = {S0, NULL};	/*Value = False*/
const Property P18158_314 = {S31, 0, &V18158_314_57261, &P18158_312};	/*Property = Html5*/
const Value V18158_313_57258 = {S0, NULL};	/*Value = False*/
const Property P18158_313 = {S32, 0, &V18158_313_57258, &P18158_314};	/*Property = Html-Media-Capture*/
const Value V18158_315_57264 = {S0, NULL};	/*Value = False*/
const Property P18158_315 = {S33, 0, &V18158_315_57264, &P18158_313};	/*Property = Iframe*/
const Value V18158_316_57267 = {S0, NULL};	/*Value = False*/
const Property P18158_316 = {S34, 0, &V18158_316_57267, &P18158_315};	/*Property = IndexedDB*/
const Value V18158_317_57269 = {S2, NULL};	/*Value = True*/
const Property P18158_317 = {S35, 0, &V18158_317_57269, &P18158_316};	/*Property = Json*/
const Value V18158_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P18158_15 = {S37, 0, &V18158_15_72, &P18158_317};	/*Property = LayoutEngine*/
const Value V18158_344_57272 = {S2, NULL};	/*Value = True*/
const Property P18158_344 = {S38, 0, &V18158_344_57272, &P18158_15};	/*Property = Masking*/
const Value V18158_320_57290 = {S2, NULL};	/*Value = True*/
const Property P18158_320 = {S39, 0, &V18158_320_57290, &P18158_344};	/*Property = PostMessage*/
const Value V18158_321_57293 = {S2, NULL};	/*Value = True*/
const Property P18158_321 = {S40, 0, &V18158_321_57293, &P18158_320};	/*Property = Progress*/
const Value V18158_322_57296 = {S2, NULL};	/*Value = True*/
const Property P18158_322 = {S41, 0, &V18158_322_57296, &P18158_321};	/*Property = Prompts*/
const Value V18158_323_57299 = {S2, NULL};	/*Value = True*/
const Property P18158_323 = {S42, 0, &V18158_323_57299, &P18158_322};	/*Property = Selector*/
const Value V18158_324_57306 = {S0, NULL};	/*Value = False*/
const Property P18158_324 = {S43, 0, &V18158_324_57306, &P18158_323};	/*Property = Svg*/
const Value V18158_327_57315 = {S0, NULL};	/*Value = False*/
const Property P18158_327 = {S44, 0, &V18158_327_57315, &P18158_324};	/*Property = TouchEvents*/
const Value V18158_328_57318 = {S0, NULL};	/*Value = False*/
const Property P18158_328 = {S45, 0, &V18158_328_57318, &P18158_327};	/*Property = Track*/
const Value V18158_329_57323 = {S2, NULL};	/*Value = True*/
const Property P18158_329 = {S46, 0, &V18158_329_57323, &P18158_328};	/*Property = Video*/
const Value V18158_330_57326 = {S2, NULL};	/*Value = True*/
const Property P18158_330 = {S47, 0, &V18158_330_57326, &P18158_329};	/*Property = Viewport*/
const Value V18158_332_57339 = {S0, NULL};	/*Value = False*/
const Property P18158_332 = {S48, 0, &V18158_332_57339, &P18158_330};	/*Property = WebWorkers*/
const Value V18158_333_57342 = {S0, NULL};	/*Value = False*/
const Property P18158_333 = {S49, 0, &V18158_333_57342, &P18158_332};	/*Property = Xhr2*/
const Profile P18158 = {"18158", &P18158_333};	/*Profile ID - 18158*/
const Value V21508_1_1 = {S2, NULL};	/*Value = True*/
const Property P21508_1 = {S50, 0, &V21508_1_1, NULL};	/*Property = IsMobile*/
const Value V21508_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21508_5 = {S52, 0, &V21508_5_1458, &P21508_1};	/*Property = ScreenPixelsHeight*/
const Value V21508_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21508_4 = {S54, 0, &V21508_4_37617, &P21508_5};	/*Property = ScreenPixelsWidth*/
const Profile P21508 = {"21508", &P21508_4};	/*Profile ID - 21508*/
const Value V21071_1_1 = {S2, NULL};	/*Value = True*/
const Property P21071_1 = {S50, 0, &V21071_1_1, NULL};	/*Property = IsMobile*/
const Value V21071_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21071_5 = {S52, 0, &V21071_5_1227, &P21071_1};	/*Property = ScreenPixelsHeight*/
const Value V21071_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P21071_4 = {S54, 0, &V21071_4_1477, &P21071_5};	/*Property = ScreenPixelsWidth*/
const Profile P21071 = {"21071", &P21071_4};	/*Profile ID - 21071*/
const Value V20921_1_1 = {S2, NULL};	/*Value = True*/
const Property P20921_1 = {S50, 0, &V20921_1_1, NULL};	/*Property = IsMobile*/
const Value V20921_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20921_5 = {S52, 0, &V20921_5_1227, &P20921_1};	/*Property = ScreenPixelsHeight*/
const Value V20921_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20921_4 = {S54, 0, &V20921_4_1229, &P20921_5};	/*Property = ScreenPixelsWidth*/
const Profile P20921 = {"20921", &P20921_4};	/*Profile ID - 20921*/
const Value V20987_1_1 = {S2, NULL};	/*Value = True*/
const Property P20987_1 = {S50, 0, &V20987_1_1, NULL};	/*Property = IsMobile*/
const Value V20987_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20987_5 = {S52, 0, &V20987_5_9981, &P20987_1};	/*Property = ScreenPixelsHeight*/
const Value V20987_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20987_4 = {S54, 0, &V20987_4_2336, &P20987_5};	/*Property = ScreenPixelsWidth*/
const Profile P20987 = {"20987", &P20987_4};	/*Profile ID - 20987*/
const Value V21346_1_1 = {S2, NULL};	/*Value = True*/
const Property P21346_1 = {S50, 0, &V21346_1_1, NULL};	/*Property = IsMobile*/
const Value V21346_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21346_5 = {S52, 0, &V21346_5_1458, &P21346_1};	/*Property = ScreenPixelsHeight*/
const Value V21346_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21346_4 = {S54, 0, &V21346_4_1460, &P21346_5};	/*Property = ScreenPixelsWidth*/
const Profile P21346 = {"21346", &P21346_4};	/*Profile ID - 21346*/
const Value V20813_1_1 = {S2, NULL};	/*Value = True*/
const Property P20813_1 = {S50, 0, &V20813_1_1, NULL};	/*Property = IsMobile*/
const Value V20813_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20813_5 = {S52, 0, &V20813_5_1227, &P20813_1};	/*Property = ScreenPixelsHeight*/
const Value V20813_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20813_4 = {S54, 0, &V20813_4_1229, &P20813_5};	/*Property = ScreenPixelsWidth*/
const Profile P20813 = {"20813", &P20813_4};	/*Profile ID - 20813*/
const Value V19965_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_279 = {S1, 0, &V19965_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V19965_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_282 = {S3, 0, &V19965_282_57155, &P19965_279};	/*Property = BlobBuilder*/
const Value V19965_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_350 = {S4, 0, &V19965_350_57446, &P19965_282};	/*Property = Canvas*/
const Value V19965_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_283 = {S5, 0, &V19965_283_57161, &P19965_350};	/*Property = CssBackground*/
const Value V19965_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_284 = {S6, 0, &V19965_284_57164, &P19965_283};	/*Property = CssBorderImage*/
const Value V19965_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_285 = {S7, 0, &V19965_285_57167, &P19965_284};	/*Property = CssCanvas*/
const Value V19965_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_286 = {S8, 0, &V19965_286_57170, &P19965_285};	/*Property = CssColor*/
const Value V19965_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_237 = {S9, 0, &V19965_237_55786, &P19965_286};	/*Property = CssColumn*/
const Value V19965_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_287 = {S10, 0, &V19965_287_57173, &P19965_237};	/*Property = CssFlexbox*/
const Value V19965_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_288 = {S11, 0, &V19965_288_57176, &P19965_287};	/*Property = CssFont*/
const Value V19965_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_289 = {S12, 0, &V19965_289_86899, &P19965_288};	/*Property = CssImages*/
const Value V19965_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_291 = {S13, 0, &V19965_291_57182, &P19965_289};	/*Property = CssMediaQueries*/
const Value V19965_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_292 = {S14, 0, &V19965_292_57185, &P19965_291};	/*Property = CssMinMax*/
const Value V19965_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_293 = {S15, 0, &V19965_293_57188, &P19965_292};	/*Property = CssOverflow*/
const Value V19965_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_294 = {S16, 0, &V19965_294_57191, &P19965_293};	/*Property = CssPosition*/
const Value V19965_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_295 = {S17, 0, &V19965_295_57194, &P19965_294};	/*Property = CssText*/
const Value V19965_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_297 = {S18, 0, &V19965_297_57197, &P19965_295};	/*Property = CssTransforms*/
const Value V19965_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_241 = {S19, 0, &V19965_241_55801, &P19965_297};	/*Property = CssTransitions*/
const Value V19965_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_298 = {S20, 0, &V19965_298_57208, &P19965_241};	/*Property = CssUI*/
const Value V19965_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_300 = {S21, 0, &V19965_300_57217, &P19965_298};	/*Property = DataSet*/
const Value V19965_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_301 = {S22, 0, &V19965_301_57220, &P19965_300};	/*Property = DataUrl*/
const Value V19965_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_302 = {S23, 0, &V19965_302_57223, &P19965_301};	/*Property = DeviceOrientation*/
const Value V19965_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_304 = {S24, 0, &V19965_304_57229, &P19965_302};	/*Property = FileReader*/
const Value V19965_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_305 = {S25, 0, &V19965_305_57232, &P19965_304};	/*Property = FileSaver*/
const Value V19965_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_306 = {S26, 0, &V19965_306_57238, &P19965_305};	/*Property = FileWriter*/
const Value V19965_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_307 = {S27, 0, &V19965_307_57241, &P19965_306};	/*Property = FormData*/
const Value V19965_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_309 = {S28, 0, &V19965_309_57247, &P19965_307};	/*Property = Fullscreen*/
const Value V19965_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_310 = {S29, 0, &V19965_310_57250, &P19965_309};	/*Property = GeoLocation*/
const Value V19965_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_312 = {S30, 0, &V19965_312_57256, &P19965_310};	/*Property = History*/
const Value V19965_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_314 = {S31, 0, &V19965_314_57262, &P19965_312};	/*Property = Html5*/
const Value V19965_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_313 = {S32, 0, &V19965_313_57259, &P19965_314};	/*Property = Html-Media-Capture*/
const Value V19965_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_315 = {S33, 0, &V19965_315_57265, &P19965_313};	/*Property = Iframe*/
const Value V19965_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_316 = {S34, 0, &V19965_316_57268, &P19965_315};	/*Property = IndexedDB*/
const Value V19965_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_317 = {S35, 0, &V19965_317_57271, &P19965_316};	/*Property = Json*/
const Value V19965_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P19965_15 = {S37, 0, &V19965_15_29, &P19965_317};	/*Property = LayoutEngine*/
const Value V19965_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_344 = {S38, 0, &V19965_344_57274, &P19965_15};	/*Property = Masking*/
const Value V19965_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_320 = {S39, 0, &V19965_320_57292, &P19965_344};	/*Property = PostMessage*/
const Value V19965_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_321 = {S40, 0, &V19965_321_57295, &P19965_320};	/*Property = Progress*/
const Value V19965_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_322 = {S41, 0, &V19965_322_57298, &P19965_321};	/*Property = Prompts*/
const Value V19965_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_323 = {S42, 0, &V19965_323_57301, &P19965_322};	/*Property = Selector*/
const Value V19965_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_324 = {S43, 0, &V19965_324_57307, &P19965_323};	/*Property = Svg*/
const Value V19965_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_327 = {S44, 0, &V19965_327_57316, &P19965_324};	/*Property = TouchEvents*/
const Value V19965_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_328 = {S45, 0, &V19965_328_57319, &P19965_327};	/*Property = Track*/
const Value V19965_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_329 = {S46, 0, &V19965_329_57325, &P19965_328};	/*Property = Video*/
const Value V19965_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_330 = {S47, 0, &V19965_330_57328, &P19965_329};	/*Property = Viewport*/
const Value V19965_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_332 = {S48, 0, &V19965_332_57340, &P19965_330};	/*Property = WebWorkers*/
const Value V19965_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P19965_333 = {S49, 0, &V19965_333_57343, &P19965_332};	/*Property = Xhr2*/
const Profile P19965 = {"19965", &P19965_333};	/*Profile ID - 19965*/
const Value V22377_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22377_279 = {S1, 0, &V22377_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22377_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22377_282 = {S3, 0, &V22377_282_57154, &P22377_279};	/*Property = BlobBuilder*/
const Value V22377_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22377_350 = {S4, 0, &V22377_350_57444, &P22377_282};	/*Property = Canvas*/
const Value V22377_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22377_283 = {S5, 0, &V22377_283_57159, &P22377_350};	/*Property = CssBackground*/
const Value V22377_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22377_284 = {S6, 0, &V22377_284_57164, &P22377_283};	/*Property = CssBorderImage*/
const Value V22377_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22377_285 = {S7, 0, &V22377_285_57166, &P22377_284};	/*Property = CssCanvas*/
const Value V22377_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22377_286 = {S8, 0, &V22377_286_57168, &P22377_285};	/*Property = CssColor*/
const Value V22377_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22377_237 = {S9, 0, &V22377_237_55786, &P22377_286};	/*Property = CssColumn*/
const Value V22377_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22377_287 = {S10, 0, &V22377_287_57172, &P22377_237};	/*Property = CssFlexbox*/
const Value V22377_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22377_288 = {S11, 0, &V22377_288_57174, &P22377_287};	/*Property = CssFont*/
const Value V22377_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22377_289 = {S12, 0, &V22377_289_57759, &P22377_288};	/*Property = CssImages*/
const Value V22377_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22377_291 = {S13, 0, &V22377_291_57180, &P22377_289};	/*Property = CssMediaQueries*/
const Value V22377_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22377_292 = {S14, 0, &V22377_292_57183, &P22377_291};	/*Property = CssMinMax*/
const Value V22377_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22377_293 = {S15, 0, &V22377_293_57187, &P22377_292};	/*Property = CssOverflow*/
const Value V22377_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22377_294 = {S16, 0, &V22377_294_57189, &P22377_293};	/*Property = CssPosition*/
const Value V22377_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22377_295 = {S17, 0, &V22377_295_57192, &P22377_294};	/*Property = CssText*/
const Value V22377_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22377_297 = {S18, 0, &V22377_297_57195, &P22377_295};	/*Property = CssTransforms*/
const Value V22377_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22377_241 = {S19, 0, &V22377_241_55799, &P22377_297};	/*Property = CssTransitions*/
const Value V22377_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22377_298 = {S20, 0, &V22377_298_57206, &P22377_241};	/*Property = CssUI*/
const Value V22377_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22377_300 = {S21, 0, &V22377_300_57217, &P22377_298};	/*Property = DataSet*/
const Value V22377_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22377_301 = {S22, 0, &V22377_301_57218, &P22377_300};	/*Property = DataUrl*/
const Value V22377_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22377_302 = {S23, 0, &V22377_302_57223, &P22377_301};	/*Property = DeviceOrientation*/
const Value V22377_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22377_304 = {S24, 0, &V22377_304_57227, &P22377_302};	/*Property = FileReader*/
const Value V22377_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22377_305 = {S25, 0, &V22377_305_57231, &P22377_304};	/*Property = FileSaver*/
const Value V22377_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22377_306 = {S26, 0, &V22377_306_57237, &P22377_305};	/*Property = FileWriter*/
const Value V22377_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22377_307 = {S27, 0, &V22377_307_57239, &P22377_306};	/*Property = FormData*/
const Value V22377_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22377_309 = {S28, 0, &V22377_309_57246, &P22377_307};	/*Property = Fullscreen*/
const Value V22377_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22377_310 = {S29, 0, &V22377_310_57248, &P22377_309};	/*Property = GeoLocation*/
const Value V22377_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22377_312 = {S30, 0, &V22377_312_57256, &P22377_310};	/*Property = History*/
const Value V22377_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22377_314 = {S31, 0, &V22377_314_57262, &P22377_312};	/*Property = Html5*/
const Value V22377_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22377_313 = {S32, 0, &V22377_313_57258, &P22377_314};	/*Property = Html-Media-Capture*/
const Value V22377_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22377_315 = {S33, 0, &V22377_315_57264, &P22377_313};	/*Property = Iframe*/
const Value V22377_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22377_316 = {S34, 0, &V22377_316_57266, &P22377_315};	/*Property = IndexedDB*/
const Value V22377_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22377_317 = {S35, 0, &V22377_317_57269, &P22377_316};	/*Property = Json*/
const Value V22377_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22377_15 = {S37, 0, &V22377_15_28, &P22377_317};	/*Property = LayoutEngine*/
const Value V22377_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22377_344 = {S38, 0, &V22377_344_57272, &P22377_15};	/*Property = Masking*/
const Value V22377_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22377_320 = {S39, 0, &V22377_320_57290, &P22377_344};	/*Property = PostMessage*/
const Value V22377_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22377_321 = {S40, 0, &V22377_321_57293, &P22377_320};	/*Property = Progress*/
const Value V22377_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22377_322 = {S41, 0, &V22377_322_57296, &P22377_321};	/*Property = Prompts*/
const Value V22377_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22377_323 = {S42, 0, &V22377_323_57299, &P22377_322};	/*Property = Selector*/
const Value V22377_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22377_324 = {S43, 0, &V22377_324_57307, &P22377_323};	/*Property = Svg*/
const Value V22377_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22377_327 = {S44, 0, &V22377_327_57316, &P22377_324};	/*Property = TouchEvents*/
const Value V22377_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22377_328 = {S45, 0, &V22377_328_57318, &P22377_327};	/*Property = Track*/
const Value V22377_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22377_329 = {S46, 0, &V22377_329_57323, &P22377_328};	/*Property = Video*/
const Value V22377_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22377_330 = {S47, 0, &V22377_330_57326, &P22377_329};	/*Property = Viewport*/
const Value V22377_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22377_332 = {S48, 0, &V22377_332_57340, &P22377_330};	/*Property = WebWorkers*/
const Value V22377_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22377_333 = {S49, 0, &V22377_333_57343, &P22377_332};	/*Property = Xhr2*/
const Profile P22377 = {"22377", &P22377_333};	/*Profile ID - 22377*/
const Value V19820_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19820_279 = {S1, 0, &V19820_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19820_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19820_282 = {S3, 0, &V19820_282_57154, &P19820_279};	/*Property = BlobBuilder*/
const Value V19820_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19820_350 = {S4, 0, &V19820_350_57444, &P19820_282};	/*Property = Canvas*/
const Value V19820_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19820_283 = {S5, 0, &V19820_283_57159, &P19820_350};	/*Property = CssBackground*/
const Value V19820_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19820_284 = {S6, 0, &V19820_284_57163, &P19820_283};	/*Property = CssBorderImage*/
const Value V19820_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19820_285 = {S7, 0, &V19820_285_57166, &P19820_284};	/*Property = CssCanvas*/
const Value V19820_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19820_286 = {S8, 0, &V19820_286_57168, &P19820_285};	/*Property = CssColor*/
const Value V19820_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19820_237 = {S9, 0, &V19820_237_55786, &P19820_286};	/*Property = CssColumn*/
const Value V19820_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19820_287 = {S10, 0, &V19820_287_57172, &P19820_237};	/*Property = CssFlexbox*/
const Value V19820_288_57175 = {S0, NULL};	/*Value = False*/
const Property P19820_288 = {S11, 0, &V19820_288_57175, &P19820_287};	/*Property = CssFont*/
const Value V19820_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19820_289 = {S12, 0, &V19820_289_57759, &P19820_288};	/*Property = CssImages*/
const Value V19820_291_57181 = {S0, NULL};	/*Value = False*/
const Property P19820_291 = {S13, 0, &V19820_291_57181, &P19820_289};	/*Property = CssMediaQueries*/
const Value V19820_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19820_292 = {S14, 0, &V19820_292_57183, &P19820_291};	/*Property = CssMinMax*/
const Value V19820_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19820_293 = {S15, 0, &V19820_293_57187, &P19820_292};	/*Property = CssOverflow*/
const Value V19820_294_57190 = {S0, NULL};	/*Value = False*/
const Property P19820_294 = {S16, 0, &V19820_294_57190, &P19820_293};	/*Property = CssPosition*/
const Value V19820_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19820_295 = {S17, 0, &V19820_295_57192, &P19820_294};	/*Property = CssText*/
const Value V19820_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19820_297 = {S18, 0, &V19820_297_57195, &P19820_295};	/*Property = CssTransforms*/
const Value V19820_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19820_241 = {S19, 0, &V19820_241_55799, &P19820_297};	/*Property = CssTransitions*/
const Value V19820_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19820_298 = {S20, 0, &V19820_298_57206, &P19820_241};	/*Property = CssUI*/
const Value V19820_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19820_300 = {S21, 0, &V19820_300_57216, &P19820_298};	/*Property = DataSet*/
const Value V19820_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19820_301 = {S22, 0, &V19820_301_57218, &P19820_300};	/*Property = DataUrl*/
const Value V19820_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19820_302 = {S23, 0, &V19820_302_57222, &P19820_301};	/*Property = DeviceOrientation*/
const Value V19820_304_57228 = {S0, NULL};	/*Value = False*/
const Property P19820_304 = {S24, 0, &V19820_304_57228, &P19820_302};	/*Property = FileReader*/
const Value V19820_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19820_305 = {S25, 0, &V19820_305_57231, &P19820_304};	/*Property = FileSaver*/
const Value V19820_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19820_306 = {S26, 0, &V19820_306_57237, &P19820_305};	/*Property = FileWriter*/
const Value V19820_307_57240 = {S0, NULL};	/*Value = False*/
const Property P19820_307 = {S27, 0, &V19820_307_57240, &P19820_306};	/*Property = FormData*/
const Value V19820_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19820_309 = {S28, 0, &V19820_309_57246, &P19820_307};	/*Property = Fullscreen*/
const Value V19820_310_57249 = {S0, NULL};	/*Value = False*/
const Property P19820_310 = {S29, 0, &V19820_310_57249, &P19820_309};	/*Property = GeoLocation*/
const Value V19820_312_57255 = {S0, NULL};	/*Value = False*/
const Property P19820_312 = {S30, 0, &V19820_312_57255, &P19820_310};	/*Property = History*/
const Value V19820_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19820_314 = {S31, 0, &V19820_314_57261, &P19820_312};	/*Property = Html5*/
const Value V19820_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19820_313 = {S32, 0, &V19820_313_57258, &P19820_314};	/*Property = Html-Media-Capture*/
const Value V19820_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19820_315 = {S33, 0, &V19820_315_57264, &P19820_313};	/*Property = Iframe*/
const Value V19820_316_57267 = {S0, NULL};	/*Value = False*/
const Property P19820_316 = {S34, 0, &V19820_316_57267, &P19820_315};	/*Property = IndexedDB*/
const Value V19820_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19820_317 = {S35, 0, &V19820_317_57269, &P19820_316};	/*Property = Json*/
const Value V19820_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19820_15 = {S37, 0, &V19820_15_28, &P19820_317};	/*Property = LayoutEngine*/
const Value V19820_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19820_344 = {S38, 0, &V19820_344_57272, &P19820_15};	/*Property = Masking*/
const Value V19820_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19820_320 = {S39, 0, &V19820_320_57290, &P19820_344};	/*Property = PostMessage*/
const Value V19820_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19820_321 = {S40, 0, &V19820_321_57293, &P19820_320};	/*Property = Progress*/
const Value V19820_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19820_322 = {S41, 0, &V19820_322_57296, &P19820_321};	/*Property = Prompts*/
const Value V19820_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19820_323 = {S42, 0, &V19820_323_57299, &P19820_322};	/*Property = Selector*/
const Value V19820_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19820_324 = {S43, 0, &V19820_324_57306, &P19820_323};	/*Property = Svg*/
const Value V19820_327_57315 = {S0, NULL};	/*Value = False*/
const Property P19820_327 = {S44, 0, &V19820_327_57315, &P19820_324};	/*Property = TouchEvents*/
const Value V19820_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19820_328 = {S45, 0, &V19820_328_57318, &P19820_327};	/*Property = Track*/
const Value V19820_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19820_329 = {S46, 0, &V19820_329_57323, &P19820_328};	/*Property = Video*/
const Value V19820_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19820_330 = {S47, 0, &V19820_330_57326, &P19820_329};	/*Property = Viewport*/
const Value V19820_332_57339 = {S0, NULL};	/*Value = False*/
const Property P19820_332 = {S48, 0, &V19820_332_57339, &P19820_330};	/*Property = WebWorkers*/
const Value V19820_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19820_333 = {S49, 0, &V19820_333_57342, &P19820_332};	/*Property = Xhr2*/
const Profile P19820 = {"19820", &P19820_333};	/*Profile ID - 19820*/
const Value V22514_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22514_279 = {S1, 0, &V22514_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22514_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22514_282 = {S3, 0, &V22514_282_57154, &P22514_279};	/*Property = BlobBuilder*/
const Value V22514_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22514_350 = {S4, 0, &V22514_350_57444, &P22514_282};	/*Property = Canvas*/
const Value V22514_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22514_283 = {S5, 0, &V22514_283_57159, &P22514_350};	/*Property = CssBackground*/
const Value V22514_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22514_284 = {S6, 0, &V22514_284_57163, &P22514_283};	/*Property = CssBorderImage*/
const Value V22514_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22514_285 = {S7, 0, &V22514_285_57166, &P22514_284};	/*Property = CssCanvas*/
const Value V22514_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22514_286 = {S8, 0, &V22514_286_57168, &P22514_285};	/*Property = CssColor*/
const Value V22514_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22514_237 = {S9, 0, &V22514_237_55784, &P22514_286};	/*Property = CssColumn*/
const Value V22514_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22514_287 = {S10, 0, &V22514_287_57172, &P22514_237};	/*Property = CssFlexbox*/
const Value V22514_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22514_288 = {S11, 0, &V22514_288_57174, &P22514_287};	/*Property = CssFont*/
const Value V22514_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22514_289 = {S12, 0, &V22514_289_57759, &P22514_288};	/*Property = CssImages*/
const Value V22514_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22514_291 = {S13, 0, &V22514_291_57180, &P22514_289};	/*Property = CssMediaQueries*/
const Value V22514_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22514_292 = {S14, 0, &V22514_292_57183, &P22514_291};	/*Property = CssMinMax*/
const Value V22514_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22514_293 = {S15, 0, &V22514_293_57187, &P22514_292};	/*Property = CssOverflow*/
const Value V22514_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22514_294 = {S16, 0, &V22514_294_57189, &P22514_293};	/*Property = CssPosition*/
const Value V22514_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22514_295 = {S17, 0, &V22514_295_57192, &P22514_294};	/*Property = CssText*/
const Value V22514_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22514_297 = {S18, 0, &V22514_297_57195, &P22514_295};	/*Property = CssTransforms*/
const Value V22514_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22514_241 = {S19, 0, &V22514_241_55799, &P22514_297};	/*Property = CssTransitions*/
const Value V22514_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22514_298 = {S20, 0, &V22514_298_57206, &P22514_241};	/*Property = CssUI*/
const Value V22514_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22514_300 = {S21, 0, &V22514_300_57215, &P22514_298};	/*Property = DataSet*/
const Value V22514_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22514_301 = {S22, 0, &V22514_301_57218, &P22514_300};	/*Property = DataUrl*/
const Value V22514_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22514_302 = {S23, 0, &V22514_302_57221, &P22514_301};	/*Property = DeviceOrientation*/
const Value V22514_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22514_304 = {S24, 0, &V22514_304_57227, &P22514_302};	/*Property = FileReader*/
const Value V22514_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22514_305 = {S25, 0, &V22514_305_57231, &P22514_304};	/*Property = FileSaver*/
const Value V22514_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22514_306 = {S26, 0, &V22514_306_57237, &P22514_305};	/*Property = FileWriter*/
const Value V22514_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22514_307 = {S27, 0, &V22514_307_57239, &P22514_306};	/*Property = FormData*/
const Value V22514_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22514_309 = {S28, 0, &V22514_309_57246, &P22514_307};	/*Property = Fullscreen*/
const Value V22514_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22514_310 = {S29, 0, &V22514_310_57248, &P22514_309};	/*Property = GeoLocation*/
const Value V22514_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22514_312 = {S30, 0, &V22514_312_57254, &P22514_310};	/*Property = History*/
const Value V22514_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22514_314 = {S31, 0, &V22514_314_57261, &P22514_312};	/*Property = Html5*/
const Value V22514_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22514_313 = {S32, 0, &V22514_313_57258, &P22514_314};	/*Property = Html-Media-Capture*/
const Value V22514_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22514_315 = {S33, 0, &V22514_315_57264, &P22514_313};	/*Property = Iframe*/
const Value V22514_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22514_316 = {S34, 0, &V22514_316_57266, &P22514_315};	/*Property = IndexedDB*/
const Value V22514_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22514_317 = {S35, 0, &V22514_317_57269, &P22514_316};	/*Property = Json*/
const Value V22514_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22514_15 = {S37, 0, &V22514_15_29, &P22514_317};	/*Property = LayoutEngine*/
const Value V22514_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22514_344 = {S38, 0, &V22514_344_57272, &P22514_15};	/*Property = Masking*/
const Value V22514_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22514_320 = {S39, 0, &V22514_320_57290, &P22514_344};	/*Property = PostMessage*/
const Value V22514_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22514_321 = {S40, 0, &V22514_321_57293, &P22514_320};	/*Property = Progress*/
const Value V22514_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22514_322 = {S41, 0, &V22514_322_57296, &P22514_321};	/*Property = Prompts*/
const Value V22514_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22514_323 = {S42, 0, &V22514_323_57299, &P22514_322};	/*Property = Selector*/
const Value V22514_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22514_324 = {S43, 0, &V22514_324_57305, &P22514_323};	/*Property = Svg*/
const Value V22514_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22514_327 = {S44, 0, &V22514_327_57315, &P22514_324};	/*Property = TouchEvents*/
const Value V22514_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22514_328 = {S45, 0, &V22514_328_57318, &P22514_327};	/*Property = Track*/
const Value V22514_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22514_329 = {S46, 0, &V22514_329_57323, &P22514_328};	/*Property = Video*/
const Value V22514_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22514_330 = {S47, 0, &V22514_330_57326, &P22514_329};	/*Property = Viewport*/
const Value V22514_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22514_332 = {S48, 0, &V22514_332_57338, &P22514_330};	/*Property = WebWorkers*/
const Value V22514_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22514_333 = {S49, 0, &V22514_333_57341, &P22514_332};	/*Property = Xhr2*/
const Profile P22514 = {"22514", &P22514_333};	/*Profile ID - 22514*/
const Value V19994_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_279 = {S1, 0, &V19994_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V19994_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_282 = {S3, 0, &V19994_282_57155, &P19994_279};	/*Property = BlobBuilder*/
const Value V19994_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_350 = {S4, 0, &V19994_350_57446, &P19994_282};	/*Property = Canvas*/
const Value V19994_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_283 = {S5, 0, &V19994_283_57161, &P19994_350};	/*Property = CssBackground*/
const Value V19994_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_284 = {S6, 0, &V19994_284_57164, &P19994_283};	/*Property = CssBorderImage*/
const Value V19994_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_285 = {S7, 0, &V19994_285_57167, &P19994_284};	/*Property = CssCanvas*/
const Value V19994_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_286 = {S8, 0, &V19994_286_57170, &P19994_285};	/*Property = CssColor*/
const Value V19994_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_237 = {S9, 0, &V19994_237_55786, &P19994_286};	/*Property = CssColumn*/
const Value V19994_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_287 = {S10, 0, &V19994_287_57173, &P19994_237};	/*Property = CssFlexbox*/
const Value V19994_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_288 = {S11, 0, &V19994_288_57176, &P19994_287};	/*Property = CssFont*/
const Value V19994_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_289 = {S12, 0, &V19994_289_86899, &P19994_288};	/*Property = CssImages*/
const Value V19994_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_291 = {S13, 0, &V19994_291_57182, &P19994_289};	/*Property = CssMediaQueries*/
const Value V19994_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_292 = {S14, 0, &V19994_292_57185, &P19994_291};	/*Property = CssMinMax*/
const Value V19994_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_293 = {S15, 0, &V19994_293_57188, &P19994_292};	/*Property = CssOverflow*/
const Value V19994_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_294 = {S16, 0, &V19994_294_57191, &P19994_293};	/*Property = CssPosition*/
const Value V19994_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_295 = {S17, 0, &V19994_295_57194, &P19994_294};	/*Property = CssText*/
const Value V19994_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_297 = {S18, 0, &V19994_297_57197, &P19994_295};	/*Property = CssTransforms*/
const Value V19994_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_241 = {S19, 0, &V19994_241_55801, &P19994_297};	/*Property = CssTransitions*/
const Value V19994_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_298 = {S20, 0, &V19994_298_57208, &P19994_241};	/*Property = CssUI*/
const Value V19994_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_300 = {S21, 0, &V19994_300_57217, &P19994_298};	/*Property = DataSet*/
const Value V19994_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_301 = {S22, 0, &V19994_301_57220, &P19994_300};	/*Property = DataUrl*/
const Value V19994_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_302 = {S23, 0, &V19994_302_57223, &P19994_301};	/*Property = DeviceOrientation*/
const Value V19994_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_304 = {S24, 0, &V19994_304_57229, &P19994_302};	/*Property = FileReader*/
const Value V19994_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_305 = {S25, 0, &V19994_305_57232, &P19994_304};	/*Property = FileSaver*/
const Value V19994_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_306 = {S26, 0, &V19994_306_57238, &P19994_305};	/*Property = FileWriter*/
const Value V19994_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_307 = {S27, 0, &V19994_307_57241, &P19994_306};	/*Property = FormData*/
const Value V19994_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_309 = {S28, 0, &V19994_309_57247, &P19994_307};	/*Property = Fullscreen*/
const Value V19994_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_310 = {S29, 0, &V19994_310_57250, &P19994_309};	/*Property = GeoLocation*/
const Value V19994_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_312 = {S30, 0, &V19994_312_57256, &P19994_310};	/*Property = History*/
const Value V19994_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_314 = {S31, 0, &V19994_314_57262, &P19994_312};	/*Property = Html5*/
const Value V19994_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_313 = {S32, 0, &V19994_313_57259, &P19994_314};	/*Property = Html-Media-Capture*/
const Value V19994_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_315 = {S33, 0, &V19994_315_57265, &P19994_313};	/*Property = Iframe*/
const Value V19994_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_316 = {S34, 0, &V19994_316_57268, &P19994_315};	/*Property = IndexedDB*/
const Value V19994_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_317 = {S35, 0, &V19994_317_57271, &P19994_316};	/*Property = Json*/
const Value V19994_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P19994_15 = {S37, 0, &V19994_15_30, &P19994_317};	/*Property = LayoutEngine*/
const Value V19994_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_344 = {S38, 0, &V19994_344_57274, &P19994_15};	/*Property = Masking*/
const Value V19994_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_320 = {S39, 0, &V19994_320_57292, &P19994_344};	/*Property = PostMessage*/
const Value V19994_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_321 = {S40, 0, &V19994_321_57295, &P19994_320};	/*Property = Progress*/
const Value V19994_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_322 = {S41, 0, &V19994_322_57298, &P19994_321};	/*Property = Prompts*/
const Value V19994_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_323 = {S42, 0, &V19994_323_57301, &P19994_322};	/*Property = Selector*/
const Value V19994_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_324 = {S43, 0, &V19994_324_57307, &P19994_323};	/*Property = Svg*/
const Value V19994_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_327 = {S44, 0, &V19994_327_57316, &P19994_324};	/*Property = TouchEvents*/
const Value V19994_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_328 = {S45, 0, &V19994_328_57319, &P19994_327};	/*Property = Track*/
const Value V19994_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_329 = {S46, 0, &V19994_329_57325, &P19994_328};	/*Property = Video*/
const Value V19994_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_330 = {S47, 0, &V19994_330_57328, &P19994_329};	/*Property = Viewport*/
const Value V19994_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_332 = {S48, 0, &V19994_332_57340, &P19994_330};	/*Property = WebWorkers*/
const Value V19994_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P19994_333 = {S49, 0, &V19994_333_57343, &P19994_332};	/*Property = Xhr2*/
const Profile P19994 = {"19994", &P19994_333};	/*Profile ID - 19994*/
const Value V16886_1_1 = {S2, NULL};	/*Value = True*/
const Property P16886_1 = {S50, 0, &V16886_1_1, NULL};	/*Property = IsMobile*/
const Value V16886_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P16886_5 = {S52, 0, &V16886_5_1293, &P16886_1};	/*Property = ScreenPixelsHeight*/
const Value V16886_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P16886_4 = {S54, 0, &V16886_4_1296, &P16886_5};	/*Property = ScreenPixelsWidth*/
const Profile P16886 = {"16886", &P16886_4};	/*Profile ID - 16886*/
const Value V23659_1_2 = {S0, NULL};	/*Value = False*/
const Property P23659_1 = {S50, 0, &V23659_1_2, NULL};	/*Property = IsMobile*/
const Value V23659_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23659_5 = {S52, 0, &V23659_5_60417, &P23659_1};	/*Property = ScreenPixelsHeight*/
const Value V23659_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23659_4 = {S54, 0, &V23659_4_57095, &P23659_5};	/*Property = ScreenPixelsWidth*/
const Profile P23659 = {"23659", &P23659_4};	/*Profile ID - 23659*/
const Value V20048_1_1 = {S2, NULL};	/*Value = True*/
const Property P20048_1 = {S50, 0, &V20048_1_1, NULL};	/*Property = IsMobile*/
const Value V20048_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20048_5 = {S52, 0, &V20048_5_1222, &P20048_1};	/*Property = ScreenPixelsHeight*/
const Value V20048_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20048_4 = {S54, 0, &V20048_4_1224, &P20048_5};	/*Property = ScreenPixelsWidth*/
const Profile P20048 = {"20048", &P20048_4};	/*Profile ID - 20048*/
const Value V23057_1_1 = {S2, NULL};	/*Value = True*/
const Property P23057_1 = {S50, 0, &V23057_1_1, NULL};	/*Property = IsMobile*/
const Value V23057_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23057_5 = {S52, 0, &V23057_5_1222, &P23057_1};	/*Property = ScreenPixelsHeight*/
const Value V23057_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23057_4 = {S54, 0, &V23057_4_1224, &P23057_5};	/*Property = ScreenPixelsWidth*/
const Profile P23057 = {"23057", &P23057_4};	/*Profile ID - 23057*/
const Value V20242_1_1 = {S2, NULL};	/*Value = True*/
const Property P20242_1 = {S50, 0, &V20242_1_1, NULL};	/*Property = IsMobile*/
const Value V20242_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20242_5 = {S52, 0, &V20242_5_1227, &P20242_1};	/*Property = ScreenPixelsHeight*/
const Value V20242_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20242_4 = {S54, 0, &V20242_4_1229, &P20242_5};	/*Property = ScreenPixelsWidth*/
const Profile P20242 = {"20242", &P20242_4};	/*Profile ID - 20242*/
const Value V19847_1_1 = {S2, NULL};	/*Value = True*/
const Property P19847_1 = {S50, 0, &V19847_1_1, NULL};	/*Property = IsMobile*/
const Value V19847_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P19847_5 = {S52, 0, &V19847_5_1222, &P19847_1};	/*Property = ScreenPixelsHeight*/
const Value V19847_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P19847_4 = {S54, 0, &V19847_4_1224, &P19847_5};	/*Property = ScreenPixelsWidth*/
const Profile P19847 = {"19847", &P19847_4};	/*Profile ID - 19847*/
const Value V23597_1_2 = {S0, NULL};	/*Value = False*/
const Property P23597_1 = {S50, 0, &V23597_1_2, NULL};	/*Property = IsMobile*/
const Value V23597_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23597_5 = {S52, 0, &V23597_5_60417, &P23597_1};	/*Property = ScreenPixelsHeight*/
const Value V23597_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23597_4 = {S54, 0, &V23597_4_57095, &P23597_5};	/*Property = ScreenPixelsWidth*/
const Profile P23597 = {"23597", &P23597_4};	/*Profile ID - 23597*/
const Value V18354_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_279 = {S1, 0, &V18354_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18354_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_282 = {S3, 0, &V18354_282_57155, &P18354_279};	/*Property = BlobBuilder*/
const Value V18354_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_350 = {S4, 0, &V18354_350_57446, &P18354_282};	/*Property = Canvas*/
const Value V18354_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_283 = {S5, 0, &V18354_283_57161, &P18354_350};	/*Property = CssBackground*/
const Value V18354_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_284 = {S6, 0, &V18354_284_57164, &P18354_283};	/*Property = CssBorderImage*/
const Value V18354_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_285 = {S7, 0, &V18354_285_57167, &P18354_284};	/*Property = CssCanvas*/
const Value V18354_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_286 = {S8, 0, &V18354_286_57170, &P18354_285};	/*Property = CssColor*/
const Value V18354_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_237 = {S9, 0, &V18354_237_55786, &P18354_286};	/*Property = CssColumn*/
const Value V18354_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_287 = {S10, 0, &V18354_287_57173, &P18354_237};	/*Property = CssFlexbox*/
const Value V18354_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_288 = {S11, 0, &V18354_288_57176, &P18354_287};	/*Property = CssFont*/
const Value V18354_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_289 = {S12, 0, &V18354_289_86899, &P18354_288};	/*Property = CssImages*/
const Value V18354_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_291 = {S13, 0, &V18354_291_57182, &P18354_289};	/*Property = CssMediaQueries*/
const Value V18354_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_292 = {S14, 0, &V18354_292_57185, &P18354_291};	/*Property = CssMinMax*/
const Value V18354_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_293 = {S15, 0, &V18354_293_57188, &P18354_292};	/*Property = CssOverflow*/
const Value V18354_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_294 = {S16, 0, &V18354_294_57191, &P18354_293};	/*Property = CssPosition*/
const Value V18354_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_295 = {S17, 0, &V18354_295_57194, &P18354_294};	/*Property = CssText*/
const Value V18354_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_297 = {S18, 0, &V18354_297_57197, &P18354_295};	/*Property = CssTransforms*/
const Value V18354_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_241 = {S19, 0, &V18354_241_55801, &P18354_297};	/*Property = CssTransitions*/
const Value V18354_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_298 = {S20, 0, &V18354_298_57208, &P18354_241};	/*Property = CssUI*/
const Value V18354_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_300 = {S21, 0, &V18354_300_57217, &P18354_298};	/*Property = DataSet*/
const Value V18354_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_301 = {S22, 0, &V18354_301_57220, &P18354_300};	/*Property = DataUrl*/
const Value V18354_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_302 = {S23, 0, &V18354_302_57223, &P18354_301};	/*Property = DeviceOrientation*/
const Value V18354_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_304 = {S24, 0, &V18354_304_57229, &P18354_302};	/*Property = FileReader*/
const Value V18354_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_305 = {S25, 0, &V18354_305_57232, &P18354_304};	/*Property = FileSaver*/
const Value V18354_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_306 = {S26, 0, &V18354_306_57238, &P18354_305};	/*Property = FileWriter*/
const Value V18354_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_307 = {S27, 0, &V18354_307_57241, &P18354_306};	/*Property = FormData*/
const Value V18354_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_309 = {S28, 0, &V18354_309_57247, &P18354_307};	/*Property = Fullscreen*/
const Value V18354_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_310 = {S29, 0, &V18354_310_57250, &P18354_309};	/*Property = GeoLocation*/
const Value V18354_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_312 = {S30, 0, &V18354_312_57256, &P18354_310};	/*Property = History*/
const Value V18354_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_314 = {S31, 0, &V18354_314_57262, &P18354_312};	/*Property = Html5*/
const Value V18354_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_313 = {S32, 0, &V18354_313_57259, &P18354_314};	/*Property = Html-Media-Capture*/
const Value V18354_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_315 = {S33, 0, &V18354_315_57265, &P18354_313};	/*Property = Iframe*/
const Value V18354_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_316 = {S34, 0, &V18354_316_57268, &P18354_315};	/*Property = IndexedDB*/
const Value V18354_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_317 = {S35, 0, &V18354_317_57271, &P18354_316};	/*Property = Json*/
const Value V18354_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P18354_15 = {S37, 0, &V18354_15_30, &P18354_317};	/*Property = LayoutEngine*/
const Value V18354_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_344 = {S38, 0, &V18354_344_57274, &P18354_15};	/*Property = Masking*/
const Value V18354_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_320 = {S39, 0, &V18354_320_57292, &P18354_344};	/*Property = PostMessage*/
const Value V18354_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_321 = {S40, 0, &V18354_321_57295, &P18354_320};	/*Property = Progress*/
const Value V18354_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_322 = {S41, 0, &V18354_322_57298, &P18354_321};	/*Property = Prompts*/
const Value V18354_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_323 = {S42, 0, &V18354_323_57301, &P18354_322};	/*Property = Selector*/
const Value V18354_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_324 = {S43, 0, &V18354_324_57307, &P18354_323};	/*Property = Svg*/
const Value V18354_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_327 = {S44, 0, &V18354_327_57316, &P18354_324};	/*Property = TouchEvents*/
const Value V18354_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_328 = {S45, 0, &V18354_328_57319, &P18354_327};	/*Property = Track*/
const Value V18354_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_329 = {S46, 0, &V18354_329_57325, &P18354_328};	/*Property = Video*/
const Value V18354_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_330 = {S47, 0, &V18354_330_57328, &P18354_329};	/*Property = Viewport*/
const Value V18354_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_332 = {S48, 0, &V18354_332_57340, &P18354_330};	/*Property = WebWorkers*/
const Value V18354_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18354_333 = {S49, 0, &V18354_333_57343, &P18354_332};	/*Property = Xhr2*/
const Profile P18354 = {"18354", &P18354_333};	/*Profile ID - 18354*/
const Value V20364_1_1 = {S2, NULL};	/*Value = True*/
const Property P20364_1 = {S50, 0, &V20364_1_1, NULL};	/*Property = IsMobile*/
const Value V20364_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20364_5 = {S52, 0, &V20364_5_1458, &P20364_1};	/*Property = ScreenPixelsHeight*/
const Value V20364_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20364_4 = {S54, 0, &V20364_4_1460, &P20364_5};	/*Property = ScreenPixelsWidth*/
const Profile P20364 = {"20364", &P20364_4};	/*Profile ID - 20364*/
const Value V20236_1_1 = {S2, NULL};	/*Value = True*/
const Property P20236_1 = {S50, 0, &V20236_1_1, NULL};	/*Property = IsMobile*/
const Value V20236_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20236_5 = {S52, 0, &V20236_5_1222, &P20236_1};	/*Property = ScreenPixelsHeight*/
const Value V20236_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20236_4 = {S54, 0, &V20236_4_1224, &P20236_5};	/*Property = ScreenPixelsWidth*/
const Profile P20236 = {"20236", &P20236_4};	/*Profile ID - 20236*/
const Value V20099_1_2 = {S0, NULL};	/*Value = False*/
const Property P20099_1 = {S50, 0, &V20099_1_2, NULL};	/*Property = IsMobile*/
const Value V20099_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20099_5 = {S52, 0, &V20099_5_37, &P20099_1};	/*Property = ScreenPixelsHeight*/
const Value V20099_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20099_4 = {S54, 0, &V20099_4_36, &P20099_5};	/*Property = ScreenPixelsWidth*/
const Profile P20099 = {"20099", &P20099_4};	/*Profile ID - 20099*/
const Value V22146_1_1 = {S2, NULL};	/*Value = True*/
const Property P22146_1 = {S50, 0, &V22146_1_1, NULL};	/*Property = IsMobile*/
const Value V22146_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22146_5 = {S52, 0, &V22146_5_37569, &P22146_1};	/*Property = ScreenPixelsHeight*/
const Value V22146_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22146_4 = {S54, 0, &V22146_4_5645, &P22146_5};	/*Property = ScreenPixelsWidth*/
const Profile P22146 = {"22146", &P22146_4};	/*Profile ID - 22146*/
const Value V20292_1_2 = {S0, NULL};	/*Value = False*/
const Property P20292_1 = {S50, 0, &V20292_1_2, NULL};	/*Property = IsMobile*/
const Value V20292_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20292_5 = {S52, 0, &V20292_5_37, &P20292_1};	/*Property = ScreenPixelsHeight*/
const Value V20292_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20292_4 = {S54, 0, &V20292_4_36, &P20292_5};	/*Property = ScreenPixelsWidth*/
const Profile P20292 = {"20292", &P20292_4};	/*Profile ID - 20292*/
const Value V21041_1_1 = {S2, NULL};	/*Value = True*/
const Property P21041_1 = {S50, 0, &V21041_1_1, NULL};	/*Property = IsMobile*/
const Value V21041_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21041_5 = {S52, 0, &V21041_5_1458, &P21041_1};	/*Property = ScreenPixelsHeight*/
const Value V21041_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21041_4 = {S54, 0, &V21041_4_37617, &P21041_5};	/*Property = ScreenPixelsWidth*/
const Profile P21041 = {"21041", &P21041_4};	/*Profile ID - 21041*/
const Value V19955_1_1 = {S2, NULL};	/*Value = True*/
const Property P19955_1 = {S50, 0, &V19955_1_1, NULL};	/*Property = IsMobile*/
const Value V19955_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P19955_5 = {S52, 0, &V19955_5_1222, &P19955_1};	/*Property = ScreenPixelsHeight*/
const Value V19955_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P19955_4 = {S54, 0, &V19955_4_1224, &P19955_5};	/*Property = ScreenPixelsWidth*/
const Profile P19955 = {"19955", &P19955_4};	/*Profile ID - 19955*/
const Value V20462_1_1 = {S2, NULL};	/*Value = True*/
const Property P20462_1 = {S50, 0, &V20462_1_1, NULL};	/*Property = IsMobile*/
const Value V20462_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20462_5 = {S52, 0, &V20462_5_9981, &P20462_1};	/*Property = ScreenPixelsHeight*/
const Value V20462_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20462_4 = {S54, 0, &V20462_4_39098, &P20462_5};	/*Property = ScreenPixelsWidth*/
const Profile P20462 = {"20462", &P20462_4};	/*Profile ID - 20462*/
const Value V22089_1_1 = {S2, NULL};	/*Value = True*/
const Property P22089_1 = {S50, 0, &V22089_1_1, NULL};	/*Property = IsMobile*/
const Value V22089_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22089_5 = {S52, 0, &V22089_5_37569, &P22089_1};	/*Property = ScreenPixelsHeight*/
const Value V22089_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22089_4 = {S54, 0, &V22089_4_5645, &P22089_5};	/*Property = ScreenPixelsWidth*/
const Profile P22089 = {"22089", &P22089_4};	/*Profile ID - 22089*/
const Value V21924_1_1 = {S2, NULL};	/*Value = True*/
const Property P21924_1 = {S50, 0, &V21924_1_1, NULL};	/*Property = IsMobile*/
const Value V21924_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21924_5 = {S52, 0, &V21924_5_1277, &P21924_1};	/*Property = ScreenPixelsHeight*/
const Value V21924_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21924_4 = {S54, 0, &V21924_4_2336, &P21924_5};	/*Property = ScreenPixelsWidth*/
const Profile P21924 = {"21924", &P21924_4};	/*Profile ID - 21924*/
const Value V21875_1_1 = {S2, NULL};	/*Value = True*/
const Property P21875_1 = {S50, 0, &V21875_1_1, NULL};	/*Property = IsMobile*/
const Value V21875_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21875_5 = {S52, 0, &V21875_5_1227, &P21875_1};	/*Property = ScreenPixelsHeight*/
const Value V21875_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21875_4 = {S54, 0, &V21875_4_1229, &P21875_5};	/*Property = ScreenPixelsWidth*/
const Profile P21875 = {"21875", &P21875_4};	/*Profile ID - 21875*/
const Value V22691_1_1 = {S2, NULL};	/*Value = True*/
const Property P22691_1 = {S50, 0, &V22691_1_1, NULL};	/*Property = IsMobile*/
const Value V22691_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22691_5 = {S52, 0, &V22691_5_1458, &P22691_1};	/*Property = ScreenPixelsHeight*/
const Value V22691_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22691_4 = {S54, 0, &V22691_4_1460, &P22691_5};	/*Property = ScreenPixelsWidth*/
const Profile P22691 = {"22691", &P22691_4};	/*Profile ID - 22691*/
const Value V20982_1_1 = {S2, NULL};	/*Value = True*/
const Property P20982_1 = {S50, 0, &V20982_1_1, NULL};	/*Property = IsMobile*/
const Value V20982_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20982_5 = {S52, 0, &V20982_5_1403, &P20982_1};	/*Property = ScreenPixelsHeight*/
const Value V20982_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20982_4 = {S54, 0, &V20982_4_1404, &P20982_5};	/*Property = ScreenPixelsWidth*/
const Profile P20982 = {"20982", &P20982_4};	/*Profile ID - 20982*/
const Value V23064_1_1 = {S2, NULL};	/*Value = True*/
const Property P23064_1 = {S50, 0, &V23064_1_1, NULL};	/*Property = IsMobile*/
const Value V23064_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23064_5 = {S52, 0, &V23064_5_1403, &P23064_1};	/*Property = ScreenPixelsHeight*/
const Value V23064_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23064_4 = {S54, 0, &V23064_4_1404, &P23064_5};	/*Property = ScreenPixelsWidth*/
const Profile P23064 = {"23064", &P23064_4};	/*Profile ID - 23064*/
const Value V21843_1_1 = {S2, NULL};	/*Value = True*/
const Property P21843_1 = {S50, 0, &V21843_1_1, NULL};	/*Property = IsMobile*/
const Value V21843_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21843_5 = {S52, 0, &V21843_5_1458, &P21843_1};	/*Property = ScreenPixelsHeight*/
const Value V21843_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21843_4 = {S54, 0, &V21843_4_37617, &P21843_5};	/*Property = ScreenPixelsWidth*/
const Profile P21843 = {"21843", &P21843_4};	/*Profile ID - 21843*/
const Value V21404_1_1 = {S2, NULL};	/*Value = True*/
const Property P21404_1 = {S50, 0, &V21404_1_1, NULL};	/*Property = IsMobile*/
const Value V21404_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21404_5 = {S52, 0, &V21404_5_1227, &P21404_1};	/*Property = ScreenPixelsHeight*/
const Value V21404_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21404_4 = {S54, 0, &V21404_4_1229, &P21404_5};	/*Property = ScreenPixelsWidth*/
const Profile P21404 = {"21404", &P21404_4};	/*Profile ID - 21404*/
const Value V21136_1_1 = {S2, NULL};	/*Value = True*/
const Property P21136_1 = {S50, 0, &V21136_1_1, NULL};	/*Property = IsMobile*/
const Value V21136_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21136_5 = {S52, 0, &V21136_5_1227, &P21136_1};	/*Property = ScreenPixelsHeight*/
const Value V21136_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21136_4 = {S54, 0, &V21136_4_1229, &P21136_5};	/*Property = ScreenPixelsWidth*/
const Profile P21136 = {"21136", &P21136_4};	/*Profile ID - 21136*/
const Value V19845_1_1 = {S2, NULL};	/*Value = True*/
const Property P19845_1 = {S50, 0, &V19845_1_1, NULL};	/*Property = IsMobile*/
const Value V19845_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19845_5 = {S52, 0, &V19845_5_1227, &P19845_1};	/*Property = ScreenPixelsHeight*/
const Value V19845_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19845_4 = {S54, 0, &V19845_4_1229, &P19845_5};	/*Property = ScreenPixelsWidth*/
const Profile P19845 = {"19845", &P19845_4};	/*Profile ID - 19845*/
const Value V22650_1_1 = {S2, NULL};	/*Value = True*/
const Property P22650_1 = {S50, 0, &V22650_1_1, NULL};	/*Property = IsMobile*/
const Value V22650_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22650_5 = {S52, 0, &V22650_5_1227, &P22650_1};	/*Property = ScreenPixelsHeight*/
const Value V22650_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22650_4 = {S54, 0, &V22650_4_1229, &P22650_5};	/*Property = ScreenPixelsWidth*/
const Profile P22650 = {"22650", &P22650_4};	/*Profile ID - 22650*/
const Value V20522_1_1 = {S2, NULL};	/*Value = True*/
const Property P20522_1 = {S50, 0, &V20522_1_1, NULL};	/*Property = IsMobile*/
const Value V20522_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20522_5 = {S52, 0, &V20522_5_1227, &P20522_1};	/*Property = ScreenPixelsHeight*/
const Value V20522_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20522_4 = {S54, 0, &V20522_4_1460, &P20522_5};	/*Property = ScreenPixelsWidth*/
const Profile P20522 = {"20522", &P20522_4};	/*Profile ID - 20522*/
const Value V24027_1_1 = {S2, NULL};	/*Value = True*/
const Property P24027_1 = {S50, 0, &V24027_1_1, NULL};	/*Property = IsMobile*/
const Value V24027_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24027_5 = {S52, 0, &V24027_5_1277, &P24027_1};	/*Property = ScreenPixelsHeight*/
const Value V24027_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24027_4 = {S54, 0, &V24027_4_1279, &P24027_5};	/*Property = ScreenPixelsWidth*/
const Profile P24027 = {"24027", &P24027_4};	/*Profile ID - 24027*/
const Value V21379_1_1 = {S2, NULL};	/*Value = True*/
const Property P21379_1 = {S50, 0, &V21379_1_1, NULL};	/*Property = IsMobile*/
const Value V21379_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21379_5 = {S52, 0, &V21379_5_1403, &P21379_1};	/*Property = ScreenPixelsHeight*/
const Value V21379_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21379_4 = {S54, 0, &V21379_4_1404, &P21379_5};	/*Property = ScreenPixelsWidth*/
const Profile P21379 = {"21379", &P21379_4};	/*Profile ID - 21379*/
const Value V21547_1_1 = {S2, NULL};	/*Value = True*/
const Property P21547_1 = {S50, 0, &V21547_1_1, NULL};	/*Property = IsMobile*/
const Value V21547_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21547_5 = {S52, 0, &V21547_5_1222, &P21547_1};	/*Property = ScreenPixelsHeight*/
const Value V21547_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21547_4 = {S54, 0, &V21547_4_1224, &P21547_5};	/*Property = ScreenPixelsWidth*/
const Profile P21547 = {"21547", &P21547_4};	/*Profile ID - 21547*/
const Value V20732_1_1 = {S2, NULL};	/*Value = True*/
const Property P20732_1 = {S50, 0, &V20732_1_1, NULL};	/*Property = IsMobile*/
const Value V20732_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20732_5 = {S52, 0, &V20732_5_1403, &P20732_1};	/*Property = ScreenPixelsHeight*/
const Value V20732_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20732_4 = {S54, 0, &V20732_4_1404, &P20732_5};	/*Property = ScreenPixelsWidth*/
const Profile P20732 = {"20732", &P20732_4};	/*Profile ID - 20732*/
const Value V20482_1_2 = {S0, NULL};	/*Value = False*/
const Property P20482_1 = {S50, 0, &V20482_1_2, NULL};	/*Property = IsMobile*/
const Value V20482_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20482_5 = {S52, 0, &V20482_5_1458, &P20482_1};	/*Property = ScreenPixelsHeight*/
const Value V20482_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20482_4 = {S54, 0, &V20482_4_1460, &P20482_5};	/*Property = ScreenPixelsWidth*/
const Profile P20482 = {"20482", &P20482_4};	/*Profile ID - 20482*/
const Value V21650_1_1 = {S2, NULL};	/*Value = True*/
const Property P21650_1 = {S50, 0, &V21650_1_1, NULL};	/*Property = IsMobile*/
const Value V21650_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21650_5 = {S52, 0, &V21650_5_37, &P21650_1};	/*Property = ScreenPixelsHeight*/
const Value V21650_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21650_4 = {S54, 0, &V21650_4_36, &P21650_5};	/*Property = ScreenPixelsWidth*/
const Profile P21650 = {"21650", &P21650_4};	/*Profile ID - 21650*/
const Value V19843_1_1 = {S2, NULL};	/*Value = True*/
const Property P19843_1 = {S50, 0, &V19843_1_1, NULL};	/*Property = IsMobile*/
const Value V19843_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19843_5 = {S52, 0, &V19843_5_1227, &P19843_1};	/*Property = ScreenPixelsHeight*/
const Value V19843_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19843_4 = {S54, 0, &V19843_4_1229, &P19843_5};	/*Property = ScreenPixelsWidth*/
const Profile P19843 = {"19843", &P19843_4};	/*Profile ID - 19843*/
const Value V21975_1_1 = {S2, NULL};	/*Value = True*/
const Property P21975_1 = {S50, 0, &V21975_1_1, NULL};	/*Property = IsMobile*/
const Value V21975_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P21975_5 = {S52, 0, &V21975_5_37569, &P21975_1};	/*Property = ScreenPixelsHeight*/
const Value V21975_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21975_4 = {S54, 0, &V21975_4_5645, &P21975_5};	/*Property = ScreenPixelsWidth*/
const Profile P21975 = {"21975", &P21975_4};	/*Profile ID - 21975*/
const Value V21441_1_1 = {S2, NULL};	/*Value = True*/
const Property P21441_1 = {S50, 0, &V21441_1_1, NULL};	/*Property = IsMobile*/
const Value V21441_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21441_5 = {S52, 0, &V21441_5_1227, &P21441_1};	/*Property = ScreenPixelsHeight*/
const Value V21441_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21441_4 = {S54, 0, &V21441_4_1229, &P21441_5};	/*Property = ScreenPixelsWidth*/
const Profile P21441 = {"21441", &P21441_4};	/*Profile ID - 21441*/
const Value V19920_1_1 = {S2, NULL};	/*Value = True*/
const Property P19920_1 = {S50, 0, &V19920_1_1, NULL};	/*Property = IsMobile*/
const Value V19920_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P19920_5 = {S52, 0, &V19920_5_1403, &P19920_1};	/*Property = ScreenPixelsHeight*/
const Value V19920_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P19920_4 = {S54, 0, &V19920_4_1404, &P19920_5};	/*Property = ScreenPixelsWidth*/
const Profile P19920 = {"19920", &P19920_4};	/*Profile ID - 19920*/
const Value V20205_1_1 = {S2, NULL};	/*Value = True*/
const Property P20205_1 = {S50, 0, &V20205_1_1, NULL};	/*Property = IsMobile*/
const Value V20205_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20205_5 = {S52, 0, &V20205_5_1277, &P20205_1};	/*Property = ScreenPixelsHeight*/
const Value V20205_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20205_4 = {S54, 0, &V20205_4_1279, &P20205_5};	/*Property = ScreenPixelsWidth*/
const Profile P20205 = {"20205", &P20205_4};	/*Profile ID - 20205*/
const Value V18637_1_1 = {S2, NULL};	/*Value = True*/
const Property P18637_1 = {S50, 0, &V18637_1_1, NULL};	/*Property = IsMobile*/
const Value V18637_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18637_5 = {S52, 0, &V18637_5_9981, &P18637_1};	/*Property = ScreenPixelsHeight*/
const Value V18637_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18637_4 = {S54, 0, &V18637_4_39098, &P18637_5};	/*Property = ScreenPixelsWidth*/
const Profile P18637 = {"18637", &P18637_4};	/*Profile ID - 18637*/
const Value V23909_1_1 = {S2, NULL};	/*Value = True*/
const Property P23909_1 = {S50, 0, &V23909_1_1, NULL};	/*Property = IsMobile*/
const Value V23909_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23909_5 = {S52, 0, &V23909_5_1227, &P23909_1};	/*Property = ScreenPixelsHeight*/
const Value V23909_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P23909_4 = {S54, 0, &V23909_4_1943, &P23909_5};	/*Property = ScreenPixelsWidth*/
const Profile P23909 = {"23909", &P23909_4};	/*Profile ID - 23909*/
const Value V21537_1_1 = {S2, NULL};	/*Value = True*/
const Property P21537_1 = {S50, 0, &V21537_1_1, NULL};	/*Property = IsMobile*/
const Value V21537_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21537_5 = {S52, 0, &V21537_5_1403, &P21537_1};	/*Property = ScreenPixelsHeight*/
const Value V21537_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21537_4 = {S54, 0, &V21537_4_1404, &P21537_5};	/*Property = ScreenPixelsWidth*/
const Profile P21537 = {"21537", &P21537_4};	/*Profile ID - 21537*/
const Value V20511_1_1 = {S2, NULL};	/*Value = True*/
const Property P20511_1 = {S50, 0, &V20511_1_1, NULL};	/*Property = IsMobile*/
const Value V20511_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20511_5 = {S52, 0, &V20511_5_1277, &P20511_1};	/*Property = ScreenPixelsHeight*/
const Value V20511_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20511_4 = {S54, 0, &V20511_4_2336, &P20511_5};	/*Property = ScreenPixelsWidth*/
const Profile P20511 = {"20511", &P20511_4};	/*Profile ID - 20511*/
const Value V20521_1_1 = {S2, NULL};	/*Value = True*/
const Property P20521_1 = {S50, 0, &V20521_1_1, NULL};	/*Property = IsMobile*/
const Value V20521_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20521_5 = {S52, 0, &V20521_5_1458, &P20521_1};	/*Property = ScreenPixelsHeight*/
const Value V20521_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20521_4 = {S54, 0, &V20521_4_37617, &P20521_5};	/*Property = ScreenPixelsWidth*/
const Profile P20521 = {"20521", &P20521_4};	/*Profile ID - 20521*/
const Value V20060_1_1 = {S2, NULL};	/*Value = True*/
const Property P20060_1 = {S50, 0, &V20060_1_1, NULL};	/*Property = IsMobile*/
const Value V20060_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P20060_5 = {S52, 0, &V20060_5_1233, &P20060_1};	/*Property = ScreenPixelsHeight*/
const Value V20060_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P20060_4 = {S54, 0, &V20060_4_1235, &P20060_5};	/*Property = ScreenPixelsWidth*/
const Profile P20060 = {"20060", &P20060_4};	/*Profile ID - 20060*/
const Value V23501_1_1 = {S2, NULL};	/*Value = True*/
const Property P23501_1 = {S50, 0, &V23501_1_1, NULL};	/*Property = IsMobile*/
const Value V23501_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23501_5 = {S52, 0, &V23501_5_1277, &P23501_1};	/*Property = ScreenPixelsHeight*/
const Value V23501_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23501_4 = {S54, 0, &V23501_4_1279, &P23501_5};	/*Property = ScreenPixelsWidth*/
const Profile P23501 = {"23501", &P23501_4};	/*Profile ID - 23501*/
const Value V20246_1_1 = {S2, NULL};	/*Value = True*/
const Property P20246_1 = {S50, 0, &V20246_1_1, NULL};	/*Property = IsMobile*/
const Value V20246_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20246_5 = {S52, 0, &V20246_5_1227, &P20246_1};	/*Property = ScreenPixelsHeight*/
const Value V20246_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20246_4 = {S54, 0, &V20246_4_1229, &P20246_5};	/*Property = ScreenPixelsWidth*/
const Profile P20246 = {"20246", &P20246_4};	/*Profile ID - 20246*/
const Value V20150_1_1 = {S2, NULL};	/*Value = True*/
const Property P20150_1 = {S50, 0, &V20150_1_1, NULL};	/*Property = IsMobile*/
const Value V20150_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20150_5 = {S52, 0, &V20150_5_1227, &P20150_1};	/*Property = ScreenPixelsHeight*/
const Value V20150_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P20150_4 = {S54, 0, &V20150_4_1943, &P20150_5};	/*Property = ScreenPixelsWidth*/
const Profile P20150 = {"20150", &P20150_4};	/*Profile ID - 20150*/
const Value V20102_1_2 = {S0, NULL};	/*Value = False*/
const Property P20102_1 = {S50, 0, &V20102_1_2, NULL};	/*Property = IsMobile*/
const Value V20102_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20102_5 = {S52, 0, &V20102_5_37, &P20102_1};	/*Property = ScreenPixelsHeight*/
const Value V20102_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20102_4 = {S54, 0, &V20102_4_36, &P20102_5};	/*Property = ScreenPixelsWidth*/
const Profile P20102 = {"20102", &P20102_4};	/*Profile ID - 20102*/
const Value V20469_1_1 = {S2, NULL};	/*Value = True*/
const Property P20469_1 = {S50, 0, &V20469_1_1, NULL};	/*Property = IsMobile*/
const Value V20469_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20469_5 = {S52, 0, &V20469_5_1222, &P20469_1};	/*Property = ScreenPixelsHeight*/
const Value V20469_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20469_4 = {S54, 0, &V20469_4_1224, &P20469_5};	/*Property = ScreenPixelsWidth*/
const Profile P20469 = {"20469", &P20469_4};	/*Profile ID - 20469*/
const Value V20830_1_1 = {S2, NULL};	/*Value = True*/
const Property P20830_1 = {S50, 0, &V20830_1_1, NULL};	/*Property = IsMobile*/
const Value V20830_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20830_5 = {S52, 0, &V20830_5_37, &P20830_1};	/*Property = ScreenPixelsHeight*/
const Value V20830_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20830_4 = {S54, 0, &V20830_4_36, &P20830_5};	/*Property = ScreenPixelsWidth*/
const Profile P20830 = {"20830", &P20830_4};	/*Profile ID - 20830*/
const Value V23872_1_1 = {S2, NULL};	/*Value = True*/
const Property P23872_1 = {S50, 0, &V23872_1_1, NULL};	/*Property = IsMobile*/
const Value V23872_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23872_5 = {S52, 0, &V23872_5_1277, &P23872_1};	/*Property = ScreenPixelsHeight*/
const Value V23872_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23872_4 = {S54, 0, &V23872_4_1279, &P23872_5};	/*Property = ScreenPixelsWidth*/
const Profile P23872 = {"23872", &P23872_4};	/*Profile ID - 23872*/
const Value V20524_1_1 = {S2, NULL};	/*Value = True*/
const Property P20524_1 = {S50, 0, &V20524_1_1, NULL};	/*Property = IsMobile*/
const Value V20524_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20524_5 = {S52, 0, &V20524_5_37, &P20524_1};	/*Property = ScreenPixelsHeight*/
const Value V20524_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20524_4 = {S54, 0, &V20524_4_36, &P20524_5};	/*Property = ScreenPixelsWidth*/
const Profile P20524 = {"20524", &P20524_4};	/*Profile ID - 20524*/
const Value V20234_1_1 = {S2, NULL};	/*Value = True*/
const Property P20234_1 = {S50, 0, &V20234_1_1, NULL};	/*Property = IsMobile*/
const Value V20234_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20234_5 = {S52, 0, &V20234_5_1458, &P20234_1};	/*Property = ScreenPixelsHeight*/
const Value V20234_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20234_4 = {S54, 0, &V20234_4_37617, &P20234_5};	/*Property = ScreenPixelsWidth*/
const Profile P20234 = {"20234", &P20234_4};	/*Profile ID - 20234*/
const Value V20826_1_1 = {S2, NULL};	/*Value = True*/
const Property P20826_1 = {S50, 0, &V20826_1_1, NULL};	/*Property = IsMobile*/
const Value V20826_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20826_5 = {S52, 0, &V20826_5_1227, &P20826_1};	/*Property = ScreenPixelsHeight*/
const Value V20826_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20826_4 = {S54, 0, &V20826_4_1229, &P20826_5};	/*Property = ScreenPixelsWidth*/
const Profile P20826 = {"20826", &P20826_4};	/*Profile ID - 20826*/
const Value V22125_1_1 = {S2, NULL};	/*Value = True*/
const Property P22125_1 = {S50, 0, &V22125_1_1, NULL};	/*Property = IsMobile*/
const Value V22125_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22125_5 = {S52, 0, &V22125_5_1227, &P22125_1};	/*Property = ScreenPixelsHeight*/
const Value V22125_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22125_4 = {S54, 0, &V22125_4_1229, &P22125_5};	/*Property = ScreenPixelsWidth*/
const Profile P22125 = {"22125", &P22125_4};	/*Profile ID - 22125*/
const Value V20266_1_1 = {S2, NULL};	/*Value = True*/
const Property P20266_1 = {S50, 0, &V20266_1_1, NULL};	/*Property = IsMobile*/
const Value V20266_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20266_5 = {S52, 0, &V20266_5_1277, &P20266_1};	/*Property = ScreenPixelsHeight*/
const Value V20266_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20266_4 = {S54, 0, &V20266_4_1279, &P20266_5};	/*Property = ScreenPixelsWidth*/
const Profile P20266 = {"20266", &P20266_4};	/*Profile ID - 20266*/
const Value V23925_1_1 = {S2, NULL};	/*Value = True*/
const Property P23925_1 = {S50, 0, &V23925_1_1, NULL};	/*Property = IsMobile*/
const Value V23925_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23925_5 = {S52, 0, &V23925_5_1227, &P23925_1};	/*Property = ScreenPixelsHeight*/
const Value V23925_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23925_4 = {S54, 0, &V23925_4_1229, &P23925_5};	/*Property = ScreenPixelsWidth*/
const Profile P23925 = {"23925", &P23925_4};	/*Profile ID - 23925*/
const Value V23928_1_1 = {S2, NULL};	/*Value = True*/
const Property P23928_1 = {S50, 0, &V23928_1_1, NULL};	/*Property = IsMobile*/
const Value V23928_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23928_5 = {S52, 0, &V23928_5_1227, &P23928_1};	/*Property = ScreenPixelsHeight*/
const Value V23928_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23928_4 = {S54, 0, &V23928_4_1229, &P23928_5};	/*Property = ScreenPixelsWidth*/
const Profile P23928 = {"23928", &P23928_4};	/*Profile ID - 23928*/
const Value V20956_1_2 = {S0, NULL};	/*Value = False*/
const Property P20956_1 = {S50, 0, &V20956_1_2, NULL};	/*Property = IsMobile*/
const Value V20956_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20956_5 = {S52, 0, &V20956_5_37, &P20956_1};	/*Property = ScreenPixelsHeight*/
const Value V20956_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20956_4 = {S54, 0, &V20956_4_36, &P20956_5};	/*Property = ScreenPixelsWidth*/
const Profile P20956 = {"20956", &P20956_4};	/*Profile ID - 20956*/
const Value V20081_1_2 = {S0, NULL};	/*Value = False*/
const Property P20081_1 = {S50, 0, &V20081_1_2, NULL};	/*Property = IsMobile*/
const Value V20081_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20081_5 = {S52, 0, &V20081_5_37, &P20081_1};	/*Property = ScreenPixelsHeight*/
const Value V20081_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20081_4 = {S54, 0, &V20081_4_36, &P20081_5};	/*Property = ScreenPixelsWidth*/
const Profile P20081 = {"20081", &P20081_4};	/*Profile ID - 20081*/
const Value V23020_1_1 = {S2, NULL};	/*Value = True*/
const Property P23020_1 = {S50, 0, &V23020_1_1, NULL};	/*Property = IsMobile*/
const Value V23020_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23020_5 = {S52, 0, &V23020_5_1222, &P23020_1};	/*Property = ScreenPixelsHeight*/
const Value V23020_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23020_4 = {S54, 0, &V23020_4_1224, &P23020_5};	/*Property = ScreenPixelsWidth*/
const Profile P23020 = {"23020", &P23020_4};	/*Profile ID - 23020*/
const Value V20153_1_1 = {S2, NULL};	/*Value = True*/
const Property P20153_1 = {S50, 0, &V20153_1_1, NULL};	/*Property = IsMobile*/
const Value V20153_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20153_5 = {S52, 0, &V20153_5_1227, &P20153_1};	/*Property = ScreenPixelsHeight*/
const Value V20153_4_63403 = {S143, NULL};	/*Value = 40*/
const Property P20153_4 = {S54, 0, &V20153_4_63403, &P20153_5};	/*Property = ScreenPixelsWidth*/
const Profile P20153 = {"20153", &P20153_4};	/*Profile ID - 20153*/
const Value V20253_1_1 = {S2, NULL};	/*Value = True*/
const Property P20253_1 = {S50, 0, &V20253_1_1, NULL};	/*Property = IsMobile*/
const Value V20253_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P20253_5 = {S52, 0, &V20253_5_1269, &P20253_1};	/*Property = ScreenPixelsHeight*/
const Value V20253_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20253_4 = {S54, 0, &V20253_4_1229, &P20253_5};	/*Property = ScreenPixelsWidth*/
const Profile P20253 = {"20253", &P20253_4};	/*Profile ID - 20253*/
const Value V20862_1_1 = {S2, NULL};	/*Value = True*/
const Property P20862_1 = {S50, 0, &V20862_1_1, NULL};	/*Property = IsMobile*/
const Value V20862_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20862_5 = {S52, 0, &V20862_5_9981, &P20862_1};	/*Property = ScreenPixelsHeight*/
const Value V20862_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20862_4 = {S54, 0, &V20862_4_39098, &P20862_5};	/*Property = ScreenPixelsWidth*/
const Profile P20862 = {"20862", &P20862_4};	/*Profile ID - 20862*/
const Value V22114_1_1 = {S2, NULL};	/*Value = True*/
const Property P22114_1 = {S50, 0, &V22114_1_1, NULL};	/*Property = IsMobile*/
const Value V22114_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22114_5 = {S52, 0, &V22114_5_1222, &P22114_1};	/*Property = ScreenPixelsHeight*/
const Value V22114_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22114_4 = {S54, 0, &V22114_4_1224, &P22114_5};	/*Property = ScreenPixelsWidth*/
const Profile P22114 = {"22114", &P22114_4};	/*Profile ID - 22114*/
const Value V23772_1_1 = {S2, NULL};	/*Value = True*/
const Property P23772_1 = {S50, 0, &V23772_1_1, NULL};	/*Property = IsMobile*/
const Value V23772_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23772_5 = {S52, 0, &V23772_5_1227, &P23772_1};	/*Property = ScreenPixelsHeight*/
const Value V23772_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23772_4 = {S54, 0, &V23772_4_1229, &P23772_5};	/*Property = ScreenPixelsWidth*/
const Profile P23772 = {"23772", &P23772_4};	/*Profile ID - 23772*/
const Value V20131_1_1 = {S2, NULL};	/*Value = True*/
const Property P20131_1 = {S50, 0, &V20131_1_1, NULL};	/*Property = IsMobile*/
const Value V20131_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20131_5 = {S52, 0, &V20131_5_1403, &P20131_1};	/*Property = ScreenPixelsHeight*/
const Value V20131_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20131_4 = {S54, 0, &V20131_4_1404, &P20131_5};	/*Property = ScreenPixelsWidth*/
const Profile P20131 = {"20131", &P20131_4};	/*Profile ID - 20131*/
const Value V21360_1_1 = {S2, NULL};	/*Value = True*/
const Property P21360_1 = {S50, 0, &V21360_1_1, NULL};	/*Property = IsMobile*/
const Value V21360_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21360_5 = {S52, 0, &V21360_5_1227, &P21360_1};	/*Property = ScreenPixelsHeight*/
const Value V21360_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21360_4 = {S54, 0, &V21360_4_1229, &P21360_5};	/*Property = ScreenPixelsWidth*/
const Profile P21360 = {"21360", &P21360_4};	/*Profile ID - 21360*/
const Value V21280_1_1 = {S2, NULL};	/*Value = True*/
const Property P21280_1 = {S50, 0, &V21280_1_1, NULL};	/*Property = IsMobile*/
const Value V21280_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21280_5 = {S52, 0, &V21280_5_1458, &P21280_1};	/*Property = ScreenPixelsHeight*/
const Value V21280_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21280_4 = {S54, 0, &V21280_4_37617, &P21280_5};	/*Property = ScreenPixelsWidth*/
const Profile P21280 = {"21280", &P21280_4};	/*Profile ID - 21280*/
const Value V23847_1_1 = {S2, NULL};	/*Value = True*/
const Property P23847_1 = {S50, 0, &V23847_1_1, NULL};	/*Property = IsMobile*/
const Value V23847_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23847_5 = {S52, 0, &V23847_5_1458, &P23847_1};	/*Property = ScreenPixelsHeight*/
const Value V23847_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23847_4 = {S54, 0, &V23847_4_37617, &P23847_5};	/*Property = ScreenPixelsWidth*/
const Profile P23847 = {"23847", &P23847_4};	/*Profile ID - 23847*/
const Value V20106_1_1 = {S2, NULL};	/*Value = True*/
const Property P20106_1 = {S50, 0, &V20106_1_1, NULL};	/*Property = IsMobile*/
const Value V20106_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20106_5 = {S52, 0, &V20106_5_1277, &P20106_1};	/*Property = ScreenPixelsHeight*/
const Value V20106_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20106_4 = {S54, 0, &V20106_4_1279, &P20106_5};	/*Property = ScreenPixelsWidth*/
const Profile P20106 = {"20106", &P20106_4};	/*Profile ID - 20106*/
const Value V19853_1_1 = {S2, NULL};	/*Value = True*/
const Property P19853_1 = {S50, 0, &V19853_1_1, NULL};	/*Property = IsMobile*/
const Value V19853_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19853_5 = {S52, 0, &V19853_5_9981, &P19853_1};	/*Property = ScreenPixelsHeight*/
const Value V19853_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P19853_4 = {S54, 0, &V19853_4_39098, &P19853_5};	/*Property = ScreenPixelsWidth*/
const Profile P19853 = {"19853", &P19853_4};	/*Profile ID - 19853*/
const Value V24551_1_1 = {S2, NULL};	/*Value = True*/
const Property P24551_1 = {S50, 0, &V24551_1_1, NULL};	/*Property = IsMobile*/
const Value V24551_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24551_5 = {S52, 0, &V24551_5_1277, &P24551_1};	/*Property = ScreenPixelsHeight*/
const Value V24551_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24551_4 = {S54, 0, &V24551_4_1279, &P24551_5};	/*Property = ScreenPixelsWidth*/
const Profile P24551 = {"24551", &P24551_4};	/*Profile ID - 24551*/
const Value V19973_1_1 = {S2, NULL};	/*Value = True*/
const Property P19973_1 = {S50, 0, &V19973_1_1, NULL};	/*Property = IsMobile*/
const Value V19973_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P19973_5 = {S52, 0, &V19973_5_1293, &P19973_1};	/*Property = ScreenPixelsHeight*/
const Value V19973_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P19973_4 = {S54, 0, &V19973_4_1296, &P19973_5};	/*Property = ScreenPixelsWidth*/
const Profile P19973 = {"19973", &P19973_4};	/*Profile ID - 19973*/
const Value V20241_1_1 = {S2, NULL};	/*Value = True*/
const Property P20241_1 = {S50, 0, &V20241_1_1, NULL};	/*Property = IsMobile*/
const Value V20241_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20241_5 = {S52, 0, &V20241_5_1458, &P20241_1};	/*Property = ScreenPixelsHeight*/
const Value V20241_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20241_4 = {S54, 0, &V20241_4_1460, &P20241_5};	/*Property = ScreenPixelsWidth*/
const Profile P20241 = {"20241", &P20241_4};	/*Profile ID - 20241*/
const Value V20843_1_1 = {S2, NULL};	/*Value = True*/
const Property P20843_1 = {S50, 0, &V20843_1_1, NULL};	/*Property = IsMobile*/
const Value V20843_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20843_5 = {S52, 0, &V20843_5_1227, &P20843_1};	/*Property = ScreenPixelsHeight*/
const Value V20843_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20843_4 = {S54, 0, &V20843_4_1229, &P20843_5};	/*Property = ScreenPixelsWidth*/
const Profile P20843 = {"20843", &P20843_4};	/*Profile ID - 20843*/
const Value V20190_1_1 = {S2, NULL};	/*Value = True*/
const Property P20190_1 = {S50, 0, &V20190_1_1, NULL};	/*Property = IsMobile*/
const Value V20190_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20190_5 = {S52, 0, &V20190_5_9981, &P20190_1};	/*Property = ScreenPixelsHeight*/
const Value V20190_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20190_4 = {S54, 0, &V20190_4_37617, &P20190_5};	/*Property = ScreenPixelsWidth*/
const Profile P20190 = {"20190", &P20190_4};	/*Profile ID - 20190*/
const Value V20821_1_1 = {S2, NULL};	/*Value = True*/
const Property P20821_1 = {S50, 0, &V20821_1_1, NULL};	/*Property = IsMobile*/
const Value V20821_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20821_5 = {S52, 0, &V20821_5_1227, &P20821_1};	/*Property = ScreenPixelsHeight*/
const Value V20821_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20821_4 = {S54, 0, &V20821_4_1229, &P20821_5};	/*Property = ScreenPixelsWidth*/
const Profile P20821 = {"20821", &P20821_4};	/*Profile ID - 20821*/
const Value V21657_1_1 = {S2, NULL};	/*Value = True*/
const Property P21657_1 = {S50, 0, &V21657_1_1, NULL};	/*Property = IsMobile*/
const Value V21657_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21657_5 = {S52, 0, &V21657_5_1403, &P21657_1};	/*Property = ScreenPixelsHeight*/
const Value V21657_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21657_4 = {S54, 0, &V21657_4_1404, &P21657_5};	/*Property = ScreenPixelsWidth*/
const Profile P21657 = {"21657", &P21657_4};	/*Profile ID - 21657*/
const Value V20117_1_1 = {S2, NULL};	/*Value = True*/
const Property P20117_1 = {S50, 0, &V20117_1_1, NULL};	/*Property = IsMobile*/
const Value V20117_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20117_5 = {S52, 0, &V20117_5_1227, &P20117_1};	/*Property = ScreenPixelsHeight*/
const Value V20117_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20117_4 = {S54, 0, &V20117_4_1229, &P20117_5};	/*Property = ScreenPixelsWidth*/
const Profile P20117 = {"20117", &P20117_4};	/*Profile ID - 20117*/
const Value V20397_1_2 = {S0, NULL};	/*Value = False*/
const Property P20397_1 = {S50, 0, &V20397_1_2, NULL};	/*Property = IsMobile*/
const Value V20397_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20397_5 = {S52, 0, &V20397_5_37, &P20397_1};	/*Property = ScreenPixelsHeight*/
const Value V20397_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20397_4 = {S54, 0, &V20397_4_36, &P20397_5};	/*Property = ScreenPixelsWidth*/
const Profile P20397 = {"20397", &P20397_4};	/*Profile ID - 20397*/
const Value V23891_1_1 = {S2, NULL};	/*Value = True*/
const Property P23891_1 = {S50, 0, &V23891_1_1, NULL};	/*Property = IsMobile*/
const Value V23891_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23891_5 = {S52, 0, &V23891_5_9981, &P23891_1};	/*Property = ScreenPixelsHeight*/
const Value V23891_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23891_4 = {S54, 0, &V23891_4_2336, &P23891_5};	/*Property = ScreenPixelsWidth*/
const Profile P23891 = {"23891", &P23891_4};	/*Profile ID - 23891*/
const Value V23912_1_1 = {S2, NULL};	/*Value = True*/
const Property P23912_1 = {S50, 0, &V23912_1_1, NULL};	/*Property = IsMobile*/
const Value V23912_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23912_5 = {S52, 0, &V23912_5_1458, &P23912_1};	/*Property = ScreenPixelsHeight*/
const Value V23912_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23912_4 = {S54, 0, &V23912_4_37617, &P23912_5};	/*Property = ScreenPixelsWidth*/
const Profile P23912 = {"23912", &P23912_4};	/*Profile ID - 23912*/
const Profile P24698 = {"24698", NULL};	/*Profile ID - 24698*/
const Value V20875_1_1 = {S2, NULL};	/*Value = True*/
const Property P20875_1 = {S50, 0, &V20875_1_1, NULL};	/*Property = IsMobile*/
const Value V20875_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20875_5 = {S52, 0, &V20875_5_1458, &P20875_1};	/*Property = ScreenPixelsHeight*/
const Value V20875_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20875_4 = {S54, 0, &V20875_4_37617, &P20875_5};	/*Property = ScreenPixelsWidth*/
const Profile P20875 = {"20875", &P20875_4};	/*Profile ID - 20875*/
const Value V23852_1_1 = {S2, NULL};	/*Value = True*/
const Property P23852_1 = {S50, 0, &V23852_1_1, NULL};	/*Property = IsMobile*/
const Value V23852_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23852_5 = {S52, 0, &V23852_5_1458, &P23852_1};	/*Property = ScreenPixelsHeight*/
const Value V23852_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23852_4 = {S54, 0, &V23852_4_37617, &P23852_5};	/*Property = ScreenPixelsWidth*/
const Profile P23852 = {"23852", &P23852_4};	/*Profile ID - 23852*/
const Value V20270_1_1 = {S2, NULL};	/*Value = True*/
const Property P20270_1 = {S50, 0, &V20270_1_1, NULL};	/*Property = IsMobile*/
const Value V20270_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20270_5 = {S52, 0, &V20270_5_1458, &P20270_1};	/*Property = ScreenPixelsHeight*/
const Value V20270_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20270_4 = {S54, 0, &V20270_4_37617, &P20270_5};	/*Property = ScreenPixelsWidth*/
const Profile P20270 = {"20270", &P20270_4};	/*Profile ID - 20270*/
const Value V20228_1_1 = {S2, NULL};	/*Value = True*/
const Property P20228_1 = {S50, 0, &V20228_1_1, NULL};	/*Property = IsMobile*/
const Value V20228_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20228_5 = {S52, 0, &V20228_5_1227, &P20228_1};	/*Property = ScreenPixelsHeight*/
const Value V20228_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20228_4 = {S54, 0, &V20228_4_1229, &P20228_5};	/*Property = ScreenPixelsWidth*/
const Profile P20228 = {"20228", &P20228_4};	/*Profile ID - 20228*/
const Value V20849_1_1 = {S2, NULL};	/*Value = True*/
const Property P20849_1 = {S50, 0, &V20849_1_1, NULL};	/*Property = IsMobile*/
const Value V20849_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20849_5 = {S52, 0, &V20849_5_1227, &P20849_1};	/*Property = ScreenPixelsHeight*/
const Value V20849_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20849_4 = {S54, 0, &V20849_4_1460, &P20849_5};	/*Property = ScreenPixelsWidth*/
const Profile P20849 = {"20849", &P20849_4};	/*Profile ID - 20849*/
const Value V23725_1_1 = {S2, NULL};	/*Value = True*/
const Property P23725_1 = {S50, 0, &V23725_1_1, NULL};	/*Property = IsMobile*/
const Value V23725_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23725_5 = {S52, 0, &V23725_5_1227, &P23725_1};	/*Property = ScreenPixelsHeight*/
const Value V23725_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23725_4 = {S54, 0, &V23725_4_1229, &P23725_5};	/*Property = ScreenPixelsWidth*/
const Profile P23725 = {"23725", &P23725_4};	/*Profile ID - 23725*/
const Value V20233_1_2 = {S0, NULL};	/*Value = False*/
const Property P20233_1 = {S50, 0, &V20233_1_2, NULL};	/*Property = IsMobile*/
const Value V20233_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P20233_5 = {S52, 0, &V20233_5_59138, &P20233_1};	/*Property = ScreenPixelsHeight*/
const Value V20233_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P20233_4 = {S54, 0, &V20233_4_59970, &P20233_5};	/*Property = ScreenPixelsWidth*/
const Profile P20233 = {"20233", &P20233_4};	/*Profile ID - 20233*/
const Value V21271_1_1 = {S2, NULL};	/*Value = True*/
const Property P21271_1 = {S50, 0, &V21271_1_1, NULL};	/*Property = IsMobile*/
const Value V21271_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21271_5 = {S52, 0, &V21271_5_1227, &P21271_1};	/*Property = ScreenPixelsHeight*/
const Value V21271_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21271_4 = {S54, 0, &V21271_4_1229, &P21271_5};	/*Property = ScreenPixelsWidth*/
const Profile P21271 = {"21271", &P21271_4};	/*Profile ID - 21271*/
const Value V20752_1_1 = {S2, NULL};	/*Value = True*/
const Property P20752_1 = {S50, 0, &V20752_1_1, NULL};	/*Property = IsMobile*/
const Value V20752_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20752_5 = {S52, 0, &V20752_5_1458, &P20752_1};	/*Property = ScreenPixelsHeight*/
const Value V20752_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20752_4 = {S54, 0, &V20752_4_1460, &P20752_5};	/*Property = ScreenPixelsWidth*/
const Profile P20752 = {"20752", &P20752_4};	/*Profile ID - 20752*/
const Value V20426_1_1 = {S2, NULL};	/*Value = True*/
const Property P20426_1 = {S50, 0, &V20426_1_1, NULL};	/*Property = IsMobile*/
const Value V20426_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20426_5 = {S52, 0, &V20426_5_1277, &P20426_1};	/*Property = ScreenPixelsHeight*/
const Value V20426_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20426_4 = {S54, 0, &V20426_4_1279, &P20426_5};	/*Property = ScreenPixelsWidth*/
const Profile P20426 = {"20426", &P20426_4};	/*Profile ID - 20426*/
const Value V18736_1_1 = {S2, NULL};	/*Value = True*/
const Property P18736_1 = {S50, 0, &V18736_1_1, NULL};	/*Property = IsMobile*/
const Value V18736_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18736_5 = {S52, 0, &V18736_5_1227, &P18736_1};	/*Property = ScreenPixelsHeight*/
const Value V18736_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18736_4 = {S54, 0, &V18736_4_1229, &P18736_5};	/*Property = ScreenPixelsWidth*/
const Profile P18736 = {"18736", &P18736_4};	/*Profile ID - 18736*/
const Value V19844_1_1 = {S2, NULL};	/*Value = True*/
const Property P19844_1 = {S50, 0, &V19844_1_1, NULL};	/*Property = IsMobile*/
const Value V19844_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P19844_5 = {S52, 0, &V19844_5_1458, &P19844_1};	/*Property = ScreenPixelsHeight*/
const Value V19844_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P19844_4 = {S54, 0, &V19844_4_37617, &P19844_5};	/*Property = ScreenPixelsWidth*/
const Profile P19844 = {"19844", &P19844_4};	/*Profile ID - 19844*/
const Value V23637_1_2 = {S0, NULL};	/*Value = False*/
const Property P23637_1 = {S50, 0, &V23637_1_2, NULL};	/*Property = IsMobile*/
const Value V23637_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23637_5 = {S52, 0, &V23637_5_37, &P23637_1};	/*Property = ScreenPixelsHeight*/
const Value V23637_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23637_4 = {S54, 0, &V23637_4_36, &P23637_5};	/*Property = ScreenPixelsWidth*/
const Profile P23637 = {"23637", &P23637_4};	/*Profile ID - 23637*/
const Value V21289_1_1 = {S2, NULL};	/*Value = True*/
const Property P21289_1 = {S50, 0, &V21289_1_1, NULL};	/*Property = IsMobile*/
const Value V21289_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P21289_5 = {S52, 0, &V21289_5_37569, &P21289_1};	/*Property = ScreenPixelsHeight*/
const Value V21289_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21289_4 = {S54, 0, &V21289_4_5645, &P21289_5};	/*Property = ScreenPixelsWidth*/
const Profile P21289 = {"21289", &P21289_4};	/*Profile ID - 21289*/
const Value V20544_1_1 = {S2, NULL};	/*Value = True*/
const Property P20544_1 = {S50, 0, &V20544_1_1, NULL};	/*Property = IsMobile*/
const Value V20544_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20544_5 = {S52, 0, &V20544_5_1458, &P20544_1};	/*Property = ScreenPixelsHeight*/
const Value V20544_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20544_4 = {S54, 0, &V20544_4_1460, &P20544_5};	/*Property = ScreenPixelsWidth*/
const Profile P20544 = {"20544", &P20544_4};	/*Profile ID - 20544*/
const Value V24012_1_1 = {S2, NULL};	/*Value = True*/
const Property P24012_1 = {S50, 0, &V24012_1_1, NULL};	/*Property = IsMobile*/
const Value V24012_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24012_5 = {S52, 0, &V24012_5_1277, &P24012_1};	/*Property = ScreenPixelsHeight*/
const Value V24012_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P24012_4 = {S54, 0, &V24012_4_2336, &P24012_5};	/*Property = ScreenPixelsWidth*/
const Profile P24012 = {"24012", &P24012_4};	/*Profile ID - 24012*/
const Value V23913_1_1 = {S2, NULL};	/*Value = True*/
const Property P23913_1 = {S50, 0, &V23913_1_1, NULL};	/*Property = IsMobile*/
const Value V23913_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23913_5 = {S52, 0, &V23913_5_1403, &P23913_1};	/*Property = ScreenPixelsHeight*/
const Value V23913_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23913_4 = {S54, 0, &V23913_4_1404, &P23913_5};	/*Property = ScreenPixelsWidth*/
const Profile P23913 = {"23913", &P23913_4};	/*Profile ID - 23913*/
const Value V22688_1_1 = {S2, NULL};	/*Value = True*/
const Property P22688_1 = {S50, 0, &V22688_1_1, NULL};	/*Property = IsMobile*/
const Value V22688_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22688_5 = {S52, 0, &V22688_5_1227, &P22688_1};	/*Property = ScreenPixelsHeight*/
const Value V22688_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22688_4 = {S54, 0, &V22688_4_1229, &P22688_5};	/*Property = ScreenPixelsWidth*/
const Profile P22688 = {"22688", &P22688_4};	/*Profile ID - 22688*/
const Value V20733_1_1 = {S2, NULL};	/*Value = True*/
const Property P20733_1 = {S50, 0, &V20733_1_1, NULL};	/*Property = IsMobile*/
const Value V20733_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20733_5 = {S52, 0, &V20733_5_1458, &P20733_1};	/*Property = ScreenPixelsHeight*/
const Value V20733_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20733_4 = {S54, 0, &V20733_4_1460, &P20733_5};	/*Property = ScreenPixelsWidth*/
const Profile P20733 = {"20733", &P20733_4};	/*Profile ID - 20733*/
const Value V20171_1_1 = {S2, NULL};	/*Value = True*/
const Property P20171_1 = {S50, 0, &V20171_1_1, NULL};	/*Property = IsMobile*/
const Value V20171_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20171_5 = {S52, 0, &V20171_5_1458, &P20171_1};	/*Property = ScreenPixelsHeight*/
const Value V20171_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20171_4 = {S54, 0, &V20171_4_1460, &P20171_5};	/*Property = ScreenPixelsWidth*/
const Profile P20171 = {"20171", &P20171_4};	/*Profile ID - 20171*/
const Value V19863_1_1 = {S2, NULL};	/*Value = True*/
const Property P19863_1 = {S50, 0, &V19863_1_1, NULL};	/*Property = IsMobile*/
const Value V19863_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19863_5 = {S52, 0, &V19863_5_9981, &P19863_1};	/*Property = ScreenPixelsHeight*/
const Value V19863_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P19863_4 = {S54, 0, &V19863_4_39098, &P19863_5};	/*Property = ScreenPixelsWidth*/
const Profile P19863 = {"19863", &P19863_4};	/*Profile ID - 19863*/
const Value V23088_1_1 = {S2, NULL};	/*Value = True*/
const Property P23088_1 = {S50, 0, &V23088_1_1, NULL};	/*Property = IsMobile*/
const Value V23088_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23088_5 = {S52, 0, &V23088_5_1222, &P23088_1};	/*Property = ScreenPixelsHeight*/
const Value V23088_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23088_4 = {S54, 0, &V23088_4_1224, &P23088_5};	/*Property = ScreenPixelsWidth*/
const Profile P23088 = {"23088", &P23088_4};	/*Profile ID - 23088*/
const Value V20057_1_1 = {S2, NULL};	/*Value = True*/
const Property P20057_1 = {S50, 0, &V20057_1_1, NULL};	/*Property = IsMobile*/
const Value V20057_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P20057_5 = {S52, 0, &V20057_5_1275, &P20057_1};	/*Property = ScreenPixelsHeight*/
const Value V20057_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P20057_4 = {S54, 0, &V20057_4_1278, &P20057_5};	/*Property = ScreenPixelsWidth*/
const Profile P20057 = {"20057", &P20057_4};	/*Profile ID - 20057*/
const Value V20155_1_1 = {S2, NULL};	/*Value = True*/
const Property P20155_1 = {S50, 0, &V20155_1_1, NULL};	/*Property = IsMobile*/
const Value V20155_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20155_5 = {S52, 0, &V20155_5_1227, &P20155_1};	/*Property = ScreenPixelsHeight*/
const Value V20155_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20155_4 = {S54, 0, &V20155_4_1229, &P20155_5};	/*Property = ScreenPixelsWidth*/
const Profile P20155 = {"20155", &P20155_4};	/*Profile ID - 20155*/
const Value V20277_1_1 = {S2, NULL};	/*Value = True*/
const Property P20277_1 = {S50, 0, &V20277_1_1, NULL};	/*Property = IsMobile*/
const Value V20277_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20277_5 = {S52, 0, &V20277_5_1227, &P20277_1};	/*Property = ScreenPixelsHeight*/
const Value V20277_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20277_4 = {S54, 0, &V20277_4_1229, &P20277_5};	/*Property = ScreenPixelsWidth*/
const Profile P20277 = {"20277", &P20277_4};	/*Profile ID - 20277*/
const Value V20181_1_1 = {S2, NULL};	/*Value = True*/
const Property P20181_1 = {S50, 0, &V20181_1_1, NULL};	/*Property = IsMobile*/
const Value V20181_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20181_5 = {S52, 0, &V20181_5_1227, &P20181_1};	/*Property = ScreenPixelsHeight*/
const Value V20181_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20181_4 = {S54, 0, &V20181_4_1229, &P20181_5};	/*Property = ScreenPixelsWidth*/
const Profile P20181 = {"20181", &P20181_4};	/*Profile ID - 20181*/
const Value V23084_1_1 = {S2, NULL};	/*Value = True*/
const Property P23084_1 = {S50, 0, &V23084_1_1, NULL};	/*Property = IsMobile*/
const Value V23084_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23084_5 = {S52, 0, &V23084_5_37569, &P23084_1};	/*Property = ScreenPixelsHeight*/
const Value V23084_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23084_4 = {S54, 0, &V23084_4_5645, &P23084_5};	/*Property = ScreenPixelsWidth*/
const Profile P23084 = {"23084", &P23084_4};	/*Profile ID - 23084*/
const Value V23287_1_1 = {S2, NULL};	/*Value = True*/
const Property P23287_1 = {S50, 0, &V23287_1_1, NULL};	/*Property = IsMobile*/
const Value V23287_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23287_5 = {S52, 0, &V23287_5_1227, &P23287_1};	/*Property = ScreenPixelsHeight*/
const Value V23287_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P23287_4 = {S54, 0, &V23287_4_1477, &P23287_5};	/*Property = ScreenPixelsWidth*/
const Profile P23287 = {"23287", &P23287_4};	/*Profile ID - 23287*/
const Value V20548_1_1 = {S2, NULL};	/*Value = True*/
const Property P20548_1 = {S50, 0, &V20548_1_1, NULL};	/*Property = IsMobile*/
const Value V20548_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20548_5 = {S52, 0, &V20548_5_9981, &P20548_1};	/*Property = ScreenPixelsHeight*/
const Value V20548_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20548_4 = {S54, 0, &V20548_4_2336, &P20548_5};	/*Property = ScreenPixelsWidth*/
const Profile P20548 = {"20548", &P20548_4};	/*Profile ID - 20548*/
const Value V20825_1_1 = {S2, NULL};	/*Value = True*/
const Property P20825_1 = {S50, 0, &V20825_1_1, NULL};	/*Property = IsMobile*/
const Value V20825_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20825_5 = {S52, 0, &V20825_5_1227, &P20825_1};	/*Property = ScreenPixelsHeight*/
const Value V20825_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20825_4 = {S54, 0, &V20825_4_1229, &P20825_5};	/*Property = ScreenPixelsWidth*/
const Profile P20825 = {"20825", &P20825_4};	/*Profile ID - 20825*/
const Value V20880_1_1 = {S2, NULL};	/*Value = True*/
const Property P20880_1 = {S50, 0, &V20880_1_1, NULL};	/*Property = IsMobile*/
const Value V20880_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20880_5 = {S52, 0, &V20880_5_1403, &P20880_1};	/*Property = ScreenPixelsHeight*/
const Value V20880_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20880_4 = {S54, 0, &V20880_4_1404, &P20880_5};	/*Property = ScreenPixelsWidth*/
const Profile P20880 = {"20880", &P20880_4};	/*Profile ID - 20880*/
const Value V24427_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_279 = {S1, 0, &V24427_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24427_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_282 = {S3, 0, &V24427_282_57155, &P24427_279};	/*Property = BlobBuilder*/
const Value V24427_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_350 = {S4, 0, &V24427_350_57446, &P24427_282};	/*Property = Canvas*/
const Value V24427_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_283 = {S5, 0, &V24427_283_57161, &P24427_350};	/*Property = CssBackground*/
const Value V24427_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_284 = {S6, 0, &V24427_284_57164, &P24427_283};	/*Property = CssBorderImage*/
const Value V24427_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_285 = {S7, 0, &V24427_285_57167, &P24427_284};	/*Property = CssCanvas*/
const Value V24427_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_286 = {S8, 0, &V24427_286_57170, &P24427_285};	/*Property = CssColor*/
const Value V24427_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_237 = {S9, 0, &V24427_237_55786, &P24427_286};	/*Property = CssColumn*/
const Value V24427_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_287 = {S10, 0, &V24427_287_57173, &P24427_237};	/*Property = CssFlexbox*/
const Value V24427_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_288 = {S11, 0, &V24427_288_57176, &P24427_287};	/*Property = CssFont*/
const Value V24427_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_289 = {S12, 0, &V24427_289_86899, &P24427_288};	/*Property = CssImages*/
const Value V24427_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_291 = {S13, 0, &V24427_291_57182, &P24427_289};	/*Property = CssMediaQueries*/
const Value V24427_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_292 = {S14, 0, &V24427_292_57185, &P24427_291};	/*Property = CssMinMax*/
const Value V24427_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_293 = {S15, 0, &V24427_293_57188, &P24427_292};	/*Property = CssOverflow*/
const Value V24427_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_294 = {S16, 0, &V24427_294_57191, &P24427_293};	/*Property = CssPosition*/
const Value V24427_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_295 = {S17, 0, &V24427_295_57194, &P24427_294};	/*Property = CssText*/
const Value V24427_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_297 = {S18, 0, &V24427_297_57197, &P24427_295};	/*Property = CssTransforms*/
const Value V24427_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_241 = {S19, 0, &V24427_241_55801, &P24427_297};	/*Property = CssTransitions*/
const Value V24427_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_298 = {S20, 0, &V24427_298_57208, &P24427_241};	/*Property = CssUI*/
const Value V24427_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_300 = {S21, 0, &V24427_300_57217, &P24427_298};	/*Property = DataSet*/
const Value V24427_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_301 = {S22, 0, &V24427_301_57220, &P24427_300};	/*Property = DataUrl*/
const Value V24427_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_302 = {S23, 0, &V24427_302_57223, &P24427_301};	/*Property = DeviceOrientation*/
const Value V24427_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_304 = {S24, 0, &V24427_304_57229, &P24427_302};	/*Property = FileReader*/
const Value V24427_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_305 = {S25, 0, &V24427_305_57232, &P24427_304};	/*Property = FileSaver*/
const Value V24427_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_306 = {S26, 0, &V24427_306_57238, &P24427_305};	/*Property = FileWriter*/
const Value V24427_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_307 = {S27, 0, &V24427_307_57241, &P24427_306};	/*Property = FormData*/
const Value V24427_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_309 = {S28, 0, &V24427_309_57247, &P24427_307};	/*Property = Fullscreen*/
const Value V24427_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_310 = {S29, 0, &V24427_310_57250, &P24427_309};	/*Property = GeoLocation*/
const Value V24427_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_312 = {S30, 0, &V24427_312_57256, &P24427_310};	/*Property = History*/
const Value V24427_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_314 = {S31, 0, &V24427_314_57262, &P24427_312};	/*Property = Html5*/
const Value V24427_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_313 = {S32, 0, &V24427_313_57259, &P24427_314};	/*Property = Html-Media-Capture*/
const Value V24427_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_315 = {S33, 0, &V24427_315_57265, &P24427_313};	/*Property = Iframe*/
const Value V24427_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_316 = {S34, 0, &V24427_316_57268, &P24427_315};	/*Property = IndexedDB*/
const Value V24427_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_317 = {S35, 0, &V24427_317_57271, &P24427_316};	/*Property = Json*/
const Value V24427_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24427_15 = {S37, 0, &V24427_15_28, &P24427_317};	/*Property = LayoutEngine*/
const Value V24427_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_344 = {S38, 0, &V24427_344_57274, &P24427_15};	/*Property = Masking*/
const Value V24427_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_320 = {S39, 0, &V24427_320_57292, &P24427_344};	/*Property = PostMessage*/
const Value V24427_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_321 = {S40, 0, &V24427_321_57295, &P24427_320};	/*Property = Progress*/
const Value V24427_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_322 = {S41, 0, &V24427_322_57298, &P24427_321};	/*Property = Prompts*/
const Value V24427_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_323 = {S42, 0, &V24427_323_57301, &P24427_322};	/*Property = Selector*/
const Value V24427_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_324 = {S43, 0, &V24427_324_57307, &P24427_323};	/*Property = Svg*/
const Value V24427_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_327 = {S44, 0, &V24427_327_57316, &P24427_324};	/*Property = TouchEvents*/
const Value V24427_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_328 = {S45, 0, &V24427_328_57319, &P24427_327};	/*Property = Track*/
const Value V24427_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_329 = {S46, 0, &V24427_329_57325, &P24427_328};	/*Property = Video*/
const Value V24427_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_330 = {S47, 0, &V24427_330_57328, &P24427_329};	/*Property = Viewport*/
const Value V24427_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_332 = {S48, 0, &V24427_332_57340, &P24427_330};	/*Property = WebWorkers*/
const Value V24427_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24427_333 = {S49, 0, &V24427_333_57343, &P24427_332};	/*Property = Xhr2*/
const Profile P24427 = {"24427", &P24427_333};	/*Profile ID - 24427*/
const Value V20954_1_1 = {S2, NULL};	/*Value = True*/
const Property P20954_1 = {S50, 0, &V20954_1_1, NULL};	/*Property = IsMobile*/
const Value V20954_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20954_5 = {S52, 0, &V20954_5_1227, &P20954_1};	/*Property = ScreenPixelsHeight*/
const Value V20954_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20954_4 = {S54, 0, &V20954_4_1229, &P20954_5};	/*Property = ScreenPixelsWidth*/
const Profile P20954 = {"20954", &P20954_4};	/*Profile ID - 20954*/
const Value V20932_1_1 = {S2, NULL};	/*Value = True*/
const Property P20932_1 = {S50, 0, &V20932_1_1, NULL};	/*Property = IsMobile*/
const Value V20932_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20932_5 = {S52, 0, &V20932_5_1227, &P20932_1};	/*Property = ScreenPixelsHeight*/
const Value V20932_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20932_4 = {S54, 0, &V20932_4_1229, &P20932_5};	/*Property = ScreenPixelsWidth*/
const Profile P20932 = {"20932", &P20932_4};	/*Profile ID - 20932*/
const Value V22100_1_1 = {S2, NULL};	/*Value = True*/
const Property P22100_1 = {S50, 0, &V22100_1_1, NULL};	/*Property = IsMobile*/
const Value V22100_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22100_5 = {S52, 0, &V22100_5_1227, &P22100_1};	/*Property = ScreenPixelsHeight*/
const Value V22100_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P22100_4 = {S54, 0, &V22100_4_1477, &P22100_5};	/*Property = ScreenPixelsWidth*/
const Profile P22100 = {"22100", &P22100_4};	/*Profile ID - 22100*/
const Value V20097_1_1 = {S2, NULL};	/*Value = True*/
const Property P20097_1 = {S50, 0, &V20097_1_1, NULL};	/*Property = IsMobile*/
const Value V20097_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20097_5 = {S52, 0, &V20097_5_1458, &P20097_1};	/*Property = ScreenPixelsHeight*/
const Value V20097_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20097_4 = {S54, 0, &V20097_4_37617, &P20097_5};	/*Property = ScreenPixelsWidth*/
const Profile P20097 = {"20097", &P20097_4};	/*Profile ID - 20097*/
const Value V21085_1_1 = {S2, NULL};	/*Value = True*/
const Property P21085_1 = {S50, 0, &V21085_1_1, NULL};	/*Property = IsMobile*/
const Value V21085_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21085_5 = {S52, 0, &V21085_5_1227, &P21085_1};	/*Property = ScreenPixelsHeight*/
const Value V21085_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P21085_4 = {S54, 0, &V21085_4_1943, &P21085_5};	/*Property = ScreenPixelsWidth*/
const Profile P21085 = {"21085", &P21085_4};	/*Profile ID - 21085*/
const Value V21651_1_1 = {S2, NULL};	/*Value = True*/
const Property P21651_1 = {S50, 0, &V21651_1_1, NULL};	/*Property = IsMobile*/
const Value V21651_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21651_5 = {S52, 0, &V21651_5_37, &P21651_1};	/*Property = ScreenPixelsHeight*/
const Value V21651_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21651_4 = {S54, 0, &V21651_4_36, &P21651_5};	/*Property = ScreenPixelsWidth*/
const Profile P21651 = {"21651", &P21651_4};	/*Profile ID - 21651*/
const Value V21256_1_1 = {S2, NULL};	/*Value = True*/
const Property P21256_1 = {S50, 0, &V21256_1_1, NULL};	/*Property = IsMobile*/
const Value V21256_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21256_5 = {S52, 0, &V21256_5_1227, &P21256_1};	/*Property = ScreenPixelsHeight*/
const Value V21256_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21256_4 = {S54, 0, &V21256_4_1229, &P21256_5};	/*Property = ScreenPixelsWidth*/
const Profile P21256 = {"21256", &P21256_4};	/*Profile ID - 21256*/
const Value V17615_1_1 = {S2, NULL};	/*Value = True*/
const Property P17615_1 = {S50, 0, &V17615_1_1, NULL};	/*Property = IsMobile*/
const Value V17615_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P17615_5 = {S52, 0, &V17615_5_1293, &P17615_1};	/*Property = ScreenPixelsHeight*/
const Value V17615_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P17615_4 = {S54, 0, &V17615_4_1296, &P17615_5};	/*Property = ScreenPixelsWidth*/
const Profile P17615 = {"17615", &P17615_4};	/*Profile ID - 17615*/
const Value V20507_1_1 = {S2, NULL};	/*Value = True*/
const Property P20507_1 = {S50, 0, &V20507_1_1, NULL};	/*Property = IsMobile*/
const Value V20507_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20507_5 = {S52, 0, &V20507_5_1458, &P20507_1};	/*Property = ScreenPixelsHeight*/
const Value V20507_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20507_4 = {S54, 0, &V20507_4_1460, &P20507_5};	/*Property = ScreenPixelsWidth*/
const Profile P20507 = {"20507", &P20507_4};	/*Profile ID - 20507*/
const Value V20705_1_1 = {S2, NULL};	/*Value = True*/
const Property P20705_1 = {S50, 0, &V20705_1_1, NULL};	/*Property = IsMobile*/
const Value V20705_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P20705_5 = {S52, 0, &V20705_5_59138, &P20705_1};	/*Property = ScreenPixelsHeight*/
const Value V20705_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P20705_4 = {S54, 0, &V20705_4_59970, &P20705_5};	/*Property = ScreenPixelsWidth*/
const Profile P20705 = {"20705", &P20705_4};	/*Profile ID - 20705*/
const Value V20090_1_1 = {S2, NULL};	/*Value = True*/
const Property P20090_1 = {S50, 0, &V20090_1_1, NULL};	/*Property = IsMobile*/
const Value V20090_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20090_5 = {S52, 0, &V20090_5_1227, &P20090_1};	/*Property = ScreenPixelsHeight*/
const Value V20090_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20090_4 = {S54, 0, &V20090_4_1229, &P20090_5};	/*Property = ScreenPixelsWidth*/
const Profile P20090 = {"20090", &P20090_4};	/*Profile ID - 20090*/
const Value V20098_1_1 = {S2, NULL};	/*Value = True*/
const Property P20098_1 = {S50, 0, &V20098_1_1, NULL};	/*Property = IsMobile*/
const Value V20098_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20098_5 = {S52, 0, &V20098_5_1227, &P20098_1};	/*Property = ScreenPixelsHeight*/
const Value V20098_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20098_4 = {S54, 0, &V20098_4_1229, &P20098_5};	/*Property = ScreenPixelsWidth*/
const Profile P20098 = {"20098", &P20098_4};	/*Profile ID - 20098*/
const Value V20133_1_1 = {S2, NULL};	/*Value = True*/
const Property P20133_1 = {S50, 0, &V20133_1_1, NULL};	/*Property = IsMobile*/
const Value V20133_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20133_5 = {S52, 0, &V20133_5_1403, &P20133_1};	/*Property = ScreenPixelsHeight*/
const Value V20133_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20133_4 = {S54, 0, &V20133_4_1404, &P20133_5};	/*Property = ScreenPixelsWidth*/
const Profile P20133 = {"20133", &P20133_4};	/*Profile ID - 20133*/
const Value V20401_1_1 = {S2, NULL};	/*Value = True*/
const Property P20401_1 = {S50, 0, &V20401_1_1, NULL};	/*Property = IsMobile*/
const Value V20401_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20401_5 = {S52, 0, &V20401_5_37, &P20401_1};	/*Property = ScreenPixelsHeight*/
const Value V20401_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20401_4 = {S54, 0, &V20401_4_36, &P20401_5};	/*Property = ScreenPixelsWidth*/
const Profile P20401 = {"20401", &P20401_4};	/*Profile ID - 20401*/
const Value V23934_1_1 = {S2, NULL};	/*Value = True*/
const Property P23934_1 = {S50, 0, &V23934_1_1, NULL};	/*Property = IsMobile*/
const Value V23934_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P23934_5 = {S52, 0, &V23934_5_1304, &P23934_1};	/*Property = ScreenPixelsHeight*/
const Value V23934_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23934_4 = {S54, 0, &V23934_4_2336, &P23934_5};	/*Property = ScreenPixelsWidth*/
const Profile P23934 = {"23934", &P23934_4};	/*Profile ID - 23934*/
const Value V20152_1_1 = {S2, NULL};	/*Value = True*/
const Property P20152_1 = {S50, 0, &V20152_1_1, NULL};	/*Property = IsMobile*/
const Value V20152_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20152_5 = {S52, 0, &V20152_5_1403, &P20152_1};	/*Property = ScreenPixelsHeight*/
const Value V20152_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20152_4 = {S54, 0, &V20152_4_1404, &P20152_5};	/*Property = ScreenPixelsWidth*/
const Profile P20152 = {"20152", &P20152_4};	/*Profile ID - 20152*/
const Value V20818_1_1 = {S2, NULL};	/*Value = True*/
const Property P20818_1 = {S50, 0, &V20818_1_1, NULL};	/*Property = IsMobile*/
const Value V20818_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20818_5 = {S52, 0, &V20818_5_1227, &P20818_1};	/*Property = ScreenPixelsHeight*/
const Value V20818_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20818_4 = {S54, 0, &V20818_4_1229, &P20818_5};	/*Property = ScreenPixelsWidth*/
const Profile P20818 = {"20818", &P20818_4};	/*Profile ID - 20818*/
const Value V20790_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_279 = {S1, 0, &V20790_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20790_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_282 = {S3, 0, &V20790_282_57155, &P20790_279};	/*Property = BlobBuilder*/
const Value V20790_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_350 = {S4, 0, &V20790_350_57446, &P20790_282};	/*Property = Canvas*/
const Value V20790_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_283 = {S5, 0, &V20790_283_57161, &P20790_350};	/*Property = CssBackground*/
const Value V20790_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_284 = {S6, 0, &V20790_284_57164, &P20790_283};	/*Property = CssBorderImage*/
const Value V20790_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_285 = {S7, 0, &V20790_285_57167, &P20790_284};	/*Property = CssCanvas*/
const Value V20790_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_286 = {S8, 0, &V20790_286_57170, &P20790_285};	/*Property = CssColor*/
const Value V20790_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_237 = {S9, 0, &V20790_237_55786, &P20790_286};	/*Property = CssColumn*/
const Value V20790_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_287 = {S10, 0, &V20790_287_57173, &P20790_237};	/*Property = CssFlexbox*/
const Value V20790_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_288 = {S11, 0, &V20790_288_57176, &P20790_287};	/*Property = CssFont*/
const Value V20790_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_289 = {S12, 0, &V20790_289_86899, &P20790_288};	/*Property = CssImages*/
const Value V20790_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_291 = {S13, 0, &V20790_291_57182, &P20790_289};	/*Property = CssMediaQueries*/
const Value V20790_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_292 = {S14, 0, &V20790_292_57185, &P20790_291};	/*Property = CssMinMax*/
const Value V20790_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_293 = {S15, 0, &V20790_293_57188, &P20790_292};	/*Property = CssOverflow*/
const Value V20790_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_294 = {S16, 0, &V20790_294_57191, &P20790_293};	/*Property = CssPosition*/
const Value V20790_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_295 = {S17, 0, &V20790_295_57194, &P20790_294};	/*Property = CssText*/
const Value V20790_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_297 = {S18, 0, &V20790_297_57197, &P20790_295};	/*Property = CssTransforms*/
const Value V20790_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_241 = {S19, 0, &V20790_241_55801, &P20790_297};	/*Property = CssTransitions*/
const Value V20790_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_298 = {S20, 0, &V20790_298_57208, &P20790_241};	/*Property = CssUI*/
const Value V20790_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_300 = {S21, 0, &V20790_300_57217, &P20790_298};	/*Property = DataSet*/
const Value V20790_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_301 = {S22, 0, &V20790_301_57220, &P20790_300};	/*Property = DataUrl*/
const Value V20790_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_302 = {S23, 0, &V20790_302_57223, &P20790_301};	/*Property = DeviceOrientation*/
const Value V20790_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_304 = {S24, 0, &V20790_304_57229, &P20790_302};	/*Property = FileReader*/
const Value V20790_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_305 = {S25, 0, &V20790_305_57232, &P20790_304};	/*Property = FileSaver*/
const Value V20790_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_306 = {S26, 0, &V20790_306_57238, &P20790_305};	/*Property = FileWriter*/
const Value V20790_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_307 = {S27, 0, &V20790_307_57241, &P20790_306};	/*Property = FormData*/
const Value V20790_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_309 = {S28, 0, &V20790_309_57247, &P20790_307};	/*Property = Fullscreen*/
const Value V20790_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_310 = {S29, 0, &V20790_310_57250, &P20790_309};	/*Property = GeoLocation*/
const Value V20790_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_312 = {S30, 0, &V20790_312_57256, &P20790_310};	/*Property = History*/
const Value V20790_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_314 = {S31, 0, &V20790_314_57262, &P20790_312};	/*Property = Html5*/
const Value V20790_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_313 = {S32, 0, &V20790_313_57259, &P20790_314};	/*Property = Html-Media-Capture*/
const Value V20790_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_315 = {S33, 0, &V20790_315_57265, &P20790_313};	/*Property = Iframe*/
const Value V20790_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_316 = {S34, 0, &V20790_316_57268, &P20790_315};	/*Property = IndexedDB*/
const Value V20790_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_317 = {S35, 0, &V20790_317_57271, &P20790_316};	/*Property = Json*/
const Value V20790_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P20790_15 = {S37, 0, &V20790_15_54167, &P20790_317};	/*Property = LayoutEngine*/
const Value V20790_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_344 = {S38, 0, &V20790_344_57274, &P20790_15};	/*Property = Masking*/
const Value V20790_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_320 = {S39, 0, &V20790_320_57292, &P20790_344};	/*Property = PostMessage*/
const Value V20790_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_321 = {S40, 0, &V20790_321_57295, &P20790_320};	/*Property = Progress*/
const Value V20790_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_322 = {S41, 0, &V20790_322_57298, &P20790_321};	/*Property = Prompts*/
const Value V20790_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_323 = {S42, 0, &V20790_323_57301, &P20790_322};	/*Property = Selector*/
const Value V20790_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_324 = {S43, 0, &V20790_324_57307, &P20790_323};	/*Property = Svg*/
const Value V20790_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_327 = {S44, 0, &V20790_327_57316, &P20790_324};	/*Property = TouchEvents*/
const Value V20790_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_328 = {S45, 0, &V20790_328_57319, &P20790_327};	/*Property = Track*/
const Value V20790_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_329 = {S46, 0, &V20790_329_57325, &P20790_328};	/*Property = Video*/
const Value V20790_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_330 = {S47, 0, &V20790_330_57328, &P20790_329};	/*Property = Viewport*/
const Value V20790_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_332 = {S48, 0, &V20790_332_57340, &P20790_330};	/*Property = WebWorkers*/
const Value V20790_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20790_333 = {S49, 0, &V20790_333_57343, &P20790_332};	/*Property = Xhr2*/
const Profile P20790 = {"20790", &P20790_333};	/*Profile ID - 20790*/
const Value V20278_1_1 = {S2, NULL};	/*Value = True*/
const Property P20278_1 = {S50, 0, &V20278_1_1, NULL};	/*Property = IsMobile*/
const Value V20278_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20278_5 = {S52, 0, &V20278_5_1458, &P20278_1};	/*Property = ScreenPixelsHeight*/
const Value V20278_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20278_4 = {S54, 0, &V20278_4_1460, &P20278_5};	/*Property = ScreenPixelsWidth*/
const Profile P20278 = {"20278", &P20278_4};	/*Profile ID - 20278*/
const Value V20881_1_1 = {S2, NULL};	/*Value = True*/
const Property P20881_1 = {S50, 0, &V20881_1_1, NULL};	/*Property = IsMobile*/
const Value V20881_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20881_5 = {S52, 0, &V20881_5_1227, &P20881_1};	/*Property = ScreenPixelsHeight*/
const Value V20881_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20881_4 = {S54, 0, &V20881_4_1229, &P20881_5};	/*Property = ScreenPixelsWidth*/
const Profile P20881 = {"20881", &P20881_4};	/*Profile ID - 20881*/
const Value V20829_1_1 = {S2, NULL};	/*Value = True*/
const Property P20829_1 = {S50, 0, &V20829_1_1, NULL};	/*Property = IsMobile*/
const Value V20829_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20829_5 = {S52, 0, &V20829_5_1458, &P20829_1};	/*Property = ScreenPixelsHeight*/
const Value V20829_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20829_4 = {S54, 0, &V20829_4_37617, &P20829_5};	/*Property = ScreenPixelsWidth*/
const Profile P20829 = {"20829", &P20829_4};	/*Profile ID - 20829*/
const Value V20446_1_1 = {S2, NULL};	/*Value = True*/
const Property P20446_1 = {S50, 0, &V20446_1_1, NULL};	/*Property = IsMobile*/
const Value V20446_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P20446_5 = {S52, 0, &V20446_5_59138, &P20446_1};	/*Property = ScreenPixelsHeight*/
const Value V20446_4_52971 = {S122, NULL};	/*Value = 1200*/
const Property P20446_4 = {S54, 0, &V20446_4_52971, &P20446_5};	/*Property = ScreenPixelsWidth*/
const Profile P20446 = {"20446", &P20446_4};	/*Profile ID - 20446*/
const Value V20840_1_1 = {S2, NULL};	/*Value = True*/
const Property P20840_1 = {S50, 0, &V20840_1_1, NULL};	/*Property = IsMobile*/
const Value V20840_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20840_5 = {S52, 0, &V20840_5_1227, &P20840_1};	/*Property = ScreenPixelsHeight*/
const Value V20840_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20840_4 = {S54, 0, &V20840_4_1229, &P20840_5};	/*Property = ScreenPixelsWidth*/
const Profile P20840 = {"20840", &P20840_4};	/*Profile ID - 20840*/
const Value V21887_1_1 = {S2, NULL};	/*Value = True*/
const Property P21887_1 = {S50, 0, &V21887_1_1, NULL};	/*Property = IsMobile*/
const Value V21887_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21887_5 = {S52, 0, &V21887_5_1458, &P21887_1};	/*Property = ScreenPixelsHeight*/
const Value V21887_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21887_4 = {S54, 0, &V21887_4_37617, &P21887_5};	/*Property = ScreenPixelsWidth*/
const Profile P21887 = {"21887", &P21887_4};	/*Profile ID - 21887*/
const Value V23595_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_279 = {S1, 0, &V23595_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23595_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_282 = {S3, 0, &V23595_282_57155, &P23595_279};	/*Property = BlobBuilder*/
const Value V23595_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_350 = {S4, 0, &V23595_350_57446, &P23595_282};	/*Property = Canvas*/
const Value V23595_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_283 = {S5, 0, &V23595_283_57161, &P23595_350};	/*Property = CssBackground*/
const Value V23595_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_284 = {S6, 0, &V23595_284_57164, &P23595_283};	/*Property = CssBorderImage*/
const Value V23595_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_285 = {S7, 0, &V23595_285_57167, &P23595_284};	/*Property = CssCanvas*/
const Value V23595_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_286 = {S8, 0, &V23595_286_57170, &P23595_285};	/*Property = CssColor*/
const Value V23595_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_237 = {S9, 0, &V23595_237_55786, &P23595_286};	/*Property = CssColumn*/
const Value V23595_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_287 = {S10, 0, &V23595_287_57173, &P23595_237};	/*Property = CssFlexbox*/
const Value V23595_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_288 = {S11, 0, &V23595_288_57176, &P23595_287};	/*Property = CssFont*/
const Value V23595_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_289 = {S12, 0, &V23595_289_86899, &P23595_288};	/*Property = CssImages*/
const Value V23595_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_291 = {S13, 0, &V23595_291_57182, &P23595_289};	/*Property = CssMediaQueries*/
const Value V23595_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_292 = {S14, 0, &V23595_292_57185, &P23595_291};	/*Property = CssMinMax*/
const Value V23595_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_293 = {S15, 0, &V23595_293_57188, &P23595_292};	/*Property = CssOverflow*/
const Value V23595_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_294 = {S16, 0, &V23595_294_57191, &P23595_293};	/*Property = CssPosition*/
const Value V23595_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_295 = {S17, 0, &V23595_295_57194, &P23595_294};	/*Property = CssText*/
const Value V23595_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_297 = {S18, 0, &V23595_297_57197, &P23595_295};	/*Property = CssTransforms*/
const Value V23595_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_241 = {S19, 0, &V23595_241_55801, &P23595_297};	/*Property = CssTransitions*/
const Value V23595_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_298 = {S20, 0, &V23595_298_57208, &P23595_241};	/*Property = CssUI*/
const Value V23595_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_300 = {S21, 0, &V23595_300_57217, &P23595_298};	/*Property = DataSet*/
const Value V23595_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_301 = {S22, 0, &V23595_301_57220, &P23595_300};	/*Property = DataUrl*/
const Value V23595_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_302 = {S23, 0, &V23595_302_57223, &P23595_301};	/*Property = DeviceOrientation*/
const Value V23595_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_304 = {S24, 0, &V23595_304_57229, &P23595_302};	/*Property = FileReader*/
const Value V23595_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_305 = {S25, 0, &V23595_305_57232, &P23595_304};	/*Property = FileSaver*/
const Value V23595_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_306 = {S26, 0, &V23595_306_57238, &P23595_305};	/*Property = FileWriter*/
const Value V23595_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_307 = {S27, 0, &V23595_307_57241, &P23595_306};	/*Property = FormData*/
const Value V23595_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_309 = {S28, 0, &V23595_309_57247, &P23595_307};	/*Property = Fullscreen*/
const Value V23595_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_310 = {S29, 0, &V23595_310_57250, &P23595_309};	/*Property = GeoLocation*/
const Value V23595_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_312 = {S30, 0, &V23595_312_57256, &P23595_310};	/*Property = History*/
const Value V23595_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_314 = {S31, 0, &V23595_314_57262, &P23595_312};	/*Property = Html5*/
const Value V23595_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_313 = {S32, 0, &V23595_313_57259, &P23595_314};	/*Property = Html-Media-Capture*/
const Value V23595_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_315 = {S33, 0, &V23595_315_57265, &P23595_313};	/*Property = Iframe*/
const Value V23595_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_316 = {S34, 0, &V23595_316_57268, &P23595_315};	/*Property = IndexedDB*/
const Value V23595_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_317 = {S35, 0, &V23595_317_57271, &P23595_316};	/*Property = Json*/
const Value V23595_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_15 = {S37, 0, &V23595_15_72, &P23595_317};	/*Property = LayoutEngine*/
const Value V23595_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_344 = {S38, 0, &V23595_344_57274, &P23595_15};	/*Property = Masking*/
const Value V23595_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_320 = {S39, 0, &V23595_320_57292, &P23595_344};	/*Property = PostMessage*/
const Value V23595_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_321 = {S40, 0, &V23595_321_57295, &P23595_320};	/*Property = Progress*/
const Value V23595_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_322 = {S41, 0, &V23595_322_57298, &P23595_321};	/*Property = Prompts*/
const Value V23595_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_323 = {S42, 0, &V23595_323_57301, &P23595_322};	/*Property = Selector*/
const Value V23595_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_324 = {S43, 0, &V23595_324_57307, &P23595_323};	/*Property = Svg*/
const Value V23595_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_327 = {S44, 0, &V23595_327_57316, &P23595_324};	/*Property = TouchEvents*/
const Value V23595_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_328 = {S45, 0, &V23595_328_57319, &P23595_327};	/*Property = Track*/
const Value V23595_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_329 = {S46, 0, &V23595_329_57325, &P23595_328};	/*Property = Video*/
const Value V23595_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_330 = {S47, 0, &V23595_330_57328, &P23595_329};	/*Property = Viewport*/
const Value V23595_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_332 = {S48, 0, &V23595_332_57340, &P23595_330};	/*Property = WebWorkers*/
const Value V23595_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23595_333 = {S49, 0, &V23595_333_57343, &P23595_332};	/*Property = Xhr2*/
const Profile P23595 = {"23595", &P23595_333};	/*Profile ID - 23595*/
const Value V19858_1_1 = {S2, NULL};	/*Value = True*/
const Property P19858_1 = {S50, 0, &V19858_1_1, NULL};	/*Property = IsMobile*/
const Value V19858_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19858_5 = {S52, 0, &V19858_5_1227, &P19858_1};	/*Property = ScreenPixelsHeight*/
const Value V19858_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19858_4 = {S54, 0, &V19858_4_1229, &P19858_5};	/*Property = ScreenPixelsWidth*/
const Profile P19858 = {"19858", &P19858_4};	/*Profile ID - 19858*/
const Value V20922_1_1 = {S2, NULL};	/*Value = True*/
const Property P20922_1 = {S50, 0, &V20922_1_1, NULL};	/*Property = IsMobile*/
const Value V20922_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20922_5 = {S52, 0, &V20922_5_1227, &P20922_1};	/*Property = ScreenPixelsHeight*/
const Value V20922_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20922_4 = {S54, 0, &V20922_4_1229, &P20922_5};	/*Property = ScreenPixelsWidth*/
const Profile P20922 = {"20922", &P20922_4};	/*Profile ID - 20922*/
const Value V20178_1_1 = {S2, NULL};	/*Value = True*/
const Property P20178_1 = {S50, 0, &V20178_1_1, NULL};	/*Property = IsMobile*/
const Value V20178_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20178_5 = {S52, 0, &V20178_5_1458, &P20178_1};	/*Property = ScreenPixelsHeight*/
const Value V20178_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20178_4 = {S54, 0, &V20178_4_37617, &P20178_5};	/*Property = ScreenPixelsWidth*/
const Profile P20178 = {"20178", &P20178_4};	/*Profile ID - 20178*/
const Value V20174_1_1 = {S2, NULL};	/*Value = True*/
const Property P20174_1 = {S50, 0, &V20174_1_1, NULL};	/*Property = IsMobile*/
const Value V20174_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P20174_5 = {S52, 0, &V20174_5_1269, &P20174_1};	/*Property = ScreenPixelsHeight*/
const Value V20174_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20174_4 = {S54, 0, &V20174_4_1229, &P20174_5};	/*Property = ScreenPixelsWidth*/
const Profile P20174 = {"20174", &P20174_4};	/*Profile ID - 20174*/
const Value V20545_1_1 = {S2, NULL};	/*Value = True*/
const Property P20545_1 = {S50, 0, &V20545_1_1, NULL};	/*Property = IsMobile*/
const Value V20545_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20545_5 = {S52, 0, &V20545_5_1458, &P20545_1};	/*Property = ScreenPixelsHeight*/
const Value V20545_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20545_4 = {S54, 0, &V20545_4_1460, &P20545_5};	/*Property = ScreenPixelsWidth*/
const Profile P20545 = {"20545", &P20545_4};	/*Profile ID - 20545*/
const Value V21366_1_1 = {S2, NULL};	/*Value = True*/
const Property P21366_1 = {S50, 0, &V21366_1_1, NULL};	/*Property = IsMobile*/
const Value V21366_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21366_5 = {S52, 0, &V21366_5_1227, &P21366_1};	/*Property = ScreenPixelsHeight*/
const Value V21366_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21366_4 = {S54, 0, &V21366_4_1229, &P21366_5};	/*Property = ScreenPixelsWidth*/
const Profile P21366 = {"21366", &P21366_4};	/*Profile ID - 21366*/
const Value V20463_1_1 = {S2, NULL};	/*Value = True*/
const Property P20463_1 = {S50, 0, &V20463_1_1, NULL};	/*Property = IsMobile*/
const Value V20463_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20463_5 = {S52, 0, &V20463_5_9981, &P20463_1};	/*Property = ScreenPixelsHeight*/
const Value V20463_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20463_4 = {S54, 0, &V20463_4_2336, &P20463_5};	/*Property = ScreenPixelsWidth*/
const Profile P20463 = {"20463", &P20463_4};	/*Profile ID - 20463*/
const Value V20479_1_1 = {S2, NULL};	/*Value = True*/
const Property P20479_1 = {S50, 0, &V20479_1_1, NULL};	/*Property = IsMobile*/
const Value V20479_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20479_5 = {S52, 0, &V20479_5_1227, &P20479_1};	/*Property = ScreenPixelsHeight*/
const Value V20479_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20479_4 = {S54, 0, &V20479_4_1229, &P20479_5};	/*Property = ScreenPixelsWidth*/
const Profile P20479 = {"20479", &P20479_4};	/*Profile ID - 20479*/
const Value V20505_1_1 = {S2, NULL};	/*Value = True*/
const Property P20505_1 = {S50, 0, &V20505_1_1, NULL};	/*Property = IsMobile*/
const Value V20505_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20505_5 = {S52, 0, &V20505_5_1458, &P20505_1};	/*Property = ScreenPixelsHeight*/
const Value V20505_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20505_4 = {S54, 0, &V20505_4_1460, &P20505_5};	/*Property = ScreenPixelsWidth*/
const Profile P20505 = {"20505", &P20505_4};	/*Profile ID - 20505*/
const Value V21615_1_1 = {S2, NULL};	/*Value = True*/
const Property P21615_1 = {S50, 0, &V21615_1_1, NULL};	/*Property = IsMobile*/
const Value V21615_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21615_5 = {S52, 0, &V21615_5_1277, &P21615_1};	/*Property = ScreenPixelsHeight*/
const Value V21615_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21615_4 = {S54, 0, &V21615_4_1279, &P21615_5};	/*Property = ScreenPixelsWidth*/
const Profile P21615 = {"21615", &P21615_4};	/*Profile ID - 21615*/
const Value V23845_1_1 = {S2, NULL};	/*Value = True*/
const Property P23845_1 = {S50, 0, &V23845_1_1, NULL};	/*Property = IsMobile*/
const Value V23845_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23845_5 = {S52, 0, &V23845_5_9981, &P23845_1};	/*Property = ScreenPixelsHeight*/
const Value V23845_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23845_4 = {S54, 0, &V23845_4_2336, &P23845_5};	/*Property = ScreenPixelsWidth*/
const Profile P23845 = {"23845", &P23845_4};	/*Profile ID - 23845*/
const Value V19999_1_1 = {S2, NULL};	/*Value = True*/
const Property P19999_1 = {S50, 0, &V19999_1_1, NULL};	/*Property = IsMobile*/
const Value V19999_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19999_5 = {S52, 0, &V19999_5_1227, &P19999_1};	/*Property = ScreenPixelsHeight*/
const Value V19999_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19999_4 = {S54, 0, &V19999_4_1229, &P19999_5};	/*Property = ScreenPixelsWidth*/
const Profile P19999 = {"19999", &P19999_4};	/*Profile ID - 19999*/
const Value V21254_1_1 = {S2, NULL};	/*Value = True*/
const Property P21254_1 = {S50, 0, &V21254_1_1, NULL};	/*Property = IsMobile*/
const Value V21254_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21254_5 = {S52, 0, &V21254_5_1277, &P21254_1};	/*Property = ScreenPixelsHeight*/
const Value V21254_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21254_4 = {S54, 0, &V21254_4_1279, &P21254_5};	/*Property = ScreenPixelsWidth*/
const Profile P21254 = {"21254", &P21254_4};	/*Profile ID - 21254*/
const Value V20169_1_1 = {S2, NULL};	/*Value = True*/
const Property P20169_1 = {S50, 0, &V20169_1_1, NULL};	/*Property = IsMobile*/
const Value V20169_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20169_5 = {S52, 0, &V20169_5_1277, &P20169_1};	/*Property = ScreenPixelsHeight*/
const Value V20169_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20169_4 = {S54, 0, &V20169_4_1279, &P20169_5};	/*Property = ScreenPixelsWidth*/
const Profile P20169 = {"20169", &P20169_4};	/*Profile ID - 20169*/
const Value V19996_1_1 = {S2, NULL};	/*Value = True*/
const Property P19996_1 = {S50, 0, &V19996_1_1, NULL};	/*Property = IsMobile*/
const Value V19996_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19996_5 = {S52, 0, &V19996_5_1227, &P19996_1};	/*Property = ScreenPixelsHeight*/
const Value V19996_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19996_4 = {S54, 0, &V19996_4_1229, &P19996_5};	/*Property = ScreenPixelsWidth*/
const Profile P19996 = {"19996", &P19996_4};	/*Profile ID - 19996*/
const Value V19839_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19839_279 = {S1, 0, &V19839_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19839_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19839_282 = {S3, 0, &V19839_282_57154, &P19839_279};	/*Property = BlobBuilder*/
const Value V19839_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19839_350 = {S4, 0, &V19839_350_57444, &P19839_282};	/*Property = Canvas*/
const Value V19839_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19839_283 = {S5, 0, &V19839_283_57159, &P19839_350};	/*Property = CssBackground*/
const Value V19839_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19839_284 = {S6, 0, &V19839_284_57163, &P19839_283};	/*Property = CssBorderImage*/
const Value V19839_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19839_285 = {S7, 0, &V19839_285_57166, &P19839_284};	/*Property = CssCanvas*/
const Value V19839_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19839_286 = {S8, 0, &V19839_286_57168, &P19839_285};	/*Property = CssColor*/
const Value V19839_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19839_237 = {S9, 0, &V19839_237_55786, &P19839_286};	/*Property = CssColumn*/
const Value V19839_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19839_287 = {S10, 0, &V19839_287_57172, &P19839_237};	/*Property = CssFlexbox*/
const Value V19839_288_57174 = {S2, NULL};	/*Value = True*/
const Property P19839_288 = {S11, 0, &V19839_288_57174, &P19839_287};	/*Property = CssFont*/
const Value V19839_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19839_289 = {S12, 0, &V19839_289_57759, &P19839_288};	/*Property = CssImages*/
const Value V19839_291_57180 = {S2, NULL};	/*Value = True*/
const Property P19839_291 = {S13, 0, &V19839_291_57180, &P19839_289};	/*Property = CssMediaQueries*/
const Value V19839_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19839_292 = {S14, 0, &V19839_292_57183, &P19839_291};	/*Property = CssMinMax*/
const Value V19839_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19839_293 = {S15, 0, &V19839_293_57187, &P19839_292};	/*Property = CssOverflow*/
const Value V19839_294_57189 = {S2, NULL};	/*Value = True*/
const Property P19839_294 = {S16, 0, &V19839_294_57189, &P19839_293};	/*Property = CssPosition*/
const Value V19839_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19839_295 = {S17, 0, &V19839_295_57192, &P19839_294};	/*Property = CssText*/
const Value V19839_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19839_297 = {S18, 0, &V19839_297_57195, &P19839_295};	/*Property = CssTransforms*/
const Value V19839_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19839_241 = {S19, 0, &V19839_241_55799, &P19839_297};	/*Property = CssTransitions*/
const Value V19839_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19839_298 = {S20, 0, &V19839_298_57206, &P19839_241};	/*Property = CssUI*/
const Value V19839_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19839_300 = {S21, 0, &V19839_300_57216, &P19839_298};	/*Property = DataSet*/
const Value V19839_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19839_301 = {S22, 0, &V19839_301_57218, &P19839_300};	/*Property = DataUrl*/
const Value V19839_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19839_302 = {S23, 0, &V19839_302_57222, &P19839_301};	/*Property = DeviceOrientation*/
const Value V19839_304_57227 = {S2, NULL};	/*Value = True*/
const Property P19839_304 = {S24, 0, &V19839_304_57227, &P19839_302};	/*Property = FileReader*/
const Value V19839_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19839_305 = {S25, 0, &V19839_305_57231, &P19839_304};	/*Property = FileSaver*/
const Value V19839_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19839_306 = {S26, 0, &V19839_306_57237, &P19839_305};	/*Property = FileWriter*/
const Value V19839_307_57239 = {S2, NULL};	/*Value = True*/
const Property P19839_307 = {S27, 0, &V19839_307_57239, &P19839_306};	/*Property = FormData*/
const Value V19839_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19839_309 = {S28, 0, &V19839_309_57246, &P19839_307};	/*Property = Fullscreen*/
const Value V19839_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19839_310 = {S29, 0, &V19839_310_57248, &P19839_309};	/*Property = GeoLocation*/
const Value V19839_312_57254 = {S2, NULL};	/*Value = True*/
const Property P19839_312 = {S30, 0, &V19839_312_57254, &P19839_310};	/*Property = History*/
const Value V19839_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19839_314 = {S31, 0, &V19839_314_57261, &P19839_312};	/*Property = Html5*/
const Value V19839_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19839_313 = {S32, 0, &V19839_313_57258, &P19839_314};	/*Property = Html-Media-Capture*/
const Value V19839_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19839_315 = {S33, 0, &V19839_315_57264, &P19839_313};	/*Property = Iframe*/
const Value V19839_316_57266 = {S2, NULL};	/*Value = True*/
const Property P19839_316 = {S34, 0, &V19839_316_57266, &P19839_315};	/*Property = IndexedDB*/
const Value V19839_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19839_317 = {S35, 0, &V19839_317_57269, &P19839_316};	/*Property = Json*/
const Value V19839_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19839_15 = {S37, 0, &V19839_15_28, &P19839_317};	/*Property = LayoutEngine*/
const Value V19839_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19839_344 = {S38, 0, &V19839_344_57272, &P19839_15};	/*Property = Masking*/
const Value V19839_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19839_320 = {S39, 0, &V19839_320_57290, &P19839_344};	/*Property = PostMessage*/
const Value V19839_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19839_321 = {S40, 0, &V19839_321_57293, &P19839_320};	/*Property = Progress*/
const Value V19839_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19839_322 = {S41, 0, &V19839_322_57296, &P19839_321};	/*Property = Prompts*/
const Value V19839_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19839_323 = {S42, 0, &V19839_323_57299, &P19839_322};	/*Property = Selector*/
const Value V19839_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19839_324 = {S43, 0, &V19839_324_57306, &P19839_323};	/*Property = Svg*/
const Value V19839_327_57315 = {S0, NULL};	/*Value = False*/
const Property P19839_327 = {S44, 0, &V19839_327_57315, &P19839_324};	/*Property = TouchEvents*/
const Value V19839_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19839_328 = {S45, 0, &V19839_328_57318, &P19839_327};	/*Property = Track*/
const Value V19839_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19839_329 = {S46, 0, &V19839_329_57323, &P19839_328};	/*Property = Video*/
const Value V19839_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19839_330 = {S47, 0, &V19839_330_57326, &P19839_329};	/*Property = Viewport*/
const Value V19839_332_57338 = {S2, NULL};	/*Value = True*/
const Property P19839_332 = {S48, 0, &V19839_332_57338, &P19839_330};	/*Property = WebWorkers*/
const Value V19839_333_57341 = {S2, NULL};	/*Value = True*/
const Property P19839_333 = {S49, 0, &V19839_333_57341, &P19839_332};	/*Property = Xhr2*/
const Profile P19839 = {"19839", &P19839_333};	/*Profile ID - 19839*/
const Value V23933_1_1 = {S2, NULL};	/*Value = True*/
const Property P23933_1 = {S50, 0, &V23933_1_1, NULL};	/*Property = IsMobile*/
const Value V23933_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23933_5 = {S52, 0, &V23933_5_1458, &P23933_1};	/*Property = ScreenPixelsHeight*/
const Value V23933_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23933_4 = {S54, 0, &V23933_4_37617, &P23933_5};	/*Property = ScreenPixelsWidth*/
const Profile P23933 = {"23933", &P23933_4};	/*Profile ID - 23933*/
const Value V21417_1_1 = {S2, NULL};	/*Value = True*/
const Property P21417_1 = {S50, 0, &V21417_1_1, NULL};	/*Property = IsMobile*/
const Value V21417_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21417_5 = {S52, 0, &V21417_5_1458, &P21417_1};	/*Property = ScreenPixelsHeight*/
const Value V21417_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21417_4 = {S54, 0, &V21417_4_1460, &P21417_5};	/*Property = ScreenPixelsWidth*/
const Profile P21417 = {"21417", &P21417_4};	/*Profile ID - 21417*/
const Value V20138_1_1 = {S2, NULL};	/*Value = True*/
const Property P20138_1 = {S50, 0, &V20138_1_1, NULL};	/*Property = IsMobile*/
const Value V20138_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20138_5 = {S52, 0, &V20138_5_1277, &P20138_1};	/*Property = ScreenPixelsHeight*/
const Value V20138_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20138_4 = {S54, 0, &V20138_4_1279, &P20138_5};	/*Property = ScreenPixelsWidth*/
const Profile P20138 = {"20138", &P20138_4};	/*Profile ID - 20138*/
const Value V21257_1_1 = {S2, NULL};	/*Value = True*/
const Property P21257_1 = {S50, 0, &V21257_1_1, NULL};	/*Property = IsMobile*/
const Value V21257_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21257_5 = {S52, 0, &V21257_5_1227, &P21257_1};	/*Property = ScreenPixelsHeight*/
const Value V21257_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21257_4 = {S54, 0, &V21257_4_1229, &P21257_5};	/*Property = ScreenPixelsWidth*/
const Profile P21257 = {"21257", &P21257_4};	/*Profile ID - 21257*/
const Value V19842_1_1 = {S2, NULL};	/*Value = True*/
const Property P19842_1 = {S50, 0, &V19842_1_1, NULL};	/*Property = IsMobile*/
const Value V19842_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P19842_5 = {S52, 0, &V19842_5_59138, &P19842_1};	/*Property = ScreenPixelsHeight*/
const Value V19842_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P19842_4 = {S54, 0, &V19842_4_59970, &P19842_5};	/*Property = ScreenPixelsWidth*/
const Profile P19842 = {"19842", &P19842_4};	/*Profile ID - 19842*/
const Value V21933_1_1 = {S2, NULL};	/*Value = True*/
const Property P21933_1 = {S50, 0, &V21933_1_1, NULL};	/*Property = IsMobile*/
const Value V21933_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21933_5 = {S52, 0, &V21933_5_1277, &P21933_1};	/*Property = ScreenPixelsHeight*/
const Value V21933_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21933_4 = {S54, 0, &V21933_4_2336, &P21933_5};	/*Property = ScreenPixelsWidth*/
const Profile P21933 = {"21933", &P21933_4};	/*Profile ID - 21933*/
const Value V20870_1_1 = {S2, NULL};	/*Value = True*/
const Property P20870_1 = {S50, 0, &V20870_1_1, NULL};	/*Property = IsMobile*/
const Value V20870_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20870_5 = {S52, 0, &V20870_5_1227, &P20870_1};	/*Property = ScreenPixelsHeight*/
const Value V20870_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20870_4 = {S54, 0, &V20870_4_1229, &P20870_5};	/*Property = ScreenPixelsWidth*/
const Profile P20870 = {"20870", &P20870_4};	/*Profile ID - 20870*/
const Value V19812_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_279 = {S1, 0, &V19812_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V19812_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_282 = {S3, 0, &V19812_282_57155, &P19812_279};	/*Property = BlobBuilder*/
const Value V19812_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_350 = {S4, 0, &V19812_350_57446, &P19812_282};	/*Property = Canvas*/
const Value V19812_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_283 = {S5, 0, &V19812_283_57161, &P19812_350};	/*Property = CssBackground*/
const Value V19812_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_284 = {S6, 0, &V19812_284_57164, &P19812_283};	/*Property = CssBorderImage*/
const Value V19812_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_285 = {S7, 0, &V19812_285_57167, &P19812_284};	/*Property = CssCanvas*/
const Value V19812_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_286 = {S8, 0, &V19812_286_57170, &P19812_285};	/*Property = CssColor*/
const Value V19812_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_237 = {S9, 0, &V19812_237_55786, &P19812_286};	/*Property = CssColumn*/
const Value V19812_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_287 = {S10, 0, &V19812_287_57173, &P19812_237};	/*Property = CssFlexbox*/
const Value V19812_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_288 = {S11, 0, &V19812_288_57176, &P19812_287};	/*Property = CssFont*/
const Value V19812_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_289 = {S12, 0, &V19812_289_86899, &P19812_288};	/*Property = CssImages*/
const Value V19812_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_291 = {S13, 0, &V19812_291_57182, &P19812_289};	/*Property = CssMediaQueries*/
const Value V19812_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_292 = {S14, 0, &V19812_292_57185, &P19812_291};	/*Property = CssMinMax*/
const Value V19812_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_293 = {S15, 0, &V19812_293_57188, &P19812_292};	/*Property = CssOverflow*/
const Value V19812_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_294 = {S16, 0, &V19812_294_57191, &P19812_293};	/*Property = CssPosition*/
const Value V19812_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_295 = {S17, 0, &V19812_295_57194, &P19812_294};	/*Property = CssText*/
const Value V19812_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_297 = {S18, 0, &V19812_297_57197, &P19812_295};	/*Property = CssTransforms*/
const Value V19812_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_241 = {S19, 0, &V19812_241_55801, &P19812_297};	/*Property = CssTransitions*/
const Value V19812_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_298 = {S20, 0, &V19812_298_57208, &P19812_241};	/*Property = CssUI*/
const Value V19812_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_300 = {S21, 0, &V19812_300_57217, &P19812_298};	/*Property = DataSet*/
const Value V19812_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_301 = {S22, 0, &V19812_301_57220, &P19812_300};	/*Property = DataUrl*/
const Value V19812_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_302 = {S23, 0, &V19812_302_57223, &P19812_301};	/*Property = DeviceOrientation*/
const Value V19812_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_304 = {S24, 0, &V19812_304_57229, &P19812_302};	/*Property = FileReader*/
const Value V19812_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_305 = {S25, 0, &V19812_305_57232, &P19812_304};	/*Property = FileSaver*/
const Value V19812_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_306 = {S26, 0, &V19812_306_57238, &P19812_305};	/*Property = FileWriter*/
const Value V19812_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_307 = {S27, 0, &V19812_307_57241, &P19812_306};	/*Property = FormData*/
const Value V19812_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_309 = {S28, 0, &V19812_309_57247, &P19812_307};	/*Property = Fullscreen*/
const Value V19812_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_310 = {S29, 0, &V19812_310_57250, &P19812_309};	/*Property = GeoLocation*/
const Value V19812_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_312 = {S30, 0, &V19812_312_57256, &P19812_310};	/*Property = History*/
const Value V19812_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_314 = {S31, 0, &V19812_314_57262, &P19812_312};	/*Property = Html5*/
const Value V19812_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_313 = {S32, 0, &V19812_313_57259, &P19812_314};	/*Property = Html-Media-Capture*/
const Value V19812_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_315 = {S33, 0, &V19812_315_57265, &P19812_313};	/*Property = Iframe*/
const Value V19812_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_316 = {S34, 0, &V19812_316_57268, &P19812_315};	/*Property = IndexedDB*/
const Value V19812_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_317 = {S35, 0, &V19812_317_57271, &P19812_316};	/*Property = Json*/
const Value V19812_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P19812_15 = {S37, 0, &V19812_15_29, &P19812_317};	/*Property = LayoutEngine*/
const Value V19812_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_344 = {S38, 0, &V19812_344_57274, &P19812_15};	/*Property = Masking*/
const Value V19812_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_320 = {S39, 0, &V19812_320_57292, &P19812_344};	/*Property = PostMessage*/
const Value V19812_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_321 = {S40, 0, &V19812_321_57295, &P19812_320};	/*Property = Progress*/
const Value V19812_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_322 = {S41, 0, &V19812_322_57298, &P19812_321};	/*Property = Prompts*/
const Value V19812_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_323 = {S42, 0, &V19812_323_57301, &P19812_322};	/*Property = Selector*/
const Value V19812_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_324 = {S43, 0, &V19812_324_57307, &P19812_323};	/*Property = Svg*/
const Value V19812_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_327 = {S44, 0, &V19812_327_57316, &P19812_324};	/*Property = TouchEvents*/
const Value V19812_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_328 = {S45, 0, &V19812_328_57319, &P19812_327};	/*Property = Track*/
const Value V19812_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_329 = {S46, 0, &V19812_329_57325, &P19812_328};	/*Property = Video*/
const Value V19812_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_330 = {S47, 0, &V19812_330_57328, &P19812_329};	/*Property = Viewport*/
const Value V19812_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_332 = {S48, 0, &V19812_332_57340, &P19812_330};	/*Property = WebWorkers*/
const Value V19812_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P19812_333 = {S49, 0, &V19812_333_57343, &P19812_332};	/*Property = Xhr2*/
const Profile P19812 = {"19812", &P19812_333};	/*Profile ID - 19812*/
const Profile P22211 = {"22211", NULL};	/*Profile ID - 22211*/
const Value V20852_1_1 = {S2, NULL};	/*Value = True*/
const Property P20852_1 = {S50, 0, &V20852_1_1, NULL};	/*Property = IsMobile*/
const Value V20852_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20852_5 = {S52, 0, &V20852_5_1227, &P20852_1};	/*Property = ScreenPixelsHeight*/
const Value V20852_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20852_4 = {S54, 0, &V20852_4_1229, &P20852_5};	/*Property = ScreenPixelsWidth*/
const Profile P20852 = {"20852", &P20852_4};	/*Profile ID - 20852*/
const Value V20955_1_1 = {S2, NULL};	/*Value = True*/
const Property P20955_1 = {S50, 0, &V20955_1_1, NULL};	/*Property = IsMobile*/
const Value V20955_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20955_5 = {S52, 0, &V20955_5_37, &P20955_1};	/*Property = ScreenPixelsHeight*/
const Value V20955_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20955_4 = {S54, 0, &V20955_4_36, &P20955_5};	/*Property = ScreenPixelsWidth*/
const Profile P20955 = {"20955", &P20955_4};	/*Profile ID - 20955*/
const Value V20765_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_279 = {S1, 0, &V20765_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20765_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_282 = {S3, 0, &V20765_282_57155, &P20765_279};	/*Property = BlobBuilder*/
const Value V20765_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_350 = {S4, 0, &V20765_350_57446, &P20765_282};	/*Property = Canvas*/
const Value V20765_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_283 = {S5, 0, &V20765_283_57161, &P20765_350};	/*Property = CssBackground*/
const Value V20765_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_284 = {S6, 0, &V20765_284_57164, &P20765_283};	/*Property = CssBorderImage*/
const Value V20765_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_285 = {S7, 0, &V20765_285_57167, &P20765_284};	/*Property = CssCanvas*/
const Value V20765_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_286 = {S8, 0, &V20765_286_57170, &P20765_285};	/*Property = CssColor*/
const Value V20765_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_237 = {S9, 0, &V20765_237_55786, &P20765_286};	/*Property = CssColumn*/
const Value V20765_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_287 = {S10, 0, &V20765_287_57173, &P20765_237};	/*Property = CssFlexbox*/
const Value V20765_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_288 = {S11, 0, &V20765_288_57176, &P20765_287};	/*Property = CssFont*/
const Value V20765_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_289 = {S12, 0, &V20765_289_86899, &P20765_288};	/*Property = CssImages*/
const Value V20765_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_291 = {S13, 0, &V20765_291_57182, &P20765_289};	/*Property = CssMediaQueries*/
const Value V20765_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_292 = {S14, 0, &V20765_292_57185, &P20765_291};	/*Property = CssMinMax*/
const Value V20765_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_293 = {S15, 0, &V20765_293_57188, &P20765_292};	/*Property = CssOverflow*/
const Value V20765_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_294 = {S16, 0, &V20765_294_57191, &P20765_293};	/*Property = CssPosition*/
const Value V20765_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_295 = {S17, 0, &V20765_295_57194, &P20765_294};	/*Property = CssText*/
const Value V20765_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_297 = {S18, 0, &V20765_297_57197, &P20765_295};	/*Property = CssTransforms*/
const Value V20765_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_241 = {S19, 0, &V20765_241_55801, &P20765_297};	/*Property = CssTransitions*/
const Value V20765_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_298 = {S20, 0, &V20765_298_57208, &P20765_241};	/*Property = CssUI*/
const Value V20765_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_300 = {S21, 0, &V20765_300_57217, &P20765_298};	/*Property = DataSet*/
const Value V20765_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_301 = {S22, 0, &V20765_301_57220, &P20765_300};	/*Property = DataUrl*/
const Value V20765_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_302 = {S23, 0, &V20765_302_57223, &P20765_301};	/*Property = DeviceOrientation*/
const Value V20765_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_304 = {S24, 0, &V20765_304_57229, &P20765_302};	/*Property = FileReader*/
const Value V20765_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_305 = {S25, 0, &V20765_305_57232, &P20765_304};	/*Property = FileSaver*/
const Value V20765_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_306 = {S26, 0, &V20765_306_57238, &P20765_305};	/*Property = FileWriter*/
const Value V20765_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_307 = {S27, 0, &V20765_307_57241, &P20765_306};	/*Property = FormData*/
const Value V20765_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_309 = {S28, 0, &V20765_309_57247, &P20765_307};	/*Property = Fullscreen*/
const Value V20765_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_310 = {S29, 0, &V20765_310_57250, &P20765_309};	/*Property = GeoLocation*/
const Value V20765_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_312 = {S30, 0, &V20765_312_57256, &P20765_310};	/*Property = History*/
const Value V20765_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_314 = {S31, 0, &V20765_314_57262, &P20765_312};	/*Property = Html5*/
const Value V20765_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_313 = {S32, 0, &V20765_313_57259, &P20765_314};	/*Property = Html-Media-Capture*/
const Value V20765_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_315 = {S33, 0, &V20765_315_57265, &P20765_313};	/*Property = Iframe*/
const Value V20765_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_316 = {S34, 0, &V20765_316_57268, &P20765_315};	/*Property = IndexedDB*/
const Value V20765_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_317 = {S35, 0, &V20765_317_57271, &P20765_316};	/*Property = Json*/
const Value V20765_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P20765_15 = {S37, 0, &V20765_15_90837, &P20765_317};	/*Property = LayoutEngine*/
const Value V20765_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_344 = {S38, 0, &V20765_344_57274, &P20765_15};	/*Property = Masking*/
const Value V20765_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_320 = {S39, 0, &V20765_320_57292, &P20765_344};	/*Property = PostMessage*/
const Value V20765_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_321 = {S40, 0, &V20765_321_57295, &P20765_320};	/*Property = Progress*/
const Value V20765_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_322 = {S41, 0, &V20765_322_57298, &P20765_321};	/*Property = Prompts*/
const Value V20765_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_323 = {S42, 0, &V20765_323_57301, &P20765_322};	/*Property = Selector*/
const Value V20765_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_324 = {S43, 0, &V20765_324_57307, &P20765_323};	/*Property = Svg*/
const Value V20765_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_327 = {S44, 0, &V20765_327_57316, &P20765_324};	/*Property = TouchEvents*/
const Value V20765_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_328 = {S45, 0, &V20765_328_57319, &P20765_327};	/*Property = Track*/
const Value V20765_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_329 = {S46, 0, &V20765_329_57325, &P20765_328};	/*Property = Video*/
const Value V20765_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_330 = {S47, 0, &V20765_330_57328, &P20765_329};	/*Property = Viewport*/
const Value V20765_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_332 = {S48, 0, &V20765_332_57340, &P20765_330};	/*Property = WebWorkers*/
const Value V20765_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20765_333 = {S49, 0, &V20765_333_57343, &P20765_332};	/*Property = Xhr2*/
const Profile P20765 = {"20765", &P20765_333};	/*Profile ID - 20765*/
const Value V20002_1_1 = {S2, NULL};	/*Value = True*/
const Property P20002_1 = {S50, 0, &V20002_1_1, NULL};	/*Property = IsMobile*/
const Value V20002_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20002_5 = {S52, 0, &V20002_5_1227, &P20002_1};	/*Property = ScreenPixelsHeight*/
const Value V20002_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20002_4 = {S54, 0, &V20002_4_1229, &P20002_5};	/*Property = ScreenPixelsWidth*/
const Profile P20002 = {"20002", &P20002_4};	/*Profile ID - 20002*/
const Value V20069_1_1 = {S2, NULL};	/*Value = True*/
const Property P20069_1 = {S50, 0, &V20069_1_1, NULL};	/*Property = IsMobile*/
const Value V20069_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20069_5 = {S52, 0, &V20069_5_1222, &P20069_1};	/*Property = ScreenPixelsHeight*/
const Value V20069_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20069_4 = {S54, 0, &V20069_4_1224, &P20069_5};	/*Property = ScreenPixelsWidth*/
const Profile P20069 = {"20069", &P20069_4};	/*Profile ID - 20069*/
const Value V20425_1_1 = {S2, NULL};	/*Value = True*/
const Property P20425_1 = {S50, 0, &V20425_1_1, NULL};	/*Property = IsMobile*/
const Value V20425_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20425_5 = {S52, 0, &V20425_5_9981, &P20425_1};	/*Property = ScreenPixelsHeight*/
const Value V20425_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20425_4 = {S54, 0, &V20425_4_39098, &P20425_5};	/*Property = ScreenPixelsWidth*/
const Profile P20425 = {"20425", &P20425_4};	/*Profile ID - 20425*/
const Value V20438_1_1 = {S2, NULL};	/*Value = True*/
const Property P20438_1 = {S50, 0, &V20438_1_1, NULL};	/*Property = IsMobile*/
const Value V20438_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20438_5 = {S52, 0, &V20438_5_1227, &P20438_1};	/*Property = ScreenPixelsHeight*/
const Value V20438_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P20438_4 = {S54, 0, &V20438_4_1943, &P20438_5};	/*Property = ScreenPixelsWidth*/
const Profile P20438 = {"20438", &P20438_4};	/*Profile ID - 20438*/
const Value V21788_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_279 = {S1, 0, &V21788_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21788_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_282 = {S3, 0, &V21788_282_57155, &P21788_279};	/*Property = BlobBuilder*/
const Value V21788_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_350 = {S4, 0, &V21788_350_57446, &P21788_282};	/*Property = Canvas*/
const Value V21788_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_283 = {S5, 0, &V21788_283_57161, &P21788_350};	/*Property = CssBackground*/
const Value V21788_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_284 = {S6, 0, &V21788_284_57164, &P21788_283};	/*Property = CssBorderImage*/
const Value V21788_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_285 = {S7, 0, &V21788_285_57167, &P21788_284};	/*Property = CssCanvas*/
const Value V21788_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_286 = {S8, 0, &V21788_286_57170, &P21788_285};	/*Property = CssColor*/
const Value V21788_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_237 = {S9, 0, &V21788_237_55786, &P21788_286};	/*Property = CssColumn*/
const Value V21788_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_287 = {S10, 0, &V21788_287_57173, &P21788_237};	/*Property = CssFlexbox*/
const Value V21788_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_288 = {S11, 0, &V21788_288_57176, &P21788_287};	/*Property = CssFont*/
const Value V21788_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_289 = {S12, 0, &V21788_289_86899, &P21788_288};	/*Property = CssImages*/
const Value V21788_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_291 = {S13, 0, &V21788_291_57182, &P21788_289};	/*Property = CssMediaQueries*/
const Value V21788_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_292 = {S14, 0, &V21788_292_57185, &P21788_291};	/*Property = CssMinMax*/
const Value V21788_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_293 = {S15, 0, &V21788_293_57188, &P21788_292};	/*Property = CssOverflow*/
const Value V21788_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_294 = {S16, 0, &V21788_294_57191, &P21788_293};	/*Property = CssPosition*/
const Value V21788_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_295 = {S17, 0, &V21788_295_57194, &P21788_294};	/*Property = CssText*/
const Value V21788_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_297 = {S18, 0, &V21788_297_57197, &P21788_295};	/*Property = CssTransforms*/
const Value V21788_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_241 = {S19, 0, &V21788_241_55801, &P21788_297};	/*Property = CssTransitions*/
const Value V21788_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_298 = {S20, 0, &V21788_298_57208, &P21788_241};	/*Property = CssUI*/
const Value V21788_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_300 = {S21, 0, &V21788_300_57217, &P21788_298};	/*Property = DataSet*/
const Value V21788_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_301 = {S22, 0, &V21788_301_57220, &P21788_300};	/*Property = DataUrl*/
const Value V21788_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_302 = {S23, 0, &V21788_302_57223, &P21788_301};	/*Property = DeviceOrientation*/
const Value V21788_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_304 = {S24, 0, &V21788_304_57229, &P21788_302};	/*Property = FileReader*/
const Value V21788_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_305 = {S25, 0, &V21788_305_57232, &P21788_304};	/*Property = FileSaver*/
const Value V21788_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_306 = {S26, 0, &V21788_306_57238, &P21788_305};	/*Property = FileWriter*/
const Value V21788_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_307 = {S27, 0, &V21788_307_57241, &P21788_306};	/*Property = FormData*/
const Value V21788_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_309 = {S28, 0, &V21788_309_57247, &P21788_307};	/*Property = Fullscreen*/
const Value V21788_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_310 = {S29, 0, &V21788_310_57250, &P21788_309};	/*Property = GeoLocation*/
const Value V21788_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_312 = {S30, 0, &V21788_312_57256, &P21788_310};	/*Property = History*/
const Value V21788_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_314 = {S31, 0, &V21788_314_57262, &P21788_312};	/*Property = Html5*/
const Value V21788_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_313 = {S32, 0, &V21788_313_57259, &P21788_314};	/*Property = Html-Media-Capture*/
const Value V21788_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_315 = {S33, 0, &V21788_315_57265, &P21788_313};	/*Property = Iframe*/
const Value V21788_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_316 = {S34, 0, &V21788_316_57268, &P21788_315};	/*Property = IndexedDB*/
const Value V21788_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_317 = {S35, 0, &V21788_317_57271, &P21788_316};	/*Property = Json*/
const Value V21788_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_15 = {S37, 0, &V21788_15_72, &P21788_317};	/*Property = LayoutEngine*/
const Value V21788_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_344 = {S38, 0, &V21788_344_57274, &P21788_15};	/*Property = Masking*/
const Value V21788_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_320 = {S39, 0, &V21788_320_57292, &P21788_344};	/*Property = PostMessage*/
const Value V21788_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_321 = {S40, 0, &V21788_321_57295, &P21788_320};	/*Property = Progress*/
const Value V21788_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_322 = {S41, 0, &V21788_322_57298, &P21788_321};	/*Property = Prompts*/
const Value V21788_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_323 = {S42, 0, &V21788_323_57301, &P21788_322};	/*Property = Selector*/
const Value V21788_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_324 = {S43, 0, &V21788_324_57307, &P21788_323};	/*Property = Svg*/
const Value V21788_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_327 = {S44, 0, &V21788_327_57316, &P21788_324};	/*Property = TouchEvents*/
const Value V21788_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_328 = {S45, 0, &V21788_328_57319, &P21788_327};	/*Property = Track*/
const Value V21788_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_329 = {S46, 0, &V21788_329_57325, &P21788_328};	/*Property = Video*/
const Value V21788_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_330 = {S47, 0, &V21788_330_57328, &P21788_329};	/*Property = Viewport*/
const Value V21788_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_332 = {S48, 0, &V21788_332_57340, &P21788_330};	/*Property = WebWorkers*/
const Value V21788_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21788_333 = {S49, 0, &V21788_333_57343, &P21788_332};	/*Property = Xhr2*/
const Profile P21788 = {"21788", &P21788_333};	/*Profile ID - 21788*/
const Value V20264_1_1 = {S2, NULL};	/*Value = True*/
const Property P20264_1 = {S50, 0, &V20264_1_1, NULL};	/*Property = IsMobile*/
const Value V20264_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20264_5 = {S52, 0, &V20264_5_1458, &P20264_1};	/*Property = ScreenPixelsHeight*/
const Value V20264_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20264_4 = {S54, 0, &V20264_4_37617, &P20264_5};	/*Property = ScreenPixelsWidth*/
const Profile P20264 = {"20264", &P20264_4};	/*Profile ID - 20264*/
const Value V23156_1_1 = {S2, NULL};	/*Value = True*/
const Property P23156_1 = {S50, 0, &V23156_1_1, NULL};	/*Property = IsMobile*/
const Value V23156_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23156_5 = {S52, 0, &V23156_5_1403, &P23156_1};	/*Property = ScreenPixelsHeight*/
const Value V23156_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23156_4 = {S54, 0, &V23156_4_1404, &P23156_5};	/*Property = ScreenPixelsWidth*/
const Profile P23156 = {"23156", &P23156_4};	/*Profile ID - 23156*/
const Value V20510_1_1 = {S2, NULL};	/*Value = True*/
const Property P20510_1 = {S50, 0, &V20510_1_1, NULL};	/*Property = IsMobile*/
const Value V20510_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20510_5 = {S52, 0, &V20510_5_1227, &P20510_1};	/*Property = ScreenPixelsHeight*/
const Value V20510_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20510_4 = {S54, 0, &V20510_4_1229, &P20510_5};	/*Property = ScreenPixelsWidth*/
const Profile P20510 = {"20510", &P20510_4};	/*Profile ID - 20510*/
const Value V19851_1_1 = {S2, NULL};	/*Value = True*/
const Property P19851_1 = {S50, 0, &V19851_1_1, NULL};	/*Property = IsMobile*/
const Value V19851_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19851_5 = {S52, 0, &V19851_5_1227, &P19851_1};	/*Property = ScreenPixelsHeight*/
const Value V19851_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19851_4 = {S54, 0, &V19851_4_1229, &P19851_5};	/*Property = ScreenPixelsWidth*/
const Profile P19851 = {"19851", &P19851_4};	/*Profile ID - 19851*/
const Value V23881_1_1 = {S2, NULL};	/*Value = True*/
const Property P23881_1 = {S50, 0, &V23881_1_1, NULL};	/*Property = IsMobile*/
const Value V23881_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23881_5 = {S52, 0, &V23881_5_1832, &P23881_1};	/*Property = ScreenPixelsHeight*/
const Value V23881_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23881_4 = {S54, 0, &V23881_4_5645, &P23881_5};	/*Property = ScreenPixelsWidth*/
const Profile P23881 = {"23881", &P23881_4};	/*Profile ID - 23881*/
const Value V19963_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19963_279 = {S1, 0, &V19963_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19963_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19963_282 = {S3, 0, &V19963_282_57154, &P19963_279};	/*Property = BlobBuilder*/
const Value V19963_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19963_350 = {S4, 0, &V19963_350_57444, &P19963_282};	/*Property = Canvas*/
const Value V19963_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19963_283 = {S5, 0, &V19963_283_57159, &P19963_350};	/*Property = CssBackground*/
const Value V19963_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19963_284 = {S6, 0, &V19963_284_57163, &P19963_283};	/*Property = CssBorderImage*/
const Value V19963_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19963_285 = {S7, 0, &V19963_285_57166, &P19963_284};	/*Property = CssCanvas*/
const Value V19963_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19963_286 = {S8, 0, &V19963_286_57168, &P19963_285};	/*Property = CssColor*/
const Value V19963_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19963_237 = {S9, 0, &V19963_237_55786, &P19963_286};	/*Property = CssColumn*/
const Value V19963_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19963_287 = {S10, 0, &V19963_287_57172, &P19963_237};	/*Property = CssFlexbox*/
const Value V19963_288_57174 = {S2, NULL};	/*Value = True*/
const Property P19963_288 = {S11, 0, &V19963_288_57174, &P19963_287};	/*Property = CssFont*/
const Value V19963_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19963_289 = {S12, 0, &V19963_289_57759, &P19963_288};	/*Property = CssImages*/
const Value V19963_291_57180 = {S2, NULL};	/*Value = True*/
const Property P19963_291 = {S13, 0, &V19963_291_57180, &P19963_289};	/*Property = CssMediaQueries*/
const Value V19963_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19963_292 = {S14, 0, &V19963_292_57183, &P19963_291};	/*Property = CssMinMax*/
const Value V19963_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19963_293 = {S15, 0, &V19963_293_57187, &P19963_292};	/*Property = CssOverflow*/
const Value V19963_294_57189 = {S2, NULL};	/*Value = True*/
const Property P19963_294 = {S16, 0, &V19963_294_57189, &P19963_293};	/*Property = CssPosition*/
const Value V19963_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19963_295 = {S17, 0, &V19963_295_57192, &P19963_294};	/*Property = CssText*/
const Value V19963_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19963_297 = {S18, 0, &V19963_297_57195, &P19963_295};	/*Property = CssTransforms*/
const Value V19963_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19963_241 = {S19, 0, &V19963_241_55799, &P19963_297};	/*Property = CssTransitions*/
const Value V19963_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19963_298 = {S20, 0, &V19963_298_57206, &P19963_241};	/*Property = CssUI*/
const Value V19963_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19963_300 = {S21, 0, &V19963_300_57216, &P19963_298};	/*Property = DataSet*/
const Value V19963_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19963_301 = {S22, 0, &V19963_301_57218, &P19963_300};	/*Property = DataUrl*/
const Value V19963_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19963_302 = {S23, 0, &V19963_302_57222, &P19963_301};	/*Property = DeviceOrientation*/
const Value V19963_304_57227 = {S2, NULL};	/*Value = True*/
const Property P19963_304 = {S24, 0, &V19963_304_57227, &P19963_302};	/*Property = FileReader*/
const Value V19963_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19963_305 = {S25, 0, &V19963_305_57231, &P19963_304};	/*Property = FileSaver*/
const Value V19963_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19963_306 = {S26, 0, &V19963_306_57237, &P19963_305};	/*Property = FileWriter*/
const Value V19963_307_57239 = {S2, NULL};	/*Value = True*/
const Property P19963_307 = {S27, 0, &V19963_307_57239, &P19963_306};	/*Property = FormData*/
const Value V19963_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19963_309 = {S28, 0, &V19963_309_57246, &P19963_307};	/*Property = Fullscreen*/
const Value V19963_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19963_310 = {S29, 0, &V19963_310_57248, &P19963_309};	/*Property = GeoLocation*/
const Value V19963_312_57254 = {S2, NULL};	/*Value = True*/
const Property P19963_312 = {S30, 0, &V19963_312_57254, &P19963_310};	/*Property = History*/
const Value V19963_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19963_314 = {S31, 0, &V19963_314_57261, &P19963_312};	/*Property = Html5*/
const Value V19963_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19963_313 = {S32, 0, &V19963_313_57258, &P19963_314};	/*Property = Html-Media-Capture*/
const Value V19963_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19963_315 = {S33, 0, &V19963_315_57264, &P19963_313};	/*Property = Iframe*/
const Value V19963_316_57266 = {S2, NULL};	/*Value = True*/
const Property P19963_316 = {S34, 0, &V19963_316_57266, &P19963_315};	/*Property = IndexedDB*/
const Value V19963_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19963_317 = {S35, 0, &V19963_317_57269, &P19963_316};	/*Property = Json*/
const Value V19963_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P19963_15 = {S37, 0, &V19963_15_30, &P19963_317};	/*Property = LayoutEngine*/
const Value V19963_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19963_344 = {S38, 0, &V19963_344_57272, &P19963_15};	/*Property = Masking*/
const Value V19963_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19963_320 = {S39, 0, &V19963_320_57290, &P19963_344};	/*Property = PostMessage*/
const Value V19963_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19963_321 = {S40, 0, &V19963_321_57293, &P19963_320};	/*Property = Progress*/
const Value V19963_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19963_322 = {S41, 0, &V19963_322_57296, &P19963_321};	/*Property = Prompts*/
const Value V19963_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19963_323 = {S42, 0, &V19963_323_57299, &P19963_322};	/*Property = Selector*/
const Value V19963_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19963_324 = {S43, 0, &V19963_324_57306, &P19963_323};	/*Property = Svg*/
const Value V19963_327_57315 = {S0, NULL};	/*Value = False*/
const Property P19963_327 = {S44, 0, &V19963_327_57315, &P19963_324};	/*Property = TouchEvents*/
const Value V19963_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19963_328 = {S45, 0, &V19963_328_57318, &P19963_327};	/*Property = Track*/
const Value V19963_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19963_329 = {S46, 0, &V19963_329_57323, &P19963_328};	/*Property = Video*/
const Value V19963_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19963_330 = {S47, 0, &V19963_330_57326, &P19963_329};	/*Property = Viewport*/
const Value V19963_332_57338 = {S2, NULL};	/*Value = True*/
const Property P19963_332 = {S48, 0, &V19963_332_57338, &P19963_330};	/*Property = WebWorkers*/
const Value V19963_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P19963_333 = {S49, 0, &V19963_333_57343, &P19963_332};	/*Property = Xhr2*/
const Profile P19963 = {"19963", &P19963_333};	/*Profile ID - 19963*/
const Value V19748_1_1 = {S2, NULL};	/*Value = True*/
const Property P19748_1 = {S50, 0, &V19748_1_1, NULL};	/*Property = IsMobile*/
const Value V19748_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P19748_5 = {S52, 0, &V19748_5_1380, &P19748_1};	/*Property = ScreenPixelsHeight*/
const Value V19748_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P19748_4 = {S54, 0, &V19748_4_1279, &P19748_5};	/*Property = ScreenPixelsWidth*/
const Profile P19748 = {"19748", &P19748_4};	/*Profile ID - 19748*/
const Value V22678_1_1 = {S2, NULL};	/*Value = True*/
const Property P22678_1 = {S50, 0, &V22678_1_1, NULL};	/*Property = IsMobile*/
const Value V22678_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P22678_5 = {S52, 0, &V22678_5_37, &P22678_1};	/*Property = ScreenPixelsHeight*/
const Value V22678_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P22678_4 = {S54, 0, &V22678_4_36, &P22678_5};	/*Property = ScreenPixelsWidth*/
const Profile P22678 = {"22678", &P22678_4};	/*Profile ID - 22678*/
const Value V20963_1_1 = {S2, NULL};	/*Value = True*/
const Property P20963_1 = {S50, 0, &V20963_1_1, NULL};	/*Property = IsMobile*/
const Value V20963_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20963_5 = {S52, 0, &V20963_5_9981, &P20963_1};	/*Property = ScreenPixelsHeight*/
const Value V20963_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20963_4 = {S54, 0, &V20963_4_39098, &P20963_5};	/*Property = ScreenPixelsWidth*/
const Profile P20963 = {"20963", &P20963_4};	/*Profile ID - 20963*/
const Profile P19825 = {"19825", NULL};	/*Profile ID - 19825*/
const Value V22326_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22326_279 = {S1, 0, &V22326_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22326_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22326_282 = {S3, 0, &V22326_282_57153, &P22326_279};	/*Property = BlobBuilder*/
const Value V22326_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22326_350 = {S4, 0, &V22326_350_57444, &P22326_282};	/*Property = Canvas*/
const Value V22326_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22326_283 = {S5, 0, &V22326_283_57159, &P22326_350};	/*Property = CssBackground*/
const Value V22326_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22326_284 = {S6, 0, &V22326_284_57163, &P22326_283};	/*Property = CssBorderImage*/
const Value V22326_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22326_285 = {S7, 0, &V22326_285_57166, &P22326_284};	/*Property = CssCanvas*/
const Value V22326_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22326_286 = {S8, 0, &V22326_286_57168, &P22326_285};	/*Property = CssColor*/
const Value V22326_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22326_237 = {S9, 0, &V22326_237_55784, &P22326_286};	/*Property = CssColumn*/
const Value V22326_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22326_287 = {S10, 0, &V22326_287_57172, &P22326_237};	/*Property = CssFlexbox*/
const Value V22326_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22326_288 = {S11, 0, &V22326_288_57175, &P22326_287};	/*Property = CssFont*/
const Value V22326_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22326_289 = {S12, 0, &V22326_289_57759, &P22326_288};	/*Property = CssImages*/
const Value V22326_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22326_291 = {S13, 0, &V22326_291_57180, &P22326_289};	/*Property = CssMediaQueries*/
const Value V22326_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22326_292 = {S14, 0, &V22326_292_57183, &P22326_291};	/*Property = CssMinMax*/
const Value V22326_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22326_293 = {S15, 0, &V22326_293_57187, &P22326_292};	/*Property = CssOverflow*/
const Value V22326_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22326_294 = {S16, 0, &V22326_294_57189, &P22326_293};	/*Property = CssPosition*/
const Value V22326_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22326_295 = {S17, 0, &V22326_295_57192, &P22326_294};	/*Property = CssText*/
const Value V22326_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22326_297 = {S18, 0, &V22326_297_57195, &P22326_295};	/*Property = CssTransforms*/
const Value V22326_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22326_241 = {S19, 0, &V22326_241_55799, &P22326_297};	/*Property = CssTransitions*/
const Value V22326_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22326_298 = {S20, 0, &V22326_298_57206, &P22326_241};	/*Property = CssUI*/
const Value V22326_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22326_300 = {S21, 0, &V22326_300_57216, &P22326_298};	/*Property = DataSet*/
const Value V22326_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22326_301 = {S22, 0, &V22326_301_57218, &P22326_300};	/*Property = DataUrl*/
const Value V22326_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22326_302 = {S23, 0, &V22326_302_57222, &P22326_301};	/*Property = DeviceOrientation*/
const Value V22326_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22326_304 = {S24, 0, &V22326_304_57227, &P22326_302};	/*Property = FileReader*/
const Value V22326_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22326_305 = {S25, 0, &V22326_305_57231, &P22326_304};	/*Property = FileSaver*/
const Value V22326_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22326_306 = {S26, 0, &V22326_306_57237, &P22326_305};	/*Property = FileWriter*/
const Value V22326_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22326_307 = {S27, 0, &V22326_307_57239, &P22326_306};	/*Property = FormData*/
const Value V22326_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22326_309 = {S28, 0, &V22326_309_57246, &P22326_307};	/*Property = Fullscreen*/
const Value V22326_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22326_310 = {S29, 0, &V22326_310_57248, &P22326_309};	/*Property = GeoLocation*/
const Value V22326_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22326_312 = {S30, 0, &V22326_312_57255, &P22326_310};	/*Property = History*/
const Value V22326_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22326_314 = {S31, 0, &V22326_314_57261, &P22326_312};	/*Property = Html5*/
const Value V22326_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22326_313 = {S32, 0, &V22326_313_57258, &P22326_314};	/*Property = Html-Media-Capture*/
const Value V22326_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22326_315 = {S33, 0, &V22326_315_57264, &P22326_313};	/*Property = Iframe*/
const Value V22326_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22326_316 = {S34, 0, &V22326_316_57266, &P22326_315};	/*Property = IndexedDB*/
const Value V22326_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22326_317 = {S35, 0, &V22326_317_57269, &P22326_316};	/*Property = Json*/
const Value V22326_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22326_15 = {S37, 0, &V22326_15_28, &P22326_317};	/*Property = LayoutEngine*/
const Value V22326_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22326_344 = {S38, 0, &V22326_344_57272, &P22326_15};	/*Property = Masking*/
const Value V22326_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22326_320 = {S39, 0, &V22326_320_57290, &P22326_344};	/*Property = PostMessage*/
const Value V22326_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22326_321 = {S40, 0, &V22326_321_57293, &P22326_320};	/*Property = Progress*/
const Value V22326_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22326_322 = {S41, 0, &V22326_322_57296, &P22326_321};	/*Property = Prompts*/
const Value V22326_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22326_323 = {S42, 0, &V22326_323_57299, &P22326_322};	/*Property = Selector*/
const Value V22326_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22326_324 = {S43, 0, &V22326_324_57306, &P22326_323};	/*Property = Svg*/
const Value V22326_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22326_327 = {S44, 0, &V22326_327_57315, &P22326_324};	/*Property = TouchEvents*/
const Value V22326_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22326_328 = {S45, 0, &V22326_328_57318, &P22326_327};	/*Property = Track*/
const Value V22326_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22326_329 = {S46, 0, &V22326_329_57323, &P22326_328};	/*Property = Video*/
const Value V22326_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22326_330 = {S47, 0, &V22326_330_57326, &P22326_329};	/*Property = Viewport*/
const Value V22326_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22326_332 = {S48, 0, &V22326_332_57338, &P22326_330};	/*Property = WebWorkers*/
const Value V22326_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22326_333 = {S49, 0, &V22326_333_57342, &P22326_332};	/*Property = Xhr2*/
const Profile P22326 = {"22326", &P22326_333};	/*Profile ID - 22326*/
const Value V20520_1_1 = {S2, NULL};	/*Value = True*/
const Property P20520_1 = {S50, 0, &V20520_1_1, NULL};	/*Property = IsMobile*/
const Value V20520_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20520_5 = {S52, 0, &V20520_5_1277, &P20520_1};	/*Property = ScreenPixelsHeight*/
const Value V20520_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20520_4 = {S54, 0, &V20520_4_1279, &P20520_5};	/*Property = ScreenPixelsWidth*/
const Profile P20520 = {"20520", &P20520_4};	/*Profile ID - 20520*/
const Value V22562_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22562_279 = {S1, 0, &V22562_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22562_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22562_282 = {S3, 0, &V22562_282_57153, &P22562_279};	/*Property = BlobBuilder*/
const Value V22562_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22562_350 = {S4, 0, &V22562_350_57444, &P22562_282};	/*Property = Canvas*/
const Value V22562_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22562_283 = {S5, 0, &V22562_283_57159, &P22562_350};	/*Property = CssBackground*/
const Value V22562_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22562_284 = {S6, 0, &V22562_284_57163, &P22562_283};	/*Property = CssBorderImage*/
const Value V22562_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22562_285 = {S7, 0, &V22562_285_57166, &P22562_284};	/*Property = CssCanvas*/
const Value V22562_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22562_286 = {S8, 0, &V22562_286_57168, &P22562_285};	/*Property = CssColor*/
const Value V22562_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22562_237 = {S9, 0, &V22562_237_55786, &P22562_286};	/*Property = CssColumn*/
const Value V22562_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22562_287 = {S10, 0, &V22562_287_57172, &P22562_237};	/*Property = CssFlexbox*/
const Value V22562_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22562_288 = {S11, 0, &V22562_288_57175, &P22562_287};	/*Property = CssFont*/
const Value V22562_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22562_289 = {S12, 0, &V22562_289_57759, &P22562_288};	/*Property = CssImages*/
const Value V22562_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22562_291 = {S13, 0, &V22562_291_57181, &P22562_289};	/*Property = CssMediaQueries*/
const Value V22562_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22562_292 = {S14, 0, &V22562_292_57183, &P22562_291};	/*Property = CssMinMax*/
const Value V22562_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22562_293 = {S15, 0, &V22562_293_57187, &P22562_292};	/*Property = CssOverflow*/
const Value V22562_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22562_294 = {S16, 0, &V22562_294_57189, &P22562_293};	/*Property = CssPosition*/
const Value V22562_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22562_295 = {S17, 0, &V22562_295_57192, &P22562_294};	/*Property = CssText*/
const Value V22562_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22562_297 = {S18, 0, &V22562_297_57195, &P22562_295};	/*Property = CssTransforms*/
const Value V22562_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22562_241 = {S19, 0, &V22562_241_55799, &P22562_297};	/*Property = CssTransitions*/
const Value V22562_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22562_298 = {S20, 0, &V22562_298_57206, &P22562_241};	/*Property = CssUI*/
const Value V22562_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22562_300 = {S21, 0, &V22562_300_57216, &P22562_298};	/*Property = DataSet*/
const Value V22562_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22562_301 = {S22, 0, &V22562_301_57218, &P22562_300};	/*Property = DataUrl*/
const Value V22562_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22562_302 = {S23, 0, &V22562_302_57221, &P22562_301};	/*Property = DeviceOrientation*/
const Value V22562_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22562_304 = {S24, 0, &V22562_304_57227, &P22562_302};	/*Property = FileReader*/
const Value V22562_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22562_305 = {S25, 0, &V22562_305_57231, &P22562_304};	/*Property = FileSaver*/
const Value V22562_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22562_306 = {S26, 0, &V22562_306_57237, &P22562_305};	/*Property = FileWriter*/
const Value V22562_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22562_307 = {S27, 0, &V22562_307_57239, &P22562_306};	/*Property = FormData*/
const Value V22562_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22562_309 = {S28, 0, &V22562_309_57246, &P22562_307};	/*Property = Fullscreen*/
const Value V22562_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22562_310 = {S29, 0, &V22562_310_57248, &P22562_309};	/*Property = GeoLocation*/
const Value V22562_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22562_312 = {S30, 0, &V22562_312_57254, &P22562_310};	/*Property = History*/
const Value V22562_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22562_314 = {S31, 0, &V22562_314_57261, &P22562_312};	/*Property = Html5*/
const Value V22562_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22562_313 = {S32, 0, &V22562_313_57258, &P22562_314};	/*Property = Html-Media-Capture*/
const Value V22562_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22562_315 = {S33, 0, &V22562_315_57264, &P22562_313};	/*Property = Iframe*/
const Value V22562_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22562_316 = {S34, 0, &V22562_316_57266, &P22562_315};	/*Property = IndexedDB*/
const Value V22562_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22562_317 = {S35, 0, &V22562_317_57269, &P22562_316};	/*Property = Json*/
const Value V22562_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22562_15 = {S37, 0, &V22562_15_29, &P22562_317};	/*Property = LayoutEngine*/
const Value V22562_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22562_344 = {S38, 0, &V22562_344_57272, &P22562_15};	/*Property = Masking*/
const Value V22562_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22562_320 = {S39, 0, &V22562_320_57290, &P22562_344};	/*Property = PostMessage*/
const Value V22562_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22562_321 = {S40, 0, &V22562_321_57293, &P22562_320};	/*Property = Progress*/
const Value V22562_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22562_322 = {S41, 0, &V22562_322_57296, &P22562_321};	/*Property = Prompts*/
const Value V22562_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22562_323 = {S42, 0, &V22562_323_57299, &P22562_322};	/*Property = Selector*/
const Value V22562_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22562_324 = {S43, 0, &V22562_324_57306, &P22562_323};	/*Property = Svg*/
const Value V22562_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22562_327 = {S44, 0, &V22562_327_57315, &P22562_324};	/*Property = TouchEvents*/
const Value V22562_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22562_328 = {S45, 0, &V22562_328_57318, &P22562_327};	/*Property = Track*/
const Value V22562_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22562_329 = {S46, 0, &V22562_329_57323, &P22562_328};	/*Property = Video*/
const Value V22562_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22562_330 = {S47, 0, &V22562_330_57326, &P22562_329};	/*Property = Viewport*/
const Value V22562_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22562_332 = {S48, 0, &V22562_332_57339, &P22562_330};	/*Property = WebWorkers*/
const Value V22562_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22562_333 = {S49, 0, &V22562_333_57341, &P22562_332};	/*Property = Xhr2*/
const Profile P22562 = {"22562", &P22562_333};	/*Profile ID - 22562*/
const Profile P24702 = {"24702", NULL};	/*Profile ID - 24702*/
const Value V20200_1_1 = {S2, NULL};	/*Value = True*/
const Property P20200_1 = {S50, 0, &V20200_1_1, NULL};	/*Property = IsMobile*/
const Value V20200_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20200_5 = {S52, 0, &V20200_5_1458, &P20200_1};	/*Property = ScreenPixelsHeight*/
const Value V20200_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20200_4 = {S54, 0, &V20200_4_1460, &P20200_5};	/*Property = ScreenPixelsWidth*/
const Profile P20200 = {"20200", &P20200_4};	/*Profile ID - 20200*/
const Value V20984_1_1 = {S2, NULL};	/*Value = True*/
const Property P20984_1 = {S50, 0, &V20984_1_1, NULL};	/*Property = IsMobile*/
const Value V20984_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20984_5 = {S52, 0, &V20984_5_1227, &P20984_1};	/*Property = ScreenPixelsHeight*/
const Value V20984_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20984_4 = {S54, 0, &V20984_4_1229, &P20984_5};	/*Property = ScreenPixelsWidth*/
const Profile P20984 = {"20984", &P20984_4};	/*Profile ID - 20984*/
const Profile P22882 = {"22882", NULL};	/*Profile ID - 22882*/
const Value V23883_1_1 = {S2, NULL};	/*Value = True*/
const Property P23883_1 = {S50, 0, &V23883_1_1, NULL};	/*Property = IsMobile*/
const Value V23883_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23883_5 = {S52, 0, &V23883_5_37569, &P23883_1};	/*Property = ScreenPixelsHeight*/
const Value V23883_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23883_4 = {S54, 0, &V23883_4_5645, &P23883_5};	/*Property = ScreenPixelsWidth*/
const Profile P23883 = {"23883", &P23883_4};	/*Profile ID - 23883*/
const Value V23080_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_279 = {S1, 0, &V23080_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23080_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_282 = {S3, 0, &V23080_282_57155, &P23080_279};	/*Property = BlobBuilder*/
const Value V23080_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_350 = {S4, 0, &V23080_350_57446, &P23080_282};	/*Property = Canvas*/
const Value V23080_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_283 = {S5, 0, &V23080_283_57161, &P23080_350};	/*Property = CssBackground*/
const Value V23080_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_284 = {S6, 0, &V23080_284_57164, &P23080_283};	/*Property = CssBorderImage*/
const Value V23080_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_285 = {S7, 0, &V23080_285_57167, &P23080_284};	/*Property = CssCanvas*/
const Value V23080_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_286 = {S8, 0, &V23080_286_57170, &P23080_285};	/*Property = CssColor*/
const Value V23080_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_237 = {S9, 0, &V23080_237_55786, &P23080_286};	/*Property = CssColumn*/
const Value V23080_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_287 = {S10, 0, &V23080_287_57173, &P23080_237};	/*Property = CssFlexbox*/
const Value V23080_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_288 = {S11, 0, &V23080_288_57176, &P23080_287};	/*Property = CssFont*/
const Value V23080_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_289 = {S12, 0, &V23080_289_86899, &P23080_288};	/*Property = CssImages*/
const Value V23080_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_291 = {S13, 0, &V23080_291_57182, &P23080_289};	/*Property = CssMediaQueries*/
const Value V23080_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_292 = {S14, 0, &V23080_292_57185, &P23080_291};	/*Property = CssMinMax*/
const Value V23080_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_293 = {S15, 0, &V23080_293_57188, &P23080_292};	/*Property = CssOverflow*/
const Value V23080_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_294 = {S16, 0, &V23080_294_57191, &P23080_293};	/*Property = CssPosition*/
const Value V23080_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_295 = {S17, 0, &V23080_295_57194, &P23080_294};	/*Property = CssText*/
const Value V23080_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_297 = {S18, 0, &V23080_297_57197, &P23080_295};	/*Property = CssTransforms*/
const Value V23080_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_241 = {S19, 0, &V23080_241_55801, &P23080_297};	/*Property = CssTransitions*/
const Value V23080_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_298 = {S20, 0, &V23080_298_57208, &P23080_241};	/*Property = CssUI*/
const Value V23080_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_300 = {S21, 0, &V23080_300_57217, &P23080_298};	/*Property = DataSet*/
const Value V23080_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_301 = {S22, 0, &V23080_301_57220, &P23080_300};	/*Property = DataUrl*/
const Value V23080_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_302 = {S23, 0, &V23080_302_57223, &P23080_301};	/*Property = DeviceOrientation*/
const Value V23080_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_304 = {S24, 0, &V23080_304_57229, &P23080_302};	/*Property = FileReader*/
const Value V23080_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_305 = {S25, 0, &V23080_305_57232, &P23080_304};	/*Property = FileSaver*/
const Value V23080_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_306 = {S26, 0, &V23080_306_57238, &P23080_305};	/*Property = FileWriter*/
const Value V23080_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_307 = {S27, 0, &V23080_307_57241, &P23080_306};	/*Property = FormData*/
const Value V23080_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_309 = {S28, 0, &V23080_309_57247, &P23080_307};	/*Property = Fullscreen*/
const Value V23080_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_310 = {S29, 0, &V23080_310_57250, &P23080_309};	/*Property = GeoLocation*/
const Value V23080_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_312 = {S30, 0, &V23080_312_57256, &P23080_310};	/*Property = History*/
const Value V23080_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_314 = {S31, 0, &V23080_314_57262, &P23080_312};	/*Property = Html5*/
const Value V23080_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_313 = {S32, 0, &V23080_313_57259, &P23080_314};	/*Property = Html-Media-Capture*/
const Value V23080_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_315 = {S33, 0, &V23080_315_57265, &P23080_313};	/*Property = Iframe*/
const Value V23080_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_316 = {S34, 0, &V23080_316_57268, &P23080_315};	/*Property = IndexedDB*/
const Value V23080_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_317 = {S35, 0, &V23080_317_57271, &P23080_316};	/*Property = Json*/
const Value V23080_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_15 = {S37, 0, &V23080_15_72, &P23080_317};	/*Property = LayoutEngine*/
const Value V23080_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_344 = {S38, 0, &V23080_344_57274, &P23080_15};	/*Property = Masking*/
const Value V23080_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_320 = {S39, 0, &V23080_320_57292, &P23080_344};	/*Property = PostMessage*/
const Value V23080_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_321 = {S40, 0, &V23080_321_57295, &P23080_320};	/*Property = Progress*/
const Value V23080_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_322 = {S41, 0, &V23080_322_57298, &P23080_321};	/*Property = Prompts*/
const Value V23080_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_323 = {S42, 0, &V23080_323_57301, &P23080_322};	/*Property = Selector*/
const Value V23080_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_324 = {S43, 0, &V23080_324_57307, &P23080_323};	/*Property = Svg*/
const Value V23080_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_327 = {S44, 0, &V23080_327_57316, &P23080_324};	/*Property = TouchEvents*/
const Value V23080_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_328 = {S45, 0, &V23080_328_57319, &P23080_327};	/*Property = Track*/
const Value V23080_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_329 = {S46, 0, &V23080_329_57325, &P23080_328};	/*Property = Video*/
const Value V23080_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_330 = {S47, 0, &V23080_330_57328, &P23080_329};	/*Property = Viewport*/
const Value V23080_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_332 = {S48, 0, &V23080_332_57340, &P23080_330};	/*Property = WebWorkers*/
const Value V23080_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23080_333 = {S49, 0, &V23080_333_57343, &P23080_332};	/*Property = Xhr2*/
const Profile P23080 = {"23080", &P23080_333};	/*Profile ID - 23080*/
const Value V22074_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_279 = {S1, 0, &V22074_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22074_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_282 = {S3, 0, &V22074_282_57155, &P22074_279};	/*Property = BlobBuilder*/
const Value V22074_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_350 = {S4, 0, &V22074_350_57446, &P22074_282};	/*Property = Canvas*/
const Value V22074_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_283 = {S5, 0, &V22074_283_57161, &P22074_350};	/*Property = CssBackground*/
const Value V22074_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_284 = {S6, 0, &V22074_284_57164, &P22074_283};	/*Property = CssBorderImage*/
const Value V22074_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_285 = {S7, 0, &V22074_285_57167, &P22074_284};	/*Property = CssCanvas*/
const Value V22074_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_286 = {S8, 0, &V22074_286_57170, &P22074_285};	/*Property = CssColor*/
const Value V22074_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_237 = {S9, 0, &V22074_237_55786, &P22074_286};	/*Property = CssColumn*/
const Value V22074_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_287 = {S10, 0, &V22074_287_57173, &P22074_237};	/*Property = CssFlexbox*/
const Value V22074_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_288 = {S11, 0, &V22074_288_57176, &P22074_287};	/*Property = CssFont*/
const Value V22074_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_289 = {S12, 0, &V22074_289_86899, &P22074_288};	/*Property = CssImages*/
const Value V22074_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_291 = {S13, 0, &V22074_291_57182, &P22074_289};	/*Property = CssMediaQueries*/
const Value V22074_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_292 = {S14, 0, &V22074_292_57185, &P22074_291};	/*Property = CssMinMax*/
const Value V22074_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_293 = {S15, 0, &V22074_293_57188, &P22074_292};	/*Property = CssOverflow*/
const Value V22074_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_294 = {S16, 0, &V22074_294_57191, &P22074_293};	/*Property = CssPosition*/
const Value V22074_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_295 = {S17, 0, &V22074_295_57194, &P22074_294};	/*Property = CssText*/
const Value V22074_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_297 = {S18, 0, &V22074_297_57197, &P22074_295};	/*Property = CssTransforms*/
const Value V22074_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_241 = {S19, 0, &V22074_241_55801, &P22074_297};	/*Property = CssTransitions*/
const Value V22074_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_298 = {S20, 0, &V22074_298_57208, &P22074_241};	/*Property = CssUI*/
const Value V22074_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_300 = {S21, 0, &V22074_300_57217, &P22074_298};	/*Property = DataSet*/
const Value V22074_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_301 = {S22, 0, &V22074_301_57220, &P22074_300};	/*Property = DataUrl*/
const Value V22074_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_302 = {S23, 0, &V22074_302_57223, &P22074_301};	/*Property = DeviceOrientation*/
const Value V22074_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_304 = {S24, 0, &V22074_304_57229, &P22074_302};	/*Property = FileReader*/
const Value V22074_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_305 = {S25, 0, &V22074_305_57232, &P22074_304};	/*Property = FileSaver*/
const Value V22074_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_306 = {S26, 0, &V22074_306_57238, &P22074_305};	/*Property = FileWriter*/
const Value V22074_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_307 = {S27, 0, &V22074_307_57241, &P22074_306};	/*Property = FormData*/
const Value V22074_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_309 = {S28, 0, &V22074_309_57247, &P22074_307};	/*Property = Fullscreen*/
const Value V22074_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_310 = {S29, 0, &V22074_310_57250, &P22074_309};	/*Property = GeoLocation*/
const Value V22074_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_312 = {S30, 0, &V22074_312_57256, &P22074_310};	/*Property = History*/
const Value V22074_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_314 = {S31, 0, &V22074_314_57262, &P22074_312};	/*Property = Html5*/
const Value V22074_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_313 = {S32, 0, &V22074_313_57259, &P22074_314};	/*Property = Html-Media-Capture*/
const Value V22074_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_315 = {S33, 0, &V22074_315_57265, &P22074_313};	/*Property = Iframe*/
const Value V22074_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_316 = {S34, 0, &V22074_316_57268, &P22074_315};	/*Property = IndexedDB*/
const Value V22074_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_317 = {S35, 0, &V22074_317_57271, &P22074_316};	/*Property = Json*/
const Value V22074_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_15 = {S37, 0, &V22074_15_72, &P22074_317};	/*Property = LayoutEngine*/
const Value V22074_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_344 = {S38, 0, &V22074_344_57274, &P22074_15};	/*Property = Masking*/
const Value V22074_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_320 = {S39, 0, &V22074_320_57292, &P22074_344};	/*Property = PostMessage*/
const Value V22074_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_321 = {S40, 0, &V22074_321_57295, &P22074_320};	/*Property = Progress*/
const Value V22074_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_322 = {S41, 0, &V22074_322_57298, &P22074_321};	/*Property = Prompts*/
const Value V22074_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_323 = {S42, 0, &V22074_323_57301, &P22074_322};	/*Property = Selector*/
const Value V22074_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_324 = {S43, 0, &V22074_324_57307, &P22074_323};	/*Property = Svg*/
const Value V22074_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_327 = {S44, 0, &V22074_327_57316, &P22074_324};	/*Property = TouchEvents*/
const Value V22074_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_328 = {S45, 0, &V22074_328_57319, &P22074_327};	/*Property = Track*/
const Value V22074_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_329 = {S46, 0, &V22074_329_57325, &P22074_328};	/*Property = Video*/
const Value V22074_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_330 = {S47, 0, &V22074_330_57328, &P22074_329};	/*Property = Viewport*/
const Value V22074_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_332 = {S48, 0, &V22074_332_57340, &P22074_330};	/*Property = WebWorkers*/
const Value V22074_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22074_333 = {S49, 0, &V22074_333_57343, &P22074_332};	/*Property = Xhr2*/
const Profile P22074 = {"22074", &P22074_333};	/*Profile ID - 22074*/
const Value V22900_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22900_279 = {S1, 0, &V22900_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22900_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22900_282 = {S3, 0, &V22900_282_57154, &P22900_279};	/*Property = BlobBuilder*/
const Value V22900_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22900_350 = {S4, 0, &V22900_350_57444, &P22900_282};	/*Property = Canvas*/
const Value V22900_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22900_283 = {S5, 0, &V22900_283_57159, &P22900_350};	/*Property = CssBackground*/
const Value V22900_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22900_284 = {S6, 0, &V22900_284_57162, &P22900_283};	/*Property = CssBorderImage*/
const Value V22900_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22900_285 = {S7, 0, &V22900_285_57166, &P22900_284};	/*Property = CssCanvas*/
const Value V22900_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22900_286 = {S8, 0, &V22900_286_57168, &P22900_285};	/*Property = CssColor*/
const Value V22900_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22900_237 = {S9, 0, &V22900_237_55786, &P22900_286};	/*Property = CssColumn*/
const Value V22900_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22900_287 = {S10, 0, &V22900_287_57172, &P22900_237};	/*Property = CssFlexbox*/
const Value V22900_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22900_288 = {S11, 0, &V22900_288_57174, &P22900_287};	/*Property = CssFont*/
const Value V22900_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22900_289 = {S12, 0, &V22900_289_57759, &P22900_288};	/*Property = CssImages*/
const Value V22900_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22900_291 = {S13, 0, &V22900_291_57180, &P22900_289};	/*Property = CssMediaQueries*/
const Value V22900_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22900_292 = {S14, 0, &V22900_292_57183, &P22900_291};	/*Property = CssMinMax*/
const Value V22900_293_57186 = {S2, NULL};	/*Value = True*/
const Property P22900_293 = {S15, 0, &V22900_293_57186, &P22900_292};	/*Property = CssOverflow*/
const Value V22900_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22900_294 = {S16, 0, &V22900_294_57189, &P22900_293};	/*Property = CssPosition*/
const Value V22900_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22900_295 = {S17, 0, &V22900_295_57192, &P22900_294};	/*Property = CssText*/
const Value V22900_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22900_297 = {S18, 0, &V22900_297_57195, &P22900_295};	/*Property = CssTransforms*/
const Value V22900_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22900_241 = {S19, 0, &V22900_241_55799, &P22900_297};	/*Property = CssTransitions*/
const Value V22900_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22900_298 = {S20, 0, &V22900_298_57206, &P22900_241};	/*Property = CssUI*/
const Value V22900_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22900_300 = {S21, 0, &V22900_300_57215, &P22900_298};	/*Property = DataSet*/
const Value V22900_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22900_301 = {S22, 0, &V22900_301_57218, &P22900_300};	/*Property = DataUrl*/
const Value V22900_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22900_302 = {S23, 0, &V22900_302_57221, &P22900_301};	/*Property = DeviceOrientation*/
const Value V22900_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22900_304 = {S24, 0, &V22900_304_57227, &P22900_302};	/*Property = FileReader*/
const Value V22900_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22900_305 = {S25, 0, &V22900_305_57231, &P22900_304};	/*Property = FileSaver*/
const Value V22900_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22900_306 = {S26, 0, &V22900_306_57237, &P22900_305};	/*Property = FileWriter*/
const Value V22900_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22900_307 = {S27, 0, &V22900_307_57239, &P22900_306};	/*Property = FormData*/
const Value V22900_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22900_309 = {S28, 0, &V22900_309_57246, &P22900_307};	/*Property = Fullscreen*/
const Value V22900_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22900_310 = {S29, 0, &V22900_310_57248, &P22900_309};	/*Property = GeoLocation*/
const Value V22900_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22900_312 = {S30, 0, &V22900_312_57254, &P22900_310};	/*Property = History*/
const Value V22900_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22900_314 = {S31, 0, &V22900_314_57260, &P22900_312};	/*Property = Html5*/
const Value V22900_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22900_313 = {S32, 0, &V22900_313_57258, &P22900_314};	/*Property = Html-Media-Capture*/
const Value V22900_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22900_315 = {S33, 0, &V22900_315_57264, &P22900_313};	/*Property = Iframe*/
const Value V22900_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22900_316 = {S34, 0, &V22900_316_57267, &P22900_315};	/*Property = IndexedDB*/
const Value V22900_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22900_317 = {S35, 0, &V22900_317_57269, &P22900_316};	/*Property = Json*/
const Value V22900_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22900_15 = {S37, 0, &V22900_15_28, &P22900_317};	/*Property = LayoutEngine*/
const Value V22900_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22900_344 = {S38, 0, &V22900_344_57272, &P22900_15};	/*Property = Masking*/
const Value V22900_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22900_320 = {S39, 0, &V22900_320_57290, &P22900_344};	/*Property = PostMessage*/
const Value V22900_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22900_321 = {S40, 0, &V22900_321_57293, &P22900_320};	/*Property = Progress*/
const Value V22900_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22900_322 = {S41, 0, &V22900_322_57296, &P22900_321};	/*Property = Prompts*/
const Value V22900_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22900_323 = {S42, 0, &V22900_323_57299, &P22900_322};	/*Property = Selector*/
const Value V22900_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22900_324 = {S43, 0, &V22900_324_57305, &P22900_323};	/*Property = Svg*/
const Value V22900_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22900_327 = {S44, 0, &V22900_327_57314, &P22900_324};	/*Property = TouchEvents*/
const Value V22900_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22900_328 = {S45, 0, &V22900_328_57318, &P22900_327};	/*Property = Track*/
const Value V22900_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22900_329 = {S46, 0, &V22900_329_57323, &P22900_328};	/*Property = Video*/
const Value V22900_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22900_330 = {S47, 0, &V22900_330_57326, &P22900_329};	/*Property = Viewport*/
const Value V22900_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22900_332 = {S48, 0, &V22900_332_57340, &P22900_330};	/*Property = WebWorkers*/
const Value V22900_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22900_333 = {S49, 0, &V22900_333_57342, &P22900_332};	/*Property = Xhr2*/
const Profile P22900 = {"22900", &P22900_333};	/*Profile ID - 22900*/
const Value V20498_1_1 = {S2, NULL};	/*Value = True*/
const Property P20498_1 = {S50, 0, &V20498_1_1, NULL};	/*Property = IsMobile*/
const Value V20498_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20498_5 = {S52, 0, &V20498_5_1227, &P20498_1};	/*Property = ScreenPixelsHeight*/
const Value V20498_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P20498_4 = {S54, 0, &V20498_4_1477, &P20498_5};	/*Property = ScreenPixelsWidth*/
const Profile P20498 = {"20498", &P20498_4};	/*Profile ID - 20498*/
const Value V24439_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_279 = {S1, 0, &V24439_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24439_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_282 = {S3, 0, &V24439_282_57155, &P24439_279};	/*Property = BlobBuilder*/
const Value V24439_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_350 = {S4, 0, &V24439_350_57446, &P24439_282};	/*Property = Canvas*/
const Value V24439_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_283 = {S5, 0, &V24439_283_57161, &P24439_350};	/*Property = CssBackground*/
const Value V24439_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_284 = {S6, 0, &V24439_284_57164, &P24439_283};	/*Property = CssBorderImage*/
const Value V24439_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_285 = {S7, 0, &V24439_285_57167, &P24439_284};	/*Property = CssCanvas*/
const Value V24439_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_286 = {S8, 0, &V24439_286_57170, &P24439_285};	/*Property = CssColor*/
const Value V24439_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_237 = {S9, 0, &V24439_237_55786, &P24439_286};	/*Property = CssColumn*/
const Value V24439_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_287 = {S10, 0, &V24439_287_57173, &P24439_237};	/*Property = CssFlexbox*/
const Value V24439_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_288 = {S11, 0, &V24439_288_57176, &P24439_287};	/*Property = CssFont*/
const Value V24439_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_289 = {S12, 0, &V24439_289_86899, &P24439_288};	/*Property = CssImages*/
const Value V24439_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_291 = {S13, 0, &V24439_291_57182, &P24439_289};	/*Property = CssMediaQueries*/
const Value V24439_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_292 = {S14, 0, &V24439_292_57185, &P24439_291};	/*Property = CssMinMax*/
const Value V24439_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_293 = {S15, 0, &V24439_293_57188, &P24439_292};	/*Property = CssOverflow*/
const Value V24439_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_294 = {S16, 0, &V24439_294_57191, &P24439_293};	/*Property = CssPosition*/
const Value V24439_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_295 = {S17, 0, &V24439_295_57194, &P24439_294};	/*Property = CssText*/
const Value V24439_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_297 = {S18, 0, &V24439_297_57197, &P24439_295};	/*Property = CssTransforms*/
const Value V24439_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_241 = {S19, 0, &V24439_241_55801, &P24439_297};	/*Property = CssTransitions*/
const Value V24439_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_298 = {S20, 0, &V24439_298_57208, &P24439_241};	/*Property = CssUI*/
const Value V24439_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_300 = {S21, 0, &V24439_300_57217, &P24439_298};	/*Property = DataSet*/
const Value V24439_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_301 = {S22, 0, &V24439_301_57220, &P24439_300};	/*Property = DataUrl*/
const Value V24439_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_302 = {S23, 0, &V24439_302_57223, &P24439_301};	/*Property = DeviceOrientation*/
const Value V24439_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_304 = {S24, 0, &V24439_304_57229, &P24439_302};	/*Property = FileReader*/
const Value V24439_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_305 = {S25, 0, &V24439_305_57232, &P24439_304};	/*Property = FileSaver*/
const Value V24439_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_306 = {S26, 0, &V24439_306_57238, &P24439_305};	/*Property = FileWriter*/
const Value V24439_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_307 = {S27, 0, &V24439_307_57241, &P24439_306};	/*Property = FormData*/
const Value V24439_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_309 = {S28, 0, &V24439_309_57247, &P24439_307};	/*Property = Fullscreen*/
const Value V24439_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_310 = {S29, 0, &V24439_310_57250, &P24439_309};	/*Property = GeoLocation*/
const Value V24439_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_312 = {S30, 0, &V24439_312_57256, &P24439_310};	/*Property = History*/
const Value V24439_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_314 = {S31, 0, &V24439_314_57262, &P24439_312};	/*Property = Html5*/
const Value V24439_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_313 = {S32, 0, &V24439_313_57259, &P24439_314};	/*Property = Html-Media-Capture*/
const Value V24439_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_315 = {S33, 0, &V24439_315_57265, &P24439_313};	/*Property = Iframe*/
const Value V24439_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_316 = {S34, 0, &V24439_316_57268, &P24439_315};	/*Property = IndexedDB*/
const Value V24439_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_317 = {S35, 0, &V24439_317_57271, &P24439_316};	/*Property = Json*/
const Value V24439_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24439_15 = {S37, 0, &V24439_15_28, &P24439_317};	/*Property = LayoutEngine*/
const Value V24439_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_344 = {S38, 0, &V24439_344_57274, &P24439_15};	/*Property = Masking*/
const Value V24439_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_320 = {S39, 0, &V24439_320_57292, &P24439_344};	/*Property = PostMessage*/
const Value V24439_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_321 = {S40, 0, &V24439_321_57295, &P24439_320};	/*Property = Progress*/
const Value V24439_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_322 = {S41, 0, &V24439_322_57298, &P24439_321};	/*Property = Prompts*/
const Value V24439_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_323 = {S42, 0, &V24439_323_57301, &P24439_322};	/*Property = Selector*/
const Value V24439_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_324 = {S43, 0, &V24439_324_57307, &P24439_323};	/*Property = Svg*/
const Value V24439_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_327 = {S44, 0, &V24439_327_57316, &P24439_324};	/*Property = TouchEvents*/
const Value V24439_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_328 = {S45, 0, &V24439_328_57319, &P24439_327};	/*Property = Track*/
const Value V24439_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_329 = {S46, 0, &V24439_329_57325, &P24439_328};	/*Property = Video*/
const Value V24439_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_330 = {S47, 0, &V24439_330_57328, &P24439_329};	/*Property = Viewport*/
const Value V24439_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_332 = {S48, 0, &V24439_332_57340, &P24439_330};	/*Property = WebWorkers*/
const Value V24439_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24439_333 = {S49, 0, &V24439_333_57343, &P24439_332};	/*Property = Xhr2*/
const Profile P24439 = {"24439", &P24439_333};	/*Profile ID - 24439*/
const Value V20473_1_1 = {S2, NULL};	/*Value = True*/
const Property P20473_1 = {S50, 0, &V20473_1_1, NULL};	/*Property = IsMobile*/
const Value V20473_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20473_5 = {S52, 0, &V20473_5_9981, &P20473_1};	/*Property = ScreenPixelsHeight*/
const Value V20473_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20473_4 = {S54, 0, &V20473_4_2336, &P20473_5};	/*Property = ScreenPixelsWidth*/
const Profile P20473 = {"20473", &P20473_4};	/*Profile ID - 20473*/
const Value V22477_1_1 = {S2, NULL};	/*Value = True*/
const Property P22477_1 = {S50, 0, &V22477_1_1, NULL};	/*Property = IsMobile*/
const Value V22477_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22477_5 = {S52, 0, &V22477_5_1277, &P22477_1};	/*Property = ScreenPixelsHeight*/
const Value V22477_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22477_4 = {S54, 0, &V22477_4_1279, &P22477_5};	/*Property = ScreenPixelsWidth*/
const Profile P22477 = {"22477", &P22477_4};	/*Profile ID - 22477*/
const Value V20854_1_1 = {S2, NULL};	/*Value = True*/
const Property P20854_1 = {S50, 0, &V20854_1_1, NULL};	/*Property = IsMobile*/
const Value V20854_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20854_5 = {S52, 0, &V20854_5_1458, &P20854_1};	/*Property = ScreenPixelsHeight*/
const Value V20854_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20854_4 = {S54, 0, &V20854_4_1460, &P20854_5};	/*Property = ScreenPixelsWidth*/
const Profile P20854 = {"20854", &P20854_4};	/*Profile ID - 20854*/
const Value V19761_1_1 = {S2, NULL};	/*Value = True*/
const Property P19761_1 = {S50, 0, &V19761_1_1, NULL};	/*Property = IsMobile*/
const Value V19761_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19761_5 = {S52, 0, &V19761_5_1227, &P19761_1};	/*Property = ScreenPixelsHeight*/
const Value V19761_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19761_4 = {S54, 0, &V19761_4_1229, &P19761_5};	/*Property = ScreenPixelsWidth*/
const Profile P19761 = {"19761", &P19761_4};	/*Profile ID - 19761*/
const Value V23764_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_279 = {S1, 0, &V23764_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23764_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_282 = {S3, 0, &V23764_282_57155, &P23764_279};	/*Property = BlobBuilder*/
const Value V23764_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_350 = {S4, 0, &V23764_350_57446, &P23764_282};	/*Property = Canvas*/
const Value V23764_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_283 = {S5, 0, &V23764_283_57161, &P23764_350};	/*Property = CssBackground*/
const Value V23764_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_284 = {S6, 0, &V23764_284_57164, &P23764_283};	/*Property = CssBorderImage*/
const Value V23764_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_285 = {S7, 0, &V23764_285_57167, &P23764_284};	/*Property = CssCanvas*/
const Value V23764_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_286 = {S8, 0, &V23764_286_57170, &P23764_285};	/*Property = CssColor*/
const Value V23764_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_237 = {S9, 0, &V23764_237_55786, &P23764_286};	/*Property = CssColumn*/
const Value V23764_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_287 = {S10, 0, &V23764_287_57173, &P23764_237};	/*Property = CssFlexbox*/
const Value V23764_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_288 = {S11, 0, &V23764_288_57176, &P23764_287};	/*Property = CssFont*/
const Value V23764_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_289 = {S12, 0, &V23764_289_86899, &P23764_288};	/*Property = CssImages*/
const Value V23764_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_291 = {S13, 0, &V23764_291_57182, &P23764_289};	/*Property = CssMediaQueries*/
const Value V23764_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_292 = {S14, 0, &V23764_292_57185, &P23764_291};	/*Property = CssMinMax*/
const Value V23764_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_293 = {S15, 0, &V23764_293_57188, &P23764_292};	/*Property = CssOverflow*/
const Value V23764_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_294 = {S16, 0, &V23764_294_57191, &P23764_293};	/*Property = CssPosition*/
const Value V23764_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_295 = {S17, 0, &V23764_295_57194, &P23764_294};	/*Property = CssText*/
const Value V23764_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_297 = {S18, 0, &V23764_297_57197, &P23764_295};	/*Property = CssTransforms*/
const Value V23764_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_241 = {S19, 0, &V23764_241_55801, &P23764_297};	/*Property = CssTransitions*/
const Value V23764_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_298 = {S20, 0, &V23764_298_57208, &P23764_241};	/*Property = CssUI*/
const Value V23764_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_300 = {S21, 0, &V23764_300_57217, &P23764_298};	/*Property = DataSet*/
const Value V23764_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_301 = {S22, 0, &V23764_301_57220, &P23764_300};	/*Property = DataUrl*/
const Value V23764_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_302 = {S23, 0, &V23764_302_57223, &P23764_301};	/*Property = DeviceOrientation*/
const Value V23764_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_304 = {S24, 0, &V23764_304_57229, &P23764_302};	/*Property = FileReader*/
const Value V23764_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_305 = {S25, 0, &V23764_305_57232, &P23764_304};	/*Property = FileSaver*/
const Value V23764_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_306 = {S26, 0, &V23764_306_57238, &P23764_305};	/*Property = FileWriter*/
const Value V23764_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_307 = {S27, 0, &V23764_307_57241, &P23764_306};	/*Property = FormData*/
const Value V23764_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_309 = {S28, 0, &V23764_309_57247, &P23764_307};	/*Property = Fullscreen*/
const Value V23764_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_310 = {S29, 0, &V23764_310_57250, &P23764_309};	/*Property = GeoLocation*/
const Value V23764_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_312 = {S30, 0, &V23764_312_57256, &P23764_310};	/*Property = History*/
const Value V23764_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_314 = {S31, 0, &V23764_314_57262, &P23764_312};	/*Property = Html5*/
const Value V23764_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_313 = {S32, 0, &V23764_313_57259, &P23764_314};	/*Property = Html-Media-Capture*/
const Value V23764_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_315 = {S33, 0, &V23764_315_57265, &P23764_313};	/*Property = Iframe*/
const Value V23764_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_316 = {S34, 0, &V23764_316_57268, &P23764_315};	/*Property = IndexedDB*/
const Value V23764_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_317 = {S35, 0, &V23764_317_57271, &P23764_316};	/*Property = Json*/
const Value V23764_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_15 = {S37, 0, &V23764_15_72, &P23764_317};	/*Property = LayoutEngine*/
const Value V23764_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_344 = {S38, 0, &V23764_344_57274, &P23764_15};	/*Property = Masking*/
const Value V23764_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_320 = {S39, 0, &V23764_320_57292, &P23764_344};	/*Property = PostMessage*/
const Value V23764_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_321 = {S40, 0, &V23764_321_57295, &P23764_320};	/*Property = Progress*/
const Value V23764_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_322 = {S41, 0, &V23764_322_57298, &P23764_321};	/*Property = Prompts*/
const Value V23764_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_323 = {S42, 0, &V23764_323_57301, &P23764_322};	/*Property = Selector*/
const Value V23764_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_324 = {S43, 0, &V23764_324_57307, &P23764_323};	/*Property = Svg*/
const Value V23764_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_327 = {S44, 0, &V23764_327_57316, &P23764_324};	/*Property = TouchEvents*/
const Value V23764_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_328 = {S45, 0, &V23764_328_57319, &P23764_327};	/*Property = Track*/
const Value V23764_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_329 = {S46, 0, &V23764_329_57325, &P23764_328};	/*Property = Video*/
const Value V23764_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_330 = {S47, 0, &V23764_330_57328, &P23764_329};	/*Property = Viewport*/
const Value V23764_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_332 = {S48, 0, &V23764_332_57340, &P23764_330};	/*Property = WebWorkers*/
const Value V23764_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23764_333 = {S49, 0, &V23764_333_57343, &P23764_332};	/*Property = Xhr2*/
const Profile P23764 = {"23764", &P23764_333};	/*Profile ID - 23764*/
const Value V19992_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_279 = {S1, 0, &V19992_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V19992_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_282 = {S3, 0, &V19992_282_57155, &P19992_279};	/*Property = BlobBuilder*/
const Value V19992_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_350 = {S4, 0, &V19992_350_57446, &P19992_282};	/*Property = Canvas*/
const Value V19992_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_283 = {S5, 0, &V19992_283_57161, &P19992_350};	/*Property = CssBackground*/
const Value V19992_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_284 = {S6, 0, &V19992_284_57164, &P19992_283};	/*Property = CssBorderImage*/
const Value V19992_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_285 = {S7, 0, &V19992_285_57167, &P19992_284};	/*Property = CssCanvas*/
const Value V19992_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_286 = {S8, 0, &V19992_286_57170, &P19992_285};	/*Property = CssColor*/
const Value V19992_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_237 = {S9, 0, &V19992_237_55786, &P19992_286};	/*Property = CssColumn*/
const Value V19992_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_287 = {S10, 0, &V19992_287_57173, &P19992_237};	/*Property = CssFlexbox*/
const Value V19992_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_288 = {S11, 0, &V19992_288_57176, &P19992_287};	/*Property = CssFont*/
const Value V19992_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_289 = {S12, 0, &V19992_289_86899, &P19992_288};	/*Property = CssImages*/
const Value V19992_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_291 = {S13, 0, &V19992_291_57182, &P19992_289};	/*Property = CssMediaQueries*/
const Value V19992_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_292 = {S14, 0, &V19992_292_57185, &P19992_291};	/*Property = CssMinMax*/
const Value V19992_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_293 = {S15, 0, &V19992_293_57188, &P19992_292};	/*Property = CssOverflow*/
const Value V19992_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_294 = {S16, 0, &V19992_294_57191, &P19992_293};	/*Property = CssPosition*/
const Value V19992_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_295 = {S17, 0, &V19992_295_57194, &P19992_294};	/*Property = CssText*/
const Value V19992_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_297 = {S18, 0, &V19992_297_57197, &P19992_295};	/*Property = CssTransforms*/
const Value V19992_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_241 = {S19, 0, &V19992_241_55801, &P19992_297};	/*Property = CssTransitions*/
const Value V19992_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_298 = {S20, 0, &V19992_298_57208, &P19992_241};	/*Property = CssUI*/
const Value V19992_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_300 = {S21, 0, &V19992_300_57217, &P19992_298};	/*Property = DataSet*/
const Value V19992_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_301 = {S22, 0, &V19992_301_57220, &P19992_300};	/*Property = DataUrl*/
const Value V19992_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_302 = {S23, 0, &V19992_302_57223, &P19992_301};	/*Property = DeviceOrientation*/
const Value V19992_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_304 = {S24, 0, &V19992_304_57229, &P19992_302};	/*Property = FileReader*/
const Value V19992_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_305 = {S25, 0, &V19992_305_57232, &P19992_304};	/*Property = FileSaver*/
const Value V19992_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_306 = {S26, 0, &V19992_306_57238, &P19992_305};	/*Property = FileWriter*/
const Value V19992_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_307 = {S27, 0, &V19992_307_57241, &P19992_306};	/*Property = FormData*/
const Value V19992_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_309 = {S28, 0, &V19992_309_57247, &P19992_307};	/*Property = Fullscreen*/
const Value V19992_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_310 = {S29, 0, &V19992_310_57250, &P19992_309};	/*Property = GeoLocation*/
const Value V19992_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_312 = {S30, 0, &V19992_312_57256, &P19992_310};	/*Property = History*/
const Value V19992_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_314 = {S31, 0, &V19992_314_57262, &P19992_312};	/*Property = Html5*/
const Value V19992_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_313 = {S32, 0, &V19992_313_57259, &P19992_314};	/*Property = Html-Media-Capture*/
const Value V19992_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_315 = {S33, 0, &V19992_315_57265, &P19992_313};	/*Property = Iframe*/
const Value V19992_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_316 = {S34, 0, &V19992_316_57268, &P19992_315};	/*Property = IndexedDB*/
const Value V19992_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_317 = {S35, 0, &V19992_317_57271, &P19992_316};	/*Property = Json*/
const Value V19992_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P19992_15 = {S37, 0, &V19992_15_30, &P19992_317};	/*Property = LayoutEngine*/
const Value V19992_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_344 = {S38, 0, &V19992_344_57274, &P19992_15};	/*Property = Masking*/
const Value V19992_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_320 = {S39, 0, &V19992_320_57292, &P19992_344};	/*Property = PostMessage*/
const Value V19992_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_321 = {S40, 0, &V19992_321_57295, &P19992_320};	/*Property = Progress*/
const Value V19992_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_322 = {S41, 0, &V19992_322_57298, &P19992_321};	/*Property = Prompts*/
const Value V19992_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_323 = {S42, 0, &V19992_323_57301, &P19992_322};	/*Property = Selector*/
const Value V19992_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_324 = {S43, 0, &V19992_324_57307, &P19992_323};	/*Property = Svg*/
const Value V19992_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_327 = {S44, 0, &V19992_327_57316, &P19992_324};	/*Property = TouchEvents*/
const Value V19992_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_328 = {S45, 0, &V19992_328_57319, &P19992_327};	/*Property = Track*/
const Value V19992_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_329 = {S46, 0, &V19992_329_57325, &P19992_328};	/*Property = Video*/
const Value V19992_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_330 = {S47, 0, &V19992_330_57328, &P19992_329};	/*Property = Viewport*/
const Value V19992_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_332 = {S48, 0, &V19992_332_57340, &P19992_330};	/*Property = WebWorkers*/
const Value V19992_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P19992_333 = {S49, 0, &V19992_333_57343, &P19992_332};	/*Property = Xhr2*/
const Profile P19992 = {"19992", &P19992_333};	/*Profile ID - 19992*/
const Value V22942_1_1 = {S2, NULL};	/*Value = True*/
const Property P22942_1 = {S50, 0, &V22942_1_1, NULL};	/*Property = IsMobile*/
const Value V22942_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22942_5 = {S52, 0, &V22942_5_1403, &P22942_1};	/*Property = ScreenPixelsHeight*/
const Value V22942_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22942_4 = {S54, 0, &V22942_4_1404, &P22942_5};	/*Property = ScreenPixelsWidth*/
const Profile P22942 = {"22942", &P22942_4};	/*Profile ID - 22942*/
const Value V16778_1_1 = {S2, NULL};	/*Value = True*/
const Property P16778_1 = {S50, 0, &V16778_1_1, NULL};	/*Property = IsMobile*/
const Value V16778_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P16778_5 = {S52, 0, &V16778_5_1222, &P16778_1};	/*Property = ScreenPixelsHeight*/
const Value V16778_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16778_4 = {S54, 0, &V16778_4_1224, &P16778_5};	/*Property = ScreenPixelsWidth*/
const Profile P16778 = {"16778", &P16778_4};	/*Profile ID - 16778*/
const Value V20191_1_1 = {S2, NULL};	/*Value = True*/
const Property P20191_1 = {S50, 0, &V20191_1_1, NULL};	/*Property = IsMobile*/
const Value V20191_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20191_5 = {S52, 0, &V20191_5_9981, &P20191_1};	/*Property = ScreenPixelsHeight*/
const Value V20191_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20191_4 = {S54, 0, &V20191_4_2336, &P20191_5};	/*Property = ScreenPixelsWidth*/
const Profile P20191 = {"20191", &P20191_4};	/*Profile ID - 20191*/
const Value V23062_1_1 = {S2, NULL};	/*Value = True*/
const Property P23062_1 = {S50, 0, &V23062_1_1, NULL};	/*Property = IsMobile*/
const Value V23062_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23062_5 = {S52, 0, &V23062_5_1222, &P23062_1};	/*Property = ScreenPixelsHeight*/
const Value V23062_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23062_4 = {S54, 0, &V23062_4_1224, &P23062_5};	/*Property = ScreenPixelsWidth*/
const Profile P23062 = {"23062", &P23062_4};	/*Profile ID - 23062*/
const Value V18473_1_1 = {S2, NULL};	/*Value = True*/
const Property P18473_1 = {S50, 0, &V18473_1_1, NULL};	/*Property = IsMobile*/
const Value V18473_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P18473_5 = {S52, 0, &V18473_5_9981, &P18473_1};	/*Property = ScreenPixelsHeight*/
const Value V18473_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P18473_4 = {S54, 0, &V18473_4_39098, &P18473_5};	/*Property = ScreenPixelsWidth*/
const Profile P18473 = {"18473", &P18473_4};	/*Profile ID - 18473*/
const Value V21663_1_1 = {S2, NULL};	/*Value = True*/
const Property P21663_1 = {S50, 0, &V21663_1_1, NULL};	/*Property = IsMobile*/
const Value V21663_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21663_5 = {S52, 0, &V21663_5_1227, &P21663_1};	/*Property = ScreenPixelsHeight*/
const Value V21663_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21663_4 = {S54, 0, &V21663_4_1229, &P21663_5};	/*Property = ScreenPixelsWidth*/
const Profile P21663 = {"21663", &P21663_4};	/*Profile ID - 21663*/
const Value V20367_1_1 = {S2, NULL};	/*Value = True*/
const Property P20367_1 = {S50, 0, &V20367_1_1, NULL};	/*Property = IsMobile*/
const Value V20367_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20367_5 = {S52, 0, &V20367_5_9981, &P20367_1};	/*Property = ScreenPixelsHeight*/
const Value V20367_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20367_4 = {S54, 0, &V20367_4_39098, &P20367_5};	/*Property = ScreenPixelsWidth*/
const Profile P20367 = {"20367", &P20367_4};	/*Profile ID - 20367*/
const Value V20543_1_1 = {S2, NULL};	/*Value = True*/
const Property P20543_1 = {S50, 0, &V20543_1_1, NULL};	/*Property = IsMobile*/
const Value V20543_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20543_5 = {S52, 0, &V20543_5_1227, &P20543_1};	/*Property = ScreenPixelsHeight*/
const Value V20543_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20543_4 = {S54, 0, &V20543_4_1229, &P20543_5};	/*Property = ScreenPixelsWidth*/
const Profile P20543 = {"20543", &P20543_4};	/*Profile ID - 20543*/
const Value V23052_1_1 = {S2, NULL};	/*Value = True*/
const Property P23052_1 = {S50, 0, &V23052_1_1, NULL};	/*Property = IsMobile*/
const Value V23052_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23052_5 = {S52, 0, &V23052_5_1222, &P23052_1};	/*Property = ScreenPixelsHeight*/
const Value V23052_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23052_4 = {S54, 0, &V23052_4_1224, &P23052_5};	/*Property = ScreenPixelsWidth*/
const Profile P23052 = {"23052", &P23052_4};	/*Profile ID - 23052*/
const Value V22543_1_1 = {S2, NULL};	/*Value = True*/
const Property P22543_1 = {S50, 0, &V22543_1_1, NULL};	/*Property = IsMobile*/
const Value V22543_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22543_5 = {S52, 0, &V22543_5_1227, &P22543_1};	/*Property = ScreenPixelsHeight*/
const Value V22543_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22543_4 = {S54, 0, &V22543_4_1229, &P22543_5};	/*Property = ScreenPixelsWidth*/
const Profile P22543 = {"22543", &P22543_4};	/*Profile ID - 22543*/
const Value V20100_1_1 = {S2, NULL};	/*Value = True*/
const Property P20100_1 = {S50, 0, &V20100_1_1, NULL};	/*Property = IsMobile*/
const Value V20100_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20100_5 = {S52, 0, &V20100_5_1277, &P20100_1};	/*Property = ScreenPixelsHeight*/
const Value V20100_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20100_4 = {S54, 0, &V20100_4_1279, &P20100_5};	/*Property = ScreenPixelsWidth*/
const Profile P20100 = {"20100", &P20100_4};	/*Profile ID - 20100*/
const Value V23427_1_1 = {S2, NULL};	/*Value = True*/
const Property P23427_1 = {S50, 0, &V23427_1_1, NULL};	/*Property = IsMobile*/
const Value V23427_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23427_5 = {S52, 0, &V23427_5_1277, &P23427_1};	/*Property = ScreenPixelsHeight*/
const Value V23427_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23427_4 = {S54, 0, &V23427_4_1279, &P23427_5};	/*Property = ScreenPixelsWidth*/
const Profile P23427 = {"23427", &P23427_4};	/*Profile ID - 23427*/
const Value V20727_1_1 = {S2, NULL};	/*Value = True*/
const Property P20727_1 = {S50, 0, &V20727_1_1, NULL};	/*Property = IsMobile*/
const Value V20727_5_75223 = {S129, NULL};	/*Value = 1366*/
const Property P20727_5 = {S52, 0, &V20727_5_75223, &P20727_1};	/*Property = ScreenPixelsHeight*/
const Value V20727_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20727_4 = {S54, 0, &V20727_4_37617, &P20727_5};	/*Property = ScreenPixelsWidth*/
const Profile P20727 = {"20727", &P20727_4};	/*Profile ID - 20727*/
const Value V21446_1_1 = {S2, NULL};	/*Value = True*/
const Property P21446_1 = {S50, 0, &V21446_1_1, NULL};	/*Property = IsMobile*/
const Value V21446_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21446_5 = {S52, 0, &V21446_5_9981, &P21446_1};	/*Property = ScreenPixelsHeight*/
const Value V21446_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21446_4 = {S54, 0, &V21446_4_39098, &P21446_5};	/*Property = ScreenPixelsWidth*/
const Profile P21446 = {"21446", &P21446_4};	/*Profile ID - 21446*/
const Value V21259_1_1 = {S2, NULL};	/*Value = True*/
const Property P21259_1 = {S50, 0, &V21259_1_1, NULL};	/*Property = IsMobile*/
const Value V21259_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21259_5 = {S52, 0, &V21259_5_1277, &P21259_1};	/*Property = ScreenPixelsHeight*/
const Value V21259_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21259_4 = {S54, 0, &V21259_4_1279, &P21259_5};	/*Property = ScreenPixelsWidth*/
const Profile P21259 = {"21259", &P21259_4};	/*Profile ID - 21259*/
const Value V20130_1_2 = {S0, NULL};	/*Value = False*/
const Property P20130_1 = {S50, 0, &V20130_1_2, NULL};	/*Property = IsMobile*/
const Value V20130_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20130_5 = {S52, 0, &V20130_5_37, &P20130_1};	/*Property = ScreenPixelsHeight*/
const Value V20130_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20130_4 = {S54, 0, &V20130_4_36, &P20130_5};	/*Property = ScreenPixelsWidth*/
const Profile P20130 = {"20130", &P20130_4};	/*Profile ID - 20130*/
const Value V23947_1_1 = {S2, NULL};	/*Value = True*/
const Property P23947_1 = {S50, 0, &V23947_1_1, NULL};	/*Property = IsMobile*/
const Value V23947_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23947_5 = {S52, 0, &V23947_5_9981, &P23947_1};	/*Property = ScreenPixelsHeight*/
const Value V23947_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23947_4 = {S54, 0, &V23947_4_39098, &P23947_5};	/*Property = ScreenPixelsWidth*/
const Profile P23947 = {"23947", &P23947_4};	/*Profile ID - 23947*/
const Value V20475_1_1 = {S2, NULL};	/*Value = True*/
const Property P20475_1 = {S50, 0, &V20475_1_1, NULL};	/*Property = IsMobile*/
const Value V20475_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20475_5 = {S52, 0, &V20475_5_1458, &P20475_1};	/*Property = ScreenPixelsHeight*/
const Value V20475_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20475_4 = {S54, 0, &V20475_4_37617, &P20475_5};	/*Property = ScreenPixelsWidth*/
const Profile P20475 = {"20475", &P20475_4};	/*Profile ID - 20475*/
const Value V20839_1_1 = {S2, NULL};	/*Value = True*/
const Property P20839_1 = {S50, 0, &V20839_1_1, NULL};	/*Property = IsMobile*/
const Value V20839_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20839_5 = {S52, 0, &V20839_5_1403, &P20839_1};	/*Property = ScreenPixelsHeight*/
const Value V20839_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20839_4 = {S54, 0, &V20839_4_1404, &P20839_5};	/*Property = ScreenPixelsWidth*/
const Profile P20839 = {"20839", &P20839_4};	/*Profile ID - 20839*/
const Value V23469_1_1 = {S2, NULL};	/*Value = True*/
const Property P23469_1 = {S50, 0, &V23469_1_1, NULL};	/*Property = IsMobile*/
const Value V23469_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23469_5 = {S52, 0, &V23469_5_1458, &P23469_1};	/*Property = ScreenPixelsHeight*/
const Value V23469_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23469_4 = {S54, 0, &V23469_4_37617, &P23469_5};	/*Property = ScreenPixelsWidth*/
const Profile P23469 = {"23469", &P23469_4};	/*Profile ID - 23469*/
const Value V24563_1_1 = {S2, NULL};	/*Value = True*/
const Property P24563_1 = {S50, 0, &V24563_1_1, NULL};	/*Property = IsMobile*/
const Value V24563_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24563_5 = {S52, 0, &V24563_5_1227, &P24563_1};	/*Property = ScreenPixelsHeight*/
const Value V24563_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24563_4 = {S54, 0, &V24563_4_1229, &P24563_5};	/*Property = ScreenPixelsWidth*/
const Profile P24563 = {"24563", &P24563_4};	/*Profile ID - 24563*/
const Value V23466_1_1 = {S2, NULL};	/*Value = True*/
const Property P23466_1 = {S50, 0, &V23466_1_1, NULL};	/*Property = IsMobile*/
const Value V23466_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23466_5 = {S52, 0, &V23466_5_1227, &P23466_1};	/*Property = ScreenPixelsHeight*/
const Value V23466_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23466_4 = {S54, 0, &V23466_4_1460, &P23466_5};	/*Property = ScreenPixelsWidth*/
const Profile P23466 = {"23466", &P23466_4};	/*Profile ID - 23466*/
const Value V21614_1_1 = {S2, NULL};	/*Value = True*/
const Property P21614_1 = {S50, 0, &V21614_1_1, NULL};	/*Property = IsMobile*/
const Value V21614_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21614_5 = {S52, 0, &V21614_5_1227, &P21614_1};	/*Property = ScreenPixelsHeight*/
const Value V21614_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21614_4 = {S54, 0, &V21614_4_1229, &P21614_5};	/*Property = ScreenPixelsWidth*/
const Profile P21614 = {"21614", &P21614_4};	/*Profile ID - 21614*/
const Value V24029_1_1 = {S2, NULL};	/*Value = True*/
const Property P24029_1 = {S50, 0, &V24029_1_1, NULL};	/*Property = IsMobile*/
const Value V24029_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24029_5 = {S52, 0, &V24029_5_1227, &P24029_1};	/*Property = ScreenPixelsHeight*/
const Value V24029_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24029_4 = {S54, 0, &V24029_4_1229, &P24029_5};	/*Property = ScreenPixelsWidth*/
const Profile P24029 = {"24029", &P24029_4};	/*Profile ID - 24029*/
const Value V23178_1_1 = {S2, NULL};	/*Value = True*/
const Property P23178_1 = {S50, 0, &V23178_1_1, NULL};	/*Property = IsMobile*/
const Value V23178_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23178_5 = {S52, 0, &V23178_5_1458, &P23178_1};	/*Property = ScreenPixelsHeight*/
const Value V23178_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23178_4 = {S54, 0, &V23178_4_37617, &P23178_5};	/*Property = ScreenPixelsWidth*/
const Profile P23178 = {"23178", &P23178_4};	/*Profile ID - 23178*/
const Value V20528_1_1 = {S2, NULL};	/*Value = True*/
const Property P20528_1 = {S50, 0, &V20528_1_1, NULL};	/*Property = IsMobile*/
const Value V20528_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20528_5 = {S52, 0, &V20528_5_1458, &P20528_1};	/*Property = ScreenPixelsHeight*/
const Value V20528_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20528_4 = {S54, 0, &V20528_4_37617, &P20528_5};	/*Property = ScreenPixelsWidth*/
const Profile P20528 = {"20528", &P20528_4};	/*Profile ID - 20528*/
const Value V20512_1_1 = {S2, NULL};	/*Value = True*/
const Property P20512_1 = {S50, 0, &V20512_1_1, NULL};	/*Property = IsMobile*/
const Value V20512_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20512_5 = {S52, 0, &V20512_5_1222, &P20512_1};	/*Property = ScreenPixelsHeight*/
const Value V20512_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20512_4 = {S54, 0, &V20512_4_1229, &P20512_5};	/*Property = ScreenPixelsWidth*/
const Profile P20512 = {"20512", &P20512_4};	/*Profile ID - 20512*/
const Value V21038_1_1 = {S2, NULL};	/*Value = True*/
const Property P21038_1 = {S50, 0, &V21038_1_1, NULL};	/*Property = IsMobile*/
const Value V21038_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21038_5 = {S52, 0, &V21038_5_1227, &P21038_1};	/*Property = ScreenPixelsHeight*/
const Value V21038_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21038_4 = {S54, 0, &V21038_4_1229, &P21038_5};	/*Property = ScreenPixelsWidth*/
const Profile P21038 = {"21038", &P21038_4};	/*Profile ID - 21038*/
const Value V19956_1_1 = {S2, NULL};	/*Value = True*/
const Property P19956_1 = {S50, 0, &V19956_1_1, NULL};	/*Property = IsMobile*/
const Value V19956_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P19956_5 = {S52, 0, &V19956_5_1222, &P19956_1};	/*Property = ScreenPixelsHeight*/
const Value V19956_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P19956_4 = {S54, 0, &V19956_4_1224, &P19956_5};	/*Property = ScreenPixelsWidth*/
const Profile P19956 = {"19956", &P19956_4};	/*Profile ID - 19956*/
const Value V22048_1_1 = {S2, NULL};	/*Value = True*/
const Property P22048_1 = {S50, 0, &V22048_1_1, NULL};	/*Property = IsMobile*/
const Value V22048_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22048_5 = {S52, 0, &V22048_5_1277, &P22048_1};	/*Property = ScreenPixelsHeight*/
const Value V22048_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22048_4 = {S54, 0, &V22048_4_2336, &P22048_5};	/*Property = ScreenPixelsWidth*/
const Profile P22048 = {"22048", &P22048_4};	/*Profile ID - 22048*/
const Value V20146_1_1 = {S2, NULL};	/*Value = True*/
const Property P20146_1 = {S50, 0, &V20146_1_1, NULL};	/*Property = IsMobile*/
const Value V20146_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20146_5 = {S52, 0, &V20146_5_1227, &P20146_1};	/*Property = ScreenPixelsHeight*/
const Value V20146_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20146_4 = {S54, 0, &V20146_4_1229, &P20146_5};	/*Property = ScreenPixelsWidth*/
const Profile P20146 = {"20146", &P20146_4};	/*Profile ID - 20146*/
const Value V20258_1_1 = {S2, NULL};	/*Value = True*/
const Property P20258_1 = {S50, 0, &V20258_1_1, NULL};	/*Property = IsMobile*/
const Value V20258_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20258_5 = {S52, 0, &V20258_5_1227, &P20258_1};	/*Property = ScreenPixelsHeight*/
const Value V20258_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20258_4 = {S54, 0, &V20258_4_1229, &P20258_5};	/*Property = ScreenPixelsWidth*/
const Profile P20258 = {"20258", &P20258_4};	/*Profile ID - 20258*/
const Value V18455_1_1 = {S2, NULL};	/*Value = True*/
const Property P18455_1 = {S50, 0, &V18455_1_1, NULL};	/*Property = IsMobile*/
const Value V18455_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18455_5 = {S52, 0, &V18455_5_1227, &P18455_1};	/*Property = ScreenPixelsHeight*/
const Value V18455_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18455_4 = {S54, 0, &V18455_4_1229, &P18455_5};	/*Property = ScreenPixelsWidth*/
const Profile P18455 = {"18455", &P18455_4};	/*Profile ID - 18455*/
const Value V23024_1_1 = {S2, NULL};	/*Value = True*/
const Property P23024_1 = {S50, 0, &V23024_1_1, NULL};	/*Property = IsMobile*/
const Value V23024_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23024_5 = {S52, 0, &V23024_5_1222, &P23024_1};	/*Property = ScreenPixelsHeight*/
const Value V23024_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23024_4 = {S54, 0, &V23024_4_1224, &P23024_5};	/*Property = ScreenPixelsWidth*/
const Profile P23024 = {"23024", &P23024_4};	/*Profile ID - 23024*/
const Value V21231_1_1 = {S2, NULL};	/*Value = True*/
const Property P21231_1 = {S50, 0, &V21231_1_1, NULL};	/*Property = IsMobile*/
const Value V21231_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21231_5 = {S52, 0, &V21231_5_9981, &P21231_1};	/*Property = ScreenPixelsHeight*/
const Value V21231_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21231_4 = {S54, 0, &V21231_4_39098, &P21231_5};	/*Property = ScreenPixelsWidth*/
const Profile P21231 = {"21231", &P21231_4};	/*Profile ID - 21231*/
const Value V20985_1_1 = {S2, NULL};	/*Value = True*/
const Property P20985_1 = {S50, 0, &V20985_1_1, NULL};	/*Property = IsMobile*/
const Value V20985_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20985_5 = {S52, 0, &V20985_5_1458, &P20985_1};	/*Property = ScreenPixelsHeight*/
const Value V20985_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20985_4 = {S54, 0, &V20985_4_1460, &P20985_5};	/*Property = ScreenPixelsWidth*/
const Profile P20985 = {"20985", &P20985_4};	/*Profile ID - 20985*/
const Value V20476_1_1 = {S2, NULL};	/*Value = True*/
const Property P20476_1 = {S50, 0, &V20476_1_1, NULL};	/*Property = IsMobile*/
const Value V20476_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20476_5 = {S52, 0, &V20476_5_1227, &P20476_1};	/*Property = ScreenPixelsHeight*/
const Value V20476_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20476_4 = {S54, 0, &V20476_4_1229, &P20476_5};	/*Property = ScreenPixelsWidth*/
const Profile P20476 = {"20476", &P20476_4};	/*Profile ID - 20476*/
const Value V20835_1_1 = {S2, NULL};	/*Value = True*/
const Property P20835_1 = {S50, 0, &V20835_1_1, NULL};	/*Property = IsMobile*/
const Value V20835_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20835_5 = {S52, 0, &V20835_5_1227, &P20835_1};	/*Property = ScreenPixelsHeight*/
const Value V20835_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20835_4 = {S54, 0, &V20835_4_1229, &P20835_5};	/*Property = ScreenPixelsWidth*/
const Profile P20835 = {"20835", &P20835_4};	/*Profile ID - 20835*/
const Value V24553_1_1 = {S2, NULL};	/*Value = True*/
const Property P24553_1 = {S50, 0, &V24553_1_1, NULL};	/*Property = IsMobile*/
const Value V24553_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P24553_5 = {S52, 0, &V24553_5_1269, &P24553_1};	/*Property = ScreenPixelsHeight*/
const Value V24553_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24553_4 = {S54, 0, &V24553_4_1229, &P24553_5};	/*Property = ScreenPixelsWidth*/
const Profile P24553 = {"24553", &P24553_4};	/*Profile ID - 24553*/
const Value V21995_1_1 = {S2, NULL};	/*Value = True*/
const Property P21995_1 = {S50, 0, &V21995_1_1, NULL};	/*Property = IsMobile*/
const Value V21995_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P21995_5 = {S52, 0, &V21995_5_37569, &P21995_1};	/*Property = ScreenPixelsHeight*/
const Value V21995_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21995_4 = {S54, 0, &V21995_4_5645, &P21995_5};	/*Property = ScreenPixelsWidth*/
const Profile P21995 = {"21995", &P21995_4};	/*Profile ID - 21995*/
const Value V20740_1_1 = {S2, NULL};	/*Value = True*/
const Property P20740_1 = {S50, 0, &V20740_1_1, NULL};	/*Property = IsMobile*/
const Value V20740_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20740_5 = {S52, 0, &V20740_5_1227, &P20740_1};	/*Property = ScreenPixelsHeight*/
const Value V20740_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20740_4 = {S54, 0, &V20740_4_1229, &P20740_5};	/*Property = ScreenPixelsWidth*/
const Profile P20740 = {"20740", &P20740_4};	/*Profile ID - 20740*/
const Value V20020_1_1 = {S2, NULL};	/*Value = True*/
const Property P20020_1 = {S50, 0, &V20020_1_1, NULL};	/*Property = IsMobile*/
const Value V20020_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20020_5 = {S52, 0, &V20020_5_1277, &P20020_1};	/*Property = ScreenPixelsHeight*/
const Value V20020_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20020_4 = {S54, 0, &V20020_4_1279, &P20020_5};	/*Property = ScreenPixelsWidth*/
const Profile P20020 = {"20020", &P20020_4};	/*Profile ID - 20020*/
const Value V20514_1_1 = {S2, NULL};	/*Value = True*/
const Property P20514_1 = {S50, 0, &V20514_1_1, NULL};	/*Property = IsMobile*/
const Value V20514_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20514_5 = {S52, 0, &V20514_5_1227, &P20514_1};	/*Property = ScreenPixelsHeight*/
const Value V20514_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20514_4 = {S54, 0, &V20514_4_1229, &P20514_5};	/*Property = ScreenPixelsWidth*/
const Profile P20514 = {"20514", &P20514_4};	/*Profile ID - 20514*/
const Value V20248_1_1 = {S2, NULL};	/*Value = True*/
const Property P20248_1 = {S50, 0, &V20248_1_1, NULL};	/*Property = IsMobile*/
const Value V20248_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20248_5 = {S52, 0, &V20248_5_1227, &P20248_1};	/*Property = ScreenPixelsHeight*/
const Value V20248_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20248_4 = {S54, 0, &V20248_4_1229, &P20248_5};	/*Property = ScreenPixelsWidth*/
const Profile P20248 = {"20248", &P20248_4};	/*Profile ID - 20248*/
const Value V23885_1_1 = {S2, NULL};	/*Value = True*/
const Property P23885_1 = {S50, 0, &V23885_1_1, NULL};	/*Property = IsMobile*/
const Value V23885_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23885_5 = {S52, 0, &V23885_5_1458, &P23885_1};	/*Property = ScreenPixelsHeight*/
const Value V23885_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23885_4 = {S54, 0, &V23885_4_1460, &P23885_5};	/*Property = ScreenPixelsWidth*/
const Profile P23885 = {"23885", &P23885_4};	/*Profile ID - 23885*/
const Value V19864_1_1 = {S2, NULL};	/*Value = True*/
const Property P19864_1 = {S50, 0, &V19864_1_1, NULL};	/*Property = IsMobile*/
const Value V19864_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P19864_5 = {S52, 0, &V19864_5_1458, &P19864_1};	/*Property = ScreenPixelsHeight*/
const Value V19864_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P19864_4 = {S54, 0, &V19864_4_37617, &P19864_5};	/*Property = ScreenPixelsWidth*/
const Profile P19864 = {"19864", &P19864_4};	/*Profile ID - 19864*/
const Value V20883_1_1 = {S2, NULL};	/*Value = True*/
const Property P20883_1 = {S50, 0, &V20883_1_1, NULL};	/*Property = IsMobile*/
const Value V20883_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20883_5 = {S52, 0, &V20883_5_1403, &P20883_1};	/*Property = ScreenPixelsHeight*/
const Value V20883_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20883_4 = {S54, 0, &V20883_4_1404, &P20883_5};	/*Property = ScreenPixelsWidth*/
const Profile P20883 = {"20883", &P20883_4};	/*Profile ID - 20883*/
const Value V20137_1_1 = {S2, NULL};	/*Value = True*/
const Property P20137_1 = {S50, 0, &V20137_1_1, NULL};	/*Property = IsMobile*/
const Value V20137_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20137_5 = {S52, 0, &V20137_5_1227, &P20137_1};	/*Property = ScreenPixelsHeight*/
const Value V20137_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20137_4 = {S54, 0, &V20137_4_1229, &P20137_5};	/*Property = ScreenPixelsWidth*/
const Profile P20137 = {"20137", &P20137_4};	/*Profile ID - 20137*/
const Value V20867_1_1 = {S2, NULL};	/*Value = True*/
const Property P20867_1 = {S50, 0, &V20867_1_1, NULL};	/*Property = IsMobile*/
const Value V20867_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20867_5 = {S52, 0, &V20867_5_1458, &P20867_1};	/*Property = ScreenPixelsHeight*/
const Value V20867_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20867_4 = {S54, 0, &V20867_4_37617, &P20867_5};	/*Property = ScreenPixelsWidth*/
const Profile P20867 = {"20867", &P20867_4};	/*Profile ID - 20867*/
const Value V23731_1_1 = {S2, NULL};	/*Value = True*/
const Property P23731_1 = {S50, 0, &V23731_1_1, NULL};	/*Property = IsMobile*/
const Value V23731_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23731_5 = {S52, 0, &V23731_5_1222, &P23731_1};	/*Property = ScreenPixelsHeight*/
const Value V23731_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23731_4 = {S54, 0, &V23731_4_1224, &P23731_5};	/*Property = ScreenPixelsWidth*/
const Profile P23731 = {"23731", &P23731_4};	/*Profile ID - 23731*/
const Value V18718_1_1 = {S2, NULL};	/*Value = True*/
const Property P18718_1 = {S50, 0, &V18718_1_1, NULL};	/*Property = IsMobile*/
const Value V18718_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18718_5 = {S52, 0, &V18718_5_1227, &P18718_1};	/*Property = ScreenPixelsHeight*/
const Value V18718_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18718_4 = {S54, 0, &V18718_4_1229, &P18718_5};	/*Property = ScreenPixelsWidth*/
const Profile P18718 = {"18718", &P18718_4};	/*Profile ID - 18718*/
const Value V20928_1_1 = {S2, NULL};	/*Value = True*/
const Property P20928_1 = {S50, 0, &V20928_1_1, NULL};	/*Property = IsMobile*/
const Value V20928_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20928_5 = {S52, 0, &V20928_5_1222, &P20928_1};	/*Property = ScreenPixelsHeight*/
const Value V20928_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20928_4 = {S54, 0, &V20928_4_1224, &P20928_5};	/*Property = ScreenPixelsWidth*/
const Profile P20928 = {"20928", &P20928_4};	/*Profile ID - 20928*/
const Value V20139_1_2 = {S0, NULL};	/*Value = False*/
const Property P20139_1 = {S50, 0, &V20139_1_2, NULL};	/*Property = IsMobile*/
const Value V20139_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20139_5 = {S52, 0, &V20139_5_37, &P20139_1};	/*Property = ScreenPixelsHeight*/
const Value V20139_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20139_4 = {S54, 0, &V20139_4_36, &P20139_5};	/*Property = ScreenPixelsWidth*/
const Profile P20139 = {"20139", &P20139_4};	/*Profile ID - 20139*/
const Value V23090_1_1 = {S2, NULL};	/*Value = True*/
const Property P23090_1 = {S50, 0, &V23090_1_1, NULL};	/*Property = IsMobile*/
const Value V23090_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23090_5 = {S52, 0, &V23090_5_1227, &P23090_1};	/*Property = ScreenPixelsHeight*/
const Value V23090_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23090_4 = {S54, 0, &V23090_4_1229, &P23090_5};	/*Property = ScreenPixelsWidth*/
const Profile P23090 = {"23090", &P23090_4};	/*Profile ID - 23090*/
const Value V22027_1_1 = {S2, NULL};	/*Value = True*/
const Property P22027_1 = {S50, 0, &V22027_1_1, NULL};	/*Property = IsMobile*/
const Value V22027_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22027_5 = {S52, 0, &V22027_5_1277, &P22027_1};	/*Property = ScreenPixelsHeight*/
const Value V22027_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22027_4 = {S54, 0, &V22027_4_1279, &P22027_5};	/*Property = ScreenPixelsWidth*/
const Profile P22027 = {"22027", &P22027_4};	/*Profile ID - 22027*/
const Value V22225_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_279 = {S1, 0, &V22225_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22225_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_282 = {S3, 0, &V22225_282_57155, &P22225_279};	/*Property = BlobBuilder*/
const Value V22225_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_350 = {S4, 0, &V22225_350_57446, &P22225_282};	/*Property = Canvas*/
const Value V22225_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_283 = {S5, 0, &V22225_283_57161, &P22225_350};	/*Property = CssBackground*/
const Value V22225_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_284 = {S6, 0, &V22225_284_57164, &P22225_283};	/*Property = CssBorderImage*/
const Value V22225_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_285 = {S7, 0, &V22225_285_57167, &P22225_284};	/*Property = CssCanvas*/
const Value V22225_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_286 = {S8, 0, &V22225_286_57170, &P22225_285};	/*Property = CssColor*/
const Value V22225_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_237 = {S9, 0, &V22225_237_55786, &P22225_286};	/*Property = CssColumn*/
const Value V22225_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_287 = {S10, 0, &V22225_287_57173, &P22225_237};	/*Property = CssFlexbox*/
const Value V22225_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_288 = {S11, 0, &V22225_288_57176, &P22225_287};	/*Property = CssFont*/
const Value V22225_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_289 = {S12, 0, &V22225_289_86899, &P22225_288};	/*Property = CssImages*/
const Value V22225_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_291 = {S13, 0, &V22225_291_57182, &P22225_289};	/*Property = CssMediaQueries*/
const Value V22225_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_292 = {S14, 0, &V22225_292_57185, &P22225_291};	/*Property = CssMinMax*/
const Value V22225_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_293 = {S15, 0, &V22225_293_57188, &P22225_292};	/*Property = CssOverflow*/
const Value V22225_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_294 = {S16, 0, &V22225_294_57191, &P22225_293};	/*Property = CssPosition*/
const Value V22225_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_295 = {S17, 0, &V22225_295_57194, &P22225_294};	/*Property = CssText*/
const Value V22225_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_297 = {S18, 0, &V22225_297_57197, &P22225_295};	/*Property = CssTransforms*/
const Value V22225_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_241 = {S19, 0, &V22225_241_55801, &P22225_297};	/*Property = CssTransitions*/
const Value V22225_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_298 = {S20, 0, &V22225_298_57208, &P22225_241};	/*Property = CssUI*/
const Value V22225_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_300 = {S21, 0, &V22225_300_57217, &P22225_298};	/*Property = DataSet*/
const Value V22225_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_301 = {S22, 0, &V22225_301_57220, &P22225_300};	/*Property = DataUrl*/
const Value V22225_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_302 = {S23, 0, &V22225_302_57223, &P22225_301};	/*Property = DeviceOrientation*/
const Value V22225_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_304 = {S24, 0, &V22225_304_57229, &P22225_302};	/*Property = FileReader*/
const Value V22225_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_305 = {S25, 0, &V22225_305_57232, &P22225_304};	/*Property = FileSaver*/
const Value V22225_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_306 = {S26, 0, &V22225_306_57238, &P22225_305};	/*Property = FileWriter*/
const Value V22225_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_307 = {S27, 0, &V22225_307_57241, &P22225_306};	/*Property = FormData*/
const Value V22225_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_309 = {S28, 0, &V22225_309_57247, &P22225_307};	/*Property = Fullscreen*/
const Value V22225_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_310 = {S29, 0, &V22225_310_57250, &P22225_309};	/*Property = GeoLocation*/
const Value V22225_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_312 = {S30, 0, &V22225_312_57256, &P22225_310};	/*Property = History*/
const Value V22225_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_314 = {S31, 0, &V22225_314_57262, &P22225_312};	/*Property = Html5*/
const Value V22225_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_313 = {S32, 0, &V22225_313_57259, &P22225_314};	/*Property = Html-Media-Capture*/
const Value V22225_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_315 = {S33, 0, &V22225_315_57265, &P22225_313};	/*Property = Iframe*/
const Value V22225_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_316 = {S34, 0, &V22225_316_57268, &P22225_315};	/*Property = IndexedDB*/
const Value V22225_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_317 = {S35, 0, &V22225_317_57271, &P22225_316};	/*Property = Json*/
const Value V22225_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22225_15 = {S37, 0, &V22225_15_29, &P22225_317};	/*Property = LayoutEngine*/
const Value V22225_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_344 = {S38, 0, &V22225_344_57274, &P22225_15};	/*Property = Masking*/
const Value V22225_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_320 = {S39, 0, &V22225_320_57292, &P22225_344};	/*Property = PostMessage*/
const Value V22225_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_321 = {S40, 0, &V22225_321_57295, &P22225_320};	/*Property = Progress*/
const Value V22225_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_322 = {S41, 0, &V22225_322_57298, &P22225_321};	/*Property = Prompts*/
const Value V22225_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_323 = {S42, 0, &V22225_323_57301, &P22225_322};	/*Property = Selector*/
const Value V22225_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_324 = {S43, 0, &V22225_324_57307, &P22225_323};	/*Property = Svg*/
const Value V22225_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_327 = {S44, 0, &V22225_327_57316, &P22225_324};	/*Property = TouchEvents*/
const Value V22225_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_328 = {S45, 0, &V22225_328_57319, &P22225_327};	/*Property = Track*/
const Value V22225_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_329 = {S46, 0, &V22225_329_57325, &P22225_328};	/*Property = Video*/
const Value V22225_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_330 = {S47, 0, &V22225_330_57328, &P22225_329};	/*Property = Viewport*/
const Value V22225_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_332 = {S48, 0, &V22225_332_57340, &P22225_330};	/*Property = WebWorkers*/
const Value V22225_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22225_333 = {S49, 0, &V22225_333_57343, &P22225_332};	/*Property = Xhr2*/
const Profile P22225 = {"22225", &P22225_333};	/*Profile ID - 22225*/
const Value V22138_1_1 = {S2, NULL};	/*Value = True*/
const Property P22138_1 = {S50, 0, &V22138_1_1, NULL};	/*Property = IsMobile*/
const Value V22138_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22138_5 = {S52, 0, &V22138_5_1227, &P22138_1};	/*Property = ScreenPixelsHeight*/
const Value V22138_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22138_4 = {S54, 0, &V22138_4_1229, &P22138_5};	/*Property = ScreenPixelsWidth*/
const Profile P22138 = {"22138", &P22138_4};	/*Profile ID - 22138*/
const Value V22003_1_1 = {S2, NULL};	/*Value = True*/
const Property P22003_1 = {S50, 0, &V22003_1_1, NULL};	/*Property = IsMobile*/
const Value V22003_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22003_5 = {S52, 0, &V22003_5_1227, &P22003_1};	/*Property = ScreenPixelsHeight*/
const Value V22003_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22003_4 = {S54, 0, &V22003_4_1229, &P22003_5};	/*Property = ScreenPixelsWidth*/
const Profile P22003 = {"22003", &P22003_4};	/*Profile ID - 22003*/
const Value V20888_1_1 = {S2, NULL};	/*Value = True*/
const Property P20888_1 = {S50, 0, &V20888_1_1, NULL};	/*Property = IsMobile*/
const Value V20888_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20888_5 = {S52, 0, &V20888_5_1227, &P20888_1};	/*Property = ScreenPixelsHeight*/
const Value V20888_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20888_4 = {S54, 0, &V20888_4_1229, &P20888_5};	/*Property = ScreenPixelsWidth*/
const Profile P20888 = {"20888", &P20888_4};	/*Profile ID - 20888*/
const Value V20736_1_1 = {S2, NULL};	/*Value = True*/
const Property P20736_1 = {S50, 0, &V20736_1_1, NULL};	/*Property = IsMobile*/
const Value V20736_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20736_5 = {S52, 0, &V20736_5_1458, &P20736_1};	/*Property = ScreenPixelsHeight*/
const Value V20736_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20736_4 = {S54, 0, &V20736_4_1460, &P20736_5};	/*Property = ScreenPixelsWidth*/
const Profile P20736 = {"20736", &P20736_4};	/*Profile ID - 20736*/
const Value V22787_1_1 = {S2, NULL};	/*Value = True*/
const Property P22787_1 = {S50, 0, &V22787_1_1, NULL};	/*Property = IsMobile*/
const Value V22787_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22787_5 = {S52, 0, &V22787_5_1277, &P22787_1};	/*Property = ScreenPixelsHeight*/
const Value V22787_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22787_4 = {S54, 0, &V22787_4_1279, &P22787_5};	/*Property = ScreenPixelsWidth*/
const Profile P22787 = {"22787", &P22787_4};	/*Profile ID - 22787*/
const Value V21132_1_1 = {S2, NULL};	/*Value = True*/
const Property P21132_1 = {S50, 0, &V21132_1_1, NULL};	/*Property = IsMobile*/
const Value V21132_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21132_5 = {S52, 0, &V21132_5_1403, &P21132_1};	/*Property = ScreenPixelsHeight*/
const Value V21132_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21132_4 = {S54, 0, &V21132_4_1404, &P21132_5};	/*Property = ScreenPixelsWidth*/
const Profile P21132 = {"21132", &P21132_4};	/*Profile ID - 21132*/
const Value V20868_1_1 = {S2, NULL};	/*Value = True*/
const Property P20868_1 = {S50, 0, &V20868_1_1, NULL};	/*Property = IsMobile*/
const Value V20868_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20868_5 = {S52, 0, &V20868_5_9981, &P20868_1};	/*Property = ScreenPixelsHeight*/
const Value V20868_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20868_4 = {S54, 0, &V20868_4_2336, &P20868_5};	/*Property = ScreenPixelsWidth*/
const Profile P20868 = {"20868", &P20868_4};	/*Profile ID - 20868*/
const Value V20180_1_1 = {S2, NULL};	/*Value = True*/
const Property P20180_1 = {S50, 0, &V20180_1_1, NULL};	/*Property = IsMobile*/
const Value V20180_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20180_5 = {S52, 0, &V20180_5_1403, &P20180_1};	/*Property = ScreenPixelsHeight*/
const Value V20180_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20180_4 = {S54, 0, &V20180_4_1404, &P20180_5};	/*Property = ScreenPixelsWidth*/
const Profile P20180 = {"20180", &P20180_4};	/*Profile ID - 20180*/
const Value V20250_1_1 = {S2, NULL};	/*Value = True*/
const Property P20250_1 = {S50, 0, &V20250_1_1, NULL};	/*Property = IsMobile*/
const Value V20250_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20250_5 = {S52, 0, &V20250_5_1227, &P20250_1};	/*Property = ScreenPixelsHeight*/
const Value V20250_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20250_4 = {S54, 0, &V20250_4_1229, &P20250_5};	/*Property = ScreenPixelsWidth*/
const Profile P20250 = {"20250", &P20250_4};	/*Profile ID - 20250*/
const Value V20363_1_1 = {S2, NULL};	/*Value = True*/
const Property P20363_1 = {S50, 0, &V20363_1_1, NULL};	/*Property = IsMobile*/
const Value V20363_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20363_5 = {S52, 0, &V20363_5_1458, &P20363_1};	/*Property = ScreenPixelsHeight*/
const Value V20363_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20363_4 = {S54, 0, &V20363_4_1460, &P20363_5};	/*Property = ScreenPixelsWidth*/
const Profile P20363 = {"20363", &P20363_4};	/*Profile ID - 20363*/
const Value V20856_1_1 = {S2, NULL};	/*Value = True*/
const Property P20856_1 = {S50, 0, &V20856_1_1, NULL};	/*Property = IsMobile*/
const Value V20856_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20856_5 = {S52, 0, &V20856_5_1458, &P20856_1};	/*Property = ScreenPixelsHeight*/
const Value V20856_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20856_4 = {S54, 0, &V20856_4_37617, &P20856_5};	/*Property = ScreenPixelsWidth*/
const Profile P20856 = {"20856", &P20856_4};	/*Profile ID - 20856*/
const Value V20245_1_1 = {S2, NULL};	/*Value = True*/
const Property P20245_1 = {S50, 0, &V20245_1_1, NULL};	/*Property = IsMobile*/
const Value V20245_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20245_5 = {S52, 0, &V20245_5_9981, &P20245_1};	/*Property = ScreenPixelsHeight*/
const Value V20245_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20245_4 = {S54, 0, &V20245_4_2336, &P20245_5};	/*Property = ScreenPixelsWidth*/
const Profile P20245 = {"20245", &P20245_4};	/*Profile ID - 20245*/
const Value V20176_1_1 = {S2, NULL};	/*Value = True*/
const Property P20176_1 = {S50, 0, &V20176_1_1, NULL};	/*Property = IsMobile*/
const Value V20176_5_37526 = {S122, NULL};	/*Value = 1200*/
const Property P20176_5 = {S52, 0, &V20176_5_37526, &P20176_1};	/*Property = ScreenPixelsHeight*/
const Value V20176_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20176_4 = {S54, 0, &V20176_4_2336, &P20176_5};	/*Property = ScreenPixelsWidth*/
const Profile P20176 = {"20176", &P20176_4};	/*Profile ID - 20176*/
const Value V23893_1_1 = {S2, NULL};	/*Value = True*/
const Property P23893_1 = {S50, 0, &V23893_1_1, NULL};	/*Property = IsMobile*/
const Value V23893_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23893_5 = {S52, 0, &V23893_5_9981, &P23893_1};	/*Property = ScreenPixelsHeight*/
const Value V23893_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23893_4 = {S54, 0, &V23893_4_2336, &P23893_5};	/*Property = ScreenPixelsWidth*/
const Profile P23893 = {"23893", &P23893_4};	/*Profile ID - 23893*/
const Value V20923_1_1 = {S2, NULL};	/*Value = True*/
const Property P20923_1 = {S50, 0, &V20923_1_1, NULL};	/*Property = IsMobile*/
const Value V20923_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20923_5 = {S52, 0, &V20923_5_1458, &P20923_1};	/*Property = ScreenPixelsHeight*/
const Value V20923_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20923_4 = {S54, 0, &V20923_4_1460, &P20923_5};	/*Property = ScreenPixelsWidth*/
const Profile P20923 = {"20923", &P20923_4};	/*Profile ID - 20923*/
const Value V20216_1_1 = {S2, NULL};	/*Value = True*/
const Property P20216_1 = {S50, 0, &V20216_1_1, NULL};	/*Property = IsMobile*/
const Value V20216_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20216_5 = {S52, 0, &V20216_5_1458, &P20216_1};	/*Property = ScreenPixelsHeight*/
const Value V20216_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20216_4 = {S54, 0, &V20216_4_1460, &P20216_5};	/*Property = ScreenPixelsWidth*/
const Profile P20216 = {"20216", &P20216_4};	/*Profile ID - 20216*/
const Value V20300_1_1 = {S2, NULL};	/*Value = True*/
const Property P20300_1 = {S50, 0, &V20300_1_1, NULL};	/*Property = IsMobile*/
const Value V20300_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20300_5 = {S52, 0, &V20300_5_1227, &P20300_1};	/*Property = ScreenPixelsHeight*/
const Value V20300_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20300_4 = {S54, 0, &V20300_4_1229, &P20300_5};	/*Property = ScreenPixelsWidth*/
const Profile P20300 = {"20300", &P20300_4};	/*Profile ID - 20300*/
const Value V23824_1_1 = {S2, NULL};	/*Value = True*/
const Property P23824_1 = {S50, 0, &V23824_1_1, NULL};	/*Property = IsMobile*/
const Value V23824_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23824_5 = {S52, 0, &V23824_5_1227, &P23824_1};	/*Property = ScreenPixelsHeight*/
const Value V23824_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23824_4 = {S54, 0, &V23824_4_1460, &P23824_5};	/*Property = ScreenPixelsWidth*/
const Profile P23824 = {"23824", &P23824_4};	/*Profile ID - 23824*/
const Value V22115_1_1 = {S2, NULL};	/*Value = True*/
const Property P22115_1 = {S50, 0, &V22115_1_1, NULL};	/*Property = IsMobile*/
const Value V22115_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22115_5 = {S52, 0, &V22115_5_1277, &P22115_1};	/*Property = ScreenPixelsHeight*/
const Value V22115_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22115_4 = {S54, 0, &V22115_4_2336, &P22115_5};	/*Property = ScreenPixelsWidth*/
const Profile P22115 = {"22115", &P22115_4};	/*Profile ID - 22115*/
const Value V22564_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22564_279 = {S1, 0, &V22564_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22564_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22564_282 = {S3, 0, &V22564_282_57153, &P22564_279};	/*Property = BlobBuilder*/
const Value V22564_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22564_350 = {S4, 0, &V22564_350_57444, &P22564_282};	/*Property = Canvas*/
const Value V22564_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22564_283 = {S5, 0, &V22564_283_57159, &P22564_350};	/*Property = CssBackground*/
const Value V22564_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22564_284 = {S6, 0, &V22564_284_57163, &P22564_283};	/*Property = CssBorderImage*/
const Value V22564_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22564_285 = {S7, 0, &V22564_285_57166, &P22564_284};	/*Property = CssCanvas*/
const Value V22564_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22564_286 = {S8, 0, &V22564_286_57168, &P22564_285};	/*Property = CssColor*/
const Value V22564_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22564_237 = {S9, 0, &V22564_237_55786, &P22564_286};	/*Property = CssColumn*/
const Value V22564_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22564_287 = {S10, 0, &V22564_287_57172, &P22564_237};	/*Property = CssFlexbox*/
const Value V22564_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22564_288 = {S11, 0, &V22564_288_57175, &P22564_287};	/*Property = CssFont*/
const Value V22564_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22564_289 = {S12, 0, &V22564_289_57759, &P22564_288};	/*Property = CssImages*/
const Value V22564_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22564_291 = {S13, 0, &V22564_291_57181, &P22564_289};	/*Property = CssMediaQueries*/
const Value V22564_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22564_292 = {S14, 0, &V22564_292_57183, &P22564_291};	/*Property = CssMinMax*/
const Value V22564_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22564_293 = {S15, 0, &V22564_293_57187, &P22564_292};	/*Property = CssOverflow*/
const Value V22564_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22564_294 = {S16, 0, &V22564_294_57189, &P22564_293};	/*Property = CssPosition*/
const Value V22564_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22564_295 = {S17, 0, &V22564_295_57192, &P22564_294};	/*Property = CssText*/
const Value V22564_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22564_297 = {S18, 0, &V22564_297_57195, &P22564_295};	/*Property = CssTransforms*/
const Value V22564_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22564_241 = {S19, 0, &V22564_241_55799, &P22564_297};	/*Property = CssTransitions*/
const Value V22564_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22564_298 = {S20, 0, &V22564_298_57206, &P22564_241};	/*Property = CssUI*/
const Value V22564_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22564_300 = {S21, 0, &V22564_300_57216, &P22564_298};	/*Property = DataSet*/
const Value V22564_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22564_301 = {S22, 0, &V22564_301_57218, &P22564_300};	/*Property = DataUrl*/
const Value V22564_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22564_302 = {S23, 0, &V22564_302_57221, &P22564_301};	/*Property = DeviceOrientation*/
const Value V22564_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22564_304 = {S24, 0, &V22564_304_57227, &P22564_302};	/*Property = FileReader*/
const Value V22564_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22564_305 = {S25, 0, &V22564_305_57231, &P22564_304};	/*Property = FileSaver*/
const Value V22564_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22564_306 = {S26, 0, &V22564_306_57237, &P22564_305};	/*Property = FileWriter*/
const Value V22564_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22564_307 = {S27, 0, &V22564_307_57239, &P22564_306};	/*Property = FormData*/
const Value V22564_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22564_309 = {S28, 0, &V22564_309_57246, &P22564_307};	/*Property = Fullscreen*/
const Value V22564_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22564_310 = {S29, 0, &V22564_310_57248, &P22564_309};	/*Property = GeoLocation*/
const Value V22564_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22564_312 = {S30, 0, &V22564_312_57254, &P22564_310};	/*Property = History*/
const Value V22564_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22564_314 = {S31, 0, &V22564_314_57261, &P22564_312};	/*Property = Html5*/
const Value V22564_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22564_313 = {S32, 0, &V22564_313_57258, &P22564_314};	/*Property = Html-Media-Capture*/
const Value V22564_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22564_315 = {S33, 0, &V22564_315_57264, &P22564_313};	/*Property = Iframe*/
const Value V22564_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22564_316 = {S34, 0, &V22564_316_57266, &P22564_315};	/*Property = IndexedDB*/
const Value V22564_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22564_317 = {S35, 0, &V22564_317_57269, &P22564_316};	/*Property = Json*/
const Value V22564_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22564_15 = {S37, 0, &V22564_15_29, &P22564_317};	/*Property = LayoutEngine*/
const Value V22564_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22564_344 = {S38, 0, &V22564_344_57272, &P22564_15};	/*Property = Masking*/
const Value V22564_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22564_320 = {S39, 0, &V22564_320_57290, &P22564_344};	/*Property = PostMessage*/
const Value V22564_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22564_321 = {S40, 0, &V22564_321_57293, &P22564_320};	/*Property = Progress*/
const Value V22564_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22564_322 = {S41, 0, &V22564_322_57296, &P22564_321};	/*Property = Prompts*/
const Value V22564_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22564_323 = {S42, 0, &V22564_323_57299, &P22564_322};	/*Property = Selector*/
const Value V22564_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22564_324 = {S43, 0, &V22564_324_57306, &P22564_323};	/*Property = Svg*/
const Value V22564_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22564_327 = {S44, 0, &V22564_327_57315, &P22564_324};	/*Property = TouchEvents*/
const Value V22564_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22564_328 = {S45, 0, &V22564_328_57318, &P22564_327};	/*Property = Track*/
const Value V22564_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22564_329 = {S46, 0, &V22564_329_57323, &P22564_328};	/*Property = Video*/
const Value V22564_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22564_330 = {S47, 0, &V22564_330_57326, &P22564_329};	/*Property = Viewport*/
const Value V22564_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22564_332 = {S48, 0, &V22564_332_57339, &P22564_330};	/*Property = WebWorkers*/
const Value V22564_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22564_333 = {S49, 0, &V22564_333_57341, &P22564_332};	/*Property = Xhr2*/
const Profile P22564 = {"22564", &P22564_333};	/*Profile ID - 22564*/
const Value V20420_1_1 = {S2, NULL};	/*Value = True*/
const Property P20420_1 = {S50, 0, &V20420_1_1, NULL};	/*Property = IsMobile*/
const Value V20420_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20420_5 = {S52, 0, &V20420_5_1458, &P20420_1};	/*Property = ScreenPixelsHeight*/
const Value V20420_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20420_4 = {S54, 0, &V20420_4_37617, &P20420_5};	/*Property = ScreenPixelsWidth*/
const Profile P20420 = {"20420", &P20420_4};	/*Profile ID - 20420*/
const Value V20915_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_279 = {S1, 0, &V20915_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20915_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_282 = {S3, 0, &V20915_282_57155, &P20915_279};	/*Property = BlobBuilder*/
const Value V20915_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_350 = {S4, 0, &V20915_350_57446, &P20915_282};	/*Property = Canvas*/
const Value V20915_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_283 = {S5, 0, &V20915_283_57161, &P20915_350};	/*Property = CssBackground*/
const Value V20915_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_284 = {S6, 0, &V20915_284_57164, &P20915_283};	/*Property = CssBorderImage*/
const Value V20915_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_285 = {S7, 0, &V20915_285_57167, &P20915_284};	/*Property = CssCanvas*/
const Value V20915_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_286 = {S8, 0, &V20915_286_57170, &P20915_285};	/*Property = CssColor*/
const Value V20915_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_237 = {S9, 0, &V20915_237_55786, &P20915_286};	/*Property = CssColumn*/
const Value V20915_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_287 = {S10, 0, &V20915_287_57173, &P20915_237};	/*Property = CssFlexbox*/
const Value V20915_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_288 = {S11, 0, &V20915_288_57176, &P20915_287};	/*Property = CssFont*/
const Value V20915_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_289 = {S12, 0, &V20915_289_86899, &P20915_288};	/*Property = CssImages*/
const Value V20915_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_291 = {S13, 0, &V20915_291_57182, &P20915_289};	/*Property = CssMediaQueries*/
const Value V20915_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_292 = {S14, 0, &V20915_292_57185, &P20915_291};	/*Property = CssMinMax*/
const Value V20915_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_293 = {S15, 0, &V20915_293_57188, &P20915_292};	/*Property = CssOverflow*/
const Value V20915_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_294 = {S16, 0, &V20915_294_57191, &P20915_293};	/*Property = CssPosition*/
const Value V20915_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_295 = {S17, 0, &V20915_295_57194, &P20915_294};	/*Property = CssText*/
const Value V20915_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_297 = {S18, 0, &V20915_297_57197, &P20915_295};	/*Property = CssTransforms*/
const Value V20915_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_241 = {S19, 0, &V20915_241_55801, &P20915_297};	/*Property = CssTransitions*/
const Value V20915_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_298 = {S20, 0, &V20915_298_57208, &P20915_241};	/*Property = CssUI*/
const Value V20915_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_300 = {S21, 0, &V20915_300_57217, &P20915_298};	/*Property = DataSet*/
const Value V20915_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_301 = {S22, 0, &V20915_301_57220, &P20915_300};	/*Property = DataUrl*/
const Value V20915_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_302 = {S23, 0, &V20915_302_57223, &P20915_301};	/*Property = DeviceOrientation*/
const Value V20915_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_304 = {S24, 0, &V20915_304_57229, &P20915_302};	/*Property = FileReader*/
const Value V20915_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_305 = {S25, 0, &V20915_305_57232, &P20915_304};	/*Property = FileSaver*/
const Value V20915_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_306 = {S26, 0, &V20915_306_57238, &P20915_305};	/*Property = FileWriter*/
const Value V20915_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_307 = {S27, 0, &V20915_307_57241, &P20915_306};	/*Property = FormData*/
const Value V20915_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_309 = {S28, 0, &V20915_309_57247, &P20915_307};	/*Property = Fullscreen*/
const Value V20915_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_310 = {S29, 0, &V20915_310_57250, &P20915_309};	/*Property = GeoLocation*/
const Value V20915_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_312 = {S30, 0, &V20915_312_57256, &P20915_310};	/*Property = History*/
const Value V20915_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_314 = {S31, 0, &V20915_314_57262, &P20915_312};	/*Property = Html5*/
const Value V20915_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_313 = {S32, 0, &V20915_313_57259, &P20915_314};	/*Property = Html-Media-Capture*/
const Value V20915_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_315 = {S33, 0, &V20915_315_57265, &P20915_313};	/*Property = Iframe*/
const Value V20915_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_316 = {S34, 0, &V20915_316_57268, &P20915_315};	/*Property = IndexedDB*/
const Value V20915_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_317 = {S35, 0, &V20915_317_57271, &P20915_316};	/*Property = Json*/
const Value V20915_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P20915_15 = {S37, 0, &V20915_15_28, &P20915_317};	/*Property = LayoutEngine*/
const Value V20915_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_344 = {S38, 0, &V20915_344_57274, &P20915_15};	/*Property = Masking*/
const Value V20915_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_320 = {S39, 0, &V20915_320_57292, &P20915_344};	/*Property = PostMessage*/
const Value V20915_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_321 = {S40, 0, &V20915_321_57295, &P20915_320};	/*Property = Progress*/
const Value V20915_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_322 = {S41, 0, &V20915_322_57298, &P20915_321};	/*Property = Prompts*/
const Value V20915_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_323 = {S42, 0, &V20915_323_57301, &P20915_322};	/*Property = Selector*/
const Value V20915_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_324 = {S43, 0, &V20915_324_57307, &P20915_323};	/*Property = Svg*/
const Value V20915_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_327 = {S44, 0, &V20915_327_57316, &P20915_324};	/*Property = TouchEvents*/
const Value V20915_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_328 = {S45, 0, &V20915_328_57319, &P20915_327};	/*Property = Track*/
const Value V20915_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_329 = {S46, 0, &V20915_329_57325, &P20915_328};	/*Property = Video*/
const Value V20915_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_330 = {S47, 0, &V20915_330_57328, &P20915_329};	/*Property = Viewport*/
const Value V20915_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_332 = {S48, 0, &V20915_332_57340, &P20915_330};	/*Property = WebWorkers*/
const Value V20915_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20915_333 = {S49, 0, &V20915_333_57343, &P20915_332};	/*Property = Xhr2*/
const Profile P20915 = {"20915", &P20915_333};	/*Profile ID - 20915*/
const Value V20260_1_1 = {S2, NULL};	/*Value = True*/
const Property P20260_1 = {S50, 0, &V20260_1_1, NULL};	/*Property = IsMobile*/
const Value V20260_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20260_5 = {S52, 0, &V20260_5_1458, &P20260_1};	/*Property = ScreenPixelsHeight*/
const Value V20260_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20260_4 = {S54, 0, &V20260_4_1460, &P20260_5};	/*Property = ScreenPixelsWidth*/
const Profile P20260 = {"20260", &P20260_4};	/*Profile ID - 20260*/
const Value V22169_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22169_279 = {S1, 0, &V22169_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22169_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22169_282 = {S3, 0, &V22169_282_57154, &P22169_279};	/*Property = BlobBuilder*/
const Value V22169_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22169_350 = {S4, 0, &V22169_350_57444, &P22169_282};	/*Property = Canvas*/
const Value V22169_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22169_283 = {S5, 0, &V22169_283_57159, &P22169_350};	/*Property = CssBackground*/
const Value V22169_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22169_284 = {S6, 0, &V22169_284_57163, &P22169_283};	/*Property = CssBorderImage*/
const Value V22169_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22169_285 = {S7, 0, &V22169_285_57166, &P22169_284};	/*Property = CssCanvas*/
const Value V22169_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22169_286 = {S8, 0, &V22169_286_57168, &P22169_285};	/*Property = CssColor*/
const Value V22169_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22169_237 = {S9, 0, &V22169_237_55786, &P22169_286};	/*Property = CssColumn*/
const Value V22169_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22169_287 = {S10, 0, &V22169_287_57172, &P22169_237};	/*Property = CssFlexbox*/
const Value V22169_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22169_288 = {S11, 0, &V22169_288_57174, &P22169_287};	/*Property = CssFont*/
const Value V22169_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22169_289 = {S12, 0, &V22169_289_57759, &P22169_288};	/*Property = CssImages*/
const Value V22169_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22169_291 = {S13, 0, &V22169_291_57180, &P22169_289};	/*Property = CssMediaQueries*/
const Value V22169_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22169_292 = {S14, 0, &V22169_292_57183, &P22169_291};	/*Property = CssMinMax*/
const Value V22169_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22169_293 = {S15, 0, &V22169_293_57187, &P22169_292};	/*Property = CssOverflow*/
const Value V22169_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22169_294 = {S16, 0, &V22169_294_57189, &P22169_293};	/*Property = CssPosition*/
const Value V22169_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22169_295 = {S17, 0, &V22169_295_57192, &P22169_294};	/*Property = CssText*/
const Value V22169_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22169_297 = {S18, 0, &V22169_297_57195, &P22169_295};	/*Property = CssTransforms*/
const Value V22169_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22169_241 = {S19, 0, &V22169_241_55799, &P22169_297};	/*Property = CssTransitions*/
const Value V22169_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22169_298 = {S20, 0, &V22169_298_57206, &P22169_241};	/*Property = CssUI*/
const Value V22169_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22169_300 = {S21, 0, &V22169_300_57216, &P22169_298};	/*Property = DataSet*/
const Value V22169_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22169_301 = {S22, 0, &V22169_301_57218, &P22169_300};	/*Property = DataUrl*/
const Value V22169_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22169_302 = {S23, 0, &V22169_302_57221, &P22169_301};	/*Property = DeviceOrientation*/
const Value V22169_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22169_304 = {S24, 0, &V22169_304_57227, &P22169_302};	/*Property = FileReader*/
const Value V22169_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22169_305 = {S25, 0, &V22169_305_57231, &P22169_304};	/*Property = FileSaver*/
const Value V22169_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22169_306 = {S26, 0, &V22169_306_57237, &P22169_305};	/*Property = FileWriter*/
const Value V22169_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22169_307 = {S27, 0, &V22169_307_57239, &P22169_306};	/*Property = FormData*/
const Value V22169_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22169_309 = {S28, 0, &V22169_309_57246, &P22169_307};	/*Property = Fullscreen*/
const Value V22169_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22169_310 = {S29, 0, &V22169_310_57248, &P22169_309};	/*Property = GeoLocation*/
const Value V22169_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22169_312 = {S30, 0, &V22169_312_57254, &P22169_310};	/*Property = History*/
const Value V22169_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22169_314 = {S31, 0, &V22169_314_57261, &P22169_312};	/*Property = Html5*/
const Value V22169_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22169_313 = {S32, 0, &V22169_313_57258, &P22169_314};	/*Property = Html-Media-Capture*/
const Value V22169_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22169_315 = {S33, 0, &V22169_315_57264, &P22169_313};	/*Property = Iframe*/
const Value V22169_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22169_316 = {S34, 0, &V22169_316_57266, &P22169_315};	/*Property = IndexedDB*/
const Value V22169_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22169_317 = {S35, 0, &V22169_317_57269, &P22169_316};	/*Property = Json*/
const Value V22169_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22169_15 = {S37, 0, &V22169_15_29, &P22169_317};	/*Property = LayoutEngine*/
const Value V22169_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22169_344 = {S38, 0, &V22169_344_57272, &P22169_15};	/*Property = Masking*/
const Value V22169_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22169_320 = {S39, 0, &V22169_320_57290, &P22169_344};	/*Property = PostMessage*/
const Value V22169_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22169_321 = {S40, 0, &V22169_321_57293, &P22169_320};	/*Property = Progress*/
const Value V22169_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22169_322 = {S41, 0, &V22169_322_57296, &P22169_321};	/*Property = Prompts*/
const Value V22169_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22169_323 = {S42, 0, &V22169_323_57299, &P22169_322};	/*Property = Selector*/
const Value V22169_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22169_324 = {S43, 0, &V22169_324_57306, &P22169_323};	/*Property = Svg*/
const Value V22169_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22169_327 = {S44, 0, &V22169_327_57314, &P22169_324};	/*Property = TouchEvents*/
const Value V22169_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22169_328 = {S45, 0, &V22169_328_57318, &P22169_327};	/*Property = Track*/
const Value V22169_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22169_329 = {S46, 0, &V22169_329_57323, &P22169_328};	/*Property = Video*/
const Value V22169_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22169_330 = {S47, 0, &V22169_330_57326, &P22169_329};	/*Property = Viewport*/
const Value V22169_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22169_332 = {S48, 0, &V22169_332_57338, &P22169_330};	/*Property = WebWorkers*/
const Value V22169_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22169_333 = {S49, 0, &V22169_333_57341, &P22169_332};	/*Property = Xhr2*/
const Profile P22169 = {"22169", &P22169_333};	/*Profile ID - 22169*/
const Value V20257_1_1 = {S2, NULL};	/*Value = True*/
const Property P20257_1 = {S50, 0, &V20257_1_1, NULL};	/*Property = IsMobile*/
const Value V20257_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20257_5 = {S52, 0, &V20257_5_1458, &P20257_1};	/*Property = ScreenPixelsHeight*/
const Value V20257_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20257_4 = {S54, 0, &V20257_4_37617, &P20257_5};	/*Property = ScreenPixelsWidth*/
const Profile P20257 = {"20257", &P20257_4};	/*Profile ID - 20257*/
const Value V23868_1_1 = {S2, NULL};	/*Value = True*/
const Property P23868_1 = {S50, 0, &V23868_1_1, NULL};	/*Property = IsMobile*/
const Value V23868_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23868_5 = {S52, 0, &V23868_5_1458, &P23868_1};	/*Property = ScreenPixelsHeight*/
const Value V23868_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23868_4 = {S54, 0, &V23868_4_37617, &P23868_5};	/*Property = ScreenPixelsWidth*/
const Profile P23868 = {"23868", &P23868_4};	/*Profile ID - 23868*/
const Value V22556_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22556_279 = {S1, 0, &V22556_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22556_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22556_282 = {S3, 0, &V22556_282_57154, &P22556_279};	/*Property = BlobBuilder*/
const Value V22556_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22556_350 = {S4, 0, &V22556_350_57444, &P22556_282};	/*Property = Canvas*/
const Value V22556_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22556_283 = {S5, 0, &V22556_283_57159, &P22556_350};	/*Property = CssBackground*/
const Value V22556_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22556_284 = {S6, 0, &V22556_284_57163, &P22556_283};	/*Property = CssBorderImage*/
const Value V22556_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22556_285 = {S7, 0, &V22556_285_57166, &P22556_284};	/*Property = CssCanvas*/
const Value V22556_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22556_286 = {S8, 0, &V22556_286_57168, &P22556_285};	/*Property = CssColor*/
const Value V22556_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22556_237 = {S9, 0, &V22556_237_55786, &P22556_286};	/*Property = CssColumn*/
const Value V22556_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22556_287 = {S10, 0, &V22556_287_57172, &P22556_237};	/*Property = CssFlexbox*/
const Value V22556_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22556_288 = {S11, 0, &V22556_288_57175, &P22556_287};	/*Property = CssFont*/
const Value V22556_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22556_289 = {S12, 0, &V22556_289_57759, &P22556_288};	/*Property = CssImages*/
const Value V22556_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22556_291 = {S13, 0, &V22556_291_57180, &P22556_289};	/*Property = CssMediaQueries*/
const Value V22556_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22556_292 = {S14, 0, &V22556_292_57183, &P22556_291};	/*Property = CssMinMax*/
const Value V22556_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22556_293 = {S15, 0, &V22556_293_57187, &P22556_292};	/*Property = CssOverflow*/
const Value V22556_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22556_294 = {S16, 0, &V22556_294_57189, &P22556_293};	/*Property = CssPosition*/
const Value V22556_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22556_295 = {S17, 0, &V22556_295_57192, &P22556_294};	/*Property = CssText*/
const Value V22556_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22556_297 = {S18, 0, &V22556_297_57195, &P22556_295};	/*Property = CssTransforms*/
const Value V22556_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22556_241 = {S19, 0, &V22556_241_55799, &P22556_297};	/*Property = CssTransitions*/
const Value V22556_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22556_298 = {S20, 0, &V22556_298_57206, &P22556_241};	/*Property = CssUI*/
const Value V22556_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22556_300 = {S21, 0, &V22556_300_57216, &P22556_298};	/*Property = DataSet*/
const Value V22556_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22556_301 = {S22, 0, &V22556_301_57218, &P22556_300};	/*Property = DataUrl*/
const Value V22556_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22556_302 = {S23, 0, &V22556_302_57221, &P22556_301};	/*Property = DeviceOrientation*/
const Value V22556_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22556_304 = {S24, 0, &V22556_304_57227, &P22556_302};	/*Property = FileReader*/
const Value V22556_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22556_305 = {S25, 0, &V22556_305_57231, &P22556_304};	/*Property = FileSaver*/
const Value V22556_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22556_306 = {S26, 0, &V22556_306_57237, &P22556_305};	/*Property = FileWriter*/
const Value V22556_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22556_307 = {S27, 0, &V22556_307_57239, &P22556_306};	/*Property = FormData*/
const Value V22556_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22556_309 = {S28, 0, &V22556_309_57246, &P22556_307};	/*Property = Fullscreen*/
const Value V22556_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22556_310 = {S29, 0, &V22556_310_57248, &P22556_309};	/*Property = GeoLocation*/
const Value V22556_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22556_312 = {S30, 0, &V22556_312_57254, &P22556_310};	/*Property = History*/
const Value V22556_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22556_314 = {S31, 0, &V22556_314_57261, &P22556_312};	/*Property = Html5*/
const Value V22556_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22556_313 = {S32, 0, &V22556_313_57258, &P22556_314};	/*Property = Html-Media-Capture*/
const Value V22556_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22556_315 = {S33, 0, &V22556_315_57264, &P22556_313};	/*Property = Iframe*/
const Value V22556_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22556_316 = {S34, 0, &V22556_316_57266, &P22556_315};	/*Property = IndexedDB*/
const Value V22556_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22556_317 = {S35, 0, &V22556_317_57269, &P22556_316};	/*Property = Json*/
const Value V22556_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22556_15 = {S37, 0, &V22556_15_29, &P22556_317};	/*Property = LayoutEngine*/
const Value V22556_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22556_344 = {S38, 0, &V22556_344_57272, &P22556_15};	/*Property = Masking*/
const Value V22556_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22556_320 = {S39, 0, &V22556_320_57290, &P22556_344};	/*Property = PostMessage*/
const Value V22556_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22556_321 = {S40, 0, &V22556_321_57293, &P22556_320};	/*Property = Progress*/
const Value V22556_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22556_322 = {S41, 0, &V22556_322_57296, &P22556_321};	/*Property = Prompts*/
const Value V22556_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22556_323 = {S42, 0, &V22556_323_57299, &P22556_322};	/*Property = Selector*/
const Value V22556_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22556_324 = {S43, 0, &V22556_324_57306, &P22556_323};	/*Property = Svg*/
const Value V22556_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22556_327 = {S44, 0, &V22556_327_57315, &P22556_324};	/*Property = TouchEvents*/
const Value V22556_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22556_328 = {S45, 0, &V22556_328_57318, &P22556_327};	/*Property = Track*/
const Value V22556_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22556_329 = {S46, 0, &V22556_329_57323, &P22556_328};	/*Property = Video*/
const Value V22556_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22556_330 = {S47, 0, &V22556_330_57326, &P22556_329};	/*Property = Viewport*/
const Value V22556_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22556_332 = {S48, 0, &V22556_332_57338, &P22556_330};	/*Property = WebWorkers*/
const Value V22556_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22556_333 = {S49, 0, &V22556_333_57341, &P22556_332};	/*Property = Xhr2*/
const Profile P22556 = {"22556", &P22556_333};	/*Profile ID - 22556*/
const Value V19764_1_1 = {S2, NULL};	/*Value = True*/
const Property P19764_1 = {S50, 0, &V19764_1_1, NULL};	/*Property = IsMobile*/
const Value V19764_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P19764_5 = {S52, 0, &V19764_5_59138, &P19764_1};	/*Property = ScreenPixelsHeight*/
const Value V19764_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P19764_4 = {S54, 0, &V19764_4_59970, &P19764_5};	/*Property = ScreenPixelsWidth*/
const Profile P19764 = {"19764", &P19764_4};	/*Profile ID - 19764*/
const Value V19763_1_1 = {S2, NULL};	/*Value = True*/
const Property P19763_1 = {S50, 0, &V19763_1_1, NULL};	/*Property = IsMobile*/
const Value V19763_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P19763_5 = {S52, 0, &V19763_5_59138, &P19763_1};	/*Property = ScreenPixelsHeight*/
const Value V19763_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P19763_4 = {S54, 0, &V19763_4_59970, &P19763_5};	/*Property = ScreenPixelsWidth*/
const Profile P19763 = {"19763", &P19763_4};	/*Profile ID - 19763*/
const Value V19776_1_1 = {S2, NULL};	/*Value = True*/
const Property P19776_1 = {S50, 0, &V19776_1_1, NULL};	/*Property = IsMobile*/
const Value V19776_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P19776_5 = {S52, 0, &V19776_5_59138, &P19776_1};	/*Property = ScreenPixelsHeight*/
const Value V19776_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P19776_4 = {S54, 0, &V19776_4_59970, &P19776_5};	/*Property = ScreenPixelsWidth*/
const Profile P19776 = {"19776", &P19776_4};	/*Profile ID - 19776*/
const Value V19777_1_1 = {S2, NULL};	/*Value = True*/
const Property P19777_1 = {S50, 0, &V19777_1_1, NULL};	/*Property = IsMobile*/
const Value V19777_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19777_5 = {S52, 0, &V19777_5_1227, &P19777_1};	/*Property = ScreenPixelsHeight*/
const Value V19777_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19777_4 = {S54, 0, &V19777_4_1229, &P19777_5};	/*Property = ScreenPixelsWidth*/
const Profile P19777 = {"19777", &P19777_4};	/*Profile ID - 19777*/
const Value V19749_1_1 = {S2, NULL};	/*Value = True*/
const Property P19749_1 = {S50, 0, &V19749_1_1, NULL};	/*Property = IsMobile*/
const Value V19749_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P19749_5 = {S52, 0, &V19749_5_1380, &P19749_1};	/*Property = ScreenPixelsHeight*/
const Value V19749_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P19749_4 = {S54, 0, &V19749_4_1279, &P19749_5};	/*Property = ScreenPixelsWidth*/
const Profile P19749 = {"19749", &P19749_4};	/*Profile ID - 19749*/
const Value V19747_1_1 = {S2, NULL};	/*Value = True*/
const Property P19747_1 = {S50, 0, &V19747_1_1, NULL};	/*Property = IsMobile*/
const Value V19747_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P19747_5 = {S52, 0, &V19747_5_1222, &P19747_1};	/*Property = ScreenPixelsHeight*/
const Value V19747_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P19747_4 = {S54, 0, &V19747_4_1224, &P19747_5};	/*Property = ScreenPixelsWidth*/
const Profile P19747 = {"19747", &P19747_4};	/*Profile ID - 19747*/
const Value V18719_1_1 = {S2, NULL};	/*Value = True*/
const Property P18719_1 = {S50, 0, &V18719_1_1, NULL};	/*Property = IsMobile*/
const Value V18719_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P18719_5 = {S52, 0, &V18719_5_1277, &P18719_1};	/*Property = ScreenPixelsHeight*/
const Value V18719_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P18719_4 = {S54, 0, &V18719_4_1279, &P18719_5};	/*Property = ScreenPixelsWidth*/
const Profile P18719 = {"18719", &P18719_4};	/*Profile ID - 18719*/
const Value V19783_1_1 = {S2, NULL};	/*Value = True*/
const Property P19783_1 = {S50, 0, &V19783_1_1, NULL};	/*Property = IsMobile*/
const Value V19783_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P19783_5 = {S52, 0, &V19783_5_1458, &P19783_1};	/*Property = ScreenPixelsHeight*/
const Value V19783_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P19783_4 = {S54, 0, &V19783_4_1460, &P19783_5};	/*Property = ScreenPixelsWidth*/
const Profile P19783 = {"19783", &P19783_4};	/*Profile ID - 19783*/
const Value V19782_1_1 = {S2, NULL};	/*Value = True*/
const Property P19782_1 = {S50, 0, &V19782_1_1, NULL};	/*Property = IsMobile*/
const Value V19782_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P19782_5 = {S52, 0, &V19782_5_1458, &P19782_1};	/*Property = ScreenPixelsHeight*/
const Value V19782_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P19782_4 = {S54, 0, &V19782_4_1460, &P19782_5};	/*Property = ScreenPixelsWidth*/
const Profile P19782 = {"19782", &P19782_4};	/*Profile ID - 19782*/
const Value V19781_1_1 = {S2, NULL};	/*Value = True*/
const Property P19781_1 = {S50, 0, &V19781_1_1, NULL};	/*Property = IsMobile*/
const Value V19781_5_62105 = {S144, NULL};	/*Value = 2048*/
const Property P19781_5 = {S52, 0, &V19781_5_62105, &P19781_1};	/*Property = ScreenPixelsHeight*/
const Value V19781_4_62106 = {S145, NULL};	/*Value = 1536*/
const Property P19781_4 = {S54, 0, &V19781_4_62106, &P19781_5};	/*Property = ScreenPixelsWidth*/
const Profile P19781 = {"19781", &P19781_4};	/*Profile ID - 19781*/
const Value V20749_1_1 = {S2, NULL};	/*Value = True*/
const Property P20749_1 = {S50, 0, &V20749_1_1, NULL};	/*Property = IsMobile*/
const Value V20749_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20749_5 = {S52, 0, &V20749_5_9981, &P20749_1};	/*Property = ScreenPixelsHeight*/
const Value V20749_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20749_4 = {S54, 0, &V20749_4_39098, &P20749_5};	/*Property = ScreenPixelsWidth*/
const Profile P20749 = {"20749", &P20749_4};	/*Profile ID - 20749*/
const Value V19770_1_1 = {S2, NULL};	/*Value = True*/
const Property P19770_1 = {S50, 0, &V19770_1_1, NULL};	/*Property = IsMobile*/
const Value V19770_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19770_5 = {S52, 0, &V19770_5_1227, &P19770_1};	/*Property = ScreenPixelsHeight*/
const Value V19770_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19770_4 = {S54, 0, &V19770_4_1229, &P19770_5};	/*Property = ScreenPixelsWidth*/
const Profile P19770 = {"19770", &P19770_4};	/*Profile ID - 19770*/
const Value V19768_1_1 = {S2, NULL};	/*Value = True*/
const Property P19768_1 = {S50, 0, &V19768_1_1, NULL};	/*Property = IsMobile*/
const Value V19768_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P19768_5 = {S52, 0, &V19768_5_1277, &P19768_1};	/*Property = ScreenPixelsHeight*/
const Value V19768_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P19768_4 = {S54, 0, &V19768_4_1279, &P19768_5};	/*Property = ScreenPixelsWidth*/
const Profile P19768 = {"19768", &P19768_4};	/*Profile ID - 19768*/
const Value V20070_1_1 = {S2, NULL};	/*Value = True*/
const Property P20070_1 = {S50, 0, &V20070_1_1, NULL};	/*Property = IsMobile*/
const Value V20070_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P20070_5 = {S52, 0, &V20070_5_1233, &P20070_1};	/*Property = ScreenPixelsHeight*/
const Value V20070_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P20070_4 = {S54, 0, &V20070_4_1235, &P20070_5};	/*Property = ScreenPixelsWidth*/
const Profile P20070 = {"20070", &P20070_4};	/*Profile ID - 20070*/
const Value V20723_1_1 = {S2, NULL};	/*Value = True*/
const Property P20723_1 = {S50, 0, &V20723_1_1, NULL};	/*Property = IsMobile*/
const Value V20723_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20723_5 = {S52, 0, &V20723_5_1227, &P20723_1};	/*Property = ScreenPixelsHeight*/
const Value V20723_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20723_4 = {S54, 0, &V20723_4_1229, &P20723_5};	/*Property = ScreenPixelsWidth*/
const Profile P20723 = {"20723", &P20723_4};	/*Profile ID - 20723*/
const Value V24547_1_1 = {S2, NULL};	/*Value = True*/
const Property P24547_1 = {S50, 0, &V24547_1_1, NULL};	/*Property = IsMobile*/
const Value V24547_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24547_5 = {S52, 0, &V24547_5_1277, &P24547_1};	/*Property = ScreenPixelsHeight*/
const Value V24547_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24547_4 = {S54, 0, &V24547_4_1279, &P24547_5};	/*Property = ScreenPixelsWidth*/
const Profile P24547 = {"24547", &P24547_4};	/*Profile ID - 24547*/
const Value V22800_1_1 = {S2, NULL};	/*Value = True*/
const Property P22800_1 = {S50, 0, &V22800_1_1, NULL};	/*Property = IsMobile*/
const Value V22800_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P22800_5 = {S52, 0, &V22800_5_1380, &P22800_1};	/*Property = ScreenPixelsHeight*/
const Value V22800_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22800_4 = {S54, 0, &V22800_4_1279, &P22800_5};	/*Property = ScreenPixelsWidth*/
const Profile P22800 = {"22800", &P22800_4};	/*Profile ID - 22800*/
const Value V22147_1_1 = {S2, NULL};	/*Value = True*/
const Property P22147_1 = {S50, 0, &V22147_1_1, NULL};	/*Property = IsMobile*/
const Value V22147_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22147_5 = {S52, 0, &V22147_5_1227, &P22147_1};	/*Property = ScreenPixelsHeight*/
const Value V22147_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22147_4 = {S54, 0, &V22147_4_1229, &P22147_5};	/*Property = ScreenPixelsWidth*/
const Profile P22147 = {"22147", &P22147_4};	/*Profile ID - 22147*/
const Value V20255_1_1 = {S2, NULL};	/*Value = True*/
const Property P20255_1 = {S50, 0, &V20255_1_1, NULL};	/*Property = IsMobile*/
const Value V20255_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20255_5 = {S52, 0, &V20255_5_1277, &P20255_1};	/*Property = ScreenPixelsHeight*/
const Value V20255_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20255_4 = {S54, 0, &V20255_4_1279, &P20255_5};	/*Property = ScreenPixelsWidth*/
const Profile P20255 = {"20255", &P20255_4};	/*Profile ID - 20255*/
const Value V22522_1_1 = {S2, NULL};	/*Value = True*/
const Property P22522_1 = {S50, 0, &V22522_1_1, NULL};	/*Property = IsMobile*/
const Value V22522_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22522_5 = {S52, 0, &V22522_5_1227, &P22522_1};	/*Property = ScreenPixelsHeight*/
const Value V22522_4_82725 = {S146, NULL};	/*Value = 1240*/
const Property P22522_4 = {S54, 0, &V22522_4_82725, &P22522_5};	/*Property = ScreenPixelsWidth*/
const Profile P22522 = {"22522", &P22522_4};	/*Profile ID - 22522*/
const Value V21400_1_1 = {S2, NULL};	/*Value = True*/
const Property P21400_1 = {S50, 0, &V21400_1_1, NULL};	/*Property = IsMobile*/
const Value V21400_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21400_5 = {S52, 0, &V21400_5_1227, &P21400_1};	/*Property = ScreenPixelsHeight*/
const Value V21400_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21400_4 = {S54, 0, &V21400_4_1229, &P21400_5};	/*Property = ScreenPixelsWidth*/
const Profile P21400 = {"21400", &P21400_4};	/*Profile ID - 21400*/
const Value V22681_1_1 = {S2, NULL};	/*Value = True*/
const Property P22681_1 = {S50, 0, &V22681_1_1, NULL};	/*Property = IsMobile*/
const Value V22681_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22681_5 = {S52, 0, &V22681_5_9981, &P22681_1};	/*Property = ScreenPixelsHeight*/
const Value V22681_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22681_4 = {S54, 0, &V22681_4_39098, &P22681_5};	/*Property = ScreenPixelsWidth*/
const Profile P22681 = {"22681", &P22681_4};	/*Profile ID - 22681*/
const Value V21548_1_1 = {S2, NULL};	/*Value = True*/
const Property P21548_1 = {S50, 0, &V21548_1_1, NULL};	/*Property = IsMobile*/
const Value V21548_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21548_5 = {S52, 0, &V21548_5_1458, &P21548_1};	/*Property = ScreenPixelsHeight*/
const Value V21548_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21548_4 = {S54, 0, &V21548_4_37617, &P21548_5};	/*Property = ScreenPixelsWidth*/
const Profile P21548 = {"21548", &P21548_4};	/*Profile ID - 21548*/
const Value V23714_1_1 = {S2, NULL};	/*Value = True*/
const Property P23714_1 = {S50, 0, &V23714_1_1, NULL};	/*Property = IsMobile*/
const Value V23714_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23714_5 = {S52, 0, &V23714_5_1227, &P23714_1};	/*Property = ScreenPixelsHeight*/
const Value V23714_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23714_4 = {S54, 0, &V23714_4_1229, &P23714_5};	/*Property = ScreenPixelsWidth*/
const Profile P23714 = {"23714", &P23714_4};	/*Profile ID - 23714*/
const Value V20109_1_2 = {S0, NULL};	/*Value = False*/
const Property P20109_1 = {S50, 0, &V20109_1_2, NULL};	/*Property = IsMobile*/
const Value V20109_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20109_5 = {S52, 0, &V20109_5_37, &P20109_1};	/*Property = ScreenPixelsHeight*/
const Value V20109_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20109_4 = {S54, 0, &V20109_4_36, &P20109_5};	/*Property = ScreenPixelsWidth*/
const Profile P20109 = {"20109", &P20109_4};	/*Profile ID - 20109*/
const Value V21402_1_1 = {S2, NULL};	/*Value = True*/
const Property P21402_1 = {S50, 0, &V21402_1_1, NULL};	/*Property = IsMobile*/
const Value V21402_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21402_5 = {S52, 0, &V21402_5_1403, &P21402_1};	/*Property = ScreenPixelsHeight*/
const Value V21402_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21402_4 = {S54, 0, &V21402_4_1404, &P21402_5};	/*Property = ScreenPixelsWidth*/
const Profile P21402 = {"21402", &P21402_4};	/*Profile ID - 21402*/
const Value V23976_1_1 = {S2, NULL};	/*Value = True*/
const Property P23976_1 = {S50, 0, &V23976_1_1, NULL};	/*Property = IsMobile*/
const Value V23976_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23976_5 = {S52, 0, &V23976_5_37, &P23976_1};	/*Property = ScreenPixelsHeight*/
const Value V23976_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23976_4 = {S54, 0, &V23976_4_36, &P23976_5};	/*Property = ScreenPixelsWidth*/
const Profile P23976 = {"23976", &P23976_4};	/*Profile ID - 23976*/
const Value V20857_1_1 = {S2, NULL};	/*Value = True*/
const Property P20857_1 = {S50, 0, &V20857_1_1, NULL};	/*Property = IsMobile*/
const Value V20857_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20857_5 = {S52, 0, &V20857_5_1227, &P20857_1};	/*Property = ScreenPixelsHeight*/
const Value V20857_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20857_4 = {S54, 0, &V20857_4_1229, &P20857_5};	/*Property = ScreenPixelsWidth*/
const Profile P20857 = {"20857", &P20857_4};	/*Profile ID - 20857*/
const Value V23634_1_1 = {S2, NULL};	/*Value = True*/
const Property P23634_1 = {S50, 0, &V23634_1_1, NULL};	/*Property = IsMobile*/
const Value V23634_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23634_5 = {S52, 0, &V23634_5_9981, &P23634_1};	/*Property = ScreenPixelsHeight*/
const Value V23634_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23634_4 = {S54, 0, &V23634_4_2336, &P23634_5};	/*Property = ScreenPixelsWidth*/
const Profile P23634 = {"23634", &P23634_4};	/*Profile ID - 23634*/
const Value V21845_1_1 = {S2, NULL};	/*Value = True*/
const Property P21845_1 = {S50, 0, &V21845_1_1, NULL};	/*Property = IsMobile*/
const Value V21845_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21845_5 = {S52, 0, &V21845_5_1227, &P21845_1};	/*Property = ScreenPixelsHeight*/
const Value V21845_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21845_4 = {S54, 0, &V21845_4_1229, &P21845_5};	/*Property = ScreenPixelsWidth*/
const Profile P21845 = {"21845", &P21845_4};	/*Profile ID - 21845*/
const Value V23394_1_1 = {S2, NULL};	/*Value = True*/
const Property P23394_1 = {S50, 0, &V23394_1_1, NULL};	/*Property = IsMobile*/
const Value V23394_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23394_5 = {S52, 0, &V23394_5_37569, &P23394_1};	/*Property = ScreenPixelsHeight*/
const Value V23394_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23394_4 = {S54, 0, &V23394_4_5645, &P23394_5};	/*Property = ScreenPixelsWidth*/
const Profile P23394 = {"23394", &P23394_4};	/*Profile ID - 23394*/
const Value V23802_1_1 = {S2, NULL};	/*Value = True*/
const Property P23802_1 = {S50, 0, &V23802_1_1, NULL};	/*Property = IsMobile*/
const Value V23802_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23802_5 = {S52, 0, &V23802_5_9981, &P23802_1};	/*Property = ScreenPixelsHeight*/
const Value V23802_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23802_4 = {S54, 0, &V23802_4_2336, &P23802_5};	/*Property = ScreenPixelsWidth*/
const Profile P23802 = {"23802", &P23802_4};	/*Profile ID - 23802*/
const Value V21617_1_1 = {S2, NULL};	/*Value = True*/
const Property P21617_1 = {S50, 0, &V21617_1_1, NULL};	/*Property = IsMobile*/
const Value V21617_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21617_5 = {S52, 0, &V21617_5_1403, &P21617_1};	/*Property = ScreenPixelsHeight*/
const Value V21617_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21617_4 = {S54, 0, &V21617_4_1404, &P21617_5};	/*Property = ScreenPixelsWidth*/
const Profile P21617 = {"21617", &P21617_4};	/*Profile ID - 21617*/
const Value V23858_1_1 = {S2, NULL};	/*Value = True*/
const Property P23858_1 = {S50, 0, &V23858_1_1, NULL};	/*Property = IsMobile*/
const Value V23858_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23858_5 = {S52, 0, &V23858_5_1458, &P23858_1};	/*Property = ScreenPixelsHeight*/
const Value V23858_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23858_4 = {S54, 0, &V23858_4_37617, &P23858_5};	/*Property = ScreenPixelsWidth*/
const Profile P23858 = {"23858", &P23858_4};	/*Profile ID - 23858*/
const Value V22178_1_1 = {S2, NULL};	/*Value = True*/
const Property P22178_1 = {S50, 0, &V22178_1_1, NULL};	/*Property = IsMobile*/
const Value V22178_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22178_5 = {S52, 0, &V22178_5_1832, &P22178_1};	/*Property = ScreenPixelsHeight*/
const Value V22178_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22178_4 = {S54, 0, &V22178_4_5645, &P22178_5};	/*Property = ScreenPixelsWidth*/
const Profile P22178 = {"22178", &P22178_4};	/*Profile ID - 22178*/
const Value V19869_1_1 = {S2, NULL};	/*Value = True*/
const Property P19869_1 = {S50, 0, &V19869_1_1, NULL};	/*Property = IsMobile*/
const Value V19869_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P19869_5 = {S52, 0, &V19869_5_1222, &P19869_1};	/*Property = ScreenPixelsHeight*/
const Value V19869_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P19869_4 = {S54, 0, &V19869_4_1224, &P19869_5};	/*Property = ScreenPixelsWidth*/
const Profile P19869 = {"19869", &P19869_4};	/*Profile ID - 19869*/
const Value V22671_1_1 = {S2, NULL};	/*Value = True*/
const Property P22671_1 = {S50, 0, &V22671_1_1, NULL};	/*Property = IsMobile*/
const Value V22671_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22671_5 = {S52, 0, &V22671_5_1458, &P22671_1};	/*Property = ScreenPixelsHeight*/
const Value V22671_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22671_4 = {S54, 0, &V22671_4_1460, &P22671_5};	/*Property = ScreenPixelsWidth*/
const Profile P22671 = {"22671", &P22671_4};	/*Profile ID - 22671*/
const Value V23089_1_1 = {S2, NULL};	/*Value = True*/
const Property P23089_1 = {S50, 0, &V23089_1_1, NULL};	/*Property = IsMobile*/
const Value V23089_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23089_5 = {S52, 0, &V23089_5_1832, &P23089_1};	/*Property = ScreenPixelsHeight*/
const Value V23089_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23089_4 = {S54, 0, &V23089_4_5645, &P23089_5};	/*Property = ScreenPixelsWidth*/
const Profile P23089 = {"23089", &P23089_4};	/*Profile ID - 23089*/
const Value V21892_1_1 = {S2, NULL};	/*Value = True*/
const Property P21892_1 = {S50, 0, &V21892_1_1, NULL};	/*Property = IsMobile*/
const Value V21892_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21892_5 = {S52, 0, &V21892_5_1277, &P21892_1};	/*Property = ScreenPixelsHeight*/
const Value V21892_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21892_4 = {S54, 0, &V21892_4_2336, &P21892_5};	/*Property = ScreenPixelsWidth*/
const Profile P21892 = {"21892", &P21892_4};	/*Profile ID - 21892*/
const Value V21652_1_1 = {S2, NULL};	/*Value = True*/
const Property P21652_1 = {S50, 0, &V21652_1_1, NULL};	/*Property = IsMobile*/
const Value V21652_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21652_5 = {S52, 0, &V21652_5_1227, &P21652_1};	/*Property = ScreenPixelsHeight*/
const Value V21652_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21652_4 = {S54, 0, &V21652_4_1229, &P21652_5};	/*Property = ScreenPixelsWidth*/
const Profile P21652 = {"21652", &P21652_4};	/*Profile ID - 21652*/
const Value V19805_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19805_279 = {S1, 0, &V19805_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19805_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19805_282 = {S3, 0, &V19805_282_57154, &P19805_279};	/*Property = BlobBuilder*/
const Value V19805_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19805_350 = {S4, 0, &V19805_350_57444, &P19805_282};	/*Property = Canvas*/
const Value V19805_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19805_283 = {S5, 0, &V19805_283_57159, &P19805_350};	/*Property = CssBackground*/
const Value V19805_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19805_284 = {S6, 0, &V19805_284_57163, &P19805_283};	/*Property = CssBorderImage*/
const Value V19805_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19805_285 = {S7, 0, &V19805_285_57166, &P19805_284};	/*Property = CssCanvas*/
const Value V19805_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19805_286 = {S8, 0, &V19805_286_57168, &P19805_285};	/*Property = CssColor*/
const Value V19805_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19805_237 = {S9, 0, &V19805_237_55786, &P19805_286};	/*Property = CssColumn*/
const Value V19805_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19805_287 = {S10, 0, &V19805_287_57172, &P19805_237};	/*Property = CssFlexbox*/
const Value V19805_288_57174 = {S2, NULL};	/*Value = True*/
const Property P19805_288 = {S11, 0, &V19805_288_57174, &P19805_287};	/*Property = CssFont*/
const Value V19805_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19805_289 = {S12, 0, &V19805_289_57759, &P19805_288};	/*Property = CssImages*/
const Value V19805_291_57180 = {S2, NULL};	/*Value = True*/
const Property P19805_291 = {S13, 0, &V19805_291_57180, &P19805_289};	/*Property = CssMediaQueries*/
const Value V19805_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19805_292 = {S14, 0, &V19805_292_57183, &P19805_291};	/*Property = CssMinMax*/
const Value V19805_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19805_293 = {S15, 0, &V19805_293_57187, &P19805_292};	/*Property = CssOverflow*/
const Value V19805_294_57189 = {S2, NULL};	/*Value = True*/
const Property P19805_294 = {S16, 0, &V19805_294_57189, &P19805_293};	/*Property = CssPosition*/
const Value V19805_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19805_295 = {S17, 0, &V19805_295_57192, &P19805_294};	/*Property = CssText*/
const Value V19805_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19805_297 = {S18, 0, &V19805_297_57195, &P19805_295};	/*Property = CssTransforms*/
const Value V19805_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19805_241 = {S19, 0, &V19805_241_55799, &P19805_297};	/*Property = CssTransitions*/
const Value V19805_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19805_298 = {S20, 0, &V19805_298_57206, &P19805_241};	/*Property = CssUI*/
const Value V19805_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19805_300 = {S21, 0, &V19805_300_57216, &P19805_298};	/*Property = DataSet*/
const Value V19805_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19805_301 = {S22, 0, &V19805_301_57218, &P19805_300};	/*Property = DataUrl*/
const Value V19805_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19805_302 = {S23, 0, &V19805_302_57222, &P19805_301};	/*Property = DeviceOrientation*/
const Value V19805_304_57227 = {S2, NULL};	/*Value = True*/
const Property P19805_304 = {S24, 0, &V19805_304_57227, &P19805_302};	/*Property = FileReader*/
const Value V19805_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19805_305 = {S25, 0, &V19805_305_57231, &P19805_304};	/*Property = FileSaver*/
const Value V19805_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19805_306 = {S26, 0, &V19805_306_57237, &P19805_305};	/*Property = FileWriter*/
const Value V19805_307_57239 = {S2, NULL};	/*Value = True*/
const Property P19805_307 = {S27, 0, &V19805_307_57239, &P19805_306};	/*Property = FormData*/
const Value V19805_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19805_309 = {S28, 0, &V19805_309_57246, &P19805_307};	/*Property = Fullscreen*/
const Value V19805_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19805_310 = {S29, 0, &V19805_310_57248, &P19805_309};	/*Property = GeoLocation*/
const Value V19805_312_57254 = {S2, NULL};	/*Value = True*/
const Property P19805_312 = {S30, 0, &V19805_312_57254, &P19805_310};	/*Property = History*/
const Value V19805_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19805_314 = {S31, 0, &V19805_314_57261, &P19805_312};	/*Property = Html5*/
const Value V19805_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19805_313 = {S32, 0, &V19805_313_57258, &P19805_314};	/*Property = Html-Media-Capture*/
const Value V19805_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19805_315 = {S33, 0, &V19805_315_57264, &P19805_313};	/*Property = Iframe*/
const Value V19805_316_57266 = {S2, NULL};	/*Value = True*/
const Property P19805_316 = {S34, 0, &V19805_316_57266, &P19805_315};	/*Property = IndexedDB*/
const Value V19805_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19805_317 = {S35, 0, &V19805_317_57269, &P19805_316};	/*Property = Json*/
const Value V19805_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19805_15 = {S37, 0, &V19805_15_28, &P19805_317};	/*Property = LayoutEngine*/
const Value V19805_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19805_344 = {S38, 0, &V19805_344_57272, &P19805_15};	/*Property = Masking*/
const Value V19805_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19805_320 = {S39, 0, &V19805_320_57290, &P19805_344};	/*Property = PostMessage*/
const Value V19805_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19805_321 = {S40, 0, &V19805_321_57293, &P19805_320};	/*Property = Progress*/
const Value V19805_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19805_322 = {S41, 0, &V19805_322_57296, &P19805_321};	/*Property = Prompts*/
const Value V19805_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19805_323 = {S42, 0, &V19805_323_57299, &P19805_322};	/*Property = Selector*/
const Value V19805_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19805_324 = {S43, 0, &V19805_324_57306, &P19805_323};	/*Property = Svg*/
const Value V19805_327_57314 = {S2, NULL};	/*Value = True*/
const Property P19805_327 = {S44, 0, &V19805_327_57314, &P19805_324};	/*Property = TouchEvents*/
const Value V19805_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19805_328 = {S45, 0, &V19805_328_57318, &P19805_327};	/*Property = Track*/
const Value V19805_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19805_329 = {S46, 0, &V19805_329_57323, &P19805_328};	/*Property = Video*/
const Value V19805_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19805_330 = {S47, 0, &V19805_330_57326, &P19805_329};	/*Property = Viewport*/
const Value V19805_332_57339 = {S0, NULL};	/*Value = False*/
const Property P19805_332 = {S48, 0, &V19805_332_57339, &P19805_330};	/*Property = WebWorkers*/
const Value V19805_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19805_333 = {S49, 0, &V19805_333_57342, &P19805_332};	/*Property = Xhr2*/
const Profile P19805 = {"19805", &P19805_333};	/*Profile ID - 19805*/
const Value V21515_1_1 = {S2, NULL};	/*Value = True*/
const Property P21515_1 = {S50, 0, &V21515_1_1, NULL};	/*Property = IsMobile*/
const Value V21515_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21515_5 = {S52, 0, &V21515_5_1458, &P21515_1};	/*Property = ScreenPixelsHeight*/
const Value V21515_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21515_4 = {S54, 0, &V21515_4_37617, &P21515_5};	/*Property = ScreenPixelsWidth*/
const Profile P21515 = {"21515", &P21515_4};	/*Profile ID - 21515*/
const Value V20471_1_1 = {S2, NULL};	/*Value = True*/
const Property P20471_1 = {S50, 0, &V20471_1_1, NULL};	/*Property = IsMobile*/
const Value V20471_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20471_5 = {S52, 0, &V20471_5_1227, &P20471_1};	/*Property = ScreenPixelsHeight*/
const Value V20471_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20471_4 = {S54, 0, &V20471_4_1229, &P20471_5};	/*Property = ScreenPixelsWidth*/
const Profile P20471 = {"20471", &P20471_4};	/*Profile ID - 20471*/
const Value V24011_1_1 = {S2, NULL};	/*Value = True*/
const Property P24011_1 = {S50, 0, &V24011_1_1, NULL};	/*Property = IsMobile*/
const Value V24011_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24011_5 = {S52, 0, &V24011_5_1277, &P24011_1};	/*Property = ScreenPixelsHeight*/
const Value V24011_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P24011_4 = {S54, 0, &V24011_4_2336, &P24011_5};	/*Property = ScreenPixelsWidth*/
const Profile P24011 = {"24011", &P24011_4};	/*Profile ID - 24011*/
const Value V21618_1_1 = {S2, NULL};	/*Value = True*/
const Property P21618_1 = {S50, 0, &V21618_1_1, NULL};	/*Property = IsMobile*/
const Value V21618_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21618_5 = {S52, 0, &V21618_5_1269, &P21618_1};	/*Property = ScreenPixelsHeight*/
const Value V21618_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21618_4 = {S54, 0, &V21618_4_1229, &P21618_5};	/*Property = ScreenPixelsWidth*/
const Profile P21618 = {"21618", &P21618_4};	/*Profile ID - 21618*/
const Value V23383_1_1 = {S2, NULL};	/*Value = True*/
const Property P23383_1 = {S50, 0, &V23383_1_1, NULL};	/*Property = IsMobile*/
const Value V23383_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23383_5 = {S52, 0, &V23383_5_1403, &P23383_1};	/*Property = ScreenPixelsHeight*/
const Value V23383_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23383_4 = {S54, 0, &V23383_4_1404, &P23383_5};	/*Property = ScreenPixelsWidth*/
const Profile P23383 = {"23383", &P23383_4};	/*Profile ID - 23383*/
const Value V21729_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_279 = {S1, 0, &V21729_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21729_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_282 = {S3, 0, &V21729_282_57155, &P21729_279};	/*Property = BlobBuilder*/
const Value V21729_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_350 = {S4, 0, &V21729_350_57446, &P21729_282};	/*Property = Canvas*/
const Value V21729_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_283 = {S5, 0, &V21729_283_57161, &P21729_350};	/*Property = CssBackground*/
const Value V21729_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_284 = {S6, 0, &V21729_284_57164, &P21729_283};	/*Property = CssBorderImage*/
const Value V21729_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_285 = {S7, 0, &V21729_285_57167, &P21729_284};	/*Property = CssCanvas*/
const Value V21729_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_286 = {S8, 0, &V21729_286_57170, &P21729_285};	/*Property = CssColor*/
const Value V21729_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_237 = {S9, 0, &V21729_237_55786, &P21729_286};	/*Property = CssColumn*/
const Value V21729_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_287 = {S10, 0, &V21729_287_57173, &P21729_237};	/*Property = CssFlexbox*/
const Value V21729_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_288 = {S11, 0, &V21729_288_57176, &P21729_287};	/*Property = CssFont*/
const Value V21729_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_289 = {S12, 0, &V21729_289_86899, &P21729_288};	/*Property = CssImages*/
const Value V21729_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_291 = {S13, 0, &V21729_291_57182, &P21729_289};	/*Property = CssMediaQueries*/
const Value V21729_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_292 = {S14, 0, &V21729_292_57185, &P21729_291};	/*Property = CssMinMax*/
const Value V21729_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_293 = {S15, 0, &V21729_293_57188, &P21729_292};	/*Property = CssOverflow*/
const Value V21729_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_294 = {S16, 0, &V21729_294_57191, &P21729_293};	/*Property = CssPosition*/
const Value V21729_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_295 = {S17, 0, &V21729_295_57194, &P21729_294};	/*Property = CssText*/
const Value V21729_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_297 = {S18, 0, &V21729_297_57197, &P21729_295};	/*Property = CssTransforms*/
const Value V21729_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_241 = {S19, 0, &V21729_241_55801, &P21729_297};	/*Property = CssTransitions*/
const Value V21729_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_298 = {S20, 0, &V21729_298_57208, &P21729_241};	/*Property = CssUI*/
const Value V21729_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_300 = {S21, 0, &V21729_300_57217, &P21729_298};	/*Property = DataSet*/
const Value V21729_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_301 = {S22, 0, &V21729_301_57220, &P21729_300};	/*Property = DataUrl*/
const Value V21729_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_302 = {S23, 0, &V21729_302_57223, &P21729_301};	/*Property = DeviceOrientation*/
const Value V21729_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_304 = {S24, 0, &V21729_304_57229, &P21729_302};	/*Property = FileReader*/
const Value V21729_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_305 = {S25, 0, &V21729_305_57232, &P21729_304};	/*Property = FileSaver*/
const Value V21729_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_306 = {S26, 0, &V21729_306_57238, &P21729_305};	/*Property = FileWriter*/
const Value V21729_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_307 = {S27, 0, &V21729_307_57241, &P21729_306};	/*Property = FormData*/
const Value V21729_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_309 = {S28, 0, &V21729_309_57247, &P21729_307};	/*Property = Fullscreen*/
const Value V21729_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_310 = {S29, 0, &V21729_310_57250, &P21729_309};	/*Property = GeoLocation*/
const Value V21729_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_312 = {S30, 0, &V21729_312_57256, &P21729_310};	/*Property = History*/
const Value V21729_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_314 = {S31, 0, &V21729_314_57262, &P21729_312};	/*Property = Html5*/
const Value V21729_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_313 = {S32, 0, &V21729_313_57259, &P21729_314};	/*Property = Html-Media-Capture*/
const Value V21729_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_315 = {S33, 0, &V21729_315_57265, &P21729_313};	/*Property = Iframe*/
const Value V21729_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_316 = {S34, 0, &V21729_316_57268, &P21729_315};	/*Property = IndexedDB*/
const Value V21729_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_317 = {S35, 0, &V21729_317_57271, &P21729_316};	/*Property = Json*/
const Value V21729_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21729_15 = {S37, 0, &V21729_15_54167, &P21729_317};	/*Property = LayoutEngine*/
const Value V21729_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_344 = {S38, 0, &V21729_344_57274, &P21729_15};	/*Property = Masking*/
const Value V21729_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_320 = {S39, 0, &V21729_320_57292, &P21729_344};	/*Property = PostMessage*/
const Value V21729_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_321 = {S40, 0, &V21729_321_57295, &P21729_320};	/*Property = Progress*/
const Value V21729_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_322 = {S41, 0, &V21729_322_57298, &P21729_321};	/*Property = Prompts*/
const Value V21729_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_323 = {S42, 0, &V21729_323_57301, &P21729_322};	/*Property = Selector*/
const Value V21729_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_324 = {S43, 0, &V21729_324_57307, &P21729_323};	/*Property = Svg*/
const Value V21729_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_327 = {S44, 0, &V21729_327_57316, &P21729_324};	/*Property = TouchEvents*/
const Value V21729_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_328 = {S45, 0, &V21729_328_57319, &P21729_327};	/*Property = Track*/
const Value V21729_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_329 = {S46, 0, &V21729_329_57325, &P21729_328};	/*Property = Video*/
const Value V21729_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_330 = {S47, 0, &V21729_330_57328, &P21729_329};	/*Property = Viewport*/
const Value V21729_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_332 = {S48, 0, &V21729_332_57340, &P21729_330};	/*Property = WebWorkers*/
const Value V21729_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21729_333 = {S49, 0, &V21729_333_57343, &P21729_332};	/*Property = Xhr2*/
const Profile P21729 = {"21729", &P21729_333};	/*Profile ID - 21729*/
const Value V20354_1_1 = {S2, NULL};	/*Value = True*/
const Property P20354_1 = {S50, 0, &V20354_1_1, NULL};	/*Property = IsMobile*/
const Value V20354_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20354_5 = {S52, 0, &V20354_5_1227, &P20354_1};	/*Property = ScreenPixelsHeight*/
const Value V20354_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20354_4 = {S54, 0, &V20354_4_1229, &P20354_5};	/*Property = ScreenPixelsWidth*/
const Profile P20354 = {"20354", &P20354_4};	/*Profile ID - 20354*/
const Value V19818_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_279 = {S1, 0, &V19818_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V19818_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_282 = {S3, 0, &V19818_282_57155, &P19818_279};	/*Property = BlobBuilder*/
const Value V19818_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_350 = {S4, 0, &V19818_350_57446, &P19818_282};	/*Property = Canvas*/
const Value V19818_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_283 = {S5, 0, &V19818_283_57161, &P19818_350};	/*Property = CssBackground*/
const Value V19818_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_284 = {S6, 0, &V19818_284_57164, &P19818_283};	/*Property = CssBorderImage*/
const Value V19818_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_285 = {S7, 0, &V19818_285_57167, &P19818_284};	/*Property = CssCanvas*/
const Value V19818_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_286 = {S8, 0, &V19818_286_57170, &P19818_285};	/*Property = CssColor*/
const Value V19818_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_237 = {S9, 0, &V19818_237_55786, &P19818_286};	/*Property = CssColumn*/
const Value V19818_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_287 = {S10, 0, &V19818_287_57173, &P19818_237};	/*Property = CssFlexbox*/
const Value V19818_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_288 = {S11, 0, &V19818_288_57176, &P19818_287};	/*Property = CssFont*/
const Value V19818_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_289 = {S12, 0, &V19818_289_86899, &P19818_288};	/*Property = CssImages*/
const Value V19818_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_291 = {S13, 0, &V19818_291_57182, &P19818_289};	/*Property = CssMediaQueries*/
const Value V19818_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_292 = {S14, 0, &V19818_292_57185, &P19818_291};	/*Property = CssMinMax*/
const Value V19818_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_293 = {S15, 0, &V19818_293_57188, &P19818_292};	/*Property = CssOverflow*/
const Value V19818_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_294 = {S16, 0, &V19818_294_57191, &P19818_293};	/*Property = CssPosition*/
const Value V19818_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_295 = {S17, 0, &V19818_295_57194, &P19818_294};	/*Property = CssText*/
const Value V19818_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_297 = {S18, 0, &V19818_297_57197, &P19818_295};	/*Property = CssTransforms*/
const Value V19818_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_241 = {S19, 0, &V19818_241_55801, &P19818_297};	/*Property = CssTransitions*/
const Value V19818_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_298 = {S20, 0, &V19818_298_57208, &P19818_241};	/*Property = CssUI*/
const Value V19818_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_300 = {S21, 0, &V19818_300_57217, &P19818_298};	/*Property = DataSet*/
const Value V19818_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_301 = {S22, 0, &V19818_301_57220, &P19818_300};	/*Property = DataUrl*/
const Value V19818_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_302 = {S23, 0, &V19818_302_57223, &P19818_301};	/*Property = DeviceOrientation*/
const Value V19818_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_304 = {S24, 0, &V19818_304_57229, &P19818_302};	/*Property = FileReader*/
const Value V19818_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_305 = {S25, 0, &V19818_305_57232, &P19818_304};	/*Property = FileSaver*/
const Value V19818_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_306 = {S26, 0, &V19818_306_57238, &P19818_305};	/*Property = FileWriter*/
const Value V19818_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_307 = {S27, 0, &V19818_307_57241, &P19818_306};	/*Property = FormData*/
const Value V19818_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_309 = {S28, 0, &V19818_309_57247, &P19818_307};	/*Property = Fullscreen*/
const Value V19818_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_310 = {S29, 0, &V19818_310_57250, &P19818_309};	/*Property = GeoLocation*/
const Value V19818_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_312 = {S30, 0, &V19818_312_57256, &P19818_310};	/*Property = History*/
const Value V19818_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_314 = {S31, 0, &V19818_314_57262, &P19818_312};	/*Property = Html5*/
const Value V19818_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_313 = {S32, 0, &V19818_313_57259, &P19818_314};	/*Property = Html-Media-Capture*/
const Value V19818_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_315 = {S33, 0, &V19818_315_57265, &P19818_313};	/*Property = Iframe*/
const Value V19818_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_316 = {S34, 0, &V19818_316_57268, &P19818_315};	/*Property = IndexedDB*/
const Value V19818_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_317 = {S35, 0, &V19818_317_57271, &P19818_316};	/*Property = Json*/
const Value V19818_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19818_15 = {S37, 0, &V19818_15_28, &P19818_317};	/*Property = LayoutEngine*/
const Value V19818_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_344 = {S38, 0, &V19818_344_57274, &P19818_15};	/*Property = Masking*/
const Value V19818_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_320 = {S39, 0, &V19818_320_57292, &P19818_344};	/*Property = PostMessage*/
const Value V19818_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_321 = {S40, 0, &V19818_321_57295, &P19818_320};	/*Property = Progress*/
const Value V19818_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_322 = {S41, 0, &V19818_322_57298, &P19818_321};	/*Property = Prompts*/
const Value V19818_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_323 = {S42, 0, &V19818_323_57301, &P19818_322};	/*Property = Selector*/
const Value V19818_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_324 = {S43, 0, &V19818_324_57307, &P19818_323};	/*Property = Svg*/
const Value V19818_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_327 = {S44, 0, &V19818_327_57316, &P19818_324};	/*Property = TouchEvents*/
const Value V19818_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_328 = {S45, 0, &V19818_328_57319, &P19818_327};	/*Property = Track*/
const Value V19818_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_329 = {S46, 0, &V19818_329_57325, &P19818_328};	/*Property = Video*/
const Value V19818_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_330 = {S47, 0, &V19818_330_57328, &P19818_329};	/*Property = Viewport*/
const Value V19818_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_332 = {S48, 0, &V19818_332_57340, &P19818_330};	/*Property = WebWorkers*/
const Value V19818_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P19818_333 = {S49, 0, &V19818_333_57343, &P19818_332};	/*Property = Xhr2*/
const Profile P19818 = {"19818", &P19818_333};	/*Profile ID - 19818*/
const Value V20878_1_1 = {S2, NULL};	/*Value = True*/
const Property P20878_1 = {S50, 0, &V20878_1_1, NULL};	/*Property = IsMobile*/
const Value V20878_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20878_5 = {S52, 0, &V20878_5_1227, &P20878_1};	/*Property = ScreenPixelsHeight*/
const Value V20878_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20878_4 = {S54, 0, &V20878_4_1229, &P20878_5};	/*Property = ScreenPixelsWidth*/
const Profile P20878 = {"20878", &P20878_4};	/*Profile ID - 20878*/
const Value V19975_1_1 = {S2, NULL};	/*Value = True*/
const Property P19975_1 = {S50, 0, &V19975_1_1, NULL};	/*Property = IsMobile*/
const Value V19975_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19975_5 = {S52, 0, &V19975_5_1227, &P19975_1};	/*Property = ScreenPixelsHeight*/
const Value V19975_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19975_4 = {S54, 0, &V19975_4_1229, &P19975_5};	/*Property = ScreenPixelsWidth*/
const Profile P19975 = {"19975", &P19975_4};	/*Profile ID - 19975*/
const Value V23665_279_57148 = {S0, NULL};	/*Value = False*/
const Property P23665_279 = {S1, 0, &V23665_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V23665_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23665_282 = {S3, 0, &V23665_282_57154, &P23665_279};	/*Property = BlobBuilder*/
const Value V23665_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23665_350 = {S4, 0, &V23665_350_57444, &P23665_282};	/*Property = Canvas*/
const Value V23665_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23665_283 = {S5, 0, &V23665_283_57159, &P23665_350};	/*Property = CssBackground*/
const Value V23665_284_57163 = {S0, NULL};	/*Value = False*/
const Property P23665_284 = {S6, 0, &V23665_284_57163, &P23665_283};	/*Property = CssBorderImage*/
const Value V23665_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23665_285 = {S7, 0, &V23665_285_57166, &P23665_284};	/*Property = CssCanvas*/
const Value V23665_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23665_286 = {S8, 0, &V23665_286_57168, &P23665_285};	/*Property = CssColor*/
const Value V23665_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23665_237 = {S9, 0, &V23665_237_55786, &P23665_286};	/*Property = CssColumn*/
const Value V23665_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23665_287 = {S10, 0, &V23665_287_57172, &P23665_237};	/*Property = CssFlexbox*/
const Value V23665_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23665_288 = {S11, 0, &V23665_288_57174, &P23665_287};	/*Property = CssFont*/
const Value V23665_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23665_289 = {S12, 0, &V23665_289_57759, &P23665_288};	/*Property = CssImages*/
const Value V23665_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23665_291 = {S13, 0, &V23665_291_57180, &P23665_289};	/*Property = CssMediaQueries*/
const Value V23665_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23665_292 = {S14, 0, &V23665_292_57183, &P23665_291};	/*Property = CssMinMax*/
const Value V23665_293_57187 = {S0, NULL};	/*Value = False*/
const Property P23665_293 = {S15, 0, &V23665_293_57187, &P23665_292};	/*Property = CssOverflow*/
const Value V23665_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23665_294 = {S16, 0, &V23665_294_57189, &P23665_293};	/*Property = CssPosition*/
const Value V23665_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23665_295 = {S17, 0, &V23665_295_57192, &P23665_294};	/*Property = CssText*/
const Value V23665_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23665_297 = {S18, 0, &V23665_297_57195, &P23665_295};	/*Property = CssTransforms*/
const Value V23665_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23665_241 = {S19, 0, &V23665_241_55799, &P23665_297};	/*Property = CssTransitions*/
const Value V23665_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23665_298 = {S20, 0, &V23665_298_57206, &P23665_241};	/*Property = CssUI*/
const Value V23665_300_57216 = {S0, NULL};	/*Value = False*/
const Property P23665_300 = {S21, 0, &V23665_300_57216, &P23665_298};	/*Property = DataSet*/
const Value V23665_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23665_301 = {S22, 0, &V23665_301_57218, &P23665_300};	/*Property = DataUrl*/
const Value V23665_302_57221 = {S2, NULL};	/*Value = True*/
const Property P23665_302 = {S23, 0, &V23665_302_57221, &P23665_301};	/*Property = DeviceOrientation*/
const Value V23665_304_57227 = {S2, NULL};	/*Value = True*/
const Property P23665_304 = {S24, 0, &V23665_304_57227, &P23665_302};	/*Property = FileReader*/
const Value V23665_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23665_305 = {S25, 0, &V23665_305_57231, &P23665_304};	/*Property = FileSaver*/
const Value V23665_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23665_306 = {S26, 0, &V23665_306_57237, &P23665_305};	/*Property = FileWriter*/
const Value V23665_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23665_307 = {S27, 0, &V23665_307_57239, &P23665_306};	/*Property = FormData*/
const Value V23665_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23665_309 = {S28, 0, &V23665_309_57246, &P23665_307};	/*Property = Fullscreen*/
const Value V23665_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23665_310 = {S29, 0, &V23665_310_57248, &P23665_309};	/*Property = GeoLocation*/
const Value V23665_312_57254 = {S2, NULL};	/*Value = True*/
const Property P23665_312 = {S30, 0, &V23665_312_57254, &P23665_310};	/*Property = History*/
const Value V23665_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23665_314 = {S31, 0, &V23665_314_57261, &P23665_312};	/*Property = Html5*/
const Value V23665_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23665_313 = {S32, 0, &V23665_313_57258, &P23665_314};	/*Property = Html-Media-Capture*/
const Value V23665_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23665_315 = {S33, 0, &V23665_315_57264, &P23665_313};	/*Property = Iframe*/
const Value V23665_316_57266 = {S2, NULL};	/*Value = True*/
const Property P23665_316 = {S34, 0, &V23665_316_57266, &P23665_315};	/*Property = IndexedDB*/
const Value V23665_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23665_317 = {S35, 0, &V23665_317_57269, &P23665_316};	/*Property = Json*/
const Value V23665_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P23665_15 = {S37, 0, &V23665_15_29, &P23665_317};	/*Property = LayoutEngine*/
const Value V23665_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23665_344 = {S38, 0, &V23665_344_57272, &P23665_15};	/*Property = Masking*/
const Value V23665_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23665_320 = {S39, 0, &V23665_320_57290, &P23665_344};	/*Property = PostMessage*/
const Value V23665_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23665_321 = {S40, 0, &V23665_321_57293, &P23665_320};	/*Property = Progress*/
const Value V23665_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23665_322 = {S41, 0, &V23665_322_57296, &P23665_321};	/*Property = Prompts*/
const Value V23665_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23665_323 = {S42, 0, &V23665_323_57299, &P23665_322};	/*Property = Selector*/
const Value V23665_324_57306 = {S0, NULL};	/*Value = False*/
const Property P23665_324 = {S43, 0, &V23665_324_57306, &P23665_323};	/*Property = Svg*/
const Value V23665_327_57314 = {S2, NULL};	/*Value = True*/
const Property P23665_327 = {S44, 0, &V23665_327_57314, &P23665_324};	/*Property = TouchEvents*/
const Value V23665_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23665_328 = {S45, 0, &V23665_328_57318, &P23665_327};	/*Property = Track*/
const Value V23665_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23665_329 = {S46, 0, &V23665_329_57323, &P23665_328};	/*Property = Video*/
const Value V23665_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23665_330 = {S47, 0, &V23665_330_57326, &P23665_329};	/*Property = Viewport*/
const Value V23665_332_57338 = {S2, NULL};	/*Value = True*/
const Property P23665_332 = {S48, 0, &V23665_332_57338, &P23665_330};	/*Property = WebWorkers*/
const Value V23665_333_57341 = {S2, NULL};	/*Value = True*/
const Property P23665_333 = {S49, 0, &V23665_333_57341, &P23665_332};	/*Property = Xhr2*/
const Profile P23665 = {"23665", &P23665_333};	/*Profile ID - 23665*/
const Value V20261_1_1 = {S2, NULL};	/*Value = True*/
const Property P20261_1 = {S50, 0, &V20261_1_1, NULL};	/*Property = IsMobile*/
const Value V20261_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20261_5 = {S52, 0, &V20261_5_1403, &P20261_1};	/*Property = ScreenPixelsHeight*/
const Value V20261_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20261_4 = {S54, 0, &V20261_4_1404, &P20261_5};	/*Property = ScreenPixelsWidth*/
const Profile P20261 = {"20261", &P20261_4};	/*Profile ID - 20261*/
const Value V19817_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19817_279 = {S1, 0, &V19817_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19817_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19817_282 = {S3, 0, &V19817_282_57154, &P19817_279};	/*Property = BlobBuilder*/
const Value V19817_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19817_350 = {S4, 0, &V19817_350_57444, &P19817_282};	/*Property = Canvas*/
const Value V19817_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19817_283 = {S5, 0, &V19817_283_57159, &P19817_350};	/*Property = CssBackground*/
const Value V19817_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19817_284 = {S6, 0, &V19817_284_57163, &P19817_283};	/*Property = CssBorderImage*/
const Value V19817_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19817_285 = {S7, 0, &V19817_285_57166, &P19817_284};	/*Property = CssCanvas*/
const Value V19817_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19817_286 = {S8, 0, &V19817_286_57168, &P19817_285};	/*Property = CssColor*/
const Value V19817_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19817_237 = {S9, 0, &V19817_237_55786, &P19817_286};	/*Property = CssColumn*/
const Value V19817_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19817_287 = {S10, 0, &V19817_287_57172, &P19817_237};	/*Property = CssFlexbox*/
const Value V19817_288_57174 = {S2, NULL};	/*Value = True*/
const Property P19817_288 = {S11, 0, &V19817_288_57174, &P19817_287};	/*Property = CssFont*/
const Value V19817_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19817_289 = {S12, 0, &V19817_289_57759, &P19817_288};	/*Property = CssImages*/
const Value V19817_291_57180 = {S2, NULL};	/*Value = True*/
const Property P19817_291 = {S13, 0, &V19817_291_57180, &P19817_289};	/*Property = CssMediaQueries*/
const Value V19817_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19817_292 = {S14, 0, &V19817_292_57183, &P19817_291};	/*Property = CssMinMax*/
const Value V19817_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19817_293 = {S15, 0, &V19817_293_57187, &P19817_292};	/*Property = CssOverflow*/
const Value V19817_294_57189 = {S2, NULL};	/*Value = True*/
const Property P19817_294 = {S16, 0, &V19817_294_57189, &P19817_293};	/*Property = CssPosition*/
const Value V19817_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19817_295 = {S17, 0, &V19817_295_57192, &P19817_294};	/*Property = CssText*/
const Value V19817_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19817_297 = {S18, 0, &V19817_297_57195, &P19817_295};	/*Property = CssTransforms*/
const Value V19817_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19817_241 = {S19, 0, &V19817_241_55799, &P19817_297};	/*Property = CssTransitions*/
const Value V19817_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19817_298 = {S20, 0, &V19817_298_57206, &P19817_241};	/*Property = CssUI*/
const Value V19817_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19817_300 = {S21, 0, &V19817_300_57216, &P19817_298};	/*Property = DataSet*/
const Value V19817_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19817_301 = {S22, 0, &V19817_301_57218, &P19817_300};	/*Property = DataUrl*/
const Value V19817_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19817_302 = {S23, 0, &V19817_302_57222, &P19817_301};	/*Property = DeviceOrientation*/
const Value V19817_304_57227 = {S2, NULL};	/*Value = True*/
const Property P19817_304 = {S24, 0, &V19817_304_57227, &P19817_302};	/*Property = FileReader*/
const Value V19817_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19817_305 = {S25, 0, &V19817_305_57231, &P19817_304};	/*Property = FileSaver*/
const Value V19817_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19817_306 = {S26, 0, &V19817_306_57237, &P19817_305};	/*Property = FileWriter*/
const Value V19817_307_57239 = {S2, NULL};	/*Value = True*/
const Property P19817_307 = {S27, 0, &V19817_307_57239, &P19817_306};	/*Property = FormData*/
const Value V19817_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19817_309 = {S28, 0, &V19817_309_57246, &P19817_307};	/*Property = Fullscreen*/
const Value V19817_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19817_310 = {S29, 0, &V19817_310_57248, &P19817_309};	/*Property = GeoLocation*/
const Value V19817_312_57254 = {S2, NULL};	/*Value = True*/
const Property P19817_312 = {S30, 0, &V19817_312_57254, &P19817_310};	/*Property = History*/
const Value V19817_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19817_314 = {S31, 0, &V19817_314_57261, &P19817_312};	/*Property = Html5*/
const Value V19817_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19817_313 = {S32, 0, &V19817_313_57258, &P19817_314};	/*Property = Html-Media-Capture*/
const Value V19817_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19817_315 = {S33, 0, &V19817_315_57264, &P19817_313};	/*Property = Iframe*/
const Value V19817_316_57266 = {S2, NULL};	/*Value = True*/
const Property P19817_316 = {S34, 0, &V19817_316_57266, &P19817_315};	/*Property = IndexedDB*/
const Value V19817_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19817_317 = {S35, 0, &V19817_317_57269, &P19817_316};	/*Property = Json*/
const Value V19817_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19817_15 = {S37, 0, &V19817_15_28, &P19817_317};	/*Property = LayoutEngine*/
const Value V19817_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19817_344 = {S38, 0, &V19817_344_57272, &P19817_15};	/*Property = Masking*/
const Value V19817_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19817_320 = {S39, 0, &V19817_320_57290, &P19817_344};	/*Property = PostMessage*/
const Value V19817_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19817_321 = {S40, 0, &V19817_321_57293, &P19817_320};	/*Property = Progress*/
const Value V19817_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19817_322 = {S41, 0, &V19817_322_57296, &P19817_321};	/*Property = Prompts*/
const Value V19817_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19817_323 = {S42, 0, &V19817_323_57299, &P19817_322};	/*Property = Selector*/
const Value V19817_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19817_324 = {S43, 0, &V19817_324_57306, &P19817_323};	/*Property = Svg*/
const Value V19817_327_57314 = {S2, NULL};	/*Value = True*/
const Property P19817_327 = {S44, 0, &V19817_327_57314, &P19817_324};	/*Property = TouchEvents*/
const Value V19817_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19817_328 = {S45, 0, &V19817_328_57318, &P19817_327};	/*Property = Track*/
const Value V19817_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19817_329 = {S46, 0, &V19817_329_57323, &P19817_328};	/*Property = Video*/
const Value V19817_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19817_330 = {S47, 0, &V19817_330_57326, &P19817_329};	/*Property = Viewport*/
const Value V19817_332_57339 = {S0, NULL};	/*Value = False*/
const Property P19817_332 = {S48, 0, &V19817_332_57339, &P19817_330};	/*Property = WebWorkers*/
const Value V19817_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19817_333 = {S49, 0, &V19817_333_57342, &P19817_332};	/*Property = Xhr2*/
const Profile P19817 = {"19817", &P19817_333};	/*Profile ID - 19817*/
const Value V20244_1_1 = {S2, NULL};	/*Value = True*/
const Property P20244_1 = {S50, 0, &V20244_1_1, NULL};	/*Property = IsMobile*/
const Value V20244_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20244_5 = {S52, 0, &V20244_5_9981, &P20244_1};	/*Property = ScreenPixelsHeight*/
const Value V20244_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20244_4 = {S54, 0, &V20244_4_2336, &P20244_5};	/*Property = ScreenPixelsWidth*/
const Profile P20244 = {"20244", &P20244_4};	/*Profile ID - 20244*/
const Value V21789_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_279 = {S1, 0, &V21789_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21789_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_282 = {S3, 0, &V21789_282_57155, &P21789_279};	/*Property = BlobBuilder*/
const Value V21789_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_350 = {S4, 0, &V21789_350_57446, &P21789_282};	/*Property = Canvas*/
const Value V21789_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_283 = {S5, 0, &V21789_283_57161, &P21789_350};	/*Property = CssBackground*/
const Value V21789_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_284 = {S6, 0, &V21789_284_57164, &P21789_283};	/*Property = CssBorderImage*/
const Value V21789_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_285 = {S7, 0, &V21789_285_57167, &P21789_284};	/*Property = CssCanvas*/
const Value V21789_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_286 = {S8, 0, &V21789_286_57170, &P21789_285};	/*Property = CssColor*/
const Value V21789_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_237 = {S9, 0, &V21789_237_55786, &P21789_286};	/*Property = CssColumn*/
const Value V21789_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_287 = {S10, 0, &V21789_287_57173, &P21789_237};	/*Property = CssFlexbox*/
const Value V21789_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_288 = {S11, 0, &V21789_288_57176, &P21789_287};	/*Property = CssFont*/
const Value V21789_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_289 = {S12, 0, &V21789_289_86899, &P21789_288};	/*Property = CssImages*/
const Value V21789_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_291 = {S13, 0, &V21789_291_57182, &P21789_289};	/*Property = CssMediaQueries*/
const Value V21789_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_292 = {S14, 0, &V21789_292_57185, &P21789_291};	/*Property = CssMinMax*/
const Value V21789_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_293 = {S15, 0, &V21789_293_57188, &P21789_292};	/*Property = CssOverflow*/
const Value V21789_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_294 = {S16, 0, &V21789_294_57191, &P21789_293};	/*Property = CssPosition*/
const Value V21789_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_295 = {S17, 0, &V21789_295_57194, &P21789_294};	/*Property = CssText*/
const Value V21789_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_297 = {S18, 0, &V21789_297_57197, &P21789_295};	/*Property = CssTransforms*/
const Value V21789_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_241 = {S19, 0, &V21789_241_55801, &P21789_297};	/*Property = CssTransitions*/
const Value V21789_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_298 = {S20, 0, &V21789_298_57208, &P21789_241};	/*Property = CssUI*/
const Value V21789_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_300 = {S21, 0, &V21789_300_57217, &P21789_298};	/*Property = DataSet*/
const Value V21789_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_301 = {S22, 0, &V21789_301_57220, &P21789_300};	/*Property = DataUrl*/
const Value V21789_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_302 = {S23, 0, &V21789_302_57223, &P21789_301};	/*Property = DeviceOrientation*/
const Value V21789_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_304 = {S24, 0, &V21789_304_57229, &P21789_302};	/*Property = FileReader*/
const Value V21789_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_305 = {S25, 0, &V21789_305_57232, &P21789_304};	/*Property = FileSaver*/
const Value V21789_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_306 = {S26, 0, &V21789_306_57238, &P21789_305};	/*Property = FileWriter*/
const Value V21789_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_307 = {S27, 0, &V21789_307_57241, &P21789_306};	/*Property = FormData*/
const Value V21789_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_309 = {S28, 0, &V21789_309_57247, &P21789_307};	/*Property = Fullscreen*/
const Value V21789_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_310 = {S29, 0, &V21789_310_57250, &P21789_309};	/*Property = GeoLocation*/
const Value V21789_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_312 = {S30, 0, &V21789_312_57256, &P21789_310};	/*Property = History*/
const Value V21789_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_314 = {S31, 0, &V21789_314_57262, &P21789_312};	/*Property = Html5*/
const Value V21789_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_313 = {S32, 0, &V21789_313_57259, &P21789_314};	/*Property = Html-Media-Capture*/
const Value V21789_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_315 = {S33, 0, &V21789_315_57265, &P21789_313};	/*Property = Iframe*/
const Value V21789_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_316 = {S34, 0, &V21789_316_57268, &P21789_315};	/*Property = IndexedDB*/
const Value V21789_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_317 = {S35, 0, &V21789_317_57271, &P21789_316};	/*Property = Json*/
const Value V21789_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_15 = {S37, 0, &V21789_15_72, &P21789_317};	/*Property = LayoutEngine*/
const Value V21789_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_344 = {S38, 0, &V21789_344_57274, &P21789_15};	/*Property = Masking*/
const Value V21789_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_320 = {S39, 0, &V21789_320_57292, &P21789_344};	/*Property = PostMessage*/
const Value V21789_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_321 = {S40, 0, &V21789_321_57295, &P21789_320};	/*Property = Progress*/
const Value V21789_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_322 = {S41, 0, &V21789_322_57298, &P21789_321};	/*Property = Prompts*/
const Value V21789_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_323 = {S42, 0, &V21789_323_57301, &P21789_322};	/*Property = Selector*/
const Value V21789_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_324 = {S43, 0, &V21789_324_57307, &P21789_323};	/*Property = Svg*/
const Value V21789_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_327 = {S44, 0, &V21789_327_57316, &P21789_324};	/*Property = TouchEvents*/
const Value V21789_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_328 = {S45, 0, &V21789_328_57319, &P21789_327};	/*Property = Track*/
const Value V21789_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_329 = {S46, 0, &V21789_329_57325, &P21789_328};	/*Property = Video*/
const Value V21789_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_330 = {S47, 0, &V21789_330_57328, &P21789_329};	/*Property = Viewport*/
const Value V21789_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_332 = {S48, 0, &V21789_332_57340, &P21789_330};	/*Property = WebWorkers*/
const Value V21789_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21789_333 = {S49, 0, &V21789_333_57343, &P21789_332};	/*Property = Xhr2*/
const Profile P21789 = {"21789", &P21789_333};	/*Profile ID - 21789*/
const Value V22368_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22368_279 = {S1, 0, &V22368_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22368_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22368_282 = {S3, 0, &V22368_282_57153, &P22368_279};	/*Property = BlobBuilder*/
const Value V22368_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22368_350 = {S4, 0, &V22368_350_57444, &P22368_282};	/*Property = Canvas*/
const Value V22368_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22368_283 = {S5, 0, &V22368_283_57159, &P22368_350};	/*Property = CssBackground*/
const Value V22368_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22368_284 = {S6, 0, &V22368_284_57163, &P22368_283};	/*Property = CssBorderImage*/
const Value V22368_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22368_285 = {S7, 0, &V22368_285_57166, &P22368_284};	/*Property = CssCanvas*/
const Value V22368_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22368_286 = {S8, 0, &V22368_286_57168, &P22368_285};	/*Property = CssColor*/
const Value V22368_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22368_237 = {S9, 0, &V22368_237_55786, &P22368_286};	/*Property = CssColumn*/
const Value V22368_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22368_287 = {S10, 0, &V22368_287_57172, &P22368_237};	/*Property = CssFlexbox*/
const Value V22368_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22368_288 = {S11, 0, &V22368_288_57175, &P22368_287};	/*Property = CssFont*/
const Value V22368_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22368_289 = {S12, 0, &V22368_289_57759, &P22368_288};	/*Property = CssImages*/
const Value V22368_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22368_291 = {S13, 0, &V22368_291_57181, &P22368_289};	/*Property = CssMediaQueries*/
const Value V22368_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22368_292 = {S14, 0, &V22368_292_57183, &P22368_291};	/*Property = CssMinMax*/
const Value V22368_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22368_293 = {S15, 0, &V22368_293_57187, &P22368_292};	/*Property = CssOverflow*/
const Value V22368_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22368_294 = {S16, 0, &V22368_294_57189, &P22368_293};	/*Property = CssPosition*/
const Value V22368_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22368_295 = {S17, 0, &V22368_295_57192, &P22368_294};	/*Property = CssText*/
const Value V22368_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22368_297 = {S18, 0, &V22368_297_57195, &P22368_295};	/*Property = CssTransforms*/
const Value V22368_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22368_241 = {S19, 0, &V22368_241_55799, &P22368_297};	/*Property = CssTransitions*/
const Value V22368_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22368_298 = {S20, 0, &V22368_298_57206, &P22368_241};	/*Property = CssUI*/
const Value V22368_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22368_300 = {S21, 0, &V22368_300_57216, &P22368_298};	/*Property = DataSet*/
const Value V22368_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22368_301 = {S22, 0, &V22368_301_57218, &P22368_300};	/*Property = DataUrl*/
const Value V22368_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22368_302 = {S23, 0, &V22368_302_57221, &P22368_301};	/*Property = DeviceOrientation*/
const Value V22368_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22368_304 = {S24, 0, &V22368_304_57227, &P22368_302};	/*Property = FileReader*/
const Value V22368_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22368_305 = {S25, 0, &V22368_305_57231, &P22368_304};	/*Property = FileSaver*/
const Value V22368_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22368_306 = {S26, 0, &V22368_306_57237, &P22368_305};	/*Property = FileWriter*/
const Value V22368_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22368_307 = {S27, 0, &V22368_307_57239, &P22368_306};	/*Property = FormData*/
const Value V22368_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22368_309 = {S28, 0, &V22368_309_57246, &P22368_307};	/*Property = Fullscreen*/
const Value V22368_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22368_310 = {S29, 0, &V22368_310_57248, &P22368_309};	/*Property = GeoLocation*/
const Value V22368_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22368_312 = {S30, 0, &V22368_312_57254, &P22368_310};	/*Property = History*/
const Value V22368_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22368_314 = {S31, 0, &V22368_314_57261, &P22368_312};	/*Property = Html5*/
const Value V22368_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22368_313 = {S32, 0, &V22368_313_57258, &P22368_314};	/*Property = Html-Media-Capture*/
const Value V22368_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22368_315 = {S33, 0, &V22368_315_57264, &P22368_313};	/*Property = Iframe*/
const Value V22368_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22368_316 = {S34, 0, &V22368_316_57266, &P22368_315};	/*Property = IndexedDB*/
const Value V22368_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22368_317 = {S35, 0, &V22368_317_57269, &P22368_316};	/*Property = Json*/
const Value V22368_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22368_15 = {S37, 0, &V22368_15_29, &P22368_317};	/*Property = LayoutEngine*/
const Value V22368_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22368_344 = {S38, 0, &V22368_344_57272, &P22368_15};	/*Property = Masking*/
const Value V22368_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22368_320 = {S39, 0, &V22368_320_57290, &P22368_344};	/*Property = PostMessage*/
const Value V22368_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22368_321 = {S40, 0, &V22368_321_57293, &P22368_320};	/*Property = Progress*/
const Value V22368_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22368_322 = {S41, 0, &V22368_322_57296, &P22368_321};	/*Property = Prompts*/
const Value V22368_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22368_323 = {S42, 0, &V22368_323_57299, &P22368_322};	/*Property = Selector*/
const Value V22368_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22368_324 = {S43, 0, &V22368_324_57306, &P22368_323};	/*Property = Svg*/
const Value V22368_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22368_327 = {S44, 0, &V22368_327_57315, &P22368_324};	/*Property = TouchEvents*/
const Value V22368_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22368_328 = {S45, 0, &V22368_328_57318, &P22368_327};	/*Property = Track*/
const Value V22368_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22368_329 = {S46, 0, &V22368_329_57323, &P22368_328};	/*Property = Video*/
const Value V22368_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22368_330 = {S47, 0, &V22368_330_57326, &P22368_329};	/*Property = Viewport*/
const Value V22368_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22368_332 = {S48, 0, &V22368_332_57339, &P22368_330};	/*Property = WebWorkers*/
const Value V22368_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22368_333 = {S49, 0, &V22368_333_57341, &P22368_332};	/*Property = Xhr2*/
const Profile P22368 = {"22368", &P22368_333};	/*Profile ID - 22368*/
const Value V20742_1_1 = {S2, NULL};	/*Value = True*/
const Property P20742_1 = {S50, 0, &V20742_1_1, NULL};	/*Property = IsMobile*/
const Value V20742_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20742_5 = {S52, 0, &V20742_5_1458, &P20742_1};	/*Property = ScreenPixelsHeight*/
const Value V20742_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20742_4 = {S54, 0, &V20742_4_37617, &P20742_5};	/*Property = ScreenPixelsWidth*/
const Profile P20742 = {"20742", &P20742_4};	/*Profile ID - 20742*/
const Value V21919_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21919_279 = {S1, 0, &V21919_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21919_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21919_282 = {S3, 0, &V21919_282_57154, &P21919_279};	/*Property = BlobBuilder*/
const Value V21919_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21919_350 = {S4, 0, &V21919_350_57444, &P21919_282};	/*Property = Canvas*/
const Value V21919_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21919_283 = {S5, 0, &V21919_283_57159, &P21919_350};	/*Property = CssBackground*/
const Value V21919_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21919_284 = {S6, 0, &V21919_284_57163, &P21919_283};	/*Property = CssBorderImage*/
const Value V21919_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21919_285 = {S7, 0, &V21919_285_57166, &P21919_284};	/*Property = CssCanvas*/
const Value V21919_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21919_286 = {S8, 0, &V21919_286_57168, &P21919_285};	/*Property = CssColor*/
const Value V21919_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21919_237 = {S9, 0, &V21919_237_55786, &P21919_286};	/*Property = CssColumn*/
const Value V21919_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21919_287 = {S10, 0, &V21919_287_57172, &P21919_237};	/*Property = CssFlexbox*/
const Value V21919_288_57174 = {S2, NULL};	/*Value = True*/
const Property P21919_288 = {S11, 0, &V21919_288_57174, &P21919_287};	/*Property = CssFont*/
const Value V21919_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21919_289 = {S12, 0, &V21919_289_57759, &P21919_288};	/*Property = CssImages*/
const Value V21919_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21919_291 = {S13, 0, &V21919_291_57181, &P21919_289};	/*Property = CssMediaQueries*/
const Value V21919_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21919_292 = {S14, 0, &V21919_292_57183, &P21919_291};	/*Property = CssMinMax*/
const Value V21919_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21919_293 = {S15, 0, &V21919_293_57187, &P21919_292};	/*Property = CssOverflow*/
const Value V21919_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21919_294 = {S16, 0, &V21919_294_57189, &P21919_293};	/*Property = CssPosition*/
const Value V21919_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21919_295 = {S17, 0, &V21919_295_57192, &P21919_294};	/*Property = CssText*/
const Value V21919_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21919_297 = {S18, 0, &V21919_297_57195, &P21919_295};	/*Property = CssTransforms*/
const Value V21919_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21919_241 = {S19, 0, &V21919_241_55799, &P21919_297};	/*Property = CssTransitions*/
const Value V21919_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21919_298 = {S20, 0, &V21919_298_57206, &P21919_241};	/*Property = CssUI*/
const Value V21919_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21919_300 = {S21, 0, &V21919_300_57216, &P21919_298};	/*Property = DataSet*/
const Value V21919_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21919_301 = {S22, 0, &V21919_301_57218, &P21919_300};	/*Property = DataUrl*/
const Value V21919_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21919_302 = {S23, 0, &V21919_302_57222, &P21919_301};	/*Property = DeviceOrientation*/
const Value V21919_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21919_304 = {S24, 0, &V21919_304_57227, &P21919_302};	/*Property = FileReader*/
const Value V21919_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21919_305 = {S25, 0, &V21919_305_57231, &P21919_304};	/*Property = FileSaver*/
const Value V21919_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21919_306 = {S26, 0, &V21919_306_57237, &P21919_305};	/*Property = FileWriter*/
const Value V21919_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21919_307 = {S27, 0, &V21919_307_57239, &P21919_306};	/*Property = FormData*/
const Value V21919_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21919_309 = {S28, 0, &V21919_309_57246, &P21919_307};	/*Property = Fullscreen*/
const Value V21919_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21919_310 = {S29, 0, &V21919_310_57248, &P21919_309};	/*Property = GeoLocation*/
const Value V21919_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21919_312 = {S30, 0, &V21919_312_57254, &P21919_310};	/*Property = History*/
const Value V21919_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21919_314 = {S31, 0, &V21919_314_57261, &P21919_312};	/*Property = Html5*/
const Value V21919_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21919_313 = {S32, 0, &V21919_313_57258, &P21919_314};	/*Property = Html-Media-Capture*/
const Value V21919_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21919_315 = {S33, 0, &V21919_315_57264, &P21919_313};	/*Property = Iframe*/
const Value V21919_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21919_316 = {S34, 0, &V21919_316_57266, &P21919_315};	/*Property = IndexedDB*/
const Value V21919_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21919_317 = {S35, 0, &V21919_317_57269, &P21919_316};	/*Property = Json*/
const Value V21919_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21919_15 = {S37, 0, &V21919_15_28, &P21919_317};	/*Property = LayoutEngine*/
const Value V21919_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21919_344 = {S38, 0, &V21919_344_57272, &P21919_15};	/*Property = Masking*/
const Value V21919_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21919_320 = {S39, 0, &V21919_320_57290, &P21919_344};	/*Property = PostMessage*/
const Value V21919_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21919_321 = {S40, 0, &V21919_321_57293, &P21919_320};	/*Property = Progress*/
const Value V21919_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21919_322 = {S41, 0, &V21919_322_57296, &P21919_321};	/*Property = Prompts*/
const Value V21919_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21919_323 = {S42, 0, &V21919_323_57299, &P21919_322};	/*Property = Selector*/
const Value V21919_324_57305 = {S2, NULL};	/*Value = True*/
const Property P21919_324 = {S43, 0, &V21919_324_57305, &P21919_323};	/*Property = Svg*/
const Value V21919_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21919_327 = {S44, 0, &V21919_327_57315, &P21919_324};	/*Property = TouchEvents*/
const Value V21919_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21919_328 = {S45, 0, &V21919_328_57318, &P21919_327};	/*Property = Track*/
const Value V21919_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21919_329 = {S46, 0, &V21919_329_57323, &P21919_328};	/*Property = Video*/
const Value V21919_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21919_330 = {S47, 0, &V21919_330_57326, &P21919_329};	/*Property = Viewport*/
const Value V21919_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21919_332 = {S48, 0, &V21919_332_57339, &P21919_330};	/*Property = WebWorkers*/
const Value V21919_333_57341 = {S2, NULL};	/*Value = True*/
const Property P21919_333 = {S49, 0, &V21919_333_57341, &P21919_332};	/*Property = Xhr2*/
const Profile P21919 = {"21919", &P21919_333};	/*Profile ID - 21919*/
const Value V22540_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22540_279 = {S1, 0, &V22540_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22540_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22540_282 = {S3, 0, &V22540_282_57154, &P22540_279};	/*Property = BlobBuilder*/
const Value V22540_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22540_350 = {S4, 0, &V22540_350_57444, &P22540_282};	/*Property = Canvas*/
const Value V22540_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22540_283 = {S5, 0, &V22540_283_57159, &P22540_350};	/*Property = CssBackground*/
const Value V22540_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22540_284 = {S6, 0, &V22540_284_57163, &P22540_283};	/*Property = CssBorderImage*/
const Value V22540_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22540_285 = {S7, 0, &V22540_285_57166, &P22540_284};	/*Property = CssCanvas*/
const Value V22540_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22540_286 = {S8, 0, &V22540_286_57168, &P22540_285};	/*Property = CssColor*/
const Value V22540_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22540_237 = {S9, 0, &V22540_237_55784, &P22540_286};	/*Property = CssColumn*/
const Value V22540_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22540_287 = {S10, 0, &V22540_287_57172, &P22540_237};	/*Property = CssFlexbox*/
const Value V22540_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22540_288 = {S11, 0, &V22540_288_57175, &P22540_287};	/*Property = CssFont*/
const Value V22540_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22540_289 = {S12, 0, &V22540_289_57759, &P22540_288};	/*Property = CssImages*/
const Value V22540_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22540_291 = {S13, 0, &V22540_291_57181, &P22540_289};	/*Property = CssMediaQueries*/
const Value V22540_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22540_292 = {S14, 0, &V22540_292_57183, &P22540_291};	/*Property = CssMinMax*/
const Value V22540_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22540_293 = {S15, 0, &V22540_293_57187, &P22540_292};	/*Property = CssOverflow*/
const Value V22540_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22540_294 = {S16, 0, &V22540_294_57190, &P22540_293};	/*Property = CssPosition*/
const Value V22540_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22540_295 = {S17, 0, &V22540_295_57193, &P22540_294};	/*Property = CssText*/
const Value V22540_297_57196 = {S0, NULL};	/*Value = False*/
const Property P22540_297 = {S18, 0, &V22540_297_57196, &P22540_295};	/*Property = CssTransforms*/
const Value V22540_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22540_241 = {S19, 0, &V22540_241_55799, &P22540_297};	/*Property = CssTransitions*/
const Value V22540_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22540_298 = {S20, 0, &V22540_298_57206, &P22540_241};	/*Property = CssUI*/
const Value V22540_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22540_300 = {S21, 0, &V22540_300_57216, &P22540_298};	/*Property = DataSet*/
const Value V22540_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22540_301 = {S22, 0, &V22540_301_57218, &P22540_300};	/*Property = DataUrl*/
const Value V22540_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22540_302 = {S23, 0, &V22540_302_57222, &P22540_301};	/*Property = DeviceOrientation*/
const Value V22540_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22540_304 = {S24, 0, &V22540_304_57228, &P22540_302};	/*Property = FileReader*/
const Value V22540_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22540_305 = {S25, 0, &V22540_305_57231, &P22540_304};	/*Property = FileSaver*/
const Value V22540_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22540_306 = {S26, 0, &V22540_306_57237, &P22540_305};	/*Property = FileWriter*/
const Value V22540_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22540_307 = {S27, 0, &V22540_307_57240, &P22540_306};	/*Property = FormData*/
const Value V22540_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22540_309 = {S28, 0, &V22540_309_57246, &P22540_307};	/*Property = Fullscreen*/
const Value V22540_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22540_310 = {S29, 0, &V22540_310_57248, &P22540_309};	/*Property = GeoLocation*/
const Value V22540_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22540_312 = {S30, 0, &V22540_312_57255, &P22540_310};	/*Property = History*/
const Value V22540_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22540_314 = {S31, 0, &V22540_314_57261, &P22540_312};	/*Property = Html5*/
const Value V22540_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22540_313 = {S32, 0, &V22540_313_57258, &P22540_314};	/*Property = Html-Media-Capture*/
const Value V22540_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22540_315 = {S33, 0, &V22540_315_57264, &P22540_313};	/*Property = Iframe*/
const Value V22540_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22540_316 = {S34, 0, &V22540_316_57267, &P22540_315};	/*Property = IndexedDB*/
const Value V22540_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22540_317 = {S35, 0, &V22540_317_57269, &P22540_316};	/*Property = Json*/
const Value V22540_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P22540_15 = {S37, 0, &V22540_15_31, &P22540_317};	/*Property = LayoutEngine*/
const Value V22540_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22540_344 = {S38, 0, &V22540_344_57272, &P22540_15};	/*Property = Masking*/
const Value V22540_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22540_320 = {S39, 0, &V22540_320_57290, &P22540_344};	/*Property = PostMessage*/
const Value V22540_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22540_321 = {S40, 0, &V22540_321_57293, &P22540_320};	/*Property = Progress*/
const Value V22540_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22540_322 = {S41, 0, &V22540_322_57296, &P22540_321};	/*Property = Prompts*/
const Value V22540_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22540_323 = {S42, 0, &V22540_323_57299, &P22540_322};	/*Property = Selector*/
const Value V22540_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22540_324 = {S43, 0, &V22540_324_57306, &P22540_323};	/*Property = Svg*/
const Value V22540_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22540_327 = {S44, 0, &V22540_327_57315, &P22540_324};	/*Property = TouchEvents*/
const Value V22540_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22540_328 = {S45, 0, &V22540_328_57318, &P22540_327};	/*Property = Track*/
const Value V22540_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22540_329 = {S46, 0, &V22540_329_57323, &P22540_328};	/*Property = Video*/
const Value V22540_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22540_330 = {S47, 0, &V22540_330_57326, &P22540_329};	/*Property = Viewport*/
const Value V22540_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22540_332 = {S48, 0, &V22540_332_57339, &P22540_330};	/*Property = WebWorkers*/
const Value V22540_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22540_333 = {S49, 0, &V22540_333_57342, &P22540_332};	/*Property = Xhr2*/
const Profile P22540 = {"22540", &P22540_333};	/*Profile ID - 22540*/
const Value V18414_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_279 = {S1, 0, &V18414_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18414_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_282 = {S3, 0, &V18414_282_57155, &P18414_279};	/*Property = BlobBuilder*/
const Value V18414_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_350 = {S4, 0, &V18414_350_57446, &P18414_282};	/*Property = Canvas*/
const Value V18414_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_283 = {S5, 0, &V18414_283_57161, &P18414_350};	/*Property = CssBackground*/
const Value V18414_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_284 = {S6, 0, &V18414_284_57164, &P18414_283};	/*Property = CssBorderImage*/
const Value V18414_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_285 = {S7, 0, &V18414_285_57167, &P18414_284};	/*Property = CssCanvas*/
const Value V18414_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_286 = {S8, 0, &V18414_286_57170, &P18414_285};	/*Property = CssColor*/
const Value V18414_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_237 = {S9, 0, &V18414_237_55786, &P18414_286};	/*Property = CssColumn*/
const Value V18414_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_287 = {S10, 0, &V18414_287_57173, &P18414_237};	/*Property = CssFlexbox*/
const Value V18414_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_288 = {S11, 0, &V18414_288_57176, &P18414_287};	/*Property = CssFont*/
const Value V18414_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_289 = {S12, 0, &V18414_289_86899, &P18414_288};	/*Property = CssImages*/
const Value V18414_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_291 = {S13, 0, &V18414_291_57182, &P18414_289};	/*Property = CssMediaQueries*/
const Value V18414_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_292 = {S14, 0, &V18414_292_57185, &P18414_291};	/*Property = CssMinMax*/
const Value V18414_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_293 = {S15, 0, &V18414_293_57188, &P18414_292};	/*Property = CssOverflow*/
const Value V18414_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_294 = {S16, 0, &V18414_294_57191, &P18414_293};	/*Property = CssPosition*/
const Value V18414_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_295 = {S17, 0, &V18414_295_57194, &P18414_294};	/*Property = CssText*/
const Value V18414_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_297 = {S18, 0, &V18414_297_57197, &P18414_295};	/*Property = CssTransforms*/
const Value V18414_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_241 = {S19, 0, &V18414_241_55801, &P18414_297};	/*Property = CssTransitions*/
const Value V18414_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_298 = {S20, 0, &V18414_298_57208, &P18414_241};	/*Property = CssUI*/
const Value V18414_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_300 = {S21, 0, &V18414_300_57217, &P18414_298};	/*Property = DataSet*/
const Value V18414_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_301 = {S22, 0, &V18414_301_57220, &P18414_300};	/*Property = DataUrl*/
const Value V18414_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_302 = {S23, 0, &V18414_302_57223, &P18414_301};	/*Property = DeviceOrientation*/
const Value V18414_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_304 = {S24, 0, &V18414_304_57229, &P18414_302};	/*Property = FileReader*/
const Value V18414_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_305 = {S25, 0, &V18414_305_57232, &P18414_304};	/*Property = FileSaver*/
const Value V18414_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_306 = {S26, 0, &V18414_306_57238, &P18414_305};	/*Property = FileWriter*/
const Value V18414_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_307 = {S27, 0, &V18414_307_57241, &P18414_306};	/*Property = FormData*/
const Value V18414_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_309 = {S28, 0, &V18414_309_57247, &P18414_307};	/*Property = Fullscreen*/
const Value V18414_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_310 = {S29, 0, &V18414_310_57250, &P18414_309};	/*Property = GeoLocation*/
const Value V18414_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_312 = {S30, 0, &V18414_312_57256, &P18414_310};	/*Property = History*/
const Value V18414_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_314 = {S31, 0, &V18414_314_57262, &P18414_312};	/*Property = Html5*/
const Value V18414_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_313 = {S32, 0, &V18414_313_57259, &P18414_314};	/*Property = Html-Media-Capture*/
const Value V18414_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_315 = {S33, 0, &V18414_315_57265, &P18414_313};	/*Property = Iframe*/
const Value V18414_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_316 = {S34, 0, &V18414_316_57268, &P18414_315};	/*Property = IndexedDB*/
const Value V18414_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_317 = {S35, 0, &V18414_317_57271, &P18414_316};	/*Property = Json*/
const Value V18414_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18414_15 = {S37, 0, &V18414_15_31, &P18414_317};	/*Property = LayoutEngine*/
const Value V18414_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_344 = {S38, 0, &V18414_344_57274, &P18414_15};	/*Property = Masking*/
const Value V18414_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_320 = {S39, 0, &V18414_320_57292, &P18414_344};	/*Property = PostMessage*/
const Value V18414_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_321 = {S40, 0, &V18414_321_57295, &P18414_320};	/*Property = Progress*/
const Value V18414_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_322 = {S41, 0, &V18414_322_57298, &P18414_321};	/*Property = Prompts*/
const Value V18414_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_323 = {S42, 0, &V18414_323_57301, &P18414_322};	/*Property = Selector*/
const Value V18414_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_324 = {S43, 0, &V18414_324_57307, &P18414_323};	/*Property = Svg*/
const Value V18414_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_327 = {S44, 0, &V18414_327_57316, &P18414_324};	/*Property = TouchEvents*/
const Value V18414_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_328 = {S45, 0, &V18414_328_57319, &P18414_327};	/*Property = Track*/
const Value V18414_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_329 = {S46, 0, &V18414_329_57325, &P18414_328};	/*Property = Video*/
const Value V18414_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_330 = {S47, 0, &V18414_330_57328, &P18414_329};	/*Property = Viewport*/
const Value V18414_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_332 = {S48, 0, &V18414_332_57340, &P18414_330};	/*Property = WebWorkers*/
const Value V18414_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18414_333 = {S49, 0, &V18414_333_57343, &P18414_332};	/*Property = Xhr2*/
const Profile P18414 = {"18414", &P18414_333};	/*Profile ID - 18414*/
const Value V24440_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_279 = {S1, 0, &V24440_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24440_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_282 = {S3, 0, &V24440_282_57155, &P24440_279};	/*Property = BlobBuilder*/
const Value V24440_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_350 = {S4, 0, &V24440_350_57446, &P24440_282};	/*Property = Canvas*/
const Value V24440_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_283 = {S5, 0, &V24440_283_57161, &P24440_350};	/*Property = CssBackground*/
const Value V24440_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_284 = {S6, 0, &V24440_284_57164, &P24440_283};	/*Property = CssBorderImage*/
const Value V24440_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_285 = {S7, 0, &V24440_285_57167, &P24440_284};	/*Property = CssCanvas*/
const Value V24440_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_286 = {S8, 0, &V24440_286_57170, &P24440_285};	/*Property = CssColor*/
const Value V24440_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_237 = {S9, 0, &V24440_237_55786, &P24440_286};	/*Property = CssColumn*/
const Value V24440_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_287 = {S10, 0, &V24440_287_57173, &P24440_237};	/*Property = CssFlexbox*/
const Value V24440_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_288 = {S11, 0, &V24440_288_57176, &P24440_287};	/*Property = CssFont*/
const Value V24440_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_289 = {S12, 0, &V24440_289_86899, &P24440_288};	/*Property = CssImages*/
const Value V24440_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_291 = {S13, 0, &V24440_291_57182, &P24440_289};	/*Property = CssMediaQueries*/
const Value V24440_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_292 = {S14, 0, &V24440_292_57185, &P24440_291};	/*Property = CssMinMax*/
const Value V24440_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_293 = {S15, 0, &V24440_293_57188, &P24440_292};	/*Property = CssOverflow*/
const Value V24440_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_294 = {S16, 0, &V24440_294_57191, &P24440_293};	/*Property = CssPosition*/
const Value V24440_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_295 = {S17, 0, &V24440_295_57194, &P24440_294};	/*Property = CssText*/
const Value V24440_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_297 = {S18, 0, &V24440_297_57197, &P24440_295};	/*Property = CssTransforms*/
const Value V24440_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_241 = {S19, 0, &V24440_241_55801, &P24440_297};	/*Property = CssTransitions*/
const Value V24440_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_298 = {S20, 0, &V24440_298_57208, &P24440_241};	/*Property = CssUI*/
const Value V24440_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_300 = {S21, 0, &V24440_300_57217, &P24440_298};	/*Property = DataSet*/
const Value V24440_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_301 = {S22, 0, &V24440_301_57220, &P24440_300};	/*Property = DataUrl*/
const Value V24440_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_302 = {S23, 0, &V24440_302_57223, &P24440_301};	/*Property = DeviceOrientation*/
const Value V24440_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_304 = {S24, 0, &V24440_304_57229, &P24440_302};	/*Property = FileReader*/
const Value V24440_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_305 = {S25, 0, &V24440_305_57232, &P24440_304};	/*Property = FileSaver*/
const Value V24440_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_306 = {S26, 0, &V24440_306_57238, &P24440_305};	/*Property = FileWriter*/
const Value V24440_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_307 = {S27, 0, &V24440_307_57241, &P24440_306};	/*Property = FormData*/
const Value V24440_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_309 = {S28, 0, &V24440_309_57247, &P24440_307};	/*Property = Fullscreen*/
const Value V24440_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_310 = {S29, 0, &V24440_310_57250, &P24440_309};	/*Property = GeoLocation*/
const Value V24440_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_312 = {S30, 0, &V24440_312_57256, &P24440_310};	/*Property = History*/
const Value V24440_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_314 = {S31, 0, &V24440_314_57262, &P24440_312};	/*Property = Html5*/
const Value V24440_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_313 = {S32, 0, &V24440_313_57259, &P24440_314};	/*Property = Html-Media-Capture*/
const Value V24440_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_315 = {S33, 0, &V24440_315_57265, &P24440_313};	/*Property = Iframe*/
const Value V24440_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_316 = {S34, 0, &V24440_316_57268, &P24440_315};	/*Property = IndexedDB*/
const Value V24440_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_317 = {S35, 0, &V24440_317_57271, &P24440_316};	/*Property = Json*/
const Value V24440_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24440_15 = {S37, 0, &V24440_15_28, &P24440_317};	/*Property = LayoutEngine*/
const Value V24440_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_344 = {S38, 0, &V24440_344_57274, &P24440_15};	/*Property = Masking*/
const Value V24440_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_320 = {S39, 0, &V24440_320_57292, &P24440_344};	/*Property = PostMessage*/
const Value V24440_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_321 = {S40, 0, &V24440_321_57295, &P24440_320};	/*Property = Progress*/
const Value V24440_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_322 = {S41, 0, &V24440_322_57298, &P24440_321};	/*Property = Prompts*/
const Value V24440_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_323 = {S42, 0, &V24440_323_57301, &P24440_322};	/*Property = Selector*/
const Value V24440_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_324 = {S43, 0, &V24440_324_57307, &P24440_323};	/*Property = Svg*/
const Value V24440_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_327 = {S44, 0, &V24440_327_57316, &P24440_324};	/*Property = TouchEvents*/
const Value V24440_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_328 = {S45, 0, &V24440_328_57319, &P24440_327};	/*Property = Track*/
const Value V24440_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_329 = {S46, 0, &V24440_329_57325, &P24440_328};	/*Property = Video*/
const Value V24440_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_330 = {S47, 0, &V24440_330_57328, &P24440_329};	/*Property = Viewport*/
const Value V24440_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_332 = {S48, 0, &V24440_332_57340, &P24440_330};	/*Property = WebWorkers*/
const Value V24440_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24440_333 = {S49, 0, &V24440_333_57343, &P24440_332};	/*Property = Xhr2*/
const Profile P24440 = {"24440", &P24440_333};	/*Profile ID - 24440*/
const Value V19771_1_1 = {S2, NULL};	/*Value = True*/
const Property P19771_1 = {S50, 0, &V19771_1_1, NULL};	/*Property = IsMobile*/
const Value V19771_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P19771_5 = {S52, 0, &V19771_5_1403, &P19771_1};	/*Property = ScreenPixelsHeight*/
const Value V19771_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P19771_4 = {S54, 0, &V19771_4_1404, &P19771_5};	/*Property = ScreenPixelsWidth*/
const Profile P19771 = {"19771", &P19771_4};	/*Profile ID - 19771*/
const Value V19792_1_1 = {S2, NULL};	/*Value = True*/
const Property P19792_1 = {S50, 0, &V19792_1_1, NULL};	/*Property = IsMobile*/
const Value V19792_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P19792_5 = {S52, 0, &V19792_5_1227, &P19792_1};	/*Property = ScreenPixelsHeight*/
const Value V19792_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P19792_4 = {S54, 0, &V19792_4_1229, &P19792_5};	/*Property = ScreenPixelsWidth*/
const Profile P19792 = {"19792", &P19792_4};	/*Profile ID - 19792*/
const Value V19793_1_1 = {S2, NULL};	/*Value = True*/
const Property P19793_1 = {S50, 0, &V19793_1_1, NULL};	/*Property = IsMobile*/
const Value V19793_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P19793_5 = {S52, 0, &V19793_5_1458, &P19793_1};	/*Property = ScreenPixelsHeight*/
const Value V19793_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P19793_4 = {S54, 0, &V19793_4_1460, &P19793_5};	/*Property = ScreenPixelsWidth*/
const Profile P19793 = {"19793", &P19793_4};	/*Profile ID - 19793*/
const Value V19755_1_1 = {S2, NULL};	/*Value = True*/
const Property P19755_1 = {S50, 0, &V19755_1_1, NULL};	/*Property = IsMobile*/
const Value V19755_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P19755_5 = {S52, 0, &V19755_5_1458, &P19755_1};	/*Property = ScreenPixelsHeight*/
const Value V19755_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P19755_4 = {S54, 0, &V19755_4_37617, &P19755_5};	/*Property = ScreenPixelsWidth*/
const Profile P19755 = {"19755", &P19755_4};	/*Profile ID - 19755*/
const Value V23644_1_2 = {S0, NULL};	/*Value = False*/
const Property P23644_1 = {S50, 0, &V23644_1_2, NULL};	/*Property = IsMobile*/
const Value V23644_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23644_5 = {S52, 0, &V23644_5_60417, &P23644_1};	/*Property = ScreenPixelsHeight*/
const Value V23644_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23644_4 = {S54, 0, &V23644_4_57095, &P23644_5};	/*Property = ScreenPixelsWidth*/
const Profile P23644 = {"23644", &P23644_4};	/*Profile ID - 23644*/
const Value V21296_1_1 = {S2, NULL};	/*Value = True*/
const Property P21296_1 = {S50, 0, &V21296_1_1, NULL};	/*Property = IsMobile*/
const Value V21296_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21296_5 = {S52, 0, &V21296_5_37, &P21296_1};	/*Property = ScreenPixelsHeight*/
const Value V21296_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21296_4 = {S54, 0, &V21296_4_36, &P21296_5};	/*Property = ScreenPixelsWidth*/
const Profile P21296 = {"21296", &P21296_4};	/*Profile ID - 21296*/
const Value V21872_1_1 = {S2, NULL};	/*Value = True*/
const Property P21872_1 = {S50, 0, &V21872_1_1, NULL};	/*Property = IsMobile*/
const Value V21872_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21872_5 = {S52, 0, &V21872_5_1227, &P21872_1};	/*Property = ScreenPixelsHeight*/
const Value V21872_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21872_4 = {S54, 0, &V21872_4_1229, &P21872_5};	/*Property = ScreenPixelsWidth*/
const Profile P21872 = {"21872", &P21872_4};	/*Profile ID - 21872*/
const Value V22042_1_1 = {S2, NULL};	/*Value = True*/
const Property P22042_1 = {S50, 0, &V22042_1_1, NULL};	/*Property = IsMobile*/
const Value V22042_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22042_5 = {S52, 0, &V22042_5_1458, &P22042_1};	/*Property = ScreenPixelsHeight*/
const Value V22042_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22042_4 = {S54, 0, &V22042_4_1460, &P22042_5};	/*Property = ScreenPixelsWidth*/
const Profile P22042 = {"22042", &P22042_4};	/*Profile ID - 22042*/
const Value V22112_1_1 = {S2, NULL};	/*Value = True*/
const Property P22112_1 = {S50, 0, &V22112_1_1, NULL};	/*Property = IsMobile*/
const Value V22112_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22112_5 = {S52, 0, &V22112_5_1277, &P22112_1};	/*Property = ScreenPixelsHeight*/
const Value V22112_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22112_4 = {S54, 0, &V22112_4_1279, &P22112_5};	/*Property = ScreenPixelsWidth*/
const Profile P22112 = {"22112", &P22112_4};	/*Profile ID - 22112*/
const Value V20141_1_1 = {S2, NULL};	/*Value = True*/
const Property P20141_1 = {S50, 0, &V20141_1_1, NULL};	/*Property = IsMobile*/
const Value V20141_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20141_5 = {S52, 0, &V20141_5_1222, &P20141_1};	/*Property = ScreenPixelsHeight*/
const Value V20141_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20141_4 = {S54, 0, &V20141_4_1224, &P20141_5};	/*Property = ScreenPixelsWidth*/
const Profile P20141 = {"20141", &P20141_4};	/*Profile ID - 20141*/
const Value V19895_1_1 = {S2, NULL};	/*Value = True*/
const Property P19895_1 = {S50, 0, &V19895_1_1, NULL};	/*Property = IsMobile*/
const Value V19895_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P19895_5 = {S52, 0, &V19895_5_1222, &P19895_1};	/*Property = ScreenPixelsHeight*/
const Value V19895_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P19895_4 = {S54, 0, &V19895_4_1224, &P19895_5};	/*Property = ScreenPixelsWidth*/
const Profile P19895 = {"19895", &P19895_4};	/*Profile ID - 19895*/
const Value V21403_1_1 = {S2, NULL};	/*Value = True*/
const Property P21403_1 = {S50, 0, &V21403_1_1, NULL};	/*Property = IsMobile*/
const Value V21403_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21403_5 = {S52, 0, &V21403_5_1227, &P21403_1};	/*Property = ScreenPixelsHeight*/
const Value V21403_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21403_4 = {S54, 0, &V21403_4_1229, &P21403_5};	/*Property = ScreenPixelsWidth*/
const Profile P21403 = {"21403", &P21403_4};	/*Profile ID - 21403*/
const Value V21840_1_1 = {S2, NULL};	/*Value = True*/
const Property P21840_1 = {S50, 0, &V21840_1_1, NULL};	/*Property = IsMobile*/
const Value V21840_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21840_5 = {S52, 0, &V21840_5_1458, &P21840_1};	/*Property = ScreenPixelsHeight*/
const Value V21840_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21840_4 = {S54, 0, &V21840_4_37617, &P21840_5};	/*Property = ScreenPixelsWidth*/
const Profile P21840 = {"21840", &P21840_4};	/*Profile ID - 21840*/
const Value V20552_1_1 = {S2, NULL};	/*Value = True*/
const Property P20552_1 = {S50, 0, &V20552_1_1, NULL};	/*Property = IsMobile*/
const Value V20552_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20552_5 = {S52, 0, &V20552_5_37, &P20552_1};	/*Property = ScreenPixelsHeight*/
const Value V20552_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20552_4 = {S54, 0, &V20552_4_36, &P20552_5};	/*Property = ScreenPixelsWidth*/
const Profile P20552 = {"20552", &P20552_4};	/*Profile ID - 20552*/
const Value V21523_1_1 = {S2, NULL};	/*Value = True*/
const Property P21523_1 = {S50, 0, &V21523_1_1, NULL};	/*Property = IsMobile*/
const Value V21523_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P21523_5 = {S52, 0, &V21523_5_59138, &P21523_1};	/*Property = ScreenPixelsHeight*/
const Value V21523_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P21523_4 = {S54, 0, &V21523_4_59970, &P21523_5};	/*Property = ScreenPixelsWidth*/
const Profile P21523 = {"21523", &P21523_4};	/*Profile ID - 21523*/
const Value V20536_1_1 = {S2, NULL};	/*Value = True*/
const Property P20536_1 = {S50, 0, &V20536_1_1, NULL};	/*Property = IsMobile*/
const Value V20536_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20536_5 = {S52, 0, &V20536_5_1277, &P20536_1};	/*Property = ScreenPixelsHeight*/
const Value V20536_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20536_4 = {S54, 0, &V20536_4_1279, &P20536_5};	/*Property = ScreenPixelsWidth*/
const Profile P20536 = {"20536", &P20536_4};	/*Profile ID - 20536*/
const Value V21978_1_1 = {S2, NULL};	/*Value = True*/
const Property P21978_1 = {S50, 0, &V21978_1_1, NULL};	/*Property = IsMobile*/
const Value V21978_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21978_5 = {S52, 0, &V21978_5_1227, &P21978_1};	/*Property = ScreenPixelsHeight*/
const Value V21978_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21978_4 = {S54, 0, &V21978_4_1229, &P21978_5};	/*Property = ScreenPixelsWidth*/
const Profile P21978 = {"21978", &P21978_4};	/*Profile ID - 21978*/
const Value V20157_1_1 = {S2, NULL};	/*Value = True*/
const Property P20157_1 = {S50, 0, &V20157_1_1, NULL};	/*Property = IsMobile*/
const Value V20157_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20157_5 = {S52, 0, &V20157_5_1277, &P20157_1};	/*Property = ScreenPixelsHeight*/
const Value V20157_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20157_4 = {S54, 0, &V20157_4_1279, &P20157_5};	/*Property = ScreenPixelsWidth*/
const Profile P20157 = {"20157", &P20157_4};	/*Profile ID - 20157*/
const Value V20394_1_2 = {S0, NULL};	/*Value = False*/
const Property P20394_1 = {S50, 0, &V20394_1_2, NULL};	/*Property = IsMobile*/
const Value V20394_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20394_5 = {S52, 0, &V20394_5_37, &P20394_1};	/*Property = ScreenPixelsHeight*/
const Value V20394_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20394_4 = {S54, 0, &V20394_4_36, &P20394_5};	/*Property = ScreenPixelsWidth*/
const Profile P20394 = {"20394", &P20394_4};	/*Profile ID - 20394*/
const Value V20986_1_1 = {S2, NULL};	/*Value = True*/
const Property P20986_1 = {S50, 0, &V20986_1_1, NULL};	/*Property = IsMobile*/
const Value V20986_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20986_5 = {S52, 0, &V20986_5_1227, &P20986_1};	/*Property = ScreenPixelsHeight*/
const Value V20986_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20986_4 = {S54, 0, &V20986_4_1229, &P20986_5};	/*Property = ScreenPixelsWidth*/
const Profile P20986 = {"20986", &P20986_4};	/*Profile ID - 20986*/
const Value V21607_1_1 = {S2, NULL};	/*Value = True*/
const Property P21607_1 = {S50, 0, &V21607_1_1, NULL};	/*Property = IsMobile*/
const Value V21607_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21607_5 = {S52, 0, &V21607_5_1403, &P21607_1};	/*Property = ScreenPixelsHeight*/
const Value V21607_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21607_4 = {S54, 0, &V21607_4_1404, &P21607_5};	/*Property = ScreenPixelsWidth*/
const Profile P21607 = {"21607", &P21607_4};	/*Profile ID - 21607*/
const Value V20355_1_1 = {S2, NULL};	/*Value = True*/
const Property P20355_1 = {S50, 0, &V20355_1_1, NULL};	/*Property = IsMobile*/
const Value V20355_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20355_5 = {S52, 0, &V20355_5_1277, &P20355_1};	/*Property = ScreenPixelsHeight*/
const Value V20355_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20355_4 = {S54, 0, &V20355_4_1279, &P20355_5};	/*Property = ScreenPixelsWidth*/
const Profile P20355 = {"20355", &P20355_4};	/*Profile ID - 20355*/
const Value V23359_1_1 = {S2, NULL};	/*Value = True*/
const Property P23359_1 = {S50, 0, &V23359_1_1, NULL};	/*Property = IsMobile*/
const Value V23359_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23359_5 = {S52, 0, &V23359_5_1277, &P23359_1};	/*Property = ScreenPixelsHeight*/
const Value V23359_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23359_4 = {S54, 0, &V23359_4_1279, &P23359_5};	/*Property = ScreenPixelsWidth*/
const Profile P23359 = {"23359", &P23359_4};	/*Profile ID - 23359*/
const Value V21950_1_1 = {S2, NULL};	/*Value = True*/
const Property P21950_1 = {S50, 0, &V21950_1_1, NULL};	/*Property = IsMobile*/
const Value V21950_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21950_5 = {S52, 0, &V21950_5_1458, &P21950_1};	/*Property = ScreenPixelsHeight*/
const Value V21950_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21950_4 = {S54, 0, &V21950_4_37617, &P21950_5};	/*Property = ScreenPixelsWidth*/
const Profile P21950 = {"21950", &P21950_4};	/*Profile ID - 21950*/
const Value V20059_1_1 = {S2, NULL};	/*Value = True*/
const Property P20059_1 = {S50, 0, &V20059_1_1, NULL};	/*Property = IsMobile*/
const Value V20059_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20059_5 = {S52, 0, &V20059_5_1222, &P20059_1};	/*Property = ScreenPixelsHeight*/
const Value V20059_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20059_4 = {S54, 0, &V20059_4_1224, &P20059_5};	/*Property = ScreenPixelsWidth*/
const Profile P20059 = {"20059", &P20059_4};	/*Profile ID - 20059*/
const Value V20136_1_1 = {S2, NULL};	/*Value = True*/
const Property P20136_1 = {S50, 0, &V20136_1_1, NULL};	/*Property = IsMobile*/
const Value V20136_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20136_5 = {S52, 0, &V20136_5_1277, &P20136_1};	/*Property = ScreenPixelsHeight*/
const Value V20136_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20136_4 = {S54, 0, &V20136_4_1279, &P20136_5};	/*Property = ScreenPixelsWidth*/
const Profile P20136 = {"20136", &P20136_4};	/*Profile ID - 20136*/
const Value V23977_1_1 = {S2, NULL};	/*Value = True*/
const Property P23977_1 = {S50, 0, &V23977_1_1, NULL};	/*Property = IsMobile*/
const Value V23977_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23977_5 = {S52, 0, &V23977_5_1227, &P23977_1};	/*Property = ScreenPixelsHeight*/
const Value V23977_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23977_4 = {S54, 0, &V23977_4_1229, &P23977_5};	/*Property = ScreenPixelsWidth*/
const Profile P23977 = {"23977", &P23977_4};	/*Profile ID - 23977*/
const Value V23452_1_1 = {S2, NULL};	/*Value = True*/
const Property P23452_1 = {S50, 0, &V23452_1_1, NULL};	/*Property = IsMobile*/
const Value V23452_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23452_5 = {S52, 0, &V23452_5_1227, &P23452_1};	/*Property = ScreenPixelsHeight*/
const Value V23452_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23452_4 = {S54, 0, &V23452_4_1460, &P23452_5};	/*Property = ScreenPixelsWidth*/
const Profile P23452 = {"23452", &P23452_4};	/*Profile ID - 23452*/
const Value V20327_1_2 = {S0, NULL};	/*Value = False*/
const Property P20327_1 = {S50, 0, &V20327_1_2, NULL};	/*Property = IsMobile*/
const Value V20327_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20327_5 = {S52, 0, &V20327_5_37, &P20327_1};	/*Property = ScreenPixelsHeight*/
const Value V20327_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20327_4 = {S54, 0, &V20327_4_36, &P20327_5};	/*Property = ScreenPixelsWidth*/
const Profile P20327 = {"20327", &P20327_4};	/*Profile ID - 20327*/
const Value V18612_1_1 = {S2, NULL};	/*Value = True*/
const Property P18612_1 = {S50, 0, &V18612_1_1, NULL};	/*Property = IsMobile*/
const Value V18612_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18612_5 = {S52, 0, &V18612_5_1227, &P18612_1};	/*Property = ScreenPixelsHeight*/
const Value V18612_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18612_4 = {S54, 0, &V18612_4_1229, &P18612_5};	/*Property = ScreenPixelsWidth*/
const Profile P18612 = {"18612", &P18612_4};	/*Profile ID - 18612*/
const Value V22673_1_1 = {S2, NULL};	/*Value = True*/
const Property P22673_1 = {S50, 0, &V22673_1_1, NULL};	/*Property = IsMobile*/
const Value V22673_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22673_5 = {S52, 0, &V22673_5_1277, &P22673_1};	/*Property = ScreenPixelsHeight*/
const Value V22673_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22673_4 = {S54, 0, &V22673_4_1279, &P22673_5};	/*Property = ScreenPixelsWidth*/
const Profile P22673 = {"22673", &P22673_4};	/*Profile ID - 22673*/
const Value V23599_1_2 = {S0, NULL};	/*Value = False*/
const Property P23599_1 = {S50, 0, &V23599_1_2, NULL};	/*Property = IsMobile*/
const Value V23599_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23599_5 = {S52, 0, &V23599_5_60417, &P23599_1};	/*Property = ScreenPixelsHeight*/
const Value V23599_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23599_4 = {S54, 0, &V23599_4_57095, &P23599_5};	/*Property = ScreenPixelsWidth*/
const Profile P23599 = {"23599", &P23599_4};	/*Profile ID - 23599*/
const Value V21936_1_1 = {S2, NULL};	/*Value = True*/
const Property P21936_1 = {S50, 0, &V21936_1_1, NULL};	/*Property = IsMobile*/
const Value V21936_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21936_5 = {S52, 0, &V21936_5_1403, &P21936_1};	/*Property = ScreenPixelsHeight*/
const Value V21936_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21936_4 = {S54, 0, &V21936_4_1404, &P21936_5};	/*Property = ScreenPixelsWidth*/
const Profile P21936 = {"21936", &P21936_4};	/*Profile ID - 21936*/
const Value V20873_1_1 = {S2, NULL};	/*Value = True*/
const Property P20873_1 = {S50, 0, &V20873_1_1, NULL};	/*Property = IsMobile*/
const Value V20873_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20873_5 = {S52, 0, &V20873_5_1458, &P20873_1};	/*Property = ScreenPixelsHeight*/
const Value V20873_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20873_4 = {S54, 0, &V20873_4_37617, &P20873_5};	/*Property = ScreenPixelsWidth*/
const Profile P20873 = {"20873", &P20873_4};	/*Profile ID - 20873*/
const Value V20914_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_279 = {S1, 0, &V20914_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20914_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_282 = {S3, 0, &V20914_282_57155, &P20914_279};	/*Property = BlobBuilder*/
const Value V20914_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_350 = {S4, 0, &V20914_350_57446, &P20914_282};	/*Property = Canvas*/
const Value V20914_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_283 = {S5, 0, &V20914_283_57161, &P20914_350};	/*Property = CssBackground*/
const Value V20914_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_284 = {S6, 0, &V20914_284_57164, &P20914_283};	/*Property = CssBorderImage*/
const Value V20914_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_285 = {S7, 0, &V20914_285_57167, &P20914_284};	/*Property = CssCanvas*/
const Value V20914_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_286 = {S8, 0, &V20914_286_57170, &P20914_285};	/*Property = CssColor*/
const Value V20914_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_237 = {S9, 0, &V20914_237_55786, &P20914_286};	/*Property = CssColumn*/
const Value V20914_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_287 = {S10, 0, &V20914_287_57173, &P20914_237};	/*Property = CssFlexbox*/
const Value V20914_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_288 = {S11, 0, &V20914_288_57176, &P20914_287};	/*Property = CssFont*/
const Value V20914_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_289 = {S12, 0, &V20914_289_86899, &P20914_288};	/*Property = CssImages*/
const Value V20914_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_291 = {S13, 0, &V20914_291_57182, &P20914_289};	/*Property = CssMediaQueries*/
const Value V20914_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_292 = {S14, 0, &V20914_292_57185, &P20914_291};	/*Property = CssMinMax*/
const Value V20914_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_293 = {S15, 0, &V20914_293_57188, &P20914_292};	/*Property = CssOverflow*/
const Value V20914_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_294 = {S16, 0, &V20914_294_57191, &P20914_293};	/*Property = CssPosition*/
const Value V20914_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_295 = {S17, 0, &V20914_295_57194, &P20914_294};	/*Property = CssText*/
const Value V20914_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_297 = {S18, 0, &V20914_297_57197, &P20914_295};	/*Property = CssTransforms*/
const Value V20914_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_241 = {S19, 0, &V20914_241_55801, &P20914_297};	/*Property = CssTransitions*/
const Value V20914_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_298 = {S20, 0, &V20914_298_57208, &P20914_241};	/*Property = CssUI*/
const Value V20914_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_300 = {S21, 0, &V20914_300_57217, &P20914_298};	/*Property = DataSet*/
const Value V20914_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_301 = {S22, 0, &V20914_301_57220, &P20914_300};	/*Property = DataUrl*/
const Value V20914_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_302 = {S23, 0, &V20914_302_57223, &P20914_301};	/*Property = DeviceOrientation*/
const Value V20914_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_304 = {S24, 0, &V20914_304_57229, &P20914_302};	/*Property = FileReader*/
const Value V20914_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_305 = {S25, 0, &V20914_305_57232, &P20914_304};	/*Property = FileSaver*/
const Value V20914_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_306 = {S26, 0, &V20914_306_57238, &P20914_305};	/*Property = FileWriter*/
const Value V20914_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_307 = {S27, 0, &V20914_307_57241, &P20914_306};	/*Property = FormData*/
const Value V20914_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_309 = {S28, 0, &V20914_309_57247, &P20914_307};	/*Property = Fullscreen*/
const Value V20914_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_310 = {S29, 0, &V20914_310_57250, &P20914_309};	/*Property = GeoLocation*/
const Value V20914_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_312 = {S30, 0, &V20914_312_57256, &P20914_310};	/*Property = History*/
const Value V20914_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_314 = {S31, 0, &V20914_314_57262, &P20914_312};	/*Property = Html5*/
const Value V20914_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_313 = {S32, 0, &V20914_313_57259, &P20914_314};	/*Property = Html-Media-Capture*/
const Value V20914_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_315 = {S33, 0, &V20914_315_57265, &P20914_313};	/*Property = Iframe*/
const Value V20914_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_316 = {S34, 0, &V20914_316_57268, &P20914_315};	/*Property = IndexedDB*/
const Value V20914_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_317 = {S35, 0, &V20914_317_57271, &P20914_316};	/*Property = Json*/
const Value V20914_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P20914_15 = {S37, 0, &V20914_15_28, &P20914_317};	/*Property = LayoutEngine*/
const Value V20914_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_344 = {S38, 0, &V20914_344_57274, &P20914_15};	/*Property = Masking*/
const Value V20914_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_320 = {S39, 0, &V20914_320_57292, &P20914_344};	/*Property = PostMessage*/
const Value V20914_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_321 = {S40, 0, &V20914_321_57295, &P20914_320};	/*Property = Progress*/
const Value V20914_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_322 = {S41, 0, &V20914_322_57298, &P20914_321};	/*Property = Prompts*/
const Value V20914_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_323 = {S42, 0, &V20914_323_57301, &P20914_322};	/*Property = Selector*/
const Value V20914_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_324 = {S43, 0, &V20914_324_57307, &P20914_323};	/*Property = Svg*/
const Value V20914_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_327 = {S44, 0, &V20914_327_57316, &P20914_324};	/*Property = TouchEvents*/
const Value V20914_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_328 = {S45, 0, &V20914_328_57319, &P20914_327};	/*Property = Track*/
const Value V20914_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_329 = {S46, 0, &V20914_329_57325, &P20914_328};	/*Property = Video*/
const Value V20914_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_330 = {S47, 0, &V20914_330_57328, &P20914_329};	/*Property = Viewport*/
const Value V20914_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_332 = {S48, 0, &V20914_332_57340, &P20914_330};	/*Property = WebWorkers*/
const Value V20914_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20914_333 = {S49, 0, &V20914_333_57343, &P20914_332};	/*Property = Xhr2*/
const Profile P20914 = {"20914", &P20914_333};	/*Profile ID - 20914*/
const Value V20351_1_1 = {S2, NULL};	/*Value = True*/
const Property P20351_1 = {S50, 0, &V20351_1_1, NULL};	/*Property = IsMobile*/
const Value V20351_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20351_5 = {S52, 0, &V20351_5_1458, &P20351_1};	/*Property = ScreenPixelsHeight*/
const Value V20351_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20351_4 = {S54, 0, &V20351_4_1460, &P20351_5};	/*Property = ScreenPixelsWidth*/
const Profile P20351 = {"20351", &P20351_4};	/*Profile ID - 20351*/
const Value V23559_1_1 = {S2, NULL};	/*Value = True*/
const Property P23559_1 = {S50, 0, &V23559_1_1, NULL};	/*Property = IsMobile*/
const Value V23559_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23559_5 = {S52, 0, &V23559_5_1403, &P23559_1};	/*Property = ScreenPixelsHeight*/
const Value V23559_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23559_4 = {S54, 0, &V23559_4_1404, &P23559_5};	/*Property = ScreenPixelsWidth*/
const Profile P23559 = {"23559", &P23559_4};	/*Profile ID - 23559*/
const Value V21443_1_1 = {S2, NULL};	/*Value = True*/
const Property P21443_1 = {S50, 0, &V21443_1_1, NULL};	/*Property = IsMobile*/
const Value V21443_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21443_5 = {S52, 0, &V21443_5_9981, &P21443_1};	/*Property = ScreenPixelsHeight*/
const Value V21443_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21443_4 = {S54, 0, &V21443_4_2336, &P21443_5};	/*Property = ScreenPixelsWidth*/
const Profile P21443 = {"21443", &P21443_4};	/*Profile ID - 21443*/
const Value V20237_1_1 = {S2, NULL};	/*Value = True*/
const Property P20237_1 = {S50, 0, &V20237_1_1, NULL};	/*Property = IsMobile*/
const Value V20237_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20237_5 = {S52, 0, &V20237_5_9981, &P20237_1};	/*Property = ScreenPixelsHeight*/
const Value V20237_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20237_4 = {S54, 0, &V20237_4_39098, &P20237_5};	/*Property = ScreenPixelsWidth*/
const Profile P20237 = {"20237", &P20237_4};	/*Profile ID - 20237*/
const Value V20468_1_1 = {S2, NULL};	/*Value = True*/
const Property P20468_1 = {S50, 0, &V20468_1_1, NULL};	/*Property = IsMobile*/
const Value V20468_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20468_5 = {S52, 0, &V20468_5_1403, &P20468_1};	/*Property = ScreenPixelsHeight*/
const Value V20468_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P20468_4 = {S54, 0, &V20468_4_8554, &P20468_5};	/*Property = ScreenPixelsWidth*/
const Profile P20468 = {"20468", &P20468_4};	/*Profile ID - 20468*/
const Value V22004_1_1 = {S2, NULL};	/*Value = True*/
const Property P22004_1 = {S50, 0, &V22004_1_1, NULL};	/*Property = IsMobile*/
const Value V22004_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22004_5 = {S52, 0, &V22004_5_9981, &P22004_1};	/*Property = ScreenPixelsHeight*/
const Value V22004_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22004_4 = {S54, 0, &V22004_4_39098, &P22004_5};	/*Property = ScreenPixelsWidth*/
const Profile P22004 = {"22004", &P22004_4};	/*Profile ID - 22004*/
const Value V21261_1_1 = {S2, NULL};	/*Value = True*/
const Property P21261_1 = {S50, 0, &V21261_1_1, NULL};	/*Property = IsMobile*/
const Value V21261_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P21261_5 = {S52, 0, &V21261_5_59138, &P21261_1};	/*Property = ScreenPixelsHeight*/
const Value V21261_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P21261_4 = {S54, 0, &V21261_4_59970, &P21261_5};	/*Property = ScreenPixelsWidth*/
const Profile P21261 = {"21261", &P21261_4};	/*Profile ID - 21261*/
const Value V20280_1_1 = {S2, NULL};	/*Value = True*/
const Property P20280_1 = {S50, 0, &V20280_1_1, NULL};	/*Property = IsMobile*/
const Value V20280_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20280_5 = {S52, 0, &V20280_5_1403, &P20280_1};	/*Property = ScreenPixelsHeight*/
const Value V20280_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20280_4 = {S54, 0, &V20280_4_1404, &P20280_5};	/*Property = ScreenPixelsWidth*/
const Profile P20280 = {"20280", &P20280_4};	/*Profile ID - 20280*/
const Value V20172_1_1 = {S2, NULL};	/*Value = True*/
const Property P20172_1 = {S50, 0, &V20172_1_1, NULL};	/*Property = IsMobile*/
const Value V20172_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20172_5 = {S52, 0, &V20172_5_1277, &P20172_1};	/*Property = ScreenPixelsHeight*/
const Value V20172_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20172_4 = {S54, 0, &V20172_4_1279, &P20172_5};	/*Property = ScreenPixelsWidth*/
const Profile P20172 = {"20172", &P20172_4};	/*Profile ID - 20172*/
const Value V23946_1_1 = {S2, NULL};	/*Value = True*/
const Property P23946_1 = {S50, 0, &V23946_1_1, NULL};	/*Property = IsMobile*/
const Value V23946_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23946_5 = {S52, 0, &V23946_5_1403, &P23946_1};	/*Property = ScreenPixelsHeight*/
const Value V23946_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23946_4 = {S54, 0, &V23946_4_1404, &P23946_5};	/*Property = ScreenPixelsWidth*/
const Profile P23946 = {"23946", &P23946_4};	/*Profile ID - 23946*/
const Value V21081_1_1 = {S2, NULL};	/*Value = True*/
const Property P21081_1 = {S50, 0, &V21081_1_1, NULL};	/*Property = IsMobile*/
const Value V21081_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21081_5 = {S52, 0, &V21081_5_1277, &P21081_1};	/*Property = ScreenPixelsHeight*/
const Value V21081_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21081_4 = {S54, 0, &V21081_4_1279, &P21081_5};	/*Property = ScreenPixelsWidth*/
const Profile P21081 = {"21081", &P21081_4};	/*Profile ID - 21081*/
const Value V20226_1_1 = {S2, NULL};	/*Value = True*/
const Property P20226_1 = {S50, 0, &V20226_1_1, NULL};	/*Property = IsMobile*/
const Value V20226_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20226_5 = {S52, 0, &V20226_5_9981, &P20226_1};	/*Property = ScreenPixelsHeight*/
const Value V20226_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20226_4 = {S54, 0, &V20226_4_37617, &P20226_5};	/*Property = ScreenPixelsWidth*/
const Profile P20226 = {"20226", &P20226_4};	/*Profile ID - 20226*/
const Value V23968_1_1 = {S2, NULL};	/*Value = True*/
const Property P23968_1 = {S50, 0, &V23968_1_1, NULL};	/*Property = IsMobile*/
const Value V23968_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23968_5 = {S52, 0, &V23968_5_1227, &P23968_1};	/*Property = ScreenPixelsHeight*/
const Value V23968_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23968_4 = {S54, 0, &V23968_4_1229, &P23968_5};	/*Property = ScreenPixelsWidth*/
const Profile P23968 = {"23968", &P23968_4};	/*Profile ID - 23968*/
const Value V22077_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_279 = {S1, 0, &V22077_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22077_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_282 = {S3, 0, &V22077_282_57155, &P22077_279};	/*Property = BlobBuilder*/
const Value V22077_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_350 = {S4, 0, &V22077_350_57446, &P22077_282};	/*Property = Canvas*/
const Value V22077_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_283 = {S5, 0, &V22077_283_57161, &P22077_350};	/*Property = CssBackground*/
const Value V22077_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_284 = {S6, 0, &V22077_284_57164, &P22077_283};	/*Property = CssBorderImage*/
const Value V22077_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_285 = {S7, 0, &V22077_285_57167, &P22077_284};	/*Property = CssCanvas*/
const Value V22077_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_286 = {S8, 0, &V22077_286_57170, &P22077_285};	/*Property = CssColor*/
const Value V22077_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_237 = {S9, 0, &V22077_237_55786, &P22077_286};	/*Property = CssColumn*/
const Value V22077_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_287 = {S10, 0, &V22077_287_57173, &P22077_237};	/*Property = CssFlexbox*/
const Value V22077_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_288 = {S11, 0, &V22077_288_57176, &P22077_287};	/*Property = CssFont*/
const Value V22077_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_289 = {S12, 0, &V22077_289_86899, &P22077_288};	/*Property = CssImages*/
const Value V22077_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_291 = {S13, 0, &V22077_291_57182, &P22077_289};	/*Property = CssMediaQueries*/
const Value V22077_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_292 = {S14, 0, &V22077_292_57185, &P22077_291};	/*Property = CssMinMax*/
const Value V22077_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_293 = {S15, 0, &V22077_293_57188, &P22077_292};	/*Property = CssOverflow*/
const Value V22077_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_294 = {S16, 0, &V22077_294_57191, &P22077_293};	/*Property = CssPosition*/
const Value V22077_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_295 = {S17, 0, &V22077_295_57194, &P22077_294};	/*Property = CssText*/
const Value V22077_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_297 = {S18, 0, &V22077_297_57197, &P22077_295};	/*Property = CssTransforms*/
const Value V22077_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_241 = {S19, 0, &V22077_241_55801, &P22077_297};	/*Property = CssTransitions*/
const Value V22077_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_298 = {S20, 0, &V22077_298_57208, &P22077_241};	/*Property = CssUI*/
const Value V22077_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_300 = {S21, 0, &V22077_300_57217, &P22077_298};	/*Property = DataSet*/
const Value V22077_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_301 = {S22, 0, &V22077_301_57220, &P22077_300};	/*Property = DataUrl*/
const Value V22077_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_302 = {S23, 0, &V22077_302_57223, &P22077_301};	/*Property = DeviceOrientation*/
const Value V22077_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_304 = {S24, 0, &V22077_304_57229, &P22077_302};	/*Property = FileReader*/
const Value V22077_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_305 = {S25, 0, &V22077_305_57232, &P22077_304};	/*Property = FileSaver*/
const Value V22077_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_306 = {S26, 0, &V22077_306_57238, &P22077_305};	/*Property = FileWriter*/
const Value V22077_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_307 = {S27, 0, &V22077_307_57241, &P22077_306};	/*Property = FormData*/
const Value V22077_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_309 = {S28, 0, &V22077_309_57247, &P22077_307};	/*Property = Fullscreen*/
const Value V22077_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_310 = {S29, 0, &V22077_310_57250, &P22077_309};	/*Property = GeoLocation*/
const Value V22077_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_312 = {S30, 0, &V22077_312_57256, &P22077_310};	/*Property = History*/
const Value V22077_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_314 = {S31, 0, &V22077_314_57262, &P22077_312};	/*Property = Html5*/
const Value V22077_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_313 = {S32, 0, &V22077_313_57259, &P22077_314};	/*Property = Html-Media-Capture*/
const Value V22077_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_315 = {S33, 0, &V22077_315_57265, &P22077_313};	/*Property = Iframe*/
const Value V22077_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_316 = {S34, 0, &V22077_316_57268, &P22077_315};	/*Property = IndexedDB*/
const Value V22077_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_317 = {S35, 0, &V22077_317_57271, &P22077_316};	/*Property = Json*/
const Value V22077_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_15 = {S37, 0, &V22077_15_72, &P22077_317};	/*Property = LayoutEngine*/
const Value V22077_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_344 = {S38, 0, &V22077_344_57274, &P22077_15};	/*Property = Masking*/
const Value V22077_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_320 = {S39, 0, &V22077_320_57292, &P22077_344};	/*Property = PostMessage*/
const Value V22077_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_321 = {S40, 0, &V22077_321_57295, &P22077_320};	/*Property = Progress*/
const Value V22077_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_322 = {S41, 0, &V22077_322_57298, &P22077_321};	/*Property = Prompts*/
const Value V22077_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_323 = {S42, 0, &V22077_323_57301, &P22077_322};	/*Property = Selector*/
const Value V22077_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_324 = {S43, 0, &V22077_324_57307, &P22077_323};	/*Property = Svg*/
const Value V22077_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_327 = {S44, 0, &V22077_327_57316, &P22077_324};	/*Property = TouchEvents*/
const Value V22077_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_328 = {S45, 0, &V22077_328_57319, &P22077_327};	/*Property = Track*/
const Value V22077_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_329 = {S46, 0, &V22077_329_57325, &P22077_328};	/*Property = Video*/
const Value V22077_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_330 = {S47, 0, &V22077_330_57328, &P22077_329};	/*Property = Viewport*/
const Value V22077_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_332 = {S48, 0, &V22077_332_57340, &P22077_330};	/*Property = WebWorkers*/
const Value V22077_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22077_333 = {S49, 0, &V22077_333_57343, &P22077_332};	/*Property = Xhr2*/
const Profile P22077 = {"22077", &P22077_333};	/*Profile ID - 22077*/
const Value V20353_1_1 = {S2, NULL};	/*Value = True*/
const Property P20353_1 = {S50, 0, &V20353_1_1, NULL};	/*Property = IsMobile*/
const Value V20353_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20353_5 = {S52, 0, &V20353_5_1227, &P20353_1};	/*Property = ScreenPixelsHeight*/
const Value V20353_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20353_4 = {S54, 0, &V20353_4_1229, &P20353_5};	/*Property = ScreenPixelsWidth*/
const Profile P20353 = {"20353", &P20353_4};	/*Profile ID - 20353*/
const Value V20838_1_1 = {S2, NULL};	/*Value = True*/
const Property P20838_1 = {S50, 0, &V20838_1_1, NULL};	/*Property = IsMobile*/
const Value V20838_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20838_5 = {S52, 0, &V20838_5_9981, &P20838_1};	/*Property = ScreenPixelsHeight*/
const Value V20838_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20838_4 = {S54, 0, &V20838_4_2336, &P20838_5};	/*Property = ScreenPixelsWidth*/
const Profile P20838 = {"20838", &P20838_4};	/*Profile ID - 20838*/
const Value V20884_1_1 = {S2, NULL};	/*Value = True*/
const Property P20884_1 = {S50, 0, &V20884_1_1, NULL};	/*Property = IsMobile*/
const Value V20884_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20884_5 = {S52, 0, &V20884_5_1227, &P20884_1};	/*Property = ScreenPixelsHeight*/
const Value V20884_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20884_4 = {S54, 0, &V20884_4_1229, &P20884_5};	/*Property = ScreenPixelsWidth*/
const Profile P20884 = {"20884", &P20884_4};	/*Profile ID - 20884*/
const Value V20467_1_1 = {S2, NULL};	/*Value = True*/
const Property P20467_1 = {S50, 0, &V20467_1_1, NULL};	/*Property = IsMobile*/
const Value V20467_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20467_5 = {S52, 0, &V20467_5_1227, &P20467_1};	/*Property = ScreenPixelsHeight*/
const Value V20467_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P20467_4 = {S54, 0, &V20467_4_1943, &P20467_5};	/*Property = ScreenPixelsWidth*/
const Profile P20467 = {"20467", &P20467_4};	/*Profile ID - 20467*/
const Value V21110_1_1 = {S2, NULL};	/*Value = True*/
const Property P21110_1 = {S50, 0, &V21110_1_1, NULL};	/*Property = IsMobile*/
const Value V21110_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21110_5 = {S52, 0, &V21110_5_1403, &P21110_1};	/*Property = ScreenPixelsHeight*/
const Value V21110_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21110_4 = {S54, 0, &V21110_4_1404, &P21110_5};	/*Property = ScreenPixelsWidth*/
const Profile P21110 = {"21110", &P21110_4};	/*Profile ID - 21110*/
const Value V20836_1_1 = {S2, NULL};	/*Value = True*/
const Property P20836_1 = {S50, 0, &V20836_1_1, NULL};	/*Property = IsMobile*/
const Value V20836_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20836_5 = {S52, 0, &V20836_5_1277, &P20836_1};	/*Property = ScreenPixelsHeight*/
const Value V20836_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20836_4 = {S54, 0, &V20836_4_1279, &P20836_5};	/*Property = ScreenPixelsWidth*/
const Profile P20836 = {"20836", &P20836_4};	/*Profile ID - 20836*/
const Value V20893_1_1 = {S2, NULL};	/*Value = True*/
const Property P20893_1 = {S50, 0, &V20893_1_1, NULL};	/*Property = IsMobile*/
const Value V20893_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20893_5 = {S52, 0, &V20893_5_1458, &P20893_1};	/*Property = ScreenPixelsHeight*/
const Value V20893_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20893_4 = {S54, 0, &V20893_4_1460, &P20893_5};	/*Property = ScreenPixelsWidth*/
const Profile P20893 = {"20893", &P20893_4};	/*Profile ID - 20893*/
const Value V21796_1_1 = {S2, NULL};	/*Value = True*/
const Property P21796_1 = {S50, 0, &V21796_1_1, NULL};	/*Property = IsMobile*/
const Value V21796_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21796_5 = {S52, 0, &V21796_5_1277, &P21796_1};	/*Property = ScreenPixelsHeight*/
const Value V21796_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21796_4 = {S54, 0, &V21796_4_1279, &P21796_5};	/*Property = ScreenPixelsWidth*/
const Profile P21796 = {"21796", &P21796_4};	/*Profile ID - 21796*/
const Value V20519_1_1 = {S2, NULL};	/*Value = True*/
const Property P20519_1 = {S50, 0, &V20519_1_1, NULL};	/*Property = IsMobile*/
const Value V20519_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20519_5 = {S52, 0, &V20519_5_1227, &P20519_1};	/*Property = ScreenPixelsHeight*/
const Value V20519_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20519_4 = {S54, 0, &V20519_4_1229, &P20519_5};	/*Property = ScreenPixelsWidth*/
const Profile P20519 = {"20519", &P20519_4};	/*Profile ID - 20519*/
const Value V22145_1_1 = {S2, NULL};	/*Value = True*/
const Property P22145_1 = {S50, 0, &V22145_1_1, NULL};	/*Property = IsMobile*/
const Value V22145_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22145_5 = {S52, 0, &V22145_5_1227, &P22145_1};	/*Property = ScreenPixelsHeight*/
const Value V22145_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22145_4 = {S54, 0, &V22145_4_1229, &P22145_5};	/*Property = ScreenPixelsWidth*/
const Profile P22145 = {"22145", &P22145_4};	/*Profile ID - 22145*/
const Value V21274_1_1 = {S2, NULL};	/*Value = True*/
const Property P21274_1 = {S50, 0, &V21274_1_1, NULL};	/*Property = IsMobile*/
const Value V21274_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21274_5 = {S52, 0, &V21274_5_1277, &P21274_1};	/*Property = ScreenPixelsHeight*/
const Value V21274_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21274_4 = {S54, 0, &V21274_4_1279, &P21274_5};	/*Property = ScreenPixelsWidth*/
const Profile P21274 = {"21274", &P21274_4};	/*Profile ID - 21274*/
const Value V20221_1_1 = {S2, NULL};	/*Value = True*/
const Property P20221_1 = {S50, 0, &V20221_1_1, NULL};	/*Property = IsMobile*/
const Value V20221_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20221_5 = {S52, 0, &V20221_5_1403, &P20221_1};	/*Property = ScreenPixelsHeight*/
const Value V20221_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20221_4 = {S54, 0, &V20221_4_1404, &P20221_5};	/*Property = ScreenPixelsWidth*/
const Profile P20221 = {"20221", &P20221_4};	/*Profile ID - 20221*/
const Value V21726_1_1 = {S2, NULL};	/*Value = True*/
const Property P21726_1 = {S50, 0, &V21726_1_1, NULL};	/*Property = IsMobile*/
const Value V21726_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21726_5 = {S52, 0, &V21726_5_1227, &P21726_1};	/*Property = ScreenPixelsHeight*/
const Value V21726_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21726_4 = {S54, 0, &V21726_4_1229, &P21726_5};	/*Property = ScreenPixelsWidth*/
const Profile P21726 = {"21726", &P21726_4};	/*Profile ID - 21726*/
const Value V21295_1_1 = {S2, NULL};	/*Value = True*/
const Property P21295_1 = {S50, 0, &V21295_1_1, NULL};	/*Property = IsMobile*/
const Value V21295_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21295_5 = {S52, 0, &V21295_5_1227, &P21295_1};	/*Property = ScreenPixelsHeight*/
const Value V21295_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P21295_4 = {S54, 0, &V21295_4_1477, &P21295_5};	/*Property = ScreenPixelsWidth*/
const Profile P21295 = {"21295", &P21295_4};	/*Profile ID - 21295*/
const Value V22075_1_1 = {S2, NULL};	/*Value = True*/
const Property P22075_1 = {S50, 0, &V22075_1_1, NULL};	/*Property = IsMobile*/
const Value V22075_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22075_5 = {S52, 0, &V22075_5_1458, &P22075_1};	/*Property = ScreenPixelsHeight*/
const Value V22075_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P22075_4 = {S54, 0, &V22075_4_37617, &P22075_5};	/*Property = ScreenPixelsWidth*/
const Profile P22075 = {"22075", &P22075_4};	/*Profile ID - 22075*/
const Value V23313_1_1 = {S2, NULL};	/*Value = True*/
const Property P23313_1 = {S50, 0, &V23313_1_1, NULL};	/*Property = IsMobile*/
const Value V23313_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23313_5 = {S52, 0, &V23313_5_1227, &P23313_1};	/*Property = ScreenPixelsHeight*/
const Value V23313_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P23313_4 = {S54, 0, &V23313_4_1477, &P23313_5};	/*Property = ScreenPixelsWidth*/
const Profile P23313 = {"23313", &P23313_4};	/*Profile ID - 23313*/
const Value V23804_1_1 = {S2, NULL};	/*Value = True*/
const Property P23804_1 = {S50, 0, &V23804_1_1, NULL};	/*Property = IsMobile*/
const Value V23804_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23804_5 = {S52, 0, &V23804_5_9981, &P23804_1};	/*Property = ScreenPixelsHeight*/
const Value V23804_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23804_4 = {S54, 0, &V23804_4_2336, &P23804_5};	/*Property = ScreenPixelsWidth*/
const Profile P23804 = {"23804", &P23804_4};	/*Profile ID - 23804*/
const Value V21378_1_1 = {S2, NULL};	/*Value = True*/
const Property P21378_1 = {S50, 0, &V21378_1_1, NULL};	/*Property = IsMobile*/
const Value V21378_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21378_5 = {S52, 0, &V21378_5_1458, &P21378_1};	/*Property = ScreenPixelsHeight*/
const Value V21378_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21378_4 = {S54, 0, &V21378_4_37617, &P21378_5};	/*Property = ScreenPixelsWidth*/
const Profile P21378 = {"21378", &P21378_4};	/*Profile ID - 21378*/
const Value V23876_1_1 = {S2, NULL};	/*Value = True*/
const Property P23876_1 = {S50, 0, &V23876_1_1, NULL};	/*Property = IsMobile*/
const Value V23876_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23876_5 = {S52, 0, &V23876_5_1227, &P23876_1};	/*Property = ScreenPixelsHeight*/
const Value V23876_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23876_4 = {S54, 0, &V23876_4_1229, &P23876_5};	/*Property = ScreenPixelsWidth*/
const Profile P23876 = {"23876", &P23876_4};	/*Profile ID - 23876*/
const Value V20526_1_1 = {S2, NULL};	/*Value = True*/
const Property P20526_1 = {S50, 0, &V20526_1_1, NULL};	/*Property = IsMobile*/
const Value V20526_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20526_5 = {S52, 0, &V20526_5_1458, &P20526_1};	/*Property = ScreenPixelsHeight*/
const Value V20526_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20526_4 = {S54, 0, &V20526_4_37617, &P20526_5};	/*Property = ScreenPixelsWidth*/
const Profile P20526 = {"20526", &P20526_4};	/*Profile ID - 20526*/
const Value V22707_1_1 = {S2, NULL};	/*Value = True*/
const Property P22707_1 = {S50, 0, &V22707_1_1, NULL};	/*Property = IsMobile*/
const Value V22707_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22707_5 = {S52, 0, &V22707_5_9981, &P22707_1};	/*Property = ScreenPixelsHeight*/
const Value V22707_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22707_4 = {S54, 0, &V22707_4_2336, &P22707_5};	/*Property = ScreenPixelsWidth*/
const Profile P22707 = {"22707", &P22707_4};	/*Profile ID - 22707*/
const Value V23776_1_1 = {S2, NULL};	/*Value = True*/
const Property P23776_1 = {S50, 0, &V23776_1_1, NULL};	/*Property = IsMobile*/
const Value V23776_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P23776_5 = {S52, 0, &V23776_5_59138, &P23776_1};	/*Property = ScreenPixelsHeight*/
const Value V23776_4_52971 = {S122, NULL};	/*Value = 1200*/
const Property P23776_4 = {S54, 0, &V23776_4_52971, &P23776_5};	/*Property = ScreenPixelsWidth*/
const Profile P23776 = {"23776", &P23776_4};	/*Profile ID - 23776*/
const Value V21262_1_1 = {S2, NULL};	/*Value = True*/
const Property P21262_1 = {S50, 0, &V21262_1_1, NULL};	/*Property = IsMobile*/
const Value V21262_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21262_5 = {S52, 0, &V21262_5_1227, &P21262_1};	/*Property = ScreenPixelsHeight*/
const Value V21262_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21262_4 = {S54, 0, &V21262_4_1229, &P21262_5};	/*Property = ScreenPixelsWidth*/
const Profile P21262 = {"21262", &P21262_4};	/*Profile ID - 21262*/
const Value V20891_1_1 = {S2, NULL};	/*Value = True*/
const Property P20891_1 = {S50, 0, &V20891_1_1, NULL};	/*Property = IsMobile*/
const Value V20891_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20891_5 = {S52, 0, &V20891_5_1227, &P20891_1};	/*Property = ScreenPixelsHeight*/
const Value V20891_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20891_4 = {S54, 0, &V20891_4_1229, &P20891_5};	/*Property = ScreenPixelsWidth*/
const Profile P20891 = {"20891", &P20891_4};	/*Profile ID - 20891*/
const Value V21852_1_1 = {S2, NULL};	/*Value = True*/
const Property P21852_1 = {S50, 0, &V21852_1_1, NULL};	/*Property = IsMobile*/
const Value V21852_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21852_5 = {S52, 0, &V21852_5_1403, &P21852_1};	/*Property = ScreenPixelsHeight*/
const Value V21852_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21852_4 = {S54, 0, &V21852_4_1404, &P21852_5};	/*Property = ScreenPixelsWidth*/
const Profile P21852 = {"21852", &P21852_4};	/*Profile ID - 21852*/
const Value V20768_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20768_279 = {S1, 0, &V20768_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20768_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20768_282 = {S3, 0, &V20768_282_57154, &P20768_279};	/*Property = BlobBuilder*/
const Value V20768_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20768_350 = {S4, 0, &V20768_350_57444, &P20768_282};	/*Property = Canvas*/
const Value V20768_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20768_283 = {S5, 0, &V20768_283_57159, &P20768_350};	/*Property = CssBackground*/
const Value V20768_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20768_284 = {S6, 0, &V20768_284_57163, &P20768_283};	/*Property = CssBorderImage*/
const Value V20768_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20768_285 = {S7, 0, &V20768_285_57166, &P20768_284};	/*Property = CssCanvas*/
const Value V20768_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20768_286 = {S8, 0, &V20768_286_57168, &P20768_285};	/*Property = CssColor*/
const Value V20768_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20768_237 = {S9, 0, &V20768_237_55786, &P20768_286};	/*Property = CssColumn*/
const Value V20768_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20768_287 = {S10, 0, &V20768_287_57172, &P20768_237};	/*Property = CssFlexbox*/
const Value V20768_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20768_288 = {S11, 0, &V20768_288_57176, &P20768_287};	/*Property = CssFont*/
const Value V20768_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20768_289 = {S12, 0, &V20768_289_57759, &P20768_288};	/*Property = CssImages*/
const Value V20768_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20768_291 = {S13, 0, &V20768_291_57182, &P20768_289};	/*Property = CssMediaQueries*/
const Value V20768_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20768_292 = {S14, 0, &V20768_292_57183, &P20768_291};	/*Property = CssMinMax*/
const Value V20768_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20768_293 = {S15, 0, &V20768_293_57188, &P20768_292};	/*Property = CssOverflow*/
const Value V20768_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20768_294 = {S16, 0, &V20768_294_57191, &P20768_293};	/*Property = CssPosition*/
const Value V20768_295_57192 = {S2, NULL};	/*Value = True*/
const Property P20768_295 = {S17, 0, &V20768_295_57192, &P20768_294};	/*Property = CssText*/
const Value V20768_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20768_297 = {S18, 0, &V20768_297_57195, &P20768_295};	/*Property = CssTransforms*/
const Value V20768_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20768_241 = {S19, 0, &V20768_241_55799, &P20768_297};	/*Property = CssTransitions*/
const Value V20768_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20768_298 = {S20, 0, &V20768_298_57206, &P20768_241};	/*Property = CssUI*/
const Value V20768_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20768_300 = {S21, 0, &V20768_300_57216, &P20768_298};	/*Property = DataSet*/
const Value V20768_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20768_301 = {S22, 0, &V20768_301_57218, &P20768_300};	/*Property = DataUrl*/
const Value V20768_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20768_302 = {S23, 0, &V20768_302_57223, &P20768_301};	/*Property = DeviceOrientation*/
const Value V20768_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20768_304 = {S24, 0, &V20768_304_57229, &P20768_302};	/*Property = FileReader*/
const Value V20768_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20768_305 = {S25, 0, &V20768_305_57231, &P20768_304};	/*Property = FileSaver*/
const Value V20768_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20768_306 = {S26, 0, &V20768_306_57237, &P20768_305};	/*Property = FileWriter*/
const Value V20768_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20768_307 = {S27, 0, &V20768_307_57241, &P20768_306};	/*Property = FormData*/
const Value V20768_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20768_309 = {S28, 0, &V20768_309_57246, &P20768_307};	/*Property = Fullscreen*/
const Value V20768_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20768_310 = {S29, 0, &V20768_310_57248, &P20768_309};	/*Property = GeoLocation*/
const Value V20768_312_57255 = {S0, NULL};	/*Value = False*/
const Property P20768_312 = {S30, 0, &V20768_312_57255, &P20768_310};	/*Property = History*/
const Value V20768_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20768_314 = {S31, 0, &V20768_314_57261, &P20768_312};	/*Property = Html5*/
const Value V20768_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20768_313 = {S32, 0, &V20768_313_57258, &P20768_314};	/*Property = Html-Media-Capture*/
const Value V20768_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20768_315 = {S33, 0, &V20768_315_57264, &P20768_313};	/*Property = Iframe*/
const Value V20768_316_57267 = {S0, NULL};	/*Value = False*/
const Property P20768_316 = {S34, 0, &V20768_316_57267, &P20768_315};	/*Property = IndexedDB*/
const Value V20768_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20768_317 = {S35, 0, &V20768_317_57269, &P20768_316};	/*Property = Json*/
const Value V20768_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P20768_15 = {S37, 0, &V20768_15_28, &P20768_317};	/*Property = LayoutEngine*/
const Value V20768_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20768_344 = {S38, 0, &V20768_344_57272, &P20768_15};	/*Property = Masking*/
const Value V20768_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20768_320 = {S39, 0, &V20768_320_57290, &P20768_344};	/*Property = PostMessage*/
const Value V20768_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20768_321 = {S40, 0, &V20768_321_57293, &P20768_320};	/*Property = Progress*/
const Value V20768_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20768_322 = {S41, 0, &V20768_322_57296, &P20768_321};	/*Property = Prompts*/
const Value V20768_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20768_323 = {S42, 0, &V20768_323_57299, &P20768_322};	/*Property = Selector*/
const Value V20768_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20768_324 = {S43, 0, &V20768_324_57306, &P20768_323};	/*Property = Svg*/
const Value V20768_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20768_327 = {S44, 0, &V20768_327_57316, &P20768_324};	/*Property = TouchEvents*/
const Value V20768_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20768_328 = {S45, 0, &V20768_328_57318, &P20768_327};	/*Property = Track*/
const Value V20768_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20768_329 = {S46, 0, &V20768_329_57323, &P20768_328};	/*Property = Video*/
const Value V20768_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20768_330 = {S47, 0, &V20768_330_57326, &P20768_329};	/*Property = Viewport*/
const Value V20768_332_57339 = {S0, NULL};	/*Value = False*/
const Property P20768_332 = {S48, 0, &V20768_332_57339, &P20768_330};	/*Property = WebWorkers*/
const Value V20768_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20768_333 = {S49, 0, &V20768_333_57342, &P20768_332};	/*Property = Xhr2*/
const Profile P20768 = {"20768", &P20768_333};	/*Profile ID - 20768*/
const Value V20192_1_1 = {S2, NULL};	/*Value = True*/
const Property P20192_1 = {S50, 0, &V20192_1_1, NULL};	/*Property = IsMobile*/
const Value V20192_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P20192_5 = {S52, 0, &V20192_5_1269, &P20192_1};	/*Property = ScreenPixelsHeight*/
const Value V20192_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20192_4 = {S54, 0, &V20192_4_1229, &P20192_5};	/*Property = ScreenPixelsWidth*/
const Profile P20192 = {"20192", &P20192_4};	/*Profile ID - 20192*/
const Value V22767_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22767_279 = {S1, 0, &V22767_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22767_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22767_282 = {S3, 0, &V22767_282_57154, &P22767_279};	/*Property = BlobBuilder*/
const Value V22767_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22767_350 = {S4, 0, &V22767_350_57444, &P22767_282};	/*Property = Canvas*/
const Value V22767_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22767_283 = {S5, 0, &V22767_283_57159, &P22767_350};	/*Property = CssBackground*/
const Value V22767_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22767_284 = {S6, 0, &V22767_284_57163, &P22767_283};	/*Property = CssBorderImage*/
const Value V22767_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22767_285 = {S7, 0, &V22767_285_57166, &P22767_284};	/*Property = CssCanvas*/
const Value V22767_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22767_286 = {S8, 0, &V22767_286_57168, &P22767_285};	/*Property = CssColor*/
const Value V22767_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22767_237 = {S9, 0, &V22767_237_55786, &P22767_286};	/*Property = CssColumn*/
const Value V22767_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22767_287 = {S10, 0, &V22767_287_57172, &P22767_237};	/*Property = CssFlexbox*/
const Value V22767_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22767_288 = {S11, 0, &V22767_288_57174, &P22767_287};	/*Property = CssFont*/
const Value V22767_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22767_289 = {S12, 0, &V22767_289_57759, &P22767_288};	/*Property = CssImages*/
const Value V22767_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22767_291 = {S13, 0, &V22767_291_57181, &P22767_289};	/*Property = CssMediaQueries*/
const Value V22767_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22767_292 = {S14, 0, &V22767_292_57183, &P22767_291};	/*Property = CssMinMax*/
const Value V22767_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22767_293 = {S15, 0, &V22767_293_57187, &P22767_292};	/*Property = CssOverflow*/
const Value V22767_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22767_294 = {S16, 0, &V22767_294_57189, &P22767_293};	/*Property = CssPosition*/
const Value V22767_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22767_295 = {S17, 0, &V22767_295_57193, &P22767_294};	/*Property = CssText*/
const Value V22767_297_57196 = {S0, NULL};	/*Value = False*/
const Property P22767_297 = {S18, 0, &V22767_297_57196, &P22767_295};	/*Property = CssTransforms*/
const Value V22767_241_55800 = {S0, NULL};	/*Value = False*/
const Property P22767_241 = {S19, 0, &V22767_241_55800, &P22767_297};	/*Property = CssTransitions*/
const Value V22767_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22767_298 = {S20, 0, &V22767_298_57206, &P22767_241};	/*Property = CssUI*/
const Value V22767_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22767_300 = {S21, 0, &V22767_300_57216, &P22767_298};	/*Property = DataSet*/
const Value V22767_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22767_301 = {S22, 0, &V22767_301_57218, &P22767_300};	/*Property = DataUrl*/
const Value V22767_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22767_302 = {S23, 0, &V22767_302_57222, &P22767_301};	/*Property = DeviceOrientation*/
const Value V22767_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22767_304 = {S24, 0, &V22767_304_57228, &P22767_302};	/*Property = FileReader*/
const Value V22767_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22767_305 = {S25, 0, &V22767_305_57231, &P22767_304};	/*Property = FileSaver*/
const Value V22767_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22767_306 = {S26, 0, &V22767_306_57237, &P22767_305};	/*Property = FileWriter*/
const Value V22767_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22767_307 = {S27, 0, &V22767_307_57240, &P22767_306};	/*Property = FormData*/
const Value V22767_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22767_309 = {S28, 0, &V22767_309_57246, &P22767_307};	/*Property = Fullscreen*/
const Value V22767_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22767_310 = {S29, 0, &V22767_310_57248, &P22767_309};	/*Property = GeoLocation*/
const Value V22767_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22767_312 = {S30, 0, &V22767_312_57255, &P22767_310};	/*Property = History*/
const Value V22767_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22767_314 = {S31, 0, &V22767_314_57261, &P22767_312};	/*Property = Html5*/
const Value V22767_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22767_313 = {S32, 0, &V22767_313_57258, &P22767_314};	/*Property = Html-Media-Capture*/
const Value V22767_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22767_315 = {S33, 0, &V22767_315_57264, &P22767_313};	/*Property = Iframe*/
const Value V22767_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22767_316 = {S34, 0, &V22767_316_57267, &P22767_315};	/*Property = IndexedDB*/
const Value V22767_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22767_317 = {S35, 0, &V22767_317_57269, &P22767_316};	/*Property = Json*/
const Value V22767_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P22767_15 = {S37, 0, &V22767_15_30, &P22767_317};	/*Property = LayoutEngine*/
const Value V22767_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22767_344 = {S38, 0, &V22767_344_57272, &P22767_15};	/*Property = Masking*/
const Value V22767_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22767_320 = {S39, 0, &V22767_320_57290, &P22767_344};	/*Property = PostMessage*/
const Value V22767_321_57294 = {S0, NULL};	/*Value = False*/
const Property P22767_321 = {S40, 0, &V22767_321_57294, &P22767_320};	/*Property = Progress*/
const Value V22767_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22767_322 = {S41, 0, &V22767_322_57296, &P22767_321};	/*Property = Prompts*/
const Value V22767_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22767_323 = {S42, 0, &V22767_323_57299, &P22767_322};	/*Property = Selector*/
const Value V22767_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22767_324 = {S43, 0, &V22767_324_57305, &P22767_323};	/*Property = Svg*/
const Value V22767_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22767_327 = {S44, 0, &V22767_327_57315, &P22767_324};	/*Property = TouchEvents*/
const Value V22767_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22767_328 = {S45, 0, &V22767_328_57318, &P22767_327};	/*Property = Track*/
const Value V22767_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22767_329 = {S46, 0, &V22767_329_57323, &P22767_328};	/*Property = Video*/
const Value V22767_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22767_330 = {S47, 0, &V22767_330_57326, &P22767_329};	/*Property = Viewport*/
const Value V22767_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22767_332 = {S48, 0, &V22767_332_57339, &P22767_330};	/*Property = WebWorkers*/
const Value V22767_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22767_333 = {S49, 0, &V22767_333_57342, &P22767_332};	/*Property = Xhr2*/
const Profile P22767 = {"22767", &P22767_333};	/*Profile ID - 22767*/
const Value V23317_1_2 = {S0, NULL};	/*Value = False*/
const Property P23317_1 = {S50, 0, &V23317_1_2, NULL};	/*Property = IsMobile*/
const Value V23317_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23317_5 = {S52, 0, &V23317_5_37, &P23317_1};	/*Property = ScreenPixelsHeight*/
const Value V23317_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23317_4 = {S54, 0, &V23317_4_36, &P23317_5};	/*Property = ScreenPixelsWidth*/
const Profile P23317 = {"23317", &P23317_4};	/*Profile ID - 23317*/
const Value V19838_279_57147 = {S2, NULL};	/*Value = True*/
const Property P19838_279 = {S1, 0, &V19838_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V19838_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19838_282 = {S3, 0, &V19838_282_57154, &P19838_279};	/*Property = BlobBuilder*/
const Value V19838_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19838_350 = {S4, 0, &V19838_350_57444, &P19838_282};	/*Property = Canvas*/
const Value V19838_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19838_283 = {S5, 0, &V19838_283_57159, &P19838_350};	/*Property = CssBackground*/
const Value V19838_284_57162 = {S2, NULL};	/*Value = True*/
const Property P19838_284 = {S6, 0, &V19838_284_57162, &P19838_283};	/*Property = CssBorderImage*/
const Value V19838_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19838_285 = {S7, 0, &V19838_285_57166, &P19838_284};	/*Property = CssCanvas*/
const Value V19838_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19838_286 = {S8, 0, &V19838_286_57168, &P19838_285};	/*Property = CssColor*/
const Value V19838_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19838_237 = {S9, 0, &V19838_237_55786, &P19838_286};	/*Property = CssColumn*/
const Value V19838_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19838_287 = {S10, 0, &V19838_287_57172, &P19838_237};	/*Property = CssFlexbox*/
const Value V19838_288_57174 = {S2, NULL};	/*Value = True*/
const Property P19838_288 = {S11, 0, &V19838_288_57174, &P19838_287};	/*Property = CssFont*/
const Value V19838_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19838_289 = {S12, 0, &V19838_289_57759, &P19838_288};	/*Property = CssImages*/
const Value V19838_291_57180 = {S2, NULL};	/*Value = True*/
const Property P19838_291 = {S13, 0, &V19838_291_57180, &P19838_289};	/*Property = CssMediaQueries*/
const Value V19838_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19838_292 = {S14, 0, &V19838_292_57183, &P19838_291};	/*Property = CssMinMax*/
const Value V19838_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19838_293 = {S15, 0, &V19838_293_57187, &P19838_292};	/*Property = CssOverflow*/
const Value V19838_294_57189 = {S2, NULL};	/*Value = True*/
const Property P19838_294 = {S16, 0, &V19838_294_57189, &P19838_293};	/*Property = CssPosition*/
const Value V19838_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19838_295 = {S17, 0, &V19838_295_57192, &P19838_294};	/*Property = CssText*/
const Value V19838_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19838_297 = {S18, 0, &V19838_297_57195, &P19838_295};	/*Property = CssTransforms*/
const Value V19838_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19838_241 = {S19, 0, &V19838_241_55799, &P19838_297};	/*Property = CssTransitions*/
const Value V19838_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19838_298 = {S20, 0, &V19838_298_57206, &P19838_241};	/*Property = CssUI*/
const Value V19838_300_57215 = {S2, NULL};	/*Value = True*/
const Property P19838_300 = {S21, 0, &V19838_300_57215, &P19838_298};	/*Property = DataSet*/
const Value V19838_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19838_301 = {S22, 0, &V19838_301_57218, &P19838_300};	/*Property = DataUrl*/
const Value V19838_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19838_302 = {S23, 0, &V19838_302_57222, &P19838_301};	/*Property = DeviceOrientation*/
const Value V19838_304_57227 = {S2, NULL};	/*Value = True*/
const Property P19838_304 = {S24, 0, &V19838_304_57227, &P19838_302};	/*Property = FileReader*/
const Value V19838_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19838_305 = {S25, 0, &V19838_305_57231, &P19838_304};	/*Property = FileSaver*/
const Value V19838_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19838_306 = {S26, 0, &V19838_306_57237, &P19838_305};	/*Property = FileWriter*/
const Value V19838_307_57239 = {S2, NULL};	/*Value = True*/
const Property P19838_307 = {S27, 0, &V19838_307_57239, &P19838_306};	/*Property = FormData*/
const Value V19838_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19838_309 = {S28, 0, &V19838_309_57246, &P19838_307};	/*Property = Fullscreen*/
const Value V19838_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19838_310 = {S29, 0, &V19838_310_57248, &P19838_309};	/*Property = GeoLocation*/
const Value V19838_312_57254 = {S2, NULL};	/*Value = True*/
const Property P19838_312 = {S30, 0, &V19838_312_57254, &P19838_310};	/*Property = History*/
const Value V19838_314_57260 = {S2, NULL};	/*Value = True*/
const Property P19838_314 = {S31, 0, &V19838_314_57260, &P19838_312};	/*Property = Html5*/
const Value V19838_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19838_313 = {S32, 0, &V19838_313_57258, &P19838_314};	/*Property = Html-Media-Capture*/
const Value V19838_315_57263 = {S2, NULL};	/*Value = True*/
const Property P19838_315 = {S33, 0, &V19838_315_57263, &P19838_313};	/*Property = Iframe*/
const Value V19838_316_57266 = {S2, NULL};	/*Value = True*/
const Property P19838_316 = {S34, 0, &V19838_316_57266, &P19838_315};	/*Property = IndexedDB*/
const Value V19838_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19838_317 = {S35, 0, &V19838_317_57269, &P19838_316};	/*Property = Json*/
const Value V19838_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19838_15 = {S37, 0, &V19838_15_28, &P19838_317};	/*Property = LayoutEngine*/
const Value V19838_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19838_344 = {S38, 0, &V19838_344_57272, &P19838_15};	/*Property = Masking*/
const Value V19838_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19838_320 = {S39, 0, &V19838_320_57290, &P19838_344};	/*Property = PostMessage*/
const Value V19838_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19838_321 = {S40, 0, &V19838_321_57293, &P19838_320};	/*Property = Progress*/
const Value V19838_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19838_322 = {S41, 0, &V19838_322_57296, &P19838_321};	/*Property = Prompts*/
const Value V19838_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19838_323 = {S42, 0, &V19838_323_57299, &P19838_322};	/*Property = Selector*/
const Value V19838_324_57305 = {S2, NULL};	/*Value = True*/
const Property P19838_324 = {S43, 0, &V19838_324_57305, &P19838_323};	/*Property = Svg*/
const Value V19838_327_57314 = {S2, NULL};	/*Value = True*/
const Property P19838_327 = {S44, 0, &V19838_327_57314, &P19838_324};	/*Property = TouchEvents*/
const Value V19838_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19838_328 = {S45, 0, &V19838_328_57318, &P19838_327};	/*Property = Track*/
const Value V19838_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19838_329 = {S46, 0, &V19838_329_57323, &P19838_328};	/*Property = Video*/
const Value V19838_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19838_330 = {S47, 0, &V19838_330_57326, &P19838_329};	/*Property = Viewport*/
const Value V19838_332_57339 = {S0, NULL};	/*Value = False*/
const Property P19838_332 = {S48, 0, &V19838_332_57339, &P19838_330};	/*Property = WebWorkers*/
const Value V19838_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19838_333 = {S49, 0, &V19838_333_57342, &P19838_332};	/*Property = Xhr2*/
const Profile P19838 = {"19838", &P19838_333};	/*Profile ID - 19838*/
const Value V22121_1_1 = {S2, NULL};	/*Value = True*/
const Property P22121_1 = {S50, 0, &V22121_1_1, NULL};	/*Property = IsMobile*/
const Value V22121_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22121_5 = {S52, 0, &V22121_5_37569, &P22121_1};	/*Property = ScreenPixelsHeight*/
const Value V22121_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22121_4 = {S54, 0, &V22121_4_5645, &P22121_5};	/*Property = ScreenPixelsWidth*/
const Profile P22121 = {"22121", &P22121_4};	/*Profile ID - 22121*/
const Value V20407_1_1 = {S2, NULL};	/*Value = True*/
const Property P20407_1 = {S50, 0, &V20407_1_1, NULL};	/*Property = IsMobile*/
const Value V20407_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20407_5 = {S52, 0, &V20407_5_1277, &P20407_1};	/*Property = ScreenPixelsHeight*/
const Value V20407_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20407_4 = {S54, 0, &V20407_4_1279, &P20407_5};	/*Property = ScreenPixelsWidth*/
const Profile P20407 = {"20407", &P20407_4};	/*Profile ID - 20407*/
const Value V21889_1_1 = {S2, NULL};	/*Value = True*/
const Property P21889_1 = {S50, 0, &V21889_1_1, NULL};	/*Property = IsMobile*/
const Value V21889_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21889_5 = {S52, 0, &V21889_5_1227, &P21889_1};	/*Property = ScreenPixelsHeight*/
const Value V21889_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21889_4 = {S54, 0, &V21889_4_1229, &P21889_5};	/*Property = ScreenPixelsWidth*/
const Profile P21889 = {"21889", &P21889_4};	/*Profile ID - 21889*/
const Value V20509_1_1 = {S2, NULL};	/*Value = True*/
const Property P20509_1 = {S50, 0, &V20509_1_1, NULL};	/*Property = IsMobile*/
const Value V20509_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20509_5 = {S52, 0, &V20509_5_1403, &P20509_1};	/*Property = ScreenPixelsHeight*/
const Value V20509_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20509_4 = {S54, 0, &V20509_4_1404, &P20509_5};	/*Property = ScreenPixelsWidth*/
const Profile P20509 = {"20509", &P20509_4};	/*Profile ID - 20509*/
const Value V20005_1_1 = {S2, NULL};	/*Value = True*/
const Property P20005_1 = {S50, 0, &V20005_1_1, NULL};	/*Property = IsMobile*/
const Value V20005_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20005_5 = {S52, 0, &V20005_5_1227, &P20005_1};	/*Property = ScreenPixelsHeight*/
const Value V20005_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20005_4 = {S54, 0, &V20005_4_1229, &P20005_5};	/*Property = ScreenPixelsWidth*/
const Profile P20005 = {"20005", &P20005_4};	/*Profile ID - 20005*/
const Value V19981_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_279 = {S1, 0, &V19981_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V19981_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_282 = {S3, 0, &V19981_282_57155, &P19981_279};	/*Property = BlobBuilder*/
const Value V19981_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_350 = {S4, 0, &V19981_350_57446, &P19981_282};	/*Property = Canvas*/
const Value V19981_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_283 = {S5, 0, &V19981_283_57161, &P19981_350};	/*Property = CssBackground*/
const Value V19981_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_284 = {S6, 0, &V19981_284_57164, &P19981_283};	/*Property = CssBorderImage*/
const Value V19981_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_285 = {S7, 0, &V19981_285_57167, &P19981_284};	/*Property = CssCanvas*/
const Value V19981_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_286 = {S8, 0, &V19981_286_57170, &P19981_285};	/*Property = CssColor*/
const Value V19981_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_237 = {S9, 0, &V19981_237_55786, &P19981_286};	/*Property = CssColumn*/
const Value V19981_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_287 = {S10, 0, &V19981_287_57173, &P19981_237};	/*Property = CssFlexbox*/
const Value V19981_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_288 = {S11, 0, &V19981_288_57176, &P19981_287};	/*Property = CssFont*/
const Value V19981_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_289 = {S12, 0, &V19981_289_86899, &P19981_288};	/*Property = CssImages*/
const Value V19981_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_291 = {S13, 0, &V19981_291_57182, &P19981_289};	/*Property = CssMediaQueries*/
const Value V19981_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_292 = {S14, 0, &V19981_292_57185, &P19981_291};	/*Property = CssMinMax*/
const Value V19981_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_293 = {S15, 0, &V19981_293_57188, &P19981_292};	/*Property = CssOverflow*/
const Value V19981_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_294 = {S16, 0, &V19981_294_57191, &P19981_293};	/*Property = CssPosition*/
const Value V19981_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_295 = {S17, 0, &V19981_295_57194, &P19981_294};	/*Property = CssText*/
const Value V19981_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_297 = {S18, 0, &V19981_297_57197, &P19981_295};	/*Property = CssTransforms*/
const Value V19981_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_241 = {S19, 0, &V19981_241_55801, &P19981_297};	/*Property = CssTransitions*/
const Value V19981_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_298 = {S20, 0, &V19981_298_57208, &P19981_241};	/*Property = CssUI*/
const Value V19981_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_300 = {S21, 0, &V19981_300_57217, &P19981_298};	/*Property = DataSet*/
const Value V19981_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_301 = {S22, 0, &V19981_301_57220, &P19981_300};	/*Property = DataUrl*/
const Value V19981_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_302 = {S23, 0, &V19981_302_57223, &P19981_301};	/*Property = DeviceOrientation*/
const Value V19981_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_304 = {S24, 0, &V19981_304_57229, &P19981_302};	/*Property = FileReader*/
const Value V19981_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_305 = {S25, 0, &V19981_305_57232, &P19981_304};	/*Property = FileSaver*/
const Value V19981_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_306 = {S26, 0, &V19981_306_57238, &P19981_305};	/*Property = FileWriter*/
const Value V19981_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_307 = {S27, 0, &V19981_307_57241, &P19981_306};	/*Property = FormData*/
const Value V19981_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_309 = {S28, 0, &V19981_309_57247, &P19981_307};	/*Property = Fullscreen*/
const Value V19981_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_310 = {S29, 0, &V19981_310_57250, &P19981_309};	/*Property = GeoLocation*/
const Value V19981_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_312 = {S30, 0, &V19981_312_57256, &P19981_310};	/*Property = History*/
const Value V19981_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_314 = {S31, 0, &V19981_314_57262, &P19981_312};	/*Property = Html5*/
const Value V19981_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_313 = {S32, 0, &V19981_313_57259, &P19981_314};	/*Property = Html-Media-Capture*/
const Value V19981_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_315 = {S33, 0, &V19981_315_57265, &P19981_313};	/*Property = Iframe*/
const Value V19981_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_316 = {S34, 0, &V19981_316_57268, &P19981_315};	/*Property = IndexedDB*/
const Value V19981_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_317 = {S35, 0, &V19981_317_57271, &P19981_316};	/*Property = Json*/
const Value V19981_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P19981_15 = {S37, 0, &V19981_15_29, &P19981_317};	/*Property = LayoutEngine*/
const Value V19981_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_344 = {S38, 0, &V19981_344_57274, &P19981_15};	/*Property = Masking*/
const Value V19981_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_320 = {S39, 0, &V19981_320_57292, &P19981_344};	/*Property = PostMessage*/
const Value V19981_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_321 = {S40, 0, &V19981_321_57295, &P19981_320};	/*Property = Progress*/
const Value V19981_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_322 = {S41, 0, &V19981_322_57298, &P19981_321};	/*Property = Prompts*/
const Value V19981_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_323 = {S42, 0, &V19981_323_57301, &P19981_322};	/*Property = Selector*/
const Value V19981_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_324 = {S43, 0, &V19981_324_57307, &P19981_323};	/*Property = Svg*/
const Value V19981_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_327 = {S44, 0, &V19981_327_57316, &P19981_324};	/*Property = TouchEvents*/
const Value V19981_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_328 = {S45, 0, &V19981_328_57319, &P19981_327};	/*Property = Track*/
const Value V19981_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_329 = {S46, 0, &V19981_329_57325, &P19981_328};	/*Property = Video*/
const Value V19981_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_330 = {S47, 0, &V19981_330_57328, &P19981_329};	/*Property = Viewport*/
const Value V19981_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_332 = {S48, 0, &V19981_332_57340, &P19981_330};	/*Property = WebWorkers*/
const Value V19981_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P19981_333 = {S49, 0, &V19981_333_57343, &P19981_332};	/*Property = Xhr2*/
const Profile P19981 = {"19981", &P19981_333};	/*Profile ID - 19981*/
const Value V20605_1_1 = {S2, NULL};	/*Value = True*/
const Property P20605_1 = {S50, 0, &V20605_1_1, NULL};	/*Property = IsMobile*/
const Value V20605_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20605_5 = {S52, 0, &V20605_5_1227, &P20605_1};	/*Property = ScreenPixelsHeight*/
const Value V20605_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20605_4 = {S54, 0, &V20605_4_1229, &P20605_5};	/*Property = ScreenPixelsWidth*/
const Profile P20605 = {"20605", &P20605_4};	/*Profile ID - 20605*/
const Value V23871_1_1 = {S2, NULL};	/*Value = True*/
const Property P23871_1 = {S50, 0, &V23871_1_1, NULL};	/*Property = IsMobile*/
const Value V23871_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23871_5 = {S52, 0, &V23871_5_1277, &P23871_1};	/*Property = ScreenPixelsHeight*/
const Value V23871_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23871_4 = {S54, 0, &V23871_4_1279, &P23871_5};	/*Property = ScreenPixelsWidth*/
const Profile P23871 = {"23871", &P23871_4};	/*Profile ID - 23871*/
const Value V22464_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22464_279 = {S1, 0, &V22464_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22464_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22464_282 = {S3, 0, &V22464_282_57154, &P22464_279};	/*Property = BlobBuilder*/
const Value V22464_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22464_350 = {S4, 0, &V22464_350_57444, &P22464_282};	/*Property = Canvas*/
const Value V22464_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22464_283 = {S5, 0, &V22464_283_57159, &P22464_350};	/*Property = CssBackground*/
const Value V22464_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22464_284 = {S6, 0, &V22464_284_57163, &P22464_283};	/*Property = CssBorderImage*/
const Value V22464_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22464_285 = {S7, 0, &V22464_285_57166, &P22464_284};	/*Property = CssCanvas*/
const Value V22464_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22464_286 = {S8, 0, &V22464_286_57168, &P22464_285};	/*Property = CssColor*/
const Value V22464_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22464_237 = {S9, 0, &V22464_237_55786, &P22464_286};	/*Property = CssColumn*/
const Value V22464_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22464_287 = {S10, 0, &V22464_287_57172, &P22464_237};	/*Property = CssFlexbox*/
const Value V22464_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22464_288 = {S11, 0, &V22464_288_57174, &P22464_287};	/*Property = CssFont*/
const Value V22464_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22464_289 = {S12, 0, &V22464_289_57759, &P22464_288};	/*Property = CssImages*/
const Value V22464_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22464_291 = {S13, 0, &V22464_291_57180, &P22464_289};	/*Property = CssMediaQueries*/
const Value V22464_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22464_292 = {S14, 0, &V22464_292_57183, &P22464_291};	/*Property = CssMinMax*/
const Value V22464_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22464_293 = {S15, 0, &V22464_293_57187, &P22464_292};	/*Property = CssOverflow*/
const Value V22464_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22464_294 = {S16, 0, &V22464_294_57189, &P22464_293};	/*Property = CssPosition*/
const Value V22464_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22464_295 = {S17, 0, &V22464_295_57192, &P22464_294};	/*Property = CssText*/
const Value V22464_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22464_297 = {S18, 0, &V22464_297_57195, &P22464_295};	/*Property = CssTransforms*/
const Value V22464_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22464_241 = {S19, 0, &V22464_241_55799, &P22464_297};	/*Property = CssTransitions*/
const Value V22464_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22464_298 = {S20, 0, &V22464_298_57206, &P22464_241};	/*Property = CssUI*/
const Value V22464_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22464_300 = {S21, 0, &V22464_300_57216, &P22464_298};	/*Property = DataSet*/
const Value V22464_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22464_301 = {S22, 0, &V22464_301_57218, &P22464_300};	/*Property = DataUrl*/
const Value V22464_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22464_302 = {S23, 0, &V22464_302_57222, &P22464_301};	/*Property = DeviceOrientation*/
const Value V22464_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22464_304 = {S24, 0, &V22464_304_57227, &P22464_302};	/*Property = FileReader*/
const Value V22464_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22464_305 = {S25, 0, &V22464_305_57231, &P22464_304};	/*Property = FileSaver*/
const Value V22464_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22464_306 = {S26, 0, &V22464_306_57237, &P22464_305};	/*Property = FileWriter*/
const Value V22464_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22464_307 = {S27, 0, &V22464_307_57239, &P22464_306};	/*Property = FormData*/
const Value V22464_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22464_309 = {S28, 0, &V22464_309_57246, &P22464_307};	/*Property = Fullscreen*/
const Value V22464_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22464_310 = {S29, 0, &V22464_310_57248, &P22464_309};	/*Property = GeoLocation*/
const Value V22464_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22464_312 = {S30, 0, &V22464_312_57254, &P22464_310};	/*Property = History*/
const Value V22464_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22464_314 = {S31, 0, &V22464_314_57261, &P22464_312};	/*Property = Html5*/
const Value V22464_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22464_313 = {S32, 0, &V22464_313_57258, &P22464_314};	/*Property = Html-Media-Capture*/
const Value V22464_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22464_315 = {S33, 0, &V22464_315_57264, &P22464_313};	/*Property = Iframe*/
const Value V22464_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22464_316 = {S34, 0, &V22464_316_57266, &P22464_315};	/*Property = IndexedDB*/
const Value V22464_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22464_317 = {S35, 0, &V22464_317_57269, &P22464_316};	/*Property = Json*/
const Value V22464_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22464_15 = {S37, 0, &V22464_15_28, &P22464_317};	/*Property = LayoutEngine*/
const Value V22464_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22464_344 = {S38, 0, &V22464_344_57272, &P22464_15};	/*Property = Masking*/
const Value V22464_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22464_320 = {S39, 0, &V22464_320_57290, &P22464_344};	/*Property = PostMessage*/
const Value V22464_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22464_321 = {S40, 0, &V22464_321_57293, &P22464_320};	/*Property = Progress*/
const Value V22464_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22464_322 = {S41, 0, &V22464_322_57296, &P22464_321};	/*Property = Prompts*/
const Value V22464_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22464_323 = {S42, 0, &V22464_323_57299, &P22464_322};	/*Property = Selector*/
const Value V22464_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22464_324 = {S43, 0, &V22464_324_57306, &P22464_323};	/*Property = Svg*/
const Value V22464_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22464_327 = {S44, 0, &V22464_327_57315, &P22464_324};	/*Property = TouchEvents*/
const Value V22464_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22464_328 = {S45, 0, &V22464_328_57318, &P22464_327};	/*Property = Track*/
const Value V22464_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22464_329 = {S46, 0, &V22464_329_57323, &P22464_328};	/*Property = Video*/
const Value V22464_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22464_330 = {S47, 0, &V22464_330_57326, &P22464_329};	/*Property = Viewport*/
const Value V22464_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22464_332 = {S48, 0, &V22464_332_57339, &P22464_330};	/*Property = WebWorkers*/
const Value V22464_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22464_333 = {S49, 0, &V22464_333_57342, &P22464_332};	/*Property = Xhr2*/
const Profile P22464 = {"22464", &P22464_333};	/*Profile ID - 22464*/
const Value V24676_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_279 = {S1, 0, &V24676_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24676_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_282 = {S3, 0, &V24676_282_57155, &P24676_279};	/*Property = BlobBuilder*/
const Value V24676_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_350 = {S4, 0, &V24676_350_57446, &P24676_282};	/*Property = Canvas*/
const Value V24676_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_283 = {S5, 0, &V24676_283_57161, &P24676_350};	/*Property = CssBackground*/
const Value V24676_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_284 = {S6, 0, &V24676_284_57164, &P24676_283};	/*Property = CssBorderImage*/
const Value V24676_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_285 = {S7, 0, &V24676_285_57167, &P24676_284};	/*Property = CssCanvas*/
const Value V24676_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_286 = {S8, 0, &V24676_286_57170, &P24676_285};	/*Property = CssColor*/
const Value V24676_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_237 = {S9, 0, &V24676_237_55786, &P24676_286};	/*Property = CssColumn*/
const Value V24676_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_287 = {S10, 0, &V24676_287_57173, &P24676_237};	/*Property = CssFlexbox*/
const Value V24676_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_288 = {S11, 0, &V24676_288_57176, &P24676_287};	/*Property = CssFont*/
const Value V24676_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_289 = {S12, 0, &V24676_289_86899, &P24676_288};	/*Property = CssImages*/
const Value V24676_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_291 = {S13, 0, &V24676_291_57182, &P24676_289};	/*Property = CssMediaQueries*/
const Value V24676_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_292 = {S14, 0, &V24676_292_57185, &P24676_291};	/*Property = CssMinMax*/
const Value V24676_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_293 = {S15, 0, &V24676_293_57188, &P24676_292};	/*Property = CssOverflow*/
const Value V24676_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_294 = {S16, 0, &V24676_294_57191, &P24676_293};	/*Property = CssPosition*/
const Value V24676_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_295 = {S17, 0, &V24676_295_57194, &P24676_294};	/*Property = CssText*/
const Value V24676_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_297 = {S18, 0, &V24676_297_57197, &P24676_295};	/*Property = CssTransforms*/
const Value V24676_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_241 = {S19, 0, &V24676_241_55801, &P24676_297};	/*Property = CssTransitions*/
const Value V24676_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_298 = {S20, 0, &V24676_298_57208, &P24676_241};	/*Property = CssUI*/
const Value V24676_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_300 = {S21, 0, &V24676_300_57217, &P24676_298};	/*Property = DataSet*/
const Value V24676_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_301 = {S22, 0, &V24676_301_57220, &P24676_300};	/*Property = DataUrl*/
const Value V24676_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_302 = {S23, 0, &V24676_302_57223, &P24676_301};	/*Property = DeviceOrientation*/
const Value V24676_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_304 = {S24, 0, &V24676_304_57229, &P24676_302};	/*Property = FileReader*/
const Value V24676_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_305 = {S25, 0, &V24676_305_57232, &P24676_304};	/*Property = FileSaver*/
const Value V24676_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_306 = {S26, 0, &V24676_306_57238, &P24676_305};	/*Property = FileWriter*/
const Value V24676_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_307 = {S27, 0, &V24676_307_57241, &P24676_306};	/*Property = FormData*/
const Value V24676_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_309 = {S28, 0, &V24676_309_57247, &P24676_307};	/*Property = Fullscreen*/
const Value V24676_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_310 = {S29, 0, &V24676_310_57250, &P24676_309};	/*Property = GeoLocation*/
const Value V24676_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_312 = {S30, 0, &V24676_312_57256, &P24676_310};	/*Property = History*/
const Value V24676_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_314 = {S31, 0, &V24676_314_57262, &P24676_312};	/*Property = Html5*/
const Value V24676_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_313 = {S32, 0, &V24676_313_57259, &P24676_314};	/*Property = Html-Media-Capture*/
const Value V24676_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_315 = {S33, 0, &V24676_315_57265, &P24676_313};	/*Property = Iframe*/
const Value V24676_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_316 = {S34, 0, &V24676_316_57268, &P24676_315};	/*Property = IndexedDB*/
const Value V24676_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_317 = {S35, 0, &V24676_317_57271, &P24676_316};	/*Property = Json*/
const Value V24676_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24676_15 = {S37, 0, &V24676_15_90837, &P24676_317};	/*Property = LayoutEngine*/
const Value V24676_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_344 = {S38, 0, &V24676_344_57274, &P24676_15};	/*Property = Masking*/
const Value V24676_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_320 = {S39, 0, &V24676_320_57292, &P24676_344};	/*Property = PostMessage*/
const Value V24676_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_321 = {S40, 0, &V24676_321_57295, &P24676_320};	/*Property = Progress*/
const Value V24676_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_322 = {S41, 0, &V24676_322_57298, &P24676_321};	/*Property = Prompts*/
const Value V24676_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_323 = {S42, 0, &V24676_323_57301, &P24676_322};	/*Property = Selector*/
const Value V24676_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_324 = {S43, 0, &V24676_324_57307, &P24676_323};	/*Property = Svg*/
const Value V24676_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_327 = {S44, 0, &V24676_327_57316, &P24676_324};	/*Property = TouchEvents*/
const Value V24676_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_328 = {S45, 0, &V24676_328_57319, &P24676_327};	/*Property = Track*/
const Value V24676_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_329 = {S46, 0, &V24676_329_57325, &P24676_328};	/*Property = Video*/
const Value V24676_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_330 = {S47, 0, &V24676_330_57328, &P24676_329};	/*Property = Viewport*/
const Value V24676_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_332 = {S48, 0, &V24676_332_57340, &P24676_330};	/*Property = WebWorkers*/
const Value V24676_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24676_333 = {S49, 0, &V24676_333_57343, &P24676_332};	/*Property = Xhr2*/
const Profile P24676 = {"24676", &P24676_333};	/*Profile ID - 24676*/
const Value V19892_1_1 = {S2, NULL};	/*Value = True*/
const Property P19892_1 = {S50, 0, &V19892_1_1, NULL};	/*Property = IsMobile*/
const Value V19892_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P19892_5 = {S52, 0, &V19892_5_1222, &P19892_1};	/*Property = ScreenPixelsHeight*/
const Value V19892_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P19892_4 = {S54, 0, &V19892_4_1224, &P19892_5};	/*Property = ScreenPixelsWidth*/
const Profile P19892 = {"19892", &P19892_4};	/*Profile ID - 19892*/
const Value V20819_1_1 = {S2, NULL};	/*Value = True*/
const Property P20819_1 = {S50, 0, &V20819_1_1, NULL};	/*Property = IsMobile*/
const Value V20819_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20819_5 = {S52, 0, &V20819_5_1458, &P20819_1};	/*Property = ScreenPixelsHeight*/
const Value V20819_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20819_4 = {S54, 0, &V20819_4_1460, &P20819_5};	/*Property = ScreenPixelsWidth*/
const Profile P20819 = {"20819", &P20819_4};	/*Profile ID - 20819*/
const Value V19837_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19837_279 = {S1, 0, &V19837_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19837_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19837_282 = {S3, 0, &V19837_282_57154, &P19837_279};	/*Property = BlobBuilder*/
const Value V19837_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19837_350 = {S4, 0, &V19837_350_57444, &P19837_282};	/*Property = Canvas*/
const Value V19837_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19837_283 = {S5, 0, &V19837_283_57159, &P19837_350};	/*Property = CssBackground*/
const Value V19837_284_57163 = {S0, NULL};	/*Value = False*/
const Property P19837_284 = {S6, 0, &V19837_284_57163, &P19837_283};	/*Property = CssBorderImage*/
const Value V19837_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19837_285 = {S7, 0, &V19837_285_57166, &P19837_284};	/*Property = CssCanvas*/
const Value V19837_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19837_286 = {S8, 0, &V19837_286_57168, &P19837_285};	/*Property = CssColor*/
const Value V19837_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19837_237 = {S9, 0, &V19837_237_55786, &P19837_286};	/*Property = CssColumn*/
const Value V19837_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19837_287 = {S10, 0, &V19837_287_57172, &P19837_237};	/*Property = CssFlexbox*/
const Value V19837_288_57174 = {S2, NULL};	/*Value = True*/
const Property P19837_288 = {S11, 0, &V19837_288_57174, &P19837_287};	/*Property = CssFont*/
const Value V19837_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19837_289 = {S12, 0, &V19837_289_57759, &P19837_288};	/*Property = CssImages*/
const Value V19837_291_57180 = {S2, NULL};	/*Value = True*/
const Property P19837_291 = {S13, 0, &V19837_291_57180, &P19837_289};	/*Property = CssMediaQueries*/
const Value V19837_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19837_292 = {S14, 0, &V19837_292_57183, &P19837_291};	/*Property = CssMinMax*/
const Value V19837_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19837_293 = {S15, 0, &V19837_293_57187, &P19837_292};	/*Property = CssOverflow*/
const Value V19837_294_57189 = {S2, NULL};	/*Value = True*/
const Property P19837_294 = {S16, 0, &V19837_294_57189, &P19837_293};	/*Property = CssPosition*/
const Value V19837_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19837_295 = {S17, 0, &V19837_295_57192, &P19837_294};	/*Property = CssText*/
const Value V19837_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19837_297 = {S18, 0, &V19837_297_57195, &P19837_295};	/*Property = CssTransforms*/
const Value V19837_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19837_241 = {S19, 0, &V19837_241_55799, &P19837_297};	/*Property = CssTransitions*/
const Value V19837_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19837_298 = {S20, 0, &V19837_298_57206, &P19837_241};	/*Property = CssUI*/
const Value V19837_300_57216 = {S0, NULL};	/*Value = False*/
const Property P19837_300 = {S21, 0, &V19837_300_57216, &P19837_298};	/*Property = DataSet*/
const Value V19837_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19837_301 = {S22, 0, &V19837_301_57218, &P19837_300};	/*Property = DataUrl*/
const Value V19837_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19837_302 = {S23, 0, &V19837_302_57222, &P19837_301};	/*Property = DeviceOrientation*/
const Value V19837_304_57227 = {S2, NULL};	/*Value = True*/
const Property P19837_304 = {S24, 0, &V19837_304_57227, &P19837_302};	/*Property = FileReader*/
const Value V19837_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19837_305 = {S25, 0, &V19837_305_57231, &P19837_304};	/*Property = FileSaver*/
const Value V19837_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19837_306 = {S26, 0, &V19837_306_57237, &P19837_305};	/*Property = FileWriter*/
const Value V19837_307_57239 = {S2, NULL};	/*Value = True*/
const Property P19837_307 = {S27, 0, &V19837_307_57239, &P19837_306};	/*Property = FormData*/
const Value V19837_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19837_309 = {S28, 0, &V19837_309_57246, &P19837_307};	/*Property = Fullscreen*/
const Value V19837_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19837_310 = {S29, 0, &V19837_310_57248, &P19837_309};	/*Property = GeoLocation*/
const Value V19837_312_57254 = {S2, NULL};	/*Value = True*/
const Property P19837_312 = {S30, 0, &V19837_312_57254, &P19837_310};	/*Property = History*/
const Value V19837_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19837_314 = {S31, 0, &V19837_314_57261, &P19837_312};	/*Property = Html5*/
const Value V19837_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19837_313 = {S32, 0, &V19837_313_57258, &P19837_314};	/*Property = Html-Media-Capture*/
const Value V19837_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19837_315 = {S33, 0, &V19837_315_57264, &P19837_313};	/*Property = Iframe*/
const Value V19837_316_57266 = {S2, NULL};	/*Value = True*/
const Property P19837_316 = {S34, 0, &V19837_316_57266, &P19837_315};	/*Property = IndexedDB*/
const Value V19837_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19837_317 = {S35, 0, &V19837_317_57269, &P19837_316};	/*Property = Json*/
const Value V19837_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19837_15 = {S37, 0, &V19837_15_28, &P19837_317};	/*Property = LayoutEngine*/
const Value V19837_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19837_344 = {S38, 0, &V19837_344_57272, &P19837_15};	/*Property = Masking*/
const Value V19837_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19837_320 = {S39, 0, &V19837_320_57290, &P19837_344};	/*Property = PostMessage*/
const Value V19837_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19837_321 = {S40, 0, &V19837_321_57293, &P19837_320};	/*Property = Progress*/
const Value V19837_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19837_322 = {S41, 0, &V19837_322_57296, &P19837_321};	/*Property = Prompts*/
const Value V19837_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19837_323 = {S42, 0, &V19837_323_57299, &P19837_322};	/*Property = Selector*/
const Value V19837_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19837_324 = {S43, 0, &V19837_324_57306, &P19837_323};	/*Property = Svg*/
const Value V19837_327_57314 = {S2, NULL};	/*Value = True*/
const Property P19837_327 = {S44, 0, &V19837_327_57314, &P19837_324};	/*Property = TouchEvents*/
const Value V19837_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19837_328 = {S45, 0, &V19837_328_57318, &P19837_327};	/*Property = Track*/
const Value V19837_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19837_329 = {S46, 0, &V19837_329_57323, &P19837_328};	/*Property = Video*/
const Value V19837_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19837_330 = {S47, 0, &V19837_330_57326, &P19837_329};	/*Property = Viewport*/
const Value V19837_332_57339 = {S0, NULL};	/*Value = False*/
const Property P19837_332 = {S48, 0, &V19837_332_57339, &P19837_330};	/*Property = WebWorkers*/
const Value V19837_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19837_333 = {S49, 0, &V19837_333_57342, &P19837_332};	/*Property = Xhr2*/
const Profile P19837 = {"19837", &P19837_333};	/*Profile ID - 19837*/
const Value V20160_1_1 = {S2, NULL};	/*Value = True*/
const Property P20160_1 = {S50, 0, &V20160_1_1, NULL};	/*Property = IsMobile*/
const Value V20160_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20160_5 = {S52, 0, &V20160_5_9981, &P20160_1};	/*Property = ScreenPixelsHeight*/
const Value V20160_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20160_4 = {S54, 0, &V20160_4_2336, &P20160_5};	/*Property = ScreenPixelsWidth*/
const Profile P20160 = {"20160", &P20160_4};	/*Profile ID - 20160*/
const Value V22394_1_1 = {S2, NULL};	/*Value = True*/
const Property P22394_1 = {S50, 0, &V22394_1_1, NULL};	/*Property = IsMobile*/
const Value V22394_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22394_5 = {S52, 0, &V22394_5_59138, &P22394_1};	/*Property = ScreenPixelsHeight*/
const Value V22394_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22394_4 = {S54, 0, &V22394_4_59970, &P22394_5};	/*Property = ScreenPixelsWidth*/
const Profile P22394 = {"22394", &P22394_4};	/*Profile ID - 22394*/
const Value V21767_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_279 = {S1, 0, &V21767_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21767_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_282 = {S3, 0, &V21767_282_57155, &P21767_279};	/*Property = BlobBuilder*/
const Value V21767_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_350 = {S4, 0, &V21767_350_57446, &P21767_282};	/*Property = Canvas*/
const Value V21767_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_283 = {S5, 0, &V21767_283_57161, &P21767_350};	/*Property = CssBackground*/
const Value V21767_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_284 = {S6, 0, &V21767_284_57164, &P21767_283};	/*Property = CssBorderImage*/
const Value V21767_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_285 = {S7, 0, &V21767_285_57167, &P21767_284};	/*Property = CssCanvas*/
const Value V21767_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_286 = {S8, 0, &V21767_286_57170, &P21767_285};	/*Property = CssColor*/
const Value V21767_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_237 = {S9, 0, &V21767_237_55786, &P21767_286};	/*Property = CssColumn*/
const Value V21767_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_287 = {S10, 0, &V21767_287_57173, &P21767_237};	/*Property = CssFlexbox*/
const Value V21767_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_288 = {S11, 0, &V21767_288_57176, &P21767_287};	/*Property = CssFont*/
const Value V21767_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_289 = {S12, 0, &V21767_289_86899, &P21767_288};	/*Property = CssImages*/
const Value V21767_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_291 = {S13, 0, &V21767_291_57182, &P21767_289};	/*Property = CssMediaQueries*/
const Value V21767_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_292 = {S14, 0, &V21767_292_57185, &P21767_291};	/*Property = CssMinMax*/
const Value V21767_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_293 = {S15, 0, &V21767_293_57188, &P21767_292};	/*Property = CssOverflow*/
const Value V21767_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_294 = {S16, 0, &V21767_294_57191, &P21767_293};	/*Property = CssPosition*/
const Value V21767_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_295 = {S17, 0, &V21767_295_57194, &P21767_294};	/*Property = CssText*/
const Value V21767_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_297 = {S18, 0, &V21767_297_57197, &P21767_295};	/*Property = CssTransforms*/
const Value V21767_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_241 = {S19, 0, &V21767_241_55801, &P21767_297};	/*Property = CssTransitions*/
const Value V21767_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_298 = {S20, 0, &V21767_298_57208, &P21767_241};	/*Property = CssUI*/
const Value V21767_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_300 = {S21, 0, &V21767_300_57217, &P21767_298};	/*Property = DataSet*/
const Value V21767_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_301 = {S22, 0, &V21767_301_57220, &P21767_300};	/*Property = DataUrl*/
const Value V21767_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_302 = {S23, 0, &V21767_302_57223, &P21767_301};	/*Property = DeviceOrientation*/
const Value V21767_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_304 = {S24, 0, &V21767_304_57229, &P21767_302};	/*Property = FileReader*/
const Value V21767_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_305 = {S25, 0, &V21767_305_57232, &P21767_304};	/*Property = FileSaver*/
const Value V21767_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_306 = {S26, 0, &V21767_306_57238, &P21767_305};	/*Property = FileWriter*/
const Value V21767_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_307 = {S27, 0, &V21767_307_57241, &P21767_306};	/*Property = FormData*/
const Value V21767_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_309 = {S28, 0, &V21767_309_57247, &P21767_307};	/*Property = Fullscreen*/
const Value V21767_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_310 = {S29, 0, &V21767_310_57250, &P21767_309};	/*Property = GeoLocation*/
const Value V21767_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_312 = {S30, 0, &V21767_312_57256, &P21767_310};	/*Property = History*/
const Value V21767_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_314 = {S31, 0, &V21767_314_57262, &P21767_312};	/*Property = Html5*/
const Value V21767_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_313 = {S32, 0, &V21767_313_57259, &P21767_314};	/*Property = Html-Media-Capture*/
const Value V21767_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_315 = {S33, 0, &V21767_315_57265, &P21767_313};	/*Property = Iframe*/
const Value V21767_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_316 = {S34, 0, &V21767_316_57268, &P21767_315};	/*Property = IndexedDB*/
const Value V21767_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_317 = {S35, 0, &V21767_317_57271, &P21767_316};	/*Property = Json*/
const Value V21767_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21767_15 = {S37, 0, &V21767_15_54167, &P21767_317};	/*Property = LayoutEngine*/
const Value V21767_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_344 = {S38, 0, &V21767_344_57274, &P21767_15};	/*Property = Masking*/
const Value V21767_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_320 = {S39, 0, &V21767_320_57292, &P21767_344};	/*Property = PostMessage*/
const Value V21767_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_321 = {S40, 0, &V21767_321_57295, &P21767_320};	/*Property = Progress*/
const Value V21767_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_322 = {S41, 0, &V21767_322_57298, &P21767_321};	/*Property = Prompts*/
const Value V21767_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_323 = {S42, 0, &V21767_323_57301, &P21767_322};	/*Property = Selector*/
const Value V21767_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_324 = {S43, 0, &V21767_324_57307, &P21767_323};	/*Property = Svg*/
const Value V21767_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_327 = {S44, 0, &V21767_327_57316, &P21767_324};	/*Property = TouchEvents*/
const Value V21767_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_328 = {S45, 0, &V21767_328_57319, &P21767_327};	/*Property = Track*/
const Value V21767_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_329 = {S46, 0, &V21767_329_57325, &P21767_328};	/*Property = Video*/
const Value V21767_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_330 = {S47, 0, &V21767_330_57328, &P21767_329};	/*Property = Viewport*/
const Value V21767_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_332 = {S48, 0, &V21767_332_57340, &P21767_330};	/*Property = WebWorkers*/
const Value V21767_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21767_333 = {S49, 0, &V21767_333_57343, &P21767_332};	/*Property = Xhr2*/
const Profile P21767 = {"21767", &P21767_333};	/*Profile ID - 21767*/
const Value V21178_279_57147 = {S2, NULL};	/*Value = True*/
const Property P21178_279 = {S1, 0, &V21178_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V21178_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21178_282 = {S3, 0, &V21178_282_57154, &P21178_279};	/*Property = BlobBuilder*/
const Value V21178_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21178_350 = {S4, 0, &V21178_350_57444, &P21178_282};	/*Property = Canvas*/
const Value V21178_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21178_283 = {S5, 0, &V21178_283_57159, &P21178_350};	/*Property = CssBackground*/
const Value V21178_284_57162 = {S2, NULL};	/*Value = True*/
const Property P21178_284 = {S6, 0, &V21178_284_57162, &P21178_283};	/*Property = CssBorderImage*/
const Value V21178_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21178_285 = {S7, 0, &V21178_285_57166, &P21178_284};	/*Property = CssCanvas*/
const Value V21178_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21178_286 = {S8, 0, &V21178_286_57168, &P21178_285};	/*Property = CssColor*/
const Value V21178_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21178_237 = {S9, 0, &V21178_237_55784, &P21178_286};	/*Property = CssColumn*/
const Value V21178_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21178_287 = {S10, 0, &V21178_287_57172, &P21178_237};	/*Property = CssFlexbox*/
const Value V21178_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21178_288 = {S11, 0, &V21178_288_57175, &P21178_287};	/*Property = CssFont*/
const Value V21178_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21178_289 = {S12, 0, &V21178_289_57759, &P21178_288};	/*Property = CssImages*/
const Value V21178_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21178_291 = {S13, 0, &V21178_291_57180, &P21178_289};	/*Property = CssMediaQueries*/
const Value V21178_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21178_292 = {S14, 0, &V21178_292_57183, &P21178_291};	/*Property = CssMinMax*/
const Value V21178_293_57186 = {S2, NULL};	/*Value = True*/
const Property P21178_293 = {S15, 0, &V21178_293_57186, &P21178_292};	/*Property = CssOverflow*/
const Value V21178_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21178_294 = {S16, 0, &V21178_294_57189, &P21178_293};	/*Property = CssPosition*/
const Value V21178_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21178_295 = {S17, 0, &V21178_295_57192, &P21178_294};	/*Property = CssText*/
const Value V21178_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21178_297 = {S18, 0, &V21178_297_57195, &P21178_295};	/*Property = CssTransforms*/
const Value V21178_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21178_241 = {S19, 0, &V21178_241_55799, &P21178_297};	/*Property = CssTransitions*/
const Value V21178_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21178_298 = {S20, 0, &V21178_298_57206, &P21178_241};	/*Property = CssUI*/
const Value V21178_300_57215 = {S2, NULL};	/*Value = True*/
const Property P21178_300 = {S21, 0, &V21178_300_57215, &P21178_298};	/*Property = DataSet*/
const Value V21178_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21178_301 = {S22, 0, &V21178_301_57218, &P21178_300};	/*Property = DataUrl*/
const Value V21178_302_57221 = {S2, NULL};	/*Value = True*/
const Property P21178_302 = {S23, 0, &V21178_302_57221, &P21178_301};	/*Property = DeviceOrientation*/
const Value V21178_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21178_304 = {S24, 0, &V21178_304_57227, &P21178_302};	/*Property = FileReader*/
const Value V21178_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21178_305 = {S25, 0, &V21178_305_57231, &P21178_304};	/*Property = FileSaver*/
const Value V21178_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21178_306 = {S26, 0, &V21178_306_57237, &P21178_305};	/*Property = FileWriter*/
const Value V21178_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21178_307 = {S27, 0, &V21178_307_57239, &P21178_306};	/*Property = FormData*/
const Value V21178_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21178_309 = {S28, 0, &V21178_309_57246, &P21178_307};	/*Property = Fullscreen*/
const Value V21178_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21178_310 = {S29, 0, &V21178_310_57248, &P21178_309};	/*Property = GeoLocation*/
const Value V21178_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21178_312 = {S30, 0, &V21178_312_57254, &P21178_310};	/*Property = History*/
const Value V21178_314_57260 = {S2, NULL};	/*Value = True*/
const Property P21178_314 = {S31, 0, &V21178_314_57260, &P21178_312};	/*Property = Html5*/
const Value V21178_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21178_313 = {S32, 0, &V21178_313_57258, &P21178_314};	/*Property = Html-Media-Capture*/
const Value V21178_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21178_315 = {S33, 0, &V21178_315_57264, &P21178_313};	/*Property = Iframe*/
const Value V21178_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21178_316 = {S34, 0, &V21178_316_57267, &P21178_315};	/*Property = IndexedDB*/
const Value V21178_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21178_317 = {S35, 0, &V21178_317_57269, &P21178_316};	/*Property = Json*/
const Value V21178_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21178_15 = {S37, 0, &V21178_15_28, &P21178_317};	/*Property = LayoutEngine*/
const Value V21178_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21178_344 = {S38, 0, &V21178_344_57272, &P21178_15};	/*Property = Masking*/
const Value V21178_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21178_320 = {S39, 0, &V21178_320_57290, &P21178_344};	/*Property = PostMessage*/
const Value V21178_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21178_321 = {S40, 0, &V21178_321_57293, &P21178_320};	/*Property = Progress*/
const Value V21178_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21178_322 = {S41, 0, &V21178_322_57296, &P21178_321};	/*Property = Prompts*/
const Value V21178_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21178_323 = {S42, 0, &V21178_323_57299, &P21178_322};	/*Property = Selector*/
const Value V21178_324_57305 = {S2, NULL};	/*Value = True*/
const Property P21178_324 = {S43, 0, &V21178_324_57305, &P21178_323};	/*Property = Svg*/
const Value V21178_327_57314 = {S2, NULL};	/*Value = True*/
const Property P21178_327 = {S44, 0, &V21178_327_57314, &P21178_324};	/*Property = TouchEvents*/
const Value V21178_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21178_328 = {S45, 0, &V21178_328_57318, &P21178_327};	/*Property = Track*/
const Value V21178_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21178_329 = {S46, 0, &V21178_329_57323, &P21178_328};	/*Property = Video*/
const Value V21178_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21178_330 = {S47, 0, &V21178_330_57326, &P21178_329};	/*Property = Viewport*/
const Value V21178_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21178_332 = {S48, 0, &V21178_332_57339, &P21178_330};	/*Property = WebWorkers*/
const Value V21178_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21178_333 = {S49, 0, &V21178_333_57342, &P21178_332};	/*Property = Xhr2*/
const Profile P21178 = {"21178", &P21178_333};	/*Profile ID - 21178*/
const Value V23899_1_1 = {S2, NULL};	/*Value = True*/
const Property P23899_1 = {S50, 0, &V23899_1_1, NULL};	/*Property = IsMobile*/
const Value V23899_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23899_5 = {S52, 0, &V23899_5_1458, &P23899_1};	/*Property = ScreenPixelsHeight*/
const Value V23899_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23899_4 = {S54, 0, &V23899_4_37617, &P23899_5};	/*Property = ScreenPixelsWidth*/
const Profile P23899 = {"23899", &P23899_4};	/*Profile ID - 23899*/
const Profile P21721 = {"21721", NULL};	/*Profile ID - 21721*/
const Profile P21460 = {"21460", NULL};	/*Profile ID - 21460*/
const Profile P20531 = {"20531", NULL};	/*Profile ID - 20531*/
const Value V19995_279_57147 = {S2, NULL};	/*Value = True*/
const Property P19995_279 = {S1, 0, &V19995_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V19995_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19995_282 = {S3, 0, &V19995_282_57154, &P19995_279};	/*Property = BlobBuilder*/
const Value V19995_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19995_350 = {S4, 0, &V19995_350_57444, &P19995_282};	/*Property = Canvas*/
const Value V19995_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19995_283 = {S5, 0, &V19995_283_57159, &P19995_350};	/*Property = CssBackground*/
const Value V19995_284_57162 = {S2, NULL};	/*Value = True*/
const Property P19995_284 = {S6, 0, &V19995_284_57162, &P19995_283};	/*Property = CssBorderImage*/
const Value V19995_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19995_285 = {S7, 0, &V19995_285_57166, &P19995_284};	/*Property = CssCanvas*/
const Value V19995_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19995_286 = {S8, 0, &V19995_286_57168, &P19995_285};	/*Property = CssColor*/
const Value V19995_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19995_237 = {S9, 0, &V19995_237_55786, &P19995_286};	/*Property = CssColumn*/
const Value V19995_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19995_287 = {S10, 0, &V19995_287_57172, &P19995_237};	/*Property = CssFlexbox*/
const Value V19995_288_57174 = {S2, NULL};	/*Value = True*/
const Property P19995_288 = {S11, 0, &V19995_288_57174, &P19995_287};	/*Property = CssFont*/
const Value V19995_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19995_289 = {S12, 0, &V19995_289_57759, &P19995_288};	/*Property = CssImages*/
const Value V19995_291_57180 = {S2, NULL};	/*Value = True*/
const Property P19995_291 = {S13, 0, &V19995_291_57180, &P19995_289};	/*Property = CssMediaQueries*/
const Value V19995_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19995_292 = {S14, 0, &V19995_292_57183, &P19995_291};	/*Property = CssMinMax*/
const Value V19995_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19995_293 = {S15, 0, &V19995_293_57187, &P19995_292};	/*Property = CssOverflow*/
const Value V19995_294_57189 = {S2, NULL};	/*Value = True*/
const Property P19995_294 = {S16, 0, &V19995_294_57189, &P19995_293};	/*Property = CssPosition*/
const Value V19995_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19995_295 = {S17, 0, &V19995_295_57192, &P19995_294};	/*Property = CssText*/
const Value V19995_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19995_297 = {S18, 0, &V19995_297_57195, &P19995_295};	/*Property = CssTransforms*/
const Value V19995_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19995_241 = {S19, 0, &V19995_241_55799, &P19995_297};	/*Property = CssTransitions*/
const Value V19995_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19995_298 = {S20, 0, &V19995_298_57206, &P19995_241};	/*Property = CssUI*/
const Value V19995_300_57215 = {S2, NULL};	/*Value = True*/
const Property P19995_300 = {S21, 0, &V19995_300_57215, &P19995_298};	/*Property = DataSet*/
const Value V19995_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19995_301 = {S22, 0, &V19995_301_57218, &P19995_300};	/*Property = DataUrl*/
const Value V19995_302_57222 = {S0, NULL};	/*Value = False*/
const Property P19995_302 = {S23, 0, &V19995_302_57222, &P19995_301};	/*Property = DeviceOrientation*/
const Value V19995_304_57227 = {S2, NULL};	/*Value = True*/
const Property P19995_304 = {S24, 0, &V19995_304_57227, &P19995_302};	/*Property = FileReader*/
const Value V19995_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19995_305 = {S25, 0, &V19995_305_57231, &P19995_304};	/*Property = FileSaver*/
const Value V19995_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19995_306 = {S26, 0, &V19995_306_57237, &P19995_305};	/*Property = FileWriter*/
const Value V19995_307_57239 = {S2, NULL};	/*Value = True*/
const Property P19995_307 = {S27, 0, &V19995_307_57239, &P19995_306};	/*Property = FormData*/
const Value V19995_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19995_309 = {S28, 0, &V19995_309_57246, &P19995_307};	/*Property = Fullscreen*/
const Value V19995_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19995_310 = {S29, 0, &V19995_310_57248, &P19995_309};	/*Property = GeoLocation*/
const Value V19995_312_57254 = {S2, NULL};	/*Value = True*/
const Property P19995_312 = {S30, 0, &V19995_312_57254, &P19995_310};	/*Property = History*/
const Value V19995_314_57260 = {S2, NULL};	/*Value = True*/
const Property P19995_314 = {S31, 0, &V19995_314_57260, &P19995_312};	/*Property = Html5*/
const Value V19995_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19995_313 = {S32, 0, &V19995_313_57258, &P19995_314};	/*Property = Html-Media-Capture*/
const Value V19995_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19995_315 = {S33, 0, &V19995_315_57264, &P19995_313};	/*Property = Iframe*/
const Value V19995_316_57266 = {S2, NULL};	/*Value = True*/
const Property P19995_316 = {S34, 0, &V19995_316_57266, &P19995_315};	/*Property = IndexedDB*/
const Value V19995_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19995_317 = {S35, 0, &V19995_317_57269, &P19995_316};	/*Property = Json*/
const Value V19995_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P19995_15 = {S37, 0, &V19995_15_30, &P19995_317};	/*Property = LayoutEngine*/
const Value V19995_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19995_344 = {S38, 0, &V19995_344_57272, &P19995_15};	/*Property = Masking*/
const Value V19995_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19995_320 = {S39, 0, &V19995_320_57290, &P19995_344};	/*Property = PostMessage*/
const Value V19995_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19995_321 = {S40, 0, &V19995_321_57293, &P19995_320};	/*Property = Progress*/
const Value V19995_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19995_322 = {S41, 0, &V19995_322_57296, &P19995_321};	/*Property = Prompts*/
const Value V19995_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19995_323 = {S42, 0, &V19995_323_57299, &P19995_322};	/*Property = Selector*/
const Value V19995_324_57305 = {S2, NULL};	/*Value = True*/
const Property P19995_324 = {S43, 0, &V19995_324_57305, &P19995_323};	/*Property = Svg*/
const Value V19995_327_57315 = {S0, NULL};	/*Value = False*/
const Property P19995_327 = {S44, 0, &V19995_327_57315, &P19995_324};	/*Property = TouchEvents*/
const Value V19995_328_57317 = {S2, NULL};	/*Value = True*/
const Property P19995_328 = {S45, 0, &V19995_328_57317, &P19995_327};	/*Property = Track*/
const Value V19995_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19995_329 = {S46, 0, &V19995_329_57323, &P19995_328};	/*Property = Video*/
const Value V19995_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19995_330 = {S47, 0, &V19995_330_57326, &P19995_329};	/*Property = Viewport*/
const Value V19995_332_57338 = {S2, NULL};	/*Value = True*/
const Property P19995_332 = {S48, 0, &V19995_332_57338, &P19995_330};	/*Property = WebWorkers*/
const Value V19995_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19995_333 = {S49, 0, &V19995_333_57342, &P19995_332};	/*Property = Xhr2*/
const Profile P19995 = {"19995", &P19995_333};	/*Profile ID - 19995*/
const Value V21484_1_1 = {S2, NULL};	/*Value = True*/
const Property P21484_1 = {S50, 0, &V21484_1_1, NULL};	/*Property = IsMobile*/
const Value V21484_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21484_5 = {S52, 0, &V21484_5_1277, &P21484_1};	/*Property = ScreenPixelsHeight*/
const Value V21484_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21484_4 = {S54, 0, &V21484_4_1279, &P21484_5};	/*Property = ScreenPixelsWidth*/
const Profile P21484 = {"21484", &P21484_4};	/*Profile ID - 21484*/
const Value V21486_1_1 = {S2, NULL};	/*Value = True*/
const Property P21486_1 = {S50, 0, &V21486_1_1, NULL};	/*Property = IsMobile*/
const Value V21486_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21486_5 = {S52, 0, &V21486_5_1458, &P21486_1};	/*Property = ScreenPixelsHeight*/
const Value V21486_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21486_4 = {S54, 0, &V21486_4_1460, &P21486_5};	/*Property = ScreenPixelsWidth*/
const Profile P21486 = {"21486", &P21486_4};	/*Profile ID - 21486*/
const Value V23066_1_1 = {S2, NULL};	/*Value = True*/
const Property P23066_1 = {S50, 0, &V23066_1_1, NULL};	/*Property = IsMobile*/
const Value V23066_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23066_5 = {S52, 0, &V23066_5_1222, &P23066_1};	/*Property = ScreenPixelsHeight*/
const Value V23066_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23066_4 = {S54, 0, &V23066_4_1224, &P23066_5};	/*Property = ScreenPixelsWidth*/
const Profile P23066 = {"23066", &P23066_4};	/*Profile ID - 23066*/
const Value V20737_1_1 = {S2, NULL};	/*Value = True*/
const Property P20737_1 = {S50, 0, &V20737_1_1, NULL};	/*Property = IsMobile*/
const Value V20737_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20737_5 = {S52, 0, &V20737_5_1277, &P20737_1};	/*Property = ScreenPixelsHeight*/
const Value V20737_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20737_4 = {S54, 0, &V20737_4_1279, &P20737_5};	/*Property = ScreenPixelsWidth*/
const Profile P20737 = {"20737", &P20737_4};	/*Profile ID - 20737*/
const Value V22983_1_1 = {S2, NULL};	/*Value = True*/
const Property P22983_1 = {S50, 0, &V22983_1_1, NULL};	/*Property = IsMobile*/
const Value V22983_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22983_5 = {S52, 0, &V22983_5_1277, &P22983_1};	/*Property = ScreenPixelsHeight*/
const Value V22983_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22983_4 = {S54, 0, &V22983_4_1279, &P22983_5};	/*Property = ScreenPixelsWidth*/
const Profile P22983 = {"22983", &P22983_4};	/*Profile ID - 22983*/
const Value V21299_1_1 = {S2, NULL};	/*Value = True*/
const Property P21299_1 = {S50, 0, &V21299_1_1, NULL};	/*Property = IsMobile*/
const Value V21299_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21299_5 = {S52, 0, &V21299_5_1227, &P21299_1};	/*Property = ScreenPixelsHeight*/
const Value V21299_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21299_4 = {S54, 0, &V21299_4_1229, &P21299_5};	/*Property = ScreenPixelsWidth*/
const Profile P21299 = {"21299", &P21299_4};	/*Profile ID - 21299*/
const Value V23433_1_1 = {S2, NULL};	/*Value = True*/
const Property P23433_1 = {S50, 0, &V23433_1_1, NULL};	/*Property = IsMobile*/
const Value V23433_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23433_5 = {S52, 0, &V23433_5_1277, &P23433_1};	/*Property = ScreenPixelsHeight*/
const Value V23433_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23433_4 = {S54, 0, &V23433_4_2336, &P23433_5};	/*Property = ScreenPixelsWidth*/
const Profile P23433 = {"23433", &P23433_4};	/*Profile ID - 23433*/
const Value V23636_1_2 = {S0, NULL};	/*Value = False*/
const Property P23636_1 = {S50, 0, &V23636_1_2, NULL};	/*Property = IsMobile*/
const Value V23636_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23636_5 = {S52, 0, &V23636_5_60417, &P23636_1};	/*Property = ScreenPixelsHeight*/
const Value V23636_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23636_4 = {S54, 0, &V23636_4_57095, &P23636_5};	/*Property = ScreenPixelsWidth*/
const Profile P23636 = {"23636", &P23636_4};	/*Profile ID - 23636*/
const Value V23927_1_1 = {S2, NULL};	/*Value = True*/
const Property P23927_1 = {S50, 0, &V23927_1_1, NULL};	/*Property = IsMobile*/
const Value V23927_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23927_5 = {S52, 0, &V23927_5_1227, &P23927_1};	/*Property = ScreenPixelsHeight*/
const Value V23927_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23927_4 = {S54, 0, &V23927_4_1229, &P23927_5};	/*Property = ScreenPixelsWidth*/
const Profile P23927 = {"23927", &P23927_4};	/*Profile ID - 23927*/
const Value V21287_1_1 = {S2, NULL};	/*Value = True*/
const Property P21287_1 = {S50, 0, &V21287_1_1, NULL};	/*Property = IsMobile*/
const Value V21287_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21287_5 = {S52, 0, &V21287_5_1458, &P21287_1};	/*Property = ScreenPixelsHeight*/
const Value V21287_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21287_4 = {S54, 0, &V21287_4_1460, &P21287_5};	/*Property = ScreenPixelsWidth*/
const Profile P21287 = {"21287", &P21287_4};	/*Profile ID - 21287*/
const Value V23630_1_2 = {S0, NULL};	/*Value = False*/
const Property P23630_1 = {S50, 0, &V23630_1_2, NULL};	/*Property = IsMobile*/
const Value V23630_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23630_5 = {S52, 0, &V23630_5_60417, &P23630_1};	/*Property = ScreenPixelsHeight*/
const Value V23630_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23630_4 = {S54, 0, &V23630_4_57095, &P23630_5};	/*Property = ScreenPixelsWidth*/
const Profile P23630 = {"23630", &P23630_4};	/*Profile ID - 23630*/
const Value V24572_1_1 = {S2, NULL};	/*Value = True*/
const Property P24572_1 = {S50, 0, &V24572_1_1, NULL};	/*Property = IsMobile*/
const Value V24572_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24572_5 = {S52, 0, &V24572_5_1403, &P24572_1};	/*Property = ScreenPixelsHeight*/
const Value V24572_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24572_4 = {S54, 0, &V24572_4_1404, &P24572_5};	/*Property = ScreenPixelsWidth*/
const Profile P24572 = {"24572", &P24572_4};	/*Profile ID - 24572*/
const Value V23092_1_1 = {S2, NULL};	/*Value = True*/
const Property P23092_1 = {S50, 0, &V23092_1_1, NULL};	/*Property = IsMobile*/
const Value V23092_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P23092_5 = {S52, 0, &V23092_5_1380, &P23092_1};	/*Property = ScreenPixelsHeight*/
const Value V23092_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23092_4 = {S54, 0, &V23092_4_1279, &P23092_5};	/*Property = ScreenPixelsWidth*/
const Profile P23092 = {"23092", &P23092_4};	/*Profile ID - 23092*/
const Value V23047_1_1 = {S2, NULL};	/*Value = True*/
const Property P23047_1 = {S50, 0, &V23047_1_1, NULL};	/*Property = IsMobile*/
const Value V23047_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23047_5 = {S52, 0, &V23047_5_37, &P23047_1};	/*Property = ScreenPixelsHeight*/
const Value V23047_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23047_4 = {S54, 0, &V23047_4_36, &P23047_5};	/*Property = ScreenPixelsWidth*/
const Profile P23047 = {"23047", &P23047_4};	/*Profile ID - 23047*/
const Value V21267_1_1 = {S2, NULL};	/*Value = True*/
const Property P21267_1 = {S50, 0, &V21267_1_1, NULL};	/*Property = IsMobile*/
const Value V21267_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21267_5 = {S52, 0, &V21267_5_1269, &P21267_1};	/*Property = ScreenPixelsHeight*/
const Value V21267_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21267_4 = {S54, 0, &V21267_4_1229, &P21267_5};	/*Property = ScreenPixelsWidth*/
const Profile P21267 = {"21267", &P21267_4};	/*Profile ID - 21267*/
const Value V23050_1_1 = {S2, NULL};	/*Value = True*/
const Property P23050_1 = {S50, 0, &V23050_1_1, NULL};	/*Property = IsMobile*/
const Value V23050_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23050_5 = {S52, 0, &V23050_5_1222, &P23050_1};	/*Property = ScreenPixelsHeight*/
const Value V23050_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23050_4 = {S54, 0, &V23050_4_1224, &P23050_5};	/*Property = ScreenPixelsWidth*/
const Profile P23050 = {"23050", &P23050_4};	/*Profile ID - 23050*/
const Value V20523_1_1 = {S2, NULL};	/*Value = True*/
const Property P20523_1 = {S50, 0, &V20523_1_1, NULL};	/*Property = IsMobile*/
const Value V20523_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20523_5 = {S52, 0, &V20523_5_1458, &P20523_1};	/*Property = ScreenPixelsHeight*/
const Value V20523_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20523_4 = {S54, 0, &V20523_4_1460, &P20523_5};	/*Property = ScreenPixelsWidth*/
const Profile P20523 = {"20523", &P20523_4};	/*Profile ID - 20523*/
const Value V23758_1_1 = {S2, NULL};	/*Value = True*/
const Property P23758_1 = {S50, 0, &V23758_1_1, NULL};	/*Property = IsMobile*/
const Value V23758_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23758_5 = {S52, 0, &V23758_5_1227, &P23758_1};	/*Property = ScreenPixelsHeight*/
const Value V23758_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23758_4 = {S54, 0, &V23758_4_1229, &P23758_5};	/*Property = ScreenPixelsWidth*/
const Profile P23758 = {"23758", &P23758_4};	/*Profile ID - 23758*/
const Value V20530_1_1 = {S2, NULL};	/*Value = True*/
const Property P20530_1 = {S50, 0, &V20530_1_1, NULL};	/*Property = IsMobile*/
const Value V20530_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20530_5 = {S52, 0, &V20530_5_1403, &P20530_1};	/*Property = ScreenPixelsHeight*/
const Value V20530_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20530_4 = {S54, 0, &V20530_4_1404, &P20530_5};	/*Property = ScreenPixelsWidth*/
const Profile P20530 = {"20530", &P20530_4};	/*Profile ID - 20530*/
const Value V21500_1_1 = {S2, NULL};	/*Value = True*/
const Property P21500_1 = {S50, 0, &V21500_1_1, NULL};	/*Property = IsMobile*/
const Value V21500_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21500_5 = {S52, 0, &V21500_5_1277, &P21500_1};	/*Property = ScreenPixelsHeight*/
const Value V21500_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21500_4 = {S54, 0, &V21500_4_1279, &P21500_5};	/*Property = ScreenPixelsWidth*/
const Profile P21500 = {"21500", &P21500_4};	/*Profile ID - 21500*/
const Value V22015_1_1 = {S2, NULL};	/*Value = True*/
const Property P22015_1 = {S50, 0, &V22015_1_1, NULL};	/*Property = IsMobile*/
const Value V22015_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22015_5 = {S52, 0, &V22015_5_37569, &P22015_1};	/*Property = ScreenPixelsHeight*/
const Value V22015_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22015_4 = {S54, 0, &V22015_4_5645, &P22015_5};	/*Property = ScreenPixelsWidth*/
const Profile P22015 = {"22015", &P22015_4};	/*Profile ID - 22015*/
const Value V21367_1_1 = {S2, NULL};	/*Value = True*/
const Property P21367_1 = {S50, 0, &V21367_1_1, NULL};	/*Property = IsMobile*/
const Value V21367_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21367_5 = {S52, 0, &V21367_5_1458, &P21367_1};	/*Property = ScreenPixelsHeight*/
const Value V21367_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21367_4 = {S54, 0, &V21367_4_37617, &P21367_5};	/*Property = ScreenPixelsWidth*/
const Profile P21367 = {"21367", &P21367_4};	/*Profile ID - 21367*/
const Value V21168_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21168_279 = {S1, 0, &V21168_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21168_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21168_282 = {S3, 0, &V21168_282_57154, &P21168_279};	/*Property = BlobBuilder*/
const Value V21168_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21168_350 = {S4, 0, &V21168_350_57444, &P21168_282};	/*Property = Canvas*/
const Value V21168_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21168_283 = {S5, 0, &V21168_283_57159, &P21168_350};	/*Property = CssBackground*/
const Value V21168_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21168_284 = {S6, 0, &V21168_284_57163, &P21168_283};	/*Property = CssBorderImage*/
const Value V21168_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21168_285 = {S7, 0, &V21168_285_57166, &P21168_284};	/*Property = CssCanvas*/
const Value V21168_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21168_286 = {S8, 0, &V21168_286_57168, &P21168_285};	/*Property = CssColor*/
const Value V21168_237_55785 = {S0, NULL};	/*Value = False*/
const Property P21168_237 = {S9, 0, &V21168_237_55785, &P21168_286};	/*Property = CssColumn*/
const Value V21168_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21168_287 = {S10, 0, &V21168_287_57172, &P21168_237};	/*Property = CssFlexbox*/
const Value V21168_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21168_288 = {S11, 0, &V21168_288_57175, &P21168_287};	/*Property = CssFont*/
const Value V21168_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21168_289 = {S12, 0, &V21168_289_57759, &P21168_288};	/*Property = CssImages*/
const Value V21168_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21168_291 = {S13, 0, &V21168_291_57181, &P21168_289};	/*Property = CssMediaQueries*/
const Value V21168_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21168_292 = {S14, 0, &V21168_292_57183, &P21168_291};	/*Property = CssMinMax*/
const Value V21168_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21168_293 = {S15, 0, &V21168_293_57187, &P21168_292};	/*Property = CssOverflow*/
const Value V21168_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21168_294 = {S16, 0, &V21168_294_57190, &P21168_293};	/*Property = CssPosition*/
const Value V21168_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21168_295 = {S17, 0, &V21168_295_57192, &P21168_294};	/*Property = CssText*/
const Value V21168_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21168_297 = {S18, 0, &V21168_297_57195, &P21168_295};	/*Property = CssTransforms*/
const Value V21168_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21168_241 = {S19, 0, &V21168_241_55799, &P21168_297};	/*Property = CssTransitions*/
const Value V21168_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21168_298 = {S20, 0, &V21168_298_57206, &P21168_241};	/*Property = CssUI*/
const Value V21168_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21168_300 = {S21, 0, &V21168_300_57216, &P21168_298};	/*Property = DataSet*/
const Value V21168_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21168_301 = {S22, 0, &V21168_301_57218, &P21168_300};	/*Property = DataUrl*/
const Value V21168_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21168_302 = {S23, 0, &V21168_302_57222, &P21168_301};	/*Property = DeviceOrientation*/
const Value V21168_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21168_304 = {S24, 0, &V21168_304_57228, &P21168_302};	/*Property = FileReader*/
const Value V21168_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21168_305 = {S25, 0, &V21168_305_57231, &P21168_304};	/*Property = FileSaver*/
const Value V21168_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21168_306 = {S26, 0, &V21168_306_57237, &P21168_305};	/*Property = FileWriter*/
const Value V21168_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21168_307 = {S27, 0, &V21168_307_57240, &P21168_306};	/*Property = FormData*/
const Value V21168_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21168_309 = {S28, 0, &V21168_309_57246, &P21168_307};	/*Property = Fullscreen*/
const Value V21168_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21168_310 = {S29, 0, &V21168_310_57248, &P21168_309};	/*Property = GeoLocation*/
const Value V21168_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21168_312 = {S30, 0, &V21168_312_57255, &P21168_310};	/*Property = History*/
const Value V21168_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21168_314 = {S31, 0, &V21168_314_57261, &P21168_312};	/*Property = Html5*/
const Value V21168_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21168_313 = {S32, 0, &V21168_313_57258, &P21168_314};	/*Property = Html-Media-Capture*/
const Value V21168_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21168_315 = {S33, 0, &V21168_315_57264, &P21168_313};	/*Property = Iframe*/
const Value V21168_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21168_316 = {S34, 0, &V21168_316_57267, &P21168_315};	/*Property = IndexedDB*/
const Value V21168_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21168_317 = {S35, 0, &V21168_317_57269, &P21168_316};	/*Property = Json*/
const Value V21168_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P21168_15 = {S37, 0, &V21168_15_30, &P21168_317};	/*Property = LayoutEngine*/
const Value V21168_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21168_344 = {S38, 0, &V21168_344_57272, &P21168_15};	/*Property = Masking*/
const Value V21168_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21168_320 = {S39, 0, &V21168_320_57290, &P21168_344};	/*Property = PostMessage*/
const Value V21168_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21168_321 = {S40, 0, &V21168_321_57293, &P21168_320};	/*Property = Progress*/
const Value V21168_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21168_322 = {S41, 0, &V21168_322_57296, &P21168_321};	/*Property = Prompts*/
const Value V21168_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21168_323 = {S42, 0, &V21168_323_57299, &P21168_322};	/*Property = Selector*/
const Value V21168_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21168_324 = {S43, 0, &V21168_324_57306, &P21168_323};	/*Property = Svg*/
const Value V21168_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21168_327 = {S44, 0, &V21168_327_57315, &P21168_324};	/*Property = TouchEvents*/
const Value V21168_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21168_328 = {S45, 0, &V21168_328_57318, &P21168_327};	/*Property = Track*/
const Value V21168_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21168_329 = {S46, 0, &V21168_329_57323, &P21168_328};	/*Property = Video*/
const Value V21168_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21168_330 = {S47, 0, &V21168_330_57326, &P21168_329};	/*Property = Viewport*/
const Value V21168_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21168_332 = {S48, 0, &V21168_332_57339, &P21168_330};	/*Property = WebWorkers*/
const Value V21168_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21168_333 = {S49, 0, &V21168_333_57342, &P21168_332};	/*Property = Xhr2*/
const Profile P21168 = {"21168", &P21168_333};	/*Profile ID - 21168*/
const Value V21323_1_1 = {S2, NULL};	/*Value = True*/
const Property P21323_1 = {S50, 0, &V21323_1_1, NULL};	/*Property = IsMobile*/
const Value V21323_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21323_5 = {S52, 0, &V21323_5_9981, &P21323_1};	/*Property = ScreenPixelsHeight*/
const Value V21323_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21323_4 = {S54, 0, &V21323_4_2336, &P21323_5};	/*Property = ScreenPixelsWidth*/
const Profile P21323 = {"21323", &P21323_4};	/*Profile ID - 21323*/
const Value V21804_1_1 = {S2, NULL};	/*Value = True*/
const Property P21804_1 = {S50, 0, &V21804_1_1, NULL};	/*Property = IsMobile*/
const Value V21804_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21804_5 = {S52, 0, &V21804_5_1277, &P21804_1};	/*Property = ScreenPixelsHeight*/
const Value V21804_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21804_4 = {S54, 0, &V21804_4_1279, &P21804_5};	/*Property = ScreenPixelsWidth*/
const Profile P21804 = {"21804", &P21804_4};	/*Profile ID - 21804*/
const Value V22717_1_1 = {S2, NULL};	/*Value = True*/
const Property P22717_1 = {S50, 0, &V22717_1_1, NULL};	/*Property = IsMobile*/
const Value V22717_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P22717_5 = {S52, 0, &V22717_5_1293, &P22717_1};	/*Property = ScreenPixelsHeight*/
const Value V22717_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P22717_4 = {S54, 0, &V22717_4_1296, &P22717_5};	/*Property = ScreenPixelsWidth*/
const Profile P22717 = {"22717", &P22717_4};	/*Profile ID - 22717*/
const Value V20533_1_1 = {S2, NULL};	/*Value = True*/
const Property P20533_1 = {S50, 0, &V20533_1_1, NULL};	/*Property = IsMobile*/
const Value V20533_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20533_5 = {S52, 0, &V20533_5_1458, &P20533_1};	/*Property = ScreenPixelsHeight*/
const Value V20533_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20533_4 = {S54, 0, &V20533_4_37617, &P20533_5};	/*Property = ScreenPixelsWidth*/
const Profile P20533 = {"20533", &P20533_4};	/*Profile ID - 20533*/
const Value V21610_1_1 = {S2, NULL};	/*Value = True*/
const Property P21610_1 = {S50, 0, &V21610_1_1, NULL};	/*Property = IsMobile*/
const Value V21610_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21610_5 = {S52, 0, &V21610_5_1458, &P21610_1};	/*Property = ScreenPixelsHeight*/
const Value V21610_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21610_4 = {S54, 0, &V21610_4_1460, &P21610_5};	/*Property = ScreenPixelsWidth*/
const Profile P21610 = {"21610", &P21610_4};	/*Profile ID - 21610*/
const Value V23445_1_1 = {S2, NULL};	/*Value = True*/
const Property P23445_1 = {S50, 0, &V23445_1_1, NULL};	/*Property = IsMobile*/
const Value V23445_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23445_5 = {S52, 0, &V23445_5_37, &P23445_1};	/*Property = ScreenPixelsHeight*/
const Value V23445_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23445_4 = {S54, 0, &V23445_4_36, &P23445_5};	/*Property = ScreenPixelsWidth*/
const Profile P23445 = {"23445", &P23445_4};	/*Profile ID - 23445*/
const Value V22152_1_1 = {S2, NULL};	/*Value = True*/
const Property P22152_1 = {S50, 0, &V22152_1_1, NULL};	/*Property = IsMobile*/
const Value V22152_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22152_5 = {S52, 0, &V22152_5_1458, &P22152_1};	/*Property = ScreenPixelsHeight*/
const Value V22152_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22152_4 = {S54, 0, &V22152_4_1460, &P22152_5};	/*Property = ScreenPixelsWidth*/
const Profile P22152 = {"22152", &P22152_4};	/*Profile ID - 22152*/
const Value V21545_1_1 = {S2, NULL};	/*Value = True*/
const Property P21545_1 = {S50, 0, &V21545_1_1, NULL};	/*Property = IsMobile*/
const Value V21545_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21545_5 = {S52, 0, &V21545_5_37, &P21545_1};	/*Property = ScreenPixelsHeight*/
const Value V21545_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21545_4 = {S54, 0, &V21545_4_36, &P21545_5};	/*Property = ScreenPixelsWidth*/
const Profile P21545 = {"21545", &P21545_4};	/*Profile ID - 21545*/
const Value V21350_1_1 = {S2, NULL};	/*Value = True*/
const Property P21350_1 = {S50, 0, &V21350_1_1, NULL};	/*Property = IsMobile*/
const Value V21350_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21350_5 = {S52, 0, &V21350_5_1227, &P21350_1};	/*Property = ScreenPixelsHeight*/
const Value V21350_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21350_4 = {S54, 0, &V21350_4_1229, &P21350_5};	/*Property = ScreenPixelsWidth*/
const Profile P21350 = {"21350", &P21350_4};	/*Profile ID - 21350*/
const Value V23566_1_1 = {S2, NULL};	/*Value = True*/
const Property P23566_1 = {S50, 0, &V23566_1_1, NULL};	/*Property = IsMobile*/
const Value V23566_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23566_5 = {S52, 0, &V23566_5_1227, &P23566_1};	/*Property = ScreenPixelsHeight*/
const Value V23566_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23566_4 = {S54, 0, &V23566_4_1229, &P23566_5};	/*Property = ScreenPixelsWidth*/
const Profile P23566 = {"23566", &P23566_4};	/*Profile ID - 23566*/
const Value V21894_1_1 = {S2, NULL};	/*Value = True*/
const Property P21894_1 = {S50, 0, &V21894_1_1, NULL};	/*Property = IsMobile*/
const Value V21894_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21894_5 = {S52, 0, &V21894_5_1277, &P21894_1};	/*Property = ScreenPixelsHeight*/
const Value V21894_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21894_4 = {S54, 0, &V21894_4_1279, &P21894_5};	/*Property = ScreenPixelsWidth*/
const Profile P21894 = {"21894", &P21894_4};	/*Profile ID - 21894*/
const Value V22067_1_1 = {S2, NULL};	/*Value = True*/
const Property P22067_1 = {S50, 0, &V22067_1_1, NULL};	/*Property = IsMobile*/
const Value V22067_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22067_5 = {S52, 0, &V22067_5_1458, &P22067_1};	/*Property = ScreenPixelsHeight*/
const Value V22067_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22067_4 = {S54, 0, &V22067_4_1460, &P22067_5};	/*Property = ScreenPixelsWidth*/
const Profile P22067 = {"22067", &P22067_4};	/*Profile ID - 22067*/
const Value V23812_1_1 = {S2, NULL};	/*Value = True*/
const Property P23812_1 = {S50, 0, &V23812_1_1, NULL};	/*Property = IsMobile*/
const Value V23812_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23812_5 = {S52, 0, &V23812_5_1227, &P23812_1};	/*Property = ScreenPixelsHeight*/
const Value V23812_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23812_4 = {S54, 0, &V23812_4_1229, &P23812_5};	/*Property = ScreenPixelsWidth*/
const Profile P23812 = {"23812", &P23812_4};	/*Profile ID - 23812*/
const Value V20735_1_1 = {S2, NULL};	/*Value = True*/
const Property P20735_1 = {S50, 0, &V20735_1_1, NULL};	/*Property = IsMobile*/
const Value V20735_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P20735_5 = {S52, 0, &V20735_5_1269, &P20735_1};	/*Property = ScreenPixelsHeight*/
const Value V20735_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20735_4 = {S54, 0, &V20735_4_1229, &P20735_5};	/*Property = ScreenPixelsWidth*/
const Profile P20735 = {"20735", &P20735_4};	/*Profile ID - 20735*/
const Value V21815_1_1 = {S2, NULL};	/*Value = True*/
const Property P21815_1 = {S50, 0, &V21815_1_1, NULL};	/*Property = IsMobile*/
const Value V21815_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P21815_5 = {S52, 0, &V21815_5_59138, &P21815_1};	/*Property = ScreenPixelsHeight*/
const Value V21815_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P21815_4 = {S54, 0, &V21815_4_59970, &P21815_5};	/*Property = ScreenPixelsWidth*/
const Profile P21815 = {"21815", &P21815_4};	/*Profile ID - 21815*/
const Value V21304_1_1 = {S2, NULL};	/*Value = True*/
const Property P21304_1 = {S50, 0, &V21304_1_1, NULL};	/*Property = IsMobile*/
const Value V21304_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21304_5 = {S52, 0, &V21304_5_1269, &P21304_1};	/*Property = ScreenPixelsHeight*/
const Value V21304_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21304_4 = {S54, 0, &V21304_4_1229, &P21304_5};	/*Property = ScreenPixelsWidth*/
const Profile P21304 = {"21304", &P21304_4};	/*Profile ID - 21304*/
const Value V21885_1_1 = {S2, NULL};	/*Value = True*/
const Property P21885_1 = {S50, 0, &V21885_1_1, NULL};	/*Property = IsMobile*/
const Value V21885_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21885_5 = {S52, 0, &V21885_5_1227, &P21885_1};	/*Property = ScreenPixelsHeight*/
const Value V21885_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21885_4 = {S54, 0, &V21885_4_1229, &P21885_5};	/*Property = ScreenPixelsWidth*/
const Profile P21885 = {"21885", &P21885_4};	/*Profile ID - 21885*/
const Profile P21697 = {"21697", NULL};	/*Profile ID - 21697*/
const Value V21076_1_1 = {S2, NULL};	/*Value = True*/
const Property P21076_1 = {S50, 0, &V21076_1_1, NULL};	/*Property = IsMobile*/
const Value V21076_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21076_5 = {S52, 0, &V21076_5_1458, &P21076_1};	/*Property = ScreenPixelsHeight*/
const Value V21076_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21076_4 = {S54, 0, &V21076_4_37617, &P21076_5};	/*Property = ScreenPixelsWidth*/
const Profile P21076 = {"21076", &P21076_4};	/*Profile ID - 21076*/
const Value V20738_1_1 = {S2, NULL};	/*Value = True*/
const Property P20738_1 = {S50, 0, &V20738_1_1, NULL};	/*Property = IsMobile*/
const Value V20738_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20738_5 = {S52, 0, &V20738_5_1403, &P20738_1};	/*Property = ScreenPixelsHeight*/
const Value V20738_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20738_4 = {S54, 0, &V20738_4_1404, &P20738_5};	/*Property = ScreenPixelsWidth*/
const Profile P20738 = {"20738", &P20738_4};	/*Profile ID - 20738*/
const Value V21544_1_1 = {S2, NULL};	/*Value = True*/
const Property P21544_1 = {S50, 0, &V21544_1_1, NULL};	/*Property = IsMobile*/
const Value V21544_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21544_5 = {S52, 0, &V21544_5_37, &P21544_1};	/*Property = ScreenPixelsHeight*/
const Value V21544_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21544_4 = {S54, 0, &V21544_4_36, &P21544_5};	/*Property = ScreenPixelsWidth*/
const Profile P21544 = {"21544", &P21544_4};	/*Profile ID - 21544*/
const Value V21155_1_1 = {S2, NULL};	/*Value = True*/
const Property P21155_1 = {S50, 0, &V21155_1_1, NULL};	/*Property = IsMobile*/
const Value V21155_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21155_5 = {S52, 0, &V21155_5_1269, &P21155_1};	/*Property = ScreenPixelsHeight*/
const Value V21155_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21155_4 = {S54, 0, &V21155_4_1229, &P21155_5};	/*Property = ScreenPixelsWidth*/
const Profile P21155 = {"21155", &P21155_4};	/*Profile ID - 21155*/
const Value V21720_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_279 = {S1, 0, &V21720_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21720_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_282 = {S3, 0, &V21720_282_57155, &P21720_279};	/*Property = BlobBuilder*/
const Value V21720_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_350 = {S4, 0, &V21720_350_57446, &P21720_282};	/*Property = Canvas*/
const Value V21720_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_283 = {S5, 0, &V21720_283_57161, &P21720_350};	/*Property = CssBackground*/
const Value V21720_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_284 = {S6, 0, &V21720_284_57164, &P21720_283};	/*Property = CssBorderImage*/
const Value V21720_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_285 = {S7, 0, &V21720_285_57167, &P21720_284};	/*Property = CssCanvas*/
const Value V21720_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_286 = {S8, 0, &V21720_286_57170, &P21720_285};	/*Property = CssColor*/
const Value V21720_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_237 = {S9, 0, &V21720_237_55786, &P21720_286};	/*Property = CssColumn*/
const Value V21720_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_287 = {S10, 0, &V21720_287_57173, &P21720_237};	/*Property = CssFlexbox*/
const Value V21720_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_288 = {S11, 0, &V21720_288_57176, &P21720_287};	/*Property = CssFont*/
const Value V21720_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_289 = {S12, 0, &V21720_289_86899, &P21720_288};	/*Property = CssImages*/
const Value V21720_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_291 = {S13, 0, &V21720_291_57182, &P21720_289};	/*Property = CssMediaQueries*/
const Value V21720_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_292 = {S14, 0, &V21720_292_57185, &P21720_291};	/*Property = CssMinMax*/
const Value V21720_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_293 = {S15, 0, &V21720_293_57188, &P21720_292};	/*Property = CssOverflow*/
const Value V21720_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_294 = {S16, 0, &V21720_294_57191, &P21720_293};	/*Property = CssPosition*/
const Value V21720_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_295 = {S17, 0, &V21720_295_57194, &P21720_294};	/*Property = CssText*/
const Value V21720_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_297 = {S18, 0, &V21720_297_57197, &P21720_295};	/*Property = CssTransforms*/
const Value V21720_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_241 = {S19, 0, &V21720_241_55801, &P21720_297};	/*Property = CssTransitions*/
const Value V21720_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_298 = {S20, 0, &V21720_298_57208, &P21720_241};	/*Property = CssUI*/
const Value V21720_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_300 = {S21, 0, &V21720_300_57217, &P21720_298};	/*Property = DataSet*/
const Value V21720_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_301 = {S22, 0, &V21720_301_57220, &P21720_300};	/*Property = DataUrl*/
const Value V21720_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_302 = {S23, 0, &V21720_302_57223, &P21720_301};	/*Property = DeviceOrientation*/
const Value V21720_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_304 = {S24, 0, &V21720_304_57229, &P21720_302};	/*Property = FileReader*/
const Value V21720_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_305 = {S25, 0, &V21720_305_57232, &P21720_304};	/*Property = FileSaver*/
const Value V21720_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_306 = {S26, 0, &V21720_306_57238, &P21720_305};	/*Property = FileWriter*/
const Value V21720_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_307 = {S27, 0, &V21720_307_57241, &P21720_306};	/*Property = FormData*/
const Value V21720_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_309 = {S28, 0, &V21720_309_57247, &P21720_307};	/*Property = Fullscreen*/
const Value V21720_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_310 = {S29, 0, &V21720_310_57250, &P21720_309};	/*Property = GeoLocation*/
const Value V21720_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_312 = {S30, 0, &V21720_312_57256, &P21720_310};	/*Property = History*/
const Value V21720_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_314 = {S31, 0, &V21720_314_57262, &P21720_312};	/*Property = Html5*/
const Value V21720_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_313 = {S32, 0, &V21720_313_57259, &P21720_314};	/*Property = Html-Media-Capture*/
const Value V21720_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_315 = {S33, 0, &V21720_315_57265, &P21720_313};	/*Property = Iframe*/
const Value V21720_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_316 = {S34, 0, &V21720_316_57268, &P21720_315};	/*Property = IndexedDB*/
const Value V21720_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_317 = {S35, 0, &V21720_317_57271, &P21720_316};	/*Property = Json*/
const Value V21720_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21720_15 = {S37, 0, &V21720_15_54167, &P21720_317};	/*Property = LayoutEngine*/
const Value V21720_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_344 = {S38, 0, &V21720_344_57274, &P21720_15};	/*Property = Masking*/
const Value V21720_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_320 = {S39, 0, &V21720_320_57292, &P21720_344};	/*Property = PostMessage*/
const Value V21720_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_321 = {S40, 0, &V21720_321_57295, &P21720_320};	/*Property = Progress*/
const Value V21720_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_322 = {S41, 0, &V21720_322_57298, &P21720_321};	/*Property = Prompts*/
const Value V21720_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_323 = {S42, 0, &V21720_323_57301, &P21720_322};	/*Property = Selector*/
const Value V21720_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_324 = {S43, 0, &V21720_324_57307, &P21720_323};	/*Property = Svg*/
const Value V21720_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_327 = {S44, 0, &V21720_327_57316, &P21720_324};	/*Property = TouchEvents*/
const Value V21720_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_328 = {S45, 0, &V21720_328_57319, &P21720_327};	/*Property = Track*/
const Value V21720_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_329 = {S46, 0, &V21720_329_57325, &P21720_328};	/*Property = Video*/
const Value V21720_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_330 = {S47, 0, &V21720_330_57328, &P21720_329};	/*Property = Viewport*/
const Value V21720_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_332 = {S48, 0, &V21720_332_57340, &P21720_330};	/*Property = WebWorkers*/
const Value V21720_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21720_333 = {S49, 0, &V21720_333_57343, &P21720_332};	/*Property = Xhr2*/
const Profile P21720 = {"21720", &P21720_333};	/*Profile ID - 21720*/
const Value V21104_1_1 = {S2, NULL};	/*Value = True*/
const Property P21104_1 = {S50, 0, &V21104_1_1, NULL};	/*Property = IsMobile*/
const Value V21104_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21104_5 = {S52, 0, &V21104_5_1403, &P21104_1};	/*Property = ScreenPixelsHeight*/
const Value V21104_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21104_4 = {S54, 0, &V21104_4_1404, &P21104_5};	/*Property = ScreenPixelsWidth*/
const Profile P21104 = {"21104", &P21104_4};	/*Profile ID - 21104*/
const Profile P23108 = {"23108", NULL};	/*Profile ID - 23108*/
const Value V21890_1_1 = {S2, NULL};	/*Value = True*/
const Property P21890_1 = {S50, 0, &V21890_1_1, NULL};	/*Property = IsMobile*/
const Value V21890_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P21890_5 = {S52, 0, &V21890_5_1832, &P21890_1};	/*Property = ScreenPixelsHeight*/
const Value V21890_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21890_4 = {S54, 0, &V21890_4_5645, &P21890_5};	/*Property = ScreenPixelsWidth*/
const Profile P21890 = {"21890", &P21890_4};	/*Profile ID - 21890*/
const Value V21455_1_1 = {S2, NULL};	/*Value = True*/
const Property P21455_1 = {S50, 0, &V21455_1_1, NULL};	/*Property = IsMobile*/
const Value V21455_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21455_5 = {S52, 0, &V21455_5_1269, &P21455_1};	/*Property = ScreenPixelsHeight*/
const Value V21455_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21455_4 = {S54, 0, &V21455_4_1229, &P21455_5};	/*Property = ScreenPixelsWidth*/
const Profile P21455 = {"21455", &P21455_4};	/*Profile ID - 21455*/
const Value V23093_1_1 = {S2, NULL};	/*Value = True*/
const Property P23093_1 = {S50, 0, &V23093_1_1, NULL};	/*Property = IsMobile*/
const Value V23093_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23093_5 = {S52, 0, &V23093_5_1277, &P23093_1};	/*Property = ScreenPixelsHeight*/
const Value V23093_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23093_4 = {S54, 0, &V23093_4_2336, &P23093_5};	/*Property = ScreenPixelsWidth*/
const Profile P23093 = {"23093", &P23093_4};	/*Profile ID - 23093*/
const Value V23869_1_1 = {S2, NULL};	/*Value = True*/
const Property P23869_1 = {S50, 0, &V23869_1_1, NULL};	/*Property = IsMobile*/
const Value V23869_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23869_5 = {S52, 0, &V23869_5_1227, &P23869_1};	/*Property = ScreenPixelsHeight*/
const Value V23869_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23869_4 = {S54, 0, &V23869_4_1229, &P23869_5};	/*Property = ScreenPixelsWidth*/
const Profile P23869 = {"23869", &P23869_4};	/*Profile ID - 23869*/
const Value V22612_1_1 = {S2, NULL};	/*Value = True*/
const Property P22612_1 = {S50, 0, &V22612_1_1, NULL};	/*Property = IsMobile*/
const Value V22612_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P22612_5 = {S52, 0, &V22612_5_1269, &P22612_1};	/*Property = ScreenPixelsHeight*/
const Value V22612_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22612_4 = {S54, 0, &V22612_4_1229, &P22612_5};	/*Property = ScreenPixelsWidth*/
const Profile P22612 = {"22612", &P22612_4};	/*Profile ID - 22612*/
const Value V21411_1_1 = {S2, NULL};	/*Value = True*/
const Property P21411_1 = {S50, 0, &V21411_1_1, NULL};	/*Property = IsMobile*/
const Value V21411_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21411_5 = {S52, 0, &V21411_5_1227, &P21411_1};	/*Property = ScreenPixelsHeight*/
const Value V21411_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21411_4 = {S54, 0, &V21411_4_1229, &P21411_5};	/*Property = ScreenPixelsWidth*/
const Profile P21411 = {"21411", &P21411_4};	/*Profile ID - 21411*/
const Value V20731_1_1 = {S2, NULL};	/*Value = True*/
const Property P20731_1 = {S50, 0, &V20731_1_1, NULL};	/*Property = IsMobile*/
const Value V20731_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P20731_5 = {S52, 0, &V20731_5_1269, &P20731_1};	/*Property = ScreenPixelsHeight*/
const Value V20731_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20731_4 = {S54, 0, &V20731_4_1229, &P20731_5};	/*Property = ScreenPixelsWidth*/
const Profile P20731 = {"20731", &P20731_4};	/*Profile ID - 20731*/
const Value V21622_1_1 = {S2, NULL};	/*Value = True*/
const Property P21622_1 = {S50, 0, &V21622_1_1, NULL};	/*Property = IsMobile*/
const Value V21622_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21622_5 = {S52, 0, &V21622_5_1227, &P21622_1};	/*Property = ScreenPixelsHeight*/
const Value V21622_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21622_4 = {S54, 0, &V21622_4_1229, &P21622_5};	/*Property = ScreenPixelsWidth*/
const Profile P21622 = {"21622", &P21622_4};	/*Profile ID - 21622*/
const Value V21141_1_1 = {S2, NULL};	/*Value = True*/
const Property P21141_1 = {S50, 0, &V21141_1_1, NULL};	/*Property = IsMobile*/
const Value V21141_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21141_5 = {S52, 0, &V21141_5_1403, &P21141_1};	/*Property = ScreenPixelsHeight*/
const Value V21141_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21141_4 = {S54, 0, &V21141_4_1404, &P21141_5};	/*Property = ScreenPixelsWidth*/
const Profile P21141 = {"21141", &P21141_4};	/*Profile ID - 21141*/
const Value V21712_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_279 = {S1, 0, &V21712_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21712_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_282 = {S3, 0, &V21712_282_57155, &P21712_279};	/*Property = BlobBuilder*/
const Value V21712_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_350 = {S4, 0, &V21712_350_57446, &P21712_282};	/*Property = Canvas*/
const Value V21712_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_283 = {S5, 0, &V21712_283_57161, &P21712_350};	/*Property = CssBackground*/
const Value V21712_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_284 = {S6, 0, &V21712_284_57164, &P21712_283};	/*Property = CssBorderImage*/
const Value V21712_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_285 = {S7, 0, &V21712_285_57167, &P21712_284};	/*Property = CssCanvas*/
const Value V21712_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_286 = {S8, 0, &V21712_286_57170, &P21712_285};	/*Property = CssColor*/
const Value V21712_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_237 = {S9, 0, &V21712_237_55786, &P21712_286};	/*Property = CssColumn*/
const Value V21712_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_287 = {S10, 0, &V21712_287_57173, &P21712_237};	/*Property = CssFlexbox*/
const Value V21712_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_288 = {S11, 0, &V21712_288_57176, &P21712_287};	/*Property = CssFont*/
const Value V21712_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_289 = {S12, 0, &V21712_289_86899, &P21712_288};	/*Property = CssImages*/
const Value V21712_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_291 = {S13, 0, &V21712_291_57182, &P21712_289};	/*Property = CssMediaQueries*/
const Value V21712_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_292 = {S14, 0, &V21712_292_57185, &P21712_291};	/*Property = CssMinMax*/
const Value V21712_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_293 = {S15, 0, &V21712_293_57188, &P21712_292};	/*Property = CssOverflow*/
const Value V21712_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_294 = {S16, 0, &V21712_294_57191, &P21712_293};	/*Property = CssPosition*/
const Value V21712_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_295 = {S17, 0, &V21712_295_57194, &P21712_294};	/*Property = CssText*/
const Value V21712_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_297 = {S18, 0, &V21712_297_57197, &P21712_295};	/*Property = CssTransforms*/
const Value V21712_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_241 = {S19, 0, &V21712_241_55801, &P21712_297};	/*Property = CssTransitions*/
const Value V21712_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_298 = {S20, 0, &V21712_298_57208, &P21712_241};	/*Property = CssUI*/
const Value V21712_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_300 = {S21, 0, &V21712_300_57217, &P21712_298};	/*Property = DataSet*/
const Value V21712_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_301 = {S22, 0, &V21712_301_57220, &P21712_300};	/*Property = DataUrl*/
const Value V21712_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_302 = {S23, 0, &V21712_302_57223, &P21712_301};	/*Property = DeviceOrientation*/
const Value V21712_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_304 = {S24, 0, &V21712_304_57229, &P21712_302};	/*Property = FileReader*/
const Value V21712_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_305 = {S25, 0, &V21712_305_57232, &P21712_304};	/*Property = FileSaver*/
const Value V21712_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_306 = {S26, 0, &V21712_306_57238, &P21712_305};	/*Property = FileWriter*/
const Value V21712_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_307 = {S27, 0, &V21712_307_57241, &P21712_306};	/*Property = FormData*/
const Value V21712_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_309 = {S28, 0, &V21712_309_57247, &P21712_307};	/*Property = Fullscreen*/
const Value V21712_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_310 = {S29, 0, &V21712_310_57250, &P21712_309};	/*Property = GeoLocation*/
const Value V21712_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_312 = {S30, 0, &V21712_312_57256, &P21712_310};	/*Property = History*/
const Value V21712_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_314 = {S31, 0, &V21712_314_57262, &P21712_312};	/*Property = Html5*/
const Value V21712_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_313 = {S32, 0, &V21712_313_57259, &P21712_314};	/*Property = Html-Media-Capture*/
const Value V21712_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_315 = {S33, 0, &V21712_315_57265, &P21712_313};	/*Property = Iframe*/
const Value V21712_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_316 = {S34, 0, &V21712_316_57268, &P21712_315};	/*Property = IndexedDB*/
const Value V21712_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_317 = {S35, 0, &V21712_317_57271, &P21712_316};	/*Property = Json*/
const Value V21712_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21712_15 = {S37, 0, &V21712_15_28, &P21712_317};	/*Property = LayoutEngine*/
const Value V21712_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_344 = {S38, 0, &V21712_344_57274, &P21712_15};	/*Property = Masking*/
const Value V21712_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_320 = {S39, 0, &V21712_320_57292, &P21712_344};	/*Property = PostMessage*/
const Value V21712_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_321 = {S40, 0, &V21712_321_57295, &P21712_320};	/*Property = Progress*/
const Value V21712_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_322 = {S41, 0, &V21712_322_57298, &P21712_321};	/*Property = Prompts*/
const Value V21712_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_323 = {S42, 0, &V21712_323_57301, &P21712_322};	/*Property = Selector*/
const Value V21712_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_324 = {S43, 0, &V21712_324_57307, &P21712_323};	/*Property = Svg*/
const Value V21712_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_327 = {S44, 0, &V21712_327_57316, &P21712_324};	/*Property = TouchEvents*/
const Value V21712_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_328 = {S45, 0, &V21712_328_57319, &P21712_327};	/*Property = Track*/
const Value V21712_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_329 = {S46, 0, &V21712_329_57325, &P21712_328};	/*Property = Video*/
const Value V21712_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_330 = {S47, 0, &V21712_330_57328, &P21712_329};	/*Property = Viewport*/
const Value V21712_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_332 = {S48, 0, &V21712_332_57340, &P21712_330};	/*Property = WebWorkers*/
const Value V21712_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21712_333 = {S49, 0, &V21712_333_57343, &P21712_332};	/*Property = Xhr2*/
const Profile P21712 = {"21712", &P21712_333};	/*Profile ID - 21712*/
const Value V22497_1_1 = {S2, NULL};	/*Value = True*/
const Property P22497_1 = {S50, 0, &V22497_1_1, NULL};	/*Property = IsMobile*/
const Value V22497_5_37526 = {S122, NULL};	/*Value = 1200*/
const Property P22497_5 = {S52, 0, &V22497_5_37526, &P22497_1};	/*Property = ScreenPixelsHeight*/
const Value V22497_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P22497_4 = {S54, 0, &V22497_4_57095, &P22497_5};	/*Property = ScreenPixelsWidth*/
const Profile P22497 = {"22497", &P22497_4};	/*Profile ID - 22497*/
const Value V19807_279_57148 = {S0, NULL};	/*Value = False*/
const Property P19807_279 = {S1, 0, &V19807_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V19807_282_57154 = {S0, NULL};	/*Value = False*/
const Property P19807_282 = {S3, 0, &V19807_282_57154, &P19807_279};	/*Property = BlobBuilder*/
const Value V19807_350_57444 = {S2, NULL};	/*Value = True*/
const Property P19807_350 = {S4, 0, &V19807_350_57444, &P19807_282};	/*Property = Canvas*/
const Value V19807_283_57159 = {S2, NULL};	/*Value = True*/
const Property P19807_283 = {S5, 0, &V19807_283_57159, &P19807_350};	/*Property = CssBackground*/
const Value V19807_284_57162 = {S2, NULL};	/*Value = True*/
const Property P19807_284 = {S6, 0, &V19807_284_57162, &P19807_283};	/*Property = CssBorderImage*/
const Value V19807_285_57166 = {S0, NULL};	/*Value = False*/
const Property P19807_285 = {S7, 0, &V19807_285_57166, &P19807_284};	/*Property = CssCanvas*/
const Value V19807_286_57168 = {S2, NULL};	/*Value = True*/
const Property P19807_286 = {S8, 0, &V19807_286_57168, &P19807_285};	/*Property = CssColor*/
const Value V19807_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P19807_237 = {S9, 0, &V19807_237_55786, &P19807_286};	/*Property = CssColumn*/
const Value V19807_287_57172 = {S0, NULL};	/*Value = False*/
const Property P19807_287 = {S10, 0, &V19807_287_57172, &P19807_237};	/*Property = CssFlexbox*/
const Value V19807_288_57175 = {S0, NULL};	/*Value = False*/
const Property P19807_288 = {S11, 0, &V19807_288_57175, &P19807_287};	/*Property = CssFont*/
const Value V19807_289_57759 = {S0, NULL};	/*Value = False*/
const Property P19807_289 = {S12, 0, &V19807_289_57759, &P19807_288};	/*Property = CssImages*/
const Value V19807_291_57180 = {S2, NULL};	/*Value = True*/
const Property P19807_291 = {S13, 0, &V19807_291_57180, &P19807_289};	/*Property = CssMediaQueries*/
const Value V19807_292_57183 = {S2, NULL};	/*Value = True*/
const Property P19807_292 = {S14, 0, &V19807_292_57183, &P19807_291};	/*Property = CssMinMax*/
const Value V19807_293_57187 = {S0, NULL};	/*Value = False*/
const Property P19807_293 = {S15, 0, &V19807_293_57187, &P19807_292};	/*Property = CssOverflow*/
const Value V19807_294_57189 = {S2, NULL};	/*Value = True*/
const Property P19807_294 = {S16, 0, &V19807_294_57189, &P19807_293};	/*Property = CssPosition*/
const Value V19807_295_57192 = {S2, NULL};	/*Value = True*/
const Property P19807_295 = {S17, 0, &V19807_295_57192, &P19807_294};	/*Property = CssText*/
const Value V19807_297_57195 = {S2, NULL};	/*Value = True*/
const Property P19807_297 = {S18, 0, &V19807_297_57195, &P19807_295};	/*Property = CssTransforms*/
const Value V19807_241_55799 = {S2, NULL};	/*Value = True*/
const Property P19807_241 = {S19, 0, &V19807_241_55799, &P19807_297};	/*Property = CssTransitions*/
const Value V19807_298_57206 = {S2, NULL};	/*Value = True*/
const Property P19807_298 = {S20, 0, &V19807_298_57206, &P19807_241};	/*Property = CssUI*/
const Value V19807_300_57215 = {S2, NULL};	/*Value = True*/
const Property P19807_300 = {S21, 0, &V19807_300_57215, &P19807_298};	/*Property = DataSet*/
const Value V19807_301_57218 = {S2, NULL};	/*Value = True*/
const Property P19807_301 = {S22, 0, &V19807_301_57218, &P19807_300};	/*Property = DataUrl*/
const Value V19807_302_57221 = {S2, NULL};	/*Value = True*/
const Property P19807_302 = {S23, 0, &V19807_302_57221, &P19807_301};	/*Property = DeviceOrientation*/
const Value V19807_304_57227 = {S2, NULL};	/*Value = True*/
const Property P19807_304 = {S24, 0, &V19807_304_57227, &P19807_302};	/*Property = FileReader*/
const Value V19807_305_57231 = {S0, NULL};	/*Value = False*/
const Property P19807_305 = {S25, 0, &V19807_305_57231, &P19807_304};	/*Property = FileSaver*/
const Value V19807_306_57237 = {S0, NULL};	/*Value = False*/
const Property P19807_306 = {S26, 0, &V19807_306_57237, &P19807_305};	/*Property = FileWriter*/
const Value V19807_307_57239 = {S2, NULL};	/*Value = True*/
const Property P19807_307 = {S27, 0, &V19807_307_57239, &P19807_306};	/*Property = FormData*/
const Value V19807_309_57246 = {S0, NULL};	/*Value = False*/
const Property P19807_309 = {S28, 0, &V19807_309_57246, &P19807_307};	/*Property = Fullscreen*/
const Value V19807_310_57248 = {S2, NULL};	/*Value = True*/
const Property P19807_310 = {S29, 0, &V19807_310_57248, &P19807_309};	/*Property = GeoLocation*/
const Value V19807_312_57254 = {S2, NULL};	/*Value = True*/
const Property P19807_312 = {S30, 0, &V19807_312_57254, &P19807_310};	/*Property = History*/
const Value V19807_314_57261 = {S0, NULL};	/*Value = False*/
const Property P19807_314 = {S31, 0, &V19807_314_57261, &P19807_312};	/*Property = Html5*/
const Value V19807_313_57258 = {S0, NULL};	/*Value = False*/
const Property P19807_313 = {S32, 0, &V19807_313_57258, &P19807_314};	/*Property = Html-Media-Capture*/
const Value V19807_315_57264 = {S0, NULL};	/*Value = False*/
const Property P19807_315 = {S33, 0, &V19807_315_57264, &P19807_313};	/*Property = Iframe*/
const Value V19807_316_57267 = {S0, NULL};	/*Value = False*/
const Property P19807_316 = {S34, 0, &V19807_316_57267, &P19807_315};	/*Property = IndexedDB*/
const Value V19807_317_57269 = {S2, NULL};	/*Value = True*/
const Property P19807_317 = {S35, 0, &V19807_317_57269, &P19807_316};	/*Property = Json*/
const Value V19807_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P19807_15 = {S37, 0, &V19807_15_28, &P19807_317};	/*Property = LayoutEngine*/
const Value V19807_344_57272 = {S2, NULL};	/*Value = True*/
const Property P19807_344 = {S38, 0, &V19807_344_57272, &P19807_15};	/*Property = Masking*/
const Value V19807_320_57290 = {S2, NULL};	/*Value = True*/
const Property P19807_320 = {S39, 0, &V19807_320_57290, &P19807_344};	/*Property = PostMessage*/
const Value V19807_321_57293 = {S2, NULL};	/*Value = True*/
const Property P19807_321 = {S40, 0, &V19807_321_57293, &P19807_320};	/*Property = Progress*/
const Value V19807_322_57296 = {S2, NULL};	/*Value = True*/
const Property P19807_322 = {S41, 0, &V19807_322_57296, &P19807_321};	/*Property = Prompts*/
const Value V19807_323_57299 = {S2, NULL};	/*Value = True*/
const Property P19807_323 = {S42, 0, &V19807_323_57299, &P19807_322};	/*Property = Selector*/
const Value V19807_324_57306 = {S0, NULL};	/*Value = False*/
const Property P19807_324 = {S43, 0, &V19807_324_57306, &P19807_323};	/*Property = Svg*/
const Value V19807_327_57315 = {S0, NULL};	/*Value = False*/
const Property P19807_327 = {S44, 0, &V19807_327_57315, &P19807_324};	/*Property = TouchEvents*/
const Value V19807_328_57318 = {S0, NULL};	/*Value = False*/
const Property P19807_328 = {S45, 0, &V19807_328_57318, &P19807_327};	/*Property = Track*/
const Value V19807_329_57323 = {S2, NULL};	/*Value = True*/
const Property P19807_329 = {S46, 0, &V19807_329_57323, &P19807_328};	/*Property = Video*/
const Value V19807_330_57326 = {S2, NULL};	/*Value = True*/
const Property P19807_330 = {S47, 0, &V19807_330_57326, &P19807_329};	/*Property = Viewport*/
const Value V19807_332_57339 = {S0, NULL};	/*Value = False*/
const Property P19807_332 = {S48, 0, &V19807_332_57339, &P19807_330};	/*Property = WebWorkers*/
const Value V19807_333_57342 = {S0, NULL};	/*Value = False*/
const Property P19807_333 = {S49, 0, &V19807_333_57342, &P19807_332};	/*Property = Xhr2*/
const Profile P19807 = {"19807", &P19807_333};	/*Profile ID - 19807*/
const Value V23347_1_1 = {S2, NULL};	/*Value = True*/
const Property P23347_1 = {S50, 0, &V23347_1_1, NULL};	/*Property = IsMobile*/
const Value V23347_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23347_5 = {S52, 0, &V23347_5_37569, &P23347_1};	/*Property = ScreenPixelsHeight*/
const Value V23347_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23347_4 = {S54, 0, &V23347_4_5645, &P23347_5};	/*Property = ScreenPixelsWidth*/
const Profile P23347 = {"23347", &P23347_4};	/*Profile ID - 23347*/
const Value V20824_1_1 = {S2, NULL};	/*Value = True*/
const Property P20824_1 = {S50, 0, &V20824_1_1, NULL};	/*Property = IsMobile*/
const Value V20824_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20824_5 = {S52, 0, &V20824_5_9981, &P20824_1};	/*Property = ScreenPixelsHeight*/
const Value V20824_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20824_4 = {S54, 0, &V20824_4_2336, &P20824_5};	/*Property = ScreenPixelsWidth*/
const Profile P20824 = {"20824", &P20824_4};	/*Profile ID - 20824*/
const Value V21853_1_1 = {S2, NULL};	/*Value = True*/
const Property P21853_1 = {S50, 0, &V21853_1_1, NULL};	/*Property = IsMobile*/
const Value V21853_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P21853_5 = {S52, 0, &V21853_5_1832, &P21853_1};	/*Property = ScreenPixelsHeight*/
const Value V21853_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21853_4 = {S54, 0, &V21853_4_5645, &P21853_5};	/*Property = ScreenPixelsWidth*/
const Profile P21853 = {"21853", &P21853_4};	/*Profile ID - 21853*/
const Value V21977_1_1 = {S2, NULL};	/*Value = True*/
const Property P21977_1 = {S50, 0, &V21977_1_1, NULL};	/*Property = IsMobile*/
const Value V21977_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P21977_5 = {S52, 0, &V21977_5_1832, &P21977_1};	/*Property = ScreenPixelsHeight*/
const Value V21977_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21977_4 = {S54, 0, &V21977_4_5645, &P21977_5};	/*Property = ScreenPixelsWidth*/
const Profile P21977 = {"21977", &P21977_4};	/*Profile ID - 21977*/
const Value V20837_1_1 = {S2, NULL};	/*Value = True*/
const Property P20837_1 = {S50, 0, &V20837_1_1, NULL};	/*Property = IsMobile*/
const Value V20837_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20837_5 = {S52, 0, &V20837_5_1227, &P20837_1};	/*Property = ScreenPixelsHeight*/
const Value V20837_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20837_4 = {S54, 0, &V20837_4_1229, &P20837_5};	/*Property = ScreenPixelsWidth*/
const Profile P20837 = {"20837", &P20837_4};	/*Profile ID - 20837*/
const Value V22174_1_1 = {S2, NULL};	/*Value = True*/
const Property P22174_1 = {S50, 0, &V22174_1_1, NULL};	/*Property = IsMobile*/
const Value V22174_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22174_5 = {S52, 0, &V22174_5_1227, &P22174_1};	/*Property = ScreenPixelsHeight*/
const Value V22174_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22174_4 = {S54, 0, &V22174_4_1229, &P22174_5};	/*Property = ScreenPixelsWidth*/
const Profile P22174 = {"22174", &P22174_4};	/*Profile ID - 22174*/
const Value V21351_1_1 = {S2, NULL};	/*Value = True*/
const Property P21351_1 = {S50, 0, &V21351_1_1, NULL};	/*Property = IsMobile*/
const Value V21351_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21351_5 = {S52, 0, &V21351_5_1269, &P21351_1};	/*Property = ScreenPixelsHeight*/
const Value V21351_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21351_4 = {S54, 0, &V21351_4_1229, &P21351_5};	/*Property = ScreenPixelsWidth*/
const Profile P21351 = {"21351", &P21351_4};	/*Profile ID - 21351*/
const Value V21349_1_1 = {S2, NULL};	/*Value = True*/
const Property P21349_1 = {S50, 0, &V21349_1_1, NULL};	/*Property = IsMobile*/
const Value V21349_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21349_5 = {S52, 0, &V21349_5_1277, &P21349_1};	/*Property = ScreenPixelsHeight*/
const Value V21349_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21349_4 = {S54, 0, &V21349_4_1279, &P21349_5};	/*Property = ScreenPixelsWidth*/
const Profile P21349 = {"21349", &P21349_4};	/*Profile ID - 21349*/
const Value V21507_1_1 = {S2, NULL};	/*Value = True*/
const Property P21507_1 = {S50, 0, &V21507_1_1, NULL};	/*Property = IsMobile*/
const Value V21507_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21507_5 = {S52, 0, &V21507_5_1277, &P21507_1};	/*Property = ScreenPixelsHeight*/
const Value V21507_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21507_4 = {S54, 0, &V21507_4_1279, &P21507_5};	/*Property = ScreenPixelsWidth*/
const Profile P21507 = {"21507", &P21507_4};	/*Profile ID - 21507*/
const Value V20517_1_1 = {S2, NULL};	/*Value = True*/
const Property P20517_1 = {S50, 0, &V20517_1_1, NULL};	/*Property = IsMobile*/
const Value V20517_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20517_5 = {S52, 0, &V20517_5_1403, &P20517_1};	/*Property = ScreenPixelsHeight*/
const Value V20517_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20517_4 = {S54, 0, &V20517_4_1404, &P20517_5};	/*Property = ScreenPixelsWidth*/
const Profile P20517 = {"20517", &P20517_4};	/*Profile ID - 20517*/
const Value V21019_1_1 = {S2, NULL};	/*Value = True*/
const Property P21019_1 = {S50, 0, &V21019_1_1, NULL};	/*Property = IsMobile*/
const Value V21019_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21019_5 = {S52, 0, &V21019_5_1227, &P21019_1};	/*Property = ScreenPixelsHeight*/
const Value V21019_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21019_4 = {S54, 0, &V21019_4_1229, &P21019_5};	/*Property = ScreenPixelsWidth*/
const Profile P21019 = {"21019", &P21019_4};	/*Profile ID - 21019*/
const Value V20770_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_279 = {S1, 0, &V20770_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20770_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_282 = {S3, 0, &V20770_282_57155, &P20770_279};	/*Property = BlobBuilder*/
const Value V20770_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_350 = {S4, 0, &V20770_350_57446, &P20770_282};	/*Property = Canvas*/
const Value V20770_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_283 = {S5, 0, &V20770_283_57161, &P20770_350};	/*Property = CssBackground*/
const Value V20770_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_284 = {S6, 0, &V20770_284_57164, &P20770_283};	/*Property = CssBorderImage*/
const Value V20770_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_285 = {S7, 0, &V20770_285_57167, &P20770_284};	/*Property = CssCanvas*/
const Value V20770_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_286 = {S8, 0, &V20770_286_57170, &P20770_285};	/*Property = CssColor*/
const Value V20770_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_237 = {S9, 0, &V20770_237_55786, &P20770_286};	/*Property = CssColumn*/
const Value V20770_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_287 = {S10, 0, &V20770_287_57173, &P20770_237};	/*Property = CssFlexbox*/
const Value V20770_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_288 = {S11, 0, &V20770_288_57176, &P20770_287};	/*Property = CssFont*/
const Value V20770_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_289 = {S12, 0, &V20770_289_86899, &P20770_288};	/*Property = CssImages*/
const Value V20770_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_291 = {S13, 0, &V20770_291_57182, &P20770_289};	/*Property = CssMediaQueries*/
const Value V20770_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_292 = {S14, 0, &V20770_292_57185, &P20770_291};	/*Property = CssMinMax*/
const Value V20770_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_293 = {S15, 0, &V20770_293_57188, &P20770_292};	/*Property = CssOverflow*/
const Value V20770_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_294 = {S16, 0, &V20770_294_57191, &P20770_293};	/*Property = CssPosition*/
const Value V20770_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_295 = {S17, 0, &V20770_295_57194, &P20770_294};	/*Property = CssText*/
const Value V20770_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_297 = {S18, 0, &V20770_297_57197, &P20770_295};	/*Property = CssTransforms*/
const Value V20770_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_241 = {S19, 0, &V20770_241_55801, &P20770_297};	/*Property = CssTransitions*/
const Value V20770_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_298 = {S20, 0, &V20770_298_57208, &P20770_241};	/*Property = CssUI*/
const Value V20770_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_300 = {S21, 0, &V20770_300_57217, &P20770_298};	/*Property = DataSet*/
const Value V20770_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_301 = {S22, 0, &V20770_301_57220, &P20770_300};	/*Property = DataUrl*/
const Value V20770_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_302 = {S23, 0, &V20770_302_57223, &P20770_301};	/*Property = DeviceOrientation*/
const Value V20770_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_304 = {S24, 0, &V20770_304_57229, &P20770_302};	/*Property = FileReader*/
const Value V20770_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_305 = {S25, 0, &V20770_305_57232, &P20770_304};	/*Property = FileSaver*/
const Value V20770_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_306 = {S26, 0, &V20770_306_57238, &P20770_305};	/*Property = FileWriter*/
const Value V20770_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_307 = {S27, 0, &V20770_307_57241, &P20770_306};	/*Property = FormData*/
const Value V20770_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_309 = {S28, 0, &V20770_309_57247, &P20770_307};	/*Property = Fullscreen*/
const Value V20770_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_310 = {S29, 0, &V20770_310_57250, &P20770_309};	/*Property = GeoLocation*/
const Value V20770_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_312 = {S30, 0, &V20770_312_57256, &P20770_310};	/*Property = History*/
const Value V20770_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_314 = {S31, 0, &V20770_314_57262, &P20770_312};	/*Property = Html5*/
const Value V20770_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_313 = {S32, 0, &V20770_313_57259, &P20770_314};	/*Property = Html-Media-Capture*/
const Value V20770_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_315 = {S33, 0, &V20770_315_57265, &P20770_313};	/*Property = Iframe*/
const Value V20770_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_316 = {S34, 0, &V20770_316_57268, &P20770_315};	/*Property = IndexedDB*/
const Value V20770_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_317 = {S35, 0, &V20770_317_57271, &P20770_316};	/*Property = Json*/
const Value V20770_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P20770_15 = {S37, 0, &V20770_15_90837, &P20770_317};	/*Property = LayoutEngine*/
const Value V20770_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_344 = {S38, 0, &V20770_344_57274, &P20770_15};	/*Property = Masking*/
const Value V20770_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_320 = {S39, 0, &V20770_320_57292, &P20770_344};	/*Property = PostMessage*/
const Value V20770_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_321 = {S40, 0, &V20770_321_57295, &P20770_320};	/*Property = Progress*/
const Value V20770_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_322 = {S41, 0, &V20770_322_57298, &P20770_321};	/*Property = Prompts*/
const Value V20770_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_323 = {S42, 0, &V20770_323_57301, &P20770_322};	/*Property = Selector*/
const Value V20770_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_324 = {S43, 0, &V20770_324_57307, &P20770_323};	/*Property = Svg*/
const Value V20770_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_327 = {S44, 0, &V20770_327_57316, &P20770_324};	/*Property = TouchEvents*/
const Value V20770_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_328 = {S45, 0, &V20770_328_57319, &P20770_327};	/*Property = Track*/
const Value V20770_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_329 = {S46, 0, &V20770_329_57325, &P20770_328};	/*Property = Video*/
const Value V20770_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_330 = {S47, 0, &V20770_330_57328, &P20770_329};	/*Property = Viewport*/
const Value V20770_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_332 = {S48, 0, &V20770_332_57340, &P20770_330};	/*Property = WebWorkers*/
const Value V20770_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20770_333 = {S49, 0, &V20770_333_57343, &P20770_332};	/*Property = Xhr2*/
const Profile P20770 = {"20770", &P20770_333};	/*Profile ID - 20770*/
const Value V21449_1_1 = {S2, NULL};	/*Value = True*/
const Property P21449_1 = {S50, 0, &V21449_1_1, NULL};	/*Property = IsMobile*/
const Value V21449_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21449_5 = {S52, 0, &V21449_5_9981, &P21449_1};	/*Property = ScreenPixelsHeight*/
const Value V21449_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21449_4 = {S54, 0, &V21449_4_39098, &P21449_5};	/*Property = ScreenPixelsWidth*/
const Profile P21449 = {"21449", &P21449_4};	/*Profile ID - 21449*/
const Value V21586_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_279 = {S1, 0, &V21586_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21586_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_282 = {S3, 0, &V21586_282_57155, &P21586_279};	/*Property = BlobBuilder*/
const Value V21586_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_350 = {S4, 0, &V21586_350_57446, &P21586_282};	/*Property = Canvas*/
const Value V21586_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_283 = {S5, 0, &V21586_283_57161, &P21586_350};	/*Property = CssBackground*/
const Value V21586_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_284 = {S6, 0, &V21586_284_57164, &P21586_283};	/*Property = CssBorderImage*/
const Value V21586_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_285 = {S7, 0, &V21586_285_57167, &P21586_284};	/*Property = CssCanvas*/
const Value V21586_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_286 = {S8, 0, &V21586_286_57170, &P21586_285};	/*Property = CssColor*/
const Value V21586_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_237 = {S9, 0, &V21586_237_55786, &P21586_286};	/*Property = CssColumn*/
const Value V21586_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_287 = {S10, 0, &V21586_287_57173, &P21586_237};	/*Property = CssFlexbox*/
const Value V21586_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_288 = {S11, 0, &V21586_288_57176, &P21586_287};	/*Property = CssFont*/
const Value V21586_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_289 = {S12, 0, &V21586_289_86899, &P21586_288};	/*Property = CssImages*/
const Value V21586_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_291 = {S13, 0, &V21586_291_57182, &P21586_289};	/*Property = CssMediaQueries*/
const Value V21586_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_292 = {S14, 0, &V21586_292_57185, &P21586_291};	/*Property = CssMinMax*/
const Value V21586_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_293 = {S15, 0, &V21586_293_57188, &P21586_292};	/*Property = CssOverflow*/
const Value V21586_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_294 = {S16, 0, &V21586_294_57191, &P21586_293};	/*Property = CssPosition*/
const Value V21586_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_295 = {S17, 0, &V21586_295_57194, &P21586_294};	/*Property = CssText*/
const Value V21586_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_297 = {S18, 0, &V21586_297_57197, &P21586_295};	/*Property = CssTransforms*/
const Value V21586_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_241 = {S19, 0, &V21586_241_55801, &P21586_297};	/*Property = CssTransitions*/
const Value V21586_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_298 = {S20, 0, &V21586_298_57208, &P21586_241};	/*Property = CssUI*/
const Value V21586_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_300 = {S21, 0, &V21586_300_57217, &P21586_298};	/*Property = DataSet*/
const Value V21586_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_301 = {S22, 0, &V21586_301_57220, &P21586_300};	/*Property = DataUrl*/
const Value V21586_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_302 = {S23, 0, &V21586_302_57223, &P21586_301};	/*Property = DeviceOrientation*/
const Value V21586_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_304 = {S24, 0, &V21586_304_57229, &P21586_302};	/*Property = FileReader*/
const Value V21586_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_305 = {S25, 0, &V21586_305_57232, &P21586_304};	/*Property = FileSaver*/
const Value V21586_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_306 = {S26, 0, &V21586_306_57238, &P21586_305};	/*Property = FileWriter*/
const Value V21586_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_307 = {S27, 0, &V21586_307_57241, &P21586_306};	/*Property = FormData*/
const Value V21586_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_309 = {S28, 0, &V21586_309_57247, &P21586_307};	/*Property = Fullscreen*/
const Value V21586_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_310 = {S29, 0, &V21586_310_57250, &P21586_309};	/*Property = GeoLocation*/
const Value V21586_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_312 = {S30, 0, &V21586_312_57256, &P21586_310};	/*Property = History*/
const Value V21586_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_314 = {S31, 0, &V21586_314_57262, &P21586_312};	/*Property = Html5*/
const Value V21586_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_313 = {S32, 0, &V21586_313_57259, &P21586_314};	/*Property = Html-Media-Capture*/
const Value V21586_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_315 = {S33, 0, &V21586_315_57265, &P21586_313};	/*Property = Iframe*/
const Value V21586_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_316 = {S34, 0, &V21586_316_57268, &P21586_315};	/*Property = IndexedDB*/
const Value V21586_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_317 = {S35, 0, &V21586_317_57271, &P21586_316};	/*Property = Json*/
const Value V21586_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_15 = {S37, 0, &V21586_15_72, &P21586_317};	/*Property = LayoutEngine*/
const Value V21586_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_344 = {S38, 0, &V21586_344_57274, &P21586_15};	/*Property = Masking*/
const Value V21586_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_320 = {S39, 0, &V21586_320_57292, &P21586_344};	/*Property = PostMessage*/
const Value V21586_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_321 = {S40, 0, &V21586_321_57295, &P21586_320};	/*Property = Progress*/
const Value V21586_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_322 = {S41, 0, &V21586_322_57298, &P21586_321};	/*Property = Prompts*/
const Value V21586_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_323 = {S42, 0, &V21586_323_57301, &P21586_322};	/*Property = Selector*/
const Value V21586_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_324 = {S43, 0, &V21586_324_57307, &P21586_323};	/*Property = Svg*/
const Value V21586_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_327 = {S44, 0, &V21586_327_57316, &P21586_324};	/*Property = TouchEvents*/
const Value V21586_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_328 = {S45, 0, &V21586_328_57319, &P21586_327};	/*Property = Track*/
const Value V21586_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_329 = {S46, 0, &V21586_329_57325, &P21586_328};	/*Property = Video*/
const Value V21586_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_330 = {S47, 0, &V21586_330_57328, &P21586_329};	/*Property = Viewport*/
const Value V21586_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_332 = {S48, 0, &V21586_332_57340, &P21586_330};	/*Property = WebWorkers*/
const Value V21586_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21586_333 = {S49, 0, &V21586_333_57343, &P21586_332};	/*Property = Xhr2*/
const Profile P21586 = {"21586", &P21586_333};	/*Profile ID - 21586*/
const Value V20820_1_1 = {S2, NULL};	/*Value = True*/
const Property P20820_1 = {S50, 0, &V20820_1_1, NULL};	/*Property = IsMobile*/
const Value V20820_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20820_5 = {S52, 0, &V20820_5_1458, &P20820_1};	/*Property = ScreenPixelsHeight*/
const Value V20820_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20820_4 = {S54, 0, &V20820_4_37617, &P20820_5};	/*Property = ScreenPixelsWidth*/
const Profile P20820 = {"20820", &P20820_4};	/*Profile ID - 20820*/
const Value V20529_1_1 = {S2, NULL};	/*Value = True*/
const Property P20529_1 = {S50, 0, &V20529_1_1, NULL};	/*Property = IsMobile*/
const Value V20529_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20529_5 = {S52, 0, &V20529_5_1227, &P20529_1};	/*Property = ScreenPixelsHeight*/
const Value V20529_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20529_4 = {S54, 0, &V20529_4_1229, &P20529_5};	/*Property = ScreenPixelsWidth*/
const Profile P20529 = {"20529", &P20529_4};	/*Profile ID - 20529*/
const Value V21172_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21172_279 = {S1, 0, &V21172_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21172_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21172_282 = {S3, 0, &V21172_282_57154, &P21172_279};	/*Property = BlobBuilder*/
const Value V21172_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21172_350 = {S4, 0, &V21172_350_57444, &P21172_282};	/*Property = Canvas*/
const Value V21172_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21172_283 = {S5, 0, &V21172_283_57159, &P21172_350};	/*Property = CssBackground*/
const Value V21172_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21172_284 = {S6, 0, &V21172_284_57163, &P21172_283};	/*Property = CssBorderImage*/
const Value V21172_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21172_285 = {S7, 0, &V21172_285_57166, &P21172_284};	/*Property = CssCanvas*/
const Value V21172_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21172_286 = {S8, 0, &V21172_286_57168, &P21172_285};	/*Property = CssColor*/
const Value V21172_237_55784 = {S2, NULL};	/*Value = True*/
const Property P21172_237 = {S9, 0, &V21172_237_55784, &P21172_286};	/*Property = CssColumn*/
const Value V21172_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21172_287 = {S10, 0, &V21172_287_57172, &P21172_237};	/*Property = CssFlexbox*/
const Value V21172_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21172_288 = {S11, 0, &V21172_288_57175, &P21172_287};	/*Property = CssFont*/
const Value V21172_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21172_289 = {S12, 0, &V21172_289_57759, &P21172_288};	/*Property = CssImages*/
const Value V21172_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21172_291 = {S13, 0, &V21172_291_57181, &P21172_289};	/*Property = CssMediaQueries*/
const Value V21172_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21172_292 = {S14, 0, &V21172_292_57183, &P21172_291};	/*Property = CssMinMax*/
const Value V21172_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21172_293 = {S15, 0, &V21172_293_57187, &P21172_292};	/*Property = CssOverflow*/
const Value V21172_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21172_294 = {S16, 0, &V21172_294_57190, &P21172_293};	/*Property = CssPosition*/
const Value V21172_295_57193 = {S0, NULL};	/*Value = False*/
const Property P21172_295 = {S17, 0, &V21172_295_57193, &P21172_294};	/*Property = CssText*/
const Value V21172_297_57196 = {S0, NULL};	/*Value = False*/
const Property P21172_297 = {S18, 0, &V21172_297_57196, &P21172_295};	/*Property = CssTransforms*/
const Value V21172_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21172_241 = {S19, 0, &V21172_241_55799, &P21172_297};	/*Property = CssTransitions*/
const Value V21172_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21172_298 = {S20, 0, &V21172_298_57206, &P21172_241};	/*Property = CssUI*/
const Value V21172_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21172_300 = {S21, 0, &V21172_300_57216, &P21172_298};	/*Property = DataSet*/
const Value V21172_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21172_301 = {S22, 0, &V21172_301_57218, &P21172_300};	/*Property = DataUrl*/
const Value V21172_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21172_302 = {S23, 0, &V21172_302_57222, &P21172_301};	/*Property = DeviceOrientation*/
const Value V21172_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21172_304 = {S24, 0, &V21172_304_57228, &P21172_302};	/*Property = FileReader*/
const Value V21172_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21172_305 = {S25, 0, &V21172_305_57231, &P21172_304};	/*Property = FileSaver*/
const Value V21172_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21172_306 = {S26, 0, &V21172_306_57237, &P21172_305};	/*Property = FileWriter*/
const Value V21172_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21172_307 = {S27, 0, &V21172_307_57240, &P21172_306};	/*Property = FormData*/
const Value V21172_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21172_309 = {S28, 0, &V21172_309_57246, &P21172_307};	/*Property = Fullscreen*/
const Value V21172_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21172_310 = {S29, 0, &V21172_310_57248, &P21172_309};	/*Property = GeoLocation*/
const Value V21172_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21172_312 = {S30, 0, &V21172_312_57255, &P21172_310};	/*Property = History*/
const Value V21172_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21172_314 = {S31, 0, &V21172_314_57261, &P21172_312};	/*Property = Html5*/
const Value V21172_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21172_313 = {S32, 0, &V21172_313_57258, &P21172_314};	/*Property = Html-Media-Capture*/
const Value V21172_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21172_315 = {S33, 0, &V21172_315_57264, &P21172_313};	/*Property = Iframe*/
const Value V21172_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21172_316 = {S34, 0, &V21172_316_57267, &P21172_315};	/*Property = IndexedDB*/
const Value V21172_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21172_317 = {S35, 0, &V21172_317_57269, &P21172_316};	/*Property = Json*/
const Value V21172_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P21172_15 = {S37, 0, &V21172_15_31, &P21172_317};	/*Property = LayoutEngine*/
const Value V21172_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21172_344 = {S38, 0, &V21172_344_57272, &P21172_15};	/*Property = Masking*/
const Value V21172_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21172_320 = {S39, 0, &V21172_320_57290, &P21172_344};	/*Property = PostMessage*/
const Value V21172_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21172_321 = {S40, 0, &V21172_321_57293, &P21172_320};	/*Property = Progress*/
const Value V21172_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21172_322 = {S41, 0, &V21172_322_57296, &P21172_321};	/*Property = Prompts*/
const Value V21172_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21172_323 = {S42, 0, &V21172_323_57299, &P21172_322};	/*Property = Selector*/
const Value V21172_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21172_324 = {S43, 0, &V21172_324_57306, &P21172_323};	/*Property = Svg*/
const Value V21172_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21172_327 = {S44, 0, &V21172_327_57315, &P21172_324};	/*Property = TouchEvents*/
const Value V21172_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21172_328 = {S45, 0, &V21172_328_57318, &P21172_327};	/*Property = Track*/
const Value V21172_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21172_329 = {S46, 0, &V21172_329_57323, &P21172_328};	/*Property = Video*/
const Value V21172_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21172_330 = {S47, 0, &V21172_330_57326, &P21172_329};	/*Property = Viewport*/
const Value V21172_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21172_332 = {S48, 0, &V21172_332_57339, &P21172_330};	/*Property = WebWorkers*/
const Value V21172_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21172_333 = {S49, 0, &V21172_333_57342, &P21172_332};	/*Property = Xhr2*/
const Profile P21172 = {"21172", &P21172_333};	/*Profile ID - 21172*/
const Value V20865_1_1 = {S2, NULL};	/*Value = True*/
const Property P20865_1 = {S50, 0, &V20865_1_1, NULL};	/*Property = IsMobile*/
const Value V20865_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20865_5 = {S52, 0, &V20865_5_1227, &P20865_1};	/*Property = ScreenPixelsHeight*/
const Value V20865_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20865_4 = {S54, 0, &V20865_4_1229, &P20865_5};	/*Property = ScreenPixelsWidth*/
const Profile P20865 = {"20865", &P20865_4};	/*Profile ID - 20865*/
const Value V21833_1_1 = {S2, NULL};	/*Value = True*/
const Property P21833_1 = {S50, 0, &V21833_1_1, NULL};	/*Property = IsMobile*/
const Value V21833_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21833_5 = {S52, 0, &V21833_5_1277, &P21833_1};	/*Property = ScreenPixelsHeight*/
const Value V21833_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21833_4 = {S54, 0, &V21833_4_1279, &P21833_5};	/*Property = ScreenPixelsWidth*/
const Profile P21833 = {"21833", &P21833_4};	/*Profile ID - 21833*/
const Value V21152_1_1 = {S2, NULL};	/*Value = True*/
const Property P21152_1 = {S50, 0, &V21152_1_1, NULL};	/*Property = IsMobile*/
const Value V21152_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21152_5 = {S52, 0, &V21152_5_1222, &P21152_1};	/*Property = ScreenPixelsHeight*/
const Value V21152_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21152_4 = {S54, 0, &V21152_4_1224, &P21152_5};	/*Property = ScreenPixelsWidth*/
const Profile P21152 = {"21152", &P21152_4};	/*Profile ID - 21152*/
const Value V20916_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_279 = {S1, 0, &V20916_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20916_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_282 = {S3, 0, &V20916_282_57155, &P20916_279};	/*Property = BlobBuilder*/
const Value V20916_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_350 = {S4, 0, &V20916_350_57446, &P20916_282};	/*Property = Canvas*/
const Value V20916_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_283 = {S5, 0, &V20916_283_57161, &P20916_350};	/*Property = CssBackground*/
const Value V20916_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_284 = {S6, 0, &V20916_284_57164, &P20916_283};	/*Property = CssBorderImage*/
const Value V20916_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_285 = {S7, 0, &V20916_285_57167, &P20916_284};	/*Property = CssCanvas*/
const Value V20916_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_286 = {S8, 0, &V20916_286_57170, &P20916_285};	/*Property = CssColor*/
const Value V20916_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_237 = {S9, 0, &V20916_237_55786, &P20916_286};	/*Property = CssColumn*/
const Value V20916_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_287 = {S10, 0, &V20916_287_57173, &P20916_237};	/*Property = CssFlexbox*/
const Value V20916_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_288 = {S11, 0, &V20916_288_57176, &P20916_287};	/*Property = CssFont*/
const Value V20916_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_289 = {S12, 0, &V20916_289_86899, &P20916_288};	/*Property = CssImages*/
const Value V20916_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_291 = {S13, 0, &V20916_291_57182, &P20916_289};	/*Property = CssMediaQueries*/
const Value V20916_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_292 = {S14, 0, &V20916_292_57185, &P20916_291};	/*Property = CssMinMax*/
const Value V20916_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_293 = {S15, 0, &V20916_293_57188, &P20916_292};	/*Property = CssOverflow*/
const Value V20916_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_294 = {S16, 0, &V20916_294_57191, &P20916_293};	/*Property = CssPosition*/
const Value V20916_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_295 = {S17, 0, &V20916_295_57194, &P20916_294};	/*Property = CssText*/
const Value V20916_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_297 = {S18, 0, &V20916_297_57197, &P20916_295};	/*Property = CssTransforms*/
const Value V20916_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_241 = {S19, 0, &V20916_241_55801, &P20916_297};	/*Property = CssTransitions*/
const Value V20916_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_298 = {S20, 0, &V20916_298_57208, &P20916_241};	/*Property = CssUI*/
const Value V20916_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_300 = {S21, 0, &V20916_300_57217, &P20916_298};	/*Property = DataSet*/
const Value V20916_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_301 = {S22, 0, &V20916_301_57220, &P20916_300};	/*Property = DataUrl*/
const Value V20916_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_302 = {S23, 0, &V20916_302_57223, &P20916_301};	/*Property = DeviceOrientation*/
const Value V20916_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_304 = {S24, 0, &V20916_304_57229, &P20916_302};	/*Property = FileReader*/
const Value V20916_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_305 = {S25, 0, &V20916_305_57232, &P20916_304};	/*Property = FileSaver*/
const Value V20916_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_306 = {S26, 0, &V20916_306_57238, &P20916_305};	/*Property = FileWriter*/
const Value V20916_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_307 = {S27, 0, &V20916_307_57241, &P20916_306};	/*Property = FormData*/
const Value V20916_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_309 = {S28, 0, &V20916_309_57247, &P20916_307};	/*Property = Fullscreen*/
const Value V20916_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_310 = {S29, 0, &V20916_310_57250, &P20916_309};	/*Property = GeoLocation*/
const Value V20916_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_312 = {S30, 0, &V20916_312_57256, &P20916_310};	/*Property = History*/
const Value V20916_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_314 = {S31, 0, &V20916_314_57262, &P20916_312};	/*Property = Html5*/
const Value V20916_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_313 = {S32, 0, &V20916_313_57259, &P20916_314};	/*Property = Html-Media-Capture*/
const Value V20916_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_315 = {S33, 0, &V20916_315_57265, &P20916_313};	/*Property = Iframe*/
const Value V20916_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_316 = {S34, 0, &V20916_316_57268, &P20916_315};	/*Property = IndexedDB*/
const Value V20916_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_317 = {S35, 0, &V20916_317_57271, &P20916_316};	/*Property = Json*/
const Value V20916_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P20916_15 = {S37, 0, &V20916_15_28, &P20916_317};	/*Property = LayoutEngine*/
const Value V20916_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_344 = {S38, 0, &V20916_344_57274, &P20916_15};	/*Property = Masking*/
const Value V20916_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_320 = {S39, 0, &V20916_320_57292, &P20916_344};	/*Property = PostMessage*/
const Value V20916_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_321 = {S40, 0, &V20916_321_57295, &P20916_320};	/*Property = Progress*/
const Value V20916_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_322 = {S41, 0, &V20916_322_57298, &P20916_321};	/*Property = Prompts*/
const Value V20916_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_323 = {S42, 0, &V20916_323_57301, &P20916_322};	/*Property = Selector*/
const Value V20916_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_324 = {S43, 0, &V20916_324_57307, &P20916_323};	/*Property = Svg*/
const Value V20916_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_327 = {S44, 0, &V20916_327_57316, &P20916_324};	/*Property = TouchEvents*/
const Value V20916_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_328 = {S45, 0, &V20916_328_57319, &P20916_327};	/*Property = Track*/
const Value V20916_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_329 = {S46, 0, &V20916_329_57325, &P20916_328};	/*Property = Video*/
const Value V20916_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_330 = {S47, 0, &V20916_330_57328, &P20916_329};	/*Property = Viewport*/
const Value V20916_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_332 = {S48, 0, &V20916_332_57340, &P20916_330};	/*Property = WebWorkers*/
const Value V20916_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20916_333 = {S49, 0, &V20916_333_57343, &P20916_332};	/*Property = Xhr2*/
const Profile P20916 = {"20916", &P20916_333};	/*Profile ID - 20916*/
const Value V24441_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_279 = {S1, 0, &V24441_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24441_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_282 = {S3, 0, &V24441_282_57155, &P24441_279};	/*Property = BlobBuilder*/
const Value V24441_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_350 = {S4, 0, &V24441_350_57446, &P24441_282};	/*Property = Canvas*/
const Value V24441_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_283 = {S5, 0, &V24441_283_57161, &P24441_350};	/*Property = CssBackground*/
const Value V24441_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_284 = {S6, 0, &V24441_284_57164, &P24441_283};	/*Property = CssBorderImage*/
const Value V24441_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_285 = {S7, 0, &V24441_285_57167, &P24441_284};	/*Property = CssCanvas*/
const Value V24441_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_286 = {S8, 0, &V24441_286_57170, &P24441_285};	/*Property = CssColor*/
const Value V24441_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_237 = {S9, 0, &V24441_237_55786, &P24441_286};	/*Property = CssColumn*/
const Value V24441_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_287 = {S10, 0, &V24441_287_57173, &P24441_237};	/*Property = CssFlexbox*/
const Value V24441_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_288 = {S11, 0, &V24441_288_57176, &P24441_287};	/*Property = CssFont*/
const Value V24441_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_289 = {S12, 0, &V24441_289_86899, &P24441_288};	/*Property = CssImages*/
const Value V24441_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_291 = {S13, 0, &V24441_291_57182, &P24441_289};	/*Property = CssMediaQueries*/
const Value V24441_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_292 = {S14, 0, &V24441_292_57185, &P24441_291};	/*Property = CssMinMax*/
const Value V24441_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_293 = {S15, 0, &V24441_293_57188, &P24441_292};	/*Property = CssOverflow*/
const Value V24441_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_294 = {S16, 0, &V24441_294_57191, &P24441_293};	/*Property = CssPosition*/
const Value V24441_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_295 = {S17, 0, &V24441_295_57194, &P24441_294};	/*Property = CssText*/
const Value V24441_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_297 = {S18, 0, &V24441_297_57197, &P24441_295};	/*Property = CssTransforms*/
const Value V24441_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_241 = {S19, 0, &V24441_241_55801, &P24441_297};	/*Property = CssTransitions*/
const Value V24441_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_298 = {S20, 0, &V24441_298_57208, &P24441_241};	/*Property = CssUI*/
const Value V24441_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_300 = {S21, 0, &V24441_300_57217, &P24441_298};	/*Property = DataSet*/
const Value V24441_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_301 = {S22, 0, &V24441_301_57220, &P24441_300};	/*Property = DataUrl*/
const Value V24441_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_302 = {S23, 0, &V24441_302_57223, &P24441_301};	/*Property = DeviceOrientation*/
const Value V24441_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_304 = {S24, 0, &V24441_304_57229, &P24441_302};	/*Property = FileReader*/
const Value V24441_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_305 = {S25, 0, &V24441_305_57232, &P24441_304};	/*Property = FileSaver*/
const Value V24441_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_306 = {S26, 0, &V24441_306_57238, &P24441_305};	/*Property = FileWriter*/
const Value V24441_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_307 = {S27, 0, &V24441_307_57241, &P24441_306};	/*Property = FormData*/
const Value V24441_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_309 = {S28, 0, &V24441_309_57247, &P24441_307};	/*Property = Fullscreen*/
const Value V24441_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_310 = {S29, 0, &V24441_310_57250, &P24441_309};	/*Property = GeoLocation*/
const Value V24441_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_312 = {S30, 0, &V24441_312_57256, &P24441_310};	/*Property = History*/
const Value V24441_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_314 = {S31, 0, &V24441_314_57262, &P24441_312};	/*Property = Html5*/
const Value V24441_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_313 = {S32, 0, &V24441_313_57259, &P24441_314};	/*Property = Html-Media-Capture*/
const Value V24441_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_315 = {S33, 0, &V24441_315_57265, &P24441_313};	/*Property = Iframe*/
const Value V24441_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_316 = {S34, 0, &V24441_316_57268, &P24441_315};	/*Property = IndexedDB*/
const Value V24441_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_317 = {S35, 0, &V24441_317_57271, &P24441_316};	/*Property = Json*/
const Value V24441_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24441_15 = {S37, 0, &V24441_15_28, &P24441_317};	/*Property = LayoutEngine*/
const Value V24441_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_344 = {S38, 0, &V24441_344_57274, &P24441_15};	/*Property = Masking*/
const Value V24441_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_320 = {S39, 0, &V24441_320_57292, &P24441_344};	/*Property = PostMessage*/
const Value V24441_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_321 = {S40, 0, &V24441_321_57295, &P24441_320};	/*Property = Progress*/
const Value V24441_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_322 = {S41, 0, &V24441_322_57298, &P24441_321};	/*Property = Prompts*/
const Value V24441_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_323 = {S42, 0, &V24441_323_57301, &P24441_322};	/*Property = Selector*/
const Value V24441_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_324 = {S43, 0, &V24441_324_57307, &P24441_323};	/*Property = Svg*/
const Value V24441_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_327 = {S44, 0, &V24441_327_57316, &P24441_324};	/*Property = TouchEvents*/
const Value V24441_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_328 = {S45, 0, &V24441_328_57319, &P24441_327};	/*Property = Track*/
const Value V24441_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_329 = {S46, 0, &V24441_329_57325, &P24441_328};	/*Property = Video*/
const Value V24441_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_330 = {S47, 0, &V24441_330_57328, &P24441_329};	/*Property = Viewport*/
const Value V24441_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_332 = {S48, 0, &V24441_332_57340, &P24441_330};	/*Property = WebWorkers*/
const Value V24441_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24441_333 = {S49, 0, &V24441_333_57343, &P24441_332};	/*Property = Xhr2*/
const Profile P24441 = {"24441", &P24441_333};	/*Profile ID - 24441*/
const Value V20515_1_1 = {S2, NULL};	/*Value = True*/
const Property P20515_1 = {S50, 0, &V20515_1_1, NULL};	/*Property = IsMobile*/
const Value V20515_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20515_5 = {S52, 0, &V20515_5_1277, &P20515_1};	/*Property = ScreenPixelsHeight*/
const Value V20515_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20515_4 = {S54, 0, &V20515_4_1279, &P20515_5};	/*Property = ScreenPixelsWidth*/
const Profile P20515 = {"20515", &P20515_4};	/*Profile ID - 20515*/
const Value V21635_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21635_279 = {S1, 0, &V21635_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21635_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21635_282 = {S3, 0, &V21635_282_57154, &P21635_279};	/*Property = BlobBuilder*/
const Value V21635_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21635_350 = {S4, 0, &V21635_350_57444, &P21635_282};	/*Property = Canvas*/
const Value V21635_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21635_283 = {S5, 0, &V21635_283_57159, &P21635_350};	/*Property = CssBackground*/
const Value V21635_284_57162 = {S2, NULL};	/*Value = True*/
const Property P21635_284 = {S6, 0, &V21635_284_57162, &P21635_283};	/*Property = CssBorderImage*/
const Value V21635_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21635_285 = {S7, 0, &V21635_285_57166, &P21635_284};	/*Property = CssCanvas*/
const Value V21635_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21635_286 = {S8, 0, &V21635_286_57168, &P21635_285};	/*Property = CssColor*/
const Value V21635_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21635_237 = {S9, 0, &V21635_237_55786, &P21635_286};	/*Property = CssColumn*/
const Value V21635_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21635_287 = {S10, 0, &V21635_287_57172, &P21635_237};	/*Property = CssFlexbox*/
const Value V21635_288_57174 = {S2, NULL};	/*Value = True*/
const Property P21635_288 = {S11, 0, &V21635_288_57174, &P21635_287};	/*Property = CssFont*/
const Value V21635_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21635_289 = {S12, 0, &V21635_289_57759, &P21635_288};	/*Property = CssImages*/
const Value V21635_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21635_291 = {S13, 0, &V21635_291_57180, &P21635_289};	/*Property = CssMediaQueries*/
const Value V21635_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21635_292 = {S14, 0, &V21635_292_57183, &P21635_291};	/*Property = CssMinMax*/
const Value V21635_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21635_293 = {S15, 0, &V21635_293_57187, &P21635_292};	/*Property = CssOverflow*/
const Value V21635_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21635_294 = {S16, 0, &V21635_294_57189, &P21635_293};	/*Property = CssPosition*/
const Value V21635_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21635_295 = {S17, 0, &V21635_295_57192, &P21635_294};	/*Property = CssText*/
const Value V21635_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21635_297 = {S18, 0, &V21635_297_57195, &P21635_295};	/*Property = CssTransforms*/
const Value V21635_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21635_241 = {S19, 0, &V21635_241_55799, &P21635_297};	/*Property = CssTransitions*/
const Value V21635_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21635_298 = {S20, 0, &V21635_298_57206, &P21635_241};	/*Property = CssUI*/
const Value V21635_300_57215 = {S2, NULL};	/*Value = True*/
const Property P21635_300 = {S21, 0, &V21635_300_57215, &P21635_298};	/*Property = DataSet*/
const Value V21635_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21635_301 = {S22, 0, &V21635_301_57218, &P21635_300};	/*Property = DataUrl*/
const Value V21635_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21635_302 = {S23, 0, &V21635_302_57222, &P21635_301};	/*Property = DeviceOrientation*/
const Value V21635_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21635_304 = {S24, 0, &V21635_304_57227, &P21635_302};	/*Property = FileReader*/
const Value V21635_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21635_305 = {S25, 0, &V21635_305_57231, &P21635_304};	/*Property = FileSaver*/
const Value V21635_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21635_306 = {S26, 0, &V21635_306_57237, &P21635_305};	/*Property = FileWriter*/
const Value V21635_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21635_307 = {S27, 0, &V21635_307_57239, &P21635_306};	/*Property = FormData*/
const Value V21635_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21635_309 = {S28, 0, &V21635_309_57246, &P21635_307};	/*Property = Fullscreen*/
const Value V21635_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21635_310 = {S29, 0, &V21635_310_57248, &P21635_309};	/*Property = GeoLocation*/
const Value V21635_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21635_312 = {S30, 0, &V21635_312_57254, &P21635_310};	/*Property = History*/
const Value V21635_314_57260 = {S2, NULL};	/*Value = True*/
const Property P21635_314 = {S31, 0, &V21635_314_57260, &P21635_312};	/*Property = Html5*/
const Value V21635_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21635_313 = {S32, 0, &V21635_313_57258, &P21635_314};	/*Property = Html-Media-Capture*/
const Value V21635_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21635_315 = {S33, 0, &V21635_315_57264, &P21635_313};	/*Property = Iframe*/
const Value V21635_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21635_316 = {S34, 0, &V21635_316_57266, &P21635_315};	/*Property = IndexedDB*/
const Value V21635_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21635_317 = {S35, 0, &V21635_317_57269, &P21635_316};	/*Property = Json*/
const Value V21635_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21635_15 = {S37, 0, &V21635_15_28, &P21635_317};	/*Property = LayoutEngine*/
const Value V21635_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21635_344 = {S38, 0, &V21635_344_57272, &P21635_15};	/*Property = Masking*/
const Value V21635_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21635_320 = {S39, 0, &V21635_320_57290, &P21635_344};	/*Property = PostMessage*/
const Value V21635_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21635_321 = {S40, 0, &V21635_321_57293, &P21635_320};	/*Property = Progress*/
const Value V21635_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21635_322 = {S41, 0, &V21635_322_57296, &P21635_321};	/*Property = Prompts*/
const Value V21635_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21635_323 = {S42, 0, &V21635_323_57299, &P21635_322};	/*Property = Selector*/
const Value V21635_324_57305 = {S2, NULL};	/*Value = True*/
const Property P21635_324 = {S43, 0, &V21635_324_57305, &P21635_323};	/*Property = Svg*/
const Value V21635_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21635_327 = {S44, 0, &V21635_327_57315, &P21635_324};	/*Property = TouchEvents*/
const Value V21635_328_57317 = {S2, NULL};	/*Value = True*/
const Property P21635_328 = {S45, 0, &V21635_328_57317, &P21635_327};	/*Property = Track*/
const Value V21635_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21635_329 = {S46, 0, &V21635_329_57323, &P21635_328};	/*Property = Video*/
const Value V21635_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21635_330 = {S47, 0, &V21635_330_57326, &P21635_329};	/*Property = Viewport*/
const Value V21635_332_57338 = {S2, NULL};	/*Value = True*/
const Property P21635_332 = {S48, 0, &V21635_332_57338, &P21635_330};	/*Property = WebWorkers*/
const Value V21635_333_57341 = {S2, NULL};	/*Value = True*/
const Property P21635_333 = {S49, 0, &V21635_333_57341, &P21635_332};	/*Property = Xhr2*/
const Profile P21635 = {"21635", &P21635_333};	/*Profile ID - 21635*/
const Value V20729_1_1 = {S2, NULL};	/*Value = True*/
const Property P20729_1 = {S50, 0, &V20729_1_1, NULL};	/*Property = IsMobile*/
const Value V20729_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20729_5 = {S52, 0, &V20729_5_1277, &P20729_1};	/*Property = ScreenPixelsHeight*/
const Value V20729_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20729_4 = {S54, 0, &V20729_4_1279, &P20729_5};	/*Property = ScreenPixelsWidth*/
const Profile P20729 = {"20729", &P20729_4};	/*Profile ID - 20729*/
const Value V21522_1_1 = {S2, NULL};	/*Value = True*/
const Property P21522_1 = {S50, 0, &V21522_1_1, NULL};	/*Property = IsMobile*/
const Value V21522_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21522_5 = {S52, 0, &V21522_5_1458, &P21522_1};	/*Property = ScreenPixelsHeight*/
const Value V21522_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21522_4 = {S54, 0, &V21522_4_37617, &P21522_5};	/*Property = ScreenPixelsWidth*/
const Profile P21522 = {"21522", &P21522_4};	/*Profile ID - 21522*/
const Value V22529_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22529_279 = {S1, 0, &V22529_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22529_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22529_282 = {S3, 0, &V22529_282_57153, &P22529_279};	/*Property = BlobBuilder*/
const Value V22529_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22529_350 = {S4, 0, &V22529_350_57444, &P22529_282};	/*Property = Canvas*/
const Value V22529_283_57160 = {S0, NULL};	/*Value = False*/
const Property P22529_283 = {S5, 0, &V22529_283_57160, &P22529_350};	/*Property = CssBackground*/
const Value V22529_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22529_284 = {S6, 0, &V22529_284_57163, &P22529_283};	/*Property = CssBorderImage*/
const Value V22529_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22529_285 = {S7, 0, &V22529_285_57166, &P22529_284};	/*Property = CssCanvas*/
const Value V22529_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22529_286 = {S8, 0, &V22529_286_57168, &P22529_285};	/*Property = CssColor*/
const Value V22529_237_55785 = {S0, NULL};	/*Value = False*/
const Property P22529_237 = {S9, 0, &V22529_237_55785, &P22529_286};	/*Property = CssColumn*/
const Value V22529_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22529_287 = {S10, 0, &V22529_287_57172, &P22529_237};	/*Property = CssFlexbox*/
const Value V22529_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22529_288 = {S11, 0, &V22529_288_57175, &P22529_287};	/*Property = CssFont*/
const Value V22529_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22529_289 = {S12, 0, &V22529_289_86899, &P22529_288};	/*Property = CssImages*/
const Value V22529_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22529_291 = {S13, 0, &V22529_291_57181, &P22529_289};	/*Property = CssMediaQueries*/
const Value V22529_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22529_292 = {S14, 0, &V22529_292_57183, &P22529_291};	/*Property = CssMinMax*/
const Value V22529_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22529_293 = {S15, 0, &V22529_293_57187, &P22529_292};	/*Property = CssOverflow*/
const Value V22529_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22529_294 = {S16, 0, &V22529_294_57189, &P22529_293};	/*Property = CssPosition*/
const Value V22529_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22529_295 = {S17, 0, &V22529_295_57193, &P22529_294};	/*Property = CssText*/
const Value V22529_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22529_297 = {S18, 0, &V22529_297_57195, &P22529_295};	/*Property = CssTransforms*/
const Value V22529_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22529_241 = {S19, 0, &V22529_241_55799, &P22529_297};	/*Property = CssTransitions*/
const Value V22529_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22529_298 = {S20, 0, &V22529_298_57206, &P22529_241};	/*Property = CssUI*/
const Value V22529_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22529_300 = {S21, 0, &V22529_300_57215, &P22529_298};	/*Property = DataSet*/
const Value V22529_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22529_301 = {S22, 0, &V22529_301_57218, &P22529_300};	/*Property = DataUrl*/
const Value V22529_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22529_302 = {S23, 0, &V22529_302_57221, &P22529_301};	/*Property = DeviceOrientation*/
const Value V22529_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22529_304 = {S24, 0, &V22529_304_57227, &P22529_302};	/*Property = FileReader*/
const Value V22529_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22529_305 = {S25, 0, &V22529_305_57231, &P22529_304};	/*Property = FileSaver*/
const Value V22529_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22529_306 = {S26, 0, &V22529_306_57237, &P22529_305};	/*Property = FileWriter*/
const Value V22529_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22529_307 = {S27, 0, &V22529_307_57239, &P22529_306};	/*Property = FormData*/
const Value V22529_309_57245 = {S2, NULL};	/*Value = True*/
const Property P22529_309 = {S28, 0, &V22529_309_57245, &P22529_307};	/*Property = Fullscreen*/
const Value V22529_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22529_310 = {S29, 0, &V22529_310_57248, &P22529_309};	/*Property = GeoLocation*/
const Value V22529_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22529_312 = {S30, 0, &V22529_312_57254, &P22529_310};	/*Property = History*/
const Value V22529_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22529_314 = {S31, 0, &V22529_314_57261, &P22529_312};	/*Property = Html5*/
const Value V22529_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22529_313 = {S32, 0, &V22529_313_57258, &P22529_314};	/*Property = Html-Media-Capture*/
const Value V22529_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22529_315 = {S33, 0, &V22529_315_57264, &P22529_313};	/*Property = Iframe*/
const Value V22529_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22529_316 = {S34, 0, &V22529_316_57266, &P22529_315};	/*Property = IndexedDB*/
const Value V22529_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22529_317 = {S35, 0, &V22529_317_57269, &P22529_316};	/*Property = Json*/
const Value V22529_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22529_15 = {S37, 0, &V22529_15_29, &P22529_317};	/*Property = LayoutEngine*/
const Value V22529_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22529_344 = {S38, 0, &V22529_344_57272, &P22529_15};	/*Property = Masking*/
const Value V22529_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22529_320 = {S39, 0, &V22529_320_57290, &P22529_344};	/*Property = PostMessage*/
const Value V22529_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22529_321 = {S40, 0, &V22529_321_57293, &P22529_320};	/*Property = Progress*/
const Value V22529_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22529_322 = {S41, 0, &V22529_322_57296, &P22529_321};	/*Property = Prompts*/
const Value V22529_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22529_323 = {S42, 0, &V22529_323_57299, &P22529_322};	/*Property = Selector*/
const Value V22529_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22529_324 = {S43, 0, &V22529_324_57305, &P22529_323};	/*Property = Svg*/
const Value V22529_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22529_327 = {S44, 0, &V22529_327_57315, &P22529_324};	/*Property = TouchEvents*/
const Value V22529_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22529_328 = {S45, 0, &V22529_328_57318, &P22529_327};	/*Property = Track*/
const Value V22529_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22529_329 = {S46, 0, &V22529_329_57323, &P22529_328};	/*Property = Video*/
const Value V22529_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22529_330 = {S47, 0, &V22529_330_57326, &P22529_329};	/*Property = Viewport*/
const Value V22529_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22529_332 = {S48, 0, &V22529_332_57339, &P22529_330};	/*Property = WebWorkers*/
const Value V22529_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22529_333 = {S49, 0, &V22529_333_57342, &P22529_332};	/*Property = Xhr2*/
const Profile P22529 = {"22529", &P22529_333};	/*Profile ID - 22529*/
const Value V23625_1_2 = {S0, NULL};	/*Value = False*/
const Property P23625_1 = {S50, 0, &V23625_1_2, NULL};	/*Property = IsMobile*/
const Value V23625_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23625_5 = {S52, 0, &V23625_5_60417, &P23625_1};	/*Property = ScreenPixelsHeight*/
const Value V23625_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23625_4 = {S54, 0, &V23625_4_57095, &P23625_5};	/*Property = ScreenPixelsWidth*/
const Profile P23625 = {"23625", &P23625_4};	/*Profile ID - 23625*/
const Value V20610_1_1 = {S2, NULL};	/*Value = True*/
const Property P20610_1 = {S50, 0, &V20610_1_1, NULL};	/*Property = IsMobile*/
const Value V20610_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20610_5 = {S52, 0, &V20610_5_1227, &P20610_1};	/*Property = ScreenPixelsHeight*/
const Value V20610_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20610_4 = {S54, 0, &V20610_4_1229, &P20610_5};	/*Property = ScreenPixelsWidth*/
const Profile P20610 = {"20610", &P20610_4};	/*Profile ID - 20610*/
const Value V22539_1_1 = {S2, NULL};	/*Value = True*/
const Property P22539_1 = {S50, 0, &V22539_1_1, NULL};	/*Property = IsMobile*/
const Value V22539_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22539_5 = {S52, 0, &V22539_5_1832, &P22539_1};	/*Property = ScreenPixelsHeight*/
const Value V22539_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22539_4 = {S54, 0, &V22539_4_5645, &P22539_5};	/*Property = ScreenPixelsWidth*/
const Profile P22539 = {"22539", &P22539_4};	/*Profile ID - 22539*/
const Value V21564_1_1 = {S2, NULL};	/*Value = True*/
const Property P21564_1 = {S50, 0, &V21564_1_1, NULL};	/*Property = IsMobile*/
const Value V21564_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21564_5 = {S52, 0, &V21564_5_1403, &P21564_1};	/*Property = ScreenPixelsHeight*/
const Value V21564_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21564_4 = {S54, 0, &V21564_4_1404, &P21564_5};	/*Property = ScreenPixelsWidth*/
const Profile P21564 = {"21564", &P21564_4};	/*Profile ID - 21564*/
const Value V23058_1_1 = {S2, NULL};	/*Value = True*/
const Property P23058_1 = {S50, 0, &V23058_1_1, NULL};	/*Property = IsMobile*/
const Value V23058_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23058_5 = {S52, 0, &V23058_5_1227, &P23058_1};	/*Property = ScreenPixelsHeight*/
const Value V23058_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23058_4 = {S54, 0, &V23058_4_1229, &P23058_5};	/*Property = ScreenPixelsWidth*/
const Profile P23058 = {"23058", &P23058_4};	/*Profile ID - 23058*/
const Value V22703_1_1 = {S2, NULL};	/*Value = True*/
const Property P22703_1 = {S50, 0, &V22703_1_1, NULL};	/*Property = IsMobile*/
const Value V22703_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22703_5 = {S52, 0, &V22703_5_1222, &P22703_1};	/*Property = ScreenPixelsHeight*/
const Value V22703_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22703_4 = {S54, 0, &V22703_4_1224, &P22703_5};	/*Property = ScreenPixelsWidth*/
const Profile P22703 = {"22703", &P22703_4};	/*Profile ID - 22703*/
const Value V21606_1_1 = {S2, NULL};	/*Value = True*/
const Property P21606_1 = {S50, 0, &V21606_1_1, NULL};	/*Property = IsMobile*/
const Value V21606_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21606_5 = {S52, 0, &V21606_5_1227, &P21606_1};	/*Property = ScreenPixelsHeight*/
const Value V21606_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P21606_4 = {S54, 0, &V21606_4_1943, &P21606_5};	/*Property = ScreenPixelsWidth*/
const Profile P21606 = {"21606", &P21606_4};	/*Profile ID - 21606*/
const Value V21300_1_1 = {S2, NULL};	/*Value = True*/
const Property P21300_1 = {S50, 0, &V21300_1_1, NULL};	/*Property = IsMobile*/
const Value V21300_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21300_5 = {S52, 0, &V21300_5_1458, &P21300_1};	/*Property = ScreenPixelsHeight*/
const Value V21300_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21300_4 = {S54, 0, &V21300_4_1460, &P21300_5};	/*Property = ScreenPixelsWidth*/
const Profile P21300 = {"21300", &P21300_4};	/*Profile ID - 21300*/
const Value V22118_1_1 = {S2, NULL};	/*Value = True*/
const Property P22118_1 = {S50, 0, &V22118_1_1, NULL};	/*Property = IsMobile*/
const Value V22118_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22118_5 = {S52, 0, &V22118_5_1277, &P22118_1};	/*Property = ScreenPixelsHeight*/
const Value V22118_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22118_4 = {S54, 0, &V22118_4_1279, &P22118_5};	/*Property = ScreenPixelsWidth*/
const Profile P22118 = {"22118", &P22118_4};	/*Profile ID - 22118*/
const Value V21631_1_1 = {S2, NULL};	/*Value = True*/
const Property P21631_1 = {S50, 0, &V21631_1_1, NULL};	/*Property = IsMobile*/
const Value V21631_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21631_5 = {S52, 0, &V21631_5_1227, &P21631_1};	/*Property = ScreenPixelsHeight*/
const Value V21631_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21631_4 = {S54, 0, &V21631_4_1229, &P21631_5};	/*Property = ScreenPixelsWidth*/
const Profile P21631 = {"21631", &P21631_4};	/*Profile ID - 21631*/
const Value V21516_1_1 = {S2, NULL};	/*Value = True*/
const Property P21516_1 = {S50, 0, &V21516_1_1, NULL};	/*Property = IsMobile*/
const Value V21516_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21516_5 = {S52, 0, &V21516_5_1277, &P21516_1};	/*Property = ScreenPixelsHeight*/
const Value V21516_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21516_4 = {S54, 0, &V21516_4_1279, &P21516_5};	/*Property = ScreenPixelsWidth*/
const Profile P21516 = {"21516", &P21516_4};	/*Profile ID - 21516*/
const Value V21433_1_1 = {S2, NULL};	/*Value = True*/
const Property P21433_1 = {S50, 0, &V21433_1_1, NULL};	/*Property = IsMobile*/
const Value V21433_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21433_5 = {S52, 0, &V21433_5_9981, &P21433_1};	/*Property = ScreenPixelsHeight*/
const Value V21433_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21433_4 = {S54, 0, &V21433_4_39098, &P21433_5};	/*Property = ScreenPixelsWidth*/
const Profile P21433 = {"21433", &P21433_4};	/*Profile ID - 21433*/
const Value V23470_1_1 = {S2, NULL};	/*Value = True*/
const Property P23470_1 = {S50, 0, &V23470_1_1, NULL};	/*Property = IsMobile*/
const Value V23470_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23470_5 = {S52, 0, &V23470_5_1227, &P23470_1};	/*Property = ScreenPixelsHeight*/
const Value V23470_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23470_4 = {S54, 0, &V23470_4_1229, &P23470_5};	/*Property = ScreenPixelsWidth*/
const Profile P23470 = {"23470", &P23470_4};	/*Profile ID - 23470*/
const Value V23467_1_1 = {S2, NULL};	/*Value = True*/
const Property P23467_1 = {S50, 0, &V23467_1_1, NULL};	/*Property = IsMobile*/
const Value V23467_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23467_5 = {S52, 0, &V23467_5_1227, &P23467_1};	/*Property = ScreenPixelsHeight*/
const Value V23467_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23467_4 = {S54, 0, &V23467_4_1229, &P23467_5};	/*Property = ScreenPixelsWidth*/
const Profile P23467 = {"23467", &P23467_4};	/*Profile ID - 23467*/
const Value V23475_1_1 = {S2, NULL};	/*Value = True*/
const Property P23475_1 = {S50, 0, &V23475_1_1, NULL};	/*Property = IsMobile*/
const Value V23475_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23475_5 = {S52, 0, &V23475_5_1227, &P23475_1};	/*Property = ScreenPixelsHeight*/
const Value V23475_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23475_4 = {S54, 0, &V23475_4_1229, &P23475_5};	/*Property = ScreenPixelsWidth*/
const Profile P23475 = {"23475", &P23475_4};	/*Profile ID - 23475*/
const Value V22022_1_1 = {S2, NULL};	/*Value = True*/
const Property P22022_1 = {S50, 0, &V22022_1_1, NULL};	/*Property = IsMobile*/
const Value V22022_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22022_5 = {S52, 0, &V22022_5_1277, &P22022_1};	/*Property = ScreenPixelsHeight*/
const Value V22022_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22022_4 = {S54, 0, &V22022_4_2336, &P22022_5};	/*Property = ScreenPixelsWidth*/
const Profile P22022 = {"22022", &P22022_4};	/*Profile ID - 22022*/
const Value V21595_1_1 = {S2, NULL};	/*Value = True*/
const Property P21595_1 = {S50, 0, &V21595_1_1, NULL};	/*Property = IsMobile*/
const Value V21595_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21595_5 = {S52, 0, &V21595_5_1227, &P21595_1};	/*Property = ScreenPixelsHeight*/
const Value V21595_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21595_4 = {S54, 0, &V21595_4_1229, &P21595_5};	/*Property = ScreenPixelsWidth*/
const Profile P21595 = {"21595", &P21595_4};	/*Profile ID - 21595*/
const Profile P22324 = {"22324", NULL};	/*Profile ID - 22324*/
const Value V23345_1_1 = {S2, NULL};	/*Value = True*/
const Property P23345_1 = {S50, 0, &V23345_1_1, NULL};	/*Property = IsMobile*/
const Value V23345_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23345_5 = {S52, 0, &V23345_5_37569, &P23345_1};	/*Property = ScreenPixelsHeight*/
const Value V23345_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23345_4 = {S54, 0, &V23345_4_5645, &P23345_5};	/*Property = ScreenPixelsWidth*/
const Profile P23345 = {"23345", &P23345_4};	/*Profile ID - 23345*/
const Value V21514_1_1 = {S2, NULL};	/*Value = True*/
const Property P21514_1 = {S50, 0, &V21514_1_1, NULL};	/*Property = IsMobile*/
const Value V21514_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21514_5 = {S52, 0, &V21514_5_1277, &P21514_1};	/*Property = ScreenPixelsHeight*/
const Value V21514_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21514_4 = {S54, 0, &V21514_4_1279, &P21514_5};	/*Property = ScreenPixelsWidth*/
const Profile P21514 = {"21514", &P21514_4};	/*Profile ID - 21514*/
const Value V22675_1_1 = {S2, NULL};	/*Value = True*/
const Property P22675_1 = {S50, 0, &V22675_1_1, NULL};	/*Property = IsMobile*/
const Value V22675_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22675_5 = {S52, 0, &V22675_5_1458, &P22675_1};	/*Property = ScreenPixelsHeight*/
const Value V22675_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22675_4 = {S54, 0, &V22675_4_1460, &P22675_5};	/*Property = ScreenPixelsWidth*/
const Profile P22675 = {"22675", &P22675_4};	/*Profile ID - 22675*/
const Value V21260_1_1 = {S2, NULL};	/*Value = True*/
const Property P21260_1 = {S50, 0, &V21260_1_1, NULL};	/*Property = IsMobile*/
const Value V21260_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21260_5 = {S52, 0, &V21260_5_1227, &P21260_1};	/*Property = ScreenPixelsHeight*/
const Value V21260_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21260_4 = {S54, 0, &V21260_4_1229, &P21260_5};	/*Property = ScreenPixelsWidth*/
const Profile P21260 = {"21260", &P21260_4};	/*Profile ID - 21260*/
const Value V23880_1_1 = {S2, NULL};	/*Value = True*/
const Property P23880_1 = {S50, 0, &V23880_1_1, NULL};	/*Property = IsMobile*/
const Value V23880_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23880_5 = {S52, 0, &V23880_5_1277, &P23880_1};	/*Property = ScreenPixelsHeight*/
const Value V23880_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23880_4 = {S54, 0, &V23880_4_1279, &P23880_5};	/*Property = ScreenPixelsWidth*/
const Profile P23880 = {"23880", &P23880_4};	/*Profile ID - 23880*/
const Value V21538_1_1 = {S2, NULL};	/*Value = True*/
const Property P21538_1 = {S50, 0, &V21538_1_1, NULL};	/*Property = IsMobile*/
const Value V21538_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21538_5 = {S52, 0, &V21538_5_1277, &P21538_1};	/*Property = ScreenPixelsHeight*/
const Value V21538_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21538_4 = {S54, 0, &V21538_4_1279, &P21538_5};	/*Property = ScreenPixelsWidth*/
const Profile P21538 = {"21538", &P21538_4};	/*Profile ID - 21538*/
const Value V22698_1_1 = {S2, NULL};	/*Value = True*/
const Property P22698_1 = {S50, 0, &V22698_1_1, NULL};	/*Property = IsMobile*/
const Value V22698_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22698_5 = {S52, 0, &V22698_5_1277, &P22698_1};	/*Property = ScreenPixelsHeight*/
const Value V22698_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22698_4 = {S54, 0, &V22698_4_1279, &P22698_5};	/*Property = ScreenPixelsWidth*/
const Profile P22698 = {"22698", &P22698_4};	/*Profile ID - 22698*/
const Value V22166_1_1 = {S2, NULL};	/*Value = True*/
const Property P22166_1 = {S50, 0, &V22166_1_1, NULL};	/*Property = IsMobile*/
const Value V22166_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22166_5 = {S52, 0, &V22166_5_1458, &P22166_1};	/*Property = ScreenPixelsHeight*/
const Value V22166_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P22166_4 = {S54, 0, &V22166_4_37617, &P22166_5};	/*Property = ScreenPixelsWidth*/
const Profile P22166 = {"22166", &P22166_4};	/*Profile ID - 22166*/
const Value V20541_1_1 = {S2, NULL};	/*Value = True*/
const Property P20541_1 = {S50, 0, &V20541_1_1, NULL};	/*Property = IsMobile*/
const Value V20541_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20541_5 = {S52, 0, &V20541_5_1403, &P20541_1};	/*Property = ScreenPixelsHeight*/
const Value V20541_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20541_4 = {S54, 0, &V20541_4_1404, &P20541_5};	/*Property = ScreenPixelsWidth*/
const Profile P20541 = {"20541", &P20541_4};	/*Profile ID - 20541*/
const Value V21134_1_1 = {S2, NULL};	/*Value = True*/
const Property P21134_1 = {S50, 0, &V21134_1_1, NULL};	/*Property = IsMobile*/
const Value V21134_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21134_5 = {S52, 0, &V21134_5_1403, &P21134_1};	/*Property = ScreenPixelsHeight*/
const Value V21134_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21134_4 = {S54, 0, &V21134_4_1404, &P21134_5};	/*Property = ScreenPixelsWidth*/
const Profile P21134 = {"21134", &P21134_4};	/*Profile ID - 21134*/
const Value V21996_1_1 = {S2, NULL};	/*Value = True*/
const Property P21996_1 = {S50, 0, &V21996_1_1, NULL};	/*Property = IsMobile*/
const Value V21996_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21996_5 = {S52, 0, &V21996_5_1277, &P21996_1};	/*Property = ScreenPixelsHeight*/
const Value V21996_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21996_4 = {S54, 0, &V21996_4_1279, &P21996_5};	/*Property = ScreenPixelsWidth*/
const Profile P21996 = {"21996", &P21996_4};	/*Profile ID - 21996*/
const Value V23814_1_1 = {S2, NULL};	/*Value = True*/
const Property P23814_1 = {S50, 0, &V23814_1_1, NULL};	/*Property = IsMobile*/
const Value V23814_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23814_5 = {S52, 0, &V23814_5_1227, &P23814_1};	/*Property = ScreenPixelsHeight*/
const Value V23814_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23814_4 = {S54, 0, &V23814_4_1229, &P23814_5};	/*Property = ScreenPixelsWidth*/
const Profile P23814 = {"23814", &P23814_4};	/*Profile ID - 23814*/
const Value V22085_1_1 = {S2, NULL};	/*Value = True*/
const Property P22085_1 = {S50, 0, &V22085_1_1, NULL};	/*Property = IsMobile*/
const Value V22085_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22085_5 = {S52, 0, &V22085_5_9981, &P22085_1};	/*Property = ScreenPixelsHeight*/
const Value V22085_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22085_4 = {S54, 0, &V22085_4_39098, &P22085_5};	/*Property = ScreenPixelsWidth*/
const Profile P22085 = {"22085", &P22085_4};	/*Profile ID - 22085*/
const Value V21153_1_1 = {S2, NULL};	/*Value = True*/
const Property P21153_1 = {S50, 0, &V21153_1_1, NULL};	/*Property = IsMobile*/
const Value V21153_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21153_5 = {S52, 0, &V21153_5_1403, &P21153_1};	/*Property = ScreenPixelsHeight*/
const Value V21153_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21153_4 = {S54, 0, &V21153_4_1404, &P21153_5};	/*Property = ScreenPixelsWidth*/
const Profile P21153 = {"21153", &P21153_4};	/*Profile ID - 21153*/
const Value V21974_1_1 = {S2, NULL};	/*Value = True*/
const Property P21974_1 = {S50, 0, &V21974_1_1, NULL};	/*Property = IsMobile*/
const Value V21974_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21974_5 = {S52, 0, &V21974_5_1458, &P21974_1};	/*Property = ScreenPixelsHeight*/
const Value V21974_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21974_4 = {S54, 0, &V21974_4_37617, &P21974_5};	/*Property = ScreenPixelsWidth*/
const Profile P21974 = {"21974", &P21974_4};	/*Profile ID - 21974*/
const Profile P22379 = {"22379", NULL};	/*Profile ID - 22379*/
const Value V22687_1_1 = {S2, NULL};	/*Value = True*/
const Property P22687_1 = {S50, 0, &V22687_1_1, NULL};	/*Property = IsMobile*/
const Value V22687_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22687_5 = {S52, 0, &V22687_5_1227, &P22687_1};	/*Property = ScreenPixelsHeight*/
const Value V22687_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22687_4 = {S54, 0, &V22687_4_1229, &P22687_5};	/*Property = ScreenPixelsWidth*/
const Profile P22687 = {"22687", &P22687_4};	/*Profile ID - 22687*/
const Value V23902_1_1 = {S2, NULL};	/*Value = True*/
const Property P23902_1 = {S50, 0, &V23902_1_1, NULL};	/*Property = IsMobile*/
const Value V23902_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23902_5 = {S52, 0, &V23902_5_1277, &P23902_1};	/*Property = ScreenPixelsHeight*/
const Value V23902_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23902_4 = {S54, 0, &V23902_4_1279, &P23902_5};	/*Property = ScreenPixelsWidth*/
const Profile P23902 = {"23902", &P23902_4};	/*Profile ID - 23902*/
const Value V21873_1_1 = {S2, NULL};	/*Value = True*/
const Property P21873_1 = {S50, 0, &V21873_1_1, NULL};	/*Property = IsMobile*/
const Value V21873_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21873_5 = {S52, 0, &V21873_5_1458, &P21873_1};	/*Property = ScreenPixelsHeight*/
const Value V21873_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21873_4 = {S54, 0, &V21873_4_1460, &P21873_5};	/*Property = ScreenPixelsWidth*/
const Profile P21873 = {"21873", &P21873_4};	/*Profile ID - 21873*/
const Value V21817_1_1 = {S2, NULL};	/*Value = True*/
const Property P21817_1 = {S50, 0, &V21817_1_1, NULL};	/*Property = IsMobile*/
const Value V21817_5_80732 = {S147, NULL};	/*Value = 601*/
const Property P21817_5 = {S52, 0, &V21817_5_80732, &P21817_1};	/*Property = ScreenPixelsHeight*/
const Value V21817_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21817_4 = {S54, 0, &V21817_4_5645, &P21817_5};	/*Property = ScreenPixelsWidth*/
const Profile P21817 = {"21817", &P21817_4};	/*Profile ID - 21817*/
const Value V20493_1_1 = {S2, NULL};	/*Value = True*/
const Property P20493_1 = {S50, 0, &V20493_1_1, NULL};	/*Property = IsMobile*/
const Value V20493_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20493_5 = {S52, 0, &V20493_5_1227, &P20493_1};	/*Property = ScreenPixelsHeight*/
const Value V20493_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20493_4 = {S54, 0, &V20493_4_1229, &P20493_5};	/*Property = ScreenPixelsWidth*/
const Profile P20493 = {"20493", &P20493_4};	/*Profile ID - 20493*/
const Value V23819_1_1 = {S2, NULL};	/*Value = True*/
const Property P23819_1 = {S50, 0, &V23819_1_1, NULL};	/*Property = IsMobile*/
const Value V23819_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23819_5 = {S52, 0, &V23819_5_1458, &P23819_1};	/*Property = ScreenPixelsHeight*/
const Value V23819_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23819_4 = {S54, 0, &V23819_4_37617, &P23819_5};	/*Property = ScreenPixelsWidth*/
const Profile P23819 = {"23819", &P23819_4};	/*Profile ID - 23819*/
const Value V22608_1_1 = {S2, NULL};	/*Value = True*/
const Property P22608_1 = {S50, 0, &V22608_1_1, NULL};	/*Property = IsMobile*/
const Value V22608_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22608_5 = {S52, 0, &V22608_5_1403, &P22608_1};	/*Property = ScreenPixelsHeight*/
const Value V22608_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22608_4 = {S54, 0, &V22608_4_1404, &P22608_5};	/*Property = ScreenPixelsWidth*/
const Profile P22608 = {"22608", &P22608_4};	/*Profile ID - 22608*/
const Value V23049_1_1 = {S2, NULL};	/*Value = True*/
const Property P23049_1 = {S50, 0, &V23049_1_1, NULL};	/*Property = IsMobile*/
const Value V23049_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23049_5 = {S52, 0, &V23049_5_1277, &P23049_1};	/*Property = ScreenPixelsHeight*/
const Value V23049_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23049_4 = {S54, 0, &V23049_4_1279, &P23049_5};	/*Property = ScreenPixelsWidth*/
const Profile P23049 = {"23049", &P23049_4};	/*Profile ID - 23049*/
const Value V23257_1_1 = {S2, NULL};	/*Value = True*/
const Property P23257_1 = {S50, 0, &V23257_1_1, NULL};	/*Property = IsMobile*/
const Value V23257_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23257_5 = {S52, 0, &V23257_5_1227, &P23257_1};	/*Property = ScreenPixelsHeight*/
const Value V23257_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23257_4 = {S54, 0, &V23257_4_1229, &P23257_5};	/*Property = ScreenPixelsWidth*/
const Profile P23257 = {"23257", &P23257_4};	/*Profile ID - 23257*/
const Value V21305_1_1 = {S2, NULL};	/*Value = True*/
const Property P21305_1 = {S50, 0, &V21305_1_1, NULL};	/*Property = IsMobile*/
const Value V21305_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21305_5 = {S52, 0, &V21305_5_1222, &P21305_1};	/*Property = ScreenPixelsHeight*/
const Value V21305_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21305_4 = {S54, 0, &V21305_4_1224, &P21305_5};	/*Property = ScreenPixelsWidth*/
const Profile P21305 = {"21305", &P21305_4};	/*Profile ID - 21305*/
const Value V21884_1_1 = {S2, NULL};	/*Value = True*/
const Property P21884_1 = {S50, 0, &V21884_1_1, NULL};	/*Property = IsMobile*/
const Value V21884_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21884_5 = {S52, 0, &V21884_5_9981, &P21884_1};	/*Property = ScreenPixelsHeight*/
const Value V21884_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21884_4 = {S54, 0, &V21884_4_2336, &P21884_5};	/*Property = ScreenPixelsWidth*/
const Profile P21884 = {"21884", &P21884_4};	/*Profile ID - 21884*/
const Value V23572_1_1 = {S2, NULL};	/*Value = True*/
const Property P23572_1 = {S50, 0, &V23572_1_1, NULL};	/*Property = IsMobile*/
const Value V23572_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23572_5 = {S52, 0, &V23572_5_1277, &P23572_1};	/*Property = ScreenPixelsHeight*/
const Value V23572_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23572_4 = {S54, 0, &V23572_4_1279, &P23572_5};	/*Property = ScreenPixelsWidth*/
const Profile P23572 = {"23572", &P23572_4};	/*Profile ID - 23572*/
const Value V22672_1_1 = {S2, NULL};	/*Value = True*/
const Property P22672_1 = {S50, 0, &V22672_1_1, NULL};	/*Property = IsMobile*/
const Value V22672_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P22672_5 = {S52, 0, &V22672_5_37, &P22672_1};	/*Property = ScreenPixelsHeight*/
const Value V22672_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P22672_4 = {S54, 0, &V22672_4_36, &P22672_5};	/*Property = ScreenPixelsWidth*/
const Profile P22672 = {"22672", &P22672_4};	/*Profile ID - 22672*/
const Value V22088_1_1 = {S2, NULL};	/*Value = True*/
const Property P22088_1 = {S50, 0, &V22088_1_1, NULL};	/*Property = IsMobile*/
const Value V22088_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22088_5 = {S52, 0, &V22088_5_1227, &P22088_1};	/*Property = ScreenPixelsHeight*/
const Value V22088_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P22088_4 = {S54, 0, &V22088_4_1477, &P22088_5};	/*Property = ScreenPixelsWidth*/
const Profile P22088 = {"22088", &P22088_4};	/*Profile ID - 22088*/
const Value V21980_1_1 = {S2, NULL};	/*Value = True*/
const Property P21980_1 = {S50, 0, &V21980_1_1, NULL};	/*Property = IsMobile*/
const Value V21980_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21980_5 = {S52, 0, &V21980_5_1277, &P21980_1};	/*Property = ScreenPixelsHeight*/
const Value V21980_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21980_4 = {S54, 0, &V21980_4_2336, &P21980_5};	/*Property = ScreenPixelsWidth*/
const Profile P21980 = {"21980", &P21980_4};	/*Profile ID - 21980*/
const Value V23437_1_1 = {S2, NULL};	/*Value = True*/
const Property P23437_1 = {S50, 0, &V23437_1_1, NULL};	/*Property = IsMobile*/
const Value V23437_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23437_5 = {S52, 0, &V23437_5_1227, &P23437_1};	/*Property = ScreenPixelsHeight*/
const Value V23437_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23437_4 = {S54, 0, &V23437_4_1229, &P23437_5};	/*Property = ScreenPixelsWidth*/
const Profile P23437 = {"23437", &P23437_4};	/*Profile ID - 23437*/
const Value V21906_1_1 = {S2, NULL};	/*Value = True*/
const Property P21906_1 = {S50, 0, &V21906_1_1, NULL};	/*Property = IsMobile*/
const Value V21906_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21906_5 = {S52, 0, &V21906_5_1227, &P21906_1};	/*Property = ScreenPixelsHeight*/
const Value V21906_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21906_4 = {S54, 0, &V21906_4_1229, &P21906_5};	/*Property = ScreenPixelsWidth*/
const Profile P21906 = {"21906", &P21906_4};	/*Profile ID - 21906*/
const Value V24421_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_279 = {S1, 0, &V24421_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24421_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_282 = {S3, 0, &V24421_282_57155, &P24421_279};	/*Property = BlobBuilder*/
const Value V24421_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_350 = {S4, 0, &V24421_350_57446, &P24421_282};	/*Property = Canvas*/
const Value V24421_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_283 = {S5, 0, &V24421_283_57161, &P24421_350};	/*Property = CssBackground*/
const Value V24421_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_284 = {S6, 0, &V24421_284_57164, &P24421_283};	/*Property = CssBorderImage*/
const Value V24421_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_285 = {S7, 0, &V24421_285_57167, &P24421_284};	/*Property = CssCanvas*/
const Value V24421_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_286 = {S8, 0, &V24421_286_57170, &P24421_285};	/*Property = CssColor*/
const Value V24421_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_237 = {S9, 0, &V24421_237_55786, &P24421_286};	/*Property = CssColumn*/
const Value V24421_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_287 = {S10, 0, &V24421_287_57173, &P24421_237};	/*Property = CssFlexbox*/
const Value V24421_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_288 = {S11, 0, &V24421_288_57176, &P24421_287};	/*Property = CssFont*/
const Value V24421_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_289 = {S12, 0, &V24421_289_86899, &P24421_288};	/*Property = CssImages*/
const Value V24421_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_291 = {S13, 0, &V24421_291_57182, &P24421_289};	/*Property = CssMediaQueries*/
const Value V24421_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_292 = {S14, 0, &V24421_292_57185, &P24421_291};	/*Property = CssMinMax*/
const Value V24421_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_293 = {S15, 0, &V24421_293_57188, &P24421_292};	/*Property = CssOverflow*/
const Value V24421_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_294 = {S16, 0, &V24421_294_57191, &P24421_293};	/*Property = CssPosition*/
const Value V24421_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_295 = {S17, 0, &V24421_295_57194, &P24421_294};	/*Property = CssText*/
const Value V24421_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_297 = {S18, 0, &V24421_297_57197, &P24421_295};	/*Property = CssTransforms*/
const Value V24421_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_241 = {S19, 0, &V24421_241_55801, &P24421_297};	/*Property = CssTransitions*/
const Value V24421_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_298 = {S20, 0, &V24421_298_57208, &P24421_241};	/*Property = CssUI*/
const Value V24421_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_300 = {S21, 0, &V24421_300_57217, &P24421_298};	/*Property = DataSet*/
const Value V24421_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_301 = {S22, 0, &V24421_301_57220, &P24421_300};	/*Property = DataUrl*/
const Value V24421_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_302 = {S23, 0, &V24421_302_57223, &P24421_301};	/*Property = DeviceOrientation*/
const Value V24421_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_304 = {S24, 0, &V24421_304_57229, &P24421_302};	/*Property = FileReader*/
const Value V24421_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_305 = {S25, 0, &V24421_305_57232, &P24421_304};	/*Property = FileSaver*/
const Value V24421_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_306 = {S26, 0, &V24421_306_57238, &P24421_305};	/*Property = FileWriter*/
const Value V24421_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_307 = {S27, 0, &V24421_307_57241, &P24421_306};	/*Property = FormData*/
const Value V24421_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_309 = {S28, 0, &V24421_309_57247, &P24421_307};	/*Property = Fullscreen*/
const Value V24421_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_310 = {S29, 0, &V24421_310_57250, &P24421_309};	/*Property = GeoLocation*/
const Value V24421_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_312 = {S30, 0, &V24421_312_57256, &P24421_310};	/*Property = History*/
const Value V24421_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_314 = {S31, 0, &V24421_314_57262, &P24421_312};	/*Property = Html5*/
const Value V24421_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_313 = {S32, 0, &V24421_313_57259, &P24421_314};	/*Property = Html-Media-Capture*/
const Value V24421_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_315 = {S33, 0, &V24421_315_57265, &P24421_313};	/*Property = Iframe*/
const Value V24421_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_316 = {S34, 0, &V24421_316_57268, &P24421_315};	/*Property = IndexedDB*/
const Value V24421_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_317 = {S35, 0, &V24421_317_57271, &P24421_316};	/*Property = Json*/
const Value V24421_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24421_15 = {S37, 0, &V24421_15_28, &P24421_317};	/*Property = LayoutEngine*/
const Value V24421_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_344 = {S38, 0, &V24421_344_57274, &P24421_15};	/*Property = Masking*/
const Value V24421_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_320 = {S39, 0, &V24421_320_57292, &P24421_344};	/*Property = PostMessage*/
const Value V24421_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_321 = {S40, 0, &V24421_321_57295, &P24421_320};	/*Property = Progress*/
const Value V24421_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_322 = {S41, 0, &V24421_322_57298, &P24421_321};	/*Property = Prompts*/
const Value V24421_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_323 = {S42, 0, &V24421_323_57301, &P24421_322};	/*Property = Selector*/
const Value V24421_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_324 = {S43, 0, &V24421_324_57307, &P24421_323};	/*Property = Svg*/
const Value V24421_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_327 = {S44, 0, &V24421_327_57316, &P24421_324};	/*Property = TouchEvents*/
const Value V24421_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_328 = {S45, 0, &V24421_328_57319, &P24421_327};	/*Property = Track*/
const Value V24421_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_329 = {S46, 0, &V24421_329_57325, &P24421_328};	/*Property = Video*/
const Value V24421_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_330 = {S47, 0, &V24421_330_57328, &P24421_329};	/*Property = Viewport*/
const Value V24421_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_332 = {S48, 0, &V24421_332_57340, &P24421_330};	/*Property = WebWorkers*/
const Value V24421_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24421_333 = {S49, 0, &V24421_333_57343, &P24421_332};	/*Property = Xhr2*/
const Profile P24421 = {"24421", &P24421_333};	/*Profile ID - 24421*/
const Value V22630_1_1 = {S2, NULL};	/*Value = True*/
const Property P22630_1 = {S50, 0, &V22630_1_1, NULL};	/*Property = IsMobile*/
const Value V22630_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22630_5 = {S52, 0, &V22630_5_1227, &P22630_1};	/*Property = ScreenPixelsHeight*/
const Value V22630_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22630_4 = {S54, 0, &V22630_4_1229, &P22630_5};	/*Property = ScreenPixelsWidth*/
const Profile P22630 = {"22630", &P22630_4};	/*Profile ID - 22630*/
const Value V21290_1_1 = {S2, NULL};	/*Value = True*/
const Property P21290_1 = {S50, 0, &V21290_1_1, NULL};	/*Property = IsMobile*/
const Value V21290_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P21290_5 = {S52, 0, &V21290_5_37569, &P21290_1};	/*Property = ScreenPixelsHeight*/
const Value V21290_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21290_4 = {S54, 0, &V21290_4_5645, &P21290_5};	/*Property = ScreenPixelsWidth*/
const Profile P21290 = {"21290", &P21290_4};	/*Profile ID - 21290*/
const Profile P24699 = {"24699", NULL};	/*Profile ID - 24699*/
const Value V21546_1_1 = {S2, NULL};	/*Value = True*/
const Property P21546_1 = {S50, 0, &V21546_1_1, NULL};	/*Property = IsMobile*/
const Value V21546_5_62105 = {S144, NULL};	/*Value = 2048*/
const Property P21546_5 = {S52, 0, &V21546_5_62105, &P21546_1};	/*Property = ScreenPixelsHeight*/
const Value V21546_4_62106 = {S145, NULL};	/*Value = 1536*/
const Property P21546_4 = {S54, 0, &V21546_4_62106, &P21546_5};	/*Property = ScreenPixelsWidth*/
const Profile P21546 = {"21546", &P21546_4};	/*Profile ID - 21546*/
const Value V21710_279_57147 = {S2, NULL};	/*Value = True*/
const Property P21710_279 = {S1, 0, &V21710_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V21710_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21710_282 = {S3, 0, &V21710_282_57154, &P21710_279};	/*Property = BlobBuilder*/
const Value V21710_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21710_350 = {S4, 0, &V21710_350_57444, &P21710_282};	/*Property = Canvas*/
const Value V21710_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21710_283 = {S5, 0, &V21710_283_57159, &P21710_350};	/*Property = CssBackground*/
const Value V21710_284_57162 = {S2, NULL};	/*Value = True*/
const Property P21710_284 = {S6, 0, &V21710_284_57162, &P21710_283};	/*Property = CssBorderImage*/
const Value V21710_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21710_285 = {S7, 0, &V21710_285_57166, &P21710_284};	/*Property = CssCanvas*/
const Value V21710_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21710_286 = {S8, 0, &V21710_286_57168, &P21710_285};	/*Property = CssColor*/
const Value V21710_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21710_237 = {S9, 0, &V21710_237_55786, &P21710_286};	/*Property = CssColumn*/
const Value V21710_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21710_287 = {S10, 0, &V21710_287_57172, &P21710_237};	/*Property = CssFlexbox*/
const Value V21710_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21710_288 = {S11, 0, &V21710_288_57175, &P21710_287};	/*Property = CssFont*/
const Value V21710_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21710_289 = {S12, 0, &V21710_289_57759, &P21710_288};	/*Property = CssImages*/
const Value V21710_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21710_291 = {S13, 0, &V21710_291_57180, &P21710_289};	/*Property = CssMediaQueries*/
const Value V21710_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21710_292 = {S14, 0, &V21710_292_57183, &P21710_291};	/*Property = CssMinMax*/
const Value V21710_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21710_293 = {S15, 0, &V21710_293_57187, &P21710_292};	/*Property = CssOverflow*/
const Value V21710_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21710_294 = {S16, 0, &V21710_294_57189, &P21710_293};	/*Property = CssPosition*/
const Value V21710_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21710_295 = {S17, 0, &V21710_295_57192, &P21710_294};	/*Property = CssText*/
const Value V21710_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21710_297 = {S18, 0, &V21710_297_57195, &P21710_295};	/*Property = CssTransforms*/
const Value V21710_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21710_241 = {S19, 0, &V21710_241_55799, &P21710_297};	/*Property = CssTransitions*/
const Value V21710_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21710_298 = {S20, 0, &V21710_298_57206, &P21710_241};	/*Property = CssUI*/
const Value V21710_300_57215 = {S2, NULL};	/*Value = True*/
const Property P21710_300 = {S21, 0, &V21710_300_57215, &P21710_298};	/*Property = DataSet*/
const Value V21710_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21710_301 = {S22, 0, &V21710_301_57218, &P21710_300};	/*Property = DataUrl*/
const Value V21710_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21710_302 = {S23, 0, &V21710_302_57222, &P21710_301};	/*Property = DeviceOrientation*/
const Value V21710_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21710_304 = {S24, 0, &V21710_304_57227, &P21710_302};	/*Property = FileReader*/
const Value V21710_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21710_305 = {S25, 0, &V21710_305_57231, &P21710_304};	/*Property = FileSaver*/
const Value V21710_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21710_306 = {S26, 0, &V21710_306_57237, &P21710_305};	/*Property = FileWriter*/
const Value V21710_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21710_307 = {S27, 0, &V21710_307_57239, &P21710_306};	/*Property = FormData*/
const Value V21710_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21710_309 = {S28, 0, &V21710_309_57246, &P21710_307};	/*Property = Fullscreen*/
const Value V21710_310_57249 = {S0, NULL};	/*Value = False*/
const Property P21710_310 = {S29, 0, &V21710_310_57249, &P21710_309};	/*Property = GeoLocation*/
const Value V21710_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21710_312 = {S30, 0, &V21710_312_57254, &P21710_310};	/*Property = History*/
const Value V21710_314_57260 = {S2, NULL};	/*Value = True*/
const Property P21710_314 = {S31, 0, &V21710_314_57260, &P21710_312};	/*Property = Html5*/
const Value V21710_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21710_313 = {S32, 0, &V21710_313_57258, &P21710_314};	/*Property = Html-Media-Capture*/
const Value V21710_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21710_315 = {S33, 0, &V21710_315_57264, &P21710_313};	/*Property = Iframe*/
const Value V21710_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21710_316 = {S34, 0, &V21710_316_57267, &P21710_315};	/*Property = IndexedDB*/
const Value V21710_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21710_317 = {S35, 0, &V21710_317_57269, &P21710_316};	/*Property = Json*/
const Value V21710_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21710_15 = {S37, 0, &V21710_15_54167, &P21710_317};	/*Property = LayoutEngine*/
const Value V21710_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21710_344 = {S38, 0, &V21710_344_57272, &P21710_15};	/*Property = Masking*/
const Value V21710_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21710_320 = {S39, 0, &V21710_320_57290, &P21710_344};	/*Property = PostMessage*/
const Value V21710_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21710_321 = {S40, 0, &V21710_321_57293, &P21710_320};	/*Property = Progress*/
const Value V21710_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21710_322 = {S41, 0, &V21710_322_57296, &P21710_321};	/*Property = Prompts*/
const Value V21710_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21710_323 = {S42, 0, &V21710_323_57299, &P21710_322};	/*Property = Selector*/
const Value V21710_324_57305 = {S2, NULL};	/*Value = True*/
const Property P21710_324 = {S43, 0, &V21710_324_57305, &P21710_323};	/*Property = Svg*/
const Value V21710_327_57314 = {S2, NULL};	/*Value = True*/
const Property P21710_327 = {S44, 0, &V21710_327_57314, &P21710_324};	/*Property = TouchEvents*/
const Value V21710_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21710_328 = {S45, 0, &V21710_328_57318, &P21710_327};	/*Property = Track*/
const Value V21710_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21710_329 = {S46, 0, &V21710_329_57323, &P21710_328};	/*Property = Video*/
const Value V21710_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21710_330 = {S47, 0, &V21710_330_57326, &P21710_329};	/*Property = Viewport*/
const Value V21710_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21710_332 = {S48, 0, &V21710_332_57339, &P21710_330};	/*Property = WebWorkers*/
const Value V21710_333_57341 = {S2, NULL};	/*Value = True*/
const Property P21710_333 = {S49, 0, &V21710_333_57341, &P21710_332};	/*Property = Xhr2*/
const Profile P21710 = {"21710", &P21710_333};	/*Profile ID - 21710*/
const Value V23841_1_1 = {S2, NULL};	/*Value = True*/
const Property P23841_1 = {S50, 0, &V23841_1_1, NULL};	/*Property = IsMobile*/
const Value V23841_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23841_5 = {S52, 0, &V23841_5_1277, &P23841_1};	/*Property = ScreenPixelsHeight*/
const Value V23841_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23841_4 = {S54, 0, &V23841_4_1279, &P23841_5};	/*Property = ScreenPixelsWidth*/
const Profile P23841 = {"23841", &P23841_4};	/*Profile ID - 23841*/
const Value V23970_1_1 = {S2, NULL};	/*Value = True*/
const Property P23970_1 = {S50, 0, &V23970_1_1, NULL};	/*Property = IsMobile*/
const Value V23970_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23970_5 = {S52, 0, &V23970_5_1227, &P23970_1};	/*Property = ScreenPixelsHeight*/
const Value V23970_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23970_4 = {S54, 0, &V23970_4_1229, &P23970_5};	/*Property = ScreenPixelsWidth*/
const Profile P23970 = {"23970", &P23970_4};	/*Profile ID - 23970*/
const Value V21413_1_1 = {S2, NULL};	/*Value = True*/
const Property P21413_1 = {S50, 0, &V21413_1_1, NULL};	/*Property = IsMobile*/
const Value V21413_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21413_5 = {S52, 0, &V21413_5_1277, &P21413_1};	/*Property = ScreenPixelsHeight*/
const Value V21413_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21413_4 = {S54, 0, &V21413_4_1279, &P21413_5};	/*Property = ScreenPixelsWidth*/
const Profile P21413 = {"21413", &P21413_4};	/*Profile ID - 21413*/
const Value V20492_1_1 = {S2, NULL};	/*Value = True*/
const Property P20492_1 = {S50, 0, &V20492_1_1, NULL};	/*Property = IsMobile*/
const Value V20492_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20492_5 = {S52, 0, &V20492_5_1227, &P20492_1};	/*Property = ScreenPixelsHeight*/
const Value V20492_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20492_4 = {S54, 0, &V20492_4_1229, &P20492_5};	/*Property = ScreenPixelsWidth*/
const Profile P20492 = {"20492", &P20492_4};	/*Profile ID - 20492*/
const Value V22634_1_1 = {S2, NULL};	/*Value = True*/
const Property P22634_1 = {S50, 0, &V22634_1_1, NULL};	/*Property = IsMobile*/
const Value V22634_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22634_5 = {S52, 0, &V22634_5_9981, &P22634_1};	/*Property = ScreenPixelsHeight*/
const Value V22634_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22634_4 = {S54, 0, &V22634_4_2336, &P22634_5};	/*Property = ScreenPixelsWidth*/
const Profile P22634 = {"22634", &P22634_4};	/*Profile ID - 22634*/
const Value V21869_1_1 = {S2, NULL};	/*Value = True*/
const Property P21869_1 = {S50, 0, &V21869_1_1, NULL};	/*Property = IsMobile*/
const Value V21869_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21869_5 = {S52, 0, &V21869_5_9981, &P21869_1};	/*Property = ScreenPixelsHeight*/
const Value V21869_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21869_4 = {S54, 0, &V21869_4_39098, &P21869_5};	/*Property = ScreenPixelsWidth*/
const Profile P21869 = {"21869", &P21869_4};	/*Profile ID - 21869*/
const Value V21150_1_1 = {S2, NULL};	/*Value = True*/
const Property P21150_1 = {S50, 0, &V21150_1_1, NULL};	/*Property = IsMobile*/
const Value V21150_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21150_5 = {S52, 0, &V21150_5_1458, &P21150_1};	/*Property = ScreenPixelsHeight*/
const Value V21150_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21150_4 = {S54, 0, &V21150_4_37617, &P21150_5};	/*Property = ScreenPixelsWidth*/
const Profile P21150 = {"21150", &P21150_4};	/*Profile ID - 21150*/
const Value V18693_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_279 = {S1, 0, &V18693_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18693_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_282 = {S3, 0, &V18693_282_57155, &P18693_279};	/*Property = BlobBuilder*/
const Value V18693_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_350 = {S4, 0, &V18693_350_57446, &P18693_282};	/*Property = Canvas*/
const Value V18693_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_283 = {S5, 0, &V18693_283_57161, &P18693_350};	/*Property = CssBackground*/
const Value V18693_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_284 = {S6, 0, &V18693_284_57164, &P18693_283};	/*Property = CssBorderImage*/
const Value V18693_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_285 = {S7, 0, &V18693_285_57167, &P18693_284};	/*Property = CssCanvas*/
const Value V18693_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_286 = {S8, 0, &V18693_286_57170, &P18693_285};	/*Property = CssColor*/
const Value V18693_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_237 = {S9, 0, &V18693_237_55786, &P18693_286};	/*Property = CssColumn*/
const Value V18693_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_287 = {S10, 0, &V18693_287_57173, &P18693_237};	/*Property = CssFlexbox*/
const Value V18693_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_288 = {S11, 0, &V18693_288_57176, &P18693_287};	/*Property = CssFont*/
const Value V18693_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_289 = {S12, 0, &V18693_289_86899, &P18693_288};	/*Property = CssImages*/
const Value V18693_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_291 = {S13, 0, &V18693_291_57182, &P18693_289};	/*Property = CssMediaQueries*/
const Value V18693_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_292 = {S14, 0, &V18693_292_57185, &P18693_291};	/*Property = CssMinMax*/
const Value V18693_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_293 = {S15, 0, &V18693_293_57188, &P18693_292};	/*Property = CssOverflow*/
const Value V18693_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_294 = {S16, 0, &V18693_294_57191, &P18693_293};	/*Property = CssPosition*/
const Value V18693_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_295 = {S17, 0, &V18693_295_57194, &P18693_294};	/*Property = CssText*/
const Value V18693_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_297 = {S18, 0, &V18693_297_57197, &P18693_295};	/*Property = CssTransforms*/
const Value V18693_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_241 = {S19, 0, &V18693_241_55801, &P18693_297};	/*Property = CssTransitions*/
const Value V18693_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_298 = {S20, 0, &V18693_298_57208, &P18693_241};	/*Property = CssUI*/
const Value V18693_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_300 = {S21, 0, &V18693_300_57217, &P18693_298};	/*Property = DataSet*/
const Value V18693_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_301 = {S22, 0, &V18693_301_57220, &P18693_300};	/*Property = DataUrl*/
const Value V18693_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_302 = {S23, 0, &V18693_302_57223, &P18693_301};	/*Property = DeviceOrientation*/
const Value V18693_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_304 = {S24, 0, &V18693_304_57229, &P18693_302};	/*Property = FileReader*/
const Value V18693_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_305 = {S25, 0, &V18693_305_57232, &P18693_304};	/*Property = FileSaver*/
const Value V18693_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_306 = {S26, 0, &V18693_306_57238, &P18693_305};	/*Property = FileWriter*/
const Value V18693_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_307 = {S27, 0, &V18693_307_57241, &P18693_306};	/*Property = FormData*/
const Value V18693_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_309 = {S28, 0, &V18693_309_57247, &P18693_307};	/*Property = Fullscreen*/
const Value V18693_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_310 = {S29, 0, &V18693_310_57250, &P18693_309};	/*Property = GeoLocation*/
const Value V18693_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_312 = {S30, 0, &V18693_312_57256, &P18693_310};	/*Property = History*/
const Value V18693_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_314 = {S31, 0, &V18693_314_57262, &P18693_312};	/*Property = Html5*/
const Value V18693_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_313 = {S32, 0, &V18693_313_57259, &P18693_314};	/*Property = Html-Media-Capture*/
const Value V18693_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_315 = {S33, 0, &V18693_315_57265, &P18693_313};	/*Property = Iframe*/
const Value V18693_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_316 = {S34, 0, &V18693_316_57268, &P18693_315};	/*Property = IndexedDB*/
const Value V18693_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_317 = {S35, 0, &V18693_317_57271, &P18693_316};	/*Property = Json*/
const Value V18693_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18693_15 = {S37, 0, &V18693_15_28, &P18693_317};	/*Property = LayoutEngine*/
const Value V18693_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_344 = {S38, 0, &V18693_344_57274, &P18693_15};	/*Property = Masking*/
const Value V18693_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_320 = {S39, 0, &V18693_320_57292, &P18693_344};	/*Property = PostMessage*/
const Value V18693_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_321 = {S40, 0, &V18693_321_57295, &P18693_320};	/*Property = Progress*/
const Value V18693_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_322 = {S41, 0, &V18693_322_57298, &P18693_321};	/*Property = Prompts*/
const Value V18693_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_323 = {S42, 0, &V18693_323_57301, &P18693_322};	/*Property = Selector*/
const Value V18693_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_324 = {S43, 0, &V18693_324_57307, &P18693_323};	/*Property = Svg*/
const Value V18693_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_327 = {S44, 0, &V18693_327_57316, &P18693_324};	/*Property = TouchEvents*/
const Value V18693_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_328 = {S45, 0, &V18693_328_57319, &P18693_327};	/*Property = Track*/
const Value V18693_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_329 = {S46, 0, &V18693_329_57325, &P18693_328};	/*Property = Video*/
const Value V18693_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_330 = {S47, 0, &V18693_330_57328, &P18693_329};	/*Property = Viewport*/
const Value V18693_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_332 = {S48, 0, &V18693_332_57340, &P18693_330};	/*Property = WebWorkers*/
const Value V18693_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18693_333 = {S49, 0, &V18693_333_57343, &P18693_332};	/*Property = Xhr2*/
const Profile P18693 = {"18693", &P18693_333};	/*Profile ID - 18693*/
const Value V23510_1_1 = {S2, NULL};	/*Value = True*/
const Property P23510_1 = {S50, 0, &V23510_1_1, NULL};	/*Property = IsMobile*/
const Value V23510_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23510_5 = {S52, 0, &V23510_5_1227, &P23510_1};	/*Property = ScreenPixelsHeight*/
const Value V23510_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23510_4 = {S54, 0, &V23510_4_1229, &P23510_5};	/*Property = ScreenPixelsWidth*/
const Profile P23510 = {"23510", &P23510_4};	/*Profile ID - 23510*/
const Value V23464_1_1 = {S2, NULL};	/*Value = True*/
const Property P23464_1 = {S50, 0, &V23464_1_1, NULL};	/*Property = IsMobile*/
const Value V23464_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23464_5 = {S52, 0, &V23464_5_1227, &P23464_1};	/*Property = ScreenPixelsHeight*/
const Value V23464_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P23464_4 = {S54, 0, &V23464_4_1943, &P23464_5};	/*Property = ScreenPixelsWidth*/
const Profile P23464 = {"23464", &P23464_4};	/*Profile ID - 23464*/
const Value V21167_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21167_279 = {S1, 0, &V21167_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21167_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21167_282 = {S3, 0, &V21167_282_57154, &P21167_279};	/*Property = BlobBuilder*/
const Value V21167_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21167_350 = {S4, 0, &V21167_350_57444, &P21167_282};	/*Property = Canvas*/
const Value V21167_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21167_283 = {S5, 0, &V21167_283_57159, &P21167_350};	/*Property = CssBackground*/
const Value V21167_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21167_284 = {S6, 0, &V21167_284_57163, &P21167_283};	/*Property = CssBorderImage*/
const Value V21167_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21167_285 = {S7, 0, &V21167_285_57166, &P21167_284};	/*Property = CssCanvas*/
const Value V21167_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21167_286 = {S8, 0, &V21167_286_57168, &P21167_285};	/*Property = CssColor*/
const Value V21167_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21167_237 = {S9, 0, &V21167_237_55786, &P21167_286};	/*Property = CssColumn*/
const Value V21167_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21167_287 = {S10, 0, &V21167_287_57172, &P21167_237};	/*Property = CssFlexbox*/
const Value V21167_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21167_288 = {S11, 0, &V21167_288_57175, &P21167_287};	/*Property = CssFont*/
const Value V21167_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21167_289 = {S12, 0, &V21167_289_57759, &P21167_288};	/*Property = CssImages*/
const Value V21167_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21167_291 = {S13, 0, &V21167_291_57180, &P21167_289};	/*Property = CssMediaQueries*/
const Value V21167_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21167_292 = {S14, 0, &V21167_292_57183, &P21167_291};	/*Property = CssMinMax*/
const Value V21167_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21167_293 = {S15, 0, &V21167_293_57187, &P21167_292};	/*Property = CssOverflow*/
const Value V21167_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21167_294 = {S16, 0, &V21167_294_57189, &P21167_293};	/*Property = CssPosition*/
const Value V21167_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21167_295 = {S17, 0, &V21167_295_57192, &P21167_294};	/*Property = CssText*/
const Value V21167_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21167_297 = {S18, 0, &V21167_297_57195, &P21167_295};	/*Property = CssTransforms*/
const Value V21167_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21167_241 = {S19, 0, &V21167_241_55799, &P21167_297};	/*Property = CssTransitions*/
const Value V21167_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21167_298 = {S20, 0, &V21167_298_57206, &P21167_241};	/*Property = CssUI*/
const Value V21167_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21167_300 = {S21, 0, &V21167_300_57216, &P21167_298};	/*Property = DataSet*/
const Value V21167_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21167_301 = {S22, 0, &V21167_301_57218, &P21167_300};	/*Property = DataUrl*/
const Value V21167_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21167_302 = {S23, 0, &V21167_302_57222, &P21167_301};	/*Property = DeviceOrientation*/
const Value V21167_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21167_304 = {S24, 0, &V21167_304_57227, &P21167_302};	/*Property = FileReader*/
const Value V21167_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21167_305 = {S25, 0, &V21167_305_57231, &P21167_304};	/*Property = FileSaver*/
const Value V21167_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21167_306 = {S26, 0, &V21167_306_57237, &P21167_305};	/*Property = FileWriter*/
const Value V21167_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21167_307 = {S27, 0, &V21167_307_57239, &P21167_306};	/*Property = FormData*/
const Value V21167_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21167_309 = {S28, 0, &V21167_309_57246, &P21167_307};	/*Property = Fullscreen*/
const Value V21167_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21167_310 = {S29, 0, &V21167_310_57248, &P21167_309};	/*Property = GeoLocation*/
const Value V21167_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21167_312 = {S30, 0, &V21167_312_57254, &P21167_310};	/*Property = History*/
const Value V21167_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21167_314 = {S31, 0, &V21167_314_57261, &P21167_312};	/*Property = Html5*/
const Value V21167_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21167_313 = {S32, 0, &V21167_313_57258, &P21167_314};	/*Property = Html-Media-Capture*/
const Value V21167_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21167_315 = {S33, 0, &V21167_315_57264, &P21167_313};	/*Property = Iframe*/
const Value V21167_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21167_316 = {S34, 0, &V21167_316_57266, &P21167_315};	/*Property = IndexedDB*/
const Value V21167_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21167_317 = {S35, 0, &V21167_317_57269, &P21167_316};	/*Property = Json*/
const Value V21167_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21167_15 = {S37, 0, &V21167_15_28, &P21167_317};	/*Property = LayoutEngine*/
const Value V21167_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21167_344 = {S38, 0, &V21167_344_57272, &P21167_15};	/*Property = Masking*/
const Value V21167_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21167_320 = {S39, 0, &V21167_320_57290, &P21167_344};	/*Property = PostMessage*/
const Value V21167_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21167_321 = {S40, 0, &V21167_321_57293, &P21167_320};	/*Property = Progress*/
const Value V21167_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21167_322 = {S41, 0, &V21167_322_57296, &P21167_321};	/*Property = Prompts*/
const Value V21167_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21167_323 = {S42, 0, &V21167_323_57299, &P21167_322};	/*Property = Selector*/
const Value V21167_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21167_324 = {S43, 0, &V21167_324_57306, &P21167_323};	/*Property = Svg*/
const Value V21167_327_57314 = {S2, NULL};	/*Value = True*/
const Property P21167_327 = {S44, 0, &V21167_327_57314, &P21167_324};	/*Property = TouchEvents*/
const Value V21167_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21167_328 = {S45, 0, &V21167_328_57318, &P21167_327};	/*Property = Track*/
const Value V21167_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21167_329 = {S46, 0, &V21167_329_57323, &P21167_328};	/*Property = Video*/
const Value V21167_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21167_330 = {S47, 0, &V21167_330_57326, &P21167_329};	/*Property = Viewport*/
const Value V21167_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21167_332 = {S48, 0, &V21167_332_57339, &P21167_330};	/*Property = WebWorkers*/
const Value V21167_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21167_333 = {S49, 0, &V21167_333_57342, &P21167_332};	/*Property = Xhr2*/
const Profile P21167 = {"21167", &P21167_333};	/*Profile ID - 21167*/
const Value V21472_1_1 = {S2, NULL};	/*Value = True*/
const Property P21472_1 = {S50, 0, &V21472_1_1, NULL};	/*Property = IsMobile*/
const Value V21472_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21472_5 = {S52, 0, &V21472_5_1227, &P21472_1};	/*Property = ScreenPixelsHeight*/
const Value V21472_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21472_4 = {S54, 0, &V21472_4_1229, &P21472_5};	/*Property = ScreenPixelsWidth*/
const Profile P21472 = {"21472", &P21472_4};	/*Profile ID - 21472*/
const Value V21194_1_1 = {S2, NULL};	/*Value = True*/
const Property P21194_1 = {S50, 0, &V21194_1_1, NULL};	/*Property = IsMobile*/
const Value V21194_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21194_5 = {S52, 0, &V21194_5_37, &P21194_1};	/*Property = ScreenPixelsHeight*/
const Value V21194_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21194_4 = {S54, 0, &V21194_4_36, &P21194_5};	/*Property = ScreenPixelsWidth*/
const Profile P21194 = {"21194", &P21194_4};	/*Profile ID - 21194*/
const Value V21769_1_1 = {S2, NULL};	/*Value = True*/
const Property P21769_1 = {S50, 0, &V21769_1_1, NULL};	/*Property = IsMobile*/
const Value V21769_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21769_5 = {S52, 0, &V21769_5_1227, &P21769_1};	/*Property = ScreenPixelsHeight*/
const Value V21769_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P21769_4 = {S54, 0, &V21769_4_1477, &P21769_5};	/*Property = ScreenPixelsWidth*/
const Profile P21769 = {"21769", &P21769_4};	/*Profile ID - 21769*/
const Value V21880_1_1 = {S2, NULL};	/*Value = True*/
const Property P21880_1 = {S50, 0, &V21880_1_1, NULL};	/*Property = IsMobile*/
const Value V21880_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21880_5 = {S52, 0, &V21880_5_1227, &P21880_1};	/*Property = ScreenPixelsHeight*/
const Value V21880_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21880_4 = {S54, 0, &V21880_4_1229, &P21880_5};	/*Property = ScreenPixelsWidth*/
const Profile P21880 = {"21880", &P21880_4};	/*Profile ID - 21880*/
const Value V23803_1_1 = {S2, NULL};	/*Value = True*/
const Property P23803_1 = {S50, 0, &V23803_1_1, NULL};	/*Property = IsMobile*/
const Value V23803_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23803_5 = {S52, 0, &V23803_5_1227, &P23803_1};	/*Property = ScreenPixelsHeight*/
const Value V23803_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23803_4 = {S54, 0, &V23803_4_1229, &P23803_5};	/*Property = ScreenPixelsWidth*/
const Profile P23803 = {"23803", &P23803_4};	/*Profile ID - 23803*/
const Value V21851_1_1 = {S2, NULL};	/*Value = True*/
const Property P21851_1 = {S50, 0, &V21851_1_1, NULL};	/*Property = IsMobile*/
const Value V21851_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21851_5 = {S52, 0, &V21851_5_1277, &P21851_1};	/*Property = ScreenPixelsHeight*/
const Value V21851_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21851_4 = {S54, 0, &V21851_4_1279, &P21851_5};	/*Property = ScreenPixelsWidth*/
const Profile P21851 = {"21851", &P21851_4};	/*Profile ID - 21851*/
const Value V21771_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21771_279 = {S1, 0, &V21771_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21771_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21771_282 = {S3, 0, &V21771_282_57154, &P21771_279};	/*Property = BlobBuilder*/
const Value V21771_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21771_350 = {S4, 0, &V21771_350_57444, &P21771_282};	/*Property = Canvas*/
const Value V21771_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21771_283 = {S5, 0, &V21771_283_57159, &P21771_350};	/*Property = CssBackground*/
const Value V21771_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21771_284 = {S6, 0, &V21771_284_57163, &P21771_283};	/*Property = CssBorderImage*/
const Value V21771_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21771_285 = {S7, 0, &V21771_285_57166, &P21771_284};	/*Property = CssCanvas*/
const Value V21771_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21771_286 = {S8, 0, &V21771_286_57168, &P21771_285};	/*Property = CssColor*/
const Value V21771_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21771_237 = {S9, 0, &V21771_237_55786, &P21771_286};	/*Property = CssColumn*/
const Value V21771_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21771_287 = {S10, 0, &V21771_287_57172, &P21771_237};	/*Property = CssFlexbox*/
const Value V21771_288_57174 = {S2, NULL};	/*Value = True*/
const Property P21771_288 = {S11, 0, &V21771_288_57174, &P21771_287};	/*Property = CssFont*/
const Value V21771_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21771_289 = {S12, 0, &V21771_289_57759, &P21771_288};	/*Property = CssImages*/
const Value V21771_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21771_291 = {S13, 0, &V21771_291_57180, &P21771_289};	/*Property = CssMediaQueries*/
const Value V21771_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21771_292 = {S14, 0, &V21771_292_57183, &P21771_291};	/*Property = CssMinMax*/
const Value V21771_293_57186 = {S2, NULL};	/*Value = True*/
const Property P21771_293 = {S15, 0, &V21771_293_57186, &P21771_292};	/*Property = CssOverflow*/
const Value V21771_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21771_294 = {S16, 0, &V21771_294_57189, &P21771_293};	/*Property = CssPosition*/
const Value V21771_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21771_295 = {S17, 0, &V21771_295_57192, &P21771_294};	/*Property = CssText*/
const Value V21771_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21771_297 = {S18, 0, &V21771_297_57195, &P21771_295};	/*Property = CssTransforms*/
const Value V21771_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21771_241 = {S19, 0, &V21771_241_55799, &P21771_297};	/*Property = CssTransitions*/
const Value V21771_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21771_298 = {S20, 0, &V21771_298_57206, &P21771_241};	/*Property = CssUI*/
const Value V21771_300_57215 = {S2, NULL};	/*Value = True*/
const Property P21771_300 = {S21, 0, &V21771_300_57215, &P21771_298};	/*Property = DataSet*/
const Value V21771_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21771_301 = {S22, 0, &V21771_301_57218, &P21771_300};	/*Property = DataUrl*/
const Value V21771_302_57221 = {S2, NULL};	/*Value = True*/
const Property P21771_302 = {S23, 0, &V21771_302_57221, &P21771_301};	/*Property = DeviceOrientation*/
const Value V21771_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21771_304 = {S24, 0, &V21771_304_57227, &P21771_302};	/*Property = FileReader*/
const Value V21771_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21771_305 = {S25, 0, &V21771_305_57231, &P21771_304};	/*Property = FileSaver*/
const Value V21771_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21771_306 = {S26, 0, &V21771_306_57237, &P21771_305};	/*Property = FileWriter*/
const Value V21771_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21771_307 = {S27, 0, &V21771_307_57239, &P21771_306};	/*Property = FormData*/
const Value V21771_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21771_309 = {S28, 0, &V21771_309_57246, &P21771_307};	/*Property = Fullscreen*/
const Value V21771_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21771_310 = {S29, 0, &V21771_310_57248, &P21771_309};	/*Property = GeoLocation*/
const Value V21771_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21771_312 = {S30, 0, &V21771_312_57255, &P21771_310};	/*Property = History*/
const Value V21771_314_57260 = {S2, NULL};	/*Value = True*/
const Property P21771_314 = {S31, 0, &V21771_314_57260, &P21771_312};	/*Property = Html5*/
const Value V21771_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21771_313 = {S32, 0, &V21771_313_57258, &P21771_314};	/*Property = Html-Media-Capture*/
const Value V21771_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21771_315 = {S33, 0, &V21771_315_57264, &P21771_313};	/*Property = Iframe*/
const Value V21771_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21771_316 = {S34, 0, &V21771_316_57267, &P21771_315};	/*Property = IndexedDB*/
const Value V21771_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21771_317 = {S35, 0, &V21771_317_57269, &P21771_316};	/*Property = Json*/
const Value V21771_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21771_15 = {S37, 0, &V21771_15_54167, &P21771_317};	/*Property = LayoutEngine*/
const Value V21771_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21771_344 = {S38, 0, &V21771_344_57272, &P21771_15};	/*Property = Masking*/
const Value V21771_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21771_320 = {S39, 0, &V21771_320_57290, &P21771_344};	/*Property = PostMessage*/
const Value V21771_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21771_321 = {S40, 0, &V21771_321_57293, &P21771_320};	/*Property = Progress*/
const Value V21771_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21771_322 = {S41, 0, &V21771_322_57296, &P21771_321};	/*Property = Prompts*/
const Value V21771_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21771_323 = {S42, 0, &V21771_323_57299, &P21771_322};	/*Property = Selector*/
const Value V21771_324_57305 = {S2, NULL};	/*Value = True*/
const Property P21771_324 = {S43, 0, &V21771_324_57305, &P21771_323};	/*Property = Svg*/
const Value V21771_327_57314 = {S2, NULL};	/*Value = True*/
const Property P21771_327 = {S44, 0, &V21771_327_57314, &P21771_324};	/*Property = TouchEvents*/
const Value V21771_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21771_328 = {S45, 0, &V21771_328_57318, &P21771_327};	/*Property = Track*/
const Value V21771_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21771_329 = {S46, 0, &V21771_329_57323, &P21771_328};	/*Property = Video*/
const Value V21771_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21771_330 = {S47, 0, &V21771_330_57326, &P21771_329};	/*Property = Viewport*/
const Value V21771_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21771_332 = {S48, 0, &V21771_332_57339, &P21771_330};	/*Property = WebWorkers*/
const Value V21771_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21771_333 = {S49, 0, &V21771_333_57342, &P21771_332};	/*Property = Xhr2*/
const Profile P21771 = {"21771", &P21771_333};	/*Profile ID - 21771*/
const Value V21160_1_1 = {S2, NULL};	/*Value = True*/
const Property P21160_1 = {S50, 0, &V21160_1_1, NULL};	/*Property = IsMobile*/
const Value V21160_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21160_5 = {S52, 0, &V21160_5_37, &P21160_1};	/*Property = ScreenPixelsHeight*/
const Value V21160_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21160_4 = {S54, 0, &V21160_4_36, &P21160_5};	/*Property = ScreenPixelsWidth*/
const Profile P21160 = {"21160", &P21160_4};	/*Profile ID - 21160*/
const Value V21623_1_1 = {S2, NULL};	/*Value = True*/
const Property P21623_1 = {S50, 0, &V21623_1_1, NULL};	/*Property = IsMobile*/
const Value V21623_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21623_5 = {S52, 0, &V21623_5_9981, &P21623_1};	/*Property = ScreenPixelsHeight*/
const Value V21623_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21623_4 = {S54, 0, &V21623_4_39098, &P21623_5};	/*Property = ScreenPixelsWidth*/
const Profile P21623 = {"21623", &P21623_4};	/*Profile ID - 21623*/
const Value V21820_1_1 = {S2, NULL};	/*Value = True*/
const Property P21820_1 = {S50, 0, &V21820_1_1, NULL};	/*Property = IsMobile*/
const Value V21820_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21820_5 = {S52, 0, &V21820_5_1227, &P21820_1};	/*Property = ScreenPixelsHeight*/
const Value V21820_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21820_4 = {S54, 0, &V21820_4_1229, &P21820_5};	/*Property = ScreenPixelsWidth*/
const Profile P21820 = {"21820", &P21820_4};	/*Profile ID - 21820*/
const Value V21407_1_1 = {S2, NULL};	/*Value = True*/
const Property P21407_1 = {S50, 0, &V21407_1_1, NULL};	/*Property = IsMobile*/
const Value V21407_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21407_5 = {S52, 0, &V21407_5_9981, &P21407_1};	/*Property = ScreenPixelsHeight*/
const Value V21407_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21407_4 = {S54, 0, &V21407_4_37617, &P21407_5};	/*Property = ScreenPixelsWidth*/
const Profile P21407 = {"21407", &P21407_4};	/*Profile ID - 21407*/
const Value V21736_1_1 = {S2, NULL};	/*Value = True*/
const Property P21736_1 = {S50, 0, &V21736_1_1, NULL};	/*Property = IsMobile*/
const Value V21736_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21736_5 = {S52, 0, &V21736_5_1277, &P21736_1};	/*Property = ScreenPixelsHeight*/
const Value V21736_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21736_4 = {S54, 0, &V21736_4_1279, &P21736_5};	/*Property = ScreenPixelsWidth*/
const Profile P21736 = {"21736", &P21736_4};	/*Profile ID - 21736*/
const Value V21850_1_1 = {S2, NULL};	/*Value = True*/
const Property P21850_1 = {S50, 0, &V21850_1_1, NULL};	/*Property = IsMobile*/
const Value V21850_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21850_5 = {S52, 0, &V21850_5_1227, &P21850_1};	/*Property = ScreenPixelsHeight*/
const Value V21850_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21850_4 = {S54, 0, &V21850_4_1229, &P21850_5};	/*Property = ScreenPixelsWidth*/
const Profile P21850 = {"21850", &P21850_4};	/*Profile ID - 21850*/
const Value V21169_279_57147 = {S2, NULL};	/*Value = True*/
const Property P21169_279 = {S1, 0, &V21169_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V21169_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21169_282 = {S3, 0, &V21169_282_57154, &P21169_279};	/*Property = BlobBuilder*/
const Value V21169_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21169_350 = {S4, 0, &V21169_350_57444, &P21169_282};	/*Property = Canvas*/
const Value V21169_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21169_283 = {S5, 0, &V21169_283_57159, &P21169_350};	/*Property = CssBackground*/
const Value V21169_284_57162 = {S2, NULL};	/*Value = True*/
const Property P21169_284 = {S6, 0, &V21169_284_57162, &P21169_283};	/*Property = CssBorderImage*/
const Value V21169_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21169_285 = {S7, 0, &V21169_285_57166, &P21169_284};	/*Property = CssCanvas*/
const Value V21169_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21169_286 = {S8, 0, &V21169_286_57168, &P21169_285};	/*Property = CssColor*/
const Value V21169_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21169_237 = {S9, 0, &V21169_237_55786, &P21169_286};	/*Property = CssColumn*/
const Value V21169_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21169_287 = {S10, 0, &V21169_287_57172, &P21169_237};	/*Property = CssFlexbox*/
const Value V21169_288_57174 = {S2, NULL};	/*Value = True*/
const Property P21169_288 = {S11, 0, &V21169_288_57174, &P21169_287};	/*Property = CssFont*/
const Value V21169_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21169_289 = {S12, 0, &V21169_289_57759, &P21169_288};	/*Property = CssImages*/
const Value V21169_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21169_291 = {S13, 0, &V21169_291_57180, &P21169_289};	/*Property = CssMediaQueries*/
const Value V21169_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21169_292 = {S14, 0, &V21169_292_57183, &P21169_291};	/*Property = CssMinMax*/
const Value V21169_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21169_293 = {S15, 0, &V21169_293_57187, &P21169_292};	/*Property = CssOverflow*/
const Value V21169_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21169_294 = {S16, 0, &V21169_294_57189, &P21169_293};	/*Property = CssPosition*/
const Value V21169_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21169_295 = {S17, 0, &V21169_295_57192, &P21169_294};	/*Property = CssText*/
const Value V21169_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21169_297 = {S18, 0, &V21169_297_57195, &P21169_295};	/*Property = CssTransforms*/
const Value V21169_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21169_241 = {S19, 0, &V21169_241_55799, &P21169_297};	/*Property = CssTransitions*/
const Value V21169_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21169_298 = {S20, 0, &V21169_298_57206, &P21169_241};	/*Property = CssUI*/
const Value V21169_300_57215 = {S2, NULL};	/*Value = True*/
const Property P21169_300 = {S21, 0, &V21169_300_57215, &P21169_298};	/*Property = DataSet*/
const Value V21169_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21169_301 = {S22, 0, &V21169_301_57218, &P21169_300};	/*Property = DataUrl*/
const Value V21169_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21169_302 = {S23, 0, &V21169_302_57222, &P21169_301};	/*Property = DeviceOrientation*/
const Value V21169_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21169_304 = {S24, 0, &V21169_304_57227, &P21169_302};	/*Property = FileReader*/
const Value V21169_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21169_305 = {S25, 0, &V21169_305_57231, &P21169_304};	/*Property = FileSaver*/
const Value V21169_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21169_306 = {S26, 0, &V21169_306_57237, &P21169_305};	/*Property = FileWriter*/
const Value V21169_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21169_307 = {S27, 0, &V21169_307_57239, &P21169_306};	/*Property = FormData*/
const Value V21169_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21169_309 = {S28, 0, &V21169_309_57246, &P21169_307};	/*Property = Fullscreen*/
const Value V21169_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21169_310 = {S29, 0, &V21169_310_57248, &P21169_309};	/*Property = GeoLocation*/
const Value V21169_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21169_312 = {S30, 0, &V21169_312_57254, &P21169_310};	/*Property = History*/
const Value V21169_314_57260 = {S2, NULL};	/*Value = True*/
const Property P21169_314 = {S31, 0, &V21169_314_57260, &P21169_312};	/*Property = Html5*/
const Value V21169_313_57257 = {S2, NULL};	/*Value = True*/
const Property P21169_313 = {S32, 0, &V21169_313_57257, &P21169_314};	/*Property = Html-Media-Capture*/
const Value V21169_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21169_315 = {S33, 0, &V21169_315_57264, &P21169_313};	/*Property = Iframe*/
const Value V21169_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21169_316 = {S34, 0, &V21169_316_57266, &P21169_315};	/*Property = IndexedDB*/
const Value V21169_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21169_317 = {S35, 0, &V21169_317_57269, &P21169_316};	/*Property = Json*/
const Value V21169_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21169_15 = {S37, 0, &V21169_15_28, &P21169_317};	/*Property = LayoutEngine*/
const Value V21169_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21169_344 = {S38, 0, &V21169_344_57272, &P21169_15};	/*Property = Masking*/
const Value V21169_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21169_320 = {S39, 0, &V21169_320_57290, &P21169_344};	/*Property = PostMessage*/
const Value V21169_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21169_321 = {S40, 0, &V21169_321_57293, &P21169_320};	/*Property = Progress*/
const Value V21169_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21169_322 = {S41, 0, &V21169_322_57296, &P21169_321};	/*Property = Prompts*/
const Value V21169_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21169_323 = {S42, 0, &V21169_323_57299, &P21169_322};	/*Property = Selector*/
const Value V21169_324_57305 = {S2, NULL};	/*Value = True*/
const Property P21169_324 = {S43, 0, &V21169_324_57305, &P21169_323};	/*Property = Svg*/
const Value V21169_327_57314 = {S2, NULL};	/*Value = True*/
const Property P21169_327 = {S44, 0, &V21169_327_57314, &P21169_324};	/*Property = TouchEvents*/
const Value V21169_328_57317 = {S2, NULL};	/*Value = True*/
const Property P21169_328 = {S45, 0, &V21169_328_57317, &P21169_327};	/*Property = Track*/
const Value V21169_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21169_329 = {S46, 0, &V21169_329_57323, &P21169_328};	/*Property = Video*/
const Value V21169_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21169_330 = {S47, 0, &V21169_330_57326, &P21169_329};	/*Property = Viewport*/
const Value V21169_332_57338 = {S2, NULL};	/*Value = True*/
const Property P21169_332 = {S48, 0, &V21169_332_57338, &P21169_330};	/*Property = WebWorkers*/
const Value V21169_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21169_333 = {S49, 0, &V21169_333_57342, &P21169_332};	/*Property = Xhr2*/
const Profile P21169 = {"21169", &P21169_333};	/*Profile ID - 21169*/
const Value V23653_279_57148 = {S0, NULL};	/*Value = False*/
const Property P23653_279 = {S1, 0, &V23653_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V23653_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23653_282 = {S3, 0, &V23653_282_57154, &P23653_279};	/*Property = BlobBuilder*/
const Value V23653_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23653_350 = {S4, 0, &V23653_350_57444, &P23653_282};	/*Property = Canvas*/
const Value V23653_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23653_283 = {S5, 0, &V23653_283_57159, &P23653_350};	/*Property = CssBackground*/
const Value V23653_284_57162 = {S2, NULL};	/*Value = True*/
const Property P23653_284 = {S6, 0, &V23653_284_57162, &P23653_283};	/*Property = CssBorderImage*/
const Value V23653_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23653_285 = {S7, 0, &V23653_285_57166, &P23653_284};	/*Property = CssCanvas*/
const Value V23653_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23653_286 = {S8, 0, &V23653_286_57168, &P23653_285};	/*Property = CssColor*/
const Value V23653_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23653_237 = {S9, 0, &V23653_237_55786, &P23653_286};	/*Property = CssColumn*/
const Value V23653_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23653_287 = {S10, 0, &V23653_287_57172, &P23653_237};	/*Property = CssFlexbox*/
const Value V23653_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23653_288 = {S11, 0, &V23653_288_57174, &P23653_287};	/*Property = CssFont*/
const Value V23653_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23653_289 = {S12, 0, &V23653_289_57759, &P23653_288};	/*Property = CssImages*/
const Value V23653_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23653_291 = {S13, 0, &V23653_291_57180, &P23653_289};	/*Property = CssMediaQueries*/
const Value V23653_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23653_292 = {S14, 0, &V23653_292_57183, &P23653_291};	/*Property = CssMinMax*/
const Value V23653_293_57187 = {S0, NULL};	/*Value = False*/
const Property P23653_293 = {S15, 0, &V23653_293_57187, &P23653_292};	/*Property = CssOverflow*/
const Value V23653_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23653_294 = {S16, 0, &V23653_294_57189, &P23653_293};	/*Property = CssPosition*/
const Value V23653_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23653_295 = {S17, 0, &V23653_295_57192, &P23653_294};	/*Property = CssText*/
const Value V23653_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23653_297 = {S18, 0, &V23653_297_57195, &P23653_295};	/*Property = CssTransforms*/
const Value V23653_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23653_241 = {S19, 0, &V23653_241_55799, &P23653_297};	/*Property = CssTransitions*/
const Value V23653_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23653_298 = {S20, 0, &V23653_298_57206, &P23653_241};	/*Property = CssUI*/
const Value V23653_300_57215 = {S2, NULL};	/*Value = True*/
const Property P23653_300 = {S21, 0, &V23653_300_57215, &P23653_298};	/*Property = DataSet*/
const Value V23653_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23653_301 = {S22, 0, &V23653_301_57218, &P23653_300};	/*Property = DataUrl*/
const Value V23653_302_57221 = {S2, NULL};	/*Value = True*/
const Property P23653_302 = {S23, 0, &V23653_302_57221, &P23653_301};	/*Property = DeviceOrientation*/
const Value V23653_304_57227 = {S2, NULL};	/*Value = True*/
const Property P23653_304 = {S24, 0, &V23653_304_57227, &P23653_302};	/*Property = FileReader*/
const Value V23653_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23653_305 = {S25, 0, &V23653_305_57231, &P23653_304};	/*Property = FileSaver*/
const Value V23653_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23653_306 = {S26, 0, &V23653_306_57237, &P23653_305};	/*Property = FileWriter*/
const Value V23653_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23653_307 = {S27, 0, &V23653_307_57239, &P23653_306};	/*Property = FormData*/
const Value V23653_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23653_309 = {S28, 0, &V23653_309_57246, &P23653_307};	/*Property = Fullscreen*/
const Value V23653_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23653_310 = {S29, 0, &V23653_310_57248, &P23653_309};	/*Property = GeoLocation*/
const Value V23653_312_57254 = {S2, NULL};	/*Value = True*/
const Property P23653_312 = {S30, 0, &V23653_312_57254, &P23653_310};	/*Property = History*/
const Value V23653_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23653_314 = {S31, 0, &V23653_314_57261, &P23653_312};	/*Property = Html5*/
const Value V23653_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23653_313 = {S32, 0, &V23653_313_57258, &P23653_314};	/*Property = Html-Media-Capture*/
const Value V23653_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23653_315 = {S33, 0, &V23653_315_57264, &P23653_313};	/*Property = Iframe*/
const Value V23653_316_57266 = {S2, NULL};	/*Value = True*/
const Property P23653_316 = {S34, 0, &V23653_316_57266, &P23653_315};	/*Property = IndexedDB*/
const Value V23653_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23653_317 = {S35, 0, &V23653_317_57269, &P23653_316};	/*Property = Json*/
const Value V23653_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P23653_15 = {S37, 0, &V23653_15_29, &P23653_317};	/*Property = LayoutEngine*/
const Value V23653_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23653_344 = {S38, 0, &V23653_344_57272, &P23653_15};	/*Property = Masking*/
const Value V23653_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23653_320 = {S39, 0, &V23653_320_57290, &P23653_344};	/*Property = PostMessage*/
const Value V23653_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23653_321 = {S40, 0, &V23653_321_57293, &P23653_320};	/*Property = Progress*/
const Value V23653_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23653_322 = {S41, 0, &V23653_322_57296, &P23653_321};	/*Property = Prompts*/
const Value V23653_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23653_323 = {S42, 0, &V23653_323_57299, &P23653_322};	/*Property = Selector*/
const Value V23653_324_57305 = {S2, NULL};	/*Value = True*/
const Property P23653_324 = {S43, 0, &V23653_324_57305, &P23653_323};	/*Property = Svg*/
const Value V23653_327_57314 = {S2, NULL};	/*Value = True*/
const Property P23653_327 = {S44, 0, &V23653_327_57314, &P23653_324};	/*Property = TouchEvents*/
const Value V23653_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23653_328 = {S45, 0, &V23653_328_57318, &P23653_327};	/*Property = Track*/
const Value V23653_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23653_329 = {S46, 0, &V23653_329_57323, &P23653_328};	/*Property = Video*/
const Value V23653_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23653_330 = {S47, 0, &V23653_330_57326, &P23653_329};	/*Property = Viewport*/
const Value V23653_332_57338 = {S2, NULL};	/*Value = True*/
const Property P23653_332 = {S48, 0, &V23653_332_57338, &P23653_330};	/*Property = WebWorkers*/
const Value V23653_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23653_333 = {S49, 0, &V23653_333_57342, &P23653_332};	/*Property = Xhr2*/
const Profile P23653 = {"23653", &P23653_333};	/*Profile ID - 23653*/
const Value V22170_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22170_279 = {S1, 0, &V22170_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22170_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22170_282 = {S3, 0, &V22170_282_57154, &P22170_279};	/*Property = BlobBuilder*/
const Value V22170_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22170_350 = {S4, 0, &V22170_350_57444, &P22170_282};	/*Property = Canvas*/
const Value V22170_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22170_283 = {S5, 0, &V22170_283_57159, &P22170_350};	/*Property = CssBackground*/
const Value V22170_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22170_284 = {S6, 0, &V22170_284_57163, &P22170_283};	/*Property = CssBorderImage*/
const Value V22170_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22170_285 = {S7, 0, &V22170_285_57166, &P22170_284};	/*Property = CssCanvas*/
const Value V22170_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22170_286 = {S8, 0, &V22170_286_57168, &P22170_285};	/*Property = CssColor*/
const Value V22170_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22170_237 = {S9, 0, &V22170_237_55786, &P22170_286};	/*Property = CssColumn*/
const Value V22170_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22170_287 = {S10, 0, &V22170_287_57172, &P22170_237};	/*Property = CssFlexbox*/
const Value V22170_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22170_288 = {S11, 0, &V22170_288_57174, &P22170_287};	/*Property = CssFont*/
const Value V22170_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22170_289 = {S12, 0, &V22170_289_57759, &P22170_288};	/*Property = CssImages*/
const Value V22170_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22170_291 = {S13, 0, &V22170_291_57180, &P22170_289};	/*Property = CssMediaQueries*/
const Value V22170_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22170_292 = {S14, 0, &V22170_292_57183, &P22170_291};	/*Property = CssMinMax*/
const Value V22170_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22170_293 = {S15, 0, &V22170_293_57187, &P22170_292};	/*Property = CssOverflow*/
const Value V22170_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22170_294 = {S16, 0, &V22170_294_57189, &P22170_293};	/*Property = CssPosition*/
const Value V22170_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22170_295 = {S17, 0, &V22170_295_57192, &P22170_294};	/*Property = CssText*/
const Value V22170_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22170_297 = {S18, 0, &V22170_297_57195, &P22170_295};	/*Property = CssTransforms*/
const Value V22170_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22170_241 = {S19, 0, &V22170_241_55799, &P22170_297};	/*Property = CssTransitions*/
const Value V22170_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22170_298 = {S20, 0, &V22170_298_57206, &P22170_241};	/*Property = CssUI*/
const Value V22170_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22170_300 = {S21, 0, &V22170_300_57216, &P22170_298};	/*Property = DataSet*/
const Value V22170_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22170_301 = {S22, 0, &V22170_301_57218, &P22170_300};	/*Property = DataUrl*/
const Value V22170_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22170_302 = {S23, 0, &V22170_302_57221, &P22170_301};	/*Property = DeviceOrientation*/
const Value V22170_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22170_304 = {S24, 0, &V22170_304_57227, &P22170_302};	/*Property = FileReader*/
const Value V22170_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22170_305 = {S25, 0, &V22170_305_57231, &P22170_304};	/*Property = FileSaver*/
const Value V22170_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22170_306 = {S26, 0, &V22170_306_57237, &P22170_305};	/*Property = FileWriter*/
const Value V22170_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22170_307 = {S27, 0, &V22170_307_57239, &P22170_306};	/*Property = FormData*/
const Value V22170_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22170_309 = {S28, 0, &V22170_309_57246, &P22170_307};	/*Property = Fullscreen*/
const Value V22170_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22170_310 = {S29, 0, &V22170_310_57248, &P22170_309};	/*Property = GeoLocation*/
const Value V22170_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22170_312 = {S30, 0, &V22170_312_57254, &P22170_310};	/*Property = History*/
const Value V22170_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22170_314 = {S31, 0, &V22170_314_57261, &P22170_312};	/*Property = Html5*/
const Value V22170_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22170_313 = {S32, 0, &V22170_313_57258, &P22170_314};	/*Property = Html-Media-Capture*/
const Value V22170_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22170_315 = {S33, 0, &V22170_315_57264, &P22170_313};	/*Property = Iframe*/
const Value V22170_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22170_316 = {S34, 0, &V22170_316_57266, &P22170_315};	/*Property = IndexedDB*/
const Value V22170_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22170_317 = {S35, 0, &V22170_317_57269, &P22170_316};	/*Property = Json*/
const Value V22170_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22170_15 = {S37, 0, &V22170_15_29, &P22170_317};	/*Property = LayoutEngine*/
const Value V22170_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22170_344 = {S38, 0, &V22170_344_57272, &P22170_15};	/*Property = Masking*/
const Value V22170_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22170_320 = {S39, 0, &V22170_320_57290, &P22170_344};	/*Property = PostMessage*/
const Value V22170_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22170_321 = {S40, 0, &V22170_321_57293, &P22170_320};	/*Property = Progress*/
const Value V22170_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22170_322 = {S41, 0, &V22170_322_57296, &P22170_321};	/*Property = Prompts*/
const Value V22170_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22170_323 = {S42, 0, &V22170_323_57299, &P22170_322};	/*Property = Selector*/
const Value V22170_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22170_324 = {S43, 0, &V22170_324_57306, &P22170_323};	/*Property = Svg*/
const Value V22170_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22170_327 = {S44, 0, &V22170_327_57314, &P22170_324};	/*Property = TouchEvents*/
const Value V22170_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22170_328 = {S45, 0, &V22170_328_57318, &P22170_327};	/*Property = Track*/
const Value V22170_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22170_329 = {S46, 0, &V22170_329_57323, &P22170_328};	/*Property = Video*/
const Value V22170_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22170_330 = {S47, 0, &V22170_330_57326, &P22170_329};	/*Property = Viewport*/
const Value V22170_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22170_332 = {S48, 0, &V22170_332_57338, &P22170_330};	/*Property = WebWorkers*/
const Value V22170_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22170_333 = {S49, 0, &V22170_333_57342, &P22170_332};	/*Property = Xhr2*/
const Profile P22170 = {"22170", &P22170_333};	/*Profile ID - 22170*/
const Value V23594_279_57148 = {S0, NULL};	/*Value = False*/
const Property P23594_279 = {S1, 0, &V23594_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V23594_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23594_282 = {S3, 0, &V23594_282_57154, &P23594_279};	/*Property = BlobBuilder*/
const Value V23594_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23594_350 = {S4, 0, &V23594_350_57444, &P23594_282};	/*Property = Canvas*/
const Value V23594_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23594_283 = {S5, 0, &V23594_283_57159, &P23594_350};	/*Property = CssBackground*/
const Value V23594_284_57163 = {S0, NULL};	/*Value = False*/
const Property P23594_284 = {S6, 0, &V23594_284_57163, &P23594_283};	/*Property = CssBorderImage*/
const Value V23594_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23594_285 = {S7, 0, &V23594_285_57166, &P23594_284};	/*Property = CssCanvas*/
const Value V23594_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23594_286 = {S8, 0, &V23594_286_57168, &P23594_285};	/*Property = CssColor*/
const Value V23594_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23594_237 = {S9, 0, &V23594_237_55786, &P23594_286};	/*Property = CssColumn*/
const Value V23594_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23594_287 = {S10, 0, &V23594_287_57172, &P23594_237};	/*Property = CssFlexbox*/
const Value V23594_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23594_288 = {S11, 0, &V23594_288_57174, &P23594_287};	/*Property = CssFont*/
const Value V23594_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23594_289 = {S12, 0, &V23594_289_57759, &P23594_288};	/*Property = CssImages*/
const Value V23594_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23594_291 = {S13, 0, &V23594_291_57180, &P23594_289};	/*Property = CssMediaQueries*/
const Value V23594_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23594_292 = {S14, 0, &V23594_292_57183, &P23594_291};	/*Property = CssMinMax*/
const Value V23594_293_57186 = {S2, NULL};	/*Value = True*/
const Property P23594_293 = {S15, 0, &V23594_293_57186, &P23594_292};	/*Property = CssOverflow*/
const Value V23594_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23594_294 = {S16, 0, &V23594_294_57189, &P23594_293};	/*Property = CssPosition*/
const Value V23594_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23594_295 = {S17, 0, &V23594_295_57192, &P23594_294};	/*Property = CssText*/
const Value V23594_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23594_297 = {S18, 0, &V23594_297_57195, &P23594_295};	/*Property = CssTransforms*/
const Value V23594_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23594_241 = {S19, 0, &V23594_241_55799, &P23594_297};	/*Property = CssTransitions*/
const Value V23594_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23594_298 = {S20, 0, &V23594_298_57206, &P23594_241};	/*Property = CssUI*/
const Value V23594_300_57216 = {S0, NULL};	/*Value = False*/
const Property P23594_300 = {S21, 0, &V23594_300_57216, &P23594_298};	/*Property = DataSet*/
const Value V23594_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23594_301 = {S22, 0, &V23594_301_57218, &P23594_300};	/*Property = DataUrl*/
const Value V23594_302_57221 = {S2, NULL};	/*Value = True*/
const Property P23594_302 = {S23, 0, &V23594_302_57221, &P23594_301};	/*Property = DeviceOrientation*/
const Value V23594_304_57227 = {S2, NULL};	/*Value = True*/
const Property P23594_304 = {S24, 0, &V23594_304_57227, &P23594_302};	/*Property = FileReader*/
const Value V23594_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23594_305 = {S25, 0, &V23594_305_57231, &P23594_304};	/*Property = FileSaver*/
const Value V23594_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23594_306 = {S26, 0, &V23594_306_57237, &P23594_305};	/*Property = FileWriter*/
const Value V23594_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23594_307 = {S27, 0, &V23594_307_57239, &P23594_306};	/*Property = FormData*/
const Value V23594_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23594_309 = {S28, 0, &V23594_309_57246, &P23594_307};	/*Property = Fullscreen*/
const Value V23594_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23594_310 = {S29, 0, &V23594_310_57248, &P23594_309};	/*Property = GeoLocation*/
const Value V23594_312_57254 = {S2, NULL};	/*Value = True*/
const Property P23594_312 = {S30, 0, &V23594_312_57254, &P23594_310};	/*Property = History*/
const Value V23594_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23594_314 = {S31, 0, &V23594_314_57261, &P23594_312};	/*Property = Html5*/
const Value V23594_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23594_313 = {S32, 0, &V23594_313_57258, &P23594_314};	/*Property = Html-Media-Capture*/
const Value V23594_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23594_315 = {S33, 0, &V23594_315_57264, &P23594_313};	/*Property = Iframe*/
const Value V23594_316_57267 = {S0, NULL};	/*Value = False*/
const Property P23594_316 = {S34, 0, &V23594_316_57267, &P23594_315};	/*Property = IndexedDB*/
const Value V23594_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23594_317 = {S35, 0, &V23594_317_57269, &P23594_316};	/*Property = Json*/
const Value V23594_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P23594_15 = {S37, 0, &V23594_15_72, &P23594_317};	/*Property = LayoutEngine*/
const Value V23594_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23594_344 = {S38, 0, &V23594_344_57272, &P23594_15};	/*Property = Masking*/
const Value V23594_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23594_320 = {S39, 0, &V23594_320_57290, &P23594_344};	/*Property = PostMessage*/
const Value V23594_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23594_321 = {S40, 0, &V23594_321_57293, &P23594_320};	/*Property = Progress*/
const Value V23594_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23594_322 = {S41, 0, &V23594_322_57296, &P23594_321};	/*Property = Prompts*/
const Value V23594_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23594_323 = {S42, 0, &V23594_323_57299, &P23594_322};	/*Property = Selector*/
const Value V23594_324_57306 = {S0, NULL};	/*Value = False*/
const Property P23594_324 = {S43, 0, &V23594_324_57306, &P23594_323};	/*Property = Svg*/
const Value V23594_327_57314 = {S2, NULL};	/*Value = True*/
const Property P23594_327 = {S44, 0, &V23594_327_57314, &P23594_324};	/*Property = TouchEvents*/
const Value V23594_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23594_328 = {S45, 0, &V23594_328_57318, &P23594_327};	/*Property = Track*/
const Value V23594_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23594_329 = {S46, 0, &V23594_329_57323, &P23594_328};	/*Property = Video*/
const Value V23594_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23594_330 = {S47, 0, &V23594_330_57326, &P23594_329};	/*Property = Viewport*/
const Value V23594_332_57338 = {S2, NULL};	/*Value = True*/
const Property P23594_332 = {S48, 0, &V23594_332_57338, &P23594_330};	/*Property = WebWorkers*/
const Value V23594_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23594_333 = {S49, 0, &V23594_333_57342, &P23594_332};	/*Property = Xhr2*/
const Profile P23594 = {"23594", &P23594_333};	/*Profile ID - 23594*/
const Value V22638_1_1 = {S2, NULL};	/*Value = True*/
const Property P22638_1 = {S50, 0, &V22638_1_1, NULL};	/*Property = IsMobile*/
const Value V22638_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22638_5 = {S52, 0, &V22638_5_1277, &P22638_1};	/*Property = ScreenPixelsHeight*/
const Value V22638_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22638_4 = {S54, 0, &V22638_4_1279, &P22638_5};	/*Property = ScreenPixelsWidth*/
const Profile P22638 = {"22638", &P22638_4};	/*Profile ID - 22638*/
const Value V22940_1_1 = {S2, NULL};	/*Value = True*/
const Property P22940_1 = {S50, 0, &V22940_1_1, NULL};	/*Property = IsMobile*/
const Value V22940_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22940_5 = {S52, 0, &V22940_5_1458, &P22940_1};	/*Property = ScreenPixelsHeight*/
const Value V22940_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22940_4 = {S54, 0, &V22940_4_1460, &P22940_5};	/*Property = ScreenPixelsWidth*/
const Profile P22940 = {"22940", &P22940_4};	/*Profile ID - 22940*/
const Value V21638_1_1 = {S2, NULL};	/*Value = True*/
const Property P21638_1 = {S50, 0, &V21638_1_1, NULL};	/*Property = IsMobile*/
const Value V21638_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21638_5 = {S52, 0, &V21638_5_1458, &P21638_1};	/*Property = ScreenPixelsHeight*/
const Value V21638_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21638_4 = {S54, 0, &V21638_4_1460, &P21638_5};	/*Property = ScreenPixelsWidth*/
const Profile P21638 = {"21638", &P21638_4};	/*Profile ID - 21638*/
const Value V21660_1_1 = {S2, NULL};	/*Value = True*/
const Property P21660_1 = {S50, 0, &V21660_1_1, NULL};	/*Property = IsMobile*/
const Value V21660_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21660_5 = {S52, 0, &V21660_5_9981, &P21660_1};	/*Property = ScreenPixelsHeight*/
const Value V21660_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21660_4 = {S54, 0, &V21660_4_2336, &P21660_5};	/*Property = ScreenPixelsWidth*/
const Profile P21660 = {"21660", &P21660_4};	/*Profile ID - 21660*/
const Value V23434_1_1 = {S2, NULL};	/*Value = True*/
const Property P23434_1 = {S50, 0, &V23434_1_1, NULL};	/*Property = IsMobile*/
const Value V23434_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23434_5 = {S52, 0, &V23434_5_37569, &P23434_1};	/*Property = ScreenPixelsHeight*/
const Value V23434_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23434_4 = {S54, 0, &V23434_4_5645, &P23434_5};	/*Property = ScreenPixelsWidth*/
const Profile P23434 = {"23434", &P23434_4};	/*Profile ID - 23434*/
const Value V21854_1_1 = {S2, NULL};	/*Value = True*/
const Property P21854_1 = {S50, 0, &V21854_1_1, NULL};	/*Property = IsMobile*/
const Value V21854_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21854_5 = {S52, 0, &V21854_5_1277, &P21854_1};	/*Property = ScreenPixelsHeight*/
const Value V21854_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21854_4 = {S54, 0, &V21854_4_2336, &P21854_5};	/*Property = ScreenPixelsWidth*/
const Profile P21854 = {"21854", &P21854_4};	/*Profile ID - 21854*/
const Value V22044_1_1 = {S2, NULL};	/*Value = True*/
const Property P22044_1 = {S50, 0, &V22044_1_1, NULL};	/*Property = IsMobile*/
const Value V22044_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22044_5 = {S52, 0, &V22044_5_1227, &P22044_1};	/*Property = ScreenPixelsHeight*/
const Value V22044_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P22044_4 = {S54, 0, &V22044_4_1477, &P22044_5};	/*Property = ScreenPixelsWidth*/
const Profile P22044 = {"22044", &P22044_4};	/*Profile ID - 22044*/
const Value V20927_1_1 = {S2, NULL};	/*Value = True*/
const Property P20927_1 = {S50, 0, &V20927_1_1, NULL};	/*Property = IsMobile*/
const Value V20927_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20927_5 = {S52, 0, &V20927_5_1403, &P20927_1};	/*Property = ScreenPixelsHeight*/
const Value V20927_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20927_4 = {S54, 0, &V20927_4_1404, &P20927_5};	/*Property = ScreenPixelsWidth*/
const Profile P20927 = {"20927", &P20927_4};	/*Profile ID - 20927*/
const Value V23509_1_1 = {S2, NULL};	/*Value = True*/
const Property P23509_1 = {S50, 0, &V23509_1_1, NULL};	/*Property = IsMobile*/
const Value V23509_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23509_5 = {S52, 0, &V23509_5_1227, &P23509_1};	/*Property = ScreenPixelsHeight*/
const Value V23509_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23509_4 = {S54, 0, &V23509_4_1229, &P23509_5};	/*Property = ScreenPixelsWidth*/
const Profile P23509 = {"23509", &P23509_4};	/*Profile ID - 23509*/
const Value V20495_1_1 = {S2, NULL};	/*Value = True*/
const Property P20495_1 = {S50, 0, &V20495_1_1, NULL};	/*Property = IsMobile*/
const Value V20495_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20495_5 = {S52, 0, &V20495_5_1403, &P20495_1};	/*Property = ScreenPixelsHeight*/
const Value V20495_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20495_4 = {S54, 0, &V20495_4_1404, &P20495_5};	/*Property = ScreenPixelsWidth*/
const Profile P20495 = {"20495", &P20495_4};	/*Profile ID - 20495*/
const Value V22162_1_1 = {S2, NULL};	/*Value = True*/
const Property P22162_1 = {S50, 0, &V22162_1_1, NULL};	/*Property = IsMobile*/
const Value V22162_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22162_5 = {S52, 0, &V22162_5_1458, &P22162_1};	/*Property = ScreenPixelsHeight*/
const Value V22162_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P22162_4 = {S54, 0, &V22162_4_37617, &P22162_5};	/*Property = ScreenPixelsWidth*/
const Profile P22162 = {"22162", &P22162_4};	/*Profile ID - 22162*/
const Value V22369_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22369_279 = {S1, 0, &V22369_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22369_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22369_282 = {S3, 0, &V22369_282_57154, &P22369_279};	/*Property = BlobBuilder*/
const Value V22369_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22369_350 = {S4, 0, &V22369_350_57444, &P22369_282};	/*Property = Canvas*/
const Value V22369_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22369_283 = {S5, 0, &V22369_283_57159, &P22369_350};	/*Property = CssBackground*/
const Value V22369_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22369_284 = {S6, 0, &V22369_284_57163, &P22369_283};	/*Property = CssBorderImage*/
const Value V22369_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22369_285 = {S7, 0, &V22369_285_57166, &P22369_284};	/*Property = CssCanvas*/
const Value V22369_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22369_286 = {S8, 0, &V22369_286_57168, &P22369_285};	/*Property = CssColor*/
const Value V22369_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22369_237 = {S9, 0, &V22369_237_55786, &P22369_286};	/*Property = CssColumn*/
const Value V22369_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22369_287 = {S10, 0, &V22369_287_57172, &P22369_237};	/*Property = CssFlexbox*/
const Value V22369_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22369_288 = {S11, 0, &V22369_288_57174, &P22369_287};	/*Property = CssFont*/
const Value V22369_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22369_289 = {S12, 0, &V22369_289_57759, &P22369_288};	/*Property = CssImages*/
const Value V22369_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22369_291 = {S13, 0, &V22369_291_57180, &P22369_289};	/*Property = CssMediaQueries*/
const Value V22369_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22369_292 = {S14, 0, &V22369_292_57183, &P22369_291};	/*Property = CssMinMax*/
const Value V22369_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22369_293 = {S15, 0, &V22369_293_57187, &P22369_292};	/*Property = CssOverflow*/
const Value V22369_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22369_294 = {S16, 0, &V22369_294_57189, &P22369_293};	/*Property = CssPosition*/
const Value V22369_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22369_295 = {S17, 0, &V22369_295_57192, &P22369_294};	/*Property = CssText*/
const Value V22369_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22369_297 = {S18, 0, &V22369_297_57195, &P22369_295};	/*Property = CssTransforms*/
const Value V22369_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22369_241 = {S19, 0, &V22369_241_55799, &P22369_297};	/*Property = CssTransitions*/
const Value V22369_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22369_298 = {S20, 0, &V22369_298_57206, &P22369_241};	/*Property = CssUI*/
const Value V22369_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22369_300 = {S21, 0, &V22369_300_57216, &P22369_298};	/*Property = DataSet*/
const Value V22369_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22369_301 = {S22, 0, &V22369_301_57218, &P22369_300};	/*Property = DataUrl*/
const Value V22369_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22369_302 = {S23, 0, &V22369_302_57221, &P22369_301};	/*Property = DeviceOrientation*/
const Value V22369_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22369_304 = {S24, 0, &V22369_304_57227, &P22369_302};	/*Property = FileReader*/
const Value V22369_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22369_305 = {S25, 0, &V22369_305_57231, &P22369_304};	/*Property = FileSaver*/
const Value V22369_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22369_306 = {S26, 0, &V22369_306_57237, &P22369_305};	/*Property = FileWriter*/
const Value V22369_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22369_307 = {S27, 0, &V22369_307_57239, &P22369_306};	/*Property = FormData*/
const Value V22369_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22369_309 = {S28, 0, &V22369_309_57246, &P22369_307};	/*Property = Fullscreen*/
const Value V22369_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22369_310 = {S29, 0, &V22369_310_57248, &P22369_309};	/*Property = GeoLocation*/
const Value V22369_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22369_312 = {S30, 0, &V22369_312_57254, &P22369_310};	/*Property = History*/
const Value V22369_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22369_314 = {S31, 0, &V22369_314_57261, &P22369_312};	/*Property = Html5*/
const Value V22369_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22369_313 = {S32, 0, &V22369_313_57258, &P22369_314};	/*Property = Html-Media-Capture*/
const Value V22369_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22369_315 = {S33, 0, &V22369_315_57264, &P22369_313};	/*Property = Iframe*/
const Value V22369_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22369_316 = {S34, 0, &V22369_316_57266, &P22369_315};	/*Property = IndexedDB*/
const Value V22369_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22369_317 = {S35, 0, &V22369_317_57269, &P22369_316};	/*Property = Json*/
const Value V22369_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22369_15 = {S37, 0, &V22369_15_29, &P22369_317};	/*Property = LayoutEngine*/
const Value V22369_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22369_344 = {S38, 0, &V22369_344_57272, &P22369_15};	/*Property = Masking*/
const Value V22369_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22369_320 = {S39, 0, &V22369_320_57290, &P22369_344};	/*Property = PostMessage*/
const Value V22369_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22369_321 = {S40, 0, &V22369_321_57293, &P22369_320};	/*Property = Progress*/
const Value V22369_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22369_322 = {S41, 0, &V22369_322_57296, &P22369_321};	/*Property = Prompts*/
const Value V22369_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22369_323 = {S42, 0, &V22369_323_57299, &P22369_322};	/*Property = Selector*/
const Value V22369_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22369_324 = {S43, 0, &V22369_324_57306, &P22369_323};	/*Property = Svg*/
const Value V22369_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22369_327 = {S44, 0, &V22369_327_57315, &P22369_324};	/*Property = TouchEvents*/
const Value V22369_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22369_328 = {S45, 0, &V22369_328_57318, &P22369_327};	/*Property = Track*/
const Value V22369_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22369_329 = {S46, 0, &V22369_329_57323, &P22369_328};	/*Property = Video*/
const Value V22369_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22369_330 = {S47, 0, &V22369_330_57326, &P22369_329};	/*Property = Viewport*/
const Value V22369_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22369_332 = {S48, 0, &V22369_332_57338, &P22369_330};	/*Property = WebWorkers*/
const Value V22369_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22369_333 = {S49, 0, &V22369_333_57341, &P22369_332};	/*Property = Xhr2*/
const Profile P22369 = {"22369", &P22369_333};	/*Profile ID - 22369*/
const Value V21419_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21419_279 = {S1, 0, &V21419_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21419_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21419_282 = {S3, 0, &V21419_282_57154, &P21419_279};	/*Property = BlobBuilder*/
const Value V21419_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21419_350 = {S4, 0, &V21419_350_57444, &P21419_282};	/*Property = Canvas*/
const Value V21419_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21419_283 = {S5, 0, &V21419_283_57159, &P21419_350};	/*Property = CssBackground*/
const Value V21419_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21419_284 = {S6, 0, &V21419_284_57163, &P21419_283};	/*Property = CssBorderImage*/
const Value V21419_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21419_285 = {S7, 0, &V21419_285_57166, &P21419_284};	/*Property = CssCanvas*/
const Value V21419_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21419_286 = {S8, 0, &V21419_286_57168, &P21419_285};	/*Property = CssColor*/
const Value V21419_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21419_237 = {S9, 0, &V21419_237_55786, &P21419_286};	/*Property = CssColumn*/
const Value V21419_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21419_287 = {S10, 0, &V21419_287_57172, &P21419_237};	/*Property = CssFlexbox*/
const Value V21419_288_57174 = {S2, NULL};	/*Value = True*/
const Property P21419_288 = {S11, 0, &V21419_288_57174, &P21419_287};	/*Property = CssFont*/
const Value V21419_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21419_289 = {S12, 0, &V21419_289_57759, &P21419_288};	/*Property = CssImages*/
const Value V21419_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21419_291 = {S13, 0, &V21419_291_57180, &P21419_289};	/*Property = CssMediaQueries*/
const Value V21419_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21419_292 = {S14, 0, &V21419_292_57183, &P21419_291};	/*Property = CssMinMax*/
const Value V21419_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21419_293 = {S15, 0, &V21419_293_57187, &P21419_292};	/*Property = CssOverflow*/
const Value V21419_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21419_294 = {S16, 0, &V21419_294_57189, &P21419_293};	/*Property = CssPosition*/
const Value V21419_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21419_295 = {S17, 0, &V21419_295_57192, &P21419_294};	/*Property = CssText*/
const Value V21419_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21419_297 = {S18, 0, &V21419_297_57195, &P21419_295};	/*Property = CssTransforms*/
const Value V21419_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21419_241 = {S19, 0, &V21419_241_55799, &P21419_297};	/*Property = CssTransitions*/
const Value V21419_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21419_298 = {S20, 0, &V21419_298_57206, &P21419_241};	/*Property = CssUI*/
const Value V21419_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21419_300 = {S21, 0, &V21419_300_57216, &P21419_298};	/*Property = DataSet*/
const Value V21419_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21419_301 = {S22, 0, &V21419_301_57218, &P21419_300};	/*Property = DataUrl*/
const Value V21419_302_57221 = {S2, NULL};	/*Value = True*/
const Property P21419_302 = {S23, 0, &V21419_302_57221, &P21419_301};	/*Property = DeviceOrientation*/
const Value V21419_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21419_304 = {S24, 0, &V21419_304_57227, &P21419_302};	/*Property = FileReader*/
const Value V21419_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21419_305 = {S25, 0, &V21419_305_57231, &P21419_304};	/*Property = FileSaver*/
const Value V21419_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21419_306 = {S26, 0, &V21419_306_57237, &P21419_305};	/*Property = FileWriter*/
const Value V21419_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21419_307 = {S27, 0, &V21419_307_57239, &P21419_306};	/*Property = FormData*/
const Value V21419_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21419_309 = {S28, 0, &V21419_309_57246, &P21419_307};	/*Property = Fullscreen*/
const Value V21419_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21419_310 = {S29, 0, &V21419_310_57248, &P21419_309};	/*Property = GeoLocation*/
const Value V21419_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21419_312 = {S30, 0, &V21419_312_57254, &P21419_310};	/*Property = History*/
const Value V21419_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21419_314 = {S31, 0, &V21419_314_57261, &P21419_312};	/*Property = Html5*/
const Value V21419_313_57257 = {S2, NULL};	/*Value = True*/
const Property P21419_313 = {S32, 0, &V21419_313_57257, &P21419_314};	/*Property = Html-Media-Capture*/
const Value V21419_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21419_315 = {S33, 0, &V21419_315_57264, &P21419_313};	/*Property = Iframe*/
const Value V21419_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21419_316 = {S34, 0, &V21419_316_57266, &P21419_315};	/*Property = IndexedDB*/
const Value V21419_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21419_317 = {S35, 0, &V21419_317_57269, &P21419_316};	/*Property = Json*/
const Value V21419_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21419_15 = {S37, 0, &V21419_15_54167, &P21419_317};	/*Property = LayoutEngine*/
const Value V21419_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21419_344 = {S38, 0, &V21419_344_57272, &P21419_15};	/*Property = Masking*/
const Value V21419_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21419_320 = {S39, 0, &V21419_320_57290, &P21419_344};	/*Property = PostMessage*/
const Value V21419_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21419_321 = {S40, 0, &V21419_321_57293, &P21419_320};	/*Property = Progress*/
const Value V21419_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21419_322 = {S41, 0, &V21419_322_57296, &P21419_321};	/*Property = Prompts*/
const Value V21419_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21419_323 = {S42, 0, &V21419_323_57299, &P21419_322};	/*Property = Selector*/
const Value V21419_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21419_324 = {S43, 0, &V21419_324_57306, &P21419_323};	/*Property = Svg*/
const Value V21419_327_57314 = {S2, NULL};	/*Value = True*/
const Property P21419_327 = {S44, 0, &V21419_327_57314, &P21419_324};	/*Property = TouchEvents*/
const Value V21419_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21419_328 = {S45, 0, &V21419_328_57318, &P21419_327};	/*Property = Track*/
const Value V21419_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21419_329 = {S46, 0, &V21419_329_57323, &P21419_328};	/*Property = Video*/
const Value V21419_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21419_330 = {S47, 0, &V21419_330_57326, &P21419_329};	/*Property = Viewport*/
const Value V21419_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21419_332 = {S48, 0, &V21419_332_57339, &P21419_330};	/*Property = WebWorkers*/
const Value V21419_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21419_333 = {S49, 0, &V21419_333_57342, &P21419_332};	/*Property = Xhr2*/
const Profile P21419 = {"21419", &P21419_333};	/*Profile ID - 21419*/
const Value V23506_1_1 = {S2, NULL};	/*Value = True*/
const Property P23506_1 = {S50, 0, &V23506_1_1, NULL};	/*Property = IsMobile*/
const Value V23506_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23506_5 = {S52, 0, &V23506_5_1458, &P23506_1};	/*Property = ScreenPixelsHeight*/
const Value V23506_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23506_4 = {S54, 0, &V23506_4_1460, &P23506_5};	/*Property = ScreenPixelsWidth*/
const Profile P23506 = {"23506", &P23506_4};	/*Profile ID - 23506*/
const Value V21020_1_1 = {S2, NULL};	/*Value = True*/
const Property P21020_1 = {S50, 0, &V21020_1_1, NULL};	/*Property = IsMobile*/
const Value V21020_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21020_5 = {S52, 0, &V21020_5_9981, &P21020_1};	/*Property = ScreenPixelsHeight*/
const Value V21020_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21020_4 = {S54, 0, &V21020_4_39098, &P21020_5};	/*Property = ScreenPixelsWidth*/
const Profile P21020 = {"21020", &P21020_4};	/*Profile ID - 21020*/
const Value V21542_1_1 = {S2, NULL};	/*Value = True*/
const Property P21542_1 = {S50, 0, &V21542_1_1, NULL};	/*Property = IsMobile*/
const Value V21542_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P21542_5 = {S52, 0, &V21542_5_59138, &P21542_1};	/*Property = ScreenPixelsHeight*/
const Value V21542_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P21542_4 = {S54, 0, &V21542_4_59970, &P21542_5};	/*Property = ScreenPixelsWidth*/
const Profile P21542 = {"21542", &P21542_4};	/*Profile ID - 21542*/
const Value V21772_1_1 = {S2, NULL};	/*Value = True*/
const Property P21772_1 = {S50, 0, &V21772_1_1, NULL};	/*Property = IsMobile*/
const Value V21772_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P21772_5 = {S52, 0, &V21772_5_1832, &P21772_1};	/*Property = ScreenPixelsHeight*/
const Value V21772_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21772_4 = {S54, 0, &V21772_4_2336, &P21772_5};	/*Property = ScreenPixelsWidth*/
const Profile P21772 = {"21772", &P21772_4};	/*Profile ID - 21772*/
const Value V23770_1_1 = {S2, NULL};	/*Value = True*/
const Property P23770_1 = {S50, 0, &V23770_1_1, NULL};	/*Property = IsMobile*/
const Value V23770_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23770_5 = {S52, 0, &V23770_5_1227, &P23770_1};	/*Property = ScreenPixelsHeight*/
const Value V23770_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23770_4 = {S54, 0, &V23770_4_1229, &P23770_5};	/*Property = ScreenPixelsWidth*/
const Profile P23770 = {"23770", &P23770_4};	/*Profile ID - 23770*/
const Value V22327_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22327_279 = {S1, 0, &V22327_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22327_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22327_282 = {S3, 0, &V22327_282_57153, &P22327_279};	/*Property = BlobBuilder*/
const Value V22327_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22327_350 = {S4, 0, &V22327_350_57444, &P22327_282};	/*Property = Canvas*/
const Value V22327_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22327_283 = {S5, 0, &V22327_283_57159, &P22327_350};	/*Property = CssBackground*/
const Value V22327_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22327_284 = {S6, 0, &V22327_284_57163, &P22327_283};	/*Property = CssBorderImage*/
const Value V22327_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22327_285 = {S7, 0, &V22327_285_57166, &P22327_284};	/*Property = CssCanvas*/
const Value V22327_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22327_286 = {S8, 0, &V22327_286_57168, &P22327_285};	/*Property = CssColor*/
const Value V22327_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22327_237 = {S9, 0, &V22327_237_55784, &P22327_286};	/*Property = CssColumn*/
const Value V22327_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22327_287 = {S10, 0, &V22327_287_57172, &P22327_237};	/*Property = CssFlexbox*/
const Value V22327_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22327_288 = {S11, 0, &V22327_288_57175, &P22327_287};	/*Property = CssFont*/
const Value V22327_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22327_289 = {S12, 0, &V22327_289_57759, &P22327_288};	/*Property = CssImages*/
const Value V22327_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22327_291 = {S13, 0, &V22327_291_57180, &P22327_289};	/*Property = CssMediaQueries*/
const Value V22327_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22327_292 = {S14, 0, &V22327_292_57183, &P22327_291};	/*Property = CssMinMax*/
const Value V22327_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22327_293 = {S15, 0, &V22327_293_57187, &P22327_292};	/*Property = CssOverflow*/
const Value V22327_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22327_294 = {S16, 0, &V22327_294_57189, &P22327_293};	/*Property = CssPosition*/
const Value V22327_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22327_295 = {S17, 0, &V22327_295_57192, &P22327_294};	/*Property = CssText*/
const Value V22327_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22327_297 = {S18, 0, &V22327_297_57195, &P22327_295};	/*Property = CssTransforms*/
const Value V22327_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22327_241 = {S19, 0, &V22327_241_55799, &P22327_297};	/*Property = CssTransitions*/
const Value V22327_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22327_298 = {S20, 0, &V22327_298_57206, &P22327_241};	/*Property = CssUI*/
const Value V22327_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22327_300 = {S21, 0, &V22327_300_57216, &P22327_298};	/*Property = DataSet*/
const Value V22327_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22327_301 = {S22, 0, &V22327_301_57218, &P22327_300};	/*Property = DataUrl*/
const Value V22327_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22327_302 = {S23, 0, &V22327_302_57222, &P22327_301};	/*Property = DeviceOrientation*/
const Value V22327_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22327_304 = {S24, 0, &V22327_304_57227, &P22327_302};	/*Property = FileReader*/
const Value V22327_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22327_305 = {S25, 0, &V22327_305_57231, &P22327_304};	/*Property = FileSaver*/
const Value V22327_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22327_306 = {S26, 0, &V22327_306_57237, &P22327_305};	/*Property = FileWriter*/
const Value V22327_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22327_307 = {S27, 0, &V22327_307_57239, &P22327_306};	/*Property = FormData*/
const Value V22327_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22327_309 = {S28, 0, &V22327_309_57246, &P22327_307};	/*Property = Fullscreen*/
const Value V22327_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22327_310 = {S29, 0, &V22327_310_57248, &P22327_309};	/*Property = GeoLocation*/
const Value V22327_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22327_312 = {S30, 0, &V22327_312_57255, &P22327_310};	/*Property = History*/
const Value V22327_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22327_314 = {S31, 0, &V22327_314_57261, &P22327_312};	/*Property = Html5*/
const Value V22327_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22327_313 = {S32, 0, &V22327_313_57258, &P22327_314};	/*Property = Html-Media-Capture*/
const Value V22327_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22327_315 = {S33, 0, &V22327_315_57264, &P22327_313};	/*Property = Iframe*/
const Value V22327_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22327_316 = {S34, 0, &V22327_316_57266, &P22327_315};	/*Property = IndexedDB*/
const Value V22327_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22327_317 = {S35, 0, &V22327_317_57269, &P22327_316};	/*Property = Json*/
const Value V22327_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22327_15 = {S37, 0, &V22327_15_28, &P22327_317};	/*Property = LayoutEngine*/
const Value V22327_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22327_344 = {S38, 0, &V22327_344_57272, &P22327_15};	/*Property = Masking*/
const Value V22327_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22327_320 = {S39, 0, &V22327_320_57290, &P22327_344};	/*Property = PostMessage*/
const Value V22327_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22327_321 = {S40, 0, &V22327_321_57293, &P22327_320};	/*Property = Progress*/
const Value V22327_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22327_322 = {S41, 0, &V22327_322_57296, &P22327_321};	/*Property = Prompts*/
const Value V22327_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22327_323 = {S42, 0, &V22327_323_57299, &P22327_322};	/*Property = Selector*/
const Value V22327_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22327_324 = {S43, 0, &V22327_324_57306, &P22327_323};	/*Property = Svg*/
const Value V22327_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22327_327 = {S44, 0, &V22327_327_57315, &P22327_324};	/*Property = TouchEvents*/
const Value V22327_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22327_328 = {S45, 0, &V22327_328_57318, &P22327_327};	/*Property = Track*/
const Value V22327_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22327_329 = {S46, 0, &V22327_329_57323, &P22327_328};	/*Property = Video*/
const Value V22327_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22327_330 = {S47, 0, &V22327_330_57326, &P22327_329};	/*Property = Viewport*/
const Value V22327_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22327_332 = {S48, 0, &V22327_332_57338, &P22327_330};	/*Property = WebWorkers*/
const Value V22327_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22327_333 = {S49, 0, &V22327_333_57342, &P22327_332};	/*Property = Xhr2*/
const Profile P22327 = {"22327", &P22327_333};	/*Profile ID - 22327*/
const Value V21603_1_1 = {S2, NULL};	/*Value = True*/
const Property P21603_1 = {S50, 0, &V21603_1_1, NULL};	/*Property = IsMobile*/
const Value V21603_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21603_5 = {S52, 0, &V21603_5_1458, &P21603_1};	/*Property = ScreenPixelsHeight*/
const Value V21603_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21603_4 = {S54, 0, &V21603_4_37617, &P21603_5};	/*Property = ScreenPixelsWidth*/
const Profile P21603 = {"21603", &P21603_4};	/*Profile ID - 21603*/
const Value V21920_279_57147 = {S2, NULL};	/*Value = True*/
const Property P21920_279 = {S1, 0, &V21920_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V21920_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21920_282 = {S3, 0, &V21920_282_57154, &P21920_279};	/*Property = BlobBuilder*/
const Value V21920_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21920_350 = {S4, 0, &V21920_350_57444, &P21920_282};	/*Property = Canvas*/
const Value V21920_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21920_283 = {S5, 0, &V21920_283_57159, &P21920_350};	/*Property = CssBackground*/
const Value V21920_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21920_284 = {S6, 0, &V21920_284_57163, &P21920_283};	/*Property = CssBorderImage*/
const Value V21920_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21920_285 = {S7, 0, &V21920_285_57166, &P21920_284};	/*Property = CssCanvas*/
const Value V21920_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21920_286 = {S8, 0, &V21920_286_57168, &P21920_285};	/*Property = CssColor*/
const Value V21920_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21920_237 = {S9, 0, &V21920_237_55786, &P21920_286};	/*Property = CssColumn*/
const Value V21920_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21920_287 = {S10, 0, &V21920_287_57172, &P21920_237};	/*Property = CssFlexbox*/
const Value V21920_288_57174 = {S2, NULL};	/*Value = True*/
const Property P21920_288 = {S11, 0, &V21920_288_57174, &P21920_287};	/*Property = CssFont*/
const Value V21920_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21920_289 = {S12, 0, &V21920_289_57759, &P21920_288};	/*Property = CssImages*/
const Value V21920_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21920_291 = {S13, 0, &V21920_291_57181, &P21920_289};	/*Property = CssMediaQueries*/
const Value V21920_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21920_292 = {S14, 0, &V21920_292_57183, &P21920_291};	/*Property = CssMinMax*/
const Value V21920_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21920_293 = {S15, 0, &V21920_293_57187, &P21920_292};	/*Property = CssOverflow*/
const Value V21920_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21920_294 = {S16, 0, &V21920_294_57189, &P21920_293};	/*Property = CssPosition*/
const Value V21920_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21920_295 = {S17, 0, &V21920_295_57192, &P21920_294};	/*Property = CssText*/
const Value V21920_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21920_297 = {S18, 0, &V21920_297_57195, &P21920_295};	/*Property = CssTransforms*/
const Value V21920_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21920_241 = {S19, 0, &V21920_241_55799, &P21920_297};	/*Property = CssTransitions*/
const Value V21920_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21920_298 = {S20, 0, &V21920_298_57206, &P21920_241};	/*Property = CssUI*/
const Value V21920_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21920_300 = {S21, 0, &V21920_300_57216, &P21920_298};	/*Property = DataSet*/
const Value V21920_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21920_301 = {S22, 0, &V21920_301_57218, &P21920_300};	/*Property = DataUrl*/
const Value V21920_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21920_302 = {S23, 0, &V21920_302_57222, &P21920_301};	/*Property = DeviceOrientation*/
const Value V21920_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21920_304 = {S24, 0, &V21920_304_57227, &P21920_302};	/*Property = FileReader*/
const Value V21920_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21920_305 = {S25, 0, &V21920_305_57231, &P21920_304};	/*Property = FileSaver*/
const Value V21920_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21920_306 = {S26, 0, &V21920_306_57237, &P21920_305};	/*Property = FileWriter*/
const Value V21920_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21920_307 = {S27, 0, &V21920_307_57239, &P21920_306};	/*Property = FormData*/
const Value V21920_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21920_309 = {S28, 0, &V21920_309_57246, &P21920_307};	/*Property = Fullscreen*/
const Value V21920_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21920_310 = {S29, 0, &V21920_310_57248, &P21920_309};	/*Property = GeoLocation*/
const Value V21920_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21920_312 = {S30, 0, &V21920_312_57254, &P21920_310};	/*Property = History*/
const Value V21920_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21920_314 = {S31, 0, &V21920_314_57261, &P21920_312};	/*Property = Html5*/
const Value V21920_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21920_313 = {S32, 0, &V21920_313_57258, &P21920_314};	/*Property = Html-Media-Capture*/
const Value V21920_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21920_315 = {S33, 0, &V21920_315_57264, &P21920_313};	/*Property = Iframe*/
const Value V21920_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21920_316 = {S34, 0, &V21920_316_57266, &P21920_315};	/*Property = IndexedDB*/
const Value V21920_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21920_317 = {S35, 0, &V21920_317_57269, &P21920_316};	/*Property = Json*/
const Value V21920_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21920_15 = {S37, 0, &V21920_15_28, &P21920_317};	/*Property = LayoutEngine*/
const Value V21920_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21920_344 = {S38, 0, &V21920_344_57272, &P21920_15};	/*Property = Masking*/
const Value V21920_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21920_320 = {S39, 0, &V21920_320_57290, &P21920_344};	/*Property = PostMessage*/
const Value V21920_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21920_321 = {S40, 0, &V21920_321_57293, &P21920_320};	/*Property = Progress*/
const Value V21920_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21920_322 = {S41, 0, &V21920_322_57296, &P21920_321};	/*Property = Prompts*/
const Value V21920_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21920_323 = {S42, 0, &V21920_323_57299, &P21920_322};	/*Property = Selector*/
const Value V21920_324_57305 = {S2, NULL};	/*Value = True*/
const Property P21920_324 = {S43, 0, &V21920_324_57305, &P21920_323};	/*Property = Svg*/
const Value V21920_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21920_327 = {S44, 0, &V21920_327_57315, &P21920_324};	/*Property = TouchEvents*/
const Value V21920_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21920_328 = {S45, 0, &V21920_328_57318, &P21920_327};	/*Property = Track*/
const Value V21920_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21920_329 = {S46, 0, &V21920_329_57323, &P21920_328};	/*Property = Video*/
const Value V21920_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21920_330 = {S47, 0, &V21920_330_57326, &P21920_329};	/*Property = Viewport*/
const Value V21920_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21920_332 = {S48, 0, &V21920_332_57339, &P21920_330};	/*Property = WebWorkers*/
const Value V21920_333_57341 = {S2, NULL};	/*Value = True*/
const Property P21920_333 = {S49, 0, &V21920_333_57341, &P21920_332};	/*Property = Xhr2*/
const Profile P21920 = {"21920", &P21920_333};	/*Profile ID - 21920*/
const Value V21418_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21418_279 = {S1, 0, &V21418_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21418_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21418_282 = {S3, 0, &V21418_282_57154, &P21418_279};	/*Property = BlobBuilder*/
const Value V21418_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21418_350 = {S4, 0, &V21418_350_57444, &P21418_282};	/*Property = Canvas*/
const Value V21418_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21418_283 = {S5, 0, &V21418_283_57159, &P21418_350};	/*Property = CssBackground*/
const Value V21418_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21418_284 = {S6, 0, &V21418_284_57163, &P21418_283};	/*Property = CssBorderImage*/
const Value V21418_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21418_285 = {S7, 0, &V21418_285_57166, &P21418_284};	/*Property = CssCanvas*/
const Value V21418_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21418_286 = {S8, 0, &V21418_286_57168, &P21418_285};	/*Property = CssColor*/
const Value V21418_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21418_237 = {S9, 0, &V21418_237_55786, &P21418_286};	/*Property = CssColumn*/
const Value V21418_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21418_287 = {S10, 0, &V21418_287_57172, &P21418_237};	/*Property = CssFlexbox*/
const Value V21418_288_57174 = {S2, NULL};	/*Value = True*/
const Property P21418_288 = {S11, 0, &V21418_288_57174, &P21418_287};	/*Property = CssFont*/
const Value V21418_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21418_289 = {S12, 0, &V21418_289_57759, &P21418_288};	/*Property = CssImages*/
const Value V21418_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21418_291 = {S13, 0, &V21418_291_57180, &P21418_289};	/*Property = CssMediaQueries*/
const Value V21418_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21418_292 = {S14, 0, &V21418_292_57183, &P21418_291};	/*Property = CssMinMax*/
const Value V21418_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21418_293 = {S15, 0, &V21418_293_57187, &P21418_292};	/*Property = CssOverflow*/
const Value V21418_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21418_294 = {S16, 0, &V21418_294_57189, &P21418_293};	/*Property = CssPosition*/
const Value V21418_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21418_295 = {S17, 0, &V21418_295_57192, &P21418_294};	/*Property = CssText*/
const Value V21418_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21418_297 = {S18, 0, &V21418_297_57195, &P21418_295};	/*Property = CssTransforms*/
const Value V21418_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21418_241 = {S19, 0, &V21418_241_55799, &P21418_297};	/*Property = CssTransitions*/
const Value V21418_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21418_298 = {S20, 0, &V21418_298_57206, &P21418_241};	/*Property = CssUI*/
const Value V21418_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21418_300 = {S21, 0, &V21418_300_57216, &P21418_298};	/*Property = DataSet*/
const Value V21418_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21418_301 = {S22, 0, &V21418_301_57218, &P21418_300};	/*Property = DataUrl*/
const Value V21418_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21418_302 = {S23, 0, &V21418_302_57222, &P21418_301};	/*Property = DeviceOrientation*/
const Value V21418_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21418_304 = {S24, 0, &V21418_304_57227, &P21418_302};	/*Property = FileReader*/
const Value V21418_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21418_305 = {S25, 0, &V21418_305_57231, &P21418_304};	/*Property = FileSaver*/
const Value V21418_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21418_306 = {S26, 0, &V21418_306_57237, &P21418_305};	/*Property = FileWriter*/
const Value V21418_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21418_307 = {S27, 0, &V21418_307_57239, &P21418_306};	/*Property = FormData*/
const Value V21418_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21418_309 = {S28, 0, &V21418_309_57246, &P21418_307};	/*Property = Fullscreen*/
const Value V21418_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21418_310 = {S29, 0, &V21418_310_57248, &P21418_309};	/*Property = GeoLocation*/
const Value V21418_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21418_312 = {S30, 0, &V21418_312_57254, &P21418_310};	/*Property = History*/
const Value V21418_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21418_314 = {S31, 0, &V21418_314_57261, &P21418_312};	/*Property = Html5*/
const Value V21418_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21418_313 = {S32, 0, &V21418_313_57258, &P21418_314};	/*Property = Html-Media-Capture*/
const Value V21418_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21418_315 = {S33, 0, &V21418_315_57264, &P21418_313};	/*Property = Iframe*/
const Value V21418_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21418_316 = {S34, 0, &V21418_316_57266, &P21418_315};	/*Property = IndexedDB*/
const Value V21418_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21418_317 = {S35, 0, &V21418_317_57269, &P21418_316};	/*Property = Json*/
const Value V21418_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21418_15 = {S37, 0, &V21418_15_28, &P21418_317};	/*Property = LayoutEngine*/
const Value V21418_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21418_344 = {S38, 0, &V21418_344_57272, &P21418_15};	/*Property = Masking*/
const Value V21418_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21418_320 = {S39, 0, &V21418_320_57290, &P21418_344};	/*Property = PostMessage*/
const Value V21418_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21418_321 = {S40, 0, &V21418_321_57293, &P21418_320};	/*Property = Progress*/
const Value V21418_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21418_322 = {S41, 0, &V21418_322_57296, &P21418_321};	/*Property = Prompts*/
const Value V21418_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21418_323 = {S42, 0, &V21418_323_57299, &P21418_322};	/*Property = Selector*/
const Value V21418_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21418_324 = {S43, 0, &V21418_324_57306, &P21418_323};	/*Property = Svg*/
const Value V21418_327_57314 = {S2, NULL};	/*Value = True*/
const Property P21418_327 = {S44, 0, &V21418_327_57314, &P21418_324};	/*Property = TouchEvents*/
const Value V21418_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21418_328 = {S45, 0, &V21418_328_57318, &P21418_327};	/*Property = Track*/
const Value V21418_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21418_329 = {S46, 0, &V21418_329_57323, &P21418_328};	/*Property = Video*/
const Value V21418_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21418_330 = {S47, 0, &V21418_330_57326, &P21418_329};	/*Property = Viewport*/
const Value V21418_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21418_332 = {S48, 0, &V21418_332_57339, &P21418_330};	/*Property = WebWorkers*/
const Value V21418_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21418_333 = {S49, 0, &V21418_333_57342, &P21418_332};	/*Property = Xhr2*/
const Profile P21418 = {"21418", &P21418_333};	/*Profile ID - 21418*/
const Value V21714_1_1 = {S2, NULL};	/*Value = True*/
const Property P21714_1 = {S50, 0, &V21714_1_1, NULL};	/*Property = IsMobile*/
const Value V21714_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P21714_5 = {S52, 0, &V21714_5_59138, &P21714_1};	/*Property = ScreenPixelsHeight*/
const Value V21714_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P21714_4 = {S54, 0, &V21714_4_59970, &P21714_5};	/*Property = ScreenPixelsWidth*/
const Profile P21714 = {"21714", &P21714_4};	/*Profile ID - 21714*/
const Value V21283_1_1 = {S2, NULL};	/*Value = True*/
const Property P21283_1 = {S50, 0, &V21283_1_1, NULL};	/*Property = IsMobile*/
const Value V21283_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21283_5 = {S52, 0, &V21283_5_1277, &P21283_1};	/*Property = ScreenPixelsHeight*/
const Value V21283_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21283_4 = {S54, 0, &V21283_4_1279, &P21283_5};	/*Property = ScreenPixelsWidth*/
const Profile P21283 = {"21283", &P21283_4};	/*Profile ID - 21283*/
const Value V21264_1_1 = {S2, NULL};	/*Value = True*/
const Property P21264_1 = {S50, 0, &V21264_1_1, NULL};	/*Property = IsMobile*/
const Value V21264_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P21264_5 = {S52, 0, &V21264_5_59138, &P21264_1};	/*Property = ScreenPixelsHeight*/
const Value V21264_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P21264_4 = {S54, 0, &V21264_4_59970, &P21264_5};	/*Property = ScreenPixelsWidth*/
const Profile P21264 = {"21264", &P21264_4};	/*Profile ID - 21264*/
const Value V20760_1_1 = {S2, NULL};	/*Value = True*/
const Property P20760_1 = {S50, 0, &V20760_1_1, NULL};	/*Property = IsMobile*/
const Value V20760_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20760_5 = {S52, 0, &V20760_5_1277, &P20760_1};	/*Property = ScreenPixelsHeight*/
const Value V20760_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20760_4 = {S54, 0, &V20760_4_1279, &P20760_5};	/*Property = ScreenPixelsWidth*/
const Profile P20760 = {"20760", &P20760_4};	/*Profile ID - 20760*/
const Value V20999_1_1 = {S2, NULL};	/*Value = True*/
const Property P20999_1 = {S50, 0, &V20999_1_1, NULL};	/*Property = IsMobile*/
const Value V20999_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20999_5 = {S52, 0, &V20999_5_1277, &P20999_1};	/*Property = ScreenPixelsHeight*/
const Value V20999_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20999_4 = {S54, 0, &V20999_4_1279, &P20999_5};	/*Property = ScreenPixelsWidth*/
const Profile P20999 = {"20999", &P20999_4};	/*Profile ID - 20999*/
const Value V21000_1_1 = {S2, NULL};	/*Value = True*/
const Property P21000_1 = {S50, 0, &V21000_1_1, NULL};	/*Property = IsMobile*/
const Value V21000_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21000_5 = {S52, 0, &V21000_5_1222, &P21000_1};	/*Property = ScreenPixelsHeight*/
const Value V21000_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21000_4 = {S54, 0, &V21000_4_1224, &P21000_5};	/*Property = ScreenPixelsWidth*/
const Profile P21000 = {"21000", &P21000_4};	/*Profile ID - 21000*/
const Value V21001_1_1 = {S2, NULL};	/*Value = True*/
const Property P21001_1 = {S50, 0, &V21001_1_1, NULL};	/*Property = IsMobile*/
const Value V21001_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21001_5 = {S52, 0, &V21001_5_1222, &P21001_1};	/*Property = ScreenPixelsHeight*/
const Value V21001_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21001_4 = {S54, 0, &V21001_4_1224, &P21001_5};	/*Property = ScreenPixelsWidth*/
const Profile P21001 = {"21001", &P21001_4};	/*Profile ID - 21001*/
const Value V21002_1_1 = {S2, NULL};	/*Value = True*/
const Property P21002_1 = {S50, 0, &V21002_1_1, NULL};	/*Property = IsMobile*/
const Value V21002_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21002_5 = {S52, 0, &V21002_5_1222, &P21002_1};	/*Property = ScreenPixelsHeight*/
const Value V21002_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21002_4 = {S54, 0, &V21002_4_1224, &P21002_5};	/*Property = ScreenPixelsWidth*/
const Profile P21002 = {"21002", &P21002_4};	/*Profile ID - 21002*/
const Value V20587_1_1 = {S2, NULL};	/*Value = True*/
const Property P20587_1 = {S50, 0, &V20587_1_1, NULL};	/*Property = IsMobile*/
const Value V20587_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20587_5 = {S52, 0, &V20587_5_1227, &P20587_1};	/*Property = ScreenPixelsHeight*/
const Value V20587_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20587_4 = {S54, 0, &V20587_4_1229, &P20587_5};	/*Property = ScreenPixelsWidth*/
const Profile P20587 = {"20587", &P20587_4};	/*Profile ID - 20587*/
const Value V21285_1_1 = {S2, NULL};	/*Value = True*/
const Property P21285_1 = {S50, 0, &V21285_1_1, NULL};	/*Property = IsMobile*/
const Value V21285_5_39096 = {S104, NULL};	/*Value = 720*/
const Property P21285_5 = {S52, 0, &V21285_5_39096, &P21285_1};	/*Property = ScreenPixelsHeight*/
const Value V21285_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21285_4 = {S54, 0, &V21285_4_39098, &P21285_5};	/*Property = ScreenPixelsWidth*/
const Profile P21285 = {"21285", &P21285_4};	/*Profile ID - 21285*/
const Profile P22468 = {"22468", NULL};	/*Profile ID - 22468*/
const Value V22813_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22813_279 = {S1, 0, &V22813_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22813_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22813_282 = {S3, 0, &V22813_282_57154, &P22813_279};	/*Property = BlobBuilder*/
const Value V22813_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22813_350 = {S4, 0, &V22813_350_57444, &P22813_282};	/*Property = Canvas*/
const Value V22813_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22813_283 = {S5, 0, &V22813_283_57159, &P22813_350};	/*Property = CssBackground*/
const Value V22813_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22813_284 = {S6, 0, &V22813_284_57162, &P22813_283};	/*Property = CssBorderImage*/
const Value V22813_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22813_285 = {S7, 0, &V22813_285_57166, &P22813_284};	/*Property = CssCanvas*/
const Value V22813_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22813_286 = {S8, 0, &V22813_286_57168, &P22813_285};	/*Property = CssColor*/
const Value V22813_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22813_237 = {S9, 0, &V22813_237_55786, &P22813_286};	/*Property = CssColumn*/
const Value V22813_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22813_287 = {S10, 0, &V22813_287_57172, &P22813_237};	/*Property = CssFlexbox*/
const Value V22813_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22813_288 = {S11, 0, &V22813_288_57174, &P22813_287};	/*Property = CssFont*/
const Value V22813_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22813_289 = {S12, 0, &V22813_289_57759, &P22813_288};	/*Property = CssImages*/
const Value V22813_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22813_291 = {S13, 0, &V22813_291_57180, &P22813_289};	/*Property = CssMediaQueries*/
const Value V22813_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22813_292 = {S14, 0, &V22813_292_57183, &P22813_291};	/*Property = CssMinMax*/
const Value V22813_293_57186 = {S2, NULL};	/*Value = True*/
const Property P22813_293 = {S15, 0, &V22813_293_57186, &P22813_292};	/*Property = CssOverflow*/
const Value V22813_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22813_294 = {S16, 0, &V22813_294_57189, &P22813_293};	/*Property = CssPosition*/
const Value V22813_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22813_295 = {S17, 0, &V22813_295_57192, &P22813_294};	/*Property = CssText*/
const Value V22813_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22813_297 = {S18, 0, &V22813_297_57195, &P22813_295};	/*Property = CssTransforms*/
const Value V22813_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22813_241 = {S19, 0, &V22813_241_55799, &P22813_297};	/*Property = CssTransitions*/
const Value V22813_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22813_298 = {S20, 0, &V22813_298_57206, &P22813_241};	/*Property = CssUI*/
const Value V22813_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22813_300 = {S21, 0, &V22813_300_57215, &P22813_298};	/*Property = DataSet*/
const Value V22813_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22813_301 = {S22, 0, &V22813_301_57218, &P22813_300};	/*Property = DataUrl*/
const Value V22813_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22813_302 = {S23, 0, &V22813_302_57221, &P22813_301};	/*Property = DeviceOrientation*/
const Value V22813_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22813_304 = {S24, 0, &V22813_304_57227, &P22813_302};	/*Property = FileReader*/
const Value V22813_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22813_305 = {S25, 0, &V22813_305_57231, &P22813_304};	/*Property = FileSaver*/
const Value V22813_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22813_306 = {S26, 0, &V22813_306_57237, &P22813_305};	/*Property = FileWriter*/
const Value V22813_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22813_307 = {S27, 0, &V22813_307_57239, &P22813_306};	/*Property = FormData*/
const Value V22813_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22813_309 = {S28, 0, &V22813_309_57246, &P22813_307};	/*Property = Fullscreen*/
const Value V22813_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22813_310 = {S29, 0, &V22813_310_57248, &P22813_309};	/*Property = GeoLocation*/
const Value V22813_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22813_312 = {S30, 0, &V22813_312_57254, &P22813_310};	/*Property = History*/
const Value V22813_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22813_314 = {S31, 0, &V22813_314_57260, &P22813_312};	/*Property = Html5*/
const Value V22813_313_57257 = {S2, NULL};	/*Value = True*/
const Property P22813_313 = {S32, 0, &V22813_313_57257, &P22813_314};	/*Property = Html-Media-Capture*/
const Value V22813_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22813_315 = {S33, 0, &V22813_315_57264, &P22813_313};	/*Property = Iframe*/
const Value V22813_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22813_316 = {S34, 0, &V22813_316_57266, &P22813_315};	/*Property = IndexedDB*/
const Value V22813_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22813_317 = {S35, 0, &V22813_317_57269, &P22813_316};	/*Property = Json*/
const Value V22813_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22813_15 = {S37, 0, &V22813_15_28, &P22813_317};	/*Property = LayoutEngine*/
const Value V22813_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22813_344 = {S38, 0, &V22813_344_57272, &P22813_15};	/*Property = Masking*/
const Value V22813_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22813_320 = {S39, 0, &V22813_320_57290, &P22813_344};	/*Property = PostMessage*/
const Value V22813_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22813_321 = {S40, 0, &V22813_321_57293, &P22813_320};	/*Property = Progress*/
const Value V22813_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22813_322 = {S41, 0, &V22813_322_57296, &P22813_321};	/*Property = Prompts*/
const Value V22813_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22813_323 = {S42, 0, &V22813_323_57299, &P22813_322};	/*Property = Selector*/
const Value V22813_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22813_324 = {S43, 0, &V22813_324_57305, &P22813_323};	/*Property = Svg*/
const Value V22813_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22813_327 = {S44, 0, &V22813_327_57314, &P22813_324};	/*Property = TouchEvents*/
const Value V22813_328_57317 = {S2, NULL};	/*Value = True*/
const Property P22813_328 = {S45, 0, &V22813_328_57317, &P22813_327};	/*Property = Track*/
const Value V22813_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22813_329 = {S46, 0, &V22813_329_57323, &P22813_328};	/*Property = Video*/
const Value V22813_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22813_330 = {S47, 0, &V22813_330_57326, &P22813_329};	/*Property = Viewport*/
const Value V22813_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22813_332 = {S48, 0, &V22813_332_57338, &P22813_330};	/*Property = WebWorkers*/
const Value V22813_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22813_333 = {S49, 0, &V22813_333_57341, &P22813_332};	/*Property = Xhr2*/
const Profile P22813 = {"22813", &P22813_333};	/*Profile ID - 22813*/
const Value V22888_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_279 = {S1, 0, &V22888_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22888_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_282 = {S3, 0, &V22888_282_57155, &P22888_279};	/*Property = BlobBuilder*/
const Value V22888_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_350 = {S4, 0, &V22888_350_57446, &P22888_282};	/*Property = Canvas*/
const Value V22888_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_283 = {S5, 0, &V22888_283_57161, &P22888_350};	/*Property = CssBackground*/
const Value V22888_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_284 = {S6, 0, &V22888_284_57164, &P22888_283};	/*Property = CssBorderImage*/
const Value V22888_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_285 = {S7, 0, &V22888_285_57167, &P22888_284};	/*Property = CssCanvas*/
const Value V22888_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_286 = {S8, 0, &V22888_286_57170, &P22888_285};	/*Property = CssColor*/
const Value V22888_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_237 = {S9, 0, &V22888_237_55786, &P22888_286};	/*Property = CssColumn*/
const Value V22888_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_287 = {S10, 0, &V22888_287_57173, &P22888_237};	/*Property = CssFlexbox*/
const Value V22888_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_288 = {S11, 0, &V22888_288_57176, &P22888_287};	/*Property = CssFont*/
const Value V22888_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_289 = {S12, 0, &V22888_289_86899, &P22888_288};	/*Property = CssImages*/
const Value V22888_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_291 = {S13, 0, &V22888_291_57182, &P22888_289};	/*Property = CssMediaQueries*/
const Value V22888_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_292 = {S14, 0, &V22888_292_57185, &P22888_291};	/*Property = CssMinMax*/
const Value V22888_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_293 = {S15, 0, &V22888_293_57188, &P22888_292};	/*Property = CssOverflow*/
const Value V22888_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_294 = {S16, 0, &V22888_294_57191, &P22888_293};	/*Property = CssPosition*/
const Value V22888_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_295 = {S17, 0, &V22888_295_57194, &P22888_294};	/*Property = CssText*/
const Value V22888_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_297 = {S18, 0, &V22888_297_57197, &P22888_295};	/*Property = CssTransforms*/
const Value V22888_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_241 = {S19, 0, &V22888_241_55801, &P22888_297};	/*Property = CssTransitions*/
const Value V22888_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_298 = {S20, 0, &V22888_298_57208, &P22888_241};	/*Property = CssUI*/
const Value V22888_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_300 = {S21, 0, &V22888_300_57217, &P22888_298};	/*Property = DataSet*/
const Value V22888_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_301 = {S22, 0, &V22888_301_57220, &P22888_300};	/*Property = DataUrl*/
const Value V22888_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_302 = {S23, 0, &V22888_302_57223, &P22888_301};	/*Property = DeviceOrientation*/
const Value V22888_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_304 = {S24, 0, &V22888_304_57229, &P22888_302};	/*Property = FileReader*/
const Value V22888_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_305 = {S25, 0, &V22888_305_57232, &P22888_304};	/*Property = FileSaver*/
const Value V22888_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_306 = {S26, 0, &V22888_306_57238, &P22888_305};	/*Property = FileWriter*/
const Value V22888_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_307 = {S27, 0, &V22888_307_57241, &P22888_306};	/*Property = FormData*/
const Value V22888_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_309 = {S28, 0, &V22888_309_57247, &P22888_307};	/*Property = Fullscreen*/
const Value V22888_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_310 = {S29, 0, &V22888_310_57250, &P22888_309};	/*Property = GeoLocation*/
const Value V22888_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_312 = {S30, 0, &V22888_312_57256, &P22888_310};	/*Property = History*/
const Value V22888_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_314 = {S31, 0, &V22888_314_57262, &P22888_312};	/*Property = Html5*/
const Value V22888_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_313 = {S32, 0, &V22888_313_57259, &P22888_314};	/*Property = Html-Media-Capture*/
const Value V22888_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_315 = {S33, 0, &V22888_315_57265, &P22888_313};	/*Property = Iframe*/
const Value V22888_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_316 = {S34, 0, &V22888_316_57268, &P22888_315};	/*Property = IndexedDB*/
const Value V22888_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_317 = {S35, 0, &V22888_317_57271, &P22888_316};	/*Property = Json*/
const Value V22888_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_15 = {S37, 0, &V22888_15_72, &P22888_317};	/*Property = LayoutEngine*/
const Value V22888_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_344 = {S38, 0, &V22888_344_57274, &P22888_15};	/*Property = Masking*/
const Value V22888_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_320 = {S39, 0, &V22888_320_57292, &P22888_344};	/*Property = PostMessage*/
const Value V22888_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_321 = {S40, 0, &V22888_321_57295, &P22888_320};	/*Property = Progress*/
const Value V22888_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_322 = {S41, 0, &V22888_322_57298, &P22888_321};	/*Property = Prompts*/
const Value V22888_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_323 = {S42, 0, &V22888_323_57301, &P22888_322};	/*Property = Selector*/
const Value V22888_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_324 = {S43, 0, &V22888_324_57307, &P22888_323};	/*Property = Svg*/
const Value V22888_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_327 = {S44, 0, &V22888_327_57316, &P22888_324};	/*Property = TouchEvents*/
const Value V22888_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_328 = {S45, 0, &V22888_328_57319, &P22888_327};	/*Property = Track*/
const Value V22888_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_329 = {S46, 0, &V22888_329_57325, &P22888_328};	/*Property = Video*/
const Value V22888_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_330 = {S47, 0, &V22888_330_57328, &P22888_329};	/*Property = Viewport*/
const Value V22888_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_332 = {S48, 0, &V22888_332_57340, &P22888_330};	/*Property = WebWorkers*/
const Value V22888_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22888_333 = {S49, 0, &V22888_333_57343, &P22888_332};	/*Property = Xhr2*/
const Profile P22888 = {"22888", &P22888_333};	/*Profile ID - 22888*/
const Value V22493_1_1 = {S2, NULL};	/*Value = True*/
const Property P22493_1 = {S50, 0, &V22493_1_1, NULL};	/*Property = IsMobile*/
const Value V22493_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22493_5 = {S52, 0, &V22493_5_1277, &P22493_1};	/*Property = ScreenPixelsHeight*/
const Value V22493_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22493_4 = {S54, 0, &V22493_4_2336, &P22493_5};	/*Property = ScreenPixelsWidth*/
const Profile P22493 = {"22493", &P22493_4};	/*Profile ID - 22493*/
const Value V24014_1_1 = {S2, NULL};	/*Value = True*/
const Property P24014_1 = {S50, 0, &V24014_1_1, NULL};	/*Property = IsMobile*/
const Value V24014_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24014_5 = {S52, 0, &V24014_5_1277, &P24014_1};	/*Property = ScreenPixelsHeight*/
const Value V24014_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24014_4 = {S54, 0, &V24014_4_1279, &P24014_5};	/*Property = ScreenPixelsWidth*/
const Profile P24014 = {"24014", &P24014_4};	/*Profile ID - 24014*/
const Value V22724_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_279 = {S1, 0, &V22724_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22724_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_282 = {S3, 0, &V22724_282_57155, &P22724_279};	/*Property = BlobBuilder*/
const Value V22724_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_350 = {S4, 0, &V22724_350_57446, &P22724_282};	/*Property = Canvas*/
const Value V22724_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_283 = {S5, 0, &V22724_283_57161, &P22724_350};	/*Property = CssBackground*/
const Value V22724_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_284 = {S6, 0, &V22724_284_57164, &P22724_283};	/*Property = CssBorderImage*/
const Value V22724_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_285 = {S7, 0, &V22724_285_57167, &P22724_284};	/*Property = CssCanvas*/
const Value V22724_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_286 = {S8, 0, &V22724_286_57170, &P22724_285};	/*Property = CssColor*/
const Value V22724_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_237 = {S9, 0, &V22724_237_55786, &P22724_286};	/*Property = CssColumn*/
const Value V22724_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_287 = {S10, 0, &V22724_287_57173, &P22724_237};	/*Property = CssFlexbox*/
const Value V22724_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_288 = {S11, 0, &V22724_288_57176, &P22724_287};	/*Property = CssFont*/
const Value V22724_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_289 = {S12, 0, &V22724_289_86899, &P22724_288};	/*Property = CssImages*/
const Value V22724_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_291 = {S13, 0, &V22724_291_57182, &P22724_289};	/*Property = CssMediaQueries*/
const Value V22724_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_292 = {S14, 0, &V22724_292_57185, &P22724_291};	/*Property = CssMinMax*/
const Value V22724_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_293 = {S15, 0, &V22724_293_57188, &P22724_292};	/*Property = CssOverflow*/
const Value V22724_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_294 = {S16, 0, &V22724_294_57191, &P22724_293};	/*Property = CssPosition*/
const Value V22724_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_295 = {S17, 0, &V22724_295_57194, &P22724_294};	/*Property = CssText*/
const Value V22724_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_297 = {S18, 0, &V22724_297_57197, &P22724_295};	/*Property = CssTransforms*/
const Value V22724_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_241 = {S19, 0, &V22724_241_55801, &P22724_297};	/*Property = CssTransitions*/
const Value V22724_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_298 = {S20, 0, &V22724_298_57208, &P22724_241};	/*Property = CssUI*/
const Value V22724_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_300 = {S21, 0, &V22724_300_57217, &P22724_298};	/*Property = DataSet*/
const Value V22724_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_301 = {S22, 0, &V22724_301_57220, &P22724_300};	/*Property = DataUrl*/
const Value V22724_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_302 = {S23, 0, &V22724_302_57223, &P22724_301};	/*Property = DeviceOrientation*/
const Value V22724_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_304 = {S24, 0, &V22724_304_57229, &P22724_302};	/*Property = FileReader*/
const Value V22724_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_305 = {S25, 0, &V22724_305_57232, &P22724_304};	/*Property = FileSaver*/
const Value V22724_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_306 = {S26, 0, &V22724_306_57238, &P22724_305};	/*Property = FileWriter*/
const Value V22724_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_307 = {S27, 0, &V22724_307_57241, &P22724_306};	/*Property = FormData*/
const Value V22724_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_309 = {S28, 0, &V22724_309_57247, &P22724_307};	/*Property = Fullscreen*/
const Value V22724_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_310 = {S29, 0, &V22724_310_57250, &P22724_309};	/*Property = GeoLocation*/
const Value V22724_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_312 = {S30, 0, &V22724_312_57256, &P22724_310};	/*Property = History*/
const Value V22724_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_314 = {S31, 0, &V22724_314_57262, &P22724_312};	/*Property = Html5*/
const Value V22724_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_313 = {S32, 0, &V22724_313_57259, &P22724_314};	/*Property = Html-Media-Capture*/
const Value V22724_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_315 = {S33, 0, &V22724_315_57265, &P22724_313};	/*Property = Iframe*/
const Value V22724_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_316 = {S34, 0, &V22724_316_57268, &P22724_315};	/*Property = IndexedDB*/
const Value V22724_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_317 = {S35, 0, &V22724_317_57271, &P22724_316};	/*Property = Json*/
const Value V22724_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22724_15 = {S37, 0, &V22724_15_28, &P22724_317};	/*Property = LayoutEngine*/
const Value V22724_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_344 = {S38, 0, &V22724_344_57274, &P22724_15};	/*Property = Masking*/
const Value V22724_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_320 = {S39, 0, &V22724_320_57292, &P22724_344};	/*Property = PostMessage*/
const Value V22724_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_321 = {S40, 0, &V22724_321_57295, &P22724_320};	/*Property = Progress*/
const Value V22724_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_322 = {S41, 0, &V22724_322_57298, &P22724_321};	/*Property = Prompts*/
const Value V22724_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_323 = {S42, 0, &V22724_323_57301, &P22724_322};	/*Property = Selector*/
const Value V22724_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_324 = {S43, 0, &V22724_324_57307, &P22724_323};	/*Property = Svg*/
const Value V22724_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_327 = {S44, 0, &V22724_327_57316, &P22724_324};	/*Property = TouchEvents*/
const Value V22724_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_328 = {S45, 0, &V22724_328_57319, &P22724_327};	/*Property = Track*/
const Value V22724_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_329 = {S46, 0, &V22724_329_57325, &P22724_328};	/*Property = Video*/
const Value V22724_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_330 = {S47, 0, &V22724_330_57328, &P22724_329};	/*Property = Viewport*/
const Value V22724_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_332 = {S48, 0, &V22724_332_57340, &P22724_330};	/*Property = WebWorkers*/
const Value V22724_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22724_333 = {S49, 0, &V22724_333_57343, &P22724_332};	/*Property = Xhr2*/
const Profile P22724 = {"22724", &P22724_333};	/*Profile ID - 22724*/
const Profile P22723 = {"22723", NULL};	/*Profile ID - 22723*/
const Value V23282_1_1 = {S2, NULL};	/*Value = True*/
const Property P23282_1 = {S50, 0, &V23282_1_1, NULL};	/*Property = IsMobile*/
const Value V23282_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23282_5 = {S52, 0, &V23282_5_1277, &P23282_1};	/*Property = ScreenPixelsHeight*/
const Value V23282_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23282_4 = {S54, 0, &V23282_4_1279, &P23282_5};	/*Property = ScreenPixelsWidth*/
const Profile P23282 = {"23282", &P23282_4};	/*Profile ID - 23282*/
const Value V21837_1_1 = {S2, NULL};	/*Value = True*/
const Property P21837_1 = {S50, 0, &V21837_1_1, NULL};	/*Property = IsMobile*/
const Value V21837_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21837_5 = {S52, 0, &V21837_5_1227, &P21837_1};	/*Property = ScreenPixelsHeight*/
const Value V21837_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21837_4 = {S54, 0, &V21837_4_1229, &P21837_5};	/*Property = ScreenPixelsWidth*/
const Profile P21837 = {"21837", &P21837_4};	/*Profile ID - 21837*/
const Value V23421_1_1 = {S2, NULL};	/*Value = True*/
const Property P23421_1 = {S50, 0, &V23421_1_1, NULL};	/*Property = IsMobile*/
const Value V23421_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23421_5 = {S52, 0, &V23421_5_1832, &P23421_1};	/*Property = ScreenPixelsHeight*/
const Value V23421_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23421_4 = {S54, 0, &V23421_4_5645, &P23421_5};	/*Property = ScreenPixelsWidth*/
const Profile P23421 = {"23421", &P23421_4};	/*Profile ID - 23421*/
const Value V22746_1_1 = {S2, NULL};	/*Value = True*/
const Property P22746_1 = {S50, 0, &V22746_1_1, NULL};	/*Property = IsMobile*/
const Value V22746_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22746_5 = {S52, 0, &V22746_5_1227, &P22746_1};	/*Property = ScreenPixelsHeight*/
const Value V22746_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22746_4 = {S54, 0, &V22746_4_1229, &P22746_5};	/*Property = ScreenPixelsWidth*/
const Profile P22746 = {"22746", &P22746_4};	/*Profile ID - 22746*/
const Value V22116_1_1 = {S2, NULL};	/*Value = True*/
const Property P22116_1 = {S50, 0, &V22116_1_1, NULL};	/*Property = IsMobile*/
const Value V22116_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22116_5 = {S52, 0, &V22116_5_1277, &P22116_1};	/*Property = ScreenPixelsHeight*/
const Value V22116_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22116_4 = {S54, 0, &V22116_4_1279, &P22116_5};	/*Property = ScreenPixelsWidth*/
const Profile P22116 = {"22116", &P22116_4};	/*Profile ID - 22116*/
const Value V23751_1_1 = {S2, NULL};	/*Value = True*/
const Property P23751_1 = {S50, 0, &V23751_1_1, NULL};	/*Property = IsMobile*/
const Value V23751_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23751_5 = {S52, 0, &V23751_5_1222, &P23751_1};	/*Property = ScreenPixelsHeight*/
const Value V23751_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23751_4 = {S54, 0, &V23751_4_1224, &P23751_5};	/*Property = ScreenPixelsWidth*/
const Profile P23751 = {"23751", &P23751_4};	/*Profile ID - 23751*/
const Value V20195_1_1 = {S2, NULL};	/*Value = True*/
const Property P20195_1 = {S50, 0, &V20195_1_1, NULL};	/*Property = IsMobile*/
const Value V20195_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20195_5 = {S52, 0, &V20195_5_1458, &P20195_1};	/*Property = ScreenPixelsHeight*/
const Value V20195_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20195_4 = {S54, 0, &V20195_4_37617, &P20195_5};	/*Property = ScreenPixelsWidth*/
const Profile P20195 = {"20195", &P20195_4};	/*Profile ID - 20195*/
const Value V21758_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_279 = {S1, 0, &V21758_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21758_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_282 = {S3, 0, &V21758_282_57155, &P21758_279};	/*Property = BlobBuilder*/
const Value V21758_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_350 = {S4, 0, &V21758_350_57446, &P21758_282};	/*Property = Canvas*/
const Value V21758_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_283 = {S5, 0, &V21758_283_57161, &P21758_350};	/*Property = CssBackground*/
const Value V21758_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_284 = {S6, 0, &V21758_284_57164, &P21758_283};	/*Property = CssBorderImage*/
const Value V21758_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_285 = {S7, 0, &V21758_285_57167, &P21758_284};	/*Property = CssCanvas*/
const Value V21758_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_286 = {S8, 0, &V21758_286_57170, &P21758_285};	/*Property = CssColor*/
const Value V21758_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_237 = {S9, 0, &V21758_237_55786, &P21758_286};	/*Property = CssColumn*/
const Value V21758_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_287 = {S10, 0, &V21758_287_57173, &P21758_237};	/*Property = CssFlexbox*/
const Value V21758_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_288 = {S11, 0, &V21758_288_57176, &P21758_287};	/*Property = CssFont*/
const Value V21758_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_289 = {S12, 0, &V21758_289_86899, &P21758_288};	/*Property = CssImages*/
const Value V21758_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_291 = {S13, 0, &V21758_291_57182, &P21758_289};	/*Property = CssMediaQueries*/
const Value V21758_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_292 = {S14, 0, &V21758_292_57185, &P21758_291};	/*Property = CssMinMax*/
const Value V21758_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_293 = {S15, 0, &V21758_293_57188, &P21758_292};	/*Property = CssOverflow*/
const Value V21758_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_294 = {S16, 0, &V21758_294_57191, &P21758_293};	/*Property = CssPosition*/
const Value V21758_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_295 = {S17, 0, &V21758_295_57194, &P21758_294};	/*Property = CssText*/
const Value V21758_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_297 = {S18, 0, &V21758_297_57197, &P21758_295};	/*Property = CssTransforms*/
const Value V21758_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_241 = {S19, 0, &V21758_241_55801, &P21758_297};	/*Property = CssTransitions*/
const Value V21758_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_298 = {S20, 0, &V21758_298_57208, &P21758_241};	/*Property = CssUI*/
const Value V21758_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_300 = {S21, 0, &V21758_300_57217, &P21758_298};	/*Property = DataSet*/
const Value V21758_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_301 = {S22, 0, &V21758_301_57220, &P21758_300};	/*Property = DataUrl*/
const Value V21758_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_302 = {S23, 0, &V21758_302_57223, &P21758_301};	/*Property = DeviceOrientation*/
const Value V21758_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_304 = {S24, 0, &V21758_304_57229, &P21758_302};	/*Property = FileReader*/
const Value V21758_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_305 = {S25, 0, &V21758_305_57232, &P21758_304};	/*Property = FileSaver*/
const Value V21758_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_306 = {S26, 0, &V21758_306_57238, &P21758_305};	/*Property = FileWriter*/
const Value V21758_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_307 = {S27, 0, &V21758_307_57241, &P21758_306};	/*Property = FormData*/
const Value V21758_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_309 = {S28, 0, &V21758_309_57247, &P21758_307};	/*Property = Fullscreen*/
const Value V21758_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_310 = {S29, 0, &V21758_310_57250, &P21758_309};	/*Property = GeoLocation*/
const Value V21758_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_312 = {S30, 0, &V21758_312_57256, &P21758_310};	/*Property = History*/
const Value V21758_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_314 = {S31, 0, &V21758_314_57262, &P21758_312};	/*Property = Html5*/
const Value V21758_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_313 = {S32, 0, &V21758_313_57259, &P21758_314};	/*Property = Html-Media-Capture*/
const Value V21758_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_315 = {S33, 0, &V21758_315_57265, &P21758_313};	/*Property = Iframe*/
const Value V21758_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_316 = {S34, 0, &V21758_316_57268, &P21758_315};	/*Property = IndexedDB*/
const Value V21758_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_317 = {S35, 0, &V21758_317_57271, &P21758_316};	/*Property = Json*/
const Value V21758_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21758_15 = {S37, 0, &V21758_15_54167, &P21758_317};	/*Property = LayoutEngine*/
const Value V21758_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_344 = {S38, 0, &V21758_344_57274, &P21758_15};	/*Property = Masking*/
const Value V21758_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_320 = {S39, 0, &V21758_320_57292, &P21758_344};	/*Property = PostMessage*/
const Value V21758_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_321 = {S40, 0, &V21758_321_57295, &P21758_320};	/*Property = Progress*/
const Value V21758_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_322 = {S41, 0, &V21758_322_57298, &P21758_321};	/*Property = Prompts*/
const Value V21758_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_323 = {S42, 0, &V21758_323_57301, &P21758_322};	/*Property = Selector*/
const Value V21758_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_324 = {S43, 0, &V21758_324_57307, &P21758_323};	/*Property = Svg*/
const Value V21758_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_327 = {S44, 0, &V21758_327_57316, &P21758_324};	/*Property = TouchEvents*/
const Value V21758_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_328 = {S45, 0, &V21758_328_57319, &P21758_327};	/*Property = Track*/
const Value V21758_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_329 = {S46, 0, &V21758_329_57325, &P21758_328};	/*Property = Video*/
const Value V21758_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_330 = {S47, 0, &V21758_330_57328, &P21758_329};	/*Property = Viewport*/
const Value V21758_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_332 = {S48, 0, &V21758_332_57340, &P21758_330};	/*Property = WebWorkers*/
const Value V21758_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21758_333 = {S49, 0, &V21758_333_57343, &P21758_332};	/*Property = Xhr2*/
const Profile P21758 = {"21758", &P21758_333};	/*Profile ID - 21758*/
const Value V21375_1_1 = {S2, NULL};	/*Value = True*/
const Property P21375_1 = {S50, 0, &V21375_1_1, NULL};	/*Property = IsMobile*/
const Value V21375_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21375_5 = {S52, 0, &V21375_5_1227, &P21375_1};	/*Property = ScreenPixelsHeight*/
const Value V21375_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21375_4 = {S54, 0, &V21375_4_1229, &P21375_5};	/*Property = ScreenPixelsWidth*/
const Profile P21375 = {"21375", &P21375_4};	/*Profile ID - 21375*/
const Value V22107_1_1 = {S2, NULL};	/*Value = True*/
const Property P22107_1 = {S50, 0, &V22107_1_1, NULL};	/*Property = IsMobile*/
const Value V22107_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P22107_5 = {S52, 0, &V22107_5_1269, &P22107_1};	/*Property = ScreenPixelsHeight*/
const Value V22107_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22107_4 = {S54, 0, &V22107_4_1229, &P22107_5};	/*Property = ScreenPixelsWidth*/
const Profile P22107 = {"22107", &P22107_4};	/*Profile ID - 22107*/
const Value V22005_1_1 = {S2, NULL};	/*Value = True*/
const Property P22005_1 = {S50, 0, &V22005_1_1, NULL};	/*Property = IsMobile*/
const Value V22005_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22005_5 = {S52, 0, &V22005_5_37569, &P22005_1};	/*Property = ScreenPixelsHeight*/
const Value V22005_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P22005_4 = {S54, 0, &V22005_4_1477, &P22005_5};	/*Property = ScreenPixelsWidth*/
const Profile P22005 = {"22005", &P22005_4};	/*Profile ID - 22005*/
const Value V21492_1_1 = {S2, NULL};	/*Value = True*/
const Property P21492_1 = {S50, 0, &V21492_1_1, NULL};	/*Property = IsMobile*/
const Value V21492_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21492_5 = {S52, 0, &V21492_5_1227, &P21492_1};	/*Property = ScreenPixelsHeight*/
const Value V21492_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21492_4 = {S54, 0, &V21492_4_1229, &P21492_5};	/*Property = ScreenPixelsWidth*/
const Profile P21492 = {"21492", &P21492_4};	/*Profile ID - 21492*/
const Value V22175_1_1 = {S2, NULL};	/*Value = True*/
const Property P22175_1 = {S50, 0, &V22175_1_1, NULL};	/*Property = IsMobile*/
const Value V22175_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22175_5 = {S52, 0, &V22175_5_1403, &P22175_1};	/*Property = ScreenPixelsHeight*/
const Value V22175_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22175_4 = {S54, 0, &V22175_4_1404, &P22175_5};	/*Property = ScreenPixelsWidth*/
const Profile P22175 = {"22175", &P22175_4};	/*Profile ID - 22175*/
const Value V22109_1_1 = {S2, NULL};	/*Value = True*/
const Property P22109_1 = {S50, 0, &V22109_1_1, NULL};	/*Property = IsMobile*/
const Value V22109_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22109_5 = {S52, 0, &V22109_5_1227, &P22109_1};	/*Property = ScreenPixelsHeight*/
const Value V22109_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22109_4 = {S54, 0, &V22109_4_1229, &P22109_5};	/*Property = ScreenPixelsWidth*/
const Profile P22109 = {"22109", &P22109_4};	/*Profile ID - 22109*/
const Value V23632_1_2 = {S0, NULL};	/*Value = False*/
const Property P23632_1 = {S50, 0, &V23632_1_2, NULL};	/*Property = IsMobile*/
const Value V23632_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23632_5 = {S52, 0, &V23632_5_60417, &P23632_1};	/*Property = ScreenPixelsHeight*/
const Value V23632_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23632_4 = {S54, 0, &V23632_4_57095, &P23632_5};	/*Property = ScreenPixelsWidth*/
const Profile P23632 = {"23632", &P23632_4};	/*Profile ID - 23632*/
const Value V22137_1_1 = {S2, NULL};	/*Value = True*/
const Property P22137_1 = {S50, 0, &V22137_1_1, NULL};	/*Property = IsMobile*/
const Value V22137_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22137_5 = {S52, 0, &V22137_5_1277, &P22137_1};	/*Property = ScreenPixelsHeight*/
const Value V22137_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22137_4 = {S54, 0, &V22137_4_2336, &P22137_5};	/*Property = ScreenPixelsWidth*/
const Profile P22137 = {"22137", &P22137_4};	/*Profile ID - 22137*/
const Value V21384_1_1 = {S2, NULL};	/*Value = True*/
const Property P21384_1 = {S50, 0, &V21384_1_1, NULL};	/*Property = IsMobile*/
const Value V21384_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P21384_5 = {S52, 0, &V21384_5_59138, &P21384_1};	/*Property = ScreenPixelsHeight*/
const Value V21384_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P21384_4 = {S54, 0, &V21384_4_59970, &P21384_5};	/*Property = ScreenPixelsWidth*/
const Profile P21384 = {"21384", &P21384_4};	/*Profile ID - 21384*/
const Value V22104_1_1 = {S2, NULL};	/*Value = True*/
const Property P22104_1 = {S50, 0, &V22104_1_1, NULL};	/*Property = IsMobile*/
const Value V22104_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22104_5 = {S52, 0, &V22104_5_1227, &P22104_1};	/*Property = ScreenPixelsHeight*/
const Value V22104_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22104_4 = {S54, 0, &V22104_4_1229, &P22104_5};	/*Property = ScreenPixelsWidth*/
const Profile P22104 = {"22104", &P22104_4};	/*Profile ID - 22104*/
const Value V23415_1_1 = {S2, NULL};	/*Value = True*/
const Property P23415_1 = {S50, 0, &V23415_1_1, NULL};	/*Property = IsMobile*/
const Value V23415_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23415_5 = {S52, 0, &V23415_5_37569, &P23415_1};	/*Property = ScreenPixelsHeight*/
const Value V23415_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23415_4 = {S54, 0, &V23415_4_5645, &P23415_5};	/*Property = ScreenPixelsWidth*/
const Profile P23415 = {"23415", &P23415_4};	/*Profile ID - 23415*/
const Value V21611_1_1 = {S2, NULL};	/*Value = True*/
const Property P21611_1 = {S50, 0, &V21611_1_1, NULL};	/*Property = IsMobile*/
const Value V21611_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21611_5 = {S52, 0, &V21611_5_1227, &P21611_1};	/*Property = ScreenPixelsHeight*/
const Value V21611_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21611_4 = {S54, 0, &V21611_4_1229, &P21611_5};	/*Property = ScreenPixelsWidth*/
const Profile P21611 = {"21611", &P21611_4};	/*Profile ID - 21611*/
const Value V21485_1_1 = {S2, NULL};	/*Value = True*/
const Property P21485_1 = {S50, 0, &V21485_1_1, NULL};	/*Property = IsMobile*/
const Value V21485_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21485_5 = {S52, 0, &V21485_5_9981, &P21485_1};	/*Property = ScreenPixelsHeight*/
const Value V21485_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21485_4 = {S54, 0, &V21485_4_39098, &P21485_5};	/*Property = ScreenPixelsWidth*/
const Profile P21485 = {"21485", &P21485_4};	/*Profile ID - 21485*/
const Value V22669_1_1 = {S2, NULL};	/*Value = True*/
const Property P22669_1 = {S50, 0, &V22669_1_1, NULL};	/*Property = IsMobile*/
const Value V22669_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P22669_5 = {S52, 0, &V22669_5_1269, &P22669_1};	/*Property = ScreenPixelsHeight*/
const Value V22669_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22669_4 = {S54, 0, &V22669_4_1229, &P22669_5};	/*Property = ScreenPixelsWidth*/
const Profile P22669 = {"22669", &P22669_4};	/*Profile ID - 22669*/
const Value V21858_1_1 = {S2, NULL};	/*Value = True*/
const Property P21858_1 = {S50, 0, &V21858_1_1, NULL};	/*Property = IsMobile*/
const Value V21858_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21858_5 = {S52, 0, &V21858_5_1227, &P21858_1};	/*Property = ScreenPixelsHeight*/
const Value V21858_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21858_4 = {S54, 0, &V21858_4_1229, &P21858_5};	/*Property = ScreenPixelsWidth*/
const Profile P21858 = {"21858", &P21858_4};	/*Profile ID - 21858*/
const Value V22665_1_1 = {S2, NULL};	/*Value = True*/
const Property P22665_1 = {S50, 0, &V22665_1_1, NULL};	/*Property = IsMobile*/
const Value V22665_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22665_5 = {S52, 0, &V22665_5_1222, &P22665_1};	/*Property = ScreenPixelsHeight*/
const Value V22665_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22665_4 = {S54, 0, &V22665_4_1224, &P22665_5};	/*Property = ScreenPixelsWidth*/
const Profile P22665 = {"22665", &P22665_4};	/*Profile ID - 22665*/
const Value V21874_1_1 = {S2, NULL};	/*Value = True*/
const Property P21874_1 = {S50, 0, &V21874_1_1, NULL};	/*Property = IsMobile*/
const Value V21874_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21874_5 = {S52, 0, &V21874_5_1277, &P21874_1};	/*Property = ScreenPixelsHeight*/
const Value V21874_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21874_4 = {S54, 0, &V21874_4_1279, &P21874_5};	/*Property = ScreenPixelsWidth*/
const Profile P21874 = {"21874", &P21874_4};	/*Profile ID - 21874*/
const Value V23811_1_1 = {S2, NULL};	/*Value = True*/
const Property P23811_1 = {S50, 0, &V23811_1_1, NULL};	/*Property = IsMobile*/
const Value V23811_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23811_5 = {S52, 0, &V23811_5_1227, &P23811_1};	/*Property = ScreenPixelsHeight*/
const Value V23811_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23811_4 = {S54, 0, &V23811_4_1229, &P23811_5};	/*Property = ScreenPixelsWidth*/
const Profile P23811 = {"23811", &P23811_4};	/*Profile ID - 23811*/
const Value V23269_1_1 = {S2, NULL};	/*Value = True*/
const Property P23269_1 = {S50, 0, &V23269_1_1, NULL};	/*Property = IsMobile*/
const Value V23269_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23269_5 = {S52, 0, &V23269_5_1222, &P23269_1};	/*Property = ScreenPixelsHeight*/
const Value V23269_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23269_4 = {S54, 0, &V23269_4_1224, &P23269_5};	/*Property = ScreenPixelsWidth*/
const Profile P23269 = {"23269", &P23269_4};	/*Profile ID - 23269*/
const Value V22126_1_1 = {S2, NULL};	/*Value = True*/
const Property P22126_1 = {S50, 0, &V22126_1_1, NULL};	/*Property = IsMobile*/
const Value V22126_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22126_5 = {S52, 0, &V22126_5_1227, &P22126_1};	/*Property = ScreenPixelsHeight*/
const Value V22126_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22126_4 = {S54, 0, &V22126_4_1229, &P22126_5};	/*Property = ScreenPixelsWidth*/
const Profile P22126 = {"22126", &P22126_4};	/*Profile ID - 22126*/
const Value V21494_1_1 = {S2, NULL};	/*Value = True*/
const Property P21494_1 = {S50, 0, &V21494_1_1, NULL};	/*Property = IsMobile*/
const Value V21494_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21494_5 = {S52, 0, &V21494_5_1227, &P21494_1};	/*Property = ScreenPixelsHeight*/
const Value V21494_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21494_4 = {S54, 0, &V21494_4_1229, &P21494_5};	/*Property = ScreenPixelsWidth*/
const Profile P21494 = {"21494", &P21494_4};	/*Profile ID - 21494*/
const Value V21742_1_1 = {S2, NULL};	/*Value = True*/
const Property P21742_1 = {S50, 0, &V21742_1_1, NULL};	/*Property = IsMobile*/
const Value V21742_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21742_5 = {S52, 0, &V21742_5_1277, &P21742_1};	/*Property = ScreenPixelsHeight*/
const Value V21742_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21742_4 = {S54, 0, &V21742_4_1279, &P21742_5};	/*Property = ScreenPixelsWidth*/
const Profile P21742 = {"21742", &P21742_4};	/*Profile ID - 21742*/
const Value V20710_1_1 = {S2, NULL};	/*Value = True*/
const Property P20710_1 = {S50, 0, &V20710_1_1, NULL};	/*Property = IsMobile*/
const Value V20710_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20710_5 = {S52, 0, &V20710_5_9981, &P20710_1};	/*Property = ScreenPixelsHeight*/
const Value V20710_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20710_4 = {S54, 0, &V20710_4_2336, &P20710_5};	/*Property = ScreenPixelsWidth*/
const Profile P20710 = {"20710", &P20710_4};	/*Profile ID - 20710*/
const Value V21385_1_1 = {S2, NULL};	/*Value = True*/
const Property P21385_1 = {S50, 0, &V21385_1_1, NULL};	/*Property = IsMobile*/
const Value V21385_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P21385_5 = {S52, 0, &V21385_5_59138, &P21385_1};	/*Property = ScreenPixelsHeight*/
const Value V21385_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P21385_4 = {S54, 0, &V21385_4_59970, &P21385_5};	/*Property = ScreenPixelsWidth*/
const Profile P21385 = {"21385", &P21385_4};	/*Profile ID - 21385*/
const Value V24554_1_1 = {S2, NULL};	/*Value = True*/
const Property P24554_1 = {S50, 0, &V24554_1_1, NULL};	/*Property = IsMobile*/
const Value V24554_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P24554_5 = {S52, 0, &V24554_5_1458, &P24554_1};	/*Property = ScreenPixelsHeight*/
const Value V24554_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P24554_4 = {S54, 0, &V24554_4_1460, &P24554_5};	/*Property = ScreenPixelsWidth*/
const Profile P24554 = {"24554", &P24554_4};	/*Profile ID - 24554*/
const Value V23322_1_1 = {S2, NULL};	/*Value = True*/
const Property P23322_1 = {S50, 0, &V23322_1_1, NULL};	/*Property = IsMobile*/
const Value V23322_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23322_5 = {S52, 0, &V23322_5_1227, &P23322_1};	/*Property = ScreenPixelsHeight*/
const Value V23322_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P23322_4 = {S54, 0, &V23322_4_1477, &P23322_5};	/*Property = ScreenPixelsWidth*/
const Profile P23322 = {"23322", &P23322_4};	/*Profile ID - 23322*/
const Value V21838_1_1 = {S2, NULL};	/*Value = True*/
const Property P21838_1 = {S50, 0, &V21838_1_1, NULL};	/*Property = IsMobile*/
const Value V21838_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21838_5 = {S52, 0, &V21838_5_1458, &P21838_1};	/*Property = ScreenPixelsHeight*/
const Value V21838_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21838_4 = {S54, 0, &V21838_4_37617, &P21838_5};	/*Property = ScreenPixelsWidth*/
const Profile P21838 = {"21838", &P21838_4};	/*Profile ID - 21838*/
const Value V22957_1_1 = {S2, NULL};	/*Value = True*/
const Property P22957_1 = {S50, 0, &V22957_1_1, NULL};	/*Property = IsMobile*/
const Value V22957_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P22957_5 = {S52, 0, &V22957_5_1233, &P22957_1};	/*Property = ScreenPixelsHeight*/
const Value V22957_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P22957_4 = {S54, 0, &V22957_4_1235, &P22957_5};	/*Property = ScreenPixelsWidth*/
const Profile P22957 = {"22957", &P22957_4};	/*Profile ID - 22957*/
const Value V21597_1_1 = {S2, NULL};	/*Value = True*/
const Property P21597_1 = {S50, 0, &V21597_1_1, NULL};	/*Property = IsMobile*/
const Value V21597_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21597_5 = {S52, 0, &V21597_5_1227, &P21597_1};	/*Property = ScreenPixelsHeight*/
const Value V21597_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21597_4 = {S54, 0, &V21597_4_1229, &P21597_5};	/*Property = ScreenPixelsWidth*/
const Profile P21597 = {"21597", &P21597_4};	/*Profile ID - 21597*/
const Value V23866_1_1 = {S2, NULL};	/*Value = True*/
const Property P23866_1 = {S50, 0, &V23866_1_1, NULL};	/*Property = IsMobile*/
const Value V23866_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23866_5 = {S52, 0, &V23866_5_1227, &P23866_1};	/*Property = ScreenPixelsHeight*/
const Value V23866_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23866_4 = {S54, 0, &V23866_4_1229, &P23866_5};	/*Property = ScreenPixelsWidth*/
const Profile P23866 = {"23866", &P23866_4};	/*Profile ID - 23866*/
const Value V21481_1_1 = {S2, NULL};	/*Value = True*/
const Property P21481_1 = {S50, 0, &V21481_1_1, NULL};	/*Property = IsMobile*/
const Value V21481_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P21481_5 = {S52, 0, &V21481_5_59138, &P21481_1};	/*Property = ScreenPixelsHeight*/
const Value V21481_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P21481_4 = {S54, 0, &V21481_4_59970, &P21481_5};	/*Property = ScreenPixelsWidth*/
const Profile P21481 = {"21481", &P21481_4};	/*Profile ID - 21481*/
const Value V21519_1_1 = {S2, NULL};	/*Value = True*/
const Property P21519_1 = {S50, 0, &V21519_1_1, NULL};	/*Property = IsMobile*/
const Value V21519_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21519_5 = {S52, 0, &V21519_5_1277, &P21519_1};	/*Property = ScreenPixelsHeight*/
const Value V21519_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21519_4 = {S54, 0, &V21519_4_1279, &P21519_5};	/*Property = ScreenPixelsWidth*/
const Profile P21519 = {"21519", &P21519_4};	/*Profile ID - 21519*/
const Value V24552_1_1 = {S2, NULL};	/*Value = True*/
const Property P24552_1 = {S50, 0, &V24552_1_1, NULL};	/*Property = IsMobile*/
const Value V24552_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P24552_5 = {S52, 0, &V24552_5_37569, &P24552_1};	/*Property = ScreenPixelsHeight*/
const Value V24552_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P24552_4 = {S54, 0, &V24552_4_5645, &P24552_5};	/*Property = ScreenPixelsWidth*/
const Profile P24552 = {"24552", &P24552_4};	/*Profile ID - 24552*/
const Value V21365_1_1 = {S2, NULL};	/*Value = True*/
const Property P21365_1 = {S50, 0, &V21365_1_1, NULL};	/*Property = IsMobile*/
const Value V21365_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21365_5 = {S52, 0, &V21365_5_9981, &P21365_1};	/*Property = ScreenPixelsHeight*/
const Value V21365_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21365_4 = {S54, 0, &V21365_4_2336, &P21365_5};	/*Property = ScreenPixelsWidth*/
const Profile P21365 = {"21365", &P21365_4};	/*Profile ID - 21365*/
const Value V21599_1_1 = {S2, NULL};	/*Value = True*/
const Property P21599_1 = {S50, 0, &V21599_1_1, NULL};	/*Property = IsMobile*/
const Value V21599_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21599_5 = {S52, 0, &V21599_5_1277, &P21599_1};	/*Property = ScreenPixelsHeight*/
const Value V21599_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21599_4 = {S54, 0, &V21599_4_1279, &P21599_5};	/*Property = ScreenPixelsWidth*/
const Profile P21599 = {"21599", &P21599_4};	/*Profile ID - 21599*/
const Value V20690_1_1 = {S2, NULL};	/*Value = True*/
const Property P20690_1 = {S50, 0, &V20690_1_1, NULL};	/*Property = IsMobile*/
const Value V20690_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20690_5 = {S52, 0, &V20690_5_1403, &P20690_1};	/*Property = ScreenPixelsHeight*/
const Value V20690_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20690_4 = {S54, 0, &V20690_4_1404, &P20690_5};	/*Property = ScreenPixelsWidth*/
const Profile P20690 = {"20690", &P20690_4};	/*Profile ID - 20690*/
const Value V23837_1_1 = {S2, NULL};	/*Value = True*/
const Property P23837_1 = {S50, 0, &V23837_1_1, NULL};	/*Property = IsMobile*/
const Value V23837_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23837_5 = {S52, 0, &V23837_5_1227, &P23837_1};	/*Property = ScreenPixelsHeight*/
const Value V23837_4_1943 = {S53, NULL};	/*Value = 400*/
const Property P23837_4 = {S54, 0, &V23837_4_1943, &P23837_5};	/*Property = ScreenPixelsWidth*/
const Profile P23837 = {"23837", &P23837_4};	/*Profile ID - 23837*/
const Value V20538_1_1 = {S2, NULL};	/*Value = True*/
const Property P20538_1 = {S50, 0, &V20538_1_1, NULL};	/*Property = IsMobile*/
const Value V20538_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20538_5 = {S52, 0, &V20538_5_1227, &P20538_1};	/*Property = ScreenPixelsHeight*/
const Value V20538_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20538_4 = {S54, 0, &V20538_4_1229, &P20538_5};	/*Property = ScreenPixelsWidth*/
const Profile P20538 = {"20538", &P20538_4};	/*Profile ID - 20538*/
const Value V23137_1_1 = {S2, NULL};	/*Value = True*/
const Property P23137_1 = {S50, 0, &V23137_1_1, NULL};	/*Property = IsMobile*/
const Value V23137_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23137_5 = {S52, 0, &V23137_5_1227, &P23137_1};	/*Property = ScreenPixelsHeight*/
const Value V23137_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23137_4 = {S54, 0, &V23137_4_1229, &P23137_5};	/*Property = ScreenPixelsWidth*/
const Profile P23137 = {"23137", &P23137_4};	/*Profile ID - 23137*/
const Value V21495_1_1 = {S2, NULL};	/*Value = True*/
const Property P21495_1 = {S50, 0, &V21495_1_1, NULL};	/*Property = IsMobile*/
const Value V21495_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21495_5 = {S52, 0, &V21495_5_9981, &P21495_1};	/*Property = ScreenPixelsHeight*/
const Value V21495_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21495_4 = {S54, 0, &V21495_4_39098, &P21495_5};	/*Property = ScreenPixelsWidth*/
const Profile P21495 = {"21495", &P21495_4};	/*Profile ID - 21495*/
const Value V21655_1_1 = {S2, NULL};	/*Value = True*/
const Property P21655_1 = {S50, 0, &V21655_1_1, NULL};	/*Property = IsMobile*/
const Value V21655_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21655_5 = {S52, 0, &V21655_5_1458, &P21655_1};	/*Property = ScreenPixelsHeight*/
const Value V21655_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21655_4 = {S54, 0, &V21655_4_1460, &P21655_5};	/*Property = ScreenPixelsWidth*/
const Profile P21655 = {"21655", &P21655_4};	/*Profile ID - 21655*/
const Value V21431_1_1 = {S2, NULL};	/*Value = True*/
const Property P21431_1 = {S50, 0, &V21431_1_1, NULL};	/*Property = IsMobile*/
const Value V21431_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21431_5 = {S52, 0, &V21431_5_1269, &P21431_1};	/*Property = ScreenPixelsHeight*/
const Value V21431_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21431_4 = {S54, 0, &V21431_4_1229, &P21431_5};	/*Property = ScreenPixelsWidth*/
const Profile P21431 = {"21431", &P21431_4};	/*Profile ID - 21431*/
const Value V21745_1_1 = {S2, NULL};	/*Value = True*/
const Property P21745_1 = {S50, 0, &V21745_1_1, NULL};	/*Property = IsMobile*/
const Value V21745_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21745_5 = {S52, 0, &V21745_5_1222, &P21745_1};	/*Property = ScreenPixelsHeight*/
const Value V21745_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21745_4 = {S54, 0, &V21745_4_1224, &P21745_5};	/*Property = ScreenPixelsWidth*/
const Profile P21745 = {"21745", &P21745_4};	/*Profile ID - 21745*/
const Value V23063_1_1 = {S2, NULL};	/*Value = True*/
const Property P23063_1 = {S50, 0, &V23063_1_1, NULL};	/*Property = IsMobile*/
const Value V23063_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23063_5 = {S52, 0, &V23063_5_1277, &P23063_1};	/*Property = ScreenPixelsHeight*/
const Value V23063_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23063_4 = {S54, 0, &V23063_4_1279, &P23063_5};	/*Property = ScreenPixelsWidth*/
const Profile P23063 = {"23063", &P23063_4};	/*Profile ID - 23063*/
const Value V21636_1_1 = {S2, NULL};	/*Value = True*/
const Property P21636_1 = {S50, 0, &V21636_1_1, NULL};	/*Property = IsMobile*/
const Value V21636_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21636_5 = {S52, 0, &V21636_5_1227, &P21636_1};	/*Property = ScreenPixelsHeight*/
const Value V21636_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21636_4 = {S54, 0, &V21636_4_1229, &P21636_5};	/*Property = ScreenPixelsWidth*/
const Profile P21636 = {"21636", &P21636_4};	/*Profile ID - 21636*/
const Value V23950_1_1 = {S2, NULL};	/*Value = True*/
const Property P23950_1 = {S50, 0, &V23950_1_1, NULL};	/*Property = IsMobile*/
const Value V23950_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23950_5 = {S52, 0, &V23950_5_1227, &P23950_1};	/*Property = ScreenPixelsHeight*/
const Value V23950_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23950_4 = {S54, 0, &V23950_4_1229, &P23950_5};	/*Property = ScreenPixelsWidth*/
const Profile P23950 = {"23950", &P23950_4};	/*Profile ID - 23950*/
const Value V22149_1_1 = {S2, NULL};	/*Value = True*/
const Property P22149_1 = {S50, 0, &V22149_1_1, NULL};	/*Property = IsMobile*/
const Value V22149_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22149_5 = {S52, 0, &V22149_5_1227, &P22149_1};	/*Property = ScreenPixelsHeight*/
const Value V22149_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22149_4 = {S54, 0, &V22149_4_1229, &P22149_5};	/*Property = ScreenPixelsWidth*/
const Profile P22149 = {"22149", &P22149_4};	/*Profile ID - 22149*/
const Value V23878_1_1 = {S2, NULL};	/*Value = True*/
const Property P23878_1 = {S50, 0, &V23878_1_1, NULL};	/*Property = IsMobile*/
const Value V23878_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23878_5 = {S52, 0, &V23878_5_1277, &P23878_1};	/*Property = ScreenPixelsHeight*/
const Value V23878_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23878_4 = {S54, 0, &V23878_4_1279, &P23878_5};	/*Property = ScreenPixelsWidth*/
const Profile P23878 = {"23878", &P23878_4};	/*Profile ID - 23878*/
const Value V23806_1_1 = {S2, NULL};	/*Value = True*/
const Property P23806_1 = {S50, 0, &V23806_1_1, NULL};	/*Property = IsMobile*/
const Value V23806_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23806_5 = {S52, 0, &V23806_5_1277, &P23806_1};	/*Property = ScreenPixelsHeight*/
const Value V23806_4_37618 = {S83, NULL};	/*Value = 272*/
const Property P23806_4 = {S54, 0, &V23806_4_37618, &P23806_5};	/*Property = ScreenPixelsWidth*/
const Profile P23806 = {"23806", &P23806_4};	/*Profile ID - 23806*/
const Value V23856_1_1 = {S2, NULL};	/*Value = True*/
const Property P23856_1 = {S50, 0, &V23856_1_1, NULL};	/*Property = IsMobile*/
const Value V23856_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23856_5 = {S52, 0, &V23856_5_1227, &P23856_1};	/*Property = ScreenPixelsHeight*/
const Value V23856_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23856_4 = {S54, 0, &V23856_4_1229, &P23856_5};	/*Property = ScreenPixelsWidth*/
const Profile P23856 = {"23856", &P23856_4};	/*Profile ID - 23856*/
const Value V22095_1_1 = {S2, NULL};	/*Value = True*/
const Property P22095_1 = {S50, 0, &V22095_1_1, NULL};	/*Property = IsMobile*/
const Value V22095_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22095_5 = {S52, 0, &V22095_5_9981, &P22095_1};	/*Property = ScreenPixelsHeight*/
const Value V22095_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22095_4 = {S54, 0, &V22095_4_39098, &P22095_5};	/*Property = ScreenPixelsWidth*/
const Profile P22095 = {"22095", &P22095_4};	/*Profile ID - 22095*/
const Value V22001_1_1 = {S2, NULL};	/*Value = True*/
const Property P22001_1 = {S50, 0, &V22001_1_1, NULL};	/*Property = IsMobile*/
const Value V22001_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22001_5 = {S52, 0, &V22001_5_1277, &P22001_1};	/*Property = ScreenPixelsHeight*/
const Value V22001_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22001_4 = {S54, 0, &V22001_4_1279, &P22001_5};	/*Property = ScreenPixelsWidth*/
const Profile P22001 = {"22001", &P22001_4};	/*Profile ID - 22001*/
const Value V21490_1_1 = {S2, NULL};	/*Value = True*/
const Property P21490_1 = {S50, 0, &V21490_1_1, NULL};	/*Property = IsMobile*/
const Value V21490_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P21490_5 = {S52, 0, &V21490_5_59138, &P21490_1};	/*Property = ScreenPixelsHeight*/
const Value V21490_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P21490_4 = {S54, 0, &V21490_4_59970, &P21490_5};	/*Property = ScreenPixelsWidth*/
const Profile P21490 = {"21490", &P21490_4};	/*Profile ID - 21490*/
const Value V22998_1_1 = {S2, NULL};	/*Value = True*/
const Property P22998_1 = {S50, 0, &V22998_1_1, NULL};	/*Property = IsMobile*/
const Value V22998_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22998_5 = {S52, 0, &V22998_5_1227, &P22998_1};	/*Property = ScreenPixelsHeight*/
const Value V22998_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22998_4 = {S54, 0, &V22998_4_1229, &P22998_5};	/*Property = ScreenPixelsWidth*/
const Profile P22998 = {"22998", &P22998_4};	/*Profile ID - 22998*/
const Value V23821_1_1 = {S2, NULL};	/*Value = True*/
const Property P23821_1 = {S50, 0, &V23821_1_1, NULL};	/*Property = IsMobile*/
const Value V23821_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23821_5 = {S52, 0, &V23821_5_9981, &P23821_1};	/*Property = ScreenPixelsHeight*/
const Value V23821_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23821_4 = {S54, 0, &V23821_4_2336, &P23821_5};	/*Property = ScreenPixelsWidth*/
const Profile P23821 = {"23821", &P23821_4};	/*Profile ID - 23821*/
const Value V23511_1_1 = {S2, NULL};	/*Value = True*/
const Property P23511_1 = {S50, 0, &V23511_1_1, NULL};	/*Property = IsMobile*/
const Value V23511_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23511_5 = {S52, 0, &V23511_5_1227, &P23511_1};	/*Property = ScreenPixelsHeight*/
const Value V23511_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23511_4 = {S54, 0, &V23511_4_1229, &P23511_5};	/*Property = ScreenPixelsWidth*/
const Profile P23511 = {"23511", &P23511_4};	/*Profile ID - 23511*/
const Value V23340_1_1 = {S2, NULL};	/*Value = True*/
const Property P23340_1 = {S50, 0, &V23340_1_1, NULL};	/*Property = IsMobile*/
const Value V23340_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23340_5 = {S52, 0, &V23340_5_37569, &P23340_1};	/*Property = ScreenPixelsHeight*/
const Value V23340_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23340_4 = {S54, 0, &V23340_4_5645, &P23340_5};	/*Property = ScreenPixelsWidth*/
const Profile P23340 = {"23340", &P23340_4};	/*Profile ID - 23340*/
const Profile P21691 = {"21691", NULL};	/*Profile ID - 21691*/
const Value V22690_1_1 = {S2, NULL};	/*Value = True*/
const Property P22690_1 = {S50, 0, &V22690_1_1, NULL};	/*Property = IsMobile*/
const Value V22690_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22690_5 = {S52, 0, &V22690_5_1227, &P22690_1};	/*Property = ScreenPixelsHeight*/
const Value V22690_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22690_4 = {S54, 0, &V22690_4_1229, &P22690_5};	/*Property = ScreenPixelsWidth*/
const Profile P22690 = {"22690", &P22690_4};	/*Profile ID - 22690*/
const Value V18737_1_1 = {S2, NULL};	/*Value = True*/
const Property P18737_1 = {S50, 0, &V18737_1_1, NULL};	/*Property = IsMobile*/
const Value V18737_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P18737_5 = {S52, 0, &V18737_5_59138, &P18737_1};	/*Property = ScreenPixelsHeight*/
const Value V18737_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P18737_4 = {S54, 0, &V18737_4_59970, &P18737_5};	/*Property = ScreenPixelsWidth*/
const Profile P18737 = {"18737", &P18737_4};	/*Profile ID - 18737*/
const Value V23961_1_1 = {S2, NULL};	/*Value = True*/
const Property P23961_1 = {S50, 0, &V23961_1_1, NULL};	/*Property = IsMobile*/
const Value V23961_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23961_5 = {S52, 0, &V23961_5_1227, &P23961_1};	/*Property = ScreenPixelsHeight*/
const Value V23961_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23961_4 = {S54, 0, &V23961_4_1229, &P23961_5};	/*Property = ScreenPixelsWidth*/
const Profile P23961 = {"23961", &P23961_4};	/*Profile ID - 23961*/
const Value V24524_1_1 = {S2, NULL};	/*Value = True*/
const Property P24524_1 = {S50, 0, &V24524_1_1, NULL};	/*Property = IsMobile*/
const Value V24524_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P24524_5 = {S52, 0, &V24524_5_1458, &P24524_1};	/*Property = ScreenPixelsHeight*/
const Value V24524_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P24524_4 = {S54, 0, &V24524_4_37617, &P24524_5};	/*Property = ScreenPixelsWidth*/
const Profile P24524 = {"24524", &P24524_4};	/*Profile ID - 24524*/
const Value V23512_1_1 = {S2, NULL};	/*Value = True*/
const Property P23512_1 = {S50, 0, &V23512_1_1, NULL};	/*Property = IsMobile*/
const Value V23512_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23512_5 = {S52, 0, &V23512_5_1227, &P23512_1};	/*Property = ScreenPixelsHeight*/
const Value V23512_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23512_4 = {S54, 0, &V23512_4_1229, &P23512_5};	/*Property = ScreenPixelsWidth*/
const Profile P23512 = {"23512", &P23512_4};	/*Profile ID - 23512*/
const Value V21475_1_1 = {S2, NULL};	/*Value = True*/
const Property P21475_1 = {S50, 0, &V21475_1_1, NULL};	/*Property = IsMobile*/
const Value V21475_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21475_5 = {S52, 0, &V21475_5_1227, &P21475_1};	/*Property = ScreenPixelsHeight*/
const Value V21475_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21475_4 = {S54, 0, &V21475_4_1229, &P21475_5};	/*Property = ScreenPixelsWidth*/
const Profile P21475 = {"21475", &P21475_4};	/*Profile ID - 21475*/
const Value V23232_1_1 = {S2, NULL};	/*Value = True*/
const Property P23232_1 = {S50, 0, &V23232_1_1, NULL};	/*Property = IsMobile*/
const Value V23232_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23232_5 = {S52, 0, &V23232_5_1832, &P23232_1};	/*Property = ScreenPixelsHeight*/
const Value V23232_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23232_4 = {S54, 0, &V23232_4_5645, &P23232_5};	/*Property = ScreenPixelsWidth*/
const Profile P23232 = {"23232", &P23232_4};	/*Profile ID - 23232*/
const Value V21740_1_1 = {S2, NULL};	/*Value = True*/
const Property P21740_1 = {S50, 0, &V21740_1_1, NULL};	/*Property = IsMobile*/
const Value V21740_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21740_5 = {S52, 0, &V21740_5_1227, &P21740_1};	/*Property = ScreenPixelsHeight*/
const Value V21740_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21740_4 = {S54, 0, &V21740_4_1229, &P21740_5};	/*Property = ScreenPixelsWidth*/
const Profile P21740 = {"21740", &P21740_4};	/*Profile ID - 21740*/
const Value V23208_1_1 = {S2, NULL};	/*Value = True*/
const Property P23208_1 = {S50, 0, &V23208_1_1, NULL};	/*Property = IsMobile*/
const Value V23208_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23208_5 = {S52, 0, &V23208_5_1227, &P23208_1};	/*Property = ScreenPixelsHeight*/
const Value V23208_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23208_4 = {S54, 0, &V23208_4_1229, &P23208_5};	/*Property = ScreenPixelsWidth*/
const Profile P23208 = {"23208", &P23208_4};	/*Profile ID - 23208*/
const Value V21973_1_1 = {S2, NULL};	/*Value = True*/
const Property P21973_1 = {S50, 0, &V21973_1_1, NULL};	/*Property = IsMobile*/
const Value V21973_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21973_5 = {S52, 0, &V21973_5_1227, &P21973_1};	/*Property = ScreenPixelsHeight*/
const Value V21973_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21973_4 = {S54, 0, &V21973_4_1229, &P21973_5};	/*Property = ScreenPixelsWidth*/
const Profile P21973 = {"21973", &P21973_4};	/*Profile ID - 21973*/
const Value V21935_1_1 = {S2, NULL};	/*Value = True*/
const Property P21935_1 = {S50, 0, &V21935_1_1, NULL};	/*Property = IsMobile*/
const Value V21935_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P21935_5 = {S52, 0, &V21935_5_59138, &P21935_1};	/*Property = ScreenPixelsHeight*/
const Value V21935_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P21935_4 = {S54, 0, &V21935_4_59970, &P21935_5};	/*Property = ScreenPixelsWidth*/
const Profile P21935 = {"21935", &P21935_4};	/*Profile ID - 21935*/
const Value V23449_1_1 = {S2, NULL};	/*Value = True*/
const Property P23449_1 = {S50, 0, &V23449_1_1, NULL};	/*Property = IsMobile*/
const Value V23449_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23449_5 = {S52, 0, &V23449_5_1227, &P23449_1};	/*Property = ScreenPixelsHeight*/
const Value V23449_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23449_4 = {S54, 0, &V23449_4_1229, &P23449_5};	/*Property = ScreenPixelsWidth*/
const Profile P23449 = {"23449", &P23449_4};	/*Profile ID - 23449*/
const Value V22617_1_1 = {S2, NULL};	/*Value = True*/
const Property P22617_1 = {S50, 0, &V22617_1_1, NULL};	/*Property = IsMobile*/
const Value V22617_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22617_5 = {S52, 0, &V22617_5_1222, &P22617_1};	/*Property = ScreenPixelsHeight*/
const Value V22617_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22617_4 = {S54, 0, &V22617_4_1224, &P22617_5};	/*Property = ScreenPixelsWidth*/
const Profile P22617 = {"22617", &P22617_4};	/*Profile ID - 22617*/
const Value V22618_1_1 = {S2, NULL};	/*Value = True*/
const Property P22618_1 = {S50, 0, &V22618_1_1, NULL};	/*Property = IsMobile*/
const Value V22618_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22618_5 = {S52, 0, &V22618_5_1222, &P22618_1};	/*Property = ScreenPixelsHeight*/
const Value V22618_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22618_4 = {S54, 0, &V22618_4_1224, &P22618_5};	/*Property = ScreenPixelsWidth*/
const Profile P22618 = {"22618", &P22618_4};	/*Profile ID - 22618*/
const Value V23969_1_1 = {S2, NULL};	/*Value = True*/
const Property P23969_1 = {S50, 0, &V23969_1_1, NULL};	/*Property = IsMobile*/
const Value V23969_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23969_5 = {S52, 0, &V23969_5_1227, &P23969_1};	/*Property = ScreenPixelsHeight*/
const Value V23969_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23969_4 = {S54, 0, &V23969_4_1229, &P23969_5};	/*Property = ScreenPixelsWidth*/
const Profile P23969 = {"23969", &P23969_4};	/*Profile ID - 23969*/
const Value V20920_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_279 = {S1, 0, &V20920_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V20920_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_282 = {S3, 0, &V20920_282_57155, &P20920_279};	/*Property = BlobBuilder*/
const Value V20920_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_350 = {S4, 0, &V20920_350_57446, &P20920_282};	/*Property = Canvas*/
const Value V20920_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_283 = {S5, 0, &V20920_283_57161, &P20920_350};	/*Property = CssBackground*/
const Value V20920_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_284 = {S6, 0, &V20920_284_57164, &P20920_283};	/*Property = CssBorderImage*/
const Value V20920_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_285 = {S7, 0, &V20920_285_57167, &P20920_284};	/*Property = CssCanvas*/
const Value V20920_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_286 = {S8, 0, &V20920_286_57170, &P20920_285};	/*Property = CssColor*/
const Value V20920_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_237 = {S9, 0, &V20920_237_55786, &P20920_286};	/*Property = CssColumn*/
const Value V20920_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_287 = {S10, 0, &V20920_287_57173, &P20920_237};	/*Property = CssFlexbox*/
const Value V20920_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_288 = {S11, 0, &V20920_288_57176, &P20920_287};	/*Property = CssFont*/
const Value V20920_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_289 = {S12, 0, &V20920_289_86899, &P20920_288};	/*Property = CssImages*/
const Value V20920_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_291 = {S13, 0, &V20920_291_57182, &P20920_289};	/*Property = CssMediaQueries*/
const Value V20920_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_292 = {S14, 0, &V20920_292_57185, &P20920_291};	/*Property = CssMinMax*/
const Value V20920_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_293 = {S15, 0, &V20920_293_57188, &P20920_292};	/*Property = CssOverflow*/
const Value V20920_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_294 = {S16, 0, &V20920_294_57191, &P20920_293};	/*Property = CssPosition*/
const Value V20920_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_295 = {S17, 0, &V20920_295_57194, &P20920_294};	/*Property = CssText*/
const Value V20920_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_297 = {S18, 0, &V20920_297_57197, &P20920_295};	/*Property = CssTransforms*/
const Value V20920_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_241 = {S19, 0, &V20920_241_55801, &P20920_297};	/*Property = CssTransitions*/
const Value V20920_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_298 = {S20, 0, &V20920_298_57208, &P20920_241};	/*Property = CssUI*/
const Value V20920_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_300 = {S21, 0, &V20920_300_57217, &P20920_298};	/*Property = DataSet*/
const Value V20920_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_301 = {S22, 0, &V20920_301_57220, &P20920_300};	/*Property = DataUrl*/
const Value V20920_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_302 = {S23, 0, &V20920_302_57223, &P20920_301};	/*Property = DeviceOrientation*/
const Value V20920_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_304 = {S24, 0, &V20920_304_57229, &P20920_302};	/*Property = FileReader*/
const Value V20920_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_305 = {S25, 0, &V20920_305_57232, &P20920_304};	/*Property = FileSaver*/
const Value V20920_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_306 = {S26, 0, &V20920_306_57238, &P20920_305};	/*Property = FileWriter*/
const Value V20920_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_307 = {S27, 0, &V20920_307_57241, &P20920_306};	/*Property = FormData*/
const Value V20920_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_309 = {S28, 0, &V20920_309_57247, &P20920_307};	/*Property = Fullscreen*/
const Value V20920_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_310 = {S29, 0, &V20920_310_57250, &P20920_309};	/*Property = GeoLocation*/
const Value V20920_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_312 = {S30, 0, &V20920_312_57256, &P20920_310};	/*Property = History*/
const Value V20920_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_314 = {S31, 0, &V20920_314_57262, &P20920_312};	/*Property = Html5*/
const Value V20920_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_313 = {S32, 0, &V20920_313_57259, &P20920_314};	/*Property = Html-Media-Capture*/
const Value V20920_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_315 = {S33, 0, &V20920_315_57265, &P20920_313};	/*Property = Iframe*/
const Value V20920_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_316 = {S34, 0, &V20920_316_57268, &P20920_315};	/*Property = IndexedDB*/
const Value V20920_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_317 = {S35, 0, &V20920_317_57271, &P20920_316};	/*Property = Json*/
const Value V20920_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P20920_15 = {S37, 0, &V20920_15_31, &P20920_317};	/*Property = LayoutEngine*/
const Value V20920_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_344 = {S38, 0, &V20920_344_57274, &P20920_15};	/*Property = Masking*/
const Value V20920_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_320 = {S39, 0, &V20920_320_57292, &P20920_344};	/*Property = PostMessage*/
const Value V20920_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_321 = {S40, 0, &V20920_321_57295, &P20920_320};	/*Property = Progress*/
const Value V20920_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_322 = {S41, 0, &V20920_322_57298, &P20920_321};	/*Property = Prompts*/
const Value V20920_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_323 = {S42, 0, &V20920_323_57301, &P20920_322};	/*Property = Selector*/
const Value V20920_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_324 = {S43, 0, &V20920_324_57307, &P20920_323};	/*Property = Svg*/
const Value V20920_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_327 = {S44, 0, &V20920_327_57316, &P20920_324};	/*Property = TouchEvents*/
const Value V20920_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_328 = {S45, 0, &V20920_328_57319, &P20920_327};	/*Property = Track*/
const Value V20920_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_329 = {S46, 0, &V20920_329_57325, &P20920_328};	/*Property = Video*/
const Value V20920_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_330 = {S47, 0, &V20920_330_57328, &P20920_329};	/*Property = Viewport*/
const Value V20920_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_332 = {S48, 0, &V20920_332_57340, &P20920_330};	/*Property = WebWorkers*/
const Value V20920_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P20920_333 = {S49, 0, &V20920_333_57343, &P20920_332};	/*Property = Xhr2*/
const Profile P20920 = {"20920", &P20920_333};	/*Profile ID - 20920*/
const Value V22168_1_1 = {S2, NULL};	/*Value = True*/
const Property P22168_1 = {S50, 0, &V22168_1_1, NULL};	/*Property = IsMobile*/
const Value V22168_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22168_5 = {S52, 0, &V22168_5_1227, &P22168_1};	/*Property = ScreenPixelsHeight*/
const Value V22168_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P22168_4 = {S54, 0, &V22168_4_1477, &P22168_5};	/*Property = ScreenPixelsWidth*/
const Profile P22168 = {"22168", &P22168_4};	/*Profile ID - 22168*/
const Value V21405_1_1 = {S2, NULL};	/*Value = True*/
const Property P21405_1 = {S50, 0, &V21405_1_1, NULL};	/*Property = IsMobile*/
const Value V21405_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21405_5 = {S52, 0, &V21405_5_1277, &P21405_1};	/*Property = ScreenPixelsHeight*/
const Value V21405_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21405_4 = {S54, 0, &V21405_4_1279, &P21405_5};	/*Property = ScreenPixelsWidth*/
const Profile P21405 = {"21405", &P21405_4};	/*Profile ID - 21405*/
const Value V23300_1_1 = {S2, NULL};	/*Value = True*/
const Property P23300_1 = {S50, 0, &V23300_1_1, NULL};	/*Property = IsMobile*/
const Value V23300_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23300_5 = {S52, 0, &V23300_5_37569, &P23300_1};	/*Property = ScreenPixelsHeight*/
const Value V23300_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23300_4 = {S54, 0, &V23300_4_5645, &P23300_5};	/*Property = ScreenPixelsWidth*/
const Profile P23300 = {"23300", &P23300_4};	/*Profile ID - 23300*/
const Value V21488_1_1 = {S2, NULL};	/*Value = True*/
const Property P21488_1 = {S50, 0, &V21488_1_1, NULL};	/*Property = IsMobile*/
const Value V21488_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P21488_5 = {S52, 0, &V21488_5_59138, &P21488_1};	/*Property = ScreenPixelsHeight*/
const Value V21488_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P21488_4 = {S54, 0, &V21488_4_59970, &P21488_5};	/*Property = ScreenPixelsWidth*/
const Profile P21488 = {"21488", &P21488_4};	/*Profile ID - 21488*/
const Value V22616_1_1 = {S2, NULL};	/*Value = True*/
const Property P22616_1 = {S50, 0, &V22616_1_1, NULL};	/*Property = IsMobile*/
const Value V22616_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22616_5 = {S52, 0, &V22616_5_1222, &P22616_1};	/*Property = ScreenPixelsHeight*/
const Value V22616_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22616_4 = {S54, 0, &V22616_4_1224, &P22616_5};	/*Property = ScreenPixelsWidth*/
const Profile P22616 = {"22616", &P22616_4};	/*Profile ID - 22616*/
const Value V23035_1_1 = {S2, NULL};	/*Value = True*/
const Property P23035_1 = {S50, 0, &V23035_1_1, NULL};	/*Property = IsMobile*/
const Value V23035_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23035_5 = {S52, 0, &V23035_5_1277, &P23035_1};	/*Property = ScreenPixelsHeight*/
const Value V23035_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23035_4 = {S54, 0, &V23035_4_1279, &P23035_5};	/*Property = ScreenPixelsWidth*/
const Profile P23035 = {"23035", &P23035_4};	/*Profile ID - 23035*/
const Value V23875_1_1 = {S2, NULL};	/*Value = True*/
const Property P23875_1 = {S50, 0, &V23875_1_1, NULL};	/*Property = IsMobile*/
const Value V23875_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23875_5 = {S52, 0, &V23875_5_1227, &P23875_1};	/*Property = ScreenPixelsHeight*/
const Value V23875_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23875_4 = {S54, 0, &V23875_4_1460, &P23875_5};	/*Property = ScreenPixelsWidth*/
const Profile P23875 = {"23875", &P23875_4};	/*Profile ID - 23875*/
const Value V23451_1_1 = {S2, NULL};	/*Value = True*/
const Property P23451_1 = {S50, 0, &V23451_1_1, NULL};	/*Property = IsMobile*/
const Value V23451_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23451_5 = {S52, 0, &V23451_5_1227, &P23451_1};	/*Property = ScreenPixelsHeight*/
const Value V23451_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P23451_4 = {S54, 0, &V23451_4_1477, &P23451_5};	/*Property = ScreenPixelsWidth*/
const Profile P23451 = {"23451", &P23451_4};	/*Profile ID - 23451*/
const Value V21965_1_1 = {S2, NULL};	/*Value = True*/
const Property P21965_1 = {S50, 0, &V21965_1_1, NULL};	/*Property = IsMobile*/
const Value V21965_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21965_5 = {S52, 0, &V21965_5_1269, &P21965_1};	/*Property = ScreenPixelsHeight*/
const Value V21965_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21965_4 = {S54, 0, &V21965_4_1229, &P21965_5};	/*Property = ScreenPixelsWidth*/
const Profile P21965 = {"21965", &P21965_4};	/*Profile ID - 21965*/
const Value V21983_1_1 = {S2, NULL};	/*Value = True*/
const Property P21983_1 = {S50, 0, &V21983_1_1, NULL};	/*Property = IsMobile*/
const Value V21983_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P21983_5 = {S52, 0, &V21983_5_37569, &P21983_1};	/*Property = ScreenPixelsHeight*/
const Value V21983_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P21983_4 = {S54, 0, &V21983_4_5645, &P21983_5};	/*Property = ScreenPixelsWidth*/
const Profile P21983 = {"21983", &P21983_4};	/*Profile ID - 21983*/
const Value V21823_1_1 = {S2, NULL};	/*Value = True*/
const Property P21823_1 = {S50, 0, &V21823_1_1, NULL};	/*Property = IsMobile*/
const Value V21823_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21823_5 = {S52, 0, &V21823_5_9981, &P21823_1};	/*Property = ScreenPixelsHeight*/
const Value V21823_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21823_4 = {S54, 0, &V21823_4_39098, &P21823_5};	/*Property = ScreenPixelsWidth*/
const Profile P21823 = {"21823", &P21823_4};	/*Profile ID - 21823*/
const Value V22658_1_1 = {S2, NULL};	/*Value = True*/
const Property P22658_1 = {S50, 0, &V22658_1_1, NULL};	/*Property = IsMobile*/
const Value V22658_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22658_5 = {S52, 0, &V22658_5_1227, &P22658_1};	/*Property = ScreenPixelsHeight*/
const Value V22658_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22658_4 = {S54, 0, &V22658_4_1229, &P22658_5};	/*Property = ScreenPixelsWidth*/
const Profile P22658 = {"22658", &P22658_4};	/*Profile ID - 22658*/
const Value V23964_1_1 = {S2, NULL};	/*Value = True*/
const Property P23964_1 = {S50, 0, &V23964_1_1, NULL};	/*Property = IsMobile*/
const Value V23964_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23964_5 = {S52, 0, &V23964_5_37, &P23964_1};	/*Property = ScreenPixelsHeight*/
const Value V23964_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23964_4 = {S54, 0, &V23964_4_36, &P23964_5};	/*Property = ScreenPixelsWidth*/
const Profile P23964 = {"23964", &P23964_4};	/*Profile ID - 23964*/
const Value V21971_1_1 = {S2, NULL};	/*Value = True*/
const Property P21971_1 = {S50, 0, &V21971_1_1, NULL};	/*Property = IsMobile*/
const Value V21971_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21971_5 = {S52, 0, &V21971_5_1277, &P21971_1};	/*Property = ScreenPixelsHeight*/
const Value V21971_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21971_4 = {S54, 0, &V21971_4_1279, &P21971_5};	/*Property = ScreenPixelsWidth*/
const Profile P21971 = {"21971", &P21971_4};	/*Profile ID - 21971*/
const Value V22659_1_1 = {S2, NULL};	/*Value = True*/
const Property P22659_1 = {S50, 0, &V22659_1_1, NULL};	/*Property = IsMobile*/
const Value V22659_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P22659_5 = {S52, 0, &V22659_5_37, &P22659_1};	/*Property = ScreenPixelsHeight*/
const Value V22659_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P22659_4 = {S54, 0, &V22659_4_36, &P22659_5};	/*Property = ScreenPixelsWidth*/
const Profile P22659 = {"22659", &P22659_4};	/*Profile ID - 22659*/
const Value V20542_1_1 = {S2, NULL};	/*Value = True*/
const Property P20542_1 = {S50, 0, &V20542_1_1, NULL};	/*Property = IsMobile*/
const Value V20542_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20542_5 = {S52, 0, &V20542_5_1227, &P20542_1};	/*Property = ScreenPixelsHeight*/
const Value V20542_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20542_4 = {S54, 0, &V20542_4_1229, &P20542_5};	/*Property = ScreenPixelsWidth*/
const Profile P20542 = {"20542", &P20542_4};	/*Profile ID - 20542*/
const Value V23499_1_1 = {S2, NULL};	/*Value = True*/
const Property P23499_1 = {S50, 0, &V23499_1_1, NULL};	/*Property = IsMobile*/
const Value V23499_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23499_5 = {S52, 0, &V23499_5_1227, &P23499_1};	/*Property = ScreenPixelsHeight*/
const Value V23499_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23499_4 = {S54, 0, &V23499_4_1229, &P23499_5};	/*Property = ScreenPixelsWidth*/
const Profile P23499 = {"23499", &P23499_4};	/*Profile ID - 23499*/
const Value V21416_279_57147 = {S2, NULL};	/*Value = True*/
const Property P21416_279 = {S1, 0, &V21416_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V21416_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21416_282 = {S3, 0, &V21416_282_57154, &P21416_279};	/*Property = BlobBuilder*/
const Value V21416_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21416_350 = {S4, 0, &V21416_350_57444, &P21416_282};	/*Property = Canvas*/
const Value V21416_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21416_283 = {S5, 0, &V21416_283_57159, &P21416_350};	/*Property = CssBackground*/
const Value V21416_284_57162 = {S2, NULL};	/*Value = True*/
const Property P21416_284 = {S6, 0, &V21416_284_57162, &P21416_283};	/*Property = CssBorderImage*/
const Value V21416_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21416_285 = {S7, 0, &V21416_285_57166, &P21416_284};	/*Property = CssCanvas*/
const Value V21416_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21416_286 = {S8, 0, &V21416_286_57168, &P21416_285};	/*Property = CssColor*/
const Value V21416_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21416_237 = {S9, 0, &V21416_237_55786, &P21416_286};	/*Property = CssColumn*/
const Value V21416_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21416_287 = {S10, 0, &V21416_287_57172, &P21416_237};	/*Property = CssFlexbox*/
const Value V21416_288_57174 = {S2, NULL};	/*Value = True*/
const Property P21416_288 = {S11, 0, &V21416_288_57174, &P21416_287};	/*Property = CssFont*/
const Value V21416_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21416_289 = {S12, 0, &V21416_289_57759, &P21416_288};	/*Property = CssImages*/
const Value V21416_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21416_291 = {S13, 0, &V21416_291_57180, &P21416_289};	/*Property = CssMediaQueries*/
const Value V21416_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21416_292 = {S14, 0, &V21416_292_57183, &P21416_291};	/*Property = CssMinMax*/
const Value V21416_293_57186 = {S2, NULL};	/*Value = True*/
const Property P21416_293 = {S15, 0, &V21416_293_57186, &P21416_292};	/*Property = CssOverflow*/
const Value V21416_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21416_294 = {S16, 0, &V21416_294_57189, &P21416_293};	/*Property = CssPosition*/
const Value V21416_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21416_295 = {S17, 0, &V21416_295_57192, &P21416_294};	/*Property = CssText*/
const Value V21416_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21416_297 = {S18, 0, &V21416_297_57195, &P21416_295};	/*Property = CssTransforms*/
const Value V21416_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21416_241 = {S19, 0, &V21416_241_55799, &P21416_297};	/*Property = CssTransitions*/
const Value V21416_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21416_298 = {S20, 0, &V21416_298_57206, &P21416_241};	/*Property = CssUI*/
const Value V21416_300_57215 = {S2, NULL};	/*Value = True*/
const Property P21416_300 = {S21, 0, &V21416_300_57215, &P21416_298};	/*Property = DataSet*/
const Value V21416_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21416_301 = {S22, 0, &V21416_301_57218, &P21416_300};	/*Property = DataUrl*/
const Value V21416_302_57221 = {S2, NULL};	/*Value = True*/
const Property P21416_302 = {S23, 0, &V21416_302_57221, &P21416_301};	/*Property = DeviceOrientation*/
const Value V21416_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21416_304 = {S24, 0, &V21416_304_57227, &P21416_302};	/*Property = FileReader*/
const Value V21416_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21416_305 = {S25, 0, &V21416_305_57231, &P21416_304};	/*Property = FileSaver*/
const Value V21416_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21416_306 = {S26, 0, &V21416_306_57237, &P21416_305};	/*Property = FileWriter*/
const Value V21416_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21416_307 = {S27, 0, &V21416_307_57239, &P21416_306};	/*Property = FormData*/
const Value V21416_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21416_309 = {S28, 0, &V21416_309_57246, &P21416_307};	/*Property = Fullscreen*/
const Value V21416_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21416_310 = {S29, 0, &V21416_310_57248, &P21416_309};	/*Property = GeoLocation*/
const Value V21416_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21416_312 = {S30, 0, &V21416_312_57254, &P21416_310};	/*Property = History*/
const Value V21416_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21416_314 = {S31, 0, &V21416_314_57261, &P21416_312};	/*Property = Html5*/
const Value V21416_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21416_313 = {S32, 0, &V21416_313_57258, &P21416_314};	/*Property = Html-Media-Capture*/
const Value V21416_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21416_315 = {S33, 0, &V21416_315_57264, &P21416_313};	/*Property = Iframe*/
const Value V21416_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21416_316 = {S34, 0, &V21416_316_57267, &P21416_315};	/*Property = IndexedDB*/
const Value V21416_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21416_317 = {S35, 0, &V21416_317_57269, &P21416_316};	/*Property = Json*/
const Value V21416_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21416_15 = {S37, 0, &V21416_15_28, &P21416_317};	/*Property = LayoutEngine*/
const Value V21416_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21416_344 = {S38, 0, &V21416_344_57272, &P21416_15};	/*Property = Masking*/
const Value V21416_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21416_320 = {S39, 0, &V21416_320_57290, &P21416_344};	/*Property = PostMessage*/
const Value V21416_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21416_321 = {S40, 0, &V21416_321_57293, &P21416_320};	/*Property = Progress*/
const Value V21416_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21416_322 = {S41, 0, &V21416_322_57296, &P21416_321};	/*Property = Prompts*/
const Value V21416_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21416_323 = {S42, 0, &V21416_323_57299, &P21416_322};	/*Property = Selector*/
const Value V21416_324_57305 = {S2, NULL};	/*Value = True*/
const Property P21416_324 = {S43, 0, &V21416_324_57305, &P21416_323};	/*Property = Svg*/
const Value V21416_327_57314 = {S2, NULL};	/*Value = True*/
const Property P21416_327 = {S44, 0, &V21416_327_57314, &P21416_324};	/*Property = TouchEvents*/
const Value V21416_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21416_328 = {S45, 0, &V21416_328_57318, &P21416_327};	/*Property = Track*/
const Value V21416_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21416_329 = {S46, 0, &V21416_329_57323, &P21416_328};	/*Property = Video*/
const Value V21416_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21416_330 = {S47, 0, &V21416_330_57326, &P21416_329};	/*Property = Viewport*/
const Value V21416_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21416_332 = {S48, 0, &V21416_332_57339, &P21416_330};	/*Property = WebWorkers*/
const Value V21416_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21416_333 = {S49, 0, &V21416_333_57342, &P21416_332};	/*Property = Xhr2*/
const Profile P21416 = {"21416", &P21416_333};	/*Profile ID - 21416*/
const Value V21511_1_1 = {S2, NULL};	/*Value = True*/
const Property P21511_1 = {S50, 0, &V21511_1_1, NULL};	/*Property = IsMobile*/
const Value V21511_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21511_5 = {S52, 0, &V21511_5_1227, &P21511_1};	/*Property = ScreenPixelsHeight*/
const Value V21511_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21511_4 = {S54, 0, &V21511_4_1229, &P21511_5};	/*Property = ScreenPixelsWidth*/
const Profile P21511 = {"21511", &P21511_4};	/*Profile ID - 21511*/
const Value V22624_1_1 = {S2, NULL};	/*Value = True*/
const Property P22624_1 = {S50, 0, &V22624_1_1, NULL};	/*Property = IsMobile*/
const Value V22624_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22624_5 = {S52, 0, &V22624_5_1227, &P22624_1};	/*Property = ScreenPixelsHeight*/
const Value V22624_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22624_4 = {S54, 0, &V22624_4_1229, &P22624_5};	/*Property = ScreenPixelsWidth*/
const Profile P22624 = {"22624", &P22624_4};	/*Profile ID - 22624*/
const Value V21895_1_1 = {S2, NULL};	/*Value = True*/
const Property P21895_1 = {S50, 0, &V21895_1_1, NULL};	/*Property = IsMobile*/
const Value V21895_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21895_5 = {S52, 0, &V21895_5_1227, &P21895_1};	/*Property = ScreenPixelsHeight*/
const Value V21895_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21895_4 = {S54, 0, &V21895_4_1460, &P21895_5};	/*Property = ScreenPixelsWidth*/
const Profile P21895 = {"21895", &P21895_4};	/*Profile ID - 21895*/
const Value V20694_1_1 = {S2, NULL};	/*Value = True*/
const Property P20694_1 = {S50, 0, &V20694_1_1, NULL};	/*Property = IsMobile*/
const Value V20694_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20694_5 = {S52, 0, &V20694_5_1227, &P20694_1};	/*Property = ScreenPixelsHeight*/
const Value V20694_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20694_4 = {S54, 0, &V20694_4_1229, &P20694_5};	/*Property = ScreenPixelsWidth*/
const Profile P20694 = {"20694", &P20694_4};	/*Profile ID - 20694*/
const Value V21625_1_1 = {S2, NULL};	/*Value = True*/
const Property P21625_1 = {S50, 0, &V21625_1_1, NULL};	/*Property = IsMobile*/
const Value V21625_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21625_5 = {S52, 0, &V21625_5_1277, &P21625_1};	/*Property = ScreenPixelsHeight*/
const Value V21625_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21625_4 = {S54, 0, &V21625_4_1279, &P21625_5};	/*Property = ScreenPixelsWidth*/
const Profile P21625 = {"21625", &P21625_4};	/*Profile ID - 21625*/
const Value V21466_1_1 = {S2, NULL};	/*Value = True*/
const Property P21466_1 = {S50, 0, &V21466_1_1, NULL};	/*Property = IsMobile*/
const Value V21466_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21466_5 = {S52, 0, &V21466_5_1227, &P21466_1};	/*Property = ScreenPixelsHeight*/
const Value V21466_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21466_4 = {S54, 0, &V21466_4_1229, &P21466_5};	/*Property = ScreenPixelsWidth*/
const Profile P21466 = {"21466", &P21466_4};	/*Profile ID - 21466*/
const Value V22227_1_1 = {S2, NULL};	/*Value = True*/
const Property P22227_1 = {S50, 0, &V22227_1_1, NULL};	/*Property = IsMobile*/
const Value V22227_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22227_5 = {S52, 0, &V22227_5_1277, &P22227_1};	/*Property = ScreenPixelsHeight*/
const Value V22227_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22227_4 = {S54, 0, &V22227_4_2336, &P22227_5};	/*Property = ScreenPixelsWidth*/
const Profile P22227 = {"22227", &P22227_4};	/*Profile ID - 22227*/
const Value V22231_1_1 = {S2, NULL};	/*Value = True*/
const Property P22231_1 = {S50, 0, &V22231_1_1, NULL};	/*Property = IsMobile*/
const Value V22231_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22231_5 = {S52, 0, &V22231_5_1277, &P22231_1};	/*Property = ScreenPixelsHeight*/
const Value V22231_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22231_4 = {S54, 0, &V22231_4_2336, &P22231_5};	/*Property = ScreenPixelsWidth*/
const Profile P22231 = {"22231", &P22231_4};	/*Profile ID - 22231*/
const Value V21362_1_1 = {S2, NULL};	/*Value = True*/
const Property P21362_1 = {S50, 0, &V21362_1_1, NULL};	/*Property = IsMobile*/
const Value V21362_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21362_5 = {S52, 0, &V21362_5_1227, &P21362_1};	/*Property = ScreenPixelsHeight*/
const Value V21362_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21362_4 = {S54, 0, &V21362_4_1229, &P21362_5};	/*Property = ScreenPixelsWidth*/
const Profile P21362 = {"21362", &P21362_4};	/*Profile ID - 21362*/
const Value V22206_1_1 = {S2, NULL};	/*Value = True*/
const Property P22206_1 = {S50, 0, &V22206_1_1, NULL};	/*Property = IsMobile*/
const Value V22206_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22206_5 = {S52, 0, &V22206_5_1227, &P22206_1};	/*Property = ScreenPixelsHeight*/
const Value V22206_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22206_4 = {S54, 0, &V22206_4_1460, &P22206_5};	/*Property = ScreenPixelsWidth*/
const Profile P22206 = {"22206", &P22206_4};	/*Profile ID - 22206*/
const Value V21987_1_1 = {S2, NULL};	/*Value = True*/
const Property P21987_1 = {S50, 0, &V21987_1_1, NULL};	/*Property = IsMobile*/
const Value V21987_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21987_5 = {S52, 0, &V21987_5_1227, &P21987_1};	/*Property = ScreenPixelsHeight*/
const Value V21987_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21987_4 = {S54, 0, &V21987_4_1229, &P21987_5};	/*Property = ScreenPixelsWidth*/
const Profile P21987 = {"21987", &P21987_4};	/*Profile ID - 21987*/
const Value V21374_1_1 = {S2, NULL};	/*Value = True*/
const Property P21374_1 = {S50, 0, &V21374_1_1, NULL};	/*Property = IsMobile*/
const Value V21374_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21374_5 = {S52, 0, &V21374_5_9981, &P21374_1};	/*Property = ScreenPixelsHeight*/
const Value V21374_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21374_4 = {S54, 0, &V21374_4_2336, &P21374_5};	/*Property = ScreenPixelsWidth*/
const Profile P21374 = {"21374", &P21374_4};	/*Profile ID - 21374*/
const Value V22229_1_1 = {S2, NULL};	/*Value = True*/
const Property P22229_1 = {S50, 0, &V22229_1_1, NULL};	/*Property = IsMobile*/
const Value V22229_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22229_5 = {S52, 0, &V22229_5_37569, &P22229_1};	/*Property = ScreenPixelsHeight*/
const Value V22229_4_53077 = {S129, NULL};	/*Value = 1366*/
const Property P22229_4 = {S54, 0, &V22229_4_53077, &P22229_5};	/*Property = ScreenPixelsWidth*/
const Profile P22229 = {"22229", &P22229_4};	/*Profile ID - 22229*/
const Value V23813_1_1 = {S2, NULL};	/*Value = True*/
const Property P23813_1 = {S50, 0, &V23813_1_1, NULL};	/*Property = IsMobile*/
const Value V23813_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23813_5 = {S52, 0, &V23813_5_1227, &P23813_1};	/*Property = ScreenPixelsHeight*/
const Value V23813_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23813_4 = {S54, 0, &V23813_4_1229, &P23813_5};	/*Property = ScreenPixelsWidth*/
const Profile P23813 = {"23813", &P23813_4};	/*Profile ID - 23813*/
const Value V21358_1_1 = {S2, NULL};	/*Value = True*/
const Property P21358_1 = {S50, 0, &V21358_1_1, NULL};	/*Property = IsMobile*/
const Value V21358_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21358_5 = {S52, 0, &V21358_5_1277, &P21358_1};	/*Property = ScreenPixelsHeight*/
const Value V21358_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21358_4 = {S54, 0, &V21358_4_1279, &P21358_5};	/*Property = ScreenPixelsWidth*/
const Profile P21358 = {"21358", &P21358_4};	/*Profile ID - 21358*/
const Value V22945_1_1 = {S2, NULL};	/*Value = True*/
const Property P22945_1 = {S50, 0, &V22945_1_1, NULL};	/*Property = IsMobile*/
const Value V22945_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22945_5 = {S52, 0, &V22945_5_1458, &P22945_1};	/*Property = ScreenPixelsHeight*/
const Value V22945_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22945_4 = {S54, 0, &V22945_4_1460, &P22945_5};	/*Property = ScreenPixelsWidth*/
const Profile P22945 = {"22945", &P22945_4};	/*Profile ID - 22945*/
const Value V23073_1_1 = {S2, NULL};	/*Value = True*/
const Property P23073_1 = {S50, 0, &V23073_1_1, NULL};	/*Property = IsMobile*/
const Value V23073_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23073_5 = {S52, 0, &V23073_5_1832, &P23073_1};	/*Property = ScreenPixelsHeight*/
const Value V23073_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23073_4 = {S54, 0, &V23073_4_5645, &P23073_5};	/*Property = ScreenPixelsWidth*/
const Profile P23073 = {"23073", &P23073_4};	/*Profile ID - 23073*/
const Value V21842_1_1 = {S2, NULL};	/*Value = True*/
const Property P21842_1 = {S50, 0, &V21842_1_1, NULL};	/*Property = IsMobile*/
const Value V21842_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P21842_5 = {S52, 0, &V21842_5_1293, &P21842_1};	/*Property = ScreenPixelsHeight*/
const Value V21842_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P21842_4 = {S54, 0, &V21842_4_1296, &P21842_5};	/*Property = ScreenPixelsWidth*/
const Profile P21842 = {"21842", &P21842_4};	/*Profile ID - 21842*/
const Value V21829_1_1 = {S2, NULL};	/*Value = True*/
const Property P21829_1 = {S50, 0, &V21829_1_1, NULL};	/*Property = IsMobile*/
const Value V21829_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P21829_5 = {S52, 0, &V21829_5_59138, &P21829_1};	/*Property = ScreenPixelsHeight*/
const Value V21829_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P21829_4 = {S54, 0, &V21829_4_59970, &P21829_5};	/*Property = ScreenPixelsWidth*/
const Profile P21829 = {"21829", &P21829_4};	/*Profile ID - 21829*/
const Value V21886_1_1 = {S2, NULL};	/*Value = True*/
const Property P21886_1 = {S50, 0, &V21886_1_1, NULL};	/*Property = IsMobile*/
const Value V21886_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21886_5 = {S52, 0, &V21886_5_1458, &P21886_1};	/*Property = ScreenPixelsHeight*/
const Value V21886_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21886_4 = {S54, 0, &V21886_4_37617, &P21886_5};	/*Property = ScreenPixelsWidth*/
const Profile P21886 = {"21886", &P21886_4};	/*Profile ID - 21886*/
const Value V21738_1_1 = {S2, NULL};	/*Value = True*/
const Property P21738_1 = {S50, 0, &V21738_1_1, NULL};	/*Property = IsMobile*/
const Value V21738_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21738_5 = {S52, 0, &V21738_5_1227, &P21738_1};	/*Property = ScreenPixelsHeight*/
const Value V21738_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21738_4 = {S54, 0, &V21738_4_1229, &P21738_5};	/*Property = ScreenPixelsWidth*/
const Profile P21738 = {"21738", &P21738_4};	/*Profile ID - 21738*/
const Value V21737_1_1 = {S2, NULL};	/*Value = True*/
const Property P21737_1 = {S50, 0, &V21737_1_1, NULL};	/*Property = IsMobile*/
const Value V21737_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21737_5 = {S52, 0, &V21737_5_1277, &P21737_1};	/*Property = ScreenPixelsHeight*/
const Value V21737_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21737_4 = {S54, 0, &V21737_4_1279, &P21737_5};	/*Property = ScreenPixelsWidth*/
const Profile P21737 = {"21737", &P21737_4};	/*Profile ID - 21737*/
const Value V22657_1_1 = {S2, NULL};	/*Value = True*/
const Property P22657_1 = {S50, 0, &V22657_1_1, NULL};	/*Property = IsMobile*/
const Value V22657_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22657_5 = {S52, 0, &V22657_5_1403, &P22657_1};	/*Property = ScreenPixelsHeight*/
const Value V22657_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22657_4 = {S54, 0, &V22657_4_1404, &P22657_5};	/*Property = ScreenPixelsWidth*/
const Profile P22657 = {"22657", &P22657_4};	/*Profile ID - 22657*/
const Value V22901_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22901_279 = {S1, 0, &V22901_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22901_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22901_282 = {S3, 0, &V22901_282_57154, &P22901_279};	/*Property = BlobBuilder*/
const Value V22901_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22901_350 = {S4, 0, &V22901_350_57444, &P22901_282};	/*Property = Canvas*/
const Value V22901_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22901_283 = {S5, 0, &V22901_283_57159, &P22901_350};	/*Property = CssBackground*/
const Value V22901_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22901_284 = {S6, 0, &V22901_284_57162, &P22901_283};	/*Property = CssBorderImage*/
const Value V22901_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22901_285 = {S7, 0, &V22901_285_57166, &P22901_284};	/*Property = CssCanvas*/
const Value V22901_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22901_286 = {S8, 0, &V22901_286_57168, &P22901_285};	/*Property = CssColor*/
const Value V22901_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22901_237 = {S9, 0, &V22901_237_55786, &P22901_286};	/*Property = CssColumn*/
const Value V22901_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22901_287 = {S10, 0, &V22901_287_57172, &P22901_237};	/*Property = CssFlexbox*/
const Value V22901_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22901_288 = {S11, 0, &V22901_288_57174, &P22901_287};	/*Property = CssFont*/
const Value V22901_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22901_289 = {S12, 0, &V22901_289_57759, &P22901_288};	/*Property = CssImages*/
const Value V22901_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22901_291 = {S13, 0, &V22901_291_57180, &P22901_289};	/*Property = CssMediaQueries*/
const Value V22901_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22901_292 = {S14, 0, &V22901_292_57183, &P22901_291};	/*Property = CssMinMax*/
const Value V22901_293_57186 = {S2, NULL};	/*Value = True*/
const Property P22901_293 = {S15, 0, &V22901_293_57186, &P22901_292};	/*Property = CssOverflow*/
const Value V22901_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22901_294 = {S16, 0, &V22901_294_57189, &P22901_293};	/*Property = CssPosition*/
const Value V22901_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22901_295 = {S17, 0, &V22901_295_57192, &P22901_294};	/*Property = CssText*/
const Value V22901_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22901_297 = {S18, 0, &V22901_297_57195, &P22901_295};	/*Property = CssTransforms*/
const Value V22901_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22901_241 = {S19, 0, &V22901_241_55799, &P22901_297};	/*Property = CssTransitions*/
const Value V22901_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22901_298 = {S20, 0, &V22901_298_57206, &P22901_241};	/*Property = CssUI*/
const Value V22901_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22901_300 = {S21, 0, &V22901_300_57215, &P22901_298};	/*Property = DataSet*/
const Value V22901_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22901_301 = {S22, 0, &V22901_301_57218, &P22901_300};	/*Property = DataUrl*/
const Value V22901_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22901_302 = {S23, 0, &V22901_302_57221, &P22901_301};	/*Property = DeviceOrientation*/
const Value V22901_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22901_304 = {S24, 0, &V22901_304_57227, &P22901_302};	/*Property = FileReader*/
const Value V22901_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22901_305 = {S25, 0, &V22901_305_57231, &P22901_304};	/*Property = FileSaver*/
const Value V22901_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22901_306 = {S26, 0, &V22901_306_57237, &P22901_305};	/*Property = FileWriter*/
const Value V22901_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22901_307 = {S27, 0, &V22901_307_57239, &P22901_306};	/*Property = FormData*/
const Value V22901_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22901_309 = {S28, 0, &V22901_309_57246, &P22901_307};	/*Property = Fullscreen*/
const Value V22901_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22901_310 = {S29, 0, &V22901_310_57248, &P22901_309};	/*Property = GeoLocation*/
const Value V22901_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22901_312 = {S30, 0, &V22901_312_57254, &P22901_310};	/*Property = History*/
const Value V22901_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22901_314 = {S31, 0, &V22901_314_57260, &P22901_312};	/*Property = Html5*/
const Value V22901_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22901_313 = {S32, 0, &V22901_313_57258, &P22901_314};	/*Property = Html-Media-Capture*/
const Value V22901_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22901_315 = {S33, 0, &V22901_315_57264, &P22901_313};	/*Property = Iframe*/
const Value V22901_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22901_316 = {S34, 0, &V22901_316_57267, &P22901_315};	/*Property = IndexedDB*/
const Value V22901_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22901_317 = {S35, 0, &V22901_317_57269, &P22901_316};	/*Property = Json*/
const Value V22901_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22901_15 = {S37, 0, &V22901_15_28, &P22901_317};	/*Property = LayoutEngine*/
const Value V22901_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22901_344 = {S38, 0, &V22901_344_57272, &P22901_15};	/*Property = Masking*/
const Value V22901_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22901_320 = {S39, 0, &V22901_320_57290, &P22901_344};	/*Property = PostMessage*/
const Value V22901_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22901_321 = {S40, 0, &V22901_321_57293, &P22901_320};	/*Property = Progress*/
const Value V22901_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22901_322 = {S41, 0, &V22901_322_57296, &P22901_321};	/*Property = Prompts*/
const Value V22901_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22901_323 = {S42, 0, &V22901_323_57299, &P22901_322};	/*Property = Selector*/
const Value V22901_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22901_324 = {S43, 0, &V22901_324_57305, &P22901_323};	/*Property = Svg*/
const Value V22901_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22901_327 = {S44, 0, &V22901_327_57314, &P22901_324};	/*Property = TouchEvents*/
const Value V22901_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22901_328 = {S45, 0, &V22901_328_57318, &P22901_327};	/*Property = Track*/
const Value V22901_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22901_329 = {S46, 0, &V22901_329_57323, &P22901_328};	/*Property = Video*/
const Value V22901_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22901_330 = {S47, 0, &V22901_330_57326, &P22901_329};	/*Property = Viewport*/
const Value V22901_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22901_332 = {S48, 0, &V22901_332_57338, &P22901_330};	/*Property = WebWorkers*/
const Value V22901_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22901_333 = {S49, 0, &V22901_333_57342, &P22901_332};	/*Property = Xhr2*/
const Profile P22901 = {"22901", &P22901_333};	/*Profile ID - 22901*/
const Value V22465_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22465_279 = {S1, 0, &V22465_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22465_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22465_282 = {S3, 0, &V22465_282_57154, &P22465_279};	/*Property = BlobBuilder*/
const Value V22465_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22465_350 = {S4, 0, &V22465_350_57444, &P22465_282};	/*Property = Canvas*/
const Value V22465_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22465_283 = {S5, 0, &V22465_283_57159, &P22465_350};	/*Property = CssBackground*/
const Value V22465_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22465_284 = {S6, 0, &V22465_284_57162, &P22465_283};	/*Property = CssBorderImage*/
const Value V22465_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22465_285 = {S7, 0, &V22465_285_57166, &P22465_284};	/*Property = CssCanvas*/
const Value V22465_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22465_286 = {S8, 0, &V22465_286_57168, &P22465_285};	/*Property = CssColor*/
const Value V22465_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22465_237 = {S9, 0, &V22465_237_55786, &P22465_286};	/*Property = CssColumn*/
const Value V22465_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22465_287 = {S10, 0, &V22465_287_57172, &P22465_237};	/*Property = CssFlexbox*/
const Value V22465_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22465_288 = {S11, 0, &V22465_288_57175, &P22465_287};	/*Property = CssFont*/
const Value V22465_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22465_289 = {S12, 0, &V22465_289_57759, &P22465_288};	/*Property = CssImages*/
const Value V22465_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22465_291 = {S13, 0, &V22465_291_57180, &P22465_289};	/*Property = CssMediaQueries*/
const Value V22465_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22465_292 = {S14, 0, &V22465_292_57183, &P22465_291};	/*Property = CssMinMax*/
const Value V22465_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22465_293 = {S15, 0, &V22465_293_57187, &P22465_292};	/*Property = CssOverflow*/
const Value V22465_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22465_294 = {S16, 0, &V22465_294_57189, &P22465_293};	/*Property = CssPosition*/
const Value V22465_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22465_295 = {S17, 0, &V22465_295_57192, &P22465_294};	/*Property = CssText*/
const Value V22465_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22465_297 = {S18, 0, &V22465_297_57195, &P22465_295};	/*Property = CssTransforms*/
const Value V22465_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22465_241 = {S19, 0, &V22465_241_55799, &P22465_297};	/*Property = CssTransitions*/
const Value V22465_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22465_298 = {S20, 0, &V22465_298_57206, &P22465_241};	/*Property = CssUI*/
const Value V22465_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22465_300 = {S21, 0, &V22465_300_57215, &P22465_298};	/*Property = DataSet*/
const Value V22465_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22465_301 = {S22, 0, &V22465_301_57218, &P22465_300};	/*Property = DataUrl*/
const Value V22465_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22465_302 = {S23, 0, &V22465_302_57222, &P22465_301};	/*Property = DeviceOrientation*/
const Value V22465_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22465_304 = {S24, 0, &V22465_304_57227, &P22465_302};	/*Property = FileReader*/
const Value V22465_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22465_305 = {S25, 0, &V22465_305_57231, &P22465_304};	/*Property = FileSaver*/
const Value V22465_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22465_306 = {S26, 0, &V22465_306_57237, &P22465_305};	/*Property = FileWriter*/
const Value V22465_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22465_307 = {S27, 0, &V22465_307_57239, &P22465_306};	/*Property = FormData*/
const Value V22465_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22465_309 = {S28, 0, &V22465_309_57246, &P22465_307};	/*Property = Fullscreen*/
const Value V22465_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22465_310 = {S29, 0, &V22465_310_57248, &P22465_309};	/*Property = GeoLocation*/
const Value V22465_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22465_312 = {S30, 0, &V22465_312_57254, &P22465_310};	/*Property = History*/
const Value V22465_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22465_314 = {S31, 0, &V22465_314_57260, &P22465_312};	/*Property = Html5*/
const Value V22465_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22465_313 = {S32, 0, &V22465_313_57258, &P22465_314};	/*Property = Html-Media-Capture*/
const Value V22465_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22465_315 = {S33, 0, &V22465_315_57264, &P22465_313};	/*Property = Iframe*/
const Value V22465_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22465_316 = {S34, 0, &V22465_316_57266, &P22465_315};	/*Property = IndexedDB*/
const Value V22465_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22465_317 = {S35, 0, &V22465_317_57269, &P22465_316};	/*Property = Json*/
const Value V22465_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22465_15 = {S37, 0, &V22465_15_28, &P22465_317};	/*Property = LayoutEngine*/
const Value V22465_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22465_344 = {S38, 0, &V22465_344_57272, &P22465_15};	/*Property = Masking*/
const Value V22465_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22465_320 = {S39, 0, &V22465_320_57290, &P22465_344};	/*Property = PostMessage*/
const Value V22465_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22465_321 = {S40, 0, &V22465_321_57293, &P22465_320};	/*Property = Progress*/
const Value V22465_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22465_322 = {S41, 0, &V22465_322_57296, &P22465_321};	/*Property = Prompts*/
const Value V22465_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22465_323 = {S42, 0, &V22465_323_57299, &P22465_322};	/*Property = Selector*/
const Value V22465_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22465_324 = {S43, 0, &V22465_324_57305, &P22465_323};	/*Property = Svg*/
const Value V22465_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22465_327 = {S44, 0, &V22465_327_57315, &P22465_324};	/*Property = TouchEvents*/
const Value V22465_328_57317 = {S2, NULL};	/*Value = True*/
const Property P22465_328 = {S45, 0, &V22465_328_57317, &P22465_327};	/*Property = Track*/
const Value V22465_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22465_329 = {S46, 0, &V22465_329_57323, &P22465_328};	/*Property = Video*/
const Value V22465_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22465_330 = {S47, 0, &V22465_330_57326, &P22465_329};	/*Property = Viewport*/
const Value V22465_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22465_332 = {S48, 0, &V22465_332_57338, &P22465_330};	/*Property = WebWorkers*/
const Value V22465_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22465_333 = {S49, 0, &V22465_333_57341, &P22465_332};	/*Property = Xhr2*/
const Profile P22465 = {"22465", &P22465_333};	/*Profile ID - 22465*/
const Value V21782_279_57147 = {S2, NULL};	/*Value = True*/
const Property P21782_279 = {S1, 0, &V21782_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V21782_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21782_282 = {S3, 0, &V21782_282_57154, &P21782_279};	/*Property = BlobBuilder*/
const Value V21782_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21782_350 = {S4, 0, &V21782_350_57444, &P21782_282};	/*Property = Canvas*/
const Value V21782_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21782_283 = {S5, 0, &V21782_283_57159, &P21782_350};	/*Property = CssBackground*/
const Value V21782_284_57162 = {S2, NULL};	/*Value = True*/
const Property P21782_284 = {S6, 0, &V21782_284_57162, &P21782_283};	/*Property = CssBorderImage*/
const Value V21782_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21782_285 = {S7, 0, &V21782_285_57166, &P21782_284};	/*Property = CssCanvas*/
const Value V21782_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21782_286 = {S8, 0, &V21782_286_57168, &P21782_285};	/*Property = CssColor*/
const Value V21782_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21782_237 = {S9, 0, &V21782_237_55786, &P21782_286};	/*Property = CssColumn*/
const Value V21782_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21782_287 = {S10, 0, &V21782_287_57172, &P21782_237};	/*Property = CssFlexbox*/
const Value V21782_288_57174 = {S2, NULL};	/*Value = True*/
const Property P21782_288 = {S11, 0, &V21782_288_57174, &P21782_287};	/*Property = CssFont*/
const Value V21782_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21782_289 = {S12, 0, &V21782_289_57759, &P21782_288};	/*Property = CssImages*/
const Value V21782_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21782_291 = {S13, 0, &V21782_291_57180, &P21782_289};	/*Property = CssMediaQueries*/
const Value V21782_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21782_292 = {S14, 0, &V21782_292_57183, &P21782_291};	/*Property = CssMinMax*/
const Value V21782_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21782_293 = {S15, 0, &V21782_293_57187, &P21782_292};	/*Property = CssOverflow*/
const Value V21782_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21782_294 = {S16, 0, &V21782_294_57189, &P21782_293};	/*Property = CssPosition*/
const Value V21782_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21782_295 = {S17, 0, &V21782_295_57192, &P21782_294};	/*Property = CssText*/
const Value V21782_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21782_297 = {S18, 0, &V21782_297_57195, &P21782_295};	/*Property = CssTransforms*/
const Value V21782_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21782_241 = {S19, 0, &V21782_241_55799, &P21782_297};	/*Property = CssTransitions*/
const Value V21782_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21782_298 = {S20, 0, &V21782_298_57206, &P21782_241};	/*Property = CssUI*/
const Value V21782_300_57215 = {S2, NULL};	/*Value = True*/
const Property P21782_300 = {S21, 0, &V21782_300_57215, &P21782_298};	/*Property = DataSet*/
const Value V21782_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21782_301 = {S22, 0, &V21782_301_57218, &P21782_300};	/*Property = DataUrl*/
const Value V21782_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21782_302 = {S23, 0, &V21782_302_57222, &P21782_301};	/*Property = DeviceOrientation*/
const Value V21782_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21782_304 = {S24, 0, &V21782_304_57227, &P21782_302};	/*Property = FileReader*/
const Value V21782_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21782_305 = {S25, 0, &V21782_305_57231, &P21782_304};	/*Property = FileSaver*/
const Value V21782_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21782_306 = {S26, 0, &V21782_306_57237, &P21782_305};	/*Property = FileWriter*/
const Value V21782_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21782_307 = {S27, 0, &V21782_307_57239, &P21782_306};	/*Property = FormData*/
const Value V21782_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21782_309 = {S28, 0, &V21782_309_57246, &P21782_307};	/*Property = Fullscreen*/
const Value V21782_310_57249 = {S0, NULL};	/*Value = False*/
const Property P21782_310 = {S29, 0, &V21782_310_57249, &P21782_309};	/*Property = GeoLocation*/
const Value V21782_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21782_312 = {S30, 0, &V21782_312_57254, &P21782_310};	/*Property = History*/
const Value V21782_314_57260 = {S2, NULL};	/*Value = True*/
const Property P21782_314 = {S31, 0, &V21782_314_57260, &P21782_312};	/*Property = Html5*/
const Value V21782_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21782_313 = {S32, 0, &V21782_313_57258, &P21782_314};	/*Property = Html-Media-Capture*/
const Value V21782_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21782_315 = {S33, 0, &V21782_315_57264, &P21782_313};	/*Property = Iframe*/
const Value V21782_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21782_316 = {S34, 0, &V21782_316_57267, &P21782_315};	/*Property = IndexedDB*/
const Value V21782_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21782_317 = {S35, 0, &V21782_317_57269, &P21782_316};	/*Property = Json*/
const Value V21782_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21782_15 = {S37, 0, &V21782_15_28, &P21782_317};	/*Property = LayoutEngine*/
const Value V21782_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21782_344 = {S38, 0, &V21782_344_57272, &P21782_15};	/*Property = Masking*/
const Value V21782_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21782_320 = {S39, 0, &V21782_320_57290, &P21782_344};	/*Property = PostMessage*/
const Value V21782_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21782_321 = {S40, 0, &V21782_321_57293, &P21782_320};	/*Property = Progress*/
const Value V21782_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21782_322 = {S41, 0, &V21782_322_57296, &P21782_321};	/*Property = Prompts*/
const Value V21782_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21782_323 = {S42, 0, &V21782_323_57299, &P21782_322};	/*Property = Selector*/
const Value V21782_324_57305 = {S2, NULL};	/*Value = True*/
const Property P21782_324 = {S43, 0, &V21782_324_57305, &P21782_323};	/*Property = Svg*/
const Value V21782_327_57314 = {S2, NULL};	/*Value = True*/
const Property P21782_327 = {S44, 0, &V21782_327_57314, &P21782_324};	/*Property = TouchEvents*/
const Value V21782_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21782_328 = {S45, 0, &V21782_328_57318, &P21782_327};	/*Property = Track*/
const Value V21782_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21782_329 = {S46, 0, &V21782_329_57323, &P21782_328};	/*Property = Video*/
const Value V21782_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21782_330 = {S47, 0, &V21782_330_57326, &P21782_329};	/*Property = Viewport*/
const Value V21782_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21782_332 = {S48, 0, &V21782_332_57339, &P21782_330};	/*Property = WebWorkers*/
const Value V21782_333_57341 = {S2, NULL};	/*Value = True*/
const Property P21782_333 = {S49, 0, &V21782_333_57341, &P21782_332};	/*Property = Xhr2*/
const Profile P21782 = {"21782", &P21782_333};	/*Profile ID - 21782*/
const Value V21569_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_279 = {S1, 0, &V21569_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21569_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_282 = {S3, 0, &V21569_282_57155, &P21569_279};	/*Property = BlobBuilder*/
const Value V21569_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_350 = {S4, 0, &V21569_350_57446, &P21569_282};	/*Property = Canvas*/
const Value V21569_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_283 = {S5, 0, &V21569_283_57161, &P21569_350};	/*Property = CssBackground*/
const Value V21569_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_284 = {S6, 0, &V21569_284_57164, &P21569_283};	/*Property = CssBorderImage*/
const Value V21569_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_285 = {S7, 0, &V21569_285_57167, &P21569_284};	/*Property = CssCanvas*/
const Value V21569_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_286 = {S8, 0, &V21569_286_57170, &P21569_285};	/*Property = CssColor*/
const Value V21569_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_237 = {S9, 0, &V21569_237_55786, &P21569_286};	/*Property = CssColumn*/
const Value V21569_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_287 = {S10, 0, &V21569_287_57173, &P21569_237};	/*Property = CssFlexbox*/
const Value V21569_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_288 = {S11, 0, &V21569_288_57176, &P21569_287};	/*Property = CssFont*/
const Value V21569_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_289 = {S12, 0, &V21569_289_86899, &P21569_288};	/*Property = CssImages*/
const Value V21569_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_291 = {S13, 0, &V21569_291_57182, &P21569_289};	/*Property = CssMediaQueries*/
const Value V21569_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_292 = {S14, 0, &V21569_292_57185, &P21569_291};	/*Property = CssMinMax*/
const Value V21569_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_293 = {S15, 0, &V21569_293_57188, &P21569_292};	/*Property = CssOverflow*/
const Value V21569_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_294 = {S16, 0, &V21569_294_57191, &P21569_293};	/*Property = CssPosition*/
const Value V21569_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_295 = {S17, 0, &V21569_295_57194, &P21569_294};	/*Property = CssText*/
const Value V21569_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_297 = {S18, 0, &V21569_297_57197, &P21569_295};	/*Property = CssTransforms*/
const Value V21569_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_241 = {S19, 0, &V21569_241_55801, &P21569_297};	/*Property = CssTransitions*/
const Value V21569_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_298 = {S20, 0, &V21569_298_57208, &P21569_241};	/*Property = CssUI*/
const Value V21569_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_300 = {S21, 0, &V21569_300_57217, &P21569_298};	/*Property = DataSet*/
const Value V21569_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_301 = {S22, 0, &V21569_301_57220, &P21569_300};	/*Property = DataUrl*/
const Value V21569_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_302 = {S23, 0, &V21569_302_57223, &P21569_301};	/*Property = DeviceOrientation*/
const Value V21569_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_304 = {S24, 0, &V21569_304_57229, &P21569_302};	/*Property = FileReader*/
const Value V21569_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_305 = {S25, 0, &V21569_305_57232, &P21569_304};	/*Property = FileSaver*/
const Value V21569_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_306 = {S26, 0, &V21569_306_57238, &P21569_305};	/*Property = FileWriter*/
const Value V21569_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_307 = {S27, 0, &V21569_307_57241, &P21569_306};	/*Property = FormData*/
const Value V21569_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_309 = {S28, 0, &V21569_309_57247, &P21569_307};	/*Property = Fullscreen*/
const Value V21569_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_310 = {S29, 0, &V21569_310_57250, &P21569_309};	/*Property = GeoLocation*/
const Value V21569_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_312 = {S30, 0, &V21569_312_57256, &P21569_310};	/*Property = History*/
const Value V21569_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_314 = {S31, 0, &V21569_314_57262, &P21569_312};	/*Property = Html5*/
const Value V21569_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_313 = {S32, 0, &V21569_313_57259, &P21569_314};	/*Property = Html-Media-Capture*/
const Value V21569_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_315 = {S33, 0, &V21569_315_57265, &P21569_313};	/*Property = Iframe*/
const Value V21569_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_316 = {S34, 0, &V21569_316_57268, &P21569_315};	/*Property = IndexedDB*/
const Value V21569_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_317 = {S35, 0, &V21569_317_57271, &P21569_316};	/*Property = Json*/
const Value V21569_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21569_15 = {S37, 0, &V21569_15_28, &P21569_317};	/*Property = LayoutEngine*/
const Value V21569_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_344 = {S38, 0, &V21569_344_57274, &P21569_15};	/*Property = Masking*/
const Value V21569_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_320 = {S39, 0, &V21569_320_57292, &P21569_344};	/*Property = PostMessage*/
const Value V21569_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_321 = {S40, 0, &V21569_321_57295, &P21569_320};	/*Property = Progress*/
const Value V21569_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_322 = {S41, 0, &V21569_322_57298, &P21569_321};	/*Property = Prompts*/
const Value V21569_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_323 = {S42, 0, &V21569_323_57301, &P21569_322};	/*Property = Selector*/
const Value V21569_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_324 = {S43, 0, &V21569_324_57307, &P21569_323};	/*Property = Svg*/
const Value V21569_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_327 = {S44, 0, &V21569_327_57316, &P21569_324};	/*Property = TouchEvents*/
const Value V21569_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_328 = {S45, 0, &V21569_328_57319, &P21569_327};	/*Property = Track*/
const Value V21569_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_329 = {S46, 0, &V21569_329_57325, &P21569_328};	/*Property = Video*/
const Value V21569_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_330 = {S47, 0, &V21569_330_57328, &P21569_329};	/*Property = Viewport*/
const Value V21569_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_332 = {S48, 0, &V21569_332_57340, &P21569_330};	/*Property = WebWorkers*/
const Value V21569_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21569_333 = {S49, 0, &V21569_333_57343, &P21569_332};	/*Property = Xhr2*/
const Profile P21569 = {"21569", &P21569_333};	/*Profile ID - 21569*/
const Value V21784_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21784_279 = {S1, 0, &V21784_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21784_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21784_282 = {S3, 0, &V21784_282_57154, &P21784_279};	/*Property = BlobBuilder*/
const Value V21784_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21784_350 = {S4, 0, &V21784_350_57444, &P21784_282};	/*Property = Canvas*/
const Value V21784_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21784_283 = {S5, 0, &V21784_283_57159, &P21784_350};	/*Property = CssBackground*/
const Value V21784_284_57162 = {S2, NULL};	/*Value = True*/
const Property P21784_284 = {S6, 0, &V21784_284_57162, &P21784_283};	/*Property = CssBorderImage*/
const Value V21784_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21784_285 = {S7, 0, &V21784_285_57166, &P21784_284};	/*Property = CssCanvas*/
const Value V21784_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21784_286 = {S8, 0, &V21784_286_57168, &P21784_285};	/*Property = CssColor*/
const Value V21784_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21784_237 = {S9, 0, &V21784_237_55786, &P21784_286};	/*Property = CssColumn*/
const Value V21784_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21784_287 = {S10, 0, &V21784_287_57172, &P21784_237};	/*Property = CssFlexbox*/
const Value V21784_288_57174 = {S2, NULL};	/*Value = True*/
const Property P21784_288 = {S11, 0, &V21784_288_57174, &P21784_287};	/*Property = CssFont*/
const Value V21784_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21784_289 = {S12, 0, &V21784_289_57759, &P21784_288};	/*Property = CssImages*/
const Value V21784_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21784_291 = {S13, 0, &V21784_291_57180, &P21784_289};	/*Property = CssMediaQueries*/
const Value V21784_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21784_292 = {S14, 0, &V21784_292_57183, &P21784_291};	/*Property = CssMinMax*/
const Value V21784_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21784_293 = {S15, 0, &V21784_293_57187, &P21784_292};	/*Property = CssOverflow*/
const Value V21784_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21784_294 = {S16, 0, &V21784_294_57189, &P21784_293};	/*Property = CssPosition*/
const Value V21784_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21784_295 = {S17, 0, &V21784_295_57192, &P21784_294};	/*Property = CssText*/
const Value V21784_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21784_297 = {S18, 0, &V21784_297_57195, &P21784_295};	/*Property = CssTransforms*/
const Value V21784_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21784_241 = {S19, 0, &V21784_241_55799, &P21784_297};	/*Property = CssTransitions*/
const Value V21784_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21784_298 = {S20, 0, &V21784_298_57206, &P21784_241};	/*Property = CssUI*/
const Value V21784_300_57215 = {S2, NULL};	/*Value = True*/
const Property P21784_300 = {S21, 0, &V21784_300_57215, &P21784_298};	/*Property = DataSet*/
const Value V21784_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21784_301 = {S22, 0, &V21784_301_57218, &P21784_300};	/*Property = DataUrl*/
const Value V21784_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21784_302 = {S23, 0, &V21784_302_57222, &P21784_301};	/*Property = DeviceOrientation*/
const Value V21784_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21784_304 = {S24, 0, &V21784_304_57227, &P21784_302};	/*Property = FileReader*/
const Value V21784_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21784_305 = {S25, 0, &V21784_305_57231, &P21784_304};	/*Property = FileSaver*/
const Value V21784_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21784_306 = {S26, 0, &V21784_306_57237, &P21784_305};	/*Property = FileWriter*/
const Value V21784_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21784_307 = {S27, 0, &V21784_307_57239, &P21784_306};	/*Property = FormData*/
const Value V21784_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21784_309 = {S28, 0, &V21784_309_57246, &P21784_307};	/*Property = Fullscreen*/
const Value V21784_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21784_310 = {S29, 0, &V21784_310_57248, &P21784_309};	/*Property = GeoLocation*/
const Value V21784_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21784_312 = {S30, 0, &V21784_312_57254, &P21784_310};	/*Property = History*/
const Value V21784_314_57260 = {S2, NULL};	/*Value = True*/
const Property P21784_314 = {S31, 0, &V21784_314_57260, &P21784_312};	/*Property = Html5*/
const Value V21784_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21784_313 = {S32, 0, &V21784_313_57258, &P21784_314};	/*Property = Html-Media-Capture*/
const Value V21784_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21784_315 = {S33, 0, &V21784_315_57264, &P21784_313};	/*Property = Iframe*/
const Value V21784_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21784_316 = {S34, 0, &V21784_316_57266, &P21784_315};	/*Property = IndexedDB*/
const Value V21784_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21784_317 = {S35, 0, &V21784_317_57269, &P21784_316};	/*Property = Json*/
const Value V21784_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21784_15 = {S37, 0, &V21784_15_28, &P21784_317};	/*Property = LayoutEngine*/
const Value V21784_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21784_344 = {S38, 0, &V21784_344_57272, &P21784_15};	/*Property = Masking*/
const Value V21784_320_57291 = {S0, NULL};	/*Value = False*/
const Property P21784_320 = {S39, 0, &V21784_320_57291, &P21784_344};	/*Property = PostMessage*/
const Value V21784_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21784_321 = {S40, 0, &V21784_321_57293, &P21784_320};	/*Property = Progress*/
const Value V21784_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21784_322 = {S41, 0, &V21784_322_57296, &P21784_321};	/*Property = Prompts*/
const Value V21784_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21784_323 = {S42, 0, &V21784_323_57299, &P21784_322};	/*Property = Selector*/
const Value V21784_324_57305 = {S2, NULL};	/*Value = True*/
const Property P21784_324 = {S43, 0, &V21784_324_57305, &P21784_323};	/*Property = Svg*/
const Value V21784_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21784_327 = {S44, 0, &V21784_327_57315, &P21784_324};	/*Property = TouchEvents*/
const Value V21784_328_57317 = {S2, NULL};	/*Value = True*/
const Property P21784_328 = {S45, 0, &V21784_328_57317, &P21784_327};	/*Property = Track*/
const Value V21784_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21784_329 = {S46, 0, &V21784_329_57323, &P21784_328};	/*Property = Video*/
const Value V21784_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21784_330 = {S47, 0, &V21784_330_57326, &P21784_329};	/*Property = Viewport*/
const Value V21784_332_57338 = {S2, NULL};	/*Value = True*/
const Property P21784_332 = {S48, 0, &V21784_332_57338, &P21784_330};	/*Property = WebWorkers*/
const Value V21784_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21784_333 = {S49, 0, &V21784_333_57342, &P21784_332};	/*Property = Xhr2*/
const Profile P21784 = {"21784", &P21784_333};	/*Profile ID - 21784*/
const Value V22559_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22559_279 = {S1, 0, &V22559_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22559_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22559_282 = {S3, 0, &V22559_282_57154, &P22559_279};	/*Property = BlobBuilder*/
const Value V22559_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22559_350 = {S4, 0, &V22559_350_57444, &P22559_282};	/*Property = Canvas*/
const Value V22559_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22559_283 = {S5, 0, &V22559_283_57159, &P22559_350};	/*Property = CssBackground*/
const Value V22559_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22559_284 = {S6, 0, &V22559_284_57163, &P22559_283};	/*Property = CssBorderImage*/
const Value V22559_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22559_285 = {S7, 0, &V22559_285_57166, &P22559_284};	/*Property = CssCanvas*/
const Value V22559_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22559_286 = {S8, 0, &V22559_286_57168, &P22559_285};	/*Property = CssColor*/
const Value V22559_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22559_237 = {S9, 0, &V22559_237_55786, &P22559_286};	/*Property = CssColumn*/
const Value V22559_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22559_287 = {S10, 0, &V22559_287_57172, &P22559_237};	/*Property = CssFlexbox*/
const Value V22559_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22559_288 = {S11, 0, &V22559_288_57174, &P22559_287};	/*Property = CssFont*/
const Value V22559_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22559_289 = {S12, 0, &V22559_289_57759, &P22559_288};	/*Property = CssImages*/
const Value V22559_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22559_291 = {S13, 0, &V22559_291_57180, &P22559_289};	/*Property = CssMediaQueries*/
const Value V22559_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22559_292 = {S14, 0, &V22559_292_57183, &P22559_291};	/*Property = CssMinMax*/
const Value V22559_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22559_293 = {S15, 0, &V22559_293_57187, &P22559_292};	/*Property = CssOverflow*/
const Value V22559_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22559_294 = {S16, 0, &V22559_294_57189, &P22559_293};	/*Property = CssPosition*/
const Value V22559_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22559_295 = {S17, 0, &V22559_295_57192, &P22559_294};	/*Property = CssText*/
const Value V22559_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22559_297 = {S18, 0, &V22559_297_57195, &P22559_295};	/*Property = CssTransforms*/
const Value V22559_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22559_241 = {S19, 0, &V22559_241_55799, &P22559_297};	/*Property = CssTransitions*/
const Value V22559_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22559_298 = {S20, 0, &V22559_298_57206, &P22559_241};	/*Property = CssUI*/
const Value V22559_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22559_300 = {S21, 0, &V22559_300_57216, &P22559_298};	/*Property = DataSet*/
const Value V22559_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22559_301 = {S22, 0, &V22559_301_57218, &P22559_300};	/*Property = DataUrl*/
const Value V22559_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22559_302 = {S23, 0, &V22559_302_57221, &P22559_301};	/*Property = DeviceOrientation*/
const Value V22559_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22559_304 = {S24, 0, &V22559_304_57227, &P22559_302};	/*Property = FileReader*/
const Value V22559_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22559_305 = {S25, 0, &V22559_305_57231, &P22559_304};	/*Property = FileSaver*/
const Value V22559_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22559_306 = {S26, 0, &V22559_306_57237, &P22559_305};	/*Property = FileWriter*/
const Value V22559_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22559_307 = {S27, 0, &V22559_307_57239, &P22559_306};	/*Property = FormData*/
const Value V22559_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22559_309 = {S28, 0, &V22559_309_57246, &P22559_307};	/*Property = Fullscreen*/
const Value V22559_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22559_310 = {S29, 0, &V22559_310_57248, &P22559_309};	/*Property = GeoLocation*/
const Value V22559_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22559_312 = {S30, 0, &V22559_312_57254, &P22559_310};	/*Property = History*/
const Value V22559_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22559_314 = {S31, 0, &V22559_314_57261, &P22559_312};	/*Property = Html5*/
const Value V22559_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22559_313 = {S32, 0, &V22559_313_57258, &P22559_314};	/*Property = Html-Media-Capture*/
const Value V22559_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22559_315 = {S33, 0, &V22559_315_57264, &P22559_313};	/*Property = Iframe*/
const Value V22559_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22559_316 = {S34, 0, &V22559_316_57266, &P22559_315};	/*Property = IndexedDB*/
const Value V22559_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22559_317 = {S35, 0, &V22559_317_57269, &P22559_316};	/*Property = Json*/
const Value V22559_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22559_15 = {S37, 0, &V22559_15_29, &P22559_317};	/*Property = LayoutEngine*/
const Value V22559_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22559_344 = {S38, 0, &V22559_344_57272, &P22559_15};	/*Property = Masking*/
const Value V22559_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22559_320 = {S39, 0, &V22559_320_57290, &P22559_344};	/*Property = PostMessage*/
const Value V22559_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22559_321 = {S40, 0, &V22559_321_57293, &P22559_320};	/*Property = Progress*/
const Value V22559_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22559_322 = {S41, 0, &V22559_322_57296, &P22559_321};	/*Property = Prompts*/
const Value V22559_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22559_323 = {S42, 0, &V22559_323_57299, &P22559_322};	/*Property = Selector*/
const Value V22559_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22559_324 = {S43, 0, &V22559_324_57306, &P22559_323};	/*Property = Svg*/
const Value V22559_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22559_327 = {S44, 0, &V22559_327_57315, &P22559_324};	/*Property = TouchEvents*/
const Value V22559_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22559_328 = {S45, 0, &V22559_328_57318, &P22559_327};	/*Property = Track*/
const Value V22559_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22559_329 = {S46, 0, &V22559_329_57323, &P22559_328};	/*Property = Video*/
const Value V22559_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22559_330 = {S47, 0, &V22559_330_57326, &P22559_329};	/*Property = Viewport*/
const Value V22559_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22559_332 = {S48, 0, &V22559_332_57338, &P22559_330};	/*Property = WebWorkers*/
const Value V22559_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22559_333 = {S49, 0, &V22559_333_57341, &P22559_332};	/*Property = Xhr2*/
const Profile P22559 = {"22559", &P22559_333};	/*Profile ID - 22559*/
const Value V24173_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24173_279 = {S1, 0, &V24173_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24173_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24173_282 = {S3, 0, &V24173_282_57154, &P24173_279};	/*Property = BlobBuilder*/
const Value V24173_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24173_350 = {S4, 0, &V24173_350_57444, &P24173_282};	/*Property = Canvas*/
const Value V24173_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24173_283 = {S5, 0, &V24173_283_57159, &P24173_350};	/*Property = CssBackground*/
const Value V24173_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24173_284 = {S6, 0, &V24173_284_57162, &P24173_283};	/*Property = CssBorderImage*/
const Value V24173_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24173_285 = {S7, 0, &V24173_285_57166, &P24173_284};	/*Property = CssCanvas*/
const Value V24173_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24173_286 = {S8, 0, &V24173_286_57168, &P24173_285};	/*Property = CssColor*/
const Value V24173_237_55784 = {S2, NULL};	/*Value = True*/
const Property P24173_237 = {S9, 0, &V24173_237_55784, &P24173_286};	/*Property = CssColumn*/
const Value V24173_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24173_287 = {S10, 0, &V24173_287_57172, &P24173_237};	/*Property = CssFlexbox*/
const Value V24173_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24173_288 = {S11, 0, &V24173_288_57174, &P24173_287};	/*Property = CssFont*/
const Value V24173_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24173_289 = {S12, 0, &V24173_289_57759, &P24173_288};	/*Property = CssImages*/
const Value V24173_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24173_291 = {S13, 0, &V24173_291_57180, &P24173_289};	/*Property = CssMediaQueries*/
const Value V24173_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24173_292 = {S14, 0, &V24173_292_57183, &P24173_291};	/*Property = CssMinMax*/
const Value V24173_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24173_293 = {S15, 0, &V24173_293_57187, &P24173_292};	/*Property = CssOverflow*/
const Value V24173_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24173_294 = {S16, 0, &V24173_294_57189, &P24173_293};	/*Property = CssPosition*/
const Value V24173_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24173_295 = {S17, 0, &V24173_295_57192, &P24173_294};	/*Property = CssText*/
const Value V24173_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24173_297 = {S18, 0, &V24173_297_57195, &P24173_295};	/*Property = CssTransforms*/
const Value V24173_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24173_241 = {S19, 0, &V24173_241_55799, &P24173_297};	/*Property = CssTransitions*/
const Value V24173_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24173_298 = {S20, 0, &V24173_298_57206, &P24173_241};	/*Property = CssUI*/
const Value V24173_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24173_300 = {S21, 0, &V24173_300_57215, &P24173_298};	/*Property = DataSet*/
const Value V24173_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24173_301 = {S22, 0, &V24173_301_57218, &P24173_300};	/*Property = DataUrl*/
const Value V24173_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24173_302 = {S23, 0, &V24173_302_57222, &P24173_301};	/*Property = DeviceOrientation*/
const Value V24173_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24173_304 = {S24, 0, &V24173_304_57227, &P24173_302};	/*Property = FileReader*/
const Value V24173_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24173_305 = {S25, 0, &V24173_305_57231, &P24173_304};	/*Property = FileSaver*/
const Value V24173_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24173_306 = {S26, 0, &V24173_306_57237, &P24173_305};	/*Property = FileWriter*/
const Value V24173_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24173_307 = {S27, 0, &V24173_307_57239, &P24173_306};	/*Property = FormData*/
const Value V24173_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24173_309 = {S28, 0, &V24173_309_57246, &P24173_307};	/*Property = Fullscreen*/
const Value V24173_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24173_310 = {S29, 0, &V24173_310_57248, &P24173_309};	/*Property = GeoLocation*/
const Value V24173_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24173_312 = {S30, 0, &V24173_312_57254, &P24173_310};	/*Property = History*/
const Value V24173_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24173_314 = {S31, 0, &V24173_314_57260, &P24173_312};	/*Property = Html5*/
const Value V24173_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24173_313 = {S32, 0, &V24173_313_57257, &P24173_314};	/*Property = Html-Media-Capture*/
const Value V24173_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24173_315 = {S33, 0, &V24173_315_57264, &P24173_313};	/*Property = Iframe*/
const Value V24173_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24173_316 = {S34, 0, &V24173_316_57266, &P24173_315};	/*Property = IndexedDB*/
const Value V24173_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24173_317 = {S35, 0, &V24173_317_57269, &P24173_316};	/*Property = Json*/
const Value V24173_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24173_15 = {S37, 0, &V24173_15_28, &P24173_317};	/*Property = LayoutEngine*/
const Value V24173_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24173_344 = {S38, 0, &V24173_344_57272, &P24173_15};	/*Property = Masking*/
const Value V24173_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24173_320 = {S39, 0, &V24173_320_57290, &P24173_344};	/*Property = PostMessage*/
const Value V24173_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24173_321 = {S40, 0, &V24173_321_57293, &P24173_320};	/*Property = Progress*/
const Value V24173_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24173_322 = {S41, 0, &V24173_322_57296, &P24173_321};	/*Property = Prompts*/
const Value V24173_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24173_323 = {S42, 0, &V24173_323_57299, &P24173_322};	/*Property = Selector*/
const Value V24173_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24173_324 = {S43, 0, &V24173_324_57305, &P24173_323};	/*Property = Svg*/
const Value V24173_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24173_327 = {S44, 0, &V24173_327_57314, &P24173_324};	/*Property = TouchEvents*/
const Value V24173_328_57317 = {S2, NULL};	/*Value = True*/
const Property P24173_328 = {S45, 0, &V24173_328_57317, &P24173_327};	/*Property = Track*/
const Value V24173_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24173_329 = {S46, 0, &V24173_329_57323, &P24173_328};	/*Property = Video*/
const Value V24173_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24173_330 = {S47, 0, &V24173_330_57326, &P24173_329};	/*Property = Viewport*/
const Value V24173_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24173_332 = {S48, 0, &V24173_332_57339, &P24173_330};	/*Property = WebWorkers*/
const Value V24173_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24173_333 = {S49, 0, &V24173_333_57342, &P24173_332};	/*Property = Xhr2*/
const Profile P24173 = {"24173", &P24173_333};	/*Profile ID - 24173*/
const Value V21646_1_1 = {S2, NULL};	/*Value = True*/
const Property P21646_1 = {S50, 0, &V21646_1_1, NULL};	/*Property = IsMobile*/
const Value V21646_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21646_5 = {S52, 0, &V21646_5_1403, &P21646_1};	/*Property = ScreenPixelsHeight*/
const Value V21646_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21646_4 = {S54, 0, &V21646_4_1404, &P21646_5};	/*Property = ScreenPixelsWidth*/
const Profile P21646 = {"21646", &P21646_4};	/*Profile ID - 21646*/
const Profile P21717 = {"21717", NULL};	/*Profile ID - 21717*/
const Value V24406_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24406_15 = {S37, 0, &V24406_15_28, NULL};	/*Property = LayoutEngine*/
const Profile P24406 = {"24406", &P24406_15};	/*Profile ID - 24406*/
const Value V21982_1_1 = {S2, NULL};	/*Value = True*/
const Property P21982_1 = {S50, 0, &V21982_1_1, NULL};	/*Property = IsMobile*/
const Value V21982_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21982_5 = {S52, 0, &V21982_5_1227, &P21982_1};	/*Property = ScreenPixelsHeight*/
const Value V21982_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21982_4 = {S54, 0, &V21982_4_1229, &P21982_5};	/*Property = ScreenPixelsWidth*/
const Profile P21982 = {"21982", &P21982_4};	/*Profile ID - 21982*/
const Value V21454_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_279 = {S1, 0, &V21454_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21454_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_282 = {S3, 0, &V21454_282_57155, &P21454_279};	/*Property = BlobBuilder*/
const Value V21454_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_350 = {S4, 0, &V21454_350_57446, &P21454_282};	/*Property = Canvas*/
const Value V21454_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_283 = {S5, 0, &V21454_283_57161, &P21454_350};	/*Property = CssBackground*/
const Value V21454_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_284 = {S6, 0, &V21454_284_57164, &P21454_283};	/*Property = CssBorderImage*/
const Value V21454_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_285 = {S7, 0, &V21454_285_57167, &P21454_284};	/*Property = CssCanvas*/
const Value V21454_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_286 = {S8, 0, &V21454_286_57170, &P21454_285};	/*Property = CssColor*/
const Value V21454_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_237 = {S9, 0, &V21454_237_55786, &P21454_286};	/*Property = CssColumn*/
const Value V21454_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_287 = {S10, 0, &V21454_287_57173, &P21454_237};	/*Property = CssFlexbox*/
const Value V21454_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_288 = {S11, 0, &V21454_288_57176, &P21454_287};	/*Property = CssFont*/
const Value V21454_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_289 = {S12, 0, &V21454_289_86899, &P21454_288};	/*Property = CssImages*/
const Value V21454_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_291 = {S13, 0, &V21454_291_57182, &P21454_289};	/*Property = CssMediaQueries*/
const Value V21454_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_292 = {S14, 0, &V21454_292_57185, &P21454_291};	/*Property = CssMinMax*/
const Value V21454_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_293 = {S15, 0, &V21454_293_57188, &P21454_292};	/*Property = CssOverflow*/
const Value V21454_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_294 = {S16, 0, &V21454_294_57191, &P21454_293};	/*Property = CssPosition*/
const Value V21454_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_295 = {S17, 0, &V21454_295_57194, &P21454_294};	/*Property = CssText*/
const Value V21454_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_297 = {S18, 0, &V21454_297_57197, &P21454_295};	/*Property = CssTransforms*/
const Value V21454_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_241 = {S19, 0, &V21454_241_55801, &P21454_297};	/*Property = CssTransitions*/
const Value V21454_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_298 = {S20, 0, &V21454_298_57208, &P21454_241};	/*Property = CssUI*/
const Value V21454_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_300 = {S21, 0, &V21454_300_57217, &P21454_298};	/*Property = DataSet*/
const Value V21454_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_301 = {S22, 0, &V21454_301_57220, &P21454_300};	/*Property = DataUrl*/
const Value V21454_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_302 = {S23, 0, &V21454_302_57223, &P21454_301};	/*Property = DeviceOrientation*/
const Value V21454_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_304 = {S24, 0, &V21454_304_57229, &P21454_302};	/*Property = FileReader*/
const Value V21454_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_305 = {S25, 0, &V21454_305_57232, &P21454_304};	/*Property = FileSaver*/
const Value V21454_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_306 = {S26, 0, &V21454_306_57238, &P21454_305};	/*Property = FileWriter*/
const Value V21454_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_307 = {S27, 0, &V21454_307_57241, &P21454_306};	/*Property = FormData*/
const Value V21454_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_309 = {S28, 0, &V21454_309_57247, &P21454_307};	/*Property = Fullscreen*/
const Value V21454_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_310 = {S29, 0, &V21454_310_57250, &P21454_309};	/*Property = GeoLocation*/
const Value V21454_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_312 = {S30, 0, &V21454_312_57256, &P21454_310};	/*Property = History*/
const Value V21454_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_314 = {S31, 0, &V21454_314_57262, &P21454_312};	/*Property = Html5*/
const Value V21454_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_313 = {S32, 0, &V21454_313_57259, &P21454_314};	/*Property = Html-Media-Capture*/
const Value V21454_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_315 = {S33, 0, &V21454_315_57265, &P21454_313};	/*Property = Iframe*/
const Value V21454_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_316 = {S34, 0, &V21454_316_57268, &P21454_315};	/*Property = IndexedDB*/
const Value V21454_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_317 = {S35, 0, &V21454_317_57271, &P21454_316};	/*Property = Json*/
const Value V21454_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P21454_15 = {S37, 0, &V21454_15_31, &P21454_317};	/*Property = LayoutEngine*/
const Value V21454_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_344 = {S38, 0, &V21454_344_57274, &P21454_15};	/*Property = Masking*/
const Value V21454_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_320 = {S39, 0, &V21454_320_57292, &P21454_344};	/*Property = PostMessage*/
const Value V21454_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_321 = {S40, 0, &V21454_321_57295, &P21454_320};	/*Property = Progress*/
const Value V21454_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_322 = {S41, 0, &V21454_322_57298, &P21454_321};	/*Property = Prompts*/
const Value V21454_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_323 = {S42, 0, &V21454_323_57301, &P21454_322};	/*Property = Selector*/
const Value V21454_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_324 = {S43, 0, &V21454_324_57307, &P21454_323};	/*Property = Svg*/
const Value V21454_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_327 = {S44, 0, &V21454_327_57316, &P21454_324};	/*Property = TouchEvents*/
const Value V21454_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_328 = {S45, 0, &V21454_328_57319, &P21454_327};	/*Property = Track*/
const Value V21454_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_329 = {S46, 0, &V21454_329_57325, &P21454_328};	/*Property = Video*/
const Value V21454_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_330 = {S47, 0, &V21454_330_57328, &P21454_329};	/*Property = Viewport*/
const Value V21454_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_332 = {S48, 0, &V21454_332_57340, &P21454_330};	/*Property = WebWorkers*/
const Value V21454_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21454_333 = {S49, 0, &V21454_333_57343, &P21454_332};	/*Property = Xhr2*/
const Profile P21454 = {"21454", &P21454_333};	/*Profile ID - 21454*/
const Value V23253_1_1 = {S2, NULL};	/*Value = True*/
const Property P23253_1 = {S50, 0, &V23253_1_1, NULL};	/*Property = IsMobile*/
const Value V23253_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23253_5 = {S52, 0, &V23253_5_1277, &P23253_1};	/*Property = ScreenPixelsHeight*/
const Value V23253_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23253_4 = {S54, 0, &V23253_4_2336, &P23253_5};	/*Property = ScreenPixelsWidth*/
const Profile P23253 = {"23253", &P23253_4};	/*Profile ID - 23253*/
const Value V21777_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21777_279 = {S1, 0, &V21777_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21777_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21777_282 = {S3, 0, &V21777_282_57154, &P21777_279};	/*Property = BlobBuilder*/
const Value V21777_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21777_350 = {S4, 0, &V21777_350_57444, &P21777_282};	/*Property = Canvas*/
const Value V21777_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21777_283 = {S5, 0, &V21777_283_57159, &P21777_350};	/*Property = CssBackground*/
const Value V21777_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21777_284 = {S6, 0, &V21777_284_57163, &P21777_283};	/*Property = CssBorderImage*/
const Value V21777_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21777_285 = {S7, 0, &V21777_285_57166, &P21777_284};	/*Property = CssCanvas*/
const Value V21777_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21777_286 = {S8, 0, &V21777_286_57168, &P21777_285};	/*Property = CssColor*/
const Value V21777_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21777_237 = {S9, 0, &V21777_237_55786, &P21777_286};	/*Property = CssColumn*/
const Value V21777_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21777_287 = {S10, 0, &V21777_287_57172, &P21777_237};	/*Property = CssFlexbox*/
const Value V21777_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21777_288 = {S11, 0, &V21777_288_57175, &P21777_287};	/*Property = CssFont*/
const Value V21777_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21777_289 = {S12, 0, &V21777_289_57759, &P21777_288};	/*Property = CssImages*/
const Value V21777_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21777_291 = {S13, 0, &V21777_291_57181, &P21777_289};	/*Property = CssMediaQueries*/
const Value V21777_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21777_292 = {S14, 0, &V21777_292_57183, &P21777_291};	/*Property = CssMinMax*/
const Value V21777_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21777_293 = {S15, 0, &V21777_293_57187, &P21777_292};	/*Property = CssOverflow*/
const Value V21777_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21777_294 = {S16, 0, &V21777_294_57190, &P21777_293};	/*Property = CssPosition*/
const Value V21777_295_57193 = {S0, NULL};	/*Value = False*/
const Property P21777_295 = {S17, 0, &V21777_295_57193, &P21777_294};	/*Property = CssText*/
const Value V21777_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21777_297 = {S18, 0, &V21777_297_57195, &P21777_295};	/*Property = CssTransforms*/
const Value V21777_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21777_241 = {S19, 0, &V21777_241_55799, &P21777_297};	/*Property = CssTransitions*/
const Value V21777_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21777_298 = {S20, 0, &V21777_298_57206, &P21777_241};	/*Property = CssUI*/
const Value V21777_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21777_300 = {S21, 0, &V21777_300_57216, &P21777_298};	/*Property = DataSet*/
const Value V21777_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21777_301 = {S22, 0, &V21777_301_57218, &P21777_300};	/*Property = DataUrl*/
const Value V21777_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21777_302 = {S23, 0, &V21777_302_57222, &P21777_301};	/*Property = DeviceOrientation*/
const Value V21777_304_57228 = {S0, NULL};	/*Value = False*/
const Property P21777_304 = {S24, 0, &V21777_304_57228, &P21777_302};	/*Property = FileReader*/
const Value V21777_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21777_305 = {S25, 0, &V21777_305_57231, &P21777_304};	/*Property = FileSaver*/
const Value V21777_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21777_306 = {S26, 0, &V21777_306_57237, &P21777_305};	/*Property = FileWriter*/
const Value V21777_307_57240 = {S0, NULL};	/*Value = False*/
const Property P21777_307 = {S27, 0, &V21777_307_57240, &P21777_306};	/*Property = FormData*/
const Value V21777_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21777_309 = {S28, 0, &V21777_309_57246, &P21777_307};	/*Property = Fullscreen*/
const Value V21777_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21777_310 = {S29, 0, &V21777_310_57248, &P21777_309};	/*Property = GeoLocation*/
const Value V21777_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21777_312 = {S30, 0, &V21777_312_57255, &P21777_310};	/*Property = History*/
const Value V21777_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21777_314 = {S31, 0, &V21777_314_57261, &P21777_312};	/*Property = Html5*/
const Value V21777_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21777_313 = {S32, 0, &V21777_313_57258, &P21777_314};	/*Property = Html-Media-Capture*/
const Value V21777_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21777_315 = {S33, 0, &V21777_315_57264, &P21777_313};	/*Property = Iframe*/
const Value V21777_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21777_316 = {S34, 0, &V21777_316_57267, &P21777_315};	/*Property = IndexedDB*/
const Value V21777_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21777_317 = {S35, 0, &V21777_317_57269, &P21777_316};	/*Property = Json*/
const Value V21777_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P21777_15 = {S37, 0, &V21777_15_30, &P21777_317};	/*Property = LayoutEngine*/
const Value V21777_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21777_344 = {S38, 0, &V21777_344_57272, &P21777_15};	/*Property = Masking*/
const Value V21777_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21777_320 = {S39, 0, &V21777_320_57290, &P21777_344};	/*Property = PostMessage*/
const Value V21777_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21777_321 = {S40, 0, &V21777_321_57293, &P21777_320};	/*Property = Progress*/
const Value V21777_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21777_322 = {S41, 0, &V21777_322_57296, &P21777_321};	/*Property = Prompts*/
const Value V21777_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21777_323 = {S42, 0, &V21777_323_57299, &P21777_322};	/*Property = Selector*/
const Value V21777_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21777_324 = {S43, 0, &V21777_324_57306, &P21777_323};	/*Property = Svg*/
const Value V21777_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21777_327 = {S44, 0, &V21777_327_57315, &P21777_324};	/*Property = TouchEvents*/
const Value V21777_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21777_328 = {S45, 0, &V21777_328_57318, &P21777_327};	/*Property = Track*/
const Value V21777_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21777_329 = {S46, 0, &V21777_329_57323, &P21777_328};	/*Property = Video*/
const Value V21777_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21777_330 = {S47, 0, &V21777_330_57326, &P21777_329};	/*Property = Viewport*/
const Value V21777_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21777_332 = {S48, 0, &V21777_332_57339, &P21777_330};	/*Property = WebWorkers*/
const Value V21777_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21777_333 = {S49, 0, &V21777_333_57342, &P21777_332};	/*Property = Xhr2*/
const Profile P21777 = {"21777", &P21777_333};	/*Profile ID - 21777*/
const Value V20926_1_1 = {S2, NULL};	/*Value = True*/
const Property P20926_1 = {S50, 0, &V20926_1_1, NULL};	/*Property = IsMobile*/
const Value V20926_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20926_5 = {S52, 0, &V20926_5_1227, &P20926_1};	/*Property = ScreenPixelsHeight*/
const Value V20926_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20926_4 = {S54, 0, &V20926_4_1229, &P20926_5};	/*Property = ScreenPixelsWidth*/
const Profile P20926 = {"20926", &P20926_4};	/*Profile ID - 20926*/
const Value V21373_1_1 = {S2, NULL};	/*Value = True*/
const Property P21373_1 = {S50, 0, &V21373_1_1, NULL};	/*Property = IsMobile*/
const Value V21373_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P21373_5 = {S52, 0, &V21373_5_59138, &P21373_1};	/*Property = ScreenPixelsHeight*/
const Value V21373_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P21373_4 = {S54, 0, &V21373_4_59970, &P21373_5};	/*Property = ScreenPixelsWidth*/
const Profile P21373 = {"21373", &P21373_4};	/*Profile ID - 21373*/
const Value V21711_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_279 = {S1, 0, &V21711_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21711_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_282 = {S3, 0, &V21711_282_57155, &P21711_279};	/*Property = BlobBuilder*/
const Value V21711_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_350 = {S4, 0, &V21711_350_57446, &P21711_282};	/*Property = Canvas*/
const Value V21711_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_283 = {S5, 0, &V21711_283_57161, &P21711_350};	/*Property = CssBackground*/
const Value V21711_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_284 = {S6, 0, &V21711_284_57164, &P21711_283};	/*Property = CssBorderImage*/
const Value V21711_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_285 = {S7, 0, &V21711_285_57167, &P21711_284};	/*Property = CssCanvas*/
const Value V21711_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_286 = {S8, 0, &V21711_286_57170, &P21711_285};	/*Property = CssColor*/
const Value V21711_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_237 = {S9, 0, &V21711_237_55786, &P21711_286};	/*Property = CssColumn*/
const Value V21711_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_287 = {S10, 0, &V21711_287_57173, &P21711_237};	/*Property = CssFlexbox*/
const Value V21711_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_288 = {S11, 0, &V21711_288_57176, &P21711_287};	/*Property = CssFont*/
const Value V21711_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_289 = {S12, 0, &V21711_289_86899, &P21711_288};	/*Property = CssImages*/
const Value V21711_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_291 = {S13, 0, &V21711_291_57182, &P21711_289};	/*Property = CssMediaQueries*/
const Value V21711_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_292 = {S14, 0, &V21711_292_57185, &P21711_291};	/*Property = CssMinMax*/
const Value V21711_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_293 = {S15, 0, &V21711_293_57188, &P21711_292};	/*Property = CssOverflow*/
const Value V21711_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_294 = {S16, 0, &V21711_294_57191, &P21711_293};	/*Property = CssPosition*/
const Value V21711_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_295 = {S17, 0, &V21711_295_57194, &P21711_294};	/*Property = CssText*/
const Value V21711_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_297 = {S18, 0, &V21711_297_57197, &P21711_295};	/*Property = CssTransforms*/
const Value V21711_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_241 = {S19, 0, &V21711_241_55801, &P21711_297};	/*Property = CssTransitions*/
const Value V21711_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_298 = {S20, 0, &V21711_298_57208, &P21711_241};	/*Property = CssUI*/
const Value V21711_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_300 = {S21, 0, &V21711_300_57217, &P21711_298};	/*Property = DataSet*/
const Value V21711_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_301 = {S22, 0, &V21711_301_57220, &P21711_300};	/*Property = DataUrl*/
const Value V21711_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_302 = {S23, 0, &V21711_302_57223, &P21711_301};	/*Property = DeviceOrientation*/
const Value V21711_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_304 = {S24, 0, &V21711_304_57229, &P21711_302};	/*Property = FileReader*/
const Value V21711_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_305 = {S25, 0, &V21711_305_57232, &P21711_304};	/*Property = FileSaver*/
const Value V21711_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_306 = {S26, 0, &V21711_306_57238, &P21711_305};	/*Property = FileWriter*/
const Value V21711_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_307 = {S27, 0, &V21711_307_57241, &P21711_306};	/*Property = FormData*/
const Value V21711_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_309 = {S28, 0, &V21711_309_57247, &P21711_307};	/*Property = Fullscreen*/
const Value V21711_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_310 = {S29, 0, &V21711_310_57250, &P21711_309};	/*Property = GeoLocation*/
const Value V21711_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_312 = {S30, 0, &V21711_312_57256, &P21711_310};	/*Property = History*/
const Value V21711_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_314 = {S31, 0, &V21711_314_57262, &P21711_312};	/*Property = Html5*/
const Value V21711_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_313 = {S32, 0, &V21711_313_57259, &P21711_314};	/*Property = Html-Media-Capture*/
const Value V21711_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_315 = {S33, 0, &V21711_315_57265, &P21711_313};	/*Property = Iframe*/
const Value V21711_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_316 = {S34, 0, &V21711_316_57268, &P21711_315};	/*Property = IndexedDB*/
const Value V21711_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_317 = {S35, 0, &V21711_317_57271, &P21711_316};	/*Property = Json*/
const Value V21711_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P21711_15 = {S37, 0, &V21711_15_29, &P21711_317};	/*Property = LayoutEngine*/
const Value V21711_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_344 = {S38, 0, &V21711_344_57274, &P21711_15};	/*Property = Masking*/
const Value V21711_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_320 = {S39, 0, &V21711_320_57292, &P21711_344};	/*Property = PostMessage*/
const Value V21711_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_321 = {S40, 0, &V21711_321_57295, &P21711_320};	/*Property = Progress*/
const Value V21711_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_322 = {S41, 0, &V21711_322_57298, &P21711_321};	/*Property = Prompts*/
const Value V21711_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_323 = {S42, 0, &V21711_323_57301, &P21711_322};	/*Property = Selector*/
const Value V21711_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_324 = {S43, 0, &V21711_324_57307, &P21711_323};	/*Property = Svg*/
const Value V21711_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_327 = {S44, 0, &V21711_327_57316, &P21711_324};	/*Property = TouchEvents*/
const Value V21711_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_328 = {S45, 0, &V21711_328_57319, &P21711_327};	/*Property = Track*/
const Value V21711_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_329 = {S46, 0, &V21711_329_57325, &P21711_328};	/*Property = Video*/
const Value V21711_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_330 = {S47, 0, &V21711_330_57328, &P21711_329};	/*Property = Viewport*/
const Value V21711_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_332 = {S48, 0, &V21711_332_57340, &P21711_330};	/*Property = WebWorkers*/
const Value V21711_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21711_333 = {S49, 0, &V21711_333_57343, &P21711_332};	/*Property = Xhr2*/
const Profile P21711 = {"21711", &P21711_333};	/*Profile ID - 21711*/
const Value V23850_1_1 = {S2, NULL};	/*Value = True*/
const Property P23850_1 = {S50, 0, &V23850_1_1, NULL};	/*Property = IsMobile*/
const Value V23850_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23850_5 = {S52, 0, &V23850_5_1227, &P23850_1};	/*Property = ScreenPixelsHeight*/
const Value V23850_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23850_4 = {S54, 0, &V23850_4_1229, &P23850_5};	/*Property = ScreenPixelsWidth*/
const Profile P23850 = {"23850", &P23850_4};	/*Profile ID - 23850*/
const Value V21969_1_1 = {S2, NULL};	/*Value = True*/
const Property P21969_1 = {S50, 0, &V21969_1_1, NULL};	/*Property = IsMobile*/
const Value V21969_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21969_5 = {S52, 0, &V21969_5_1277, &P21969_1};	/*Property = ScreenPixelsHeight*/
const Value V21969_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21969_4 = {S54, 0, &V21969_4_2336, &P21969_5};	/*Property = ScreenPixelsWidth*/
const Profile P21969 = {"21969", &P21969_4};	/*Profile ID - 21969*/
const Value V23072_1_1 = {S2, NULL};	/*Value = True*/
const Property P23072_1 = {S50, 0, &V23072_1_1, NULL};	/*Property = IsMobile*/
const Value V23072_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23072_5 = {S52, 0, &V23072_5_1277, &P23072_1};	/*Property = ScreenPixelsHeight*/
const Value V23072_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23072_4 = {S54, 0, &V23072_4_2336, &P23072_5};	/*Property = ScreenPixelsWidth*/
const Profile P23072 = {"23072", &P23072_4};	/*Profile ID - 23072*/
const Value V21525_1_1 = {S2, NULL};	/*Value = True*/
const Property P21525_1 = {S50, 0, &V21525_1_1, NULL};	/*Property = IsMobile*/
const Value V21525_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21525_5 = {S52, 0, &V21525_5_9981, &P21525_1};	/*Property = ScreenPixelsHeight*/
const Value V21525_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21525_4 = {S54, 0, &V21525_4_39098, &P21525_5};	/*Property = ScreenPixelsWidth*/
const Profile P21525 = {"21525", &P21525_4};	/*Profile ID - 21525*/
const Value V23205_1_1 = {S2, NULL};	/*Value = True*/
const Property P23205_1 = {S50, 0, &V23205_1_1, NULL};	/*Property = IsMobile*/
const Value V23205_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23205_5 = {S52, 0, &V23205_5_1227, &P23205_1};	/*Property = ScreenPixelsHeight*/
const Value V23205_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23205_4 = {S54, 0, &V23205_4_1229, &P23205_5};	/*Property = ScreenPixelsWidth*/
const Profile P23205 = {"23205", &P23205_4};	/*Profile ID - 23205*/
const Value V22132_1_1 = {S2, NULL};	/*Value = True*/
const Property P22132_1 = {S50, 0, &V22132_1_1, NULL};	/*Property = IsMobile*/
const Value V22132_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22132_5 = {S52, 0, &V22132_5_1227, &P22132_1};	/*Property = ScreenPixelsHeight*/
const Value V22132_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22132_4 = {S54, 0, &V22132_4_1229, &P22132_5};	/*Property = ScreenPixelsWidth*/
const Profile P22132 = {"22132", &P22132_4};	/*Profile ID - 22132*/
const Value V23658_1_2 = {S0, NULL};	/*Value = False*/
const Property P23658_1 = {S50, 0, &V23658_1_2, NULL};	/*Property = IsMobile*/
const Value V23658_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23658_5 = {S52, 0, &V23658_5_60417, &P23658_1};	/*Property = ScreenPixelsHeight*/
const Value V23658_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23658_4 = {S54, 0, &V23658_4_57095, &P23658_5};	/*Property = ScreenPixelsWidth*/
const Profile P23658 = {"23658", &P23658_4};	/*Profile ID - 23658*/
const Value V21654_1_1 = {S2, NULL};	/*Value = True*/
const Property P21654_1 = {S50, 0, &V21654_1_1, NULL};	/*Property = IsMobile*/
const Value V21654_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21654_5 = {S52, 0, &V21654_5_1458, &P21654_1};	/*Property = ScreenPixelsHeight*/
const Value V21654_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21654_4 = {S54, 0, &V21654_4_37617, &P21654_5};	/*Property = ScreenPixelsWidth*/
const Profile P21654 = {"21654", &P21654_4};	/*Profile ID - 21654*/
const Value V21984_1_1 = {S2, NULL};	/*Value = True*/
const Property P21984_1 = {S50, 0, &V21984_1_1, NULL};	/*Property = IsMobile*/
const Value V21984_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21984_5 = {S52, 0, &V21984_5_1227, &P21984_1};	/*Property = ScreenPixelsHeight*/
const Value V21984_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21984_4 = {S54, 0, &V21984_4_1229, &P21984_5};	/*Property = ScreenPixelsWidth*/
const Profile P21984 = {"21984", &P21984_4};	/*Profile ID - 21984*/
const Value V22059_1_1 = {S2, NULL};	/*Value = True*/
const Property P22059_1 = {S50, 0, &V22059_1_1, NULL};	/*Property = IsMobile*/
const Value V22059_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22059_5 = {S52, 0, &V22059_5_1277, &P22059_1};	/*Property = ScreenPixelsHeight*/
const Value V22059_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22059_4 = {S54, 0, &V22059_4_1279, &P22059_5};	/*Property = ScreenPixelsWidth*/
const Profile P22059 = {"22059", &P22059_4};	/*Profile ID - 22059*/
const Value V23037_1_1 = {S2, NULL};	/*Value = True*/
const Property P23037_1 = {S50, 0, &V23037_1_1, NULL};	/*Property = IsMobile*/
const Value V23037_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23037_5 = {S52, 0, &V23037_5_1222, &P23037_1};	/*Property = ScreenPixelsHeight*/
const Value V23037_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23037_4 = {S54, 0, &V23037_4_1224, &P23037_5};	/*Property = ScreenPixelsWidth*/
const Profile P23037 = {"23037", &P23037_4};	/*Profile ID - 23037*/
const Value V21868_1_1 = {S2, NULL};	/*Value = True*/
const Property P21868_1 = {S50, 0, &V21868_1_1, NULL};	/*Property = IsMobile*/
const Value V21868_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21868_5 = {S52, 0, &V21868_5_1458, &P21868_1};	/*Property = ScreenPixelsHeight*/
const Value V21868_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P21868_4 = {S54, 0, &V21868_4_37617, &P21868_5};	/*Property = ScreenPixelsWidth*/
const Profile P21868 = {"21868", &P21868_4};	/*Profile ID - 21868*/
const Value V23460_1_1 = {S2, NULL};	/*Value = True*/
const Property P23460_1 = {S50, 0, &V23460_1_1, NULL};	/*Property = IsMobile*/
const Value V23460_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23460_5 = {S52, 0, &V23460_5_1227, &P23460_1};	/*Property = ScreenPixelsHeight*/
const Value V23460_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23460_4 = {S54, 0, &V23460_4_1229, &P23460_5};	/*Property = ScreenPixelsWidth*/
const Profile P23460 = {"23460", &P23460_4};	/*Profile ID - 23460*/
const Value V23412_1_1 = {S2, NULL};	/*Value = True*/
const Property P23412_1 = {S50, 0, &V23412_1_1, NULL};	/*Property = IsMobile*/
const Value V23412_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23412_5 = {S52, 0, &V23412_5_37569, &P23412_1};	/*Property = ScreenPixelsHeight*/
const Value V23412_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23412_4 = {S54, 0, &V23412_4_5645, &P23412_5};	/*Property = ScreenPixelsWidth*/
const Profile P23412 = {"23412", &P23412_4};	/*Profile ID - 23412*/
const Value V23534_1_1 = {S2, NULL};	/*Value = True*/
const Property P23534_1 = {S50, 0, &V23534_1_1, NULL};	/*Property = IsMobile*/
const Value V23534_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23534_5 = {S52, 0, &V23534_5_37, &P23534_1};	/*Property = ScreenPixelsHeight*/
const Value V23534_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23534_4 = {S54, 0, &V23534_4_36, &P23534_5};	/*Property = ScreenPixelsWidth*/
const Profile P23534 = {"23534", &P23534_4};	/*Profile ID - 23534*/
const Value V21518_1_1 = {S2, NULL};	/*Value = True*/
const Property P21518_1 = {S50, 0, &V21518_1_1, NULL};	/*Property = IsMobile*/
const Value V21518_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21518_5 = {S52, 0, &V21518_5_1222, &P21518_1};	/*Property = ScreenPixelsHeight*/
const Value V21518_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21518_4 = {S54, 0, &V21518_4_1224, &P21518_5};	/*Property = ScreenPixelsWidth*/
const Profile P21518 = {"21518", &P21518_4};	/*Profile ID - 21518*/
const Value V22788_1_1 = {S2, NULL};	/*Value = True*/
const Property P22788_1 = {S50, 0, &V22788_1_1, NULL};	/*Property = IsMobile*/
const Value V22788_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P22788_5 = {S52, 0, &V22788_5_1275, &P22788_1};	/*Property = ScreenPixelsHeight*/
const Value V22788_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22788_4 = {S54, 0, &V22788_4_1229, &P22788_5};	/*Property = ScreenPixelsWidth*/
const Profile P22788 = {"22788", &P22788_4};	/*Profile ID - 22788*/
const Profile P22831 = {"22831", NULL};	/*Profile ID - 22831*/
const Value V21493_1_1 = {S2, NULL};	/*Value = True*/
const Property P21493_1 = {S50, 0, &V21493_1_1, NULL};	/*Property = IsMobile*/
const Value V21493_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21493_5 = {S52, 0, &V21493_5_9981, &P21493_1};	/*Property = ScreenPixelsHeight*/
const Value V21493_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21493_4 = {S54, 0, &V21493_4_39098, &P21493_5};	/*Property = ScreenPixelsWidth*/
const Profile P21493 = {"21493", &P21493_4};	/*Profile ID - 21493*/
const Value V21600_1_1 = {S2, NULL};	/*Value = True*/
const Property P21600_1 = {S50, 0, &V21600_1_1, NULL};	/*Property = IsMobile*/
const Value V21600_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21600_5 = {S52, 0, &V21600_5_1269, &P21600_1};	/*Property = ScreenPixelsHeight*/
const Value V21600_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21600_4 = {S54, 0, &V21600_4_1229, &P21600_5};	/*Property = ScreenPixelsWidth*/
const Profile P21600 = {"21600", &P21600_4};	/*Profile ID - 21600*/
const Value V22789_1_1 = {S2, NULL};	/*Value = True*/
const Property P22789_1 = {S50, 0, &V22789_1_1, NULL};	/*Property = IsMobile*/
const Value V22789_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22789_5 = {S52, 0, &V22789_5_1222, &P22789_1};	/*Property = ScreenPixelsHeight*/
const Value V22789_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22789_4 = {S54, 0, &V22789_4_1224, &P22789_5};	/*Property = ScreenPixelsWidth*/
const Profile P22789 = {"22789", &P22789_4};	/*Profile ID - 22789*/
const Value V21848_1_1 = {S2, NULL};	/*Value = True*/
const Property P21848_1 = {S50, 0, &V21848_1_1, NULL};	/*Property = IsMobile*/
const Value V21848_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21848_5 = {S52, 0, &V21848_5_1227, &P21848_1};	/*Property = ScreenPixelsHeight*/
const Value V21848_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P21848_4 = {S54, 0, &V21848_4_1477, &P21848_5};	/*Property = ScreenPixelsWidth*/
const Profile P21848 = {"21848", &P21848_4};	/*Profile ID - 21848*/
const Value V23862_1_1 = {S2, NULL};	/*Value = True*/
const Property P23862_1 = {S50, 0, &V23862_1_1, NULL};	/*Property = IsMobile*/
const Value V23862_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23862_5 = {S52, 0, &V23862_5_1403, &P23862_1};	/*Property = ScreenPixelsHeight*/
const Value V23862_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23862_4 = {S54, 0, &V23862_4_1404, &P23862_5};	/*Property = ScreenPixelsWidth*/
const Profile P23862 = {"23862", &P23862_4};	/*Profile ID - 23862*/
const Value V21903_1_1 = {S2, NULL};	/*Value = True*/
const Property P21903_1 = {S50, 0, &V21903_1_1, NULL};	/*Property = IsMobile*/
const Value V21903_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21903_5 = {S52, 0, &V21903_5_1403, &P21903_1};	/*Property = ScreenPixelsHeight*/
const Value V21903_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21903_4 = {S54, 0, &V21903_4_1404, &P21903_5};	/*Property = ScreenPixelsWidth*/
const Profile P21903 = {"21903", &P21903_4};	/*Profile ID - 21903*/
const Value V21944_1_1 = {S2, NULL};	/*Value = True*/
const Property P21944_1 = {S50, 0, &V21944_1_1, NULL};	/*Property = IsMobile*/
const Value V21944_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21944_5 = {S52, 0, &V21944_5_1227, &P21944_1};	/*Property = ScreenPixelsHeight*/
const Value V21944_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21944_4 = {S54, 0, &V21944_4_1229, &P21944_5};	/*Property = ScreenPixelsWidth*/
const Profile P21944 = {"21944", &P21944_4};	/*Profile ID - 21944*/
const Value V23472_1_1 = {S2, NULL};	/*Value = True*/
const Property P23472_1 = {S50, 0, &V23472_1_1, NULL};	/*Property = IsMobile*/
const Value V23472_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23472_5 = {S52, 0, &V23472_5_1458, &P23472_1};	/*Property = ScreenPixelsHeight*/
const Value V23472_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23472_4 = {S54, 0, &V23472_4_1460, &P23472_5};	/*Property = ScreenPixelsWidth*/
const Profile P23472 = {"23472", &P23472_4};	/*Profile ID - 23472*/
const Value V23555_1_1 = {S2, NULL};	/*Value = True*/
const Property P23555_1 = {S50, 0, &V23555_1_1, NULL};	/*Property = IsMobile*/
const Value V23555_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23555_5 = {S52, 0, &V23555_5_1227, &P23555_1};	/*Property = ScreenPixelsHeight*/
const Value V23555_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23555_4 = {S54, 0, &V23555_4_1229, &P23555_5};	/*Property = ScreenPixelsWidth*/
const Profile P23555 = {"23555", &P23555_4};	/*Profile ID - 23555*/
const Value V23822_1_1 = {S2, NULL};	/*Value = True*/
const Property P23822_1 = {S50, 0, &V23822_1_1, NULL};	/*Property = IsMobile*/
const Value V23822_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23822_5 = {S52, 0, &V23822_5_1277, &P23822_1};	/*Property = ScreenPixelsHeight*/
const Value V23822_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23822_4 = {S54, 0, &V23822_4_1279, &P23822_5};	/*Property = ScreenPixelsWidth*/
const Profile P23822 = {"23822", &P23822_4};	/*Profile ID - 23822*/
const Value V23794_1_1 = {S2, NULL};	/*Value = True*/
const Property P23794_1 = {S50, 0, &V23794_1_1, NULL};	/*Property = IsMobile*/
const Value V23794_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23794_5 = {S52, 0, &V23794_5_1227, &P23794_1};	/*Property = ScreenPixelsHeight*/
const Value V23794_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23794_4 = {S54, 0, &V23794_4_1229, &P23794_5};	/*Property = ScreenPixelsWidth*/
const Profile P23794 = {"23794", &P23794_4};	/*Profile ID - 23794*/
const Value V21496_1_1 = {S2, NULL};	/*Value = True*/
const Property P21496_1 = {S50, 0, &V21496_1_1, NULL};	/*Property = IsMobile*/
const Value V21496_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21496_5 = {S52, 0, &V21496_5_1227, &P21496_1};	/*Property = ScreenPixelsHeight*/
const Value V21496_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21496_4 = {S54, 0, &V21496_4_1229, &P21496_5};	/*Property = ScreenPixelsWidth*/
const Profile P21496 = {"21496", &P21496_4};	/*Profile ID - 21496*/
const Value V23189_1_1 = {S2, NULL};	/*Value = True*/
const Property P23189_1 = {S50, 0, &V23189_1_1, NULL};	/*Property = IsMobile*/
const Value V23189_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23189_5 = {S52, 0, &V23189_5_1458, &P23189_1};	/*Property = ScreenPixelsHeight*/
const Value V23189_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23189_4 = {S54, 0, &V23189_4_37617, &P23189_5};	/*Property = ScreenPixelsWidth*/
const Profile P23189 = {"23189", &P23189_4};	/*Profile ID - 23189*/
const Value V21656_1_2 = {S0, NULL};	/*Value = False*/
const Property P21656_1 = {S50, 0, &V21656_1_2, NULL};	/*Property = IsMobile*/
const Value V21656_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21656_5 = {S52, 0, &V21656_5_37, &P21656_1};	/*Property = ScreenPixelsHeight*/
const Value V21656_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21656_4 = {S54, 0, &V21656_4_36, &P21656_5};	/*Property = ScreenPixelsWidth*/
const Profile P21656 = {"21656", &P21656_4};	/*Profile ID - 21656*/
const Value V23500_1_1 = {S2, NULL};	/*Value = True*/
const Property P23500_1 = {S50, 0, &V23500_1_1, NULL};	/*Property = IsMobile*/
const Value V23500_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23500_5 = {S52, 0, &V23500_5_1222, &P23500_1};	/*Property = ScreenPixelsHeight*/
const Value V23500_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23500_4 = {S54, 0, &V23500_4_1224, &P23500_5};	/*Property = ScreenPixelsWidth*/
const Profile P23500 = {"23500", &P23500_4};	/*Profile ID - 23500*/
const Value V24037_1_1 = {S2, NULL};	/*Value = True*/
const Property P24037_1 = {S50, 0, &V24037_1_1, NULL};	/*Property = IsMobile*/
const Value V24037_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P24037_5 = {S52, 0, &V24037_5_37569, &P24037_1};	/*Property = ScreenPixelsHeight*/
const Value V24037_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P24037_4 = {S54, 0, &V24037_4_5645, &P24037_5};	/*Property = ScreenPixelsWidth*/
const Profile P24037 = {"24037", &P24037_4};	/*Profile ID - 24037*/
const Value V21185_1_1 = {S2, NULL};	/*Value = True*/
const Property P21185_1 = {S50, 0, &V21185_1_1, NULL};	/*Property = IsMobile*/
const Value V21185_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21185_5 = {S52, 0, &V21185_5_37, &P21185_1};	/*Property = ScreenPixelsHeight*/
const Value V21185_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21185_4 = {S54, 0, &V21185_4_36, &P21185_5};	/*Property = ScreenPixelsWidth*/
const Profile P21185 = {"21185", &P21185_4};	/*Profile ID - 21185*/
const Value V21893_1_1 = {S2, NULL};	/*Value = True*/
const Property P21893_1 = {S50, 0, &V21893_1_1, NULL};	/*Property = IsMobile*/
const Value V21893_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21893_5 = {S52, 0, &V21893_5_37, &P21893_1};	/*Property = ScreenPixelsHeight*/
const Value V21893_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21893_4 = {S54, 0, &V21893_4_36, &P21893_5};	/*Property = ScreenPixelsWidth*/
const Profile P21893 = {"21893", &P21893_4};	/*Profile ID - 21893*/
const Value V21506_1_1 = {S2, NULL};	/*Value = True*/
const Property P21506_1 = {S50, 0, &V21506_1_1, NULL};	/*Property = IsMobile*/
const Value V21506_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21506_5 = {S52, 0, &V21506_5_1227, &P21506_1};	/*Property = ScreenPixelsHeight*/
const Value V21506_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21506_4 = {S54, 0, &V21506_4_1229, &P21506_5};	/*Property = ScreenPixelsWidth*/
const Profile P21506 = {"21506", &P21506_4};	/*Profile ID - 21506*/
const Value V21878_1_1 = {S2, NULL};	/*Value = True*/
const Property P21878_1 = {S50, 0, &V21878_1_1, NULL};	/*Property = IsMobile*/
const Value V21878_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21878_5 = {S52, 0, &V21878_5_9981, &P21878_1};	/*Property = ScreenPixelsHeight*/
const Value V21878_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21878_4 = {S54, 0, &V21878_4_39098, &P21878_5};	/*Property = ScreenPixelsWidth*/
const Profile P21878 = {"21878", &P21878_4};	/*Profile ID - 21878*/
const Value V20491_1_1 = {S2, NULL};	/*Value = True*/
const Property P20491_1 = {S50, 0, &V20491_1_1, NULL};	/*Property = IsMobile*/
const Value V20491_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P20491_5 = {S52, 0, &V20491_5_1269, &P20491_1};	/*Property = ScreenPixelsHeight*/
const Value V20491_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20491_4 = {S54, 0, &V20491_4_1229, &P20491_5};	/*Property = ScreenPixelsWidth*/
const Profile P20491 = {"20491", &P20491_4};	/*Profile ID - 20491*/
const Value V21498_1_1 = {S2, NULL};	/*Value = True*/
const Property P21498_1 = {S50, 0, &V21498_1_1, NULL};	/*Property = IsMobile*/
const Value V21498_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21498_5 = {S52, 0, &V21498_5_1403, &P21498_1};	/*Property = ScreenPixelsHeight*/
const Value V21498_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P21498_4 = {S54, 0, &V21498_4_8554, &P21498_5};	/*Property = ScreenPixelsWidth*/
const Profile P21498 = {"21498", &P21498_4};	/*Profile ID - 21498*/
const Value V23879_1_1 = {S2, NULL};	/*Value = True*/
const Property P23879_1 = {S50, 0, &V23879_1_1, NULL};	/*Property = IsMobile*/
const Value V23879_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23879_5 = {S52, 0, &V23879_5_1227, &P23879_1};	/*Property = ScreenPixelsHeight*/
const Value V23879_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23879_4 = {S54, 0, &V23879_4_1229, &P23879_5};	/*Property = ScreenPixelsWidth*/
const Profile P23879 = {"23879", &P23879_4};	/*Profile ID - 23879*/
const Value V22647_1_1 = {S2, NULL};	/*Value = True*/
const Property P22647_1 = {S50, 0, &V22647_1_1, NULL};	/*Property = IsMobile*/
const Value V22647_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22647_5 = {S52, 0, &V22647_5_1227, &P22647_1};	/*Property = ScreenPixelsHeight*/
const Value V22647_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22647_4 = {S54, 0, &V22647_4_1229, &P22647_5};	/*Property = ScreenPixelsWidth*/
const Profile P22647 = {"22647", &P22647_4};	/*Profile ID - 22647*/
const Value V21739_1_1 = {S2, NULL};	/*Value = True*/
const Property P21739_1 = {S50, 0, &V21739_1_1, NULL};	/*Property = IsMobile*/
const Value V21739_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21739_5 = {S52, 0, &V21739_5_1403, &P21739_1};	/*Property = ScreenPixelsHeight*/
const Value V21739_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21739_4 = {S54, 0, &V21739_4_1404, &P21739_5};	/*Property = ScreenPixelsWidth*/
const Profile P21739 = {"21739", &P21739_4};	/*Profile ID - 21739*/
const Value V22662_1_1 = {S2, NULL};	/*Value = True*/
const Property P22662_1 = {S50, 0, &V22662_1_1, NULL};	/*Property = IsMobile*/
const Value V22662_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22662_5 = {S52, 0, &V22662_5_1458, &P22662_1};	/*Property = ScreenPixelsHeight*/
const Value V22662_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22662_4 = {S54, 0, &V22662_4_1460, &P22662_5};	/*Property = ScreenPixelsWidth*/
const Profile P22662 = {"22662", &P22662_4};	/*Profile ID - 22662*/
const Value V22992_1_1 = {S2, NULL};	/*Value = True*/
const Property P22992_1 = {S50, 0, &V22992_1_1, NULL};	/*Property = IsMobile*/
const Value V22992_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22992_5 = {S52, 0, &V22992_5_1227, &P22992_1};	/*Property = ScreenPixelsHeight*/
const Value V22992_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22992_4 = {S54, 0, &V22992_4_1229, &P22992_5};	/*Property = ScreenPixelsWidth*/
const Profile P22992 = {"22992", &P22992_4};	/*Profile ID - 22992*/
const Value V23388_1_1 = {S2, NULL};	/*Value = True*/
const Property P23388_1 = {S50, 0, &V23388_1_1, NULL};	/*Property = IsMobile*/
const Value V23388_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23388_5 = {S52, 0, &V23388_5_37, &P23388_1};	/*Property = ScreenPixelsHeight*/
const Value V23388_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23388_4 = {S54, 0, &V23388_4_36, &P23388_5};	/*Property = ScreenPixelsWidth*/
const Profile P23388 = {"23388", &P23388_4};	/*Profile ID - 23388*/
const Value V24139_1_1 = {S2, NULL};	/*Value = True*/
const Property P24139_1 = {S50, 0, &V24139_1_1, NULL};	/*Property = IsMobile*/
const Value V24139_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P24139_5 = {S52, 0, &V24139_5_1458, &P24139_1};	/*Property = ScreenPixelsHeight*/
const Value V24139_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P24139_4 = {S54, 0, &V24139_4_37617, &P24139_5};	/*Property = ScreenPixelsWidth*/
const Profile P24139 = {"24139", &P24139_4};	/*Profile ID - 24139*/
const Value V23301_1_1 = {S2, NULL};	/*Value = True*/
const Property P23301_1 = {S50, 0, &V23301_1_1, NULL};	/*Property = IsMobile*/
const Value V23301_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23301_5 = {S52, 0, &V23301_5_1458, &P23301_1};	/*Property = ScreenPixelsHeight*/
const Value V23301_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23301_4 = {S54, 0, &V23301_4_1460, &P23301_5};	/*Property = ScreenPixelsWidth*/
const Profile P23301 = {"23301", &P23301_4};	/*Profile ID - 23301*/
const Value V23016_1_1 = {S2, NULL};	/*Value = True*/
const Property P23016_1 = {S50, 0, &V23016_1_1, NULL};	/*Property = IsMobile*/
const Value V23016_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23016_5 = {S52, 0, &V23016_5_1277, &P23016_1};	/*Property = ScreenPixelsHeight*/
const Value V23016_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23016_4 = {S54, 0, &V23016_4_1279, &P23016_5};	/*Property = ScreenPixelsWidth*/
const Profile P23016 = {"23016", &P23016_4};	/*Profile ID - 23016*/
const Value V21857_1_1 = {S2, NULL};	/*Value = True*/
const Property P21857_1 = {S50, 0, &V21857_1_1, NULL};	/*Property = IsMobile*/
const Value V21857_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21857_5 = {S52, 0, &V21857_5_1227, &P21857_1};	/*Property = ScreenPixelsHeight*/
const Value V21857_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21857_4 = {S54, 0, &V21857_4_1229, &P21857_5};	/*Property = ScreenPixelsWidth*/
const Profile P21857 = {"21857", &P21857_4};	/*Profile ID - 21857*/
const Value V21883_1_1 = {S2, NULL};	/*Value = True*/
const Property P21883_1 = {S50, 0, &V21883_1_1, NULL};	/*Property = IsMobile*/
const Value V21883_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21883_5 = {S52, 0, &V21883_5_1458, &P21883_1};	/*Property = ScreenPixelsHeight*/
const Value V21883_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21883_4 = {S54, 0, &V21883_4_1460, &P21883_5};	/*Property = ScreenPixelsWidth*/
const Profile P21883 = {"21883", &P21883_4};	/*Profile ID - 21883*/
const Value V22057_1_1 = {S2, NULL};	/*Value = True*/
const Property P22057_1 = {S50, 0, &V22057_1_1, NULL};	/*Property = IsMobile*/
const Value V22057_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22057_5 = {S52, 0, &V22057_5_1832, &P22057_1};	/*Property = ScreenPixelsHeight*/
const Value V22057_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22057_4 = {S54, 0, &V22057_4_5645, &P22057_5};	/*Property = ScreenPixelsWidth*/
const Profile P22057 = {"22057", &P22057_4};	/*Profile ID - 22057*/
const Value V22628_1_1 = {S2, NULL};	/*Value = True*/
const Property P22628_1 = {S50, 0, &V22628_1_1, NULL};	/*Property = IsMobile*/
const Value V22628_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22628_5 = {S52, 0, &V22628_5_1227, &P22628_1};	/*Property = ScreenPixelsHeight*/
const Value V22628_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22628_4 = {S54, 0, &V22628_4_1229, &P22628_5};	/*Property = ScreenPixelsWidth*/
const Profile P22628 = {"22628", &P22628_4};	/*Profile ID - 22628*/
const Value V23077_1_1 = {S2, NULL};	/*Value = True*/
const Property P23077_1 = {S50, 0, &V23077_1_1, NULL};	/*Property = IsMobile*/
const Value V23077_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23077_5 = {S52, 0, &V23077_5_1277, &P23077_1};	/*Property = ScreenPixelsHeight*/
const Value V23077_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23077_4 = {S54, 0, &V23077_4_2336, &P23077_5};	/*Property = ScreenPixelsWidth*/
const Profile P23077 = {"23077", &P23077_4};	/*Profile ID - 23077*/
const Value V21645_1_1 = {S2, NULL};	/*Value = True*/
const Property P21645_1 = {S50, 0, &V21645_1_1, NULL};	/*Property = IsMobile*/
const Value V21645_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21645_5 = {S52, 0, &V21645_5_1403, &P21645_1};	/*Property = ScreenPixelsHeight*/
const Value V21645_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21645_4 = {S54, 0, &V21645_4_1404, &P21645_5};	/*Property = ScreenPixelsWidth*/
const Profile P21645 = {"21645", &P21645_4};	/*Profile ID - 21645*/
const Value V23326_1_1 = {S2, NULL};	/*Value = True*/
const Property P23326_1 = {S50, 0, &V23326_1_1, NULL};	/*Property = IsMobile*/
const Value V23326_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23326_5 = {S52, 0, &V23326_5_1222, &P23326_1};	/*Property = ScreenPixelsHeight*/
const Value V23326_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23326_4 = {S54, 0, &V23326_4_1224, &P23326_5};	/*Property = ScreenPixelsWidth*/
const Profile P23326 = {"23326", &P23326_4};	/*Profile ID - 23326*/
const Value V23960_1_1 = {S2, NULL};	/*Value = True*/
const Property P23960_1 = {S50, 0, &V23960_1_1, NULL};	/*Property = IsMobile*/
const Value V23960_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23960_5 = {S52, 0, &V23960_5_1458, &P23960_1};	/*Property = ScreenPixelsHeight*/
const Value V23960_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23960_4 = {S54, 0, &V23960_4_1460, &P23960_5};	/*Property = ScreenPixelsWidth*/
const Profile P23960 = {"23960", &P23960_4};	/*Profile ID - 23960*/
const Value V23183_1_1 = {S2, NULL};	/*Value = True*/
const Property P23183_1 = {S50, 0, &V23183_1_1, NULL};	/*Property = IsMobile*/
const Value V23183_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23183_5 = {S52, 0, &V23183_5_1277, &P23183_1};	/*Property = ScreenPixelsHeight*/
const Value V23183_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23183_4 = {S54, 0, &V23183_4_2336, &P23183_5};	/*Property = ScreenPixelsWidth*/
const Profile P23183 = {"23183", &P23183_4};	/*Profile ID - 23183*/
const Value V22614_1_1 = {S2, NULL};	/*Value = True*/
const Property P22614_1 = {S50, 0, &V22614_1_1, NULL};	/*Property = IsMobile*/
const Value V22614_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22614_5 = {S52, 0, &V22614_5_9981, &P22614_1};	/*Property = ScreenPixelsHeight*/
const Value V22614_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22614_4 = {S54, 0, &V22614_4_39098, &P22614_5};	/*Property = ScreenPixelsWidth*/
const Profile P22614 = {"22614", &P22614_4};	/*Profile ID - 22614*/
const Value V21644_1_1 = {S2, NULL};	/*Value = True*/
const Property P21644_1 = {S50, 0, &V21644_1_1, NULL};	/*Property = IsMobile*/
const Value V21644_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21644_5 = {S52, 0, &V21644_5_1403, &P21644_1};	/*Property = ScreenPixelsHeight*/
const Value V21644_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21644_4 = {S54, 0, &V21644_4_1404, &P21644_5};	/*Property = ScreenPixelsWidth*/
const Profile P21644 = {"21644", &P21644_4};	/*Profile ID - 21644*/
const Value V24137_1_1 = {S2, NULL};	/*Value = True*/
const Property P24137_1 = {S50, 0, &V24137_1_1, NULL};	/*Property = IsMobile*/
const Value V24137_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P24137_5 = {S52, 0, &V24137_5_1458, &P24137_1};	/*Property = ScreenPixelsHeight*/
const Value V24137_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P24137_4 = {S54, 0, &V24137_4_37617, &P24137_5};	/*Property = ScreenPixelsWidth*/
const Profile P24137 = {"24137", &P24137_4};	/*Profile ID - 24137*/
const Value V21579_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_279 = {S1, 0, &V21579_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V21579_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_282 = {S3, 0, &V21579_282_57155, &P21579_279};	/*Property = BlobBuilder*/
const Value V21579_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_350 = {S4, 0, &V21579_350_57446, &P21579_282};	/*Property = Canvas*/
const Value V21579_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_283 = {S5, 0, &V21579_283_57161, &P21579_350};	/*Property = CssBackground*/
const Value V21579_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_284 = {S6, 0, &V21579_284_57164, &P21579_283};	/*Property = CssBorderImage*/
const Value V21579_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_285 = {S7, 0, &V21579_285_57167, &P21579_284};	/*Property = CssCanvas*/
const Value V21579_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_286 = {S8, 0, &V21579_286_57170, &P21579_285};	/*Property = CssColor*/
const Value V21579_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_237 = {S9, 0, &V21579_237_55786, &P21579_286};	/*Property = CssColumn*/
const Value V21579_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_287 = {S10, 0, &V21579_287_57173, &P21579_237};	/*Property = CssFlexbox*/
const Value V21579_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_288 = {S11, 0, &V21579_288_57176, &P21579_287};	/*Property = CssFont*/
const Value V21579_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_289 = {S12, 0, &V21579_289_86899, &P21579_288};	/*Property = CssImages*/
const Value V21579_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_291 = {S13, 0, &V21579_291_57182, &P21579_289};	/*Property = CssMediaQueries*/
const Value V21579_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_292 = {S14, 0, &V21579_292_57185, &P21579_291};	/*Property = CssMinMax*/
const Value V21579_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_293 = {S15, 0, &V21579_293_57188, &P21579_292};	/*Property = CssOverflow*/
const Value V21579_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_294 = {S16, 0, &V21579_294_57191, &P21579_293};	/*Property = CssPosition*/
const Value V21579_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_295 = {S17, 0, &V21579_295_57194, &P21579_294};	/*Property = CssText*/
const Value V21579_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_297 = {S18, 0, &V21579_297_57197, &P21579_295};	/*Property = CssTransforms*/
const Value V21579_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_241 = {S19, 0, &V21579_241_55801, &P21579_297};	/*Property = CssTransitions*/
const Value V21579_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_298 = {S20, 0, &V21579_298_57208, &P21579_241};	/*Property = CssUI*/
const Value V21579_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_300 = {S21, 0, &V21579_300_57217, &P21579_298};	/*Property = DataSet*/
const Value V21579_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_301 = {S22, 0, &V21579_301_57220, &P21579_300};	/*Property = DataUrl*/
const Value V21579_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_302 = {S23, 0, &V21579_302_57223, &P21579_301};	/*Property = DeviceOrientation*/
const Value V21579_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_304 = {S24, 0, &V21579_304_57229, &P21579_302};	/*Property = FileReader*/
const Value V21579_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_305 = {S25, 0, &V21579_305_57232, &P21579_304};	/*Property = FileSaver*/
const Value V21579_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_306 = {S26, 0, &V21579_306_57238, &P21579_305};	/*Property = FileWriter*/
const Value V21579_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_307 = {S27, 0, &V21579_307_57241, &P21579_306};	/*Property = FormData*/
const Value V21579_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_309 = {S28, 0, &V21579_309_57247, &P21579_307};	/*Property = Fullscreen*/
const Value V21579_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_310 = {S29, 0, &V21579_310_57250, &P21579_309};	/*Property = GeoLocation*/
const Value V21579_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_312 = {S30, 0, &V21579_312_57256, &P21579_310};	/*Property = History*/
const Value V21579_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_314 = {S31, 0, &V21579_314_57262, &P21579_312};	/*Property = Html5*/
const Value V21579_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_313 = {S32, 0, &V21579_313_57259, &P21579_314};	/*Property = Html-Media-Capture*/
const Value V21579_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_315 = {S33, 0, &V21579_315_57265, &P21579_313};	/*Property = Iframe*/
const Value V21579_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_316 = {S34, 0, &V21579_316_57268, &P21579_315};	/*Property = IndexedDB*/
const Value V21579_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_317 = {S35, 0, &V21579_317_57271, &P21579_316};	/*Property = Json*/
const Value V21579_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21579_15 = {S37, 0, &V21579_15_28, &P21579_317};	/*Property = LayoutEngine*/
const Value V21579_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_344 = {S38, 0, &V21579_344_57274, &P21579_15};	/*Property = Masking*/
const Value V21579_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_320 = {S39, 0, &V21579_320_57292, &P21579_344};	/*Property = PostMessage*/
const Value V21579_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_321 = {S40, 0, &V21579_321_57295, &P21579_320};	/*Property = Progress*/
const Value V21579_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_322 = {S41, 0, &V21579_322_57298, &P21579_321};	/*Property = Prompts*/
const Value V21579_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_323 = {S42, 0, &V21579_323_57301, &P21579_322};	/*Property = Selector*/
const Value V21579_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_324 = {S43, 0, &V21579_324_57307, &P21579_323};	/*Property = Svg*/
const Value V21579_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_327 = {S44, 0, &V21579_327_57316, &P21579_324};	/*Property = TouchEvents*/
const Value V21579_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_328 = {S45, 0, &V21579_328_57319, &P21579_327};	/*Property = Track*/
const Value V21579_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_329 = {S46, 0, &V21579_329_57325, &P21579_328};	/*Property = Video*/
const Value V21579_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_330 = {S47, 0, &V21579_330_57328, &P21579_329};	/*Property = Viewport*/
const Value V21579_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_332 = {S48, 0, &V21579_332_57340, &P21579_330};	/*Property = WebWorkers*/
const Value V21579_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P21579_333 = {S49, 0, &V21579_333_57343, &P21579_332};	/*Property = Xhr2*/
const Profile P21579 = {"21579", &P21579_333};	/*Profile ID - 21579*/
const Value V23309_1_1 = {S2, NULL};	/*Value = True*/
const Property P23309_1 = {S50, 0, &V23309_1_1, NULL};	/*Property = IsMobile*/
const Value V23309_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23309_5 = {S52, 0, &V23309_5_1277, &P23309_1};	/*Property = ScreenPixelsHeight*/
const Value V23309_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23309_4 = {S54, 0, &V23309_4_1279, &P23309_5};	/*Property = ScreenPixelsWidth*/
const Profile P23309 = {"23309", &P23309_4};	/*Profile ID - 23309*/
const Value V22577_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22577_279 = {S1, 0, &V22577_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22577_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22577_282 = {S3, 0, &V22577_282_57154, &P22577_279};	/*Property = BlobBuilder*/
const Value V22577_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22577_350 = {S4, 0, &V22577_350_57444, &P22577_282};	/*Property = Canvas*/
const Value V22577_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22577_283 = {S5, 0, &V22577_283_57159, &P22577_350};	/*Property = CssBackground*/
const Value V22577_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22577_284 = {S6, 0, &V22577_284_57163, &P22577_283};	/*Property = CssBorderImage*/
const Value V22577_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22577_285 = {S7, 0, &V22577_285_57166, &P22577_284};	/*Property = CssCanvas*/
const Value V22577_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22577_286 = {S8, 0, &V22577_286_57168, &P22577_285};	/*Property = CssColor*/
const Value V22577_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22577_237 = {S9, 0, &V22577_237_55784, &P22577_286};	/*Property = CssColumn*/
const Value V22577_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22577_287 = {S10, 0, &V22577_287_57172, &P22577_237};	/*Property = CssFlexbox*/
const Value V22577_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22577_288 = {S11, 0, &V22577_288_57174, &P22577_287};	/*Property = CssFont*/
const Value V22577_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22577_289 = {S12, 0, &V22577_289_57759, &P22577_288};	/*Property = CssImages*/
const Value V22577_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22577_291 = {S13, 0, &V22577_291_57180, &P22577_289};	/*Property = CssMediaQueries*/
const Value V22577_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22577_292 = {S14, 0, &V22577_292_57183, &P22577_291};	/*Property = CssMinMax*/
const Value V22577_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22577_293 = {S15, 0, &V22577_293_57187, &P22577_292};	/*Property = CssOverflow*/
const Value V22577_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22577_294 = {S16, 0, &V22577_294_57189, &P22577_293};	/*Property = CssPosition*/
const Value V22577_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22577_295 = {S17, 0, &V22577_295_57193, &P22577_294};	/*Property = CssText*/
const Value V22577_297_57196 = {S0, NULL};	/*Value = False*/
const Property P22577_297 = {S18, 0, &V22577_297_57196, &P22577_295};	/*Property = CssTransforms*/
const Value V22577_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22577_241 = {S19, 0, &V22577_241_55799, &P22577_297};	/*Property = CssTransitions*/
const Value V22577_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22577_298 = {S20, 0, &V22577_298_57206, &P22577_241};	/*Property = CssUI*/
const Value V22577_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22577_300 = {S21, 0, &V22577_300_57215, &P22577_298};	/*Property = DataSet*/
const Value V22577_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22577_301 = {S22, 0, &V22577_301_57218, &P22577_300};	/*Property = DataUrl*/
const Value V22577_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22577_302 = {S23, 0, &V22577_302_57222, &P22577_301};	/*Property = DeviceOrientation*/
const Value V22577_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22577_304 = {S24, 0, &V22577_304_57227, &P22577_302};	/*Property = FileReader*/
const Value V22577_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22577_305 = {S25, 0, &V22577_305_57231, &P22577_304};	/*Property = FileSaver*/
const Value V22577_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22577_306 = {S26, 0, &V22577_306_57237, &P22577_305};	/*Property = FileWriter*/
const Value V22577_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22577_307 = {S27, 0, &V22577_307_57239, &P22577_306};	/*Property = FormData*/
const Value V22577_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22577_309 = {S28, 0, &V22577_309_57246, &P22577_307};	/*Property = Fullscreen*/
const Value V22577_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22577_310 = {S29, 0, &V22577_310_57248, &P22577_309};	/*Property = GeoLocation*/
const Value V22577_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22577_312 = {S30, 0, &V22577_312_57254, &P22577_310};	/*Property = History*/
const Value V22577_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22577_314 = {S31, 0, &V22577_314_57261, &P22577_312};	/*Property = Html5*/
const Value V22577_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22577_313 = {S32, 0, &V22577_313_57258, &P22577_314};	/*Property = Html-Media-Capture*/
const Value V22577_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22577_315 = {S33, 0, &V22577_315_57264, &P22577_313};	/*Property = Iframe*/
const Value V22577_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22577_316 = {S34, 0, &V22577_316_57267, &P22577_315};	/*Property = IndexedDB*/
const Value V22577_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22577_317 = {S35, 0, &V22577_317_57269, &P22577_316};	/*Property = Json*/
const Value V22577_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P22577_15 = {S37, 0, &V22577_15_31, &P22577_317};	/*Property = LayoutEngine*/
const Value V22577_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22577_344 = {S38, 0, &V22577_344_57272, &P22577_15};	/*Property = Masking*/
const Value V22577_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22577_320 = {S39, 0, &V22577_320_57290, &P22577_344};	/*Property = PostMessage*/
const Value V22577_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22577_321 = {S40, 0, &V22577_321_57293, &P22577_320};	/*Property = Progress*/
const Value V22577_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22577_322 = {S41, 0, &V22577_322_57296, &P22577_321};	/*Property = Prompts*/
const Value V22577_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22577_323 = {S42, 0, &V22577_323_57299, &P22577_322};	/*Property = Selector*/
const Value V22577_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22577_324 = {S43, 0, &V22577_324_57305, &P22577_323};	/*Property = Svg*/
const Value V22577_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22577_327 = {S44, 0, &V22577_327_57315, &P22577_324};	/*Property = TouchEvents*/
const Value V22577_328_57317 = {S2, NULL};	/*Value = True*/
const Property P22577_328 = {S45, 0, &V22577_328_57317, &P22577_327};	/*Property = Track*/
const Value V22577_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22577_329 = {S46, 0, &V22577_329_57323, &P22577_328};	/*Property = Video*/
const Value V22577_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22577_330 = {S47, 0, &V22577_330_57326, &P22577_329};	/*Property = Viewport*/
const Value V22577_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22577_332 = {S48, 0, &V22577_332_57339, &P22577_330};	/*Property = WebWorkers*/
const Value V22577_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22577_333 = {S49, 0, &V22577_333_57342, &P22577_332};	/*Property = Xhr2*/
const Profile P22577 = {"22577", &P22577_333};	/*Profile ID - 22577*/
const Profile P21715 = {"21715", NULL};	/*Profile ID - 21715*/
const Value V21602_1_1 = {S2, NULL};	/*Value = True*/
const Property P21602_1 = {S50, 0, &V21602_1_1, NULL};	/*Property = IsMobile*/
const Value V21602_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21602_5 = {S52, 0, &V21602_5_9981, &P21602_1};	/*Property = ScreenPixelsHeight*/
const Value V21602_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21602_4 = {S54, 0, &V21602_4_39098, &P21602_5};	/*Property = ScreenPixelsWidth*/
const Profile P21602 = {"21602", &P21602_4};	/*Profile ID - 21602*/
const Value V18696_1_1 = {S2, NULL};	/*Value = True*/
const Property P18696_1 = {S50, 0, &V18696_1_1, NULL};	/*Property = IsMobile*/
const Value V18696_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P18696_5 = {S52, 0, &V18696_5_59138, &P18696_1};	/*Property = ScreenPixelsHeight*/
const Value V18696_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P18696_4 = {S54, 0, &V18696_4_59970, &P18696_5};	/*Property = ScreenPixelsWidth*/
const Profile P18696 = {"18696", &P18696_4};	/*Profile ID - 18696*/
const Value V21604_1_1 = {S2, NULL};	/*Value = True*/
const Property P21604_1 = {S50, 0, &V21604_1_1, NULL};	/*Property = IsMobile*/
const Value V21604_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21604_5 = {S52, 0, &V21604_5_1269, &P21604_1};	/*Property = ScreenPixelsHeight*/
const Value V21604_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21604_4 = {S54, 0, &V21604_4_1229, &P21604_5};	/*Property = ScreenPixelsWidth*/
const Profile P21604 = {"21604", &P21604_4};	/*Profile ID - 21604*/
const Value V21605_1_1 = {S2, NULL};	/*Value = True*/
const Property P21605_1 = {S50, 0, &V21605_1_1, NULL};	/*Property = IsMobile*/
const Value V21605_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P21605_5 = {S52, 0, &V21605_5_1269, &P21605_1};	/*Property = ScreenPixelsHeight*/
const Value V21605_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21605_4 = {S54, 0, &V21605_4_1229, &P21605_5};	/*Property = ScreenPixelsWidth*/
const Profile P21605 = {"21605", &P21605_4};	/*Profile ID - 21605*/
const Value V21628_1_1 = {S2, NULL};	/*Value = True*/
const Property P21628_1 = {S50, 0, &V21628_1_1, NULL};	/*Property = IsMobile*/
const Value V21628_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21628_5 = {S52, 0, &V21628_5_1227, &P21628_1};	/*Property = ScreenPixelsHeight*/
const Value V21628_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21628_4 = {S54, 0, &V21628_4_1229, &P21628_5};	/*Property = ScreenPixelsWidth*/
const Profile P21628 = {"21628", &P21628_4};	/*Profile ID - 21628*/
const Value V21627_1_1 = {S2, NULL};	/*Value = True*/
const Property P21627_1 = {S50, 0, &V21627_1_1, NULL};	/*Property = IsMobile*/
const Value V21627_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21627_5 = {S52, 0, &V21627_5_9981, &P21627_1};	/*Property = ScreenPixelsHeight*/
const Value V21627_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21627_4 = {S54, 0, &V21627_4_39098, &P21627_5};	/*Property = ScreenPixelsWidth*/
const Profile P21627 = {"21627", &P21627_4};	/*Profile ID - 21627*/
const Value V21629_1_1 = {S2, NULL};	/*Value = True*/
const Property P21629_1 = {S50, 0, &V21629_1_1, NULL};	/*Property = IsMobile*/
const Value V21629_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21629_5 = {S52, 0, &V21629_5_9981, &P21629_1};	/*Property = ScreenPixelsHeight*/
const Value V21629_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21629_4 = {S54, 0, &V21629_4_39098, &P21629_5};	/*Property = ScreenPixelsWidth*/
const Profile P21629 = {"21629", &P21629_4};	/*Profile ID - 21629*/
const Value V16573_1_1 = {S2, NULL};	/*Value = True*/
const Property P16573_1 = {S50, 0, &V16573_1_1, NULL};	/*Property = IsMobile*/
const Value V16573_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P16573_5 = {S52, 0, &V16573_5_1380, &P16573_1};	/*Property = ScreenPixelsHeight*/
const Value V16573_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16573_4 = {S54, 0, &V16573_4_1279, &P16573_5};	/*Property = ScreenPixelsWidth*/
const Profile P16573 = {"16573", &P16573_4};	/*Profile ID - 16573*/
const Value V23640_1_2 = {S0, NULL};	/*Value = False*/
const Property P23640_1 = {S50, 0, &V23640_1_2, NULL};	/*Property = IsMobile*/
const Value V23640_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23640_5 = {S52, 0, &V23640_5_60417, &P23640_1};	/*Property = ScreenPixelsHeight*/
const Value V23640_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23640_4 = {S54, 0, &V23640_4_57095, &P23640_5};	/*Property = ScreenPixelsWidth*/
const Profile P23640 = {"23640", &P23640_4};	/*Profile ID - 23640*/
const Value V17597_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_279 = {S1, 0, &V17597_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V17597_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_282 = {S3, 0, &V17597_282_57155, &P17597_279};	/*Property = BlobBuilder*/
const Value V17597_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_350 = {S4, 0, &V17597_350_57446, &P17597_282};	/*Property = Canvas*/
const Value V17597_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_283 = {S5, 0, &V17597_283_57161, &P17597_350};	/*Property = CssBackground*/
const Value V17597_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_284 = {S6, 0, &V17597_284_57164, &P17597_283};	/*Property = CssBorderImage*/
const Value V17597_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_285 = {S7, 0, &V17597_285_57167, &P17597_284};	/*Property = CssCanvas*/
const Value V17597_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_286 = {S8, 0, &V17597_286_57170, &P17597_285};	/*Property = CssColor*/
const Value V17597_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_237 = {S9, 0, &V17597_237_55786, &P17597_286};	/*Property = CssColumn*/
const Value V17597_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_287 = {S10, 0, &V17597_287_57173, &P17597_237};	/*Property = CssFlexbox*/
const Value V17597_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_288 = {S11, 0, &V17597_288_57176, &P17597_287};	/*Property = CssFont*/
const Value V17597_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_289 = {S12, 0, &V17597_289_86899, &P17597_288};	/*Property = CssImages*/
const Value V17597_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_291 = {S13, 0, &V17597_291_57182, &P17597_289};	/*Property = CssMediaQueries*/
const Value V17597_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_292 = {S14, 0, &V17597_292_57185, &P17597_291};	/*Property = CssMinMax*/
const Value V17597_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_293 = {S15, 0, &V17597_293_57188, &P17597_292};	/*Property = CssOverflow*/
const Value V17597_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_294 = {S16, 0, &V17597_294_57191, &P17597_293};	/*Property = CssPosition*/
const Value V17597_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_295 = {S17, 0, &V17597_295_57194, &P17597_294};	/*Property = CssText*/
const Value V17597_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_297 = {S18, 0, &V17597_297_57197, &P17597_295};	/*Property = CssTransforms*/
const Value V17597_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_241 = {S19, 0, &V17597_241_55801, &P17597_297};	/*Property = CssTransitions*/
const Value V17597_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_298 = {S20, 0, &V17597_298_57208, &P17597_241};	/*Property = CssUI*/
const Value V17597_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_300 = {S21, 0, &V17597_300_57217, &P17597_298};	/*Property = DataSet*/
const Value V17597_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_301 = {S22, 0, &V17597_301_57220, &P17597_300};	/*Property = DataUrl*/
const Value V17597_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_302 = {S23, 0, &V17597_302_57223, &P17597_301};	/*Property = DeviceOrientation*/
const Value V17597_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_304 = {S24, 0, &V17597_304_57229, &P17597_302};	/*Property = FileReader*/
const Value V17597_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_305 = {S25, 0, &V17597_305_57232, &P17597_304};	/*Property = FileSaver*/
const Value V17597_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_306 = {S26, 0, &V17597_306_57238, &P17597_305};	/*Property = FileWriter*/
const Value V17597_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_307 = {S27, 0, &V17597_307_57241, &P17597_306};	/*Property = FormData*/
const Value V17597_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_309 = {S28, 0, &V17597_309_57247, &P17597_307};	/*Property = Fullscreen*/
const Value V17597_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_310 = {S29, 0, &V17597_310_57250, &P17597_309};	/*Property = GeoLocation*/
const Value V17597_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_312 = {S30, 0, &V17597_312_57256, &P17597_310};	/*Property = History*/
const Value V17597_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_314 = {S31, 0, &V17597_314_57262, &P17597_312};	/*Property = Html5*/
const Value V17597_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_313 = {S32, 0, &V17597_313_57259, &P17597_314};	/*Property = Html-Media-Capture*/
const Value V17597_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_315 = {S33, 0, &V17597_315_57265, &P17597_313};	/*Property = Iframe*/
const Value V17597_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_316 = {S34, 0, &V17597_316_57268, &P17597_315};	/*Property = IndexedDB*/
const Value V17597_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_317 = {S35, 0, &V17597_317_57271, &P17597_316};	/*Property = Json*/
const Value V17597_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P17597_15 = {S37, 0, &V17597_15_29, &P17597_317};	/*Property = LayoutEngine*/
const Value V17597_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_344 = {S38, 0, &V17597_344_57274, &P17597_15};	/*Property = Masking*/
const Value V17597_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_320 = {S39, 0, &V17597_320_57292, &P17597_344};	/*Property = PostMessage*/
const Value V17597_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_321 = {S40, 0, &V17597_321_57295, &P17597_320};	/*Property = Progress*/
const Value V17597_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_322 = {S41, 0, &V17597_322_57298, &P17597_321};	/*Property = Prompts*/
const Value V17597_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_323 = {S42, 0, &V17597_323_57301, &P17597_322};	/*Property = Selector*/
const Value V17597_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_324 = {S43, 0, &V17597_324_57307, &P17597_323};	/*Property = Svg*/
const Value V17597_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_327 = {S44, 0, &V17597_327_57316, &P17597_324};	/*Property = TouchEvents*/
const Value V17597_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_328 = {S45, 0, &V17597_328_57319, &P17597_327};	/*Property = Track*/
const Value V17597_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_329 = {S46, 0, &V17597_329_57325, &P17597_328};	/*Property = Video*/
const Value V17597_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_330 = {S47, 0, &V17597_330_57328, &P17597_329};	/*Property = Viewport*/
const Value V17597_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_332 = {S48, 0, &V17597_332_57340, &P17597_330};	/*Property = WebWorkers*/
const Value V17597_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P17597_333 = {S49, 0, &V17597_333_57343, &P17597_332};	/*Property = Xhr2*/
const Profile P17597 = {"17597", &P17597_333};	/*Profile ID - 17597*/
const Value V21847_1_1 = {S2, NULL};	/*Value = True*/
const Property P21847_1 = {S50, 0, &V21847_1_1, NULL};	/*Property = IsMobile*/
const Value V21847_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21847_5 = {S52, 0, &V21847_5_1227, &P21847_1};	/*Property = ScreenPixelsHeight*/
const Value V21847_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21847_4 = {S54, 0, &V21847_4_1229, &P21847_5};	/*Property = ScreenPixelsWidth*/
const Profile P21847 = {"21847", &P21847_4};	/*Profile ID - 21847*/
const Profile P22712 = {"22712", NULL};	/*Profile ID - 22712*/
const Value V22150_1_1 = {S2, NULL};	/*Value = True*/
const Property P22150_1 = {S50, 0, &V22150_1_1, NULL};	/*Property = IsMobile*/
const Value V22150_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22150_5 = {S52, 0, &V22150_5_1222, &P22150_1};	/*Property = ScreenPixelsHeight*/
const Value V22150_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22150_4 = {S54, 0, &V22150_4_1224, &P22150_5};	/*Property = ScreenPixelsWidth*/
const Profile P22150 = {"22150", &P22150_4};	/*Profile ID - 22150*/
const Value V22056_1_1 = {S2, NULL};	/*Value = True*/
const Property P22056_1 = {S50, 0, &V22056_1_1, NULL};	/*Property = IsMobile*/
const Value V22056_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22056_5 = {S52, 0, &V22056_5_1277, &P22056_1};	/*Property = ScreenPixelsHeight*/
const Value V22056_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22056_4 = {S54, 0, &V22056_4_2336, &P22056_5};	/*Property = ScreenPixelsWidth*/
const Profile P22056 = {"22056", &P22056_4};	/*Profile ID - 22056*/
const Value V22087_1_1 = {S2, NULL};	/*Value = True*/
const Property P22087_1 = {S50, 0, &V22087_1_1, NULL};	/*Property = IsMobile*/
const Value V22087_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22087_5 = {S52, 0, &V22087_5_1277, &P22087_1};	/*Property = ScreenPixelsHeight*/
const Value V22087_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22087_4 = {S54, 0, &V22087_4_1279, &P22087_5};	/*Property = ScreenPixelsWidth*/
const Profile P22087 = {"22087", &P22087_4};	/*Profile ID - 22087*/
const Value V23607_1_2 = {S0, NULL};	/*Value = False*/
const Property P23607_1 = {S50, 0, &V23607_1_2, NULL};	/*Property = IsMobile*/
const Value V23607_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23607_5 = {S52, 0, &V23607_5_60417, &P23607_1};	/*Property = ScreenPixelsHeight*/
const Value V23607_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23607_4 = {S54, 0, &V23607_4_57095, &P23607_5};	/*Property = ScreenPixelsWidth*/
const Profile P23607 = {"23607", &P23607_4};	/*Profile ID - 23607*/
const Value V23608_1_2 = {S0, NULL};	/*Value = False*/
const Property P23608_1 = {S50, 0, &V23608_1_2, NULL};	/*Property = IsMobile*/
const Value V23608_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23608_5 = {S52, 0, &V23608_5_60417, &P23608_1};	/*Property = ScreenPixelsHeight*/
const Value V23608_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23608_4 = {S54, 0, &V23608_4_57095, &P23608_5};	/*Property = ScreenPixelsWidth*/
const Profile P23608 = {"23608", &P23608_4};	/*Profile ID - 23608*/
const Value V23587_1_1 = {S2, NULL};	/*Value = True*/
const Property P23587_1 = {S50, 0, &V23587_1_1, NULL};	/*Property = IsMobile*/
const Value V23587_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P23587_5 = {S52, 0, &V23587_5_1380, &P23587_1};	/*Property = ScreenPixelsHeight*/
const Value V23587_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23587_4 = {S54, 0, &V23587_4_1279, &P23587_5};	/*Property = ScreenPixelsWidth*/
const Profile P23587 = {"23587", &P23587_4};	/*Profile ID - 23587*/
const Value V21836_1_1 = {S2, NULL};	/*Value = True*/
const Property P21836_1 = {S50, 0, &V21836_1_1, NULL};	/*Property = IsMobile*/
const Value V21836_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21836_5 = {S52, 0, &V21836_5_1403, &P21836_1};	/*Property = ScreenPixelsHeight*/
const Value V21836_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21836_4 = {S54, 0, &V21836_4_1404, &P21836_5};	/*Property = ScreenPixelsWidth*/
const Profile P21836 = {"21836", &P21836_4};	/*Profile ID - 21836*/
const Value V20096_1_2 = {S0, NULL};	/*Value = False*/
const Property P20096_1 = {S50, 0, &V20096_1_2, NULL};	/*Property = IsMobile*/
const Value V20096_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P20096_5 = {S52, 0, &V20096_5_37, &P20096_1};	/*Property = ScreenPixelsHeight*/
const Value V20096_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P20096_4 = {S54, 0, &V20096_4_36, &P20096_5};	/*Property = ScreenPixelsWidth*/
const Profile P20096 = {"20096", &P20096_4};	/*Profile ID - 20096*/
const Value V21998_1_1 = {S2, NULL};	/*Value = True*/
const Property P21998_1 = {S50, 0, &V21998_1_1, NULL};	/*Property = IsMobile*/
const Value V21998_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21998_5 = {S52, 0, &V21998_5_1227, &P21998_1};	/*Property = ScreenPixelsHeight*/
const Value V21998_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21998_4 = {S54, 0, &V21998_4_1229, &P21998_5};	/*Property = ScreenPixelsWidth*/
const Profile P21998 = {"21998", &P21998_4};	/*Profile ID - 21998*/
const Value V22385_1_1 = {S2, NULL};	/*Value = True*/
const Property P22385_1 = {S50, 0, &V22385_1_1, NULL};	/*Property = IsMobile*/
const Value V22385_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22385_5 = {S52, 0, &V22385_5_1227, &P22385_1};	/*Property = ScreenPixelsHeight*/
const Value V22385_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22385_4 = {S54, 0, &V22385_4_1229, &P22385_5};	/*Property = ScreenPixelsWidth*/
const Profile P22385 = {"22385", &P22385_4};	/*Profile ID - 22385*/
const Value V22148_1_1 = {S2, NULL};	/*Value = True*/
const Property P22148_1 = {S50, 0, &V22148_1_1, NULL};	/*Property = IsMobile*/
const Value V22148_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22148_5 = {S52, 0, &V22148_5_1277, &P22148_1};	/*Property = ScreenPixelsHeight*/
const Value V22148_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22148_4 = {S54, 0, &V22148_4_2336, &P22148_5};	/*Property = ScreenPixelsWidth*/
const Profile P22148 = {"22148", &P22148_4};	/*Profile ID - 22148*/
const Value V22014_1_1 = {S2, NULL};	/*Value = True*/
const Property P22014_1 = {S50, 0, &V22014_1_1, NULL};	/*Property = IsMobile*/
const Value V22014_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22014_5 = {S52, 0, &V22014_5_37569, &P22014_1};	/*Property = ScreenPixelsHeight*/
const Value V22014_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22014_4 = {S54, 0, &V22014_4_5645, &P22014_5};	/*Property = ScreenPixelsWidth*/
const Profile P22014 = {"22014", &P22014_4};	/*Profile ID - 22014*/
const Value V22054_1_1 = {S2, NULL};	/*Value = True*/
const Property P22054_1 = {S50, 0, &V22054_1_1, NULL};	/*Property = IsMobile*/
const Value V22054_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22054_5 = {S52, 0, &V22054_5_1277, &P22054_1};	/*Property = ScreenPixelsHeight*/
const Value V22054_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22054_4 = {S54, 0, &V22054_4_1279, &P22054_5};	/*Property = ScreenPixelsWidth*/
const Profile P22054 = {"22054", &P22054_4};	/*Profile ID - 22054*/
const Value V23091_1_1 = {S2, NULL};	/*Value = True*/
const Property P23091_1 = {S50, 0, &V23091_1_1, NULL};	/*Property = IsMobile*/
const Value V23091_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23091_5 = {S52, 0, &V23091_5_1222, &P23091_1};	/*Property = ScreenPixelsHeight*/
const Value V23091_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23091_4 = {S54, 0, &V23091_4_1224, &P23091_5};	/*Property = ScreenPixelsWidth*/
const Profile P23091 = {"23091", &P23091_4};	/*Profile ID - 23091*/
const Value V22113_1_1 = {S2, NULL};	/*Value = True*/
const Property P22113_1 = {S50, 0, &V22113_1_1, NULL};	/*Property = IsMobile*/
const Value V22113_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22113_5 = {S52, 0, &V22113_5_1277, &P22113_1};	/*Property = ScreenPixelsHeight*/
const Value V22113_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22113_4 = {S54, 0, &V22113_4_1279, &P22113_5};	/*Property = ScreenPixelsWidth*/
const Profile P22113 = {"22113", &P22113_4};	/*Profile ID - 22113*/
const Value V20713_1_1 = {S2, NULL};	/*Value = True*/
const Property P20713_1 = {S50, 0, &V20713_1_1, NULL};	/*Property = IsMobile*/
const Value V20713_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20713_5 = {S52, 0, &V20713_5_1458, &P20713_1};	/*Property = ScreenPixelsHeight*/
const Value V20713_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20713_4 = {S54, 0, &V20713_4_1460, &P20713_5};	/*Property = ScreenPixelsWidth*/
const Profile P20713 = {"20713", &P20713_4};	/*Profile ID - 20713*/
const Value V22020_1_1 = {S2, NULL};	/*Value = True*/
const Property P22020_1 = {S50, 0, &V22020_1_1, NULL};	/*Property = IsMobile*/
const Value V22020_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22020_5 = {S52, 0, &V22020_5_9981, &P22020_1};	/*Property = ScreenPixelsHeight*/
const Value V22020_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22020_4 = {S54, 0, &V22020_4_39098, &P22020_5};	/*Property = ScreenPixelsWidth*/
const Profile P22020 = {"22020", &P22020_4};	/*Profile ID - 22020*/
const Value V23609_1_2 = {S0, NULL};	/*Value = False*/
const Property P23609_1 = {S50, 0, &V23609_1_2, NULL};	/*Property = IsMobile*/
const Value V23609_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23609_5 = {S52, 0, &V23609_5_60417, &P23609_1};	/*Property = ScreenPixelsHeight*/
const Value V23609_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23609_4 = {S54, 0, &V23609_4_57095, &P23609_5};	/*Property = ScreenPixelsWidth*/
const Profile P23609 = {"23609", &P23609_4};	/*Profile ID - 23609*/
const Value V22154_1_1 = {S2, NULL};	/*Value = True*/
const Property P22154_1 = {S50, 0, &V22154_1_1, NULL};	/*Property = IsMobile*/
const Value V22154_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22154_5 = {S52, 0, &V22154_5_1227, &P22154_1};	/*Property = ScreenPixelsHeight*/
const Value V22154_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22154_4 = {S54, 0, &V22154_4_1229, &P22154_5};	/*Property = ScreenPixelsWidth*/
const Profile P22154 = {"22154", &P22154_4};	/*Profile ID - 22154*/
const Value V23446_1_1 = {S2, NULL};	/*Value = True*/
const Property P23446_1 = {S50, 0, &V23446_1_1, NULL};	/*Property = IsMobile*/
const Value V23446_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23446_5 = {S52, 0, &V23446_5_1277, &P23446_1};	/*Property = ScreenPixelsHeight*/
const Value V23446_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23446_4 = {S54, 0, &V23446_4_1279, &P23446_5};	/*Property = ScreenPixelsWidth*/
const Profile P23446 = {"23446", &P23446_4};	/*Profile ID - 23446*/
const Value V23298_1_1 = {S2, NULL};	/*Value = True*/
const Property P23298_1 = {S50, 0, &V23298_1_1, NULL};	/*Property = IsMobile*/
const Value V23298_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23298_5 = {S52, 0, &V23298_5_9981, &P23298_1};	/*Property = ScreenPixelsHeight*/
const Value V23298_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23298_4 = {S54, 0, &V23298_4_2336, &P23298_5};	/*Property = ScreenPixelsWidth*/
const Profile P23298 = {"23298", &P23298_4};	/*Profile ID - 23298*/
const Value V22780_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_279 = {S1, 0, &V22780_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22780_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_282 = {S3, 0, &V22780_282_57155, &P22780_279};	/*Property = BlobBuilder*/
const Value V22780_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_350 = {S4, 0, &V22780_350_57446, &P22780_282};	/*Property = Canvas*/
const Value V22780_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_283 = {S5, 0, &V22780_283_57161, &P22780_350};	/*Property = CssBackground*/
const Value V22780_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_284 = {S6, 0, &V22780_284_57164, &P22780_283};	/*Property = CssBorderImage*/
const Value V22780_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_285 = {S7, 0, &V22780_285_57167, &P22780_284};	/*Property = CssCanvas*/
const Value V22780_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_286 = {S8, 0, &V22780_286_57170, &P22780_285};	/*Property = CssColor*/
const Value V22780_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_237 = {S9, 0, &V22780_237_55786, &P22780_286};	/*Property = CssColumn*/
const Value V22780_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_287 = {S10, 0, &V22780_287_57173, &P22780_237};	/*Property = CssFlexbox*/
const Value V22780_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_288 = {S11, 0, &V22780_288_57176, &P22780_287};	/*Property = CssFont*/
const Value V22780_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_289 = {S12, 0, &V22780_289_86899, &P22780_288};	/*Property = CssImages*/
const Value V22780_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_291 = {S13, 0, &V22780_291_57182, &P22780_289};	/*Property = CssMediaQueries*/
const Value V22780_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_292 = {S14, 0, &V22780_292_57185, &P22780_291};	/*Property = CssMinMax*/
const Value V22780_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_293 = {S15, 0, &V22780_293_57188, &P22780_292};	/*Property = CssOverflow*/
const Value V22780_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_294 = {S16, 0, &V22780_294_57191, &P22780_293};	/*Property = CssPosition*/
const Value V22780_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_295 = {S17, 0, &V22780_295_57194, &P22780_294};	/*Property = CssText*/
const Value V22780_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_297 = {S18, 0, &V22780_297_57197, &P22780_295};	/*Property = CssTransforms*/
const Value V22780_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_241 = {S19, 0, &V22780_241_55801, &P22780_297};	/*Property = CssTransitions*/
const Value V22780_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_298 = {S20, 0, &V22780_298_57208, &P22780_241};	/*Property = CssUI*/
const Value V22780_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_300 = {S21, 0, &V22780_300_57217, &P22780_298};	/*Property = DataSet*/
const Value V22780_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_301 = {S22, 0, &V22780_301_57220, &P22780_300};	/*Property = DataUrl*/
const Value V22780_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_302 = {S23, 0, &V22780_302_57223, &P22780_301};	/*Property = DeviceOrientation*/
const Value V22780_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_304 = {S24, 0, &V22780_304_57229, &P22780_302};	/*Property = FileReader*/
const Value V22780_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_305 = {S25, 0, &V22780_305_57232, &P22780_304};	/*Property = FileSaver*/
const Value V22780_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_306 = {S26, 0, &V22780_306_57238, &P22780_305};	/*Property = FileWriter*/
const Value V22780_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_307 = {S27, 0, &V22780_307_57241, &P22780_306};	/*Property = FormData*/
const Value V22780_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_309 = {S28, 0, &V22780_309_57247, &P22780_307};	/*Property = Fullscreen*/
const Value V22780_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_310 = {S29, 0, &V22780_310_57250, &P22780_309};	/*Property = GeoLocation*/
const Value V22780_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_312 = {S30, 0, &V22780_312_57256, &P22780_310};	/*Property = History*/
const Value V22780_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_314 = {S31, 0, &V22780_314_57262, &P22780_312};	/*Property = Html5*/
const Value V22780_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_313 = {S32, 0, &V22780_313_57259, &P22780_314};	/*Property = Html-Media-Capture*/
const Value V22780_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_315 = {S33, 0, &V22780_315_57265, &P22780_313};	/*Property = Iframe*/
const Value V22780_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_316 = {S34, 0, &V22780_316_57268, &P22780_315};	/*Property = IndexedDB*/
const Value V22780_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_317 = {S35, 0, &V22780_317_57271, &P22780_316};	/*Property = Json*/
const Value V22780_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22780_15 = {S37, 0, &V22780_15_29, &P22780_317};	/*Property = LayoutEngine*/
const Value V22780_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_344 = {S38, 0, &V22780_344_57274, &P22780_15};	/*Property = Masking*/
const Value V22780_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_320 = {S39, 0, &V22780_320_57292, &P22780_344};	/*Property = PostMessage*/
const Value V22780_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_321 = {S40, 0, &V22780_321_57295, &P22780_320};	/*Property = Progress*/
const Value V22780_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_322 = {S41, 0, &V22780_322_57298, &P22780_321};	/*Property = Prompts*/
const Value V22780_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_323 = {S42, 0, &V22780_323_57301, &P22780_322};	/*Property = Selector*/
const Value V22780_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_324 = {S43, 0, &V22780_324_57307, &P22780_323};	/*Property = Svg*/
const Value V22780_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_327 = {S44, 0, &V22780_327_57316, &P22780_324};	/*Property = TouchEvents*/
const Value V22780_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_328 = {S45, 0, &V22780_328_57319, &P22780_327};	/*Property = Track*/
const Value V22780_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_329 = {S46, 0, &V22780_329_57325, &P22780_328};	/*Property = Video*/
const Value V22780_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_330 = {S47, 0, &V22780_330_57328, &P22780_329};	/*Property = Viewport*/
const Value V22780_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_332 = {S48, 0, &V22780_332_57340, &P22780_330};	/*Property = WebWorkers*/
const Value V22780_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22780_333 = {S49, 0, &V22780_333_57343, &P22780_332};	/*Property = Xhr2*/
const Profile P22780 = {"22780", &P22780_333};	/*Profile ID - 22780*/
const Value V22598_1_1 = {S2, NULL};	/*Value = True*/
const Property P22598_1 = {S50, 0, &V22598_1_1, NULL};	/*Property = IsMobile*/
const Value V22598_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22598_5 = {S52, 0, &V22598_5_59138, &P22598_1};	/*Property = ScreenPixelsHeight*/
const Value V22598_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22598_4 = {S54, 0, &V22598_4_59970, &P22598_5};	/*Property = ScreenPixelsWidth*/
const Profile P22598 = {"22598", &P22598_4};	/*Profile ID - 22598*/
const Value V21844_1_1 = {S2, NULL};	/*Value = True*/
const Property P21844_1 = {S50, 0, &V21844_1_1, NULL};	/*Property = IsMobile*/
const Value V21844_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21844_5 = {S52, 0, &V21844_5_1227, &P21844_1};	/*Property = ScreenPixelsHeight*/
const Value V21844_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21844_4 = {S54, 0, &V21844_4_1229, &P21844_5};	/*Property = ScreenPixelsWidth*/
const Profile P21844 = {"21844", &P21844_4};	/*Profile ID - 21844*/
const Value V20871_1_1 = {S2, NULL};	/*Value = True*/
const Property P20871_1 = {S50, 0, &V20871_1_1, NULL};	/*Property = IsMobile*/
const Value V20871_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20871_5 = {S52, 0, &V20871_5_1277, &P20871_1};	/*Property = ScreenPixelsHeight*/
const Value V20871_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20871_4 = {S54, 0, &V20871_4_1279, &P20871_5};	/*Property = ScreenPixelsWidth*/
const Profile P20871 = {"20871", &P20871_4};	/*Profile ID - 20871*/
const Value V23718_1_1 = {S2, NULL};	/*Value = True*/
const Property P23718_1 = {S50, 0, &V23718_1_1, NULL};	/*Property = IsMobile*/
const Value V23718_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23718_5 = {S52, 0, &V23718_5_1403, &P23718_1};	/*Property = ScreenPixelsHeight*/
const Value V23718_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23718_4 = {S54, 0, &V23718_4_1404, &P23718_5};	/*Property = ScreenPixelsWidth*/
const Profile P23718 = {"23718", &P23718_4};	/*Profile ID - 23718*/
const Value V23471_1_1 = {S2, NULL};	/*Value = True*/
const Property P23471_1 = {S50, 0, &V23471_1_1, NULL};	/*Property = IsMobile*/
const Value V23471_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23471_5 = {S52, 0, &V23471_5_1227, &P23471_1};	/*Property = ScreenPixelsHeight*/
const Value V23471_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23471_4 = {S54, 0, &V23471_4_1229, &P23471_5};	/*Property = ScreenPixelsWidth*/
const Profile P23471 = {"23471", &P23471_4};	/*Profile ID - 23471*/
const Value V21953_1_1 = {S2, NULL};	/*Value = True*/
const Property P21953_1 = {S50, 0, &V21953_1_1, NULL};	/*Property = IsMobile*/
const Value V21953_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21953_5 = {S52, 0, &V21953_5_1277, &P21953_1};	/*Property = ScreenPixelsHeight*/
const Value V21953_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21953_4 = {S54, 0, &V21953_4_1279, &P21953_5};	/*Property = ScreenPixelsWidth*/
const Profile P21953 = {"21953", &P21953_4};	/*Profile ID - 21953*/
const Value V23745_1_1 = {S2, NULL};	/*Value = True*/
const Property P23745_1 = {S50, 0, &V23745_1_1, NULL};	/*Property = IsMobile*/
const Value V23745_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23745_5 = {S52, 0, &V23745_5_1458, &P23745_1};	/*Property = ScreenPixelsHeight*/
const Value V23745_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23745_4 = {S54, 0, &V23745_4_1460, &P23745_5};	/*Property = ScreenPixelsWidth*/
const Profile P23745 = {"23745", &P23745_4};	/*Profile ID - 23745*/
const Value V23941_1_1 = {S2, NULL};	/*Value = True*/
const Property P23941_1 = {S50, 0, &V23941_1_1, NULL};	/*Property = IsMobile*/
const Value V23941_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23941_5 = {S52, 0, &V23941_5_1227, &P23941_1};	/*Property = ScreenPixelsHeight*/
const Value V23941_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23941_4 = {S54, 0, &V23941_4_1229, &P23941_5};	/*Property = ScreenPixelsWidth*/
const Profile P23941 = {"23941", &P23941_4};	/*Profile ID - 23941*/
const Value V21517_1_1 = {S2, NULL};	/*Value = True*/
const Property P21517_1 = {S50, 0, &V21517_1_1, NULL};	/*Property = IsMobile*/
const Value V21517_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21517_5 = {S52, 0, &V21517_5_1222, &P21517_1};	/*Property = ScreenPixelsHeight*/
const Value V21517_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21517_4 = {S54, 0, &V21517_4_1224, &P21517_5};	/*Property = ScreenPixelsWidth*/
const Profile P21517 = {"21517", &P21517_4};	/*Profile ID - 21517*/
const Value V21993_1_1 = {S2, NULL};	/*Value = True*/
const Property P21993_1 = {S50, 0, &V21993_1_1, NULL};	/*Property = IsMobile*/
const Value V21993_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P21993_5 = {S52, 0, &V21993_5_1275, &P21993_1};	/*Property = ScreenPixelsHeight*/
const Value V21993_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21993_4 = {S54, 0, &V21993_4_1229, &P21993_5};	/*Property = ScreenPixelsWidth*/
const Profile P21993 = {"21993", &P21993_4};	/*Profile ID - 21993*/
const Value V24461_1_1 = {S2, NULL};	/*Value = True*/
const Property P24461_1 = {S50, 0, &V24461_1_1, NULL};	/*Property = IsMobile*/
const Value V24461_5_64659 = {S111, NULL};	/*Value = 420*/
const Property P24461_5 = {S52, 0, &V24461_5_64659, &P24461_1};	/*Property = ScreenPixelsHeight*/
const Value V24461_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24461_4 = {S54, 0, &V24461_4_1279, &P24461_5};	/*Property = ScreenPixelsWidth*/
const Profile P24461 = {"24461", &P24461_4};	/*Profile ID - 24461*/
const Value V21735_1_1 = {S2, NULL};	/*Value = True*/
const Property P21735_1 = {S50, 0, &V21735_1_1, NULL};	/*Property = IsMobile*/
const Value V21735_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21735_5 = {S52, 0, &V21735_5_1227, &P21735_1};	/*Property = ScreenPixelsHeight*/
const Value V21735_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21735_4 = {S54, 0, &V21735_4_1229, &P21735_5};	/*Property = ScreenPixelsWidth*/
const Profile P21735 = {"21735", &P21735_4};	/*Profile ID - 21735*/
const Value V23284_1_1 = {S2, NULL};	/*Value = True*/
const Property P23284_1 = {S50, 0, &V23284_1_1, NULL};	/*Property = IsMobile*/
const Value V23284_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23284_5 = {S52, 0, &V23284_5_1277, &P23284_1};	/*Property = ScreenPixelsHeight*/
const Value V23284_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23284_4 = {S54, 0, &V23284_4_2336, &P23284_5};	/*Property = ScreenPixelsWidth*/
const Profile P23284 = {"23284", &P23284_4};	/*Profile ID - 23284*/
const Value V21003_1_1 = {S2, NULL};	/*Value = True*/
const Property P21003_1 = {S50, 0, &V21003_1_1, NULL};	/*Property = IsMobile*/
const Value V21003_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P21003_5 = {S52, 0, &V21003_5_1304, &P21003_1};	/*Property = ScreenPixelsHeight*/
const Value V21003_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21003_4 = {S54, 0, &V21003_4_1224, &P21003_5};	/*Property = ScreenPixelsWidth*/
const Profile P21003 = {"21003", &P21003_4};	/*Profile ID - 21003*/
const Value V21947_1_1 = {S2, NULL};	/*Value = True*/
const Property P21947_1 = {S50, 0, &V21947_1_1, NULL};	/*Property = IsMobile*/
const Value V21947_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21947_5 = {S52, 0, &V21947_5_1227, &P21947_1};	/*Property = ScreenPixelsHeight*/
const Value V21947_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21947_4 = {S54, 0, &V21947_4_1229, &P21947_5};	/*Property = ScreenPixelsWidth*/
const Profile P21947 = {"21947", &P21947_4};	/*Profile ID - 21947*/
const Value V21707_279_57147 = {S2, NULL};	/*Value = True*/
const Property P21707_279 = {S1, 0, &V21707_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V21707_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21707_282 = {S3, 0, &V21707_282_57154, &P21707_279};	/*Property = BlobBuilder*/
const Value V21707_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21707_350 = {S4, 0, &V21707_350_57444, &P21707_282};	/*Property = Canvas*/
const Value V21707_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21707_283 = {S5, 0, &V21707_283_57159, &P21707_350};	/*Property = CssBackground*/
const Value V21707_284_57162 = {S2, NULL};	/*Value = True*/
const Property P21707_284 = {S6, 0, &V21707_284_57162, &P21707_283};	/*Property = CssBorderImage*/
const Value V21707_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21707_285 = {S7, 0, &V21707_285_57166, &P21707_284};	/*Property = CssCanvas*/
const Value V21707_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21707_286 = {S8, 0, &V21707_286_57168, &P21707_285};	/*Property = CssColor*/
const Value V21707_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21707_237 = {S9, 0, &V21707_237_55786, &P21707_286};	/*Property = CssColumn*/
const Value V21707_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21707_287 = {S10, 0, &V21707_287_57172, &P21707_237};	/*Property = CssFlexbox*/
const Value V21707_288_57174 = {S2, NULL};	/*Value = True*/
const Property P21707_288 = {S11, 0, &V21707_288_57174, &P21707_287};	/*Property = CssFont*/
const Value V21707_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21707_289 = {S12, 0, &V21707_289_57759, &P21707_288};	/*Property = CssImages*/
const Value V21707_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21707_291 = {S13, 0, &V21707_291_57180, &P21707_289};	/*Property = CssMediaQueries*/
const Value V21707_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21707_292 = {S14, 0, &V21707_292_57183, &P21707_291};	/*Property = CssMinMax*/
const Value V21707_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21707_293 = {S15, 0, &V21707_293_57187, &P21707_292};	/*Property = CssOverflow*/
const Value V21707_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21707_294 = {S16, 0, &V21707_294_57189, &P21707_293};	/*Property = CssPosition*/
const Value V21707_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21707_295 = {S17, 0, &V21707_295_57192, &P21707_294};	/*Property = CssText*/
const Value V21707_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21707_297 = {S18, 0, &V21707_297_57195, &P21707_295};	/*Property = CssTransforms*/
const Value V21707_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21707_241 = {S19, 0, &V21707_241_55799, &P21707_297};	/*Property = CssTransitions*/
const Value V21707_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21707_298 = {S20, 0, &V21707_298_57206, &P21707_241};	/*Property = CssUI*/
const Value V21707_300_57215 = {S2, NULL};	/*Value = True*/
const Property P21707_300 = {S21, 0, &V21707_300_57215, &P21707_298};	/*Property = DataSet*/
const Value V21707_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21707_301 = {S22, 0, &V21707_301_57218, &P21707_300};	/*Property = DataUrl*/
const Value V21707_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21707_302 = {S23, 0, &V21707_302_57222, &P21707_301};	/*Property = DeviceOrientation*/
const Value V21707_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21707_304 = {S24, 0, &V21707_304_57227, &P21707_302};	/*Property = FileReader*/
const Value V21707_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21707_305 = {S25, 0, &V21707_305_57231, &P21707_304};	/*Property = FileSaver*/
const Value V21707_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21707_306 = {S26, 0, &V21707_306_57237, &P21707_305};	/*Property = FileWriter*/
const Value V21707_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21707_307 = {S27, 0, &V21707_307_57239, &P21707_306};	/*Property = FormData*/
const Value V21707_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21707_309 = {S28, 0, &V21707_309_57246, &P21707_307};	/*Property = Fullscreen*/
const Value V21707_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21707_310 = {S29, 0, &V21707_310_57248, &P21707_309};	/*Property = GeoLocation*/
const Value V21707_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21707_312 = {S30, 0, &V21707_312_57254, &P21707_310};	/*Property = History*/
const Value V21707_314_57260 = {S2, NULL};	/*Value = True*/
const Property P21707_314 = {S31, 0, &V21707_314_57260, &P21707_312};	/*Property = Html5*/
const Value V21707_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21707_313 = {S32, 0, &V21707_313_57258, &P21707_314};	/*Property = Html-Media-Capture*/
const Value V21707_315_57263 = {S2, NULL};	/*Value = True*/
const Property P21707_315 = {S33, 0, &V21707_315_57263, &P21707_313};	/*Property = Iframe*/
const Value V21707_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21707_316 = {S34, 0, &V21707_316_57266, &P21707_315};	/*Property = IndexedDB*/
const Value V21707_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21707_317 = {S35, 0, &V21707_317_57269, &P21707_316};	/*Property = Json*/
const Value V21707_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21707_15 = {S37, 0, &V21707_15_28, &P21707_317};	/*Property = LayoutEngine*/
const Value V21707_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21707_344 = {S38, 0, &V21707_344_57272, &P21707_15};	/*Property = Masking*/
const Value V21707_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21707_320 = {S39, 0, &V21707_320_57290, &P21707_344};	/*Property = PostMessage*/
const Value V21707_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21707_321 = {S40, 0, &V21707_321_57293, &P21707_320};	/*Property = Progress*/
const Value V21707_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21707_322 = {S41, 0, &V21707_322_57296, &P21707_321};	/*Property = Prompts*/
const Value V21707_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21707_323 = {S42, 0, &V21707_323_57299, &P21707_322};	/*Property = Selector*/
const Value V21707_324_57305 = {S2, NULL};	/*Value = True*/
const Property P21707_324 = {S43, 0, &V21707_324_57305, &P21707_323};	/*Property = Svg*/
const Value V21707_327_57314 = {S2, NULL};	/*Value = True*/
const Property P21707_327 = {S44, 0, &V21707_327_57314, &P21707_324};	/*Property = TouchEvents*/
const Value V21707_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21707_328 = {S45, 0, &V21707_328_57318, &P21707_327};	/*Property = Track*/
const Value V21707_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21707_329 = {S46, 0, &V21707_329_57323, &P21707_328};	/*Property = Video*/
const Value V21707_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21707_330 = {S47, 0, &V21707_330_57326, &P21707_329};	/*Property = Viewport*/
const Value V21707_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21707_332 = {S48, 0, &V21707_332_57339, &P21707_330};	/*Property = WebWorkers*/
const Value V21707_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21707_333 = {S49, 0, &V21707_333_57342, &P21707_332};	/*Property = Xhr2*/
const Profile P21707 = {"21707", &P21707_333};	/*Profile ID - 21707*/
const Value V21911_1_1 = {S2, NULL};	/*Value = True*/
const Property P21911_1 = {S50, 0, &V21911_1_1, NULL};	/*Property = IsMobile*/
const Value V21911_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21911_5 = {S52, 0, &V21911_5_37, &P21911_1};	/*Property = ScreenPixelsHeight*/
const Value V21911_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21911_4 = {S54, 0, &V21911_4_36, &P21911_5};	/*Property = ScreenPixelsWidth*/
const Profile P21911 = {"21911", &P21911_4};	/*Profile ID - 21911*/
const Value V21891_1_1 = {S2, NULL};	/*Value = True*/
const Property P21891_1 = {S50, 0, &V21891_1_1, NULL};	/*Property = IsMobile*/
const Value V21891_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21891_5 = {S52, 0, &V21891_5_1227, &P21891_1};	/*Property = ScreenPixelsHeight*/
const Value V21891_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21891_4 = {S54, 0, &V21891_4_1229, &P21891_5};	/*Property = ScreenPixelsWidth*/
const Profile P21891 = {"21891", &P21891_4};	/*Profile ID - 21891*/
const Value V21991_1_1 = {S2, NULL};	/*Value = True*/
const Property P21991_1 = {S50, 0, &V21991_1_1, NULL};	/*Property = IsMobile*/
const Value V21991_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21991_5 = {S52, 0, &V21991_5_1403, &P21991_1};	/*Property = ScreenPixelsHeight*/
const Value V21991_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21991_4 = {S54, 0, &V21991_4_1404, &P21991_5};	/*Property = ScreenPixelsWidth*/
const Profile P21991 = {"21991", &P21991_4};	/*Profile ID - 21991*/
const Value V21733_1_1 = {S2, NULL};	/*Value = True*/
const Property P21733_1 = {S50, 0, &V21733_1_1, NULL};	/*Property = IsMobile*/
const Value V21733_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21733_5 = {S52, 0, &V21733_5_1403, &P21733_1};	/*Property = ScreenPixelsHeight*/
const Value V21733_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21733_4 = {S54, 0, &V21733_4_1404, &P21733_5};	/*Property = ScreenPixelsWidth*/
const Profile P21733 = {"21733", &P21733_4};	/*Profile ID - 21733*/
const Value V21708_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21708_279 = {S1, 0, &V21708_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21708_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21708_282 = {S3, 0, &V21708_282_57154, &P21708_279};	/*Property = BlobBuilder*/
const Value V21708_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21708_350 = {S4, 0, &V21708_350_57444, &P21708_282};	/*Property = Canvas*/
const Value V21708_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21708_283 = {S5, 0, &V21708_283_57159, &P21708_350};	/*Property = CssBackground*/
const Value V21708_284_57163 = {S0, NULL};	/*Value = False*/
const Property P21708_284 = {S6, 0, &V21708_284_57163, &P21708_283};	/*Property = CssBorderImage*/
const Value V21708_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21708_285 = {S7, 0, &V21708_285_57166, &P21708_284};	/*Property = CssCanvas*/
const Value V21708_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21708_286 = {S8, 0, &V21708_286_57168, &P21708_285};	/*Property = CssColor*/
const Value V21708_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21708_237 = {S9, 0, &V21708_237_55786, &P21708_286};	/*Property = CssColumn*/
const Value V21708_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21708_287 = {S10, 0, &V21708_287_57172, &P21708_237};	/*Property = CssFlexbox*/
const Value V21708_288_57174 = {S2, NULL};	/*Value = True*/
const Property P21708_288 = {S11, 0, &V21708_288_57174, &P21708_287};	/*Property = CssFont*/
const Value V21708_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21708_289 = {S12, 0, &V21708_289_57759, &P21708_288};	/*Property = CssImages*/
const Value V21708_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21708_291 = {S13, 0, &V21708_291_57180, &P21708_289};	/*Property = CssMediaQueries*/
const Value V21708_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21708_292 = {S14, 0, &V21708_292_57183, &P21708_291};	/*Property = CssMinMax*/
const Value V21708_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21708_293 = {S15, 0, &V21708_293_57187, &P21708_292};	/*Property = CssOverflow*/
const Value V21708_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21708_294 = {S16, 0, &V21708_294_57189, &P21708_293};	/*Property = CssPosition*/
const Value V21708_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21708_295 = {S17, 0, &V21708_295_57192, &P21708_294};	/*Property = CssText*/
const Value V21708_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21708_297 = {S18, 0, &V21708_297_57195, &P21708_295};	/*Property = CssTransforms*/
const Value V21708_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21708_241 = {S19, 0, &V21708_241_55799, &P21708_297};	/*Property = CssTransitions*/
const Value V21708_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21708_298 = {S20, 0, &V21708_298_57206, &P21708_241};	/*Property = CssUI*/
const Value V21708_300_57216 = {S0, NULL};	/*Value = False*/
const Property P21708_300 = {S21, 0, &V21708_300_57216, &P21708_298};	/*Property = DataSet*/
const Value V21708_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21708_301 = {S22, 0, &V21708_301_57218, &P21708_300};	/*Property = DataUrl*/
const Value V21708_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21708_302 = {S23, 0, &V21708_302_57222, &P21708_301};	/*Property = DeviceOrientation*/
const Value V21708_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21708_304 = {S24, 0, &V21708_304_57227, &P21708_302};	/*Property = FileReader*/
const Value V21708_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21708_305 = {S25, 0, &V21708_305_57231, &P21708_304};	/*Property = FileSaver*/
const Value V21708_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21708_306 = {S26, 0, &V21708_306_57237, &P21708_305};	/*Property = FileWriter*/
const Value V21708_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21708_307 = {S27, 0, &V21708_307_57239, &P21708_306};	/*Property = FormData*/
const Value V21708_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21708_309 = {S28, 0, &V21708_309_57246, &P21708_307};	/*Property = Fullscreen*/
const Value V21708_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21708_310 = {S29, 0, &V21708_310_57248, &P21708_309};	/*Property = GeoLocation*/
const Value V21708_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21708_312 = {S30, 0, &V21708_312_57254, &P21708_310};	/*Property = History*/
const Value V21708_314_57261 = {S0, NULL};	/*Value = False*/
const Property P21708_314 = {S31, 0, &V21708_314_57261, &P21708_312};	/*Property = Html5*/
const Value V21708_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21708_313 = {S32, 0, &V21708_313_57258, &P21708_314};	/*Property = Html-Media-Capture*/
const Value V21708_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21708_315 = {S33, 0, &V21708_315_57264, &P21708_313};	/*Property = Iframe*/
const Value V21708_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21708_316 = {S34, 0, &V21708_316_57266, &P21708_315};	/*Property = IndexedDB*/
const Value V21708_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21708_317 = {S35, 0, &V21708_317_57269, &P21708_316};	/*Property = Json*/
const Value V21708_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P21708_15 = {S37, 0, &V21708_15_28, &P21708_317};	/*Property = LayoutEngine*/
const Value V21708_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21708_344 = {S38, 0, &V21708_344_57272, &P21708_15};	/*Property = Masking*/
const Value V21708_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21708_320 = {S39, 0, &V21708_320_57290, &P21708_344};	/*Property = PostMessage*/
const Value V21708_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21708_321 = {S40, 0, &V21708_321_57293, &P21708_320};	/*Property = Progress*/
const Value V21708_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21708_322 = {S41, 0, &V21708_322_57296, &P21708_321};	/*Property = Prompts*/
const Value V21708_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21708_323 = {S42, 0, &V21708_323_57299, &P21708_322};	/*Property = Selector*/
const Value V21708_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21708_324 = {S43, 0, &V21708_324_57306, &P21708_323};	/*Property = Svg*/
const Value V21708_327_57314 = {S2, NULL};	/*Value = True*/
const Property P21708_327 = {S44, 0, &V21708_327_57314, &P21708_324};	/*Property = TouchEvents*/
const Value V21708_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21708_328 = {S45, 0, &V21708_328_57318, &P21708_327};	/*Property = Track*/
const Value V21708_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21708_329 = {S46, 0, &V21708_329_57323, &P21708_328};	/*Property = Video*/
const Value V21708_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21708_330 = {S47, 0, &V21708_330_57326, &P21708_329};	/*Property = Viewport*/
const Value V21708_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21708_332 = {S48, 0, &V21708_332_57339, &P21708_330};	/*Property = WebWorkers*/
const Value V21708_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21708_333 = {S49, 0, &V21708_333_57342, &P21708_332};	/*Property = Xhr2*/
const Profile P21708 = {"21708", &P21708_333};	/*Profile ID - 21708*/
const Value V23176_1_1 = {S2, NULL};	/*Value = True*/
const Property P23176_1 = {S50, 0, &V23176_1_1, NULL};	/*Property = IsMobile*/
const Value V23176_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23176_5 = {S52, 0, &V23176_5_1269, &P23176_1};	/*Property = ScreenPixelsHeight*/
const Value V23176_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23176_4 = {S54, 0, &V23176_4_1229, &P23176_5};	/*Property = ScreenPixelsWidth*/
const Profile P23176 = {"23176", &P23176_4};	/*Profile ID - 23176*/
const Value V23447_1_1 = {S2, NULL};	/*Value = True*/
const Property P23447_1 = {S50, 0, &V23447_1_1, NULL};	/*Property = IsMobile*/
const Value V23447_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23447_5 = {S52, 0, &V23447_5_9981, &P23447_1};	/*Property = ScreenPixelsHeight*/
const Value V23447_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23447_4 = {S54, 0, &V23447_4_39098, &P23447_5};	/*Property = ScreenPixelsWidth*/
const Profile P23447 = {"23447", &P23447_4};	/*Profile ID - 23447*/
const Value V21703_1_1 = {S2, NULL};	/*Value = True*/
const Property P21703_1 = {S50, 0, &V21703_1_1, NULL};	/*Property = IsMobile*/
const Value V21703_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P21703_5 = {S52, 0, &V21703_5_1458, &P21703_1};	/*Property = ScreenPixelsHeight*/
const Value V21703_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21703_4 = {S54, 0, &V21703_4_1460, &P21703_5};	/*Property = ScreenPixelsWidth*/
const Profile P21703 = {"21703", &P21703_4};	/*Profile ID - 21703*/
const Value V23782_1_1 = {S2, NULL};	/*Value = True*/
const Property P23782_1 = {S50, 0, &V23782_1_1, NULL};	/*Property = IsMobile*/
const Value V23782_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23782_5 = {S52, 0, &V23782_5_1458, &P23782_1};	/*Property = ScreenPixelsHeight*/
const Value V23782_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23782_4 = {S54, 0, &V23782_4_1460, &P23782_5};	/*Property = ScreenPixelsWidth*/
const Profile P23782 = {"23782", &P23782_4};	/*Profile ID - 23782*/
const Value V21502_1_1 = {S2, NULL};	/*Value = True*/
const Property P21502_1 = {S50, 0, &V21502_1_1, NULL};	/*Property = IsMobile*/
const Value V21502_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21502_5 = {S52, 0, &V21502_5_1403, &P21502_1};	/*Property = ScreenPixelsHeight*/
const Value V21502_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P21502_4 = {S54, 0, &V21502_4_8554, &P21502_5};	/*Property = ScreenPixelsWidth*/
const Profile P21502 = {"21502", &P21502_4};	/*Profile ID - 21502*/
const Value V23476_1_1 = {S2, NULL};	/*Value = True*/
const Property P23476_1 = {S50, 0, &V23476_1_1, NULL};	/*Property = IsMobile*/
const Value V23476_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23476_5 = {S52, 0, &V23476_5_1277, &P23476_1};	/*Property = ScreenPixelsHeight*/
const Value V23476_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23476_4 = {S54, 0, &V23476_4_1279, &P23476_5};	/*Property = ScreenPixelsWidth*/
const Profile P23476 = {"23476", &P23476_4};	/*Profile ID - 23476*/
const Value V21985_1_1 = {S2, NULL};	/*Value = True*/
const Property P21985_1 = {S50, 0, &V21985_1_1, NULL};	/*Property = IsMobile*/
const Value V21985_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21985_5 = {S52, 0, &V21985_5_1277, &P21985_1};	/*Property = ScreenPixelsHeight*/
const Value V21985_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21985_4 = {S54, 0, &V21985_4_2336, &P21985_5};	/*Property = ScreenPixelsWidth*/
const Profile P21985 = {"21985", &P21985_4};	/*Profile ID - 21985*/
const Value V22944_1_1 = {S2, NULL};	/*Value = True*/
const Property P22944_1 = {S50, 0, &V22944_1_1, NULL};	/*Property = IsMobile*/
const Value V22944_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22944_5 = {S52, 0, &V22944_5_1403, &P22944_1};	/*Property = ScreenPixelsHeight*/
const Value V22944_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22944_4 = {S54, 0, &V22944_4_1404, &P22944_5};	/*Property = ScreenPixelsWidth*/
const Profile P22944 = {"22944", &P22944_4};	/*Profile ID - 22944*/
const Value V21856_1_1 = {S2, NULL};	/*Value = True*/
const Property P21856_1 = {S50, 0, &V21856_1_1, NULL};	/*Property = IsMobile*/
const Value V21856_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21856_5 = {S52, 0, &V21856_5_1227, &P21856_1};	/*Property = ScreenPixelsHeight*/
const Value V21856_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21856_4 = {S54, 0, &V21856_4_1229, &P21856_5};	/*Property = ScreenPixelsWidth*/
const Profile P21856 = {"21856", &P21856_4};	/*Profile ID - 21856*/
const Value V23122_1_1 = {S2, NULL};	/*Value = True*/
const Property P23122_1 = {S50, 0, &V23122_1_1, NULL};	/*Property = IsMobile*/
const Value V23122_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23122_5 = {S52, 0, &V23122_5_1227, &P23122_1};	/*Property = ScreenPixelsHeight*/
const Value V23122_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23122_4 = {S54, 0, &V23122_4_1229, &P23122_5};	/*Property = ScreenPixelsWidth*/
const Profile P23122 = {"23122", &P23122_4};	/*Profile ID - 23122*/
const Value V21881_1_1 = {S2, NULL};	/*Value = True*/
const Property P21881_1 = {S50, 0, &V21881_1_1, NULL};	/*Property = IsMobile*/
const Value V21881_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21881_5 = {S52, 0, &V21881_5_1277, &P21881_1};	/*Property = ScreenPixelsHeight*/
const Value V21881_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21881_4 = {S54, 0, &V21881_4_1279, &P21881_5};	/*Property = ScreenPixelsWidth*/
const Profile P21881 = {"21881", &P21881_4};	/*Profile ID - 21881*/
const Value V21187_1_1 = {S2, NULL};	/*Value = True*/
const Property P21187_1 = {S50, 0, &V21187_1_1, NULL};	/*Property = IsMobile*/
const Value V21187_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21187_5 = {S52, 0, &V21187_5_37, &P21187_1};	/*Property = ScreenPixelsHeight*/
const Value V21187_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21187_4 = {S54, 0, &V21187_4_36, &P21187_5};	/*Property = ScreenPixelsWidth*/
const Profile P21187 = {"21187", &P21187_4};	/*Profile ID - 21187*/
const Value V21882_1_1 = {S2, NULL};	/*Value = True*/
const Property P21882_1 = {S50, 0, &V21882_1_1, NULL};	/*Property = IsMobile*/
const Value V21882_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21882_5 = {S52, 0, &V21882_5_1227, &P21882_1};	/*Property = ScreenPixelsHeight*/
const Value V21882_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21882_4 = {S54, 0, &V21882_4_1229, &P21882_5};	/*Property = ScreenPixelsWidth*/
const Profile P21882 = {"21882", &P21882_4};	/*Profile ID - 21882*/
const Value V22593_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22593_279 = {S1, 0, &V22593_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22593_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22593_282 = {S3, 0, &V22593_282_57154, &P22593_279};	/*Property = BlobBuilder*/
const Value V22593_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22593_350 = {S4, 0, &V22593_350_57444, &P22593_282};	/*Property = Canvas*/
const Value V22593_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22593_283 = {S5, 0, &V22593_283_57159, &P22593_350};	/*Property = CssBackground*/
const Value V22593_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22593_284 = {S6, 0, &V22593_284_57162, &P22593_283};	/*Property = CssBorderImage*/
const Value V22593_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22593_285 = {S7, 0, &V22593_285_57166, &P22593_284};	/*Property = CssCanvas*/
const Value V22593_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22593_286 = {S8, 0, &V22593_286_57168, &P22593_285};	/*Property = CssColor*/
const Value V22593_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22593_237 = {S9, 0, &V22593_237_55786, &P22593_286};	/*Property = CssColumn*/
const Value V22593_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22593_287 = {S10, 0, &V22593_287_57172, &P22593_237};	/*Property = CssFlexbox*/
const Value V22593_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22593_288 = {S11, 0, &V22593_288_57175, &P22593_287};	/*Property = CssFont*/
const Value V22593_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22593_289 = {S12, 0, &V22593_289_57759, &P22593_288};	/*Property = CssImages*/
const Value V22593_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22593_291 = {S13, 0, &V22593_291_57181, &P22593_289};	/*Property = CssMediaQueries*/
const Value V22593_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22593_292 = {S14, 0, &V22593_292_57183, &P22593_291};	/*Property = CssMinMax*/
const Value V22593_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22593_293 = {S15, 0, &V22593_293_57187, &P22593_292};	/*Property = CssOverflow*/
const Value V22593_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22593_294 = {S16, 0, &V22593_294_57189, &P22593_293};	/*Property = CssPosition*/
const Value V22593_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22593_295 = {S17, 0, &V22593_295_57192, &P22593_294};	/*Property = CssText*/
const Value V22593_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22593_297 = {S18, 0, &V22593_297_57195, &P22593_295};	/*Property = CssTransforms*/
const Value V22593_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22593_241 = {S19, 0, &V22593_241_55799, &P22593_297};	/*Property = CssTransitions*/
const Value V22593_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22593_298 = {S20, 0, &V22593_298_57206, &P22593_241};	/*Property = CssUI*/
const Value V22593_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22593_300 = {S21, 0, &V22593_300_57215, &P22593_298};	/*Property = DataSet*/
const Value V22593_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22593_301 = {S22, 0, &V22593_301_57218, &P22593_300};	/*Property = DataUrl*/
const Value V22593_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22593_302 = {S23, 0, &V22593_302_57222, &P22593_301};	/*Property = DeviceOrientation*/
const Value V22593_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22593_304 = {S24, 0, &V22593_304_57227, &P22593_302};	/*Property = FileReader*/
const Value V22593_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22593_305 = {S25, 0, &V22593_305_57231, &P22593_304};	/*Property = FileSaver*/
const Value V22593_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22593_306 = {S26, 0, &V22593_306_57237, &P22593_305};	/*Property = FileWriter*/
const Value V22593_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22593_307 = {S27, 0, &V22593_307_57239, &P22593_306};	/*Property = FormData*/
const Value V22593_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22593_309 = {S28, 0, &V22593_309_57246, &P22593_307};	/*Property = Fullscreen*/
const Value V22593_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22593_310 = {S29, 0, &V22593_310_57248, &P22593_309};	/*Property = GeoLocation*/
const Value V22593_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22593_312 = {S30, 0, &V22593_312_57254, &P22593_310};	/*Property = History*/
const Value V22593_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22593_314 = {S31, 0, &V22593_314_57261, &P22593_312};	/*Property = Html5*/
const Value V22593_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22593_313 = {S32, 0, &V22593_313_57258, &P22593_314};	/*Property = Html-Media-Capture*/
const Value V22593_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22593_315 = {S33, 0, &V22593_315_57264, &P22593_313};	/*Property = Iframe*/
const Value V22593_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22593_316 = {S34, 0, &V22593_316_57267, &P22593_315};	/*Property = IndexedDB*/
const Value V22593_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22593_317 = {S35, 0, &V22593_317_57269, &P22593_316};	/*Property = Json*/
const Value V22593_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22593_15 = {S37, 0, &V22593_15_28, &P22593_317};	/*Property = LayoutEngine*/
const Value V22593_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22593_344 = {S38, 0, &V22593_344_57272, &P22593_15};	/*Property = Masking*/
const Value V22593_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22593_320 = {S39, 0, &V22593_320_57290, &P22593_344};	/*Property = PostMessage*/
const Value V22593_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22593_321 = {S40, 0, &V22593_321_57293, &P22593_320};	/*Property = Progress*/
const Value V22593_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22593_322 = {S41, 0, &V22593_322_57296, &P22593_321};	/*Property = Prompts*/
const Value V22593_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22593_323 = {S42, 0, &V22593_323_57299, &P22593_322};	/*Property = Selector*/
const Value V22593_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22593_324 = {S43, 0, &V22593_324_57306, &P22593_323};	/*Property = Svg*/
const Value V22593_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22593_327 = {S44, 0, &V22593_327_57315, &P22593_324};	/*Property = TouchEvents*/
const Value V22593_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22593_328 = {S45, 0, &V22593_328_57318, &P22593_327};	/*Property = Track*/
const Value V22593_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22593_329 = {S46, 0, &V22593_329_57323, &P22593_328};	/*Property = Video*/
const Value V22593_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22593_330 = {S47, 0, &V22593_330_57326, &P22593_329};	/*Property = Viewport*/
const Value V22593_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22593_332 = {S48, 0, &V22593_332_57339, &P22593_330};	/*Property = WebWorkers*/
const Value V22593_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22593_333 = {S49, 0, &V22593_333_57342, &P22593_332};	/*Property = Xhr2*/
const Profile P22593 = {"22593", &P22593_333};	/*Profile ID - 22593*/
const Value V23846_1_1 = {S2, NULL};	/*Value = True*/
const Property P23846_1 = {S50, 0, &V23846_1_1, NULL};	/*Property = IsMobile*/
const Value V23846_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23846_5 = {S52, 0, &V23846_5_1269, &P23846_1};	/*Property = ScreenPixelsHeight*/
const Value V23846_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23846_4 = {S54, 0, &V23846_4_1229, &P23846_5};	/*Property = ScreenPixelsWidth*/
const Profile P23846 = {"23846", &P23846_4};	/*Profile ID - 23846*/
const Value V23741_1_1 = {S2, NULL};	/*Value = True*/
const Property P23741_1 = {S50, 0, &V23741_1_1, NULL};	/*Property = IsMobile*/
const Value V23741_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23741_5 = {S52, 0, &V23741_5_1458, &P23741_1};	/*Property = ScreenPixelsHeight*/
const Value V23741_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23741_4 = {S54, 0, &V23741_4_37617, &P23741_5};	/*Property = ScreenPixelsWidth*/
const Profile P23741 = {"23741", &P23741_4};	/*Profile ID - 23741*/
const Value V22757_1_1 = {S2, NULL};	/*Value = True*/
const Property P22757_1 = {S50, 0, &V22757_1_1, NULL};	/*Property = IsMobile*/
const Value V22757_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22757_5 = {S52, 0, &V22757_5_1277, &P22757_1};	/*Property = ScreenPixelsHeight*/
const Value V22757_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22757_4 = {S54, 0, &V22757_4_1279, &P22757_5};	/*Property = ScreenPixelsWidth*/
const Profile P22757 = {"22757", &P22757_4};	/*Profile ID - 22757*/
const Value V22729_1_1 = {S2, NULL};	/*Value = True*/
const Property P22729_1 = {S50, 0, &V22729_1_1, NULL};	/*Property = IsMobile*/
const Value V22729_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22729_5 = {S52, 0, &V22729_5_1277, &P22729_1};	/*Property = ScreenPixelsHeight*/
const Value V22729_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22729_4 = {S54, 0, &V22729_4_1279, &P22729_5};	/*Property = ScreenPixelsWidth*/
const Profile P22729 = {"22729", &P22729_4};	/*Profile ID - 22729*/
const Value V23202_1_1 = {S2, NULL};	/*Value = True*/
const Property P23202_1 = {S50, 0, &V23202_1_1, NULL};	/*Property = IsMobile*/
const Value V23202_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23202_5 = {S52, 0, &V23202_5_9981, &P23202_1};	/*Property = ScreenPixelsHeight*/
const Value V23202_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23202_4 = {S54, 0, &V23202_4_37617, &P23202_5};	/*Property = ScreenPixelsWidth*/
const Profile P23202 = {"23202", &P23202_4};	/*Profile ID - 23202*/
const Value V21934_279_57148 = {S0, NULL};	/*Value = False*/
const Property P21934_279 = {S1, 0, &V21934_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V21934_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21934_282 = {S3, 0, &V21934_282_57154, &P21934_279};	/*Property = BlobBuilder*/
const Value V21934_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21934_350 = {S4, 0, &V21934_350_57444, &P21934_282};	/*Property = Canvas*/
const Value V21934_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21934_283 = {S5, 0, &V21934_283_57159, &P21934_350};	/*Property = CssBackground*/
const Value V21934_284_57162 = {S2, NULL};	/*Value = True*/
const Property P21934_284 = {S6, 0, &V21934_284_57162, &P21934_283};	/*Property = CssBorderImage*/
const Value V21934_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21934_285 = {S7, 0, &V21934_285_57166, &P21934_284};	/*Property = CssCanvas*/
const Value V21934_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21934_286 = {S8, 0, &V21934_286_57168, &P21934_285};	/*Property = CssColor*/
const Value V21934_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21934_237 = {S9, 0, &V21934_237_55786, &P21934_286};	/*Property = CssColumn*/
const Value V21934_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21934_287 = {S10, 0, &V21934_287_57172, &P21934_237};	/*Property = CssFlexbox*/
const Value V21934_288_57174 = {S2, NULL};	/*Value = True*/
const Property P21934_288 = {S11, 0, &V21934_288_57174, &P21934_287};	/*Property = CssFont*/
const Value V21934_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21934_289 = {S12, 0, &V21934_289_57759, &P21934_288};	/*Property = CssImages*/
const Value V21934_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21934_291 = {S13, 0, &V21934_291_57180, &P21934_289};	/*Property = CssMediaQueries*/
const Value V21934_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21934_292 = {S14, 0, &V21934_292_57183, &P21934_291};	/*Property = CssMinMax*/
const Value V21934_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21934_293 = {S15, 0, &V21934_293_57187, &P21934_292};	/*Property = CssOverflow*/
const Value V21934_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21934_294 = {S16, 0, &V21934_294_57189, &P21934_293};	/*Property = CssPosition*/
const Value V21934_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21934_295 = {S17, 0, &V21934_295_57192, &P21934_294};	/*Property = CssText*/
const Value V21934_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21934_297 = {S18, 0, &V21934_297_57195, &P21934_295};	/*Property = CssTransforms*/
const Value V21934_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21934_241 = {S19, 0, &V21934_241_55799, &P21934_297};	/*Property = CssTransitions*/
const Value V21934_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21934_298 = {S20, 0, &V21934_298_57206, &P21934_241};	/*Property = CssUI*/
const Value V21934_300_57215 = {S2, NULL};	/*Value = True*/
const Property P21934_300 = {S21, 0, &V21934_300_57215, &P21934_298};	/*Property = DataSet*/
const Value V21934_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21934_301 = {S22, 0, &V21934_301_57218, &P21934_300};	/*Property = DataUrl*/
const Value V21934_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21934_302 = {S23, 0, &V21934_302_57222, &P21934_301};	/*Property = DeviceOrientation*/
const Value V21934_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21934_304 = {S24, 0, &V21934_304_57227, &P21934_302};	/*Property = FileReader*/
const Value V21934_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21934_305 = {S25, 0, &V21934_305_57231, &P21934_304};	/*Property = FileSaver*/
const Value V21934_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21934_306 = {S26, 0, &V21934_306_57237, &P21934_305};	/*Property = FileWriter*/
const Value V21934_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21934_307 = {S27, 0, &V21934_307_57239, &P21934_306};	/*Property = FormData*/
const Value V21934_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21934_309 = {S28, 0, &V21934_309_57246, &P21934_307};	/*Property = Fullscreen*/
const Value V21934_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21934_310 = {S29, 0, &V21934_310_57248, &P21934_309};	/*Property = GeoLocation*/
const Value V21934_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21934_312 = {S30, 0, &V21934_312_57254, &P21934_310};	/*Property = History*/
const Value V21934_314_57260 = {S2, NULL};	/*Value = True*/
const Property P21934_314 = {S31, 0, &V21934_314_57260, &P21934_312};	/*Property = Html5*/
const Value V21934_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21934_313 = {S32, 0, &V21934_313_57258, &P21934_314};	/*Property = Html-Media-Capture*/
const Value V21934_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21934_315 = {S33, 0, &V21934_315_57264, &P21934_313};	/*Property = Iframe*/
const Value V21934_316_57266 = {S2, NULL};	/*Value = True*/
const Property P21934_316 = {S34, 0, &V21934_316_57266, &P21934_315};	/*Property = IndexedDB*/
const Value V21934_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21934_317 = {S35, 0, &V21934_317_57269, &P21934_316};	/*Property = Json*/
const Value V21934_15_81502 = {S141, NULL};	/*Value = Blink*/
const Property P21934_15 = {S37, 0, &V21934_15_81502, &P21934_317};	/*Property = LayoutEngine*/
const Value V21934_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21934_344 = {S38, 0, &V21934_344_57272, &P21934_15};	/*Property = Masking*/
const Value V21934_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21934_320 = {S39, 0, &V21934_320_57290, &P21934_344};	/*Property = PostMessage*/
const Value V21934_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21934_321 = {S40, 0, &V21934_321_57293, &P21934_320};	/*Property = Progress*/
const Value V21934_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21934_322 = {S41, 0, &V21934_322_57296, &P21934_321};	/*Property = Prompts*/
const Value V21934_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21934_323 = {S42, 0, &V21934_323_57299, &P21934_322};	/*Property = Selector*/
const Value V21934_324_57305 = {S2, NULL};	/*Value = True*/
const Property P21934_324 = {S43, 0, &V21934_324_57305, &P21934_323};	/*Property = Svg*/
const Value V21934_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21934_327 = {S44, 0, &V21934_327_57315, &P21934_324};	/*Property = TouchEvents*/
const Value V21934_328_57317 = {S2, NULL};	/*Value = True*/
const Property P21934_328 = {S45, 0, &V21934_328_57317, &P21934_327};	/*Property = Track*/
const Value V21934_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21934_329 = {S46, 0, &V21934_329_57323, &P21934_328};	/*Property = Video*/
const Value V21934_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21934_330 = {S47, 0, &V21934_330_57326, &P21934_329};	/*Property = Viewport*/
const Value V21934_332_57338 = {S2, NULL};	/*Value = True*/
const Property P21934_332 = {S48, 0, &V21934_332_57338, &P21934_330};	/*Property = WebWorkers*/
const Value V21934_333_57341 = {S2, NULL};	/*Value = True*/
const Property P21934_333 = {S49, 0, &V21934_333_57341, &P21934_332};	/*Property = Xhr2*/
const Profile P21934 = {"21934", &P21934_333};	/*Profile ID - 21934*/
const Value V23311_1_1 = {S2, NULL};	/*Value = True*/
const Property P23311_1 = {S50, 0, &V23311_1_1, NULL};	/*Property = IsMobile*/
const Value V23311_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23311_5 = {S52, 0, &V23311_5_1458, &P23311_1};	/*Property = ScreenPixelsHeight*/
const Value V23311_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23311_4 = {S54, 0, &V23311_4_37617, &P23311_5};	/*Property = ScreenPixelsWidth*/
const Profile P23311 = {"23311", &P23311_4};	/*Profile ID - 23311*/
const Value V23536_1_1 = {S2, NULL};	/*Value = True*/
const Property P23536_1 = {S50, 0, &V23536_1_1, NULL};	/*Property = IsMobile*/
const Value V23536_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23536_5 = {S52, 0, &V23536_5_37, &P23536_1};	/*Property = ScreenPixelsHeight*/
const Value V23536_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23536_4 = {S54, 0, &V23536_4_36, &P23536_5};	/*Property = ScreenPixelsWidth*/
const Profile P23536 = {"23536", &P23536_4};	/*Profile ID - 23536*/
const Value V21968_1_1 = {S2, NULL};	/*Value = True*/
const Property P21968_1 = {S50, 0, &V21968_1_1, NULL};	/*Property = IsMobile*/
const Value V21968_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21968_5 = {S52, 0, &V21968_5_1277, &P21968_1};	/*Property = ScreenPixelsHeight*/
const Value V21968_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P21968_4 = {S54, 0, &V21968_4_1279, &P21968_5};	/*Property = ScreenPixelsWidth*/
const Profile P21968 = {"21968", &P21968_4};	/*Profile ID - 21968*/
const Value V23582_1_1 = {S2, NULL};	/*Value = True*/
const Property P23582_1 = {S50, 0, &V23582_1_1, NULL};	/*Property = IsMobile*/
const Value V23582_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23582_5 = {S52, 0, &V23582_5_1403, &P23582_1};	/*Property = ScreenPixelsHeight*/
const Value V23582_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23582_4 = {S54, 0, &V23582_4_1404, &P23582_5};	/*Property = ScreenPixelsWidth*/
const Profile P23582 = {"23582", &P23582_4};	/*Profile ID - 23582*/
const Value V23238_1_1 = {S2, NULL};	/*Value = True*/
const Property P23238_1 = {S50, 0, &V23238_1_1, NULL};	/*Property = IsMobile*/
const Value V23238_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23238_5 = {S52, 0, &V23238_5_1227, &P23238_1};	/*Property = ScreenPixelsHeight*/
const Value V23238_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P23238_4 = {S54, 0, &V23238_4_1477, &P23238_5};	/*Property = ScreenPixelsWidth*/
const Profile P23238 = {"23238", &P23238_4};	/*Profile ID - 23238*/
const Value V23809_1_1 = {S2, NULL};	/*Value = True*/
const Property P23809_1 = {S50, 0, &V23809_1_1, NULL};	/*Property = IsMobile*/
const Value V23809_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23809_5 = {S52, 0, &V23809_5_1227, &P23809_1};	/*Property = ScreenPixelsHeight*/
const Value V23809_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23809_4 = {S54, 0, &V23809_4_1229, &P23809_5};	/*Property = ScreenPixelsWidth*/
const Profile P23809 = {"23809", &P23809_4};	/*Profile ID - 23809*/
const Value V21783_279_57147 = {S2, NULL};	/*Value = True*/
const Property P21783_279 = {S1, 0, &V21783_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V21783_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21783_282 = {S3, 0, &V21783_282_57154, &P21783_279};	/*Property = BlobBuilder*/
const Value V21783_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21783_350 = {S4, 0, &V21783_350_57444, &P21783_282};	/*Property = Canvas*/
const Value V21783_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21783_283 = {S5, 0, &V21783_283_57159, &P21783_350};	/*Property = CssBackground*/
const Value V21783_284_57162 = {S2, NULL};	/*Value = True*/
const Property P21783_284 = {S6, 0, &V21783_284_57162, &P21783_283};	/*Property = CssBorderImage*/
const Value V21783_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21783_285 = {S7, 0, &V21783_285_57166, &P21783_284};	/*Property = CssCanvas*/
const Value V21783_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21783_286 = {S8, 0, &V21783_286_57168, &P21783_285};	/*Property = CssColor*/
const Value V21783_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21783_237 = {S9, 0, &V21783_237_55786, &P21783_286};	/*Property = CssColumn*/
const Value V21783_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21783_287 = {S10, 0, &V21783_287_57172, &P21783_237};	/*Property = CssFlexbox*/
const Value V21783_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21783_288 = {S11, 0, &V21783_288_57175, &P21783_287};	/*Property = CssFont*/
const Value V21783_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21783_289 = {S12, 0, &V21783_289_57759, &P21783_288};	/*Property = CssImages*/
const Value V21783_291_57181 = {S0, NULL};	/*Value = False*/
const Property P21783_291 = {S13, 0, &V21783_291_57181, &P21783_289};	/*Property = CssMediaQueries*/
const Value V21783_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21783_292 = {S14, 0, &V21783_292_57183, &P21783_291};	/*Property = CssMinMax*/
const Value V21783_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21783_293 = {S15, 0, &V21783_293_57187, &P21783_292};	/*Property = CssOverflow*/
const Value V21783_294_57190 = {S0, NULL};	/*Value = False*/
const Property P21783_294 = {S16, 0, &V21783_294_57190, &P21783_293};	/*Property = CssPosition*/
const Value V21783_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21783_295 = {S17, 0, &V21783_295_57192, &P21783_294};	/*Property = CssText*/
const Value V21783_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21783_297 = {S18, 0, &V21783_297_57195, &P21783_295};	/*Property = CssTransforms*/
const Value V21783_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21783_241 = {S19, 0, &V21783_241_55799, &P21783_297};	/*Property = CssTransitions*/
const Value V21783_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21783_298 = {S20, 0, &V21783_298_57206, &P21783_241};	/*Property = CssUI*/
const Value V21783_300_57215 = {S2, NULL};	/*Value = True*/
const Property P21783_300 = {S21, 0, &V21783_300_57215, &P21783_298};	/*Property = DataSet*/
const Value V21783_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21783_301 = {S22, 0, &V21783_301_57218, &P21783_300};	/*Property = DataUrl*/
const Value V21783_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21783_302 = {S23, 0, &V21783_302_57222, &P21783_301};	/*Property = DeviceOrientation*/
const Value V21783_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21783_304 = {S24, 0, &V21783_304_57227, &P21783_302};	/*Property = FileReader*/
const Value V21783_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21783_305 = {S25, 0, &V21783_305_57231, &P21783_304};	/*Property = FileSaver*/
const Value V21783_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21783_306 = {S26, 0, &V21783_306_57237, &P21783_305};	/*Property = FileWriter*/
const Value V21783_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21783_307 = {S27, 0, &V21783_307_57239, &P21783_306};	/*Property = FormData*/
const Value V21783_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21783_309 = {S28, 0, &V21783_309_57246, &P21783_307};	/*Property = Fullscreen*/
const Value V21783_310_57248 = {S2, NULL};	/*Value = True*/
const Property P21783_310 = {S29, 0, &V21783_310_57248, &P21783_309};	/*Property = GeoLocation*/
const Value V21783_312_57255 = {S0, NULL};	/*Value = False*/
const Property P21783_312 = {S30, 0, &V21783_312_57255, &P21783_310};	/*Property = History*/
const Value V21783_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P21783_314 = {S31, 0, &V21783_314_57262, &P21783_312};	/*Property = Html5*/
const Value V21783_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21783_313 = {S32, 0, &V21783_313_57258, &P21783_314};	/*Property = Html-Media-Capture*/
const Value V21783_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21783_315 = {S33, 0, &V21783_315_57264, &P21783_313};	/*Property = Iframe*/
const Value V21783_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21783_316 = {S34, 0, &V21783_316_57267, &P21783_315};	/*Property = IndexedDB*/
const Value V21783_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21783_317 = {S35, 0, &V21783_317_57269, &P21783_316};	/*Property = Json*/
const Value V21783_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P21783_15 = {S37, 0, &V21783_15_30, &P21783_317};	/*Property = LayoutEngine*/
const Value V21783_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21783_344 = {S38, 0, &V21783_344_57272, &P21783_15};	/*Property = Masking*/
const Value V21783_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21783_320 = {S39, 0, &V21783_320_57290, &P21783_344};	/*Property = PostMessage*/
const Value V21783_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21783_321 = {S40, 0, &V21783_321_57293, &P21783_320};	/*Property = Progress*/
const Value V21783_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21783_322 = {S41, 0, &V21783_322_57296, &P21783_321};	/*Property = Prompts*/
const Value V21783_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21783_323 = {S42, 0, &V21783_323_57299, &P21783_322};	/*Property = Selector*/
const Value V21783_324_57306 = {S0, NULL};	/*Value = False*/
const Property P21783_324 = {S43, 0, &V21783_324_57306, &P21783_323};	/*Property = Svg*/
const Value V21783_327_57315 = {S0, NULL};	/*Value = False*/
const Property P21783_327 = {S44, 0, &V21783_327_57315, &P21783_324};	/*Property = TouchEvents*/
const Value V21783_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21783_328 = {S45, 0, &V21783_328_57318, &P21783_327};	/*Property = Track*/
const Value V21783_329_57323 = {S2, NULL};	/*Value = True*/
const Property P21783_329 = {S46, 0, &V21783_329_57323, &P21783_328};	/*Property = Video*/
const Value V21783_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21783_330 = {S47, 0, &V21783_330_57326, &P21783_329};	/*Property = Viewport*/
const Value V21783_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P21783_332 = {S48, 0, &V21783_332_57340, &P21783_330};	/*Property = WebWorkers*/
const Value V21783_333_57342 = {S0, NULL};	/*Value = False*/
const Property P21783_333 = {S49, 0, &V21783_333_57342, &P21783_332};	/*Property = Xhr2*/
const Profile P21783 = {"21783", &P21783_333};	/*Profile ID - 21783*/
const Value V23366_1_1 = {S2, NULL};	/*Value = True*/
const Property P23366_1 = {S50, 0, &V23366_1_1, NULL};	/*Property = IsMobile*/
const Value V23366_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23366_5 = {S52, 0, &V23366_5_1277, &P23366_1};	/*Property = ScreenPixelsHeight*/
const Value V23366_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23366_4 = {S54, 0, &V23366_4_1279, &P23366_5};	/*Property = ScreenPixelsWidth*/
const Profile P23366 = {"23366", &P23366_4};	/*Profile ID - 23366*/
const Value V21824_1_1 = {S2, NULL};	/*Value = True*/
const Property P21824_1 = {S50, 0, &V21824_1_1, NULL};	/*Property = IsMobile*/
const Value V21824_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P21824_5 = {S52, 0, &V21824_5_1277, &P21824_1};	/*Property = ScreenPixelsHeight*/
const Value V21824_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P21824_4 = {S54, 0, &V21824_4_2336, &P21824_5};	/*Property = ScreenPixelsWidth*/
const Profile P21824 = {"21824", &P21824_4};	/*Profile ID - 21824*/
const Value V23310_1_1 = {S2, NULL};	/*Value = True*/
const Property P23310_1 = {S50, 0, &V23310_1_1, NULL};	/*Property = IsMobile*/
const Value V23310_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23310_5 = {S52, 0, &V23310_5_1227, &P23310_1};	/*Property = ScreenPixelsHeight*/
const Value V23310_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P23310_4 = {S54, 0, &V23310_4_1477, &P23310_5};	/*Property = ScreenPixelsWidth*/
const Profile P23310 = {"23310", &P23310_4};	/*Profile ID - 23310*/
const Value V23853_1_1 = {S2, NULL};	/*Value = True*/
const Property P23853_1 = {S50, 0, &V23853_1_1, NULL};	/*Property = IsMobile*/
const Value V23853_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23853_5 = {S52, 0, &V23853_5_37569, &P23853_1};	/*Property = ScreenPixelsHeight*/
const Value V23853_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23853_4 = {S54, 0, &V23853_4_5645, &P23853_5};	/*Property = ScreenPixelsWidth*/
const Profile P23853 = {"23853", &P23853_4};	/*Profile ID - 23853*/
const Value V22245_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22245_279 = {S1, 0, &V22245_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22245_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22245_282 = {S3, 0, &V22245_282_57154, &P22245_279};	/*Property = BlobBuilder*/
const Value V22245_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22245_350 = {S4, 0, &V22245_350_57444, &P22245_282};	/*Property = Canvas*/
const Value V22245_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22245_283 = {S5, 0, &V22245_283_57159, &P22245_350};	/*Property = CssBackground*/
const Value V22245_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22245_284 = {S6, 0, &V22245_284_57163, &P22245_283};	/*Property = CssBorderImage*/
const Value V22245_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22245_285 = {S7, 0, &V22245_285_57166, &P22245_284};	/*Property = CssCanvas*/
const Value V22245_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22245_286 = {S8, 0, &V22245_286_57168, &P22245_285};	/*Property = CssColor*/
const Value V22245_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22245_237 = {S9, 0, &V22245_237_55786, &P22245_286};	/*Property = CssColumn*/
const Value V22245_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22245_287 = {S10, 0, &V22245_287_57172, &P22245_237};	/*Property = CssFlexbox*/
const Value V22245_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22245_288 = {S11, 0, &V22245_288_57175, &P22245_287};	/*Property = CssFont*/
const Value V22245_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22245_289 = {S12, 0, &V22245_289_57759, &P22245_288};	/*Property = CssImages*/
const Value V22245_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22245_291 = {S13, 0, &V22245_291_57181, &P22245_289};	/*Property = CssMediaQueries*/
const Value V22245_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22245_292 = {S14, 0, &V22245_292_57183, &P22245_291};	/*Property = CssMinMax*/
const Value V22245_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22245_293 = {S15, 0, &V22245_293_57187, &P22245_292};	/*Property = CssOverflow*/
const Value V22245_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22245_294 = {S16, 0, &V22245_294_57190, &P22245_293};	/*Property = CssPosition*/
const Value V22245_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22245_295 = {S17, 0, &V22245_295_57192, &P22245_294};	/*Property = CssText*/
const Value V22245_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22245_297 = {S18, 0, &V22245_297_57195, &P22245_295};	/*Property = CssTransforms*/
const Value V22245_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22245_241 = {S19, 0, &V22245_241_55799, &P22245_297};	/*Property = CssTransitions*/
const Value V22245_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22245_298 = {S20, 0, &V22245_298_57206, &P22245_241};	/*Property = CssUI*/
const Value V22245_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22245_300 = {S21, 0, &V22245_300_57216, &P22245_298};	/*Property = DataSet*/
const Value V22245_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22245_301 = {S22, 0, &V22245_301_57218, &P22245_300};	/*Property = DataUrl*/
const Value V22245_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22245_302 = {S23, 0, &V22245_302_57222, &P22245_301};	/*Property = DeviceOrientation*/
const Value V22245_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22245_304 = {S24, 0, &V22245_304_57228, &P22245_302};	/*Property = FileReader*/
const Value V22245_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22245_305 = {S25, 0, &V22245_305_57231, &P22245_304};	/*Property = FileSaver*/
const Value V22245_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22245_306 = {S26, 0, &V22245_306_57237, &P22245_305};	/*Property = FileWriter*/
const Value V22245_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22245_307 = {S27, 0, &V22245_307_57240, &P22245_306};	/*Property = FormData*/
const Value V22245_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22245_309 = {S28, 0, &V22245_309_57246, &P22245_307};	/*Property = Fullscreen*/
const Value V22245_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22245_310 = {S29, 0, &V22245_310_57248, &P22245_309};	/*Property = GeoLocation*/
const Value V22245_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22245_312 = {S30, 0, &V22245_312_57255, &P22245_310};	/*Property = History*/
const Value V22245_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22245_314 = {S31, 0, &V22245_314_57261, &P22245_312};	/*Property = Html5*/
const Value V22245_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22245_313 = {S32, 0, &V22245_313_57258, &P22245_314};	/*Property = Html-Media-Capture*/
const Value V22245_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22245_315 = {S33, 0, &V22245_315_57264, &P22245_313};	/*Property = Iframe*/
const Value V22245_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22245_316 = {S34, 0, &V22245_316_57267, &P22245_315};	/*Property = IndexedDB*/
const Value V22245_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22245_317 = {S35, 0, &V22245_317_57269, &P22245_316};	/*Property = Json*/
const Value V22245_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22245_15 = {S37, 0, &V22245_15_72, &P22245_317};	/*Property = LayoutEngine*/
const Value V22245_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22245_344 = {S38, 0, &V22245_344_57272, &P22245_15};	/*Property = Masking*/
const Value V22245_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22245_320 = {S39, 0, &V22245_320_57290, &P22245_344};	/*Property = PostMessage*/
const Value V22245_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22245_321 = {S40, 0, &V22245_321_57293, &P22245_320};	/*Property = Progress*/
const Value V22245_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22245_322 = {S41, 0, &V22245_322_57296, &P22245_321};	/*Property = Prompts*/
const Value V22245_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22245_323 = {S42, 0, &V22245_323_57299, &P22245_322};	/*Property = Selector*/
const Value V22245_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22245_324 = {S43, 0, &V22245_324_57306, &P22245_323};	/*Property = Svg*/
const Value V22245_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22245_327 = {S44, 0, &V22245_327_57315, &P22245_324};	/*Property = TouchEvents*/
const Value V22245_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22245_328 = {S45, 0, &V22245_328_57318, &P22245_327};	/*Property = Track*/
const Value V22245_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22245_329 = {S46, 0, &V22245_329_57323, &P22245_328};	/*Property = Video*/
const Value V22245_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22245_330 = {S47, 0, &V22245_330_57326, &P22245_329};	/*Property = Viewport*/
const Value V22245_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22245_332 = {S48, 0, &V22245_332_57339, &P22245_330};	/*Property = WebWorkers*/
const Value V22245_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22245_333 = {S49, 0, &V22245_333_57342, &P22245_332};	/*Property = Xhr2*/
const Profile P22245 = {"22245", &P22245_333};	/*Profile ID - 22245*/
const Value V22060_1_1 = {S2, NULL};	/*Value = True*/
const Property P22060_1 = {S50, 0, &V22060_1_1, NULL};	/*Property = IsMobile*/
const Value V22060_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P22060_5 = {S52, 0, &V22060_5_1269, &P22060_1};	/*Property = ScreenPixelsHeight*/
const Value V22060_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22060_4 = {S54, 0, &V22060_4_1229, &P22060_5};	/*Property = ScreenPixelsWidth*/
const Profile P22060 = {"22060", &P22060_4};	/*Profile ID - 22060*/
const Value V21888_1_1 = {S2, NULL};	/*Value = True*/
const Property P21888_1 = {S50, 0, &V21888_1_1, NULL};	/*Property = IsMobile*/
const Value V21888_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21888_5 = {S52, 0, &V21888_5_1227, &P21888_1};	/*Property = ScreenPixelsHeight*/
const Value V21888_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21888_4 = {S54, 0, &V21888_4_1229, &P21888_5};	/*Property = ScreenPixelsWidth*/
const Profile P21888 = {"21888", &P21888_4};	/*Profile ID - 21888*/
const Value V21648_1_1 = {S2, NULL};	/*Value = True*/
const Property P21648_1 = {S50, 0, &V21648_1_1, NULL};	/*Property = IsMobile*/
const Value V21648_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P21648_5 = {S52, 0, &V21648_5_59138, &P21648_1};	/*Property = ScreenPixelsHeight*/
const Value V21648_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P21648_4 = {S54, 0, &V21648_4_59970, &P21648_5};	/*Property = ScreenPixelsWidth*/
const Profile P21648 = {"21648", &P21648_4};	/*Profile ID - 21648*/
const Value V21864_1_1 = {S2, NULL};	/*Value = True*/
const Property P21864_1 = {S50, 0, &V21864_1_1, NULL};	/*Property = IsMobile*/
const Value V21864_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21864_5 = {S52, 0, &V21864_5_1227, &P21864_1};	/*Property = ScreenPixelsHeight*/
const Value V21864_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21864_4 = {S54, 0, &V21864_4_1229, &P21864_5};	/*Property = ScreenPixelsWidth*/
const Profile P21864 = {"21864", &P21864_4};	/*Profile ID - 21864*/
const Value V21320_1_1 = {S2, NULL};	/*Value = True*/
const Property P21320_1 = {S50, 0, &V21320_1_1, NULL};	/*Property = IsMobile*/
const Value V21320_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21320_5 = {S52, 0, &V21320_5_9981, &P21320_1};	/*Property = ScreenPixelsHeight*/
const Value V21320_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21320_4 = {S54, 0, &V21320_4_39098, &P21320_5};	/*Property = ScreenPixelsWidth*/
const Profile P21320 = {"21320", &P21320_4};	/*Profile ID - 21320*/
const Value V18413_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_279 = {S1, 0, &V18413_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18413_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_282 = {S3, 0, &V18413_282_57155, &P18413_279};	/*Property = BlobBuilder*/
const Value V18413_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_350 = {S4, 0, &V18413_350_57446, &P18413_282};	/*Property = Canvas*/
const Value V18413_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_283 = {S5, 0, &V18413_283_57161, &P18413_350};	/*Property = CssBackground*/
const Value V18413_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_284 = {S6, 0, &V18413_284_57164, &P18413_283};	/*Property = CssBorderImage*/
const Value V18413_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_285 = {S7, 0, &V18413_285_57167, &P18413_284};	/*Property = CssCanvas*/
const Value V18413_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_286 = {S8, 0, &V18413_286_57170, &P18413_285};	/*Property = CssColor*/
const Value V18413_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_237 = {S9, 0, &V18413_237_55786, &P18413_286};	/*Property = CssColumn*/
const Value V18413_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_287 = {S10, 0, &V18413_287_57173, &P18413_237};	/*Property = CssFlexbox*/
const Value V18413_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_288 = {S11, 0, &V18413_288_57176, &P18413_287};	/*Property = CssFont*/
const Value V18413_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_289 = {S12, 0, &V18413_289_86899, &P18413_288};	/*Property = CssImages*/
const Value V18413_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_291 = {S13, 0, &V18413_291_57182, &P18413_289};	/*Property = CssMediaQueries*/
const Value V18413_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_292 = {S14, 0, &V18413_292_57185, &P18413_291};	/*Property = CssMinMax*/
const Value V18413_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_293 = {S15, 0, &V18413_293_57188, &P18413_292};	/*Property = CssOverflow*/
const Value V18413_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_294 = {S16, 0, &V18413_294_57191, &P18413_293};	/*Property = CssPosition*/
const Value V18413_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_295 = {S17, 0, &V18413_295_57194, &P18413_294};	/*Property = CssText*/
const Value V18413_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_297 = {S18, 0, &V18413_297_57197, &P18413_295};	/*Property = CssTransforms*/
const Value V18413_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_241 = {S19, 0, &V18413_241_55801, &P18413_297};	/*Property = CssTransitions*/
const Value V18413_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_298 = {S20, 0, &V18413_298_57208, &P18413_241};	/*Property = CssUI*/
const Value V18413_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_300 = {S21, 0, &V18413_300_57217, &P18413_298};	/*Property = DataSet*/
const Value V18413_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_301 = {S22, 0, &V18413_301_57220, &P18413_300};	/*Property = DataUrl*/
const Value V18413_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_302 = {S23, 0, &V18413_302_57223, &P18413_301};	/*Property = DeviceOrientation*/
const Value V18413_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_304 = {S24, 0, &V18413_304_57229, &P18413_302};	/*Property = FileReader*/
const Value V18413_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_305 = {S25, 0, &V18413_305_57232, &P18413_304};	/*Property = FileSaver*/
const Value V18413_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_306 = {S26, 0, &V18413_306_57238, &P18413_305};	/*Property = FileWriter*/
const Value V18413_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_307 = {S27, 0, &V18413_307_57241, &P18413_306};	/*Property = FormData*/
const Value V18413_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_309 = {S28, 0, &V18413_309_57247, &P18413_307};	/*Property = Fullscreen*/
const Value V18413_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_310 = {S29, 0, &V18413_310_57250, &P18413_309};	/*Property = GeoLocation*/
const Value V18413_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_312 = {S30, 0, &V18413_312_57256, &P18413_310};	/*Property = History*/
const Value V18413_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_314 = {S31, 0, &V18413_314_57262, &P18413_312};	/*Property = Html5*/
const Value V18413_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_313 = {S32, 0, &V18413_313_57259, &P18413_314};	/*Property = Html-Media-Capture*/
const Value V18413_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_315 = {S33, 0, &V18413_315_57265, &P18413_313};	/*Property = Iframe*/
const Value V18413_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_316 = {S34, 0, &V18413_316_57268, &P18413_315};	/*Property = IndexedDB*/
const Value V18413_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_317 = {S35, 0, &V18413_317_57271, &P18413_316};	/*Property = Json*/
const Value V18413_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18413_15 = {S37, 0, &V18413_15_31, &P18413_317};	/*Property = LayoutEngine*/
const Value V18413_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_344 = {S38, 0, &V18413_344_57274, &P18413_15};	/*Property = Masking*/
const Value V18413_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_320 = {S39, 0, &V18413_320_57292, &P18413_344};	/*Property = PostMessage*/
const Value V18413_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_321 = {S40, 0, &V18413_321_57295, &P18413_320};	/*Property = Progress*/
const Value V18413_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_322 = {S41, 0, &V18413_322_57298, &P18413_321};	/*Property = Prompts*/
const Value V18413_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_323 = {S42, 0, &V18413_323_57301, &P18413_322};	/*Property = Selector*/
const Value V18413_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_324 = {S43, 0, &V18413_324_57307, &P18413_323};	/*Property = Svg*/
const Value V18413_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_327 = {S44, 0, &V18413_327_57316, &P18413_324};	/*Property = TouchEvents*/
const Value V18413_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_328 = {S45, 0, &V18413_328_57319, &P18413_327};	/*Property = Track*/
const Value V18413_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_329 = {S46, 0, &V18413_329_57325, &P18413_328};	/*Property = Video*/
const Value V18413_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_330 = {S47, 0, &V18413_330_57328, &P18413_329};	/*Property = Viewport*/
const Value V18413_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_332 = {S48, 0, &V18413_332_57340, &P18413_330};	/*Property = WebWorkers*/
const Value V18413_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18413_333 = {S49, 0, &V18413_333_57343, &P18413_332};	/*Property = Xhr2*/
const Profile P18413 = {"18413", &P18413_333};	/*Profile ID - 18413*/
const Value V21909_1_1 = {S2, NULL};	/*Value = True*/
const Property P21909_1 = {S50, 0, &V21909_1_1, NULL};	/*Property = IsMobile*/
const Value V21909_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P21909_5 = {S52, 0, &V21909_5_59138, &P21909_1};	/*Property = ScreenPixelsHeight*/
const Value V21909_4_52971 = {S122, NULL};	/*Value = 1200*/
const Property P21909_4 = {S54, 0, &V21909_4_52971, &P21909_5};	/*Property = ScreenPixelsWidth*/
const Profile P21909 = {"21909", &P21909_4};	/*Profile ID - 21909*/
const Value V22171_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22171_279 = {S1, 0, &V22171_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22171_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22171_282 = {S3, 0, &V22171_282_57154, &P22171_279};	/*Property = BlobBuilder*/
const Value V22171_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22171_350 = {S4, 0, &V22171_350_57444, &P22171_282};	/*Property = Canvas*/
const Value V22171_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22171_283 = {S5, 0, &V22171_283_57159, &P22171_350};	/*Property = CssBackground*/
const Value V22171_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22171_284 = {S6, 0, &V22171_284_57162, &P22171_283};	/*Property = CssBorderImage*/
const Value V22171_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22171_285 = {S7, 0, &V22171_285_57166, &P22171_284};	/*Property = CssCanvas*/
const Value V22171_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22171_286 = {S8, 0, &V22171_286_57168, &P22171_285};	/*Property = CssColor*/
const Value V22171_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22171_237 = {S9, 0, &V22171_237_55786, &P22171_286};	/*Property = CssColumn*/
const Value V22171_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22171_287 = {S10, 0, &V22171_287_57172, &P22171_237};	/*Property = CssFlexbox*/
const Value V22171_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22171_288 = {S11, 0, &V22171_288_57174, &P22171_287};	/*Property = CssFont*/
const Value V22171_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22171_289 = {S12, 0, &V22171_289_57759, &P22171_288};	/*Property = CssImages*/
const Value V22171_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22171_291 = {S13, 0, &V22171_291_57180, &P22171_289};	/*Property = CssMediaQueries*/
const Value V22171_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22171_292 = {S14, 0, &V22171_292_57183, &P22171_291};	/*Property = CssMinMax*/
const Value V22171_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22171_293 = {S15, 0, &V22171_293_57187, &P22171_292};	/*Property = CssOverflow*/
const Value V22171_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22171_294 = {S16, 0, &V22171_294_57189, &P22171_293};	/*Property = CssPosition*/
const Value V22171_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22171_295 = {S17, 0, &V22171_295_57192, &P22171_294};	/*Property = CssText*/
const Value V22171_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22171_297 = {S18, 0, &V22171_297_57195, &P22171_295};	/*Property = CssTransforms*/
const Value V22171_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22171_241 = {S19, 0, &V22171_241_55799, &P22171_297};	/*Property = CssTransitions*/
const Value V22171_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22171_298 = {S20, 0, &V22171_298_57206, &P22171_241};	/*Property = CssUI*/
const Value V22171_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22171_300 = {S21, 0, &V22171_300_57215, &P22171_298};	/*Property = DataSet*/
const Value V22171_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22171_301 = {S22, 0, &V22171_301_57218, &P22171_300};	/*Property = DataUrl*/
const Value V22171_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22171_302 = {S23, 0, &V22171_302_57221, &P22171_301};	/*Property = DeviceOrientation*/
const Value V22171_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22171_304 = {S24, 0, &V22171_304_57227, &P22171_302};	/*Property = FileReader*/
const Value V22171_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22171_305 = {S25, 0, &V22171_305_57231, &P22171_304};	/*Property = FileSaver*/
const Value V22171_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22171_306 = {S26, 0, &V22171_306_57237, &P22171_305};	/*Property = FileWriter*/
const Value V22171_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22171_307 = {S27, 0, &V22171_307_57239, &P22171_306};	/*Property = FormData*/
const Value V22171_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22171_309 = {S28, 0, &V22171_309_57246, &P22171_307};	/*Property = Fullscreen*/
const Value V22171_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22171_310 = {S29, 0, &V22171_310_57248, &P22171_309};	/*Property = GeoLocation*/
const Value V22171_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22171_312 = {S30, 0, &V22171_312_57254, &P22171_310};	/*Property = History*/
const Value V22171_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22171_314 = {S31, 0, &V22171_314_57261, &P22171_312};	/*Property = Html5*/
const Value V22171_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22171_313 = {S32, 0, &V22171_313_57258, &P22171_314};	/*Property = Html-Media-Capture*/
const Value V22171_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22171_315 = {S33, 0, &V22171_315_57264, &P22171_313};	/*Property = Iframe*/
const Value V22171_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22171_316 = {S34, 0, &V22171_316_57266, &P22171_315};	/*Property = IndexedDB*/
const Value V22171_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22171_317 = {S35, 0, &V22171_317_57269, &P22171_316};	/*Property = Json*/
const Value V22171_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22171_15 = {S37, 0, &V22171_15_29, &P22171_317};	/*Property = LayoutEngine*/
const Value V22171_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22171_344 = {S38, 0, &V22171_344_57272, &P22171_15};	/*Property = Masking*/
const Value V22171_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22171_320 = {S39, 0, &V22171_320_57290, &P22171_344};	/*Property = PostMessage*/
const Value V22171_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22171_321 = {S40, 0, &V22171_321_57293, &P22171_320};	/*Property = Progress*/
const Value V22171_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22171_322 = {S41, 0, &V22171_322_57296, &P22171_321};	/*Property = Prompts*/
const Value V22171_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22171_323 = {S42, 0, &V22171_323_57299, &P22171_322};	/*Property = Selector*/
const Value V22171_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22171_324 = {S43, 0, &V22171_324_57305, &P22171_323};	/*Property = Svg*/
const Value V22171_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22171_327 = {S44, 0, &V22171_327_57314, &P22171_324};	/*Property = TouchEvents*/
const Value V22171_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22171_328 = {S45, 0, &V22171_328_57318, &P22171_327};	/*Property = Track*/
const Value V22171_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22171_329 = {S46, 0, &V22171_329_57323, &P22171_328};	/*Property = Video*/
const Value V22171_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22171_330 = {S47, 0, &V22171_330_57326, &P22171_329};	/*Property = Viewport*/
const Value V22171_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22171_332 = {S48, 0, &V22171_332_57338, &P22171_330};	/*Property = WebWorkers*/
const Value V22171_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22171_333 = {S49, 0, &V22171_333_57342, &P22171_332};	/*Property = Xhr2*/
const Profile P22171 = {"22171", &P22171_333};	/*Profile ID - 22171*/
const Value V21963_1_1 = {S2, NULL};	/*Value = True*/
const Property P21963_1 = {S50, 0, &V21963_1_1, NULL};	/*Property = IsMobile*/
const Value V21963_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21963_5 = {S52, 0, &V21963_5_1403, &P21963_1};	/*Property = ScreenPixelsHeight*/
const Value V21963_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21963_4 = {S54, 0, &V21963_4_1404, &P21963_5};	/*Property = ScreenPixelsWidth*/
const Profile P21963 = {"21963", &P21963_4};	/*Profile ID - 21963*/
const Value V21781_279_57147 = {S2, NULL};	/*Value = True*/
const Property P21781_279 = {S1, 0, &V21781_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V21781_282_57154 = {S0, NULL};	/*Value = False*/
const Property P21781_282 = {S3, 0, &V21781_282_57154, &P21781_279};	/*Property = BlobBuilder*/
const Value V21781_350_57444 = {S2, NULL};	/*Value = True*/
const Property P21781_350 = {S4, 0, &V21781_350_57444, &P21781_282};	/*Property = Canvas*/
const Value V21781_283_57159 = {S2, NULL};	/*Value = True*/
const Property P21781_283 = {S5, 0, &V21781_283_57159, &P21781_350};	/*Property = CssBackground*/
const Value V21781_284_57162 = {S2, NULL};	/*Value = True*/
const Property P21781_284 = {S6, 0, &V21781_284_57162, &P21781_283};	/*Property = CssBorderImage*/
const Value V21781_285_57166 = {S0, NULL};	/*Value = False*/
const Property P21781_285 = {S7, 0, &V21781_285_57166, &P21781_284};	/*Property = CssCanvas*/
const Value V21781_286_57168 = {S2, NULL};	/*Value = True*/
const Property P21781_286 = {S8, 0, &V21781_286_57168, &P21781_285};	/*Property = CssColor*/
const Value V21781_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P21781_237 = {S9, 0, &V21781_237_55786, &P21781_286};	/*Property = CssColumn*/
const Value V21781_287_57172 = {S0, NULL};	/*Value = False*/
const Property P21781_287 = {S10, 0, &V21781_287_57172, &P21781_237};	/*Property = CssFlexbox*/
const Value V21781_288_57175 = {S0, NULL};	/*Value = False*/
const Property P21781_288 = {S11, 0, &V21781_288_57175, &P21781_287};	/*Property = CssFont*/
const Value V21781_289_57759 = {S0, NULL};	/*Value = False*/
const Property P21781_289 = {S12, 0, &V21781_289_57759, &P21781_288};	/*Property = CssImages*/
const Value V21781_291_57180 = {S2, NULL};	/*Value = True*/
const Property P21781_291 = {S13, 0, &V21781_291_57180, &P21781_289};	/*Property = CssMediaQueries*/
const Value V21781_292_57183 = {S2, NULL};	/*Value = True*/
const Property P21781_292 = {S14, 0, &V21781_292_57183, &P21781_291};	/*Property = CssMinMax*/
const Value V21781_293_57187 = {S0, NULL};	/*Value = False*/
const Property P21781_293 = {S15, 0, &V21781_293_57187, &P21781_292};	/*Property = CssOverflow*/
const Value V21781_294_57189 = {S2, NULL};	/*Value = True*/
const Property P21781_294 = {S16, 0, &V21781_294_57189, &P21781_293};	/*Property = CssPosition*/
const Value V21781_295_57192 = {S2, NULL};	/*Value = True*/
const Property P21781_295 = {S17, 0, &V21781_295_57192, &P21781_294};	/*Property = CssText*/
const Value V21781_297_57195 = {S2, NULL};	/*Value = True*/
const Property P21781_297 = {S18, 0, &V21781_297_57195, &P21781_295};	/*Property = CssTransforms*/
const Value V21781_241_55799 = {S2, NULL};	/*Value = True*/
const Property P21781_241 = {S19, 0, &V21781_241_55799, &P21781_297};	/*Property = CssTransitions*/
const Value V21781_298_57206 = {S2, NULL};	/*Value = True*/
const Property P21781_298 = {S20, 0, &V21781_298_57206, &P21781_241};	/*Property = CssUI*/
const Value V21781_300_57215 = {S2, NULL};	/*Value = True*/
const Property P21781_300 = {S21, 0, &V21781_300_57215, &P21781_298};	/*Property = DataSet*/
const Value V21781_301_57218 = {S2, NULL};	/*Value = True*/
const Property P21781_301 = {S22, 0, &V21781_301_57218, &P21781_300};	/*Property = DataUrl*/
const Value V21781_302_57222 = {S0, NULL};	/*Value = False*/
const Property P21781_302 = {S23, 0, &V21781_302_57222, &P21781_301};	/*Property = DeviceOrientation*/
const Value V21781_304_57227 = {S2, NULL};	/*Value = True*/
const Property P21781_304 = {S24, 0, &V21781_304_57227, &P21781_302};	/*Property = FileReader*/
const Value V21781_305_57231 = {S0, NULL};	/*Value = False*/
const Property P21781_305 = {S25, 0, &V21781_305_57231, &P21781_304};	/*Property = FileSaver*/
const Value V21781_306_57237 = {S0, NULL};	/*Value = False*/
const Property P21781_306 = {S26, 0, &V21781_306_57237, &P21781_305};	/*Property = FileWriter*/
const Value V21781_307_57239 = {S2, NULL};	/*Value = True*/
const Property P21781_307 = {S27, 0, &V21781_307_57239, &P21781_306};	/*Property = FormData*/
const Value V21781_309_57246 = {S0, NULL};	/*Value = False*/
const Property P21781_309 = {S28, 0, &V21781_309_57246, &P21781_307};	/*Property = Fullscreen*/
const Value V21781_310_57249 = {S0, NULL};	/*Value = False*/
const Property P21781_310 = {S29, 0, &V21781_310_57249, &P21781_309};	/*Property = GeoLocation*/
const Value V21781_312_57254 = {S2, NULL};	/*Value = True*/
const Property P21781_312 = {S30, 0, &V21781_312_57254, &P21781_310};	/*Property = History*/
const Value V21781_314_57260 = {S2, NULL};	/*Value = True*/
const Property P21781_314 = {S31, 0, &V21781_314_57260, &P21781_312};	/*Property = Html5*/
const Value V21781_313_57258 = {S0, NULL};	/*Value = False*/
const Property P21781_313 = {S32, 0, &V21781_313_57258, &P21781_314};	/*Property = Html-Media-Capture*/
const Value V21781_315_57264 = {S0, NULL};	/*Value = False*/
const Property P21781_315 = {S33, 0, &V21781_315_57264, &P21781_313};	/*Property = Iframe*/
const Value V21781_316_57267 = {S0, NULL};	/*Value = False*/
const Property P21781_316 = {S34, 0, &V21781_316_57267, &P21781_315};	/*Property = IndexedDB*/
const Value V21781_317_57269 = {S2, NULL};	/*Value = True*/
const Property P21781_317 = {S35, 0, &V21781_317_57269, &P21781_316};	/*Property = Json*/
const Value V21781_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P21781_15 = {S37, 0, &V21781_15_54167, &P21781_317};	/*Property = LayoutEngine*/
const Value V21781_344_57272 = {S2, NULL};	/*Value = True*/
const Property P21781_344 = {S38, 0, &V21781_344_57272, &P21781_15};	/*Property = Masking*/
const Value V21781_320_57290 = {S2, NULL};	/*Value = True*/
const Property P21781_320 = {S39, 0, &V21781_320_57290, &P21781_344};	/*Property = PostMessage*/
const Value V21781_321_57293 = {S2, NULL};	/*Value = True*/
const Property P21781_321 = {S40, 0, &V21781_321_57293, &P21781_320};	/*Property = Progress*/
const Value V21781_322_57296 = {S2, NULL};	/*Value = True*/
const Property P21781_322 = {S41, 0, &V21781_322_57296, &P21781_321};	/*Property = Prompts*/
const Value V21781_323_57299 = {S2, NULL};	/*Value = True*/
const Property P21781_323 = {S42, 0, &V21781_323_57299, &P21781_322};	/*Property = Selector*/
const Value V21781_324_57305 = {S2, NULL};	/*Value = True*/
const Property P21781_324 = {S43, 0, &V21781_324_57305, &P21781_323};	/*Property = Svg*/
const Value V21781_327_57314 = {S2, NULL};	/*Value = True*/
const Property P21781_327 = {S44, 0, &V21781_327_57314, &P21781_324};	/*Property = TouchEvents*/
const Value V21781_328_57318 = {S0, NULL};	/*Value = False*/
const Property P21781_328 = {S45, 0, &V21781_328_57318, &P21781_327};	/*Property = Track*/
const Value V21781_329_57324 = {S0, NULL};	/*Value = False*/
const Property P21781_329 = {S46, 0, &V21781_329_57324, &P21781_328};	/*Property = Video*/
const Value V21781_330_57326 = {S2, NULL};	/*Value = True*/
const Property P21781_330 = {S47, 0, &V21781_330_57326, &P21781_329};	/*Property = Viewport*/
const Value V21781_332_57339 = {S0, NULL};	/*Value = False*/
const Property P21781_332 = {S48, 0, &V21781_332_57339, &P21781_330};	/*Property = WebWorkers*/
const Value V21781_333_57341 = {S2, NULL};	/*Value = True*/
const Property P21781_333 = {S49, 0, &V21781_333_57341, &P21781_332};	/*Property = Xhr2*/
const Profile P21781 = {"21781", &P21781_333};	/*Profile ID - 21781*/
const Value V20769_279_57148 = {S0, NULL};	/*Value = False*/
const Property P20769_279 = {S1, 0, &V20769_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V20769_282_57154 = {S0, NULL};	/*Value = False*/
const Property P20769_282 = {S3, 0, &V20769_282_57154, &P20769_279};	/*Property = BlobBuilder*/
const Value V20769_350_57444 = {S2, NULL};	/*Value = True*/
const Property P20769_350 = {S4, 0, &V20769_350_57444, &P20769_282};	/*Property = Canvas*/
const Value V20769_283_57159 = {S2, NULL};	/*Value = True*/
const Property P20769_283 = {S5, 0, &V20769_283_57159, &P20769_350};	/*Property = CssBackground*/
const Value V20769_284_57163 = {S0, NULL};	/*Value = False*/
const Property P20769_284 = {S6, 0, &V20769_284_57163, &P20769_283};	/*Property = CssBorderImage*/
const Value V20769_285_57166 = {S0, NULL};	/*Value = False*/
const Property P20769_285 = {S7, 0, &V20769_285_57166, &P20769_284};	/*Property = CssCanvas*/
const Value V20769_286_57168 = {S2, NULL};	/*Value = True*/
const Property P20769_286 = {S8, 0, &V20769_286_57168, &P20769_285};	/*Property = CssColor*/
const Value V20769_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P20769_237 = {S9, 0, &V20769_237_55786, &P20769_286};	/*Property = CssColumn*/
const Value V20769_287_57172 = {S0, NULL};	/*Value = False*/
const Property P20769_287 = {S10, 0, &V20769_287_57172, &P20769_237};	/*Property = CssFlexbox*/
const Value V20769_288_57174 = {S2, NULL};	/*Value = True*/
const Property P20769_288 = {S11, 0, &V20769_288_57174, &P20769_287};	/*Property = CssFont*/
const Value V20769_289_57759 = {S0, NULL};	/*Value = False*/
const Property P20769_289 = {S12, 0, &V20769_289_57759, &P20769_288};	/*Property = CssImages*/
const Value V20769_291_57181 = {S0, NULL};	/*Value = False*/
const Property P20769_291 = {S13, 0, &V20769_291_57181, &P20769_289};	/*Property = CssMediaQueries*/
const Value V20769_292_57183 = {S2, NULL};	/*Value = True*/
const Property P20769_292 = {S14, 0, &V20769_292_57183, &P20769_291};	/*Property = CssMinMax*/
const Value V20769_293_57187 = {S0, NULL};	/*Value = False*/
const Property P20769_293 = {S15, 0, &V20769_293_57187, &P20769_292};	/*Property = CssOverflow*/
const Value V20769_294_57189 = {S2, NULL};	/*Value = True*/
const Property P20769_294 = {S16, 0, &V20769_294_57189, &P20769_293};	/*Property = CssPosition*/
const Value V20769_295_57192 = {S2, NULL};	/*Value = True*/
const Property P20769_295 = {S17, 0, &V20769_295_57192, &P20769_294};	/*Property = CssText*/
const Value V20769_297_57195 = {S2, NULL};	/*Value = True*/
const Property P20769_297 = {S18, 0, &V20769_297_57195, &P20769_295};	/*Property = CssTransforms*/
const Value V20769_241_55799 = {S2, NULL};	/*Value = True*/
const Property P20769_241 = {S19, 0, &V20769_241_55799, &P20769_297};	/*Property = CssTransitions*/
const Value V20769_298_57206 = {S2, NULL};	/*Value = True*/
const Property P20769_298 = {S20, 0, &V20769_298_57206, &P20769_241};	/*Property = CssUI*/
const Value V20769_300_57216 = {S0, NULL};	/*Value = False*/
const Property P20769_300 = {S21, 0, &V20769_300_57216, &P20769_298};	/*Property = DataSet*/
const Value V20769_301_57218 = {S2, NULL};	/*Value = True*/
const Property P20769_301 = {S22, 0, &V20769_301_57218, &P20769_300};	/*Property = DataUrl*/
const Value V20769_302_57222 = {S0, NULL};	/*Value = False*/
const Property P20769_302 = {S23, 0, &V20769_302_57222, &P20769_301};	/*Property = DeviceOrientation*/
const Value V20769_304_57228 = {S0, NULL};	/*Value = False*/
const Property P20769_304 = {S24, 0, &V20769_304_57228, &P20769_302};	/*Property = FileReader*/
const Value V20769_305_57231 = {S0, NULL};	/*Value = False*/
const Property P20769_305 = {S25, 0, &V20769_305_57231, &P20769_304};	/*Property = FileSaver*/
const Value V20769_306_57237 = {S0, NULL};	/*Value = False*/
const Property P20769_306 = {S26, 0, &V20769_306_57237, &P20769_305};	/*Property = FileWriter*/
const Value V20769_307_57240 = {S0, NULL};	/*Value = False*/
const Property P20769_307 = {S27, 0, &V20769_307_57240, &P20769_306};	/*Property = FormData*/
const Value V20769_309_57246 = {S0, NULL};	/*Value = False*/
const Property P20769_309 = {S28, 0, &V20769_309_57246, &P20769_307};	/*Property = Fullscreen*/
const Value V20769_310_57248 = {S2, NULL};	/*Value = True*/
const Property P20769_310 = {S29, 0, &V20769_310_57248, &P20769_309};	/*Property = GeoLocation*/
const Value V20769_312_57255 = {S0, NULL};	/*Value = False*/
const Property P20769_312 = {S30, 0, &V20769_312_57255, &P20769_310};	/*Property = History*/
const Value V20769_314_57261 = {S0, NULL};	/*Value = False*/
const Property P20769_314 = {S31, 0, &V20769_314_57261, &P20769_312};	/*Property = Html5*/
const Value V20769_313_57258 = {S0, NULL};	/*Value = False*/
const Property P20769_313 = {S32, 0, &V20769_313_57258, &P20769_314};	/*Property = Html-Media-Capture*/
const Value V20769_315_57264 = {S0, NULL};	/*Value = False*/
const Property P20769_315 = {S33, 0, &V20769_315_57264, &P20769_313};	/*Property = Iframe*/
const Value V20769_316_57267 = {S0, NULL};	/*Value = False*/
const Property P20769_316 = {S34, 0, &V20769_316_57267, &P20769_315};	/*Property = IndexedDB*/
const Value V20769_317_57269 = {S2, NULL};	/*Value = True*/
const Property P20769_317 = {S35, 0, &V20769_317_57269, &P20769_316};	/*Property = Json*/
const Value V20769_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P20769_15 = {S37, 0, &V20769_15_28, &P20769_317};	/*Property = LayoutEngine*/
const Value V20769_344_57272 = {S2, NULL};	/*Value = True*/
const Property P20769_344 = {S38, 0, &V20769_344_57272, &P20769_15};	/*Property = Masking*/
const Value V20769_320_57290 = {S2, NULL};	/*Value = True*/
const Property P20769_320 = {S39, 0, &V20769_320_57290, &P20769_344};	/*Property = PostMessage*/
const Value V20769_321_57293 = {S2, NULL};	/*Value = True*/
const Property P20769_321 = {S40, 0, &V20769_321_57293, &P20769_320};	/*Property = Progress*/
const Value V20769_322_57296 = {S2, NULL};	/*Value = True*/
const Property P20769_322 = {S41, 0, &V20769_322_57296, &P20769_321};	/*Property = Prompts*/
const Value V20769_323_57299 = {S2, NULL};	/*Value = True*/
const Property P20769_323 = {S42, 0, &V20769_323_57299, &P20769_322};	/*Property = Selector*/
const Value V20769_324_57306 = {S0, NULL};	/*Value = False*/
const Property P20769_324 = {S43, 0, &V20769_324_57306, &P20769_323};	/*Property = Svg*/
const Value V20769_327_57314 = {S2, NULL};	/*Value = True*/
const Property P20769_327 = {S44, 0, &V20769_327_57314, &P20769_324};	/*Property = TouchEvents*/
const Value V20769_328_57318 = {S0, NULL};	/*Value = False*/
const Property P20769_328 = {S45, 0, &V20769_328_57318, &P20769_327};	/*Property = Track*/
const Value V20769_329_57323 = {S2, NULL};	/*Value = True*/
const Property P20769_329 = {S46, 0, &V20769_329_57323, &P20769_328};	/*Property = Video*/
const Value V20769_330_57326 = {S2, NULL};	/*Value = True*/
const Property P20769_330 = {S47, 0, &V20769_330_57326, &P20769_329};	/*Property = Viewport*/
const Value V20769_332_57339 = {S0, NULL};	/*Value = False*/
const Property P20769_332 = {S48, 0, &V20769_332_57339, &P20769_330};	/*Property = WebWorkers*/
const Value V20769_333_57342 = {S0, NULL};	/*Value = False*/
const Property P20769_333 = {S49, 0, &V20769_333_57342, &P20769_332};	/*Property = Xhr2*/
const Profile P20769 = {"20769", &P20769_333};	/*Profile ID - 20769*/
const Value V22756_1_1 = {S2, NULL};	/*Value = True*/
const Property P22756_1 = {S50, 0, &V22756_1_1, NULL};	/*Property = IsMobile*/
const Value V22756_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22756_5 = {S52, 0, &V22756_5_1277, &P22756_1};	/*Property = ScreenPixelsHeight*/
const Value V22756_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22756_4 = {S54, 0, &V22756_4_1279, &P22756_5};	/*Property = ScreenPixelsWidth*/
const Profile P22756 = {"22756", &P22756_4};	/*Profile ID - 22756*/
const Value V20991_1_1 = {S2, NULL};	/*Value = True*/
const Property P20991_1 = {S50, 0, &V20991_1_1, NULL};	/*Property = IsMobile*/
const Value V20991_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20991_5 = {S52, 0, &V20991_5_9981, &P20991_1};	/*Property = ScreenPixelsHeight*/
const Value V20991_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20991_4 = {S54, 0, &V20991_4_2336, &P20991_5};	/*Property = ScreenPixelsWidth*/
const Profile P20991 = {"20991", &P20991_4};	/*Profile ID - 20991*/
const Value V21793_1_1 = {S2, NULL};	/*Value = True*/
const Property P21793_1 = {S50, 0, &V21793_1_1, NULL};	/*Property = IsMobile*/
const Value V21793_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21793_5 = {S52, 0, &V21793_5_9981, &P21793_1};	/*Property = ScreenPixelsHeight*/
const Value V21793_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21793_4 = {S54, 0, &V21793_4_39098, &P21793_5};	/*Property = ScreenPixelsWidth*/
const Profile P21793 = {"21793", &P21793_4};	/*Profile ID - 21793*/
const Value V21792_1_1 = {S2, NULL};	/*Value = True*/
const Property P21792_1 = {S50, 0, &V21792_1_1, NULL};	/*Property = IsMobile*/
const Value V21792_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P21792_5 = {S52, 0, &V21792_5_9981, &P21792_1};	/*Property = ScreenPixelsHeight*/
const Value V21792_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P21792_4 = {S54, 0, &V21792_4_39098, &P21792_5};	/*Property = ScreenPixelsWidth*/
const Profile P21792 = {"21792", &P21792_4};	/*Profile ID - 21792*/
const Value V21794_1_1 = {S2, NULL};	/*Value = True*/
const Property P21794_1 = {S50, 0, &V21794_1_1, NULL};	/*Property = IsMobile*/
const Value V21794_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21794_5 = {S52, 0, &V21794_5_1403, &P21794_1};	/*Property = ScreenPixelsHeight*/
const Value V21794_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21794_4 = {S54, 0, &V21794_4_1404, &P21794_5};	/*Property = ScreenPixelsWidth*/
const Profile P21794 = {"21794", &P21794_4};	/*Profile ID - 21794*/
const Value V21802_1_1 = {S2, NULL};	/*Value = True*/
const Property P21802_1 = {S50, 0, &V21802_1_1, NULL};	/*Property = IsMobile*/
const Value V21802_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P21802_5 = {S52, 0, &V21802_5_1403, &P21802_1};	/*Property = ScreenPixelsHeight*/
const Value V21802_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P21802_4 = {S54, 0, &V21802_4_1404, &P21802_5};	/*Property = ScreenPixelsWidth*/
const Profile P21802 = {"21802", &P21802_4};	/*Profile ID - 21802*/
const Value V21812_1_1 = {S2, NULL};	/*Value = True*/
const Property P21812_1 = {S50, 0, &V21812_1_1, NULL};	/*Property = IsMobile*/
const Value V21812_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21812_5 = {S52, 0, &V21812_5_1227, &P21812_1};	/*Property = ScreenPixelsHeight*/
const Value V21812_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21812_4 = {S54, 0, &V21812_4_1229, &P21812_5};	/*Property = ScreenPixelsWidth*/
const Profile P21812 = {"21812", &P21812_4};	/*Profile ID - 21812*/
const Value V22069_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_279 = {S1, 0, &V22069_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22069_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_282 = {S3, 0, &V22069_282_57155, &P22069_279};	/*Property = BlobBuilder*/
const Value V22069_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_350 = {S4, 0, &V22069_350_57446, &P22069_282};	/*Property = Canvas*/
const Value V22069_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_283 = {S5, 0, &V22069_283_57161, &P22069_350};	/*Property = CssBackground*/
const Value V22069_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_284 = {S6, 0, &V22069_284_57164, &P22069_283};	/*Property = CssBorderImage*/
const Value V22069_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_285 = {S7, 0, &V22069_285_57167, &P22069_284};	/*Property = CssCanvas*/
const Value V22069_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_286 = {S8, 0, &V22069_286_57170, &P22069_285};	/*Property = CssColor*/
const Value V22069_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_237 = {S9, 0, &V22069_237_55786, &P22069_286};	/*Property = CssColumn*/
const Value V22069_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_287 = {S10, 0, &V22069_287_57173, &P22069_237};	/*Property = CssFlexbox*/
const Value V22069_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_288 = {S11, 0, &V22069_288_57176, &P22069_287};	/*Property = CssFont*/
const Value V22069_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_289 = {S12, 0, &V22069_289_86899, &P22069_288};	/*Property = CssImages*/
const Value V22069_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_291 = {S13, 0, &V22069_291_57182, &P22069_289};	/*Property = CssMediaQueries*/
const Value V22069_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_292 = {S14, 0, &V22069_292_57185, &P22069_291};	/*Property = CssMinMax*/
const Value V22069_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_293 = {S15, 0, &V22069_293_57188, &P22069_292};	/*Property = CssOverflow*/
const Value V22069_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_294 = {S16, 0, &V22069_294_57191, &P22069_293};	/*Property = CssPosition*/
const Value V22069_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_295 = {S17, 0, &V22069_295_57194, &P22069_294};	/*Property = CssText*/
const Value V22069_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_297 = {S18, 0, &V22069_297_57197, &P22069_295};	/*Property = CssTransforms*/
const Value V22069_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_241 = {S19, 0, &V22069_241_55801, &P22069_297};	/*Property = CssTransitions*/
const Value V22069_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_298 = {S20, 0, &V22069_298_57208, &P22069_241};	/*Property = CssUI*/
const Value V22069_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_300 = {S21, 0, &V22069_300_57217, &P22069_298};	/*Property = DataSet*/
const Value V22069_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_301 = {S22, 0, &V22069_301_57220, &P22069_300};	/*Property = DataUrl*/
const Value V22069_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_302 = {S23, 0, &V22069_302_57223, &P22069_301};	/*Property = DeviceOrientation*/
const Value V22069_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_304 = {S24, 0, &V22069_304_57229, &P22069_302};	/*Property = FileReader*/
const Value V22069_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_305 = {S25, 0, &V22069_305_57232, &P22069_304};	/*Property = FileSaver*/
const Value V22069_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_306 = {S26, 0, &V22069_306_57238, &P22069_305};	/*Property = FileWriter*/
const Value V22069_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_307 = {S27, 0, &V22069_307_57241, &P22069_306};	/*Property = FormData*/
const Value V22069_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_309 = {S28, 0, &V22069_309_57247, &P22069_307};	/*Property = Fullscreen*/
const Value V22069_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_310 = {S29, 0, &V22069_310_57250, &P22069_309};	/*Property = GeoLocation*/
const Value V22069_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_312 = {S30, 0, &V22069_312_57256, &P22069_310};	/*Property = History*/
const Value V22069_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_314 = {S31, 0, &V22069_314_57262, &P22069_312};	/*Property = Html5*/
const Value V22069_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_313 = {S32, 0, &V22069_313_57259, &P22069_314};	/*Property = Html-Media-Capture*/
const Value V22069_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_315 = {S33, 0, &V22069_315_57265, &P22069_313};	/*Property = Iframe*/
const Value V22069_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_316 = {S34, 0, &V22069_316_57268, &P22069_315};	/*Property = IndexedDB*/
const Value V22069_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_317 = {S35, 0, &V22069_317_57271, &P22069_316};	/*Property = Json*/
const Value V22069_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_15 = {S37, 0, &V22069_15_72, &P22069_317};	/*Property = LayoutEngine*/
const Value V22069_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_344 = {S38, 0, &V22069_344_57274, &P22069_15};	/*Property = Masking*/
const Value V22069_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_320 = {S39, 0, &V22069_320_57292, &P22069_344};	/*Property = PostMessage*/
const Value V22069_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_321 = {S40, 0, &V22069_321_57295, &P22069_320};	/*Property = Progress*/
const Value V22069_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_322 = {S41, 0, &V22069_322_57298, &P22069_321};	/*Property = Prompts*/
const Value V22069_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_323 = {S42, 0, &V22069_323_57301, &P22069_322};	/*Property = Selector*/
const Value V22069_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_324 = {S43, 0, &V22069_324_57307, &P22069_323};	/*Property = Svg*/
const Value V22069_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_327 = {S44, 0, &V22069_327_57316, &P22069_324};	/*Property = TouchEvents*/
const Value V22069_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_328 = {S45, 0, &V22069_328_57319, &P22069_327};	/*Property = Track*/
const Value V22069_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_329 = {S46, 0, &V22069_329_57325, &P22069_328};	/*Property = Video*/
const Value V22069_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_330 = {S47, 0, &V22069_330_57328, &P22069_329};	/*Property = Viewport*/
const Value V22069_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_332 = {S48, 0, &V22069_332_57340, &P22069_330};	/*Property = WebWorkers*/
const Value V22069_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22069_333 = {S49, 0, &V22069_333_57343, &P22069_332};	/*Property = Xhr2*/
const Profile P22069 = {"22069", &P22069_333};	/*Profile ID - 22069*/
const Value V22697_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_279 = {S1, 0, &V22697_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22697_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_282 = {S3, 0, &V22697_282_57155, &P22697_279};	/*Property = BlobBuilder*/
const Value V22697_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_350 = {S4, 0, &V22697_350_57446, &P22697_282};	/*Property = Canvas*/
const Value V22697_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_283 = {S5, 0, &V22697_283_57161, &P22697_350};	/*Property = CssBackground*/
const Value V22697_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_284 = {S6, 0, &V22697_284_57164, &P22697_283};	/*Property = CssBorderImage*/
const Value V22697_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_285 = {S7, 0, &V22697_285_57167, &P22697_284};	/*Property = CssCanvas*/
const Value V22697_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_286 = {S8, 0, &V22697_286_57170, &P22697_285};	/*Property = CssColor*/
const Value V22697_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_237 = {S9, 0, &V22697_237_55786, &P22697_286};	/*Property = CssColumn*/
const Value V22697_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_287 = {S10, 0, &V22697_287_57173, &P22697_237};	/*Property = CssFlexbox*/
const Value V22697_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_288 = {S11, 0, &V22697_288_57176, &P22697_287};	/*Property = CssFont*/
const Value V22697_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_289 = {S12, 0, &V22697_289_86899, &P22697_288};	/*Property = CssImages*/
const Value V22697_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_291 = {S13, 0, &V22697_291_57182, &P22697_289};	/*Property = CssMediaQueries*/
const Value V22697_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_292 = {S14, 0, &V22697_292_57185, &P22697_291};	/*Property = CssMinMax*/
const Value V22697_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_293 = {S15, 0, &V22697_293_57188, &P22697_292};	/*Property = CssOverflow*/
const Value V22697_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_294 = {S16, 0, &V22697_294_57191, &P22697_293};	/*Property = CssPosition*/
const Value V22697_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_295 = {S17, 0, &V22697_295_57194, &P22697_294};	/*Property = CssText*/
const Value V22697_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_297 = {S18, 0, &V22697_297_57197, &P22697_295};	/*Property = CssTransforms*/
const Value V22697_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_241 = {S19, 0, &V22697_241_55801, &P22697_297};	/*Property = CssTransitions*/
const Value V22697_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_298 = {S20, 0, &V22697_298_57208, &P22697_241};	/*Property = CssUI*/
const Value V22697_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_300 = {S21, 0, &V22697_300_57217, &P22697_298};	/*Property = DataSet*/
const Value V22697_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_301 = {S22, 0, &V22697_301_57220, &P22697_300};	/*Property = DataUrl*/
const Value V22697_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_302 = {S23, 0, &V22697_302_57223, &P22697_301};	/*Property = DeviceOrientation*/
const Value V22697_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_304 = {S24, 0, &V22697_304_57229, &P22697_302};	/*Property = FileReader*/
const Value V22697_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_305 = {S25, 0, &V22697_305_57232, &P22697_304};	/*Property = FileSaver*/
const Value V22697_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_306 = {S26, 0, &V22697_306_57238, &P22697_305};	/*Property = FileWriter*/
const Value V22697_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_307 = {S27, 0, &V22697_307_57241, &P22697_306};	/*Property = FormData*/
const Value V22697_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_309 = {S28, 0, &V22697_309_57247, &P22697_307};	/*Property = Fullscreen*/
const Value V22697_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_310 = {S29, 0, &V22697_310_57250, &P22697_309};	/*Property = GeoLocation*/
const Value V22697_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_312 = {S30, 0, &V22697_312_57256, &P22697_310};	/*Property = History*/
const Value V22697_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_314 = {S31, 0, &V22697_314_57262, &P22697_312};	/*Property = Html5*/
const Value V22697_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_313 = {S32, 0, &V22697_313_57259, &P22697_314};	/*Property = Html-Media-Capture*/
const Value V22697_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_315 = {S33, 0, &V22697_315_57265, &P22697_313};	/*Property = Iframe*/
const Value V22697_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_316 = {S34, 0, &V22697_316_57268, &P22697_315};	/*Property = IndexedDB*/
const Value V22697_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_317 = {S35, 0, &V22697_317_57271, &P22697_316};	/*Property = Json*/
const Value V22697_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_15 = {S37, 0, &V22697_15_72, &P22697_317};	/*Property = LayoutEngine*/
const Value V22697_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_344 = {S38, 0, &V22697_344_57274, &P22697_15};	/*Property = Masking*/
const Value V22697_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_320 = {S39, 0, &V22697_320_57292, &P22697_344};	/*Property = PostMessage*/
const Value V22697_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_321 = {S40, 0, &V22697_321_57295, &P22697_320};	/*Property = Progress*/
const Value V22697_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_322 = {S41, 0, &V22697_322_57298, &P22697_321};	/*Property = Prompts*/
const Value V22697_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_323 = {S42, 0, &V22697_323_57301, &P22697_322};	/*Property = Selector*/
const Value V22697_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_324 = {S43, 0, &V22697_324_57307, &P22697_323};	/*Property = Svg*/
const Value V22697_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_327 = {S44, 0, &V22697_327_57316, &P22697_324};	/*Property = TouchEvents*/
const Value V22697_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_328 = {S45, 0, &V22697_328_57319, &P22697_327};	/*Property = Track*/
const Value V22697_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_329 = {S46, 0, &V22697_329_57325, &P22697_328};	/*Property = Video*/
const Value V22697_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_330 = {S47, 0, &V22697_330_57328, &P22697_329};	/*Property = Viewport*/
const Value V22697_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_332 = {S48, 0, &V22697_332_57340, &P22697_330};	/*Property = WebWorkers*/
const Value V22697_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22697_333 = {S49, 0, &V22697_333_57343, &P22697_332};	/*Property = Xhr2*/
const Profile P22697 = {"22697", &P22697_333};	/*Profile ID - 22697*/
const Value V22076_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_279 = {S1, 0, &V22076_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22076_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_282 = {S3, 0, &V22076_282_57155, &P22076_279};	/*Property = BlobBuilder*/
const Value V22076_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_350 = {S4, 0, &V22076_350_57446, &P22076_282};	/*Property = Canvas*/
const Value V22076_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_283 = {S5, 0, &V22076_283_57161, &P22076_350};	/*Property = CssBackground*/
const Value V22076_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_284 = {S6, 0, &V22076_284_57164, &P22076_283};	/*Property = CssBorderImage*/
const Value V22076_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_285 = {S7, 0, &V22076_285_57167, &P22076_284};	/*Property = CssCanvas*/
const Value V22076_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_286 = {S8, 0, &V22076_286_57170, &P22076_285};	/*Property = CssColor*/
const Value V22076_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_237 = {S9, 0, &V22076_237_55786, &P22076_286};	/*Property = CssColumn*/
const Value V22076_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_287 = {S10, 0, &V22076_287_57173, &P22076_237};	/*Property = CssFlexbox*/
const Value V22076_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_288 = {S11, 0, &V22076_288_57176, &P22076_287};	/*Property = CssFont*/
const Value V22076_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_289 = {S12, 0, &V22076_289_86899, &P22076_288};	/*Property = CssImages*/
const Value V22076_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_291 = {S13, 0, &V22076_291_57182, &P22076_289};	/*Property = CssMediaQueries*/
const Value V22076_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_292 = {S14, 0, &V22076_292_57185, &P22076_291};	/*Property = CssMinMax*/
const Value V22076_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_293 = {S15, 0, &V22076_293_57188, &P22076_292};	/*Property = CssOverflow*/
const Value V22076_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_294 = {S16, 0, &V22076_294_57191, &P22076_293};	/*Property = CssPosition*/
const Value V22076_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_295 = {S17, 0, &V22076_295_57194, &P22076_294};	/*Property = CssText*/
const Value V22076_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_297 = {S18, 0, &V22076_297_57197, &P22076_295};	/*Property = CssTransforms*/
const Value V22076_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_241 = {S19, 0, &V22076_241_55801, &P22076_297};	/*Property = CssTransitions*/
const Value V22076_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_298 = {S20, 0, &V22076_298_57208, &P22076_241};	/*Property = CssUI*/
const Value V22076_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_300 = {S21, 0, &V22076_300_57217, &P22076_298};	/*Property = DataSet*/
const Value V22076_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_301 = {S22, 0, &V22076_301_57220, &P22076_300};	/*Property = DataUrl*/
const Value V22076_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_302 = {S23, 0, &V22076_302_57223, &P22076_301};	/*Property = DeviceOrientation*/
const Value V22076_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_304 = {S24, 0, &V22076_304_57229, &P22076_302};	/*Property = FileReader*/
const Value V22076_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_305 = {S25, 0, &V22076_305_57232, &P22076_304};	/*Property = FileSaver*/
const Value V22076_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_306 = {S26, 0, &V22076_306_57238, &P22076_305};	/*Property = FileWriter*/
const Value V22076_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_307 = {S27, 0, &V22076_307_57241, &P22076_306};	/*Property = FormData*/
const Value V22076_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_309 = {S28, 0, &V22076_309_57247, &P22076_307};	/*Property = Fullscreen*/
const Value V22076_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_310 = {S29, 0, &V22076_310_57250, &P22076_309};	/*Property = GeoLocation*/
const Value V22076_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_312 = {S30, 0, &V22076_312_57256, &P22076_310};	/*Property = History*/
const Value V22076_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_314 = {S31, 0, &V22076_314_57262, &P22076_312};	/*Property = Html5*/
const Value V22076_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_313 = {S32, 0, &V22076_313_57259, &P22076_314};	/*Property = Html-Media-Capture*/
const Value V22076_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_315 = {S33, 0, &V22076_315_57265, &P22076_313};	/*Property = Iframe*/
const Value V22076_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_316 = {S34, 0, &V22076_316_57268, &P22076_315};	/*Property = IndexedDB*/
const Value V22076_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_317 = {S35, 0, &V22076_317_57271, &P22076_316};	/*Property = Json*/
const Value V22076_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_15 = {S37, 0, &V22076_15_72, &P22076_317};	/*Property = LayoutEngine*/
const Value V22076_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_344 = {S38, 0, &V22076_344_57274, &P22076_15};	/*Property = Masking*/
const Value V22076_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_320 = {S39, 0, &V22076_320_57292, &P22076_344};	/*Property = PostMessage*/
const Value V22076_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_321 = {S40, 0, &V22076_321_57295, &P22076_320};	/*Property = Progress*/
const Value V22076_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_322 = {S41, 0, &V22076_322_57298, &P22076_321};	/*Property = Prompts*/
const Value V22076_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_323 = {S42, 0, &V22076_323_57301, &P22076_322};	/*Property = Selector*/
const Value V22076_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_324 = {S43, 0, &V22076_324_57307, &P22076_323};	/*Property = Svg*/
const Value V22076_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_327 = {S44, 0, &V22076_327_57316, &P22076_324};	/*Property = TouchEvents*/
const Value V22076_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_328 = {S45, 0, &V22076_328_57319, &P22076_327};	/*Property = Track*/
const Value V22076_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_329 = {S46, 0, &V22076_329_57325, &P22076_328};	/*Property = Video*/
const Value V22076_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_330 = {S47, 0, &V22076_330_57328, &P22076_329};	/*Property = Viewport*/
const Value V22076_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_332 = {S48, 0, &V22076_332_57340, &P22076_330};	/*Property = WebWorkers*/
const Value V22076_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22076_333 = {S49, 0, &V22076_333_57343, &P22076_332};	/*Property = Xhr2*/
const Profile P22076 = {"22076", &P22076_333};	/*Profile ID - 22076*/
const Value V22038_1_1 = {S2, NULL};	/*Value = True*/
const Property P22038_1 = {S50, 0, &V22038_1_1, NULL};	/*Property = IsMobile*/
const Value V22038_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22038_5 = {S52, 0, &V22038_5_1227, &P22038_1};	/*Property = ScreenPixelsHeight*/
const Value V22038_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P22038_4 = {S54, 0, &V22038_4_1477, &P22038_5};	/*Property = ScreenPixelsWidth*/
const Profile P22038 = {"22038", &P22038_4};	/*Profile ID - 22038*/
const Value V22040_1_1 = {S2, NULL};	/*Value = True*/
const Property P22040_1 = {S50, 0, &V22040_1_1, NULL};	/*Property = IsMobile*/
const Value V22040_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22040_5 = {S52, 0, &V22040_5_1832, &P22040_1};	/*Property = ScreenPixelsHeight*/
const Value V22040_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22040_4 = {S54, 0, &V22040_4_5645, &P22040_5};	/*Property = ScreenPixelsWidth*/
const Profile P22040 = {"22040", &P22040_4};	/*Profile ID - 22040*/
const Value V22286_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_279 = {S1, 0, &V22286_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22286_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_282 = {S3, 0, &V22286_282_57155, &P22286_279};	/*Property = BlobBuilder*/
const Value V22286_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_350 = {S4, 0, &V22286_350_57446, &P22286_282};	/*Property = Canvas*/
const Value V22286_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_283 = {S5, 0, &V22286_283_57161, &P22286_350};	/*Property = CssBackground*/
const Value V22286_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_284 = {S6, 0, &V22286_284_57164, &P22286_283};	/*Property = CssBorderImage*/
const Value V22286_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_285 = {S7, 0, &V22286_285_57167, &P22286_284};	/*Property = CssCanvas*/
const Value V22286_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_286 = {S8, 0, &V22286_286_57170, &P22286_285};	/*Property = CssColor*/
const Value V22286_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_237 = {S9, 0, &V22286_237_55786, &P22286_286};	/*Property = CssColumn*/
const Value V22286_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_287 = {S10, 0, &V22286_287_57173, &P22286_237};	/*Property = CssFlexbox*/
const Value V22286_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_288 = {S11, 0, &V22286_288_57176, &P22286_287};	/*Property = CssFont*/
const Value V22286_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_289 = {S12, 0, &V22286_289_86899, &P22286_288};	/*Property = CssImages*/
const Value V22286_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_291 = {S13, 0, &V22286_291_57182, &P22286_289};	/*Property = CssMediaQueries*/
const Value V22286_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_292 = {S14, 0, &V22286_292_57185, &P22286_291};	/*Property = CssMinMax*/
const Value V22286_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_293 = {S15, 0, &V22286_293_57188, &P22286_292};	/*Property = CssOverflow*/
const Value V22286_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_294 = {S16, 0, &V22286_294_57191, &P22286_293};	/*Property = CssPosition*/
const Value V22286_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_295 = {S17, 0, &V22286_295_57194, &P22286_294};	/*Property = CssText*/
const Value V22286_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_297 = {S18, 0, &V22286_297_57197, &P22286_295};	/*Property = CssTransforms*/
const Value V22286_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_241 = {S19, 0, &V22286_241_55801, &P22286_297};	/*Property = CssTransitions*/
const Value V22286_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_298 = {S20, 0, &V22286_298_57208, &P22286_241};	/*Property = CssUI*/
const Value V22286_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_300 = {S21, 0, &V22286_300_57217, &P22286_298};	/*Property = DataSet*/
const Value V22286_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_301 = {S22, 0, &V22286_301_57220, &P22286_300};	/*Property = DataUrl*/
const Value V22286_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_302 = {S23, 0, &V22286_302_57223, &P22286_301};	/*Property = DeviceOrientation*/
const Value V22286_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_304 = {S24, 0, &V22286_304_57229, &P22286_302};	/*Property = FileReader*/
const Value V22286_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_305 = {S25, 0, &V22286_305_57232, &P22286_304};	/*Property = FileSaver*/
const Value V22286_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_306 = {S26, 0, &V22286_306_57238, &P22286_305};	/*Property = FileWriter*/
const Value V22286_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_307 = {S27, 0, &V22286_307_57241, &P22286_306};	/*Property = FormData*/
const Value V22286_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_309 = {S28, 0, &V22286_309_57247, &P22286_307};	/*Property = Fullscreen*/
const Value V22286_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_310 = {S29, 0, &V22286_310_57250, &P22286_309};	/*Property = GeoLocation*/
const Value V22286_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_312 = {S30, 0, &V22286_312_57256, &P22286_310};	/*Property = History*/
const Value V22286_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_314 = {S31, 0, &V22286_314_57262, &P22286_312};	/*Property = Html5*/
const Value V22286_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_313 = {S32, 0, &V22286_313_57259, &P22286_314};	/*Property = Html-Media-Capture*/
const Value V22286_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_315 = {S33, 0, &V22286_315_57265, &P22286_313};	/*Property = Iframe*/
const Value V22286_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_316 = {S34, 0, &V22286_316_57268, &P22286_315};	/*Property = IndexedDB*/
const Value V22286_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_317 = {S35, 0, &V22286_317_57271, &P22286_316};	/*Property = Json*/
const Value V22286_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_15 = {S37, 0, &V22286_15_72, &P22286_317};	/*Property = LayoutEngine*/
const Value V22286_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_344 = {S38, 0, &V22286_344_57274, &P22286_15};	/*Property = Masking*/
const Value V22286_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_320 = {S39, 0, &V22286_320_57292, &P22286_344};	/*Property = PostMessage*/
const Value V22286_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_321 = {S40, 0, &V22286_321_57295, &P22286_320};	/*Property = Progress*/
const Value V22286_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_322 = {S41, 0, &V22286_322_57298, &P22286_321};	/*Property = Prompts*/
const Value V22286_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_323 = {S42, 0, &V22286_323_57301, &P22286_322};	/*Property = Selector*/
const Value V22286_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_324 = {S43, 0, &V22286_324_57307, &P22286_323};	/*Property = Svg*/
const Value V22286_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_327 = {S44, 0, &V22286_327_57316, &P22286_324};	/*Property = TouchEvents*/
const Value V22286_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_328 = {S45, 0, &V22286_328_57319, &P22286_327};	/*Property = Track*/
const Value V22286_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_329 = {S46, 0, &V22286_329_57325, &P22286_328};	/*Property = Video*/
const Value V22286_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_330 = {S47, 0, &V22286_330_57328, &P22286_329};	/*Property = Viewport*/
const Value V22286_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_332 = {S48, 0, &V22286_332_57340, &P22286_330};	/*Property = WebWorkers*/
const Value V22286_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22286_333 = {S49, 0, &V22286_333_57343, &P22286_332};	/*Property = Xhr2*/
const Profile P22286 = {"22286", &P22286_333};	/*Profile ID - 22286*/
const Value V22068_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_279 = {S1, 0, &V22068_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22068_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_282 = {S3, 0, &V22068_282_57155, &P22068_279};	/*Property = BlobBuilder*/
const Value V22068_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_350 = {S4, 0, &V22068_350_57446, &P22068_282};	/*Property = Canvas*/
const Value V22068_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_283 = {S5, 0, &V22068_283_57161, &P22068_350};	/*Property = CssBackground*/
const Value V22068_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_284 = {S6, 0, &V22068_284_57164, &P22068_283};	/*Property = CssBorderImage*/
const Value V22068_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_285 = {S7, 0, &V22068_285_57167, &P22068_284};	/*Property = CssCanvas*/
const Value V22068_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_286 = {S8, 0, &V22068_286_57170, &P22068_285};	/*Property = CssColor*/
const Value V22068_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_237 = {S9, 0, &V22068_237_55786, &P22068_286};	/*Property = CssColumn*/
const Value V22068_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_287 = {S10, 0, &V22068_287_57173, &P22068_237};	/*Property = CssFlexbox*/
const Value V22068_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_288 = {S11, 0, &V22068_288_57176, &P22068_287};	/*Property = CssFont*/
const Value V22068_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_289 = {S12, 0, &V22068_289_86899, &P22068_288};	/*Property = CssImages*/
const Value V22068_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_291 = {S13, 0, &V22068_291_57182, &P22068_289};	/*Property = CssMediaQueries*/
const Value V22068_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_292 = {S14, 0, &V22068_292_57185, &P22068_291};	/*Property = CssMinMax*/
const Value V22068_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_293 = {S15, 0, &V22068_293_57188, &P22068_292};	/*Property = CssOverflow*/
const Value V22068_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_294 = {S16, 0, &V22068_294_57191, &P22068_293};	/*Property = CssPosition*/
const Value V22068_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_295 = {S17, 0, &V22068_295_57194, &P22068_294};	/*Property = CssText*/
const Value V22068_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_297 = {S18, 0, &V22068_297_57197, &P22068_295};	/*Property = CssTransforms*/
const Value V22068_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_241 = {S19, 0, &V22068_241_55801, &P22068_297};	/*Property = CssTransitions*/
const Value V22068_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_298 = {S20, 0, &V22068_298_57208, &P22068_241};	/*Property = CssUI*/
const Value V22068_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_300 = {S21, 0, &V22068_300_57217, &P22068_298};	/*Property = DataSet*/
const Value V22068_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_301 = {S22, 0, &V22068_301_57220, &P22068_300};	/*Property = DataUrl*/
const Value V22068_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_302 = {S23, 0, &V22068_302_57223, &P22068_301};	/*Property = DeviceOrientation*/
const Value V22068_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_304 = {S24, 0, &V22068_304_57229, &P22068_302};	/*Property = FileReader*/
const Value V22068_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_305 = {S25, 0, &V22068_305_57232, &P22068_304};	/*Property = FileSaver*/
const Value V22068_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_306 = {S26, 0, &V22068_306_57238, &P22068_305};	/*Property = FileWriter*/
const Value V22068_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_307 = {S27, 0, &V22068_307_57241, &P22068_306};	/*Property = FormData*/
const Value V22068_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_309 = {S28, 0, &V22068_309_57247, &P22068_307};	/*Property = Fullscreen*/
const Value V22068_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_310 = {S29, 0, &V22068_310_57250, &P22068_309};	/*Property = GeoLocation*/
const Value V22068_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_312 = {S30, 0, &V22068_312_57256, &P22068_310};	/*Property = History*/
const Value V22068_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_314 = {S31, 0, &V22068_314_57262, &P22068_312};	/*Property = Html5*/
const Value V22068_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_313 = {S32, 0, &V22068_313_57259, &P22068_314};	/*Property = Html-Media-Capture*/
const Value V22068_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_315 = {S33, 0, &V22068_315_57265, &P22068_313};	/*Property = Iframe*/
const Value V22068_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_316 = {S34, 0, &V22068_316_57268, &P22068_315};	/*Property = IndexedDB*/
const Value V22068_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_317 = {S35, 0, &V22068_317_57271, &P22068_316};	/*Property = Json*/
const Value V22068_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_15 = {S37, 0, &V22068_15_72, &P22068_317};	/*Property = LayoutEngine*/
const Value V22068_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_344 = {S38, 0, &V22068_344_57274, &P22068_15};	/*Property = Masking*/
const Value V22068_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_320 = {S39, 0, &V22068_320_57292, &P22068_344};	/*Property = PostMessage*/
const Value V22068_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_321 = {S40, 0, &V22068_321_57295, &P22068_320};	/*Property = Progress*/
const Value V22068_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_322 = {S41, 0, &V22068_322_57298, &P22068_321};	/*Property = Prompts*/
const Value V22068_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_323 = {S42, 0, &V22068_323_57301, &P22068_322};	/*Property = Selector*/
const Value V22068_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_324 = {S43, 0, &V22068_324_57307, &P22068_323};	/*Property = Svg*/
const Value V22068_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_327 = {S44, 0, &V22068_327_57316, &P22068_324};	/*Property = TouchEvents*/
const Value V22068_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_328 = {S45, 0, &V22068_328_57319, &P22068_327};	/*Property = Track*/
const Value V22068_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_329 = {S46, 0, &V22068_329_57325, &P22068_328};	/*Property = Video*/
const Value V22068_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_330 = {S47, 0, &V22068_330_57328, &P22068_329};	/*Property = Viewport*/
const Value V22068_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_332 = {S48, 0, &V22068_332_57340, &P22068_330};	/*Property = WebWorkers*/
const Value V22068_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22068_333 = {S49, 0, &V22068_333_57343, &P22068_332};	/*Property = Xhr2*/
const Profile P22068 = {"22068", &P22068_333};	/*Profile ID - 22068*/
const Value V22285_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_279 = {S1, 0, &V22285_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22285_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_282 = {S3, 0, &V22285_282_57155, &P22285_279};	/*Property = BlobBuilder*/
const Value V22285_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_350 = {S4, 0, &V22285_350_57446, &P22285_282};	/*Property = Canvas*/
const Value V22285_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_283 = {S5, 0, &V22285_283_57161, &P22285_350};	/*Property = CssBackground*/
const Value V22285_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_284 = {S6, 0, &V22285_284_57164, &P22285_283};	/*Property = CssBorderImage*/
const Value V22285_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_285 = {S7, 0, &V22285_285_57167, &P22285_284};	/*Property = CssCanvas*/
const Value V22285_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_286 = {S8, 0, &V22285_286_57170, &P22285_285};	/*Property = CssColor*/
const Value V22285_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_237 = {S9, 0, &V22285_237_55786, &P22285_286};	/*Property = CssColumn*/
const Value V22285_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_287 = {S10, 0, &V22285_287_57173, &P22285_237};	/*Property = CssFlexbox*/
const Value V22285_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_288 = {S11, 0, &V22285_288_57176, &P22285_287};	/*Property = CssFont*/
const Value V22285_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_289 = {S12, 0, &V22285_289_86899, &P22285_288};	/*Property = CssImages*/
const Value V22285_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_291 = {S13, 0, &V22285_291_57182, &P22285_289};	/*Property = CssMediaQueries*/
const Value V22285_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_292 = {S14, 0, &V22285_292_57185, &P22285_291};	/*Property = CssMinMax*/
const Value V22285_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_293 = {S15, 0, &V22285_293_57188, &P22285_292};	/*Property = CssOverflow*/
const Value V22285_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_294 = {S16, 0, &V22285_294_57191, &P22285_293};	/*Property = CssPosition*/
const Value V22285_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_295 = {S17, 0, &V22285_295_57194, &P22285_294};	/*Property = CssText*/
const Value V22285_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_297 = {S18, 0, &V22285_297_57197, &P22285_295};	/*Property = CssTransforms*/
const Value V22285_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_241 = {S19, 0, &V22285_241_55801, &P22285_297};	/*Property = CssTransitions*/
const Value V22285_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_298 = {S20, 0, &V22285_298_57208, &P22285_241};	/*Property = CssUI*/
const Value V22285_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_300 = {S21, 0, &V22285_300_57217, &P22285_298};	/*Property = DataSet*/
const Value V22285_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_301 = {S22, 0, &V22285_301_57220, &P22285_300};	/*Property = DataUrl*/
const Value V22285_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_302 = {S23, 0, &V22285_302_57223, &P22285_301};	/*Property = DeviceOrientation*/
const Value V22285_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_304 = {S24, 0, &V22285_304_57229, &P22285_302};	/*Property = FileReader*/
const Value V22285_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_305 = {S25, 0, &V22285_305_57232, &P22285_304};	/*Property = FileSaver*/
const Value V22285_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_306 = {S26, 0, &V22285_306_57238, &P22285_305};	/*Property = FileWriter*/
const Value V22285_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_307 = {S27, 0, &V22285_307_57241, &P22285_306};	/*Property = FormData*/
const Value V22285_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_309 = {S28, 0, &V22285_309_57247, &P22285_307};	/*Property = Fullscreen*/
const Value V22285_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_310 = {S29, 0, &V22285_310_57250, &P22285_309};	/*Property = GeoLocation*/
const Value V22285_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_312 = {S30, 0, &V22285_312_57256, &P22285_310};	/*Property = History*/
const Value V22285_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_314 = {S31, 0, &V22285_314_57262, &P22285_312};	/*Property = Html5*/
const Value V22285_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_313 = {S32, 0, &V22285_313_57259, &P22285_314};	/*Property = Html-Media-Capture*/
const Value V22285_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_315 = {S33, 0, &V22285_315_57265, &P22285_313};	/*Property = Iframe*/
const Value V22285_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_316 = {S34, 0, &V22285_316_57268, &P22285_315};	/*Property = IndexedDB*/
const Value V22285_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_317 = {S35, 0, &V22285_317_57271, &P22285_316};	/*Property = Json*/
const Value V22285_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_15 = {S37, 0, &V22285_15_72, &P22285_317};	/*Property = LayoutEngine*/
const Value V22285_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_344 = {S38, 0, &V22285_344_57274, &P22285_15};	/*Property = Masking*/
const Value V22285_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_320 = {S39, 0, &V22285_320_57292, &P22285_344};	/*Property = PostMessage*/
const Value V22285_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_321 = {S40, 0, &V22285_321_57295, &P22285_320};	/*Property = Progress*/
const Value V22285_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_322 = {S41, 0, &V22285_322_57298, &P22285_321};	/*Property = Prompts*/
const Value V22285_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_323 = {S42, 0, &V22285_323_57301, &P22285_322};	/*Property = Selector*/
const Value V22285_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_324 = {S43, 0, &V22285_324_57307, &P22285_323};	/*Property = Svg*/
const Value V22285_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_327 = {S44, 0, &V22285_327_57316, &P22285_324};	/*Property = TouchEvents*/
const Value V22285_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_328 = {S45, 0, &V22285_328_57319, &P22285_327};	/*Property = Track*/
const Value V22285_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_329 = {S46, 0, &V22285_329_57325, &P22285_328};	/*Property = Video*/
const Value V22285_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_330 = {S47, 0, &V22285_330_57328, &P22285_329};	/*Property = Viewport*/
const Value V22285_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_332 = {S48, 0, &V22285_332_57340, &P22285_330};	/*Property = WebWorkers*/
const Value V22285_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22285_333 = {S49, 0, &V22285_333_57343, &P22285_332};	/*Property = Xhr2*/
const Profile P22285 = {"22285", &P22285_333};	/*Profile ID - 22285*/
const Value V22055_1_1 = {S2, NULL};	/*Value = True*/
const Property P22055_1 = {S50, 0, &V22055_1_1, NULL};	/*Property = IsMobile*/
const Value V22055_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22055_5 = {S52, 0, &V22055_5_1458, &P22055_1};	/*Property = ScreenPixelsHeight*/
const Value V22055_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22055_4 = {S54, 0, &V22055_4_1460, &P22055_5};	/*Property = ScreenPixelsWidth*/
const Profile P22055 = {"22055", &P22055_4};	/*Profile ID - 22055*/
const Value V22291_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_279 = {S1, 0, &V22291_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22291_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_282 = {S3, 0, &V22291_282_57155, &P22291_279};	/*Property = BlobBuilder*/
const Value V22291_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_350 = {S4, 0, &V22291_350_57446, &P22291_282};	/*Property = Canvas*/
const Value V22291_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_283 = {S5, 0, &V22291_283_57161, &P22291_350};	/*Property = CssBackground*/
const Value V22291_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_284 = {S6, 0, &V22291_284_57164, &P22291_283};	/*Property = CssBorderImage*/
const Value V22291_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_285 = {S7, 0, &V22291_285_57167, &P22291_284};	/*Property = CssCanvas*/
const Value V22291_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_286 = {S8, 0, &V22291_286_57170, &P22291_285};	/*Property = CssColor*/
const Value V22291_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_237 = {S9, 0, &V22291_237_55786, &P22291_286};	/*Property = CssColumn*/
const Value V22291_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_287 = {S10, 0, &V22291_287_57173, &P22291_237};	/*Property = CssFlexbox*/
const Value V22291_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_288 = {S11, 0, &V22291_288_57176, &P22291_287};	/*Property = CssFont*/
const Value V22291_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_289 = {S12, 0, &V22291_289_86899, &P22291_288};	/*Property = CssImages*/
const Value V22291_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_291 = {S13, 0, &V22291_291_57182, &P22291_289};	/*Property = CssMediaQueries*/
const Value V22291_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_292 = {S14, 0, &V22291_292_57185, &P22291_291};	/*Property = CssMinMax*/
const Value V22291_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_293 = {S15, 0, &V22291_293_57188, &P22291_292};	/*Property = CssOverflow*/
const Value V22291_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_294 = {S16, 0, &V22291_294_57191, &P22291_293};	/*Property = CssPosition*/
const Value V22291_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_295 = {S17, 0, &V22291_295_57194, &P22291_294};	/*Property = CssText*/
const Value V22291_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_297 = {S18, 0, &V22291_297_57197, &P22291_295};	/*Property = CssTransforms*/
const Value V22291_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_241 = {S19, 0, &V22291_241_55801, &P22291_297};	/*Property = CssTransitions*/
const Value V22291_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_298 = {S20, 0, &V22291_298_57208, &P22291_241};	/*Property = CssUI*/
const Value V22291_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_300 = {S21, 0, &V22291_300_57217, &P22291_298};	/*Property = DataSet*/
const Value V22291_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_301 = {S22, 0, &V22291_301_57220, &P22291_300};	/*Property = DataUrl*/
const Value V22291_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_302 = {S23, 0, &V22291_302_57223, &P22291_301};	/*Property = DeviceOrientation*/
const Value V22291_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_304 = {S24, 0, &V22291_304_57229, &P22291_302};	/*Property = FileReader*/
const Value V22291_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_305 = {S25, 0, &V22291_305_57232, &P22291_304};	/*Property = FileSaver*/
const Value V22291_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_306 = {S26, 0, &V22291_306_57238, &P22291_305};	/*Property = FileWriter*/
const Value V22291_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_307 = {S27, 0, &V22291_307_57241, &P22291_306};	/*Property = FormData*/
const Value V22291_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_309 = {S28, 0, &V22291_309_57247, &P22291_307};	/*Property = Fullscreen*/
const Value V22291_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_310 = {S29, 0, &V22291_310_57250, &P22291_309};	/*Property = GeoLocation*/
const Value V22291_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_312 = {S30, 0, &V22291_312_57256, &P22291_310};	/*Property = History*/
const Value V22291_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_314 = {S31, 0, &V22291_314_57262, &P22291_312};	/*Property = Html5*/
const Value V22291_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_313 = {S32, 0, &V22291_313_57259, &P22291_314};	/*Property = Html-Media-Capture*/
const Value V22291_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_315 = {S33, 0, &V22291_315_57265, &P22291_313};	/*Property = Iframe*/
const Value V22291_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_316 = {S34, 0, &V22291_316_57268, &P22291_315};	/*Property = IndexedDB*/
const Value V22291_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_317 = {S35, 0, &V22291_317_57271, &P22291_316};	/*Property = Json*/
const Value V22291_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_15 = {S37, 0, &V22291_15_72, &P22291_317};	/*Property = LayoutEngine*/
const Value V22291_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_344 = {S38, 0, &V22291_344_57274, &P22291_15};	/*Property = Masking*/
const Value V22291_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_320 = {S39, 0, &V22291_320_57292, &P22291_344};	/*Property = PostMessage*/
const Value V22291_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_321 = {S40, 0, &V22291_321_57295, &P22291_320};	/*Property = Progress*/
const Value V22291_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_322 = {S41, 0, &V22291_322_57298, &P22291_321};	/*Property = Prompts*/
const Value V22291_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_323 = {S42, 0, &V22291_323_57301, &P22291_322};	/*Property = Selector*/
const Value V22291_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_324 = {S43, 0, &V22291_324_57307, &P22291_323};	/*Property = Svg*/
const Value V22291_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_327 = {S44, 0, &V22291_327_57316, &P22291_324};	/*Property = TouchEvents*/
const Value V22291_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_328 = {S45, 0, &V22291_328_57319, &P22291_327};	/*Property = Track*/
const Value V22291_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_329 = {S46, 0, &V22291_329_57325, &P22291_328};	/*Property = Video*/
const Value V22291_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_330 = {S47, 0, &V22291_330_57328, &P22291_329};	/*Property = Viewport*/
const Value V22291_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_332 = {S48, 0, &V22291_332_57340, &P22291_330};	/*Property = WebWorkers*/
const Value V22291_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22291_333 = {S49, 0, &V22291_333_57343, &P22291_332};	/*Property = Xhr2*/
const Profile P22291 = {"22291", &P22291_333};	/*Profile ID - 22291*/
const Value V22292_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_279 = {S1, 0, &V22292_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22292_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_282 = {S3, 0, &V22292_282_57155, &P22292_279};	/*Property = BlobBuilder*/
const Value V22292_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_350 = {S4, 0, &V22292_350_57446, &P22292_282};	/*Property = Canvas*/
const Value V22292_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_283 = {S5, 0, &V22292_283_57161, &P22292_350};	/*Property = CssBackground*/
const Value V22292_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_284 = {S6, 0, &V22292_284_57164, &P22292_283};	/*Property = CssBorderImage*/
const Value V22292_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_285 = {S7, 0, &V22292_285_57167, &P22292_284};	/*Property = CssCanvas*/
const Value V22292_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_286 = {S8, 0, &V22292_286_57170, &P22292_285};	/*Property = CssColor*/
const Value V22292_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_237 = {S9, 0, &V22292_237_55786, &P22292_286};	/*Property = CssColumn*/
const Value V22292_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_287 = {S10, 0, &V22292_287_57173, &P22292_237};	/*Property = CssFlexbox*/
const Value V22292_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_288 = {S11, 0, &V22292_288_57176, &P22292_287};	/*Property = CssFont*/
const Value V22292_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_289 = {S12, 0, &V22292_289_86899, &P22292_288};	/*Property = CssImages*/
const Value V22292_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_291 = {S13, 0, &V22292_291_57182, &P22292_289};	/*Property = CssMediaQueries*/
const Value V22292_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_292 = {S14, 0, &V22292_292_57185, &P22292_291};	/*Property = CssMinMax*/
const Value V22292_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_293 = {S15, 0, &V22292_293_57188, &P22292_292};	/*Property = CssOverflow*/
const Value V22292_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_294 = {S16, 0, &V22292_294_57191, &P22292_293};	/*Property = CssPosition*/
const Value V22292_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_295 = {S17, 0, &V22292_295_57194, &P22292_294};	/*Property = CssText*/
const Value V22292_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_297 = {S18, 0, &V22292_297_57197, &P22292_295};	/*Property = CssTransforms*/
const Value V22292_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_241 = {S19, 0, &V22292_241_55801, &P22292_297};	/*Property = CssTransitions*/
const Value V22292_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_298 = {S20, 0, &V22292_298_57208, &P22292_241};	/*Property = CssUI*/
const Value V22292_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_300 = {S21, 0, &V22292_300_57217, &P22292_298};	/*Property = DataSet*/
const Value V22292_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_301 = {S22, 0, &V22292_301_57220, &P22292_300};	/*Property = DataUrl*/
const Value V22292_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_302 = {S23, 0, &V22292_302_57223, &P22292_301};	/*Property = DeviceOrientation*/
const Value V22292_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_304 = {S24, 0, &V22292_304_57229, &P22292_302};	/*Property = FileReader*/
const Value V22292_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_305 = {S25, 0, &V22292_305_57232, &P22292_304};	/*Property = FileSaver*/
const Value V22292_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_306 = {S26, 0, &V22292_306_57238, &P22292_305};	/*Property = FileWriter*/
const Value V22292_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_307 = {S27, 0, &V22292_307_57241, &P22292_306};	/*Property = FormData*/
const Value V22292_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_309 = {S28, 0, &V22292_309_57247, &P22292_307};	/*Property = Fullscreen*/
const Value V22292_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_310 = {S29, 0, &V22292_310_57250, &P22292_309};	/*Property = GeoLocation*/
const Value V22292_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_312 = {S30, 0, &V22292_312_57256, &P22292_310};	/*Property = History*/
const Value V22292_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_314 = {S31, 0, &V22292_314_57262, &P22292_312};	/*Property = Html5*/
const Value V22292_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_313 = {S32, 0, &V22292_313_57259, &P22292_314};	/*Property = Html-Media-Capture*/
const Value V22292_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_315 = {S33, 0, &V22292_315_57265, &P22292_313};	/*Property = Iframe*/
const Value V22292_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_316 = {S34, 0, &V22292_316_57268, &P22292_315};	/*Property = IndexedDB*/
const Value V22292_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_317 = {S35, 0, &V22292_317_57271, &P22292_316};	/*Property = Json*/
const Value V22292_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_15 = {S37, 0, &V22292_15_72, &P22292_317};	/*Property = LayoutEngine*/
const Value V22292_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_344 = {S38, 0, &V22292_344_57274, &P22292_15};	/*Property = Masking*/
const Value V22292_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_320 = {S39, 0, &V22292_320_57292, &P22292_344};	/*Property = PostMessage*/
const Value V22292_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_321 = {S40, 0, &V22292_321_57295, &P22292_320};	/*Property = Progress*/
const Value V22292_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_322 = {S41, 0, &V22292_322_57298, &P22292_321};	/*Property = Prompts*/
const Value V22292_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_323 = {S42, 0, &V22292_323_57301, &P22292_322};	/*Property = Selector*/
const Value V22292_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_324 = {S43, 0, &V22292_324_57307, &P22292_323};	/*Property = Svg*/
const Value V22292_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_327 = {S44, 0, &V22292_327_57316, &P22292_324};	/*Property = TouchEvents*/
const Value V22292_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_328 = {S45, 0, &V22292_328_57319, &P22292_327};	/*Property = Track*/
const Value V22292_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_329 = {S46, 0, &V22292_329_57325, &P22292_328};	/*Property = Video*/
const Value V22292_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_330 = {S47, 0, &V22292_330_57328, &P22292_329};	/*Property = Viewport*/
const Value V22292_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_332 = {S48, 0, &V22292_332_57340, &P22292_330};	/*Property = WebWorkers*/
const Value V22292_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22292_333 = {S49, 0, &V22292_333_57343, &P22292_332};	/*Property = Xhr2*/
const Profile P22292 = {"22292", &P22292_333};	/*Profile ID - 22292*/
const Value V22290_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_279 = {S1, 0, &V22290_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22290_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_282 = {S3, 0, &V22290_282_57155, &P22290_279};	/*Property = BlobBuilder*/
const Value V22290_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_350 = {S4, 0, &V22290_350_57446, &P22290_282};	/*Property = Canvas*/
const Value V22290_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_283 = {S5, 0, &V22290_283_57161, &P22290_350};	/*Property = CssBackground*/
const Value V22290_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_284 = {S6, 0, &V22290_284_57164, &P22290_283};	/*Property = CssBorderImage*/
const Value V22290_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_285 = {S7, 0, &V22290_285_57167, &P22290_284};	/*Property = CssCanvas*/
const Value V22290_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_286 = {S8, 0, &V22290_286_57170, &P22290_285};	/*Property = CssColor*/
const Value V22290_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_237 = {S9, 0, &V22290_237_55786, &P22290_286};	/*Property = CssColumn*/
const Value V22290_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_287 = {S10, 0, &V22290_287_57173, &P22290_237};	/*Property = CssFlexbox*/
const Value V22290_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_288 = {S11, 0, &V22290_288_57176, &P22290_287};	/*Property = CssFont*/
const Value V22290_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_289 = {S12, 0, &V22290_289_86899, &P22290_288};	/*Property = CssImages*/
const Value V22290_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_291 = {S13, 0, &V22290_291_57182, &P22290_289};	/*Property = CssMediaQueries*/
const Value V22290_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_292 = {S14, 0, &V22290_292_57185, &P22290_291};	/*Property = CssMinMax*/
const Value V22290_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_293 = {S15, 0, &V22290_293_57188, &P22290_292};	/*Property = CssOverflow*/
const Value V22290_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_294 = {S16, 0, &V22290_294_57191, &P22290_293};	/*Property = CssPosition*/
const Value V22290_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_295 = {S17, 0, &V22290_295_57194, &P22290_294};	/*Property = CssText*/
const Value V22290_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_297 = {S18, 0, &V22290_297_57197, &P22290_295};	/*Property = CssTransforms*/
const Value V22290_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_241 = {S19, 0, &V22290_241_55801, &P22290_297};	/*Property = CssTransitions*/
const Value V22290_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_298 = {S20, 0, &V22290_298_57208, &P22290_241};	/*Property = CssUI*/
const Value V22290_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_300 = {S21, 0, &V22290_300_57217, &P22290_298};	/*Property = DataSet*/
const Value V22290_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_301 = {S22, 0, &V22290_301_57220, &P22290_300};	/*Property = DataUrl*/
const Value V22290_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_302 = {S23, 0, &V22290_302_57223, &P22290_301};	/*Property = DeviceOrientation*/
const Value V22290_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_304 = {S24, 0, &V22290_304_57229, &P22290_302};	/*Property = FileReader*/
const Value V22290_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_305 = {S25, 0, &V22290_305_57232, &P22290_304};	/*Property = FileSaver*/
const Value V22290_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_306 = {S26, 0, &V22290_306_57238, &P22290_305};	/*Property = FileWriter*/
const Value V22290_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_307 = {S27, 0, &V22290_307_57241, &P22290_306};	/*Property = FormData*/
const Value V22290_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_309 = {S28, 0, &V22290_309_57247, &P22290_307};	/*Property = Fullscreen*/
const Value V22290_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_310 = {S29, 0, &V22290_310_57250, &P22290_309};	/*Property = GeoLocation*/
const Value V22290_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_312 = {S30, 0, &V22290_312_57256, &P22290_310};	/*Property = History*/
const Value V22290_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_314 = {S31, 0, &V22290_314_57262, &P22290_312};	/*Property = Html5*/
const Value V22290_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_313 = {S32, 0, &V22290_313_57259, &P22290_314};	/*Property = Html-Media-Capture*/
const Value V22290_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_315 = {S33, 0, &V22290_315_57265, &P22290_313};	/*Property = Iframe*/
const Value V22290_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_316 = {S34, 0, &V22290_316_57268, &P22290_315};	/*Property = IndexedDB*/
const Value V22290_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_317 = {S35, 0, &V22290_317_57271, &P22290_316};	/*Property = Json*/
const Value V22290_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_15 = {S37, 0, &V22290_15_72, &P22290_317};	/*Property = LayoutEngine*/
const Value V22290_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_344 = {S38, 0, &V22290_344_57274, &P22290_15};	/*Property = Masking*/
const Value V22290_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_320 = {S39, 0, &V22290_320_57292, &P22290_344};	/*Property = PostMessage*/
const Value V22290_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_321 = {S40, 0, &V22290_321_57295, &P22290_320};	/*Property = Progress*/
const Value V22290_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_322 = {S41, 0, &V22290_322_57298, &P22290_321};	/*Property = Prompts*/
const Value V22290_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_323 = {S42, 0, &V22290_323_57301, &P22290_322};	/*Property = Selector*/
const Value V22290_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_324 = {S43, 0, &V22290_324_57307, &P22290_323};	/*Property = Svg*/
const Value V22290_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_327 = {S44, 0, &V22290_327_57316, &P22290_324};	/*Property = TouchEvents*/
const Value V22290_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_328 = {S45, 0, &V22290_328_57319, &P22290_327};	/*Property = Track*/
const Value V22290_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_329 = {S46, 0, &V22290_329_57325, &P22290_328};	/*Property = Video*/
const Value V22290_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_330 = {S47, 0, &V22290_330_57328, &P22290_329};	/*Property = Viewport*/
const Value V22290_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_332 = {S48, 0, &V22290_332_57340, &P22290_330};	/*Property = WebWorkers*/
const Value V22290_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22290_333 = {S49, 0, &V22290_333_57343, &P22290_332};	/*Property = Xhr2*/
const Profile P22290 = {"22290", &P22290_333};	/*Profile ID - 22290*/
const Value V22293_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_279 = {S1, 0, &V22293_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22293_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_282 = {S3, 0, &V22293_282_57155, &P22293_279};	/*Property = BlobBuilder*/
const Value V22293_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_350 = {S4, 0, &V22293_350_57446, &P22293_282};	/*Property = Canvas*/
const Value V22293_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_283 = {S5, 0, &V22293_283_57161, &P22293_350};	/*Property = CssBackground*/
const Value V22293_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_284 = {S6, 0, &V22293_284_57164, &P22293_283};	/*Property = CssBorderImage*/
const Value V22293_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_285 = {S7, 0, &V22293_285_57167, &P22293_284};	/*Property = CssCanvas*/
const Value V22293_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_286 = {S8, 0, &V22293_286_57170, &P22293_285};	/*Property = CssColor*/
const Value V22293_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_237 = {S9, 0, &V22293_237_55786, &P22293_286};	/*Property = CssColumn*/
const Value V22293_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_287 = {S10, 0, &V22293_287_57173, &P22293_237};	/*Property = CssFlexbox*/
const Value V22293_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_288 = {S11, 0, &V22293_288_57176, &P22293_287};	/*Property = CssFont*/
const Value V22293_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_289 = {S12, 0, &V22293_289_86899, &P22293_288};	/*Property = CssImages*/
const Value V22293_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_291 = {S13, 0, &V22293_291_57182, &P22293_289};	/*Property = CssMediaQueries*/
const Value V22293_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_292 = {S14, 0, &V22293_292_57185, &P22293_291};	/*Property = CssMinMax*/
const Value V22293_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_293 = {S15, 0, &V22293_293_57188, &P22293_292};	/*Property = CssOverflow*/
const Value V22293_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_294 = {S16, 0, &V22293_294_57191, &P22293_293};	/*Property = CssPosition*/
const Value V22293_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_295 = {S17, 0, &V22293_295_57194, &P22293_294};	/*Property = CssText*/
const Value V22293_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_297 = {S18, 0, &V22293_297_57197, &P22293_295};	/*Property = CssTransforms*/
const Value V22293_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_241 = {S19, 0, &V22293_241_55801, &P22293_297};	/*Property = CssTransitions*/
const Value V22293_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_298 = {S20, 0, &V22293_298_57208, &P22293_241};	/*Property = CssUI*/
const Value V22293_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_300 = {S21, 0, &V22293_300_57217, &P22293_298};	/*Property = DataSet*/
const Value V22293_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_301 = {S22, 0, &V22293_301_57220, &P22293_300};	/*Property = DataUrl*/
const Value V22293_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_302 = {S23, 0, &V22293_302_57223, &P22293_301};	/*Property = DeviceOrientation*/
const Value V22293_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_304 = {S24, 0, &V22293_304_57229, &P22293_302};	/*Property = FileReader*/
const Value V22293_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_305 = {S25, 0, &V22293_305_57232, &P22293_304};	/*Property = FileSaver*/
const Value V22293_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_306 = {S26, 0, &V22293_306_57238, &P22293_305};	/*Property = FileWriter*/
const Value V22293_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_307 = {S27, 0, &V22293_307_57241, &P22293_306};	/*Property = FormData*/
const Value V22293_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_309 = {S28, 0, &V22293_309_57247, &P22293_307};	/*Property = Fullscreen*/
const Value V22293_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_310 = {S29, 0, &V22293_310_57250, &P22293_309};	/*Property = GeoLocation*/
const Value V22293_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_312 = {S30, 0, &V22293_312_57256, &P22293_310};	/*Property = History*/
const Value V22293_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_314 = {S31, 0, &V22293_314_57262, &P22293_312};	/*Property = Html5*/
const Value V22293_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_313 = {S32, 0, &V22293_313_57259, &P22293_314};	/*Property = Html-Media-Capture*/
const Value V22293_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_315 = {S33, 0, &V22293_315_57265, &P22293_313};	/*Property = Iframe*/
const Value V22293_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_316 = {S34, 0, &V22293_316_57268, &P22293_315};	/*Property = IndexedDB*/
const Value V22293_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_317 = {S35, 0, &V22293_317_57271, &P22293_316};	/*Property = Json*/
const Value V22293_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_15 = {S37, 0, &V22293_15_72, &P22293_317};	/*Property = LayoutEngine*/
const Value V22293_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_344 = {S38, 0, &V22293_344_57274, &P22293_15};	/*Property = Masking*/
const Value V22293_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_320 = {S39, 0, &V22293_320_57292, &P22293_344};	/*Property = PostMessage*/
const Value V22293_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_321 = {S40, 0, &V22293_321_57295, &P22293_320};	/*Property = Progress*/
const Value V22293_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_322 = {S41, 0, &V22293_322_57298, &P22293_321};	/*Property = Prompts*/
const Value V22293_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_323 = {S42, 0, &V22293_323_57301, &P22293_322};	/*Property = Selector*/
const Value V22293_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_324 = {S43, 0, &V22293_324_57307, &P22293_323};	/*Property = Svg*/
const Value V22293_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_327 = {S44, 0, &V22293_327_57316, &P22293_324};	/*Property = TouchEvents*/
const Value V22293_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_328 = {S45, 0, &V22293_328_57319, &P22293_327};	/*Property = Track*/
const Value V22293_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_329 = {S46, 0, &V22293_329_57325, &P22293_328};	/*Property = Video*/
const Value V22293_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_330 = {S47, 0, &V22293_330_57328, &P22293_329};	/*Property = Viewport*/
const Value V22293_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_332 = {S48, 0, &V22293_332_57340, &P22293_330};	/*Property = WebWorkers*/
const Value V22293_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22293_333 = {S49, 0, &V22293_333_57343, &P22293_332};	/*Property = Xhr2*/
const Profile P22293 = {"22293", &P22293_333};	/*Profile ID - 22293*/
const Value V22230_1_1 = {S2, NULL};	/*Value = True*/
const Property P22230_1 = {S50, 0, &V22230_1_1, NULL};	/*Property = IsMobile*/
const Value V22230_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22230_5 = {S52, 0, &V22230_5_1458, &P22230_1};	/*Property = ScreenPixelsHeight*/
const Value V22230_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22230_4 = {S54, 0, &V22230_4_1460, &P22230_5};	/*Property = ScreenPixelsWidth*/
const Profile P22230 = {"22230", &P22230_4};	/*Profile ID - 22230*/
const Value V22299_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_279 = {S1, 0, &V22299_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22299_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_282 = {S3, 0, &V22299_282_57155, &P22299_279};	/*Property = BlobBuilder*/
const Value V22299_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_350 = {S4, 0, &V22299_350_57446, &P22299_282};	/*Property = Canvas*/
const Value V22299_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_283 = {S5, 0, &V22299_283_57161, &P22299_350};	/*Property = CssBackground*/
const Value V22299_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_284 = {S6, 0, &V22299_284_57164, &P22299_283};	/*Property = CssBorderImage*/
const Value V22299_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_285 = {S7, 0, &V22299_285_57167, &P22299_284};	/*Property = CssCanvas*/
const Value V22299_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_286 = {S8, 0, &V22299_286_57170, &P22299_285};	/*Property = CssColor*/
const Value V22299_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_237 = {S9, 0, &V22299_237_55786, &P22299_286};	/*Property = CssColumn*/
const Value V22299_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_287 = {S10, 0, &V22299_287_57173, &P22299_237};	/*Property = CssFlexbox*/
const Value V22299_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_288 = {S11, 0, &V22299_288_57176, &P22299_287};	/*Property = CssFont*/
const Value V22299_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_289 = {S12, 0, &V22299_289_86899, &P22299_288};	/*Property = CssImages*/
const Value V22299_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_291 = {S13, 0, &V22299_291_57182, &P22299_289};	/*Property = CssMediaQueries*/
const Value V22299_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_292 = {S14, 0, &V22299_292_57185, &P22299_291};	/*Property = CssMinMax*/
const Value V22299_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_293 = {S15, 0, &V22299_293_57188, &P22299_292};	/*Property = CssOverflow*/
const Value V22299_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_294 = {S16, 0, &V22299_294_57191, &P22299_293};	/*Property = CssPosition*/
const Value V22299_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_295 = {S17, 0, &V22299_295_57194, &P22299_294};	/*Property = CssText*/
const Value V22299_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_297 = {S18, 0, &V22299_297_57197, &P22299_295};	/*Property = CssTransforms*/
const Value V22299_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_241 = {S19, 0, &V22299_241_55801, &P22299_297};	/*Property = CssTransitions*/
const Value V22299_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_298 = {S20, 0, &V22299_298_57208, &P22299_241};	/*Property = CssUI*/
const Value V22299_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_300 = {S21, 0, &V22299_300_57217, &P22299_298};	/*Property = DataSet*/
const Value V22299_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_301 = {S22, 0, &V22299_301_57220, &P22299_300};	/*Property = DataUrl*/
const Value V22299_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_302 = {S23, 0, &V22299_302_57223, &P22299_301};	/*Property = DeviceOrientation*/
const Value V22299_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_304 = {S24, 0, &V22299_304_57229, &P22299_302};	/*Property = FileReader*/
const Value V22299_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_305 = {S25, 0, &V22299_305_57232, &P22299_304};	/*Property = FileSaver*/
const Value V22299_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_306 = {S26, 0, &V22299_306_57238, &P22299_305};	/*Property = FileWriter*/
const Value V22299_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_307 = {S27, 0, &V22299_307_57241, &P22299_306};	/*Property = FormData*/
const Value V22299_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_309 = {S28, 0, &V22299_309_57247, &P22299_307};	/*Property = Fullscreen*/
const Value V22299_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_310 = {S29, 0, &V22299_310_57250, &P22299_309};	/*Property = GeoLocation*/
const Value V22299_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_312 = {S30, 0, &V22299_312_57256, &P22299_310};	/*Property = History*/
const Value V22299_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_314 = {S31, 0, &V22299_314_57262, &P22299_312};	/*Property = Html5*/
const Value V22299_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_313 = {S32, 0, &V22299_313_57259, &P22299_314};	/*Property = Html-Media-Capture*/
const Value V22299_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_315 = {S33, 0, &V22299_315_57265, &P22299_313};	/*Property = Iframe*/
const Value V22299_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_316 = {S34, 0, &V22299_316_57268, &P22299_315};	/*Property = IndexedDB*/
const Value V22299_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_317 = {S35, 0, &V22299_317_57271, &P22299_316};	/*Property = Json*/
const Value V22299_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_15 = {S37, 0, &V22299_15_72, &P22299_317};	/*Property = LayoutEngine*/
const Value V22299_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_344 = {S38, 0, &V22299_344_57274, &P22299_15};	/*Property = Masking*/
const Value V22299_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_320 = {S39, 0, &V22299_320_57292, &P22299_344};	/*Property = PostMessage*/
const Value V22299_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_321 = {S40, 0, &V22299_321_57295, &P22299_320};	/*Property = Progress*/
const Value V22299_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_322 = {S41, 0, &V22299_322_57298, &P22299_321};	/*Property = Prompts*/
const Value V22299_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_323 = {S42, 0, &V22299_323_57301, &P22299_322};	/*Property = Selector*/
const Value V22299_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_324 = {S43, 0, &V22299_324_57307, &P22299_323};	/*Property = Svg*/
const Value V22299_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_327 = {S44, 0, &V22299_327_57316, &P22299_324};	/*Property = TouchEvents*/
const Value V22299_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_328 = {S45, 0, &V22299_328_57319, &P22299_327};	/*Property = Track*/
const Value V22299_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_329 = {S46, 0, &V22299_329_57325, &P22299_328};	/*Property = Video*/
const Value V22299_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_330 = {S47, 0, &V22299_330_57328, &P22299_329};	/*Property = Viewport*/
const Value V22299_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_332 = {S48, 0, &V22299_332_57340, &P22299_330};	/*Property = WebWorkers*/
const Value V22299_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22299_333 = {S49, 0, &V22299_333_57343, &P22299_332};	/*Property = Xhr2*/
const Profile P22299 = {"22299", &P22299_333};	/*Profile ID - 22299*/
const Value V22073_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_279 = {S1, 0, &V22073_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22073_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_282 = {S3, 0, &V22073_282_57155, &P22073_279};	/*Property = BlobBuilder*/
const Value V22073_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_350 = {S4, 0, &V22073_350_57446, &P22073_282};	/*Property = Canvas*/
const Value V22073_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_283 = {S5, 0, &V22073_283_57161, &P22073_350};	/*Property = CssBackground*/
const Value V22073_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_284 = {S6, 0, &V22073_284_57164, &P22073_283};	/*Property = CssBorderImage*/
const Value V22073_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_285 = {S7, 0, &V22073_285_57167, &P22073_284};	/*Property = CssCanvas*/
const Value V22073_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_286 = {S8, 0, &V22073_286_57170, &P22073_285};	/*Property = CssColor*/
const Value V22073_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_237 = {S9, 0, &V22073_237_55786, &P22073_286};	/*Property = CssColumn*/
const Value V22073_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_287 = {S10, 0, &V22073_287_57173, &P22073_237};	/*Property = CssFlexbox*/
const Value V22073_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_288 = {S11, 0, &V22073_288_57176, &P22073_287};	/*Property = CssFont*/
const Value V22073_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_289 = {S12, 0, &V22073_289_86899, &P22073_288};	/*Property = CssImages*/
const Value V22073_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_291 = {S13, 0, &V22073_291_57182, &P22073_289};	/*Property = CssMediaQueries*/
const Value V22073_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_292 = {S14, 0, &V22073_292_57185, &P22073_291};	/*Property = CssMinMax*/
const Value V22073_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_293 = {S15, 0, &V22073_293_57188, &P22073_292};	/*Property = CssOverflow*/
const Value V22073_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_294 = {S16, 0, &V22073_294_57191, &P22073_293};	/*Property = CssPosition*/
const Value V22073_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_295 = {S17, 0, &V22073_295_57194, &P22073_294};	/*Property = CssText*/
const Value V22073_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_297 = {S18, 0, &V22073_297_57197, &P22073_295};	/*Property = CssTransforms*/
const Value V22073_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_241 = {S19, 0, &V22073_241_55801, &P22073_297};	/*Property = CssTransitions*/
const Value V22073_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_298 = {S20, 0, &V22073_298_57208, &P22073_241};	/*Property = CssUI*/
const Value V22073_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_300 = {S21, 0, &V22073_300_57217, &P22073_298};	/*Property = DataSet*/
const Value V22073_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_301 = {S22, 0, &V22073_301_57220, &P22073_300};	/*Property = DataUrl*/
const Value V22073_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_302 = {S23, 0, &V22073_302_57223, &P22073_301};	/*Property = DeviceOrientation*/
const Value V22073_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_304 = {S24, 0, &V22073_304_57229, &P22073_302};	/*Property = FileReader*/
const Value V22073_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_305 = {S25, 0, &V22073_305_57232, &P22073_304};	/*Property = FileSaver*/
const Value V22073_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_306 = {S26, 0, &V22073_306_57238, &P22073_305};	/*Property = FileWriter*/
const Value V22073_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_307 = {S27, 0, &V22073_307_57241, &P22073_306};	/*Property = FormData*/
const Value V22073_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_309 = {S28, 0, &V22073_309_57247, &P22073_307};	/*Property = Fullscreen*/
const Value V22073_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_310 = {S29, 0, &V22073_310_57250, &P22073_309};	/*Property = GeoLocation*/
const Value V22073_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_312 = {S30, 0, &V22073_312_57256, &P22073_310};	/*Property = History*/
const Value V22073_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_314 = {S31, 0, &V22073_314_57262, &P22073_312};	/*Property = Html5*/
const Value V22073_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_313 = {S32, 0, &V22073_313_57259, &P22073_314};	/*Property = Html-Media-Capture*/
const Value V22073_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_315 = {S33, 0, &V22073_315_57265, &P22073_313};	/*Property = Iframe*/
const Value V22073_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_316 = {S34, 0, &V22073_316_57268, &P22073_315};	/*Property = IndexedDB*/
const Value V22073_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_317 = {S35, 0, &V22073_317_57271, &P22073_316};	/*Property = Json*/
const Value V22073_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_15 = {S37, 0, &V22073_15_72, &P22073_317};	/*Property = LayoutEngine*/
const Value V22073_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_344 = {S38, 0, &V22073_344_57274, &P22073_15};	/*Property = Masking*/
const Value V22073_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_320 = {S39, 0, &V22073_320_57292, &P22073_344};	/*Property = PostMessage*/
const Value V22073_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_321 = {S40, 0, &V22073_321_57295, &P22073_320};	/*Property = Progress*/
const Value V22073_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_322 = {S41, 0, &V22073_322_57298, &P22073_321};	/*Property = Prompts*/
const Value V22073_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_323 = {S42, 0, &V22073_323_57301, &P22073_322};	/*Property = Selector*/
const Value V22073_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_324 = {S43, 0, &V22073_324_57307, &P22073_323};	/*Property = Svg*/
const Value V22073_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_327 = {S44, 0, &V22073_327_57316, &P22073_324};	/*Property = TouchEvents*/
const Value V22073_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_328 = {S45, 0, &V22073_328_57319, &P22073_327};	/*Property = Track*/
const Value V22073_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_329 = {S46, 0, &V22073_329_57325, &P22073_328};	/*Property = Video*/
const Value V22073_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_330 = {S47, 0, &V22073_330_57328, &P22073_329};	/*Property = Viewport*/
const Value V22073_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_332 = {S48, 0, &V22073_332_57340, &P22073_330};	/*Property = WebWorkers*/
const Value V22073_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22073_333 = {S49, 0, &V22073_333_57343, &P22073_332};	/*Property = Xhr2*/
const Profile P22073 = {"22073", &P22073_333};	/*Profile ID - 22073*/
const Value V22298_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_279 = {S1, 0, &V22298_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22298_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_282 = {S3, 0, &V22298_282_57155, &P22298_279};	/*Property = BlobBuilder*/
const Value V22298_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_350 = {S4, 0, &V22298_350_57446, &P22298_282};	/*Property = Canvas*/
const Value V22298_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_283 = {S5, 0, &V22298_283_57161, &P22298_350};	/*Property = CssBackground*/
const Value V22298_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_284 = {S6, 0, &V22298_284_57164, &P22298_283};	/*Property = CssBorderImage*/
const Value V22298_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_285 = {S7, 0, &V22298_285_57167, &P22298_284};	/*Property = CssCanvas*/
const Value V22298_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_286 = {S8, 0, &V22298_286_57170, &P22298_285};	/*Property = CssColor*/
const Value V22298_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_237 = {S9, 0, &V22298_237_55786, &P22298_286};	/*Property = CssColumn*/
const Value V22298_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_287 = {S10, 0, &V22298_287_57173, &P22298_237};	/*Property = CssFlexbox*/
const Value V22298_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_288 = {S11, 0, &V22298_288_57176, &P22298_287};	/*Property = CssFont*/
const Value V22298_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_289 = {S12, 0, &V22298_289_86899, &P22298_288};	/*Property = CssImages*/
const Value V22298_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_291 = {S13, 0, &V22298_291_57182, &P22298_289};	/*Property = CssMediaQueries*/
const Value V22298_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_292 = {S14, 0, &V22298_292_57185, &P22298_291};	/*Property = CssMinMax*/
const Value V22298_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_293 = {S15, 0, &V22298_293_57188, &P22298_292};	/*Property = CssOverflow*/
const Value V22298_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_294 = {S16, 0, &V22298_294_57191, &P22298_293};	/*Property = CssPosition*/
const Value V22298_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_295 = {S17, 0, &V22298_295_57194, &P22298_294};	/*Property = CssText*/
const Value V22298_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_297 = {S18, 0, &V22298_297_57197, &P22298_295};	/*Property = CssTransforms*/
const Value V22298_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_241 = {S19, 0, &V22298_241_55801, &P22298_297};	/*Property = CssTransitions*/
const Value V22298_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_298 = {S20, 0, &V22298_298_57208, &P22298_241};	/*Property = CssUI*/
const Value V22298_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_300 = {S21, 0, &V22298_300_57217, &P22298_298};	/*Property = DataSet*/
const Value V22298_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_301 = {S22, 0, &V22298_301_57220, &P22298_300};	/*Property = DataUrl*/
const Value V22298_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_302 = {S23, 0, &V22298_302_57223, &P22298_301};	/*Property = DeviceOrientation*/
const Value V22298_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_304 = {S24, 0, &V22298_304_57229, &P22298_302};	/*Property = FileReader*/
const Value V22298_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_305 = {S25, 0, &V22298_305_57232, &P22298_304};	/*Property = FileSaver*/
const Value V22298_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_306 = {S26, 0, &V22298_306_57238, &P22298_305};	/*Property = FileWriter*/
const Value V22298_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_307 = {S27, 0, &V22298_307_57241, &P22298_306};	/*Property = FormData*/
const Value V22298_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_309 = {S28, 0, &V22298_309_57247, &P22298_307};	/*Property = Fullscreen*/
const Value V22298_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_310 = {S29, 0, &V22298_310_57250, &P22298_309};	/*Property = GeoLocation*/
const Value V22298_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_312 = {S30, 0, &V22298_312_57256, &P22298_310};	/*Property = History*/
const Value V22298_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_314 = {S31, 0, &V22298_314_57262, &P22298_312};	/*Property = Html5*/
const Value V22298_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_313 = {S32, 0, &V22298_313_57259, &P22298_314};	/*Property = Html-Media-Capture*/
const Value V22298_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_315 = {S33, 0, &V22298_315_57265, &P22298_313};	/*Property = Iframe*/
const Value V22298_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_316 = {S34, 0, &V22298_316_57268, &P22298_315};	/*Property = IndexedDB*/
const Value V22298_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_317 = {S35, 0, &V22298_317_57271, &P22298_316};	/*Property = Json*/
const Value V22298_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_15 = {S37, 0, &V22298_15_72, &P22298_317};	/*Property = LayoutEngine*/
const Value V22298_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_344 = {S38, 0, &V22298_344_57274, &P22298_15};	/*Property = Masking*/
const Value V22298_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_320 = {S39, 0, &V22298_320_57292, &P22298_344};	/*Property = PostMessage*/
const Value V22298_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_321 = {S40, 0, &V22298_321_57295, &P22298_320};	/*Property = Progress*/
const Value V22298_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_322 = {S41, 0, &V22298_322_57298, &P22298_321};	/*Property = Prompts*/
const Value V22298_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_323 = {S42, 0, &V22298_323_57301, &P22298_322};	/*Property = Selector*/
const Value V22298_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_324 = {S43, 0, &V22298_324_57307, &P22298_323};	/*Property = Svg*/
const Value V22298_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_327 = {S44, 0, &V22298_327_57316, &P22298_324};	/*Property = TouchEvents*/
const Value V22298_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_328 = {S45, 0, &V22298_328_57319, &P22298_327};	/*Property = Track*/
const Value V22298_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_329 = {S46, 0, &V22298_329_57325, &P22298_328};	/*Property = Video*/
const Value V22298_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_330 = {S47, 0, &V22298_330_57328, &P22298_329};	/*Property = Viewport*/
const Value V22298_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_332 = {S48, 0, &V22298_332_57340, &P22298_330};	/*Property = WebWorkers*/
const Value V22298_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22298_333 = {S49, 0, &V22298_333_57343, &P22298_332};	/*Property = Xhr2*/
const Profile P22298 = {"22298", &P22298_333};	/*Profile ID - 22298*/
const Value V22300_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_279 = {S1, 0, &V22300_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22300_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_282 = {S3, 0, &V22300_282_57155, &P22300_279};	/*Property = BlobBuilder*/
const Value V22300_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_350 = {S4, 0, &V22300_350_57446, &P22300_282};	/*Property = Canvas*/
const Value V22300_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_283 = {S5, 0, &V22300_283_57161, &P22300_350};	/*Property = CssBackground*/
const Value V22300_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_284 = {S6, 0, &V22300_284_57164, &P22300_283};	/*Property = CssBorderImage*/
const Value V22300_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_285 = {S7, 0, &V22300_285_57167, &P22300_284};	/*Property = CssCanvas*/
const Value V22300_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_286 = {S8, 0, &V22300_286_57170, &P22300_285};	/*Property = CssColor*/
const Value V22300_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_237 = {S9, 0, &V22300_237_55786, &P22300_286};	/*Property = CssColumn*/
const Value V22300_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_287 = {S10, 0, &V22300_287_57173, &P22300_237};	/*Property = CssFlexbox*/
const Value V22300_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_288 = {S11, 0, &V22300_288_57176, &P22300_287};	/*Property = CssFont*/
const Value V22300_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_289 = {S12, 0, &V22300_289_86899, &P22300_288};	/*Property = CssImages*/
const Value V22300_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_291 = {S13, 0, &V22300_291_57182, &P22300_289};	/*Property = CssMediaQueries*/
const Value V22300_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_292 = {S14, 0, &V22300_292_57185, &P22300_291};	/*Property = CssMinMax*/
const Value V22300_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_293 = {S15, 0, &V22300_293_57188, &P22300_292};	/*Property = CssOverflow*/
const Value V22300_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_294 = {S16, 0, &V22300_294_57191, &P22300_293};	/*Property = CssPosition*/
const Value V22300_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_295 = {S17, 0, &V22300_295_57194, &P22300_294};	/*Property = CssText*/
const Value V22300_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_297 = {S18, 0, &V22300_297_57197, &P22300_295};	/*Property = CssTransforms*/
const Value V22300_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_241 = {S19, 0, &V22300_241_55801, &P22300_297};	/*Property = CssTransitions*/
const Value V22300_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_298 = {S20, 0, &V22300_298_57208, &P22300_241};	/*Property = CssUI*/
const Value V22300_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_300 = {S21, 0, &V22300_300_57217, &P22300_298};	/*Property = DataSet*/
const Value V22300_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_301 = {S22, 0, &V22300_301_57220, &P22300_300};	/*Property = DataUrl*/
const Value V22300_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_302 = {S23, 0, &V22300_302_57223, &P22300_301};	/*Property = DeviceOrientation*/
const Value V22300_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_304 = {S24, 0, &V22300_304_57229, &P22300_302};	/*Property = FileReader*/
const Value V22300_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_305 = {S25, 0, &V22300_305_57232, &P22300_304};	/*Property = FileSaver*/
const Value V22300_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_306 = {S26, 0, &V22300_306_57238, &P22300_305};	/*Property = FileWriter*/
const Value V22300_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_307 = {S27, 0, &V22300_307_57241, &P22300_306};	/*Property = FormData*/
const Value V22300_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_309 = {S28, 0, &V22300_309_57247, &P22300_307};	/*Property = Fullscreen*/
const Value V22300_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_310 = {S29, 0, &V22300_310_57250, &P22300_309};	/*Property = GeoLocation*/
const Value V22300_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_312 = {S30, 0, &V22300_312_57256, &P22300_310};	/*Property = History*/
const Value V22300_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_314 = {S31, 0, &V22300_314_57262, &P22300_312};	/*Property = Html5*/
const Value V22300_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_313 = {S32, 0, &V22300_313_57259, &P22300_314};	/*Property = Html-Media-Capture*/
const Value V22300_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_315 = {S33, 0, &V22300_315_57265, &P22300_313};	/*Property = Iframe*/
const Value V22300_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_316 = {S34, 0, &V22300_316_57268, &P22300_315};	/*Property = IndexedDB*/
const Value V22300_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_317 = {S35, 0, &V22300_317_57271, &P22300_316};	/*Property = Json*/
const Value V22300_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_15 = {S37, 0, &V22300_15_72, &P22300_317};	/*Property = LayoutEngine*/
const Value V22300_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_344 = {S38, 0, &V22300_344_57274, &P22300_15};	/*Property = Masking*/
const Value V22300_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_320 = {S39, 0, &V22300_320_57292, &P22300_344};	/*Property = PostMessage*/
const Value V22300_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_321 = {S40, 0, &V22300_321_57295, &P22300_320};	/*Property = Progress*/
const Value V22300_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_322 = {S41, 0, &V22300_322_57298, &P22300_321};	/*Property = Prompts*/
const Value V22300_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_323 = {S42, 0, &V22300_323_57301, &P22300_322};	/*Property = Selector*/
const Value V22300_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_324 = {S43, 0, &V22300_324_57307, &P22300_323};	/*Property = Svg*/
const Value V22300_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_327 = {S44, 0, &V22300_327_57316, &P22300_324};	/*Property = TouchEvents*/
const Value V22300_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_328 = {S45, 0, &V22300_328_57319, &P22300_327};	/*Property = Track*/
const Value V22300_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_329 = {S46, 0, &V22300_329_57325, &P22300_328};	/*Property = Video*/
const Value V22300_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_330 = {S47, 0, &V22300_330_57328, &P22300_329};	/*Property = Viewport*/
const Value V22300_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_332 = {S48, 0, &V22300_332_57340, &P22300_330};	/*Property = WebWorkers*/
const Value V22300_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22300_333 = {S49, 0, &V22300_333_57343, &P22300_332};	/*Property = Xhr2*/
const Profile P22300 = {"22300", &P22300_333};	/*Profile ID - 22300*/
const Value V22039_1_1 = {S2, NULL};	/*Value = True*/
const Property P22039_1 = {S50, 0, &V22039_1_1, NULL};	/*Property = IsMobile*/
const Value V22039_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22039_5 = {S52, 0, &V22039_5_1832, &P22039_1};	/*Property = ScreenPixelsHeight*/
const Value V22039_4_52971 = {S122, NULL};	/*Value = 1200*/
const Property P22039_4 = {S54, 0, &V22039_4_52971, &P22039_5};	/*Property = ScreenPixelsWidth*/
const Profile P22039 = {"22039", &P22039_4};	/*Profile ID - 22039*/
const Value V22080_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_279 = {S1, 0, &V22080_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22080_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_282 = {S3, 0, &V22080_282_57155, &P22080_279};	/*Property = BlobBuilder*/
const Value V22080_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_350 = {S4, 0, &V22080_350_57446, &P22080_282};	/*Property = Canvas*/
const Value V22080_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_283 = {S5, 0, &V22080_283_57161, &P22080_350};	/*Property = CssBackground*/
const Value V22080_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_284 = {S6, 0, &V22080_284_57164, &P22080_283};	/*Property = CssBorderImage*/
const Value V22080_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_285 = {S7, 0, &V22080_285_57167, &P22080_284};	/*Property = CssCanvas*/
const Value V22080_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_286 = {S8, 0, &V22080_286_57170, &P22080_285};	/*Property = CssColor*/
const Value V22080_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_237 = {S9, 0, &V22080_237_55786, &P22080_286};	/*Property = CssColumn*/
const Value V22080_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_287 = {S10, 0, &V22080_287_57173, &P22080_237};	/*Property = CssFlexbox*/
const Value V22080_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_288 = {S11, 0, &V22080_288_57176, &P22080_287};	/*Property = CssFont*/
const Value V22080_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_289 = {S12, 0, &V22080_289_86899, &P22080_288};	/*Property = CssImages*/
const Value V22080_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_291 = {S13, 0, &V22080_291_57182, &P22080_289};	/*Property = CssMediaQueries*/
const Value V22080_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_292 = {S14, 0, &V22080_292_57185, &P22080_291};	/*Property = CssMinMax*/
const Value V22080_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_293 = {S15, 0, &V22080_293_57188, &P22080_292};	/*Property = CssOverflow*/
const Value V22080_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_294 = {S16, 0, &V22080_294_57191, &P22080_293};	/*Property = CssPosition*/
const Value V22080_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_295 = {S17, 0, &V22080_295_57194, &P22080_294};	/*Property = CssText*/
const Value V22080_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_297 = {S18, 0, &V22080_297_57197, &P22080_295};	/*Property = CssTransforms*/
const Value V22080_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_241 = {S19, 0, &V22080_241_55801, &P22080_297};	/*Property = CssTransitions*/
const Value V22080_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_298 = {S20, 0, &V22080_298_57208, &P22080_241};	/*Property = CssUI*/
const Value V22080_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_300 = {S21, 0, &V22080_300_57217, &P22080_298};	/*Property = DataSet*/
const Value V22080_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_301 = {S22, 0, &V22080_301_57220, &P22080_300};	/*Property = DataUrl*/
const Value V22080_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_302 = {S23, 0, &V22080_302_57223, &P22080_301};	/*Property = DeviceOrientation*/
const Value V22080_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_304 = {S24, 0, &V22080_304_57229, &P22080_302};	/*Property = FileReader*/
const Value V22080_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_305 = {S25, 0, &V22080_305_57232, &P22080_304};	/*Property = FileSaver*/
const Value V22080_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_306 = {S26, 0, &V22080_306_57238, &P22080_305};	/*Property = FileWriter*/
const Value V22080_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_307 = {S27, 0, &V22080_307_57241, &P22080_306};	/*Property = FormData*/
const Value V22080_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_309 = {S28, 0, &V22080_309_57247, &P22080_307};	/*Property = Fullscreen*/
const Value V22080_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_310 = {S29, 0, &V22080_310_57250, &P22080_309};	/*Property = GeoLocation*/
const Value V22080_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_312 = {S30, 0, &V22080_312_57256, &P22080_310};	/*Property = History*/
const Value V22080_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_314 = {S31, 0, &V22080_314_57262, &P22080_312};	/*Property = Html5*/
const Value V22080_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_313 = {S32, 0, &V22080_313_57259, &P22080_314};	/*Property = Html-Media-Capture*/
const Value V22080_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_315 = {S33, 0, &V22080_315_57265, &P22080_313};	/*Property = Iframe*/
const Value V22080_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_316 = {S34, 0, &V22080_316_57268, &P22080_315};	/*Property = IndexedDB*/
const Value V22080_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_317 = {S35, 0, &V22080_317_57271, &P22080_316};	/*Property = Json*/
const Value V22080_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_15 = {S37, 0, &V22080_15_72, &P22080_317};	/*Property = LayoutEngine*/
const Value V22080_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_344 = {S38, 0, &V22080_344_57274, &P22080_15};	/*Property = Masking*/
const Value V22080_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_320 = {S39, 0, &V22080_320_57292, &P22080_344};	/*Property = PostMessage*/
const Value V22080_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_321 = {S40, 0, &V22080_321_57295, &P22080_320};	/*Property = Progress*/
const Value V22080_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_322 = {S41, 0, &V22080_322_57298, &P22080_321};	/*Property = Prompts*/
const Value V22080_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_323 = {S42, 0, &V22080_323_57301, &P22080_322};	/*Property = Selector*/
const Value V22080_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_324 = {S43, 0, &V22080_324_57307, &P22080_323};	/*Property = Svg*/
const Value V22080_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_327 = {S44, 0, &V22080_327_57316, &P22080_324};	/*Property = TouchEvents*/
const Value V22080_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_328 = {S45, 0, &V22080_328_57319, &P22080_327};	/*Property = Track*/
const Value V22080_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_329 = {S46, 0, &V22080_329_57325, &P22080_328};	/*Property = Video*/
const Value V22080_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_330 = {S47, 0, &V22080_330_57328, &P22080_329};	/*Property = Viewport*/
const Value V22080_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_332 = {S48, 0, &V22080_332_57340, &P22080_330};	/*Property = WebWorkers*/
const Value V22080_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22080_333 = {S49, 0, &V22080_333_57343, &P22080_332};	/*Property = Xhr2*/
const Profile P22080 = {"22080", &P22080_333};	/*Profile ID - 22080*/
const Value V22301_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_279 = {S1, 0, &V22301_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22301_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_282 = {S3, 0, &V22301_282_57155, &P22301_279};	/*Property = BlobBuilder*/
const Value V22301_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_350 = {S4, 0, &V22301_350_57446, &P22301_282};	/*Property = Canvas*/
const Value V22301_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_283 = {S5, 0, &V22301_283_57161, &P22301_350};	/*Property = CssBackground*/
const Value V22301_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_284 = {S6, 0, &V22301_284_57164, &P22301_283};	/*Property = CssBorderImage*/
const Value V22301_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_285 = {S7, 0, &V22301_285_57167, &P22301_284};	/*Property = CssCanvas*/
const Value V22301_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_286 = {S8, 0, &V22301_286_57170, &P22301_285};	/*Property = CssColor*/
const Value V22301_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_237 = {S9, 0, &V22301_237_55786, &P22301_286};	/*Property = CssColumn*/
const Value V22301_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_287 = {S10, 0, &V22301_287_57173, &P22301_237};	/*Property = CssFlexbox*/
const Value V22301_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_288 = {S11, 0, &V22301_288_57176, &P22301_287};	/*Property = CssFont*/
const Value V22301_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_289 = {S12, 0, &V22301_289_86899, &P22301_288};	/*Property = CssImages*/
const Value V22301_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_291 = {S13, 0, &V22301_291_57182, &P22301_289};	/*Property = CssMediaQueries*/
const Value V22301_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_292 = {S14, 0, &V22301_292_57185, &P22301_291};	/*Property = CssMinMax*/
const Value V22301_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_293 = {S15, 0, &V22301_293_57188, &P22301_292};	/*Property = CssOverflow*/
const Value V22301_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_294 = {S16, 0, &V22301_294_57191, &P22301_293};	/*Property = CssPosition*/
const Value V22301_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_295 = {S17, 0, &V22301_295_57194, &P22301_294};	/*Property = CssText*/
const Value V22301_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_297 = {S18, 0, &V22301_297_57197, &P22301_295};	/*Property = CssTransforms*/
const Value V22301_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_241 = {S19, 0, &V22301_241_55801, &P22301_297};	/*Property = CssTransitions*/
const Value V22301_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_298 = {S20, 0, &V22301_298_57208, &P22301_241};	/*Property = CssUI*/
const Value V22301_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_300 = {S21, 0, &V22301_300_57217, &P22301_298};	/*Property = DataSet*/
const Value V22301_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_301 = {S22, 0, &V22301_301_57220, &P22301_300};	/*Property = DataUrl*/
const Value V22301_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_302 = {S23, 0, &V22301_302_57223, &P22301_301};	/*Property = DeviceOrientation*/
const Value V22301_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_304 = {S24, 0, &V22301_304_57229, &P22301_302};	/*Property = FileReader*/
const Value V22301_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_305 = {S25, 0, &V22301_305_57232, &P22301_304};	/*Property = FileSaver*/
const Value V22301_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_306 = {S26, 0, &V22301_306_57238, &P22301_305};	/*Property = FileWriter*/
const Value V22301_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_307 = {S27, 0, &V22301_307_57241, &P22301_306};	/*Property = FormData*/
const Value V22301_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_309 = {S28, 0, &V22301_309_57247, &P22301_307};	/*Property = Fullscreen*/
const Value V22301_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_310 = {S29, 0, &V22301_310_57250, &P22301_309};	/*Property = GeoLocation*/
const Value V22301_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_312 = {S30, 0, &V22301_312_57256, &P22301_310};	/*Property = History*/
const Value V22301_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_314 = {S31, 0, &V22301_314_57262, &P22301_312};	/*Property = Html5*/
const Value V22301_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_313 = {S32, 0, &V22301_313_57259, &P22301_314};	/*Property = Html-Media-Capture*/
const Value V22301_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_315 = {S33, 0, &V22301_315_57265, &P22301_313};	/*Property = Iframe*/
const Value V22301_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_316 = {S34, 0, &V22301_316_57268, &P22301_315};	/*Property = IndexedDB*/
const Value V22301_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_317 = {S35, 0, &V22301_317_57271, &P22301_316};	/*Property = Json*/
const Value V22301_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_15 = {S37, 0, &V22301_15_72, &P22301_317};	/*Property = LayoutEngine*/
const Value V22301_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_344 = {S38, 0, &V22301_344_57274, &P22301_15};	/*Property = Masking*/
const Value V22301_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_320 = {S39, 0, &V22301_320_57292, &P22301_344};	/*Property = PostMessage*/
const Value V22301_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_321 = {S40, 0, &V22301_321_57295, &P22301_320};	/*Property = Progress*/
const Value V22301_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_322 = {S41, 0, &V22301_322_57298, &P22301_321};	/*Property = Prompts*/
const Value V22301_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_323 = {S42, 0, &V22301_323_57301, &P22301_322};	/*Property = Selector*/
const Value V22301_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_324 = {S43, 0, &V22301_324_57307, &P22301_323};	/*Property = Svg*/
const Value V22301_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_327 = {S44, 0, &V22301_327_57316, &P22301_324};	/*Property = TouchEvents*/
const Value V22301_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_328 = {S45, 0, &V22301_328_57319, &P22301_327};	/*Property = Track*/
const Value V22301_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_329 = {S46, 0, &V22301_329_57325, &P22301_328};	/*Property = Video*/
const Value V22301_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_330 = {S47, 0, &V22301_330_57328, &P22301_329};	/*Property = Viewport*/
const Value V22301_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_332 = {S48, 0, &V22301_332_57340, &P22301_330};	/*Property = WebWorkers*/
const Value V22301_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22301_333 = {S49, 0, &V22301_333_57343, &P22301_332};	/*Property = Xhr2*/
const Profile P22301 = {"22301", &P22301_333};	/*Profile ID - 22301*/
const Value V22289_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_279 = {S1, 0, &V22289_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22289_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_282 = {S3, 0, &V22289_282_57155, &P22289_279};	/*Property = BlobBuilder*/
const Value V22289_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_350 = {S4, 0, &V22289_350_57446, &P22289_282};	/*Property = Canvas*/
const Value V22289_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_283 = {S5, 0, &V22289_283_57161, &P22289_350};	/*Property = CssBackground*/
const Value V22289_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_284 = {S6, 0, &V22289_284_57164, &P22289_283};	/*Property = CssBorderImage*/
const Value V22289_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_285 = {S7, 0, &V22289_285_57167, &P22289_284};	/*Property = CssCanvas*/
const Value V22289_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_286 = {S8, 0, &V22289_286_57170, &P22289_285};	/*Property = CssColor*/
const Value V22289_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_237 = {S9, 0, &V22289_237_55786, &P22289_286};	/*Property = CssColumn*/
const Value V22289_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_287 = {S10, 0, &V22289_287_57173, &P22289_237};	/*Property = CssFlexbox*/
const Value V22289_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_288 = {S11, 0, &V22289_288_57176, &P22289_287};	/*Property = CssFont*/
const Value V22289_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_289 = {S12, 0, &V22289_289_86899, &P22289_288};	/*Property = CssImages*/
const Value V22289_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_291 = {S13, 0, &V22289_291_57182, &P22289_289};	/*Property = CssMediaQueries*/
const Value V22289_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_292 = {S14, 0, &V22289_292_57185, &P22289_291};	/*Property = CssMinMax*/
const Value V22289_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_293 = {S15, 0, &V22289_293_57188, &P22289_292};	/*Property = CssOverflow*/
const Value V22289_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_294 = {S16, 0, &V22289_294_57191, &P22289_293};	/*Property = CssPosition*/
const Value V22289_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_295 = {S17, 0, &V22289_295_57194, &P22289_294};	/*Property = CssText*/
const Value V22289_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_297 = {S18, 0, &V22289_297_57197, &P22289_295};	/*Property = CssTransforms*/
const Value V22289_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_241 = {S19, 0, &V22289_241_55801, &P22289_297};	/*Property = CssTransitions*/
const Value V22289_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_298 = {S20, 0, &V22289_298_57208, &P22289_241};	/*Property = CssUI*/
const Value V22289_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_300 = {S21, 0, &V22289_300_57217, &P22289_298};	/*Property = DataSet*/
const Value V22289_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_301 = {S22, 0, &V22289_301_57220, &P22289_300};	/*Property = DataUrl*/
const Value V22289_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_302 = {S23, 0, &V22289_302_57223, &P22289_301};	/*Property = DeviceOrientation*/
const Value V22289_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_304 = {S24, 0, &V22289_304_57229, &P22289_302};	/*Property = FileReader*/
const Value V22289_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_305 = {S25, 0, &V22289_305_57232, &P22289_304};	/*Property = FileSaver*/
const Value V22289_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_306 = {S26, 0, &V22289_306_57238, &P22289_305};	/*Property = FileWriter*/
const Value V22289_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_307 = {S27, 0, &V22289_307_57241, &P22289_306};	/*Property = FormData*/
const Value V22289_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_309 = {S28, 0, &V22289_309_57247, &P22289_307};	/*Property = Fullscreen*/
const Value V22289_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_310 = {S29, 0, &V22289_310_57250, &P22289_309};	/*Property = GeoLocation*/
const Value V22289_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_312 = {S30, 0, &V22289_312_57256, &P22289_310};	/*Property = History*/
const Value V22289_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_314 = {S31, 0, &V22289_314_57262, &P22289_312};	/*Property = Html5*/
const Value V22289_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_313 = {S32, 0, &V22289_313_57259, &P22289_314};	/*Property = Html-Media-Capture*/
const Value V22289_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_315 = {S33, 0, &V22289_315_57265, &P22289_313};	/*Property = Iframe*/
const Value V22289_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_316 = {S34, 0, &V22289_316_57268, &P22289_315};	/*Property = IndexedDB*/
const Value V22289_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_317 = {S35, 0, &V22289_317_57271, &P22289_316};	/*Property = Json*/
const Value V22289_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_15 = {S37, 0, &V22289_15_72, &P22289_317};	/*Property = LayoutEngine*/
const Value V22289_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_344 = {S38, 0, &V22289_344_57274, &P22289_15};	/*Property = Masking*/
const Value V22289_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_320 = {S39, 0, &V22289_320_57292, &P22289_344};	/*Property = PostMessage*/
const Value V22289_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_321 = {S40, 0, &V22289_321_57295, &P22289_320};	/*Property = Progress*/
const Value V22289_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_322 = {S41, 0, &V22289_322_57298, &P22289_321};	/*Property = Prompts*/
const Value V22289_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_323 = {S42, 0, &V22289_323_57301, &P22289_322};	/*Property = Selector*/
const Value V22289_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_324 = {S43, 0, &V22289_324_57307, &P22289_323};	/*Property = Svg*/
const Value V22289_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_327 = {S44, 0, &V22289_327_57316, &P22289_324};	/*Property = TouchEvents*/
const Value V22289_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_328 = {S45, 0, &V22289_328_57319, &P22289_327};	/*Property = Track*/
const Value V22289_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_329 = {S46, 0, &V22289_329_57325, &P22289_328};	/*Property = Video*/
const Value V22289_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_330 = {S47, 0, &V22289_330_57328, &P22289_329};	/*Property = Viewport*/
const Value V22289_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_332 = {S48, 0, &V22289_332_57340, &P22289_330};	/*Property = WebWorkers*/
const Value V22289_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22289_333 = {S49, 0, &V22289_333_57343, &P22289_332};	/*Property = Xhr2*/
const Profile P22289 = {"22289", &P22289_333};	/*Profile ID - 22289*/
const Value V18529_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_279 = {S1, 0, &V18529_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18529_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_282 = {S3, 0, &V18529_282_57155, &P18529_279};	/*Property = BlobBuilder*/
const Value V18529_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_350 = {S4, 0, &V18529_350_57446, &P18529_282};	/*Property = Canvas*/
const Value V18529_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_283 = {S5, 0, &V18529_283_57161, &P18529_350};	/*Property = CssBackground*/
const Value V18529_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_284 = {S6, 0, &V18529_284_57164, &P18529_283};	/*Property = CssBorderImage*/
const Value V18529_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_285 = {S7, 0, &V18529_285_57167, &P18529_284};	/*Property = CssCanvas*/
const Value V18529_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_286 = {S8, 0, &V18529_286_57170, &P18529_285};	/*Property = CssColor*/
const Value V18529_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_237 = {S9, 0, &V18529_237_55786, &P18529_286};	/*Property = CssColumn*/
const Value V18529_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_287 = {S10, 0, &V18529_287_57173, &P18529_237};	/*Property = CssFlexbox*/
const Value V18529_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_288 = {S11, 0, &V18529_288_57176, &P18529_287};	/*Property = CssFont*/
const Value V18529_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_289 = {S12, 0, &V18529_289_86899, &P18529_288};	/*Property = CssImages*/
const Value V18529_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_291 = {S13, 0, &V18529_291_57182, &P18529_289};	/*Property = CssMediaQueries*/
const Value V18529_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_292 = {S14, 0, &V18529_292_57185, &P18529_291};	/*Property = CssMinMax*/
const Value V18529_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_293 = {S15, 0, &V18529_293_57188, &P18529_292};	/*Property = CssOverflow*/
const Value V18529_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_294 = {S16, 0, &V18529_294_57191, &P18529_293};	/*Property = CssPosition*/
const Value V18529_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_295 = {S17, 0, &V18529_295_57194, &P18529_294};	/*Property = CssText*/
const Value V18529_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_297 = {S18, 0, &V18529_297_57197, &P18529_295};	/*Property = CssTransforms*/
const Value V18529_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_241 = {S19, 0, &V18529_241_55801, &P18529_297};	/*Property = CssTransitions*/
const Value V18529_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_298 = {S20, 0, &V18529_298_57208, &P18529_241};	/*Property = CssUI*/
const Value V18529_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_300 = {S21, 0, &V18529_300_57217, &P18529_298};	/*Property = DataSet*/
const Value V18529_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_301 = {S22, 0, &V18529_301_57220, &P18529_300};	/*Property = DataUrl*/
const Value V18529_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_302 = {S23, 0, &V18529_302_57223, &P18529_301};	/*Property = DeviceOrientation*/
const Value V18529_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_304 = {S24, 0, &V18529_304_57229, &P18529_302};	/*Property = FileReader*/
const Value V18529_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_305 = {S25, 0, &V18529_305_57232, &P18529_304};	/*Property = FileSaver*/
const Value V18529_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_306 = {S26, 0, &V18529_306_57238, &P18529_305};	/*Property = FileWriter*/
const Value V18529_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_307 = {S27, 0, &V18529_307_57241, &P18529_306};	/*Property = FormData*/
const Value V18529_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_309 = {S28, 0, &V18529_309_57247, &P18529_307};	/*Property = Fullscreen*/
const Value V18529_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_310 = {S29, 0, &V18529_310_57250, &P18529_309};	/*Property = GeoLocation*/
const Value V18529_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_312 = {S30, 0, &V18529_312_57256, &P18529_310};	/*Property = History*/
const Value V18529_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_314 = {S31, 0, &V18529_314_57262, &P18529_312};	/*Property = Html5*/
const Value V18529_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_313 = {S32, 0, &V18529_313_57259, &P18529_314};	/*Property = Html-Media-Capture*/
const Value V18529_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_315 = {S33, 0, &V18529_315_57265, &P18529_313};	/*Property = Iframe*/
const Value V18529_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_316 = {S34, 0, &V18529_316_57268, &P18529_315};	/*Property = IndexedDB*/
const Value V18529_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_317 = {S35, 0, &V18529_317_57271, &P18529_316};	/*Property = Json*/
const Value V18529_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P18529_15 = {S37, 0, &V18529_15_28, &P18529_317};	/*Property = LayoutEngine*/
const Value V18529_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_344 = {S38, 0, &V18529_344_57274, &P18529_15};	/*Property = Masking*/
const Value V18529_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_320 = {S39, 0, &V18529_320_57292, &P18529_344};	/*Property = PostMessage*/
const Value V18529_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_321 = {S40, 0, &V18529_321_57295, &P18529_320};	/*Property = Progress*/
const Value V18529_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_322 = {S41, 0, &V18529_322_57298, &P18529_321};	/*Property = Prompts*/
const Value V18529_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_323 = {S42, 0, &V18529_323_57301, &P18529_322};	/*Property = Selector*/
const Value V18529_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_324 = {S43, 0, &V18529_324_57307, &P18529_323};	/*Property = Svg*/
const Value V18529_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_327 = {S44, 0, &V18529_327_57316, &P18529_324};	/*Property = TouchEvents*/
const Value V18529_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_328 = {S45, 0, &V18529_328_57319, &P18529_327};	/*Property = Track*/
const Value V18529_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_329 = {S46, 0, &V18529_329_57325, &P18529_328};	/*Property = Video*/
const Value V18529_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_330 = {S47, 0, &V18529_330_57328, &P18529_329};	/*Property = Viewport*/
const Value V18529_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_332 = {S48, 0, &V18529_332_57340, &P18529_330};	/*Property = WebWorkers*/
const Value V18529_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18529_333 = {S49, 0, &V18529_333_57343, &P18529_332};	/*Property = Xhr2*/
const Profile P18529 = {"18529", &P18529_333};	/*Profile ID - 18529*/
const Value V22297_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_279 = {S1, 0, &V22297_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22297_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_282 = {S3, 0, &V22297_282_57155, &P22297_279};	/*Property = BlobBuilder*/
const Value V22297_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_350 = {S4, 0, &V22297_350_57446, &P22297_282};	/*Property = Canvas*/
const Value V22297_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_283 = {S5, 0, &V22297_283_57161, &P22297_350};	/*Property = CssBackground*/
const Value V22297_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_284 = {S6, 0, &V22297_284_57164, &P22297_283};	/*Property = CssBorderImage*/
const Value V22297_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_285 = {S7, 0, &V22297_285_57167, &P22297_284};	/*Property = CssCanvas*/
const Value V22297_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_286 = {S8, 0, &V22297_286_57170, &P22297_285};	/*Property = CssColor*/
const Value V22297_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_237 = {S9, 0, &V22297_237_55786, &P22297_286};	/*Property = CssColumn*/
const Value V22297_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_287 = {S10, 0, &V22297_287_57173, &P22297_237};	/*Property = CssFlexbox*/
const Value V22297_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_288 = {S11, 0, &V22297_288_57176, &P22297_287};	/*Property = CssFont*/
const Value V22297_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_289 = {S12, 0, &V22297_289_86899, &P22297_288};	/*Property = CssImages*/
const Value V22297_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_291 = {S13, 0, &V22297_291_57182, &P22297_289};	/*Property = CssMediaQueries*/
const Value V22297_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_292 = {S14, 0, &V22297_292_57185, &P22297_291};	/*Property = CssMinMax*/
const Value V22297_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_293 = {S15, 0, &V22297_293_57188, &P22297_292};	/*Property = CssOverflow*/
const Value V22297_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_294 = {S16, 0, &V22297_294_57191, &P22297_293};	/*Property = CssPosition*/
const Value V22297_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_295 = {S17, 0, &V22297_295_57194, &P22297_294};	/*Property = CssText*/
const Value V22297_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_297 = {S18, 0, &V22297_297_57197, &P22297_295};	/*Property = CssTransforms*/
const Value V22297_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_241 = {S19, 0, &V22297_241_55801, &P22297_297};	/*Property = CssTransitions*/
const Value V22297_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_298 = {S20, 0, &V22297_298_57208, &P22297_241};	/*Property = CssUI*/
const Value V22297_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_300 = {S21, 0, &V22297_300_57217, &P22297_298};	/*Property = DataSet*/
const Value V22297_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_301 = {S22, 0, &V22297_301_57220, &P22297_300};	/*Property = DataUrl*/
const Value V22297_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_302 = {S23, 0, &V22297_302_57223, &P22297_301};	/*Property = DeviceOrientation*/
const Value V22297_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_304 = {S24, 0, &V22297_304_57229, &P22297_302};	/*Property = FileReader*/
const Value V22297_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_305 = {S25, 0, &V22297_305_57232, &P22297_304};	/*Property = FileSaver*/
const Value V22297_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_306 = {S26, 0, &V22297_306_57238, &P22297_305};	/*Property = FileWriter*/
const Value V22297_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_307 = {S27, 0, &V22297_307_57241, &P22297_306};	/*Property = FormData*/
const Value V22297_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_309 = {S28, 0, &V22297_309_57247, &P22297_307};	/*Property = Fullscreen*/
const Value V22297_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_310 = {S29, 0, &V22297_310_57250, &P22297_309};	/*Property = GeoLocation*/
const Value V22297_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_312 = {S30, 0, &V22297_312_57256, &P22297_310};	/*Property = History*/
const Value V22297_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_314 = {S31, 0, &V22297_314_57262, &P22297_312};	/*Property = Html5*/
const Value V22297_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_313 = {S32, 0, &V22297_313_57259, &P22297_314};	/*Property = Html-Media-Capture*/
const Value V22297_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_315 = {S33, 0, &V22297_315_57265, &P22297_313};	/*Property = Iframe*/
const Value V22297_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_316 = {S34, 0, &V22297_316_57268, &P22297_315};	/*Property = IndexedDB*/
const Value V22297_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_317 = {S35, 0, &V22297_317_57271, &P22297_316};	/*Property = Json*/
const Value V22297_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_15 = {S37, 0, &V22297_15_72, &P22297_317};	/*Property = LayoutEngine*/
const Value V22297_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_344 = {S38, 0, &V22297_344_57274, &P22297_15};	/*Property = Masking*/
const Value V22297_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_320 = {S39, 0, &V22297_320_57292, &P22297_344};	/*Property = PostMessage*/
const Value V22297_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_321 = {S40, 0, &V22297_321_57295, &P22297_320};	/*Property = Progress*/
const Value V22297_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_322 = {S41, 0, &V22297_322_57298, &P22297_321};	/*Property = Prompts*/
const Value V22297_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_323 = {S42, 0, &V22297_323_57301, &P22297_322};	/*Property = Selector*/
const Value V22297_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_324 = {S43, 0, &V22297_324_57307, &P22297_323};	/*Property = Svg*/
const Value V22297_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_327 = {S44, 0, &V22297_327_57316, &P22297_324};	/*Property = TouchEvents*/
const Value V22297_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_328 = {S45, 0, &V22297_328_57319, &P22297_327};	/*Property = Track*/
const Value V22297_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_329 = {S46, 0, &V22297_329_57325, &P22297_328};	/*Property = Video*/
const Value V22297_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_330 = {S47, 0, &V22297_330_57328, &P22297_329};	/*Property = Viewport*/
const Value V22297_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_332 = {S48, 0, &V22297_332_57340, &P22297_330};	/*Property = WebWorkers*/
const Value V22297_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22297_333 = {S49, 0, &V22297_333_57343, &P22297_332};	/*Property = Xhr2*/
const Profile P22297 = {"22297", &P22297_333};	/*Profile ID - 22297*/
const Value V22294_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_279 = {S1, 0, &V22294_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22294_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_282 = {S3, 0, &V22294_282_57155, &P22294_279};	/*Property = BlobBuilder*/
const Value V22294_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_350 = {S4, 0, &V22294_350_57446, &P22294_282};	/*Property = Canvas*/
const Value V22294_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_283 = {S5, 0, &V22294_283_57161, &P22294_350};	/*Property = CssBackground*/
const Value V22294_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_284 = {S6, 0, &V22294_284_57164, &P22294_283};	/*Property = CssBorderImage*/
const Value V22294_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_285 = {S7, 0, &V22294_285_57167, &P22294_284};	/*Property = CssCanvas*/
const Value V22294_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_286 = {S8, 0, &V22294_286_57170, &P22294_285};	/*Property = CssColor*/
const Value V22294_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_237 = {S9, 0, &V22294_237_55786, &P22294_286};	/*Property = CssColumn*/
const Value V22294_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_287 = {S10, 0, &V22294_287_57173, &P22294_237};	/*Property = CssFlexbox*/
const Value V22294_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_288 = {S11, 0, &V22294_288_57176, &P22294_287};	/*Property = CssFont*/
const Value V22294_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_289 = {S12, 0, &V22294_289_86899, &P22294_288};	/*Property = CssImages*/
const Value V22294_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_291 = {S13, 0, &V22294_291_57182, &P22294_289};	/*Property = CssMediaQueries*/
const Value V22294_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_292 = {S14, 0, &V22294_292_57185, &P22294_291};	/*Property = CssMinMax*/
const Value V22294_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_293 = {S15, 0, &V22294_293_57188, &P22294_292};	/*Property = CssOverflow*/
const Value V22294_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_294 = {S16, 0, &V22294_294_57191, &P22294_293};	/*Property = CssPosition*/
const Value V22294_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_295 = {S17, 0, &V22294_295_57194, &P22294_294};	/*Property = CssText*/
const Value V22294_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_297 = {S18, 0, &V22294_297_57197, &P22294_295};	/*Property = CssTransforms*/
const Value V22294_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_241 = {S19, 0, &V22294_241_55801, &P22294_297};	/*Property = CssTransitions*/
const Value V22294_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_298 = {S20, 0, &V22294_298_57208, &P22294_241};	/*Property = CssUI*/
const Value V22294_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_300 = {S21, 0, &V22294_300_57217, &P22294_298};	/*Property = DataSet*/
const Value V22294_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_301 = {S22, 0, &V22294_301_57220, &P22294_300};	/*Property = DataUrl*/
const Value V22294_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_302 = {S23, 0, &V22294_302_57223, &P22294_301};	/*Property = DeviceOrientation*/
const Value V22294_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_304 = {S24, 0, &V22294_304_57229, &P22294_302};	/*Property = FileReader*/
const Value V22294_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_305 = {S25, 0, &V22294_305_57232, &P22294_304};	/*Property = FileSaver*/
const Value V22294_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_306 = {S26, 0, &V22294_306_57238, &P22294_305};	/*Property = FileWriter*/
const Value V22294_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_307 = {S27, 0, &V22294_307_57241, &P22294_306};	/*Property = FormData*/
const Value V22294_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_309 = {S28, 0, &V22294_309_57247, &P22294_307};	/*Property = Fullscreen*/
const Value V22294_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_310 = {S29, 0, &V22294_310_57250, &P22294_309};	/*Property = GeoLocation*/
const Value V22294_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_312 = {S30, 0, &V22294_312_57256, &P22294_310};	/*Property = History*/
const Value V22294_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_314 = {S31, 0, &V22294_314_57262, &P22294_312};	/*Property = Html5*/
const Value V22294_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_313 = {S32, 0, &V22294_313_57259, &P22294_314};	/*Property = Html-Media-Capture*/
const Value V22294_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_315 = {S33, 0, &V22294_315_57265, &P22294_313};	/*Property = Iframe*/
const Value V22294_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_316 = {S34, 0, &V22294_316_57268, &P22294_315};	/*Property = IndexedDB*/
const Value V22294_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_317 = {S35, 0, &V22294_317_57271, &P22294_316};	/*Property = Json*/
const Value V22294_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_15 = {S37, 0, &V22294_15_72, &P22294_317};	/*Property = LayoutEngine*/
const Value V22294_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_344 = {S38, 0, &V22294_344_57274, &P22294_15};	/*Property = Masking*/
const Value V22294_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_320 = {S39, 0, &V22294_320_57292, &P22294_344};	/*Property = PostMessage*/
const Value V22294_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_321 = {S40, 0, &V22294_321_57295, &P22294_320};	/*Property = Progress*/
const Value V22294_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_322 = {S41, 0, &V22294_322_57298, &P22294_321};	/*Property = Prompts*/
const Value V22294_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_323 = {S42, 0, &V22294_323_57301, &P22294_322};	/*Property = Selector*/
const Value V22294_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_324 = {S43, 0, &V22294_324_57307, &P22294_323};	/*Property = Svg*/
const Value V22294_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_327 = {S44, 0, &V22294_327_57316, &P22294_324};	/*Property = TouchEvents*/
const Value V22294_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_328 = {S45, 0, &V22294_328_57319, &P22294_327};	/*Property = Track*/
const Value V22294_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_329 = {S46, 0, &V22294_329_57325, &P22294_328};	/*Property = Video*/
const Value V22294_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_330 = {S47, 0, &V22294_330_57328, &P22294_329};	/*Property = Viewport*/
const Value V22294_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_332 = {S48, 0, &V22294_332_57340, &P22294_330};	/*Property = WebWorkers*/
const Value V22294_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22294_333 = {S49, 0, &V22294_333_57343, &P22294_332};	/*Property = Xhr2*/
const Profile P22294 = {"22294", &P22294_333};	/*Profile ID - 22294*/
const Value V22278_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_279 = {S1, 0, &V22278_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22278_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_282 = {S3, 0, &V22278_282_57155, &P22278_279};	/*Property = BlobBuilder*/
const Value V22278_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_350 = {S4, 0, &V22278_350_57446, &P22278_282};	/*Property = Canvas*/
const Value V22278_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_283 = {S5, 0, &V22278_283_57161, &P22278_350};	/*Property = CssBackground*/
const Value V22278_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_284 = {S6, 0, &V22278_284_57164, &P22278_283};	/*Property = CssBorderImage*/
const Value V22278_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_285 = {S7, 0, &V22278_285_57167, &P22278_284};	/*Property = CssCanvas*/
const Value V22278_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_286 = {S8, 0, &V22278_286_57170, &P22278_285};	/*Property = CssColor*/
const Value V22278_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_237 = {S9, 0, &V22278_237_55786, &P22278_286};	/*Property = CssColumn*/
const Value V22278_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_287 = {S10, 0, &V22278_287_57173, &P22278_237};	/*Property = CssFlexbox*/
const Value V22278_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_288 = {S11, 0, &V22278_288_57176, &P22278_287};	/*Property = CssFont*/
const Value V22278_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_289 = {S12, 0, &V22278_289_86899, &P22278_288};	/*Property = CssImages*/
const Value V22278_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_291 = {S13, 0, &V22278_291_57182, &P22278_289};	/*Property = CssMediaQueries*/
const Value V22278_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_292 = {S14, 0, &V22278_292_57185, &P22278_291};	/*Property = CssMinMax*/
const Value V22278_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_293 = {S15, 0, &V22278_293_57188, &P22278_292};	/*Property = CssOverflow*/
const Value V22278_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_294 = {S16, 0, &V22278_294_57191, &P22278_293};	/*Property = CssPosition*/
const Value V22278_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_295 = {S17, 0, &V22278_295_57194, &P22278_294};	/*Property = CssText*/
const Value V22278_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_297 = {S18, 0, &V22278_297_57197, &P22278_295};	/*Property = CssTransforms*/
const Value V22278_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_241 = {S19, 0, &V22278_241_55801, &P22278_297};	/*Property = CssTransitions*/
const Value V22278_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_298 = {S20, 0, &V22278_298_57208, &P22278_241};	/*Property = CssUI*/
const Value V22278_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_300 = {S21, 0, &V22278_300_57217, &P22278_298};	/*Property = DataSet*/
const Value V22278_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_301 = {S22, 0, &V22278_301_57220, &P22278_300};	/*Property = DataUrl*/
const Value V22278_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_302 = {S23, 0, &V22278_302_57223, &P22278_301};	/*Property = DeviceOrientation*/
const Value V22278_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_304 = {S24, 0, &V22278_304_57229, &P22278_302};	/*Property = FileReader*/
const Value V22278_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_305 = {S25, 0, &V22278_305_57232, &P22278_304};	/*Property = FileSaver*/
const Value V22278_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_306 = {S26, 0, &V22278_306_57238, &P22278_305};	/*Property = FileWriter*/
const Value V22278_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_307 = {S27, 0, &V22278_307_57241, &P22278_306};	/*Property = FormData*/
const Value V22278_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_309 = {S28, 0, &V22278_309_57247, &P22278_307};	/*Property = Fullscreen*/
const Value V22278_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_310 = {S29, 0, &V22278_310_57250, &P22278_309};	/*Property = GeoLocation*/
const Value V22278_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_312 = {S30, 0, &V22278_312_57256, &P22278_310};	/*Property = History*/
const Value V22278_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_314 = {S31, 0, &V22278_314_57262, &P22278_312};	/*Property = Html5*/
const Value V22278_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_313 = {S32, 0, &V22278_313_57259, &P22278_314};	/*Property = Html-Media-Capture*/
const Value V22278_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_315 = {S33, 0, &V22278_315_57265, &P22278_313};	/*Property = Iframe*/
const Value V22278_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_316 = {S34, 0, &V22278_316_57268, &P22278_315};	/*Property = IndexedDB*/
const Value V22278_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_317 = {S35, 0, &V22278_317_57271, &P22278_316};	/*Property = Json*/
const Value V22278_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_15 = {S37, 0, &V22278_15_72, &P22278_317};	/*Property = LayoutEngine*/
const Value V22278_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_344 = {S38, 0, &V22278_344_57274, &P22278_15};	/*Property = Masking*/
const Value V22278_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_320 = {S39, 0, &V22278_320_57292, &P22278_344};	/*Property = PostMessage*/
const Value V22278_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_321 = {S40, 0, &V22278_321_57295, &P22278_320};	/*Property = Progress*/
const Value V22278_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_322 = {S41, 0, &V22278_322_57298, &P22278_321};	/*Property = Prompts*/
const Value V22278_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_323 = {S42, 0, &V22278_323_57301, &P22278_322};	/*Property = Selector*/
const Value V22278_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_324 = {S43, 0, &V22278_324_57307, &P22278_323};	/*Property = Svg*/
const Value V22278_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_327 = {S44, 0, &V22278_327_57316, &P22278_324};	/*Property = TouchEvents*/
const Value V22278_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_328 = {S45, 0, &V22278_328_57319, &P22278_327};	/*Property = Track*/
const Value V22278_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_329 = {S46, 0, &V22278_329_57325, &P22278_328};	/*Property = Video*/
const Value V22278_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_330 = {S47, 0, &V22278_330_57328, &P22278_329};	/*Property = Viewport*/
const Value V22278_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_332 = {S48, 0, &V22278_332_57340, &P22278_330};	/*Property = WebWorkers*/
const Value V22278_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22278_333 = {S49, 0, &V22278_333_57343, &P22278_332};	/*Property = Xhr2*/
const Profile P22278 = {"22278", &P22278_333};	/*Profile ID - 22278*/
const Value V22279_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_279 = {S1, 0, &V22279_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22279_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_282 = {S3, 0, &V22279_282_57155, &P22279_279};	/*Property = BlobBuilder*/
const Value V22279_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_350 = {S4, 0, &V22279_350_57446, &P22279_282};	/*Property = Canvas*/
const Value V22279_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_283 = {S5, 0, &V22279_283_57161, &P22279_350};	/*Property = CssBackground*/
const Value V22279_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_284 = {S6, 0, &V22279_284_57164, &P22279_283};	/*Property = CssBorderImage*/
const Value V22279_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_285 = {S7, 0, &V22279_285_57167, &P22279_284};	/*Property = CssCanvas*/
const Value V22279_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_286 = {S8, 0, &V22279_286_57170, &P22279_285};	/*Property = CssColor*/
const Value V22279_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_237 = {S9, 0, &V22279_237_55786, &P22279_286};	/*Property = CssColumn*/
const Value V22279_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_287 = {S10, 0, &V22279_287_57173, &P22279_237};	/*Property = CssFlexbox*/
const Value V22279_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_288 = {S11, 0, &V22279_288_57176, &P22279_287};	/*Property = CssFont*/
const Value V22279_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_289 = {S12, 0, &V22279_289_86899, &P22279_288};	/*Property = CssImages*/
const Value V22279_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_291 = {S13, 0, &V22279_291_57182, &P22279_289};	/*Property = CssMediaQueries*/
const Value V22279_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_292 = {S14, 0, &V22279_292_57185, &P22279_291};	/*Property = CssMinMax*/
const Value V22279_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_293 = {S15, 0, &V22279_293_57188, &P22279_292};	/*Property = CssOverflow*/
const Value V22279_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_294 = {S16, 0, &V22279_294_57191, &P22279_293};	/*Property = CssPosition*/
const Value V22279_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_295 = {S17, 0, &V22279_295_57194, &P22279_294};	/*Property = CssText*/
const Value V22279_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_297 = {S18, 0, &V22279_297_57197, &P22279_295};	/*Property = CssTransforms*/
const Value V22279_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_241 = {S19, 0, &V22279_241_55801, &P22279_297};	/*Property = CssTransitions*/
const Value V22279_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_298 = {S20, 0, &V22279_298_57208, &P22279_241};	/*Property = CssUI*/
const Value V22279_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_300 = {S21, 0, &V22279_300_57217, &P22279_298};	/*Property = DataSet*/
const Value V22279_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_301 = {S22, 0, &V22279_301_57220, &P22279_300};	/*Property = DataUrl*/
const Value V22279_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_302 = {S23, 0, &V22279_302_57223, &P22279_301};	/*Property = DeviceOrientation*/
const Value V22279_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_304 = {S24, 0, &V22279_304_57229, &P22279_302};	/*Property = FileReader*/
const Value V22279_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_305 = {S25, 0, &V22279_305_57232, &P22279_304};	/*Property = FileSaver*/
const Value V22279_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_306 = {S26, 0, &V22279_306_57238, &P22279_305};	/*Property = FileWriter*/
const Value V22279_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_307 = {S27, 0, &V22279_307_57241, &P22279_306};	/*Property = FormData*/
const Value V22279_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_309 = {S28, 0, &V22279_309_57247, &P22279_307};	/*Property = Fullscreen*/
const Value V22279_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_310 = {S29, 0, &V22279_310_57250, &P22279_309};	/*Property = GeoLocation*/
const Value V22279_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_312 = {S30, 0, &V22279_312_57256, &P22279_310};	/*Property = History*/
const Value V22279_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_314 = {S31, 0, &V22279_314_57262, &P22279_312};	/*Property = Html5*/
const Value V22279_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_313 = {S32, 0, &V22279_313_57259, &P22279_314};	/*Property = Html-Media-Capture*/
const Value V22279_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_315 = {S33, 0, &V22279_315_57265, &P22279_313};	/*Property = Iframe*/
const Value V22279_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_316 = {S34, 0, &V22279_316_57268, &P22279_315};	/*Property = IndexedDB*/
const Value V22279_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_317 = {S35, 0, &V22279_317_57271, &P22279_316};	/*Property = Json*/
const Value V22279_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_15 = {S37, 0, &V22279_15_72, &P22279_317};	/*Property = LayoutEngine*/
const Value V22279_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_344 = {S38, 0, &V22279_344_57274, &P22279_15};	/*Property = Masking*/
const Value V22279_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_320 = {S39, 0, &V22279_320_57292, &P22279_344};	/*Property = PostMessage*/
const Value V22279_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_321 = {S40, 0, &V22279_321_57295, &P22279_320};	/*Property = Progress*/
const Value V22279_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_322 = {S41, 0, &V22279_322_57298, &P22279_321};	/*Property = Prompts*/
const Value V22279_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_323 = {S42, 0, &V22279_323_57301, &P22279_322};	/*Property = Selector*/
const Value V22279_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_324 = {S43, 0, &V22279_324_57307, &P22279_323};	/*Property = Svg*/
const Value V22279_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_327 = {S44, 0, &V22279_327_57316, &P22279_324};	/*Property = TouchEvents*/
const Value V22279_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_328 = {S45, 0, &V22279_328_57319, &P22279_327};	/*Property = Track*/
const Value V22279_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_329 = {S46, 0, &V22279_329_57325, &P22279_328};	/*Property = Video*/
const Value V22279_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_330 = {S47, 0, &V22279_330_57328, &P22279_329};	/*Property = Viewport*/
const Value V22279_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_332 = {S48, 0, &V22279_332_57340, &P22279_330};	/*Property = WebWorkers*/
const Value V22279_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22279_333 = {S49, 0, &V22279_333_57343, &P22279_332};	/*Property = Xhr2*/
const Profile P22279 = {"22279", &P22279_333};	/*Profile ID - 22279*/
const Value V22284_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_279 = {S1, 0, &V22284_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22284_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_282 = {S3, 0, &V22284_282_57155, &P22284_279};	/*Property = BlobBuilder*/
const Value V22284_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_350 = {S4, 0, &V22284_350_57446, &P22284_282};	/*Property = Canvas*/
const Value V22284_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_283 = {S5, 0, &V22284_283_57161, &P22284_350};	/*Property = CssBackground*/
const Value V22284_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_284 = {S6, 0, &V22284_284_57164, &P22284_283};	/*Property = CssBorderImage*/
const Value V22284_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_285 = {S7, 0, &V22284_285_57167, &P22284_284};	/*Property = CssCanvas*/
const Value V22284_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_286 = {S8, 0, &V22284_286_57170, &P22284_285};	/*Property = CssColor*/
const Value V22284_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_237 = {S9, 0, &V22284_237_55786, &P22284_286};	/*Property = CssColumn*/
const Value V22284_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_287 = {S10, 0, &V22284_287_57173, &P22284_237};	/*Property = CssFlexbox*/
const Value V22284_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_288 = {S11, 0, &V22284_288_57176, &P22284_287};	/*Property = CssFont*/
const Value V22284_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_289 = {S12, 0, &V22284_289_86899, &P22284_288};	/*Property = CssImages*/
const Value V22284_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_291 = {S13, 0, &V22284_291_57182, &P22284_289};	/*Property = CssMediaQueries*/
const Value V22284_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_292 = {S14, 0, &V22284_292_57185, &P22284_291};	/*Property = CssMinMax*/
const Value V22284_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_293 = {S15, 0, &V22284_293_57188, &P22284_292};	/*Property = CssOverflow*/
const Value V22284_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_294 = {S16, 0, &V22284_294_57191, &P22284_293};	/*Property = CssPosition*/
const Value V22284_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_295 = {S17, 0, &V22284_295_57194, &P22284_294};	/*Property = CssText*/
const Value V22284_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_297 = {S18, 0, &V22284_297_57197, &P22284_295};	/*Property = CssTransforms*/
const Value V22284_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_241 = {S19, 0, &V22284_241_55801, &P22284_297};	/*Property = CssTransitions*/
const Value V22284_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_298 = {S20, 0, &V22284_298_57208, &P22284_241};	/*Property = CssUI*/
const Value V22284_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_300 = {S21, 0, &V22284_300_57217, &P22284_298};	/*Property = DataSet*/
const Value V22284_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_301 = {S22, 0, &V22284_301_57220, &P22284_300};	/*Property = DataUrl*/
const Value V22284_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_302 = {S23, 0, &V22284_302_57223, &P22284_301};	/*Property = DeviceOrientation*/
const Value V22284_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_304 = {S24, 0, &V22284_304_57229, &P22284_302};	/*Property = FileReader*/
const Value V22284_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_305 = {S25, 0, &V22284_305_57232, &P22284_304};	/*Property = FileSaver*/
const Value V22284_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_306 = {S26, 0, &V22284_306_57238, &P22284_305};	/*Property = FileWriter*/
const Value V22284_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_307 = {S27, 0, &V22284_307_57241, &P22284_306};	/*Property = FormData*/
const Value V22284_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_309 = {S28, 0, &V22284_309_57247, &P22284_307};	/*Property = Fullscreen*/
const Value V22284_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_310 = {S29, 0, &V22284_310_57250, &P22284_309};	/*Property = GeoLocation*/
const Value V22284_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_312 = {S30, 0, &V22284_312_57256, &P22284_310};	/*Property = History*/
const Value V22284_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_314 = {S31, 0, &V22284_314_57262, &P22284_312};	/*Property = Html5*/
const Value V22284_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_313 = {S32, 0, &V22284_313_57259, &P22284_314};	/*Property = Html-Media-Capture*/
const Value V22284_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_315 = {S33, 0, &V22284_315_57265, &P22284_313};	/*Property = Iframe*/
const Value V22284_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_316 = {S34, 0, &V22284_316_57268, &P22284_315};	/*Property = IndexedDB*/
const Value V22284_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_317 = {S35, 0, &V22284_317_57271, &P22284_316};	/*Property = Json*/
const Value V22284_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_15 = {S37, 0, &V22284_15_72, &P22284_317};	/*Property = LayoutEngine*/
const Value V22284_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_344 = {S38, 0, &V22284_344_57274, &P22284_15};	/*Property = Masking*/
const Value V22284_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_320 = {S39, 0, &V22284_320_57292, &P22284_344};	/*Property = PostMessage*/
const Value V22284_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_321 = {S40, 0, &V22284_321_57295, &P22284_320};	/*Property = Progress*/
const Value V22284_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_322 = {S41, 0, &V22284_322_57298, &P22284_321};	/*Property = Prompts*/
const Value V22284_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_323 = {S42, 0, &V22284_323_57301, &P22284_322};	/*Property = Selector*/
const Value V22284_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_324 = {S43, 0, &V22284_324_57307, &P22284_323};	/*Property = Svg*/
const Value V22284_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_327 = {S44, 0, &V22284_327_57316, &P22284_324};	/*Property = TouchEvents*/
const Value V22284_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_328 = {S45, 0, &V22284_328_57319, &P22284_327};	/*Property = Track*/
const Value V22284_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_329 = {S46, 0, &V22284_329_57325, &P22284_328};	/*Property = Video*/
const Value V22284_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_330 = {S47, 0, &V22284_330_57328, &P22284_329};	/*Property = Viewport*/
const Value V22284_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_332 = {S48, 0, &V22284_332_57340, &P22284_330};	/*Property = WebWorkers*/
const Value V22284_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22284_333 = {S49, 0, &V22284_333_57343, &P22284_332};	/*Property = Xhr2*/
const Profile P22284 = {"22284", &P22284_333};	/*Profile ID - 22284*/
const Value V22280_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_279 = {S1, 0, &V22280_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22280_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_282 = {S3, 0, &V22280_282_57155, &P22280_279};	/*Property = BlobBuilder*/
const Value V22280_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_350 = {S4, 0, &V22280_350_57446, &P22280_282};	/*Property = Canvas*/
const Value V22280_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_283 = {S5, 0, &V22280_283_57161, &P22280_350};	/*Property = CssBackground*/
const Value V22280_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_284 = {S6, 0, &V22280_284_57164, &P22280_283};	/*Property = CssBorderImage*/
const Value V22280_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_285 = {S7, 0, &V22280_285_57167, &P22280_284};	/*Property = CssCanvas*/
const Value V22280_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_286 = {S8, 0, &V22280_286_57170, &P22280_285};	/*Property = CssColor*/
const Value V22280_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_237 = {S9, 0, &V22280_237_55786, &P22280_286};	/*Property = CssColumn*/
const Value V22280_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_287 = {S10, 0, &V22280_287_57173, &P22280_237};	/*Property = CssFlexbox*/
const Value V22280_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_288 = {S11, 0, &V22280_288_57176, &P22280_287};	/*Property = CssFont*/
const Value V22280_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_289 = {S12, 0, &V22280_289_86899, &P22280_288};	/*Property = CssImages*/
const Value V22280_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_291 = {S13, 0, &V22280_291_57182, &P22280_289};	/*Property = CssMediaQueries*/
const Value V22280_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_292 = {S14, 0, &V22280_292_57185, &P22280_291};	/*Property = CssMinMax*/
const Value V22280_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_293 = {S15, 0, &V22280_293_57188, &P22280_292};	/*Property = CssOverflow*/
const Value V22280_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_294 = {S16, 0, &V22280_294_57191, &P22280_293};	/*Property = CssPosition*/
const Value V22280_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_295 = {S17, 0, &V22280_295_57194, &P22280_294};	/*Property = CssText*/
const Value V22280_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_297 = {S18, 0, &V22280_297_57197, &P22280_295};	/*Property = CssTransforms*/
const Value V22280_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_241 = {S19, 0, &V22280_241_55801, &P22280_297};	/*Property = CssTransitions*/
const Value V22280_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_298 = {S20, 0, &V22280_298_57208, &P22280_241};	/*Property = CssUI*/
const Value V22280_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_300 = {S21, 0, &V22280_300_57217, &P22280_298};	/*Property = DataSet*/
const Value V22280_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_301 = {S22, 0, &V22280_301_57220, &P22280_300};	/*Property = DataUrl*/
const Value V22280_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_302 = {S23, 0, &V22280_302_57223, &P22280_301};	/*Property = DeviceOrientation*/
const Value V22280_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_304 = {S24, 0, &V22280_304_57229, &P22280_302};	/*Property = FileReader*/
const Value V22280_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_305 = {S25, 0, &V22280_305_57232, &P22280_304};	/*Property = FileSaver*/
const Value V22280_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_306 = {S26, 0, &V22280_306_57238, &P22280_305};	/*Property = FileWriter*/
const Value V22280_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_307 = {S27, 0, &V22280_307_57241, &P22280_306};	/*Property = FormData*/
const Value V22280_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_309 = {S28, 0, &V22280_309_57247, &P22280_307};	/*Property = Fullscreen*/
const Value V22280_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_310 = {S29, 0, &V22280_310_57250, &P22280_309};	/*Property = GeoLocation*/
const Value V22280_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_312 = {S30, 0, &V22280_312_57256, &P22280_310};	/*Property = History*/
const Value V22280_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_314 = {S31, 0, &V22280_314_57262, &P22280_312};	/*Property = Html5*/
const Value V22280_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_313 = {S32, 0, &V22280_313_57259, &P22280_314};	/*Property = Html-Media-Capture*/
const Value V22280_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_315 = {S33, 0, &V22280_315_57265, &P22280_313};	/*Property = Iframe*/
const Value V22280_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_316 = {S34, 0, &V22280_316_57268, &P22280_315};	/*Property = IndexedDB*/
const Value V22280_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_317 = {S35, 0, &V22280_317_57271, &P22280_316};	/*Property = Json*/
const Value V22280_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_15 = {S37, 0, &V22280_15_72, &P22280_317};	/*Property = LayoutEngine*/
const Value V22280_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_344 = {S38, 0, &V22280_344_57274, &P22280_15};	/*Property = Masking*/
const Value V22280_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_320 = {S39, 0, &V22280_320_57292, &P22280_344};	/*Property = PostMessage*/
const Value V22280_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_321 = {S40, 0, &V22280_321_57295, &P22280_320};	/*Property = Progress*/
const Value V22280_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_322 = {S41, 0, &V22280_322_57298, &P22280_321};	/*Property = Prompts*/
const Value V22280_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_323 = {S42, 0, &V22280_323_57301, &P22280_322};	/*Property = Selector*/
const Value V22280_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_324 = {S43, 0, &V22280_324_57307, &P22280_323};	/*Property = Svg*/
const Value V22280_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_327 = {S44, 0, &V22280_327_57316, &P22280_324};	/*Property = TouchEvents*/
const Value V22280_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_328 = {S45, 0, &V22280_328_57319, &P22280_327};	/*Property = Track*/
const Value V22280_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_329 = {S46, 0, &V22280_329_57325, &P22280_328};	/*Property = Video*/
const Value V22280_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_330 = {S47, 0, &V22280_330_57328, &P22280_329};	/*Property = Viewport*/
const Value V22280_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_332 = {S48, 0, &V22280_332_57340, &P22280_330};	/*Property = WebWorkers*/
const Value V22280_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22280_333 = {S49, 0, &V22280_333_57343, &P22280_332};	/*Property = Xhr2*/
const Profile P22280 = {"22280", &P22280_333};	/*Profile ID - 22280*/
const Value V22234_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22234_279 = {S1, 0, &V22234_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22234_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22234_282 = {S3, 0, &V22234_282_57154, &P22234_279};	/*Property = BlobBuilder*/
const Value V22234_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22234_350 = {S4, 0, &V22234_350_57444, &P22234_282};	/*Property = Canvas*/
const Value V22234_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22234_283 = {S5, 0, &V22234_283_57159, &P22234_350};	/*Property = CssBackground*/
const Value V22234_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22234_284 = {S6, 0, &V22234_284_57162, &P22234_283};	/*Property = CssBorderImage*/
const Value V22234_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22234_285 = {S7, 0, &V22234_285_57166, &P22234_284};	/*Property = CssCanvas*/
const Value V22234_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22234_286 = {S8, 0, &V22234_286_57168, &P22234_285};	/*Property = CssColor*/
const Value V22234_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22234_237 = {S9, 0, &V22234_237_55786, &P22234_286};	/*Property = CssColumn*/
const Value V22234_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22234_287 = {S10, 0, &V22234_287_57172, &P22234_237};	/*Property = CssFlexbox*/
const Value V22234_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22234_288 = {S11, 0, &V22234_288_57174, &P22234_287};	/*Property = CssFont*/
const Value V22234_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22234_289 = {S12, 0, &V22234_289_57759, &P22234_288};	/*Property = CssImages*/
const Value V22234_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22234_291 = {S13, 0, &V22234_291_57180, &P22234_289};	/*Property = CssMediaQueries*/
const Value V22234_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22234_292 = {S14, 0, &V22234_292_57183, &P22234_291};	/*Property = CssMinMax*/
const Value V22234_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22234_293 = {S15, 0, &V22234_293_57187, &P22234_292};	/*Property = CssOverflow*/
const Value V22234_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22234_294 = {S16, 0, &V22234_294_57189, &P22234_293};	/*Property = CssPosition*/
const Value V22234_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22234_295 = {S17, 0, &V22234_295_57192, &P22234_294};	/*Property = CssText*/
const Value V22234_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22234_297 = {S18, 0, &V22234_297_57195, &P22234_295};	/*Property = CssTransforms*/
const Value V22234_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22234_241 = {S19, 0, &V22234_241_55799, &P22234_297};	/*Property = CssTransitions*/
const Value V22234_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22234_298 = {S20, 0, &V22234_298_57206, &P22234_241};	/*Property = CssUI*/
const Value V22234_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22234_300 = {S21, 0, &V22234_300_57215, &P22234_298};	/*Property = DataSet*/
const Value V22234_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22234_301 = {S22, 0, &V22234_301_57218, &P22234_300};	/*Property = DataUrl*/
const Value V22234_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22234_302 = {S23, 0, &V22234_302_57222, &P22234_301};	/*Property = DeviceOrientation*/
const Value V22234_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22234_304 = {S24, 0, &V22234_304_57227, &P22234_302};	/*Property = FileReader*/
const Value V22234_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22234_305 = {S25, 0, &V22234_305_57231, &P22234_304};	/*Property = FileSaver*/
const Value V22234_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22234_306 = {S26, 0, &V22234_306_57237, &P22234_305};	/*Property = FileWriter*/
const Value V22234_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22234_307 = {S27, 0, &V22234_307_57239, &P22234_306};	/*Property = FormData*/
const Value V22234_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22234_309 = {S28, 0, &V22234_309_57246, &P22234_307};	/*Property = Fullscreen*/
const Value V22234_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22234_310 = {S29, 0, &V22234_310_57248, &P22234_309};	/*Property = GeoLocation*/
const Value V22234_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22234_312 = {S30, 0, &V22234_312_57254, &P22234_310};	/*Property = History*/
const Value V22234_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22234_314 = {S31, 0, &V22234_314_57260, &P22234_312};	/*Property = Html5*/
const Value V22234_313_57257 = {S2, NULL};	/*Value = True*/
const Property P22234_313 = {S32, 0, &V22234_313_57257, &P22234_314};	/*Property = Html-Media-Capture*/
const Value V22234_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22234_315 = {S33, 0, &V22234_315_57264, &P22234_313};	/*Property = Iframe*/
const Value V22234_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22234_316 = {S34, 0, &V22234_316_57266, &P22234_315};	/*Property = IndexedDB*/
const Value V22234_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22234_317 = {S35, 0, &V22234_317_57269, &P22234_316};	/*Property = Json*/
const Value V22234_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22234_15 = {S37, 0, &V22234_15_28, &P22234_317};	/*Property = LayoutEngine*/
const Value V22234_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22234_344 = {S38, 0, &V22234_344_57272, &P22234_15};	/*Property = Masking*/
const Value V22234_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22234_320 = {S39, 0, &V22234_320_57290, &P22234_344};	/*Property = PostMessage*/
const Value V22234_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22234_321 = {S40, 0, &V22234_321_57293, &P22234_320};	/*Property = Progress*/
const Value V22234_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22234_322 = {S41, 0, &V22234_322_57296, &P22234_321};	/*Property = Prompts*/
const Value V22234_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22234_323 = {S42, 0, &V22234_323_57299, &P22234_322};	/*Property = Selector*/
const Value V22234_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22234_324 = {S43, 0, &V22234_324_57305, &P22234_323};	/*Property = Svg*/
const Value V22234_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22234_327 = {S44, 0, &V22234_327_57314, &P22234_324};	/*Property = TouchEvents*/
const Value V22234_328_57317 = {S2, NULL};	/*Value = True*/
const Property P22234_328 = {S45, 0, &V22234_328_57317, &P22234_327};	/*Property = Track*/
const Value V22234_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22234_329 = {S46, 0, &V22234_329_57323, &P22234_328};	/*Property = Video*/
const Value V22234_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22234_330 = {S47, 0, &V22234_330_57326, &P22234_329};	/*Property = Viewport*/
const Value V22234_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22234_332 = {S48, 0, &V22234_332_57338, &P22234_330};	/*Property = WebWorkers*/
const Value V22234_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22234_333 = {S49, 0, &V22234_333_57342, &P22234_332};	/*Property = Xhr2*/
const Profile P22234 = {"22234", &P22234_333};	/*Profile ID - 22234*/
const Value V22319_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_279 = {S1, 0, &V22319_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22319_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_282 = {S3, 0, &V22319_282_57155, &P22319_279};	/*Property = BlobBuilder*/
const Value V22319_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_350 = {S4, 0, &V22319_350_57446, &P22319_282};	/*Property = Canvas*/
const Value V22319_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_283 = {S5, 0, &V22319_283_57161, &P22319_350};	/*Property = CssBackground*/
const Value V22319_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_284 = {S6, 0, &V22319_284_57164, &P22319_283};	/*Property = CssBorderImage*/
const Value V22319_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_285 = {S7, 0, &V22319_285_57167, &P22319_284};	/*Property = CssCanvas*/
const Value V22319_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_286 = {S8, 0, &V22319_286_57170, &P22319_285};	/*Property = CssColor*/
const Value V22319_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_237 = {S9, 0, &V22319_237_55786, &P22319_286};	/*Property = CssColumn*/
const Value V22319_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_287 = {S10, 0, &V22319_287_57173, &P22319_237};	/*Property = CssFlexbox*/
const Value V22319_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_288 = {S11, 0, &V22319_288_57176, &P22319_287};	/*Property = CssFont*/
const Value V22319_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_289 = {S12, 0, &V22319_289_86899, &P22319_288};	/*Property = CssImages*/
const Value V22319_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_291 = {S13, 0, &V22319_291_57182, &P22319_289};	/*Property = CssMediaQueries*/
const Value V22319_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_292 = {S14, 0, &V22319_292_57185, &P22319_291};	/*Property = CssMinMax*/
const Value V22319_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_293 = {S15, 0, &V22319_293_57188, &P22319_292};	/*Property = CssOverflow*/
const Value V22319_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_294 = {S16, 0, &V22319_294_57191, &P22319_293};	/*Property = CssPosition*/
const Value V22319_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_295 = {S17, 0, &V22319_295_57194, &P22319_294};	/*Property = CssText*/
const Value V22319_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_297 = {S18, 0, &V22319_297_57197, &P22319_295};	/*Property = CssTransforms*/
const Value V22319_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_241 = {S19, 0, &V22319_241_55801, &P22319_297};	/*Property = CssTransitions*/
const Value V22319_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_298 = {S20, 0, &V22319_298_57208, &P22319_241};	/*Property = CssUI*/
const Value V22319_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_300 = {S21, 0, &V22319_300_57217, &P22319_298};	/*Property = DataSet*/
const Value V22319_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_301 = {S22, 0, &V22319_301_57220, &P22319_300};	/*Property = DataUrl*/
const Value V22319_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_302 = {S23, 0, &V22319_302_57223, &P22319_301};	/*Property = DeviceOrientation*/
const Value V22319_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_304 = {S24, 0, &V22319_304_57229, &P22319_302};	/*Property = FileReader*/
const Value V22319_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_305 = {S25, 0, &V22319_305_57232, &P22319_304};	/*Property = FileSaver*/
const Value V22319_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_306 = {S26, 0, &V22319_306_57238, &P22319_305};	/*Property = FileWriter*/
const Value V22319_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_307 = {S27, 0, &V22319_307_57241, &P22319_306};	/*Property = FormData*/
const Value V22319_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_309 = {S28, 0, &V22319_309_57247, &P22319_307};	/*Property = Fullscreen*/
const Value V22319_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_310 = {S29, 0, &V22319_310_57250, &P22319_309};	/*Property = GeoLocation*/
const Value V22319_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_312 = {S30, 0, &V22319_312_57256, &P22319_310};	/*Property = History*/
const Value V22319_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_314 = {S31, 0, &V22319_314_57262, &P22319_312};	/*Property = Html5*/
const Value V22319_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_313 = {S32, 0, &V22319_313_57259, &P22319_314};	/*Property = Html-Media-Capture*/
const Value V22319_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_315 = {S33, 0, &V22319_315_57265, &P22319_313};	/*Property = Iframe*/
const Value V22319_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_316 = {S34, 0, &V22319_316_57268, &P22319_315};	/*Property = IndexedDB*/
const Value V22319_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_317 = {S35, 0, &V22319_317_57271, &P22319_316};	/*Property = Json*/
const Value V22319_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22319_15 = {S37, 0, &V22319_15_29, &P22319_317};	/*Property = LayoutEngine*/
const Value V22319_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_344 = {S38, 0, &V22319_344_57274, &P22319_15};	/*Property = Masking*/
const Value V22319_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_320 = {S39, 0, &V22319_320_57292, &P22319_344};	/*Property = PostMessage*/
const Value V22319_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_321 = {S40, 0, &V22319_321_57295, &P22319_320};	/*Property = Progress*/
const Value V22319_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_322 = {S41, 0, &V22319_322_57298, &P22319_321};	/*Property = Prompts*/
const Value V22319_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_323 = {S42, 0, &V22319_323_57301, &P22319_322};	/*Property = Selector*/
const Value V22319_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_324 = {S43, 0, &V22319_324_57307, &P22319_323};	/*Property = Svg*/
const Value V22319_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_327 = {S44, 0, &V22319_327_57316, &P22319_324};	/*Property = TouchEvents*/
const Value V22319_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_328 = {S45, 0, &V22319_328_57319, &P22319_327};	/*Property = Track*/
const Value V22319_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_329 = {S46, 0, &V22319_329_57325, &P22319_328};	/*Property = Video*/
const Value V22319_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_330 = {S47, 0, &V22319_330_57328, &P22319_329};	/*Property = Viewport*/
const Value V22319_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_332 = {S48, 0, &V22319_332_57340, &P22319_330};	/*Property = WebWorkers*/
const Value V22319_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22319_333 = {S49, 0, &V22319_333_57343, &P22319_332};	/*Property = Xhr2*/
const Profile P22319 = {"22319", &P22319_333};	/*Profile ID - 22319*/
const Value V23737_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_279 = {S1, 0, &V23737_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23737_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_282 = {S3, 0, &V23737_282_57155, &P23737_279};	/*Property = BlobBuilder*/
const Value V23737_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_350 = {S4, 0, &V23737_350_57446, &P23737_282};	/*Property = Canvas*/
const Value V23737_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_283 = {S5, 0, &V23737_283_57161, &P23737_350};	/*Property = CssBackground*/
const Value V23737_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_284 = {S6, 0, &V23737_284_57164, &P23737_283};	/*Property = CssBorderImage*/
const Value V23737_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_285 = {S7, 0, &V23737_285_57167, &P23737_284};	/*Property = CssCanvas*/
const Value V23737_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_286 = {S8, 0, &V23737_286_57170, &P23737_285};	/*Property = CssColor*/
const Value V23737_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_237 = {S9, 0, &V23737_237_55786, &P23737_286};	/*Property = CssColumn*/
const Value V23737_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_287 = {S10, 0, &V23737_287_57173, &P23737_237};	/*Property = CssFlexbox*/
const Value V23737_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_288 = {S11, 0, &V23737_288_57176, &P23737_287};	/*Property = CssFont*/
const Value V23737_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_289 = {S12, 0, &V23737_289_86899, &P23737_288};	/*Property = CssImages*/
const Value V23737_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_291 = {S13, 0, &V23737_291_57182, &P23737_289};	/*Property = CssMediaQueries*/
const Value V23737_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_292 = {S14, 0, &V23737_292_57185, &P23737_291};	/*Property = CssMinMax*/
const Value V23737_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_293 = {S15, 0, &V23737_293_57188, &P23737_292};	/*Property = CssOverflow*/
const Value V23737_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_294 = {S16, 0, &V23737_294_57191, &P23737_293};	/*Property = CssPosition*/
const Value V23737_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_295 = {S17, 0, &V23737_295_57194, &P23737_294};	/*Property = CssText*/
const Value V23737_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_297 = {S18, 0, &V23737_297_57197, &P23737_295};	/*Property = CssTransforms*/
const Value V23737_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_241 = {S19, 0, &V23737_241_55801, &P23737_297};	/*Property = CssTransitions*/
const Value V23737_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_298 = {S20, 0, &V23737_298_57208, &P23737_241};	/*Property = CssUI*/
const Value V23737_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_300 = {S21, 0, &V23737_300_57217, &P23737_298};	/*Property = DataSet*/
const Value V23737_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_301 = {S22, 0, &V23737_301_57220, &P23737_300};	/*Property = DataUrl*/
const Value V23737_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_302 = {S23, 0, &V23737_302_57223, &P23737_301};	/*Property = DeviceOrientation*/
const Value V23737_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_304 = {S24, 0, &V23737_304_57229, &P23737_302};	/*Property = FileReader*/
const Value V23737_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_305 = {S25, 0, &V23737_305_57232, &P23737_304};	/*Property = FileSaver*/
const Value V23737_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_306 = {S26, 0, &V23737_306_57238, &P23737_305};	/*Property = FileWriter*/
const Value V23737_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_307 = {S27, 0, &V23737_307_57241, &P23737_306};	/*Property = FormData*/
const Value V23737_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_309 = {S28, 0, &V23737_309_57247, &P23737_307};	/*Property = Fullscreen*/
const Value V23737_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_310 = {S29, 0, &V23737_310_57250, &P23737_309};	/*Property = GeoLocation*/
const Value V23737_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_312 = {S30, 0, &V23737_312_57256, &P23737_310};	/*Property = History*/
const Value V23737_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_314 = {S31, 0, &V23737_314_57262, &P23737_312};	/*Property = Html5*/
const Value V23737_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_313 = {S32, 0, &V23737_313_57259, &P23737_314};	/*Property = Html-Media-Capture*/
const Value V23737_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_315 = {S33, 0, &V23737_315_57265, &P23737_313};	/*Property = Iframe*/
const Value V23737_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_316 = {S34, 0, &V23737_316_57268, &P23737_315};	/*Property = IndexedDB*/
const Value V23737_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_317 = {S35, 0, &V23737_317_57271, &P23737_316};	/*Property = Json*/
const Value V23737_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P23737_15 = {S37, 0, &V23737_15_28, &P23737_317};	/*Property = LayoutEngine*/
const Value V23737_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_344 = {S38, 0, &V23737_344_57274, &P23737_15};	/*Property = Masking*/
const Value V23737_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_320 = {S39, 0, &V23737_320_57292, &P23737_344};	/*Property = PostMessage*/
const Value V23737_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_321 = {S40, 0, &V23737_321_57295, &P23737_320};	/*Property = Progress*/
const Value V23737_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_322 = {S41, 0, &V23737_322_57298, &P23737_321};	/*Property = Prompts*/
const Value V23737_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_323 = {S42, 0, &V23737_323_57301, &P23737_322};	/*Property = Selector*/
const Value V23737_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_324 = {S43, 0, &V23737_324_57307, &P23737_323};	/*Property = Svg*/
const Value V23737_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_327 = {S44, 0, &V23737_327_57316, &P23737_324};	/*Property = TouchEvents*/
const Value V23737_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_328 = {S45, 0, &V23737_328_57319, &P23737_327};	/*Property = Track*/
const Value V23737_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_329 = {S46, 0, &V23737_329_57325, &P23737_328};	/*Property = Video*/
const Value V23737_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_330 = {S47, 0, &V23737_330_57328, &P23737_329};	/*Property = Viewport*/
const Value V23737_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_332 = {S48, 0, &V23737_332_57340, &P23737_330};	/*Property = WebWorkers*/
const Value V23737_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23737_333 = {S49, 0, &V23737_333_57343, &P23737_332};	/*Property = Xhr2*/
const Profile P23737 = {"23737", &P23737_333};	/*Profile ID - 23737*/
const Value V22351_1_1 = {S2, NULL};	/*Value = True*/
const Property P22351_1 = {S50, 0, &V22351_1_1, NULL};	/*Property = IsMobile*/
const Value V22351_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P22351_5 = {S52, 0, &V22351_5_37, &P22351_1};	/*Property = ScreenPixelsHeight*/
const Value V22351_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P22351_4 = {S54, 0, &V22351_4_36, &P22351_5};	/*Property = ScreenPixelsWidth*/
const Profile P22351 = {"22351", &P22351_4};	/*Profile ID - 22351*/
const Value V23068_1_1 = {S2, NULL};	/*Value = True*/
const Property P23068_1 = {S50, 0, &V23068_1_1, NULL};	/*Property = IsMobile*/
const Value V23068_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23068_5 = {S52, 0, &V23068_5_1269, &P23068_1};	/*Property = ScreenPixelsHeight*/
const Value V23068_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23068_4 = {S54, 0, &V23068_4_1229, &P23068_5};	/*Property = ScreenPixelsWidth*/
const Profile P23068 = {"23068", &P23068_4};	/*Profile ID - 23068*/
const Value V22258_1_1 = {S2, NULL};	/*Value = True*/
const Property P22258_1 = {S50, 0, &V22258_1_1, NULL};	/*Property = IsMobile*/
const Value V22258_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22258_5 = {S52, 0, &V22258_5_1277, &P22258_1};	/*Property = ScreenPixelsHeight*/
const Value V22258_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22258_4 = {S54, 0, &V22258_4_2336, &P22258_5};	/*Property = ScreenPixelsWidth*/
const Profile P22258 = {"22258", &P22258_4};	/*Profile ID - 22258*/
const Value V22416_1_1 = {S2, NULL};	/*Value = True*/
const Property P22416_1 = {S50, 0, &V22416_1_1, NULL};	/*Property = IsMobile*/
const Value V22416_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22416_5 = {S52, 0, &V22416_5_37569, &P22416_1};	/*Property = ScreenPixelsHeight*/
const Value V22416_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22416_4 = {S54, 0, &V22416_4_5645, &P22416_5};	/*Property = ScreenPixelsWidth*/
const Profile P22416 = {"22416", &P22416_4};	/*Profile ID - 22416*/
const Value V22515_1_1 = {S2, NULL};	/*Value = True*/
const Property P22515_1 = {S50, 0, &V22515_1_1, NULL};	/*Property = IsMobile*/
const Value V22515_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22515_5 = {S52, 0, &V22515_5_1227, &P22515_1};	/*Property = ScreenPixelsHeight*/
const Value V22515_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22515_4 = {S54, 0, &V22515_4_1229, &P22515_5};	/*Property = ScreenPixelsWidth*/
const Profile P22515 = {"22515", &P22515_4};	/*Profile ID - 22515*/
const Value V23418_1_1 = {S2, NULL};	/*Value = True*/
const Property P23418_1 = {S50, 0, &V23418_1_1, NULL};	/*Property = IsMobile*/
const Value V23418_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23418_5 = {S52, 0, &V23418_5_1277, &P23418_1};	/*Property = ScreenPixelsHeight*/
const Value V23418_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23418_4 = {S54, 0, &V23418_4_2336, &P23418_5};	/*Property = ScreenPixelsWidth*/
const Profile P23418 = {"23418", &P23418_4};	/*Profile ID - 23418*/
const Value V23598_1_2 = {S0, NULL};	/*Value = False*/
const Property P23598_1 = {S50, 0, &V23598_1_2, NULL};	/*Property = IsMobile*/
const Value V23598_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23598_5 = {S52, 0, &V23598_5_60417, &P23598_1};	/*Property = ScreenPixelsHeight*/
const Value V23598_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23598_4 = {S54, 0, &V23598_4_57095, &P23598_5};	/*Property = ScreenPixelsWidth*/
const Profile P23598 = {"23598", &P23598_4};	/*Profile ID - 23598*/
const Value V22425_1_1 = {S2, NULL};	/*Value = True*/
const Property P22425_1 = {S50, 0, &V22425_1_1, NULL};	/*Property = IsMobile*/
const Value V22425_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22425_5 = {S52, 0, &V22425_5_1277, &P22425_1};	/*Property = ScreenPixelsHeight*/
const Value V22425_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22425_4 = {S54, 0, &V22425_4_2336, &P22425_5};	/*Property = ScreenPixelsWidth*/
const Profile P22425 = {"22425", &P22425_4};	/*Profile ID - 22425*/
const Value V20754_1_1 = {S2, NULL};	/*Value = True*/
const Property P20754_1 = {S50, 0, &V20754_1_1, NULL};	/*Property = IsMobile*/
const Value V20754_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20754_5 = {S52, 0, &V20754_5_1227, &P20754_1};	/*Property = ScreenPixelsHeight*/
const Value V20754_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20754_4 = {S54, 0, &V20754_4_1229, &P20754_5};	/*Property = ScreenPixelsWidth*/
const Profile P20754 = {"20754", &P20754_4};	/*Profile ID - 20754*/
const Value V24649_1_1 = {S2, NULL};	/*Value = True*/
const Property P24649_1 = {S50, 0, &V24649_1_1, NULL};	/*Property = IsMobile*/
const Value V24649_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24649_5 = {S52, 0, &V24649_5_1277, &P24649_1};	/*Property = ScreenPixelsHeight*/
const Value V24649_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24649_4 = {S54, 0, &V24649_4_1279, &P24649_5};	/*Property = ScreenPixelsWidth*/
const Profile P24649 = {"24649", &P24649_4};	/*Profile ID - 24649*/
const Value V23619_1_2 = {S0, NULL};	/*Value = False*/
const Property P23619_1 = {S50, 0, &V23619_1_2, NULL};	/*Property = IsMobile*/
const Value V23619_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23619_5 = {S52, 0, &V23619_5_60417, &P23619_1};	/*Property = ScreenPixelsHeight*/
const Value V23619_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23619_4 = {S54, 0, &V23619_4_57095, &P23619_5};	/*Property = ScreenPixelsWidth*/
const Profile P23619 = {"23619", &P23619_4};	/*Profile ID - 23619*/
const Value V22549_1_1 = {S2, NULL};	/*Value = True*/
const Property P22549_1 = {S50, 0, &V22549_1_1, NULL};	/*Property = IsMobile*/
const Value V22549_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22549_5 = {S52, 0, &V22549_5_1227, &P22549_1};	/*Property = ScreenPixelsHeight*/
const Value V22549_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22549_4 = {S54, 0, &V22549_4_1229, &P22549_5};	/*Property = ScreenPixelsWidth*/
const Profile P22549 = {"22549", &P22549_4};	/*Profile ID - 22549*/
const Value V22548_1_1 = {S2, NULL};	/*Value = True*/
const Property P22548_1 = {S50, 0, &V22548_1_1, NULL};	/*Property = IsMobile*/
const Value V22548_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22548_5 = {S52, 0, &V22548_5_37569, &P22548_1};	/*Property = ScreenPixelsHeight*/
const Value V22548_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22548_4 = {S54, 0, &V22548_4_5645, &P22548_5};	/*Property = ScreenPixelsWidth*/
const Profile P22548 = {"22548", &P22548_4};	/*Profile ID - 22548*/
const Value V24503_1_1 = {S2, NULL};	/*Value = True*/
const Property P24503_1 = {S50, 0, &V24503_1_1, NULL};	/*Property = IsMobile*/
const Value V24503_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24503_5 = {S52, 0, &V24503_5_1227, &P24503_1};	/*Property = ScreenPixelsHeight*/
const Value V24503_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24503_4 = {S54, 0, &V24503_4_1229, &P24503_5};	/*Property = ScreenPixelsWidth*/
const Profile P24503 = {"24503", &P24503_4};	/*Profile ID - 24503*/
const Value V23787_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_279 = {S1, 0, &V23787_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23787_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_282 = {S3, 0, &V23787_282_57155, &P23787_279};	/*Property = BlobBuilder*/
const Value V23787_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_350 = {S4, 0, &V23787_350_57446, &P23787_282};	/*Property = Canvas*/
const Value V23787_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_283 = {S5, 0, &V23787_283_57161, &P23787_350};	/*Property = CssBackground*/
const Value V23787_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_284 = {S6, 0, &V23787_284_57164, &P23787_283};	/*Property = CssBorderImage*/
const Value V23787_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_285 = {S7, 0, &V23787_285_57167, &P23787_284};	/*Property = CssCanvas*/
const Value V23787_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_286 = {S8, 0, &V23787_286_57170, &P23787_285};	/*Property = CssColor*/
const Value V23787_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_237 = {S9, 0, &V23787_237_55786, &P23787_286};	/*Property = CssColumn*/
const Value V23787_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_287 = {S10, 0, &V23787_287_57173, &P23787_237};	/*Property = CssFlexbox*/
const Value V23787_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_288 = {S11, 0, &V23787_288_57176, &P23787_287};	/*Property = CssFont*/
const Value V23787_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_289 = {S12, 0, &V23787_289_86899, &P23787_288};	/*Property = CssImages*/
const Value V23787_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_291 = {S13, 0, &V23787_291_57182, &P23787_289};	/*Property = CssMediaQueries*/
const Value V23787_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_292 = {S14, 0, &V23787_292_57185, &P23787_291};	/*Property = CssMinMax*/
const Value V23787_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_293 = {S15, 0, &V23787_293_57188, &P23787_292};	/*Property = CssOverflow*/
const Value V23787_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_294 = {S16, 0, &V23787_294_57191, &P23787_293};	/*Property = CssPosition*/
const Value V23787_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_295 = {S17, 0, &V23787_295_57194, &P23787_294};	/*Property = CssText*/
const Value V23787_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_297 = {S18, 0, &V23787_297_57197, &P23787_295};	/*Property = CssTransforms*/
const Value V23787_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_241 = {S19, 0, &V23787_241_55801, &P23787_297};	/*Property = CssTransitions*/
const Value V23787_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_298 = {S20, 0, &V23787_298_57208, &P23787_241};	/*Property = CssUI*/
const Value V23787_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_300 = {S21, 0, &V23787_300_57217, &P23787_298};	/*Property = DataSet*/
const Value V23787_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_301 = {S22, 0, &V23787_301_57220, &P23787_300};	/*Property = DataUrl*/
const Value V23787_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_302 = {S23, 0, &V23787_302_57223, &P23787_301};	/*Property = DeviceOrientation*/
const Value V23787_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_304 = {S24, 0, &V23787_304_57229, &P23787_302};	/*Property = FileReader*/
const Value V23787_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_305 = {S25, 0, &V23787_305_57232, &P23787_304};	/*Property = FileSaver*/
const Value V23787_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_306 = {S26, 0, &V23787_306_57238, &P23787_305};	/*Property = FileWriter*/
const Value V23787_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_307 = {S27, 0, &V23787_307_57241, &P23787_306};	/*Property = FormData*/
const Value V23787_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_309 = {S28, 0, &V23787_309_57247, &P23787_307};	/*Property = Fullscreen*/
const Value V23787_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_310 = {S29, 0, &V23787_310_57250, &P23787_309};	/*Property = GeoLocation*/
const Value V23787_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_312 = {S30, 0, &V23787_312_57256, &P23787_310};	/*Property = History*/
const Value V23787_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_314 = {S31, 0, &V23787_314_57262, &P23787_312};	/*Property = Html5*/
const Value V23787_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_313 = {S32, 0, &V23787_313_57259, &P23787_314};	/*Property = Html-Media-Capture*/
const Value V23787_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_315 = {S33, 0, &V23787_315_57265, &P23787_313};	/*Property = Iframe*/
const Value V23787_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_316 = {S34, 0, &V23787_316_57268, &P23787_315};	/*Property = IndexedDB*/
const Value V23787_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_317 = {S35, 0, &V23787_317_57271, &P23787_316};	/*Property = Json*/
const Value V23787_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_15 = {S37, 0, &V23787_15_72, &P23787_317};	/*Property = LayoutEngine*/
const Value V23787_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_344 = {S38, 0, &V23787_344_57274, &P23787_15};	/*Property = Masking*/
const Value V23787_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_320 = {S39, 0, &V23787_320_57292, &P23787_344};	/*Property = PostMessage*/
const Value V23787_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_321 = {S40, 0, &V23787_321_57295, &P23787_320};	/*Property = Progress*/
const Value V23787_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_322 = {S41, 0, &V23787_322_57298, &P23787_321};	/*Property = Prompts*/
const Value V23787_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_323 = {S42, 0, &V23787_323_57301, &P23787_322};	/*Property = Selector*/
const Value V23787_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_324 = {S43, 0, &V23787_324_57307, &P23787_323};	/*Property = Svg*/
const Value V23787_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_327 = {S44, 0, &V23787_327_57316, &P23787_324};	/*Property = TouchEvents*/
const Value V23787_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_328 = {S45, 0, &V23787_328_57319, &P23787_327};	/*Property = Track*/
const Value V23787_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_329 = {S46, 0, &V23787_329_57325, &P23787_328};	/*Property = Video*/
const Value V23787_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_330 = {S47, 0, &V23787_330_57328, &P23787_329};	/*Property = Viewport*/
const Value V23787_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_332 = {S48, 0, &V23787_332_57340, &P23787_330};	/*Property = WebWorkers*/
const Value V23787_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23787_333 = {S49, 0, &V23787_333_57343, &P23787_332};	/*Property = Xhr2*/
const Profile P23787 = {"23787", &P23787_333};	/*Profile ID - 23787*/
const Value V16985_1_1 = {S2, NULL};	/*Value = True*/
const Property P16985_1 = {S50, 0, &V16985_1_1, NULL};	/*Property = IsMobile*/
const Value V16985_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P16985_5 = {S52, 0, &V16985_5_1380, &P16985_1};	/*Property = ScreenPixelsHeight*/
const Value V16985_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16985_4 = {S54, 0, &V16985_4_1279, &P16985_5};	/*Property = ScreenPixelsWidth*/
const Profile P16985 = {"16985", &P16985_4};	/*Profile ID - 16985*/
const Value V22682_1_1 = {S2, NULL};	/*Value = True*/
const Property P22682_1 = {S50, 0, &V22682_1_1, NULL};	/*Property = IsMobile*/
const Value V22682_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P22682_5 = {S52, 0, &V22682_5_37, &P22682_1};	/*Property = ScreenPixelsHeight*/
const Value V22682_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P22682_4 = {S54, 0, &V22682_4_36, &P22682_5};	/*Property = ScreenPixelsWidth*/
const Profile P22682 = {"22682", &P22682_4};	/*Profile ID - 22682*/
const Value V23578_1_1 = {S2, NULL};	/*Value = True*/
const Property P23578_1 = {S50, 0, &V23578_1_1, NULL};	/*Property = IsMobile*/
const Value V23578_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23578_5 = {S52, 0, &V23578_5_1269, &P23578_1};	/*Property = ScreenPixelsHeight*/
const Value V23578_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23578_4 = {S54, 0, &V23578_4_1229, &P23578_5};	/*Property = ScreenPixelsWidth*/
const Profile P23578 = {"23578", &P23578_4};	/*Profile ID - 23578*/
const Value V22842_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_279 = {S1, 0, &V22842_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22842_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_282 = {S3, 0, &V22842_282_57155, &P22842_279};	/*Property = BlobBuilder*/
const Value V22842_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_350 = {S4, 0, &V22842_350_57446, &P22842_282};	/*Property = Canvas*/
const Value V22842_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_283 = {S5, 0, &V22842_283_57161, &P22842_350};	/*Property = CssBackground*/
const Value V22842_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_284 = {S6, 0, &V22842_284_57164, &P22842_283};	/*Property = CssBorderImage*/
const Value V22842_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_285 = {S7, 0, &V22842_285_57167, &P22842_284};	/*Property = CssCanvas*/
const Value V22842_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_286 = {S8, 0, &V22842_286_57170, &P22842_285};	/*Property = CssColor*/
const Value V22842_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_237 = {S9, 0, &V22842_237_55786, &P22842_286};	/*Property = CssColumn*/
const Value V22842_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_287 = {S10, 0, &V22842_287_57173, &P22842_237};	/*Property = CssFlexbox*/
const Value V22842_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_288 = {S11, 0, &V22842_288_57176, &P22842_287};	/*Property = CssFont*/
const Value V22842_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_289 = {S12, 0, &V22842_289_86899, &P22842_288};	/*Property = CssImages*/
const Value V22842_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_291 = {S13, 0, &V22842_291_57182, &P22842_289};	/*Property = CssMediaQueries*/
const Value V22842_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_292 = {S14, 0, &V22842_292_57185, &P22842_291};	/*Property = CssMinMax*/
const Value V22842_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_293 = {S15, 0, &V22842_293_57188, &P22842_292};	/*Property = CssOverflow*/
const Value V22842_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_294 = {S16, 0, &V22842_294_57191, &P22842_293};	/*Property = CssPosition*/
const Value V22842_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_295 = {S17, 0, &V22842_295_57194, &P22842_294};	/*Property = CssText*/
const Value V22842_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_297 = {S18, 0, &V22842_297_57197, &P22842_295};	/*Property = CssTransforms*/
const Value V22842_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_241 = {S19, 0, &V22842_241_55801, &P22842_297};	/*Property = CssTransitions*/
const Value V22842_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_298 = {S20, 0, &V22842_298_57208, &P22842_241};	/*Property = CssUI*/
const Value V22842_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_300 = {S21, 0, &V22842_300_57217, &P22842_298};	/*Property = DataSet*/
const Value V22842_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_301 = {S22, 0, &V22842_301_57220, &P22842_300};	/*Property = DataUrl*/
const Value V22842_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_302 = {S23, 0, &V22842_302_57223, &P22842_301};	/*Property = DeviceOrientation*/
const Value V22842_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_304 = {S24, 0, &V22842_304_57229, &P22842_302};	/*Property = FileReader*/
const Value V22842_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_305 = {S25, 0, &V22842_305_57232, &P22842_304};	/*Property = FileSaver*/
const Value V22842_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_306 = {S26, 0, &V22842_306_57238, &P22842_305};	/*Property = FileWriter*/
const Value V22842_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_307 = {S27, 0, &V22842_307_57241, &P22842_306};	/*Property = FormData*/
const Value V22842_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_309 = {S28, 0, &V22842_309_57247, &P22842_307};	/*Property = Fullscreen*/
const Value V22842_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_310 = {S29, 0, &V22842_310_57250, &P22842_309};	/*Property = GeoLocation*/
const Value V22842_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_312 = {S30, 0, &V22842_312_57256, &P22842_310};	/*Property = History*/
const Value V22842_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_314 = {S31, 0, &V22842_314_57262, &P22842_312};	/*Property = Html5*/
const Value V22842_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_313 = {S32, 0, &V22842_313_57259, &P22842_314};	/*Property = Html-Media-Capture*/
const Value V22842_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_315 = {S33, 0, &V22842_315_57265, &P22842_313};	/*Property = Iframe*/
const Value V22842_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_316 = {S34, 0, &V22842_316_57268, &P22842_315};	/*Property = IndexedDB*/
const Value V22842_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_317 = {S35, 0, &V22842_317_57271, &P22842_316};	/*Property = Json*/
const Value V22842_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_15 = {S37, 0, &V22842_15_72, &P22842_317};	/*Property = LayoutEngine*/
const Value V22842_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_344 = {S38, 0, &V22842_344_57274, &P22842_15};	/*Property = Masking*/
const Value V22842_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_320 = {S39, 0, &V22842_320_57292, &P22842_344};	/*Property = PostMessage*/
const Value V22842_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_321 = {S40, 0, &V22842_321_57295, &P22842_320};	/*Property = Progress*/
const Value V22842_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_322 = {S41, 0, &V22842_322_57298, &P22842_321};	/*Property = Prompts*/
const Value V22842_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_323 = {S42, 0, &V22842_323_57301, &P22842_322};	/*Property = Selector*/
const Value V22842_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_324 = {S43, 0, &V22842_324_57307, &P22842_323};	/*Property = Svg*/
const Value V22842_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_327 = {S44, 0, &V22842_327_57316, &P22842_324};	/*Property = TouchEvents*/
const Value V22842_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_328 = {S45, 0, &V22842_328_57319, &P22842_327};	/*Property = Track*/
const Value V22842_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_329 = {S46, 0, &V22842_329_57325, &P22842_328};	/*Property = Video*/
const Value V22842_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_330 = {S47, 0, &V22842_330_57328, &P22842_329};	/*Property = Viewport*/
const Value V22842_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_332 = {S48, 0, &V22842_332_57340, &P22842_330};	/*Property = WebWorkers*/
const Value V22842_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22842_333 = {S49, 0, &V22842_333_57343, &P22842_332};	/*Property = Xhr2*/
const Profile P22842 = {"22842", &P22842_333};	/*Profile ID - 22842*/
const Value V23870_1_1 = {S2, NULL};	/*Value = True*/
const Property P23870_1 = {S50, 0, &V23870_1_1, NULL};	/*Property = IsMobile*/
const Value V23870_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23870_5 = {S52, 0, &V23870_5_1227, &P23870_1};	/*Property = ScreenPixelsHeight*/
const Value V23870_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23870_4 = {S54, 0, &V23870_4_1229, &P23870_5};	/*Property = ScreenPixelsWidth*/
const Profile P23870 = {"23870", &P23870_4};	/*Profile ID - 23870*/
const Value V23767_1_1 = {S2, NULL};	/*Value = True*/
const Property P23767_1 = {S50, 0, &V23767_1_1, NULL};	/*Property = IsMobile*/
const Value V23767_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23767_5 = {S52, 0, &V23767_5_1832, &P23767_1};	/*Property = ScreenPixelsHeight*/
const Value V23767_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23767_4 = {S54, 0, &V23767_4_5645, &P23767_5};	/*Property = ScreenPixelsWidth*/
const Profile P23767 = {"23767", &P23767_4};	/*Profile ID - 23767*/
const Value V22981_1_1 = {S2, NULL};	/*Value = True*/
const Property P22981_1 = {S50, 0, &V22981_1_1, NULL};	/*Property = IsMobile*/
const Value V22981_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P22981_5 = {S52, 0, &V22981_5_1380, &P22981_1};	/*Property = ScreenPixelsHeight*/
const Value V22981_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22981_4 = {S54, 0, &V22981_4_1279, &P22981_5};	/*Property = ScreenPixelsWidth*/
const Profile P22981 = {"22981", &P22981_4};	/*Profile ID - 22981*/
const Value V23457_1_1 = {S2, NULL};	/*Value = True*/
const Property P23457_1 = {S50, 0, &V23457_1_1, NULL};	/*Property = IsMobile*/
const Value V23457_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23457_5 = {S52, 0, &V23457_5_1269, &P23457_1};	/*Property = ScreenPixelsHeight*/
const Value V23457_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23457_4 = {S54, 0, &V23457_4_1229, &P23457_5};	/*Property = ScreenPixelsWidth*/
const Profile P23457 = {"23457", &P23457_4};	/*Profile ID - 23457*/
const Value V24581_1_1 = {S2, NULL};	/*Value = True*/
const Property P24581_1 = {S50, 0, &V24581_1_1, NULL};	/*Property = IsMobile*/
const Value V24581_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P24581_5 = {S52, 0, &V24581_5_1458, &P24581_1};	/*Property = ScreenPixelsHeight*/
const Value V24581_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P24581_4 = {S54, 0, &V24581_4_1460, &P24581_5};	/*Property = ScreenPixelsWidth*/
const Profile P24581 = {"24581", &P24581_4};	/*Profile ID - 24581*/
const Value V22674_1_1 = {S2, NULL};	/*Value = True*/
const Property P22674_1 = {S50, 0, &V22674_1_1, NULL};	/*Property = IsMobile*/
const Value V22674_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22674_5 = {S52, 0, &V22674_5_1227, &P22674_1};	/*Property = ScreenPixelsHeight*/
const Value V22674_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22674_4 = {S54, 0, &V22674_4_1460, &P22674_5};	/*Property = ScreenPixelsWidth*/
const Profile P22674 = {"22674", &P22674_4};	/*Profile ID - 22674*/
const Value V24568_1_1 = {S2, NULL};	/*Value = True*/
const Property P24568_1 = {S50, 0, &V24568_1_1, NULL};	/*Property = IsMobile*/
const Value V24568_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24568_5 = {S52, 0, &V24568_5_9981, &P24568_1};	/*Property = ScreenPixelsHeight*/
const Value V24568_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24568_4 = {S54, 0, &V24568_4_39098, &P24568_5};	/*Property = ScreenPixelsWidth*/
const Profile P24568 = {"24568", &P24568_4};	/*Profile ID - 24568*/
const Value V23639_1_2 = {S0, NULL};	/*Value = False*/
const Property P23639_1 = {S50, 0, &V23639_1_2, NULL};	/*Property = IsMobile*/
const Value V23639_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23639_5 = {S52, 0, &V23639_5_37, &P23639_1};	/*Property = ScreenPixelsHeight*/
const Value V23639_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23639_4 = {S54, 0, &V23639_4_36, &P23639_5};	/*Property = ScreenPixelsWidth*/
const Profile P23639 = {"23639", &P23639_4};	/*Profile ID - 23639*/
const Value V23439_1_1 = {S2, NULL};	/*Value = True*/
const Property P23439_1 = {S50, 0, &V23439_1_1, NULL};	/*Property = IsMobile*/
const Value V23439_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23439_5 = {S52, 0, &V23439_5_1269, &P23439_1};	/*Property = ScreenPixelsHeight*/
const Value V23439_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23439_4 = {S54, 0, &V23439_4_1229, &P23439_5};	/*Property = ScreenPixelsWidth*/
const Profile P23439 = {"23439", &P23439_4};	/*Profile ID - 23439*/
const Value V22661_1_1 = {S2, NULL};	/*Value = True*/
const Property P22661_1 = {S50, 0, &V22661_1_1, NULL};	/*Property = IsMobile*/
const Value V22661_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22661_5 = {S52, 0, &V22661_5_1458, &P22661_1};	/*Property = ScreenPixelsHeight*/
const Value V22661_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P22661_4 = {S54, 0, &V22661_4_37617, &P22661_5};	/*Property = ScreenPixelsWidth*/
const Profile P22661 = {"22661", &P22661_4};	/*Profile ID - 22661*/
const Value V22639_1_1 = {S2, NULL};	/*Value = True*/
const Property P22639_1 = {S50, 0, &V22639_1_1, NULL};	/*Property = IsMobile*/
const Value V22639_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22639_5 = {S52, 0, &V22639_5_1277, &P22639_1};	/*Property = ScreenPixelsHeight*/
const Value V22639_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22639_4 = {S54, 0, &V22639_4_1279, &P22639_5};	/*Property = ScreenPixelsWidth*/
const Profile P22639 = {"22639", &P22639_4};	/*Profile ID - 22639*/
const Value V22397_1_1 = {S2, NULL};	/*Value = True*/
const Property P22397_1 = {S50, 0, &V22397_1_1, NULL};	/*Property = IsMobile*/
const Value V22397_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22397_5 = {S52, 0, &V22397_5_1832, &P22397_1};	/*Property = ScreenPixelsHeight*/
const Value V22397_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P22397_4 = {S54, 0, &V22397_4_1278, &P22397_5};	/*Property = ScreenPixelsWidth*/
const Profile P22397 = {"22397", &P22397_4};	/*Profile ID - 22397*/
const Value V22651_1_1 = {S2, NULL};	/*Value = True*/
const Property P22651_1 = {S50, 0, &V22651_1_1, NULL};	/*Property = IsMobile*/
const Value V22651_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22651_5 = {S52, 0, &V22651_5_1222, &P22651_1};	/*Property = ScreenPixelsHeight*/
const Value V22651_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22651_4 = {S54, 0, &V22651_4_1224, &P22651_5};	/*Property = ScreenPixelsWidth*/
const Profile P22651 = {"22651", &P22651_4};	/*Profile ID - 22651*/
const Value V22958_1_1 = {S2, NULL};	/*Value = True*/
const Property P22958_1 = {S50, 0, &V22958_1_1, NULL};	/*Property = IsMobile*/
const Value V22958_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22958_5 = {S52, 0, &V22958_5_9981, &P22958_1};	/*Property = ScreenPixelsHeight*/
const Value V22958_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22958_4 = {S54, 0, &V22958_4_39098, &P22958_5};	/*Property = ScreenPixelsWidth*/
const Profile P22958 = {"22958", &P22958_4};	/*Profile ID - 22958*/
const Value V22680_1_1 = {S2, NULL};	/*Value = True*/
const Property P22680_1 = {S50, 0, &V22680_1_1, NULL};	/*Property = IsMobile*/
const Value V22680_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22680_5 = {S52, 0, &V22680_5_1458, &P22680_1};	/*Property = ScreenPixelsHeight*/
const Value V22680_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22680_4 = {S54, 0, &V22680_4_1460, &P22680_5};	/*Property = ScreenPixelsWidth*/
const Profile P22680 = {"22680", &P22680_4};	/*Profile ID - 22680*/
const Value V23465_1_1 = {S2, NULL};	/*Value = True*/
const Property P23465_1 = {S50, 0, &V23465_1_1, NULL};	/*Property = IsMobile*/
const Value V23465_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23465_5 = {S52, 0, &V23465_5_1832, &P23465_1};	/*Property = ScreenPixelsHeight*/
const Value V23465_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23465_4 = {S54, 0, &V23465_4_2336, &P23465_5};	/*Property = ScreenPixelsWidth*/
const Profile P23465 = {"23465", &P23465_4};	/*Profile ID - 23465*/
const Value V23335_1_1 = {S2, NULL};	/*Value = True*/
const Property P23335_1 = {S50, 0, &V23335_1_1, NULL};	/*Property = IsMobile*/
const Value V23335_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23335_5 = {S52, 0, &V23335_5_1832, &P23335_1};	/*Property = ScreenPixelsHeight*/
const Value V23335_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23335_4 = {S54, 0, &V23335_4_5645, &P23335_5};	/*Property = ScreenPixelsWidth*/
const Profile P23335 = {"23335", &P23335_4};	/*Profile ID - 23335*/
const Value V22609_1_1 = {S2, NULL};	/*Value = True*/
const Property P22609_1 = {S50, 0, &V22609_1_1, NULL};	/*Property = IsMobile*/
const Value V22609_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22609_5 = {S52, 0, &V22609_5_1277, &P22609_1};	/*Property = ScreenPixelsHeight*/
const Value V22609_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22609_4 = {S54, 0, &V22609_4_1279, &P22609_5};	/*Property = ScreenPixelsWidth*/
const Profile P22609 = {"22609", &P22609_4};	/*Profile ID - 22609*/
const Value V23531_1_1 = {S2, NULL};	/*Value = True*/
const Property P23531_1 = {S50, 0, &V23531_1_1, NULL};	/*Property = IsMobile*/
const Value V23531_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23531_5 = {S52, 0, &V23531_5_1227, &P23531_1};	/*Property = ScreenPixelsHeight*/
const Value V23531_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23531_4 = {S54, 0, &V23531_4_1229, &P23531_5};	/*Property = ScreenPixelsWidth*/
const Profile P23531 = {"23531", &P23531_4};	/*Profile ID - 23531*/
const Value V22994_1_1 = {S2, NULL};	/*Value = True*/
const Property P22994_1 = {S50, 0, &V22994_1_1, NULL};	/*Property = IsMobile*/
const Value V22994_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22994_5 = {S52, 0, &V22994_5_1277, &P22994_1};	/*Property = ScreenPixelsHeight*/
const Value V22994_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22994_4 = {S54, 0, &V22994_4_1279, &P22994_5};	/*Property = ScreenPixelsWidth*/
const Profile P22994 = {"22994", &P22994_4};	/*Profile ID - 22994*/
const Value V23004_1_1 = {S2, NULL};	/*Value = True*/
const Property P23004_1 = {S50, 0, &V23004_1_1, NULL};	/*Property = IsMobile*/
const Value V23004_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23004_5 = {S52, 0, &V23004_5_1277, &P23004_1};	/*Property = ScreenPixelsHeight*/
const Value V23004_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23004_4 = {S54, 0, &V23004_4_1279, &P23004_5};	/*Property = ScreenPixelsWidth*/
const Profile P23004 = {"23004", &P23004_4};	/*Profile ID - 23004*/
const Value V23304_1_1 = {S2, NULL};	/*Value = True*/
const Property P23304_1 = {S50, 0, &V23304_1_1, NULL};	/*Property = IsMobile*/
const Value V23304_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23304_5 = {S52, 0, &V23304_5_1227, &P23304_1};	/*Property = ScreenPixelsHeight*/
const Value V23304_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23304_4 = {S54, 0, &V23304_4_1229, &P23304_5};	/*Property = ScreenPixelsWidth*/
const Profile P23304 = {"23304", &P23304_4};	/*Profile ID - 23304*/
const Value V21741_1_1 = {S2, NULL};	/*Value = True*/
const Property P21741_1 = {S50, 0, &V21741_1_1, NULL};	/*Property = IsMobile*/
const Value V21741_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P21741_5 = {S52, 0, &V21741_5_1222, &P21741_1};	/*Property = ScreenPixelsHeight*/
const Value V21741_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P21741_4 = {S54, 0, &V21741_4_1224, &P21741_5};	/*Property = ScreenPixelsWidth*/
const Profile P21741 = {"21741", &P21741_4};	/*Profile ID - 21741*/
const Value V23199_1_1 = {S2, NULL};	/*Value = True*/
const Property P23199_1 = {S50, 0, &V23199_1_1, NULL};	/*Property = IsMobile*/
const Value V23199_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23199_5 = {S52, 0, &V23199_5_1403, &P23199_1};	/*Property = ScreenPixelsHeight*/
const Value V23199_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23199_4 = {S54, 0, &V23199_4_1404, &P23199_5};	/*Property = ScreenPixelsWidth*/
const Profile P23199 = {"23199", &P23199_4};	/*Profile ID - 23199*/
const Value V23171_1_1 = {S2, NULL};	/*Value = True*/
const Property P23171_1 = {S50, 0, &V23171_1_1, NULL};	/*Property = IsMobile*/
const Value V23171_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23171_5 = {S52, 0, &V23171_5_1403, &P23171_1};	/*Property = ScreenPixelsHeight*/
const Value V23171_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23171_4 = {S54, 0, &V23171_4_1404, &P23171_5};	/*Property = ScreenPixelsWidth*/
const Profile P23171 = {"23171", &P23171_4};	/*Profile ID - 23171*/
const Value V22256_1_1 = {S2, NULL};	/*Value = True*/
const Property P22256_1 = {S50, 0, &V22256_1_1, NULL};	/*Property = IsMobile*/
const Value V22256_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P22256_5 = {S52, 0, &V22256_5_37569, &P22256_1};	/*Property = ScreenPixelsHeight*/
const Value V22256_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22256_4 = {S54, 0, &V22256_4_5645, &P22256_5};	/*Property = ScreenPixelsWidth*/
const Profile P22256 = {"22256", &P22256_4};	/*Profile ID - 22256*/
const Value V23514_1_1 = {S2, NULL};	/*Value = True*/
const Property P23514_1 = {S50, 0, &V23514_1_1, NULL};	/*Property = IsMobile*/
const Value V23514_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23514_5 = {S52, 0, &V23514_5_9981, &P23514_1};	/*Property = ScreenPixelsHeight*/
const Value V23514_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23514_4 = {S54, 0, &V23514_4_39098, &P23514_5};	/*Property = ScreenPixelsWidth*/
const Profile P23514 = {"23514", &P23514_4};	/*Profile ID - 23514*/
const Profile P22274 = {"22274", NULL};	/*Profile ID - 22274*/
const Value V21189_1_1 = {S2, NULL};	/*Value = True*/
const Property P21189_1 = {S50, 0, &V21189_1_1, NULL};	/*Property = IsMobile*/
const Value V21189_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21189_5 = {S52, 0, &V21189_5_37, &P21189_1};	/*Property = ScreenPixelsHeight*/
const Value V21189_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21189_4 = {S54, 0, &V21189_4_36, &P21189_5};	/*Property = ScreenPixelsWidth*/
const Profile P21189 = {"21189", &P21189_4};	/*Profile ID - 21189*/
const Value V23295_1_1 = {S2, NULL};	/*Value = True*/
const Property P23295_1 = {S50, 0, &V23295_1_1, NULL};	/*Property = IsMobile*/
const Value V23295_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23295_5 = {S52, 0, &V23295_5_1227, &P23295_1};	/*Property = ScreenPixelsHeight*/
const Value V23295_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23295_4 = {S54, 0, &V23295_4_1229, &P23295_5};	/*Property = ScreenPixelsWidth*/
const Profile P23295 = {"23295", &P23295_4};	/*Profile ID - 23295*/
const Value V22792_1_1 = {S2, NULL};	/*Value = True*/
const Property P22792_1 = {S50, 0, &V22792_1_1, NULL};	/*Property = IsMobile*/
const Value V22792_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22792_5 = {S52, 0, &V22792_5_1277, &P22792_1};	/*Property = ScreenPixelsHeight*/
const Value V22792_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22792_4 = {S54, 0, &V22792_4_1279, &P22792_5};	/*Property = ScreenPixelsWidth*/
const Profile P22792 = {"22792", &P22792_4};	/*Profile ID - 22792*/
const Value V24512_1_1 = {S2, NULL};	/*Value = True*/
const Property P24512_1 = {S50, 0, &V24512_1_1, NULL};	/*Property = IsMobile*/
const Value V24512_5_90456 = {S145, NULL};	/*Value = 1536*/
const Property P24512_5 = {S52, 0, &V24512_5_90456, &P24512_1};	/*Property = ScreenPixelsHeight*/
const Value V24512_4_90457 = {S144, NULL};	/*Value = 2048*/
const Property P24512_4 = {S54, 0, &V24512_4_90457, &P24512_5};	/*Property = ScreenPixelsWidth*/
const Profile P24512 = {"24512", &P24512_4};	/*Profile ID - 24512*/
const Value V21318_1_1 = {S2, NULL};	/*Value = True*/
const Property P21318_1 = {S50, 0, &V21318_1_1, NULL};	/*Property = IsMobile*/
const Value V21318_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21318_5 = {S52, 0, &V21318_5_1227, &P21318_1};	/*Property = ScreenPixelsHeight*/
const Value V21318_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P21318_4 = {S54, 0, &V21318_4_1460, &P21318_5};	/*Property = ScreenPixelsWidth*/
const Profile P21318 = {"21318", &P21318_4};	/*Profile ID - 21318*/
const Value V23836_1_1 = {S2, NULL};	/*Value = True*/
const Property P23836_1 = {S50, 0, &V23836_1_1, NULL};	/*Property = IsMobile*/
const Value V23836_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P23836_5 = {S52, 0, &V23836_5_59138, &P23836_1};	/*Property = ScreenPixelsHeight*/
const Value V23836_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P23836_4 = {S54, 0, &V23836_4_59970, &P23836_5};	/*Property = ScreenPixelsWidth*/
const Profile P23836 = {"23836", &P23836_4};	/*Profile ID - 23836*/
const Value V23241_1_1 = {S2, NULL};	/*Value = True*/
const Property P23241_1 = {S50, 0, &V23241_1_1, NULL};	/*Property = IsMobile*/
const Value V23241_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23241_5 = {S52, 0, &V23241_5_1403, &P23241_1};	/*Property = ScreenPixelsHeight*/
const Value V23241_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23241_4 = {S54, 0, &V23241_4_1404, &P23241_5};	/*Property = ScreenPixelsWidth*/
const Profile P23241 = {"23241", &P23241_4};	/*Profile ID - 23241*/
const Value V23830_1_1 = {S2, NULL};	/*Value = True*/
const Property P23830_1 = {S50, 0, &V23830_1_1, NULL};	/*Property = IsMobile*/
const Value V23830_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23830_5 = {S52, 0, &V23830_5_1227, &P23830_1};	/*Property = ScreenPixelsHeight*/
const Value V23830_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23830_4 = {S54, 0, &V23830_4_1229, &P23830_5};	/*Property = ScreenPixelsWidth*/
const Profile P23830 = {"23830", &P23830_4};	/*Profile ID - 23830*/
const Value V22606_1_1 = {S2, NULL};	/*Value = True*/
const Property P22606_1 = {S50, 0, &V22606_1_1, NULL};	/*Property = IsMobile*/
const Value V22606_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22606_5 = {S52, 0, &V22606_5_9981, &P22606_1};	/*Property = ScreenPixelsHeight*/
const Value V22606_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22606_4 = {S54, 0, &V22606_4_2336, &P22606_5};	/*Property = ScreenPixelsWidth*/
const Profile P22606 = {"22606", &P22606_4};	/*Profile ID - 22606*/
const Value V23299_1_1 = {S2, NULL};	/*Value = True*/
const Property P23299_1 = {S50, 0, &V23299_1_1, NULL};	/*Property = IsMobile*/
const Value V23299_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23299_5 = {S52, 0, &V23299_5_1227, &P23299_1};	/*Property = ScreenPixelsHeight*/
const Value V23299_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23299_4 = {S54, 0, &V23299_4_1229, &P23299_5};	/*Property = ScreenPixelsWidth*/
const Profile P23299 = {"23299", &P23299_4};	/*Profile ID - 23299*/
const Value V23774_1_1 = {S2, NULL};	/*Value = True*/
const Property P23774_1 = {S50, 0, &V23774_1_1, NULL};	/*Property = IsMobile*/
const Value V23774_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23774_5 = {S52, 0, &V23774_5_1277, &P23774_1};	/*Property = ScreenPixelsHeight*/
const Value V23774_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23774_4 = {S54, 0, &V23774_4_1279, &P23774_5};	/*Property = ScreenPixelsWidth*/
const Profile P23774 = {"23774", &P23774_4};	/*Profile ID - 23774*/
const Value V22758_1_1 = {S2, NULL};	/*Value = True*/
const Property P22758_1 = {S50, 0, &V22758_1_1, NULL};	/*Property = IsMobile*/
const Value V22758_5_83469 = {S148, NULL};	/*Value = 853*/
const Property P22758_5 = {S52, 0, &V22758_5_83469, &P22758_1};	/*Property = ScreenPixelsHeight*/
const Value V22758_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22758_4 = {S54, 0, &V22758_4_1229, &P22758_5};	/*Property = ScreenPixelsWidth*/
const Profile P22758 = {"22758", &P22758_4};	/*Profile ID - 22758*/
const Value V22668_1_1 = {S2, NULL};	/*Value = True*/
const Property P22668_1 = {S50, 0, &V22668_1_1, NULL};	/*Property = IsMobile*/
const Value V22668_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22668_5 = {S52, 0, &V22668_5_9981, &P22668_1};	/*Property = ScreenPixelsHeight*/
const Value V22668_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22668_4 = {S54, 0, &V22668_4_39098, &P22668_5};	/*Property = ScreenPixelsWidth*/
const Profile P22668 = {"22668", &P22668_4};	/*Profile ID - 22668*/
const Value V23370_1_1 = {S2, NULL};	/*Value = True*/
const Property P23370_1 = {S50, 0, &V23370_1_1, NULL};	/*Property = IsMobile*/
const Value V23370_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23370_5 = {S52, 0, &V23370_5_1458, &P23370_1};	/*Property = ScreenPixelsHeight*/
const Value V23370_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23370_4 = {S54, 0, &V23370_4_37617, &P23370_5};	/*Property = ScreenPixelsWidth*/
const Profile P23370 = {"23370", &P23370_4};	/*Profile ID - 23370*/
const Value V23834_1_1 = {S2, NULL};	/*Value = True*/
const Property P23834_1 = {S50, 0, &V23834_1_1, NULL};	/*Property = IsMobile*/
const Value V23834_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23834_5 = {S52, 0, &V23834_5_1227, &P23834_1};	/*Property = ScreenPixelsHeight*/
const Value V23834_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23834_4 = {S54, 0, &V23834_4_1229, &P23834_5};	/*Property = ScreenPixelsWidth*/
const Profile P23834 = {"23834", &P23834_4};	/*Profile ID - 23834*/
const Value V23120_1_1 = {S2, NULL};	/*Value = True*/
const Property P23120_1 = {S50, 0, &V23120_1_1, NULL};	/*Property = IsMobile*/
const Value V23120_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23120_5 = {S52, 0, &V23120_5_1227, &P23120_1};	/*Property = ScreenPixelsHeight*/
const Value V23120_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23120_4 = {S54, 0, &V23120_4_1229, &P23120_5};	/*Property = ScreenPixelsWidth*/
const Profile P23120 = {"23120", &P23120_4};	/*Profile ID - 23120*/
const Value V22642_1_1 = {S2, NULL};	/*Value = True*/
const Property P22642_1 = {S50, 0, &V22642_1_1, NULL};	/*Property = IsMobile*/
const Value V22642_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P22642_5 = {S52, 0, &V22642_5_37, &P22642_1};	/*Property = ScreenPixelsHeight*/
const Value V22642_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P22642_4 = {S54, 0, &V22642_4_36, &P22642_5};	/*Property = ScreenPixelsWidth*/
const Profile P22642 = {"22642", &P22642_4};	/*Profile ID - 22642*/
const Value V23217_1_1 = {S2, NULL};	/*Value = True*/
const Property P23217_1 = {S50, 0, &V23217_1_1, NULL};	/*Property = IsMobile*/
const Value V23217_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23217_5 = {S52, 0, &V23217_5_1277, &P23217_1};	/*Property = ScreenPixelsHeight*/
const Value V23217_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23217_4 = {S54, 0, &V23217_4_2336, &P23217_5};	/*Property = ScreenPixelsWidth*/
const Profile P23217 = {"23217", &P23217_4};	/*Profile ID - 23217*/
const Value V22704_1_1 = {S2, NULL};	/*Value = True*/
const Property P22704_1 = {S50, 0, &V22704_1_1, NULL};	/*Property = IsMobile*/
const Value V22704_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P22704_5 = {S52, 0, &V22704_5_1269, &P22704_1};	/*Property = ScreenPixelsHeight*/
const Value V22704_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22704_4 = {S54, 0, &V22704_4_1229, &P22704_5};	/*Property = ScreenPixelsWidth*/
const Profile P22704 = {"22704", &P22704_4};	/*Profile ID - 22704*/
const Value V22648_1_1 = {S2, NULL};	/*Value = True*/
const Property P22648_1 = {S50, 0, &V22648_1_1, NULL};	/*Property = IsMobile*/
const Value V22648_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P22648_5 = {S52, 0, &V22648_5_37, &P22648_1};	/*Property = ScreenPixelsHeight*/
const Value V22648_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P22648_4 = {S54, 0, &V22648_4_36, &P22648_5};	/*Property = ScreenPixelsWidth*/
const Profile P22648 = {"22648", &P22648_4};	/*Profile ID - 22648*/
const Value V22607_1_1 = {S2, NULL};	/*Value = True*/
const Property P22607_1 = {S50, 0, &V22607_1_1, NULL};	/*Property = IsMobile*/
const Value V22607_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22607_5 = {S52, 0, &V22607_5_1222, &P22607_1};	/*Property = ScreenPixelsHeight*/
const Value V22607_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22607_4 = {S54, 0, &V22607_4_1224, &P22607_5};	/*Property = ScreenPixelsWidth*/
const Profile P22607 = {"22607", &P22607_4};	/*Profile ID - 22607*/
const Value V21190_1_1 = {S2, NULL};	/*Value = True*/
const Property P21190_1 = {S50, 0, &V21190_1_1, NULL};	/*Property = IsMobile*/
const Value V21190_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21190_5 = {S52, 0, &V21190_5_37, &P21190_1};	/*Property = ScreenPixelsHeight*/
const Value V21190_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21190_4 = {S54, 0, &V21190_4_36, &P21190_5};	/*Property = ScreenPixelsWidth*/
const Profile P21190 = {"21190", &P21190_4};	/*Profile ID - 21190*/
const Value V23170_1_1 = {S2, NULL};	/*Value = True*/
const Property P23170_1 = {S50, 0, &V23170_1_1, NULL};	/*Property = IsMobile*/
const Value V23170_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23170_5 = {S52, 0, &V23170_5_1269, &P23170_1};	/*Property = ScreenPixelsHeight*/
const Value V23170_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23170_4 = {S54, 0, &V23170_4_1229, &P23170_5};	/*Property = ScreenPixelsWidth*/
const Profile P23170 = {"23170", &P23170_4};	/*Profile ID - 23170*/
const Value V22649_1_1 = {S2, NULL};	/*Value = True*/
const Property P22649_1 = {S50, 0, &V22649_1_1, NULL};	/*Property = IsMobile*/
const Value V22649_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22649_5 = {S52, 0, &V22649_5_1403, &P22649_1};	/*Property = ScreenPixelsHeight*/
const Value V22649_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22649_4 = {S54, 0, &V22649_4_1404, &P22649_5};	/*Property = ScreenPixelsWidth*/
const Profile P22649 = {"22649", &P22649_4};	/*Profile ID - 22649*/
const Value V23720_1_1 = {S2, NULL};	/*Value = True*/
const Property P23720_1 = {S50, 0, &V23720_1_1, NULL};	/*Property = IsMobile*/
const Value V23720_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23720_5 = {S52, 0, &V23720_5_37, &P23720_1};	/*Property = ScreenPixelsHeight*/
const Value V23720_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23720_4 = {S54, 0, &V23720_4_36, &P23720_5};	/*Property = ScreenPixelsWidth*/
const Profile P23720 = {"23720", &P23720_4};	/*Profile ID - 23720*/
const Value V21004_1_1 = {S2, NULL};	/*Value = True*/
const Property P21004_1 = {S50, 0, &V21004_1_1, NULL};	/*Property = IsMobile*/
const Value V21004_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21004_5 = {S52, 0, &V21004_5_1227, &P21004_1};	/*Property = ScreenPixelsHeight*/
const Value V21004_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21004_4 = {S54, 0, &V21004_4_1229, &P21004_5};	/*Property = ScreenPixelsWidth*/
const Profile P21004 = {"21004", &P21004_4};	/*Profile ID - 21004*/
const Value V22689_1_1 = {S2, NULL};	/*Value = True*/
const Property P22689_1 = {S50, 0, &V22689_1_1, NULL};	/*Property = IsMobile*/
const Value V22689_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22689_5 = {S52, 0, &V22689_5_1403, &P22689_1};	/*Property = ScreenPixelsHeight*/
const Value V22689_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22689_4 = {S54, 0, &V22689_4_1404, &P22689_5};	/*Property = ScreenPixelsWidth*/
const Profile P22689 = {"22689", &P22689_4};	/*Profile ID - 22689*/
const Value V20741_1_1 = {S2, NULL};	/*Value = True*/
const Property P20741_1 = {S50, 0, &V20741_1_1, NULL};	/*Property = IsMobile*/
const Value V20741_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20741_5 = {S52, 0, &V20741_5_1227, &P20741_1};	/*Property = ScreenPixelsHeight*/
const Value V20741_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20741_4 = {S54, 0, &V20741_4_1229, &P20741_5};	/*Property = ScreenPixelsWidth*/
const Profile P20741 = {"20741", &P20741_4};	/*Profile ID - 20741*/
const Value V22645_1_1 = {S2, NULL};	/*Value = True*/
const Property P22645_1 = {S50, 0, &V22645_1_1, NULL};	/*Property = IsMobile*/
const Value V22645_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22645_5 = {S52, 0, &V22645_5_1227, &P22645_1};	/*Property = ScreenPixelsHeight*/
const Value V22645_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22645_4 = {S54, 0, &V22645_4_1229, &P22645_5};	/*Property = ScreenPixelsWidth*/
const Profile P22645 = {"22645", &P22645_4};	/*Profile ID - 22645*/
const Value V23817_1_1 = {S2, NULL};	/*Value = True*/
const Property P23817_1 = {S50, 0, &V23817_1_1, NULL};	/*Property = IsMobile*/
const Value V23817_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23817_5 = {S52, 0, &V23817_5_1458, &P23817_1};	/*Property = ScreenPixelsHeight*/
const Value V23817_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23817_4 = {S54, 0, &V23817_4_37617, &P23817_5};	/*Property = ScreenPixelsWidth*/
const Profile P23817 = {"23817", &P23817_4};	/*Profile ID - 23817*/
const Value V22790_1_1 = {S2, NULL};	/*Value = True*/
const Property P22790_1 = {S50, 0, &V22790_1_1, NULL};	/*Property = IsMobile*/
const Value V22790_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22790_5 = {S52, 0, &V22790_5_1277, &P22790_1};	/*Property = ScreenPixelsHeight*/
const Value V22790_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22790_4 = {S54, 0, &V22790_4_1279, &P22790_5};	/*Property = ScreenPixelsWidth*/
const Profile P22790 = {"22790", &P22790_4};	/*Profile ID - 22790*/
const Value V23743_1_1 = {S2, NULL};	/*Value = True*/
const Property P23743_1 = {S50, 0, &V23743_1_1, NULL};	/*Property = IsMobile*/
const Value V23743_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23743_5 = {S52, 0, &V23743_5_1227, &P23743_1};	/*Property = ScreenPixelsHeight*/
const Value V23743_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23743_4 = {S54, 0, &V23743_4_1229, &P23743_5};	/*Property = ScreenPixelsWidth*/
const Profile P23743 = {"23743", &P23743_4};	/*Profile ID - 23743*/
const Value V22666_1_1 = {S2, NULL};	/*Value = True*/
const Property P22666_1 = {S50, 0, &V22666_1_1, NULL};	/*Property = IsMobile*/
const Value V22666_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22666_5 = {S52, 0, &V22666_5_1458, &P22666_1};	/*Property = ScreenPixelsHeight*/
const Value V22666_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22666_4 = {S54, 0, &V22666_4_1460, &P22666_5};	/*Property = ScreenPixelsWidth*/
const Profile P22666 = {"22666", &P22666_4};	/*Profile ID - 22666*/
const Value V23185_1_1 = {S2, NULL};	/*Value = True*/
const Property P23185_1 = {S50, 0, &V23185_1_1, NULL};	/*Property = IsMobile*/
const Value V23185_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23185_5 = {S52, 0, &V23185_5_1458, &P23185_1};	/*Property = ScreenPixelsHeight*/
const Value V23185_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23185_4 = {S54, 0, &V23185_4_37617, &P23185_5};	/*Property = ScreenPixelsWidth*/
const Profile P23185 = {"23185", &P23185_4};	/*Profile ID - 23185*/
const Value V23848_1_1 = {S2, NULL};	/*Value = True*/
const Property P23848_1 = {S50, 0, &V23848_1_1, NULL};	/*Property = IsMobile*/
const Value V23848_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23848_5 = {S52, 0, &V23848_5_1832, &P23848_1};	/*Property = ScreenPixelsHeight*/
const Value V23848_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23848_4 = {S54, 0, &V23848_4_5645, &P23848_5};	/*Property = ScreenPixelsWidth*/
const Profile P23848 = {"23848", &P23848_4};	/*Profile ID - 23848*/
const Value V22644_1_1 = {S2, NULL};	/*Value = True*/
const Property P22644_1 = {S50, 0, &V22644_1_1, NULL};	/*Property = IsMobile*/
const Value V22644_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22644_5 = {S52, 0, &V22644_5_1277, &P22644_1};	/*Property = ScreenPixelsHeight*/
const Value V22644_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22644_4 = {S54, 0, &V22644_4_1279, &P22644_5};	/*Property = ScreenPixelsWidth*/
const Profile P22644 = {"22644", &P22644_4};	/*Profile ID - 22644*/
const Value V22641_1_1 = {S2, NULL};	/*Value = True*/
const Property P22641_1 = {S50, 0, &V22641_1_1, NULL};	/*Property = IsMobile*/
const Value V22641_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22641_5 = {S52, 0, &V22641_5_1277, &P22641_1};	/*Property = ScreenPixelsHeight*/
const Value V22641_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22641_4 = {S54, 0, &V22641_4_1279, &P22641_5};	/*Property = ScreenPixelsWidth*/
const Profile P22641 = {"22641", &P22641_4};	/*Profile ID - 22641*/
const Value V23166_1_1 = {S2, NULL};	/*Value = True*/
const Property P23166_1 = {S50, 0, &V23166_1_1, NULL};	/*Property = IsMobile*/
const Value V23166_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23166_5 = {S52, 0, &V23166_5_1227, &P23166_1};	/*Property = ScreenPixelsHeight*/
const Value V23166_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23166_4 = {S54, 0, &V23166_4_1229, &P23166_5};	/*Property = ScreenPixelsWidth*/
const Profile P23166 = {"23166", &P23166_4};	/*Profile ID - 23166*/
const Value V22615_1_1 = {S2, NULL};	/*Value = True*/
const Property P22615_1 = {S50, 0, &V22615_1_1, NULL};	/*Property = IsMobile*/
const Value V22615_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22615_5 = {S52, 0, &V22615_5_1277, &P22615_1};	/*Property = ScreenPixelsHeight*/
const Value V22615_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22615_4 = {S54, 0, &V22615_4_1279, &P22615_5};	/*Property = ScreenPixelsWidth*/
const Profile P22615 = {"22615", &P22615_4};	/*Profile ID - 22615*/
const Value V23723_1_1 = {S2, NULL};	/*Value = True*/
const Property P23723_1 = {S50, 0, &V23723_1_1, NULL};	/*Property = IsMobile*/
const Value V23723_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23723_5 = {S52, 0, &V23723_5_1277, &P23723_1};	/*Property = ScreenPixelsHeight*/
const Value V23723_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23723_4 = {S54, 0, &V23723_4_1279, &P23723_5};	/*Property = ScreenPixelsWidth*/
const Profile P23723 = {"23723", &P23723_4};	/*Profile ID - 23723*/
const Value V22613_1_1 = {S2, NULL};	/*Value = True*/
const Property P22613_1 = {S50, 0, &V22613_1_1, NULL};	/*Property = IsMobile*/
const Value V22613_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22613_5 = {S52, 0, &V22613_5_1222, &P22613_1};	/*Property = ScreenPixelsHeight*/
const Value V22613_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22613_4 = {S54, 0, &V22613_4_1224, &P22613_5};	/*Property = ScreenPixelsWidth*/
const Profile P22613 = {"22613", &P22613_4};	/*Profile ID - 22613*/
const Value V23008_1_1 = {S2, NULL};	/*Value = True*/
const Property P23008_1 = {S50, 0, &V23008_1_1, NULL};	/*Property = IsMobile*/
const Value V23008_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23008_5 = {S52, 0, &V23008_5_1269, &P23008_1};	/*Property = ScreenPixelsHeight*/
const Value V23008_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23008_4 = {S54, 0, &V23008_4_1229, &P23008_5};	/*Property = ScreenPixelsWidth*/
const Profile P23008 = {"23008", &P23008_4};	/*Profile ID - 23008*/
const Value V24409_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_279 = {S1, 0, &V24409_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24409_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_282 = {S3, 0, &V24409_282_57155, &P24409_279};	/*Property = BlobBuilder*/
const Value V24409_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_350 = {S4, 0, &V24409_350_57446, &P24409_282};	/*Property = Canvas*/
const Value V24409_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_283 = {S5, 0, &V24409_283_57161, &P24409_350};	/*Property = CssBackground*/
const Value V24409_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_284 = {S6, 0, &V24409_284_57164, &P24409_283};	/*Property = CssBorderImage*/
const Value V24409_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_285 = {S7, 0, &V24409_285_57167, &P24409_284};	/*Property = CssCanvas*/
const Value V24409_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_286 = {S8, 0, &V24409_286_57170, &P24409_285};	/*Property = CssColor*/
const Value V24409_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_237 = {S9, 0, &V24409_237_55786, &P24409_286};	/*Property = CssColumn*/
const Value V24409_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_287 = {S10, 0, &V24409_287_57173, &P24409_237};	/*Property = CssFlexbox*/
const Value V24409_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_288 = {S11, 0, &V24409_288_57176, &P24409_287};	/*Property = CssFont*/
const Value V24409_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_289 = {S12, 0, &V24409_289_86899, &P24409_288};	/*Property = CssImages*/
const Value V24409_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_291 = {S13, 0, &V24409_291_57182, &P24409_289};	/*Property = CssMediaQueries*/
const Value V24409_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_292 = {S14, 0, &V24409_292_57185, &P24409_291};	/*Property = CssMinMax*/
const Value V24409_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_293 = {S15, 0, &V24409_293_57188, &P24409_292};	/*Property = CssOverflow*/
const Value V24409_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_294 = {S16, 0, &V24409_294_57191, &P24409_293};	/*Property = CssPosition*/
const Value V24409_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_295 = {S17, 0, &V24409_295_57194, &P24409_294};	/*Property = CssText*/
const Value V24409_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_297 = {S18, 0, &V24409_297_57197, &P24409_295};	/*Property = CssTransforms*/
const Value V24409_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_241 = {S19, 0, &V24409_241_55801, &P24409_297};	/*Property = CssTransitions*/
const Value V24409_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_298 = {S20, 0, &V24409_298_57208, &P24409_241};	/*Property = CssUI*/
const Value V24409_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_300 = {S21, 0, &V24409_300_57217, &P24409_298};	/*Property = DataSet*/
const Value V24409_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_301 = {S22, 0, &V24409_301_57220, &P24409_300};	/*Property = DataUrl*/
const Value V24409_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_302 = {S23, 0, &V24409_302_57223, &P24409_301};	/*Property = DeviceOrientation*/
const Value V24409_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_304 = {S24, 0, &V24409_304_57229, &P24409_302};	/*Property = FileReader*/
const Value V24409_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_305 = {S25, 0, &V24409_305_57232, &P24409_304};	/*Property = FileSaver*/
const Value V24409_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_306 = {S26, 0, &V24409_306_57238, &P24409_305};	/*Property = FileWriter*/
const Value V24409_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_307 = {S27, 0, &V24409_307_57241, &P24409_306};	/*Property = FormData*/
const Value V24409_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_309 = {S28, 0, &V24409_309_57247, &P24409_307};	/*Property = Fullscreen*/
const Value V24409_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_310 = {S29, 0, &V24409_310_57250, &P24409_309};	/*Property = GeoLocation*/
const Value V24409_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_312 = {S30, 0, &V24409_312_57256, &P24409_310};	/*Property = History*/
const Value V24409_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_314 = {S31, 0, &V24409_314_57262, &P24409_312};	/*Property = Html5*/
const Value V24409_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_313 = {S32, 0, &V24409_313_57259, &P24409_314};	/*Property = Html-Media-Capture*/
const Value V24409_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_315 = {S33, 0, &V24409_315_57265, &P24409_313};	/*Property = Iframe*/
const Value V24409_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_316 = {S34, 0, &V24409_316_57268, &P24409_315};	/*Property = IndexedDB*/
const Value V24409_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_317 = {S35, 0, &V24409_317_57271, &P24409_316};	/*Property = Json*/
const Value V24409_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24409_15 = {S37, 0, &V24409_15_28, &P24409_317};	/*Property = LayoutEngine*/
const Value V24409_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_344 = {S38, 0, &V24409_344_57274, &P24409_15};	/*Property = Masking*/
const Value V24409_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_320 = {S39, 0, &V24409_320_57292, &P24409_344};	/*Property = PostMessage*/
const Value V24409_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_321 = {S40, 0, &V24409_321_57295, &P24409_320};	/*Property = Progress*/
const Value V24409_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_322 = {S41, 0, &V24409_322_57298, &P24409_321};	/*Property = Prompts*/
const Value V24409_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_323 = {S42, 0, &V24409_323_57301, &P24409_322};	/*Property = Selector*/
const Value V24409_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_324 = {S43, 0, &V24409_324_57307, &P24409_323};	/*Property = Svg*/
const Value V24409_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_327 = {S44, 0, &V24409_327_57316, &P24409_324};	/*Property = TouchEvents*/
const Value V24409_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_328 = {S45, 0, &V24409_328_57319, &P24409_327};	/*Property = Track*/
const Value V24409_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_329 = {S46, 0, &V24409_329_57325, &P24409_328};	/*Property = Video*/
const Value V24409_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_330 = {S47, 0, &V24409_330_57328, &P24409_329};	/*Property = Viewport*/
const Value V24409_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_332 = {S48, 0, &V24409_332_57340, &P24409_330};	/*Property = WebWorkers*/
const Value V24409_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24409_333 = {S49, 0, &V24409_333_57343, &P24409_332};	/*Property = Xhr2*/
const Profile P24409 = {"24409", &P24409_333};	/*Profile ID - 24409*/
const Value V22747_1_1 = {S2, NULL};	/*Value = True*/
const Property P22747_1 = {S50, 0, &V22747_1_1, NULL};	/*Property = IsMobile*/
const Value V22747_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22747_5 = {S52, 0, &V22747_5_1277, &P22747_1};	/*Property = ScreenPixelsHeight*/
const Value V22747_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22747_4 = {S54, 0, &V22747_4_1279, &P22747_5};	/*Property = ScreenPixelsWidth*/
const Profile P22747 = {"22747", &P22747_4};	/*Profile ID - 22747*/
const Value V22928_1_1 = {S2, NULL};	/*Value = True*/
const Property P22928_1 = {S50, 0, &V22928_1_1, NULL};	/*Property = IsMobile*/
const Value V22928_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22928_5 = {S52, 0, &V22928_5_1458, &P22928_1};	/*Property = ScreenPixelsHeight*/
const Value V22928_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22928_4 = {S54, 0, &V22928_4_1460, &P22928_5};	/*Property = ScreenPixelsWidth*/
const Profile P22928 = {"22928", &P22928_4};	/*Profile ID - 22928*/
const Value V23369_279_57148 = {S0, NULL};	/*Value = False*/
const Property P23369_279 = {S1, 0, &V23369_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V23369_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23369_282 = {S3, 0, &V23369_282_57154, &P23369_279};	/*Property = BlobBuilder*/
const Value V23369_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23369_350 = {S4, 0, &V23369_350_57444, &P23369_282};	/*Property = Canvas*/
const Value V23369_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23369_283 = {S5, 0, &V23369_283_57159, &P23369_350};	/*Property = CssBackground*/
const Value V23369_284_57163 = {S0, NULL};	/*Value = False*/
const Property P23369_284 = {S6, 0, &V23369_284_57163, &P23369_283};	/*Property = CssBorderImage*/
const Value V23369_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23369_285 = {S7, 0, &V23369_285_57166, &P23369_284};	/*Property = CssCanvas*/
const Value V23369_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23369_286 = {S8, 0, &V23369_286_57168, &P23369_285};	/*Property = CssColor*/
const Value V23369_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23369_237 = {S9, 0, &V23369_237_55786, &P23369_286};	/*Property = CssColumn*/
const Value V23369_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23369_287 = {S10, 0, &V23369_287_57172, &P23369_237};	/*Property = CssFlexbox*/
const Value V23369_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23369_288 = {S11, 0, &V23369_288_57174, &P23369_287};	/*Property = CssFont*/
const Value V23369_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23369_289 = {S12, 0, &V23369_289_57759, &P23369_288};	/*Property = CssImages*/
const Value V23369_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23369_291 = {S13, 0, &V23369_291_57180, &P23369_289};	/*Property = CssMediaQueries*/
const Value V23369_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23369_292 = {S14, 0, &V23369_292_57183, &P23369_291};	/*Property = CssMinMax*/
const Value V23369_293_57187 = {S0, NULL};	/*Value = False*/
const Property P23369_293 = {S15, 0, &V23369_293_57187, &P23369_292};	/*Property = CssOverflow*/
const Value V23369_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23369_294 = {S16, 0, &V23369_294_57189, &P23369_293};	/*Property = CssPosition*/
const Value V23369_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23369_295 = {S17, 0, &V23369_295_57192, &P23369_294};	/*Property = CssText*/
const Value V23369_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23369_297 = {S18, 0, &V23369_297_57195, &P23369_295};	/*Property = CssTransforms*/
const Value V23369_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23369_241 = {S19, 0, &V23369_241_55799, &P23369_297};	/*Property = CssTransitions*/
const Value V23369_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23369_298 = {S20, 0, &V23369_298_57206, &P23369_241};	/*Property = CssUI*/
const Value V23369_300_57216 = {S0, NULL};	/*Value = False*/
const Property P23369_300 = {S21, 0, &V23369_300_57216, &P23369_298};	/*Property = DataSet*/
const Value V23369_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23369_301 = {S22, 0, &V23369_301_57218, &P23369_300};	/*Property = DataUrl*/
const Value V23369_302_57222 = {S0, NULL};	/*Value = False*/
const Property P23369_302 = {S23, 0, &V23369_302_57222, &P23369_301};	/*Property = DeviceOrientation*/
const Value V23369_304_57227 = {S2, NULL};	/*Value = True*/
const Property P23369_304 = {S24, 0, &V23369_304_57227, &P23369_302};	/*Property = FileReader*/
const Value V23369_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23369_305 = {S25, 0, &V23369_305_57231, &P23369_304};	/*Property = FileSaver*/
const Value V23369_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23369_306 = {S26, 0, &V23369_306_57237, &P23369_305};	/*Property = FileWriter*/
const Value V23369_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23369_307 = {S27, 0, &V23369_307_57239, &P23369_306};	/*Property = FormData*/
const Value V23369_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23369_309 = {S28, 0, &V23369_309_57246, &P23369_307};	/*Property = Fullscreen*/
const Value V23369_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23369_310 = {S29, 0, &V23369_310_57248, &P23369_309};	/*Property = GeoLocation*/
const Value V23369_312_57254 = {S2, NULL};	/*Value = True*/
const Property P23369_312 = {S30, 0, &V23369_312_57254, &P23369_310};	/*Property = History*/
const Value V23369_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23369_314 = {S31, 0, &V23369_314_57261, &P23369_312};	/*Property = Html5*/
const Value V23369_313_57257 = {S2, NULL};	/*Value = True*/
const Property P23369_313 = {S32, 0, &V23369_313_57257, &P23369_314};	/*Property = Html-Media-Capture*/
const Value V23369_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23369_315 = {S33, 0, &V23369_315_57264, &P23369_313};	/*Property = Iframe*/
const Value V23369_316_57266 = {S2, NULL};	/*Value = True*/
const Property P23369_316 = {S34, 0, &V23369_316_57266, &P23369_315};	/*Property = IndexedDB*/
const Value V23369_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23369_317 = {S35, 0, &V23369_317_57269, &P23369_316};	/*Property = Json*/
const Value V23369_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P23369_15 = {S37, 0, &V23369_15_28, &P23369_317};	/*Property = LayoutEngine*/
const Value V23369_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23369_344 = {S38, 0, &V23369_344_57272, &P23369_15};	/*Property = Masking*/
const Value V23369_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23369_320 = {S39, 0, &V23369_320_57290, &P23369_344};	/*Property = PostMessage*/
const Value V23369_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23369_321 = {S40, 0, &V23369_321_57293, &P23369_320};	/*Property = Progress*/
const Value V23369_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23369_322 = {S41, 0, &V23369_322_57296, &P23369_321};	/*Property = Prompts*/
const Value V23369_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23369_323 = {S42, 0, &V23369_323_57299, &P23369_322};	/*Property = Selector*/
const Value V23369_324_57306 = {S0, NULL};	/*Value = False*/
const Property P23369_324 = {S43, 0, &V23369_324_57306, &P23369_323};	/*Property = Svg*/
const Value V23369_327_57314 = {S2, NULL};	/*Value = True*/
const Property P23369_327 = {S44, 0, &V23369_327_57314, &P23369_324};	/*Property = TouchEvents*/
const Value V23369_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23369_328 = {S45, 0, &V23369_328_57318, &P23369_327};	/*Property = Track*/
const Value V23369_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23369_329 = {S46, 0, &V23369_329_57323, &P23369_328};	/*Property = Video*/
const Value V23369_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23369_330 = {S47, 0, &V23369_330_57326, &P23369_329};	/*Property = Viewport*/
const Value V23369_332_57338 = {S2, NULL};	/*Value = True*/
const Property P23369_332 = {S48, 0, &V23369_332_57338, &P23369_330};	/*Property = WebWorkers*/
const Value V23369_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23369_333 = {S49, 0, &V23369_333_57342, &P23369_332};	/*Property = Xhr2*/
const Profile P23369 = {"23369", &P23369_333};	/*Profile ID - 23369*/
const Value V23280_1_1 = {S2, NULL};	/*Value = True*/
const Property P23280_1 = {S50, 0, &V23280_1_1, NULL};	/*Property = IsMobile*/
const Value V23280_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23280_5 = {S52, 0, &V23280_5_1227, &P23280_1};	/*Property = ScreenPixelsHeight*/
const Value V23280_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23280_4 = {S54, 0, &V23280_4_1229, &P23280_5};	/*Property = ScreenPixelsWidth*/
const Profile P23280 = {"23280", &P23280_4};	/*Profile ID - 23280*/
const Value V22640_1_1 = {S2, NULL};	/*Value = True*/
const Property P22640_1 = {S50, 0, &V22640_1_1, NULL};	/*Property = IsMobile*/
const Value V22640_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22640_5 = {S52, 0, &V22640_5_1403, &P22640_1};	/*Property = ScreenPixelsHeight*/
const Value V22640_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22640_4 = {S54, 0, &V22640_4_1404, &P22640_5};	/*Property = ScreenPixelsWidth*/
const Profile P22640 = {"22640", &P22640_4};	/*Profile ID - 22640*/
const Value V22395_1_1 = {S2, NULL};	/*Value = True*/
const Property P22395_1 = {S50, 0, &V22395_1_1, NULL};	/*Property = IsMobile*/
const Value V22395_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22395_5 = {S52, 0, &V22395_5_59138, &P22395_1};	/*Property = ScreenPixelsHeight*/
const Value V22395_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22395_4 = {S54, 0, &V22395_4_59970, &P22395_5};	/*Property = ScreenPixelsWidth*/
const Profile P22395 = {"22395", &P22395_4};	/*Profile ID - 22395*/
const Value V20693_1_1 = {S2, NULL};	/*Value = True*/
const Property P20693_1 = {S50, 0, &V20693_1_1, NULL};	/*Property = IsMobile*/
const Value V20693_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20693_5 = {S52, 0, &V20693_5_1227, &P20693_1};	/*Property = ScreenPixelsHeight*/
const Value V20693_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20693_4 = {S54, 0, &V20693_4_1229, &P20693_5};	/*Property = ScreenPixelsWidth*/
const Profile P20693 = {"20693", &P20693_4};	/*Profile ID - 20693*/
const Value V23061_1_1 = {S2, NULL};	/*Value = True*/
const Property P23061_1 = {S50, 0, &V23061_1_1, NULL};	/*Property = IsMobile*/
const Value V23061_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23061_5 = {S52, 0, &V23061_5_1277, &P23061_1};	/*Property = ScreenPixelsHeight*/
const Value V23061_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23061_4 = {S54, 0, &V23061_4_1279, &P23061_5};	/*Property = ScreenPixelsWidth*/
const Profile P23061 = {"23061", &P23061_4};	/*Profile ID - 23061*/
const Value V22633_1_1 = {S2, NULL};	/*Value = True*/
const Property P22633_1 = {S50, 0, &V22633_1_1, NULL};	/*Property = IsMobile*/
const Value V22633_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22633_5 = {S52, 0, &V22633_5_1227, &P22633_1};	/*Property = ScreenPixelsHeight*/
const Value V22633_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22633_4 = {S54, 0, &V22633_4_1460, &P22633_5};	/*Property = ScreenPixelsWidth*/
const Profile P22633 = {"22633", &P22633_4};	/*Profile ID - 22633*/
const Value V22476_1_1 = {S2, NULL};	/*Value = True*/
const Property P22476_1 = {S50, 0, &V22476_1_1, NULL};	/*Property = IsMobile*/
const Value V22476_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22476_5 = {S52, 0, &V22476_5_1277, &P22476_1};	/*Property = ScreenPixelsHeight*/
const Value V22476_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22476_4 = {S54, 0, &V22476_4_2336, &P22476_5};	/*Property = ScreenPixelsWidth*/
const Profile P22476 = {"22476", &P22476_4};	/*Profile ID - 22476*/
const Value V22569_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_279 = {S1, 0, &V22569_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22569_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_282 = {S3, 0, &V22569_282_57155, &P22569_279};	/*Property = BlobBuilder*/
const Value V22569_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_350 = {S4, 0, &V22569_350_57446, &P22569_282};	/*Property = Canvas*/
const Value V22569_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_283 = {S5, 0, &V22569_283_57161, &P22569_350};	/*Property = CssBackground*/
const Value V22569_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_284 = {S6, 0, &V22569_284_57164, &P22569_283};	/*Property = CssBorderImage*/
const Value V22569_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_285 = {S7, 0, &V22569_285_57167, &P22569_284};	/*Property = CssCanvas*/
const Value V22569_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_286 = {S8, 0, &V22569_286_57170, &P22569_285};	/*Property = CssColor*/
const Value V22569_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_237 = {S9, 0, &V22569_237_55786, &P22569_286};	/*Property = CssColumn*/
const Value V22569_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_287 = {S10, 0, &V22569_287_57173, &P22569_237};	/*Property = CssFlexbox*/
const Value V22569_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_288 = {S11, 0, &V22569_288_57176, &P22569_287};	/*Property = CssFont*/
const Value V22569_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_289 = {S12, 0, &V22569_289_86899, &P22569_288};	/*Property = CssImages*/
const Value V22569_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_291 = {S13, 0, &V22569_291_57182, &P22569_289};	/*Property = CssMediaQueries*/
const Value V22569_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_292 = {S14, 0, &V22569_292_57185, &P22569_291};	/*Property = CssMinMax*/
const Value V22569_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_293 = {S15, 0, &V22569_293_57188, &P22569_292};	/*Property = CssOverflow*/
const Value V22569_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_294 = {S16, 0, &V22569_294_57191, &P22569_293};	/*Property = CssPosition*/
const Value V22569_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_295 = {S17, 0, &V22569_295_57194, &P22569_294};	/*Property = CssText*/
const Value V22569_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_297 = {S18, 0, &V22569_297_57197, &P22569_295};	/*Property = CssTransforms*/
const Value V22569_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_241 = {S19, 0, &V22569_241_55801, &P22569_297};	/*Property = CssTransitions*/
const Value V22569_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_298 = {S20, 0, &V22569_298_57208, &P22569_241};	/*Property = CssUI*/
const Value V22569_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_300 = {S21, 0, &V22569_300_57217, &P22569_298};	/*Property = DataSet*/
const Value V22569_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_301 = {S22, 0, &V22569_301_57220, &P22569_300};	/*Property = DataUrl*/
const Value V22569_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_302 = {S23, 0, &V22569_302_57223, &P22569_301};	/*Property = DeviceOrientation*/
const Value V22569_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_304 = {S24, 0, &V22569_304_57229, &P22569_302};	/*Property = FileReader*/
const Value V22569_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_305 = {S25, 0, &V22569_305_57232, &P22569_304};	/*Property = FileSaver*/
const Value V22569_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_306 = {S26, 0, &V22569_306_57238, &P22569_305};	/*Property = FileWriter*/
const Value V22569_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_307 = {S27, 0, &V22569_307_57241, &P22569_306};	/*Property = FormData*/
const Value V22569_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_309 = {S28, 0, &V22569_309_57247, &P22569_307};	/*Property = Fullscreen*/
const Value V22569_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_310 = {S29, 0, &V22569_310_57250, &P22569_309};	/*Property = GeoLocation*/
const Value V22569_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_312 = {S30, 0, &V22569_312_57256, &P22569_310};	/*Property = History*/
const Value V22569_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_314 = {S31, 0, &V22569_314_57262, &P22569_312};	/*Property = Html5*/
const Value V22569_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_313 = {S32, 0, &V22569_313_57259, &P22569_314};	/*Property = Html-Media-Capture*/
const Value V22569_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_315 = {S33, 0, &V22569_315_57265, &P22569_313};	/*Property = Iframe*/
const Value V22569_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_316 = {S34, 0, &V22569_316_57268, &P22569_315};	/*Property = IndexedDB*/
const Value V22569_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_317 = {S35, 0, &V22569_317_57271, &P22569_316};	/*Property = Json*/
const Value V22569_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P22569_15 = {S37, 0, &V22569_15_54167, &P22569_317};	/*Property = LayoutEngine*/
const Value V22569_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_344 = {S38, 0, &V22569_344_57274, &P22569_15};	/*Property = Masking*/
const Value V22569_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_320 = {S39, 0, &V22569_320_57292, &P22569_344};	/*Property = PostMessage*/
const Value V22569_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_321 = {S40, 0, &V22569_321_57295, &P22569_320};	/*Property = Progress*/
const Value V22569_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_322 = {S41, 0, &V22569_322_57298, &P22569_321};	/*Property = Prompts*/
const Value V22569_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_323 = {S42, 0, &V22569_323_57301, &P22569_322};	/*Property = Selector*/
const Value V22569_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_324 = {S43, 0, &V22569_324_57307, &P22569_323};	/*Property = Svg*/
const Value V22569_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_327 = {S44, 0, &V22569_327_57316, &P22569_324};	/*Property = TouchEvents*/
const Value V22569_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_328 = {S45, 0, &V22569_328_57319, &P22569_327};	/*Property = Track*/
const Value V22569_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_329 = {S46, 0, &V22569_329_57325, &P22569_328};	/*Property = Video*/
const Value V22569_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_330 = {S47, 0, &V22569_330_57328, &P22569_329};	/*Property = Viewport*/
const Value V22569_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_332 = {S48, 0, &V22569_332_57340, &P22569_330};	/*Property = WebWorkers*/
const Value V22569_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22569_333 = {S49, 0, &V22569_333_57343, &P22569_332};	/*Property = Xhr2*/
const Profile P22569 = {"22569", &P22569_333};	/*Profile ID - 22569*/
const Value V23455_1_1 = {S2, NULL};	/*Value = True*/
const Property P23455_1 = {S50, 0, &V23455_1_1, NULL};	/*Property = IsMobile*/
const Value V23455_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23455_5 = {S52, 0, &V23455_5_1227, &P23455_1};	/*Property = ScreenPixelsHeight*/
const Value V23455_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23455_4 = {S54, 0, &V23455_4_1229, &P23455_5};	/*Property = ScreenPixelsWidth*/
const Profile P23455 = {"23455", &P23455_4};	/*Profile ID - 23455*/
const Value V24744_1_1 = {S2, NULL};	/*Value = True*/
const Property P24744_1 = {S50, 0, &V24744_1_1, NULL};	/*Property = IsMobile*/
const Value V24744_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24744_5 = {S52, 0, &V24744_5_1403, &P24744_1};	/*Property = ScreenPixelsHeight*/
const Value V24744_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24744_4 = {S54, 0, &V24744_4_1404, &P24744_5};	/*Property = ScreenPixelsWidth*/
const Profile P24744 = {"24744", &P24744_4};	/*Profile ID - 24744*/
const Value V22980_1_1 = {S2, NULL};	/*Value = True*/
const Property P22980_1 = {S50, 0, &V22980_1_1, NULL};	/*Property = IsMobile*/
const Value V22980_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22980_5 = {S52, 0, &V22980_5_1227, &P22980_1};	/*Property = ScreenPixelsHeight*/
const Value V22980_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22980_4 = {S54, 0, &V22980_4_1229, &P22980_5};	/*Property = ScreenPixelsWidth*/
const Profile P22980 = {"22980", &P22980_4};	/*Profile ID - 22980*/
const Value V22589_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22589_279 = {S1, 0, &V22589_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22589_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22589_282 = {S3, 0, &V22589_282_57154, &P22589_279};	/*Property = BlobBuilder*/
const Value V22589_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22589_350 = {S4, 0, &V22589_350_57444, &P22589_282};	/*Property = Canvas*/
const Value V22589_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22589_283 = {S5, 0, &V22589_283_57159, &P22589_350};	/*Property = CssBackground*/
const Value V22589_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22589_284 = {S6, 0, &V22589_284_57162, &P22589_283};	/*Property = CssBorderImage*/
const Value V22589_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22589_285 = {S7, 0, &V22589_285_57166, &P22589_284};	/*Property = CssCanvas*/
const Value V22589_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22589_286 = {S8, 0, &V22589_286_57168, &P22589_285};	/*Property = CssColor*/
const Value V22589_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22589_237 = {S9, 0, &V22589_237_55786, &P22589_286};	/*Property = CssColumn*/
const Value V22589_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22589_287 = {S10, 0, &V22589_287_57172, &P22589_237};	/*Property = CssFlexbox*/
const Value V22589_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22589_288 = {S11, 0, &V22589_288_57174, &P22589_287};	/*Property = CssFont*/
const Value V22589_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22589_289 = {S12, 0, &V22589_289_57759, &P22589_288};	/*Property = CssImages*/
const Value V22589_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22589_291 = {S13, 0, &V22589_291_57180, &P22589_289};	/*Property = CssMediaQueries*/
const Value V22589_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22589_292 = {S14, 0, &V22589_292_57183, &P22589_291};	/*Property = CssMinMax*/
const Value V22589_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22589_293 = {S15, 0, &V22589_293_57187, &P22589_292};	/*Property = CssOverflow*/
const Value V22589_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22589_294 = {S16, 0, &V22589_294_57189, &P22589_293};	/*Property = CssPosition*/
const Value V22589_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22589_295 = {S17, 0, &V22589_295_57192, &P22589_294};	/*Property = CssText*/
const Value V22589_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22589_297 = {S18, 0, &V22589_297_57195, &P22589_295};	/*Property = CssTransforms*/
const Value V22589_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22589_241 = {S19, 0, &V22589_241_55799, &P22589_297};	/*Property = CssTransitions*/
const Value V22589_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22589_298 = {S20, 0, &V22589_298_57206, &P22589_241};	/*Property = CssUI*/
const Value V22589_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22589_300 = {S21, 0, &V22589_300_57215, &P22589_298};	/*Property = DataSet*/
const Value V22589_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22589_301 = {S22, 0, &V22589_301_57218, &P22589_300};	/*Property = DataUrl*/
const Value V22589_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22589_302 = {S23, 0, &V22589_302_57221, &P22589_301};	/*Property = DeviceOrientation*/
const Value V22589_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22589_304 = {S24, 0, &V22589_304_57227, &P22589_302};	/*Property = FileReader*/
const Value V22589_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22589_305 = {S25, 0, &V22589_305_57231, &P22589_304};	/*Property = FileSaver*/
const Value V22589_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22589_306 = {S26, 0, &V22589_306_57237, &P22589_305};	/*Property = FileWriter*/
const Value V22589_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22589_307 = {S27, 0, &V22589_307_57239, &P22589_306};	/*Property = FormData*/
const Value V22589_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22589_309 = {S28, 0, &V22589_309_57246, &P22589_307};	/*Property = Fullscreen*/
const Value V22589_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22589_310 = {S29, 0, &V22589_310_57248, &P22589_309};	/*Property = GeoLocation*/
const Value V22589_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22589_312 = {S30, 0, &V22589_312_57254, &P22589_310};	/*Property = History*/
const Value V22589_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22589_314 = {S31, 0, &V22589_314_57260, &P22589_312};	/*Property = Html5*/
const Value V22589_313_57257 = {S2, NULL};	/*Value = True*/
const Property P22589_313 = {S32, 0, &V22589_313_57257, &P22589_314};	/*Property = Html-Media-Capture*/
const Value V22589_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22589_315 = {S33, 0, &V22589_315_57264, &P22589_313};	/*Property = Iframe*/
const Value V22589_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22589_316 = {S34, 0, &V22589_316_57266, &P22589_315};	/*Property = IndexedDB*/
const Value V22589_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22589_317 = {S35, 0, &V22589_317_57269, &P22589_316};	/*Property = Json*/
const Value V22589_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P22589_15 = {S37, 0, &V22589_15_54167, &P22589_317};	/*Property = LayoutEngine*/
const Value V22589_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22589_344 = {S38, 0, &V22589_344_57272, &P22589_15};	/*Property = Masking*/
const Value V22589_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22589_320 = {S39, 0, &V22589_320_57290, &P22589_344};	/*Property = PostMessage*/
const Value V22589_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22589_321 = {S40, 0, &V22589_321_57293, &P22589_320};	/*Property = Progress*/
const Value V22589_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22589_322 = {S41, 0, &V22589_322_57296, &P22589_321};	/*Property = Prompts*/
const Value V22589_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22589_323 = {S42, 0, &V22589_323_57299, &P22589_322};	/*Property = Selector*/
const Value V22589_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22589_324 = {S43, 0, &V22589_324_57306, &P22589_323};	/*Property = Svg*/
const Value V22589_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22589_327 = {S44, 0, &V22589_327_57314, &P22589_324};	/*Property = TouchEvents*/
const Value V22589_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22589_328 = {S45, 0, &V22589_328_57318, &P22589_327};	/*Property = Track*/
const Value V22589_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22589_329 = {S46, 0, &V22589_329_57323, &P22589_328};	/*Property = Video*/
const Value V22589_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22589_330 = {S47, 0, &V22589_330_57326, &P22589_329};	/*Property = Viewport*/
const Value V22589_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22589_332 = {S48, 0, &V22589_332_57339, &P22589_330};	/*Property = WebWorkers*/
const Value V22589_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22589_333 = {S49, 0, &V22589_333_57342, &P22589_332};	/*Property = Xhr2*/
const Profile P22589 = {"22589", &P22589_333};	/*Profile ID - 22589*/
const Value V23182_1_1 = {S2, NULL};	/*Value = True*/
const Property P23182_1 = {S50, 0, &V23182_1_1, NULL};	/*Property = IsMobile*/
const Value V23182_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23182_5 = {S52, 0, &V23182_5_37569, &P23182_1};	/*Property = ScreenPixelsHeight*/
const Value V23182_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23182_4 = {S54, 0, &V23182_4_5645, &P23182_5};	/*Property = ScreenPixelsWidth*/
const Profile P23182 = {"23182", &P23182_4};	/*Profile ID - 23182*/
const Value V22611_1_1 = {S2, NULL};	/*Value = True*/
const Property P22611_1 = {S50, 0, &V22611_1_1, NULL};	/*Property = IsMobile*/
const Value V22611_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22611_5 = {S52, 0, &V22611_5_1227, &P22611_1};	/*Property = ScreenPixelsHeight*/
const Value V22611_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22611_4 = {S54, 0, &V22611_4_1229, &P22611_5};	/*Property = ScreenPixelsWidth*/
const Profile P22611 = {"22611", &P22611_4};	/*Profile ID - 22611*/
const Value V23140_1_1 = {S2, NULL};	/*Value = True*/
const Property P23140_1 = {S50, 0, &V23140_1_1, NULL};	/*Property = IsMobile*/
const Value V23140_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23140_5 = {S52, 0, &V23140_5_1277, &P23140_1};	/*Property = ScreenPixelsHeight*/
const Value V23140_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23140_4 = {S54, 0, &V23140_4_1279, &P23140_5};	/*Property = ScreenPixelsWidth*/
const Profile P23140 = {"23140", &P23140_4};	/*Profile ID - 23140*/
const Value V22755_1_1 = {S2, NULL};	/*Value = True*/
const Property P22755_1 = {S50, 0, &V22755_1_1, NULL};	/*Property = IsMobile*/
const Value V22755_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22755_5 = {S52, 0, &V22755_5_9981, &P22755_1};	/*Property = ScreenPixelsHeight*/
const Value V22755_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22755_4 = {S54, 0, &V22755_4_39098, &P22755_5};	/*Property = ScreenPixelsWidth*/
const Profile P22755 = {"22755", &P22755_4};	/*Profile ID - 22755*/
const Value V23291_1_2 = {S0, NULL};	/*Value = False*/
const Property P23291_1 = {S50, 0, &V23291_1_2, NULL};	/*Property = IsMobile*/
const Value V23291_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23291_5 = {S52, 0, &V23291_5_37, &P23291_1};	/*Property = ScreenPixelsHeight*/
const Value V23291_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23291_4 = {S54, 0, &V23291_4_36, &P23291_5};	/*Property = ScreenPixelsWidth*/
const Profile P23291 = {"23291", &P23291_4};	/*Profile ID - 23291*/
const Value V24140_1_1 = {S2, NULL};	/*Value = True*/
const Property P24140_1 = {S50, 0, &V24140_1_1, NULL};	/*Property = IsMobile*/
const Value V24140_5_62105 = {S144, NULL};	/*Value = 2048*/
const Property P24140_5 = {S52, 0, &V24140_5_62105, &P24140_1};	/*Property = ScreenPixelsHeight*/
const Value V24140_4_62106 = {S145, NULL};	/*Value = 1536*/
const Property P24140_4 = {S54, 0, &V24140_4_62106, &P24140_5};	/*Property = ScreenPixelsWidth*/
const Profile P24140 = {"24140", &P24140_4};	/*Profile ID - 24140*/
const Value V22636_1_1 = {S2, NULL};	/*Value = True*/
const Property P22636_1 = {S50, 0, &V22636_1_1, NULL};	/*Property = IsMobile*/
const Value V22636_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22636_5 = {S52, 0, &V22636_5_1403, &P22636_1};	/*Property = ScreenPixelsHeight*/
const Value V22636_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22636_4 = {S54, 0, &V22636_4_1404, &P22636_5};	/*Property = ScreenPixelsWidth*/
const Profile P22636 = {"22636", &P22636_4};	/*Profile ID - 22636*/
const Value V22686_1_1 = {S2, NULL};	/*Value = True*/
const Property P22686_1 = {S50, 0, &V22686_1_1, NULL};	/*Property = IsMobile*/
const Value V22686_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22686_5 = {S52, 0, &V22686_5_1227, &P22686_1};	/*Property = ScreenPixelsHeight*/
const Value V22686_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22686_4 = {S54, 0, &V22686_4_1229, &P22686_5};	/*Property = ScreenPixelsWidth*/
const Profile P22686 = {"22686", &P22686_4};	/*Profile ID - 22686*/
const Value V22952_1_1 = {S2, NULL};	/*Value = True*/
const Property P22952_1 = {S50, 0, &V22952_1_1, NULL};	/*Property = IsMobile*/
const Value V22952_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22952_5 = {S52, 0, &V22952_5_9981, &P22952_1};	/*Property = ScreenPixelsHeight*/
const Value V22952_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22952_4 = {S54, 0, &V22952_4_39098, &P22952_5};	/*Property = ScreenPixelsWidth*/
const Profile P22952 = {"22952", &P22952_4};	/*Profile ID - 22952*/
const Value V22631_1_1 = {S2, NULL};	/*Value = True*/
const Property P22631_1 = {S50, 0, &V22631_1_1, NULL};	/*Property = IsMobile*/
const Value V22631_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22631_5 = {S52, 0, &V22631_5_1277, &P22631_1};	/*Property = ScreenPixelsHeight*/
const Value V22631_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22631_4 = {S54, 0, &V22631_4_1279, &P22631_5};	/*Property = ScreenPixelsWidth*/
const Profile P22631 = {"22631", &P22631_4};	/*Profile ID - 22631*/
const Value V23233_1_1 = {S2, NULL};	/*Value = True*/
const Property P23233_1 = {S50, 0, &V23233_1_1, NULL};	/*Property = IsMobile*/
const Value V23233_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23233_5 = {S52, 0, &V23233_5_1277, &P23233_1};	/*Property = ScreenPixelsHeight*/
const Value V23233_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23233_4 = {S54, 0, &V23233_4_2336, &P23233_5};	/*Property = ScreenPixelsWidth*/
const Profile P23233 = {"23233", &P23233_4};	/*Profile ID - 23233*/
const Value V23429_1_1 = {S2, NULL};	/*Value = True*/
const Property P23429_1 = {S50, 0, &V23429_1_1, NULL};	/*Property = IsMobile*/
const Value V23429_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23429_5 = {S52, 0, &V23429_5_1277, &P23429_1};	/*Property = ScreenPixelsHeight*/
const Value V23429_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23429_4 = {S54, 0, &V23429_4_2336, &P23429_5};	/*Property = ScreenPixelsWidth*/
const Profile P23429 = {"23429", &P23429_4};	/*Profile ID - 23429*/
const Value V23793_1_1 = {S2, NULL};	/*Value = True*/
const Property P23793_1 = {S50, 0, &V23793_1_1, NULL};	/*Property = IsMobile*/
const Value V23793_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23793_5 = {S52, 0, &V23793_5_1458, &P23793_1};	/*Property = ScreenPixelsHeight*/
const Value V23793_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23793_4 = {S54, 0, &V23793_4_37617, &P23793_5};	/*Property = ScreenPixelsWidth*/
const Profile P23793 = {"23793", &P23793_4};	/*Profile ID - 23793*/
const Value V23815_1_1 = {S2, NULL};	/*Value = True*/
const Property P23815_1 = {S50, 0, &V23815_1_1, NULL};	/*Property = IsMobile*/
const Value V23815_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23815_5 = {S52, 0, &V23815_5_1227, &P23815_1};	/*Property = ScreenPixelsHeight*/
const Value V23815_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23815_4 = {S54, 0, &V23815_4_1229, &P23815_5};	/*Property = ScreenPixelsWidth*/
const Profile P23815 = {"23815", &P23815_4};	/*Profile ID - 23815*/
const Value V22646_1_1 = {S2, NULL};	/*Value = True*/
const Property P22646_1 = {S50, 0, &V22646_1_1, NULL};	/*Property = IsMobile*/
const Value V22646_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22646_5 = {S52, 0, &V22646_5_9981, &P22646_1};	/*Property = ScreenPixelsHeight*/
const Value V22646_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22646_4 = {S54, 0, &V22646_4_39098, &P22646_5};	/*Property = ScreenPixelsWidth*/
const Profile P22646 = {"22646", &P22646_4};	/*Profile ID - 22646*/
const Value V22582_1_1 = {S2, NULL};	/*Value = True*/
const Property P22582_1 = {S50, 0, &V22582_1_1, NULL};	/*Property = IsMobile*/
const Value V22582_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22582_5 = {S52, 0, &V22582_5_59138, &P22582_1};	/*Property = ScreenPixelsHeight*/
const Value V22582_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22582_4 = {S54, 0, &V22582_4_59970, &P22582_5};	/*Property = ScreenPixelsWidth*/
const Profile P22582 = {"22582", &P22582_4};	/*Profile ID - 22582*/
const Value V22521_1_1 = {S2, NULL};	/*Value = True*/
const Property P22521_1 = {S50, 0, &V22521_1_1, NULL};	/*Property = IsMobile*/
const Value V22521_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22521_5 = {S52, 0, &V22521_5_1277, &P22521_1};	/*Property = ScreenPixelsHeight*/
const Value V22521_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22521_4 = {S54, 0, &V22521_4_2336, &P22521_5};	/*Property = ScreenPixelsWidth*/
const Profile P22521 = {"22521", &P22521_4};	/*Profile ID - 22521*/
const Value V23162_1_1 = {S2, NULL};	/*Value = True*/
const Property P23162_1 = {S50, 0, &V23162_1_1, NULL};	/*Property = IsMobile*/
const Value V23162_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23162_5 = {S52, 0, &V23162_5_1227, &P23162_1};	/*Property = ScreenPixelsHeight*/
const Value V23162_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23162_4 = {S54, 0, &V23162_4_1229, &P23162_5};	/*Property = ScreenPixelsWidth*/
const Profile P23162 = {"23162", &P23162_4};	/*Profile ID - 23162*/
const Value V22632_1_1 = {S2, NULL};	/*Value = True*/
const Property P22632_1 = {S50, 0, &V22632_1_1, NULL};	/*Property = IsMobile*/
const Value V22632_5_37564 = {S149, NULL};	/*Value = 690*/
const Property P22632_5 = {S52, 0, &V22632_5_37564, &P22632_1};	/*Property = ScreenPixelsHeight*/
const Value V22632_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22632_4 = {S54, 0, &V22632_4_1404, &P22632_5};	/*Property = ScreenPixelsWidth*/
const Profile P22632 = {"22632", &P22632_4};	/*Profile ID - 22632*/
const Value V22936_1_1 = {S2, NULL};	/*Value = True*/
const Property P22936_1 = {S50, 0, &V22936_1_1, NULL};	/*Property = IsMobile*/
const Value V22936_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22936_5 = {S52, 0, &V22936_5_1227, &P22936_1};	/*Property = ScreenPixelsHeight*/
const Value V22936_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22936_4 = {S54, 0, &V22936_4_1229, &P22936_5};	/*Property = ScreenPixelsWidth*/
const Profile P22936 = {"22936", &P22936_4};	/*Profile ID - 22936*/
const Value V20584_1_1 = {S2, NULL};	/*Value = True*/
const Property P20584_1 = {S50, 0, &V20584_1_1, NULL};	/*Property = IsMobile*/
const Value V20584_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20584_5 = {S52, 0, &V20584_5_1277, &P20584_1};	/*Property = ScreenPixelsHeight*/
const Value V20584_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P20584_4 = {S54, 0, &V20584_4_1279, &P20584_5};	/*Property = ScreenPixelsWidth*/
const Profile P20584 = {"20584", &P20584_4};	/*Profile ID - 20584*/
const Value V22597_1_1 = {S2, NULL};	/*Value = True*/
const Property P22597_1 = {S50, 0, &V22597_1_1, NULL};	/*Property = IsMobile*/
const Value V22597_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22597_5 = {S52, 0, &V22597_5_59138, &P22597_1};	/*Property = ScreenPixelsHeight*/
const Value V22597_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22597_4 = {S54, 0, &V22597_4_59970, &P22597_5};	/*Property = ScreenPixelsWidth*/
const Profile P22597 = {"22597", &P22597_4};	/*Profile ID - 22597*/
const Value V22848_1_1 = {S2, NULL};	/*Value = True*/
const Property P22848_1 = {S50, 0, &V22848_1_1, NULL};	/*Property = IsMobile*/
const Value V22848_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22848_5 = {S52, 0, &V22848_5_1227, &P22848_1};	/*Property = ScreenPixelsHeight*/
const Value V22848_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22848_4 = {S54, 0, &V22848_4_1229, &P22848_5};	/*Property = ScreenPixelsWidth*/
const Profile P22848 = {"22848", &P22848_4};	/*Profile ID - 22848*/
const Value V23314_1_1 = {S2, NULL};	/*Value = True*/
const Property P23314_1 = {S50, 0, &V23314_1_1, NULL};	/*Property = IsMobile*/
const Value V23314_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23314_5 = {S52, 0, &V23314_5_9981, &P23314_1};	/*Property = ScreenPixelsHeight*/
const Value V23314_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23314_4 = {S54, 0, &V23314_4_39098, &P23314_5};	/*Property = ScreenPixelsWidth*/
const Profile P23314 = {"23314", &P23314_4};	/*Profile ID - 23314*/
const Value V22261_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22261_279 = {S1, 0, &V22261_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22261_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22261_282 = {S3, 0, &V22261_282_57154, &P22261_279};	/*Property = BlobBuilder*/
const Value V22261_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22261_350 = {S4, 0, &V22261_350_57444, &P22261_282};	/*Property = Canvas*/
const Value V22261_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22261_283 = {S5, 0, &V22261_283_57159, &P22261_350};	/*Property = CssBackground*/
const Value V22261_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22261_284 = {S6, 0, &V22261_284_57162, &P22261_283};	/*Property = CssBorderImage*/
const Value V22261_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22261_285 = {S7, 0, &V22261_285_57166, &P22261_284};	/*Property = CssCanvas*/
const Value V22261_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22261_286 = {S8, 0, &V22261_286_57168, &P22261_285};	/*Property = CssColor*/
const Value V22261_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22261_237 = {S9, 0, &V22261_237_55786, &P22261_286};	/*Property = CssColumn*/
const Value V22261_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22261_287 = {S10, 0, &V22261_287_57172, &P22261_237};	/*Property = CssFlexbox*/
const Value V22261_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22261_288 = {S11, 0, &V22261_288_57175, &P22261_287};	/*Property = CssFont*/
const Value V22261_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22261_289 = {S12, 0, &V22261_289_57759, &P22261_288};	/*Property = CssImages*/
const Value V22261_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22261_291 = {S13, 0, &V22261_291_57180, &P22261_289};	/*Property = CssMediaQueries*/
const Value V22261_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22261_292 = {S14, 0, &V22261_292_57183, &P22261_291};	/*Property = CssMinMax*/
const Value V22261_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22261_293 = {S15, 0, &V22261_293_57187, &P22261_292};	/*Property = CssOverflow*/
const Value V22261_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22261_294 = {S16, 0, &V22261_294_57189, &P22261_293};	/*Property = CssPosition*/
const Value V22261_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22261_295 = {S17, 0, &V22261_295_57192, &P22261_294};	/*Property = CssText*/
const Value V22261_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22261_297 = {S18, 0, &V22261_297_57195, &P22261_295};	/*Property = CssTransforms*/
const Value V22261_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22261_241 = {S19, 0, &V22261_241_55799, &P22261_297};	/*Property = CssTransitions*/
const Value V22261_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22261_298 = {S20, 0, &V22261_298_57206, &P22261_241};	/*Property = CssUI*/
const Value V22261_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22261_300 = {S21, 0, &V22261_300_57215, &P22261_298};	/*Property = DataSet*/
const Value V22261_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22261_301 = {S22, 0, &V22261_301_57218, &P22261_300};	/*Property = DataUrl*/
const Value V22261_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22261_302 = {S23, 0, &V22261_302_57221, &P22261_301};	/*Property = DeviceOrientation*/
const Value V22261_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22261_304 = {S24, 0, &V22261_304_57227, &P22261_302};	/*Property = FileReader*/
const Value V22261_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22261_305 = {S25, 0, &V22261_305_57231, &P22261_304};	/*Property = FileSaver*/
const Value V22261_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22261_306 = {S26, 0, &V22261_306_57237, &P22261_305};	/*Property = FileWriter*/
const Value V22261_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22261_307 = {S27, 0, &V22261_307_57239, &P22261_306};	/*Property = FormData*/
const Value V22261_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22261_309 = {S28, 0, &V22261_309_57246, &P22261_307};	/*Property = Fullscreen*/
const Value V22261_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22261_310 = {S29, 0, &V22261_310_57248, &P22261_309};	/*Property = GeoLocation*/
const Value V22261_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22261_312 = {S30, 0, &V22261_312_57254, &P22261_310};	/*Property = History*/
const Value V22261_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22261_314 = {S31, 0, &V22261_314_57261, &P22261_312};	/*Property = Html5*/
const Value V22261_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22261_313 = {S32, 0, &V22261_313_57258, &P22261_314};	/*Property = Html-Media-Capture*/
const Value V22261_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22261_315 = {S33, 0, &V22261_315_57264, &P22261_313};	/*Property = Iframe*/
const Value V22261_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22261_316 = {S34, 0, &V22261_316_57267, &P22261_315};	/*Property = IndexedDB*/
const Value V22261_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22261_317 = {S35, 0, &V22261_317_57269, &P22261_316};	/*Property = Json*/
const Value V22261_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22261_15 = {S37, 0, &V22261_15_28, &P22261_317};	/*Property = LayoutEngine*/
const Value V22261_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22261_344 = {S38, 0, &V22261_344_57272, &P22261_15};	/*Property = Masking*/
const Value V22261_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22261_320 = {S39, 0, &V22261_320_57290, &P22261_344};	/*Property = PostMessage*/
const Value V22261_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22261_321 = {S40, 0, &V22261_321_57293, &P22261_320};	/*Property = Progress*/
const Value V22261_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22261_322 = {S41, 0, &V22261_322_57296, &P22261_321};	/*Property = Prompts*/
const Value V22261_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22261_323 = {S42, 0, &V22261_323_57299, &P22261_322};	/*Property = Selector*/
const Value V22261_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22261_324 = {S43, 0, &V22261_324_57306, &P22261_323};	/*Property = Svg*/
const Value V22261_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22261_327 = {S44, 0, &V22261_327_57314, &P22261_324};	/*Property = TouchEvents*/
const Value V22261_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22261_328 = {S45, 0, &V22261_328_57318, &P22261_327};	/*Property = Track*/
const Value V22261_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22261_329 = {S46, 0, &V22261_329_57323, &P22261_328};	/*Property = Video*/
const Value V22261_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22261_330 = {S47, 0, &V22261_330_57326, &P22261_329};	/*Property = Viewport*/
const Value V22261_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22261_332 = {S48, 0, &V22261_332_57339, &P22261_330};	/*Property = WebWorkers*/
const Value V22261_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22261_333 = {S49, 0, &V22261_333_57342, &P22261_332};	/*Property = Xhr2*/
const Profile P22261 = {"22261", &P22261_333};	/*Profile ID - 22261*/
const Value V22281_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_279 = {S1, 0, &V22281_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22281_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_282 = {S3, 0, &V22281_282_57155, &P22281_279};	/*Property = BlobBuilder*/
const Value V22281_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_350 = {S4, 0, &V22281_350_57446, &P22281_282};	/*Property = Canvas*/
const Value V22281_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_283 = {S5, 0, &V22281_283_57161, &P22281_350};	/*Property = CssBackground*/
const Value V22281_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_284 = {S6, 0, &V22281_284_57164, &P22281_283};	/*Property = CssBorderImage*/
const Value V22281_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_285 = {S7, 0, &V22281_285_57167, &P22281_284};	/*Property = CssCanvas*/
const Value V22281_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_286 = {S8, 0, &V22281_286_57170, &P22281_285};	/*Property = CssColor*/
const Value V22281_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_237 = {S9, 0, &V22281_237_55786, &P22281_286};	/*Property = CssColumn*/
const Value V22281_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_287 = {S10, 0, &V22281_287_57173, &P22281_237};	/*Property = CssFlexbox*/
const Value V22281_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_288 = {S11, 0, &V22281_288_57176, &P22281_287};	/*Property = CssFont*/
const Value V22281_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_289 = {S12, 0, &V22281_289_86899, &P22281_288};	/*Property = CssImages*/
const Value V22281_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_291 = {S13, 0, &V22281_291_57182, &P22281_289};	/*Property = CssMediaQueries*/
const Value V22281_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_292 = {S14, 0, &V22281_292_57185, &P22281_291};	/*Property = CssMinMax*/
const Value V22281_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_293 = {S15, 0, &V22281_293_57188, &P22281_292};	/*Property = CssOverflow*/
const Value V22281_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_294 = {S16, 0, &V22281_294_57191, &P22281_293};	/*Property = CssPosition*/
const Value V22281_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_295 = {S17, 0, &V22281_295_57194, &P22281_294};	/*Property = CssText*/
const Value V22281_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_297 = {S18, 0, &V22281_297_57197, &P22281_295};	/*Property = CssTransforms*/
const Value V22281_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_241 = {S19, 0, &V22281_241_55801, &P22281_297};	/*Property = CssTransitions*/
const Value V22281_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_298 = {S20, 0, &V22281_298_57208, &P22281_241};	/*Property = CssUI*/
const Value V22281_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_300 = {S21, 0, &V22281_300_57217, &P22281_298};	/*Property = DataSet*/
const Value V22281_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_301 = {S22, 0, &V22281_301_57220, &P22281_300};	/*Property = DataUrl*/
const Value V22281_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_302 = {S23, 0, &V22281_302_57223, &P22281_301};	/*Property = DeviceOrientation*/
const Value V22281_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_304 = {S24, 0, &V22281_304_57229, &P22281_302};	/*Property = FileReader*/
const Value V22281_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_305 = {S25, 0, &V22281_305_57232, &P22281_304};	/*Property = FileSaver*/
const Value V22281_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_306 = {S26, 0, &V22281_306_57238, &P22281_305};	/*Property = FileWriter*/
const Value V22281_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_307 = {S27, 0, &V22281_307_57241, &P22281_306};	/*Property = FormData*/
const Value V22281_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_309 = {S28, 0, &V22281_309_57247, &P22281_307};	/*Property = Fullscreen*/
const Value V22281_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_310 = {S29, 0, &V22281_310_57250, &P22281_309};	/*Property = GeoLocation*/
const Value V22281_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_312 = {S30, 0, &V22281_312_57256, &P22281_310};	/*Property = History*/
const Value V22281_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_314 = {S31, 0, &V22281_314_57262, &P22281_312};	/*Property = Html5*/
const Value V22281_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_313 = {S32, 0, &V22281_313_57259, &P22281_314};	/*Property = Html-Media-Capture*/
const Value V22281_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_315 = {S33, 0, &V22281_315_57265, &P22281_313};	/*Property = Iframe*/
const Value V22281_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_316 = {S34, 0, &V22281_316_57268, &P22281_315};	/*Property = IndexedDB*/
const Value V22281_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_317 = {S35, 0, &V22281_317_57271, &P22281_316};	/*Property = Json*/
const Value V22281_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P22281_15 = {S37, 0, &V22281_15_54167, &P22281_317};	/*Property = LayoutEngine*/
const Value V22281_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_344 = {S38, 0, &V22281_344_57274, &P22281_15};	/*Property = Masking*/
const Value V22281_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_320 = {S39, 0, &V22281_320_57292, &P22281_344};	/*Property = PostMessage*/
const Value V22281_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_321 = {S40, 0, &V22281_321_57295, &P22281_320};	/*Property = Progress*/
const Value V22281_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_322 = {S41, 0, &V22281_322_57298, &P22281_321};	/*Property = Prompts*/
const Value V22281_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_323 = {S42, 0, &V22281_323_57301, &P22281_322};	/*Property = Selector*/
const Value V22281_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_324 = {S43, 0, &V22281_324_57307, &P22281_323};	/*Property = Svg*/
const Value V22281_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_327 = {S44, 0, &V22281_327_57316, &P22281_324};	/*Property = TouchEvents*/
const Value V22281_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_328 = {S45, 0, &V22281_328_57319, &P22281_327};	/*Property = Track*/
const Value V22281_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_329 = {S46, 0, &V22281_329_57325, &P22281_328};	/*Property = Video*/
const Value V22281_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_330 = {S47, 0, &V22281_330_57328, &P22281_329};	/*Property = Viewport*/
const Value V22281_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_332 = {S48, 0, &V22281_332_57340, &P22281_330};	/*Property = WebWorkers*/
const Value V22281_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22281_333 = {S49, 0, &V22281_333_57343, &P22281_332};	/*Property = Xhr2*/
const Profile P22281 = {"22281", &P22281_333};	/*Profile ID - 22281*/
const Value V22329_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22329_279 = {S1, 0, &V22329_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22329_282_57153 = {S2, NULL};	/*Value = True*/
const Property P22329_282 = {S3, 0, &V22329_282_57153, &P22329_279};	/*Property = BlobBuilder*/
const Value V22329_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22329_350 = {S4, 0, &V22329_350_57444, &P22329_282};	/*Property = Canvas*/
const Value V22329_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22329_283 = {S5, 0, &V22329_283_57159, &P22329_350};	/*Property = CssBackground*/
const Value V22329_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22329_284 = {S6, 0, &V22329_284_57163, &P22329_283};	/*Property = CssBorderImage*/
const Value V22329_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22329_285 = {S7, 0, &V22329_285_57166, &P22329_284};	/*Property = CssCanvas*/
const Value V22329_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22329_286 = {S8, 0, &V22329_286_57168, &P22329_285};	/*Property = CssColor*/
const Value V22329_237_55784 = {S2, NULL};	/*Value = True*/
const Property P22329_237 = {S9, 0, &V22329_237_55784, &P22329_286};	/*Property = CssColumn*/
const Value V22329_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22329_287 = {S10, 0, &V22329_287_57172, &P22329_237};	/*Property = CssFlexbox*/
const Value V22329_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22329_288 = {S11, 0, &V22329_288_57175, &P22329_287};	/*Property = CssFont*/
const Value V22329_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22329_289 = {S12, 0, &V22329_289_57759, &P22329_288};	/*Property = CssImages*/
const Value V22329_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22329_291 = {S13, 0, &V22329_291_57180, &P22329_289};	/*Property = CssMediaQueries*/
const Value V22329_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22329_292 = {S14, 0, &V22329_292_57183, &P22329_291};	/*Property = CssMinMax*/
const Value V22329_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22329_293 = {S15, 0, &V22329_293_57187, &P22329_292};	/*Property = CssOverflow*/
const Value V22329_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22329_294 = {S16, 0, &V22329_294_57189, &P22329_293};	/*Property = CssPosition*/
const Value V22329_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22329_295 = {S17, 0, &V22329_295_57192, &P22329_294};	/*Property = CssText*/
const Value V22329_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22329_297 = {S18, 0, &V22329_297_57195, &P22329_295};	/*Property = CssTransforms*/
const Value V22329_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22329_241 = {S19, 0, &V22329_241_55799, &P22329_297};	/*Property = CssTransitions*/
const Value V22329_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22329_298 = {S20, 0, &V22329_298_57206, &P22329_241};	/*Property = CssUI*/
const Value V22329_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22329_300 = {S21, 0, &V22329_300_57216, &P22329_298};	/*Property = DataSet*/
const Value V22329_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22329_301 = {S22, 0, &V22329_301_57218, &P22329_300};	/*Property = DataUrl*/
const Value V22329_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22329_302 = {S23, 0, &V22329_302_57222, &P22329_301};	/*Property = DeviceOrientation*/
const Value V22329_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22329_304 = {S24, 0, &V22329_304_57227, &P22329_302};	/*Property = FileReader*/
const Value V22329_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22329_305 = {S25, 0, &V22329_305_57231, &P22329_304};	/*Property = FileSaver*/
const Value V22329_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22329_306 = {S26, 0, &V22329_306_57237, &P22329_305};	/*Property = FileWriter*/
const Value V22329_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22329_307 = {S27, 0, &V22329_307_57239, &P22329_306};	/*Property = FormData*/
const Value V22329_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22329_309 = {S28, 0, &V22329_309_57246, &P22329_307};	/*Property = Fullscreen*/
const Value V22329_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22329_310 = {S29, 0, &V22329_310_57248, &P22329_309};	/*Property = GeoLocation*/
const Value V22329_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22329_312 = {S30, 0, &V22329_312_57255, &P22329_310};	/*Property = History*/
const Value V22329_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22329_314 = {S31, 0, &V22329_314_57261, &P22329_312};	/*Property = Html5*/
const Value V22329_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22329_313 = {S32, 0, &V22329_313_57258, &P22329_314};	/*Property = Html-Media-Capture*/
const Value V22329_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22329_315 = {S33, 0, &V22329_315_57264, &P22329_313};	/*Property = Iframe*/
const Value V22329_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22329_316 = {S34, 0, &V22329_316_57266, &P22329_315};	/*Property = IndexedDB*/
const Value V22329_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22329_317 = {S35, 0, &V22329_317_57269, &P22329_316};	/*Property = Json*/
const Value V22329_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22329_15 = {S37, 0, &V22329_15_28, &P22329_317};	/*Property = LayoutEngine*/
const Value V22329_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22329_344 = {S38, 0, &V22329_344_57272, &P22329_15};	/*Property = Masking*/
const Value V22329_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22329_320 = {S39, 0, &V22329_320_57290, &P22329_344};	/*Property = PostMessage*/
const Value V22329_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22329_321 = {S40, 0, &V22329_321_57293, &P22329_320};	/*Property = Progress*/
const Value V22329_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22329_322 = {S41, 0, &V22329_322_57296, &P22329_321};	/*Property = Prompts*/
const Value V22329_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22329_323 = {S42, 0, &V22329_323_57299, &P22329_322};	/*Property = Selector*/
const Value V22329_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22329_324 = {S43, 0, &V22329_324_57306, &P22329_323};	/*Property = Svg*/
const Value V22329_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22329_327 = {S44, 0, &V22329_327_57315, &P22329_324};	/*Property = TouchEvents*/
const Value V22329_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22329_328 = {S45, 0, &V22329_328_57318, &P22329_327};	/*Property = Track*/
const Value V22329_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22329_329 = {S46, 0, &V22329_329_57323, &P22329_328};	/*Property = Video*/
const Value V22329_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22329_330 = {S47, 0, &V22329_330_57326, &P22329_329};	/*Property = Viewport*/
const Value V22329_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22329_332 = {S48, 0, &V22329_332_57338, &P22329_330};	/*Property = WebWorkers*/
const Value V22329_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22329_333 = {S49, 0, &V22329_333_57342, &P22329_332};	/*Property = Xhr2*/
const Profile P22329 = {"22329", &P22329_333};	/*Profile ID - 22329*/
const Value V22760_1_1 = {S2, NULL};	/*Value = True*/
const Property P22760_1 = {S50, 0, &V22760_1_1, NULL};	/*Property = IsMobile*/
const Value V22760_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22760_5 = {S52, 0, &V22760_5_9981, &P22760_1};	/*Property = ScreenPixelsHeight*/
const Value V22760_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22760_4 = {S54, 0, &V22760_4_39098, &P22760_5};	/*Property = ScreenPixelsWidth*/
const Profile P22760 = {"22760", &P22760_4};	/*Profile ID - 22760*/
const Value V20743_1_1 = {S2, NULL};	/*Value = True*/
const Property P20743_1 = {S50, 0, &V20743_1_1, NULL};	/*Property = IsMobile*/
const Value V20743_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20743_5 = {S52, 0, &V20743_5_1227, &P20743_1};	/*Property = ScreenPixelsHeight*/
const Value V20743_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20743_4 = {S54, 0, &V20743_4_1229, &P20743_5};	/*Property = ScreenPixelsWidth*/
const Profile P20743 = {"20743", &P20743_4};	/*Profile ID - 20743*/
const Value V22594_1_1 = {S2, NULL};	/*Value = True*/
const Property P22594_1 = {S50, 0, &V22594_1_1, NULL};	/*Property = IsMobile*/
const Value V22594_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22594_5 = {S52, 0, &V22594_5_59138, &P22594_1};	/*Property = ScreenPixelsHeight*/
const Value V22594_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22594_4 = {S54, 0, &V22594_4_59970, &P22594_5};	/*Property = ScreenPixelsWidth*/
const Profile P22594 = {"22594", &P22594_4};	/*Profile ID - 22594*/
const Profile P22276 = {"22276", NULL};	/*Profile ID - 22276*/
const Value V22621_1_1 = {S2, NULL};	/*Value = True*/
const Property P22621_1 = {S50, 0, &V22621_1_1, NULL};	/*Property = IsMobile*/
const Value V22621_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22621_5 = {S52, 0, &V22621_5_1227, &P22621_1};	/*Property = ScreenPixelsHeight*/
const Value V22621_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22621_4 = {S54, 0, &V22621_4_1229, &P22621_5};	/*Property = ScreenPixelsWidth*/
const Profile P22621 = {"22621", &P22621_4};	/*Profile ID - 22621*/
const Value V22626_1_1 = {S2, NULL};	/*Value = True*/
const Property P22626_1 = {S50, 0, &V22626_1_1, NULL};	/*Property = IsMobile*/
const Value V22626_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22626_5 = {S52, 0, &V22626_5_1277, &P22626_1};	/*Property = ScreenPixelsHeight*/
const Value V22626_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22626_4 = {S54, 0, &V22626_4_1279, &P22626_5};	/*Property = ScreenPixelsWidth*/
const Profile P22626 = {"22626", &P22626_4};	/*Profile ID - 22626*/
const Value V19762_1_1 = {S2, NULL};	/*Value = True*/
const Property P19762_1 = {S50, 0, &V19762_1_1, NULL};	/*Property = IsMobile*/
const Value V19762_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19762_5 = {S52, 0, &V19762_5_9981, &P19762_1};	/*Property = ScreenPixelsHeight*/
const Value V19762_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P19762_4 = {S54, 0, &V19762_4_39098, &P19762_5};	/*Property = ScreenPixelsWidth*/
const Profile P19762 = {"19762", &P19762_4};	/*Profile ID - 19762*/
const Value V23152_1_1 = {S2, NULL};	/*Value = True*/
const Property P23152_1 = {S50, 0, &V23152_1_1, NULL};	/*Property = IsMobile*/
const Value V23152_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23152_5 = {S52, 0, &V23152_5_1277, &P23152_1};	/*Property = ScreenPixelsHeight*/
const Value V23152_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23152_4 = {S54, 0, &V23152_4_1279, &P23152_5};	/*Property = ScreenPixelsWidth*/
const Profile P23152 = {"23152", &P23152_4};	/*Profile ID - 23152*/
const Value V22224_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22224_279 = {S1, 0, &V22224_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22224_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22224_282 = {S3, 0, &V22224_282_57154, &P22224_279};	/*Property = BlobBuilder*/
const Value V22224_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22224_350 = {S4, 0, &V22224_350_57444, &P22224_282};	/*Property = Canvas*/
const Value V22224_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22224_283 = {S5, 0, &V22224_283_57159, &P22224_350};	/*Property = CssBackground*/
const Value V22224_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22224_284 = {S6, 0, &V22224_284_57162, &P22224_283};	/*Property = CssBorderImage*/
const Value V22224_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22224_285 = {S7, 0, &V22224_285_57166, &P22224_284};	/*Property = CssCanvas*/
const Value V22224_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22224_286 = {S8, 0, &V22224_286_57168, &P22224_285};	/*Property = CssColor*/
const Value V22224_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22224_237 = {S9, 0, &V22224_237_55786, &P22224_286};	/*Property = CssColumn*/
const Value V22224_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22224_287 = {S10, 0, &V22224_287_57172, &P22224_237};	/*Property = CssFlexbox*/
const Value V22224_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22224_288 = {S11, 0, &V22224_288_57174, &P22224_287};	/*Property = CssFont*/
const Value V22224_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22224_289 = {S12, 0, &V22224_289_57759, &P22224_288};	/*Property = CssImages*/
const Value V22224_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22224_291 = {S13, 0, &V22224_291_57180, &P22224_289};	/*Property = CssMediaQueries*/
const Value V22224_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22224_292 = {S14, 0, &V22224_292_57183, &P22224_291};	/*Property = CssMinMax*/
const Value V22224_293_57186 = {S2, NULL};	/*Value = True*/
const Property P22224_293 = {S15, 0, &V22224_293_57186, &P22224_292};	/*Property = CssOverflow*/
const Value V22224_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22224_294 = {S16, 0, &V22224_294_57189, &P22224_293};	/*Property = CssPosition*/
const Value V22224_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22224_295 = {S17, 0, &V22224_295_57192, &P22224_294};	/*Property = CssText*/
const Value V22224_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22224_297 = {S18, 0, &V22224_297_57195, &P22224_295};	/*Property = CssTransforms*/
const Value V22224_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22224_241 = {S19, 0, &V22224_241_55799, &P22224_297};	/*Property = CssTransitions*/
const Value V22224_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22224_298 = {S20, 0, &V22224_298_57206, &P22224_241};	/*Property = CssUI*/
const Value V22224_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22224_300 = {S21, 0, &V22224_300_57215, &P22224_298};	/*Property = DataSet*/
const Value V22224_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22224_301 = {S22, 0, &V22224_301_57218, &P22224_300};	/*Property = DataUrl*/
const Value V22224_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22224_302 = {S23, 0, &V22224_302_57221, &P22224_301};	/*Property = DeviceOrientation*/
const Value V22224_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22224_304 = {S24, 0, &V22224_304_57227, &P22224_302};	/*Property = FileReader*/
const Value V22224_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22224_305 = {S25, 0, &V22224_305_57231, &P22224_304};	/*Property = FileSaver*/
const Value V22224_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22224_306 = {S26, 0, &V22224_306_57237, &P22224_305};	/*Property = FileWriter*/
const Value V22224_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22224_307 = {S27, 0, &V22224_307_57239, &P22224_306};	/*Property = FormData*/
const Value V22224_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22224_309 = {S28, 0, &V22224_309_57246, &P22224_307};	/*Property = Fullscreen*/
const Value V22224_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22224_310 = {S29, 0, &V22224_310_57248, &P22224_309};	/*Property = GeoLocation*/
const Value V22224_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22224_312 = {S30, 0, &V22224_312_57254, &P22224_310};	/*Property = History*/
const Value V22224_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22224_314 = {S31, 0, &V22224_314_57260, &P22224_312};	/*Property = Html5*/
const Value V22224_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22224_313 = {S32, 0, &V22224_313_57258, &P22224_314};	/*Property = Html-Media-Capture*/
const Value V22224_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22224_315 = {S33, 0, &V22224_315_57264, &P22224_313};	/*Property = Iframe*/
const Value V22224_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22224_316 = {S34, 0, &V22224_316_57267, &P22224_315};	/*Property = IndexedDB*/
const Value V22224_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22224_317 = {S35, 0, &V22224_317_57269, &P22224_316};	/*Property = Json*/
const Value V22224_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22224_15 = {S37, 0, &V22224_15_28, &P22224_317};	/*Property = LayoutEngine*/
const Value V22224_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22224_344 = {S38, 0, &V22224_344_57272, &P22224_15};	/*Property = Masking*/
const Value V22224_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22224_320 = {S39, 0, &V22224_320_57290, &P22224_344};	/*Property = PostMessage*/
const Value V22224_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22224_321 = {S40, 0, &V22224_321_57293, &P22224_320};	/*Property = Progress*/
const Value V22224_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22224_322 = {S41, 0, &V22224_322_57296, &P22224_321};	/*Property = Prompts*/
const Value V22224_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22224_323 = {S42, 0, &V22224_323_57299, &P22224_322};	/*Property = Selector*/
const Value V22224_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22224_324 = {S43, 0, &V22224_324_57305, &P22224_323};	/*Property = Svg*/
const Value V22224_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22224_327 = {S44, 0, &V22224_327_57314, &P22224_324};	/*Property = TouchEvents*/
const Value V22224_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22224_328 = {S45, 0, &V22224_328_57318, &P22224_327};	/*Property = Track*/
const Value V22224_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22224_329 = {S46, 0, &V22224_329_57323, &P22224_328};	/*Property = Video*/
const Value V22224_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22224_330 = {S47, 0, &V22224_330_57326, &P22224_329};	/*Property = Viewport*/
const Value V22224_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22224_332 = {S48, 0, &V22224_332_57339, &P22224_330};	/*Property = WebWorkers*/
const Value V22224_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22224_333 = {S49, 0, &V22224_333_57342, &P22224_332};	/*Property = Xhr2*/
const Profile P22224 = {"22224", &P22224_333};	/*Profile ID - 22224*/
const Value V22904_1_1 = {S2, NULL};	/*Value = True*/
const Property P22904_1 = {S50, 0, &V22904_1_1, NULL};	/*Property = IsMobile*/
const Value V22904_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22904_5 = {S52, 0, &V22904_5_1227, &P22904_1};	/*Property = ScreenPixelsHeight*/
const Value V22904_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22904_4 = {S54, 0, &V22904_4_1460, &P22904_5};	/*Property = ScreenPixelsWidth*/
const Profile P22904 = {"22904", &P22904_4};	/*Profile ID - 22904*/
const Value V22865_1_1 = {S2, NULL};	/*Value = True*/
const Property P22865_1 = {S50, 0, &V22865_1_1, NULL};	/*Property = IsMobile*/
const Value V22865_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22865_5 = {S52, 0, &V22865_5_1403, &P22865_1};	/*Property = ScreenPixelsHeight*/
const Value V22865_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22865_4 = {S54, 0, &V22865_4_1404, &P22865_5};	/*Property = ScreenPixelsWidth*/
const Profile P22865 = {"22865", &P22865_4};	/*Profile ID - 22865*/
const Value V24428_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_279 = {S1, 0, &V24428_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24428_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_282 = {S3, 0, &V24428_282_57155, &P24428_279};	/*Property = BlobBuilder*/
const Value V24428_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_350 = {S4, 0, &V24428_350_57446, &P24428_282};	/*Property = Canvas*/
const Value V24428_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_283 = {S5, 0, &V24428_283_57161, &P24428_350};	/*Property = CssBackground*/
const Value V24428_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_284 = {S6, 0, &V24428_284_57164, &P24428_283};	/*Property = CssBorderImage*/
const Value V24428_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_285 = {S7, 0, &V24428_285_57167, &P24428_284};	/*Property = CssCanvas*/
const Value V24428_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_286 = {S8, 0, &V24428_286_57170, &P24428_285};	/*Property = CssColor*/
const Value V24428_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_237 = {S9, 0, &V24428_237_55786, &P24428_286};	/*Property = CssColumn*/
const Value V24428_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_287 = {S10, 0, &V24428_287_57173, &P24428_237};	/*Property = CssFlexbox*/
const Value V24428_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_288 = {S11, 0, &V24428_288_57176, &P24428_287};	/*Property = CssFont*/
const Value V24428_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_289 = {S12, 0, &V24428_289_86899, &P24428_288};	/*Property = CssImages*/
const Value V24428_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_291 = {S13, 0, &V24428_291_57182, &P24428_289};	/*Property = CssMediaQueries*/
const Value V24428_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_292 = {S14, 0, &V24428_292_57185, &P24428_291};	/*Property = CssMinMax*/
const Value V24428_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_293 = {S15, 0, &V24428_293_57188, &P24428_292};	/*Property = CssOverflow*/
const Value V24428_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_294 = {S16, 0, &V24428_294_57191, &P24428_293};	/*Property = CssPosition*/
const Value V24428_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_295 = {S17, 0, &V24428_295_57194, &P24428_294};	/*Property = CssText*/
const Value V24428_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_297 = {S18, 0, &V24428_297_57197, &P24428_295};	/*Property = CssTransforms*/
const Value V24428_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_241 = {S19, 0, &V24428_241_55801, &P24428_297};	/*Property = CssTransitions*/
const Value V24428_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_298 = {S20, 0, &V24428_298_57208, &P24428_241};	/*Property = CssUI*/
const Value V24428_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_300 = {S21, 0, &V24428_300_57217, &P24428_298};	/*Property = DataSet*/
const Value V24428_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_301 = {S22, 0, &V24428_301_57220, &P24428_300};	/*Property = DataUrl*/
const Value V24428_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_302 = {S23, 0, &V24428_302_57223, &P24428_301};	/*Property = DeviceOrientation*/
const Value V24428_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_304 = {S24, 0, &V24428_304_57229, &P24428_302};	/*Property = FileReader*/
const Value V24428_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_305 = {S25, 0, &V24428_305_57232, &P24428_304};	/*Property = FileSaver*/
const Value V24428_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_306 = {S26, 0, &V24428_306_57238, &P24428_305};	/*Property = FileWriter*/
const Value V24428_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_307 = {S27, 0, &V24428_307_57241, &P24428_306};	/*Property = FormData*/
const Value V24428_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_309 = {S28, 0, &V24428_309_57247, &P24428_307};	/*Property = Fullscreen*/
const Value V24428_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_310 = {S29, 0, &V24428_310_57250, &P24428_309};	/*Property = GeoLocation*/
const Value V24428_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_312 = {S30, 0, &V24428_312_57256, &P24428_310};	/*Property = History*/
const Value V24428_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_314 = {S31, 0, &V24428_314_57262, &P24428_312};	/*Property = Html5*/
const Value V24428_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_313 = {S32, 0, &V24428_313_57259, &P24428_314};	/*Property = Html-Media-Capture*/
const Value V24428_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_315 = {S33, 0, &V24428_315_57265, &P24428_313};	/*Property = Iframe*/
const Value V24428_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_316 = {S34, 0, &V24428_316_57268, &P24428_315};	/*Property = IndexedDB*/
const Value V24428_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_317 = {S35, 0, &V24428_317_57271, &P24428_316};	/*Property = Json*/
const Value V24428_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24428_15 = {S37, 0, &V24428_15_28, &P24428_317};	/*Property = LayoutEngine*/
const Value V24428_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_344 = {S38, 0, &V24428_344_57274, &P24428_15};	/*Property = Masking*/
const Value V24428_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_320 = {S39, 0, &V24428_320_57292, &P24428_344};	/*Property = PostMessage*/
const Value V24428_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_321 = {S40, 0, &V24428_321_57295, &P24428_320};	/*Property = Progress*/
const Value V24428_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_322 = {S41, 0, &V24428_322_57298, &P24428_321};	/*Property = Prompts*/
const Value V24428_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_323 = {S42, 0, &V24428_323_57301, &P24428_322};	/*Property = Selector*/
const Value V24428_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_324 = {S43, 0, &V24428_324_57307, &P24428_323};	/*Property = Svg*/
const Value V24428_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_327 = {S44, 0, &V24428_327_57316, &P24428_324};	/*Property = TouchEvents*/
const Value V24428_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_328 = {S45, 0, &V24428_328_57319, &P24428_327};	/*Property = Track*/
const Value V24428_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_329 = {S46, 0, &V24428_329_57325, &P24428_328};	/*Property = Video*/
const Value V24428_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_330 = {S47, 0, &V24428_330_57328, &P24428_329};	/*Property = Viewport*/
const Value V24428_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_332 = {S48, 0, &V24428_332_57340, &P24428_330};	/*Property = WebWorkers*/
const Value V24428_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24428_333 = {S49, 0, &V24428_333_57343, &P24428_332};	/*Property = Xhr2*/
const Profile P24428 = {"24428", &P24428_333};	/*Profile ID - 24428*/
const Value V24342_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24342_279 = {S1, 0, &V24342_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24342_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24342_282 = {S3, 0, &V24342_282_57154, &P24342_279};	/*Property = BlobBuilder*/
const Value V24342_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24342_350 = {S4, 0, &V24342_350_57444, &P24342_282};	/*Property = Canvas*/
const Value V24342_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24342_283 = {S5, 0, &V24342_283_57159, &P24342_350};	/*Property = CssBackground*/
const Value V24342_284_57163 = {S0, NULL};	/*Value = False*/
const Property P24342_284 = {S6, 0, &V24342_284_57163, &P24342_283};	/*Property = CssBorderImage*/
const Value V24342_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24342_285 = {S7, 0, &V24342_285_57166, &P24342_284};	/*Property = CssCanvas*/
const Value V24342_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24342_286 = {S8, 0, &V24342_286_57168, &P24342_285};	/*Property = CssColor*/
const Value V24342_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24342_237 = {S9, 0, &V24342_237_55786, &P24342_286};	/*Property = CssColumn*/
const Value V24342_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24342_287 = {S10, 0, &V24342_287_57172, &P24342_237};	/*Property = CssFlexbox*/
const Value V24342_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24342_288 = {S11, 0, &V24342_288_57175, &P24342_287};	/*Property = CssFont*/
const Value V24342_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24342_289 = {S12, 0, &V24342_289_57759, &P24342_288};	/*Property = CssImages*/
const Value V24342_291_57181 = {S0, NULL};	/*Value = False*/
const Property P24342_291 = {S13, 0, &V24342_291_57181, &P24342_289};	/*Property = CssMediaQueries*/
const Value V24342_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24342_292 = {S14, 0, &V24342_292_57183, &P24342_291};	/*Property = CssMinMax*/
const Value V24342_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24342_293 = {S15, 0, &V24342_293_57187, &P24342_292};	/*Property = CssOverflow*/
const Value V24342_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24342_294 = {S16, 0, &V24342_294_57189, &P24342_293};	/*Property = CssPosition*/
const Value V24342_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24342_295 = {S17, 0, &V24342_295_57192, &P24342_294};	/*Property = CssText*/
const Value V24342_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24342_297 = {S18, 0, &V24342_297_57195, &P24342_295};	/*Property = CssTransforms*/
const Value V24342_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24342_241 = {S19, 0, &V24342_241_55799, &P24342_297};	/*Property = CssTransitions*/
const Value V24342_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24342_298 = {S20, 0, &V24342_298_57206, &P24342_241};	/*Property = CssUI*/
const Value V24342_300_57216 = {S0, NULL};	/*Value = False*/
const Property P24342_300 = {S21, 0, &V24342_300_57216, &P24342_298};	/*Property = DataSet*/
const Value V24342_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24342_301 = {S22, 0, &V24342_301_57218, &P24342_300};	/*Property = DataUrl*/
const Value V24342_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24342_302 = {S23, 0, &V24342_302_57222, &P24342_301};	/*Property = DeviceOrientation*/
const Value V24342_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24342_304 = {S24, 0, &V24342_304_57227, &P24342_302};	/*Property = FileReader*/
const Value V24342_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24342_305 = {S25, 0, &V24342_305_57231, &P24342_304};	/*Property = FileSaver*/
const Value V24342_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24342_306 = {S26, 0, &V24342_306_57237, &P24342_305};	/*Property = FileWriter*/
const Value V24342_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24342_307 = {S27, 0, &V24342_307_57239, &P24342_306};	/*Property = FormData*/
const Value V24342_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24342_309 = {S28, 0, &V24342_309_57246, &P24342_307};	/*Property = Fullscreen*/
const Value V24342_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24342_310 = {S29, 0, &V24342_310_57248, &P24342_309};	/*Property = GeoLocation*/
const Value V24342_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24342_312 = {S30, 0, &V24342_312_57254, &P24342_310};	/*Property = History*/
const Value V24342_314_57261 = {S0, NULL};	/*Value = False*/
const Property P24342_314 = {S31, 0, &V24342_314_57261, &P24342_312};	/*Property = Html5*/
const Value V24342_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24342_313 = {S32, 0, &V24342_313_57258, &P24342_314};	/*Property = Html-Media-Capture*/
const Value V24342_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24342_315 = {S33, 0, &V24342_315_57264, &P24342_313};	/*Property = Iframe*/
const Value V24342_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24342_316 = {S34, 0, &V24342_316_57266, &P24342_315};	/*Property = IndexedDB*/
const Value V24342_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24342_317 = {S35, 0, &V24342_317_57269, &P24342_316};	/*Property = Json*/
const Value V24342_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24342_15 = {S37, 0, &V24342_15_54167, &P24342_317};	/*Property = LayoutEngine*/
const Value V24342_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24342_344 = {S38, 0, &V24342_344_57272, &P24342_15};	/*Property = Masking*/
const Value V24342_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24342_320 = {S39, 0, &V24342_320_57290, &P24342_344};	/*Property = PostMessage*/
const Value V24342_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24342_321 = {S40, 0, &V24342_321_57293, &P24342_320};	/*Property = Progress*/
const Value V24342_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24342_322 = {S41, 0, &V24342_322_57296, &P24342_321};	/*Property = Prompts*/
const Value V24342_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24342_323 = {S42, 0, &V24342_323_57299, &P24342_322};	/*Property = Selector*/
const Value V24342_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24342_324 = {S43, 0, &V24342_324_57305, &P24342_323};	/*Property = Svg*/
const Value V24342_327_57315 = {S0, NULL};	/*Value = False*/
const Property P24342_327 = {S44, 0, &V24342_327_57315, &P24342_324};	/*Property = TouchEvents*/
const Value V24342_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24342_328 = {S45, 0, &V24342_328_57318, &P24342_327};	/*Property = Track*/
const Value V24342_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24342_329 = {S46, 0, &V24342_329_57323, &P24342_328};	/*Property = Video*/
const Value V24342_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24342_330 = {S47, 0, &V24342_330_57326, &P24342_329};	/*Property = Viewport*/
const Value V24342_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24342_332 = {S48, 0, &V24342_332_57339, &P24342_330};	/*Property = WebWorkers*/
const Value V24342_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24342_333 = {S49, 0, &V24342_333_57342, &P24342_332};	/*Property = Xhr2*/
const Profile P24342 = {"24342", &P24342_333};	/*Profile ID - 24342*/
const Value V24344_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24344_279 = {S1, 0, &V24344_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24344_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24344_282 = {S3, 0, &V24344_282_57154, &P24344_279};	/*Property = BlobBuilder*/
const Value V24344_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24344_350 = {S4, 0, &V24344_350_57444, &P24344_282};	/*Property = Canvas*/
const Value V24344_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24344_283 = {S5, 0, &V24344_283_57159, &P24344_350};	/*Property = CssBackground*/
const Value V24344_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24344_284 = {S6, 0, &V24344_284_57162, &P24344_283};	/*Property = CssBorderImage*/
const Value V24344_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24344_285 = {S7, 0, &V24344_285_57166, &P24344_284};	/*Property = CssCanvas*/
const Value V24344_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24344_286 = {S8, 0, &V24344_286_57168, &P24344_285};	/*Property = CssColor*/
const Value V24344_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24344_237 = {S9, 0, &V24344_237_55786, &P24344_286};	/*Property = CssColumn*/
const Value V24344_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24344_287 = {S10, 0, &V24344_287_57172, &P24344_237};	/*Property = CssFlexbox*/
const Value V24344_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24344_288 = {S11, 0, &V24344_288_57174, &P24344_287};	/*Property = CssFont*/
const Value V24344_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24344_289 = {S12, 0, &V24344_289_57759, &P24344_288};	/*Property = CssImages*/
const Value V24344_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24344_291 = {S13, 0, &V24344_291_57180, &P24344_289};	/*Property = CssMediaQueries*/
const Value V24344_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24344_292 = {S14, 0, &V24344_292_57183, &P24344_291};	/*Property = CssMinMax*/
const Value V24344_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24344_293 = {S15, 0, &V24344_293_57187, &P24344_292};	/*Property = CssOverflow*/
const Value V24344_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24344_294 = {S16, 0, &V24344_294_57189, &P24344_293};	/*Property = CssPosition*/
const Value V24344_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24344_295 = {S17, 0, &V24344_295_57192, &P24344_294};	/*Property = CssText*/
const Value V24344_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24344_297 = {S18, 0, &V24344_297_57195, &P24344_295};	/*Property = CssTransforms*/
const Value V24344_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24344_241 = {S19, 0, &V24344_241_55799, &P24344_297};	/*Property = CssTransitions*/
const Value V24344_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24344_298 = {S20, 0, &V24344_298_57206, &P24344_241};	/*Property = CssUI*/
const Value V24344_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24344_300 = {S21, 0, &V24344_300_57215, &P24344_298};	/*Property = DataSet*/
const Value V24344_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24344_301 = {S22, 0, &V24344_301_57218, &P24344_300};	/*Property = DataUrl*/
const Value V24344_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24344_302 = {S23, 0, &V24344_302_57221, &P24344_301};	/*Property = DeviceOrientation*/
const Value V24344_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24344_304 = {S24, 0, &V24344_304_57227, &P24344_302};	/*Property = FileReader*/
const Value V24344_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24344_305 = {S25, 0, &V24344_305_57231, &P24344_304};	/*Property = FileSaver*/
const Value V24344_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24344_306 = {S26, 0, &V24344_306_57237, &P24344_305};	/*Property = FileWriter*/
const Value V24344_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24344_307 = {S27, 0, &V24344_307_57239, &P24344_306};	/*Property = FormData*/
const Value V24344_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24344_309 = {S28, 0, &V24344_309_57246, &P24344_307};	/*Property = Fullscreen*/
const Value V24344_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24344_310 = {S29, 0, &V24344_310_57248, &P24344_309};	/*Property = GeoLocation*/
const Value V24344_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24344_312 = {S30, 0, &V24344_312_57254, &P24344_310};	/*Property = History*/
const Value V24344_314_57261 = {S0, NULL};	/*Value = False*/
const Property P24344_314 = {S31, 0, &V24344_314_57261, &P24344_312};	/*Property = Html5*/
const Value V24344_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24344_313 = {S32, 0, &V24344_313_57258, &P24344_314};	/*Property = Html-Media-Capture*/
const Value V24344_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24344_315 = {S33, 0, &V24344_315_57264, &P24344_313};	/*Property = Iframe*/
const Value V24344_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24344_316 = {S34, 0, &V24344_316_57266, &P24344_315};	/*Property = IndexedDB*/
const Value V24344_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24344_317 = {S35, 0, &V24344_317_57269, &P24344_316};	/*Property = Json*/
const Value V24344_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P24344_15 = {S37, 0, &V24344_15_29, &P24344_317};	/*Property = LayoutEngine*/
const Value V24344_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24344_344 = {S38, 0, &V24344_344_57272, &P24344_15};	/*Property = Masking*/
const Value V24344_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24344_320 = {S39, 0, &V24344_320_57290, &P24344_344};	/*Property = PostMessage*/
const Value V24344_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24344_321 = {S40, 0, &V24344_321_57293, &P24344_320};	/*Property = Progress*/
const Value V24344_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24344_322 = {S41, 0, &V24344_322_57296, &P24344_321};	/*Property = Prompts*/
const Value V24344_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24344_323 = {S42, 0, &V24344_323_57299, &P24344_322};	/*Property = Selector*/
const Value V24344_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24344_324 = {S43, 0, &V24344_324_57305, &P24344_323};	/*Property = Svg*/
const Value V24344_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24344_327 = {S44, 0, &V24344_327_57314, &P24344_324};	/*Property = TouchEvents*/
const Value V24344_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24344_328 = {S45, 0, &V24344_328_57318, &P24344_327};	/*Property = Track*/
const Value V24344_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24344_329 = {S46, 0, &V24344_329_57323, &P24344_328};	/*Property = Video*/
const Value V24344_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24344_330 = {S47, 0, &V24344_330_57326, &P24344_329};	/*Property = Viewport*/
const Value V24344_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24344_332 = {S48, 0, &V24344_332_57338, &P24344_330};	/*Property = WebWorkers*/
const Value V24344_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24344_333 = {S49, 0, &V24344_333_57342, &P24344_332};	/*Property = Xhr2*/
const Profile P24344 = {"24344", &P24344_333};	/*Profile ID - 24344*/
const Value V22902_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22902_279 = {S1, 0, &V22902_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22902_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22902_282 = {S3, 0, &V22902_282_57154, &P22902_279};	/*Property = BlobBuilder*/
const Value V22902_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22902_350 = {S4, 0, &V22902_350_57444, &P22902_282};	/*Property = Canvas*/
const Value V22902_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22902_283 = {S5, 0, &V22902_283_57159, &P22902_350};	/*Property = CssBackground*/
const Value V22902_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22902_284 = {S6, 0, &V22902_284_57162, &P22902_283};	/*Property = CssBorderImage*/
const Value V22902_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22902_285 = {S7, 0, &V22902_285_57166, &P22902_284};	/*Property = CssCanvas*/
const Value V22902_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22902_286 = {S8, 0, &V22902_286_57168, &P22902_285};	/*Property = CssColor*/
const Value V22902_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22902_237 = {S9, 0, &V22902_237_55786, &P22902_286};	/*Property = CssColumn*/
const Value V22902_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22902_287 = {S10, 0, &V22902_287_57172, &P22902_237};	/*Property = CssFlexbox*/
const Value V22902_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22902_288 = {S11, 0, &V22902_288_57174, &P22902_287};	/*Property = CssFont*/
const Value V22902_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22902_289 = {S12, 0, &V22902_289_57759, &P22902_288};	/*Property = CssImages*/
const Value V22902_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22902_291 = {S13, 0, &V22902_291_57180, &P22902_289};	/*Property = CssMediaQueries*/
const Value V22902_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22902_292 = {S14, 0, &V22902_292_57183, &P22902_291};	/*Property = CssMinMax*/
const Value V22902_293_57186 = {S2, NULL};	/*Value = True*/
const Property P22902_293 = {S15, 0, &V22902_293_57186, &P22902_292};	/*Property = CssOverflow*/
const Value V22902_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22902_294 = {S16, 0, &V22902_294_57189, &P22902_293};	/*Property = CssPosition*/
const Value V22902_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22902_295 = {S17, 0, &V22902_295_57192, &P22902_294};	/*Property = CssText*/
const Value V22902_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22902_297 = {S18, 0, &V22902_297_57195, &P22902_295};	/*Property = CssTransforms*/
const Value V22902_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22902_241 = {S19, 0, &V22902_241_55799, &P22902_297};	/*Property = CssTransitions*/
const Value V22902_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22902_298 = {S20, 0, &V22902_298_57206, &P22902_241};	/*Property = CssUI*/
const Value V22902_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22902_300 = {S21, 0, &V22902_300_57215, &P22902_298};	/*Property = DataSet*/
const Value V22902_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22902_301 = {S22, 0, &V22902_301_57218, &P22902_300};	/*Property = DataUrl*/
const Value V22902_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22902_302 = {S23, 0, &V22902_302_57221, &P22902_301};	/*Property = DeviceOrientation*/
const Value V22902_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22902_304 = {S24, 0, &V22902_304_57227, &P22902_302};	/*Property = FileReader*/
const Value V22902_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22902_305 = {S25, 0, &V22902_305_57231, &P22902_304};	/*Property = FileSaver*/
const Value V22902_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22902_306 = {S26, 0, &V22902_306_57237, &P22902_305};	/*Property = FileWriter*/
const Value V22902_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22902_307 = {S27, 0, &V22902_307_57239, &P22902_306};	/*Property = FormData*/
const Value V22902_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22902_309 = {S28, 0, &V22902_309_57246, &P22902_307};	/*Property = Fullscreen*/
const Value V22902_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22902_310 = {S29, 0, &V22902_310_57248, &P22902_309};	/*Property = GeoLocation*/
const Value V22902_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22902_312 = {S30, 0, &V22902_312_57254, &P22902_310};	/*Property = History*/
const Value V22902_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22902_314 = {S31, 0, &V22902_314_57262, &P22902_312};	/*Property = Html5*/
const Value V22902_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22902_313 = {S32, 0, &V22902_313_57258, &P22902_314};	/*Property = Html-Media-Capture*/
const Value V22902_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22902_315 = {S33, 0, &V22902_315_57264, &P22902_313};	/*Property = Iframe*/
const Value V22902_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22902_316 = {S34, 0, &V22902_316_57267, &P22902_315};	/*Property = IndexedDB*/
const Value V22902_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22902_317 = {S35, 0, &V22902_317_57269, &P22902_316};	/*Property = Json*/
const Value V22902_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22902_15 = {S37, 0, &V22902_15_28, &P22902_317};	/*Property = LayoutEngine*/
const Value V22902_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22902_344 = {S38, 0, &V22902_344_57272, &P22902_15};	/*Property = Masking*/
const Value V22902_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22902_320 = {S39, 0, &V22902_320_57290, &P22902_344};	/*Property = PostMessage*/
const Value V22902_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22902_321 = {S40, 0, &V22902_321_57293, &P22902_320};	/*Property = Progress*/
const Value V22902_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22902_322 = {S41, 0, &V22902_322_57296, &P22902_321};	/*Property = Prompts*/
const Value V22902_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22902_323 = {S42, 0, &V22902_323_57299, &P22902_322};	/*Property = Selector*/
const Value V22902_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22902_324 = {S43, 0, &V22902_324_57305, &P22902_323};	/*Property = Svg*/
const Value V22902_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22902_327 = {S44, 0, &V22902_327_57314, &P22902_324};	/*Property = TouchEvents*/
const Value V22902_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22902_328 = {S45, 0, &V22902_328_57318, &P22902_327};	/*Property = Track*/
const Value V22902_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22902_329 = {S46, 0, &V22902_329_57323, &P22902_328};	/*Property = Video*/
const Value V22902_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22902_330 = {S47, 0, &V22902_330_57326, &P22902_329};	/*Property = Viewport*/
const Value V22902_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22902_332 = {S48, 0, &V22902_332_57338, &P22902_330};	/*Property = WebWorkers*/
const Value V22902_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22902_333 = {S49, 0, &V22902_333_57342, &P22902_332};	/*Property = Xhr2*/
const Profile P22902 = {"22902", &P22902_333};	/*Profile ID - 22902*/
const Value V22317_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_279 = {S1, 0, &V22317_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22317_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_282 = {S3, 0, &V22317_282_57155, &P22317_279};	/*Property = BlobBuilder*/
const Value V22317_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_350 = {S4, 0, &V22317_350_57446, &P22317_282};	/*Property = Canvas*/
const Value V22317_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_283 = {S5, 0, &V22317_283_57161, &P22317_350};	/*Property = CssBackground*/
const Value V22317_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_284 = {S6, 0, &V22317_284_57164, &P22317_283};	/*Property = CssBorderImage*/
const Value V22317_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_285 = {S7, 0, &V22317_285_57167, &P22317_284};	/*Property = CssCanvas*/
const Value V22317_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_286 = {S8, 0, &V22317_286_57170, &P22317_285};	/*Property = CssColor*/
const Value V22317_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_237 = {S9, 0, &V22317_237_55786, &P22317_286};	/*Property = CssColumn*/
const Value V22317_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_287 = {S10, 0, &V22317_287_57173, &P22317_237};	/*Property = CssFlexbox*/
const Value V22317_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_288 = {S11, 0, &V22317_288_57176, &P22317_287};	/*Property = CssFont*/
const Value V22317_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_289 = {S12, 0, &V22317_289_86899, &P22317_288};	/*Property = CssImages*/
const Value V22317_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_291 = {S13, 0, &V22317_291_57182, &P22317_289};	/*Property = CssMediaQueries*/
const Value V22317_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_292 = {S14, 0, &V22317_292_57185, &P22317_291};	/*Property = CssMinMax*/
const Value V22317_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_293 = {S15, 0, &V22317_293_57188, &P22317_292};	/*Property = CssOverflow*/
const Value V22317_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_294 = {S16, 0, &V22317_294_57191, &P22317_293};	/*Property = CssPosition*/
const Value V22317_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_295 = {S17, 0, &V22317_295_57194, &P22317_294};	/*Property = CssText*/
const Value V22317_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_297 = {S18, 0, &V22317_297_57197, &P22317_295};	/*Property = CssTransforms*/
const Value V22317_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_241 = {S19, 0, &V22317_241_55801, &P22317_297};	/*Property = CssTransitions*/
const Value V22317_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_298 = {S20, 0, &V22317_298_57208, &P22317_241};	/*Property = CssUI*/
const Value V22317_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_300 = {S21, 0, &V22317_300_57217, &P22317_298};	/*Property = DataSet*/
const Value V22317_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_301 = {S22, 0, &V22317_301_57220, &P22317_300};	/*Property = DataUrl*/
const Value V22317_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_302 = {S23, 0, &V22317_302_57223, &P22317_301};	/*Property = DeviceOrientation*/
const Value V22317_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_304 = {S24, 0, &V22317_304_57229, &P22317_302};	/*Property = FileReader*/
const Value V22317_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_305 = {S25, 0, &V22317_305_57232, &P22317_304};	/*Property = FileSaver*/
const Value V22317_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_306 = {S26, 0, &V22317_306_57238, &P22317_305};	/*Property = FileWriter*/
const Value V22317_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_307 = {S27, 0, &V22317_307_57241, &P22317_306};	/*Property = FormData*/
const Value V22317_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_309 = {S28, 0, &V22317_309_57247, &P22317_307};	/*Property = Fullscreen*/
const Value V22317_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_310 = {S29, 0, &V22317_310_57250, &P22317_309};	/*Property = GeoLocation*/
const Value V22317_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_312 = {S30, 0, &V22317_312_57256, &P22317_310};	/*Property = History*/
const Value V22317_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_314 = {S31, 0, &V22317_314_57262, &P22317_312};	/*Property = Html5*/
const Value V22317_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_313 = {S32, 0, &V22317_313_57259, &P22317_314};	/*Property = Html-Media-Capture*/
const Value V22317_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_315 = {S33, 0, &V22317_315_57265, &P22317_313};	/*Property = Iframe*/
const Value V22317_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_316 = {S34, 0, &V22317_316_57268, &P22317_315};	/*Property = IndexedDB*/
const Value V22317_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_317 = {S35, 0, &V22317_317_57271, &P22317_316};	/*Property = Json*/
const Value V22317_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_15 = {S37, 0, &V22317_15_72, &P22317_317};	/*Property = LayoutEngine*/
const Value V22317_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_344 = {S38, 0, &V22317_344_57274, &P22317_15};	/*Property = Masking*/
const Value V22317_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_320 = {S39, 0, &V22317_320_57292, &P22317_344};	/*Property = PostMessage*/
const Value V22317_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_321 = {S40, 0, &V22317_321_57295, &P22317_320};	/*Property = Progress*/
const Value V22317_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_322 = {S41, 0, &V22317_322_57298, &P22317_321};	/*Property = Prompts*/
const Value V22317_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_323 = {S42, 0, &V22317_323_57301, &P22317_322};	/*Property = Selector*/
const Value V22317_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_324 = {S43, 0, &V22317_324_57307, &P22317_323};	/*Property = Svg*/
const Value V22317_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_327 = {S44, 0, &V22317_327_57316, &P22317_324};	/*Property = TouchEvents*/
const Value V22317_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_328 = {S45, 0, &V22317_328_57319, &P22317_327};	/*Property = Track*/
const Value V22317_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_329 = {S46, 0, &V22317_329_57325, &P22317_328};	/*Property = Video*/
const Value V22317_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_330 = {S47, 0, &V22317_330_57328, &P22317_329};	/*Property = Viewport*/
const Value V22317_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_332 = {S48, 0, &V22317_332_57340, &P22317_330};	/*Property = WebWorkers*/
const Value V22317_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22317_333 = {S49, 0, &V22317_333_57343, &P22317_332};	/*Property = Xhr2*/
const Profile P22317 = {"22317", &P22317_333};	/*Profile ID - 22317*/
const Value V22738_1_1 = {S2, NULL};	/*Value = True*/
const Property P22738_1 = {S50, 0, &V22738_1_1, NULL};	/*Property = IsMobile*/
const Value V22738_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22738_5 = {S52, 0, &V22738_5_1277, &P22738_1};	/*Property = ScreenPixelsHeight*/
const Value V22738_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22738_4 = {S54, 0, &V22738_4_1279, &P22738_5};	/*Property = ScreenPixelsWidth*/
const Profile P22738 = {"22738", &P22738_4};	/*Profile ID - 22738*/
const Value V22282_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22282_279 = {S1, 0, &V22282_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22282_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22282_282 = {S3, 0, &V22282_282_57154, &P22282_279};	/*Property = BlobBuilder*/
const Value V22282_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22282_350 = {S4, 0, &V22282_350_57444, &P22282_282};	/*Property = Canvas*/
const Value V22282_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22282_283 = {S5, 0, &V22282_283_57159, &P22282_350};	/*Property = CssBackground*/
const Value V22282_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22282_284 = {S6, 0, &V22282_284_57163, &P22282_283};	/*Property = CssBorderImage*/
const Value V22282_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22282_285 = {S7, 0, &V22282_285_57166, &P22282_284};	/*Property = CssCanvas*/
const Value V22282_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22282_286 = {S8, 0, &V22282_286_57168, &P22282_285};	/*Property = CssColor*/
const Value V22282_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22282_237 = {S9, 0, &V22282_237_55786, &P22282_286};	/*Property = CssColumn*/
const Value V22282_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22282_287 = {S10, 0, &V22282_287_57172, &P22282_237};	/*Property = CssFlexbox*/
const Value V22282_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22282_288 = {S11, 0, &V22282_288_57174, &P22282_287};	/*Property = CssFont*/
const Value V22282_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22282_289 = {S12, 0, &V22282_289_57759, &P22282_288};	/*Property = CssImages*/
const Value V22282_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22282_291 = {S13, 0, &V22282_291_57181, &P22282_289};	/*Property = CssMediaQueries*/
const Value V22282_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22282_292 = {S14, 0, &V22282_292_57183, &P22282_291};	/*Property = CssMinMax*/
const Value V22282_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22282_293 = {S15, 0, &V22282_293_57187, &P22282_292};	/*Property = CssOverflow*/
const Value V22282_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22282_294 = {S16, 0, &V22282_294_57189, &P22282_293};	/*Property = CssPosition*/
const Value V22282_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22282_295 = {S17, 0, &V22282_295_57193, &P22282_294};	/*Property = CssText*/
const Value V22282_297_57196 = {S0, NULL};	/*Value = False*/
const Property P22282_297 = {S18, 0, &V22282_297_57196, &P22282_295};	/*Property = CssTransforms*/
const Value V22282_241_55800 = {S0, NULL};	/*Value = False*/
const Property P22282_241 = {S19, 0, &V22282_241_55800, &P22282_297};	/*Property = CssTransitions*/
const Value V22282_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22282_298 = {S20, 0, &V22282_298_57206, &P22282_241};	/*Property = CssUI*/
const Value V22282_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22282_300 = {S21, 0, &V22282_300_57216, &P22282_298};	/*Property = DataSet*/
const Value V22282_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22282_301 = {S22, 0, &V22282_301_57218, &P22282_300};	/*Property = DataUrl*/
const Value V22282_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22282_302 = {S23, 0, &V22282_302_57222, &P22282_301};	/*Property = DeviceOrientation*/
const Value V22282_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22282_304 = {S24, 0, &V22282_304_57228, &P22282_302};	/*Property = FileReader*/
const Value V22282_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22282_305 = {S25, 0, &V22282_305_57231, &P22282_304};	/*Property = FileSaver*/
const Value V22282_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22282_306 = {S26, 0, &V22282_306_57237, &P22282_305};	/*Property = FileWriter*/
const Value V22282_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22282_307 = {S27, 0, &V22282_307_57240, &P22282_306};	/*Property = FormData*/
const Value V22282_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22282_309 = {S28, 0, &V22282_309_57246, &P22282_307};	/*Property = Fullscreen*/
const Value V22282_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22282_310 = {S29, 0, &V22282_310_57248, &P22282_309};	/*Property = GeoLocation*/
const Value V22282_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22282_312 = {S30, 0, &V22282_312_57255, &P22282_310};	/*Property = History*/
const Value V22282_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22282_314 = {S31, 0, &V22282_314_57261, &P22282_312};	/*Property = Html5*/
const Value V22282_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22282_313 = {S32, 0, &V22282_313_57258, &P22282_314};	/*Property = Html-Media-Capture*/
const Value V22282_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22282_315 = {S33, 0, &V22282_315_57264, &P22282_313};	/*Property = Iframe*/
const Value V22282_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22282_316 = {S34, 0, &V22282_316_57267, &P22282_315};	/*Property = IndexedDB*/
const Value V22282_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22282_317 = {S35, 0, &V22282_317_57269, &P22282_316};	/*Property = Json*/
const Value V22282_15_81502 = {S141, NULL};	/*Value = Blink*/
const Property P22282_15 = {S37, 0, &V22282_15_81502, &P22282_317};	/*Property = LayoutEngine*/
const Value V22282_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22282_344 = {S38, 0, &V22282_344_57272, &P22282_15};	/*Property = Masking*/
const Value V22282_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22282_320 = {S39, 0, &V22282_320_57290, &P22282_344};	/*Property = PostMessage*/
const Value V22282_321_57294 = {S0, NULL};	/*Value = False*/
const Property P22282_321 = {S40, 0, &V22282_321_57294, &P22282_320};	/*Property = Progress*/
const Value V22282_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22282_322 = {S41, 0, &V22282_322_57296, &P22282_321};	/*Property = Prompts*/
const Value V22282_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22282_323 = {S42, 0, &V22282_323_57299, &P22282_322};	/*Property = Selector*/
const Value V22282_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22282_324 = {S43, 0, &V22282_324_57305, &P22282_323};	/*Property = Svg*/
const Value V22282_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22282_327 = {S44, 0, &V22282_327_57315, &P22282_324};	/*Property = TouchEvents*/
const Value V22282_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22282_328 = {S45, 0, &V22282_328_57318, &P22282_327};	/*Property = Track*/
const Value V22282_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22282_329 = {S46, 0, &V22282_329_57323, &P22282_328};	/*Property = Video*/
const Value V22282_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22282_330 = {S47, 0, &V22282_330_57326, &P22282_329};	/*Property = Viewport*/
const Value V22282_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22282_332 = {S48, 0, &V22282_332_57339, &P22282_330};	/*Property = WebWorkers*/
const Value V22282_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22282_333 = {S49, 0, &V22282_333_57342, &P22282_332};	/*Property = Xhr2*/
const Profile P22282 = {"22282", &P22282_333};	/*Profile ID - 22282*/
const Value V22774_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22774_279 = {S1, 0, &V22774_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22774_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22774_282 = {S3, 0, &V22774_282_57154, &P22774_279};	/*Property = BlobBuilder*/
const Value V22774_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22774_350 = {S4, 0, &V22774_350_57444, &P22774_282};	/*Property = Canvas*/
const Value V22774_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22774_283 = {S5, 0, &V22774_283_57159, &P22774_350};	/*Property = CssBackground*/
const Value V22774_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22774_284 = {S6, 0, &V22774_284_57163, &P22774_283};	/*Property = CssBorderImage*/
const Value V22774_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22774_285 = {S7, 0, &V22774_285_57166, &P22774_284};	/*Property = CssCanvas*/
const Value V22774_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22774_286 = {S8, 0, &V22774_286_57168, &P22774_285};	/*Property = CssColor*/
const Value V22774_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22774_237 = {S9, 0, &V22774_237_55786, &P22774_286};	/*Property = CssColumn*/
const Value V22774_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22774_287 = {S10, 0, &V22774_287_57172, &P22774_237};	/*Property = CssFlexbox*/
const Value V22774_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22774_288 = {S11, 0, &V22774_288_57174, &P22774_287};	/*Property = CssFont*/
const Value V22774_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22774_289 = {S12, 0, &V22774_289_57759, &P22774_288};	/*Property = CssImages*/
const Value V22774_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22774_291 = {S13, 0, &V22774_291_57181, &P22774_289};	/*Property = CssMediaQueries*/
const Value V22774_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22774_292 = {S14, 0, &V22774_292_57183, &P22774_291};	/*Property = CssMinMax*/
const Value V22774_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22774_293 = {S15, 0, &V22774_293_57187, &P22774_292};	/*Property = CssOverflow*/
const Value V22774_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22774_294 = {S16, 0, &V22774_294_57189, &P22774_293};	/*Property = CssPosition*/
const Value V22774_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22774_295 = {S17, 0, &V22774_295_57192, &P22774_294};	/*Property = CssText*/
const Value V22774_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22774_297 = {S18, 0, &V22774_297_57195, &P22774_295};	/*Property = CssTransforms*/
const Value V22774_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22774_241 = {S19, 0, &V22774_241_55799, &P22774_297};	/*Property = CssTransitions*/
const Value V22774_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22774_298 = {S20, 0, &V22774_298_57206, &P22774_241};	/*Property = CssUI*/
const Value V22774_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22774_300 = {S21, 0, &V22774_300_57216, &P22774_298};	/*Property = DataSet*/
const Value V22774_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22774_301 = {S22, 0, &V22774_301_57218, &P22774_300};	/*Property = DataUrl*/
const Value V22774_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22774_302 = {S23, 0, &V22774_302_57222, &P22774_301};	/*Property = DeviceOrientation*/
const Value V22774_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22774_304 = {S24, 0, &V22774_304_57227, &P22774_302};	/*Property = FileReader*/
const Value V22774_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22774_305 = {S25, 0, &V22774_305_57231, &P22774_304};	/*Property = FileSaver*/
const Value V22774_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22774_306 = {S26, 0, &V22774_306_57237, &P22774_305};	/*Property = FileWriter*/
const Value V22774_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22774_307 = {S27, 0, &V22774_307_57239, &P22774_306};	/*Property = FormData*/
const Value V22774_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22774_309 = {S28, 0, &V22774_309_57246, &P22774_307};	/*Property = Fullscreen*/
const Value V22774_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22774_310 = {S29, 0, &V22774_310_57248, &P22774_309};	/*Property = GeoLocation*/
const Value V22774_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22774_312 = {S30, 0, &V22774_312_57254, &P22774_310};	/*Property = History*/
const Value V22774_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22774_314 = {S31, 0, &V22774_314_57261, &P22774_312};	/*Property = Html5*/
const Value V22774_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22774_313 = {S32, 0, &V22774_313_57258, &P22774_314};	/*Property = Html-Media-Capture*/
const Value V22774_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22774_315 = {S33, 0, &V22774_315_57264, &P22774_313};	/*Property = Iframe*/
const Value V22774_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22774_316 = {S34, 0, &V22774_316_57266, &P22774_315};	/*Property = IndexedDB*/
const Value V22774_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22774_317 = {S35, 0, &V22774_317_57269, &P22774_316};	/*Property = Json*/
const Value V22774_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P22774_15 = {S37, 0, &V22774_15_30, &P22774_317};	/*Property = LayoutEngine*/
const Value V22774_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22774_344 = {S38, 0, &V22774_344_57272, &P22774_15};	/*Property = Masking*/
const Value V22774_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22774_320 = {S39, 0, &V22774_320_57290, &P22774_344};	/*Property = PostMessage*/
const Value V22774_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22774_321 = {S40, 0, &V22774_321_57293, &P22774_320};	/*Property = Progress*/
const Value V22774_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22774_322 = {S41, 0, &V22774_322_57296, &P22774_321};	/*Property = Prompts*/
const Value V22774_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22774_323 = {S42, 0, &V22774_323_57299, &P22774_322};	/*Property = Selector*/
const Value V22774_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22774_324 = {S43, 0, &V22774_324_57305, &P22774_323};	/*Property = Svg*/
const Value V22774_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22774_327 = {S44, 0, &V22774_327_57315, &P22774_324};	/*Property = TouchEvents*/
const Value V22774_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22774_328 = {S45, 0, &V22774_328_57318, &P22774_327};	/*Property = Track*/
const Value V22774_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22774_329 = {S46, 0, &V22774_329_57323, &P22774_328};	/*Property = Video*/
const Value V22774_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22774_330 = {S47, 0, &V22774_330_57326, &P22774_329};	/*Property = Viewport*/
const Value V22774_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22774_332 = {S48, 0, &V22774_332_57339, &P22774_330};	/*Property = WebWorkers*/
const Value V22774_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22774_333 = {S49, 0, &V22774_333_57341, &P22774_332};	/*Property = Xhr2*/
const Profile P22774 = {"22774", &P22774_333};	/*Profile ID - 22774*/
const Value V24677_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_279 = {S1, 0, &V24677_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24677_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_282 = {S3, 0, &V24677_282_57155, &P24677_279};	/*Property = BlobBuilder*/
const Value V24677_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_350 = {S4, 0, &V24677_350_57446, &P24677_282};	/*Property = Canvas*/
const Value V24677_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_283 = {S5, 0, &V24677_283_57161, &P24677_350};	/*Property = CssBackground*/
const Value V24677_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_284 = {S6, 0, &V24677_284_57164, &P24677_283};	/*Property = CssBorderImage*/
const Value V24677_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_285 = {S7, 0, &V24677_285_57167, &P24677_284};	/*Property = CssCanvas*/
const Value V24677_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_286 = {S8, 0, &V24677_286_57170, &P24677_285};	/*Property = CssColor*/
const Value V24677_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_237 = {S9, 0, &V24677_237_55786, &P24677_286};	/*Property = CssColumn*/
const Value V24677_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_287 = {S10, 0, &V24677_287_57173, &P24677_237};	/*Property = CssFlexbox*/
const Value V24677_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_288 = {S11, 0, &V24677_288_57176, &P24677_287};	/*Property = CssFont*/
const Value V24677_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_289 = {S12, 0, &V24677_289_86899, &P24677_288};	/*Property = CssImages*/
const Value V24677_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_291 = {S13, 0, &V24677_291_57182, &P24677_289};	/*Property = CssMediaQueries*/
const Value V24677_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_292 = {S14, 0, &V24677_292_57185, &P24677_291};	/*Property = CssMinMax*/
const Value V24677_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_293 = {S15, 0, &V24677_293_57188, &P24677_292};	/*Property = CssOverflow*/
const Value V24677_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_294 = {S16, 0, &V24677_294_57191, &P24677_293};	/*Property = CssPosition*/
const Value V24677_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_295 = {S17, 0, &V24677_295_57194, &P24677_294};	/*Property = CssText*/
const Value V24677_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_297 = {S18, 0, &V24677_297_57197, &P24677_295};	/*Property = CssTransforms*/
const Value V24677_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_241 = {S19, 0, &V24677_241_55801, &P24677_297};	/*Property = CssTransitions*/
const Value V24677_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_298 = {S20, 0, &V24677_298_57208, &P24677_241};	/*Property = CssUI*/
const Value V24677_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_300 = {S21, 0, &V24677_300_57217, &P24677_298};	/*Property = DataSet*/
const Value V24677_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_301 = {S22, 0, &V24677_301_57220, &P24677_300};	/*Property = DataUrl*/
const Value V24677_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_302 = {S23, 0, &V24677_302_57223, &P24677_301};	/*Property = DeviceOrientation*/
const Value V24677_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_304 = {S24, 0, &V24677_304_57229, &P24677_302};	/*Property = FileReader*/
const Value V24677_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_305 = {S25, 0, &V24677_305_57232, &P24677_304};	/*Property = FileSaver*/
const Value V24677_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_306 = {S26, 0, &V24677_306_57238, &P24677_305};	/*Property = FileWriter*/
const Value V24677_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_307 = {S27, 0, &V24677_307_57241, &P24677_306};	/*Property = FormData*/
const Value V24677_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_309 = {S28, 0, &V24677_309_57247, &P24677_307};	/*Property = Fullscreen*/
const Value V24677_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_310 = {S29, 0, &V24677_310_57250, &P24677_309};	/*Property = GeoLocation*/
const Value V24677_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_312 = {S30, 0, &V24677_312_57256, &P24677_310};	/*Property = History*/
const Value V24677_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_314 = {S31, 0, &V24677_314_57262, &P24677_312};	/*Property = Html5*/
const Value V24677_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_313 = {S32, 0, &V24677_313_57259, &P24677_314};	/*Property = Html-Media-Capture*/
const Value V24677_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_315 = {S33, 0, &V24677_315_57265, &P24677_313};	/*Property = Iframe*/
const Value V24677_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_316 = {S34, 0, &V24677_316_57268, &P24677_315};	/*Property = IndexedDB*/
const Value V24677_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_317 = {S35, 0, &V24677_317_57271, &P24677_316};	/*Property = Json*/
const Value V24677_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24677_15 = {S37, 0, &V24677_15_90837, &P24677_317};	/*Property = LayoutEngine*/
const Value V24677_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_344 = {S38, 0, &V24677_344_57274, &P24677_15};	/*Property = Masking*/
const Value V24677_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_320 = {S39, 0, &V24677_320_57292, &P24677_344};	/*Property = PostMessage*/
const Value V24677_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_321 = {S40, 0, &V24677_321_57295, &P24677_320};	/*Property = Progress*/
const Value V24677_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_322 = {S41, 0, &V24677_322_57298, &P24677_321};	/*Property = Prompts*/
const Value V24677_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_323 = {S42, 0, &V24677_323_57301, &P24677_322};	/*Property = Selector*/
const Value V24677_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_324 = {S43, 0, &V24677_324_57307, &P24677_323};	/*Property = Svg*/
const Value V24677_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_327 = {S44, 0, &V24677_327_57316, &P24677_324};	/*Property = TouchEvents*/
const Value V24677_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_328 = {S45, 0, &V24677_328_57319, &P24677_327};	/*Property = Track*/
const Value V24677_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_329 = {S46, 0, &V24677_329_57325, &P24677_328};	/*Property = Video*/
const Value V24677_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_330 = {S47, 0, &V24677_330_57328, &P24677_329};	/*Property = Viewport*/
const Value V24677_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_332 = {S48, 0, &V24677_332_57340, &P24677_330};	/*Property = WebWorkers*/
const Value V24677_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24677_333 = {S49, 0, &V24677_333_57343, &P24677_332};	/*Property = Xhr2*/
const Profile P24677 = {"24677", &P24677_333};	/*Profile ID - 24677*/
const Value V22370_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22370_279 = {S1, 0, &V22370_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22370_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22370_282 = {S3, 0, &V22370_282_57154, &P22370_279};	/*Property = BlobBuilder*/
const Value V22370_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22370_350 = {S4, 0, &V22370_350_57444, &P22370_282};	/*Property = Canvas*/
const Value V22370_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22370_283 = {S5, 0, &V22370_283_57159, &P22370_350};	/*Property = CssBackground*/
const Value V22370_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22370_284 = {S6, 0, &V22370_284_57163, &P22370_283};	/*Property = CssBorderImage*/
const Value V22370_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22370_285 = {S7, 0, &V22370_285_57166, &P22370_284};	/*Property = CssCanvas*/
const Value V22370_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22370_286 = {S8, 0, &V22370_286_57168, &P22370_285};	/*Property = CssColor*/
const Value V22370_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22370_237 = {S9, 0, &V22370_237_55786, &P22370_286};	/*Property = CssColumn*/
const Value V22370_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22370_287 = {S10, 0, &V22370_287_57172, &P22370_237};	/*Property = CssFlexbox*/
const Value V22370_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22370_288 = {S11, 0, &V22370_288_57174, &P22370_287};	/*Property = CssFont*/
const Value V22370_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22370_289 = {S12, 0, &V22370_289_57759, &P22370_288};	/*Property = CssImages*/
const Value V22370_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22370_291 = {S13, 0, &V22370_291_57180, &P22370_289};	/*Property = CssMediaQueries*/
const Value V22370_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22370_292 = {S14, 0, &V22370_292_57183, &P22370_291};	/*Property = CssMinMax*/
const Value V22370_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22370_293 = {S15, 0, &V22370_293_57187, &P22370_292};	/*Property = CssOverflow*/
const Value V22370_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22370_294 = {S16, 0, &V22370_294_57189, &P22370_293};	/*Property = CssPosition*/
const Value V22370_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22370_295 = {S17, 0, &V22370_295_57192, &P22370_294};	/*Property = CssText*/
const Value V22370_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22370_297 = {S18, 0, &V22370_297_57195, &P22370_295};	/*Property = CssTransforms*/
const Value V22370_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22370_241 = {S19, 0, &V22370_241_55799, &P22370_297};	/*Property = CssTransitions*/
const Value V22370_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22370_298 = {S20, 0, &V22370_298_57206, &P22370_241};	/*Property = CssUI*/
const Value V22370_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22370_300 = {S21, 0, &V22370_300_57216, &P22370_298};	/*Property = DataSet*/
const Value V22370_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22370_301 = {S22, 0, &V22370_301_57218, &P22370_300};	/*Property = DataUrl*/
const Value V22370_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22370_302 = {S23, 0, &V22370_302_57221, &P22370_301};	/*Property = DeviceOrientation*/
const Value V22370_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22370_304 = {S24, 0, &V22370_304_57227, &P22370_302};	/*Property = FileReader*/
const Value V22370_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22370_305 = {S25, 0, &V22370_305_57231, &P22370_304};	/*Property = FileSaver*/
const Value V22370_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22370_306 = {S26, 0, &V22370_306_57237, &P22370_305};	/*Property = FileWriter*/
const Value V22370_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22370_307 = {S27, 0, &V22370_307_57239, &P22370_306};	/*Property = FormData*/
const Value V22370_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22370_309 = {S28, 0, &V22370_309_57246, &P22370_307};	/*Property = Fullscreen*/
const Value V22370_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22370_310 = {S29, 0, &V22370_310_57248, &P22370_309};	/*Property = GeoLocation*/
const Value V22370_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22370_312 = {S30, 0, &V22370_312_57254, &P22370_310};	/*Property = History*/
const Value V22370_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22370_314 = {S31, 0, &V22370_314_57261, &P22370_312};	/*Property = Html5*/
const Value V22370_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22370_313 = {S32, 0, &V22370_313_57258, &P22370_314};	/*Property = Html-Media-Capture*/
const Value V22370_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22370_315 = {S33, 0, &V22370_315_57264, &P22370_313};	/*Property = Iframe*/
const Value V22370_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22370_316 = {S34, 0, &V22370_316_57266, &P22370_315};	/*Property = IndexedDB*/
const Value V22370_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22370_317 = {S35, 0, &V22370_317_57269, &P22370_316};	/*Property = Json*/
const Value V22370_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22370_15 = {S37, 0, &V22370_15_29, &P22370_317};	/*Property = LayoutEngine*/
const Value V22370_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22370_344 = {S38, 0, &V22370_344_57272, &P22370_15};	/*Property = Masking*/
const Value V22370_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22370_320 = {S39, 0, &V22370_320_57290, &P22370_344};	/*Property = PostMessage*/
const Value V22370_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22370_321 = {S40, 0, &V22370_321_57293, &P22370_320};	/*Property = Progress*/
const Value V22370_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22370_322 = {S41, 0, &V22370_322_57296, &P22370_321};	/*Property = Prompts*/
const Value V22370_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22370_323 = {S42, 0, &V22370_323_57299, &P22370_322};	/*Property = Selector*/
const Value V22370_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22370_324 = {S43, 0, &V22370_324_57306, &P22370_323};	/*Property = Svg*/
const Value V22370_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22370_327 = {S44, 0, &V22370_327_57315, &P22370_324};	/*Property = TouchEvents*/
const Value V22370_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22370_328 = {S45, 0, &V22370_328_57318, &P22370_327};	/*Property = Track*/
const Value V22370_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22370_329 = {S46, 0, &V22370_329_57323, &P22370_328};	/*Property = Video*/
const Value V22370_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22370_330 = {S47, 0, &V22370_330_57326, &P22370_329};	/*Property = Viewport*/
const Value V22370_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22370_332 = {S48, 0, &V22370_332_57338, &P22370_330};	/*Property = WebWorkers*/
const Value V22370_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22370_333 = {S49, 0, &V22370_333_57341, &P22370_332};	/*Property = Xhr2*/
const Profile P22370 = {"22370", &P22370_333};	/*Profile ID - 22370*/
const Value V24633_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_279 = {S1, 0, &V24633_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24633_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_282 = {S3, 0, &V24633_282_57155, &P24633_279};	/*Property = BlobBuilder*/
const Value V24633_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_350 = {S4, 0, &V24633_350_57446, &P24633_282};	/*Property = Canvas*/
const Value V24633_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_283 = {S5, 0, &V24633_283_57161, &P24633_350};	/*Property = CssBackground*/
const Value V24633_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_284 = {S6, 0, &V24633_284_57164, &P24633_283};	/*Property = CssBorderImage*/
const Value V24633_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_285 = {S7, 0, &V24633_285_57167, &P24633_284};	/*Property = CssCanvas*/
const Value V24633_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_286 = {S8, 0, &V24633_286_57170, &P24633_285};	/*Property = CssColor*/
const Value V24633_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_237 = {S9, 0, &V24633_237_55786, &P24633_286};	/*Property = CssColumn*/
const Value V24633_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_287 = {S10, 0, &V24633_287_57173, &P24633_237};	/*Property = CssFlexbox*/
const Value V24633_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_288 = {S11, 0, &V24633_288_57176, &P24633_287};	/*Property = CssFont*/
const Value V24633_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_289 = {S12, 0, &V24633_289_86899, &P24633_288};	/*Property = CssImages*/
const Value V24633_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_291 = {S13, 0, &V24633_291_57182, &P24633_289};	/*Property = CssMediaQueries*/
const Value V24633_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_292 = {S14, 0, &V24633_292_57185, &P24633_291};	/*Property = CssMinMax*/
const Value V24633_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_293 = {S15, 0, &V24633_293_57188, &P24633_292};	/*Property = CssOverflow*/
const Value V24633_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_294 = {S16, 0, &V24633_294_57191, &P24633_293};	/*Property = CssPosition*/
const Value V24633_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_295 = {S17, 0, &V24633_295_57194, &P24633_294};	/*Property = CssText*/
const Value V24633_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_297 = {S18, 0, &V24633_297_57197, &P24633_295};	/*Property = CssTransforms*/
const Value V24633_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_241 = {S19, 0, &V24633_241_55801, &P24633_297};	/*Property = CssTransitions*/
const Value V24633_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_298 = {S20, 0, &V24633_298_57208, &P24633_241};	/*Property = CssUI*/
const Value V24633_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_300 = {S21, 0, &V24633_300_57217, &P24633_298};	/*Property = DataSet*/
const Value V24633_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_301 = {S22, 0, &V24633_301_57220, &P24633_300};	/*Property = DataUrl*/
const Value V24633_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_302 = {S23, 0, &V24633_302_57223, &P24633_301};	/*Property = DeviceOrientation*/
const Value V24633_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_304 = {S24, 0, &V24633_304_57229, &P24633_302};	/*Property = FileReader*/
const Value V24633_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_305 = {S25, 0, &V24633_305_57232, &P24633_304};	/*Property = FileSaver*/
const Value V24633_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_306 = {S26, 0, &V24633_306_57238, &P24633_305};	/*Property = FileWriter*/
const Value V24633_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_307 = {S27, 0, &V24633_307_57241, &P24633_306};	/*Property = FormData*/
const Value V24633_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_309 = {S28, 0, &V24633_309_57247, &P24633_307};	/*Property = Fullscreen*/
const Value V24633_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_310 = {S29, 0, &V24633_310_57250, &P24633_309};	/*Property = GeoLocation*/
const Value V24633_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_312 = {S30, 0, &V24633_312_57256, &P24633_310};	/*Property = History*/
const Value V24633_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_314 = {S31, 0, &V24633_314_57262, &P24633_312};	/*Property = Html5*/
const Value V24633_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_313 = {S32, 0, &V24633_313_57259, &P24633_314};	/*Property = Html-Media-Capture*/
const Value V24633_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_315 = {S33, 0, &V24633_315_57265, &P24633_313};	/*Property = Iframe*/
const Value V24633_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_316 = {S34, 0, &V24633_316_57268, &P24633_315};	/*Property = IndexedDB*/
const Value V24633_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_317 = {S35, 0, &V24633_317_57271, &P24633_316};	/*Property = Json*/
const Value V24633_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24633_15 = {S37, 0, &V24633_15_90837, &P24633_317};	/*Property = LayoutEngine*/
const Value V24633_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_344 = {S38, 0, &V24633_344_57274, &P24633_15};	/*Property = Masking*/
const Value V24633_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_320 = {S39, 0, &V24633_320_57292, &P24633_344};	/*Property = PostMessage*/
const Value V24633_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_321 = {S40, 0, &V24633_321_57295, &P24633_320};	/*Property = Progress*/
const Value V24633_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_322 = {S41, 0, &V24633_322_57298, &P24633_321};	/*Property = Prompts*/
const Value V24633_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_323 = {S42, 0, &V24633_323_57301, &P24633_322};	/*Property = Selector*/
const Value V24633_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_324 = {S43, 0, &V24633_324_57307, &P24633_323};	/*Property = Svg*/
const Value V24633_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_327 = {S44, 0, &V24633_327_57316, &P24633_324};	/*Property = TouchEvents*/
const Value V24633_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_328 = {S45, 0, &V24633_328_57319, &P24633_327};	/*Property = Track*/
const Value V24633_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_329 = {S46, 0, &V24633_329_57325, &P24633_328};	/*Property = Video*/
const Value V24633_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_330 = {S47, 0, &V24633_330_57328, &P24633_329};	/*Property = Viewport*/
const Value V24633_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_332 = {S48, 0, &V24633_332_57340, &P24633_330};	/*Property = WebWorkers*/
const Value V24633_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24633_333 = {S49, 0, &V24633_333_57343, &P24633_332};	/*Property = Xhr2*/
const Profile P24633 = {"24633", &P24633_333};	/*Profile ID - 24633*/
const Value V22466_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22466_279 = {S1, 0, &V22466_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22466_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22466_282 = {S3, 0, &V22466_282_57154, &P22466_279};	/*Property = BlobBuilder*/
const Value V22466_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22466_350 = {S4, 0, &V22466_350_57444, &P22466_282};	/*Property = Canvas*/
const Value V22466_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22466_283 = {S5, 0, &V22466_283_57159, &P22466_350};	/*Property = CssBackground*/
const Value V22466_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22466_284 = {S6, 0, &V22466_284_57162, &P22466_283};	/*Property = CssBorderImage*/
const Value V22466_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22466_285 = {S7, 0, &V22466_285_57166, &P22466_284};	/*Property = CssCanvas*/
const Value V22466_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22466_286 = {S8, 0, &V22466_286_57168, &P22466_285};	/*Property = CssColor*/
const Value V22466_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22466_237 = {S9, 0, &V22466_237_55786, &P22466_286};	/*Property = CssColumn*/
const Value V22466_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22466_287 = {S10, 0, &V22466_287_57172, &P22466_237};	/*Property = CssFlexbox*/
const Value V22466_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22466_288 = {S11, 0, &V22466_288_57174, &P22466_287};	/*Property = CssFont*/
const Value V22466_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22466_289 = {S12, 0, &V22466_289_57759, &P22466_288};	/*Property = CssImages*/
const Value V22466_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22466_291 = {S13, 0, &V22466_291_57180, &P22466_289};	/*Property = CssMediaQueries*/
const Value V22466_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22466_292 = {S14, 0, &V22466_292_57183, &P22466_291};	/*Property = CssMinMax*/
const Value V22466_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22466_293 = {S15, 0, &V22466_293_57187, &P22466_292};	/*Property = CssOverflow*/
const Value V22466_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22466_294 = {S16, 0, &V22466_294_57189, &P22466_293};	/*Property = CssPosition*/
const Value V22466_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22466_295 = {S17, 0, &V22466_295_57192, &P22466_294};	/*Property = CssText*/
const Value V22466_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22466_297 = {S18, 0, &V22466_297_57195, &P22466_295};	/*Property = CssTransforms*/
const Value V22466_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22466_241 = {S19, 0, &V22466_241_55799, &P22466_297};	/*Property = CssTransitions*/
const Value V22466_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22466_298 = {S20, 0, &V22466_298_57206, &P22466_241};	/*Property = CssUI*/
const Value V22466_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22466_300 = {S21, 0, &V22466_300_57215, &P22466_298};	/*Property = DataSet*/
const Value V22466_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22466_301 = {S22, 0, &V22466_301_57218, &P22466_300};	/*Property = DataUrl*/
const Value V22466_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22466_302 = {S23, 0, &V22466_302_57222, &P22466_301};	/*Property = DeviceOrientation*/
const Value V22466_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22466_304 = {S24, 0, &V22466_304_57227, &P22466_302};	/*Property = FileReader*/
const Value V22466_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22466_305 = {S25, 0, &V22466_305_57231, &P22466_304};	/*Property = FileSaver*/
const Value V22466_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22466_306 = {S26, 0, &V22466_306_57237, &P22466_305};	/*Property = FileWriter*/
const Value V22466_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22466_307 = {S27, 0, &V22466_307_57239, &P22466_306};	/*Property = FormData*/
const Value V22466_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22466_309 = {S28, 0, &V22466_309_57246, &P22466_307};	/*Property = Fullscreen*/
const Value V22466_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22466_310 = {S29, 0, &V22466_310_57248, &P22466_309};	/*Property = GeoLocation*/
const Value V22466_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22466_312 = {S30, 0, &V22466_312_57254, &P22466_310};	/*Property = History*/
const Value V22466_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22466_314 = {S31, 0, &V22466_314_57260, &P22466_312};	/*Property = Html5*/
const Value V22466_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22466_313 = {S32, 0, &V22466_313_57258, &P22466_314};	/*Property = Html-Media-Capture*/
const Value V22466_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22466_315 = {S33, 0, &V22466_315_57264, &P22466_313};	/*Property = Iframe*/
const Value V22466_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22466_316 = {S34, 0, &V22466_316_57266, &P22466_315};	/*Property = IndexedDB*/
const Value V22466_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22466_317 = {S35, 0, &V22466_317_57269, &P22466_316};	/*Property = Json*/
const Value V22466_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22466_15 = {S37, 0, &V22466_15_28, &P22466_317};	/*Property = LayoutEngine*/
const Value V22466_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22466_344 = {S38, 0, &V22466_344_57272, &P22466_15};	/*Property = Masking*/
const Value V22466_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22466_320 = {S39, 0, &V22466_320_57290, &P22466_344};	/*Property = PostMessage*/
const Value V22466_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22466_321 = {S40, 0, &V22466_321_57293, &P22466_320};	/*Property = Progress*/
const Value V22466_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22466_322 = {S41, 0, &V22466_322_57296, &P22466_321};	/*Property = Prompts*/
const Value V22466_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22466_323 = {S42, 0, &V22466_323_57299, &P22466_322};	/*Property = Selector*/
const Value V22466_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22466_324 = {S43, 0, &V22466_324_57305, &P22466_323};	/*Property = Svg*/
const Value V22466_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22466_327 = {S44, 0, &V22466_327_57315, &P22466_324};	/*Property = TouchEvents*/
const Value V22466_328_57317 = {S2, NULL};	/*Value = True*/
const Property P22466_328 = {S45, 0, &V22466_328_57317, &P22466_327};	/*Property = Track*/
const Value V22466_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22466_329 = {S46, 0, &V22466_329_57323, &P22466_328};	/*Property = Video*/
const Value V22466_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22466_330 = {S47, 0, &V22466_330_57326, &P22466_329};	/*Property = Viewport*/
const Value V22466_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22466_332 = {S48, 0, &V22466_332_57338, &P22466_330};	/*Property = WebWorkers*/
const Value V22466_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22466_333 = {S49, 0, &V22466_333_57341, &P22466_332};	/*Property = Xhr2*/
const Profile P22466 = {"22466", &P22466_333};	/*Profile ID - 22466*/
const Value V22226_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_279 = {S1, 0, &V22226_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22226_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_282 = {S3, 0, &V22226_282_57155, &P22226_279};	/*Property = BlobBuilder*/
const Value V22226_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_350 = {S4, 0, &V22226_350_57446, &P22226_282};	/*Property = Canvas*/
const Value V22226_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_283 = {S5, 0, &V22226_283_57161, &P22226_350};	/*Property = CssBackground*/
const Value V22226_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_284 = {S6, 0, &V22226_284_57164, &P22226_283};	/*Property = CssBorderImage*/
const Value V22226_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_285 = {S7, 0, &V22226_285_57167, &P22226_284};	/*Property = CssCanvas*/
const Value V22226_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_286 = {S8, 0, &V22226_286_57170, &P22226_285};	/*Property = CssColor*/
const Value V22226_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_237 = {S9, 0, &V22226_237_55786, &P22226_286};	/*Property = CssColumn*/
const Value V22226_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_287 = {S10, 0, &V22226_287_57173, &P22226_237};	/*Property = CssFlexbox*/
const Value V22226_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_288 = {S11, 0, &V22226_288_57176, &P22226_287};	/*Property = CssFont*/
const Value V22226_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_289 = {S12, 0, &V22226_289_86899, &P22226_288};	/*Property = CssImages*/
const Value V22226_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_291 = {S13, 0, &V22226_291_57182, &P22226_289};	/*Property = CssMediaQueries*/
const Value V22226_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_292 = {S14, 0, &V22226_292_57185, &P22226_291};	/*Property = CssMinMax*/
const Value V22226_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_293 = {S15, 0, &V22226_293_57188, &P22226_292};	/*Property = CssOverflow*/
const Value V22226_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_294 = {S16, 0, &V22226_294_57191, &P22226_293};	/*Property = CssPosition*/
const Value V22226_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_295 = {S17, 0, &V22226_295_57194, &P22226_294};	/*Property = CssText*/
const Value V22226_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_297 = {S18, 0, &V22226_297_57197, &P22226_295};	/*Property = CssTransforms*/
const Value V22226_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_241 = {S19, 0, &V22226_241_55801, &P22226_297};	/*Property = CssTransitions*/
const Value V22226_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_298 = {S20, 0, &V22226_298_57208, &P22226_241};	/*Property = CssUI*/
const Value V22226_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_300 = {S21, 0, &V22226_300_57217, &P22226_298};	/*Property = DataSet*/
const Value V22226_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_301 = {S22, 0, &V22226_301_57220, &P22226_300};	/*Property = DataUrl*/
const Value V22226_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_302 = {S23, 0, &V22226_302_57223, &P22226_301};	/*Property = DeviceOrientation*/
const Value V22226_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_304 = {S24, 0, &V22226_304_57229, &P22226_302};	/*Property = FileReader*/
const Value V22226_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_305 = {S25, 0, &V22226_305_57232, &P22226_304};	/*Property = FileSaver*/
const Value V22226_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_306 = {S26, 0, &V22226_306_57238, &P22226_305};	/*Property = FileWriter*/
const Value V22226_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_307 = {S27, 0, &V22226_307_57241, &P22226_306};	/*Property = FormData*/
const Value V22226_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_309 = {S28, 0, &V22226_309_57247, &P22226_307};	/*Property = Fullscreen*/
const Value V22226_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_310 = {S29, 0, &V22226_310_57250, &P22226_309};	/*Property = GeoLocation*/
const Value V22226_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_312 = {S30, 0, &V22226_312_57256, &P22226_310};	/*Property = History*/
const Value V22226_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_314 = {S31, 0, &V22226_314_57262, &P22226_312};	/*Property = Html5*/
const Value V22226_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_313 = {S32, 0, &V22226_313_57259, &P22226_314};	/*Property = Html-Media-Capture*/
const Value V22226_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_315 = {S33, 0, &V22226_315_57265, &P22226_313};	/*Property = Iframe*/
const Value V22226_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_316 = {S34, 0, &V22226_316_57268, &P22226_315};	/*Property = IndexedDB*/
const Value V22226_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_317 = {S35, 0, &V22226_317_57271, &P22226_316};	/*Property = Json*/
const Value V22226_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22226_15 = {S37, 0, &V22226_15_29, &P22226_317};	/*Property = LayoutEngine*/
const Value V22226_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_344 = {S38, 0, &V22226_344_57274, &P22226_15};	/*Property = Masking*/
const Value V22226_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_320 = {S39, 0, &V22226_320_57292, &P22226_344};	/*Property = PostMessage*/
const Value V22226_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_321 = {S40, 0, &V22226_321_57295, &P22226_320};	/*Property = Progress*/
const Value V22226_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_322 = {S41, 0, &V22226_322_57298, &P22226_321};	/*Property = Prompts*/
const Value V22226_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_323 = {S42, 0, &V22226_323_57301, &P22226_322};	/*Property = Selector*/
const Value V22226_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_324 = {S43, 0, &V22226_324_57307, &P22226_323};	/*Property = Svg*/
const Value V22226_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_327 = {S44, 0, &V22226_327_57316, &P22226_324};	/*Property = TouchEvents*/
const Value V22226_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_328 = {S45, 0, &V22226_328_57319, &P22226_327};	/*Property = Track*/
const Value V22226_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_329 = {S46, 0, &V22226_329_57325, &P22226_328};	/*Property = Video*/
const Value V22226_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_330 = {S47, 0, &V22226_330_57328, &P22226_329};	/*Property = Viewport*/
const Value V22226_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_332 = {S48, 0, &V22226_332_57340, &P22226_330};	/*Property = WebWorkers*/
const Value V22226_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22226_333 = {S49, 0, &V22226_333_57343, &P22226_332};	/*Property = Xhr2*/
const Profile P22226 = {"22226", &P22226_333};	/*Profile ID - 22226*/
const Value V22209_1_1 = {S2, NULL};	/*Value = True*/
const Property P22209_1 = {S50, 0, &V22209_1_1, NULL};	/*Property = IsMobile*/
const Value V22209_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22209_5 = {S52, 0, &V22209_5_1403, &P22209_1};	/*Property = ScreenPixelsHeight*/
const Value V22209_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22209_4 = {S54, 0, &V22209_4_1404, &P22209_5};	/*Property = ScreenPixelsWidth*/
const Profile P22209 = {"22209", &P22209_4};	/*Profile ID - 22209*/
const Value V23235_1_1 = {S2, NULL};	/*Value = True*/
const Property P23235_1 = {S50, 0, &V23235_1_1, NULL};	/*Property = IsMobile*/
const Value V23235_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23235_5 = {S52, 0, &V23235_5_1277, &P23235_1};	/*Property = ScreenPixelsHeight*/
const Value V23235_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23235_4 = {S54, 0, &V23235_4_2336, &P23235_5};	/*Property = ScreenPixelsWidth*/
const Profile P23235 = {"23235", &P23235_4};	/*Profile ID - 23235*/
const Profile P22277 = {"22277", NULL};	/*Profile ID - 22277*/
const Value V22727_1_1 = {S2, NULL};	/*Value = True*/
const Property P22727_1 = {S50, 0, &V22727_1_1, NULL};	/*Property = IsMobile*/
const Value V22727_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22727_5 = {S52, 0, &V22727_5_59138, &P22727_1};	/*Property = ScreenPixelsHeight*/
const Value V22727_4_52971 = {S122, NULL};	/*Value = 1200*/
const Property P22727_4 = {S54, 0, &V22727_4_52971, &P22727_5};	/*Property = ScreenPixelsWidth*/
const Profile P22727 = {"22727", &P22727_4};	/*Profile ID - 22727*/
const Value V22930_1_1 = {S2, NULL};	/*Value = True*/
const Property P22930_1 = {S50, 0, &V22930_1_1, NULL};	/*Property = IsMobile*/
const Value V22930_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22930_5 = {S52, 0, &V22930_5_1227, &P22930_1};	/*Property = ScreenPixelsHeight*/
const Value V22930_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22930_4 = {S54, 0, &V22930_4_1229, &P22930_5};	/*Property = ScreenPixelsWidth*/
const Profile P22930 = {"22930", &P22930_4};	/*Profile ID - 22930*/
const Value V22776_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_279 = {S1, 0, &V22776_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22776_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_282 = {S3, 0, &V22776_282_57155, &P22776_279};	/*Property = BlobBuilder*/
const Value V22776_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_350 = {S4, 0, &V22776_350_57446, &P22776_282};	/*Property = Canvas*/
const Value V22776_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_283 = {S5, 0, &V22776_283_57161, &P22776_350};	/*Property = CssBackground*/
const Value V22776_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_284 = {S6, 0, &V22776_284_57164, &P22776_283};	/*Property = CssBorderImage*/
const Value V22776_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_285 = {S7, 0, &V22776_285_57167, &P22776_284};	/*Property = CssCanvas*/
const Value V22776_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_286 = {S8, 0, &V22776_286_57170, &P22776_285};	/*Property = CssColor*/
const Value V22776_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_237 = {S9, 0, &V22776_237_55786, &P22776_286};	/*Property = CssColumn*/
const Value V22776_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_287 = {S10, 0, &V22776_287_57173, &P22776_237};	/*Property = CssFlexbox*/
const Value V22776_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_288 = {S11, 0, &V22776_288_57176, &P22776_287};	/*Property = CssFont*/
const Value V22776_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_289 = {S12, 0, &V22776_289_86899, &P22776_288};	/*Property = CssImages*/
const Value V22776_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_291 = {S13, 0, &V22776_291_57182, &P22776_289};	/*Property = CssMediaQueries*/
const Value V22776_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_292 = {S14, 0, &V22776_292_57185, &P22776_291};	/*Property = CssMinMax*/
const Value V22776_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_293 = {S15, 0, &V22776_293_57188, &P22776_292};	/*Property = CssOverflow*/
const Value V22776_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_294 = {S16, 0, &V22776_294_57191, &P22776_293};	/*Property = CssPosition*/
const Value V22776_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_295 = {S17, 0, &V22776_295_57194, &P22776_294};	/*Property = CssText*/
const Value V22776_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_297 = {S18, 0, &V22776_297_57197, &P22776_295};	/*Property = CssTransforms*/
const Value V22776_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_241 = {S19, 0, &V22776_241_55801, &P22776_297};	/*Property = CssTransitions*/
const Value V22776_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_298 = {S20, 0, &V22776_298_57208, &P22776_241};	/*Property = CssUI*/
const Value V22776_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_300 = {S21, 0, &V22776_300_57217, &P22776_298};	/*Property = DataSet*/
const Value V22776_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_301 = {S22, 0, &V22776_301_57220, &P22776_300};	/*Property = DataUrl*/
const Value V22776_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_302 = {S23, 0, &V22776_302_57223, &P22776_301};	/*Property = DeviceOrientation*/
const Value V22776_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_304 = {S24, 0, &V22776_304_57229, &P22776_302};	/*Property = FileReader*/
const Value V22776_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_305 = {S25, 0, &V22776_305_57232, &P22776_304};	/*Property = FileSaver*/
const Value V22776_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_306 = {S26, 0, &V22776_306_57238, &P22776_305};	/*Property = FileWriter*/
const Value V22776_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_307 = {S27, 0, &V22776_307_57241, &P22776_306};	/*Property = FormData*/
const Value V22776_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_309 = {S28, 0, &V22776_309_57247, &P22776_307};	/*Property = Fullscreen*/
const Value V22776_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_310 = {S29, 0, &V22776_310_57250, &P22776_309};	/*Property = GeoLocation*/
const Value V22776_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_312 = {S30, 0, &V22776_312_57256, &P22776_310};	/*Property = History*/
const Value V22776_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_314 = {S31, 0, &V22776_314_57262, &P22776_312};	/*Property = Html5*/
const Value V22776_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_313 = {S32, 0, &V22776_313_57259, &P22776_314};	/*Property = Html-Media-Capture*/
const Value V22776_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_315 = {S33, 0, &V22776_315_57265, &P22776_313};	/*Property = Iframe*/
const Value V22776_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_316 = {S34, 0, &V22776_316_57268, &P22776_315};	/*Property = IndexedDB*/
const Value V22776_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_317 = {S35, 0, &V22776_317_57271, &P22776_316};	/*Property = Json*/
const Value V22776_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22776_15 = {S37, 0, &V22776_15_29, &P22776_317};	/*Property = LayoutEngine*/
const Value V22776_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_344 = {S38, 0, &V22776_344_57274, &P22776_15};	/*Property = Masking*/
const Value V22776_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_320 = {S39, 0, &V22776_320_57292, &P22776_344};	/*Property = PostMessage*/
const Value V22776_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_321 = {S40, 0, &V22776_321_57295, &P22776_320};	/*Property = Progress*/
const Value V22776_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_322 = {S41, 0, &V22776_322_57298, &P22776_321};	/*Property = Prompts*/
const Value V22776_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_323 = {S42, 0, &V22776_323_57301, &P22776_322};	/*Property = Selector*/
const Value V22776_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_324 = {S43, 0, &V22776_324_57307, &P22776_323};	/*Property = Svg*/
const Value V22776_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_327 = {S44, 0, &V22776_327_57316, &P22776_324};	/*Property = TouchEvents*/
const Value V22776_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_328 = {S45, 0, &V22776_328_57319, &P22776_327};	/*Property = Track*/
const Value V22776_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_329 = {S46, 0, &V22776_329_57325, &P22776_328};	/*Property = Video*/
const Value V22776_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_330 = {S47, 0, &V22776_330_57328, &P22776_329};	/*Property = Viewport*/
const Value V22776_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_332 = {S48, 0, &V22776_332_57340, &P22776_330};	/*Property = WebWorkers*/
const Value V22776_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22776_333 = {S49, 0, &V22776_333_57343, &P22776_332};	/*Property = Xhr2*/
const Profile P22776 = {"22776", &P22776_333};	/*Profile ID - 22776*/
const Value V20714_1_1 = {S2, NULL};	/*Value = True*/
const Property P20714_1 = {S50, 0, &V20714_1_1, NULL};	/*Property = IsMobile*/
const Value V20714_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20714_5 = {S52, 0, &V20714_5_1458, &P20714_1};	/*Property = ScreenPixelsHeight*/
const Value V20714_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P20714_4 = {S54, 0, &V20714_4_37617, &P20714_5};	/*Property = ScreenPixelsWidth*/
const Profile P20714 = {"20714", &P20714_4};	/*Profile ID - 20714*/
const Value V22560_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22560_279 = {S1, 0, &V22560_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22560_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22560_282 = {S3, 0, &V22560_282_57154, &P22560_279};	/*Property = BlobBuilder*/
const Value V22560_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22560_350 = {S4, 0, &V22560_350_57444, &P22560_282};	/*Property = Canvas*/
const Value V22560_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22560_283 = {S5, 0, &V22560_283_57159, &P22560_350};	/*Property = CssBackground*/
const Value V22560_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22560_284 = {S6, 0, &V22560_284_57163, &P22560_283};	/*Property = CssBorderImage*/
const Value V22560_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22560_285 = {S7, 0, &V22560_285_57166, &P22560_284};	/*Property = CssCanvas*/
const Value V22560_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22560_286 = {S8, 0, &V22560_286_57168, &P22560_285};	/*Property = CssColor*/
const Value V22560_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22560_237 = {S9, 0, &V22560_237_55786, &P22560_286};	/*Property = CssColumn*/
const Value V22560_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22560_287 = {S10, 0, &V22560_287_57172, &P22560_237};	/*Property = CssFlexbox*/
const Value V22560_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22560_288 = {S11, 0, &V22560_288_57174, &P22560_287};	/*Property = CssFont*/
const Value V22560_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22560_289 = {S12, 0, &V22560_289_57759, &P22560_288};	/*Property = CssImages*/
const Value V22560_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22560_291 = {S13, 0, &V22560_291_57180, &P22560_289};	/*Property = CssMediaQueries*/
const Value V22560_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22560_292 = {S14, 0, &V22560_292_57183, &P22560_291};	/*Property = CssMinMax*/
const Value V22560_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22560_293 = {S15, 0, &V22560_293_57187, &P22560_292};	/*Property = CssOverflow*/
const Value V22560_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22560_294 = {S16, 0, &V22560_294_57189, &P22560_293};	/*Property = CssPosition*/
const Value V22560_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22560_295 = {S17, 0, &V22560_295_57192, &P22560_294};	/*Property = CssText*/
const Value V22560_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22560_297 = {S18, 0, &V22560_297_57195, &P22560_295};	/*Property = CssTransforms*/
const Value V22560_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22560_241 = {S19, 0, &V22560_241_55799, &P22560_297};	/*Property = CssTransitions*/
const Value V22560_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22560_298 = {S20, 0, &V22560_298_57206, &P22560_241};	/*Property = CssUI*/
const Value V22560_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22560_300 = {S21, 0, &V22560_300_57216, &P22560_298};	/*Property = DataSet*/
const Value V22560_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22560_301 = {S22, 0, &V22560_301_57218, &P22560_300};	/*Property = DataUrl*/
const Value V22560_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22560_302 = {S23, 0, &V22560_302_57221, &P22560_301};	/*Property = DeviceOrientation*/
const Value V22560_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22560_304 = {S24, 0, &V22560_304_57227, &P22560_302};	/*Property = FileReader*/
const Value V22560_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22560_305 = {S25, 0, &V22560_305_57231, &P22560_304};	/*Property = FileSaver*/
const Value V22560_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22560_306 = {S26, 0, &V22560_306_57237, &P22560_305};	/*Property = FileWriter*/
const Value V22560_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22560_307 = {S27, 0, &V22560_307_57239, &P22560_306};	/*Property = FormData*/
const Value V22560_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22560_309 = {S28, 0, &V22560_309_57246, &P22560_307};	/*Property = Fullscreen*/
const Value V22560_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22560_310 = {S29, 0, &V22560_310_57248, &P22560_309};	/*Property = GeoLocation*/
const Value V22560_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22560_312 = {S30, 0, &V22560_312_57254, &P22560_310};	/*Property = History*/
const Value V22560_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22560_314 = {S31, 0, &V22560_314_57261, &P22560_312};	/*Property = Html5*/
const Value V22560_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22560_313 = {S32, 0, &V22560_313_57258, &P22560_314};	/*Property = Html-Media-Capture*/
const Value V22560_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22560_315 = {S33, 0, &V22560_315_57264, &P22560_313};	/*Property = Iframe*/
const Value V22560_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22560_316 = {S34, 0, &V22560_316_57266, &P22560_315};	/*Property = IndexedDB*/
const Value V22560_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22560_317 = {S35, 0, &V22560_317_57269, &P22560_316};	/*Property = Json*/
const Value V22560_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22560_15 = {S37, 0, &V22560_15_29, &P22560_317};	/*Property = LayoutEngine*/
const Value V22560_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22560_344 = {S38, 0, &V22560_344_57272, &P22560_15};	/*Property = Masking*/
const Value V22560_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22560_320 = {S39, 0, &V22560_320_57290, &P22560_344};	/*Property = PostMessage*/
const Value V22560_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22560_321 = {S40, 0, &V22560_321_57293, &P22560_320};	/*Property = Progress*/
const Value V22560_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22560_322 = {S41, 0, &V22560_322_57296, &P22560_321};	/*Property = Prompts*/
const Value V22560_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22560_323 = {S42, 0, &V22560_323_57299, &P22560_322};	/*Property = Selector*/
const Value V22560_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22560_324 = {S43, 0, &V22560_324_57306, &P22560_323};	/*Property = Svg*/
const Value V22560_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22560_327 = {S44, 0, &V22560_327_57315, &P22560_324};	/*Property = TouchEvents*/
const Value V22560_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22560_328 = {S45, 0, &V22560_328_57318, &P22560_327};	/*Property = Track*/
const Value V22560_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22560_329 = {S46, 0, &V22560_329_57323, &P22560_328};	/*Property = Video*/
const Value V22560_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22560_330 = {S47, 0, &V22560_330_57326, &P22560_329};	/*Property = Viewport*/
const Value V22560_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22560_332 = {S48, 0, &V22560_332_57338, &P22560_330};	/*Property = WebWorkers*/
const Value V22560_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22560_333 = {S49, 0, &V22560_333_57341, &P22560_332};	/*Property = Xhr2*/
const Profile P22560 = {"22560", &P22560_333};	/*Profile ID - 22560*/
const Value V22527_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22527_279 = {S1, 0, &V22527_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22527_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22527_282 = {S3, 0, &V22527_282_57154, &P22527_279};	/*Property = BlobBuilder*/
const Value V22527_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22527_350 = {S4, 0, &V22527_350_57444, &P22527_282};	/*Property = Canvas*/
const Value V22527_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22527_283 = {S5, 0, &V22527_283_57159, &P22527_350};	/*Property = CssBackground*/
const Value V22527_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22527_284 = {S6, 0, &V22527_284_57162, &P22527_283};	/*Property = CssBorderImage*/
const Value V22527_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22527_285 = {S7, 0, &V22527_285_57166, &P22527_284};	/*Property = CssCanvas*/
const Value V22527_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22527_286 = {S8, 0, &V22527_286_57168, &P22527_285};	/*Property = CssColor*/
const Value V22527_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22527_237 = {S9, 0, &V22527_237_55786, &P22527_286};	/*Property = CssColumn*/
const Value V22527_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22527_287 = {S10, 0, &V22527_287_57172, &P22527_237};	/*Property = CssFlexbox*/
const Value V22527_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22527_288 = {S11, 0, &V22527_288_57174, &P22527_287};	/*Property = CssFont*/
const Value V22527_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22527_289 = {S12, 0, &V22527_289_57759, &P22527_288};	/*Property = CssImages*/
const Value V22527_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22527_291 = {S13, 0, &V22527_291_57180, &P22527_289};	/*Property = CssMediaQueries*/
const Value V22527_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22527_292 = {S14, 0, &V22527_292_57183, &P22527_291};	/*Property = CssMinMax*/
const Value V22527_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22527_293 = {S15, 0, &V22527_293_57187, &P22527_292};	/*Property = CssOverflow*/
const Value V22527_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22527_294 = {S16, 0, &V22527_294_57189, &P22527_293};	/*Property = CssPosition*/
const Value V22527_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22527_295 = {S17, 0, &V22527_295_57192, &P22527_294};	/*Property = CssText*/
const Value V22527_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22527_297 = {S18, 0, &V22527_297_57195, &P22527_295};	/*Property = CssTransforms*/
const Value V22527_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22527_241 = {S19, 0, &V22527_241_55799, &P22527_297};	/*Property = CssTransitions*/
const Value V22527_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22527_298 = {S20, 0, &V22527_298_57206, &P22527_241};	/*Property = CssUI*/
const Value V22527_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22527_300 = {S21, 0, &V22527_300_57215, &P22527_298};	/*Property = DataSet*/
const Value V22527_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22527_301 = {S22, 0, &V22527_301_57218, &P22527_300};	/*Property = DataUrl*/
const Value V22527_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22527_302 = {S23, 0, &V22527_302_57222, &P22527_301};	/*Property = DeviceOrientation*/
const Value V22527_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22527_304 = {S24, 0, &V22527_304_57227, &P22527_302};	/*Property = FileReader*/
const Value V22527_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22527_305 = {S25, 0, &V22527_305_57231, &P22527_304};	/*Property = FileSaver*/
const Value V22527_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22527_306 = {S26, 0, &V22527_306_57237, &P22527_305};	/*Property = FileWriter*/
const Value V22527_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22527_307 = {S27, 0, &V22527_307_57239, &P22527_306};	/*Property = FormData*/
const Value V22527_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22527_309 = {S28, 0, &V22527_309_57246, &P22527_307};	/*Property = Fullscreen*/
const Value V22527_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22527_310 = {S29, 0, &V22527_310_57248, &P22527_309};	/*Property = GeoLocation*/
const Value V22527_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22527_312 = {S30, 0, &V22527_312_57254, &P22527_310};	/*Property = History*/
const Value V22527_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22527_314 = {S31, 0, &V22527_314_57261, &P22527_312};	/*Property = Html5*/
const Value V22527_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22527_313 = {S32, 0, &V22527_313_57258, &P22527_314};	/*Property = Html-Media-Capture*/
const Value V22527_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22527_315 = {S33, 0, &V22527_315_57264, &P22527_313};	/*Property = Iframe*/
const Value V22527_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22527_316 = {S34, 0, &V22527_316_57267, &P22527_315};	/*Property = IndexedDB*/
const Value V22527_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22527_317 = {S35, 0, &V22527_317_57269, &P22527_316};	/*Property = Json*/
const Value V22527_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22527_15 = {S37, 0, &V22527_15_28, &P22527_317};	/*Property = LayoutEngine*/
const Value V22527_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22527_344 = {S38, 0, &V22527_344_57272, &P22527_15};	/*Property = Masking*/
const Value V22527_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22527_320 = {S39, 0, &V22527_320_57290, &P22527_344};	/*Property = PostMessage*/
const Value V22527_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22527_321 = {S40, 0, &V22527_321_57293, &P22527_320};	/*Property = Progress*/
const Value V22527_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22527_322 = {S41, 0, &V22527_322_57296, &P22527_321};	/*Property = Prompts*/
const Value V22527_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22527_323 = {S42, 0, &V22527_323_57299, &P22527_322};	/*Property = Selector*/
const Value V22527_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22527_324 = {S43, 0, &V22527_324_57305, &P22527_323};	/*Property = Svg*/
const Value V22527_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22527_327 = {S44, 0, &V22527_327_57315, &P22527_324};	/*Property = TouchEvents*/
const Value V22527_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22527_328 = {S45, 0, &V22527_328_57318, &P22527_327};	/*Property = Track*/
const Value V22527_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22527_329 = {S46, 0, &V22527_329_57323, &P22527_328};	/*Property = Video*/
const Value V22527_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22527_330 = {S47, 0, &V22527_330_57326, &P22527_329};	/*Property = Viewport*/
const Value V22527_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22527_332 = {S48, 0, &V22527_332_57339, &P22527_330};	/*Property = WebWorkers*/
const Value V22527_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22527_333 = {S49, 0, &V22527_333_57342, &P22527_332};	/*Property = Xhr2*/
const Profile P22527 = {"22527", &P22527_333};	/*Profile ID - 22527*/
const Value V22718_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_279 = {S1, 0, &V22718_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22718_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_282 = {S3, 0, &V22718_282_57155, &P22718_279};	/*Property = BlobBuilder*/
const Value V22718_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_350 = {S4, 0, &V22718_350_57446, &P22718_282};	/*Property = Canvas*/
const Value V22718_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_283 = {S5, 0, &V22718_283_57161, &P22718_350};	/*Property = CssBackground*/
const Value V22718_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_284 = {S6, 0, &V22718_284_57164, &P22718_283};	/*Property = CssBorderImage*/
const Value V22718_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_285 = {S7, 0, &V22718_285_57167, &P22718_284};	/*Property = CssCanvas*/
const Value V22718_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_286 = {S8, 0, &V22718_286_57170, &P22718_285};	/*Property = CssColor*/
const Value V22718_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_237 = {S9, 0, &V22718_237_55786, &P22718_286};	/*Property = CssColumn*/
const Value V22718_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_287 = {S10, 0, &V22718_287_57173, &P22718_237};	/*Property = CssFlexbox*/
const Value V22718_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_288 = {S11, 0, &V22718_288_57176, &P22718_287};	/*Property = CssFont*/
const Value V22718_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_289 = {S12, 0, &V22718_289_86899, &P22718_288};	/*Property = CssImages*/
const Value V22718_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_291 = {S13, 0, &V22718_291_57182, &P22718_289};	/*Property = CssMediaQueries*/
const Value V22718_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_292 = {S14, 0, &V22718_292_57185, &P22718_291};	/*Property = CssMinMax*/
const Value V22718_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_293 = {S15, 0, &V22718_293_57188, &P22718_292};	/*Property = CssOverflow*/
const Value V22718_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_294 = {S16, 0, &V22718_294_57191, &P22718_293};	/*Property = CssPosition*/
const Value V22718_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_295 = {S17, 0, &V22718_295_57194, &P22718_294};	/*Property = CssText*/
const Value V22718_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_297 = {S18, 0, &V22718_297_57197, &P22718_295};	/*Property = CssTransforms*/
const Value V22718_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_241 = {S19, 0, &V22718_241_55801, &P22718_297};	/*Property = CssTransitions*/
const Value V22718_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_298 = {S20, 0, &V22718_298_57208, &P22718_241};	/*Property = CssUI*/
const Value V22718_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_300 = {S21, 0, &V22718_300_57217, &P22718_298};	/*Property = DataSet*/
const Value V22718_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_301 = {S22, 0, &V22718_301_57220, &P22718_300};	/*Property = DataUrl*/
const Value V22718_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_302 = {S23, 0, &V22718_302_57223, &P22718_301};	/*Property = DeviceOrientation*/
const Value V22718_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_304 = {S24, 0, &V22718_304_57229, &P22718_302};	/*Property = FileReader*/
const Value V22718_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_305 = {S25, 0, &V22718_305_57232, &P22718_304};	/*Property = FileSaver*/
const Value V22718_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_306 = {S26, 0, &V22718_306_57238, &P22718_305};	/*Property = FileWriter*/
const Value V22718_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_307 = {S27, 0, &V22718_307_57241, &P22718_306};	/*Property = FormData*/
const Value V22718_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_309 = {S28, 0, &V22718_309_57247, &P22718_307};	/*Property = Fullscreen*/
const Value V22718_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_310 = {S29, 0, &V22718_310_57250, &P22718_309};	/*Property = GeoLocation*/
const Value V22718_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_312 = {S30, 0, &V22718_312_57256, &P22718_310};	/*Property = History*/
const Value V22718_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_314 = {S31, 0, &V22718_314_57262, &P22718_312};	/*Property = Html5*/
const Value V22718_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_313 = {S32, 0, &V22718_313_57259, &P22718_314};	/*Property = Html-Media-Capture*/
const Value V22718_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_315 = {S33, 0, &V22718_315_57265, &P22718_313};	/*Property = Iframe*/
const Value V22718_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_316 = {S34, 0, &V22718_316_57268, &P22718_315};	/*Property = IndexedDB*/
const Value V22718_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_317 = {S35, 0, &V22718_317_57271, &P22718_316};	/*Property = Json*/
const Value V22718_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P22718_15 = {S37, 0, &V22718_15_30, &P22718_317};	/*Property = LayoutEngine*/
const Value V22718_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_344 = {S38, 0, &V22718_344_57274, &P22718_15};	/*Property = Masking*/
const Value V22718_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_320 = {S39, 0, &V22718_320_57292, &P22718_344};	/*Property = PostMessage*/
const Value V22718_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_321 = {S40, 0, &V22718_321_57295, &P22718_320};	/*Property = Progress*/
const Value V22718_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_322 = {S41, 0, &V22718_322_57298, &P22718_321};	/*Property = Prompts*/
const Value V22718_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_323 = {S42, 0, &V22718_323_57301, &P22718_322};	/*Property = Selector*/
const Value V22718_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_324 = {S43, 0, &V22718_324_57307, &P22718_323};	/*Property = Svg*/
const Value V22718_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_327 = {S44, 0, &V22718_327_57316, &P22718_324};	/*Property = TouchEvents*/
const Value V22718_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_328 = {S45, 0, &V22718_328_57319, &P22718_327};	/*Property = Track*/
const Value V22718_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_329 = {S46, 0, &V22718_329_57325, &P22718_328};	/*Property = Video*/
const Value V22718_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_330 = {S47, 0, &V22718_330_57328, &P22718_329};	/*Property = Viewport*/
const Value V22718_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_332 = {S48, 0, &V22718_332_57340, &P22718_330};	/*Property = WebWorkers*/
const Value V22718_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22718_333 = {S49, 0, &V22718_333_57343, &P22718_332};	/*Property = Xhr2*/
const Profile P22718 = {"22718", &P22718_333};	/*Profile ID - 22718*/
const Profile P22275 = {"22275", NULL};	/*Profile ID - 22275*/
const Value V22382_1_1 = {S2, NULL};	/*Value = True*/
const Property P22382_1 = {S50, 0, &V22382_1_1, NULL};	/*Property = IsMobile*/
const Value V22382_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22382_5 = {S52, 0, &V22382_5_1227, &P22382_1};	/*Property = ScreenPixelsHeight*/
const Value V22382_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22382_4 = {S54, 0, &V22382_4_1229, &P22382_5};	/*Property = ScreenPixelsWidth*/
const Profile P22382 = {"22382", &P22382_4};	/*Profile ID - 22382*/
const Value V22390_1_1 = {S2, NULL};	/*Value = True*/
const Property P22390_1 = {S50, 0, &V22390_1_1, NULL};	/*Property = IsMobile*/
const Value V22390_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22390_5 = {S52, 0, &V22390_5_1222, &P22390_1};	/*Property = ScreenPixelsHeight*/
const Value V22390_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22390_4 = {S54, 0, &V22390_4_1224, &P22390_5};	/*Property = ScreenPixelsWidth*/
const Profile P22390 = {"22390", &P22390_4};	/*Profile ID - 22390*/
const Value V22396_1_1 = {S2, NULL};	/*Value = True*/
const Property P22396_1 = {S50, 0, &V22396_1_1, NULL};	/*Property = IsMobile*/
const Value V22396_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22396_5 = {S52, 0, &V22396_5_1222, &P22396_1};	/*Property = ScreenPixelsHeight*/
const Value V22396_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22396_4 = {S54, 0, &V22396_4_1224, &P22396_5};	/*Property = ScreenPixelsWidth*/
const Profile P22396 = {"22396", &P22396_4};	/*Profile ID - 22396*/
const Value V23657_1_2 = {S0, NULL};	/*Value = False*/
const Property P23657_1 = {S50, 0, &V23657_1_2, NULL};	/*Property = IsMobile*/
const Value V23657_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23657_5 = {S52, 0, &V23657_5_37569, &P23657_1};	/*Property = ScreenPixelsHeight*/
const Value V23657_4_53077 = {S129, NULL};	/*Value = 1366*/
const Property P23657_4 = {S54, 0, &V23657_4_53077, &P23657_5};	/*Property = ScreenPixelsWidth*/
const Profile P23657 = {"23657", &P23657_4};	/*Profile ID - 23657*/
const Value V23419_1_1 = {S2, NULL};	/*Value = True*/
const Property P23419_1 = {S50, 0, &V23419_1_1, NULL};	/*Property = IsMobile*/
const Value V23419_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23419_5 = {S52, 0, &V23419_5_1277, &P23419_1};	/*Property = ScreenPixelsHeight*/
const Value V23419_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23419_4 = {S54, 0, &V23419_4_2336, &P23419_5};	/*Property = ScreenPixelsWidth*/
const Profile P23419 = {"23419", &P23419_4};	/*Profile ID - 23419*/
const Value V23705_1_1 = {S2, NULL};	/*Value = True*/
const Property P23705_1 = {S50, 0, &V23705_1_1, NULL};	/*Property = IsMobile*/
const Value V23705_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23705_5 = {S52, 0, &V23705_5_1227, &P23705_1};	/*Property = ScreenPixelsHeight*/
const Value V23705_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23705_4 = {S54, 0, &V23705_4_1229, &P23705_5};	/*Property = ScreenPixelsWidth*/
const Profile P23705 = {"23705", &P23705_4};	/*Profile ID - 23705*/
const Value V23424_1_1 = {S2, NULL};	/*Value = True*/
const Property P23424_1 = {S50, 0, &V23424_1_1, NULL};	/*Property = IsMobile*/
const Value V23424_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23424_5 = {S52, 0, &V23424_5_1227, &P23424_1};	/*Property = ScreenPixelsHeight*/
const Value V23424_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23424_4 = {S54, 0, &V23424_4_1229, &P23424_5};	/*Property = ScreenPixelsWidth*/
const Profile P23424 = {"23424", &P23424_4};	/*Profile ID - 23424*/
const Value V16798_1_1 = {S2, NULL};	/*Value = True*/
const Property P16798_1 = {S50, 0, &V16798_1_1, NULL};	/*Property = IsMobile*/
const Value V16798_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P16798_5 = {S52, 0, &V16798_5_1304, &P16798_1};	/*Property = ScreenPixelsHeight*/
const Value V16798_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16798_4 = {S54, 0, &V16798_4_1224, &P16798_5};	/*Property = ScreenPixelsWidth*/
const Profile P16798 = {"16798", &P16798_4};	/*Profile ID - 16798*/
const Value V23391_1_1 = {S2, NULL};	/*Value = True*/
const Property P23391_1 = {S50, 0, &V23391_1_1, NULL};	/*Property = IsMobile*/
const Value V23391_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23391_5 = {S52, 0, &V23391_5_1832, &P23391_1};	/*Property = ScreenPixelsHeight*/
const Value V23391_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23391_4 = {S54, 0, &V23391_4_2336, &P23391_5};	/*Property = ScreenPixelsWidth*/
const Profile P23391 = {"23391", &P23391_4};	/*Profile ID - 23391*/
const Value V23468_1_1 = {S2, NULL};	/*Value = True*/
const Property P23468_1 = {S50, 0, &V23468_1_1, NULL};	/*Property = IsMobile*/
const Value V23468_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23468_5 = {S52, 0, &V23468_5_1227, &P23468_1};	/*Property = ScreenPixelsHeight*/
const Value V23468_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23468_4 = {S54, 0, &V23468_4_1229, &P23468_5};	/*Property = ScreenPixelsWidth*/
const Profile P23468 = {"23468", &P23468_4};	/*Profile ID - 23468*/
const Value V23070_1_1 = {S2, NULL};	/*Value = True*/
const Property P23070_1 = {S50, 0, &V23070_1_1, NULL};	/*Property = IsMobile*/
const Value V23070_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23070_5 = {S52, 0, &V23070_5_1403, &P23070_1};	/*Property = ScreenPixelsHeight*/
const Value V23070_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23070_4 = {S54, 0, &V23070_4_1404, &P23070_5};	/*Property = ScreenPixelsWidth*/
const Profile P23070 = {"23070", &P23070_4};	/*Profile ID - 23070*/
const Value V23431_1_1 = {S2, NULL};	/*Value = True*/
const Property P23431_1 = {S50, 0, &V23431_1_1, NULL};	/*Property = IsMobile*/
const Value V23431_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23431_5 = {S52, 0, &V23431_5_37569, &P23431_1};	/*Property = ScreenPixelsHeight*/
const Value V23431_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23431_4 = {S54, 0, &V23431_4_5645, &P23431_5};	/*Property = ScreenPixelsWidth*/
const Profile P23431 = {"23431", &P23431_4};	/*Profile ID - 23431*/
const Value V23262_1_1 = {S2, NULL};	/*Value = True*/
const Property P23262_1 = {S50, 0, &V23262_1_1, NULL};	/*Property = IsMobile*/
const Value V23262_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23262_5 = {S52, 0, &V23262_5_1227, &P23262_1};	/*Property = ScreenPixelsHeight*/
const Value V23262_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23262_4 = {S54, 0, &V23262_4_1229, &P23262_5};	/*Property = ScreenPixelsWidth*/
const Profile P23262 = {"23262", &P23262_4};	/*Profile ID - 23262*/
const Value V23290_1_1 = {S2, NULL};	/*Value = True*/
const Property P23290_1 = {S50, 0, &V23290_1_1, NULL};	/*Property = IsMobile*/
const Value V23290_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23290_5 = {S52, 0, &V23290_5_1277, &P23290_1};	/*Property = ScreenPixelsHeight*/
const Value V23290_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23290_4 = {S54, 0, &V23290_4_1279, &P23290_5};	/*Property = ScreenPixelsWidth*/
const Profile P23290 = {"23290", &P23290_4};	/*Profile ID - 23290*/
const Value V23617_1_1 = {S2, NULL};	/*Value = True*/
const Property P23617_1 = {S50, 0, &V23617_1_1, NULL};	/*Property = IsMobile*/
const Value V23617_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23617_5 = {S52, 0, &V23617_5_37, &P23617_1};	/*Property = ScreenPixelsHeight*/
const Value V23617_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23617_4 = {S54, 0, &V23617_4_36, &P23617_5};	/*Property = ScreenPixelsWidth*/
const Profile P23617 = {"23617", &P23617_4};	/*Profile ID - 23617*/
const Value V22954_1_1 = {S2, NULL};	/*Value = True*/
const Property P22954_1 = {S50, 0, &V22954_1_1, NULL};	/*Property = IsMobile*/
const Value V22954_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22954_5 = {S52, 0, &V22954_5_1227, &P22954_1};	/*Property = ScreenPixelsHeight*/
const Value V22954_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22954_4 = {S54, 0, &V22954_4_1229, &P22954_5};	/*Property = ScreenPixelsWidth*/
const Profile P22954 = {"22954", &P22954_4};	/*Profile ID - 22954*/
const Value V22605_1_1 = {S2, NULL};	/*Value = True*/
const Property P22605_1 = {S50, 0, &V22605_1_1, NULL};	/*Property = IsMobile*/
const Value V22605_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22605_5 = {S52, 0, &V22605_5_1227, &P22605_1};	/*Property = ScreenPixelsHeight*/
const Value V22605_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22605_4 = {S54, 0, &V22605_4_1229, &P22605_5};	/*Property = ScreenPixelsWidth*/
const Profile P22605 = {"22605", &P22605_4};	/*Profile ID - 22605*/
const Value V23164_1_1 = {S2, NULL};	/*Value = True*/
const Property P23164_1 = {S50, 0, &V23164_1_1, NULL};	/*Property = IsMobile*/
const Value V23164_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23164_5 = {S52, 0, &V23164_5_1832, &P23164_1};	/*Property = ScreenPixelsHeight*/
const Value V23164_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23164_4 = {S54, 0, &V23164_4_2336, &P23164_5};	/*Property = ScreenPixelsWidth*/
const Profile P23164 = {"23164", &P23164_4};	/*Profile ID - 23164*/
const Value V22856_1_1 = {S2, NULL};	/*Value = True*/
const Property P22856_1 = {S50, 0, &V22856_1_1, NULL};	/*Property = IsMobile*/
const Value V22856_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22856_5 = {S52, 0, &V22856_5_1227, &P22856_1};	/*Property = ScreenPixelsHeight*/
const Value V22856_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22856_4 = {S54, 0, &V22856_4_1229, &P22856_5};	/*Property = ScreenPixelsWidth*/
const Profile P22856 = {"22856", &P22856_4};	/*Profile ID - 22856*/
const Value V24584_1_1 = {S2, NULL};	/*Value = True*/
const Property P24584_1 = {S50, 0, &V24584_1_1, NULL};	/*Property = IsMobile*/
const Value V24584_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24584_5 = {S52, 0, &V24584_5_1277, &P24584_1};	/*Property = ScreenPixelsHeight*/
const Value V24584_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24584_4 = {S54, 0, &V24584_4_1279, &P24584_5};	/*Property = ScreenPixelsWidth*/
const Profile P24584 = {"24584", &P24584_4};	/*Profile ID - 24584*/
const Value V23436_1_1 = {S2, NULL};	/*Value = True*/
const Property P23436_1 = {S50, 0, &V23436_1_1, NULL};	/*Property = IsMobile*/
const Value V23436_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23436_5 = {S52, 0, &V23436_5_1403, &P23436_1};	/*Property = ScreenPixelsHeight*/
const Value V23436_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23436_4 = {S54, 0, &V23436_4_1404, &P23436_5};	/*Property = ScreenPixelsWidth*/
const Profile P23436 = {"23436", &P23436_4};	/*Profile ID - 23436*/
const Value V23384_1_1 = {S2, NULL};	/*Value = True*/
const Property P23384_1 = {S50, 0, &V23384_1_1, NULL};	/*Property = IsMobile*/
const Value V23384_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23384_5 = {S52, 0, &V23384_5_1277, &P23384_1};	/*Property = ScreenPixelsHeight*/
const Value V23384_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23384_4 = {S54, 0, &V23384_4_2336, &P23384_5};	/*Property = ScreenPixelsWidth*/
const Profile P23384 = {"23384", &P23384_4};	/*Profile ID - 23384*/
const Value V23622_1_1 = {S2, NULL};	/*Value = True*/
const Property P23622_1 = {S50, 0, &V23622_1_1, NULL};	/*Property = IsMobile*/
const Value V23622_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23622_5 = {S52, 0, &V23622_5_1458, &P23622_1};	/*Property = ScreenPixelsHeight*/
const Value V23622_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23622_4 = {S54, 0, &V23622_4_37617, &P23622_5};	/*Property = ScreenPixelsWidth*/
const Profile P23622 = {"23622", &P23622_4};	/*Profile ID - 23622*/
const Value V23558_1_1 = {S2, NULL};	/*Value = True*/
const Property P23558_1 = {S50, 0, &V23558_1_1, NULL};	/*Property = IsMobile*/
const Value V23558_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23558_5 = {S52, 0, &V23558_5_1269, &P23558_1};	/*Property = ScreenPixelsHeight*/
const Value V23558_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23558_4 = {S54, 0, &V23558_4_1229, &P23558_5};	/*Property = ScreenPixelsWidth*/
const Profile P23558 = {"23558", &P23558_4};	/*Profile ID - 23558*/
const Value V23255_1_1 = {S2, NULL};	/*Value = True*/
const Property P23255_1 = {S50, 0, &V23255_1_1, NULL};	/*Property = IsMobile*/
const Value V23255_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23255_5 = {S52, 0, &V23255_5_1269, &P23255_1};	/*Property = ScreenPixelsHeight*/
const Value V23255_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23255_4 = {S54, 0, &V23255_4_1229, &P23255_5};	/*Property = ScreenPixelsWidth*/
const Profile P23255 = {"23255", &P23255_4};	/*Profile ID - 23255*/
const Value V23959_1_1 = {S2, NULL};	/*Value = True*/
const Property P23959_1 = {S50, 0, &V23959_1_1, NULL};	/*Property = IsMobile*/
const Value V23959_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23959_5 = {S52, 0, &V23959_5_1277, &P23959_1};	/*Property = ScreenPixelsHeight*/
const Value V23959_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23959_4 = {S54, 0, &V23959_4_1279, &P23959_5};	/*Property = ScreenPixelsWidth*/
const Profile P23959 = {"23959", &P23959_4};	/*Profile ID - 23959*/
const Value V22745_1_1 = {S2, NULL};	/*Value = True*/
const Property P22745_1 = {S50, 0, &V22745_1_1, NULL};	/*Property = IsMobile*/
const Value V22745_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22745_5 = {S52, 0, &V22745_5_1227, &P22745_1};	/*Property = ScreenPixelsHeight*/
const Value V22745_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22745_4 = {S54, 0, &V22745_4_1229, &P22745_5};	/*Property = ScreenPixelsWidth*/
const Profile P22745 = {"22745", &P22745_4};	/*Profile ID - 22745*/
const Value V22619_1_1 = {S2, NULL};	/*Value = True*/
const Property P22619_1 = {S50, 0, &V22619_1_1, NULL};	/*Property = IsMobile*/
const Value V22619_5_37570 = {S132, NULL};	/*Value = 340*/
const Property P22619_5 = {S52, 0, &V22619_5_37570, &P22619_1};	/*Property = ScreenPixelsHeight*/
const Value V22619_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22619_4 = {S54, 0, &V22619_4_1224, &P22619_5};	/*Property = ScreenPixelsWidth*/
const Profile P22619 = {"22619", &P22619_4};	/*Profile ID - 22619*/
const Value V22754_1_1 = {S2, NULL};	/*Value = True*/
const Property P22754_1 = {S50, 0, &V22754_1_1, NULL};	/*Property = IsMobile*/
const Value V22754_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22754_5 = {S52, 0, &V22754_5_1458, &P22754_1};	/*Property = ScreenPixelsHeight*/
const Value V22754_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22754_4 = {S54, 0, &V22754_4_1460, &P22754_5};	/*Property = ScreenPixelsWidth*/
const Profile P22754 = {"22754", &P22754_4};	/*Profile ID - 22754*/
const Value V22713_1_1 = {S2, NULL};	/*Value = True*/
const Property P22713_1 = {S50, 0, &V22713_1_1, NULL};	/*Property = IsMobile*/
const Value V22713_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22713_5 = {S52, 0, &V22713_5_1227, &P22713_1};	/*Property = ScreenPixelsHeight*/
const Value V22713_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22713_4 = {S54, 0, &V22713_4_1229, &P22713_5};	/*Property = ScreenPixelsWidth*/
const Profile P22713 = {"22713", &P22713_4};	/*Profile ID - 22713*/
const Value V22683_1_1 = {S2, NULL};	/*Value = True*/
const Property P22683_1 = {S50, 0, &V22683_1_1, NULL};	/*Property = IsMobile*/
const Value V22683_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22683_5 = {S52, 0, &V22683_5_1277, &P22683_1};	/*Property = ScreenPixelsHeight*/
const Value V22683_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22683_4 = {S54, 0, &V22683_4_1279, &P22683_5};	/*Property = ScreenPixelsWidth*/
const Profile P22683 = {"22683", &P22683_4};	/*Profile ID - 22683*/
const Value V23154_1_1 = {S2, NULL};	/*Value = True*/
const Property P23154_1 = {S50, 0, &V23154_1_1, NULL};	/*Property = IsMobile*/
const Value V23154_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23154_5 = {S52, 0, &V23154_5_9981, &P23154_1};	/*Property = ScreenPixelsHeight*/
const Value V23154_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23154_4 = {S54, 0, &V23154_4_39098, &P23154_5};	/*Property = ScreenPixelsWidth*/
const Profile P23154 = {"23154", &P23154_4};	/*Profile ID - 23154*/
const Value V23296_1_1 = {S2, NULL};	/*Value = True*/
const Property P23296_1 = {S50, 0, &V23296_1_1, NULL};	/*Property = IsMobile*/
const Value V23296_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23296_5 = {S52, 0, &V23296_5_1227, &P23296_1};	/*Property = ScreenPixelsHeight*/
const Value V23296_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23296_4 = {S54, 0, &V23296_4_1229, &P23296_5};	/*Property = ScreenPixelsWidth*/
const Profile P23296 = {"23296", &P23296_4};	/*Profile ID - 23296*/
const Value V23781_1_1 = {S2, NULL};	/*Value = True*/
const Property P23781_1 = {S50, 0, &V23781_1_1, NULL};	/*Property = IsMobile*/
const Value V23781_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23781_5 = {S52, 0, &V23781_5_1277, &P23781_1};	/*Property = ScreenPixelsHeight*/
const Value V23781_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23781_4 = {S54, 0, &V23781_4_1279, &P23781_5};	/*Property = ScreenPixelsWidth*/
const Profile P23781 = {"23781", &P23781_4};	/*Profile ID - 23781*/
const Value V20712_1_1 = {S2, NULL};	/*Value = True*/
const Property P20712_1 = {S50, 0, &V20712_1_1, NULL};	/*Property = IsMobile*/
const Value V20712_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P20712_5 = {S52, 0, &V20712_5_1277, &P20712_1};	/*Property = ScreenPixelsHeight*/
const Value V20712_4_37618 = {S83, NULL};	/*Value = 272*/
const Property P20712_4 = {S54, 0, &V20712_4_37618, &P20712_5};	/*Property = ScreenPixelsWidth*/
const Profile P20712 = {"20712", &P20712_4};	/*Profile ID - 20712*/
const Value V22602_1_1 = {S2, NULL};	/*Value = True*/
const Property P22602_1 = {S50, 0, &V22602_1_1, NULL};	/*Property = IsMobile*/
const Value V22602_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22602_5 = {S52, 0, &V22602_5_1277, &P22602_1};	/*Property = ScreenPixelsHeight*/
const Value V22602_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22602_4 = {S54, 0, &V22602_4_1279, &P22602_5};	/*Property = ScreenPixelsWidth*/
const Profile P22602 = {"22602", &P22602_4};	/*Profile ID - 22602*/
const Value V23854_1_1 = {S2, NULL};	/*Value = True*/
const Property P23854_1 = {S50, 0, &V23854_1_1, NULL};	/*Property = IsMobile*/
const Value V23854_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23854_5 = {S52, 0, &V23854_5_1403, &P23854_1};	/*Property = ScreenPixelsHeight*/
const Value V23854_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23854_4 = {S54, 0, &V23854_4_1404, &P23854_5};	/*Property = ScreenPixelsWidth*/
const Profile P23854 = {"23854", &P23854_4};	/*Profile ID - 23854*/
const Profile P22887 = {"22887", NULL};	/*Profile ID - 22887*/
const Value V22873_1_1 = {S2, NULL};	/*Value = True*/
const Property P22873_1 = {S50, 0, &V22873_1_1, NULL};	/*Property = IsMobile*/
const Value V22873_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22873_5 = {S52, 0, &V22873_5_9981, &P22873_1};	/*Property = ScreenPixelsHeight*/
const Value V22873_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22873_4 = {S54, 0, &V22873_4_2336, &P22873_5};	/*Property = ScreenPixelsWidth*/
const Profile P22873 = {"22873", &P22873_4};	/*Profile ID - 22873*/
const Value V22610_1_1 = {S2, NULL};	/*Value = True*/
const Property P22610_1 = {S50, 0, &V22610_1_1, NULL};	/*Property = IsMobile*/
const Value V22610_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22610_5 = {S52, 0, &V22610_5_1277, &P22610_1};	/*Property = ScreenPixelsHeight*/
const Value V22610_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22610_4 = {S54, 0, &V22610_4_1279, &P22610_5};	/*Property = ScreenPixelsWidth*/
const Profile P22610 = {"22610", &P22610_4};	/*Profile ID - 22610*/
const Value V20598_1_1 = {S2, NULL};	/*Value = True*/
const Property P20598_1 = {S50, 0, &V20598_1_1, NULL};	/*Property = IsMobile*/
const Value V20598_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20598_5 = {S52, 0, &V20598_5_1227, &P20598_1};	/*Property = ScreenPixelsHeight*/
const Value V20598_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20598_4 = {S54, 0, &V20598_4_1229, &P20598_5};	/*Property = ScreenPixelsWidth*/
const Profile P20598 = {"20598", &P20598_4};	/*Profile ID - 20598*/
const Value V23196_1_1 = {S2, NULL};	/*Value = True*/
const Property P23196_1 = {S50, 0, &V23196_1_1, NULL};	/*Property = IsMobile*/
const Value V23196_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23196_5 = {S52, 0, &V23196_5_1458, &P23196_1};	/*Property = ScreenPixelsHeight*/
const Value V23196_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23196_4 = {S54, 0, &V23196_4_37617, &P23196_5};	/*Property = ScreenPixelsWidth*/
const Profile P23196 = {"23196", &P23196_4};	/*Profile ID - 23196*/
const Value V22903_1_1 = {S2, NULL};	/*Value = True*/
const Property P22903_1 = {S50, 0, &V22903_1_1, NULL};	/*Property = IsMobile*/
const Value V22903_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22903_5 = {S52, 0, &V22903_5_1277, &P22903_1};	/*Property = ScreenPixelsHeight*/
const Value V22903_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22903_4 = {S54, 0, &V22903_4_1279, &P22903_5};	/*Property = ScreenPixelsWidth*/
const Profile P22903 = {"22903", &P22903_4};	/*Profile ID - 22903*/
const Value V19790_1_1 = {S2, NULL};	/*Value = True*/
const Property P19790_1 = {S50, 0, &V19790_1_1, NULL};	/*Property = IsMobile*/
const Value V19790_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P19790_5 = {S52, 0, &V19790_5_59138, &P19790_1};	/*Property = ScreenPixelsHeight*/
const Value V19790_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P19790_4 = {S54, 0, &V19790_4_59970, &P19790_5};	/*Property = ScreenPixelsWidth*/
const Profile P19790 = {"19790", &P19790_4};	/*Profile ID - 19790*/
const Value V22587_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22587_279 = {S1, 0, &V22587_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22587_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22587_282 = {S3, 0, &V22587_282_57154, &P22587_279};	/*Property = BlobBuilder*/
const Value V22587_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22587_350 = {S4, 0, &V22587_350_57444, &P22587_282};	/*Property = Canvas*/
const Value V22587_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22587_283 = {S5, 0, &V22587_283_57159, &P22587_350};	/*Property = CssBackground*/
const Value V22587_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22587_284 = {S6, 0, &V22587_284_57162, &P22587_283};	/*Property = CssBorderImage*/
const Value V22587_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22587_285 = {S7, 0, &V22587_285_57166, &P22587_284};	/*Property = CssCanvas*/
const Value V22587_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22587_286 = {S8, 0, &V22587_286_57168, &P22587_285};	/*Property = CssColor*/
const Value V22587_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22587_237 = {S9, 0, &V22587_237_55786, &P22587_286};	/*Property = CssColumn*/
const Value V22587_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22587_287 = {S10, 0, &V22587_287_57172, &P22587_237};	/*Property = CssFlexbox*/
const Value V22587_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22587_288 = {S11, 0, &V22587_288_57174, &P22587_287};	/*Property = CssFont*/
const Value V22587_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22587_289 = {S12, 0, &V22587_289_57759, &P22587_288};	/*Property = CssImages*/
const Value V22587_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22587_291 = {S13, 0, &V22587_291_57180, &P22587_289};	/*Property = CssMediaQueries*/
const Value V22587_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22587_292 = {S14, 0, &V22587_292_57183, &P22587_291};	/*Property = CssMinMax*/
const Value V22587_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22587_293 = {S15, 0, &V22587_293_57187, &P22587_292};	/*Property = CssOverflow*/
const Value V22587_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22587_294 = {S16, 0, &V22587_294_57189, &P22587_293};	/*Property = CssPosition*/
const Value V22587_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22587_295 = {S17, 0, &V22587_295_57192, &P22587_294};	/*Property = CssText*/
const Value V22587_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22587_297 = {S18, 0, &V22587_297_57195, &P22587_295};	/*Property = CssTransforms*/
const Value V22587_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22587_241 = {S19, 0, &V22587_241_55799, &P22587_297};	/*Property = CssTransitions*/
const Value V22587_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22587_298 = {S20, 0, &V22587_298_57206, &P22587_241};	/*Property = CssUI*/
const Value V22587_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22587_300 = {S21, 0, &V22587_300_57215, &P22587_298};	/*Property = DataSet*/
const Value V22587_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22587_301 = {S22, 0, &V22587_301_57218, &P22587_300};	/*Property = DataUrl*/
const Value V22587_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22587_302 = {S23, 0, &V22587_302_57222, &P22587_301};	/*Property = DeviceOrientation*/
const Value V22587_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22587_304 = {S24, 0, &V22587_304_57227, &P22587_302};	/*Property = FileReader*/
const Value V22587_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22587_305 = {S25, 0, &V22587_305_57231, &P22587_304};	/*Property = FileSaver*/
const Value V22587_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22587_306 = {S26, 0, &V22587_306_57237, &P22587_305};	/*Property = FileWriter*/
const Value V22587_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22587_307 = {S27, 0, &V22587_307_57239, &P22587_306};	/*Property = FormData*/
const Value V22587_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22587_309 = {S28, 0, &V22587_309_57246, &P22587_307};	/*Property = Fullscreen*/
const Value V22587_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22587_310 = {S29, 0, &V22587_310_57248, &P22587_309};	/*Property = GeoLocation*/
const Value V22587_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22587_312 = {S30, 0, &V22587_312_57254, &P22587_310};	/*Property = History*/
const Value V22587_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22587_314 = {S31, 0, &V22587_314_57260, &P22587_312};	/*Property = Html5*/
const Value V22587_313_57257 = {S2, NULL};	/*Value = True*/
const Property P22587_313 = {S32, 0, &V22587_313_57257, &P22587_314};	/*Property = Html-Media-Capture*/
const Value V22587_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22587_315 = {S33, 0, &V22587_315_57264, &P22587_313};	/*Property = Iframe*/
const Value V22587_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22587_316 = {S34, 0, &V22587_316_57266, &P22587_315};	/*Property = IndexedDB*/
const Value V22587_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22587_317 = {S35, 0, &V22587_317_57269, &P22587_316};	/*Property = Json*/
const Value V22587_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22587_15 = {S37, 0, &V22587_15_28, &P22587_317};	/*Property = LayoutEngine*/
const Value V22587_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22587_344 = {S38, 0, &V22587_344_57272, &P22587_15};	/*Property = Masking*/
const Value V22587_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22587_320 = {S39, 0, &V22587_320_57290, &P22587_344};	/*Property = PostMessage*/
const Value V22587_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22587_321 = {S40, 0, &V22587_321_57293, &P22587_320};	/*Property = Progress*/
const Value V22587_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22587_322 = {S41, 0, &V22587_322_57296, &P22587_321};	/*Property = Prompts*/
const Value V22587_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22587_323 = {S42, 0, &V22587_323_57299, &P22587_322};	/*Property = Selector*/
const Value V22587_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22587_324 = {S43, 0, &V22587_324_57305, &P22587_323};	/*Property = Svg*/
const Value V22587_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22587_327 = {S44, 0, &V22587_327_57314, &P22587_324};	/*Property = TouchEvents*/
const Value V22587_328_57317 = {S2, NULL};	/*Value = True*/
const Property P22587_328 = {S45, 0, &V22587_328_57317, &P22587_327};	/*Property = Track*/
const Value V22587_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22587_329 = {S46, 0, &V22587_329_57323, &P22587_328};	/*Property = Video*/
const Value V22587_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22587_330 = {S47, 0, &V22587_330_57326, &P22587_329};	/*Property = Viewport*/
const Value V22587_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22587_332 = {S48, 0, &V22587_332_57338, &P22587_330};	/*Property = WebWorkers*/
const Value V22587_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22587_333 = {S49, 0, &V22587_333_57341, &P22587_332};	/*Property = Xhr2*/
const Profile P22587 = {"22587", &P22587_333};	/*Profile ID - 22587*/
const Value V23358_1_1 = {S2, NULL};	/*Value = True*/
const Property P23358_1 = {S50, 0, &V23358_1_1, NULL};	/*Property = IsMobile*/
const Value V23358_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23358_5 = {S52, 0, &V23358_5_37, &P23358_1};	/*Property = ScreenPixelsHeight*/
const Value V23358_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23358_4 = {S54, 0, &V23358_4_36, &P23358_5};	/*Property = ScreenPixelsWidth*/
const Profile P23358 = {"23358", &P23358_4};	/*Profile ID - 23358*/
const Value V23529_1_1 = {S2, NULL};	/*Value = True*/
const Property P23529_1 = {S50, 0, &V23529_1_1, NULL};	/*Property = IsMobile*/
const Value V23529_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23529_5 = {S52, 0, &V23529_5_1227, &P23529_1};	/*Property = ScreenPixelsHeight*/
const Value V23529_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23529_4 = {S54, 0, &V23529_4_1229, &P23529_5};	/*Property = ScreenPixelsWidth*/
const Profile P23529 = {"23529", &P23529_4};	/*Profile ID - 23529*/
const Value V23816_1_1 = {S2, NULL};	/*Value = True*/
const Property P23816_1 = {S50, 0, &V23816_1_1, NULL};	/*Property = IsMobile*/
const Value V23816_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23816_5 = {S52, 0, &V23816_5_1458, &P23816_1};	/*Property = ScreenPixelsHeight*/
const Value V23816_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23816_4 = {S54, 0, &V23816_4_1460, &P23816_5};	/*Property = ScreenPixelsWidth*/
const Profile P23816 = {"23816", &P23816_4};	/*Profile ID - 23816*/
const Value V22735_1_1 = {S2, NULL};	/*Value = True*/
const Property P22735_1 = {S50, 0, &V22735_1_1, NULL};	/*Property = IsMobile*/
const Value V22735_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22735_5 = {S52, 0, &V22735_5_1403, &P22735_1};	/*Property = ScreenPixelsHeight*/
const Value V22735_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22735_4 = {S54, 0, &V22735_4_1404, &P22735_5};	/*Property = ScreenPixelsWidth*/
const Profile P22735 = {"22735", &P22735_4};	/*Profile ID - 22735*/
const Value V22660_1_1 = {S2, NULL};	/*Value = True*/
const Property P22660_1 = {S50, 0, &V22660_1_1, NULL};	/*Property = IsMobile*/
const Value V22660_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22660_5 = {S52, 0, &V22660_5_1403, &P22660_1};	/*Property = ScreenPixelsHeight*/
const Value V22660_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22660_4 = {S54, 0, &V22660_4_1404, &P22660_5};	/*Property = ScreenPixelsWidth*/
const Profile P22660 = {"22660", &P22660_4};	/*Profile ID - 22660*/
const Value V23013_1_1 = {S2, NULL};	/*Value = True*/
const Property P23013_1 = {S50, 0, &V23013_1_1, NULL};	/*Property = IsMobile*/
const Value V23013_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23013_5 = {S52, 0, &V23013_5_1227, &P23013_1};	/*Property = ScreenPixelsHeight*/
const Value V23013_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23013_4 = {S54, 0, &V23013_4_1229, &P23013_5};	/*Property = ScreenPixelsWidth*/
const Profile P23013 = {"23013", &P23013_4};	/*Profile ID - 23013*/
const Value V22960_1_1 = {S2, NULL};	/*Value = True*/
const Property P22960_1 = {S50, 0, &V22960_1_1, NULL};	/*Property = IsMobile*/
const Value V22960_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22960_5 = {S52, 0, &V22960_5_1403, &P22960_1};	/*Property = ScreenPixelsHeight*/
const Value V22960_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22960_4 = {S54, 0, &V22960_4_1404, &P22960_5};	/*Property = ScreenPixelsWidth*/
const Profile P22960 = {"22960", &P22960_4};	/*Profile ID - 22960*/
const Value V22810_1_1 = {S2, NULL};	/*Value = True*/
const Property P22810_1 = {S50, 0, &V22810_1_1, NULL};	/*Property = IsMobile*/
const Value V22810_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22810_5 = {S52, 0, &V22810_5_9981, &P22810_1};	/*Property = ScreenPixelsHeight*/
const Value V22810_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22810_4 = {S54, 0, &V22810_4_2336, &P22810_5};	/*Property = ScreenPixelsWidth*/
const Profile P22810 = {"22810", &P22810_4};	/*Profile ID - 22810*/
const Value V22761_1_1 = {S2, NULL};	/*Value = True*/
const Property P22761_1 = {S50, 0, &V22761_1_1, NULL};	/*Property = IsMobile*/
const Value V22761_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22761_5 = {S52, 0, &V22761_5_1227, &P22761_1};	/*Property = ScreenPixelsHeight*/
const Value V22761_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22761_4 = {S54, 0, &V22761_4_1229, &P22761_5};	/*Property = ScreenPixelsWidth*/
const Profile P22761 = {"22761", &P22761_4};	/*Profile ID - 22761*/
const Value V23842_1_1 = {S2, NULL};	/*Value = True*/
const Property P23842_1 = {S50, 0, &V23842_1_1, NULL};	/*Property = IsMobile*/
const Value V23842_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23842_5 = {S52, 0, &V23842_5_1403, &P23842_1};	/*Property = ScreenPixelsHeight*/
const Value V23842_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23842_4 = {S54, 0, &V23842_4_1404, &P23842_5};	/*Property = ScreenPixelsWidth*/
const Profile P23842 = {"23842", &P23842_4};	/*Profile ID - 23842*/
const Value V22805_1_1 = {S2, NULL};	/*Value = True*/
const Property P22805_1 = {S50, 0, &V22805_1_1, NULL};	/*Property = IsMobile*/
const Value V22805_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22805_5 = {S52, 0, &V22805_5_1227, &P22805_1};	/*Property = ScreenPixelsHeight*/
const Value V22805_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22805_4 = {S54, 0, &V22805_4_1229, &P22805_5};	/*Property = ScreenPixelsWidth*/
const Profile P22805 = {"22805", &P22805_4};	/*Profile ID - 22805*/
const Value V22652_1_1 = {S2, NULL};	/*Value = True*/
const Property P22652_1 = {S50, 0, &V22652_1_1, NULL};	/*Property = IsMobile*/
const Value V22652_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22652_5 = {S52, 0, &V22652_5_1277, &P22652_1};	/*Property = ScreenPixelsHeight*/
const Value V22652_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22652_4 = {S54, 0, &V22652_4_1279, &P22652_5};	/*Property = ScreenPixelsWidth*/
const Profile P22652 = {"22652", &P22652_4};	/*Profile ID - 22652*/
const Value V22584_1_1 = {S2, NULL};	/*Value = True*/
const Property P22584_1 = {S50, 0, &V22584_1_1, NULL};	/*Property = IsMobile*/
const Value V22584_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22584_5 = {S52, 0, &V22584_5_59138, &P22584_1};	/*Property = ScreenPixelsHeight*/
const Value V22584_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22584_4 = {S54, 0, &V22584_4_59970, &P22584_5};	/*Property = ScreenPixelsWidth*/
const Profile P22584 = {"22584", &P22584_4};	/*Profile ID - 22584*/
const Value V23118_1_1 = {S2, NULL};	/*Value = True*/
const Property P23118_1 = {S50, 0, &V23118_1_1, NULL};	/*Property = IsMobile*/
const Value V23118_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23118_5 = {S52, 0, &V23118_5_1277, &P23118_1};	/*Property = ScreenPixelsHeight*/
const Value V23118_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23118_4 = {S54, 0, &V23118_4_1279, &P23118_5};	/*Property = ScreenPixelsWidth*/
const Profile P23118 = {"23118", &P23118_4};	/*Profile ID - 23118*/
const Value V23165_1_1 = {S2, NULL};	/*Value = True*/
const Property P23165_1 = {S50, 0, &V23165_1_1, NULL};	/*Property = IsMobile*/
const Value V23165_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23165_5 = {S52, 0, &V23165_5_9981, &P23165_1};	/*Property = ScreenPixelsHeight*/
const Value V23165_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23165_4 = {S54, 0, &V23165_4_39098, &P23165_5};	/*Property = ScreenPixelsWidth*/
const Profile P23165 = {"23165", &P23165_4};	/*Profile ID - 23165*/
const Value V20855_1_1 = {S2, NULL};	/*Value = True*/
const Property P20855_1 = {S50, 0, &V20855_1_1, NULL};	/*Property = IsMobile*/
const Value V20855_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20855_5 = {S52, 0, &V20855_5_1458, &P20855_1};	/*Property = ScreenPixelsHeight*/
const Value V20855_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20855_4 = {S54, 0, &V20855_4_1460, &P20855_5};	/*Property = ScreenPixelsWidth*/
const Profile P20855 = {"20855", &P20855_4};	/*Profile ID - 20855*/
const Value V22588_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22588_279 = {S1, 0, &V22588_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22588_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22588_282 = {S3, 0, &V22588_282_57154, &P22588_279};	/*Property = BlobBuilder*/
const Value V22588_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22588_350 = {S4, 0, &V22588_350_57444, &P22588_282};	/*Property = Canvas*/
const Value V22588_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22588_283 = {S5, 0, &V22588_283_57159, &P22588_350};	/*Property = CssBackground*/
const Value V22588_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22588_284 = {S6, 0, &V22588_284_57162, &P22588_283};	/*Property = CssBorderImage*/
const Value V22588_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22588_285 = {S7, 0, &V22588_285_57166, &P22588_284};	/*Property = CssCanvas*/
const Value V22588_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22588_286 = {S8, 0, &V22588_286_57168, &P22588_285};	/*Property = CssColor*/
const Value V22588_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22588_237 = {S9, 0, &V22588_237_55786, &P22588_286};	/*Property = CssColumn*/
const Value V22588_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22588_287 = {S10, 0, &V22588_287_57172, &P22588_237};	/*Property = CssFlexbox*/
const Value V22588_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22588_288 = {S11, 0, &V22588_288_57174, &P22588_287};	/*Property = CssFont*/
const Value V22588_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22588_289 = {S12, 0, &V22588_289_57759, &P22588_288};	/*Property = CssImages*/
const Value V22588_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22588_291 = {S13, 0, &V22588_291_57180, &P22588_289};	/*Property = CssMediaQueries*/
const Value V22588_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22588_292 = {S14, 0, &V22588_292_57183, &P22588_291};	/*Property = CssMinMax*/
const Value V22588_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22588_293 = {S15, 0, &V22588_293_57187, &P22588_292};	/*Property = CssOverflow*/
const Value V22588_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22588_294 = {S16, 0, &V22588_294_57189, &P22588_293};	/*Property = CssPosition*/
const Value V22588_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22588_295 = {S17, 0, &V22588_295_57192, &P22588_294};	/*Property = CssText*/
const Value V22588_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22588_297 = {S18, 0, &V22588_297_57195, &P22588_295};	/*Property = CssTransforms*/
const Value V22588_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22588_241 = {S19, 0, &V22588_241_55799, &P22588_297};	/*Property = CssTransitions*/
const Value V22588_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22588_298 = {S20, 0, &V22588_298_57206, &P22588_241};	/*Property = CssUI*/
const Value V22588_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22588_300 = {S21, 0, &V22588_300_57215, &P22588_298};	/*Property = DataSet*/
const Value V22588_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22588_301 = {S22, 0, &V22588_301_57218, &P22588_300};	/*Property = DataUrl*/
const Value V22588_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22588_302 = {S23, 0, &V22588_302_57222, &P22588_301};	/*Property = DeviceOrientation*/
const Value V22588_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22588_304 = {S24, 0, &V22588_304_57227, &P22588_302};	/*Property = FileReader*/
const Value V22588_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22588_305 = {S25, 0, &V22588_305_57231, &P22588_304};	/*Property = FileSaver*/
const Value V22588_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22588_306 = {S26, 0, &V22588_306_57237, &P22588_305};	/*Property = FileWriter*/
const Value V22588_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22588_307 = {S27, 0, &V22588_307_57239, &P22588_306};	/*Property = FormData*/
const Value V22588_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22588_309 = {S28, 0, &V22588_309_57246, &P22588_307};	/*Property = Fullscreen*/
const Value V22588_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22588_310 = {S29, 0, &V22588_310_57248, &P22588_309};	/*Property = GeoLocation*/
const Value V22588_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22588_312 = {S30, 0, &V22588_312_57254, &P22588_310};	/*Property = History*/
const Value V22588_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22588_314 = {S31, 0, &V22588_314_57260, &P22588_312};	/*Property = Html5*/
const Value V22588_313_57257 = {S2, NULL};	/*Value = True*/
const Property P22588_313 = {S32, 0, &V22588_313_57257, &P22588_314};	/*Property = Html-Media-Capture*/
const Value V22588_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22588_315 = {S33, 0, &V22588_315_57264, &P22588_313};	/*Property = Iframe*/
const Value V22588_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22588_316 = {S34, 0, &V22588_316_57266, &P22588_315};	/*Property = IndexedDB*/
const Value V22588_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22588_317 = {S35, 0, &V22588_317_57269, &P22588_316};	/*Property = Json*/
const Value V22588_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22588_15 = {S37, 0, &V22588_15_28, &P22588_317};	/*Property = LayoutEngine*/
const Value V22588_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22588_344 = {S38, 0, &V22588_344_57272, &P22588_15};	/*Property = Masking*/
const Value V22588_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22588_320 = {S39, 0, &V22588_320_57290, &P22588_344};	/*Property = PostMessage*/
const Value V22588_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22588_321 = {S40, 0, &V22588_321_57293, &P22588_320};	/*Property = Progress*/
const Value V22588_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22588_322 = {S41, 0, &V22588_322_57296, &P22588_321};	/*Property = Prompts*/
const Value V22588_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22588_323 = {S42, 0, &V22588_323_57299, &P22588_322};	/*Property = Selector*/
const Value V22588_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22588_324 = {S43, 0, &V22588_324_57305, &P22588_323};	/*Property = Svg*/
const Value V22588_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22588_327 = {S44, 0, &V22588_327_57314, &P22588_324};	/*Property = TouchEvents*/
const Value V22588_328_57317 = {S2, NULL};	/*Value = True*/
const Property P22588_328 = {S45, 0, &V22588_328_57317, &P22588_327};	/*Property = Track*/
const Value V22588_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22588_329 = {S46, 0, &V22588_329_57323, &P22588_328};	/*Property = Video*/
const Value V22588_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22588_330 = {S47, 0, &V22588_330_57326, &P22588_329};	/*Property = Viewport*/
const Value V22588_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22588_332 = {S48, 0, &V22588_332_57339, &P22588_330};	/*Property = WebWorkers*/
const Value V22588_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22588_333 = {S49, 0, &V22588_333_57341, &P22588_332};	/*Property = Xhr2*/
const Profile P22588 = {"22588", &P22588_333};	/*Profile ID - 22588*/
const Value V22926_1_1 = {S2, NULL};	/*Value = True*/
const Property P22926_1 = {S50, 0, &V22926_1_1, NULL};	/*Property = IsMobile*/
const Value V22926_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22926_5 = {S52, 0, &V22926_5_1458, &P22926_1};	/*Property = ScreenPixelsHeight*/
const Value V22926_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22926_4 = {S54, 0, &V22926_4_1460, &P22926_5};	/*Property = ScreenPixelsWidth*/
const Profile P22926 = {"22926", &P22926_4};	/*Profile ID - 22926*/
const Value V22620_1_1 = {S2, NULL};	/*Value = True*/
const Property P22620_1 = {S50, 0, &V22620_1_1, NULL};	/*Property = IsMobile*/
const Value V22620_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22620_5 = {S52, 0, &V22620_5_1222, &P22620_1};	/*Property = ScreenPixelsHeight*/
const Value V22620_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22620_4 = {S54, 0, &V22620_4_1224, &P22620_5};	/*Property = ScreenPixelsWidth*/
const Profile P22620 = {"22620", &P22620_4};	/*Profile ID - 22620*/
const Value V23373_1_1 = {S2, NULL};	/*Value = True*/
const Property P23373_1 = {S50, 0, &V23373_1_1, NULL};	/*Property = IsMobile*/
const Value V23373_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23373_5 = {S52, 0, &V23373_5_1227, &P23373_1};	/*Property = ScreenPixelsHeight*/
const Value V23373_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23373_4 = {S54, 0, &V23373_4_1229, &P23373_5};	/*Property = ScreenPixelsWidth*/
const Profile P23373 = {"23373", &P23373_4};	/*Profile ID - 23373*/
const Value V22596_1_1 = {S2, NULL};	/*Value = True*/
const Property P22596_1 = {S50, 0, &V22596_1_1, NULL};	/*Property = IsMobile*/
const Value V22596_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22596_5 = {S52, 0, &V22596_5_59138, &P22596_1};	/*Property = ScreenPixelsHeight*/
const Value V22596_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22596_4 = {S54, 0, &V22596_4_59970, &P22596_5};	/*Property = ScreenPixelsWidth*/
const Profile P22596 = {"22596", &P22596_4};	/*Profile ID - 22596*/
const Value V22809_1_1 = {S2, NULL};	/*Value = True*/
const Property P22809_1 = {S50, 0, &V22809_1_1, NULL};	/*Property = IsMobile*/
const Value V22809_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22809_5 = {S52, 0, &V22809_5_1403, &P22809_1};	/*Property = ScreenPixelsHeight*/
const Value V22809_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22809_4 = {S54, 0, &V22809_4_1404, &P22809_5};	/*Property = ScreenPixelsWidth*/
const Profile P22809 = {"22809", &P22809_4};	/*Profile ID - 22809*/
const Value V23076_1_1 = {S2, NULL};	/*Value = True*/
const Property P23076_1 = {S50, 0, &V23076_1_1, NULL};	/*Property = IsMobile*/
const Value V23076_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23076_5 = {S52, 0, &V23076_5_37569, &P23076_1};	/*Property = ScreenPixelsHeight*/
const Value V23076_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23076_4 = {S54, 0, &V23076_4_5645, &P23076_5};	/*Property = ScreenPixelsWidth*/
const Profile P23076 = {"23076", &P23076_4};	/*Profile ID - 23076*/
const Value V21186_1_1 = {S2, NULL};	/*Value = True*/
const Property P21186_1 = {S50, 0, &V21186_1_1, NULL};	/*Property = IsMobile*/
const Value V21186_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21186_5 = {S52, 0, &V21186_5_37, &P21186_1};	/*Property = ScreenPixelsHeight*/
const Value V21186_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21186_4 = {S54, 0, &V21186_4_36, &P21186_5};	/*Property = ScreenPixelsWidth*/
const Profile P21186 = {"21186", &P21186_4};	/*Profile ID - 21186*/
const Value V22585_1_1 = {S2, NULL};	/*Value = True*/
const Property P22585_1 = {S50, 0, &V22585_1_1, NULL};	/*Property = IsMobile*/
const Value V22585_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22585_5 = {S52, 0, &V22585_5_59138, &P22585_1};	/*Property = ScreenPixelsHeight*/
const Value V22585_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22585_4 = {S54, 0, &V22585_4_59970, &P22585_5};	/*Property = ScreenPixelsWidth*/
const Profile P22585 = {"22585", &P22585_4};	/*Profile ID - 22585*/
const Value V22603_1_1 = {S2, NULL};	/*Value = True*/
const Property P22603_1 = {S50, 0, &V22603_1_1, NULL};	/*Property = IsMobile*/
const Value V22603_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22603_5 = {S52, 0, &V22603_5_1227, &P22603_1};	/*Property = ScreenPixelsHeight*/
const Value V22603_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22603_4 = {S54, 0, &V22603_4_1229, &P22603_5};	/*Property = ScreenPixelsWidth*/
const Profile P22603 = {"22603", &P22603_4};	/*Profile ID - 22603*/
const Value V22601_1_1 = {S2, NULL};	/*Value = True*/
const Property P22601_1 = {S50, 0, &V22601_1_1, NULL};	/*Property = IsMobile*/
const Value V22601_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22601_5 = {S52, 0, &V22601_5_59138, &P22601_1};	/*Property = ScreenPixelsHeight*/
const Value V22601_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22601_4 = {S54, 0, &V22601_4_59970, &P22601_5};	/*Property = ScreenPixelsWidth*/
const Profile P22601 = {"22601", &P22601_4};	/*Profile ID - 22601*/
const Value V23195_1_1 = {S2, NULL};	/*Value = True*/
const Property P23195_1 = {S50, 0, &V23195_1_1, NULL};	/*Property = IsMobile*/
const Value V23195_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23195_5 = {S52, 0, &V23195_5_1458, &P23195_1};	/*Property = ScreenPixelsHeight*/
const Value V23195_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23195_4 = {S54, 0, &V23195_4_37617, &P23195_5};	/*Property = ScreenPixelsWidth*/
const Profile P23195 = {"23195", &P23195_4};	/*Profile ID - 23195*/
const Value V22663_1_1 = {S2, NULL};	/*Value = True*/
const Property P22663_1 = {S50, 0, &V22663_1_1, NULL};	/*Property = IsMobile*/
const Value V22663_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22663_5 = {S52, 0, &V22663_5_1227, &P22663_1};	/*Property = ScreenPixelsHeight*/
const Value V22663_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22663_4 = {S54, 0, &V22663_4_1229, &P22663_5};	/*Property = ScreenPixelsWidth*/
const Profile P22663 = {"22663", &P22663_4};	/*Profile ID - 22663*/
const Value V24138_1_1 = {S2, NULL};	/*Value = True*/
const Property P24138_1 = {S50, 0, &V24138_1_1, NULL};	/*Property = IsMobile*/
const Value V24138_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P24138_5 = {S52, 0, &V24138_5_1458, &P24138_1};	/*Property = ScreenPixelsHeight*/
const Value V24138_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P24138_4 = {S54, 0, &V24138_4_37617, &P24138_5};	/*Property = ScreenPixelsWidth*/
const Profile P24138 = {"24138", &P24138_4};	/*Profile ID - 24138*/
const Value V22748_1_1 = {S2, NULL};	/*Value = True*/
const Property P22748_1 = {S50, 0, &V22748_1_1, NULL};	/*Property = IsMobile*/
const Value V22748_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22748_5 = {S52, 0, &V22748_5_1222, &P22748_1};	/*Property = ScreenPixelsHeight*/
const Value V22748_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22748_4 = {S54, 0, &V22748_4_1224, &P22748_5};	/*Property = ScreenPixelsWidth*/
const Profile P22748 = {"22748", &P22748_4};	/*Profile ID - 22748*/
const Value V23200_1_1 = {S2, NULL};	/*Value = True*/
const Property P23200_1 = {S50, 0, &V23200_1_1, NULL};	/*Property = IsMobile*/
const Value V23200_5_37564 = {S149, NULL};	/*Value = 690*/
const Property P23200_5 = {S52, 0, &V23200_5_37564, &P23200_1};	/*Property = ScreenPixelsHeight*/
const Value V23200_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23200_4 = {S54, 0, &V23200_4_1404, &P23200_5};	/*Property = ScreenPixelsWidth*/
const Profile P23200 = {"23200", &P23200_4};	/*Profile ID - 23200*/
const Value V22867_1_1 = {S2, NULL};	/*Value = True*/
const Property P22867_1 = {S50, 0, &V22867_1_1, NULL};	/*Property = IsMobile*/
const Value V22867_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22867_5 = {S52, 0, &V22867_5_9981, &P22867_1};	/*Property = ScreenPixelsHeight*/
const Value V22867_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22867_4 = {S54, 0, &V22867_4_39098, &P22867_5};	/*Property = ScreenPixelsWidth*/
const Profile P22867 = {"22867", &P22867_4};	/*Profile ID - 22867*/
const Value V22599_1_1 = {S2, NULL};	/*Value = True*/
const Property P22599_1 = {S50, 0, &V22599_1_1, NULL};	/*Property = IsMobile*/
const Value V22599_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22599_5 = {S52, 0, &V22599_5_59138, &P22599_1};	/*Property = ScreenPixelsHeight*/
const Value V22599_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22599_4 = {S54, 0, &V22599_4_59970, &P22599_5};	/*Property = ScreenPixelsWidth*/
const Profile P22599 = {"22599", &P22599_4};	/*Profile ID - 22599*/
const Value V23513_1_1 = {S2, NULL};	/*Value = True*/
const Property P23513_1 = {S50, 0, &V23513_1_1, NULL};	/*Property = IsMobile*/
const Value V23513_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23513_5 = {S52, 0, &V23513_5_1227, &P23513_1};	/*Property = ScreenPixelsHeight*/
const Value V23513_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23513_4 = {S54, 0, &V23513_4_1229, &P23513_5};	/*Property = ScreenPixelsWidth*/
const Profile P23513 = {"23513", &P23513_4};	/*Profile ID - 23513*/
const Value V22964_1_1 = {S2, NULL};	/*Value = True*/
const Property P22964_1 = {S50, 0, &V22964_1_1, NULL};	/*Property = IsMobile*/
const Value V22964_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22964_5 = {S52, 0, &V22964_5_1227, &P22964_1};	/*Property = ScreenPixelsHeight*/
const Value V22964_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22964_4 = {S54, 0, &V22964_4_1229, &P22964_5};	/*Property = ScreenPixelsWidth*/
const Profile P22964 = {"22964", &P22964_4};	/*Profile ID - 22964*/
const Value V23181_1_1 = {S2, NULL};	/*Value = True*/
const Property P23181_1 = {S50, 0, &V23181_1_1, NULL};	/*Property = IsMobile*/
const Value V23181_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23181_5 = {S52, 0, &V23181_5_1227, &P23181_1};	/*Property = ScreenPixelsHeight*/
const Value V23181_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23181_4 = {S54, 0, &V23181_4_1229, &P23181_5};	/*Property = ScreenPixelsWidth*/
const Profile P23181 = {"23181", &P23181_4};	/*Profile ID - 23181*/
const Value V22685_1_1 = {S2, NULL};	/*Value = True*/
const Property P22685_1 = {S50, 0, &V22685_1_1, NULL};	/*Property = IsMobile*/
const Value V22685_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22685_5 = {S52, 0, &V22685_5_1227, &P22685_1};	/*Property = ScreenPixelsHeight*/
const Value V22685_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22685_4 = {S54, 0, &V22685_4_1229, &P22685_5};	/*Property = ScreenPixelsWidth*/
const Profile P22685 = {"22685", &P22685_4};	/*Profile ID - 22685*/
const Value V23236_1_1 = {S2, NULL};	/*Value = True*/
const Property P23236_1 = {S50, 0, &V23236_1_1, NULL};	/*Property = IsMobile*/
const Value V23236_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23236_5 = {S52, 0, &V23236_5_37569, &P23236_1};	/*Property = ScreenPixelsHeight*/
const Value V23236_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23236_4 = {S54, 0, &V23236_4_5645, &P23236_5};	/*Property = ScreenPixelsWidth*/
const Profile P23236 = {"23236", &P23236_4};	/*Profile ID - 23236*/
const Value V22961_1_1 = {S2, NULL};	/*Value = True*/
const Property P22961_1 = {S50, 0, &V22961_1_1, NULL};	/*Property = IsMobile*/
const Value V22961_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22961_5 = {S52, 0, &V22961_5_1227, &P22961_1};	/*Property = ScreenPixelsHeight*/
const Value V22961_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22961_4 = {S54, 0, &V22961_4_1229, &P22961_5};	/*Property = ScreenPixelsWidth*/
const Profile P22961 = {"22961", &P22961_4};	/*Profile ID - 22961*/
const Value V22934_1_1 = {S2, NULL};	/*Value = True*/
const Property P22934_1 = {S50, 0, &V22934_1_1, NULL};	/*Property = IsMobile*/
const Value V22934_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22934_5 = {S52, 0, &V22934_5_9981, &P22934_1};	/*Property = ScreenPixelsHeight*/
const Value V22934_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22934_4 = {S54, 0, &V22934_4_2336, &P22934_5};	/*Property = ScreenPixelsWidth*/
const Profile P22934 = {"22934", &P22934_4};	/*Profile ID - 22934*/
const Value V22627_1_1 = {S2, NULL};	/*Value = True*/
const Property P22627_1 = {S50, 0, &V22627_1_1, NULL};	/*Property = IsMobile*/
const Value V22627_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22627_5 = {S52, 0, &V22627_5_1403, &P22627_1};	/*Property = ScreenPixelsHeight*/
const Value V22627_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22627_4 = {S54, 0, &V22627_4_1404, &P22627_5};	/*Property = ScreenPixelsWidth*/
const Profile P22627 = {"22627", &P22627_4};	/*Profile ID - 22627*/
const Value V22753_1_1 = {S2, NULL};	/*Value = True*/
const Property P22753_1 = {S50, 0, &V22753_1_1, NULL};	/*Property = IsMobile*/
const Value V22753_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22753_5 = {S52, 0, &V22753_5_1277, &P22753_1};	/*Property = ScreenPixelsHeight*/
const Value V22753_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22753_4 = {S54, 0, &V22753_4_1279, &P22753_5};	/*Property = ScreenPixelsWidth*/
const Profile P22753 = {"22753", &P22753_4};	/*Profile ID - 22753*/
const Value V23838_1_1 = {S2, NULL};	/*Value = True*/
const Property P23838_1 = {S50, 0, &V23838_1_1, NULL};	/*Property = IsMobile*/
const Value V23838_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23838_5 = {S52, 0, &V23838_5_1277, &P23838_1};	/*Property = ScreenPixelsHeight*/
const Value V23838_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23838_4 = {S54, 0, &V23838_4_1279, &P23838_5};	/*Property = ScreenPixelsWidth*/
const Profile P23838 = {"23838", &P23838_4};	/*Profile ID - 23838*/
const Value V22622_1_1 = {S2, NULL};	/*Value = True*/
const Property P22622_1 = {S50, 0, &V22622_1_1, NULL};	/*Property = IsMobile*/
const Value V22622_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22622_5 = {S52, 0, &V22622_5_1227, &P22622_1};	/*Property = ScreenPixelsHeight*/
const Value V22622_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22622_4 = {S54, 0, &V22622_4_1229, &P22622_5};	/*Property = ScreenPixelsWidth*/
const Profile P22622 = {"22622", &P22622_4};	/*Profile ID - 22622*/
const Value V22863_1_1 = {S2, NULL};	/*Value = True*/
const Property P22863_1 = {S50, 0, &V22863_1_1, NULL};	/*Property = IsMobile*/
const Value V22863_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P22863_5 = {S52, 0, &V22863_5_37, &P22863_1};	/*Property = ScreenPixelsHeight*/
const Value V22863_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P22863_4 = {S54, 0, &V22863_4_36, &P22863_5};	/*Property = ScreenPixelsWidth*/
const Profile P22863 = {"22863", &P22863_4};	/*Profile ID - 22863*/
const Value V22595_1_1 = {S2, NULL};	/*Value = True*/
const Property P22595_1 = {S50, 0, &V22595_1_1, NULL};	/*Property = IsMobile*/
const Value V22595_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22595_5 = {S52, 0, &V22595_5_59138, &P22595_1};	/*Property = ScreenPixelsHeight*/
const Value V22595_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22595_4 = {S54, 0, &V22595_4_59970, &P22595_5};	/*Property = ScreenPixelsWidth*/
const Profile P22595 = {"22595", &P22595_4};	/*Profile ID - 22595*/
const Value V22919_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_279 = {S1, 0, &V22919_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22919_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_282 = {S3, 0, &V22919_282_57155, &P22919_279};	/*Property = BlobBuilder*/
const Value V22919_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_350 = {S4, 0, &V22919_350_57446, &P22919_282};	/*Property = Canvas*/
const Value V22919_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_283 = {S5, 0, &V22919_283_57161, &P22919_350};	/*Property = CssBackground*/
const Value V22919_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_284 = {S6, 0, &V22919_284_57164, &P22919_283};	/*Property = CssBorderImage*/
const Value V22919_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_285 = {S7, 0, &V22919_285_57167, &P22919_284};	/*Property = CssCanvas*/
const Value V22919_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_286 = {S8, 0, &V22919_286_57170, &P22919_285};	/*Property = CssColor*/
const Value V22919_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_237 = {S9, 0, &V22919_237_55786, &P22919_286};	/*Property = CssColumn*/
const Value V22919_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_287 = {S10, 0, &V22919_287_57173, &P22919_237};	/*Property = CssFlexbox*/
const Value V22919_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_288 = {S11, 0, &V22919_288_57176, &P22919_287};	/*Property = CssFont*/
const Value V22919_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_289 = {S12, 0, &V22919_289_86899, &P22919_288};	/*Property = CssImages*/
const Value V22919_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_291 = {S13, 0, &V22919_291_57182, &P22919_289};	/*Property = CssMediaQueries*/
const Value V22919_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_292 = {S14, 0, &V22919_292_57185, &P22919_291};	/*Property = CssMinMax*/
const Value V22919_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_293 = {S15, 0, &V22919_293_57188, &P22919_292};	/*Property = CssOverflow*/
const Value V22919_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_294 = {S16, 0, &V22919_294_57191, &P22919_293};	/*Property = CssPosition*/
const Value V22919_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_295 = {S17, 0, &V22919_295_57194, &P22919_294};	/*Property = CssText*/
const Value V22919_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_297 = {S18, 0, &V22919_297_57197, &P22919_295};	/*Property = CssTransforms*/
const Value V22919_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_241 = {S19, 0, &V22919_241_55801, &P22919_297};	/*Property = CssTransitions*/
const Value V22919_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_298 = {S20, 0, &V22919_298_57208, &P22919_241};	/*Property = CssUI*/
const Value V22919_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_300 = {S21, 0, &V22919_300_57217, &P22919_298};	/*Property = DataSet*/
const Value V22919_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_301 = {S22, 0, &V22919_301_57220, &P22919_300};	/*Property = DataUrl*/
const Value V22919_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_302 = {S23, 0, &V22919_302_57223, &P22919_301};	/*Property = DeviceOrientation*/
const Value V22919_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_304 = {S24, 0, &V22919_304_57229, &P22919_302};	/*Property = FileReader*/
const Value V22919_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_305 = {S25, 0, &V22919_305_57232, &P22919_304};	/*Property = FileSaver*/
const Value V22919_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_306 = {S26, 0, &V22919_306_57238, &P22919_305};	/*Property = FileWriter*/
const Value V22919_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_307 = {S27, 0, &V22919_307_57241, &P22919_306};	/*Property = FormData*/
const Value V22919_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_309 = {S28, 0, &V22919_309_57247, &P22919_307};	/*Property = Fullscreen*/
const Value V22919_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_310 = {S29, 0, &V22919_310_57250, &P22919_309};	/*Property = GeoLocation*/
const Value V22919_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_312 = {S30, 0, &V22919_312_57256, &P22919_310};	/*Property = History*/
const Value V22919_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_314 = {S31, 0, &V22919_314_57262, &P22919_312};	/*Property = Html5*/
const Value V22919_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_313 = {S32, 0, &V22919_313_57259, &P22919_314};	/*Property = Html-Media-Capture*/
const Value V22919_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_315 = {S33, 0, &V22919_315_57265, &P22919_313};	/*Property = Iframe*/
const Value V22919_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_316 = {S34, 0, &V22919_316_57268, &P22919_315};	/*Property = IndexedDB*/
const Value V22919_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_317 = {S35, 0, &V22919_317_57271, &P22919_316};	/*Property = Json*/
const Value V22919_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22919_15 = {S37, 0, &V22919_15_28, &P22919_317};	/*Property = LayoutEngine*/
const Value V22919_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_344 = {S38, 0, &V22919_344_57274, &P22919_15};	/*Property = Masking*/
const Value V22919_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_320 = {S39, 0, &V22919_320_57292, &P22919_344};	/*Property = PostMessage*/
const Value V22919_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_321 = {S40, 0, &V22919_321_57295, &P22919_320};	/*Property = Progress*/
const Value V22919_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_322 = {S41, 0, &V22919_322_57298, &P22919_321};	/*Property = Prompts*/
const Value V22919_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_323 = {S42, 0, &V22919_323_57301, &P22919_322};	/*Property = Selector*/
const Value V22919_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_324 = {S43, 0, &V22919_324_57307, &P22919_323};	/*Property = Svg*/
const Value V22919_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_327 = {S44, 0, &V22919_327_57316, &P22919_324};	/*Property = TouchEvents*/
const Value V22919_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_328 = {S45, 0, &V22919_328_57319, &P22919_327};	/*Property = Track*/
const Value V22919_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_329 = {S46, 0, &V22919_329_57325, &P22919_328};	/*Property = Video*/
const Value V22919_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_330 = {S47, 0, &V22919_330_57328, &P22919_329};	/*Property = Viewport*/
const Value V22919_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_332 = {S48, 0, &V22919_332_57340, &P22919_330};	/*Property = WebWorkers*/
const Value V22919_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22919_333 = {S49, 0, &V22919_333_57343, &P22919_332};	/*Property = Xhr2*/
const Profile P22919 = {"22919", &P22919_333};	/*Profile ID - 22919*/
const Value V22664_1_1 = {S2, NULL};	/*Value = True*/
const Property P22664_1 = {S50, 0, &V22664_1_1, NULL};	/*Property = IsMobile*/
const Value V22664_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22664_5 = {S52, 0, &V22664_5_1277, &P22664_1};	/*Property = ScreenPixelsHeight*/
const Value V22664_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22664_4 = {S54, 0, &V22664_4_1279, &P22664_5};	/*Property = ScreenPixelsWidth*/
const Profile P22664 = {"22664", &P22664_4};	/*Profile ID - 22664*/
const Profile P22583 = {"22583", NULL};	/*Profile ID - 22583*/
const Value V23319_1_1 = {S2, NULL};	/*Value = True*/
const Property P23319_1 = {S50, 0, &V23319_1_1, NULL};	/*Property = IsMobile*/
const Value V23319_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23319_5 = {S52, 0, &V23319_5_1227, &P23319_1};	/*Property = ScreenPixelsHeight*/
const Value V23319_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23319_4 = {S54, 0, &V23319_4_1229, &P23319_5};	/*Property = ScreenPixelsWidth*/
const Profile P23319 = {"23319", &P23319_4};	/*Profile ID - 23319*/
const Value V22604_1_1 = {S2, NULL};	/*Value = True*/
const Property P22604_1 = {S50, 0, &V22604_1_1, NULL};	/*Property = IsMobile*/
const Value V22604_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22604_5 = {S52, 0, &V22604_5_1227, &P22604_1};	/*Property = ScreenPixelsHeight*/
const Value V22604_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22604_4 = {S54, 0, &V22604_4_1229, &P22604_5};	/*Property = ScreenPixelsWidth*/
const Profile P22604 = {"22604", &P22604_4};	/*Profile ID - 22604*/
const Value V22676_1_1 = {S2, NULL};	/*Value = True*/
const Property P22676_1 = {S50, 0, &V22676_1_1, NULL};	/*Property = IsMobile*/
const Value V22676_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22676_5 = {S52, 0, &V22676_5_9981, &P22676_1};	/*Property = ScreenPixelsHeight*/
const Value V22676_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22676_4 = {S54, 0, &V22676_4_39098, &P22676_5};	/*Property = ScreenPixelsWidth*/
const Profile P22676 = {"22676", &P22676_4};	/*Profile ID - 22676*/
const Value V22966_1_1 = {S2, NULL};	/*Value = True*/
const Property P22966_1 = {S50, 0, &V22966_1_1, NULL};	/*Property = IsMobile*/
const Value V22966_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22966_5 = {S52, 0, &V22966_5_1227, &P22966_1};	/*Property = ScreenPixelsHeight*/
const Value V22966_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22966_4 = {S54, 0, &V22966_4_1229, &P22966_5};	/*Property = ScreenPixelsWidth*/
const Profile P22966 = {"22966", &P22966_4};	/*Profile ID - 22966*/
const Value V23015_1_1 = {S2, NULL};	/*Value = True*/
const Property P23015_1 = {S50, 0, &V23015_1_1, NULL};	/*Property = IsMobile*/
const Value V23015_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23015_5 = {S52, 0, &V23015_5_1403, &P23015_1};	/*Property = ScreenPixelsHeight*/
const Value V23015_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23015_4 = {S54, 0, &V23015_4_1404, &P23015_5};	/*Property = ScreenPixelsWidth*/
const Profile P23015 = {"23015", &P23015_4};	/*Profile ID - 23015*/
const Value V23135_1_1 = {S2, NULL};	/*Value = True*/
const Property P23135_1 = {S50, 0, &V23135_1_1, NULL};	/*Property = IsMobile*/
const Value V23135_5_37526 = {S122, NULL};	/*Value = 1200*/
const Property P23135_5 = {S52, 0, &V23135_5_37526, &P23135_1};	/*Property = ScreenPixelsHeight*/
const Value V23135_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23135_4 = {S54, 0, &V23135_4_57095, &P23135_5};	/*Property = ScreenPixelsWidth*/
const Profile P23135 = {"23135", &P23135_4};	/*Profile ID - 23135*/
const Value V22679_1_1 = {S2, NULL};	/*Value = True*/
const Property P22679_1 = {S50, 0, &V22679_1_1, NULL};	/*Property = IsMobile*/
const Value V22679_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22679_5 = {S52, 0, &V22679_5_1227, &P22679_1};	/*Property = ScreenPixelsHeight*/
const Value V22679_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22679_4 = {S54, 0, &V22679_4_1229, &P22679_5};	/*Property = ScreenPixelsWidth*/
const Profile P22679 = {"22679", &P22679_4};	/*Profile ID - 22679*/
const Value V23227_1_1 = {S2, NULL};	/*Value = True*/
const Property P23227_1 = {S50, 0, &V23227_1_1, NULL};	/*Property = IsMobile*/
const Value V23227_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23227_5 = {S52, 0, &V23227_5_9981, &P23227_1};	/*Property = ScreenPixelsHeight*/
const Value V23227_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23227_4 = {S54, 0, &V23227_4_2336, &P23227_5};	/*Property = ScreenPixelsWidth*/
const Profile P23227 = {"23227", &P23227_4};	/*Profile ID - 23227*/
const Value V18415_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_279 = {S1, 0, &V18415_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V18415_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_282 = {S3, 0, &V18415_282_57155, &P18415_279};	/*Property = BlobBuilder*/
const Value V18415_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_350 = {S4, 0, &V18415_350_57446, &P18415_282};	/*Property = Canvas*/
const Value V18415_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_283 = {S5, 0, &V18415_283_57161, &P18415_350};	/*Property = CssBackground*/
const Value V18415_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_284 = {S6, 0, &V18415_284_57164, &P18415_283};	/*Property = CssBorderImage*/
const Value V18415_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_285 = {S7, 0, &V18415_285_57167, &P18415_284};	/*Property = CssCanvas*/
const Value V18415_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_286 = {S8, 0, &V18415_286_57170, &P18415_285};	/*Property = CssColor*/
const Value V18415_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_237 = {S9, 0, &V18415_237_55786, &P18415_286};	/*Property = CssColumn*/
const Value V18415_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_287 = {S10, 0, &V18415_287_57173, &P18415_237};	/*Property = CssFlexbox*/
const Value V18415_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_288 = {S11, 0, &V18415_288_57176, &P18415_287};	/*Property = CssFont*/
const Value V18415_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_289 = {S12, 0, &V18415_289_86899, &P18415_288};	/*Property = CssImages*/
const Value V18415_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_291 = {S13, 0, &V18415_291_57182, &P18415_289};	/*Property = CssMediaQueries*/
const Value V18415_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_292 = {S14, 0, &V18415_292_57185, &P18415_291};	/*Property = CssMinMax*/
const Value V18415_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_293 = {S15, 0, &V18415_293_57188, &P18415_292};	/*Property = CssOverflow*/
const Value V18415_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_294 = {S16, 0, &V18415_294_57191, &P18415_293};	/*Property = CssPosition*/
const Value V18415_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_295 = {S17, 0, &V18415_295_57194, &P18415_294};	/*Property = CssText*/
const Value V18415_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_297 = {S18, 0, &V18415_297_57197, &P18415_295};	/*Property = CssTransforms*/
const Value V18415_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_241 = {S19, 0, &V18415_241_55801, &P18415_297};	/*Property = CssTransitions*/
const Value V18415_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_298 = {S20, 0, &V18415_298_57208, &P18415_241};	/*Property = CssUI*/
const Value V18415_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_300 = {S21, 0, &V18415_300_57217, &P18415_298};	/*Property = DataSet*/
const Value V18415_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_301 = {S22, 0, &V18415_301_57220, &P18415_300};	/*Property = DataUrl*/
const Value V18415_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_302 = {S23, 0, &V18415_302_57223, &P18415_301};	/*Property = DeviceOrientation*/
const Value V18415_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_304 = {S24, 0, &V18415_304_57229, &P18415_302};	/*Property = FileReader*/
const Value V18415_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_305 = {S25, 0, &V18415_305_57232, &P18415_304};	/*Property = FileSaver*/
const Value V18415_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_306 = {S26, 0, &V18415_306_57238, &P18415_305};	/*Property = FileWriter*/
const Value V18415_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_307 = {S27, 0, &V18415_307_57241, &P18415_306};	/*Property = FormData*/
const Value V18415_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_309 = {S28, 0, &V18415_309_57247, &P18415_307};	/*Property = Fullscreen*/
const Value V18415_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_310 = {S29, 0, &V18415_310_57250, &P18415_309};	/*Property = GeoLocation*/
const Value V18415_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_312 = {S30, 0, &V18415_312_57256, &P18415_310};	/*Property = History*/
const Value V18415_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_314 = {S31, 0, &V18415_314_57262, &P18415_312};	/*Property = Html5*/
const Value V18415_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_313 = {S32, 0, &V18415_313_57259, &P18415_314};	/*Property = Html-Media-Capture*/
const Value V18415_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_315 = {S33, 0, &V18415_315_57265, &P18415_313};	/*Property = Iframe*/
const Value V18415_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_316 = {S34, 0, &V18415_316_57268, &P18415_315};	/*Property = IndexedDB*/
const Value V18415_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_317 = {S35, 0, &V18415_317_57271, &P18415_316};	/*Property = Json*/
const Value V18415_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P18415_15 = {S37, 0, &V18415_15_31, &P18415_317};	/*Property = LayoutEngine*/
const Value V18415_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_344 = {S38, 0, &V18415_344_57274, &P18415_15};	/*Property = Masking*/
const Value V18415_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_320 = {S39, 0, &V18415_320_57292, &P18415_344};	/*Property = PostMessage*/
const Value V18415_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_321 = {S40, 0, &V18415_321_57295, &P18415_320};	/*Property = Progress*/
const Value V18415_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_322 = {S41, 0, &V18415_322_57298, &P18415_321};	/*Property = Prompts*/
const Value V18415_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_323 = {S42, 0, &V18415_323_57301, &P18415_322};	/*Property = Selector*/
const Value V18415_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_324 = {S43, 0, &V18415_324_57307, &P18415_323};	/*Property = Svg*/
const Value V18415_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_327 = {S44, 0, &V18415_327_57316, &P18415_324};	/*Property = TouchEvents*/
const Value V18415_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_328 = {S45, 0, &V18415_328_57319, &P18415_327};	/*Property = Track*/
const Value V18415_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_329 = {S46, 0, &V18415_329_57325, &P18415_328};	/*Property = Video*/
const Value V18415_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_330 = {S47, 0, &V18415_330_57328, &P18415_329};	/*Property = Viewport*/
const Value V18415_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_332 = {S48, 0, &V18415_332_57340, &P18415_330};	/*Property = WebWorkers*/
const Value V18415_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P18415_333 = {S49, 0, &V18415_333_57343, &P18415_332};	/*Property = Xhr2*/
const Profile P18415 = {"18415", &P18415_333};	/*Profile ID - 18415*/
const Value V23086_1_1 = {S2, NULL};	/*Value = True*/
const Property P23086_1 = {S50, 0, &V23086_1_1, NULL};	/*Property = IsMobile*/
const Value V23086_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23086_5 = {S52, 0, &V23086_5_9981, &P23086_1};	/*Property = ScreenPixelsHeight*/
const Value V23086_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23086_4 = {S54, 0, &V23086_4_2336, &P23086_5};	/*Property = ScreenPixelsWidth*/
const Profile P23086 = {"23086", &P23086_4};	/*Profile ID - 23086*/
const Value V24549_1_1 = {S2, NULL};	/*Value = True*/
const Property P24549_1 = {S50, 0, &V24549_1_1, NULL};	/*Property = IsMobile*/
const Value V24549_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24549_5 = {S52, 0, &V24549_5_9981, &P24549_1};	/*Property = ScreenPixelsHeight*/
const Value V24549_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24549_4 = {S54, 0, &V24549_4_39098, &P24549_5};	/*Property = ScreenPixelsWidth*/
const Profile P24549 = {"24549", &P24549_4};	/*Profile ID - 24549*/
const Value V22895_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_279 = {S1, 0, &V22895_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22895_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_282 = {S3, 0, &V22895_282_57155, &P22895_279};	/*Property = BlobBuilder*/
const Value V22895_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_350 = {S4, 0, &V22895_350_57446, &P22895_282};	/*Property = Canvas*/
const Value V22895_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_283 = {S5, 0, &V22895_283_57161, &P22895_350};	/*Property = CssBackground*/
const Value V22895_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_284 = {S6, 0, &V22895_284_57164, &P22895_283};	/*Property = CssBorderImage*/
const Value V22895_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_285 = {S7, 0, &V22895_285_57167, &P22895_284};	/*Property = CssCanvas*/
const Value V22895_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_286 = {S8, 0, &V22895_286_57170, &P22895_285};	/*Property = CssColor*/
const Value V22895_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_237 = {S9, 0, &V22895_237_55786, &P22895_286};	/*Property = CssColumn*/
const Value V22895_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_287 = {S10, 0, &V22895_287_57173, &P22895_237};	/*Property = CssFlexbox*/
const Value V22895_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_288 = {S11, 0, &V22895_288_57176, &P22895_287};	/*Property = CssFont*/
const Value V22895_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_289 = {S12, 0, &V22895_289_86899, &P22895_288};	/*Property = CssImages*/
const Value V22895_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_291 = {S13, 0, &V22895_291_57182, &P22895_289};	/*Property = CssMediaQueries*/
const Value V22895_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_292 = {S14, 0, &V22895_292_57185, &P22895_291};	/*Property = CssMinMax*/
const Value V22895_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_293 = {S15, 0, &V22895_293_57188, &P22895_292};	/*Property = CssOverflow*/
const Value V22895_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_294 = {S16, 0, &V22895_294_57191, &P22895_293};	/*Property = CssPosition*/
const Value V22895_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_295 = {S17, 0, &V22895_295_57194, &P22895_294};	/*Property = CssText*/
const Value V22895_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_297 = {S18, 0, &V22895_297_57197, &P22895_295};	/*Property = CssTransforms*/
const Value V22895_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_241 = {S19, 0, &V22895_241_55801, &P22895_297};	/*Property = CssTransitions*/
const Value V22895_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_298 = {S20, 0, &V22895_298_57208, &P22895_241};	/*Property = CssUI*/
const Value V22895_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_300 = {S21, 0, &V22895_300_57217, &P22895_298};	/*Property = DataSet*/
const Value V22895_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_301 = {S22, 0, &V22895_301_57220, &P22895_300};	/*Property = DataUrl*/
const Value V22895_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_302 = {S23, 0, &V22895_302_57223, &P22895_301};	/*Property = DeviceOrientation*/
const Value V22895_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_304 = {S24, 0, &V22895_304_57229, &P22895_302};	/*Property = FileReader*/
const Value V22895_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_305 = {S25, 0, &V22895_305_57232, &P22895_304};	/*Property = FileSaver*/
const Value V22895_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_306 = {S26, 0, &V22895_306_57238, &P22895_305};	/*Property = FileWriter*/
const Value V22895_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_307 = {S27, 0, &V22895_307_57241, &P22895_306};	/*Property = FormData*/
const Value V22895_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_309 = {S28, 0, &V22895_309_57247, &P22895_307};	/*Property = Fullscreen*/
const Value V22895_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_310 = {S29, 0, &V22895_310_57250, &P22895_309};	/*Property = GeoLocation*/
const Value V22895_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_312 = {S30, 0, &V22895_312_57256, &P22895_310};	/*Property = History*/
const Value V22895_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_314 = {S31, 0, &V22895_314_57262, &P22895_312};	/*Property = Html5*/
const Value V22895_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_313 = {S32, 0, &V22895_313_57259, &P22895_314};	/*Property = Html-Media-Capture*/
const Value V22895_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_315 = {S33, 0, &V22895_315_57265, &P22895_313};	/*Property = Iframe*/
const Value V22895_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_316 = {S34, 0, &V22895_316_57268, &P22895_315};	/*Property = IndexedDB*/
const Value V22895_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_317 = {S35, 0, &V22895_317_57271, &P22895_316};	/*Property = Json*/
const Value V22895_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_15 = {S37, 0, &V22895_15_72, &P22895_317};	/*Property = LayoutEngine*/
const Value V22895_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_344 = {S38, 0, &V22895_344_57274, &P22895_15};	/*Property = Masking*/
const Value V22895_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_320 = {S39, 0, &V22895_320_57292, &P22895_344};	/*Property = PostMessage*/
const Value V22895_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_321 = {S40, 0, &V22895_321_57295, &P22895_320};	/*Property = Progress*/
const Value V22895_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_322 = {S41, 0, &V22895_322_57298, &P22895_321};	/*Property = Prompts*/
const Value V22895_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_323 = {S42, 0, &V22895_323_57301, &P22895_322};	/*Property = Selector*/
const Value V22895_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_324 = {S43, 0, &V22895_324_57307, &P22895_323};	/*Property = Svg*/
const Value V22895_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_327 = {S44, 0, &V22895_327_57316, &P22895_324};	/*Property = TouchEvents*/
const Value V22895_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_328 = {S45, 0, &V22895_328_57319, &P22895_327};	/*Property = Track*/
const Value V22895_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_329 = {S46, 0, &V22895_329_57325, &P22895_328};	/*Property = Video*/
const Value V22895_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_330 = {S47, 0, &V22895_330_57328, &P22895_329};	/*Property = Viewport*/
const Value V22895_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_332 = {S48, 0, &V22895_332_57340, &P22895_330};	/*Property = WebWorkers*/
const Value V22895_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22895_333 = {S49, 0, &V22895_333_57343, &P22895_332};	/*Property = Xhr2*/
const Profile P22895 = {"22895", &P22895_333};	/*Profile ID - 22895*/
const Value V22822_1_1 = {S2, NULL};	/*Value = True*/
const Property P22822_1 = {S50, 0, &V22822_1_1, NULL};	/*Property = IsMobile*/
const Value V22822_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22822_5 = {S52, 0, &V22822_5_9981, &P22822_1};	/*Property = ScreenPixelsHeight*/
const Value V22822_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22822_4 = {S54, 0, &V22822_4_39098, &P22822_5};	/*Property = ScreenPixelsWidth*/
const Profile P22822 = {"22822", &P22822_4};	/*Profile ID - 22822*/
const Value V22935_1_1 = {S2, NULL};	/*Value = True*/
const Property P22935_1 = {S50, 0, &V22935_1_1, NULL};	/*Property = IsMobile*/
const Value V22935_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22935_5 = {S52, 0, &V22935_5_1227, &P22935_1};	/*Property = ScreenPixelsHeight*/
const Value V22935_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22935_4 = {S54, 0, &V22935_4_1229, &P22935_5};	/*Property = ScreenPixelsWidth*/
const Profile P22935 = {"22935", &P22935_4};	/*Profile ID - 22935*/
const Value V22586_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22586_279 = {S1, 0, &V22586_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22586_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22586_282 = {S3, 0, &V22586_282_57154, &P22586_279};	/*Property = BlobBuilder*/
const Value V22586_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22586_350 = {S4, 0, &V22586_350_57444, &P22586_282};	/*Property = Canvas*/
const Value V22586_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22586_283 = {S5, 0, &V22586_283_57159, &P22586_350};	/*Property = CssBackground*/
const Value V22586_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22586_284 = {S6, 0, &V22586_284_57163, &P22586_283};	/*Property = CssBorderImage*/
const Value V22586_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22586_285 = {S7, 0, &V22586_285_57166, &P22586_284};	/*Property = CssCanvas*/
const Value V22586_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22586_286 = {S8, 0, &V22586_286_57168, &P22586_285};	/*Property = CssColor*/
const Value V22586_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22586_237 = {S9, 0, &V22586_237_55786, &P22586_286};	/*Property = CssColumn*/
const Value V22586_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22586_287 = {S10, 0, &V22586_287_57172, &P22586_237};	/*Property = CssFlexbox*/
const Value V22586_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22586_288 = {S11, 0, &V22586_288_57175, &P22586_287};	/*Property = CssFont*/
const Value V22586_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22586_289 = {S12, 0, &V22586_289_57759, &P22586_288};	/*Property = CssImages*/
const Value V22586_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22586_291 = {S13, 0, &V22586_291_57180, &P22586_289};	/*Property = CssMediaQueries*/
const Value V22586_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22586_292 = {S14, 0, &V22586_292_57183, &P22586_291};	/*Property = CssMinMax*/
const Value V22586_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22586_293 = {S15, 0, &V22586_293_57187, &P22586_292};	/*Property = CssOverflow*/
const Value V22586_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22586_294 = {S16, 0, &V22586_294_57189, &P22586_293};	/*Property = CssPosition*/
const Value V22586_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22586_295 = {S17, 0, &V22586_295_57192, &P22586_294};	/*Property = CssText*/
const Value V22586_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22586_297 = {S18, 0, &V22586_297_57195, &P22586_295};	/*Property = CssTransforms*/
const Value V22586_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22586_241 = {S19, 0, &V22586_241_55799, &P22586_297};	/*Property = CssTransitions*/
const Value V22586_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22586_298 = {S20, 0, &V22586_298_57206, &P22586_241};	/*Property = CssUI*/
const Value V22586_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22586_300 = {S21, 0, &V22586_300_57215, &P22586_298};	/*Property = DataSet*/
const Value V22586_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22586_301 = {S22, 0, &V22586_301_57218, &P22586_300};	/*Property = DataUrl*/
const Value V22586_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22586_302 = {S23, 0, &V22586_302_57221, &P22586_301};	/*Property = DeviceOrientation*/
const Value V22586_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22586_304 = {S24, 0, &V22586_304_57228, &P22586_302};	/*Property = FileReader*/
const Value V22586_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22586_305 = {S25, 0, &V22586_305_57231, &P22586_304};	/*Property = FileSaver*/
const Value V22586_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22586_306 = {S26, 0, &V22586_306_57237, &P22586_305};	/*Property = FileWriter*/
const Value V22586_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22586_307 = {S27, 0, &V22586_307_57239, &P22586_306};	/*Property = FormData*/
const Value V22586_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22586_309 = {S28, 0, &V22586_309_57246, &P22586_307};	/*Property = Fullscreen*/
const Value V22586_310_57249 = {S0, NULL};	/*Value = False*/
const Property P22586_310 = {S29, 0, &V22586_310_57249, &P22586_309};	/*Property = GeoLocation*/
const Value V22586_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22586_312 = {S30, 0, &V22586_312_57255, &P22586_310};	/*Property = History*/
const Value V22586_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22586_314 = {S31, 0, &V22586_314_57260, &P22586_312};	/*Property = Html5*/
const Value V22586_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22586_313 = {S32, 0, &V22586_313_57258, &P22586_314};	/*Property = Html-Media-Capture*/
const Value V22586_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22586_315 = {S33, 0, &V22586_315_57264, &P22586_313};	/*Property = Iframe*/
const Value V22586_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22586_316 = {S34, 0, &V22586_316_57267, &P22586_315};	/*Property = IndexedDB*/
const Value V22586_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22586_317 = {S35, 0, &V22586_317_57269, &P22586_316};	/*Property = Json*/
const Value V22586_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22586_15 = {S37, 0, &V22586_15_28, &P22586_317};	/*Property = LayoutEngine*/
const Value V22586_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22586_344 = {S38, 0, &V22586_344_57272, &P22586_15};	/*Property = Masking*/
const Value V22586_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22586_320 = {S39, 0, &V22586_320_57290, &P22586_344};	/*Property = PostMessage*/
const Value V22586_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22586_321 = {S40, 0, &V22586_321_57293, &P22586_320};	/*Property = Progress*/
const Value V22586_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22586_322 = {S41, 0, &V22586_322_57296, &P22586_321};	/*Property = Prompts*/
const Value V22586_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22586_323 = {S42, 0, &V22586_323_57299, &P22586_322};	/*Property = Selector*/
const Value V22586_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22586_324 = {S43, 0, &V22586_324_57305, &P22586_323};	/*Property = Svg*/
const Value V22586_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22586_327 = {S44, 0, &V22586_327_57314, &P22586_324};	/*Property = TouchEvents*/
const Value V22586_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22586_328 = {S45, 0, &V22586_328_57318, &P22586_327};	/*Property = Track*/
const Value V22586_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22586_329 = {S46, 0, &V22586_329_57323, &P22586_328};	/*Property = Video*/
const Value V22586_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22586_330 = {S47, 0, &V22586_330_57326, &P22586_329};	/*Property = Viewport*/
const Value V22586_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22586_332 = {S48, 0, &V22586_332_57339, &P22586_330};	/*Property = WebWorkers*/
const Value V22586_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22586_333 = {S49, 0, &V22586_333_57342, &P22586_332};	/*Property = Xhr2*/
const Profile P22586 = {"22586", &P22586_333};	/*Profile ID - 22586*/
const Value V24442_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_279 = {S1, 0, &V24442_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24442_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_282 = {S3, 0, &V24442_282_57155, &P24442_279};	/*Property = BlobBuilder*/
const Value V24442_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_350 = {S4, 0, &V24442_350_57446, &P24442_282};	/*Property = Canvas*/
const Value V24442_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_283 = {S5, 0, &V24442_283_57161, &P24442_350};	/*Property = CssBackground*/
const Value V24442_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_284 = {S6, 0, &V24442_284_57164, &P24442_283};	/*Property = CssBorderImage*/
const Value V24442_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_285 = {S7, 0, &V24442_285_57167, &P24442_284};	/*Property = CssCanvas*/
const Value V24442_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_286 = {S8, 0, &V24442_286_57170, &P24442_285};	/*Property = CssColor*/
const Value V24442_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_237 = {S9, 0, &V24442_237_55786, &P24442_286};	/*Property = CssColumn*/
const Value V24442_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_287 = {S10, 0, &V24442_287_57173, &P24442_237};	/*Property = CssFlexbox*/
const Value V24442_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_288 = {S11, 0, &V24442_288_57176, &P24442_287};	/*Property = CssFont*/
const Value V24442_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_289 = {S12, 0, &V24442_289_86899, &P24442_288};	/*Property = CssImages*/
const Value V24442_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_291 = {S13, 0, &V24442_291_57182, &P24442_289};	/*Property = CssMediaQueries*/
const Value V24442_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_292 = {S14, 0, &V24442_292_57185, &P24442_291};	/*Property = CssMinMax*/
const Value V24442_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_293 = {S15, 0, &V24442_293_57188, &P24442_292};	/*Property = CssOverflow*/
const Value V24442_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_294 = {S16, 0, &V24442_294_57191, &P24442_293};	/*Property = CssPosition*/
const Value V24442_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_295 = {S17, 0, &V24442_295_57194, &P24442_294};	/*Property = CssText*/
const Value V24442_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_297 = {S18, 0, &V24442_297_57197, &P24442_295};	/*Property = CssTransforms*/
const Value V24442_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_241 = {S19, 0, &V24442_241_55801, &P24442_297};	/*Property = CssTransitions*/
const Value V24442_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_298 = {S20, 0, &V24442_298_57208, &P24442_241};	/*Property = CssUI*/
const Value V24442_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_300 = {S21, 0, &V24442_300_57217, &P24442_298};	/*Property = DataSet*/
const Value V24442_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_301 = {S22, 0, &V24442_301_57220, &P24442_300};	/*Property = DataUrl*/
const Value V24442_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_302 = {S23, 0, &V24442_302_57223, &P24442_301};	/*Property = DeviceOrientation*/
const Value V24442_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_304 = {S24, 0, &V24442_304_57229, &P24442_302};	/*Property = FileReader*/
const Value V24442_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_305 = {S25, 0, &V24442_305_57232, &P24442_304};	/*Property = FileSaver*/
const Value V24442_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_306 = {S26, 0, &V24442_306_57238, &P24442_305};	/*Property = FileWriter*/
const Value V24442_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_307 = {S27, 0, &V24442_307_57241, &P24442_306};	/*Property = FormData*/
const Value V24442_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_309 = {S28, 0, &V24442_309_57247, &P24442_307};	/*Property = Fullscreen*/
const Value V24442_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_310 = {S29, 0, &V24442_310_57250, &P24442_309};	/*Property = GeoLocation*/
const Value V24442_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_312 = {S30, 0, &V24442_312_57256, &P24442_310};	/*Property = History*/
const Value V24442_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_314 = {S31, 0, &V24442_314_57262, &P24442_312};	/*Property = Html5*/
const Value V24442_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_313 = {S32, 0, &V24442_313_57259, &P24442_314};	/*Property = Html-Media-Capture*/
const Value V24442_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_315 = {S33, 0, &V24442_315_57265, &P24442_313};	/*Property = Iframe*/
const Value V24442_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_316 = {S34, 0, &V24442_316_57268, &P24442_315};	/*Property = IndexedDB*/
const Value V24442_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_317 = {S35, 0, &V24442_317_57271, &P24442_316};	/*Property = Json*/
const Value V24442_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24442_15 = {S37, 0, &V24442_15_28, &P24442_317};	/*Property = LayoutEngine*/
const Value V24442_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_344 = {S38, 0, &V24442_344_57274, &P24442_15};	/*Property = Masking*/
const Value V24442_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_320 = {S39, 0, &V24442_320_57292, &P24442_344};	/*Property = PostMessage*/
const Value V24442_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_321 = {S40, 0, &V24442_321_57295, &P24442_320};	/*Property = Progress*/
const Value V24442_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_322 = {S41, 0, &V24442_322_57298, &P24442_321};	/*Property = Prompts*/
const Value V24442_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_323 = {S42, 0, &V24442_323_57301, &P24442_322};	/*Property = Selector*/
const Value V24442_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_324 = {S43, 0, &V24442_324_57307, &P24442_323};	/*Property = Svg*/
const Value V24442_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_327 = {S44, 0, &V24442_327_57316, &P24442_324};	/*Property = TouchEvents*/
const Value V24442_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_328 = {S45, 0, &V24442_328_57319, &P24442_327};	/*Property = Track*/
const Value V24442_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_329 = {S46, 0, &V24442_329_57325, &P24442_328};	/*Property = Video*/
const Value V24442_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_330 = {S47, 0, &V24442_330_57328, &P24442_329};	/*Property = Viewport*/
const Value V24442_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_332 = {S48, 0, &V24442_332_57340, &P24442_330};	/*Property = WebWorkers*/
const Value V24442_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24442_333 = {S49, 0, &V24442_333_57343, &P24442_332};	/*Property = Xhr2*/
const Profile P24442 = {"24442", &P24442_333};	/*Profile ID - 24442*/
const Profile P21637 = {"21637", NULL};	/*Profile ID - 21637*/
const Value V23146_1_1 = {S2, NULL};	/*Value = True*/
const Property P23146_1 = {S50, 0, &V23146_1_1, NULL};	/*Property = IsMobile*/
const Value V23146_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23146_5 = {S52, 0, &V23146_5_1227, &P23146_1};	/*Property = ScreenPixelsHeight*/
const Value V23146_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P23146_4 = {S54, 0, &V23146_4_1477, &P23146_5};	/*Property = ScreenPixelsWidth*/
const Profile P23146 = {"23146", &P23146_4};	/*Profile ID - 23146*/
const Value V22866_1_1 = {S2, NULL};	/*Value = True*/
const Property P22866_1 = {S50, 0, &V22866_1_1, NULL};	/*Property = IsMobile*/
const Value V22866_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22866_5 = {S52, 0, &V22866_5_1227, &P22866_1};	/*Property = ScreenPixelsHeight*/
const Value V22866_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22866_4 = {S54, 0, &V22866_4_1229, &P22866_5};	/*Property = ScreenPixelsWidth*/
const Profile P22866 = {"22866", &P22866_4};	/*Profile ID - 22866*/
const Value V23265_1_1 = {S2, NULL};	/*Value = True*/
const Property P23265_1 = {S50, 0, &V23265_1_1, NULL};	/*Property = IsMobile*/
const Value V23265_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23265_5 = {S52, 0, &V23265_5_1227, &P23265_1};	/*Property = ScreenPixelsHeight*/
const Value V23265_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23265_4 = {S54, 0, &V23265_4_1460, &P23265_5};	/*Property = ScreenPixelsWidth*/
const Profile P23265 = {"23265", &P23265_4};	/*Profile ID - 23265*/
const Value V22705_1_1 = {S2, NULL};	/*Value = True*/
const Property P22705_1 = {S50, 0, &V22705_1_1, NULL};	/*Property = IsMobile*/
const Value V22705_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22705_5 = {S52, 0, &V22705_5_9981, &P22705_1};	/*Property = ScreenPixelsHeight*/
const Value V22705_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22705_4 = {S54, 0, &V22705_4_39098, &P22705_5};	/*Property = ScreenPixelsWidth*/
const Profile P22705 = {"22705", &P22705_4};	/*Profile ID - 22705*/
const Value V23107_1_1 = {S2, NULL};	/*Value = True*/
const Property P23107_1 = {S50, 0, &V23107_1_1, NULL};	/*Property = IsMobile*/
const Value V23107_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23107_5 = {S52, 0, &V23107_5_1227, &P23107_1};	/*Property = ScreenPixelsHeight*/
const Value V23107_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23107_4 = {S54, 0, &V23107_4_1229, &P23107_5};	/*Property = ScreenPixelsWidth*/
const Profile P23107 = {"23107", &P23107_4};	/*Profile ID - 23107*/
const Value V22750_1_1 = {S2, NULL};	/*Value = True*/
const Property P22750_1 = {S50, 0, &V22750_1_1, NULL};	/*Property = IsMobile*/
const Value V22750_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22750_5 = {S52, 0, &V22750_5_1403, &P22750_1};	/*Property = ScreenPixelsHeight*/
const Value V22750_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22750_4 = {S54, 0, &V22750_4_1404, &P22750_5};	/*Property = ScreenPixelsWidth*/
const Profile P22750 = {"22750", &P22750_4};	/*Profile ID - 22750*/
const Value V23728_1_1 = {S2, NULL};	/*Value = True*/
const Property P23728_1 = {S50, 0, &V23728_1_1, NULL};	/*Property = IsMobile*/
const Value V23728_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23728_5 = {S52, 0, &V23728_5_1227, &P23728_1};	/*Property = ScreenPixelsHeight*/
const Value V23728_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23728_4 = {S54, 0, &V23728_4_1229, &P23728_5};	/*Property = ScreenPixelsWidth*/
const Profile P23728 = {"23728", &P23728_4};	/*Profile ID - 23728*/
const Value V22782_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22782_279 = {S1, 0, &V22782_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22782_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22782_282 = {S3, 0, &V22782_282_57154, &P22782_279};	/*Property = BlobBuilder*/
const Value V22782_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22782_350 = {S4, 0, &V22782_350_57444, &P22782_282};	/*Property = Canvas*/
const Value V22782_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22782_283 = {S5, 0, &V22782_283_57159, &P22782_350};	/*Property = CssBackground*/
const Value V22782_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22782_284 = {S6, 0, &V22782_284_57162, &P22782_283};	/*Property = CssBorderImage*/
const Value V22782_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22782_285 = {S7, 0, &V22782_285_57166, &P22782_284};	/*Property = CssCanvas*/
const Value V22782_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22782_286 = {S8, 0, &V22782_286_57168, &P22782_285};	/*Property = CssColor*/
const Value V22782_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22782_237 = {S9, 0, &V22782_237_55786, &P22782_286};	/*Property = CssColumn*/
const Value V22782_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22782_287 = {S10, 0, &V22782_287_57172, &P22782_237};	/*Property = CssFlexbox*/
const Value V22782_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22782_288 = {S11, 0, &V22782_288_57174, &P22782_287};	/*Property = CssFont*/
const Value V22782_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22782_289 = {S12, 0, &V22782_289_57759, &P22782_288};	/*Property = CssImages*/
const Value V22782_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22782_291 = {S13, 0, &V22782_291_57181, &P22782_289};	/*Property = CssMediaQueries*/
const Value V22782_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22782_292 = {S14, 0, &V22782_292_57183, &P22782_291};	/*Property = CssMinMax*/
const Value V22782_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22782_293 = {S15, 0, &V22782_293_57187, &P22782_292};	/*Property = CssOverflow*/
const Value V22782_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22782_294 = {S16, 0, &V22782_294_57189, &P22782_293};	/*Property = CssPosition*/
const Value V22782_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22782_295 = {S17, 0, &V22782_295_57192, &P22782_294};	/*Property = CssText*/
const Value V22782_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22782_297 = {S18, 0, &V22782_297_57195, &P22782_295};	/*Property = CssTransforms*/
const Value V22782_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22782_241 = {S19, 0, &V22782_241_55799, &P22782_297};	/*Property = CssTransitions*/
const Value V22782_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22782_298 = {S20, 0, &V22782_298_57206, &P22782_241};	/*Property = CssUI*/
const Value V22782_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22782_300 = {S21, 0, &V22782_300_57216, &P22782_298};	/*Property = DataSet*/
const Value V22782_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22782_301 = {S22, 0, &V22782_301_57218, &P22782_300};	/*Property = DataUrl*/
const Value V22782_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22782_302 = {S23, 0, &V22782_302_57221, &P22782_301};	/*Property = DeviceOrientation*/
const Value V22782_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22782_304 = {S24, 0, &V22782_304_57227, &P22782_302};	/*Property = FileReader*/
const Value V22782_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22782_305 = {S25, 0, &V22782_305_57231, &P22782_304};	/*Property = FileSaver*/
const Value V22782_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22782_306 = {S26, 0, &V22782_306_57237, &P22782_305};	/*Property = FileWriter*/
const Value V22782_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22782_307 = {S27, 0, &V22782_307_57239, &P22782_306};	/*Property = FormData*/
const Value V22782_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22782_309 = {S28, 0, &V22782_309_57246, &P22782_307};	/*Property = Fullscreen*/
const Value V22782_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22782_310 = {S29, 0, &V22782_310_57248, &P22782_309};	/*Property = GeoLocation*/
const Value V22782_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22782_312 = {S30, 0, &V22782_312_57254, &P22782_310};	/*Property = History*/
const Value V22782_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22782_314 = {S31, 0, &V22782_314_57261, &P22782_312};	/*Property = Html5*/
const Value V22782_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22782_313 = {S32, 0, &V22782_313_57258, &P22782_314};	/*Property = Html-Media-Capture*/
const Value V22782_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22782_315 = {S33, 0, &V22782_315_57264, &P22782_313};	/*Property = Iframe*/
const Value V22782_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22782_316 = {S34, 0, &V22782_316_57266, &P22782_315};	/*Property = IndexedDB*/
const Value V22782_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22782_317 = {S35, 0, &V22782_317_57269, &P22782_316};	/*Property = Json*/
const Value V22782_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P22782_15 = {S37, 0, &V22782_15_30, &P22782_317};	/*Property = LayoutEngine*/
const Value V22782_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22782_344 = {S38, 0, &V22782_344_57272, &P22782_15};	/*Property = Masking*/
const Value V22782_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22782_320 = {S39, 0, &V22782_320_57290, &P22782_344};	/*Property = PostMessage*/
const Value V22782_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22782_321 = {S40, 0, &V22782_321_57293, &P22782_320};	/*Property = Progress*/
const Value V22782_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22782_322 = {S41, 0, &V22782_322_57296, &P22782_321};	/*Property = Prompts*/
const Value V22782_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22782_323 = {S42, 0, &V22782_323_57299, &P22782_322};	/*Property = Selector*/
const Value V22782_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22782_324 = {S43, 0, &V22782_324_57305, &P22782_323};	/*Property = Svg*/
const Value V22782_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22782_327 = {S44, 0, &V22782_327_57315, &P22782_324};	/*Property = TouchEvents*/
const Value V22782_328_57317 = {S2, NULL};	/*Value = True*/
const Property P22782_328 = {S45, 0, &V22782_328_57317, &P22782_327};	/*Property = Track*/
const Value V22782_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22782_329 = {S46, 0, &V22782_329_57323, &P22782_328};	/*Property = Video*/
const Value V22782_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22782_330 = {S47, 0, &V22782_330_57326, &P22782_329};	/*Property = Viewport*/
const Value V22782_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22782_332 = {S48, 0, &V22782_332_57339, &P22782_330};	/*Property = WebWorkers*/
const Value V22782_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22782_333 = {S49, 0, &V22782_333_57342, &P22782_332};	/*Property = Xhr2*/
const Profile P22782 = {"22782", &P22782_333};	/*Profile ID - 22782*/
const Value V22739_1_1 = {S2, NULL};	/*Value = True*/
const Property P22739_1 = {S50, 0, &V22739_1_1, NULL};	/*Property = IsMobile*/
const Value V22739_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22739_5 = {S52, 0, &V22739_5_1227, &P22739_1};	/*Property = ScreenPixelsHeight*/
const Value V22739_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22739_4 = {S54, 0, &V22739_4_1229, &P22739_5};	/*Property = ScreenPixelsWidth*/
const Profile P22739 = {"22739", &P22739_4};	/*Profile ID - 22739*/
const Value V22909_1_1 = {S2, NULL};	/*Value = True*/
const Property P22909_1 = {S50, 0, &V22909_1_1, NULL};	/*Property = IsMobile*/
const Value V22909_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22909_5 = {S52, 0, &V22909_5_1403, &P22909_1};	/*Property = ScreenPixelsHeight*/
const Value V22909_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22909_4 = {S54, 0, &V22909_4_1404, &P22909_5};	/*Property = ScreenPixelsWidth*/
const Profile P22909 = {"22909", &P22909_4};	/*Profile ID - 22909*/
const Value V22708_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22708_279 = {S1, 0, &V22708_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22708_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22708_282 = {S3, 0, &V22708_282_57154, &P22708_279};	/*Property = BlobBuilder*/
const Value V22708_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22708_350 = {S4, 0, &V22708_350_57444, &P22708_282};	/*Property = Canvas*/
const Value V22708_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22708_283 = {S5, 0, &V22708_283_57159, &P22708_350};	/*Property = CssBackground*/
const Value V22708_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22708_284 = {S6, 0, &V22708_284_57162, &P22708_283};	/*Property = CssBorderImage*/
const Value V22708_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22708_285 = {S7, 0, &V22708_285_57166, &P22708_284};	/*Property = CssCanvas*/
const Value V22708_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22708_286 = {S8, 0, &V22708_286_57168, &P22708_285};	/*Property = CssColor*/
const Value V22708_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22708_237 = {S9, 0, &V22708_237_55786, &P22708_286};	/*Property = CssColumn*/
const Value V22708_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22708_287 = {S10, 0, &V22708_287_57172, &P22708_237};	/*Property = CssFlexbox*/
const Value V22708_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22708_288 = {S11, 0, &V22708_288_57174, &P22708_287};	/*Property = CssFont*/
const Value V22708_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22708_289 = {S12, 0, &V22708_289_57759, &P22708_288};	/*Property = CssImages*/
const Value V22708_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22708_291 = {S13, 0, &V22708_291_57180, &P22708_289};	/*Property = CssMediaQueries*/
const Value V22708_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22708_292 = {S14, 0, &V22708_292_57183, &P22708_291};	/*Property = CssMinMax*/
const Value V22708_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22708_293 = {S15, 0, &V22708_293_57187, &P22708_292};	/*Property = CssOverflow*/
const Value V22708_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22708_294 = {S16, 0, &V22708_294_57189, &P22708_293};	/*Property = CssPosition*/
const Value V22708_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22708_295 = {S17, 0, &V22708_295_57192, &P22708_294};	/*Property = CssText*/
const Value V22708_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22708_297 = {S18, 0, &V22708_297_57195, &P22708_295};	/*Property = CssTransforms*/
const Value V22708_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22708_241 = {S19, 0, &V22708_241_55799, &P22708_297};	/*Property = CssTransitions*/
const Value V22708_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22708_298 = {S20, 0, &V22708_298_57206, &P22708_241};	/*Property = CssUI*/
const Value V22708_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22708_300 = {S21, 0, &V22708_300_57215, &P22708_298};	/*Property = DataSet*/
const Value V22708_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22708_301 = {S22, 0, &V22708_301_57218, &P22708_300};	/*Property = DataUrl*/
const Value V22708_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22708_302 = {S23, 0, &V22708_302_57222, &P22708_301};	/*Property = DeviceOrientation*/
const Value V22708_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22708_304 = {S24, 0, &V22708_304_57227, &P22708_302};	/*Property = FileReader*/
const Value V22708_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22708_305 = {S25, 0, &V22708_305_57231, &P22708_304};	/*Property = FileSaver*/
const Value V22708_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22708_306 = {S26, 0, &V22708_306_57237, &P22708_305};	/*Property = FileWriter*/
const Value V22708_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22708_307 = {S27, 0, &V22708_307_57239, &P22708_306};	/*Property = FormData*/
const Value V22708_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22708_309 = {S28, 0, &V22708_309_57246, &P22708_307};	/*Property = Fullscreen*/
const Value V22708_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22708_310 = {S29, 0, &V22708_310_57248, &P22708_309};	/*Property = GeoLocation*/
const Value V22708_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22708_312 = {S30, 0, &V22708_312_57254, &P22708_310};	/*Property = History*/
const Value V22708_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22708_314 = {S31, 0, &V22708_314_57260, &P22708_312};	/*Property = Html5*/
const Value V22708_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22708_313 = {S32, 0, &V22708_313_57258, &P22708_314};	/*Property = Html-Media-Capture*/
const Value V22708_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22708_315 = {S33, 0, &V22708_315_57264, &P22708_313};	/*Property = Iframe*/
const Value V22708_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22708_316 = {S34, 0, &V22708_316_57266, &P22708_315};	/*Property = IndexedDB*/
const Value V22708_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22708_317 = {S35, 0, &V22708_317_57269, &P22708_316};	/*Property = Json*/
const Value V22708_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22708_15 = {S37, 0, &V22708_15_28, &P22708_317};	/*Property = LayoutEngine*/
const Value V22708_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22708_344 = {S38, 0, &V22708_344_57272, &P22708_15};	/*Property = Masking*/
const Value V22708_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22708_320 = {S39, 0, &V22708_320_57290, &P22708_344};	/*Property = PostMessage*/
const Value V22708_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22708_321 = {S40, 0, &V22708_321_57293, &P22708_320};	/*Property = Progress*/
const Value V22708_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22708_322 = {S41, 0, &V22708_322_57296, &P22708_321};	/*Property = Prompts*/
const Value V22708_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22708_323 = {S42, 0, &V22708_323_57299, &P22708_322};	/*Property = Selector*/
const Value V22708_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22708_324 = {S43, 0, &V22708_324_57305, &P22708_323};	/*Property = Svg*/
const Value V22708_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22708_327 = {S44, 0, &V22708_327_57315, &P22708_324};	/*Property = TouchEvents*/
const Value V22708_328_57317 = {S2, NULL};	/*Value = True*/
const Property P22708_328 = {S45, 0, &V22708_328_57317, &P22708_327};	/*Property = Track*/
const Value V22708_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22708_329 = {S46, 0, &V22708_329_57323, &P22708_328};	/*Property = Video*/
const Value V22708_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22708_330 = {S47, 0, &V22708_330_57326, &P22708_329};	/*Property = Viewport*/
const Value V22708_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22708_332 = {S48, 0, &V22708_332_57338, &P22708_330};	/*Property = WebWorkers*/
const Value V22708_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22708_333 = {S49, 0, &V22708_333_57341, &P22708_332};	/*Property = Xhr2*/
const Profile P22708 = {"22708", &P22708_333};	/*Profile ID - 22708*/
const Value V23106_1_1 = {S2, NULL};	/*Value = True*/
const Property P23106_1 = {S50, 0, &V23106_1_1, NULL};	/*Property = IsMobile*/
const Value V23106_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23106_5 = {S52, 0, &V23106_5_1458, &P23106_1};	/*Property = ScreenPixelsHeight*/
const Value V23106_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23106_4 = {S54, 0, &V23106_4_1460, &P23106_5};	/*Property = ScreenPixelsWidth*/
const Profile P23106 = {"23106", &P23106_4};	/*Profile ID - 23106*/
const Value V22623_1_1 = {S2, NULL};	/*Value = True*/
const Property P22623_1 = {S50, 0, &V22623_1_1, NULL};	/*Property = IsMobile*/
const Value V22623_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22623_5 = {S52, 0, &V22623_5_1277, &P22623_1};	/*Property = ScreenPixelsHeight*/
const Value V22623_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22623_4 = {S54, 0, &V22623_4_1279, &P22623_5};	/*Property = ScreenPixelsWidth*/
const Profile P22623 = {"22623", &P22623_4};	/*Profile ID - 22623*/
const Value V22927_1_1 = {S2, NULL};	/*Value = True*/
const Property P22927_1 = {S50, 0, &V22927_1_1, NULL};	/*Property = IsMobile*/
const Value V22927_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22927_5 = {S52, 0, &V22927_5_1832, &P22927_1};	/*Property = ScreenPixelsHeight*/
const Value V22927_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22927_4 = {S54, 0, &V22927_4_5645, &P22927_5};	/*Property = ScreenPixelsWidth*/
const Profile P22927 = {"22927", &P22927_4};	/*Profile ID - 22927*/
const Value V22779_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_279 = {S1, 0, &V22779_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22779_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_282 = {S3, 0, &V22779_282_57155, &P22779_279};	/*Property = BlobBuilder*/
const Value V22779_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_350 = {S4, 0, &V22779_350_57446, &P22779_282};	/*Property = Canvas*/
const Value V22779_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_283 = {S5, 0, &V22779_283_57161, &P22779_350};	/*Property = CssBackground*/
const Value V22779_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_284 = {S6, 0, &V22779_284_57164, &P22779_283};	/*Property = CssBorderImage*/
const Value V22779_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_285 = {S7, 0, &V22779_285_57167, &P22779_284};	/*Property = CssCanvas*/
const Value V22779_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_286 = {S8, 0, &V22779_286_57170, &P22779_285};	/*Property = CssColor*/
const Value V22779_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_237 = {S9, 0, &V22779_237_55786, &P22779_286};	/*Property = CssColumn*/
const Value V22779_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_287 = {S10, 0, &V22779_287_57173, &P22779_237};	/*Property = CssFlexbox*/
const Value V22779_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_288 = {S11, 0, &V22779_288_57176, &P22779_287};	/*Property = CssFont*/
const Value V22779_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_289 = {S12, 0, &V22779_289_86899, &P22779_288};	/*Property = CssImages*/
const Value V22779_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_291 = {S13, 0, &V22779_291_57182, &P22779_289};	/*Property = CssMediaQueries*/
const Value V22779_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_292 = {S14, 0, &V22779_292_57185, &P22779_291};	/*Property = CssMinMax*/
const Value V22779_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_293 = {S15, 0, &V22779_293_57188, &P22779_292};	/*Property = CssOverflow*/
const Value V22779_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_294 = {S16, 0, &V22779_294_57191, &P22779_293};	/*Property = CssPosition*/
const Value V22779_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_295 = {S17, 0, &V22779_295_57194, &P22779_294};	/*Property = CssText*/
const Value V22779_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_297 = {S18, 0, &V22779_297_57197, &P22779_295};	/*Property = CssTransforms*/
const Value V22779_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_241 = {S19, 0, &V22779_241_55801, &P22779_297};	/*Property = CssTransitions*/
const Value V22779_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_298 = {S20, 0, &V22779_298_57208, &P22779_241};	/*Property = CssUI*/
const Value V22779_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_300 = {S21, 0, &V22779_300_57217, &P22779_298};	/*Property = DataSet*/
const Value V22779_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_301 = {S22, 0, &V22779_301_57220, &P22779_300};	/*Property = DataUrl*/
const Value V22779_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_302 = {S23, 0, &V22779_302_57223, &P22779_301};	/*Property = DeviceOrientation*/
const Value V22779_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_304 = {S24, 0, &V22779_304_57229, &P22779_302};	/*Property = FileReader*/
const Value V22779_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_305 = {S25, 0, &V22779_305_57232, &P22779_304};	/*Property = FileSaver*/
const Value V22779_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_306 = {S26, 0, &V22779_306_57238, &P22779_305};	/*Property = FileWriter*/
const Value V22779_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_307 = {S27, 0, &V22779_307_57241, &P22779_306};	/*Property = FormData*/
const Value V22779_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_309 = {S28, 0, &V22779_309_57247, &P22779_307};	/*Property = Fullscreen*/
const Value V22779_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_310 = {S29, 0, &V22779_310_57250, &P22779_309};	/*Property = GeoLocation*/
const Value V22779_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_312 = {S30, 0, &V22779_312_57256, &P22779_310};	/*Property = History*/
const Value V22779_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_314 = {S31, 0, &V22779_314_57262, &P22779_312};	/*Property = Html5*/
const Value V22779_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_313 = {S32, 0, &V22779_313_57259, &P22779_314};	/*Property = Html-Media-Capture*/
const Value V22779_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_315 = {S33, 0, &V22779_315_57265, &P22779_313};	/*Property = Iframe*/
const Value V22779_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_316 = {S34, 0, &V22779_316_57268, &P22779_315};	/*Property = IndexedDB*/
const Value V22779_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_317 = {S35, 0, &V22779_317_57271, &P22779_316};	/*Property = Json*/
const Value V22779_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22779_15 = {S37, 0, &V22779_15_29, &P22779_317};	/*Property = LayoutEngine*/
const Value V22779_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_344 = {S38, 0, &V22779_344_57274, &P22779_15};	/*Property = Masking*/
const Value V22779_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_320 = {S39, 0, &V22779_320_57292, &P22779_344};	/*Property = PostMessage*/
const Value V22779_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_321 = {S40, 0, &V22779_321_57295, &P22779_320};	/*Property = Progress*/
const Value V22779_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_322 = {S41, 0, &V22779_322_57298, &P22779_321};	/*Property = Prompts*/
const Value V22779_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_323 = {S42, 0, &V22779_323_57301, &P22779_322};	/*Property = Selector*/
const Value V22779_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_324 = {S43, 0, &V22779_324_57307, &P22779_323};	/*Property = Svg*/
const Value V22779_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_327 = {S44, 0, &V22779_327_57316, &P22779_324};	/*Property = TouchEvents*/
const Value V22779_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_328 = {S45, 0, &V22779_328_57319, &P22779_327};	/*Property = Track*/
const Value V22779_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_329 = {S46, 0, &V22779_329_57325, &P22779_328};	/*Property = Video*/
const Value V22779_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_330 = {S47, 0, &V22779_330_57328, &P22779_329};	/*Property = Viewport*/
const Value V22779_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_332 = {S48, 0, &V22779_332_57340, &P22779_330};	/*Property = WebWorkers*/
const Value V22779_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22779_333 = {S49, 0, &V22779_333_57343, &P22779_332};	/*Property = Xhr2*/
const Profile P22779 = {"22779", &P22779_333};	/*Profile ID - 22779*/
const Value V23123_1_1 = {S2, NULL};	/*Value = True*/
const Property P23123_1 = {S50, 0, &V23123_1_1, NULL};	/*Property = IsMobile*/
const Value V23123_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23123_5 = {S52, 0, &V23123_5_1222, &P23123_1};	/*Property = ScreenPixelsHeight*/
const Value V23123_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23123_4 = {S54, 0, &V23123_4_1224, &P23123_5};	/*Property = ScreenPixelsWidth*/
const Profile P23123 = {"23123", &P23123_4};	/*Profile ID - 23123*/
const Value V22777_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_279 = {S1, 0, &V22777_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22777_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_282 = {S3, 0, &V22777_282_57155, &P22777_279};	/*Property = BlobBuilder*/
const Value V22777_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_350 = {S4, 0, &V22777_350_57446, &P22777_282};	/*Property = Canvas*/
const Value V22777_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_283 = {S5, 0, &V22777_283_57161, &P22777_350};	/*Property = CssBackground*/
const Value V22777_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_284 = {S6, 0, &V22777_284_57164, &P22777_283};	/*Property = CssBorderImage*/
const Value V22777_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_285 = {S7, 0, &V22777_285_57167, &P22777_284};	/*Property = CssCanvas*/
const Value V22777_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_286 = {S8, 0, &V22777_286_57170, &P22777_285};	/*Property = CssColor*/
const Value V22777_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_237 = {S9, 0, &V22777_237_55786, &P22777_286};	/*Property = CssColumn*/
const Value V22777_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_287 = {S10, 0, &V22777_287_57173, &P22777_237};	/*Property = CssFlexbox*/
const Value V22777_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_288 = {S11, 0, &V22777_288_57176, &P22777_287};	/*Property = CssFont*/
const Value V22777_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_289 = {S12, 0, &V22777_289_86899, &P22777_288};	/*Property = CssImages*/
const Value V22777_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_291 = {S13, 0, &V22777_291_57182, &P22777_289};	/*Property = CssMediaQueries*/
const Value V22777_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_292 = {S14, 0, &V22777_292_57185, &P22777_291};	/*Property = CssMinMax*/
const Value V22777_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_293 = {S15, 0, &V22777_293_57188, &P22777_292};	/*Property = CssOverflow*/
const Value V22777_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_294 = {S16, 0, &V22777_294_57191, &P22777_293};	/*Property = CssPosition*/
const Value V22777_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_295 = {S17, 0, &V22777_295_57194, &P22777_294};	/*Property = CssText*/
const Value V22777_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_297 = {S18, 0, &V22777_297_57197, &P22777_295};	/*Property = CssTransforms*/
const Value V22777_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_241 = {S19, 0, &V22777_241_55801, &P22777_297};	/*Property = CssTransitions*/
const Value V22777_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_298 = {S20, 0, &V22777_298_57208, &P22777_241};	/*Property = CssUI*/
const Value V22777_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_300 = {S21, 0, &V22777_300_57217, &P22777_298};	/*Property = DataSet*/
const Value V22777_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_301 = {S22, 0, &V22777_301_57220, &P22777_300};	/*Property = DataUrl*/
const Value V22777_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_302 = {S23, 0, &V22777_302_57223, &P22777_301};	/*Property = DeviceOrientation*/
const Value V22777_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_304 = {S24, 0, &V22777_304_57229, &P22777_302};	/*Property = FileReader*/
const Value V22777_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_305 = {S25, 0, &V22777_305_57232, &P22777_304};	/*Property = FileSaver*/
const Value V22777_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_306 = {S26, 0, &V22777_306_57238, &P22777_305};	/*Property = FileWriter*/
const Value V22777_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_307 = {S27, 0, &V22777_307_57241, &P22777_306};	/*Property = FormData*/
const Value V22777_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_309 = {S28, 0, &V22777_309_57247, &P22777_307};	/*Property = Fullscreen*/
const Value V22777_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_310 = {S29, 0, &V22777_310_57250, &P22777_309};	/*Property = GeoLocation*/
const Value V22777_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_312 = {S30, 0, &V22777_312_57256, &P22777_310};	/*Property = History*/
const Value V22777_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_314 = {S31, 0, &V22777_314_57262, &P22777_312};	/*Property = Html5*/
const Value V22777_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_313 = {S32, 0, &V22777_313_57259, &P22777_314};	/*Property = Html-Media-Capture*/
const Value V22777_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_315 = {S33, 0, &V22777_315_57265, &P22777_313};	/*Property = Iframe*/
const Value V22777_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_316 = {S34, 0, &V22777_316_57268, &P22777_315};	/*Property = IndexedDB*/
const Value V22777_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_317 = {S35, 0, &V22777_317_57271, &P22777_316};	/*Property = Json*/
const Value V22777_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22777_15 = {S37, 0, &V22777_15_29, &P22777_317};	/*Property = LayoutEngine*/
const Value V22777_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_344 = {S38, 0, &V22777_344_57274, &P22777_15};	/*Property = Masking*/
const Value V22777_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_320 = {S39, 0, &V22777_320_57292, &P22777_344};	/*Property = PostMessage*/
const Value V22777_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_321 = {S40, 0, &V22777_321_57295, &P22777_320};	/*Property = Progress*/
const Value V22777_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_322 = {S41, 0, &V22777_322_57298, &P22777_321};	/*Property = Prompts*/
const Value V22777_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_323 = {S42, 0, &V22777_323_57301, &P22777_322};	/*Property = Selector*/
const Value V22777_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_324 = {S43, 0, &V22777_324_57307, &P22777_323};	/*Property = Svg*/
const Value V22777_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_327 = {S44, 0, &V22777_327_57316, &P22777_324};	/*Property = TouchEvents*/
const Value V22777_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_328 = {S45, 0, &V22777_328_57319, &P22777_327};	/*Property = Track*/
const Value V22777_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_329 = {S46, 0, &V22777_329_57325, &P22777_328};	/*Property = Video*/
const Value V22777_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_330 = {S47, 0, &V22777_330_57328, &P22777_329};	/*Property = Viewport*/
const Value V22777_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_332 = {S48, 0, &V22777_332_57340, &P22777_330};	/*Property = WebWorkers*/
const Value V22777_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22777_333 = {S49, 0, &V22777_333_57343, &P22777_332};	/*Property = Xhr2*/
const Profile P22777 = {"22777", &P22777_333};	/*Profile ID - 22777*/
const Value V23129_1_1 = {S2, NULL};	/*Value = True*/
const Property P23129_1 = {S50, 0, &V23129_1_1, NULL};	/*Property = IsMobile*/
const Value V23129_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23129_5 = {S52, 0, &V23129_5_1222, &P23129_1};	/*Property = ScreenPixelsHeight*/
const Value V23129_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23129_4 = {S54, 0, &V23129_4_1224, &P23129_5};	/*Property = ScreenPixelsWidth*/
const Profile P23129 = {"23129", &P23129_4};	/*Profile ID - 23129*/
const Value V24142_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24142_279 = {S1, 0, &V24142_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24142_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24142_282 = {S3, 0, &V24142_282_57154, &P24142_279};	/*Property = BlobBuilder*/
const Value V24142_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24142_350 = {S4, 0, &V24142_350_57444, &P24142_282};	/*Property = Canvas*/
const Value V24142_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24142_283 = {S5, 0, &V24142_283_57159, &P24142_350};	/*Property = CssBackground*/
const Value V24142_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24142_284 = {S6, 0, &V24142_284_57162, &P24142_283};	/*Property = CssBorderImage*/
const Value V24142_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24142_285 = {S7, 0, &V24142_285_57166, &P24142_284};	/*Property = CssCanvas*/
const Value V24142_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24142_286 = {S8, 0, &V24142_286_57168, &P24142_285};	/*Property = CssColor*/
const Value V24142_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24142_237 = {S9, 0, &V24142_237_55786, &P24142_286};	/*Property = CssColumn*/
const Value V24142_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24142_287 = {S10, 0, &V24142_287_57172, &P24142_237};	/*Property = CssFlexbox*/
const Value V24142_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24142_288 = {S11, 0, &V24142_288_57174, &P24142_287};	/*Property = CssFont*/
const Value V24142_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24142_289 = {S12, 0, &V24142_289_57759, &P24142_288};	/*Property = CssImages*/
const Value V24142_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24142_291 = {S13, 0, &V24142_291_57180, &P24142_289};	/*Property = CssMediaQueries*/
const Value V24142_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24142_292 = {S14, 0, &V24142_292_57183, &P24142_291};	/*Property = CssMinMax*/
const Value V24142_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24142_293 = {S15, 0, &V24142_293_57187, &P24142_292};	/*Property = CssOverflow*/
const Value V24142_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24142_294 = {S16, 0, &V24142_294_57189, &P24142_293};	/*Property = CssPosition*/
const Value V24142_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24142_295 = {S17, 0, &V24142_295_57192, &P24142_294};	/*Property = CssText*/
const Value V24142_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24142_297 = {S18, 0, &V24142_297_57195, &P24142_295};	/*Property = CssTransforms*/
const Value V24142_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24142_241 = {S19, 0, &V24142_241_55799, &P24142_297};	/*Property = CssTransitions*/
const Value V24142_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24142_298 = {S20, 0, &V24142_298_57206, &P24142_241};	/*Property = CssUI*/
const Value V24142_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24142_300 = {S21, 0, &V24142_300_57215, &P24142_298};	/*Property = DataSet*/
const Value V24142_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24142_301 = {S22, 0, &V24142_301_57218, &P24142_300};	/*Property = DataUrl*/
const Value V24142_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24142_302 = {S23, 0, &V24142_302_57222, &P24142_301};	/*Property = DeviceOrientation*/
const Value V24142_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24142_304 = {S24, 0, &V24142_304_57227, &P24142_302};	/*Property = FileReader*/
const Value V24142_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24142_305 = {S25, 0, &V24142_305_57231, &P24142_304};	/*Property = FileSaver*/
const Value V24142_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24142_306 = {S26, 0, &V24142_306_57237, &P24142_305};	/*Property = FileWriter*/
const Value V24142_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24142_307 = {S27, 0, &V24142_307_57239, &P24142_306};	/*Property = FormData*/
const Value V24142_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24142_309 = {S28, 0, &V24142_309_57246, &P24142_307};	/*Property = Fullscreen*/
const Value V24142_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24142_310 = {S29, 0, &V24142_310_57248, &P24142_309};	/*Property = GeoLocation*/
const Value V24142_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24142_312 = {S30, 0, &V24142_312_57254, &P24142_310};	/*Property = History*/
const Value V24142_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24142_314 = {S31, 0, &V24142_314_57260, &P24142_312};	/*Property = Html5*/
const Value V24142_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24142_313 = {S32, 0, &V24142_313_57258, &P24142_314};	/*Property = Html-Media-Capture*/
const Value V24142_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24142_315 = {S33, 0, &V24142_315_57264, &P24142_313};	/*Property = Iframe*/
const Value V24142_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24142_316 = {S34, 0, &V24142_316_57266, &P24142_315};	/*Property = IndexedDB*/
const Value V24142_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24142_317 = {S35, 0, &V24142_317_57269, &P24142_316};	/*Property = Json*/
const Value V24142_15_81502 = {S141, NULL};	/*Value = Blink*/
const Property P24142_15 = {S37, 0, &V24142_15_81502, &P24142_317};	/*Property = LayoutEngine*/
const Value V24142_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24142_344 = {S38, 0, &V24142_344_57272, &P24142_15};	/*Property = Masking*/
const Value V24142_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24142_320 = {S39, 0, &V24142_320_57290, &P24142_344};	/*Property = PostMessage*/
const Value V24142_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24142_321 = {S40, 0, &V24142_321_57293, &P24142_320};	/*Property = Progress*/
const Value V24142_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24142_322 = {S41, 0, &V24142_322_57296, &P24142_321};	/*Property = Prompts*/
const Value V24142_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24142_323 = {S42, 0, &V24142_323_57299, &P24142_322};	/*Property = Selector*/
const Value V24142_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24142_324 = {S43, 0, &V24142_324_57305, &P24142_323};	/*Property = Svg*/
const Value V24142_327_57315 = {S0, NULL};	/*Value = False*/
const Property P24142_327 = {S44, 0, &V24142_327_57315, &P24142_324};	/*Property = TouchEvents*/
const Value V24142_328_57317 = {S2, NULL};	/*Value = True*/
const Property P24142_328 = {S45, 0, &V24142_328_57317, &P24142_327};	/*Property = Track*/
const Value V24142_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24142_329 = {S46, 0, &V24142_329_57323, &P24142_328};	/*Property = Video*/
const Value V24142_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24142_330 = {S47, 0, &V24142_330_57326, &P24142_329};	/*Property = Viewport*/
const Value V24142_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24142_332 = {S48, 0, &V24142_332_57338, &P24142_330};	/*Property = WebWorkers*/
const Value V24142_333_57341 = {S2, NULL};	/*Value = True*/
const Property P24142_333 = {S49, 0, &V24142_333_57341, &P24142_332};	/*Property = Xhr2*/
const Profile P24142 = {"24142", &P24142_333};	/*Profile ID - 24142*/
const Value V22925_1_1 = {S2, NULL};	/*Value = True*/
const Property P22925_1 = {S50, 0, &V22925_1_1, NULL};	/*Property = IsMobile*/
const Value V22925_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22925_5 = {S52, 0, &V22925_5_1403, &P22925_1};	/*Property = ScreenPixelsHeight*/
const Value V22925_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22925_4 = {S54, 0, &V22925_4_1404, &P22925_5};	/*Property = ScreenPixelsWidth*/
const Profile P22925 = {"22925", &P22925_4};	/*Profile ID - 22925*/
const Value V24685_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24685_279 = {S1, 0, &V24685_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24685_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24685_282 = {S3, 0, &V24685_282_57154, &P24685_279};	/*Property = BlobBuilder*/
const Value V24685_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24685_350 = {S4, 0, &V24685_350_57444, &P24685_282};	/*Property = Canvas*/
const Value V24685_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24685_283 = {S5, 0, &V24685_283_57159, &P24685_350};	/*Property = CssBackground*/
const Value V24685_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24685_284 = {S6, 0, &V24685_284_57162, &P24685_283};	/*Property = CssBorderImage*/
const Value V24685_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24685_285 = {S7, 0, &V24685_285_57166, &P24685_284};	/*Property = CssCanvas*/
const Value V24685_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24685_286 = {S8, 0, &V24685_286_57168, &P24685_285};	/*Property = CssColor*/
const Value V24685_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24685_237 = {S9, 0, &V24685_237_55786, &P24685_286};	/*Property = CssColumn*/
const Value V24685_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24685_287 = {S10, 0, &V24685_287_57172, &P24685_237};	/*Property = CssFlexbox*/
const Value V24685_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24685_288 = {S11, 0, &V24685_288_57174, &P24685_287};	/*Property = CssFont*/
const Value V24685_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24685_289 = {S12, 0, &V24685_289_57759, &P24685_288};	/*Property = CssImages*/
const Value V24685_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24685_291 = {S13, 0, &V24685_291_57180, &P24685_289};	/*Property = CssMediaQueries*/
const Value V24685_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24685_292 = {S14, 0, &V24685_292_57183, &P24685_291};	/*Property = CssMinMax*/
const Value V24685_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24685_293 = {S15, 0, &V24685_293_57187, &P24685_292};	/*Property = CssOverflow*/
const Value V24685_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24685_294 = {S16, 0, &V24685_294_57189, &P24685_293};	/*Property = CssPosition*/
const Value V24685_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24685_295 = {S17, 0, &V24685_295_57192, &P24685_294};	/*Property = CssText*/
const Value V24685_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24685_297 = {S18, 0, &V24685_297_57195, &P24685_295};	/*Property = CssTransforms*/
const Value V24685_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24685_241 = {S19, 0, &V24685_241_55799, &P24685_297};	/*Property = CssTransitions*/
const Value V24685_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24685_298 = {S20, 0, &V24685_298_57206, &P24685_241};	/*Property = CssUI*/
const Value V24685_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24685_300 = {S21, 0, &V24685_300_57215, &P24685_298};	/*Property = DataSet*/
const Value V24685_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24685_301 = {S22, 0, &V24685_301_57218, &P24685_300};	/*Property = DataUrl*/
const Value V24685_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24685_302 = {S23, 0, &V24685_302_57222, &P24685_301};	/*Property = DeviceOrientation*/
const Value V24685_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24685_304 = {S24, 0, &V24685_304_57227, &P24685_302};	/*Property = FileReader*/
const Value V24685_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24685_305 = {S25, 0, &V24685_305_57231, &P24685_304};	/*Property = FileSaver*/
const Value V24685_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24685_306 = {S26, 0, &V24685_306_57237, &P24685_305};	/*Property = FileWriter*/
const Value V24685_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24685_307 = {S27, 0, &V24685_307_57239, &P24685_306};	/*Property = FormData*/
const Value V24685_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24685_309 = {S28, 0, &V24685_309_57246, &P24685_307};	/*Property = Fullscreen*/
const Value V24685_310_57249 = {S0, NULL};	/*Value = False*/
const Property P24685_310 = {S29, 0, &V24685_310_57249, &P24685_309};	/*Property = GeoLocation*/
const Value V24685_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24685_312 = {S30, 0, &V24685_312_57254, &P24685_310};	/*Property = History*/
const Value V24685_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24685_314 = {S31, 0, &V24685_314_57260, &P24685_312};	/*Property = Html5*/
const Value V24685_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24685_313 = {S32, 0, &V24685_313_57258, &P24685_314};	/*Property = Html-Media-Capture*/
const Value V24685_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24685_315 = {S33, 0, &V24685_315_57264, &P24685_313};	/*Property = Iframe*/
const Value V24685_316_57267 = {S0, NULL};	/*Value = False*/
const Property P24685_316 = {S34, 0, &V24685_316_57267, &P24685_315};	/*Property = IndexedDB*/
const Value V24685_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24685_317 = {S35, 0, &V24685_317_57269, &P24685_316};	/*Property = Json*/
const Value V24685_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24685_15 = {S37, 0, &V24685_15_28, &P24685_317};	/*Property = LayoutEngine*/
const Value V24685_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24685_344 = {S38, 0, &V24685_344_57272, &P24685_15};	/*Property = Masking*/
const Value V24685_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24685_320 = {S39, 0, &V24685_320_57290, &P24685_344};	/*Property = PostMessage*/
const Value V24685_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24685_321 = {S40, 0, &V24685_321_57293, &P24685_320};	/*Property = Progress*/
const Value V24685_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24685_322 = {S41, 0, &V24685_322_57296, &P24685_321};	/*Property = Prompts*/
const Value V24685_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24685_323 = {S42, 0, &V24685_323_57299, &P24685_322};	/*Property = Selector*/
const Value V24685_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24685_324 = {S43, 0, &V24685_324_57305, &P24685_323};	/*Property = Svg*/
const Value V24685_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24685_327 = {S44, 0, &V24685_327_57314, &P24685_324};	/*Property = TouchEvents*/
const Value V24685_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24685_328 = {S45, 0, &V24685_328_57318, &P24685_327};	/*Property = Track*/
const Value V24685_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24685_329 = {S46, 0, &V24685_329_57323, &P24685_328};	/*Property = Video*/
const Value V24685_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24685_330 = {S47, 0, &V24685_330_57326, &P24685_329};	/*Property = Viewport*/
const Value V24685_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24685_332 = {S48, 0, &V24685_332_57339, &P24685_330};	/*Property = WebWorkers*/
const Value V24685_333_57341 = {S2, NULL};	/*Value = True*/
const Property P24685_333 = {S49, 0, &V24685_333_57341, &P24685_332};	/*Property = Xhr2*/
const Profile P24685 = {"24685", &P24685_333};	/*Profile ID - 24685*/
const Profile P22711 = {"22711", NULL};	/*Profile ID - 22711*/
const Value V22600_1_1 = {S2, NULL};	/*Value = True*/
const Property P22600_1 = {S50, 0, &V22600_1_1, NULL};	/*Property = IsMobile*/
const Value V22600_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22600_5 = {S52, 0, &V22600_5_59138, &P22600_1};	/*Property = ScreenPixelsHeight*/
const Value V22600_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22600_4 = {S54, 0, &V22600_4_59970, &P22600_5};	/*Property = ScreenPixelsWidth*/
const Profile P22600 = {"22600", &P22600_4};	/*Profile ID - 22600*/
const Value V22883_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_279 = {S1, 0, &V22883_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22883_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_282 = {S3, 0, &V22883_282_57155, &P22883_279};	/*Property = BlobBuilder*/
const Value V22883_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_350 = {S4, 0, &V22883_350_57446, &P22883_282};	/*Property = Canvas*/
const Value V22883_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_283 = {S5, 0, &V22883_283_57161, &P22883_350};	/*Property = CssBackground*/
const Value V22883_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_284 = {S6, 0, &V22883_284_57164, &P22883_283};	/*Property = CssBorderImage*/
const Value V22883_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_285 = {S7, 0, &V22883_285_57167, &P22883_284};	/*Property = CssCanvas*/
const Value V22883_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_286 = {S8, 0, &V22883_286_57170, &P22883_285};	/*Property = CssColor*/
const Value V22883_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_237 = {S9, 0, &V22883_237_55786, &P22883_286};	/*Property = CssColumn*/
const Value V22883_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_287 = {S10, 0, &V22883_287_57173, &P22883_237};	/*Property = CssFlexbox*/
const Value V22883_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_288 = {S11, 0, &V22883_288_57176, &P22883_287};	/*Property = CssFont*/
const Value V22883_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_289 = {S12, 0, &V22883_289_86899, &P22883_288};	/*Property = CssImages*/
const Value V22883_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_291 = {S13, 0, &V22883_291_57182, &P22883_289};	/*Property = CssMediaQueries*/
const Value V22883_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_292 = {S14, 0, &V22883_292_57185, &P22883_291};	/*Property = CssMinMax*/
const Value V22883_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_293 = {S15, 0, &V22883_293_57188, &P22883_292};	/*Property = CssOverflow*/
const Value V22883_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_294 = {S16, 0, &V22883_294_57191, &P22883_293};	/*Property = CssPosition*/
const Value V22883_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_295 = {S17, 0, &V22883_295_57194, &P22883_294};	/*Property = CssText*/
const Value V22883_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_297 = {S18, 0, &V22883_297_57197, &P22883_295};	/*Property = CssTransforms*/
const Value V22883_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_241 = {S19, 0, &V22883_241_55801, &P22883_297};	/*Property = CssTransitions*/
const Value V22883_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_298 = {S20, 0, &V22883_298_57208, &P22883_241};	/*Property = CssUI*/
const Value V22883_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_300 = {S21, 0, &V22883_300_57217, &P22883_298};	/*Property = DataSet*/
const Value V22883_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_301 = {S22, 0, &V22883_301_57220, &P22883_300};	/*Property = DataUrl*/
const Value V22883_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_302 = {S23, 0, &V22883_302_57223, &P22883_301};	/*Property = DeviceOrientation*/
const Value V22883_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_304 = {S24, 0, &V22883_304_57229, &P22883_302};	/*Property = FileReader*/
const Value V22883_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_305 = {S25, 0, &V22883_305_57232, &P22883_304};	/*Property = FileSaver*/
const Value V22883_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_306 = {S26, 0, &V22883_306_57238, &P22883_305};	/*Property = FileWriter*/
const Value V22883_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_307 = {S27, 0, &V22883_307_57241, &P22883_306};	/*Property = FormData*/
const Value V22883_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_309 = {S28, 0, &V22883_309_57247, &P22883_307};	/*Property = Fullscreen*/
const Value V22883_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_310 = {S29, 0, &V22883_310_57250, &P22883_309};	/*Property = GeoLocation*/
const Value V22883_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_312 = {S30, 0, &V22883_312_57256, &P22883_310};	/*Property = History*/
const Value V22883_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_314 = {S31, 0, &V22883_314_57262, &P22883_312};	/*Property = Html5*/
const Value V22883_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_313 = {S32, 0, &V22883_313_57259, &P22883_314};	/*Property = Html-Media-Capture*/
const Value V22883_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_315 = {S33, 0, &V22883_315_57265, &P22883_313};	/*Property = Iframe*/
const Value V22883_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_316 = {S34, 0, &V22883_316_57268, &P22883_315};	/*Property = IndexedDB*/
const Value V22883_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_317 = {S35, 0, &V22883_317_57271, &P22883_316};	/*Property = Json*/
const Value V22883_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22883_15 = {S37, 0, &V22883_15_28, &P22883_317};	/*Property = LayoutEngine*/
const Value V22883_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_344 = {S38, 0, &V22883_344_57274, &P22883_15};	/*Property = Masking*/
const Value V22883_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_320 = {S39, 0, &V22883_320_57292, &P22883_344};	/*Property = PostMessage*/
const Value V22883_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_321 = {S40, 0, &V22883_321_57295, &P22883_320};	/*Property = Progress*/
const Value V22883_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_322 = {S41, 0, &V22883_322_57298, &P22883_321};	/*Property = Prompts*/
const Value V22883_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_323 = {S42, 0, &V22883_323_57301, &P22883_322};	/*Property = Selector*/
const Value V22883_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_324 = {S43, 0, &V22883_324_57307, &P22883_323};	/*Property = Svg*/
const Value V22883_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_327 = {S44, 0, &V22883_327_57316, &P22883_324};	/*Property = TouchEvents*/
const Value V22883_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_328 = {S45, 0, &V22883_328_57319, &P22883_327};	/*Property = Track*/
const Value V22883_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_329 = {S46, 0, &V22883_329_57325, &P22883_328};	/*Property = Video*/
const Value V22883_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_330 = {S47, 0, &V22883_330_57328, &P22883_329};	/*Property = Viewport*/
const Value V22883_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_332 = {S48, 0, &V22883_332_57340, &P22883_330};	/*Property = WebWorkers*/
const Value V22883_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22883_333 = {S49, 0, &V22883_333_57343, &P22883_332};	/*Property = Xhr2*/
const Profile P22883 = {"22883", &P22883_333};	/*Profile ID - 22883*/
const Value V22579_1_1 = {S2, NULL};	/*Value = True*/
const Property P22579_1 = {S50, 0, &V22579_1_1, NULL};	/*Property = IsMobile*/
const Value V22579_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22579_5 = {S52, 0, &V22579_5_9981, &P22579_1};	/*Property = ScreenPixelsHeight*/
const Value V22579_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P22579_4 = {S54, 0, &V22579_4_37617, &P22579_5};	/*Property = ScreenPixelsWidth*/
const Profile P22579 = {"22579", &P22579_4};	/*Profile ID - 22579*/
const Value V22655_1_1 = {S2, NULL};	/*Value = True*/
const Property P22655_1 = {S50, 0, &V22655_1_1, NULL};	/*Property = IsMobile*/
const Value V22655_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22655_5 = {S52, 0, &V22655_5_9981, &P22655_1};	/*Property = ScreenPixelsHeight*/
const Value V22655_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22655_4 = {S54, 0, &V22655_4_39098, &P22655_5};	/*Property = ScreenPixelsWidth*/
const Profile P22655 = {"22655", &P22655_4};	/*Profile ID - 22655*/
const Value V22654_1_1 = {S2, NULL};	/*Value = True*/
const Property P22654_1 = {S50, 0, &V22654_1_1, NULL};	/*Property = IsMobile*/
const Value V22654_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22654_5 = {S52, 0, &V22654_5_9981, &P22654_1};	/*Property = ScreenPixelsHeight*/
const Value V22654_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22654_4 = {S54, 0, &V22654_4_39098, &P22654_5};	/*Property = ScreenPixelsWidth*/
const Profile P22654 = {"22654", &P22654_4};	/*Profile ID - 22654*/
const Value V22656_1_1 = {S2, NULL};	/*Value = True*/
const Property P22656_1 = {S50, 0, &V22656_1_1, NULL};	/*Property = IsMobile*/
const Value V22656_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22656_5 = {S52, 0, &V22656_5_59138, &P22656_1};	/*Property = ScreenPixelsHeight*/
const Value V22656_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22656_4 = {S54, 0, &V22656_4_59970, &P22656_5};	/*Property = ScreenPixelsWidth*/
const Profile P22656 = {"22656", &P22656_4};	/*Profile ID - 22656*/
const Value V22653_1_1 = {S2, NULL};	/*Value = True*/
const Property P22653_1 = {S50, 0, &V22653_1_1, NULL};	/*Property = IsMobile*/
const Value V22653_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22653_5 = {S52, 0, &V22653_5_59138, &P22653_1};	/*Property = ScreenPixelsHeight*/
const Value V22653_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22653_4 = {S54, 0, &V22653_4_59970, &P22653_5};	/*Property = ScreenPixelsWidth*/
const Profile P22653 = {"22653", &P22653_4};	/*Profile ID - 22653*/
const Value V24189_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_279 = {S1, 0, &V24189_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24189_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_282 = {S3, 0, &V24189_282_57155, &P24189_279};	/*Property = BlobBuilder*/
const Value V24189_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_350 = {S4, 0, &V24189_350_57446, &P24189_282};	/*Property = Canvas*/
const Value V24189_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_283 = {S5, 0, &V24189_283_57161, &P24189_350};	/*Property = CssBackground*/
const Value V24189_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_284 = {S6, 0, &V24189_284_57164, &P24189_283};	/*Property = CssBorderImage*/
const Value V24189_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_285 = {S7, 0, &V24189_285_57167, &P24189_284};	/*Property = CssCanvas*/
const Value V24189_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_286 = {S8, 0, &V24189_286_57170, &P24189_285};	/*Property = CssColor*/
const Value V24189_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_237 = {S9, 0, &V24189_237_55786, &P24189_286};	/*Property = CssColumn*/
const Value V24189_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_287 = {S10, 0, &V24189_287_57173, &P24189_237};	/*Property = CssFlexbox*/
const Value V24189_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_288 = {S11, 0, &V24189_288_57176, &P24189_287};	/*Property = CssFont*/
const Value V24189_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_289 = {S12, 0, &V24189_289_86899, &P24189_288};	/*Property = CssImages*/
const Value V24189_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_291 = {S13, 0, &V24189_291_57182, &P24189_289};	/*Property = CssMediaQueries*/
const Value V24189_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_292 = {S14, 0, &V24189_292_57185, &P24189_291};	/*Property = CssMinMax*/
const Value V24189_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_293 = {S15, 0, &V24189_293_57188, &P24189_292};	/*Property = CssOverflow*/
const Value V24189_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_294 = {S16, 0, &V24189_294_57191, &P24189_293};	/*Property = CssPosition*/
const Value V24189_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_295 = {S17, 0, &V24189_295_57194, &P24189_294};	/*Property = CssText*/
const Value V24189_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_297 = {S18, 0, &V24189_297_57197, &P24189_295};	/*Property = CssTransforms*/
const Value V24189_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_241 = {S19, 0, &V24189_241_55801, &P24189_297};	/*Property = CssTransitions*/
const Value V24189_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_298 = {S20, 0, &V24189_298_57208, &P24189_241};	/*Property = CssUI*/
const Value V24189_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_300 = {S21, 0, &V24189_300_57217, &P24189_298};	/*Property = DataSet*/
const Value V24189_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_301 = {S22, 0, &V24189_301_57220, &P24189_300};	/*Property = DataUrl*/
const Value V24189_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_302 = {S23, 0, &V24189_302_57223, &P24189_301};	/*Property = DeviceOrientation*/
const Value V24189_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_304 = {S24, 0, &V24189_304_57229, &P24189_302};	/*Property = FileReader*/
const Value V24189_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_305 = {S25, 0, &V24189_305_57232, &P24189_304};	/*Property = FileSaver*/
const Value V24189_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_306 = {S26, 0, &V24189_306_57238, &P24189_305};	/*Property = FileWriter*/
const Value V24189_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_307 = {S27, 0, &V24189_307_57241, &P24189_306};	/*Property = FormData*/
const Value V24189_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_309 = {S28, 0, &V24189_309_57247, &P24189_307};	/*Property = Fullscreen*/
const Value V24189_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_310 = {S29, 0, &V24189_310_57250, &P24189_309};	/*Property = GeoLocation*/
const Value V24189_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_312 = {S30, 0, &V24189_312_57256, &P24189_310};	/*Property = History*/
const Value V24189_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_314 = {S31, 0, &V24189_314_57262, &P24189_312};	/*Property = Html5*/
const Value V24189_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_313 = {S32, 0, &V24189_313_57259, &P24189_314};	/*Property = Html-Media-Capture*/
const Value V24189_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_315 = {S33, 0, &V24189_315_57265, &P24189_313};	/*Property = Iframe*/
const Value V24189_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_316 = {S34, 0, &V24189_316_57268, &P24189_315};	/*Property = IndexedDB*/
const Value V24189_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_317 = {S35, 0, &V24189_317_57271, &P24189_316};	/*Property = Json*/
const Value V24189_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P24189_15 = {S37, 0, &V24189_15_29, &P24189_317};	/*Property = LayoutEngine*/
const Value V24189_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_344 = {S38, 0, &V24189_344_57274, &P24189_15};	/*Property = Masking*/
const Value V24189_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_320 = {S39, 0, &V24189_320_57292, &P24189_344};	/*Property = PostMessage*/
const Value V24189_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_321 = {S40, 0, &V24189_321_57295, &P24189_320};	/*Property = Progress*/
const Value V24189_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_322 = {S41, 0, &V24189_322_57298, &P24189_321};	/*Property = Prompts*/
const Value V24189_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_323 = {S42, 0, &V24189_323_57301, &P24189_322};	/*Property = Selector*/
const Value V24189_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_324 = {S43, 0, &V24189_324_57307, &P24189_323};	/*Property = Svg*/
const Value V24189_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_327 = {S44, 0, &V24189_327_57316, &P24189_324};	/*Property = TouchEvents*/
const Value V24189_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_328 = {S45, 0, &V24189_328_57319, &P24189_327};	/*Property = Track*/
const Value V24189_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_329 = {S46, 0, &V24189_329_57325, &P24189_328};	/*Property = Video*/
const Value V24189_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_330 = {S47, 0, &V24189_330_57328, &P24189_329};	/*Property = Viewport*/
const Value V24189_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_332 = {S48, 0, &V24189_332_57340, &P24189_330};	/*Property = WebWorkers*/
const Value V24189_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24189_333 = {S49, 0, &V24189_333_57343, &P24189_332};	/*Property = Xhr2*/
const Profile P24189 = {"24189", &P24189_333};	/*Profile ID - 24189*/
const Value V23422_1_1 = {S2, NULL};	/*Value = True*/
const Property P23422_1 = {S50, 0, &V23422_1_1, NULL};	/*Property = IsMobile*/
const Value V23422_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23422_5 = {S52, 0, &V23422_5_37569, &P23422_1};	/*Property = ScreenPixelsHeight*/
const Value V23422_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23422_4 = {S54, 0, &V23422_4_5645, &P23422_5};	/*Property = ScreenPixelsWidth*/
const Profile P23422 = {"23422", &P23422_4};	/*Profile ID - 23422*/
const Value V23661_1_2 = {S0, NULL};	/*Value = False*/
const Property P23661_1 = {S50, 0, &V23661_1_2, NULL};	/*Property = IsMobile*/
const Value V23661_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23661_5 = {S52, 0, &V23661_5_60417, &P23661_1};	/*Property = ScreenPixelsHeight*/
const Value V23661_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23661_4 = {S54, 0, &V23661_4_57095, &P23661_5};	/*Property = ScreenPixelsWidth*/
const Profile P23661 = {"23661", &P23661_4};	/*Profile ID - 23661*/
const Value V23673_1_2 = {S0, NULL};	/*Value = False*/
const Property P23673_1 = {S50, 0, &V23673_1_2, NULL};	/*Property = IsMobile*/
const Value V23673_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23673_5 = {S52, 0, &V23673_5_60417, &P23673_1};	/*Property = ScreenPixelsHeight*/
const Value V23673_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23673_4 = {S54, 0, &V23673_4_57095, &P23673_5};	/*Property = ScreenPixelsWidth*/
const Profile P23673 = {"23673", &P23673_4};	/*Profile ID - 23673*/
const Value V23289_1_1 = {S2, NULL};	/*Value = True*/
const Property P23289_1 = {S50, 0, &V23289_1_1, NULL};	/*Property = IsMobile*/
const Value V23289_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23289_5 = {S52, 0, &V23289_5_1277, &P23289_1};	/*Property = ScreenPixelsHeight*/
const Value V23289_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23289_4 = {S54, 0, &V23289_4_1279, &P23289_5};	/*Property = ScreenPixelsWidth*/
const Profile P23289 = {"23289", &P23289_4};	/*Profile ID - 23289*/
const Value V24511_1_1 = {S2, NULL};	/*Value = True*/
const Property P24511_1 = {S50, 0, &V24511_1_1, NULL};	/*Property = IsMobile*/
const Value V24511_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24511_5 = {S52, 0, &V24511_5_1227, &P24511_1};	/*Property = ScreenPixelsHeight*/
const Value V24511_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24511_4 = {S54, 0, &V24511_4_1229, &P24511_5};	/*Property = ScreenPixelsWidth*/
const Profile P24511 = {"24511", &P24511_4};	/*Profile ID - 24511*/
const Value V23283_1_1 = {S2, NULL};	/*Value = True*/
const Property P23283_1 = {S50, 0, &V23283_1_1, NULL};	/*Property = IsMobile*/
const Value V23283_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23283_5 = {S52, 0, &V23283_5_1227, &P23283_1};	/*Property = ScreenPixelsHeight*/
const Value V23283_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23283_4 = {S54, 0, &V23283_4_1229, &P23283_5};	/*Property = ScreenPixelsWidth*/
const Profile P23283 = {"23283", &P23283_4};	/*Profile ID - 23283*/
const Value V23159_1_1 = {S2, NULL};	/*Value = True*/
const Property P23159_1 = {S50, 0, &V23159_1_1, NULL};	/*Property = IsMobile*/
const Value V23159_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23159_5 = {S52, 0, &V23159_5_9981, &P23159_1};	/*Property = ScreenPixelsHeight*/
const Value V23159_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23159_4 = {S54, 0, &V23159_4_39098, &P23159_5};	/*Property = ScreenPixelsWidth*/
const Profile P23159 = {"23159", &P23159_4};	/*Profile ID - 23159*/
const Value V22970_1_1 = {S2, NULL};	/*Value = True*/
const Property P22970_1 = {S50, 0, &V22970_1_1, NULL};	/*Property = IsMobile*/
const Value V22970_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22970_5 = {S52, 0, &V22970_5_1227, &P22970_1};	/*Property = ScreenPixelsHeight*/
const Value V22970_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22970_4 = {S54, 0, &V22970_4_1229, &P22970_5};	/*Property = ScreenPixelsWidth*/
const Profile P22970 = {"22970", &P22970_4};	/*Profile ID - 22970*/
const Value V23432_1_1 = {S2, NULL};	/*Value = True*/
const Property P23432_1 = {S50, 0, &V23432_1_1, NULL};	/*Property = IsMobile*/
const Value V23432_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23432_5 = {S52, 0, &V23432_5_1458, &P23432_1};	/*Property = ScreenPixelsHeight*/
const Value V23432_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23432_4 = {S54, 0, &V23432_4_37617, &P23432_5};	/*Property = ScreenPixelsWidth*/
const Profile P23432 = {"23432", &P23432_4};	/*Profile ID - 23432*/
const Value V22947_1_1 = {S2, NULL};	/*Value = True*/
const Property P22947_1 = {S50, 0, &V22947_1_1, NULL};	/*Property = IsMobile*/
const Value V22947_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22947_5 = {S52, 0, &V22947_5_1458, &P22947_1};	/*Property = ScreenPixelsHeight*/
const Value V22947_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22947_4 = {S54, 0, &V22947_4_1460, &P22947_5};	/*Property = ScreenPixelsWidth*/
const Profile P22947 = {"22947", &P22947_4};	/*Profile ID - 22947*/
const Value V23456_1_1 = {S2, NULL};	/*Value = True*/
const Property P23456_1 = {S50, 0, &V23456_1_1, NULL};	/*Property = IsMobile*/
const Value V23456_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23456_5 = {S52, 0, &V23456_5_37569, &P23456_1};	/*Property = ScreenPixelsHeight*/
const Value V23456_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23456_4 = {S54, 0, &V23456_4_5645, &P23456_5};	/*Property = ScreenPixelsWidth*/
const Profile P23456 = {"23456", &P23456_4};	/*Profile ID - 23456*/
const Value V22962_1_1 = {S2, NULL};	/*Value = True*/
const Property P22962_1 = {S50, 0, &V22962_1_1, NULL};	/*Property = IsMobile*/
const Value V22962_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22962_5 = {S52, 0, &V22962_5_1403, &P22962_1};	/*Property = ScreenPixelsHeight*/
const Value V22962_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22962_4 = {S54, 0, &V22962_4_1404, &P22962_5};	/*Property = ScreenPixelsWidth*/
const Profile P22962 = {"22962", &P22962_4};	/*Profile ID - 22962*/
const Value V23547_1_1 = {S2, NULL};	/*Value = True*/
const Property P23547_1 = {S50, 0, &V23547_1_1, NULL};	/*Property = IsMobile*/
const Value V23547_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23547_5 = {S52, 0, &V23547_5_1227, &P23547_1};	/*Property = ScreenPixelsHeight*/
const Value V23547_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23547_4 = {S54, 0, &V23547_4_1229, &P23547_5};	/*Property = ScreenPixelsWidth*/
const Profile P23547 = {"23547", &P23547_4};	/*Profile ID - 23547*/
const Value V23034_1_1 = {S2, NULL};	/*Value = True*/
const Property P23034_1 = {S50, 0, &V23034_1_1, NULL};	/*Property = IsMobile*/
const Value V23034_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23034_5 = {S52, 0, &V23034_5_1227, &P23034_1};	/*Property = ScreenPixelsHeight*/
const Value V23034_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23034_4 = {S54, 0, &V23034_4_1229, &P23034_5};	/*Property = ScreenPixelsWidth*/
const Profile P23034 = {"23034", &P23034_4};	/*Profile ID - 23034*/
const Value V23302_1_1 = {S2, NULL};	/*Value = True*/
const Property P23302_1 = {S50, 0, &V23302_1_1, NULL};	/*Property = IsMobile*/
const Value V23302_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23302_5 = {S52, 0, &V23302_5_1277, &P23302_1};	/*Property = ScreenPixelsHeight*/
const Value V23302_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23302_4 = {S54, 0, &V23302_4_1279, &P23302_5};	/*Property = ScreenPixelsWidth*/
const Profile P23302 = {"23302", &P23302_4};	/*Profile ID - 23302*/
const Value V23007_1_1 = {S2, NULL};	/*Value = True*/
const Property P23007_1 = {S50, 0, &V23007_1_1, NULL};	/*Property = IsMobile*/
const Value V23007_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23007_5 = {S52, 0, &V23007_5_1277, &P23007_1};	/*Property = ScreenPixelsHeight*/
const Value V23007_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23007_4 = {S54, 0, &V23007_4_1279, &P23007_5};	/*Property = ScreenPixelsWidth*/
const Profile P23007 = {"23007", &P23007_4};	/*Profile ID - 23007*/
const Value V23251_1_1 = {S2, NULL};	/*Value = True*/
const Property P23251_1 = {S50, 0, &V23251_1_1, NULL};	/*Property = IsMobile*/
const Value V23251_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23251_5 = {S52, 0, &V23251_5_1227, &P23251_1};	/*Property = ScreenPixelsHeight*/
const Value V23251_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23251_4 = {S54, 0, &V23251_4_1229, &P23251_5};	/*Property = ScreenPixelsWidth*/
const Profile P23251 = {"23251", &P23251_4};	/*Profile ID - 23251*/
const Value V23454_1_1 = {S2, NULL};	/*Value = True*/
const Property P23454_1 = {S50, 0, &V23454_1_1, NULL};	/*Property = IsMobile*/
const Value V23454_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23454_5 = {S52, 0, &V23454_5_37569, &P23454_1};	/*Property = ScreenPixelsHeight*/
const Value V23454_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23454_4 = {S54, 0, &V23454_4_5645, &P23454_5};	/*Property = ScreenPixelsWidth*/
const Profile P23454 = {"23454", &P23454_4};	/*Profile ID - 23454*/
const Value V22999_1_1 = {S2, NULL};	/*Value = True*/
const Property P22999_1 = {S50, 0, &V22999_1_1, NULL};	/*Property = IsMobile*/
const Value V22999_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22999_5 = {S52, 0, &V22999_5_9981, &P22999_1};	/*Property = ScreenPixelsHeight*/
const Value V22999_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22999_4 = {S54, 0, &V22999_4_39098, &P22999_5};	/*Property = ScreenPixelsWidth*/
const Profile P22999 = {"22999", &P22999_4};	/*Profile ID - 22999*/
const Value V23962_1_1 = {S2, NULL};	/*Value = True*/
const Property P23962_1 = {S50, 0, &V23962_1_1, NULL};	/*Property = IsMobile*/
const Value V23962_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23962_5 = {S52, 0, &V23962_5_1227, &P23962_1};	/*Property = ScreenPixelsHeight*/
const Value V23962_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23962_4 = {S54, 0, &V23962_4_1229, &P23962_5};	/*Property = ScreenPixelsWidth*/
const Profile P23962 = {"23962", &P23962_4};	/*Profile ID - 23962*/
const Value V22918_1_1 = {S2, NULL};	/*Value = True*/
const Property P22918_1 = {S50, 0, &V22918_1_1, NULL};	/*Property = IsMobile*/
const Value V22918_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22918_5 = {S52, 0, &V22918_5_59138, &P22918_1};	/*Property = ScreenPixelsHeight*/
const Value V22918_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22918_4 = {S54, 0, &V22918_4_59970, &P22918_5};	/*Property = ScreenPixelsWidth*/
const Profile P22918 = {"22918", &P22918_4};	/*Profile ID - 22918*/
const Value V23382_1_1 = {S2, NULL};	/*Value = True*/
const Property P23382_1 = {S50, 0, &V23382_1_1, NULL};	/*Property = IsMobile*/
const Value V23382_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23382_5 = {S52, 0, &V23382_5_37569, &P23382_1};	/*Property = ScreenPixelsHeight*/
const Value V23382_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23382_4 = {S54, 0, &V23382_4_5645, &P23382_5};	/*Property = ScreenPixelsWidth*/
const Profile P23382 = {"23382", &P23382_4};	/*Profile ID - 23382*/
const Value V23796_1_1 = {S2, NULL};	/*Value = True*/
const Property P23796_1 = {S50, 0, &V23796_1_1, NULL};	/*Property = IsMobile*/
const Value V23796_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23796_5 = {S52, 0, &V23796_5_1227, &P23796_1};	/*Property = ScreenPixelsHeight*/
const Value V23796_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23796_4 = {S54, 0, &V23796_4_1229, &P23796_5};	/*Property = ScreenPixelsWidth*/
const Profile P23796 = {"23796", &P23796_4};	/*Profile ID - 23796*/
const Value V23267_1_1 = {S2, NULL};	/*Value = True*/
const Property P23267_1 = {S50, 0, &V23267_1_1, NULL};	/*Property = IsMobile*/
const Value V23267_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23267_5 = {S52, 0, &V23267_5_1458, &P23267_1};	/*Property = ScreenPixelsHeight*/
const Value V23267_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23267_4 = {S54, 0, &V23267_4_37617, &P23267_5};	/*Property = ScreenPixelsWidth*/
const Profile P23267 = {"23267", &P23267_4};	/*Profile ID - 23267*/
const Value V23239_1_1 = {S2, NULL};	/*Value = True*/
const Property P23239_1 = {S50, 0, &V23239_1_1, NULL};	/*Property = IsMobile*/
const Value V23239_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23239_5 = {S52, 0, &V23239_5_1403, &P23239_1};	/*Property = ScreenPixelsHeight*/
const Value V23239_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23239_4 = {S54, 0, &V23239_4_1404, &P23239_5};	/*Property = ScreenPixelsWidth*/
const Profile P23239 = {"23239", &P23239_4};	/*Profile ID - 23239*/
const Value V23540_1_1 = {S2, NULL};	/*Value = True*/
const Property P23540_1 = {S50, 0, &V23540_1_1, NULL};	/*Property = IsMobile*/
const Value V23540_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23540_5 = {S52, 0, &V23540_5_1269, &P23540_1};	/*Property = ScreenPixelsHeight*/
const Value V23540_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23540_4 = {S54, 0, &V23540_4_1229, &P23540_5};	/*Property = ScreenPixelsWidth*/
const Profile P23540 = {"23540", &P23540_4};	/*Profile ID - 23540*/
const Value V23409_1_1 = {S2, NULL};	/*Value = True*/
const Property P23409_1 = {S50, 0, &V23409_1_1, NULL};	/*Property = IsMobile*/
const Value V23409_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23409_5 = {S52, 0, &V23409_5_1227, &P23409_1};	/*Property = ScreenPixelsHeight*/
const Value V23409_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23409_4 = {S54, 0, &V23409_4_1229, &P23409_5};	/*Property = ScreenPixelsWidth*/
const Profile P23409 = {"23409", &P23409_4};	/*Profile ID - 23409*/
const Value V20877_1_1 = {S2, NULL};	/*Value = True*/
const Property P20877_1 = {S50, 0, &V20877_1_1, NULL};	/*Property = IsMobile*/
const Value V20877_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20877_5 = {S52, 0, &V20877_5_1227, &P20877_1};	/*Property = ScreenPixelsHeight*/
const Value V20877_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P20877_4 = {S54, 0, &V20877_4_1229, &P20877_5};	/*Property = ScreenPixelsWidth*/
const Profile P20877 = {"20877", &P20877_4};	/*Profile ID - 20877*/
const Value V23518_1_1 = {S2, NULL};	/*Value = True*/
const Property P23518_1 = {S50, 0, &V23518_1_1, NULL};	/*Property = IsMobile*/
const Value V23518_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23518_5 = {S52, 0, &V23518_5_9981, &P23518_1};	/*Property = ScreenPixelsHeight*/
const Value V23518_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23518_4 = {S54, 0, &V23518_4_39098, &P23518_5};	/*Property = ScreenPixelsWidth*/
const Profile P23518 = {"23518", &P23518_4};	/*Profile ID - 23518*/
const Value V22905_1_1 = {S2, NULL};	/*Value = True*/
const Property P22905_1 = {S50, 0, &V22905_1_1, NULL};	/*Property = IsMobile*/
const Value V22905_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22905_5 = {S52, 0, &V22905_5_1222, &P22905_1};	/*Property = ScreenPixelsHeight*/
const Value V22905_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22905_4 = {S54, 0, &V22905_4_1224, &P22905_5};	/*Property = ScreenPixelsWidth*/
const Profile P22905 = {"22905", &P22905_4};	/*Profile ID - 22905*/
const Value V22916_1_1 = {S2, NULL};	/*Value = True*/
const Property P22916_1 = {S50, 0, &V22916_1_1, NULL};	/*Property = IsMobile*/
const Value V22916_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22916_5 = {S52, 0, &V22916_5_1227, &P22916_1};	/*Property = ScreenPixelsHeight*/
const Value V22916_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22916_4 = {S54, 0, &V22916_4_1229, &P22916_5};	/*Property = ScreenPixelsWidth*/
const Profile P22916 = {"22916", &P22916_4};	/*Profile ID - 22916*/
const Value V22635_1_1 = {S2, NULL};	/*Value = True*/
const Property P22635_1 = {S50, 0, &V22635_1_1, NULL};	/*Property = IsMobile*/
const Value V22635_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22635_5 = {S52, 0, &V22635_5_1458, &P22635_1};	/*Property = ScreenPixelsHeight*/
const Value V22635_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P22635_4 = {S54, 0, &V22635_4_37617, &P22635_5};	/*Property = ScreenPixelsWidth*/
const Profile P22635 = {"22635", &P22635_4};	/*Profile ID - 22635*/
const Value V21191_1_1 = {S2, NULL};	/*Value = True*/
const Property P21191_1 = {S50, 0, &V21191_1_1, NULL};	/*Property = IsMobile*/
const Value V21191_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21191_5 = {S52, 0, &V21191_5_37, &P21191_1};	/*Property = ScreenPixelsHeight*/
const Value V21191_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21191_4 = {S54, 0, &V21191_4_36, &P21191_5};	/*Property = ScreenPixelsWidth*/
const Profile P21191 = {"21191", &P21191_4};	/*Profile ID - 21191*/
const Value V23498_1_1 = {S2, NULL};	/*Value = True*/
const Property P23498_1 = {S50, 0, &V23498_1_1, NULL};	/*Property = IsMobile*/
const Value V23498_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23498_5 = {S52, 0, &V23498_5_1269, &P23498_1};	/*Property = ScreenPixelsHeight*/
const Value V23498_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23498_4 = {S54, 0, &V23498_4_1229, &P23498_5};	/*Property = ScreenPixelsWidth*/
const Profile P23498 = {"23498", &P23498_4};	/*Profile ID - 23498*/
const Value V22759_1_1 = {S2, NULL};	/*Value = True*/
const Property P22759_1 = {S50, 0, &V22759_1_1, NULL};	/*Property = IsMobile*/
const Value V22759_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22759_5 = {S52, 0, &V22759_5_1227, &P22759_1};	/*Property = ScreenPixelsHeight*/
const Value V22759_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22759_4 = {S54, 0, &V22759_4_1460, &P22759_5};	/*Property = ScreenPixelsWidth*/
const Profile P22759 = {"22759", &P22759_4};	/*Profile ID - 22759*/
const Value V22742_1_1 = {S2, NULL};	/*Value = True*/
const Property P22742_1 = {S50, 0, &V22742_1_1, NULL};	/*Property = IsMobile*/
const Value V22742_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22742_5 = {S52, 0, &V22742_5_1458, &P22742_1};	/*Property = ScreenPixelsHeight*/
const Value V22742_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22742_4 = {S54, 0, &V22742_4_1460, &P22742_5};	/*Property = ScreenPixelsWidth*/
const Profile P22742 = {"22742", &P22742_4};	/*Profile ID - 22742*/
const Value V23843_1_1 = {S2, NULL};	/*Value = True*/
const Property P23843_1 = {S50, 0, &V23843_1_1, NULL};	/*Property = IsMobile*/
const Value V23843_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23843_5 = {S52, 0, &V23843_5_1403, &P23843_1};	/*Property = ScreenPixelsHeight*/
const Value V23843_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23843_4 = {S54, 0, &V23843_4_1404, &P23843_5};	/*Property = ScreenPixelsWidth*/
const Profile P23843 = {"23843", &P23843_4};	/*Profile ID - 23843*/
const Value V22826_1_1 = {S2, NULL};	/*Value = True*/
const Property P22826_1 = {S50, 0, &V22826_1_1, NULL};	/*Property = IsMobile*/
const Value V22826_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22826_5 = {S52, 0, &V22826_5_1222, &P22826_1};	/*Property = ScreenPixelsHeight*/
const Value V22826_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22826_4 = {S54, 0, &V22826_4_1224, &P22826_5};	/*Property = ScreenPixelsWidth*/
const Profile P22826 = {"22826", &P22826_4};	/*Profile ID - 22826*/
const Value V22808_1_1 = {S2, NULL};	/*Value = True*/
const Property P22808_1 = {S50, 0, &V22808_1_1, NULL};	/*Property = IsMobile*/
const Value V22808_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22808_5 = {S52, 0, &V22808_5_1277, &P22808_1};	/*Property = ScreenPixelsHeight*/
const Value V22808_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22808_4 = {S54, 0, &V22808_4_1279, &P22808_5};	/*Property = ScreenPixelsWidth*/
const Profile P22808 = {"22808", &P22808_4};	/*Profile ID - 22808*/
const Value V22906_1_1 = {S2, NULL};	/*Value = True*/
const Property P22906_1 = {S50, 0, &V22906_1_1, NULL};	/*Property = IsMobile*/
const Value V22906_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22906_5 = {S52, 0, &V22906_5_1227, &P22906_1};	/*Property = ScreenPixelsHeight*/
const Value V22906_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22906_4 = {S54, 0, &V22906_4_1229, &P22906_5};	/*Property = ScreenPixelsWidth*/
const Profile P22906 = {"22906", &P22906_4};	/*Profile ID - 22906*/
const Value V23230_1_1 = {S2, NULL};	/*Value = True*/
const Property P23230_1 = {S50, 0, &V23230_1_1, NULL};	/*Property = IsMobile*/
const Value V23230_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23230_5 = {S52, 0, &V23230_5_37, &P23230_1};	/*Property = ScreenPixelsHeight*/
const Value V23230_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23230_4 = {S54, 0, &V23230_4_36, &P23230_5};	/*Property = ScreenPixelsWidth*/
const Profile P23230 = {"23230", &P23230_4};	/*Profile ID - 23230*/
const Value V23277_1_1 = {S2, NULL};	/*Value = True*/
const Property P23277_1 = {S50, 0, &V23277_1_1, NULL};	/*Property = IsMobile*/
const Value V23277_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23277_5 = {S52, 0, &V23277_5_1277, &P23277_1};	/*Property = ScreenPixelsHeight*/
const Value V23277_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23277_4 = {S54, 0, &V23277_4_1279, &P23277_5};	/*Property = ScreenPixelsWidth*/
const Profile P23277 = {"23277", &P23277_4};	/*Profile ID - 23277*/
const Value V22959_1_1 = {S2, NULL};	/*Value = True*/
const Property P22959_1 = {S50, 0, &V22959_1_1, NULL};	/*Property = IsMobile*/
const Value V22959_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22959_5 = {S52, 0, &V22959_5_1227, &P22959_1};	/*Property = ScreenPixelsHeight*/
const Value V22959_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22959_4 = {S54, 0, &V22959_4_1229, &P22959_5};	/*Property = ScreenPixelsWidth*/
const Profile P22959 = {"22959", &P22959_4};	/*Profile ID - 22959*/
const Value V22751_1_1 = {S2, NULL};	/*Value = True*/
const Property P22751_1 = {S50, 0, &V22751_1_1, NULL};	/*Property = IsMobile*/
const Value V22751_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22751_5 = {S52, 0, &V22751_5_1277, &P22751_1};	/*Property = ScreenPixelsHeight*/
const Value V22751_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22751_4 = {S54, 0, &V22751_4_1279, &P22751_5};	/*Property = ScreenPixelsWidth*/
const Profile P22751 = {"22751", &P22751_4};	/*Profile ID - 22751*/
const Value V22915_1_1 = {S2, NULL};	/*Value = True*/
const Property P22915_1 = {S50, 0, &V22915_1_1, NULL};	/*Property = IsMobile*/
const Value V22915_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22915_5 = {S52, 0, &V22915_5_1277, &P22915_1};	/*Property = ScreenPixelsHeight*/
const Value V22915_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22915_4 = {S54, 0, &V22915_4_1279, &P22915_5};	/*Property = ScreenPixelsWidth*/
const Profile P22915 = {"22915", &P22915_4};	/*Profile ID - 22915*/
const Value V24733_1_1 = {S2, NULL};	/*Value = True*/
const Property P24733_1 = {S50, 0, &V24733_1_1, NULL};	/*Property = IsMobile*/
const Value V24733_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24733_5 = {S52, 0, &V24733_5_1227, &P24733_1};	/*Property = ScreenPixelsHeight*/
const Value V24733_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24733_4 = {S54, 0, &V24733_4_1229, &P24733_5};	/*Property = ScreenPixelsWidth*/
const Profile P24733 = {"24733", &P24733_4};	/*Profile ID - 24733*/
const Value V23256_1_1 = {S2, NULL};	/*Value = True*/
const Property P23256_1 = {S50, 0, &V23256_1_1, NULL};	/*Property = IsMobile*/
const Value V23256_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23256_5 = {S52, 0, &V23256_5_9981, &P23256_1};	/*Property = ScreenPixelsHeight*/
const Value V23256_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23256_4 = {S54, 0, &V23256_4_39098, &P23256_5};	/*Property = ScreenPixelsWidth*/
const Profile P23256 = {"23256", &P23256_4};	/*Profile ID - 23256*/
const Value V22979_1_1 = {S2, NULL};	/*Value = True*/
const Property P22979_1 = {S50, 0, &V22979_1_1, NULL};	/*Property = IsMobile*/
const Value V22979_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22979_5 = {S52, 0, &V22979_5_1227, &P22979_1};	/*Property = ScreenPixelsHeight*/
const Value V22979_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22979_4 = {S54, 0, &V22979_4_1229, &P22979_5};	/*Property = ScreenPixelsWidth*/
const Profile P22979 = {"22979", &P22979_4};	/*Profile ID - 22979*/
const Value V23220_1_1 = {S2, NULL};	/*Value = True*/
const Property P23220_1 = {S50, 0, &V23220_1_1, NULL};	/*Property = IsMobile*/
const Value V23220_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23220_5 = {S52, 0, &V23220_5_37569, &P23220_1};	/*Property = ScreenPixelsHeight*/
const Value V23220_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23220_4 = {S54, 0, &V23220_4_5645, &P23220_5};	/*Property = ScreenPixelsWidth*/
const Profile P23220 = {"23220", &P23220_4};	/*Profile ID - 23220*/
const Value V22807_1_1 = {S2, NULL};	/*Value = True*/
const Property P22807_1 = {S50, 0, &V22807_1_1, NULL};	/*Property = IsMobile*/
const Value V22807_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22807_5 = {S52, 0, &V22807_5_1277, &P22807_1};	/*Property = ScreenPixelsHeight*/
const Value V22807_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22807_4 = {S54, 0, &V22807_4_1279, &P22807_5};	/*Property = ScreenPixelsWidth*/
const Profile P22807 = {"22807", &P22807_4};	/*Profile ID - 22807*/
const Value V23222_1_1 = {S2, NULL};	/*Value = True*/
const Property P23222_1 = {S50, 0, &V23222_1_1, NULL};	/*Property = IsMobile*/
const Value V23222_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23222_5 = {S52, 0, &V23222_5_1277, &P23222_1};	/*Property = ScreenPixelsHeight*/
const Value V23222_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23222_4 = {S54, 0, &V23222_4_2336, &P23222_5};	/*Property = ScreenPixelsWidth*/
const Profile P23222 = {"23222", &P23222_4};	/*Profile ID - 23222*/
const Value V24445_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24445_15 = {S37, 0, &V24445_15_28, NULL};	/*Property = LayoutEngine*/
const Profile P24445 = {"24445", &P24445_15};	/*Profile ID - 24445*/
const Value V20711_1_1 = {S2, NULL};	/*Value = True*/
const Property P20711_1 = {S50, 0, &V20711_1_1, NULL};	/*Property = IsMobile*/
const Value V20711_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P20711_5 = {S52, 0, &V20711_5_1458, &P20711_1};	/*Property = ScreenPixelsHeight*/
const Value V20711_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20711_4 = {S54, 0, &V20711_4_1460, &P20711_5};	/*Property = ScreenPixelsWidth*/
const Profile P20711 = {"20711", &P20711_4};	/*Profile ID - 20711*/
const Value V24542_1_1 = {S2, NULL};	/*Value = True*/
const Property P24542_1 = {S50, 0, &V24542_1_1, NULL};	/*Property = IsMobile*/
const Value V24542_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24542_5 = {S52, 0, &V24542_5_1227, &P24542_1};	/*Property = ScreenPixelsHeight*/
const Value V24542_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24542_4 = {S54, 0, &V24542_4_1229, &P24542_5};	/*Property = ScreenPixelsWidth*/
const Profile P24542 = {"24542", &P24542_4};	/*Profile ID - 24542*/
const Value V23127_1_1 = {S2, NULL};	/*Value = True*/
const Property P23127_1 = {S50, 0, &V23127_1_1, NULL};	/*Property = IsMobile*/
const Value V23127_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23127_5 = {S52, 0, &V23127_5_1227, &P23127_1};	/*Property = ScreenPixelsHeight*/
const Value V23127_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23127_4 = {S54, 0, &V23127_4_1229, &P23127_5};	/*Property = ScreenPixelsWidth*/
const Profile P23127 = {"23127", &P23127_4};	/*Profile ID - 23127*/
const Value V23832_1_1 = {S2, NULL};	/*Value = True*/
const Property P23832_1 = {S50, 0, &V23832_1_1, NULL};	/*Property = IsMobile*/
const Value V23832_5_87473 = {S150, NULL};	/*Value = 752*/
const Property P23832_5 = {S52, 0, &V23832_5_87473, &P23832_1};	/*Property = ScreenPixelsHeight*/
const Value V23832_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23832_4 = {S54, 0, &V23832_4_1229, &P23832_5};	/*Property = ScreenPixelsWidth*/
const Profile P23832 = {"23832", &P23832_4};	/*Profile ID - 23832*/
const Value V23828_1_1 = {S2, NULL};	/*Value = True*/
const Property P23828_1 = {S50, 0, &V23828_1_1, NULL};	/*Property = IsMobile*/
const Value V23828_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23828_5 = {S52, 0, &V23828_5_1227, &P23828_1};	/*Property = ScreenPixelsHeight*/
const Value V23828_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23828_4 = {S54, 0, &V23828_4_1229, &P23828_5};	/*Property = ScreenPixelsWidth*/
const Profile P23828 = {"23828", &P23828_4};	/*Profile ID - 23828*/
const Value V24571_1_1 = {S2, NULL};	/*Value = True*/
const Property P24571_1 = {S50, 0, &V24571_1_1, NULL};	/*Property = IsMobile*/
const Value V24571_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P24571_5 = {S52, 0, &V24571_5_1832, &P24571_1};	/*Property = ScreenPixelsHeight*/
const Value V24571_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P24571_4 = {S54, 0, &V24571_4_5645, &P24571_5};	/*Property = ScreenPixelsWidth*/
const Profile P24571 = {"24571", &P24571_4};	/*Profile ID - 24571*/
const Value V24330_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_279 = {S1, 0, &V24330_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24330_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_282 = {S3, 0, &V24330_282_57155, &P24330_279};	/*Property = BlobBuilder*/
const Value V24330_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_350 = {S4, 0, &V24330_350_57446, &P24330_282};	/*Property = Canvas*/
const Value V24330_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_283 = {S5, 0, &V24330_283_57161, &P24330_350};	/*Property = CssBackground*/
const Value V24330_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_284 = {S6, 0, &V24330_284_57164, &P24330_283};	/*Property = CssBorderImage*/
const Value V24330_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_285 = {S7, 0, &V24330_285_57167, &P24330_284};	/*Property = CssCanvas*/
const Value V24330_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_286 = {S8, 0, &V24330_286_57170, &P24330_285};	/*Property = CssColor*/
const Value V24330_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_237 = {S9, 0, &V24330_237_55786, &P24330_286};	/*Property = CssColumn*/
const Value V24330_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_287 = {S10, 0, &V24330_287_57173, &P24330_237};	/*Property = CssFlexbox*/
const Value V24330_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_288 = {S11, 0, &V24330_288_57176, &P24330_287};	/*Property = CssFont*/
const Value V24330_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_289 = {S12, 0, &V24330_289_86899, &P24330_288};	/*Property = CssImages*/
const Value V24330_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_291 = {S13, 0, &V24330_291_57182, &P24330_289};	/*Property = CssMediaQueries*/
const Value V24330_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_292 = {S14, 0, &V24330_292_57185, &P24330_291};	/*Property = CssMinMax*/
const Value V24330_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_293 = {S15, 0, &V24330_293_57188, &P24330_292};	/*Property = CssOverflow*/
const Value V24330_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_294 = {S16, 0, &V24330_294_57191, &P24330_293};	/*Property = CssPosition*/
const Value V24330_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_295 = {S17, 0, &V24330_295_57194, &P24330_294};	/*Property = CssText*/
const Value V24330_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_297 = {S18, 0, &V24330_297_57197, &P24330_295};	/*Property = CssTransforms*/
const Value V24330_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_241 = {S19, 0, &V24330_241_55801, &P24330_297};	/*Property = CssTransitions*/
const Value V24330_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_298 = {S20, 0, &V24330_298_57208, &P24330_241};	/*Property = CssUI*/
const Value V24330_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_300 = {S21, 0, &V24330_300_57217, &P24330_298};	/*Property = DataSet*/
const Value V24330_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_301 = {S22, 0, &V24330_301_57220, &P24330_300};	/*Property = DataUrl*/
const Value V24330_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_302 = {S23, 0, &V24330_302_57223, &P24330_301};	/*Property = DeviceOrientation*/
const Value V24330_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_304 = {S24, 0, &V24330_304_57229, &P24330_302};	/*Property = FileReader*/
const Value V24330_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_305 = {S25, 0, &V24330_305_57232, &P24330_304};	/*Property = FileSaver*/
const Value V24330_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_306 = {S26, 0, &V24330_306_57238, &P24330_305};	/*Property = FileWriter*/
const Value V24330_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_307 = {S27, 0, &V24330_307_57241, &P24330_306};	/*Property = FormData*/
const Value V24330_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_309 = {S28, 0, &V24330_309_57247, &P24330_307};	/*Property = Fullscreen*/
const Value V24330_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_310 = {S29, 0, &V24330_310_57250, &P24330_309};	/*Property = GeoLocation*/
const Value V24330_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_312 = {S30, 0, &V24330_312_57256, &P24330_310};	/*Property = History*/
const Value V24330_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_314 = {S31, 0, &V24330_314_57262, &P24330_312};	/*Property = Html5*/
const Value V24330_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_313 = {S32, 0, &V24330_313_57259, &P24330_314};	/*Property = Html-Media-Capture*/
const Value V24330_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_315 = {S33, 0, &V24330_315_57265, &P24330_313};	/*Property = Iframe*/
const Value V24330_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_316 = {S34, 0, &V24330_316_57268, &P24330_315};	/*Property = IndexedDB*/
const Value V24330_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_317 = {S35, 0, &V24330_317_57271, &P24330_316};	/*Property = Json*/
const Value V24330_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P24330_15 = {S37, 0, &V24330_15_29, &P24330_317};	/*Property = LayoutEngine*/
const Value V24330_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_344 = {S38, 0, &V24330_344_57274, &P24330_15};	/*Property = Masking*/
const Value V24330_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_320 = {S39, 0, &V24330_320_57292, &P24330_344};	/*Property = PostMessage*/
const Value V24330_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_321 = {S40, 0, &V24330_321_57295, &P24330_320};	/*Property = Progress*/
const Value V24330_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_322 = {S41, 0, &V24330_322_57298, &P24330_321};	/*Property = Prompts*/
const Value V24330_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_323 = {S42, 0, &V24330_323_57301, &P24330_322};	/*Property = Selector*/
const Value V24330_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_324 = {S43, 0, &V24330_324_57307, &P24330_323};	/*Property = Svg*/
const Value V24330_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_327 = {S44, 0, &V24330_327_57316, &P24330_324};	/*Property = TouchEvents*/
const Value V24330_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_328 = {S45, 0, &V24330_328_57319, &P24330_327};	/*Property = Track*/
const Value V24330_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_329 = {S46, 0, &V24330_329_57325, &P24330_328};	/*Property = Video*/
const Value V24330_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_330 = {S47, 0, &V24330_330_57328, &P24330_329};	/*Property = Viewport*/
const Value V24330_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_332 = {S48, 0, &V24330_332_57340, &P24330_330};	/*Property = WebWorkers*/
const Value V24330_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24330_333 = {S49, 0, &V24330_333_57343, &P24330_332};	/*Property = Xhr2*/
const Profile P24330 = {"24330", &P24330_333};	/*Profile ID - 24330*/
const Value V23003_1_1 = {S2, NULL};	/*Value = True*/
const Property P23003_1 = {S50, 0, &V23003_1_1, NULL};	/*Property = IsMobile*/
const Value V23003_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23003_5 = {S52, 0, &V23003_5_1277, &P23003_1};	/*Property = ScreenPixelsHeight*/
const Value V23003_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23003_4 = {S54, 0, &V23003_4_1279, &P23003_5};	/*Property = ScreenPixelsWidth*/
const Profile P23003 = {"23003", &P23003_4};	/*Profile ID - 23003*/
const Value V23103_1_1 = {S2, NULL};	/*Value = True*/
const Property P23103_1 = {S50, 0, &V23103_1_1, NULL};	/*Property = IsMobile*/
const Value V23103_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23103_5 = {S52, 0, &V23103_5_9981, &P23103_1};	/*Property = ScreenPixelsHeight*/
const Value V23103_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23103_4 = {S54, 0, &V23103_4_39098, &P23103_5};	/*Property = ScreenPixelsWidth*/
const Profile P23103 = {"23103", &P23103_4};	/*Profile ID - 23103*/
const Value V23420_1_1 = {S2, NULL};	/*Value = True*/
const Property P23420_1 = {S50, 0, &V23420_1_1, NULL};	/*Property = IsMobile*/
const Value V23420_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23420_5 = {S52, 0, &V23420_5_1403, &P23420_1};	/*Property = ScreenPixelsHeight*/
const Value V23420_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23420_4 = {S54, 0, &V23420_4_1404, &P23420_5};	/*Property = ScreenPixelsWidth*/
const Profile P23420 = {"23420", &P23420_4};	/*Profile ID - 23420*/
const Value V23840_1_1 = {S2, NULL};	/*Value = True*/
const Property P23840_1 = {S50, 0, &V23840_1_1, NULL};	/*Property = IsMobile*/
const Value V23840_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23840_5 = {S52, 0, &V23840_5_1277, &P23840_1};	/*Property = ScreenPixelsHeight*/
const Value V23840_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23840_4 = {S54, 0, &V23840_4_1279, &P23840_5};	/*Property = ScreenPixelsWidth*/
const Profile P23840 = {"23840", &P23840_4};	/*Profile ID - 23840*/
const Value V23704_1_1 = {S2, NULL};	/*Value = True*/
const Property P23704_1 = {S50, 0, &V23704_1_1, NULL};	/*Property = IsMobile*/
const Value V23704_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23704_5 = {S52, 0, &V23704_5_9981, &P23704_1};	/*Property = ScreenPixelsHeight*/
const Value V23704_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23704_4 = {S54, 0, &V23704_4_2336, &P23704_5};	/*Property = ScreenPixelsWidth*/
const Profile P23704 = {"23704", &P23704_4};	/*Profile ID - 23704*/
const Value V22806_1_1 = {S2, NULL};	/*Value = True*/
const Property P22806_1 = {S50, 0, &V22806_1_1, NULL};	/*Property = IsMobile*/
const Value V22806_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22806_5 = {S52, 0, &V22806_5_1227, &P22806_1};	/*Property = ScreenPixelsHeight*/
const Value V22806_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22806_4 = {S54, 0, &V22806_4_1229, &P22806_5};	/*Property = ScreenPixelsWidth*/
const Profile P22806 = {"22806", &P22806_4};	/*Profile ID - 22806*/
const Value V22804_1_1 = {S2, NULL};	/*Value = True*/
const Property P22804_1 = {S50, 0, &V22804_1_1, NULL};	/*Property = IsMobile*/
const Value V22804_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22804_5 = {S52, 0, &V22804_5_1227, &P22804_1};	/*Property = ScreenPixelsHeight*/
const Value V22804_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22804_4 = {S54, 0, &V22804_4_1229, &P22804_5};	/*Property = ScreenPixelsWidth*/
const Profile P22804 = {"22804", &P22804_4};	/*Profile ID - 22804*/
const Value V23110_1_1 = {S2, NULL};	/*Value = True*/
const Property P23110_1 = {S50, 0, &V23110_1_1, NULL};	/*Property = IsMobile*/
const Value V23110_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23110_5 = {S52, 0, &V23110_5_1277, &P23110_1};	/*Property = ScreenPixelsHeight*/
const Value V23110_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23110_4 = {S54, 0, &V23110_4_1279, &P23110_5};	/*Property = ScreenPixelsWidth*/
const Profile P23110 = {"23110", &P23110_4};	/*Profile ID - 23110*/
const Value V20872_1_1 = {S2, NULL};	/*Value = True*/
const Property P20872_1 = {S50, 0, &V20872_1_1, NULL};	/*Property = IsMobile*/
const Value V20872_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P20872_5 = {S52, 0, &V20872_5_59138, &P20872_1};	/*Property = ScreenPixelsHeight*/
const Value V20872_4_52971 = {S122, NULL};	/*Value = 1200*/
const Property P20872_4 = {S54, 0, &V20872_4_52971, &P20872_5};	/*Property = ScreenPixelsWidth*/
const Profile P20872 = {"20872", &P20872_4};	/*Profile ID - 20872*/
const Value V23169_1_1 = {S2, NULL};	/*Value = True*/
const Property P23169_1 = {S50, 0, &V23169_1_1, NULL};	/*Property = IsMobile*/
const Value V23169_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23169_5 = {S52, 0, &V23169_5_1277, &P23169_1};	/*Property = ScreenPixelsHeight*/
const Value V23169_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23169_4 = {S54, 0, &V23169_4_1279, &P23169_5};	/*Property = ScreenPixelsWidth*/
const Profile P23169 = {"23169", &P23169_4};	/*Profile ID - 23169*/
const Value V23739_1_1 = {S2, NULL};	/*Value = True*/
const Property P23739_1 = {S50, 0, &V23739_1_1, NULL};	/*Property = IsMobile*/
const Value V23739_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23739_5 = {S52, 0, &V23739_5_1227, &P23739_1};	/*Property = ScreenPixelsHeight*/
const Value V23739_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23739_4 = {S54, 0, &V23739_4_1229, &P23739_5};	/*Property = ScreenPixelsWidth*/
const Profile P23739 = {"23739", &P23739_4};	/*Profile ID - 23739*/
const Value V23011_1_1 = {S2, NULL};	/*Value = True*/
const Property P23011_1 = {S50, 0, &V23011_1_1, NULL};	/*Property = IsMobile*/
const Value V23011_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23011_5 = {S52, 0, &V23011_5_1227, &P23011_1};	/*Property = ScreenPixelsHeight*/
const Value V23011_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23011_4 = {S54, 0, &V23011_4_1229, &P23011_5};	/*Property = ScreenPixelsWidth*/
const Profile P23011 = {"23011", &P23011_4};	/*Profile ID - 23011*/
const Value V22752_1_1 = {S2, NULL};	/*Value = True*/
const Property P22752_1 = {S50, 0, &V22752_1_1, NULL};	/*Property = IsMobile*/
const Value V22752_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22752_5 = {S52, 0, &V22752_5_1227, &P22752_1};	/*Property = ScreenPixelsHeight*/
const Value V22752_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22752_4 = {S54, 0, &V22752_4_1229, &P22752_5};	/*Property = ScreenPixelsWidth*/
const Profile P22752 = {"22752", &P22752_4};	/*Profile ID - 22752*/
const Value V23477_1_1 = {S2, NULL};	/*Value = True*/
const Property P23477_1 = {S50, 0, &V23477_1_1, NULL};	/*Property = IsMobile*/
const Value V23477_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23477_5 = {S52, 0, &V23477_5_9981, &P23477_1};	/*Property = ScreenPixelsHeight*/
const Value V23477_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23477_4 = {S54, 0, &V23477_4_39098, &P23477_5};	/*Property = ScreenPixelsWidth*/
const Profile P23477 = {"23477", &P23477_4};	/*Profile ID - 23477*/
const Value V23997_279_57147 = {S2, NULL};	/*Value = True*/
const Property P23997_279 = {S1, 0, &V23997_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V23997_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23997_282 = {S3, 0, &V23997_282_57154, &P23997_279};	/*Property = BlobBuilder*/
const Value V23997_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23997_350 = {S4, 0, &V23997_350_57444, &P23997_282};	/*Property = Canvas*/
const Value V23997_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23997_283 = {S5, 0, &V23997_283_57159, &P23997_350};	/*Property = CssBackground*/
const Value V23997_284_57162 = {S2, NULL};	/*Value = True*/
const Property P23997_284 = {S6, 0, &V23997_284_57162, &P23997_283};	/*Property = CssBorderImage*/
const Value V23997_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23997_285 = {S7, 0, &V23997_285_57166, &P23997_284};	/*Property = CssCanvas*/
const Value V23997_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23997_286 = {S8, 0, &V23997_286_57168, &P23997_285};	/*Property = CssColor*/
const Value V23997_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23997_237 = {S9, 0, &V23997_237_55786, &P23997_286};	/*Property = CssColumn*/
const Value V23997_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23997_287 = {S10, 0, &V23997_287_57172, &P23997_237};	/*Property = CssFlexbox*/
const Value V23997_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23997_288 = {S11, 0, &V23997_288_57174, &P23997_287};	/*Property = CssFont*/
const Value V23997_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23997_289 = {S12, 0, &V23997_289_57759, &P23997_288};	/*Property = CssImages*/
const Value V23997_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23997_291 = {S13, 0, &V23997_291_57180, &P23997_289};	/*Property = CssMediaQueries*/
const Value V23997_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23997_292 = {S14, 0, &V23997_292_57183, &P23997_291};	/*Property = CssMinMax*/
const Value V23997_293_57186 = {S2, NULL};	/*Value = True*/
const Property P23997_293 = {S15, 0, &V23997_293_57186, &P23997_292};	/*Property = CssOverflow*/
const Value V23997_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23997_294 = {S16, 0, &V23997_294_57189, &P23997_293};	/*Property = CssPosition*/
const Value V23997_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23997_295 = {S17, 0, &V23997_295_57192, &P23997_294};	/*Property = CssText*/
const Value V23997_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23997_297 = {S18, 0, &V23997_297_57195, &P23997_295};	/*Property = CssTransforms*/
const Value V23997_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23997_241 = {S19, 0, &V23997_241_55799, &P23997_297};	/*Property = CssTransitions*/
const Value V23997_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23997_298 = {S20, 0, &V23997_298_57206, &P23997_241};	/*Property = CssUI*/
const Value V23997_300_57215 = {S2, NULL};	/*Value = True*/
const Property P23997_300 = {S21, 0, &V23997_300_57215, &P23997_298};	/*Property = DataSet*/
const Value V23997_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23997_301 = {S22, 0, &V23997_301_57218, &P23997_300};	/*Property = DataUrl*/
const Value V23997_302_57221 = {S2, NULL};	/*Value = True*/
const Property P23997_302 = {S23, 0, &V23997_302_57221, &P23997_301};	/*Property = DeviceOrientation*/
const Value V23997_304_57227 = {S2, NULL};	/*Value = True*/
const Property P23997_304 = {S24, 0, &V23997_304_57227, &P23997_302};	/*Property = FileReader*/
const Value V23997_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23997_305 = {S25, 0, &V23997_305_57231, &P23997_304};	/*Property = FileSaver*/
const Value V23997_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23997_306 = {S26, 0, &V23997_306_57237, &P23997_305};	/*Property = FileWriter*/
const Value V23997_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23997_307 = {S27, 0, &V23997_307_57239, &P23997_306};	/*Property = FormData*/
const Value V23997_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23997_309 = {S28, 0, &V23997_309_57246, &P23997_307};	/*Property = Fullscreen*/
const Value V23997_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23997_310 = {S29, 0, &V23997_310_57248, &P23997_309};	/*Property = GeoLocation*/
const Value V23997_312_57254 = {S2, NULL};	/*Value = True*/
const Property P23997_312 = {S30, 0, &V23997_312_57254, &P23997_310};	/*Property = History*/
const Value V23997_314_57260 = {S2, NULL};	/*Value = True*/
const Property P23997_314 = {S31, 0, &V23997_314_57260, &P23997_312};	/*Property = Html5*/
const Value V23997_313_57257 = {S2, NULL};	/*Value = True*/
const Property P23997_313 = {S32, 0, &V23997_313_57257, &P23997_314};	/*Property = Html-Media-Capture*/
const Value V23997_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23997_315 = {S33, 0, &V23997_315_57264, &P23997_313};	/*Property = Iframe*/
const Value V23997_316_57266 = {S2, NULL};	/*Value = True*/
const Property P23997_316 = {S34, 0, &V23997_316_57266, &P23997_315};	/*Property = IndexedDB*/
const Value V23997_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23997_317 = {S35, 0, &V23997_317_57269, &P23997_316};	/*Property = Json*/
const Value V23997_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P23997_15 = {S37, 0, &V23997_15_28, &P23997_317};	/*Property = LayoutEngine*/
const Value V23997_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23997_344 = {S38, 0, &V23997_344_57272, &P23997_15};	/*Property = Masking*/
const Value V23997_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23997_320 = {S39, 0, &V23997_320_57290, &P23997_344};	/*Property = PostMessage*/
const Value V23997_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23997_321 = {S40, 0, &V23997_321_57293, &P23997_320};	/*Property = Progress*/
const Value V23997_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23997_322 = {S41, 0, &V23997_322_57296, &P23997_321};	/*Property = Prompts*/
const Value V23997_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23997_323 = {S42, 0, &V23997_323_57299, &P23997_322};	/*Property = Selector*/
const Value V23997_324_57305 = {S2, NULL};	/*Value = True*/
const Property P23997_324 = {S43, 0, &V23997_324_57305, &P23997_323};	/*Property = Svg*/
const Value V23997_327_57314 = {S2, NULL};	/*Value = True*/
const Property P23997_327 = {S44, 0, &V23997_327_57314, &P23997_324};	/*Property = TouchEvents*/
const Value V23997_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23997_328 = {S45, 0, &V23997_328_57318, &P23997_327};	/*Property = Track*/
const Value V23997_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23997_329 = {S46, 0, &V23997_329_57323, &P23997_328};	/*Property = Video*/
const Value V23997_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23997_330 = {S47, 0, &V23997_330_57326, &P23997_329};	/*Property = Viewport*/
const Value V23997_332_57338 = {S2, NULL};	/*Value = True*/
const Property P23997_332 = {S48, 0, &V23997_332_57338, &P23997_330};	/*Property = WebWorkers*/
const Value V23997_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23997_333 = {S49, 0, &V23997_333_57342, &P23997_332};	/*Property = Xhr2*/
const Profile P23997 = {"23997", &P23997_333};	/*Profile ID - 23997*/
const Value V22908_1_1 = {S2, NULL};	/*Value = True*/
const Property P22908_1 = {S50, 0, &V22908_1_1, NULL};	/*Property = IsMobile*/
const Value V22908_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22908_5 = {S52, 0, &V22908_5_1277, &P22908_1};	/*Property = ScreenPixelsHeight*/
const Value V22908_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22908_4 = {S54, 0, &V22908_4_1279, &P22908_5};	/*Property = ScreenPixelsWidth*/
const Profile P22908 = {"22908", &P22908_4};	/*Profile ID - 22908*/
const Value V22851_1_1 = {S2, NULL};	/*Value = True*/
const Property P22851_1 = {S50, 0, &V22851_1_1, NULL};	/*Property = IsMobile*/
const Value V22851_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22851_5 = {S52, 0, &V22851_5_1227, &P22851_1};	/*Property = ScreenPixelsHeight*/
const Value V22851_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22851_4 = {S54, 0, &V22851_4_1229, &P22851_5};	/*Property = ScreenPixelsWidth*/
const Profile P22851 = {"22851", &P22851_4};	/*Profile ID - 22851*/
const Value V22978_1_1 = {S2, NULL};	/*Value = True*/
const Property P22978_1 = {S50, 0, &V22978_1_1, NULL};	/*Property = IsMobile*/
const Value V22978_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22978_5 = {S52, 0, &V22978_5_1277, &P22978_1};	/*Property = ScreenPixelsHeight*/
const Value V22978_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22978_4 = {S54, 0, &V22978_4_1279, &P22978_5};	/*Property = ScreenPixelsWidth*/
const Profile P22978 = {"22978", &P22978_4};	/*Profile ID - 22978*/
const Value V23833_1_1 = {S2, NULL};	/*Value = True*/
const Property P23833_1 = {S50, 0, &V23833_1_1, NULL};	/*Property = IsMobile*/
const Value V23833_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23833_5 = {S52, 0, &V23833_5_1277, &P23833_1};	/*Property = ScreenPixelsHeight*/
const Value V23833_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23833_4 = {S54, 0, &V23833_4_1279, &P23833_5};	/*Property = ScreenPixelsWidth*/
const Profile P23833 = {"23833", &P23833_4};	/*Profile ID - 23833*/
const Value V23167_1_1 = {S2, NULL};	/*Value = True*/
const Property P23167_1 = {S50, 0, &V23167_1_1, NULL};	/*Property = IsMobile*/
const Value V23167_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23167_5 = {S52, 0, &V23167_5_60417, &P23167_1};	/*Property = ScreenPixelsHeight*/
const Value V23167_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23167_4 = {S54, 0, &V23167_4_39098, &P23167_5};	/*Property = ScreenPixelsWidth*/
const Profile P23167 = {"23167", &P23167_4};	/*Profile ID - 23167*/
const Value V22910_1_1 = {S2, NULL};	/*Value = True*/
const Property P22910_1 = {S50, 0, &V22910_1_1, NULL};	/*Property = IsMobile*/
const Value V22910_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22910_5 = {S52, 0, &V22910_5_1277, &P22910_1};	/*Property = ScreenPixelsHeight*/
const Value V22910_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22910_4 = {S54, 0, &V22910_4_1279, &P22910_5};	/*Property = ScreenPixelsWidth*/
const Profile P22910 = {"22910", &P22910_4};	/*Profile ID - 22910*/
const Value V23155_1_2 = {S0, NULL};	/*Value = False*/
const Property P23155_1 = {S50, 0, &V23155_1_2, NULL};	/*Property = IsMobile*/
const Value V23155_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23155_5 = {S52, 0, &V23155_5_1227, &P23155_1};	/*Property = ScreenPixelsHeight*/
const Value V23155_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P23155_4 = {S54, 0, &V23155_4_1477, &P23155_5};	/*Property = ScreenPixelsWidth*/
const Profile P23155 = {"23155", &P23155_4};	/*Profile ID - 23155*/
const Value V23017_1_1 = {S2, NULL};	/*Value = True*/
const Property P23017_1 = {S50, 0, &V23017_1_1, NULL};	/*Property = IsMobile*/
const Value V23017_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23017_5 = {S52, 0, &V23017_5_1227, &P23017_1};	/*Property = ScreenPixelsHeight*/
const Value V23017_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23017_4 = {S54, 0, &V23017_4_1229, &P23017_5};	/*Property = ScreenPixelsWidth*/
const Profile P23017 = {"23017", &P23017_4};	/*Profile ID - 23017*/
const Value V22709_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22709_279 = {S1, 0, &V22709_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22709_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22709_282 = {S3, 0, &V22709_282_57154, &P22709_279};	/*Property = BlobBuilder*/
const Value V22709_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22709_350 = {S4, 0, &V22709_350_57444, &P22709_282};	/*Property = Canvas*/
const Value V22709_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22709_283 = {S5, 0, &V22709_283_57159, &P22709_350};	/*Property = CssBackground*/
const Value V22709_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22709_284 = {S6, 0, &V22709_284_57162, &P22709_283};	/*Property = CssBorderImage*/
const Value V22709_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22709_285 = {S7, 0, &V22709_285_57166, &P22709_284};	/*Property = CssCanvas*/
const Value V22709_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22709_286 = {S8, 0, &V22709_286_57168, &P22709_285};	/*Property = CssColor*/
const Value V22709_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22709_237 = {S9, 0, &V22709_237_55786, &P22709_286};	/*Property = CssColumn*/
const Value V22709_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22709_287 = {S10, 0, &V22709_287_57172, &P22709_237};	/*Property = CssFlexbox*/
const Value V22709_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22709_288 = {S11, 0, &V22709_288_57174, &P22709_287};	/*Property = CssFont*/
const Value V22709_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22709_289 = {S12, 0, &V22709_289_57759, &P22709_288};	/*Property = CssImages*/
const Value V22709_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22709_291 = {S13, 0, &V22709_291_57180, &P22709_289};	/*Property = CssMediaQueries*/
const Value V22709_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22709_292 = {S14, 0, &V22709_292_57183, &P22709_291};	/*Property = CssMinMax*/
const Value V22709_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22709_293 = {S15, 0, &V22709_293_57187, &P22709_292};	/*Property = CssOverflow*/
const Value V22709_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22709_294 = {S16, 0, &V22709_294_57189, &P22709_293};	/*Property = CssPosition*/
const Value V22709_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22709_295 = {S17, 0, &V22709_295_57192, &P22709_294};	/*Property = CssText*/
const Value V22709_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22709_297 = {S18, 0, &V22709_297_57195, &P22709_295};	/*Property = CssTransforms*/
const Value V22709_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22709_241 = {S19, 0, &V22709_241_55799, &P22709_297};	/*Property = CssTransitions*/
const Value V22709_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22709_298 = {S20, 0, &V22709_298_57206, &P22709_241};	/*Property = CssUI*/
const Value V22709_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22709_300 = {S21, 0, &V22709_300_57215, &P22709_298};	/*Property = DataSet*/
const Value V22709_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22709_301 = {S22, 0, &V22709_301_57218, &P22709_300};	/*Property = DataUrl*/
const Value V22709_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22709_302 = {S23, 0, &V22709_302_57221, &P22709_301};	/*Property = DeviceOrientation*/
const Value V22709_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22709_304 = {S24, 0, &V22709_304_57227, &P22709_302};	/*Property = FileReader*/
const Value V22709_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22709_305 = {S25, 0, &V22709_305_57231, &P22709_304};	/*Property = FileSaver*/
const Value V22709_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22709_306 = {S26, 0, &V22709_306_57237, &P22709_305};	/*Property = FileWriter*/
const Value V22709_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22709_307 = {S27, 0, &V22709_307_57239, &P22709_306};	/*Property = FormData*/
const Value V22709_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22709_309 = {S28, 0, &V22709_309_57246, &P22709_307};	/*Property = Fullscreen*/
const Value V22709_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22709_310 = {S29, 0, &V22709_310_57248, &P22709_309};	/*Property = GeoLocation*/
const Value V22709_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22709_312 = {S30, 0, &V22709_312_57254, &P22709_310};	/*Property = History*/
const Value V22709_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22709_314 = {S31, 0, &V22709_314_57260, &P22709_312};	/*Property = Html5*/
const Value V22709_313_57257 = {S2, NULL};	/*Value = True*/
const Property P22709_313 = {S32, 0, &V22709_313_57257, &P22709_314};	/*Property = Html-Media-Capture*/
const Value V22709_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22709_315 = {S33, 0, &V22709_315_57264, &P22709_313};	/*Property = Iframe*/
const Value V22709_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22709_316 = {S34, 0, &V22709_316_57266, &P22709_315};	/*Property = IndexedDB*/
const Value V22709_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22709_317 = {S35, 0, &V22709_317_57269, &P22709_316};	/*Property = Json*/
const Value V22709_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P22709_15 = {S37, 0, &V22709_15_54167, &P22709_317};	/*Property = LayoutEngine*/
const Value V22709_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22709_344 = {S38, 0, &V22709_344_57272, &P22709_15};	/*Property = Masking*/
const Value V22709_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22709_320 = {S39, 0, &V22709_320_57290, &P22709_344};	/*Property = PostMessage*/
const Value V22709_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22709_321 = {S40, 0, &V22709_321_57293, &P22709_320};	/*Property = Progress*/
const Value V22709_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22709_322 = {S41, 0, &V22709_322_57296, &P22709_321};	/*Property = Prompts*/
const Value V22709_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22709_323 = {S42, 0, &V22709_323_57299, &P22709_322};	/*Property = Selector*/
const Value V22709_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22709_324 = {S43, 0, &V22709_324_57306, &P22709_323};	/*Property = Svg*/
const Value V22709_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22709_327 = {S44, 0, &V22709_327_57314, &P22709_324};	/*Property = TouchEvents*/
const Value V22709_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22709_328 = {S45, 0, &V22709_328_57318, &P22709_327};	/*Property = Track*/
const Value V22709_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22709_329 = {S46, 0, &V22709_329_57323, &P22709_328};	/*Property = Video*/
const Value V22709_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22709_330 = {S47, 0, &V22709_330_57326, &P22709_329};	/*Property = Viewport*/
const Value V22709_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22709_332 = {S48, 0, &V22709_332_57339, &P22709_330};	/*Property = WebWorkers*/
const Value V22709_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22709_333 = {S49, 0, &V22709_333_57342, &P22709_332};	/*Property = Xhr2*/
const Profile P22709 = {"22709", &P22709_333};	/*Profile ID - 22709*/
const Value V22864_1_1 = {S2, NULL};	/*Value = True*/
const Property P22864_1 = {S50, 0, &V22864_1_1, NULL};	/*Property = IsMobile*/
const Value V22864_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22864_5 = {S52, 0, &V22864_5_1277, &P22864_1};	/*Property = ScreenPixelsHeight*/
const Value V22864_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22864_4 = {S54, 0, &V22864_4_1279, &P22864_5};	/*Property = ScreenPixelsWidth*/
const Profile P22864 = {"22864", &P22864_4};	/*Profile ID - 22864*/
const Value V22749_1_1 = {S2, NULL};	/*Value = True*/
const Property P22749_1 = {S50, 0, &V22749_1_1, NULL};	/*Property = IsMobile*/
const Value V22749_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22749_5 = {S52, 0, &V22749_5_9981, &P22749_1};	/*Property = ScreenPixelsHeight*/
const Value V22749_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22749_4 = {S54, 0, &V22749_4_39098, &P22749_5};	/*Property = ScreenPixelsWidth*/
const Profile P22749 = {"22749", &P22749_4};	/*Profile ID - 22749*/
const Value V23177_1_1 = {S2, NULL};	/*Value = True*/
const Property P23177_1 = {S50, 0, &V23177_1_1, NULL};	/*Property = IsMobile*/
const Value V23177_5_77419 = {S138, NULL};	/*Value = 728*/
const Property P23177_5 = {S52, 0, &V23177_5_77419, &P23177_1};	/*Property = ScreenPixelsHeight*/
const Value V23177_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23177_4 = {S54, 0, &V23177_4_5645, &P23177_5};	/*Property = ScreenPixelsWidth*/
const Profile P23177 = {"23177", &P23177_4};	/*Profile ID - 23177*/
const Value V23413_1_1 = {S2, NULL};	/*Value = True*/
const Property P23413_1 = {S50, 0, &V23413_1_1, NULL};	/*Property = IsMobile*/
const Value V23413_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23413_5 = {S52, 0, &V23413_5_1227, &P23413_1};	/*Property = ScreenPixelsHeight*/
const Value V23413_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23413_4 = {S54, 0, &V23413_4_1229, &P23413_5};	/*Property = ScreenPixelsWidth*/
const Profile P23413 = {"23413", &P23413_4};	/*Profile ID - 23413*/
const Value V22737_1_1 = {S2, NULL};	/*Value = True*/
const Property P22737_1 = {S50, 0, &V22737_1_1, NULL};	/*Property = IsMobile*/
const Value V22737_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22737_5 = {S52, 0, &V22737_5_1227, &P22737_1};	/*Property = ScreenPixelsHeight*/
const Value V22737_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22737_4 = {S54, 0, &V22737_4_1229, &P22737_5};	/*Property = ScreenPixelsWidth*/
const Profile P22737 = {"22737", &P22737_4};	/*Profile ID - 22737*/
const Value V23124_1_1 = {S2, NULL};	/*Value = True*/
const Property P23124_1 = {S50, 0, &V23124_1_1, NULL};	/*Property = IsMobile*/
const Value V23124_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23124_5 = {S52, 0, &V23124_5_9981, &P23124_1};	/*Property = ScreenPixelsHeight*/
const Value V23124_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23124_4 = {S54, 0, &V23124_4_39098, &P23124_5};	/*Property = ScreenPixelsWidth*/
const Profile P23124 = {"23124", &P23124_4};	/*Profile ID - 23124*/
const Value V22772_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22772_279 = {S1, 0, &V22772_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22772_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22772_282 = {S3, 0, &V22772_282_57154, &P22772_279};	/*Property = BlobBuilder*/
const Value V22772_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22772_350 = {S4, 0, &V22772_350_57444, &P22772_282};	/*Property = Canvas*/
const Value V22772_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22772_283 = {S5, 0, &V22772_283_57159, &P22772_350};	/*Property = CssBackground*/
const Value V22772_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22772_284 = {S6, 0, &V22772_284_57162, &P22772_283};	/*Property = CssBorderImage*/
const Value V22772_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22772_285 = {S7, 0, &V22772_285_57166, &P22772_284};	/*Property = CssCanvas*/
const Value V22772_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22772_286 = {S8, 0, &V22772_286_57168, &P22772_285};	/*Property = CssColor*/
const Value V22772_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22772_237 = {S9, 0, &V22772_237_55786, &P22772_286};	/*Property = CssColumn*/
const Value V22772_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22772_287 = {S10, 0, &V22772_287_57172, &P22772_237};	/*Property = CssFlexbox*/
const Value V22772_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22772_288 = {S11, 0, &V22772_288_57174, &P22772_287};	/*Property = CssFont*/
const Value V22772_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22772_289 = {S12, 0, &V22772_289_57759, &P22772_288};	/*Property = CssImages*/
const Value V22772_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22772_291 = {S13, 0, &V22772_291_57180, &P22772_289};	/*Property = CssMediaQueries*/
const Value V22772_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22772_292 = {S14, 0, &V22772_292_57183, &P22772_291};	/*Property = CssMinMax*/
const Value V22772_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22772_293 = {S15, 0, &V22772_293_57187, &P22772_292};	/*Property = CssOverflow*/
const Value V22772_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22772_294 = {S16, 0, &V22772_294_57189, &P22772_293};	/*Property = CssPosition*/
const Value V22772_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22772_295 = {S17, 0, &V22772_295_57192, &P22772_294};	/*Property = CssText*/
const Value V22772_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22772_297 = {S18, 0, &V22772_297_57195, &P22772_295};	/*Property = CssTransforms*/
const Value V22772_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22772_241 = {S19, 0, &V22772_241_55799, &P22772_297};	/*Property = CssTransitions*/
const Value V22772_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22772_298 = {S20, 0, &V22772_298_57206, &P22772_241};	/*Property = CssUI*/
const Value V22772_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22772_300 = {S21, 0, &V22772_300_57215, &P22772_298};	/*Property = DataSet*/
const Value V22772_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22772_301 = {S22, 0, &V22772_301_57218, &P22772_300};	/*Property = DataUrl*/
const Value V22772_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22772_302 = {S23, 0, &V22772_302_57221, &P22772_301};	/*Property = DeviceOrientation*/
const Value V22772_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22772_304 = {S24, 0, &V22772_304_57227, &P22772_302};	/*Property = FileReader*/
const Value V22772_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22772_305 = {S25, 0, &V22772_305_57231, &P22772_304};	/*Property = FileSaver*/
const Value V22772_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22772_306 = {S26, 0, &V22772_306_57237, &P22772_305};	/*Property = FileWriter*/
const Value V22772_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22772_307 = {S27, 0, &V22772_307_57239, &P22772_306};	/*Property = FormData*/
const Value V22772_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22772_309 = {S28, 0, &V22772_309_57246, &P22772_307};	/*Property = Fullscreen*/
const Value V22772_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22772_310 = {S29, 0, &V22772_310_57248, &P22772_309};	/*Property = GeoLocation*/
const Value V22772_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22772_312 = {S30, 0, &V22772_312_57254, &P22772_310};	/*Property = History*/
const Value V22772_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22772_314 = {S31, 0, &V22772_314_57261, &P22772_312};	/*Property = Html5*/
const Value V22772_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22772_313 = {S32, 0, &V22772_313_57258, &P22772_314};	/*Property = Html-Media-Capture*/
const Value V22772_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22772_315 = {S33, 0, &V22772_315_57264, &P22772_313};	/*Property = Iframe*/
const Value V22772_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22772_316 = {S34, 0, &V22772_316_57266, &P22772_315};	/*Property = IndexedDB*/
const Value V22772_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22772_317 = {S35, 0, &V22772_317_57269, &P22772_316};	/*Property = Json*/
const Value V22772_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22772_15 = {S37, 0, &V22772_15_29, &P22772_317};	/*Property = LayoutEngine*/
const Value V22772_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22772_344 = {S38, 0, &V22772_344_57272, &P22772_15};	/*Property = Masking*/
const Value V22772_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22772_320 = {S39, 0, &V22772_320_57290, &P22772_344};	/*Property = PostMessage*/
const Value V22772_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22772_321 = {S40, 0, &V22772_321_57293, &P22772_320};	/*Property = Progress*/
const Value V22772_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22772_322 = {S41, 0, &V22772_322_57296, &P22772_321};	/*Property = Prompts*/
const Value V22772_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22772_323 = {S42, 0, &V22772_323_57299, &P22772_322};	/*Property = Selector*/
const Value V22772_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22772_324 = {S43, 0, &V22772_324_57305, &P22772_323};	/*Property = Svg*/
const Value V22772_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22772_327 = {S44, 0, &V22772_327_57314, &P22772_324};	/*Property = TouchEvents*/
const Value V22772_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22772_328 = {S45, 0, &V22772_328_57318, &P22772_327};	/*Property = Track*/
const Value V22772_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22772_329 = {S46, 0, &V22772_329_57323, &P22772_328};	/*Property = Video*/
const Value V22772_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22772_330 = {S47, 0, &V22772_330_57326, &P22772_329};	/*Property = Viewport*/
const Value V22772_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22772_332 = {S48, 0, &V22772_332_57338, &P22772_330};	/*Property = WebWorkers*/
const Value V22772_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22772_333 = {S49, 0, &V22772_333_57342, &P22772_332};	/*Property = Xhr2*/
const Profile P22772 = {"22772", &P22772_333};	/*Profile ID - 22772*/
const Value V23198_1_1 = {S2, NULL};	/*Value = True*/
const Property P23198_1 = {S50, 0, &V23198_1_1, NULL};	/*Property = IsMobile*/
const Value V23198_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23198_5 = {S52, 0, &V23198_5_37, &P23198_1};	/*Property = ScreenPixelsHeight*/
const Value V23198_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23198_4 = {S54, 0, &V23198_4_36, &P23198_5};	/*Property = ScreenPixelsWidth*/
const Profile P23198 = {"23198", &P23198_4};	/*Profile ID - 23198*/
const Value V22850_1_1 = {S2, NULL};	/*Value = True*/
const Property P22850_1 = {S50, 0, &V22850_1_1, NULL};	/*Property = IsMobile*/
const Value V22850_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22850_5 = {S52, 0, &V22850_5_1227, &P22850_1};	/*Property = ScreenPixelsHeight*/
const Value V22850_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22850_4 = {S54, 0, &V22850_4_1229, &P22850_5};	/*Property = ScreenPixelsWidth*/
const Profile P22850 = {"22850", &P22850_4};	/*Profile ID - 22850*/
const Value V22773_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22773_279 = {S1, 0, &V22773_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22773_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22773_282 = {S3, 0, &V22773_282_57154, &P22773_279};	/*Property = BlobBuilder*/
const Value V22773_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22773_350 = {S4, 0, &V22773_350_57444, &P22773_282};	/*Property = Canvas*/
const Value V22773_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22773_283 = {S5, 0, &V22773_283_57159, &P22773_350};	/*Property = CssBackground*/
const Value V22773_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22773_284 = {S6, 0, &V22773_284_57162, &P22773_283};	/*Property = CssBorderImage*/
const Value V22773_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22773_285 = {S7, 0, &V22773_285_57166, &P22773_284};	/*Property = CssCanvas*/
const Value V22773_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22773_286 = {S8, 0, &V22773_286_57168, &P22773_285};	/*Property = CssColor*/
const Value V22773_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22773_237 = {S9, 0, &V22773_237_55786, &P22773_286};	/*Property = CssColumn*/
const Value V22773_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22773_287 = {S10, 0, &V22773_287_57172, &P22773_237};	/*Property = CssFlexbox*/
const Value V22773_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22773_288 = {S11, 0, &V22773_288_57174, &P22773_287};	/*Property = CssFont*/
const Value V22773_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22773_289 = {S12, 0, &V22773_289_57759, &P22773_288};	/*Property = CssImages*/
const Value V22773_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22773_291 = {S13, 0, &V22773_291_57180, &P22773_289};	/*Property = CssMediaQueries*/
const Value V22773_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22773_292 = {S14, 0, &V22773_292_57183, &P22773_291};	/*Property = CssMinMax*/
const Value V22773_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22773_293 = {S15, 0, &V22773_293_57187, &P22773_292};	/*Property = CssOverflow*/
const Value V22773_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22773_294 = {S16, 0, &V22773_294_57189, &P22773_293};	/*Property = CssPosition*/
const Value V22773_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22773_295 = {S17, 0, &V22773_295_57192, &P22773_294};	/*Property = CssText*/
const Value V22773_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22773_297 = {S18, 0, &V22773_297_57195, &P22773_295};	/*Property = CssTransforms*/
const Value V22773_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22773_241 = {S19, 0, &V22773_241_55799, &P22773_297};	/*Property = CssTransitions*/
const Value V22773_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22773_298 = {S20, 0, &V22773_298_57206, &P22773_241};	/*Property = CssUI*/
const Value V22773_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22773_300 = {S21, 0, &V22773_300_57215, &P22773_298};	/*Property = DataSet*/
const Value V22773_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22773_301 = {S22, 0, &V22773_301_57218, &P22773_300};	/*Property = DataUrl*/
const Value V22773_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22773_302 = {S23, 0, &V22773_302_57221, &P22773_301};	/*Property = DeviceOrientation*/
const Value V22773_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22773_304 = {S24, 0, &V22773_304_57227, &P22773_302};	/*Property = FileReader*/
const Value V22773_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22773_305 = {S25, 0, &V22773_305_57231, &P22773_304};	/*Property = FileSaver*/
const Value V22773_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22773_306 = {S26, 0, &V22773_306_57237, &P22773_305};	/*Property = FileWriter*/
const Value V22773_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22773_307 = {S27, 0, &V22773_307_57239, &P22773_306};	/*Property = FormData*/
const Value V22773_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22773_309 = {S28, 0, &V22773_309_57246, &P22773_307};	/*Property = Fullscreen*/
const Value V22773_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22773_310 = {S29, 0, &V22773_310_57248, &P22773_309};	/*Property = GeoLocation*/
const Value V22773_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22773_312 = {S30, 0, &V22773_312_57254, &P22773_310};	/*Property = History*/
const Value V22773_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22773_314 = {S31, 0, &V22773_314_57261, &P22773_312};	/*Property = Html5*/
const Value V22773_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22773_313 = {S32, 0, &V22773_313_57258, &P22773_314};	/*Property = Html-Media-Capture*/
const Value V22773_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22773_315 = {S33, 0, &V22773_315_57264, &P22773_313};	/*Property = Iframe*/
const Value V22773_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22773_316 = {S34, 0, &V22773_316_57266, &P22773_315};	/*Property = IndexedDB*/
const Value V22773_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22773_317 = {S35, 0, &V22773_317_57269, &P22773_316};	/*Property = Json*/
const Value V22773_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22773_15 = {S37, 0, &V22773_15_29, &P22773_317};	/*Property = LayoutEngine*/
const Value V22773_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22773_344 = {S38, 0, &V22773_344_57272, &P22773_15};	/*Property = Masking*/
const Value V22773_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22773_320 = {S39, 0, &V22773_320_57290, &P22773_344};	/*Property = PostMessage*/
const Value V22773_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22773_321 = {S40, 0, &V22773_321_57293, &P22773_320};	/*Property = Progress*/
const Value V22773_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22773_322 = {S41, 0, &V22773_322_57296, &P22773_321};	/*Property = Prompts*/
const Value V22773_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22773_323 = {S42, 0, &V22773_323_57299, &P22773_322};	/*Property = Selector*/
const Value V22773_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22773_324 = {S43, 0, &V22773_324_57305, &P22773_323};	/*Property = Svg*/
const Value V22773_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22773_327 = {S44, 0, &V22773_327_57314, &P22773_324};	/*Property = TouchEvents*/
const Value V22773_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22773_328 = {S45, 0, &V22773_328_57318, &P22773_327};	/*Property = Track*/
const Value V22773_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22773_329 = {S46, 0, &V22773_329_57323, &P22773_328};	/*Property = Video*/
const Value V22773_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22773_330 = {S47, 0, &V22773_330_57326, &P22773_329};	/*Property = Viewport*/
const Value V22773_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22773_332 = {S48, 0, &V22773_332_57338, &P22773_330};	/*Property = WebWorkers*/
const Value V22773_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22773_333 = {S49, 0, &V22773_333_57342, &P22773_332};	/*Property = Xhr2*/
const Profile P22773 = {"22773", &P22773_333};	/*Profile ID - 22773*/
const Value V23226_1_1 = {S2, NULL};	/*Value = True*/
const Property P23226_1 = {S50, 0, &V23226_1_1, NULL};	/*Property = IsMobile*/
const Value V23226_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23226_5 = {S52, 0, &V23226_5_1277, &P23226_1};	/*Property = ScreenPixelsHeight*/
const Value V23226_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23226_4 = {S54, 0, &V23226_4_1279, &P23226_5};	/*Property = ScreenPixelsWidth*/
const Profile P23226 = {"23226", &P23226_4};	/*Profile ID - 23226*/
const Value V22783_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_279 = {S1, 0, &V22783_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22783_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_282 = {S3, 0, &V22783_282_57155, &P22783_279};	/*Property = BlobBuilder*/
const Value V22783_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_350 = {S4, 0, &V22783_350_57446, &P22783_282};	/*Property = Canvas*/
const Value V22783_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_283 = {S5, 0, &V22783_283_57161, &P22783_350};	/*Property = CssBackground*/
const Value V22783_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_284 = {S6, 0, &V22783_284_57164, &P22783_283};	/*Property = CssBorderImage*/
const Value V22783_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_285 = {S7, 0, &V22783_285_57167, &P22783_284};	/*Property = CssCanvas*/
const Value V22783_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_286 = {S8, 0, &V22783_286_57170, &P22783_285};	/*Property = CssColor*/
const Value V22783_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_237 = {S9, 0, &V22783_237_55786, &P22783_286};	/*Property = CssColumn*/
const Value V22783_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_287 = {S10, 0, &V22783_287_57173, &P22783_237};	/*Property = CssFlexbox*/
const Value V22783_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_288 = {S11, 0, &V22783_288_57176, &P22783_287};	/*Property = CssFont*/
const Value V22783_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_289 = {S12, 0, &V22783_289_86899, &P22783_288};	/*Property = CssImages*/
const Value V22783_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_291 = {S13, 0, &V22783_291_57182, &P22783_289};	/*Property = CssMediaQueries*/
const Value V22783_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_292 = {S14, 0, &V22783_292_57185, &P22783_291};	/*Property = CssMinMax*/
const Value V22783_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_293 = {S15, 0, &V22783_293_57188, &P22783_292};	/*Property = CssOverflow*/
const Value V22783_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_294 = {S16, 0, &V22783_294_57191, &P22783_293};	/*Property = CssPosition*/
const Value V22783_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_295 = {S17, 0, &V22783_295_57194, &P22783_294};	/*Property = CssText*/
const Value V22783_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_297 = {S18, 0, &V22783_297_57197, &P22783_295};	/*Property = CssTransforms*/
const Value V22783_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_241 = {S19, 0, &V22783_241_55801, &P22783_297};	/*Property = CssTransitions*/
const Value V22783_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_298 = {S20, 0, &V22783_298_57208, &P22783_241};	/*Property = CssUI*/
const Value V22783_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_300 = {S21, 0, &V22783_300_57217, &P22783_298};	/*Property = DataSet*/
const Value V22783_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_301 = {S22, 0, &V22783_301_57220, &P22783_300};	/*Property = DataUrl*/
const Value V22783_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_302 = {S23, 0, &V22783_302_57223, &P22783_301};	/*Property = DeviceOrientation*/
const Value V22783_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_304 = {S24, 0, &V22783_304_57229, &P22783_302};	/*Property = FileReader*/
const Value V22783_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_305 = {S25, 0, &V22783_305_57232, &P22783_304};	/*Property = FileSaver*/
const Value V22783_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_306 = {S26, 0, &V22783_306_57238, &P22783_305};	/*Property = FileWriter*/
const Value V22783_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_307 = {S27, 0, &V22783_307_57241, &P22783_306};	/*Property = FormData*/
const Value V22783_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_309 = {S28, 0, &V22783_309_57247, &P22783_307};	/*Property = Fullscreen*/
const Value V22783_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_310 = {S29, 0, &V22783_310_57250, &P22783_309};	/*Property = GeoLocation*/
const Value V22783_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_312 = {S30, 0, &V22783_312_57256, &P22783_310};	/*Property = History*/
const Value V22783_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_314 = {S31, 0, &V22783_314_57262, &P22783_312};	/*Property = Html5*/
const Value V22783_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_313 = {S32, 0, &V22783_313_57259, &P22783_314};	/*Property = Html-Media-Capture*/
const Value V22783_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_315 = {S33, 0, &V22783_315_57265, &P22783_313};	/*Property = Iframe*/
const Value V22783_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_316 = {S34, 0, &V22783_316_57268, &P22783_315};	/*Property = IndexedDB*/
const Value V22783_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_317 = {S35, 0, &V22783_317_57271, &P22783_316};	/*Property = Json*/
const Value V22783_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_15 = {S37, 0, &V22783_15_72, &P22783_317};	/*Property = LayoutEngine*/
const Value V22783_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_344 = {S38, 0, &V22783_344_57274, &P22783_15};	/*Property = Masking*/
const Value V22783_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_320 = {S39, 0, &V22783_320_57292, &P22783_344};	/*Property = PostMessage*/
const Value V22783_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_321 = {S40, 0, &V22783_321_57295, &P22783_320};	/*Property = Progress*/
const Value V22783_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_322 = {S41, 0, &V22783_322_57298, &P22783_321};	/*Property = Prompts*/
const Value V22783_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_323 = {S42, 0, &V22783_323_57301, &P22783_322};	/*Property = Selector*/
const Value V22783_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_324 = {S43, 0, &V22783_324_57307, &P22783_323};	/*Property = Svg*/
const Value V22783_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_327 = {S44, 0, &V22783_327_57316, &P22783_324};	/*Property = TouchEvents*/
const Value V22783_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_328 = {S45, 0, &V22783_328_57319, &P22783_327};	/*Property = Track*/
const Value V22783_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_329 = {S46, 0, &V22783_329_57325, &P22783_328};	/*Property = Video*/
const Value V22783_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_330 = {S47, 0, &V22783_330_57328, &P22783_329};	/*Property = Viewport*/
const Value V22783_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_332 = {S48, 0, &V22783_332_57340, &P22783_330};	/*Property = WebWorkers*/
const Value V22783_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22783_333 = {S49, 0, &V22783_333_57343, &P22783_332};	/*Property = Xhr2*/
const Profile P22783 = {"22783", &P22783_333};	/*Profile ID - 22783*/
const Value V22929_1_1 = {S2, NULL};	/*Value = True*/
const Property P22929_1 = {S50, 0, &V22929_1_1, NULL};	/*Property = IsMobile*/
const Value V22929_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22929_5 = {S52, 0, &V22929_5_1227, &P22929_1};	/*Property = ScreenPixelsHeight*/
const Value V22929_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P22929_4 = {S54, 0, &V22929_4_1477, &P22929_5};	/*Property = ScreenPixelsWidth*/
const Profile P22929 = {"22929", &P22929_4};	/*Profile ID - 22929*/
const Value V22714_1_1 = {S2, NULL};	/*Value = True*/
const Property P22714_1 = {S50, 0, &V22714_1_1, NULL};	/*Property = IsMobile*/
const Value V22714_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22714_5 = {S52, 0, &V22714_5_9981, &P22714_1};	/*Property = ScreenPixelsHeight*/
const Value V22714_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P22714_4 = {S54, 0, &V22714_4_37617, &P22714_5};	/*Property = ScreenPixelsWidth*/
const Profile P22714 = {"22714", &P22714_4};	/*Profile ID - 22714*/
const Value V22715_1_2 = {S0, NULL};	/*Value = False*/
const Property P22715_1 = {S50, 0, &V22715_1_2, NULL};	/*Property = IsMobile*/
const Value V22715_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P22715_5 = {S52, 0, &V22715_5_37, &P22715_1};	/*Property = ScreenPixelsHeight*/
const Value V22715_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P22715_4 = {S54, 0, &V22715_4_36, &P22715_5};	/*Property = ScreenPixelsWidth*/
const Profile P22715 = {"22715", &P22715_4};	/*Profile ID - 22715*/
const Value V23133_1_1 = {S2, NULL};	/*Value = True*/
const Property P23133_1 = {S50, 0, &V23133_1_1, NULL};	/*Property = IsMobile*/
const Value V23133_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23133_5 = {S52, 0, &V23133_5_9981, &P23133_1};	/*Property = ScreenPixelsHeight*/
const Value V23133_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23133_4 = {S54, 0, &V23133_4_2336, &P23133_5};	/*Property = ScreenPixelsWidth*/
const Profile P23133 = {"23133", &P23133_4};	/*Profile ID - 23133*/
const Value V23621_1_2 = {S0, NULL};	/*Value = False*/
const Property P23621_1 = {S50, 0, &V23621_1_2, NULL};	/*Property = IsMobile*/
const Value V23621_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23621_5 = {S52, 0, &V23621_5_60417, &P23621_1};	/*Property = ScreenPixelsHeight*/
const Value V23621_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23621_4 = {S54, 0, &V23621_4_57095, &P23621_5};	/*Property = ScreenPixelsWidth*/
const Profile P23621 = {"23621", &P23621_4};	/*Profile ID - 23621*/
const Value V23387_1_1 = {S2, NULL};	/*Value = True*/
const Property P23387_1 = {S50, 0, &V23387_1_1, NULL};	/*Property = IsMobile*/
const Value V23387_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23387_5 = {S52, 0, &V23387_5_1832, &P23387_1};	/*Property = ScreenPixelsHeight*/
const Value V23387_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23387_4 = {S54, 0, &V23387_4_5645, &P23387_5};	/*Property = ScreenPixelsWidth*/
const Profile P23387 = {"23387", &P23387_4};	/*Profile ID - 23387*/
const Value V23249_1_1 = {S2, NULL};	/*Value = True*/
const Property P23249_1 = {S50, 0, &V23249_1_1, NULL};	/*Property = IsMobile*/
const Value V23249_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23249_5 = {S52, 0, &V23249_5_37569, &P23249_1};	/*Property = ScreenPixelsHeight*/
const Value V23249_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23249_4 = {S54, 0, &V23249_4_5645, &P23249_5};	/*Property = ScreenPixelsWidth*/
const Profile P23249 = {"23249", &P23249_4};	/*Profile ID - 23249*/
const Value V23342_1_1 = {S2, NULL};	/*Value = True*/
const Property P23342_1 = {S50, 0, &V23342_1_1, NULL};	/*Property = IsMobile*/
const Value V23342_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23342_5 = {S52, 0, &V23342_5_1277, &P23342_1};	/*Property = ScreenPixelsHeight*/
const Value V23342_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23342_4 = {S54, 0, &V23342_4_2336, &P23342_5};	/*Property = ScreenPixelsWidth*/
const Profile P23342 = {"23342", &P23342_4};	/*Profile ID - 23342*/
const Value V23377_1_1 = {S2, NULL};	/*Value = True*/
const Property P23377_1 = {S50, 0, &V23377_1_1, NULL};	/*Property = IsMobile*/
const Value V23377_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23377_5 = {S52, 0, &V23377_5_1277, &P23377_1};	/*Property = ScreenPixelsHeight*/
const Value V23377_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23377_4 = {S54, 0, &V23377_4_1279, &P23377_5};	/*Property = ScreenPixelsWidth*/
const Profile P23377 = {"23377", &P23377_4};	/*Profile ID - 23377*/
const Value V23055_1_1 = {S2, NULL};	/*Value = True*/
const Property P23055_1 = {S50, 0, &V23055_1_1, NULL};	/*Property = IsMobile*/
const Value V23055_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23055_5 = {S52, 0, &V23055_5_1403, &P23055_1};	/*Property = ScreenPixelsHeight*/
const Value V23055_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23055_4 = {S54, 0, &V23055_4_1404, &P23055_5};	/*Property = ScreenPixelsWidth*/
const Profile P23055 = {"23055", &P23055_4};	/*Profile ID - 23055*/
const Value V23538_1_1 = {S2, NULL};	/*Value = True*/
const Property P23538_1 = {S50, 0, &V23538_1_1, NULL};	/*Property = IsMobile*/
const Value V23538_5_62105 = {S144, NULL};	/*Value = 2048*/
const Property P23538_5 = {S52, 0, &V23538_5_62105, &P23538_1};	/*Property = ScreenPixelsHeight*/
const Value V23538_4_62106 = {S145, NULL};	/*Value = 1536*/
const Property P23538_4 = {S54, 0, &V23538_4_62106, &P23538_5};	/*Property = ScreenPixelsWidth*/
const Profile P23538 = {"23538", &P23538_4};	/*Profile ID - 23538*/
const Value V22941_1_1 = {S2, NULL};	/*Value = True*/
const Property P22941_1 = {S50, 0, &V22941_1_1, NULL};	/*Property = IsMobile*/
const Value V22941_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22941_5 = {S52, 0, &V22941_5_1277, &P22941_1};	/*Property = ScreenPixelsHeight*/
const Value V22941_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22941_4 = {S54, 0, &V22941_4_1279, &P22941_5};	/*Property = ScreenPixelsWidth*/
const Profile P22941 = {"22941", &P22941_4};	/*Profile ID - 22941*/
const Value V23643_1_1 = {S2, NULL};	/*Value = True*/
const Property P23643_1 = {S50, 0, &V23643_1_1, NULL};	/*Property = IsMobile*/
const Value V23643_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P23643_5 = {S52, 0, &V23643_5_59138, &P23643_1};	/*Property = ScreenPixelsHeight*/
const Value V23643_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P23643_4 = {S54, 0, &V23643_4_59970, &P23643_5};	/*Property = ScreenPixelsWidth*/
const Profile P23643 = {"23643", &P23643_4};	/*Profile ID - 23643*/
const Value V22955_1_1 = {S2, NULL};	/*Value = True*/
const Property P22955_1 = {S50, 0, &V22955_1_1, NULL};	/*Property = IsMobile*/
const Value V22955_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22955_5 = {S52, 0, &V22955_5_1227, &P22955_1};	/*Property = ScreenPixelsHeight*/
const Value V22955_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22955_4 = {S54, 0, &V22955_4_1229, &P22955_5};	/*Property = ScreenPixelsWidth*/
const Profile P22955 = {"22955", &P22955_4};	/*Profile ID - 22955*/
const Value V23428_1_1 = {S2, NULL};	/*Value = True*/
const Property P23428_1 = {S50, 0, &V23428_1_1, NULL};	/*Property = IsMobile*/
const Value V23428_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23428_5 = {S52, 0, &V23428_5_1227, &P23428_1};	/*Property = ScreenPixelsHeight*/
const Value V23428_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23428_4 = {S54, 0, &V23428_4_1229, &P23428_5};	/*Property = ScreenPixelsWidth*/
const Profile P23428 = {"23428", &P23428_4};	/*Profile ID - 23428*/
const Value V22963_1_1 = {S2, NULL};	/*Value = True*/
const Property P22963_1 = {S50, 0, &V22963_1_1, NULL};	/*Property = IsMobile*/
const Value V22963_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22963_5 = {S52, 0, &V22963_5_1277, &P22963_1};	/*Property = ScreenPixelsHeight*/
const Value V22963_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22963_4 = {S54, 0, &V22963_4_1279, &P22963_5};	/*Property = ScreenPixelsWidth*/
const Profile P22963 = {"22963", &P22963_4};	/*Profile ID - 22963*/
const Value V23416_1_1 = {S2, NULL};	/*Value = True*/
const Property P23416_1 = {S50, 0, &V23416_1_1, NULL};	/*Property = IsMobile*/
const Value V23416_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23416_5 = {S52, 0, &V23416_5_37569, &P23416_1};	/*Property = ScreenPixelsHeight*/
const Value V23416_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23416_4 = {S54, 0, &V23416_4_5645, &P23416_5};	/*Property = ScreenPixelsWidth*/
const Profile P23416 = {"23416", &P23416_4};	/*Profile ID - 23416*/
const Value V23188_1_1 = {S2, NULL};	/*Value = True*/
const Property P23188_1 = {S50, 0, &V23188_1_1, NULL};	/*Property = IsMobile*/
const Value V23188_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23188_5 = {S52, 0, &V23188_5_1227, &P23188_1};	/*Property = ScreenPixelsHeight*/
const Value V23188_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23188_4 = {S54, 0, &V23188_4_1229, &P23188_5};	/*Property = ScreenPixelsWidth*/
const Profile P23188 = {"23188", &P23188_4};	/*Profile ID - 23188*/
const Value V22950_1_1 = {S2, NULL};	/*Value = True*/
const Property P22950_1 = {S50, 0, &V22950_1_1, NULL};	/*Property = IsMobile*/
const Value V22950_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22950_5 = {S52, 0, &V22950_5_9981, &P22950_1};	/*Property = ScreenPixelsHeight*/
const Value V22950_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P22950_4 = {S54, 0, &V22950_4_37617, &P22950_5};	/*Property = ScreenPixelsWidth*/
const Profile P22950 = {"22950", &P22950_4};	/*Profile ID - 22950*/
const Value V23517_1_1 = {S2, NULL};	/*Value = True*/
const Property P23517_1 = {S50, 0, &V23517_1_1, NULL};	/*Property = IsMobile*/
const Value V23517_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23517_5 = {S52, 0, &V23517_5_1227, &P23517_1};	/*Property = ScreenPixelsHeight*/
const Value V23517_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23517_4 = {S54, 0, &V23517_4_1229, &P23517_5};	/*Property = ScreenPixelsWidth*/
const Profile P23517 = {"23517", &P23517_4};	/*Profile ID - 23517*/
const Value V22872_1_1 = {S2, NULL};	/*Value = True*/
const Property P22872_1 = {S50, 0, &V22872_1_1, NULL};	/*Property = IsMobile*/
const Value V22872_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P22872_5 = {S52, 0, &V22872_5_1222, &P22872_1};	/*Property = ScreenPixelsHeight*/
const Value V22872_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P22872_4 = {S54, 0, &V22872_4_1224, &P22872_5};	/*Property = ScreenPixelsWidth*/
const Profile P22872 = {"22872", &P22872_4};	/*Profile ID - 22872*/
const Value V22911_1_1 = {S2, NULL};	/*Value = True*/
const Property P22911_1 = {S50, 0, &V22911_1_1, NULL};	/*Property = IsMobile*/
const Value V22911_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22911_5 = {S52, 0, &V22911_5_1277, &P22911_1};	/*Property = ScreenPixelsHeight*/
const Value V22911_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22911_4 = {S54, 0, &V22911_4_1279, &P22911_5};	/*Property = ScreenPixelsWidth*/
const Profile P22911 = {"22911", &P22911_4};	/*Profile ID - 22911*/
const Value V22811_1_1 = {S2, NULL};	/*Value = True*/
const Property P22811_1 = {S50, 0, &V22811_1_1, NULL};	/*Property = IsMobile*/
const Value V22811_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22811_5 = {S52, 0, &V22811_5_1227, &P22811_1};	/*Property = ScreenPixelsHeight*/
const Value V22811_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22811_4 = {S54, 0, &V22811_4_1229, &P22811_5};	/*Property = ScreenPixelsWidth*/
const Profile P22811 = {"22811", &P22811_4};	/*Profile ID - 22811*/
const Value V23561_1_1 = {S2, NULL};	/*Value = True*/
const Property P23561_1 = {S50, 0, &V23561_1_1, NULL};	/*Property = IsMobile*/
const Value V23561_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23561_5 = {S52, 0, &V23561_5_1403, &P23561_1};	/*Property = ScreenPixelsHeight*/
const Value V23561_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23561_4 = {S54, 0, &V23561_4_1404, &P23561_5};	/*Property = ScreenPixelsWidth*/
const Profile P23561 = {"23561", &P23561_4};	/*Profile ID - 23561*/
const Value V22922_1_1 = {S2, NULL};	/*Value = True*/
const Property P22922_1 = {S50, 0, &V22922_1_1, NULL};	/*Property = IsMobile*/
const Value V22922_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22922_5 = {S52, 0, &V22922_5_9981, &P22922_1};	/*Property = ScreenPixelsHeight*/
const Value V22922_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22922_4 = {S54, 0, &V22922_4_2336, &P22922_5};	/*Property = ScreenPixelsWidth*/
const Profile P22922 = {"22922", &P22922_4};	/*Profile ID - 22922*/
const Value V22844_1_1 = {S2, NULL};	/*Value = True*/
const Property P22844_1 = {S50, 0, &V22844_1_1, NULL};	/*Property = IsMobile*/
const Value V22844_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22844_5 = {S52, 0, &V22844_5_1403, &P22844_1};	/*Property = ScreenPixelsHeight*/
const Value V22844_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22844_4 = {S54, 0, &V22844_4_1404, &P22844_5};	/*Property = ScreenPixelsWidth*/
const Profile P22844 = {"22844", &P22844_4};	/*Profile ID - 22844*/
const Value V23275_1_1 = {S2, NULL};	/*Value = True*/
const Property P23275_1 = {S50, 0, &V23275_1_1, NULL};	/*Property = IsMobile*/
const Value V23275_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23275_5 = {S52, 0, &V23275_5_1458, &P23275_1};	/*Property = ScreenPixelsHeight*/
const Value V23275_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23275_4 = {S54, 0, &V23275_4_37617, &P23275_5};	/*Property = ScreenPixelsWidth*/
const Profile P23275 = {"23275", &P23275_4};	/*Profile ID - 23275*/
const Value V23119_1_1 = {S2, NULL};	/*Value = True*/
const Property P23119_1 = {S50, 0, &V23119_1_1, NULL};	/*Property = IsMobile*/
const Value V23119_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P23119_5 = {S52, 0, &V23119_5_1304, &P23119_1};	/*Property = ScreenPixelsHeight*/
const Value V23119_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23119_4 = {S54, 0, &V23119_4_1224, &P23119_5};	/*Property = ScreenPixelsWidth*/
const Profile P23119 = {"23119", &P23119_4};	/*Profile ID - 23119*/
const Value V22988_1_1 = {S2, NULL};	/*Value = True*/
const Property P22988_1 = {S50, 0, &V22988_1_1, NULL};	/*Property = IsMobile*/
const Value V22988_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22988_5 = {S52, 0, &V22988_5_1227, &P22988_1};	/*Property = ScreenPixelsHeight*/
const Value V22988_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22988_4 = {S54, 0, &V22988_4_1229, &P22988_5};	/*Property = ScreenPixelsWidth*/
const Profile P22988 = {"22988", &P22988_4};	/*Profile ID - 22988*/
const Value V23478_1_1 = {S2, NULL};	/*Value = True*/
const Property P23478_1 = {S50, 0, &V23478_1_1, NULL};	/*Property = IsMobile*/
const Value V23478_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23478_5 = {S52, 0, &V23478_5_9981, &P23478_1};	/*Property = ScreenPixelsHeight*/
const Value V23478_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23478_4 = {S54, 0, &V23478_4_39098, &P23478_5};	/*Property = ScreenPixelsWidth*/
const Profile P23478 = {"23478", &P23478_4};	/*Profile ID - 23478*/
const Value V22855_1_1 = {S2, NULL};	/*Value = True*/
const Property P22855_1 = {S50, 0, &V22855_1_1, NULL};	/*Property = IsMobile*/
const Value V22855_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22855_5 = {S52, 0, &V22855_5_1277, &P22855_1};	/*Property = ScreenPixelsHeight*/
const Value V22855_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22855_4 = {S54, 0, &V22855_4_1279, &P22855_5};	/*Property = ScreenPixelsWidth*/
const Profile P22855 = {"22855", &P22855_4};	/*Profile ID - 22855*/
const Value V22989_1_1 = {S2, NULL};	/*Value = True*/
const Property P22989_1 = {S50, 0, &V22989_1_1, NULL};	/*Property = IsMobile*/
const Value V22989_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22989_5 = {S52, 0, &V22989_5_9981, &P22989_1};	/*Property = ScreenPixelsHeight*/
const Value V22989_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22989_4 = {S54, 0, &V22989_4_39098, &P22989_5};	/*Property = ScreenPixelsWidth*/
const Profile P22989 = {"22989", &P22989_4};	/*Profile ID - 22989*/
const Value V22836_1_1 = {S2, NULL};	/*Value = True*/
const Property P22836_1 = {S50, 0, &V22836_1_1, NULL};	/*Property = IsMobile*/
const Value V22836_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22836_5 = {S52, 0, &V22836_5_59138, &P22836_1};	/*Property = ScreenPixelsHeight*/
const Value V22836_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22836_4 = {S54, 0, &V22836_4_59970, &P22836_5};	/*Property = ScreenPixelsWidth*/
const Profile P22836 = {"22836", &P22836_4};	/*Profile ID - 22836*/
const Value V22874_1_1 = {S2, NULL};	/*Value = True*/
const Property P22874_1 = {S50, 0, &V22874_1_1, NULL};	/*Property = IsMobile*/
const Value V22874_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22874_5 = {S52, 0, &V22874_5_59138, &P22874_1};	/*Property = ScreenPixelsHeight*/
const Value V22874_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22874_4 = {S54, 0, &V22874_4_59970, &P22874_5};	/*Property = ScreenPixelsWidth*/
const Profile P22874 = {"22874", &P22874_4};	/*Profile ID - 22874*/
const Value V23717_1_1 = {S2, NULL};	/*Value = True*/
const Property P23717_1 = {S50, 0, &V23717_1_1, NULL};	/*Property = IsMobile*/
const Value V23717_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23717_5 = {S52, 0, &V23717_5_1403, &P23717_1};	/*Property = ScreenPixelsHeight*/
const Value V23717_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23717_4 = {S54, 0, &V23717_4_1404, &P23717_5};	/*Property = ScreenPixelsWidth*/
const Profile P23717 = {"23717", &P23717_4};	/*Profile ID - 23717*/
const Value V23163_1_1 = {S2, NULL};	/*Value = True*/
const Property P23163_1 = {S50, 0, &V23163_1_1, NULL};	/*Property = IsMobile*/
const Value V23163_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P23163_5 = {S52, 0, &V23163_5_59138, &P23163_1};	/*Property = ScreenPixelsHeight*/
const Value V23163_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P23163_4 = {S54, 0, &V23163_4_59970, &P23163_5};	/*Property = ScreenPixelsWidth*/
const Profile P23163 = {"23163", &P23163_4};	/*Profile ID - 23163*/
const Value V23105_1_1 = {S2, NULL};	/*Value = True*/
const Property P23105_1 = {S50, 0, &V23105_1_1, NULL};	/*Property = IsMobile*/
const Value V23105_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23105_5 = {S52, 0, &V23105_5_1277, &P23105_1};	/*Property = ScreenPixelsHeight*/
const Value V23105_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23105_4 = {S54, 0, &V23105_4_1279, &P23105_5};	/*Property = ScreenPixelsWidth*/
const Profile P23105 = {"23105", &P23105_4};	/*Profile ID - 23105*/
const Value V23722_1_1 = {S2, NULL};	/*Value = True*/
const Property P23722_1 = {S50, 0, &V23722_1_1, NULL};	/*Property = IsMobile*/
const Value V23722_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23722_5 = {S52, 0, &V23722_5_1277, &P23722_1};	/*Property = ScreenPixelsHeight*/
const Value V23722_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23722_4 = {S54, 0, &V23722_4_1279, &P23722_5};	/*Property = ScreenPixelsWidth*/
const Profile P23722 = {"23722", &P23722_4};	/*Profile ID - 23722*/
const Value V20689_1_1 = {S2, NULL};	/*Value = True*/
const Property P20689_1 = {S50, 0, &V20689_1_1, NULL};	/*Property = IsMobile*/
const Value V20689_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P20689_5 = {S52, 0, &V20689_5_1403, &P20689_1};	/*Property = ScreenPixelsHeight*/
const Value V20689_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P20689_4 = {S54, 0, &V20689_4_1404, &P20689_5};	/*Property = ScreenPixelsWidth*/
const Profile P20689 = {"20689", &P20689_4};	/*Profile ID - 20689*/
const Value V22812_1_1 = {S2, NULL};	/*Value = True*/
const Property P22812_1 = {S50, 0, &V22812_1_1, NULL};	/*Property = IsMobile*/
const Value V22812_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22812_5 = {S52, 0, &V22812_5_1458, &P22812_1};	/*Property = ScreenPixelsHeight*/
const Value V22812_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22812_4 = {S54, 0, &V22812_4_1460, &P22812_5};	/*Property = ScreenPixelsWidth*/
const Profile P22812 = {"22812", &P22812_4};	/*Profile ID - 22812*/
const Value V24583_1_1 = {S2, NULL};	/*Value = True*/
const Property P24583_1 = {S50, 0, &V24583_1_1, NULL};	/*Property = IsMobile*/
const Value V24583_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24583_5 = {S52, 0, &V24583_5_9981, &P24583_1};	/*Property = ScreenPixelsHeight*/
const Value V24583_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24583_4 = {S54, 0, &V24583_4_39098, &P24583_5};	/*Property = ScreenPixelsWidth*/
const Profile P24583 = {"24583", &P24583_4};	/*Profile ID - 24583*/
const Value V22854_1_1 = {S2, NULL};	/*Value = True*/
const Property P22854_1 = {S50, 0, &V22854_1_1, NULL};	/*Property = IsMobile*/
const Value V22854_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22854_5 = {S52, 0, &V22854_5_59138, &P22854_1};	/*Property = ScreenPixelsHeight*/
const Value V22854_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22854_4 = {S54, 0, &V22854_4_59970, &P22854_5};	/*Property = ScreenPixelsWidth*/
const Profile P22854 = {"22854", &P22854_4};	/*Profile ID - 22854*/
const Value V22913_1_1 = {S2, NULL};	/*Value = True*/
const Property P22913_1 = {S50, 0, &V22913_1_1, NULL};	/*Property = IsMobile*/
const Value V22913_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22913_5 = {S52, 0, &V22913_5_1227, &P22913_1};	/*Property = ScreenPixelsHeight*/
const Value V22913_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22913_4 = {S54, 0, &V22913_4_1229, &P22913_5};	/*Property = ScreenPixelsWidth*/
const Profile P22913 = {"22913", &P22913_4};	/*Profile ID - 22913*/
const Value V22853_1_1 = {S2, NULL};	/*Value = True*/
const Property P22853_1 = {S50, 0, &V22853_1_1, NULL};	/*Property = IsMobile*/
const Value V22853_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22853_5 = {S52, 0, &V22853_5_59138, &P22853_1};	/*Property = ScreenPixelsHeight*/
const Value V22853_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22853_4 = {S54, 0, &V22853_4_59970, &P22853_5};	/*Property = ScreenPixelsWidth*/
const Profile P22853 = {"22853", &P22853_4};	/*Profile ID - 22853*/
const Value V23242_1_1 = {S2, NULL};	/*Value = True*/
const Property P23242_1 = {S50, 0, &V23242_1_1, NULL};	/*Property = IsMobile*/
const Value V23242_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23242_5 = {S52, 0, &V23242_5_9981, &P23242_1};	/*Property = ScreenPixelsHeight*/
const Value V23242_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23242_4 = {S54, 0, &V23242_4_39098, &P23242_5};	/*Property = ScreenPixelsWidth*/
const Profile P23242 = {"23242", &P23242_4};	/*Profile ID - 23242*/
const Value V22860_1_1 = {S2, NULL};	/*Value = True*/
const Property P22860_1 = {S50, 0, &V22860_1_1, NULL};	/*Property = IsMobile*/
const Value V22860_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22860_5 = {S52, 0, &V22860_5_1403, &P22860_1};	/*Property = ScreenPixelsHeight*/
const Value V22860_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22860_4 = {S54, 0, &V22860_4_1404, &P22860_5};	/*Property = ScreenPixelsWidth*/
const Profile P22860 = {"22860", &P22860_4};	/*Profile ID - 22860*/
const Value V22763_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22763_279 = {S1, 0, &V22763_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22763_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22763_282 = {S3, 0, &V22763_282_57154, &P22763_279};	/*Property = BlobBuilder*/
const Value V22763_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22763_350 = {S4, 0, &V22763_350_57444, &P22763_282};	/*Property = Canvas*/
const Value V22763_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22763_283 = {S5, 0, &V22763_283_57159, &P22763_350};	/*Property = CssBackground*/
const Value V22763_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22763_284 = {S6, 0, &V22763_284_57162, &P22763_283};	/*Property = CssBorderImage*/
const Value V22763_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22763_285 = {S7, 0, &V22763_285_57166, &P22763_284};	/*Property = CssCanvas*/
const Value V22763_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22763_286 = {S8, 0, &V22763_286_57168, &P22763_285};	/*Property = CssColor*/
const Value V22763_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22763_237 = {S9, 0, &V22763_237_55786, &P22763_286};	/*Property = CssColumn*/
const Value V22763_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22763_287 = {S10, 0, &V22763_287_57172, &P22763_237};	/*Property = CssFlexbox*/
const Value V22763_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22763_288 = {S11, 0, &V22763_288_57174, &P22763_287};	/*Property = CssFont*/
const Value V22763_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22763_289 = {S12, 0, &V22763_289_57759, &P22763_288};	/*Property = CssImages*/
const Value V22763_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22763_291 = {S13, 0, &V22763_291_57180, &P22763_289};	/*Property = CssMediaQueries*/
const Value V22763_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22763_292 = {S14, 0, &V22763_292_57183, &P22763_291};	/*Property = CssMinMax*/
const Value V22763_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22763_293 = {S15, 0, &V22763_293_57187, &P22763_292};	/*Property = CssOverflow*/
const Value V22763_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22763_294 = {S16, 0, &V22763_294_57189, &P22763_293};	/*Property = CssPosition*/
const Value V22763_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22763_295 = {S17, 0, &V22763_295_57192, &P22763_294};	/*Property = CssText*/
const Value V22763_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22763_297 = {S18, 0, &V22763_297_57195, &P22763_295};	/*Property = CssTransforms*/
const Value V22763_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22763_241 = {S19, 0, &V22763_241_55799, &P22763_297};	/*Property = CssTransitions*/
const Value V22763_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22763_298 = {S20, 0, &V22763_298_57206, &P22763_241};	/*Property = CssUI*/
const Value V22763_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22763_300 = {S21, 0, &V22763_300_57216, &P22763_298};	/*Property = DataSet*/
const Value V22763_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22763_301 = {S22, 0, &V22763_301_57218, &P22763_300};	/*Property = DataUrl*/
const Value V22763_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22763_302 = {S23, 0, &V22763_302_57222, &P22763_301};	/*Property = DeviceOrientation*/
const Value V22763_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22763_304 = {S24, 0, &V22763_304_57227, &P22763_302};	/*Property = FileReader*/
const Value V22763_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22763_305 = {S25, 0, &V22763_305_57231, &P22763_304};	/*Property = FileSaver*/
const Value V22763_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22763_306 = {S26, 0, &V22763_306_57237, &P22763_305};	/*Property = FileWriter*/
const Value V22763_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22763_307 = {S27, 0, &V22763_307_57239, &P22763_306};	/*Property = FormData*/
const Value V22763_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22763_309 = {S28, 0, &V22763_309_57246, &P22763_307};	/*Property = Fullscreen*/
const Value V22763_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22763_310 = {S29, 0, &V22763_310_57248, &P22763_309};	/*Property = GeoLocation*/
const Value V22763_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22763_312 = {S30, 0, &V22763_312_57254, &P22763_310};	/*Property = History*/
const Value V22763_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22763_314 = {S31, 0, &V22763_314_57260, &P22763_312};	/*Property = Html5*/
const Value V22763_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22763_313 = {S32, 0, &V22763_313_57258, &P22763_314};	/*Property = Html-Media-Capture*/
const Value V22763_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22763_315 = {S33, 0, &V22763_315_57264, &P22763_313};	/*Property = Iframe*/
const Value V22763_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22763_316 = {S34, 0, &V22763_316_57266, &P22763_315};	/*Property = IndexedDB*/
const Value V22763_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22763_317 = {S35, 0, &V22763_317_57269, &P22763_316};	/*Property = Json*/
const Value V22763_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22763_15 = {S37, 0, &V22763_15_28, &P22763_317};	/*Property = LayoutEngine*/
const Value V22763_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22763_344 = {S38, 0, &V22763_344_57272, &P22763_15};	/*Property = Masking*/
const Value V22763_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22763_320 = {S39, 0, &V22763_320_57290, &P22763_344};	/*Property = PostMessage*/
const Value V22763_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22763_321 = {S40, 0, &V22763_321_57293, &P22763_320};	/*Property = Progress*/
const Value V22763_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22763_322 = {S41, 0, &V22763_322_57296, &P22763_321};	/*Property = Prompts*/
const Value V22763_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22763_323 = {S42, 0, &V22763_323_57299, &P22763_322};	/*Property = Selector*/
const Value V22763_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22763_324 = {S43, 0, &V22763_324_57305, &P22763_323};	/*Property = Svg*/
const Value V22763_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22763_327 = {S44, 0, &V22763_327_57315, &P22763_324};	/*Property = TouchEvents*/
const Value V22763_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22763_328 = {S45, 0, &V22763_328_57318, &P22763_327};	/*Property = Track*/
const Value V22763_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22763_329 = {S46, 0, &V22763_329_57323, &P22763_328};	/*Property = Video*/
const Value V22763_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22763_330 = {S47, 0, &V22763_330_57326, &P22763_329};	/*Property = Viewport*/
const Value V22763_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22763_332 = {S48, 0, &V22763_332_57338, &P22763_330};	/*Property = WebWorkers*/
const Value V22763_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22763_333 = {S49, 0, &V22763_333_57341, &P22763_332};	/*Property = Xhr2*/
const Profile P22763 = {"22763", &P22763_333};	/*Profile ID - 22763*/
const Value V23807_1_1 = {S2, NULL};	/*Value = True*/
const Property P23807_1 = {S50, 0, &V23807_1_1, NULL};	/*Property = IsMobile*/
const Value V23807_5_59394 = {S151, NULL};	/*Value = 1136*/
const Property P23807_5 = {S52, 0, &V23807_5_59394, &P23807_1};	/*Property = ScreenPixelsHeight*/
const Value V23807_4_8554 = {S63, NULL};	/*Value = 640*/
const Property P23807_4 = {S54, 0, &V23807_4_8554, &P23807_5};	/*Property = ScreenPixelsWidth*/
const Profile P23807 = {"23807", &P23807_4};	/*Profile ID - 23807*/
const Value V24343_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24343_279 = {S1, 0, &V24343_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24343_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24343_282 = {S3, 0, &V24343_282_57154, &P24343_279};	/*Property = BlobBuilder*/
const Value V24343_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24343_350 = {S4, 0, &V24343_350_57444, &P24343_282};	/*Property = Canvas*/
const Value V24343_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24343_283 = {S5, 0, &V24343_283_57159, &P24343_350};	/*Property = CssBackground*/
const Value V24343_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24343_284 = {S6, 0, &V24343_284_57162, &P24343_283};	/*Property = CssBorderImage*/
const Value V24343_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24343_285 = {S7, 0, &V24343_285_57166, &P24343_284};	/*Property = CssCanvas*/
const Value V24343_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24343_286 = {S8, 0, &V24343_286_57168, &P24343_285};	/*Property = CssColor*/
const Value V24343_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24343_237 = {S9, 0, &V24343_237_55786, &P24343_286};	/*Property = CssColumn*/
const Value V24343_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24343_287 = {S10, 0, &V24343_287_57172, &P24343_237};	/*Property = CssFlexbox*/
const Value V24343_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24343_288 = {S11, 0, &V24343_288_57174, &P24343_287};	/*Property = CssFont*/
const Value V24343_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24343_289 = {S12, 0, &V24343_289_57759, &P24343_288};	/*Property = CssImages*/
const Value V24343_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24343_291 = {S13, 0, &V24343_291_57180, &P24343_289};	/*Property = CssMediaQueries*/
const Value V24343_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24343_292 = {S14, 0, &V24343_292_57183, &P24343_291};	/*Property = CssMinMax*/
const Value V24343_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24343_293 = {S15, 0, &V24343_293_57187, &P24343_292};	/*Property = CssOverflow*/
const Value V24343_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24343_294 = {S16, 0, &V24343_294_57189, &P24343_293};	/*Property = CssPosition*/
const Value V24343_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24343_295 = {S17, 0, &V24343_295_57192, &P24343_294};	/*Property = CssText*/
const Value V24343_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24343_297 = {S18, 0, &V24343_297_57195, &P24343_295};	/*Property = CssTransforms*/
const Value V24343_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24343_241 = {S19, 0, &V24343_241_55799, &P24343_297};	/*Property = CssTransitions*/
const Value V24343_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24343_298 = {S20, 0, &V24343_298_57206, &P24343_241};	/*Property = CssUI*/
const Value V24343_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24343_300 = {S21, 0, &V24343_300_57215, &P24343_298};	/*Property = DataSet*/
const Value V24343_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24343_301 = {S22, 0, &V24343_301_57218, &P24343_300};	/*Property = DataUrl*/
const Value V24343_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24343_302 = {S23, 0, &V24343_302_57222, &P24343_301};	/*Property = DeviceOrientation*/
const Value V24343_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24343_304 = {S24, 0, &V24343_304_57227, &P24343_302};	/*Property = FileReader*/
const Value V24343_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24343_305 = {S25, 0, &V24343_305_57231, &P24343_304};	/*Property = FileSaver*/
const Value V24343_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24343_306 = {S26, 0, &V24343_306_57237, &P24343_305};	/*Property = FileWriter*/
const Value V24343_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24343_307 = {S27, 0, &V24343_307_57239, &P24343_306};	/*Property = FormData*/
const Value V24343_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24343_309 = {S28, 0, &V24343_309_57246, &P24343_307};	/*Property = Fullscreen*/
const Value V24343_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24343_310 = {S29, 0, &V24343_310_57248, &P24343_309};	/*Property = GeoLocation*/
const Value V24343_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24343_312 = {S30, 0, &V24343_312_57254, &P24343_310};	/*Property = History*/
const Value V24343_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24343_314 = {S31, 0, &V24343_314_57260, &P24343_312};	/*Property = Html5*/
const Value V24343_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24343_313 = {S32, 0, &V24343_313_57257, &P24343_314};	/*Property = Html-Media-Capture*/
const Value V24343_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24343_315 = {S33, 0, &V24343_315_57264, &P24343_313};	/*Property = Iframe*/
const Value V24343_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24343_316 = {S34, 0, &V24343_316_57266, &P24343_315};	/*Property = IndexedDB*/
const Value V24343_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24343_317 = {S35, 0, &V24343_317_57269, &P24343_316};	/*Property = Json*/
const Value V24343_15_81502 = {S141, NULL};	/*Value = Blink*/
const Property P24343_15 = {S37, 0, &V24343_15_81502, &P24343_317};	/*Property = LayoutEngine*/
const Value V24343_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24343_344 = {S38, 0, &V24343_344_57272, &P24343_15};	/*Property = Masking*/
const Value V24343_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24343_320 = {S39, 0, &V24343_320_57290, &P24343_344};	/*Property = PostMessage*/
const Value V24343_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24343_321 = {S40, 0, &V24343_321_57293, &P24343_320};	/*Property = Progress*/
const Value V24343_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24343_322 = {S41, 0, &V24343_322_57296, &P24343_321};	/*Property = Prompts*/
const Value V24343_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24343_323 = {S42, 0, &V24343_323_57299, &P24343_322};	/*Property = Selector*/
const Value V24343_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24343_324 = {S43, 0, &V24343_324_57305, &P24343_323};	/*Property = Svg*/
const Value V24343_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24343_327 = {S44, 0, &V24343_327_57314, &P24343_324};	/*Property = TouchEvents*/
const Value V24343_328_57317 = {S2, NULL};	/*Value = True*/
const Property P24343_328 = {S45, 0, &V24343_328_57317, &P24343_327};	/*Property = Track*/
const Value V24343_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24343_329 = {S46, 0, &V24343_329_57323, &P24343_328};	/*Property = Video*/
const Value V24343_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24343_330 = {S47, 0, &V24343_330_57326, &P24343_329};	/*Property = Viewport*/
const Value V24343_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24343_332 = {S48, 0, &V24343_332_57338, &P24343_330};	/*Property = WebWorkers*/
const Value V24343_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24343_333 = {S49, 0, &V24343_333_57342, &P24343_332};	/*Property = Xhr2*/
const Profile P24343 = {"24343", &P24343_333};	/*Profile ID - 24343*/
const Value V23835_1_1 = {S2, NULL};	/*Value = True*/
const Property P23835_1 = {S50, 0, &V23835_1_1, NULL};	/*Property = IsMobile*/
const Value V23835_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23835_5 = {S52, 0, &V23835_5_1832, &P23835_1};	/*Property = ScreenPixelsHeight*/
const Value V23835_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23835_4 = {S54, 0, &V23835_4_5645, &P23835_5};	/*Property = ScreenPixelsWidth*/
const Profile P23835 = {"23835", &P23835_4};	/*Profile ID - 23835*/
const Value V23000_1_1 = {S2, NULL};	/*Value = True*/
const Property P23000_1 = {S50, 0, &V23000_1_1, NULL};	/*Property = IsMobile*/
const Value V23000_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23000_5 = {S52, 0, &V23000_5_1277, &P23000_1};	/*Property = ScreenPixelsHeight*/
const Value V23000_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23000_4 = {S54, 0, &V23000_4_1279, &P23000_5};	/*Property = ScreenPixelsWidth*/
const Profile P23000 = {"23000", &P23000_4};	/*Profile ID - 23000*/
const Value V24175_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24175_279 = {S1, 0, &V24175_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24175_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24175_282 = {S3, 0, &V24175_282_57154, &P24175_279};	/*Property = BlobBuilder*/
const Value V24175_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24175_350 = {S4, 0, &V24175_350_57444, &P24175_282};	/*Property = Canvas*/
const Value V24175_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24175_283 = {S5, 0, &V24175_283_57159, &P24175_350};	/*Property = CssBackground*/
const Value V24175_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24175_284 = {S6, 0, &V24175_284_57162, &P24175_283};	/*Property = CssBorderImage*/
const Value V24175_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24175_285 = {S7, 0, &V24175_285_57166, &P24175_284};	/*Property = CssCanvas*/
const Value V24175_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24175_286 = {S8, 0, &V24175_286_57168, &P24175_285};	/*Property = CssColor*/
const Value V24175_237_55784 = {S2, NULL};	/*Value = True*/
const Property P24175_237 = {S9, 0, &V24175_237_55784, &P24175_286};	/*Property = CssColumn*/
const Value V24175_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24175_287 = {S10, 0, &V24175_287_57172, &P24175_237};	/*Property = CssFlexbox*/
const Value V24175_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24175_288 = {S11, 0, &V24175_288_57174, &P24175_287};	/*Property = CssFont*/
const Value V24175_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24175_289 = {S12, 0, &V24175_289_57759, &P24175_288};	/*Property = CssImages*/
const Value V24175_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24175_291 = {S13, 0, &V24175_291_57180, &P24175_289};	/*Property = CssMediaQueries*/
const Value V24175_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24175_292 = {S14, 0, &V24175_292_57183, &P24175_291};	/*Property = CssMinMax*/
const Value V24175_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24175_293 = {S15, 0, &V24175_293_57187, &P24175_292};	/*Property = CssOverflow*/
const Value V24175_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24175_294 = {S16, 0, &V24175_294_57189, &P24175_293};	/*Property = CssPosition*/
const Value V24175_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24175_295 = {S17, 0, &V24175_295_57192, &P24175_294};	/*Property = CssText*/
const Value V24175_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24175_297 = {S18, 0, &V24175_297_57195, &P24175_295};	/*Property = CssTransforms*/
const Value V24175_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24175_241 = {S19, 0, &V24175_241_55799, &P24175_297};	/*Property = CssTransitions*/
const Value V24175_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24175_298 = {S20, 0, &V24175_298_57206, &P24175_241};	/*Property = CssUI*/
const Value V24175_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24175_300 = {S21, 0, &V24175_300_57215, &P24175_298};	/*Property = DataSet*/
const Value V24175_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24175_301 = {S22, 0, &V24175_301_57218, &P24175_300};	/*Property = DataUrl*/
const Value V24175_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24175_302 = {S23, 0, &V24175_302_57222, &P24175_301};	/*Property = DeviceOrientation*/
const Value V24175_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24175_304 = {S24, 0, &V24175_304_57227, &P24175_302};	/*Property = FileReader*/
const Value V24175_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24175_305 = {S25, 0, &V24175_305_57231, &P24175_304};	/*Property = FileSaver*/
const Value V24175_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24175_306 = {S26, 0, &V24175_306_57237, &P24175_305};	/*Property = FileWriter*/
const Value V24175_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24175_307 = {S27, 0, &V24175_307_57239, &P24175_306};	/*Property = FormData*/
const Value V24175_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24175_309 = {S28, 0, &V24175_309_57246, &P24175_307};	/*Property = Fullscreen*/
const Value V24175_310_57249 = {S0, NULL};	/*Value = False*/
const Property P24175_310 = {S29, 0, &V24175_310_57249, &P24175_309};	/*Property = GeoLocation*/
const Value V24175_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24175_312 = {S30, 0, &V24175_312_57254, &P24175_310};	/*Property = History*/
const Value V24175_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24175_314 = {S31, 0, &V24175_314_57260, &P24175_312};	/*Property = Html5*/
const Value V24175_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24175_313 = {S32, 0, &V24175_313_57258, &P24175_314};	/*Property = Html-Media-Capture*/
const Value V24175_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24175_315 = {S33, 0, &V24175_315_57264, &P24175_313};	/*Property = Iframe*/
const Value V24175_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24175_316 = {S34, 0, &V24175_316_57266, &P24175_315};	/*Property = IndexedDB*/
const Value V24175_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24175_317 = {S35, 0, &V24175_317_57269, &P24175_316};	/*Property = Json*/
const Value V24175_15_81502 = {S141, NULL};	/*Value = Blink*/
const Property P24175_15 = {S37, 0, &V24175_15_81502, &P24175_317};	/*Property = LayoutEngine*/
const Value V24175_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24175_344 = {S38, 0, &V24175_344_57272, &P24175_15};	/*Property = Masking*/
const Value V24175_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24175_320 = {S39, 0, &V24175_320_57290, &P24175_344};	/*Property = PostMessage*/
const Value V24175_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24175_321 = {S40, 0, &V24175_321_57293, &P24175_320};	/*Property = Progress*/
const Value V24175_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24175_322 = {S41, 0, &V24175_322_57296, &P24175_321};	/*Property = Prompts*/
const Value V24175_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24175_323 = {S42, 0, &V24175_323_57299, &P24175_322};	/*Property = Selector*/
const Value V24175_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24175_324 = {S43, 0, &V24175_324_57305, &P24175_323};	/*Property = Svg*/
const Value V24175_327_57315 = {S0, NULL};	/*Value = False*/
const Property P24175_327 = {S44, 0, &V24175_327_57315, &P24175_324};	/*Property = TouchEvents*/
const Value V24175_328_57317 = {S2, NULL};	/*Value = True*/
const Property P24175_328 = {S45, 0, &V24175_328_57317, &P24175_327};	/*Property = Track*/
const Value V24175_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24175_329 = {S46, 0, &V24175_329_57323, &P24175_328};	/*Property = Video*/
const Value V24175_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24175_330 = {S47, 0, &V24175_330_57326, &P24175_329};	/*Property = Viewport*/
const Value V24175_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24175_332 = {S48, 0, &V24175_332_57338, &P24175_330};	/*Property = WebWorkers*/
const Value V24175_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24175_333 = {S49, 0, &V24175_333_57342, &P24175_332};	/*Property = Xhr2*/
const Profile P24175 = {"24175", &P24175_333};	/*Profile ID - 24175*/
const Value V22778_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22778_279 = {S1, 0, &V22778_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22778_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22778_282 = {S3, 0, &V22778_282_57154, &P22778_279};	/*Property = BlobBuilder*/
const Value V22778_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22778_350 = {S4, 0, &V22778_350_57444, &P22778_282};	/*Property = Canvas*/
const Value V22778_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22778_283 = {S5, 0, &V22778_283_57159, &P22778_350};	/*Property = CssBackground*/
const Value V22778_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22778_284 = {S6, 0, &V22778_284_57162, &P22778_283};	/*Property = CssBorderImage*/
const Value V22778_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22778_285 = {S7, 0, &V22778_285_57166, &P22778_284};	/*Property = CssCanvas*/
const Value V22778_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22778_286 = {S8, 0, &V22778_286_57168, &P22778_285};	/*Property = CssColor*/
const Value V22778_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22778_237 = {S9, 0, &V22778_237_55786, &P22778_286};	/*Property = CssColumn*/
const Value V22778_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22778_287 = {S10, 0, &V22778_287_57172, &P22778_237};	/*Property = CssFlexbox*/
const Value V22778_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22778_288 = {S11, 0, &V22778_288_57174, &P22778_287};	/*Property = CssFont*/
const Value V22778_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22778_289 = {S12, 0, &V22778_289_57759, &P22778_288};	/*Property = CssImages*/
const Value V22778_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22778_291 = {S13, 0, &V22778_291_57180, &P22778_289};	/*Property = CssMediaQueries*/
const Value V22778_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22778_292 = {S14, 0, &V22778_292_57183, &P22778_291};	/*Property = CssMinMax*/
const Value V22778_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22778_293 = {S15, 0, &V22778_293_57187, &P22778_292};	/*Property = CssOverflow*/
const Value V22778_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22778_294 = {S16, 0, &V22778_294_57189, &P22778_293};	/*Property = CssPosition*/
const Value V22778_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22778_295 = {S17, 0, &V22778_295_57192, &P22778_294};	/*Property = CssText*/
const Value V22778_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22778_297 = {S18, 0, &V22778_297_57195, &P22778_295};	/*Property = CssTransforms*/
const Value V22778_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22778_241 = {S19, 0, &V22778_241_55799, &P22778_297};	/*Property = CssTransitions*/
const Value V22778_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22778_298 = {S20, 0, &V22778_298_57206, &P22778_241};	/*Property = CssUI*/
const Value V22778_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22778_300 = {S21, 0, &V22778_300_57215, &P22778_298};	/*Property = DataSet*/
const Value V22778_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22778_301 = {S22, 0, &V22778_301_57218, &P22778_300};	/*Property = DataUrl*/
const Value V22778_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22778_302 = {S23, 0, &V22778_302_57222, &P22778_301};	/*Property = DeviceOrientation*/
const Value V22778_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22778_304 = {S24, 0, &V22778_304_57227, &P22778_302};	/*Property = FileReader*/
const Value V22778_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22778_305 = {S25, 0, &V22778_305_57231, &P22778_304};	/*Property = FileSaver*/
const Value V22778_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22778_306 = {S26, 0, &V22778_306_57237, &P22778_305};	/*Property = FileWriter*/
const Value V22778_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22778_307 = {S27, 0, &V22778_307_57239, &P22778_306};	/*Property = FormData*/
const Value V22778_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22778_309 = {S28, 0, &V22778_309_57246, &P22778_307};	/*Property = Fullscreen*/
const Value V22778_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22778_310 = {S29, 0, &V22778_310_57248, &P22778_309};	/*Property = GeoLocation*/
const Value V22778_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22778_312 = {S30, 0, &V22778_312_57254, &P22778_310};	/*Property = History*/
const Value V22778_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22778_314 = {S31, 0, &V22778_314_57260, &P22778_312};	/*Property = Html5*/
const Value V22778_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22778_313 = {S32, 0, &V22778_313_57258, &P22778_314};	/*Property = Html-Media-Capture*/
const Value V22778_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22778_315 = {S33, 0, &V22778_315_57264, &P22778_313};	/*Property = Iframe*/
const Value V22778_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22778_316 = {S34, 0, &V22778_316_57267, &P22778_315};	/*Property = IndexedDB*/
const Value V22778_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22778_317 = {S35, 0, &V22778_317_57269, &P22778_316};	/*Property = Json*/
const Value V22778_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22778_15 = {S37, 0, &V22778_15_28, &P22778_317};	/*Property = LayoutEngine*/
const Value V22778_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22778_344 = {S38, 0, &V22778_344_57272, &P22778_15};	/*Property = Masking*/
const Value V22778_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22778_320 = {S39, 0, &V22778_320_57290, &P22778_344};	/*Property = PostMessage*/
const Value V22778_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22778_321 = {S40, 0, &V22778_321_57293, &P22778_320};	/*Property = Progress*/
const Value V22778_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22778_322 = {S41, 0, &V22778_322_57296, &P22778_321};	/*Property = Prompts*/
const Value V22778_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22778_323 = {S42, 0, &V22778_323_57299, &P22778_322};	/*Property = Selector*/
const Value V22778_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22778_324 = {S43, 0, &V22778_324_57305, &P22778_323};	/*Property = Svg*/
const Value V22778_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22778_327 = {S44, 0, &V22778_327_57315, &P22778_324};	/*Property = TouchEvents*/
const Value V22778_328_57317 = {S2, NULL};	/*Value = True*/
const Property P22778_328 = {S45, 0, &V22778_328_57317, &P22778_327};	/*Property = Track*/
const Value V22778_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22778_329 = {S46, 0, &V22778_329_57323, &P22778_328};	/*Property = Video*/
const Value V22778_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22778_330 = {S47, 0, &V22778_330_57326, &P22778_329};	/*Property = Viewport*/
const Value V22778_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22778_332 = {S48, 0, &V22778_332_57339, &P22778_330};	/*Property = WebWorkers*/
const Value V22778_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22778_333 = {S49, 0, &V22778_333_57342, &P22778_332};	/*Property = Xhr2*/
const Profile P22778 = {"22778", &P22778_333};	/*Profile ID - 22778*/
const Value V22827_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_279 = {S1, 0, &V22827_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22827_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_282 = {S3, 0, &V22827_282_57155, &P22827_279};	/*Property = BlobBuilder*/
const Value V22827_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_350 = {S4, 0, &V22827_350_57446, &P22827_282};	/*Property = Canvas*/
const Value V22827_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_283 = {S5, 0, &V22827_283_57161, &P22827_350};	/*Property = CssBackground*/
const Value V22827_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_284 = {S6, 0, &V22827_284_57164, &P22827_283};	/*Property = CssBorderImage*/
const Value V22827_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_285 = {S7, 0, &V22827_285_57167, &P22827_284};	/*Property = CssCanvas*/
const Value V22827_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_286 = {S8, 0, &V22827_286_57170, &P22827_285};	/*Property = CssColor*/
const Value V22827_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_237 = {S9, 0, &V22827_237_55786, &P22827_286};	/*Property = CssColumn*/
const Value V22827_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_287 = {S10, 0, &V22827_287_57173, &P22827_237};	/*Property = CssFlexbox*/
const Value V22827_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_288 = {S11, 0, &V22827_288_57176, &P22827_287};	/*Property = CssFont*/
const Value V22827_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_289 = {S12, 0, &V22827_289_86899, &P22827_288};	/*Property = CssImages*/
const Value V22827_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_291 = {S13, 0, &V22827_291_57182, &P22827_289};	/*Property = CssMediaQueries*/
const Value V22827_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_292 = {S14, 0, &V22827_292_57185, &P22827_291};	/*Property = CssMinMax*/
const Value V22827_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_293 = {S15, 0, &V22827_293_57188, &P22827_292};	/*Property = CssOverflow*/
const Value V22827_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_294 = {S16, 0, &V22827_294_57191, &P22827_293};	/*Property = CssPosition*/
const Value V22827_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_295 = {S17, 0, &V22827_295_57194, &P22827_294};	/*Property = CssText*/
const Value V22827_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_297 = {S18, 0, &V22827_297_57197, &P22827_295};	/*Property = CssTransforms*/
const Value V22827_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_241 = {S19, 0, &V22827_241_55801, &P22827_297};	/*Property = CssTransitions*/
const Value V22827_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_298 = {S20, 0, &V22827_298_57208, &P22827_241};	/*Property = CssUI*/
const Value V22827_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_300 = {S21, 0, &V22827_300_57217, &P22827_298};	/*Property = DataSet*/
const Value V22827_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_301 = {S22, 0, &V22827_301_57220, &P22827_300};	/*Property = DataUrl*/
const Value V22827_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_302 = {S23, 0, &V22827_302_57223, &P22827_301};	/*Property = DeviceOrientation*/
const Value V22827_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_304 = {S24, 0, &V22827_304_57229, &P22827_302};	/*Property = FileReader*/
const Value V22827_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_305 = {S25, 0, &V22827_305_57232, &P22827_304};	/*Property = FileSaver*/
const Value V22827_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_306 = {S26, 0, &V22827_306_57238, &P22827_305};	/*Property = FileWriter*/
const Value V22827_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_307 = {S27, 0, &V22827_307_57241, &P22827_306};	/*Property = FormData*/
const Value V22827_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_309 = {S28, 0, &V22827_309_57247, &P22827_307};	/*Property = Fullscreen*/
const Value V22827_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_310 = {S29, 0, &V22827_310_57250, &P22827_309};	/*Property = GeoLocation*/
const Value V22827_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_312 = {S30, 0, &V22827_312_57256, &P22827_310};	/*Property = History*/
const Value V22827_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_314 = {S31, 0, &V22827_314_57262, &P22827_312};	/*Property = Html5*/
const Value V22827_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_313 = {S32, 0, &V22827_313_57259, &P22827_314};	/*Property = Html-Media-Capture*/
const Value V22827_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_315 = {S33, 0, &V22827_315_57265, &P22827_313};	/*Property = Iframe*/
const Value V22827_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_316 = {S34, 0, &V22827_316_57268, &P22827_315};	/*Property = IndexedDB*/
const Value V22827_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_317 = {S35, 0, &V22827_317_57271, &P22827_316};	/*Property = Json*/
const Value V22827_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P22827_15 = {S37, 0, &V22827_15_30, &P22827_317};	/*Property = LayoutEngine*/
const Value V22827_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_344 = {S38, 0, &V22827_344_57274, &P22827_15};	/*Property = Masking*/
const Value V22827_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_320 = {S39, 0, &V22827_320_57292, &P22827_344};	/*Property = PostMessage*/
const Value V22827_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_321 = {S40, 0, &V22827_321_57295, &P22827_320};	/*Property = Progress*/
const Value V22827_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_322 = {S41, 0, &V22827_322_57298, &P22827_321};	/*Property = Prompts*/
const Value V22827_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_323 = {S42, 0, &V22827_323_57301, &P22827_322};	/*Property = Selector*/
const Value V22827_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_324 = {S43, 0, &V22827_324_57307, &P22827_323};	/*Property = Svg*/
const Value V22827_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_327 = {S44, 0, &V22827_327_57316, &P22827_324};	/*Property = TouchEvents*/
const Value V22827_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_328 = {S45, 0, &V22827_328_57319, &P22827_327};	/*Property = Track*/
const Value V22827_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_329 = {S46, 0, &V22827_329_57325, &P22827_328};	/*Property = Video*/
const Value V22827_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_330 = {S47, 0, &V22827_330_57328, &P22827_329};	/*Property = Viewport*/
const Value V22827_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_332 = {S48, 0, &V22827_332_57340, &P22827_330};	/*Property = WebWorkers*/
const Value V22827_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22827_333 = {S49, 0, &V22827_333_57343, &P22827_332};	/*Property = Xhr2*/
const Profile P22827 = {"22827", &P22827_333};	/*Profile ID - 22827*/
const Value V24209_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24209_279 = {S1, 0, &V24209_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24209_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24209_282 = {S3, 0, &V24209_282_57154, &P24209_279};	/*Property = BlobBuilder*/
const Value V24209_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24209_350 = {S4, 0, &V24209_350_57444, &P24209_282};	/*Property = Canvas*/
const Value V24209_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24209_283 = {S5, 0, &V24209_283_57159, &P24209_350};	/*Property = CssBackground*/
const Value V24209_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24209_284 = {S6, 0, &V24209_284_57162, &P24209_283};	/*Property = CssBorderImage*/
const Value V24209_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24209_285 = {S7, 0, &V24209_285_57166, &P24209_284};	/*Property = CssCanvas*/
const Value V24209_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24209_286 = {S8, 0, &V24209_286_57168, &P24209_285};	/*Property = CssColor*/
const Value V24209_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24209_237 = {S9, 0, &V24209_237_55786, &P24209_286};	/*Property = CssColumn*/
const Value V24209_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24209_287 = {S10, 0, &V24209_287_57172, &P24209_237};	/*Property = CssFlexbox*/
const Value V24209_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24209_288 = {S11, 0, &V24209_288_57174, &P24209_287};	/*Property = CssFont*/
const Value V24209_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24209_289 = {S12, 0, &V24209_289_57759, &P24209_288};	/*Property = CssImages*/
const Value V24209_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24209_291 = {S13, 0, &V24209_291_57180, &P24209_289};	/*Property = CssMediaQueries*/
const Value V24209_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24209_292 = {S14, 0, &V24209_292_57183, &P24209_291};	/*Property = CssMinMax*/
const Value V24209_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24209_293 = {S15, 0, &V24209_293_57187, &P24209_292};	/*Property = CssOverflow*/
const Value V24209_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24209_294 = {S16, 0, &V24209_294_57189, &P24209_293};	/*Property = CssPosition*/
const Value V24209_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24209_295 = {S17, 0, &V24209_295_57192, &P24209_294};	/*Property = CssText*/
const Value V24209_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24209_297 = {S18, 0, &V24209_297_57195, &P24209_295};	/*Property = CssTransforms*/
const Value V24209_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24209_241 = {S19, 0, &V24209_241_55799, &P24209_297};	/*Property = CssTransitions*/
const Value V24209_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24209_298 = {S20, 0, &V24209_298_57206, &P24209_241};	/*Property = CssUI*/
const Value V24209_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24209_300 = {S21, 0, &V24209_300_57215, &P24209_298};	/*Property = DataSet*/
const Value V24209_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24209_301 = {S22, 0, &V24209_301_57218, &P24209_300};	/*Property = DataUrl*/
const Value V24209_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24209_302 = {S23, 0, &V24209_302_57222, &P24209_301};	/*Property = DeviceOrientation*/
const Value V24209_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24209_304 = {S24, 0, &V24209_304_57227, &P24209_302};	/*Property = FileReader*/
const Value V24209_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24209_305 = {S25, 0, &V24209_305_57231, &P24209_304};	/*Property = FileSaver*/
const Value V24209_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24209_306 = {S26, 0, &V24209_306_57237, &P24209_305};	/*Property = FileWriter*/
const Value V24209_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24209_307 = {S27, 0, &V24209_307_57239, &P24209_306};	/*Property = FormData*/
const Value V24209_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24209_309 = {S28, 0, &V24209_309_57246, &P24209_307};	/*Property = Fullscreen*/
const Value V24209_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24209_310 = {S29, 0, &V24209_310_57248, &P24209_309};	/*Property = GeoLocation*/
const Value V24209_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24209_312 = {S30, 0, &V24209_312_57254, &P24209_310};	/*Property = History*/
const Value V24209_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24209_314 = {S31, 0, &V24209_314_57260, &P24209_312};	/*Property = Html5*/
const Value V24209_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24209_313 = {S32, 0, &V24209_313_57258, &P24209_314};	/*Property = Html-Media-Capture*/
const Value V24209_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24209_315 = {S33, 0, &V24209_315_57264, &P24209_313};	/*Property = Iframe*/
const Value V24209_316_57267 = {S0, NULL};	/*Value = False*/
const Property P24209_316 = {S34, 0, &V24209_316_57267, &P24209_315};	/*Property = IndexedDB*/
const Value V24209_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24209_317 = {S35, 0, &V24209_317_57269, &P24209_316};	/*Property = Json*/
const Value V24209_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24209_15 = {S37, 0, &V24209_15_28, &P24209_317};	/*Property = LayoutEngine*/
const Value V24209_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24209_344 = {S38, 0, &V24209_344_57272, &P24209_15};	/*Property = Masking*/
const Value V24209_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24209_320 = {S39, 0, &V24209_320_57290, &P24209_344};	/*Property = PostMessage*/
const Value V24209_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24209_321 = {S40, 0, &V24209_321_57293, &P24209_320};	/*Property = Progress*/
const Value V24209_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24209_322 = {S41, 0, &V24209_322_57296, &P24209_321};	/*Property = Prompts*/
const Value V24209_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24209_323 = {S42, 0, &V24209_323_57299, &P24209_322};	/*Property = Selector*/
const Value V24209_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24209_324 = {S43, 0, &V24209_324_57305, &P24209_323};	/*Property = Svg*/
const Value V24209_327_57315 = {S0, NULL};	/*Value = False*/
const Property P24209_327 = {S44, 0, &V24209_327_57315, &P24209_324};	/*Property = TouchEvents*/
const Value V24209_328_57317 = {S2, NULL};	/*Value = True*/
const Property P24209_328 = {S45, 0, &V24209_328_57317, &P24209_327};	/*Property = Track*/
const Value V24209_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24209_329 = {S46, 0, &V24209_329_57323, &P24209_328};	/*Property = Video*/
const Value V24209_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24209_330 = {S47, 0, &V24209_330_57326, &P24209_329};	/*Property = Viewport*/
const Value V24209_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24209_332 = {S48, 0, &V24209_332_57339, &P24209_330};	/*Property = WebWorkers*/
const Value V24209_333_57341 = {S2, NULL};	/*Value = True*/
const Property P24209_333 = {S49, 0, &V24209_333_57341, &P24209_332};	/*Property = Xhr2*/
const Profile P24209 = {"24209", &P24209_333};	/*Profile ID - 24209*/
const Value V22769_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22769_279 = {S1, 0, &V22769_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22769_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22769_282 = {S3, 0, &V22769_282_57154, &P22769_279};	/*Property = BlobBuilder*/
const Value V22769_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22769_350 = {S4, 0, &V22769_350_57444, &P22769_282};	/*Property = Canvas*/
const Value V22769_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22769_283 = {S5, 0, &V22769_283_57159, &P22769_350};	/*Property = CssBackground*/
const Value V22769_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22769_284 = {S6, 0, &V22769_284_57163, &P22769_283};	/*Property = CssBorderImage*/
const Value V22769_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22769_285 = {S7, 0, &V22769_285_57166, &P22769_284};	/*Property = CssCanvas*/
const Value V22769_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22769_286 = {S8, 0, &V22769_286_57168, &P22769_285};	/*Property = CssColor*/
const Value V22769_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22769_237 = {S9, 0, &V22769_237_55786, &P22769_286};	/*Property = CssColumn*/
const Value V22769_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22769_287 = {S10, 0, &V22769_287_57172, &P22769_237};	/*Property = CssFlexbox*/
const Value V22769_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22769_288 = {S11, 0, &V22769_288_57174, &P22769_287};	/*Property = CssFont*/
const Value V22769_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22769_289 = {S12, 0, &V22769_289_57759, &P22769_288};	/*Property = CssImages*/
const Value V22769_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22769_291 = {S13, 0, &V22769_291_57181, &P22769_289};	/*Property = CssMediaQueries*/
const Value V22769_292_57184 = {S0, NULL};	/*Value = False*/
const Property P22769_292 = {S14, 0, &V22769_292_57184, &P22769_291};	/*Property = CssMinMax*/
const Value V22769_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22769_293 = {S15, 0, &V22769_293_57187, &P22769_292};	/*Property = CssOverflow*/
const Value V22769_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22769_294 = {S16, 0, &V22769_294_57189, &P22769_293};	/*Property = CssPosition*/
const Value V22769_295_57193 = {S0, NULL};	/*Value = False*/
const Property P22769_295 = {S17, 0, &V22769_295_57193, &P22769_294};	/*Property = CssText*/
const Value V22769_297_57196 = {S0, NULL};	/*Value = False*/
const Property P22769_297 = {S18, 0, &V22769_297_57196, &P22769_295};	/*Property = CssTransforms*/
const Value V22769_241_55800 = {S0, NULL};	/*Value = False*/
const Property P22769_241 = {S19, 0, &V22769_241_55800, &P22769_297};	/*Property = CssTransitions*/
const Value V22769_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22769_298 = {S20, 0, &V22769_298_57206, &P22769_241};	/*Property = CssUI*/
const Value V22769_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22769_300 = {S21, 0, &V22769_300_57216, &P22769_298};	/*Property = DataSet*/
const Value V22769_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22769_301 = {S22, 0, &V22769_301_57218, &P22769_300};	/*Property = DataUrl*/
const Value V22769_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22769_302 = {S23, 0, &V22769_302_57222, &P22769_301};	/*Property = DeviceOrientation*/
const Value V22769_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22769_304 = {S24, 0, &V22769_304_57228, &P22769_302};	/*Property = FileReader*/
const Value V22769_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22769_305 = {S25, 0, &V22769_305_57231, &P22769_304};	/*Property = FileSaver*/
const Value V22769_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22769_306 = {S26, 0, &V22769_306_57237, &P22769_305};	/*Property = FileWriter*/
const Value V22769_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22769_307 = {S27, 0, &V22769_307_57240, &P22769_306};	/*Property = FormData*/
const Value V22769_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22769_309 = {S28, 0, &V22769_309_57246, &P22769_307};	/*Property = Fullscreen*/
const Value V22769_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22769_310 = {S29, 0, &V22769_310_57248, &P22769_309};	/*Property = GeoLocation*/
const Value V22769_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22769_312 = {S30, 0, &V22769_312_57255, &P22769_310};	/*Property = History*/
const Value V22769_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22769_314 = {S31, 0, &V22769_314_57261, &P22769_312};	/*Property = Html5*/
const Value V22769_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22769_313 = {S32, 0, &V22769_313_57258, &P22769_314};	/*Property = Html-Media-Capture*/
const Value V22769_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22769_315 = {S33, 0, &V22769_315_57264, &P22769_313};	/*Property = Iframe*/
const Value V22769_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22769_316 = {S34, 0, &V22769_316_57267, &P22769_315};	/*Property = IndexedDB*/
const Value V22769_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22769_317 = {S35, 0, &V22769_317_57269, &P22769_316};	/*Property = Json*/
const Value V22769_15_81502 = {S141, NULL};	/*Value = Blink*/
const Property P22769_15 = {S37, 0, &V22769_15_81502, &P22769_317};	/*Property = LayoutEngine*/
const Value V22769_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22769_344 = {S38, 0, &V22769_344_57272, &P22769_15};	/*Property = Masking*/
const Value V22769_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22769_320 = {S39, 0, &V22769_320_57290, &P22769_344};	/*Property = PostMessage*/
const Value V22769_321_57294 = {S0, NULL};	/*Value = False*/
const Property P22769_321 = {S40, 0, &V22769_321_57294, &P22769_320};	/*Property = Progress*/
const Value V22769_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22769_322 = {S41, 0, &V22769_322_57296, &P22769_321};	/*Property = Prompts*/
const Value V22769_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22769_323 = {S42, 0, &V22769_323_57299, &P22769_322};	/*Property = Selector*/
const Value V22769_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22769_324 = {S43, 0, &V22769_324_57305, &P22769_323};	/*Property = Svg*/
const Value V22769_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22769_327 = {S44, 0, &V22769_327_57315, &P22769_324};	/*Property = TouchEvents*/
const Value V22769_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22769_328 = {S45, 0, &V22769_328_57318, &P22769_327};	/*Property = Track*/
const Value V22769_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22769_329 = {S46, 0, &V22769_329_57323, &P22769_328};	/*Property = Video*/
const Value V22769_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22769_330 = {S47, 0, &V22769_330_57326, &P22769_329};	/*Property = Viewport*/
const Value V22769_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22769_332 = {S48, 0, &V22769_332_57339, &P22769_330};	/*Property = WebWorkers*/
const Value V22769_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22769_333 = {S49, 0, &V22769_333_57342, &P22769_332};	/*Property = Xhr2*/
const Profile P22769 = {"22769", &P22769_333};	/*Profile ID - 22769*/
const Value V22768_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_279 = {S1, 0, &V22768_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V22768_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_282 = {S3, 0, &V22768_282_57155, &P22768_279};	/*Property = BlobBuilder*/
const Value V22768_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_350 = {S4, 0, &V22768_350_57446, &P22768_282};	/*Property = Canvas*/
const Value V22768_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_283 = {S5, 0, &V22768_283_57161, &P22768_350};	/*Property = CssBackground*/
const Value V22768_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_284 = {S6, 0, &V22768_284_57164, &P22768_283};	/*Property = CssBorderImage*/
const Value V22768_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_285 = {S7, 0, &V22768_285_57167, &P22768_284};	/*Property = CssCanvas*/
const Value V22768_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_286 = {S8, 0, &V22768_286_57170, &P22768_285};	/*Property = CssColor*/
const Value V22768_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_237 = {S9, 0, &V22768_237_55786, &P22768_286};	/*Property = CssColumn*/
const Value V22768_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_287 = {S10, 0, &V22768_287_57173, &P22768_237};	/*Property = CssFlexbox*/
const Value V22768_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_288 = {S11, 0, &V22768_288_57176, &P22768_287};	/*Property = CssFont*/
const Value V22768_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_289 = {S12, 0, &V22768_289_86899, &P22768_288};	/*Property = CssImages*/
const Value V22768_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_291 = {S13, 0, &V22768_291_57182, &P22768_289};	/*Property = CssMediaQueries*/
const Value V22768_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_292 = {S14, 0, &V22768_292_57185, &P22768_291};	/*Property = CssMinMax*/
const Value V22768_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_293 = {S15, 0, &V22768_293_57188, &P22768_292};	/*Property = CssOverflow*/
const Value V22768_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_294 = {S16, 0, &V22768_294_57191, &P22768_293};	/*Property = CssPosition*/
const Value V22768_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_295 = {S17, 0, &V22768_295_57194, &P22768_294};	/*Property = CssText*/
const Value V22768_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_297 = {S18, 0, &V22768_297_57197, &P22768_295};	/*Property = CssTransforms*/
const Value V22768_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_241 = {S19, 0, &V22768_241_55801, &P22768_297};	/*Property = CssTransitions*/
const Value V22768_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_298 = {S20, 0, &V22768_298_57208, &P22768_241};	/*Property = CssUI*/
const Value V22768_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_300 = {S21, 0, &V22768_300_57217, &P22768_298};	/*Property = DataSet*/
const Value V22768_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_301 = {S22, 0, &V22768_301_57220, &P22768_300};	/*Property = DataUrl*/
const Value V22768_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_302 = {S23, 0, &V22768_302_57223, &P22768_301};	/*Property = DeviceOrientation*/
const Value V22768_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_304 = {S24, 0, &V22768_304_57229, &P22768_302};	/*Property = FileReader*/
const Value V22768_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_305 = {S25, 0, &V22768_305_57232, &P22768_304};	/*Property = FileSaver*/
const Value V22768_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_306 = {S26, 0, &V22768_306_57238, &P22768_305};	/*Property = FileWriter*/
const Value V22768_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_307 = {S27, 0, &V22768_307_57241, &P22768_306};	/*Property = FormData*/
const Value V22768_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_309 = {S28, 0, &V22768_309_57247, &P22768_307};	/*Property = Fullscreen*/
const Value V22768_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_310 = {S29, 0, &V22768_310_57250, &P22768_309};	/*Property = GeoLocation*/
const Value V22768_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_312 = {S30, 0, &V22768_312_57256, &P22768_310};	/*Property = History*/
const Value V22768_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_314 = {S31, 0, &V22768_314_57262, &P22768_312};	/*Property = Html5*/
const Value V22768_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_313 = {S32, 0, &V22768_313_57259, &P22768_314};	/*Property = Html-Media-Capture*/
const Value V22768_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_315 = {S33, 0, &V22768_315_57265, &P22768_313};	/*Property = Iframe*/
const Value V22768_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_316 = {S34, 0, &V22768_316_57268, &P22768_315};	/*Property = IndexedDB*/
const Value V22768_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_317 = {S35, 0, &V22768_317_57271, &P22768_316};	/*Property = Json*/
const Value V22768_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P22768_15 = {S37, 0, &V22768_15_29, &P22768_317};	/*Property = LayoutEngine*/
const Value V22768_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_344 = {S38, 0, &V22768_344_57274, &P22768_15};	/*Property = Masking*/
const Value V22768_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_320 = {S39, 0, &V22768_320_57292, &P22768_344};	/*Property = PostMessage*/
const Value V22768_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_321 = {S40, 0, &V22768_321_57295, &P22768_320};	/*Property = Progress*/
const Value V22768_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_322 = {S41, 0, &V22768_322_57298, &P22768_321};	/*Property = Prompts*/
const Value V22768_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_323 = {S42, 0, &V22768_323_57301, &P22768_322};	/*Property = Selector*/
const Value V22768_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_324 = {S43, 0, &V22768_324_57307, &P22768_323};	/*Property = Svg*/
const Value V22768_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_327 = {S44, 0, &V22768_327_57316, &P22768_324};	/*Property = TouchEvents*/
const Value V22768_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_328 = {S45, 0, &V22768_328_57319, &P22768_327};	/*Property = Track*/
const Value V22768_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_329 = {S46, 0, &V22768_329_57325, &P22768_328};	/*Property = Video*/
const Value V22768_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_330 = {S47, 0, &V22768_330_57328, &P22768_329};	/*Property = Viewport*/
const Value V22768_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_332 = {S48, 0, &V22768_332_57340, &P22768_330};	/*Property = WebWorkers*/
const Value V22768_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P22768_333 = {S49, 0, &V22768_333_57343, &P22768_332};	/*Property = Xhr2*/
const Profile P22768 = {"22768", &P22768_333};	/*Profile ID - 22768*/
const Value V22785_1_1 = {S2, NULL};	/*Value = True*/
const Property P22785_1 = {S50, 0, &V22785_1_1, NULL};	/*Property = IsMobile*/
const Value V22785_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22785_5 = {S52, 0, &V22785_5_1403, &P22785_1};	/*Property = ScreenPixelsHeight*/
const Value V22785_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22785_4 = {S54, 0, &V22785_4_1404, &P22785_5};	/*Property = ScreenPixelsWidth*/
const Profile P22785 = {"22785", &P22785_4};	/*Profile ID - 22785*/
const Profile P24184 = {"24184", NULL};	/*Profile ID - 24184*/
const Value V23341_1_1 = {S2, NULL};	/*Value = True*/
const Property P23341_1 = {S50, 0, &V23341_1_1, NULL};	/*Property = IsMobile*/
const Value V23341_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23341_5 = {S52, 0, &V23341_5_37569, &P23341_1};	/*Property = ScreenPixelsHeight*/
const Value V23341_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23341_4 = {S54, 0, &V23341_4_5645, &P23341_5};	/*Property = ScreenPixelsWidth*/
const Profile P23341 = {"23341", &P23341_4};	/*Profile ID - 23341*/
const Value V23762_1_1 = {S2, NULL};	/*Value = True*/
const Property P23762_1 = {S50, 0, &V23762_1_1, NULL};	/*Property = IsMobile*/
const Value V23762_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23762_5 = {S52, 0, &V23762_5_1227, &P23762_1};	/*Property = ScreenPixelsHeight*/
const Value V23762_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23762_4 = {S54, 0, &V23762_4_1229, &P23762_5};	/*Property = ScreenPixelsWidth*/
const Profile P23762 = {"23762", &P23762_4};	/*Profile ID - 23762*/
const Value V23564_1_1 = {S2, NULL};	/*Value = True*/
const Property P23564_1 = {S50, 0, &V23564_1_1, NULL};	/*Property = IsMobile*/
const Value V23564_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23564_5 = {S52, 0, &V23564_5_1227, &P23564_1};	/*Property = ScreenPixelsHeight*/
const Value V23564_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23564_4 = {S54, 0, &V23564_4_1229, &P23564_5};	/*Property = ScreenPixelsWidth*/
const Profile P23564 = {"23564", &P23564_4};	/*Profile ID - 23564*/
const Value V18613_1_1 = {S2, NULL};	/*Value = True*/
const Property P18613_1 = {S50, 0, &V18613_1_1, NULL};	/*Property = IsMobile*/
const Value V18613_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18613_5 = {S52, 0, &V18613_5_1227, &P18613_1};	/*Property = ScreenPixelsHeight*/
const Value V18613_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18613_4 = {S54, 0, &V18613_4_1229, &P18613_5};	/*Property = ScreenPixelsWidth*/
const Profile P18613 = {"18613", &P18613_4};	/*Profile ID - 18613*/
const Value V22956_1_1 = {S2, NULL};	/*Value = True*/
const Property P22956_1 = {S50, 0, &V22956_1_1, NULL};	/*Property = IsMobile*/
const Value V22956_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P22956_5 = {S52, 0, &V22956_5_1403, &P22956_1};	/*Property = ScreenPixelsHeight*/
const Value V22956_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P22956_4 = {S54, 0, &V22956_4_1404, &P22956_5};	/*Property = ScreenPixelsWidth*/
const Profile P22956 = {"22956", &P22956_4};	/*Profile ID - 22956*/
const Value V23533_1_1 = {S2, NULL};	/*Value = True*/
const Property P23533_1 = {S50, 0, &V23533_1_1, NULL};	/*Property = IsMobile*/
const Value V23533_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23533_5 = {S52, 0, &V23533_5_1227, &P23533_1};	/*Property = ScreenPixelsHeight*/
const Value V23533_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23533_4 = {S54, 0, &V23533_4_1229, &P23533_5};	/*Property = ScreenPixelsWidth*/
const Profile P23533 = {"23533", &P23533_4};	/*Profile ID - 23533*/
const Value V23244_1_1 = {S2, NULL};	/*Value = True*/
const Property P23244_1 = {S50, 0, &V23244_1_1, NULL};	/*Property = IsMobile*/
const Value V23244_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23244_5 = {S52, 0, &V23244_5_1403, &P23244_1};	/*Property = ScreenPixelsHeight*/
const Value V23244_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23244_4 = {S54, 0, &V23244_4_1404, &P23244_5};	/*Property = ScreenPixelsWidth*/
const Profile P23244 = {"23244", &P23244_4};	/*Profile ID - 23244*/
const Value V23726_1_1 = {S2, NULL};	/*Value = True*/
const Property P23726_1 = {S50, 0, &V23726_1_1, NULL};	/*Property = IsMobile*/
const Value V23726_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23726_5 = {S52, 0, &V23726_5_1403, &P23726_1};	/*Property = ScreenPixelsHeight*/
const Value V23726_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23726_4 = {S54, 0, &V23726_4_1404, &P23726_5};	/*Property = ScreenPixelsWidth*/
const Profile P23726 = {"23726", &P23726_4};	/*Profile ID - 23726*/
const Value V23325_1_1 = {S2, NULL};	/*Value = True*/
const Property P23325_1 = {S50, 0, &V23325_1_1, NULL};	/*Property = IsMobile*/
const Value V23325_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23325_5 = {S52, 0, &V23325_5_37569, &P23325_1};	/*Property = ScreenPixelsHeight*/
const Value V23325_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23325_4 = {S54, 0, &V23325_4_5645, &P23325_5};	/*Property = ScreenPixelsWidth*/
const Profile P23325 = {"23325", &P23325_4};	/*Profile ID - 23325*/
const Value V23240_1_1 = {S2, NULL};	/*Value = True*/
const Property P23240_1 = {S50, 0, &V23240_1_1, NULL};	/*Property = IsMobile*/
const Value V23240_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23240_5 = {S52, 0, &V23240_5_1227, &P23240_1};	/*Property = ScreenPixelsHeight*/
const Value V23240_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23240_4 = {S54, 0, &V23240_4_1229, &P23240_5};	/*Property = ScreenPixelsWidth*/
const Profile P23240 = {"23240", &P23240_4};	/*Profile ID - 23240*/
const Value V23036_1_1 = {S2, NULL};	/*Value = True*/
const Property P23036_1 = {S50, 0, &V23036_1_1, NULL};	/*Property = IsMobile*/
const Value V23036_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23036_5 = {S52, 0, &V23036_5_1227, &P23036_1};	/*Property = ScreenPixelsHeight*/
const Value V23036_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23036_4 = {S54, 0, &V23036_4_1229, &P23036_5};	/*Property = ScreenPixelsWidth*/
const Profile P23036 = {"23036", &P23036_4};	/*Profile ID - 23036*/
const Value V23172_1_1 = {S2, NULL};	/*Value = True*/
const Property P23172_1 = {S50, 0, &V23172_1_1, NULL};	/*Property = IsMobile*/
const Value V23172_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23172_5 = {S52, 0, &V23172_5_1277, &P23172_1};	/*Property = ScreenPixelsHeight*/
const Value V23172_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23172_4 = {S54, 0, &V23172_4_2336, &P23172_5};	/*Property = ScreenPixelsWidth*/
const Profile P23172 = {"23172", &P23172_4};	/*Profile ID - 23172*/
const Value V23895_1_1 = {S2, NULL};	/*Value = True*/
const Property P23895_1 = {S50, 0, &V23895_1_1, NULL};	/*Property = IsMobile*/
const Value V23895_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23895_5 = {S52, 0, &V23895_5_9981, &P23895_1};	/*Property = ScreenPixelsHeight*/
const Value V23895_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23895_4 = {S54, 0, &V23895_4_2336, &P23895_5};	/*Property = ScreenPixelsWidth*/
const Profile P23895 = {"23895", &P23895_4};	/*Profile ID - 23895*/
const Value V23294_1_1 = {S2, NULL};	/*Value = True*/
const Property P23294_1 = {S50, 0, &V23294_1_1, NULL};	/*Property = IsMobile*/
const Value V23294_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23294_5 = {S52, 0, &V23294_5_1269, &P23294_1};	/*Property = ScreenPixelsHeight*/
const Value V23294_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23294_4 = {S54, 0, &V23294_4_1229, &P23294_5};	/*Property = ScreenPixelsWidth*/
const Profile P23294 = {"23294", &P23294_4};	/*Profile ID - 23294*/
const Value V23142_1_1 = {S2, NULL};	/*Value = True*/
const Property P23142_1 = {S50, 0, &V23142_1_1, NULL};	/*Property = IsMobile*/
const Value V23142_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23142_5 = {S52, 0, &V23142_5_1277, &P23142_1};	/*Property = ScreenPixelsHeight*/
const Value V23142_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23142_4 = {S54, 0, &V23142_4_1279, &P23142_5};	/*Property = ScreenPixelsWidth*/
const Profile P23142 = {"23142", &P23142_4};	/*Profile ID - 23142*/
const Value V22982_1_1 = {S2, NULL};	/*Value = True*/
const Property P22982_1 = {S50, 0, &V22982_1_1, NULL};	/*Property = IsMobile*/
const Value V22982_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22982_5 = {S52, 0, &V22982_5_1458, &P22982_1};	/*Property = ScreenPixelsHeight*/
const Value V22982_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P22982_4 = {S54, 0, &V22982_4_37617, &P22982_5};	/*Property = ScreenPixelsWidth*/
const Profile P22982 = {"22982", &P22982_4};	/*Profile ID - 22982*/
const Value V23577_1_1 = {S2, NULL};	/*Value = True*/
const Property P23577_1 = {S50, 0, &V23577_1_1, NULL};	/*Property = IsMobile*/
const Value V23577_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23577_5 = {S52, 0, &V23577_5_1227, &P23577_1};	/*Property = ScreenPixelsHeight*/
const Value V23577_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23577_4 = {S54, 0, &V23577_4_1229, &P23577_5};	/*Property = ScreenPixelsWidth*/
const Profile P23577 = {"23577", &P23577_4};	/*Profile ID - 23577*/
const Value V24073_1_1 = {S2, NULL};	/*Value = True*/
const Property P24073_1 = {S50, 0, &V24073_1_1, NULL};	/*Property = IsMobile*/
const Value V24073_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24073_5 = {S52, 0, &V24073_5_9981, &P24073_1};	/*Property = ScreenPixelsHeight*/
const Value V24073_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24073_4 = {S54, 0, &V24073_4_39098, &P24073_5};	/*Property = ScreenPixelsWidth*/
const Profile P24073 = {"24073", &P24073_4};	/*Profile ID - 24073*/
const Value V22917_1_1 = {S2, NULL};	/*Value = True*/
const Property P22917_1 = {S50, 0, &V22917_1_1, NULL};	/*Property = IsMobile*/
const Value V22917_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22917_5 = {S52, 0, &V22917_5_1277, &P22917_1};	/*Property = ScreenPixelsHeight*/
const Value V22917_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22917_4 = {S54, 0, &V22917_4_1279, &P22917_5};	/*Property = ScreenPixelsWidth*/
const Profile P22917 = {"22917", &P22917_4};	/*Profile ID - 22917*/
const Value V22967_1_1 = {S2, NULL};	/*Value = True*/
const Property P22967_1 = {S50, 0, &V22967_1_1, NULL};	/*Property = IsMobile*/
const Value V22967_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22967_5 = {S52, 0, &V22967_5_1227, &P22967_1};	/*Property = ScreenPixelsHeight*/
const Value V22967_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22967_4 = {S54, 0, &V22967_4_1229, &P22967_5};	/*Property = ScreenPixelsWidth*/
const Profile P22967 = {"22967", &P22967_4};	/*Profile ID - 22967*/
const Value V22852_1_1 = {S2, NULL};	/*Value = True*/
const Property P22852_1 = {S50, 0, &V22852_1_1, NULL};	/*Property = IsMobile*/
const Value V22852_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22852_5 = {S52, 0, &V22852_5_1227, &P22852_1};	/*Property = ScreenPixelsHeight*/
const Value V22852_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22852_4 = {S54, 0, &V22852_4_1229, &P22852_5};	/*Property = ScreenPixelsWidth*/
const Profile P22852 = {"22852", &P22852_4};	/*Profile ID - 22852*/
const Value V23145_1_1 = {S2, NULL};	/*Value = True*/
const Property P23145_1 = {S50, 0, &V23145_1_1, NULL};	/*Property = IsMobile*/
const Value V23145_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23145_5 = {S52, 0, &V23145_5_1227, &P23145_1};	/*Property = ScreenPixelsHeight*/
const Value V23145_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23145_4 = {S54, 0, &V23145_4_1229, &P23145_5};	/*Property = ScreenPixelsWidth*/
const Profile P23145 = {"23145", &P23145_4};	/*Profile ID - 23145*/
const Value V23218_1_1 = {S2, NULL};	/*Value = True*/
const Property P23218_1 = {S50, 0, &V23218_1_1, NULL};	/*Property = IsMobile*/
const Value V23218_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23218_5 = {S52, 0, &V23218_5_37569, &P23218_1};	/*Property = ScreenPixelsHeight*/
const Value V23218_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23218_4 = {S54, 0, &V23218_4_5645, &P23218_5};	/*Property = ScreenPixelsWidth*/
const Profile P23218 = {"23218", &P23218_4};	/*Profile ID - 23218*/
const Value V23104_1_1 = {S2, NULL};	/*Value = True*/
const Property P23104_1 = {S50, 0, &V23104_1_1, NULL};	/*Property = IsMobile*/
const Value V23104_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23104_5 = {S52, 0, &V23104_5_1269, &P23104_1};	/*Property = ScreenPixelsHeight*/
const Value V23104_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23104_4 = {S54, 0, &V23104_4_1229, &P23104_5};	/*Property = ScreenPixelsWidth*/
const Profile P23104 = {"23104", &P23104_4};	/*Profile ID - 23104*/
const Value V23116_1_1 = {S2, NULL};	/*Value = True*/
const Property P23116_1 = {S50, 0, &V23116_1_1, NULL};	/*Property = IsMobile*/
const Value V23116_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23116_5 = {S52, 0, &V23116_5_1277, &P23116_1};	/*Property = ScreenPixelsHeight*/
const Value V23116_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23116_4 = {S54, 0, &V23116_4_1279, &P23116_5};	/*Property = ScreenPixelsWidth*/
const Profile P23116 = {"23116", &P23116_4};	/*Profile ID - 23116*/
const Value V23179_1_1 = {S2, NULL};	/*Value = True*/
const Property P23179_1 = {S50, 0, &V23179_1_1, NULL};	/*Property = IsMobile*/
const Value V23179_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P23179_5 = {S52, 0, &V23179_5_59138, &P23179_1};	/*Property = ScreenPixelsHeight*/
const Value V23179_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P23179_4 = {S54, 0, &V23179_4_59970, &P23179_5};	/*Property = ScreenPixelsWidth*/
const Profile P23179 = {"23179", &P23179_4};	/*Profile ID - 23179*/
const Value V22861_1_1 = {S2, NULL};	/*Value = True*/
const Property P22861_1 = {S50, 0, &V22861_1_1, NULL};	/*Property = IsMobile*/
const Value V22861_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22861_5 = {S52, 0, &V22861_5_1277, &P22861_1};	/*Property = ScreenPixelsHeight*/
const Value V22861_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22861_4 = {S54, 0, &V22861_4_1279, &P22861_5};	/*Property = ScreenPixelsWidth*/
const Profile P22861 = {"22861", &P22861_4};	/*Profile ID - 22861*/
const Value V22984_1_1 = {S2, NULL};	/*Value = True*/
const Property P22984_1 = {S50, 0, &V22984_1_1, NULL};	/*Property = IsMobile*/
const Value V22984_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22984_5 = {S52, 0, &V22984_5_1227, &P22984_1};	/*Property = ScreenPixelsHeight*/
const Value V22984_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22984_4 = {S54, 0, &V22984_4_1229, &P22984_5};	/*Property = ScreenPixelsWidth*/
const Profile P22984 = {"22984", &P22984_4};	/*Profile ID - 22984*/
const Value V23408_1_1 = {S2, NULL};	/*Value = True*/
const Property P23408_1 = {S50, 0, &V23408_1_1, NULL};	/*Property = IsMobile*/
const Value V23408_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23408_5 = {S52, 0, &V23408_5_37, &P23408_1};	/*Property = ScreenPixelsHeight*/
const Value V23408_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23408_4 = {S54, 0, &V23408_4_36, &P23408_5};	/*Property = ScreenPixelsWidth*/
const Profile P23408 = {"23408", &P23408_4};	/*Profile ID - 23408*/
const Value V24070_1_1 = {S2, NULL};	/*Value = True*/
const Property P24070_1 = {S50, 0, &V24070_1_1, NULL};	/*Property = IsMobile*/
const Value V24070_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24070_5 = {S52, 0, &V24070_5_1227, &P24070_1};	/*Property = ScreenPixelsHeight*/
const Value V24070_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24070_4 = {S54, 0, &V24070_4_1229, &P24070_5};	/*Property = ScreenPixelsWidth*/
const Profile P24070 = {"24070", &P24070_4};	/*Profile ID - 24070*/
const Value V22996_1_1 = {S2, NULL};	/*Value = True*/
const Property P22996_1 = {S50, 0, &V22996_1_1, NULL};	/*Property = IsMobile*/
const Value V22996_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P22996_5 = {S52, 0, &V22996_5_1269, &P22996_1};	/*Property = ScreenPixelsHeight*/
const Value V22996_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22996_4 = {S54, 0, &V22996_4_1229, &P22996_5};	/*Property = ScreenPixelsWidth*/
const Profile P22996 = {"22996", &P22996_4};	/*Profile ID - 22996*/
const Value V23121_1_1 = {S2, NULL};	/*Value = True*/
const Property P23121_1 = {S50, 0, &V23121_1_1, NULL};	/*Property = IsMobile*/
const Value V23121_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P23121_5 = {S52, 0, &V23121_5_1275, &P23121_1};	/*Property = ScreenPixelsHeight*/
const Value V23121_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23121_4 = {S54, 0, &V23121_4_1229, &P23121_5};	/*Property = ScreenPixelsWidth*/
const Profile P23121 = {"23121", &P23121_4};	/*Profile ID - 23121*/
const Value V24509_1_1 = {S2, NULL};	/*Value = True*/
const Property P24509_1 = {S50, 0, &V24509_1_1, NULL};	/*Property = IsMobile*/
const Value V24509_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P24509_5 = {S52, 0, &V24509_5_37569, &P24509_1};	/*Property = ScreenPixelsHeight*/
const Value V24509_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P24509_4 = {S54, 0, &V24509_4_5645, &P24509_5};	/*Property = ScreenPixelsWidth*/
const Profile P24509 = {"24509", &P24509_4};	/*Profile ID - 24509*/
const Value V22920_1_1 = {S2, NULL};	/*Value = True*/
const Property P22920_1 = {S50, 0, &V22920_1_1, NULL};	/*Property = IsMobile*/
const Value V22920_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22920_5 = {S52, 0, &V22920_5_9981, &P22920_1};	/*Property = ScreenPixelsHeight*/
const Value V22920_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P22920_4 = {S54, 0, &V22920_4_2336, &P22920_5};	/*Property = ScreenPixelsWidth*/
const Profile P22920 = {"22920", &P22920_4};	/*Profile ID - 22920*/
const Value V23721_1_1 = {S2, NULL};	/*Value = True*/
const Property P23721_1 = {S50, 0, &V23721_1_1, NULL};	/*Property = IsMobile*/
const Value V23721_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23721_5 = {S52, 0, &V23721_5_1227, &P23721_1};	/*Property = ScreenPixelsHeight*/
const Value V23721_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23721_4 = {S54, 0, &V23721_4_1229, &P23721_5};	/*Property = ScreenPixelsWidth*/
const Profile P23721 = {"23721", &P23721_4};	/*Profile ID - 23721*/
const Value V23021_1_1 = {S2, NULL};	/*Value = True*/
const Property P23021_1 = {S50, 0, &V23021_1_1, NULL};	/*Property = IsMobile*/
const Value V23021_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23021_5 = {S52, 0, &V23021_5_1403, &P23021_1};	/*Property = ScreenPixelsHeight*/
const Value V23021_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23021_4 = {S54, 0, &V23021_4_1404, &P23021_5};	/*Property = ScreenPixelsWidth*/
const Profile P23021 = {"23021", &P23021_4};	/*Profile ID - 23021*/
const Value V22914_1_1 = {S2, NULL};	/*Value = True*/
const Property P22914_1 = {S50, 0, &V22914_1_1, NULL};	/*Property = IsMobile*/
const Value V22914_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22914_5 = {S52, 0, &V22914_5_1277, &P22914_1};	/*Property = ScreenPixelsHeight*/
const Value V22914_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22914_4 = {S54, 0, &V22914_4_1279, &P22914_5};	/*Property = ScreenPixelsWidth*/
const Profile P22914 = {"22914", &P22914_4};	/*Profile ID - 22914*/
const Value V22933_1_1 = {S2, NULL};	/*Value = True*/
const Property P22933_1 = {S50, 0, &V22933_1_1, NULL};	/*Property = IsMobile*/
const Value V22933_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22933_5 = {S52, 0, &V22933_5_1227, &P22933_1};	/*Property = ScreenPixelsHeight*/
const Value V22933_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22933_4 = {S54, 0, &V22933_4_1229, &P22933_5};	/*Property = ScreenPixelsWidth*/
const Profile P22933 = {"22933", &P22933_4};	/*Profile ID - 22933*/
const Value V20744_1_1 = {S2, NULL};	/*Value = True*/
const Property P20744_1 = {S50, 0, &V20744_1_1, NULL};	/*Property = IsMobile*/
const Value V20744_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P20744_5 = {S52, 0, &V20744_5_1222, &P20744_1};	/*Property = ScreenPixelsHeight*/
const Value V20744_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P20744_4 = {S54, 0, &V20744_4_1224, &P20744_5};	/*Property = ScreenPixelsWidth*/
const Profile P20744 = {"20744", &P20744_4};	/*Profile ID - 20744*/
const Value V23149_1_1 = {S2, NULL};	/*Value = True*/
const Property P23149_1 = {S50, 0, &V23149_1_1, NULL};	/*Property = IsMobile*/
const Value V23149_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23149_5 = {S52, 0, &V23149_5_1277, &P23149_1};	/*Property = ScreenPixelsHeight*/
const Value V23149_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23149_4 = {S54, 0, &V23149_4_1279, &P23149_5};	/*Property = ScreenPixelsWidth*/
const Profile P23149 = {"23149", &P23149_4};	/*Profile ID - 23149*/
const Value V22820_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22820_279 = {S1, 0, &V22820_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22820_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22820_282 = {S3, 0, &V22820_282_57154, &P22820_279};	/*Property = BlobBuilder*/
const Value V22820_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22820_350 = {S4, 0, &V22820_350_57444, &P22820_282};	/*Property = Canvas*/
const Value V22820_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22820_283 = {S5, 0, &V22820_283_57159, &P22820_350};	/*Property = CssBackground*/
const Value V22820_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22820_284 = {S6, 0, &V22820_284_57162, &P22820_283};	/*Property = CssBorderImage*/
const Value V22820_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22820_285 = {S7, 0, &V22820_285_57166, &P22820_284};	/*Property = CssCanvas*/
const Value V22820_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22820_286 = {S8, 0, &V22820_286_57168, &P22820_285};	/*Property = CssColor*/
const Value V22820_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22820_237 = {S9, 0, &V22820_237_55786, &P22820_286};	/*Property = CssColumn*/
const Value V22820_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22820_287 = {S10, 0, &V22820_287_57172, &P22820_237};	/*Property = CssFlexbox*/
const Value V22820_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22820_288 = {S11, 0, &V22820_288_57174, &P22820_287};	/*Property = CssFont*/
const Value V22820_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22820_289 = {S12, 0, &V22820_289_57759, &P22820_288};	/*Property = CssImages*/
const Value V22820_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22820_291 = {S13, 0, &V22820_291_57180, &P22820_289};	/*Property = CssMediaQueries*/
const Value V22820_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22820_292 = {S14, 0, &V22820_292_57183, &P22820_291};	/*Property = CssMinMax*/
const Value V22820_293_57186 = {S2, NULL};	/*Value = True*/
const Property P22820_293 = {S15, 0, &V22820_293_57186, &P22820_292};	/*Property = CssOverflow*/
const Value V22820_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22820_294 = {S16, 0, &V22820_294_57189, &P22820_293};	/*Property = CssPosition*/
const Value V22820_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22820_295 = {S17, 0, &V22820_295_57192, &P22820_294};	/*Property = CssText*/
const Value V22820_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22820_297 = {S18, 0, &V22820_297_57195, &P22820_295};	/*Property = CssTransforms*/
const Value V22820_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22820_241 = {S19, 0, &V22820_241_55799, &P22820_297};	/*Property = CssTransitions*/
const Value V22820_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22820_298 = {S20, 0, &V22820_298_57206, &P22820_241};	/*Property = CssUI*/
const Value V22820_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22820_300 = {S21, 0, &V22820_300_57215, &P22820_298};	/*Property = DataSet*/
const Value V22820_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22820_301 = {S22, 0, &V22820_301_57218, &P22820_300};	/*Property = DataUrl*/
const Value V22820_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22820_302 = {S23, 0, &V22820_302_57221, &P22820_301};	/*Property = DeviceOrientation*/
const Value V22820_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22820_304 = {S24, 0, &V22820_304_57227, &P22820_302};	/*Property = FileReader*/
const Value V22820_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22820_305 = {S25, 0, &V22820_305_57231, &P22820_304};	/*Property = FileSaver*/
const Value V22820_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22820_306 = {S26, 0, &V22820_306_57237, &P22820_305};	/*Property = FileWriter*/
const Value V22820_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22820_307 = {S27, 0, &V22820_307_57239, &P22820_306};	/*Property = FormData*/
const Value V22820_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22820_309 = {S28, 0, &V22820_309_57246, &P22820_307};	/*Property = Fullscreen*/
const Value V22820_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22820_310 = {S29, 0, &V22820_310_57248, &P22820_309};	/*Property = GeoLocation*/
const Value V22820_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22820_312 = {S30, 0, &V22820_312_57254, &P22820_310};	/*Property = History*/
const Value V22820_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22820_314 = {S31, 0, &V22820_314_57260, &P22820_312};	/*Property = Html5*/
const Value V22820_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22820_313 = {S32, 0, &V22820_313_57258, &P22820_314};	/*Property = Html-Media-Capture*/
const Value V22820_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22820_315 = {S33, 0, &V22820_315_57264, &P22820_313};	/*Property = Iframe*/
const Value V22820_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22820_316 = {S34, 0, &V22820_316_57267, &P22820_315};	/*Property = IndexedDB*/
const Value V22820_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22820_317 = {S35, 0, &V22820_317_57269, &P22820_316};	/*Property = Json*/
const Value V22820_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22820_15 = {S37, 0, &V22820_15_28, &P22820_317};	/*Property = LayoutEngine*/
const Value V22820_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22820_344 = {S38, 0, &V22820_344_57272, &P22820_15};	/*Property = Masking*/
const Value V22820_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22820_320 = {S39, 0, &V22820_320_57290, &P22820_344};	/*Property = PostMessage*/
const Value V22820_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22820_321 = {S40, 0, &V22820_321_57293, &P22820_320};	/*Property = Progress*/
const Value V22820_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22820_322 = {S41, 0, &V22820_322_57296, &P22820_321};	/*Property = Prompts*/
const Value V22820_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22820_323 = {S42, 0, &V22820_323_57299, &P22820_322};	/*Property = Selector*/
const Value V22820_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22820_324 = {S43, 0, &V22820_324_57305, &P22820_323};	/*Property = Svg*/
const Value V22820_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22820_327 = {S44, 0, &V22820_327_57314, &P22820_324};	/*Property = TouchEvents*/
const Value V22820_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22820_328 = {S45, 0, &V22820_328_57318, &P22820_327};	/*Property = Track*/
const Value V22820_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22820_329 = {S46, 0, &V22820_329_57323, &P22820_328};	/*Property = Video*/
const Value V22820_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22820_330 = {S47, 0, &V22820_330_57326, &P22820_329};	/*Property = Viewport*/
const Value V22820_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22820_332 = {S48, 0, &V22820_332_57339, &P22820_330};	/*Property = WebWorkers*/
const Value V22820_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22820_333 = {S49, 0, &V22820_333_57342, &P22820_332};	/*Property = Xhr2*/
const Profile P22820 = {"22820", &P22820_333};	/*Profile ID - 22820*/
const Value V22845_1_1 = {S2, NULL};	/*Value = True*/
const Property P22845_1 = {S50, 0, &V22845_1_1, NULL};	/*Property = IsMobile*/
const Value V22845_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22845_5 = {S52, 0, &V22845_5_1227, &P22845_1};	/*Property = ScreenPixelsHeight*/
const Value V22845_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22845_4 = {S54, 0, &V22845_4_1229, &P22845_5};	/*Property = ScreenPixelsWidth*/
const Profile P22845 = {"22845", &P22845_4};	/*Profile ID - 22845*/
const Value V22948_1_1 = {S2, NULL};	/*Value = True*/
const Property P22948_1 = {S50, 0, &V22948_1_1, NULL};	/*Property = IsMobile*/
const Value V22948_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22948_5 = {S52, 0, &V22948_5_1227, &P22948_1};	/*Property = ScreenPixelsHeight*/
const Value V22948_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22948_4 = {S54, 0, &V22948_4_1229, &P22948_5};	/*Property = ScreenPixelsWidth*/
const Profile P22948 = {"22948", &P22948_4};	/*Profile ID - 22948*/
const Value V23187_1_1 = {S2, NULL};	/*Value = True*/
const Property P23187_1 = {S50, 0, &V23187_1_1, NULL};	/*Property = IsMobile*/
const Value V23187_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23187_5 = {S52, 0, &V23187_5_1403, &P23187_1};	/*Property = ScreenPixelsHeight*/
const Value V23187_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23187_4 = {S54, 0, &V23187_4_1404, &P23187_5};	/*Property = ScreenPixelsWidth*/
const Profile P23187 = {"23187", &P23187_4};	/*Profile ID - 23187*/
const Value V23371_279_57147 = {S2, NULL};	/*Value = True*/
const Property P23371_279 = {S1, 0, &V23371_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V23371_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23371_282 = {S3, 0, &V23371_282_57154, &P23371_279};	/*Property = BlobBuilder*/
const Value V23371_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23371_350 = {S4, 0, &V23371_350_57444, &P23371_282};	/*Property = Canvas*/
const Value V23371_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23371_283 = {S5, 0, &V23371_283_57159, &P23371_350};	/*Property = CssBackground*/
const Value V23371_284_57162 = {S2, NULL};	/*Value = True*/
const Property P23371_284 = {S6, 0, &V23371_284_57162, &P23371_283};	/*Property = CssBorderImage*/
const Value V23371_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23371_285 = {S7, 0, &V23371_285_57166, &P23371_284};	/*Property = CssCanvas*/
const Value V23371_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23371_286 = {S8, 0, &V23371_286_57168, &P23371_285};	/*Property = CssColor*/
const Value V23371_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23371_237 = {S9, 0, &V23371_237_55786, &P23371_286};	/*Property = CssColumn*/
const Value V23371_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23371_287 = {S10, 0, &V23371_287_57172, &P23371_237};	/*Property = CssFlexbox*/
const Value V23371_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23371_288 = {S11, 0, &V23371_288_57174, &P23371_287};	/*Property = CssFont*/
const Value V23371_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23371_289 = {S12, 0, &V23371_289_57759, &P23371_288};	/*Property = CssImages*/
const Value V23371_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23371_291 = {S13, 0, &V23371_291_57180, &P23371_289};	/*Property = CssMediaQueries*/
const Value V23371_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23371_292 = {S14, 0, &V23371_292_57183, &P23371_291};	/*Property = CssMinMax*/
const Value V23371_293_57187 = {S0, NULL};	/*Value = False*/
const Property P23371_293 = {S15, 0, &V23371_293_57187, &P23371_292};	/*Property = CssOverflow*/
const Value V23371_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23371_294 = {S16, 0, &V23371_294_57189, &P23371_293};	/*Property = CssPosition*/
const Value V23371_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23371_295 = {S17, 0, &V23371_295_57192, &P23371_294};	/*Property = CssText*/
const Value V23371_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23371_297 = {S18, 0, &V23371_297_57195, &P23371_295};	/*Property = CssTransforms*/
const Value V23371_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23371_241 = {S19, 0, &V23371_241_55799, &P23371_297};	/*Property = CssTransitions*/
const Value V23371_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23371_298 = {S20, 0, &V23371_298_57206, &P23371_241};	/*Property = CssUI*/
const Value V23371_300_57215 = {S2, NULL};	/*Value = True*/
const Property P23371_300 = {S21, 0, &V23371_300_57215, &P23371_298};	/*Property = DataSet*/
const Value V23371_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23371_301 = {S22, 0, &V23371_301_57218, &P23371_300};	/*Property = DataUrl*/
const Value V23371_302_57222 = {S0, NULL};	/*Value = False*/
const Property P23371_302 = {S23, 0, &V23371_302_57222, &P23371_301};	/*Property = DeviceOrientation*/
const Value V23371_304_57227 = {S2, NULL};	/*Value = True*/
const Property P23371_304 = {S24, 0, &V23371_304_57227, &P23371_302};	/*Property = FileReader*/
const Value V23371_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23371_305 = {S25, 0, &V23371_305_57231, &P23371_304};	/*Property = FileSaver*/
const Value V23371_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23371_306 = {S26, 0, &V23371_306_57237, &P23371_305};	/*Property = FileWriter*/
const Value V23371_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23371_307 = {S27, 0, &V23371_307_57239, &P23371_306};	/*Property = FormData*/
const Value V23371_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23371_309 = {S28, 0, &V23371_309_57246, &P23371_307};	/*Property = Fullscreen*/
const Value V23371_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23371_310 = {S29, 0, &V23371_310_57248, &P23371_309};	/*Property = GeoLocation*/
const Value V23371_312_57254 = {S2, NULL};	/*Value = True*/
const Property P23371_312 = {S30, 0, &V23371_312_57254, &P23371_310};	/*Property = History*/
const Value V23371_314_57260 = {S2, NULL};	/*Value = True*/
const Property P23371_314 = {S31, 0, &V23371_314_57260, &P23371_312};	/*Property = Html5*/
const Value V23371_313_57257 = {S2, NULL};	/*Value = True*/
const Property P23371_313 = {S32, 0, &V23371_313_57257, &P23371_314};	/*Property = Html-Media-Capture*/
const Value V23371_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23371_315 = {S33, 0, &V23371_315_57264, &P23371_313};	/*Property = Iframe*/
const Value V23371_316_57266 = {S2, NULL};	/*Value = True*/
const Property P23371_316 = {S34, 0, &V23371_316_57266, &P23371_315};	/*Property = IndexedDB*/
const Value V23371_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23371_317 = {S35, 0, &V23371_317_57269, &P23371_316};	/*Property = Json*/
const Value V23371_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P23371_15 = {S37, 0, &V23371_15_28, &P23371_317};	/*Property = LayoutEngine*/
const Value V23371_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23371_344 = {S38, 0, &V23371_344_57272, &P23371_15};	/*Property = Masking*/
const Value V23371_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23371_320 = {S39, 0, &V23371_320_57290, &P23371_344};	/*Property = PostMessage*/
const Value V23371_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23371_321 = {S40, 0, &V23371_321_57293, &P23371_320};	/*Property = Progress*/
const Value V23371_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23371_322 = {S41, 0, &V23371_322_57296, &P23371_321};	/*Property = Prompts*/
const Value V23371_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23371_323 = {S42, 0, &V23371_323_57299, &P23371_322};	/*Property = Selector*/
const Value V23371_324_57305 = {S2, NULL};	/*Value = True*/
const Property P23371_324 = {S43, 0, &V23371_324_57305, &P23371_323};	/*Property = Svg*/
const Value V23371_327_57314 = {S2, NULL};	/*Value = True*/
const Property P23371_327 = {S44, 0, &V23371_327_57314, &P23371_324};	/*Property = TouchEvents*/
const Value V23371_328_57317 = {S2, NULL};	/*Value = True*/
const Property P23371_328 = {S45, 0, &V23371_328_57317, &P23371_327};	/*Property = Track*/
const Value V23371_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23371_329 = {S46, 0, &V23371_329_57323, &P23371_328};	/*Property = Video*/
const Value V23371_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23371_330 = {S47, 0, &V23371_330_57326, &P23371_329};	/*Property = Viewport*/
const Value V23371_332_57339 = {S0, NULL};	/*Value = False*/
const Property P23371_332 = {S48, 0, &V23371_332_57339, &P23371_330};	/*Property = WebWorkers*/
const Value V23371_333_57341 = {S2, NULL};	/*Value = True*/
const Property P23371_333 = {S49, 0, &V23371_333_57341, &P23371_332};	/*Property = Xhr2*/
const Profile P23371 = {"23371", &P23371_333};	/*Profile ID - 23371*/
const Value V22859_1_1 = {S2, NULL};	/*Value = True*/
const Property P22859_1 = {S50, 0, &V22859_1_1, NULL};	/*Property = IsMobile*/
const Value V22859_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22859_5 = {S52, 0, &V22859_5_1227, &P22859_1};	/*Property = ScreenPixelsHeight*/
const Value V22859_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22859_4 = {S54, 0, &V22859_4_1229, &P22859_5};	/*Property = ScreenPixelsWidth*/
const Profile P22859 = {"22859", &P22859_4};	/*Profile ID - 22859*/
const Profile P22880 = {"22880", NULL};	/*Profile ID - 22880*/
const Value V22893_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22893_279 = {S1, 0, &V22893_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22893_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22893_282 = {S3, 0, &V22893_282_57154, &P22893_279};	/*Property = BlobBuilder*/
const Value V22893_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22893_350 = {S4, 0, &V22893_350_57444, &P22893_282};	/*Property = Canvas*/
const Value V22893_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22893_283 = {S5, 0, &V22893_283_57159, &P22893_350};	/*Property = CssBackground*/
const Value V22893_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22893_284 = {S6, 0, &V22893_284_57162, &P22893_283};	/*Property = CssBorderImage*/
const Value V22893_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22893_285 = {S7, 0, &V22893_285_57166, &P22893_284};	/*Property = CssCanvas*/
const Value V22893_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22893_286 = {S8, 0, &V22893_286_57168, &P22893_285};	/*Property = CssColor*/
const Value V22893_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22893_237 = {S9, 0, &V22893_237_55786, &P22893_286};	/*Property = CssColumn*/
const Value V22893_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22893_287 = {S10, 0, &V22893_287_57172, &P22893_237};	/*Property = CssFlexbox*/
const Value V22893_288_57174 = {S2, NULL};	/*Value = True*/
const Property P22893_288 = {S11, 0, &V22893_288_57174, &P22893_287};	/*Property = CssFont*/
const Value V22893_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22893_289 = {S12, 0, &V22893_289_57759, &P22893_288};	/*Property = CssImages*/
const Value V22893_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22893_291 = {S13, 0, &V22893_291_57180, &P22893_289};	/*Property = CssMediaQueries*/
const Value V22893_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22893_292 = {S14, 0, &V22893_292_57183, &P22893_291};	/*Property = CssMinMax*/
const Value V22893_293_57186 = {S2, NULL};	/*Value = True*/
const Property P22893_293 = {S15, 0, &V22893_293_57186, &P22893_292};	/*Property = CssOverflow*/
const Value V22893_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22893_294 = {S16, 0, &V22893_294_57189, &P22893_293};	/*Property = CssPosition*/
const Value V22893_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22893_295 = {S17, 0, &V22893_295_57192, &P22893_294};	/*Property = CssText*/
const Value V22893_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22893_297 = {S18, 0, &V22893_297_57195, &P22893_295};	/*Property = CssTransforms*/
const Value V22893_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22893_241 = {S19, 0, &V22893_241_55799, &P22893_297};	/*Property = CssTransitions*/
const Value V22893_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22893_298 = {S20, 0, &V22893_298_57206, &P22893_241};	/*Property = CssUI*/
const Value V22893_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22893_300 = {S21, 0, &V22893_300_57215, &P22893_298};	/*Property = DataSet*/
const Value V22893_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22893_301 = {S22, 0, &V22893_301_57218, &P22893_300};	/*Property = DataUrl*/
const Value V22893_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22893_302 = {S23, 0, &V22893_302_57221, &P22893_301};	/*Property = DeviceOrientation*/
const Value V22893_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22893_304 = {S24, 0, &V22893_304_57227, &P22893_302};	/*Property = FileReader*/
const Value V22893_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22893_305 = {S25, 0, &V22893_305_57231, &P22893_304};	/*Property = FileSaver*/
const Value V22893_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22893_306 = {S26, 0, &V22893_306_57237, &P22893_305};	/*Property = FileWriter*/
const Value V22893_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22893_307 = {S27, 0, &V22893_307_57239, &P22893_306};	/*Property = FormData*/
const Value V22893_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22893_309 = {S28, 0, &V22893_309_57246, &P22893_307};	/*Property = Fullscreen*/
const Value V22893_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22893_310 = {S29, 0, &V22893_310_57248, &P22893_309};	/*Property = GeoLocation*/
const Value V22893_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22893_312 = {S30, 0, &V22893_312_57254, &P22893_310};	/*Property = History*/
const Value V22893_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22893_314 = {S31, 0, &V22893_314_57260, &P22893_312};	/*Property = Html5*/
const Value V22893_313_57257 = {S2, NULL};	/*Value = True*/
const Property P22893_313 = {S32, 0, &V22893_313_57257, &P22893_314};	/*Property = Html-Media-Capture*/
const Value V22893_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22893_315 = {S33, 0, &V22893_315_57264, &P22893_313};	/*Property = Iframe*/
const Value V22893_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22893_316 = {S34, 0, &V22893_316_57266, &P22893_315};	/*Property = IndexedDB*/
const Value V22893_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22893_317 = {S35, 0, &V22893_317_57269, &P22893_316};	/*Property = Json*/
const Value V22893_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22893_15 = {S37, 0, &V22893_15_28, &P22893_317};	/*Property = LayoutEngine*/
const Value V22893_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22893_344 = {S38, 0, &V22893_344_57272, &P22893_15};	/*Property = Masking*/
const Value V22893_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22893_320 = {S39, 0, &V22893_320_57290, &P22893_344};	/*Property = PostMessage*/
const Value V22893_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22893_321 = {S40, 0, &V22893_321_57293, &P22893_320};	/*Property = Progress*/
const Value V22893_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22893_322 = {S41, 0, &V22893_322_57296, &P22893_321};	/*Property = Prompts*/
const Value V22893_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22893_323 = {S42, 0, &V22893_323_57299, &P22893_322};	/*Property = Selector*/
const Value V22893_324_57305 = {S2, NULL};	/*Value = True*/
const Property P22893_324 = {S43, 0, &V22893_324_57305, &P22893_323};	/*Property = Svg*/
const Value V22893_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22893_327 = {S44, 0, &V22893_327_57314, &P22893_324};	/*Property = TouchEvents*/
const Value V22893_328_57317 = {S2, NULL};	/*Value = True*/
const Property P22893_328 = {S45, 0, &V22893_328_57317, &P22893_327};	/*Property = Track*/
const Value V22893_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22893_329 = {S46, 0, &V22893_329_57323, &P22893_328};	/*Property = Video*/
const Value V22893_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22893_330 = {S47, 0, &V22893_330_57326, &P22893_329};	/*Property = Viewport*/
const Value V22893_332_57338 = {S2, NULL};	/*Value = True*/
const Property P22893_332 = {S48, 0, &V22893_332_57338, &P22893_330};	/*Property = WebWorkers*/
const Value V22893_333_57341 = {S2, NULL};	/*Value = True*/
const Property P22893_333 = {S49, 0, &V22893_333_57341, &P22893_332};	/*Property = Xhr2*/
const Profile P22893 = {"22893", &P22893_333};	/*Profile ID - 22893*/
const Value V22953_1_1 = {S2, NULL};	/*Value = True*/
const Property P22953_1 = {S50, 0, &V22953_1_1, NULL};	/*Property = IsMobile*/
const Value V22953_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22953_5 = {S52, 0, &V22953_5_59138, &P22953_1};	/*Property = ScreenPixelsHeight*/
const Value V22953_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22953_4 = {S54, 0, &V22953_4_59970, &P22953_5};	/*Property = ScreenPixelsWidth*/
const Profile P22953 = {"22953", &P22953_4};	/*Profile ID - 22953*/
const Value V22932_1_1 = {S2, NULL};	/*Value = True*/
const Property P22932_1 = {S50, 0, &V22932_1_1, NULL};	/*Property = IsMobile*/
const Value V22932_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22932_5 = {S52, 0, &V22932_5_1227, &P22932_1};	/*Property = ScreenPixelsHeight*/
const Value V22932_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22932_4 = {S54, 0, &V22932_4_1229, &P22932_5};	/*Property = ScreenPixelsWidth*/
const Profile P22932 = {"22932", &P22932_4};	/*Profile ID - 22932*/
const Value V23131_1_1 = {S2, NULL};	/*Value = True*/
const Property P23131_1 = {S50, 0, &V23131_1_1, NULL};	/*Property = IsMobile*/
const Value V23131_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23131_5 = {S52, 0, &V23131_5_37, &P23131_1};	/*Property = ScreenPixelsHeight*/
const Value V23131_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23131_4 = {S54, 0, &V23131_4_36, &P23131_5};	/*Property = ScreenPixelsWidth*/
const Profile P23131 = {"23131", &P23131_4};	/*Profile ID - 23131*/
const Value V22825_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22825_279 = {S1, 0, &V22825_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22825_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22825_282 = {S3, 0, &V22825_282_57154, &P22825_279};	/*Property = BlobBuilder*/
const Value V22825_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22825_350 = {S4, 0, &V22825_350_57444, &P22825_282};	/*Property = Canvas*/
const Value V22825_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22825_283 = {S5, 0, &V22825_283_57159, &P22825_350};	/*Property = CssBackground*/
const Value V22825_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22825_284 = {S6, 0, &V22825_284_57163, &P22825_283};	/*Property = CssBorderImage*/
const Value V22825_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22825_285 = {S7, 0, &V22825_285_57166, &P22825_284};	/*Property = CssCanvas*/
const Value V22825_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22825_286 = {S8, 0, &V22825_286_57168, &P22825_285};	/*Property = CssColor*/
const Value V22825_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22825_237 = {S9, 0, &V22825_237_55786, &P22825_286};	/*Property = CssColumn*/
const Value V22825_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22825_287 = {S10, 0, &V22825_287_57172, &P22825_237};	/*Property = CssFlexbox*/
const Value V22825_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22825_288 = {S11, 0, &V22825_288_57175, &P22825_287};	/*Property = CssFont*/
const Value V22825_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22825_289 = {S12, 0, &V22825_289_57759, &P22825_288};	/*Property = CssImages*/
const Value V22825_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22825_291 = {S13, 0, &V22825_291_57181, &P22825_289};	/*Property = CssMediaQueries*/
const Value V22825_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22825_292 = {S14, 0, &V22825_292_57183, &P22825_291};	/*Property = CssMinMax*/
const Value V22825_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22825_293 = {S15, 0, &V22825_293_57187, &P22825_292};	/*Property = CssOverflow*/
const Value V22825_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22825_294 = {S16, 0, &V22825_294_57190, &P22825_293};	/*Property = CssPosition*/
const Value V22825_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22825_295 = {S17, 0, &V22825_295_57192, &P22825_294};	/*Property = CssText*/
const Value V22825_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22825_297 = {S18, 0, &V22825_297_57195, &P22825_295};	/*Property = CssTransforms*/
const Value V22825_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22825_241 = {S19, 0, &V22825_241_55799, &P22825_297};	/*Property = CssTransitions*/
const Value V22825_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22825_298 = {S20, 0, &V22825_298_57206, &P22825_241};	/*Property = CssUI*/
const Value V22825_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22825_300 = {S21, 0, &V22825_300_57216, &P22825_298};	/*Property = DataSet*/
const Value V22825_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22825_301 = {S22, 0, &V22825_301_57218, &P22825_300};	/*Property = DataUrl*/
const Value V22825_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22825_302 = {S23, 0, &V22825_302_57222, &P22825_301};	/*Property = DeviceOrientation*/
const Value V22825_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22825_304 = {S24, 0, &V22825_304_57228, &P22825_302};	/*Property = FileReader*/
const Value V22825_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22825_305 = {S25, 0, &V22825_305_57231, &P22825_304};	/*Property = FileSaver*/
const Value V22825_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22825_306 = {S26, 0, &V22825_306_57237, &P22825_305};	/*Property = FileWriter*/
const Value V22825_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22825_307 = {S27, 0, &V22825_307_57240, &P22825_306};	/*Property = FormData*/
const Value V22825_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22825_309 = {S28, 0, &V22825_309_57246, &P22825_307};	/*Property = Fullscreen*/
const Value V22825_310_57249 = {S0, NULL};	/*Value = False*/
const Property P22825_310 = {S29, 0, &V22825_310_57249, &P22825_309};	/*Property = GeoLocation*/
const Value V22825_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22825_312 = {S30, 0, &V22825_312_57255, &P22825_310};	/*Property = History*/
const Value V22825_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22825_314 = {S31, 0, &V22825_314_57261, &P22825_312};	/*Property = Html5*/
const Value V22825_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22825_313 = {S32, 0, &V22825_313_57258, &P22825_314};	/*Property = Html-Media-Capture*/
const Value V22825_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22825_315 = {S33, 0, &V22825_315_57264, &P22825_313};	/*Property = Iframe*/
const Value V22825_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22825_316 = {S34, 0, &V22825_316_57267, &P22825_315};	/*Property = IndexedDB*/
const Value V22825_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22825_317 = {S35, 0, &V22825_317_57269, &P22825_316};	/*Property = Json*/
const Value V22825_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P22825_15 = {S37, 0, &V22825_15_54167, &P22825_317};	/*Property = LayoutEngine*/
const Value V22825_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22825_344 = {S38, 0, &V22825_344_57272, &P22825_15};	/*Property = Masking*/
const Value V22825_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22825_320 = {S39, 0, &V22825_320_57290, &P22825_344};	/*Property = PostMessage*/
const Value V22825_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22825_321 = {S40, 0, &V22825_321_57293, &P22825_320};	/*Property = Progress*/
const Value V22825_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22825_322 = {S41, 0, &V22825_322_57296, &P22825_321};	/*Property = Prompts*/
const Value V22825_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22825_323 = {S42, 0, &V22825_323_57299, &P22825_322};	/*Property = Selector*/
const Value V22825_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22825_324 = {S43, 0, &V22825_324_57306, &P22825_323};	/*Property = Svg*/
const Value V22825_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22825_327 = {S44, 0, &V22825_327_57315, &P22825_324};	/*Property = TouchEvents*/
const Value V22825_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22825_328 = {S45, 0, &V22825_328_57318, &P22825_327};	/*Property = Track*/
const Value V22825_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22825_329 = {S46, 0, &V22825_329_57323, &P22825_328};	/*Property = Video*/
const Value V22825_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22825_330 = {S47, 0, &V22825_330_57326, &P22825_329};	/*Property = Viewport*/
const Value V22825_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22825_332 = {S48, 0, &V22825_332_57339, &P22825_330};	/*Property = WebWorkers*/
const Value V22825_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22825_333 = {S49, 0, &V22825_333_57342, &P22825_332};	/*Property = Xhr2*/
const Profile P22825 = {"22825", &P22825_333};	/*Profile ID - 22825*/
const Value V22824_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22824_279 = {S1, 0, &V22824_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22824_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22824_282 = {S3, 0, &V22824_282_57154, &P22824_279};	/*Property = BlobBuilder*/
const Value V22824_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22824_350 = {S4, 0, &V22824_350_57444, &P22824_282};	/*Property = Canvas*/
const Value V22824_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22824_283 = {S5, 0, &V22824_283_57159, &P22824_350};	/*Property = CssBackground*/
const Value V22824_284_57163 = {S0, NULL};	/*Value = False*/
const Property P22824_284 = {S6, 0, &V22824_284_57163, &P22824_283};	/*Property = CssBorderImage*/
const Value V22824_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22824_285 = {S7, 0, &V22824_285_57166, &P22824_284};	/*Property = CssCanvas*/
const Value V22824_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22824_286 = {S8, 0, &V22824_286_57168, &P22824_285};	/*Property = CssColor*/
const Value V22824_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22824_237 = {S9, 0, &V22824_237_55786, &P22824_286};	/*Property = CssColumn*/
const Value V22824_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22824_287 = {S10, 0, &V22824_287_57172, &P22824_237};	/*Property = CssFlexbox*/
const Value V22824_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22824_288 = {S11, 0, &V22824_288_57175, &P22824_287};	/*Property = CssFont*/
const Value V22824_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22824_289 = {S12, 0, &V22824_289_57759, &P22824_288};	/*Property = CssImages*/
const Value V22824_291_57181 = {S0, NULL};	/*Value = False*/
const Property P22824_291 = {S13, 0, &V22824_291_57181, &P22824_289};	/*Property = CssMediaQueries*/
const Value V22824_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22824_292 = {S14, 0, &V22824_292_57183, &P22824_291};	/*Property = CssMinMax*/
const Value V22824_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22824_293 = {S15, 0, &V22824_293_57187, &P22824_292};	/*Property = CssOverflow*/
const Value V22824_294_57190 = {S0, NULL};	/*Value = False*/
const Property P22824_294 = {S16, 0, &V22824_294_57190, &P22824_293};	/*Property = CssPosition*/
const Value V22824_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22824_295 = {S17, 0, &V22824_295_57192, &P22824_294};	/*Property = CssText*/
const Value V22824_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22824_297 = {S18, 0, &V22824_297_57195, &P22824_295};	/*Property = CssTransforms*/
const Value V22824_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22824_241 = {S19, 0, &V22824_241_55799, &P22824_297};	/*Property = CssTransitions*/
const Value V22824_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22824_298 = {S20, 0, &V22824_298_57206, &P22824_241};	/*Property = CssUI*/
const Value V22824_300_57216 = {S0, NULL};	/*Value = False*/
const Property P22824_300 = {S21, 0, &V22824_300_57216, &P22824_298};	/*Property = DataSet*/
const Value V22824_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22824_301 = {S22, 0, &V22824_301_57218, &P22824_300};	/*Property = DataUrl*/
const Value V22824_302_57222 = {S0, NULL};	/*Value = False*/
const Property P22824_302 = {S23, 0, &V22824_302_57222, &P22824_301};	/*Property = DeviceOrientation*/
const Value V22824_304_57228 = {S0, NULL};	/*Value = False*/
const Property P22824_304 = {S24, 0, &V22824_304_57228, &P22824_302};	/*Property = FileReader*/
const Value V22824_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22824_305 = {S25, 0, &V22824_305_57231, &P22824_304};	/*Property = FileSaver*/
const Value V22824_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22824_306 = {S26, 0, &V22824_306_57237, &P22824_305};	/*Property = FileWriter*/
const Value V22824_307_57240 = {S0, NULL};	/*Value = False*/
const Property P22824_307 = {S27, 0, &V22824_307_57240, &P22824_306};	/*Property = FormData*/
const Value V22824_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22824_309 = {S28, 0, &V22824_309_57246, &P22824_307};	/*Property = Fullscreen*/
const Value V22824_310_57249 = {S0, NULL};	/*Value = False*/
const Property P22824_310 = {S29, 0, &V22824_310_57249, &P22824_309};	/*Property = GeoLocation*/
const Value V22824_312_57255 = {S0, NULL};	/*Value = False*/
const Property P22824_312 = {S30, 0, &V22824_312_57255, &P22824_310};	/*Property = History*/
const Value V22824_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22824_314 = {S31, 0, &V22824_314_57261, &P22824_312};	/*Property = Html5*/
const Value V22824_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22824_313 = {S32, 0, &V22824_313_57258, &P22824_314};	/*Property = Html-Media-Capture*/
const Value V22824_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22824_315 = {S33, 0, &V22824_315_57264, &P22824_313};	/*Property = Iframe*/
const Value V22824_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22824_316 = {S34, 0, &V22824_316_57267, &P22824_315};	/*Property = IndexedDB*/
const Value V22824_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22824_317 = {S35, 0, &V22824_317_57269, &P22824_316};	/*Property = Json*/
const Value V22824_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P22824_15 = {S37, 0, &V22824_15_54167, &P22824_317};	/*Property = LayoutEngine*/
const Value V22824_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22824_344 = {S38, 0, &V22824_344_57272, &P22824_15};	/*Property = Masking*/
const Value V22824_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22824_320 = {S39, 0, &V22824_320_57290, &P22824_344};	/*Property = PostMessage*/
const Value V22824_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22824_321 = {S40, 0, &V22824_321_57293, &P22824_320};	/*Property = Progress*/
const Value V22824_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22824_322 = {S41, 0, &V22824_322_57296, &P22824_321};	/*Property = Prompts*/
const Value V22824_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22824_323 = {S42, 0, &V22824_323_57299, &P22824_322};	/*Property = Selector*/
const Value V22824_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22824_324 = {S43, 0, &V22824_324_57306, &P22824_323};	/*Property = Svg*/
const Value V22824_327_57315 = {S0, NULL};	/*Value = False*/
const Property P22824_327 = {S44, 0, &V22824_327_57315, &P22824_324};	/*Property = TouchEvents*/
const Value V22824_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22824_328 = {S45, 0, &V22824_328_57318, &P22824_327};	/*Property = Track*/
const Value V22824_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22824_329 = {S46, 0, &V22824_329_57323, &P22824_328};	/*Property = Video*/
const Value V22824_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22824_330 = {S47, 0, &V22824_330_57326, &P22824_329};	/*Property = Viewport*/
const Value V22824_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22824_332 = {S48, 0, &V22824_332_57339, &P22824_330};	/*Property = WebWorkers*/
const Value V22824_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22824_333 = {S49, 0, &V22824_333_57342, &P22824_332};	/*Property = Xhr2*/
const Profile P22824 = {"22824", &P22824_333};	/*Profile ID - 22824*/
const Value V24639_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_279 = {S1, 0, &V24639_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24639_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_282 = {S3, 0, &V24639_282_57155, &P24639_279};	/*Property = BlobBuilder*/
const Value V24639_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_350 = {S4, 0, &V24639_350_57446, &P24639_282};	/*Property = Canvas*/
const Value V24639_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_283 = {S5, 0, &V24639_283_57161, &P24639_350};	/*Property = CssBackground*/
const Value V24639_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_284 = {S6, 0, &V24639_284_57164, &P24639_283};	/*Property = CssBorderImage*/
const Value V24639_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_285 = {S7, 0, &V24639_285_57167, &P24639_284};	/*Property = CssCanvas*/
const Value V24639_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_286 = {S8, 0, &V24639_286_57170, &P24639_285};	/*Property = CssColor*/
const Value V24639_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_237 = {S9, 0, &V24639_237_55786, &P24639_286};	/*Property = CssColumn*/
const Value V24639_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_287 = {S10, 0, &V24639_287_57173, &P24639_237};	/*Property = CssFlexbox*/
const Value V24639_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_288 = {S11, 0, &V24639_288_57176, &P24639_287};	/*Property = CssFont*/
const Value V24639_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_289 = {S12, 0, &V24639_289_86899, &P24639_288};	/*Property = CssImages*/
const Value V24639_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_291 = {S13, 0, &V24639_291_57182, &P24639_289};	/*Property = CssMediaQueries*/
const Value V24639_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_292 = {S14, 0, &V24639_292_57185, &P24639_291};	/*Property = CssMinMax*/
const Value V24639_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_293 = {S15, 0, &V24639_293_57188, &P24639_292};	/*Property = CssOverflow*/
const Value V24639_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_294 = {S16, 0, &V24639_294_57191, &P24639_293};	/*Property = CssPosition*/
const Value V24639_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_295 = {S17, 0, &V24639_295_57194, &P24639_294};	/*Property = CssText*/
const Value V24639_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_297 = {S18, 0, &V24639_297_57197, &P24639_295};	/*Property = CssTransforms*/
const Value V24639_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_241 = {S19, 0, &V24639_241_55801, &P24639_297};	/*Property = CssTransitions*/
const Value V24639_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_298 = {S20, 0, &V24639_298_57208, &P24639_241};	/*Property = CssUI*/
const Value V24639_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_300 = {S21, 0, &V24639_300_57217, &P24639_298};	/*Property = DataSet*/
const Value V24639_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_301 = {S22, 0, &V24639_301_57220, &P24639_300};	/*Property = DataUrl*/
const Value V24639_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_302 = {S23, 0, &V24639_302_57223, &P24639_301};	/*Property = DeviceOrientation*/
const Value V24639_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_304 = {S24, 0, &V24639_304_57229, &P24639_302};	/*Property = FileReader*/
const Value V24639_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_305 = {S25, 0, &V24639_305_57232, &P24639_304};	/*Property = FileSaver*/
const Value V24639_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_306 = {S26, 0, &V24639_306_57238, &P24639_305};	/*Property = FileWriter*/
const Value V24639_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_307 = {S27, 0, &V24639_307_57241, &P24639_306};	/*Property = FormData*/
const Value V24639_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_309 = {S28, 0, &V24639_309_57247, &P24639_307};	/*Property = Fullscreen*/
const Value V24639_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_310 = {S29, 0, &V24639_310_57250, &P24639_309};	/*Property = GeoLocation*/
const Value V24639_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_312 = {S30, 0, &V24639_312_57256, &P24639_310};	/*Property = History*/
const Value V24639_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_314 = {S31, 0, &V24639_314_57262, &P24639_312};	/*Property = Html5*/
const Value V24639_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_313 = {S32, 0, &V24639_313_57259, &P24639_314};	/*Property = Html-Media-Capture*/
const Value V24639_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_315 = {S33, 0, &V24639_315_57265, &P24639_313};	/*Property = Iframe*/
const Value V24639_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_316 = {S34, 0, &V24639_316_57268, &P24639_315};	/*Property = IndexedDB*/
const Value V24639_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_317 = {S35, 0, &V24639_317_57271, &P24639_316};	/*Property = Json*/
const Value V24639_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24639_15 = {S37, 0, &V24639_15_90837, &P24639_317};	/*Property = LayoutEngine*/
const Value V24639_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_344 = {S38, 0, &V24639_344_57274, &P24639_15};	/*Property = Masking*/
const Value V24639_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_320 = {S39, 0, &V24639_320_57292, &P24639_344};	/*Property = PostMessage*/
const Value V24639_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_321 = {S40, 0, &V24639_321_57295, &P24639_320};	/*Property = Progress*/
const Value V24639_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_322 = {S41, 0, &V24639_322_57298, &P24639_321};	/*Property = Prompts*/
const Value V24639_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_323 = {S42, 0, &V24639_323_57301, &P24639_322};	/*Property = Selector*/
const Value V24639_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_324 = {S43, 0, &V24639_324_57307, &P24639_323};	/*Property = Svg*/
const Value V24639_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_327 = {S44, 0, &V24639_327_57316, &P24639_324};	/*Property = TouchEvents*/
const Value V24639_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_328 = {S45, 0, &V24639_328_57319, &P24639_327};	/*Property = Track*/
const Value V24639_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_329 = {S46, 0, &V24639_329_57325, &P24639_328};	/*Property = Video*/
const Value V24639_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_330 = {S47, 0, &V24639_330_57328, &P24639_329};	/*Property = Viewport*/
const Value V24639_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_332 = {S48, 0, &V24639_332_57340, &P24639_330};	/*Property = WebWorkers*/
const Value V24639_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24639_333 = {S49, 0, &V24639_333_57343, &P24639_332};	/*Property = Xhr2*/
const Profile P24639 = {"24639", &P24639_333};	/*Profile ID - 24639*/
const Value V22862_1_1 = {S2, NULL};	/*Value = True*/
const Property P22862_1 = {S50, 0, &V22862_1_1, NULL};	/*Property = IsMobile*/
const Value V22862_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P22862_5 = {S52, 0, &V22862_5_1269, &P22862_1};	/*Property = ScreenPixelsHeight*/
const Value V22862_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22862_4 = {S54, 0, &V22862_4_1229, &P22862_5};	/*Property = ScreenPixelsWidth*/
const Profile P22862 = {"22862", &P22862_4};	/*Profile ID - 22862*/
const Value V23006_1_1 = {S2, NULL};	/*Value = True*/
const Property P23006_1 = {S50, 0, &V23006_1_1, NULL};	/*Property = IsMobile*/
const Value V23006_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23006_5 = {S52, 0, &V23006_5_1277, &P23006_1};	/*Property = ScreenPixelsHeight*/
const Value V23006_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23006_4 = {S54, 0, &V23006_4_1279, &P23006_5};	/*Property = ScreenPixelsWidth*/
const Profile P23006 = {"23006", &P23006_4};	/*Profile ID - 23006*/
const Value V23245_1_1 = {S2, NULL};	/*Value = True*/
const Property P23245_1 = {S50, 0, &V23245_1_1, NULL};	/*Property = IsMobile*/
const Value V23245_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23245_5 = {S52, 0, &V23245_5_1277, &P23245_1};	/*Property = ScreenPixelsHeight*/
const Value V23245_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23245_4 = {S54, 0, &V23245_4_1279, &P23245_5};	/*Property = ScreenPixelsWidth*/
const Profile P23245 = {"23245", &P23245_4};	/*Profile ID - 23245*/
const Value V23005_1_1 = {S2, NULL};	/*Value = True*/
const Property P23005_1 = {S50, 0, &V23005_1_1, NULL};	/*Property = IsMobile*/
const Value V23005_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23005_5 = {S52, 0, &V23005_5_1277, &P23005_1};	/*Property = ScreenPixelsHeight*/
const Value V23005_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23005_4 = {S54, 0, &V23005_4_2336, &P23005_5};	/*Property = ScreenPixelsWidth*/
const Profile P23005 = {"23005", &P23005_4};	/*Profile ID - 23005*/
const Value V23203_1_1 = {S2, NULL};	/*Value = True*/
const Property P23203_1 = {S50, 0, &V23203_1_1, NULL};	/*Property = IsMobile*/
const Value V23203_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23203_5 = {S52, 0, &V23203_5_37569, &P23203_1};	/*Property = ScreenPixelsHeight*/
const Value V23203_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23203_4 = {S54, 0, &V23203_4_5645, &P23203_5};	/*Property = ScreenPixelsWidth*/
const Profile P23203 = {"23203", &P23203_4};	/*Profile ID - 23203*/
const Value V23157_1_1 = {S2, NULL};	/*Value = True*/
const Property P23157_1 = {S50, 0, &V23157_1_1, NULL};	/*Property = IsMobile*/
const Value V23157_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23157_5 = {S52, 0, &V23157_5_1832, &P23157_1};	/*Property = ScreenPixelsHeight*/
const Value V23157_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23157_4 = {S54, 0, &V23157_4_5645, &P23157_5};	/*Property = ScreenPixelsWidth*/
const Profile P23157 = {"23157", &P23157_4};	/*Profile ID - 23157*/
const Value V23204_1_1 = {S2, NULL};	/*Value = True*/
const Property P23204_1 = {S50, 0, &V23204_1_1, NULL};	/*Property = IsMobile*/
const Value V23204_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23204_5 = {S52, 0, &V23204_5_1277, &P23204_1};	/*Property = ScreenPixelsHeight*/
const Value V23204_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23204_4 = {S54, 0, &V23204_4_2336, &P23204_5};	/*Property = ScreenPixelsWidth*/
const Profile P23204 = {"23204", &P23204_4};	/*Profile ID - 23204*/
const Value V22847_1_1 = {S2, NULL};	/*Value = True*/
const Property P22847_1 = {S50, 0, &V22847_1_1, NULL};	/*Property = IsMobile*/
const Value V22847_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22847_5 = {S52, 0, &V22847_5_59138, &P22847_1};	/*Property = ScreenPixelsHeight*/
const Value V22847_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22847_4 = {S54, 0, &V22847_4_59970, &P22847_5};	/*Property = ScreenPixelsWidth*/
const Profile P22847 = {"22847", &P22847_4};	/*Profile ID - 22847*/
const Value V22943_1_1 = {S2, NULL};	/*Value = True*/
const Property P22943_1 = {S50, 0, &V22943_1_1, NULL};	/*Property = IsMobile*/
const Value V22943_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P22943_5 = {S52, 0, &V22943_5_1832, &P22943_1};	/*Property = ScreenPixelsHeight*/
const Value V22943_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P22943_4 = {S54, 0, &V22943_4_5645, &P22943_5};	/*Property = ScreenPixelsWidth*/
const Profile P22943 = {"22943", &P22943_4};	/*Profile ID - 22943*/
const Value V22871_1_1 = {S2, NULL};	/*Value = True*/
const Property P22871_1 = {S50, 0, &V22871_1_1, NULL};	/*Property = IsMobile*/
const Value V22871_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22871_5 = {S52, 0, &V22871_5_59138, &P22871_1};	/*Property = ScreenPixelsHeight*/
const Value V22871_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22871_4 = {S54, 0, &V22871_4_59970, &P22871_5};	/*Property = ScreenPixelsWidth*/
const Profile P22871 = {"22871", &P22871_4};	/*Profile ID - 22871*/
const Value V20087_1_1 = {S2, NULL};	/*Value = True*/
const Property P20087_1 = {S50, 0, &V20087_1_1, NULL};	/*Property = IsMobile*/
const Value V20087_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20087_5 = {S52, 0, &V20087_5_9981, &P20087_1};	/*Property = ScreenPixelsHeight*/
const Value V20087_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20087_4 = {S54, 0, &V20087_4_39098, &P20087_5};	/*Property = ScreenPixelsWidth*/
const Profile P20087 = {"20087", &P20087_4};	/*Profile ID - 20087*/
const Value V24506_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_279 = {S1, 0, &V24506_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24506_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_282 = {S3, 0, &V24506_282_57155, &P24506_279};	/*Property = BlobBuilder*/
const Value V24506_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_350 = {S4, 0, &V24506_350_57446, &P24506_282};	/*Property = Canvas*/
const Value V24506_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_283 = {S5, 0, &V24506_283_57161, &P24506_350};	/*Property = CssBackground*/
const Value V24506_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_284 = {S6, 0, &V24506_284_57164, &P24506_283};	/*Property = CssBorderImage*/
const Value V24506_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_285 = {S7, 0, &V24506_285_57167, &P24506_284};	/*Property = CssCanvas*/
const Value V24506_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_286 = {S8, 0, &V24506_286_57170, &P24506_285};	/*Property = CssColor*/
const Value V24506_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_237 = {S9, 0, &V24506_237_55786, &P24506_286};	/*Property = CssColumn*/
const Value V24506_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_287 = {S10, 0, &V24506_287_57173, &P24506_237};	/*Property = CssFlexbox*/
const Value V24506_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_288 = {S11, 0, &V24506_288_57176, &P24506_287};	/*Property = CssFont*/
const Value V24506_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_289 = {S12, 0, &V24506_289_86899, &P24506_288};	/*Property = CssImages*/
const Value V24506_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_291 = {S13, 0, &V24506_291_57182, &P24506_289};	/*Property = CssMediaQueries*/
const Value V24506_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_292 = {S14, 0, &V24506_292_57185, &P24506_291};	/*Property = CssMinMax*/
const Value V24506_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_293 = {S15, 0, &V24506_293_57188, &P24506_292};	/*Property = CssOverflow*/
const Value V24506_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_294 = {S16, 0, &V24506_294_57191, &P24506_293};	/*Property = CssPosition*/
const Value V24506_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_295 = {S17, 0, &V24506_295_57194, &P24506_294};	/*Property = CssText*/
const Value V24506_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_297 = {S18, 0, &V24506_297_57197, &P24506_295};	/*Property = CssTransforms*/
const Value V24506_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_241 = {S19, 0, &V24506_241_55801, &P24506_297};	/*Property = CssTransitions*/
const Value V24506_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_298 = {S20, 0, &V24506_298_57208, &P24506_241};	/*Property = CssUI*/
const Value V24506_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_300 = {S21, 0, &V24506_300_57217, &P24506_298};	/*Property = DataSet*/
const Value V24506_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_301 = {S22, 0, &V24506_301_57220, &P24506_300};	/*Property = DataUrl*/
const Value V24506_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_302 = {S23, 0, &V24506_302_57223, &P24506_301};	/*Property = DeviceOrientation*/
const Value V24506_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_304 = {S24, 0, &V24506_304_57229, &P24506_302};	/*Property = FileReader*/
const Value V24506_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_305 = {S25, 0, &V24506_305_57232, &P24506_304};	/*Property = FileSaver*/
const Value V24506_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_306 = {S26, 0, &V24506_306_57238, &P24506_305};	/*Property = FileWriter*/
const Value V24506_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_307 = {S27, 0, &V24506_307_57241, &P24506_306};	/*Property = FormData*/
const Value V24506_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_309 = {S28, 0, &V24506_309_57247, &P24506_307};	/*Property = Fullscreen*/
const Value V24506_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_310 = {S29, 0, &V24506_310_57250, &P24506_309};	/*Property = GeoLocation*/
const Value V24506_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_312 = {S30, 0, &V24506_312_57256, &P24506_310};	/*Property = History*/
const Value V24506_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_314 = {S31, 0, &V24506_314_57262, &P24506_312};	/*Property = Html5*/
const Value V24506_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_313 = {S32, 0, &V24506_313_57259, &P24506_314};	/*Property = Html-Media-Capture*/
const Value V24506_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_315 = {S33, 0, &V24506_315_57265, &P24506_313};	/*Property = Iframe*/
const Value V24506_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_316 = {S34, 0, &V24506_316_57268, &P24506_315};	/*Property = IndexedDB*/
const Value V24506_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_317 = {S35, 0, &V24506_317_57271, &P24506_316};	/*Property = Json*/
const Value V24506_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P24506_15 = {S37, 0, &V24506_15_31, &P24506_317};	/*Property = LayoutEngine*/
const Value V24506_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_344 = {S38, 0, &V24506_344_57274, &P24506_15};	/*Property = Masking*/
const Value V24506_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_320 = {S39, 0, &V24506_320_57292, &P24506_344};	/*Property = PostMessage*/
const Value V24506_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_321 = {S40, 0, &V24506_321_57295, &P24506_320};	/*Property = Progress*/
const Value V24506_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_322 = {S41, 0, &V24506_322_57298, &P24506_321};	/*Property = Prompts*/
const Value V24506_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_323 = {S42, 0, &V24506_323_57301, &P24506_322};	/*Property = Selector*/
const Value V24506_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_324 = {S43, 0, &V24506_324_57307, &P24506_323};	/*Property = Svg*/
const Value V24506_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_327 = {S44, 0, &V24506_327_57316, &P24506_324};	/*Property = TouchEvents*/
const Value V24506_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_328 = {S45, 0, &V24506_328_57319, &P24506_327};	/*Property = Track*/
const Value V24506_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_329 = {S46, 0, &V24506_329_57325, &P24506_328};	/*Property = Video*/
const Value V24506_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_330 = {S47, 0, &V24506_330_57328, &P24506_329};	/*Property = Viewport*/
const Value V24506_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_332 = {S48, 0, &V24506_332_57340, &P24506_330};	/*Property = WebWorkers*/
const Value V24506_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24506_333 = {S49, 0, &V24506_333_57343, &P24506_332};	/*Property = Xhr2*/
const Profile P24506 = {"24506", &P24506_333};	/*Profile ID - 24506*/
const Value V17036_1_1 = {S2, NULL};	/*Value = True*/
const Property P17036_1 = {S50, 0, &V17036_1_1, NULL};	/*Property = IsMobile*/
const Value V17036_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P17036_5 = {S52, 0, &V17036_5_1222, &P17036_1};	/*Property = ScreenPixelsHeight*/
const Value V17036_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17036_4 = {S54, 0, &V17036_4_1224, &P17036_5};	/*Property = ScreenPixelsWidth*/
const Profile P17036 = {"17036", &P17036_4};	/*Profile ID - 17036*/
const Value V23526_1_1 = {S2, NULL};	/*Value = True*/
const Property P23526_1 = {S50, 0, &V23526_1_1, NULL};	/*Property = IsMobile*/
const Value V23526_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23526_5 = {S52, 0, &V23526_5_1227, &P23526_1};	/*Property = ScreenPixelsHeight*/
const Value V23526_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23526_4 = {S54, 0, &V23526_4_1229, &P23526_5};	/*Property = ScreenPixelsWidth*/
const Profile P23526 = {"23526", &P23526_4};	/*Profile ID - 23526*/
const Value V23541_1_1 = {S2, NULL};	/*Value = True*/
const Property P23541_1 = {S50, 0, &V23541_1_1, NULL};	/*Property = IsMobile*/
const Value V23541_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23541_5 = {S52, 0, &V23541_5_9981, &P23541_1};	/*Property = ScreenPixelsHeight*/
const Value V23541_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23541_4 = {S54, 0, &V23541_4_39098, &P23541_5};	/*Property = ScreenPixelsWidth*/
const Profile P23541 = {"23541", &P23541_4};	/*Profile ID - 23541*/
const Value V23795_1_1 = {S2, NULL};	/*Value = True*/
const Property P23795_1 = {S50, 0, &V23795_1_1, NULL};	/*Property = IsMobile*/
const Value V23795_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23795_5 = {S52, 0, &V23795_5_1227, &P23795_1};	/*Property = ScreenPixelsHeight*/
const Value V23795_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23795_4 = {S54, 0, &V23795_4_1229, &P23795_5};	/*Property = ScreenPixelsWidth*/
const Profile P23795 = {"23795", &P23795_4};	/*Profile ID - 23795*/
const Value V23281_1_1 = {S2, NULL};	/*Value = True*/
const Property P23281_1 = {S50, 0, &V23281_1_1, NULL};	/*Property = IsMobile*/
const Value V23281_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23281_5 = {S52, 0, &V23281_5_1277, &P23281_1};	/*Property = ScreenPixelsHeight*/
const Value V23281_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23281_4 = {S54, 0, &V23281_4_1279, &P23281_5};	/*Property = ScreenPixelsWidth*/
const Profile P23281 = {"23281", &P23281_4};	/*Profile ID - 23281*/
const Value V23141_1_1 = {S2, NULL};	/*Value = True*/
const Property P23141_1 = {S50, 0, &V23141_1_1, NULL};	/*Property = IsMobile*/
const Value V23141_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23141_5 = {S52, 0, &V23141_5_1227, &P23141_1};	/*Property = ScreenPixelsHeight*/
const Value V23141_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23141_4 = {S54, 0, &V23141_4_1229, &P23141_5};	/*Property = ScreenPixelsWidth*/
const Profile P23141 = {"23141", &P23141_4};	/*Profile ID - 23141*/
const Value V18611_1_1 = {S2, NULL};	/*Value = True*/
const Property P18611_1 = {S50, 0, &V18611_1_1, NULL};	/*Property = IsMobile*/
const Value V18611_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P18611_5 = {S52, 0, &V18611_5_1227, &P18611_1};	/*Property = ScreenPixelsHeight*/
const Value V18611_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P18611_4 = {S54, 0, &V18611_4_1229, &P18611_5};	/*Property = ScreenPixelsWidth*/
const Profile P18611 = {"18611", &P18611_4};	/*Profile ID - 18611*/
const Value V23328_1_1 = {S2, NULL};	/*Value = True*/
const Property P23328_1 = {S50, 0, &V23328_1_1, NULL};	/*Property = IsMobile*/
const Value V23328_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23328_5 = {S52, 0, &V23328_5_1458, &P23328_1};	/*Property = ScreenPixelsHeight*/
const Value V23328_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23328_4 = {S54, 0, &V23328_4_37617, &P23328_5};	/*Property = ScreenPixelsWidth*/
const Profile P23328 = {"23328", &P23328_4};	/*Profile ID - 23328*/
const Value V23435_1_1 = {S2, NULL};	/*Value = True*/
const Property P23435_1 = {S50, 0, &V23435_1_1, NULL};	/*Property = IsMobile*/
const Value V23435_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23435_5 = {S52, 0, &V23435_5_1227, &P23435_1};	/*Property = ScreenPixelsHeight*/
const Value V23435_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23435_4 = {S54, 0, &V23435_4_1229, &P23435_5};	/*Property = ScreenPixelsWidth*/
const Profile P23435 = {"23435", &P23435_4};	/*Profile ID - 23435*/
const Value V23528_1_1 = {S2, NULL};	/*Value = True*/
const Property P23528_1 = {S50, 0, &V23528_1_1, NULL};	/*Property = IsMobile*/
const Value V23528_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23528_5 = {S52, 0, &V23528_5_1227, &P23528_1};	/*Property = ScreenPixelsHeight*/
const Value V23528_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23528_4 = {S54, 0, &V23528_4_1229, &P23528_5};	/*Property = ScreenPixelsWidth*/
const Profile P23528 = {"23528", &P23528_4};	/*Profile ID - 23528*/
const Value V23736_1_1 = {S2, NULL};	/*Value = True*/
const Property P23736_1 = {S50, 0, &V23736_1_1, NULL};	/*Property = IsMobile*/
const Value V23736_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23736_5 = {S52, 0, &V23736_5_1458, &P23736_1};	/*Property = ScreenPixelsHeight*/
const Value V23736_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23736_4 = {S54, 0, &V23736_4_1460, &P23736_5};	/*Property = ScreenPixelsWidth*/
const Profile P23736 = {"23736", &P23736_4};	/*Profile ID - 23736*/
const Value V23958_1_1 = {S2, NULL};	/*Value = True*/
const Property P23958_1 = {S50, 0, &V23958_1_1, NULL};	/*Property = IsMobile*/
const Value V23958_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23958_5 = {S52, 0, &V23958_5_1277, &P23958_1};	/*Property = ScreenPixelsHeight*/
const Value V23958_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23958_4 = {S54, 0, &V23958_4_1279, &P23958_5};	/*Property = ScreenPixelsWidth*/
const Profile P23958 = {"23958", &P23958_4};	/*Profile ID - 23958*/
const Value V23709_1_1 = {S2, NULL};	/*Value = True*/
const Property P23709_1 = {S50, 0, &V23709_1_1, NULL};	/*Property = IsMobile*/
const Value V23709_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23709_5 = {S52, 0, &V23709_5_1403, &P23709_1};	/*Property = ScreenPixelsHeight*/
const Value V23709_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23709_4 = {S54, 0, &V23709_4_1404, &P23709_5};	/*Property = ScreenPixelsWidth*/
const Profile P23709 = {"23709", &P23709_4};	/*Profile ID - 23709*/
const Value V20848_1_1 = {S2, NULL};	/*Value = True*/
const Property P20848_1 = {S50, 0, &V20848_1_1, NULL};	/*Property = IsMobile*/
const Value V20848_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20848_5 = {S52, 0, &V20848_5_9981, &P20848_1};	/*Property = ScreenPixelsHeight*/
const Value V20848_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20848_4 = {S54, 0, &V20848_4_2336, &P20848_5};	/*Property = ScreenPixelsWidth*/
const Profile P20848 = {"20848", &P20848_4};	/*Profile ID - 20848*/
const Value V23580_1_1 = {S2, NULL};	/*Value = True*/
const Property P23580_1 = {S50, 0, &V23580_1_1, NULL};	/*Property = IsMobile*/
const Value V23580_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23580_5 = {S52, 0, &V23580_5_1227, &P23580_1};	/*Property = ScreenPixelsHeight*/
const Value V23580_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23580_4 = {S54, 0, &V23580_4_1229, &P23580_5};	/*Property = ScreenPixelsWidth*/
const Profile P23580 = {"23580", &P23580_4};	/*Profile ID - 23580*/
const Value V24586_1_1 = {S2, NULL};	/*Value = True*/
const Property P24586_1 = {S50, 0, &V24586_1_1, NULL};	/*Property = IsMobile*/
const Value V24586_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24586_5 = {S52, 0, &V24586_5_1227, &P24586_1};	/*Property = ScreenPixelsHeight*/
const Value V24586_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24586_4 = {S54, 0, &V24586_4_1229, &P24586_5};	/*Property = ScreenPixelsWidth*/
const Profile P24586 = {"24586", &P24586_4};	/*Profile ID - 24586*/
const Value V24585_1_1 = {S2, NULL};	/*Value = True*/
const Property P24585_1 = {S50, 0, &V24585_1_1, NULL};	/*Property = IsMobile*/
const Value V24585_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24585_5 = {S52, 0, &V24585_5_9981, &P24585_1};	/*Property = ScreenPixelsHeight*/
const Value V24585_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24585_4 = {S54, 0, &V24585_4_39098, &P24585_5};	/*Property = ScreenPixelsWidth*/
const Profile P24585 = {"24585", &P24585_4};	/*Profile ID - 24585*/
const Value V23423_1_1 = {S2, NULL};	/*Value = True*/
const Property P23423_1 = {S50, 0, &V23423_1_1, NULL};	/*Property = IsMobile*/
const Value V23423_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23423_5 = {S52, 0, &V23423_5_1227, &P23423_1};	/*Property = ScreenPixelsHeight*/
const Value V23423_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23423_4 = {S54, 0, &V23423_4_1229, &P23423_5};	/*Property = ScreenPixelsWidth*/
const Profile P23423 = {"23423", &P23423_4};	/*Profile ID - 23423*/
const Value V22975_1_1 = {S2, NULL};	/*Value = True*/
const Property P22975_1 = {S50, 0, &V22975_1_1, NULL};	/*Property = IsMobile*/
const Value V22975_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22975_5 = {S52, 0, &V22975_5_1277, &P22975_1};	/*Property = ScreenPixelsHeight*/
const Value V22975_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22975_4 = {S54, 0, &V22975_4_1279, &P22975_5};	/*Property = ScreenPixelsWidth*/
const Profile P22975 = {"22975", &P22975_4};	/*Profile ID - 22975*/
const Value V23321_1_1 = {S2, NULL};	/*Value = True*/
const Property P23321_1 = {S50, 0, &V23321_1_1, NULL};	/*Property = IsMobile*/
const Value V23321_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23321_5 = {S52, 0, &V23321_5_9981, &P23321_1};	/*Property = ScreenPixelsHeight*/
const Value V23321_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23321_4 = {S54, 0, &V23321_4_2336, &P23321_5};	/*Property = ScreenPixelsWidth*/
const Profile P23321 = {"23321", &P23321_4};	/*Profile ID - 23321*/
const Value V23272_1_1 = {S2, NULL};	/*Value = True*/
const Property P23272_1 = {S50, 0, &V23272_1_1, NULL};	/*Property = IsMobile*/
const Value V23272_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23272_5 = {S52, 0, &V23272_5_1227, &P23272_1};	/*Property = ScreenPixelsHeight*/
const Value V23272_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23272_4 = {S54, 0, &V23272_4_1229, &P23272_5};	/*Property = ScreenPixelsWidth*/
const Profile P23272 = {"23272", &P23272_4};	/*Profile ID - 23272*/
const Value V23173_1_1 = {S2, NULL};	/*Value = True*/
const Property P23173_1 = {S50, 0, &V23173_1_1, NULL};	/*Property = IsMobile*/
const Value V23173_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23173_5 = {S52, 0, &V23173_5_1277, &P23173_1};	/*Property = ScreenPixelsHeight*/
const Value V23173_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23173_4 = {S54, 0, &V23173_4_2336, &P23173_5};	/*Property = ScreenPixelsWidth*/
const Profile P23173 = {"23173", &P23173_4};	/*Profile ID - 23173*/
const Value V23386_1_1 = {S2, NULL};	/*Value = True*/
const Property P23386_1 = {S50, 0, &V23386_1_1, NULL};	/*Property = IsMobile*/
const Value V23386_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23386_5 = {S52, 0, &V23386_5_1458, &P23386_1};	/*Property = ScreenPixelsHeight*/
const Value V23386_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23386_4 = {S54, 0, &V23386_4_37617, &P23386_5};	/*Property = ScreenPixelsWidth*/
const Profile P23386 = {"23386", &P23386_4};	/*Profile ID - 23386*/
const Value V24146_1_1 = {S2, NULL};	/*Value = True*/
const Property P24146_1 = {S50, 0, &V24146_1_1, NULL};	/*Property = IsMobile*/
const Value V24146_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24146_5 = {S52, 0, &V24146_5_1277, &P24146_1};	/*Property = ScreenPixelsHeight*/
const Value V24146_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24146_4 = {S54, 0, &V24146_4_1279, &P24146_5};	/*Property = ScreenPixelsWidth*/
const Profile P24146 = {"24146", &P24146_4};	/*Profile ID - 24146*/
const Value V23009_1_1 = {S2, NULL};	/*Value = True*/
const Property P23009_1 = {S50, 0, &V23009_1_1, NULL};	/*Property = IsMobile*/
const Value V23009_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23009_5 = {S52, 0, &V23009_5_1222, &P23009_1};	/*Property = ScreenPixelsHeight*/
const Value V23009_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23009_4 = {S54, 0, &V23009_4_1224, &P23009_5};	/*Property = ScreenPixelsWidth*/
const Profile P23009 = {"23009", &P23009_4};	/*Profile ID - 23009*/
const Value V23126_1_1 = {S2, NULL};	/*Value = True*/
const Property P23126_1 = {S50, 0, &V23126_1_1, NULL};	/*Property = IsMobile*/
const Value V23126_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23126_5 = {S52, 0, &V23126_5_1269, &P23126_1};	/*Property = ScreenPixelsHeight*/
const Value V23126_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23126_4 = {S54, 0, &V23126_4_1229, &P23126_5};	/*Property = ScreenPixelsWidth*/
const Profile P23126 = {"23126", &P23126_4};	/*Profile ID - 23126*/
const Profile P22878 = {"22878", NULL};	/*Profile ID - 22878*/
const Value V23128_1_1 = {S2, NULL};	/*Value = True*/
const Property P23128_1 = {S50, 0, &V23128_1_1, NULL};	/*Property = IsMobile*/
const Value V23128_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23128_5 = {S52, 0, &V23128_5_1277, &P23128_1};	/*Property = ScreenPixelsHeight*/
const Value V23128_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23128_4 = {S54, 0, &V23128_4_1279, &P23128_5};	/*Property = ScreenPixelsWidth*/
const Profile P23128 = {"23128", &P23128_4};	/*Profile ID - 23128*/
const Value V24531_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_279 = {S1, 0, &V24531_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24531_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_282 = {S3, 0, &V24531_282_57155, &P24531_279};	/*Property = BlobBuilder*/
const Value V24531_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_350 = {S4, 0, &V24531_350_57446, &P24531_282};	/*Property = Canvas*/
const Value V24531_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_283 = {S5, 0, &V24531_283_57161, &P24531_350};	/*Property = CssBackground*/
const Value V24531_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_284 = {S6, 0, &V24531_284_57164, &P24531_283};	/*Property = CssBorderImage*/
const Value V24531_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_285 = {S7, 0, &V24531_285_57167, &P24531_284};	/*Property = CssCanvas*/
const Value V24531_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_286 = {S8, 0, &V24531_286_57170, &P24531_285};	/*Property = CssColor*/
const Value V24531_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_237 = {S9, 0, &V24531_237_55786, &P24531_286};	/*Property = CssColumn*/
const Value V24531_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_287 = {S10, 0, &V24531_287_57173, &P24531_237};	/*Property = CssFlexbox*/
const Value V24531_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_288 = {S11, 0, &V24531_288_57176, &P24531_287};	/*Property = CssFont*/
const Value V24531_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_289 = {S12, 0, &V24531_289_86899, &P24531_288};	/*Property = CssImages*/
const Value V24531_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_291 = {S13, 0, &V24531_291_57182, &P24531_289};	/*Property = CssMediaQueries*/
const Value V24531_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_292 = {S14, 0, &V24531_292_57185, &P24531_291};	/*Property = CssMinMax*/
const Value V24531_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_293 = {S15, 0, &V24531_293_57188, &P24531_292};	/*Property = CssOverflow*/
const Value V24531_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_294 = {S16, 0, &V24531_294_57191, &P24531_293};	/*Property = CssPosition*/
const Value V24531_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_295 = {S17, 0, &V24531_295_57194, &P24531_294};	/*Property = CssText*/
const Value V24531_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_297 = {S18, 0, &V24531_297_57197, &P24531_295};	/*Property = CssTransforms*/
const Value V24531_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_241 = {S19, 0, &V24531_241_55801, &P24531_297};	/*Property = CssTransitions*/
const Value V24531_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_298 = {S20, 0, &V24531_298_57208, &P24531_241};	/*Property = CssUI*/
const Value V24531_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_300 = {S21, 0, &V24531_300_57217, &P24531_298};	/*Property = DataSet*/
const Value V24531_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_301 = {S22, 0, &V24531_301_57220, &P24531_300};	/*Property = DataUrl*/
const Value V24531_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_302 = {S23, 0, &V24531_302_57223, &P24531_301};	/*Property = DeviceOrientation*/
const Value V24531_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_304 = {S24, 0, &V24531_304_57229, &P24531_302};	/*Property = FileReader*/
const Value V24531_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_305 = {S25, 0, &V24531_305_57232, &P24531_304};	/*Property = FileSaver*/
const Value V24531_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_306 = {S26, 0, &V24531_306_57238, &P24531_305};	/*Property = FileWriter*/
const Value V24531_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_307 = {S27, 0, &V24531_307_57241, &P24531_306};	/*Property = FormData*/
const Value V24531_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_309 = {S28, 0, &V24531_309_57247, &P24531_307};	/*Property = Fullscreen*/
const Value V24531_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_310 = {S29, 0, &V24531_310_57250, &P24531_309};	/*Property = GeoLocation*/
const Value V24531_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_312 = {S30, 0, &V24531_312_57256, &P24531_310};	/*Property = History*/
const Value V24531_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_314 = {S31, 0, &V24531_314_57262, &P24531_312};	/*Property = Html5*/
const Value V24531_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_313 = {S32, 0, &V24531_313_57259, &P24531_314};	/*Property = Html-Media-Capture*/
const Value V24531_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_315 = {S33, 0, &V24531_315_57265, &P24531_313};	/*Property = Iframe*/
const Value V24531_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_316 = {S34, 0, &V24531_316_57268, &P24531_315};	/*Property = IndexedDB*/
const Value V24531_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_317 = {S35, 0, &V24531_317_57271, &P24531_316};	/*Property = Json*/
const Value V24531_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_15 = {S37, 0, &V24531_15_72, &P24531_317};	/*Property = LayoutEngine*/
const Value V24531_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_344 = {S38, 0, &V24531_344_57274, &P24531_15};	/*Property = Masking*/
const Value V24531_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_320 = {S39, 0, &V24531_320_57292, &P24531_344};	/*Property = PostMessage*/
const Value V24531_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_321 = {S40, 0, &V24531_321_57295, &P24531_320};	/*Property = Progress*/
const Value V24531_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_322 = {S41, 0, &V24531_322_57298, &P24531_321};	/*Property = Prompts*/
const Value V24531_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_323 = {S42, 0, &V24531_323_57301, &P24531_322};	/*Property = Selector*/
const Value V24531_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_324 = {S43, 0, &V24531_324_57307, &P24531_323};	/*Property = Svg*/
const Value V24531_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_327 = {S44, 0, &V24531_327_57316, &P24531_324};	/*Property = TouchEvents*/
const Value V24531_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_328 = {S45, 0, &V24531_328_57319, &P24531_327};	/*Property = Track*/
const Value V24531_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_329 = {S46, 0, &V24531_329_57325, &P24531_328};	/*Property = Video*/
const Value V24531_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_330 = {S47, 0, &V24531_330_57328, &P24531_329};	/*Property = Viewport*/
const Value V24531_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_332 = {S48, 0, &V24531_332_57340, &P24531_330};	/*Property = WebWorkers*/
const Value V24531_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24531_333 = {S49, 0, &V24531_333_57343, &P24531_332};	/*Property = Xhr2*/
const Profile P24531 = {"24531", &P24531_333};	/*Profile ID - 24531*/
const Value V23125_1_1 = {S2, NULL};	/*Value = True*/
const Property P23125_1 = {S50, 0, &V23125_1_1, NULL};	/*Property = IsMobile*/
const Value V23125_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23125_5 = {S52, 0, &V23125_5_1227, &P23125_1};	/*Property = ScreenPixelsHeight*/
const Value V23125_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23125_4 = {S54, 0, &V23125_4_1229, &P23125_5};	/*Property = ScreenPixelsWidth*/
const Profile P23125 = {"23125", &P23125_4};	/*Profile ID - 23125*/
const Value V23033_1_1 = {S2, NULL};	/*Value = True*/
const Property P23033_1 = {S50, 0, &V23033_1_1, NULL};	/*Property = IsMobile*/
const Value V23033_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23033_5 = {S52, 0, &V23033_5_1458, &P23033_1};	/*Property = ScreenPixelsHeight*/
const Value V23033_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23033_4 = {S54, 0, &V23033_4_1460, &P23033_5};	/*Property = ScreenPixelsWidth*/
const Profile P23033 = {"23033", &P23033_4};	/*Profile ID - 23033*/
const Value V23201_1_1 = {S2, NULL};	/*Value = True*/
const Property P23201_1 = {S50, 0, &V23201_1_1, NULL};	/*Property = IsMobile*/
const Value V23201_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23201_5 = {S52, 0, &V23201_5_1269, &P23201_1};	/*Property = ScreenPixelsHeight*/
const Value V23201_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23201_4 = {S54, 0, &V23201_4_1229, &P23201_5};	/*Property = ScreenPixelsWidth*/
const Profile P23201 = {"23201", &P23201_4};	/*Profile ID - 23201*/
const Value V23027_1_1 = {S2, NULL};	/*Value = True*/
const Property P23027_1 = {S50, 0, &V23027_1_1, NULL};	/*Property = IsMobile*/
const Value V23027_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23027_5 = {S52, 0, &V23027_5_1269, &P23027_1};	/*Property = ScreenPixelsHeight*/
const Value V23027_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23027_4 = {S54, 0, &V23027_4_1229, &P23027_5};	/*Property = ScreenPixelsWidth*/
const Profile P23027 = {"23027", &P23027_4};	/*Profile ID - 23027*/
const Value V23143_1_1 = {S2, NULL};	/*Value = True*/
const Property P23143_1 = {S50, 0, &V23143_1_1, NULL};	/*Property = IsMobile*/
const Value V23143_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23143_5 = {S52, 0, &V23143_5_1277, &P23143_1};	/*Property = ScreenPixelsHeight*/
const Value V23143_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23143_4 = {S54, 0, &V23143_4_2336, &P23143_5};	/*Property = ScreenPixelsWidth*/
const Profile P23143 = {"23143", &P23143_4};	/*Profile ID - 23143*/
const Value V23440_1_1 = {S2, NULL};	/*Value = True*/
const Property P23440_1 = {S50, 0, &V23440_1_1, NULL};	/*Property = IsMobile*/
const Value V23440_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23440_5 = {S52, 0, &V23440_5_1227, &P23440_1};	/*Property = ScreenPixelsHeight*/
const Value V23440_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23440_4 = {S54, 0, &V23440_4_1229, &P23440_5};	/*Property = ScreenPixelsWidth*/
const Profile P23440 = {"23440", &P23440_4};	/*Profile ID - 23440*/
const Value V23184_1_1 = {S2, NULL};	/*Value = True*/
const Property P23184_1 = {S50, 0, &V23184_1_1, NULL};	/*Property = IsMobile*/
const Value V23184_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P23184_5 = {S52, 0, &V23184_5_59138, &P23184_1};	/*Property = ScreenPixelsHeight*/
const Value V23184_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P23184_4 = {S54, 0, &V23184_4_59970, &P23184_5};	/*Property = ScreenPixelsWidth*/
const Profile P23184 = {"23184", &P23184_4};	/*Profile ID - 23184*/
const Value V23206_1_1 = {S2, NULL};	/*Value = True*/
const Property P23206_1 = {S50, 0, &V23206_1_1, NULL};	/*Property = IsMobile*/
const Value V23206_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23206_5 = {S52, 0, &V23206_5_1227, &P23206_1};	/*Property = ScreenPixelsHeight*/
const Value V23206_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23206_4 = {S54, 0, &V23206_4_1229, &P23206_5};	/*Property = ScreenPixelsWidth*/
const Profile P23206 = {"23206", &P23206_4};	/*Profile ID - 23206*/
const Value V24077_1_1 = {S2, NULL};	/*Value = True*/
const Property P24077_1 = {S50, 0, &V24077_1_1, NULL};	/*Property = IsMobile*/
const Value V24077_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24077_5 = {S52, 0, &V24077_5_9981, &P24077_1};	/*Property = ScreenPixelsHeight*/
const Value V24077_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24077_4 = {S54, 0, &V24077_4_39098, &P24077_5};	/*Property = ScreenPixelsWidth*/
const Profile P24077 = {"24077", &P24077_4};	/*Profile ID - 24077*/
const Value V23175_1_1 = {S2, NULL};	/*Value = True*/
const Property P23175_1 = {S50, 0, &V23175_1_1, NULL};	/*Property = IsMobile*/
const Value V23175_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23175_5 = {S52, 0, &V23175_5_1227, &P23175_1};	/*Property = ScreenPixelsHeight*/
const Value V23175_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23175_4 = {S54, 0, &V23175_4_1229, &P23175_5};	/*Property = ScreenPixelsWidth*/
const Profile P23175 = {"23175", &P23175_4};	/*Profile ID - 23175*/
const Value V22997_1_1 = {S2, NULL};	/*Value = True*/
const Property P22997_1 = {S50, 0, &V22997_1_1, NULL};	/*Property = IsMobile*/
const Value V22997_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P22997_5 = {S52, 0, &V22997_5_1227, &P22997_1};	/*Property = ScreenPixelsHeight*/
const Value V22997_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22997_4 = {S54, 0, &V22997_4_1229, &P22997_5};	/*Property = ScreenPixelsWidth*/
const Profile P22997 = {"22997", &P22997_4};	/*Profile ID - 22997*/
const Value V23148_1_1 = {S2, NULL};	/*Value = True*/
const Property P23148_1 = {S50, 0, &V23148_1_1, NULL};	/*Property = IsMobile*/
const Value V23148_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23148_5 = {S52, 0, &V23148_5_1227, &P23148_1};	/*Property = ScreenPixelsHeight*/
const Value V23148_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P23148_4 = {S54, 0, &V23148_4_1477, &P23148_5};	/*Property = ScreenPixelsWidth*/
const Profile P23148 = {"23148", &P23148_4};	/*Profile ID - 23148*/
const Value V24714_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_279 = {S1, 0, &V24714_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24714_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_282 = {S3, 0, &V24714_282_57155, &P24714_279};	/*Property = BlobBuilder*/
const Value V24714_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_350 = {S4, 0, &V24714_350_57446, &P24714_282};	/*Property = Canvas*/
const Value V24714_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_283 = {S5, 0, &V24714_283_57161, &P24714_350};	/*Property = CssBackground*/
const Value V24714_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_284 = {S6, 0, &V24714_284_57164, &P24714_283};	/*Property = CssBorderImage*/
const Value V24714_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_285 = {S7, 0, &V24714_285_57167, &P24714_284};	/*Property = CssCanvas*/
const Value V24714_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_286 = {S8, 0, &V24714_286_57170, &P24714_285};	/*Property = CssColor*/
const Value V24714_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_237 = {S9, 0, &V24714_237_55786, &P24714_286};	/*Property = CssColumn*/
const Value V24714_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_287 = {S10, 0, &V24714_287_57173, &P24714_237};	/*Property = CssFlexbox*/
const Value V24714_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_288 = {S11, 0, &V24714_288_57176, &P24714_287};	/*Property = CssFont*/
const Value V24714_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_289 = {S12, 0, &V24714_289_86899, &P24714_288};	/*Property = CssImages*/
const Value V24714_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_291 = {S13, 0, &V24714_291_57182, &P24714_289};	/*Property = CssMediaQueries*/
const Value V24714_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_292 = {S14, 0, &V24714_292_57185, &P24714_291};	/*Property = CssMinMax*/
const Value V24714_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_293 = {S15, 0, &V24714_293_57188, &P24714_292};	/*Property = CssOverflow*/
const Value V24714_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_294 = {S16, 0, &V24714_294_57191, &P24714_293};	/*Property = CssPosition*/
const Value V24714_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_295 = {S17, 0, &V24714_295_57194, &P24714_294};	/*Property = CssText*/
const Value V24714_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_297 = {S18, 0, &V24714_297_57197, &P24714_295};	/*Property = CssTransforms*/
const Value V24714_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_241 = {S19, 0, &V24714_241_55801, &P24714_297};	/*Property = CssTransitions*/
const Value V24714_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_298 = {S20, 0, &V24714_298_57208, &P24714_241};	/*Property = CssUI*/
const Value V24714_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_300 = {S21, 0, &V24714_300_57217, &P24714_298};	/*Property = DataSet*/
const Value V24714_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_301 = {S22, 0, &V24714_301_57220, &P24714_300};	/*Property = DataUrl*/
const Value V24714_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_302 = {S23, 0, &V24714_302_57223, &P24714_301};	/*Property = DeviceOrientation*/
const Value V24714_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_304 = {S24, 0, &V24714_304_57229, &P24714_302};	/*Property = FileReader*/
const Value V24714_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_305 = {S25, 0, &V24714_305_57232, &P24714_304};	/*Property = FileSaver*/
const Value V24714_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_306 = {S26, 0, &V24714_306_57238, &P24714_305};	/*Property = FileWriter*/
const Value V24714_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_307 = {S27, 0, &V24714_307_57241, &P24714_306};	/*Property = FormData*/
const Value V24714_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_309 = {S28, 0, &V24714_309_57247, &P24714_307};	/*Property = Fullscreen*/
const Value V24714_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_310 = {S29, 0, &V24714_310_57250, &P24714_309};	/*Property = GeoLocation*/
const Value V24714_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_312 = {S30, 0, &V24714_312_57256, &P24714_310};	/*Property = History*/
const Value V24714_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_314 = {S31, 0, &V24714_314_57262, &P24714_312};	/*Property = Html5*/
const Value V24714_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_313 = {S32, 0, &V24714_313_57259, &P24714_314};	/*Property = Html-Media-Capture*/
const Value V24714_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_315 = {S33, 0, &V24714_315_57265, &P24714_313};	/*Property = Iframe*/
const Value V24714_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_316 = {S34, 0, &V24714_316_57268, &P24714_315};	/*Property = IndexedDB*/
const Value V24714_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_317 = {S35, 0, &V24714_317_57271, &P24714_316};	/*Property = Json*/
const Value V24714_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24714_15 = {S37, 0, &V24714_15_28, &P24714_317};	/*Property = LayoutEngine*/
const Value V24714_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_344 = {S38, 0, &V24714_344_57274, &P24714_15};	/*Property = Masking*/
const Value V24714_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_320 = {S39, 0, &V24714_320_57292, &P24714_344};	/*Property = PostMessage*/
const Value V24714_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_321 = {S40, 0, &V24714_321_57295, &P24714_320};	/*Property = Progress*/
const Value V24714_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_322 = {S41, 0, &V24714_322_57298, &P24714_321};	/*Property = Prompts*/
const Value V24714_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_323 = {S42, 0, &V24714_323_57301, &P24714_322};	/*Property = Selector*/
const Value V24714_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_324 = {S43, 0, &V24714_324_57307, &P24714_323};	/*Property = Svg*/
const Value V24714_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_327 = {S44, 0, &V24714_327_57316, &P24714_324};	/*Property = TouchEvents*/
const Value V24714_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_328 = {S45, 0, &V24714_328_57319, &P24714_327};	/*Property = Track*/
const Value V24714_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_329 = {S46, 0, &V24714_329_57325, &P24714_328};	/*Property = Video*/
const Value V24714_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_330 = {S47, 0, &V24714_330_57328, &P24714_329};	/*Property = Viewport*/
const Value V24714_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_332 = {S48, 0, &V24714_332_57340, &P24714_330};	/*Property = WebWorkers*/
const Value V24714_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24714_333 = {S49, 0, &V24714_333_57343, &P24714_332};	/*Property = Xhr2*/
const Profile P24714 = {"24714", &P24714_333};	/*Profile ID - 24714*/
const Value V22879_279_57147 = {S2, NULL};	/*Value = True*/
const Property P22879_279 = {S1, 0, &V22879_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V22879_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22879_282 = {S3, 0, &V22879_282_57154, &P22879_279};	/*Property = BlobBuilder*/
const Value V22879_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22879_350 = {S4, 0, &V22879_350_57444, &P22879_282};	/*Property = Canvas*/
const Value V22879_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22879_283 = {S5, 0, &V22879_283_57159, &P22879_350};	/*Property = CssBackground*/
const Value V22879_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22879_284 = {S6, 0, &V22879_284_57162, &P22879_283};	/*Property = CssBorderImage*/
const Value V22879_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22879_285 = {S7, 0, &V22879_285_57166, &P22879_284};	/*Property = CssCanvas*/
const Value V22879_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22879_286 = {S8, 0, &V22879_286_57168, &P22879_285};	/*Property = CssColor*/
const Value V22879_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22879_237 = {S9, 0, &V22879_237_55786, &P22879_286};	/*Property = CssColumn*/
const Value V22879_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22879_287 = {S10, 0, &V22879_287_57172, &P22879_237};	/*Property = CssFlexbox*/
const Value V22879_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22879_288 = {S11, 0, &V22879_288_57175, &P22879_287};	/*Property = CssFont*/
const Value V22879_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22879_289 = {S12, 0, &V22879_289_57759, &P22879_288};	/*Property = CssImages*/
const Value V22879_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22879_291 = {S13, 0, &V22879_291_57180, &P22879_289};	/*Property = CssMediaQueries*/
const Value V22879_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22879_292 = {S14, 0, &V22879_292_57183, &P22879_291};	/*Property = CssMinMax*/
const Value V22879_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22879_293 = {S15, 0, &V22879_293_57187, &P22879_292};	/*Property = CssOverflow*/
const Value V22879_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22879_294 = {S16, 0, &V22879_294_57189, &P22879_293};	/*Property = CssPosition*/
const Value V22879_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22879_295 = {S17, 0, &V22879_295_57192, &P22879_294};	/*Property = CssText*/
const Value V22879_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22879_297 = {S18, 0, &V22879_297_57195, &P22879_295};	/*Property = CssTransforms*/
const Value V22879_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22879_241 = {S19, 0, &V22879_241_55799, &P22879_297};	/*Property = CssTransitions*/
const Value V22879_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22879_298 = {S20, 0, &V22879_298_57206, &P22879_241};	/*Property = CssUI*/
const Value V22879_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22879_300 = {S21, 0, &V22879_300_57215, &P22879_298};	/*Property = DataSet*/
const Value V22879_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22879_301 = {S22, 0, &V22879_301_57218, &P22879_300};	/*Property = DataUrl*/
const Value V22879_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22879_302 = {S23, 0, &V22879_302_57221, &P22879_301};	/*Property = DeviceOrientation*/
const Value V22879_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22879_304 = {S24, 0, &V22879_304_57227, &P22879_302};	/*Property = FileReader*/
const Value V22879_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22879_305 = {S25, 0, &V22879_305_57231, &P22879_304};	/*Property = FileSaver*/
const Value V22879_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22879_306 = {S26, 0, &V22879_306_57237, &P22879_305};	/*Property = FileWriter*/
const Value V22879_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22879_307 = {S27, 0, &V22879_307_57239, &P22879_306};	/*Property = FormData*/
const Value V22879_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22879_309 = {S28, 0, &V22879_309_57246, &P22879_307};	/*Property = Fullscreen*/
const Value V22879_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22879_310 = {S29, 0, &V22879_310_57248, &P22879_309};	/*Property = GeoLocation*/
const Value V22879_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22879_312 = {S30, 0, &V22879_312_57254, &P22879_310};	/*Property = History*/
const Value V22879_314_57260 = {S2, NULL};	/*Value = True*/
const Property P22879_314 = {S31, 0, &V22879_314_57260, &P22879_312};	/*Property = Html5*/
const Value V22879_313_57257 = {S2, NULL};	/*Value = True*/
const Property P22879_313 = {S32, 0, &V22879_313_57257, &P22879_314};	/*Property = Html-Media-Capture*/
const Value V22879_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22879_315 = {S33, 0, &V22879_315_57264, &P22879_313};	/*Property = Iframe*/
const Value V22879_316_57266 = {S2, NULL};	/*Value = True*/
const Property P22879_316 = {S34, 0, &V22879_316_57266, &P22879_315};	/*Property = IndexedDB*/
const Value V22879_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22879_317 = {S35, 0, &V22879_317_57269, &P22879_316};	/*Property = Json*/
const Value V22879_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P22879_15 = {S37, 0, &V22879_15_54167, &P22879_317};	/*Property = LayoutEngine*/
const Value V22879_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22879_344 = {S38, 0, &V22879_344_57272, &P22879_15};	/*Property = Masking*/
const Value V22879_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22879_320 = {S39, 0, &V22879_320_57290, &P22879_344};	/*Property = PostMessage*/
const Value V22879_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22879_321 = {S40, 0, &V22879_321_57293, &P22879_320};	/*Property = Progress*/
const Value V22879_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22879_322 = {S41, 0, &V22879_322_57296, &P22879_321};	/*Property = Prompts*/
const Value V22879_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22879_323 = {S42, 0, &V22879_323_57299, &P22879_322};	/*Property = Selector*/
const Value V22879_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22879_324 = {S43, 0, &V22879_324_57306, &P22879_323};	/*Property = Svg*/
const Value V22879_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22879_327 = {S44, 0, &V22879_327_57314, &P22879_324};	/*Property = TouchEvents*/
const Value V22879_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22879_328 = {S45, 0, &V22879_328_57318, &P22879_327};	/*Property = Track*/
const Value V22879_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22879_329 = {S46, 0, &V22879_329_57323, &P22879_328};	/*Property = Video*/
const Value V22879_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22879_330 = {S47, 0, &V22879_330_57326, &P22879_329};	/*Property = Viewport*/
const Value V22879_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22879_332 = {S48, 0, &V22879_332_57339, &P22879_330};	/*Property = WebWorkers*/
const Value V22879_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22879_333 = {S49, 0, &V22879_333_57342, &P22879_332};	/*Property = Xhr2*/
const Profile P22879 = {"22879", &P22879_333};	/*Profile ID - 22879*/
const Value V23161_1_1 = {S2, NULL};	/*Value = True*/
const Property P23161_1 = {S50, 0, &V23161_1_1, NULL};	/*Property = IsMobile*/
const Value V23161_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23161_5 = {S52, 0, &V23161_5_9981, &P23161_1};	/*Property = ScreenPixelsHeight*/
const Value V23161_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23161_4 = {S54, 0, &V23161_4_39098, &P23161_5};	/*Property = ScreenPixelsWidth*/
const Profile P23161 = {"23161", &P23161_4};	/*Profile ID - 23161*/
const Value V22881_279_57148 = {S0, NULL};	/*Value = False*/
const Property P22881_279 = {S1, 0, &V22881_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V22881_282_57154 = {S0, NULL};	/*Value = False*/
const Property P22881_282 = {S3, 0, &V22881_282_57154, &P22881_279};	/*Property = BlobBuilder*/
const Value V22881_350_57444 = {S2, NULL};	/*Value = True*/
const Property P22881_350 = {S4, 0, &V22881_350_57444, &P22881_282};	/*Property = Canvas*/
const Value V22881_283_57159 = {S2, NULL};	/*Value = True*/
const Property P22881_283 = {S5, 0, &V22881_283_57159, &P22881_350};	/*Property = CssBackground*/
const Value V22881_284_57162 = {S2, NULL};	/*Value = True*/
const Property P22881_284 = {S6, 0, &V22881_284_57162, &P22881_283};	/*Property = CssBorderImage*/
const Value V22881_285_57166 = {S0, NULL};	/*Value = False*/
const Property P22881_285 = {S7, 0, &V22881_285_57166, &P22881_284};	/*Property = CssCanvas*/
const Value V22881_286_57168 = {S2, NULL};	/*Value = True*/
const Property P22881_286 = {S8, 0, &V22881_286_57168, &P22881_285};	/*Property = CssColor*/
const Value V22881_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P22881_237 = {S9, 0, &V22881_237_55786, &P22881_286};	/*Property = CssColumn*/
const Value V22881_287_57172 = {S0, NULL};	/*Value = False*/
const Property P22881_287 = {S10, 0, &V22881_287_57172, &P22881_237};	/*Property = CssFlexbox*/
const Value V22881_288_57175 = {S0, NULL};	/*Value = False*/
const Property P22881_288 = {S11, 0, &V22881_288_57175, &P22881_287};	/*Property = CssFont*/
const Value V22881_289_57759 = {S0, NULL};	/*Value = False*/
const Property P22881_289 = {S12, 0, &V22881_289_57759, &P22881_288};	/*Property = CssImages*/
const Value V22881_291_57180 = {S2, NULL};	/*Value = True*/
const Property P22881_291 = {S13, 0, &V22881_291_57180, &P22881_289};	/*Property = CssMediaQueries*/
const Value V22881_292_57183 = {S2, NULL};	/*Value = True*/
const Property P22881_292 = {S14, 0, &V22881_292_57183, &P22881_291};	/*Property = CssMinMax*/
const Value V22881_293_57187 = {S0, NULL};	/*Value = False*/
const Property P22881_293 = {S15, 0, &V22881_293_57187, &P22881_292};	/*Property = CssOverflow*/
const Value V22881_294_57189 = {S2, NULL};	/*Value = True*/
const Property P22881_294 = {S16, 0, &V22881_294_57189, &P22881_293};	/*Property = CssPosition*/
const Value V22881_295_57192 = {S2, NULL};	/*Value = True*/
const Property P22881_295 = {S17, 0, &V22881_295_57192, &P22881_294};	/*Property = CssText*/
const Value V22881_297_57195 = {S2, NULL};	/*Value = True*/
const Property P22881_297 = {S18, 0, &V22881_297_57195, &P22881_295};	/*Property = CssTransforms*/
const Value V22881_241_55799 = {S2, NULL};	/*Value = True*/
const Property P22881_241 = {S19, 0, &V22881_241_55799, &P22881_297};	/*Property = CssTransitions*/
const Value V22881_298_57206 = {S2, NULL};	/*Value = True*/
const Property P22881_298 = {S20, 0, &V22881_298_57206, &P22881_241};	/*Property = CssUI*/
const Value V22881_300_57215 = {S2, NULL};	/*Value = True*/
const Property P22881_300 = {S21, 0, &V22881_300_57215, &P22881_298};	/*Property = DataSet*/
const Value V22881_301_57218 = {S2, NULL};	/*Value = True*/
const Property P22881_301 = {S22, 0, &V22881_301_57218, &P22881_300};	/*Property = DataUrl*/
const Value V22881_302_57221 = {S2, NULL};	/*Value = True*/
const Property P22881_302 = {S23, 0, &V22881_302_57221, &P22881_301};	/*Property = DeviceOrientation*/
const Value V22881_304_57227 = {S2, NULL};	/*Value = True*/
const Property P22881_304 = {S24, 0, &V22881_304_57227, &P22881_302};	/*Property = FileReader*/
const Value V22881_305_57231 = {S0, NULL};	/*Value = False*/
const Property P22881_305 = {S25, 0, &V22881_305_57231, &P22881_304};	/*Property = FileSaver*/
const Value V22881_306_57237 = {S0, NULL};	/*Value = False*/
const Property P22881_306 = {S26, 0, &V22881_306_57237, &P22881_305};	/*Property = FileWriter*/
const Value V22881_307_57239 = {S2, NULL};	/*Value = True*/
const Property P22881_307 = {S27, 0, &V22881_307_57239, &P22881_306};	/*Property = FormData*/
const Value V22881_309_57246 = {S0, NULL};	/*Value = False*/
const Property P22881_309 = {S28, 0, &V22881_309_57246, &P22881_307};	/*Property = Fullscreen*/
const Value V22881_310_57248 = {S2, NULL};	/*Value = True*/
const Property P22881_310 = {S29, 0, &V22881_310_57248, &P22881_309};	/*Property = GeoLocation*/
const Value V22881_312_57254 = {S2, NULL};	/*Value = True*/
const Property P22881_312 = {S30, 0, &V22881_312_57254, &P22881_310};	/*Property = History*/
const Value V22881_314_57261 = {S0, NULL};	/*Value = False*/
const Property P22881_314 = {S31, 0, &V22881_314_57261, &P22881_312};	/*Property = Html5*/
const Value V22881_313_57258 = {S0, NULL};	/*Value = False*/
const Property P22881_313 = {S32, 0, &V22881_313_57258, &P22881_314};	/*Property = Html-Media-Capture*/
const Value V22881_315_57264 = {S0, NULL};	/*Value = False*/
const Property P22881_315 = {S33, 0, &V22881_315_57264, &P22881_313};	/*Property = Iframe*/
const Value V22881_316_57267 = {S0, NULL};	/*Value = False*/
const Property P22881_316 = {S34, 0, &V22881_316_57267, &P22881_315};	/*Property = IndexedDB*/
const Value V22881_317_57269 = {S2, NULL};	/*Value = True*/
const Property P22881_317 = {S35, 0, &V22881_317_57269, &P22881_316};	/*Property = Json*/
const Value V22881_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P22881_15 = {S37, 0, &V22881_15_28, &P22881_317};	/*Property = LayoutEngine*/
const Value V22881_344_57272 = {S2, NULL};	/*Value = True*/
const Property P22881_344 = {S38, 0, &V22881_344_57272, &P22881_15};	/*Property = Masking*/
const Value V22881_320_57290 = {S2, NULL};	/*Value = True*/
const Property P22881_320 = {S39, 0, &V22881_320_57290, &P22881_344};	/*Property = PostMessage*/
const Value V22881_321_57293 = {S2, NULL};	/*Value = True*/
const Property P22881_321 = {S40, 0, &V22881_321_57293, &P22881_320};	/*Property = Progress*/
const Value V22881_322_57296 = {S2, NULL};	/*Value = True*/
const Property P22881_322 = {S41, 0, &V22881_322_57296, &P22881_321};	/*Property = Prompts*/
const Value V22881_323_57299 = {S2, NULL};	/*Value = True*/
const Property P22881_323 = {S42, 0, &V22881_323_57299, &P22881_322};	/*Property = Selector*/
const Value V22881_324_57306 = {S0, NULL};	/*Value = False*/
const Property P22881_324 = {S43, 0, &V22881_324_57306, &P22881_323};	/*Property = Svg*/
const Value V22881_327_57314 = {S2, NULL};	/*Value = True*/
const Property P22881_327 = {S44, 0, &V22881_327_57314, &P22881_324};	/*Property = TouchEvents*/
const Value V22881_328_57318 = {S0, NULL};	/*Value = False*/
const Property P22881_328 = {S45, 0, &V22881_328_57318, &P22881_327};	/*Property = Track*/
const Value V22881_329_57323 = {S2, NULL};	/*Value = True*/
const Property P22881_329 = {S46, 0, &V22881_329_57323, &P22881_328};	/*Property = Video*/
const Value V22881_330_57326 = {S2, NULL};	/*Value = True*/
const Property P22881_330 = {S47, 0, &V22881_330_57326, &P22881_329};	/*Property = Viewport*/
const Value V22881_332_57339 = {S0, NULL};	/*Value = False*/
const Property P22881_332 = {S48, 0, &V22881_332_57339, &P22881_330};	/*Property = WebWorkers*/
const Value V22881_333_57342 = {S0, NULL};	/*Value = False*/
const Property P22881_333 = {S49, 0, &V22881_333_57342, &P22881_332};	/*Property = Xhr2*/
const Profile P22881 = {"22881", &P22881_333};	/*Profile ID - 22881*/
const Value V23030_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_279 = {S1, 0, &V23030_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23030_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_282 = {S3, 0, &V23030_282_57155, &P23030_279};	/*Property = BlobBuilder*/
const Value V23030_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_350 = {S4, 0, &V23030_350_57446, &P23030_282};	/*Property = Canvas*/
const Value V23030_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_283 = {S5, 0, &V23030_283_57161, &P23030_350};	/*Property = CssBackground*/
const Value V23030_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_284 = {S6, 0, &V23030_284_57164, &P23030_283};	/*Property = CssBorderImage*/
const Value V23030_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_285 = {S7, 0, &V23030_285_57167, &P23030_284};	/*Property = CssCanvas*/
const Value V23030_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_286 = {S8, 0, &V23030_286_57170, &P23030_285};	/*Property = CssColor*/
const Value V23030_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_237 = {S9, 0, &V23030_237_55786, &P23030_286};	/*Property = CssColumn*/
const Value V23030_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_287 = {S10, 0, &V23030_287_57173, &P23030_237};	/*Property = CssFlexbox*/
const Value V23030_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_288 = {S11, 0, &V23030_288_57176, &P23030_287};	/*Property = CssFont*/
const Value V23030_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_289 = {S12, 0, &V23030_289_86899, &P23030_288};	/*Property = CssImages*/
const Value V23030_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_291 = {S13, 0, &V23030_291_57182, &P23030_289};	/*Property = CssMediaQueries*/
const Value V23030_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_292 = {S14, 0, &V23030_292_57185, &P23030_291};	/*Property = CssMinMax*/
const Value V23030_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_293 = {S15, 0, &V23030_293_57188, &P23030_292};	/*Property = CssOverflow*/
const Value V23030_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_294 = {S16, 0, &V23030_294_57191, &P23030_293};	/*Property = CssPosition*/
const Value V23030_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_295 = {S17, 0, &V23030_295_57194, &P23030_294};	/*Property = CssText*/
const Value V23030_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_297 = {S18, 0, &V23030_297_57197, &P23030_295};	/*Property = CssTransforms*/
const Value V23030_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_241 = {S19, 0, &V23030_241_55801, &P23030_297};	/*Property = CssTransitions*/
const Value V23030_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_298 = {S20, 0, &V23030_298_57208, &P23030_241};	/*Property = CssUI*/
const Value V23030_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_300 = {S21, 0, &V23030_300_57217, &P23030_298};	/*Property = DataSet*/
const Value V23030_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_301 = {S22, 0, &V23030_301_57220, &P23030_300};	/*Property = DataUrl*/
const Value V23030_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_302 = {S23, 0, &V23030_302_57223, &P23030_301};	/*Property = DeviceOrientation*/
const Value V23030_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_304 = {S24, 0, &V23030_304_57229, &P23030_302};	/*Property = FileReader*/
const Value V23030_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_305 = {S25, 0, &V23030_305_57232, &P23030_304};	/*Property = FileSaver*/
const Value V23030_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_306 = {S26, 0, &V23030_306_57238, &P23030_305};	/*Property = FileWriter*/
const Value V23030_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_307 = {S27, 0, &V23030_307_57241, &P23030_306};	/*Property = FormData*/
const Value V23030_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_309 = {S28, 0, &V23030_309_57247, &P23030_307};	/*Property = Fullscreen*/
const Value V23030_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_310 = {S29, 0, &V23030_310_57250, &P23030_309};	/*Property = GeoLocation*/
const Value V23030_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_312 = {S30, 0, &V23030_312_57256, &P23030_310};	/*Property = History*/
const Value V23030_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_314 = {S31, 0, &V23030_314_57262, &P23030_312};	/*Property = Html5*/
const Value V23030_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_313 = {S32, 0, &V23030_313_57259, &P23030_314};	/*Property = Html-Media-Capture*/
const Value V23030_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_315 = {S33, 0, &V23030_315_57265, &P23030_313};	/*Property = Iframe*/
const Value V23030_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_316 = {S34, 0, &V23030_316_57268, &P23030_315};	/*Property = IndexedDB*/
const Value V23030_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_317 = {S35, 0, &V23030_317_57271, &P23030_316};	/*Property = Json*/
const Value V23030_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P23030_15 = {S37, 0, &V23030_15_28, &P23030_317};	/*Property = LayoutEngine*/
const Value V23030_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_344 = {S38, 0, &V23030_344_57274, &P23030_15};	/*Property = Masking*/
const Value V23030_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_320 = {S39, 0, &V23030_320_57292, &P23030_344};	/*Property = PostMessage*/
const Value V23030_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_321 = {S40, 0, &V23030_321_57295, &P23030_320};	/*Property = Progress*/
const Value V23030_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_322 = {S41, 0, &V23030_322_57298, &P23030_321};	/*Property = Prompts*/
const Value V23030_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_323 = {S42, 0, &V23030_323_57301, &P23030_322};	/*Property = Selector*/
const Value V23030_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_324 = {S43, 0, &V23030_324_57307, &P23030_323};	/*Property = Svg*/
const Value V23030_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_327 = {S44, 0, &V23030_327_57316, &P23030_324};	/*Property = TouchEvents*/
const Value V23030_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_328 = {S45, 0, &V23030_328_57319, &P23030_327};	/*Property = Track*/
const Value V23030_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_329 = {S46, 0, &V23030_329_57325, &P23030_328};	/*Property = Video*/
const Value V23030_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_330 = {S47, 0, &V23030_330_57328, &P23030_329};	/*Property = Viewport*/
const Value V23030_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_332 = {S48, 0, &V23030_332_57340, &P23030_330};	/*Property = WebWorkers*/
const Value V23030_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23030_333 = {S49, 0, &V23030_333_57343, &P23030_332};	/*Property = Xhr2*/
const Profile P23030 = {"23030", &P23030_333};	/*Profile ID - 23030*/
const Value V23677_1_2 = {S0, NULL};	/*Value = False*/
const Property P23677_1 = {S50, 0, &V23677_1_2, NULL};	/*Property = IsMobile*/
const Value V23677_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23677_5 = {S52, 0, &V23677_5_60417, &P23677_1};	/*Property = ScreenPixelsHeight*/
const Value V23677_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23677_4 = {S54, 0, &V23677_4_57095, &P23677_5};	/*Property = ScreenPixelsWidth*/
const Profile P23677 = {"23677", &P23677_4};	/*Profile ID - 23677*/
const Value V23375_1_1 = {S2, NULL};	/*Value = True*/
const Property P23375_1 = {S50, 0, &V23375_1_1, NULL};	/*Property = IsMobile*/
const Value V23375_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23375_5 = {S52, 0, &V23375_5_37, &P23375_1};	/*Property = ScreenPixelsHeight*/
const Value V23375_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23375_4 = {S54, 0, &V23375_4_36, &P23375_5};	/*Property = ScreenPixelsWidth*/
const Profile P23375 = {"23375", &P23375_4};	/*Profile ID - 23375*/
const Value V22986_1_1 = {S2, NULL};	/*Value = True*/
const Property P22986_1 = {S50, 0, &V22986_1_1, NULL};	/*Property = IsMobile*/
const Value V22986_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P22986_5 = {S52, 0, &V22986_5_1304, &P22986_1};	/*Property = ScreenPixelsHeight*/
const Value V22986_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22986_4 = {S54, 0, &V22986_4_1279, &P22986_5};	/*Property = ScreenPixelsWidth*/
const Profile P22986 = {"22986", &P22986_4};	/*Profile ID - 22986*/
const Value V23753_1_1 = {S2, NULL};	/*Value = True*/
const Property P23753_1 = {S50, 0, &V23753_1_1, NULL};	/*Property = IsMobile*/
const Value V23753_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23753_5 = {S52, 0, &V23753_5_1458, &P23753_1};	/*Property = ScreenPixelsHeight*/
const Value V23753_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23753_4 = {S54, 0, &V23753_4_37617, &P23753_5};	/*Property = ScreenPixelsWidth*/
const Profile P23753 = {"23753", &P23753_4};	/*Profile ID - 23753*/
const Value V23234_1_1 = {S2, NULL};	/*Value = True*/
const Property P23234_1 = {S50, 0, &V23234_1_1, NULL};	/*Property = IsMobile*/
const Value V23234_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23234_5 = {S52, 0, &V23234_5_9981, &P23234_1};	/*Property = ScreenPixelsHeight*/
const Value V23234_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23234_4 = {S54, 0, &V23234_4_39098, &P23234_5};	/*Property = ScreenPixelsWidth*/
const Profile P23234 = {"23234", &P23234_4};	/*Profile ID - 23234*/
const Value V23147_1_1 = {S2, NULL};	/*Value = True*/
const Property P23147_1 = {S50, 0, &V23147_1_1, NULL};	/*Property = IsMobile*/
const Value V23147_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23147_5 = {S52, 0, &V23147_5_1403, &P23147_1};	/*Property = ScreenPixelsHeight*/
const Value V23147_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23147_4 = {S54, 0, &V23147_4_1404, &P23147_5};	/*Property = ScreenPixelsWidth*/
const Profile P23147 = {"23147", &P23147_4};	/*Profile ID - 23147*/
const Value V23334_1_1 = {S2, NULL};	/*Value = True*/
const Property P23334_1 = {S50, 0, &V23334_1_1, NULL};	/*Property = IsMobile*/
const Value V23334_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23334_5 = {S52, 0, &V23334_5_1277, &P23334_1};	/*Property = ScreenPixelsHeight*/
const Value V23334_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23334_4 = {S54, 0, &V23334_4_2336, &P23334_5};	/*Property = ScreenPixelsWidth*/
const Profile P23334 = {"23334", &P23334_4};	/*Profile ID - 23334*/
const Value V23339_1_1 = {S2, NULL};	/*Value = True*/
const Property P23339_1 = {S50, 0, &V23339_1_1, NULL};	/*Property = IsMobile*/
const Value V23339_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23339_5 = {S52, 0, &V23339_5_1227, &P23339_1};	/*Property = ScreenPixelsHeight*/
const Value V23339_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23339_4 = {S54, 0, &V23339_4_1229, &P23339_5};	/*Property = ScreenPixelsWidth*/
const Profile P23339 = {"23339", &P23339_4};	/*Profile ID - 23339*/
const Value V23306_1_1 = {S2, NULL};	/*Value = True*/
const Property P23306_1 = {S50, 0, &V23306_1_1, NULL};	/*Property = IsMobile*/
const Value V23306_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23306_5 = {S52, 0, &V23306_5_1277, &P23306_1};	/*Property = ScreenPixelsHeight*/
const Value V23306_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23306_4 = {S54, 0, &V23306_4_2336, &P23306_5};	/*Property = ScreenPixelsWidth*/
const Profile P23306 = {"23306", &P23306_4};	/*Profile ID - 23306*/
const Value V24098_1_1 = {S2, NULL};	/*Value = True*/
const Property P24098_1 = {S50, 0, &V24098_1_1, NULL};	/*Property = IsMobile*/
const Value V24098_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24098_5 = {S52, 0, &V24098_5_1403, &P24098_1};	/*Property = ScreenPixelsHeight*/
const Value V24098_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24098_4 = {S54, 0, &V24098_4_1404, &P24098_5};	/*Property = ScreenPixelsWidth*/
const Profile P24098 = {"24098", &P24098_4};	/*Profile ID - 24098*/
const Value V22990_1_1 = {S2, NULL};	/*Value = True*/
const Property P22990_1 = {S50, 0, &V22990_1_1, NULL};	/*Property = IsMobile*/
const Value V22990_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P22990_5 = {S52, 0, &V22990_5_1269, &P22990_1};	/*Property = ScreenPixelsHeight*/
const Value V22990_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P22990_4 = {S54, 0, &V22990_4_1229, &P22990_5};	/*Property = ScreenPixelsWidth*/
const Profile P22990 = {"22990", &P22990_4};	/*Profile ID - 22990*/
const Value V23014_1_1 = {S2, NULL};	/*Value = True*/
const Property P23014_1 = {S50, 0, &V23014_1_1, NULL};	/*Property = IsMobile*/
const Value V23014_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23014_5 = {S52, 0, &V23014_5_1458, &P23014_1};	/*Property = ScreenPixelsHeight*/
const Value V23014_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23014_4 = {S54, 0, &V23014_4_1460, &P23014_5};	/*Property = ScreenPixelsWidth*/
const Profile P23014 = {"23014", &P23014_4};	/*Profile ID - 23014*/
const Value V24582_1_1 = {S2, NULL};	/*Value = True*/
const Property P24582_1 = {S50, 0, &V24582_1_1, NULL};	/*Property = IsMobile*/
const Value V24582_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24582_5 = {S52, 0, &V24582_5_1277, &P24582_1};	/*Property = ScreenPixelsHeight*/
const Value V24582_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24582_4 = {S54, 0, &V24582_4_1279, &P24582_5};	/*Property = ScreenPixelsWidth*/
const Profile P24582 = {"24582", &P24582_4};	/*Profile ID - 24582*/
const Value V23569_1_1 = {S2, NULL};	/*Value = True*/
const Property P23569_1 = {S50, 0, &V23569_1_1, NULL};	/*Property = IsMobile*/
const Value V23569_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23569_5 = {S52, 0, &V23569_5_1458, &P23569_1};	/*Property = ScreenPixelsHeight*/
const Value V23569_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23569_4 = {S54, 0, &V23569_4_37617, &P23569_5};	/*Property = ScreenPixelsWidth*/
const Profile P23569 = {"23569", &P23569_4};	/*Profile ID - 23569*/
const Value V20993_1_1 = {S2, NULL};	/*Value = True*/
const Property P20993_1 = {S50, 0, &V20993_1_1, NULL};	/*Property = IsMobile*/
const Value V20993_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20993_5 = {S52, 0, &V20993_5_9981, &P20993_1};	/*Property = ScreenPixelsHeight*/
const Value V20993_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P20993_4 = {S54, 0, &V20993_4_39098, &P20993_5};	/*Property = ScreenPixelsWidth*/
const Profile P20993 = {"20993", &P20993_4};	/*Profile ID - 20993*/
const Value V23286_1_1 = {S2, NULL};	/*Value = True*/
const Property P23286_1 = {S50, 0, &V23286_1_1, NULL};	/*Property = IsMobile*/
const Value V23286_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23286_5 = {S52, 0, &V23286_5_9981, &P23286_1};	/*Property = ScreenPixelsHeight*/
const Value V23286_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23286_4 = {S54, 0, &V23286_4_39098, &P23286_5};	/*Property = ScreenPixelsWidth*/
const Profile P23286 = {"23286", &P23286_4};	/*Profile ID - 23286*/
const Value V24493_1_1 = {S2, NULL};	/*Value = True*/
const Property P24493_1 = {S50, 0, &V24493_1_1, NULL};	/*Property = IsMobile*/
const Value V24493_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24493_5 = {S52, 0, &V24493_5_1403, &P24493_1};	/*Property = ScreenPixelsHeight*/
const Value V24493_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24493_4 = {S54, 0, &V24493_4_1404, &P24493_5};	/*Property = ScreenPixelsWidth*/
const Profile P24493 = {"24493", &P24493_4};	/*Profile ID - 24493*/
const Value V22995_1_1 = {S2, NULL};	/*Value = True*/
const Property P22995_1 = {S50, 0, &V22995_1_1, NULL};	/*Property = IsMobile*/
const Value V22995_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22995_5 = {S52, 0, &V22995_5_1277, &P22995_1};	/*Property = ScreenPixelsHeight*/
const Value V22995_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22995_4 = {S54, 0, &V22995_4_1279, &P22995_5};	/*Property = ScreenPixelsWidth*/
const Profile P22995 = {"22995", &P22995_4};	/*Profile ID - 22995*/
const Value V22972_1_1 = {S2, NULL};	/*Value = True*/
const Property P22972_1 = {S50, 0, &V22972_1_1, NULL};	/*Property = IsMobile*/
const Value V22972_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22972_5 = {S52, 0, &V22972_5_59138, &P22972_1};	/*Property = ScreenPixelsHeight*/
const Value V22972_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22972_4 = {S54, 0, &V22972_4_59970, &P22972_5};	/*Property = ScreenPixelsWidth*/
const Profile P22972 = {"22972", &P22972_4};	/*Profile ID - 22972*/
const Value V23374_1_1 = {S2, NULL};	/*Value = True*/
const Property P23374_1 = {S50, 0, &V23374_1_1, NULL};	/*Property = IsMobile*/
const Value V23374_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23374_5 = {S52, 0, &V23374_5_37, &P23374_1};	/*Property = ScreenPixelsHeight*/
const Value V23374_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23374_4 = {S54, 0, &V23374_4_36, &P23374_5};	/*Property = ScreenPixelsWidth*/
const Profile P23374 = {"23374", &P23374_4};	/*Profile ID - 23374*/
const Value V23194_1_1 = {S2, NULL};	/*Value = True*/
const Property P23194_1 = {S50, 0, &V23194_1_1, NULL};	/*Property = IsMobile*/
const Value V23194_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23194_5 = {S52, 0, &V23194_5_9981, &P23194_1};	/*Property = ScreenPixelsHeight*/
const Value V23194_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23194_4 = {S54, 0, &V23194_4_2336, &P23194_5};	/*Property = ScreenPixelsWidth*/
const Profile P23194 = {"23194", &P23194_4};	/*Profile ID - 23194*/
const Value V23474_1_1 = {S2, NULL};	/*Value = True*/
const Property P23474_1 = {S50, 0, &V23474_1_1, NULL};	/*Property = IsMobile*/
const Value V23474_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23474_5 = {S52, 0, &V23474_5_1227, &P23474_1};	/*Property = ScreenPixelsHeight*/
const Value V23474_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23474_4 = {S54, 0, &V23474_4_1229, &P23474_5};	/*Property = ScreenPixelsWidth*/
const Profile P23474 = {"23474", &P23474_4};	/*Profile ID - 23474*/
const Value V23151_1_1 = {S2, NULL};	/*Value = True*/
const Property P23151_1 = {S50, 0, &V23151_1_1, NULL};	/*Property = IsMobile*/
const Value V23151_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23151_5 = {S52, 0, &V23151_5_1227, &P23151_1};	/*Property = ScreenPixelsHeight*/
const Value V23151_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23151_4 = {S54, 0, &V23151_4_1229, &P23151_5};	/*Property = ScreenPixelsWidth*/
const Profile P23151 = {"23151", &P23151_4};	/*Profile ID - 23151*/
const Value V23139_1_1 = {S2, NULL};	/*Value = True*/
const Property P23139_1 = {S50, 0, &V23139_1_1, NULL};	/*Property = IsMobile*/
const Value V23139_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23139_5 = {S52, 0, &V23139_5_1403, &P23139_1};	/*Property = ScreenPixelsHeight*/
const Value V23139_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23139_4 = {S54, 0, &V23139_4_1404, &P23139_5};	/*Property = ScreenPixelsWidth*/
const Profile P23139 = {"23139", &P23139_4};	/*Profile ID - 23139*/
const Value V23337_1_1 = {S2, NULL};	/*Value = True*/
const Property P23337_1 = {S50, 0, &V23337_1_1, NULL};	/*Property = IsMobile*/
const Value V23337_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23337_5 = {S52, 0, &V23337_5_1222, &P23337_1};	/*Property = ScreenPixelsHeight*/
const Value V23337_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23337_4 = {S54, 0, &V23337_4_1224, &P23337_5};	/*Property = ScreenPixelsWidth*/
const Profile P23337 = {"23337", &P23337_4};	/*Profile ID - 23337*/
const Value V23207_1_1 = {S2, NULL};	/*Value = True*/
const Property P23207_1 = {S50, 0, &V23207_1_1, NULL};	/*Property = IsMobile*/
const Value V23207_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23207_5 = {S52, 0, &V23207_5_1227, &P23207_1};	/*Property = ScreenPixelsHeight*/
const Value V23207_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23207_4 = {S54, 0, &V23207_4_1229, &P23207_5};	/*Property = ScreenPixelsWidth*/
const Profile P23207 = {"23207", &P23207_4};	/*Profile ID - 23207*/
const Value V23174_1_1 = {S2, NULL};	/*Value = True*/
const Property P23174_1 = {S50, 0, &V23174_1_1, NULL};	/*Property = IsMobile*/
const Value V23174_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23174_5 = {S52, 0, &V23174_5_1277, &P23174_1};	/*Property = ScreenPixelsHeight*/
const Value V23174_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23174_4 = {S54, 0, &V23174_4_1279, &P23174_5};	/*Property = ScreenPixelsWidth*/
const Profile P23174 = {"23174", &P23174_4};	/*Profile ID - 23174*/
const Value V23573_1_1 = {S2, NULL};	/*Value = True*/
const Property P23573_1 = {S50, 0, &V23573_1_1, NULL};	/*Property = IsMobile*/
const Value V23573_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23573_5 = {S52, 0, &V23573_5_1227, &P23573_1};	/*Property = ScreenPixelsHeight*/
const Value V23573_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23573_4 = {S54, 0, &V23573_4_1229, &P23573_5};	/*Property = ScreenPixelsWidth*/
const Profile P23573 = {"23573", &P23573_4};	/*Profile ID - 23573*/
const Value V23407_1_1 = {S2, NULL};	/*Value = True*/
const Property P23407_1 = {S50, 0, &V23407_1_1, NULL};	/*Property = IsMobile*/
const Value V23407_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23407_5 = {S52, 0, &V23407_5_9981, &P23407_1};	/*Property = ScreenPixelsHeight*/
const Value V23407_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23407_4 = {S54, 0, &V23407_4_39098, &P23407_5};	/*Property = ScreenPixelsWidth*/
const Profile P23407 = {"23407", &P23407_4};	/*Profile ID - 23407*/
const Profile P23096 = {"23096", NULL};	/*Profile ID - 23096*/
const Value V23136_1_1 = {S2, NULL};	/*Value = True*/
const Property P23136_1 = {S50, 0, &V23136_1_1, NULL};	/*Property = IsMobile*/
const Value V23136_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23136_5 = {S52, 0, &V23136_5_1227, &P23136_1};	/*Property = ScreenPixelsHeight*/
const Value V23136_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23136_4 = {S54, 0, &V23136_4_1229, &P23136_5};	/*Property = ScreenPixelsWidth*/
const Profile P23136 = {"23136", &P23136_4};	/*Profile ID - 23136*/
const Value V23023_1_1 = {S2, NULL};	/*Value = True*/
const Property P23023_1 = {S50, 0, &V23023_1_1, NULL};	/*Property = IsMobile*/
const Value V23023_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23023_5 = {S52, 0, &V23023_5_1227, &P23023_1};	/*Property = ScreenPixelsHeight*/
const Value V23023_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23023_4 = {S54, 0, &V23023_4_1229, &P23023_5};	/*Property = ScreenPixelsWidth*/
const Profile P23023 = {"23023", &P23023_4};	/*Profile ID - 23023*/
const Value V23001_1_1 = {S2, NULL};	/*Value = True*/
const Property P23001_1 = {S50, 0, &V23001_1_1, NULL};	/*Property = IsMobile*/
const Value V23001_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23001_5 = {S52, 0, &V23001_5_37569, &P23001_1};	/*Property = ScreenPixelsHeight*/
const Value V23001_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23001_4 = {S54, 0, &V23001_4_5645, &P23001_5};	/*Property = ScreenPixelsWidth*/
const Profile P23001 = {"23001", &P23001_4};	/*Profile ID - 23001*/
const Value V23002_1_1 = {S2, NULL};	/*Value = True*/
const Property P23002_1 = {S50, 0, &V23002_1_1, NULL};	/*Property = IsMobile*/
const Value V23002_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23002_5 = {S52, 0, &V23002_5_1277, &P23002_1};	/*Property = ScreenPixelsHeight*/
const Value V23002_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23002_4 = {S54, 0, &V23002_4_2336, &P23002_5};	/*Property = ScreenPixelsWidth*/
const Profile P23002 = {"23002", &P23002_4};	/*Profile ID - 23002*/
const Value V23402_1_1 = {S2, NULL};	/*Value = True*/
const Property P23402_1 = {S50, 0, &V23402_1_1, NULL};	/*Property = IsMobile*/
const Value V23402_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23402_5 = {S52, 0, &V23402_5_1227, &P23402_1};	/*Property = ScreenPixelsHeight*/
const Value V23402_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23402_4 = {S54, 0, &V23402_4_1229, &P23402_5};	/*Property = ScreenPixelsWidth*/
const Profile P23402 = {"23402", &P23402_4};	/*Profile ID - 23402*/
const Value V23094_1_1 = {S2, NULL};	/*Value = True*/
const Property P23094_1 = {S50, 0, &V23094_1_1, NULL};	/*Property = IsMobile*/
const Value V23094_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P23094_5 = {S52, 0, &V23094_5_59138, &P23094_1};	/*Property = ScreenPixelsHeight*/
const Value V23094_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P23094_4 = {S54, 0, &V23094_4_59970, &P23094_5};	/*Property = ScreenPixelsWidth*/
const Profile P23094 = {"23094", &P23094_4};	/*Profile ID - 23094*/
const Value V23153_1_1 = {S2, NULL};	/*Value = True*/
const Property P23153_1 = {S50, 0, &V23153_1_1, NULL};	/*Property = IsMobile*/
const Value V23153_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23153_5 = {S52, 0, &V23153_5_9981, &P23153_1};	/*Property = ScreenPixelsHeight*/
const Value V23153_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23153_4 = {S54, 0, &V23153_4_39098, &P23153_5};	/*Property = ScreenPixelsWidth*/
const Profile P23153 = {"23153", &P23153_4};	/*Profile ID - 23153*/
const Value V23150_1_1 = {S2, NULL};	/*Value = True*/
const Property P23150_1 = {S50, 0, &V23150_1_1, NULL};	/*Property = IsMobile*/
const Value V23150_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23150_5 = {S52, 0, &V23150_5_9981, &P23150_1};	/*Property = ScreenPixelsHeight*/
const Value V23150_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23150_4 = {S54, 0, &V23150_4_39098, &P23150_5};	/*Property = ScreenPixelsWidth*/
const Profile P23150 = {"23150", &P23150_4};	/*Profile ID - 23150*/
const Value V24111_1_1 = {S2, NULL};	/*Value = True*/
const Property P24111_1 = {S50, 0, &V24111_1_1, NULL};	/*Property = IsMobile*/
const Value V24111_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P24111_5 = {S52, 0, &V24111_5_1832, &P24111_1};	/*Property = ScreenPixelsHeight*/
const Value V24111_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P24111_4 = {S54, 0, &V24111_4_2336, &P24111_5};	/*Property = ScreenPixelsWidth*/
const Profile P24111 = {"24111", &P24111_4};	/*Profile ID - 24111*/
const Value V22969_1_1 = {S2, NULL};	/*Value = True*/
const Property P22969_1 = {S50, 0, &V22969_1_1, NULL};	/*Property = IsMobile*/
const Value V22969_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22969_5 = {S52, 0, &V22969_5_1277, &P22969_1};	/*Property = ScreenPixelsHeight*/
const Value V22969_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22969_4 = {S54, 0, &V22969_4_1279, &P22969_5};	/*Property = ScreenPixelsWidth*/
const Profile P22969 = {"22969", &P22969_4};	/*Profile ID - 22969*/
const Value V23138_1_1 = {S2, NULL};	/*Value = True*/
const Property P23138_1 = {S50, 0, &V23138_1_1, NULL};	/*Property = IsMobile*/
const Value V23138_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23138_5 = {S52, 0, &V23138_5_1403, &P23138_1};	/*Property = ScreenPixelsHeight*/
const Value V23138_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23138_4 = {S54, 0, &V23138_4_1404, &P23138_5};	/*Property = ScreenPixelsWidth*/
const Profile P23138 = {"23138", &P23138_4};	/*Profile ID - 23138*/
const Value V23186_1_1 = {S2, NULL};	/*Value = True*/
const Property P23186_1 = {S50, 0, &V23186_1_1, NULL};	/*Property = IsMobile*/
const Value V23186_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23186_5 = {S52, 0, &V23186_5_1277, &P23186_1};	/*Property = ScreenPixelsHeight*/
const Value V23186_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23186_4 = {S54, 0, &V23186_4_1279, &P23186_5};	/*Property = ScreenPixelsWidth*/
const Profile P23186 = {"23186", &P23186_4};	/*Profile ID - 23186*/
const Value V19767_1_1 = {S2, NULL};	/*Value = True*/
const Property P19767_1 = {S50, 0, &V19767_1_1, NULL};	/*Property = IsMobile*/
const Value V19767_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P19767_5 = {S52, 0, &V19767_5_9981, &P19767_1};	/*Property = ScreenPixelsHeight*/
const Value V19767_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P19767_4 = {S54, 0, &V19767_4_39098, &P19767_5};	/*Property = ScreenPixelsWidth*/
const Profile P19767 = {"19767", &P19767_4};	/*Profile ID - 19767*/
const Value V23395_1_1 = {S2, NULL};	/*Value = True*/
const Property P23395_1 = {S50, 0, &V23395_1_1, NULL};	/*Property = IsMobile*/
const Value V23395_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23395_5 = {S52, 0, &V23395_5_1458, &P23395_1};	/*Property = ScreenPixelsHeight*/
const Value V23395_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23395_4 = {S54, 0, &V23395_4_37617, &P23395_5};	/*Property = ScreenPixelsWidth*/
const Profile P23395 = {"23395", &P23395_4};	/*Profile ID - 23395*/
const Value V23243_1_1 = {S2, NULL};	/*Value = True*/
const Property P23243_1 = {S50, 0, &V23243_1_1, NULL};	/*Property = IsMobile*/
const Value V23243_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23243_5 = {S52, 0, &V23243_5_1227, &P23243_1};	/*Property = ScreenPixelsHeight*/
const Value V23243_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23243_4 = {S54, 0, &V23243_4_1229, &P23243_5};	/*Property = ScreenPixelsWidth*/
const Profile P23243 = {"23243", &P23243_4};	/*Profile ID - 23243*/
const Value V23247_1_1 = {S2, NULL};	/*Value = True*/
const Property P23247_1 = {S50, 0, &V23247_1_1, NULL};	/*Property = IsMobile*/
const Value V23247_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23247_5 = {S52, 0, &V23247_5_9981, &P23247_1};	/*Property = ScreenPixelsHeight*/
const Value V23247_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23247_4 = {S54, 0, &V23247_4_39098, &P23247_5};	/*Property = ScreenPixelsWidth*/
const Profile P23247 = {"23247", &P23247_4};	/*Profile ID - 23247*/
const Value V23212_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_279 = {S1, 0, &V23212_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23212_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_282 = {S3, 0, &V23212_282_57155, &P23212_279};	/*Property = BlobBuilder*/
const Value V23212_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_350 = {S4, 0, &V23212_350_57446, &P23212_282};	/*Property = Canvas*/
const Value V23212_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_283 = {S5, 0, &V23212_283_57161, &P23212_350};	/*Property = CssBackground*/
const Value V23212_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_284 = {S6, 0, &V23212_284_57164, &P23212_283};	/*Property = CssBorderImage*/
const Value V23212_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_285 = {S7, 0, &V23212_285_57167, &P23212_284};	/*Property = CssCanvas*/
const Value V23212_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_286 = {S8, 0, &V23212_286_57170, &P23212_285};	/*Property = CssColor*/
const Value V23212_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_237 = {S9, 0, &V23212_237_55786, &P23212_286};	/*Property = CssColumn*/
const Value V23212_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_287 = {S10, 0, &V23212_287_57173, &P23212_237};	/*Property = CssFlexbox*/
const Value V23212_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_288 = {S11, 0, &V23212_288_57176, &P23212_287};	/*Property = CssFont*/
const Value V23212_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_289 = {S12, 0, &V23212_289_86899, &P23212_288};	/*Property = CssImages*/
const Value V23212_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_291 = {S13, 0, &V23212_291_57182, &P23212_289};	/*Property = CssMediaQueries*/
const Value V23212_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_292 = {S14, 0, &V23212_292_57185, &P23212_291};	/*Property = CssMinMax*/
const Value V23212_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_293 = {S15, 0, &V23212_293_57188, &P23212_292};	/*Property = CssOverflow*/
const Value V23212_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_294 = {S16, 0, &V23212_294_57191, &P23212_293};	/*Property = CssPosition*/
const Value V23212_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_295 = {S17, 0, &V23212_295_57194, &P23212_294};	/*Property = CssText*/
const Value V23212_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_297 = {S18, 0, &V23212_297_57197, &P23212_295};	/*Property = CssTransforms*/
const Value V23212_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_241 = {S19, 0, &V23212_241_55801, &P23212_297};	/*Property = CssTransitions*/
const Value V23212_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_298 = {S20, 0, &V23212_298_57208, &P23212_241};	/*Property = CssUI*/
const Value V23212_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_300 = {S21, 0, &V23212_300_57217, &P23212_298};	/*Property = DataSet*/
const Value V23212_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_301 = {S22, 0, &V23212_301_57220, &P23212_300};	/*Property = DataUrl*/
const Value V23212_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_302 = {S23, 0, &V23212_302_57223, &P23212_301};	/*Property = DeviceOrientation*/
const Value V23212_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_304 = {S24, 0, &V23212_304_57229, &P23212_302};	/*Property = FileReader*/
const Value V23212_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_305 = {S25, 0, &V23212_305_57232, &P23212_304};	/*Property = FileSaver*/
const Value V23212_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_306 = {S26, 0, &V23212_306_57238, &P23212_305};	/*Property = FileWriter*/
const Value V23212_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_307 = {S27, 0, &V23212_307_57241, &P23212_306};	/*Property = FormData*/
const Value V23212_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_309 = {S28, 0, &V23212_309_57247, &P23212_307};	/*Property = Fullscreen*/
const Value V23212_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_310 = {S29, 0, &V23212_310_57250, &P23212_309};	/*Property = GeoLocation*/
const Value V23212_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_312 = {S30, 0, &V23212_312_57256, &P23212_310};	/*Property = History*/
const Value V23212_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_314 = {S31, 0, &V23212_314_57262, &P23212_312};	/*Property = Html5*/
const Value V23212_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_313 = {S32, 0, &V23212_313_57259, &P23212_314};	/*Property = Html-Media-Capture*/
const Value V23212_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_315 = {S33, 0, &V23212_315_57265, &P23212_313};	/*Property = Iframe*/
const Value V23212_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_316 = {S34, 0, &V23212_316_57268, &P23212_315};	/*Property = IndexedDB*/
const Value V23212_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_317 = {S35, 0, &V23212_317_57271, &P23212_316};	/*Property = Json*/
const Value V23212_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P23212_15 = {S37, 0, &V23212_15_29, &P23212_317};	/*Property = LayoutEngine*/
const Value V23212_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_344 = {S38, 0, &V23212_344_57274, &P23212_15};	/*Property = Masking*/
const Value V23212_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_320 = {S39, 0, &V23212_320_57292, &P23212_344};	/*Property = PostMessage*/
const Value V23212_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_321 = {S40, 0, &V23212_321_57295, &P23212_320};	/*Property = Progress*/
const Value V23212_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_322 = {S41, 0, &V23212_322_57298, &P23212_321};	/*Property = Prompts*/
const Value V23212_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_323 = {S42, 0, &V23212_323_57301, &P23212_322};	/*Property = Selector*/
const Value V23212_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_324 = {S43, 0, &V23212_324_57307, &P23212_323};	/*Property = Svg*/
const Value V23212_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_327 = {S44, 0, &V23212_327_57316, &P23212_324};	/*Property = TouchEvents*/
const Value V23212_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_328 = {S45, 0, &V23212_328_57319, &P23212_327};	/*Property = Track*/
const Value V23212_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_329 = {S46, 0, &V23212_329_57325, &P23212_328};	/*Property = Video*/
const Value V23212_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_330 = {S47, 0, &V23212_330_57328, &P23212_329};	/*Property = Viewport*/
const Value V23212_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_332 = {S48, 0, &V23212_332_57340, &P23212_330};	/*Property = WebWorkers*/
const Value V23212_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23212_333 = {S49, 0, &V23212_333_57343, &P23212_332};	/*Property = Xhr2*/
const Profile P23212 = {"23212", &P23212_333};	/*Profile ID - 23212*/
const Value V23180_1_1 = {S2, NULL};	/*Value = True*/
const Property P23180_1 = {S50, 0, &V23180_1_1, NULL};	/*Property = IsMobile*/
const Value V23180_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23180_5 = {S52, 0, &V23180_5_1832, &P23180_1};	/*Property = ScreenPixelsHeight*/
const Value V23180_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23180_4 = {S54, 0, &V23180_4_5645, &P23180_5};	/*Property = ScreenPixelsWidth*/
const Profile P23180 = {"23180", &P23180_4};	/*Profile ID - 23180*/
const Value V24729_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24729_15 = {S37, 0, &V24729_15_28, NULL};	/*Property = LayoutEngine*/
const Profile P24729 = {"24729", &P24729_15};	/*Profile ID - 24729*/
const Value V22968_1_1 = {S2, NULL};	/*Value = True*/
const Property P22968_1 = {S50, 0, &V22968_1_1, NULL};	/*Property = IsMobile*/
const Value V22968_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P22968_5 = {S52, 0, &V22968_5_9981, &P22968_1};	/*Property = ScreenPixelsHeight*/
const Value V22968_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P22968_4 = {S54, 0, &V22968_4_39098, &P22968_5};	/*Property = ScreenPixelsWidth*/
const Profile P22968 = {"22968", &P22968_4};	/*Profile ID - 22968*/
const Value V22971_1_1 = {S2, NULL};	/*Value = True*/
const Property P22971_1 = {S50, 0, &V22971_1_1, NULL};	/*Property = IsMobile*/
const Value V22971_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22971_5 = {S52, 0, &V22971_5_1277, &P22971_1};	/*Property = ScreenPixelsHeight*/
const Value V22971_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22971_4 = {S54, 0, &V22971_4_1279, &P22971_5};	/*Property = ScreenPixelsWidth*/
const Profile P22971 = {"22971", &P22971_4};	/*Profile ID - 22971*/
const Value V23689_279_57148 = {S0, NULL};	/*Value = False*/
const Property P23689_279 = {S1, 0, &V23689_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V23689_282_57153 = {S2, NULL};	/*Value = True*/
const Property P23689_282 = {S3, 0, &V23689_282_57153, &P23689_279};	/*Property = BlobBuilder*/
const Value V23689_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23689_350 = {S4, 0, &V23689_350_57444, &P23689_282};	/*Property = Canvas*/
const Value V23689_283_57160 = {S0, NULL};	/*Value = False*/
const Property P23689_283 = {S5, 0, &V23689_283_57160, &P23689_350};	/*Property = CssBackground*/
const Value V23689_284_57163 = {S0, NULL};	/*Value = False*/
const Property P23689_284 = {S6, 0, &V23689_284_57163, &P23689_283};	/*Property = CssBorderImage*/
const Value V23689_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23689_285 = {S7, 0, &V23689_285_57166, &P23689_284};	/*Property = CssCanvas*/
const Value V23689_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23689_286 = {S8, 0, &V23689_286_57168, &P23689_285};	/*Property = CssColor*/
const Value V23689_237_55785 = {S0, NULL};	/*Value = False*/
const Property P23689_237 = {S9, 0, &V23689_237_55785, &P23689_286};	/*Property = CssColumn*/
const Value V23689_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23689_287 = {S10, 0, &V23689_287_57172, &P23689_237};	/*Property = CssFlexbox*/
const Value V23689_288_57175 = {S0, NULL};	/*Value = False*/
const Property P23689_288 = {S11, 0, &V23689_288_57175, &P23689_287};	/*Property = CssFont*/
const Value V23689_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23689_289 = {S12, 0, &V23689_289_86899, &P23689_288};	/*Property = CssImages*/
const Value V23689_291_57181 = {S0, NULL};	/*Value = False*/
const Property P23689_291 = {S13, 0, &V23689_291_57181, &P23689_289};	/*Property = CssMediaQueries*/
const Value V23689_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23689_292 = {S14, 0, &V23689_292_57183, &P23689_291};	/*Property = CssMinMax*/
const Value V23689_293_57187 = {S0, NULL};	/*Value = False*/
const Property P23689_293 = {S15, 0, &V23689_293_57187, &P23689_292};	/*Property = CssOverflow*/
const Value V23689_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23689_294 = {S16, 0, &V23689_294_57189, &P23689_293};	/*Property = CssPosition*/
const Value V23689_295_57193 = {S0, NULL};	/*Value = False*/
const Property P23689_295 = {S17, 0, &V23689_295_57193, &P23689_294};	/*Property = CssText*/
const Value V23689_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23689_297 = {S18, 0, &V23689_297_57195, &P23689_295};	/*Property = CssTransforms*/
const Value V23689_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23689_241 = {S19, 0, &V23689_241_55799, &P23689_297};	/*Property = CssTransitions*/
const Value V23689_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23689_298 = {S20, 0, &V23689_298_57206, &P23689_241};	/*Property = CssUI*/
const Value V23689_300_57215 = {S2, NULL};	/*Value = True*/
const Property P23689_300 = {S21, 0, &V23689_300_57215, &P23689_298};	/*Property = DataSet*/
const Value V23689_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23689_301 = {S22, 0, &V23689_301_57218, &P23689_300};	/*Property = DataUrl*/
const Value V23689_302_57221 = {S2, NULL};	/*Value = True*/
const Property P23689_302 = {S23, 0, &V23689_302_57221, &P23689_301};	/*Property = DeviceOrientation*/
const Value V23689_304_57227 = {S2, NULL};	/*Value = True*/
const Property P23689_304 = {S24, 0, &V23689_304_57227, &P23689_302};	/*Property = FileReader*/
const Value V23689_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23689_305 = {S25, 0, &V23689_305_57231, &P23689_304};	/*Property = FileSaver*/
const Value V23689_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23689_306 = {S26, 0, &V23689_306_57237, &P23689_305};	/*Property = FileWriter*/
const Value V23689_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23689_307 = {S27, 0, &V23689_307_57239, &P23689_306};	/*Property = FormData*/
const Value V23689_309_57245 = {S2, NULL};	/*Value = True*/
const Property P23689_309 = {S28, 0, &V23689_309_57245, &P23689_307};	/*Property = Fullscreen*/
const Value V23689_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23689_310 = {S29, 0, &V23689_310_57248, &P23689_309};	/*Property = GeoLocation*/
const Value V23689_312_57254 = {S2, NULL};	/*Value = True*/
const Property P23689_312 = {S30, 0, &V23689_312_57254, &P23689_310};	/*Property = History*/
const Value V23689_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23689_314 = {S31, 0, &V23689_314_57261, &P23689_312};	/*Property = Html5*/
const Value V23689_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23689_313 = {S32, 0, &V23689_313_57258, &P23689_314};	/*Property = Html-Media-Capture*/
const Value V23689_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23689_315 = {S33, 0, &V23689_315_57264, &P23689_313};	/*Property = Iframe*/
const Value V23689_316_57266 = {S2, NULL};	/*Value = True*/
const Property P23689_316 = {S34, 0, &V23689_316_57266, &P23689_315};	/*Property = IndexedDB*/
const Value V23689_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23689_317 = {S35, 0, &V23689_317_57269, &P23689_316};	/*Property = Json*/
const Value V23689_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P23689_15 = {S37, 0, &V23689_15_29, &P23689_317};	/*Property = LayoutEngine*/
const Value V23689_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23689_344 = {S38, 0, &V23689_344_57272, &P23689_15};	/*Property = Masking*/
const Value V23689_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23689_320 = {S39, 0, &V23689_320_57290, &P23689_344};	/*Property = PostMessage*/
const Value V23689_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23689_321 = {S40, 0, &V23689_321_57293, &P23689_320};	/*Property = Progress*/
const Value V23689_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23689_322 = {S41, 0, &V23689_322_57296, &P23689_321};	/*Property = Prompts*/
const Value V23689_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23689_323 = {S42, 0, &V23689_323_57299, &P23689_322};	/*Property = Selector*/
const Value V23689_324_57305 = {S2, NULL};	/*Value = True*/
const Property P23689_324 = {S43, 0, &V23689_324_57305, &P23689_323};	/*Property = Svg*/
const Value V23689_327_57315 = {S0, NULL};	/*Value = False*/
const Property P23689_327 = {S44, 0, &V23689_327_57315, &P23689_324};	/*Property = TouchEvents*/
const Value V23689_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23689_328 = {S45, 0, &V23689_328_57318, &P23689_327};	/*Property = Track*/
const Value V23689_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23689_329 = {S46, 0, &V23689_329_57323, &P23689_328};	/*Property = Video*/
const Value V23689_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23689_330 = {S47, 0, &V23689_330_57326, &P23689_329};	/*Property = Viewport*/
const Value V23689_332_57339 = {S0, NULL};	/*Value = False*/
const Property P23689_332 = {S48, 0, &V23689_332_57339, &P23689_330};	/*Property = WebWorkers*/
const Value V23689_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23689_333 = {S49, 0, &V23689_333_57342, &P23689_332};	/*Property = Xhr2*/
const Profile P23689 = {"23689", &P23689_333};	/*Profile ID - 23689*/
const Value V23688_279_57148 = {S0, NULL};	/*Value = False*/
const Property P23688_279 = {S1, 0, &V23688_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V23688_282_57153 = {S2, NULL};	/*Value = True*/
const Property P23688_282 = {S3, 0, &V23688_282_57153, &P23688_279};	/*Property = BlobBuilder*/
const Value V23688_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23688_350 = {S4, 0, &V23688_350_57444, &P23688_282};	/*Property = Canvas*/
const Value V23688_283_57160 = {S0, NULL};	/*Value = False*/
const Property P23688_283 = {S5, 0, &V23688_283_57160, &P23688_350};	/*Property = CssBackground*/
const Value V23688_284_57163 = {S0, NULL};	/*Value = False*/
const Property P23688_284 = {S6, 0, &V23688_284_57163, &P23688_283};	/*Property = CssBorderImage*/
const Value V23688_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23688_285 = {S7, 0, &V23688_285_57166, &P23688_284};	/*Property = CssCanvas*/
const Value V23688_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23688_286 = {S8, 0, &V23688_286_57168, &P23688_285};	/*Property = CssColor*/
const Value V23688_237_55785 = {S0, NULL};	/*Value = False*/
const Property P23688_237 = {S9, 0, &V23688_237_55785, &P23688_286};	/*Property = CssColumn*/
const Value V23688_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23688_287 = {S10, 0, &V23688_287_57172, &P23688_237};	/*Property = CssFlexbox*/
const Value V23688_288_57175 = {S0, NULL};	/*Value = False*/
const Property P23688_288 = {S11, 0, &V23688_288_57175, &P23688_287};	/*Property = CssFont*/
const Value V23688_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23688_289 = {S12, 0, &V23688_289_86899, &P23688_288};	/*Property = CssImages*/
const Value V23688_291_57181 = {S0, NULL};	/*Value = False*/
const Property P23688_291 = {S13, 0, &V23688_291_57181, &P23688_289};	/*Property = CssMediaQueries*/
const Value V23688_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23688_292 = {S14, 0, &V23688_292_57183, &P23688_291};	/*Property = CssMinMax*/
const Value V23688_293_57187 = {S0, NULL};	/*Value = False*/
const Property P23688_293 = {S15, 0, &V23688_293_57187, &P23688_292};	/*Property = CssOverflow*/
const Value V23688_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23688_294 = {S16, 0, &V23688_294_57189, &P23688_293};	/*Property = CssPosition*/
const Value V23688_295_57193 = {S0, NULL};	/*Value = False*/
const Property P23688_295 = {S17, 0, &V23688_295_57193, &P23688_294};	/*Property = CssText*/
const Value V23688_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23688_297 = {S18, 0, &V23688_297_57195, &P23688_295};	/*Property = CssTransforms*/
const Value V23688_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23688_241 = {S19, 0, &V23688_241_55799, &P23688_297};	/*Property = CssTransitions*/
const Value V23688_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23688_298 = {S20, 0, &V23688_298_57206, &P23688_241};	/*Property = CssUI*/
const Value V23688_300_57215 = {S2, NULL};	/*Value = True*/
const Property P23688_300 = {S21, 0, &V23688_300_57215, &P23688_298};	/*Property = DataSet*/
const Value V23688_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23688_301 = {S22, 0, &V23688_301_57218, &P23688_300};	/*Property = DataUrl*/
const Value V23688_302_57221 = {S2, NULL};	/*Value = True*/
const Property P23688_302 = {S23, 0, &V23688_302_57221, &P23688_301};	/*Property = DeviceOrientation*/
const Value V23688_304_57227 = {S2, NULL};	/*Value = True*/
const Property P23688_304 = {S24, 0, &V23688_304_57227, &P23688_302};	/*Property = FileReader*/
const Value V23688_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23688_305 = {S25, 0, &V23688_305_57231, &P23688_304};	/*Property = FileSaver*/
const Value V23688_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23688_306 = {S26, 0, &V23688_306_57237, &P23688_305};	/*Property = FileWriter*/
const Value V23688_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23688_307 = {S27, 0, &V23688_307_57239, &P23688_306};	/*Property = FormData*/
const Value V23688_309_57245 = {S2, NULL};	/*Value = True*/
const Property P23688_309 = {S28, 0, &V23688_309_57245, &P23688_307};	/*Property = Fullscreen*/
const Value V23688_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23688_310 = {S29, 0, &V23688_310_57248, &P23688_309};	/*Property = GeoLocation*/
const Value V23688_312_57254 = {S2, NULL};	/*Value = True*/
const Property P23688_312 = {S30, 0, &V23688_312_57254, &P23688_310};	/*Property = History*/
const Value V23688_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23688_314 = {S31, 0, &V23688_314_57261, &P23688_312};	/*Property = Html5*/
const Value V23688_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23688_313 = {S32, 0, &V23688_313_57258, &P23688_314};	/*Property = Html-Media-Capture*/
const Value V23688_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23688_315 = {S33, 0, &V23688_315_57264, &P23688_313};	/*Property = Iframe*/
const Value V23688_316_57266 = {S2, NULL};	/*Value = True*/
const Property P23688_316 = {S34, 0, &V23688_316_57266, &P23688_315};	/*Property = IndexedDB*/
const Value V23688_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23688_317 = {S35, 0, &V23688_317_57269, &P23688_316};	/*Property = Json*/
const Value V23688_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P23688_15 = {S37, 0, &V23688_15_29, &P23688_317};	/*Property = LayoutEngine*/
const Value V23688_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23688_344 = {S38, 0, &V23688_344_57272, &P23688_15};	/*Property = Masking*/
const Value V23688_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23688_320 = {S39, 0, &V23688_320_57290, &P23688_344};	/*Property = PostMessage*/
const Value V23688_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23688_321 = {S40, 0, &V23688_321_57293, &P23688_320};	/*Property = Progress*/
const Value V23688_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23688_322 = {S41, 0, &V23688_322_57296, &P23688_321};	/*Property = Prompts*/
const Value V23688_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23688_323 = {S42, 0, &V23688_323_57299, &P23688_322};	/*Property = Selector*/
const Value V23688_324_57305 = {S2, NULL};	/*Value = True*/
const Property P23688_324 = {S43, 0, &V23688_324_57305, &P23688_323};	/*Property = Svg*/
const Value V23688_327_57315 = {S0, NULL};	/*Value = False*/
const Property P23688_327 = {S44, 0, &V23688_327_57315, &P23688_324};	/*Property = TouchEvents*/
const Value V23688_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23688_328 = {S45, 0, &V23688_328_57318, &P23688_327};	/*Property = Track*/
const Value V23688_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23688_329 = {S46, 0, &V23688_329_57323, &P23688_328};	/*Property = Video*/
const Value V23688_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23688_330 = {S47, 0, &V23688_330_57326, &P23688_329};	/*Property = Viewport*/
const Value V23688_332_57339 = {S0, NULL};	/*Value = False*/
const Property P23688_332 = {S48, 0, &V23688_332_57339, &P23688_330};	/*Property = WebWorkers*/
const Value V23688_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23688_333 = {S49, 0, &V23688_333_57342, &P23688_332};	/*Property = Xhr2*/
const Profile P23688 = {"23688", &P23688_333};	/*Profile ID - 23688*/
const Value V17872_1_1 = {S2, NULL};	/*Value = True*/
const Property P17872_1 = {S50, 0, &V17872_1_1, NULL};	/*Property = IsMobile*/
const Value V17872_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P17872_5 = {S52, 0, &V17872_5_1304, &P17872_1};	/*Property = ScreenPixelsHeight*/
const Value V17872_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P17872_4 = {S54, 0, &V17872_4_1224, &P17872_5};	/*Property = ScreenPixelsWidth*/
const Profile P17872 = {"17872", &P17872_4};	/*Profile ID - 17872*/
const Value V16800_1_1 = {S2, NULL};	/*Value = True*/
const Property P16800_1 = {S50, 0, &V16800_1_1, NULL};	/*Property = IsMobile*/
const Value V16800_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P16800_5 = {S52, 0, &V16800_5_1233, &P16800_1};	/*Property = ScreenPixelsHeight*/
const Value V16800_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P16800_4 = {S54, 0, &V16800_4_1235, &P16800_5};	/*Property = ScreenPixelsWidth*/
const Profile P16800 = {"16800", &P16800_4};	/*Profile ID - 16800*/
const Value V23656_1_2 = {S0, NULL};	/*Value = False*/
const Property P23656_1 = {S50, 0, &V23656_1_2, NULL};	/*Property = IsMobile*/
const Value V23656_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23656_5 = {S52, 0, &V23656_5_60417, &P23656_1};	/*Property = ScreenPixelsHeight*/
const Value V23656_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23656_4 = {S54, 0, &V23656_4_57095, &P23656_5};	/*Property = ScreenPixelsWidth*/
const Profile P23656 = {"23656", &P23656_4};	/*Profile ID - 23656*/
const Value V23669_1_2 = {S0, NULL};	/*Value = False*/
const Property P23669_1 = {S50, 0, &V23669_1_2, NULL};	/*Property = IsMobile*/
const Value V23669_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23669_5 = {S52, 0, &V23669_5_60417, &P23669_1};	/*Property = ScreenPixelsHeight*/
const Value V23669_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23669_4 = {S54, 0, &V23669_4_57095, &P23669_5};	/*Property = ScreenPixelsWidth*/
const Profile P23669 = {"23669", &P23669_4};	/*Profile ID - 23669*/
const Value V15929_1_1 = {S2, NULL};	/*Value = True*/
const Property P15929_1 = {S50, 0, &V15929_1_1, NULL};	/*Property = IsMobile*/
const Value V15929_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P15929_5 = {S52, 0, &V15929_5_1222, &P15929_1};	/*Property = ScreenPixelsHeight*/
const Value V15929_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P15929_4 = {S54, 0, &V15929_4_1224, &P15929_5};	/*Property = ScreenPixelsWidth*/
const Profile P15929 = {"15929", &P15929_4};	/*Profile ID - 15929*/
const Value V19932_1_1 = {S2, NULL};	/*Value = True*/
const Property P19932_1 = {S50, 0, &V19932_1_1, NULL};	/*Property = IsMobile*/
const Value V19932_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P19932_5 = {S52, 0, &V19932_5_1222, &P19932_1};	/*Property = ScreenPixelsHeight*/
const Value V19932_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P19932_4 = {S54, 0, &V19932_4_1224, &P19932_5};	/*Property = ScreenPixelsWidth*/
const Profile P19932 = {"19932", &P19932_4};	/*Profile ID - 19932*/
const Value V23675_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_279 = {S1, 0, &V23675_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23675_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_282 = {S3, 0, &V23675_282_57155, &P23675_279};	/*Property = BlobBuilder*/
const Value V23675_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_350 = {S4, 0, &V23675_350_57446, &P23675_282};	/*Property = Canvas*/
const Value V23675_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_283 = {S5, 0, &V23675_283_57161, &P23675_350};	/*Property = CssBackground*/
const Value V23675_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_284 = {S6, 0, &V23675_284_57164, &P23675_283};	/*Property = CssBorderImage*/
const Value V23675_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_285 = {S7, 0, &V23675_285_57167, &P23675_284};	/*Property = CssCanvas*/
const Value V23675_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_286 = {S8, 0, &V23675_286_57170, &P23675_285};	/*Property = CssColor*/
const Value V23675_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_237 = {S9, 0, &V23675_237_55786, &P23675_286};	/*Property = CssColumn*/
const Value V23675_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_287 = {S10, 0, &V23675_287_57173, &P23675_237};	/*Property = CssFlexbox*/
const Value V23675_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_288 = {S11, 0, &V23675_288_57176, &P23675_287};	/*Property = CssFont*/
const Value V23675_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_289 = {S12, 0, &V23675_289_86899, &P23675_288};	/*Property = CssImages*/
const Value V23675_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_291 = {S13, 0, &V23675_291_57182, &P23675_289};	/*Property = CssMediaQueries*/
const Value V23675_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_292 = {S14, 0, &V23675_292_57185, &P23675_291};	/*Property = CssMinMax*/
const Value V23675_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_293 = {S15, 0, &V23675_293_57188, &P23675_292};	/*Property = CssOverflow*/
const Value V23675_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_294 = {S16, 0, &V23675_294_57191, &P23675_293};	/*Property = CssPosition*/
const Value V23675_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_295 = {S17, 0, &V23675_295_57194, &P23675_294};	/*Property = CssText*/
const Value V23675_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_297 = {S18, 0, &V23675_297_57197, &P23675_295};	/*Property = CssTransforms*/
const Value V23675_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_241 = {S19, 0, &V23675_241_55801, &P23675_297};	/*Property = CssTransitions*/
const Value V23675_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_298 = {S20, 0, &V23675_298_57208, &P23675_241};	/*Property = CssUI*/
const Value V23675_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_300 = {S21, 0, &V23675_300_57217, &P23675_298};	/*Property = DataSet*/
const Value V23675_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_301 = {S22, 0, &V23675_301_57220, &P23675_300};	/*Property = DataUrl*/
const Value V23675_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_302 = {S23, 0, &V23675_302_57223, &P23675_301};	/*Property = DeviceOrientation*/
const Value V23675_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_304 = {S24, 0, &V23675_304_57229, &P23675_302};	/*Property = FileReader*/
const Value V23675_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_305 = {S25, 0, &V23675_305_57232, &P23675_304};	/*Property = FileSaver*/
const Value V23675_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_306 = {S26, 0, &V23675_306_57238, &P23675_305};	/*Property = FileWriter*/
const Value V23675_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_307 = {S27, 0, &V23675_307_57241, &P23675_306};	/*Property = FormData*/
const Value V23675_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_309 = {S28, 0, &V23675_309_57247, &P23675_307};	/*Property = Fullscreen*/
const Value V23675_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_310 = {S29, 0, &V23675_310_57250, &P23675_309};	/*Property = GeoLocation*/
const Value V23675_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_312 = {S30, 0, &V23675_312_57256, &P23675_310};	/*Property = History*/
const Value V23675_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_314 = {S31, 0, &V23675_314_57262, &P23675_312};	/*Property = Html5*/
const Value V23675_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_313 = {S32, 0, &V23675_313_57259, &P23675_314};	/*Property = Html-Media-Capture*/
const Value V23675_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_315 = {S33, 0, &V23675_315_57265, &P23675_313};	/*Property = Iframe*/
const Value V23675_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_316 = {S34, 0, &V23675_316_57268, &P23675_315};	/*Property = IndexedDB*/
const Value V23675_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_317 = {S35, 0, &V23675_317_57271, &P23675_316};	/*Property = Json*/
const Value V23675_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_15 = {S37, 0, &V23675_15_72, &P23675_317};	/*Property = LayoutEngine*/
const Value V23675_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_344 = {S38, 0, &V23675_344_57274, &P23675_15};	/*Property = Masking*/
const Value V23675_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_320 = {S39, 0, &V23675_320_57292, &P23675_344};	/*Property = PostMessage*/
const Value V23675_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_321 = {S40, 0, &V23675_321_57295, &P23675_320};	/*Property = Progress*/
const Value V23675_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_322 = {S41, 0, &V23675_322_57298, &P23675_321};	/*Property = Prompts*/
const Value V23675_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_323 = {S42, 0, &V23675_323_57301, &P23675_322};	/*Property = Selector*/
const Value V23675_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_324 = {S43, 0, &V23675_324_57307, &P23675_323};	/*Property = Svg*/
const Value V23675_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_327 = {S44, 0, &V23675_327_57316, &P23675_324};	/*Property = TouchEvents*/
const Value V23675_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_328 = {S45, 0, &V23675_328_57319, &P23675_327};	/*Property = Track*/
const Value V23675_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_329 = {S46, 0, &V23675_329_57325, &P23675_328};	/*Property = Video*/
const Value V23675_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_330 = {S47, 0, &V23675_330_57328, &P23675_329};	/*Property = Viewport*/
const Value V23675_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_332 = {S48, 0, &V23675_332_57340, &P23675_330};	/*Property = WebWorkers*/
const Value V23675_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23675_333 = {S49, 0, &V23675_333_57343, &P23675_332};	/*Property = Xhr2*/
const Profile P23675 = {"23675", &P23675_333};	/*Profile ID - 23675*/
const Value V23747_1_1 = {S2, NULL};	/*Value = True*/
const Property P23747_1 = {S50, 0, &V23747_1_1, NULL};	/*Property = IsMobile*/
const Value V23747_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23747_5 = {S52, 0, &V23747_5_1222, &P23747_1};	/*Property = ScreenPixelsHeight*/
const Value V23747_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23747_4 = {S54, 0, &V23747_4_1224, &P23747_5};	/*Property = ScreenPixelsWidth*/
const Profile P23747 = {"23747", &P23747_4};	/*Profile ID - 23747*/
const Value V23393_1_1 = {S2, NULL};	/*Value = True*/
const Property P23393_1 = {S50, 0, &V23393_1_1, NULL};	/*Property = IsMobile*/
const Value V23393_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23393_5 = {S52, 0, &V23393_5_1277, &P23393_1};	/*Property = ScreenPixelsHeight*/
const Value V23393_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23393_4 = {S54, 0, &V23393_4_2336, &P23393_5};	/*Property = ScreenPixelsWidth*/
const Profile P23393 = {"23393", &P23393_4};	/*Profile ID - 23393*/
const Value V23676_1_2 = {S0, NULL};	/*Value = False*/
const Property P23676_1 = {S50, 0, &V23676_1_2, NULL};	/*Property = IsMobile*/
const Value V23676_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23676_5 = {S52, 0, &V23676_5_37569, &P23676_1};	/*Property = ScreenPixelsHeight*/
const Value V23676_4_53077 = {S129, NULL};	/*Value = 1366*/
const Property P23676_4 = {S54, 0, &V23676_4_53077, &P23676_5};	/*Property = ScreenPixelsWidth*/
const Profile P23676 = {"23676", &P23676_4};	/*Profile ID - 23676*/
const Value V20186_1_1 = {S2, NULL};	/*Value = True*/
const Property P20186_1 = {S50, 0, &V20186_1_1, NULL};	/*Property = IsMobile*/
const Value V20186_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P20186_5 = {S52, 0, &V20186_5_1227, &P20186_1};	/*Property = ScreenPixelsHeight*/
const Value V20186_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P20186_4 = {S54, 0, &V20186_4_1460, &P20186_5};	/*Property = ScreenPixelsWidth*/
const Profile P20186 = {"20186", &P20186_4};	/*Profile ID - 20186*/
const Value V23378_1_1 = {S2, NULL};	/*Value = True*/
const Property P23378_1 = {S50, 0, &V23378_1_1, NULL};	/*Property = IsMobile*/
const Value V23378_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23378_5 = {S52, 0, &V23378_5_1277, &P23378_1};	/*Property = ScreenPixelsHeight*/
const Value V23378_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23378_4 = {S54, 0, &V23378_4_2336, &P23378_5};	/*Property = ScreenPixelsWidth*/
const Profile P23378 = {"23378", &P23378_4};	/*Profile ID - 23378*/
const Value V23638_1_2 = {S0, NULL};	/*Value = False*/
const Property P23638_1 = {S50, 0, &V23638_1_2, NULL};	/*Property = IsMobile*/
const Value V23638_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P23638_5 = {S52, 0, &V23638_5_60417, &P23638_1};	/*Property = ScreenPixelsHeight*/
const Value V23638_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P23638_4 = {S54, 0, &V23638_4_57095, &P23638_5};	/*Property = ScreenPixelsWidth*/
const Profile P23638 = {"23638", &P23638_4};	/*Profile ID - 23638*/
const Value V23411_1_1 = {S2, NULL};	/*Value = True*/
const Property P23411_1 = {S50, 0, &V23411_1_1, NULL};	/*Property = IsMobile*/
const Value V23411_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23411_5 = {S52, 0, &V23411_5_37, &P23411_1};	/*Property = ScreenPixelsHeight*/
const Value V23411_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23411_4 = {S54, 0, &V23411_4_36, &P23411_5};	/*Property = ScreenPixelsWidth*/
const Profile P23411 = {"23411", &P23411_4};	/*Profile ID - 23411*/
const Value V23750_1_1 = {S2, NULL};	/*Value = True*/
const Property P23750_1 = {S50, 0, &V23750_1_1, NULL};	/*Property = IsMobile*/
const Value V23750_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23750_5 = {S52, 0, &V23750_5_1227, &P23750_1};	/*Property = ScreenPixelsHeight*/
const Value V23750_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23750_4 = {S54, 0, &V23750_4_1229, &P23750_5};	/*Property = ScreenPixelsWidth*/
const Profile P23750 = {"23750", &P23750_4};	/*Profile ID - 23750*/
const Value V23642_1_1 = {S2, NULL};	/*Value = True*/
const Property P23642_1 = {S50, 0, &V23642_1_1, NULL};	/*Property = IsMobile*/
const Value V23642_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23642_5 = {S52, 0, &V23642_5_1227, &P23642_1};	/*Property = ScreenPixelsHeight*/
const Value V23642_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23642_4 = {S54, 0, &V23642_4_1229, &P23642_5};	/*Property = ScreenPixelsWidth*/
const Profile P23642 = {"23642", &P23642_4};	/*Profile ID - 23642*/
const Value V23450_1_1 = {S2, NULL};	/*Value = True*/
const Property P23450_1 = {S50, 0, &V23450_1_1, NULL};	/*Property = IsMobile*/
const Value V23450_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23450_5 = {S52, 0, &V23450_5_1403, &P23450_1};	/*Property = ScreenPixelsHeight*/
const Value V23450_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23450_4 = {S54, 0, &V23450_4_1404, &P23450_5};	/*Property = ScreenPixelsWidth*/
const Profile P23450 = {"23450", &P23450_4};	/*Profile ID - 23450*/
const Value V23571_1_1 = {S2, NULL};	/*Value = True*/
const Property P23571_1 = {S50, 0, &V23571_1_1, NULL};	/*Property = IsMobile*/
const Value V23571_5_1233 = {S67, NULL};	/*Value = 220*/
const Property P23571_5 = {S52, 0, &V23571_5_1233, &P23571_1};	/*Property = ScreenPixelsHeight*/
const Value V23571_4_1235 = {S68, NULL};	/*Value = 176*/
const Property P23571_4 = {S54, 0, &V23571_4_1235, &P23571_5};	/*Property = ScreenPixelsWidth*/
const Profile P23571 = {"23571", &P23571_4};	/*Profile ID - 23571*/
const Value V23481_279_57148 = {S0, NULL};	/*Value = False*/
const Property P23481_279 = {S1, 0, &V23481_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V23481_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23481_282 = {S3, 0, &V23481_282_57154, &P23481_279};	/*Property = BlobBuilder*/
const Value V23481_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23481_350 = {S4, 0, &V23481_350_57444, &P23481_282};	/*Property = Canvas*/
const Value V23481_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23481_283 = {S5, 0, &V23481_283_57159, &P23481_350};	/*Property = CssBackground*/
const Value V23481_284_57163 = {S0, NULL};	/*Value = False*/
const Property P23481_284 = {S6, 0, &V23481_284_57163, &P23481_283};	/*Property = CssBorderImage*/
const Value V23481_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23481_285 = {S7, 0, &V23481_285_57166, &P23481_284};	/*Property = CssCanvas*/
const Value V23481_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23481_286 = {S8, 0, &V23481_286_57168, &P23481_285};	/*Property = CssColor*/
const Value V23481_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23481_237 = {S9, 0, &V23481_237_55786, &P23481_286};	/*Property = CssColumn*/
const Value V23481_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23481_287 = {S10, 0, &V23481_287_57172, &P23481_237};	/*Property = CssFlexbox*/
const Value V23481_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23481_288 = {S11, 0, &V23481_288_57174, &P23481_287};	/*Property = CssFont*/
const Value V23481_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23481_289 = {S12, 0, &V23481_289_57759, &P23481_288};	/*Property = CssImages*/
const Value V23481_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23481_291 = {S13, 0, &V23481_291_57180, &P23481_289};	/*Property = CssMediaQueries*/
const Value V23481_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23481_292 = {S14, 0, &V23481_292_57183, &P23481_291};	/*Property = CssMinMax*/
const Value V23481_293_57186 = {S2, NULL};	/*Value = True*/
const Property P23481_293 = {S15, 0, &V23481_293_57186, &P23481_292};	/*Property = CssOverflow*/
const Value V23481_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23481_294 = {S16, 0, &V23481_294_57189, &P23481_293};	/*Property = CssPosition*/
const Value V23481_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23481_295 = {S17, 0, &V23481_295_57192, &P23481_294};	/*Property = CssText*/
const Value V23481_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23481_297 = {S18, 0, &V23481_297_57195, &P23481_295};	/*Property = CssTransforms*/
const Value V23481_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23481_241 = {S19, 0, &V23481_241_55799, &P23481_297};	/*Property = CssTransitions*/
const Value V23481_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23481_298 = {S20, 0, &V23481_298_57206, &P23481_241};	/*Property = CssUI*/
const Value V23481_300_57216 = {S0, NULL};	/*Value = False*/
const Property P23481_300 = {S21, 0, &V23481_300_57216, &P23481_298};	/*Property = DataSet*/
const Value V23481_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23481_301 = {S22, 0, &V23481_301_57218, &P23481_300};	/*Property = DataUrl*/
const Value V23481_302_57222 = {S0, NULL};	/*Value = False*/
const Property P23481_302 = {S23, 0, &V23481_302_57222, &P23481_301};	/*Property = DeviceOrientation*/
const Value V23481_304_57227 = {S2, NULL};	/*Value = True*/
const Property P23481_304 = {S24, 0, &V23481_304_57227, &P23481_302};	/*Property = FileReader*/
const Value V23481_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23481_305 = {S25, 0, &V23481_305_57231, &P23481_304};	/*Property = FileSaver*/
const Value V23481_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23481_306 = {S26, 0, &V23481_306_57237, &P23481_305};	/*Property = FileWriter*/
const Value V23481_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23481_307 = {S27, 0, &V23481_307_57239, &P23481_306};	/*Property = FormData*/
const Value V23481_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23481_309 = {S28, 0, &V23481_309_57246, &P23481_307};	/*Property = Fullscreen*/
const Value V23481_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23481_310 = {S29, 0, &V23481_310_57248, &P23481_309};	/*Property = GeoLocation*/
const Value V23481_312_57254 = {S2, NULL};	/*Value = True*/
const Property P23481_312 = {S30, 0, &V23481_312_57254, &P23481_310};	/*Property = History*/
const Value V23481_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23481_314 = {S31, 0, &V23481_314_57261, &P23481_312};	/*Property = Html5*/
const Value V23481_313_57257 = {S2, NULL};	/*Value = True*/
const Property P23481_313 = {S32, 0, &V23481_313_57257, &P23481_314};	/*Property = Html-Media-Capture*/
const Value V23481_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23481_315 = {S33, 0, &V23481_315_57264, &P23481_313};	/*Property = Iframe*/
const Value V23481_316_57266 = {S2, NULL};	/*Value = True*/
const Property P23481_316 = {S34, 0, &V23481_316_57266, &P23481_315};	/*Property = IndexedDB*/
const Value V23481_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23481_317 = {S35, 0, &V23481_317_57269, &P23481_316};	/*Property = Json*/
const Value V23481_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P23481_15 = {S37, 0, &V23481_15_28, &P23481_317};	/*Property = LayoutEngine*/
const Value V23481_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23481_344 = {S38, 0, &V23481_344_57272, &P23481_15};	/*Property = Masking*/
const Value V23481_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23481_320 = {S39, 0, &V23481_320_57290, &P23481_344};	/*Property = PostMessage*/
const Value V23481_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23481_321 = {S40, 0, &V23481_321_57293, &P23481_320};	/*Property = Progress*/
const Value V23481_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23481_322 = {S41, 0, &V23481_322_57296, &P23481_321};	/*Property = Prompts*/
const Value V23481_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23481_323 = {S42, 0, &V23481_323_57299, &P23481_322};	/*Property = Selector*/
const Value V23481_324_57306 = {S0, NULL};	/*Value = False*/
const Property P23481_324 = {S43, 0, &V23481_324_57306, &P23481_323};	/*Property = Svg*/
const Value V23481_327_57314 = {S2, NULL};	/*Value = True*/
const Property P23481_327 = {S44, 0, &V23481_327_57314, &P23481_324};	/*Property = TouchEvents*/
const Value V23481_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23481_328 = {S45, 0, &V23481_328_57318, &P23481_327};	/*Property = Track*/
const Value V23481_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23481_329 = {S46, 0, &V23481_329_57323, &P23481_328};	/*Property = Video*/
const Value V23481_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23481_330 = {S47, 0, &V23481_330_57326, &P23481_329};	/*Property = Viewport*/
const Value V23481_332_57338 = {S2, NULL};	/*Value = True*/
const Property P23481_332 = {S48, 0, &V23481_332_57338, &P23481_330};	/*Property = WebWorkers*/
const Value V23481_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23481_333 = {S49, 0, &V23481_333_57342, &P23481_332};	/*Property = Xhr2*/
const Profile P23481 = {"23481", &P23481_333};	/*Profile ID - 23481*/
const Value V23338_1_1 = {S2, NULL};	/*Value = True*/
const Property P23338_1 = {S50, 0, &V23338_1_1, NULL};	/*Property = IsMobile*/
const Value V23338_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23338_5 = {S52, 0, &V23338_5_1832, &P23338_1};	/*Property = ScreenPixelsHeight*/
const Value V23338_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23338_4 = {S54, 0, &V23338_4_5645, &P23338_5};	/*Property = ScreenPixelsWidth*/
const Profile P23338 = {"23338", &P23338_4};	/*Profile ID - 23338*/
const Value V23316_1_1 = {S2, NULL};	/*Value = True*/
const Property P23316_1 = {S50, 0, &V23316_1_1, NULL};	/*Property = IsMobile*/
const Value V23316_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23316_5 = {S52, 0, &V23316_5_1832, &P23316_1};	/*Property = ScreenPixelsHeight*/
const Value V23316_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23316_4 = {S54, 0, &V23316_4_5645, &P23316_5};	/*Property = ScreenPixelsWidth*/
const Profile P23316 = {"23316", &P23316_4};	/*Profile ID - 23316*/
const Value V23462_1_1 = {S2, NULL};	/*Value = True*/
const Property P23462_1 = {S50, 0, &V23462_1_1, NULL};	/*Property = IsMobile*/
const Value V23462_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23462_5 = {S52, 0, &V23462_5_1277, &P23462_1};	/*Property = ScreenPixelsHeight*/
const Value V23462_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23462_4 = {S54, 0, &V23462_4_2336, &P23462_5};	/*Property = ScreenPixelsWidth*/
const Profile P23462 = {"23462", &P23462_4};	/*Profile ID - 23462*/
const Value V23489_279_57148 = {S0, NULL};	/*Value = False*/
const Property P23489_279 = {S1, 0, &V23489_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V23489_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23489_282 = {S3, 0, &V23489_282_57154, &P23489_279};	/*Property = BlobBuilder*/
const Value V23489_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23489_350 = {S4, 0, &V23489_350_57444, &P23489_282};	/*Property = Canvas*/
const Value V23489_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23489_283 = {S5, 0, &V23489_283_57159, &P23489_350};	/*Property = CssBackground*/
const Value V23489_284_57163 = {S0, NULL};	/*Value = False*/
const Property P23489_284 = {S6, 0, &V23489_284_57163, &P23489_283};	/*Property = CssBorderImage*/
const Value V23489_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23489_285 = {S7, 0, &V23489_285_57166, &P23489_284};	/*Property = CssCanvas*/
const Value V23489_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23489_286 = {S8, 0, &V23489_286_57168, &P23489_285};	/*Property = CssColor*/
const Value V23489_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23489_237 = {S9, 0, &V23489_237_55786, &P23489_286};	/*Property = CssColumn*/
const Value V23489_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23489_287 = {S10, 0, &V23489_287_57172, &P23489_237};	/*Property = CssFlexbox*/
const Value V23489_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23489_288 = {S11, 0, &V23489_288_57174, &P23489_287};	/*Property = CssFont*/
const Value V23489_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23489_289 = {S12, 0, &V23489_289_57759, &P23489_288};	/*Property = CssImages*/
const Value V23489_291_57181 = {S0, NULL};	/*Value = False*/
const Property P23489_291 = {S13, 0, &V23489_291_57181, &P23489_289};	/*Property = CssMediaQueries*/
const Value V23489_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23489_292 = {S14, 0, &V23489_292_57183, &P23489_291};	/*Property = CssMinMax*/
const Value V23489_293_57187 = {S0, NULL};	/*Value = False*/
const Property P23489_293 = {S15, 0, &V23489_293_57187, &P23489_292};	/*Property = CssOverflow*/
const Value V23489_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23489_294 = {S16, 0, &V23489_294_57189, &P23489_293};	/*Property = CssPosition*/
const Value V23489_295_57193 = {S0, NULL};	/*Value = False*/
const Property P23489_295 = {S17, 0, &V23489_295_57193, &P23489_294};	/*Property = CssText*/
const Value V23489_297_57196 = {S0, NULL};	/*Value = False*/
const Property P23489_297 = {S18, 0, &V23489_297_57196, &P23489_295};	/*Property = CssTransforms*/
const Value V23489_241_55800 = {S0, NULL};	/*Value = False*/
const Property P23489_241 = {S19, 0, &V23489_241_55800, &P23489_297};	/*Property = CssTransitions*/
const Value V23489_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23489_298 = {S20, 0, &V23489_298_57206, &P23489_241};	/*Property = CssUI*/
const Value V23489_300_57216 = {S0, NULL};	/*Value = False*/
const Property P23489_300 = {S21, 0, &V23489_300_57216, &P23489_298};	/*Property = DataSet*/
const Value V23489_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23489_301 = {S22, 0, &V23489_301_57218, &P23489_300};	/*Property = DataUrl*/
const Value V23489_302_57222 = {S0, NULL};	/*Value = False*/
const Property P23489_302 = {S23, 0, &V23489_302_57222, &P23489_301};	/*Property = DeviceOrientation*/
const Value V23489_304_57228 = {S0, NULL};	/*Value = False*/
const Property P23489_304 = {S24, 0, &V23489_304_57228, &P23489_302};	/*Property = FileReader*/
const Value V23489_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23489_305 = {S25, 0, &V23489_305_57231, &P23489_304};	/*Property = FileSaver*/
const Value V23489_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23489_306 = {S26, 0, &V23489_306_57237, &P23489_305};	/*Property = FileWriter*/
const Value V23489_307_57240 = {S0, NULL};	/*Value = False*/
const Property P23489_307 = {S27, 0, &V23489_307_57240, &P23489_306};	/*Property = FormData*/
const Value V23489_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23489_309 = {S28, 0, &V23489_309_57246, &P23489_307};	/*Property = Fullscreen*/
const Value V23489_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23489_310 = {S29, 0, &V23489_310_57248, &P23489_309};	/*Property = GeoLocation*/
const Value V23489_312_57255 = {S0, NULL};	/*Value = False*/
const Property P23489_312 = {S30, 0, &V23489_312_57255, &P23489_310};	/*Property = History*/
const Value V23489_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23489_314 = {S31, 0, &V23489_314_57261, &P23489_312};	/*Property = Html5*/
const Value V23489_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23489_313 = {S32, 0, &V23489_313_57258, &P23489_314};	/*Property = Html-Media-Capture*/
const Value V23489_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23489_315 = {S33, 0, &V23489_315_57264, &P23489_313};	/*Property = Iframe*/
const Value V23489_316_57267 = {S0, NULL};	/*Value = False*/
const Property P23489_316 = {S34, 0, &V23489_316_57267, &P23489_315};	/*Property = IndexedDB*/
const Value V23489_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23489_317 = {S35, 0, &V23489_317_57269, &P23489_316};	/*Property = Json*/
const Value V23489_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P23489_15 = {S37, 0, &V23489_15_72, &P23489_317};	/*Property = LayoutEngine*/
const Value V23489_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23489_344 = {S38, 0, &V23489_344_57272, &P23489_15};	/*Property = Masking*/
const Value V23489_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23489_320 = {S39, 0, &V23489_320_57290, &P23489_344};	/*Property = PostMessage*/
const Value V23489_321_57294 = {S0, NULL};	/*Value = False*/
const Property P23489_321 = {S40, 0, &V23489_321_57294, &P23489_320};	/*Property = Progress*/
const Value V23489_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23489_322 = {S41, 0, &V23489_322_57296, &P23489_321};	/*Property = Prompts*/
const Value V23489_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23489_323 = {S42, 0, &V23489_323_57299, &P23489_322};	/*Property = Selector*/
const Value V23489_324_57305 = {S2, NULL};	/*Value = True*/
const Property P23489_324 = {S43, 0, &V23489_324_57305, &P23489_323};	/*Property = Svg*/
const Value V23489_327_57315 = {S0, NULL};	/*Value = False*/
const Property P23489_327 = {S44, 0, &V23489_327_57315, &P23489_324};	/*Property = TouchEvents*/
const Value V23489_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23489_328 = {S45, 0, &V23489_328_57318, &P23489_327};	/*Property = Track*/
const Value V23489_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23489_329 = {S46, 0, &V23489_329_57323, &P23489_328};	/*Property = Video*/
const Value V23489_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23489_330 = {S47, 0, &V23489_330_57326, &P23489_329};	/*Property = Viewport*/
const Value V23489_332_57339 = {S0, NULL};	/*Value = False*/
const Property P23489_332 = {S48, 0, &V23489_332_57339, &P23489_330};	/*Property = WebWorkers*/
const Value V23489_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23489_333 = {S49, 0, &V23489_333_57342, &P23489_332};	/*Property = Xhr2*/
const Profile P23489 = {"23489", &P23489_333};	/*Profile ID - 23489*/
const Value V23305_1_1 = {S2, NULL};	/*Value = True*/
const Property P23305_1 = {S50, 0, &V23305_1_1, NULL};	/*Property = IsMobile*/
const Value V23305_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23305_5 = {S52, 0, &V23305_5_1277, &P23305_1};	/*Property = ScreenPixelsHeight*/
const Value V23305_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23305_4 = {S54, 0, &V23305_4_2336, &P23305_5};	/*Property = ScreenPixelsWidth*/
const Profile P23305 = {"23305", &P23305_4};	/*Profile ID - 23305*/
const Value V23810_1_1 = {S2, NULL};	/*Value = True*/
const Property P23810_1 = {S50, 0, &V23810_1_1, NULL};	/*Property = IsMobile*/
const Value V23810_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23810_5 = {S52, 0, &V23810_5_1222, &P23810_1};	/*Property = ScreenPixelsHeight*/
const Value V23810_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23810_4 = {S54, 0, &V23810_4_1224, &P23810_5};	/*Property = ScreenPixelsWidth*/
const Profile P23810 = {"23810", &P23810_4};	/*Profile ID - 23810*/
const Value V23519_1_1 = {S2, NULL};	/*Value = True*/
const Property P23519_1 = {S50, 0, &V23519_1_1, NULL};	/*Property = IsMobile*/
const Value V23519_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23519_5 = {S52, 0, &V23519_5_1403, &P23519_1};	/*Property = ScreenPixelsHeight*/
const Value V23519_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23519_4 = {S54, 0, &V23519_4_1404, &P23519_5};	/*Property = ScreenPixelsWidth*/
const Profile P23519 = {"23519", &P23519_4};	/*Profile ID - 23519*/
const Value V23320_1_1 = {S2, NULL};	/*Value = True*/
const Property P23320_1 = {S50, 0, &V23320_1_1, NULL};	/*Property = IsMobile*/
const Value V23320_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23320_5 = {S52, 0, &V23320_5_37569, &P23320_1};	/*Property = ScreenPixelsHeight*/
const Value V23320_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23320_4 = {S54, 0, &V23320_4_5645, &P23320_5};	/*Property = ScreenPixelsWidth*/
const Profile P23320 = {"23320", &P23320_4};	/*Profile ID - 23320*/
const Value V23292_1_1 = {S2, NULL};	/*Value = True*/
const Property P23292_1 = {S50, 0, &V23292_1_1, NULL};	/*Property = IsMobile*/
const Value V23292_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23292_5 = {S52, 0, &V23292_5_37, &P23292_1};	/*Property = ScreenPixelsHeight*/
const Value V23292_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23292_4 = {S54, 0, &V23292_4_36, &P23292_5};	/*Property = ScreenPixelsWidth*/
const Profile P23292 = {"23292", &P23292_4};	/*Profile ID - 23292*/
const Value V23344_1_1 = {S2, NULL};	/*Value = True*/
const Property P23344_1 = {S50, 0, &V23344_1_1, NULL};	/*Property = IsMobile*/
const Value V23344_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23344_5 = {S52, 0, &V23344_5_1269, &P23344_1};	/*Property = ScreenPixelsHeight*/
const Value V23344_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23344_4 = {S54, 0, &V23344_4_1229, &P23344_5};	/*Property = ScreenPixelsWidth*/
const Profile P23344 = {"23344", &P23344_4};	/*Profile ID - 23344*/
const Value V23778_1_1 = {S2, NULL};	/*Value = True*/
const Property P23778_1 = {S50, 0, &V23778_1_1, NULL};	/*Property = IsMobile*/
const Value V23778_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23778_5 = {S52, 0, &V23778_5_1227, &P23778_1};	/*Property = ScreenPixelsHeight*/
const Value V23778_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23778_4 = {S54, 0, &V23778_4_1229, &P23778_5};	/*Property = ScreenPixelsWidth*/
const Profile P23778 = {"23778", &P23778_4};	/*Profile ID - 23778*/
const Value V23525_1_1 = {S2, NULL};	/*Value = True*/
const Property P23525_1 = {S50, 0, &V23525_1_1, NULL};	/*Property = IsMobile*/
const Value V23525_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23525_5 = {S52, 0, &V23525_5_1222, &P23525_1};	/*Property = ScreenPixelsHeight*/
const Value V23525_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23525_4 = {S54, 0, &V23525_4_1224, &P23525_5};	/*Property = ScreenPixelsWidth*/
const Profile P23525 = {"23525", &P23525_4};	/*Profile ID - 23525*/
const Value V23288_1_1 = {S2, NULL};	/*Value = True*/
const Property P23288_1 = {S50, 0, &V23288_1_1, NULL};	/*Property = IsMobile*/
const Value V23288_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23288_5 = {S52, 0, &V23288_5_1227, &P23288_1};	/*Property = ScreenPixelsHeight*/
const Value V23288_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23288_4 = {S54, 0, &V23288_4_1229, &P23288_5};	/*Property = ScreenPixelsWidth*/
const Profile P23288 = {"23288", &P23288_4};	/*Profile ID - 23288*/
const Value V23293_1_1 = {S2, NULL};	/*Value = True*/
const Property P23293_1 = {S50, 0, &V23293_1_1, NULL};	/*Property = IsMobile*/
const Value V23293_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23293_5 = {S52, 0, &V23293_5_1227, &P23293_1};	/*Property = ScreenPixelsHeight*/
const Value V23293_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23293_4 = {S54, 0, &V23293_4_1229, &P23293_5};	/*Property = ScreenPixelsWidth*/
const Profile P23293 = {"23293", &P23293_4};	/*Profile ID - 23293*/
const Value V23357_1_1 = {S2, NULL};	/*Value = True*/
const Property P23357_1 = {S50, 0, &V23357_1_1, NULL};	/*Property = IsMobile*/
const Value V23357_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23357_5 = {S52, 0, &V23357_5_1832, &P23357_1};	/*Property = ScreenPixelsHeight*/
const Value V23357_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23357_4 = {S54, 0, &V23357_4_5645, &P23357_5};	/*Property = ScreenPixelsWidth*/
const Profile P23357 = {"23357", &P23357_4};	/*Profile ID - 23357*/
const Value V23568_1_1 = {S2, NULL};	/*Value = True*/
const Property P23568_1 = {S50, 0, &V23568_1_1, NULL};	/*Property = IsMobile*/
const Value V23568_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23568_5 = {S52, 0, &V23568_5_1227, &P23568_1};	/*Property = ScreenPixelsHeight*/
const Value V23568_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23568_4 = {S54, 0, &V23568_4_1229, &P23568_5};	/*Property = ScreenPixelsWidth*/
const Profile P23568 = {"23568", &P23568_4};	/*Profile ID - 23568*/
const Value V23438_1_1 = {S2, NULL};	/*Value = True*/
const Property P23438_1 = {S50, 0, &V23438_1_1, NULL};	/*Property = IsMobile*/
const Value V23438_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23438_5 = {S52, 0, &V23438_5_1277, &P23438_1};	/*Property = ScreenPixelsHeight*/
const Value V23438_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23438_4 = {S54, 0, &V23438_4_2336, &P23438_5};	/*Property = ScreenPixelsWidth*/
const Profile P23438 = {"23438", &P23438_4};	/*Profile ID - 23438*/
const Value V23628_1_1 = {S2, NULL};	/*Value = True*/
const Property P23628_1 = {S50, 0, &V23628_1_1, NULL};	/*Property = IsMobile*/
const Value V23628_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23628_5 = {S52, 0, &V23628_5_1227, &P23628_1};	/*Property = ScreenPixelsHeight*/
const Value V23628_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23628_4 = {S54, 0, &V23628_4_1229, &P23628_5};	/*Property = ScreenPixelsWidth*/
const Profile P23628 = {"23628", &P23628_4};	/*Profile ID - 23628*/
const Value V23362_1_1 = {S2, NULL};	/*Value = True*/
const Property P23362_1 = {S50, 0, &V23362_1_1, NULL};	/*Property = IsMobile*/
const Value V23362_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23362_5 = {S52, 0, &V23362_5_9981, &P23362_1};	/*Property = ScreenPixelsHeight*/
const Value V23362_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23362_4 = {S54, 0, &V23362_4_39098, &P23362_5};	/*Property = ScreenPixelsWidth*/
const Profile P23362 = {"23362", &P23362_4};	/*Profile ID - 23362*/
const Value V21188_1_1 = {S2, NULL};	/*Value = True*/
const Property P21188_1 = {S50, 0, &V21188_1_1, NULL};	/*Property = IsMobile*/
const Value V21188_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P21188_5 = {S52, 0, &V21188_5_37, &P21188_1};	/*Property = ScreenPixelsHeight*/
const Value V21188_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P21188_4 = {S54, 0, &V21188_4_36, &P21188_5};	/*Property = ScreenPixelsWidth*/
const Profile P21188 = {"21188", &P21188_4};	/*Profile ID - 21188*/
const Value V23261_1_1 = {S2, NULL};	/*Value = True*/
const Property P23261_1 = {S50, 0, &V23261_1_1, NULL};	/*Property = IsMobile*/
const Value V23261_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23261_5 = {S52, 0, &V23261_5_1277, &P23261_1};	/*Property = ScreenPixelsHeight*/
const Value V23261_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23261_4 = {S54, 0, &V23261_4_2336, &P23261_5};	/*Property = ScreenPixelsWidth*/
const Profile P23261 = {"23261", &P23261_4};	/*Profile ID - 23261*/
const Value V23250_1_1 = {S2, NULL};	/*Value = True*/
const Property P23250_1 = {S50, 0, &V23250_1_1, NULL};	/*Property = IsMobile*/
const Value V23250_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23250_5 = {S52, 0, &V23250_5_1277, &P23250_1};	/*Property = ScreenPixelsHeight*/
const Value V23250_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23250_4 = {S54, 0, &V23250_4_2336, &P23250_5};	/*Property = ScreenPixelsWidth*/
const Profile P23250 = {"23250", &P23250_4};	/*Profile ID - 23250*/
const Value V23380_1_1 = {S2, NULL};	/*Value = True*/
const Property P23380_1 = {S50, 0, &V23380_1_1, NULL};	/*Property = IsMobile*/
const Value V23380_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23380_5 = {S52, 0, &V23380_5_1277, &P23380_1};	/*Property = ScreenPixelsHeight*/
const Value V23380_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23380_4 = {S54, 0, &V23380_4_2336, &P23380_5};	/*Property = ScreenPixelsWidth*/
const Profile P23380 = {"23380", &P23380_4};	/*Profile ID - 23380*/
const Value V23570_1_1 = {S2, NULL};	/*Value = True*/
const Property P23570_1 = {S50, 0, &V23570_1_1, NULL};	/*Property = IsMobile*/
const Value V23570_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23570_5 = {S52, 0, &V23570_5_1227, &P23570_1};	/*Property = ScreenPixelsHeight*/
const Value V23570_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23570_4 = {S54, 0, &V23570_4_1229, &P23570_5};	/*Property = ScreenPixelsWidth*/
const Profile P23570 = {"23570", &P23570_4};	/*Profile ID - 23570*/
const Value V23315_1_1 = {S2, NULL};	/*Value = True*/
const Property P23315_1 = {S50, 0, &V23315_1_1, NULL};	/*Property = IsMobile*/
const Value V23315_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23315_5 = {S52, 0, &V23315_5_1227, &P23315_1};	/*Property = ScreenPixelsHeight*/
const Value V23315_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23315_4 = {S54, 0, &V23315_4_1229, &P23315_5};	/*Property = ScreenPixelsWidth*/
const Profile P23315 = {"23315", &P23315_4};	/*Profile ID - 23315*/
const Value V23231_1_1 = {S2, NULL};	/*Value = True*/
const Property P23231_1 = {S50, 0, &V23231_1_1, NULL};	/*Property = IsMobile*/
const Value V23231_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P23231_5 = {S52, 0, &V23231_5_59138, &P23231_1};	/*Property = ScreenPixelsHeight*/
const Value V23231_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P23231_4 = {S54, 0, &V23231_4_59970, &P23231_5};	/*Property = ScreenPixelsWidth*/
const Profile P23231 = {"23231", &P23231_4};	/*Profile ID - 23231*/
const Value V23724_1_1 = {S2, NULL};	/*Value = True*/
const Property P23724_1 = {S50, 0, &V23724_1_1, NULL};	/*Property = IsMobile*/
const Value V23724_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23724_5 = {S52, 0, &V23724_5_1277, &P23724_1};	/*Property = ScreenPixelsHeight*/
const Value V23724_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23724_4 = {S54, 0, &V23724_4_1279, &P23724_5};	/*Property = ScreenPixelsWidth*/
const Profile P23724 = {"23724", &P23724_4};	/*Profile ID - 23724*/
const Value V23252_1_1 = {S2, NULL};	/*Value = True*/
const Property P23252_1 = {S50, 0, &V23252_1_1, NULL};	/*Property = IsMobile*/
const Value V23252_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23252_5 = {S52, 0, &V23252_5_9981, &P23252_1};	/*Property = ScreenPixelsHeight*/
const Value V23252_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23252_4 = {S54, 0, &V23252_4_39098, &P23252_5};	/*Property = ScreenPixelsWidth*/
const Profile P23252 = {"23252", &P23252_4};	/*Profile ID - 23252*/
const Value V23957_1_1 = {S2, NULL};	/*Value = True*/
const Property P23957_1 = {S50, 0, &V23957_1_1, NULL};	/*Property = IsMobile*/
const Value V23957_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23957_5 = {S52, 0, &V23957_5_1403, &P23957_1};	/*Property = ScreenPixelsHeight*/
const Value V23957_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23957_4 = {S54, 0, &V23957_4_1404, &P23957_5};	/*Property = ScreenPixelsWidth*/
const Profile P23957 = {"23957", &P23957_4};	/*Profile ID - 23957*/
const Value V23264_1_1 = {S2, NULL};	/*Value = True*/
const Property P23264_1 = {S50, 0, &V23264_1_1, NULL};	/*Property = IsMobile*/
const Value V23264_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23264_5 = {S52, 0, &V23264_5_1222, &P23264_1};	/*Property = ScreenPixelsHeight*/
const Value V23264_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23264_4 = {S54, 0, &V23264_4_1224, &P23264_5};	/*Property = ScreenPixelsWidth*/
const Profile P23264 = {"23264", &P23264_4};	/*Profile ID - 23264*/
const Value V23392_1_1 = {S2, NULL};	/*Value = True*/
const Property P23392_1 = {S50, 0, &V23392_1_1, NULL};	/*Property = IsMobile*/
const Value V23392_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23392_5 = {S52, 0, &V23392_5_1458, &P23392_1};	/*Property = ScreenPixelsHeight*/
const Value V23392_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23392_4 = {S54, 0, &V23392_4_1460, &P23392_5};	/*Property = ScreenPixelsWidth*/
const Profile P23392 = {"23392", &P23392_4};	/*Profile ID - 23392*/
const Value V23224_1_1 = {S2, NULL};	/*Value = True*/
const Property P23224_1 = {S50, 0, &V23224_1_1, NULL};	/*Property = IsMobile*/
const Value V23224_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23224_5 = {S52, 0, &V23224_5_1277, &P23224_1};	/*Property = ScreenPixelsHeight*/
const Value V23224_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23224_4 = {S54, 0, &V23224_4_1279, &P23224_5};	/*Property = ScreenPixelsWidth*/
const Profile P23224 = {"23224", &P23224_4};	/*Profile ID - 23224*/
const Value V23502_1_1 = {S2, NULL};	/*Value = True*/
const Property P23502_1 = {S50, 0, &V23502_1_1, NULL};	/*Property = IsMobile*/
const Value V23502_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23502_5 = {S52, 0, &V23502_5_1277, &P23502_1};	/*Property = ScreenPixelsHeight*/
const Value V23502_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23502_4 = {S54, 0, &V23502_4_1279, &P23502_5};	/*Property = ScreenPixelsWidth*/
const Profile P23502 = {"23502", &P23502_4};	/*Profile ID - 23502*/
const Value V23228_1_1 = {S2, NULL};	/*Value = True*/
const Property P23228_1 = {S50, 0, &V23228_1_1, NULL};	/*Property = IsMobile*/
const Value V23228_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23228_5 = {S52, 0, &V23228_5_1227, &P23228_1};	/*Property = ScreenPixelsHeight*/
const Value V23228_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23228_4 = {S54, 0, &V23228_4_1229, &P23228_5};	/*Property = ScreenPixelsWidth*/
const Profile P23228 = {"23228", &P23228_4};	/*Profile ID - 23228*/
const Value V23263_1_1 = {S2, NULL};	/*Value = True*/
const Property P23263_1 = {S50, 0, &V23263_1_1, NULL};	/*Property = IsMobile*/
const Value V23263_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23263_5 = {S52, 0, &V23263_5_1222, &P23263_1};	/*Property = ScreenPixelsHeight*/
const Value V23263_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23263_4 = {S54, 0, &V23263_4_1224, &P23263_5};	/*Property = ScreenPixelsWidth*/
const Profile P23263 = {"23263", &P23263_4};	/*Profile ID - 23263*/
const Value V23333_1_1 = {S2, NULL};	/*Value = True*/
const Property P23333_1 = {S50, 0, &V23333_1_1, NULL};	/*Property = IsMobile*/
const Value V23333_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23333_5 = {S52, 0, &V23333_5_1222, &P23333_1};	/*Property = ScreenPixelsHeight*/
const Value V23333_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23333_4 = {S54, 0, &V23333_4_1224, &P23333_5};	/*Property = ScreenPixelsWidth*/
const Profile P23333 = {"23333", &P23333_4};	/*Profile ID - 23333*/
const Value V23332_1_1 = {S2, NULL};	/*Value = True*/
const Property P23332_1 = {S50, 0, &V23332_1_1, NULL};	/*Property = IsMobile*/
const Value V23332_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23332_5 = {S52, 0, &V23332_5_1222, &P23332_1};	/*Property = ScreenPixelsHeight*/
const Value V23332_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23332_4 = {S54, 0, &V23332_4_1224, &P23332_5};	/*Property = ScreenPixelsWidth*/
const Profile P23332 = {"23332", &P23332_4};	/*Profile ID - 23332*/
const Value V21503_1_1 = {S2, NULL};	/*Value = True*/
const Property P21503_1 = {S50, 0, &V21503_1_1, NULL};	/*Property = IsMobile*/
const Value V21503_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P21503_5 = {S52, 0, &V21503_5_1227, &P21503_1};	/*Property = ScreenPixelsHeight*/
const Value V21503_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P21503_4 = {S54, 0, &V21503_4_1229, &P21503_5};	/*Property = ScreenPixelsWidth*/
const Profile P21503 = {"21503", &P21503_4};	/*Profile ID - 21503*/
const Value V24329_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_279 = {S1, 0, &V24329_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24329_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_282 = {S3, 0, &V24329_282_57155, &P24329_279};	/*Property = BlobBuilder*/
const Value V24329_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_350 = {S4, 0, &V24329_350_57446, &P24329_282};	/*Property = Canvas*/
const Value V24329_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_283 = {S5, 0, &V24329_283_57161, &P24329_350};	/*Property = CssBackground*/
const Value V24329_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_284 = {S6, 0, &V24329_284_57164, &P24329_283};	/*Property = CssBorderImage*/
const Value V24329_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_285 = {S7, 0, &V24329_285_57167, &P24329_284};	/*Property = CssCanvas*/
const Value V24329_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_286 = {S8, 0, &V24329_286_57170, &P24329_285};	/*Property = CssColor*/
const Value V24329_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_237 = {S9, 0, &V24329_237_55786, &P24329_286};	/*Property = CssColumn*/
const Value V24329_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_287 = {S10, 0, &V24329_287_57173, &P24329_237};	/*Property = CssFlexbox*/
const Value V24329_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_288 = {S11, 0, &V24329_288_57176, &P24329_287};	/*Property = CssFont*/
const Value V24329_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_289 = {S12, 0, &V24329_289_86899, &P24329_288};	/*Property = CssImages*/
const Value V24329_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_291 = {S13, 0, &V24329_291_57182, &P24329_289};	/*Property = CssMediaQueries*/
const Value V24329_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_292 = {S14, 0, &V24329_292_57185, &P24329_291};	/*Property = CssMinMax*/
const Value V24329_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_293 = {S15, 0, &V24329_293_57188, &P24329_292};	/*Property = CssOverflow*/
const Value V24329_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_294 = {S16, 0, &V24329_294_57191, &P24329_293};	/*Property = CssPosition*/
const Value V24329_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_295 = {S17, 0, &V24329_295_57194, &P24329_294};	/*Property = CssText*/
const Value V24329_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_297 = {S18, 0, &V24329_297_57197, &P24329_295};	/*Property = CssTransforms*/
const Value V24329_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_241 = {S19, 0, &V24329_241_55801, &P24329_297};	/*Property = CssTransitions*/
const Value V24329_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_298 = {S20, 0, &V24329_298_57208, &P24329_241};	/*Property = CssUI*/
const Value V24329_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_300 = {S21, 0, &V24329_300_57217, &P24329_298};	/*Property = DataSet*/
const Value V24329_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_301 = {S22, 0, &V24329_301_57220, &P24329_300};	/*Property = DataUrl*/
const Value V24329_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_302 = {S23, 0, &V24329_302_57223, &P24329_301};	/*Property = DeviceOrientation*/
const Value V24329_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_304 = {S24, 0, &V24329_304_57229, &P24329_302};	/*Property = FileReader*/
const Value V24329_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_305 = {S25, 0, &V24329_305_57232, &P24329_304};	/*Property = FileSaver*/
const Value V24329_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_306 = {S26, 0, &V24329_306_57238, &P24329_305};	/*Property = FileWriter*/
const Value V24329_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_307 = {S27, 0, &V24329_307_57241, &P24329_306};	/*Property = FormData*/
const Value V24329_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_309 = {S28, 0, &V24329_309_57247, &P24329_307};	/*Property = Fullscreen*/
const Value V24329_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_310 = {S29, 0, &V24329_310_57250, &P24329_309};	/*Property = GeoLocation*/
const Value V24329_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_312 = {S30, 0, &V24329_312_57256, &P24329_310};	/*Property = History*/
const Value V24329_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_314 = {S31, 0, &V24329_314_57262, &P24329_312};	/*Property = Html5*/
const Value V24329_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_313 = {S32, 0, &V24329_313_57259, &P24329_314};	/*Property = Html-Media-Capture*/
const Value V24329_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_315 = {S33, 0, &V24329_315_57265, &P24329_313};	/*Property = Iframe*/
const Value V24329_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_316 = {S34, 0, &V24329_316_57268, &P24329_315};	/*Property = IndexedDB*/
const Value V24329_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_317 = {S35, 0, &V24329_317_57271, &P24329_316};	/*Property = Json*/
const Value V24329_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P24329_15 = {S37, 0, &V24329_15_29, &P24329_317};	/*Property = LayoutEngine*/
const Value V24329_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_344 = {S38, 0, &V24329_344_57274, &P24329_15};	/*Property = Masking*/
const Value V24329_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_320 = {S39, 0, &V24329_320_57292, &P24329_344};	/*Property = PostMessage*/
const Value V24329_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_321 = {S40, 0, &V24329_321_57295, &P24329_320};	/*Property = Progress*/
const Value V24329_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_322 = {S41, 0, &V24329_322_57298, &P24329_321};	/*Property = Prompts*/
const Value V24329_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_323 = {S42, 0, &V24329_323_57301, &P24329_322};	/*Property = Selector*/
const Value V24329_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_324 = {S43, 0, &V24329_324_57307, &P24329_323};	/*Property = Svg*/
const Value V24329_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_327 = {S44, 0, &V24329_327_57316, &P24329_324};	/*Property = TouchEvents*/
const Value V24329_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_328 = {S45, 0, &V24329_328_57319, &P24329_327};	/*Property = Track*/
const Value V24329_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_329 = {S46, 0, &V24329_329_57325, &P24329_328};	/*Property = Video*/
const Value V24329_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_330 = {S47, 0, &V24329_330_57328, &P24329_329};	/*Property = Viewport*/
const Value V24329_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_332 = {S48, 0, &V24329_332_57340, &P24329_330};	/*Property = WebWorkers*/
const Value V24329_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24329_333 = {S49, 0, &V24329_333_57343, &P24329_332};	/*Property = Xhr2*/
const Profile P24329 = {"24329", &P24329_333};	/*Profile ID - 24329*/
const Value V23610_1_1 = {S2, NULL};	/*Value = True*/
const Property P23610_1 = {S50, 0, &V23610_1_1, NULL};	/*Property = IsMobile*/
const Value V23610_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23610_5 = {S52, 0, &V23610_5_1458, &P23610_1};	/*Property = ScreenPixelsHeight*/
const Value V23610_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23610_4 = {S54, 0, &V23610_4_37617, &P23610_5};	/*Property = ScreenPixelsWidth*/
const Profile P23610 = {"23610", &P23610_4};	/*Profile ID - 23610*/
const Value V23221_1_1 = {S2, NULL};	/*Value = True*/
const Property P23221_1 = {S50, 0, &V23221_1_1, NULL};	/*Property = IsMobile*/
const Value V23221_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23221_5 = {S52, 0, &V23221_5_1269, &P23221_1};	/*Property = ScreenPixelsHeight*/
const Value V23221_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23221_4 = {S54, 0, &V23221_4_1229, &P23221_5};	/*Property = ScreenPixelsWidth*/
const Profile P23221 = {"23221", &P23221_4};	/*Profile ID - 23221*/
const Value V24136_1_1 = {S2, NULL};	/*Value = True*/
const Property P24136_1 = {S50, 0, &V24136_1_1, NULL};	/*Property = IsMobile*/
const Value V24136_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24136_5 = {S52, 0, &V24136_5_1277, &P24136_1};	/*Property = ScreenPixelsHeight*/
const Value V24136_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24136_4 = {S54, 0, &V24136_4_1279, &P24136_5};	/*Property = ScreenPixelsWidth*/
const Profile P24136 = {"24136", &P24136_4};	/*Profile ID - 24136*/
const Value V23748_1_1 = {S2, NULL};	/*Value = True*/
const Property P23748_1 = {S50, 0, &V23748_1_1, NULL};	/*Property = IsMobile*/
const Value V23748_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23748_5 = {S52, 0, &V23748_5_1403, &P23748_1};	/*Property = ScreenPixelsHeight*/
const Value V23748_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23748_4 = {S54, 0, &V23748_4_1404, &P23748_5};	/*Property = ScreenPixelsWidth*/
const Profile P23748 = {"23748", &P23748_4};	/*Profile ID - 23748*/
const Value V23759_1_1 = {S2, NULL};	/*Value = True*/
const Property P23759_1 = {S50, 0, &V23759_1_1, NULL};	/*Property = IsMobile*/
const Value V23759_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23759_5 = {S52, 0, &V23759_5_1458, &P23759_1};	/*Property = ScreenPixelsHeight*/
const Value V23759_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23759_4 = {S54, 0, &V23759_4_1460, &P23759_5};	/*Property = ScreenPixelsWidth*/
const Profile P23759 = {"23759", &P23759_4};	/*Profile ID - 23759*/
const Value V23503_1_1 = {S2, NULL};	/*Value = True*/
const Property P23503_1 = {S50, 0, &V23503_1_1, NULL};	/*Property = IsMobile*/
const Value V23503_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23503_5 = {S52, 0, &V23503_5_1227, &P23503_1};	/*Property = ScreenPixelsHeight*/
const Value V23503_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23503_4 = {S54, 0, &V23503_4_1229, &P23503_5};	/*Property = ScreenPixelsWidth*/
const Profile P23503 = {"23503", &P23503_4};	/*Profile ID - 23503*/
const Value V23666_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_279 = {S1, 0, &V23666_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23666_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_282 = {S3, 0, &V23666_282_57155, &P23666_279};	/*Property = BlobBuilder*/
const Value V23666_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_350 = {S4, 0, &V23666_350_57446, &P23666_282};	/*Property = Canvas*/
const Value V23666_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_283 = {S5, 0, &V23666_283_57161, &P23666_350};	/*Property = CssBackground*/
const Value V23666_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_284 = {S6, 0, &V23666_284_57164, &P23666_283};	/*Property = CssBorderImage*/
const Value V23666_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_285 = {S7, 0, &V23666_285_57167, &P23666_284};	/*Property = CssCanvas*/
const Value V23666_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_286 = {S8, 0, &V23666_286_57170, &P23666_285};	/*Property = CssColor*/
const Value V23666_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_237 = {S9, 0, &V23666_237_55786, &P23666_286};	/*Property = CssColumn*/
const Value V23666_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_287 = {S10, 0, &V23666_287_57173, &P23666_237};	/*Property = CssFlexbox*/
const Value V23666_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_288 = {S11, 0, &V23666_288_57176, &P23666_287};	/*Property = CssFont*/
const Value V23666_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_289 = {S12, 0, &V23666_289_86899, &P23666_288};	/*Property = CssImages*/
const Value V23666_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_291 = {S13, 0, &V23666_291_57182, &P23666_289};	/*Property = CssMediaQueries*/
const Value V23666_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_292 = {S14, 0, &V23666_292_57185, &P23666_291};	/*Property = CssMinMax*/
const Value V23666_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_293 = {S15, 0, &V23666_293_57188, &P23666_292};	/*Property = CssOverflow*/
const Value V23666_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_294 = {S16, 0, &V23666_294_57191, &P23666_293};	/*Property = CssPosition*/
const Value V23666_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_295 = {S17, 0, &V23666_295_57194, &P23666_294};	/*Property = CssText*/
const Value V23666_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_297 = {S18, 0, &V23666_297_57197, &P23666_295};	/*Property = CssTransforms*/
const Value V23666_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_241 = {S19, 0, &V23666_241_55801, &P23666_297};	/*Property = CssTransitions*/
const Value V23666_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_298 = {S20, 0, &V23666_298_57208, &P23666_241};	/*Property = CssUI*/
const Value V23666_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_300 = {S21, 0, &V23666_300_57217, &P23666_298};	/*Property = DataSet*/
const Value V23666_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_301 = {S22, 0, &V23666_301_57220, &P23666_300};	/*Property = DataUrl*/
const Value V23666_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_302 = {S23, 0, &V23666_302_57223, &P23666_301};	/*Property = DeviceOrientation*/
const Value V23666_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_304 = {S24, 0, &V23666_304_57229, &P23666_302};	/*Property = FileReader*/
const Value V23666_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_305 = {S25, 0, &V23666_305_57232, &P23666_304};	/*Property = FileSaver*/
const Value V23666_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_306 = {S26, 0, &V23666_306_57238, &P23666_305};	/*Property = FileWriter*/
const Value V23666_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_307 = {S27, 0, &V23666_307_57241, &P23666_306};	/*Property = FormData*/
const Value V23666_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_309 = {S28, 0, &V23666_309_57247, &P23666_307};	/*Property = Fullscreen*/
const Value V23666_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_310 = {S29, 0, &V23666_310_57250, &P23666_309};	/*Property = GeoLocation*/
const Value V23666_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_312 = {S30, 0, &V23666_312_57256, &P23666_310};	/*Property = History*/
const Value V23666_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_314 = {S31, 0, &V23666_314_57262, &P23666_312};	/*Property = Html5*/
const Value V23666_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_313 = {S32, 0, &V23666_313_57259, &P23666_314};	/*Property = Html-Media-Capture*/
const Value V23666_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_315 = {S33, 0, &V23666_315_57265, &P23666_313};	/*Property = Iframe*/
const Value V23666_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_316 = {S34, 0, &V23666_316_57268, &P23666_315};	/*Property = IndexedDB*/
const Value V23666_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_317 = {S35, 0, &V23666_317_57271, &P23666_316};	/*Property = Json*/
const Value V23666_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P23666_15 = {S37, 0, &V23666_15_29, &P23666_317};	/*Property = LayoutEngine*/
const Value V23666_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_344 = {S38, 0, &V23666_344_57274, &P23666_15};	/*Property = Masking*/
const Value V23666_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_320 = {S39, 0, &V23666_320_57292, &P23666_344};	/*Property = PostMessage*/
const Value V23666_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_321 = {S40, 0, &V23666_321_57295, &P23666_320};	/*Property = Progress*/
const Value V23666_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_322 = {S41, 0, &V23666_322_57298, &P23666_321};	/*Property = Prompts*/
const Value V23666_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_323 = {S42, 0, &V23666_323_57301, &P23666_322};	/*Property = Selector*/
const Value V23666_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_324 = {S43, 0, &V23666_324_57307, &P23666_323};	/*Property = Svg*/
const Value V23666_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_327 = {S44, 0, &V23666_327_57316, &P23666_324};	/*Property = TouchEvents*/
const Value V23666_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_328 = {S45, 0, &V23666_328_57319, &P23666_327};	/*Property = Track*/
const Value V23666_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_329 = {S46, 0, &V23666_329_57325, &P23666_328};	/*Property = Video*/
const Value V23666_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_330 = {S47, 0, &V23666_330_57328, &P23666_329};	/*Property = Viewport*/
const Value V23666_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_332 = {S48, 0, &V23666_332_57340, &P23666_330};	/*Property = WebWorkers*/
const Value V23666_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23666_333 = {S49, 0, &V23666_333_57343, &P23666_332};	/*Property = Xhr2*/
const Profile P23666 = {"23666", &P23666_333};	/*Profile ID - 23666*/
const Value V23797_1_1 = {S2, NULL};	/*Value = True*/
const Property P23797_1 = {S50, 0, &V23797_1_1, NULL};	/*Property = IsMobile*/
const Value V23797_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23797_5 = {S52, 0, &V23797_5_1227, &P23797_1};	/*Property = ScreenPixelsHeight*/
const Value V23797_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P23797_4 = {S54, 0, &V23797_4_1477, &P23797_5};	/*Property = ScreenPixelsWidth*/
const Profile P23797 = {"23797", &P23797_4};	/*Profile ID - 23797*/
const Value V23258_1_1 = {S2, NULL};	/*Value = True*/
const Property P23258_1 = {S50, 0, &V23258_1_1, NULL};	/*Property = IsMobile*/
const Value V23258_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23258_5 = {S52, 0, &V23258_5_1227, &P23258_1};	/*Property = ScreenPixelsHeight*/
const Value V23258_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23258_4 = {S54, 0, &V23258_4_1229, &P23258_5};	/*Property = ScreenPixelsWidth*/
const Profile P23258 = {"23258", &P23258_4};	/*Profile ID - 23258*/
const Value V23742_1_1 = {S2, NULL};	/*Value = True*/
const Property P23742_1 = {S50, 0, &V23742_1_1, NULL};	/*Property = IsMobile*/
const Value V23742_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23742_5 = {S52, 0, &V23742_5_9981, &P23742_1};	/*Property = ScreenPixelsHeight*/
const Value V23742_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23742_4 = {S54, 0, &V23742_4_39098, &P23742_5};	/*Property = ScreenPixelsWidth*/
const Profile P23742 = {"23742", &P23742_4};	/*Profile ID - 23742*/
const Value V23248_1_1 = {S2, NULL};	/*Value = True*/
const Property P23248_1 = {S50, 0, &V23248_1_1, NULL};	/*Property = IsMobile*/
const Value V23248_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23248_5 = {S52, 0, &V23248_5_1403, &P23248_1};	/*Property = ScreenPixelsHeight*/
const Value V23248_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23248_4 = {S54, 0, &V23248_4_1404, &P23248_5};	/*Property = ScreenPixelsWidth*/
const Profile P23248 = {"23248", &P23248_4};	/*Profile ID - 23248*/
const Value V23389_1_1 = {S2, NULL};	/*Value = True*/
const Property P23389_1 = {S50, 0, &V23389_1_1, NULL};	/*Property = IsMobile*/
const Value V23389_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23389_5 = {S52, 0, &V23389_5_1227, &P23389_1};	/*Property = ScreenPixelsHeight*/
const Value V23389_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23389_4 = {S54, 0, &V23389_4_1229, &P23389_5};	/*Property = ScreenPixelsWidth*/
const Profile P23389 = {"23389", &P23389_4};	/*Profile ID - 23389*/
const Value V23237_1_1 = {S2, NULL};	/*Value = True*/
const Property P23237_1 = {S50, 0, &V23237_1_1, NULL};	/*Property = IsMobile*/
const Value V23237_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23237_5 = {S52, 0, &V23237_5_1403, &P23237_1};	/*Property = ScreenPixelsHeight*/
const Value V23237_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23237_4 = {S54, 0, &V23237_4_1404, &P23237_5};	/*Property = ScreenPixelsWidth*/
const Profile P23237 = {"23237", &P23237_4};	/*Profile ID - 23237*/
const Value V23585_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_279 = {S1, 0, &V23585_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23585_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_282 = {S3, 0, &V23585_282_57155, &P23585_279};	/*Property = BlobBuilder*/
const Value V23585_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_350 = {S4, 0, &V23585_350_57446, &P23585_282};	/*Property = Canvas*/
const Value V23585_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_283 = {S5, 0, &V23585_283_57161, &P23585_350};	/*Property = CssBackground*/
const Value V23585_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_284 = {S6, 0, &V23585_284_57164, &P23585_283};	/*Property = CssBorderImage*/
const Value V23585_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_285 = {S7, 0, &V23585_285_57167, &P23585_284};	/*Property = CssCanvas*/
const Value V23585_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_286 = {S8, 0, &V23585_286_57170, &P23585_285};	/*Property = CssColor*/
const Value V23585_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_237 = {S9, 0, &V23585_237_55786, &P23585_286};	/*Property = CssColumn*/
const Value V23585_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_287 = {S10, 0, &V23585_287_57173, &P23585_237};	/*Property = CssFlexbox*/
const Value V23585_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_288 = {S11, 0, &V23585_288_57176, &P23585_287};	/*Property = CssFont*/
const Value V23585_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_289 = {S12, 0, &V23585_289_86899, &P23585_288};	/*Property = CssImages*/
const Value V23585_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_291 = {S13, 0, &V23585_291_57182, &P23585_289};	/*Property = CssMediaQueries*/
const Value V23585_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_292 = {S14, 0, &V23585_292_57185, &P23585_291};	/*Property = CssMinMax*/
const Value V23585_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_293 = {S15, 0, &V23585_293_57188, &P23585_292};	/*Property = CssOverflow*/
const Value V23585_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_294 = {S16, 0, &V23585_294_57191, &P23585_293};	/*Property = CssPosition*/
const Value V23585_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_295 = {S17, 0, &V23585_295_57194, &P23585_294};	/*Property = CssText*/
const Value V23585_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_297 = {S18, 0, &V23585_297_57197, &P23585_295};	/*Property = CssTransforms*/
const Value V23585_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_241 = {S19, 0, &V23585_241_55801, &P23585_297};	/*Property = CssTransitions*/
const Value V23585_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_298 = {S20, 0, &V23585_298_57208, &P23585_241};	/*Property = CssUI*/
const Value V23585_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_300 = {S21, 0, &V23585_300_57217, &P23585_298};	/*Property = DataSet*/
const Value V23585_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_301 = {S22, 0, &V23585_301_57220, &P23585_300};	/*Property = DataUrl*/
const Value V23585_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_302 = {S23, 0, &V23585_302_57223, &P23585_301};	/*Property = DeviceOrientation*/
const Value V23585_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_304 = {S24, 0, &V23585_304_57229, &P23585_302};	/*Property = FileReader*/
const Value V23585_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_305 = {S25, 0, &V23585_305_57232, &P23585_304};	/*Property = FileSaver*/
const Value V23585_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_306 = {S26, 0, &V23585_306_57238, &P23585_305};	/*Property = FileWriter*/
const Value V23585_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_307 = {S27, 0, &V23585_307_57241, &P23585_306};	/*Property = FormData*/
const Value V23585_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_309 = {S28, 0, &V23585_309_57247, &P23585_307};	/*Property = Fullscreen*/
const Value V23585_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_310 = {S29, 0, &V23585_310_57250, &P23585_309};	/*Property = GeoLocation*/
const Value V23585_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_312 = {S30, 0, &V23585_312_57256, &P23585_310};	/*Property = History*/
const Value V23585_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_314 = {S31, 0, &V23585_314_57262, &P23585_312};	/*Property = Html5*/
const Value V23585_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_313 = {S32, 0, &V23585_313_57259, &P23585_314};	/*Property = Html-Media-Capture*/
const Value V23585_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_315 = {S33, 0, &V23585_315_57265, &P23585_313};	/*Property = Iframe*/
const Value V23585_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_316 = {S34, 0, &V23585_316_57268, &P23585_315};	/*Property = IndexedDB*/
const Value V23585_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_317 = {S35, 0, &V23585_317_57271, &P23585_316};	/*Property = Json*/
const Value V23585_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P23585_15 = {S37, 0, &V23585_15_29, &P23585_317};	/*Property = LayoutEngine*/
const Value V23585_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_344 = {S38, 0, &V23585_344_57274, &P23585_15};	/*Property = Masking*/
const Value V23585_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_320 = {S39, 0, &V23585_320_57292, &P23585_344};	/*Property = PostMessage*/
const Value V23585_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_321 = {S40, 0, &V23585_321_57295, &P23585_320};	/*Property = Progress*/
const Value V23585_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_322 = {S41, 0, &V23585_322_57298, &P23585_321};	/*Property = Prompts*/
const Value V23585_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_323 = {S42, 0, &V23585_323_57301, &P23585_322};	/*Property = Selector*/
const Value V23585_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_324 = {S43, 0, &V23585_324_57307, &P23585_323};	/*Property = Svg*/
const Value V23585_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_327 = {S44, 0, &V23585_327_57316, &P23585_324};	/*Property = TouchEvents*/
const Value V23585_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_328 = {S45, 0, &V23585_328_57319, &P23585_327};	/*Property = Track*/
const Value V23585_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_329 = {S46, 0, &V23585_329_57325, &P23585_328};	/*Property = Video*/
const Value V23585_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_330 = {S47, 0, &V23585_330_57328, &P23585_329};	/*Property = Viewport*/
const Value V23585_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_332 = {S48, 0, &V23585_332_57340, &P23585_330};	/*Property = WebWorkers*/
const Value V23585_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23585_333 = {S49, 0, &V23585_333_57343, &P23585_332};	/*Property = Xhr2*/
const Profile P23585 = {"23585", &P23585_333};	/*Profile ID - 23585*/
const Value V22158_1_1 = {S2, NULL};	/*Value = True*/
const Property P22158_1 = {S50, 0, &V22158_1_1, NULL};	/*Property = IsMobile*/
const Value V22158_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P22158_5 = {S52, 0, &V22158_5_1277, &P22158_1};	/*Property = ScreenPixelsHeight*/
const Value V22158_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P22158_4 = {S54, 0, &V22158_4_1279, &P22158_5};	/*Property = ScreenPixelsWidth*/
const Profile P22158 = {"22158", &P22158_4};	/*Profile ID - 22158*/
const Value V24648_1_1 = {S2, NULL};	/*Value = True*/
const Property P24648_1 = {S50, 0, &V24648_1_1, NULL};	/*Property = IsMobile*/
const Value V24648_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24648_5 = {S52, 0, &V24648_5_1227, &P24648_1};	/*Property = ScreenPixelsHeight*/
const Value V24648_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24648_4 = {S54, 0, &V24648_4_1229, &P24648_5};	/*Property = ScreenPixelsWidth*/
const Profile P24648 = {"24648", &P24648_4};	/*Profile ID - 24648*/
const Value V23667_1_1 = {S2, NULL};	/*Value = True*/
const Property P23667_1 = {S50, 0, &V23667_1_1, NULL};	/*Property = IsMobile*/
const Value V23667_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23667_5 = {S52, 0, &V23667_5_9981, &P23667_1};	/*Property = ScreenPixelsHeight*/
const Value V23667_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23667_4 = {S54, 0, &V23667_4_39098, &P23667_5};	/*Property = ScreenPixelsWidth*/
const Profile P23667 = {"23667", &P23667_4};	/*Profile ID - 23667*/
const Value V23276_1_1 = {S2, NULL};	/*Value = True*/
const Property P23276_1 = {S50, 0, &V23276_1_1, NULL};	/*Property = IsMobile*/
const Value V23276_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23276_5 = {S52, 0, &V23276_5_1458, &P23276_1};	/*Property = ScreenPixelsHeight*/
const Value V23276_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23276_4 = {S54, 0, &V23276_4_1460, &P23276_5};	/*Property = ScreenPixelsWidth*/
const Profile P23276 = {"23276", &P23276_4};	/*Profile ID - 23276*/
const Value V23425_1_1 = {S2, NULL};	/*Value = True*/
const Property P23425_1 = {S50, 0, &V23425_1_1, NULL};	/*Property = IsMobile*/
const Value V23425_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23425_5 = {S52, 0, &V23425_5_1458, &P23425_1};	/*Property = ScreenPixelsHeight*/
const Value V23425_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23425_4 = {S54, 0, &V23425_4_1460, &P23425_5};	/*Property = ScreenPixelsWidth*/
const Profile P23425 = {"23425", &P23425_4};	/*Profile ID - 23425*/
const Value V23461_1_1 = {S2, NULL};	/*Value = True*/
const Property P23461_1 = {S50, 0, &V23461_1_1, NULL};	/*Property = IsMobile*/
const Value V23461_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23461_5 = {S52, 0, &V23461_5_1227, &P23461_1};	/*Property = ScreenPixelsHeight*/
const Value V23461_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23461_4 = {S54, 0, &V23461_4_1229, &P23461_5};	/*Property = ScreenPixelsWidth*/
const Profile P23461 = {"23461", &P23461_4};	/*Profile ID - 23461*/
const Value V23254_1_1 = {S2, NULL};	/*Value = True*/
const Property P23254_1 = {S50, 0, &V23254_1_1, NULL};	/*Property = IsMobile*/
const Value V23254_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23254_5 = {S52, 0, &V23254_5_9981, &P23254_1};	/*Property = ScreenPixelsHeight*/
const Value V23254_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23254_4 = {S54, 0, &V23254_4_39098, &P23254_5};	/*Property = ScreenPixelsWidth*/
const Profile P23254 = {"23254", &P23254_4};	/*Profile ID - 23254*/
const Value V23733_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_279 = {S1, 0, &V23733_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23733_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_282 = {S3, 0, &V23733_282_57155, &P23733_279};	/*Property = BlobBuilder*/
const Value V23733_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_350 = {S4, 0, &V23733_350_57446, &P23733_282};	/*Property = Canvas*/
const Value V23733_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_283 = {S5, 0, &V23733_283_57161, &P23733_350};	/*Property = CssBackground*/
const Value V23733_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_284 = {S6, 0, &V23733_284_57164, &P23733_283};	/*Property = CssBorderImage*/
const Value V23733_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_285 = {S7, 0, &V23733_285_57167, &P23733_284};	/*Property = CssCanvas*/
const Value V23733_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_286 = {S8, 0, &V23733_286_57170, &P23733_285};	/*Property = CssColor*/
const Value V23733_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_237 = {S9, 0, &V23733_237_55786, &P23733_286};	/*Property = CssColumn*/
const Value V23733_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_287 = {S10, 0, &V23733_287_57173, &P23733_237};	/*Property = CssFlexbox*/
const Value V23733_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_288 = {S11, 0, &V23733_288_57176, &P23733_287};	/*Property = CssFont*/
const Value V23733_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_289 = {S12, 0, &V23733_289_86899, &P23733_288};	/*Property = CssImages*/
const Value V23733_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_291 = {S13, 0, &V23733_291_57182, &P23733_289};	/*Property = CssMediaQueries*/
const Value V23733_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_292 = {S14, 0, &V23733_292_57185, &P23733_291};	/*Property = CssMinMax*/
const Value V23733_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_293 = {S15, 0, &V23733_293_57188, &P23733_292};	/*Property = CssOverflow*/
const Value V23733_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_294 = {S16, 0, &V23733_294_57191, &P23733_293};	/*Property = CssPosition*/
const Value V23733_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_295 = {S17, 0, &V23733_295_57194, &P23733_294};	/*Property = CssText*/
const Value V23733_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_297 = {S18, 0, &V23733_297_57197, &P23733_295};	/*Property = CssTransforms*/
const Value V23733_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_241 = {S19, 0, &V23733_241_55801, &P23733_297};	/*Property = CssTransitions*/
const Value V23733_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_298 = {S20, 0, &V23733_298_57208, &P23733_241};	/*Property = CssUI*/
const Value V23733_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_300 = {S21, 0, &V23733_300_57217, &P23733_298};	/*Property = DataSet*/
const Value V23733_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_301 = {S22, 0, &V23733_301_57220, &P23733_300};	/*Property = DataUrl*/
const Value V23733_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_302 = {S23, 0, &V23733_302_57223, &P23733_301};	/*Property = DeviceOrientation*/
const Value V23733_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_304 = {S24, 0, &V23733_304_57229, &P23733_302};	/*Property = FileReader*/
const Value V23733_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_305 = {S25, 0, &V23733_305_57232, &P23733_304};	/*Property = FileSaver*/
const Value V23733_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_306 = {S26, 0, &V23733_306_57238, &P23733_305};	/*Property = FileWriter*/
const Value V23733_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_307 = {S27, 0, &V23733_307_57241, &P23733_306};	/*Property = FormData*/
const Value V23733_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_309 = {S28, 0, &V23733_309_57247, &P23733_307};	/*Property = Fullscreen*/
const Value V23733_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_310 = {S29, 0, &V23733_310_57250, &P23733_309};	/*Property = GeoLocation*/
const Value V23733_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_312 = {S30, 0, &V23733_312_57256, &P23733_310};	/*Property = History*/
const Value V23733_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_314 = {S31, 0, &V23733_314_57262, &P23733_312};	/*Property = Html5*/
const Value V23733_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_313 = {S32, 0, &V23733_313_57259, &P23733_314};	/*Property = Html-Media-Capture*/
const Value V23733_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_315 = {S33, 0, &V23733_315_57265, &P23733_313};	/*Property = Iframe*/
const Value V23733_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_316 = {S34, 0, &V23733_316_57268, &P23733_315};	/*Property = IndexedDB*/
const Value V23733_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_317 = {S35, 0, &V23733_317_57271, &P23733_316};	/*Property = Json*/
const Value V23733_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P23733_15 = {S37, 0, &V23733_15_28, &P23733_317};	/*Property = LayoutEngine*/
const Value V23733_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_344 = {S38, 0, &V23733_344_57274, &P23733_15};	/*Property = Masking*/
const Value V23733_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_320 = {S39, 0, &V23733_320_57292, &P23733_344};	/*Property = PostMessage*/
const Value V23733_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_321 = {S40, 0, &V23733_321_57295, &P23733_320};	/*Property = Progress*/
const Value V23733_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_322 = {S41, 0, &V23733_322_57298, &P23733_321};	/*Property = Prompts*/
const Value V23733_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_323 = {S42, 0, &V23733_323_57301, &P23733_322};	/*Property = Selector*/
const Value V23733_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_324 = {S43, 0, &V23733_324_57307, &P23733_323};	/*Property = Svg*/
const Value V23733_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_327 = {S44, 0, &V23733_327_57316, &P23733_324};	/*Property = TouchEvents*/
const Value V23733_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_328 = {S45, 0, &V23733_328_57319, &P23733_327};	/*Property = Track*/
const Value V23733_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_329 = {S46, 0, &V23733_329_57325, &P23733_328};	/*Property = Video*/
const Value V23733_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_330 = {S47, 0, &V23733_330_57328, &P23733_329};	/*Property = Viewport*/
const Value V23733_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_332 = {S48, 0, &V23733_332_57340, &P23733_330};	/*Property = WebWorkers*/
const Value V23733_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23733_333 = {S49, 0, &V23733_333_57343, &P23733_332};	/*Property = Xhr2*/
const Profile P23733 = {"23733", &P23733_333};	/*Profile ID - 23733*/
const Value V23716_1_1 = {S2, NULL};	/*Value = True*/
const Property P23716_1 = {S50, 0, &V23716_1_1, NULL};	/*Property = IsMobile*/
const Value V23716_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23716_5 = {S52, 0, &V23716_5_9981, &P23716_1};	/*Property = ScreenPixelsHeight*/
const Value V23716_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23716_4 = {S54, 0, &V23716_4_39098, &P23716_5};	/*Property = ScreenPixelsWidth*/
const Profile P23716 = {"23716", &P23716_4};	/*Profile ID - 23716*/
const Value V24168_1_1 = {S2, NULL};	/*Value = True*/
const Property P24168_1 = {S50, 0, &V24168_1_1, NULL};	/*Property = IsMobile*/
const Value V24168_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24168_5 = {S52, 0, &V24168_5_1227, &P24168_1};	/*Property = ScreenPixelsHeight*/
const Value V24168_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24168_4 = {S54, 0, &V24168_4_1229, &P24168_5};	/*Property = ScreenPixelsWidth*/
const Profile P24168 = {"24168", &P24168_4};	/*Profile ID - 24168*/
const Value V23259_1_1 = {S2, NULL};	/*Value = True*/
const Property P23259_1 = {S50, 0, &V23259_1_1, NULL};	/*Property = IsMobile*/
const Value V23259_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23259_5 = {S52, 0, &V23259_5_1227, &P23259_1};	/*Property = ScreenPixelsHeight*/
const Value V23259_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23259_4 = {S54, 0, &V23259_4_1229, &P23259_5};	/*Property = ScreenPixelsWidth*/
const Profile P23259 = {"23259", &P23259_4};	/*Profile ID - 23259*/
const Value V23719_1_1 = {S2, NULL};	/*Value = True*/
const Property P23719_1 = {S50, 0, &V23719_1_1, NULL};	/*Property = IsMobile*/
const Value V23719_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23719_5 = {S52, 0, &V23719_5_1277, &P23719_1};	/*Property = ScreenPixelsHeight*/
const Value V23719_4_37618 = {S83, NULL};	/*Value = 272*/
const Property P23719_4 = {S54, 0, &V23719_4_37618, &P23719_5};	/*Property = ScreenPixelsWidth*/
const Profile P23719 = {"23719", &P23719_4};	/*Profile ID - 23719*/
const Value V24150_1_1 = {S2, NULL};	/*Value = True*/
const Property P24150_1 = {S50, 0, &V24150_1_1, NULL};	/*Property = IsMobile*/
const Value V24150_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P24150_5 = {S52, 0, &V24150_5_59138, &P24150_1};	/*Property = ScreenPixelsHeight*/
const Value V24150_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P24150_4 = {S54, 0, &V24150_4_59970, &P24150_5};	/*Property = ScreenPixelsWidth*/
const Profile P24150 = {"24150", &P24150_4};	/*Profile ID - 24150*/
const Value V23615_1_1 = {S2, NULL};	/*Value = True*/
const Property P23615_1 = {S50, 0, &V23615_1_1, NULL};	/*Property = IsMobile*/
const Value V23615_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23615_5 = {S52, 0, &V23615_5_1227, &P23615_1};	/*Property = ScreenPixelsHeight*/
const Value V23615_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23615_4 = {S54, 0, &V23615_4_1229, &P23615_5};	/*Property = ScreenPixelsWidth*/
const Profile P23615 = {"23615", &P23615_4};	/*Profile ID - 23615*/
const Value V23271_1_1 = {S2, NULL};	/*Value = True*/
const Property P23271_1 = {S50, 0, &V23271_1_1, NULL};	/*Property = IsMobile*/
const Value V23271_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23271_5 = {S52, 0, &V23271_5_1227, &P23271_1};	/*Property = ScreenPixelsHeight*/
const Value V23271_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23271_4 = {S54, 0, &V23271_4_1460, &P23271_5};	/*Property = ScreenPixelsWidth*/
const Profile P23271 = {"23271", &P23271_4};	/*Profile ID - 23271*/
const Value V23410_1_1 = {S2, NULL};	/*Value = True*/
const Property P23410_1 = {S50, 0, &V23410_1_1, NULL};	/*Property = IsMobile*/
const Value V23410_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P23410_5 = {S52, 0, &V23410_5_1269, &P23410_1};	/*Property = ScreenPixelsHeight*/
const Value V23410_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23410_4 = {S54, 0, &V23410_4_1229, &P23410_5};	/*Property = ScreenPixelsWidth*/
const Profile P23410 = {"23410", &P23410_4};	/*Profile ID - 23410*/
const Value V23323_1_1 = {S2, NULL};	/*Value = True*/
const Property P23323_1 = {S50, 0, &V23323_1_1, NULL};	/*Property = IsMobile*/
const Value V23323_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23323_5 = {S52, 0, &V23323_5_1403, &P23323_1};	/*Property = ScreenPixelsHeight*/
const Value V23323_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23323_4 = {S54, 0, &V23323_4_1404, &P23323_5};	/*Property = ScreenPixelsWidth*/
const Profile P23323 = {"23323", &P23323_4};	/*Profile ID - 23323*/
const Value V23591_1_1 = {S2, NULL};	/*Value = True*/
const Property P23591_1 = {S50, 0, &V23591_1_1, NULL};	/*Property = IsMobile*/
const Value V23591_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23591_5 = {S52, 0, &V23591_5_1458, &P23591_1};	/*Property = ScreenPixelsHeight*/
const Value V23591_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23591_4 = {S54, 0, &V23591_4_1460, &P23591_5};	/*Property = ScreenPixelsWidth*/
const Profile P23591 = {"23591", &P23591_4};	/*Profile ID - 23591*/
const Value V24192_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24192_279 = {S1, 0, &V24192_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24192_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24192_282 = {S3, 0, &V24192_282_57154, &P24192_279};	/*Property = BlobBuilder*/
const Value V24192_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24192_350 = {S4, 0, &V24192_350_57444, &P24192_282};	/*Property = Canvas*/
const Value V24192_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24192_283 = {S5, 0, &V24192_283_57159, &P24192_350};	/*Property = CssBackground*/
const Value V24192_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24192_284 = {S6, 0, &V24192_284_57162, &P24192_283};	/*Property = CssBorderImage*/
const Value V24192_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24192_285 = {S7, 0, &V24192_285_57166, &P24192_284};	/*Property = CssCanvas*/
const Value V24192_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24192_286 = {S8, 0, &V24192_286_57168, &P24192_285};	/*Property = CssColor*/
const Value V24192_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24192_237 = {S9, 0, &V24192_237_55786, &P24192_286};	/*Property = CssColumn*/
const Value V24192_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24192_287 = {S10, 0, &V24192_287_57172, &P24192_237};	/*Property = CssFlexbox*/
const Value V24192_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24192_288 = {S11, 0, &V24192_288_57174, &P24192_287};	/*Property = CssFont*/
const Value V24192_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24192_289 = {S12, 0, &V24192_289_57759, &P24192_288};	/*Property = CssImages*/
const Value V24192_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24192_291 = {S13, 0, &V24192_291_57180, &P24192_289};	/*Property = CssMediaQueries*/
const Value V24192_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24192_292 = {S14, 0, &V24192_292_57183, &P24192_291};	/*Property = CssMinMax*/
const Value V24192_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24192_293 = {S15, 0, &V24192_293_57187, &P24192_292};	/*Property = CssOverflow*/
const Value V24192_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24192_294 = {S16, 0, &V24192_294_57189, &P24192_293};	/*Property = CssPosition*/
const Value V24192_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24192_295 = {S17, 0, &V24192_295_57192, &P24192_294};	/*Property = CssText*/
const Value V24192_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24192_297 = {S18, 0, &V24192_297_57195, &P24192_295};	/*Property = CssTransforms*/
const Value V24192_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24192_241 = {S19, 0, &V24192_241_55799, &P24192_297};	/*Property = CssTransitions*/
const Value V24192_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24192_298 = {S20, 0, &V24192_298_57206, &P24192_241};	/*Property = CssUI*/
const Value V24192_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24192_300 = {S21, 0, &V24192_300_57215, &P24192_298};	/*Property = DataSet*/
const Value V24192_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24192_301 = {S22, 0, &V24192_301_57218, &P24192_300};	/*Property = DataUrl*/
const Value V24192_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24192_302 = {S23, 0, &V24192_302_57222, &P24192_301};	/*Property = DeviceOrientation*/
const Value V24192_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24192_304 = {S24, 0, &V24192_304_57227, &P24192_302};	/*Property = FileReader*/
const Value V24192_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24192_305 = {S25, 0, &V24192_305_57231, &P24192_304};	/*Property = FileSaver*/
const Value V24192_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24192_306 = {S26, 0, &V24192_306_57237, &P24192_305};	/*Property = FileWriter*/
const Value V24192_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24192_307 = {S27, 0, &V24192_307_57239, &P24192_306};	/*Property = FormData*/
const Value V24192_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24192_309 = {S28, 0, &V24192_309_57246, &P24192_307};	/*Property = Fullscreen*/
const Value V24192_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24192_310 = {S29, 0, &V24192_310_57248, &P24192_309};	/*Property = GeoLocation*/
const Value V24192_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24192_312 = {S30, 0, &V24192_312_57254, &P24192_310};	/*Property = History*/
const Value V24192_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24192_314 = {S31, 0, &V24192_314_57260, &P24192_312};	/*Property = Html5*/
const Value V24192_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24192_313 = {S32, 0, &V24192_313_57257, &P24192_314};	/*Property = Html-Media-Capture*/
const Value V24192_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24192_315 = {S33, 0, &V24192_315_57264, &P24192_313};	/*Property = Iframe*/
const Value V24192_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24192_316 = {S34, 0, &V24192_316_57266, &P24192_315};	/*Property = IndexedDB*/
const Value V24192_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24192_317 = {S35, 0, &V24192_317_57269, &P24192_316};	/*Property = Json*/
const Value V24192_15_81502 = {S141, NULL};	/*Value = Blink*/
const Property P24192_15 = {S37, 0, &V24192_15_81502, &P24192_317};	/*Property = LayoutEngine*/
const Value V24192_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24192_344 = {S38, 0, &V24192_344_57272, &P24192_15};	/*Property = Masking*/
const Value V24192_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24192_320 = {S39, 0, &V24192_320_57290, &P24192_344};	/*Property = PostMessage*/
const Value V24192_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24192_321 = {S40, 0, &V24192_321_57293, &P24192_320};	/*Property = Progress*/
const Value V24192_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24192_322 = {S41, 0, &V24192_322_57296, &P24192_321};	/*Property = Prompts*/
const Value V24192_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24192_323 = {S42, 0, &V24192_323_57299, &P24192_322};	/*Property = Selector*/
const Value V24192_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24192_324 = {S43, 0, &V24192_324_57305, &P24192_323};	/*Property = Svg*/
const Value V24192_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24192_327 = {S44, 0, &V24192_327_57314, &P24192_324};	/*Property = TouchEvents*/
const Value V24192_328_57317 = {S2, NULL};	/*Value = True*/
const Property P24192_328 = {S45, 0, &V24192_328_57317, &P24192_327};	/*Property = Track*/
const Value V24192_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24192_329 = {S46, 0, &V24192_329_57323, &P24192_328};	/*Property = Video*/
const Value V24192_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24192_330 = {S47, 0, &V24192_330_57326, &P24192_329};	/*Property = Viewport*/
const Value V24192_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24192_332 = {S48, 0, &V24192_332_57338, &P24192_330};	/*Property = WebWorkers*/
const Value V24192_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24192_333 = {S49, 0, &V24192_333_57342, &P24192_332};	/*Property = Xhr2*/
const Profile P24192 = {"24192", &P24192_333};	/*Profile ID - 24192*/
const Value V23363_1_1 = {S2, NULL};	/*Value = True*/
const Property P23363_1 = {S50, 0, &V23363_1_1, NULL};	/*Property = IsMobile*/
const Value V23363_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23363_5 = {S52, 0, &V23363_5_1227, &P23363_1};	/*Property = ScreenPixelsHeight*/
const Value V23363_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23363_4 = {S54, 0, &V23363_4_1229, &P23363_5};	/*Property = ScreenPixelsWidth*/
const Profile P23363 = {"23363", &P23363_4};	/*Profile ID - 23363*/
const Value V23505_1_1 = {S2, NULL};	/*Value = True*/
const Property P23505_1 = {S50, 0, &V23505_1_1, NULL};	/*Property = IsMobile*/
const Value V23505_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23505_5 = {S52, 0, &V23505_5_1277, &P23505_1};	/*Property = ScreenPixelsHeight*/
const Value V23505_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23505_4 = {S54, 0, &V23505_4_1279, &P23505_5};	/*Property = ScreenPixelsWidth*/
const Profile P23505 = {"23505", &P23505_4};	/*Profile ID - 23505*/
const Value V23223_1_1 = {S2, NULL};	/*Value = True*/
const Property P23223_1 = {S50, 0, &V23223_1_1, NULL};	/*Property = IsMobile*/
const Value V23223_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23223_5 = {S52, 0, &V23223_5_37569, &P23223_1};	/*Property = ScreenPixelsHeight*/
const Value V23223_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P23223_4 = {S54, 0, &V23223_4_1477, &P23223_5};	/*Property = ScreenPixelsWidth*/
const Profile P23223 = {"23223", &P23223_4};	/*Profile ID - 23223*/
const Value V23327_1_1 = {S2, NULL};	/*Value = True*/
const Property P23327_1 = {S50, 0, &V23327_1_1, NULL};	/*Property = IsMobile*/
const Value V23327_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23327_5 = {S52, 0, &V23327_5_1227, &P23327_1};	/*Property = ScreenPixelsHeight*/
const Value V23327_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23327_4 = {S54, 0, &V23327_4_1229, &P23327_5};	/*Property = ScreenPixelsWidth*/
const Profile P23327 = {"23327", &P23327_4};	/*Profile ID - 23327*/
const Value V23674_1_1 = {S2, NULL};	/*Value = True*/
const Property P23674_1 = {S50, 0, &V23674_1_1, NULL};	/*Property = IsMobile*/
const Value V23674_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23674_5 = {S52, 0, &V23674_5_1832, &P23674_1};	/*Property = ScreenPixelsHeight*/
const Value V23674_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23674_4 = {S54, 0, &V23674_4_5645, &P23674_5};	/*Property = ScreenPixelsWidth*/
const Profile P23674 = {"23674", &P23674_4};	/*Profile ID - 23674*/
const Value V23278_1_1 = {S2, NULL};	/*Value = True*/
const Property P23278_1 = {S50, 0, &V23278_1_1, NULL};	/*Property = IsMobile*/
const Value V23278_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23278_5 = {S52, 0, &V23278_5_1458, &P23278_1};	/*Property = ScreenPixelsHeight*/
const Value V23278_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P23278_4 = {S54, 0, &V23278_4_37617, &P23278_5};	/*Property = ScreenPixelsWidth*/
const Profile P23278 = {"23278", &P23278_4};	/*Profile ID - 23278*/
const Value V24211_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_279 = {S1, 0, &V24211_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24211_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_282 = {S3, 0, &V24211_282_57155, &P24211_279};	/*Property = BlobBuilder*/
const Value V24211_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_350 = {S4, 0, &V24211_350_57446, &P24211_282};	/*Property = Canvas*/
const Value V24211_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_283 = {S5, 0, &V24211_283_57161, &P24211_350};	/*Property = CssBackground*/
const Value V24211_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_284 = {S6, 0, &V24211_284_57164, &P24211_283};	/*Property = CssBorderImage*/
const Value V24211_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_285 = {S7, 0, &V24211_285_57167, &P24211_284};	/*Property = CssCanvas*/
const Value V24211_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_286 = {S8, 0, &V24211_286_57170, &P24211_285};	/*Property = CssColor*/
const Value V24211_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_237 = {S9, 0, &V24211_237_55786, &P24211_286};	/*Property = CssColumn*/
const Value V24211_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_287 = {S10, 0, &V24211_287_57173, &P24211_237};	/*Property = CssFlexbox*/
const Value V24211_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_288 = {S11, 0, &V24211_288_57176, &P24211_287};	/*Property = CssFont*/
const Value V24211_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_289 = {S12, 0, &V24211_289_86899, &P24211_288};	/*Property = CssImages*/
const Value V24211_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_291 = {S13, 0, &V24211_291_57182, &P24211_289};	/*Property = CssMediaQueries*/
const Value V24211_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_292 = {S14, 0, &V24211_292_57185, &P24211_291};	/*Property = CssMinMax*/
const Value V24211_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_293 = {S15, 0, &V24211_293_57188, &P24211_292};	/*Property = CssOverflow*/
const Value V24211_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_294 = {S16, 0, &V24211_294_57191, &P24211_293};	/*Property = CssPosition*/
const Value V24211_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_295 = {S17, 0, &V24211_295_57194, &P24211_294};	/*Property = CssText*/
const Value V24211_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_297 = {S18, 0, &V24211_297_57197, &P24211_295};	/*Property = CssTransforms*/
const Value V24211_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_241 = {S19, 0, &V24211_241_55801, &P24211_297};	/*Property = CssTransitions*/
const Value V24211_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_298 = {S20, 0, &V24211_298_57208, &P24211_241};	/*Property = CssUI*/
const Value V24211_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_300 = {S21, 0, &V24211_300_57217, &P24211_298};	/*Property = DataSet*/
const Value V24211_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_301 = {S22, 0, &V24211_301_57220, &P24211_300};	/*Property = DataUrl*/
const Value V24211_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_302 = {S23, 0, &V24211_302_57223, &P24211_301};	/*Property = DeviceOrientation*/
const Value V24211_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_304 = {S24, 0, &V24211_304_57229, &P24211_302};	/*Property = FileReader*/
const Value V24211_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_305 = {S25, 0, &V24211_305_57232, &P24211_304};	/*Property = FileSaver*/
const Value V24211_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_306 = {S26, 0, &V24211_306_57238, &P24211_305};	/*Property = FileWriter*/
const Value V24211_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_307 = {S27, 0, &V24211_307_57241, &P24211_306};	/*Property = FormData*/
const Value V24211_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_309 = {S28, 0, &V24211_309_57247, &P24211_307};	/*Property = Fullscreen*/
const Value V24211_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_310 = {S29, 0, &V24211_310_57250, &P24211_309};	/*Property = GeoLocation*/
const Value V24211_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_312 = {S30, 0, &V24211_312_57256, &P24211_310};	/*Property = History*/
const Value V24211_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_314 = {S31, 0, &V24211_314_57262, &P24211_312};	/*Property = Html5*/
const Value V24211_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_313 = {S32, 0, &V24211_313_57259, &P24211_314};	/*Property = Html-Media-Capture*/
const Value V24211_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_315 = {S33, 0, &V24211_315_57265, &P24211_313};	/*Property = Iframe*/
const Value V24211_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_316 = {S34, 0, &V24211_316_57268, &P24211_315};	/*Property = IndexedDB*/
const Value V24211_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_317 = {S35, 0, &V24211_317_57271, &P24211_316};	/*Property = Json*/
const Value V24211_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P24211_15 = {S37, 0, &V24211_15_31, &P24211_317};	/*Property = LayoutEngine*/
const Value V24211_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_344 = {S38, 0, &V24211_344_57274, &P24211_15};	/*Property = Masking*/
const Value V24211_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_320 = {S39, 0, &V24211_320_57292, &P24211_344};	/*Property = PostMessage*/
const Value V24211_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_321 = {S40, 0, &V24211_321_57295, &P24211_320};	/*Property = Progress*/
const Value V24211_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_322 = {S41, 0, &V24211_322_57298, &P24211_321};	/*Property = Prompts*/
const Value V24211_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_323 = {S42, 0, &V24211_323_57301, &P24211_322};	/*Property = Selector*/
const Value V24211_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_324 = {S43, 0, &V24211_324_57307, &P24211_323};	/*Property = Svg*/
const Value V24211_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_327 = {S44, 0, &V24211_327_57316, &P24211_324};	/*Property = TouchEvents*/
const Value V24211_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_328 = {S45, 0, &V24211_328_57319, &P24211_327};	/*Property = Track*/
const Value V24211_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_329 = {S46, 0, &V24211_329_57325, &P24211_328};	/*Property = Video*/
const Value V24211_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_330 = {S47, 0, &V24211_330_57328, &P24211_329};	/*Property = Viewport*/
const Value V24211_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_332 = {S48, 0, &V24211_332_57340, &P24211_330};	/*Property = WebWorkers*/
const Value V24211_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24211_333 = {S49, 0, &V24211_333_57343, &P24211_332};	/*Property = Xhr2*/
const Profile P24211 = {"24211", &P24211_333};	/*Profile ID - 24211*/
const Value V24164_1_1 = {S2, NULL};	/*Value = True*/
const Property P24164_1 = {S50, 0, &V24164_1_1, NULL};	/*Property = IsMobile*/
const Value V24164_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24164_5 = {S52, 0, &V24164_5_1403, &P24164_1};	/*Property = ScreenPixelsHeight*/
const Value V24164_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24164_4 = {S54, 0, &V24164_4_1404, &P24164_5};	/*Property = ScreenPixelsWidth*/
const Profile P24164 = {"24164", &P24164_4};	/*Profile ID - 24164*/
const Value V23219_1_1 = {S2, NULL};	/*Value = True*/
const Property P23219_1 = {S50, 0, &V23219_1_1, NULL};	/*Property = IsMobile*/
const Value V23219_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23219_5 = {S52, 0, &V23219_5_1227, &P23219_1};	/*Property = ScreenPixelsHeight*/
const Value V23219_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P23219_4 = {S54, 0, &V23219_4_1477, &P23219_5};	/*Property = ScreenPixelsWidth*/
const Profile P23219 = {"23219", &P23219_4};	/*Profile ID - 23219*/
const Profile P23209 = {"23209", NULL};	/*Profile ID - 23209*/
const Value V23486_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_279 = {S1, 0, &V23486_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V23486_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_282 = {S3, 0, &V23486_282_57155, &P23486_279};	/*Property = BlobBuilder*/
const Value V23486_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_350 = {S4, 0, &V23486_350_57446, &P23486_282};	/*Property = Canvas*/
const Value V23486_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_283 = {S5, 0, &V23486_283_57161, &P23486_350};	/*Property = CssBackground*/
const Value V23486_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_284 = {S6, 0, &V23486_284_57164, &P23486_283};	/*Property = CssBorderImage*/
const Value V23486_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_285 = {S7, 0, &V23486_285_57167, &P23486_284};	/*Property = CssCanvas*/
const Value V23486_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_286 = {S8, 0, &V23486_286_57170, &P23486_285};	/*Property = CssColor*/
const Value V23486_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_237 = {S9, 0, &V23486_237_55786, &P23486_286};	/*Property = CssColumn*/
const Value V23486_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_287 = {S10, 0, &V23486_287_57173, &P23486_237};	/*Property = CssFlexbox*/
const Value V23486_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_288 = {S11, 0, &V23486_288_57176, &P23486_287};	/*Property = CssFont*/
const Value V23486_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_289 = {S12, 0, &V23486_289_86899, &P23486_288};	/*Property = CssImages*/
const Value V23486_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_291 = {S13, 0, &V23486_291_57182, &P23486_289};	/*Property = CssMediaQueries*/
const Value V23486_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_292 = {S14, 0, &V23486_292_57185, &P23486_291};	/*Property = CssMinMax*/
const Value V23486_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_293 = {S15, 0, &V23486_293_57188, &P23486_292};	/*Property = CssOverflow*/
const Value V23486_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_294 = {S16, 0, &V23486_294_57191, &P23486_293};	/*Property = CssPosition*/
const Value V23486_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_295 = {S17, 0, &V23486_295_57194, &P23486_294};	/*Property = CssText*/
const Value V23486_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_297 = {S18, 0, &V23486_297_57197, &P23486_295};	/*Property = CssTransforms*/
const Value V23486_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_241 = {S19, 0, &V23486_241_55801, &P23486_297};	/*Property = CssTransitions*/
const Value V23486_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_298 = {S20, 0, &V23486_298_57208, &P23486_241};	/*Property = CssUI*/
const Value V23486_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_300 = {S21, 0, &V23486_300_57217, &P23486_298};	/*Property = DataSet*/
const Value V23486_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_301 = {S22, 0, &V23486_301_57220, &P23486_300};	/*Property = DataUrl*/
const Value V23486_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_302 = {S23, 0, &V23486_302_57223, &P23486_301};	/*Property = DeviceOrientation*/
const Value V23486_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_304 = {S24, 0, &V23486_304_57229, &P23486_302};	/*Property = FileReader*/
const Value V23486_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_305 = {S25, 0, &V23486_305_57232, &P23486_304};	/*Property = FileSaver*/
const Value V23486_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_306 = {S26, 0, &V23486_306_57238, &P23486_305};	/*Property = FileWriter*/
const Value V23486_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_307 = {S27, 0, &V23486_307_57241, &P23486_306};	/*Property = FormData*/
const Value V23486_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_309 = {S28, 0, &V23486_309_57247, &P23486_307};	/*Property = Fullscreen*/
const Value V23486_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_310 = {S29, 0, &V23486_310_57250, &P23486_309};	/*Property = GeoLocation*/
const Value V23486_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_312 = {S30, 0, &V23486_312_57256, &P23486_310};	/*Property = History*/
const Value V23486_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_314 = {S31, 0, &V23486_314_57262, &P23486_312};	/*Property = Html5*/
const Value V23486_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_313 = {S32, 0, &V23486_313_57259, &P23486_314};	/*Property = Html-Media-Capture*/
const Value V23486_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_315 = {S33, 0, &V23486_315_57265, &P23486_313};	/*Property = Iframe*/
const Value V23486_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_316 = {S34, 0, &V23486_316_57268, &P23486_315};	/*Property = IndexedDB*/
const Value V23486_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_317 = {S35, 0, &V23486_317_57271, &P23486_316};	/*Property = Json*/
const Value V23486_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P23486_15 = {S37, 0, &V23486_15_29, &P23486_317};	/*Property = LayoutEngine*/
const Value V23486_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_344 = {S38, 0, &V23486_344_57274, &P23486_15};	/*Property = Masking*/
const Value V23486_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_320 = {S39, 0, &V23486_320_57292, &P23486_344};	/*Property = PostMessage*/
const Value V23486_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_321 = {S40, 0, &V23486_321_57295, &P23486_320};	/*Property = Progress*/
const Value V23486_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_322 = {S41, 0, &V23486_322_57298, &P23486_321};	/*Property = Prompts*/
const Value V23486_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_323 = {S42, 0, &V23486_323_57301, &P23486_322};	/*Property = Selector*/
const Value V23486_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_324 = {S43, 0, &V23486_324_57307, &P23486_323};	/*Property = Svg*/
const Value V23486_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_327 = {S44, 0, &V23486_327_57316, &P23486_324};	/*Property = TouchEvents*/
const Value V23486_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_328 = {S45, 0, &V23486_328_57319, &P23486_327};	/*Property = Track*/
const Value V23486_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_329 = {S46, 0, &V23486_329_57325, &P23486_328};	/*Property = Video*/
const Value V23486_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_330 = {S47, 0, &V23486_330_57328, &P23486_329};	/*Property = Viewport*/
const Value V23486_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_332 = {S48, 0, &V23486_332_57340, &P23486_330};	/*Property = WebWorkers*/
const Value V23486_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P23486_333 = {S49, 0, &V23486_333_57343, &P23486_332};	/*Property = Xhr2*/
const Profile P23486 = {"23486", &P23486_333};	/*Profile ID - 23486*/
const Value V23798_1_1 = {S2, NULL};	/*Value = True*/
const Property P23798_1 = {S50, 0, &V23798_1_1, NULL};	/*Property = IsMobile*/
const Value V23798_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23798_5 = {S52, 0, &V23798_5_1277, &P23798_1};	/*Property = ScreenPixelsHeight*/
const Value V23798_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23798_4 = {S54, 0, &V23798_4_1279, &P23798_5};	/*Property = ScreenPixelsWidth*/
const Profile P23798 = {"23798", &P23798_4};	/*Profile ID - 23798*/
const Value V23211_1_1 = {S2, NULL};	/*Value = True*/
const Property P23211_1 = {S50, 0, &V23211_1_1, NULL};	/*Property = IsMobile*/
const Value V23211_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P23211_5 = {S52, 0, &V23211_5_59138, &P23211_1};	/*Property = ScreenPixelsHeight*/
const Value V23211_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P23211_4 = {S54, 0, &V23211_4_59970, &P23211_5};	/*Property = ScreenPixelsWidth*/
const Profile P23211 = {"23211", &P23211_4};	/*Profile ID - 23211*/
const Value V23213_1_1 = {S2, NULL};	/*Value = True*/
const Property P23213_1 = {S50, 0, &V23213_1_1, NULL};	/*Property = IsMobile*/
const Value V23213_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P23213_5 = {S52, 0, &V23213_5_59138, &P23213_1};	/*Property = ScreenPixelsHeight*/
const Value V23213_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P23213_4 = {S54, 0, &V23213_4_59970, &P23213_5};	/*Property = ScreenPixelsWidth*/
const Profile P23213 = {"23213", &P23213_4};	/*Profile ID - 23213*/
const Value V23214_1_1 = {S2, NULL};	/*Value = True*/
const Property P23214_1 = {S50, 0, &V23214_1_1, NULL};	/*Property = IsMobile*/
const Value V23214_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P23214_5 = {S52, 0, &V23214_5_59138, &P23214_1};	/*Property = ScreenPixelsHeight*/
const Value V23214_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P23214_4 = {S54, 0, &V23214_4_59970, &P23214_5};	/*Property = ScreenPixelsWidth*/
const Profile P23214 = {"23214", &P23214_4};	/*Profile ID - 23214*/
const Value V23215_1_1 = {S2, NULL};	/*Value = True*/
const Property P23215_1 = {S50, 0, &V23215_1_1, NULL};	/*Property = IsMobile*/
const Value V23215_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P23215_5 = {S52, 0, &V23215_5_59138, &P23215_1};	/*Property = ScreenPixelsHeight*/
const Value V23215_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P23215_4 = {S54, 0, &V23215_4_59970, &P23215_5};	/*Property = ScreenPixelsWidth*/
const Profile P23215 = {"23215", &P23215_4};	/*Profile ID - 23215*/
const Value V12845_1_1 = {S2, NULL};	/*Value = True*/
const Property P12845_1 = {S50, 0, &V12845_1_1, NULL};	/*Property = IsMobile*/
const Value V12845_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P12845_5 = {S52, 0, &V12845_5_1304, &P12845_1};	/*Property = ScreenPixelsHeight*/
const Value V12845_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12845_4 = {S54, 0, &V12845_4_1224, &P12845_5};	/*Property = ScreenPixelsWidth*/
const Profile P12845 = {"12845", &P12845_4};	/*Profile ID - 12845*/
const Value V23700_1_1 = {S2, NULL};	/*Value = True*/
const Property P23700_1 = {S50, 0, &V23700_1_1, NULL};	/*Property = IsMobile*/
const Value V23700_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23700_5 = {S52, 0, &V23700_5_1227, &P23700_1};	/*Property = ScreenPixelsHeight*/
const Value V23700_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23700_4 = {S54, 0, &V23700_4_1229, &P23700_5};	/*Property = ScreenPixelsWidth*/
const Profile P23700 = {"23700", &P23700_4};	/*Profile ID - 23700*/
const Value V23683_1_1 = {S2, NULL};	/*Value = True*/
const Property P23683_1 = {S50, 0, &V23683_1_1, NULL};	/*Property = IsMobile*/
const Value V23683_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23683_5 = {S52, 0, &V23683_5_1277, &P23683_1};	/*Property = ScreenPixelsHeight*/
const Value V23683_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23683_4 = {S54, 0, &V23683_4_1279, &P23683_5};	/*Property = ScreenPixelsWidth*/
const Profile P23683 = {"23683", &P23683_4};	/*Profile ID - 23683*/
const Value V23553_1_1 = {S2, NULL};	/*Value = True*/
const Property P23553_1 = {S50, 0, &V23553_1_1, NULL};	/*Property = IsMobile*/
const Value V23553_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23553_5 = {S52, 0, &V23553_5_9981, &P23553_1};	/*Property = ScreenPixelsHeight*/
const Value V23553_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23553_4 = {S54, 0, &V23553_4_39098, &P23553_5};	/*Property = ScreenPixelsWidth*/
const Profile P23553 = {"23553", &P23553_4};	/*Profile ID - 23553*/
const Value V24161_1_1 = {S2, NULL};	/*Value = True*/
const Property P24161_1 = {S50, 0, &V24161_1_1, NULL};	/*Property = IsMobile*/
const Value V24161_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24161_5 = {S52, 0, &V24161_5_1277, &P24161_1};	/*Property = ScreenPixelsHeight*/
const Value V24161_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24161_4 = {S54, 0, &V24161_4_1279, &P24161_5};	/*Property = ScreenPixelsWidth*/
const Profile P24161 = {"24161", &P24161_4};	/*Profile ID - 24161*/
const Value V23497_1_1 = {S2, NULL};	/*Value = True*/
const Property P23497_1 = {S50, 0, &V23497_1_1, NULL};	/*Property = IsMobile*/
const Value V23497_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P23497_5 = {S52, 0, &V23497_5_59138, &P23497_1};	/*Property = ScreenPixelsHeight*/
const Value V23497_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P23497_4 = {S54, 0, &V23497_4_59970, &P23497_5};	/*Property = ScreenPixelsWidth*/
const Profile P23497 = {"23497", &P23497_4};	/*Profile ID - 23497*/
const Value V23801_1_1 = {S2, NULL};	/*Value = True*/
const Property P23801_1 = {S50, 0, &V23801_1_1, NULL};	/*Property = IsMobile*/
const Value V23801_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23801_5 = {S52, 0, &V23801_5_1277, &P23801_1};	/*Property = ScreenPixelsHeight*/
const Value V23801_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23801_4 = {S54, 0, &V23801_4_2336, &P23801_5};	/*Property = ScreenPixelsWidth*/
const Profile P23801 = {"23801", &P23801_4};	/*Profile ID - 23801*/
const Value V23483_279_57147 = {S2, NULL};	/*Value = True*/
const Property P23483_279 = {S1, 0, &V23483_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V23483_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23483_282 = {S3, 0, &V23483_282_57154, &P23483_279};	/*Property = BlobBuilder*/
const Value V23483_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23483_350 = {S4, 0, &V23483_350_57444, &P23483_282};	/*Property = Canvas*/
const Value V23483_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23483_283 = {S5, 0, &V23483_283_57159, &P23483_350};	/*Property = CssBackground*/
const Value V23483_284_57162 = {S2, NULL};	/*Value = True*/
const Property P23483_284 = {S6, 0, &V23483_284_57162, &P23483_283};	/*Property = CssBorderImage*/
const Value V23483_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23483_285 = {S7, 0, &V23483_285_57166, &P23483_284};	/*Property = CssCanvas*/
const Value V23483_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23483_286 = {S8, 0, &V23483_286_57168, &P23483_285};	/*Property = CssColor*/
const Value V23483_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23483_237 = {S9, 0, &V23483_237_55786, &P23483_286};	/*Property = CssColumn*/
const Value V23483_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23483_287 = {S10, 0, &V23483_287_57172, &P23483_237};	/*Property = CssFlexbox*/
const Value V23483_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23483_288 = {S11, 0, &V23483_288_57174, &P23483_287};	/*Property = CssFont*/
const Value V23483_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23483_289 = {S12, 0, &V23483_289_57759, &P23483_288};	/*Property = CssImages*/
const Value V23483_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23483_291 = {S13, 0, &V23483_291_57180, &P23483_289};	/*Property = CssMediaQueries*/
const Value V23483_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23483_292 = {S14, 0, &V23483_292_57183, &P23483_291};	/*Property = CssMinMax*/
const Value V23483_293_57187 = {S0, NULL};	/*Value = False*/
const Property P23483_293 = {S15, 0, &V23483_293_57187, &P23483_292};	/*Property = CssOverflow*/
const Value V23483_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23483_294 = {S16, 0, &V23483_294_57189, &P23483_293};	/*Property = CssPosition*/
const Value V23483_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23483_295 = {S17, 0, &V23483_295_57192, &P23483_294};	/*Property = CssText*/
const Value V23483_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23483_297 = {S18, 0, &V23483_297_57195, &P23483_295};	/*Property = CssTransforms*/
const Value V23483_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23483_241 = {S19, 0, &V23483_241_55799, &P23483_297};	/*Property = CssTransitions*/
const Value V23483_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23483_298 = {S20, 0, &V23483_298_57206, &P23483_241};	/*Property = CssUI*/
const Value V23483_300_57215 = {S2, NULL};	/*Value = True*/
const Property P23483_300 = {S21, 0, &V23483_300_57215, &P23483_298};	/*Property = DataSet*/
const Value V23483_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23483_301 = {S22, 0, &V23483_301_57218, &P23483_300};	/*Property = DataUrl*/
const Value V23483_302_57222 = {S0, NULL};	/*Value = False*/
const Property P23483_302 = {S23, 0, &V23483_302_57222, &P23483_301};	/*Property = DeviceOrientation*/
const Value V23483_304_57227 = {S2, NULL};	/*Value = True*/
const Property P23483_304 = {S24, 0, &V23483_304_57227, &P23483_302};	/*Property = FileReader*/
const Value V23483_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23483_305 = {S25, 0, &V23483_305_57231, &P23483_304};	/*Property = FileSaver*/
const Value V23483_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23483_306 = {S26, 0, &V23483_306_57237, &P23483_305};	/*Property = FileWriter*/
const Value V23483_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23483_307 = {S27, 0, &V23483_307_57239, &P23483_306};	/*Property = FormData*/
const Value V23483_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23483_309 = {S28, 0, &V23483_309_57246, &P23483_307};	/*Property = Fullscreen*/
const Value V23483_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23483_310 = {S29, 0, &V23483_310_57248, &P23483_309};	/*Property = GeoLocation*/
const Value V23483_312_57254 = {S2, NULL};	/*Value = True*/
const Property P23483_312 = {S30, 0, &V23483_312_57254, &P23483_310};	/*Property = History*/
const Value V23483_314_57260 = {S2, NULL};	/*Value = True*/
const Property P23483_314 = {S31, 0, &V23483_314_57260, &P23483_312};	/*Property = Html5*/
const Value V23483_313_57257 = {S2, NULL};	/*Value = True*/
const Property P23483_313 = {S32, 0, &V23483_313_57257, &P23483_314};	/*Property = Html-Media-Capture*/
const Value V23483_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23483_315 = {S33, 0, &V23483_315_57264, &P23483_313};	/*Property = Iframe*/
const Value V23483_316_57266 = {S2, NULL};	/*Value = True*/
const Property P23483_316 = {S34, 0, &V23483_316_57266, &P23483_315};	/*Property = IndexedDB*/
const Value V23483_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23483_317 = {S35, 0, &V23483_317_57269, &P23483_316};	/*Property = Json*/
const Value V23483_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P23483_15 = {S37, 0, &V23483_15_28, &P23483_317};	/*Property = LayoutEngine*/
const Value V23483_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23483_344 = {S38, 0, &V23483_344_57272, &P23483_15};	/*Property = Masking*/
const Value V23483_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23483_320 = {S39, 0, &V23483_320_57290, &P23483_344};	/*Property = PostMessage*/
const Value V23483_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23483_321 = {S40, 0, &V23483_321_57293, &P23483_320};	/*Property = Progress*/
const Value V23483_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23483_322 = {S41, 0, &V23483_322_57296, &P23483_321};	/*Property = Prompts*/
const Value V23483_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23483_323 = {S42, 0, &V23483_323_57299, &P23483_322};	/*Property = Selector*/
const Value V23483_324_57305 = {S2, NULL};	/*Value = True*/
const Property P23483_324 = {S43, 0, &V23483_324_57305, &P23483_323};	/*Property = Svg*/
const Value V23483_327_57314 = {S2, NULL};	/*Value = True*/
const Property P23483_327 = {S44, 0, &V23483_327_57314, &P23483_324};	/*Property = TouchEvents*/
const Value V23483_328_57317 = {S2, NULL};	/*Value = True*/
const Property P23483_328 = {S45, 0, &V23483_328_57317, &P23483_327};	/*Property = Track*/
const Value V23483_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23483_329 = {S46, 0, &V23483_329_57323, &P23483_328};	/*Property = Video*/
const Value V23483_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23483_330 = {S47, 0, &V23483_330_57326, &P23483_329};	/*Property = Viewport*/
const Value V23483_332_57338 = {S2, NULL};	/*Value = True*/
const Property P23483_332 = {S48, 0, &V23483_332_57338, &P23483_330};	/*Property = WebWorkers*/
const Value V23483_333_57341 = {S2, NULL};	/*Value = True*/
const Property P23483_333 = {S49, 0, &V23483_333_57341, &P23483_332};	/*Property = Xhr2*/
const Profile P23483 = {"23483", &P23483_333};	/*Profile ID - 23483*/
const Value V23527_1_1 = {S2, NULL};	/*Value = True*/
const Property P23527_1 = {S50, 0, &V23527_1_1, NULL};	/*Property = IsMobile*/
const Value V23527_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23527_5 = {S52, 0, &V23527_5_1222, &P23527_1};	/*Property = ScreenPixelsHeight*/
const Value V23527_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23527_4 = {S54, 0, &V23527_4_1224, &P23527_5};	/*Property = ScreenPixelsWidth*/
const Profile P23527 = {"23527", &P23527_4};	/*Profile ID - 23527*/
const Value V23521_1_1 = {S2, NULL};	/*Value = True*/
const Property P23521_1 = {S50, 0, &V23521_1_1, NULL};	/*Property = IsMobile*/
const Value V23521_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23521_5 = {S52, 0, &V23521_5_1403, &P23521_1};	/*Property = ScreenPixelsHeight*/
const Value V23521_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23521_4 = {S54, 0, &V23521_4_1404, &P23521_5};	/*Property = ScreenPixelsWidth*/
const Profile P23521 = {"23521", &P23521_4};	/*Profile ID - 23521*/
const Value V23646_1_1 = {S2, NULL};	/*Value = True*/
const Property P23646_1 = {S50, 0, &V23646_1_1, NULL};	/*Property = IsMobile*/
const Value V23646_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23646_5 = {S52, 0, &V23646_5_1277, &P23646_1};	/*Property = ScreenPixelsHeight*/
const Value V23646_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23646_4 = {S54, 0, &V23646_4_2336, &P23646_5};	/*Property = ScreenPixelsWidth*/
const Profile P23646 = {"23646", &P23646_4};	/*Profile ID - 23646*/
const Value V24078_1_1 = {S2, NULL};	/*Value = True*/
const Property P24078_1 = {S50, 0, &V24078_1_1, NULL};	/*Property = IsMobile*/
const Value V24078_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24078_5 = {S52, 0, &V24078_5_9981, &P24078_1};	/*Property = ScreenPixelsHeight*/
const Value V24078_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24078_4 = {S54, 0, &V24078_4_39098, &P24078_5};	/*Property = ScreenPixelsWidth*/
const Profile P24078 = {"24078", &P24078_4};	/*Profile ID - 24078*/
const Value V23755_1_1 = {S2, NULL};	/*Value = True*/
const Property P23755_1 = {S50, 0, &V23755_1_1, NULL};	/*Property = IsMobile*/
const Value V23755_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23755_5 = {S52, 0, &V23755_5_9981, &P23755_1};	/*Property = ScreenPixelsHeight*/
const Value V23755_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23755_4 = {S54, 0, &V23755_4_2336, &P23755_5};	/*Property = ScreenPixelsWidth*/
const Profile P23755 = {"23755", &P23755_4};	/*Profile ID - 23755*/
const Value V23575_1_1 = {S2, NULL};	/*Value = True*/
const Property P23575_1 = {S50, 0, &V23575_1_1, NULL};	/*Property = IsMobile*/
const Value V23575_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23575_5 = {S52, 0, &V23575_5_1227, &P23575_1};	/*Property = ScreenPixelsHeight*/
const Value V23575_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23575_4 = {S54, 0, &V23575_4_1229, &P23575_5};	/*Property = ScreenPixelsWidth*/
const Profile P23575 = {"23575", &P23575_4};	/*Profile ID - 23575*/
const Value V23672_1_1 = {S2, NULL};	/*Value = True*/
const Property P23672_1 = {S50, 0, &V23672_1_1, NULL};	/*Property = IsMobile*/
const Value V23672_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23672_5 = {S52, 0, &V23672_5_1277, &P23672_1};	/*Property = ScreenPixelsHeight*/
const Value V23672_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23672_4 = {S54, 0, &V23672_4_1279, &P23672_5};	/*Property = ScreenPixelsWidth*/
const Profile P23672 = {"23672", &P23672_4};	/*Profile ID - 23672*/
const Value V23641_1_1 = {S2, NULL};	/*Value = True*/
const Property P23641_1 = {S50, 0, &V23641_1_1, NULL};	/*Property = IsMobile*/
const Value V23641_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P23641_5 = {S52, 0, &V23641_5_1832, &P23641_1};	/*Property = ScreenPixelsHeight*/
const Value V23641_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23641_4 = {S54, 0, &V23641_4_5645, &P23641_5};	/*Property = ScreenPixelsWidth*/
const Profile P23641 = {"23641", &P23641_4};	/*Profile ID - 23641*/
const Value V23522_1_1 = {S2, NULL};	/*Value = True*/
const Property P23522_1 = {S50, 0, &V23522_1_1, NULL};	/*Property = IsMobile*/
const Value V23522_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23522_5 = {S52, 0, &V23522_5_1222, &P23522_1};	/*Property = ScreenPixelsHeight*/
const Value V23522_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23522_4 = {S54, 0, &V23522_4_1224, &P23522_5};	/*Property = ScreenPixelsWidth*/
const Profile P23522 = {"23522", &P23522_4};	/*Profile ID - 23522*/
const Value V24335_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_279 = {S1, 0, &V24335_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24335_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_282 = {S3, 0, &V24335_282_57155, &P24335_279};	/*Property = BlobBuilder*/
const Value V24335_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_350 = {S4, 0, &V24335_350_57446, &P24335_282};	/*Property = Canvas*/
const Value V24335_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_283 = {S5, 0, &V24335_283_57161, &P24335_350};	/*Property = CssBackground*/
const Value V24335_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_284 = {S6, 0, &V24335_284_57164, &P24335_283};	/*Property = CssBorderImage*/
const Value V24335_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_285 = {S7, 0, &V24335_285_57167, &P24335_284};	/*Property = CssCanvas*/
const Value V24335_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_286 = {S8, 0, &V24335_286_57170, &P24335_285};	/*Property = CssColor*/
const Value V24335_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_237 = {S9, 0, &V24335_237_55786, &P24335_286};	/*Property = CssColumn*/
const Value V24335_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_287 = {S10, 0, &V24335_287_57173, &P24335_237};	/*Property = CssFlexbox*/
const Value V24335_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_288 = {S11, 0, &V24335_288_57176, &P24335_287};	/*Property = CssFont*/
const Value V24335_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_289 = {S12, 0, &V24335_289_86899, &P24335_288};	/*Property = CssImages*/
const Value V24335_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_291 = {S13, 0, &V24335_291_57182, &P24335_289};	/*Property = CssMediaQueries*/
const Value V24335_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_292 = {S14, 0, &V24335_292_57185, &P24335_291};	/*Property = CssMinMax*/
const Value V24335_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_293 = {S15, 0, &V24335_293_57188, &P24335_292};	/*Property = CssOverflow*/
const Value V24335_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_294 = {S16, 0, &V24335_294_57191, &P24335_293};	/*Property = CssPosition*/
const Value V24335_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_295 = {S17, 0, &V24335_295_57194, &P24335_294};	/*Property = CssText*/
const Value V24335_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_297 = {S18, 0, &V24335_297_57197, &P24335_295};	/*Property = CssTransforms*/
const Value V24335_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_241 = {S19, 0, &V24335_241_55801, &P24335_297};	/*Property = CssTransitions*/
const Value V24335_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_298 = {S20, 0, &V24335_298_57208, &P24335_241};	/*Property = CssUI*/
const Value V24335_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_300 = {S21, 0, &V24335_300_57217, &P24335_298};	/*Property = DataSet*/
const Value V24335_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_301 = {S22, 0, &V24335_301_57220, &P24335_300};	/*Property = DataUrl*/
const Value V24335_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_302 = {S23, 0, &V24335_302_57223, &P24335_301};	/*Property = DeviceOrientation*/
const Value V24335_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_304 = {S24, 0, &V24335_304_57229, &P24335_302};	/*Property = FileReader*/
const Value V24335_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_305 = {S25, 0, &V24335_305_57232, &P24335_304};	/*Property = FileSaver*/
const Value V24335_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_306 = {S26, 0, &V24335_306_57238, &P24335_305};	/*Property = FileWriter*/
const Value V24335_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_307 = {S27, 0, &V24335_307_57241, &P24335_306};	/*Property = FormData*/
const Value V24335_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_309 = {S28, 0, &V24335_309_57247, &P24335_307};	/*Property = Fullscreen*/
const Value V24335_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_310 = {S29, 0, &V24335_310_57250, &P24335_309};	/*Property = GeoLocation*/
const Value V24335_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_312 = {S30, 0, &V24335_312_57256, &P24335_310};	/*Property = History*/
const Value V24335_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_314 = {S31, 0, &V24335_314_57262, &P24335_312};	/*Property = Html5*/
const Value V24335_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_313 = {S32, 0, &V24335_313_57259, &P24335_314};	/*Property = Html-Media-Capture*/
const Value V24335_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_315 = {S33, 0, &V24335_315_57265, &P24335_313};	/*Property = Iframe*/
const Value V24335_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_316 = {S34, 0, &V24335_316_57268, &P24335_315};	/*Property = IndexedDB*/
const Value V24335_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_317 = {S35, 0, &V24335_317_57271, &P24335_316};	/*Property = Json*/
const Value V24335_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P24335_15 = {S37, 0, &V24335_15_29, &P24335_317};	/*Property = LayoutEngine*/
const Value V24335_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_344 = {S38, 0, &V24335_344_57274, &P24335_15};	/*Property = Masking*/
const Value V24335_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_320 = {S39, 0, &V24335_320_57292, &P24335_344};	/*Property = PostMessage*/
const Value V24335_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_321 = {S40, 0, &V24335_321_57295, &P24335_320};	/*Property = Progress*/
const Value V24335_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_322 = {S41, 0, &V24335_322_57298, &P24335_321};	/*Property = Prompts*/
const Value V24335_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_323 = {S42, 0, &V24335_323_57301, &P24335_322};	/*Property = Selector*/
const Value V24335_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_324 = {S43, 0, &V24335_324_57307, &P24335_323};	/*Property = Svg*/
const Value V24335_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_327 = {S44, 0, &V24335_327_57316, &P24335_324};	/*Property = TouchEvents*/
const Value V24335_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_328 = {S45, 0, &V24335_328_57319, &P24335_327};	/*Property = Track*/
const Value V24335_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_329 = {S46, 0, &V24335_329_57325, &P24335_328};	/*Property = Video*/
const Value V24335_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_330 = {S47, 0, &V24335_330_57328, &P24335_329};	/*Property = Viewport*/
const Value V24335_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_332 = {S48, 0, &V24335_332_57340, &P24335_330};	/*Property = WebWorkers*/
const Value V24335_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24335_333 = {S49, 0, &V24335_333_57343, &P24335_332};	/*Property = Xhr2*/
const Profile P24335 = {"24335", &P24335_333};	/*Profile ID - 24335*/
const Value V24570_1_1 = {S2, NULL};	/*Value = True*/
const Property P24570_1 = {S50, 0, &V24570_1_1, NULL};	/*Property = IsMobile*/
const Value V24570_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24570_5 = {S52, 0, &V24570_5_1403, &P24570_1};	/*Property = ScreenPixelsHeight*/
const Value V24570_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24570_4 = {S54, 0, &V24570_4_1404, &P24570_5};	/*Property = ScreenPixelsWidth*/
const Profile P24570 = {"24570", &P24570_4};	/*Profile ID - 24570*/
const Value V24170_1_1 = {S2, NULL};	/*Value = True*/
const Property P24170_1 = {S50, 0, &V24170_1_1, NULL};	/*Property = IsMobile*/
const Value V24170_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P24170_5 = {S52, 0, &V24170_5_1832, &P24170_1};	/*Property = ScreenPixelsHeight*/
const Value V24170_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P24170_4 = {S54, 0, &V24170_4_5645, &P24170_5};	/*Property = ScreenPixelsWidth*/
const Profile P24170 = {"24170", &P24170_4};	/*Profile ID - 24170*/
const Value V24069_1_1 = {S2, NULL};	/*Value = True*/
const Property P24069_1 = {S50, 0, &V24069_1_1, NULL};	/*Property = IsMobile*/
const Value V24069_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24069_5 = {S52, 0, &V24069_5_1227, &P24069_1};	/*Property = ScreenPixelsHeight*/
const Value V24069_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24069_4 = {S54, 0, &V24069_4_1229, &P24069_5};	/*Property = ScreenPixelsWidth*/
const Profile P24069 = {"24069", &P24069_4};	/*Profile ID - 24069*/
const Value V23504_1_1 = {S2, NULL};	/*Value = True*/
const Property P23504_1 = {S50, 0, &V23504_1_1, NULL};	/*Property = IsMobile*/
const Value V23504_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P23504_5 = {S52, 0, &V23504_5_59138, &P23504_1};	/*Property = ScreenPixelsHeight*/
const Value V23504_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P23504_4 = {S54, 0, &V23504_4_59970, &P23504_5};	/*Property = ScreenPixelsWidth*/
const Profile P23504 = {"23504", &P23504_4};	/*Profile ID - 23504*/
const Value V23691_1_1 = {S2, NULL};	/*Value = True*/
const Property P23691_1 = {S50, 0, &V23691_1_1, NULL};	/*Property = IsMobile*/
const Value V23691_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23691_5 = {S52, 0, &V23691_5_1222, &P23691_1};	/*Property = ScreenPixelsHeight*/
const Value V23691_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23691_4 = {S54, 0, &V23691_4_1224, &P23691_5};	/*Property = ScreenPixelsWidth*/
const Profile P23691 = {"23691", &P23691_4};	/*Profile ID - 23691*/
const Value V23494_1_1 = {S2, NULL};	/*Value = True*/
const Property P23494_1 = {S50, 0, &V23494_1_1, NULL};	/*Property = IsMobile*/
const Value V23494_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23494_5 = {S52, 0, &V23494_5_1227, &P23494_1};	/*Property = ScreenPixelsHeight*/
const Value V23494_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23494_4 = {S54, 0, &V23494_4_1229, &P23494_5};	/*Property = ScreenPixelsWidth*/
const Profile P23494 = {"23494", &P23494_4};	/*Profile ID - 23494*/
const Value V24678_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_279 = {S1, 0, &V24678_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24678_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_282 = {S3, 0, &V24678_282_57155, &P24678_279};	/*Property = BlobBuilder*/
const Value V24678_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_350 = {S4, 0, &V24678_350_57446, &P24678_282};	/*Property = Canvas*/
const Value V24678_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_283 = {S5, 0, &V24678_283_57161, &P24678_350};	/*Property = CssBackground*/
const Value V24678_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_284 = {S6, 0, &V24678_284_57164, &P24678_283};	/*Property = CssBorderImage*/
const Value V24678_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_285 = {S7, 0, &V24678_285_57167, &P24678_284};	/*Property = CssCanvas*/
const Value V24678_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_286 = {S8, 0, &V24678_286_57170, &P24678_285};	/*Property = CssColor*/
const Value V24678_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_237 = {S9, 0, &V24678_237_55786, &P24678_286};	/*Property = CssColumn*/
const Value V24678_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_287 = {S10, 0, &V24678_287_57173, &P24678_237};	/*Property = CssFlexbox*/
const Value V24678_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_288 = {S11, 0, &V24678_288_57176, &P24678_287};	/*Property = CssFont*/
const Value V24678_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_289 = {S12, 0, &V24678_289_86899, &P24678_288};	/*Property = CssImages*/
const Value V24678_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_291 = {S13, 0, &V24678_291_57182, &P24678_289};	/*Property = CssMediaQueries*/
const Value V24678_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_292 = {S14, 0, &V24678_292_57185, &P24678_291};	/*Property = CssMinMax*/
const Value V24678_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_293 = {S15, 0, &V24678_293_57188, &P24678_292};	/*Property = CssOverflow*/
const Value V24678_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_294 = {S16, 0, &V24678_294_57191, &P24678_293};	/*Property = CssPosition*/
const Value V24678_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_295 = {S17, 0, &V24678_295_57194, &P24678_294};	/*Property = CssText*/
const Value V24678_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_297 = {S18, 0, &V24678_297_57197, &P24678_295};	/*Property = CssTransforms*/
const Value V24678_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_241 = {S19, 0, &V24678_241_55801, &P24678_297};	/*Property = CssTransitions*/
const Value V24678_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_298 = {S20, 0, &V24678_298_57208, &P24678_241};	/*Property = CssUI*/
const Value V24678_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_300 = {S21, 0, &V24678_300_57217, &P24678_298};	/*Property = DataSet*/
const Value V24678_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_301 = {S22, 0, &V24678_301_57220, &P24678_300};	/*Property = DataUrl*/
const Value V24678_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_302 = {S23, 0, &V24678_302_57223, &P24678_301};	/*Property = DeviceOrientation*/
const Value V24678_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_304 = {S24, 0, &V24678_304_57229, &P24678_302};	/*Property = FileReader*/
const Value V24678_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_305 = {S25, 0, &V24678_305_57232, &P24678_304};	/*Property = FileSaver*/
const Value V24678_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_306 = {S26, 0, &V24678_306_57238, &P24678_305};	/*Property = FileWriter*/
const Value V24678_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_307 = {S27, 0, &V24678_307_57241, &P24678_306};	/*Property = FormData*/
const Value V24678_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_309 = {S28, 0, &V24678_309_57247, &P24678_307};	/*Property = Fullscreen*/
const Value V24678_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_310 = {S29, 0, &V24678_310_57250, &P24678_309};	/*Property = GeoLocation*/
const Value V24678_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_312 = {S30, 0, &V24678_312_57256, &P24678_310};	/*Property = History*/
const Value V24678_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_314 = {S31, 0, &V24678_314_57262, &P24678_312};	/*Property = Html5*/
const Value V24678_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_313 = {S32, 0, &V24678_313_57259, &P24678_314};	/*Property = Html-Media-Capture*/
const Value V24678_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_315 = {S33, 0, &V24678_315_57265, &P24678_313};	/*Property = Iframe*/
const Value V24678_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_316 = {S34, 0, &V24678_316_57268, &P24678_315};	/*Property = IndexedDB*/
const Value V24678_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_317 = {S35, 0, &V24678_317_57271, &P24678_316};	/*Property = Json*/
const Value V24678_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24678_15 = {S37, 0, &V24678_15_90837, &P24678_317};	/*Property = LayoutEngine*/
const Value V24678_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_344 = {S38, 0, &V24678_344_57274, &P24678_15};	/*Property = Masking*/
const Value V24678_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_320 = {S39, 0, &V24678_320_57292, &P24678_344};	/*Property = PostMessage*/
const Value V24678_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_321 = {S40, 0, &V24678_321_57295, &P24678_320};	/*Property = Progress*/
const Value V24678_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_322 = {S41, 0, &V24678_322_57298, &P24678_321};	/*Property = Prompts*/
const Value V24678_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_323 = {S42, 0, &V24678_323_57301, &P24678_322};	/*Property = Selector*/
const Value V24678_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_324 = {S43, 0, &V24678_324_57307, &P24678_323};	/*Property = Svg*/
const Value V24678_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_327 = {S44, 0, &V24678_327_57316, &P24678_324};	/*Property = TouchEvents*/
const Value V24678_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_328 = {S45, 0, &V24678_328_57319, &P24678_327};	/*Property = Track*/
const Value V24678_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_329 = {S46, 0, &V24678_329_57325, &P24678_328};	/*Property = Video*/
const Value V24678_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_330 = {S47, 0, &V24678_330_57328, &P24678_329};	/*Property = Viewport*/
const Value V24678_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_332 = {S48, 0, &V24678_332_57340, &P24678_330};	/*Property = WebWorkers*/
const Value V24678_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24678_333 = {S49, 0, &V24678_333_57343, &P24678_332};	/*Property = Xhr2*/
const Profile P24678 = {"24678", &P24678_333};	/*Profile ID - 24678*/
const Value V23692_1_1 = {S2, NULL};	/*Value = True*/
const Property P23692_1 = {S50, 0, &V23692_1_1, NULL};	/*Property = IsMobile*/
const Value V23692_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23692_5 = {S52, 0, &V23692_5_1277, &P23692_1};	/*Property = ScreenPixelsHeight*/
const Value V23692_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23692_4 = {S54, 0, &V23692_4_1279, &P23692_5};	/*Property = ScreenPixelsWidth*/
const Profile P23692 = {"23692", &P23692_4};	/*Profile ID - 23692*/
const Value V23693_1_1 = {S2, NULL};	/*Value = True*/
const Property P23693_1 = {S50, 0, &V23693_1_1, NULL};	/*Property = IsMobile*/
const Value V23693_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P23693_5 = {S52, 0, &V23693_5_37569, &P23693_1};	/*Property = ScreenPixelsHeight*/
const Value V23693_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P23693_4 = {S54, 0, &V23693_4_5645, &P23693_5};	/*Property = ScreenPixelsWidth*/
const Profile P23693 = {"23693", &P23693_4};	/*Profile ID - 23693*/
const Value V23495_1_1 = {S2, NULL};	/*Value = True*/
const Property P23495_1 = {S50, 0, &V23495_1_1, NULL};	/*Property = IsMobile*/
const Value V23495_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23495_5 = {S52, 0, &V23495_5_1227, &P23495_1};	/*Property = ScreenPixelsHeight*/
const Value V23495_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23495_4 = {S54, 0, &V23495_4_1229, &P23495_5};	/*Property = ScreenPixelsWidth*/
const Profile P23495 = {"23495", &P23495_4};	/*Profile ID - 23495*/
const Value V23482_279_57147 = {S2, NULL};	/*Value = True*/
const Property P23482_279 = {S1, 0, &V23482_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V23482_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23482_282 = {S3, 0, &V23482_282_57154, &P23482_279};	/*Property = BlobBuilder*/
const Value V23482_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23482_350 = {S4, 0, &V23482_350_57444, &P23482_282};	/*Property = Canvas*/
const Value V23482_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23482_283 = {S5, 0, &V23482_283_57159, &P23482_350};	/*Property = CssBackground*/
const Value V23482_284_57162 = {S2, NULL};	/*Value = True*/
const Property P23482_284 = {S6, 0, &V23482_284_57162, &P23482_283};	/*Property = CssBorderImage*/
const Value V23482_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23482_285 = {S7, 0, &V23482_285_57166, &P23482_284};	/*Property = CssCanvas*/
const Value V23482_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23482_286 = {S8, 0, &V23482_286_57168, &P23482_285};	/*Property = CssColor*/
const Value V23482_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23482_237 = {S9, 0, &V23482_237_55786, &P23482_286};	/*Property = CssColumn*/
const Value V23482_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23482_287 = {S10, 0, &V23482_287_57172, &P23482_237};	/*Property = CssFlexbox*/
const Value V23482_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23482_288 = {S11, 0, &V23482_288_57174, &P23482_287};	/*Property = CssFont*/
const Value V23482_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23482_289 = {S12, 0, &V23482_289_57759, &P23482_288};	/*Property = CssImages*/
const Value V23482_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23482_291 = {S13, 0, &V23482_291_57180, &P23482_289};	/*Property = CssMediaQueries*/
const Value V23482_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23482_292 = {S14, 0, &V23482_292_57183, &P23482_291};	/*Property = CssMinMax*/
const Value V23482_293_57187 = {S0, NULL};	/*Value = False*/
const Property P23482_293 = {S15, 0, &V23482_293_57187, &P23482_292};	/*Property = CssOverflow*/
const Value V23482_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23482_294 = {S16, 0, &V23482_294_57189, &P23482_293};	/*Property = CssPosition*/
const Value V23482_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23482_295 = {S17, 0, &V23482_295_57192, &P23482_294};	/*Property = CssText*/
const Value V23482_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23482_297 = {S18, 0, &V23482_297_57195, &P23482_295};	/*Property = CssTransforms*/
const Value V23482_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23482_241 = {S19, 0, &V23482_241_55799, &P23482_297};	/*Property = CssTransitions*/
const Value V23482_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23482_298 = {S20, 0, &V23482_298_57206, &P23482_241};	/*Property = CssUI*/
const Value V23482_300_57215 = {S2, NULL};	/*Value = True*/
const Property P23482_300 = {S21, 0, &V23482_300_57215, &P23482_298};	/*Property = DataSet*/
const Value V23482_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23482_301 = {S22, 0, &V23482_301_57218, &P23482_300};	/*Property = DataUrl*/
const Value V23482_302_57222 = {S0, NULL};	/*Value = False*/
const Property P23482_302 = {S23, 0, &V23482_302_57222, &P23482_301};	/*Property = DeviceOrientation*/
const Value V23482_304_57227 = {S2, NULL};	/*Value = True*/
const Property P23482_304 = {S24, 0, &V23482_304_57227, &P23482_302};	/*Property = FileReader*/
const Value V23482_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23482_305 = {S25, 0, &V23482_305_57231, &P23482_304};	/*Property = FileSaver*/
const Value V23482_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23482_306 = {S26, 0, &V23482_306_57237, &P23482_305};	/*Property = FileWriter*/
const Value V23482_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23482_307 = {S27, 0, &V23482_307_57239, &P23482_306};	/*Property = FormData*/
const Value V23482_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23482_309 = {S28, 0, &V23482_309_57246, &P23482_307};	/*Property = Fullscreen*/
const Value V23482_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23482_310 = {S29, 0, &V23482_310_57248, &P23482_309};	/*Property = GeoLocation*/
const Value V23482_312_57254 = {S2, NULL};	/*Value = True*/
const Property P23482_312 = {S30, 0, &V23482_312_57254, &P23482_310};	/*Property = History*/
const Value V23482_314_57260 = {S2, NULL};	/*Value = True*/
const Property P23482_314 = {S31, 0, &V23482_314_57260, &P23482_312};	/*Property = Html5*/
const Value V23482_313_57257 = {S2, NULL};	/*Value = True*/
const Property P23482_313 = {S32, 0, &V23482_313_57257, &P23482_314};	/*Property = Html-Media-Capture*/
const Value V23482_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23482_315 = {S33, 0, &V23482_315_57264, &P23482_313};	/*Property = Iframe*/
const Value V23482_316_57266 = {S2, NULL};	/*Value = True*/
const Property P23482_316 = {S34, 0, &V23482_316_57266, &P23482_315};	/*Property = IndexedDB*/
const Value V23482_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23482_317 = {S35, 0, &V23482_317_57269, &P23482_316};	/*Property = Json*/
const Value V23482_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P23482_15 = {S37, 0, &V23482_15_28, &P23482_317};	/*Property = LayoutEngine*/
const Value V23482_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23482_344 = {S38, 0, &V23482_344_57272, &P23482_15};	/*Property = Masking*/
const Value V23482_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23482_320 = {S39, 0, &V23482_320_57290, &P23482_344};	/*Property = PostMessage*/
const Value V23482_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23482_321 = {S40, 0, &V23482_321_57293, &P23482_320};	/*Property = Progress*/
const Value V23482_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23482_322 = {S41, 0, &V23482_322_57296, &P23482_321};	/*Property = Prompts*/
const Value V23482_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23482_323 = {S42, 0, &V23482_323_57299, &P23482_322};	/*Property = Selector*/
const Value V23482_324_57305 = {S2, NULL};	/*Value = True*/
const Property P23482_324 = {S43, 0, &V23482_324_57305, &P23482_323};	/*Property = Svg*/
const Value V23482_327_57314 = {S2, NULL};	/*Value = True*/
const Property P23482_327 = {S44, 0, &V23482_327_57314, &P23482_324};	/*Property = TouchEvents*/
const Value V23482_328_57317 = {S2, NULL};	/*Value = True*/
const Property P23482_328 = {S45, 0, &V23482_328_57317, &P23482_327};	/*Property = Track*/
const Value V23482_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23482_329 = {S46, 0, &V23482_329_57323, &P23482_328};	/*Property = Video*/
const Value V23482_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23482_330 = {S47, 0, &V23482_330_57326, &P23482_329};	/*Property = Viewport*/
const Value V23482_332_57338 = {S2, NULL};	/*Value = True*/
const Property P23482_332 = {S48, 0, &V23482_332_57338, &P23482_330};	/*Property = WebWorkers*/
const Value V23482_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23482_333 = {S49, 0, &V23482_333_57342, &P23482_332};	/*Property = Xhr2*/
const Profile P23482 = {"23482", &P23482_333};	/*Profile ID - 23482*/
const Value V23493_1_1 = {S2, NULL};	/*Value = True*/
const Property P23493_1 = {S50, 0, &V23493_1_1, NULL};	/*Property = IsMobile*/
const Value V23493_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23493_5 = {S52, 0, &V23493_5_1222, &P23493_1};	/*Property = ScreenPixelsHeight*/
const Value V23493_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23493_4 = {S54, 0, &V23493_4_1224, &P23493_5};	/*Property = ScreenPixelsWidth*/
const Profile P23493 = {"23493", &P23493_4};	/*Profile ID - 23493*/
const Value V23647_1_1 = {S2, NULL};	/*Value = True*/
const Property P23647_1 = {S50, 0, &V23647_1_1, NULL};	/*Property = IsMobile*/
const Value V23647_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23647_5 = {S52, 0, &V23647_5_9981, &P23647_1};	/*Property = ScreenPixelsHeight*/
const Value V23647_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23647_4 = {S54, 0, &V23647_4_39098, &P23647_5};	/*Property = ScreenPixelsWidth*/
const Profile P23647 = {"23647", &P23647_4};	/*Profile ID - 23647*/
const Value V23491_279_57147 = {S2, NULL};	/*Value = True*/
const Property P23491_279 = {S1, 0, &V23491_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V23491_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23491_282 = {S3, 0, &V23491_282_57154, &P23491_279};	/*Property = BlobBuilder*/
const Value V23491_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23491_350 = {S4, 0, &V23491_350_57444, &P23491_282};	/*Property = Canvas*/
const Value V23491_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23491_283 = {S5, 0, &V23491_283_57159, &P23491_350};	/*Property = CssBackground*/
const Value V23491_284_57162 = {S2, NULL};	/*Value = True*/
const Property P23491_284 = {S6, 0, &V23491_284_57162, &P23491_283};	/*Property = CssBorderImage*/
const Value V23491_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23491_285 = {S7, 0, &V23491_285_57166, &P23491_284};	/*Property = CssCanvas*/
const Value V23491_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23491_286 = {S8, 0, &V23491_286_57168, &P23491_285};	/*Property = CssColor*/
const Value V23491_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23491_237 = {S9, 0, &V23491_237_55786, &P23491_286};	/*Property = CssColumn*/
const Value V23491_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23491_287 = {S10, 0, &V23491_287_57172, &P23491_237};	/*Property = CssFlexbox*/
const Value V23491_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23491_288 = {S11, 0, &V23491_288_57174, &P23491_287};	/*Property = CssFont*/
const Value V23491_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23491_289 = {S12, 0, &V23491_289_57759, &P23491_288};	/*Property = CssImages*/
const Value V23491_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23491_291 = {S13, 0, &V23491_291_57180, &P23491_289};	/*Property = CssMediaQueries*/
const Value V23491_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23491_292 = {S14, 0, &V23491_292_57183, &P23491_291};	/*Property = CssMinMax*/
const Value V23491_293_57187 = {S0, NULL};	/*Value = False*/
const Property P23491_293 = {S15, 0, &V23491_293_57187, &P23491_292};	/*Property = CssOverflow*/
const Value V23491_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23491_294 = {S16, 0, &V23491_294_57189, &P23491_293};	/*Property = CssPosition*/
const Value V23491_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23491_295 = {S17, 0, &V23491_295_57192, &P23491_294};	/*Property = CssText*/
const Value V23491_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23491_297 = {S18, 0, &V23491_297_57195, &P23491_295};	/*Property = CssTransforms*/
const Value V23491_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23491_241 = {S19, 0, &V23491_241_55799, &P23491_297};	/*Property = CssTransitions*/
const Value V23491_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23491_298 = {S20, 0, &V23491_298_57206, &P23491_241};	/*Property = CssUI*/
const Value V23491_300_57215 = {S2, NULL};	/*Value = True*/
const Property P23491_300 = {S21, 0, &V23491_300_57215, &P23491_298};	/*Property = DataSet*/
const Value V23491_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23491_301 = {S22, 0, &V23491_301_57218, &P23491_300};	/*Property = DataUrl*/
const Value V23491_302_57221 = {S2, NULL};	/*Value = True*/
const Property P23491_302 = {S23, 0, &V23491_302_57221, &P23491_301};	/*Property = DeviceOrientation*/
const Value V23491_304_57227 = {S2, NULL};	/*Value = True*/
const Property P23491_304 = {S24, 0, &V23491_304_57227, &P23491_302};	/*Property = FileReader*/
const Value V23491_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23491_305 = {S25, 0, &V23491_305_57231, &P23491_304};	/*Property = FileSaver*/
const Value V23491_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23491_306 = {S26, 0, &V23491_306_57237, &P23491_305};	/*Property = FileWriter*/
const Value V23491_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23491_307 = {S27, 0, &V23491_307_57239, &P23491_306};	/*Property = FormData*/
const Value V23491_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23491_309 = {S28, 0, &V23491_309_57246, &P23491_307};	/*Property = Fullscreen*/
const Value V23491_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23491_310 = {S29, 0, &V23491_310_57248, &P23491_309};	/*Property = GeoLocation*/
const Value V23491_312_57254 = {S2, NULL};	/*Value = True*/
const Property P23491_312 = {S30, 0, &V23491_312_57254, &P23491_310};	/*Property = History*/
const Value V23491_314_57260 = {S2, NULL};	/*Value = True*/
const Property P23491_314 = {S31, 0, &V23491_314_57260, &P23491_312};	/*Property = Html5*/
const Value V23491_313_57257 = {S2, NULL};	/*Value = True*/
const Property P23491_313 = {S32, 0, &V23491_313_57257, &P23491_314};	/*Property = Html-Media-Capture*/
const Value V23491_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23491_315 = {S33, 0, &V23491_315_57264, &P23491_313};	/*Property = Iframe*/
const Value V23491_316_57266 = {S2, NULL};	/*Value = True*/
const Property P23491_316 = {S34, 0, &V23491_316_57266, &P23491_315};	/*Property = IndexedDB*/
const Value V23491_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23491_317 = {S35, 0, &V23491_317_57269, &P23491_316};	/*Property = Json*/
const Value V23491_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P23491_15 = {S37, 0, &V23491_15_54167, &P23491_317};	/*Property = LayoutEngine*/
const Value V23491_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23491_344 = {S38, 0, &V23491_344_57272, &P23491_15};	/*Property = Masking*/
const Value V23491_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23491_320 = {S39, 0, &V23491_320_57290, &P23491_344};	/*Property = PostMessage*/
const Value V23491_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23491_321 = {S40, 0, &V23491_321_57293, &P23491_320};	/*Property = Progress*/
const Value V23491_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23491_322 = {S41, 0, &V23491_322_57296, &P23491_321};	/*Property = Prompts*/
const Value V23491_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23491_323 = {S42, 0, &V23491_323_57299, &P23491_322};	/*Property = Selector*/
const Value V23491_324_57306 = {S0, NULL};	/*Value = False*/
const Property P23491_324 = {S43, 0, &V23491_324_57306, &P23491_323};	/*Property = Svg*/
const Value V23491_327_57314 = {S2, NULL};	/*Value = True*/
const Property P23491_327 = {S44, 0, &V23491_327_57314, &P23491_324};	/*Property = TouchEvents*/
const Value V23491_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23491_328 = {S45, 0, &V23491_328_57318, &P23491_327};	/*Property = Track*/
const Value V23491_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23491_329 = {S46, 0, &V23491_329_57323, &P23491_328};	/*Property = Video*/
const Value V23491_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23491_330 = {S47, 0, &V23491_330_57326, &P23491_329};	/*Property = Viewport*/
const Value V23491_332_57339 = {S0, NULL};	/*Value = False*/
const Property P23491_332 = {S48, 0, &V23491_332_57339, &P23491_330};	/*Property = WebWorkers*/
const Value V23491_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23491_333 = {S49, 0, &V23491_333_57342, &P23491_332};	/*Property = Xhr2*/
const Profile P23491 = {"23491", &P23491_333};	/*Profile ID - 23491*/
const Value V23694_1_1 = {S2, NULL};	/*Value = True*/
const Property P23694_1 = {S50, 0, &V23694_1_1, NULL};	/*Property = IsMobile*/
const Value V23694_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23694_5 = {S52, 0, &V23694_5_1277, &P23694_1};	/*Property = ScreenPixelsHeight*/
const Value V23694_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23694_4 = {S54, 0, &V23694_4_1279, &P23694_5};	/*Property = ScreenPixelsWidth*/
const Profile P23694 = {"23694", &P23694_4};	/*Profile ID - 23694*/
const Value V23654_1_1 = {S2, NULL};	/*Value = True*/
const Property P23654_1 = {S50, 0, &V23654_1_1, NULL};	/*Property = IsMobile*/
const Value V23654_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23654_5 = {S52, 0, &V23654_5_9981, &P23654_1};	/*Property = ScreenPixelsHeight*/
const Value V23654_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23654_4 = {S54, 0, &V23654_4_39098, &P23654_5};	/*Property = ScreenPixelsWidth*/
const Profile P23654 = {"23654", &P23654_4};	/*Profile ID - 23654*/
const Value V24159_1_1 = {S2, NULL};	/*Value = True*/
const Property P24159_1 = {S50, 0, &V24159_1_1, NULL};	/*Property = IsMobile*/
const Value V24159_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P24159_5 = {S52, 0, &V24159_5_1458, &P24159_1};	/*Property = ScreenPixelsHeight*/
const Value V24159_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P24159_4 = {S54, 0, &V24159_4_37617, &P24159_5};	/*Property = ScreenPixelsWidth*/
const Profile P24159 = {"24159", &P24159_4};	/*Profile ID - 24159*/
const Value V24336_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_279 = {S1, 0, &V24336_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24336_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_282 = {S3, 0, &V24336_282_57155, &P24336_279};	/*Property = BlobBuilder*/
const Value V24336_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_350 = {S4, 0, &V24336_350_57446, &P24336_282};	/*Property = Canvas*/
const Value V24336_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_283 = {S5, 0, &V24336_283_57161, &P24336_350};	/*Property = CssBackground*/
const Value V24336_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_284 = {S6, 0, &V24336_284_57164, &P24336_283};	/*Property = CssBorderImage*/
const Value V24336_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_285 = {S7, 0, &V24336_285_57167, &P24336_284};	/*Property = CssCanvas*/
const Value V24336_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_286 = {S8, 0, &V24336_286_57170, &P24336_285};	/*Property = CssColor*/
const Value V24336_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_237 = {S9, 0, &V24336_237_55786, &P24336_286};	/*Property = CssColumn*/
const Value V24336_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_287 = {S10, 0, &V24336_287_57173, &P24336_237};	/*Property = CssFlexbox*/
const Value V24336_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_288 = {S11, 0, &V24336_288_57176, &P24336_287};	/*Property = CssFont*/
const Value V24336_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_289 = {S12, 0, &V24336_289_86899, &P24336_288};	/*Property = CssImages*/
const Value V24336_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_291 = {S13, 0, &V24336_291_57182, &P24336_289};	/*Property = CssMediaQueries*/
const Value V24336_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_292 = {S14, 0, &V24336_292_57185, &P24336_291};	/*Property = CssMinMax*/
const Value V24336_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_293 = {S15, 0, &V24336_293_57188, &P24336_292};	/*Property = CssOverflow*/
const Value V24336_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_294 = {S16, 0, &V24336_294_57191, &P24336_293};	/*Property = CssPosition*/
const Value V24336_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_295 = {S17, 0, &V24336_295_57194, &P24336_294};	/*Property = CssText*/
const Value V24336_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_297 = {S18, 0, &V24336_297_57197, &P24336_295};	/*Property = CssTransforms*/
const Value V24336_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_241 = {S19, 0, &V24336_241_55801, &P24336_297};	/*Property = CssTransitions*/
const Value V24336_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_298 = {S20, 0, &V24336_298_57208, &P24336_241};	/*Property = CssUI*/
const Value V24336_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_300 = {S21, 0, &V24336_300_57217, &P24336_298};	/*Property = DataSet*/
const Value V24336_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_301 = {S22, 0, &V24336_301_57220, &P24336_300};	/*Property = DataUrl*/
const Value V24336_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_302 = {S23, 0, &V24336_302_57223, &P24336_301};	/*Property = DeviceOrientation*/
const Value V24336_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_304 = {S24, 0, &V24336_304_57229, &P24336_302};	/*Property = FileReader*/
const Value V24336_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_305 = {S25, 0, &V24336_305_57232, &P24336_304};	/*Property = FileSaver*/
const Value V24336_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_306 = {S26, 0, &V24336_306_57238, &P24336_305};	/*Property = FileWriter*/
const Value V24336_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_307 = {S27, 0, &V24336_307_57241, &P24336_306};	/*Property = FormData*/
const Value V24336_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_309 = {S28, 0, &V24336_309_57247, &P24336_307};	/*Property = Fullscreen*/
const Value V24336_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_310 = {S29, 0, &V24336_310_57250, &P24336_309};	/*Property = GeoLocation*/
const Value V24336_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_312 = {S30, 0, &V24336_312_57256, &P24336_310};	/*Property = History*/
const Value V24336_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_314 = {S31, 0, &V24336_314_57262, &P24336_312};	/*Property = Html5*/
const Value V24336_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_313 = {S32, 0, &V24336_313_57259, &P24336_314};	/*Property = Html-Media-Capture*/
const Value V24336_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_315 = {S33, 0, &V24336_315_57265, &P24336_313};	/*Property = Iframe*/
const Value V24336_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_316 = {S34, 0, &V24336_316_57268, &P24336_315};	/*Property = IndexedDB*/
const Value V24336_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_317 = {S35, 0, &V24336_317_57271, &P24336_316};	/*Property = Json*/
const Value V24336_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P24336_15 = {S37, 0, &V24336_15_29, &P24336_317};	/*Property = LayoutEngine*/
const Value V24336_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_344 = {S38, 0, &V24336_344_57274, &P24336_15};	/*Property = Masking*/
const Value V24336_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_320 = {S39, 0, &V24336_320_57292, &P24336_344};	/*Property = PostMessage*/
const Value V24336_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_321 = {S40, 0, &V24336_321_57295, &P24336_320};	/*Property = Progress*/
const Value V24336_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_322 = {S41, 0, &V24336_322_57298, &P24336_321};	/*Property = Prompts*/
const Value V24336_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_323 = {S42, 0, &V24336_323_57301, &P24336_322};	/*Property = Selector*/
const Value V24336_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_324 = {S43, 0, &V24336_324_57307, &P24336_323};	/*Property = Svg*/
const Value V24336_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_327 = {S44, 0, &V24336_327_57316, &P24336_324};	/*Property = TouchEvents*/
const Value V24336_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_328 = {S45, 0, &V24336_328_57319, &P24336_327};	/*Property = Track*/
const Value V24336_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_329 = {S46, 0, &V24336_329_57325, &P24336_328};	/*Property = Video*/
const Value V24336_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_330 = {S47, 0, &V24336_330_57328, &P24336_329};	/*Property = Viewport*/
const Value V24336_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_332 = {S48, 0, &V24336_332_57340, &P24336_330};	/*Property = WebWorkers*/
const Value V24336_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24336_333 = {S49, 0, &V24336_333_57343, &P24336_332};	/*Property = Xhr2*/
const Profile P24336 = {"24336", &P24336_333};	/*Profile ID - 24336*/
const Value V24178_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24178_279 = {S1, 0, &V24178_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24178_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24178_282 = {S3, 0, &V24178_282_57154, &P24178_279};	/*Property = BlobBuilder*/
const Value V24178_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24178_350 = {S4, 0, &V24178_350_57444, &P24178_282};	/*Property = Canvas*/
const Value V24178_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24178_283 = {S5, 0, &V24178_283_57159, &P24178_350};	/*Property = CssBackground*/
const Value V24178_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24178_284 = {S6, 0, &V24178_284_57162, &P24178_283};	/*Property = CssBorderImage*/
const Value V24178_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24178_285 = {S7, 0, &V24178_285_57166, &P24178_284};	/*Property = CssCanvas*/
const Value V24178_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24178_286 = {S8, 0, &V24178_286_57168, &P24178_285};	/*Property = CssColor*/
const Value V24178_237_55784 = {S2, NULL};	/*Value = True*/
const Property P24178_237 = {S9, 0, &V24178_237_55784, &P24178_286};	/*Property = CssColumn*/
const Value V24178_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24178_287 = {S10, 0, &V24178_287_57172, &P24178_237};	/*Property = CssFlexbox*/
const Value V24178_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24178_288 = {S11, 0, &V24178_288_57174, &P24178_287};	/*Property = CssFont*/
const Value V24178_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24178_289 = {S12, 0, &V24178_289_57759, &P24178_288};	/*Property = CssImages*/
const Value V24178_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24178_291 = {S13, 0, &V24178_291_57180, &P24178_289};	/*Property = CssMediaQueries*/
const Value V24178_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24178_292 = {S14, 0, &V24178_292_57183, &P24178_291};	/*Property = CssMinMax*/
const Value V24178_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24178_293 = {S15, 0, &V24178_293_57187, &P24178_292};	/*Property = CssOverflow*/
const Value V24178_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24178_294 = {S16, 0, &V24178_294_57189, &P24178_293};	/*Property = CssPosition*/
const Value V24178_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24178_295 = {S17, 0, &V24178_295_57192, &P24178_294};	/*Property = CssText*/
const Value V24178_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24178_297 = {S18, 0, &V24178_297_57195, &P24178_295};	/*Property = CssTransforms*/
const Value V24178_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24178_241 = {S19, 0, &V24178_241_55799, &P24178_297};	/*Property = CssTransitions*/
const Value V24178_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24178_298 = {S20, 0, &V24178_298_57206, &P24178_241};	/*Property = CssUI*/
const Value V24178_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24178_300 = {S21, 0, &V24178_300_57215, &P24178_298};	/*Property = DataSet*/
const Value V24178_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24178_301 = {S22, 0, &V24178_301_57218, &P24178_300};	/*Property = DataUrl*/
const Value V24178_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24178_302 = {S23, 0, &V24178_302_57222, &P24178_301};	/*Property = DeviceOrientation*/
const Value V24178_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24178_304 = {S24, 0, &V24178_304_57227, &P24178_302};	/*Property = FileReader*/
const Value V24178_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24178_305 = {S25, 0, &V24178_305_57231, &P24178_304};	/*Property = FileSaver*/
const Value V24178_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24178_306 = {S26, 0, &V24178_306_57237, &P24178_305};	/*Property = FileWriter*/
const Value V24178_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24178_307 = {S27, 0, &V24178_307_57239, &P24178_306};	/*Property = FormData*/
const Value V24178_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24178_309 = {S28, 0, &V24178_309_57246, &P24178_307};	/*Property = Fullscreen*/
const Value V24178_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24178_310 = {S29, 0, &V24178_310_57248, &P24178_309};	/*Property = GeoLocation*/
const Value V24178_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24178_312 = {S30, 0, &V24178_312_57254, &P24178_310};	/*Property = History*/
const Value V24178_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24178_314 = {S31, 0, &V24178_314_57260, &P24178_312};	/*Property = Html5*/
const Value V24178_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24178_313 = {S32, 0, &V24178_313_57257, &P24178_314};	/*Property = Html-Media-Capture*/
const Value V24178_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24178_315 = {S33, 0, &V24178_315_57264, &P24178_313};	/*Property = Iframe*/
const Value V24178_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24178_316 = {S34, 0, &V24178_316_57266, &P24178_315};	/*Property = IndexedDB*/
const Value V24178_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24178_317 = {S35, 0, &V24178_317_57269, &P24178_316};	/*Property = Json*/
const Value V24178_15_81502 = {S141, NULL};	/*Value = Blink*/
const Property P24178_15 = {S37, 0, &V24178_15_81502, &P24178_317};	/*Property = LayoutEngine*/
const Value V24178_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24178_344 = {S38, 0, &V24178_344_57272, &P24178_15};	/*Property = Masking*/
const Value V24178_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24178_320 = {S39, 0, &V24178_320_57290, &P24178_344};	/*Property = PostMessage*/
const Value V24178_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24178_321 = {S40, 0, &V24178_321_57293, &P24178_320};	/*Property = Progress*/
const Value V24178_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24178_322 = {S41, 0, &V24178_322_57296, &P24178_321};	/*Property = Prompts*/
const Value V24178_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24178_323 = {S42, 0, &V24178_323_57299, &P24178_322};	/*Property = Selector*/
const Value V24178_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24178_324 = {S43, 0, &V24178_324_57305, &P24178_323};	/*Property = Svg*/
const Value V24178_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24178_327 = {S44, 0, &V24178_327_57314, &P24178_324};	/*Property = TouchEvents*/
const Value V24178_328_57317 = {S2, NULL};	/*Value = True*/
const Property P24178_328 = {S45, 0, &V24178_328_57317, &P24178_327};	/*Property = Track*/
const Value V24178_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24178_329 = {S46, 0, &V24178_329_57323, &P24178_328};	/*Property = Video*/
const Value V24178_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24178_330 = {S47, 0, &V24178_330_57326, &P24178_329};	/*Property = Viewport*/
const Value V24178_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24178_332 = {S48, 0, &V24178_332_57339, &P24178_330};	/*Property = WebWorkers*/
const Value V24178_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24178_333 = {S49, 0, &V24178_333_57342, &P24178_332};	/*Property = Xhr2*/
const Profile P24178 = {"24178", &P24178_333};	/*Profile ID - 24178*/
const Value V16082_1_1 = {S2, NULL};	/*Value = True*/
const Property P16082_1 = {S50, 0, &V16082_1_1, NULL};	/*Property = IsMobile*/
const Value V16082_5_1380 = {S58, NULL};	/*Value = 240*/
const Property P16082_5 = {S52, 0, &V16082_5_1380, &P16082_1};	/*Property = ScreenPixelsHeight*/
const Value V16082_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P16082_4 = {S54, 0, &V16082_4_1279, &P16082_5};	/*Property = ScreenPixelsWidth*/
const Profile P16082 = {"16082", &P16082_4};	/*Profile ID - 16082*/
const Value V23490_279_57148 = {S0, NULL};	/*Value = False*/
const Property P23490_279 = {S1, 0, &V23490_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V23490_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23490_282 = {S3, 0, &V23490_282_57154, &P23490_279};	/*Property = BlobBuilder*/
const Value V23490_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23490_350 = {S4, 0, &V23490_350_57444, &P23490_282};	/*Property = Canvas*/
const Value V23490_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23490_283 = {S5, 0, &V23490_283_57159, &P23490_350};	/*Property = CssBackground*/
const Value V23490_284_57163 = {S0, NULL};	/*Value = False*/
const Property P23490_284 = {S6, 0, &V23490_284_57163, &P23490_283};	/*Property = CssBorderImage*/
const Value V23490_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23490_285 = {S7, 0, &V23490_285_57166, &P23490_284};	/*Property = CssCanvas*/
const Value V23490_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23490_286 = {S8, 0, &V23490_286_57168, &P23490_285};	/*Property = CssColor*/
const Value V23490_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23490_237 = {S9, 0, &V23490_237_55786, &P23490_286};	/*Property = CssColumn*/
const Value V23490_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23490_287 = {S10, 0, &V23490_287_57172, &P23490_237};	/*Property = CssFlexbox*/
const Value V23490_288_57175 = {S0, NULL};	/*Value = False*/
const Property P23490_288 = {S11, 0, &V23490_288_57175, &P23490_287};	/*Property = CssFont*/
const Value V23490_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23490_289 = {S12, 0, &V23490_289_57759, &P23490_288};	/*Property = CssImages*/
const Value V23490_291_57181 = {S0, NULL};	/*Value = False*/
const Property P23490_291 = {S13, 0, &V23490_291_57181, &P23490_289};	/*Property = CssMediaQueries*/
const Value V23490_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23490_292 = {S14, 0, &V23490_292_57183, &P23490_291};	/*Property = CssMinMax*/
const Value V23490_293_57187 = {S0, NULL};	/*Value = False*/
const Property P23490_293 = {S15, 0, &V23490_293_57187, &P23490_292};	/*Property = CssOverflow*/
const Value V23490_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23490_294 = {S16, 0, &V23490_294_57189, &P23490_293};	/*Property = CssPosition*/
const Value V23490_295_57193 = {S0, NULL};	/*Value = False*/
const Property P23490_295 = {S17, 0, &V23490_295_57193, &P23490_294};	/*Property = CssText*/
const Value V23490_297_57196 = {S0, NULL};	/*Value = False*/
const Property P23490_297 = {S18, 0, &V23490_297_57196, &P23490_295};	/*Property = CssTransforms*/
const Value V23490_241_55800 = {S0, NULL};	/*Value = False*/
const Property P23490_241 = {S19, 0, &V23490_241_55800, &P23490_297};	/*Property = CssTransitions*/
const Value V23490_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23490_298 = {S20, 0, &V23490_298_57206, &P23490_241};	/*Property = CssUI*/
const Value V23490_300_57216 = {S0, NULL};	/*Value = False*/
const Property P23490_300 = {S21, 0, &V23490_300_57216, &P23490_298};	/*Property = DataSet*/
const Value V23490_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23490_301 = {S22, 0, &V23490_301_57218, &P23490_300};	/*Property = DataUrl*/
const Value V23490_302_57222 = {S0, NULL};	/*Value = False*/
const Property P23490_302 = {S23, 0, &V23490_302_57222, &P23490_301};	/*Property = DeviceOrientation*/
const Value V23490_304_57228 = {S0, NULL};	/*Value = False*/
const Property P23490_304 = {S24, 0, &V23490_304_57228, &P23490_302};	/*Property = FileReader*/
const Value V23490_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23490_305 = {S25, 0, &V23490_305_57231, &P23490_304};	/*Property = FileSaver*/
const Value V23490_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23490_306 = {S26, 0, &V23490_306_57237, &P23490_305};	/*Property = FileWriter*/
const Value V23490_307_57240 = {S0, NULL};	/*Value = False*/
const Property P23490_307 = {S27, 0, &V23490_307_57240, &P23490_306};	/*Property = FormData*/
const Value V23490_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23490_309 = {S28, 0, &V23490_309_57246, &P23490_307};	/*Property = Fullscreen*/
const Value V23490_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23490_310 = {S29, 0, &V23490_310_57248, &P23490_309};	/*Property = GeoLocation*/
const Value V23490_312_57255 = {S0, NULL};	/*Value = False*/
const Property P23490_312 = {S30, 0, &V23490_312_57255, &P23490_310};	/*Property = History*/
const Value V23490_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23490_314 = {S31, 0, &V23490_314_57261, &P23490_312};	/*Property = Html5*/
const Value V23490_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23490_313 = {S32, 0, &V23490_313_57258, &P23490_314};	/*Property = Html-Media-Capture*/
const Value V23490_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23490_315 = {S33, 0, &V23490_315_57264, &P23490_313};	/*Property = Iframe*/
const Value V23490_316_57267 = {S0, NULL};	/*Value = False*/
const Property P23490_316 = {S34, 0, &V23490_316_57267, &P23490_315};	/*Property = IndexedDB*/
const Value V23490_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23490_317 = {S35, 0, &V23490_317_57269, &P23490_316};	/*Property = Json*/
const Value V23490_15_30 = {S60, NULL};	/*Value = Trident*/
const Property P23490_15 = {S37, 0, &V23490_15_30, &P23490_317};	/*Property = LayoutEngine*/
const Value V23490_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23490_344 = {S38, 0, &V23490_344_57272, &P23490_15};	/*Property = Masking*/
const Value V23490_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23490_320 = {S39, 0, &V23490_320_57290, &P23490_344};	/*Property = PostMessage*/
const Value V23490_321_57294 = {S0, NULL};	/*Value = False*/
const Property P23490_321 = {S40, 0, &V23490_321_57294, &P23490_320};	/*Property = Progress*/
const Value V23490_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23490_322 = {S41, 0, &V23490_322_57296, &P23490_321};	/*Property = Prompts*/
const Value V23490_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23490_323 = {S42, 0, &V23490_323_57299, &P23490_322};	/*Property = Selector*/
const Value V23490_324_57305 = {S2, NULL};	/*Value = True*/
const Property P23490_324 = {S43, 0, &V23490_324_57305, &P23490_323};	/*Property = Svg*/
const Value V23490_327_57315 = {S0, NULL};	/*Value = False*/
const Property P23490_327 = {S44, 0, &V23490_327_57315, &P23490_324};	/*Property = TouchEvents*/
const Value V23490_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23490_328 = {S45, 0, &V23490_328_57318, &P23490_327};	/*Property = Track*/
const Value V23490_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23490_329 = {S46, 0, &V23490_329_57323, &P23490_328};	/*Property = Video*/
const Value V23490_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23490_330 = {S47, 0, &V23490_330_57326, &P23490_329};	/*Property = Viewport*/
const Value V23490_332_57339 = {S0, NULL};	/*Value = False*/
const Property P23490_332 = {S48, 0, &V23490_332_57339, &P23490_330};	/*Property = WebWorkers*/
const Value V23490_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23490_333 = {S49, 0, &V23490_333_57342, &P23490_332};	/*Property = Xhr2*/
const Profile P23490 = {"23490", &P23490_333};	/*Profile ID - 23490*/
const Value V23496_1_1 = {S2, NULL};	/*Value = True*/
const Property P23496_1 = {S50, 0, &V23496_1_1, NULL};	/*Property = IsMobile*/
const Value V23496_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23496_5 = {S52, 0, &V23496_5_1227, &P23496_1};	/*Property = ScreenPixelsHeight*/
const Value V23496_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23496_4 = {S54, 0, &V23496_4_1229, &P23496_5};	/*Property = ScreenPixelsWidth*/
const Profile P23496 = {"23496", &P23496_4};	/*Profile ID - 23496*/
const Value V24143_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24143_279 = {S1, 0, &V24143_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24143_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24143_282 = {S3, 0, &V24143_282_57154, &P24143_279};	/*Property = BlobBuilder*/
const Value V24143_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24143_350 = {S4, 0, &V24143_350_57444, &P24143_282};	/*Property = Canvas*/
const Value V24143_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24143_283 = {S5, 0, &V24143_283_57159, &P24143_350};	/*Property = CssBackground*/
const Value V24143_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24143_284 = {S6, 0, &V24143_284_57162, &P24143_283};	/*Property = CssBorderImage*/
const Value V24143_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24143_285 = {S7, 0, &V24143_285_57166, &P24143_284};	/*Property = CssCanvas*/
const Value V24143_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24143_286 = {S8, 0, &V24143_286_57168, &P24143_285};	/*Property = CssColor*/
const Value V24143_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24143_237 = {S9, 0, &V24143_237_55786, &P24143_286};	/*Property = CssColumn*/
const Value V24143_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24143_287 = {S10, 0, &V24143_287_57172, &P24143_237};	/*Property = CssFlexbox*/
const Value V24143_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24143_288 = {S11, 0, &V24143_288_57174, &P24143_287};	/*Property = CssFont*/
const Value V24143_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24143_289 = {S12, 0, &V24143_289_57759, &P24143_288};	/*Property = CssImages*/
const Value V24143_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24143_291 = {S13, 0, &V24143_291_57180, &P24143_289};	/*Property = CssMediaQueries*/
const Value V24143_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24143_292 = {S14, 0, &V24143_292_57183, &P24143_291};	/*Property = CssMinMax*/
const Value V24143_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24143_293 = {S15, 0, &V24143_293_57187, &P24143_292};	/*Property = CssOverflow*/
const Value V24143_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24143_294 = {S16, 0, &V24143_294_57189, &P24143_293};	/*Property = CssPosition*/
const Value V24143_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24143_295 = {S17, 0, &V24143_295_57192, &P24143_294};	/*Property = CssText*/
const Value V24143_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24143_297 = {S18, 0, &V24143_297_57195, &P24143_295};	/*Property = CssTransforms*/
const Value V24143_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24143_241 = {S19, 0, &V24143_241_55799, &P24143_297};	/*Property = CssTransitions*/
const Value V24143_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24143_298 = {S20, 0, &V24143_298_57206, &P24143_241};	/*Property = CssUI*/
const Value V24143_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24143_300 = {S21, 0, &V24143_300_57215, &P24143_298};	/*Property = DataSet*/
const Value V24143_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24143_301 = {S22, 0, &V24143_301_57218, &P24143_300};	/*Property = DataUrl*/
const Value V24143_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24143_302 = {S23, 0, &V24143_302_57222, &P24143_301};	/*Property = DeviceOrientation*/
const Value V24143_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24143_304 = {S24, 0, &V24143_304_57227, &P24143_302};	/*Property = FileReader*/
const Value V24143_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24143_305 = {S25, 0, &V24143_305_57231, &P24143_304};	/*Property = FileSaver*/
const Value V24143_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24143_306 = {S26, 0, &V24143_306_57237, &P24143_305};	/*Property = FileWriter*/
const Value V24143_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24143_307 = {S27, 0, &V24143_307_57239, &P24143_306};	/*Property = FormData*/
const Value V24143_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24143_309 = {S28, 0, &V24143_309_57246, &P24143_307};	/*Property = Fullscreen*/
const Value V24143_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24143_310 = {S29, 0, &V24143_310_57248, &P24143_309};	/*Property = GeoLocation*/
const Value V24143_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24143_312 = {S30, 0, &V24143_312_57254, &P24143_310};	/*Property = History*/
const Value V24143_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24143_314 = {S31, 0, &V24143_314_57260, &P24143_312};	/*Property = Html5*/
const Value V24143_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24143_313 = {S32, 0, &V24143_313_57258, &P24143_314};	/*Property = Html-Media-Capture*/
const Value V24143_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24143_315 = {S33, 0, &V24143_315_57264, &P24143_313};	/*Property = Iframe*/
const Value V24143_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24143_316 = {S34, 0, &V24143_316_57266, &P24143_315};	/*Property = IndexedDB*/
const Value V24143_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24143_317 = {S35, 0, &V24143_317_57269, &P24143_316};	/*Property = Json*/
const Value V24143_15_81502 = {S141, NULL};	/*Value = Blink*/
const Property P24143_15 = {S37, 0, &V24143_15_81502, &P24143_317};	/*Property = LayoutEngine*/
const Value V24143_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24143_344 = {S38, 0, &V24143_344_57272, &P24143_15};	/*Property = Masking*/
const Value V24143_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24143_320 = {S39, 0, &V24143_320_57290, &P24143_344};	/*Property = PostMessage*/
const Value V24143_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24143_321 = {S40, 0, &V24143_321_57293, &P24143_320};	/*Property = Progress*/
const Value V24143_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24143_322 = {S41, 0, &V24143_322_57296, &P24143_321};	/*Property = Prompts*/
const Value V24143_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24143_323 = {S42, 0, &V24143_323_57299, &P24143_322};	/*Property = Selector*/
const Value V24143_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24143_324 = {S43, 0, &V24143_324_57305, &P24143_323};	/*Property = Svg*/
const Value V24143_327_57315 = {S0, NULL};	/*Value = False*/
const Property P24143_327 = {S44, 0, &V24143_327_57315, &P24143_324};	/*Property = TouchEvents*/
const Value V24143_328_57317 = {S2, NULL};	/*Value = True*/
const Property P24143_328 = {S45, 0, &V24143_328_57317, &P24143_327};	/*Property = Track*/
const Value V24143_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24143_329 = {S46, 0, &V24143_329_57323, &P24143_328};	/*Property = Video*/
const Value V24143_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24143_330 = {S47, 0, &V24143_330_57326, &P24143_329};	/*Property = Viewport*/
const Value V24143_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24143_332 = {S48, 0, &V24143_332_57338, &P24143_330};	/*Property = WebWorkers*/
const Value V24143_333_57341 = {S2, NULL};	/*Value = True*/
const Property P24143_333 = {S49, 0, &V24143_333_57341, &P24143_332};	/*Property = Xhr2*/
const Profile P24143 = {"24143", &P24143_333};	/*Profile ID - 24143*/
const Profile P23592 = {"23592", NULL};	/*Profile ID - 23592*/
const Value V23567_1_1 = {S2, NULL};	/*Value = True*/
const Property P23567_1 = {S50, 0, &V23567_1_1, NULL};	/*Property = IsMobile*/
const Value V23567_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23567_5 = {S52, 0, &V23567_5_1277, &P23567_1};	/*Property = ScreenPixelsHeight*/
const Value V23567_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23567_4 = {S54, 0, &V23567_4_1279, &P23567_5};	/*Property = ScreenPixelsWidth*/
const Profile P23567 = {"23567", &P23567_4};	/*Profile ID - 23567*/
const Value V23820_279_57148 = {S0, NULL};	/*Value = False*/
const Property P23820_279 = {S1, 0, &V23820_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V23820_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23820_282 = {S3, 0, &V23820_282_57154, &P23820_279};	/*Property = BlobBuilder*/
const Value V23820_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23820_350 = {S4, 0, &V23820_350_57444, &P23820_282};	/*Property = Canvas*/
const Value V23820_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23820_283 = {S5, 0, &V23820_283_57159, &P23820_350};	/*Property = CssBackground*/
const Value V23820_284_57163 = {S0, NULL};	/*Value = False*/
const Property P23820_284 = {S6, 0, &V23820_284_57163, &P23820_283};	/*Property = CssBorderImage*/
const Value V23820_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23820_285 = {S7, 0, &V23820_285_57166, &P23820_284};	/*Property = CssCanvas*/
const Value V23820_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23820_286 = {S8, 0, &V23820_286_57168, &P23820_285};	/*Property = CssColor*/
const Value V23820_237_55784 = {S2, NULL};	/*Value = True*/
const Property P23820_237 = {S9, 0, &V23820_237_55784, &P23820_286};	/*Property = CssColumn*/
const Value V23820_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23820_287 = {S10, 0, &V23820_287_57172, &P23820_237};	/*Property = CssFlexbox*/
const Value V23820_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23820_288 = {S11, 0, &V23820_288_57174, &P23820_287};	/*Property = CssFont*/
const Value V23820_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23820_289 = {S12, 0, &V23820_289_57759, &P23820_288};	/*Property = CssImages*/
const Value V23820_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23820_291 = {S13, 0, &V23820_291_57180, &P23820_289};	/*Property = CssMediaQueries*/
const Value V23820_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23820_292 = {S14, 0, &V23820_292_57183, &P23820_291};	/*Property = CssMinMax*/
const Value V23820_293_57187 = {S0, NULL};	/*Value = False*/
const Property P23820_293 = {S15, 0, &V23820_293_57187, &P23820_292};	/*Property = CssOverflow*/
const Value V23820_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23820_294 = {S16, 0, &V23820_294_57189, &P23820_293};	/*Property = CssPosition*/
const Value V23820_295_57193 = {S0, NULL};	/*Value = False*/
const Property P23820_295 = {S17, 0, &V23820_295_57193, &P23820_294};	/*Property = CssText*/
const Value V23820_297_57196 = {S0, NULL};	/*Value = False*/
const Property P23820_297 = {S18, 0, &V23820_297_57196, &P23820_295};	/*Property = CssTransforms*/
const Value V23820_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23820_241 = {S19, 0, &V23820_241_55799, &P23820_297};	/*Property = CssTransitions*/
const Value V23820_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23820_298 = {S20, 0, &V23820_298_57206, &P23820_241};	/*Property = CssUI*/
const Value V23820_300_57215 = {S2, NULL};	/*Value = True*/
const Property P23820_300 = {S21, 0, &V23820_300_57215, &P23820_298};	/*Property = DataSet*/
const Value V23820_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23820_301 = {S22, 0, &V23820_301_57218, &P23820_300};	/*Property = DataUrl*/
const Value V23820_302_57222 = {S0, NULL};	/*Value = False*/
const Property P23820_302 = {S23, 0, &V23820_302_57222, &P23820_301};	/*Property = DeviceOrientation*/
const Value V23820_304_57227 = {S2, NULL};	/*Value = True*/
const Property P23820_304 = {S24, 0, &V23820_304_57227, &P23820_302};	/*Property = FileReader*/
const Value V23820_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23820_305 = {S25, 0, &V23820_305_57231, &P23820_304};	/*Property = FileSaver*/
const Value V23820_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23820_306 = {S26, 0, &V23820_306_57237, &P23820_305};	/*Property = FileWriter*/
const Value V23820_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23820_307 = {S27, 0, &V23820_307_57239, &P23820_306};	/*Property = FormData*/
const Value V23820_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23820_309 = {S28, 0, &V23820_309_57246, &P23820_307};	/*Property = Fullscreen*/
const Value V23820_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23820_310 = {S29, 0, &V23820_310_57248, &P23820_309};	/*Property = GeoLocation*/
const Value V23820_312_57254 = {S2, NULL};	/*Value = True*/
const Property P23820_312 = {S30, 0, &V23820_312_57254, &P23820_310};	/*Property = History*/
const Value V23820_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23820_314 = {S31, 0, &V23820_314_57261, &P23820_312};	/*Property = Html5*/
const Value V23820_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23820_313 = {S32, 0, &V23820_313_57258, &P23820_314};	/*Property = Html-Media-Capture*/
const Value V23820_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23820_315 = {S33, 0, &V23820_315_57264, &P23820_313};	/*Property = Iframe*/
const Value V23820_316_57267 = {S0, NULL};	/*Value = False*/
const Property P23820_316 = {S34, 0, &V23820_316_57267, &P23820_315};	/*Property = IndexedDB*/
const Value V23820_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23820_317 = {S35, 0, &V23820_317_57269, &P23820_316};	/*Property = Json*/
const Value V23820_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P23820_15 = {S37, 0, &V23820_15_31, &P23820_317};	/*Property = LayoutEngine*/
const Value V23820_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23820_344 = {S38, 0, &V23820_344_57272, &P23820_15};	/*Property = Masking*/
const Value V23820_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23820_320 = {S39, 0, &V23820_320_57290, &P23820_344};	/*Property = PostMessage*/
const Value V23820_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23820_321 = {S40, 0, &V23820_321_57293, &P23820_320};	/*Property = Progress*/
const Value V23820_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23820_322 = {S41, 0, &V23820_322_57296, &P23820_321};	/*Property = Prompts*/
const Value V23820_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23820_323 = {S42, 0, &V23820_323_57299, &P23820_322};	/*Property = Selector*/
const Value V23820_324_57305 = {S2, NULL};	/*Value = True*/
const Property P23820_324 = {S43, 0, &V23820_324_57305, &P23820_323};	/*Property = Svg*/
const Value V23820_327_57315 = {S0, NULL};	/*Value = False*/
const Property P23820_327 = {S44, 0, &V23820_327_57315, &P23820_324};	/*Property = TouchEvents*/
const Value V23820_328_57317 = {S2, NULL};	/*Value = True*/
const Property P23820_328 = {S45, 0, &V23820_328_57317, &P23820_327};	/*Property = Track*/
const Value V23820_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23820_329 = {S46, 0, &V23820_329_57323, &P23820_328};	/*Property = Video*/
const Value V23820_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23820_330 = {S47, 0, &V23820_330_57326, &P23820_329};	/*Property = Viewport*/
const Value V23820_332_57339 = {S0, NULL};	/*Value = False*/
const Property P23820_332 = {S48, 0, &V23820_332_57339, &P23820_330};	/*Property = WebWorkers*/
const Value V23820_333_57342 = {S0, NULL};	/*Value = False*/
const Property P23820_333 = {S49, 0, &V23820_333_57342, &P23820_332};	/*Property = Xhr2*/
const Profile P23820 = {"23820", &P23820_333};	/*Profile ID - 23820*/
const Value V23588_1_1 = {S2, NULL};	/*Value = True*/
const Property P23588_1 = {S50, 0, &V23588_1_1, NULL};	/*Property = IsMobile*/
const Value V23588_5_1275 = {S63, NULL};	/*Value = 640*/
const Property P23588_5 = {S52, 0, &V23588_5_1275, &P23588_1};	/*Property = ScreenPixelsHeight*/
const Value V23588_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P23588_4 = {S54, 0, &V23588_4_1278, &P23588_5};	/*Property = ScreenPixelsWidth*/
const Profile P23588 = {"23588", &P23588_4};	/*Profile ID - 23588*/
const Value V23754_1_1 = {S2, NULL};	/*Value = True*/
const Property P23754_1 = {S50, 0, &V23754_1_1, NULL};	/*Property = IsMobile*/
const Value V23754_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23754_5 = {S52, 0, &V23754_5_1277, &P23754_1};	/*Property = ScreenPixelsHeight*/
const Value V23754_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23754_4 = {S54, 0, &V23754_4_1279, &P23754_5};	/*Property = ScreenPixelsWidth*/
const Profile P23754 = {"23754", &P23754_4};	/*Profile ID - 23754*/
const Value V24732_1_1 = {S2, NULL};	/*Value = True*/
const Property P24732_1 = {S50, 0, &V24732_1_1, NULL};	/*Property = IsMobile*/
const Value V24732_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24732_5 = {S52, 0, &V24732_5_1277, &P24732_1};	/*Property = ScreenPixelsHeight*/
const Value V24732_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24732_4 = {S54, 0, &V24732_4_1279, &P24732_5};	/*Property = ScreenPixelsWidth*/
const Profile P24732 = {"24732", &P24732_4};	/*Profile ID - 24732*/
const Value V24562_1_1 = {S2, NULL};	/*Value = True*/
const Property P24562_1 = {S50, 0, &V24562_1_1, NULL};	/*Property = IsMobile*/
const Value V24562_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24562_5 = {S52, 0, &V24562_5_1277, &P24562_1};	/*Property = ScreenPixelsHeight*/
const Value V24562_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24562_4 = {S54, 0, &V24562_4_1279, &P24562_5};	/*Property = ScreenPixelsWidth*/
const Profile P24562 = {"24562", &P24562_4};	/*Profile ID - 24562*/
const Value V23769_1_1 = {S2, NULL};	/*Value = True*/
const Property P23769_1 = {S50, 0, &V23769_1_1, NULL};	/*Property = IsMobile*/
const Value V23769_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23769_5 = {S52, 0, &V23769_5_1277, &P23769_1};	/*Property = ScreenPixelsHeight*/
const Value V23769_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23769_4 = {S54, 0, &V23769_4_1279, &P23769_5};	/*Property = ScreenPixelsWidth*/
const Profile P23769 = {"23769", &P23769_4};	/*Profile ID - 23769*/
const Value V23760_1_1 = {S2, NULL};	/*Value = True*/
const Property P23760_1 = {S50, 0, &V23760_1_1, NULL};	/*Property = IsMobile*/
const Value V23760_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23760_5 = {S52, 0, &V23760_5_9981, &P23760_1};	/*Property = ScreenPixelsHeight*/
const Value V23760_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23760_4 = {S54, 0, &V23760_4_39098, &P23760_5};	/*Property = ScreenPixelsWidth*/
const Profile P23760 = {"23760", &P23760_4};	/*Profile ID - 23760*/
const Value V24528_1_1 = {S2, NULL};	/*Value = True*/
const Property P24528_1 = {S50, 0, &V24528_1_1, NULL};	/*Property = IsMobile*/
const Value V24528_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24528_5 = {S52, 0, &V24528_5_9981, &P24528_1};	/*Property = ScreenPixelsHeight*/
const Value V24528_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P24528_4 = {S54, 0, &V24528_4_37617, &P24528_5};	/*Property = ScreenPixelsWidth*/
const Profile P24528 = {"24528", &P24528_4};	/*Profile ID - 24528*/
const Value V23710_1_1 = {S2, NULL};	/*Value = True*/
const Property P23710_1 = {S50, 0, &V23710_1_1, NULL};	/*Property = IsMobile*/
const Value V23710_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23710_5 = {S52, 0, &V23710_5_1277, &P23710_1};	/*Property = ScreenPixelsHeight*/
const Value V23710_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23710_4 = {S54, 0, &V23710_4_1279, &P23710_5};	/*Property = ScreenPixelsWidth*/
const Profile P23710 = {"23710", &P23710_4};	/*Profile ID - 23710*/
const Value V22228_1_1 = {S2, NULL};	/*Value = True*/
const Property P22228_1 = {S50, 0, &V22228_1_1, NULL};	/*Property = IsMobile*/
const Value V22228_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P22228_5 = {S52, 0, &V22228_5_1458, &P22228_1};	/*Property = ScreenPixelsHeight*/
const Value V22228_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P22228_4 = {S54, 0, &V22228_4_1460, &P22228_5};	/*Property = ScreenPixelsWidth*/
const Profile P22228 = {"22228", &P22228_4};	/*Profile ID - 22228*/
const Value V23756_1_1 = {S2, NULL};	/*Value = True*/
const Property P23756_1 = {S50, 0, &V23756_1_1, NULL};	/*Property = IsMobile*/
const Value V23756_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23756_5 = {S52, 0, &V23756_5_1227, &P23756_1};	/*Property = ScreenPixelsHeight*/
const Value V23756_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23756_4 = {S54, 0, &V23756_4_1229, &P23756_5};	/*Property = ScreenPixelsWidth*/
const Profile P23756 = {"23756", &P23756_4};	/*Profile ID - 23756*/
const Value V23707_1_1 = {S2, NULL};	/*Value = True*/
const Property P23707_1 = {S50, 0, &V23707_1_1, NULL};	/*Property = IsMobile*/
const Value V23707_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23707_5 = {S52, 0, &V23707_5_37, &P23707_1};	/*Property = ScreenPixelsHeight*/
const Value V23707_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23707_4 = {S54, 0, &V23707_4_36, &P23707_5};	/*Property = ScreenPixelsWidth*/
const Profile P23707 = {"23707", &P23707_4};	/*Profile ID - 23707*/
const Value V22991_1_1 = {S2, NULL};	/*Value = True*/
const Property P22991_1 = {S50, 0, &V22991_1_1, NULL};	/*Property = IsMobile*/
const Value V22991_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P22991_5 = {S52, 0, &V22991_5_59138, &P22991_1};	/*Property = ScreenPixelsHeight*/
const Value V22991_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P22991_4 = {S54, 0, &V22991_4_59970, &P22991_5};	/*Property = ScreenPixelsWidth*/
const Profile P22991 = {"22991", &P22991_4};	/*Profile ID - 22991*/
const Value V24131_1_1 = {S2, NULL};	/*Value = True*/
const Property P24131_1 = {S50, 0, &V24131_1_1, NULL};	/*Property = IsMobile*/
const Value V24131_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24131_5 = {S52, 0, &V24131_5_1227, &P24131_1};	/*Property = ScreenPixelsHeight*/
const Value V24131_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24131_4 = {S54, 0, &V24131_4_1229, &P24131_5};	/*Property = ScreenPixelsWidth*/
const Profile P24131 = {"24131", &P24131_4};	/*Profile ID - 24131*/
const Value V24715_1_1 = {S2, NULL};	/*Value = True*/
const Property P24715_1 = {S50, 0, &V24715_1_1, NULL};	/*Property = IsMobile*/
const Value V24715_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24715_5 = {S52, 0, &V24715_5_1227, &P24715_1};	/*Property = ScreenPixelsHeight*/
const Value V24715_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24715_4 = {S54, 0, &V24715_4_1229, &P24715_5};	/*Property = ScreenPixelsWidth*/
const Profile P24715 = {"24715", &P24715_4};	/*Profile ID - 24715*/
const Value V23711_1_1 = {S2, NULL};	/*Value = True*/
const Property P23711_1 = {S50, 0, &V23711_1_1, NULL};	/*Property = IsMobile*/
const Value V23711_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23711_5 = {S52, 0, &V23711_5_1403, &P23711_1};	/*Property = ScreenPixelsHeight*/
const Value V23711_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23711_4 = {S54, 0, &V23711_4_1404, &P23711_5};	/*Property = ScreenPixelsWidth*/
const Profile P23711 = {"23711", &P23711_4};	/*Profile ID - 23711*/
const Value V23713_1_1 = {S2, NULL};	/*Value = True*/
const Property P23713_1 = {S50, 0, &V23713_1_1, NULL};	/*Property = IsMobile*/
const Value V23713_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23713_5 = {S52, 0, &V23713_5_1277, &P23713_1};	/*Property = ScreenPixelsHeight*/
const Value V23713_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23713_4 = {S54, 0, &V23713_4_1279, &P23713_5};	/*Property = ScreenPixelsWidth*/
const Profile P23713 = {"23713", &P23713_4};	/*Profile ID - 23713*/
const Value V24072_1_1 = {S2, NULL};	/*Value = True*/
const Property P24072_1 = {S50, 0, &V24072_1_1, NULL};	/*Property = IsMobile*/
const Value V24072_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24072_5 = {S52, 0, &V24072_5_9981, &P24072_1};	/*Property = ScreenPixelsHeight*/
const Value V24072_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24072_4 = {S54, 0, &V24072_4_39098, &P24072_5};	/*Property = ScreenPixelsWidth*/
const Profile P24072 = {"24072", &P24072_4};	/*Profile ID - 24072*/
const Value V24096_1_1 = {S2, NULL};	/*Value = True*/
const Property P24096_1 = {S50, 0, &V24096_1_1, NULL};	/*Property = IsMobile*/
const Value V24096_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24096_5 = {S52, 0, &V24096_5_1227, &P24096_1};	/*Property = ScreenPixelsHeight*/
const Value V24096_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24096_4 = {S54, 0, &V24096_4_1229, &P24096_5};	/*Property = ScreenPixelsWidth*/
const Profile P24096 = {"24096", &P24096_4};	/*Profile ID - 24096*/
const Value V23697_1_1 = {S2, NULL};	/*Value = True*/
const Property P23697_1 = {S50, 0, &V23697_1_1, NULL};	/*Property = IsMobile*/
const Value V23697_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P23697_5 = {S52, 0, &V23697_5_1403, &P23697_1};	/*Property = ScreenPixelsHeight*/
const Value V23697_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P23697_4 = {S54, 0, &V23697_4_1404, &P23697_5};	/*Property = ScreenPixelsWidth*/
const Profile P23697 = {"23697", &P23697_4};	/*Profile ID - 23697*/
const Value V23708_1_1 = {S2, NULL};	/*Value = True*/
const Property P23708_1 = {S50, 0, &V23708_1_1, NULL};	/*Property = IsMobile*/
const Value V23708_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P23708_5 = {S52, 0, &V23708_5_37, &P23708_1};	/*Property = ScreenPixelsHeight*/
const Value V23708_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P23708_4 = {S54, 0, &V23708_4_36, &P23708_5};	/*Property = ScreenPixelsWidth*/
const Profile P23708 = {"23708", &P23708_4};	/*Profile ID - 23708*/
const Value V24407_1_1 = {S2, NULL};	/*Value = True*/
const Property P24407_1 = {S50, 0, &V24407_1_1, NULL};	/*Property = IsMobile*/
const Value V24407_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24407_5 = {S52, 0, &V24407_5_9981, &P24407_1};	/*Property = ScreenPixelsHeight*/
const Value V24407_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24407_4 = {S54, 0, &V24407_4_39098, &P24407_5};	/*Property = ScreenPixelsWidth*/
const Profile P24407 = {"24407", &P24407_4};	/*Profile ID - 24407*/
const Value V23712_1_1 = {S2, NULL};	/*Value = True*/
const Property P23712_1 = {S50, 0, &V23712_1_1, NULL};	/*Property = IsMobile*/
const Value V23712_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23712_5 = {S52, 0, &V23712_5_9981, &P23712_1};	/*Property = ScreenPixelsHeight*/
const Value V23712_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23712_4 = {S54, 0, &V23712_4_39098, &P23712_5};	/*Property = ScreenPixelsWidth*/
const Profile P23712 = {"23712", &P23712_4};	/*Profile ID - 23712*/
const Value V24687_1_1 = {S2, NULL};	/*Value = True*/
const Property P24687_1 = {S50, 0, &V24687_1_1, NULL};	/*Property = IsMobile*/
const Value V24687_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P24687_5 = {S52, 0, &V24687_5_59138, &P24687_1};	/*Property = ScreenPixelsHeight*/
const Value V24687_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P24687_4 = {S54, 0, &V24687_4_59970, &P24687_5};	/*Property = ScreenPixelsWidth*/
const Profile P24687 = {"24687", &P24687_4};	/*Profile ID - 24687*/
const Value V23715_1_1 = {S2, NULL};	/*Value = True*/
const Property P23715_1 = {S50, 0, &V23715_1_1, NULL};	/*Property = IsMobile*/
const Value V23715_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23715_5 = {S52, 0, &V23715_5_1227, &P23715_1};	/*Property = ScreenPixelsHeight*/
const Value V23715_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23715_4 = {S54, 0, &V23715_4_1229, &P23715_5};	/*Property = ScreenPixelsWidth*/
const Profile P23715 = {"23715", &P23715_4};	/*Profile ID - 23715*/
const Value V23702_1_1 = {S2, NULL};	/*Value = True*/
const Property P23702_1 = {S50, 0, &V23702_1_1, NULL};	/*Property = IsMobile*/
const Value V23702_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23702_5 = {S52, 0, &V23702_5_1227, &P23702_1};	/*Property = ScreenPixelsHeight*/
const Value V23702_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P23702_4 = {S54, 0, &V23702_4_1477, &P23702_5};	/*Property = ScreenPixelsWidth*/
const Profile P23702 = {"23702", &P23702_4};	/*Profile ID - 23702*/
const Value V23800_1_1 = {S2, NULL};	/*Value = True*/
const Property P23800_1 = {S50, 0, &V23800_1_1, NULL};	/*Property = IsMobile*/
const Value V23800_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23800_5 = {S52, 0, &V23800_5_1277, &P23800_1};	/*Property = ScreenPixelsHeight*/
const Value V23800_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23800_4 = {S54, 0, &V23800_4_2336, &P23800_5};	/*Property = ScreenPixelsWidth*/
const Profile P23800 = {"23800", &P23800_4};	/*Profile ID - 23800*/
const Value V24103_1_1 = {S2, NULL};	/*Value = True*/
const Property P24103_1 = {S50, 0, &V24103_1_1, NULL};	/*Property = IsMobile*/
const Value V24103_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P24103_5 = {S52, 0, &V24103_5_59138, &P24103_1};	/*Property = ScreenPixelsHeight*/
const Value V24103_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P24103_4 = {S54, 0, &V24103_4_59970, &P24103_5};	/*Property = ScreenPixelsWidth*/
const Profile P24103 = {"24103", &P24103_4};	/*Profile ID - 24103*/
const Value V24144_1_1 = {S2, NULL};	/*Value = True*/
const Property P24144_1 = {S50, 0, &V24144_1_1, NULL};	/*Property = IsMobile*/
const Value V24144_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24144_5 = {S52, 0, &V24144_5_1277, &P24144_1};	/*Property = ScreenPixelsHeight*/
const Value V24144_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P24144_4 = {S54, 0, &V24144_4_2336, &P24144_5};	/*Property = ScreenPixelsWidth*/
const Profile P24144 = {"24144", &P24144_4};	/*Profile ID - 24144*/
const Value V23680_1_1 = {S2, NULL};	/*Value = True*/
const Property P23680_1 = {S50, 0, &V23680_1_1, NULL};	/*Property = IsMobile*/
const Value V23680_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23680_5 = {S52, 0, &V23680_5_1227, &P23680_1};	/*Property = ScreenPixelsHeight*/
const Value V23680_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23680_4 = {S54, 0, &V23680_4_1229, &P23680_5};	/*Property = ScreenPixelsWidth*/
const Profile P23680 = {"23680", &P23680_4};	/*Profile ID - 23680*/
const Value V23660_279_57147 = {S2, NULL};	/*Value = True*/
const Property P23660_279 = {S1, 0, &V23660_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V23660_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23660_282 = {S3, 0, &V23660_282_57154, &P23660_279};	/*Property = BlobBuilder*/
const Value V23660_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23660_350 = {S4, 0, &V23660_350_57444, &P23660_282};	/*Property = Canvas*/
const Value V23660_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23660_283 = {S5, 0, &V23660_283_57159, &P23660_350};	/*Property = CssBackground*/
const Value V23660_284_57162 = {S2, NULL};	/*Value = True*/
const Property P23660_284 = {S6, 0, &V23660_284_57162, &P23660_283};	/*Property = CssBorderImage*/
const Value V23660_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23660_285 = {S7, 0, &V23660_285_57166, &P23660_284};	/*Property = CssCanvas*/
const Value V23660_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23660_286 = {S8, 0, &V23660_286_57168, &P23660_285};	/*Property = CssColor*/
const Value V23660_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23660_237 = {S9, 0, &V23660_237_55786, &P23660_286};	/*Property = CssColumn*/
const Value V23660_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23660_287 = {S10, 0, &V23660_287_57172, &P23660_237};	/*Property = CssFlexbox*/
const Value V23660_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23660_288 = {S11, 0, &V23660_288_57174, &P23660_287};	/*Property = CssFont*/
const Value V23660_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23660_289 = {S12, 0, &V23660_289_57759, &P23660_288};	/*Property = CssImages*/
const Value V23660_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23660_291 = {S13, 0, &V23660_291_57180, &P23660_289};	/*Property = CssMediaQueries*/
const Value V23660_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23660_292 = {S14, 0, &V23660_292_57183, &P23660_291};	/*Property = CssMinMax*/
const Value V23660_293_57187 = {S0, NULL};	/*Value = False*/
const Property P23660_293 = {S15, 0, &V23660_293_57187, &P23660_292};	/*Property = CssOverflow*/
const Value V23660_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23660_294 = {S16, 0, &V23660_294_57189, &P23660_293};	/*Property = CssPosition*/
const Value V23660_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23660_295 = {S17, 0, &V23660_295_57192, &P23660_294};	/*Property = CssText*/
const Value V23660_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23660_297 = {S18, 0, &V23660_297_57195, &P23660_295};	/*Property = CssTransforms*/
const Value V23660_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23660_241 = {S19, 0, &V23660_241_55799, &P23660_297};	/*Property = CssTransitions*/
const Value V23660_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23660_298 = {S20, 0, &V23660_298_57206, &P23660_241};	/*Property = CssUI*/
const Value V23660_300_57215 = {S2, NULL};	/*Value = True*/
const Property P23660_300 = {S21, 0, &V23660_300_57215, &P23660_298};	/*Property = DataSet*/
const Value V23660_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23660_301 = {S22, 0, &V23660_301_57218, &P23660_300};	/*Property = DataUrl*/
const Value V23660_302_57221 = {S2, NULL};	/*Value = True*/
const Property P23660_302 = {S23, 0, &V23660_302_57221, &P23660_301};	/*Property = DeviceOrientation*/
const Value V23660_304_57227 = {S2, NULL};	/*Value = True*/
const Property P23660_304 = {S24, 0, &V23660_304_57227, &P23660_302};	/*Property = FileReader*/
const Value V23660_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23660_305 = {S25, 0, &V23660_305_57231, &P23660_304};	/*Property = FileSaver*/
const Value V23660_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23660_306 = {S26, 0, &V23660_306_57237, &P23660_305};	/*Property = FileWriter*/
const Value V23660_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23660_307 = {S27, 0, &V23660_307_57239, &P23660_306};	/*Property = FormData*/
const Value V23660_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23660_309 = {S28, 0, &V23660_309_57246, &P23660_307};	/*Property = Fullscreen*/
const Value V23660_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23660_310 = {S29, 0, &V23660_310_57248, &P23660_309};	/*Property = GeoLocation*/
const Value V23660_312_57254 = {S2, NULL};	/*Value = True*/
const Property P23660_312 = {S30, 0, &V23660_312_57254, &P23660_310};	/*Property = History*/
const Value V23660_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23660_314 = {S31, 0, &V23660_314_57261, &P23660_312};	/*Property = Html5*/
const Value V23660_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23660_313 = {S32, 0, &V23660_313_57258, &P23660_314};	/*Property = Html-Media-Capture*/
const Value V23660_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23660_315 = {S33, 0, &V23660_315_57264, &P23660_313};	/*Property = Iframe*/
const Value V23660_316_57266 = {S2, NULL};	/*Value = True*/
const Property P23660_316 = {S34, 0, &V23660_316_57266, &P23660_315};	/*Property = IndexedDB*/
const Value V23660_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23660_317 = {S35, 0, &V23660_317_57269, &P23660_316};	/*Property = Json*/
const Value V23660_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P23660_15 = {S37, 0, &V23660_15_29, &P23660_317};	/*Property = LayoutEngine*/
const Value V23660_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23660_344 = {S38, 0, &V23660_344_57272, &P23660_15};	/*Property = Masking*/
const Value V23660_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23660_320 = {S39, 0, &V23660_320_57290, &P23660_344};	/*Property = PostMessage*/
const Value V23660_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23660_321 = {S40, 0, &V23660_321_57293, &P23660_320};	/*Property = Progress*/
const Value V23660_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23660_322 = {S41, 0, &V23660_322_57296, &P23660_321};	/*Property = Prompts*/
const Value V23660_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23660_323 = {S42, 0, &V23660_323_57299, &P23660_322};	/*Property = Selector*/
const Value V23660_324_57305 = {S2, NULL};	/*Value = True*/
const Property P23660_324 = {S43, 0, &V23660_324_57305, &P23660_323};	/*Property = Svg*/
const Value V23660_327_57314 = {S2, NULL};	/*Value = True*/
const Property P23660_327 = {S44, 0, &V23660_327_57314, &P23660_324};	/*Property = TouchEvents*/
const Value V23660_328_57317 = {S2, NULL};	/*Value = True*/
const Property P23660_328 = {S45, 0, &V23660_328_57317, &P23660_327};	/*Property = Track*/
const Value V23660_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23660_329 = {S46, 0, &V23660_329_57323, &P23660_328};	/*Property = Video*/
const Value V23660_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23660_330 = {S47, 0, &V23660_330_57326, &P23660_329};	/*Property = Viewport*/
const Value V23660_332_57338 = {S2, NULL};	/*Value = True*/
const Property P23660_332 = {S48, 0, &V23660_332_57338, &P23660_330};	/*Property = WebWorkers*/
const Value V23660_333_57341 = {S2, NULL};	/*Value = True*/
const Property P23660_333 = {S49, 0, &V23660_333_57341, &P23660_332};	/*Property = Xhr2*/
const Profile P23660 = {"23660", &P23660_333};	/*Profile ID - 23660*/
const Value V24532_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_279 = {S1, 0, &V24532_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24532_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_282 = {S3, 0, &V24532_282_57155, &P24532_279};	/*Property = BlobBuilder*/
const Value V24532_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_350 = {S4, 0, &V24532_350_57446, &P24532_282};	/*Property = Canvas*/
const Value V24532_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_283 = {S5, 0, &V24532_283_57161, &P24532_350};	/*Property = CssBackground*/
const Value V24532_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_284 = {S6, 0, &V24532_284_57164, &P24532_283};	/*Property = CssBorderImage*/
const Value V24532_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_285 = {S7, 0, &V24532_285_57167, &P24532_284};	/*Property = CssCanvas*/
const Value V24532_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_286 = {S8, 0, &V24532_286_57170, &P24532_285};	/*Property = CssColor*/
const Value V24532_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_237 = {S9, 0, &V24532_237_55786, &P24532_286};	/*Property = CssColumn*/
const Value V24532_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_287 = {S10, 0, &V24532_287_57173, &P24532_237};	/*Property = CssFlexbox*/
const Value V24532_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_288 = {S11, 0, &V24532_288_57176, &P24532_287};	/*Property = CssFont*/
const Value V24532_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_289 = {S12, 0, &V24532_289_86899, &P24532_288};	/*Property = CssImages*/
const Value V24532_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_291 = {S13, 0, &V24532_291_57182, &P24532_289};	/*Property = CssMediaQueries*/
const Value V24532_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_292 = {S14, 0, &V24532_292_57185, &P24532_291};	/*Property = CssMinMax*/
const Value V24532_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_293 = {S15, 0, &V24532_293_57188, &P24532_292};	/*Property = CssOverflow*/
const Value V24532_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_294 = {S16, 0, &V24532_294_57191, &P24532_293};	/*Property = CssPosition*/
const Value V24532_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_295 = {S17, 0, &V24532_295_57194, &P24532_294};	/*Property = CssText*/
const Value V24532_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_297 = {S18, 0, &V24532_297_57197, &P24532_295};	/*Property = CssTransforms*/
const Value V24532_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_241 = {S19, 0, &V24532_241_55801, &P24532_297};	/*Property = CssTransitions*/
const Value V24532_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_298 = {S20, 0, &V24532_298_57208, &P24532_241};	/*Property = CssUI*/
const Value V24532_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_300 = {S21, 0, &V24532_300_57217, &P24532_298};	/*Property = DataSet*/
const Value V24532_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_301 = {S22, 0, &V24532_301_57220, &P24532_300};	/*Property = DataUrl*/
const Value V24532_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_302 = {S23, 0, &V24532_302_57223, &P24532_301};	/*Property = DeviceOrientation*/
const Value V24532_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_304 = {S24, 0, &V24532_304_57229, &P24532_302};	/*Property = FileReader*/
const Value V24532_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_305 = {S25, 0, &V24532_305_57232, &P24532_304};	/*Property = FileSaver*/
const Value V24532_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_306 = {S26, 0, &V24532_306_57238, &P24532_305};	/*Property = FileWriter*/
const Value V24532_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_307 = {S27, 0, &V24532_307_57241, &P24532_306};	/*Property = FormData*/
const Value V24532_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_309 = {S28, 0, &V24532_309_57247, &P24532_307};	/*Property = Fullscreen*/
const Value V24532_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_310 = {S29, 0, &V24532_310_57250, &P24532_309};	/*Property = GeoLocation*/
const Value V24532_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_312 = {S30, 0, &V24532_312_57256, &P24532_310};	/*Property = History*/
const Value V24532_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_314 = {S31, 0, &V24532_314_57262, &P24532_312};	/*Property = Html5*/
const Value V24532_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_313 = {S32, 0, &V24532_313_57259, &P24532_314};	/*Property = Html-Media-Capture*/
const Value V24532_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_315 = {S33, 0, &V24532_315_57265, &P24532_313};	/*Property = Iframe*/
const Value V24532_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_316 = {S34, 0, &V24532_316_57268, &P24532_315};	/*Property = IndexedDB*/
const Value V24532_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_317 = {S35, 0, &V24532_317_57271, &P24532_316};	/*Property = Json*/
const Value V24532_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24532_15 = {S37, 0, &V24532_15_28, &P24532_317};	/*Property = LayoutEngine*/
const Value V24532_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_344 = {S38, 0, &V24532_344_57274, &P24532_15};	/*Property = Masking*/
const Value V24532_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_320 = {S39, 0, &V24532_320_57292, &P24532_344};	/*Property = PostMessage*/
const Value V24532_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_321 = {S40, 0, &V24532_321_57295, &P24532_320};	/*Property = Progress*/
const Value V24532_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_322 = {S41, 0, &V24532_322_57298, &P24532_321};	/*Property = Prompts*/
const Value V24532_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_323 = {S42, 0, &V24532_323_57301, &P24532_322};	/*Property = Selector*/
const Value V24532_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_324 = {S43, 0, &V24532_324_57307, &P24532_323};	/*Property = Svg*/
const Value V24532_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_327 = {S44, 0, &V24532_327_57316, &P24532_324};	/*Property = TouchEvents*/
const Value V24532_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_328 = {S45, 0, &V24532_328_57319, &P24532_327};	/*Property = Track*/
const Value V24532_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_329 = {S46, 0, &V24532_329_57325, &P24532_328};	/*Property = Video*/
const Value V24532_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_330 = {S47, 0, &V24532_330_57328, &P24532_329};	/*Property = Viewport*/
const Value V24532_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_332 = {S48, 0, &V24532_332_57340, &P24532_330};	/*Property = WebWorkers*/
const Value V24532_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24532_333 = {S49, 0, &V24532_333_57343, &P24532_332};	/*Property = Xhr2*/
const Profile P24532 = {"24532", &P24532_333};	/*Profile ID - 24532*/
const Value V24155_1_1 = {S2, NULL};	/*Value = True*/
const Property P24155_1 = {S50, 0, &V24155_1_1, NULL};	/*Property = IsMobile*/
const Value V24155_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P24155_5 = {S52, 0, &V24155_5_37, &P24155_1};	/*Property = ScreenPixelsHeight*/
const Value V24155_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P24155_4 = {S54, 0, &V24155_4_36, &P24155_5};	/*Property = ScreenPixelsWidth*/
const Profile P24155 = {"24155", &P24155_4};	/*Profile ID - 24155*/
const Value V23698_1_1 = {S2, NULL};	/*Value = True*/
const Property P23698_1 = {S50, 0, &V23698_1_1, NULL};	/*Property = IsMobile*/
const Value V23698_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23698_5 = {S52, 0, &V23698_5_9981, &P23698_1};	/*Property = ScreenPixelsHeight*/
const Value V23698_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23698_4 = {S54, 0, &V23698_4_39098, &P23698_5};	/*Property = ScreenPixelsWidth*/
const Profile P23698 = {"23698", &P23698_4};	/*Profile ID - 23698*/
const Value V23655_279_57148 = {S0, NULL};	/*Value = False*/
const Property P23655_279 = {S1, 0, &V23655_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V23655_282_57154 = {S0, NULL};	/*Value = False*/
const Property P23655_282 = {S3, 0, &V23655_282_57154, &P23655_279};	/*Property = BlobBuilder*/
const Value V23655_350_57444 = {S2, NULL};	/*Value = True*/
const Property P23655_350 = {S4, 0, &V23655_350_57444, &P23655_282};	/*Property = Canvas*/
const Value V23655_283_57159 = {S2, NULL};	/*Value = True*/
const Property P23655_283 = {S5, 0, &V23655_283_57159, &P23655_350};	/*Property = CssBackground*/
const Value V23655_284_57163 = {S0, NULL};	/*Value = False*/
const Property P23655_284 = {S6, 0, &V23655_284_57163, &P23655_283};	/*Property = CssBorderImage*/
const Value V23655_285_57166 = {S0, NULL};	/*Value = False*/
const Property P23655_285 = {S7, 0, &V23655_285_57166, &P23655_284};	/*Property = CssCanvas*/
const Value V23655_286_57168 = {S2, NULL};	/*Value = True*/
const Property P23655_286 = {S8, 0, &V23655_286_57168, &P23655_285};	/*Property = CssColor*/
const Value V23655_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P23655_237 = {S9, 0, &V23655_237_55786, &P23655_286};	/*Property = CssColumn*/
const Value V23655_287_57172 = {S0, NULL};	/*Value = False*/
const Property P23655_287 = {S10, 0, &V23655_287_57172, &P23655_237};	/*Property = CssFlexbox*/
const Value V23655_288_57174 = {S2, NULL};	/*Value = True*/
const Property P23655_288 = {S11, 0, &V23655_288_57174, &P23655_287};	/*Property = CssFont*/
const Value V23655_289_57759 = {S0, NULL};	/*Value = False*/
const Property P23655_289 = {S12, 0, &V23655_289_57759, &P23655_288};	/*Property = CssImages*/
const Value V23655_291_57180 = {S2, NULL};	/*Value = True*/
const Property P23655_291 = {S13, 0, &V23655_291_57180, &P23655_289};	/*Property = CssMediaQueries*/
const Value V23655_292_57183 = {S2, NULL};	/*Value = True*/
const Property P23655_292 = {S14, 0, &V23655_292_57183, &P23655_291};	/*Property = CssMinMax*/
const Value V23655_293_57187 = {S0, NULL};	/*Value = False*/
const Property P23655_293 = {S15, 0, &V23655_293_57187, &P23655_292};	/*Property = CssOverflow*/
const Value V23655_294_57189 = {S2, NULL};	/*Value = True*/
const Property P23655_294 = {S16, 0, &V23655_294_57189, &P23655_293};	/*Property = CssPosition*/
const Value V23655_295_57192 = {S2, NULL};	/*Value = True*/
const Property P23655_295 = {S17, 0, &V23655_295_57192, &P23655_294};	/*Property = CssText*/
const Value V23655_297_57195 = {S2, NULL};	/*Value = True*/
const Property P23655_297 = {S18, 0, &V23655_297_57195, &P23655_295};	/*Property = CssTransforms*/
const Value V23655_241_55799 = {S2, NULL};	/*Value = True*/
const Property P23655_241 = {S19, 0, &V23655_241_55799, &P23655_297};	/*Property = CssTransitions*/
const Value V23655_298_57206 = {S2, NULL};	/*Value = True*/
const Property P23655_298 = {S20, 0, &V23655_298_57206, &P23655_241};	/*Property = CssUI*/
const Value V23655_300_57216 = {S0, NULL};	/*Value = False*/
const Property P23655_300 = {S21, 0, &V23655_300_57216, &P23655_298};	/*Property = DataSet*/
const Value V23655_301_57218 = {S2, NULL};	/*Value = True*/
const Property P23655_301 = {S22, 0, &V23655_301_57218, &P23655_300};	/*Property = DataUrl*/
const Value V23655_302_57221 = {S2, NULL};	/*Value = True*/
const Property P23655_302 = {S23, 0, &V23655_302_57221, &P23655_301};	/*Property = DeviceOrientation*/
const Value V23655_304_57227 = {S2, NULL};	/*Value = True*/
const Property P23655_304 = {S24, 0, &V23655_304_57227, &P23655_302};	/*Property = FileReader*/
const Value V23655_305_57231 = {S0, NULL};	/*Value = False*/
const Property P23655_305 = {S25, 0, &V23655_305_57231, &P23655_304};	/*Property = FileSaver*/
const Value V23655_306_57237 = {S0, NULL};	/*Value = False*/
const Property P23655_306 = {S26, 0, &V23655_306_57237, &P23655_305};	/*Property = FileWriter*/
const Value V23655_307_57239 = {S2, NULL};	/*Value = True*/
const Property P23655_307 = {S27, 0, &V23655_307_57239, &P23655_306};	/*Property = FormData*/
const Value V23655_309_57246 = {S0, NULL};	/*Value = False*/
const Property P23655_309 = {S28, 0, &V23655_309_57246, &P23655_307};	/*Property = Fullscreen*/
const Value V23655_310_57248 = {S2, NULL};	/*Value = True*/
const Property P23655_310 = {S29, 0, &V23655_310_57248, &P23655_309};	/*Property = GeoLocation*/
const Value V23655_312_57254 = {S2, NULL};	/*Value = True*/
const Property P23655_312 = {S30, 0, &V23655_312_57254, &P23655_310};	/*Property = History*/
const Value V23655_314_57261 = {S0, NULL};	/*Value = False*/
const Property P23655_314 = {S31, 0, &V23655_314_57261, &P23655_312};	/*Property = Html5*/
const Value V23655_313_57258 = {S0, NULL};	/*Value = False*/
const Property P23655_313 = {S32, 0, &V23655_313_57258, &P23655_314};	/*Property = Html-Media-Capture*/
const Value V23655_315_57264 = {S0, NULL};	/*Value = False*/
const Property P23655_315 = {S33, 0, &V23655_315_57264, &P23655_313};	/*Property = Iframe*/
const Value V23655_316_57266 = {S2, NULL};	/*Value = True*/
const Property P23655_316 = {S34, 0, &V23655_316_57266, &P23655_315};	/*Property = IndexedDB*/
const Value V23655_317_57269 = {S2, NULL};	/*Value = True*/
const Property P23655_317 = {S35, 0, &V23655_317_57269, &P23655_316};	/*Property = Json*/
const Value V23655_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P23655_15 = {S37, 0, &V23655_15_29, &P23655_317};	/*Property = LayoutEngine*/
const Value V23655_344_57272 = {S2, NULL};	/*Value = True*/
const Property P23655_344 = {S38, 0, &V23655_344_57272, &P23655_15};	/*Property = Masking*/
const Value V23655_320_57290 = {S2, NULL};	/*Value = True*/
const Property P23655_320 = {S39, 0, &V23655_320_57290, &P23655_344};	/*Property = PostMessage*/
const Value V23655_321_57293 = {S2, NULL};	/*Value = True*/
const Property P23655_321 = {S40, 0, &V23655_321_57293, &P23655_320};	/*Property = Progress*/
const Value V23655_322_57296 = {S2, NULL};	/*Value = True*/
const Property P23655_322 = {S41, 0, &V23655_322_57296, &P23655_321};	/*Property = Prompts*/
const Value V23655_323_57299 = {S2, NULL};	/*Value = True*/
const Property P23655_323 = {S42, 0, &V23655_323_57299, &P23655_322};	/*Property = Selector*/
const Value V23655_324_57306 = {S0, NULL};	/*Value = False*/
const Property P23655_324 = {S43, 0, &V23655_324_57306, &P23655_323};	/*Property = Svg*/
const Value V23655_327_57315 = {S0, NULL};	/*Value = False*/
const Property P23655_327 = {S44, 0, &V23655_327_57315, &P23655_324};	/*Property = TouchEvents*/
const Value V23655_328_57318 = {S0, NULL};	/*Value = False*/
const Property P23655_328 = {S45, 0, &V23655_328_57318, &P23655_327};	/*Property = Track*/
const Value V23655_329_57323 = {S2, NULL};	/*Value = True*/
const Property P23655_329 = {S46, 0, &V23655_329_57323, &P23655_328};	/*Property = Video*/
const Value V23655_330_57326 = {S2, NULL};	/*Value = True*/
const Property P23655_330 = {S47, 0, &V23655_330_57326, &P23655_329};	/*Property = Viewport*/
const Value V23655_332_57338 = {S2, NULL};	/*Value = True*/
const Property P23655_332 = {S48, 0, &V23655_332_57338, &P23655_330};	/*Property = WebWorkers*/
const Value V23655_333_57341 = {S2, NULL};	/*Value = True*/
const Property P23655_333 = {S49, 0, &V23655_333_57341, &P23655_332};	/*Property = Xhr2*/
const Profile P23655 = {"23655", &P23655_333};	/*Profile ID - 23655*/
const Value V23799_1_1 = {S2, NULL};	/*Value = True*/
const Property P23799_1 = {S50, 0, &V23799_1_1, NULL};	/*Property = IsMobile*/
const Value V23799_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23799_5 = {S52, 0, &V23799_5_1277, &P23799_1};	/*Property = ScreenPixelsHeight*/
const Value V23799_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23799_4 = {S54, 0, &V23799_4_1279, &P23799_5};	/*Property = ScreenPixelsWidth*/
const Profile P23799 = {"23799", &P23799_4};	/*Profile ID - 23799*/
const Value V23687_1_1 = {S2, NULL};	/*Value = True*/
const Property P23687_1 = {S50, 0, &V23687_1_1, NULL};	/*Property = IsMobile*/
const Value V23687_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23687_5 = {S52, 0, &V23687_5_9981, &P23687_1};	/*Property = ScreenPixelsHeight*/
const Value V23687_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P23687_4 = {S54, 0, &V23687_4_39098, &P23687_5};	/*Property = ScreenPixelsWidth*/
const Profile P23687 = {"23687", &P23687_4};	/*Profile ID - 23687*/
const Value V24157_1_1 = {S2, NULL};	/*Value = True*/
const Property P24157_1 = {S50, 0, &V24157_1_1, NULL};	/*Property = IsMobile*/
const Value V24157_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24157_5 = {S52, 0, &V24157_5_1277, &P24157_1};	/*Property = ScreenPixelsHeight*/
const Value V24157_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24157_4 = {S54, 0, &V24157_4_1279, &P24157_5};	/*Property = ScreenPixelsWidth*/
const Profile P24157 = {"24157", &P24157_4};	/*Profile ID - 24157*/
const Value V23967_1_1 = {S2, NULL};	/*Value = True*/
const Property P23967_1 = {S50, 0, &V23967_1_1, NULL};	/*Property = IsMobile*/
const Value V23967_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23967_5 = {S52, 0, &V23967_5_1227, &P23967_1};	/*Property = ScreenPixelsHeight*/
const Value V23967_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23967_4 = {S54, 0, &V23967_4_1229, &P23967_5};	/*Property = ScreenPixelsWidth*/
const Profile P23967 = {"23967", &P23967_4};	/*Profile ID - 23967*/
const Value V24737_1_1 = {S2, NULL};	/*Value = True*/
const Property P24737_1 = {S50, 0, &V24737_1_1, NULL};	/*Property = IsMobile*/
const Value V24737_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24737_5 = {S52, 0, &V24737_5_9981, &P24737_1};	/*Property = ScreenPixelsHeight*/
const Value V24737_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24737_4 = {S54, 0, &V24737_4_39098, &P24737_5};	/*Property = ScreenPixelsWidth*/
const Profile P24737 = {"24737", &P24737_4};	/*Profile ID - 24737*/
const Value V24564_1_1 = {S2, NULL};	/*Value = True*/
const Property P24564_1 = {S50, 0, &V24564_1_1, NULL};	/*Property = IsMobile*/
const Value V24564_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24564_5 = {S52, 0, &V24564_5_1403, &P24564_1};	/*Property = ScreenPixelsHeight*/
const Value V24564_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24564_4 = {S54, 0, &V24564_4_1404, &P24564_5};	/*Property = ScreenPixelsWidth*/
const Profile P24564 = {"24564", &P24564_4};	/*Profile ID - 24564*/
const Value V24692_1_1 = {S2, NULL};	/*Value = True*/
const Property P24692_1 = {S50, 0, &V24692_1_1, NULL};	/*Property = IsMobile*/
const Value V24692_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24692_5 = {S52, 0, &V24692_5_1277, &P24692_1};	/*Property = ScreenPixelsHeight*/
const Value V24692_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24692_4 = {S54, 0, &V24692_4_1279, &P24692_5};	/*Property = ScreenPixelsWidth*/
const Profile P24692 = {"24692", &P24692_4};	/*Profile ID - 24692*/
const Value V24097_1_1 = {S2, NULL};	/*Value = True*/
const Property P24097_1 = {S50, 0, &V24097_1_1, NULL};	/*Property = IsMobile*/
const Value V24097_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P24097_5 = {S52, 0, &V24097_5_59138, &P24097_1};	/*Property = ScreenPixelsHeight*/
const Value V24097_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P24097_4 = {S54, 0, &V24097_4_59970, &P24097_5};	/*Property = ScreenPixelsWidth*/
const Profile P24097 = {"24097", &P24097_4};	/*Profile ID - 24097*/
const Value V24539_1_1 = {S2, NULL};	/*Value = True*/
const Property P24539_1 = {S50, 0, &V24539_1_1, NULL};	/*Property = IsMobile*/
const Value V24539_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24539_5 = {S52, 0, &V24539_5_1227, &P24539_1};	/*Property = ScreenPixelsHeight*/
const Value V24539_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24539_4 = {S54, 0, &V24539_4_1229, &P24539_5};	/*Property = ScreenPixelsWidth*/
const Profile P24539 = {"24539", &P24539_4};	/*Profile ID - 24539*/
const Value V24102_1_1 = {S2, NULL};	/*Value = True*/
const Property P24102_1 = {S50, 0, &V24102_1_1, NULL};	/*Property = IsMobile*/
const Value V24102_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24102_5 = {S52, 0, &V24102_5_1403, &P24102_1};	/*Property = ScreenPixelsHeight*/
const Value V24102_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24102_4 = {S54, 0, &V24102_4_1404, &P24102_5};	/*Property = ScreenPixelsWidth*/
const Profile P24102 = {"24102", &P24102_4};	/*Profile ID - 24102*/
const Value V23956_1_1 = {S2, NULL};	/*Value = True*/
const Property P23956_1 = {S50, 0, &V23956_1_1, NULL};	/*Property = IsMobile*/
const Value V23956_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23956_5 = {S52, 0, &V23956_5_9981, &P23956_1};	/*Property = ScreenPixelsHeight*/
const Value V23956_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23956_4 = {S54, 0, &V23956_4_2336, &P23956_5};	/*Property = ScreenPixelsWidth*/
const Profile P23956 = {"23956", &P23956_4};	/*Profile ID - 23956*/
const Value V24169_1_1 = {S2, NULL};	/*Value = True*/
const Property P24169_1 = {S50, 0, &V24169_1_1, NULL};	/*Property = IsMobile*/
const Value V24169_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24169_5 = {S52, 0, &V24169_5_1403, &P24169_1};	/*Property = ScreenPixelsHeight*/
const Value V24169_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24169_4 = {S54, 0, &V24169_4_1404, &P24169_5};	/*Property = ScreenPixelsWidth*/
const Profile P24169 = {"24169", &P24169_4};	/*Profile ID - 24169*/
const Value V24160_1_1 = {S2, NULL};	/*Value = True*/
const Property P24160_1 = {S50, 0, &V24160_1_1, NULL};	/*Property = IsMobile*/
const Value V24160_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P24160_5 = {S52, 0, &V24160_5_1458, &P24160_1};	/*Property = ScreenPixelsHeight*/
const Value V24160_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P24160_4 = {S54, 0, &V24160_4_1460, &P24160_5};	/*Property = ScreenPixelsWidth*/
const Profile P24160 = {"24160", &P24160_4};	/*Profile ID - 24160*/
const Value V24488_1_1 = {S2, NULL};	/*Value = True*/
const Property P24488_1 = {S50, 0, &V24488_1_1, NULL};	/*Property = IsMobile*/
const Value V24488_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24488_5 = {S52, 0, &V24488_5_9981, &P24488_1};	/*Property = ScreenPixelsHeight*/
const Value V24488_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24488_4 = {S54, 0, &V24488_4_39098, &P24488_5};	/*Property = ScreenPixelsWidth*/
const Profile P24488 = {"24488", &P24488_4};	/*Profile ID - 24488*/
const Value V24048_1_1 = {S2, NULL};	/*Value = True*/
const Property P24048_1 = {S50, 0, &V24048_1_1, NULL};	/*Property = IsMobile*/
const Value V24048_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P24048_5 = {S52, 0, &V24048_5_59138, &P24048_1};	/*Property = ScreenPixelsHeight*/
const Value V24048_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P24048_4 = {S54, 0, &V24048_4_59970, &P24048_5};	/*Property = ScreenPixelsWidth*/
const Profile P24048 = {"24048", &P24048_4};	/*Profile ID - 24048*/
const Value V24366_1_1 = {S2, NULL};	/*Value = True*/
const Property P24366_1 = {S50, 0, &V24366_1_1, NULL};	/*Property = IsMobile*/
const Value V24366_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24366_5 = {S52, 0, &V24366_5_9981, &P24366_1};	/*Property = ScreenPixelsHeight*/
const Value V24366_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P24366_4 = {S54, 0, &V24366_4_2336, &P24366_5};	/*Property = ScreenPixelsWidth*/
const Profile P24366 = {"24366", &P24366_4};	/*Profile ID - 24366*/
const Value V23955_1_1 = {S2, NULL};	/*Value = True*/
const Property P23955_1 = {S50, 0, &V23955_1_1, NULL};	/*Property = IsMobile*/
const Value V23955_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P23955_5 = {S52, 0, &V23955_5_9981, &P23955_1};	/*Property = ScreenPixelsHeight*/
const Value V23955_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P23955_4 = {S54, 0, &V23955_4_2336, &P23955_5};	/*Property = ScreenPixelsWidth*/
const Profile P23955 = {"23955", &P23955_4};	/*Profile ID - 23955*/
const Value V24112_1_1 = {S2, NULL};	/*Value = True*/
const Property P24112_1 = {S50, 0, &V24112_1_1, NULL};	/*Property = IsMobile*/
const Value V24112_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24112_5 = {S52, 0, &V24112_5_9981, &P24112_1};	/*Property = ScreenPixelsHeight*/
const Value V24112_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24112_4 = {S54, 0, &V24112_4_39098, &P24112_5};	/*Property = ScreenPixelsWidth*/
const Profile P24112 = {"24112", &P24112_4};	/*Profile ID - 24112*/
const Value V24158_1_1 = {S2, NULL};	/*Value = True*/
const Property P24158_1 = {S50, 0, &V24158_1_1, NULL};	/*Property = IsMobile*/
const Value V24158_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24158_5 = {S52, 0, &V24158_5_1227, &P24158_1};	/*Property = ScreenPixelsHeight*/
const Value V24158_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24158_4 = {S54, 0, &V24158_4_1229, &P24158_5};	/*Property = ScreenPixelsWidth*/
const Profile P24158 = {"24158", &P24158_4};	/*Profile ID - 24158*/
const Value V24180_1_1 = {S2, NULL};	/*Value = True*/
const Property P24180_1 = {S50, 0, &V24180_1_1, NULL};	/*Property = IsMobile*/
const Value V24180_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24180_5 = {S52, 0, &V24180_5_1227, &P24180_1};	/*Property = ScreenPixelsHeight*/
const Value V24180_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24180_4 = {S54, 0, &V24180_4_1229, &P24180_5};	/*Property = ScreenPixelsWidth*/
const Profile P24180 = {"24180", &P24180_4};	/*Profile ID - 24180*/
const Value V24135_1_1 = {S2, NULL};	/*Value = True*/
const Property P24135_1 = {S50, 0, &V24135_1_1, NULL};	/*Property = IsMobile*/
const Value V24135_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24135_5 = {S52, 0, &V24135_5_1403, &P24135_1};	/*Property = ScreenPixelsHeight*/
const Value V24135_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24135_4 = {S54, 0, &V24135_4_1404, &P24135_5};	/*Property = ScreenPixelsWidth*/
const Profile P24135 = {"24135", &P24135_4};	/*Profile ID - 24135*/
const Value V24481_1_1 = {S2, NULL};	/*Value = True*/
const Property P24481_1 = {S50, 0, &V24481_1_1, NULL};	/*Property = IsMobile*/
const Value V24481_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24481_5 = {S52, 0, &V24481_5_1227, &P24481_1};	/*Property = ScreenPixelsHeight*/
const Value V24481_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24481_4 = {S54, 0, &V24481_4_1229, &P24481_5};	/*Property = ScreenPixelsWidth*/
const Profile P24481 = {"24481", &P24481_4};	/*Profile ID - 24481*/
const Value V24132_1_1 = {S2, NULL};	/*Value = True*/
const Property P24132_1 = {S50, 0, &V24132_1_1, NULL};	/*Property = IsMobile*/
const Value V24132_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24132_5 = {S52, 0, &V24132_5_9981, &P24132_1};	/*Property = ScreenPixelsHeight*/
const Value V24132_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24132_4 = {S54, 0, &V24132_4_39098, &P24132_5};	/*Property = ScreenPixelsWidth*/
const Profile P24132 = {"24132", &P24132_4};	/*Profile ID - 24132*/
const Value V24130_1_1 = {S2, NULL};	/*Value = True*/
const Property P24130_1 = {S50, 0, &V24130_1_1, NULL};	/*Property = IsMobile*/
const Value V24130_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P24130_5 = {S52, 0, &V24130_5_1458, &P24130_1};	/*Property = ScreenPixelsHeight*/
const Value V24130_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P24130_4 = {S54, 0, &V24130_4_1460, &P24130_5};	/*Property = ScreenPixelsWidth*/
const Profile P24130 = {"24130", &P24130_4};	/*Profile ID - 24130*/
const Value V24165_1_1 = {S2, NULL};	/*Value = True*/
const Property P24165_1 = {S50, 0, &V24165_1_1, NULL};	/*Property = IsMobile*/
const Value V24165_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P24165_5 = {S52, 0, &V24165_5_1458, &P24165_1};	/*Property = ScreenPixelsHeight*/
const Value V24165_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P24165_4 = {S54, 0, &V24165_4_1460, &P24165_5};	/*Property = ScreenPixelsWidth*/
const Profile P24165 = {"24165", &P24165_4};	/*Profile ID - 24165*/
const Value V24128_1_1 = {S2, NULL};	/*Value = True*/
const Property P24128_1 = {S50, 0, &V24128_1_1, NULL};	/*Property = IsMobile*/
const Value V24128_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24128_5 = {S52, 0, &V24128_5_9981, &P24128_1};	/*Property = ScreenPixelsHeight*/
const Value V24128_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24128_4 = {S54, 0, &V24128_4_39098, &P24128_5};	/*Property = ScreenPixelsWidth*/
const Profile P24128 = {"24128", &P24128_4};	/*Profile ID - 24128*/
const Value V24149_1_1 = {S2, NULL};	/*Value = True*/
const Property P24149_1 = {S50, 0, &V24149_1_1, NULL};	/*Property = IsMobile*/
const Value V24149_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24149_5 = {S52, 0, &V24149_5_1403, &P24149_1};	/*Property = ScreenPixelsHeight*/
const Value V24149_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24149_4 = {S54, 0, &V24149_4_1404, &P24149_5};	/*Property = ScreenPixelsWidth*/
const Profile P24149 = {"24149", &P24149_4};	/*Profile ID - 24149*/
const Value V23972_1_1 = {S2, NULL};	/*Value = True*/
const Property P23972_1 = {S50, 0, &V23972_1_1, NULL};	/*Property = IsMobile*/
const Value V23972_5_62105 = {S144, NULL};	/*Value = 2048*/
const Property P23972_5 = {S52, 0, &V23972_5_62105, &P23972_1};	/*Property = ScreenPixelsHeight*/
const Value V23972_4_62106 = {S145, NULL};	/*Value = 1536*/
const Property P23972_4 = {S54, 0, &V23972_4_62106, &P23972_5};	/*Property = ScreenPixelsWidth*/
const Profile P23972 = {"23972", &P23972_4};	/*Profile ID - 23972*/
const Value V23975_1_1 = {S2, NULL};	/*Value = True*/
const Property P23975_1 = {S50, 0, &V23975_1_1, NULL};	/*Property = IsMobile*/
const Value V23975_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P23975_5 = {S52, 0, &V23975_5_1458, &P23975_1};	/*Property = ScreenPixelsHeight*/
const Value V23975_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P23975_4 = {S54, 0, &V23975_4_1460, &P23975_5};	/*Property = ScreenPixelsWidth*/
const Profile P23975 = {"23975", &P23975_4};	/*Profile ID - 23975*/
const Value V23986_1_1 = {S2, NULL};	/*Value = True*/
const Property P23986_1 = {S50, 0, &V23986_1_1, NULL};	/*Property = IsMobile*/
const Value V23986_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P23986_5 = {S52, 0, &V23986_5_59138, &P23986_1};	/*Property = ScreenPixelsHeight*/
const Value V23986_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P23986_4 = {S54, 0, &V23986_4_59970, &P23986_5};	/*Property = ScreenPixelsWidth*/
const Profile P23986 = {"23986", &P23986_4};	/*Profile ID - 23986*/
const Value V24183_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24183_279 = {S1, 0, &V24183_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24183_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24183_282 = {S3, 0, &V24183_282_57154, &P24183_279};	/*Property = BlobBuilder*/
const Value V24183_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24183_350 = {S4, 0, &V24183_350_57444, &P24183_282};	/*Property = Canvas*/
const Value V24183_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24183_283 = {S5, 0, &V24183_283_57159, &P24183_350};	/*Property = CssBackground*/
const Value V24183_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24183_284 = {S6, 0, &V24183_284_57162, &P24183_283};	/*Property = CssBorderImage*/
const Value V24183_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24183_285 = {S7, 0, &V24183_285_57166, &P24183_284};	/*Property = CssCanvas*/
const Value V24183_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24183_286 = {S8, 0, &V24183_286_57168, &P24183_285};	/*Property = CssColor*/
const Value V24183_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24183_237 = {S9, 0, &V24183_237_55786, &P24183_286};	/*Property = CssColumn*/
const Value V24183_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24183_287 = {S10, 0, &V24183_287_57172, &P24183_237};	/*Property = CssFlexbox*/
const Value V24183_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24183_288 = {S11, 0, &V24183_288_57174, &P24183_287};	/*Property = CssFont*/
const Value V24183_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24183_289 = {S12, 0, &V24183_289_57759, &P24183_288};	/*Property = CssImages*/
const Value V24183_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24183_291 = {S13, 0, &V24183_291_57180, &P24183_289};	/*Property = CssMediaQueries*/
const Value V24183_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24183_292 = {S14, 0, &V24183_292_57183, &P24183_291};	/*Property = CssMinMax*/
const Value V24183_293_57186 = {S2, NULL};	/*Value = True*/
const Property P24183_293 = {S15, 0, &V24183_293_57186, &P24183_292};	/*Property = CssOverflow*/
const Value V24183_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24183_294 = {S16, 0, &V24183_294_57189, &P24183_293};	/*Property = CssPosition*/
const Value V24183_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24183_295 = {S17, 0, &V24183_295_57192, &P24183_294};	/*Property = CssText*/
const Value V24183_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24183_297 = {S18, 0, &V24183_297_57195, &P24183_295};	/*Property = CssTransforms*/
const Value V24183_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24183_241 = {S19, 0, &V24183_241_55799, &P24183_297};	/*Property = CssTransitions*/
const Value V24183_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24183_298 = {S20, 0, &V24183_298_57206, &P24183_241};	/*Property = CssUI*/
const Value V24183_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24183_300 = {S21, 0, &V24183_300_57215, &P24183_298};	/*Property = DataSet*/
const Value V24183_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24183_301 = {S22, 0, &V24183_301_57218, &P24183_300};	/*Property = DataUrl*/
const Value V24183_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24183_302 = {S23, 0, &V24183_302_57221, &P24183_301};	/*Property = DeviceOrientation*/
const Value V24183_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24183_304 = {S24, 0, &V24183_304_57227, &P24183_302};	/*Property = FileReader*/
const Value V24183_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24183_305 = {S25, 0, &V24183_305_57231, &P24183_304};	/*Property = FileSaver*/
const Value V24183_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24183_306 = {S26, 0, &V24183_306_57237, &P24183_305};	/*Property = FileWriter*/
const Value V24183_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24183_307 = {S27, 0, &V24183_307_57239, &P24183_306};	/*Property = FormData*/
const Value V24183_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24183_309 = {S28, 0, &V24183_309_57246, &P24183_307};	/*Property = Fullscreen*/
const Value V24183_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24183_310 = {S29, 0, &V24183_310_57248, &P24183_309};	/*Property = GeoLocation*/
const Value V24183_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24183_312 = {S30, 0, &V24183_312_57254, &P24183_310};	/*Property = History*/
const Value V24183_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24183_314 = {S31, 0, &V24183_314_57260, &P24183_312};	/*Property = Html5*/
const Value V24183_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24183_313 = {S32, 0, &V24183_313_57258, &P24183_314};	/*Property = Html-Media-Capture*/
const Value V24183_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24183_315 = {S33, 0, &V24183_315_57264, &P24183_313};	/*Property = Iframe*/
const Value V24183_316_57267 = {S0, NULL};	/*Value = False*/
const Property P24183_316 = {S34, 0, &V24183_316_57267, &P24183_315};	/*Property = IndexedDB*/
const Value V24183_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24183_317 = {S35, 0, &V24183_317_57269, &P24183_316};	/*Property = Json*/
const Value V24183_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24183_15 = {S37, 0, &V24183_15_28, &P24183_317};	/*Property = LayoutEngine*/
const Value V24183_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24183_344 = {S38, 0, &V24183_344_57272, &P24183_15};	/*Property = Masking*/
const Value V24183_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24183_320 = {S39, 0, &V24183_320_57290, &P24183_344};	/*Property = PostMessage*/
const Value V24183_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24183_321 = {S40, 0, &V24183_321_57293, &P24183_320};	/*Property = Progress*/
const Value V24183_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24183_322 = {S41, 0, &V24183_322_57296, &P24183_321};	/*Property = Prompts*/
const Value V24183_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24183_323 = {S42, 0, &V24183_323_57299, &P24183_322};	/*Property = Selector*/
const Value V24183_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24183_324 = {S43, 0, &V24183_324_57305, &P24183_323};	/*Property = Svg*/
const Value V24183_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24183_327 = {S44, 0, &V24183_327_57314, &P24183_324};	/*Property = TouchEvents*/
const Value V24183_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24183_328 = {S45, 0, &V24183_328_57318, &P24183_327};	/*Property = Track*/
const Value V24183_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24183_329 = {S46, 0, &V24183_329_57323, &P24183_328};	/*Property = Video*/
const Value V24183_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24183_330 = {S47, 0, &V24183_330_57326, &P24183_329};	/*Property = Viewport*/
const Value V24183_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24183_332 = {S48, 0, &V24183_332_57339, &P24183_330};	/*Property = WebWorkers*/
const Value V24183_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24183_333 = {S49, 0, &V24183_333_57342, &P24183_332};	/*Property = Xhr2*/
const Profile P24183 = {"24183", &P24183_333};	/*Profile ID - 24183*/
const Value V24464_1_1 = {S2, NULL};	/*Value = True*/
const Property P24464_1 = {S50, 0, &V24464_1_1, NULL};	/*Property = IsMobile*/
const Value V24464_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24464_5 = {S52, 0, &V24464_5_1227, &P24464_1};	/*Property = ScreenPixelsHeight*/
const Value V24464_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24464_4 = {S54, 0, &V24464_4_1229, &P24464_5};	/*Property = ScreenPixelsWidth*/
const Profile P24464 = {"24464", &P24464_4};	/*Profile ID - 24464*/
const Value V13127_1_1 = {S2, NULL};	/*Value = True*/
const Property P13127_1 = {S50, 0, &V13127_1_1, NULL};	/*Property = IsMobile*/
const Value V13127_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P13127_5 = {S52, 0, &V13127_5_1222, &P13127_1};	/*Property = ScreenPixelsHeight*/
const Value V13127_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P13127_4 = {S54, 0, &V13127_4_1224, &P13127_5};	/*Property = ScreenPixelsWidth*/
const Profile P13127 = {"13127", &P13127_4};	/*Profile ID - 13127*/
const Value V24577_1_1 = {S2, NULL};	/*Value = True*/
const Property P24577_1 = {S50, 0, &V24577_1_1, NULL};	/*Property = IsMobile*/
const Value V24577_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24577_5 = {S52, 0, &V24577_5_1227, &P24577_1};	/*Property = ScreenPixelsHeight*/
const Value V24577_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24577_4 = {S54, 0, &V24577_4_1229, &P24577_5};	/*Property = ScreenPixelsWidth*/
const Profile P24577 = {"24577", &P24577_4};	/*Profile ID - 24577*/
const Value V24540_1_1 = {S2, NULL};	/*Value = True*/
const Property P24540_1 = {S50, 0, &V24540_1_1, NULL};	/*Property = IsMobile*/
const Value V24540_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24540_5 = {S52, 0, &V24540_5_1277, &P24540_1};	/*Property = ScreenPixelsHeight*/
const Value V24540_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24540_4 = {S54, 0, &V24540_4_1279, &P24540_5};	/*Property = ScreenPixelsWidth*/
const Profile P24540 = {"24540", &P24540_4};	/*Profile ID - 24540*/
const Value V24508_1_1 = {S2, NULL};	/*Value = True*/
const Property P24508_1 = {S50, 0, &V24508_1_1, NULL};	/*Property = IsMobile*/
const Value V24508_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24508_5 = {S52, 0, &V24508_5_1277, &P24508_1};	/*Property = ScreenPixelsHeight*/
const Value V24508_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24508_4 = {S54, 0, &V24508_4_1279, &P24508_5};	/*Property = ScreenPixelsWidth*/
const Profile P24508 = {"24508", &P24508_4};	/*Profile ID - 24508*/
const Value V24557_1_1 = {S2, NULL};	/*Value = True*/
const Property P24557_1 = {S50, 0, &V24557_1_1, NULL};	/*Property = IsMobile*/
const Value V24557_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24557_5 = {S52, 0, &V24557_5_1227, &P24557_1};	/*Property = ScreenPixelsHeight*/
const Value V24557_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24557_4 = {S54, 0, &V24557_4_1229, &P24557_5};	/*Property = ScreenPixelsWidth*/
const Profile P24557 = {"24557", &P24557_4};	/*Profile ID - 24557*/
const Value V24044_1_1 = {S2, NULL};	/*Value = True*/
const Property P24044_1 = {S50, 0, &V24044_1_1, NULL};	/*Property = IsMobile*/
const Value V24044_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24044_5 = {S52, 0, &V24044_5_1277, &P24044_1};	/*Property = ScreenPixelsHeight*/
const Value V24044_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24044_4 = {S54, 0, &V24044_4_1279, &P24044_5};	/*Property = ScreenPixelsWidth*/
const Profile P24044 = {"24044", &P24044_4};	/*Profile ID - 24044*/
const Value V24045_1_1 = {S2, NULL};	/*Value = True*/
const Property P24045_1 = {S50, 0, &V24045_1_1, NULL};	/*Property = IsMobile*/
const Value V24045_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24045_5 = {S52, 0, &V24045_5_9981, &P24045_1};	/*Property = ScreenPixelsHeight*/
const Value V24045_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24045_4 = {S54, 0, &V24045_4_39098, &P24045_5};	/*Property = ScreenPixelsWidth*/
const Profile P24045 = {"24045", &P24045_4};	/*Profile ID - 24045*/
const Value V24482_1_1 = {S2, NULL};	/*Value = True*/
const Property P24482_1 = {S50, 0, &V24482_1_1, NULL};	/*Property = IsMobile*/
const Value V24482_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24482_5 = {S52, 0, &V24482_5_1277, &P24482_1};	/*Property = ScreenPixelsHeight*/
const Value V24482_4_1278 = {S64, NULL};	/*Value = 360*/
const Property P24482_4 = {S54, 0, &V24482_4_1278, &P24482_5};	/*Property = ScreenPixelsWidth*/
const Profile P24482 = {"24482", &P24482_4};	/*Profile ID - 24482*/
const Value V24368_1_1 = {S2, NULL};	/*Value = True*/
const Property P24368_1 = {S50, 0, &V24368_1_1, NULL};	/*Property = IsMobile*/
const Value V24368_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24368_5 = {S52, 0, &V24368_5_1227, &P24368_1};	/*Property = ScreenPixelsHeight*/
const Value V24368_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24368_4 = {S54, 0, &V24368_4_1229, &P24368_5};	/*Property = ScreenPixelsWidth*/
const Profile P24368 = {"24368", &P24368_4};	/*Profile ID - 24368*/
const Value V24559_1_1 = {S2, NULL};	/*Value = True*/
const Property P24559_1 = {S50, 0, &V24559_1_1, NULL};	/*Property = IsMobile*/
const Value V24559_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24559_5 = {S52, 0, &V24559_5_1277, &P24559_1};	/*Property = ScreenPixelsHeight*/
const Value V24559_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24559_4 = {S54, 0, &V24559_4_1279, &P24559_5};	/*Property = ScreenPixelsWidth*/
const Profile P24559 = {"24559", &P24559_4};	/*Profile ID - 24559*/
const Value V24365_1_1 = {S2, NULL};	/*Value = True*/
const Property P24365_1 = {S50, 0, &V24365_1_1, NULL};	/*Property = IsMobile*/
const Value V24365_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P24365_5 = {S52, 0, &V24365_5_1458, &P24365_1};	/*Property = ScreenPixelsHeight*/
const Value V24365_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P24365_4 = {S54, 0, &V24365_4_1460, &P24365_5};	/*Property = ScreenPixelsWidth*/
const Profile P24365 = {"24365", &P24365_4};	/*Profile ID - 24365*/
const Value V24688_1_1 = {S2, NULL};	/*Value = True*/
const Property P24688_1 = {S50, 0, &V24688_1_1, NULL};	/*Property = IsMobile*/
const Value V24688_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24688_5 = {S52, 0, &V24688_5_1403, &P24688_1};	/*Property = ScreenPixelsHeight*/
const Value V24688_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24688_4 = {S54, 0, &V24688_4_1404, &P24688_5};	/*Property = ScreenPixelsWidth*/
const Profile P24688 = {"24688", &P24688_4};	/*Profile ID - 24688*/
const Value V24700_1_1 = {S2, NULL};	/*Value = True*/
const Property P24700_1 = {S50, 0, &V24700_1_1, NULL};	/*Property = IsMobile*/
const Value V24700_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24700_5 = {S52, 0, &V24700_5_1403, &P24700_1};	/*Property = ScreenPixelsHeight*/
const Value V24700_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24700_4 = {S54, 0, &V24700_4_1404, &P24700_5};	/*Property = ScreenPixelsWidth*/
const Profile P24700 = {"24700", &P24700_4};	/*Profile ID - 24700*/
const Value V24039_1_1 = {S2, NULL};	/*Value = True*/
const Property P24039_1 = {S50, 0, &V24039_1_1, NULL};	/*Property = IsMobile*/
const Value V24039_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24039_5 = {S52, 0, &V24039_5_1403, &P24039_1};	/*Property = ScreenPixelsHeight*/
const Value V24039_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24039_4 = {S54, 0, &V24039_4_1404, &P24039_5};	/*Property = ScreenPixelsWidth*/
const Profile P24039 = {"24039", &P24039_4};	/*Profile ID - 24039*/
const Value V24127_1_1 = {S2, NULL};	/*Value = True*/
const Property P24127_1 = {S50, 0, &V24127_1_1, NULL};	/*Property = IsMobile*/
const Value V24127_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P24127_5 = {S52, 0, &V24127_5_59138, &P24127_1};	/*Property = ScreenPixelsHeight*/
const Value V24127_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P24127_4 = {S54, 0, &V24127_4_59970, &P24127_5};	/*Property = ScreenPixelsWidth*/
const Profile P24127 = {"24127", &P24127_4};	/*Profile ID - 24127*/
const Value V24148_1_1 = {S2, NULL};	/*Value = True*/
const Property P24148_1 = {S50, 0, &V24148_1_1, NULL};	/*Property = IsMobile*/
const Value V24148_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24148_5 = {S52, 0, &V24148_5_1403, &P24148_1};	/*Property = ScreenPixelsHeight*/
const Value V24148_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24148_4 = {S54, 0, &V24148_4_1404, &P24148_5};	/*Property = ScreenPixelsWidth*/
const Profile P24148 = {"24148", &P24148_4};	/*Profile ID - 24148*/
const Value V24114_1_1 = {S2, NULL};	/*Value = True*/
const Property P24114_1 = {S50, 0, &V24114_1_1, NULL};	/*Property = IsMobile*/
const Value V24114_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24114_5 = {S52, 0, &V24114_5_1277, &P24114_1};	/*Property = ScreenPixelsHeight*/
const Value V24114_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24114_4 = {S54, 0, &V24114_4_1279, &P24114_5};	/*Property = ScreenPixelsWidth*/
const Profile P24114 = {"24114", &P24114_4};	/*Profile ID - 24114*/
const Value V24171_1_1 = {S2, NULL};	/*Value = True*/
const Property P24171_1 = {S50, 0, &V24171_1_1, NULL};	/*Property = IsMobile*/
const Value V24171_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24171_5 = {S52, 0, &V24171_5_1277, &P24171_1};	/*Property = ScreenPixelsHeight*/
const Value V24171_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24171_4 = {S54, 0, &V24171_4_1279, &P24171_5};	/*Property = ScreenPixelsWidth*/
const Profile P24171 = {"24171", &P24171_4};	/*Profile ID - 24171*/
const Value V24066_1_1 = {S2, NULL};	/*Value = True*/
const Property P24066_1 = {S50, 0, &V24066_1_1, NULL};	/*Property = IsMobile*/
const Value V24066_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P24066_5 = {S52, 0, &V24066_5_1269, &P24066_1};	/*Property = ScreenPixelsHeight*/
const Value V24066_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24066_4 = {S54, 0, &V24066_4_1229, &P24066_5};	/*Property = ScreenPixelsWidth*/
const Profile P24066 = {"24066", &P24066_4};	/*Profile ID - 24066*/
const Value V24638_1_1 = {S2, NULL};	/*Value = True*/
const Property P24638_1 = {S50, 0, &V24638_1_1, NULL};	/*Property = IsMobile*/
const Value V24638_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24638_5 = {S52, 0, &V24638_5_1403, &P24638_1};	/*Property = ScreenPixelsHeight*/
const Value V24638_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24638_4 = {S54, 0, &V24638_4_1404, &P24638_5};	/*Property = ScreenPixelsWidth*/
const Profile P24638 = {"24638", &P24638_4};	/*Profile ID - 24638*/
const Value V24133_1_1 = {S2, NULL};	/*Value = True*/
const Property P24133_1 = {S50, 0, &V24133_1_1, NULL};	/*Property = IsMobile*/
const Value V24133_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P24133_5 = {S52, 0, &V24133_5_1458, &P24133_1};	/*Property = ScreenPixelsHeight*/
const Value V24133_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P24133_4 = {S54, 0, &V24133_4_37617, &P24133_5};	/*Property = ScreenPixelsWidth*/
const Profile P24133 = {"24133", &P24133_4};	/*Profile ID - 24133*/
const Value V24095_1_1 = {S2, NULL};	/*Value = True*/
const Property P24095_1 = {S50, 0, &V24095_1_1, NULL};	/*Property = IsMobile*/
const Value V24095_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24095_5 = {S52, 0, &V24095_5_9981, &P24095_1};	/*Property = ScreenPixelsHeight*/
const Value V24095_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24095_4 = {S54, 0, &V24095_4_39098, &P24095_5};	/*Property = ScreenPixelsWidth*/
const Profile P24095 = {"24095", &P24095_4};	/*Profile ID - 24095*/
const Value V24587_1_1 = {S2, NULL};	/*Value = True*/
const Property P24587_1 = {S50, 0, &V24587_1_1, NULL};	/*Property = IsMobile*/
const Value V24587_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24587_5 = {S52, 0, &V24587_5_1227, &P24587_1};	/*Property = ScreenPixelsHeight*/
const Value V24587_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24587_4 = {S54, 0, &V24587_4_1229, &P24587_5};	/*Property = ScreenPixelsWidth*/
const Profile P24587 = {"24587", &P24587_4};	/*Profile ID - 24587*/
const Value V24068_1_1 = {S2, NULL};	/*Value = True*/
const Property P24068_1 = {S50, 0, &V24068_1_1, NULL};	/*Property = IsMobile*/
const Value V24068_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24068_5 = {S52, 0, &V24068_5_9981, &P24068_1};	/*Property = ScreenPixelsHeight*/
const Value V24068_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24068_4 = {S54, 0, &V24068_4_39098, &P24068_5};	/*Property = ScreenPixelsWidth*/
const Profile P24068 = {"24068", &P24068_4};	/*Profile ID - 24068*/
const Value V24724_1_1 = {S2, NULL};	/*Value = True*/
const Property P24724_1 = {S50, 0, &V24724_1_1, NULL};	/*Property = IsMobile*/
const Value V24724_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24724_5 = {S52, 0, &V24724_5_1277, &P24724_1};	/*Property = ScreenPixelsHeight*/
const Value V24724_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24724_4 = {S54, 0, &V24724_4_1279, &P24724_5};	/*Property = ScreenPixelsWidth*/
const Profile P24724 = {"24724", &P24724_4};	/*Profile ID - 24724*/
const Value V24655_1_1 = {S2, NULL};	/*Value = True*/
const Property P24655_1 = {S50, 0, &V24655_1_1, NULL};	/*Property = IsMobile*/
const Value V24655_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P24655_5 = {S52, 0, &V24655_5_1832, &P24655_1};	/*Property = ScreenPixelsHeight*/
const Value V24655_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P24655_4 = {S54, 0, &V24655_4_5645, &P24655_5};	/*Property = ScreenPixelsWidth*/
const Profile P24655 = {"24655", &P24655_4};	/*Profile ID - 24655*/
const Value V24049_1_1 = {S2, NULL};	/*Value = True*/
const Property P24049_1 = {S50, 0, &V24049_1_1, NULL};	/*Property = IsMobile*/
const Value V24049_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P24049_5 = {S52, 0, &V24049_5_1269, &P24049_1};	/*Property = ScreenPixelsHeight*/
const Value V24049_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24049_4 = {S54, 0, &V24049_4_1229, &P24049_5};	/*Property = ScreenPixelsWidth*/
const Profile P24049 = {"24049", &P24049_4};	/*Profile ID - 24049*/
const Value V24105_1_1 = {S2, NULL};	/*Value = True*/
const Property P24105_1 = {S50, 0, &V24105_1_1, NULL};	/*Property = IsMobile*/
const Value V24105_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P24105_5 = {S52, 0, &V24105_5_37569, &P24105_1};	/*Property = ScreenPixelsHeight*/
const Value V24105_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P24105_4 = {S54, 0, &V24105_4_5645, &P24105_5};	/*Property = ScreenPixelsWidth*/
const Profile P24105 = {"24105", &P24105_4};	/*Profile ID - 24105*/
const Value V24042_1_1 = {S2, NULL};	/*Value = True*/
const Property P24042_1 = {S50, 0, &V24042_1_1, NULL};	/*Property = IsMobile*/
const Value V24042_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P24042_5 = {S52, 0, &V24042_5_1832, &P24042_1};	/*Property = ScreenPixelsHeight*/
const Value V24042_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P24042_4 = {S54, 0, &V24042_4_5645, &P24042_5};	/*Property = ScreenPixelsWidth*/
const Profile P24042 = {"24042", &P24042_4};	/*Profile ID - 24042*/
const Value V24041_1_1 = {S2, NULL};	/*Value = True*/
const Property P24041_1 = {S50, 0, &V24041_1_1, NULL};	/*Property = IsMobile*/
const Value V24041_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24041_5 = {S52, 0, &V24041_5_1277, &P24041_1};	/*Property = ScreenPixelsHeight*/
const Value V24041_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24041_4 = {S54, 0, &V24041_4_1279, &P24041_5};	/*Property = ScreenPixelsWidth*/
const Profile P24041 = {"24041", &P24041_4};	/*Profile ID - 24041*/
const Value V24108_1_1 = {S2, NULL};	/*Value = True*/
const Property P24108_1 = {S50, 0, &V24108_1_1, NULL};	/*Property = IsMobile*/
const Value V24108_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P24108_5 = {S52, 0, &V24108_5_59138, &P24108_1};	/*Property = ScreenPixelsHeight*/
const Value V24108_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P24108_4 = {S54, 0, &V24108_4_59970, &P24108_5};	/*Property = ScreenPixelsWidth*/
const Profile P24108 = {"24108", &P24108_4};	/*Profile ID - 24108*/
const Value V24106_1_1 = {S2, NULL};	/*Value = True*/
const Property P24106_1 = {S50, 0, &V24106_1_1, NULL};	/*Property = IsMobile*/
const Value V24106_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P24106_5 = {S52, 0, &V24106_5_1832, &P24106_1};	/*Property = ScreenPixelsHeight*/
const Value V24106_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P24106_4 = {S54, 0, &V24106_4_5645, &P24106_5};	/*Property = ScreenPixelsWidth*/
const Profile P24106 = {"24106", &P24106_4};	/*Profile ID - 24106*/
const Value V24052_1_1 = {S2, NULL};	/*Value = True*/
const Property P24052_1 = {S50, 0, &V24052_1_1, NULL};	/*Property = IsMobile*/
const Value V24052_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24052_5 = {S52, 0, &V24052_5_9981, &P24052_1};	/*Property = ScreenPixelsHeight*/
const Value V24052_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24052_4 = {S54, 0, &V24052_4_39098, &P24052_5};	/*Property = ScreenPixelsWidth*/
const Profile P24052 = {"24052", &P24052_4};	/*Profile ID - 24052*/
const Value V24063_1_1 = {S2, NULL};	/*Value = True*/
const Property P24063_1 = {S50, 0, &V24063_1_1, NULL};	/*Property = IsMobile*/
const Value V24063_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24063_5 = {S52, 0, &V24063_5_1227, &P24063_1};	/*Property = ScreenPixelsHeight*/
const Value V24063_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P24063_4 = {S54, 0, &V24063_4_1477, &P24063_5};	/*Property = ScreenPixelsWidth*/
const Profile P24063 = {"24063", &P24063_4};	/*Profile ID - 24063*/
const Value V24109_1_1 = {S2, NULL};	/*Value = True*/
const Property P24109_1 = {S50, 0, &V24109_1_1, NULL};	/*Property = IsMobile*/
const Value V24109_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24109_5 = {S52, 0, &V24109_5_9981, &P24109_1};	/*Property = ScreenPixelsHeight*/
const Value V24109_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24109_4 = {S54, 0, &V24109_4_39098, &P24109_5};	/*Property = ScreenPixelsWidth*/
const Profile P24109 = {"24109", &P24109_4};	/*Profile ID - 24109*/
const Value V24062_1_1 = {S2, NULL};	/*Value = True*/
const Property P24062_1 = {S50, 0, &V24062_1_1, NULL};	/*Property = IsMobile*/
const Value V24062_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24062_5 = {S52, 0, &V24062_5_1277, &P24062_1};	/*Property = ScreenPixelsHeight*/
const Value V24062_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24062_4 = {S54, 0, &V24062_4_1279, &P24062_5};	/*Property = ScreenPixelsWidth*/
const Profile P24062 = {"24062", &P24062_4};	/*Profile ID - 24062*/
const Value V24134_1_1 = {S2, NULL};	/*Value = True*/
const Property P24134_1 = {S50, 0, &V24134_1_1, NULL};	/*Property = IsMobile*/
const Value V24134_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24134_5 = {S52, 0, &V24134_5_1277, &P24134_1};	/*Property = ScreenPixelsHeight*/
const Value V24134_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24134_4 = {S54, 0, &V24134_4_1279, &P24134_5};	/*Property = ScreenPixelsWidth*/
const Profile P24134 = {"24134", &P24134_4};	/*Profile ID - 24134*/
const Value V24053_1_1 = {S2, NULL};	/*Value = True*/
const Property P24053_1 = {S50, 0, &V24053_1_1, NULL};	/*Property = IsMobile*/
const Value V24053_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24053_5 = {S52, 0, &V24053_5_9981, &P24053_1};	/*Property = ScreenPixelsHeight*/
const Value V24053_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24053_4 = {S54, 0, &V24053_4_39098, &P24053_5};	/*Property = ScreenPixelsWidth*/
const Profile P24053 = {"24053", &P24053_4};	/*Profile ID - 24053*/
const Value V24075_1_1 = {S2, NULL};	/*Value = True*/
const Property P24075_1 = {S50, 0, &V24075_1_1, NULL};	/*Property = IsMobile*/
const Value V24075_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P24075_5 = {S52, 0, &V24075_5_1269, &P24075_1};	/*Property = ScreenPixelsHeight*/
const Value V24075_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24075_4 = {S54, 0, &V24075_4_1229, &P24075_5};	/*Property = ScreenPixelsWidth*/
const Profile P24075 = {"24075", &P24075_4};	/*Profile ID - 24075*/
const Value V24156_1_1 = {S2, NULL};	/*Value = True*/
const Property P24156_1 = {S50, 0, &V24156_1_1, NULL};	/*Property = IsMobile*/
const Value V24156_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P24156_5 = {S52, 0, &V24156_5_59138, &P24156_1};	/*Property = ScreenPixelsHeight*/
const Value V24156_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P24156_4 = {S54, 0, &V24156_4_59970, &P24156_5};	/*Property = ScreenPixelsWidth*/
const Profile P24156 = {"24156", &P24156_4};	/*Profile ID - 24156*/
const Value V24076_1_1 = {S2, NULL};	/*Value = True*/
const Property P24076_1 = {S50, 0, &V24076_1_1, NULL};	/*Property = IsMobile*/
const Value V24076_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24076_5 = {S52, 0, &V24076_5_1277, &P24076_1};	/*Property = ScreenPixelsHeight*/
const Value V24076_4_37618 = {S83, NULL};	/*Value = 272*/
const Property P24076_4 = {S54, 0, &V24076_4_37618, &P24076_5};	/*Property = ScreenPixelsWidth*/
const Profile P24076 = {"24076", &P24076_4};	/*Profile ID - 24076*/
const Value V24176_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24176_279 = {S1, 0, &V24176_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24176_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24176_282 = {S3, 0, &V24176_282_57154, &P24176_279};	/*Property = BlobBuilder*/
const Value V24176_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24176_350 = {S4, 0, &V24176_350_57444, &P24176_282};	/*Property = Canvas*/
const Value V24176_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24176_283 = {S5, 0, &V24176_283_57159, &P24176_350};	/*Property = CssBackground*/
const Value V24176_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24176_284 = {S6, 0, &V24176_284_57162, &P24176_283};	/*Property = CssBorderImage*/
const Value V24176_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24176_285 = {S7, 0, &V24176_285_57166, &P24176_284};	/*Property = CssCanvas*/
const Value V24176_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24176_286 = {S8, 0, &V24176_286_57168, &P24176_285};	/*Property = CssColor*/
const Value V24176_237_55784 = {S2, NULL};	/*Value = True*/
const Property P24176_237 = {S9, 0, &V24176_237_55784, &P24176_286};	/*Property = CssColumn*/
const Value V24176_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24176_287 = {S10, 0, &V24176_287_57172, &P24176_237};	/*Property = CssFlexbox*/
const Value V24176_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24176_288 = {S11, 0, &V24176_288_57174, &P24176_287};	/*Property = CssFont*/
const Value V24176_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24176_289 = {S12, 0, &V24176_289_57759, &P24176_288};	/*Property = CssImages*/
const Value V24176_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24176_291 = {S13, 0, &V24176_291_57180, &P24176_289};	/*Property = CssMediaQueries*/
const Value V24176_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24176_292 = {S14, 0, &V24176_292_57183, &P24176_291};	/*Property = CssMinMax*/
const Value V24176_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24176_293 = {S15, 0, &V24176_293_57187, &P24176_292};	/*Property = CssOverflow*/
const Value V24176_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24176_294 = {S16, 0, &V24176_294_57189, &P24176_293};	/*Property = CssPosition*/
const Value V24176_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24176_295 = {S17, 0, &V24176_295_57192, &P24176_294};	/*Property = CssText*/
const Value V24176_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24176_297 = {S18, 0, &V24176_297_57195, &P24176_295};	/*Property = CssTransforms*/
const Value V24176_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24176_241 = {S19, 0, &V24176_241_55799, &P24176_297};	/*Property = CssTransitions*/
const Value V24176_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24176_298 = {S20, 0, &V24176_298_57206, &P24176_241};	/*Property = CssUI*/
const Value V24176_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24176_300 = {S21, 0, &V24176_300_57215, &P24176_298};	/*Property = DataSet*/
const Value V24176_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24176_301 = {S22, 0, &V24176_301_57218, &P24176_300};	/*Property = DataUrl*/
const Value V24176_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24176_302 = {S23, 0, &V24176_302_57222, &P24176_301};	/*Property = DeviceOrientation*/
const Value V24176_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24176_304 = {S24, 0, &V24176_304_57227, &P24176_302};	/*Property = FileReader*/
const Value V24176_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24176_305 = {S25, 0, &V24176_305_57231, &P24176_304};	/*Property = FileSaver*/
const Value V24176_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24176_306 = {S26, 0, &V24176_306_57237, &P24176_305};	/*Property = FileWriter*/
const Value V24176_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24176_307 = {S27, 0, &V24176_307_57239, &P24176_306};	/*Property = FormData*/
const Value V24176_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24176_309 = {S28, 0, &V24176_309_57246, &P24176_307};	/*Property = Fullscreen*/
const Value V24176_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24176_310 = {S29, 0, &V24176_310_57248, &P24176_309};	/*Property = GeoLocation*/
const Value V24176_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24176_312 = {S30, 0, &V24176_312_57254, &P24176_310};	/*Property = History*/
const Value V24176_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24176_314 = {S31, 0, &V24176_314_57260, &P24176_312};	/*Property = Html5*/
const Value V24176_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24176_313 = {S32, 0, &V24176_313_57258, &P24176_314};	/*Property = Html-Media-Capture*/
const Value V24176_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24176_315 = {S33, 0, &V24176_315_57264, &P24176_313};	/*Property = Iframe*/
const Value V24176_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24176_316 = {S34, 0, &V24176_316_57266, &P24176_315};	/*Property = IndexedDB*/
const Value V24176_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24176_317 = {S35, 0, &V24176_317_57269, &P24176_316};	/*Property = Json*/
const Value V24176_15_81502 = {S141, NULL};	/*Value = Blink*/
const Property P24176_15 = {S37, 0, &V24176_15_81502, &P24176_317};	/*Property = LayoutEngine*/
const Value V24176_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24176_344 = {S38, 0, &V24176_344_57272, &P24176_15};	/*Property = Masking*/
const Value V24176_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24176_320 = {S39, 0, &V24176_320_57290, &P24176_344};	/*Property = PostMessage*/
const Value V24176_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24176_321 = {S40, 0, &V24176_321_57293, &P24176_320};	/*Property = Progress*/
const Value V24176_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24176_322 = {S41, 0, &V24176_322_57296, &P24176_321};	/*Property = Prompts*/
const Value V24176_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24176_323 = {S42, 0, &V24176_323_57299, &P24176_322};	/*Property = Selector*/
const Value V24176_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24176_324 = {S43, 0, &V24176_324_57305, &P24176_323};	/*Property = Svg*/
const Value V24176_327_57315 = {S0, NULL};	/*Value = False*/
const Property P24176_327 = {S44, 0, &V24176_327_57315, &P24176_324};	/*Property = TouchEvents*/
const Value V24176_328_57317 = {S2, NULL};	/*Value = True*/
const Property P24176_328 = {S45, 0, &V24176_328_57317, &P24176_327};	/*Property = Track*/
const Value V24176_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24176_329 = {S46, 0, &V24176_329_57323, &P24176_328};	/*Property = Video*/
const Value V24176_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24176_330 = {S47, 0, &V24176_330_57326, &P24176_329};	/*Property = Viewport*/
const Value V24176_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24176_332 = {S48, 0, &V24176_332_57338, &P24176_330};	/*Property = WebWorkers*/
const Value V24176_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24176_333 = {S49, 0, &V24176_333_57342, &P24176_332};	/*Property = Xhr2*/
const Profile P24176 = {"24176", &P24176_333};	/*Profile ID - 24176*/
const Value V24047_1_1 = {S2, NULL};	/*Value = True*/
const Property P24047_1 = {S50, 0, &V24047_1_1, NULL};	/*Property = IsMobile*/
const Value V24047_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P24047_5 = {S52, 0, &V24047_5_1458, &P24047_1};	/*Property = ScreenPixelsHeight*/
const Value V24047_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P24047_4 = {S54, 0, &V24047_4_37617, &P24047_5};	/*Property = ScreenPixelsWidth*/
const Profile P24047 = {"24047", &P24047_4};	/*Profile ID - 24047*/
const Value V24055_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24055_279 = {S1, 0, &V24055_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24055_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24055_282 = {S3, 0, &V24055_282_57154, &P24055_279};	/*Property = BlobBuilder*/
const Value V24055_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24055_350 = {S4, 0, &V24055_350_57444, &P24055_282};	/*Property = Canvas*/
const Value V24055_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24055_283 = {S5, 0, &V24055_283_57159, &P24055_350};	/*Property = CssBackground*/
const Value V24055_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24055_284 = {S6, 0, &V24055_284_57162, &P24055_283};	/*Property = CssBorderImage*/
const Value V24055_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24055_285 = {S7, 0, &V24055_285_57166, &P24055_284};	/*Property = CssCanvas*/
const Value V24055_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24055_286 = {S8, 0, &V24055_286_57168, &P24055_285};	/*Property = CssColor*/
const Value V24055_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24055_237 = {S9, 0, &V24055_237_55786, &P24055_286};	/*Property = CssColumn*/
const Value V24055_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24055_287 = {S10, 0, &V24055_287_57172, &P24055_237};	/*Property = CssFlexbox*/
const Value V24055_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24055_288 = {S11, 0, &V24055_288_57175, &P24055_287};	/*Property = CssFont*/
const Value V24055_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24055_289 = {S12, 0, &V24055_289_57759, &P24055_288};	/*Property = CssImages*/
const Value V24055_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24055_291 = {S13, 0, &V24055_291_57180, &P24055_289};	/*Property = CssMediaQueries*/
const Value V24055_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24055_292 = {S14, 0, &V24055_292_57183, &P24055_291};	/*Property = CssMinMax*/
const Value V24055_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24055_293 = {S15, 0, &V24055_293_57187, &P24055_292};	/*Property = CssOverflow*/
const Value V24055_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24055_294 = {S16, 0, &V24055_294_57189, &P24055_293};	/*Property = CssPosition*/
const Value V24055_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24055_295 = {S17, 0, &V24055_295_57192, &P24055_294};	/*Property = CssText*/
const Value V24055_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24055_297 = {S18, 0, &V24055_297_57195, &P24055_295};	/*Property = CssTransforms*/
const Value V24055_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24055_241 = {S19, 0, &V24055_241_55799, &P24055_297};	/*Property = CssTransitions*/
const Value V24055_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24055_298 = {S20, 0, &V24055_298_57206, &P24055_241};	/*Property = CssUI*/
const Value V24055_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24055_300 = {S21, 0, &V24055_300_57215, &P24055_298};	/*Property = DataSet*/
const Value V24055_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24055_301 = {S22, 0, &V24055_301_57218, &P24055_300};	/*Property = DataUrl*/
const Value V24055_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24055_302 = {S23, 0, &V24055_302_57221, &P24055_301};	/*Property = DeviceOrientation*/
const Value V24055_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24055_304 = {S24, 0, &V24055_304_57227, &P24055_302};	/*Property = FileReader*/
const Value V24055_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24055_305 = {S25, 0, &V24055_305_57231, &P24055_304};	/*Property = FileSaver*/
const Value V24055_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24055_306 = {S26, 0, &V24055_306_57237, &P24055_305};	/*Property = FileWriter*/
const Value V24055_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24055_307 = {S27, 0, &V24055_307_57239, &P24055_306};	/*Property = FormData*/
const Value V24055_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24055_309 = {S28, 0, &V24055_309_57246, &P24055_307};	/*Property = Fullscreen*/
const Value V24055_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24055_310 = {S29, 0, &V24055_310_57248, &P24055_309};	/*Property = GeoLocation*/
const Value V24055_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24055_312 = {S30, 0, &V24055_312_57254, &P24055_310};	/*Property = History*/
const Value V24055_314_57261 = {S0, NULL};	/*Value = False*/
const Property P24055_314 = {S31, 0, &V24055_314_57261, &P24055_312};	/*Property = Html5*/
const Value V24055_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24055_313 = {S32, 0, &V24055_313_57258, &P24055_314};	/*Property = Html-Media-Capture*/
const Value V24055_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24055_315 = {S33, 0, &V24055_315_57264, &P24055_313};	/*Property = Iframe*/
const Value V24055_316_57267 = {S0, NULL};	/*Value = False*/
const Property P24055_316 = {S34, 0, &V24055_316_57267, &P24055_315};	/*Property = IndexedDB*/
const Value V24055_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24055_317 = {S35, 0, &V24055_317_57269, &P24055_316};	/*Property = Json*/
const Value V24055_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24055_15 = {S37, 0, &V24055_15_28, &P24055_317};	/*Property = LayoutEngine*/
const Value V24055_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24055_344 = {S38, 0, &V24055_344_57272, &P24055_15};	/*Property = Masking*/
const Value V24055_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24055_320 = {S39, 0, &V24055_320_57290, &P24055_344};	/*Property = PostMessage*/
const Value V24055_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24055_321 = {S40, 0, &V24055_321_57293, &P24055_320};	/*Property = Progress*/
const Value V24055_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24055_322 = {S41, 0, &V24055_322_57296, &P24055_321};	/*Property = Prompts*/
const Value V24055_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24055_323 = {S42, 0, &V24055_323_57299, &P24055_322};	/*Property = Selector*/
const Value V24055_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24055_324 = {S43, 0, &V24055_324_57306, &P24055_323};	/*Property = Svg*/
const Value V24055_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24055_327 = {S44, 0, &V24055_327_57314, &P24055_324};	/*Property = TouchEvents*/
const Value V24055_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24055_328 = {S45, 0, &V24055_328_57318, &P24055_327};	/*Property = Track*/
const Value V24055_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24055_329 = {S46, 0, &V24055_329_57323, &P24055_328};	/*Property = Video*/
const Value V24055_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24055_330 = {S47, 0, &V24055_330_57326, &P24055_329};	/*Property = Viewport*/
const Value V24055_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24055_332 = {S48, 0, &V24055_332_57339, &P24055_330};	/*Property = WebWorkers*/
const Value V24055_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24055_333 = {S49, 0, &V24055_333_57342, &P24055_332};	/*Property = Xhr2*/
const Profile P24055 = {"24055", &P24055_333};	/*Profile ID - 24055*/
const Value V24337_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_279 = {S1, 0, &V24337_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24337_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_282 = {S3, 0, &V24337_282_57155, &P24337_279};	/*Property = BlobBuilder*/
const Value V24337_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_350 = {S4, 0, &V24337_350_57446, &P24337_282};	/*Property = Canvas*/
const Value V24337_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_283 = {S5, 0, &V24337_283_57161, &P24337_350};	/*Property = CssBackground*/
const Value V24337_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_284 = {S6, 0, &V24337_284_57164, &P24337_283};	/*Property = CssBorderImage*/
const Value V24337_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_285 = {S7, 0, &V24337_285_57167, &P24337_284};	/*Property = CssCanvas*/
const Value V24337_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_286 = {S8, 0, &V24337_286_57170, &P24337_285};	/*Property = CssColor*/
const Value V24337_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_237 = {S9, 0, &V24337_237_55786, &P24337_286};	/*Property = CssColumn*/
const Value V24337_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_287 = {S10, 0, &V24337_287_57173, &P24337_237};	/*Property = CssFlexbox*/
const Value V24337_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_288 = {S11, 0, &V24337_288_57176, &P24337_287};	/*Property = CssFont*/
const Value V24337_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_289 = {S12, 0, &V24337_289_86899, &P24337_288};	/*Property = CssImages*/
const Value V24337_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_291 = {S13, 0, &V24337_291_57182, &P24337_289};	/*Property = CssMediaQueries*/
const Value V24337_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_292 = {S14, 0, &V24337_292_57185, &P24337_291};	/*Property = CssMinMax*/
const Value V24337_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_293 = {S15, 0, &V24337_293_57188, &P24337_292};	/*Property = CssOverflow*/
const Value V24337_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_294 = {S16, 0, &V24337_294_57191, &P24337_293};	/*Property = CssPosition*/
const Value V24337_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_295 = {S17, 0, &V24337_295_57194, &P24337_294};	/*Property = CssText*/
const Value V24337_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_297 = {S18, 0, &V24337_297_57197, &P24337_295};	/*Property = CssTransforms*/
const Value V24337_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_241 = {S19, 0, &V24337_241_55801, &P24337_297};	/*Property = CssTransitions*/
const Value V24337_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_298 = {S20, 0, &V24337_298_57208, &P24337_241};	/*Property = CssUI*/
const Value V24337_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_300 = {S21, 0, &V24337_300_57217, &P24337_298};	/*Property = DataSet*/
const Value V24337_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_301 = {S22, 0, &V24337_301_57220, &P24337_300};	/*Property = DataUrl*/
const Value V24337_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_302 = {S23, 0, &V24337_302_57223, &P24337_301};	/*Property = DeviceOrientation*/
const Value V24337_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_304 = {S24, 0, &V24337_304_57229, &P24337_302};	/*Property = FileReader*/
const Value V24337_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_305 = {S25, 0, &V24337_305_57232, &P24337_304};	/*Property = FileSaver*/
const Value V24337_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_306 = {S26, 0, &V24337_306_57238, &P24337_305};	/*Property = FileWriter*/
const Value V24337_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_307 = {S27, 0, &V24337_307_57241, &P24337_306};	/*Property = FormData*/
const Value V24337_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_309 = {S28, 0, &V24337_309_57247, &P24337_307};	/*Property = Fullscreen*/
const Value V24337_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_310 = {S29, 0, &V24337_310_57250, &P24337_309};	/*Property = GeoLocation*/
const Value V24337_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_312 = {S30, 0, &V24337_312_57256, &P24337_310};	/*Property = History*/
const Value V24337_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_314 = {S31, 0, &V24337_314_57262, &P24337_312};	/*Property = Html5*/
const Value V24337_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_313 = {S32, 0, &V24337_313_57259, &P24337_314};	/*Property = Html-Media-Capture*/
const Value V24337_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_315 = {S33, 0, &V24337_315_57265, &P24337_313};	/*Property = Iframe*/
const Value V24337_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_316 = {S34, 0, &V24337_316_57268, &P24337_315};	/*Property = IndexedDB*/
const Value V24337_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_317 = {S35, 0, &V24337_317_57271, &P24337_316};	/*Property = Json*/
const Value V24337_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P24337_15 = {S37, 0, &V24337_15_29, &P24337_317};	/*Property = LayoutEngine*/
const Value V24337_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_344 = {S38, 0, &V24337_344_57274, &P24337_15};	/*Property = Masking*/
const Value V24337_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_320 = {S39, 0, &V24337_320_57292, &P24337_344};	/*Property = PostMessage*/
const Value V24337_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_321 = {S40, 0, &V24337_321_57295, &P24337_320};	/*Property = Progress*/
const Value V24337_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_322 = {S41, 0, &V24337_322_57298, &P24337_321};	/*Property = Prompts*/
const Value V24337_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_323 = {S42, 0, &V24337_323_57301, &P24337_322};	/*Property = Selector*/
const Value V24337_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_324 = {S43, 0, &V24337_324_57307, &P24337_323};	/*Property = Svg*/
const Value V24337_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_327 = {S44, 0, &V24337_327_57316, &P24337_324};	/*Property = TouchEvents*/
const Value V24337_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_328 = {S45, 0, &V24337_328_57319, &P24337_327};	/*Property = Track*/
const Value V24337_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_329 = {S46, 0, &V24337_329_57325, &P24337_328};	/*Property = Video*/
const Value V24337_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_330 = {S47, 0, &V24337_330_57328, &P24337_329};	/*Property = Viewport*/
const Value V24337_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_332 = {S48, 0, &V24337_332_57340, &P24337_330};	/*Property = WebWorkers*/
const Value V24337_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24337_333 = {S49, 0, &V24337_333_57343, &P24337_332};	/*Property = Xhr2*/
const Profile P24337 = {"24337", &P24337_333};	/*Profile ID - 24337*/
const Value V24080_1_2 = {S0, NULL};	/*Value = False*/
const Property P24080_1 = {S50, 0, &V24080_1_2, NULL};	/*Property = IsMobile*/
const Value V24080_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24080_5 = {S52, 0, &V24080_5_60417, &P24080_1};	/*Property = ScreenPixelsHeight*/
const Value V24080_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24080_4 = {S54, 0, &V24080_4_57095, &P24080_5};	/*Property = ScreenPixelsWidth*/
const Profile P24080 = {"24080", &P24080_4};	/*Profile ID - 24080*/
const Value V24051_1_2 = {S0, NULL};	/*Value = False*/
const Property P24051_1 = {S50, 0, &V24051_1_2, NULL};	/*Property = IsMobile*/
const Value V24051_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24051_5 = {S52, 0, &V24051_5_60417, &P24051_1};	/*Property = ScreenPixelsHeight*/
const Value V24051_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24051_4 = {S54, 0, &V24051_4_57095, &P24051_5};	/*Property = ScreenPixelsWidth*/
const Profile P24051 = {"24051", &P24051_4};	/*Profile ID - 24051*/
const Value V24081_1_2 = {S0, NULL};	/*Value = False*/
const Property P24081_1 = {S50, 0, &V24081_1_2, NULL};	/*Property = IsMobile*/
const Value V24081_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24081_5 = {S52, 0, &V24081_5_60417, &P24081_1};	/*Property = ScreenPixelsHeight*/
const Value V24081_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24081_4 = {S54, 0, &V24081_4_57095, &P24081_5};	/*Property = ScreenPixelsWidth*/
const Profile P24081 = {"24081", &P24081_4};	/*Profile ID - 24081*/
const Value V24083_1_2 = {S0, NULL};	/*Value = False*/
const Property P24083_1 = {S50, 0, &V24083_1_2, NULL};	/*Property = IsMobile*/
const Value V24083_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24083_5 = {S52, 0, &V24083_5_60417, &P24083_1};	/*Property = ScreenPixelsHeight*/
const Value V24083_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24083_4 = {S54, 0, &V24083_4_57095, &P24083_5};	/*Property = ScreenPixelsWidth*/
const Profile P24083 = {"24083", &P24083_4};	/*Profile ID - 24083*/
const Value V24084_1_2 = {S0, NULL};	/*Value = False*/
const Property P24084_1 = {S50, 0, &V24084_1_2, NULL};	/*Property = IsMobile*/
const Value V24084_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24084_5 = {S52, 0, &V24084_5_60417, &P24084_1};	/*Property = ScreenPixelsHeight*/
const Value V24084_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24084_4 = {S54, 0, &V24084_4_57095, &P24084_5};	/*Property = ScreenPixelsWidth*/
const Profile P24084 = {"24084", &P24084_4};	/*Profile ID - 24084*/
const Value V24085_1_2 = {S0, NULL};	/*Value = False*/
const Property P24085_1 = {S50, 0, &V24085_1_2, NULL};	/*Property = IsMobile*/
const Value V24085_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24085_5 = {S52, 0, &V24085_5_60417, &P24085_1};	/*Property = ScreenPixelsHeight*/
const Value V24085_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24085_4 = {S54, 0, &V24085_4_57095, &P24085_5};	/*Property = ScreenPixelsWidth*/
const Profile P24085 = {"24085", &P24085_4};	/*Profile ID - 24085*/
const Value V24086_1_2 = {S0, NULL};	/*Value = False*/
const Property P24086_1 = {S50, 0, &V24086_1_2, NULL};	/*Property = IsMobile*/
const Value V24086_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24086_5 = {S52, 0, &V24086_5_60417, &P24086_1};	/*Property = ScreenPixelsHeight*/
const Value V24086_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24086_4 = {S54, 0, &V24086_4_57095, &P24086_5};	/*Property = ScreenPixelsWidth*/
const Profile P24086 = {"24086", &P24086_4};	/*Profile ID - 24086*/
const Value V24087_1_2 = {S0, NULL};	/*Value = False*/
const Property P24087_1 = {S50, 0, &V24087_1_2, NULL};	/*Property = IsMobile*/
const Value V24087_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24087_5 = {S52, 0, &V24087_5_60417, &P24087_1};	/*Property = ScreenPixelsHeight*/
const Value V24087_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24087_4 = {S54, 0, &V24087_4_57095, &P24087_5};	/*Property = ScreenPixelsWidth*/
const Profile P24087 = {"24087", &P24087_4};	/*Profile ID - 24087*/
const Value V24088_1_2 = {S0, NULL};	/*Value = False*/
const Property P24088_1 = {S50, 0, &V24088_1_2, NULL};	/*Property = IsMobile*/
const Value V24088_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24088_5 = {S52, 0, &V24088_5_60417, &P24088_1};	/*Property = ScreenPixelsHeight*/
const Value V24088_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24088_4 = {S54, 0, &V24088_4_57095, &P24088_5};	/*Property = ScreenPixelsWidth*/
const Profile P24088 = {"24088", &P24088_4};	/*Profile ID - 24088*/
const Value V24089_1_2 = {S0, NULL};	/*Value = False*/
const Property P24089_1 = {S50, 0, &V24089_1_2, NULL};	/*Property = IsMobile*/
const Value V24089_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24089_5 = {S52, 0, &V24089_5_60417, &P24089_1};	/*Property = ScreenPixelsHeight*/
const Value V24089_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24089_4 = {S54, 0, &V24089_4_57095, &P24089_5};	/*Property = ScreenPixelsWidth*/
const Profile P24089 = {"24089", &P24089_4};	/*Profile ID - 24089*/
const Value V24091_1_2 = {S0, NULL};	/*Value = False*/
const Property P24091_1 = {S50, 0, &V24091_1_2, NULL};	/*Property = IsMobile*/
const Value V24091_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24091_5 = {S52, 0, &V24091_5_60417, &P24091_1};	/*Property = ScreenPixelsHeight*/
const Value V24091_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24091_4 = {S54, 0, &V24091_4_57095, &P24091_5};	/*Property = ScreenPixelsWidth*/
const Profile P24091 = {"24091", &P24091_4};	/*Profile ID - 24091*/
const Value V24090_1_2 = {S0, NULL};	/*Value = False*/
const Property P24090_1 = {S50, 0, &V24090_1_2, NULL};	/*Property = IsMobile*/
const Value V24090_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24090_5 = {S52, 0, &V24090_5_60417, &P24090_1};	/*Property = ScreenPixelsHeight*/
const Value V24090_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24090_4 = {S54, 0, &V24090_4_57095, &P24090_5};	/*Property = ScreenPixelsWidth*/
const Profile P24090 = {"24090", &P24090_4};	/*Profile ID - 24090*/
const Value V24061_1_1 = {S2, NULL};	/*Value = True*/
const Property P24061_1 = {S50, 0, &V24061_1_1, NULL};	/*Property = IsMobile*/
const Value V24061_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24061_5 = {S52, 0, &V24061_5_1227, &P24061_1};	/*Property = ScreenPixelsHeight*/
const Value V24061_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24061_4 = {S54, 0, &V24061_4_1229, &P24061_5};	/*Property = ScreenPixelsWidth*/
const Profile P24061 = {"24061", &P24061_4};	/*Profile ID - 24061*/
const Value V24064_1_1 = {S2, NULL};	/*Value = True*/
const Property P24064_1 = {S50, 0, &V24064_1_1, NULL};	/*Property = IsMobile*/
const Value V24064_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24064_5 = {S52, 0, &V24064_5_1227, &P24064_1};	/*Property = ScreenPixelsHeight*/
const Value V24064_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24064_4 = {S54, 0, &V24064_4_1229, &P24064_5};	/*Property = ScreenPixelsWidth*/
const Profile P24064 = {"24064", &P24064_4};	/*Profile ID - 24064*/
const Value V24065_1_1 = {S2, NULL};	/*Value = True*/
const Property P24065_1 = {S50, 0, &V24065_1_1, NULL};	/*Property = IsMobile*/
const Value V24065_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24065_5 = {S52, 0, &V24065_5_1227, &P24065_1};	/*Property = ScreenPixelsHeight*/
const Value V24065_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24065_4 = {S54, 0, &V24065_4_1229, &P24065_5};	/*Property = ScreenPixelsWidth*/
const Profile P24065 = {"24065", &P24065_4};	/*Profile ID - 24065*/
const Value V24186_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24186_279 = {S1, 0, &V24186_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24186_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24186_282 = {S3, 0, &V24186_282_57154, &P24186_279};	/*Property = BlobBuilder*/
const Value V24186_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24186_350 = {S4, 0, &V24186_350_57444, &P24186_282};	/*Property = Canvas*/
const Value V24186_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24186_283 = {S5, 0, &V24186_283_57159, &P24186_350};	/*Property = CssBackground*/
const Value V24186_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24186_284 = {S6, 0, &V24186_284_57162, &P24186_283};	/*Property = CssBorderImage*/
const Value V24186_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24186_285 = {S7, 0, &V24186_285_57166, &P24186_284};	/*Property = CssCanvas*/
const Value V24186_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24186_286 = {S8, 0, &V24186_286_57168, &P24186_285};	/*Property = CssColor*/
const Value V24186_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24186_237 = {S9, 0, &V24186_237_55786, &P24186_286};	/*Property = CssColumn*/
const Value V24186_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24186_287 = {S10, 0, &V24186_287_57172, &P24186_237};	/*Property = CssFlexbox*/
const Value V24186_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24186_288 = {S11, 0, &V24186_288_57174, &P24186_287};	/*Property = CssFont*/
const Value V24186_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24186_289 = {S12, 0, &V24186_289_57759, &P24186_288};	/*Property = CssImages*/
const Value V24186_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24186_291 = {S13, 0, &V24186_291_57180, &P24186_289};	/*Property = CssMediaQueries*/
const Value V24186_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24186_292 = {S14, 0, &V24186_292_57183, &P24186_291};	/*Property = CssMinMax*/
const Value V24186_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24186_293 = {S15, 0, &V24186_293_57187, &P24186_292};	/*Property = CssOverflow*/
const Value V24186_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24186_294 = {S16, 0, &V24186_294_57189, &P24186_293};	/*Property = CssPosition*/
const Value V24186_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24186_295 = {S17, 0, &V24186_295_57192, &P24186_294};	/*Property = CssText*/
const Value V24186_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24186_297 = {S18, 0, &V24186_297_57195, &P24186_295};	/*Property = CssTransforms*/
const Value V24186_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24186_241 = {S19, 0, &V24186_241_55799, &P24186_297};	/*Property = CssTransitions*/
const Value V24186_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24186_298 = {S20, 0, &V24186_298_57206, &P24186_241};	/*Property = CssUI*/
const Value V24186_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24186_300 = {S21, 0, &V24186_300_57215, &P24186_298};	/*Property = DataSet*/
const Value V24186_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24186_301 = {S22, 0, &V24186_301_57218, &P24186_300};	/*Property = DataUrl*/
const Value V24186_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24186_302 = {S23, 0, &V24186_302_57221, &P24186_301};	/*Property = DeviceOrientation*/
const Value V24186_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24186_304 = {S24, 0, &V24186_304_57227, &P24186_302};	/*Property = FileReader*/
const Value V24186_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24186_305 = {S25, 0, &V24186_305_57231, &P24186_304};	/*Property = FileSaver*/
const Value V24186_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24186_306 = {S26, 0, &V24186_306_57237, &P24186_305};	/*Property = FileWriter*/
const Value V24186_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24186_307 = {S27, 0, &V24186_307_57239, &P24186_306};	/*Property = FormData*/
const Value V24186_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24186_309 = {S28, 0, &V24186_309_57246, &P24186_307};	/*Property = Fullscreen*/
const Value V24186_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24186_310 = {S29, 0, &V24186_310_57248, &P24186_309};	/*Property = GeoLocation*/
const Value V24186_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24186_312 = {S30, 0, &V24186_312_57254, &P24186_310};	/*Property = History*/
const Value V24186_314_57261 = {S0, NULL};	/*Value = False*/
const Property P24186_314 = {S31, 0, &V24186_314_57261, &P24186_312};	/*Property = Html5*/
const Value V24186_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24186_313 = {S32, 0, &V24186_313_57258, &P24186_314};	/*Property = Html-Media-Capture*/
const Value V24186_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24186_315 = {S33, 0, &V24186_315_57264, &P24186_313};	/*Property = Iframe*/
const Value V24186_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24186_316 = {S34, 0, &V24186_316_57266, &P24186_315};	/*Property = IndexedDB*/
const Value V24186_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24186_317 = {S35, 0, &V24186_317_57269, &P24186_316};	/*Property = Json*/
const Value V24186_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P24186_15 = {S37, 0, &V24186_15_29, &P24186_317};	/*Property = LayoutEngine*/
const Value V24186_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24186_344 = {S38, 0, &V24186_344_57272, &P24186_15};	/*Property = Masking*/
const Value V24186_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24186_320 = {S39, 0, &V24186_320_57290, &P24186_344};	/*Property = PostMessage*/
const Value V24186_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24186_321 = {S40, 0, &V24186_321_57293, &P24186_320};	/*Property = Progress*/
const Value V24186_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24186_322 = {S41, 0, &V24186_322_57296, &P24186_321};	/*Property = Prompts*/
const Value V24186_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24186_323 = {S42, 0, &V24186_323_57299, &P24186_322};	/*Property = Selector*/
const Value V24186_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24186_324 = {S43, 0, &V24186_324_57305, &P24186_323};	/*Property = Svg*/
const Value V24186_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24186_327 = {S44, 0, &V24186_327_57314, &P24186_324};	/*Property = TouchEvents*/
const Value V24186_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24186_328 = {S45, 0, &V24186_328_57318, &P24186_327};	/*Property = Track*/
const Value V24186_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24186_329 = {S46, 0, &V24186_329_57323, &P24186_328};	/*Property = Video*/
const Value V24186_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24186_330 = {S47, 0, &V24186_330_57326, &P24186_329};	/*Property = Viewport*/
const Value V24186_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24186_332 = {S48, 0, &V24186_332_57338, &P24186_330};	/*Property = WebWorkers*/
const Value V24186_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24186_333 = {S49, 0, &V24186_333_57342, &P24186_332};	/*Property = Xhr2*/
const Profile P24186 = {"24186", &P24186_333};	/*Profile ID - 24186*/
const Value V4473_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_279 = {S1, 0, &V4473_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V4473_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_282 = {S3, 0, &V4473_282_57155, &P4473_279};	/*Property = BlobBuilder*/
const Value V4473_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_350 = {S4, 0, &V4473_350_57446, &P4473_282};	/*Property = Canvas*/
const Value V4473_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_283 = {S5, 0, &V4473_283_57161, &P4473_350};	/*Property = CssBackground*/
const Value V4473_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_284 = {S6, 0, &V4473_284_57164, &P4473_283};	/*Property = CssBorderImage*/
const Value V4473_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_285 = {S7, 0, &V4473_285_57167, &P4473_284};	/*Property = CssCanvas*/
const Value V4473_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_286 = {S8, 0, &V4473_286_57170, &P4473_285};	/*Property = CssColor*/
const Value V4473_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_237 = {S9, 0, &V4473_237_55786, &P4473_286};	/*Property = CssColumn*/
const Value V4473_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_287 = {S10, 0, &V4473_287_57173, &P4473_237};	/*Property = CssFlexbox*/
const Value V4473_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_288 = {S11, 0, &V4473_288_57176, &P4473_287};	/*Property = CssFont*/
const Value V4473_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_289 = {S12, 0, &V4473_289_86899, &P4473_288};	/*Property = CssImages*/
const Value V4473_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_291 = {S13, 0, &V4473_291_57182, &P4473_289};	/*Property = CssMediaQueries*/
const Value V4473_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_292 = {S14, 0, &V4473_292_57185, &P4473_291};	/*Property = CssMinMax*/
const Value V4473_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_293 = {S15, 0, &V4473_293_57188, &P4473_292};	/*Property = CssOverflow*/
const Value V4473_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_294 = {S16, 0, &V4473_294_57191, &P4473_293};	/*Property = CssPosition*/
const Value V4473_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_295 = {S17, 0, &V4473_295_57194, &P4473_294};	/*Property = CssText*/
const Value V4473_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_297 = {S18, 0, &V4473_297_57197, &P4473_295};	/*Property = CssTransforms*/
const Value V4473_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_241 = {S19, 0, &V4473_241_55801, &P4473_297};	/*Property = CssTransitions*/
const Value V4473_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_298 = {S20, 0, &V4473_298_57208, &P4473_241};	/*Property = CssUI*/
const Value V4473_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_300 = {S21, 0, &V4473_300_57217, &P4473_298};	/*Property = DataSet*/
const Value V4473_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_301 = {S22, 0, &V4473_301_57220, &P4473_300};	/*Property = DataUrl*/
const Value V4473_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_302 = {S23, 0, &V4473_302_57223, &P4473_301};	/*Property = DeviceOrientation*/
const Value V4473_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_304 = {S24, 0, &V4473_304_57229, &P4473_302};	/*Property = FileReader*/
const Value V4473_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_305 = {S25, 0, &V4473_305_57232, &P4473_304};	/*Property = FileSaver*/
const Value V4473_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_306 = {S26, 0, &V4473_306_57238, &P4473_305};	/*Property = FileWriter*/
const Value V4473_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_307 = {S27, 0, &V4473_307_57241, &P4473_306};	/*Property = FormData*/
const Value V4473_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_309 = {S28, 0, &V4473_309_57247, &P4473_307};	/*Property = Fullscreen*/
const Value V4473_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_310 = {S29, 0, &V4473_310_57250, &P4473_309};	/*Property = GeoLocation*/
const Value V4473_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_312 = {S30, 0, &V4473_312_57256, &P4473_310};	/*Property = History*/
const Value V4473_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_314 = {S31, 0, &V4473_314_57262, &P4473_312};	/*Property = Html5*/
const Value V4473_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_313 = {S32, 0, &V4473_313_57259, &P4473_314};	/*Property = Html-Media-Capture*/
const Value V4473_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_315 = {S33, 0, &V4473_315_57265, &P4473_313};	/*Property = Iframe*/
const Value V4473_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_316 = {S34, 0, &V4473_316_57268, &P4473_315};	/*Property = IndexedDB*/
const Value V4473_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_317 = {S35, 0, &V4473_317_57271, &P4473_316};	/*Property = Json*/
const Value V4473_15_43264 = {S59, NULL};	/*Value = Mango*/
const Property P4473_15 = {S37, 0, &V4473_15_43264, &P4473_317};	/*Property = LayoutEngine*/
const Value V4473_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_344 = {S38, 0, &V4473_344_57274, &P4473_15};	/*Property = Masking*/
const Value V4473_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_320 = {S39, 0, &V4473_320_57292, &P4473_344};	/*Property = PostMessage*/
const Value V4473_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_321 = {S40, 0, &V4473_321_57295, &P4473_320};	/*Property = Progress*/
const Value V4473_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_322 = {S41, 0, &V4473_322_57298, &P4473_321};	/*Property = Prompts*/
const Value V4473_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_323 = {S42, 0, &V4473_323_57301, &P4473_322};	/*Property = Selector*/
const Value V4473_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_324 = {S43, 0, &V4473_324_57307, &P4473_323};	/*Property = Svg*/
const Value V4473_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_327 = {S44, 0, &V4473_327_57316, &P4473_324};	/*Property = TouchEvents*/
const Value V4473_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_328 = {S45, 0, &V4473_328_57319, &P4473_327};	/*Property = Track*/
const Value V4473_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_329 = {S46, 0, &V4473_329_57325, &P4473_328};	/*Property = Video*/
const Value V4473_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_330 = {S47, 0, &V4473_330_57328, &P4473_329};	/*Property = Viewport*/
const Value V4473_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_332 = {S48, 0, &V4473_332_57340, &P4473_330};	/*Property = WebWorkers*/
const Value V4473_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P4473_333 = {S49, 0, &V4473_333_57343, &P4473_332};	/*Property = Xhr2*/
const Profile P4473 = {"4473", &P4473_333};	/*Profile ID - 4473*/
const Value V23576_1_1 = {S2, NULL};	/*Value = True*/
const Property P23576_1 = {S50, 0, &V23576_1_1, NULL};	/*Property = IsMobile*/
const Value V23576_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P23576_5 = {S52, 0, &V23576_5_1277, &P23576_1};	/*Property = ScreenPixelsHeight*/
const Value V23576_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P23576_4 = {S54, 0, &V23576_4_1279, &P23576_5};	/*Property = ScreenPixelsWidth*/
const Profile P23576 = {"23576", &P23576_4};	/*Profile ID - 23576*/
const Value V24681_1_1 = {S2, NULL};	/*Value = True*/
const Property P24681_1 = {S50, 0, &V24681_1_1, NULL};	/*Property = IsMobile*/
const Value V24681_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24681_5 = {S52, 0, &V24681_5_1277, &P24681_1};	/*Property = ScreenPixelsHeight*/
const Value V24681_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P24681_4 = {S54, 0, &V24681_4_2336, &P24681_5};	/*Property = ScreenPixelsWidth*/
const Profile P24681 = {"24681", &P24681_4};	/*Profile ID - 24681*/
const Value V24478_1_1 = {S2, NULL};	/*Value = True*/
const Property P24478_1 = {S50, 0, &V24478_1_1, NULL};	/*Property = IsMobile*/
const Value V24478_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24478_5 = {S52, 0, &V24478_5_1227, &P24478_1};	/*Property = ScreenPixelsHeight*/
const Value V24478_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24478_4 = {S54, 0, &V24478_4_1229, &P24478_5};	/*Property = ScreenPixelsWidth*/
const Profile P24478 = {"24478", &P24478_4};	/*Profile ID - 24478*/
const Value V24597_1_1 = {S2, NULL};	/*Value = True*/
const Property P24597_1 = {S50, 0, &V24597_1_1, NULL};	/*Property = IsMobile*/
const Value V24597_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24597_5 = {S52, 0, &V24597_5_9981, &P24597_1};	/*Property = ScreenPixelsHeight*/
const Value V24597_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P24597_4 = {S54, 0, &V24597_4_2336, &P24597_5};	/*Property = ScreenPixelsWidth*/
const Profile P24597 = {"24597", &P24597_4};	/*Profile ID - 24597*/
const Value V24579_1_1 = {S2, NULL};	/*Value = True*/
const Property P24579_1 = {S50, 0, &V24579_1_1, NULL};	/*Property = IsMobile*/
const Value V24579_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24579_5 = {S52, 0, &V24579_5_9981, &P24579_1};	/*Property = ScreenPixelsHeight*/
const Value V24579_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P24579_4 = {S54, 0, &V24579_4_2336, &P24579_5};	/*Property = ScreenPixelsWidth*/
const Profile P24579 = {"24579", &P24579_4};	/*Profile ID - 24579*/
const Value V24578_1_1 = {S2, NULL};	/*Value = True*/
const Property P24578_1 = {S50, 0, &V24578_1_1, NULL};	/*Property = IsMobile*/
const Value V24578_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24578_5 = {S52, 0, &V24578_5_9981, &P24578_1};	/*Property = ScreenPixelsHeight*/
const Value V24578_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P24578_4 = {S54, 0, &V24578_4_2336, &P24578_5};	/*Property = ScreenPixelsWidth*/
const Profile P24578 = {"24578", &P24578_4};	/*Profile ID - 24578*/
const Value V24373_1_1 = {S2, NULL};	/*Value = True*/
const Property P24373_1 = {S50, 0, &V24373_1_1, NULL};	/*Property = IsMobile*/
const Value V24373_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24373_5 = {S52, 0, &V24373_5_1227, &P24373_1};	/*Property = ScreenPixelsHeight*/
const Value V24373_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24373_4 = {S54, 0, &V24373_4_1229, &P24373_5};	/*Property = ScreenPixelsWidth*/
const Profile P24373 = {"24373", &P24373_4};	/*Profile ID - 24373*/
const Value V24580_1_1 = {S2, NULL};	/*Value = True*/
const Property P24580_1 = {S50, 0, &V24580_1_1, NULL};	/*Property = IsMobile*/
const Value V24580_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P24580_5 = {S52, 0, &V24580_5_1458, &P24580_1};	/*Property = ScreenPixelsHeight*/
const Value V24580_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P24580_4 = {S54, 0, &V24580_4_1460, &P24580_5};	/*Property = ScreenPixelsWidth*/
const Profile P24580 = {"24580", &P24580_4};	/*Profile ID - 24580*/
const Value V24565_1_1 = {S2, NULL};	/*Value = True*/
const Property P24565_1 = {S50, 0, &V24565_1_1, NULL};	/*Property = IsMobile*/
const Value V24565_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24565_5 = {S52, 0, &V24565_5_1403, &P24565_1};	/*Property = ScreenPixelsHeight*/
const Value V24565_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24565_4 = {S54, 0, &V24565_4_1404, &P24565_5};	/*Property = ScreenPixelsWidth*/
const Profile P24565 = {"24565", &P24565_4};	/*Profile ID - 24565*/
const Value V24415_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24415_279 = {S1, 0, &V24415_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24415_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24415_282 = {S3, 0, &V24415_282_57154, &P24415_279};	/*Property = BlobBuilder*/
const Value V24415_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24415_350 = {S4, 0, &V24415_350_57444, &P24415_282};	/*Property = Canvas*/
const Value V24415_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24415_283 = {S5, 0, &V24415_283_57159, &P24415_350};	/*Property = CssBackground*/
const Value V24415_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24415_284 = {S6, 0, &V24415_284_57162, &P24415_283};	/*Property = CssBorderImage*/
const Value V24415_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24415_285 = {S7, 0, &V24415_285_57166, &P24415_284};	/*Property = CssCanvas*/
const Value V24415_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24415_286 = {S8, 0, &V24415_286_57168, &P24415_285};	/*Property = CssColor*/
const Value V24415_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24415_237 = {S9, 0, &V24415_237_55786, &P24415_286};	/*Property = CssColumn*/
const Value V24415_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24415_287 = {S10, 0, &V24415_287_57172, &P24415_237};	/*Property = CssFlexbox*/
const Value V24415_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24415_288 = {S11, 0, &V24415_288_57174, &P24415_287};	/*Property = CssFont*/
const Value V24415_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24415_289 = {S12, 0, &V24415_289_57759, &P24415_288};	/*Property = CssImages*/
const Value V24415_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24415_291 = {S13, 0, &V24415_291_57180, &P24415_289};	/*Property = CssMediaQueries*/
const Value V24415_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24415_292 = {S14, 0, &V24415_292_57183, &P24415_291};	/*Property = CssMinMax*/
const Value V24415_293_57186 = {S2, NULL};	/*Value = True*/
const Property P24415_293 = {S15, 0, &V24415_293_57186, &P24415_292};	/*Property = CssOverflow*/
const Value V24415_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24415_294 = {S16, 0, &V24415_294_57189, &P24415_293};	/*Property = CssPosition*/
const Value V24415_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24415_295 = {S17, 0, &V24415_295_57192, &P24415_294};	/*Property = CssText*/
const Value V24415_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24415_297 = {S18, 0, &V24415_297_57195, &P24415_295};	/*Property = CssTransforms*/
const Value V24415_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24415_241 = {S19, 0, &V24415_241_55799, &P24415_297};	/*Property = CssTransitions*/
const Value V24415_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24415_298 = {S20, 0, &V24415_298_57206, &P24415_241};	/*Property = CssUI*/
const Value V24415_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24415_300 = {S21, 0, &V24415_300_57215, &P24415_298};	/*Property = DataSet*/
const Value V24415_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24415_301 = {S22, 0, &V24415_301_57218, &P24415_300};	/*Property = DataUrl*/
const Value V24415_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24415_302 = {S23, 0, &V24415_302_57221, &P24415_301};	/*Property = DeviceOrientation*/
const Value V24415_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24415_304 = {S24, 0, &V24415_304_57227, &P24415_302};	/*Property = FileReader*/
const Value V24415_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24415_305 = {S25, 0, &V24415_305_57231, &P24415_304};	/*Property = FileSaver*/
const Value V24415_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24415_306 = {S26, 0, &V24415_306_57237, &P24415_305};	/*Property = FileWriter*/
const Value V24415_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24415_307 = {S27, 0, &V24415_307_57239, &P24415_306};	/*Property = FormData*/
const Value V24415_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24415_309 = {S28, 0, &V24415_309_57246, &P24415_307};	/*Property = Fullscreen*/
const Value V24415_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24415_310 = {S29, 0, &V24415_310_57248, &P24415_309};	/*Property = GeoLocation*/
const Value V24415_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24415_312 = {S30, 0, &V24415_312_57254, &P24415_310};	/*Property = History*/
const Value V24415_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24415_314 = {S31, 0, &V24415_314_57260, &P24415_312};	/*Property = Html5*/
const Value V24415_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24415_313 = {S32, 0, &V24415_313_57257, &P24415_314};	/*Property = Html-Media-Capture*/
const Value V24415_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24415_315 = {S33, 0, &V24415_315_57264, &P24415_313};	/*Property = Iframe*/
const Value V24415_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24415_316 = {S34, 0, &V24415_316_57266, &P24415_315};	/*Property = IndexedDB*/
const Value V24415_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24415_317 = {S35, 0, &V24415_317_57269, &P24415_316};	/*Property = Json*/
const Value V24415_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24415_15 = {S37, 0, &V24415_15_28, &P24415_317};	/*Property = LayoutEngine*/
const Value V24415_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24415_344 = {S38, 0, &V24415_344_57272, &P24415_15};	/*Property = Masking*/
const Value V24415_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24415_320 = {S39, 0, &V24415_320_57290, &P24415_344};	/*Property = PostMessage*/
const Value V24415_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24415_321 = {S40, 0, &V24415_321_57293, &P24415_320};	/*Property = Progress*/
const Value V24415_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24415_322 = {S41, 0, &V24415_322_57296, &P24415_321};	/*Property = Prompts*/
const Value V24415_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24415_323 = {S42, 0, &V24415_323_57299, &P24415_322};	/*Property = Selector*/
const Value V24415_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24415_324 = {S43, 0, &V24415_324_57305, &P24415_323};	/*Property = Svg*/
const Value V24415_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24415_327 = {S44, 0, &V24415_327_57314, &P24415_324};	/*Property = TouchEvents*/
const Value V24415_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24415_328 = {S45, 0, &V24415_328_57318, &P24415_327};	/*Property = Track*/
const Value V24415_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24415_329 = {S46, 0, &V24415_329_57323, &P24415_328};	/*Property = Video*/
const Value V24415_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24415_330 = {S47, 0, &V24415_330_57326, &P24415_329};	/*Property = Viewport*/
const Value V24415_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24415_332 = {S48, 0, &V24415_332_57338, &P24415_330};	/*Property = WebWorkers*/
const Value V24415_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24415_333 = {S49, 0, &V24415_333_57342, &P24415_332};	/*Property = Xhr2*/
const Profile P24415 = {"24415", &P24415_333};	/*Profile ID - 24415*/
const Profile P24371 = {"24371", NULL};	/*Profile ID - 24371*/
const Value V24546_1_1 = {S2, NULL};	/*Value = True*/
const Property P24546_1 = {S50, 0, &V24546_1_1, NULL};	/*Property = IsMobile*/
const Value V24546_5_1832 = {S84, NULL};	/*Value = 600*/
const Property P24546_5 = {S52, 0, &V24546_5_1832, &P24546_1};	/*Property = ScreenPixelsHeight*/
const Value V24546_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P24546_4 = {S54, 0, &V24546_4_5645, &P24546_5};	/*Property = ScreenPixelsWidth*/
const Profile P24546 = {"24546", &P24546_4};	/*Profile ID - 24546*/
const Value V24367_1_1 = {S2, NULL};	/*Value = True*/
const Property P24367_1 = {S50, 0, &V24367_1_1, NULL};	/*Property = IsMobile*/
const Value V24367_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24367_5 = {S52, 0, &V24367_5_9981, &P24367_1};	/*Property = ScreenPixelsHeight*/
const Value V24367_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P24367_4 = {S54, 0, &V24367_4_2336, &P24367_5};	/*Property = ScreenPixelsWidth*/
const Profile P24367 = {"24367", &P24367_4};	/*Profile ID - 24367*/
const Value V24624_1_1 = {S2, NULL};	/*Value = True*/
const Property P24624_1 = {S50, 0, &V24624_1_1, NULL};	/*Property = IsMobile*/
const Value V24624_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P24624_5 = {S52, 0, &V24624_5_1458, &P24624_1};	/*Property = ScreenPixelsHeight*/
const Value V24624_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P24624_4 = {S54, 0, &V24624_4_37617, &P24624_5};	/*Property = ScreenPixelsWidth*/
const Profile P24624 = {"24624", &P24624_4};	/*Profile ID - 24624*/
const Value V24538_1_1 = {S2, NULL};	/*Value = True*/
const Property P24538_1 = {S50, 0, &V24538_1_1, NULL};	/*Property = IsMobile*/
const Value V24538_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24538_5 = {S52, 0, &V24538_5_9981, &P24538_1};	/*Property = ScreenPixelsHeight*/
const Value V24538_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24538_4 = {S54, 0, &V24538_4_39098, &P24538_5};	/*Property = ScreenPixelsWidth*/
const Profile P24538 = {"24538", &P24538_4};	/*Profile ID - 24538*/
const Value V24721_1_1 = {S2, NULL};	/*Value = True*/
const Property P24721_1 = {S50, 0, &V24721_1_1, NULL};	/*Property = IsMobile*/
const Value V24721_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P24721_5 = {S52, 0, &V24721_5_1269, &P24721_1};	/*Property = ScreenPixelsHeight*/
const Value V24721_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24721_4 = {S54, 0, &V24721_4_1229, &P24721_5};	/*Property = ScreenPixelsWidth*/
const Profile P24721 = {"24721", &P24721_4};	/*Profile ID - 24721*/
const Value V24413_1_1 = {S2, NULL};	/*Value = True*/
const Property P24413_1 = {S50, 0, &V24413_1_1, NULL};	/*Property = IsMobile*/
const Value V24413_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P24413_5 = {S52, 0, &V24413_5_37569, &P24413_1};	/*Property = ScreenPixelsHeight*/
const Value V24413_4_53077 = {S129, NULL};	/*Value = 1366*/
const Property P24413_4 = {S54, 0, &V24413_4_53077, &P24413_5};	/*Property = ScreenPixelsWidth*/
const Profile P24413 = {"24413", &P24413_4};	/*Profile ID - 24413*/
const Value V24541_1_1 = {S2, NULL};	/*Value = True*/
const Property P24541_1 = {S50, 0, &V24541_1_1, NULL};	/*Property = IsMobile*/
const Value V24541_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24541_5 = {S52, 0, &V24541_5_9981, &P24541_1};	/*Property = ScreenPixelsHeight*/
const Value V24541_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24541_4 = {S54, 0, &V24541_4_39098, &P24541_5};	/*Property = ScreenPixelsWidth*/
const Profile P24541 = {"24541", &P24541_4};	/*Profile ID - 24541*/
const Value V24419_1_1 = {S2, NULL};	/*Value = True*/
const Property P24419_1 = {S50, 0, &V24419_1_1, NULL};	/*Property = IsMobile*/
const Value V24419_5_37526 = {S122, NULL};	/*Value = 1200*/
const Property P24419_5 = {S52, 0, &V24419_5_37526, &P24419_1};	/*Property = ScreenPixelsHeight*/
const Value V24419_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24419_4 = {S54, 0, &V24419_4_57095, &P24419_5};	/*Property = ScreenPixelsWidth*/
const Profile P24419 = {"24419", &P24419_4};	/*Profile ID - 24419*/
const Value V24741_1_1 = {S2, NULL};	/*Value = True*/
const Property P24741_1 = {S50, 0, &V24741_1_1, NULL};	/*Property = IsMobile*/
const Value V24741_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P24741_5 = {S52, 0, &V24741_5_1458, &P24741_1};	/*Property = ScreenPixelsHeight*/
const Value V24741_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P24741_4 = {S54, 0, &V24741_4_37617, &P24741_5};	/*Property = ScreenPixelsWidth*/
const Profile P24741 = {"24741", &P24741_4};	/*Profile ID - 24741*/
const Value V24653_1_1 = {S2, NULL};	/*Value = True*/
const Property P24653_1 = {S50, 0, &V24653_1_1, NULL};	/*Property = IsMobile*/
const Value V24653_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24653_5 = {S52, 0, &V24653_5_9981, &P24653_1};	/*Property = ScreenPixelsHeight*/
const Value V24653_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P24653_4 = {S54, 0, &V24653_4_2336, &P24653_5};	/*Property = ScreenPixelsWidth*/
const Profile P24653 = {"24653", &P24653_4};	/*Profile ID - 24653*/
const Value V20847_1_1 = {S2, NULL};	/*Value = True*/
const Property P20847_1 = {S50, 0, &V20847_1_1, NULL};	/*Property = IsMobile*/
const Value V20847_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P20847_5 = {S52, 0, &V20847_5_9981, &P20847_1};	/*Property = ScreenPixelsHeight*/
const Value V20847_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P20847_4 = {S54, 0, &V20847_4_2336, &P20847_5};	/*Property = ScreenPixelsWidth*/
const Profile P20847 = {"20847", &P20847_4};	/*Profile ID - 20847*/
const Value V24504_1_1 = {S2, NULL};	/*Value = True*/
const Property P24504_1 = {S50, 0, &V24504_1_1, NULL};	/*Property = IsMobile*/
const Value V24504_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24504_5 = {S52, 0, &V24504_5_1403, &P24504_1};	/*Property = ScreenPixelsHeight*/
const Value V24504_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24504_4 = {S54, 0, &V24504_4_1404, &P24504_5};	/*Property = ScreenPixelsWidth*/
const Profile P24504 = {"24504", &P24504_4};	/*Profile ID - 24504*/
const Value V24566_1_1 = {S2, NULL};	/*Value = True*/
const Property P24566_1 = {S50, 0, &V24566_1_1, NULL};	/*Property = IsMobile*/
const Value V24566_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24566_5 = {S52, 0, &V24566_5_1227, &P24566_1};	/*Property = ScreenPixelsHeight*/
const Value V24566_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24566_4 = {S54, 0, &V24566_4_1229, &P24566_5};	/*Property = ScreenPixelsWidth*/
const Profile P24566 = {"24566", &P24566_4};	/*Profile ID - 24566*/
const Value V24628_1_1 = {S2, NULL};	/*Value = True*/
const Property P24628_1 = {S50, 0, &V24628_1_1, NULL};	/*Property = IsMobile*/
const Value V24628_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P24628_5 = {S52, 0, &V24628_5_37, &P24628_1};	/*Property = ScreenPixelsHeight*/
const Value V24628_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P24628_4 = {S54, 0, &V24628_4_36, &P24628_5};	/*Property = ScreenPixelsWidth*/
const Profile P24628 = {"24628", &P24628_4};	/*Profile ID - 24628*/
const Value V24513_1_1 = {S2, NULL};	/*Value = True*/
const Property P24513_1 = {S50, 0, &V24513_1_1, NULL};	/*Property = IsMobile*/
const Value V24513_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24513_5 = {S52, 0, &V24513_5_9981, &P24513_1};	/*Property = ScreenPixelsHeight*/
const Value V24513_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24513_4 = {S54, 0, &V24513_4_39098, &P24513_5};	/*Property = ScreenPixelsWidth*/
const Profile P24513 = {"24513", &P24513_4};	/*Profile ID - 24513*/
const Value V24400_1_1 = {S2, NULL};	/*Value = True*/
const Property P24400_1 = {S50, 0, &V24400_1_1, NULL};	/*Property = IsMobile*/
const Value V24400_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24400_5 = {S52, 0, &V24400_5_9981, &P24400_1};	/*Property = ScreenPixelsHeight*/
const Value V24400_4_2336 = {S51, NULL};	/*Value = 800*/
const Property P24400_4 = {S54, 0, &V24400_4_2336, &P24400_5};	/*Property = ScreenPixelsWidth*/
const Profile P24400 = {"24400", &P24400_4};	/*Profile ID - 24400*/
const Value V23324_1_1 = {S2, NULL};	/*Value = True*/
const Property P23324_1 = {S50, 0, &V23324_1_1, NULL};	/*Property = IsMobile*/
const Value V23324_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P23324_5 = {S52, 0, &V23324_5_1222, &P23324_1};	/*Property = ScreenPixelsHeight*/
const Value V23324_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P23324_4 = {S54, 0, &V23324_4_1224, &P23324_5};	/*Property = ScreenPixelsWidth*/
const Profile P23324 = {"23324", &P23324_4};	/*Profile ID - 23324*/
const Value V24387_1_1 = {S2, NULL};	/*Value = True*/
const Property P24387_1 = {S50, 0, &V24387_1_1, NULL};	/*Property = IsMobile*/
const Value V24387_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24387_5 = {S52, 0, &V24387_5_9981, &P24387_1};	/*Property = ScreenPixelsHeight*/
const Value V24387_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24387_4 = {S54, 0, &V24387_4_39098, &P24387_5};	/*Property = ScreenPixelsWidth*/
const Profile P24387 = {"24387", &P24387_4};	/*Profile ID - 24387*/
const Value V24494_1_1 = {S2, NULL};	/*Value = True*/
const Property P24494_1 = {S50, 0, &V24494_1_1, NULL};	/*Property = IsMobile*/
const Value V24494_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24494_5 = {S52, 0, &V24494_5_1403, &P24494_1};	/*Property = ScreenPixelsHeight*/
const Value V24494_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24494_4 = {S54, 0, &V24494_4_1404, &P24494_5};	/*Property = ScreenPixelsWidth*/
const Profile P24494 = {"24494", &P24494_4};	/*Profile ID - 24494*/
const Value V24740_1_1 = {S2, NULL};	/*Value = True*/
const Property P24740_1 = {S50, 0, &V24740_1_1, NULL};	/*Property = IsMobile*/
const Value V24740_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24740_5 = {S52, 0, &V24740_5_1277, &P24740_1};	/*Property = ScreenPixelsHeight*/
const Value V24740_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24740_4 = {S54, 0, &V24740_4_1279, &P24740_5};	/*Property = ScreenPixelsWidth*/
const Profile P24740 = {"24740", &P24740_4};	/*Profile ID - 24740*/
const Value V24612_1_1 = {S2, NULL};	/*Value = True*/
const Property P24612_1 = {S50, 0, &V24612_1_1, NULL};	/*Property = IsMobile*/
const Value V24612_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P24612_5 = {S52, 0, &V24612_5_1222, &P24612_1};	/*Property = ScreenPixelsHeight*/
const Value V24612_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P24612_4 = {S54, 0, &V24612_4_1224, &P24612_5};	/*Property = ScreenPixelsWidth*/
const Profile P24612 = {"24612", &P24612_4};	/*Profile ID - 24612*/
const Value V24401_1_1 = {S2, NULL};	/*Value = True*/
const Property P24401_1 = {S50, 0, &V24401_1_1, NULL};	/*Property = IsMobile*/
const Value V24401_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24401_5 = {S52, 0, &V24401_5_1403, &P24401_1};	/*Property = ScreenPixelsHeight*/
const Value V24401_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24401_4 = {S54, 0, &V24401_4_1404, &P24401_5};	/*Property = ScreenPixelsWidth*/
const Profile P24401 = {"24401", &P24401_4};	/*Profile ID - 24401*/
const Value V16568_1_1 = {S2, NULL};	/*Value = True*/
const Property P16568_1 = {S50, 0, &V16568_1_1, NULL};	/*Property = IsMobile*/
const Value V16568_5_1304 = {S53, NULL};	/*Value = 400*/
const Property P16568_5 = {S52, 0, &V16568_5_1304, &P16568_1};	/*Property = ScreenPixelsHeight*/
const Value V16568_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P16568_4 = {S54, 0, &V16568_4_1224, &P16568_5};	/*Property = ScreenPixelsWidth*/
const Profile P16568 = {"16568", &P16568_4};	/*Profile ID - 16568*/
const Value V24341_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24341_279 = {S1, 0, &V24341_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24341_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24341_282 = {S3, 0, &V24341_282_57154, &P24341_279};	/*Property = BlobBuilder*/
const Value V24341_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24341_350 = {S4, 0, &V24341_350_57444, &P24341_282};	/*Property = Canvas*/
const Value V24341_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24341_283 = {S5, 0, &V24341_283_57159, &P24341_350};	/*Property = CssBackground*/
const Value V24341_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24341_284 = {S6, 0, &V24341_284_57162, &P24341_283};	/*Property = CssBorderImage*/
const Value V24341_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24341_285 = {S7, 0, &V24341_285_57166, &P24341_284};	/*Property = CssCanvas*/
const Value V24341_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24341_286 = {S8, 0, &V24341_286_57168, &P24341_285};	/*Property = CssColor*/
const Value V24341_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24341_237 = {S9, 0, &V24341_237_55786, &P24341_286};	/*Property = CssColumn*/
const Value V24341_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24341_287 = {S10, 0, &V24341_287_57172, &P24341_237};	/*Property = CssFlexbox*/
const Value V24341_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24341_288 = {S11, 0, &V24341_288_57174, &P24341_287};	/*Property = CssFont*/
const Value V24341_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24341_289 = {S12, 0, &V24341_289_57759, &P24341_288};	/*Property = CssImages*/
const Value V24341_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24341_291 = {S13, 0, &V24341_291_57180, &P24341_289};	/*Property = CssMediaQueries*/
const Value V24341_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24341_292 = {S14, 0, &V24341_292_57183, &P24341_291};	/*Property = CssMinMax*/
const Value V24341_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24341_293 = {S15, 0, &V24341_293_57187, &P24341_292};	/*Property = CssOverflow*/
const Value V24341_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24341_294 = {S16, 0, &V24341_294_57189, &P24341_293};	/*Property = CssPosition*/
const Value V24341_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24341_295 = {S17, 0, &V24341_295_57192, &P24341_294};	/*Property = CssText*/
const Value V24341_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24341_297 = {S18, 0, &V24341_297_57195, &P24341_295};	/*Property = CssTransforms*/
const Value V24341_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24341_241 = {S19, 0, &V24341_241_55799, &P24341_297};	/*Property = CssTransitions*/
const Value V24341_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24341_298 = {S20, 0, &V24341_298_57206, &P24341_241};	/*Property = CssUI*/
const Value V24341_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24341_300 = {S21, 0, &V24341_300_57215, &P24341_298};	/*Property = DataSet*/
const Value V24341_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24341_301 = {S22, 0, &V24341_301_57218, &P24341_300};	/*Property = DataUrl*/
const Value V24341_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24341_302 = {S23, 0, &V24341_302_57221, &P24341_301};	/*Property = DeviceOrientation*/
const Value V24341_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24341_304 = {S24, 0, &V24341_304_57227, &P24341_302};	/*Property = FileReader*/
const Value V24341_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24341_305 = {S25, 0, &V24341_305_57231, &P24341_304};	/*Property = FileSaver*/
const Value V24341_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24341_306 = {S26, 0, &V24341_306_57237, &P24341_305};	/*Property = FileWriter*/
const Value V24341_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24341_307 = {S27, 0, &V24341_307_57239, &P24341_306};	/*Property = FormData*/
const Value V24341_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24341_309 = {S28, 0, &V24341_309_57246, &P24341_307};	/*Property = Fullscreen*/
const Value V24341_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24341_310 = {S29, 0, &V24341_310_57248, &P24341_309};	/*Property = GeoLocation*/
const Value V24341_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24341_312 = {S30, 0, &V24341_312_57254, &P24341_310};	/*Property = History*/
const Value V24341_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24341_314 = {S31, 0, &V24341_314_57260, &P24341_312};	/*Property = Html5*/
const Value V24341_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24341_313 = {S32, 0, &V24341_313_57257, &P24341_314};	/*Property = Html-Media-Capture*/
const Value V24341_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24341_315 = {S33, 0, &V24341_315_57264, &P24341_313};	/*Property = Iframe*/
const Value V24341_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24341_316 = {S34, 0, &V24341_316_57266, &P24341_315};	/*Property = IndexedDB*/
const Value V24341_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24341_317 = {S35, 0, &V24341_317_57269, &P24341_316};	/*Property = Json*/
const Value V24341_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24341_15 = {S37, 0, &V24341_15_54167, &P24341_317};	/*Property = LayoutEngine*/
const Value V24341_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24341_344 = {S38, 0, &V24341_344_57272, &P24341_15};	/*Property = Masking*/
const Value V24341_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24341_320 = {S39, 0, &V24341_320_57290, &P24341_344};	/*Property = PostMessage*/
const Value V24341_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24341_321 = {S40, 0, &V24341_321_57293, &P24341_320};	/*Property = Progress*/
const Value V24341_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24341_322 = {S41, 0, &V24341_322_57296, &P24341_321};	/*Property = Prompts*/
const Value V24341_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24341_323 = {S42, 0, &V24341_323_57299, &P24341_322};	/*Property = Selector*/
const Value V24341_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24341_324 = {S43, 0, &V24341_324_57306, &P24341_323};	/*Property = Svg*/
const Value V24341_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24341_327 = {S44, 0, &V24341_327_57314, &P24341_324};	/*Property = TouchEvents*/
const Value V24341_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24341_328 = {S45, 0, &V24341_328_57318, &P24341_327};	/*Property = Track*/
const Value V24341_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24341_329 = {S46, 0, &V24341_329_57323, &P24341_328};	/*Property = Video*/
const Value V24341_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24341_330 = {S47, 0, &V24341_330_57326, &P24341_329};	/*Property = Viewport*/
const Value V24341_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24341_332 = {S48, 0, &V24341_332_57339, &P24341_330};	/*Property = WebWorkers*/
const Value V24341_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24341_333 = {S49, 0, &V24341_333_57342, &P24341_332};	/*Property = Xhr2*/
const Profile P24341 = {"24341", &P24341_333};	/*Profile ID - 24341*/
const Value V24713_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24713_279 = {S1, 0, &V24713_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24713_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24713_282 = {S3, 0, &V24713_282_57154, &P24713_279};	/*Property = BlobBuilder*/
const Value V24713_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24713_350 = {S4, 0, &V24713_350_57444, &P24713_282};	/*Property = Canvas*/
const Value V24713_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24713_283 = {S5, 0, &V24713_283_57159, &P24713_350};	/*Property = CssBackground*/
const Value V24713_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24713_284 = {S6, 0, &V24713_284_57162, &P24713_283};	/*Property = CssBorderImage*/
const Value V24713_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24713_285 = {S7, 0, &V24713_285_57166, &P24713_284};	/*Property = CssCanvas*/
const Value V24713_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24713_286 = {S8, 0, &V24713_286_57168, &P24713_285};	/*Property = CssColor*/
const Value V24713_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24713_237 = {S9, 0, &V24713_237_55786, &P24713_286};	/*Property = CssColumn*/
const Value V24713_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24713_287 = {S10, 0, &V24713_287_57172, &P24713_237};	/*Property = CssFlexbox*/
const Value V24713_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24713_288 = {S11, 0, &V24713_288_57174, &P24713_287};	/*Property = CssFont*/
const Value V24713_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24713_289 = {S12, 0, &V24713_289_57759, &P24713_288};	/*Property = CssImages*/
const Value V24713_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24713_291 = {S13, 0, &V24713_291_57180, &P24713_289};	/*Property = CssMediaQueries*/
const Value V24713_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24713_292 = {S14, 0, &V24713_292_57183, &P24713_291};	/*Property = CssMinMax*/
const Value V24713_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24713_293 = {S15, 0, &V24713_293_57187, &P24713_292};	/*Property = CssOverflow*/
const Value V24713_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24713_294 = {S16, 0, &V24713_294_57189, &P24713_293};	/*Property = CssPosition*/
const Value V24713_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24713_295 = {S17, 0, &V24713_295_57192, &P24713_294};	/*Property = CssText*/
const Value V24713_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24713_297 = {S18, 0, &V24713_297_57195, &P24713_295};	/*Property = CssTransforms*/
const Value V24713_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24713_241 = {S19, 0, &V24713_241_55799, &P24713_297};	/*Property = CssTransitions*/
const Value V24713_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24713_298 = {S20, 0, &V24713_298_57206, &P24713_241};	/*Property = CssUI*/
const Value V24713_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24713_300 = {S21, 0, &V24713_300_57215, &P24713_298};	/*Property = DataSet*/
const Value V24713_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24713_301 = {S22, 0, &V24713_301_57218, &P24713_300};	/*Property = DataUrl*/
const Value V24713_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24713_302 = {S23, 0, &V24713_302_57221, &P24713_301};	/*Property = DeviceOrientation*/
const Value V24713_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24713_304 = {S24, 0, &V24713_304_57227, &P24713_302};	/*Property = FileReader*/
const Value V24713_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24713_305 = {S25, 0, &V24713_305_57231, &P24713_304};	/*Property = FileSaver*/
const Value V24713_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24713_306 = {S26, 0, &V24713_306_57237, &P24713_305};	/*Property = FileWriter*/
const Value V24713_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24713_307 = {S27, 0, &V24713_307_57239, &P24713_306};	/*Property = FormData*/
const Value V24713_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24713_309 = {S28, 0, &V24713_309_57246, &P24713_307};	/*Property = Fullscreen*/
const Value V24713_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24713_310 = {S29, 0, &V24713_310_57248, &P24713_309};	/*Property = GeoLocation*/
const Value V24713_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24713_312 = {S30, 0, &V24713_312_57254, &P24713_310};	/*Property = History*/
const Value V24713_314_57261 = {S0, NULL};	/*Value = False*/
const Property P24713_314 = {S31, 0, &V24713_314_57261, &P24713_312};	/*Property = Html5*/
const Value V24713_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24713_313 = {S32, 0, &V24713_313_57258, &P24713_314};	/*Property = Html-Media-Capture*/
const Value V24713_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24713_315 = {S33, 0, &V24713_315_57264, &P24713_313};	/*Property = Iframe*/
const Value V24713_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24713_316 = {S34, 0, &V24713_316_57266, &P24713_315};	/*Property = IndexedDB*/
const Value V24713_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24713_317 = {S35, 0, &V24713_317_57269, &P24713_316};	/*Property = Json*/
const Value V24713_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P24713_15 = {S37, 0, &V24713_15_29, &P24713_317};	/*Property = LayoutEngine*/
const Value V24713_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24713_344 = {S38, 0, &V24713_344_57272, &P24713_15};	/*Property = Masking*/
const Value V24713_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24713_320 = {S39, 0, &V24713_320_57290, &P24713_344};	/*Property = PostMessage*/
const Value V24713_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24713_321 = {S40, 0, &V24713_321_57293, &P24713_320};	/*Property = Progress*/
const Value V24713_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24713_322 = {S41, 0, &V24713_322_57296, &P24713_321};	/*Property = Prompts*/
const Value V24713_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24713_323 = {S42, 0, &V24713_323_57299, &P24713_322};	/*Property = Selector*/
const Value V24713_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24713_324 = {S43, 0, &V24713_324_57305, &P24713_323};	/*Property = Svg*/
const Value V24713_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24713_327 = {S44, 0, &V24713_327_57314, &P24713_324};	/*Property = TouchEvents*/
const Value V24713_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24713_328 = {S45, 0, &V24713_328_57318, &P24713_327};	/*Property = Track*/
const Value V24713_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24713_329 = {S46, 0, &V24713_329_57323, &P24713_328};	/*Property = Video*/
const Value V24713_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24713_330 = {S47, 0, &V24713_330_57326, &P24713_329};	/*Property = Viewport*/
const Value V24713_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24713_332 = {S48, 0, &V24713_332_57338, &P24713_330};	/*Property = WebWorkers*/
const Value V24713_333_57341 = {S2, NULL};	/*Value = True*/
const Property P24713_333 = {S49, 0, &V24713_333_57341, &P24713_332};	/*Property = Xhr2*/
const Profile P24713 = {"24713", &P24713_333};	/*Profile ID - 24713*/
const Value V24425_1_1 = {S2, NULL};	/*Value = True*/
const Property P24425_1 = {S50, 0, &V24425_1_1, NULL};	/*Property = IsMobile*/
const Value V24425_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P24425_5 = {S52, 0, &V24425_5_1458, &P24425_1};	/*Property = ScreenPixelsHeight*/
const Value V24425_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P24425_4 = {S54, 0, &V24425_4_1460, &P24425_5};	/*Property = ScreenPixelsWidth*/
const Profile P24425 = {"24425", &P24425_4};	/*Profile ID - 24425*/
const Profile P24346 = {"24346", NULL};	/*Profile ID - 24346*/
const Value V24396_1_1 = {S2, NULL};	/*Value = True*/
const Property P24396_1 = {S50, 0, &V24396_1_1, NULL};	/*Property = IsMobile*/
const Value V24396_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24396_5 = {S52, 0, &V24396_5_1277, &P24396_1};	/*Property = ScreenPixelsHeight*/
const Value V24396_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24396_4 = {S54, 0, &V24396_4_1279, &P24396_5};	/*Property = ScreenPixelsWidth*/
const Profile P24396 = {"24396", &P24396_4};	/*Profile ID - 24396*/
const Value V24545_1_1 = {S2, NULL};	/*Value = True*/
const Property P24545_1 = {S50, 0, &V24545_1_1, NULL};	/*Property = IsMobile*/
const Value V24545_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24545_5 = {S52, 0, &V24545_5_9981, &P24545_1};	/*Property = ScreenPixelsHeight*/
const Value V24545_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24545_4 = {S54, 0, &V24545_4_39098, &P24545_5};	/*Property = ScreenPixelsWidth*/
const Profile P24545 = {"24545", &P24545_4};	/*Profile ID - 24545*/
const Value V24480_1_1 = {S2, NULL};	/*Value = True*/
const Property P24480_1 = {S50, 0, &V24480_1_1, NULL};	/*Property = IsMobile*/
const Value V24480_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24480_5 = {S52, 0, &V24480_5_1227, &P24480_1};	/*Property = ScreenPixelsHeight*/
const Value V24480_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24480_4 = {S54, 0, &V24480_4_1229, &P24480_5};	/*Property = ScreenPixelsWidth*/
const Profile P24480 = {"24480", &P24480_4};	/*Profile ID - 24480*/
const Value V24636_1_1 = {S2, NULL};	/*Value = True*/
const Property P24636_1 = {S50, 0, &V24636_1_1, NULL};	/*Property = IsMobile*/
const Value V24636_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P24636_5 = {S52, 0, &V24636_5_1269, &P24636_1};	/*Property = ScreenPixelsHeight*/
const Value V24636_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24636_4 = {S54, 0, &V24636_4_1229, &P24636_5};	/*Property = ScreenPixelsWidth*/
const Profile P24636 = {"24636", &P24636_4};	/*Profile ID - 24636*/
const Value V24454_1_1 = {S2, NULL};	/*Value = True*/
const Property P24454_1 = {S50, 0, &V24454_1_1, NULL};	/*Property = IsMobile*/
const Value V24454_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24454_5 = {S52, 0, &V24454_5_1227, &P24454_1};	/*Property = ScreenPixelsHeight*/
const Value V24454_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24454_4 = {S54, 0, &V24454_4_1229, &P24454_5};	/*Property = ScreenPixelsWidth*/
const Profile P24454 = {"24454", &P24454_4};	/*Profile ID - 24454*/
const Value V24679_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_279 = {S1, 0, &V24679_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24679_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_282 = {S3, 0, &V24679_282_57155, &P24679_279};	/*Property = BlobBuilder*/
const Value V24679_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_350 = {S4, 0, &V24679_350_57446, &P24679_282};	/*Property = Canvas*/
const Value V24679_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_283 = {S5, 0, &V24679_283_57161, &P24679_350};	/*Property = CssBackground*/
const Value V24679_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_284 = {S6, 0, &V24679_284_57164, &P24679_283};	/*Property = CssBorderImage*/
const Value V24679_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_285 = {S7, 0, &V24679_285_57167, &P24679_284};	/*Property = CssCanvas*/
const Value V24679_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_286 = {S8, 0, &V24679_286_57170, &P24679_285};	/*Property = CssColor*/
const Value V24679_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_237 = {S9, 0, &V24679_237_55786, &P24679_286};	/*Property = CssColumn*/
const Value V24679_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_287 = {S10, 0, &V24679_287_57173, &P24679_237};	/*Property = CssFlexbox*/
const Value V24679_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_288 = {S11, 0, &V24679_288_57176, &P24679_287};	/*Property = CssFont*/
const Value V24679_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_289 = {S12, 0, &V24679_289_86899, &P24679_288};	/*Property = CssImages*/
const Value V24679_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_291 = {S13, 0, &V24679_291_57182, &P24679_289};	/*Property = CssMediaQueries*/
const Value V24679_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_292 = {S14, 0, &V24679_292_57185, &P24679_291};	/*Property = CssMinMax*/
const Value V24679_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_293 = {S15, 0, &V24679_293_57188, &P24679_292};	/*Property = CssOverflow*/
const Value V24679_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_294 = {S16, 0, &V24679_294_57191, &P24679_293};	/*Property = CssPosition*/
const Value V24679_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_295 = {S17, 0, &V24679_295_57194, &P24679_294};	/*Property = CssText*/
const Value V24679_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_297 = {S18, 0, &V24679_297_57197, &P24679_295};	/*Property = CssTransforms*/
const Value V24679_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_241 = {S19, 0, &V24679_241_55801, &P24679_297};	/*Property = CssTransitions*/
const Value V24679_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_298 = {S20, 0, &V24679_298_57208, &P24679_241};	/*Property = CssUI*/
const Value V24679_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_300 = {S21, 0, &V24679_300_57217, &P24679_298};	/*Property = DataSet*/
const Value V24679_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_301 = {S22, 0, &V24679_301_57220, &P24679_300};	/*Property = DataUrl*/
const Value V24679_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_302 = {S23, 0, &V24679_302_57223, &P24679_301};	/*Property = DeviceOrientation*/
const Value V24679_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_304 = {S24, 0, &V24679_304_57229, &P24679_302};	/*Property = FileReader*/
const Value V24679_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_305 = {S25, 0, &V24679_305_57232, &P24679_304};	/*Property = FileSaver*/
const Value V24679_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_306 = {S26, 0, &V24679_306_57238, &P24679_305};	/*Property = FileWriter*/
const Value V24679_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_307 = {S27, 0, &V24679_307_57241, &P24679_306};	/*Property = FormData*/
const Value V24679_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_309 = {S28, 0, &V24679_309_57247, &P24679_307};	/*Property = Fullscreen*/
const Value V24679_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_310 = {S29, 0, &V24679_310_57250, &P24679_309};	/*Property = GeoLocation*/
const Value V24679_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_312 = {S30, 0, &V24679_312_57256, &P24679_310};	/*Property = History*/
const Value V24679_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_314 = {S31, 0, &V24679_314_57262, &P24679_312};	/*Property = Html5*/
const Value V24679_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_313 = {S32, 0, &V24679_313_57259, &P24679_314};	/*Property = Html-Media-Capture*/
const Value V24679_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_315 = {S33, 0, &V24679_315_57265, &P24679_313};	/*Property = Iframe*/
const Value V24679_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_316 = {S34, 0, &V24679_316_57268, &P24679_315};	/*Property = IndexedDB*/
const Value V24679_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_317 = {S35, 0, &V24679_317_57271, &P24679_316};	/*Property = Json*/
const Value V24679_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24679_15 = {S37, 0, &V24679_15_90837, &P24679_317};	/*Property = LayoutEngine*/
const Value V24679_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_344 = {S38, 0, &V24679_344_57274, &P24679_15};	/*Property = Masking*/
const Value V24679_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_320 = {S39, 0, &V24679_320_57292, &P24679_344};	/*Property = PostMessage*/
const Value V24679_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_321 = {S40, 0, &V24679_321_57295, &P24679_320};	/*Property = Progress*/
const Value V24679_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_322 = {S41, 0, &V24679_322_57298, &P24679_321};	/*Property = Prompts*/
const Value V24679_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_323 = {S42, 0, &V24679_323_57301, &P24679_322};	/*Property = Selector*/
const Value V24679_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_324 = {S43, 0, &V24679_324_57307, &P24679_323};	/*Property = Svg*/
const Value V24679_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_327 = {S44, 0, &V24679_327_57316, &P24679_324};	/*Property = TouchEvents*/
const Value V24679_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_328 = {S45, 0, &V24679_328_57319, &P24679_327};	/*Property = Track*/
const Value V24679_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_329 = {S46, 0, &V24679_329_57325, &P24679_328};	/*Property = Video*/
const Value V24679_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_330 = {S47, 0, &V24679_330_57328, &P24679_329};	/*Property = Viewport*/
const Value V24679_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_332 = {S48, 0, &V24679_332_57340, &P24679_330};	/*Property = WebWorkers*/
const Value V24679_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24679_333 = {S49, 0, &V24679_333_57343, &P24679_332};	/*Property = Xhr2*/
const Profile P24679 = {"24679", &P24679_333};	/*Profile ID - 24679*/
const Value V24334_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24334_279 = {S1, 0, &V24334_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24334_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24334_282 = {S3, 0, &V24334_282_57154, &P24334_279};	/*Property = BlobBuilder*/
const Value V24334_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24334_350 = {S4, 0, &V24334_350_57444, &P24334_282};	/*Property = Canvas*/
const Value V24334_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24334_283 = {S5, 0, &V24334_283_57159, &P24334_350};	/*Property = CssBackground*/
const Value V24334_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24334_284 = {S6, 0, &V24334_284_57162, &P24334_283};	/*Property = CssBorderImage*/
const Value V24334_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24334_285 = {S7, 0, &V24334_285_57166, &P24334_284};	/*Property = CssCanvas*/
const Value V24334_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24334_286 = {S8, 0, &V24334_286_57168, &P24334_285};	/*Property = CssColor*/
const Value V24334_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24334_237 = {S9, 0, &V24334_237_55786, &P24334_286};	/*Property = CssColumn*/
const Value V24334_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24334_287 = {S10, 0, &V24334_287_57172, &P24334_237};	/*Property = CssFlexbox*/
const Value V24334_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24334_288 = {S11, 0, &V24334_288_57174, &P24334_287};	/*Property = CssFont*/
const Value V24334_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24334_289 = {S12, 0, &V24334_289_57759, &P24334_288};	/*Property = CssImages*/
const Value V24334_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24334_291 = {S13, 0, &V24334_291_57180, &P24334_289};	/*Property = CssMediaQueries*/
const Value V24334_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24334_292 = {S14, 0, &V24334_292_57183, &P24334_291};	/*Property = CssMinMax*/
const Value V24334_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24334_293 = {S15, 0, &V24334_293_57187, &P24334_292};	/*Property = CssOverflow*/
const Value V24334_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24334_294 = {S16, 0, &V24334_294_57189, &P24334_293};	/*Property = CssPosition*/
const Value V24334_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24334_295 = {S17, 0, &V24334_295_57192, &P24334_294};	/*Property = CssText*/
const Value V24334_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24334_297 = {S18, 0, &V24334_297_57195, &P24334_295};	/*Property = CssTransforms*/
const Value V24334_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24334_241 = {S19, 0, &V24334_241_55799, &P24334_297};	/*Property = CssTransitions*/
const Value V24334_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24334_298 = {S20, 0, &V24334_298_57206, &P24334_241};	/*Property = CssUI*/
const Value V24334_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24334_300 = {S21, 0, &V24334_300_57215, &P24334_298};	/*Property = DataSet*/
const Value V24334_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24334_301 = {S22, 0, &V24334_301_57218, &P24334_300};	/*Property = DataUrl*/
const Value V24334_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24334_302 = {S23, 0, &V24334_302_57222, &P24334_301};	/*Property = DeviceOrientation*/
const Value V24334_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24334_304 = {S24, 0, &V24334_304_57227, &P24334_302};	/*Property = FileReader*/
const Value V24334_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24334_305 = {S25, 0, &V24334_305_57231, &P24334_304};	/*Property = FileSaver*/
const Value V24334_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24334_306 = {S26, 0, &V24334_306_57237, &P24334_305};	/*Property = FileWriter*/
const Value V24334_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24334_307 = {S27, 0, &V24334_307_57239, &P24334_306};	/*Property = FormData*/
const Value V24334_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24334_309 = {S28, 0, &V24334_309_57246, &P24334_307};	/*Property = Fullscreen*/
const Value V24334_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24334_310 = {S29, 0, &V24334_310_57248, &P24334_309};	/*Property = GeoLocation*/
const Value V24334_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24334_312 = {S30, 0, &V24334_312_57254, &P24334_310};	/*Property = History*/
const Value V24334_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24334_314 = {S31, 0, &V24334_314_57260, &P24334_312};	/*Property = Html5*/
const Value V24334_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24334_313 = {S32, 0, &V24334_313_57257, &P24334_314};	/*Property = Html-Media-Capture*/
const Value V24334_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24334_315 = {S33, 0, &V24334_315_57264, &P24334_313};	/*Property = Iframe*/
const Value V24334_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24334_316 = {S34, 0, &V24334_316_57266, &P24334_315};	/*Property = IndexedDB*/
const Value V24334_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24334_317 = {S35, 0, &V24334_317_57269, &P24334_316};	/*Property = Json*/
const Value V24334_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24334_15 = {S37, 0, &V24334_15_28, &P24334_317};	/*Property = LayoutEngine*/
const Value V24334_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24334_344 = {S38, 0, &V24334_344_57272, &P24334_15};	/*Property = Masking*/
const Value V24334_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24334_320 = {S39, 0, &V24334_320_57290, &P24334_344};	/*Property = PostMessage*/
const Value V24334_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24334_321 = {S40, 0, &V24334_321_57293, &P24334_320};	/*Property = Progress*/
const Value V24334_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24334_322 = {S41, 0, &V24334_322_57296, &P24334_321};	/*Property = Prompts*/
const Value V24334_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24334_323 = {S42, 0, &V24334_323_57299, &P24334_322};	/*Property = Selector*/
const Value V24334_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24334_324 = {S43, 0, &V24334_324_57305, &P24334_323};	/*Property = Svg*/
const Value V24334_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24334_327 = {S44, 0, &V24334_327_57314, &P24334_324};	/*Property = TouchEvents*/
const Value V24334_328_57317 = {S2, NULL};	/*Value = True*/
const Property P24334_328 = {S45, 0, &V24334_328_57317, &P24334_327};	/*Property = Track*/
const Value V24334_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24334_329 = {S46, 0, &V24334_329_57323, &P24334_328};	/*Property = Video*/
const Value V24334_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24334_330 = {S47, 0, &V24334_330_57326, &P24334_329};	/*Property = Viewport*/
const Value V24334_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24334_332 = {S48, 0, &V24334_332_57339, &P24334_330};	/*Property = WebWorkers*/
const Value V24334_333_57341 = {S2, NULL};	/*Value = True*/
const Property P24334_333 = {S49, 0, &V24334_333_57341, &P24334_332};	/*Property = Xhr2*/
const Profile P24334 = {"24334", &P24334_333};	/*Profile ID - 24334*/
const Value V24609_1_1 = {S2, NULL};	/*Value = True*/
const Property P24609_1 = {S50, 0, &V24609_1_1, NULL};	/*Property = IsMobile*/
const Value V24609_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24609_5 = {S52, 0, &V24609_5_1277, &P24609_1};	/*Property = ScreenPixelsHeight*/
const Value V24609_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24609_4 = {S54, 0, &V24609_4_1279, &P24609_5};	/*Property = ScreenPixelsWidth*/
const Profile P24609 = {"24609", &P24609_4};	/*Profile ID - 24609*/
const Value V24611_1_1 = {S2, NULL};	/*Value = True*/
const Property P24611_1 = {S50, 0, &V24611_1_1, NULL};	/*Property = IsMobile*/
const Value V24611_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P24611_5 = {S52, 0, &V24611_5_59138, &P24611_1};	/*Property = ScreenPixelsHeight*/
const Value V24611_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P24611_4 = {S54, 0, &V24611_4_59970, &P24611_5};	/*Property = ScreenPixelsWidth*/
const Profile P24611 = {"24611", &P24611_4};	/*Profile ID - 24611*/
const Value V24391_1_1 = {S2, NULL};	/*Value = True*/
const Property P24391_1 = {S50, 0, &V24391_1_1, NULL};	/*Property = IsMobile*/
const Value V24391_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24391_5 = {S52, 0, &V24391_5_1227, &P24391_1};	/*Property = ScreenPixelsHeight*/
const Value V24391_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24391_4 = {S54, 0, &V24391_4_1229, &P24391_5};	/*Property = ScreenPixelsWidth*/
const Profile P24391 = {"24391", &P24391_4};	/*Profile ID - 24391*/
const Value V24382_1_1 = {S2, NULL};	/*Value = True*/
const Property P24382_1 = {S50, 0, &V24382_1_1, NULL};	/*Property = IsMobile*/
const Value V24382_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P24382_5 = {S52, 0, &V24382_5_59138, &P24382_1};	/*Property = ScreenPixelsHeight*/
const Value V24382_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P24382_4 = {S54, 0, &V24382_4_59970, &P24382_5};	/*Property = ScreenPixelsWidth*/
const Profile P24382 = {"24382", &P24382_4};	/*Profile ID - 24382*/
const Value V24635_1_1 = {S2, NULL};	/*Value = True*/
const Property P24635_1 = {S50, 0, &V24635_1_1, NULL};	/*Property = IsMobile*/
const Value V24635_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24635_5 = {S52, 0, &V24635_5_1403, &P24635_1};	/*Property = ScreenPixelsHeight*/
const Value V24635_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24635_4 = {S54, 0, &V24635_4_1404, &P24635_5};	/*Property = ScreenPixelsWidth*/
const Profile P24635 = {"24635", &P24635_4};	/*Profile ID - 24635*/
const Value V24339_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24339_279 = {S1, 0, &V24339_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24339_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24339_282 = {S3, 0, &V24339_282_57154, &P24339_279};	/*Property = BlobBuilder*/
const Value V24339_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24339_350 = {S4, 0, &V24339_350_57444, &P24339_282};	/*Property = Canvas*/
const Value V24339_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24339_283 = {S5, 0, &V24339_283_57159, &P24339_350};	/*Property = CssBackground*/
const Value V24339_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24339_284 = {S6, 0, &V24339_284_57162, &P24339_283};	/*Property = CssBorderImage*/
const Value V24339_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24339_285 = {S7, 0, &V24339_285_57166, &P24339_284};	/*Property = CssCanvas*/
const Value V24339_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24339_286 = {S8, 0, &V24339_286_57168, &P24339_285};	/*Property = CssColor*/
const Value V24339_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24339_237 = {S9, 0, &V24339_237_55786, &P24339_286};	/*Property = CssColumn*/
const Value V24339_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24339_287 = {S10, 0, &V24339_287_57172, &P24339_237};	/*Property = CssFlexbox*/
const Value V24339_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24339_288 = {S11, 0, &V24339_288_57174, &P24339_287};	/*Property = CssFont*/
const Value V24339_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24339_289 = {S12, 0, &V24339_289_57759, &P24339_288};	/*Property = CssImages*/
const Value V24339_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24339_291 = {S13, 0, &V24339_291_57180, &P24339_289};	/*Property = CssMediaQueries*/
const Value V24339_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24339_292 = {S14, 0, &V24339_292_57183, &P24339_291};	/*Property = CssMinMax*/
const Value V24339_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24339_293 = {S15, 0, &V24339_293_57187, &P24339_292};	/*Property = CssOverflow*/
const Value V24339_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24339_294 = {S16, 0, &V24339_294_57189, &P24339_293};	/*Property = CssPosition*/
const Value V24339_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24339_295 = {S17, 0, &V24339_295_57192, &P24339_294};	/*Property = CssText*/
const Value V24339_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24339_297 = {S18, 0, &V24339_297_57195, &P24339_295};	/*Property = CssTransforms*/
const Value V24339_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24339_241 = {S19, 0, &V24339_241_55799, &P24339_297};	/*Property = CssTransitions*/
const Value V24339_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24339_298 = {S20, 0, &V24339_298_57206, &P24339_241};	/*Property = CssUI*/
const Value V24339_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24339_300 = {S21, 0, &V24339_300_57215, &P24339_298};	/*Property = DataSet*/
const Value V24339_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24339_301 = {S22, 0, &V24339_301_57218, &P24339_300};	/*Property = DataUrl*/
const Value V24339_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24339_302 = {S23, 0, &V24339_302_57222, &P24339_301};	/*Property = DeviceOrientation*/
const Value V24339_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24339_304 = {S24, 0, &V24339_304_57227, &P24339_302};	/*Property = FileReader*/
const Value V24339_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24339_305 = {S25, 0, &V24339_305_57231, &P24339_304};	/*Property = FileSaver*/
const Value V24339_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24339_306 = {S26, 0, &V24339_306_57237, &P24339_305};	/*Property = FileWriter*/
const Value V24339_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24339_307 = {S27, 0, &V24339_307_57239, &P24339_306};	/*Property = FormData*/
const Value V24339_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24339_309 = {S28, 0, &V24339_309_57246, &P24339_307};	/*Property = Fullscreen*/
const Value V24339_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24339_310 = {S29, 0, &V24339_310_57248, &P24339_309};	/*Property = GeoLocation*/
const Value V24339_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24339_312 = {S30, 0, &V24339_312_57254, &P24339_310};	/*Property = History*/
const Value V24339_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24339_314 = {S31, 0, &V24339_314_57260, &P24339_312};	/*Property = Html5*/
const Value V24339_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24339_313 = {S32, 0, &V24339_313_57257, &P24339_314};	/*Property = Html-Media-Capture*/
const Value V24339_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24339_315 = {S33, 0, &V24339_315_57264, &P24339_313};	/*Property = Iframe*/
const Value V24339_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24339_316 = {S34, 0, &V24339_316_57266, &P24339_315};	/*Property = IndexedDB*/
const Value V24339_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24339_317 = {S35, 0, &V24339_317_57269, &P24339_316};	/*Property = Json*/
const Value V24339_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24339_15 = {S37, 0, &V24339_15_28, &P24339_317};	/*Property = LayoutEngine*/
const Value V24339_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24339_344 = {S38, 0, &V24339_344_57272, &P24339_15};	/*Property = Masking*/
const Value V24339_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24339_320 = {S39, 0, &V24339_320_57290, &P24339_344};	/*Property = PostMessage*/
const Value V24339_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24339_321 = {S40, 0, &V24339_321_57293, &P24339_320};	/*Property = Progress*/
const Value V24339_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24339_322 = {S41, 0, &V24339_322_57296, &P24339_321};	/*Property = Prompts*/
const Value V24339_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24339_323 = {S42, 0, &V24339_323_57299, &P24339_322};	/*Property = Selector*/
const Value V24339_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24339_324 = {S43, 0, &V24339_324_57305, &P24339_323};	/*Property = Svg*/
const Value V24339_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24339_327 = {S44, 0, &V24339_327_57314, &P24339_324};	/*Property = TouchEvents*/
const Value V24339_328_57317 = {S2, NULL};	/*Value = True*/
const Property P24339_328 = {S45, 0, &V24339_328_57317, &P24339_327};	/*Property = Track*/
const Value V24339_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24339_329 = {S46, 0, &V24339_329_57323, &P24339_328};	/*Property = Video*/
const Value V24339_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24339_330 = {S47, 0, &V24339_330_57326, &P24339_329};	/*Property = Viewport*/
const Value V24339_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24339_332 = {S48, 0, &V24339_332_57338, &P24339_330};	/*Property = WebWorkers*/
const Value V24339_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24339_333 = {S49, 0, &V24339_333_57342, &P24339_332};	/*Property = Xhr2*/
const Profile P24339 = {"24339", &P24339_333};	/*Profile ID - 24339*/
const Value V24529_1_1 = {S2, NULL};	/*Value = True*/
const Property P24529_1 = {S50, 0, &V24529_1_1, NULL};	/*Property = IsMobile*/
const Value V24529_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P24529_5 = {S52, 0, &V24529_5_59138, &P24529_1};	/*Property = ScreenPixelsHeight*/
const Value V24529_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P24529_4 = {S54, 0, &V24529_4_59970, &P24529_5};	/*Property = ScreenPixelsWidth*/
const Profile P24529 = {"24529", &P24529_4};	/*Profile ID - 24529*/
const Profile P24162 = {"24162", NULL};	/*Profile ID - 24162*/
const Value V24305_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24305_279 = {S1, 0, &V24305_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24305_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24305_282 = {S3, 0, &V24305_282_57154, &P24305_279};	/*Property = BlobBuilder*/
const Value V24305_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24305_350 = {S4, 0, &V24305_350_57444, &P24305_282};	/*Property = Canvas*/
const Value V24305_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24305_283 = {S5, 0, &V24305_283_57159, &P24305_350};	/*Property = CssBackground*/
const Value V24305_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24305_284 = {S6, 0, &V24305_284_57162, &P24305_283};	/*Property = CssBorderImage*/
const Value V24305_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24305_285 = {S7, 0, &V24305_285_57166, &P24305_284};	/*Property = CssCanvas*/
const Value V24305_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24305_286 = {S8, 0, &V24305_286_57168, &P24305_285};	/*Property = CssColor*/
const Value V24305_237_55784 = {S2, NULL};	/*Value = True*/
const Property P24305_237 = {S9, 0, &V24305_237_55784, &P24305_286};	/*Property = CssColumn*/
const Value V24305_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24305_287 = {S10, 0, &V24305_287_57172, &P24305_237};	/*Property = CssFlexbox*/
const Value V24305_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24305_288 = {S11, 0, &V24305_288_57174, &P24305_287};	/*Property = CssFont*/
const Value V24305_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24305_289 = {S12, 0, &V24305_289_57759, &P24305_288};	/*Property = CssImages*/
const Value V24305_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24305_291 = {S13, 0, &V24305_291_57180, &P24305_289};	/*Property = CssMediaQueries*/
const Value V24305_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24305_292 = {S14, 0, &V24305_292_57183, &P24305_291};	/*Property = CssMinMax*/
const Value V24305_293_57186 = {S2, NULL};	/*Value = True*/
const Property P24305_293 = {S15, 0, &V24305_293_57186, &P24305_292};	/*Property = CssOverflow*/
const Value V24305_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24305_294 = {S16, 0, &V24305_294_57189, &P24305_293};	/*Property = CssPosition*/
const Value V24305_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24305_295 = {S17, 0, &V24305_295_57192, &P24305_294};	/*Property = CssText*/
const Value V24305_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24305_297 = {S18, 0, &V24305_297_57195, &P24305_295};	/*Property = CssTransforms*/
const Value V24305_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24305_241 = {S19, 0, &V24305_241_55799, &P24305_297};	/*Property = CssTransitions*/
const Value V24305_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24305_298 = {S20, 0, &V24305_298_57206, &P24305_241};	/*Property = CssUI*/
const Value V24305_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24305_300 = {S21, 0, &V24305_300_57215, &P24305_298};	/*Property = DataSet*/
const Value V24305_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24305_301 = {S22, 0, &V24305_301_57218, &P24305_300};	/*Property = DataUrl*/
const Value V24305_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24305_302 = {S23, 0, &V24305_302_57221, &P24305_301};	/*Property = DeviceOrientation*/
const Value V24305_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24305_304 = {S24, 0, &V24305_304_57227, &P24305_302};	/*Property = FileReader*/
const Value V24305_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24305_305 = {S25, 0, &V24305_305_57231, &P24305_304};	/*Property = FileSaver*/
const Value V24305_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24305_306 = {S26, 0, &V24305_306_57237, &P24305_305};	/*Property = FileWriter*/
const Value V24305_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24305_307 = {S27, 0, &V24305_307_57239, &P24305_306};	/*Property = FormData*/
const Value V24305_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24305_309 = {S28, 0, &V24305_309_57246, &P24305_307};	/*Property = Fullscreen*/
const Value V24305_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24305_310 = {S29, 0, &V24305_310_57248, &P24305_309};	/*Property = GeoLocation*/
const Value V24305_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24305_312 = {S30, 0, &V24305_312_57254, &P24305_310};	/*Property = History*/
const Value V24305_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24305_314 = {S31, 0, &V24305_314_57260, &P24305_312};	/*Property = Html5*/
const Value V24305_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24305_313 = {S32, 0, &V24305_313_57258, &P24305_314};	/*Property = Html-Media-Capture*/
const Value V24305_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24305_315 = {S33, 0, &V24305_315_57264, &P24305_313};	/*Property = Iframe*/
const Value V24305_316_57267 = {S0, NULL};	/*Value = False*/
const Property P24305_316 = {S34, 0, &V24305_316_57267, &P24305_315};	/*Property = IndexedDB*/
const Value V24305_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24305_317 = {S35, 0, &V24305_317_57269, &P24305_316};	/*Property = Json*/
const Value V24305_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24305_15 = {S37, 0, &V24305_15_28, &P24305_317};	/*Property = LayoutEngine*/
const Value V24305_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24305_344 = {S38, 0, &V24305_344_57272, &P24305_15};	/*Property = Masking*/
const Value V24305_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24305_320 = {S39, 0, &V24305_320_57290, &P24305_344};	/*Property = PostMessage*/
const Value V24305_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24305_321 = {S40, 0, &V24305_321_57293, &P24305_320};	/*Property = Progress*/
const Value V24305_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24305_322 = {S41, 0, &V24305_322_57296, &P24305_321};	/*Property = Prompts*/
const Value V24305_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24305_323 = {S42, 0, &V24305_323_57299, &P24305_322};	/*Property = Selector*/
const Value V24305_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24305_324 = {S43, 0, &V24305_324_57305, &P24305_323};	/*Property = Svg*/
const Value V24305_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24305_327 = {S44, 0, &V24305_327_57314, &P24305_324};	/*Property = TouchEvents*/
const Value V24305_328_57317 = {S2, NULL};	/*Value = True*/
const Property P24305_328 = {S45, 0, &V24305_328_57317, &P24305_327};	/*Property = Track*/
const Value V24305_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24305_329 = {S46, 0, &V24305_329_57323, &P24305_328};	/*Property = Video*/
const Value V24305_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24305_330 = {S47, 0, &V24305_330_57326, &P24305_329};	/*Property = Viewport*/
const Value V24305_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24305_332 = {S48, 0, &V24305_332_57339, &P24305_330};	/*Property = WebWorkers*/
const Value V24305_333_57341 = {S2, NULL};	/*Value = True*/
const Property P24305_333 = {S49, 0, &V24305_333_57341, &P24305_332};	/*Property = Xhr2*/
const Profile P24305 = {"24305", &P24305_333};	/*Profile ID - 24305*/
const Value V24377_1_1 = {S2, NULL};	/*Value = True*/
const Property P24377_1 = {S50, 0, &V24377_1_1, NULL};	/*Property = IsMobile*/
const Value V24377_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24377_5 = {S52, 0, &V24377_5_9981, &P24377_1};	/*Property = ScreenPixelsHeight*/
const Value V24377_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24377_4 = {S54, 0, &V24377_4_39098, &P24377_5};	/*Property = ScreenPixelsWidth*/
const Profile P24377 = {"24377", &P24377_4};	/*Profile ID - 24377*/
const Value V24595_1_1 = {S2, NULL};	/*Value = True*/
const Property P24595_1 = {S50, 0, &V24595_1_1, NULL};	/*Property = IsMobile*/
const Value V24595_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24595_5 = {S52, 0, &V24595_5_1403, &P24595_1};	/*Property = ScreenPixelsHeight*/
const Value V24595_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24595_4 = {S54, 0, &V24595_4_1404, &P24595_5};	/*Property = ScreenPixelsWidth*/
const Profile P24595 = {"24595", &P24595_4};	/*Profile ID - 24595*/
const Value V24386_1_1 = {S2, NULL};	/*Value = True*/
const Property P24386_1 = {S50, 0, &V24386_1_1, NULL};	/*Property = IsMobile*/
const Value V24386_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P24386_5 = {S52, 0, &V24386_5_59138, &P24386_1};	/*Property = ScreenPixelsHeight*/
const Value V24386_4_52971 = {S122, NULL};	/*Value = 1200*/
const Property P24386_4 = {S54, 0, &V24386_4_52971, &P24386_5};	/*Property = ScreenPixelsWidth*/
const Profile P24386 = {"24386", &P24386_4};	/*Profile ID - 24386*/
const Value V24458_1_1 = {S2, NULL};	/*Value = True*/
const Property P24458_1 = {S50, 0, &V24458_1_1, NULL};	/*Property = IsMobile*/
const Value V24458_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P24458_5 = {S52, 0, &V24458_5_1458, &P24458_1};	/*Property = ScreenPixelsHeight*/
const Value V24458_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P24458_4 = {S54, 0, &V24458_4_37617, &P24458_5};	/*Property = ScreenPixelsWidth*/
const Profile P24458 = {"24458", &P24458_4};	/*Profile ID - 24458*/
const Value V24615_1_1 = {S2, NULL};	/*Value = True*/
const Property P24615_1 = {S50, 0, &V24615_1_1, NULL};	/*Property = IsMobile*/
const Value V24615_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P24615_5 = {S52, 0, &V24615_5_1222, &P24615_1};	/*Property = ScreenPixelsHeight*/
const Value V24615_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P24615_4 = {S54, 0, &V24615_4_1224, &P24615_5};	/*Property = ScreenPixelsWidth*/
const Profile P24615 = {"24615", &P24615_4};	/*Profile ID - 24615*/
const Value V24712_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24712_15 = {S37, 0, &V24712_15_28, NULL};	/*Property = LayoutEngine*/
const Profile P24712 = {"24712", &P24712_15};	/*Profile ID - 24712*/
const Value V24617_1_1 = {S2, NULL};	/*Value = True*/
const Property P24617_1 = {S50, 0, &V24617_1_1, NULL};	/*Property = IsMobile*/
const Value V24617_5_1293 = {S61, NULL};	/*Value = 160*/
const Property P24617_5 = {S52, 0, &V24617_5_1293, &P24617_1};	/*Property = ScreenPixelsHeight*/
const Value V24617_4_1296 = {S62, NULL};	/*Value = 128*/
const Property P24617_4 = {S54, 0, &V24617_4_1296, &P24617_5};	/*Property = ScreenPixelsWidth*/
const Profile P24617 = {"24617", &P24617_4};	/*Profile ID - 24617*/
const Value V24217_1_2 = {S0, NULL};	/*Value = False*/
const Property P24217_1 = {S50, 0, &V24217_1_2, NULL};	/*Property = IsMobile*/
const Value V24217_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24217_5 = {S52, 0, &V24217_5_60417, &P24217_1};	/*Property = ScreenPixelsHeight*/
const Value V24217_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24217_4 = {S54, 0, &V24217_4_57095, &P24217_5};	/*Property = ScreenPixelsWidth*/
const Profile P24217 = {"24217", &P24217_4};	/*Profile ID - 24217*/
const Value V24216_1_2 = {S0, NULL};	/*Value = False*/
const Property P24216_1 = {S50, 0, &V24216_1_2, NULL};	/*Property = IsMobile*/
const Value V24216_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24216_5 = {S52, 0, &V24216_5_60417, &P24216_1};	/*Property = ScreenPixelsHeight*/
const Value V24216_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24216_4 = {S54, 0, &V24216_4_57095, &P24216_5};	/*Property = ScreenPixelsWidth*/
const Profile P24216 = {"24216", &P24216_4};	/*Profile ID - 24216*/
const Value V24302_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_279 = {S1, 0, &V24302_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24302_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_282 = {S3, 0, &V24302_282_57155, &P24302_279};	/*Property = BlobBuilder*/
const Value V24302_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_350 = {S4, 0, &V24302_350_57446, &P24302_282};	/*Property = Canvas*/
const Value V24302_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_283 = {S5, 0, &V24302_283_57161, &P24302_350};	/*Property = CssBackground*/
const Value V24302_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_284 = {S6, 0, &V24302_284_57164, &P24302_283};	/*Property = CssBorderImage*/
const Value V24302_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_285 = {S7, 0, &V24302_285_57167, &P24302_284};	/*Property = CssCanvas*/
const Value V24302_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_286 = {S8, 0, &V24302_286_57170, &P24302_285};	/*Property = CssColor*/
const Value V24302_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_237 = {S9, 0, &V24302_237_55786, &P24302_286};	/*Property = CssColumn*/
const Value V24302_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_287 = {S10, 0, &V24302_287_57173, &P24302_237};	/*Property = CssFlexbox*/
const Value V24302_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_288 = {S11, 0, &V24302_288_57176, &P24302_287};	/*Property = CssFont*/
const Value V24302_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_289 = {S12, 0, &V24302_289_86899, &P24302_288};	/*Property = CssImages*/
const Value V24302_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_291 = {S13, 0, &V24302_291_57182, &P24302_289};	/*Property = CssMediaQueries*/
const Value V24302_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_292 = {S14, 0, &V24302_292_57185, &P24302_291};	/*Property = CssMinMax*/
const Value V24302_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_293 = {S15, 0, &V24302_293_57188, &P24302_292};	/*Property = CssOverflow*/
const Value V24302_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_294 = {S16, 0, &V24302_294_57191, &P24302_293};	/*Property = CssPosition*/
const Value V24302_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_295 = {S17, 0, &V24302_295_57194, &P24302_294};	/*Property = CssText*/
const Value V24302_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_297 = {S18, 0, &V24302_297_57197, &P24302_295};	/*Property = CssTransforms*/
const Value V24302_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_241 = {S19, 0, &V24302_241_55801, &P24302_297};	/*Property = CssTransitions*/
const Value V24302_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_298 = {S20, 0, &V24302_298_57208, &P24302_241};	/*Property = CssUI*/
const Value V24302_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_300 = {S21, 0, &V24302_300_57217, &P24302_298};	/*Property = DataSet*/
const Value V24302_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_301 = {S22, 0, &V24302_301_57220, &P24302_300};	/*Property = DataUrl*/
const Value V24302_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_302 = {S23, 0, &V24302_302_57223, &P24302_301};	/*Property = DeviceOrientation*/
const Value V24302_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_304 = {S24, 0, &V24302_304_57229, &P24302_302};	/*Property = FileReader*/
const Value V24302_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_305 = {S25, 0, &V24302_305_57232, &P24302_304};	/*Property = FileSaver*/
const Value V24302_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_306 = {S26, 0, &V24302_306_57238, &P24302_305};	/*Property = FileWriter*/
const Value V24302_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_307 = {S27, 0, &V24302_307_57241, &P24302_306};	/*Property = FormData*/
const Value V24302_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_309 = {S28, 0, &V24302_309_57247, &P24302_307};	/*Property = Fullscreen*/
const Value V24302_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_310 = {S29, 0, &V24302_310_57250, &P24302_309};	/*Property = GeoLocation*/
const Value V24302_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_312 = {S30, 0, &V24302_312_57256, &P24302_310};	/*Property = History*/
const Value V24302_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_314 = {S31, 0, &V24302_314_57262, &P24302_312};	/*Property = Html5*/
const Value V24302_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_313 = {S32, 0, &V24302_313_57259, &P24302_314};	/*Property = Html-Media-Capture*/
const Value V24302_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_315 = {S33, 0, &V24302_315_57265, &P24302_313};	/*Property = Iframe*/
const Value V24302_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_316 = {S34, 0, &V24302_316_57268, &P24302_315};	/*Property = IndexedDB*/
const Value V24302_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_317 = {S35, 0, &V24302_317_57271, &P24302_316};	/*Property = Json*/
const Value V24302_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24302_15 = {S37, 0, &V24302_15_54167, &P24302_317};	/*Property = LayoutEngine*/
const Value V24302_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_344 = {S38, 0, &V24302_344_57274, &P24302_15};	/*Property = Masking*/
const Value V24302_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_320 = {S39, 0, &V24302_320_57292, &P24302_344};	/*Property = PostMessage*/
const Value V24302_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_321 = {S40, 0, &V24302_321_57295, &P24302_320};	/*Property = Progress*/
const Value V24302_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_322 = {S41, 0, &V24302_322_57298, &P24302_321};	/*Property = Prompts*/
const Value V24302_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_323 = {S42, 0, &V24302_323_57301, &P24302_322};	/*Property = Selector*/
const Value V24302_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_324 = {S43, 0, &V24302_324_57307, &P24302_323};	/*Property = Svg*/
const Value V24302_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_327 = {S44, 0, &V24302_327_57316, &P24302_324};	/*Property = TouchEvents*/
const Value V24302_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_328 = {S45, 0, &V24302_328_57319, &P24302_327};	/*Property = Track*/
const Value V24302_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_329 = {S46, 0, &V24302_329_57325, &P24302_328};	/*Property = Video*/
const Value V24302_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_330 = {S47, 0, &V24302_330_57328, &P24302_329};	/*Property = Viewport*/
const Value V24302_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_332 = {S48, 0, &V24302_332_57340, &P24302_330};	/*Property = WebWorkers*/
const Value V24302_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24302_333 = {S49, 0, &V24302_333_57343, &P24302_332};	/*Property = Xhr2*/
const Profile P24302 = {"24302", &P24302_333};	/*Profile ID - 24302*/
const Value V24281_1_2 = {S0, NULL};	/*Value = False*/
const Property P24281_1 = {S50, 0, &V24281_1_2, NULL};	/*Property = IsMobile*/
const Value V24281_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24281_5 = {S52, 0, &V24281_5_60417, &P24281_1};	/*Property = ScreenPixelsHeight*/
const Value V24281_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24281_4 = {S54, 0, &V24281_4_57095, &P24281_5};	/*Property = ScreenPixelsWidth*/
const Profile P24281 = {"24281", &P24281_4};	/*Profile ID - 24281*/
const Value V24282_1_2 = {S0, NULL};	/*Value = False*/
const Property P24282_1 = {S50, 0, &V24282_1_2, NULL};	/*Property = IsMobile*/
const Value V24282_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P24282_5 = {S52, 0, &V24282_5_37569, &P24282_1};	/*Property = ScreenPixelsHeight*/
const Value V24282_4_53077 = {S129, NULL};	/*Value = 1366*/
const Property P24282_4 = {S54, 0, &V24282_4_53077, &P24282_5};	/*Property = ScreenPixelsWidth*/
const Profile P24282 = {"24282", &P24282_4};	/*Profile ID - 24282*/
const Value V24315_1_2 = {S0, NULL};	/*Value = False*/
const Property P24315_1 = {S50, 0, &V24315_1_2, NULL};	/*Property = IsMobile*/
const Value V24315_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P24315_5 = {S52, 0, &V24315_5_59138, &P24315_1};	/*Property = ScreenPixelsHeight*/
const Value V24315_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P24315_4 = {S54, 0, &V24315_4_59970, &P24315_5};	/*Property = ScreenPixelsWidth*/
const Profile P24315 = {"24315", &P24315_4};	/*Profile ID - 24315*/
const Value V24227_1_2 = {S0, NULL};	/*Value = False*/
const Property P24227_1 = {S50, 0, &V24227_1_2, NULL};	/*Property = IsMobile*/
const Value V24227_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24227_5 = {S52, 0, &V24227_5_60417, &P24227_1};	/*Property = ScreenPixelsHeight*/
const Value V24227_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24227_4 = {S54, 0, &V24227_4_57095, &P24227_5};	/*Property = ScreenPixelsWidth*/
const Profile P24227 = {"24227", &P24227_4};	/*Profile ID - 24227*/
const Value V24317_1_2 = {S0, NULL};	/*Value = False*/
const Property P24317_1 = {S50, 0, &V24317_1_2, NULL};	/*Property = IsMobile*/
const Value V24317_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24317_5 = {S52, 0, &V24317_5_60417, &P24317_1};	/*Property = ScreenPixelsHeight*/
const Value V24317_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24317_4 = {S54, 0, &V24317_4_57095, &P24317_5};	/*Property = ScreenPixelsWidth*/
const Profile P24317 = {"24317", &P24317_4};	/*Profile ID - 24317*/
const Value V24307_1_2 = {S0, NULL};	/*Value = False*/
const Property P24307_1 = {S50, 0, &V24307_1_2, NULL};	/*Property = IsMobile*/
const Value V24307_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24307_5 = {S52, 0, &V24307_5_60417, &P24307_1};	/*Property = ScreenPixelsHeight*/
const Value V24307_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24307_4 = {S54, 0, &V24307_4_57095, &P24307_5};	/*Property = ScreenPixelsWidth*/
const Profile P24307 = {"24307", &P24307_4};	/*Profile ID - 24307*/
const Value V24229_1_2 = {S0, NULL};	/*Value = False*/
const Property P24229_1 = {S50, 0, &V24229_1_2, NULL};	/*Property = IsMobile*/
const Value V24229_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24229_5 = {S52, 0, &V24229_5_60417, &P24229_1};	/*Property = ScreenPixelsHeight*/
const Value V24229_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24229_4 = {S54, 0, &V24229_4_57095, &P24229_5};	/*Property = ScreenPixelsWidth*/
const Profile P24229 = {"24229", &P24229_4};	/*Profile ID - 24229*/
const Value V24230_1_2 = {S0, NULL};	/*Value = False*/
const Property P24230_1 = {S50, 0, &V24230_1_2, NULL};	/*Property = IsMobile*/
const Value V24230_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24230_5 = {S52, 0, &V24230_5_60417, &P24230_1};	/*Property = ScreenPixelsHeight*/
const Value V24230_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24230_4 = {S54, 0, &V24230_4_57095, &P24230_5};	/*Property = ScreenPixelsWidth*/
const Profile P24230 = {"24230", &P24230_4};	/*Profile ID - 24230*/
const Value V24233_1_2 = {S0, NULL};	/*Value = False*/
const Property P24233_1 = {S50, 0, &V24233_1_2, NULL};	/*Property = IsMobile*/
const Value V24233_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24233_5 = {S52, 0, &V24233_5_60417, &P24233_1};	/*Property = ScreenPixelsHeight*/
const Value V24233_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24233_4 = {S54, 0, &V24233_4_57095, &P24233_5};	/*Property = ScreenPixelsWidth*/
const Profile P24233 = {"24233", &P24233_4};	/*Profile ID - 24233*/
const Value V24235_1_2 = {S0, NULL};	/*Value = False*/
const Property P24235_1 = {S50, 0, &V24235_1_2, NULL};	/*Property = IsMobile*/
const Value V24235_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24235_5 = {S52, 0, &V24235_5_60417, &P24235_1};	/*Property = ScreenPixelsHeight*/
const Value V24235_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24235_4 = {S54, 0, &V24235_4_57095, &P24235_5};	/*Property = ScreenPixelsWidth*/
const Profile P24235 = {"24235", &P24235_4};	/*Profile ID - 24235*/
const Value V24237_1_2 = {S0, NULL};	/*Value = False*/
const Property P24237_1 = {S50, 0, &V24237_1_2, NULL};	/*Property = IsMobile*/
const Value V24237_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24237_5 = {S52, 0, &V24237_5_60417, &P24237_1};	/*Property = ScreenPixelsHeight*/
const Value V24237_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24237_4 = {S54, 0, &V24237_4_57095, &P24237_5};	/*Property = ScreenPixelsWidth*/
const Profile P24237 = {"24237", &P24237_4};	/*Profile ID - 24237*/
const Value V24238_1_2 = {S0, NULL};	/*Value = False*/
const Property P24238_1 = {S50, 0, &V24238_1_2, NULL};	/*Property = IsMobile*/
const Value V24238_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24238_5 = {S52, 0, &V24238_5_60417, &P24238_1};	/*Property = ScreenPixelsHeight*/
const Value V24238_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24238_4 = {S54, 0, &V24238_4_57095, &P24238_5};	/*Property = ScreenPixelsWidth*/
const Profile P24238 = {"24238", &P24238_4};	/*Profile ID - 24238*/
const Value V24239_1_2 = {S0, NULL};	/*Value = False*/
const Property P24239_1 = {S50, 0, &V24239_1_2, NULL};	/*Property = IsMobile*/
const Value V24239_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24239_5 = {S52, 0, &V24239_5_60417, &P24239_1};	/*Property = ScreenPixelsHeight*/
const Value V24239_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24239_4 = {S54, 0, &V24239_4_57095, &P24239_5};	/*Property = ScreenPixelsWidth*/
const Profile P24239 = {"24239", &P24239_4};	/*Profile ID - 24239*/
const Value V24243_1_2 = {S0, NULL};	/*Value = False*/
const Property P24243_1 = {S50, 0, &V24243_1_2, NULL};	/*Property = IsMobile*/
const Value V24243_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24243_5 = {S52, 0, &V24243_5_60417, &P24243_1};	/*Property = ScreenPixelsHeight*/
const Value V24243_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24243_4 = {S54, 0, &V24243_4_57095, &P24243_5};	/*Property = ScreenPixelsWidth*/
const Profile P24243 = {"24243", &P24243_4};	/*Profile ID - 24243*/
const Value V24244_1_2 = {S0, NULL};	/*Value = False*/
const Property P24244_1 = {S50, 0, &V24244_1_2, NULL};	/*Property = IsMobile*/
const Value V24244_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24244_5 = {S52, 0, &V24244_5_60417, &P24244_1};	/*Property = ScreenPixelsHeight*/
const Value V24244_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24244_4 = {S54, 0, &V24244_4_57095, &P24244_5};	/*Property = ScreenPixelsWidth*/
const Profile P24244 = {"24244", &P24244_4};	/*Profile ID - 24244*/
const Value V24246_1_2 = {S0, NULL};	/*Value = False*/
const Property P24246_1 = {S50, 0, &V24246_1_2, NULL};	/*Property = IsMobile*/
const Value V24246_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24246_5 = {S52, 0, &V24246_5_60417, &P24246_1};	/*Property = ScreenPixelsHeight*/
const Value V24246_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24246_4 = {S54, 0, &V24246_4_57095, &P24246_5};	/*Property = ScreenPixelsWidth*/
const Profile P24246 = {"24246", &P24246_4};	/*Profile ID - 24246*/
const Value V24251_1_2 = {S0, NULL};	/*Value = False*/
const Property P24251_1 = {S50, 0, &V24251_1_2, NULL};	/*Property = IsMobile*/
const Value V24251_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24251_5 = {S52, 0, &V24251_5_60417, &P24251_1};	/*Property = ScreenPixelsHeight*/
const Value V24251_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24251_4 = {S54, 0, &V24251_4_57095, &P24251_5};	/*Property = ScreenPixelsWidth*/
const Profile P24251 = {"24251", &P24251_4};	/*Profile ID - 24251*/
const Value V24301_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_279 = {S1, 0, &V24301_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24301_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_282 = {S3, 0, &V24301_282_57155, &P24301_279};	/*Property = BlobBuilder*/
const Value V24301_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_350 = {S4, 0, &V24301_350_57446, &P24301_282};	/*Property = Canvas*/
const Value V24301_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_283 = {S5, 0, &V24301_283_57161, &P24301_350};	/*Property = CssBackground*/
const Value V24301_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_284 = {S6, 0, &V24301_284_57164, &P24301_283};	/*Property = CssBorderImage*/
const Value V24301_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_285 = {S7, 0, &V24301_285_57167, &P24301_284};	/*Property = CssCanvas*/
const Value V24301_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_286 = {S8, 0, &V24301_286_57170, &P24301_285};	/*Property = CssColor*/
const Value V24301_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_237 = {S9, 0, &V24301_237_55786, &P24301_286};	/*Property = CssColumn*/
const Value V24301_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_287 = {S10, 0, &V24301_287_57173, &P24301_237};	/*Property = CssFlexbox*/
const Value V24301_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_288 = {S11, 0, &V24301_288_57176, &P24301_287};	/*Property = CssFont*/
const Value V24301_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_289 = {S12, 0, &V24301_289_86899, &P24301_288};	/*Property = CssImages*/
const Value V24301_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_291 = {S13, 0, &V24301_291_57182, &P24301_289};	/*Property = CssMediaQueries*/
const Value V24301_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_292 = {S14, 0, &V24301_292_57185, &P24301_291};	/*Property = CssMinMax*/
const Value V24301_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_293 = {S15, 0, &V24301_293_57188, &P24301_292};	/*Property = CssOverflow*/
const Value V24301_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_294 = {S16, 0, &V24301_294_57191, &P24301_293};	/*Property = CssPosition*/
const Value V24301_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_295 = {S17, 0, &V24301_295_57194, &P24301_294};	/*Property = CssText*/
const Value V24301_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_297 = {S18, 0, &V24301_297_57197, &P24301_295};	/*Property = CssTransforms*/
const Value V24301_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_241 = {S19, 0, &V24301_241_55801, &P24301_297};	/*Property = CssTransitions*/
const Value V24301_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_298 = {S20, 0, &V24301_298_57208, &P24301_241};	/*Property = CssUI*/
const Value V24301_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_300 = {S21, 0, &V24301_300_57217, &P24301_298};	/*Property = DataSet*/
const Value V24301_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_301 = {S22, 0, &V24301_301_57220, &P24301_300};	/*Property = DataUrl*/
const Value V24301_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_302 = {S23, 0, &V24301_302_57223, &P24301_301};	/*Property = DeviceOrientation*/
const Value V24301_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_304 = {S24, 0, &V24301_304_57229, &P24301_302};	/*Property = FileReader*/
const Value V24301_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_305 = {S25, 0, &V24301_305_57232, &P24301_304};	/*Property = FileSaver*/
const Value V24301_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_306 = {S26, 0, &V24301_306_57238, &P24301_305};	/*Property = FileWriter*/
const Value V24301_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_307 = {S27, 0, &V24301_307_57241, &P24301_306};	/*Property = FormData*/
const Value V24301_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_309 = {S28, 0, &V24301_309_57247, &P24301_307};	/*Property = Fullscreen*/
const Value V24301_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_310 = {S29, 0, &V24301_310_57250, &P24301_309};	/*Property = GeoLocation*/
const Value V24301_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_312 = {S30, 0, &V24301_312_57256, &P24301_310};	/*Property = History*/
const Value V24301_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_314 = {S31, 0, &V24301_314_57262, &P24301_312};	/*Property = Html5*/
const Value V24301_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_313 = {S32, 0, &V24301_313_57259, &P24301_314};	/*Property = Html-Media-Capture*/
const Value V24301_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_315 = {S33, 0, &V24301_315_57265, &P24301_313};	/*Property = Iframe*/
const Value V24301_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_316 = {S34, 0, &V24301_316_57268, &P24301_315};	/*Property = IndexedDB*/
const Value V24301_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_317 = {S35, 0, &V24301_317_57271, &P24301_316};	/*Property = Json*/
const Value V24301_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24301_15 = {S37, 0, &V24301_15_54167, &P24301_317};	/*Property = LayoutEngine*/
const Value V24301_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_344 = {S38, 0, &V24301_344_57274, &P24301_15};	/*Property = Masking*/
const Value V24301_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_320 = {S39, 0, &V24301_320_57292, &P24301_344};	/*Property = PostMessage*/
const Value V24301_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_321 = {S40, 0, &V24301_321_57295, &P24301_320};	/*Property = Progress*/
const Value V24301_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_322 = {S41, 0, &V24301_322_57298, &P24301_321};	/*Property = Prompts*/
const Value V24301_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_323 = {S42, 0, &V24301_323_57301, &P24301_322};	/*Property = Selector*/
const Value V24301_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_324 = {S43, 0, &V24301_324_57307, &P24301_323};	/*Property = Svg*/
const Value V24301_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_327 = {S44, 0, &V24301_327_57316, &P24301_324};	/*Property = TouchEvents*/
const Value V24301_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_328 = {S45, 0, &V24301_328_57319, &P24301_327};	/*Property = Track*/
const Value V24301_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_329 = {S46, 0, &V24301_329_57325, &P24301_328};	/*Property = Video*/
const Value V24301_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_330 = {S47, 0, &V24301_330_57328, &P24301_329};	/*Property = Viewport*/
const Value V24301_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_332 = {S48, 0, &V24301_332_57340, &P24301_330};	/*Property = WebWorkers*/
const Value V24301_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24301_333 = {S49, 0, &V24301_333_57343, &P24301_332};	/*Property = Xhr2*/
const Profile P24301 = {"24301", &P24301_333};	/*Profile ID - 24301*/
const Value V24247_1_2 = {S0, NULL};	/*Value = False*/
const Property P24247_1 = {S50, 0, &V24247_1_2, NULL};	/*Property = IsMobile*/
const Value V24247_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24247_5 = {S52, 0, &V24247_5_60417, &P24247_1};	/*Property = ScreenPixelsHeight*/
const Value V24247_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24247_4 = {S54, 0, &V24247_4_57095, &P24247_5};	/*Property = ScreenPixelsWidth*/
const Profile P24247 = {"24247", &P24247_4};	/*Profile ID - 24247*/
const Value V24249_1_2 = {S0, NULL};	/*Value = False*/
const Property P24249_1 = {S50, 0, &V24249_1_2, NULL};	/*Property = IsMobile*/
const Value V24249_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24249_5 = {S52, 0, &V24249_5_60417, &P24249_1};	/*Property = ScreenPixelsHeight*/
const Value V24249_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24249_4 = {S54, 0, &V24249_4_57095, &P24249_5};	/*Property = ScreenPixelsWidth*/
const Profile P24249 = {"24249", &P24249_4};	/*Profile ID - 24249*/
const Value V24253_1_2 = {S0, NULL};	/*Value = False*/
const Property P24253_1 = {S50, 0, &V24253_1_2, NULL};	/*Property = IsMobile*/
const Value V24253_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24253_5 = {S52, 0, &V24253_5_60417, &P24253_1};	/*Property = ScreenPixelsHeight*/
const Value V24253_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24253_4 = {S54, 0, &V24253_4_57095, &P24253_5};	/*Property = ScreenPixelsWidth*/
const Profile P24253 = {"24253", &P24253_4};	/*Profile ID - 24253*/
const Value V24254_1_2 = {S0, NULL};	/*Value = False*/
const Property P24254_1 = {S50, 0, &V24254_1_2, NULL};	/*Property = IsMobile*/
const Value V24254_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24254_5 = {S52, 0, &V24254_5_60417, &P24254_1};	/*Property = ScreenPixelsHeight*/
const Value V24254_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24254_4 = {S54, 0, &V24254_4_57095, &P24254_5};	/*Property = ScreenPixelsWidth*/
const Profile P24254 = {"24254", &P24254_4};	/*Profile ID - 24254*/
const Value V24250_1_2 = {S0, NULL};	/*Value = False*/
const Property P24250_1 = {S50, 0, &V24250_1_2, NULL};	/*Property = IsMobile*/
const Value V24250_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24250_5 = {S52, 0, &V24250_5_60417, &P24250_1};	/*Property = ScreenPixelsHeight*/
const Value V24250_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24250_4 = {S54, 0, &V24250_4_57095, &P24250_5};	/*Property = ScreenPixelsWidth*/
const Profile P24250 = {"24250", &P24250_4};	/*Profile ID - 24250*/
const Value V24255_1_2 = {S0, NULL};	/*Value = False*/
const Property P24255_1 = {S50, 0, &V24255_1_2, NULL};	/*Property = IsMobile*/
const Value V24255_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24255_5 = {S52, 0, &V24255_5_60417, &P24255_1};	/*Property = ScreenPixelsHeight*/
const Value V24255_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24255_4 = {S54, 0, &V24255_4_57095, &P24255_5};	/*Property = ScreenPixelsWidth*/
const Profile P24255 = {"24255", &P24255_4};	/*Profile ID - 24255*/
const Value V24257_1_2 = {S0, NULL};	/*Value = False*/
const Property P24257_1 = {S50, 0, &V24257_1_2, NULL};	/*Property = IsMobile*/
const Value V24257_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24257_5 = {S52, 0, &V24257_5_60417, &P24257_1};	/*Property = ScreenPixelsHeight*/
const Value V24257_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24257_4 = {S54, 0, &V24257_4_57095, &P24257_5};	/*Property = ScreenPixelsWidth*/
const Profile P24257 = {"24257", &P24257_4};	/*Profile ID - 24257*/
const Value V24241_1_2 = {S0, NULL};	/*Value = False*/
const Property P24241_1 = {S50, 0, &V24241_1_2, NULL};	/*Property = IsMobile*/
const Value V24241_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24241_5 = {S52, 0, &V24241_5_60417, &P24241_1};	/*Property = ScreenPixelsHeight*/
const Value V24241_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24241_4 = {S54, 0, &V24241_4_57095, &P24241_5};	/*Property = ScreenPixelsWidth*/
const Profile P24241 = {"24241", &P24241_4};	/*Profile ID - 24241*/
const Value V24285_1_2 = {S0, NULL};	/*Value = False*/
const Property P24285_1 = {S50, 0, &V24285_1_2, NULL};	/*Property = IsMobile*/
const Value V24285_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24285_5 = {S52, 0, &V24285_5_60417, &P24285_1};	/*Property = ScreenPixelsHeight*/
const Value V24285_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24285_4 = {S54, 0, &V24285_4_57095, &P24285_5};	/*Property = ScreenPixelsWidth*/
const Profile P24285 = {"24285", &P24285_4};	/*Profile ID - 24285*/
const Value V24312_1_2 = {S0, NULL};	/*Value = False*/
const Property P24312_1 = {S50, 0, &V24312_1_2, NULL};	/*Property = IsMobile*/
const Value V24312_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24312_5 = {S52, 0, &V24312_5_60417, &P24312_1};	/*Property = ScreenPixelsHeight*/
const Value V24312_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24312_4 = {S54, 0, &V24312_4_57095, &P24312_5};	/*Property = ScreenPixelsWidth*/
const Profile P24312 = {"24312", &P24312_4};	/*Profile ID - 24312*/
const Value V24288_1_2 = {S0, NULL};	/*Value = False*/
const Property P24288_1 = {S50, 0, &V24288_1_2, NULL};	/*Property = IsMobile*/
const Value V24288_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24288_5 = {S52, 0, &V24288_5_60417, &P24288_1};	/*Property = ScreenPixelsHeight*/
const Value V24288_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24288_4 = {S54, 0, &V24288_4_57095, &P24288_5};	/*Property = ScreenPixelsWidth*/
const Profile P24288 = {"24288", &P24288_4};	/*Profile ID - 24288*/
const Value V24289_1_2 = {S0, NULL};	/*Value = False*/
const Property P24289_1 = {S50, 0, &V24289_1_2, NULL};	/*Property = IsMobile*/
const Value V24289_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24289_5 = {S52, 0, &V24289_5_60417, &P24289_1};	/*Property = ScreenPixelsHeight*/
const Value V24289_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24289_4 = {S54, 0, &V24289_4_57095, &P24289_5};	/*Property = ScreenPixelsWidth*/
const Profile P24289 = {"24289", &P24289_4};	/*Profile ID - 24289*/
const Value V24292_1_2 = {S0, NULL};	/*Value = False*/
const Property P24292_1 = {S50, 0, &V24292_1_2, NULL};	/*Property = IsMobile*/
const Value V24292_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24292_5 = {S52, 0, &V24292_5_60417, &P24292_1};	/*Property = ScreenPixelsHeight*/
const Value V24292_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24292_4 = {S54, 0, &V24292_4_57095, &P24292_5};	/*Property = ScreenPixelsWidth*/
const Profile P24292 = {"24292", &P24292_4};	/*Profile ID - 24292*/
const Value V24294_1_2 = {S0, NULL};	/*Value = False*/
const Property P24294_1 = {S50, 0, &V24294_1_2, NULL};	/*Property = IsMobile*/
const Value V24294_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24294_5 = {S52, 0, &V24294_5_60417, &P24294_1};	/*Property = ScreenPixelsHeight*/
const Value V24294_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24294_4 = {S54, 0, &V24294_4_57095, &P24294_5};	/*Property = ScreenPixelsWidth*/
const Profile P24294 = {"24294", &P24294_4};	/*Profile ID - 24294*/
const Value V24306_1_2 = {S0, NULL};	/*Value = False*/
const Property P24306_1 = {S50, 0, &V24306_1_2, NULL};	/*Property = IsMobile*/
const Value V24306_5_37569 = {S102, NULL};	/*Value = 768*/
const Property P24306_5 = {S52, 0, &V24306_5_37569, &P24306_1};	/*Property = ScreenPixelsHeight*/
const Value V24306_4_5645 = {S85, NULL};	/*Value = 1024*/
const Property P24306_4 = {S54, 0, &V24306_4_5645, &P24306_5};	/*Property = ScreenPixelsWidth*/
const Profile P24306 = {"24306", &P24306_4};	/*Profile ID - 24306*/
const Value V24284_1_2 = {S0, NULL};	/*Value = False*/
const Property P24284_1 = {S50, 0, &V24284_1_2, NULL};	/*Property = IsMobile*/
const Value V24284_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24284_5 = {S52, 0, &V24284_5_60417, &P24284_1};	/*Property = ScreenPixelsHeight*/
const Value V24284_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24284_4 = {S54, 0, &V24284_4_57095, &P24284_5};	/*Property = ScreenPixelsWidth*/
const Profile P24284 = {"24284", &P24284_4};	/*Profile ID - 24284*/
const Value V24310_1_2 = {S0, NULL};	/*Value = False*/
const Property P24310_1 = {S50, 0, &V24310_1_2, NULL};	/*Property = IsMobile*/
const Value V24310_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24310_5 = {S52, 0, &V24310_5_60417, &P24310_1};	/*Property = ScreenPixelsHeight*/
const Value V24310_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24310_4 = {S54, 0, &V24310_4_57095, &P24310_5};	/*Property = ScreenPixelsWidth*/
const Profile P24310 = {"24310", &P24310_4};	/*Profile ID - 24310*/
const Value V24311_1_2 = {S0, NULL};	/*Value = False*/
const Property P24311_1 = {S50, 0, &V24311_1_2, NULL};	/*Property = IsMobile*/
const Value V24311_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24311_5 = {S52, 0, &V24311_5_60417, &P24311_1};	/*Property = ScreenPixelsHeight*/
const Value V24311_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24311_4 = {S54, 0, &V24311_4_57095, &P24311_5};	/*Property = ScreenPixelsWidth*/
const Profile P24311 = {"24311", &P24311_4};	/*Profile ID - 24311*/
const Value V24318_1_2 = {S0, NULL};	/*Value = False*/
const Property P24318_1 = {S50, 0, &V24318_1_2, NULL};	/*Property = IsMobile*/
const Value V24318_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24318_5 = {S52, 0, &V24318_5_60417, &P24318_1};	/*Property = ScreenPixelsHeight*/
const Value V24318_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24318_4 = {S54, 0, &V24318_4_57095, &P24318_5};	/*Property = ScreenPixelsWidth*/
const Profile P24318 = {"24318", &P24318_4};	/*Profile ID - 24318*/
const Value V24320_1_2 = {S0, NULL};	/*Value = False*/
const Property P24320_1 = {S50, 0, &V24320_1_2, NULL};	/*Property = IsMobile*/
const Value V24320_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24320_5 = {S52, 0, &V24320_5_60417, &P24320_1};	/*Property = ScreenPixelsHeight*/
const Value V24320_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24320_4 = {S54, 0, &V24320_4_57095, &P24320_5};	/*Property = ScreenPixelsWidth*/
const Profile P24320 = {"24320", &P24320_4};	/*Profile ID - 24320*/
const Value V24322_1_1 = {S2, NULL};	/*Value = True*/
const Property P24322_1 = {S50, 0, &V24322_1_1, NULL};	/*Property = IsMobile*/
const Value V24322_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24322_5 = {S52, 0, &V24322_5_60417, &P24322_1};	/*Property = ScreenPixelsHeight*/
const Value V24322_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24322_4 = {S54, 0, &V24322_4_57095, &P24322_5};	/*Property = ScreenPixelsWidth*/
const Profile P24322 = {"24322", &P24322_4};	/*Profile ID - 24322*/
const Value V24324_1_2 = {S0, NULL};	/*Value = False*/
const Property P24324_1 = {S50, 0, &V24324_1_2, NULL};	/*Property = IsMobile*/
const Value V24324_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24324_5 = {S52, 0, &V24324_5_60417, &P24324_1};	/*Property = ScreenPixelsHeight*/
const Value V24324_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24324_4 = {S54, 0, &V24324_4_57095, &P24324_5};	/*Property = ScreenPixelsWidth*/
const Profile P24324 = {"24324", &P24324_4};	/*Profile ID - 24324*/
const Value V24325_1_2 = {S0, NULL};	/*Value = False*/
const Property P24325_1 = {S50, 0, &V24325_1_2, NULL};	/*Property = IsMobile*/
const Value V24325_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24325_5 = {S52, 0, &V24325_5_60417, &P24325_1};	/*Property = ScreenPixelsHeight*/
const Value V24325_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24325_4 = {S54, 0, &V24325_4_57095, &P24325_5};	/*Property = ScreenPixelsWidth*/
const Profile P24325 = {"24325", &P24325_4};	/*Profile ID - 24325*/
const Value V24328_1_2 = {S0, NULL};	/*Value = False*/
const Property P24328_1 = {S50, 0, &V24328_1_2, NULL};	/*Property = IsMobile*/
const Value V24328_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24328_5 = {S52, 0, &V24328_5_60417, &P24328_1};	/*Property = ScreenPixelsHeight*/
const Value V24328_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24328_4 = {S54, 0, &V24328_4_57095, &P24328_5};	/*Property = ScreenPixelsWidth*/
const Profile P24328 = {"24328", &P24328_4};	/*Profile ID - 24328*/
const Value V24295_1_2 = {S0, NULL};	/*Value = False*/
const Property P24295_1 = {S50, 0, &V24295_1_2, NULL};	/*Property = IsMobile*/
const Value V24295_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24295_5 = {S52, 0, &V24295_5_60417, &P24295_1};	/*Property = ScreenPixelsHeight*/
const Value V24295_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24295_4 = {S54, 0, &V24295_4_57095, &P24295_5};	/*Property = ScreenPixelsWidth*/
const Profile P24295 = {"24295", &P24295_4};	/*Profile ID - 24295*/
const Value V24356_1_2 = {S0, NULL};	/*Value = False*/
const Property P24356_1 = {S50, 0, &V24356_1_2, NULL};	/*Property = IsMobile*/
const Value V24356_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24356_5 = {S52, 0, &V24356_5_60417, &P24356_1};	/*Property = ScreenPixelsHeight*/
const Value V24356_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24356_4 = {S54, 0, &V24356_4_57095, &P24356_5};	/*Property = ScreenPixelsWidth*/
const Profile P24356 = {"24356", &P24356_4};	/*Profile ID - 24356*/
const Value V24360_1_2 = {S0, NULL};	/*Value = False*/
const Property P24360_1 = {S50, 0, &V24360_1_2, NULL};	/*Property = IsMobile*/
const Value V24360_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24360_5 = {S52, 0, &V24360_5_60417, &P24360_1};	/*Property = ScreenPixelsHeight*/
const Value V24360_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24360_4 = {S54, 0, &V24360_4_57095, &P24360_5};	/*Property = ScreenPixelsWidth*/
const Profile P24360 = {"24360", &P24360_4};	/*Profile ID - 24360*/
const Value V24326_1_2 = {S0, NULL};	/*Value = False*/
const Property P24326_1 = {S50, 0, &V24326_1_2, NULL};	/*Property = IsMobile*/
const Value V24326_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24326_5 = {S52, 0, &V24326_5_60417, &P24326_1};	/*Property = ScreenPixelsHeight*/
const Value V24326_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24326_4 = {S54, 0, &V24326_4_57095, &P24326_5};	/*Property = ScreenPixelsWidth*/
const Profile P24326 = {"24326", &P24326_4};	/*Profile ID - 24326*/
const Value V24358_1_2 = {S0, NULL};	/*Value = False*/
const Property P24358_1 = {S50, 0, &V24358_1_2, NULL};	/*Property = IsMobile*/
const Value V24358_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24358_5 = {S52, 0, &V24358_5_60417, &P24358_1};	/*Property = ScreenPixelsHeight*/
const Value V24358_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24358_4 = {S54, 0, &V24358_4_57095, &P24358_5};	/*Property = ScreenPixelsWidth*/
const Profile P24358 = {"24358", &P24358_4};	/*Profile ID - 24358*/
const Value V24319_1_2 = {S0, NULL};	/*Value = False*/
const Property P24319_1 = {S50, 0, &V24319_1_2, NULL};	/*Property = IsMobile*/
const Value V24319_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24319_5 = {S52, 0, &V24319_5_60417, &P24319_1};	/*Property = ScreenPixelsHeight*/
const Value V24319_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24319_4 = {S54, 0, &V24319_4_57095, &P24319_5};	/*Property = ScreenPixelsWidth*/
const Profile P24319 = {"24319", &P24319_4};	/*Profile ID - 24319*/
const Value V24362_1_2 = {S0, NULL};	/*Value = False*/
const Property P24362_1 = {S50, 0, &V24362_1_2, NULL};	/*Property = IsMobile*/
const Value V24362_5_90074 = {S152, NULL};	/*Value = 2160*/
const Property P24362_5 = {S52, 0, &V24362_5_90074, &P24362_1};	/*Property = ScreenPixelsHeight*/
const Value V24362_4_90075 = {S153, NULL};	/*Value = 3840*/
const Property P24362_4 = {S54, 0, &V24362_4_90075, &P24362_5};	/*Property = ScreenPixelsWidth*/
const Profile P24362 = {"24362", &P24362_4};	/*Profile ID - 24362*/
const Value V24359_1_2 = {S0, NULL};	/*Value = False*/
const Property P24359_1 = {S50, 0, &V24359_1_2, NULL};	/*Property = IsMobile*/
const Value V24359_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24359_5 = {S52, 0, &V24359_5_60417, &P24359_1};	/*Property = ScreenPixelsHeight*/
const Value V24359_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24359_4 = {S54, 0, &V24359_4_57095, &P24359_5};	/*Property = ScreenPixelsWidth*/
const Profile P24359 = {"24359", &P24359_4};	/*Profile ID - 24359*/
const Value V24357_1_2 = {S0, NULL};	/*Value = False*/
const Property P24357_1 = {S50, 0, &V24357_1_2, NULL};	/*Property = IsMobile*/
const Value V24357_5_37 = {S55, NULL};	/*Value = Unknown*/
const Property P24357_5 = {S52, 0, &V24357_5_37, &P24357_1};	/*Property = ScreenPixelsHeight*/
const Value V24357_4_36 = {S55, NULL};	/*Value = Unknown*/
const Property P24357_4 = {S54, 0, &V24357_4_36, &P24357_5};	/*Property = ScreenPixelsWidth*/
const Profile P24357 = {"24357", &P24357_4};	/*Profile ID - 24357*/
const Value V24291_1_2 = {S0, NULL};	/*Value = False*/
const Property P24291_1 = {S50, 0, &V24291_1_2, NULL};	/*Property = IsMobile*/
const Value V24291_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24291_5 = {S52, 0, &V24291_5_60417, &P24291_1};	/*Property = ScreenPixelsHeight*/
const Value V24291_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24291_4 = {S54, 0, &V24291_4_57095, &P24291_5};	/*Property = ScreenPixelsWidth*/
const Profile P24291 = {"24291", &P24291_4};	/*Profile ID - 24291*/
const Value V24287_1_2 = {S0, NULL};	/*Value = False*/
const Property P24287_1 = {S50, 0, &V24287_1_2, NULL};	/*Property = IsMobile*/
const Value V24287_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24287_5 = {S52, 0, &V24287_5_60417, &P24287_1};	/*Property = ScreenPixelsHeight*/
const Value V24287_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24287_4 = {S54, 0, &V24287_4_57095, &P24287_5};	/*Property = ScreenPixelsWidth*/
const Profile P24287 = {"24287", &P24287_4};	/*Profile ID - 24287*/
const Value V24364_1_2 = {S0, NULL};	/*Value = False*/
const Property P24364_1 = {S50, 0, &V24364_1_2, NULL};	/*Property = IsMobile*/
const Value V24364_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24364_5 = {S52, 0, &V24364_5_60417, &P24364_1};	/*Property = ScreenPixelsHeight*/
const Value V24364_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24364_4 = {S54, 0, &V24364_4_57095, &P24364_5};	/*Property = ScreenPixelsWidth*/
const Profile P24364 = {"24364", &P24364_4};	/*Profile ID - 24364*/
const Value V24333_1_2 = {S0, NULL};	/*Value = False*/
const Property P24333_1 = {S50, 0, &V24333_1_2, NULL};	/*Property = IsMobile*/
const Value V24333_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24333_5 = {S52, 0, &V24333_5_60417, &P24333_1};	/*Property = ScreenPixelsHeight*/
const Value V24333_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24333_4 = {S54, 0, &V24333_4_57095, &P24333_5};	/*Property = ScreenPixelsWidth*/
const Profile P24333 = {"24333", &P24333_4};	/*Profile ID - 24333*/
const Value V24332_1_2 = {S0, NULL};	/*Value = False*/
const Property P24332_1 = {S50, 0, &V24332_1_2, NULL};	/*Property = IsMobile*/
const Value V24332_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24332_5 = {S52, 0, &V24332_5_60417, &P24332_1};	/*Property = ScreenPixelsHeight*/
const Value V24332_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24332_4 = {S54, 0, &V24332_4_57095, &P24332_5};	/*Property = ScreenPixelsWidth*/
const Profile P24332 = {"24332", &P24332_4};	/*Profile ID - 24332*/
const Value V24280_1_2 = {S0, NULL};	/*Value = False*/
const Property P24280_1 = {S50, 0, &V24280_1_2, NULL};	/*Property = IsMobile*/
const Value V24280_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24280_5 = {S52, 0, &V24280_5_60417, &P24280_1};	/*Property = ScreenPixelsHeight*/
const Value V24280_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24280_4 = {S54, 0, &V24280_4_57095, &P24280_5};	/*Property = ScreenPixelsWidth*/
const Profile P24280 = {"24280", &P24280_4};	/*Profile ID - 24280*/
const Value V24283_1_2 = {S0, NULL};	/*Value = False*/
const Property P24283_1 = {S50, 0, &V24283_1_2, NULL};	/*Property = IsMobile*/
const Value V24283_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24283_5 = {S52, 0, &V24283_5_60417, &P24283_1};	/*Property = ScreenPixelsHeight*/
const Value V24283_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24283_4 = {S54, 0, &V24283_4_57095, &P24283_5};	/*Property = ScreenPixelsWidth*/
const Profile P24283 = {"24283", &P24283_4};	/*Profile ID - 24283*/
const Value V24258_1_2 = {S0, NULL};	/*Value = False*/
const Property P24258_1 = {S50, 0, &V24258_1_2, NULL};	/*Property = IsMobile*/
const Value V24258_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24258_5 = {S52, 0, &V24258_5_60417, &P24258_1};	/*Property = ScreenPixelsHeight*/
const Value V24258_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24258_4 = {S54, 0, &V24258_4_57095, &P24258_5};	/*Property = ScreenPixelsWidth*/
const Profile P24258 = {"24258", &P24258_4};	/*Profile ID - 24258*/
const Value V24286_1_2 = {S0, NULL};	/*Value = False*/
const Property P24286_1 = {S50, 0, &V24286_1_2, NULL};	/*Property = IsMobile*/
const Value V24286_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24286_5 = {S52, 0, &V24286_5_60417, &P24286_1};	/*Property = ScreenPixelsHeight*/
const Value V24286_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24286_4 = {S54, 0, &V24286_4_57095, &P24286_5};	/*Property = ScreenPixelsWidth*/
const Profile P24286 = {"24286", &P24286_4};	/*Profile ID - 24286*/
const Value V24290_1_2 = {S0, NULL};	/*Value = False*/
const Property P24290_1 = {S50, 0, &V24290_1_2, NULL};	/*Property = IsMobile*/
const Value V24290_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24290_5 = {S52, 0, &V24290_5_60417, &P24290_1};	/*Property = ScreenPixelsHeight*/
const Value V24290_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24290_4 = {S54, 0, &V24290_4_57095, &P24290_5};	/*Property = ScreenPixelsWidth*/
const Profile P24290 = {"24290", &P24290_4};	/*Profile ID - 24290*/
const Value V24296_1_2 = {S0, NULL};	/*Value = False*/
const Property P24296_1 = {S50, 0, &V24296_1_2, NULL};	/*Property = IsMobile*/
const Value V24296_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24296_5 = {S52, 0, &V24296_5_60417, &P24296_1};	/*Property = ScreenPixelsHeight*/
const Value V24296_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24296_4 = {S54, 0, &V24296_4_57095, &P24296_5};	/*Property = ScreenPixelsWidth*/
const Profile P24296 = {"24296", &P24296_4};	/*Profile ID - 24296*/
const Value V24293_1_2 = {S0, NULL};	/*Value = False*/
const Property P24293_1 = {S50, 0, &V24293_1_2, NULL};	/*Property = IsMobile*/
const Value V24293_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24293_5 = {S52, 0, &V24293_5_60417, &P24293_1};	/*Property = ScreenPixelsHeight*/
const Value V24293_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24293_4 = {S54, 0, &V24293_4_57095, &P24293_5};	/*Property = ScreenPixelsWidth*/
const Profile P24293 = {"24293", &P24293_4};	/*Profile ID - 24293*/
const Value V24298_1_2 = {S0, NULL};	/*Value = False*/
const Property P24298_1 = {S50, 0, &V24298_1_2, NULL};	/*Property = IsMobile*/
const Value V24298_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24298_5 = {S52, 0, &V24298_5_60417, &P24298_1};	/*Property = ScreenPixelsHeight*/
const Value V24298_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24298_4 = {S54, 0, &V24298_4_57095, &P24298_5};	/*Property = ScreenPixelsWidth*/
const Profile P24298 = {"24298", &P24298_4};	/*Profile ID - 24298*/
const Value V24256_1_2 = {S0, NULL};	/*Value = False*/
const Property P24256_1 = {S50, 0, &V24256_1_2, NULL};	/*Property = IsMobile*/
const Value V24256_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24256_5 = {S52, 0, &V24256_5_60417, &P24256_1};	/*Property = ScreenPixelsHeight*/
const Value V24256_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24256_4 = {S54, 0, &V24256_4_57095, &P24256_5};	/*Property = ScreenPixelsWidth*/
const Profile P24256 = {"24256", &P24256_4};	/*Profile ID - 24256*/
const Value V24303_1_2 = {S0, NULL};	/*Value = False*/
const Property P24303_1 = {S50, 0, &V24303_1_2, NULL};	/*Property = IsMobile*/
const Value V24303_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24303_5 = {S52, 0, &V24303_5_60417, &P24303_1};	/*Property = ScreenPixelsHeight*/
const Value V24303_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24303_4 = {S54, 0, &V24303_4_57095, &P24303_5};	/*Property = ScreenPixelsWidth*/
const Profile P24303 = {"24303", &P24303_4};	/*Profile ID - 24303*/
const Value V24226_1_2 = {S0, NULL};	/*Value = False*/
const Property P24226_1 = {S50, 0, &V24226_1_2, NULL};	/*Property = IsMobile*/
const Value V24226_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24226_5 = {S52, 0, &V24226_5_60417, &P24226_1};	/*Property = ScreenPixelsHeight*/
const Value V24226_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24226_4 = {S54, 0, &V24226_4_57095, &P24226_5};	/*Property = ScreenPixelsWidth*/
const Profile P24226 = {"24226", &P24226_4};	/*Profile ID - 24226*/
const Value V24245_1_2 = {S0, NULL};	/*Value = False*/
const Property P24245_1 = {S50, 0, &V24245_1_2, NULL};	/*Property = IsMobile*/
const Value V24245_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24245_5 = {S52, 0, &V24245_5_60417, &P24245_1};	/*Property = ScreenPixelsHeight*/
const Value V24245_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24245_4 = {S54, 0, &V24245_4_57095, &P24245_5};	/*Property = ScreenPixelsWidth*/
const Profile P24245 = {"24245", &P24245_4};	/*Profile ID - 24245*/
const Value V24234_1_2 = {S0, NULL};	/*Value = False*/
const Property P24234_1 = {S50, 0, &V24234_1_2, NULL};	/*Property = IsMobile*/
const Value V24234_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24234_5 = {S52, 0, &V24234_5_60417, &P24234_1};	/*Property = ScreenPixelsHeight*/
const Value V24234_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24234_4 = {S54, 0, &V24234_4_57095, &P24234_5};	/*Property = ScreenPixelsWidth*/
const Profile P24234 = {"24234", &P24234_4};	/*Profile ID - 24234*/
const Value V24248_1_2 = {S0, NULL};	/*Value = False*/
const Property P24248_1 = {S50, 0, &V24248_1_2, NULL};	/*Property = IsMobile*/
const Value V24248_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24248_5 = {S52, 0, &V24248_5_60417, &P24248_1};	/*Property = ScreenPixelsHeight*/
const Value V24248_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24248_4 = {S54, 0, &V24248_4_57095, &P24248_5};	/*Property = ScreenPixelsWidth*/
const Profile P24248 = {"24248", &P24248_4};	/*Profile ID - 24248*/
const Value V24236_1_2 = {S0, NULL};	/*Value = False*/
const Property P24236_1 = {S50, 0, &V24236_1_2, NULL};	/*Property = IsMobile*/
const Value V24236_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24236_5 = {S52, 0, &V24236_5_60417, &P24236_1};	/*Property = ScreenPixelsHeight*/
const Value V24236_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24236_4 = {S54, 0, &V24236_4_57095, &P24236_5};	/*Property = ScreenPixelsWidth*/
const Profile P24236 = {"24236", &P24236_4};	/*Profile ID - 24236*/
const Value V24252_1_2 = {S0, NULL};	/*Value = False*/
const Property P24252_1 = {S50, 0, &V24252_1_2, NULL};	/*Property = IsMobile*/
const Value V24252_5_60417 = {S128, NULL};	/*Value = 1080*/
const Property P24252_5 = {S52, 0, &V24252_5_60417, &P24252_1};	/*Property = ScreenPixelsHeight*/
const Value V24252_4_57095 = {S127, NULL};	/*Value = 1920*/
const Property P24252_4 = {S54, 0, &V24252_4_57095, &P24252_5};	/*Property = ScreenPixelsWidth*/
const Profile P24252 = {"24252", &P24252_4};	/*Profile ID - 24252*/
const Value V24345_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24345_279 = {S1, 0, &V24345_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24345_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24345_282 = {S3, 0, &V24345_282_57154, &P24345_279};	/*Property = BlobBuilder*/
const Value V24345_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24345_350 = {S4, 0, &V24345_350_57444, &P24345_282};	/*Property = Canvas*/
const Value V24345_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24345_283 = {S5, 0, &V24345_283_57159, &P24345_350};	/*Property = CssBackground*/
const Value V24345_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24345_284 = {S6, 0, &V24345_284_57162, &P24345_283};	/*Property = CssBorderImage*/
const Value V24345_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24345_285 = {S7, 0, &V24345_285_57166, &P24345_284};	/*Property = CssCanvas*/
const Value V24345_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24345_286 = {S8, 0, &V24345_286_57168, &P24345_285};	/*Property = CssColor*/
const Value V24345_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24345_237 = {S9, 0, &V24345_237_55786, &P24345_286};	/*Property = CssColumn*/
const Value V24345_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24345_287 = {S10, 0, &V24345_287_57172, &P24345_237};	/*Property = CssFlexbox*/
const Value V24345_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24345_288 = {S11, 0, &V24345_288_57174, &P24345_287};	/*Property = CssFont*/
const Value V24345_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24345_289 = {S12, 0, &V24345_289_57759, &P24345_288};	/*Property = CssImages*/
const Value V24345_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24345_291 = {S13, 0, &V24345_291_57180, &P24345_289};	/*Property = CssMediaQueries*/
const Value V24345_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24345_292 = {S14, 0, &V24345_292_57183, &P24345_291};	/*Property = CssMinMax*/
const Value V24345_293_57186 = {S2, NULL};	/*Value = True*/
const Property P24345_293 = {S15, 0, &V24345_293_57186, &P24345_292};	/*Property = CssOverflow*/
const Value V24345_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24345_294 = {S16, 0, &V24345_294_57189, &P24345_293};	/*Property = CssPosition*/
const Value V24345_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24345_295 = {S17, 0, &V24345_295_57192, &P24345_294};	/*Property = CssText*/
const Value V24345_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24345_297 = {S18, 0, &V24345_297_57195, &P24345_295};	/*Property = CssTransforms*/
const Value V24345_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24345_241 = {S19, 0, &V24345_241_55799, &P24345_297};	/*Property = CssTransitions*/
const Value V24345_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24345_298 = {S20, 0, &V24345_298_57206, &P24345_241};	/*Property = CssUI*/
const Value V24345_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24345_300 = {S21, 0, &V24345_300_57215, &P24345_298};	/*Property = DataSet*/
const Value V24345_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24345_301 = {S22, 0, &V24345_301_57218, &P24345_300};	/*Property = DataUrl*/
const Value V24345_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24345_302 = {S23, 0, &V24345_302_57221, &P24345_301};	/*Property = DeviceOrientation*/
const Value V24345_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24345_304 = {S24, 0, &V24345_304_57227, &P24345_302};	/*Property = FileReader*/
const Value V24345_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24345_305 = {S25, 0, &V24345_305_57231, &P24345_304};	/*Property = FileSaver*/
const Value V24345_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24345_306 = {S26, 0, &V24345_306_57237, &P24345_305};	/*Property = FileWriter*/
const Value V24345_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24345_307 = {S27, 0, &V24345_307_57239, &P24345_306};	/*Property = FormData*/
const Value V24345_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24345_309 = {S28, 0, &V24345_309_57246, &P24345_307};	/*Property = Fullscreen*/
const Value V24345_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24345_310 = {S29, 0, &V24345_310_57248, &P24345_309};	/*Property = GeoLocation*/
const Value V24345_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24345_312 = {S30, 0, &V24345_312_57254, &P24345_310};	/*Property = History*/
const Value V24345_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24345_314 = {S31, 0, &V24345_314_57260, &P24345_312};	/*Property = Html5*/
const Value V24345_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24345_313 = {S32, 0, &V24345_313_57258, &P24345_314};	/*Property = Html-Media-Capture*/
const Value V24345_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24345_315 = {S33, 0, &V24345_315_57264, &P24345_313};	/*Property = Iframe*/
const Value V24345_316_57267 = {S0, NULL};	/*Value = False*/
const Property P24345_316 = {S34, 0, &V24345_316_57267, &P24345_315};	/*Property = IndexedDB*/
const Value V24345_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24345_317 = {S35, 0, &V24345_317_57269, &P24345_316};	/*Property = Json*/
const Value V24345_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24345_15 = {S37, 0, &V24345_15_28, &P24345_317};	/*Property = LayoutEngine*/
const Value V24345_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24345_344 = {S38, 0, &V24345_344_57272, &P24345_15};	/*Property = Masking*/
const Value V24345_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24345_320 = {S39, 0, &V24345_320_57290, &P24345_344};	/*Property = PostMessage*/
const Value V24345_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24345_321 = {S40, 0, &V24345_321_57293, &P24345_320};	/*Property = Progress*/
const Value V24345_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24345_322 = {S41, 0, &V24345_322_57296, &P24345_321};	/*Property = Prompts*/
const Value V24345_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24345_323 = {S42, 0, &V24345_323_57299, &P24345_322};	/*Property = Selector*/
const Value V24345_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24345_324 = {S43, 0, &V24345_324_57305, &P24345_323};	/*Property = Svg*/
const Value V24345_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24345_327 = {S44, 0, &V24345_327_57314, &P24345_324};	/*Property = TouchEvents*/
const Value V24345_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24345_328 = {S45, 0, &V24345_328_57318, &P24345_327};	/*Property = Track*/
const Value V24345_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24345_329 = {S46, 0, &V24345_329_57323, &P24345_328};	/*Property = Video*/
const Value V24345_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24345_330 = {S47, 0, &V24345_330_57326, &P24345_329};	/*Property = Viewport*/
const Value V24345_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24345_332 = {S48, 0, &V24345_332_57338, &P24345_330};	/*Property = WebWorkers*/
const Value V24345_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24345_333 = {S49, 0, &V24345_333_57342, &P24345_332};	/*Property = Xhr2*/
const Profile P24345 = {"24345", &P24345_333};	/*Profile ID - 24345*/
const Profile P24347 = {"24347", NULL};	/*Profile ID - 24347*/
const Profile P24348 = {"24348", NULL};	/*Profile ID - 24348*/
const Profile P24349 = {"24349", NULL};	/*Profile ID - 24349*/
const Value V24153_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24153_279 = {S1, 0, &V24153_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24153_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24153_282 = {S3, 0, &V24153_282_57154, &P24153_279};	/*Property = BlobBuilder*/
const Value V24153_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24153_350 = {S4, 0, &V24153_350_57444, &P24153_282};	/*Property = Canvas*/
const Value V24153_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24153_283 = {S5, 0, &V24153_283_57159, &P24153_350};	/*Property = CssBackground*/
const Value V24153_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24153_284 = {S6, 0, &V24153_284_57162, &P24153_283};	/*Property = CssBorderImage*/
const Value V24153_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24153_285 = {S7, 0, &V24153_285_57166, &P24153_284};	/*Property = CssCanvas*/
const Value V24153_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24153_286 = {S8, 0, &V24153_286_57168, &P24153_285};	/*Property = CssColor*/
const Value V24153_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24153_237 = {S9, 0, &V24153_237_55786, &P24153_286};	/*Property = CssColumn*/
const Value V24153_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24153_287 = {S10, 0, &V24153_287_57172, &P24153_237};	/*Property = CssFlexbox*/
const Value V24153_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24153_288 = {S11, 0, &V24153_288_57174, &P24153_287};	/*Property = CssFont*/
const Value V24153_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24153_289 = {S12, 0, &V24153_289_57759, &P24153_288};	/*Property = CssImages*/
const Value V24153_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24153_291 = {S13, 0, &V24153_291_57180, &P24153_289};	/*Property = CssMediaQueries*/
const Value V24153_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24153_292 = {S14, 0, &V24153_292_57183, &P24153_291};	/*Property = CssMinMax*/
const Value V24153_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24153_293 = {S15, 0, &V24153_293_57187, &P24153_292};	/*Property = CssOverflow*/
const Value V24153_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24153_294 = {S16, 0, &V24153_294_57189, &P24153_293};	/*Property = CssPosition*/
const Value V24153_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24153_295 = {S17, 0, &V24153_295_57192, &P24153_294};	/*Property = CssText*/
const Value V24153_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24153_297 = {S18, 0, &V24153_297_57195, &P24153_295};	/*Property = CssTransforms*/
const Value V24153_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24153_241 = {S19, 0, &V24153_241_55799, &P24153_297};	/*Property = CssTransitions*/
const Value V24153_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24153_298 = {S20, 0, &V24153_298_57206, &P24153_241};	/*Property = CssUI*/
const Value V24153_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24153_300 = {S21, 0, &V24153_300_57215, &P24153_298};	/*Property = DataSet*/
const Value V24153_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24153_301 = {S22, 0, &V24153_301_57218, &P24153_300};	/*Property = DataUrl*/
const Value V24153_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24153_302 = {S23, 0, &V24153_302_57222, &P24153_301};	/*Property = DeviceOrientation*/
const Value V24153_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24153_304 = {S24, 0, &V24153_304_57227, &P24153_302};	/*Property = FileReader*/
const Value V24153_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24153_305 = {S25, 0, &V24153_305_57231, &P24153_304};	/*Property = FileSaver*/
const Value V24153_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24153_306 = {S26, 0, &V24153_306_57237, &P24153_305};	/*Property = FileWriter*/
const Value V24153_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24153_307 = {S27, 0, &V24153_307_57239, &P24153_306};	/*Property = FormData*/
const Value V24153_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24153_309 = {S28, 0, &V24153_309_57246, &P24153_307};	/*Property = Fullscreen*/
const Value V24153_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24153_310 = {S29, 0, &V24153_310_57248, &P24153_309};	/*Property = GeoLocation*/
const Value V24153_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24153_312 = {S30, 0, &V24153_312_57254, &P24153_310};	/*Property = History*/
const Value V24153_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24153_314 = {S31, 0, &V24153_314_57260, &P24153_312};	/*Property = Html5*/
const Value V24153_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24153_313 = {S32, 0, &V24153_313_57258, &P24153_314};	/*Property = Html-Media-Capture*/
const Value V24153_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24153_315 = {S33, 0, &V24153_315_57264, &P24153_313};	/*Property = Iframe*/
const Value V24153_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24153_316 = {S34, 0, &V24153_316_57266, &P24153_315};	/*Property = IndexedDB*/
const Value V24153_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24153_317 = {S35, 0, &V24153_317_57269, &P24153_316};	/*Property = Json*/
const Value V24153_15_81502 = {S141, NULL};	/*Value = Blink*/
const Property P24153_15 = {S37, 0, &V24153_15_81502, &P24153_317};	/*Property = LayoutEngine*/
const Value V24153_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24153_344 = {S38, 0, &V24153_344_57272, &P24153_15};	/*Property = Masking*/
const Value V24153_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24153_320 = {S39, 0, &V24153_320_57290, &P24153_344};	/*Property = PostMessage*/
const Value V24153_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24153_321 = {S40, 0, &V24153_321_57293, &P24153_320};	/*Property = Progress*/
const Value V24153_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24153_322 = {S41, 0, &V24153_322_57296, &P24153_321};	/*Property = Prompts*/
const Value V24153_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24153_323 = {S42, 0, &V24153_323_57299, &P24153_322};	/*Property = Selector*/
const Value V24153_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24153_324 = {S43, 0, &V24153_324_57305, &P24153_323};	/*Property = Svg*/
const Value V24153_327_57315 = {S0, NULL};	/*Value = False*/
const Property P24153_327 = {S44, 0, &V24153_327_57315, &P24153_324};	/*Property = TouchEvents*/
const Value V24153_328_57317 = {S2, NULL};	/*Value = True*/
const Property P24153_328 = {S45, 0, &V24153_328_57317, &P24153_327};	/*Property = Track*/
const Value V24153_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24153_329 = {S46, 0, &V24153_329_57323, &P24153_328};	/*Property = Video*/
const Value V24153_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24153_330 = {S47, 0, &V24153_330_57326, &P24153_329};	/*Property = Viewport*/
const Value V24153_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24153_332 = {S48, 0, &V24153_332_57338, &P24153_330};	/*Property = WebWorkers*/
const Value V24153_333_57341 = {S2, NULL};	/*Value = True*/
const Property P24153_333 = {S49, 0, &V24153_333_57341, &P24153_332};	/*Property = Xhr2*/
const Profile P24153 = {"24153", &P24153_333};	/*Profile ID - 24153*/
const Value V24505_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_279 = {S1, 0, &V24505_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24505_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_282 = {S3, 0, &V24505_282_57155, &P24505_279};	/*Property = BlobBuilder*/
const Value V24505_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_350 = {S4, 0, &V24505_350_57446, &P24505_282};	/*Property = Canvas*/
const Value V24505_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_283 = {S5, 0, &V24505_283_57161, &P24505_350};	/*Property = CssBackground*/
const Value V24505_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_284 = {S6, 0, &V24505_284_57164, &P24505_283};	/*Property = CssBorderImage*/
const Value V24505_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_285 = {S7, 0, &V24505_285_57167, &P24505_284};	/*Property = CssCanvas*/
const Value V24505_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_286 = {S8, 0, &V24505_286_57170, &P24505_285};	/*Property = CssColor*/
const Value V24505_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_237 = {S9, 0, &V24505_237_55786, &P24505_286};	/*Property = CssColumn*/
const Value V24505_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_287 = {S10, 0, &V24505_287_57173, &P24505_237};	/*Property = CssFlexbox*/
const Value V24505_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_288 = {S11, 0, &V24505_288_57176, &P24505_287};	/*Property = CssFont*/
const Value V24505_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_289 = {S12, 0, &V24505_289_86899, &P24505_288};	/*Property = CssImages*/
const Value V24505_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_291 = {S13, 0, &V24505_291_57182, &P24505_289};	/*Property = CssMediaQueries*/
const Value V24505_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_292 = {S14, 0, &V24505_292_57185, &P24505_291};	/*Property = CssMinMax*/
const Value V24505_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_293 = {S15, 0, &V24505_293_57188, &P24505_292};	/*Property = CssOverflow*/
const Value V24505_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_294 = {S16, 0, &V24505_294_57191, &P24505_293};	/*Property = CssPosition*/
const Value V24505_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_295 = {S17, 0, &V24505_295_57194, &P24505_294};	/*Property = CssText*/
const Value V24505_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_297 = {S18, 0, &V24505_297_57197, &P24505_295};	/*Property = CssTransforms*/
const Value V24505_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_241 = {S19, 0, &V24505_241_55801, &P24505_297};	/*Property = CssTransitions*/
const Value V24505_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_298 = {S20, 0, &V24505_298_57208, &P24505_241};	/*Property = CssUI*/
const Value V24505_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_300 = {S21, 0, &V24505_300_57217, &P24505_298};	/*Property = DataSet*/
const Value V24505_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_301 = {S22, 0, &V24505_301_57220, &P24505_300};	/*Property = DataUrl*/
const Value V24505_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_302 = {S23, 0, &V24505_302_57223, &P24505_301};	/*Property = DeviceOrientation*/
const Value V24505_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_304 = {S24, 0, &V24505_304_57229, &P24505_302};	/*Property = FileReader*/
const Value V24505_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_305 = {S25, 0, &V24505_305_57232, &P24505_304};	/*Property = FileSaver*/
const Value V24505_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_306 = {S26, 0, &V24505_306_57238, &P24505_305};	/*Property = FileWriter*/
const Value V24505_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_307 = {S27, 0, &V24505_307_57241, &P24505_306};	/*Property = FormData*/
const Value V24505_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_309 = {S28, 0, &V24505_309_57247, &P24505_307};	/*Property = Fullscreen*/
const Value V24505_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_310 = {S29, 0, &V24505_310_57250, &P24505_309};	/*Property = GeoLocation*/
const Value V24505_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_312 = {S30, 0, &V24505_312_57256, &P24505_310};	/*Property = History*/
const Value V24505_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_314 = {S31, 0, &V24505_314_57262, &P24505_312};	/*Property = Html5*/
const Value V24505_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_313 = {S32, 0, &V24505_313_57259, &P24505_314};	/*Property = Html-Media-Capture*/
const Value V24505_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_315 = {S33, 0, &V24505_315_57265, &P24505_313};	/*Property = Iframe*/
const Value V24505_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_316 = {S34, 0, &V24505_316_57268, &P24505_315};	/*Property = IndexedDB*/
const Value V24505_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_317 = {S35, 0, &V24505_317_57271, &P24505_316};	/*Property = Json*/
const Value V24505_15_72 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_15 = {S37, 0, &V24505_15_72, &P24505_317};	/*Property = LayoutEngine*/
const Value V24505_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_344 = {S38, 0, &V24505_344_57274, &P24505_15};	/*Property = Masking*/
const Value V24505_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_320 = {S39, 0, &V24505_320_57292, &P24505_344};	/*Property = PostMessage*/
const Value V24505_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_321 = {S40, 0, &V24505_321_57295, &P24505_320};	/*Property = Progress*/
const Value V24505_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_322 = {S41, 0, &V24505_322_57298, &P24505_321};	/*Property = Prompts*/
const Value V24505_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_323 = {S42, 0, &V24505_323_57301, &P24505_322};	/*Property = Selector*/
const Value V24505_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_324 = {S43, 0, &V24505_324_57307, &P24505_323};	/*Property = Svg*/
const Value V24505_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_327 = {S44, 0, &V24505_327_57316, &P24505_324};	/*Property = TouchEvents*/
const Value V24505_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_328 = {S45, 0, &V24505_328_57319, &P24505_327};	/*Property = Track*/
const Value V24505_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_329 = {S46, 0, &V24505_329_57325, &P24505_328};	/*Property = Video*/
const Value V24505_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_330 = {S47, 0, &V24505_330_57328, &P24505_329};	/*Property = Viewport*/
const Value V24505_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_332 = {S48, 0, &V24505_332_57340, &P24505_330};	/*Property = WebWorkers*/
const Value V24505_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24505_333 = {S49, 0, &V24505_333_57343, &P24505_332};	/*Property = Xhr2*/
const Profile P24505 = {"24505", &P24505_333};	/*Profile ID - 24505*/
const Value V24550_1_1 = {S2, NULL};	/*Value = True*/
const Property P24550_1 = {S50, 0, &V24550_1_1, NULL};	/*Property = IsMobile*/
const Value V24550_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24550_5 = {S52, 0, &V24550_5_1227, &P24550_1};	/*Property = ScreenPixelsHeight*/
const Value V24550_4_1477 = {S94, NULL};	/*Value = 1280*/
const Property P24550_4 = {S54, 0, &V24550_4_1477, &P24550_5};	/*Property = ScreenPixelsWidth*/
const Profile P24550 = {"24550", &P24550_4};	/*Profile ID - 24550*/
const Value V24728_1_1 = {S2, NULL};	/*Value = True*/
const Property P24728_1 = {S50, 0, &V24728_1_1, NULL};	/*Property = IsMobile*/
const Value V24728_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24728_5 = {S52, 0, &V24728_5_9981, &P24728_1};	/*Property = ScreenPixelsHeight*/
const Value V24728_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24728_4 = {S54, 0, &V24728_4_39098, &P24728_5};	/*Property = ScreenPixelsWidth*/
const Profile P24728 = {"24728", &P24728_4};	/*Profile ID - 24728*/
const Value V24534_1_1 = {S2, NULL};	/*Value = True*/
const Property P24534_1 = {S50, 0, &V24534_1_1, NULL};	/*Property = IsMobile*/
const Value V24534_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P24534_5 = {S52, 0, &V24534_5_1269, &P24534_1};	/*Property = ScreenPixelsHeight*/
const Value V24534_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24534_4 = {S54, 0, &V24534_4_1229, &P24534_5};	/*Property = ScreenPixelsWidth*/
const Profile P24534 = {"24534", &P24534_4};	/*Profile ID - 24534*/
const Value V24558_1_1 = {S2, NULL};	/*Value = True*/
const Property P24558_1 = {S50, 0, &V24558_1_1, NULL};	/*Property = IsMobile*/
const Value V24558_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24558_5 = {S52, 0, &V24558_5_1227, &P24558_1};	/*Property = ScreenPixelsHeight*/
const Value V24558_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24558_4 = {S54, 0, &V24558_4_1229, &P24558_5};	/*Property = ScreenPixelsWidth*/
const Profile P24558 = {"24558", &P24558_4};	/*Profile ID - 24558*/
const Value V24631_1_1 = {S2, NULL};	/*Value = True*/
const Property P24631_1 = {S50, 0, &V24631_1_1, NULL};	/*Property = IsMobile*/
const Value V24631_5_59138 = {S127, NULL};	/*Value = 1920*/
const Property P24631_5 = {S52, 0, &V24631_5_59138, &P24631_1};	/*Property = ScreenPixelsHeight*/
const Value V24631_4_59970 = {S128, NULL};	/*Value = 1080*/
const Property P24631_4 = {S54, 0, &V24631_4_59970, &P24631_5};	/*Property = ScreenPixelsWidth*/
const Profile P24631 = {"24631", &P24631_4};	/*Profile ID - 24631*/
const Value V24476_1_1 = {S2, NULL};	/*Value = True*/
const Property P24476_1 = {S50, 0, &V24476_1_1, NULL};	/*Property = IsMobile*/
const Value V24476_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24476_5 = {S52, 0, &V24476_5_1403, &P24476_1};	/*Property = ScreenPixelsHeight*/
const Value V24476_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24476_4 = {S54, 0, &V24476_4_1404, &P24476_5};	/*Property = ScreenPixelsWidth*/
const Profile P24476 = {"24476", &P24476_4};	/*Profile ID - 24476*/
const Value V24535_1_1 = {S2, NULL};	/*Value = True*/
const Property P24535_1 = {S50, 0, &V24535_1_1, NULL};	/*Property = IsMobile*/
const Value V24535_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24535_5 = {S52, 0, &V24535_5_1227, &P24535_1};	/*Property = ScreenPixelsHeight*/
const Value V24535_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24535_4 = {S54, 0, &V24535_4_1229, &P24535_5};	/*Property = ScreenPixelsWidth*/
const Profile P24535 = {"24535", &P24535_4};	/*Profile ID - 24535*/
const Value V24606_1_1 = {S2, NULL};	/*Value = True*/
const Property P24606_1 = {S50, 0, &V24606_1_1, NULL};	/*Property = IsMobile*/
const Value V24606_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24606_5 = {S52, 0, &V24606_5_1277, &P24606_1};	/*Property = ScreenPixelsHeight*/
const Value V24606_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24606_4 = {S54, 0, &V24606_4_1279, &P24606_5};	/*Property = ScreenPixelsWidth*/
const Profile P24606 = {"24606", &P24606_4};	/*Profile ID - 24606*/
const Value V24394_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24394_279 = {S1, 0, &V24394_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24394_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24394_282 = {S3, 0, &V24394_282_57154, &P24394_279};	/*Property = BlobBuilder*/
const Value V24394_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24394_350 = {S4, 0, &V24394_350_57444, &P24394_282};	/*Property = Canvas*/
const Value V24394_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24394_283 = {S5, 0, &V24394_283_57159, &P24394_350};	/*Property = CssBackground*/
const Value V24394_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24394_284 = {S6, 0, &V24394_284_57162, &P24394_283};	/*Property = CssBorderImage*/
const Value V24394_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24394_285 = {S7, 0, &V24394_285_57166, &P24394_284};	/*Property = CssCanvas*/
const Value V24394_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24394_286 = {S8, 0, &V24394_286_57168, &P24394_285};	/*Property = CssColor*/
const Value V24394_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24394_237 = {S9, 0, &V24394_237_55786, &P24394_286};	/*Property = CssColumn*/
const Value V24394_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24394_287 = {S10, 0, &V24394_287_57172, &P24394_237};	/*Property = CssFlexbox*/
const Value V24394_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24394_288 = {S11, 0, &V24394_288_57174, &P24394_287};	/*Property = CssFont*/
const Value V24394_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24394_289 = {S12, 0, &V24394_289_57759, &P24394_288};	/*Property = CssImages*/
const Value V24394_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24394_291 = {S13, 0, &V24394_291_57180, &P24394_289};	/*Property = CssMediaQueries*/
const Value V24394_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24394_292 = {S14, 0, &V24394_292_57183, &P24394_291};	/*Property = CssMinMax*/
const Value V24394_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24394_293 = {S15, 0, &V24394_293_57187, &P24394_292};	/*Property = CssOverflow*/
const Value V24394_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24394_294 = {S16, 0, &V24394_294_57189, &P24394_293};	/*Property = CssPosition*/
const Value V24394_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24394_295 = {S17, 0, &V24394_295_57192, &P24394_294};	/*Property = CssText*/
const Value V24394_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24394_297 = {S18, 0, &V24394_297_57195, &P24394_295};	/*Property = CssTransforms*/
const Value V24394_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24394_241 = {S19, 0, &V24394_241_55799, &P24394_297};	/*Property = CssTransitions*/
const Value V24394_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24394_298 = {S20, 0, &V24394_298_57206, &P24394_241};	/*Property = CssUI*/
const Value V24394_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24394_300 = {S21, 0, &V24394_300_57215, &P24394_298};	/*Property = DataSet*/
const Value V24394_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24394_301 = {S22, 0, &V24394_301_57218, &P24394_300};	/*Property = DataUrl*/
const Value V24394_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24394_302 = {S23, 0, &V24394_302_57221, &P24394_301};	/*Property = DeviceOrientation*/
const Value V24394_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24394_304 = {S24, 0, &V24394_304_57227, &P24394_302};	/*Property = FileReader*/
const Value V24394_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24394_305 = {S25, 0, &V24394_305_57231, &P24394_304};	/*Property = FileSaver*/
const Value V24394_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24394_306 = {S26, 0, &V24394_306_57237, &P24394_305};	/*Property = FileWriter*/
const Value V24394_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24394_307 = {S27, 0, &V24394_307_57239, &P24394_306};	/*Property = FormData*/
const Value V24394_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24394_309 = {S28, 0, &V24394_309_57246, &P24394_307};	/*Property = Fullscreen*/
const Value V24394_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24394_310 = {S29, 0, &V24394_310_57248, &P24394_309};	/*Property = GeoLocation*/
const Value V24394_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24394_312 = {S30, 0, &V24394_312_57254, &P24394_310};	/*Property = History*/
const Value V24394_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24394_314 = {S31, 0, &V24394_314_57260, &P24394_312};	/*Property = Html5*/
const Value V24394_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24394_313 = {S32, 0, &V24394_313_57257, &P24394_314};	/*Property = Html-Media-Capture*/
const Value V24394_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24394_315 = {S33, 0, &V24394_315_57264, &P24394_313};	/*Property = Iframe*/
const Value V24394_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24394_316 = {S34, 0, &V24394_316_57266, &P24394_315};	/*Property = IndexedDB*/
const Value V24394_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24394_317 = {S35, 0, &V24394_317_57269, &P24394_316};	/*Property = Json*/
const Value V24394_15_81502 = {S141, NULL};	/*Value = Blink*/
const Property P24394_15 = {S37, 0, &V24394_15_81502, &P24394_317};	/*Property = LayoutEngine*/
const Value V24394_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24394_344 = {S38, 0, &V24394_344_57272, &P24394_15};	/*Property = Masking*/
const Value V24394_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24394_320 = {S39, 0, &V24394_320_57290, &P24394_344};	/*Property = PostMessage*/
const Value V24394_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24394_321 = {S40, 0, &V24394_321_57293, &P24394_320};	/*Property = Progress*/
const Value V24394_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24394_322 = {S41, 0, &V24394_322_57296, &P24394_321};	/*Property = Prompts*/
const Value V24394_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24394_323 = {S42, 0, &V24394_323_57299, &P24394_322};	/*Property = Selector*/
const Value V24394_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24394_324 = {S43, 0, &V24394_324_57305, &P24394_323};	/*Property = Svg*/
const Value V24394_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24394_327 = {S44, 0, &V24394_327_57314, &P24394_324};	/*Property = TouchEvents*/
const Value V24394_328_57317 = {S2, NULL};	/*Value = True*/
const Property P24394_328 = {S45, 0, &V24394_328_57317, &P24394_327};	/*Property = Track*/
const Value V24394_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24394_329 = {S46, 0, &V24394_329_57323, &P24394_328};	/*Property = Video*/
const Value V24394_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24394_330 = {S47, 0, &V24394_330_57326, &P24394_329};	/*Property = Viewport*/
const Value V24394_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24394_332 = {S48, 0, &V24394_332_57338, &P24394_330};	/*Property = WebWorkers*/
const Value V24394_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24394_333 = {S49, 0, &V24394_333_57342, &P24394_332};	/*Property = Xhr2*/
const Profile P24394 = {"24394", &P24394_333};	/*Profile ID - 24394*/
const Profile P24384 = {"24384", NULL};	/*Profile ID - 24384*/
const Value V24402_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24402_279 = {S1, 0, &V24402_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24402_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24402_282 = {S3, 0, &V24402_282_57154, &P24402_279};	/*Property = BlobBuilder*/
const Value V24402_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24402_350 = {S4, 0, &V24402_350_57444, &P24402_282};	/*Property = Canvas*/
const Value V24402_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24402_283 = {S5, 0, &V24402_283_57159, &P24402_350};	/*Property = CssBackground*/
const Value V24402_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24402_284 = {S6, 0, &V24402_284_57162, &P24402_283};	/*Property = CssBorderImage*/
const Value V24402_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24402_285 = {S7, 0, &V24402_285_57166, &P24402_284};	/*Property = CssCanvas*/
const Value V24402_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24402_286 = {S8, 0, &V24402_286_57168, &P24402_285};	/*Property = CssColor*/
const Value V24402_237_55784 = {S2, NULL};	/*Value = True*/
const Property P24402_237 = {S9, 0, &V24402_237_55784, &P24402_286};	/*Property = CssColumn*/
const Value V24402_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24402_287 = {S10, 0, &V24402_287_57172, &P24402_237};	/*Property = CssFlexbox*/
const Value V24402_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24402_288 = {S11, 0, &V24402_288_57175, &P24402_287};	/*Property = CssFont*/
const Value V24402_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24402_289 = {S12, 0, &V24402_289_57759, &P24402_288};	/*Property = CssImages*/
const Value V24402_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24402_291 = {S13, 0, &V24402_291_57180, &P24402_289};	/*Property = CssMediaQueries*/
const Value V24402_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24402_292 = {S14, 0, &V24402_292_57183, &P24402_291};	/*Property = CssMinMax*/
const Value V24402_293_57186 = {S2, NULL};	/*Value = True*/
const Property P24402_293 = {S15, 0, &V24402_293_57186, &P24402_292};	/*Property = CssOverflow*/
const Value V24402_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24402_294 = {S16, 0, &V24402_294_57189, &P24402_293};	/*Property = CssPosition*/
const Value V24402_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24402_295 = {S17, 0, &V24402_295_57192, &P24402_294};	/*Property = CssText*/
const Value V24402_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24402_297 = {S18, 0, &V24402_297_57195, &P24402_295};	/*Property = CssTransforms*/
const Value V24402_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24402_241 = {S19, 0, &V24402_241_55799, &P24402_297};	/*Property = CssTransitions*/
const Value V24402_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24402_298 = {S20, 0, &V24402_298_57206, &P24402_241};	/*Property = CssUI*/
const Value V24402_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24402_300 = {S21, 0, &V24402_300_57215, &P24402_298};	/*Property = DataSet*/
const Value V24402_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24402_301 = {S22, 0, &V24402_301_57218, &P24402_300};	/*Property = DataUrl*/
const Value V24402_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24402_302 = {S23, 0, &V24402_302_57221, &P24402_301};	/*Property = DeviceOrientation*/
const Value V24402_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24402_304 = {S24, 0, &V24402_304_57227, &P24402_302};	/*Property = FileReader*/
const Value V24402_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24402_305 = {S25, 0, &V24402_305_57231, &P24402_304};	/*Property = FileSaver*/
const Value V24402_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24402_306 = {S26, 0, &V24402_306_57237, &P24402_305};	/*Property = FileWriter*/
const Value V24402_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24402_307 = {S27, 0, &V24402_307_57239, &P24402_306};	/*Property = FormData*/
const Value V24402_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24402_309 = {S28, 0, &V24402_309_57246, &P24402_307};	/*Property = Fullscreen*/
const Value V24402_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24402_310 = {S29, 0, &V24402_310_57248, &P24402_309};	/*Property = GeoLocation*/
const Value V24402_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24402_312 = {S30, 0, &V24402_312_57254, &P24402_310};	/*Property = History*/
const Value V24402_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24402_314 = {S31, 0, &V24402_314_57260, &P24402_312};	/*Property = Html5*/
const Value V24402_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24402_313 = {S32, 0, &V24402_313_57258, &P24402_314};	/*Property = Html-Media-Capture*/
const Value V24402_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24402_315 = {S33, 0, &V24402_315_57264, &P24402_313};	/*Property = Iframe*/
const Value V24402_316_57267 = {S0, NULL};	/*Value = False*/
const Property P24402_316 = {S34, 0, &V24402_316_57267, &P24402_315};	/*Property = IndexedDB*/
const Value V24402_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24402_317 = {S35, 0, &V24402_317_57269, &P24402_316};	/*Property = Json*/
const Value V24402_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24402_15 = {S37, 0, &V24402_15_28, &P24402_317};	/*Property = LayoutEngine*/
const Value V24402_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24402_344 = {S38, 0, &V24402_344_57272, &P24402_15};	/*Property = Masking*/
const Value V24402_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24402_320 = {S39, 0, &V24402_320_57290, &P24402_344};	/*Property = PostMessage*/
const Value V24402_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24402_321 = {S40, 0, &V24402_321_57293, &P24402_320};	/*Property = Progress*/
const Value V24402_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24402_322 = {S41, 0, &V24402_322_57296, &P24402_321};	/*Property = Prompts*/
const Value V24402_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24402_323 = {S42, 0, &V24402_323_57299, &P24402_322};	/*Property = Selector*/
const Value V24402_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24402_324 = {S43, 0, &V24402_324_57305, &P24402_323};	/*Property = Svg*/
const Value V24402_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24402_327 = {S44, 0, &V24402_327_57314, &P24402_324};	/*Property = TouchEvents*/
const Value V24402_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24402_328 = {S45, 0, &V24402_328_57318, &P24402_327};	/*Property = Track*/
const Value V24402_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24402_329 = {S46, 0, &V24402_329_57323, &P24402_328};	/*Property = Video*/
const Value V24402_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24402_330 = {S47, 0, &V24402_330_57326, &P24402_329};	/*Property = Viewport*/
const Value V24402_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24402_332 = {S48, 0, &V24402_332_57339, &P24402_330};	/*Property = WebWorkers*/
const Value V24402_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24402_333 = {S49, 0, &V24402_333_57342, &P24402_332};	/*Property = Xhr2*/
const Profile P24402 = {"24402", &P24402_333};	/*Profile ID - 24402*/
const Value V24397_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24397_279 = {S1, 0, &V24397_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24397_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24397_282 = {S3, 0, &V24397_282_57154, &P24397_279};	/*Property = BlobBuilder*/
const Value V24397_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24397_350 = {S4, 0, &V24397_350_57444, &P24397_282};	/*Property = Canvas*/
const Value V24397_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24397_283 = {S5, 0, &V24397_283_57159, &P24397_350};	/*Property = CssBackground*/
const Value V24397_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24397_284 = {S6, 0, &V24397_284_57162, &P24397_283};	/*Property = CssBorderImage*/
const Value V24397_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24397_285 = {S7, 0, &V24397_285_57166, &P24397_284};	/*Property = CssCanvas*/
const Value V24397_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24397_286 = {S8, 0, &V24397_286_57168, &P24397_285};	/*Property = CssColor*/
const Value V24397_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24397_237 = {S9, 0, &V24397_237_55786, &P24397_286};	/*Property = CssColumn*/
const Value V24397_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24397_287 = {S10, 0, &V24397_287_57172, &P24397_237};	/*Property = CssFlexbox*/
const Value V24397_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24397_288 = {S11, 0, &V24397_288_57174, &P24397_287};	/*Property = CssFont*/
const Value V24397_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24397_289 = {S12, 0, &V24397_289_57759, &P24397_288};	/*Property = CssImages*/
const Value V24397_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24397_291 = {S13, 0, &V24397_291_57180, &P24397_289};	/*Property = CssMediaQueries*/
const Value V24397_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24397_292 = {S14, 0, &V24397_292_57183, &P24397_291};	/*Property = CssMinMax*/
const Value V24397_293_57186 = {S2, NULL};	/*Value = True*/
const Property P24397_293 = {S15, 0, &V24397_293_57186, &P24397_292};	/*Property = CssOverflow*/
const Value V24397_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24397_294 = {S16, 0, &V24397_294_57189, &P24397_293};	/*Property = CssPosition*/
const Value V24397_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24397_295 = {S17, 0, &V24397_295_57192, &P24397_294};	/*Property = CssText*/
const Value V24397_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24397_297 = {S18, 0, &V24397_297_57195, &P24397_295};	/*Property = CssTransforms*/
const Value V24397_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24397_241 = {S19, 0, &V24397_241_55799, &P24397_297};	/*Property = CssTransitions*/
const Value V24397_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24397_298 = {S20, 0, &V24397_298_57206, &P24397_241};	/*Property = CssUI*/
const Value V24397_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24397_300 = {S21, 0, &V24397_300_57215, &P24397_298};	/*Property = DataSet*/
const Value V24397_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24397_301 = {S22, 0, &V24397_301_57218, &P24397_300};	/*Property = DataUrl*/
const Value V24397_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24397_302 = {S23, 0, &V24397_302_57221, &P24397_301};	/*Property = DeviceOrientation*/
const Value V24397_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24397_304 = {S24, 0, &V24397_304_57227, &P24397_302};	/*Property = FileReader*/
const Value V24397_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24397_305 = {S25, 0, &V24397_305_57231, &P24397_304};	/*Property = FileSaver*/
const Value V24397_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24397_306 = {S26, 0, &V24397_306_57237, &P24397_305};	/*Property = FileWriter*/
const Value V24397_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24397_307 = {S27, 0, &V24397_307_57239, &P24397_306};	/*Property = FormData*/
const Value V24397_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24397_309 = {S28, 0, &V24397_309_57246, &P24397_307};	/*Property = Fullscreen*/
const Value V24397_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24397_310 = {S29, 0, &V24397_310_57248, &P24397_309};	/*Property = GeoLocation*/
const Value V24397_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24397_312 = {S30, 0, &V24397_312_57254, &P24397_310};	/*Property = History*/
const Value V24397_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24397_314 = {S31, 0, &V24397_314_57260, &P24397_312};	/*Property = Html5*/
const Value V24397_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24397_313 = {S32, 0, &V24397_313_57258, &P24397_314};	/*Property = Html-Media-Capture*/
const Value V24397_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24397_315 = {S33, 0, &V24397_315_57264, &P24397_313};	/*Property = Iframe*/
const Value V24397_316_57267 = {S0, NULL};	/*Value = False*/
const Property P24397_316 = {S34, 0, &V24397_316_57267, &P24397_315};	/*Property = IndexedDB*/
const Value V24397_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24397_317 = {S35, 0, &V24397_317_57269, &P24397_316};	/*Property = Json*/
const Value V24397_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24397_15 = {S37, 0, &V24397_15_28, &P24397_317};	/*Property = LayoutEngine*/
const Value V24397_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24397_344 = {S38, 0, &V24397_344_57272, &P24397_15};	/*Property = Masking*/
const Value V24397_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24397_320 = {S39, 0, &V24397_320_57290, &P24397_344};	/*Property = PostMessage*/
const Value V24397_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24397_321 = {S40, 0, &V24397_321_57293, &P24397_320};	/*Property = Progress*/
const Value V24397_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24397_322 = {S41, 0, &V24397_322_57296, &P24397_321};	/*Property = Prompts*/
const Value V24397_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24397_323 = {S42, 0, &V24397_323_57299, &P24397_322};	/*Property = Selector*/
const Value V24397_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24397_324 = {S43, 0, &V24397_324_57305, &P24397_323};	/*Property = Svg*/
const Value V24397_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24397_327 = {S44, 0, &V24397_327_57314, &P24397_324};	/*Property = TouchEvents*/
const Value V24397_328_57317 = {S2, NULL};	/*Value = True*/
const Property P24397_328 = {S45, 0, &V24397_328_57317, &P24397_327};	/*Property = Track*/
const Value V24397_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24397_329 = {S46, 0, &V24397_329_57323, &P24397_328};	/*Property = Video*/
const Value V24397_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24397_330 = {S47, 0, &V24397_330_57326, &P24397_329};	/*Property = Viewport*/
const Value V24397_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24397_332 = {S48, 0, &V24397_332_57339, &P24397_330};	/*Property = WebWorkers*/
const Value V24397_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24397_333 = {S49, 0, &V24397_333_57342, &P24397_332};	/*Property = Xhr2*/
const Profile P24397 = {"24397", &P24397_333};	/*Profile ID - 24397*/
const Value V24398_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24398_279 = {S1, 0, &V24398_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24398_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24398_282 = {S3, 0, &V24398_282_57154, &P24398_279};	/*Property = BlobBuilder*/
const Value V24398_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24398_350 = {S4, 0, &V24398_350_57444, &P24398_282};	/*Property = Canvas*/
const Value V24398_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24398_283 = {S5, 0, &V24398_283_57159, &P24398_350};	/*Property = CssBackground*/
const Value V24398_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24398_284 = {S6, 0, &V24398_284_57162, &P24398_283};	/*Property = CssBorderImage*/
const Value V24398_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24398_285 = {S7, 0, &V24398_285_57166, &P24398_284};	/*Property = CssCanvas*/
const Value V24398_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24398_286 = {S8, 0, &V24398_286_57168, &P24398_285};	/*Property = CssColor*/
const Value V24398_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24398_237 = {S9, 0, &V24398_237_55786, &P24398_286};	/*Property = CssColumn*/
const Value V24398_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24398_287 = {S10, 0, &V24398_287_57172, &P24398_237};	/*Property = CssFlexbox*/
const Value V24398_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24398_288 = {S11, 0, &V24398_288_57174, &P24398_287};	/*Property = CssFont*/
const Value V24398_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24398_289 = {S12, 0, &V24398_289_57759, &P24398_288};	/*Property = CssImages*/
const Value V24398_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24398_291 = {S13, 0, &V24398_291_57180, &P24398_289};	/*Property = CssMediaQueries*/
const Value V24398_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24398_292 = {S14, 0, &V24398_292_57183, &P24398_291};	/*Property = CssMinMax*/
const Value V24398_293_57186 = {S2, NULL};	/*Value = True*/
const Property P24398_293 = {S15, 0, &V24398_293_57186, &P24398_292};	/*Property = CssOverflow*/
const Value V24398_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24398_294 = {S16, 0, &V24398_294_57189, &P24398_293};	/*Property = CssPosition*/
const Value V24398_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24398_295 = {S17, 0, &V24398_295_57192, &P24398_294};	/*Property = CssText*/
const Value V24398_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24398_297 = {S18, 0, &V24398_297_57195, &P24398_295};	/*Property = CssTransforms*/
const Value V24398_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24398_241 = {S19, 0, &V24398_241_55799, &P24398_297};	/*Property = CssTransitions*/
const Value V24398_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24398_298 = {S20, 0, &V24398_298_57206, &P24398_241};	/*Property = CssUI*/
const Value V24398_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24398_300 = {S21, 0, &V24398_300_57215, &P24398_298};	/*Property = DataSet*/
const Value V24398_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24398_301 = {S22, 0, &V24398_301_57218, &P24398_300};	/*Property = DataUrl*/
const Value V24398_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24398_302 = {S23, 0, &V24398_302_57221, &P24398_301};	/*Property = DeviceOrientation*/
const Value V24398_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24398_304 = {S24, 0, &V24398_304_57227, &P24398_302};	/*Property = FileReader*/
const Value V24398_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24398_305 = {S25, 0, &V24398_305_57231, &P24398_304};	/*Property = FileSaver*/
const Value V24398_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24398_306 = {S26, 0, &V24398_306_57237, &P24398_305};	/*Property = FileWriter*/
const Value V24398_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24398_307 = {S27, 0, &V24398_307_57239, &P24398_306};	/*Property = FormData*/
const Value V24398_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24398_309 = {S28, 0, &V24398_309_57246, &P24398_307};	/*Property = Fullscreen*/
const Value V24398_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24398_310 = {S29, 0, &V24398_310_57248, &P24398_309};	/*Property = GeoLocation*/
const Value V24398_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24398_312 = {S30, 0, &V24398_312_57254, &P24398_310};	/*Property = History*/
const Value V24398_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24398_314 = {S31, 0, &V24398_314_57260, &P24398_312};	/*Property = Html5*/
const Value V24398_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24398_313 = {S32, 0, &V24398_313_57258, &P24398_314};	/*Property = Html-Media-Capture*/
const Value V24398_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24398_315 = {S33, 0, &V24398_315_57264, &P24398_313};	/*Property = Iframe*/
const Value V24398_316_57267 = {S0, NULL};	/*Value = False*/
const Property P24398_316 = {S34, 0, &V24398_316_57267, &P24398_315};	/*Property = IndexedDB*/
const Value V24398_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24398_317 = {S35, 0, &V24398_317_57269, &P24398_316};	/*Property = Json*/
const Value V24398_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24398_15 = {S37, 0, &V24398_15_28, &P24398_317};	/*Property = LayoutEngine*/
const Value V24398_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24398_344 = {S38, 0, &V24398_344_57272, &P24398_15};	/*Property = Masking*/
const Value V24398_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24398_320 = {S39, 0, &V24398_320_57290, &P24398_344};	/*Property = PostMessage*/
const Value V24398_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24398_321 = {S40, 0, &V24398_321_57293, &P24398_320};	/*Property = Progress*/
const Value V24398_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24398_322 = {S41, 0, &V24398_322_57296, &P24398_321};	/*Property = Prompts*/
const Value V24398_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24398_323 = {S42, 0, &V24398_323_57299, &P24398_322};	/*Property = Selector*/
const Value V24398_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24398_324 = {S43, 0, &V24398_324_57305, &P24398_323};	/*Property = Svg*/
const Value V24398_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24398_327 = {S44, 0, &V24398_327_57314, &P24398_324};	/*Property = TouchEvents*/
const Value V24398_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24398_328 = {S45, 0, &V24398_328_57318, &P24398_327};	/*Property = Track*/
const Value V24398_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24398_329 = {S46, 0, &V24398_329_57323, &P24398_328};	/*Property = Video*/
const Value V24398_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24398_330 = {S47, 0, &V24398_330_57326, &P24398_329};	/*Property = Viewport*/
const Value V24398_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24398_332 = {S48, 0, &V24398_332_57339, &P24398_330};	/*Property = WebWorkers*/
const Value V24398_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24398_333 = {S49, 0, &V24398_333_57342, &P24398_332};	/*Property = Xhr2*/
const Profile P24398 = {"24398", &P24398_333};	/*Profile ID - 24398*/
const Value V24399_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24399_279 = {S1, 0, &V24399_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24399_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24399_282 = {S3, 0, &V24399_282_57154, &P24399_279};	/*Property = BlobBuilder*/
const Value V24399_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24399_350 = {S4, 0, &V24399_350_57444, &P24399_282};	/*Property = Canvas*/
const Value V24399_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24399_283 = {S5, 0, &V24399_283_57159, &P24399_350};	/*Property = CssBackground*/
const Value V24399_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24399_284 = {S6, 0, &V24399_284_57162, &P24399_283};	/*Property = CssBorderImage*/
const Value V24399_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24399_285 = {S7, 0, &V24399_285_57166, &P24399_284};	/*Property = CssCanvas*/
const Value V24399_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24399_286 = {S8, 0, &V24399_286_57168, &P24399_285};	/*Property = CssColor*/
const Value V24399_237_55784 = {S2, NULL};	/*Value = True*/
const Property P24399_237 = {S9, 0, &V24399_237_55784, &P24399_286};	/*Property = CssColumn*/
const Value V24399_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24399_287 = {S10, 0, &V24399_287_57172, &P24399_237};	/*Property = CssFlexbox*/
const Value V24399_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24399_288 = {S11, 0, &V24399_288_57174, &P24399_287};	/*Property = CssFont*/
const Value V24399_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24399_289 = {S12, 0, &V24399_289_57759, &P24399_288};	/*Property = CssImages*/
const Value V24399_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24399_291 = {S13, 0, &V24399_291_57180, &P24399_289};	/*Property = CssMediaQueries*/
const Value V24399_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24399_292 = {S14, 0, &V24399_292_57183, &P24399_291};	/*Property = CssMinMax*/
const Value V24399_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24399_293 = {S15, 0, &V24399_293_57187, &P24399_292};	/*Property = CssOverflow*/
const Value V24399_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24399_294 = {S16, 0, &V24399_294_57189, &P24399_293};	/*Property = CssPosition*/
const Value V24399_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24399_295 = {S17, 0, &V24399_295_57192, &P24399_294};	/*Property = CssText*/
const Value V24399_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24399_297 = {S18, 0, &V24399_297_57195, &P24399_295};	/*Property = CssTransforms*/
const Value V24399_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24399_241 = {S19, 0, &V24399_241_55799, &P24399_297};	/*Property = CssTransitions*/
const Value V24399_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24399_298 = {S20, 0, &V24399_298_57206, &P24399_241};	/*Property = CssUI*/
const Value V24399_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24399_300 = {S21, 0, &V24399_300_57215, &P24399_298};	/*Property = DataSet*/
const Value V24399_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24399_301 = {S22, 0, &V24399_301_57218, &P24399_300};	/*Property = DataUrl*/
const Value V24399_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24399_302 = {S23, 0, &V24399_302_57222, &P24399_301};	/*Property = DeviceOrientation*/
const Value V24399_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24399_304 = {S24, 0, &V24399_304_57227, &P24399_302};	/*Property = FileReader*/
const Value V24399_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24399_305 = {S25, 0, &V24399_305_57231, &P24399_304};	/*Property = FileSaver*/
const Value V24399_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24399_306 = {S26, 0, &V24399_306_57237, &P24399_305};	/*Property = FileWriter*/
const Value V24399_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24399_307 = {S27, 0, &V24399_307_57239, &P24399_306};	/*Property = FormData*/
const Value V24399_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24399_309 = {S28, 0, &V24399_309_57246, &P24399_307};	/*Property = Fullscreen*/
const Value V24399_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24399_310 = {S29, 0, &V24399_310_57248, &P24399_309};	/*Property = GeoLocation*/
const Value V24399_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24399_312 = {S30, 0, &V24399_312_57254, &P24399_310};	/*Property = History*/
const Value V24399_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24399_314 = {S31, 0, &V24399_314_57260, &P24399_312};	/*Property = Html5*/
const Value V24399_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24399_313 = {S32, 0, &V24399_313_57257, &P24399_314};	/*Property = Html-Media-Capture*/
const Value V24399_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24399_315 = {S33, 0, &V24399_315_57264, &P24399_313};	/*Property = Iframe*/
const Value V24399_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24399_316 = {S34, 0, &V24399_316_57266, &P24399_315};	/*Property = IndexedDB*/
const Value V24399_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24399_317 = {S35, 0, &V24399_317_57269, &P24399_316};	/*Property = Json*/
const Value V24399_15_81502 = {S141, NULL};	/*Value = Blink*/
const Property P24399_15 = {S37, 0, &V24399_15_81502, &P24399_317};	/*Property = LayoutEngine*/
const Value V24399_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24399_344 = {S38, 0, &V24399_344_57272, &P24399_15};	/*Property = Masking*/
const Value V24399_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24399_320 = {S39, 0, &V24399_320_57290, &P24399_344};	/*Property = PostMessage*/
const Value V24399_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24399_321 = {S40, 0, &V24399_321_57293, &P24399_320};	/*Property = Progress*/
const Value V24399_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24399_322 = {S41, 0, &V24399_322_57296, &P24399_321};	/*Property = Prompts*/
const Value V24399_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24399_323 = {S42, 0, &V24399_323_57299, &P24399_322};	/*Property = Selector*/
const Value V24399_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24399_324 = {S43, 0, &V24399_324_57305, &P24399_323};	/*Property = Svg*/
const Value V24399_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24399_327 = {S44, 0, &V24399_327_57314, &P24399_324};	/*Property = TouchEvents*/
const Value V24399_328_57317 = {S2, NULL};	/*Value = True*/
const Property P24399_328 = {S45, 0, &V24399_328_57317, &P24399_327};	/*Property = Track*/
const Value V24399_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24399_329 = {S46, 0, &V24399_329_57323, &P24399_328};	/*Property = Video*/
const Value V24399_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24399_330 = {S47, 0, &V24399_330_57326, &P24399_329};	/*Property = Viewport*/
const Value V24399_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24399_332 = {S48, 0, &V24399_332_57338, &P24399_330};	/*Property = WebWorkers*/
const Value V24399_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24399_333 = {S49, 0, &V24399_333_57342, &P24399_332};	/*Property = Xhr2*/
const Profile P24399 = {"24399", &P24399_333};	/*Profile ID - 24399*/
const Value V24411_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24411_279 = {S1, 0, &V24411_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24411_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24411_282 = {S3, 0, &V24411_282_57154, &P24411_279};	/*Property = BlobBuilder*/
const Value V24411_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24411_350 = {S4, 0, &V24411_350_57444, &P24411_282};	/*Property = Canvas*/
const Value V24411_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24411_283 = {S5, 0, &V24411_283_57159, &P24411_350};	/*Property = CssBackground*/
const Value V24411_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24411_284 = {S6, 0, &V24411_284_57162, &P24411_283};	/*Property = CssBorderImage*/
const Value V24411_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24411_285 = {S7, 0, &V24411_285_57166, &P24411_284};	/*Property = CssCanvas*/
const Value V24411_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24411_286 = {S8, 0, &V24411_286_57168, &P24411_285};	/*Property = CssColor*/
const Value V24411_237_55784 = {S2, NULL};	/*Value = True*/
const Property P24411_237 = {S9, 0, &V24411_237_55784, &P24411_286};	/*Property = CssColumn*/
const Value V24411_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24411_287 = {S10, 0, &V24411_287_57172, &P24411_237};	/*Property = CssFlexbox*/
const Value V24411_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24411_288 = {S11, 0, &V24411_288_57175, &P24411_287};	/*Property = CssFont*/
const Value V24411_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24411_289 = {S12, 0, &V24411_289_57759, &P24411_288};	/*Property = CssImages*/
const Value V24411_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24411_291 = {S13, 0, &V24411_291_57180, &P24411_289};	/*Property = CssMediaQueries*/
const Value V24411_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24411_292 = {S14, 0, &V24411_292_57183, &P24411_291};	/*Property = CssMinMax*/
const Value V24411_293_57186 = {S2, NULL};	/*Value = True*/
const Property P24411_293 = {S15, 0, &V24411_293_57186, &P24411_292};	/*Property = CssOverflow*/
const Value V24411_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24411_294 = {S16, 0, &V24411_294_57189, &P24411_293};	/*Property = CssPosition*/
const Value V24411_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24411_295 = {S17, 0, &V24411_295_57192, &P24411_294};	/*Property = CssText*/
const Value V24411_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24411_297 = {S18, 0, &V24411_297_57195, &P24411_295};	/*Property = CssTransforms*/
const Value V24411_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24411_241 = {S19, 0, &V24411_241_55799, &P24411_297};	/*Property = CssTransitions*/
const Value V24411_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24411_298 = {S20, 0, &V24411_298_57206, &P24411_241};	/*Property = CssUI*/
const Value V24411_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24411_300 = {S21, 0, &V24411_300_57215, &P24411_298};	/*Property = DataSet*/
const Value V24411_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24411_301 = {S22, 0, &V24411_301_57218, &P24411_300};	/*Property = DataUrl*/
const Value V24411_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24411_302 = {S23, 0, &V24411_302_57221, &P24411_301};	/*Property = DeviceOrientation*/
const Value V24411_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24411_304 = {S24, 0, &V24411_304_57227, &P24411_302};	/*Property = FileReader*/
const Value V24411_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24411_305 = {S25, 0, &V24411_305_57231, &P24411_304};	/*Property = FileSaver*/
const Value V24411_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24411_306 = {S26, 0, &V24411_306_57237, &P24411_305};	/*Property = FileWriter*/
const Value V24411_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24411_307 = {S27, 0, &V24411_307_57239, &P24411_306};	/*Property = FormData*/
const Value V24411_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24411_309 = {S28, 0, &V24411_309_57246, &P24411_307};	/*Property = Fullscreen*/
const Value V24411_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24411_310 = {S29, 0, &V24411_310_57248, &P24411_309};	/*Property = GeoLocation*/
const Value V24411_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24411_312 = {S30, 0, &V24411_312_57254, &P24411_310};	/*Property = History*/
const Value V24411_314_57261 = {S0, NULL};	/*Value = False*/
const Property P24411_314 = {S31, 0, &V24411_314_57261, &P24411_312};	/*Property = Html5*/
const Value V24411_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24411_313 = {S32, 0, &V24411_313_57258, &P24411_314};	/*Property = Html-Media-Capture*/
const Value V24411_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24411_315 = {S33, 0, &V24411_315_57264, &P24411_313};	/*Property = Iframe*/
const Value V24411_316_57267 = {S0, NULL};	/*Value = False*/
const Property P24411_316 = {S34, 0, &V24411_316_57267, &P24411_315};	/*Property = IndexedDB*/
const Value V24411_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24411_317 = {S35, 0, &V24411_317_57269, &P24411_316};	/*Property = Json*/
const Value V24411_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24411_15 = {S37, 0, &V24411_15_28, &P24411_317};	/*Property = LayoutEngine*/
const Value V24411_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24411_344 = {S38, 0, &V24411_344_57272, &P24411_15};	/*Property = Masking*/
const Value V24411_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24411_320 = {S39, 0, &V24411_320_57290, &P24411_344};	/*Property = PostMessage*/
const Value V24411_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24411_321 = {S40, 0, &V24411_321_57293, &P24411_320};	/*Property = Progress*/
const Value V24411_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24411_322 = {S41, 0, &V24411_322_57296, &P24411_321};	/*Property = Prompts*/
const Value V24411_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24411_323 = {S42, 0, &V24411_323_57299, &P24411_322};	/*Property = Selector*/
const Value V24411_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24411_324 = {S43, 0, &V24411_324_57305, &P24411_323};	/*Property = Svg*/
const Value V24411_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24411_327 = {S44, 0, &V24411_327_57314, &P24411_324};	/*Property = TouchEvents*/
const Value V24411_328_57317 = {S2, NULL};	/*Value = True*/
const Property P24411_328 = {S45, 0, &V24411_328_57317, &P24411_327};	/*Property = Track*/
const Value V24411_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24411_329 = {S46, 0, &V24411_329_57323, &P24411_328};	/*Property = Video*/
const Value V24411_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24411_330 = {S47, 0, &V24411_330_57326, &P24411_329};	/*Property = Viewport*/
const Value V24411_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24411_332 = {S48, 0, &V24411_332_57339, &P24411_330};	/*Property = WebWorkers*/
const Value V24411_333_57341 = {S2, NULL};	/*Value = True*/
const Property P24411_333 = {S49, 0, &V24411_333_57341, &P24411_332};	/*Property = Xhr2*/
const Profile P24411 = {"24411", &P24411_333};	/*Profile ID - 24411*/
const Value V24462_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24462_279 = {S1, 0, &V24462_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24462_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24462_282 = {S3, 0, &V24462_282_57154, &P24462_279};	/*Property = BlobBuilder*/
const Value V24462_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24462_350 = {S4, 0, &V24462_350_57444, &P24462_282};	/*Property = Canvas*/
const Value V24462_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24462_283 = {S5, 0, &V24462_283_57159, &P24462_350};	/*Property = CssBackground*/
const Value V24462_284_57163 = {S0, NULL};	/*Value = False*/
const Property P24462_284 = {S6, 0, &V24462_284_57163, &P24462_283};	/*Property = CssBorderImage*/
const Value V24462_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24462_285 = {S7, 0, &V24462_285_57166, &P24462_284};	/*Property = CssCanvas*/
const Value V24462_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24462_286 = {S8, 0, &V24462_286_57168, &P24462_285};	/*Property = CssColor*/
const Value V24462_237_55784 = {S2, NULL};	/*Value = True*/
const Property P24462_237 = {S9, 0, &V24462_237_55784, &P24462_286};	/*Property = CssColumn*/
const Value V24462_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24462_287 = {S10, 0, &V24462_287_57172, &P24462_237};	/*Property = CssFlexbox*/
const Value V24462_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24462_288 = {S11, 0, &V24462_288_57175, &P24462_287};	/*Property = CssFont*/
const Value V24462_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24462_289 = {S12, 0, &V24462_289_57759, &P24462_288};	/*Property = CssImages*/
const Value V24462_291_57181 = {S0, NULL};	/*Value = False*/
const Property P24462_291 = {S13, 0, &V24462_291_57181, &P24462_289};	/*Property = CssMediaQueries*/
const Value V24462_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24462_292 = {S14, 0, &V24462_292_57183, &P24462_291};	/*Property = CssMinMax*/
const Value V24462_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24462_293 = {S15, 0, &V24462_293_57187, &P24462_292};	/*Property = CssOverflow*/
const Value V24462_294_57190 = {S0, NULL};	/*Value = False*/
const Property P24462_294 = {S16, 0, &V24462_294_57190, &P24462_293};	/*Property = CssPosition*/
const Value V24462_295_57193 = {S0, NULL};	/*Value = False*/
const Property P24462_295 = {S17, 0, &V24462_295_57193, &P24462_294};	/*Property = CssText*/
const Value V24462_297_57196 = {S0, NULL};	/*Value = False*/
const Property P24462_297 = {S18, 0, &V24462_297_57196, &P24462_295};	/*Property = CssTransforms*/
const Value V24462_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24462_241 = {S19, 0, &V24462_241_55799, &P24462_297};	/*Property = CssTransitions*/
const Value V24462_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24462_298 = {S20, 0, &V24462_298_57206, &P24462_241};	/*Property = CssUI*/
const Value V24462_300_57216 = {S0, NULL};	/*Value = False*/
const Property P24462_300 = {S21, 0, &V24462_300_57216, &P24462_298};	/*Property = DataSet*/
const Value V24462_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24462_301 = {S22, 0, &V24462_301_57218, &P24462_300};	/*Property = DataUrl*/
const Value V24462_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24462_302 = {S23, 0, &V24462_302_57222, &P24462_301};	/*Property = DeviceOrientation*/
const Value V24462_304_57228 = {S0, NULL};	/*Value = False*/
const Property P24462_304 = {S24, 0, &V24462_304_57228, &P24462_302};	/*Property = FileReader*/
const Value V24462_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24462_305 = {S25, 0, &V24462_305_57231, &P24462_304};	/*Property = FileSaver*/
const Value V24462_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24462_306 = {S26, 0, &V24462_306_57237, &P24462_305};	/*Property = FileWriter*/
const Value V24462_307_57240 = {S0, NULL};	/*Value = False*/
const Property P24462_307 = {S27, 0, &V24462_307_57240, &P24462_306};	/*Property = FormData*/
const Value V24462_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24462_309 = {S28, 0, &V24462_309_57246, &P24462_307};	/*Property = Fullscreen*/
const Value V24462_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24462_310 = {S29, 0, &V24462_310_57248, &P24462_309};	/*Property = GeoLocation*/
const Value V24462_312_57255 = {S0, NULL};	/*Value = False*/
const Property P24462_312 = {S30, 0, &V24462_312_57255, &P24462_310};	/*Property = History*/
const Value V24462_314_57261 = {S0, NULL};	/*Value = False*/
const Property P24462_314 = {S31, 0, &V24462_314_57261, &P24462_312};	/*Property = Html5*/
const Value V24462_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24462_313 = {S32, 0, &V24462_313_57258, &P24462_314};	/*Property = Html-Media-Capture*/
const Value V24462_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24462_315 = {S33, 0, &V24462_315_57264, &P24462_313};	/*Property = Iframe*/
const Value V24462_316_57267 = {S0, NULL};	/*Value = False*/
const Property P24462_316 = {S34, 0, &V24462_316_57267, &P24462_315};	/*Property = IndexedDB*/
const Value V24462_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24462_317 = {S35, 0, &V24462_317_57269, &P24462_316};	/*Property = Json*/
const Value V24462_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24462_15 = {S37, 0, &V24462_15_28, &P24462_317};	/*Property = LayoutEngine*/
const Value V24462_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24462_344 = {S38, 0, &V24462_344_57272, &P24462_15};	/*Property = Masking*/
const Value V24462_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24462_320 = {S39, 0, &V24462_320_57290, &P24462_344};	/*Property = PostMessage*/
const Value V24462_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24462_321 = {S40, 0, &V24462_321_57293, &P24462_320};	/*Property = Progress*/
const Value V24462_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24462_322 = {S41, 0, &V24462_322_57296, &P24462_321};	/*Property = Prompts*/
const Value V24462_323_57300 = {S0, NULL};	/*Value = False*/
const Property P24462_323 = {S42, 0, &V24462_323_57300, &P24462_322};	/*Property = Selector*/
const Value V24462_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24462_324 = {S43, 0, &V24462_324_57306, &P24462_323};	/*Property = Svg*/
const Value V24462_327_57315 = {S0, NULL};	/*Value = False*/
const Property P24462_327 = {S44, 0, &V24462_327_57315, &P24462_324};	/*Property = TouchEvents*/
const Value V24462_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24462_328 = {S45, 0, &V24462_328_57318, &P24462_327};	/*Property = Track*/
const Value V24462_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24462_329 = {S46, 0, &V24462_329_57323, &P24462_328};	/*Property = Video*/
const Value V24462_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24462_330 = {S47, 0, &V24462_330_57326, &P24462_329};	/*Property = Viewport*/
const Value V24462_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24462_332 = {S48, 0, &V24462_332_57339, &P24462_330};	/*Property = WebWorkers*/
const Value V24462_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24462_333 = {S49, 0, &V24462_333_57342, &P24462_332};	/*Property = Xhr2*/
const Profile P24462 = {"24462", &P24462_333};	/*Profile ID - 24462*/
const Profile P24412 = {"24412", NULL};	/*Profile ID - 24412*/
const Value V23973_1_1 = {S2, NULL};	/*Value = True*/
const Property P23973_1 = {S50, 0, &V23973_1_1, NULL};	/*Property = IsMobile*/
const Value V23973_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P23973_5 = {S52, 0, &V23973_5_1227, &P23973_1};	/*Property = ScreenPixelsHeight*/
const Value V23973_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P23973_4 = {S54, 0, &V23973_4_1229, &P23973_5};	/*Property = ScreenPixelsWidth*/
const Profile P23973 = {"23973", &P23973_4};	/*Profile ID - 23973*/
const Value V24682_1_1 = {S2, NULL};	/*Value = True*/
const Property P24682_1 = {S50, 0, &V24682_1_1, NULL};	/*Property = IsMobile*/
const Value V24682_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24682_5 = {S52, 0, &V24682_5_9981, &P24682_1};	/*Property = ScreenPixelsHeight*/
const Value V24682_4_39098 = {S104, NULL};	/*Value = 720*/
const Property P24682_4 = {S54, 0, &V24682_4_39098, &P24682_5};	/*Property = ScreenPixelsWidth*/
const Profile P24682 = {"24682", &P24682_4};	/*Profile ID - 24682*/
const Value V24718_1_1 = {S2, NULL};	/*Value = True*/
const Property P24718_1 = {S50, 0, &V24718_1_1, NULL};	/*Property = IsMobile*/
const Value V24718_5_1269 = {S69, NULL};	/*Value = 854*/
const Property P24718_5 = {S52, 0, &V24718_5_1269, &P24718_1};	/*Property = ScreenPixelsHeight*/
const Value V24718_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24718_4 = {S54, 0, &V24718_4_1229, &P24718_5};	/*Property = ScreenPixelsWidth*/
const Profile P24718 = {"24718", &P24718_4};	/*Profile ID - 24718*/
const Value V12109_1_1 = {S2, NULL};	/*Value = True*/
const Property P12109_1 = {S50, 0, &V12109_1_1, NULL};	/*Property = IsMobile*/
const Value V12109_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P12109_5 = {S52, 0, &V12109_5_1222, &P12109_1};	/*Property = ScreenPixelsHeight*/
const Value V12109_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P12109_4 = {S54, 0, &V12109_4_1224, &P12109_5};	/*Property = ScreenPixelsWidth*/
const Profile P12109 = {"12109", &P12109_4};	/*Profile ID - 12109*/
const Value V24533_1_1 = {S2, NULL};	/*Value = True*/
const Property P24533_1 = {S50, 0, &V24533_1_1, NULL};	/*Property = IsMobile*/
const Value V24533_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24533_5 = {S52, 0, &V24533_5_1227, &P24533_1};	/*Property = ScreenPixelsHeight*/
const Value V24533_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24533_4 = {S54, 0, &V24533_4_1229, &P24533_5};	/*Property = ScreenPixelsWidth*/
const Profile P24533 = {"24533", &P24533_4};	/*Profile ID - 24533*/
const Value V24736_1_1 = {S2, NULL};	/*Value = True*/
const Property P24736_1 = {S50, 0, &V24736_1_1, NULL};	/*Property = IsMobile*/
const Value V24736_5_9981 = {S94, NULL};	/*Value = 1280*/
const Property P24736_5 = {S52, 0, &V24736_5_9981, &P24736_1};	/*Property = ScreenPixelsHeight*/
const Value V24736_4_37617 = {S102, NULL};	/*Value = 768*/
const Property P24736_4 = {S54, 0, &V24736_4_37617, &P24736_5};	/*Property = ScreenPixelsWidth*/
const Profile P24736 = {"24736", &P24736_4};	/*Profile ID - 24736*/
const Value V24618_1_1 = {S2, NULL};	/*Value = True*/
const Property P24618_1 = {S50, 0, &V24618_1_1, NULL};	/*Property = IsMobile*/
const Value V24618_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24618_5 = {S52, 0, &V24618_5_1227, &P24618_1};	/*Property = ScreenPixelsHeight*/
const Value V24618_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24618_4 = {S54, 0, &V24618_4_1229, &P24618_5};	/*Property = ScreenPixelsWidth*/
const Profile P24618 = {"24618", &P24618_4};	/*Profile ID - 24618*/
const Value V24215_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24215_279 = {S1, 0, &V24215_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24215_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24215_282 = {S3, 0, &V24215_282_57154, &P24215_279};	/*Property = BlobBuilder*/
const Value V24215_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24215_350 = {S4, 0, &V24215_350_57444, &P24215_282};	/*Property = Canvas*/
const Value V24215_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24215_283 = {S5, 0, &V24215_283_57159, &P24215_350};	/*Property = CssBackground*/
const Value V24215_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24215_284 = {S6, 0, &V24215_284_57162, &P24215_283};	/*Property = CssBorderImage*/
const Value V24215_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24215_285 = {S7, 0, &V24215_285_57166, &P24215_284};	/*Property = CssCanvas*/
const Value V24215_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24215_286 = {S8, 0, &V24215_286_57168, &P24215_285};	/*Property = CssColor*/
const Value V24215_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24215_237 = {S9, 0, &V24215_237_55786, &P24215_286};	/*Property = CssColumn*/
const Value V24215_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24215_287 = {S10, 0, &V24215_287_57172, &P24215_237};	/*Property = CssFlexbox*/
const Value V24215_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24215_288 = {S11, 0, &V24215_288_57175, &P24215_287};	/*Property = CssFont*/
const Value V24215_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24215_289 = {S12, 0, &V24215_289_57759, &P24215_288};	/*Property = CssImages*/
const Value V24215_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24215_291 = {S13, 0, &V24215_291_57180, &P24215_289};	/*Property = CssMediaQueries*/
const Value V24215_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24215_292 = {S14, 0, &V24215_292_57183, &P24215_291};	/*Property = CssMinMax*/
const Value V24215_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24215_293 = {S15, 0, &V24215_293_57187, &P24215_292};	/*Property = CssOverflow*/
const Value V24215_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24215_294 = {S16, 0, &V24215_294_57189, &P24215_293};	/*Property = CssPosition*/
const Value V24215_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24215_295 = {S17, 0, &V24215_295_57192, &P24215_294};	/*Property = CssText*/
const Value V24215_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24215_297 = {S18, 0, &V24215_297_57195, &P24215_295};	/*Property = CssTransforms*/
const Value V24215_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24215_241 = {S19, 0, &V24215_241_55799, &P24215_297};	/*Property = CssTransitions*/
const Value V24215_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24215_298 = {S20, 0, &V24215_298_57206, &P24215_241};	/*Property = CssUI*/
const Value V24215_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24215_300 = {S21, 0, &V24215_300_57215, &P24215_298};	/*Property = DataSet*/
const Value V24215_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24215_301 = {S22, 0, &V24215_301_57218, &P24215_300};	/*Property = DataUrl*/
const Value V24215_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24215_302 = {S23, 0, &V24215_302_57221, &P24215_301};	/*Property = DeviceOrientation*/
const Value V24215_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24215_304 = {S24, 0, &V24215_304_57227, &P24215_302};	/*Property = FileReader*/
const Value V24215_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24215_305 = {S25, 0, &V24215_305_57231, &P24215_304};	/*Property = FileSaver*/
const Value V24215_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24215_306 = {S26, 0, &V24215_306_57237, &P24215_305};	/*Property = FileWriter*/
const Value V24215_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24215_307 = {S27, 0, &V24215_307_57239, &P24215_306};	/*Property = FormData*/
const Value V24215_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24215_309 = {S28, 0, &V24215_309_57246, &P24215_307};	/*Property = Fullscreen*/
const Value V24215_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24215_310 = {S29, 0, &V24215_310_57248, &P24215_309};	/*Property = GeoLocation*/
const Value V24215_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24215_312 = {S30, 0, &V24215_312_57254, &P24215_310};	/*Property = History*/
const Value V24215_314_57261 = {S0, NULL};	/*Value = False*/
const Property P24215_314 = {S31, 0, &V24215_314_57261, &P24215_312};	/*Property = Html5*/
const Value V24215_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24215_313 = {S32, 0, &V24215_313_57258, &P24215_314};	/*Property = Html-Media-Capture*/
const Value V24215_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24215_315 = {S33, 0, &V24215_315_57264, &P24215_313};	/*Property = Iframe*/
const Value V24215_316_57267 = {S0, NULL};	/*Value = False*/
const Property P24215_316 = {S34, 0, &V24215_316_57267, &P24215_315};	/*Property = IndexedDB*/
const Value V24215_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24215_317 = {S35, 0, &V24215_317_57269, &P24215_316};	/*Property = Json*/
const Value V24215_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24215_15 = {S37, 0, &V24215_15_28, &P24215_317};	/*Property = LayoutEngine*/
const Value V24215_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24215_344 = {S38, 0, &V24215_344_57272, &P24215_15};	/*Property = Masking*/
const Value V24215_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24215_320 = {S39, 0, &V24215_320_57290, &P24215_344};	/*Property = PostMessage*/
const Value V24215_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24215_321 = {S40, 0, &V24215_321_57293, &P24215_320};	/*Property = Progress*/
const Value V24215_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24215_322 = {S41, 0, &V24215_322_57296, &P24215_321};	/*Property = Prompts*/
const Value V24215_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24215_323 = {S42, 0, &V24215_323_57299, &P24215_322};	/*Property = Selector*/
const Value V24215_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24215_324 = {S43, 0, &V24215_324_57306, &P24215_323};	/*Property = Svg*/
const Value V24215_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24215_327 = {S44, 0, &V24215_327_57314, &P24215_324};	/*Property = TouchEvents*/
const Value V24215_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24215_328 = {S45, 0, &V24215_328_57318, &P24215_327};	/*Property = Track*/
const Value V24215_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24215_329 = {S46, 0, &V24215_329_57323, &P24215_328};	/*Property = Video*/
const Value V24215_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24215_330 = {S47, 0, &V24215_330_57326, &P24215_329};	/*Property = Viewport*/
const Value V24215_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24215_332 = {S48, 0, &V24215_332_57339, &P24215_330};	/*Property = WebWorkers*/
const Value V24215_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24215_333 = {S49, 0, &V24215_333_57342, &P24215_332};	/*Property = Xhr2*/
const Profile P24215 = {"24215", &P24215_333};	/*Profile ID - 24215*/
const Value V24680_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_279 = {S1, 0, &V24680_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24680_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_282 = {S3, 0, &V24680_282_57155, &P24680_279};	/*Property = BlobBuilder*/
const Value V24680_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_350 = {S4, 0, &V24680_350_57446, &P24680_282};	/*Property = Canvas*/
const Value V24680_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_283 = {S5, 0, &V24680_283_57161, &P24680_350};	/*Property = CssBackground*/
const Value V24680_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_284 = {S6, 0, &V24680_284_57164, &P24680_283};	/*Property = CssBorderImage*/
const Value V24680_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_285 = {S7, 0, &V24680_285_57167, &P24680_284};	/*Property = CssCanvas*/
const Value V24680_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_286 = {S8, 0, &V24680_286_57170, &P24680_285};	/*Property = CssColor*/
const Value V24680_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_237 = {S9, 0, &V24680_237_55786, &P24680_286};	/*Property = CssColumn*/
const Value V24680_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_287 = {S10, 0, &V24680_287_57173, &P24680_237};	/*Property = CssFlexbox*/
const Value V24680_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_288 = {S11, 0, &V24680_288_57176, &P24680_287};	/*Property = CssFont*/
const Value V24680_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_289 = {S12, 0, &V24680_289_86899, &P24680_288};	/*Property = CssImages*/
const Value V24680_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_291 = {S13, 0, &V24680_291_57182, &P24680_289};	/*Property = CssMediaQueries*/
const Value V24680_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_292 = {S14, 0, &V24680_292_57185, &P24680_291};	/*Property = CssMinMax*/
const Value V24680_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_293 = {S15, 0, &V24680_293_57188, &P24680_292};	/*Property = CssOverflow*/
const Value V24680_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_294 = {S16, 0, &V24680_294_57191, &P24680_293};	/*Property = CssPosition*/
const Value V24680_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_295 = {S17, 0, &V24680_295_57194, &P24680_294};	/*Property = CssText*/
const Value V24680_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_297 = {S18, 0, &V24680_297_57197, &P24680_295};	/*Property = CssTransforms*/
const Value V24680_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_241 = {S19, 0, &V24680_241_55801, &P24680_297};	/*Property = CssTransitions*/
const Value V24680_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_298 = {S20, 0, &V24680_298_57208, &P24680_241};	/*Property = CssUI*/
const Value V24680_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_300 = {S21, 0, &V24680_300_57217, &P24680_298};	/*Property = DataSet*/
const Value V24680_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_301 = {S22, 0, &V24680_301_57220, &P24680_300};	/*Property = DataUrl*/
const Value V24680_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_302 = {S23, 0, &V24680_302_57223, &P24680_301};	/*Property = DeviceOrientation*/
const Value V24680_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_304 = {S24, 0, &V24680_304_57229, &P24680_302};	/*Property = FileReader*/
const Value V24680_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_305 = {S25, 0, &V24680_305_57232, &P24680_304};	/*Property = FileSaver*/
const Value V24680_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_306 = {S26, 0, &V24680_306_57238, &P24680_305};	/*Property = FileWriter*/
const Value V24680_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_307 = {S27, 0, &V24680_307_57241, &P24680_306};	/*Property = FormData*/
const Value V24680_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_309 = {S28, 0, &V24680_309_57247, &P24680_307};	/*Property = Fullscreen*/
const Value V24680_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_310 = {S29, 0, &V24680_310_57250, &P24680_309};	/*Property = GeoLocation*/
const Value V24680_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_312 = {S30, 0, &V24680_312_57256, &P24680_310};	/*Property = History*/
const Value V24680_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_314 = {S31, 0, &V24680_314_57262, &P24680_312};	/*Property = Html5*/
const Value V24680_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_313 = {S32, 0, &V24680_313_57259, &P24680_314};	/*Property = Html-Media-Capture*/
const Value V24680_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_315 = {S33, 0, &V24680_315_57265, &P24680_313};	/*Property = Iframe*/
const Value V24680_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_316 = {S34, 0, &V24680_316_57268, &P24680_315};	/*Property = IndexedDB*/
const Value V24680_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_317 = {S35, 0, &V24680_317_57271, &P24680_316};	/*Property = Json*/
const Value V24680_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24680_15 = {S37, 0, &V24680_15_90837, &P24680_317};	/*Property = LayoutEngine*/
const Value V24680_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_344 = {S38, 0, &V24680_344_57274, &P24680_15};	/*Property = Masking*/
const Value V24680_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_320 = {S39, 0, &V24680_320_57292, &P24680_344};	/*Property = PostMessage*/
const Value V24680_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_321 = {S40, 0, &V24680_321_57295, &P24680_320};	/*Property = Progress*/
const Value V24680_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_322 = {S41, 0, &V24680_322_57298, &P24680_321};	/*Property = Prompts*/
const Value V24680_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_323 = {S42, 0, &V24680_323_57301, &P24680_322};	/*Property = Selector*/
const Value V24680_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_324 = {S43, 0, &V24680_324_57307, &P24680_323};	/*Property = Svg*/
const Value V24680_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_327 = {S44, 0, &V24680_327_57316, &P24680_324};	/*Property = TouchEvents*/
const Value V24680_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_328 = {S45, 0, &V24680_328_57319, &P24680_327};	/*Property = Track*/
const Value V24680_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_329 = {S46, 0, &V24680_329_57325, &P24680_328};	/*Property = Video*/
const Value V24680_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_330 = {S47, 0, &V24680_330_57328, &P24680_329};	/*Property = Viewport*/
const Value V24680_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_332 = {S48, 0, &V24680_332_57340, &P24680_330};	/*Property = WebWorkers*/
const Value V24680_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24680_333 = {S49, 0, &V24680_333_57343, &P24680_332};	/*Property = Xhr2*/
const Profile P24680 = {"24680", &P24680_333};	/*Profile ID - 24680*/
const Value V24641_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_279 = {S1, 0, &V24641_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24641_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_282 = {S3, 0, &V24641_282_57155, &P24641_279};	/*Property = BlobBuilder*/
const Value V24641_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_350 = {S4, 0, &V24641_350_57446, &P24641_282};	/*Property = Canvas*/
const Value V24641_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_283 = {S5, 0, &V24641_283_57161, &P24641_350};	/*Property = CssBackground*/
const Value V24641_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_284 = {S6, 0, &V24641_284_57164, &P24641_283};	/*Property = CssBorderImage*/
const Value V24641_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_285 = {S7, 0, &V24641_285_57167, &P24641_284};	/*Property = CssCanvas*/
const Value V24641_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_286 = {S8, 0, &V24641_286_57170, &P24641_285};	/*Property = CssColor*/
const Value V24641_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_237 = {S9, 0, &V24641_237_55786, &P24641_286};	/*Property = CssColumn*/
const Value V24641_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_287 = {S10, 0, &V24641_287_57173, &P24641_237};	/*Property = CssFlexbox*/
const Value V24641_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_288 = {S11, 0, &V24641_288_57176, &P24641_287};	/*Property = CssFont*/
const Value V24641_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_289 = {S12, 0, &V24641_289_86899, &P24641_288};	/*Property = CssImages*/
const Value V24641_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_291 = {S13, 0, &V24641_291_57182, &P24641_289};	/*Property = CssMediaQueries*/
const Value V24641_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_292 = {S14, 0, &V24641_292_57185, &P24641_291};	/*Property = CssMinMax*/
const Value V24641_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_293 = {S15, 0, &V24641_293_57188, &P24641_292};	/*Property = CssOverflow*/
const Value V24641_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_294 = {S16, 0, &V24641_294_57191, &P24641_293};	/*Property = CssPosition*/
const Value V24641_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_295 = {S17, 0, &V24641_295_57194, &P24641_294};	/*Property = CssText*/
const Value V24641_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_297 = {S18, 0, &V24641_297_57197, &P24641_295};	/*Property = CssTransforms*/
const Value V24641_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_241 = {S19, 0, &V24641_241_55801, &P24641_297};	/*Property = CssTransitions*/
const Value V24641_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_298 = {S20, 0, &V24641_298_57208, &P24641_241};	/*Property = CssUI*/
const Value V24641_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_300 = {S21, 0, &V24641_300_57217, &P24641_298};	/*Property = DataSet*/
const Value V24641_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_301 = {S22, 0, &V24641_301_57220, &P24641_300};	/*Property = DataUrl*/
const Value V24641_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_302 = {S23, 0, &V24641_302_57223, &P24641_301};	/*Property = DeviceOrientation*/
const Value V24641_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_304 = {S24, 0, &V24641_304_57229, &P24641_302};	/*Property = FileReader*/
const Value V24641_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_305 = {S25, 0, &V24641_305_57232, &P24641_304};	/*Property = FileSaver*/
const Value V24641_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_306 = {S26, 0, &V24641_306_57238, &P24641_305};	/*Property = FileWriter*/
const Value V24641_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_307 = {S27, 0, &V24641_307_57241, &P24641_306};	/*Property = FormData*/
const Value V24641_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_309 = {S28, 0, &V24641_309_57247, &P24641_307};	/*Property = Fullscreen*/
const Value V24641_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_310 = {S29, 0, &V24641_310_57250, &P24641_309};	/*Property = GeoLocation*/
const Value V24641_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_312 = {S30, 0, &V24641_312_57256, &P24641_310};	/*Property = History*/
const Value V24641_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_314 = {S31, 0, &V24641_314_57262, &P24641_312};	/*Property = Html5*/
const Value V24641_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_313 = {S32, 0, &V24641_313_57259, &P24641_314};	/*Property = Html-Media-Capture*/
const Value V24641_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_315 = {S33, 0, &V24641_315_57265, &P24641_313};	/*Property = Iframe*/
const Value V24641_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_316 = {S34, 0, &V24641_316_57268, &P24641_315};	/*Property = IndexedDB*/
const Value V24641_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_317 = {S35, 0, &V24641_317_57271, &P24641_316};	/*Property = Json*/
const Value V24641_15_90837 = {S101, NULL};	/*Value = U2*/
const Property P24641_15 = {S37, 0, &V24641_15_90837, &P24641_317};	/*Property = LayoutEngine*/
const Value V24641_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_344 = {S38, 0, &V24641_344_57274, &P24641_15};	/*Property = Masking*/
const Value V24641_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_320 = {S39, 0, &V24641_320_57292, &P24641_344};	/*Property = PostMessage*/
const Value V24641_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_321 = {S40, 0, &V24641_321_57295, &P24641_320};	/*Property = Progress*/
const Value V24641_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_322 = {S41, 0, &V24641_322_57298, &P24641_321};	/*Property = Prompts*/
const Value V24641_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_323 = {S42, 0, &V24641_323_57301, &P24641_322};	/*Property = Selector*/
const Value V24641_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_324 = {S43, 0, &V24641_324_57307, &P24641_323};	/*Property = Svg*/
const Value V24641_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_327 = {S44, 0, &V24641_327_57316, &P24641_324};	/*Property = TouchEvents*/
const Value V24641_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_328 = {S45, 0, &V24641_328_57319, &P24641_327};	/*Property = Track*/
const Value V24641_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_329 = {S46, 0, &V24641_329_57325, &P24641_328};	/*Property = Video*/
const Value V24641_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_330 = {S47, 0, &V24641_330_57328, &P24641_329};	/*Property = Viewport*/
const Value V24641_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_332 = {S48, 0, &V24641_332_57340, &P24641_330};	/*Property = WebWorkers*/
const Value V24641_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24641_333 = {S49, 0, &V24641_333_57343, &P24641_332};	/*Property = Xhr2*/
const Profile P24641 = {"24641", &P24641_333};	/*Profile ID - 24641*/
const Value V24470_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_279 = {S1, 0, &V24470_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24470_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_282 = {S3, 0, &V24470_282_57155, &P24470_279};	/*Property = BlobBuilder*/
const Value V24470_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_350 = {S4, 0, &V24470_350_57446, &P24470_282};	/*Property = Canvas*/
const Value V24470_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_283 = {S5, 0, &V24470_283_57161, &P24470_350};	/*Property = CssBackground*/
const Value V24470_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_284 = {S6, 0, &V24470_284_57164, &P24470_283};	/*Property = CssBorderImage*/
const Value V24470_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_285 = {S7, 0, &V24470_285_57167, &P24470_284};	/*Property = CssCanvas*/
const Value V24470_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_286 = {S8, 0, &V24470_286_57170, &P24470_285};	/*Property = CssColor*/
const Value V24470_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_237 = {S9, 0, &V24470_237_55786, &P24470_286};	/*Property = CssColumn*/
const Value V24470_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_287 = {S10, 0, &V24470_287_57173, &P24470_237};	/*Property = CssFlexbox*/
const Value V24470_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_288 = {S11, 0, &V24470_288_57176, &P24470_287};	/*Property = CssFont*/
const Value V24470_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_289 = {S12, 0, &V24470_289_86899, &P24470_288};	/*Property = CssImages*/
const Value V24470_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_291 = {S13, 0, &V24470_291_57182, &P24470_289};	/*Property = CssMediaQueries*/
const Value V24470_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_292 = {S14, 0, &V24470_292_57185, &P24470_291};	/*Property = CssMinMax*/
const Value V24470_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_293 = {S15, 0, &V24470_293_57188, &P24470_292};	/*Property = CssOverflow*/
const Value V24470_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_294 = {S16, 0, &V24470_294_57191, &P24470_293};	/*Property = CssPosition*/
const Value V24470_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_295 = {S17, 0, &V24470_295_57194, &P24470_294};	/*Property = CssText*/
const Value V24470_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_297 = {S18, 0, &V24470_297_57197, &P24470_295};	/*Property = CssTransforms*/
const Value V24470_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_241 = {S19, 0, &V24470_241_55801, &P24470_297};	/*Property = CssTransitions*/
const Value V24470_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_298 = {S20, 0, &V24470_298_57208, &P24470_241};	/*Property = CssUI*/
const Value V24470_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_300 = {S21, 0, &V24470_300_57217, &P24470_298};	/*Property = DataSet*/
const Value V24470_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_301 = {S22, 0, &V24470_301_57220, &P24470_300};	/*Property = DataUrl*/
const Value V24470_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_302 = {S23, 0, &V24470_302_57223, &P24470_301};	/*Property = DeviceOrientation*/
const Value V24470_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_304 = {S24, 0, &V24470_304_57229, &P24470_302};	/*Property = FileReader*/
const Value V24470_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_305 = {S25, 0, &V24470_305_57232, &P24470_304};	/*Property = FileSaver*/
const Value V24470_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_306 = {S26, 0, &V24470_306_57238, &P24470_305};	/*Property = FileWriter*/
const Value V24470_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_307 = {S27, 0, &V24470_307_57241, &P24470_306};	/*Property = FormData*/
const Value V24470_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_309 = {S28, 0, &V24470_309_57247, &P24470_307};	/*Property = Fullscreen*/
const Value V24470_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_310 = {S29, 0, &V24470_310_57250, &P24470_309};	/*Property = GeoLocation*/
const Value V24470_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_312 = {S30, 0, &V24470_312_57256, &P24470_310};	/*Property = History*/
const Value V24470_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_314 = {S31, 0, &V24470_314_57262, &P24470_312};	/*Property = Html5*/
const Value V24470_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_313 = {S32, 0, &V24470_313_57259, &P24470_314};	/*Property = Html-Media-Capture*/
const Value V24470_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_315 = {S33, 0, &V24470_315_57265, &P24470_313};	/*Property = Iframe*/
const Value V24470_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_316 = {S34, 0, &V24470_316_57268, &P24470_315};	/*Property = IndexedDB*/
const Value V24470_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_317 = {S35, 0, &V24470_317_57271, &P24470_316};	/*Property = Json*/
const Value V24470_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P24470_15 = {S37, 0, &V24470_15_29, &P24470_317};	/*Property = LayoutEngine*/
const Value V24470_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_344 = {S38, 0, &V24470_344_57274, &P24470_15};	/*Property = Masking*/
const Value V24470_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_320 = {S39, 0, &V24470_320_57292, &P24470_344};	/*Property = PostMessage*/
const Value V24470_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_321 = {S40, 0, &V24470_321_57295, &P24470_320};	/*Property = Progress*/
const Value V24470_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_322 = {S41, 0, &V24470_322_57298, &P24470_321};	/*Property = Prompts*/
const Value V24470_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_323 = {S42, 0, &V24470_323_57301, &P24470_322};	/*Property = Selector*/
const Value V24470_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_324 = {S43, 0, &V24470_324_57307, &P24470_323};	/*Property = Svg*/
const Value V24470_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_327 = {S44, 0, &V24470_327_57316, &P24470_324};	/*Property = TouchEvents*/
const Value V24470_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_328 = {S45, 0, &V24470_328_57319, &P24470_327};	/*Property = Track*/
const Value V24470_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_329 = {S46, 0, &V24470_329_57325, &P24470_328};	/*Property = Video*/
const Value V24470_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_330 = {S47, 0, &V24470_330_57328, &P24470_329};	/*Property = Viewport*/
const Value V24470_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_332 = {S48, 0, &V24470_332_57340, &P24470_330};	/*Property = WebWorkers*/
const Value V24470_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24470_333 = {S49, 0, &V24470_333_57343, &P24470_332};	/*Property = Xhr2*/
const Profile P24470 = {"24470", &P24470_333};	/*Profile ID - 24470*/
const Value V24495_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_279 = {S1, 0, &V24495_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24495_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_282 = {S3, 0, &V24495_282_57155, &P24495_279};	/*Property = BlobBuilder*/
const Value V24495_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_350 = {S4, 0, &V24495_350_57446, &P24495_282};	/*Property = Canvas*/
const Value V24495_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_283 = {S5, 0, &V24495_283_57161, &P24495_350};	/*Property = CssBackground*/
const Value V24495_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_284 = {S6, 0, &V24495_284_57164, &P24495_283};	/*Property = CssBorderImage*/
const Value V24495_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_285 = {S7, 0, &V24495_285_57167, &P24495_284};	/*Property = CssCanvas*/
const Value V24495_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_286 = {S8, 0, &V24495_286_57170, &P24495_285};	/*Property = CssColor*/
const Value V24495_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_237 = {S9, 0, &V24495_237_55786, &P24495_286};	/*Property = CssColumn*/
const Value V24495_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_287 = {S10, 0, &V24495_287_57173, &P24495_237};	/*Property = CssFlexbox*/
const Value V24495_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_288 = {S11, 0, &V24495_288_57176, &P24495_287};	/*Property = CssFont*/
const Value V24495_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_289 = {S12, 0, &V24495_289_86899, &P24495_288};	/*Property = CssImages*/
const Value V24495_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_291 = {S13, 0, &V24495_291_57182, &P24495_289};	/*Property = CssMediaQueries*/
const Value V24495_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_292 = {S14, 0, &V24495_292_57185, &P24495_291};	/*Property = CssMinMax*/
const Value V24495_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_293 = {S15, 0, &V24495_293_57188, &P24495_292};	/*Property = CssOverflow*/
const Value V24495_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_294 = {S16, 0, &V24495_294_57191, &P24495_293};	/*Property = CssPosition*/
const Value V24495_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_295 = {S17, 0, &V24495_295_57194, &P24495_294};	/*Property = CssText*/
const Value V24495_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_297 = {S18, 0, &V24495_297_57197, &P24495_295};	/*Property = CssTransforms*/
const Value V24495_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_241 = {S19, 0, &V24495_241_55801, &P24495_297};	/*Property = CssTransitions*/
const Value V24495_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_298 = {S20, 0, &V24495_298_57208, &P24495_241};	/*Property = CssUI*/
const Value V24495_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_300 = {S21, 0, &V24495_300_57217, &P24495_298};	/*Property = DataSet*/
const Value V24495_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_301 = {S22, 0, &V24495_301_57220, &P24495_300};	/*Property = DataUrl*/
const Value V24495_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_302 = {S23, 0, &V24495_302_57223, &P24495_301};	/*Property = DeviceOrientation*/
const Value V24495_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_304 = {S24, 0, &V24495_304_57229, &P24495_302};	/*Property = FileReader*/
const Value V24495_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_305 = {S25, 0, &V24495_305_57232, &P24495_304};	/*Property = FileSaver*/
const Value V24495_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_306 = {S26, 0, &V24495_306_57238, &P24495_305};	/*Property = FileWriter*/
const Value V24495_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_307 = {S27, 0, &V24495_307_57241, &P24495_306};	/*Property = FormData*/
const Value V24495_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_309 = {S28, 0, &V24495_309_57247, &P24495_307};	/*Property = Fullscreen*/
const Value V24495_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_310 = {S29, 0, &V24495_310_57250, &P24495_309};	/*Property = GeoLocation*/
const Value V24495_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_312 = {S30, 0, &V24495_312_57256, &P24495_310};	/*Property = History*/
const Value V24495_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_314 = {S31, 0, &V24495_314_57262, &P24495_312};	/*Property = Html5*/
const Value V24495_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_313 = {S32, 0, &V24495_313_57259, &P24495_314};	/*Property = Html-Media-Capture*/
const Value V24495_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_315 = {S33, 0, &V24495_315_57265, &P24495_313};	/*Property = Iframe*/
const Value V24495_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_316 = {S34, 0, &V24495_316_57268, &P24495_315};	/*Property = IndexedDB*/
const Value V24495_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_317 = {S35, 0, &V24495_317_57271, &P24495_316};	/*Property = Json*/
const Value V24495_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24495_15 = {S37, 0, &V24495_15_28, &P24495_317};	/*Property = LayoutEngine*/
const Value V24495_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_344 = {S38, 0, &V24495_344_57274, &P24495_15};	/*Property = Masking*/
const Value V24495_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_320 = {S39, 0, &V24495_320_57292, &P24495_344};	/*Property = PostMessage*/
const Value V24495_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_321 = {S40, 0, &V24495_321_57295, &P24495_320};	/*Property = Progress*/
const Value V24495_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_322 = {S41, 0, &V24495_322_57298, &P24495_321};	/*Property = Prompts*/
const Value V24495_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_323 = {S42, 0, &V24495_323_57301, &P24495_322};	/*Property = Selector*/
const Value V24495_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_324 = {S43, 0, &V24495_324_57307, &P24495_323};	/*Property = Svg*/
const Value V24495_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_327 = {S44, 0, &V24495_327_57316, &P24495_324};	/*Property = TouchEvents*/
const Value V24495_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_328 = {S45, 0, &V24495_328_57319, &P24495_327};	/*Property = Track*/
const Value V24495_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_329 = {S46, 0, &V24495_329_57325, &P24495_328};	/*Property = Video*/
const Value V24495_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_330 = {S47, 0, &V24495_330_57328, &P24495_329};	/*Property = Viewport*/
const Value V24495_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_332 = {S48, 0, &V24495_332_57340, &P24495_330};	/*Property = WebWorkers*/
const Value V24495_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24495_333 = {S49, 0, &V24495_333_57343, &P24495_332};	/*Property = Xhr2*/
const Profile P24495 = {"24495", &P24495_333};	/*Profile ID - 24495*/
const Value V24496_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_279 = {S1, 0, &V24496_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24496_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_282 = {S3, 0, &V24496_282_57155, &P24496_279};	/*Property = BlobBuilder*/
const Value V24496_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_350 = {S4, 0, &V24496_350_57446, &P24496_282};	/*Property = Canvas*/
const Value V24496_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_283 = {S5, 0, &V24496_283_57161, &P24496_350};	/*Property = CssBackground*/
const Value V24496_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_284 = {S6, 0, &V24496_284_57164, &P24496_283};	/*Property = CssBorderImage*/
const Value V24496_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_285 = {S7, 0, &V24496_285_57167, &P24496_284};	/*Property = CssCanvas*/
const Value V24496_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_286 = {S8, 0, &V24496_286_57170, &P24496_285};	/*Property = CssColor*/
const Value V24496_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_237 = {S9, 0, &V24496_237_55786, &P24496_286};	/*Property = CssColumn*/
const Value V24496_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_287 = {S10, 0, &V24496_287_57173, &P24496_237};	/*Property = CssFlexbox*/
const Value V24496_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_288 = {S11, 0, &V24496_288_57176, &P24496_287};	/*Property = CssFont*/
const Value V24496_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_289 = {S12, 0, &V24496_289_86899, &P24496_288};	/*Property = CssImages*/
const Value V24496_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_291 = {S13, 0, &V24496_291_57182, &P24496_289};	/*Property = CssMediaQueries*/
const Value V24496_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_292 = {S14, 0, &V24496_292_57185, &P24496_291};	/*Property = CssMinMax*/
const Value V24496_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_293 = {S15, 0, &V24496_293_57188, &P24496_292};	/*Property = CssOverflow*/
const Value V24496_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_294 = {S16, 0, &V24496_294_57191, &P24496_293};	/*Property = CssPosition*/
const Value V24496_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_295 = {S17, 0, &V24496_295_57194, &P24496_294};	/*Property = CssText*/
const Value V24496_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_297 = {S18, 0, &V24496_297_57197, &P24496_295};	/*Property = CssTransforms*/
const Value V24496_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_241 = {S19, 0, &V24496_241_55801, &P24496_297};	/*Property = CssTransitions*/
const Value V24496_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_298 = {S20, 0, &V24496_298_57208, &P24496_241};	/*Property = CssUI*/
const Value V24496_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_300 = {S21, 0, &V24496_300_57217, &P24496_298};	/*Property = DataSet*/
const Value V24496_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_301 = {S22, 0, &V24496_301_57220, &P24496_300};	/*Property = DataUrl*/
const Value V24496_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_302 = {S23, 0, &V24496_302_57223, &P24496_301};	/*Property = DeviceOrientation*/
const Value V24496_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_304 = {S24, 0, &V24496_304_57229, &P24496_302};	/*Property = FileReader*/
const Value V24496_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_305 = {S25, 0, &V24496_305_57232, &P24496_304};	/*Property = FileSaver*/
const Value V24496_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_306 = {S26, 0, &V24496_306_57238, &P24496_305};	/*Property = FileWriter*/
const Value V24496_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_307 = {S27, 0, &V24496_307_57241, &P24496_306};	/*Property = FormData*/
const Value V24496_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_309 = {S28, 0, &V24496_309_57247, &P24496_307};	/*Property = Fullscreen*/
const Value V24496_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_310 = {S29, 0, &V24496_310_57250, &P24496_309};	/*Property = GeoLocation*/
const Value V24496_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_312 = {S30, 0, &V24496_312_57256, &P24496_310};	/*Property = History*/
const Value V24496_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_314 = {S31, 0, &V24496_314_57262, &P24496_312};	/*Property = Html5*/
const Value V24496_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_313 = {S32, 0, &V24496_313_57259, &P24496_314};	/*Property = Html-Media-Capture*/
const Value V24496_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_315 = {S33, 0, &V24496_315_57265, &P24496_313};	/*Property = Iframe*/
const Value V24496_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_316 = {S34, 0, &V24496_316_57268, &P24496_315};	/*Property = IndexedDB*/
const Value V24496_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_317 = {S35, 0, &V24496_317_57271, &P24496_316};	/*Property = Json*/
const Value V24496_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24496_15 = {S37, 0, &V24496_15_28, &P24496_317};	/*Property = LayoutEngine*/
const Value V24496_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_344 = {S38, 0, &V24496_344_57274, &P24496_15};	/*Property = Masking*/
const Value V24496_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_320 = {S39, 0, &V24496_320_57292, &P24496_344};	/*Property = PostMessage*/
const Value V24496_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_321 = {S40, 0, &V24496_321_57295, &P24496_320};	/*Property = Progress*/
const Value V24496_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_322 = {S41, 0, &V24496_322_57298, &P24496_321};	/*Property = Prompts*/
const Value V24496_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_323 = {S42, 0, &V24496_323_57301, &P24496_322};	/*Property = Selector*/
const Value V24496_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_324 = {S43, 0, &V24496_324_57307, &P24496_323};	/*Property = Svg*/
const Value V24496_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_327 = {S44, 0, &V24496_327_57316, &P24496_324};	/*Property = TouchEvents*/
const Value V24496_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_328 = {S45, 0, &V24496_328_57319, &P24496_327};	/*Property = Track*/
const Value V24496_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_329 = {S46, 0, &V24496_329_57325, &P24496_328};	/*Property = Video*/
const Value V24496_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_330 = {S47, 0, &V24496_330_57328, &P24496_329};	/*Property = Viewport*/
const Value V24496_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_332 = {S48, 0, &V24496_332_57340, &P24496_330};	/*Property = WebWorkers*/
const Value V24496_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24496_333 = {S49, 0, &V24496_333_57343, &P24496_332};	/*Property = Xhr2*/
const Profile P24496 = {"24496", &P24496_333};	/*Profile ID - 24496*/
const Value V24498_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_279 = {S1, 0, &V24498_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24498_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_282 = {S3, 0, &V24498_282_57155, &P24498_279};	/*Property = BlobBuilder*/
const Value V24498_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_350 = {S4, 0, &V24498_350_57446, &P24498_282};	/*Property = Canvas*/
const Value V24498_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_283 = {S5, 0, &V24498_283_57161, &P24498_350};	/*Property = CssBackground*/
const Value V24498_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_284 = {S6, 0, &V24498_284_57164, &P24498_283};	/*Property = CssBorderImage*/
const Value V24498_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_285 = {S7, 0, &V24498_285_57167, &P24498_284};	/*Property = CssCanvas*/
const Value V24498_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_286 = {S8, 0, &V24498_286_57170, &P24498_285};	/*Property = CssColor*/
const Value V24498_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_237 = {S9, 0, &V24498_237_55786, &P24498_286};	/*Property = CssColumn*/
const Value V24498_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_287 = {S10, 0, &V24498_287_57173, &P24498_237};	/*Property = CssFlexbox*/
const Value V24498_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_288 = {S11, 0, &V24498_288_57176, &P24498_287};	/*Property = CssFont*/
const Value V24498_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_289 = {S12, 0, &V24498_289_86899, &P24498_288};	/*Property = CssImages*/
const Value V24498_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_291 = {S13, 0, &V24498_291_57182, &P24498_289};	/*Property = CssMediaQueries*/
const Value V24498_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_292 = {S14, 0, &V24498_292_57185, &P24498_291};	/*Property = CssMinMax*/
const Value V24498_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_293 = {S15, 0, &V24498_293_57188, &P24498_292};	/*Property = CssOverflow*/
const Value V24498_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_294 = {S16, 0, &V24498_294_57191, &P24498_293};	/*Property = CssPosition*/
const Value V24498_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_295 = {S17, 0, &V24498_295_57194, &P24498_294};	/*Property = CssText*/
const Value V24498_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_297 = {S18, 0, &V24498_297_57197, &P24498_295};	/*Property = CssTransforms*/
const Value V24498_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_241 = {S19, 0, &V24498_241_55801, &P24498_297};	/*Property = CssTransitions*/
const Value V24498_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_298 = {S20, 0, &V24498_298_57208, &P24498_241};	/*Property = CssUI*/
const Value V24498_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_300 = {S21, 0, &V24498_300_57217, &P24498_298};	/*Property = DataSet*/
const Value V24498_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_301 = {S22, 0, &V24498_301_57220, &P24498_300};	/*Property = DataUrl*/
const Value V24498_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_302 = {S23, 0, &V24498_302_57223, &P24498_301};	/*Property = DeviceOrientation*/
const Value V24498_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_304 = {S24, 0, &V24498_304_57229, &P24498_302};	/*Property = FileReader*/
const Value V24498_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_305 = {S25, 0, &V24498_305_57232, &P24498_304};	/*Property = FileSaver*/
const Value V24498_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_306 = {S26, 0, &V24498_306_57238, &P24498_305};	/*Property = FileWriter*/
const Value V24498_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_307 = {S27, 0, &V24498_307_57241, &P24498_306};	/*Property = FormData*/
const Value V24498_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_309 = {S28, 0, &V24498_309_57247, &P24498_307};	/*Property = Fullscreen*/
const Value V24498_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_310 = {S29, 0, &V24498_310_57250, &P24498_309};	/*Property = GeoLocation*/
const Value V24498_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_312 = {S30, 0, &V24498_312_57256, &P24498_310};	/*Property = History*/
const Value V24498_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_314 = {S31, 0, &V24498_314_57262, &P24498_312};	/*Property = Html5*/
const Value V24498_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_313 = {S32, 0, &V24498_313_57259, &P24498_314};	/*Property = Html-Media-Capture*/
const Value V24498_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_315 = {S33, 0, &V24498_315_57265, &P24498_313};	/*Property = Iframe*/
const Value V24498_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_316 = {S34, 0, &V24498_316_57268, &P24498_315};	/*Property = IndexedDB*/
const Value V24498_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_317 = {S35, 0, &V24498_317_57271, &P24498_316};	/*Property = Json*/
const Value V24498_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P24498_15 = {S37, 0, &V24498_15_31, &P24498_317};	/*Property = LayoutEngine*/
const Value V24498_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_344 = {S38, 0, &V24498_344_57274, &P24498_15};	/*Property = Masking*/
const Value V24498_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_320 = {S39, 0, &V24498_320_57292, &P24498_344};	/*Property = PostMessage*/
const Value V24498_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_321 = {S40, 0, &V24498_321_57295, &P24498_320};	/*Property = Progress*/
const Value V24498_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_322 = {S41, 0, &V24498_322_57298, &P24498_321};	/*Property = Prompts*/
const Value V24498_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_323 = {S42, 0, &V24498_323_57301, &P24498_322};	/*Property = Selector*/
const Value V24498_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_324 = {S43, 0, &V24498_324_57307, &P24498_323};	/*Property = Svg*/
const Value V24498_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_327 = {S44, 0, &V24498_327_57316, &P24498_324};	/*Property = TouchEvents*/
const Value V24498_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_328 = {S45, 0, &V24498_328_57319, &P24498_327};	/*Property = Track*/
const Value V24498_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_329 = {S46, 0, &V24498_329_57325, &P24498_328};	/*Property = Video*/
const Value V24498_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_330 = {S47, 0, &V24498_330_57328, &P24498_329};	/*Property = Viewport*/
const Value V24498_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_332 = {S48, 0, &V24498_332_57340, &P24498_330};	/*Property = WebWorkers*/
const Value V24498_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24498_333 = {S49, 0, &V24498_333_57343, &P24498_332};	/*Property = Xhr2*/
const Profile P24498 = {"24498", &P24498_333};	/*Profile ID - 24498*/
const Value V24500_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_279 = {S1, 0, &V24500_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24500_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_282 = {S3, 0, &V24500_282_57155, &P24500_279};	/*Property = BlobBuilder*/
const Value V24500_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_350 = {S4, 0, &V24500_350_57446, &P24500_282};	/*Property = Canvas*/
const Value V24500_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_283 = {S5, 0, &V24500_283_57161, &P24500_350};	/*Property = CssBackground*/
const Value V24500_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_284 = {S6, 0, &V24500_284_57164, &P24500_283};	/*Property = CssBorderImage*/
const Value V24500_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_285 = {S7, 0, &V24500_285_57167, &P24500_284};	/*Property = CssCanvas*/
const Value V24500_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_286 = {S8, 0, &V24500_286_57170, &P24500_285};	/*Property = CssColor*/
const Value V24500_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_237 = {S9, 0, &V24500_237_55786, &P24500_286};	/*Property = CssColumn*/
const Value V24500_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_287 = {S10, 0, &V24500_287_57173, &P24500_237};	/*Property = CssFlexbox*/
const Value V24500_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_288 = {S11, 0, &V24500_288_57176, &P24500_287};	/*Property = CssFont*/
const Value V24500_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_289 = {S12, 0, &V24500_289_86899, &P24500_288};	/*Property = CssImages*/
const Value V24500_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_291 = {S13, 0, &V24500_291_57182, &P24500_289};	/*Property = CssMediaQueries*/
const Value V24500_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_292 = {S14, 0, &V24500_292_57185, &P24500_291};	/*Property = CssMinMax*/
const Value V24500_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_293 = {S15, 0, &V24500_293_57188, &P24500_292};	/*Property = CssOverflow*/
const Value V24500_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_294 = {S16, 0, &V24500_294_57191, &P24500_293};	/*Property = CssPosition*/
const Value V24500_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_295 = {S17, 0, &V24500_295_57194, &P24500_294};	/*Property = CssText*/
const Value V24500_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_297 = {S18, 0, &V24500_297_57197, &P24500_295};	/*Property = CssTransforms*/
const Value V24500_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_241 = {S19, 0, &V24500_241_55801, &P24500_297};	/*Property = CssTransitions*/
const Value V24500_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_298 = {S20, 0, &V24500_298_57208, &P24500_241};	/*Property = CssUI*/
const Value V24500_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_300 = {S21, 0, &V24500_300_57217, &P24500_298};	/*Property = DataSet*/
const Value V24500_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_301 = {S22, 0, &V24500_301_57220, &P24500_300};	/*Property = DataUrl*/
const Value V24500_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_302 = {S23, 0, &V24500_302_57223, &P24500_301};	/*Property = DeviceOrientation*/
const Value V24500_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_304 = {S24, 0, &V24500_304_57229, &P24500_302};	/*Property = FileReader*/
const Value V24500_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_305 = {S25, 0, &V24500_305_57232, &P24500_304};	/*Property = FileSaver*/
const Value V24500_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_306 = {S26, 0, &V24500_306_57238, &P24500_305};	/*Property = FileWriter*/
const Value V24500_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_307 = {S27, 0, &V24500_307_57241, &P24500_306};	/*Property = FormData*/
const Value V24500_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_309 = {S28, 0, &V24500_309_57247, &P24500_307};	/*Property = Fullscreen*/
const Value V24500_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_310 = {S29, 0, &V24500_310_57250, &P24500_309};	/*Property = GeoLocation*/
const Value V24500_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_312 = {S30, 0, &V24500_312_57256, &P24500_310};	/*Property = History*/
const Value V24500_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_314 = {S31, 0, &V24500_314_57262, &P24500_312};	/*Property = Html5*/
const Value V24500_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_313 = {S32, 0, &V24500_313_57259, &P24500_314};	/*Property = Html-Media-Capture*/
const Value V24500_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_315 = {S33, 0, &V24500_315_57265, &P24500_313};	/*Property = Iframe*/
const Value V24500_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_316 = {S34, 0, &V24500_316_57268, &P24500_315};	/*Property = IndexedDB*/
const Value V24500_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_317 = {S35, 0, &V24500_317_57271, &P24500_316};	/*Property = Json*/
const Value V24500_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P24500_15 = {S37, 0, &V24500_15_31, &P24500_317};	/*Property = LayoutEngine*/
const Value V24500_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_344 = {S38, 0, &V24500_344_57274, &P24500_15};	/*Property = Masking*/
const Value V24500_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_320 = {S39, 0, &V24500_320_57292, &P24500_344};	/*Property = PostMessage*/
const Value V24500_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_321 = {S40, 0, &V24500_321_57295, &P24500_320};	/*Property = Progress*/
const Value V24500_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_322 = {S41, 0, &V24500_322_57298, &P24500_321};	/*Property = Prompts*/
const Value V24500_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_323 = {S42, 0, &V24500_323_57301, &P24500_322};	/*Property = Selector*/
const Value V24500_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_324 = {S43, 0, &V24500_324_57307, &P24500_323};	/*Property = Svg*/
const Value V24500_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_327 = {S44, 0, &V24500_327_57316, &P24500_324};	/*Property = TouchEvents*/
const Value V24500_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_328 = {S45, 0, &V24500_328_57319, &P24500_327};	/*Property = Track*/
const Value V24500_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_329 = {S46, 0, &V24500_329_57325, &P24500_328};	/*Property = Video*/
const Value V24500_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_330 = {S47, 0, &V24500_330_57328, &P24500_329};	/*Property = Viewport*/
const Value V24500_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_332 = {S48, 0, &V24500_332_57340, &P24500_330};	/*Property = WebWorkers*/
const Value V24500_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24500_333 = {S49, 0, &V24500_333_57343, &P24500_332};	/*Property = Xhr2*/
const Profile P24500 = {"24500", &P24500_333};	/*Profile ID - 24500*/
const Value V24501_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_279 = {S1, 0, &V24501_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24501_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_282 = {S3, 0, &V24501_282_57155, &P24501_279};	/*Property = BlobBuilder*/
const Value V24501_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_350 = {S4, 0, &V24501_350_57446, &P24501_282};	/*Property = Canvas*/
const Value V24501_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_283 = {S5, 0, &V24501_283_57161, &P24501_350};	/*Property = CssBackground*/
const Value V24501_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_284 = {S6, 0, &V24501_284_57164, &P24501_283};	/*Property = CssBorderImage*/
const Value V24501_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_285 = {S7, 0, &V24501_285_57167, &P24501_284};	/*Property = CssCanvas*/
const Value V24501_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_286 = {S8, 0, &V24501_286_57170, &P24501_285};	/*Property = CssColor*/
const Value V24501_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_237 = {S9, 0, &V24501_237_55786, &P24501_286};	/*Property = CssColumn*/
const Value V24501_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_287 = {S10, 0, &V24501_287_57173, &P24501_237};	/*Property = CssFlexbox*/
const Value V24501_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_288 = {S11, 0, &V24501_288_57176, &P24501_287};	/*Property = CssFont*/
const Value V24501_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_289 = {S12, 0, &V24501_289_86899, &P24501_288};	/*Property = CssImages*/
const Value V24501_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_291 = {S13, 0, &V24501_291_57182, &P24501_289};	/*Property = CssMediaQueries*/
const Value V24501_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_292 = {S14, 0, &V24501_292_57185, &P24501_291};	/*Property = CssMinMax*/
const Value V24501_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_293 = {S15, 0, &V24501_293_57188, &P24501_292};	/*Property = CssOverflow*/
const Value V24501_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_294 = {S16, 0, &V24501_294_57191, &P24501_293};	/*Property = CssPosition*/
const Value V24501_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_295 = {S17, 0, &V24501_295_57194, &P24501_294};	/*Property = CssText*/
const Value V24501_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_297 = {S18, 0, &V24501_297_57197, &P24501_295};	/*Property = CssTransforms*/
const Value V24501_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_241 = {S19, 0, &V24501_241_55801, &P24501_297};	/*Property = CssTransitions*/
const Value V24501_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_298 = {S20, 0, &V24501_298_57208, &P24501_241};	/*Property = CssUI*/
const Value V24501_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_300 = {S21, 0, &V24501_300_57217, &P24501_298};	/*Property = DataSet*/
const Value V24501_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_301 = {S22, 0, &V24501_301_57220, &P24501_300};	/*Property = DataUrl*/
const Value V24501_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_302 = {S23, 0, &V24501_302_57223, &P24501_301};	/*Property = DeviceOrientation*/
const Value V24501_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_304 = {S24, 0, &V24501_304_57229, &P24501_302};	/*Property = FileReader*/
const Value V24501_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_305 = {S25, 0, &V24501_305_57232, &P24501_304};	/*Property = FileSaver*/
const Value V24501_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_306 = {S26, 0, &V24501_306_57238, &P24501_305};	/*Property = FileWriter*/
const Value V24501_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_307 = {S27, 0, &V24501_307_57241, &P24501_306};	/*Property = FormData*/
const Value V24501_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_309 = {S28, 0, &V24501_309_57247, &P24501_307};	/*Property = Fullscreen*/
const Value V24501_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_310 = {S29, 0, &V24501_310_57250, &P24501_309};	/*Property = GeoLocation*/
const Value V24501_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_312 = {S30, 0, &V24501_312_57256, &P24501_310};	/*Property = History*/
const Value V24501_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_314 = {S31, 0, &V24501_314_57262, &P24501_312};	/*Property = Html5*/
const Value V24501_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_313 = {S32, 0, &V24501_313_57259, &P24501_314};	/*Property = Html-Media-Capture*/
const Value V24501_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_315 = {S33, 0, &V24501_315_57265, &P24501_313};	/*Property = Iframe*/
const Value V24501_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_316 = {S34, 0, &V24501_316_57268, &P24501_315};	/*Property = IndexedDB*/
const Value V24501_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_317 = {S35, 0, &V24501_317_57271, &P24501_316};	/*Property = Json*/
const Value V24501_15_31 = {S66, NULL};	/*Value = Presto*/
const Property P24501_15 = {S37, 0, &V24501_15_31, &P24501_317};	/*Property = LayoutEngine*/
const Value V24501_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_344 = {S38, 0, &V24501_344_57274, &P24501_15};	/*Property = Masking*/
const Value V24501_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_320 = {S39, 0, &V24501_320_57292, &P24501_344};	/*Property = PostMessage*/
const Value V24501_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_321 = {S40, 0, &V24501_321_57295, &P24501_320};	/*Property = Progress*/
const Value V24501_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_322 = {S41, 0, &V24501_322_57298, &P24501_321};	/*Property = Prompts*/
const Value V24501_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_323 = {S42, 0, &V24501_323_57301, &P24501_322};	/*Property = Selector*/
const Value V24501_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_324 = {S43, 0, &V24501_324_57307, &P24501_323};	/*Property = Svg*/
const Value V24501_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_327 = {S44, 0, &V24501_327_57316, &P24501_324};	/*Property = TouchEvents*/
const Value V24501_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_328 = {S45, 0, &V24501_328_57319, &P24501_327};	/*Property = Track*/
const Value V24501_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_329 = {S46, 0, &V24501_329_57325, &P24501_328};	/*Property = Video*/
const Value V24501_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_330 = {S47, 0, &V24501_330_57328, &P24501_329};	/*Property = Viewport*/
const Value V24501_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_332 = {S48, 0, &V24501_332_57340, &P24501_330};	/*Property = WebWorkers*/
const Value V24501_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24501_333 = {S49, 0, &V24501_333_57343, &P24501_332};	/*Property = Xhr2*/
const Profile P24501 = {"24501", &P24501_333};	/*Profile ID - 24501*/
const Value V24640_1_1 = {S2, NULL};	/*Value = True*/
const Property P24640_1 = {S50, 0, &V24640_1_1, NULL};	/*Property = IsMobile*/
const Value V24640_5_1227 = {S51, NULL};	/*Value = 800*/
const Property P24640_5 = {S52, 0, &V24640_5_1227, &P24640_1};	/*Property = ScreenPixelsHeight*/
const Value V24640_4_1229 = {S56, NULL};	/*Value = 480*/
const Property P24640_4 = {S54, 0, &V24640_4_1229, &P24640_5};	/*Property = ScreenPixelsWidth*/
const Profile P24640 = {"24640", &P24640_4};	/*Profile ID - 24640*/
const Value V24468_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24468_279 = {S1, 0, &V24468_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24468_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24468_282 = {S3, 0, &V24468_282_57154, &P24468_279};	/*Property = BlobBuilder*/
const Value V24468_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24468_350 = {S4, 0, &V24468_350_57444, &P24468_282};	/*Property = Canvas*/
const Value V24468_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24468_283 = {S5, 0, &V24468_283_57159, &P24468_350};	/*Property = CssBackground*/
const Value V24468_284_57163 = {S0, NULL};	/*Value = False*/
const Property P24468_284 = {S6, 0, &V24468_284_57163, &P24468_283};	/*Property = CssBorderImage*/
const Value V24468_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24468_285 = {S7, 0, &V24468_285_57166, &P24468_284};	/*Property = CssCanvas*/
const Value V24468_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24468_286 = {S8, 0, &V24468_286_57168, &P24468_285};	/*Property = CssColor*/
const Value V24468_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24468_237 = {S9, 0, &V24468_237_55786, &P24468_286};	/*Property = CssColumn*/
const Value V24468_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24468_287 = {S10, 0, &V24468_287_57172, &P24468_237};	/*Property = CssFlexbox*/
const Value V24468_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24468_288 = {S11, 0, &V24468_288_57175, &P24468_287};	/*Property = CssFont*/
const Value V24468_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24468_289 = {S12, 0, &V24468_289_57759, &P24468_288};	/*Property = CssImages*/
const Value V24468_291_57181 = {S0, NULL};	/*Value = False*/
const Property P24468_291 = {S13, 0, &V24468_291_57181, &P24468_289};	/*Property = CssMediaQueries*/
const Value V24468_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24468_292 = {S14, 0, &V24468_292_57183, &P24468_291};	/*Property = CssMinMax*/
const Value V24468_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24468_293 = {S15, 0, &V24468_293_57187, &P24468_292};	/*Property = CssOverflow*/
const Value V24468_294_57190 = {S0, NULL};	/*Value = False*/
const Property P24468_294 = {S16, 0, &V24468_294_57190, &P24468_293};	/*Property = CssPosition*/
const Value V24468_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24468_295 = {S17, 0, &V24468_295_57192, &P24468_294};	/*Property = CssText*/
const Value V24468_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24468_297 = {S18, 0, &V24468_297_57195, &P24468_295};	/*Property = CssTransforms*/
const Value V24468_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24468_241 = {S19, 0, &V24468_241_55799, &P24468_297};	/*Property = CssTransitions*/
const Value V24468_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24468_298 = {S20, 0, &V24468_298_57206, &P24468_241};	/*Property = CssUI*/
const Value V24468_300_57216 = {S0, NULL};	/*Value = False*/
const Property P24468_300 = {S21, 0, &V24468_300_57216, &P24468_298};	/*Property = DataSet*/
const Value V24468_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24468_301 = {S22, 0, &V24468_301_57218, &P24468_300};	/*Property = DataUrl*/
const Value V24468_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24468_302 = {S23, 0, &V24468_302_57222, &P24468_301};	/*Property = DeviceOrientation*/
const Value V24468_304_57228 = {S0, NULL};	/*Value = False*/
const Property P24468_304 = {S24, 0, &V24468_304_57228, &P24468_302};	/*Property = FileReader*/
const Value V24468_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24468_305 = {S25, 0, &V24468_305_57231, &P24468_304};	/*Property = FileSaver*/
const Value V24468_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24468_306 = {S26, 0, &V24468_306_57237, &P24468_305};	/*Property = FileWriter*/
const Value V24468_307_57240 = {S0, NULL};	/*Value = False*/
const Property P24468_307 = {S27, 0, &V24468_307_57240, &P24468_306};	/*Property = FormData*/
const Value V24468_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24468_309 = {S28, 0, &V24468_309_57246, &P24468_307};	/*Property = Fullscreen*/
const Value V24468_310_57249 = {S0, NULL};	/*Value = False*/
const Property P24468_310 = {S29, 0, &V24468_310_57249, &P24468_309};	/*Property = GeoLocation*/
const Value V24468_312_57255 = {S0, NULL};	/*Value = False*/
const Property P24468_312 = {S30, 0, &V24468_312_57255, &P24468_310};	/*Property = History*/
const Value V24468_314_57261 = {S0, NULL};	/*Value = False*/
const Property P24468_314 = {S31, 0, &V24468_314_57261, &P24468_312};	/*Property = Html5*/
const Value V24468_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24468_313 = {S32, 0, &V24468_313_57258, &P24468_314};	/*Property = Html-Media-Capture*/
const Value V24468_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24468_315 = {S33, 0, &V24468_315_57264, &P24468_313};	/*Property = Iframe*/
const Value V24468_316_57267 = {S0, NULL};	/*Value = False*/
const Property P24468_316 = {S34, 0, &V24468_316_57267, &P24468_315};	/*Property = IndexedDB*/
const Value V24468_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24468_317 = {S35, 0, &V24468_317_57269, &P24468_316};	/*Property = Json*/
const Value V24468_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24468_15 = {S37, 0, &V24468_15_54167, &P24468_317};	/*Property = LayoutEngine*/
const Value V24468_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24468_344 = {S38, 0, &V24468_344_57272, &P24468_15};	/*Property = Masking*/
const Value V24468_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24468_320 = {S39, 0, &V24468_320_57290, &P24468_344};	/*Property = PostMessage*/
const Value V24468_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24468_321 = {S40, 0, &V24468_321_57293, &P24468_320};	/*Property = Progress*/
const Value V24468_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24468_322 = {S41, 0, &V24468_322_57296, &P24468_321};	/*Property = Prompts*/
const Value V24468_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24468_323 = {S42, 0, &V24468_323_57299, &P24468_322};	/*Property = Selector*/
const Value V24468_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24468_324 = {S43, 0, &V24468_324_57306, &P24468_323};	/*Property = Svg*/
const Value V24468_327_57315 = {S0, NULL};	/*Value = False*/
const Property P24468_327 = {S44, 0, &V24468_327_57315, &P24468_324};	/*Property = TouchEvents*/
const Value V24468_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24468_328 = {S45, 0, &V24468_328_57318, &P24468_327};	/*Property = Track*/
const Value V24468_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24468_329 = {S46, 0, &V24468_329_57323, &P24468_328};	/*Property = Video*/
const Value V24468_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24468_330 = {S47, 0, &V24468_330_57326, &P24468_329};	/*Property = Viewport*/
const Value V24468_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24468_332 = {S48, 0, &V24468_332_57339, &P24468_330};	/*Property = WebWorkers*/
const Value V24468_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24468_333 = {S49, 0, &V24468_333_57342, &P24468_332};	/*Property = Xhr2*/
const Profile P24468 = {"24468", &P24468_333};	/*Profile ID - 24468*/
const Value V24521_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24521_279 = {S1, 0, &V24521_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24521_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24521_282 = {S3, 0, &V24521_282_57154, &P24521_279};	/*Property = BlobBuilder*/
const Value V24521_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24521_350 = {S4, 0, &V24521_350_57444, &P24521_282};	/*Property = Canvas*/
const Value V24521_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24521_283 = {S5, 0, &V24521_283_57159, &P24521_350};	/*Property = CssBackground*/
const Value V24521_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24521_284 = {S6, 0, &V24521_284_57162, &P24521_283};	/*Property = CssBorderImage*/
const Value V24521_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24521_285 = {S7, 0, &V24521_285_57166, &P24521_284};	/*Property = CssCanvas*/
const Value V24521_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24521_286 = {S8, 0, &V24521_286_57168, &P24521_285};	/*Property = CssColor*/
const Value V24521_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24521_237 = {S9, 0, &V24521_237_55786, &P24521_286};	/*Property = CssColumn*/
const Value V24521_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24521_287 = {S10, 0, &V24521_287_57172, &P24521_237};	/*Property = CssFlexbox*/
const Value V24521_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24521_288 = {S11, 0, &V24521_288_57174, &P24521_287};	/*Property = CssFont*/
const Value V24521_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24521_289 = {S12, 0, &V24521_289_57759, &P24521_288};	/*Property = CssImages*/
const Value V24521_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24521_291 = {S13, 0, &V24521_291_57180, &P24521_289};	/*Property = CssMediaQueries*/
const Value V24521_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24521_292 = {S14, 0, &V24521_292_57183, &P24521_291};	/*Property = CssMinMax*/
const Value V24521_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24521_293 = {S15, 0, &V24521_293_57187, &P24521_292};	/*Property = CssOverflow*/
const Value V24521_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24521_294 = {S16, 0, &V24521_294_57189, &P24521_293};	/*Property = CssPosition*/
const Value V24521_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24521_295 = {S17, 0, &V24521_295_57192, &P24521_294};	/*Property = CssText*/
const Value V24521_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24521_297 = {S18, 0, &V24521_297_57195, &P24521_295};	/*Property = CssTransforms*/
const Value V24521_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24521_241 = {S19, 0, &V24521_241_55799, &P24521_297};	/*Property = CssTransitions*/
const Value V24521_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24521_298 = {S20, 0, &V24521_298_57206, &P24521_241};	/*Property = CssUI*/
const Value V24521_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24521_300 = {S21, 0, &V24521_300_57215, &P24521_298};	/*Property = DataSet*/
const Value V24521_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24521_301 = {S22, 0, &V24521_301_57218, &P24521_300};	/*Property = DataUrl*/
const Value V24521_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24521_302 = {S23, 0, &V24521_302_57221, &P24521_301};	/*Property = DeviceOrientation*/
const Value V24521_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24521_304 = {S24, 0, &V24521_304_57227, &P24521_302};	/*Property = FileReader*/
const Value V24521_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24521_305 = {S25, 0, &V24521_305_57231, &P24521_304};	/*Property = FileSaver*/
const Value V24521_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24521_306 = {S26, 0, &V24521_306_57237, &P24521_305};	/*Property = FileWriter*/
const Value V24521_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24521_307 = {S27, 0, &V24521_307_57239, &P24521_306};	/*Property = FormData*/
const Value V24521_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24521_309 = {S28, 0, &V24521_309_57246, &P24521_307};	/*Property = Fullscreen*/
const Value V24521_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24521_310 = {S29, 0, &V24521_310_57248, &P24521_309};	/*Property = GeoLocation*/
const Value V24521_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24521_312 = {S30, 0, &V24521_312_57254, &P24521_310};	/*Property = History*/
const Value V24521_314_57261 = {S0, NULL};	/*Value = False*/
const Property P24521_314 = {S31, 0, &V24521_314_57261, &P24521_312};	/*Property = Html5*/
const Value V24521_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24521_313 = {S32, 0, &V24521_313_57258, &P24521_314};	/*Property = Html-Media-Capture*/
const Value V24521_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24521_315 = {S33, 0, &V24521_315_57264, &P24521_313};	/*Property = Iframe*/
const Value V24521_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24521_316 = {S34, 0, &V24521_316_57266, &P24521_315};	/*Property = IndexedDB*/
const Value V24521_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24521_317 = {S35, 0, &V24521_317_57269, &P24521_316};	/*Property = Json*/
const Value V24521_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P24521_15 = {S37, 0, &V24521_15_29, &P24521_317};	/*Property = LayoutEngine*/
const Value V24521_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24521_344 = {S38, 0, &V24521_344_57272, &P24521_15};	/*Property = Masking*/
const Value V24521_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24521_320 = {S39, 0, &V24521_320_57290, &P24521_344};	/*Property = PostMessage*/
const Value V24521_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24521_321 = {S40, 0, &V24521_321_57293, &P24521_320};	/*Property = Progress*/
const Value V24521_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24521_322 = {S41, 0, &V24521_322_57296, &P24521_321};	/*Property = Prompts*/
const Value V24521_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24521_323 = {S42, 0, &V24521_323_57299, &P24521_322};	/*Property = Selector*/
const Value V24521_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24521_324 = {S43, 0, &V24521_324_57305, &P24521_323};	/*Property = Svg*/
const Value V24521_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24521_327 = {S44, 0, &V24521_327_57314, &P24521_324};	/*Property = TouchEvents*/
const Value V24521_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24521_328 = {S45, 0, &V24521_328_57318, &P24521_327};	/*Property = Track*/
const Value V24521_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24521_329 = {S46, 0, &V24521_329_57323, &P24521_328};	/*Property = Video*/
const Value V24521_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24521_330 = {S47, 0, &V24521_330_57326, &P24521_329};	/*Property = Viewport*/
const Value V24521_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24521_332 = {S48, 0, &V24521_332_57338, &P24521_330};	/*Property = WebWorkers*/
const Value V24521_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24521_333 = {S49, 0, &V24521_333_57342, &P24521_332};	/*Property = Xhr2*/
const Profile P24521 = {"24521", &P24521_333};	/*Profile ID - 24521*/
const Value V24522_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24522_279 = {S1, 0, &V24522_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24522_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24522_282 = {S3, 0, &V24522_282_57154, &P24522_279};	/*Property = BlobBuilder*/
const Value V24522_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24522_350 = {S4, 0, &V24522_350_57444, &P24522_282};	/*Property = Canvas*/
const Value V24522_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24522_283 = {S5, 0, &V24522_283_57159, &P24522_350};	/*Property = CssBackground*/
const Value V24522_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24522_284 = {S6, 0, &V24522_284_57162, &P24522_283};	/*Property = CssBorderImage*/
const Value V24522_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24522_285 = {S7, 0, &V24522_285_57166, &P24522_284};	/*Property = CssCanvas*/
const Value V24522_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24522_286 = {S8, 0, &V24522_286_57168, &P24522_285};	/*Property = CssColor*/
const Value V24522_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24522_237 = {S9, 0, &V24522_237_55786, &P24522_286};	/*Property = CssColumn*/
const Value V24522_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24522_287 = {S10, 0, &V24522_287_57172, &P24522_237};	/*Property = CssFlexbox*/
const Value V24522_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24522_288 = {S11, 0, &V24522_288_57174, &P24522_287};	/*Property = CssFont*/
const Value V24522_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24522_289 = {S12, 0, &V24522_289_57759, &P24522_288};	/*Property = CssImages*/
const Value V24522_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24522_291 = {S13, 0, &V24522_291_57180, &P24522_289};	/*Property = CssMediaQueries*/
const Value V24522_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24522_292 = {S14, 0, &V24522_292_57183, &P24522_291};	/*Property = CssMinMax*/
const Value V24522_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24522_293 = {S15, 0, &V24522_293_57187, &P24522_292};	/*Property = CssOverflow*/
const Value V24522_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24522_294 = {S16, 0, &V24522_294_57189, &P24522_293};	/*Property = CssPosition*/
const Value V24522_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24522_295 = {S17, 0, &V24522_295_57192, &P24522_294};	/*Property = CssText*/
const Value V24522_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24522_297 = {S18, 0, &V24522_297_57195, &P24522_295};	/*Property = CssTransforms*/
const Value V24522_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24522_241 = {S19, 0, &V24522_241_55799, &P24522_297};	/*Property = CssTransitions*/
const Value V24522_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24522_298 = {S20, 0, &V24522_298_57206, &P24522_241};	/*Property = CssUI*/
const Value V24522_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24522_300 = {S21, 0, &V24522_300_57215, &P24522_298};	/*Property = DataSet*/
const Value V24522_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24522_301 = {S22, 0, &V24522_301_57218, &P24522_300};	/*Property = DataUrl*/
const Value V24522_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24522_302 = {S23, 0, &V24522_302_57221, &P24522_301};	/*Property = DeviceOrientation*/
const Value V24522_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24522_304 = {S24, 0, &V24522_304_57227, &P24522_302};	/*Property = FileReader*/
const Value V24522_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24522_305 = {S25, 0, &V24522_305_57231, &P24522_304};	/*Property = FileSaver*/
const Value V24522_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24522_306 = {S26, 0, &V24522_306_57237, &P24522_305};	/*Property = FileWriter*/
const Value V24522_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24522_307 = {S27, 0, &V24522_307_57239, &P24522_306};	/*Property = FormData*/
const Value V24522_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24522_309 = {S28, 0, &V24522_309_57246, &P24522_307};	/*Property = Fullscreen*/
const Value V24522_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24522_310 = {S29, 0, &V24522_310_57248, &P24522_309};	/*Property = GeoLocation*/
const Value V24522_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24522_312 = {S30, 0, &V24522_312_57254, &P24522_310};	/*Property = History*/
const Value V24522_314_57261 = {S0, NULL};	/*Value = False*/
const Property P24522_314 = {S31, 0, &V24522_314_57261, &P24522_312};	/*Property = Html5*/
const Value V24522_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24522_313 = {S32, 0, &V24522_313_57258, &P24522_314};	/*Property = Html-Media-Capture*/
const Value V24522_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24522_315 = {S33, 0, &V24522_315_57264, &P24522_313};	/*Property = Iframe*/
const Value V24522_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24522_316 = {S34, 0, &V24522_316_57266, &P24522_315};	/*Property = IndexedDB*/
const Value V24522_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24522_317 = {S35, 0, &V24522_317_57269, &P24522_316};	/*Property = Json*/
const Value V24522_15_29 = {S76, NULL};	/*Value = Gecko*/
const Property P24522_15 = {S37, 0, &V24522_15_29, &P24522_317};	/*Property = LayoutEngine*/
const Value V24522_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24522_344 = {S38, 0, &V24522_344_57272, &P24522_15};	/*Property = Masking*/
const Value V24522_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24522_320 = {S39, 0, &V24522_320_57290, &P24522_344};	/*Property = PostMessage*/
const Value V24522_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24522_321 = {S40, 0, &V24522_321_57293, &P24522_320};	/*Property = Progress*/
const Value V24522_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24522_322 = {S41, 0, &V24522_322_57296, &P24522_321};	/*Property = Prompts*/
const Value V24522_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24522_323 = {S42, 0, &V24522_323_57299, &P24522_322};	/*Property = Selector*/
const Value V24522_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24522_324 = {S43, 0, &V24522_324_57305, &P24522_323};	/*Property = Svg*/
const Value V24522_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24522_327 = {S44, 0, &V24522_327_57314, &P24522_324};	/*Property = TouchEvents*/
const Value V24522_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24522_328 = {S45, 0, &V24522_328_57318, &P24522_327};	/*Property = Track*/
const Value V24522_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24522_329 = {S46, 0, &V24522_329_57323, &P24522_328};	/*Property = Video*/
const Value V24522_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24522_330 = {S47, 0, &V24522_330_57326, &P24522_329};	/*Property = Viewport*/
const Value V24522_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24522_332 = {S48, 0, &V24522_332_57338, &P24522_330};	/*Property = WebWorkers*/
const Value V24522_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24522_333 = {S49, 0, &V24522_333_57342, &P24522_332};	/*Property = Xhr2*/
const Profile P24522 = {"24522", &P24522_333};	/*Profile ID - 24522*/
const Value V24523_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24523_279 = {S1, 0, &V24523_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24523_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24523_282 = {S3, 0, &V24523_282_57154, &P24523_279};	/*Property = BlobBuilder*/
const Value V24523_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24523_350 = {S4, 0, &V24523_350_57444, &P24523_282};	/*Property = Canvas*/
const Value V24523_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24523_283 = {S5, 0, &V24523_283_57159, &P24523_350};	/*Property = CssBackground*/
const Value V24523_284_57163 = {S0, NULL};	/*Value = False*/
const Property P24523_284 = {S6, 0, &V24523_284_57163, &P24523_283};	/*Property = CssBorderImage*/
const Value V24523_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24523_285 = {S7, 0, &V24523_285_57166, &P24523_284};	/*Property = CssCanvas*/
const Value V24523_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24523_286 = {S8, 0, &V24523_286_57168, &P24523_285};	/*Property = CssColor*/
const Value V24523_237_55784 = {S2, NULL};	/*Value = True*/
const Property P24523_237 = {S9, 0, &V24523_237_55784, &P24523_286};	/*Property = CssColumn*/
const Value V24523_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24523_287 = {S10, 0, &V24523_287_57172, &P24523_237};	/*Property = CssFlexbox*/
const Value V24523_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24523_288 = {S11, 0, &V24523_288_57175, &P24523_287};	/*Property = CssFont*/
const Value V24523_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24523_289 = {S12, 0, &V24523_289_57759, &P24523_288};	/*Property = CssImages*/
const Value V24523_291_57181 = {S0, NULL};	/*Value = False*/
const Property P24523_291 = {S13, 0, &V24523_291_57181, &P24523_289};	/*Property = CssMediaQueries*/
const Value V24523_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24523_292 = {S14, 0, &V24523_292_57183, &P24523_291};	/*Property = CssMinMax*/
const Value V24523_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24523_293 = {S15, 0, &V24523_293_57187, &P24523_292};	/*Property = CssOverflow*/
const Value V24523_294_57190 = {S0, NULL};	/*Value = False*/
const Property P24523_294 = {S16, 0, &V24523_294_57190, &P24523_293};	/*Property = CssPosition*/
const Value V24523_295_57193 = {S0, NULL};	/*Value = False*/
const Property P24523_295 = {S17, 0, &V24523_295_57193, &P24523_294};	/*Property = CssText*/
const Value V24523_297_57196 = {S0, NULL};	/*Value = False*/
const Property P24523_297 = {S18, 0, &V24523_297_57196, &P24523_295};	/*Property = CssTransforms*/
const Value V24523_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24523_241 = {S19, 0, &V24523_241_55799, &P24523_297};	/*Property = CssTransitions*/
const Value V24523_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24523_298 = {S20, 0, &V24523_298_57206, &P24523_241};	/*Property = CssUI*/
const Value V24523_300_57216 = {S0, NULL};	/*Value = False*/
const Property P24523_300 = {S21, 0, &V24523_300_57216, &P24523_298};	/*Property = DataSet*/
const Value V24523_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24523_301 = {S22, 0, &V24523_301_57218, &P24523_300};	/*Property = DataUrl*/
const Value V24523_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24523_302 = {S23, 0, &V24523_302_57222, &P24523_301};	/*Property = DeviceOrientation*/
const Value V24523_304_57228 = {S0, NULL};	/*Value = False*/
const Property P24523_304 = {S24, 0, &V24523_304_57228, &P24523_302};	/*Property = FileReader*/
const Value V24523_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24523_305 = {S25, 0, &V24523_305_57231, &P24523_304};	/*Property = FileSaver*/
const Value V24523_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24523_306 = {S26, 0, &V24523_306_57237, &P24523_305};	/*Property = FileWriter*/
const Value V24523_307_57240 = {S0, NULL};	/*Value = False*/
const Property P24523_307 = {S27, 0, &V24523_307_57240, &P24523_306};	/*Property = FormData*/
const Value V24523_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24523_309 = {S28, 0, &V24523_309_57246, &P24523_307};	/*Property = Fullscreen*/
const Value V24523_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24523_310 = {S29, 0, &V24523_310_57248, &P24523_309};	/*Property = GeoLocation*/
const Value V24523_312_57255 = {S0, NULL};	/*Value = False*/
const Property P24523_312 = {S30, 0, &V24523_312_57255, &P24523_310};	/*Property = History*/
const Value V24523_314_57261 = {S0, NULL};	/*Value = False*/
const Property P24523_314 = {S31, 0, &V24523_314_57261, &P24523_312};	/*Property = Html5*/
const Value V24523_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24523_313 = {S32, 0, &V24523_313_57258, &P24523_314};	/*Property = Html-Media-Capture*/
const Value V24523_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24523_315 = {S33, 0, &V24523_315_57264, &P24523_313};	/*Property = Iframe*/
const Value V24523_316_57267 = {S0, NULL};	/*Value = False*/
const Property P24523_316 = {S34, 0, &V24523_316_57267, &P24523_315};	/*Property = IndexedDB*/
const Value V24523_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24523_317 = {S35, 0, &V24523_317_57269, &P24523_316};	/*Property = Json*/
const Value V24523_15_81502 = {S141, NULL};	/*Value = Blink*/
const Property P24523_15 = {S37, 0, &V24523_15_81502, &P24523_317};	/*Property = LayoutEngine*/
const Value V24523_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24523_344 = {S38, 0, &V24523_344_57272, &P24523_15};	/*Property = Masking*/
const Value V24523_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24523_320 = {S39, 0, &V24523_320_57290, &P24523_344};	/*Property = PostMessage*/
const Value V24523_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24523_321 = {S40, 0, &V24523_321_57293, &P24523_320};	/*Property = Progress*/
const Value V24523_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24523_322 = {S41, 0, &V24523_322_57296, &P24523_321};	/*Property = Prompts*/
const Value V24523_323_57300 = {S0, NULL};	/*Value = False*/
const Property P24523_323 = {S42, 0, &V24523_323_57300, &P24523_322};	/*Property = Selector*/
const Value V24523_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24523_324 = {S43, 0, &V24523_324_57306, &P24523_323};	/*Property = Svg*/
const Value V24523_327_57315 = {S0, NULL};	/*Value = False*/
const Property P24523_327 = {S44, 0, &V24523_327_57315, &P24523_324};	/*Property = TouchEvents*/
const Value V24523_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24523_328 = {S45, 0, &V24523_328_57318, &P24523_327};	/*Property = Track*/
const Value V24523_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24523_329 = {S46, 0, &V24523_329_57323, &P24523_328};	/*Property = Video*/
const Value V24523_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24523_330 = {S47, 0, &V24523_330_57326, &P24523_329};	/*Property = Viewport*/
const Value V24523_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24523_332 = {S48, 0, &V24523_332_57339, &P24523_330};	/*Property = WebWorkers*/
const Value V24523_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24523_333 = {S49, 0, &V24523_333_57342, &P24523_332};	/*Property = Xhr2*/
const Profile P24523 = {"24523", &P24523_333};	/*Profile ID - 24523*/
const Profile P24527 = {"24527", NULL};	/*Profile ID - 24527*/
const Value V24651_1_1 = {S2, NULL};	/*Value = True*/
const Property P24651_1 = {S50, 0, &V24651_1_1, NULL};	/*Property = IsMobile*/
const Value V24651_5_1458 = {S85, NULL};	/*Value = 1024*/
const Property P24651_5 = {S52, 0, &V24651_5_1458, &P24651_1};	/*Property = ScreenPixelsHeight*/
const Value V24651_4_1460 = {S84, NULL};	/*Value = 600*/
const Property P24651_4 = {S54, 0, &V24651_4_1460, &P24651_5};	/*Property = ScreenPixelsWidth*/
const Profile P24651 = {"24651", &P24651_4};	/*Profile ID - 24651*/
const Value V24625_1_1 = {S2, NULL};	/*Value = True*/
const Property P24625_1 = {S50, 0, &V24625_1_1, NULL};	/*Property = IsMobile*/
const Value V24625_5_1222 = {S57, NULL};	/*Value = 320*/
const Property P24625_5 = {S52, 0, &V24625_5_1222, &P24625_1};	/*Property = ScreenPixelsHeight*/
const Value V24625_4_1224 = {S58, NULL};	/*Value = 240*/
const Property P24625_4 = {S54, 0, &V24625_4_1224, &P24625_5};	/*Property = ScreenPixelsWidth*/
const Profile P24625 = {"24625", &P24625_4};	/*Profile ID - 24625*/
const Value V24717_279_57149 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_279 = {S1, 0, &V24717_279_57149, NULL};	/*Property = AnimationTiming*/
const Value V24717_282_57155 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_282 = {S3, 0, &V24717_282_57155, &P24717_279};	/*Property = BlobBuilder*/
const Value V24717_350_57446 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_350 = {S4, 0, &V24717_350_57446, &P24717_282};	/*Property = Canvas*/
const Value V24717_283_57161 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_283 = {S5, 0, &V24717_283_57161, &P24717_350};	/*Property = CssBackground*/
const Value V24717_284_57164 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_284 = {S6, 0, &V24717_284_57164, &P24717_283};	/*Property = CssBorderImage*/
const Value V24717_285_57167 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_285 = {S7, 0, &V24717_285_57167, &P24717_284};	/*Property = CssCanvas*/
const Value V24717_286_57170 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_286 = {S8, 0, &V24717_286_57170, &P24717_285};	/*Property = CssColor*/
const Value V24717_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_237 = {S9, 0, &V24717_237_55786, &P24717_286};	/*Property = CssColumn*/
const Value V24717_287_57173 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_287 = {S10, 0, &V24717_287_57173, &P24717_237};	/*Property = CssFlexbox*/
const Value V24717_288_57176 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_288 = {S11, 0, &V24717_288_57176, &P24717_287};	/*Property = CssFont*/
const Value V24717_289_86899 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_289 = {S12, 0, &V24717_289_86899, &P24717_288};	/*Property = CssImages*/
const Value V24717_291_57182 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_291 = {S13, 0, &V24717_291_57182, &P24717_289};	/*Property = CssMediaQueries*/
const Value V24717_292_57185 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_292 = {S14, 0, &V24717_292_57185, &P24717_291};	/*Property = CssMinMax*/
const Value V24717_293_57188 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_293 = {S15, 0, &V24717_293_57188, &P24717_292};	/*Property = CssOverflow*/
const Value V24717_294_57191 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_294 = {S16, 0, &V24717_294_57191, &P24717_293};	/*Property = CssPosition*/
const Value V24717_295_57194 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_295 = {S17, 0, &V24717_295_57194, &P24717_294};	/*Property = CssText*/
const Value V24717_297_57197 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_297 = {S18, 0, &V24717_297_57197, &P24717_295};	/*Property = CssTransforms*/
const Value V24717_241_55801 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_241 = {S19, 0, &V24717_241_55801, &P24717_297};	/*Property = CssTransitions*/
const Value V24717_298_57208 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_298 = {S20, 0, &V24717_298_57208, &P24717_241};	/*Property = CssUI*/
const Value V24717_300_57217 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_300 = {S21, 0, &V24717_300_57217, &P24717_298};	/*Property = DataSet*/
const Value V24717_301_57220 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_301 = {S22, 0, &V24717_301_57220, &P24717_300};	/*Property = DataUrl*/
const Value V24717_302_57223 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_302 = {S23, 0, &V24717_302_57223, &P24717_301};	/*Property = DeviceOrientation*/
const Value V24717_304_57229 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_304 = {S24, 0, &V24717_304_57229, &P24717_302};	/*Property = FileReader*/
const Value V24717_305_57232 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_305 = {S25, 0, &V24717_305_57232, &P24717_304};	/*Property = FileSaver*/
const Value V24717_306_57238 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_306 = {S26, 0, &V24717_306_57238, &P24717_305};	/*Property = FileWriter*/
const Value V24717_307_57241 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_307 = {S27, 0, &V24717_307_57241, &P24717_306};	/*Property = FormData*/
const Value V24717_309_57247 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_309 = {S28, 0, &V24717_309_57247, &P24717_307};	/*Property = Fullscreen*/
const Value V24717_310_57250 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_310 = {S29, 0, &V24717_310_57250, &P24717_309};	/*Property = GeoLocation*/
const Value V24717_312_57256 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_312 = {S30, 0, &V24717_312_57256, &P24717_310};	/*Property = History*/
const Value V24717_314_57262 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_314 = {S31, 0, &V24717_314_57262, &P24717_312};	/*Property = Html5*/
const Value V24717_313_57259 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_313 = {S32, 0, &V24717_313_57259, &P24717_314};	/*Property = Html-Media-Capture*/
const Value V24717_315_57265 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_315 = {S33, 0, &V24717_315_57265, &P24717_313};	/*Property = Iframe*/
const Value V24717_316_57268 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_316 = {S34, 0, &V24717_316_57268, &P24717_315};	/*Property = IndexedDB*/
const Value V24717_317_57271 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_317 = {S35, 0, &V24717_317_57271, &P24717_316};	/*Property = Json*/
const Value V24717_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24717_15 = {S37, 0, &V24717_15_28, &P24717_317};	/*Property = LayoutEngine*/
const Value V24717_344_57274 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_344 = {S38, 0, &V24717_344_57274, &P24717_15};	/*Property = Masking*/
const Value V24717_320_57292 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_320 = {S39, 0, &V24717_320_57292, &P24717_344};	/*Property = PostMessage*/
const Value V24717_321_57295 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_321 = {S40, 0, &V24717_321_57295, &P24717_320};	/*Property = Progress*/
const Value V24717_322_57298 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_322 = {S41, 0, &V24717_322_57298, &P24717_321};	/*Property = Prompts*/
const Value V24717_323_57301 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_323 = {S42, 0, &V24717_323_57301, &P24717_322};	/*Property = Selector*/
const Value V24717_324_57307 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_324 = {S43, 0, &V24717_324_57307, &P24717_323};	/*Property = Svg*/
const Value V24717_327_57316 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_327 = {S44, 0, &V24717_327_57316, &P24717_324};	/*Property = TouchEvents*/
const Value V24717_328_57319 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_328 = {S45, 0, &V24717_328_57319, &P24717_327};	/*Property = Track*/
const Value V24717_329_57325 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_329 = {S46, 0, &V24717_329_57325, &P24717_328};	/*Property = Video*/
const Value V24717_330_57328 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_330 = {S47, 0, &V24717_330_57328, &P24717_329};	/*Property = Viewport*/
const Value V24717_332_57340 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_332 = {S48, 0, &V24717_332_57340, &P24717_330};	/*Property = WebWorkers*/
const Value V24717_333_57343 = {S55, NULL};	/*Value = Unknown*/
const Property P24717_333 = {S49, 0, &V24717_333_57343, &P24717_332};	/*Property = Xhr2*/
const Profile P24717 = {"24717", &P24717_333};	/*Profile ID - 24717*/
const Value V24608_1_1 = {S2, NULL};	/*Value = True*/
const Property P24608_1 = {S50, 0, &V24608_1_1, NULL};	/*Property = IsMobile*/
const Value V24608_5_1277 = {S56, NULL};	/*Value = 480*/
const Property P24608_5 = {S52, 0, &V24608_5_1277, &P24608_1};	/*Property = ScreenPixelsHeight*/
const Value V24608_4_1279 = {S57, NULL};	/*Value = 320*/
const Property P24608_4 = {S54, 0, &V24608_4_1279, &P24608_5};	/*Property = ScreenPixelsWidth*/
const Profile P24608 = {"24608", &P24608_4};	/*Profile ID - 24608*/
const Value V24656_1_1 = {S2, NULL};	/*Value = True*/
const Property P24656_1 = {S50, 0, &V24656_1_1, NULL};	/*Property = IsMobile*/
const Value V24656_5_1403 = {S82, NULL};	/*Value = 960*/
const Property P24656_5 = {S52, 0, &V24656_5_1403, &P24656_1};	/*Property = ScreenPixelsHeight*/
const Value V24656_4_1404 = {S89, NULL};	/*Value = 540*/
const Property P24656_4 = {S54, 0, &V24656_4_1404, &P24656_5};	/*Property = ScreenPixelsWidth*/
const Profile P24656 = {"24656", &P24656_4};	/*Profile ID - 24656*/
const Value V24623_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24623_279 = {S1, 0, &V24623_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24623_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24623_282 = {S3, 0, &V24623_282_57154, &P24623_279};	/*Property = BlobBuilder*/
const Value V24623_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24623_350 = {S4, 0, &V24623_350_57444, &P24623_282};	/*Property = Canvas*/
const Value V24623_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24623_283 = {S5, 0, &V24623_283_57159, &P24623_350};	/*Property = CssBackground*/
const Value V24623_284_57163 = {S0, NULL};	/*Value = False*/
const Property P24623_284 = {S6, 0, &V24623_284_57163, &P24623_283};	/*Property = CssBorderImage*/
const Value V24623_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24623_285 = {S7, 0, &V24623_285_57166, &P24623_284};	/*Property = CssCanvas*/
const Value V24623_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24623_286 = {S8, 0, &V24623_286_57168, &P24623_285};	/*Property = CssColor*/
const Value V24623_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24623_237 = {S9, 0, &V24623_237_55786, &P24623_286};	/*Property = CssColumn*/
const Value V24623_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24623_287 = {S10, 0, &V24623_287_57172, &P24623_237};	/*Property = CssFlexbox*/
const Value V24623_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24623_288 = {S11, 0, &V24623_288_57174, &P24623_287};	/*Property = CssFont*/
const Value V24623_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24623_289 = {S12, 0, &V24623_289_57759, &P24623_288};	/*Property = CssImages*/
const Value V24623_291_57181 = {S0, NULL};	/*Value = False*/
const Property P24623_291 = {S13, 0, &V24623_291_57181, &P24623_289};	/*Property = CssMediaQueries*/
const Value V24623_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24623_292 = {S14, 0, &V24623_292_57183, &P24623_291};	/*Property = CssMinMax*/
const Value V24623_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24623_293 = {S15, 0, &V24623_293_57187, &P24623_292};	/*Property = CssOverflow*/
const Value V24623_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24623_294 = {S16, 0, &V24623_294_57189, &P24623_293};	/*Property = CssPosition*/
const Value V24623_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24623_295 = {S17, 0, &V24623_295_57192, &P24623_294};	/*Property = CssText*/
const Value V24623_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24623_297 = {S18, 0, &V24623_297_57195, &P24623_295};	/*Property = CssTransforms*/
const Value V24623_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24623_241 = {S19, 0, &V24623_241_55799, &P24623_297};	/*Property = CssTransitions*/
const Value V24623_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24623_298 = {S20, 0, &V24623_298_57206, &P24623_241};	/*Property = CssUI*/
const Value V24623_300_57216 = {S0, NULL};	/*Value = False*/
const Property P24623_300 = {S21, 0, &V24623_300_57216, &P24623_298};	/*Property = DataSet*/
const Value V24623_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24623_301 = {S22, 0, &V24623_301_57218, &P24623_300};	/*Property = DataUrl*/
const Value V24623_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24623_302 = {S23, 0, &V24623_302_57222, &P24623_301};	/*Property = DeviceOrientation*/
const Value V24623_304_57228 = {S0, NULL};	/*Value = False*/
const Property P24623_304 = {S24, 0, &V24623_304_57228, &P24623_302};	/*Property = FileReader*/
const Value V24623_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24623_305 = {S25, 0, &V24623_305_57231, &P24623_304};	/*Property = FileSaver*/
const Value V24623_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24623_306 = {S26, 0, &V24623_306_57237, &P24623_305};	/*Property = FileWriter*/
const Value V24623_307_57240 = {S0, NULL};	/*Value = False*/
const Property P24623_307 = {S27, 0, &V24623_307_57240, &P24623_306};	/*Property = FormData*/
const Value V24623_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24623_309 = {S28, 0, &V24623_309_57246, &P24623_307};	/*Property = Fullscreen*/
const Value V24623_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24623_310 = {S29, 0, &V24623_310_57248, &P24623_309};	/*Property = GeoLocation*/
const Value V24623_312_57255 = {S0, NULL};	/*Value = False*/
const Property P24623_312 = {S30, 0, &V24623_312_57255, &P24623_310};	/*Property = History*/
const Value V24623_314_57261 = {S0, NULL};	/*Value = False*/
const Property P24623_314 = {S31, 0, &V24623_314_57261, &P24623_312};	/*Property = Html5*/
const Value V24623_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24623_313 = {S32, 0, &V24623_313_57258, &P24623_314};	/*Property = Html-Media-Capture*/
const Value V24623_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24623_315 = {S33, 0, &V24623_315_57264, &P24623_313};	/*Property = Iframe*/
const Value V24623_316_57267 = {S0, NULL};	/*Value = False*/
const Property P24623_316 = {S34, 0, &V24623_316_57267, &P24623_315};	/*Property = IndexedDB*/
const Value V24623_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24623_317 = {S35, 0, &V24623_317_57269, &P24623_316};	/*Property = Json*/
const Value V24623_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24623_15 = {S37, 0, &V24623_15_28, &P24623_317};	/*Property = LayoutEngine*/
const Value V24623_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24623_344 = {S38, 0, &V24623_344_57272, &P24623_15};	/*Property = Masking*/
const Value V24623_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24623_320 = {S39, 0, &V24623_320_57290, &P24623_344};	/*Property = PostMessage*/
const Value V24623_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24623_321 = {S40, 0, &V24623_321_57293, &P24623_320};	/*Property = Progress*/
const Value V24623_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24623_322 = {S41, 0, &V24623_322_57296, &P24623_321};	/*Property = Prompts*/
const Value V24623_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24623_323 = {S42, 0, &V24623_323_57299, &P24623_322};	/*Property = Selector*/
const Value V24623_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24623_324 = {S43, 0, &V24623_324_57306, &P24623_323};	/*Property = Svg*/
const Value V24623_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24623_327 = {S44, 0, &V24623_327_57314, &P24623_324};	/*Property = TouchEvents*/
const Value V24623_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24623_328 = {S45, 0, &V24623_328_57318, &P24623_327};	/*Property = Track*/
const Value V24623_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24623_329 = {S46, 0, &V24623_329_57323, &P24623_328};	/*Property = Video*/
const Value V24623_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24623_330 = {S47, 0, &V24623_330_57326, &P24623_329};	/*Property = Viewport*/
const Value V24623_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24623_332 = {S48, 0, &V24623_332_57339, &P24623_330};	/*Property = WebWorkers*/
const Value V24623_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24623_333 = {S49, 0, &V24623_333_57342, &P24623_332};	/*Property = Xhr2*/
const Profile P24623 = {"24623", &P24623_333};	/*Profile ID - 24623*/
const Value V24683_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24683_279 = {S1, 0, &V24683_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24683_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24683_282 = {S3, 0, &V24683_282_57154, &P24683_279};	/*Property = BlobBuilder*/
const Value V24683_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24683_350 = {S4, 0, &V24683_350_57444, &P24683_282};	/*Property = Canvas*/
const Value V24683_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24683_283 = {S5, 0, &V24683_283_57159, &P24683_350};	/*Property = CssBackground*/
const Value V24683_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24683_284 = {S6, 0, &V24683_284_57162, &P24683_283};	/*Property = CssBorderImage*/
const Value V24683_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24683_285 = {S7, 0, &V24683_285_57166, &P24683_284};	/*Property = CssCanvas*/
const Value V24683_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24683_286 = {S8, 0, &V24683_286_57168, &P24683_285};	/*Property = CssColor*/
const Value V24683_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24683_237 = {S9, 0, &V24683_237_55786, &P24683_286};	/*Property = CssColumn*/
const Value V24683_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24683_287 = {S10, 0, &V24683_287_57172, &P24683_237};	/*Property = CssFlexbox*/
const Value V24683_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24683_288 = {S11, 0, &V24683_288_57174, &P24683_287};	/*Property = CssFont*/
const Value V24683_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24683_289 = {S12, 0, &V24683_289_57759, &P24683_288};	/*Property = CssImages*/
const Value V24683_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24683_291 = {S13, 0, &V24683_291_57180, &P24683_289};	/*Property = CssMediaQueries*/
const Value V24683_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24683_292 = {S14, 0, &V24683_292_57183, &P24683_291};	/*Property = CssMinMax*/
const Value V24683_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24683_293 = {S15, 0, &V24683_293_57187, &P24683_292};	/*Property = CssOverflow*/
const Value V24683_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24683_294 = {S16, 0, &V24683_294_57189, &P24683_293};	/*Property = CssPosition*/
const Value V24683_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24683_295 = {S17, 0, &V24683_295_57192, &P24683_294};	/*Property = CssText*/
const Value V24683_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24683_297 = {S18, 0, &V24683_297_57195, &P24683_295};	/*Property = CssTransforms*/
const Value V24683_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24683_241 = {S19, 0, &V24683_241_55799, &P24683_297};	/*Property = CssTransitions*/
const Value V24683_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24683_298 = {S20, 0, &V24683_298_57206, &P24683_241};	/*Property = CssUI*/
const Value V24683_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24683_300 = {S21, 0, &V24683_300_57215, &P24683_298};	/*Property = DataSet*/
const Value V24683_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24683_301 = {S22, 0, &V24683_301_57218, &P24683_300};	/*Property = DataUrl*/
const Value V24683_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24683_302 = {S23, 0, &V24683_302_57221, &P24683_301};	/*Property = DeviceOrientation*/
const Value V24683_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24683_304 = {S24, 0, &V24683_304_57227, &P24683_302};	/*Property = FileReader*/
const Value V24683_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24683_305 = {S25, 0, &V24683_305_57231, &P24683_304};	/*Property = FileSaver*/
const Value V24683_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24683_306 = {S26, 0, &V24683_306_57237, &P24683_305};	/*Property = FileWriter*/
const Value V24683_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24683_307 = {S27, 0, &V24683_307_57239, &P24683_306};	/*Property = FormData*/
const Value V24683_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24683_309 = {S28, 0, &V24683_309_57246, &P24683_307};	/*Property = Fullscreen*/
const Value V24683_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24683_310 = {S29, 0, &V24683_310_57248, &P24683_309};	/*Property = GeoLocation*/
const Value V24683_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24683_312 = {S30, 0, &V24683_312_57254, &P24683_310};	/*Property = History*/
const Value V24683_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24683_314 = {S31, 0, &V24683_314_57260, &P24683_312};	/*Property = Html5*/
const Value V24683_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24683_313 = {S32, 0, &V24683_313_57257, &P24683_314};	/*Property = Html-Media-Capture*/
const Value V24683_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24683_315 = {S33, 0, &V24683_315_57264, &P24683_313};	/*Property = Iframe*/
const Value V24683_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24683_316 = {S34, 0, &V24683_316_57266, &P24683_315};	/*Property = IndexedDB*/
const Value V24683_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24683_317 = {S35, 0, &V24683_317_57269, &P24683_316};	/*Property = Json*/
const Value V24683_15_54167 = {S103, NULL};	/*Value = KHTML*/
const Property P24683_15 = {S37, 0, &V24683_15_54167, &P24683_317};	/*Property = LayoutEngine*/
const Value V24683_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24683_344 = {S38, 0, &V24683_344_57272, &P24683_15};	/*Property = Masking*/
const Value V24683_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24683_320 = {S39, 0, &V24683_320_57290, &P24683_344};	/*Property = PostMessage*/
const Value V24683_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24683_321 = {S40, 0, &V24683_321_57293, &P24683_320};	/*Property = Progress*/
const Value V24683_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24683_322 = {S41, 0, &V24683_322_57296, &P24683_321};	/*Property = Prompts*/
const Value V24683_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24683_323 = {S42, 0, &V24683_323_57299, &P24683_322};	/*Property = Selector*/
const Value V24683_324_57306 = {S0, NULL};	/*Value = False*/
const Property P24683_324 = {S43, 0, &V24683_324_57306, &P24683_323};	/*Property = Svg*/
const Value V24683_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24683_327 = {S44, 0, &V24683_327_57314, &P24683_324};	/*Property = TouchEvents*/
const Value V24683_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24683_328 = {S45, 0, &V24683_328_57318, &P24683_327};	/*Property = Track*/
const Value V24683_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24683_329 = {S46, 0, &V24683_329_57323, &P24683_328};	/*Property = Video*/
const Value V24683_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24683_330 = {S47, 0, &V24683_330_57326, &P24683_329};	/*Property = Viewport*/
const Value V24683_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24683_332 = {S48, 0, &V24683_332_57339, &P24683_330};	/*Property = WebWorkers*/
const Value V24683_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24683_333 = {S49, 0, &V24683_333_57342, &P24683_332};	/*Property = Xhr2*/
const Profile P24683 = {"24683", &P24683_333};	/*Profile ID - 24683*/
const Value V24588_279_57147 = {S2, NULL};	/*Value = True*/
const Property P24588_279 = {S1, 0, &V24588_279_57147, NULL};	/*Property = AnimationTiming*/
const Value V24588_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24588_282 = {S3, 0, &V24588_282_57154, &P24588_279};	/*Property = BlobBuilder*/
const Value V24588_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24588_350 = {S4, 0, &V24588_350_57444, &P24588_282};	/*Property = Canvas*/
const Value V24588_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24588_283 = {S5, 0, &V24588_283_57159, &P24588_350};	/*Property = CssBackground*/
const Value V24588_284_57162 = {S2, NULL};	/*Value = True*/
const Property P24588_284 = {S6, 0, &V24588_284_57162, &P24588_283};	/*Property = CssBorderImage*/
const Value V24588_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24588_285 = {S7, 0, &V24588_285_57166, &P24588_284};	/*Property = CssCanvas*/
const Value V24588_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24588_286 = {S8, 0, &V24588_286_57168, &P24588_285};	/*Property = CssColor*/
const Value V24588_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24588_237 = {S9, 0, &V24588_237_55786, &P24588_286};	/*Property = CssColumn*/
const Value V24588_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24588_287 = {S10, 0, &V24588_287_57172, &P24588_237};	/*Property = CssFlexbox*/
const Value V24588_288_57174 = {S2, NULL};	/*Value = True*/
const Property P24588_288 = {S11, 0, &V24588_288_57174, &P24588_287};	/*Property = CssFont*/
const Value V24588_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24588_289 = {S12, 0, &V24588_289_57759, &P24588_288};	/*Property = CssImages*/
const Value V24588_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24588_291 = {S13, 0, &V24588_291_57180, &P24588_289};	/*Property = CssMediaQueries*/
const Value V24588_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24588_292 = {S14, 0, &V24588_292_57183, &P24588_291};	/*Property = CssMinMax*/
const Value V24588_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24588_293 = {S15, 0, &V24588_293_57187, &P24588_292};	/*Property = CssOverflow*/
const Value V24588_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24588_294 = {S16, 0, &V24588_294_57189, &P24588_293};	/*Property = CssPosition*/
const Value V24588_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24588_295 = {S17, 0, &V24588_295_57192, &P24588_294};	/*Property = CssText*/
const Value V24588_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24588_297 = {S18, 0, &V24588_297_57195, &P24588_295};	/*Property = CssTransforms*/
const Value V24588_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24588_241 = {S19, 0, &V24588_241_55799, &P24588_297};	/*Property = CssTransitions*/
const Value V24588_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24588_298 = {S20, 0, &V24588_298_57206, &P24588_241};	/*Property = CssUI*/
const Value V24588_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24588_300 = {S21, 0, &V24588_300_57215, &P24588_298};	/*Property = DataSet*/
const Value V24588_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24588_301 = {S22, 0, &V24588_301_57218, &P24588_300};	/*Property = DataUrl*/
const Value V24588_302_57222 = {S0, NULL};	/*Value = False*/
const Property P24588_302 = {S23, 0, &V24588_302_57222, &P24588_301};	/*Property = DeviceOrientation*/
const Value V24588_304_57227 = {S2, NULL};	/*Value = True*/
const Property P24588_304 = {S24, 0, &V24588_304_57227, &P24588_302};	/*Property = FileReader*/
const Value V24588_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24588_305 = {S25, 0, &V24588_305_57231, &P24588_304};	/*Property = FileSaver*/
const Value V24588_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24588_306 = {S26, 0, &V24588_306_57237, &P24588_305};	/*Property = FileWriter*/
const Value V24588_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24588_307 = {S27, 0, &V24588_307_57239, &P24588_306};	/*Property = FormData*/
const Value V24588_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24588_309 = {S28, 0, &V24588_309_57246, &P24588_307};	/*Property = Fullscreen*/
const Value V24588_310_57248 = {S2, NULL};	/*Value = True*/
const Property P24588_310 = {S29, 0, &V24588_310_57248, &P24588_309};	/*Property = GeoLocation*/
const Value V24588_312_57254 = {S2, NULL};	/*Value = True*/
const Property P24588_312 = {S30, 0, &V24588_312_57254, &P24588_310};	/*Property = History*/
const Value V24588_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24588_314 = {S31, 0, &V24588_314_57260, &P24588_312};	/*Property = Html5*/
const Value V24588_313_57257 = {S2, NULL};	/*Value = True*/
const Property P24588_313 = {S32, 0, &V24588_313_57257, &P24588_314};	/*Property = Html-Media-Capture*/
const Value V24588_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24588_315 = {S33, 0, &V24588_315_57264, &P24588_313};	/*Property = Iframe*/
const Value V24588_316_57266 = {S2, NULL};	/*Value = True*/
const Property P24588_316 = {S34, 0, &V24588_316_57266, &P24588_315};	/*Property = IndexedDB*/
const Value V24588_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24588_317 = {S35, 0, &V24588_317_57269, &P24588_316};	/*Property = Json*/
const Value V24588_15_81502 = {S141, NULL};	/*Value = Blink*/
const Property P24588_15 = {S37, 0, &V24588_15_81502, &P24588_317};	/*Property = LayoutEngine*/
const Value V24588_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24588_344 = {S38, 0, &V24588_344_57272, &P24588_15};	/*Property = Masking*/
const Value V24588_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24588_320 = {S39, 0, &V24588_320_57290, &P24588_344};	/*Property = PostMessage*/
const Value V24588_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24588_321 = {S40, 0, &V24588_321_57293, &P24588_320};	/*Property = Progress*/
const Value V24588_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24588_322 = {S41, 0, &V24588_322_57296, &P24588_321};	/*Property = Prompts*/
const Value V24588_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24588_323 = {S42, 0, &V24588_323_57299, &P24588_322};	/*Property = Selector*/
const Value V24588_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24588_324 = {S43, 0, &V24588_324_57305, &P24588_323};	/*Property = Svg*/
const Value V24588_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24588_327 = {S44, 0, &V24588_327_57314, &P24588_324};	/*Property = TouchEvents*/
const Value V24588_328_57317 = {S2, NULL};	/*Value = True*/
const Property P24588_328 = {S45, 0, &V24588_328_57317, &P24588_327};	/*Property = Track*/
const Value V24588_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24588_329 = {S46, 0, &V24588_329_57323, &P24588_328};	/*Property = Video*/
const Value V24588_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24588_330 = {S47, 0, &V24588_330_57326, &P24588_329};	/*Property = Viewport*/
const Value V24588_332_57338 = {S2, NULL};	/*Value = True*/
const Property P24588_332 = {S48, 0, &V24588_332_57338, &P24588_330};	/*Property = WebWorkers*/
const Value V24588_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24588_333 = {S49, 0, &V24588_333_57342, &P24588_332};	/*Property = Xhr2*/
const Profile P24588 = {"24588", &P24588_333};	/*Profile ID - 24588*/
const Value V24711_279_57148 = {S0, NULL};	/*Value = False*/
const Property P24711_279 = {S1, 0, &V24711_279_57148, NULL};	/*Property = AnimationTiming*/
const Value V24711_282_57154 = {S0, NULL};	/*Value = False*/
const Property P24711_282 = {S3, 0, &V24711_282_57154, &P24711_279};	/*Property = BlobBuilder*/
const Value V24711_350_57444 = {S2, NULL};	/*Value = True*/
const Property P24711_350 = {S4, 0, &V24711_350_57444, &P24711_282};	/*Property = Canvas*/
const Value V24711_283_57159 = {S2, NULL};	/*Value = True*/
const Property P24711_283 = {S5, 0, &V24711_283_57159, &P24711_350};	/*Property = CssBackground*/
const Value V24711_284_57163 = {S0, NULL};	/*Value = False*/
const Property P24711_284 = {S6, 0, &V24711_284_57163, &P24711_283};	/*Property = CssBorderImage*/
const Value V24711_285_57166 = {S0, NULL};	/*Value = False*/
const Property P24711_285 = {S7, 0, &V24711_285_57166, &P24711_284};	/*Property = CssCanvas*/
const Value V24711_286_57168 = {S2, NULL};	/*Value = True*/
const Property P24711_286 = {S8, 0, &V24711_286_57168, &P24711_285};	/*Property = CssColor*/
const Value V24711_237_55786 = {S55, NULL};	/*Value = Unknown*/
const Property P24711_237 = {S9, 0, &V24711_237_55786, &P24711_286};	/*Property = CssColumn*/
const Value V24711_287_57172 = {S0, NULL};	/*Value = False*/
const Property P24711_287 = {S10, 0, &V24711_287_57172, &P24711_237};	/*Property = CssFlexbox*/
const Value V24711_288_57175 = {S0, NULL};	/*Value = False*/
const Property P24711_288 = {S11, 0, &V24711_288_57175, &P24711_287};	/*Property = CssFont*/
const Value V24711_289_57759 = {S0, NULL};	/*Value = False*/
const Property P24711_289 = {S12, 0, &V24711_289_57759, &P24711_288};	/*Property = CssImages*/
const Value V24711_291_57180 = {S2, NULL};	/*Value = True*/
const Property P24711_291 = {S13, 0, &V24711_291_57180, &P24711_289};	/*Property = CssMediaQueries*/
const Value V24711_292_57183 = {S2, NULL};	/*Value = True*/
const Property P24711_292 = {S14, 0, &V24711_292_57183, &P24711_291};	/*Property = CssMinMax*/
const Value V24711_293_57187 = {S0, NULL};	/*Value = False*/
const Property P24711_293 = {S15, 0, &V24711_293_57187, &P24711_292};	/*Property = CssOverflow*/
const Value V24711_294_57189 = {S2, NULL};	/*Value = True*/
const Property P24711_294 = {S16, 0, &V24711_294_57189, &P24711_293};	/*Property = CssPosition*/
const Value V24711_295_57192 = {S2, NULL};	/*Value = True*/
const Property P24711_295 = {S17, 0, &V24711_295_57192, &P24711_294};	/*Property = CssText*/
const Value V24711_297_57195 = {S2, NULL};	/*Value = True*/
const Property P24711_297 = {S18, 0, &V24711_297_57195, &P24711_295};	/*Property = CssTransforms*/
const Value V24711_241_55799 = {S2, NULL};	/*Value = True*/
const Property P24711_241 = {S19, 0, &V24711_241_55799, &P24711_297};	/*Property = CssTransitions*/
const Value V24711_298_57206 = {S2, NULL};	/*Value = True*/
const Property P24711_298 = {S20, 0, &V24711_298_57206, &P24711_241};	/*Property = CssUI*/
const Value V24711_300_57215 = {S2, NULL};	/*Value = True*/
const Property P24711_300 = {S21, 0, &V24711_300_57215, &P24711_298};	/*Property = DataSet*/
const Value V24711_301_57218 = {S2, NULL};	/*Value = True*/
const Property P24711_301 = {S22, 0, &V24711_301_57218, &P24711_300};	/*Property = DataUrl*/
const Value V24711_302_57221 = {S2, NULL};	/*Value = True*/
const Property P24711_302 = {S23, 0, &V24711_302_57221, &P24711_301};	/*Property = DeviceOrientation*/
const Value V24711_304_57228 = {S0, NULL};	/*Value = False*/
const Property P24711_304 = {S24, 0, &V24711_304_57228, &P24711_302};	/*Property = FileReader*/
const Value V24711_305_57231 = {S0, NULL};	/*Value = False*/
const Property P24711_305 = {S25, 0, &V24711_305_57231, &P24711_304};	/*Property = FileSaver*/
const Value V24711_306_57237 = {S0, NULL};	/*Value = False*/
const Property P24711_306 = {S26, 0, &V24711_306_57237, &P24711_305};	/*Property = FileWriter*/
const Value V24711_307_57239 = {S2, NULL};	/*Value = True*/
const Property P24711_307 = {S27, 0, &V24711_307_57239, &P24711_306};	/*Property = FormData*/
const Value V24711_309_57246 = {S0, NULL};	/*Value = False*/
const Property P24711_309 = {S28, 0, &V24711_309_57246, &P24711_307};	/*Property = Fullscreen*/
const Value V24711_310_57249 = {S0, NULL};	/*Value = False*/
const Property P24711_310 = {S29, 0, &V24711_310_57249, &P24711_309};	/*Property = GeoLocation*/
const Value V24711_312_57255 = {S0, NULL};	/*Value = False*/
const Property P24711_312 = {S30, 0, &V24711_312_57255, &P24711_310};	/*Property = History*/
const Value V24711_314_57260 = {S2, NULL};	/*Value = True*/
const Property P24711_314 = {S31, 0, &V24711_314_57260, &P24711_312};	/*Property = Html5*/
const Value V24711_313_57258 = {S0, NULL};	/*Value = False*/
const Property P24711_313 = {S32, 0, &V24711_313_57258, &P24711_314};	/*Property = Html-Media-Capture*/
const Value V24711_315_57264 = {S0, NULL};	/*Value = False*/
const Property P24711_315 = {S33, 0, &V24711_315_57264, &P24711_313};	/*Property = Iframe*/
const Value V24711_316_57267 = {S0, NULL};	/*Value = False*/
const Property P24711_316 = {S34, 0, &V24711_316_57267, &P24711_315};	/*Property = IndexedDB*/
const Value V24711_317_57269 = {S2, NULL};	/*Value = True*/
const Property P24711_317 = {S35, 0, &V24711_317_57269, &P24711_316};	/*Property = Json*/
const Value V24711_15_28 = {S36, NULL};	/*Value = Webkit*/
const Property P24711_15 = {S37, 0, &V24711_15_28, &P24711_317};	/*Property = LayoutEngine*/
const Value V24711_344_57272 = {S2, NULL};	/*Value = True*/
const Property P24711_344 = {S38, 0, &V24711_344_57272, &P24711_15};	/*Property = Masking*/
const Value V24711_320_57290 = {S2, NULL};	/*Value = True*/
const Property P24711_320 = {S39, 0, &V24711_320_57290, &P24711_344};	/*Property = PostMessage*/
const Value V24711_321_57293 = {S2, NULL};	/*Value = True*/
const Property P24711_321 = {S40, 0, &V24711_321_57293, &P24711_320};	/*Property = Progress*/
const Value V24711_322_57296 = {S2, NULL};	/*Value = True*/
const Property P24711_322 = {S41, 0, &V24711_322_57296, &P24711_321};	/*Property = Prompts*/
const Value V24711_323_57299 = {S2, NULL};	/*Value = True*/
const Property P24711_323 = {S42, 0, &V24711_323_57299, &P24711_322};	/*Property = Selector*/
const Value V24711_324_57305 = {S2, NULL};	/*Value = True*/
const Property P24711_324 = {S43, 0, &V24711_324_57305, &P24711_323};	/*Property = Svg*/
const Value V24711_327_57314 = {S2, NULL};	/*Value = True*/
const Property P24711_327 = {S44, 0, &V24711_327_57314, &P24711_324};	/*Property = TouchEvents*/
const Value V24711_328_57318 = {S0, NULL};	/*Value = False*/
const Property P24711_328 = {S45, 0, &V24711_328_57318, &P24711_327};	/*Property = Track*/
const Value V24711_329_57323 = {S2, NULL};	/*Value = True*/
const Property P24711_329 = {S46, 0, &V24711_329_57323, &P24711_328};	/*Property = Video*/
const Value V24711_330_57326 = {S2, NULL};	/*Value = True*/
const Property P24711_330 = {S47, 0, &V24711_330_57326, &P24711_329};	/*Property = Viewport*/
const Value V24711_332_57339 = {S0, NULL};	/*Value = False*/
const Property P24711_332 = {S48, 0, &V24711_332_57339, &P24711_330};	/*Property = WebWorkers*/
const Value V24711_333_57342 = {S0, NULL};	/*Value = False*/
const Property P24711_333 = {S49, 0, &V24711_333_57342, &P24711_332};	/*Property = Xhr2*/
const Profile P24711 = {"24711", &P24711_333};	/*Profile ID - 24711*/
const Profile P24705 = {"24705", NULL};	/*Profile ID - 24705*/


const Device UA4906 = {S154, 111, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; Media Center PC 4.0; SLCC1; .NET CLR 3.0.04320)*/
const Device UA92420 = {S155, 204, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA92690 = {S156, 63, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0)*/
const Device UA92847 = {S157, 68, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1)*/
const Device UA93274 = {S158, 50, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2)*/
const Device UA95104 = {S159, 180, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA97807 = {S160, 51, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Windows-RSS-Platform/2.0 (MSIE 8.0; Windows NT 5.1)*/
const Device UA98603 = {S161, 186, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA103155 = {S162, 244, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA107322 = {S163, 166, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA115943 = {S164, 151, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA116741 = {S165, 154, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA117540 = {S166, 247, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) ; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) (none))*/
const Device UA117804 = {S167, 198, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA119428 = {S168, 159, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA120233 = {S169, 238, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA123349 = {S170, 50, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1)*/
const Device UA127556 = {S171, 143, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; Media Center PC 2.8)*/
const Device UA132734 = {S172, 194, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA133651 = {S173, 210, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 8.0; Win32; GMX); (gmx/1.1.0.21); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA143535 = {S174, 214, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)*/
const Device UA144773 = {S175, 137, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)*/
const Device UA145237 = {S176, 196, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)*/
const Device UA146840 = {S177, 190, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)*/
const Device UA156763 = {S178, 171, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; 3M/MSIE 8.0)*/
const Device UA171022 = {S179, 51, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Windows-RSS-Platform/2.0 (MSIE 8.0; Windows NT 5.2)*/
const Device UA178738 = {S180, 214, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Tablet PC 1.7; Media Center PC 4.0; .NET4.0C; .NET4.0E; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322; InfoPath.3)*/
const Device UA179035 = {S181, 148, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; Media Center PC 2.8)*/
const Device UA273722 = {S182, 127, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)*/
const Device UA274801 = {S183, 192, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA274832 = {S184, 126, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) )*/
const Device UA277301 = {S185, 210, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 8.0; Win32; GMX); (gmx/1.1.0.21); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA293754 = {S186, 211, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0(Compatible Mozilla/4.0(Compatible-EmbeddedWB 14.59 http://bsalsa.com/ EmbeddedWB- 14.59  from: http://bsalsa.com/ ; .NET CLR 2.0.50727)*/
const Device UA313891 = {S187, 185, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; 3M/MSIE 8.0;; 3M/MSIE 8.0)*/
const Device UA366362 = {S188, 191, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; MSIE 8.0; AD1; Windows NT 5.1; Trident/4.0; AD1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; AD1)*/
const Device UA373633 = {S189, 140, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA374223 = {S190, 254, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)*/
const Device UA389765 = {S191, 197, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Win64; x64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA397246 = {S192, 50, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/6.0 (compatible; MSIE 8.0; Windows NT 5.1)*/
const Device UA443398 = {S193, 140, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Empty (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA483468 = {S194, 64, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible, MSIE 8.0; Windows NT 5.1; Trident/4.0;)*/
const Device UA495468 = {S195, 164, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; Media Center PC 3.0; Media Center PC 2.8)*/
const Device UA506803 = {S196, 61, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0.6001.18702; Windows NT 5.1)*/
const Device UA513422 = {S197, 62, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; SV1)*/
const Device UA513961 = {S198, 203, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0a; Windows NT; Getxoko Udala); InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA520209 = {S199, 168, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; Verizon Windows Live; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA520299 = {S200, 194, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; SRS MSIE 7.0; .NET4.0C; .NET4.0E; SRS MSIE 8.0)*/
const Device UA541229 = {S201, 117, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible ; MSIE 8.0.6001.18702 ; Microsoft Windows XP Professional Service Pack 3 ; Placeware RPC 1.0)*/
const Device UA570709 = {S202, 233, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 8.0; Win32; GMX); Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA580386 = {S203, 97, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)*/
const Device UA625342 = {S204, 267, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; Xerox_Win32PWS; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.2; MS-RTC LM 8; Xerox_Win32PWS)*/
const Device UA633015 = {S205, 76, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.1; SLCC1; .NET CLR 1.1.4322)*/
const Device UA659281 = {S206, 126, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; Verizon Windows Live; Verizon Windows Live)*/
const Device UA661463 = {S207, 191, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.2; 3M/MSIE 8.0)*/
const Device UA669685 = {S208, 286, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - mozilla/4.0 (compatible; msie 8.0; windows nt 5.1; trident/4.0; .net clr 1.0.3705; .net clr 1.1.4322; media center pc 4.0; .net clr 2.0.50727; .net clr 3.0.04506.30; .net clr 3.0.04506.648; .net clr 3.0.4506.2152; .net clr 3.5.30729; bri/2; officeliveconnector.1.5; officelivepatch.1.3)*/
const Device UA677076 = {S209, 210, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - mozilla/4.0 (compatible; msie 8.0; windows nt 5.1; trident/4.0; gtb6.5; .net clr 2.0.50727; .net clr 3.0.04506.648; .net clr 3.5.21022; .net clr 3.0.4506.2152; .net clr 3.5.30729; .net clr 1.1.4322; infopath.3)*/
const Device UA677248 = {S210, 163, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; MSIE 6.0 ( MSIE 7.0; MSIE 8.0; MSIE 9.0 )  Mozilla/3.0; Mozilla/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727)*/
const Device UA679270 = {S211, 122, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; Microsoft Windows Media Center PC 6.0)*/
const Device UA686117 = {S212, 242, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Tablet PC 1.7; .NET CLR 1.0.3705; MS-RTC LM 8)*/
const Device UA686355 = {S213, 191, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - mozilla/4.0 (compatible; msie 8.0; windows nt 5.1; trident/4.0; gtb7.3; .net clr 1.1.4322; .net clr 2.0.50727; .net clr 3.0.4506.2152; .net clr 3.5.30729; iph 1.1.21.4019; .net4.0c; .net4.0e)*/
const Device UA688512 = {S214, 180, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA708232 = {S215, 178, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA731932 = {S216, 250, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 8.0; Win32; GMX); (gmx/1.1.0.21); .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA735607 = {S217, 187, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; MCI Windows Corporate Image; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA750947 = {S218, 212, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8)*/
const Device UA751159 = {S219, 178, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 7.0; MyBee Browser); .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA806151 = {S220, 217, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - mozilla/4.0 (compatible; msie 8.0; windows nt 5.1; trident/4.0; searchtoolbar 1.2; gtb7.3; .net clr 1.1.4322; .net clr 2.0.50727; .net clr 3.0.04506.648; .net clr 3.5.21022; .net clr 3.0.4506.2152; .net clr 3.5.30729)*/
const Device UA814067 = {S221, 197, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; 3M/IE 6.0;; 3M/MSIE 6.0;; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; 3M/MSIE 8.0)*/
const Device UA845981 = {S222, 258, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (Compatible; MSIE 6.0;Windows NT;ASPI 1.0); .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Mozilla/4.0 (Compatible; MSIE 6.0;Windows NT;ASPI 1.0))*/
const Device UA900450 = {S223, 194, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; SRS MSIE 7.0; .NET4.0C; .NET4.0E; SRS MSIE 7.0)*/
const Device UA1836615 = {S224, 75, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Windows Live Writer 1.0)*/
const Device UA1838593 = {S225, 255, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Mozilla/4.0 (compatible; MSIE 7.0; MyBee Browser); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA1858388 = {S226, 59, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows; MSIE 8.0; Windows NT 5.1; InetLoader)*/
const Device UA1878949 = {S227, 217, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; Media Center PC 3.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)*/
const Device UA1881636 = {S228, 121, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/8.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/2.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1);)*/
const Device UA1896182 = {S229, 170, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Mozilla/4.0 (compatible; MSIE 6.0; Win32))*/
const Device UA1902306 = {S230, 185, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; 3M/MSIE 6.0;; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; 3M/MSIE 8.0)*/
const Device UA1919824 = {S231, 234, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSIE 6.0)*/
const Device UA1954287 = {S232, 266, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 7.0; Win32; WEB.DE); Mozilla/4.0 (compatible; MSIE 8.0; Win32; WEB.DE); (webde/1.1.0.21); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA1966347 = {S233, 158, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0)*/
const Device UA2039609 = {S234, 98, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; .NET CLR 2.0.50727; Zune 4.7)*/
const Device UA2127968 = {S235, 235, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; MSIE 6.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; InfoPath.1; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSIE 6.0; MSIE 6.0)*/
const Device UA2137138 = {S236, 180, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; SRS MSIE 7.0)*/
const Device UA2149081 = {S237, 195, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; MSIE 7.0; MSIE 8.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2155092 = {S238, 162, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2155609 = {S239, 70, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; MyIE2)*/
const Device UA2160015 = {S240, 151, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA2166252 = {S241, 68, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; SV1)*/
const Device UA2166285 = {S242, 68, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; SV1)*/
const Device UA2184804 = {S243, 177, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; AskTbPPC/5.15.4.23821)*/
const Device UA2203704 = {S244, 196, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; MSN 9.0;MSN 9.1;MSN 9.6;MSN 10.0;MSN 10.2;MSN 10.5; MSNbMSNI; MSNmen-us; MSNcOTH)*/
const Device UA2203708 = {S245, 188, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; MSN 9.0;MSN 9.1;MSN 9.6;MSN 10.0;MSN 10.2;MSN 10.5; MSNbMSNI; MSNmen-us; MSNcOTH)*/
const Device UA2208127 = {S246, 225, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; MSIE 6.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; InfoPath.1; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSIE 6.0)*/
const Device UA2216613 = {S247, 199, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Sicent1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Sicent1; Media Center PC 6.0; Microsoft Windows Media Center PC 6.0)*/
const Device UA2219555 = {S248, 135, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; Media Center PC 6.0; Microsoft Windows Media Center PC 6.0)*/
const Device UA2271546 = {S249, 54, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 ( ; MSIE 8.0; Windows NT 5.1; Trident/4.0)*/
const Device UA2273249 = {S250, 110, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; ARMXKD; .NET CLR 1.1.4322; .NET CLR 2.0.50727)*/
const Device UA2285771 = {S251, 158, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; MS-RTC LM 8; MSIE 7.0 (Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Talent2 Hong Kong)))*/
const Device UA2312863 = {S252, 132, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; MSIE 6.0)*/
const Device UA2332358 = {S253, 248, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.2; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; Verizon Windows Live; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; Verizon Windows Live)*/
const Device UA2336772 = {S254, 145, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; AskTbPPC/5.15.4.23821)*/
const Device UA2337432 = {S255, 70, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Win32; Trident/5.0)*/
const Device UA2339963 = {S256, 271, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0(Compatible Mozilla/4.0(Compatible-EmbeddedWB 14.59 http://bsalsa.com/ EmbeddedWB- 14.59  from: http://bsalsa.com/ ; Mozilla/4.0(Compatible Mozilla/4.0EmbeddedWB- 14.59  from: http://bsalsa.com/ )*/
const Device UA2349577 = {S257, 194, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; FunWebProducts; GTB7.4; Mozilla/4.0 (compatible; MSN 9.0;MSN 9.1;MSN 9.6;MSN 10.0;MSN 10.2;MSN 10.5; MSNbMSNI; MSNmen-us; MSNcOTH)*/
const Device UA2392203 = {S258, 251, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Internet Explorer (Windows 2k/XP, Child Support Enforcement); .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Internet Explorer (Windows 2k/XP, Child Support Enforcement))*/
const Device UA2394241 = {S259, 362, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; (webde/1.1.0.21); (1und1/1.1.0.22); Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1); Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; EasyBits GO v1.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; OfficeLiveConnector.1.5; OfficeLivePatch.1.3)*/
const Device UA2400799 = {S260, 242, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1); GTB7.4; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2412943 = {S261, 187, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0; Media Center PC 6.0)*/
const Device UA2423679 = {S262, 116, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - SINA_WEIBO; Mozilla/5.0 (Windows; U; Windows NT 5.1; MSIE8.0; zh-CN; rv:1.9.1.8) Gecko/X Firef8 (.NET CLR 3.5.30729)*/
const Device UA2429500 = {S263, 71, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.1) (http://name911.com)*/
const Device UA2431047 = {S264, 90, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows XP 5.1; MSIE 8.0.6001.18702)*/
const Device UA2433554 = {S265, 294, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; MRA 6.0 (build 6005); User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.1; .NET4.0C; .NET4.0E; MRIE8PACK 2.0.1)*/
const Device UA2439105 = {S266, 207, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1); Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1); Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1))*/
const Device UA2451812 = {S267, 196, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C; .NET4.0E; Zune 4.7)*/
const Device UA2486924 = {S268, 191, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Mozilla/4.0; compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2495472 = {S269, 265, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; Media Center PC 3.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2510422 = {S270, 87, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; QQBrowser/7.1.5753.400)*/
const Device UA2511222 = {S271, 92, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; QQBrowser/7.1.5753.400)*/
const Device UA2517723 = {S272, 202, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; mz1.9.1.4/xp (compatible;MSIE 6.0; Windows XP); .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; mz1.9.1.4/xp (compatible;MSIE 6.0; Windows XP))*/
const Device UA2521007 = {S273, 191, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Windows 98/2000/XP; BWCH-BWA-de-3.1; GTB7.4; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2545747 = {S274, 272, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0(Compatible Mozilla/4.0(Compatible-EmbeddedWB 14.58 http://bsalsa.com/ EmbeddedWB- 14.58  from: http://bsalsa.com/ ; Mozilla/4.0(Compatible RogueCleanerEmbeddedWB- 14.58  from: http://bsalsa.com/ )*/
const Device UA2574078 = {S275, 188, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7)*/
const Device UA2585994 = {S276, 87, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; QQBrowser/7.2.7006.400)*/
const Device UA2590173 = {S277, 232, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Mozilla/4.0 (compatible; MSIE 6.0; Win32); Mozilla/4.0 (compatible; MSIE 6.0; Win32))*/
const Device UA2605645 = {S278, 50, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0+(compatible;+MSIE+8.0;+Windows+NT+5.2)*/
const Device UA2607155 = {S279, 92, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; QQBrowser/7.2.7006.400)*/
const Device UA2650755 = {S280, 187, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; BRI/2; InfoPath.2; .NET4.0C)*/
const Device UA2687648 = {S281, 178, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2691336 = {S282, 70, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/5.0)*/
const Device UA2725834 = {S283, 234, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 3.0.30618; Media Center PC 5.0; SLCC1; .NET4.0C; Tablet PC 2.0; .NET4.0E)*/
const Device UA2726514 = {S284, 266, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; Trident/4.0; BTRSX; MRSPUTNIK 2, 4, 1, 44; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2742811 = {S285, 73, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; SV1; SV1)*/
const Device UA2743900 = {S286, 127, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; SV1; EmbeddedWB 14.52 from: http://www.bsalsa.com/ EmbeddedWB 14.52; Safari)*/
const Device UA2747795 = {S287, 87, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 5.2; MSIE 8.0.6001.18702)*/
const Device UA2789361 = {S288, 260, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - MOZNSLYCTBSVNEJQDJ (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; BOIE8;ENUS)*/
const Device UA2807615 = {S289, 217, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; EnableKerberos="Enable Integrated Windows Authentication")*/
const Device UA2807807 = {S290, 171, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; MSIE 6.0 ( MSIE 7.0; MSIE 8.0; MSIE 9.0 )  Mozilla/3.0; Mozilla/4.0)*/
const Device UA2811856 = {S291, 190, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; INFARMED, I.P.; .NET CLR 2.0.50727; INFARMED, I.P.)*/
const Device UA2844167 = {S292, 165, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; Trident/4.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0)*/
const Device UA2933643 = {S293, 73, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; SV1; SV1)*/
const Device UA2933758 = {S294, 73, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; SV1; SV1)*/
const Device UA2949948 = {S295, 73, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; SV1; SV1)*/
const Device UA2956723 = {S296, 282, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; BTRSX; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA2979008 = {S297, 213, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (user_patch_01; MSIE 8.0; Windows NT 5.1; Trident/4.0; pc_patch_01; pc_patch_04; pc_patch_02; pc_patch_03; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA3045864 = {S298, 300, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; IDS7 Agent; .NET4.0C; .NET4.0E; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT; pack2); InfoPath.3; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT; pack2))*/
const Device UA3069094 = {S299, 90, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (Compatible; Windows NT 5.1; MSIE 8.0) (compatible; MSIE 8.0; Windows NT 5.1;)*/
const Device UA3108754 = {S300, 38, { &P15364,&P5690,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (MSIE 8.0; Windows NT 5.1)*/
const Device UA10374 = {S301, 79, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible: MSIE 8.0; Windows NT 6.1; Trident/4.0; MooMooFroyo/2.2*/
const Device UA25414 = {S302, 50, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1)*/
const Device UA92180 = {S303, 151, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA95141 = {S304, 136, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - mozilla/4.0 (compatible; msie 8.0; windows nt 6.1; trident/4.0; gtb6.4; slcc2; .net clr 2.0.50727; .net clr 3.5.30729; .net clr 3.0.307)*/
const Device UA95262 = {S305, 75, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0)*/
const Device UA98147 = {S306, 166, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)*/
const Device UA99449 = {S307, 157, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA99995 = {S308, 63, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0)*/
const Device UA101313 = {S309, 198, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA102209 = {S310, 168, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Zune 3.0)*/
const Device UA102858 = {S311, 178, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA107328 = {S312, 214, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 3.0.04506; Media Center PC 5.0; SLCC1; Tablet PC 2.0)*/
const Device UA109124 = {S313, 226, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)*/
const Device UA112802 = {S314, 189, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Media Center PC 5.0; SLCC1)*/
const Device UA119365 = {S315, 231, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)*/
const Device UA122110 = {S316, 51, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Windows-RSS-Platform/2.0 (MSIE 8.0; Windows NT 6.1)*/
const Device UA122694 = {S317, 183, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA125318 = {S318, 162, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2; Tablet PC 2.0)*/
const Device UA138860 = {S319, 190, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; 3M/MSIE 8.0;; 3M/MSIE 8.0)*/
const Device UA140523 = {S320, 165, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA140582 = {S321, 211, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 3.0.04506; Media Center PC 5.0; SLCC1)*/
const Device UA142735 = {S322, 205, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Tablet PC 2.0; 3M/MSIE 8.0;; 3M/MSIE 8.0)*/
const Device UA143139 = {S323, 147, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2)*/
const Device UA146120 = {S324, 176, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; Zune 4.7)*/
const Device UA146665 = {S325, 224, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; GTB6.6; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Media Center PC 5.0; SLCC1; Tablet PC 2.0; .NET4.0C)*/
const Device UA148896 = {S326, 183, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7)*/
const Device UA150456 = {S327, 161, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Zune 4.7)*/
const Device UA151315 = {S328, 164, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2; Tablet PC 2.0)*/
const Device UA151399 = {S329, 246, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C)*/
const Device UA151975 = {S330, 246, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Tablet PC 2.0)*/
const Device UA153566 = {S331, 216, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA160901 = {S332, 201, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; .NET4.0C; .NET4.0E)*/
const Device UA163389 = {S333, 177, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA168799 = {S334, 215, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA176650 = {S335, 196, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C; .NET4.0E; 3M/MSIE 8.0)*/
const Device UA179266 = {S336, 175, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0(compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NETCLR3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA183398 = {S337, 174, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.1; Tridfent/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C)*/
const Device UA271076 = {S338, 208, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA282787 = {S339, 247, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Media Center PC 5.0; SLCC1)*/
const Device UA286859 = {S340, 190, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; 3M/MSIE 8.0;; 3M/MSIE 8.0)*/
const Device UA302232 = {S341, 203, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C; .NET4.0E; 3M/MSIE 8.0)*/
const Device UA305256 = {S342, 160, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Zune 4.7)*/
const Device UA339913 = {S343, 195, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; eSobiSubscriber 2.0.4.16; MAAR; .NET4.0C; AskTbPPC/5.13.1.18107)*/
const Device UA373592 = {S344, 208, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Zune 4.7)*/
const Device UA375051 = {S345, 211, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Tablet PC 2.0; .NET4.0C; .NET4.0E; 3M/MSIE 8.0)*/
const Device UA377752 = {S346, 181, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; 3M/MSIE 8.0)*/
const Device UA378688 = {S347, 228, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2))*/
const Device UA491199 = {S348, 61, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0.7600.16385; Windows NT 6.1)*/
const Device UA492877 = {S349, 39, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (MSIE 8.0; Windows NT 6.1;)*/
const Device UA514309 = {S350, 175, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Zune 4.7)*/
const Device UA517247 = {S351, 211, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Tablet PC 2.0; .NET4.0C; .NET4.0E; 3M/MSIE 8.0)*/
const Device UA518769 = {S352, 229, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7)*/
const Device UA541230 = {S353, 116, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible ; MSIE 8.0.7601.17514 ; Microsoft Windows 7 Professional Service Pack 1 ; Placeware RPC 1.0)*/
const Device UA541231 = {S354, 102, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible ; MSIE 8.0.7600.16385 ; Microsoft Windows 7 Professional  ; Placeware RPC 1.0)*/
const Device UA544673 = {S355, 188, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; SV1; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA559141 = {S356, 205, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; Tablet PC 2.0; .NET4.0E)*/
const Device UA568242 = {S357, 61, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0.7601.17514; Windows NT 6.1)*/
const Device UA570850 = {S358, 80, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Windows NT 6.1;)*/
const Device UA589282 = {S359, 182, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2; .NET4.0C; .NET4.0E; 3M/MSIE 8.0)*/
const Device UA682853 = {S360, 75, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Windows Live Writer 1.0)*/
const Device UA694634 = {S361, 197, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; MSIE 8.0; PAG-IE8; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA707289 = {S362, 223, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA716436 = {S363, 168, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SV1; [eburo v4.0]; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.3)*/
const Device UA717921 = {S364, 218, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3; Xerox_Win32PWS)*/
const Device UA719209 = {S365, 222, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Puesto Oficina Windows 7 Banesto)*/
const Device UA720141 = {S366, 217, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Puesto Oficina Windows 7 Banesto)*/
const Device UA734891 = {S367, 235, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2))*/
const Device UA738833 = {S368, 234, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; 3M/MSIE 8.0)*/
const Device UA740020 = {S369, 96, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; compatible with MSIE 8.0)*/
const Device UA751972 = {S370, 214, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 3.0.04506; Media Center PC 5.0; SLCC1; .NET4.0C)*/
const Device UA754903 = {S371, 211, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; .NET4.0C; Tablet PC 2.0; MSIE 7.0)*/
const Device UA757943 = {S372, 195, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; 3M/MSIE 8.0;; 3M/MSIE 8.0)*/
const Device UA761239 = {S373, 184, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2; .NET4.0C; .NET4.0E; Zune 4.7; NSN)*/
const Device UA762255 = {S374, 164, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA786231 = {S375, 220, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA816342 = {S376, 205, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Tablet PC 2.0; 3M/MSIE 8.0;; 3M/MSIE 8.0)*/
const Device UA819532 = {S377, 196, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C; .NET4.0E; 3M/MSIE 8.0)*/
const Device UA845513 = {S378, 219, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 3.0.04506; Media Center PC 5.0; SLCC1; Tablet PC 2.0)*/
const Device UA857357 = {S379, 206, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Windows Internet Explorer provided by ABB)*/
const Device UA879981 = {S380, 233, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; GTB6.5; QQDownload 534; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729)*/
const Device UA896058 = {S381, 338, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2))*/
const Device UA1873068 = {S382, 210, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Tablet PC 2.0; 3M/MSIE 8.0;; 3M/MSIE 8.0)*/
const Device UA1886576 = {S383, 172, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Zune 4.7)*/
const Device UA1891040 = {S384, 179, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; SV1; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3)*/
const Device UA1935419 = {S385, 249, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Tablet PC 2.0; 3M/MSIE 8.0)*/
const Device UA1966079 = {S386, 194, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; SV1; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0)*/
const Device UA1972514 = {S387, 335, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) ; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) (none); SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA1986353 = {S388, 68, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; SV1)*/
const Device UA1991124 = {S389, 345, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2))*/
const Device UA2138907 = {S390, 203, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; SV1; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA2145826 = {S391, 233, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Xerox_Win32PWS)*/
const Device UA2150709 = {S392, 133, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727)*/
const Device UA2154239 = {S393, 193, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MSIE 7.0)*/
const Device UA2156385 = {S394, 227, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1 SB.X.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MS-RTC LM 8; InfoPath.3; Active Content Browser)*/
const Device UA2168954 = {S395, 199, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; GTB7.4; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; AskTbPPC/5.15.4.23821)*/
const Device UA2221265 = {S396, 88, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; InfoPath.2; 3M/MSIE 8.0)*/
const Device UA2221314 = {S397, 166, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0)*/
const Device UA2254284 = {S398, 114, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; SV1; .NET CLR 2.0.50727; Microsoft Windows Media Center PC 6.0)*/
const Device UA2256254 = {S399, 250, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; Zune 4.0; .NET CLR 1.*/
const Device UA2258748 = {S400, 50, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.1)*/
const Device UA2267607 = {S401, 246, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Tablet PC 2.0; Zune 4.7)*/
const Device UA2272483 = {S402, 221, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Tablet PC 2.0; Windows Internet Explorer provided by ABB)*/
const Device UA2272848 = {S403, 217, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C; .NET4.0E; 3M/MSIE 8.0;; 3M/MSIE 8.0)*/
const Device UA2286215 = {S404, 73, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; MSIE 8.0)*/
const Device UA2298089 = {S405, 199, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; AskTbPPC/5.13.1.18107)*/
const Device UA2319006 = {S406, 195, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E)*/
const Device UA2319039 = {S407, 240, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2))*/
const Device UA2329329 = {S408, 354, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) ; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) (none); SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)*/
const Device UA2361161 = {S409, 122, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; GTB7.4; InfoPath.2; SV1; .NET CLR 3.3.69573; WOW64; en-US)*/
const Device UA2393206 = {S410, 324, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; GTB6.6; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Z/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; AskTbMPC2/5.12.2.16749)*/
const Device UA2397255 = {S411, 227, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; GTB7.4; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 3.0.30618; Media Center PC 5.0; SLCC1; .NET4.0C; Zune 4.7)*/
const Device UA2397579 = {S412, 258, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0(Compatible Mozilla/4.0(Compatible-EmbeddedWB 14.59 http://bsalsa.com/ EmbeddedWB- 14.59  from: http://bsalsa.com/ ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729)*/
const Device UA2409216 = {S413, 250, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (Compatible; MSIE 8.0; Windows NT 6.1; WOW64; SLCC1; .NET CLR 3.0.04506; Windows NT 6.1; Trident/4.0; Mozilla/4.0; MSIE 6.0; Windows NT 5.1; SV1 ; SLCC2; .NET CLR 2.0.50727; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NE*/
const Device UA2412933 = {S414, 286, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2413162 = {S415, 223, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3; Xerox_Win32PWS)*/
const Device UA2430280 = {S416, 87, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.1; MSIE 8.0.7601.17514)*/
const Device UA2431907 = {S417, 87, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.1; MSIE 8.0.7600.16385)*/
const Device UA2438283 = {S418, 190, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla:4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3)*/
const Device UA2478059 = {S419, 194, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; GTB7.4; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; AskTbPPC/5.15.4.23821)*/
const Device UA2493090 = {S420, 190, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible: MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3)*/
const Device UA2493485 = {S421, 217, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Media Center PC 3.1; SV1; WOW64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 4.0.20402; MS-RTC LM 8)*/
const Device UA2494509 = {S422, 350, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) ; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) (none); SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)*/
const Device UA2510873 = {S423, 175, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/7.1.5753.400)*/
const Device UA2536396 = {S424, 182, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/7.1.5753.400)*/
const Device UA2569839 = {S425, 129, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Mozilla/4.0 (compatiblt; MSIE 8.0; Windows NT; LCBSECURE))*/
const Device UA2574681 = {S426, 176, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; MSIE 8.0; PAG-IE8; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2576249 = {S427, 249, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7)*/
const Device UA2587398 = {S428, 196, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; 3M/MSIE 8.0)*/
const Device UA2588188 = {S429, 178, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SV1; [eburo v4.0]; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.3; Zune 4.7)*/
const Device UA2660022 = {S430, 154, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; QQBrowser/7.1.5753.400)*/
const Device UA2668027 = {S431, 175, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/7.2.7006.400)*/
const Device UA2668315 = {S432, 182, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/7.2.7006.400)*/
const Device UA2681304 = {S433, 183, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; Mozilla/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)*/
const Device UA2686060 = {S434, 206, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Zune 4.7; .NET4.0C; .NET4.0E; 3M/MSIE 8.0)*/
const Device UA2695234 = {S435, 308, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MDDC; Media Center PC 5.0; SLCC1; Tablet PC 2.0; .NET4.0C; .NET4.0E; AskTbCLM/5.9.1.14019)*/
const Device UA2726469 = {S436, 334, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) ; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) (none); SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2742950 = {S437, 311, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; MathPlayer 2.10b; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 3.5.21022; InfoPath.2; Media Center PC 5.0; MS-RTC LM 8; SLCC1; Tablet PC 2.0; Zune 4.0; InfoPath.3; .NET4.0C; .NET4.0E)*/
const Device UA2774783 = {S438, 236, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Mozilla/6.0 (compatible; MSIE 2.0; Windows Z; Shinigami))*/
const Device UA2775864 = {S439, 242, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1 SB.X.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MS-RTC LM 8; InfoPath.3; Tablet PC 2.0; Active Content Browser)*/
const Device UA2776079 = {S440, 73, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; SV1; SV1)*/
const Device UA2777041 = {S441, 228, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3; Zune 4.7; Xerox_Win32PWS)*/
const Device UA2779092 = {S442, 230, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Windows NT 6.1 Enterprise; .NET4.0C; .NET4.0E; InfoPath.2; MS-RTC LM 8)*/
const Device UA2780750 = {S443, 165, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Tablet PC 2.0; .NET4.0C; MSIE 6.0)*/
const Device UA2808560 = {S444, 187, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Zune 4.7)*/
const Device UA2810515 = {S445, 246, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; web-jl-13-76-45-21; web-jl-13-76-45-21; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MS-RTC LM 8; InfoPath.3)*/
const Device UA2812874 = {S446, 193, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (Compatible win32 69.124.0.167; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; AskTbTKR/5.13.2.19379)*/
const Device UA2814427 = {S447, 241, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (Compatible; MSIE 8.0; Windows NT 6.1; FDM; .NET CLR 1.1.4322; Windows NT 6.1; 001|Mozilla/4.0; MSIE 6.0; Windows NT 5.1; SV1 ; SLCC2; .NET CLR 2.0.50727; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.1.*/
const Device UA2842991 = {S448, 163, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; LG_UA; AD_LOGON=LGE.NET; SLCC2; Media Center PC 6.0; InfoPath.2; .NET4.0C; LG_UA; AD_LOGON=LGE.NET)*/
const Device UA2851256 = {S449, 190, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; QQBrowser/7.1.5753.400)*/
const Device UA2857543 = {S450, 157, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0+(compatible;+MSIE+8.0;+Windows+NT+6.1;+WOW64;+Trident/4.0;+SLCC2;+.NET+CLR+2.0.50727;+.NET+CLR+3.5.30729;+.NET+CLR+3.0.30729;+.NET4.0C;+.NET4.0E)*/
const Device UA2861257 = {S451, 358, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SIMBAR={5634CEA2-A21A-4CE0-B555-C46AEACA80C0}; MVH; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 3.0.30618; .NET CLR 3.5.21022; .NET4.0C; MDDC; Media Center PC 5.0; OfficeLiveConnector.1.3; OfficeLivePatch.0.0; SLCC1; Tablet PC 2.0; Zune 4.7)*/
const Device UA2948576 = {S452, 161, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.0.3705; .NET CLR 1.1.4322)*/
const Device UA2949515 = {S453, 170, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.30729; Media Center PC 6.0; InfoPath.3)*/
const Device UA2949963 = {S454, 289, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 3.5.21022; Media Center PC 5.0; SLCC1; .NET4.0C; .NET4.0E)*/
const Device UA2950126 = {S455, 73, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; SV1; SV1)*/
const Device UA2953209 = {S456, 134, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Mozilla/4.0 (compatiblt; MSIE 8.0; Windows NT; LCBSECURE))*/
const Device UA2980060 = {S457, 226, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SV1; [eburo v4.0]; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.3)*/
const Device UA2980376 = {S458, 240, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; SLCC1; Zune 4.7; InfoPath.3; .NET4.0C)*/
const Device UA2982015 = {S459, 216, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Tablet PC 2.0; .NET4.0C; .NET4.0E; 3M/MSIE 8.0)*/
const Device UA2982454 = {S460, 211, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; 3M/MSIE 8.0)*/
const Device UA3010245 = {S461, 253, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET4.0C; .NET4.0E; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MS-RTC LM 8; InfoPath.3; Mozilla/5.0)*/
const Device UA3024343 = {S462, 188, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Mozilla/4.0 (compatiblt; MSIE 8.0; Windows NT; LCBSECURE); Mozilla/4.0 (compatiblt; MSIE 9.0; Windows NT; LCBSECURE))*/
const Device UA3040624 = {S463, 223, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; LG_UA; AD_LOGON=LGE.NET; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C; LG_UA; AD_LOGON=LGE.NET)*/
const Device UA3041537 = {S464, 98, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; MSBrowserIE8; Windows NT 6.1; WOW64; Trident/5.0; MSBrowserIE9)*/
const Device UA3117790 = {S465, 70, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win32; Trident/5.0)*/
const Device UA3120808 = {S466, 109, { &P15364,&P7133,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; Windows NT 5.1; SV1)*/
const Device UA93983 = {S467, 126, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Win64; x64; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA95167 = {S468, 210, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; America Online Browser 1.1; rev1.5; Windows NT 5.1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; InfoPath.1; .NET CLR 2.0.50727; Media Center PC 3.0; InfoPath.2)*/
const Device UA97815 = {S469, 50, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1)*/
const Device UA98343 = {S470, 124, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2;  SLCC1;  .NET CLR 1.1.4322;  .NET CLR 2.0.50727;  .NET CLR 3.0.04506.648)*/
const Device UA98488 = {S471, 63, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0)*/
const Device UA98989 = {S472, 57, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/6.0 (compatible; MSIE 7.0a1; Windows NT 5.2; SV1)*/
const Device UA99617 = {S473, 77, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; .NET CLR 2.0.50727)*/
const Device UA100368 = {S474, 122, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2;  SV1;  .NET CLR 1.1.4322;  .NET CLR 2.0.40607;  .NET CLR 3.0.04506.648)*/
const Device UA101044 = {S475, 122, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1;  SV1;  .NET CLR 1.1.4325;  .NET CLR 2.0.40607;  .NET CLR 3.0.04506.648)*/
const Device UA101168 = {S476, 84, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (Mozilla/4.0; MSIE 7.0; Windows NT 5.1; FDM; SV1; .NET CLR 3.0.04506.30)*/
const Device UA103708 = {S477, 172, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA105219 = {S478, 72, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows; MSIE 7.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727)*/
const Device UA109614 = {S479, 244, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA113083 = {S480, 204, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA124206 = {S481, 51, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Windows-RSS-Platform/1.0 (MSIE 7.0; Windows NT 5.1)*/
const Device UA125466 = {S482, 168, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; SV1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA127296 = {S483, 50, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2)*/
const Device UA128492 = {S484, 109, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)*/
const Device UA131546 = {S485, 121, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA133535 = {S486, 58, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatiable; MSIE 7.0; Windows NT 5.1) HiScan*/
const Device UA135199 = {S487, 247, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) ; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) (none))*/
const Device UA136084 = {S488, 240, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows XP SP2; ERMC HQ); Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2)*/
const Device UA139007 = {S489, 167, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA149706 = {S490, 70, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatibile; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322)*/
const Device UA152117 = {S491, 196, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)*/
const Device UA155286 = {S492, 156, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)*/
const Device UA157922 = {S493, 226, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows XP SP2; ERMC HQ); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; .NET4.0C; .NET4.0E)*/
const Device UA165206 = {S494, 181, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; SRS MSIE 7.0; SRS MSIE 7.0)*/
const Device UA167648 = {S495, 233, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; MSIE 6.1 compatible; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.1; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA170367 = {S496, 47, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows 5.1)*/
const Device UA171346 = {S497, 113, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; Verizon Windows Live; Verizon Windows Live)*/
const Device UA183396 = {S498, 197, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; .NET CLR 1.1.4322; MS-RTC LM 8)*/
const Device UA240430 = {S499, 50, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; MSIE 7.0; Windows NT 5.2)*/
const Device UA258193 = {S500, 246, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible;MSIE 5.5;Windows NT; Customised to New Zealand Police Standards); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA264937 = {S501, 89, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727)*/
const Device UA296450 = {S502, 134, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727)*/
const Device UA300493 = {S503, 51, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1 )*/
const Device UA301384 = {S504, 178, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8; Xerox_Win32PWS; Xerox_Win32PWS)*/
const Device UA376752 = {S505, 132, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727); Windows NT 5.1; .NET CLR 2.0.50727)*/
const Device UA393750 = {S506, 57, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.2 (compatible; MSIE 7.0; Windows NT 5.2; en-US)*/
const Device UA399560 = {S507, 188, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; MCI Windows Corporate Image; .NET CLR 1.1.4322; .NET CLR 2.0.50727; MS-RTC LM 8; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA417745 = {S508, 211, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0(Compatible Mozilla/4.0(Compatible-EmbeddedWB 14.59 http://bsalsa.com/ EmbeddedWB- 14.59  from: http://bsalsa.com/ ; .NET CLR 2.0.50727)*/
const Device UA439152 = {S509, 194, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA442217 = {S510, 185, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA443070 = {S511, 50, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 5.1)*/
const Device UA449697 = {S512, 198, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.2; MSIE 6.0; .NET CLR 2.0.50727; MS-RTC LM 8; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; MSIE 6.0)*/
const Device UA467889 = {S513, 74, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1;.NET CLR 2.0.50727)*/
const Device UA469698 = {S514, 178, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 7.0; MyBee Browser); .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA473496 = {S515, 247, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT.5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30))*/
const Device UA473568 = {S516, 221, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1))*/
const Device UA482516 = {S517, 80, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; .NET CLR 1.0.3705; MSIE 7.0;)*/
const Device UA505068 = {S518, 64, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; local_Mobile)*/
const Device UA506875 = {S519, 46, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows XP)*/
const Device UA508074 = {S520, 167, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; SRS MSIE 7.0)*/
const Device UA515199 = {S521, 88, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727)*/
const Device UA529861 = {S522, 131, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Win64; x64; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA541232 = {S523, 114, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible ; MSIE 7.0.5730.13 ; Microsoft Windows XP Professional Service Pack 3 ; Placeware RPC 1.0)*/
const Device UA558562 = {S524, 200, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Mozilla/4.0 (compatible; MSIE 6.0; Win32); Mozilla/4.0 (compatible; MSIE 6.0; Win32))*/
const Device UA586293 = {S525, 97, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)*/
const Device UA641029 = {S526, 102, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Windows Live Messenger 14.0.8117.0416)*/
const Device UA643815 = {S527, 166, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0)*/
const Device UA651966 = {S528, 63, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/5.0)*/
const Device UA685467 = {S529, 62, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; WOW64; SV1)*/
const Device UA689305 = {S530, 57, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1.4; SV1)*/
const Device UA690908 = {S531, 148, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1) ; .NET CLR 3.0.04506.648; MEGAUPLOAD 2.0)*/
const Device UA704738 = {S532, 51, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Windows-RSS-Platform/1.0 (MSIE 7.0; Windows NT 5.2)*/
const Device UA715802 = {S533, 62, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/6.0 (compatible;MSIE 7.0;  Windows NT 5.1; QQDownload)*/
const Device UA731306 = {S534, 97, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; ARMXKD; .NET CLR 1.1.4322; .NET CLR 2.0.50727)*/
const Device UA739596 = {S535, 135, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; Media Center PC 2.8)*/
const Device UA742520 = {S536, 184, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Win64; x64; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA767650 = {S537, 118, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 5.2; Media Center PC 4.0; .NET CLR 3.0.4506.2152; .NET CLR 3.0.04506.30)*/
const Device UA771639 = {S538, 58, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/3.0 (compatible;MSIE 7.0;  Windows NT 5.1;  MyIE2)*/
const Device UA783580 = {S539, 228, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Crazy Browser 3.0.0 Beta2) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; Zune 2.5)*/
const Device UA798168 = {S540, 243, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Win32); .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Mozilla/4.0 (compatible; MSIE 6.0; Win32); Mozilla/4.0 (compatible; MSIE 6.0; Win32))*/
const Device UA806170 = {S541, 168, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; MS-RTC LM 8; SV1)*/
const Device UA808246 = {S542, 225, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA816128 = {S543, 263, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; MSBrowserIE7; Windows NT 5.1; msie6xpv1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8; .NET4.0C; .NET4.0E; MSIE 6.0; dmx:m; mty:p; slx:ik.hk.as; ssx:NonCitrix)*/
const Device UA826162 = {S544, 57, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; MSIE 7.0; Windows NT 5.1; en-US)*/
const Device UA889881 = {S545, 50, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.5; Windows NT 5.1)*/
const Device UA902191 = {S546, 264, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ;  Embedded Web Browser from: http://bsalsa.com/; .NET CLR 1.1.4322; .NET4.0C; .NET CLR 2.0.50727; .NET4.0E; .NET CLR 3.5.30729; .NET CLR 3.0.4506.2152)*/
const Device UA1837211 = {S547, 113, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; Mozilla/4.0 (compatible; MSIE 6.0; Win32))*/
const Device UA1841811 = {S548, 179, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA1856613 = {S549, 170, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; Xerox_Win32PWS; Xerox_Win32PWS)*/
const Device UA1887100 = {S550, 121, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/7.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/1.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1);)*/
const Device UA1890791 = {S551, 253, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Embedded Web Browser from: http://bsalsa.com/; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; Zune 2.5; .NET CLR*/
const Device UA1949297 = {S552, 173, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; 360SE)*/
const Device UA1952675 = {S553, 94, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; Microsoft Windows Media Center PC 6.0)*/
const Device UA2127683 = {S554, 186, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1); .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2155776 = {S555, 215, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Cobbetts LLP Citrix; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.1)*/
const Device UA2166719 = {S556, 68, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 5.2; SV1)*/
const Device UA2207749 = {S557, 63, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0)*/
const Device UA2211988 = {S558, 177, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; INFARMED, I.P.; INFARMED, I.P.)*/
const Device UA2243753 = {S559, 224, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SearchToolbar 1.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; AskTbNSC-N/5.12.2.16749)*/
const Device UA2244751 = {S560, 188, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; MSN 9.0;MSN 9.1;MSN 9.6;MSN 10.0;MSN 10.2;MSN 10.5; MSNbMSNI; MSNmen-us; MSNcOTH)*/
const Device UA2314616 = {S561, 146, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; FunWebProducts; GTB7.2; FunWebProducts; AskTbPPC/5.13.1.18107; .NET CLR 2.0.50727)*/
const Device UA2318144 = {S562, 179, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Media Center PC 6.0; Microsoft Windows Media Center PC 6.0)*/
const Device UA2356962 = {S563, 33, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - MSIE 7.0; Windows 5.1; compatible*/
const Device UA2371911 = {S564, 220, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; IEMB3; GTB7.2; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; MS-RTC EA 2; AskTbPPC/5.12.5.17640; BRI/2; .NET CLR 3.5.30729)*/
const Device UA2392600 = {S565, 187, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0; Media Center PC 6.0)*/
const Device UA2412126 = {S566, 117, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; SV1; .NET CLR 2.0.50727; Zune 4.7; SE 2.X MetaSr 1.0)*/
const Device UA2419011 = {S567, 196, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; MSN 9.0;MSN 9.1;MSN 9.6;MSN 10.0;MSN 10.2;MSN 10.5; MSNbMSNI; MSNmen-us; MSNcOTH)*/
const Device UA2433303 = {S568, 87, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows XP 5.1; MSIE 7.0.5730.13)*/
const Device UA2462628 = {S569, 242, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; eSobiSubscriber 2.0.4.16; InfoPath.2; OfficeLiveConnector.1.5; OfficeLivePatch.1.3)*/
const Device UA2502469 = {S570, 151, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - ('Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30)', 'Internet Explorer 7, Windows XP')*/
const Device UA2514708 = {S571, 87, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; QQBrowser/7.1.5753.400)*/
const Device UA2517511 = {S572, 199, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; SIMBAR={81E3F033-E6B1-4DC5-82DD-4B34FCB2BBE8}; Mozilla/4.0 (compatible; MSN 9.0;MSN 9.1;MSN 9.6;MSN 10.0; MSNbQ002; MSNmen-us; MSNcOTH)*/
const Device UA2540711 = {S573, 58, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0.5730.13; Windows NT 5.1)*/
const Device UA2597388 = {S574, 92, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; SV1; QQBrowser/7.1.5753.400)*/
const Device UA2667944 = {S575, 87, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; QQBrowser/7.2.7006.400)*/
const Device UA2706105 = {S576, 96, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; Windows NT 5.1; SV1)*/
const Device UA2720311 = {S577, 172, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; Verizon Windows Live; Verizon Windows Live)*/
const Device UA2754805 = {S578, 123, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Win64; x64; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA2801475 = {S579, 155, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; GoogleT5; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2; Zune 2.5)*/
const Device UA2804382 = {S580, 231, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.0.3705; .NET CLR 2.0.50727; Media Center PC 4.0; .NET CLR 3.0.04506.30; .NET CLR 1.1.4322; Media Center PC 3.0)*/
const Device UA2817913 = {S581, 91, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; Safari)*/
const Device UA2832344 = {S582, 188, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7)*/
const Device UA2845570 = {S583, 260, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1); GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2883682 = {S584, 132, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1;  SV1; Zune 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; FDM)*/
const Device UA2899266 = {S585, 129, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64;  SV1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; FDM)*/
const Device UA2932990 = {S586, 69, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compat?vel; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322)*/
const Device UA2973607 = {S587, 208, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.21022; Media Center PC 6.0; InfoPath.2; OfficeLiveConnector.1.5; OfficeLivePatch.1.3)*/
const Device UA2982600 = {S588, 276, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; WOW64; Trident/4.0; MRA 5.10 (build 5339; SE 2.x); Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET CLR 1.1.4322)*/
const Device UA3041785 = {S589, 199, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Sicent1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Sicent1; Media Center PC 6.0; Microsoft Windows Media Center PC 6.0)*/
const Device UA3075125 = {S590, 91, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 4.0.1306.3130-big; Windows XP 5.1; MSIE 7.0.5730.11)*/
const Device UA3116182 = {S591, 156, { &P15364,&P5690,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2;  SV1; GTB6.5; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3)*/
const Device UA129075 = {S592, 50, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1)*/
const Device UA135093 = {S593, 209, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA136936 = {S594, 178, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA137264 = {S595, 67, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; XBLWP7; ZuneWP7)*/
const Device UA137804 = {S596, 171, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA138807 = {S597, 176, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0)*/
const Device UA141321 = {S598, 183, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA142528 = {S599, 157, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA145276 = {S600, 171, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7)*/
const Device UA157126 = {S601, 178, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7)*/
const Device UA160570 = {S602, 189, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Media Center PC 5.0; SLCC1)*/
const Device UA161525 = {S603, 183, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7)*/
const Device UA163106 = {S604, 186, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Zune 4.7)*/
const Device UA164518 = {S605, 165, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA166734 = {S606, 201, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Media Center PC 5.0; SLCC1; .NET4.0C)*/
const Device UA170037 = {S607, 216, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Media Center PC 5.0; SLCC1; Tablet PC 2.0; .NET4.0C)*/
const Device UA170398 = {S608, 217, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Windows Live Messenger 15.4.3508.1109)*/
const Device UA174309 = {S609, 198, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Zune 4.7)*/
const Device UA177649 = {S610, 204, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Media Center PC 5.0; SLCC1; Tablet PC 2.0)*/
const Device UA189337 = {S611, 198, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA204448 = {S612, 63, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0)*/
const Device UA212865 = {S613, 215, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Windows Live Messenger 15.4.3538.0513)*/
const Device UA216690 = {S614, 207, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Windows Live Messenger 15.4.3538.0513)*/
const Device UA221306 = {S615, 200, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Windows Live Messenger 15.4.3538.0513)*/
const Device UA275362 = {S616, 236, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA284501 = {S617, 231, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)*/
const Device UA308155 = {S618, 63, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0)*/
const Device UA379388 = {S619, 95, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET4.0C; .NET4.0E)*/
const Device UA395165 = {S620, 167, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3)*/
const Device UA420362 = {S621, 174, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/4.0; SV1; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; SLCC1)*/
const Device UA433969 = {S622, 70, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0)*/
const Device UA445010 = {S623, 162, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2; Tablet PC 2.0)*/
const Device UA452172 = {S624, 186, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Windows Live Messenger 15.4.3538.0513)*/
const Device UA483746 = {S625, 209, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Windows Live Messenger 14.0.8117.0416)*/
const Device UA490991 = {S626, 207, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Windows Live Messenger 15.4.3502.0922)*/
const Device UA515396 = {S627, 164, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2; Tablet PC 2.0)*/
const Device UA515753 = {S628, 51, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.2 (compatible; MSIE 7.01; Windows NT 6.1)*/
const Device UA517125 = {S629, 215, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Windows Live Messenger 15.4.3508.1109)*/
const Device UA518841 = {S630, 186, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Windows Live Messenger 15.4.3508.1109)*/
const Device UA524023 = {S631, 217, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; BRI/2; Windows Live Messenger 15.4.3508.1109)*/
const Device UA526642 = {S632, 234, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Tablet PC 2.0; Media Center PC 6.0; .NET4.0C)*/
const Device UA530584 = {S633, 177, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA542917 = {S634, 214, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Media Center PC 5.0; SLCC1; Tablet PC 2.0; .NET4.0C; Zune 4.7)*/
const Device UA568029 = {S635, 216, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MS-RTC LM 8; Tablet PC 2.0)*/
const Device UA604996 = {S636, 215, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Windows Live Messenger 15.4.3555.0308)*/
const Device UA684609 = {S637, 167, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Zune 4.7)*/
const Device UA688331 = {S638, 248, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ;  Embedded Web Browser from: http://bsalsa.com/; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2)*/
const Device UA695019 = {S639, 200, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Windows Live Messenger 15.4.3555.0308)*/
const Device UA702664 = {S640, 217, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Windows Live Messenger 15.4.3555.0308)*/
const Device UA736054 = {S641, 218, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; eSobiSubscriber 1.0.0.40; BRI/2; MAAR; .NET4.0C; Windows Live Messenger 15.4.3538.0513)*/
const Device UA791393 = {S642, 200, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.2; MS-RTC LM 8; MSOffice 12)*/
const Device UA793651 = {S643, 196, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Windows Live Messenger 15.4.3555.0308)*/
const Device UA803508 = {S644, 215, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA828259 = {S645, 207, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; InfoPath.3; .NET4.0C; .NET4.0E)*/
const Device UA831805 = {S646, 227, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; Windows Live Messenger 15.4.3555.0308)*/
const Device UA852809 = {S647, 179, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Windows Live Messenger 15.4.3555.0308)*/
const Device UA855053 = {S648, 193, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MSIE 7.0)*/
const Device UA855420 = {S649, 200, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Windows Live Messenger 15.4.3502.0922)*/
const Device UA872197 = {S650, 213, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows XP SP2; ERMC HQ); SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.2)*/
const Device UA894308 = {S651, 88, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0(compatible;MSIE 7.0;Windows NT 6.1;SV1;.NET CLR 1.0.3705;.NET CLR 3.0.30618)*/
const Device UA1845547 = {S652, 205, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; Windows Live Messenger 15.4.3502.0922)*/
const Device UA1853520 = {S653, 185, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0); QQBrowser/5.0.6642.400 (trident)*/
const Device UA1894156 = {S654, 73, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; MSIE 7.0)*/
const Device UA1924416 = {S655, 171, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; MSIE 6.0)*/
const Device UA1934284 = {S656, 199, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; GTB7.2; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; AskTbPPC/5.12.2.16749)*/
const Device UA1942246 = {S657, 230, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Windows NT 6.1 Enterprise; .NET4.0C; .NET4.0E; InfoPath.2; MS-RTC LM 8)*/
const Device UA2006200 = {S658, 217, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Windows Live Messenger 16.4.3503.0728)*/
const Device UA2144969 = {S659, 184, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; AskTbPPC/5.13.1.18107)*/
const Device UA2153659 = {S660, 191, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA2156196 = {S661, 208, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Tablet PC 2.0; MSIE 7.0)*/
const Device UA2156482 = {S662, 194, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Tablet PC 2.0; Windows Live Messenger 15.4.3555.0308)*/
const Device UA2160335 = {S663, 68, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 6.1; SV1)*/
const Device UA2176415 = {S664, 210, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Windows Live Messenger 16.4.3503.0728)*/
const Device UA2209114 = {S665, 246, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C)*/
const Device UA2220089 = {S666, 166, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0)*/
const Device UA2230879 = {S667, 217, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Windows Live Messenger 16.4.3505.0912)*/
const Device UA2245574 = {S668, 218, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3; Xerox_Win32PWS)*/
const Device UA2268807 = {S669, 190, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Windows Live Messenger 14.0.8117.0416)*/
const Device UA2296989 = {S670, 198, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; BRI/2; NP07; .NET4.0C; NP07; Windows Live Messenger 15.4.3508.1109)*/
const Device UA2321455 = {S671, 228, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows XP SP2; ERMC HQ); SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.2; Tablet PC 2.0)*/
const Device UA2336289 = {S672, 210, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7; Windows Live Messenger 15.4.3555.0308)*/
const Device UA2355197 = {S673, 210, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Windows Live Messenger 16.4.3505.0912)*/
const Device UA2393486 = {S674, 188, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Zune 4.7; MS-RTC LM 8; Tablet PC 2.0)*/
const Device UA2420607 = {S675, 196, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Windows Live Messenger 16.4.3505.0912)*/
const Device UA2438329 = {S676, 75, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0)*/
const Device UA2495700 = {S677, 237, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 ( ; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA2536400 = {S678, 175, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/7.1.5753.400)*/
const Device UA2542226 = {S679, 188, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/4.0; SV1; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2553105 = {S680, 227, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; Windows Live Messenger 16.4.3505.0912)*/
const Device UA2553571 = {S681, 225, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Windows Live Messenger 16.4.3505.0912)*/
const Device UA2572766 = {S682, 172, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Zune 4.7)*/
const Device UA2580740 = {S683, 192, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; QQBrowser/7.1.5753.400)*/
const Device UA2639384 = {S684, 202, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; MathPlayer 2.20; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; Tablet PC 2.0)*/
const Device UA2640429 = {S685, 137, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0  (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media*/
const Device UA2690832 = {S686, 186, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.2); QQBrowser/5.1.7061.201 (trident)*/
const Device UA2722124 = {S687, 259, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; QQDownload 675; Mozilla/4.0(Compatible Mozilla/4.0(Compatible-EmbeddedWB 14.59 http://bsalsa.com/ EmbeddedWB- 14.59  from: http://bsalsa.com/ ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET*/
const Device UA2808204 = {S688, 190, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Windows Media Center PC 6.0)*/
const Device UA2934120 = {S689, 199, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Media Center PC 5.0; SLCC1; Zune 4.7)*/
const Device UA3072358 = {S690, 163, { &P15364,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; MCSC-CL-001)*/
const Device UA92827 = {S691, 63, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0)*/
const Device UA95081 = {S692, 175, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SV1; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 1.1.4322; .NET CLR 3.5.30729; .NET CLR 3.0.30618)*/
const Device UA98143 = {S693, 51, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Windows-RSS-Platform/2.0 (MSIE 8.0; Windows NT 6.0)*/
const Device UA98337 = {S694, 50, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0)*/
const Device UA106496 = {S695, 161, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA112709 = {S696, 173, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)*/
const Device UA115269 = {S697, 157, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA123821 = {S698, 166, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)*/
const Device UA126602 = {S699, 216, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729)*/
const Device UA129453 = {S700, 183, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Verizon Windows Live)*/
const Device UA136704 = {S701, 141, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 3.5.30729)*/
const Device UA140108 = {S702, 63, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/5.0)*/
const Device UA145509 = {S703, 178, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C; Zune 4.7)*/
const Device UA146611 = {S704, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 5.0.2124.2070; Windows 6.0; MSIE 8.0.6001.18241)*/
const Device UA151873 = {S705, 171, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C; Zune 4.7)*/
const Device UA158258 = {S706, 150, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C; Zune 4.7)*/
const Device UA165052 = {S707, 199, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7)*/
const Device UA210409 = {S708, 69, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE8.0; Windows NT 6.0) .NET CLR 2.0.50727)*/
const Device UA239851 = {S709, 198, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; DI60SP1001; SV1; SLCC1; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 3.0.30729; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; .NET CLR 1.1.4322)*/
const Device UA251307 = {S710, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 6.4.1321.1732; Windows 6.0; MSIE 8.0.6001.19154)*/
const Device UA263147 = {S711, 75, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Win64; x64; Trident/4.0)*/
const Device UA278226 = {S712, 202, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; Mozilla/4.0 (compatible; MSIE 8.0; Win32; WEB.DE); SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)*/
const Device UA299301 = {S713, 183, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7)*/
const Device UA334474 = {S714, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.1.2003.1856; Windows 6.0; MSIE 8.0.6001.19120)*/
const Device UA358133 = {S715, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 6.4.1321.1732; Windows 6.0; MSIE 8.0.6001.19170)*/
const Device UA397056 = {S716, 188, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.0.3705; .NET CLR 1.1.4322)*/
const Device UA414981 = {S717, 47, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows; MSIE 8.0; Windows NT 6.0)*/
const Device UA422355 = {S718, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 6.4.1321.1732; Windows 6.0; MSIE 8.0.6001.19190)*/
const Device UA493457 = {S719, 224, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)*/
const Device UA515827 = {S720, 163, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.30729; MDDC; .NET4.0C; Zune 4.7)*/
const Device UA533894 = {S721, 236, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7)*/
const Device UA568887 = {S722, 157, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC1; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)*/
const Device UA569828 = {S723, 241, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; InfoPath.2; .NET CLR 3.0.30729; Zune 4.0; MS-RTC LM 8; MSN Optimized;US)*/
const Device UA605303 = {S724, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 6.4.1321.1732; Windows 6.0; MSIE 8.0.6001.19222)*/
const Device UA623394 = {S725, 50, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0)*/
const Device UA628180 = {S726, 157, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; .NET CLR 2.0.50727; InfoPath.2)Mozilla/5.0 (Windows; U; MSIE 7.0; Windows NT 6.0; en-US)*/
const Device UA632842 = {S727, 165, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; .NET CLR 3.5.21022; .NET CLR 3.5.30729)*/
const Device UA641477 = {S728, 205, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - mozilla/4.0 (compatible; msie 8.0; windows nt 6.0; trident/4.0; gtb7.2; slcc1; .net clr 2.0.50727; media center pc 5.0; .net clr 3.5.30729; .net clr 3.0.30618; officeliveconnector.1.5; officelivepatch.1.3)*/
const Device UA651095 = {S729, 54, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; MSIE 8.0; Windows NT 6.0; en-US)*/
const Device UA714161 = {S730, 49, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0*/
const Device UA726454 = {S731, 160, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; MSIE 6.4; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; MS-RTC LM 8; .NET CLR 3.0.30729)*/
const Device UA777573 = {S732, 155, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.30729; InfoPath.2; 3M/MSIE 8.0)*/
const Device UA832411 = {S733, 229, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C; Zune 4.7)*/
const Device UA864659 = {S734, 86, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 6.3.1106.427; Windows 6.0; MSIE 8.0.6001.19048)*/
const Device UA898518 = {S735, 114, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; InfoPath.1; SV1; .NET CLR 3.8.36217; WOW64; en-US)*/
const Device UA903652 = {S736, 86, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.3.2710.138; Windows 6.0; MSIE 8.0.6001.19272)*/
const Device UA1955163 = {S737, 148, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; .NET CLR 2.7.58687; SLCC2; Media Center PC 5.0; Zune 3.4; Tablet PC 3.6; InfoPath.3)*/
const Device UA2000272 = {S738, 176, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.4; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2003621 = {S739, 197, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.4; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2004541 = {S740, 227, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2165599 = {S741, 68, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; SV1)*/
const Device UA2181639 = {S742, 239, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Win64; x64; Trident/4.0; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)*/
const Device UA2183504 = {S743, 255, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2194241 = {S744, 213, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2196507 = {S745, 192, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C; AskTbPPC/5.15.4.23821)*/
const Device UA2197325 = {S746, 216, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Win64; x64; Trident/4.0; GTB7.4; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; Tablet PC 2.0; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2198534 = {S747, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3230.2052; Windows 6.0; MSIE 8.0.6001.19298)*/
const Device UA2200269 = {S748, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3230.2052; Windows 6.0; MSIE 8.0.6001.18904)*/
const Device UA2204237 = {S749, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3230.2052; Windows 6.0; MSIE 8.0.6001.19328)*/
const Device UA2223859 = {S750, 221, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GIL 3.5; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.21022; .NET CLR 3.5.30729; InfoPath.2; .NET4.0C; .NET4.0E; Windows Server Design)*/
const Device UA2254190 = {S751, 250, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB6.6; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; OfficeLiveConnect*/
const Device UA2273044 = {S752, 152, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727)*/
const Device UA2283996 = {S753, 49, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT6.0)*/
const Device UA2311923 = {S754, 225, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.2; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.21022; .NET CLR 3.5.30729; MDDC; .NET4.0C; .NET CLR 3.0.30729; AskTbPPC/5.14.1.20007)*/
const Device UA2370085 = {S755, 191, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; Mozilla/4.0 (compatible; MSIE 8.0; Win32; WEB.DE); SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2400092 = {S756, 171, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C; AskTbPPC/5.15.4.23821)*/
const Device UA2430071 = {S757, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 8.0.6001.19393)*/
const Device UA2436217 = {S758, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 8.0.6001.19088)*/
const Device UA2439179 = {S759, 61, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0.6001.19088; Windows NT 6.0)*/
const Device UA2440366 = {S760, 221, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; GTB7.2; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)(Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Alexa Toobar)); Alexa Toolbar)*/
const Device UA2454118 = {S761, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3230.2052; Windows 6.0; MSIE 8.0.6001.18999)*/
const Device UA2462556 = {S762, 82, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 ( ; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB6.6; .NET CLR 3.5.30729)*/
const Device UA2486831 = {S763, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 8.0.6001.18882)*/
const Device UA2501917 = {S764, 218, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Win64; x64; Trident/4.0; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; SLCC1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2516652 = {S765, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 8.0.6001.19400)*/
const Device UA2569824 = {S766, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 8.0.6001.19401)*/
const Device UA2676892 = {S767, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 8.0.6001.19412)*/
const Device UA2689487 = {S768, 220, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; Mozilla/4.0 (compatible; MSIE 7.0; MyBee Browser); SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2693809 = {S769, 196, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; MSN 9.0;MSN 9.1;MSN 9.6;MSN 10.0;MSN 10.2;MSN 10.5; MSNbMSNI; MSNmen-us; MSNcOTH)*/
const Device UA2696574 = {S770, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 8.0.6001.18928)*/
const Device UA2697423 = {S771, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 8.0.6001.18702)*/
const Device UA2698193 = {S772, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 8.0.6001.19019)*/
const Device UA2728698 = {S773, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 8.0.6001.18975)*/
const Device UA2741539 = {S774, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 8.0.6001.19418)*/
const Device UA2760844 = {S775, 194, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C; Zune 4.7)*/
const Device UA2817776 = {S776, 73, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; SV1; SV1)*/
const Device UA2870752 = {S777, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.5.4209.2358; Windows 6.0; MSIE 8.0.6001.19437)*/
const Device UA2941526 = {S778, 139, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; Mozilla/4.0; compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 3.5.30729)*/
const Device UA2948342 = {S779, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.5.4209.2358; Windows 6.0; MSIE 8.0.6001.19443)*/
const Device UA2982163 = {S780, 73, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; SV1; SV1)*/
const Device UA2984255 = {S781, 122, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; InfoPath.1; SV1; .NET CLR 4.8.88265; WOW64; en-US)*/
const Device UA3031465 = {S782, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.5.4413.1752; Windows 6.0; MSIE 8.0.6001.19453)*/
const Device UA3073498 = {S783, 173, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C)*/
const Device UA3084957 = {S784, 403, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.0.3705; .NET CLR 1.1.4322))*/
const Device UA3095399 = {S785, 87, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.5.4413.1752; Windows 6.0; MSIE 8.0.6001.19458)*/
const Device UA3098616 = {S786, 322, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1); Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1); Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0))*/
const Device UA3124585 = {S787, 49, { &P15364,&P17285,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (comptible; MSIE 8.0; Windows NT 6.0)*/
const Device UA51271 = {S788, 63, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/4.0)*/
const Device UA93505 = {S789, 50, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)*/
const Device UA94047 = {S790, 57, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; MSIE 7.0; Windows NT 6.0; en-US)*/
const Device UA95385 = {S791, 50, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.0)*/
const Device UA97973 = {S792, 138, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)*/
const Device UA98483 = {S793, 111, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0b; Windows NT 6.0 ; .NET CLR 2.0.50215; SL Commerce Client v1.0; Tablet PC 2.0*/
const Device UA98562 = {S794, 57, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.0; en-US)*/
const Device UA103659 = {S795, 143, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SV1; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)*/
const Device UA108252 = {S796, 155, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA116536 = {S797, 172, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SV1; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 3.5.30729; InfoPath.2)*/
const Device UA122629 = {S798, 133, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; Tablet PC 2.0)*/
const Device UA130630 = {S799, 84, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; SLCC1; .NET CLR 3.0.04506)*/
const Device UA130968 = {S800, 51, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Windows-RSS-Platform/1.0 (MSIE 7.0; Windows NT 6.0)*/
const Device UA131031 = {S801, 129, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Media Center PC 5.0)*/
const Device UA138617 = {S802, 173, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)*/
const Device UA142690 = {S803, 201, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.30618; .NET CLR 3.5.30729; InfoPath.2; Mozilla/4.0 (compatible; MSIE 7.0b; Windows NT 6.0))*/
const Device UA160674 = {S804, 208, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; Tablet PC 2.0; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)*/
const Device UA164421 = {S805, 171, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7)*/
const Device UA166062 = {S806, 166, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; c .NET CLR 3.0.04506; .NET CLR 3.5.30707; InfoPath.1; el-GR)*/
const Device UA167889 = {S807, 178, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7)*/
const Device UA172141 = {S808, 200, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Windows Live Messenger 15.4.3508.1109)*/
const Device UA178083 = {S809, 150, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7)*/
const Device UA185731 = {S810, 215, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; InfoPath.2; Mozilla/4.0 (compatible; MSIE 7.0b; Windows NT 6.0); .NET CLR 3.0.30729; Mozilla/4.0 (compatible; MSIE 7.0b; Windows NT 6.0))*/
const Device UA186301 = {S811, 183, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7)*/
const Device UA188272 = {S812, 152, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; c .NET CLR 3.0.04506; .NET CLR 3.5.30707; InfoPath.1)*/
const Device UA230117 = {S813, 179, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Windows Live Messenger 15.4.3538.0513)*/
const Device UA282848 = {S814, 48, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible MSIE 7.0 Windows NT 6.0)*/
const Device UA332680 = {S815, 200, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Windows Live Messenger 15.4.3538.0513)*/
const Device UA371480 = {S816, 50, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; MSIE 7.0; Windows NT 6.0)*/
const Device UA394304 = {S817, 182, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC1; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA409339 = {S818, 47, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows; MSIE 7.0; Windows NT 6.0)*/
const Device UA443216 = {S819, 163, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; Zune 4.7)*/
const Device UA509692 = {S820, 191, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.04506; .NET CLR 1.1.4322; InfoPath.2)*/
const Device UA514162 = {S821, 245, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Mozilla/4.0(Compatible Mozilla/4.0(Compatible-EmbeddedWB 14.59 http://bsalsa.com/ EmbeddedWB- 14.59  from: http://bsalsa.com/ ; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618)*/
const Device UA543691 = {S822, 199, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7)*/
const Device UA551420 = {S823, 59, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/3.0 (compatible;MSIE 7.0;  Windows NT 6.0; 360safe)*/
const Device UA703641 = {S824, 48, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla /4.0 (compatible;MSIE 7.0;Windows NT6.0)*/
const Device UA708734 = {S825, 50, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0(compatible; MSIE 7.0b; Windows NT 6.0)*/
const Device UA748912 = {S826, 58, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/6.0 (compatible;MSIE 7.0;  Windows NT 6.0;  MyIE2)*/
const Device UA765016 = {S827, 222, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SV1; SU 3.011; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; .NET CLR 1.1.4322; InfoPath.1)*/
const Device UA778222 = {S828, 191, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; Tablet PC 2.0)*/
const Device UA803868 = {S829, 200, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Windows Live Messenger 15.4.3555.0308)*/
const Device UA863717 = {S830, 179, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Windows Live Messenger 15.4.3555.0308)*/
const Device UA881363 = {S831, 229, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Comcast Install 1.0; FunWebProducts; GTB5; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; Zune 3.0)*/
const Device UA891817 = {S832, 179, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Windows Live Messenger 15.4.3508.1109)*/
const Device UA1850588 = {S833, 215, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET CLR 3.0.30618)*/
const Device UA1877576 = {S834, 155, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Empty (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)*/
const Device UA1895354 = {S835, 165, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SU 3.011; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; PeoplePal 6.6; Verizon Windows Live)*/
const Device UA1896858 = {S836, 62, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows Vista; Trident/5.0)*/
const Device UA1954746 = {S837, 120, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Win64; x64; Trident/5.0; Tablet PC 2.0; .NET4.0C; .NET CLR 3.5.30729)*/
const Device UA1959324 = {S838, 207, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Windows Live Messenger 15.4.3555.0308)*/
const Device UA1983667 = {S839, 68, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 6.0; SV1)*/
const Device UA1999434 = {S840, 184, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2006022 = {S841, 193, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2146182 = {S842, 229, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GIL 3; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.21022; .NET CLR 3.5.30729; InfoPath.2; .NET4.0C; .NET4.0E; Windows Server Design; Windows Server Design)*/
const Device UA2170009 = {S843, 242, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2195298 = {S844, 221, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2203761 = {S845, 87, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3230.2052; Windows 6.0; MSIE 7.0.6000.17037)*/
const Device UA2212466 = {S846, 208, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SV1; SLCC1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.5.21022; .NET CLR 3.5.30729; InfoPath.2; Tablet PC 2.0; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2216772 = {S847, 199, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.2; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.21022; .NET CLR 3.0.30618; .NET CLR 3.5.30729; .NET4.0C; AskTbPPC/5.14.1.20007)*/
const Device UA2236447 = {S848, 226, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Win64; x64; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2245146 = {S849, 205, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; BRI/1; BRI/2; AskTbPPC/5.13.1.18107)*/
const Device UA2246387 = {S850, 169, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C; AskTbPPC/5.14.1.20007)*/
const Device UA2263320 = {S851, 70, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (Mozilla/5.0 (Windows; U; MSIE 7.0; Windows NT 6.0; en-US)*/
const Device UA2354101 = {S852, 184, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; F-6.0SP2-X; SV1; SLCC1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.30729; .NET CLR 3.5.30729; InfoPath.3; InfoPath.1)*/
const Device UA2412119 = {S853, 196, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Tablet PC 2.0; InfoPath.2; .NET CLR 1.1.4322; .NET CLR 3.5.30729; MS-RTC LM 8; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2414514 = {S854, 205, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Win64; x64; GTB7.4; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; SLCC1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2436568 = {S855, 87, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 7.0.6001.18000)*/
const Device UA2463399 = {S856, 68, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; 7.7; MSIE 7.7; Windows NT 6.0; Trident/5.0)*/
const Device UA2467821 = {S857, 87, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 7.0.6000.16982)*/
const Device UA2468589 = {S858, 87, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 7.0.6002.18005)*/
const Device UA2472752 = {S859, 218, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; Tablet PC 2.0; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7)*/
const Device UA2483469 = {S860, 87, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Internet Explorer 7 (Windows Vista): Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1)*/
const Device UA2495694 = {S861, 134, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30618; .NET CLR 3.5.30729; Zune 4.7)*/
const Device UA2502466 = {S862, 92, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - ('Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1)', 'Internet Explorer 7, Windows Vista')*/
const Device UA2522709 = {S863, 204, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Vista; Trident/4.0; FunWebProducts; GTB7.4; MS-RTC LM 8; SLCC1; SLCC2; .NET CLR 2.0.50727; InfoPath.3; .NET CLR 3.5.10729; .NET CLR 3.0.33729; .NET CLR 1.1.4322)*/
const Device UA2639789 = {S864, 61, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0.6001.18000; Windows NT 6.0)*/
const Device UA2808255 = {S865, 61, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0.6000.16982; Windows NT 6.0)*/
const Device UA2816335 = {S866, 234, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/4.0; GIL 3.5; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.21022; .NET CLR 3.5.30729; InfoPath.2; .NET4.0C; .NET4.0E; Windows Server Design; MSOffice 12)*/
const Device UA2851852 = {S867, 163, { &P15364,&P17285,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible  MSIE 7.0  Windows NT 6.0  SLCC1  .NET CLR 2.0.50727  Media Center PC 5.0  Tablet PC 2.0  .NET CLR 3.5.30729  .NET CLR 3.0.30729  .NET4.0C)*/
const Device UA99994 = {S868, 63, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0)*/
const Device UA119808 = {S869, 70, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0)*/
const Device UA135357 = {S870, 51, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Windows-RSS-Platform/2.0 (MSIE 9.0; Windows NT 6.1)*/
const Device UA136697 = {S871, 75, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0)*/
const Device UA137797 = {S872, 161, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)*/
const Device UA140808 = {S873, 50, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1)*/
const Device UA164273 = {S874, 64, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Empty (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0)*/
const Device UA164302 = {S875, 54, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 ( ; MSIE 9.0; Windows NT 6.1; Trident/5.0)*/
const Device UA164648 = {S876, 151, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0)*/
const Device UA165354 = {S877, 220, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Zune 4.0; Tablet PC 2.0; InfoPath.3; .NET4.0C; .NET4.0E)*/
const Device UA166344 = {S878, 156, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0)*/
const Device UA173206 = {S879, 75, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Windows Live Writer 1.0)*/
const Device UA175536 = {S880, 133, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; Zune 4.7)*/
const Device UA179544 = {S881, 50, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1)*/
const Device UA183894 = {S882, 61, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 ( ; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0)*/
const Device UA191734 = {S883, 80, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; XBLWP7; ZuneWP7)*/
const Device UA310320 = {S884, 57, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Empty (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0)*/
const Device UA310831 = {S885, 85, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Verizon Windows Live)*/
const Device UA421600 = {S886, 61, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0.8112.16421; Windows NT 6.1)*/
const Device UA434884 = {S887, 116, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible ; MSIE 9.0.8112.16421 ; Microsoft Windows 7 Professional Service Pack 1 ; Placeware RPC 1.0)*/
const Device UA436651 = {S888, 64, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0  (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0)*/
const Device UA505399 = {S889, 58, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows 7; Trident/5.0)*/
const Device UA541172 = {S890, 92, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Verizon Windows Live)*/
const Device UA544003 = {S891, 79, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Xerox_Win32PWS)*/
const Device UA544942 = {S892, 88, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; WindowsDownloads)*/
const Device UA559428 = {S893, 102, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible ; MSIE 9.0.8112.16421 ; Microsoft Windows 7 Professional  ; Placeware RPC 1.0)*/
const Device UA654509 = {S894, 151, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA658171 = {S895, 55, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; SV1)*/
const Device UA667757 = {S896, 87, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 6.6.1409.1944; Windows 6.1; MSIE 9.0.8080.16413)*/
const Device UA671704 = {S897, 207, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET CLR 1.1.4322; .NET4.0C; Tablet PC 2.0)*/
const Device UA683940 = {S898, 69, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT6.1; WOW64; Trident/5.0)*/
const Device UA684851 = {S899, 170, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; msie 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; SE 2.X MetaSr 1.0)*/
const Device UA685566 = {S900, 57, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1.2; SV1)*/
const Device UA686205 = {S901, 55, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; SV1)*/
const Device UA693689 = {S902, 69, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Empty (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0)*/
const Device UA695259 = {S903, 178, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA712430 = {S904, 187, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; msie 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; SE 2.X MetaSr 1.0)*/
const Device UA736381 = {S905, 122, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1))*/
const Device UA748070 = {S906, 115, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1))*/
const Device UA754691 = {S907, 180, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; msie 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; SE 2.X MetaSr 1.0)*/
const Device UA761094 = {S908, 107, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Mozilla/4.0 (compatible; MSIE 6.0; Menara))*/
const Device UA762161 = {S909, 97, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)*/
const Device UA786144 = {S910, 97, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - mozilla/5.0 (compatible; msie 9.0; windows nt 6.1; win64; x64; trident/5.0; funwebproducts; yie9)*/
const Device UA809093 = {S911, 73, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; MSIE 7.0)*/
const Device UA810737 = {S912, 69, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; MSIE 9.0; Windows NT 6.1; de-DE) Trident/5.0*/
const Device UA816716 = {S913, 51, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (MSIE 9.0; Windows NT 6.1; Trident/5.0)*/
const Device UA890201 = {S914, 177, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; msie 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; SE 2.X MetaSr 1.0)*/
const Device UA892386 = {S915, 94, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1 SB.X.1; Trident/5.0; Active Content Browser)*/
const Device UA1860543 = {S916, 66, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 ( ; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0)*/
const Device UA1890788 = {S917, 69, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; WindowsNT 6.1; WOW64; Trident/5.0)*/
const Device UA1903741 = {S918, 59, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows; MSIE 9.0; Windows NT 6.1; InetLoader)*/
const Device UA1908397 = {S919, 67, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - MSIE 9.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0)*/
const Device UA1940123 = {S920, 71, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0  (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0)*/
const Device UA1969946 = {S921, 166, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)*/
const Device UA1992546 = {S922, 159, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; msie 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; SE 2.X MetaSr 1.0)*/
const Device UA2143094 = {S923, 80, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; MSIE 6.0)*/
const Device UA2144318 = {S924, 122, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1))*/
const Device UA2146912 = {S925, 60, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - MSIE 9.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0)*/
const Device UA2155707 = {S926, 141, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727)*/
const Device UA2158162 = {S927, 200, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; msie 9.0; qdesk 2.3.1185.202; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; SE 2.X MetaSr 1.0)*/
const Device UA2161649 = {S928, 198, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA2210304 = {S929, 64, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Internet Explorer (compatible; MSIE 9.0; Windows 7; Trident/5.0)*/
const Device UA2210868 = {S930, 73, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Windows Live Mail 1.0)*/
const Device UA2212552 = {S931, 111, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1))*/
const Device UA2219681 = {S932, 136, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; msie 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MALN)*/
const Device UA2259357 = {S933, 71, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; ARMXKD)*/
const Device UA2302083 = {S934, 82, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Windows NT)*/
const Device UA2304484 = {S935, 123, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; YDP))*/
const Device UA2309322 = {S936, 167, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1); Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1))*/
const Device UA2314273 = {S937, 165, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; msie 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MALN; .NET4.0C; SE 2.X MetaSr 1.0)*/
const Device UA2314595 = {S938, 113, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) )*/
const Device UA2330521 = {S939, 176, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; msie 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; SE 2.X MetaSr 1.0)*/
const Device UA2367341 = {S940, 122, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; GTB7.4; InfoPath.1; SV1; .NET CLR 2.8.52393; WOW64; en-US)*/
const Device UA2368708 = {S941, 64, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.10; Windows NT 6.1; Trident/5.0)*/
const Device UA2395538 = {S942, 164, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; msie 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; BRI/2)*/
const Device UA2409404 = {S943, 46, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - MSIE 9.0 ( ; MSIE 9.0; Windows 7; Trident/5.0)*/
const Device UA2429011 = {S944, 87, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.1; MSIE 9.0.8112.16421)*/
const Device UA2432209 = {S945, 88, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.1; MSIE 9.10.9200.16438)*/
const Device UA2442926 = {S946, 97, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Verizon Windows Live)*/
const Device UA2444783 = {S947, 158, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0)*/
const Device UA2445207 = {S948, 76, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0  (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0)*/
const Device UA2458219 = {S949, 149, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; SV1; QQDownload 713; .NET CLR 2.0.50727; .NET4.0C; WOW64; .NET CLR 3.8.25433; Media Center PC 3.7)*/
const Device UA2490144 = {S950, 87, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; QQBrowser/7.1.5753.400)*/
const Device UA2490277 = {S951, 94, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; QQBrowser/7.1.5753.400)*/
const Device UA2507826 = {S952, 80, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; MSIE 9.0)*/
const Device UA2507947 = {S953, 87, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.1; MSIE 9.0.8112.16443)*/
const Device UA2537668 = {S954, 88, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.1; MSIE 9.10.9200.16521)*/
const Device UA2554584 = {S955, 66, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible MSIE 9.0 Windows NT 6.1 WOW64 Trident/5.0)*/
const Device UA2554787 = {S956, 76, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.10; Windows NT 6.1; Windows Live Writer 1.0)*/
const Device UA2554870 = {S957, 76, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; msie 9.0; Windows NT 6.1; WOW64; Trident/6.0; MASP)*/
const Device UA2567630 = {S958, 96, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.10; Windows NT 6.1; WOW64; Trident/6.0; SiteKiosk 7.8 Build 332)*/
const Device UA2582620 = {S959, 117, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible ; MSIE 9.10.9200.16521 ; Microsoft Windows 7 Professional Service Pack 1 ; Placeware RPC 1.0)*/
const Device UA2585281 = {S960, 94, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; QQBrowser/7.2.7006.400)*/
const Device UA2585919 = {S961, 87, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; QQBrowser/7.2.7006.400)*/
const Device UA2594586 = {S962, 51, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.10; Windows NT 6.1)*/
const Device UA2594601 = {S963, 75, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; msie 9.0; Windows NT 6.1; Trident/6.0; BOIE9;ZHCN)*/
const Device UA2639298 = {S964, 83, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Mozilla/4.0)*/
const Device UA2644010 = {S965, 145, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Tablet PC 2.0)*/
const Device UA2649564 = {S966, 73, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; SV1; SV1)*/
const Device UA2654041 = {S967, 122, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; GTB7.4; InfoPath.2; SV1; .NET CLR 4.4.58799; WOW64; en-US)*/
const Device UA2666688 = {S968, 62, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.10.9200.16521; Windows NT 6.1)*/
const Device UA2669381 = {S969, 77, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - "Mozilla/5.0 compatible MSIE 9.0 Windows NT 6.1 WOW64 Trident/5.0 BOIE9 ENUS"*/
const Device UA2669576 = {S970, 194, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.10; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; SiteKiosk 7.8 Build 332)*/
const Device UA2675743 = {S971, 117, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible ; MSIE 9.10.9200.16540 ; Microsoft Windows 7 Professional Service Pack 1 ; Placeware RPC 1.0)*/
const Device UA2677104 = {S972, 62, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.10.9200.16540; Windows NT 6.1)*/
const Device UA2679382 = {S973, 88, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.1; MSIE 9.10.9200.16540)*/
const Device UA2684596 = {S974, 118, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1))*/
const Device UA2695382 = {S975, 159, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.10; Windows NT 6.1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2728266 = {S976, 68, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - MSIE 9.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0)*/
const Device UA2740058 = {S977, 62, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.10.9200.16576; Windows NT 6.1)*/
const Device UA2740436 = {S978, 88, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.1; MSIE 9.10.9200.16576)*/
const Device UA2752919 = {S979, 117, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible ; MSIE 9.10.9200.16576 ; Microsoft Windows 7 Professional Service Pack 1 ; Placeware RPC 1.0)*/
const Device UA2772204 = {S980, 207, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - 'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center C 6.0; InfoPath.2; .NET CLR 1.1.4322; .NET4.0C; Tablet PC 2.0)*/
const Device UA2773973 = {S981, 220, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - 'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; edia Center PC 6.0; Zune 4.0; Tablet PC 2.0; InfoPath.3; .NET4.0C; .NET4.0E)*/
const Device UA2773976 = {S982, 213, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - 'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media enter PC 6.0; Zune 4.0; InfoPath.3; MS-RTC LM 8; .NET4.0C; .NET4.0E)*/
const Device UA2794805 = {S983, 71, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.11; WOW64; Trident/5.0)*/
const Device UA2796957 = {S984, 117, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible ; MSIE 9.10.9200.16614 ; Microsoft Windows 7 Professional Service Pack 1 ; Placeware RPC 1.0)*/
const Device UA2806056 = {S985, 76, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Mozilla/4.0)*/
const Device UA2806310 = {S986, 128, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Mozilla/6.0 (compatible; MSIE 2.0; Windows Z; Shinigami))*/
const Device UA2817604 = {S987, 82, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; msie 9.0; Windows NT 6.1; Trident/5.0; SE 2.X MetaSr 1.0)*/
const Device UA2839020 = {S988, 189, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Media Center PC 5.0; SLCC1)*/
const Device UA2843913 = {S989, 157, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; msie 9.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2870799 = {S990, 88, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.5.4209.2358; Windows 6.1; MSIE 9.10.9200.16614)*/
const Device UA2871483 = {S991, 62, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.10.9200.16614; Windows NT 6.1)*/
const Device UA2872308 = {S992, 88, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.5.4209.2358; Windows 6.1; MSIE 9.10.9200.16618)*/
const Device UA2879907 = {S993, 45, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - mozilla/4.0 (compatible; MSIE 9.0; windows 7)*/
const Device UA2889460 = {S994, 206, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729))*/
const Device UA2946412 = {S995, 117, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible ; MSIE 9.10.9200.16635 ; Microsoft Windows 7 Professional Service Pack 1 ; Placeware RPC 1.0)*/
const Device UA2947368 = {S996, 88, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.5.4209.2358; Windows 6.1; MSIE 9.10.9200.16635)*/
const Device UA2967411 = {S997, 62, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.10.9200.16635; Windows NT 6.1)*/
const Device UA2979504 = {S998, 73, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; MSIE 9.0)*/
const Device UA2980907 = {S999, 73, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; SV1; SV1)*/
const Device UA2981755 = {S1000, 179, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1); Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1))*/
const Device UA2988625 = {S1001, 74, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.10; Windows NT 6.1; Windows Live Mail 1.0)*/
const Device UA2994395 = {S1002, 208, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.10; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C; MALC; SiteKiosk 7.8 Build 332)*/
const Device UA2995647 = {S1003, 197, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.10; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; SiteKiosk 7.8 Build 332)*/
const Device UA3022473 = {S1004, 78, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (MSIE 9.0; Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko*/
const Device UA3024049 = {S1005, 64, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.11; Windows NT 6.1; Trident/5.0)*/
const Device UA3033969 = {S1006, 117, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible ; MSIE 9.10.9200.16660 ; Microsoft Windows 7 Professional Service Pack 1 ; Placeware RPC 1.0)*/
const Device UA3034117 = {S1007, 88, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.5.4413.1752; Windows 6.1; MSIE 9.10.9200.16660)*/
const Device UA3041750 = {S1008, 171, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7)*/
const Device UA3043280 = {S1009, 106, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Windows Internet Explorer provided by ABB)*/
const Device UA3050898 = {S1010, 117, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible ; MSIE 9.10.9200.16618 ; Microsoft Windows 7 Professional Service Pack 1 ; Placeware RPC 1.0)*/
const Device UA3061048 = {S1011, 62, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.10.9200.16660; Windows NT 6.1)*/
const Device UA3062635 = {S1012, 57, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; en-US)*/
const Device UA3069535 = {S1013, 115, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1))*/
const Device UA3083323 = {S1014, 88, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.5.4413.1752; Windows 6.1; MSIE 9.10.9200.16686)*/
const Device UA3083563 = {S1015, 117, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible ; MSIE 9.10.9200.16686 ; Microsoft Windows 7 Professional Service Pack 1 ; Placeware RPC 1.0)*/
const Device UA3098588 = {S1016, 153, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.4506.2152; Safari; InfoPath.1)*/
const Device UA3110477 = {S1017, 62, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.10.9200.16686; Windows NT 6.1)*/
const Device UA3125156 = {S1018, 83, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/7.0; rv:11.0) like Gecko*/
const Device UA3128213 = {S1019, 88, { &P15364,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.5.4413.1752; Windows 6.1; MSIE 9.11.9600.16384)*/
const Device UA8113 = {S1020, 55, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)*/
const Device UA10048 = {S1021, 58, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; XV6975)*/
const Device UA13466 = {S1022, 60, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; XV6875.1)*/
const Device UA13470 = {S1023, 58, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; XV6875)*/
const Device UA54333 = {S1024, 69, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 1.1.4322)*/
const Device UA55087 = {S1025, 50, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)*/
const Device UA91801 = {S1026, 120, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Mozilla/4.0(compatible; MSIE 6; WindowsNT; ISP26; SV1; PeoplePal 3.0)*/
const Device UA92833 = {S1027, 71, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Media Center PC*/
const Device UA93076 = {S1028, 64, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; Win64; AMD64)*/
const Device UA93087 = {S1029, 51, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible ; MSIE 6.0; Windows NT 5.1)*/
const Device UA93292 = {S1030, 46, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows XP)*/
const Device UA95099 = {S1031, 60, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.1)*/
const Device UA97395 = {S1032, 116, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; YDP); Wanadoo 5.6)*/
const Device UA98132 = {S1033, 73, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible- MSIE 6.0- Windows NT 5.1- SV1- .NET CLR 1.1.4322*/
const Device UA98602 = {S1034, 113, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) )*/
const Device UA99125 = {S1035, 94, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727)*/
const Device UA99210 = {S1036, 87, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; Win64; x64; SV1; .NET CLR 2.0.50727)*/
const Device UA99230 = {S1037, 69, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; .NET CLR 1.1.4322)*/
const Device UA99822 = {S1038, 82, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; WOW64; SV1; .NET CLR 2.0.50727)*/
const Device UA100822 = {S1039, 147, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727)*/
const Device UA101107 = {S1040, 75, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727)*/
const Device UA103999 = {S1041, 99, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Menara))*/
const Device UA104938 = {S1042, 50, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2)*/
const Device UA115577 = {S1043, 128, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (Compatible; Windows NT 5.1; MSIE 6.0) (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727)*/
const Device UA115936 = {S1044, 50, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 5.1)*/
const Device UA123993 = {S1045, 108, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) )*/
const Device UA129738 = {S1046, 172, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA130541 = {S1047, 132, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322)*/
const Device UA130577 = {S1048, 172, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)*/
const Device UA130973 = {S1049, 94, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - MSNPTC/1.0 (compatible; MSIE 6.0; Windows NT 5.2; MyIE2; .NET CLR 1.1.4322; .NET CLR 1.0.3705)*/
const Device UA132838 = {S1050, 57, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - User-Agent: Mozilla/5.0 (compatible; MSIE 6.0;Windows XP)*/
const Device UA134652 = {S1051, 39, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/6.0 (MSIE 6.0; Windows NT 5.1;)*/
const Device UA142411 = {S1052, 63, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/5.0)*/
const Device UA145173 = {S1053, 38, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (MSIE 6.0; Windows NT 5.1)*/
const Device UA151600 = {S1054, 85, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; Zune 4.7)*/
const Device UA153610 = {S1055, 51, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.08 (compatible; MSIE 6.0; Windows NT 5.1)*/
const Device UA156314 = {S1056, 75, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727)*/
const Device UA159738 = {S1057, 72, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727)*/
const Device UA170484 = {S1058, 176, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QS 4.2.1.0; Windows 2000; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.1; .NET CLR 1.1.4322)*/
const Device UA176559 = {S1059, 50, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozila/4.0 (compatible; MSIE 6.0; Windows NT 5.1;)*/
const Device UA236546 = {S1060, 255, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Poco 0.31; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) ; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) ( EmbeddedWB*/
const Device UA259789 = {S1061, 176, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; 3M/IE 6.0;; 3M/MSIE 6.0;; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.1)*/
const Device UA260496 = {S1062, 135, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; Media Center PC 2.8)*/
const Device UA261354 = {S1063, 183, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0(Compatible Mozilla/4.0(Compatible-EmbeddedWB 14.59 http://bsalsa.com/ EmbeddedWB- 14.59  from: http://bsalsa.com/ )*/
const Device UA268800 = {S1064, 51, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/45.0 (compatible; MSIE 6.0; Windows NT 5.1)*/
const Device UA272428 = {S1065, 117, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; Win 9x 4.90; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) )*/
const Device UA299436 = {S1066, 47, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows; MSIE 6.0; Windows NT 5.2)*/
const Device UA373159 = {S1067, 51, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1.)*/
const Device UA421962 = {S1068, 151, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA470901 = {S1069, 117, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; Win 9x 4.90; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) )*/
const Device UA480356 = {S1070, 62, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; SV1)*/
const Device UA490095 = {S1071, 94, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Windows Live Messenger 14.0.8117.0416)*/
const Device UA506125 = {S1072, 109, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) )*/
const Device UA553364 = {S1073, 194, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0 (Compatible; MSIE 6.0;Windows NT;ASPI 1.0); .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA559025 = {S1074, 147, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QS 4.2.1.0; Windows 2000; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 2.0.50727; InfoPath.1)*/
const Device UA584844 = {S1075, 97, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)*/
const Device UA590314 = {S1076, 50, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE6.0; Windows XP; SV1)*/
const Device UA639754 = {S1077, 61, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozila/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; MyIE2;*/
const Device UA649156 = {S1078, 58, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatiable; MSIE 6.0; Windows NT 5.1) HiScan*/
const Device UA656859 = {S1079, 158, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0)*/
const Device UA665169 = {S1080, 55, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0.2800; Windows NT 5.1)*/
const Device UA667965 = {S1081, 54, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.1 (compatible; MSIE 6.0; Windows NT 5.1; en)*/
const Device UA668552 = {S1082, 127, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322)*/
const Device UA680888 = {S1083, 57, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1.8; SV1)*/
const Device UA688948 = {S1084, 57, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1.2; SV1)*/
const Device UA693326 = {S1085, 178, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA717723 = {S1086, 191, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Sicent1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Sicent1; Media Center PC 6.0; Microsoft Windows Media Center PC 6.0)*/
const Device UA718354 = {S1087, 196, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; GX755 XPUSV1.X; SVCVersion 7.30; SVCVersion 8.11; SVCVersion 8.14; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; InfoPath.1)*/
const Device UA720819 = {S1088, 57, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1.3; SV1)*/
const Device UA737098 = {S1089, 62, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows XP 5.1) Lobo/0.98.4*/
const Device UA788572 = {S1090, 205, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (Compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; InfoPath.2; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA828742 = {S1091, 234, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) ; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) (none))*/
const Device UA869945 = {S1092, 78, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; AskTbPPC/5.12.2.16749)*/
const Device UA909865 = {S1093, 46, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (MSIE 6.0; Windows NT 5.1; Search)*/
const Device UA1866875 = {S1094, 121, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; Windows NT 5.1; Trident/1.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1);)*/
const Device UA1884923 = {S1095, 121, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/6.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/2.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1);)*/
const Device UA1969691 = {S1096, 166, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0)*/
const Device UA2195915 = {S1097, 204, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; U; MSIE 6.0; Windows NT 5.1) (Compatible; Trident/4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.0.3705; .NET CLR 1.1.4322)*/
const Device UA2207787 = {S1098, 173, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; FunWebProducts; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; Zune 2.0; MS-RTC LM 8)*/
const Device UA2218077 = {S1099, 176, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C)*/
const Device UA2266954 = {S1100, 68, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0)*/
const Device UA2275660 = {S1101, 143, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; Zune 4.7)*/
const Device UA2276221 = {S1102, 108, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) )*/
const Device UA2276781 = {S1103, 226, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Sicent; WoShiHoney.B; Mozilla/4.0(Compatible Mozilla/4.0(Compatible-EmbeddedWB 14.59 http://bsalsa.com/ EmbeddedWB- 14.59  from: http://bsalsa.com/ ; Media Center PC 6.0)*/
const Device UA2281161 = {S1104, 183, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QS 4.2.1.0; Windows 2000 CIT CS; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.1; .NET CLR 1.1.4322)*/
const Device UA2290985 = {S1105, 72, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible- MSIE 6.0- Windows NT 5.1- SV1- .NET CLR1.1.4322*/
const Device UA2312861 = {S1106, 97, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)*/
const Device UA2314138 = {S1107, 216, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; 360SE) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA2335519 = {S1108, 264, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0(Compatible Mozilla/4.0(Compatible-EmbeddedWB 14.58 http://bsalsa.com/ EmbeddedWB- 14.58  from: http://bsalsa.com/ ; Mozilla/4.0(Compatible RogueCleanerEmbeddedWB- 14.58  from: http://bsalsa.com/ )*/
const Device UA2371414 = {S1109, 151, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA2411837 = {S1110, 145, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Tablet PC 2.0)*/
const Device UA2412395 = {S1111, 179, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0; Media Center PC 6.0)*/
const Device UA2437775 = {S1112, 47, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 5.1)*/
const Device UA2437944 = {S1113, 191, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0(Compatible Mozilla/4.0(Compatible-EmbeddedWB 14.59 http://bsalsa.com/ EmbeddedWB- 14.59  from: http://bsalsa.com/ )*/
const Device UA2438188 = {S1114, 171, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)*/
const Device UA2443572 = {S1115, 134, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; SV1; Mozilla/4.0 (X11; MSIE 6.0; i686; .NET CLR 1.1.4322; .NET CLR 2.0.50727; FDM))*/
const Device UA2502467 = {S1116, 108, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - ('Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 1.1.4322)', 'Internet Explorer 6, Windows XP')*/
const Device UA2504904 = {S1117, 67, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - MSIE 6 (Win XP): Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)*/
const Device UA2505433 = {S1118, 187, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0; Media Center PC 6.0)*/
const Device UA2519594 = {S1119, 126, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; TencentTraveler 4.0; QQDownload 677); QQBrowser/5.0.6648.400 (trident)*/
const Device UA2527252 = {S1120, 357, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; Foxy/1; BTRSX; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) ; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://bsalsa.com) (Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)); .NET CLR 2.0.50727; AskTbSPC2/5.13.1.18107)*/
const Device UA2529761 = {S1121, 89, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows XP 5.1; MSIE 6.0.2900.5512)*/
const Device UA2544580 = {S1122, 188, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7)*/
const Device UA2558144 = {S1123, 79, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QQBrowser/7.1.5753.400)*/
const Device UA2589542 = {S1124, 178, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2639827 = {S1125, 116, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible ; MSIE 6.0.2900.5512 ; Microsoft Windows XP Professional Service Pack 3 ; Placeware RPC 1.0)*/
const Device UA2643612 = {S1126, 181, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7)*/
const Device UA2656626 = {S1127, 89, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows XP 5.1; MSIE 6.0.2900.2180)*/
const Device UA2658986 = {S1128, 94, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/6.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727)*/
const Device UA2692055 = {S1129, 78, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible\; MSIE 6.0\; Windows NT 5.1\; SV1\; .NET CLR 1.1.4322)*/
const Device UA2726187 = {S1130, 234, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 3.0.30618; Media Center PC 5.0; SLCC1; .NET4.0C; Tablet PC 2.0; .NET4.0E)*/
const Device UA2761370 = {S1131, 266, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; Trident/4.0; BTRSX; MRSPUTNIK 2, 4, 1, 44; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2791714 = {S1132, 63, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 5.2; Trident/1.0)*/
const Device UA2802324 = {S1133, 63, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (iPassConnect/PU 8.6.0.4; MSIE 6.0; Windows NT 5.1)*/
const Device UA2806615 = {S1134, 89, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0   (compatible; MSIE 6.0; America Online Browser 1.1; rev1.5; Windows NT 5.1;)*/
const Device UA2827719 = {S1135, 196, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C; .NET4.0E; Zune 4.7)*/
const Device UA2933224 = {S1136, 160, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; icafe8; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Microsoft Windows Media Center PC 6.0)*/
const Device UA2933997 = {S1137, 153, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7)*/
const Device UA2934422 = {S1138, 173, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; Trident/4.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; ASU2JS)*/
const Device UA2995859 = {S1139, 282, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; BTRSX; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA3042102 = {S1140, 50, { &P15364,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 5.2)*/
const Device UA86117 = {S1141, 23, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - CrawlerBoy Pinpoint.com*/
const Device UA92227 = {S1142, 46, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - msnbot/1.1 (+http://search.msn.com/msnbot.htm)*/
const Device UA92684 = {S1143, 47, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - msnbot/2.0b (+http://search.msn.com/msnbot.htm)*/
const Device UA92696 = {S1144, 53, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Baiduspider+(+http://www.baidu.com/search/spider.htm)*/
const Device UA92703 = {S1145, 17, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SimpleCrawler/0.1*/
const Device UA92771 = {S1146, 3, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - bot*/
const Device UA92922 = {S1147, 50, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - AdsBot-Google (+http://www.google.com/adsbot.html)*/
const Device UA93018 = {S1148, 82, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; sindice-fetcher/0.1.0 +http://sindice.com/developers/bot)*/
const Device UA93041 = {S1149, 42, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Baiduspider+(+http://www.baidu.jp/spider/)*/
const Device UA93281 = {S1150, 8, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - bitlybot*/
const Device UA93331 = {S1151, 64, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - TurnitinBot/2.1 (http://www.turnitin.com/robot/crawlerinfo.html)*/
const Device UA93630 = {S1152, 90, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Y!J-BRI/0.0.1 crawler ( http://help.yahoo.co.jp/help/jp/search/indexing/indexing-15.html )*/
const Device UA93937 = {S1153, 75, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ia_archiver (+http://www.alexa.com/site/help/webmasters; crawler@alexa.com)*/
const Device UA93959 = {S1154, 71, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Sogou web spider/4.0(+http://www.sogou.com/docs/help/webmasters.htm#07)*/
const Device UA93920 = {S1155, 87, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; pmoz.info ODP link checker; +http://pmoz.info/doc/botinfo.htm)*/
const Device UA94010 = {S1156, 13, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Magus Bot 1.0*/
const Device UA94407 = {S1157, 58, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - msnbot-UDiscovery/2.0b (+http://search.msn.com/msnbot.htm)*/
const Device UA94697 = {S1158, 32, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - RSSMicro.com RSS/Atom Feed Robot*/
const Device UA94694 = {S1159, 9, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - AlbertBot*/
const Device UA95232 = {S1160, 77, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; NaverBot/1.0; http://help.naver.com/delete_main.asp)*/
const Device UA95382 = {S1161, 40, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Willow Internet Crawler by Twotrees V2.1*/
const Device UA95898 = {S1162, 84, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Checklinks/1.3 (pywikipedia robot; http://toolserver.org/~dispenser/view/Checklinks)*/
const Device UA97810 = {S1163, 59, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Baiduspider-image+(+http://www.baidu.com/search/spider.htm)*/
const Device UA97822 = {S1164, 58, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; RSSMicro.com RSS/Atom Feed Robot)*/
const Device UA98196 = {S1165, 69, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Exabot/3.0; +http://www.exabot.com/go/robot)*/
const Device UA98197 = {S1166, 34, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - MLBot (www.metadatalabs.com/mlbot)*/
const Device UA98214 = {S1167, 81, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; woriobot support [at] worio [dot] com +http://worio.com)*/
const Device UA98329 = {S1168, 79, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MJ12bot/v1.2.1; http://www.majestic12.co.uk/bot.php?+)*/
const Device UA98473 = {S1169, 87, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YoudaoBot/1.0; http://www.youdao.com/help/webmaster/spider/; )*/
const Device UA98698 = {S1170, 99, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - NextGenSearchBot 1 (for information visit http://www.zoominfo.com/About/misc/NextGenSearchBot.aspx)*/
const Device UA98809 = {S1171, 52, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - msnbot-media/1.1 (+http://search.msn.com/msnbot.htm)*/
const Device UA98991 = {S1172, 39, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - R6_FeedFetcher(www.radian6.com/crawler)*/
const Device UA99006 = {S1173, 48, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Sosospider+(+http://help.soso.com/webspider.htm)*/
const Device UA99130 = {S1174, 77, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; FriendFeedBot/0.1; +Http://friendfeed.com/about/bot)*/
const Device UA99137 = {S1175, 52, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; woriobot +http://worio.com)*/
const Device UA99276 = {S1176, 50, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Baiduspider+(+http://help.baidu.jp/system/05.html)*/
const Device UA99610 = {S1177, 68, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Purebot/1.1; +http://www.puritysearch.net/)*/
const Device UA99789 = {S1178, 62, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; aiHitBot/1.0; +http://www.aihit.com/)*/
const Device UA100643 = {S1179, 54, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ShopWiki/1.0 ( +http://www.shopwiki.com/wiki/Help:Bot)*/
const Device UA100709 = {S1180, 45, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Dow Jones Searchbot)*/
const Device UA101266 = {S1181, 68, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; FatBot 2.0; http://www.thefind.com/crawler)*/
const Device UA101497 = {S1182, 22, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Xenu Link Sleuth/1.3.7*/
const Device UA101498 = {S1183, 64, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Embedly/0.2; +http://support.embed.ly/)*/
const Device UA101986 = {S1184, 26, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SuperPagesUrlVerifyBot/1.0*/
const Device UA102824 = {S1185, 81, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; discobot/1.1; +http://discoveryengine.com/discobot.html)*/
const Device UA102876 = {S1186, 6, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - spider*/
const Device UA102981 = {S1187, 68, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; seexie.com_bot/4.1; +http://www.seexie.com)*/
const Device UA103116 = {S1188, 71, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MarketwireBot/1.0; +http://www.marketwire.com)*/
const Device UA103322 = {S1189, 67, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; Inspyder-Crawler; http://www.inspyder.com)*/
const Device UA104246 = {S1190, 85, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YodaoBot/1.0; http://www.yodao.com/help/webmaster/spider/; )*/
const Device UA104662 = {S1191, 60, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - TjoosBot/1.0 (http://www.tjoos.com/Page-Content/34/TjoosBot)*/
const Device UA104738 = {S1192, 80, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ICC-Crawler/2.0 (Mozilla-compatible; ; http://kc.nict.go.jp/project1/crawl.html)*/
const Device UA105242 = {S1193, 46, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - psbot/0.1 (+http://www.picsearch.com/bot.html)*/
const Device UA105346 = {S1194, 49, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Qualidator.com Bot 1.0;)*/
const Device UA105715 = {S1195, 18, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - UCMore Crawler App*/
const Device UA106427 = {S1196, 56, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Nigma.ru/3.0; crawler@nigma.ru)*/
const Device UA107482 = {S1197, 61, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Sosoimagespider+(+http://help.soso.com/soso-image-spider.htm)*/
const Device UA107611 = {S1198, 64, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexBot/3.0; +http://yandex.com/bots)*/
const Device UA107898 = {S1199, 67, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexImages/3.0; +http://yandex.com/bots)*/
const Device UA108091 = {S1200, 47, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - crawler4j (http://code.google.com/p/crawler4j/)*/
const Device UA108126 = {S1201, 104, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; NetSeer crawler/2.0; +http://www.netseer.com/crawler.html; crawler@netseer.com)*/
const Device UA112224 = {S1202, 89, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible;YodaoBot-Image/1.0;http://www.youdao.com/help/webmaster/spider/;)*/
const Device UA112637 = {S1203, 94, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; bnf.fr_bot; +http://www.bnf.fr/fr/outils/a.dl_web_capture_robot.html)*/
const Device UA113147 = {S1204, 9, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - TwengaBot*/
const Device UA114535 = {S1205, 53, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Caliperbot/1.0 (+http://www.conductor.com/caliperbot)*/
const Device UA115001 = {S1206, 34, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - zibber-v0.1(www.zibb.com/crawler/)*/
const Device UA116076 = {S1207, 58, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Flamingo_SearchEngine (+http://www.flamingosearch.com/bot)*/
const Device UA116158 = {S1208, 12, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - oodlebot/1.0*/
const Device UA116738 = {S1209, 70, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YesupBot/1.0; +http://www.yesup.net/bot.html)*/
const Device UA120993 = {S1210, 70, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Feedage/2.0; +http://www.feedage.com/bot.php)*/
const Device UA121000 = {S1211, 43, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - HLTBot/1.3 (+http://hlt.fbk.eu/hltbot.html)*/
const Device UA121179 = {S1212, 7, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Crawler*/
const Device UA121287 = {S1213, 43, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - HubSpot Crawler 1.0 http://www.hubspot.com/*/
const Device UA121628 = {S1214, 46, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - AddThis.com robot tech.support@clearspring.com*/
const Device UA121908 = {S1215, 81, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Yahoo!-AdCrawler; http://help.yahoo.com/yahoo_adcrawler)*/
const Device UA121924 = {S1216, 50, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Gigabot/3.0 (http://www.gigablast.com/spider.html)*/
const Device UA122489 = {S1217, 10, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Who.is Bot*/
const Device UA123761 = {S1218, 8, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - web18bot*/
const Device UA123833 = {S1219, 9, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Jyxobot/1*/
const Device UA124455 = {S1220, 76, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Apercite; +http://www.apercite.fr/robot/index.html)*/
const Device UA125680 = {S1221, 103, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible;  Vagabondo/4.0; webcrawler at wise-guys dot nl; http://webagent.wise-guys.nl/)*/
const Device UA126440 = {S1222, 38, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - CatchBot/1.0; +http://www.catchbot.com*/
const Device UA126658 = {S1223, 41, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - R6_CommentReader(www.radian6.com/crawler)*/
const Device UA126814 = {S1224, 129, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible;  Vagabondo/4.0; webcrawler at wise-guys dot nl; http://webagent.wise-guys.nl/; http://www.wise-guys.nl/)*/
const Device UA127221 = {S1225, 49, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - msnbot/2.0b (+http://search.msn.com/msnbot.htm)._*/
const Device UA129118 = {S1226, 44, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - iisbot/1.0 (+http://www.iis.net/iisbot.html)*/
const Device UA130129 = {S1227, 53, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - solfo-linkchecker/1.0 (http://solfo.com/linkbot.html)*/
const Device UA130423 = {S1228, 74, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Search17Bot/1.1; http://www.search17.com/bot.php)*/
const Device UA130976 = {S1229, 14, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Twitterbot/0.1*/
const Device UA131005 = {S1230, 98, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - LinkedInBot/1.0 (compatible; Mozilla/5.0; Jakarta Commons-HttpClient/3.1 +http://www.linkedin.com)*/
const Device UA131793 = {S1231, 73, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexImageResizer/2.0; +http://yandex.com/bots)*/
const Device UA131816 = {S1232, 57, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - msnbot-NewsBlogs/2.0b (+http://search.msn.com/msnbot.htm)*/
const Device UA131935 = {S1233, 12, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - bitlybot/2.0*/
const Device UA132001 = {S1234, 48, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - adidxbot/1.1 (+http://search.msn.com/msnbot.htm)*/
const Device UA132271 = {S1235, 12, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - my-robot/0.1*/
const Device UA133053 = {S1236, 22, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Xenu Link Sleuth/1.3.8*/
const Device UA133171 = {S1237, 15, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - FlaxCrawler/1.0*/
const Device UA134084 = {S1238, 38, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - CatchBot/2.0; +http://www.catchbot.com*/
const Device UA134137 = {S1239, 8, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - eventBot*/
const Device UA135086 = {S1240, 79, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MJ12bot/v1.3.3; http://www.majestic12.co.uk/bot.php?+)*/
const Device UA135424 = {S1241, 89, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; archive.org_bot +http://www.archive.org/details/archive.org_bot)*/
const Device UA135917 = {S1242, 16, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - 123peoplebot/1.0*/
const Device UA138229 = {S1243, 15, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - wikiwix-bot-3.0*/
const Device UA139140 = {S1244, 87, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Sosospider+(+http://help.soso.com/webspider.htm),gzip(gfe) (via docs.google.com/viewer)*/
const Device UA139586 = {S1245, 70, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexAntivirus/2.0; +http://yandex.com/bots)*/
const Device UA140940 = {S1246, 64, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Synthesio Crawler release MonaLisa (contact at synthesio dot fr)*/
const Device UA144477 = {S1247, 57, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ssearch_bot (sSearch Crawler; http://www.semantissimo.de)*/
const Device UA146478 = {S1248, 34, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - R6_Primer(www.radian6.com/crawler)*/
const Device UA147692 = {S1249, 17, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Zing-BottaBot/1.0*/
const Device UA151554 = {S1250, 10, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Toplistbot*/
const Device UA151901 = {S1251, 38, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - WebZIP/x.x (http://www.spidersoft.com)*/
const Device UA152025 = {S1252, 20, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - TVersity Media Robot*/
const Device UA153187 = {S1253, 12, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - WordChampBot*/
const Device UA153680 = {S1254, 85, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; rogerBot/1.0; UrlCrawler; http://www.seomoz.org/dp/rogerbot)*/
const Device UA154010 = {S1255, 12, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - BLP_bbot/0.1*/
const Device UA154518 = {S1256, 73, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; rogerBot/1.0; http://www.seomoz.org/dp/rogerbot)*/
const Device UA154959 = {S1257, 123, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7; NextpointBot/1.0; http://nextpoint.com/*/
const Device UA155557 = {S1258, 50, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - GumGum Bot (http://gumgum.com; support@gumgum.com)*/
const Device UA156993 = {S1259, 48, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - netEstate NE Crawler (+http://www.sengine.info/)*/
const Device UA160837 = {S1260, 42, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Twitmunin Crawler http://www.twitmunin.com*/
const Device UA162400 = {S1261, 50, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Altresium/4.3 (+http://www.altresium.com/bot.html)*/
const Device UA162787 = {S1262, 13, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - eSyndiCat Bot*/
const Device UA162828 = {S1263, 27, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Xenu Link Sleuth/1.3.9 beta*/
const Device UA163219 = {S1264, 52, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 (CMS Crawler: http://www.cmscrawler.com)*/
const Device UA164159 = {S1265, 47, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - FAST Enterprise Crawler/6.7.0 (crawler@fast.no)*/
const Device UA164244 = {S1266, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ScooperBot www.customscoop.com*/
const Device UA164363 = {S1267, 69, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; spbot/3.0; +http://www.seoprofiler.com/bot )*/
const Device UA165364 = {S1268, 12, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SinaWeiboBot*/
const Device UA167093 = {S1269, 87, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; sindice-site-manager/0.1.0 +http://sindice.com/developers/bot)*/
const Device UA167353 = {S1270, 72, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; trovitBot/1.0; +http://www.trovit.com/bot.html)*/
const Device UA167726 = {S1271, 46, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - TwengaBot-2.0 (http://www.twenga.com/bot.html)*/
const Device UA168354 = {S1272, 19, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - reach-local-bot/0.1*/
const Device UA168406 = {S1273, 58, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Ezooms/1.0; ezooms.bot@gmail.com)*/
const Device UA169932 = {S1274, 52, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - RSSIncludeBot/1.0 (http://www.rssinclude.com/spider)*/
const Device UA170637 = {S1275, 36, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - YoonoCrawler/1.0 (crawler@yoono.com)*/
const Device UA170643 = {S1276, 96, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; suggybot v0.01a, http://blog.suggy.com/was-ist-suggy/suggy-webcrawler/)*/
const Device UA170764 = {S1277, 83, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Baiduspider/2.0; +http://www.baidu.com/search/spider.html)*/
const Device UA171843 = {S1278, 49, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - fastbot crawler beta 2.0 (+http://www.fastbot.de)*/
const Device UA172055 = {S1279, 70, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; InfuzApp/1.0; +http://www.infuz.com/bot.html)*/
const Device UA173812 = {S1280, 61, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ZapBot/0.2n; +http://www.zapbot.net)*/
const Device UA173868 = {S1281, 72, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; TweetedTimes Bot/1.0; +http://tweetedtimes.com)*/
const Device UA174933 = {S1282, 10, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - NL-Crawler*/
const Device UA175065 = {S1283, 75, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; NerdByNature.Bot; http://www.nerdbynature.net/bot)*/
const Device UA176043 = {S1284, 64, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; KaloogaBot; http://kalooga.com/crawler)*/
const Device UA180078 = {S1285, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/1.40*/
const Device UA180238 = {S1286, 74, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; musobot/1.0; info@muso.com; +http://www.muso.com)*/
const Device UA180351 = {S1287, 53, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - WillyBot/1.1 (http://www.willyfogg.com/info/willybot)*/
const Device UA185011 = {S1288, 5, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - vcbot*/
const Device UA187503 = {S1289, 67, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexDirect/3.0; +http://yandex.com/bots)*/
const Device UA188947 = {S1290, 14, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Twitterbot/1.0*/
const Device UA189261 = {S1291, 60, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - mysmutsearch-crawler +http://www.mysmutsearch.com/faq.shtml+*/
const Device UA189845 = {S1292, 61, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ZapBot/0.2c; +http://www.zapbot.com)*/
const Device UA189854 = {S1293, 61, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ZapBot/0.2o; +http://www.zapbot.org)*/
const Device UA190718 = {S1294, 65, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexNews/3.0; +http://yandex.com/bots)*/
const Device UA191802 = {S1295, 10, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - woobot/1.1*/
const Device UA196999 = {S1296, 9, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - yolinkBot*/
const Device UA199707 = {S1297, 110, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; pub-crawler; +http://wiki.github.com/bixo/bixo/bixocrawler; bixo-dev@yahoogroups.com)*/
const Device UA200702 = {S1298, 47, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - msnbot/0.01 (+http://search.msn.com/msnbot.htm)*/
const Device UA204265 = {S1299, 81, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; LikaholixCrawler/1.0; +http://mylikes.com/about/crawler)*/
const Device UA204280 = {S1300, 103, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexBot/3.0; +http://yandex.com/bots),gzip(gfe) (via docs.google.com/viewer)*/
const Device UA204423 = {S1301, 26, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Oxyme.Search - Web crawler*/
const Device UA204569 = {S1302, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/1.60*/
const Device UA205242 = {S1303, 69, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Bender; http://benderthewebrobot.tumblr.com)*/
const Device UA205473 = {S1304, 136, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Kyoto-Crawler/2.0 (Mozilla-compatible; kyoto-crawler-contact(at)nlp(dot)kuee(dot)kyoto-u(dot)ac(dot)jp; http://nlp.ist.i.kyoto-u.ac.jp/)*/
const Device UA206349 = {S1305, 10, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Newsle bot*/
const Device UA206992 = {S1306, 42, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SeznamBot/3.0 (+http://fulltext.sblog.cz/)*/
const Device UA208049 = {S1307, 75, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SEOkicks-Robot +http://www.seokicks.de/robot.html)*/
const Device UA208228 = {S1308, 58, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - BacklinkCrawler (http://www.backlinktest.com/crawler.html)*/
const Device UA212615 = {S1309, 86, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Y!J-BRW/1.0 crawler (http://help.yahoo.co.jp/help/jp/search/indexing/indexing-15.html)*/
const Device UA212817 = {S1310, 89, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - MozillaTest/5.0 (compatible; YodaoBot/1.0; http://www.yodao.com/help/webmaster/spider/; )*/
const Device UA215856 = {S1311, 79, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MJ12bot/v1.4.0; http://www.majestic12.co.uk/bot.php?+)*/
const Device UA219041 = {S1312, 71, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MojeekBot/0.2; http://www.mojeek.com/bot.html)*/
const Device UA224167 = {S1313, 60, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - UnisterBot (Mozilla/5.0 compatible; crawler@unister-gmbh.de)*/
const Device UA227398 = {S1314, 61, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Jetslide; +http://jetsli.de/crawler)*/
const Device UA228024 = {S1315, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/1.70*/
const Device UA228355 = {S1316, 51, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - gsa-crawler (Enterprise; GID-01422; jplastiras.com)*/
const Device UA230032 = {S1317, 81, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible;Baiduspider/2.0;+http://www.baidu.com/search/spider.html)*/
const Device UA231217 = {S1318, 70, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SISTRIX Crawler; http://crawler.sistrix.net/)*/
const Device UA232862 = {S1319, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/1,70*/
const Device UA234064 = {S1320, 60, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0+(compatible;+PiplBot;++http://www.pipl.com/bot/)*/
const Device UA239849 = {S1321, 53, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - RediffNewsBot;betatest;contact:searchops@rediff.co.in*/
const Device UA240971 = {S1322, 36, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - AdnormCrawler www.adnorm.com/crawler*/
const Device UA241736 = {S1323, 68, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; LucidWorks/; +; crawler at example dot com)*/
const Device UA241763 = {S1324, 51, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Yepi/1.0 (NHN Corp.; http://help.naver.com/robots/)*/
const Device UA242569 = {S1325, 71, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; oBot/2.3.1; +http://filterdb.iss.net/crawler/)*/
const Device UA243888 = {S1326, 15, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Content Crawler*/
const Device UA248952 = {S1327, 24, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SearQuBot/SearQuBot v1.0*/
const Device UA249443 = {S1328, 72, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexNews/3.0; robot; +http://yandex.com/bots)*/
const Device UA250303 = {S1329, 36, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - X3Watch Crawler (http://x3watch.com)*/
const Device UA250774 = {S1330, 93, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0+(compatible;+CukBot;+Not+a+spammer;+++http://www.companiesintheuk.co.uk/bot.html)*/
const Device UA250842 = {S1331, 44, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - PostPost/1.0 (+http://postpost.com/crawlers)*/
const Device UA252058 = {S1332, 52, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ZumBot/1.0 (ZUM Search; http://help.zum.com/inquiry)*/
const Device UA252558 = {S1333, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/1.80*/
const Device UA252817 = {S1334, 58, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Goodzer/2.0; crawler@goodzer.com)*/
const Device UA253398 = {S1335, 73, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Moatbot/2.2; +http://www.moat.com/pages/moatbot)*/
const Device UA255846 = {S1336, 75, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; WBSearchBot/1.1; +http://www.warebay.com/bot.html)*/
const Device UA257307 = {S1337, 9, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - BufferBot*/
const Device UA257583 = {S1338, 78, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Wepbot/3.0 (+http://www.nurago.com/en/wepbot/); Jakarta Commons-HttpClient/3.1*/
const Device UA258026 = {S1339, 106, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - FAST-WebCrawler/3.8 (crawler at trd dot overture dot com; http://www.alltheweb.com/help/webmaster/crawler)*/
const Device UA258031 = {S1340, 58, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; crawlx, crawler@trd.overture.com)*/
const Device UA259488 = {S1341, 48, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - CCBot/1.0 (+http://www.commoncrawl.org/bot.html)*/
const Device UA259503 = {S1342, 46, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Snapbot/1.0 (Snap Shots, +http://www.snap.com)*/
const Device UA263838 = {S1343, 46, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - thumbshots-de-bot (+http://www.thumbshots.de/)*/
const Device UA266413 = {S1344, 66, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexMedia/3.0; +http://yandex.com/bots)*/
const Device UA269471 = {S1345, 69, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexFavicons/1.0; +http://yandex.com/bots)*/
const Device UA270353 = {S1346, 55, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - librabot/2.0 (+http://academic.research.microsoft.com/)*/
const Device UA274750 = {S1347, 69, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SiteBot/0.1; +http://www.sitebot.org/robot/)*/
const Device UA274967 = {S1348, 4, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ibot*/
const Device UA274973 = {S1349, 54, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - GSiteCrawler/v1.23 rev. 286 (http://gsitecrawler.com/)*/
const Device UA275321 = {S1350, 99, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ia_archiver/1.0; +http://www.alexa.com/help/webmasters; crawler@alexa.com)*/
const Device UA277680 = {S1351, 12, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - IssueCrawler*/
const Device UA281734 = {S1352, 42, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - LocalBot/1.1 (+http://www.localbot.co.uk/)*/
const Device UA294794 = {S1353, 86, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - InnovantageBot/1.0 (http://www.innovantage.co.uk/technology/webmaster_information.htm)*/
const Device UA300739 = {S1354, 72, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; redditbot/1.0; +http://www.reddit.com/feedback)*/
const Device UA303349 = {S1355, 11, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Baiduspider*/
const Device UA303835 = {S1356, 87, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexBlogs/0.99; robot; B; +http://yandex.com/bots) 1 readers*/
const Device UA304097 = {S1357, 75, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; FatBot 2.0; http://www.thefind.com/crawler; panda)*/
const Device UA304770 = {S1358, 64, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Yahoo-Newscrawler/3.9 (news-search-crawler at yahoo-inc dot com)*/
const Device UA305971 = {S1359, 37, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SMXCrawler/1.0 (www.socialmetrix.com)*/
const Device UA306405 = {S1360, 59, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Zimbiobot/2.0; support@zimbio.com)*/
const Device UA311680 = {S1361, 11, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - GigaBot/2.0*/
const Device UA317805 = {S1362, 31, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - private test crawler curl-based*/
const Device UA318302 = {S1363, 69, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - FlightDeckReportsBot/2.0 (http://www.flightdeckreports.com/pages/bot)*/
const Device UA320062 = {S1364, 98, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; EmailMarketingRobot/2.1; +http://www.emailmarketingrobot.com/webmasters/)*/
const Device UA326282 = {S1365, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/1,60*/
const Device UA329806 = {S1366, 68, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SWbot/0.1; +http://app.saidwot.com/bot.html*/
const Device UA335592 = {S1367, 65, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (SpiffySearchBot http://www.spiffysearch.com/bot.php)*/
const Device UA336003 = {S1368, 49, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Qualidator.com Bot 0.9;)*/
const Device UA344541 = {S1369, 40, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - R6_FeedFetcher_(www.radian6.com/crawler)*/
const Device UA345461 = {S1370, 67, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - FAST Enterprise Crawler 6 used by LexisNexis (lnbot@lexisnexis.com)*/
const Device UA346131 = {S1371, 12, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0*/
const Device UA346316 = {S1372, 32, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - 360spider(http://webscan.360.cn)*/
const Device UA346834 = {S1373, 61, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - gsa-crawler (Enterprise; GID-01742;gsatesting@rediffmail.com)*/
const Device UA347086 = {S1374, 10, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - NikoBot1.0*/
const Device UA347112 = {S1375, 45, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - kulokobot www.kuloko.com kuloko@backweave.com*/
const Device UA347686 = {S1376, 42, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Updownerbot (+http://www.updowner.com/bot)*/
const Device UA347748 = {S1377, 100, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; archive.org_bot; Archive-It; +http://archive-it.org/files/site-owners.html)*/
const Device UA349477 = {S1378, 24, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - FTRF: Friendly robot/1.3*/
const Device UA349594 = {S1379, 13, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - minicrawler/1*/
const Device UA350598 = {S1380, 68, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SAE/fetchurl-kw31jl4xol compatible; SiBot; http://www.siteinfor.com/*/
const Device UA350724 = {S1381, 34, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Zookabot/2.4;++http://zookabot.com*/
const Device UA351684 = {S1382, 22, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - InsiteRobot/2011.12.07*/
const Device UA352092 = {S1383, 87, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; GrapeshotCrawler/2.0; +http://www.grapeshot.co.uk/crawler.php)*/
const Device UA352364 = {S1384, 8, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - microbot*/
const Device UA352795 = {S1385, 20, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - LinksCrawler 0.1beta*/
const Device UA359463 = {S1386, 79, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MJ12bot/v1.4.1; http://www.majestic12.co.uk/bot.php?+)*/
const Device UA359701 = {S1387, 71, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Sogou inst spider/4.0(+http://www.sogou.com/docs/help/webmasters.htm#07*/
const Device UA363681 = {S1388, 80, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; WASALive-Bot ; http://blog.wasalive.com/wasalive-bots/)*/
const Device UA365863 = {S1389, 81, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Semager/1.4c; +http://www.semager.de/blog/semager-bots/)*/
const Device UA366662 = {S1390, 81, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; discobot/2.0; +http://discoveryengine.com/discobot.html)*/
const Device UA367531 = {S1391, 44, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - WongBot // http://www.mister-wong.de/search/*/
const Device UA372187 = {S1392, 44, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - NewsGator/2.0 Bot (http://www.newsgator.com)*/
const Device UA372602 = {S1393, 51, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Lijit Crawler (+http://www.lijit.com/robot/crawler)*/
const Device UA374290 = {S1394, 27, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Feedjit Favicon Crawler 1.0*/
const Device UA375119 = {S1395, 13, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - DS/1.5 robots*/
const Device UA375932 = {S1396, 50, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Pressarmybot/beta (+http://beta.pressarmy.com/bot)*/
const Device UA377097 = {S1397, 38, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Castabot/0.1 (+http://topixtream.com/)*/
const Device UA377603 = {S1398, 47, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - FAST Enterprise Crawler/5.3.4 (crawler@fast.no)*/
const Device UA378123 = {S1399, 13, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - scrapybot/1.0*/
const Device UA378247 = {S1400, 16, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Baiduspider-cpro*/
const Device UA382220 = {S1401, 33, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - percbotspider <ops@percolate.com>*/
const Device UA382239 = {S1402, 79, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MJ12bot/v1.4.2; http://www.majestic12.co.uk/bot.php?+)*/
const Device UA382208 = {S1403, 65, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YioopBot; +http://www.yioop.com/bot.php)*/
const Device UA382273 = {S1404, 98, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - FAST Enterprise Crawler 6 / Scirus scirus-crawler@fast.no; http://www.scirus.com/srsapp/contactus/*/
const Device UA382342 = {S1405, 28, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SurveyBot/2.3 (Whois Source)*/
const Device UA382372 = {S1406, 67, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Sogou+web+robot+(+http://www.sogou.com/docs/help/webmasters.htm#07)*/
const Device UA382664 = {S1407, 74, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; VideoSurf_bot +http://www.videosurf.com/bot.html)*/
const Device UA383037 = {S1408, 36, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible: FDSE robot)*/
const Device UA383783 = {S1409, 58, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Swooglebot/2.0. (+http://swoogle.umbc.edu/swooglebot.html)*/
const Device UA383945 = {S1410, 46, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Web CEO Online robot)*/
const Device UA387790 = {S1411, 59, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SmartAndSimpleWebCrawler/1.3 (https://crawler.dev.java.net)*/
const Device UA389154 = {S1412, 42, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (Compatible; Supybot 0.83.4.1)*/
const Device UA389919 = {S1413, 87, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Ruky-Roboter (Version: 1.06, powered by www.ruky.de +http://www.ruky.de/zeige/bot.html)*/
const Device UA389931 = {S1414, 37, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; focuseekbot)*/
const Device UA391889 = {S1415, 68, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Liquida.it-Crawler/1.0 ( crawler@liquida.it +http://www.liquida.it )*/
const Device UA396555 = {S1416, 9, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - JoobleBot*/
const Device UA396971 = {S1417, 56, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Wootbot/1.0; +http://woot.com/)*/
const Device UA400305 = {S1418, 71, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - TweetedTimes Bot/1.0 (Mozilla/5.0 Compatible, +http://tweetedtimes.com)*/
const Device UA401277 = {S1419, 72, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Sogou inst spider/4.0(+http://www.sogou.com/docs/help/webmasters.htm#07)*/
const Device UA401362 = {S1420, 56, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; UnisterBot; crawler@unister.de)*/
const Device UA402007 = {S1421, 12, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - intelium_bot*/
const Device UA402527 = {S1422, 51, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - PagePeeker.com (info: http://pagepeeker.com/robots)*/
const Device UA407893 = {S1423, 19, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/bintellibot*/
const Device UA408642 = {S1424, 42, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - DotNetShoutout-Bot(http://dotnetshout.com)*/
const Device UA409379 = {S1425, 70, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0+(compatible; UptimeRobot/1.0; http://www.uptimerobot.com/)*/
const Device UA412994 = {S1426, 39, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ShowyouBot (http://showyou.com/crawler)*/
const Device UA413500 = {S1427, 63, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Scribdbot/1.0; +http://www.scribd.com)*/
const Device UA413577 = {S1428, 72, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - BoardReader/1.0 (http://boardreader.com/info/robots.htm)-CommentCrawler5*/
const Device UA413736 = {S1429, 4, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Jbot*/
const Device UA416026 = {S1430, 45, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - DealOzBot/7.2 (+http://www.dealoz.com/bot.pl)*/
const Device UA416844 = {S1431, 20, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - OpenSearchServer_Bot*/
const Device UA417294 = {S1432, 9, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - aranhabot*/
const Device UA417410 = {S1433, 13, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - InfohubbleBot*/
const Device UA419140 = {S1434, 43, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; AMZNKAssocBot/4.0)*/
const Device UA419588 = {S1435, 76, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; lexxbot/1.0; +http://www.artiesoft.com/lexxbot.php)*/
const Device UA421151 = {S1436, 73, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ApptusBot/2.0; +http://www.apptus.com/apptusbot)*/
const Device UA422689 = {S1437, 73, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; AskPeterBot; +http://www.askpeter.info/bot.html)*/
const Device UA423742 = {S1438, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/1.90*/
const Device UA427869 = {S1439, 29, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - GG PeekBot 1.0 (http://gg.pl)*/
const Device UA428371 = {S1440, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - checkgzipcompression.com robot*/
const Device UA429980 = {S1441, 63, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - spider-on-the-fly 20q v4.256 Discover Life www.discoverlife.org*/
const Device UA430451 = {S1442, 72, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - FAST Enterprise Crawler 6 used by My Company (administrator@example.com)*/
const Device UA432724 = {S1443, 70, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Widgetbox; http://www.widgetbox.com/bot.html)*/
const Device UA432975 = {S1444, 23, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Kliksafe Search Bot/2.0*/
const Device UA433000 = {S1445, 59, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Ralocobot; +http://raloco.com/bot)*/
const Device UA433694 = {S1446, 80, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SNBot 0.93 on human users' behalf http://www.shoppingnotes.com/help/webmasterfaq*/
const Device UA433766 = {S1447, 39, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - WikioFeedBot 1.0 (http://www.wikio.com)*/
const Device UA436848 = {S1448, 35, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; i4dwebbot)*/
const Device UA436966 = {S1449, 43, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - emefgebot/BETA (+http://emefge.de/bot.html)*/
const Device UA437315 = {S1450, 71, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; AskPeterBot; +http://www.askpeter.de/bot.html)*/
const Device UA437390 = {S1451, 59, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ltbot/1.3; bot@language-tools.com)*/
const Device UA437508 = {S1452, 62, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - WeSEE:Search/0.1 (Alpha, http://www.wesee.com/en/support/bot/)*/
const Device UA438782 = {S1453, 75, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - DomainCrawler/2.0 (info@domaincrawler.com; http://www.domaincrawler.com/bot*/
const Device UA438950 = {S1454, 28, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - HolmesBot (http://holmes.ge)*/
const Device UA439601 = {S1455, 13, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ClickTale bot*/
const Device UA441023 = {S1456, 70, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; DCPbot/1.2; +http://domains.checkparams.com/)*/
const Device UA442099 = {S1457, 65, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; OpenfosBot/2.4; +http://www.openfos.com)*/
const Device UA443538 = {S1458, 32, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Xerka WebBot v1.0.0 [Transporte]*/
const Device UA443904 = {S1459, 58, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (Kwickbot 1.0; http://hilfe.kwick.de/KwickBot)*/
const Device UA444360 = {S1460, 9, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - PGBot/0.1*/
const Device UA444910 = {S1461, 20, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - LinksManager.com_bot*/
const Device UA448171 = {S1462, 16, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - stc-crawler/0.84*/
const Device UA449754 = {S1463, 11, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - suma spider*/
const Device UA450154 = {S1464, 7, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - vspider*/
const Device UA452245 = {S1465, 72, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; LuminateBot/1.0; +http://www.luminate.com/bot/)*/
const Device UA452666 = {S1466, 60, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SEOENGWorldBot/1.0 (+http://www.seoengine.com/seoengbot.htm)*/
const Device UA452703 = {S1467, 59, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Satrix Bot/1.0; +dima@tajseed.com)*/
const Device UA452785 = {S1468, 74, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexSitelinks; Dyatel; +http://yandex.com/bots)*/
const Device UA454070 = {S1469, 9, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Sspider_X*/
const Device UA454981 = {S1470, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - BotLaju v2.0 (http://laju.my/)*/
const Device UA455963 = {S1471, 37, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; BeetleBot; )*/
const Device UA456558 = {S1472, 62, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; aiHitBot/1.1; +http://www.aihit.com/)*/
const Device UA456630 = {S1473, 63, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - GG PeekBot 1.0 ( http://gg.pl/ http://info.gadu-gadu.pl/praca )*/
const Device UA457546 = {S1474, 65, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; aiHitBot-BP/1.1; +http://www.aihit.com/)*/
const Device UA459457 = {S1475, 60, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; Mavicanet robot; www.mavicanet.org)*/
const Device UA459756 = {S1476, 43, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - TinEye/1.1 (http://tineye.com/crawler.html)*/
const Device UA459917 = {S1477, 64, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (MrCarlito-0.1 http://www.mrcarlito.com/spider.html)*/
const Device UA460000 = {S1478, 41, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - LinkAider (http://linkaider.com/crawler/)*/
const Device UA460354 = {S1479, 14, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - 'citeseerxbot'*/
const Device UA460360 = {S1480, 61, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ConveraCrawler/0.9e (+http://ews.converasearch.com/crawl.htm)*/
const Device UA460371 = {S1481, 13, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - aulinkbot/1.0*/
const Device UA460413 = {S1482, 65, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SciSpi v.1.beta - science spider (+http://www.abitofscience.com/)*/
const Device UA460575 = {S1483, 68, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - findlinks/1.1.6-beta1 (+http://wortschatz.uni-leipzig.de/findlinks/)*/
const Device UA460784 = {S1484, 65, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; OpenfosBot/2.1; +http://www.openfos.com)*/
const Device UA461159 = {S1485, 55, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - msnbot-Products/1.0 (+http://search.msn.com/msnbot.htm)*/
const Device UA462368 = {S1486, 18, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - A .NET Web Crawler*/
const Device UA468862 = {S1487, 52, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ALeadSoftbot/2.0 (+http://www.aleadsoft.com/bot.htm)*/
const Device UA473408 = {S1488, 14, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ERACrawler/1.0*/
const Device UA475285 = {S1489, 52, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - msnbot-media/1.0 (+http://search.msn.com/msnbot.htm)*/
const Device UA475788 = {S1490, 7, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - QrobotW*/
const Device UA482876 = {S1491, 56, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - BoardReader/1.0 (http://boardreader.com/info/robots.htm)*/
const Device UA485145 = {S1492, 12, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - utmailspider*/
const Device UA493410 = {S1493, 13, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - my-search-bot*/
const Device UA493490 = {S1494, 29, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - OpenLink Virtuoso RDF crawler*/
const Device UA502573 = {S1495, 58, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Baiduspider-favo+(+http://www.baidu.com/search/spider.htm)*/
const Device UA503451 = {S1496, 103, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SWAT Crawler. AGH University project. In case of problem contact: opal@tempus.metal.agh.edu.pl. Thanks.*/
const Device UA503505 = {S1497, 86, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - MetaTagRobot/2.1 (http://www.seocentro.com/tools/search-engines/metatag-analyzer.html)*/
const Device UA503677 = {S1498, 109, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Influencebot/0.9; (Automatic classification of websites; http://www.influencebox.com/; info@influencebox.com)*/
const Device UA505417 = {S1499, 51, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - fastbot.de crawler 2.0 beta (http://www.fastbot.de)*/
const Device UA509985 = {S1500, 9, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SocSciBot*/
const Device UA512003 = {S1501, 68, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexCatalog/3.0; +http://yandex.com/bots)*/
const Device UA512559 = {S1502, 49, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - webLyzard/1.1 (+http://www.webLyzard.com/crawler)*/
const Device UA515291 = {S1503, 16, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - shopstylebot/1.0*/
const Device UA516490 = {S1504, 33, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - RSSingBot (http://www.rssing.com)*/
const Device UA516713 = {S1505, 62, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - BejiBot Crawler (BNL Services; http://www.bejjan.net/crawler/)*/
const Device UA516821 = {S1506, 45, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Startsidenbot/1.0 (+http://www.startsiden.no)*/
const Device UA516868 = {S1507, 19, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ServiceUptime.robot*/
const Device UA517804 = {S1508, 16, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Baiduspider-sfkr*/
const Device UA517820 = {S1509, 61, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ApptusBot/2.0; +http://www.eniro.se)*/
const Device UA519144 = {S1510, 54, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - RyzeCrawler/1.1.1 (+http://www.domain2day.nl/crawler/)*/
const Device UA522103 = {S1511, 54, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - businessdbbot/v1.2 (http://www.businessdb.com/bot.php)*/
const Device UA522216 = {S1512, 31, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Xerka WebBot v1.0.0 [Tecnotour]*/
const Device UA522783 = {S1513, 44, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rbot http util 0.9.15 (http://ruby-rbot.org)*/
const Device UA522942 = {S1514, 15, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Traackr.com Bot*/
const Device UA523812 = {S1515, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/1.10*/
const Device UA524018 = {S1516, 65, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - DomainTaggingbot; +http://www.opendns.com/community/domaintagging*/
const Device UA524281 = {S1517, 52, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Doubanbot/1.0 (bot@douban.com http://www.douban.com)*/
const Device UA524322 = {S1518, 17, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Robots.txt finder*/
const Device UA525395 = {S1519, 18, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - sedo_parking_robot*/
const Device UA527596 = {S1520, 70, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; sharegloo_bot/1.0; +http://www.sharegloo.com)*/
const Device UA530024 = {S1521, 16, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - WikioFeedBot 1.0*/
const Device UA530916 = {S1522, 117, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0  (compatible; Wer-liefert-was Crawler 1.1; http://www.wlw.de/extern/crawler/Wer-liefert-was-Crawler.html)*/
const Device UA530924 = {S1523, 9, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SearchBot*/
const Device UA532631 = {S1524, 69, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SocialOomphLinkBot/1.0 (info@socialoomph.com http://socialoomph.com/)*/
const Device UA533622 = {S1525, 13, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - cis455crawler*/
const Device UA534717 = {S1526, 93, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Scanmine newsspider. See http://www.scanmine.com for more information. NB: We need css files!*/
const Device UA536761 = {S1527, 15, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - your-search-bot*/
const Device UA540010 = {S1528, 73, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0+(compatible;+OkidokiBot/0.1;++http://www.okidoki.ro/bot.html)*/
const Device UA540633 = {S1529, 70, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexWebmaster/2.0; +http://yandex.com/bots)*/
const Device UA540721 = {S1530, 49, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - sunwukong bot/1.0 (+http://www.swkong.com/addurl)*/
const Device UA545417 = {S1531, 84, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - MetaTagRobot/1.9 (http://www.widexl.com/remote/search-engines/metatag-analyzer.html)*/
const Device UA545597 = {S1532, 87, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexBlogs/0.99; robot; B; +http://yandex.com/bots) 2 readers*/
const Device UA548426 = {S1533, 77, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; BuzzRankingBot/1.0; +http://www.buzzrankingbot.com/)*/
const Device UA552149 = {S1534, 10, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - My Crawler*/
const Device UA552834 = {S1535, 73, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - FAST Enterprise Crawler/6.7.0 used by ThomasNet.com (admin@thomasnet.com)*/
const Device UA553278 = {S1536, 56, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.75 (Nikto/2.1.4) (Evasions:None) (Test:robots)*/
const Device UA558932 = {S1537, 36, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - OdklBot/1.0 (klass@odnoklassniki.ru)*/
const Device UA558976 = {S1538, 55, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SEOENGBot/1.2 (+http://www.seoengine.com/seoengbot.htm)*/
const Device UA563278 = {S1539, 10, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - eduCrawler*/
const Device UA564485 = {S1540, 76, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - KeepRightBot/0.2 (KeepRight OpenStreetMap Checker; http://keepright.ipax.at)*/
const Device UA568075 = {S1541, 120, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Update Profile Bot/1.0 (compatible; NZS.com mini-spider,  Local Search New Zealand; http://www.nzs.com/; bot at nzs.com)*/
const Device UA568874 = {S1542, 40, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; merlinkbot/1.0)*/
const Device UA571603 = {S1543, 47, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Aboundex/0.2 (http://www.aboundex.com/crawler/)*/
const Device UA573184 = {S1544, 10, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Pastec bot*/
const Device UA573562 = {S1545, 68, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; semvisuBot 2.0; +http://fr.semvisu.com/bot)*/
const Device UA573572 = {S1546, 70, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; jobs.de-Robot +http://www.jobs.de/robot.html)*/
const Device UA574691 = {S1547, 49, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - BuiBui-Bot/1.0 (email: buibui[at]dadapro[dot]com)*/
const Device UA577067 = {S1548, 27, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - AdBot 1.0 (www.adnetik.com)*/
const Device UA577457 = {S1549, 78, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - FreeWebMonitoring SiteChecker/0.2 (+http://www.freewebmonitoring.com/bot.html)*/
const Device UA577656 = {S1550, 60, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; geourl/3.1 - http://geourl.org/bot)*/
const Device UA584133 = {S1551, 53, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Whoismindbot/1.0 (+http://www.whoismind.com/bot.html)*/
const Device UA584774 = {S1552, 36, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - x28-job-bot; +http://x28.ch/bot.html*/
const Device UA588392 = {S1553, 96, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; special_archiver/3.1.1 +http://www.archive.org/details/archive.org_bot)*/
const Device UA588690 = {S1554, 87, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; metager2-verification-bot; +http://metager2.de/technology.php)*/
const Device UA591061 = {S1555, 8, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - 100k_bot*/
const Device UA596902 = {S1556, 52, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Statastico DK bot - Learn more at: statastico.dk/bot*/
const Device UA599562 = {S1557, 37, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SensikaBot/x.33 (+http://sensika.com)*/
const Device UA600509 = {S1558, 17, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - xabaducrawler/0.9*/
const Device UA603047 = {S1559, 118, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; c) AppleWebKit/531.2+ (KHTML, like Gecko) Version/5.0 WatchMouse/$Revision: 24072 $*/
const Device UA603614 = {S1560, 37, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Mail.RU/2.0)*/
const Device UA603869 = {S1561, 70, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Embedly/0.2; snap; +http://support.embed.ly/)*/
const Device UA605184 = {S1562, 93, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Imperialbot/1.2; +http://http://www.imperialdirectory.com/ContactUs)*/
const Device UA605686 = {S1563, 35, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (PHP 5) PPBBCrawler 0.1*/
const Device UA608049 = {S1564, 39, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; news bot /2.1)*/
const Device UA609001 = {S1565, 73, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; CareerBot/1.1; +http://www.career-x.de/bot.html)*/
const Device UA609257 = {S1566, 49, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - RSSInclude/1.0 (http://www.rssinclude.com/spyder)*/
const Device UA609603 = {S1567, 48, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - adidxbot/2.0 (+http://search.msn.com/msnbot.htm)*/
const Device UA609705 = {S1568, 79, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MJ12bot/v1.4.3; http://www.majestic12.co.uk/bot.php?+)*/
const Device UA610144 = {S1569, 12, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Explorer Bot*/
const Device UA613019 = {S1570, 78, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - FocusedCrawler (http://www2.informatik.hu-berlin.de/~brettsch/focusedcrawler/)*/
const Device UA614852 = {S1571, 31, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Cutbot; 1.5; http://cutbot.net/*/
const Device UA615858 = {S1572, 60, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - findlinks/2.6 (+http://wortschatz.uni-leipzig.de/findlinks/)*/
const Device UA622957 = {S1573, 13, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - MoodleBot/1.0*/
const Device UA622967 = {S1574, 57, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Whitevector Crawler (+http://www.whitevector.com/crawler)*/
const Device UA622990 = {S1575, 14, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Phantom.js bot*/
const Device UA622993 = {S1576, 88, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - taptubot; requests from this IP address are taptubot, see http://taptu.com/corp/taptubot*/
const Device UA623371 = {S1577, 95, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; archive.org_bot/3.1.1 +http://www.archive.org/details/archive.org_bot)*/
const Device UA624172 = {S1578, 45, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Curious George - www.analyticsseo.com/crawler*/
const Device UA625260 = {S1579, 11, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - AdMedia bot*/
const Device UA625696 = {S1580, 54, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - BuiBui-CheckBot/1.0 (email: buibui[at]dadapro[dot]com)*/
const Device UA625844 = {S1581, 12, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - PopScreenBot*/
const Device UA627505 = {S1582, 52, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - MeMoNewsBot/2.0 (http://www.memonews.com/en/crawler)*/
const Device UA627965 = {S1583, 41, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - AC-BaiduBot/1.1(+http://www.ac-baidu.com)*/
const Device UA628540 = {S1584, 80, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexBot/3.0; MirrorDetector; +http://yandex.com/bots)*/
const Device UA629020 = {S1585, 103, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; FriendFeedBot/0.1; +Http://friendfeed.com/about/bot; 3 subscribers; feed-id=X)*/
const Device UA629100 = {S1586, 56, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Healthbot/Health_and_Longevity_Project_(HealthHaven.com)*/
const Device UA629869 = {S1587, 83, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0+(compatible;+Baiduspider/2.0;++http://www.baidu.com/search/spider.html)*/
const Device UA632047 = {S1588, 41, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - QippoBot/1.0 (+http://www.qippo.com/bot/)*/
const Device UA633552 = {S1589, 66, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexNewslinks; +http://yandex.com/bots)*/
const Device UA634149 = {S1590, 87, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexBlogs/0.99; robot; B; +http://yandex.com/bots) 0 readers*/
const Device UA635294 = {S1591, 11, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SiteCrawler*/
const Device UA636270 = {S1592, 10, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - dirbot/1.0*/
const Device UA637193 = {S1593, 90, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Sosoimagespider/2.0; +http://help.soso.com/soso-image-spider.htm)*/
const Device UA637641 = {S1594, 12, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - i-market-bot*/
const Device UA640695 = {S1595, 52, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Abrave Spider v5.3 Robot 1 (http://robot.abrave.com)*/
const Device UA642786 = {S1596, 19, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - DiBot Java/1.5.0_04*/
const Device UA643332 = {S1597, 47, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - EasyDL/3.04  http://keywen.com/Encyclopedia/Bot*/
const Device UA645060 = {S1598, 7, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - betaBot*/
const Device UA645148 = {S1599, 83, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; archive.bibalex.org_bot; +http://archive.bibalex.org/bot/)*/
const Device UA645171 = {S1600, 17, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - TextTrust Crawler*/
const Device UA648818 = {S1601, 78, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Nesotebot/2.1; +http://www.inoutscripts.com/bot.html)*/
const Device UA649482 = {S1602, 85, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible;wi jobroboter spider 4 for ams;+http://www.webintegration.at)*/
const Device UA651349 = {S1603, 76, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Industrial Interface Web Crawler - http://www.industrycortex.com/crawler.php*/
const Device UA652215 = {S1604, 11, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - GermCrawler*/
const Device UA652337 = {S1605, 10, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - COIBot/2.0*/
const Device UA659298 = {S1606, 68, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 (Toread-Crawler/1.1; +http://news.toread.cc/crawler.php)*/
const Device UA663373 = {S1607, 24, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 QunarBot/1.0*/
const Device UA663836 = {S1608, 87, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Saralbot/4.2; +http://www.saralweb.com/cache/public/pages/bot)*/
const Device UA664895 = {S1609, 47, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - BaiduImagespider+(+http://www.baidu.jp/spider/)*/
const Device UA664916 = {S1610, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; DotBot/1.1; http://www.dotnetdotcom.org/, crawler@dotnetdotcom.org)*/
const Device UA664964 = {S1611, 33, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - LexxeBot/1.0 (lexxebot@lexxe.com)*/
const Device UA666050 = {S1612, 84, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Exabot/3.0 (BiggerBetter); +http://www.exabot.com/go/robot)*/
const Device UA666089 = {S1613, 43, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SheenBot/SheenBot-1.0.4 (Sheen web crawler)*/
const Device UA667067 = {S1614, 52, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - msnbot/2.0b (+http://search.msn.com/msnbot.htm) Test*/
const Device UA667229 = {S1615, 74, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; AboutUsBot Johnny5/2.0; +http://www.AboutUs.org/)*/
const Device UA667844 = {S1616, 42, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - wf_crawler (http://www.websitefigures.com)*/
const Device UA668092 = {S1617, 46, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - msnbot/1.0 (+http://search.msn.com/msnbot.htm)*/
const Device UA671048 = {S1618, 57, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Linguee Bot (http://www.linguee.com/bot; bot@linguee.com)*/
const Device UA672381 = {S1619, 60, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SummizeBot +http://www.summize.com)*/
const Device UA677204 = {S1620, 109, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Baiduspider/2.0; +http://www.baidu.com/search/spider.html) (via babelfish.yahoo.com)*/
const Device UA679558 = {S1621, 154, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Baiduspider/2.0; +http://www.baidu.com/search/spider.html) AppEngine-Google; (+http://code.google.com/appengine; appid: surf6009)*/
const Device UA688428 = {S1622, 64, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - webmeasurement-bot, http://rvs.informatik.uni-leipzig.de/bot.php*/
const Device UA692615 = {S1623, 63, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Herold Crawler; +http://www.herold.at)*/
const Device UA692876 = {S1624, 60, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; http://www.UptimeAuditor.com/bot/ )*/
const Device UA692959 = {S1625, 8, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - WebRobot*/
const Device UA696372 = {S1626, 127, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SecretSerachEngineLabs.com-SBSearch/0.9; http://www.secretsearchenginelabs.com/secret-web-crawler.php)*/
const Device UA700367 = {S1627, 15, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - JavaCrawler/1.1*/
const Device UA704556 = {S1628, 66, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - PTech/1.0 (crawler@positiontech.com; http://www.positiontech.com/)*/
const Device UA704867 = {S1629, 13, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Jayde crawler*/
const Device UA708212 = {S1630, 66, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; dlcbot/0.1; +http://www.drlinkcheck.com/)*/
const Device UA712451 = {S1631, 40, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - pd-crawler (pdsupport@purediscovery.com)*/
const Device UA721603 = {S1632, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/1,80*/
const Device UA722503 = {S1633, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/1,90*/
const Device UA726156 = {S1634, 10, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Crawler0.1*/
const Device UA738267 = {S1635, 19, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rapid-miner-crawler*/
const Device UA744262 = {S1636, 52, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Abrave Spider v5.3 Robot 2 (http://robot.abrave.com)*/
const Device UA746314 = {S1637, 22, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - CodeGator Crawler v1.0*/
const Device UA747648 = {S1638, 22, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Xenu Link Sleuth/1.3.6*/
const Device UA748219 = {S1639, 58, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - gsa-crawler (Enterprise; GID-01422; jplastiras@google.com)*/
const Device UA750157 = {S1640, 20, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - naverbookmarkcrawler*/
const Device UA752436 = {S1641, 56, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ldspider (http://code.google.com/p/ldspider/wiki/Robots)*/
const Device UA764678 = {S1642, 55, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - netEstate Trend Crawler (+http://www.trendmonitor.org/)*/
const Device UA766043 = {S1643, 111, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/3.0 (INGRID/3.0 MT; webcrawler@NOSPAMexperimental.net; http://aanmelden.ilse.nl/?aanmeld_mode=webhints)*/
const Device UA768937 = {S1644, 46, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - bot@1web.ro/1.1 (+http://www.1web.ro/bot.html)*/
const Device UA773033 = {S1645, 66, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - BilgiBot/1.0(beta) (http://www.bilgi.com/; bilgi at bilgi dot com)*/
const Device UA777266 = {S1646, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/1,40*/
const Device UA778022 = {S1647, 75, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; ChristCrawler.com ChristCrawler@ChristCENTRAL.com)*/
const Device UA788274 = {S1648, 78, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; PWeBot/3.1; http://www.programacionweb.net/robot.php)*/
const Device UA789460 = {S1649, 38, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - r2iBot/1.0 http://www.r2integrated.com*/
const Device UA789850 = {S1650, 67, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; HomeTags/1.0; +http://www.hometags.nl/bot)*/
const Device UA801246 = {S1651, 37, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Site-Perf.com performance testing bot*/
const Device UA801594 = {S1652, 17, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - websphinx.Crawler*/
const Device UA820572 = {S1653, 40, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; CrawlBot/1.0.0)*/
const Device UA831471 = {S1654, 38, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - WebZIP/7.0 (http://www.spidersoft.com)*/
const Device UA836682 = {S1655, 11, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Test Webbot*/
const Device UA842566 = {S1656, 53, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - squirrobot/1.0 (http://intro.squirro.com/squirrobot/)*/
const Device UA847808 = {S1657, 78, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; 008/0.83; http://www.80legs.com/spider.html;) Gecko/X*/
const Device UA858183 = {S1658, 65, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla 4.0(compatible; BotSeer/1.0; +http://botseer.ist.psu.edu)*/
const Device UA862224 = {S1659, 7, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - bot/1.0*/
const Device UA863625 = {S1660, 79, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SinaaBot/1.0; +http://www.marketdefender.com/bot.html)*/
const Device UA864457 = {S1661, 71, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; spbot/2.0.4; +http://www.seoprofiler.com/bot )*/
const Device UA867421 = {S1662, 17, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SiteCrawler/1.1.3*/
const Device UA868961 = {S1663, 89, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; alexa verifybot/1.0; +http://www.alexa.com/help; help@alexa.com)*/
const Device UA871532 = {S1664, 82, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 (Mozilla; http://www.mozilla.org/docs/en/bot.html; master@mozilla.com)*/
const Device UA871831 = {S1665, 81, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; BlogRefsBot/0.1; http://www.blogrefs.com/about/bloggers)*/
const Device UA874725 = {S1666, 6, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - msnbot*/
const Device UA884931 = {S1667, 63, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Peew/1.0; http://www.peew.de/crawler/)*/
const Device UA890199 = {S1668, 17, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SuperPagesBot/0.1*/
const Device UA891938 = {S1669, 98, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; bixolabs/1.0; +http://bixolabs.com/crawler/general; crawler@bixolabs.com)*/
const Device UA896061 = {S1670, 65, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org, rogerbot-crawler@seomoz.org)*/
const Device UA896091 = {S1671, 233, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - MIA DEV/search:robot/0.0.1 (This is the MIA Bot - crawling for mia research project. If you feel unhappy and do not want to be visited by our crawler send an email to spider@neofonie.de; http://spider.neofonie.de; spider@neofonie.de)*/
const Device UA896150 = {S1672, 120, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Finderbots finder bot; +http://wiki.github.com/bixo/bixo/bixocrawler; bixo-dev@yahoogroups.com)*/
const Device UA897526 = {S1673, 14, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - QroboDomainBot*/
const Device UA897839 = {S1674, 94, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - WocBot/Mozilla/5.0 (Wocodi Web Crawler 1.0; http://www.wocodi.com/crawler; crawler@wocodi.com)*/
const Device UA900399 = {S1675, 66, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Y!J-BSC/1.0 crawler (http://help.yahoo.co.jp/help/jp/blog-search/)*/
const Device UA900653 = {S1676, 71, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; imrbot/1.01.18 +http://www.internetmemory.org)*/
const Device UA900911 = {S1677, 12, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ZillaCrawler*/
const Device UA900919 = {S1678, 59, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; newslebot/1.0; +http://newsle.com)*/
const Device UA901023 = {S1679, 6, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - QroBot*/
const Device UA901559 = {S1680, 53, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - GrepNetstat.com Bot/1.0; +http://www.grepnetstat.com)*/
const Device UA901853 = {S1681, 15, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - 123pagesbot/1.0*/
const Device UA903098 = {S1682, 124, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; archive.org_bot/3.1.2 +http://www.bne.es/es/LaBNE/PreservacionDominioES/AvisoWebmasters/index.html)*/
const Device UA908708 = {S1683, 77, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; GrepNetstatBot/1.0; +http://www.grepnetstat.com/bot)*/
const Device UA908776 = {S1684, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/2.00*/
const Device UA909131 = {S1685, 52, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - wsAnalyzer/1.0; ++http://www.wsanalyzer.com/bot.html*/
const Device UA910190 = {S1686, 41, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Wotbox/2.01 (+http://www.wotbox.com/bot/)*/
const Device UA911698 = {S1687, 5, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rbbot*/
const Device UA912027 = {S1688, 79, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; DomainVader/1.0; +http://domainvader.com/bot/info.php)*/
const Device UA912232 = {S1689, 27, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - www.integromedb.org/Crawler*/
const Device UA912613 = {S1690, 75, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ProCogBot/1.0; +http://www.procog.com/spider.html)*/
const Device UA913482 = {S1691, 11, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - atwatch-bot*/
const Device UA914535 = {S1692, 58, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - brainbot/1.0 (digitalbrain@brainstreammedia @digitalbrain)*/
const Device UA914783 = {S1693, 81, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ClearSpike_bot/1.0; +http://www.clearspike.com/bot.html)*/
const Device UA914895 = {S1694, 10, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - spider/1.0*/
const Device UA923786 = {S1695, 14, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - woopingbot/1.1*/
const Device UA923819 = {S1696, 94, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SearchmetricsBot; http://www.searchmetrics.com/en/searchmetrics-bot/)*/
const Device UA923852 = {S1697, 54, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Abrave Spider v5.3 Robot 1 (http://robot.abrave.co.uk)*/
const Device UA924099 = {S1698, 83, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; linkfluence; http://linkfluence.net/; bot@linkfluence.net)*/
const Device UA924438 = {S1699, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/2.01*/
const Device UA924473 = {S1700, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/2,01*/
const Device UA924824 = {S1701, 91, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; WebNL; +http://www.web.nl/webmasters/spider.html; helpdesk@web.nl)*/
const Device UA924964 = {S1702, 76, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - riight.com/RIIGHT-1.1 (crawler; http://riight.com/spider; spider@riight.com)*/
const Device UA925089 = {S1703, 76, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; proximic; +http://www.proximic.com/info/spider.php)*/
const Device UA925205 = {S1704, 72, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SIBot/1.0; +http://www.profound.net/sibot.html)*/
const Device UA925369 = {S1705, 108, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - OSS-bot/0.02 (see http://michaelnielsen.org/blog/oss-bot/ or contact Michael Nielsen, mn@michaelnielsen.org)*/
const Device UA1836241 = {S1706, 31, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - MOMspider/1.00 libwww-perl/0.40*/
const Device UA1839103 = {S1707, 63, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ilocal company search (+http://www.ilocal.nl/about/crawler.jsp)*/
const Device UA1839305 = {S1708, 78, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (Yahoo-MMCrawler/4.0; mailto:vertical-crawl-support@yahoo-inc.com)*/
const Device UA1840266 = {S1709, 6, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - nb-bot*/
const Device UA1842254 = {S1710, 56, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - netEstate NE Crawler (+http://www.website-datenbank.de/)*/
const Device UA1858522 = {S1711, 77, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexBlogs/0.99; robot; B; +http://yandex.com/bots)*/
const Device UA1866140 = {S1712, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/1.30*/
const Device UA1866429 = {S1713, 67, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Zenbot/1.3; +http://zen.co.za/webmasters/)*/
const Device UA1867016 = {S1714, 28, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Open Web Analytics Bot 1.4.1*/
const Device UA1871063 = {S1715, 12, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Uptime Robot*/
const Device UA1875120 = {S1716, 79, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; BecomeBot/1.23; http://www.become.com/webmasters.html)*/
const Device UA1875286 = {S1717, 15, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SAS Web Crawler*/
const Device UA1879191 = {S1718, 38, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - FAST Search Web Crawler 14.0.0325.0000*/
const Device UA1879853 = {S1719, 69, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Zenbot/2.6.1; +http://zen.co.za/webmasters/)*/
const Device UA1881269 = {S1720, 63, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - User-Agent: Lijit Crawler (+http://www.lijit.com/robot/crawler)*/
const Device UA1882206 = {S1721, 96, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Proximic crawler; +http://www.proximic.com/en/about-us/contact-us.html)*/
const Device UA1884043 = {S1722, 70, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ShunixBot/1.x; http://www.shunix.com/bot.htm)*/
const Device UA1886081 = {S1723, 104, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 [en] (compatible; Gulper Web Bot 0.2.4 www.ecsl.cs.sunysb.edu/~maxim/cgi-bin/Link/GulperBot)*/
const Device UA1887341 = {S1724, 41, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Sosospider+(+help.soso.com/webspider.htm)*/
const Device UA1893602 = {S1725, 68, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Bot; +http://pressemitteilung.ws/spamfilter*/
const Device UA1893763 = {S1726, 87, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; archive.org_bot/1.10.0 +http://www.loc.gov/minerva/crawl.html)*/
const Device UA1894259 = {S1727, 94, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; archive.org_bot/1.5.0-X http://crawler.archive.org) Hurricane Katrina*/
const Device UA1894671 = {S1728, 86, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; GPU p2p crawler http://gpu.sourceforge.net/search_engine.php)*/
const Device UA1894741 = {S1729, 76, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Synoobot/0.9; http://www.synoo.com/search/bot.html)*/
const Device UA1894852 = {S1730, 66, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Charlotte/1.0b; charlotte@betaspider.com)*/
const Device UA1894869 = {S1731, 67, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.7 (compatible; WhizBang; http://www.whizbang.com/crawler)*/
const Device UA1894938 = {S1732, 12, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - YakazBot/1.0*/
const Device UA1895165 = {S1733, 80, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (Clustered-Search-Bot/1.0; support@clush.com; http://www.clush.com/)*/
const Device UA1895355 = {S1734, 71, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; AnsearchBot/1.x; +http://www.ansearch.com.au/)*/
const Device UA1895481 = {S1735, 62, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/3.0 (compatible; ScollSpider; http://www.webwobot.com)*/
const Device UA1895542 = {S1736, 64, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; robtexbot/1.0; http://www.robtex.com/ )*/
const Device UA1895615 = {S1737, 69, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Galbot/1.0; +http://www.galbot.com/bot.html)*/
const Device UA1895726 = {S1738, 56, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SkreemRBot +http://skreemr.com)*/
const Device UA1895990 = {S1739, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/3.0 (Vagabondo/2.0 MT; webcrawler@NOSPAMwise-guys.nl; http://webagent.wise-guys.nl/)*/
const Device UA1896169 = {S1740, 103, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 compatible FurlBot/Furl Search 2.0 (FurlBot; http://www.furl.net; wn.furlbot@looksmart.net)*/
const Device UA1896208 = {S1741, 94, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/3.0 (compatible; Fluffy the spider; http://www.searchhippo.com/; info@searchhippo.com)*/
const Device UA1896238 = {S1742, 104, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible;archive.org_bot/1.7.1; collectionId=316; Archive-It; +http://www.archive-it.org)*/
const Device UA1896568 = {S1743, 74, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ShunixBot/1.x.x +http://www.shunix.com/robot.htm)*/
const Device UA1896647 = {S1744, 52, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.7 (compatible; http://eidetica.com/spider)*/
const Device UA1896697 = {S1745, 79, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Webduniabot/1.0; +http://search.webdunia.com/bot.aspx)*/
const Device UA1896915 = {S1746, 76, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; archive.org_bot/1.13.1x http://crawler.archive.org)*/
const Device UA1896942 = {S1747, 88, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible;FindITAnswersbot/1.0;+http://search.it-influentials.com/bot.htm)*/
const Device UA1897881 = {S1748, 102, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Vagabondo/2.1; webcrawler at wise-guys dot nl; http://webagent.wise-guys.nl/)*/
const Device UA1898270 = {S1749, 62, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Diffbot/0.1; +http://www.diffbot.com)*/
const Device UA1898620 = {S1750, 67, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; FastCrawler3 support-fastcrawler3@fast.no)*/
const Device UA1899378 = {S1751, 37, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.7 [en](BecomeBot@exava.com)*/
const Device UA1900489 = {S1752, 114, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/3.0 (Vagabondo/2.0 MT; webcrawler@NOSPAMexperimental.net; http://aanmelden.ilse.nl/?aanmeld_mode=webhints)*/
const Device UA1900803 = {S1753, 38, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SpurlBot/0.2)*/
const Device UA1902727 = {S1754, 57, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; http://www.IsMySiteUp.Net/bot/ )*/
const Device UA1903101 = {S1755, 71, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MojeekBot/2.0; http://www.mojeek.com/bot.html)*/
const Device UA1904045 = {S1756, 71, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.04 (compatible; Dulance bot; +http://www.dulance.com/bot.jsp)*/
const Device UA1904955 = {S1757, 82, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; BecomeBot/2.0beta; http://www.become.com/webmasters.html)*/
const Device UA1907659 = {S1758, 66, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; XTbot/1.0v; +http://www.externaltest.com)*/
const Device UA1915906 = {S1759, 82, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0+(compatible;+Baiduspider/2.0;++http://www.baidu.com/search/spider.html*/
const Device UA1923970 = {S1760, 75, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - TppRFbot/1.1 (compatible; http://www.ruschamber.net, http://www.geocci.com)*/
const Device UA1925243 = {S1761, 17, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - NAVER Blog Rssbot*/
const Device UA1935965 = {S1762, 61, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Quantcastbot/1.0; www.quantcast.com)*/
const Device UA1937016 = {S1763, 74, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Wepbot/3.0 (+http://www.nurago.de/wepbot/); Jakarta Commons-HttpClient/3.1*/
const Device UA1966719 = {S1764, 17, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - PrestaShopBot/1.1*/
const Device UA1966839 = {S1765, 56, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; 3W1bot/2.6; +http://3w1.eu/bot)*/
const Device UA1966932 = {S1766, 17, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Zing-BottaBot/2.0*/
const Device UA1979728 = {S1767, 84, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Baiduspider-cpro; +http://www.baidu.com/search/spider.html)*/
const Device UA1996879 = {S1768, 17, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - WebCompanyCrawler*/
const Device UA2003652 = {S1769, 88, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - bl.uk_lddc_bot/3.1.1 (+http://www.bl.uk/aboutus/stratpolprog/digi/domresproj/index.html)*/
const Device UA2003902 = {S1770, 50, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - bot-pge.chlooe.com/1.0.0 (+http://www.chlooe.com/)*/
const Device UA2004631 = {S1771, 34, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Zookabot/2.5;++http://zookabot.com*/
const Device UA2005711 = {S1772, 35, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - coccoc/1.0 (http://help.coccoc.vn/)*/
const Device UA2017134 = {S1773, 50, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Scope PreviewBot/1.3 (+http://www.getscopeapp.com)*/
const Device UA2017940 = {S1774, 81, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Infohelfer/1.3.0; +http://www.infohelfer.de/crawler.php)*/
const Device UA2017972 = {S1775, 76, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0(compatible; Sosospider/2.0; +http://help.soso.com/webspider.htm)*/
const Device UA2039654 = {S1776, 20, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Sogou web spider/4.0*/
const Device UA2040389 = {S1777, 38, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - LSSRocketCrawler/1.0 LightspeedSystems*/
const Device UA2040852 = {S1778, 75, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SemrushBot/0.95; +http://www.semrush.com/bot.html)*/
const Device UA2042610 = {S1779, 77, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler@seomoz.org)*/
const Device UA2131047 = {S1780, 19, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - QuickFinder Crawler*/
const Device UA2132711 = {S1781, 57, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Baiduspider+(+http://www.baidu.com/search/spider_jp.html)*/
const Device UA2132813 = {S1782, 7, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Linkbot*/
const Device UA2132991 = {S1783, 84, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - FAST-WebCrawler/3.6 (atw-crawler at fast dot no; http://fast.no/support/crawler.asp)*/
const Device UA2135411 = {S1784, 61, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - aipbot/1.0 (aipbot; http://www.aipbot.com; aipbot@aipbot.com)*/
const Device UA2137178 = {S1785, 132, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - WinistaBot/1.6.7-dev (Winista - Crwaler for HTTP content indexing; http://www.netomatix.com/Winista/bot.html; support@netomatix.com)*/
const Device UA2137567 = {S1786, 102, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; Vagabondo/2.2; webcrawler at wise-guys dot nl; http://webagent.wise-guys.nl/)*/
const Device UA2142690 = {S1787, 71, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; IDBot/1.0; +http://www.id-search.org/bot.html)*/
const Device UA2143301 = {S1788, 68, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; PagestackerBot; http://www.pagestacker.com)*/
const Device UA2145421 = {S1789, 78, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; WGToolsBot/1.0; http://wgtools.com/link-checker-bot/)*/
const Device UA2145497 = {S1790, 106, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; Vagabondo/4.0Beta; webcrawler at wise-guys dot nl; http://webagent.wise-guys.nl/)*/
const Device UA2148809 = {S1791, 155, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Baiduspider/2.0; +http://www.baidu.com/search/spider.html) AppEngine-Google; (+http://code.google.com/appengine; appid: proxy6000)*/
const Device UA2149023 = {S1792, 17, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ProductAdsBot/1.0*/
const Device UA2149338 = {S1793, 70, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; BanBots/2.0b; Fetch; +http://www.banbots.com)*/
const Device UA2150355 = {S1794, 35, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Tbot/1.0;)*/
const Device UA2152813 = {S1795, 66, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; DonkeyBot/0.1; +http://www.dasdonkey.com)*/
const Device UA2152925 = {S1796, 15, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Junut Bot 1.0.2*/
const Device UA2153686 = {S1797, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/2,00*/
const Device UA2156234 = {S1798, 46, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; WebCeoOnlineBot/1.0;)*/
const Device UA2157259 = {S1799, 98, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - NextGenSearchBot 69 (for information visit http://www.zoominfo.com/business/nextgensearchbot.html)*/
const Device UA2160539 = {S1800, 62, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - gsa-crawler (Enterprise; GID-01422; pabloeinstaincordeiro.com)*/
const Device UA2162419 = {S1801, 54, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - CrawlerProcess (http://www.PowerMapper.com) /4.7.566.0*/
const Device UA2163738 = {S1802, 28, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Open Web Analytics Bot 1.5.4*/
const Device UA2164770 = {S1803, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/3.0 (Vagabondo/1.1 MT; webcrawler@NOSPAMwise-guys.nl; http://webagent.wise-guys.nl/)*/
const Device UA2167103 = {S1804, 71, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Peepowbot/1.0; +http://www.peepow.com/bot.php)*/
const Device UA2167300 = {S1805, 58, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - GarlikCrawler/1.2 (http://garlik.com/, crawler@garlik.com)*/
const Device UA2167946 = {S1806, 61, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; NewsCatchBot\/1.1; +http://mrnet.in)*/
const Device UA2168044 = {S1807, 36, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - item-db bot/1.0(http://item-db.net/)*/
const Device UA2168266 = {S1808, 91, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; bixolabs; +crawler@scaleunlimited.com; crawler@scaleunlimited.com)*/
const Device UA2187551 = {S1809, 58, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - omgilibot/0.4 +http://chatterfinder.com +http://omgili.com*/
const Device UA2193466 = {S1810, 79, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; DripfeedBot/2.0; +http://dripfeedbookmark.com/bot.html*/
const Device UA2195630 = {S1811, 44, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - bot.wsowner.com/1.0.0 (+http://wsowner.com/)*/
const Device UA2197566 = {S1812, 77, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; CompSpyBot/1.0; +http://www.compspy.com/spider.html)*/
const Device UA2199511 = {S1813, 66, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; TweetmemeBot/3.0; +http://tweetmeme.com/)*/
const Device UA2200145 = {S1814, 59, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Marketing Grader Crawler 1.0 (http://marketing.grader.com/)*/
const Device UA2200527 = {S1815, 71, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - peerindex/0.1 (http://www.peerindex.com/; crawler AT peerindex DOT com)*/
const Device UA2202320 = {S1816, 41, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SolomonoBot/1.03 (http://www.solomono.ru)*/
const Device UA2202904 = {S1817, 14, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - VX_Pingdom_Bot*/
const Device UA2203606 = {S1818, 72, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - First Search Engine Dedicated The Adult World - Bot version ( 12.09.13 )*/
const Device UA2204074 = {S1819, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/2.03*/
const Device UA2204385 = {S1820, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/2,03*/
const Device UA2204762 = {S1821, 73, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; 200PleaseBot/1.0; +http://www.200please.com/bot)*/
const Device UA2206155 = {S1822, 43, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - PercolateCrawler/3.1.30 (ops@percolate.com)*/
const Device UA2207226 = {S1823, 38, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - FAST Search Web Crawler 14.0.0291.0000*/
const Device UA2209336 = {S1824, 93, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - A1 Website Download/3.0.3 (+http://www.microsystools.com/products/website-download/) miggibot*/
const Device UA2214948 = {S1825, 80, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Arroba MailRobot/2.0 (compatible; Mozilla/5.0; By cristobal.alcayaga@gmail.com;)*/
const Device UA2216388 = {S1826, 83, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; FatBot 2.0; http://www.thefind.com/main/CrawlerFAQs.fhtml)*/
const Device UA2218222 = {S1827, 118, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; c) AppleWebKit/531.2+ (KHTML, like Gecko) Version/5.0 WatchMouse/$Revision: 24640 $*/
const Device UA2219501 = {S1828, 12, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - WikioBot 1.0*/
const Device UA2219507 = {S1829, 91, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - LinkBackBot/1.0; +http://www.linkback.co/bot.html - Automated Reciprocal LinkBack directory*/
const Device UA2221115 = {S1830, 40, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ZCrawler Alpha (+http://www.zazzle.com/)*/
const Device UA2223158 = {S1831, 41, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - QippoBot/1.4 (+http://www.qippo.com/bot/)*/
const Device UA2223365 = {S1832, 32, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - emyuu bot/1.0(http://emyuu.com/)*/
const Device UA2224706 = {S1833, 56, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - PagesInventory.com (robot http://www.pagesinvenotry.com)*/
const Device UA2228613 = {S1834, 39, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Cliqusbot/0.1)*/
const Device UA2228668 = {S1835, 53, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Baiduspider (+http://www.baidu.com/search/spider.htm)*/
const Device UA2230326 = {S1836, 7, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - NoyaBot*/
const Device UA2233608 = {S1837, 132, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - MIA DEV/search:robot/0.0.1 (MIA Bot for research project MIA (www.MIA-marktplatz.de); http://spider.neofonie.de; spider@neofonie.de)*/
const Device UA2235749 = {S1838, 134, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - MIA DEV/search:robot/0.0.1 (MIA Bot for research project MIA (www.MIA-marktplatz.de); http://spider.neofonie.de; spider-l@neofonie.de)*/
const Device UA2236468 = {S1839, 19, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - UnlimitBot(ulmt.in)*/
const Device UA2236573 = {S1840, 52, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - PagesInventory (robot http://www.pagesinvenotry.com)*/
const Device UA2236735 = {S1841, 69, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; spbot/3.1; +http://www.seoprofiler.com/bot )*/
const Device UA2236853 = {S1842, 41, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Mail.RU_Bot/2.0)*/
const Device UA2238897 = {S1843, 49, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - HubSpot Links Crawler 1.0 http://www.hubspot.com/*/
const Device UA2239295 = {S1844, 67, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; IstellaBot/1.10.2 +http://www.tiscali.it/)*/
const Device UA2239493 = {S1845, 40, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SomonesBot 1.0; +http://www.somones.com/*/
const Device UA2257201 = {S1846, 81, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; archive.org_bot/3.1.2 +http://www.eotarchive.cdlib.org/)*/
const Device UA2257347 = {S1847, 117, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; LinkBackBot/1.0; +http://www.linkback.co/bot.html - Automated Reciprocal LinkBack directory)*/
const Device UA2257579 = {S1848, 67, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MFGPagesBot/2.1; +http://www.mfgpages.com)*/
const Device UA2260458 = {S1849, 39, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Statsbot/2.1;)*/
const Device UA2260462 = {S1850, 39, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ratup bot 1.4)*/
const Device UA2260463 = {S1851, 66, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Tourlentabot/0.55; +http://tourlenta.com)*/
const Device UA2261757 = {S1852, 52, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - PagesInventory (robot http://www.pagesinventory.com)*/
const Device UA2267004 = {S1853, 15, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Smarte Bot v1.0*/
const Device UA2273590 = {S1854, 10, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Diabot/1.0*/
const Device UA2275280 = {S1855, 56, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - AlestiFeedBot/0.7 (http://www.alesti.org/; 1 subscriber)*/
const Device UA2277968 = {S1856, 120, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Confirm Domain Bot/1.0 (compatible; NZS.com mini-spider,  Local Search New Zealand; http://www.nzs.com/; bot at nzs.com)*/
const Device UA2279501 = {S1857, 59, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Baiduspider-video+(+http://www.baidu.com/search/spider.htm)*/
const Device UA2281570 = {S1858, 86, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexBlogs/0.99; robot; B; +http://yandex.com/bots)0 readers*/
const Device UA2285676 = {S1859, 11, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - TestCrawler*/
const Device UA2291031 = {S1860, 9, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - E-Crawler*/
const Device UA2291727 = {S1861, 70, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ExaBot/3.0; +http://www.exalead.com/go/robot)*/
const Device UA2298594 = {S1862, 51, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - iCorpusBot (http://www.n-grams.org/icorpusbot.html)*/
const Device UA2300383 = {S1863, 6, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - OVGbot*/
const Device UA2300685 = {S1864, 9, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - spiderman*/
const Device UA2301032 = {S1865, 12, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - scoopbot/0.1*/
const Device UA2303959 = {S1866, 155, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Baiduspider/2.0; +http://www.baidu.com/search/spider.html) AppEngine-Google; (+http://code.google.com/appengine; appid: cgiproxy6)*/
const Device UA2308500 = {S1867, 38, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - WebZIP/7.1 (http://www.spidersoft.com)*/
const Device UA2339435 = {S1868, 15, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Justdialbot/1.0*/
const Device UA2341432 = {S1869, 43, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - nrron-crawler (pdsupport@purediscovery.com)*/
const Device UA2350581 = {S1870, 82, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Kraken/0.1; http://linkfluence.net/; bot@linkfluence.net)*/
const Device UA2355410 = {S1871, 60, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Esribot/1.0; http://www.esrihu.hu/)*/
const Device UA2357152 = {S1872, 63, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - FlightDeckReportsBot/2.0 (http://www.flightdeckreports.com/bot)*/
const Device UA2357839 = {S1873, 63, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; TotientBot 1.0; birrell@microsoft.com)*/
const Device UA2358293 = {S1874, 5, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Bobot*/
const Device UA2359990 = {S1875, 70, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - JCSearch NetHungry JCSearchbot/1.0 (http://jcsearch.com/nethungry.htm)*/
const Device UA2361721 = {S1876, 78, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; EbozavrBot/1.0; +http://www.ebozavr.com/about_bot.php*/
const Device UA2362756 = {S1877, 81, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Infohelfer/1.3.3; +http://www.infohelfer.de/crawler.php)*/
const Device UA2364426 = {S1878, 101, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Alexabot/1.0; +http://www.alexa.com/help/certifyscan; certifyscan@alexa.com)*/
const Device UA2368270 = {S1879, 19, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Online Chat Crawler*/
const Device UA2368852 = {S1880, 24, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - CMS Identify Crawler/2.0*/
const Device UA2375624 = {S1881, 7, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - FeedBot*/
const Device UA2377136 = {S1882, 54, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - BizwikiBot/0.3 (http://www.bizwiki.com/bizwikibot.htm)*/
const Device UA2384739 = {S1883, 83, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - AppEngine-Google; (+http://code.google.com/appengine; appid: s~stremor-crawler-dev)*/
const Device UA2385002 = {S1884, 62, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; crawler; +; crawler.info.2@gmail.com)*/
const Device UA2385872 = {S1885, 11, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - MSN bot/1.0*/
const Device UA2386991 = {S1886, 41, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SolomonoBot/1.04 (http://www.solomono.ru)*/
const Device UA2387441 = {S1887, 9, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - pjcrawler*/
const Device UA2387785 = {S1888, 21, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Backlink Crawler v1.0*/
const Device UA2389102 = {S1889, 54, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - CollarBar logistics research bot v0.26 (collarbar.com)*/
const Device UA2391460 = {S1890, 81, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; 008/0.85; http://www.80legs.com/webcrawler.html) Gecko/X*/
const Device UA2391625 = {S1891, 99, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozzila/5.0 (compatible; YamanaLab-Robot/1.0; http://www.yama.info.waseda.ac.jp/~crawler/info.html)*/
const Device UA2391721 = {S1892, 81, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Steeler/3.5; http://www.tkl.iis.u-tokyo.ac.jp/~crawler/)*/
const Device UA2392355 = {S1893, 32, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - TasBot (http://www.httpool.com/)*/
const Device UA2393640 = {S1894, 66, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; myAbstractCrawler +http://www.mysite.com)*/
const Device UA2394368 = {S1895, 32, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Kentico CMS Smart Search Crawler*/
const Device UA2394396 = {S1896, 151, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MJ12bot/v1.4.3; http://www.majestic12.co.uk/bot.php?+) AppEngine-Google; (+http://code.google.com/appengine; appid: pr0xy--me)*/
const Device UA2397011 = {S1897, 112, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ProductAdsBot/1.0 +https://sellercentral.amazon.com/gp/help/help.html?ie=UTF8&itemID=X)*/
const Device UA2403839 = {S1898, 35, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - User-Agent:Mozilla/5.0 (SaidwotBot)*/
const Device UA2403940 = {S1899, 54, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - CrawlerProcess (http://www.PowerMapper.com) /5.0.592.0*/
const Device UA2404474 = {S1900, 21, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Sogou web spider/4.0l*/
const Device UA2405479 = {S1901, 45, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - grokkit-crawler (pdsupport@purediscovery.com)*/
const Device UA2405509 = {S1902, 79, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - AppEngine-Google; (+http://code.google.com/appengine; appid: s~stremor-crawler)*/
const Device UA2407293 = {S1903, 20, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - AltiSEO.com bot v0.1*/
const Device UA2408505 = {S1904, 11, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Uptimerobot*/
const Device UA2419673 = {S1905, 87, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Y!J-BRJ/YATS crawler (http://help.yahoo.co.jp/help/jp/search/indexing/indexing-15.html)*/
const Device UA2422620 = {S1906, 45, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - oneQube/1 (http://www.chirpaloo.com/bot.html)*/
const Device UA2424608 = {S1907, 43, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ratup.com bot 1.8)*/
const Device UA2425067 = {S1908, 13, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - PopScreen Bot*/
const Device UA2425152 = {S1909, 90, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - stremorbot AppEngine-Google; (+http://code.google.com/appengine; appid: s~stremor-crawler)*/
const Device UA2426326 = {S1910, 48, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Reportl crawler (http://reportl.com/crawler.txt)*/
const Device UA2427274 = {S1911, 23, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Sogou web spider/4.0tml*/
const Device UA2428102 = {S1912, 75, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (X11; compatible; semantic-visions.com crawler; HTTPClient 3.1)*/
const Device UA2428399 = {S1913, 94, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - stremorbot AppEngine-Google; (+http://code.google.com/appengine; appid: s~stremor-crawler-dev)*/
const Device UA2429977 = {S1914, 62, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible;Zalbot/1.0.4; +http://zal.ir/bot.html)*/
const Device UA2430878 = {S1915, 65, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YioopBot; +http://173.13.143.74/bot.php)*/
const Device UA2431858 = {S1916, 55, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ExB Language Crawler 2.0-1 (+http://www.exb.de/crawler)*/
const Device UA2431878 = {S1917, 68, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Memorybot/1.10.51 +http://www.mignify.com/)*/
const Device UA2432505 = {S1918, 50, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - WikiDo/1.1 (http://wikido.com; crawler@wikido.com)*/
const Device UA2433052 = {S1919, 38, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Statsbot/2.1)*/
const Device UA2434621 = {S1920, 46, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ExB Language Crawler 2.0 (+http://www.exb.de/)*/
const Device UA2436585 = {S1921, 73, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Mail.RU_Bot/2.0; +http://go.mail.ru/help/robots)*/
const Device UA2437149 = {S1922, 17, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Pinterest-bot/1.0*/
const Device UA2437487 = {S1923, 68, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; 4SeoHuntBot; +http://4seohunt.com/bot.html)*/
const Device UA2437705 = {S1924, 22, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - RankFlex.com Webspider*/
const Device UA2437795 = {S1925, 52, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SightupBot/1.1 (+http://www.sightup.com/en/bot.html)*/
const Device UA2438681 = {S1926, 10, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Exabot/2.0*/
const Device UA2438980 = {S1927, 56, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - envolk/1.7 (+http://www.envolk.com/envolkspiderinfo.php)*/
const Device UA2439516 = {S1928, 60, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ConveraCrawler/0.9d (+http://www.authoritativeweb.com/crawl)*/
const Device UA2440503 = {S1929, 86, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Attribot/1.1 (compatible; Attribot-Feeds; http://static.attribyte.com/robotreadme.txt)*/
const Device UA2440613 = {S1930, 11, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - spider-feed*/
const Device UA2443436 = {S1931, 28, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Open Web Analytics Bot 1.5.2*/
const Device UA2447745 = {S1932, 54, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Baiduspider (+http://www.baidu.com/search/spider.html)*/
const Device UA2453624 = {S1933, 108, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - coccoc/1.0 (http://help.coccoc.vn/) AppEngine-Google; (+http://code.google.com/appengine; appid: s~doingfly)*/
const Device UA2455400 = {S1934, 22, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Sogou web spider/4.0ml*/
const Device UA2455662 = {S1935, 100, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; special_archiver/3.1.2 bot@archive.org +http://www.archives.gov/crawl.html)*/
const Device UA2455883 = {S1936, 78, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; HeeherBot/1.0,Search for High; http://www.heeher.com)*/
const Device UA2458334 = {S1937, 19, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - NettRapport Crawler*/
const Device UA2460845 = {S1938, 68, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; emefgebot/beta; +http://emefge.de/bot.html)*/
const Device UA2461304 = {S1939, 38, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0+(compatible;+Robot/1.2 CL)*/
const Device UA2461847 = {S1940, 55, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ExB Language Crawler 2.0-2 (+http://www.exb.de/crawler)*/
const Device UA2461892 = {S1941, 79, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; woriobot support [at] zite [dot] com +http://zite.com)*/
const Device UA2462350 = {S1942, 18, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Hello Bot v.2.1.74*/
const Device UA2467860 = {S1943, 23, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - pr-cy.ru Screenshot Bot*/
const Device UA2470762 = {S1944, 53, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ExB Language Crawler 2.1 (+http://www.exb.de/crawler)*/
const Device UA2471623 = {S1945, 83, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+shiny@seomoz.org)*/
const Device UA2471646 = {S1946, 55, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ExB Language Crawler 2.0-3 (+http://www.exb.de/crawler)*/
const Device UA2477450 = {S1947, 54, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - OpenLD.ORG Spider v0.1 (http://www.openwebspider.org/)*/
const Device UA2477927 = {S1948, 67, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0+(compatible;+WebofantBot+v3.14;+http://www.webofant.ch)*/
const Device UA2483461 = {S1949, 56, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - TinEye-bot/0.02 (see http://www.tineye.com/crawler.html)*/
const Device UA2487040 = {S1950, 35, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - TiofoBot/1.1(+http://www.tiofo.com)*/
const Device UA2487645 = {S1951, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/2.10*/
const Device UA2487923 = {S1952, 82, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Baiduspider/2.0;+http://www.baidu.com/search/spider.html)*/
const Device UA2488342 = {S1953, 55, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ExB Language Crawler 2.1.2 (+http://www.exb.de/crawler)*/
const Device UA2488371 = {S1954, 55, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ExB Language Crawler 2.1.4 (+http://www.exb.de/crawler)*/
const Device UA2488878 = {S1955, 55, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ExB Language Crawler 2.1.3 (+http://www.exb.de/crawler)*/
const Device UA2489703 = {S1956, 68, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Web front page analyser (norw.acd.inst@gmail.com or edit robots.txt)*/
const Device UA2490291 = {S1957, 55, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ExB Language Crawler 2.1.1 (+http://www.exb.de/crawler)*/
const Device UA2490868 = {S1958, 16, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - UCI IR crawler X*/
const Device UA2490863 = {S1959, 71, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Web front page analyser. robots.txt complaint (norw.acd.inst@gmail.com)*/
const Device UA2491628 = {S1960, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/2,10*/
const Device UA2492687 = {S1961, 45, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - PiStato indexer (+http://pistato.ru/bot.html)*/
const Device UA2492850 = {S1962, 9, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - PyCrawler*/
const Device UA2503009 = {S1963, 63, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; imrbot/1.10.8 +http://www.mignify.com)*/
const Device UA2504966 = {S1964, 75, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; web-ceo-online-bot/1.0; +http://online.webceo.com)*/
const Device UA2505873 = {S1965, 29, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Domain-Informer.com Searchbot*/
const Device UA2507107 = {S1966, 106, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Mozilla; +http://wiki.github.com/bixo/bixo/bixocrawler; bixo-dev@yahoogroups.com)*/
const Device UA2507196 = {S1967, 59, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Ejeniobot/1.0; support@ejenio.com)*/
const Device UA2508127 = {S1968, 134, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - MiaDev/0.0.1 (MIA Bot for research project MIA (www.MIA-marktplatz.de); http://www.mia-marktplatz.de/spider; spider@mia-marktplatz.de)*/
const Device UA2512796 = {S1969, 36, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - coccoc/1.0 (http://help.coccoc.com/)*/
const Device UA2512906 = {S1970, 42, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Spyder (Mag Reiting: http://magic-top.com)*/
const Device UA2514116 = {S1971, 15, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - testCrawler/1.0*/
const Device UA2515807 = {S1972, 53, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - PagesInventory (robot +http://www.pagesinventory.com)*/
const Device UA2516448 = {S1973, 43, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - gimme60 (Gimme60 Store ID Bot; gimme60.com)*/
const Device UA2516699 = {S1974, 55, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ExB Language Crawler 2.1.5 (+http://www.exb.de/crawler)*/
const Device UA2519335 = {S1975, 75, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; AndStartBot/2.0; +http://www.andstart.ro/bot.html)*/
const Device UA2528693 = {S1976, 62, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Crawler/0.9; http://linkfluence.net/)*/
const Device UA2530690 = {S1977, 66, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Gravitybot/1.0; +http://www.gravity.com/)*/
const Device UA2531942 = {S1978, 66, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ZEERCHBOT; +http://la.zeerch.com/bot.php)*/
const Device UA2531933 = {S1979, 85, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+factors@seomoz.org)*/
const Device UA2533003 = {S1980, 65, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ZumBot/1.0; http://help.zum.com/inquiry)*/
const Device UA2533193 = {S1981, 35, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0+(compatible;+Robot/1.0)*/
const Device UA2533337 = {S1982, 123, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - PulseCrawler/1.1 (+http://www.pulse.me/) AppEngine-Google; (+http://code.google.com/appengine; appid: s~hr-pulsesubscriber)*/
const Device UA2533363 = {S1983, 35, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - FisigBot/1.1(+http://www.fisig.com)*/
const Device UA2533621 = {S1984, 15, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - MP463Crawler1.2*/
const Device UA2536193 = {S1985, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/2.11*/
const Device UA2536199 = {S1986, 39, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - socialbm_bot http://spider.socialbm.net*/
const Device UA2536294 = {S1987, 12, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - RavenCrawler*/
const Device UA2537639 = {S1988, 54, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - CrawlerProcess (http://www.PowerMapper.com) /5.1.600.0*/
const Device UA2537692 = {S1989, 18, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ip-web-crawler.com*/
const Device UA2539653 = {S1990, 47, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - gimme60/1.6 (Gimme60 Store ID Bot; gimme60.com)*/
const Device UA2540874 = {S1991, 20, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - bot(www.cuwhois.com)*/
const Device UA2546201 = {S1992, 11, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - zimmbot/1.0*/
const Device UA2546298 = {S1993, 41, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - JSpider (http://j-spider.sourceforge.net)*/
const Device UA2547228 = {S1994, 16, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - myurlcrawler/0.1*/
const Device UA2553010 = {S1995, 79, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - AppEngine-Google; (+http://code.google.com/appengine; appid: s~vvalertscrawler)*/
const Device UA2553767 = {S1996, 76, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SpellCheck Bot 0.9; +http://showword.info/bot.html)*/
const Device UA2553970 = {S1997, 54, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SNK Screenshot Bot/0.20 ( abuse at savenkeep dot_com )*/
const Device UA2557290 = {S1998, 65, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; scanbot/1.0; +https://dazzlepod.com/ip/)*/
const Device UA2557574 = {S1999, 118, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; c) AppleWebKit/531.2+ (KHTML, like Gecko) Version/5.0 WatchMouse/$Revision: 25590 $*/
const Device UA2564096 = {S2000, 36, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0+(compatible;+Robot/1.1t)*/
const Device UA2565902 = {S2001, 58, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; eCrawler +http://www.cloudda.com)*/
const Device UA2566320 = {S2002, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/2,11*/
const Device UA2566838 = {S2003, 13, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - smart-crawler*/
const Device UA2567489 = {S2004, 53, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - QuerySeekerSpider ( http://queryseeker.com/bot.html )*/
const Device UA2570705 = {S2005, 80, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; AMZNKAssocBot/4.0 +http://affiliate-program.amazon.com)*/
const Device UA2571562 = {S2006, 55, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - it2media-domain-crawler/1.0 on crawler-prod.it2media.de*/
const Device UA2572621 = {S2007, 17, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SiteCrawler/1.1.2*/
const Device UA2573149 = {S2008, 61, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - WebarooBot (Webaroo Bot; http://64.124.122.252/feedback.html)*/
const Device UA2573443 = {S2009, 86, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; VR FrontPage Fetcher; +http://www.visualrevenue.com/bot.html)*/
const Device UA2574506 = {S2010, 38, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - CorenSearchBot/1.7 en libwww-perl/6.04*/
const Device UA2576953 = {S2011, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - nahb.org link verification bot*/
const Device UA2577494 = {S2012, 36, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Autobot/1.0*/
const Device UA2578800 = {S2013, 49, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Crawler for Netopian (http://www.netopian.co.uk/)*/
const Device UA2580782 = {S2014, 35, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SearchBot)*/
const Device UA2581149 = {S2015, 10, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - InsieveBot*/
const Device UA2581379 = {S2016, 86, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexBlogs/0.99; robot; B; +http://yandex.com/bots)1 readers*/
const Device UA2584150 = {S2017, 68, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Spiderlytics/1.0; +spider@spiderlytics.com)*/
const Device UA2584180 = {S2018, 30, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - San Francisco Low Impact Robot*/
const Device UA2584208 = {S2019, 71, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; socialbm_bot/1.0; +http://spider.socialbm.net)*/
const Device UA2585625 = {S2020, 48, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MerchantCentricBot/1.0)*/
const Device UA2585757 = {S2021, 27, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - bot (+http://www.local.com)*/
const Device UA2585762 = {S2022, 28, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - adbeat_bot adbeat.com/policy*/
const Device UA2587541 = {S2023, 63, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Jabse.com Crawler v.1.0 www.jabse.com/crawler.php//imagecrawler*/
const Device UA2596440 = {S2024, 32, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - omgilibot/0.4 +http://omgili.com*/
const Device UA2597297 = {S2025, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-15@seomoz.org)*/
const Device UA2601127 = {S2026, 10, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - adbeat_bot*/
const Device UA2601991 = {S2027, 39, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - blogtop.us crawler - http://blogtop.us/*/
const Device UA2602947 = {S2028, 11, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Greegle Bot*/
const Device UA2603123 = {S2029, 82, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; jjBot/1.0; +http://www.fhnw.ch; crawler.info.2@gmail.com)*/
const Device UA2604410 = {S2030, 28, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - CCTestBot/2.0 (Experimental)*/
const Device UA2604456 = {S2031, 87, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; WebMahBot/3.0; http://www.webmah.com/bot.html; bot@webmah.com)*/
const Device UA2605396 = {S2032, 69, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Snipebot/1.0; +http://snipebot.com/bot.html)*/
const Device UA2640193 = {S2033, 8, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - NodePing*/
const Device UA2640291 = {S2034, 9, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - MyCrawler*/
const Device UA2641334 = {S2035, 60, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - BacklinkCrawler V (http://www.backlinktest.com/crawler.html)*/
const Device UA2641394 = {S2036, 8, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Swiftbot*/
const Device UA2644303 = {S2037, 11, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Snapbot/1.0*/
const Device UA2647245 = {S2038, 35, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 StashMetricsCrawler/1.0*/
const Device UA2657768 = {S2039, 6, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - IXEbot*/
const Device UA2659279 = {S2040, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-23@seomoz.org)*/
const Device UA2659278 = {S2041, 94, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-attercob-01s@seomoz.org)*/
const Device UA2659412 = {S2042, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-27@seomoz.org)*/
const Device UA2667043 = {S2043, 85, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; archive.org_bot/3.1.2 +http://pandora.nla.gov.au/crawl.html)*/
const Device UA2672653 = {S2044, 85, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; uMBot-FC/1.0; mailto: crawling@ubermetrics-technologies.com)*/
const Device UA2673220 = {S2045, 58, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ScreenerBot Crawler Beta 2.0 (+http://www.ScreenerBot.com)*/
const Device UA2674169 = {S2046, 53, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Grahambot/0.1 (+http://www.sunaga-lab.com/graham-bot)*/
const Device UA2675398 = {S2047, 27, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 LabsCrawler/1.0*/
const Device UA2675991 = {S2048, 11, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - YisouSpider*/
const Device UA2677128 = {S2049, 38, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - CatchBot/5.0; +http://www.catchbot.com*/
const Device UA2678798 = {S2050, 77, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SemrushBot/0.96.2; +http://www.semrush.com/bot.html)*/
const Device UA2678810 = {S2051, 104, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - bl.uk_lddc_bot/3.1.1 (+http://www.bl.uk/aboutus/legaldeposit/websites/websites/faqswebmaster/index.html)*/
const Device UA2679091 = {S2052, 47, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Aboundex/0.3 (http://www.aboundex.com/crawler/)*/
const Device UA2680948 = {S2053, 77, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Sosospider/2.0; +http://help.soso.com/webspider.htm)*/
const Device UA2681555 = {S2054, 67, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; BlogBeatbot/2.1; +http://www.blogbeat.nl/)*/
const Device UA2681635 = {S2055, 67, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Syntryx ANT Scout Chassis Pheromone; Mozilla/4.0 compatible crawler*/
const Device UA2681872 = {S2056, 68, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexMetrika/2.0; +http://yandex.com/bots)*/
const Device UA2682716 = {S2057, 49, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Gigabot/2.0; http://www.gigablast.com/spider.html*/
const Device UA2683687 = {S2058, 25, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - yoono/1.0 web-crawler/1.0*/
const Device UA2684275 = {S2059, 50, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - MJ12bot/v1.0.8 (http://majestic12.co.uk/bot.php?+)*/
const Device UA2684823 = {S2060, 37, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ReadBag (http://readbag.com/bot.html)*/
const Device UA2685121 = {S2061, 79, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Elmer, the Thinglink ImageBot (http://www.thinglink.com/help/ThinglinkImageBot)*/
const Device UA2689241 = {S2062, 111, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ProductAdsBot/1.0 https://sellercentral.amazon.com/gp/help/help.html?ie=UTF8&itemID=X)*/
const Device UA2693070 = {S2063, 11, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Insieve Bot*/
const Device UA2693460 = {S2064, 24, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - FTRF: Friendly robot/1.4*/
const Device UA2694864 = {S2065, 54, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Peeplo Screenshot Bot/0.20 ( abuse at peeplo dot_com )*/
const Device UA2696114 = {S2066, 11, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Crawler/3.0*/
const Device UA2697257 = {S2067, 70, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0+(compatible; UptimeRobot/2.0; http://www.uptimerobot.com/)*/
const Device UA2699152 = {S2068, 63, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - searchch UltraSpider3000/2.0 (+http://www.search.ch/about.html)*/
const Device UA2699320 = {S2069, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-18@seomoz.org)*/
const Device UA2700200 = {S2070, 72, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; trovitBot 1.0; +http://www.trovit.com/bot.html)*/
const Device UA2702189 = {S2071, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-28@seomoz.org)*/
const Device UA2702862 = {S2072, 44, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Filestubebot/1.0 (+http://www.filestube.com)*/
const Device UA2705051 = {S2073, 14, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ArtavisBot 1.0*/
const Device UA2708590 = {S2074, 29, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; bot)*/
const Device UA2709264 = {S2075, 68, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ProCogSEOBot/1.0; +http://www.procog.com/ )*/
const Device UA2709560 = {S2076, 66, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Xenu Link Sleuth/1.3.9 beta,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2709567 = {S2077, 73, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; JobdiggerSpider +http://www.jobdigger.nl/spider)*/
const Device UA2713021 = {S2078, 77, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; linkdexbot/2.1; +http://www.linkdex.com/about/bots/)*/
const Device UA2714304 = {S2079, 69, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; HDB2-Bot/2.3; +http://hdb.0xff.org/bot.html)*/
const Device UA2715203 = {S2080, 9, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - CCBot/2.0*/
const Device UA2715650 = {S2081, 67, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; IEZhanSpider/1.0; +http://www.iezhan.com/)*/
const Device UA2716793 = {S2082, 14, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - PennCISCrawler*/
const Device UA2717872 = {S2083, 13, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Skywalker Bot*/
const Device UA2718549 = {S2084, 66, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; awcheckBot; +http://awcheck.com/en/about)*/
const Device UA2719457 = {S2085, 77, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0+(compatible;+Sosospider/2.0;++http://help.soso.com/webspider.htm)*/
const Device UA2719551 = {S2086, 62, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; coccoc/1.0; +http://help.coccoc.com/)*/
const Device UA2720232 = {S2087, 122, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Baiduspider/2.0; +http://www.baidu.com/search/spider.html),gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2721061 = {S2088, 85, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; World Wide Web Wanderer (Wandex Bot)/1.4; http://wandex.net)*/
const Device UA2726942 = {S2089, 86, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; URLAppendBot/1.0; +http://www.profound.net/urlappendbot.html)*/
const Device UA2727173 = {S2090, 67, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; BazQuxBot/2.4; +http://crawler.bazqux.com)*/
const Device UA2729696 = {S2091, 79, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YRSpider; +http://www.yunyun.com/SiteInfo.php?r=about)*/
const Device UA2729726 = {S2092, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-04@seomoz.org)*/
const Device UA2729727 = {S2093, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-13@seomoz.org)*/
const Device UA2729728 = {S2094, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-05@seomoz.org)*/
const Device UA2729745 = {S2095, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-08@seomoz.org)*/
const Device UA2729746 = {S2096, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-03@seomoz.org)*/
const Device UA2729800 = {S2097, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-24@seomoz.org)*/
const Device UA2729802 = {S2098, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-10@seomoz.org)*/
const Device UA2732781 = {S2099, 12, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ElisaBot 1.0*/
const Device UA2733146 = {S2100, 34, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - CERT FigleafBot/0.1 (www.cert.org)*/
const Device UA2733269 = {S2101, 118, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; c) AppleWebKit/531.2+ (KHTML, like Gecko) Version/5.0 WatchMouse/$Revision: 26005 $*/
const Device UA2738088 = {S2102, 7, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - WMU-bot*/
const Device UA2738267 = {S2103, 66, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; GeliyooBot/1.0; +http://www.geliyoo.com/)*/
const Device UA2738344 = {S2104, 66, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - NCBot (http://netcomber.com : tool for finding true domain owners)*/
const Device UA2739321 = {S2105, 72, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - *Sogou web spider/3.0(+http://www.sogou.com/docs/help/webmasters.htm#07)*/
const Device UA2739802 = {S2106, 11, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - mbot v.1.16*/
const Device UA2739922 = {S2107, 104, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - NCBot (http://netcomber.com : tool for finding true domain owners) Queries/complaints: bot@netcomber.com*/
const Device UA2740220 = {S2108, 42, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Symfony Spider (http://symfony.com/spider)*/
const Device UA2740310 = {S2109, 22, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - visaduhoc.info Crawler*/
const Device UA2741386 = {S2110, 70, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YYSpider; +http://www.yunyun.com/spider.html)*/
const Device UA2741693 = {S2111, 44, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Noramy 0.1b (knowledge search and index bot)*/
const Device UA2741884 = {S2112, 21, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - AportCatalogRobot/2.0*/
const Device UA2741918 = {S2113, 60, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; METASpider; +http://meta.ua/spider)*/
const Device UA2742331 = {S2114, 51, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Accoona-AI-Agent/1.1.1 (crawler at accoona dot com)*/
const Device UA2742961 = {S2115, 8, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Dudu bot*/
const Device UA2748078 = {S2116, 10, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - DocolocBot*/
const Device UA2748140 = {S2117, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-20@seomoz.org)*/
const Device UA2748141 = {S2118, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-06@seomoz.org)*/
const Device UA2748177 = {S2119, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-17@seomoz.org)*/
const Device UA2751161 = {S2120, 68, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; lufsbot/0.1; +http://www.lufs.org/bot.html)*/
const Device UA2753030 = {S2121, 70, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; DCPbot/1.4; +http://domains.checkparams.com/)*/
const Device UA2753176 = {S2122, 86, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; FeedWindBot/2.0;+http://feed.mikle.com/en/guide/crawler.html)*/
const Device UA2753185 = {S2123, 72, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; firmilybot/0.2; +http://www.firmily.com/bot.php*/
const Device UA2753421 = {S2124, 20, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - boitho.com-robot/1.0*/
const Device UA2754599 = {S2125, 35, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0+(compatible;+Robot/1.3)*/
const Device UA2754771 = {S2126, 72, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; Struct.it crawler; +http://struct.it/contacts/)*/
const Device UA2755132 = {S2127, 70, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; GeliyooBot/1.0beta; +http://www.geliyoo.com/)*/
const Device UA2762831 = {S2128, 15, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Curator Crawler*/
const Device UA2763953 = {S2129, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-26@seomoz.org)*/
const Device UA2764432 = {S2130, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-12@seomoz.org)*/
const Device UA2764431 = {S2131, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-02@seomoz.org)*/
const Device UA2765107 = {S2132, 50, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; whoisrequest.org Bot/0.1)*/
const Device UA2767161 = {S2133, 147, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; HttpConduitDownloader/1.0; +http://hackage.haskell.org/package/http-conduit-downloader; Heins & Partner Impressumscrawler)*/
const Device UA2767859 = {S2134, 42, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; FindLinksBot/0.1)*/
const Device UA2768208 = {S2135, 21, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - calculon spider/0.1.0*/
const Device UA2769470 = {S2136, 35, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0+(compatible;+Robot/1.2)*/
const Device UA2769850 = {S2137, 62, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; aiHitBot/2.7; +http://www.aihit.com/)*/
const Device UA2770451 = {S2138, 84, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; meanpathbot/1.0; +http://www.meanpath.com/meanpathbot.html)*/
const Device UA2771065 = {S2139, 55, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - dorkobot/kutzooi (xxx; www.xxx.com; xxx at xxx dot xxx)*/
const Device UA2771285 = {S2140, 11, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - infobot/1.0*/
const Device UA2772417 = {S2141, 79, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - "Mozilla/5.0 (compatible; Sosospider/2.0; +http://help.soso.com/webspider.htm)"*/
const Device UA2772941 = {S2142, 7, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - BLEXBot*/
const Device UA2772952 = {S2143, 72, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; firmilybot/0.3; +http://www.firmily.com/bot.php*/
const Device UA2774159 = {S2144, 85, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; uMBot-LN/1.0; mailto: crawling@ubermetrics-technologies.com)*/
const Device UA2774272 = {S2145, 76, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ExactSeekTest; +http://yioop.exactseek.com/bot.php)*/
const Device UA2775106 = {S2146, 47, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - BrightEdge Crawler/1.0 (crawler@brightedge.com)*/
const Device UA2777308 = {S2147, 60, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - FAST Enterprise Crawler 6 used by Bygglo / Svensk Byggtjanst*/
const Device UA2778845 = {S2148, 35, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mippin feed crawler. www.mippin.com*/
const Device UA2778999 = {S2149, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-09@seomoz.org)*/
const Device UA2779027 = {S2150, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-01@seomoz.org)*/
const Device UA2779132 = {S2151, 34, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - XiblBot/1.0 (+http://www.xibl.com)*/
const Device UA2780198 = {S2152, 38, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - CyotekWebCopy/1.0 CyotekWebCrawler/1.0*/
const Device UA2784143 = {S2153, 27, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - AWS-EMR-Research-WebCrawler*/
const Device UA2784691 = {S2154, 54, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - CrawlerProcess (http://www.PowerMapper.com) /5.2.618.0*/
const Device UA2784935 = {S2155, 8, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - nrsbot/6*/
const Device UA2784977 = {S2156, 66, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; GigaBot/1.0; +http://www.gigablast.com/ )*/
const Device UA2785698 = {S2157, 58, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - DuckDuckBot/1.1; (+http://duckduckgo.com/duckduckbot.html)*/
const Device UA2786613 = {S2158, 15, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - CPQ RSS Crawler*/
const Device UA2786647 = {S2159, 65, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Taboolabot/3.7; +http://www.taboola.com)*/
const Device UA2789935 = {S2160, 28, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Website-Rating.com Searchbot*/
const Device UA2791466 = {S2161, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-25@seomoz.org)*/
const Device UA2792373 = {S2162, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-29@seomoz.org)*/
const Device UA2793702 = {S2163, 18, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - node-crawler/0.2.4*/
const Device UA2793976 = {S2164, 60, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 (squirrobot/1.0; http://squirro.com/squirrobot/)*/
const Device UA2796709 = {S2165, 7, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - "NCBot.*/
const Device UA2797239 = {S2166, 115, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SSFC 1.0 Focused Crawler, a research project which advances data mining research; Conatact: andy.xue@unimelb.edu.au*/
const Device UA2797245 = {S2167, 77, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SemrushBot/0.96.3; +http://www.semrush.com/bot.html)*/
const Device UA2799602 = {S2168, 66, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; DonkeyBot/0.2; +http://www.dasdonkey.com)*/
const Device UA2802040 = {S2169, 12, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Whatsnew Bot*/
const Device UA2805238 = {S2170, 53, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Baiduspider ( http://www.baidu.com/search/spider.htm)*/
const Device UA2805382 = {S2171, 28, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - endeca webcrawler/WebCrawler*/
const Device UA2807329 = {S2172, 13, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - carbonbot/1.0*/
const Device UA2808471 = {S2173, 46, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - msnbot/0.9 (+http://search.msn.com/msnbot.htm)*/
const Device UA2846208 = {S2174, 76, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; sitecrawler/1.1; +http://www.siteencyclopedia.com/)*/
const Device UA2849753 = {S2175, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-07@seomoz.org)*/
const Device UA2849788 = {S2176, 19, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - EbuzzingFeedBot 1.0*/
const Device UA2850170 = {S2177, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-21@seomoz.org)*/
const Device UA2850171 = {S2178, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-22@seomoz.org)*/
const Device UA2851594 = {S2179, 63, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ReplazBot/1.5; +http://www.replaz.com)*/
const Device UA2852668 = {S2180, 65, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - kraken-crawler/0.2.0; +http://www.kontera.com/kraken-crawler.html*/
const Device UA2854266 = {S2181, 123, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - gsa-crawler (Enterprise; T3-KY8WUCXTQWSQG; it@pattisonoutdoor.com,ievans@pattisonoutdoor.com,soliveira@pattisonoutdoor.com)*/
const Device UA2858659 = {S2182, 22, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Math. Comp. PA Crawler*/
const Device UA2859850 = {S2183, 9, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - FRCrawler*/
const Device UA2861433 = {S2184, 72, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; firmilybot/0.1; +http://www.firmily.com/bot.php*/
const Device UA2863637 = {S2185, 34, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (BCS spider 2003-2013)*/
const Device UA2864472 = {S2186, 44, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - g2reader-bot/1.0 (+http://www.g2reader.com/)*/
const Device UA2870781 = {S2187, 22, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - User-Agent	Baiduspider*/
const Device UA2874505 = {S2188, 41, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; BarrigasBot/2.0)*/
const Device UA2877409 = {S2189, 100, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; CloudServerMarketSpider/1.0; +http://www.cloudservermarket.com/spider.html)*/
const Device UA2877440 = {S2190, 66, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - "NCBot http://netcomber.com . Tool for finding their domain names"*/
const Device UA2877483 = {S2191, 67, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SolomonoBot/1.05; +http://www.solomono.ru)*/
const Device UA2878377 = {S2192, 67, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ZEERCHBOT; +http://www.zeerch.com/bot.php)*/
const Device UA2879904 = {S2193, 23, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - EMR-Research-WebCrawler*/
const Device UA2881622 = {S2194, 85, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; heritrix/3.1.2-SNAPSHOT-X.X +http://www.google.com/bot.html)*/
const Device UA2885337 = {S2195, 71, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; BLEXBot/1.0; +http://webmeup.com/crawler.html)*/
const Device UA2885454 = {S2196, 74, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0(compatible;Sosospider/2.0;+http://help.soso.com/webspider.htm)*/
const Device UA2885465 = {S2197, 68, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; GoblioBot/0.3.7; +http://goblio.eu/bot.htm)*/
const Device UA2888801 = {S2198, 11, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - crawler/1.0*/
const Device UA2889583 = {S2199, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-14@seomoz.org)*/
const Device UA2890828 = {S2200, 57, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - User-Agent: Curious George - www.analyticsseo.com/crawler*/
const Device UA2899262 = {S2201, 46, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Crawler; +null; null)*/
const Device UA2899373 = {S2202, 69, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - InoReader Crawler (https://www.inoreader.com) - support@inoreader.com*/
const Device UA2933019 = {S2203, 11, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - gsa-crawler*/
const Device UA2936469 = {S2204, 61, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - TrustpilotCrawler/1.0 (compatible; http://www.trustpilot.com)*/
const Device UA2952143 = {S2205, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-19@seomoz.org)*/
const Device UA2962401 = {S2206, 45, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - CrawlerProcess (http://www.PowerMapper.com) /*/
const Device UA2977568 = {S2207, 67, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MetaJobBot; http://www.metajob.at/crawler)*/
const Device UA2979276 = {S2208, 15, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - WebCrawler v1.2*/
const Device UA2984376 = {S2209, 110, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ICC-Crawler (Mozilla-compatible; icc-crawl-contact@ml.nict.go.jp; http://kc.nict.go.jp/project1/crawl-ja.html)*/
const Device UA2985935 = {S2210, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr2-crawler-02@seomoz.org)*/
const Device UA2986039 = {S2211, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-16@seomoz.org)*/
const Device UA2986544 = {S2212, 110, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Mozilla/5.0; +http://wiki.github.com/bixo/bixo/bixocrawler; bixo-dev@yahoogroups.com)*/
const Device UA2987642 = {S2213, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr2-crawler-05@seomoz.org)*/
const Device UA2988821 = {S2214, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr2-crawler-08@seomoz.org)*/
const Device UA2993179 = {S2215, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr2-crawler-11@seomoz.org)*/
const Device UA3002443 = {S2216, 10, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - FTBCrawler*/
const Device UA3018638 = {S2217, 98, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; 008/0.83; http://www.80legs.com/webcrawler.html) Gecko/X;c2VvLWNyYXdsZXI=*/
const Device UA3021427 = {S2218, 48, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Hello from Axtaris-Crawler! (http://axtaris.org)*/
const Device UA3025643 = {S2219, 70, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; oBot/2.3.1; http://filterdb.iss.net/crawler/)*/
const Device UA3031152 = {S2220, 57, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MixrankBot; crawler@mixrank.com)*/
const Device UA3032999 = {S2221, 71, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.moz.com/dp/rogerbot, rogerbot-crawler@moz.com)*/
const Device UA3033995 = {S2222, 88, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://moz.com/help/pro/what-is-rogerbot-, rogerbot-crawler+shiny@moz.com)*/
const Device UA3038853 = {S2223, 110, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; BIXOCRAWLER; +http://wiki.github.com/bixo/bixo/bixocrawler; bixo-dev@yahoogroups.com)*/
const Device UA3039361 = {S2224, 79, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MJ12bot/v1.4.4; http://www.majestic12.co.uk/bot.php?+)*/
const Device UA3042731 = {S2225, 15, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Marfeel-crawler*/
const Device UA3044633 = {S2226, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr1-crawler-30@seomoz.org)*/
const Device UA3045486 = {S2227, 23, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ownCloud Server Crawler*/
const Device UA3045767 = {S2228, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr2-crawler-06@seomoz.org)*/
const Device UA3045768 = {S2229, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr2-crawler-07@seomoz.org)*/
const Device UA3045799 = {S2230, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr2-crawler-10@seomoz.org)*/
const Device UA3047697 = {S2231, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr2-crawler-27@seomoz.org)*/
const Device UA3057448 = {S2232, 99, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; socialrep/1.0; +http://bixolabs.com/crawler/general; crawler@bixolabs.com)*/
const Device UA3057679 = {S2233, 96, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MJ12bot/v1.2.4; http://www.majestic12.co.uk/bot.php?+);c2VvLWNyYXdsZXI=*/
const Device UA3057941 = {S2234, 76, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ayna-crawler +http://www.ayna.com);c2VvLWNyYXdsZXI=*/
const Device UA3057947 = {S2235, 41, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Netchart Adv Crawler 1.0;c2VvLWNyYXdsZXI=*/
const Device UA3057959 = {S2236, 59, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - wf_crawler (http://www.websitefigures.com);c2VvLWNyYXdsZXI=*/
const Device UA3067695 = {S2237, 67, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; BLEXBot/1.0; +http://webmeup-crawler.com/)*/
const Device UA3069325 = {S2238, 80, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - updated/0.1-alpha (updated crawler; http://www.updated.com; crawler@updated.com)*/
const Device UA3071676 = {S2239, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr2-crawler-01@seomoz.org)*/
const Device UA3072837 = {S2240, 92, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://www.seomoz.org/dp/rogerbot, rogerbot-crawler+pr2-crawler-14@seomoz.org)*/
const Device UA3073930 = {S2241, 20, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - T3census-Crawler/1.0*/
const Device UA3078082 = {S2242, 36, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Ronsmap crawler/0.1;c2VvLWNyYXdsZXI=*/
const Device UA3078092 = {S2243, 96, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MJ12bot/v1.2.1; http://www.majestic12.co.uk/bot.php?+);c2VvLWNyYXdsZXI=*/
const Device UA3078100 = {S2244, 96, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MJ12bot/v1.3.3; http://www.majestic12.co.uk/bot.php?+);c2VvLWNyYXdsZXI=*/
const Device UA3078102 = {S2245, 96, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MJ12bot/v1.3.1; http://www.majestic12.co.uk/bot.php?+);c2VvLWNyYXdsZXI=*/
const Device UA3078115 = {S2246, 57, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Willow Internet Crawler by Twotrees V2.1;c2VvLWNyYXdsZXI=*/
const Device UA3078127 = {S2247, 96, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MJ12bot/v1.3.2; http://www.majestic12.co.uk/bot.php?+);c2VvLWNyYXdsZXI=*/
const Device UA3078128 = {S2248, 69, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - ReutersCrawler/1.0 (http://reuters.com/crawler.html);c2VvLWNyYXdsZXI=*/
const Device UA3078161 = {S2249, 60, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - radian6_linkcheck_(www.radian6.com/crawler);c2VvLWNyYXdsZXI=*/
const Device UA3078202 = {S2250, 58, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - PostPost/1.0 (+http://postpo.st/crawlers);c2VvLWNyYXdsZXI=*/
const Device UA3079179 = {S2251, 28, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - MultiPLX.com RSS Crawler/1.0*/
const Device UA3079223 = {S2252, 81, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MJ12bot/v1.4.3; http://catsynetwork.dyndns.org/bots.html*/
const Device UA3079411 = {S2253, 130, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - SSFC 1.0 Focused Crawler, a research project which advances data mining research; Conatact: andy(DOT)xue(AT)unimelb(DOT)edu(DOT)au*/
const Device UA3084592 = {S2254, 79, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - sproose/0.1 (sproose bot; http://www.sproose.com/bot.html; crawler@sproose.com)*/
const Device UA3084935 = {S2255, 15, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - HubSpot Crawler*/
const Device UA3086519 = {S2256, 76, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; GABot/1.0; +http://www.groupangle.com/crawler.html)*/
const Device UA3101370 = {S2257, 97, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; GABot/1.0; crawler@yoterra.com; +http://www.groupangle.com/crawler.html)*/
const Device UA3109297 = {S2258, 97, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://moz.com/help/pro/what-is-rogerbot-, rogerbot-crawler+pr1-crawler-08@moz.com)*/
const Device UA3109298 = {S2259, 97, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://moz.com/help/pro/what-is-rogerbot-, rogerbot-crawler+pr2-crawler-02@moz.com)*/
const Device UA3109407 = {S2260, 97, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://moz.com/help/pro/what-is-rogerbot-, rogerbot-crawler+pr1-crawler-31@moz.com)*/
const Device UA3109519 = {S2261, 97, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://moz.com/help/pro/what-is-rogerbot-, rogerbot-crawler+pr1-crawler-13@moz.com)*/
const Device UA3109520 = {S2262, 97, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://moz.com/help/pro/what-is-rogerbot-, rogerbot-crawler+pr1-crawler-32@moz.com)*/
const Device UA3114773 = {S2263, 42, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SEO-crawler/1.0;)*/
const Device UA3115387 = {S2264, 61, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - gsa-crawler (Enterprise; S5-BCT4XJX3FUJAB; nobody@google.com)*/
const Device UA3116504 = {S2265, 81, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ElectricMonk/3.1.1 +https://www.duedil.com/our-crawler/)*/
const Device UA3118745 = {S2266, 76, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; BlueEyeTestCrawler/1.0; +http://blueeyewebtest.com)*/
const Device UA3120921 = {S2267, 18, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - node-crawler/0.2.5*/
const Device UA3126843 = {S2268, 97, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://moz.com/help/pro/what-is-rogerbot-, rogerbot-crawler+pr2-crawler-14@moz.com)*/
const Device UA3126844 = {S2269, 97, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://moz.com/help/pro/what-is-rogerbot-, rogerbot-crawler+pr1-crawler-05@moz.com)*/
const Device UA3126847 = {S2270, 97, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://moz.com/help/pro/what-is-rogerbot-, rogerbot-crawler+pr2-crawler-04@moz.com)*/
const Device UA3126951 = {S2271, 97, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - rogerbot/1.0 (http://moz.com/help/pro/what-is-rogerbot-, rogerbot-crawler+pr1-crawler-09@moz.com)*/
const Device UA3128287 = {S2272, 54, { &P18254,&P7480,&P18245,&P18091 }};	/*Useragent - CrawlerProcess (http://www.PowerMapper.com) /5.3.640.0*/
const Device UA2359519 = {S2273, 108, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B5105c*/
const Device UA2391468 = {S2274, 108, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B5117b*/
const Device UA2437254 = {S2275, 108, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B5126b*/
const Device UA2482346 = {S2276, 92, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2486731 = {S2277, 106, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141*/
const Device UA2487675 = {S2278, 106, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143*/
const Device UA2487691 = {S2279, 106, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144*/
const Device UA2492474 = {S2280, 106, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142*/
const Device UA2509670 = {S2281, 247, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone4,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/6.1;FBSS/2; FBCR/Verizon;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA2509855 = {S2282, 135, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone5,1; iPhone; U; CPU OS 6_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N41AP (gzip)*/
const Device UA2509953 = {S2283, 42, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Pulse 3.0.4 (iPhone; iPhone OS 6.1; en_US)*/
const Device UA2511228 = {S2284, 135, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone5,2; iPhone; U; CPU OS 6_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N42AP (gzip)*/
const Device UA2511932 = {S2285, 94, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2512576 = {S2286, 135, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone4,1; iPhone; U; CPU OS 6_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N94AP (gzip)*/
const Device UA2513077 = {S2287, 108, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B311*/
const Device UA2515658 = {S2288, 134, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 iPad2,1 BingWeb/3.03.1428.X*/
const Device UA2516521 = {S2289, 108, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145*/
const Device UA2531290 = {S2290, 151, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - NAVER(inapp; cafe; 100; 2.9.6) Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName*/
const Device UA2531377 = {S2291, 248, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone4,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/6.1.1;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA2532340 = {S2292, 135, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone3,1; iPhone; U; CPU OS 6_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N90AP (gzip)*/
const Device UA2532507 = {S2293, 108, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146*/
const Device UA2532639 = {S2294, 94, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2532873 = {S2295, 137, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone4,1; iPhone; U; CPU OS 6_1_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N94AP (gzip)*/
const Device UA2533028 = {S2296, 108, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B147*/
const Device UA2533403 = {S2297, 44, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Miu Ptt 1.9.2 (iPhone; iPhone OS 6.1; zh_TW)*/
const Device UA2535185 = {S2298, 248, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_2 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone4,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/6.1.2;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA2536609 = {S2299, 108, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B318*/
const Device UA2537705 = {S2300, 137, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone4,1; iPhone; U; CPU OS 6_1_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N94AP (gzip)*/
const Device UA2538125 = {S2301, 135, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone3,3; iPhone; U; CPU OS 6_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N92AP (gzip)*/
const Device UA2560604 = {S2302, 136, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 iPad2,1 BingWeb/3.03.1428.X*/
const Device UA2561130 = {S2303, 137, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone5,2; iPhone; U; CPU OS 6_1_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N42AP (gzip)*/
const Device UA2561419 = {S2304, 137, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone3,1; iPhone; U; CPU OS 6_1_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N90AP (gzip)*/
const Device UA2562684 = {S2305, 167, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 6_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48 Instapaper/4.2.7 (+http://www.instapaper.com/iphone)*/
const Device UA2563717 = {S2306, 48, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - wp-iphone/3.5.1 (iPhone OS 6.1.2, iPhone) Mobile*/
const Device UA2563853 = {S2307, 105, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/9Z999*/
const Device UA2563878 = {S2308, 138, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/8536.26.7 DiigoBrowser*/
const Device UA2564403 = {S2309, 137, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone5,1; iPhone; U; CPU OS 6_1_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N41AP (gzip)*/
const Device UA2564407 = {S2310, 153, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - NAVER(inapp; cafe; 100; 2.9.6) Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_2 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName*/
const Device UA2567626 = {S2311, 46, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - wp-iphone/3.5.1 (iPhone OS 6.1.2, iPad) Mobile*/
const Device UA2570099 = {S2312, 145, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 bline/1.04 (iPhone OS 6.1.2, iPhone)*/
const Device UA2570332 = {S2313, 137, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone3,3; iPhone; U; CPU OS 6_1_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N92AP (gzip)*/
const Device UA2583078 = {S2314, 46, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Miu Ptt 1.9.2 (iPhone; iPhone OS 6.1.2; zh_TW)*/
const Device UA2584024 = {S2315, 108, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329*/
const Device UA2585369 = {S2316, 94, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2601719 = {S2317, 107, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146*/
const Device UA2602045 = {S2318, 251, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone4,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/6.1.3;FBSS/2; FBCR/Verizon;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA2603775 = {S2319, 107, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/9Z999*/
const Device UA2606071 = {S2320, 145, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 bline/1.06 (iPhone OS 6.1.3, iPhone)*/
const Device UA2606839 = {S2321, 136, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 iPad2,1 BingWeb/3.03.1428.X*/
const Device UA2664714 = {S2322, 175, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari IPhoneAppVersion/2.5.2 IPhoneWebkitWrapper BillingSupported*/
const Device UA2668734 = {S2323, 175, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari IPhoneAppVersion/2.5.2 IPhoneWebkitWrapper BillingSupported*/
const Device UA2669356 = {S2324, 107, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329*/
const Device UA2669734 = {S2325, 138, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/8536.26.7 DiigoBrowser*/
const Device UA2670073 = {S2326, 46, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Miu Ptt 1.9.2 (iPhone; iPhone OS 6.1.3; zh_TW)*/
const Device UA2670659 = {S2327, 153, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - NAVER(inapp; cafe; 100; 2.9.7) Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName*/
const Device UA2672716 = {S2328, 108, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329*/
const Device UA2675062 = {S2329, 137, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone5,1; iPhone; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/16077 (KHTML, like Gecko) Mobile/N41AP (gzip)*/
const Device UA2675389 = {S2330, 137, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone5,2; iPhone; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/16077 (KHTML, like Gecko) Mobile/N42AP (gzip)*/
const Device UA2709288 = {S2331, 94, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2713971 = {S2332, 108, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350*/
const Device UA2715915 = {S2333, 48, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - wp-iphone/3.5.3 (iPhone OS 6.1.3, iPhone) Mobile*/
const Device UA2715933 = {S2334, 144, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone 6_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.8 (iPhone4,1/6.1.3)*/
const Device UA2738109 = {S2335, 142, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone 6_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.8 (iPad2,1/6.1.3)*/
const Device UA2752750 = {S2336, 48, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - wp-iphone/3.5.3 (iPhone OS 6.1.4, iPhone) Mobile*/
const Device UA2752853 = {S2337, 248, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_4 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone5,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/6.1.4;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA2769454 = {S2338, 145, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 bline/1.06 (iPhone OS 6.1.4, iPhone)*/
const Device UA2779663 = {S2339, 106, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146*/
const Device UA2780748 = {S2340, 126, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/8536.25*/
const Device UA2799238 = {S2341, 153, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - NAVER(inapp; cafe; 100; 3.0.0) Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_4 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName*/
const Device UA2850848 = {S2342, 136, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Safari/8536.26.7 DiigoBrowser*/
const Device UA2852846 = {S2343, 46, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Miu Ptt 1.9.2 (iPhone; iPhone OS 6.1.1; zh_TW)*/
const Device UA2854150 = {S2344, 230, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 [FBAN/FBIOS;FBAV/5.5;FBBV/X;FBDV/iPad3,4;FBMD/iPad;FBSN/iPhone OS;FBSV/6.1.2;FBSS/2; FBCR/;FBID/tablet;FBLC/en_US;FBOP/0]*/
const Device UA2869589 = {S2345, 138, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/8536.26.7 DiigoBrowser*/
const Device UA2872520 = {S2346, 239, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 [FBAN/MessengerForiOS;FBAV/2.5;FBBV/X;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/6.1.3;FBSS/1; FBCR/;FBID/phone;FBLC/en_US;FBOP/1]*/
const Device UA2877394 = {S2347, 137, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone4,1; iPhone; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/N94AP (gzip)*/
const Device UA2880727 = {S2348, 137, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone3,1; iPhone; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/N90AP (gzip)*/
const Device UA2881312 = {S2349, 137, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone5,2; iPhone; U; CPU OS 6_1_4 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/N42AP (gzip)*/
const Device UA2881572 = {S2350, 137, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone5,1; iPhone; U; CPU OS 6_1_4 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/N41AP (gzip)*/
const Device UA2896694 = {S2351, 46, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - wp-iphone/3.6.3 (iPhone OS 6.1.3, iPad) Mobile*/
const Device UA2898498 = {S2352, 137, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone3,3; iPhone; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/N92AP (gzip)*/
const Device UA2935634 = {S2353, 118, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 ( ; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/8536.25*/
const Device UA2937999 = {S2354, 175, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari IPhoneAppVersion/2.6.0 IPhoneWebkitWrapper BillingSupported*/
const Device UA2958302 = {S2355, 158, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 TwonkyBeamBrowser/3.4.3-37 (iPhone iOS Ver:6.1.3)*/
const Device UA2992709 = {S2356, 217, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329,Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329*/
const Device UA2992863 = {S2357, 134, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 webmdapp_wbmd/3.4*/
const Device UA3000313 = {S2358, 118, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 ( ; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/8536.25*/
const Device UA3004160 = {S2359, 158, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 TwonkyBeamBrowser/3.4.3-37 (iPhone iOS Ver:6.1.4)*/
const Device UA3006641 = {S2360, 175, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari IPhoneAppVersion/2.6.0 IPhoneWebkitWrapper BillingSupported*/
const Device UA3014309 = {S2361, 137, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone2,1; iPhone; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/N88AP (gzip)*/
const Device UA3023511 = {S2362, 175, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari IPhoneAppVersion/2.6.1 IPhoneWebkitWrapper BillingSupported*/
const Device UA3024472 = {S2363, 175, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari IPhoneAppVersion/2.6.1 IPhoneWebkitWrapper BillingSupported*/
const Device UA3073621 = {S2364, 123, { &P12280,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/8536.25*/
const Device UA178034 = {S2365, 71, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0)*/
const Device UA179870 = {S2366, 64, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0)*/
const Device UA288788 = {S2367, 64, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/5.0)*/
const Device UA652806 = {S2368, 76, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Win64; x64; Trident/6.0)*/
const Device UA663218 = {S2369, 108, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/4.0; InfoPath.2; SV1; .NET CLR 2.0.50727; WOW64)*/
const Device UA663344 = {S2370, 156, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.6; Windows NT 6.1; Trident/5.0; InfoPath.2; SLCC1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 2.0.50727) 3gpp-gba*/
const Device UA2362718 = {S2371, 52, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Windows-RSS-Platform/2.0 (MSIE 10.0; Windows NT 6.1)*/
const Device UA2392444 = {S2372, 51, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.1)*/
const Device UA2408907 = {S2373, 56, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.1; SV1)*/
const Device UA2466239 = {S2374, 169, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)*/
const Device UA2503176 = {S2375, 162, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)*/
const Device UA2527597 = {S2376, 62, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 ( ; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0)*/
const Device UA2554814 = {S2377, 77, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; 3M/MSIE 8.0)*/
const Device UA2561421 = {S2378, 95, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; QQBrowser/7.1.6004.400)*/
const Device UA2564411 = {S2379, 88, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; QQBrowser/7.1.6004.400)*/
const Device UA2594573 = {S2380, 59, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows 7; Trident/6.0)*/
const Device UA2606770 = {S2381, 88, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; QQBrowser/7.2.7006.400)*/
const Device UA2607082 = {S2382, 95, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; QQBrowser/7.2.7006.400)*/
const Device UA2651848 = {S2383, 51, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1)*/
const Device UA2657100 = {S2384, 55, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 ( ; MSIE 10.0; Windows NT 6.1; Trident/6.0)*/
const Device UA2658642 = {S2385, 84, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SlimBrowser)*/
const Device UA2668288 = {S2386, 79, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; EIE10;ESARMSN)*/
const Device UA2694777 = {S2387, 177, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0)*/
const Device UA2695435 = {S2388, 86, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Verizon Windows Live)*/
const Device UA2697734 = {S2389, 84, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; 3M/MSIE 8.0)*/
const Device UA2706555 = {S2390, 86, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; EIE10;ESARMSN)*/
const Device UA2728098 = {S2391, 65, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Empty (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0)*/
const Device UA2732677 = {S2392, 80, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Xerox_Win32PWS)*/
const Device UA2734133 = {S2393, 93, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Verizon Windows Live)*/
const Device UA2734348 = {S2394, 229, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2))*/
const Device UA2764101 = {S2395, 184, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2780578 = {S2396, 130, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Mozilla/4.0 (compatiblt; MSIE 8.0; Windows NT; LCBSECURE))*/
const Device UA2780638 = {S2397, 108, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Mozilla/4.0 (compatible; MSIE 6.0; Menara))*/
const Device UA2796531 = {S2398, 199, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA2837654 = {S2399, 179, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7)*/
const Device UA2845604 = {S2400, 236, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2))*/
const Device UA2856291 = {S2401, 59, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - MASBJS (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0)*/
const Device UA2982735 = {S2402, 268, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; EmbeddedWB 14.52 from: http://www.bsalsa.com/ EmbeddedWB 14.52; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7; Tablet PC 2.0; .NET4.0E; 360SE)*/
const Device UA2993033 = {S2403, 91, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; 3M/MSIE 8.0;; 3M/MSIE 8.0)*/
const Device UA3046958 = {S2404, 65, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0  (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0)*/
const Device UA3108115 = {S2405, 79, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (MSIE 10.0; Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko*/
const Device UA3120483 = {S2406, 211, { &P15364,&P7133,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chrome/24.0 (compatible; MSIE 6.0; Windows NT; Windows 7); Mozilla/18.0.1 (Compatible; MSIE 6.0;MSIE 7.0; MSIE 9.0; Windows NT; Windows 7))*/
const Device UA96799 = {S2407, 96, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.1; Microsoft Outlook 14.0.4760; Pro; ms-office; MSOffice 14)*/
const Device UA133857 = {S2408, 96, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.1; Microsoft Outlook 14.0.4734; Pro; ms-office; MSOffice 14)*/
const Device UA152874 = {S2409, 96, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.1; Microsoft Outlook 14.0.5128; Pro; ms-office; MSOffice 14)*/
const Device UA211794 = {S2410, 210, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA217601 = {S2411, 236, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA217804 = {S2412, 251, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA218627 = {S2413, 96, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.1; Microsoft Outlook 14.0.6023; Pro; ms-office; MSOffice 14)*/
const Device UA221614 = {S2414, 231, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA223238 = {S2415, 224, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA226949 = {S2416, 253, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; Tablet PC 2.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA231451 = {S2417, 203, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA233665 = {S2418, 256, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.5136; ms-office; MSOffice 14)*/
const Device UA238866 = {S2419, 249, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.5128; ms-office; MSOffice 14)*/
const Device UA244787 = {S2420, 96, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.1; Microsoft Outlook 14.0.6106; Pro; ms-office; MSOffice 14)*/
const Device UA395071 = {S2421, 215, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA419952 = {S2422, 222, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.3; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.5136; ms-office; MSOffice 14)*/
const Device UA436864 = {S2423, 229, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MS-RTC LM 8; InfoPath.3; Microsoft Outlook 14.0.5136; ms-office; MSOffice 14)*/
const Device UA554236 = {S2424, 249, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.5139; ms-office; MSOffice 14)*/
const Device UA559157 = {S2425, 249, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.5138; ms-office; MSOffice 14)*/
const Device UA560619 = {S2426, 231, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.5128; ms-office; MSOffice 14)*/
const Device UA561647 = {S2427, 246, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; SASstaged; Microsoft Outlook 14.0.5128; ms-office; MSOffice 14)*/
const Device UA573403 = {S2428, 210, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA576122 = {S2429, 236, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA576136 = {S2430, 251, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA603723 = {S2431, 224, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA603733 = {S2432, 203, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA603809 = {S2433, 231, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA603918 = {S2434, 239, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA603934 = {S2435, 215, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA604093 = {S2436, 248, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA604300 = {S2437, 123, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA604647 = {S2438, 241, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA605509 = {S2439, 210, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA607072 = {S2440, 116, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA629361 = {S2441, 231, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA632046 = {S2442, 263, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA638664 = {S2443, 221, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.30729; .NET4.0C; Media Center PC 6.0; SLCC2; Microsoft Outlook 14.0.6106; ms-office; MSOffice 14)*/
const Device UA647089 = {S2444, 224, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA649540 = {S2445, 96, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.1; Microsoft Outlook 14.0.6112; Pro; ms-office; MSOffice 14)*/
const Device UA656289 = {S2446, 210, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6114; ms-office; MSOffice 14)*/
const Device UA668376 = {S2447, 215, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA685015 = {S2448, 258, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Zune 4.7; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA693887 = {S2449, 241, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA707458 = {S2450, 234, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.5139; ms-office; MSOffice 14)*/
const Device UA707976 = {S2451, 248, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; Zune 4.7; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA708628 = {S2452, 215, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA708675 = {S2453, 123, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA714371 = {S2454, 256, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; Tablet PC 2.0; Creative AutoUpdate v1.40.01; Microsoft Outlook 14.0.61*/
const Device UA727457 = {S2455, 236, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; Zune 4.7; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA747747 = {S2456, 229, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.5130; ms-office; MSOffice 14)*/
const Device UA748997 = {S2457, 128, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA760978 = {S2458, 218, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA783936 = {S2459, 300, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Media Center PC 5.0; SLCC1; OfficeLiveConnector.1.5; OfficeLivePatch.1.3; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA793338 = {S2460, 273, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Zune 4.7; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA832627 = {S2461, 249, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6112; ms-office; MSOffice 14)*/
const Device UA844816 = {S2462, 264, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Tablet PC 2.0; Microsoft Outlook 14.0.5136; ms-office; MSOffice 14)*/
const Device UA897371 = {S2463, 218, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA1910647 = {S2464, 254, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Media Center PC 5.0; SLCC1; InfoPath.3; .NET4.0C; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA1941706 = {S2465, 116, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA1949819 = {S2466, 227, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; IE8Mercury; Microsoft Outlook 14.0.6114; ms-office; MSOffice 14)*/
const Device UA1959416 = {S2467, 315, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Media Center PC 5.0; SLCC1; Tablet PC 2.0; OfficeLiveConnector.1.5; OfficeLivePatch.1.3; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA1965486 = {S2468, 254, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; TS7; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.5136; ms-office; MSOffice 14)*/
const Device UA1967986 = {S2469, 230, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA1983157 = {S2470, 251, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Tablet PC 2.0; Zune 4.7; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA1988751 = {S2471, 281, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Tablet PC 2.0; Zune 4.7; BRI/2; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA1995179 = {S2472, 231, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6122; ms-office; MSOffice 14)*/
const Device UA1995918 = {S2473, 72, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.1; Microsoft Outlook 14.0.6123; Pro)*/
const Device UA2142485 = {S2474, 203, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.5136; ms-office; MSOffice 14)*/
const Device UA2149119 = {S2475, 123, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2156654 = {S2476, 248, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6114; ms-office; MSOffice 14)*/
const Device UA2158959 = {S2477, 236, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Avant Browser; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2190943 = {S2478, 221, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2221036 = {S2479, 264, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; Tablet PC 2.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.5128; ms-office; MSOffice 14)*/
const Device UA2221914 = {S2480, 244, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6114; ms-office; MSOffice 14)*/
const Device UA2266829 = {S2481, 85, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.1; Microsoft Outlook 14.0.4536; Pro; MSOffice 14)*/
const Device UA2299234 = {S2482, 253, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Zune 4.7; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2303763 = {S2483, 128, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA2305563 = {S2484, 220, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Tablet PC 2.0; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA2317230 = {S2485, 224, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6114; ms-office; MSOffice 14)*/
const Device UA2360447 = {S2486, 231, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2361411 = {S2487, 238, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2362067 = {S2488, 224, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2362469 = {S2489, 123, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2362646 = {S2490, 210, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2362846 = {S2491, 251, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2362887 = {S2492, 241, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2362924 = {S2493, 229, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2363143 = {S2494, 116, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2367668 = {S2495, 203, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2367867 = {S2496, 215, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2368351 = {S2497, 221, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2368352 = {S2498, 128, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2393749 = {S2499, 243, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2395660 = {S2500, 56, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Microsoft Office Outlook 2010 (14.0.6123) Windows NT 6.1*/
const Device UA2397065 = {S2501, 256, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6123; ms-office; MSOffice 14)*/
const Device UA2397124 = {S2502, 241, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6123; ms-office; MSOffice 14)*/
const Device UA2405609 = {S2503, 72, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.1; Microsoft Outlook 14.0.6129; Pro)*/
const Device UA2422474 = {S2504, 236, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; Zune 4.7; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2424672 = {S2505, 244, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6129; ms-office; MSOffice 14)*/
const Device UA2425756 = {S2506, 258, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Zune 4.7; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2426089 = {S2507, 230, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2427559 = {S2508, 273, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Zune 4.7; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2441855 = {S2509, 265, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; Zune 4.7; InfoPath.3; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2445299 = {S2510, 231, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6120; ms-office; MSOffice 14)*/
const Device UA2447792 = {S2511, 237, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6123; ms-office; MSOffice 14)*/
const Device UA2450934 = {S2512, 263, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; MS-RTC LM 8; .NET4.0E; Tablet PC 2.0; S.N.O.W.4; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA2457138 = {S2513, 237, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6129; ms-office; MSOffice 14)*/
const Device UA2481840 = {S2514, 269, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Media Center PC 5.0; SLCC1; .NET4.0C; InfoPath.3; Tablet PC 2.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2482342 = {S2515, 300, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Media Center PC 5.0; SLCC1; OfficeLiveConnector.1.5; OfficeLivePatch.1.3; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2482529 = {S2516, 218, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2493652 = {S2517, 241, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2493804 = {S2518, 256, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2494983 = {S2519, 264, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Tablet PC 2.0; Microsoft Outlook 14.0.6112; ms-office; MSOffice 14)*/
const Device UA2497260 = {S2520, 116, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2503853 = {S2521, 239, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Zune 4.7; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2507959 = {S2522, 210, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2509124 = {S2523, 123, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2510308 = {S2524, 224, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2510788 = {S2525, 231, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2516156 = {S2526, 215, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2516440 = {S2527, 248, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2516546 = {S2528, 239, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2516603 = {S2529, 263, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2516715 = {S2530, 116, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2517435 = {S2531, 243, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; .NET4.0E; Microsoft Outlook 14.0.6102; ms-office; MSOffice 14)*/
const Device UA2531376 = {S2532, 203, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2531385 = {S2533, 128, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2538004 = {S2534, 241, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2538520 = {S2535, 221, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2542547 = {S2536, 241, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA2544322 = {S2537, 256, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA2552237 = {S2538, 256, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6112; ms-office; MSOffice 14)*/
const Device UA2556582 = {S2539, 235, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6129; ms-office; MSOffice 14)*/
const Device UA2559747 = {S2540, 230, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2562223 = {S2541, 124, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2562420 = {S2542, 218, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2581226 = {S2543, 273, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2587678 = {S2544, 56, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Microsoft Office Outlook 2010 (14.0.6112) Windows NT 6.1*/
const Device UA2590513 = {S2545, 231, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2599453 = {S2546, 129, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Win64; x64; Trident/6.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2602700 = {S2547, 117, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2640709 = {S2548, 239, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.6114; ms-office; MSOffice 14)*/
const Device UA2643908 = {S2549, 242, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; FBViewer-5.0.1.33; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MS-RTC LM 8; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6102; ms-office; MSOffice 14)*/
const Device UA2647167 = {S2550, 242, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6120; ms-office; MSOffice 14)*/
const Device UA2651585 = {S2551, 236, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2654393 = {S2552, 56, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Microsoft Office Outlook 2010 (14.0.6129) Windows NT 6.1*/
const Device UA2655487 = {S2553, 231, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2664063 = {S2554, 226, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2665421 = {S2555, 236, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; AD1; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2666940 = {S2556, 242, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; MS-RTC LM 8; SFIEAUTH1; Tablet PC 2.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2682895 = {S2557, 193, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Microsoft Outlook 14.0.5128; ms-office; MSOffice 14)*/
const Device UA2683150 = {S2558, 128, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2684422 = {S2559, 233, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.5130; ms-office; MSOffice 14)*/
const Device UA2705556 = {S2560, 224, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2706014 = {S2561, 246, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2708725 = {S2562, 249, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6133; ms-office; MSOffice 14)*/
const Device UA2714818 = {S2563, 124, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2720478 = {S2564, 218, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.5136; ms-office; MSOffice 14)*/
const Device UA2732760 = {S2565, 258, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6120; ms-office; MSOffice 14)*/
const Device UA2732792 = {S2566, 243, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2732882 = {S2567, 256, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6133; ms-office; MSOffice 14)*/
const Device UA2768668 = {S2568, 232, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2793283 = {S2569, 254, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Media Center PC 5.0; SLCC1; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2793656 = {S2570, 246, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Avant Browser; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2794904 = {S2571, 239, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2795101 = {S2572, 264, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Tablet PC 2.0; Microsoft Outlook 14.0.6133; ms-office; MSOffice 14)*/
const Device UA2795118 = {S2573, 241, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6133; ms-office; MSOffice 14)*/
const Device UA2795537 = {S2574, 221, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SV1; [eburo v4.0]; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2814837 = {S2575, 258, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; Zune 4.7; ATT; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2857436 = {S2576, 222, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2857622 = {S2577, 257, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Media Center PC 5.0; SLCC1; Tablet PC 2.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2858032 = {S2578, 226, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2866231 = {S2579, 251, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2889289 = {S2580, 124, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2892704 = {S2581, 231, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; GIL 3.5; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6136; ms-office; MSOffice 14)*/
const Device UA2937262 = {S2582, 244, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6136; ms-office; MSOffice 14)*/
const Device UA2953225 = {S2583, 236, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2953909 = {S2584, 124, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2956979 = {S2585, 237, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6136; ms-office; MSOffice 14)*/
const Device UA2957416 = {S2586, 249, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2974420 = {S2587, 224, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA2974864 = {S2588, 248, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA2974969 = {S2589, 72, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.1; Microsoft Outlook 14.0.7015; Pro)*/
const Device UA2976144 = {S2590, 231, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA2979125 = {S2591, 228, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6112; ms-office; MSOffice 14)*/
const Device UA2997740 = {S2592, 215, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2999493 = {S2593, 264, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8; Tablet PC 2.0; Microsoft Outlook 14.0.6136; ms-office; MSOffice 14)*/
const Device UA3007227 = {S2594, 256, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA3020240 = {S2595, 256, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.7101; ms-office; MSOffice 14)*/
const Device UA3022354 = {S2596, 251, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3022432 = {S2597, 239, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3022622 = {S2598, 241, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3022661 = {S2599, 215, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3022856 = {S2600, 203, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3023184 = {S2601, 117, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3023449 = {S2602, 123, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3023450 = {S2603, 210, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3024115 = {S2604, 124, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3024352 = {S2605, 116, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3025432 = {S2606, 129, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Win64; x64; Trident/6.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3025948 = {S2607, 231, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; GIL 3.5; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.7101; ms-office; MSOffice 14)*/
const Device UA3042365 = {S2608, 224, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA3046823 = {S2609, 117, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA3047994 = {S2610, 236, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA3049154 = {S2611, 251, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA3057865 = {S2612, 230, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3058082 = {S2613, 128, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3071906 = {S2614, 263, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; ADSSO; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA3078462 = {S2615, 222, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3080043 = {S2616, 72, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.1; Microsoft Outlook 14.0.7106; Pro)*/
const Device UA3080608 = {S2617, 241, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3082188 = {S2618, 222, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3083103 = {S2619, 229, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3083576 = {S2620, 251, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3083581 = {S2621, 224, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3083627 = {S2622, 123, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3083657 = {S2623, 236, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3083730 = {S2624, 124, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3083822 = {S2625, 117, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3083914 = {S2626, 231, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3094583 = {S2627, 129, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Win64; x64; Trident/6.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3095115 = {S2628, 116, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3095432 = {S2629, 215, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3095600 = {S2630, 232, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3095727 = {S2631, 128, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3096032 = {S2632, 215, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3111018 = {S2633, 218, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3111142 = {S2634, 246, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3111160 = {S2635, 230, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3112210 = {S2636, 224, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3125264 = {S2637, 236, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Media Center PC 6.0; Microsoft Outlook 14.0.7101; ms-office; MSOffice 14)*/
const Device UA3127408 = {S2638, 254, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Media Center PC 5.0; SLCC1; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3128296 = {S2639, 236, { &P15364,&P7133,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA91989 = {S2640, 63, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0)*/
const Device UA134990 = {S2641, 70, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0)*/
const Device UA140540 = {S2642, 51, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Windows-RSS-Platform/2.0 (MSIE 9.0; Windows NT 6.0)*/
const Device UA141186 = {S2643, 75, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Win64; x64; Trident/5.0)*/
const Device UA159920 = {S2644, 57, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Empty (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0)*/
const Device UA173753 = {S2645, 111, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1))*/
const Device UA177242 = {S2646, 64, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Empty (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0)*/
const Device UA179285 = {S2647, 85, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Verizon Windows Live)*/
const Device UA189286 = {S2648, 50, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.0)*/
const Device UA312663 = {S2649, 81, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; WindowsDownloads)*/
const Device UA343333 = {S2650, 199, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729))*/
const Device UA417940 = {S2651, 107, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Verizon Windows Live; Verizon Windows Live)*/
const Device UA638610 = {S2652, 130, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.0; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA695919 = {S2653, 61, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0.8112.16421; Windows NT 6.0)*/
const Device UA700901 = {S2654, 50, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0)*/
const Device UA706676 = {S2655, 54, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 ( ; MSIE 9.0; Windows NT 6.0; Trident/5.0)*/
const Device UA770092 = {S2656, 133, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC2; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; Zune 4.7)*/
const Device UA782742 = {S2657, 88, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; WindowsDownloads)*/
const Device UA820483 = {S2658, 159, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1); Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1))*/
const Device UA1868596 = {S2659, 116, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; YDP))*/
const Device UA1913860 = {S2660, 220, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Zune 4.0; Tablet PC 2.0; InfoPath.3; .NET4.0C; .NET4.0E)*/
const Device UA1980195 = {S2661, 54, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; MSIE 9.0; WIndows NT 6.0; en-US)*/
const Device UA2041260 = {S2662, 148, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2133797 = {S2663, 168, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)*/
const Device UA2135704 = {S2664, 62, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Vista; Trident/5.0)*/
const Device UA2136635 = {S2665, 80, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; WindowsDownload)*/
const Device UA2137433 = {S2666, 107, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Mozilla/4.0 (compatible; MSIE 6.0; Menara))*/
const Device UA2164681 = {S2667, 68, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; SV1)*/
const Device UA2165556 = {S2668, 68, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; SV1)*/
const Device UA2205772 = {S2669, 173, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)*/
const Device UA2211658 = {S2670, 75, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.0; Windows Live Writer 1.0)*/
const Device UA2296153 = {S2671, 61, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 ( ; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0)*/
const Device UA2367342 = {S2672, 122, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/4.0; GTB7.4; InfoPath.3; SV1; .NET CLR 3.1.76908; WOW64; en-US)*/
const Device UA2412087 = {S2673, 104, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; 360SE)*/
const Device UA2430587 = {S2674, 87, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.0; MSIE 9.0.8112.16421)*/
const Device UA2440344 = {S2675, 178, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2511615 = {S2676, 87, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; QQBrowser/7.1.5753.400)*/
const Device UA2545683 = {S2677, 161, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)*/
const Device UA2651102 = {S2678, 189, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)*/
const Device UA2656400 = {S2679, 122, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/4.0; GTB7.4; InfoPath.3; SV1; .NET CLR 3.4.53360; WOW64; en-US)*/
const Device UA2670859 = {S2680, 87, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; QQBrowser/7.2.7006.400)*/
const Device UA2726731 = {S2681, 171, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7)*/
const Device UA2787014 = {S2682, 73, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; SV1; SV1)*/
const Device UA2807505 = {S2683, 69, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; MSIE 9.0; Windows NT 6.0; en-US) Trident/5.0*/
const Device UA2823271 = {S2684, 73, { &P15364,&P17285,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; SV1; SV1)*/
const Device UA895501 = {S2685, 244, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone4,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/6.0;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA908825 = {S2686, 108, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.13 (KHTML, like Gecko) Mobile/10A5338d*/
const Device UA913409 = {S2687, 108, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A5355d*/
const Device UA1993140 = {S2688, 92, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2012818 = {S2689, 108, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A5376e*/
const Device UA2040632 = {S2690, 106, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403*/
const Device UA2043451 = {S2691, 106, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405*/
const Device UA2200096 = {S2692, 140, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone 6_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPhone4,1/6.0)*/
const Device UA2202637 = {S2693, 134, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 iPad2,1 BingWeb/3.03.1428.X*/
const Device UA2204693 = {S2694, 46, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - wp-iphone/3.1.4 (iPhone OS 6.0, iPhone) Mobile*/
const Device UA2205805 = {S2695, 167, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48 Instapaper/4.2.6 (+http://www.instapaper.com/iphone)*/
const Device UA2231040 = {S2696, 134, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone4S; iPhone; U; CPU OS 6_0 like Mac OS X; en_US) com.google.GooglePlus/10352 (KHTML, like Gecko) Mobile/N94AP (gzip)*/
const Device UA2231050 = {S2697, 151, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - NAVER(inapp; cafe; 100; 2.7.1) Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName*/
const Device UA2231149 = {S2698, 106, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A406*/
const Device UA2232693 = {S2699, 133, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone5; iPhone; U; CPU OS 6_0 like Mac OS X; en_US) com.google.GooglePlus/10352 (KHTML, like Gecko) Mobile/N41AP (gzip)*/
const Device UA2233623 = {S2700, 133, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone4; iPhone; U; CPU OS 6_0 like Mac OS X; en_US) com.google.GooglePlus/10352 (KHTML, like Gecko) Mobile/N90AP (gzip)*/
const Device UA2234300 = {S2701, 167, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10A405 Safari/7534.48 Instapaper/4.2.6 (+http://www.instapaper.com/iphone)*/
const Device UA2236014 = {S2702, 44, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - wp-iphone/3.1.4 (iPhone OS 6.0, iPad) Mobile*/
const Device UA2260994 = {S2703, 154, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper*/
const Device UA2261035 = {S2704, 94, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2261066 = {S2705, 108, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523*/
const Device UA2261077 = {S2706, 105, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403*/
const Device UA2261155 = {S2707, 108, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525*/
const Device UA2261343 = {S2708, 137, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone4CDMA; iPhone; U; CPU OS 6_0 like Mac OS X; en_US) com.google.GooglePlus/10352 (KHTML, like Gecko) Mobile/N92AP (gzip)*/
const Device UA2261351 = {S2709, 137, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone5CDMA; iPhone; U; CPU OS 6_0 like Mac OS X; en_US) com.google.GooglePlus/10352 (KHTML, like Gecko) Mobile/N42AP (gzip)*/
const Device UA2261359 = {S2710, 248, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone4,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/6.0.1;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA2356810 = {S2711, 167, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48 Instapaper/4.2.6 (+http://www.instapaper.com/iphone)*/
const Device UA2356816 = {S2712, 106, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A407*/
const Device UA2357212 = {S2713, 167, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48 Instapaper/4.2.6 (+http://www.instapaper.com/iphone)*/
const Device UA2363033 = {S2714, 136, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 iPad2,1 BingWeb/3.03.1428.X*/
const Device UA2368553 = {S2715, 109, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8426*/
const Device UA2386324 = {S2716, 138, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/8536.26.7 DiigoBrowser*/
const Device UA2386403 = {S2717, 156, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper*/
const Device UA2390486 = {S2718, 135, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone5; iPhone; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/11546 (KHTML, like Gecko) Mobile/N41AP (gzip)*/
const Device UA2404527 = {S2719, 107, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523*/
const Device UA2404782 = {S2720, 139, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone5CDMA; iPhone; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/11546 (KHTML, like Gecko) Mobile/N42AP (gzip)*/
const Device UA2426599 = {S2721, 46, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - wp-iphone/3.3 (iPhone OS 6.0.1, iPhone) Mobile*/
const Device UA2428242 = {S2722, 125, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3*/
const Device UA2428494 = {S2723, 145, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 bline/1.03 (iPhone OS 6.0.1, iPhone)*/
const Device UA2429477 = {S2724, 142, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone 6_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPad2,1/6.0.1)*/
const Device UA2429904 = {S2725, 108, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550*/
const Device UA2430746 = {S2726, 227, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPad3,4;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0.1;FBSS/2; FBCR/;FBID/tablet;FBLC/en_US]*/
const Device UA2431096 = {S2727, 109, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8500*/
const Device UA2436623 = {S2728, 248, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_2 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone5,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/6.0.2;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA2437553 = {S2729, 108, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551*/
const Device UA2437623 = {S2730, 94, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2437648 = {S2731, 145, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 bline/1.04 (iPhone OS 6.0.1, iPhone)*/
const Device UA2455484 = {S2732, 108, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A505*/
const Device UA2456156 = {S2733, 168, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser*/
const Device UA2460420 = {S2734, 167, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10A551 Safari/7534.48 Instapaper/4.2.7 (+http://www.instapaper.com/iphone)*/
const Device UA2460613 = {S2735, 153, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - NAVER(inapp; cafe; 100; 2.9.0) Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName*/
const Device UA2461223 = {S2736, 137, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone4,1; iPhone; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N94AP (gzip)*/
const Device UA2469395 = {S2737, 46, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Miu Ptt 1.9.2 (iPhone; iPhone OS 6.0.1; zh_TW)*/
const Device UA2469518 = {S2738, 138, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari/8536.26.7 DiigoBrowser*/
const Device UA2469886 = {S2739, 137, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone3,1; iPhone; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N90AP (gzip)*/
const Device UA2469890 = {S2740, 137, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone3,3; iPhone; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N92AP (gzip)*/
const Device UA2470556 = {S2741, 137, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone5,2; iPhone; U; CPU OS 6_0_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N42AP (gzip)*/
const Device UA2470555 = {S2742, 137, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone5,1; iPhone; U; CPU OS 6_0_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N41AP (gzip)*/
const Device UA2486630 = {S2743, 153, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - NAVER(inapp; cafe; 100; 2.9.0) Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_2 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName*/
const Device UA2487059 = {S2744, 145, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 bline/1.04 (iPhone OS 6.0.2, iPhone)*/
const Device UA2487100 = {S2745, 46, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - wp-iphone/3.3.1 (iPhone OS 6.0.1, iPad) Mobile*/
const Device UA2487230 = {S2746, 46, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Miu Ptt 1.9.2 (iPhone; iPhone OS 6.0.2; zh_TW)*/
const Device UA2553685 = {S2747, 136, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/8536.26.7 DiigoBrowser*/
const Device UA2578484 = {S2748, 138, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/8536.26.7 DiigoBrowser*/
const Device UA2597496 = {S2749, 175, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari IPhoneAppVersion/2.5.2 IPhoneWebkitWrapper BillingSupported*/
const Device UA2656725 = {S2750, 136, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550 iPad2,5 BingWeb/3.03.1428.X*/
const Device UA2657834 = {S2751, 44, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Miu Ptt 1.9.2 (iPhone; iPhone OS 6.0; zh_TW)*/
const Device UA2762592 = {S2752, 107, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/9Z999*/
const Device UA2840517 = {S2753, 120, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523(iPhone Vox)*/
const Device UA2847819 = {S2754, 46, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - wp-iphone/3.3 (iPhone OS 6.0.2, iPhone) Mobile*/
const Device UA2848878 = {S2755, 137, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone2,1; iPhone; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N88AP (gzip)*/
const Device UA2849415 = {S2756, 125, { &P12280,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone 4; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/8536.25*/
const Device UA90177 = {S2757, 86, { &P18088,&P17017,&P4732,&P18092 }};	/*Useragent - Vodafone/1.0/0Vodafone543/W08.40.SP1.V32.F3 Browser/Opera Mini/v4.2.0099-2010-04-14(0)*/
const Device UA241885 = {S2758, 106, { &P18088,&P17017,&P4732,&P18092 }};	/*Useragent - Z221; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.22250; U; en-US) Opera 9.50*/
const Device UA416743 = {S2759, 68, { &P18088,&P17017,&P4732,&P18092 }};	/*Useragent - Opera/9.60 (J2ME/MIDP; Opera Mini/4.2.13337/504; U; ru) Presto/2.2.0*/
const Device UA657980 = {S2760, 106, { &P18088,&P17017,&P4732,&P18092 }};	/*Useragent - Z331; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.22250; U; en-US) Opera 9.50*/
const Device UA818689 = {S2761, 86, { &P18088,&P17017,&P4732,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/4.2.13221/25.623; U; en) Presto/2.5.25 Version/10.54*/
const Device UA2155154 = {S2762, 72, { &P18088,&P17017,&P4732,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/4.2.14320/19.916; U; pl) Presto/2.5.25*/
const Device UA2404945 = {S2763, 59, { &P18088,&P17017,&P4732,&P18092 }};	/*Useragent - Opera/9.50 (J2ME/MIDP; Opera Mini/4.2.21071/28.3165; U; en)*/
const Device UA2432688 = {S2764, 90, { &P18088,&P17017,&P4732,&P18092 }};	/*Useragent - 0 Opera/9.80 (J2ME/MIDP; Opera Mini/4.2.23226/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2797829 = {S2765, 88, { &P18088,&P17017,&P4732,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/4.2.23449/30.3214; U; en) Presto/2.8.119 Version/11.10*/
const Device UA3095071 = {S2766, 81, { &P18088,&P17017,&P4732,&P18092 }};	/*Useragent - Opera/9.80 (MTK; Opera Mini/4.2.1255/31.1395; U; en) Presto/2.8.119 Version/11.10*/
const Device UA195057 = {S2767, 71, { &P15364,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0)*/
const Device UA195124 = {S2768, 76, { &P15364,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Win64; x64; Trident/6.0)*/
const Device UA195137 = {S2769, 64, { &P15364,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0)*/
const Device UA661343 = {S2770, 52, { &P15364,&P16110,&P17201,&P18092 }};	/*Useragent - Windows-RSS-Platform/2.0 (MSIE 10.0; Windows NT 6.2)*/
const Device UA2384887 = {S2771, 56, { &P15364,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.2; SV1)*/
const Device UA2418899 = {S2772, 51, { &P15364,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.2)*/
const Device UA2423443 = {S2773, 152, { &P15364,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA2463493 = {S2774, 81, { &P15364,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; XBLWP7; ZuneWP7)*/
const Device UA2526610 = {S2775, 179, { &P15364,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2533504 = {S2776, 95, { &P15364,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; QQBrowser/7.1.5753.400)*/
const Device UA2558466 = {S2777, 88, { &P15364,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; QQBrowser/7.1.5753.400)*/
const Device UA2606699 = {S2778, 95, { &P15364,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; QQBrowser/7.2.7006.400)*/
const Device UA2653213 = {S2779, 58, { &P15364,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.2; 360SE)*/
const Device UA2694879 = {S2780, 167, { &P15364,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)*/
const Device UA2695158 = {S2781, 85, { &P15364,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; SLCC1; .NET CLR 2.0.50727)*/
const Device UA2695180 = {S2782, 62, { &P15364,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 ( ; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0)*/
const Device UA2777911 = {S2783, 63, { &P15364,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; SV1)*/
const Device UA2793198 = {S2784, 174, { &P15364,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; ASU2JS)*/
const Device UA2844249 = {S2785, 54, { &P15364,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 10.0; Windows 8; UCWEB;)*/
const Device UA2996448 = {S2786, 184, { &P15364,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2)*/
const Device UA3008512 = {S2787, 58, { &P15364,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; WOW64; MSIE 10.0; Windows NT 6.2)*/
const Device UA3050254 = {S2788, 171, { &P15364,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; HPNTDFJS; Zune 4.7)*/
const Device UA2362699 = {S2789, 99, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B5105c*/
const Device UA2408165 = {S2790, 99, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B5117b*/
const Device UA2437189 = {S2791, 99, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B5126b*/
const Device UA2482242 = {S2792, 83, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2490822 = {S2793, 97, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141*/
const Device UA2491609 = {S2794, 127, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/8536.26.7 DiigoBrowser*/
const Device UA2492696 = {S2795, 212, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 [FBAN/FBIOS;FBAV/5.4.2;FBBV/X;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/6.1;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]*/
const Device UA2508014 = {S2796, 131, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad2,1; iPad; U; CPU OS 6_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/K93AP (gzip)*/
const Device UA2509291 = {S2797, 157, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser*/
const Device UA2510462 = {S2798, 130, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad3,1; iPad; U; CPU OS 6_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/J1AP (gzip)*/
const Device UA2510726 = {S2799, 96, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176*/
const Device UA2511915 = {S2800, 165, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10B141 Safari/7534.48 Instapaper/4.2.7 (+http://www.instapaper.com/iphone)*/
const Device UA2513022 = {S2801, 111, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 YaMobile/1.01*/
const Device UA2513220 = {S2802, 114, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/7534.48.3*/
const Device UA2513847 = {S2803, 246, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 6_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/6.1;FBSS/1; FBCR/unavailable;FBID/tablet;FBLC/en_US;FBSF/1.0]*/
const Device UA2516586 = {S2804, 120, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 iPad2,1/K93AP Zite/2.1*/
const Device UA2530910 = {S2805, 99, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B311*/
const Device UA2532514 = {S2806, 99, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B147*/
const Device UA2532859 = {S2807, 85, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2533724 = {S2808, 98, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176*/
const Device UA2533728 = {S2809, 129, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/8536.26.7 DiigoBrowser*/
const Device UA2535163 = {S2810, 250, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 6_1_2 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/6.1.2;FBSS/1; FBCR/unavailable;FBID/tablet;FBLC/en_US;FBSF/1.0]*/
const Device UA2535618 = {S2811, 132, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad3,4; iPad; U; CPU OS 6_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/P101AP (gzip)*/
const Device UA2537126 = {S2812, 113, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 YaMobile/1.10*/
const Device UA2538620 = {S2813, 99, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146*/
const Device UA2539701 = {S2814, 221, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 [FBAN/FBIOS;FBAV/5.5;FBBV/X;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/6.1.2;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBOP/0]*/
const Device UA2539841 = {S2815, 225, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B147 [FBAN/FBIOS;FBAV/5.5;FBBV/X;FBDV/iPad3,5;FBMD/iPad;FBSN/iPhone OS;FBSV/6.1.2;FBSS/2; FBCR/AT&T;FBID/tablet;FBLC/en_US;FBOP/0]*/
const Device UA2558714 = {S2816, 112, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/8536.25*/
const Device UA2560632 = {S2817, 133, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad2,1; iPad; U; CPU OS 6_1_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/K93AP (gzip)*/
const Device UA2564961 = {S2818, 159, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser*/
const Device UA2570507 = {S2819, 165, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48 Instapaper/4.2.7 (+http://www.instapaper.com/iphone)*/
const Device UA2570637 = {S2820, 114, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/8536.25*/
const Device UA2570903 = {S2821, 100, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B318a*/
const Device UA2570993 = {S2822, 122, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 iPad2,1/K93AP Zite/2.2*/
const Device UA2571071 = {S2823, 132, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad3,1; iPad; U; CPU OS 6_1_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/J1AP (gzip)*/
const Device UA2582179 = {S2824, 134, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad3,4; iPad; U; CPU OS 6_1_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/P101AP (gzip)*/
const Device UA2582467 = {S2825, 116, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3*/
const Device UA2582734 = {S2826, 85, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2583010 = {S2827, 189, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser*/
const Device UA2583335 = {S2828, 134, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad2,5; iPad; U; CPU OS 6_1_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/P105AP (gzip)*/
const Device UA2584751 = {S2829, 99, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329*/
const Device UA2585862 = {S2830, 129, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/8536.26.7 DiigoBrowser*/
const Device UA2600959 = {S2831, 133, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad2,2; iPad; U; CPU OS 6_1_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/K94AP (gzip)*/
const Device UA2601670 = {S2832, 165, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48 Instapaper/4.2.7 (+http://www.instapaper.com/iphone)*/
const Device UA2606450 = {S2833, 159, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser*/
const Device UA2606826 = {S2834, 113, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B147 YaMobile/1.11*/
const Device UA2606982 = {S2835, 122, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 iPad2,1/K93AP Zite/2.2*/
const Device UA2607276 = {S2836, 98, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176*/
const Device UA2607431 = {S2837, 113, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 YaMobile/1.11*/
const Device UA2607443 = {S2838, 250, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 6_1_3 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/6.1.3;FBSS/1; FBCR/unavailable;FBID/tablet;FBLC/en_US;FBSF/1.0]*/
const Device UA2699214 = {S2839, 40, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - AppCake 3.8 (iPad; iPhone OS 6.1; en_US)*/
const Device UA2700724 = {S2840, 96, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/9Z999*/
const Device UA2703224 = {S2841, 44, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Miu Ptt 1.9.2 (iPad; iPhone OS 6.1.2; zh_TW)*/
const Device UA2733258 = {S2842, 123, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B147 iPad3,6/P103AP Zite/2.2*/
const Device UA2738923 = {S2843, 223, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 [FBAN/FBIOS;FBAV/6.1.1;FBBV/X;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/6.1.3;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBOP/1]*/
const Device UA2778623 = {S2844, 42, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Miu Ptt 1.9.2 (iPad; iPhone OS 6.1; zh_TW)*/
const Device UA2779768 = {S2845, 98, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/9Z999*/
const Device UA2779965 = {S2846, 44, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Zeusmos 2.6.9 (iPad; iPhone OS 6.1.2; en_US)*/
const Device UA2780228 = {S2847, 44, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Miu Ptt 1.9.2 (iPad; iPhone OS 6.1.3; zh_TW)*/
const Device UA2850680 = {S2848, 131, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad2,2; iPad; U; CPU OS 6_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/K94AP (gzip)*/
const Device UA2851036 = {S2849, 111, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/528.16*/
const Device UA2851065 = {S2850, 132, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad2,5; iPad; U; CPU OS 6_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/P105AP (gzip)*/
const Device UA2880284 = {S2851, 134, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad2,5; iPad; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/P105AP (gzip)*/
const Device UA2882035 = {S2852, 132, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad3,1; iPad; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/J1AP (gzip)*/
const Device UA2893977 = {S2853, 42, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - QQ???HD 3.2 (iPad; iPhone OS 6.1.3; zh_CN)*/
const Device UA2898529 = {S2854, 133, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad2,1; iPad; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/K93AP (gzip)*/
const Device UA2898545 = {S2855, 134, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad3,4; iPad; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/P101AP (gzip)*/
const Device UA2935924 = {S2856, 99, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329*/
const Device UA2939388 = {S2857, 133, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad2,2; iPad; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/K94AP (gzip)*/
const Device UA2942323 = {S2858, 133, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad3,3; iPad; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/J2aAP (gzip)*/
const Device UA2954853 = {S2859, 134, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 bline/1.04 (iPhone OS 6.1.3, iPad)*/
const Device UA2959463 = {S2860, 134, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad3,6; iPad; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/P103AP (gzip)*/
const Device UA2966733 = {S2861, 132, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad3,2; iPad; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/J2AP (gzip)*/
const Device UA2973504 = {S2862, 134, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad2,4; iPad; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/K93aAP (gzip)*/
const Device UA2992611 = {S2863, 113, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/525.20*/
const Device UA2999000 = {S2864, 96, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/1.0 (KHTML, like Gecko) Mobile/10B329*/
const Device UA3014052 = {S2865, 133, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad2,3; iPad; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/K95AP (gzip)*/
const Device UA3020820 = {S2866, 96, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/4.6 (KHTML, like Gecko) Mobile/10B329*/
const Device UA3053065 = {S2867, 134, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad3,5; iPad; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/P102AP (gzip)*/
const Device UA3072377 = {S2868, 189, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser*/
const Device UA3074303 = {S2869, 96, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/1.2 (KHTML, like Gecko) Mobile/10B329*/
const Device UA3083507 = {S2870, 128, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) DLBrowser/1.0 Mobile/10B329 Safari/8536.25*/
const Device UA3086697 = {S2871, 129, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B147 Safari/8536.26.7 DiigoBrowser*/
const Device UA3087170 = {S2872, 98, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329*/
const Device UA3090113 = {S2873, 96, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/4.3 (KHTML, like Gecko) Mobile/10B329*/
const Device UA3102332 = {S2874, 106, { &P15767,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile Safari/10B329*/
const Device UA231453 = {S2875, 150, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729)*/
const Device UA340032 = {S2876, 143, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729)*/
const Device UA599087 = {S2877, 174, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7)*/
const Device UA628021 = {S2878, 181, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7)*/
const Device UA635721 = {S2879, 155, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727)*/
const Device UA723260 = {S2880, 153, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7)*/
const Device UA794372 = {S2881, 172, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3)*/
const Device UA860248 = {S2882, 171, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA892948 = {S2883, 171, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2)*/
const Device UA1901458 = {S2884, 158, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729)*/
const Device UA1902954 = {S2885, 177, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; InfoPath.3)*/
const Device UA1958462 = {S2886, 189, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Windows Live Messenger 15.4.3555.0308)*/
const Device UA1968328 = {S2887, 183, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2)*/
const Device UA1980580 = {S2888, 210, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Windows Live Messenger 15.4.3555.0308)*/
const Device UA2161476 = {S2889, 198, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0)*/
const Device UA2224971 = {S2890, 201, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Windows Live Messenger 16.4.3503.0728)*/
const Device UA2245661 = {S2891, 189, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Windows Live Messenger 16.4.3505.0912)*/
const Device UA2325423 = {S2892, 179, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0)*/
const Device UA2359896 = {S2893, 193, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Zune 4.7)*/
const Device UA2360212 = {S2894, 208, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Zune 4.7)*/
const Device UA2362280 = {S2895, 173, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; ASU2JS)*/
const Device UA2375080 = {S2896, 187, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3)*/
const Device UA2387026 = {S2897, 178, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Tablet PC 2.0; ASU2JS)*/
const Device UA2414005 = {S2898, 186, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)*/
const Device UA2424198 = {S2899, 182, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Windows Live Messenger 16.4.3505.0912)*/
const Device UA2456274 = {S2900, 210, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Windows Live Messenger 16.4.3505.0912)*/
const Device UA2495435 = {S2901, 214, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 5.0; Media Center PC 6.0; SLCC1; SLCC2; Tablet PC 2.0)*/
const Device UA2503397 = {S2902, 199, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 5.0; Media Center PC 6.0; SLCC1; SLCC2)*/
const Device UA2642454 = {S2903, 226, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 6.0; SLCC1; SLCC2; Tablet PC 2.0)*/
const Device UA2746477 = {S2904, 196, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Zune 4.7)*/
const Device UA2792638 = {S2905, 63, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/4.0)*/
const Device UA2806998 = {S2906, 168, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7)*/
const Device UA2807499 = {S2907, 180, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7)*/
const Device UA2980511 = {S2908, 211, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 6.0; SLCC1; SLCC2)*/
const Device UA3011923 = {S2909, 188, { &P15364,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; SiteKiosk 8.7 Build 1435)*/
const Device UA2967672 = {S2910, 88, { &P17968,&P17979,&P18412,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/7.5.33495/27.2338; U; en) Presto/2.8.119 480x800 Android*/
const Device UA3114786 = {S2911, 86, { &P17968,&P17979,&P18412,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/7.5.33361/31.1475; U; en) Presto/2.8.119 Version/11.10*/
const Device UA7085 = {S2912, 76, { &P15364,&P5690,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0(Windows; U; Windows NT 5.2; rv:1.9.2) Gecko/20100101 Firefox/3.6*/
const Device UA626415 = {S2913, 84, { &P15364,&P5690,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 Mozilla/5.0 (Windows; U; Windows NT 5.1; de; rv:1.9.2.13) Firefox/3.6.13*/
const Device UA638069 = {S2914, 242, { &P15364,&P5690,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.12) Gecko/X Firefox/3.6.12","Mozilla/5.0 (Windows; U; Windows NT 5.1; pl-PL; rv:1.8.1.24pre) Gecko/X K-Meleon/1.5.4","Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/540.0 (KHTML,*/
const Device UA673868 = {S2915, 100, { &P15364,&P5690,&P17558,&P18092 }};	/*Useragent - Mozilla/6.0 (Windows; U; Windows NT 5.1; pl; rv:1.6.1.6) Gecko/X Firefox/3.6.6 (.NET CLR 3.1.36666)"*/
const Device UA875434 = {S2916, 84, { &P15364,&P5690,&P17558,&P18092 }};	/*Useragent - Mozilla/10.0.2 (Windows; U; Windows NT 5.1;en-US;rv:1.9.2.12) Gecko/X Firefox/3.6.12*/
const Device UA916167 = {S2917, 80, { &P15364,&P5690,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0+(Windows;+U;+Windows+NT+5.1;+de;+rv:1.9.2.16)+Gecko/X+Firefox/3.6.16*/
const Device UA1870720 = {S2918, 107, { &P15364,&P5690,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.15) Gecko/X Firefox/3.6.15 GTB7.1 WebMoney Advisor*/
const Device UA2137031 = {S2919, 109, { &P15364,&P5690,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; Win64; x64; de; rv:1.9.2.6) Gecko/X Firefox/3.6.6 (tete009 SSE2 PGO)*/
const Device UA2260104 = {S2920, 55, { &P15364,&P5690,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0(Windows NT5.1;rv2.0) Gecko/X Firefox/3.6.15*/
const Device UA2286450 = {S2921, 68, { &P15364,&P5690,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; es; rv:1.9.2.16) Gecko/X Firefox/3.6.16*/
const Device UA2670882 = {S2922, 82, { &P15364,&P5690,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; zh-CN; rv:1.9.2) Gecko/X Firefox/3.6 GTB6*/
const Device UA347418 = {S2923, 92, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)*/
const Device UA422684 = {S2924, 105, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179*/
const Device UA441497 = {S2925, 244, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone4,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/5.1;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA599671 = {S2926, 134, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - NaverCafe/1.0 Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName*/
const Device UA615691 = {S2927, 248, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone4,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/5.1.1;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA616340 = {S2928, 107, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206*/
const Device UA616502 = {S2929, 105, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176*/
const Device UA623272 = {S2930, 94, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)*/
const Device UA661684 = {S2931, 105, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406*/
const Device UA808970 = {S2932, 105, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405*/
const Device UA852574 = {S2933, 133, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 iPad2,2 BingWeb/3.02.1641.X*/
const Device UA855729 = {S2934, 131, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone4; U; CPU iPhone OS 5_1 like Mac OS X; en_US) com.google.GooglePlus/5173 (KHTML, like Gecko) Mobile/N90AP (gzip)*/
const Device UA868873 = {S2935, 153, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper*/
const Device UA874317 = {S2936, 135, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 iPad1,1 BingWeb/3.03.1428.X*/
const Device UA896217 = {S2937, 124, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA899392 = {S2938, 107, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208*/
const Device UA899718 = {S2939, 107, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9Z999*/
const Device UA1964434 = {S2940, 101, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206*/
const Device UA1969673 = {S2941, 155, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper*/
const Device UA1994832 = {S2942, 107, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405*/
const Device UA2159175 = {S2943, 121, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/525.20*/
const Device UA2199601 = {S2944, 138, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone4CDMA; iPhone; U; CPU OS 5_1_1 like Mac OS X; en_US) com.google.GooglePlus/8654 (KHTML, like Gecko) Mobile/N92AP (gzip)*/
const Device UA2234680 = {S2945, 136, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone4S; iPhone; U; CPU OS 5_1_1 like Mac OS X; en_US) com.google.GooglePlus/10352 (KHTML, like Gecko) Mobile/N94AP (gzip)*/
const Device UA2234996 = {S2946, 166, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48 Instapaper/4.2.6 (+http://www.instapaper.com/iphone)*/
const Device UA2353318 = {S2947, 153, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - NAVER(inapp; cafe; 100; 2.7.1) Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName*/
const Device UA2363300 = {S2948, 144, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone 5_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPhone4,1/5.1.1)*/
const Device UA2394002 = {S2949, 121, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/534.46*/
const Device UA2402191 = {S2950, 135, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone4; iPhone; U; CPU OS 5_1_1 like Mac OS X; en_US) com.google.GooglePlus/10352 (KHTML, like Gecko) Mobile/N90AP (gzip)*/
const Device UA2420781 = {S2951, 142, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone 5_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPad1,1/5.1.1)*/
const Device UA2422518 = {S2952, 144, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 bline/1.03 (iPhone OS 5.1.1, iPhone)*/
const Device UA2476431 = {S2953, 107, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406*/
const Device UA2487228 = {S2954, 46, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Miu Ptt 1.9.2 (iPhone; iPhone OS 5.1.1; zh_TW)*/
const Device UA2659178 = {S2955, 174, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari IPhoneAppVersion/2.5.2 IPhoneWebkitWrapper BillingSupported*/
const Device UA2729344 = {S2956, 44, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Miu Ptt 1.9.2 (iPhone; iPhone OS 5.1; zh_TW)*/
const Device UA2816697 = {S2957, 48, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Facebook 4100.0 (iPhone; iPhone OS 5.1.1; en_AU)*/
const Device UA2817625 = {S2958, 119, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206(iPhone Vox)*/
const Device UA2840584 = {S2959, 137, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/8536.26.7 DiigoBrowser*/
const Device UA2970008 = {S2960, 238, { &P12280,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 [FBAN/MessengerForiOS;FBAV/2.5;FBBV/X;FBDV/iPad1,1;FBMD/iPad;FBSN/iPhone OS;FBSV/5.1.1;FBSS/1; FBCR/;FBID/phone;FBLC/en_US;FBOP/1]*/
const Device UA1995260 = {S2961, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996084 = {S2962, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9300 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996253 = {S2963, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996481 = {S2964, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996993 = {S2965, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030840 = {S2966, 61, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-I9300 Build/JRO03C)*/
const Device UA2031761 = {S2967, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9300 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033256 = {S2968, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2042571 = {S2969, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2204699 = {S2970, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2204705 = {S2971, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2204884 = {S2972, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2232290 = {S2973, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-i9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2232316 = {S2974, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I535 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2232999 = {S2975, 159, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; SAMSUNG GT-I9300/I9300XXDLIH Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2235161 = {S2976, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2242346 = {S2977, 61, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9300 Build/JZO54K)*/
const Device UA2257965 = {S2978, 145, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2257983 = {S2979, 130, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9300 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2258002 = {S2980, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T999 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2338576 = {S2981, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2348617 = {S2982, 61, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SCH-I535 Build/JRO03L)*/
const Device UA2355404 = {S2983, 140, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I93000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2355950 = {S2984, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2360797 = {S2985, 61, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SGH-T999 Build/JRO03L)*/
const Device UA2365726 = {S2986, 137, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2381899 = {S2987, 137, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; SGH-T999 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2385434 = {S2988, 140, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; GT-I9300T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2402363 = {S2989, 61, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-I535 Build/JZO54K)*/
const Device UA2423516 = {S2990, 158, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; GT-I9300-ORANGE/I9300XXDLJ4 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2424128 = {S2991, 140, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-T999V Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2425250 = {S2992, 62, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-I9300T Build/JRO03C)*/
const Device UA2436122 = {S2993, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; SCH-I939 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2437333 = {S2994, 159, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I9300/I9300XXELL5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455654 = {S2995, 61, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-i9300 Build/IMM76D)*/
const Device UA2457196 = {S2996, 136, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; i9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2457985 = {S2997, 130, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9300 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2458617 = {S2998, 62, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SGH-T999V Build/JRO03L)*/
const Device UA2460873 = {S2999, 158, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-I9300-ORANGE/I9300XXELL1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2466647 = {S3000, 137, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; I9300i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467257 = {S3001, 148, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; GT_I9300 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2468771 = {S3002, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469474 = {S3003, 159, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I9300/I9300XXELLA Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469859 = {S3004, 159, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I9300/I9300XXELL1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2480211 = {S3005, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483108 = {S3006, 136, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483957 = {S3007, 137, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2485312 = {S3008, 137, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; SCH-I535 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2485458 = {S3009, 130, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; SCH-I939 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2487385 = {S3010, 140, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I93000 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2489485 = {S3011, 158, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300-ORANGE/I9300XXELLA Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2502484 = {S3012, 61, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T999 Build/JZO54K)*/
const Device UA2504304 = {S3013, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T999 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2505257 = {S3014, 140, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-I9300T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2507376 = {S3015, 62, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9300T Build/JZO54K)*/
const Device UA2514680 = {S3016, 159, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I9300/I9300XXEMR2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2523829 = {S3017, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2527722 = {S3018, 140, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; SCH-I939D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528561 = {S3019, 130, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2528654 = {S3020, 61, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.9; GT-I9300 Build/IMM76D)*/
const Device UA2539083 = {S3021, 159, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; SAMSUNG GT-I9300/I9300XXEMA2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2549116 = {S3022, 145, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30*/
const Device UA2553918 = {S3023, 137, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.5; en-us; GT-I9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2555977 = {S3024, 145, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30*/
const Device UA2559571 = {S3025, 140, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T999L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2560908 = {S3026, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I939 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2577853 = {S3027, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.5; en-us; GT-I9300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597961 = {S3028, 137, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; I9300i Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2600660 = {S3029, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-l9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2603762 = {S3030, 137, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SCH-I535 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2653934 = {S3031, 134, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; i9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658337 = {S3032, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.3; en-us; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659085 = {S3033, 148, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-gb; GT_I9300 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2660078 = {S3034, 140, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I939D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665370 = {S3035, 130, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I939 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2666694 = {S3036, 158, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-I9300-ORANGE/I9300XXELLC Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2675413 = {S3037, 159, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; SAMSUNG GT-I9300/I9300XXEMR7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2678026 = {S3038, 158, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-I9300-ORANGE/I9300XXEMR7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695707 = {S3039, 152, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9300 Build/_miui.polimorph_ v1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2696810 = {S3040, 138, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; SGH-T999V Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2697602 = {S3041, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; ar-eg; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2698359 = {S3042, 137, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.4; en-us; GT-i9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2699499 = {S3043, 132, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9300 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2701304 = {S3044, 61, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-I939 Build/JZO54K)*/
const Device UA2701313 = {S3045, 62, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-I93000 Build/JRO03C)*/
const Device UA2704200 = {S3046, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; es-es; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708708 = {S3047, 137, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.4; es-es; GT-i9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2729057 = {S3048, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; mini I9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2729981 = {S3049, 151, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2_????_??????; he-il; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2731091 = {S3050, 136, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2733867 = {S3051, 155, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; I9300 Galaxy SIII Build/MocorDroid4.1.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2740417 = {S3052, 159, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I9300/I9300XXEMRA Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2741468 = {S3053, 159, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I9300/I9300XXEMC2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747185 = {S3054, 61, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SCH-I939 Build/JRO03C)*/
const Device UA2748216 = {S3055, 137, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; I9300+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2754697 = {S3056, 137, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SGH-T999 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2763128 = {S3057, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; fr-fr; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2763891 = {S3058, 141, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/By Rohit) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2764002 = {S3059, 159, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; SAMSUNG GT-I9300/I9300XXELLC Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2764232 = {S3060, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; tr-tr; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778725 = {S3061, 58, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; I9300 Build/IMM76D)*/
const Device UA2780988 = {S3062, 146, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9300 Build/MocorDroid4.1.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2781757 = {S3063, 137, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2797474 = {S3064, 62, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T999L Build/JZO54K)*/
const Device UA2839057 = {S3065, 141, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9300 Build/GT-I9300) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2843521 = {S3066, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2858037 = {S3067, 159, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-ch; SAMSUNG GT-I9300/I9300XXELL4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2864903 = {S3068, 137, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SC-06D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878193 = {S3069, 59, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SC-06D Build/JZO54K)*/
const Device UA2878266 = {S3070, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; it-it; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2891834 = {S3071, 140, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T999N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2895512 = {S3072, 158, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300-ORANGE/I9300XXEMF1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2899386 = {S3073, 159, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I9300/I9300XXEMF1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936613 = {S3074, 147, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9300. Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2937155 = {S3075, 140, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-T999V Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2971484 = {S3076, 62, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T999V Build/JZO54K)*/
const Device UA2972552 = {S3077, 159, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I9300/I9300XXEMRD Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2975754 = {S3078, 159, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-ch; SAMSUNG GT-I9300/I9300XXEMF6 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2988299 = {S3079, 143, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; GT-I9300 Build/JRO03C-1.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3028300 = {S3080, 62, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T999N Build/JZO54K)*/
const Device UA3059456 = {S3081, 158, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9300 Build/JZO54K) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 T5/2.0*/
const Device UA3063453 = {S3082, 169, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=d2vzw;devm=SCH-I535;man=samsung;brw=Android;shwd=720x1280;shwa=720x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA3070809 = {S3083, 145, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I535 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA3087045 = {S3084, 62, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-I939D Build/JZO54K)*/
const Device UA3101422 = {S3085, 138, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-I9300T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3109140 = {S3086, 169, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=d2tmo;devm=SGH-T999;man=samsung;brw=Android;shwd=720x1280;shwa=720x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA3124750 = {S3087, 138, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SGH-T999V Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125759 = {S3088, 139, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9300 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125896 = {S3089, 156, { &P18283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-; GT-I9300 Build/JZO54K) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 T5/2.0*/
const Device UA2539646 = {S3090, 127, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/26.0.1410.12) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.12 Safari/537.31*/
const Device UA2566665 = {S3091, 127, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/26.0.1410.19) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.19 Safari/537.31*/
const Device UA2567735 = {S3092, 127, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/26.0.1410.28) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.28 Safari/537.31*/
const Device UA2571020 = {S3093, 127, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/26.0.1410.33) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.33 Safari/537.31*/
const Device UA2580143 = {S3094, 125, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/26.0.1386.0) AppleWebKit/537.27 (KHTML, like Gecko) Chrome/26.0.1386.0 Safari/537.27*/
const Device UA2585525 = {S3095, 127, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/26.0.1410.40) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Safari/537.31*/
const Device UA2602786 = {S3096, 127, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/26.0.1410.43) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/537.31*/
const Device UA2605299 = {S3097, 127, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/26.0.1410.43) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/537.31*/
const Device UA2672640 = {S3098, 134, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/26.0.1410.43) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/537.31*/
const Device UA2673309 = {S3099, 192, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/26.0.1410.64; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA2673310 = {S3100, 127, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/26.0.1410.64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2673382 = {S3101, 206, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/26.0.1410.64; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2673817 = {S3102, 220, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/26.0.1410.64; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2674125 = {S3103, 160, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/26.0.1410.64; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2675656 = {S3104, 134, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/26.0.1410.64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2675667 = {S3105, 159, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/26.0.1410.64; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2676710 = {S3106, 238, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/26.0.1410.64; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2676975 = {S3107, 76, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/26.0.1410.64)*/
const Device UA2677022 = {S3108, 140, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/26.0.1410.64; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)*/
const Device UA2678533 = {S3109, 127, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/26.0.1410.64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2678686 = {S3110, 172, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/26.0.1410.64; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2678814 = {S3111, 159, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/26.0.1410.64; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2679187 = {S3112, 81, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/26.0.1410.64)*/
const Device UA2679360 = {S3113, 207, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; GTB7.4; chromeframe/26.0.1410.64; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2715748 = {S3114, 257, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; chromeframe/26.0.1410.64; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2716063 = {S3115, 261, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/26.0.1410.64; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; .NET4.0C; .NET4.0E; Zune 4.7)*/
const Device UA2719162 = {S3116, 278, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/26.0.1410.64; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2735499 = {S3117, 147, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; chromeframe/26.0.1410.64; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2736138 = {S3118, 164, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; SV1; chromeframe/26.0.1410.64; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2736373 = {S3119, 238, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/26.0.1410.64; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2752695 = {S3120, 279, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/26.0.1410.43; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322; .NET CLR 1.0.3705; Tablet PC 1.7; .NET4.0C; .NET4.0E; Zurich Financial Services; Zurich Financial Services)*/
const Device UA2862780 = {S3121, 210, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; chromeframe/26.0.1410.64; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8; .NET4.0C; .NET4.0E)*/
const Device UA2998257 = {S3122, 127, { &P15364,&P5690,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/26.0.1410.67) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.67 Safari/537.31*/
const Device UA137022 = {S3123, 175, { &P15364,&P7133,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; TmstmpExt)*/
const Device UA137832 = {S3124, 168, { &P15364,&P7133,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; TmstmpExt)*/
const Device UA138500 = {S3125, 222, { &P15364,&P7133,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; Mozilla/4.0 ; MSIE 6.0; Windows NT 5.1; SV1; ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; TmstmpExt)*/
const Device UA138775 = {S3126, 183, { &P15364,&P7133,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; TmstmpExt)*/
const Device UA165661 = {S3127, 147, { &P15364,&P7133,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; TmstmpExt)*/
const Device UA167720 = {S3128, 185, { &P15364,&P7133,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7; TmstmpExt)*/
const Device UA171030 = {S3129, 196, { &P15364,&P7133,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Media Center PC 5.0; SLCC1; TmstmpExt)*/
const Device UA175433 = {S3130, 211, { &P15364,&P7133,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Media Center PC 5.0; SLCC1; Tablet PC 2.0; TmstmpExt)*/
const Device UA182262 = {S3131, 87, { &P15364,&P7133,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; .NET4.0C; TmstmpExt)*/
const Device UA445219 = {S3132, 178, { &P15364,&P7133,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7; TmstmpExt)*/
const Device UA471290 = {S3133, 215, { &P15364,&P7133,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 ; MSIE 6.0; Windows NT 5.1; SV1; ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; TmstmpExt)*/
const Device UA472345 = {S3134, 230, { &P15364,&P7133,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 ; MSIE 6.0; Windows NT 5.1; SV1; ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; TmstmpExt)*/
const Device UA523379 = {S3135, 162, { &P15364,&P7133,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Tablet PC 2.0; TmstmpExt)*/
const Device UA782467 = {S3136, 190, { &P15364,&P7133,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Avant Browser; TmstmpExt)*/
const Device UA1884669 = {S3137, 193, { &P15364,&P7133,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Zune 4.7; TmstmpExt)*/
const Device UA1885402 = {S3138, 157, { &P15364,&P7133,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7; TmstmpExt)*/
const Device UA2164022 = {S3139, 175, { &P15364,&P7133,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; TmstmpExt)*/
const Device UA2213431 = {S3140, 164, { &P15364,&P7133,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7; TmstmpExt)*/
const Device UA2410410 = {S3141, 183, { &P15364,&P7133,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Avant Browser; TmstmpExt)*/
const Device UA2519826 = {S3142, 168, { &P15364,&P7133,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; TmstmpExt)*/
const Device UA2768564 = {S3143, 186, { &P15364,&P7133,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; TmstmpExt)*/
const Device UA3048775 = {S3144, 169, { &P15364,&P7133,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 10.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; TmstmpExt)*/
const Device UA2675845 = {S3145, 127, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/27.0.1453.15) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.15 Safari/537.36*/
const Device UA2678970 = {S3146, 127, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/27.0.1453.56) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.56 Safari/537.36*/
const Device UA2679180 = {S3147, 127, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/27.0.1453.47) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.47 Safari/537.36*/
const Device UA2715222 = {S3148, 127, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/27.0.1453.73) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.73 Safari/537.36*/
const Device UA2717948 = {S3149, 127, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/27.0.1453.81) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.81 Safari/537.36*/
const Device UA2719034 = {S3150, 127, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/27.0.1453.65) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.65 Safari/537.36*/
const Device UA2740625 = {S3151, 127, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/27.0.1453.93) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36*/
const Device UA2753239 = {S3152, 134, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/27.0.1453.94) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36*/
const Device UA2756850 = {S3153, 127, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/27.0.1453.94) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36*/
const Device UA2756948 = {S3154, 127, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/27.0.1453.94) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36*/
const Device UA2756977 = {S3155, 192, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/27.0.1453.94; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA2774226 = {S3156, 129, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/27.0.1453.110) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36*/
const Device UA2783144 = {S3157, 129, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/27.0.1453.110) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36*/
const Device UA2784519 = {S3158, 221, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/27.0.1453.110; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2799887 = {S3159, 136, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/27.0.1453.110) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36*/
const Device UA2873734 = {S3160, 262, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/27.0.1453.116; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; .NET4.0C; .NET4.0E; Zune 4.7)*/
const Device UA2873709 = {S3161, 160, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/27.0.1453.116; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2873739 = {S3162, 129, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/27.0.1453.116) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2874802 = {S3163, 82, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/27.0.1453.116)*/
const Device UA2875068 = {S3164, 173, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/27.0.1453.116; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2875650 = {S3165, 207, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/27.0.1453.116; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2875857 = {S3166, 129, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/27.0.1453.116) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2876753 = {S3167, 231, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/27.0.1453.116; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2880054 = {S3168, 161, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/27.0.1453.116; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2880145 = {S3169, 136, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/27.0.1453.116) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2882588 = {S3170, 141, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/27.0.1453.116; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)*/
const Device UA2883808 = {S3171, 200, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/27.0.1453.116; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2884374 = {S3172, 180, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/27.0.1453.116; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA2885238 = {S3173, 77, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/27.0.1453.116)*/
const Device UA2897151 = {S3174, 206, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/27.0.1453.116; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 1.1.4322; InfoPath.1)*/
const Device UA3077215 = {S3175, 258, { &P15364,&P5690,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; chromeframe/27.0.1453.116; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA38684 = {S3176, 120, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0(Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA69784 = {S3177, 120, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3;en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA69912 = {S3178, 120, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OSX 10_6_3; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA101491 = {S3179, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; nl-nl) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA101965 = {S3180, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; it-it) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA102141 = {S3181, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; de-de) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA102177 = {S3182, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA102785 = {S3183, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; nb-no) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA103045 = {S3184, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; it-it) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA103105 = {S3185, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA103165 = {S3186, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; zh-cn) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA103275 = {S3187, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; sv-se) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA103566 = {S3188, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; es-es) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA103802 = {S3189, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; de-de) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA105560 = {S3190, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-au) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA105652 = {S3191, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA105763 = {S3192, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; es-es) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA106911 = {S3193, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA106912 = {S3194, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ja-jp) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA107215 = {S3195, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fr-fr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA107265 = {S3196, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pt-br) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA107411 = {S3197, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; tr-tr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA108135 = {S3198, 118, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ar) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA110006 = {S3199, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ru-ru) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA110819 = {S3200, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pl-pl) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA112167 = {S3201, 118, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA113925 = {S3202, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pt-pt) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA118881 = {S3203, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ru-ru) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA118985 = {S3204, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; nl-nl) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA119346 = {S3205, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA119407 = {S3206, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fr-fr) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA119551 = {S3207, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; zh-cn) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA119646 = {S3208, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; it-it) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA119744 = {S3209, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ja-jp) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA119842 = {S3210, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-gb) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA120146 = {S3211, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pl-pl) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA120232 = {S3212, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; es-es) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA120337 = {S3213, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; de-de) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA121526 = {S3214, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-us) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA122992 = {S3215, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; nb-no) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA123004 = {S3216, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fi-fi) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA123028 = {S3217, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA124225 = {S3218, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pt-br) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA124717 = {S3219, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ar) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA125656 = {S3220, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fr-fr) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA126068 = {S3221, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; th-th) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA126308 = {S3222, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; sv-se) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA131841 = {S3223, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; de-de) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA132369 = {S3224, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fr-fr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA132370 = {S3225, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; nl-nl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA132410 = {S3226, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; fr-fr) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA132487 = {S3227, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA132531 = {S3228, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; sv-se) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA132506 = {S3229, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ja-jp) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA132566 = {S3230, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; cs-cz) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA132750 = {S3231, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-gb) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA132775 = {S3232, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ru-ru) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA132744 = {S3233, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; it-it) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA132797 = {S3234, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; es-es) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA132809 = {S3235, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-ca) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA132859 = {S3236, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pt-br) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA132864 = {S3237, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; zh-tw) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA132943 = {S3238, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pl-pl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA133023 = {S3239, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nl-nl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA133304 = {S3240, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ar) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA133348 = {S3241, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fr-fr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA133593 = {S3242, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA133827 = {S3243, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pt-pt) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA133841 = {S3244, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; nb-no) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA133869 = {S3245, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ko-kr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA134031 = {S3246, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; da-dk) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA134416 = {S3247, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA134542 = {S3248, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; zh-cn) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA134661 = {S3249, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; es-es) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA135201 = {S3250, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fi-fi) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA135253 = {S3251, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-au) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA135374 = {S3252, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; th-th) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA136033 = {S3253, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; he-il) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA136041 = {S3254, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; es-xl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA136333 = {S3255, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA136346 = {S3256, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; tr-tr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA137350 = {S3257, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; fr-fr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA137721 = {S3258, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; is-is) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA138353 = {S3259, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; nl-nl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA140741 = {S3260, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; de-de) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA141437 = {S3261, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ru-ru) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA141494 = {S3262, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; sv-se) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA143409 = {S3263, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fa-ir) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA144089 = {S3264, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; es-es) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA148083 = {S3265, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; es-es) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA148494 = {S3266, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-gb) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA150039 = {S3267, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; zh-tw) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA150129 = {S3268, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; de-de) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA150702 = {S3269, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; nb-no) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA150705 = {S3270, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; zh-cn) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA151033 = {S3271, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-au) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA151016 = {S3272, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ja-jp) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA151347 = {S3273, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; da-dk) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA151387 = {S3274, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; fi-fi) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA151552 = {S3275, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; pt-br) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA151658 = {S3276, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA152297 = {S3277, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; tr-tr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA152353 = {S3278, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; th-th) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA152769 = {S3279, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; it-it) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA153544 = {S3280, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; nl-nl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA153547 = {S3281, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; it-it) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA153585 = {S3282, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-gb) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA153590 = {S3283, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ko-kr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA153637 = {S3284, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA154365 = {S3285, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ko-kr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA154467 = {S3286, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; fr-fr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA154483 = {S3287, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; pt-br) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA154507 = {S3288, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ru-ru) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA154903 = {S3289, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ja-jp) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA155113 = {S3290, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; lt-lt) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA155477 = {S3291, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; zh-tw) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA155492 = {S3292, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko, Safari/6533.19.4) ADM/722*/
const Device UA155534 = {S3293, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; es-es) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA155567 = {S3294, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; pt-pt) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA156100 = {S3295, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; tr-tr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA156241 = {S3296, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; da-dk) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA156251 = {S3297, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; sv-se) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA156268 = {S3298, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; fi-fi) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA156276 = {S3299, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-au) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA156867 = {S3300, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; fa-ir) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA157011 = {S3301, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; it-it) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA157181 = {S3302, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; de-ch) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA157401 = {S3303, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA157559 = {S3304, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; cs-cz) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA157670 = {S3305, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fr-fr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA157714 = {S3306, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; nl-nl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA157744 = {S3307, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; nb-no) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA157851 = {S3308, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA157919 = {S3309, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; de-de) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA158091 = {S3310, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; es-es) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA158675 = {S3311, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; vi-vn) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA158687 = {S3312, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ru-ru) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA159063 = {S3313, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; el-gr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA159186 = {S3314, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; es-es) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA159380 = {S3315, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; es-xl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA159627 = {S3316, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-gb) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA159735 = {S3317, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; sv-se) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA160049 = {S3318, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pt-br) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA160348 = {S3319, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; da-dk) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA160383 = {S3320, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-au) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA160682 = {S3321, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ru-ru) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA161877 = {S3322, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; nl-nl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA163309 = {S3323, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; nl-nl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA163649 = {S3324, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-gb) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA165587 = {S3325, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pl-pl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA166401 = {S3326, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pt-pt) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA166403 = {S3327, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; zh-cn) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA166449 = {S3328, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; is-is) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA166466 = {S3329, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; zh-tw) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA166802 = {S3330, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; it-it) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA166810 = {S3331, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; es-es) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA166838 = {S3332, 126, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; de-de) AppleWebKit/534.15+ (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA166880 = {S3333, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ja-jp) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA166937 = {S3334, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; it-it) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA167082 = {S3335, 130, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.19.4*/
const Device UA167095 = {S3336, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pt-br) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA167099 = {S3337, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fi-fi) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA167112 = {S3338, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ko-kr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA167386 = {S3339, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-ca) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA167427 = {S3340, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; he-il) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA167432 = {S3341, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; de-de) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA167457 = {S3342, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA167461 = {S3343, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; th-th) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA167466 = {S3344, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ar) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA167495 = {S3345, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; tr-tr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA167498 = {S3346, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; de-ch) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA167554 = {S3347, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; cs-cz) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA167568 = {S3348, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; nb-no) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA167604 = {S3349, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; nb-no) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA168274 = {S3350, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-ca) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA168291 = {S3351, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-fr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA168293 = {S3352, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; de-de) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA168312 = {S3353, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ja-jp) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA168454 = {S3354, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fi-fi) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA168464 = {S3355, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; sl-si) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA168613 = {S3356, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; af-za) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA168623 = {S3357, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-fr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA168686 = {S3358, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; da-dk) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA168744 = {S3359, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; sk-sk) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA168875 = {S3360, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ko-kr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA169121 = {S3361, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; sv-se) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA169296 = {S3362, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; lt-lt) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA170182 = {S3363, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; zh-cn) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA170687 = {S3364, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pt-br) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA171470 = {S3365, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-au) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA172694 = {S3366, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; es-es) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA173813 = {S3367, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA173994 = {S3368, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA174836 = {S3369, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; tr-tr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA174993 = {S3370, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fr-fr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA175306 = {S3371, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; de-de) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA175452 = {S3372, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; nl-nl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA175603 = {S3373, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; cs-cz) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA176008 = {S3374, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; de-de) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA176009 = {S3375, 128, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA176012 = {S3376, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; zh-cn) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA176007 = {S3377, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ja-jp) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA176466 = {S3378, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-gb) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA176601 = {S3379, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; el-gr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA176847 = {S3380, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-au) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA176866 = {S3381, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; sk-sk) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA176872 = {S3382, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-ca) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA176998 = {S3383, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pt-br) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA177082 = {S3384, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fr-fr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA177202 = {S3385, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; pt-pt) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA177194 = {S3386, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; da-dk) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA177274 = {S3387, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; zh-tw) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA177375 = {S3388, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; pl-pl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA177377 = {S3389, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; es-es) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA177376 = {S3390, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; sv-se) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA177882 = {S3391, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; de-de) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA177941 = {S3392, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ru-ru) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA177960 = {S3393, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; is-is) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA177972 = {S3394, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; nb-no) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA177994 = {S3395, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; pt-br) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA178022 = {S3396, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ko-kr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA178178 = {S3397, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; es-xl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA178196 = {S3398, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fi-fi) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA178774 = {S3399, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; sv-se) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA178777 = {S3400, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA179152 = {S3401, 132, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.20.27*/
const Device UA179161 = {S3402, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; th-th) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA180170 = {S3403, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180172 = {S3404, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; da-dk) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180412 = {S3405, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180413 = {S3406, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; el-gr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180416 = {S3407, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180429 = {S3408, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180432 = {S3409, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; is-is) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180445 = {S3410, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-gb) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180458 = {S3411, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-au) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180468 = {S3412, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ja-jp) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180477 = {S3413, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180479 = {S3414, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-ca) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180484 = {S3415, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; pt-br) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180512 = {S3416, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ko-kr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180513 = {S3417, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; zh-cn) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180520 = {S3418, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180531 = {S3419, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; zh-tw) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180621 = {S3420, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; sk-sk) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA181282 = {S3421, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; lt-lt) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA181296 = {S3422, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA181320 = {S3423, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; pt-pt) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA181386 = {S3424, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA181418 = {S3425, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; cs-cz) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA181419 = {S3426, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA181424 = {S3427, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fi-fi) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA181431 = {S3428, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA181523 = {S3429, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; pl-pl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA181518 = {S3430, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; nb-no) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA181782 = {S3431, 130, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.21.1*/
const Device UA181803 = {S3432, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; tr-tr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA181822 = {S3433, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fa-ir) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA181915 = {S3434, 126, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA182085 = {S3435, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; sl-si) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA182228 = {S3436, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA182298 = {S3437, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; es-xl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA182342 = {S3438, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; vi-vn) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA182454 = {S3439, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; it-it) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA182616 = {S3440, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ja-jp) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA182765 = {S3441, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ar) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA182893 = {S3442, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; he-il) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183019 = {S3443, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA183190 = {S3444, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; nl-nl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183191 = {S3445, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; es-es) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183196 = {S3446, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; sv-se) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183197 = {S3447, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-fr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183212 = {S3448, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pl-pl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183305 = {S3449, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; sv-se) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA183371 = {S3450, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fr-ca) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA183474 = {S3451, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pt-br) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183624 = {S3452, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; it-it) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183639 = {S3453, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; zh-cn) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183641 = {S3454, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-gb) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183660 = {S3455, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183689 = {S3456, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; da-dk) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183711 = {S3457, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fi-fi) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183735 = {S3458, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; th-th) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183851 = {S3459, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; th-th) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA183859 = {S3460, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; tr-tr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA183928 = {S3461, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fr-ch) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA184097 = {S3462, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; de-de) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA184101 = {S3463, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; nb-no) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA184119 = {S3464, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA184312 = {S3465, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ru-ru) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA184314 = {S3466, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ko-kr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA184653 = {S3467, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; nn-no) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA184680 = {S3468, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; hu-hu) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA184957 = {S3469, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; zh-tw) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA185253 = {S3470, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-ca) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA185387 = {S3471, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA185440 = {S3472, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ro-ro) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA186188 = {S3473, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-au) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA186547 = {S3474, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; tr-tr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA187930 = {S3475, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; lt-lt) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA188249 = {S3476, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; de-ch) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA188477 = {S3477, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; nl-nl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA188567 = {S3478, 126, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/534.16+ (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA188911 = {S3479, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA189804 = {S3480, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; sv-se) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA190517 = {S3481, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA190607 = {S3482, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-gb) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA193534 = {S3483, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA193663 = {S3484, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA193736 = {S3485, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA195651 = {S3486, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nb-no) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA195925 = {S3487, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; hu-hu) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA197355 = {S3488, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fi-fi) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA197590 = {S3489, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; is-is) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA197563 = {S3490, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pl-pl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA197788 = {S3491, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ja-jp) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA197793 = {S3492, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pt-br) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA197800 = {S3493, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sk-sk) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA197801 = {S3494, 130, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.21.1*/
const Device UA197831 = {S3495, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ko-kr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA197875 = {S3496, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; lt-lt) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA197957 = {S3497, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; cs-cz) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA198005 = {S3498, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh-tw) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA198229 = {S3499, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; af-za) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA198231 = {S3500, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; tr-tr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA198239 = {S3501, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA198288 = {S3502, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; es-es) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA198386 = {S3503, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; id) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA198466 = {S3504, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es-xl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA198431 = {S3505, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA198526 = {S3506, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr-ca) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA198553 = {S3507, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; da-dk) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA198575 = {S3508, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pt-pt) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA198587 = {S3509, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA198589 = {S3510, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-ca) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA198608 = {S3511, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA198633 = {S3512, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-au) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA198697 = {S3513, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; el-gr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA198736 = {S3514, 128, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA198751 = {S3515, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh-cn) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA198759 = {S3516, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ru-ru) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA200133 = {S3517, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; vi-vn) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA200873 = {S3518, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA200966 = {S3519, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-ch) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA201205 = {S3520, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fa-ir) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA201539 = {S3521, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-at) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA201657 = {S3522, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; th-th) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA202358 = {S3523, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sv-se) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA202481 = {S3524, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; et-ee) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA202590 = {S3525, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA204531 = {S3526, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ar) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA205368 = {S3527, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-gb) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA205433 = {S3528, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; tl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA205607 = {S3529, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; hr-hr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA206212 = {S3530, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA207475 = {S3531, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA207842 = {S3532, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; la) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA208970 = {S3533, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sl-si) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA208990 = {S3534, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; he-il) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA209770 = {S3535, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; uk-ua) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA218308 = {S3536, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr-ch) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA221085 = {S3537, 134, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; USCCADT71US; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA225874 = {S3538, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; bg-bg) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA242793 = {S3539, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_US) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA261337 = {S3540, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ja-jp) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA261743 = {S3541, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pt-br) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA262301 = {S3542, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fr-fr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA263498 = {S3543, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; sv-se) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA263927 = {S3544, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; zh-tw) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA265429 = {S3545, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ru-ru) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA266146 = {S3546, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; da-dk) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA267094 = {S3547, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ko-kr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA267551 = {S3548, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; fr-fr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA271684 = {S3549, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pt-pt) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA272120 = {S3550, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-ca) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA273017 = {S3551, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-au) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA274054 = {S3552, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fi-fi) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA274152 = {S3553, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; da-dk) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA274649 = {S3554, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; zh-tw) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA274944 = {S3555, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ko-kr) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA276277 = {S3556, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA276688 = {S3557, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; tr-tr) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA276882 = {S3558, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; de-ch) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA277132 = {S3559, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; el-gr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA277512 = {S3560, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; vi-vn) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA277671 = {S3561, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; sk-sk) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA277973 = {S3562, 130, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.18.5*/
const Device UA278234 = {S3563, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA278481 = {S3564, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; nl-nl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA279264 = {S3565, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; hu-hu) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA279509 = {S3566, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; it-it) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA279901 = {S3567, 118, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA282328 = {S3568, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; de-de) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA282671 = {S3569, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ru-ru) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA283409 = {S3570, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; fr-fr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA285044 = {S3571, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; es-es) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA288770 = {S3572, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; cs-cz) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA288766 = {S3573, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; el-gr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA289166 = {S3574, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-ca) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA289607 = {S3575, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; pl-pl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA290676 = {S3576, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; de-ch) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA291577 = {S3577, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; zh-cn) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA292133 = {S3578, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-ca) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA292231 = {S3579, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; hu-hu) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA292448 = {S3580, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; is-is) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA292548 = {S3581, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; th-th) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA292596 = {S3582, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; pl-pl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA292651 = {S3583, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; pt-pt) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA292923 = {S3584, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ar) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA293064 = {S3585, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; vi-vn) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA293298 = {S3586, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ro-ro) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA293416 = {S3587, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ru-ru) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA293548 = {S3588, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ar) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA294288 = {S3589, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; nb-no) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA296981 = {S3590, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pt-br) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA299130 = {S3591, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; sv-se) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA299134 = {S3592, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-au) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA299926 = {S3593, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; hu-hu) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA299948 = {S3594, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ca-es) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA300002 = {S3595, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; hr-hr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA300390 = {S3596, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; el-gr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA300403 = {S3597, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA300629 = {S3598, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; vi-vn) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA301522 = {S3599, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pl-pl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA301776 = {S3600, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA301817 = {S3601, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fa-ir) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA302226 = {S3602, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; bg-bg) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA302270 = {S3603, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; es-xl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA302411 = {S3604, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; cs-cz) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA303491 = {S3605, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pt-pt) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA303513 = {S3606, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; zh-tw) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA303626 = {S3607, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; th-th) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA306249 = {S3608, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fa-ir) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA307257 = {S3609, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-au) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA307342 = {S3610, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; cs-cz) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA307423 = {S3611, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fr-ca) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA307590 = {S3612, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; it-it) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA307636 = {S3613, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pt-pt) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA307668 = {S3614, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; el-gr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA307916 = {S3615, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-ca) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA308318 = {S3616, 132, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.20.27*/
const Device UA308965 = {S3617, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ca-es) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA309658 = {S3618, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ru-ru) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA310133 = {S3619, 126, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ar) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA310318 = {S3620, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; es-es) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA310535 = {S3621, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; de-de) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA310943 = {S3622, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; af-za) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA311280 = {S3623, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; el-gr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA311547 = {S3624, 126, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ar) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA311756 = {S3625, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; de-ch) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA311768 = {S3626, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; sl-si) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA311781 = {S3627, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; he-il) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA311996 = {S3628, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ca-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA312322 = {S3629, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-gb) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA312351 = {S3630, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; af-za) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA312393 = {S3631, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; de-at) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA312521 = {S3632, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; vi-vn) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA312578 = {S3633, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-gb) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA312659 = {S3634, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; lv-lv) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA312688 = {S3635, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA312890 = {S3636, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ja-jp) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA313337 = {S3637, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; uk-ua) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA313545 = {S3638, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fa-ir) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA313957 = {S3639, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ru-ru) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA314124 = {S3640, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; la) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA314426 = {S3641, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA314465 = {S3642, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA316618 = {S3643, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pt-br) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA316826 = {S3644, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; tl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA317521 = {S3645, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; nb-no) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA317615 = {S3646, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA320985 = {S3647, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ca-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA322570 = {S3648, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nn-no) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA325086 = {S3649, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA332956 = {S3650, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ro-ro) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA334637 = {S3651, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; hy-am) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA337019 = {S3652, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA337477 = {S3653, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA342551 = {S3654, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; yi) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA372224 = {S3655, 130, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.17.8 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.17.8*/
const Device UA372374 = {S3656, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fr-ch) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA373317 = {S3657, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ca-es) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA375372 = {S3658, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; es-xl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA375568 = {S3659, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; lt-lt) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA376305 = {S3660, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; af-za) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA376586 = {S3661, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; sr-cs) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA379883 = {S3662, 133, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pcdadr6350; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA382527 = {S3663, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; zh-cn) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA383469 = {S3664, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nl-nl) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA384246 = {S3665, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ca-es) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA384431 = {S3666, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; de-de) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA386250 = {S3667, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ja-jp) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA386363 = {S3668, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; sk-sk) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA386552 = {S3669, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; hr-hr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA388272 = {S3670, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; uk-ua) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA388500 = {S3671, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-ca) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA390615 = {S3672, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; bg-bg) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA391627 = {S3673, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; he-il) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA393175 = {S3674, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; da-dk) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA393323 = {S3675, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA393447 = {S3676, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; mn-mong) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA395602 = {S3677, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ur-pk) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA396118 = {S3678, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; mk-mk) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA396146 = {S3679, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fo-fo) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA396160 = {S3680, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; mn-mong) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA399446 = {S3681, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; de-de) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA400825 = {S3682, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA400831 = {S3683, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl-be) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA401035 = {S3684, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ta) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA408513 = {S3685, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; af-za) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA412693 = {S3686, 118, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA413509 = {S3687, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ko-kr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA413541 = {S3688, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-ca) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA413864 = {S3689, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; sk-sk) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA413892 = {S3690, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fr-ca) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA414080 = {S3691, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; it-it) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA419721 = {S3692, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; el-gr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA420176 = {S3693, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; eu) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA420370 = {S3694, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA421223 = {S3695, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_GB) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA430113 = {S3696, 133, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; Flyer_P512; en-hk) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA432863 = {S3697, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ro-ro) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA433967 = {S3698, 126, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA434013 = {S3699, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; hu-hu) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA435097 = {S3700, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; hy-am) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA435120 = {S3701, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; hr-hr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA435561 = {S3702, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ga-ie) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA435584 = {S3703, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; zh-tw) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA439237 = {S3704, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; hy-am) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA440119 = {S3705, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; lv-lv) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA443691 = {S3706, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; lt-lt) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA444393 = {S3707, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; th-th) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA445141 = {S3708, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; id) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA445315 = {S3709, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ms) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA447407 = {S3710, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; de-at) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA449698 = {S3711, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ru-ru) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA451167 = {S3712, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA463946 = {S3713, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; el-gr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA465606 = {S3714, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; nl-nl) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA467188 = {S3715, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ca-es) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA469119 = {S3716, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA470241 = {S3717, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fa-ir) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA470357 = {S3718, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; hu-hu) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA472006 = {S3719, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; id) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA472645 = {S3720, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; is-is) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA473334 = {S3721, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; hr-hr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA476560 = {S3722, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; id) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA477899 = {S3723, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; lv-lv) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA478161 = {S3724, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; lt-lt) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA480685 = {S3725, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; sl-si) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA482629 = {S3726, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; es-es) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA486633 = {S3727, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; sv-se) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA488117 = {S3728, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; de-ch) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA488184 = {S3729, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; he-il) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA488189 = {S3730, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; fr-ca) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA488839 = {S3731, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; zh-cn) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA490395 = {S3732, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; es-xl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA490665 = {S3733, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ca-es) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA491108 = {S3734, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA493109 = {S3735, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ro-ro) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA493531 = {S3736, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; yi) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA501835 = {S3737, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; pt-br) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA504434 = {S3738, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko, Safari/6533.18.5) ADM/722*/
const Device UA508388 = {S3739, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; la) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA508795 = {S3740, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; mk-mk) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA509126 = {S3741, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; lv-lv) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA509348 = {S3742, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-ch) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA510823 = {S3743, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; sk-sk) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA512040 = {S3744, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ja-jp) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA516019 = {S3745, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-au) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA516498 = {S3746, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ru-ru) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA523992 = {S3747, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; he-il) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA525015 = {S3748, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ga-ie) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA526972 = {S3749, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ro-ro) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA529529 = {S3750, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; lv-lv) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA529709 = {S3751, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fr-ch) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA530580 = {S3752, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ca-es) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA533448 = {S3753, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA533734 = {S3754, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pt-br) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA536720 = {S3755, 126, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; la) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA544560 = {S3756, 126, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ms) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA545130 = {S3757, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; yi) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA546464 = {S3758, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; cs-cz) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA547039 = {S3759, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; pt-br) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA547431 = {S3760, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; pt-pt) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA548081 = {S3761, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; it-it) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA548506 = {S3762, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; id) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA552387 = {S3763, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA552841 = {S3764, 130, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.21.1*/
const Device UA554639 = {S3765, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ja-jp) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA557326 = {S3766, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA557534 = {S3767, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sw) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA557929 = {S3768, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ms) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA557962 = {S3769, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA558001 = {S3770, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA558023 = {S3771, 130, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.21.1*/
const Device UA558333 = {S3772, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sr-cs) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA558684 = {S3773, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; so) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA558710 = {S3774, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ps) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA558995 = {S3775, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; iu-ca) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA559719 = {S3776, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-au) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA560138 = {S3777, 130, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.21.1*/
const Device UA561333 = {S3778, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; hi-in) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA561397 = {S3779, 117, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/5.0.6 Safari/533.22.3*/
const Device UA565968 = {S3780, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; bo) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA571191 = {S3781, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl_NL) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA582619 = {S3782, 139, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; Sprint APX515CKT; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA623367 = {S3783, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-gb) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA623779 = {S3784, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; he-il) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA625037 = {S3785, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; hr-hr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA629502 = {S3786, 130, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; TB07FTA; pt-br) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA630472 = {S3787, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; af-za) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA630831 = {S3788, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; af-za) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA633148 = {S3789, 113, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA633444 = {S3790, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_ca) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA634021 = {S3791, 126, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.16 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.16*/
const Device UA634847 = {S3792, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; zh-tw) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA635596 = {S3793, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; th-th) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA639727 = {S3794, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; af-za) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA639736 = {S3795, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; it-it) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA640296 = {S3796, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA641473 = {S3797, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; es-es) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA642884 = {S3798, 117, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA643836 = {S3799, 117, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA644991 = {S3800, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; th-th) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA647749 = {S3801, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; zh-cn) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA653559 = {S3802, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; uk-ua) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA654901 = {S3803, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; sv-se) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA656407 = {S3804, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; et-ee) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA656882 = {S3805, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA656997 = {S3806, 117, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA657056 = {S3807, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; he-il) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA657266 = {S3808, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; uk-ua) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA660880 = {S3809, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ca-es) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA662371 = {S3810, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de_DE) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA664357 = {S3811, 130, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; TB07STA; pt-br) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA667123 = {S3812, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; sv-se) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA667756 = {S3813, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_PH) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA669668 = {S3814, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; sv-se) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA669807 = {S3815, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; es-xl) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA670640 = {S3816, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; is-is) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA671193 = {S3817, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; it-it) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA674267 = {S3818, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; pt-br) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA675289 = {S3819, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pl_PL) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA677394 = {S3820, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fr-fr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA678613 = {S3821, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; hu-hu) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA680699 = {S3822, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh_cn) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA682671 = {S3823, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ps) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA683563 = {S3824, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nb-no) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA684081 = {S3825, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; af-za) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA685595 = {S3826, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_AU) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA687237 = {S3827, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; he_il) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA688527 = {S3828, 118, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA690811 = {S3829, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fi_fi) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA691068 = {S3830, 133, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; bq DaVinci; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA691987 = {S3831, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; hu-hu) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA694947 = {S3832, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fr-ca) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA699283 = {S3833, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sq) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA700123 = {S3834, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ar) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA700366 = {S3835, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es_es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA700896 = {S3836, 117, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA701535 = {S3837, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ro-ro) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA702199 = {S3838, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; bg-bg) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA703907 = {S3839, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sv_se) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA704903 = {S3840, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_SG) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA706890 = {S3841, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_za) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA710134 = {S3842, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ja-jp) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA710789 = {S3843, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-au) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA711539 = {S3844, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_NZ) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA714374 = {S3845, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nb-no) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA716354 = {S3846, 128, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA717234 = {S3847, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it_it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA717333 = {S3848, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; sk-sk) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA720154 = {S3849, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; hr-hr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA721607 = {S3850, 130, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.18.5*/
const Device UA723057 = {S3851, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fi-fi) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA723761 = {S3852, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pt_BR) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA726623 = {S3853, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nl-nl) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA729001 = {S3854, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_IE) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA729301 = {S3855, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; sr-cs) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA729809 = {S3856, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; fa-ir) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA730099 = {S3857, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; th-th) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA731014 = {S3858, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-gb) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA731739 = {S3859, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA732909 = {S3860, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; he-il) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA735419 = {S3861, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; de-de) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA742131 = {S3862, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.5+ (KHTML, like Gecko) Version/5.0.5 Safari/533.5+*/
const Device UA745193 = {S3863, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; da_DK) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA750018 = {S3864, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ka-ge) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA753592 = {S3865, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ru_RU) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA754248 = {S3866, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; th-th) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA757136 = {S3867, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; et-ee) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA760277 = {S3868, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_HK) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA761135 = {S3869, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA762773 = {S3870, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh_HK) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA763164 = {S3871, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; bg-bg) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA763498 = {S3872, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; fr-fr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA765649 = {S3873, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; sv-se) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA765653 = {S3874, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; cs-cz) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA765753 = {S3875, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ga-ie) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA766042 = {S3876, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; sl-si) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA766455 = {S3877, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; et-ee) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA767294 = {S3878, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_IN) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA769006 = {S3879, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA773514 = {S3880, 128, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA775518 = {S3881, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; bg-bg) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA775680 = {S3882, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; et-ee) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA776279 = {S3883, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; sl-si) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA779052 = {S3884, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; de-ch) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA788048 = {S3885, 133, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ST070I00GB; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA790039 = {S3886, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; da-dk) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA792343 = {S3887, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; uk-ua) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA792440 = {S3888, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr_FR) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA796754 = {S3889, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; cy) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA796986 = {S3890, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-gb) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA797255 = {S3891, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA798062 = {S3892, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; cs-cz) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA811301 = {S3893, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; az-latn) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA814739 = {S3894, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ar) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA814930 = {S3895, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; zh-tw) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA816601 = {S3896, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nb-no) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA819306 = {S3897, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; uk-ua) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA826715 = {S3898, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; fr-ca) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA830612 = {S3899, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; it-it) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA832936 = {S3900, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ru-ru) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA837792 = {S3901, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; nl-nl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA842292 = {S3902, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ru-ru) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA863840 = {S3903, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; zh-cn) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA872323 = {S3904, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; af-za) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA874868 = {S3905, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ko_kr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA882245 = {S3906, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; tr-tr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA883024 = {S3907, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ug) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA894130 = {S3908, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; th_TH) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA896819 = {S3909, 120, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1*/
const Device UA896824 = {S3910, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ar) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA899291 = {S3911, 126, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; I; Intel Mac OS X 10_6_7; ru-ru) AppleWebKit/534.31+ (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA903066 = {S3912, 125, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_gb) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.5 Safari/6534.46*/
const Device UA903727 = {S3913, 125, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_ca) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.5 Safari/6534.46*/
const Device UA903905 = {S3914, 125, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_us) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.5 Safari/6534.46*/
const Device UA904207 = {S3915, 125, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl_nl) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.5 Safari/6534.46*/
const Device UA904747 = {S3916, 125, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_au) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.5 Safari/6534.46*/
const Device UA905958 = {S3917, 125, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de_de) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.5 Safari/6534.46*/
const Device UA911743 = {S3918, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA911749 = {S3919, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_au) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA911815 = {S3920, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_gb) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA912322 = {S3921, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl_nl) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA924175 = {S3922, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_ca) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA1836733 = {S3923, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; lt-lt) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA1837291 = {S3924, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; hr-hr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA1837770 = {S3925, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; vi-vn) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA1838387 = {S3926, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; el-gr) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA1839541 = {S3927, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; it-it) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA1841358 = {S3928, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fa-ir) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA1852566 = {S3929, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; et-ee) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA1852702 = {S3930, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-ca) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA1857620 = {S3931, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; cs-cz) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA1858683 = {S3932, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; de-at) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA1859541 = {S3933, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; tl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA1860957 = {S3934, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; yi) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA1861816 = {S3935, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; hu-hu) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA1862383 = {S3936, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fa-ir) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA1862391 = {S3937, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; nn-no) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA1863835 = {S3938, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA1866019 = {S3939, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; is-is) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA1866470 = {S3940, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; es-xl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA1866579 = {S3941, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ca-es) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA1866595 = {S3942, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-ca) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA1867663 = {S3943, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; nb-no) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA1867870 = {S3944, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; vi-vn) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA1868114 = {S3945, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; zh-cn) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA1868654 = {S3946, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; sl-si) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA1869515 = {S3947, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; sk-sk) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA1871992 = {S3948, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.5+ (KHTML, like Gecko) Version/5.0.4 Safari/533.5+*/
const Device UA1873732 = {S3949, 126, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; id) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA1874109 = {S3950, 126, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; ar) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA1875288 = {S3951, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-gb) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA1876918 = {S3952, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; cy) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA1881574 = {S3953, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ms) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA1885937 = {S3954, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; nb-no) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA1888460 = {S3955, 130, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.21.1*/
const Device UA1889957 = {S3956, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; km) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA1892934 = {S3957, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ne-np) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA1893287 = {S3958, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA1893811 = {S3959, 130, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh-cn) AppleWebKit/533.21.1 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.21.1*/
const Device UA1897772 = {S3960, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; kk) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA1904963 = {S3961, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA1918716 = {S3962, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es_MX) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA1923527 = {S3963, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; tr_TR) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA1924502 = {S3964, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ja_JP) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA1957139 = {S3965, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_sg) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA1960300 = {S3966, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nb_no) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA1961197 = {S3967, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_us) AppleWebKit/6533.17.9 (KHTML, like Gecko) Version/5.0.5 Safari/6533.17.9*/
const Device UA1961423 = {S3968, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ru_ru) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA1964835 = {S3969, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.5 Safari/533.17.9*/
const Device UA1964958 = {S3970, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh_cn) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA1965152 = {S3971, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr_fr) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA1966727 = {S3972, 134, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ADR6400L 4G; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA1967098 = {S3973, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it_it) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA1971992 = {S3974, 132, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; YPY_07FTA; pt-br) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA1986675 = {S3975, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de_de) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA1986971 = {S3976, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_za) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA1994731 = {S3977, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sv_se) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2039807 = {S3978, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2125848 = {S3979, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; vi-vn) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2125967 = {S3980, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; sl-si) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA2125984 = {S3981, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA2126033 = {S3982, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2126571 = {S3983, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; is-is) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2126724 = {S3984, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fi-fi) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2127161 = {S3985, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; zh-cn) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2127175 = {S3986, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ru-ru) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2130579 = {S3987, 136, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/6533.19.4 (KHTML, like Gecko) Version/5.0 Safari/6533.19.4 FOH:R032*/
const Device UA2130848 = {S3988, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ur-pk) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA2136754 = {S3989, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pl-pl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2139614 = {S3990, 128, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA2140927 = {S3991, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; mn-mong) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2157511 = {S3992, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es_es) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2157959 = {S3993, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_ph) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2158590 = {S3994, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; he_il) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2159347 = {S3995, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; da_dk) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2159890 = {S3996, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_nz) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2162087 = {S3997, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; id_id) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2163092 = {S3998, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fi_fi) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2163349 = {S3999, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; tr_tr) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2165023 = {S4000, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_in) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2165040 = {S4001, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh_hk) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2166766 = {S4002, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; th_th) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2166854 = {S4003, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pl_pl) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2167157 = {S4004, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it_it) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2167415 = {S4005, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_hk) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2167925 = {S4006, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_gb) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2168075 = {S4007, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_ca) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2169366 = {S4008, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de_ch) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2200828 = {S4009, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de_de) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2206134 = {S4010, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_au) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2207151 = {S4011, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; tl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2207690 = {S4012, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; et-ee) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2207696 = {S4013, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; hr-hr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2208115 = {S4014, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; tl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA2209866 = {S4015, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA2209992 = {S4016, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; cs-cz) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA2212681 = {S4017, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ru-ru) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA2212694 = {S4018, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ay) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2214118 = {S4019, 130, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) WebClip/6530.10 Safari/6533.21.1*/
const Device UA2218670 = {S4020, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ja_jp) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2223276 = {S4021, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pt_br) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2229842 = {S4022, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl_nl) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2232946 = {S4023, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_in) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2258669 = {S4024, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es_es) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2258735 = {S4025, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; el_GR) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2258781 = {S4026, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sv_se) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2265869 = {S4027, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; de-ch) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA2268918 = {S4028, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-au) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2269968 = {S4029, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; lt-lt) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA2274465 = {S4030, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; sq) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA2276864 = {S4031, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ro-ro) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA2279981 = {S4032, 126, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; tl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA2282968 = {S4033, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; zh-cn) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2283500 = {S4034, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; az-latn) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2283949 = {S4035, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ug) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2284013 = {S4036, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; iu-ca) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2284686 = {S4037, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-ca) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2290222 = {S4038, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sv-se) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2291154 = {S4039, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; gl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2291816 = {S4040, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; pt-pt) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2293609 = {S4041, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; de-ch) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2298827 = {S4042, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de_CH) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2300068 = {S4043, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh_TW) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2301290 = {S4044, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; sl-si) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2302673 = {S4045, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-au) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA2309921 = {S4046, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ar_SA) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2328491 = {S4047, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de_at) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2329509 = {S4048, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_ie) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2336913 = {S4049, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ar_sa) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2347218 = {S4050, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_nz) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2350031 = {S4051, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ru_ru) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2362161 = {S4052, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; he_il) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2381714 = {S4053, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fi_fi) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2382148 = {S4054, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pl_pl) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2382434 = {S4055, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_sg) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2382480 = {S4056, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; da_dk) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2382578 = {S4057, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_za) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2382602 = {S4058, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr_fr) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2393658 = {S4059, 111, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5 Safari/533.16*/
const Device UA2410856 = {S4060, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ms_my) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2415614 = {S4061, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh_cn) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2418066 = {S4062, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_ph) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2418485 = {S4063, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nb_no) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2418706 = {S4064, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_hk) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2419314 = {S4065, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pt_br) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2419789 = {S4066, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de_ch) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2441994 = {S4067, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh_tw) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2442335 = {S4068, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_gb) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.5 Safari/533.17.9*/
const Device UA2453961 = {S4069, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_ie) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2454616 = {S4070, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; tr_tr) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2465649 = {S4071, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; th_th) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2465864 = {S4072, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh_hk) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2472145 = {S4073, 121, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; da-dk) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2475867 = {S4074, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10\_6\_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2500429 = {S4075, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de_at) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2518366 = {S4076, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh_cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.5 Safari/533.17.9*/
const Device UA2526002 = {S4077, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ja_jp) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2548933 = {S4078, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pt_pt) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2573977 = {S4079, 128, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (AMacintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2576334 = {S4080, 132, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; YPY_07STA; pt-br) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2577478 = {S4081, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr_ca) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2577616 = {S4082, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; vi_vn) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2589712 = {S4083, 125, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_us) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.5 Safari/6536.26*/
const Device UA2590584 = {S4084, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es_mx) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2594214 = {S4085, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; cs_cz) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2638826 = {S4086, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2644269 = {S4087, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; vi_vn) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2652209 = {S4088, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; id_id) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2652572 = {S4089, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es_mx) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2673257 = {S4090, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1068; en-US) AppleWebKit/533.21.1 (KHTML like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2673262 = {S4091, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1065; en-US) AppleWebKit/533.18.1 (KHTML like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2674206 = {S4092, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1064; en-US) AppleWebKit/533.17.8 (KHTML like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA2674431 = {S4093, 126, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1067; en-US) AppleWebKit/533.20.25 (KHTML like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA2675258 = {S4094, 118, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1063; en-US) AppleWebKit/533.16 (KHTML like Gecko) Version/5.0 Safari/533.16*/
const Device UA2678791 = {S4095, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1066; haw-US) AppleWebKit/533.20.25 (KHTML like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA2682121 = {S4096, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sv-se) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA2708270 = {S4097, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1064; en-US) AppleWebKit/533.18.1 (KHTML like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2726799 = {S4098, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl_be) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2726803 = {S4099, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ms_my) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2743367 = {S4100, 120, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.5 Safari/534.46*/
const Device UA2775018 = {S4101, 133, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; CA907AAC0G; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2777603 = {S4102, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ko_kr) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2786666 = {S4103, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/531.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/531.18.5*/
const Device UA2790049 = {S4104, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ar_sa) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2801560 = {S4105, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; az-latn) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2803722 = {S4106, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; br) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2804066 = {S4107, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fi-fi) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA2805611 = {S4108, 126, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; de-DE) AppleWebKit/534.16+ (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA2830265 = {S4109, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; hu_hu) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2830325 = {S4110, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; zh_tw) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2838160 = {S4111, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; ro_ro) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2838251 = {S4112, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; el_gr) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2851651 = {S4113, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es_us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2951131 = {S4114, 131, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/534.48 (KHTML, like Gecko) Version/5.0.6 Safari/533.22.3;u=test6*/
const Device UA2977697 = {S4115, 129, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; hi-in) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA2981616 = {S4116, 123, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; hr_hr) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA2981876 = {S4117, 120, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA3042857 = {S4118, 126, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fr-FR) AppleWebKit/534.16+ (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA3042868 = {S4119, 150, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Snapito! 1.0 (CutyCapt) Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; da-dk) AppleWebKit/533.211 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA3069266 = {S4120, 124, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sa) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA3078436 = {S4121, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_us) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/5.0.5 Safari/537.51.1*/
const Device UA3108941 = {S4122, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_gb) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/5.0.5 Safari/537.51.1*/
const Device UA3125335 = {S4123, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it_it) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/5.0.5 Safari/537.51.1*/
const Device UA3128033 = {S4124, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_ca) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/5.0.5 Safari/537.51.1*/
const Device UA3128199 = {S4125, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en_au) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/5.0.5 Safari/537.51.1*/
const Device UA3130054 = {S4126, 127, { &P17595,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de_de) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/5.0.5 Safari/537.51.1*/
const Device UA2754610 = {S4127, 124, { &P15364,&P7133,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36*/
const Device UA2754613 = {S4128, 117, { &P15364,&P7133,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36*/
const Device UA2774360 = {S4129, 104, { &P15364,&P7133,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.1.0.0 Safari/537.36*/
const Device UA2774361 = {S4130, 97, { &P15364,&P7133,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.1.0.0 Safari/537.36*/
const Device UA2873606 = {S4131, 109, { &P15364,&P7133,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2873637 = {S4132, 102, { &P15364,&P7133,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2874343 = {S4133, 119, { &P15364,&P7133,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe)*/
const Device UA2874813 = {S4134, 112, { &P15364,&P7133,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe)*/
const Device UA2875505 = {S4135, 148, { &P15364,&P7133,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2876734 = {S4136, 141, { &P15364,&P7133,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2877025 = {S4137, 104, { &P15364,&P7133,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.2.0.0 Safari/537.36*/
const Device UA2877026 = {S4138, 97, { &P15364,&P7133,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.2.0.0 Safari/537.36*/
const Device UA2901775 = {S4139, 138, { &P15364,&P7133,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/534.7 ZemanaAID/FFFF0096*/
const Device UA2940886 = {S4140, 192, { &P15364,&P7133,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36 AppEngine-Google; (+http://code.google.com/appengine; appid: s~optimizelyedit-hrd)*/
const Device UA3056126 = {S4141, 179, { &P15364,&P7133,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36 AppEngine-Google; (+http://code.google.com/appengine; appid: s~alexX)*/
const Device UA3092239 = {S4142, 177, { &P15364,&P7133,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36 AppEngine-Google; (+http://code.google.com/appengine; appid: s~jiangpeili)*/
const Device UA3093716 = {S4143, 117, { &P15364,&P7133,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.11.1453.94 Safari/537.36 QIPSurf*/
const Device UA3094314 = {S4144, 110, { &P15364,&P7133,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.11.1453.94 Safari/537.36 QIPSurf*/
const Device UA2754081 = {S4145, 127, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/28.0.1500.20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.20 Safari/537.36*/
const Device UA2773262 = {S4146, 127, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/28.0.1500.29) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.29 Safari/537.36*/
const Device UA2784740 = {S4147, 127, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/28.0.1500.37) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.37 Safari/537.36*/
const Device UA2797726 = {S4148, 127, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/28.0.1500.44) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.44 Safari/537.36*/
const Device UA2877275 = {S4149, 127, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/28.0.1500.52) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36*/
const Device UA2888690 = {S4150, 127, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/28.0.1500.63) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36*/
const Device UA2897664 = {S4151, 127, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/28.0.1500.68) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.68 Safari/537.36*/
const Device UA2898331 = {S4152, 127, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/28.0.1500.71) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36*/
const Device UA2900938 = {S4153, 127, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/28.0.1500.70) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.70 Safari/537.36*/
const Device UA2945258 = {S4154, 127, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/28.0.1500.71) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36*/
const Device UA2948433 = {S4155, 127, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/28.0.1500.72) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36*/
const Device UA2964766 = {S4156, 134, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/28.0.1500.71) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36*/
const Device UA2966893 = {S4157, 205, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/28.0.1500.71; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 1.1.4322; InfoPath.1)*/
const Device UA2970602 = {S4158, 127, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/28.0.1500.72) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36*/
const Device UA2975920 = {S4159, 134, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/28.0.1500.72) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36*/
const Device UA3024746 = {S4160, 127, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/28.0.1500.95) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3024756 = {S4161, 172, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/28.0.1500.95; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA3025371 = {S4162, 159, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/28.0.1500.95; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA3025911 = {S4163, 179, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/28.0.1500.95; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA3026271 = {S4164, 127, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/28.0.1500.95) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3026595 = {S4165, 261, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/28.0.1500.95; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; .NET4.0C; .NET4.0E; Zune 4.7)*/
const Device UA3026718 = {S4166, 212, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/28.0.1500.95; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA3026960 = {S4167, 172, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/28.0.1500.95; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA3027085 = {S4168, 220, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; GTB7.5; chromeframe/28.0.1500.95; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA3027087 = {S4169, 160, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/28.0.1500.95; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA3027273 = {S4170, 81, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/28.0.1500.95)*/
const Device UA3028394 = {S4171, 76, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/28.0.1500.95)*/
const Device UA3028408 = {S4172, 206, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/28.0.1500.95; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA3028701 = {S4173, 238, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.5; chromeframe/28.0.1500.95; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA3029372 = {S4174, 134, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/28.0.1500.95) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3029866 = {S4175, 140, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/28.0.1500.95; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)*/
const Device UA3125510 = {S4176, 193, { &P15364,&P5690,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; chromeframe/28.0.1500.71; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8; InfoPath.3; .NET4.0C)*/
const Device UA444820 = {S4177, 78, { &P15364,&P7133,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; de; rv:1.9.2.8) Gecko/X Firefox 3.6.8*/
const Device UA455832 = {S4178, 96, { &P15364,&P7133,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; ru; rv:1.9.2.28) Gecko/X Firefox/3.6.28 sputnik 2.5.2.8*/
const Device UA507148 = {S4179, 104, { &P15364,&P7133,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.13) Gecko/X Firefox/3.6.13 WinNT-A8I 05.01.2011*/
const Device UA599494 = {S4180, 86, { &P15364,&P7133,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; ru; rv:1.9.2) Gecko/X Firefox/3.6	Firefox 3.6*/
const Device UA634509 = {S4181, 86, { &P15364,&P7133,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2) Gecko/X Firefox/3.6 MSIE 7.0*/
const Device UA2375903 = {S4182, 62, { &P15364,&P7133,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:1.9.2.8) Gecko/X Firefox/3.6.8*/
const Device UA2979947 = {S4183, 104, { &P15364,&P7133,&P17558,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; U; Windows NT 6.1; it; rv:1.9.2.8; Gecko/X AskTbADAP/3.9.1.14019 Firefox/3.6.8)*/
const Device UA2193071 = {S4184, 127, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/23.0.1271.17) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.17 Safari/537.11*/
const Device UA2201510 = {S4185, 127, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/23.0.1271.10) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.10 Safari/537.11*/
const Device UA2232852 = {S4186, 127, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/23.0.1271.40) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.40 Safari/537.11*/
const Device UA2236557 = {S4187, 127, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/23.0.1271.52) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.52 Safari/537.11*/
const Device UA2238021 = {S4188, 127, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/23.0.1271.26) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.26 Safari/537.11*/
const Device UA2258846 = {S4189, 127, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/23.0.1271.60) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.60 Safari/537.11*/
const Device UA2357636 = {S4190, 127, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/23.0.1271.64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2357906 = {S4191, 127, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/23.0.1271.64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2367345 = {S4192, 134, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/23.0.1271.64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2387965 = {S4193, 127, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/23.0.1271.91) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.91 Safari/537.11*/
const Device UA2388470 = {S4194, 127, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/23.0.1271.91) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.91 Safari/537.11*/
const Device UA2389904 = {S4195, 127, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/23.0.1271.95) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11*/
const Device UA2390255 = {S4196, 127, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/23.0.1271.95) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11*/
const Device UA2408308 = {S4197, 127, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/23.0.1271.97) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2408323 = {S4198, 159, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/23.0.1271.97; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2408332 = {S4199, 172, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/23.0.1271.97; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2408366 = {S4200, 81, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/23.0.1271.97)*/
const Device UA2427868 = {S4201, 199, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/23.0.1271.97; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2432300 = {S4202, 159, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/23.0.1271.97; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2432371 = {S4203, 220, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/23.0.1271.97; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2432697 = {S4204, 206, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/23.0.1271.97; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2432712 = {S4205, 172, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/23.0.1271.97; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2433217 = {S4206, 140, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/23.0.1271.97; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)*/
const Device UA2433747 = {S4207, 180, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/23.0.1271.97; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA2433760 = {S4208, 134, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/23.0.1271.97) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2433767 = {S4209, 210, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; chromeframe/23.0.1271.97; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8; .NET4.0C; .NET4.0E)*/
const Device UA2433879 = {S4210, 127, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/23.0.1271.97) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2434797 = {S4211, 230, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/23.0.1271.97; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2436318 = {S4212, 76, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/23.0.1271.97)*/
const Device UA2458252 = {S4213, 134, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/23.0.1271.95) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11*/
const Device UA2458633 = {S4214, 238, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/23.0.1271.97; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2458765 = {S4215, 182, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/23.0.1271.97; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)*/
const Device UA2459253 = {S4216, 270, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/23.0.1271.97; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2483304 = {S4217, 120, { &P15364,&P5690,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; SV1; chromeframe/23.0.1271.95; .NET CLR 1.1.4322; .NET CLR 2.0.50727)*/
const Device UA2864706 = {S4218, 106, { &P15364,&P7133,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML like Gecko) Chrome/28.0.1469.0 Safari/537.36*/
const Device UA2973133 = {S4219, 191, { &P15364,&P7133,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36 AppEngine-Google; (+http://code.google.com/appengine; appid: s~optimizelyedit-hrd)*/
const Device UA3024738 = {S4220, 108, { &P15364,&P7133,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3024740 = {S4221, 101, { &P15364,&P7133,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3026215 = {S4222, 111, { &P15364,&P7133,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe)*/
const Device UA3026282 = {S4223, 147, { &P15364,&P7133,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3026557 = {S4224, 118, { &P15364,&P7133,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe)*/
const Device UA3027645 = {S4225, 119, { &P15364,&P7133,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36; campus-TU*/
const Device UA3028067 = {S4226, 104, { &P15364,&P7133,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.1.0.0 Safari/537.36*/
const Device UA3028080 = {S4227, 97, { &P15364,&P7133,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.1.0.0 Safari/537.36*/
const Device UA3028312 = {S4228, 137, { &P15364,&P7133,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/534.7 ZemanaAID/FFFF0096*/
const Device UA3028709 = {S4229, 140, { &P15364,&P7133,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3029778 = {S4230, 121, { &P15364,&P7133,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe),gzip(gfe)*/
const Device UA3030403 = {S4231, 128, { &P15364,&P7133,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe),gzip(gfe)*/
const Device UA3033341 = {S4232, 116, { &P15364,&P7133,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; en-US;) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3062244 = {S4233, 108, { &P15364,&P7133,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0+(Windows+NT+6.1;+WOW64)+AppleWebKit/537.36+(KHTML,+like+Gecko)+Chrome/28.0.1500.95+Safari/537.36*/
const Device UA3126698 = {S4234, 182, { &P15364,&P7133,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36 AppEngine-Google; (+http://code.google.com/appengine; appid: s~zyfqwe123)*/
const Device UA2567774 = {S4235, 104, { &P15364,&P7133,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.2.0.0 Safari/537.22*/
const Device UA2567781 = {S4236, 97, { &P15364,&P7133,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.2.0.0 Safari/537.22*/
const Device UA2567790 = {S4237, 109, { &P15364,&P7133,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2567792 = {S4238, 102, { &P15364,&P7133,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2567974 = {S4239, 112, { &P15364,&P7133,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)*/
const Device UA2567985 = {S4240, 119, { &P15364,&P7133,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)*/
const Device UA2568530 = {S4241, 148, { &P15364,&P7133,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2569395 = {S4242, 141, { &P15364,&P7133,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2569742 = {S4243, 104, { &P15364,&P7133,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.1.0.0 Safari/537.22*/
const Device UA2569752 = {S4244, 97, { &P15364,&P7133,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.1.0.0 Safari/537.22*/
const Device UA2584211 = {S4245, 144, { &P15364,&P7133,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2606473 = {S4246, 138, { &P15364,&P7133,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/534.7 ZemanaAID/FFFF0096*/
const Device UA2674496 = {S4247, 108, { &P15364,&P7133,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2700858 = {S4248, 100, { &P15364,&P7133,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML like Gecko) Chrome/25.0.1364.97 Safari/537.22*/
const Device UA2708549 = {S4249, 186, { &P15364,&P7133,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22 AppEngine-Google; (+http://code.google.com/appengine; appid: s~stalkthisapp)*/
const Device UA3020400 = {S4250, 139, { &P15364,&P7133,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.7.1364.18260 (gnctrkcll) Safari/537.22*/
const Device UA3024014 = {S4251, 147, { &P15364,&P7133,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.7.1364.17262 Safari/537.22,gzip(gfe),gzip(gfe)*/
const Device UA3025937 = {S4252, 171, { &P15364,&P7133,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22 BitdefenderSafepay/17.14.0.0 (OBK; 25.0.1364.172; r1194.2013)*/
const Device UA3058318 = {S4253, 164, { &P15364,&P7133,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22 BitdefenderSafepay/17.14.0.0 (OBK; 25.0.1364.172; r1194.2013)*/
const Device UA3093214 = {S4254, 146, { &P15364,&P7133,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.7.1364.21027 (gnctrkcll) Safari/537.22*/
const Device UA908806 = {S4255, 246, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 6_0 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0;FBSS/1; FBCR/unavailable;FBID/tablet;FBLC/en_US;FBSF/1.0]*/
const Device UA908818 = {S4256, 99, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.13 (KHTML, like Gecko) Mobile/10A5338d*/
const Device UA913377 = {S4257, 99, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A5355d*/
const Device UA1993995 = {S4258, 83, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2012819 = {S4259, 99, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A5376e*/
const Device UA2041264 = {S4260, 97, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403*/
const Device UA2042494 = {S4261, 214, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]*/
const Device UA2200632 = {S4262, 122, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 iPad2,1/K93AP Zite/1.3.3*/
const Device UA2204690 = {S4263, 112, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/8536.25*/
const Device UA2205569 = {S4264, 165, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48 Instapaper/4.2.6 (+http://www.instapaper.com/iphone)*/
const Device UA2232488 = {S4265, 129, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad3G; iPad; U; CPU OS 6_0 like Mac OS X; en_US) com.google.GooglePlus/10352 (KHTML, like Gecko) Mobile/J1AP (gzip)*/
const Device UA2235160 = {S4266, 129, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad2; iPad; U; CPU OS 6_0 like Mac OS X; en_US) com.google.GooglePlus/10352 (KHTML, like Gecko) Mobile/K93AP (gzip)*/
const Device UA2260576 = {S4267, 97, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A406*/
const Device UA2260579 = {S4268, 85, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2260586 = {S4269, 127, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/8536.26.7 DiigoBrowser*/
const Device UA2260589 = {S4270, 99, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523*/
const Device UA2260648 = {S4271, 97, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A407*/
const Device UA2260647 = {S4272, 157, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser*/
const Device UA2260703 = {S4273, 250, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 6_0_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0.1;FBSS/1; FBCR/unavailable;FBID/tablet;FBLC/en_US;FBSF/1.0]*/
const Device UA2350788 = {S4274, 111, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - MQQBrowser/10 Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403*/
const Device UA2356773 = {S4275, 129, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/8536.26.7 DiigoBrowser*/
const Device UA2357092 = {S4276, 159, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser*/
const Device UA2361095 = {S4277, 100, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8426*/
const Device UA2368835 = {S4278, 113, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 YaMobile/1.00*/
const Device UA2383755 = {S4279, 54, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Educreations 1.2.3 rv:345 (iPad; iPhone OS 6.0; en_US)*/
const Device UA2386551 = {S4280, 98, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176*/
const Device UA2386600 = {S4281, 114, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/8536.25*/
const Device UA2403801 = {S4282, 113, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/528.16*/
const Device UA2403838 = {S4283, 189, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser*/
const Device UA2404168 = {S4284, 132, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A407 iPad3,4__weibo__3.1.1__ipad__os6.0*/
const Device UA2406775 = {S4285, 165, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48 Instapaper/4.2.7 (+http://www.instapaper.com/iphone)*/
const Device UA2408191 = {S4286, 122, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 iPad2,1/K93AP Zite/2.0*/
const Device UA2421372 = {S4287, 96, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176*/
const Device UA2421640 = {S4288, 116, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3*/
const Device UA2421769 = {S4289, 114, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3*/
const Device UA2422471 = {S4290, 55, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - 360 Browser 1.0.0 rv:1.0 (iPad; iPhone OS 6.0.1; en_US)*/
const Device UA2424875 = {S4291, 124, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8426 iPad3,5/P102AP Zite/2.0*/
const Device UA2427302 = {S4292, 85, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2427896 = {S4293, 123, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550 iPad2,5/P105AP Zite/2.0*/
const Device UA2428850 = {S4294, 218, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0.1;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]*/
const Device UA2428926 = {S4295, 214, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A407 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPad3,4;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0;FBSS/2; FBCR/;FBID/tablet;FBLC/en_US]*/
const Device UA2429490 = {S4296, 214, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A406 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPad2,5;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]*/
const Device UA2429983 = {S4297, 223, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8426 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPad3,5;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0.1;FBSS/2; FBCR/AT&T;FBID/tablet;FBLC/en_US]*/
const Device UA2430515 = {S4298, 223, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8500 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPad2,6;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0.2;FBSS/1; FBCR/AT&T;FBID/tablet;FBLC/en_US]*/
const Device UA2437571 = {S4299, 99, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550*/
const Device UA2437597 = {S4300, 218, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPad2,5;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0.2;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]*/
const Device UA2437601 = {S4301, 100, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8500*/
const Device UA2456680 = {S4302, 113, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - MQQBrowser/10 Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523*/
const Device UA2469003 = {S4303, 124, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8500 iPad2,7/P107AP Zite/2.0*/
const Device UA2469339 = {S4304, 133, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad2,1; iPad; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/K93AP (gzip)*/
const Device UA2470552 = {S4305, 132, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad3,1; iPad; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/J1AP (gzip)*/
const Device UA2485434 = {S4306, 134, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad2,5; iPad; U; CPU OS 6_0_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/P105AP (gzip)*/
const Device UA2487399 = {S4307, 134, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad3,4; iPad; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/P101AP (gzip)*/
const Device UA2506746 = {S4308, 165, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/10A550 Safari/7534.48 Instapaper/4.2.7 (+http://www.instapaper.com/iphone)*/
const Device UA2507299 = {S4309, 250, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 6_0_2 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,5;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0.2;FBSS/1; FBCR/unavailable;FBID/tablet;FBLC/en_US;FBSF/1.0]*/
const Device UA2529372 = {S4310, 44, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Miu Ptt 1.9.2 (iPad; iPhone OS 6.0.1; zh_TW)*/
const Device UA2556646 = {S4311, 98, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176*/
const Device UA2580547 = {S4312, 100, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26.17 (KHTML, like Gecko) Mobile/10A403*/
const Device UA2595698 = {S4313, 130, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8426 Safari/8536.26.7 DiigoBrowser*/
const Device UA2669891 = {S4314, 114, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8426 YaMobile/1.11*/
const Device UA2694808 = {S4315, 100, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8425*/
const Device UA2700712 = {S4316, 133, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad2,2; iPad; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/K94AP (gzip)*/
const Device UA2727554 = {S4317, 100, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8424*/
const Device UA2763734 = {S4318, 134, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad2,4; iPad; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/K93aAP (gzip)*/
const Device UA2764160 = {S4319, 114, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8500 YaMobile/1.01*/
const Device UA2826817 = {S4320, 111, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/528.16*/
const Device UA2840731 = {S4321, 132, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A406 iPad2,5__weibo__3.1.1__ipad__os6.0*/
const Device UA2841655 = {S4322, 219, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser*/
const Device UA2861578 = {S4323, 111, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 YaMobile/1.11*/
const Device UA2878159 = {S4324, 113, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/525.20*/
const Device UA3086234 = {S4325, 129, { &P15767,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550 Safari/8536.26.7 DiigoBrowser*/
const Device UA925687 = {S4326, 61, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-N7100 Build/JRO03C)*/
const Device UA925688 = {S4327, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2202472 = {S4328, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L900 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2203429 = {S4329, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T889 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2205398 = {S4330, 140, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; SHV-E250S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2206013 = {S4331, 147, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I317 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2206160 = {S4332, 159, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; SAMSUNG GT-N7100/N7100XXALIJ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2229547 = {S4333, 140, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; SHV-E250L Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2229783 = {S4334, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7105 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2230514 = {S4335, 140, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; SHV-E250K Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2230691 = {S4336, 140, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-I317M Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2230935 = {S4337, 149, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-R950 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 USCC-R950*/
const Device UA2233268 = {S4338, 158, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GT-N7100-ORANGE/N7100XXALIJ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2237133 = {S4339, 159, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; SAMSUNG GT-N7100/N7100XXALIE Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257645 = {S4340, 140, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-T889V Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257984 = {S4341, 130, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7100 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2354065 = {S4342, 158, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GT-N7100-ORANGE/N7100XXALID Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2355428 = {S4343, 61, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-N7105 Build/JRO03C)*/
const Device UA2355924 = {S4344, 61, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SPH-L900 Build/JRO03C)*/
const Device UA2356216 = {S4345, 61, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SGH-T889 Build/JRO03C)*/
const Device UA2356768 = {S4346, 159, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; SAMSUNG GT-N7105/N7105XXALIJ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2360748 = {S4347, 69, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SAMSUNG-SGH-I317 Build/JRO03C)*/
const Device UA2367109 = {S4348, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2367666 = {S4349, 137, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; SC-02E Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2383714 = {S4350, 62, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SHV-E250S Build/JRO03C)*/
const Device UA2384381 = {S4351, 137, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2399522 = {S4352, 140, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; GT-N7105T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2403618 = {S4353, 61, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SCH-R950 Build/JRO03C)*/
const Device UA2403665 = {S4354, 62, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SGH-I317M Build/JRO03C)*/
const Device UA2404053 = {S4355, 61, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N7100 Build/JZO54K)*/
const Device UA2404775 = {S4356, 147, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423862 = {S4357, 140, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E250L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2424137 = {S4358, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L900 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2424494 = {S4359, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; SCH-N719 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2424969 = {S4360, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7105 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426188 = {S4361, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T889 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426223 = {S4362, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2427432 = {S4363, 69, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K)*/
const Device UA2428447 = {S4364, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I317 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2428785 = {S4365, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T889 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2431875 = {S4366, 159, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-N7105/N7105XXDLL4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2432847 = {S4367, 159, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-N7100/N7100XXDLL4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2434738 = {S4368, 140, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E250K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2437682 = {S4369, 140, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SHV-E250S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2451726 = {S4370, 62, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SHV-E250K Build/JRO03C)*/
const Device UA2453304 = {S4371, 62, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SHV-E250L Build/JRO03C)*/
const Device UA2456390 = {S4372, 159, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-ch; SAMSUNG GT-N7105/N7105XXALJ5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2458187 = {S4373, 61, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N7105 Build/JZO54K)*/
const Device UA2459780 = {S4374, 140, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-nz; GT-N7100T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467206 = {S4375, 130, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; SCH-N719 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2467700 = {S4376, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467997 = {S4377, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I317 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2468317 = {S4378, 62, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E250S Build/JZO54K)*/
const Device UA2482892 = {S4379, 130, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7100 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2482976 = {S4380, 140, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I317M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483582 = {S4381, 62, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-N7105T Build/JRO03C)*/
const Device UA2483899 = {S4382, 144, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1 ALK8; en-us; SGH-T889 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2505642 = {S4383, 62, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I317M Build/JZO54K)*/
const Device UA2506601 = {S4384, 149, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1 V2.1 ALK8; en-us; SGH-T889 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2508571 = {S4385, 159, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; SAMSUNG GT-N7105/N7105XXDMA5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2514719 = {S4386, 140, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I317M Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2527938 = {S4387, 62, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SGH-T889V Build/JRO03C)*/
const Device UA2528630 = {S4388, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-gb; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2529040 = {S4389, 61, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SPH-L900 Build/JZO54K)*/
const Device UA2529866 = {S4390, 140, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-T889V Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2531481 = {S4391, 140, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-N7105T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2534587 = {S4392, 159, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-N7100/N7100XXDMA5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2539851 = {S4393, 159, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-N7105/N7105XXDMB2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2554386 = {S4394, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; zh-tw; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2554655 = {S4395, 130, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7105 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2555003 = {S4396, 59, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SC-02E Build/JRO03C)*/
const Device UA2555221 = {S4397, 61, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SCH-N719 Build/JRO03C)*/
const Device UA2556235 = {S4398, 144, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2 DLL1; en-us; SGH-T889 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556812 = {S4399, 62, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E250K Build/JZO54K)*/
const Device UA2558037 = {S4400, 149, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R950 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 USCC-R950*/
const Device UA2558874 = {S4401, 136, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2559411 = {S4402, 159, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-N7105/N7105XXDLL7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2561531 = {S4403, 158, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-N7105-ORANGE/N7105XXDMA5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567427 = {S4404, 159, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-N7100/N7100XXDMB2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2571477 = {S4405, 158, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-N7100-ORANGE/N7100XXDMB2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2579547 = {S4406, 61, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T889 Build/JZO54K)*/
const Device UA2583221 = {S4407, 61, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-R950 Build/JZO54K)*/
const Device UA2583548 = {S4408, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-N719 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2592464 = {S4409, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I317 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2594183 = {S4410, 130, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7100 Build JRO03C) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2597415 = {S4411, 62, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E250L Build/JZO54K)*/
const Device UA2597419 = {S4412, 137, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2598650 = {S4413, 137, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.3; en-us; GT-N7100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2600494 = {S4414, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2603555 = {S4415, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; es-es; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2606251 = {S4416, 140, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-nz; GT-N7100T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2653375 = {S4417, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; vi-vn; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2653581 = {S4418, 137, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; SGH-T889 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2656375 = {S4419, 137, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N7100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2657633 = {S4420, 145, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658076 = {S4421, 159, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG GT-N7105/N7105XXDLL1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2668095 = {S4422, 62, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N7105T Build/JZO54K)*/
const Device UA2673890 = {S4423, 147, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E250K/KKVMC8 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2677515 = {S4424, 159, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-N7105/N7105XXDMC3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695025 = {S4425, 150, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2 Jelly Bean; vi-vn; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2698356 = {S4426, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; de-de; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2702530 = {S4427, 158, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-N7105-ORANGE/N7105XXDMB2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2705672 = {S4428, 61, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-N719 Build/JZO54K)*/
const Device UA2707076 = {S4429, 62, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T889V Build/JZO54K)*/
const Device UA2708514 = {S4430, 137, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-US; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2730218 = {S4431, 137, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; N7100+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2731685 = {S4432, 137, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.3; es-es; GT-N7100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2738850 = {S4433, 147, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E250K/KKVME2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747554 = {S4434, 61, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I317 Build/JZO54K)*/
const Device UA2748620 = {S4435, 137, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SPH-L900 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2753853 = {S4436, 159, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-N7100/N7100XXDMC3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768752 = {S4437, 137, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2778795 = {S4438, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; it-it; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2790974 = {S4439, 61, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.9; GT-N7100 Build/IMM76D)*/
const Device UA2793799 = {S4440, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.5; en-us; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2846791 = {S4441, 137, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-N7105 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2849278 = {S4442, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; th-th; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2862898 = {S4443, 137, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SGH-T889 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2869440 = {S4444, 147, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E250K/KKVME9 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2889253 = {S4445, 150, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1 Jelly Bean; vi-vn; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2935907 = {S4446, 142, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/TELL//X/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936676 = {S4447, 149, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; N7100DXALJ1 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2974766 = {S4448, 159, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ie; SAMSUNG GT-N7100/N7100XXDMF3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2976055 = {S4449, 147, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E250K/KKVMF6 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2983184 = {S4450, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; ms-my; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985314 = {S4451, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; fr-fr; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2990723 = {S4452, 130, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-N719 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2992579 = {S4453, 132, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3025456 = {S4454, 159, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; SAMSUNG GT-N7105/N7105XXDMG1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3031639 = {S4455, 158, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-N7100-ORANGE/N7100XXDMF1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3034676 = {S4456, 158, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-N7105-ORANGE/N7105XXDMF2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044046 = {S4457, 139, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; id-id; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3047272 = {S4458, 163, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2 AmnoSperum ROM  Z.7.0.1; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3059576 = {S4459, 158, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N7100 Build/JZO54K) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 T5/2.0*/
const Device UA3062477 = {S4460, 147, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E250K/KKVMH1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3067241 = {S4461, 180, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=t0lteatt;devm=SAMSUNG-SGH-I317;man=samsung;brw=Android;shwd=720x1280;shwa=720x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA3072784 = {S4462, 62, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N7100T Build/JZO54K)*/
const Device UA3107471 = {S4463, 185, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7100 Build/JRO03C) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 T5/2.0 baidubrowser/3.1.6.4 (Baidu; P1 4.1.1)*/
const Device UA3117904 = {S4464, 150, { &P18442,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7100 Build/JRO03C KOUNGY.COM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2191137 = {S4465, 97, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.6 (KHTML like Gecko) Chrome/23.0.1239.0 Safari/537.6*/
const Device UA2201785 = {S4466, 174, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.10 (KHTML, like Gecko) Chrome/23.0.1262.0 Safari/537.10 AppEngine-Google; (+http://code.google.com/appengine; appid: s~my-gate32)*/
const Device UA2387457 = {S4467, 97, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.1.0.0 Safari/537.11*/
const Device UA2387455 = {S4468, 104, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.1.0.0 Safari/537.11*/
const Device UA2406237 = {S4469, 137, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/534.7 ZemanaAID/FFFF0061*/
const Device UA2406378 = {S4470, 108, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2408288 = {S4471, 101, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2408369 = {S4472, 111, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe)*/
const Device UA2408395 = {S4473, 118, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe)*/
const Device UA2408447 = {S4474, 104, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.2.0.0 Safari/537.11*/
const Device UA2427450 = {S4475, 107, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2428543 = {S4476, 104, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.4.0.0 Safari/537.11*/
const Device UA2429892 = {S4477, 97, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.4.0.0 Safari/537.11*/
const Device UA2432590 = {S4478, 147, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2433533 = {S4479, 140, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2436326 = {S4480, 97, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.2.0.0 Safari/537.11*/
const Device UA2437186 = {S4481, 104, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.3.0.0 Safari/537.11*/
const Device UA2437185 = {S4482, 97, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.3.0.0 Safari/537.11*/
const Device UA2457181 = {S4483, 177, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11 AppEngine-Google; (+http://code.google.com/appengine; appid: s~vivillrocku)*/
const Device UA2468450 = {S4484, 185, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11 AppEngine-Google; (+http://code.google.com/appengine; appid: optimizelyedit)*/
const Device UA2540611 = {S4485, 127, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; ; CJPMS_AAP715ACX) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11*/
const Device UA2540630 = {S4486, 136, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; ; NCLIENT50_AAP715AF4E5CBC) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2747438 = {S4487, 69, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2854685 = {S4488, 123, { &P15364,&P7133,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko GomezAgent 3.0) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2352718 = {S4489, 125, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1284.2) AppleWebKit/537.13 (KHTML, like Gecko) Chrome/24.0.1284.2 Safari/537.13*/
const Device UA2358156 = {S4490, 125, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.5) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.5 Safari/537.17*/
const Device UA2363163 = {S4491, 127, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.14) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.14 Safari/537.17*/
const Device UA2390892 = {S4492, 127, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.27) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.27 Safari/537.17*/
const Device UA2390925 = {S4493, 127, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.25) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.25 Safari/537.17*/
const Device UA2408108 = {S4494, 127, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.35) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.35 Safari/537.17*/
const Device UA2408180 = {S4495, 127, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.32) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.32 Safari/537.17*/
const Device UA2430546 = {S4496, 127, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.45) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.45 Safari/537.17*/
const Device UA2435761 = {S4497, 127, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.40) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.40 Safari/537.17*/
const Device UA2461916 = {S4498, 127, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.52) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17*/
const Device UA2462199 = {S4499, 127, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.49) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.49 Safari/537.17*/
const Device UA2470872 = {S4500, 127, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/24.0.1312.52) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17*/
const Device UA2487905 = {S4501, 81, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/24.0.1312.57)*/
const Device UA2487893 = {S4502, 172, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/24.0.1312.57; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2488069 = {S4503, 127, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.56) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17*/
const Device UA2488941 = {S4504, 127, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/24.0.1312.56) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17*/
const Device UA2492204 = {S4505, 159, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/24.0.1312.57; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2492212 = {S4506, 172, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/24.0.1312.57; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2492228 = {S4507, 127, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.57) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492512 = {S4508, 220, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/24.0.1312.57; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2508849 = {S4509, 180, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/24.0.1312.57; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA2509046 = {S4510, 127, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/24.0.1312.57) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2509288 = {S4511, 207, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; GTB7.4; chromeframe/24.0.1312.57; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2509328 = {S4512, 238, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/24.0.1312.57; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2509472 = {S4513, 210, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; chromeframe/24.0.1312.57; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8; .NET4.0C; .NET4.0E)*/
const Device UA2509978 = {S4514, 206, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/24.0.1312.57; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2510313 = {S4515, 140, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/24.0.1312.57; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)*/
const Device UA2510540 = {S4516, 76, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/24.0.1312.57)*/
const Device UA2512010 = {S4517, 134, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/24.0.1312.57) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2532022 = {S4518, 190, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/24.0.1312.57; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)*/
const Device UA2699012 = {S4519, 198, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/24.0.1312.57; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2700573 = {S4520, 127, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/24.0.1312.71) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.71 Safari/537.17*/
const Device UA2844352 = {S4521, 192, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; chromeframe/24.0.1312.27; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA2850376 = {S4522, 134, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/24.0.1312.56) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17*/
const Device UA2850713 = {S4523, 120, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; SV1; chromeframe/24.0.1312.57; .NET CLR 1.1.4322; .NET CLR 2.0.50727)*/
const Device UA2986312 = {S4524, 334, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/24.0.1312.52;  Embedded Web Browser from: http://bsalsa.com/; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.1; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA3120917 = {S4525, 187, { &P15364,&P5690,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; SV1; chromeframe/24.0.1312.71; .NET CLR 2.0.50727; InfoPath.1; .NET4.0C; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2259377 = {S4526, 182, { &P15364,&P7133,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.13 (KHTML, like Gecko) Chrome/24.0.1290.1 Safari/537.13 AppEngine-Google; (+http://code.google.com/appengine; appid: s~flybear008)*/
const Device UA2471951 = {S4527, 104, { &P15364,&P7133,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.1.0.0 Safari/537.17*/
const Device UA2471956 = {S4528, 97, { &P15364,&P7133,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.1.0.0 Safari/537.17*/
const Device UA2487696 = {S4529, 108, { &P15364,&P7133,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2488093 = {S4530, 140, { &P15364,&P7133,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2492110 = {S4531, 104, { &P15364,&P7133,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.2.0.0 Safari/537.17*/
const Device UA2492150 = {S4532, 101, { &P15364,&P7133,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492279 = {S4533, 97, { &P15364,&P7133,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.2.0.0 Safari/537.17*/
const Device UA2492341 = {S4534, 147, { &P15364,&P7133,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2492375 = {S4535, 118, { &P15364,&P7133,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)*/
const Device UA2492462 = {S4536, 111, { &P15364,&P7133,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)*/
const Device UA2508391 = {S4537, 137, { &P15364,&P7133,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/534.7 ZemanaAID/FFFF0096*/
const Device UA2535014 = {S4538, 107, { &P15364,&P7133,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML like Gecko) Chrome/24.0.1312.52 Safari/537.17*/
const Device UA2540554 = {S4539, 129, { &P15364,&P7133,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; ; NCLIENT50_AAP715B4218F57) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2540608 = {S4540, 127, { &P15364,&P7133,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; ; CJPMS_AAP715ACX) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17*/
const Device UA2599747 = {S4541, 100, { &P15364,&P7133,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.17 (KHTML like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2666664 = {S4542, 173, { &P15364,&P7133,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17 AppEngine-Google; (+http://code.google.com/appengine; appid: s~dnnwave)*/
const Device UA2713280 = {S4543, 117, { &P15364,&P7133,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.10.1312.57 Safari/537.17 QIPSurf*/
const Device UA2737939 = {S4544, 110, { &P15364,&P7133,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.10.1312.57 Safari/537.17 QIPSurf*/
const Device UA3064093 = {S4545, 140, { &P15364,&P7133,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3067541 = {S4546, 147, { &P15364,&P7133,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3067630 = {S4547, 108, { &P15364,&P7133,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36*/
const Device UA3067634 = {S4548, 101, { &P15364,&P7133,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36*/
const Device UA3067981 = {S4549, 111, { &P15364,&P7133,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe)*/
const Device UA3068128 = {S4550, 118, { &P15364,&P7133,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe)*/
const Device UA3068229 = {S4551, 121, { &P15364,&P7133,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe),gzip(gfe)*/
const Device UA3068244 = {S4552, 128, { &P15364,&P7133,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe),gzip(gfe)*/
const Device UA3082195 = {S4553, 124, { &P15364,&P7133,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/29.0.1600.1 Chrome/29.0.1600.1 Safari/537.36*/
const Device UA3082208 = {S4554, 117, { &P15364,&P7133,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Iron/29.0.1600.1 Chrome/29.0.1600.1 Safari/537.36*/
const Device UA3094274 = {S4555, 137, { &P15364,&P7133,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/534.7 ZemanaAID/FFFF0096*/
const Device UA3095637 = {S4556, 138, { &P15364,&P7133,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36Accept-Encoding: gzip, deflate*/
const Device UA3114448 = {S4557, 191, { &P15364,&P7133,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36 AppEngine-Google; (+http://code.google.com/appengine; appid: s~optimizelyedit-hrd)*/
const Device UA22295 = {S4558, 140, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA23196 = {S4559, 143, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-I9100M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA23266 = {S4560, 143, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; GT-I9100T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24693 = {S4561, 142, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32890 = {S4562, 143, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46261 = {S4563, 161, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; SAMSUNG GT-I9100/I9100XWKG1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46331 = {S4564, 150, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ar-ae; GT-I9100 Build/DR.AKRIM BLACKDROID) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46333 = {S4565, 161, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SAMSUNG GT-I9100/I9100BUKG2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46345 = {S4566, 160, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-at; GT-I9100-ORANGE/I9100BVKG8 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46399 = {S4567, 160, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; GT-I9100-ORANGE/I9100BVKG2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46409 = {S4568, 160, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; GT-I9100-ORANGE/I9100BVKH1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46412 = {S4569, 141, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46430 = {S4570, 141, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46441 = {S4571, 141, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47888 = {S4572, 160, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100-ORANGE/I9100BVKG3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59749 = {S4573, 142, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59771 = {S4574, 161, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SAMSUNG GT-I9100/I9100BUKE5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59776 = {S4575, 161, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; SAMSUNG GT-I9100/I9100BUKE4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59787 = {S4576, 161, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SAMSUNG GT-I9100/I9100BUKE1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59796 = {S4577, 160, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100-ORANGE/I9100BVKE2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59803 = {S4578, 161, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; SAMSUNG GT-I9100/I9100BUKE2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70081 = {S4579, 141, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-I9100M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71268 = {S4580, 142, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71385 = {S4581, 143, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; he-il; GT-I9100T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72692 = {S4582, 160, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100-ORANGE/I9100BVKH2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73532 = {S4583, 143, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74454 = {S4584, 160, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; pl-pl; GT-I9100-ORANGE/I9100BVKF2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74825 = {S4585, 162, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; SAMSUNG GT-I9100/I9100BUKI3F Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA77107 = {S4586, 136, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; GT-I9100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90531 = {S4587, 136, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; GT-I9100 Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA195277 = {S4588, 66, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-I9100 Build/GINGERBREAD)*/
const Device UA195639 = {S4589, 120, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; de-de; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA205000 = {S4590, 120, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA219902 = {S4591, 66, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-I9100 Build/GINGERBREAD)*/
const Device UA237718 = {S4592, 67, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-I9100T Build/GINGERBREAD)*/
const Device UA244330 = {S4593, 66, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-I9100 Build/GINGERBREAD)*/
const Device UA249016 = {S4594, 136, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-I9100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA252028 = {S4595, 161, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; SAMSUNG GT-I9100/I9100BUKI3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA347576 = {S4596, 143, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; GT-I9100T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA349124 = {S4597, 142, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA349835 = {S4598, 155, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android ANDROIDHOUSE 2.3.6; es-es; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369437 = {S4599, 78, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.4; en_GB; GT-I9100 Build/GINGERBREAD.XWKI4)*/
const Device UA370451 = {S4600, 35, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Android/1.0 (GT-I9100M GINGERBREAD)*/
const Device UA370805 = {S4601, 131, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; GT-I9100T Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA371202 = {S4602, 157, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9100 Build/GINGERBREAD; 480*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA378522 = {S4603, 120, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; nl-nl; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA395125 = {S4604, 67, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-I9100M Build/GINGERBREAD)*/
const Device UA409238 = {S4605, 120, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; fr-fr; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA409489 = {S4606, 120, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; de-ch; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA419815 = {S4607, 120, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-ie; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA436297 = {S4608, 120, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; es-es; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA451495 = {S4609, 120, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA459214 = {S4610, 141, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.4; en-gb; GT-I9100X Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA571694 = {S4611, 121, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - SAMSUNG-GT-I9108_TD/1.0 Android/2.3.3 Release/03.15.2011 Browser/AppleWebKit533.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA622971 = {S4612, 160, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100-ORANGE/I9100BVKJ4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622979 = {S4613, 143, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622977 = {S4614, 161, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; SAMSUNG GT-I9100/I9100BUKJ3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623175 = {S4615, 66, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9100 Build/GINGERBREAD)*/
const Device UA627389 = {S4616, 143, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-I9100T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627453 = {S4617, 67, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9100G Build/GINGERBREAD)*/
const Device UA628654 = {S4618, 160, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-I9100-ORANGE/I9100BVKL1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA634708 = {S4619, 131, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100G Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA636774 = {S4620, 67, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-I9100G Build/GINGERBREAD)*/
const Device UA641571 = {S4621, 130, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9100 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA653622 = {S4622, 60, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; GT-I9100 Build/GRJ22)*/
const Device UA661886 = {S4623, 137, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; GT-I9100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA662027 = {S4624, 136, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-I9100 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA680540 = {S4625, 142, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9108 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA683577 = {S4626, 130, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA683713 = {S4627, 130, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9108 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA705866 = {S4628, 130, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9108 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA708743 = {S4629, 120, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; cs-cz; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA717522 = {S4630, 67, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-I9100T Build/GINGERBREAD)*/
const Device UA733368 = {S4631, 142, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9108 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA737822 = {S4632, 130, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA738497 = {S4633, 130, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA742238 = {S4634, 130, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9108 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA751509 = {S4635, 126, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; I9100/V100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA759969 = {S4636, 142, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9108 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA771171 = {S4637, 120, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; pl-pl; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA773369 = {S4638, 131, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA792347 = {S4639, 120, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; pt-pt; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA809660 = {S4640, 120, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; de-at; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA843772 = {S4641, 120, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; sl-si; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA893337 = {S4642, 161, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.12011-10-16 20:22:55*/
const Device UA907981 = {S4643, 135, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA909474 = {S4644, 136, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1895072 = {S4645, 120, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; fr-ch; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA1914475 = {S4646, 140, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1918696 = {S4647, 138, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; iw-il; GT-I9100 Build/HebMIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1931925 = {S4648, 168, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en_gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)*/
const Device UA1934233 = {S4649, 135, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-I9100 Build/WIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1935156 = {S4650, 141, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-I9100T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1960559 = {S4651, 168, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Android 2.3.6 Gingerbread XWLA4; es-es; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1967129 = {S4652, 135, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9100 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1970192 = {S4653, 134, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; GT-I9100 Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1976969 = {S4654, 135, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1981478 = {S4655, 135, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9108 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1983892 = {S4656, 135, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9100 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1987293 = {S4657, 136, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100G Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2020725 = {S4658, 155, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android ANDROIDHOUSE 2.3.5; es-es; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020808 = {S4659, 159, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; SAMSUNG GT-I9100/I9100BUKG2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021163 = {S4660, 156, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Android 2.3.6 XWKK5; es-es; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021403 = {S4661, 131, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9100G Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2021983 = {S4662, 140, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022148 = {S4663, 136, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100 Build/GINGERBREAD.AX918) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022154 = {S4664, 142, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022486 = {S4665, 140, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022780 = {S4666, 131, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-I9100M Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022863 = {S4667, 67, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-I9100G Build/GINGERBREAD)*/
const Device UA2023289 = {S4668, 148, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9100 Build/GINGERBREAD.AX918) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023606 = {S4669, 160, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android AndroidHouse Team 2.3.6; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024698 = {S4670, 67, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9100T Build/GINGERBREAD)*/
const Device UA2025388 = {S4671, 66, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9108 Build/GINGERBREAD)*/
const Device UA2026657 = {S4672, 144, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-I9100M Build/GINGERBREAD) AppleWebKit/533.1+ (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026856 = {S4673, 136, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9100G Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2027406 = {S4674, 135, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9108 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2031073 = {S4675, 141, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100 Build/GINGERBREAD.AX918) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2032000 = {S4676, 136, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT-I9100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032877 = {S4677, 136, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; GT-I9100T Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2033203 = {S4678, 150, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2033453 = {S4679, 135, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9108 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2141648 = {S4680, 120, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; hr-hr; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2167038 = {S4681, 143, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-S959G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257102 = {S4682, 141, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2290164 = {S4683, 142, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2301197 = {S4684, 120, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; ro-ro; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2310835 = {S4685, 120, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; tr-tr; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2366714 = {S4686, 67, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-S959G Build/GINGERBREAD)*/
const Device UA2408920 = {S4687, 120, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-au; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2473135 = {S4688, 120, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; it-it; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2473949 = {S4689, 135, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2640255 = {S4690, 141, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2642235 = {S4691, 141, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-I9100T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2645383 = {S4692, 136, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; GT-i9100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652391 = {S4693, 150, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA2807223 = {S4694, 142, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.8; zh-tw; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2821067 = {S4695, 150, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2950540 = {S4696, 134, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; I9100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2979345 = {S4697, 136, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3118559 = {S4698, 141, { &P15575,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SGH-S959G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA230943 = {S4699, 55, { &P18088,&P17017,&P4307,&P18092 }};	/*Useragent - Opera/9.50 (J2ME/MIDP; Opera Mini/4.1.11355/574; U; en)*/
const Device UA280359 = {S4700, 73, { &P18088,&P17017,&P4307,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/4.1.13572/20.2485; U; en) Presto/2.5.25*/
const Device UA473978 = {S4701, 68, { &P18088,&P17017,&P4307,&P18092 }};	/*Useragent - Opera/9.60 (J2ME/MIDP; Opera Mini/4.1.11320/608; U; en) Presto/2.2.0*/
const Device UA877558 = {S4702, 86, { &P18088,&P17017,&P4307,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/4.1.13907/25.729; U; fr) Presto/2.5.25 Version/10.54*/
const Device UA2797802 = {S4703, 88, { &P18088,&P17017,&P4307,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/4.1.15082/30.3214; U; en) Presto/2.8.119 Version/11.10*/
const Device UA796281 = {S4704, 83, { &P18436,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Win64; x64; Trident/6.0; Touch)*/
const Device UA862024 = {S4705, 71, { &P18436,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; Touch)*/
const Device UA878818 = {S4706, 78, { &P18436,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Touch)*/
const Device UA2795932 = {S4707, 183, { &P18436,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; HPDTDFJS)*/
const Device UA2993141 = {S4708, 186, { &P18436,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Win64; x64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Tablet PC 2.0; ASU2JS)*/
const Device UA7772 = {S4709, 39, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA20437 = {S4710, 67, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - i-mobilehitz218/2.0 (03.15) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20517 = {S4711, 76, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - NGM_Vanity /ObigoInternetBrowser/QO3C Profile/MIDP2.0 Configuration/CLDC-1.1*/
const Device UA23052 = {S4712, 52, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Jbed-FastBCC Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23151 = {S4713, 66, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - GIONEE-M902/SW1.0.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23438 = {S4714, 80, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Duplii/1.0 TMSS-Browser/1.0.0 Profile/MIDP/2.0-compatible Configuration/CLDC-1.1*/
const Device UA23999 = {S4715, 82, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Beetel-TD590/S108 LMP/LML Release/2010.08.16 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA49175 = {S4716, 57, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - OGO-O78/Obigo WapBrowser 2.0/MIDP2.0Configuration/CLDC1.1*/
const Device UA53462 = {S4717, 60, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - TMN/1.0/G7002/SW1.0.0/WAP2.0/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54445 = {S4718, 52, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - IN2010 Nano2/Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA56928 = {S4719, 77, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - VertuConstellationF/2.0 (08w11500.01) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63813 = {S4720, 64, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - G-Fone777/Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA75512 = {S4721, 95, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Sansui_S44/22.11.2007 Browser/wap2.0 Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA86166 = {S4722, 65, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - W3C-mobileOK/DDC-1.0 (see http://www.w3.org/2006/07/mobileok-ddc)*/
const Device UA87795 = {S4723, 56, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - RT240X320/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA88485 = {S4724, 100, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - OT-S680/1.0 MTK/W07.32 Release/01.01.2007 Browser/Teleca-1.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88672 = {S4725, 100, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - OT-S686/1.0 BRCM/2124 Release/CC84-V1.0.0 Browser/Teleca-1.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA89802 = {S4726, 129, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (PUMA Phone M1/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1; 240x320) AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA90621 = {S4727, 53, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Fly_E146/WapBrowser2.0/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA97138 = {S4728, 60, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - FeedDemon/3.1 (http://www.feeddemon.com/; Microsoft Windows)*/
const Device UA102504 = {S4729, 39, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA140891 = {S4730, 59, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - M760/SW1.0.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA173611 = {S4731, 56, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - RT176X220/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA262000 = {S4732, 56, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - RT128X160/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA271952 = {S4733, 39, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA300081 = {S4734, 58, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - SJT_E861A/wap2.0Profile/MIDP-1.0/2.0Configuration/CLDC-1.1*/
const Device UA373759 = {S4735, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 941SH)*/
const Device UA374549 = {S4736, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 831SH)*/
const Device UA387696 = {S4737, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; SH01B)*/
const Device UA388938 = {S4738, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; F01C)*/
const Device UA390151 = {S4739, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; F06B)*/
const Device UA397437 = {S4740, 21, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - python-requests/0.8.1*/
const Device UA400224 = {S4741, 60, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - FeedDemon/2.7 (http://www.newsgator.com/; Microsoft Windows)*/
const Device UA412425 = {S4742, 22, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - python-requests/0.10.6*/
const Device UA414976 = {S4743, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; SH38)*/
const Device UA415917 = {S4744, 11, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - curl/7.15.5*/
const Device UA417128 = {S4745, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 923SH)*/
const Device UA420678 = {S4746, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 004SH)*/
const Device UA428350 = {S4747, 22, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - python-requests/0.10.8*/
const Device UA430641 = {S4748, 13, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Java/1.5.0_10*/
const Device UA439486 = {S4749, 17, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Amazon CloudFront*/
const Device UA439794 = {S4750, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 940SH)*/
const Device UA439955 = {S4751, 13, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Java/1.4.2_04*/
const Device UA443793 = {S4752, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 920SH)*/
const Device UA489131 = {S4753, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 931SH)*/
const Device UA492891 = {S4754, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 001SH)*/
const Device UA495844 = {S4755, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; P906i)*/
const Device UA497423 = {S4756, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; P06B)*/
const Device UA502200 = {S4757, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; N07B)*/
const Device UA505791 = {S4758, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 840P)*/
const Device UA507146 = {S4759, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; P04B)*/
const Device UA518617 = {S4760, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; SH04A)*/
const Device UA520203 = {S4761, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; P02B)*/
const Device UA520946 = {S4762, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 932SH)*/
const Device UA522210 = {S4763, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; F02B)*/
const Device UA522719 = {S4764, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; SH02A)*/
const Device UA525065 = {S4765, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 933SH)*/
const Device UA531565 = {S4766, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; N04B)*/
const Device UA541106 = {S4767, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 943SH)*/
const Device UA541286 = {S4768, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; N02C)*/
const Device UA541337 = {S4769, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; N08A3)*/
const Device UA541392 = {S4770, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; SH01C)*/
const Device UA543728 = {S4771, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; SH01A)*/
const Device UA547725 = {S4772, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 824SH)*/
const Device UA552335 = {S4773, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 841SH)*/
const Device UA562998 = {S4774, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 002SH)*/
const Device UA563679 = {S4775, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 941P)*/
const Device UA583725 = {S4776, 21, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - python-requests/0.8.6*/
const Device UA588687 = {S4777, 13, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Java/1.8.0-ea*/
const Device UA603172 = {S4778, 22, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - python-requests/0.11.1*/
const Device UA608925 = {S4779, 32, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - siclab (cboc-test@lab.ntt.co.jp)*/
const Device UA613088 = {S4780, 22, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - python-requests/0.11.2*/
const Device UA615807 = {S4781, 22, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - python-requests/0.12.1*/
const Device UA630683 = {S4782, 12, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - PHP/5.2.17p1*/
const Device UA641447 = {S4783, 11, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - curl/7.19.7*/
const Device UA643520 = {S4784, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; SH03B)*/
const Device UA652711 = {S4785, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 944SH)*/
const Device UA656247 = {S4786, 22, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - python-requests/0.10.1*/
const Device UA664179 = {S4787, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; P02A)*/
const Device UA687433 = {S4788, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; SH07B)*/
const Device UA694550 = {S4789, 46, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; N906imyu)*/
const Device UA700113 = {S4790, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; N01B)*/
const Device UA703611 = {S4791, 59, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - curl/7.18.0 (i586-pc-mingw32msvc) libcurl/7.18.0 zlib/1.2.3*/
const Device UA707416 = {S4792, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; F03B)*/
const Device UA708009 = {S4793, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; F01B)*/
const Device UA718582 = {S4794, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; N06B)*/
const Device UA719956 = {S4795, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 820SH)*/
const Device UA727936 = {S4796, 44, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; SH906i)*/
const Device UA730411 = {S4797, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 840SH)*/
const Device UA739705 = {S4798, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 830CA)*/
const Device UA746886 = {S4799, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; P01A)*/
const Device UA752335 = {S4800, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; P10A)*/
const Device UA765433 = {S4801, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; P03C)*/
const Device UA769704 = {S4802, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; F01A)*/
const Device UA774326 = {S4803, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 940SC)*/
const Device UA778747 = {S4804, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; F09A3)*/
const Device UA781553 = {S4805, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 930SH)*/
const Device UA785976 = {S4806, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; SH08B)*/
const Device UA789099 = {S4807, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; SH02B)*/
const Device UA792551 = {S4808, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 832SH)*/
const Device UA796234 = {S4809, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; N02A)*/
const Device UA803913 = {S4810, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 830SH)*/
const Device UA813775 = {S4811, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; F08B)*/
const Device UA815361 = {S4812, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; P905i)*/
const Device UA823007 = {S4813, 58, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; IRIX64 IP35; en-US; rv:1.4.3) Gecko/X*/
const Device UA852096 = {S4814, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; P03A)*/
const Device UA863668 = {S4815, 22, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - python-requests/0.12.0*/
const Device UA865558 = {S4816, 22, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - python-requests/0.10.2*/
const Device UA895006 = {S4817, 95, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - curl/7.24.0 (i686-pc-cygwin) libcurl/7.24.0 OpenSSL/1.0.1b zlib/1.2.5 libidn/1.22 libssh2/1.4.0*/
const Device UA900172 = {S4818, 22, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - python-requests/0.13.0*/
const Device UA900276 = {S4819, 11, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - curl/7.26.0*/
const Device UA901790 = {S4820, 22, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - python-requests/0.13.1*/
const Device UA902625 = {S4821, 25, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Wada.vn Vietnamese Search*/
const Device UA909046 = {S4822, 12, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Podcasts/1.0*/
const Device UA911162 = {S4823, 22, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - python-requests/0.13.2*/
const Device UA911770 = {S4824, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Add Catalog/2.1;)*/
const Device UA911779 = {S4825, 34, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Apache-HttpClient/4.2.1 (java 1.5)*/
const Device UA1850350 = {S4826, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; P01B)*/
const Device UA1856944 = {S4827, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; SH03A)*/
const Device UA1858888 = {S4828, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 936SH)*/
const Device UA1860555 = {S4829, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; N905i)*/
const Device UA1862616 = {S4830, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; P08A3)*/
const Device UA1866929 = {S4831, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; F02A)*/
const Device UA1872964 = {S4832, 44, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; SH05A3)*/
const Device UA1873890 = {S4833, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 831P)*/
const Device UA1874221 = {S4834, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; SH06B)*/
const Device UA1876365 = {S4835, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; N09A3)*/
const Device UA1880724 = {S4836, 44, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; SH905i)*/
const Device UA1884127 = {S4837, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 942P)*/
const Device UA1885072 = {S4838, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; F05C)*/
const Device UA1888080 = {S4839, 42, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 001N)*/
const Device UA1893699 = {S4840, 43, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; SH10C)*/
const Device UA1937571 = {S4841, 21, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - python-requests/0.8.2*/
const Device UA1948389 = {S4842, 21, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - python-requests/0.9.3*/
const Device UA1997057 = {S4843, 14, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Podcasts/1.0.1*/
const Device UA2010429 = {S4844, 14, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Podcasts/1.0.2*/
const Device UA2015970 = {S4845, 100, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Python-urllib/2.7 AppEngine-Google; (+http://code.google.com/appengine; appid: s~pulsecontentserver)*/
const Device UA2041931 = {S4846, 13, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Java/1.6.0_35*/
const Device UA2203304 = {S4847, 14, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Podcasts/1.1.1*/
const Device UA2205542 = {S4848, 12, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Podcasts/1.1*/
const Device UA2261881 = {S4849, 14, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Podcasts/1.1.2*/
const Device UA2355749 = {S4850, 33, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; evc/1.0)*/
const Device UA2356056 = {S4851, 34, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Apache-HttpClient/4.2.2 (java 1.5)*/
const Device UA2492145 = {S4852, 28, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Python-urllib2)*/
const Device UA2508911 = {S4853, 34, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Apache-HttpClient/4.2.3 (java 1.5)*/
const Device UA2580140 = {S4854, 12, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Podcasts/1.2*/
const Device UA2585684 = {S4855, 80, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; human_curl.async; 0.1.1; +http://h.wrttn.me/human_curl)*/
const Device UA2600849 = {S4856, 19, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Go 1.1 package http*/
const Device UA2602050 = {S4857, 14, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Podcasts/1.2.1*/
const Device UA2714997 = {S4858, 34, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Apache-HttpClient/4.2.5 (java 1.5)*/
const Device UA2740525 = {S4859, 34, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Google_Analytics_Snippet_Validator*/
const Device UA2772950 = {S4860, 54, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Prlog/1.0; +http://prlog.ru/)*/
const Device UA2780947 = {S4861, 47, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - HTTPClient/1.0 (2.3.3, ruby 1.9.3 (2013-02-22))*/
const Device UA2874015 = {S4862, 13, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Java/1.7.0_25*/
const Device UA3033187 = {S4863, 14, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Podcasts/1.2.2*/
const Device UA3067729 = {S4864, 89, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - ADmantX Platform Semantic Analyzer - ADmantX Inc. - www.admantx.com - support@admantx.com*/
const Device UA3068262 = {S4865, 14, { &P15364,&P17017,&P17470,&P18092 }};	/*Useragent - Podcasts/1.2.3*/
const Device UA589212 = {S4866, 88, { &P18088,&P17017,&P17369,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.26916/26.1424; U; ru) Presto/2.8.119 Version/10.54*/
const Device UA2709616 = {S4867, 88, { &P18088,&P17017,&P17369,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.31891/29.3417; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2799303 = {S4868, 59, { &P18088,&P17017,&P17369,&P18092 }};	/*Useragent - Opera/9.50 (J2ME/MIDP; Opera Mini/4.4.28962/29.3757; U; en)*/
const Device UA3035262 = {S4869, 97, { &P18088,&P17017,&P17369,&P18092 }};	/*Useragent - GT-E3300 Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.29644/30.3697; U; en) Presto/2.8.119 Version/11.10*/
const Device UA90617 = {S4870, 139, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5*/
const Device UA177678 = {S4871, 139, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F192 Safari/6533.18.5*/
const Device UA177920 = {S4872, 139, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F191 Safari/6533.18.5*/
const Device UA179157 = {S4873, 139, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8G4 Safari/6533.18.5*/
const Device UA179278 = {S4874, 139, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F190 Safari/6533.18.5*/
const Device UA180461 = {S4875, 139, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H8 Safari/6533.18.5*/
const Device UA190816 = {S4876, 139, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5*/
const Device UA190819 = {S4877, 139, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J3 Safari/6533.18.5*/
const Device UA194365 = {S4878, 141, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA206989 = {S4879, 141, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9Z999 Safari/6533.18.5*/
const Device UA223911 = {S4880, 139, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8K2 Safari/6533.18.5*/
const Device UA232669 = {S4881, 139, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA236477 = {S4882, 141, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA339633 = {S4883, 127, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 4_3_5 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0 Mobile/8L1 Safari/7533.17.9*/
const Device UA376506 = {S4884, 141, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA446113 = {S4885, 139, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F183 Safari/6533.18.5*/
const Device UA549890 = {S4886, 141, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA569688 = {S4887, 127, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 4_3_3 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0 Mobile/8J2 Safari/7533.17.9*/
const Device UA635265 = {S4888, 149, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 Version/5.0.2 Safari/6533.18.5 Kikin/1.9.0*/
const Device UA640534 = {S4889, 141, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9Z999 Safari/6533.18.5*/
const Device UA648202 = {S4890, 153, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; zh-cn Model:iPad2,1) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA694435 = {S4891, 140, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en_us) AppleWebKit/6533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA721638 = {S4892, 141, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F190 Safari/6533.18.5*/
const Device UA805817 = {S4893, 140, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en_us) AppleWebKit/6533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA842447 = {S4894, 139, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA850993 = {S4895, 128, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; 6533.17.9) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5*/
const Device UA860483 = {S4896, 128, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; 6533.17.9) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5*/
const Device UA1969498 = {S4897, 140, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en_us) AppleWebKit/6533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA1972053 = {S4898, 138, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en_us) AppleWebKit/6533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA2130198 = {S4899, 141, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F5166b Safari/6533.18.5*/
const Device UA2142225 = {S4900, 139, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA2297415 = {S4901, 139, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5*/
const Device UA2659316 = {S4902, 136, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 432 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8H8 Safari/6533.18.5*/
const Device UA2659420 = {S4903, 136, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 434 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8K2 Safari/6533.18.5*/
const Device UA2699295 = {S4904, 136, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 431 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8G4 Safari/6533.18.5*/
const Device UA2709415 = {S4905, 136, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 435 like Mac OS X; en-CA) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA2735142 = {S4906, 136, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 433 like Mac OS X; es-US) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5*/
const Device UA2850426 = {S4907, 145, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 433 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8J3 Safari/6533.18.5gzip(gfe)*/
const Device UA3043722 = {S4908, 144, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPad OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5*/
const Device UA3078896 = {S4909, 149, { &P15767,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.02 Guidebook/43 Mobile/8L1 Safari/6533.18.5*/
const Device UA2673796 = {S4910, 147, { &P15364,&P7133,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2676125 = {S4911, 137, { &P15364,&P7133,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/534.7 ZemanaAID/FFFF0089*/
const Device UA2676569 = {S4912, 140, { &P15364,&P7133,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2678354 = {S4913, 104, { &P15364,&P7133,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.1.3.0 Safari/537.31*/
const Device UA2678359 = {S4914, 97, { &P15364,&P7133,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.1.3.0 Safari/537.31*/
const Device UA2678386 = {S4915, 108, { &P15364,&P7133,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2678391 = {S4916, 101, { &P15364,&P7133,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2678541 = {S4917, 111, { &P15364,&P7133,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe)*/
const Device UA2678621 = {S4918, 118, { &P15364,&P7133,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe)*/
const Device UA2679926 = {S4919, 104, { &P15364,&P7133,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.2.2.0 Safari/537.31*/
const Device UA2679929 = {S4920, 97, { &P15364,&P7133,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.2.2.0 Safari/537.31*/
const Device UA2715878 = {S4921, 185, { &P15364,&P7133,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 AppEngine-Google; (+http://code.google.com/appengine; appid: optimizelyedit)*/
const Device UA2752512 = {S4922, 199, { &P15364,&P7133,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 AlexaToolbar/alxg-3.1 AppEngine-Google; (+http://code.google.com/appengine; appid: s~znyX)*/
const Device UA2996522 = {S4923, 107, { &P15364,&P7133,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA3005275 = {S4924, 171, { &P15364,&P7133,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 AppEngine-Google; (+http://code.google.com/appengine; appid: s~tinaX)*/
const Device UA3044643 = {S4925, 114, { &P15364,&P7133,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.27 (KHTML, like Gecko) Chrome/26.0.1384.0 (Dart) Safari/537.27*/
const Device UA3047829 = {S4926, 177, { &P15364,&P7133,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/537.31 AppEngine-Google; (+http://code.google.com/appengine; appid: s~easy-carson)*/
const Device UA3050327 = {S4927, 112, { &P15364,&P7133,&P19839,&P18092 }};	/*Useragent - Mozilla\5.0 (Windows NT 6.1) AppleWebKit\537.31 (KHTML, like Gecko) Chrome\26.0.1410.64 Safari\537.31; campus-TU*/
const Device UA3088225 = {S4928, 122, { &P15364,&P7133,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/0.0.0.0 Chrome/26.0.1410.43 Safari/537.1*/
const Device UA2431098 = {S4929, 88, { &P18095,&P18109,&P18369,&P18092 }};	/*Useragent - Opera/9.80 (Series 60; Opera Mini/7.0.31380/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2368157 = {S4930, 125, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1323.1) AppleWebKit/537.19 (KHTML, like Gecko) Chrome/25.0.1323.1 Safari/537.19*/
const Device UA2461172 = {S4931, 125, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.5 Safari/537.22*/
const Device UA2470718 = {S4932, 127, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.36) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.36 Safari/537.22*/
const Device UA2471082 = {S4933, 127, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.29) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.29 Safari/537.22*/
const Device UA2490957 = {S4934, 127, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.45) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.45 Safari/537.22*/
const Device UA2492403 = {S4935, 127, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.58) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.58 Safari/537.22*/
const Device UA2509584 = {S4936, 127, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.68) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.68 Safari/537.22*/
const Device UA2512829 = {S4937, 127, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.84) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.84 Safari/537.22*/
const Device UA2538362 = {S4938, 127, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/25.0.1364.97) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22*/
const Device UA2538377 = {S4939, 127, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.97) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22*/
const Device UA2567794 = {S4940, 129, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.172) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2567798 = {S4941, 173, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/25.0.1364.172; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2567880 = {S4942, 160, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/25.0.1364.172; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2568219 = {S4943, 82, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/25.0.1364.172)*/
const Device UA2568515 = {S4944, 129, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/25.0.1364.172) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2568520 = {S4945, 193, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/25.0.1364.172; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA2568678 = {S4946, 129, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.152) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22*/
const Device UA2568843 = {S4947, 221, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/25.0.1364.172; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2569461 = {S4948, 129, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.155) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.155 Safari/537.22*/
const Device UA2570045 = {S4949, 129, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/25.0.1364.152) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22*/
const Device UA2571175 = {S4950, 129, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/25.0.1364.160) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.160 Safari/537.22*/
const Device UA2583626 = {S4951, 141, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/25.0.1364.172; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)*/
const Device UA2583900 = {S4952, 207, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/25.0.1364.172; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2584014 = {S4953, 160, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/25.0.1364.172; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2584079 = {S4954, 239, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/25.0.1364.172; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2584224 = {S4955, 77, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/25.0.1364.172)*/
const Device UA2584382 = {S4956, 136, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/25.0.1364.172) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2605503 = {S4957, 161, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/25.0.1364.172; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2605844 = {S4958, 200, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/25.0.1364.172; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2752120 = {S4959, 262, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/25.0.1364.160; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; .NET4.0C; .NET4.0E; Zune 4.7)*/
const Device UA2857316 = {S4960, 134, { &P15364,&P5690,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/25.0.1364.97) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22*/
const Device UA2203723 = {S4961, 99, { &P15364,&P7133,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203726 = {S4962, 106, { &P15364,&P7133,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203876 = {S4963, 143, { &P15364,&P7133,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via translate.google.com)*/
const Device UA2203983 = {S4964, 136, { &P15364,&P7133,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via translate.google.com)*/
const Device UA2204447 = {S4965, 145, { &P15364,&P7133,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2231187 = {S4966, 137, { &P15364,&P7133,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/534.7 ZemanaAID/FFFF0061*/
const Device UA2234775 = {S4967, 116, { &P15364,&P7133,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)*/
const Device UA2234776 = {S4968, 109, { &P15364,&P7133,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)*/
const Device UA2238473 = {S4969, 138, { &P15364,&P7133,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2354732 = {S4970, 183, { &P15364,&P7133,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 AppEngine-Google; (+http://code.google.com/appengine; appid: optimizelyedit)*/
const Device UA2368280 = {S4971, 105, { &P15364,&P7133,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2369702 = {S4972, 98, { &P15364,&P7133,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2516475 = {S4973, 130, { &P15364,&P7133,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (gtsr) Safari/537.4*/
const Device UA2516759 = {S4974, 137, { &P15364,&P7133,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (gtsr) Safari/537.4*/
const Device UA2535790 = {S4975, 130, { &P15364,&P7133,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (fnrb) Safari/537.4*/
const Device UA2538176 = {S4976, 137, { &P15364,&P7133,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (fnrb) Safari/537.4*/
const Device UA2564640 = {S4977, 137, { &P15364,&P7133,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.520 YaBrowser/1.5.1105.520 (bskt) Safari/537.4*/
const Device UA2570075 = {S4978, 130, { &P15364,&P7133,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.520 YaBrowser/1.5.1105.520 (bskt) Safari/537.4*/
const Device UA2718559 = {S4979, 100, { &P15364,&P7133,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.1.9999.433 Safari/537.4*/
const Device UA2718793 = {S4980, 107, { &P15364,&P7133,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.1.9999.433 Safari/537.4*/
const Device UA2972550 = {S4981, 134, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/29.0.1547.22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.22 Safari/537.36*/
const Device UA2973989 = {S4982, 127, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/29.0.1547.22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.22 Safari/537.36*/
const Device UA3022659 = {S4983, 127, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/29.0.1547.32) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.32 Safari/537.36*/
const Device UA3023275 = {S4984, 134, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/29.0.1547.32) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.32 Safari/537.36*/
const Device UA3026699 = {S4985, 134, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/29.0.1547.41) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.41 Safari/537.36*/
const Device UA3027954 = {S4986, 127, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/29.0.1547.41) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.41 Safari/537.36*/
const Device UA3031455 = {S4987, 134, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/29.0.1547.57) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36*/
const Device UA3033164 = {S4988, 127, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/29.0.1547.49) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.49 Safari/537.36*/
const Device UA3034436 = {S4989, 127, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/29.0.1547.57) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36*/
const Device UA3036664 = {S4990, 134, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/29.0.1547.49) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.49 Safari/537.36*/
const Device UA3062176 = {S4991, 127, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/29.0.1547.62) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.62 Safari/537.36*/
const Device UA3063754 = {S4992, 134, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/29.0.1547.62) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.62 Safari/537.36*/
const Device UA3063958 = {S4993, 197, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA3067636 = {S4994, 204, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA3067752 = {S4995, 208, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)*/
const Device UA3067761 = {S4996, 96, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/29.0.1547.66)*/
const Device UA3067802 = {S4997, 127, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/29.0.1547.66) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36*/
const Device UA3067803 = {S4998, 89, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/29.0.1547.66)*/
const Device UA3067811 = {S4999, 134, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/29.0.1547.66) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36*/
const Device UA3067814 = {S5000, 97, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/29.0.1547.66)*/
const Device UA3067928 = {S5001, 90, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/29.0.1547.66)*/
const Device UA3067950 = {S5002, 176, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA3067961 = {S5003, 197, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA3067965 = {S5004, 204, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA3067991 = {S5005, 208, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)*/
const Device UA3068032 = {S5006, 191, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA3068123 = {S5007, 224, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0)*/
const Device UA3079925 = {S5008, 214, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Tablet PC 2.0)*/
const Device UA3080284 = {S5009, 176, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA3082340 = {S5010, 269, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA3093869 = {S5011, 269, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/29.0.1547.66; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA3096638 = {S5012, 134, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/29.0.1547.76) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36*/
const Device UA3096646 = {S5013, 127, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/29.0.1547.76) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36*/
const Device UA3112008 = {S5014, 209, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/29.0.1547.76; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA3115041 = {S5015, 124, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; MSBrowserIE8; Windows NT 6.1; WOW64; Trident/5.0; MSBrowserIE9; chromeframe/29.0.1547.66)*/
const Device UA3115252 = {S5016, 94, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0; chromeframe/29.0.1547.76) like Gecko*/
const Device UA3127777 = {S5017, 269, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/29.0.1547.76; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3127794 = {S5018, 222, { &P15364,&P7133,&P22769,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/29.0.1547.76; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C; .NET4.0E; 3M/MSIE 8.0)*/
const Device UA1996265 = {S5019, 139, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I747 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2006116 = {S5020, 139, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I747 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028466 = {S5021, 139, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028793 = {S5022, 139, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028819 = {S5023, 147, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033523 = {S5024, 139, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9305 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2183224 = {S5025, 146, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/tMIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2189033 = {S5026, 140, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; GT-I9305T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2190422 = {S5027, 139, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2191124 = {S5028, 147, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2194831 = {S5029, 140, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-I747M Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2203523 = {S5030, 139, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I747 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2204067 = {S5031, 159, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; SAMSUNG GT-I9305/I9305XXALI5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2204706 = {S5032, 139, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I747 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2204868 = {S5033, 139, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2228901 = {S5034, 61, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SPH-L710 Build/JRO03L)*/
const Device UA2235644 = {S5035, 139, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2235994 = {S5036, 61, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-I9305 Build/JRO03C)*/
const Device UA2237120 = {S5037, 159, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; SAMSUNG GT-I9305/I9305XXALI7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2237963 = {S5038, 139, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I747 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257712 = {S5039, 140, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; SHV-E210K Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257952 = {S5040, 147, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257959 = {S5041, 140, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; SHV-E210L Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257960 = {S5042, 140, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; SHV-E210S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257963 = {S5043, 140, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; GT-I9305N Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2353924 = {S5044, 62, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-I9305T Build/JRO03C)*/
const Device UA2356113 = {S5045, 145, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2360755 = {S5046, 137, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; SPH-L710 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2384553 = {S5047, 69, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K)*/
const Device UA2399514 = {S5048, 62, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SGH-I747M Build/JRO03L)*/
const Device UA2400685 = {S5049, 69, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L)*/
const Device UA2405031 = {S5050, 159, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; SAMSUNG GT-I9305/I9305XXALJ4 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2405630 = {S5051, 159, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; SAMSUNG GT-I9305/I9305XXALK5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2419234 = {S5052, 140, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-R530U Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2421993 = {S5053, 139, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9305 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422836 = {S5054, 147, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-I747M-parrot Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426601 = {S5055, 138, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; SGH-I747M Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2430025 = {S5056, 151, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-R530U Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 USCC-R530U*/
const Device UA2430961 = {S5057, 140, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E210K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2431759 = {S5058, 140, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E210S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2432614 = {S5059, 140, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E210L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2433810 = {S5060, 62, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SCH-R530U Build/JRO03L)*/
const Device UA2454948 = {S5061, 147, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455604 = {S5062, 61, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SPH-L710 Build/JZO54K)*/
const Device UA2461394 = {S5063, 159, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I9305/I9305XXBLL5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2462258 = {S5064, 159, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I9305/I9305XXBLL3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467689 = {S5065, 159, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-ch; SAMSUNG GT-I9305/I9305XXALK3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469333 = {S5066, 61, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9305 Build/JZO54K)*/
const Device UA2485503 = {S5067, 62, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E210S Build/JZO54K)*/
const Device UA2502864 = {S5068, 61, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I747 Build/JZO54K)*/
const Device UA2510342 = {S5069, 140, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-I9305T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2511373 = {S5070, 159, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I9305/I9305XXBMA3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528730 = {S5071, 62, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E210K Build/JZO54K)*/
const Device UA2531414 = {S5072, 158, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-I9305-ORANGE/I9305XXBMA1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2554801 = {S5073, 62, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-R530U Build/JZO54K)*/
const Device UA2555465 = {S5074, 145, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2559653 = {S5075, 151, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530U Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 USCC-R530U*/
const Device UA2563856 = {S5076, 62, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9305T Build/JZO54K)*/
const Device UA2564869 = {S5077, 159, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; SAMSUNG GT-I9305/I9305XXBMB4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2569592 = {S5078, 159, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; SAMSUNG GT-I9305/I9305XXBMA4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2570297 = {S5079, 137, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SPH-L710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2594331 = {S5080, 62, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SHV-E210S Build/JRO03C)*/
const Device UA2594783 = {S5081, 62, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-I9305N Build/JRO03C)*/
const Device UA2596464 = {S5082, 62, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E210L Build/JZO54K)*/
const Device UA2596715 = {S5083, 138, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2659396 = {S5084, 140, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530C Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2660544 = {S5085, 140, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; GT-I9305N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2669788 = {S5086, 147, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2674014 = {S5087, 159, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; SAMSUNG GT-I9305/I9305XXBMC1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2677984 = {S5088, 62, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-R530C Build/JZO54K)*/
const Device UA2680757 = {S5089, 159, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I9305/I9305XXBMD1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2696428 = {S5090, 131, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-I747M Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2716455 = {S5091, 147, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E210K/KKJMD2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2735029 = {S5092, 62, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9305N Build/JZO54K)*/
const Device UA2737495 = {S5093, 147, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I747 Build/JZO51K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2745982 = {S5094, 153, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30*/
const Device UA2752493 = {S5095, 159, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I9305/I9305XXBMD5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2777524 = {S5096, 61, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SGH-I747 Build/JRO03L)*/
const Device UA2777734 = {S5097, 62, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SHV-E210K Build/JRO03C)*/
const Device UA2789902 = {S5098, 62, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SHV-E210L Build/JRO03C)*/
const Device UA2878285 = {S5099, 62, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I747M Build/JZO54K)*/
const Device UA2879644 = {S5100, 140, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I747M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2885325 = {S5101, 147, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I747M-parrot Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2894138 = {S5102, 138, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SGH-I747M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2898642 = {S5103, 159, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I9305/I9305XXBME3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2899137 = {S5104, 158, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-I9305-ORANGE/I9305XXBMF1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2948573 = {S5105, 147, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E210K/KKJMF6 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985301 = {S5106, 159, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; el-gr; SAMSUNG GT-I9305/I9305XXBLL1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985424 = {S5107, 137, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-I9305 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2989644 = {S5108, 138, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SCH-R530U Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3033179 = {S5109, 159, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; SAMSUNG GT-I9305/I9305XXBMG3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3062173 = {S5110, 169, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=d2spr;devm=SPH-L710;man=samsung;brw=Android;shwd=720x1280;shwa=720x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA3065844 = {S5111, 177, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=d2att;devm=SAMSUNG-SGH-I747;man=samsung;brw=Android;shwd=720x1280;shwa=720x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA3067515 = {S5112, 147, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E210K/KKJMH1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3070979 = {S5113, 158, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GT-I9305-ORANGE/I9305XXALIA Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3104593 = {S5114, 145, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3120512 = {S5115, 138, { &P18645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SCH-R530C Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA366276 = {S5116, 139, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA367771 = {S5117, 136, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA368873 = {S5118, 131, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-i9100 Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA370663 = {S5119, 134, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; es-es; GT-I9100 Build/ICS) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA441506 = {S5120, 139, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA605331 = {S5121, 158, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; SAMSUNG GT-I9100/I9100BOLPD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA605436 = {S5122, 157, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100-ORANGE/I9100BVLPB Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA606701 = {S5123, 158, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SAMSUNG GT-I9100/I9100BULPC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA613070 = {S5124, 158, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SAMSUNG GT-I9100/I9100BVLPB Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA615303 = {S5125, 142, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76I-RR) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616633 = {S5126, 140, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; GT-I9100M Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616638 = {S5127, 140, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; GT-I9100T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616646 = {S5128, 69, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - BBC iPlayer (AndroidApp; 1.3; 13014) GT-I9100 (Android 4.0.3, SDK 15)*/
const Device UA622302 = {S5129, 143, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; GT-I9100 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30*/
const Device UA622424 = {S5130, 145, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.1 Mobile Safari/534.30*/
const Device UA622415 = {S5131, 90, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KH*/
const Device UA633579 = {S5132, 61, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-I9100 Build/IML74K)*/
const Device UA636265 = {S5133, 139, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA637003 = {S5134, 62, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-I9100M Build/IML74K)*/
const Device UA662954 = {S5135, 61, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9100 Build/IMM76D)*/
const Device UA739721 = {S5136, 125, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9100 Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA751862 = {S5137, 62, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-I9100T Build/IML74K)*/
const Device UA752290 = {S5138, 137, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA771131 = {S5139, 132, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA803901 = {S5140, 158, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; SAMSUNG GT-I9100/I9100BOLPE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA866403 = {S5141, 166, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-I9100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA894080 = {S5142, 142, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100 Build/IMM76I_RR) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899763 = {S5143, 140, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899899 = {S5144, 62, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-I9100G Build/IML74K)*/
const Device UA901481 = {S5145, 150, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy SII T-mobile Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA904383 = {S5146, 130, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9100 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA911102 = {S5147, 158, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SAMSUNG GT-I9100/I9100BULPD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA912224 = {S5148, 158, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SAMSUNG GT-I9100/I9100BVLPD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA913295 = {S5149, 157, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100-ORANGE/I9100BVLPD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1955818 = {S5150, 137, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; GT-I9100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1971535 = {S5151, 131, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100G Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1973690 = {S5152, 137, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/WIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1975884 = {S5153, 132, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1986308 = {S5154, 138, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/cMIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996822 = {S5155, 140, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2024400 = {S5156, 136, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.2; es-es; GT-I9100 Build/4.0.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025783 = {S5157, 148, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100 Build/AdyScorpiusMIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026176 = {S5158, 130, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2026332 = {S5159, 136, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026767 = {S5160, 140, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027880 = {S5161, 138, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-I9100T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028692 = {S5162, 132, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2028739 = {S5163, 130, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9100 Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2030191 = {S5164, 62, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9100G Build/IMM76D)*/
const Device UA2031399 = {S5165, 158, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9100/I9100BVLPH Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032359 = {S5166, 158, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; SAMSUNG GT-I9100/I9100BOLPB Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032884 = {S5167, 142, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033358 = {S5168, 148, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/Neutrino_system) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2041756 = {S5169, 157, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100-ORANGE/I9100BVLPH Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2165441 = {S5170, 137, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-I9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2203556 = {S5171, 140, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; GT-I9100T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2203975 = {S5172, 158, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SAMSUNG GT-I9100/I9100BOLPM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2222777 = {S5173, 134, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; I9100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2225143 = {S5174, 138, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-I9100G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2232800 = {S5175, 158, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9100/I9100BULPG Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257402 = {S5176, 145, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2257532 = {S5177, 145, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2339366 = {S5178, 142, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0UI; en-us; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2355027 = {S5179, 145, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-I9100 Build/XWLPW IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2380373 = {S5180, 138, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-I9100G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2396471 = {S5181, 156, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; SAMSUNG GT-I9100/I9100BULPD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2396941 = {S5182, 130, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9108 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2421988 = {S5183, 156, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG GT-I9100/I9100BULPG Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2445081 = {S5184, 131, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; GT-I9100T Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2453136 = {S5185, 139, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9108 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2499807 = {S5186, 143, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9100 Build/ICS.g12refM1005.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2502796 = {S5187, 62, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9100T Build/IMM76D)*/
const Device UA2518693 = {S5188, 137, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2519699 = {S5189, 131, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9100G Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2526722 = {S5190, 145, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; GT-I9100 Build/IMM76D.JPLPH) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2527122 = {S5191, 54, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Samsung GT-I9100 stagefright/1.2 (Linux;Android 4.0.4)*/
const Device UA2578017 = {S5192, 138, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-I9100M Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583164 = {S5193, 158, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9100/I9100BVLPJ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583615 = {S5194, 157, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100-ORANGE/I9100BVLPJ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2588165 = {S5195, 130, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100 Build/IMM76L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2702657 = {S5196, 126, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - sprd-I9100/1.0 Linux/2.6.35.7 Android 4.0.1 Release/12.29.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2708110 = {S5197, 138, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-US; GT-I9100 Build/ITL41D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2727546 = {S5198, 145, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; GT-I9100 Build/IMM76D.JHLPC) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2762428 = {S5199, 134, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-us; GT-I9100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2780875 = {S5200, 139, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-US; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2790549 = {S5201, 138, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100M Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2810859 = {S5202, 139, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2833470 = {S5203, 144, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-I9100 Build/7comp_1.3.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2833968 = {S5204, 156, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG GT-I9100/I9100BVLPH Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2848023 = {S5205, 146, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; GT-I9100 Build/vauss miui.es) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892986 = {S5206, 156, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - GT-I9100 Linux/3.0.13 Android/4.1.9 Release/05.07.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2950756 = {S5207, 144, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-I9100 Build/7comp_1.3.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2982598 = {S5208, 139, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; GT-I9100G Build/unknown) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3070337 = {S5209, 135, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-us; GT-I9100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3086237 = {S5210, 129, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-I9100 Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3117230 = {S5211, 61, { &P15575,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 4.0.9; GT-I9100 Build/GRK39F)*/
const Device UA2797797 = {S5212, 88, { &P18088,&P17017,&P18328,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/7.1.32052/30.3214; U; en) Presto/2.8.119 Version/11.10*/
const Device UA3054967 = {S5213, 59, { &P18088,&P17017,&P18328,&P18092 }};	/*Useragent - Opera/9.50 (J2ME/MIDP; Opera Mini/7.1.32052/29.3757; U; en)*/
const Device UA2798619 = {S5214, 108, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A4372q*/
const Device UA2799132 = {S5215, 92, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko)*/
const Device UA2874157 = {S5216, 106, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A401*/
const Device UA2876899 = {S5217, 106, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A396*/
const Device UA2885394 = {S5218, 108, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A4400f*/
const Device UA2892756 = {S5219, 140, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone 7_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.8 (iPhone5,2/7.0)*/
const Device UA2944863 = {S5220, 92, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko)*/
const Device UA2947243 = {S5221, 108, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Mobile/11A4414e*/
const Device UA2948001 = {S5222, 106, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Mobile/11A421*/
const Device UA3012710 = {S5223, 108, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Mobile/11A4414c*/
const Device UA3023162 = {S5224, 94, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko)*/
const Device UA3024925 = {S5225, 110, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4435d*/
const Device UA3031707 = {S5226, 108, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A458*/
const Device UA3033016 = {S5227, 110, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449a*/
const Device UA3033684 = {S5228, 108, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A462*/
const Device UA3033714 = {S5229, 110, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449b*/
const Device UA3034362 = {S5230, 110, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d*/
const Device UA3059097 = {S5231, 108, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A450*/
const Device UA3061127 = {S5232, 108, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A449*/
const Device UA3065203 = {S5233, 108, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465*/
const Device UA3078895 = {S5234, 108, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A464*/
const Device UA3093231 = {S5235, 108, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A466*/
const Device UA3094864 = {S5236, 111, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A470a*/
const Device UA3109306 = {S5237, 135, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone4,1; iPhone; U; CPU OS 7_0 like Mac OS X; en_US) com.google.GooglePlus/23341 (KHTML, like Gecko) Mobile/N94AP (gzip)*/
const Device UA3109374 = {S5238, 135, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone5,1; iPhone; U; CPU OS 7_0 like Mac OS X; en_US) com.google.GooglePlus/23341 (KHTML, like Gecko) Mobile/N41AP (gzip)*/
const Device UA3111530 = {S5239, 135, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone5,2; iPhone; U; CPU OS 7_0 like Mac OS X; en_US) com.google.GooglePlus/23341 (KHTML, like Gecko) Mobile/N42AP (gzip)*/
const Device UA3111576 = {S5240, 96, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko)*/
const Device UA3112216 = {S5241, 96, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko)*/
const Device UA3114309 = {S5242, 244, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 7_0 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone4,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/7.0;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA3115084 = {S5243, 143, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 bline/1.09 (iPhone OS 7.0, iPhone)*/
const Device UA3115124 = {S5244, 125, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari/7534.48.3*/
const Device UA3115705 = {S5245, 110, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501*/
const Device UA3125828 = {S5246, 110, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11B470*/
const Device UA3129539 = {S5247, 137, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone4,1; iPhone; U; CPU OS 7_0_2 like Mac OS X; en_US) com.google.GooglePlus/23341 (KHTML, like Gecko) Mobile/N94AP (gzip)*/
const Device UA3130252 = {S5248, 137, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone5,2; iPhone; U; CPU OS 7_0_2 like Mac OS X; en_US) com.google.GooglePlus/23341 (KHTML, like Gecko) Mobile/N42AP (gzip)*/
const Device UA3130363 = {S5249, 137, { &P12280,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone5,1; iPhone; U; CPU OS 7_0_2 like Mac OS X; en_US) com.google.GooglePlus/23341 (KHTML, like Gecko) Mobile/N41AP (gzip)*/
const Device UA2753618 = {S5250, 134, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/28.0.1500.20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.20 Safari/537.36*/
const Device UA2770625 = {S5251, 127, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/28.0.1500.20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.20 Safari/537.36*/
const Device UA2773265 = {S5252, 134, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/28.0.1500.29) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.29 Safari/537.36*/
const Device UA2774046 = {S5253, 127, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/28.0.1500.29) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.29 Safari/537.36*/
const Device UA2784873 = {S5254, 134, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/28.0.1500.37) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.37 Safari/537.36*/
const Device UA2796701 = {S5255, 127, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/28.0.1500.37) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.37 Safari/537.36*/
const Device UA2797617 = {S5256, 134, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/28.0.1500.44) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.44 Safari/537.36*/
const Device UA2875297 = {S5257, 127, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/28.0.1500.44) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.44 Safari/537.36*/
const Device UA2877555 = {S5258, 134, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/28.0.1500.52) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36*/
const Device UA2877655 = {S5259, 127, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/28.0.1500.52) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36*/
const Device UA2885426 = {S5260, 134, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/28.0.1500.63) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36*/
const Device UA2898056 = {S5261, 127, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/28.0.1500.63) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36*/
const Device UA2899357 = {S5262, 134, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/28.0.1500.71) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36*/
const Device UA2899358 = {S5263, 127, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/28.0.1500.71) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36*/
const Device UA2945889 = {S5264, 208, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/28.0.1500.71; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)*/
const Device UA2948143 = {S5265, 127, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/28.0.1500.72) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36*/
const Device UA2948500 = {S5266, 134, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/28.0.1500.72) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36*/
const Device UA3024748 = {S5267, 97, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/28.0.1500.95)*/
const Device UA3024750 = {S5268, 208, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)*/
const Device UA3024755 = {S5269, 262, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA3024948 = {S5270, 94, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0; chromeframe/28.0.1500.95) like Gecko*/
const Device UA3024949 = {S5271, 90, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/28.0.1500.95)*/
const Device UA3025064 = {S5272, 191, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA3025079 = {S5273, 96, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/28.0.1500.95)*/
const Device UA3025193 = {S5274, 204, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA3025533 = {S5275, 197, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA3025705 = {S5276, 89, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/28.0.1500.95)*/
const Device UA3025707 = {S5277, 204, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA3025717 = {S5278, 177, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA3025914 = {S5279, 134, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/28.0.1500.95) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3026166 = {S5280, 209, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/28.0.1500.95; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA3026217 = {S5281, 127, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/28.0.1500.95) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3026462 = {S5282, 212, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA3027911 = {S5283, 249, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA3028030 = {S5284, 217, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; MS-RTC LM 8; InfoPath.2)*/
const Device UA3028411 = {S5285, 222, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C; .NET4.0E; 3M/MSIE 8.0)*/
const Device UA3028466 = {S5286, 224, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/28.0.1500.95; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA3028682 = {S5287, 176, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA3029760 = {S5288, 176, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA3029877 = {S5289, 112, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Avant Browser; chromeframe/28.0.1500.95)*/
const Device UA3029972 = {S5290, 261, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA3029999 = {S5291, 188, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/28.0.1500.95; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA3030409 = {S5292, 269, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/28.0.1500.95; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA3061056 = {S5293, 124, { &P15364,&P7133,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; MSBrowserIE8; Windows NT 6.1; WOW64; Trident/5.0; MSBrowserIE9; chromeframe/28.0.1500.95)*/
const Device UA895380 = {S5294, 109, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.12 (KHTML, like Gecko) Version/6.0 Safari/536.12*/
const Device UA899124 = {S5295, 109, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.14 (KHTML, like Gecko) Version/6.0 Safari/536.14*/
const Device UA900174 = {S5296, 109, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.15 (KHTML, like Gecko) Version/6.0 Safari/536.16*/
const Device UA901971 = {S5297, 109, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.17 (KHTML, like Gecko) Version/6.0 Safari/536.17*/
const Device UA902780 = {S5298, 109, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.19 (KHTML, like Gecko) Version/6.0 Safari/536.19*/
const Device UA908222 = {S5299, 109, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.22 (KHTML, like Gecko) Version/6.0 Safari/536.22*/
const Device UA909619 = {S5300, 109, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.24 (KHTML, like Gecko) Version/6.0 Safari/536.24*/
const Device UA911417 = {S5301, 109, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.25 (KHTML, like Gecko) Version/6.0 Safari/536.25*/
const Device UA924900 = {S5302, 114, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.25 (KHTML, like Gecko) WebClip/8536.24 Safari/8536.25*/
const Device UA2258463 = {S5303, 119, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.17 (KHTML, like Gecko) Version/6.0.2 Safari/536.26.17*/
const Device UA2258480 = {S5304, 124, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) WebClip/8536.26.7 Safari/8536.26.17*/
const Device UA2258785 = {S5305, 118, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_1; en) AppleWebKit/536.25 (KHTML, like Gecko) Version/6.0 Safari/536.25*/
const Device UA2422508 = {S5306, 145, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8.2) AppleWebKit/8536.26.14 (KHTML, like Gecko) Version/6.0.2 Safari/8536.26.17 Raven for Mac/0.7.14665*/
const Device UA2480905 = {S5307, 108, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1081) AppleWebKit/536.25 (KHTML like Gecko) Version/6.0 Safari/536.25*/
const Device UA2490844 = {S5308, 119, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) Version/6.0.3 Safari/536.28.10*/
const Device UA2546092 = {S5309, 135, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8) AppleWebKit/8536.25 (KHTML, like Gecko) Version/6.0 Safari/8536.25 Raven for Mac/0.7.14665*/
const Device UA2567292 = {S5310, 124, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) WebClip/8536.28.3 Safari/8536.28.10*/
const Device UA2596475 = {S5311, 107, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 108) AppleWebKit/536.22 (KHTML like Gecko) Version/6.0 Safari/536.22*/
const Device UA2597529 = {S5312, 107, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 108) AppleWebKit/536.15 (KHTML like Gecko) Version/6.0 Safari/536.16*/
const Device UA2597639 = {S5313, 116, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1082) AppleWebKit/536.26.14 (KHTML like Gecko) Version/6.0.1 Safari/536.26.14*/
const Device UA2679273 = {S5314, 124, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) WebClip/8536.29.3 Safari/8536.29.13*/
const Device UA2679622 = {S5315, 119, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.29.13 (KHTML, like Gecko) Version/6.0.4 Safari/536.29.13*/
const Device UA2693345 = {S5316, 215, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_2; en) AppleWebKit/536.26.14 (KHTML, like Gecko) Version/6.0.1 Safari/536.26.14*/
const Device UA2717671 = {S5317, 117, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) Version/6.0.5 Safari/536.30.1*/
const Device UA2784476 = {S5318, 120, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) WebClip/8536.30 Safari/8536.30.1*/
const Device UA2937246 = {S5319, 114, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1084) AppleWebKit/536.30.1 (KHTML like Gecko) Version/6.0.5 Safari/536.30.1*/
const Device UA3003349 = {S5320, 143, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8.4) AppleWebKit/8536.30.1 (KHTML, like Gecko) Version/6.0.5 Safari/8536.30.1 Raven for Mac/0.7.14665*/
const Device UA3080312 = {S5321, 113, { &P17595,&P18207,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/538.1+ (KHTML, like Gecko) Version/6.0.5 Safari/538.1+*/
const Device UA3115142 = {S5322, 89, { &P18115,&P5764,&P18328,&P18092 }};	/*Useragent - Opera/9.80 (BlackBerry; Opera Mini/7.1.33553/31.1475; U; en) Presto/2.8.119 Version/11.10*/
const Device UA367957 = {S5323, 98, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B5141a*/
const Device UA441525 = {S5324, 235, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 5_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/5.1;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]*/
const Device UA455471 = {S5325, 121, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 iPad2,1/K93AP Zite/1.3.3*/
const Device UA615696 = {S5326, 239, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 5_1_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/5.1.1;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]*/
const Device UA616338 = {S5327, 96, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176*/
const Device UA616339 = {S5328, 98, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206*/
const Device UA616496 = {S5329, 95, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B17*/
const Device UA623096 = {S5330, 85, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)*/
const Device UA623184 = {S5331, 83, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)*/
const Device UA662578 = {S5332, 123, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 iPad1,1/K48AP Zite/1.3.3*/
const Device UA696600 = {S5333, 129, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - MQQBrowser/11 Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA703801 = {S5334, 96, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405*/
const Device UA812672 = {S5335, 112, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/525.20*/
const Device UA834507 = {S5336, 110, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/525.20*/
const Device UA842736 = {S5337, 115, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA896659 = {S5338, 112, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - MQQBrowser/10 Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206*/
const Device UA2006124 = {S5339, 164, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48 Instapaper/4.2.4 (+http://www.instapaper.com/iphone)*/
const Device UA2041882 = {S5340, 130, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad2; iPad; U; CPU OS 5_1_1 like Mac OS X; en_US) com.google.GooglePlus/8654 (KHTML, like Gecko) Mobile/K93AP (gzip)*/
const Device UA2155436 = {S5341, 130, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - MQQBrowser/1.9.1 Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3*/
const Device UA2165626 = {S5342, 98, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9Z999*/
const Device UA2189061 = {S5343, 130, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad3G; iPad; U; CPU OS 5_1_1 like Mac OS X; en_US) com.google.GooglePlus/8654 (KHTML, like Gecko) Mobile/J1AP (gzip)*/
const Device UA2202608 = {S5344, 213, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/5.1;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]*/
const Device UA2205551 = {S5345, 164, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48 Instapaper/4.2.6 (+http://www.instapaper.com/iphone)*/
const Device UA2260521 = {S5346, 128, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/8536.26.7 DiigoBrowser*/
const Device UA2316026 = {S5347, 132, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - MQQBrowser/1.9.1 Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA2364751 = {S5348, 98, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405*/
const Device UA2366948 = {S5349, 98, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176*/
const Device UA2386215 = {S5350, 158, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/8536.26.7 DiigoBrowser Safari/8536.26.7 DiigoBrowser*/
const Device UA2404673 = {S5351, 112, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 YaMobile/1.00*/
const Device UA2425805 = {S5352, 126, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/8536.26.7 DiigoBrowser*/
const Device UA2429268 = {S5353, 217, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPad1,1;FBMD/iPad;FBSN/iPhone OS;FBSV/5.1.1;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]*/
const Device UA2463149 = {S5354, 112, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/534.46*/
const Device UA2506368 = {S5355, 44, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Miu Ptt 1.9.2 (iPad; iPhone OS 5.1.1; zh_TW)*/
const Device UA2660838 = {S5356, 44, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - QQ???HD 2.8.1 (iPad; iPhone OS 5.1.1; zh_CN)*/
const Device UA2735372 = {S5357, 133, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad1,1; iPad; U; CPU OS 5_1_1 like Mac OS X; en_US) com.google.GooglePlus/16077 (KHTML, like Gecko) Mobile/K48AP (gzip)*/
const Device UA2813185 = {S5358, 110, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/534.46*/
const Device UA2824819 = {S5359, 112, { &P15767,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/528.16*/
const Device UA198301 = {S5360, 111, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/534.42 (KHTML, like Gecko) Version/5.1 Safari/534.42*/
const Device UA222529 = {S5361, 111, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50*/
const Device UA228716 = {S5362, 116, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.50 (KHTML, like Gecko) WebClip/6534.49 Safari/6534.50*/
const Device UA250824 = {S5363, 119, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22*/
const Device UA329047 = {S5364, 121, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50*/
const Device UA346841 = {S5365, 123, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.52.7 (KHTML, like Gecko) WebClip/6534.51.13 Safari/6534.52.7*/
const Device UA349718 = {S5366, 135, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - iChromy Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1 Safari/534.51.22*/
const Device UA351558 = {S5367, 125, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1 Safari/534.52.7*/
const Device UA361858 = {S5368, 140, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - DiigoBrowser Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1 Safari/534.51.22*/
const Device UA427004 = {S5369, 119, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16*/
const Device UA428876 = {S5370, 125, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.54.16 (KHTML, like Gecko) WebClip/6534.51.13 Safari/6534.54.16*/
const Device UA436522 = {S5371, 121, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.48 (KHTML, like Gecko) Version/5.1 Safari/534.48*/
const Device UA440928 = {S5372, 117, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3*/
const Device UA587682 = {S5373, 143, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6.8) AppleWebKit/6534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/6534.52.7 Raven for Mac/0.7.14612*/
const Device UA615808 = {S5374, 117, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2*/
const Device UA622545 = {S5375, 117, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7*/
const Device UA630452 = {S5376, 123, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) WebClip/6534.51.13 Safari/6534.57.2*/
const Device UA630742 = {S5377, 113, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.55.3 (KHTML, like Gecko, Safari/6534.55.3) ADM/722*/
const Device UA686725 = {S5378, 140, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1 Safari/534.51.22 DiigoBrowser*/
const Device UA756504 = {S5379, 110, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (IMDb iOS; Intel Mac OS X 10_6_8) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50*/
const Device UA852424 = {S5380, 127, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3*/
const Device UA862530 = {S5381, 124, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) WebClip/6534.51.13 Safari/6534.51.13*/
const Device UA886733 = {S5382, 127, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.6 Safari/534.57.2*/
const Device UA1892312 = {S5383, 121, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.5+ (KHTML, like Gecko) Version/5.1 Safari/533.5+*/
const Device UA1969123 = {S5384, 143, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6.8) AppleWebKit/6534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/6534.57.2 Raven for Mac/0.7.14665*/
const Device UA1989407 = {S5385, 129, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16*/
const Device UA2320120 = {S5386, 121, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.42 (KHTML, like Gecko) Version/5.1 Safari/534.42*/
const Device UA2329168 = {S5387, 125, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/534.34.4 (KHTML, like Gecko) Version/5.1 Safari/534.34.4*/
const Device UA2412111 = {S5388, 112, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Cruz/0.4 Safari/534.57.2*/
const Device UA2474939 = {S5389, 121, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10\_6\_8) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22*/
const Device UA2570878 = {S5390, 117, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.58.2 (KHTML, like Gecko) Version/5.1.8 Safari/534.58.2*/
const Device UA2584773 = {S5391, 123, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.58.2 (KHTML, like Gecko) WebClip/6534.51.13 Safari/6534.58.2*/
const Device UA2677414 = {S5392, 117, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.59.8 (KHTML, like Gecko) Version/5.1.9 Safari/534.59.8*/
const Device UA2679266 = {S5393, 123, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.59.8 (KHTML, like Gecko) WebClip/6534.51.13 Safari/6534.59.8*/
const Device UA2727355 = {S5394, 114, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1068) AppleWebKit/534.52.7 (KHTML like Gecko) Version/5.1.2 Safari/534.52.7*/
const Device UA2795195 = {S5395, 127, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.59.8 (KHTML, like Gecko) Version/5.1.8 Safari/534.59.8*/
const Device UA2832913 = {S5396, 113, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10\_6\_8) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50*/
const Device UA3073041 = {S5397, 212, { &P17595,&P17296,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.59.8 (KHTML, like Gecko) Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en) AppleWebKit/534.59.8 (KHTML, like Gecko) Version/5.1.9 Safari/534.59.8*/
const Device UA2359360 = {S5398, 135, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B5105c Safari/8536.25*/
const Device UA2368060 = {S5399, 133, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B114 Safari/8536.25*/
const Device UA2391465 = {S5400, 135, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B5117b Safari/8536.25*/
const Device UA2437231 = {S5401, 135, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B5126b Safari/8536.25*/
const Device UA2437293 = {S5402, 133, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B135 Safari/8536.25*/
const Device UA2460332 = {S5403, 133, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B130 Safari/8536.25*/
const Device UA2467983 = {S5404, 133, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B133 Safari/8536.25*/
const Device UA2469654 = {S5405, 133, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B137 Safari/8536.25*/
const Device UA2470630 = {S5406, 133, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA2471692 = {S5407, 133, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B143 Safari/8536.25*/
const Device UA2484384 = {S5408, 133, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B136 Safari/8536.25*/
const Device UA2486269 = {S5409, 133, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B142 Safari/8536.25*/
const Device UA2488397 = {S5410, 133, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B144 Safari/8536.25*/
const Device UA2508106 = {S5411, 132, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B142 Safari/8536.25*/
const Device UA2508280 = {S5412, 132, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B142Safari/8536.25*/
const Device UA2510862 = {S5413, 132, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141Safari/8536.25*/
const Device UA2515239 = {S5414, 132, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA2515454 = {S5415, 135, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B311 Safari/8536.25*/
const Device UA2516358 = {S5416, 132, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/9Z999 Safari/8536.25*/
const Device UA2516519 = {S5417, 135, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B145 Safari/8536.25*/
const Device UA2529621 = {S5418, 133, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2531508 = {S5419, 134, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B145 Safari/8536.25*/
const Device UA2532929 = {S5420, 135, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2533139 = {S5421, 134, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146Safari/8536.25*/
const Device UA2533742 = {S5422, 134, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146 Safari/8536.25*/
const Device UA2534907 = {S5423, 135, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146 Safari/8536.25*/
const Device UA2535114 = {S5424, 134, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B145Safari/8536.25*/
const Device UA2536315 = {S5425, 145, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/6.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA2537066 = {S5426, 135, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B318 Safari/8536.25*/
const Device UA2538607 = {S5427, 135, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2565895 = {S5428, 134, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/9Z999 Safari/8536.25*/
const Device UA2583702 = {S5429, 135, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329 Safari/8536.25*/
const Device UA2585384 = {S5430, 135, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2596544 = {S5431, 133, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B134 Safari/8536.25*/
const Device UA2603831 = {S5432, 135, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329 Safari/8536.25*/
const Device UA2606145 = {S5433, 134, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329Safari/8536.25*/
const Device UA2607196 = {S5434, 134, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329 Safari/8536.25*/
const Device UA2705904 = {S5435, 133, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146 Safari/8536.25*/
const Device UA2710027 = {S5436, 135, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2713008 = {S5437, 135, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B350 Safari/8536.25*/
const Device UA2739597 = {S5438, 134, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B350Safari/8536.25*/
const Device UA2752989 = {S5439, 134, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B350 Safari/8536.25*/
const Device UA2845971 = {S5440, 133, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B131 Safari/8536.25*/
const Device UA2855789 = {S5441, 134, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/9Z999 Safari/8536.25*/
const Device UA2858258 = {S5442, 135, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B143 Safari/8536.25*/
const Device UA2884017 = {S5443, 135, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA2895193 = {S5444, 135, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA2958131 = {S5445, 135, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA2991711 = {S5446, 135, { &P12280,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146 Safari/8536.25*/
const Device UA23660 = {S5447, 27, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; en-us;)*/
const Device UA92406 = {S5448, 54, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/4.61 [en] (X11; U; ) - BrowseX (2.0.0 Windows)*/
const Device UA93056 = {S5449, 91, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - curl/7.15.5 (x86_64-redhat-linux-gnu) libcurl/7.15.5 OpenSSL/0.9.8b zlib/1.2.3 libidn/0.6.5*/
const Device UA93469 = {S5450, 110, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-us) AppleWebKit/525.1+ (KHTML, like Gecko, Safari/525.1+) pythumbnail.py*/
const Device UA105833 = {S5451, 89, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - curl/7.15.5 (i686-redhat-linux-gnu) libcurl/7.15.5 OpenSSL/0.9.8b zlib/1.2.3 libidn/0.6.5*/
const Device UA114878 = {S5452, 88, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - curl/7.19.7 (x86_64-pc-linux-gnu) libcurl/7.19.7 OpenSSL/0.9.8k zlib/1.2.3.3 libidn/1.15*/
const Device UA124434 = {S5453, 44, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/3.01Gold (X11; I; Linux 2.0.32 i486)*/
const Device UA137730 = {S5454, 86, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - curl/7.19.7 (i486-pc-linux-gnu) libcurl/7.19.7 OpenSSL/0.9.8k zlib/1.2.3.3 libidn/1.15*/
const Device UA138768 = {S5455, 85, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux; en-US) AppleWebKit/531.2 (KHTML, like Gecko) Safari/531.2*/
const Device UA148246 = {S5456, 46, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/4.5b1 [en] (X11; I; Linux 2.0.35 i586)*/
const Device UA151900 = {S5457, 52, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/3.01 (compatible; Netbox/3.5 R92; Linux 2.2)*/
const Device UA151907 = {S5458, 80, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/4.72C-CCK-MCD Caldera Systems OpenLinux [en] (X11; U; Linux 2.2.14 i686)*/
const Device UA152288 = {S5459, 37, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Links (2.xpre7; Linux 2.4.18 i586; x)*/
const Device UA152328 = {S5460, 47, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Links (0.9xpre12; Linux 2.2.14-5.0 i686; 80x24)*/
const Device UA164155 = {S5461, 104, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; de-DE) AppleWebKit/534.3 (KHTML, like Gecko) WeTab-Browser Safari/534.3*/
const Device UA167009 = {S5462, 103, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Arora/0.10.2 Safari/533.3*/
const Device UA167214 = {S5463, 85, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux; ja-JP) AppleWebKit/527+ (KHTML, like Gecko, Safari/419.3)*/
const Device UA167655 = {S5464, 52, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/3.2; Linux 2.6.2)*/
const Device UA171130 = {S5465, 39, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US)*/
const Device UA174844 = {S5466, 50, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/3.1; Linux; en)*/
const Device UA178246 = {S5467, 95, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; U; Linux 35230; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+*/
const Device UA195975 = {S5468, 99, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Boxee/1.0.1 bxapi/7.1*/
const Device UA196090 = {S5469, 94, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; undefined) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.7*/
const Device UA206154 = {S5470, 108, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Epiphany/2.30.6 Safari/534.7*/
const Device UA213462 = {S5471, 105, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - curl/7.21.2 (x86_64-unknown-linux-gnu) libcurl/7.21.2 OpenSSL/1.0.0c zlib/1.2.5 libidn/1.15 libssh2/1.2.7*/
const Device UA232013 = {S5472, 41, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Infoseek SideWinder/2.0B (Linux 2.4 i686)*/
const Device UA232959 = {S5473, 102, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - curl/7.21.0 (x86_64-pc-linux-gnu) libcurl/7.21.0 OpenSSL/0.9.8o zlib/1.2.3.4 libidn/1.15 libssh2/1.2.6*/
const Device UA238811 = {S5474, 121, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; webOS/2.2.0; U; en-GB) AppleWebKit/534.6 (KHTML, like Gecko) webOSBrowser/221.54 Safari/534.6 Pre/3.0*/
const Device UA239195 = {S5475, 121, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; webOS/2.2.3; U; en-US) AppleWebKit/534.6 (KHTML, like Gecko) webOSBrowser/221.54 Safari/534.6 Pre/3.0*/
const Device UA250685 = {S5476, 95, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) FbxQmlTV Safari/534.34*/
const Device UA253804 = {S5477, 66, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/3.3; Linux) (KHTML, like Gecko)*/
const Device UA299204 = {S5478, 99, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; c) AppleWebKit/528.5+ (KHTML, like Gecko, Safari/528.5+) <unknown>*/
const Device UA300341 = {S5479, 45, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/4.73 [en] (X11; U; Linux 2.2.15 i686)*/
const Device UA301338 = {S5480, 89, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (U; Linux) AppleWebKit/533.3 (KHTML, like Gecko) konqueror/4.6.0 Safari/533.3*/
const Device UA319262 = {S5481, 90, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.6*/
const Device UA327586 = {S5482, 121, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; webOS/2.2.0; U; en-US) AppleWebKit/534.6 (KHTML, like Gecko) webOSBrowser/221.53 Safari/534.6 Pre/3.0*/
const Device UA331310 = {S5483, 145, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ Debian/sid/experimental (3.0.4-1) Epiphany/3.0.4*/
const Device UA348036 = {S5484, 47, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) PrintFriendly.com*/
const Device UA350539 = {S5485, 143, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.4+ (KHTML, like Gecko) Version/5.0 Safari/535.4+ Debian/sid/experimental (3.2.1-2) Epiphany/3.2.1*/
const Device UA352126 = {S5486, 121, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; webOS/2.2.4; U; en-GB) AppleWebKit/534.6 (KHTML, like Gecko) webOSBrowser/221.56 Safari/534.6 Pre/1.2*/
const Device UA359505 = {S5487, 121, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; webOS/2.2.4; U; en-CA) AppleWebKit/534.6 (KHTML, like Gecko) webOSBrowser/221.56 Safari/534.6 Pre/1.2*/
const Device UA363613 = {S5488, 121, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; webOS/2.2.4; U; en-GB) AppleWebKit/534.6 (KHTML, like Gecko) webOSBrowser/221.56 Safari/534.6 Pre/3.0*/
const Device UA377045 = {S5489, 100, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - curl/7.21.0 (i486-pc-linux-gnu) libcurl/7.21.0 OpenSSL/0.9.8o zlib/1.2.3.4 libidn/1.15 libssh2/1.2.6*/
const Device UA390194 = {S5490, 85, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - curl/7.15.1 (x86_64-suse-linux) libcurl/7.15.1 OpenSSL/0.9.8a zlib/1.2.3 libidn/0.6.0*/
const Device UA437096 = {S5491, 103, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - curl/7.19.7 (x86_64-unknown-linux-gnu) libcurl/7.19.7 NSS/3.12.6.2 zlib/1.2.3 libidn/1.18 libssh2/1.2.2*/
const Device UA454771 = {S5492, 102, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - curl/7.19.7 (x86_64-redhat-linux-gnu) libcurl/7.19.7 NSS/3.13.1.0 zlib/1.2.3 libidn/1.18 libssh2/1.2.2*/
const Device UA471570 = {S5493, 54, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 Galeon/1.2.0 (X11; Linux i686; U;) Gecko/X*/
const Device UA628413 = {S5494, 88, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; ) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.1*/
const Device UA628960 = {S5495, 93, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/533.3 (KHTML, like Gecko) kded/4.7.2 Safari/533.3*/
const Device UA633079 = {S5496, 93, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) imgsizer Safari/534.34*/
const Device UA664501 = {S5497, 100, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - curl/7.21.6 (x86_64-pc-linux-gnu) libcurl/7.21.6 OpenSSL/1.0.0e zlib/1.2.3.4 libidn/1.22 librtmp/2.3*/
const Device UA694889 = {S5498, 99, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - curl/7.22.0 (x86_64-pc-linux-gnu) libcurl/7.22.0 OpenSSL/1.0.1 zlib/1.2.3.4 libidn/1.23 librtmp/2.3*/
const Device UA701808 = {S5499, 94, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; undefined) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.5*/
const Device UA901236 = {S5500, 40, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US;)*/
const Device UA1979458 = {S5501, 97, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - curl/7.22.0 (i686-pc-linux-gnu) libcurl/7.22.0 OpenSSL/1.0.1 zlib/1.2.3.4 libidn/1.23 librtmp/2.3*/
const Device UA2014088 = {S5502, 69, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) Superfish/1.1 (platform image loader)*/
const Device UA2364025 = {S5503, 101, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - curl/7.19.4 (i386-redhat-linux-gnu) libcurl/7.19.4 NSS/3.12.2.0 zlib/1.2.3 libidn/0.6.14 libssh2/0.18*/
const Device UA2398169 = {S5504, 93, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/533.3 (KHTML, like Gecko) kded/4.8.5 Safari/533.3*/
const Device UA2404703 = {S5505, 59, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - python-requests/0.14.2 CPython/2.7.3 Linux/3.2.0-33-generic*/
const Device UA2408166 = {S5506, 59, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - python-requests/0.14.2 CPython/2.7.3 Linux/3.2.0-34-generic*/
const Device UA2426108 = {S5507, 94, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:7.0.1) Gecko/ /7.0.1,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2439856 = {S5508, 57, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0+(X11;+U;+Linux+i686;+en-US;+rv:1.7.3)+Gecko/X*/
const Device UA2487515 = {S5509, 58, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - python-requests/1.1.0 CPython/2.7.3 Linux/3.2.0-36-virtual*/
const Device UA2518231 = {S5510, 95, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) kded/4.8.4 Safari/534.34*/
const Device UA2537624 = {S5511, 72, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux; en-US) AppleWebKit/532.4 (KHTML, like Gecko)*/
const Device UA2570292 = {S5512, 43, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/4_77 [en] (X11; I; IRIX;64 6_5 IP30*/
const Device UA2580583 = {S5513, 58, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - python-requests/1.1.0 CPython/2.7.3 Linux/3.2.0-34-generic*/
const Device UA2654327 = {S5514, 73, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - python-requests/0.14.2 CPython/2.7.2+ Linux/3.2.13-grsec-xxxx-grs-ipv6-64*/
const Device UA2672796 = {S5515, 56, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - python-requests/1.1.0 CPython/2.6.5 Linux/2.6.32-346-ec2*/
const Device UA2674574 = {S5516, 58, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - python-requests/1.2.0 CPython/2.7.3 Linux/3.2.0-29-virtual*/
const Device UA2696359 = {S5517, 95, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) kded/4.9.4 Safari/534.34*/
const Device UA2704182 = {S5518, 102, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - curl/7.19.7 (x86_64-redhat-linux-gnu) libcurl/7.19.7 NSS/3.14.0.0 zlib/1.2.3 libidn/1.18 libssh2/1.4.2*/
const Device UA2717130 = {S5519, 60, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - python-requests/1.0.4 CPython/2.7.3 Linux/3.2.37.1.amd64-smp*/
const Device UA2740575 = {S5520, 107, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.11 (KHTML, like Gecko) DumpRenderTree/0.0.0.0 Safari/536.11*/
const Device UA2740619 = {S5521, 58, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - python-requests/1.2.0 CPython/2.7.4 Linux/3.8.0-21-generic*/
const Device UA2756425 = {S5522, 63, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - python-requests/1.2.3 CPython/2.7.3 Linux/3.9.3-x86_64-linode33*/
const Device UA2770587 = {S5523, 93, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.21 (KHTML, like Gecko) imgsizer Safari/537.21*/
const Device UA2874771 = {S5524, 96, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) kded/4.10.4 Safari/534.34*/
const Device UA2894308 = {S5525, 59, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; zh-CN; rv:1.9.1.2) Gecko/X*/
const Device UA2900781 = {S5526, 61, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - python-requests/1.2.3 CPython/2.6.6 Linux/3.2.0-0.bpo.4-amd64*/
const Device UA2996072 = {S5527, 58, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - python-requests/1.1.0 CPython/2.7.4 Linux/3.8.0-19-generic*/
const Device UA3018859 = {S5528, 55, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - python-requests/1.2.3 CPython/2.7.3 Linux/3.2.0-4-amd64*/
const Device UA3034065 = {S5529, 53, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - python-requests/1.2.3 CPython/2.7.3 Linux/3.3.8-gcg-X*/
const Device UA3084037 = {S5530, 67, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - python-requests/1.2.3 CPython/2.6.8 Linux/3.4.43-43.43.amzn1.x86_64*/
const Device UA3092330 = {S5531, 58, { &P15364,&P18110,&P17470,&P18092 }};	/*Useragent - python-requests/1.2.3 CPython/2.7.3 Linux/3.5.0-23-generic*/
const Device UA3115137 = {S5532, 88, { &P18095,&P18109,&P18328,&P18092 }};	/*Useragent - Opera/9.80 (Series 60; Opera Mini/7.1.32444/31.1475; U; en) Presto/2.8.119 Version/11.10*/
const Device UA92160 = {S5533, 97, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/416.11 (KHTML, like Gecko) Safari/416.12*/
const Device UA96766 = {S5534, 95, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/412.7 (KHTML, like Gecko) Safari/412.5*/
const Device UA98086 = {S5535, 95, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/312.8 (KHTML, like Gecko) Safari/312.5*/
const Device UA100821 = {S5536, 95, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/418 (KHTML, like Gecko) Safari/417.9.2*/
const Device UA104154 = {S5537, 95, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/312.9 (KHTML, like Gecko) Safari/312.6*/
const Device UA105700 = {S5538, 95, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/125.4 (KHTML, like Gecko) Safari/125.9*/
const Device UA114239 = {S5539, 93, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/124 (KHTML, like Gecko) Safari/125.1*/
const Device UA115423 = {S5540, 96, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/85.7 (KHTML, like Gecko) Safari/85.5*/
const Device UA119647 = {S5541, 95, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/417.9 (KHTML, like Gecko) Safari/417.8*/
const Device UA125109 = {S5542, 95, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/125.2 (KHTML, like Gecko) Safari/125.8*/
const Device UA130986 = {S5543, 51, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/12.0.0*/
const Device UA142716 = {S5544, 97, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/521.25 (KHTML, like Gecko) Safari/521.24*/
const Device UA143529 = {S5545, 98, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/125.5.5 (KHTML, like Gecko) Safari/125.11*/
const Device UA151362 = {S5546, 52, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) Excel/14.0.0*/
const Device UA152418 = {S5547, 95, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/106.2 (KHTML, like Gecko) Safari/100.1*/
const Device UA152591 = {S5548, 51, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/14.0.0*/
const Device UA157926 = {S5549, 91, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/124 (KHTML, like Gecko) Safari/125*/
const Device UA164692 = {S5550, 98, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/522.11 (KHTML, like Gecko) Safari/3.0.2*/
const Device UA171163 = {S5551, 88, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/523.10.3 (KHTML, like Gecko)*/
const Device UA180149 = {S5552, 52, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/14.10.0*/
const Device UA205629 = {S5553, 97, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.7*/
const Device UA230936 = {S5554, 96, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/419 (KHTML, like Gecko) NetNewsWire/3.0*/
const Device UA232087 = {S5555, 99, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.7.1*/
const Device UA244126 = {S5556, 55, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Pulp/2.1.2 (Mac OS X; http://www.acrylicapps.com/pulp/)*/
const Device UA249543 = {S5557, 100, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.0*/
const Device UA271463 = {S5558, 98, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; fr-fr) AppleWebKit/312.5 (KHTML, like Gecko) Safari/312.3*/
const Device UA274289 = {S5559, 95, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/412 (KHTML, like Gecko) Pandora/2.0*/
const Device UA274734 = {S5560, 95, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/412.6 (KHTML, like Gecko) Safari/412.2*/
const Device UA275086 = {S5561, 97, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/417.3 (KHTML, like Gecko) Safari/417.2*/
const Device UA275699 = {S5562, 97, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/416.12 (KHTML, like Gecko) Safari/416.13*/
const Device UA298795 = {S5563, 101, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/417.9 (KHTML, like Gecko) HistoryHound/1.9*/
const Device UA300381 = {S5564, 98, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/125.5.7 (KHTML, like Gecko) Safari/125.12*/
const Device UA374018 = {S5565, 99, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.5.1*/
const Device UA391473 = {S5566, 97, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.6*/
const Device UA440331 = {S5567, 55, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Pulp/2.0.1 (Mac OS X; http://www.acrylicapps.com/pulp/)*/
const Device UA449968 = {S5568, 53, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Pulp/2.0 (Mac OS X; http://www.acrylicapps.com/pulp/)*/
const Device UA452137 = {S5569, 53, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Pulp/2.1 (Mac OS X; http://www.acrylicapps.com/pulp/)*/
const Device UA478713 = {S5570, 98, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; de-de) AppleWebKit/125.2 (KHTML, like Gecko) Safari/125.7*/
const Device UA487605 = {S5571, 106, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/523.12 (KHTML, like Gecko, Safari/523.12) ADM/722*/
const Device UA505278 = {S5572, 100, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/526.9+ (KHTML, like Gecko) AdobeAIR/1.5.3*/
const Device UA509178 = {S5573, 96, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/85.7 (KHTML, like Gecko) Safari/85.7*/
const Device UA543888 = {S5574, 99, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.0.2*/
const Device UA570560 = {S5575, 96, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X;en) AppleWebKit/419.2.1 (KHTML, like Gecko) Safari/419.4*/
const Device UA595788 = {S5576, 100, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.2*/
const Device UA628410 = {S5577, 52, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/14.20.0*/
const Device UA635754 = {S5578, 100, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.1*/
const Device UA665464 = {S5579, 94, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/125.2 (KHTML, like Gecko) Safari/85.8*/
const Device UA669054 = {S5580, 95, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/412 (KHTML, like Gecko) Pandora/2.3*/
const Device UA670625 = {S5581, 114, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) VIDEODOWNLOAD Safari/533.3*/
const Device UA677879 = {S5582, 77, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; pl-PL; rv:1.0.1) Gecko/X Chimera/0.6*/
const Device UA794774 = {S5583, 83, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.5) Gecko/X Firebird/0.7*/
const Device UA822984 = {S5584, 103, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/521.32.1 (KHTML, like Gecko) Safari/521.32.1*/
const Device UA869435 = {S5585, 96, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/85.7 (KHTML, like Gecko) Safari/85.6*/
const Device UA869988 = {S5586, 98, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/412 (KHTML, like Gecko) Skyscraper/2.0*/
const Device UA878183 = {S5587, 99, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/125.5.5 (KHTML, like Gecko) Safari/125.5.5*/
const Device UA900996 = {S5588, 50, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Pulp/2.5.1 (Mac; http://www.acrylicapps.com/pulp/)*/
const Device UA902578 = {S5589, 100, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.3*/
const Device UA912854 = {S5590, 52, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/14.23.0*/
const Device UA917824 = {S5591, 95, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/412.7 (KHTML, like Gecko) Safari/412.6*/
const Device UA1892886 = {S5592, 108, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/125.5.7 (KHTML, like Gecko) SunriseBrowser/0.853*/
const Device UA2004710 = {S5593, 100, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.4*/
const Device UA2200481 = {S5594, 52, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/14.24.0*/
const Device UA2205247 = {S5595, 101, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/523.15 (KHTML, like Gecko) Safari/419.3 Qt*/
const Device UA2215225 = {S5596, 95, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-US) AppleWebKit/85 (KHTML, like Gecko) OmniWeb/v496*/
const Device UA2219978 = {S5597, 98, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-US) AppleWebKit/85 (KHTML, like Gecko) OmniWeb/v558.48*/
const Device UA2258542 = {S5598, 108, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.6.1 Safari/534.34*/
const Device UA2258559 = {S5599, 107, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) AppleWebKit/534.34 (KHTML, like Gecko) mention/1.1.24 Safari/534.34*/
const Device UA2263829 = {S5600, 95, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/417.2 (KHTML, like Gecko) Safari/417.1*/
const Device UA2279760 = {S5601, 119, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/418.9 (KHTML, like Gecko) AppleWebKit/418.9 Cheshire/1.0.ALPHA*/
const Device UA2357257 = {S5602, 100, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.5*/
const Device UA2362773 = {S5603, 52, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/14.25.0*/
const Device UA2362799 = {S5604, 52, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/12.35.0*/
const Device UA2383578 = {S5605, 108, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.7.0 Safari/534.34*/
const Device UA2494887 = {S5606, 97, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/521.9 (KHTML, like Gecko) Safari/521.9*/
const Device UA2511361 = {S5607, 50, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Pulp/2.5.3 (Mac; http://www.acrylicapps.com/pulp/)*/
const Device UA2513939 = {S5608, 52, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/14.31.0*/
const Device UA2534136 = {S5609, 100, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.6*/
const Device UA2568161 = {S5610, 52, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/14.32.0*/
const Device UA2569989 = {S5611, 52, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/12.36.0*/
const Device UA2577631 = {S5612, 103, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) AppleWebKit/535.14 (KHTML, like Gecko) JavaFX/2.2 Safari/535.14*/
const Device UA2678836 = {S5613, 100, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.7*/
const Device UA2719021 = {S5614, 52, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/14.34.0*/
const Device UA2796733 = {S5615, 52, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/14.35.0*/
const Device UA2833365 = {S5616, 99, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/728.18 (KHTML, like Gecko) Safari/728.18*/
const Device UA2972570 = {S5617, 52, { &P17595,&P18120,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) Word/14.36.0*/
const Device UA25416 = {S5618, 111, { &P15364,&P17285,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.13) Gecko/20101203 Firefox/3.6.13 (.NET CLR 3.5.30729)*/
const Device UA499753 = {S5619, 73, { &P15364,&P17285,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Windows NT 6.0; en-US; rv:1.9.2) Gecko/X Firefox/3.6*/
const Device UA551246 = {S5620, 249, { &P15364,&P17285,&P17558,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; User-agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)(Mozilla/5.0 (Windows; U; Windows NT 6.0; nl; rv:1.9.2.6) Gecko/X Firefox/3.6.6); SLCC2; .NET CLR 2.0.50727; .N*/
const Device UA575177 = {S5621, 131, { &P15364,&P17285,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-GB; rv:1.9.2.3) Gecko/X Firefox/3.6.3 ( .NET CLR 3.5.30729; .NET4.0C) WinTSI 25.04.2010*/
const Device UA2319158 = {S5622, 125, { &P15364,&P17285,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; fr; rv:1.9.2.20) Gecko/X Firefox/3.6.20 (Windows NT 5.1; rv:5.0) Gecko/X Firefox/5.0*/
const Device UA2370121 = {S5623, 75, { &P15364,&P17285,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0(Windows;U;Windows NT 6.0;en-US;rv:1.9.2.8)Gecko/X Firefox/3.6.8*/
const Device UA2675270 = {S5624, 134, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/27.0.1453.15) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.15 Safari/537.36*/
const Device UA2675365 = {S5625, 127, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/27.0.1453.47) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.47 Safari/537.36*/
const Device UA2678739 = {S5626, 134, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/27.0.1453.47) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.47 Safari/537.36*/
const Device UA2680856 = {S5627, 134, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/27.0.1453.56) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.56 Safari/537.36*/
const Device UA2707659 = {S5628, 127, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/27.0.1453.56) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.56 Safari/537.36*/
const Device UA2714475 = {S5629, 127, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/27.0.1453.65) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.65 Safari/537.36*/
const Device UA2715648 = {S5630, 134, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/27.0.1453.73) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.73 Safari/537.36*/
const Device UA2718664 = {S5631, 127, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/27.0.1453.73) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.73 Safari/537.36*/
const Device UA2719038 = {S5632, 134, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/27.0.1453.65) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.65 Safari/537.36*/
const Device UA2738369 = {S5633, 134, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/27.0.1453.81) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.81 Safari/537.36*/
const Device UA2740088 = {S5634, 127, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/27.0.1453.81) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.81 Safari/537.36*/
const Device UA2741586 = {S5635, 127, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/27.0.1453.93) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36*/
const Device UA2741687 = {S5636, 134, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/27.0.1453.93) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36*/
const Device UA2756910 = {S5637, 216, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/27.0.1453.94; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.2)*/
const Device UA2756972 = {S5638, 134, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/27.0.1453.94) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36*/
const Device UA2757023 = {S5639, 204, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/27.0.1453.94; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2757025 = {S5640, 127, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/27.0.1453.94) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36*/
const Device UA2770957 = {S5641, 209, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/27.0.1453.94; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2774237 = {S5642, 129, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/27.0.1453.110) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36*/
const Device UA2774255 = {S5643, 136, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/27.0.1453.110) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36*/
const Device UA2860949 = {S5644, 127, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/27.0.1453.15) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.15 Safari/537.36*/
const Device UA2873651 = {S5645, 90, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/27.0.1453.116)*/
const Device UA2873649 = {S5646, 97, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/27.0.1453.116)*/
const Device UA2873684 = {S5647, 98, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/27.0.1453.116)*/
const Device UA2873854 = {S5648, 136, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/27.0.1453.116) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2873921 = {S5649, 91, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/27.0.1453.116)*/
const Device UA2874062 = {S5650, 129, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/27.0.1453.116) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2874158 = {S5651, 198, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/27.0.1453.116; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2874160 = {S5652, 198, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/27.0.1453.116; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2874594 = {S5653, 184, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/27.0.1453.116; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2874657 = {S5654, 213, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/27.0.1453.116; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA2875740 = {S5655, 177, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/27.0.1453.116; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2876227 = {S5656, 192, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/27.0.1453.116; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA2876996 = {S5657, 184, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/27.0.1453.116; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2884932 = {S5658, 213, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/27.0.1453.116; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA2885557 = {S5659, 177, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/27.0.1453.116; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2940459 = {S5660, 270, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/27.0.1453.116; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2941790 = {S5661, 225, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/27.0.1453.116; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA2958222 = {S5662, 269, { &P15364,&P7133,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/27.0.1453.94; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA5625 = {S5663, 148, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F190 Safari/6533.18.5*/
const Device UA5637 = {S5664, 148, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8G4 Safari/6533.18.5*/
const Device UA5689 = {S5665, 150, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9Z999 Safari/6533.18.5*/
const Device UA5730 = {S5666, 148, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5*/
const Device UA37159 = {S5667, 148, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA46546 = {S5668, 149, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C99 Safari/6533.18.5*/
const Device UA46548 = {S5669, 147, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C99 Safari/6533.18.5*/
const Device UA46551 = {S5670, 149, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C99 Safari/6533.18.5*/
const Device UA46550 = {S5671, 149, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C99 Safari/6533.18.5*/
const Device UA46561 = {S5672, 148, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8K2 Safari/6533.18.5*/
const Device UA175803 = {S5673, 148, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5*/
const Device UA185626 = {S5674, 158, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone Simulator; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F192 Safari/6533.18.5*/
const Device UA192004 = {S5675, 148, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J3 Safari/6533.18.5*/
const Device UA193972 = {S5676, 148, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (device; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5*/
const Device UA200580 = {S5677, 142, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5*/
const Device UA202540 = {S5678, 150, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9Z999 Safari/6533.18.5*/
const Device UA315124 = {S5679, 142, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8G4 Safari/6533.18.5*/
const Device UA316578 = {S5680, 142, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5*/
const Device UA316743 = {S5681, 150, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9Z999 Safari/6533.18.5*/
const Device UA318720 = {S5682, 148, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5*/
const Device UA448621 = {S5683, 148, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (device; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8G4 Safari/6533.18.5*/
const Device UA555815 = {S5684, 148, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5*/
const Device UA692547 = {S5685, 152, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en_us) AppleWebKit/6533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5*/
const Device UA795188 = {S5686, 137, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; 6533.17.9) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5*/
const Device UA870565 = {S5687, 150, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F5153d Safari/6533.18.5*/
const Device UA871896 = {S5688, 152, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en_us) AppleWebKit/6533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5*/
const Device UA1894766 = {S5689, 148, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8K2 Safari/6533.18.5*/
const Device UA1985355 = {S5690, 137, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; 6533.17.9) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5*/
const Device UA2141173 = {S5691, 150, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA2214030 = {S5692, 129, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 4_3_3 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0 Mobile/8J2 Safari/7533.17.9*/
const Device UA2294565 = {S5693, 148, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA2472702 = {S5694, 150, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; es-es) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA2505243 = {S5695, 145, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5*/
const Device UA2658354 = {S5696, 146, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 43 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8F190 Safari/6533.18.5*/
const Device UA2658460 = {S5697, 145, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 433 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5*/
const Device UA2659275 = {S5698, 145, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 435 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA2660239 = {S5699, 145, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 432 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5*/
const Device UA2663477 = {S5700, 145, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 434 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8K2 Safari/6533.18.5*/
const Device UA2711580 = {S5701, 145, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 431 like Mac OS X; es-US) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8G4 Safari/6533.18.5*/
const Device UA2720656 = {S5702, 150, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA2800705 = {S5703, 151, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.02 Guidebook/39 Mobile/8L1 Safari/6533.18.5*/
const Device UA2980918 = {S5704, 148, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iphone; U; CPU iPhone OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5*/
const Device UA3041022 = {S5705, 148, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5*/
const Device UA3041423 = {S5706, 149, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5*/
const Device UA3071022 = {S5707, 151, { &P12280,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; es-es) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5*/
const Device UA178471 = {S5708, 123, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7; en-us) AppleWebKit/534.20.8 (KHTML, like Gecko) Version/5.1 Safari/534.20.8*/
const Device UA193033 = {S5709, 113, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7) AppleWebKit/534.31.2 (KHTML, like Gecko) Version/5.1 Safari/534.31.2*/
const Device UA195096 = {S5710, 109, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7) AppleWebKit/534.42 (KHTML, like Gecko) Version/5.1 Safari/534.42*/
const Device UA196539 = {S5711, 113, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7) AppleWebKit/534.34.4 (KHTML, like Gecko) Version/5.1 Safari/534.34.4*/
const Device UA202832 = {S5712, 109, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Safari/534.45*/
const Device UA212954 = {S5713, 113, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3*/
const Device UA222639 = {S5714, 113, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/534.49.1 (KHTML, like Gecko) Version/5.1 Safari/534.49*/
const Device UA249423 = {S5715, 119, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22*/
const Device UA250907 = {S5716, 125, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/534.51.22 (KHTML, like Gecko) WebClip/7534.51.13 Safari/7534.51.22*/
const Device UA252684 = {S5717, 117, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7*/
const Device UA310579 = {S5718, 113, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7) AppleWebKit/534.26.4 (KHTML, like Gecko) Version/5.1 Safari/534.26.4*/
const Device UA311882 = {S5719, 113, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7) AppleWebKit/534.28.5 (KHTML, like Gecko) Version/5.1 Safari/534.28.5*/
const Device UA337004 = {S5720, 118, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7) AppleWebKit/534.48.3 (KHTML, like Gecko) WebClip/7534.48 Safari/7534.48.3*/
const Device UA347155 = {S5721, 113, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1 Safari/534.50*/
const Device UA360420 = {S5722, 119, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.53.11 (KHTML, like Gecko) Version/5.1.3 Safari/534.53.10*/
const Device UA380830 = {S5723, 124, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.53.11 (KHTML, like Gecko) WebClip/7534.53.1 Safari/7534.53.10*/
const Device UA428874 = {S5724, 125, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.53.11 (KHTML, like Gecko) WebClip/7534.54.15 Safari/7534.54.16*/
const Device UA440726 = {S5725, 117, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3*/
const Device UA623001 = {S5726, 117, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) Version/5.1.6 Safari/534.56.5*/
const Device UA623172 = {S5727, 119, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16*/
const Device UA626575 = {S5728, 123, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.53.11 (KHTML, like Gecko) WebClip/7534.55.2 Safari/7534.55.3*/
const Device UA626628 = {S5729, 123, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/534.52.7 (KHTML, like Gecko) WebClip/7534.51.13 Safari/7534.52.7*/
const Device UA666928 = {S5730, 111, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 Macintosh Intel Mac OS X 10_7_2 AppleWebKit/534.52.7 KHTML,like Gecko Version/5.1.2 Safari/534.52.7*/
const Device UA669107 = {S5731, 109, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7) AppleWebKit/534.47 (KHTML, like Gecko) Version/5.1 Safari/534.47*/
const Device UA678500 = {S5732, 122, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) WebClip/7534.56.5 Safari/7534.56.5*/
const Device UA699917 = {S5733, 145, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7.3) AppleWebKit/7534.53.11 (KHTML, like Gecko) Version/5.1.3 Safari/7534.53.10 Raven for Mac/0.7.14612*/
const Device UA877381 = {S5734, 144, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7.3) AppleWebKit/7534.53.11 (KHTML, like Gecko) Version/5.1.5 Safari/7534.55.3 Raven for Mac/0.7.14665*/
const Device UA901370 = {S5735, 113, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1.5 Safari/534.46*/
const Device UA1962238 = {S5736, 143, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7.4) AppleWebKit/7534.56.5 (KHTML, like Gecko) Version/5.1.7 Safari/7534.57.2 Raven for Mac/0.7.14665*/
const Device UA1995596 = {S5737, 117, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.7*/
const Device UA2193781 = {S5738, 122, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) WebClip/7534.57.2 Safari/7534.57.7*/
const Device UA2273313 = {S5739, 125, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7; en-us) AppleWebKit/534.15.15 (KHTML, like Gecko) Version/5.1 Safari/534.15.16*/
const Device UA2287465 = {S5740, 109, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7) AppleWebKit/534.43 (KHTML, like Gecko) Version/5.1 Safari/534.43*/
const Device UA2574734 = {S5741, 121, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10\_7\_2) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22*/
const Device UA2597767 = {S5742, 120, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 107) AppleWebKit/534.48.3 (KHTML like Gecko) Version/5.1 Safari/534.48.3gzip(gfe)*/
const Device UA2644351 = {S5743, 119, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10\_7\_2) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7*/
const Device UA2707146 = {S5744, 117, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.1.8 Safari/534.58.2*/
const Device UA2795638 = {S5745, 117, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.1.9 Safari/534.59.8*/
const Device UA2880140 = {S5746, 139, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Digg Feed Fetcher 1.0 (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)*/
const Device UA2896172 = {S5747, 134, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Digg Fetcher 1.0 (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)*/
const Device UA3019231 = {S5748, 143, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Betaworks Data Science 1.0 (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1)AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)*/
const Device UA3098496 = {S5749, 212, { &P17595,&P17297,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en) AppleWebKit/534.56.5 (KHTML, like Gecko) Version/5.1.6 Safari/534.56.5*/
const Device UA2487839 = {S5750, 132, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/26.0.1386.0) AppleWebKit/537.27 (KHTML, like Gecko) Chrome/26.0.1386.0 Safari/537.27*/
const Device UA2567171 = {S5751, 134, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/26.0.1410.19) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.19 Safari/537.31*/
const Device UA2568010 = {S5752, 134, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/26.0.1410.28) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.28 Safari/537.31*/
const Device UA2583605 = {S5753, 134, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/26.0.1410.33) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.33 Safari/537.31*/
const Device UA2600174 = {S5754, 134, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/26.0.1410.40) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Safari/537.31*/
const Device UA2601363 = {S5755, 127, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/26.0.1410.43) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/537.31*/
const Device UA2602790 = {S5756, 134, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/26.0.1410.43) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/537.31*/
const Device UA2670520 = {S5757, 183, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/26.0.1410.43; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2673319 = {S5758, 197, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2674022 = {S5759, 191, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA2674110 = {S5760, 183, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2674404 = {S5761, 229, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)*/
const Device UA2675987 = {S5762, 176, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2678399 = {S5763, 89, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/26.0.1410.64)*/
const Device UA2678405 = {S5764, 96, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/26.0.1410.64)*/
const Device UA2678406 = {S5765, 134, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/26.0.1410.64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2678700 = {S5766, 197, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2678827 = {S5767, 127, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/26.0.1410.64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2678878 = {S5768, 97, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/26.0.1410.64)*/
const Device UA2678882 = {S5769, 90, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/26.0.1410.64)*/
const Device UA2679015 = {S5770, 214, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Tablet PC 2.0)*/
const Device UA2715340 = {S5771, 214, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Tablet PC 2.0)*/
const Device UA2715532 = {S5772, 229, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)*/
const Device UA2715832 = {S5773, 209, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/26.0.1410.64; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2716399 = {S5774, 166, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)*/
const Device UA2716778 = {S5775, 112, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Avant Browser; chromeframe/26.0.1410.64)*/
const Device UA2716934 = {S5776, 224, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/26.0.1410.64; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA2717464 = {S5777, 257, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2735686 = {S5778, 226, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Zune 4.7)*/
const Device UA2736195 = {S5779, 188, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/26.0.1410.64; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2857677 = {S5780, 134, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/26.0.1410.12) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.12 Safari/537.31*/
const Device UA2860571 = {S5781, 262, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2861560 = {S5782, 252, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2936242 = {S5783, 282, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2993575 = {S5784, 262, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA3004542 = {S5785, 282, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6136; ms-office; MSOffice 14)*/
const Device UA3111316 = {S5786, 282, { &P15364,&P7133,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/26.0.1410.64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA2430886 = {S5787, 88, { &P18088,&P17017,&P18369,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/7.0.31438/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2799283 = {S5788, 59, { &P18088,&P17017,&P18369,&P18092 }};	/*Useragent - Opera/9.50 (J2ME/MIDP; Opera Mini/7.0.32351/29.3757; U; en)*/
const Device UA2875561 = {S5789, 82, { &P18088,&P17017,&P18369,&P18092 }};	/*Useragent - Opera/9.80 (MTK; Opera Mini/7.0.31243/30.3214; U; en) Presto/2.8.119 Version/11.10*/
const Device UA94673 = {S5790, 50, { &P15364,&P7127,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0)*/
const Device UA96501 = {S5791, 150, { &P15364,&P7127,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; SV1; .NET CLR 1.1.4322; InfoPath.2; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022)*/
const Device UA98524 = {S5792, 88, { &P15364,&P7127,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322)*/
const Device UA123447 = {S5793, 107, { &P15364,&P7127,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; QS 4.2.1.0; Windows 2000; .NET CLR 1.1.4322; InfoPath.1)*/
const Device UA132090 = {S5794, 50, { &P15364,&P7127,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 5.0)*/
const Device UA145535 = {S5795, 38, { &P15364,&P7127,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (MSIE 6.0; Windows NT 5.0)*/
const Device UA153517 = {S5796, 47, { &P15364,&P7127,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows; MSIE 6.0; Windows NT 5.0)*/
const Device UA385057 = {S5797, 50, { &P15364,&P7127,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible, MSIE 6.0, Windows NT 5.0)*/
const Device UA459769 = {S5798, 51, { &P15364,&P7127,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.01; Windows NT 5.0)*/
const Device UA641328 = {S5799, 55, { &P15364,&P7127,&P17191,&P18092 }};	/*Useragent - ChangeDetect/1.0 (compatible; MSIE 6.0; Windows NT 5.0)*/
const Device UA703314 = {S5800, 77, { &P15364,&P7127,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; WOW64; Trident/4.0; SLCC1)*/
const Device UA2410540 = {S5801, 151, { &P15364,&P7127,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA2518181 = {S5802, 63, { &P15364,&P7127,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 5.0; Trident/5.0)*/
const Device UA2743130 = {S5803, 178, { &P15364,&P7127,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA46283 = {S5804, 83, { &P15216,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry8520_CMCC/5.0.0.1056 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/204*/
const Device UA62952 = {S5805, 77, { &P15216,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry8520/5.0.0.592 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/168*/
const Device UA2641673 = {S5806, 57, { &P15216,&P7116,&P3507,&P18092 }};	/*Useragent - MQQBrowser/for Blackberry2.0.0 (BlackBerry8520/5.0.0.592)*/
const Device UA359568 = {S5807, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Full Android on Wingray Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA363240 = {S5808, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; I9000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA369773 = {S5809, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Ventana Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA600388 = {S5810, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; QW TB-1207 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA629680 = {S5811, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; XELIO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA640525 = {S5812, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; MID1200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA649305 = {S5813, 150, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Full AOSP on Maguro Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA649663 = {S5814, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PTM805HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA666317 = {S5815, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; PC677 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA691645 = {S5816, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; CRYM752HCF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA704951 = {S5817, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; msm8960 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA707628 = {S5818, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; asus_laptop Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA710076 = {S5819, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Q8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA723480 = {S5820, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Notion Ink ADAM Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA748594 = {S5821, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CMP745d Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA766820 = {S5822, 51, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4 Build/IMM76D)*/
const Device UA769312 = {S5823, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; mfld_pr3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA780201 = {S5824, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; imito am801 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA783070 = {S5825, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M760W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA789264 = {S5826, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; 97FC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA791123 = {S5827, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; DFP7005 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA794661 = {S5828, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; ALODM712HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA809074 = {S5829, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; RubiX ICS Infused Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA818875 = {S5830, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PTM723HR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA864900 = {S5831, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; generic_vortex Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA872237 = {S5832, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; Gpad Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA895002 = {S5833, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; NAM1006HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA896195 = {S5834, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; mfld_dv10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA896339 = {S5835, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Newsmy Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA898999 = {S5836, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A10-MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA899013 = {S5837, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexusone Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899616 = {S5838, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; V700B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA900396 = {S5839, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VegaCream_BETA-2.1 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA902027 = {S5840, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; PC657 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA902940 = {S5841, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M1001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA906680 = {S5842, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 3C97 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA907184 = {S5843, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN8BG3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA908853 = {S5844, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; EXLP2W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1936199 = {S5845, 154, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Full Android on x86 VirtualBox Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1952249 = {S5846, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; QW TB-1217 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1952823 = {S5847, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; M-MP711i Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1953076 = {S5848, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VegaCream Beta 3 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1953637 = {S5849, 51, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4 Build/IMM76I)*/
const Device UA1953673 = {S5850, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A10_MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1953945 = {S5851, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; RICATECH-DR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1954809 = {S5852, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; EVOTAB 2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1957822 = {S5853, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; EXLP2P Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1958412 = {S5854, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; MVPM727HCF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1960712 = {S5855, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; VISION Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1960750 = {S5856, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; i-Joy-Andromeda Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1961845 = {S5857, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; C906 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1961922 = {S5858, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; RK2918 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1962145 = {S5859, 51, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4 Build/IMM76L)*/
const Device UA1962988 = {S5860, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; AllviewSpeed2+ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1963061 = {S5861, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Informer 701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1963889 = {S5862, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M9701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1964715 = {S5863, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M701C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1966528 = {S5864, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; TAB-PROTAB2.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1966551 = {S5865, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; JOYPAD C71 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1967525 = {S5866, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; Tablet724W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1967823 = {S5867, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M711 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1967862 = {S5868, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; TechTAB Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1968437 = {S5869, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EPAD Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1970043 = {S5870, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; QPad-C-X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1970115 = {S5871, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-7200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1970432 = {S5872, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Notion Ink ADAM Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1975690 = {S5873, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TB782B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1975938 = {S5874, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; hu-hu; WayteQ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1976733 = {S5875, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M1001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1978295 = {S5876, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; Gpad Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1981455 = {S5877, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; hu-hu; HFM752HCF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1981545 = {S5878, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; Newpad-K97 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1984337 = {S5879, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; 1040C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1984510 = {S5880, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; H5500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1985273 = {S5881, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TRIO 7C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1988396 = {S5882, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; RK701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1988492 = {S5883, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A10MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1988592 = {S5884, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; SP-120 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1989333 = {S5885, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M760 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1989442 = {S5886, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; X7GS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1991990 = {S5887, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; msm8960 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1993970 = {S5888, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DFP7005B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1994179 = {S5889, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; NEXT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1994435 = {S5890, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SXZ-PDX0-01 Build/SXZ-PDX0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1994851 = {S5891, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; B920 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995081 = {S5892, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; DOMO_Slate_N9SE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1997219 = {S5893, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Newsmy P72-C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1998773 = {S5894, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; WP7001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2023387 = {S5895, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; amd_brazos Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2023712 = {S5896, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; TAB200PR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2023900 = {S5897, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; TOTEMPO-sun Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023956 = {S5898, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; 970 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2024095 = {S5899, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; CRYM752HC_C930 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2024303 = {S5900, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; PC1008R Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2024527 = {S5901, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AK-MINI-PC-001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2024625 = {S5902, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; QW TB-1210 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2024639 = {S5903, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TAB364 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2024739 = {S5904, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; G-PAD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2024778 = {S5905, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; ActiveTab Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2024792 = {S5906, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; B001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2024990 = {S5907, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Tablet PC T-702 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2025021 = {S5908, 72, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Full AOSP on Maguro Build/IML74K)*/
const Device UA2025104 = {S5909, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M1005 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2025197 = {S5910, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; T-90 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2025206 = {S5911, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A721 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2025365 = {S5912, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; WT-M768 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2025505 = {S5913, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; D70W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2025719 = {S5914, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; wetab Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2025720 = {S5915, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Broncho N802 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2025788 = {S5916, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Allwinner A10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2025815 = {S5917, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID-A10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2025820 = {S5918, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Newpad Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2025861 = {S5919, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; Z5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2025884 = {S5920, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A1001T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2025926 = {S5921, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; NVSBL DRAGON Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026079 = {S5922, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; BiPad 9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026082 = {S5923, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; Nexus EvoHd2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026120 = {S5924, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; vm Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026180 = {S5925, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; vivow Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026286 = {S5926, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; AT10H-A10WP Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026397 = {S5927, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; M70 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026422 = {S5928, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VegaCream_BETA-2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026428 = {S5929, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SANEI Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026495 = {S5930, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LR97A01 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026502 = {S5931, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; el-gr; "Turbo-X BREEZE" Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026578 = {S5932, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; hyundai S900 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026603 = {S5933, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DFP1008R Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026628 = {S5934, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; PI E002 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026701 = {S5935, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; PC7315 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026755 = {S5936, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-pt; TB700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026833 = {S5937, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; Multilaser Diamond Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026937 = {S5938, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; Pentagram TAB80 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026949 = {S5939, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MYIMAN-D07 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026966 = {S5940, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; 3C97 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2027043 = {S5941, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; i9701ips Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2027096 = {S5942, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; EB-A71GJ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2027132 = {S5943, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; i-Joy-Andromeda-3G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2027133 = {S5944, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; OR-TB17 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2027267 = {S5945, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M805 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2027284 = {S5946, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LUBM816HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2027398 = {S5947, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; Artes i701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2027482 = {S5948, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Tablet7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2027552 = {S5949, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ENERGY I828HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2027580 = {S5950, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; X2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027583 = {S5951, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; PC728 Build/PC728) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2027608 = {S5952, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PITM727HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027714 = {S5953, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-7205S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2027717 = {S5954, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VegaCream RC2 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2027759 = {S5955, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AR10e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2027793 = {S5956, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; asus_laptop Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2027882 = {S5957, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TB1082B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2028044 = {S5958, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; AN8BG3-LZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2028074 = {S5959, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; SP501 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028081 = {S5960, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; 1000TPC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028112 = {S5961, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; TAB200BR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2028356 = {S5962, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Tab 820 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2028368 = {S5963, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Junior 4.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2028447 = {S5964, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TVPAD Slim K3409 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2028451 = {S5965, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A710 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2028518 = {S5966, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; TB-207 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2028598 = {S5967, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; CT0712 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2028605 = {S5968, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VegaCream Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028673 = {S5969, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GL-7TA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2028676 = {S5970, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; MID06S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2028704 = {S5971, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Yan9.7G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2028708 = {S5972, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; GASEM712HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028764 = {S5973, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; TAB200R Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029031 = {S5974, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T6 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029068 = {S5975, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; T-131 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029228 = {S5976, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; hu-hu; M-TECH aTAB 7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029237 = {S5977, 169, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Motorola Milestone MAXX Build/6.7.1-68_DHD-15_NA-12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029337 = {S5978, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; hebe_inco Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029455 = {S5979, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; RK701 Build/2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029551 = {S5980, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Vi40 Elite Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029565 = {S5981, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ET-9018 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029735 = {S5982, 161, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Teclast_P85(A9D3) Build/IMM76D; 768*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2029924 = {S5983, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; UM-760 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029915 = {S5984, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; M7012 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029978 = {S5985, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; COMEM712HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030036 = {S5986, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; My Audio Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2030381 = {S5987, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; M-701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2030638 = {S5988, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GN868 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030677 = {S5989, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; T03A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2030850 = {S5990, 122, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GN868 Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2031008 = {S5991, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; HM0712C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2031144 = {S5992, 118, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; YL-Coolpad_9900/4.0.036.P2.X.9900; 480*960; CTC/2.0) CoolpadWebkit/534.30*/
const Device UA2031302 = {S5993, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Yan9.7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2031413 = {S5994, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; RK2906 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2031467 = {S5995, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; M-MP917I Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2031592 = {S5996, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; COMEM816HC_A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2031782 = {S5997, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID970 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2031857 = {S5998, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; QPad C-X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2031877 = {S5999, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; QW TB-1010 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032017 = {S6000, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Wxt Tab PC 65CXi Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032087 = {S6001, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; 9701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032410 = {S6002, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; M753 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032415 = {S6003, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SC-73MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032484 = {S6004, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID971 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032554 = {S6005, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CT802 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032731 = {S6006, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I-TAB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032787 = {S6007, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CHARM712HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032850 = {S6008, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; TC-973D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032921 = {S6009, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; N70 DUAL CORE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032967 = {S6010, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M19 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032976 = {S6011, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; Android B1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032985 = {S6012, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 97S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033066 = {S6013, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TABULET Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033076 = {S6014, 60, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; DFP7005 Build/IML74K)*/
const Device UA2033131 = {S6015, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; GoPad Surf Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033194 = {S6016, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MID104C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033459 = {S6017, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; D70GT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033462 = {S6018, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; N80 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033520 = {S6019, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Dezire Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033565 = {S6020, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; PC8001A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033607 = {S6021, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; el-gr; N6W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033734 = {S6022, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; SpeedUp S3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033824 = {S6023, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Ideapad S10-3T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033845 = {S6024, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; STAB970 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033969 = {S6025, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S13 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2034106 = {S6026, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; NAM752HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034119 = {S6027, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-pt; GROWING GTR718 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2034122 = {S6028, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; COMEM816HC_B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2034152 = {S6029, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; MVPM805HCF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2034256 = {S6030, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A8003T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2034349 = {S6031, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; ALLDRO2SPEED Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2034574 = {S6032, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; RICATECH-DR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2161431 = {S6033, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BC1077 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2165850 = {S6034, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; Mustek-DR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2167145 = {S6035, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; disgo9104 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2167588 = {S6036, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; M971w Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2168677 = {S6037, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; G21 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2169529 = {S6038, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T1002 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2176718 = {S6039, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GOCLM712HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2185331 = {S6040, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; SOLO831 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2198899 = {S6041, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; da-dk; TAC-97042 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2202532 = {S6042, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; HLV-T702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2203513 = {S6043, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-uk; cyclone Titan Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2204045 = {S6044, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC408PR002 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2206063 = {S6045, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ACCM700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2219399 = {S6046, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-uk; TAB10212 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2223989 = {S6047, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GY-M1004 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2224089 = {S6048, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; 756TPC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2224166 = {S6049, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Pentagram TAB10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2225207 = {S6050, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GIONEE S101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2226000 = {S6051, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; T8830 Build/HuaweiT8830) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2226830 = {S6052, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; ALEXIS-RX5BTBT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2228204 = {S6053, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 101T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2229030 = {S6054, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ACTM816HCX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2229656 = {S6055, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - HUAWEI_T8830_TD/1.0 Android/4.0.3 (Linux; U; Android 4.0.3; zh-cn) Release/05.31.2012 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30*/
const Device UA2230210 = {S6056, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NM7RKTC8-C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2230529 = {S6057, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PTM712MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2232409 = {S6058, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PTM805MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2234752 = {S6059, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SP-120 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2234807 = {S6060, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Z7H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2238955 = {S6061, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; PENTAGRAM TAB 8.4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2239000 = {S6062, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; FJMID-700C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257216 = {S6063, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MID-WCDMA Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257257 = {S6064, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; Maui Surf tab TAB-10IPS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257261 = {S6065, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Z5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257281 = {S6066, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; K-BOOK Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257292 = {S6067, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; KN_MOBILE_QT_A06 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257344 = {S6068, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; V12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257413 = {S6069, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; UTAB430 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257467 = {S6070, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; M-300TC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257389 = {S6071, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; ALLVIEWSPEED Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257477 = {S6072, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Yan7.1V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257500 = {S6073, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PTM723MR Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257544 = {S6074, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M805 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257599 = {S6075, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M1061 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257709 = {S6076, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; aTab7s Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257722 = {S6077, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; platinum Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257745 = {S6078, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TRIO 7C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257792 = {S6079, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; it-it; EFOX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257921 = {S6080, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; AT-707 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2329739 = {S6081, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; Q3M752HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2332579 = {S6082, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; V930 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2335195 = {S6083, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; S800 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2336347 = {S6084, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; MID700SC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2337116 = {S6085, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; OVO 1.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2337502 = {S6086, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; F721 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2337540 = {S6087, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; 8008 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2337668 = {S6088, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; aussiePAD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2338309 = {S6089, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; RMD-830 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2338432 = {S6090, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MP847 Build/IML74K) AppleWebKit/534.30(KHTML, like Gecko) Version/4.0Safari/534.30*/
const Device UA2338492 = {S6091, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; AirTab M81g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2339146 = {S6092, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MID-GSM Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2339492 = {S6093, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; TC-971 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2340571 = {S6094, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; T5500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2340982 = {S6095, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TSM_U4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2342981 = {S6096, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Newsmy T7II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2343042 = {S6097, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M701C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2343291 = {S6098, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; B928A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2343381 = {S6099, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Pro 10 dual core Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2343583 = {S6100, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; My Audio Build/RK2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2343732 = {S6101, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N101 DUAL CORE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2343823 = {S6102, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; M7512 Build/M7512) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2344332 = {S6103, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Acqua Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2346317 = {S6104, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; STS AR SERIES Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2346900 = {S6105, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Lemon_LT9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2346992 = {S6106, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; S5-N Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2347152 = {S6107, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; INTELLIPAD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2349727 = {S6108, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Rikomagic MK802 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2350010 = {S6109, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; monga-M6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2350684 = {S6110, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; M7511 Build/M7511) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2351672 = {S6111, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2352006 = {S6112, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TB782B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2352832 = {S6113, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; INM803HCZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2353362 = {S6114, 149, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; uni-phone_MING_SII Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2353603 = {S6115, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; WT-M768 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2359408 = {S6116, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GPLUS GN700W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2410947 = {S6117, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; MVM908HWZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2411032 = {S6118, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; E900 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2411397 = {S6119, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M1060W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2411409 = {S6120, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; TREQ A10C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2411810 = {S6121, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; H8000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2412291 = {S6122, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; AGFAPHOTO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2412355 = {S6123, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; C905 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2413059 = {S6124, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MFC155EN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2413088 = {S6125, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; TX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2413156 = {S6126, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; f7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2413319 = {S6127, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; AirTab M971g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2413439 = {S6128, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; PC9701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2413559 = {S6129, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MID1106 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2413877 = {S6130, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; M91 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2413975 = {S6131, 180, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - MOT-MT887_TD/1.0 Android/4.0 (Linux; U; Android 4.0.3; zh-cn) Release/05.16.2012 Browser/WAP 2.0 Mozilla/5.0 AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2414164 = {S6132, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TABZ Build/TABZ) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2414908 = {S6133, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; Kruger Matz Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2415382 = {S6134, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; jsa_MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2415474 = {S6135, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; MORAL_N01 Build/HuaqinMORAL_N01) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2416024 = {S6136, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 101D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2416401 = {S6137, 115, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; YL-Coolpad_5870/4.0.005.X.5870; 480*800; CTC/2.0) CoolpadWebkit/534.30*/
const Device UA2416471 = {S6138, 102, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Android 4.0.1; en-us) AppleWebKit/1.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.1*/
const Device UA2416487 = {S6139, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; N80HH Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2416499 = {S6140, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; 712F Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2416599 = {S6141, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TM-MID 750 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2416753 = {S6142, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; PTM712MCF Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2417426 = {S6143, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TZ300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2417741 = {S6144, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TB1082 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2417838 = {S6145, 115, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; YL-Coolpad_5910/4.0.012.X.5910; 480*800; CTC/2.0) CoolpadWebkit/534.30*/
const Device UA2418025 = {S6146, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; ALEXIS-RX5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2418188 = {S6147, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LA-I Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2419209 = {S6148, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID-A10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2419865 = {S6149, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; TC-7120 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2419909 = {S6150, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DFP7004 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2420011 = {S6151, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; Platinum Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2420049 = {S6152, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; X12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2420165 = {S6153, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; 4S Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2420188 = {S6154, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; G11 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2420294 = {S6155, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SANEI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2420621 = {S6156, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TM-MID600 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2420633 = {S6157, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; NAM752MC_A Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2420636 = {S6158, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; 56372 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2420875 = {S6159, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; CW-V9-DUAL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2421007 = {S6160, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ICS Build/ICS.f08refem31.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2421009 = {S6161, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/Android4.0.3 Release/09.17.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2421033 = {S6162, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A10-4core Build/R13.4-rc2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2421100 = {S6163, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; IPRO i5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2421111 = {S6164, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; CT-1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2421230 = {S6165, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; mi-book Mi3 Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2421243 = {S6166, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2421248 = {S6167, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MFC180EN Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2421380 = {S6168, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2421442 = {S6169, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; JY131C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2421530 = {S6170, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VB80a Pro Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2421552 = {S6171, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; START901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2421904 = {S6172, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A9DualCore Build/ICS.d5h10_8188.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2421941 = {S6173, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-be; Axxion ATAB-912 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422032 = {S6174, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; P101g Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422214 = {S6175, 154, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Full Android on x86 VirtualBox Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422250 = {S6176, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMID800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422262 = {S6177, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JSA-MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422416 = {S6178, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; OCA7TA Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422618 = {S6179, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MFC270EN_09 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422663 = {S6180, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; PTM712MCFC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2423231 = {S6181, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MFC180FR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2423252 = {S6182, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 975TPC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2423360 = {S6183, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; BASE_Lutea_3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423411 = {S6184, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; CJ ThL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423524 = {S6185, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q88 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423561 = {S6186, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; J-Q88D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423576 = {S6187, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N77 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2423806 = {S6188, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; M81g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2423807 = {S6189, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Penta WS702C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2423895 = {S6190, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; J-Q8D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2424111 = {S6191, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MD-0696B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2424276 = {S6192, 153, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; AQUILA 097-1006 Build/ICS.g12refM1006CMX.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2424285 = {S6193, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; PT001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2424485 = {S6194, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GASEM805MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2424675 = {S6195, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U; Android 4.0.4; en-us;NEC-0912 Build/AX)AppleWebKit/534.30(KHTML, Like Gecko)Version/4.0 Mobile Safari/534.30*/
const Device UA2424796 = {S6196, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V930M10E_Burma Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2424848 = {S6197, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 704 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2425083 = {S6198, 157, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - AMOI N820 Linux/3.0.13 Android/4.0.4 Release/09.24.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2425147 = {S6199, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC408 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2425571 = {S6200, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N83 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2425608 = {S6201, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; J-Q8B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2425631 = {S6202, 150, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - N1 Linux/3.0.13 Android/4.0.4 Release/10.15.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2425716 = {S6203, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; CAPTIVA PAD 8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2425781 = {S6204, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Expad Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2425892 = {S6205, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M9701THW Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2425955 = {S6206, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID-001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426036 = {S6207, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KOCASO 970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2426152 = {S6208, 167, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; E-Boda Supreme X80 Dual Core Build/ICS.g12refM806A1YBD.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2426233 = {S6209, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ms-my; i7S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426344 = {S6210, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; PICOpad_GEA Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426410 = {S6211, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; POP 7.1C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2426857 = {S6212, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 4SPA0910C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2427061 = {S6213, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MHU001D Build/MHU001D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2427460 = {S6214, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Clear Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2428345 = {S6215, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TAB700_4GB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2428419 = {S6216, 152, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; AQUILA 080-1008 Build/ICS.g12refM805CMX.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2428471 = {S6217, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; sp0735 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2428581 = {S6218, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; 7E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2428987 = {S6219, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T-200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2432244 = {S6220, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; MANHATTAN Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2432698 = {S6221, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ATO7P Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2433591 = {S6222, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; VOLXM712MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2441675 = {S6223, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; MVPM1005HNF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2442674 = {S6224, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; AllviewSpeedEco Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2442814 = {S6225, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID_X5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2442905 = {S6226, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TCC8925_HDMI_DONGLE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2443057 = {S6227, 154, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HanZpad Build/master-ics-android-4.0.4_r1.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2443279 = {S6228, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; hu-hu; xTAB-70 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2444624 = {S6229, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V700B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2445359 = {S6230, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; T900 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2445412 = {S6231, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; BC1077 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2445547 = {S6232, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M811 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2446734 = {S6233, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; GASEM723HRF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2447142 = {S6234, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-be; Axxion ATAB-703 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2447614 = {S6235, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MIDC907PR001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2448890 = {S6236, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CW V8 Dual Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2449027 = {S6237, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; M-MP927I Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2449166 = {S6238, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; STM912HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2450282 = {S6239, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i7c Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2450377 = {S6240, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; IMPRESSPE3D E200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2450523 = {S6241, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A2000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2450875 = {S6242, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; W990 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2453117 = {S6243, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; E61 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2453425 = {S6244, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; AK8456 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2453639 = {S6245, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; V800 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2453724 = {S6246, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M-970 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2454373 = {S6247, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; WOHTC G36 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2454393 = {S6248, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; N8Zero Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2454575 = {S6249, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SS707 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2454778 = {S6250, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; 5910 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2454853 = {S6251, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TZ200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2455197 = {S6252, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Polaroid 40140 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455366 = {S6253, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; imatch i21 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455386 = {S6254, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SC-74MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2455396 = {S6255, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; A21plus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455388 = {S6256, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; NAM805ND Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2455457 = {S6257, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; CW V70 Build/ICS.g08ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2455476 = {S6258, 148, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Cat Tablet CSG 7J0 Build/ICS.g08ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2455488 = {S6259, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N9977 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455511 = {S6260, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MW0821+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2455528 = {S6261, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-uk; softwinners Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2455580 = {S6262, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; smartTV Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2455719 = {S6263, 148, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; Android Multimedia  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2455740 = {S6264, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q75 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455842 = {S6265, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CRANE EVB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2456563 = {S6266, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; UM-A13 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2456662 = {S6267, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; INM752HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2456798 = {S6268, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; GNET Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2456804 = {S6269, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; MFC155ES Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2456885 = {S6270, 161, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; RK2906 Build/rk29sdk_Android 4.0.4_X fwd.test-keys) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2456971 = {S6271, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GOTAB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2457041 = {S6272, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Obreey SURFpad Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2457051 = {S6273, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VL-109 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2457186 = {S6274, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A9DualCore Build/ICS.d5h1016_8188.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2457387 = {S6275, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST9 Marvel Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2457424 = {S6276, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; H12 Build/ICS.f08refem31.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2457462 = {S6277, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMID10C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2457474 = {S6278, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; FJMID-700 C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2457543 = {S6279, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; 7D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2457557 = {S6280, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MFC170FR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2457865 = {S6281, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PTM805ND Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2457877 = {S6282, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N10-4core Build/R13.4-rc2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2458073 = {S6283, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; x806 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2458360 = {S6284, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMID1000B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2458772 = {S6285, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MFC155EN_09 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2459724 = {S6286, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ITAB Build/ICS.g12refM806A1SPISY.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2460394 = {S6287, 148, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Playtab Build/ICS.g12refM806A1SPISY.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2460775 = {S6288, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDCD10 Build/ICS.d5h1083_8188.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2461933 = {S6289, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ATO712L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2463296 = {S6290, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ZYNC Z990 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2463810 = {S6291, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; MID05V2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2464168 = {S6292, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; FS-M787 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2464608 = {S6293, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; Z520m-parrot Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2464616 = {S6294, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2464618 = {S6295, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GTEL S101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2464869 = {S6296, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; 930i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2464954 = {S6297, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; T101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2465267 = {S6298, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; 5870 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2465578 = {S6299, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ViewPad E100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2465661 = {S6300, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EP750 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2465783 = {S6301, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; EXLW2R Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2466040 = {S6302, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; PX8837 Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2466172 = {S6303, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M750 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2466207 = {S6304, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; SMX-T701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 MYI/1.10*/
const Device UA2466217 = {S6305, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TB892B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2466484 = {S6306, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; MFC155DE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2466538 = {S6307, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MFC175FR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2466705 = {S6308, 175, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; M972w Build/M972w-eng 4.0.4 IMM76D test-keys_OTA:MID_2.1.1.1_X.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2466841 = {S6309, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; KidsPad1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2467194 = {S6310, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fa-ir; G1 Snowy Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467281 = {S6311, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Tablet X2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2467284 = {S6312, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; DIFRNCE DIT7050 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2467351 = {S6313, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; T9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2467402 = {S6314, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; CT710 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2467443 = {S6315, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; SAMURAI Build/SAMURAI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2467463 = {S6316, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; COROM752MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2467565 = {S6317, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; KBOOK Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2467662 = {S6318, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TPC0752 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467902 = {S6319, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 900TPCII Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2467952 = {S6320, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; MFC155IT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2468504 = {S6321, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SP500 Build/1.0.749_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2468634 = {S6322, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M5000 Build/1.0.528_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2468639 = {S6323, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MP880 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2469160 = {S6324, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DA988 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2469309 = {S6325, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; TAB10-07 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2469355 = {S6326, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; My Play Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2469377 = {S6327, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN13FP Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2473660 = {S6328, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; HTC VAE_WX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2474751 = {S6329, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MA70 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2474800 = {S6330, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; MVM900HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2475882 = {S6331, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; MOVIMAX P3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2475956 = {S6332, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; MVM900HWZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2476372 = {S6333, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D90T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2476957 = {S6334, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-cl; G970C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2477412 = {S6335, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2477918 = {S6336, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; F900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2478273 = {S6337, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; classic101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2478278 = {S6338, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; PENTAGRAM TAB 8.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2478607 = {S6339, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; disgo8104 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2479946 = {S6340, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; P80w Build/SuningP80w) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2480010 = {S6341, 156, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID-1088B Dual Core Build/ICS.g12refM1005XK.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2480296 = {S6342, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; T-702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2480317 = {S6343, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; EVOAT7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2480327 = {S6344, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ST0911 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2480649 = {S6345, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; CASIA TAB 10'' Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2480776 = {S6346, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A088 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2480797 = {S6347, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; tablet-c Build/tablet-c) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2480943 = {S6348, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ST0719 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2480896 = {S6349, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C3_77_5647 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2480977 = {S6350, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 704 Build/ICS.g08ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2481052 = {S6351, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-uk; PA0108C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2481288 = {S6352, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 19220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2481294 = {S6353, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Mi-A400 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2481407 = {S6354, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Penta WS703C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2481593 = {S6355, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; BUSTAB7R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2481729 = {S6356, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; QW TB-1217 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2481782 = {S6357, 150, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Zigo Nebula Tab 7.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2481944 = {S6358, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 795TPC Build/795TPC) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2482051 = {S6359, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Spark Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2482364 = {S6360, 163, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; RK2906 Build/rk29sdk_Android 4.0.4_X.X fwd.test-keys) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2482609 = {S6361, 153, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Brilliant Pad TPC7026C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2482704 = {S6362, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M5000 Build/1.0.741_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2482763 = {S6363, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; OVO 2.0 Build/FR-M768) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2482804 = {S6364, 160, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Android 4.0.3;en-us;Digitac Ultratab;C1029 Build/ICS) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2482857 = {S6365, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ALLVIEW P4i Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2482930 = {S6366, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; NXM727HCFCN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2482951 = {S6367, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-eg; Q88B:7A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2482993 = {S6368, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JY139 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483100 = {S6369, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; POM805HCF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2483139 = {S6370, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NEO-G4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2483211 = {S6371, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IMAGINE 10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2483315 = {S6372, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-gb; N9977 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483468 = {S6373, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GP501 Build/ICS_IMM76D_B162) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483483 = {S6374, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; GR-TB7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483652 = {S6375, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MFC270EN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2483889 = {S6376, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT9700DC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2483898 = {S6377, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PWS700FJ Build/eng.glwx.X.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2484098 = {S6378, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; A8826 Build/ICS.A8826.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2484460 = {S6379, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Tagus Tablet Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2484525 = {S6380, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; Motion Build/Motion) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2485179 = {S6381, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; T-M719A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2485746 = {S6382, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PTM712MCFC1 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2485822 = {S6383, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MD-0651 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2486196 = {S6384, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GY-M501 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2486203 = {S6385, 153, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - aTAB5 Linux/3.0.13 Android/4.0.4 Release/10.22.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2486314 = {S6386, 156, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Symphony Linux/3.0.13 Android/4.0.4 Release/12.06.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2486386 = {S6387, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; IFQM712MCF Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2486499 = {S6388, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KY-901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2486528 = {S6389, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; APPTB101S Build/ICS.g12refM1005APL.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2486569 = {S6390, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MIDR47B Build/eng.glwx.X.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2486604 = {S6391, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AIRIS S211 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2486637 = {S6392, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TRIO-97C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2486810 = {S6393, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AIRIS GN135 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2487328 = {S6394, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Vexia Tablet 780 GPS Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2487682 = {S6395, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; 740RS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2487867 = {S6396, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2494699 = {S6397, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; Pentagram TAB70 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2495778 = {S6398, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; AirTab M81w Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2496522 = {S6399, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; rkSDK Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2496856 = {S6400, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; B2000+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2496862 = {S6401, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; A750 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2497014 = {S6402, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; T03A_mma7660 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2497161 = {S6403, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Parkly RK2906 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2498149 = {S6404, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; P10MAN R2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2498459 = {S6405, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; RYOMA M1S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2498746 = {S6406, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; T710P Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2498790 = {S6407, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; D703B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2498896 = {S6408, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; STM908HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2498915 = {S6409, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; NAM1006HDW Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2499655 = {S6410, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ETAB 0208A Build/ICS.f08refem86.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2499676 = {S6411, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; T8830 Build/HuaweiT8830) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2500191 = {S6412, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; VITA-V5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2500408 = {S6413, 115, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; YL-Coolpad_9120/4.0.020.X.9120; 480*800; CTC/2.0) CoolpadWebkit/534.30*/
const Device UA2500553 = {S6414, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; ICE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2500808 = {S6415, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TabTop 7.16 - MGPT 07 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2501280 = {S6416, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; 10BPM3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2501949 = {S6417, 116, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; YL-Coolpad_7266/4.0.006.X.7266; 480*800; CUCC/2.0) CoolpadWebkit/534.30*/
const Device UA2502138 = {S6418, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; shooter Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2502448 = {S6419, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; 9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2502458 = {S6420, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Tab970 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2502524 = {S6421, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; TH-1129M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2502545 = {S6422, 158, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - GPLUS N930 Linux/3.0.13 Android/4.0.4 Release/09.11.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2502555 = {S6423, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; FP710 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2502570 = {S6424, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; STX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2502639 = {S6425, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ar-eg; E10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2502726 = {S6426, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MW0922 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2502734 = {S6427, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Vexia Tablet 780 Build/tablet-b) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2502879 = {S6428, 151, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - i10 Linux/3.0.13 Android/4.0.4 Release/09.13.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2502894 = {S6429, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PTM727MCF Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2503188 = {S6430, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MFC154FR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2503238 = {S6431, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; Primux_Omega Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2503425 = {S6432, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MGPT09 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2503544 = {S6433, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; FTABU Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2503761 = {S6434, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; RC9716B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2503815 = {S6435, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; TabTop 10.4-MGPT 05 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2503924 = {S6436, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MiTraveler Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2503980 = {S6437, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Informer 921 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2504150 = {S6438, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; AT97I-RK66BP Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2504316 = {S6439, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy S3 EX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2504507 = {S6440, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PICOpad_GFI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2504901 = {S6441, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-gb; SX5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2504918 = {S6442, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMID1000D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2504920 = {S6443, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; XL4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2504957 = {S6444, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-be; CHARM805NC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2505189 = {S6445, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; 1040RS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2505367 = {S6446, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Quicki-728 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2505698 = {S6447, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U; Android 4.0.4; en-us; MP777 Build/IMM76D) AppleWebKit/534.30 (KHTML,like Gecko) Version/4.0 Safari/534.30*/
const Device UA2505770 = {S6448, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S5A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2506581 = {S6449, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; disgo9000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2506904 = {S6450, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; TAB-8DC Build/ICS.g12refM805ITO.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2506952 = {S6451, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2506938 = {S6452, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; signal75_ics2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2507082 = {S6453, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; NTT 611 Build/ICS.g08refem611.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2507311 = {S6454, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T1A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2507335 = {S6455, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TPC0952 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2518781 = {S6456, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; DC-0163 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2518967 = {S6457, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; CTAB Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2518975 = {S6458, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M701F Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2519896 = {S6459, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; TPAD M1004A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2519948 = {S6460, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GN868 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2520087 = {S6461, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; I10a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2520406 = {S6462, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ro-ro; Neura WIFI Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2520438 = {S6463, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; STAR N8000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2520677 = {S6464, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Q7C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2520731 = {S6465, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Ipad3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2520783 = {S6466, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Touch Kem430 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2520822 = {S6467, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A088 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2520858 = {S6468, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; RK701 Build/RK2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2520942 = {S6469, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fa-ir; Dimo 900 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2522118 = {S6470, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZENDEX 8 Advanced Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2522282 = {S6471, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; A1001C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2524460 = {S6472, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; IMO Z6 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2524645 = {S6473, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-RU; Informer 703 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2524814 = {S6474, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; I04S Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2524919 = {S6475, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; HYUNDAI Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2525069 = {S6476, 150, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Cat Tablet Galactica 9,7CA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2525509 = {S6477, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; 5910 Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2525725 = {S6478, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; NTT-758 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2525967 = {S6479, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; H6000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2526001 = {S6480, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC408PR001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2526075 = {S6481, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; M1005D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2526121 = {S6482, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; S4001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2526355 = {S6483, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-; CTB-701H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2526388 = {S6484, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; AT7E-RK06WA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2526486 = {S6485, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Orbit_5.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2526599 = {S6486, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2526648 = {S6487, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Next7Q12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2526656 = {S6488, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; HLV-T9701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2526710 = {S6489, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; VIG07 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2526925 = {S6490, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; m720vw_vortex Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2526943 = {S6491, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Tab9.7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2526992 = {S6492, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; FPT Tablet HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2527240 = {S6493, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; MC603CH Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2527246 = {S6494, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ro-ro; AllviewSpeed2+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2527282 = {S6495, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; RC9711B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2527394 = {S6496, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; RC9724C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2527507 = {S6497, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EON3I Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2527527 = {S6498, 172, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; IMO X5 Build/IMO.X5_MARS.Vxx.(date of built/release software)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2527873 = {S6499, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Colortab-MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2527878 = {S6500, 173, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; M83w Build/M83w-eng 4.0.4 IMM76D test-keys_OTA:MID_2.1.1.1_X.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2527891 = {S6501, 163, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; Clempad_CPT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 ClemBrowser/1.0 Clempad_CPT*/
const Device UA2527909 = {S6502, 157, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; Clempad Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 ClemBrowser/1.0 Clempad X*/
const Device UA2527946 = {S6503, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T-M703A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528064 = {S6504, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMID1000/PMID10C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2528115 = {S6505, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AW101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2528154 = {S6506, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MFC270FR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2528254 = {S6507, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MW0710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2528301 = {S6508, 125, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; - Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2528315 = {S6509, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HomeChoice-V12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528456 = {S6510, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TY-Q8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528844 = {S6511, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PEDI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2528887 = {S6512, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; B902E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2529346 = {S6513, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; 8BPE0 Build/ICS.g08refem618.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2529365 = {S6514, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; T271 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2529433 = {S6515, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; AT97I-RK66WA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2529484 = {S6516, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ACGM816HCX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2529624 = {S6517, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VM5610 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2529693 = {S6518, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Matrix III Build/X.tm) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2529719 = {S6519, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CABALT Tab S1010 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2529947 = {S6520, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; aTAB7t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2529963 = {S6521, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; hr-hr; TPC-7120 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2529966 = {S6522, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; A721 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2529979 = {S6523, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Phablet PIII Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2530021 = {S6524, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MFC155EN_10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2530136 = {S6525, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Scroll Tablet PC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2530218 = {S6526, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; Fatih Build/AR.ETIS.PSI-R6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2530249 = {S6527, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D7015 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2530323 = {S6528, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 4U Sapphire Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2530600 = {S6529, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ektATB-1108 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2530804 = {S6530, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IdolPad 7+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2530813 = {S6531, 148, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fa-ir; Marshal ME-700A Build/Marshal_ME-700A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2531146 = {S6532, 175, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; full Android on Microsoft Windows, pad, pc, n*books Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2533080 = {S6533, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Symphony Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2533090 = {S6534, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT704 Build/GT704-A13005-G02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2534426 = {S6535, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; J-Q88D? Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2534596 = {S6536, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SPX-12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2536108 = {S6537, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 1688-XHB-2357 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2543213 = {S6538, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC POO_U Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2543761 = {S6539, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; m1099RA Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2545162 = {S6540, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ZM971 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2545363 = {S6541, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; AMPE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2545800 = {S6542, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; Kiwie_SEVEN-PRO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2546485 = {S6543, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; desktop_generic Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2546498 = {S6544, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; T9200 Build/HuaweiT9200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2547436 = {S6545, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; TAB5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2548010 = {S6546, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; DATAM805HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2548120 = {S6547, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; AMOI N820 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2548633 = {S6548, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N101 DUAL CORE2 V11 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2549457 = {S6549, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; PEM727HCFR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2549991 = {S6550, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; HD7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2550331 = {S6551, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; HLV-T701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2550574 = {S6552, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; PTS AR SERIES Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2550955 = {S6553, 148, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; ConCorde tab 9.7IPS PLUS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2551384 = {S6554, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; T291 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2551754 = {S6555, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; Exagerate XZPAD970H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2551985 = {S6556, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; QS0716D Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2552062 = {S6557, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; Beyond B78 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2552236 = {S6558, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PC1058 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2552629 = {S6559, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; CASIA TAB 7" Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2552814 = {S6560, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; Primux_Alpha Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2552834 = {S6561, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; CL500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2552911 = {S6562, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Infinity Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2553062 = {S6563, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A707_Infinity Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2553092 = {S6564, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; EBEST V5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2553140 = {S6565, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Rikomagic MK802II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2553232 = {S6566, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; T1003 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2553250 = {S6567, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M71G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2553229 = {S6568, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; AP-80104 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2553283 = {S6569, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; E8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2553284 = {S6570, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SXZ-PD80-X05 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2553386 = {S6571, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; PAPYRE pad 715 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2553673 = {S6572, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TB782 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2553998 = {S6573, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TB4-4/5580 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2554158 = {S6574, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Phone5 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2554170 = {S6575, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PA810G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2554217 = {S6576, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; ANGB-CH Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2554417 = {S6577, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; NAM752NC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2554436 = {S6578, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; PMID70C Build/unknown) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2554479 = {S6579, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GS3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2554593 = {S6580, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; OV-SteelCore Build/ICS.g08refem611.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2554911 = {S6581, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Digma iDn7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555009 = {S6582, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IMAGINE 9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555071 = {S6583, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i7a Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555187 = {S6584, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DTA-07HDAH Build/ICS.g08ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555199 = {S6585, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MFC250FR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555237 = {S6586, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; JUPITER Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555269 = {S6587, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Quicki-1041 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555379 = {S6588, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; ELEMENT7V2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555392 = {S6589, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M9702TH Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555433 = {S6590, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-eg; C72d Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555451 = {S6591, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; hu-hu; xTAB-70dc Build/xTAB-70dc.8GB.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555579 = {S6592, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DA788 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555670 = {S6593, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; hebe_plus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555685 = {S6594, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; T1300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555816 = {S6595, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; YF723B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555853 = {S6596, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; F1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555906 = {S6597, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; H7 Build/KO.Hanlym.H7.V2.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555938 = {S6598, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M77 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2556026 = {S6599, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; OYSTERS T7M 3G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2556028 = {S6600, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; 1707 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2556175 = {S6601, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; EverGlory Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556237 = {S6602, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; XL4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556384 = {S6603, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PICOpad GJT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2556440 = {S6604, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Z3Z6Z7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2556508 = {S6605, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; TB1000 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2556666 = {S6606, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C1029 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2556711 = {S6607, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 5I Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556719 = {S6608, 150, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; tablet Fnac7 Little Build/1.0.2.X-19:55) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2556875 = {S6609, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PICOpad GEW Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556881 = {S6610, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB 170 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2557306 = {S6611, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; E1020HD Build/ICS.E1020HD.Dualcore.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2557324 = {S6612, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PAD1001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2557436 = {S6613, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; _3QM819HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2557452 = {S6614, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EBM1005ND Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2558120 = {S6615, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; K-TOUCH W806 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558160 = {S6616, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TABLET Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2558426 = {S6617, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; hexing77_ics2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558913 = {S6618, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Z500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2561039 = {S6619, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; baoxue Build/F7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2561764 = {S6620, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IMPULSE7120 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2564155 = {S6621, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MID82C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2574294 = {S6622, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Qumo Dotcom Build/ICS.b02ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2574350 = {S6623, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; PD-970CH PLUS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2574603 = {S6624, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MX970 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2575341 = {S6625, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; USAGC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2576565 = {S6626, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Helios Build/ICS.f08ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2577219 = {S6627, 151, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Cat Tablet GalacticaX 9.7JA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2577226 = {S6628, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; TechTAB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2577521 = {S6629, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Explay Informer 801 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0.4 Safari/534.30*/
const Device UA2577515 = {S6630, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; it-it; B63 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2577592 = {S6631, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; AS-05 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2577669 = {S6632, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; M72 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2577723 = {S6633, 57, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; N900 Build/IMM76D)*/
const Device UA2577871 = {S6634, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - HUAWEI_T9510E_TD/1.0 Android/4.0.4 (Linux; U; Android 4.0.4; zh-cn) Release/05.31.2012 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30*/
const Device UA2577945 = {S6635, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; _3QM819HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2578009 = {S6636, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; el-gr; Turbo-X BREEZE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2578116 = {S6637, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; QT-A07 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2578139 = {S6638, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; T71hk Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2578158 = {S6639, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; POP8.1G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2578243 = {S6640, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; SP-A20i Build/MF_ICS_02.24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2578434 = {S6641, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q8-A13 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2578488 = {S6642, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; R71 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2578503 = {S6643, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; a106c_ics Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2578704 = {S6644, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ING100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2578764 = {S6645, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CABALT Tab S700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2578838 = {S6646, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TP710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2578994 = {S6647, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AT197F Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2579030 = {S6648, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T-FREE 5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2579323 = {S6649, 150, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V9-1688-CJ-0308-31S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2579493 = {S6650, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Sky Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2579569 = {S6651, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; T9002 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2579610 = {S6652, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T753 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2579718 = {S6653, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; E704J Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2579747 = {S6654, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PICOpad_GEA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2579756 = {S6655, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; LUXURY Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2579832 = {S6656, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; I5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2579889 = {S6657, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; Multilaser Oxy Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2579929 = {S6658, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; A77 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2580155 = {S6659, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M713 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580185 = {S6660, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ET-971 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2580196 = {S6661, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; MID08S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2580221 = {S6662, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; BT210 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2580296 = {S6663, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SXZ-PDX0-01 Build/SXZ-PDX0-01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580843 = {S6664, 151, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - X50 Linux/3.0.13 Android/4.0.4 Release/01.10.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2580896 = {S6665, 149, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; 3310-9506 Build/ICS.Zoostorm_160_NEW.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2581071 = {S6666, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; STXS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2581895 = {S6667, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4;pt-pt; tmn smart a18 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2582780 = {S6668, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; VM1017A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583262 = {S6669, 169, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Sansui Cosmic SP 1001 Linux/3.0.13 Android/4.0.4 Release/01.06.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2588153 = {S6670, 60, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; A10-MID Build/IML74K)*/
const Device UA2588417 = {S6671, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; SZENIO IPS 9700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2589047 = {S6672, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M-301T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2589167 = {S6673, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; P90 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2589284 = {S6674, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LGL96G/V100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2589626 = {S6675, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Q8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2589756 = {S6676, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; i708 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2590108 = {S6677, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; RM12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2590111 = {S6678, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; SIROCO 2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2590482 = {S6679, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; IHD908 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2590577 = {S6680, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; J1000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2590654 = {S6681, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; Airpad-DS820M Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2591506 = {S6682, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Penta WS704C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2592311 = {S6683, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AMPE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2592765 = {S6684, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Gemini Apside Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2593403 = {S6685, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Apollo Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2593592 = {S6686, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q39 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2593871 = {S6687, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; T5500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2594075 = {S6688, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; N70DC-T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2594197 = {S6689, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; 5677 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2594235 = {S6690, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; RTAB-070 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2594256 = {S6691, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; RMD-73G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2594267 = {S6692, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A930 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2594315 = {S6693, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A13 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2594320 = {S6694, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SCH-W2013 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2594335 = {S6695, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; TP9.7-1200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2594353 = {S6696, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-by; MTC Viva Build/HuaweiU8816) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2594328 = {S6697, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; P101w Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2594543 = {S6698, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DFPQ88 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2594807 = {S6699, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; AP-102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2594815 = {S6700, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M78 Build/M78) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2594859 = {S6701, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; TB-50 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2594871 = {S6702, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; TP8-1200-metal Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2594887 = {S6703, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; EPC-705 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2595015 = {S6704, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MICROLAB TABLET Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2595106 = {S6705, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; G901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2595166 = {S6706, 124, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; CMP746E Build/ITL41D) AppleWebKit/434.50 (KHTML, like Gecko) Version/4.0 Mobile*/
const Device UA2595270 = {S6707, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMID1000B_V2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2595366 = {S6708, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; SPX_8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2595470 = {S6709, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; crane-evb Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2595630 = {S6710, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; DL HD-72 X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2595637 = {S6711, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; ONEPAD940 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2595741 = {S6712, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M010SQ8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2596089 = {S6713, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Proton Lite Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2596370 = {S6714, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; myTab10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2596481 = {S6715, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fi-fi; 28126 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2596498 = {S6716, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Lafone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2596660 = {S6717, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; P10C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2596695 = {S6718, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; YPY_10FTB Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2596697 = {S6719, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M752 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2596769 = {S6720, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A704J Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2596821 = {S6721, 158, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - AMOI M8228 Linux/3.0.13 Android/4.0.4 Release/11.06.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2596869 = {S6722, 152, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ICS AOSP on imx6q Build/R13.4-beta) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2596877 = {S6723, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EXLB3B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2596912 = {S6724, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TB782B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2596942 = {S6725, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; Mio Pad A1001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2597182 = {S6726, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; MFC270ES Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2597230 = {S6727, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q88B-5S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597260 = {S6728, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/Android4.0.3 Release/11.17.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2597271 = {S6729, 163, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; UM77-V1.11-073-01-01-02-01-00-01 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597488 = {S6730, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q88B-12S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597498 = {S6731, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NuclearSX-SP5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597553 = {S6732, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; EGS-5S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2597611 = {S6733, 157, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Yummy-Duo Linux/3.0.13 Android/4.0.4 Release/12.03.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2597622 = {S6734, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M762B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597861 = {S6735, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; W92210A1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597921 = {S6736, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; BPA9730 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2598349 = {S6737, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; 10BPE0 Build/MystiQ970-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2598515 = {S6738, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Inco Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2598844 = {S6739, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; L9000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2599160 = {S6740, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; SMARTLET Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2599911 = {S6741, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Xtab745 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2600401 = {S6742, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; WTG2-07G-S1 Build/ICS.g08refem667.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2600597 = {S6743, 156, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - JXD-P300 Linux/3.0.13 Android/4.0.4 Release/01.23.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2600629 = {S6744, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; W70 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2600788 = {S6745, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Alpha GT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2601491 = {S6746, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT704 Build/GT704-A13005-F02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2601546 = {S6747, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; START901.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2602472 = {S6748, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; T101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2603032 = {S6749, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ar-eg; i7005 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2605332 = {S6750, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; LG-L160L Build/IML74K) AppleWebkit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2605534 = {S6751, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID_Robin Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2640847 = {S6752, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Tablet PC 2000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2641233 = {S6753, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; 9900 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2641602 = {S6754, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; AndroCore Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2642147 = {S6755, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Athens15_TD/V2 Linux/3.0.13 Android/4.0 Release/02.15.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2 System/Android 4.0.1;*/
const Device UA2642313 = {S6756, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; MP810C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2642738 = {S6757, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; RMD-720 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2642947 = {S6758, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; H9701C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2643170 = {S6759, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; VB97 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2643385 = {S6760, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; M1005HN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2643486 = {S6761, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Pentagram TAB70 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2644099 = {S6762, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MID970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2644309 = {S6763, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; EVOPAD_R7000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2644491 = {S6764, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC_A320e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2644639 = {S6765, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; KPT A11PLUS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2645648 = {S6766, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; MID9107CM Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2645681 = {S6767, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; AUX V920P Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2646765 = {S6768, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PC721 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2646777 = {S6769, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; DOOV_D7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2648405 = {S6770, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; GSmart G1342 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2649663 = {S6771, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Smartab MT12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2649679 = {S6772, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; QWTB-1280BL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2650274 = {S6773, 124, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; M78 Build/M78) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2650877 = {S6774, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ACGM727HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2650987 = {S6775, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GY-M2906 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2651198 = {S6776, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; EVOPad R9700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2652288 = {S6777, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; DOOV_D900 Build/DoovDOOV_D900) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2652297 = {S6778, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; A76 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2652440 = {S6779, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Coolpad 7290 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2652475 = {S6780, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I700C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2652510 = {S6781, 155, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - X9 NOTE Linux/3.0.13 Android/4.0.4 Release/09.26.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2652538 = {S6782, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; ANDROID_HDMI_DONGLE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2652547 = {S6783, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; L11 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2652634 = {S6784, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN180 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2652986 = {S6785, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TAB 7i 3G Build/WEXLER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2652995 = {S6786, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SC-90MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2653246 = {S6787, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; G1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2653249 = {S6788, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; U23GT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2653318 = {S6789, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/Android4.0.3 Release/10.27.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2653334 = {S6790, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; gs701b Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2653380 = {S6791, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; RK2906-8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2653471 = {S6792, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ALFAR 780 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2653518 = {S6793, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; RC0721B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2653530 = {S6794, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; J-88 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2653583 = {S6795, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; NB0026 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2653614 = {S6796, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; 300TC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2653860 = {S6797, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; vi-vn; 6515 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2653958 = {S6798, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AG CHROME Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654024 = {S6799, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fa-ir; G1 SKY Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654029 = {S6800, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Starway Andromeda S910 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2654051 = {S6801, 148, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; GLX Mobile Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2654110 = {S6802, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MP444 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2654220 = {S6803, 150, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - A2 Linux/3.0.13 Android/4.0.4 Release/10.29.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2654241 = {S6804, 119, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; YL-Coolpad_9960/4.0.013.P2.X.9960; 1280*720; CTC/2.0) CoolpadWebkit/534.30*/
const Device UA2654260 = {S6805, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JXD-S80 Build/ICS.g08refem618.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2654274 = {S6806, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; J-Q8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654307 = {S6807, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; 8005 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2654353 = {S6808, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; VEDIA X6 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2654690 = {S6809, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DLR_MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654839 = {S6810, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; PT7-ETOOS Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2654846 = {S6811, 180, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - MOT-MT788_TD/1.0 Android/4.0 (Linux; U; Android 4.0.4; zh-cn) Release/05.16.2012 Browser/WAP 2.0 Mozilla/5.0 AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654875 = {S6812, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; TKSP8M404BK Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654948 = {S6813, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Android 904 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654928 = {S6814, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; PMID70DC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2655003 = {S6815, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SXZ-PDX0-05M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2655082 = {S6816, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; H433 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2655094 = {S6817, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; MID7500 Build/ICS.g08ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2655267 = {S6818, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; 8190 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2655268 = {S6819, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; F9S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2655492 = {S6820, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X704 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2655494 = {S6821, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; A13-DLR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2655720 = {S6822, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SP-100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2655724 = {S6823, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; G-PAD1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2655867 = {S6824, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAB-NAVI7-3G-M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2656171 = {S6825, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; E7010HD Build/ICS.E7010HD.Dualcore.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2656244 = {S6826, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; YPY_10STB Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2656432 = {S6827, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2656604 = {S6828, 162, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; E-Boda Impresspeed E300 Build/ICS.g12refM703A1YBD.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2656699 = {S6829, 151, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; HKPHONE REVO MAX Build/Haiduongbk) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2656831 = {S6830, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; POP_smart_PS1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2657082 = {S6831, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; POM727MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2657336 = {S6832, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Mytel Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2657350 = {S6833, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SmartPadv10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2657366 = {S6834, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; TT-2500 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2657482 = {S6835, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Sygnus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2657492 = {S6836, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M702VM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2657602 = {S6837, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; Dimo900 Plus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2657683 = {S6838, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SP-700-A8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2657691 = {S6839, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; optima vtab Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2657747 = {S6840, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TB-712 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2657763 = {S6841, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; LEOTEC LETAB704 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2657779 = {S6842, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X714 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2657892 = {S6843, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AMI-TT2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2657960 = {S6844, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; S4501 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2657961 = {S6845, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SC3077 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2658044 = {S6846, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; D1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2658062 = {S6847, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q75S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658224 = {S6848, 152, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - I10S Linux/3.0.13 Android/4.0.4 Release/12.10.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2658339 = {S6849, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; MMD-X100   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2658557 = {S6850, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CDR KING FP-009-XON Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2658616 = {S6851, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; H8000+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658902 = {S6852, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; 3Q_ER71B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659080 = {S6853, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; S3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2659082 = {S6854, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MFC175FR_02 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2659155 = {S6855, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; A99 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2659188 = {S6856, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ekt KA-5926 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659256 = {S6857, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; A10GXM75 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2659313 = {S6858, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/Android4.0.3 Release/12.28.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2659596 = {S6859, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q74 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659638 = {S6860, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AKL_M501 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659690 = {S6861, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; ASM722HCN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2660049 = {S6862, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE N881F Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2660453 = {S6863, 148, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Pentagram Monster Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2660595 = {S6864, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; PA0702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2660906 = {S6865, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; 3D life+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2660940 = {S6866, 148, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4(6577); es-es; i-Call 701  Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2661282 = {S6867, 148, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MASHAIL EXTREME_I Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2662066 = {S6868, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; mi-book Mi8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2662386 = {S6869, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TELETALK_X100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2662621 = {S6870, 166, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; X40i Build/BCM21654AndroidTK_SystemRel_1.3.2P.05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2662641 = {S6871, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MID-9108 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2662948 = {S6872, 149, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Tokio_Smartphone_1_0 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2663109 = {S6873, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2664003 = {S6874, 151, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - F51 Linux/3.0.13 Android/4.0.4 Release/01.23.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2664153 = {S6875, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; START101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2664423 = {S6876, 152, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - V102 Linux/3.0.13 Android/4.0.4 Release/02.01.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2664513 = {S6877, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2664592 = {S6878, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; W80 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665095 = {S6879, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID9762 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2665267 = {S6880, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A13-Q88D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665277 = {S6881, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A751 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665281 = {S6882, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GP501S Build/ICS_IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665404 = {S6883, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; X-5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665664 = {S6884, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; G1 snowy+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665777 = {S6885, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V700NA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665797 = {S6886, 173, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; M83g Build/M83g-eng 4.0.4 IMM76D test-keys_OTA:MID_2.1.1.1_X.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2665839 = {S6887, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Z3Z6Z7W Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2665993 = {S6888, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; YPY_07STB Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2666292 = {S6889, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SP-360 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666349 = {S6890, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JOYPAD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666385 = {S6891, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; G-M7904 Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2666552 = {S6892, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; A26 Build/CROSSCROSS_A26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666814 = {S6893, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2667516 = {S6894, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; EverShine Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2667731 = {S6895, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TPC0754 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2684343 = {S6896, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; TM-9720 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2684992 = {S6897, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; T736B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2685487 = {S6898, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CRYM1005HN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2685841 = {S6899, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; VertiCool AndyPad3 Build/IML74K)  AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2686093 = {S6900, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; SPC 5064N Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2686139 = {S6901, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Infinity Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2686246 = {S6902, 124, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; 2GO Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2686694 = {S6903, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; W686 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2686780 = {S6904, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT8000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2686832 = {S6905, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; MB1003 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2687034 = {S6906, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TB 712 Build/RK2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2687058 = {S6907, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; TC-971 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2687304 = {S6908, 107, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3) AppleWebKit/535.70 (KHTML, like Gecko) Version/4.0 Mobile Safari/535.700*/
const Device UA2687325 = {S6909, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; 9120 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2687796 = {S6910, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; KaiCloud 744 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2687843 = {S6911, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; I97D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2688191 = {S6912, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; TAB-9701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2688504 = {S6913, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-ca; 9701BT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2689087 = {S6914, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LNV-Lenovo_A710e Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2689216 = {S6915, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT685 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2689834 = {S6916, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; TCL S600 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2691557 = {S6917, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; M-MP707i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2692064 = {S6918, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; E900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2692230 = {S6919, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MID7109 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2692327 = {S6920, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Haier HW-W718 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2692701 = {S6921, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; 7266 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2692858 = {S6922, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; 9220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2692925 = {S6923, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; A120 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2693041 = {S6924, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; BTPC-1010 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2693110 = {S6925, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ID1018WTA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2693183 = {S6926, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Coolpad 7290 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2693298 = {S6927, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Telus HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2693335 = {S6928, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; I 16 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2693338 = {S6929, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M762B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2693406 = {S6930, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; EGS001G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2693669 = {S6931, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; AMI PAD 10 IPS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2693723 = {S6932, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; VB73G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2693799 = {S6933, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; zh-cn; K-Touch U6 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2693824 = {S6934, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; TAB Matrix Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2693894 = {S6935, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; EVOPAD R7000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2693904 = {S6936, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; M807 Build/ICS.g08refem618.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2693905 = {S6937, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Holiday_X710a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2693960 = {S6938, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Tablet PC 2000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2693961 = {S6939, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; el-gr; me12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2694091 = {S6940, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fa-ir; Dimo 700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2694219 = {S6941, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; M736 Build/ICS.g08ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2694309 = {S6942, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; Exagerate XZPAD970H3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2694344 = {S6943, 125, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; T29 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2694352 = {S6944, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MA901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2694592 = {S6945, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MFC190BBFR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2694703 = {S6946, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SKYTEX-SP438 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2694766 = {S6947, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; I3000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2694866 = {S6948, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch E688 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695003 = {S6949, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695023 = {S6950, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; HITOMI Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2695078 = {S6951, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TB712 Build/RK2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2695086 = {S6952, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; KidsPad2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695455 = {S6953, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Axxion ATAB-902 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2695457 = {S6954, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TWZ HD8 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695480 = {S6955, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Note_mini Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695506 = {S6956, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; CW V80 Build/ICS.g08refem618.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2695550 = {S6957, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Android_53 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695563 = {S6958, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB73GP Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695694 = {S6959, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; aigoPad T710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695713 = {S6960, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; IMO X5 Build/IMO.X5_MARS.V09.XAB) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2695765 = {S6961, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V700D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2695794 = {S6962, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN700T Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2695880 = {S6963, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; ibho II  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2695983 = {S6964, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; SuzukiRx7 Build/SuzukiRx7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2696118 = {S6965, 149, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; Exagerate XZPAD700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2696123 = {S6966, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GY-M1006 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2696235 = {S6967, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Vexia Tablet 780 GPS Build/tablet-b) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2696238 = {S6968, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; SAMURAI Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2696292 = {S6969, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; TECHNAXX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2696372 = {S6970, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SC-91MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2696394 = {S6971, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XW-I8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2696661 = {S6972, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; MiPad 4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2696828 = {S6973, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CDRKING FP010XON Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2696870 = {S6974, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; TAB9 V4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2696903 = {S6975, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; MTK5000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2696927 = {S6976, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; IMO S68 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2696938 = {S6977, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S5_77_9P017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2697000 = {S6978, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V800H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2697096 = {S6979, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TB1082B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2697106 = {S6980, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q27B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2697262 = {S6981, 151, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-STYLE Q6A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2697275 = {S6982, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; 8190 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2697428 = {S6983, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SRT_AN4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2697489 = {S6984, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Tokio Tablet 2.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2697746 = {S6985, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PAD 1001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2697902 = {S6986, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; M92 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2698053 = {S6987, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q88 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2698063 = {S6988, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TS9708B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2698163 = {S6989, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; EUTB-708 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2698164 = {S6990, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MFC140FR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2698189 = {S6991, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KT968 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2698272 = {S6992, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Xtab781HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2698290 = {S6993, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; M-MP705I Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2698347 = {S6994, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; android ics 4.04 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2698388 = {S6995, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; CityTab Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2698857 = {S6996, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Proton Zero Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2699183 = {S6997, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; KDI-W360   Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2699369 = {S6998, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TPC-EA900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2699397 = {S6999, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Netbook PC 12700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2699748 = {S7000, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; TREQ 3G Turbo Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2699807 = {S7001, 149, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PMID-971C Build/ICS.g12refM1005SPIBL.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2699976 = {S7002, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; TVSmart Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2700039 = {S7003, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Z99 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2700172 = {S7004, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; mTAB StreaQ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2700183 = {S7005, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; E703J Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2700387 = {S7006, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Strike Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2700445 = {S7007, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Xtab1081HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2700523 = {S7008, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; epade_i5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2700661 = {S7009, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T98 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2700794 = {S7010, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOTE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2700886 = {S7011, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AP2-16-1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2700944 = {S7012, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAB-73 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2701112 = {S7013, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; E8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2701122 = {S7014, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VIA101TAB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2701214 = {S7015, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; Venera Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2701249 = {S7016, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; AT9N-A13WP Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2701252 = {S7017, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A704_INSPIRE_X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2701254 = {S7018, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; s4502 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2701519 = {S7019, 161, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - CETRIX-CS1280 Linux/3.0.13 Android/4.0.4 Release/12.11.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2701678 = {S7020, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; Q73 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2701745 = {S7021, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; P88(H3D7) Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2701967 = {S7022, 58, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; BT210 Build/IMM76D)*/
const Device UA2702089 = {S7023, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D26 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2702576 = {S7024, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SMP53-210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2702651 = {S7025, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; SKY Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2702674 = {S7026, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; AQ FIVE Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2703300 = {S7027, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; L29 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2703546 = {S7028, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; CTLITE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2703637 = {S7029, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Vexia Tablet 780 GPS Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2703725 = {S7030, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Brand Infinityll Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2703774 = {S7031, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; MHL3 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2703968 = {S7032, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; SM70 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2704435 = {S7033, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT7330 Build/ICS_IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2704442 = {S7034, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; YPY_07FTB Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2704593 = {S7035, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U; Android 4.0.4; en-us; MID103C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2704745 = {S7036, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2705387 = {S7037, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-en; ETAB 0307 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2705925 = {S7038, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; M84g Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2706096 = {S7039, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; CROSS A28 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2706326 = {S7040, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; RX3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2707613 = {S7041, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; PULSO 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708221 = {S7042, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; PA0910CA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2709449 = {S7043, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; haw-US; U7 Build/GRK39F) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2711776 = {S7044, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EVOTAB Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2712286 = {S7045, 152, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - E1-B Linux/3.0.13 Android/4.0.4 Release/01.31.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2721398 = {S7046, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; N007 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2722504 = {S7047, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; MP810C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2722952 = {S7048, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ALUMIUM10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2722962 = {S7049, 152, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; Zyrex_ZA987_OneScribe Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2723492 = {S7050, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SCROLL_BASIC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2723515 = {S7051, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; X825a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2723823 = {S7052, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; I93 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2723960 = {S7053, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CW-V17 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2724133 = {S7054, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; MVM908HCZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2724849 = {S7055, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Woxter Tablet PC 90BL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2725180 = {S7056, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; T820 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2725350 = {S7057, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; ELIYA S2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2725365 = {S7058, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; T9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2725512 = {S7059, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; Kesa Tablet Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2726045 = {S7060, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; P4D Sirius Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2726235 = {S7061, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2726433 = {S7062, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; aigopad Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2726455 = {S7063, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; v128 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2726542 = {S7064, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; S70 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2726552 = {S7065, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; SPC neo4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2726675 = {S7066, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru;  DIGMA iDs10 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2726895 = {S7067, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ENERGY s10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2726898 = {S7068, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; zh-cn; K-Touch W760 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2726923 = {S7069, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Informer 704 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2726970 = {S7070, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SIROCO 3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2727081 = {S7071, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TM-MID760 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2727211 = {S7072, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Tabplus 7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2727296 = {S7073, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; M08W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2727342 = {S7074, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; T910 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2727373 = {S7075, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB-N8BS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2727398 = {S7076, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; da-dk; TAC-7028 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2727673 = {S7077, 148, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; LG OPTIMUS HD LTE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2727755 = {S7078, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT10K-BT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2727740 = {S7079, 56, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Q88 Build/IMM76D)*/
const Device UA2727909 = {S7080, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GPLUS_GN878 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2727956 = {S7081, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TAB7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2728012 = {S7082, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; ENOT J101 Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2728194 = {S7083, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; Victory_S3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2728224 = {S7084, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; GTR848 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2728265 = {S7085, 152, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DEVANTE SMART CALLING Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2728428 = {S7086, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DEW H1406 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2728483 = {S7087, 174, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; S9220 Build/[S812T]S812T_HPD_H812_W12.09.10_V01B11_Release) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2728543 = {S7088, 152, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Cat Tablet Galactica X 9.7CA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2728574 = {S7089, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SW-ARMM8S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2728599 = {S7090, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.8; en-us; 9300I Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2728853 = {S7091, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GV7777 Build/GV7777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2728925 = {S7092, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M747 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2728951 = {S7093, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; NTT 617 Build/ICS.g08ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2729065 = {S7094, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; TAB0004 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2729104 = {S7095, 148, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SABRESD-MX6DQ Build/R13.4-beta) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2729220 = {S7096, 159, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - SXZ-PDX0-01 Linux/3.0.13 Android/4.0.4 Release/11.15.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2729271 = {S7097, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C92d Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2729401 = {S7098, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TPC1050 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2729543 = {S7099, 157, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - AMOI N816 Linux/3.0.13 Android/4.0.4 Release/12.19.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2729642 = {S7100, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; AT7D-TE25DA Build/AT7D-TE25DA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2729682 = {S7101, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB 170N Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2729838 = {S7102, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S708 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2729870 = {S7103, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2729943 = {S7104, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Lafone SIII Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2730028 = {S7105, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; PTS CA SERIES Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2730339 = {S7106, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I5-77 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2730381 = {S7107, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HD II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2731205 = {S7108, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; DNS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2731521 = {S7109, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; PICOpad GGD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2731683 = {S7110, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Explosion Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2732090 = {S7111, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; s4503 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2732151 = {S7112, 151, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GTEL_A706_INFINITY_S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2732170 = {S7113, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; es-es; A822 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2732247 = {S7114, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; S5301 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2732284 = {S7115, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; TH-1130M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2732362 = {S7116, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X716S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2732410 = {S7117, 160, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sk-sk; V8200-Memphis-orange-Slovakia Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2732688 = {S7118, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; M93 Build/ICS.g08refema9.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2732733 = {S7119, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; M800 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2733383 = {S7120, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GASEM712MC_WP Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2733392 = {S7121, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MD-0697 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2733955 = {S7122, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; F8GT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734188 = {S7123, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB 280 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2734377 = {S7124, 148, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; GO Live Dual Core Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734663 = {S7125, 151, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - E1A Linux/3.0.13 Android/4.0.4 Release/01.26.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2735120 = {S7126, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AW710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2735132 = {S7127, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; DL HD EV 01 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2735770 = {S7128, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2736055 = {S7129, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; GLX Jet Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2736484 = {S7130, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; F307 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2736775 = {S7131, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; i-Call 300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2738764 = {S7132, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 7878_c Build/EDU) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2742939 = {S7133, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; MD-828 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2744985 = {S7134, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; M758A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2745709 = {S7135, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID72C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2745803 = {S7136, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; S4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2745942 = {S7137, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Q8.3G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2746284 = {S7138, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 9702T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2746294 = {S7139, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; EVOLVE_FX400 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2746326 = {S7140, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ar-eg; GX8404 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2746417 = {S7141, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; RC0712B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2746578 = {S7142, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; deovo V5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2746774 = {S7143, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A106 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2746806 = {S7144, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; zh-tw; PAD1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2746784 = {S7145, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ratech77_ics2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2746996 = {S7146, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; Andronote Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747009 = {S7147, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BIRDA_MODEL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747028 = {S7148, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.6; es-es; MID7102SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2747045 = {S7149, 59, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; UTAB71 Build/IMM76D)*/
const Device UA2747105 = {S7150, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; AOLE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747127 = {S7151, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; HWA660 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747363 = {S7152, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Explay Informer 705 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2747413 = {S7153, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AC-77 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747431 = {S7154, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; DK mobile Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747675 = {S7155, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; hu-hu; voyager GT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2747814 = {S7156, 152, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; SHIRU SHOGUN(QuadCore) Build/SHIRU_SHOGUN) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2747995 = {S7157, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GFIVE Glory(A86) Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2748002 = {S7158, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; NT-1710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2748050 = {S7159, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; xgame II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2748144 = {S7160, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I90 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2748249 = {S7161, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch E621 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2748286 = {S7162, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; M009S 16GB RTB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2748292 = {S7163, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M006-Q Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2748545 = {S7164, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; NITRO8B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2748573 = {S7165, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Blast Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2748589 = {S7166, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N79 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2748607 = {S7167, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Magnum Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2749134 = {S7168, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; EZPAD 810C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2749178 = {S7169, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CRUISE_TAB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2749340 = {S7170, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TAB9008GB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2749631 = {S7171, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Walton Primo Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2749678 = {S7172, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Artel-Adi5 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2749685 = {S7173, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 7D-1A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2749804 = {S7174, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Condor Tab SMB-B9705 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2749825 = {S7175, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; 2627 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2749879 = {S7176, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S7t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2750113 = {S7177, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MTK5000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2750337 = {S7178, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; O1A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2750475 = {S7179, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; SPC P1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2750889 = {S7180, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Maxpro Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2750948 = {S7181, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; CR9A13-DC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2751441 = {S7182, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AMAZE TAB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2751733 = {S7183, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.5; en-us; MTK6515 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2751807 = {S7184, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; M009S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2758316 = {S7185, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A102C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2758386 = {S7186, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; OBSIDIAN1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2758734 = {S7187, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M901-W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2759125 = {S7188, 57, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; M71G Build/IML74K)*/
const Device UA2759191 = {S7189, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; TCL S900 Build/S900Q-2SLCCN1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2759332 = {S7190, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; California2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2759689 = {S7191, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Newpad Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2759843 = {S7192, 124, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; DOOV_D7 Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2759860 = {S7193, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; ASK 110SP Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2760106 = {S7194, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M803HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2760129 = {S7195, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A710e Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2760526 = {S7196, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 9710D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2761230 = {S7197, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GGC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2761607 = {S7198, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CW V80 Build/ICS.g08refem618.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2761643 = {S7199, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; TCL S800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2761747 = {S7200, 121, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; 9220 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2761780 = {S7201, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IT-M744AL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2762094 = {S7202, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U960s3 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2762154 = {S7203, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Vexia Navlet 2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2762183 = {S7204, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; Xfab Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2762185 = {S7205, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; SD02 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2762232 = {S7206, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; 9700DN Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2762304 = {S7207, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A530/V3.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2762333 = {S7208, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; WizTab WT 7.03 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2762433 = {S7209, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.6; es-es; MPM_AVA758 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2762544 = {S7210, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; RAPAX 070-0508 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2762559 = {S7211, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; H5000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2762589 = {S7212, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; TP7-1000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2762678 = {S7213, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PAD 9715D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2762699 = {S7214, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch E806 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2762701 = {S7215, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; DOOV_D9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2762763 = {S7216, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-eg; 07/083 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2762778 = {S7217, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TW-900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2762839 = {S7218, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; MTC_968 Build/ICECREAM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2762849 = {S7219, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; QUER KOM0330 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2762902 = {S7220, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; StarPAD 7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2762941 = {S7221, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CVD601 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2762958 = {S7222, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; E9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2762985 = {S7223, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-au; Z15A Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2763134 = {S7224, 148, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; epade_S90 Build/epadeepade_S90) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2763221 = {S7225, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LXT1001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2763447 = {S7226, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; H1W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2763660 = {S7227, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; NTT 618 Build/ICS.g08refem618.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2763698 = {S7228, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; QS0730C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2763770 = {S7229, 151, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Connspeed AS9 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2763962 = {S7230, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; NetTAB SKY 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2764046 = {S7231, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MFC415FR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2764088 = {S7232, 155, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; ar-eg; BERAQ NB898 Build/VENUS_00.03.168.H1121) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2764091 = {S7233, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; L36 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2764116 = {S7234, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; DL 3G MOBILE TG-M71 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2764271 = {S7235, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; AP-803 Build/ICS.AP-803.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2764329 = {S7236, 148, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; Avvio_790 Build/AvvioAvvio_790) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2764521 = {S7237, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ViewPad E72 Build/ICS.VSA.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2765380 = {S7238, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2765879 = {S7239, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A703J Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2766116 = {S7240, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SPT-800 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2766311 = {S7241, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; HISENSE F0 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2766687 = {S7242, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TA-734 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2766862 = {S7243, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Vision Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2766962 = {S7244, 152, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; COMPUMAX Build/ICS.compumax_30_300_8188.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2767070 = {S7245, 149, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MASHAIL EXTREME_II Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2767335 = {S7246, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; DL X10Note X1-N11 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2767530 = {S7247, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; aTAB503D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2767534 = {S7248, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Informer 705 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2767652 = {S7249, 206, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; sprd-M2/1.0 Android/4.0.4 Release/04.02.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2767857 = {S7250, 156, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android 4.0.3;en-us;Digitac Ultratab;C714 Build/ICS) AppleWebKit/531.21.10(KTHML,like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2768149 = {S7251, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HX-M02 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2768482 = {S7252, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; L29i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2775103 = {S7253, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; U910 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2775572 = {S7254, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; GPad Explorer Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2775634 = {S7255, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC S720t Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2775690 = {S7256, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ro-ro; Neura 3G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2775708 = {S7257, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; PULID F10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2775762 = {S7258, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; TC-7110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2776036 = {S7259, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; xDevice Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2777332 = {S7260, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; V7S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2777461 = {S7261, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; vi-vn; VipPhone V4S Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2777505 = {S7262, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; DOOV_D900 Build/DoovDOOV_D900) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2777643 = {S7263, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; SpeedUp S5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2777645 = {S7264, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; my-Mate 3D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2777646 = {S7265, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; JW101A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2777675 = {S7266, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Informer 706 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2777762 = {S7267, 57, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; 5910 Build/IMM76I)*/
const Device UA2777764 = {S7268, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D101B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2777790 = {S7269, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; C8715M Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2777846 = {S7270, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; 4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2778039 = {S7271, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; mi-book Mi5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778115 = {S7272, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; INFINITY_PASSION Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778173 = {S7273, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 4SPA752C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778241 = {S7274, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M750B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778291 = {S7275, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; OS458 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778370 = {S7276, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; M81w Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2778378 = {S7277, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; UM-A90 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778489 = {S7278, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE N983 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778509 = {S7279, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; MD803S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2778518 = {S7280, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch E780 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778557 = {S7281, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A99 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778666 = {S7282, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U802 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778729 = {S7283, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 4U Coral Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778785 = {S7284, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; 556 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2778792 = {S7285, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Informer 705 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2778901 = {S7286, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778935 = {S7287, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cyrus 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2779001 = {S7288, 161, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - ratech77_ics2 Linux/3.0.13 Android/4.0.4 Release/11.21.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2779126 = {S7289, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; IHA-C0901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2779129 = {S7290, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; MOBO Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2779250 = {S7291, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; 712F(0703) Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2779279 = {S7292, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; JC-898-MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2779588 = {S7293, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; Bpad2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2779721 = {S7294, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TPC-7HG Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2779878 = {S7295, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; WFJWAE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2779953 = {S7296, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB 0170N Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2780060 = {S7297, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GSM30 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2780063 = {S7298, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; TY006 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2780184 = {S7299, 115, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; YL-Coolpad_5876/4.0.021.X.5876; 540*960; CTC/2.0) CoolpadWebkit/534.30*/
const Device UA2780234 = {S7300, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Mi406 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2780308 = {S7301, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; M704 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2780950 = {S7302, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MOBICEL HERO Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781044 = {S7303, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ViewPad 7D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781106 = {S7304, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A704H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781310 = {S7305, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; T98 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781418 = {S7306, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; Phonei5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2781905 = {S7307, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A78-M02 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2782032 = {S7308, 149, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; Nova Pad 70 C124FC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2786761 = {S7309, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.8; vi-vn; S510+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2786898 = {S7310, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; Airpad-DS101M Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2787305 = {S7311, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Q3M907HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2788133 = {S7312, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; WT5.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2788285 = {S7313, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; TPAD-100A3GECA Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2789371 = {S7314, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TR976 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2789404 = {S7315, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; I-MEMO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2789658 = {S7316, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; V960 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2789667 = {S7317, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; 7266 Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2789909 = {S7318, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; ANDROpad TA-12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2789999 = {S7319, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; INW908HCZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2790002 = {S7320, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; epade N01 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2790014 = {S7321, 59, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; gs701b Build/IML74K)*/
const Device UA2790035 = {S7322, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MPCA410VD001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2790199 = {S7323, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; vi-vn; Smartphones Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2790318 = {S7324, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; N80DC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2790462 = {S7325, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Kaosa K3G1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2790754 = {S7326, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M9702T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2790821 = {S7327, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MID1107 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2790957 = {S7328, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U1203 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2791000 = {S7329, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; Ex'S1   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2791048 = {S7330, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; WT7.0g Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2791076 = {S7331, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T201 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2791085 = {S7332, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB 0170 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2791114 = {S7333, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MFC250EN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2791134 = {S7334, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; a760sSamSungGS3 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2791259 = {S7335, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; MID9711 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2791312 = {S7336, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; VM0711A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2791330 = {S7337, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; NB726A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2791380 = {S7338, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; B9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2791445 = {S7339, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; IMO Y-One Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2791518 = {S7340, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; TS 501 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2791523 = {S7341, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Livepad 7.5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2791541 = {S7342, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; PB-W20 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2791584 = {S7343, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CY26 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2791627 = {S7344, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; SP-702-A5 Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2792094 = {S7345, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; Q86 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2792152 = {S7346, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q88B-11S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2792168 = {S7347, 158, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - MALATA I10 Linux/3.0.13 Android/4.0.4 Release/01.10.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2792512 = {S7348, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AX8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2792556 = {S7349, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i800 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2792942 = {S7350, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; Everest Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2792984 = {S7351, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; E780 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2793137 = {S7352, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Sky 3G DUO Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2793351 = {S7353, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ETAB-1031 Build/ICS.g08refmid1061.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2793404 = {S7354, 148, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; TRACER OXYGEN GS1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2793866 = {S7355, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TPC0702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2793953 = {S7356, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; 8720 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2793972 = {S7357, 154, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - NIU D8 Linux/3.0.13 Android/4.0.4 Release/03.07.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2794028 = {S7358, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HBD-761 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2794070 = {S7359, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; S-534 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2794264 = {S7360, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; N8010_APT Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2794304 = {S7361, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; Tab745 Build/GT704-A13005-H01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2794382 = {S7362, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-gb; N005 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2794440 = {S7363, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 10901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2794497 = {S7364, 155, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - TWZ AA8 Linux/3.0.13 Android/4.0.4 Release/11.27.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2794554 = {S7365, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMID900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2795205 = {S7366, 150, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; DL 3G MOBILE TG-M71 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2795462 = {S7367, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; 5858 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2795515 = {S7368, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MI360 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2795659 = {S7369, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; E704H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2795761 = {S7370, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Infinix X400 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2796264 = {S7371, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EX7201 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2797207 = {S7372, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M700G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2808924 = {S7373, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; wetab Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2809080 = {S7374, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; QW TB-G100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2809941 = {S7375, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I748 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2810085 = {S7376, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CRYM805HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2810120 = {S7377, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; vi-vn; V8 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2810379 = {S7378, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; HTC VAE_WX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2811220 = {S7379, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Behold BeTAB Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2811315 = {S7380, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; LANDA Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2811484 = {S7381, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; G.Pad EXplorer Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2813123 = {S7382, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; EPAD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2814263 = {S7383, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; prevail Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2814902 = {S7384, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; TAB971 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2815194 = {S7385, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; Quicki 730 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2815235 = {S7386, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; TM-7027W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2815893 = {S7387, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC A320e Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2816366 = {S7388, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; PRO-7.0 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2816394 = {S7389, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Ideapad S10-3T Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2816506 = {S7390, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; SONY Q7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2816902 = {S7391, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; E900 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2817015 = {S7392, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; M700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2817060 = {S7393, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; E170BS Build/C006) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2817064 = {S7394, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 101S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2817198 = {S7395, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nb-no; ARGM816HCX_A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2817256 = {S7396, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; N83 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2817798 = {S7397, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; T1001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2818143 = {S7398, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; thinkpad Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2818144 = {S7399, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID700CVD102 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2818277 = {S7400, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; N7T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2818454 = {S7401, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; AUX V920 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2818688 = {S7402, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; OnePAD 1100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2818841 = {S7403, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; 7" DELTA Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2819022 = {S7404, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; BenQ R71 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2819252 = {S7405, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; msm7627a Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2819567 = {S7406, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S5/C2_77_5647 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2820120 = {S7407, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; UBT081X Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2822372 = {S7408, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT685 Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2822790 = {S7409, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; KPT A11PLUS Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2824352 = {S7410, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-ja; CTB-701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2824373 = {S7411, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Swipe Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2824459 = {S7412, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; IRUX M8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2824614 = {S7413, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GPLUS GN700W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2825043 = {S7414, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; OnePAD 735 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2825830 = {S7415, 60, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; aigopad Build/IMM76D)*/
const Device UA2825927 = {S7416, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; lt-lt; Estar Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2826855 = {S7417, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GN868 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2828459 = {S7418, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; PocketPad Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2828711 = {S7419, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; PC7342 Build/RK2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2828737 = {S7420, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sk-sk; TA8200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2829782 = {S7421, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; HWA600 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2830061 = {S7422, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Acer AK330s Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2830300 = {S7423, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2830362 = {S7424, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; CAPTIVA PAD 8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2831320 = {S7425, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; M78D Build/M78D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2831516 = {S7426, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; OR-TB7010 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2832183 = {S7427, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; treq A10G Duo Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2832954 = {S7428, 58, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; M758A Build/IMM76D)*/
const Device UA2833021 = {S7429, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ET8001C-G12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2833132 = {S7430, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; Consul 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2833400 = {S7431, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; UPAD 9A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2834461 = {S7432, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Quicki-732 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2835024 = {S7433, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; ru-ru; U920+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2835166 = {S7434, 157, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID-7089D Dual Core Build/ICS.g12refM703A1XK.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2835238 = {S7435, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; MAG101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2835662 = {S7436, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Informer 703 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2835976 = {S7437, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; virtual Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2836060 = {S7438, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MIDR409 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2836224 = {S7439, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; XPRESS PRO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2836298 = {S7440, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D07 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2836434 = {S7441, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch W780 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2836596 = {S7442, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T-907 Build/T-907) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2836841 = {S7443, 166, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MOT-XT889/6.7.2_GC-200-YTZWE-3; 540*888; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2837454 = {S7444, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; BAUSSEN BEN758N Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2837667 = {S7445, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-co; SI09A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2838044 = {S7446, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; CNC-TAB7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2838076 = {S7447, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; P80w Build/SuningP80w) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2838315 = {S7448, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MIDR477 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2838891 = {S7449, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; BKA-C1004 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2839143 = {S7450, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; A85 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2839753 = {S7451, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Mi501 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2839997 = {S7452, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; S7916E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2840144 = {S7453, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 900TPC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2840274 = {S7454, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; SP-A20i Build/MF_ICS_02.19) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2840983 = {S7455, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AM708-JH Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2841106 = {S7456, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Advanced II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2841200 = {S7457, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/Android4.0.3 Release/09.25.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2841292 = {S7458, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MIDC711PR001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2842049 = {S7459, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; MID13 Build/MID13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2842097 = {S7460, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KB706 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2843021 = {S7461, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ETON T850 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2843022 = {S7462, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; NetTAB RUNE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2843116 = {S7463, 157, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - DOUBLE IV Linux/3.0.13 Android/4.0.4 Release/10.25.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2843126 = {S7464, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SK2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2843276 = {S7465, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; E-Tab 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2843386 = {S7466, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M906G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2843513 = {S7467, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; PMID80C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2843679 = {S7468, 59, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; TRIO7C Build/IMM76D)*/
const Device UA2843782 = {S7469, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; L26 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2843799 = {S7470, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; F8A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2843870 = {S7471, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A90-2core Build/R13.4-rc2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2843989 = {S7472, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; OMSP4048MWT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2844160 = {S7473, 153, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - I9220 Linux/3.0.13 Android/4.1.1 Release/10.16.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2844267 = {S7474, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; G7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2844369 = {S7475, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; OPPOR601 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2844370 = {S7476, 58, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GN180 Build/IMM76D)*/
const Device UA2844613 = {S7477, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; WT-PAD012 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2844700 = {S7478, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U910 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2844736 = {S7479, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; FS-M787S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2844828 = {S7480, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; SmartPad1010i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2844905 = {S7481, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; REVO MAX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2845366 = {S7482, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N91 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2845497 = {S7483, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; F8G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2845958 = {S7484, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN135 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2846113 = {S7485, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; monga M8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2846140 = {S7486, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; FS-S4300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2846147 = {S7487, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AND5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2846603 = {S7488, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; SHOGUN Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2846650 = {S7489, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; zh-cn; LA-I Dual core Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2846913 = {S7490, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; tcc8800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2846939 = {S7491, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; FLY_IQ310 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2847175 = {S7492, 59, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; M702VM Build/IMM76D)*/
const Device UA2847205 = {S7493, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SKYTEX-SP705 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2847303 = {S7494, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Q88c_a13 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2847315 = {S7495, 60, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; A13-DLR Build/IMM76D)*/
const Device UA2847365 = {S7496, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NEO X5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2847476 = {S7497, 160, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Coolpad 7290 Linux/3.0.13 Android/4.0.4 Release/11.27.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2847478 = {S7498, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; INU001D Build/INU001D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2847515 = {S7499, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 975PTC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2847581 = {S7500, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S580 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2847638 = {S7501, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; PAN 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2847642 = {S7502, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MP547 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2847643 = {S7503, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MFC140FR1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2847675 = {S7504, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; MFC270DE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2847978 = {S7505, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; RC9713B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2848109 = {S7506, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; U-TAB720 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2848144 = {S7507, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID-1089D Build/ICS.g12refM1006XK.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2848168 = {S7508, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN700T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2848231 = {S7509, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Digma iDj7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2848246 = {S7510, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iTab M7i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2848248 = {S7511, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; GUNIUS713 Build/GUNIUS713) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2848320 = {S7512, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sl-si; sc3068n Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2848331 = {S7513, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; 118 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2848411 = {S7514, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; MTC 1065 Build/MTC_RU) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2848485 = {S7515, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TB-PH03 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2848808 = {S7516, 148, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SmartPadv10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2848871 = {S7517, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; PENTAGRAM TAB 8.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2848957 = {S7518, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; zh-cn; LA-I Dual core Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2849079 = {S7519, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Z1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2849489 = {S7520, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HY731 Build/ICS.f08refem31.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2849506 = {S7521, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; E830 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2849769 = {S7522, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; zh-cn; A811 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2850051 = {S7523, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2850094 = {S7524, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; FW8932 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2850134 = {S7525, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TM475 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2850265 = {S7526, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; T9108 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2850422 = {S7527, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; K-BOOK PLUS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2850667 = {S7528, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; S88 Build/S750) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2851542 = {S7529, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; JT_SmartPC02 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2853504 = {S7530, 160, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Coolpad 7235 Linux/3.0.13 Android/4.0.4 Release/01.12.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2853613 = {S7531, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; G500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2853838 = {S7532, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; B93 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2854170 = {S7533, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; A801 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2854230 = {S7534, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; konka W970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2854447 = {S7535, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; OPS-50 Build/S710) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2854467 = {S7536, 123, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; SN7005 Build) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2854578 = {S7537, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; RMD-1070 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2854673 = {S7538, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID-HC719U Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2854958 = {S7539, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; ELIYA_S868 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2855002 = {S7540, 159, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Victory_A18 Linux/3.0.13 Android/4.0.4 Release/02.23.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2855051 = {S7541, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sl-si; C450 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2855216 = {S7542, 157, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - A117_MULB Linux/3.0.13 Android/4.0.4 Release/01.24.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2855292 = {S7543, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M4304 Build/IML74K-V1.2-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2855616 = {S7544, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy S3 EX V2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2855606 = {S7545, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 741C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2855735 = {S7546, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Lenovo A356:Mozilla/5.0 (Linux; U;  Android 4.0.4; zh-cn; Lenovo A356/S030) AppleWebKit534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2857230 = {S7547, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; M73 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2857447 = {S7548, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; MIDR237 Build/MIDR237) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2857553 = {S7549, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; ZA985 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2857819 = {S7550, 123, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; SN7004 Build) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2858020 = {S7551, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; G40C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2858198 = {S7552, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; S3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2858497 = {S7553, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Sen Pro One Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2858551 = {S7554, 153, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - M1077 Linux/3.0.13 Android/4.1.1 Release/01.02.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2858904 = {S7555, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ar-eg; i7008 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2859193 = {S7556, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB 180 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2859325 = {S7557, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; AirTab P970g Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2859905 = {S7558, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; NB 1016A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2860247 = {S7559, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; A-PAD 114+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2861298 = {S7560, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; T8 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2861475 = {S7561, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; Pandora 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2861650 = {S7562, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; NXM900MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2862766 = {S7563, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; SNOWY Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2862892 = {S7564, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT7220S2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2863135 = {S7565, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; WAISONIC TABLET Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2863216 = {S7566, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; 712F(0901) Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2863357 = {S7567, 148, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; V9-1688-CJ-0308-31S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2863392 = {S7568, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-tw; HTC_PROMIN_U Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2863393 = {S7569, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Z7 Build/Android4.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2863517 = {S7570, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; equiso.smart.tv.pro Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2864227 = {S7571, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TPC0708 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2864293 = {S7572, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; VIT-T1310 Build/2.7AG) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2864652 = {S7573, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; YPY_AB7DC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2865130 = {S7574, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; J-Q88S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865180 = {S7575, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JD-716SG-1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865188 = {S7576, 210, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; zh-cn; sprd-GT8160/1.0 Android/4.0.1 Release/04.11.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2865284 = {S7577, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID9760 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2865407 = {S7578, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAB803 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2865442 = {S7579, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Y7-3L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865471 = {S7580, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KT918+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865525 = {S7581, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/Android4.0.3 Release/04.01.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2865551 = {S7582, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; M76 Build/ICS.g08refem667.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2865562 = {S7583, 155, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; PRO4611 Build/PRO4611-V1.0-2013.03.22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865618 = {S7584, 155, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mi-A401 Linux/3.0.13 Android/4.0.4 Release/03.29.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2865861 = {S7585, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AW880 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865877 = {S7586, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-gb; PI N005 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865900 = {S7587, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; f761B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2866486 = {S7588, 152, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - G515 Linux/3.0.13 Android/4.0.4 Release/12.11.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2866679 = {S7589, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; SAMURAI10(QuadCore) Build/SAMURAI10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2866917 = {S7590, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AIRIS TM475 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867141 = {S7591, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Y-B07N Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867535 = {S7592, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Spark Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2869389 = {S7593, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM204SH Build/S0004) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2874808 = {S7594, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TM450 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878108 = {S7595, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; zh-tw; imatch i15 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878153 = {S7596, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TAB 8200C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2878230 = {S7597, 61, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Q88B-12S Build/IMM76D)*/
const Device UA2878383 = {S7598, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U817 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2878872 = {S7599, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; ACTIVE 10-A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2878983 = {S7600, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SX Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2879016 = {S7601, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IT-708 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879069 = {S7602, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 3G7271 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879083 = {S7603, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S3PRO Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879403 = {S7604, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N8000_USA_RS Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879763 = {S7605, 219, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.2; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.0.2 Release/04.24.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2889002 = {S7606, 57, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; A088 Build/IML74K)*/
const Device UA2889004 = {S7607, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; TCL S600 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2889123 = {S7608, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; EBEST V7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2889191 = {S7609, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SFAS  T1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2889217 = {S7610, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CS9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2889223 = {S7611, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID09710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2889266 = {S7612, 157, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GIO-GiONEE_C610/SV1.0; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2889381 = {S7613, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GLORY Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2889395 = {S7614, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U880F1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2889421 = {S7615, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; 6577 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2889426 = {S7616, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JXD-S6600 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2889479 = {S7617, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; kiwi k5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2889661 = {S7618, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Zyrex_ZA989 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2890078 = {S7619, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; MyPad 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2890200 = {S7620, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; YPY_S400 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2890213 = {S7621, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; s4504 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2890347 = {S7622, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MT29 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2890425 = {S7623, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; GV7780 Build/GV7780) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2890764 = {S7624, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A630t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2890916 = {S7625, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; 5876 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2890968 = {S7626, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LXK901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2890975 = {S7627, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TM-MID788 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2890991 = {S7628, 148, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; SHOGUNI010(QualCore) Build/SHOGUNI010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2891052 = {S7629, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; IMO Y5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2891113 = {S7630, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; ELIYA S858 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2891184 = {S7631, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-en; bluedigit Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2891219 = {S7632, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Pixcom PGM398 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2891509 = {S7633, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; 8720 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2891517 = {S7634, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; SMP45-210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2891595 = {S7635, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; K-TOUCH W780 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2891667 = {S7636, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Vexia Tablet 780 Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2891732 = {S7637, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GPLUS-P7002 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892006 = {S7638, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; F817 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2892054 = {S7639, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; F326 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2892108 = {S7640, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HBD-761I Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892272 = {S7641, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; 7558L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892327 = {S7642, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; Next7D12 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2892382 = {S7643, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; PC7134 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2892766 = {S7644, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; @TAB-NINE-X1-B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892814 = {S7645, 206, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; sprd-M2/1.0 Android/4.0.4 Release/04.24.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2893950 = {S7646, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; POV_P517-8GB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2894452 = {S7647, 165, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; X45 Build/BCM21654AndroidTK_SystemRel_1.3.2P.05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2896966 = {S7648, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xplorer T8i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2933367 = {S7649, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; V8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2933524 = {S7650, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; P76v(A3V5) Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2933749 = {S7651, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ph; Starmobile ENGAGE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2933896 = {S7652, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2933914 = {S7653, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; X2E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2933941 = {S7654, 157, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GIO-GiONEE_C700/SV1.0; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2933944 = {S7655, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; technika8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2933952 = {S7656, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; M730 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2934021 = {S7657, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; OS969 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934046 = {S7658, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U960s3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934083 = {S7659, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; E738 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934085 = {S7660, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JXD-S9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934113 = {S7661, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; HWA680 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934306 = {S7662, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A9B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2934344 = {S7663, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; MID9035C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934394 = {S7664, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.2; zh-cn; senyujie Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2934482 = {S7665, 154, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; Navon Platinum Explorer Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934491 = {S7666, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Freelander I20 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934497 = {S7667, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; AVA747 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934525 = {S7668, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; KliTON-I51SH Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934549 = {S7669, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; OPSSON iMO1000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2934707 = {S7670, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; T9108 Build/MocorDroid2.3.5) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2934738 = {S7671, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; ar-eg; BERAQ NB9305 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2934768 = {S7672, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KONKA W990 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934811 = {S7673, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; VOVO_V5 Build/SunVOVO_V5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934863 = {S7674, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A630t Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2934870 = {S7675, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; 8150D Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934956 = {S7676, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MID-702SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2935174 = {S7677, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; T735 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2935187 = {S7678, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; START702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2935180 = {S7679, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; NIU TEK 3G 3.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2935234 = {S7680, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Coolpad 5213 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2935266 = {S7681, 213, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SAMSUNG-GT-B9388_TD/1.0 Android/4.0.4 Release/09.15.2012 Browser/AppleWebKit534.30 Build/IMM76D) ApplelWebkit/534.30 (KHTML,like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2935269 = {S7682, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; M713 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2935534 = {S7683, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Wb7-L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2935536 = {S7684, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U817 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2935981 = {S7685, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; G158 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2936001 = {S7686, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; JS401 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936008 = {S7687, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; f761I Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936067 = {S7688, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MB9500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936198 = {S7689, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 3G7334 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2936435 = {S7690, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HS-U909 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936533 = {S7691, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; Tablet Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936698 = {S7692, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; p507 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2936866 = {S7693, 163, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; AliyunOS 2.0; Android 4.0 Compatible; en-us; G9 Build/AliyunOs-2012) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937022 = {S7694, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; S85 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937060 = {S7695, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Atom Action Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937136 = {S7696, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; TAX Build/ICS.g08refmid1061.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2937205 = {S7697, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PTBL9C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937264 = {S7698, 151, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TJLMV70Q8-CPT-PHV0.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937300 = {S7699, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N78-M02 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2937310 = {S7700, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cloudpad 970d Build/FSV01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937437 = {S7701, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I9082 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2937476 = {S7702, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; TB-52 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937614 = {S7703, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Sky Plus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937793 = {S7704, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; innos i5plus Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937860 = {S7705, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; A851 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2938129 = {S7706, 160, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Infinix BUZZ Linux/3.0.13 Android/4.0.4 Release/04.13.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2938253 = {S7707, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BOGO BO-LFSP5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2938283 = {S7708, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; A22 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2938282 = {S7709, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; PA0750 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2939163 = {S7710, 155, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; PRO4611 Build/PRO4611-V1.0-2013.04.29) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2949832 = {S7711, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; iJoy Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2950080 = {S7712, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; CASPER Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2950215 = {S7713, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.6; zh-tw; PONY_Speed Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950270 = {S7714, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ZYNC Z919 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950363 = {S7715, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; S600 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950407 = {S7716, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LA-I Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950439 = {S7717, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; W20 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950672 = {S7718, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; TM801 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2950750 = {S7719, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ki60 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950835 = {S7720, 151, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Galaxy Note DK9500 B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950841 = {S7721, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; V926B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950886 = {S7722, 56, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; A76 Build/IMM76D)*/
const Device UA2951043 = {S7723, 158, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - GPLUS N809 Linux/3.0.13 Android/4.0.4 Release/10.26.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2951057 = {S7724, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB-N7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2951144 = {S7725, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; KIRA N10050 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951208 = {S7726, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; TP8-1000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951240 = {S7727, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; QN-7000HX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951282 = {S7728, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; eZee'Tab708 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951322 = {S7729, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; pad T3-1211 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951367 = {S7730, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; TPC7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951414 = {S7731, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; F8AS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2951439 = {S7732, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; PRO 9.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951504 = {S7733, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; AK45 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2951513 = {S7734, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST0916 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951596 = {S7735, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; BV4000 Build/ICS.f08refem32.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951645 = {S7736, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; TP7-1000-metal Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951656 = {S7737, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; eZee'Tab703 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951768 = {S7738, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Joypad C102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951817 = {S7739, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; DOOV_D910 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2951916 = {S7740, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Ifa303 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2951936 = {S7741, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; TA-M72CBT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951958 = {S7742, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q13T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952036 = {S7743, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; zh-tw; PB-S6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952048 = {S7744, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EXLP3P Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2952128 = {S7745, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; T-990A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952176 = {S7746, 165, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - DOMO SLATE X3G SE Linux/3.0.13 Android/4.0.4 Release/11.24.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2952277 = {S7747, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; z1000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2952299 = {S7748, 58, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; M4302 Build/IMM76D)*/
const Device UA2952499 = {S7749, 154, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Android Smart TV Box Build/ICS.MBX.GIGATV.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2952723 = {S7750, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Onliner1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2952729 = {S7751, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HS-E956 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952830 = {S7752, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Chas100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952865 = {S7753, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-T327d/1.11.1401.9) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952892 = {S7754, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8826D/C8826DV100R001C92B620; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2952893 = {S7755, 161, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - KliTON I806LS Linux/3.0.13 Android/4.0.4 Release/01.09.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2952946 = {S7756, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; S06 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952991 = {S7757, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.8; es-us; MID7102SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2953042 = {S7758, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MT-708G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2953082 = {S7759, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; DOUBLE V Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2953118 = {S7760, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru;Infinityll Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2953133 = {S7761, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Quicki-729 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2953211 = {S7762, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; TabLife Build/GT704-A13004-B02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2953306 = {S7763, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PGM 398 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2953403 = {S7764, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Solution8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2953497 = {S7765, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; LC0723B Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2953664 = {S7766, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7566 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2953797 = {S7767, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Themis II Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2953825 = {S7768, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-pt; TX200 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2953839 = {S7769, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; SGC-S3 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2953945 = {S7770, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Surf Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2954126 = {S7771, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M1052S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2954309 = {S7772, 150, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Avvio_790s Build/AvvioAvvio_790s) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2954637 = {S7773, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; softwinners Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955035 = {S7774, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; Lanix Ilium_S400 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955222 = {S7775, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB973G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2955258 = {S7776, 153, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Pro-S Linux/3.0.13 Android/4.0.4 Release/03.06.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2955338 = {S7777, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MIDR477 Build/eng.glwx.X.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2955354 = {S7778, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AXPAD 9E01 Build/ICS.g08refema9.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2955611 = {S7779, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; YF730B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955687 = {S7780, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; NE-202 Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955759 = {S7781, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; C15 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2955851 = {S7782, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TF--W700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955946 = {S7783, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Androtab7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956193 = {S7784, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; AW133 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2956298 = {S7785, 85, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - K-TouchT789_TD/1.0 Android 4.0.3 Release/10.01.2012 Browser/WAP2.0 appleWebkit/534.30*/
const Device UA2956323 = {S7786, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; DC   Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956327 = {S7787, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; NE-103T Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956398 = {S7788, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4;en-us;Wynncom G41 build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2956450 = {S7789, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; POMP_P8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956521 = {S7790, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ektATB-1108 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956557 = {S7791, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nam-Phone G40C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2956621 = {S7792, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; 588 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2956784 = {S7793, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T18+ Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956795 = {S7794, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 9930 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956958 = {S7795, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; T701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2956991 = {S7796, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DT744B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957007 = {S7797, 212, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; sprd-GT-N9300/1.0 Android/4.0.3 Release/04.02.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2957153 = {S7798, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KINGTOP977 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957834 = {S7799, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; keytak15_ics Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957951 = {S7800, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LC0725B Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2958041 = {S7801, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M707-W Build/1.0.883_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2958059 = {S7802, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; IRON Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958186 = {S7803, 208, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; sprd-9300/1.0 Android/4.0.3 Release/04.11.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2958215 = {S7804, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; W33 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958307 = {S7805, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TabletPlus Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2958454 = {S7806, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SKYPAD EXPRESS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958457 = {S7807, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; CA-201L Build/CA201LU377_00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958493 = {S7808, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NX700G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2958503 = {S7809, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; E-tel i7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958915 = {S7810, 149, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A7-11 Build/ARES_00.01.61.27.18.H1111F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2960342 = {S7811, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; Q93S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2960364 = {S7812, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; PC7058ME Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2961642 = {S7813, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; FMT-NT7A40-02 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2962081 = {S7814, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xplorer T7i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2962254 = {S7815, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; A7# Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2962698 = {S7816, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; 700P11A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2963192 = {S7817, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M980 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2964072 = {S7818, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Infinix X351 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2964736 = {S7819, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NEC-NE-201A1A Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2970974 = {S7820, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; Lanix Ilium_S210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2979296 = {S7821, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; zh-cn; 9000H Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2980328 = {S7822, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID-7088B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2980882 = {S7823, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; EBEST V8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2980891 = {S7824, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; Exagerate XZPAD970H Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2981129 = {S7825, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; X5 Build/LOVMEX5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2981288 = {S7826, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; DOUBLE I Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2981495 = {S7827, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fa-ir; LD-453 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2981621 = {S7828, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; B2000+ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2981839 = {S7829, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Easy Home Tablet 8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2981915 = {S7830, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; iB750C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2981918 = {S7831, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; M4302 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2982007 = {S7832, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.6; vi-vn; CHI_Speed Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2982017 = {S7833, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; S9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2982055 = {S7834, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; YPY_10STB Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2982089 = {S7835, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN868H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2982233 = {S7836, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A96 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2982246 = {S7837, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ar-eg; Mercury_magiQ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2982054 = {S7838, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X808 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2982538 = {S7839, 57, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LA-I Build/IMM76I)*/
const Device UA2982614 = {S7840, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; YPY_TQ7 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2982633 = {S7841, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; Polypad C504 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2982676 = {S7842, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID-8000B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2982692 = {S7843, 57, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; 5870 Build/IMM76D)*/
const Device UA2982696 = {S7844, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TB-7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2982764 = {S7845, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; EJTAB97A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2982867 = {S7846, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Slicepad L7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2983013 = {S7847, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; QW TB-1507 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2983124 = {S7848, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; N710 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2983140 = {S7849, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; 97M Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2983155 = {S7850, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HOSIN U9 Build/HOSINA75P) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2983171 = {S7851, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ULTRATAB F7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2983180 = {S7852, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; FTABC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2983191 = {S7853, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; CASPER Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2983335 = {S7854, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; NB 2012 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 ?????? ???????Safari/534.30*/
const Device UA2983378 = {S7855, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GTC7.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2983400 = {S7856, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; 1FC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2983466 = {S7857, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2983498 = {S7858, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ETON T770 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2983524 = {S7859, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Swipe Halo Edge Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2983527 = {S7860, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; ITAB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2983638 = {S7861, 153, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - My503 Linux/3.0.13 Android/4.0.4 Release/11.16.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2983645 = {S7862, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HX-GT1001 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2983650 = {S7863, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; Wmate Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2983653 = {S7864, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch U7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2983682 = {S7865, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; DOOV_D50 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2983772 = {S7866, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; PC8008 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2983843 = {S7867, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; myTab 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984049 = {S7868, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; i90 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984159 = {S7869, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U980 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984248 = {S7870, 154, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo S696 Build/IMM76I) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.2 Version/4.0 Mobile Safari/530.17*/
const Device UA2984353 = {S7871, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; TP73G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2984368 = {S7872, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Nautica 3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984499 = {S7873, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; G_33_HLI Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2984708 = {S7874, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; P860 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984763 = {S7875, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A530 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984827 = {S7876, 150, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Coolgen E69 Note II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984836 = {S7877, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; BF9200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984922 = {S7878, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; E703H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984952 = {S7879, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-one Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985027 = {S7880, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TB-7HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2985042 = {S7881, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; TURBOX STRIKE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2985062 = {S7882, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; T6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2985087 = {S7883, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; Moii E991 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985096 = {S7884, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A3-7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2985124 = {S7885, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; zh-cn; NOAIN A900S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2985328 = {S7886, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; MID7040T-3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2985401 = {S7887, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 9703 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2985485 = {S7888, 57, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; 8190 Build/IML74K)*/
const Device UA2985500 = {S7889, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SHV-E140K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2985511 = {S7890, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MID800SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2985531 = {S7891, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; CR7A10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985606 = {S7892, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 86VMC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985674 = {S7893, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; L100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985722 = {S7894, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; E960 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985752 = {S7895, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; I790 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985753 = {S7896, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.0; en-us; Android_6515M Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2985793 = {S7897, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GPLUS_GN858 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985811 = {S7898, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; SIM-Note Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985936 = {S7899, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; PICOpad GHM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2986060 = {S7900, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; MD7072 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2986066 = {S7901, 154, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - PC-828 Linux/3.0.13 Android/4.0.4 Release/11.09.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2986143 = {S7902, 170, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; TY-K Touch W656/K-Touch_W656_V1.1; 480*800;  CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986163 = {S7903, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; DOOV_i1314 Build/DOOVi1314) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986170 = {S7904, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; F918 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2986200 = {S7905, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PAD1016 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2986296 = {S7906, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Galileo Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2986299 = {S7907, 152, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - P80w Linux/3.0.13 Android/4.0.4 Release/12.11.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2986343 = {S7908, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M9703T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2986418 = {S7909, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JIL 1101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986530 = {S7910, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; RT-MOMO07 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2986564 = {S7911, 157, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; TCL-D662/EG27_YMYM_V0.6; 480*800; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2986576 = {S7912, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; HIKe 716 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986651 = {S7913, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; U360 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986628 = {S7914, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Omega S30 Build/ICS_IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986698 = {S7915, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; NV90 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2986839 = {S7916, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; BF9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986956 = {S7917, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Coolpad 7235 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2986953 = {S7918, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; w500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987011 = {S7919, 172, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LNV-Lenovo_S870e/Lenovo_S870e_S022_X; 540*960; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987081 = {S7920, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; _ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987129 = {S7921, 153, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - ZA987 Linux/3.0.13 Android/4.0.4 Release/11.22.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2987175 = {S7922, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; Q79 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987191 = {S7923, 148, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; advance Build/ICS.advance_30_0_8188.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2987206 = {S7924, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; METAPAD-C2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987242 = {S7925, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; F761 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987272 = {S7926, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; UBPH1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2987327 = {S7927, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Atom Action Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987355 = {S7928, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; K-TOUCH W610D Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987358 = {S7929, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; AK8461 Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2987384 = {S7930, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; TB-90103G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2987389 = {S7931, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; lt901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987410 = {S7932, 149, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; 3310-9506 Build/ICS.Zoostorm_130_NEW.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2987486 = {S7933, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDZ Race Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987534 = {S7934, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; Starmobile ENGAGE 9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2987667 = {S7935, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MP10 Build/ICS.g08refem611.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2987712 = {S7936, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SMP50-200 Build/1.0.316_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2987728 = {S7937, 162, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; UM77-V1.1-073-03-01-04-01-00-01 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987843 = {S7938, 152, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Sansui Cosmic SP 1001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987976 = {S7939, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; zh-cn; L107 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2988090 = {S7940, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fa-ir; G1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2988119 = {S7941, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Coolpad 7235 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2988222 = {S7942, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; T211 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2988300 = {S7943, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; USmart CL600 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2988369 = {S7944, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; RC9717B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2988425 = {S7945, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; hebe_plus_gosco Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2988513 = {S7946, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MID109 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2988917 = {S7947, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; AK926 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2989060 = {S7948, 157, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - A107_MULA Linux/3.0.13 Android/4.0.4 Release/01.08.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2989062 = {S7949, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CN97V Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2989104 = {S7950, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X406 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2989229 = {S7951, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TURBOPAD 1000 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2989262 = {S7952, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TwinTAB-T7283G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2989408 = {S7953, 156, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - GT-I8190 Linux/3.0.13 Android/4.1.9 Release/03.01.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2989448 = {S7954, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; ONEPAD 940 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2989537 = {S7955, 148, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; MMB-525.SAT(Meteorit) Build/ICS.MBX.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2989628 = {S7956, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; N90-2core Build/R13.4-rc2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2990041 = {S7957, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; C007 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2990318 = {S7958, 83, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - K-TouchT6_TD/1.0 Android 4.0.3 Release/10.01.2012 Browser/WAP2.0 appleWebkit/534.30*/
const Device UA2990372 = {S7959, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; P900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2990375 = {S7960, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; ar-eg; NNB-100 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2990488 = {S7961, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; MHL5 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2990499 = {S7962, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DG_70 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2990597 = {S7963, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; EDUPLE7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2990618 = {S7964, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; P102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2990644 = {S7965, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Mswipe Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2990771 = {S7966, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-be; TAB-9702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2990843 = {S7967, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; DualDMax Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2991445 = {S7968, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; IMO_Y-ONE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2991540 = {S7969, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EF09-2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2991670 = {S7970, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MOMO9P Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2991980 = {S7971, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; VN10-V8850/4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2992116 = {S7972, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; V71B Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2992172 = {S7973, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Swift-Tab II 70FB5G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2992182 = {S7974, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LPO-380W Build/FR-M868) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2992284 = {S7975, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.6; vi-vn; V800 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2992360 = {S7976, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; POV_TAB-P527 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2992631 = {S7977, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TF-MID702G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2992799 = {S7978, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JD-720SG Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2993031 = {S7979, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - K-Touch_U81t_TD/V2 Linux/3.0.13 Android/4.0 Release/02.15.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2 System/Android 4.0.4*/
const Device UA2993389 = {S7980, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; T-704S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2993506 = {S7981, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; AirTab P970w Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2993840 = {S7982, 209, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.2; zh-cn; sprd-AS133/1.0 Android/4.0.2 Release/04.17.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2994086 = {S7983, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4;en-us;Digitac Ultratab;C825 Build/ICS) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2994102 = {S7984, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; i9080 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2994288 = {S7985, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; RMD-855 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2994380 = {S7986, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; X802 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2994503 = {S7987, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; TAXB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2994545 = {S7988, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; G18??(E3G5) Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2994740 = {S7989, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SANSUI ETAB 902VP Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2994840 = {S7990, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; vi-vn; C29 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2994874 = {S7991, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V100M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2995015 = {S7992, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB719 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2995017 = {S7993, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Magicon Q7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2995268 = {S7994, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ar-eg; R760 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2995340 = {S7995, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; T935 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2995408 = {S7996, 85, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - LenovoS868t_TD/1.0 Android 4.0.3 Release/10.01.2012 Browser/WAP2.0 appleWebkit/534.30*/
const Device UA2995442 = {S7997, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; th-th; U7T Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2995619 = {S7998, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; GALAXY_S_II_plus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2995641 = {S7999, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Vexia tablet 7903G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2995700 = {S8000, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Y7-3LB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2995803 = {S8001, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; ELIYA S868 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2995817 = {S8002, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A60T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996008 = {S8003, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; vi-; Masstel M12  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2996034 = {S8004, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TAX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2996369 = {S8005, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PHICOMM i813 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996388 = {S8006, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Ace Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2996515 = {S8007, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; E1-B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996649 = {S8008, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Proton One Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2996674 = {S8009, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PAP5430 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996817 = {S8010, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KENWOOD DNN 2013 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997046 = {S8011, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A350 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2997056 = {S8012, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; baoxue Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2997138 = {S8013, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; RMD-1075 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2997147 = {S8014, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; GV7777 Build/GV7777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2997226 = {S8015, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; 8085 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997251 = {S8016, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - sprd-T101/1.0 Linux/2.6.35.7 Android/Android4.0.3 Release/03.19.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2997382 = {S8017, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C761 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997654 = {S8018, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; KT918+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2997717 = {S8019, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; es-es; cenon15_6626_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2997789 = {S8020, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; POV_TAB-P506 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2998102 = {S8021, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; OVO Lite GPS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2998138 = {S8022, 210, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; zh-cn; sprd-skk-A2/1.0 Android/4.0.1 Release/04.24.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2998165 = {S8023, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; S59 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998202 = {S8024, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; PNDPP41DG7SLV^ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998243 = {S8025, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SIMM-X722 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2998296 = {S8026, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; KOMU K1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998440 = {S8027, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; 8085 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2998548 = {S8028, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Philips W737 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998620 = {S8029, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID711 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998644 = {S8030, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; DNS S3502 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998902 = {S8031, 89, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - HUAWEI Y500-T00_TD/1.0 Android 4.0.3 Release/10.01.2012 Browser/WAP2.0 appleWebkit/534.30*/
const Device UA2998945 = {S8032, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; inovacel S23 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999047 = {S8033, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X712 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999059 = {S8034, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S4 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2999105 = {S8035, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-ar; RAIM7000ND_AR Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2999119 = {S8036, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB 270 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999245 = {S8037, 206, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; zh-cn; sprd-F7/1.0 Android/4.0.1 Release/05.09.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2999330 = {S8038, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; TB-120 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2999524 = {S8039, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PD10-DYD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2999663 = {S8040, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-be; Lenco CARTAB-920 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2999665 = {S8041, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; ST70104-2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999674 = {S8042, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DT9138B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999697 = {S8043, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2999808 = {S8044, 154, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Austin Linux/3.0.13 Android/4.0.4 Release/05.07.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2999922 = {S8045, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; S25 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999977 = {S8046, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KINGTOP915 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3000010 = {S8047, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; GG-2000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3000590 = {S8048, 157, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Byond B55 Linux/3.0.13 Android/4.0.4 Release/05.02.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA3001270 = {S8049, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hr-hr; TPC-7121 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3001636 = {S8050, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; V6888 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3001936 = {S8051, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; G86 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3002797 = {S8052, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ET-GSM-A78 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3002860 = {S8053, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; You40 Build/RheaTK) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3002898 = {S8054, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iBall_Andi5.9L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3003067 = {S8055, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; AS128 Build/Android4.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3003520 = {S8056, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PLT4311 Build/PLT4311-V1.2-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3003521 = {S8057, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; BO-LFSP4 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3003771 = {S8058, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; G-M7904 Build/G-M7904) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3003958 = {S8059, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 435TPC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3004131 = {S8060, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GP353 Build/RheaTK) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3004482 = {S8061, 155, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - DTC GT9 Linux/3.0.13 Android/4.0.4 Release/05.14.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA3004956 = {S8062, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; PHM727MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3005276 = {S8063, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TM-3204R Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3006274 = {S8064, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HBD-761K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3006415 = {S8065, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AT007F Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3009366 = {S8066, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BLU AMOUR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3009581 = {S8067, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE N798 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3009907 = {S8068, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; DOOV D360 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3010243 = {S8069, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XELIO_P900A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3010988 = {S8070, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; AS125 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3011189 = {S8071, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; F6S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3011619 = {S8072, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; OCEATAB7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3012164 = {S8073, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; CA-201L Build/CA201LU410_00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3012647 = {S8074, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; Smart 3 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3012783 = {S8075, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; D2-430 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3012973 = {S8076, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X700 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3014267 = {S8077, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; F307-B Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3014705 = {S8078, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3014784 = {S8079, 212, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; sprd-GT-N9300/1.0 Android/4.0.3 Release/06.06.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3015538 = {S8080, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; AIKUN_AT73C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3016463 = {S8081, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X-Treme Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3016745 = {S8082, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB507G Build/ICS_IMM76I_B511) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3018461 = {S8083, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Oysters T7D 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3018998 = {S8084, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; PC704 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3020083 = {S8085, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Z10 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3020623 = {S8086, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; i-Call 501 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022627 = {S8087, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ONIDA i555 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3024960 = {S8088, 159, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Celkon A105 Linux/3.0.13 Android/4.0.4 Release/07.05.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA3027427 = {S8089, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; L29K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3031253 = {S8090, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ec2cloud_v2 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3038893 = {S8091, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ec2cloud_v2 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3042227 = {S8092, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; X15 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3042230 = {S8093, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; M805HCX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3042416 = {S8094, 167, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch S757/K-Touch_S757_V1.1; 480*854;  CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3042549 = {S8095, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; SMX-T9701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3042564 = {S8096, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; BIRD A05 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3042610 = {S8097, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; OS805 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3042760 = {S8098, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Q8E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3042771 = {S8099, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3042780 = {S8100, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; generic_x86 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3042812 = {S8101, 126, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; D5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3042820 = {S8102, 167, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch S757/K-TOUCH_S757_V2.0; 480*854;  CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3042903 = {S8103, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; U910 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3042911 = {S8104, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; N70DC-S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3042919 = {S8105, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TM-MID725 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3043082 = {S8106, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; DOUBLE II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3043249 = {S8107, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; S9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3043290 = {S8108, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch W710 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3043338 = {S8109, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; CT7013 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3043442 = {S8110, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; T900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3043514 = {S8111, 57, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; 7266 Build/IMM76I)*/
const Device UA3043550 = {S8112, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; TB-107 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3043614 = {S8113, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDZ Flash Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3043615 = {S8114, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; SGW920 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3043637 = {S8115, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; OUKI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3043680 = {S8116, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; T-22B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3043760 = {S8117, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; ar-eg; G1   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3043795 = {S8118, 61, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Q88B-11S Build/IMM76D)*/
const Device UA3043816 = {S8119, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; DAKELE MC001 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3043985 = {S8120, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; F3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044038 = {S8121, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; Lanix Ilium_S500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044087 = {S8122, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch W689 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044110 = {S8123, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T-711D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044270 = {S8124, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; L8202 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044378 = {S8125, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; NOVO NV7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044423 = {S8126, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; NTT 730 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3044512 = {S8127, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LNV-Lenovo S870e Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044677 = {S8128, 149, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; ELEMENT 7 V2 BASIC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044655 = {S8129, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TB1082 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3044699 = {S8130, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X50 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044715 = {S8131, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; lt_MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044920 = {S8132, 149, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A356 Build/LenovoLePhone) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044980 = {S8133, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SMARTTVBOX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3045027 = {S8134, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MOFUT-F8SH Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3045123 = {S8135, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M711 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3045295 = {S8136, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.8; en-us; B1-EP2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3045325 = {S8137, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Yummy-Duo Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3045360 = {S8138, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; PGM 399 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3045400 = {S8139, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GiONEE GN600 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3045523 = {S8140, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; STS CA SERIES Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3045621 = {S8141, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; SENCOR ELEMENT 9.7V3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3045730 = {S8142, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; BF_A12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3045816 = {S8143, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; P617S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3045842 = {S8144, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AK401 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3046201 = {S8145, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; B-TAB1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3046389 = {S8146, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; NYX_FLY Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3046410 = {S8147, 162, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; UM77-V1.1-073-06-01-02-02-02-01 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3046712 = {S8148, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; DL 3D MaxView TD-M71 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3047101 = {S8149, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; E566 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3047161 = {S8150, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID1054 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3047212 = {S8151, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; TY-001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3047188 = {S8152, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MBO Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3047431 = {S8153, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-uk; mini7100 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3047439 = {S8154, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Z6 Build/Android4.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3047755 = {S8155, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NID-7006 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3048207 = {S8156, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; Diamond_Lite Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3048463 = {S8157, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; S758 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3048645 = {S8158, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A55 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3048652 = {S8159, 151, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - HTC Linux/3.0.13 Android/4.1.9 Release/02.26.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA3048687 = {S8160, 155, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - X8 NOTE Linux/3.0.13 Android/4.0.4 Release/04.07.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA3048742 = {S8161, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; E721L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3048779 = {S8162, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TW900A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3048819 = {S8163, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GEM7032G-rev2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3048930 = {S8164, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; Treq 3G Basic 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3048986 = {S8165, 172, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LNV-Lenovo_S870e/Lenovo_S870e_S028_X; 540*960; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049261 = {S8166, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Aspire A1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049287 = {S8167, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PA938D Build/PA938D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3049302 = {S8168, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TPC0902 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049335 = {S8169, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; ongo S5020 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049448 = {S8170, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-us; EX S20i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3049536 = {S8171, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; hu-hu; PHA-3850 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3049865 = {S8172, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.5; en-us; 7080 Build/MocorDroid4.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3049882 = {S8173, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; T7040 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3049953 = {S8174, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S280 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049983 = {S8175, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; 820 Build/820-V1.0-2013.03.04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050127 = {S8176, 218, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; sprd-Citycall-I930+/1.0 Android/4.0.3 Release/04.23.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3050143 = {S8177, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; i88S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3050201 = {S8178, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; F&U ETB7404 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050308 = {S8179, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TABLET LO+DEMODA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050467 = {S8180, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; Tab1031 Build/ICS.g08refmid1061.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3050473 = {S8181, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; 9 inch Economy Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050555 = {S8182, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; Q74A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050575 = {S8183, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB 171 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050683 = {S8184, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CY7809 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3051884 = {S8185, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; VIVAS_LOTUS_S1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3054445 = {S8186, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; F761K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3055544 = {S8187, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GXPAD300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3056090 = {S8188, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; N5002 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3056484 = {S8189, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CRCB_2093 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3056667 = {S8190, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; VIT-T4000 Build/0.29VSG) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3056851 = {S8191, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Mi-A351 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3056917 = {S8192, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; TR71 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3057220 = {S8193, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; GL Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3057361 = {S8194, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 43BMID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057523 = {S8195, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; Dslide 971 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3058268 = {S8196, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM204SH Build/S0016) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058905 = {S8197, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T5-B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3059728 = {S8198, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDZ Drive+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3069964 = {S8199, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TP12A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3069975 = {S8200, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; MD1020 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3070199 = {S8201, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; zh-cn; BIRDA_MODEL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3070214 = {S8202, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; PC8005 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3070282 = {S8203, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; P9000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3070294 = {S8204, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; B12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3070493 = {S8205, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; vollo Vx93 Build/volloVx93) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3070580 = {S8206, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; MID950 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3070622 = {S8207, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; PROG150 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3070711 = {S8208, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; V700H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3070763 = {S8209, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; MID7200 Build/MID7200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3070783 = {S8210, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; PMID70C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3070971 = {S8211, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; F-ONE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3070992 = {S8212, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Woxter Tablet PC 76CXi Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3071027 = {S8213, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; PMID70DC Build/unknown) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3071029 = {S8214, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Chungha Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3071078 = {S8215, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; g08ref Build/ICS.g08ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3071113 = {S8216, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M9701TW Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3071214 = {S8217, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A703H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3071271 = {S8218, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; J-Q5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3071356 = {S8219, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TAB-970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3071426 = {S8220, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; MID4502 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3071458 = {S8221, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; K-Touch T780/TBTX_8660_V0101) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Linux/3.0.8 Safari/534.30*/
const Device UA3071857 = {S8222, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; S4702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3071901 = {S8223, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; DC   Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3072371 = {S8224, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 86VE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3072505 = {S8225, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; OKU1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3072563 = {S8226, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; F8Cs Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3073267 = {S8227, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Swift-Tab II 10WBNG Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3073311 = {S8228, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Aole Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3073598 = {S8229, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Z6_FWVGA_XD Build/Android4.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3073691 = {S8230, 65, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo A630t Build/IMM76D)*/
const Device UA3073768 = {S8231, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; WTAB 704 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3073836 = {S8232, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDZ Ultimate Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3073857 = {S8233, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JD-716SG Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3073876 = {S8234, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MMD-X200 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3073921 = {S8235, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; T272 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3074050 = {S8236, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; AT7E-A13BP Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3074173 = {S8237, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; ET9-1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3074230 = {S8238, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB975G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3074295 = {S8239, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q27H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3074973 = {S8240, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M756 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3075043 = {S8241, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; PICOpad GGD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3076321 = {S8242, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Calipso4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3076914 = {S8243, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; Smart 4 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3077219 = {S8244, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; CORE DIGITAL CD-713WD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3077987 = {S8245, 153, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - MP200 Linux/3.0.13 Android/4.0.4 Release/05.16.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA3077996 = {S8246, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; E9000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3078216 = {S8247, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A704L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078262 = {S8248, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; P700+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078306 = {S8249, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X65 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078400 = {S8250, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; AS131 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3078504 = {S8251, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; A70A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3078685 = {S8252, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; TB-721 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3078797 = {S8253, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AX5_Duo Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078824 = {S8254, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; 29548 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3078899 = {S8255, 147, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.357; en-us; UMO T99 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3078932 = {S8256, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; QM734-2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079226 = {S8257, 168, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KB-Karbonn A9*+/Karbonn_A9*_V1.3; 480*800;  CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079399 = {S8258, 168, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KB-Karbonn A9*+/Karbonn_A9*_V1.2; 480*800;  CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079465 = {S8259, 161, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U; Android 4.0.4;en-us;Digitac Ultratab;C1029 Build/ICS) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA3080874 = {S8260, 168, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.0.3;dev=holiday;devm=HTC PH39100;man=HTC;brw=Android;shwd=540x960;shwa=540x923;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA3084976 = {S8261, 60, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MID-A10 Build/IMM76D)*/
const Device UA3085323 = {S8262, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; NEOBOX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3085445 = {S8263, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A710e Build/ICS) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3085501 = {S8264, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; EBEST S6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3085700 = {S8265, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch E6 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3085927 = {S8266, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; N970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3086002 = {S8267, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; NB021/22 Build/1.0.560_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3086068 = {S8268, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; ar-eg; BlackBird_I7000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3086116 = {S8269, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; PAD705 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3086164 = {S8270, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; M901D1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3086177 = {S8271, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; A9724 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3086243 = {S8272, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; W970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3086300 = {S8273, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; AigoPad Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3086316 = {S8274, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; ASK SP558 HD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3086317 = {S8275, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LNV-Lenovo S870e Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3086331 = {S8276, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; Ezcool Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3086486 = {S8277, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N9500/N7100 PAD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3086555 = {S8278, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; V10A Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3086668 = {S8279, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; 7I01 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3086677 = {S8280, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Spring W-100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3086695 = {S8281, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CTA Build/ICS.casper.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3086732 = {S8282, 158, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - MOFUT T3MT Linux/3.0.13 Android/4.0.4 Release/01.12.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA3086810 = {S8283, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D36 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3087096 = {S8284, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; T275 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3087333 = {S8285, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; FR-M768 Build/FR-M768) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3087431 = {S8286, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; QBA757 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3087437 = {S8287, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; TA1033G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3087510 = {S8288, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; 989 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3087683 = {S8289, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch W700+ Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3087808 = {S8290, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; Mobia J Build/RheaTK) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3087885 = {S8291, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; S4702M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3087936 = {S8292, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; FLY F50s Build/ICS_IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088070 = {S8293, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Z7_WVGA_TL Build/Android4.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3088233 = {S8294, 125, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - sprd-G910/1.0 Linux/2.6.35.7 Android 4.0.4 Release/04.10.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3088396 = {S8295, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; T77 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3088704 = {S8296, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; M74B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088867 = {S8297, 157, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Extreme 4 Linux/3.0.13 Android/4.0.4 Release/05.13.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA3089003 = {S8298, 89, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - HUAWEI Y320-T00_TD/1.0 Android 4.0.3 Release/10.01.2012 Browser/WAP2.0 appleWebkit/534.30*/
const Device UA3089978 = {S8299, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI M881 Build/HuaweiM881) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3091403 = {S8300, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; P900i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3091642 = {S8301, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; PI T005 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3091965 = {S8302, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A630/S001) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092205 = {S8303, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Teta Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3093047 = {S8304, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ONIDA i010 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3094044 = {S8305, 180, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.0.4;dev=SGH-I717;devm=SAMSUNG-SGH-I717;man=samsung;brw=Android;shwd=800x1280;shwa=800x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA3098460 = {S8306, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S2-38AH0 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3098479 = {S8307, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; G5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3098510 = {S8308, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; PAD511 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3098512 = {S8309, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; swipe Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3098604 = {S8310, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Z520e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3098633 = {S8311, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID-HC719 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3098663 = {S8312, 55, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Q8 Build/IMM76D)*/
const Device UA3098675 = {S8313, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ING800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3098730 = {S8314, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; T8+ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3098933 = {S8315, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; PI E005 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3098939 = {S8316, 149, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; Exagerate XZPAD750 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3098951 = {S8317, 162, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; UM77-V1.1-070-01-00-01-01-00-00 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099005 = {S8318, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; I5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099072 = {S8319, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ro-ro; EVOTAB3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099110 = {S8320, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; V929B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099129 = {S8321, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; MIRA-2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3099234 = {S8322, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; INTRACON FX2 PAD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3099271 = {S8323, 63, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; K-Touch U6 Build/IMM76I)*/
const Device UA3099305 = {S8324, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; HYP008 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3099391 = {S8325, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; E800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099427 = {S8326, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; Moii E990 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099435 = {S8327, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; 86V2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3099457 = {S8328, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; BOUBLE VI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099564 = {S8329, 58, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Clear Build/IMM76D)*/
const Device UA3099575 = {S8330, 146, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BM mobile A6000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099652 = {S8331, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TB-7 HD 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3099663 = {S8332, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MFC170FR_01 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3099682 = {S8333, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Hphone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3099775 = {S8334, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; UTime_T1+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099826 = {S8335, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; miTab SPACE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099860 = {S8336, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; FWS710EU Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099848 = {S8337, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; Full AOSP on godbox Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3100009 = {S8338, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3100077 = {S8339, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ChanghongW6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3100079 = {S8340, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; NB021/22 Build/1.0.546_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3100089 = {S8341, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; powerpadMAXX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3100094 = {S8342, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; EvoPAD T7016 Build/EvoPAD_T7016) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3100231 = {S8343, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Mercury X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3100277 = {S8344, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TM-5377 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3100297 = {S8345, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; D628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3100302 = {S8346, 60, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; A11Note Build/IMM76D)*/
const Device UA3100327 = {S8347, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; NB036 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3100355 = {S8348, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; fa-ir; T9988 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3100421 = {S8349, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GN168T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3100447 = {S8350, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EZPAD 745C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3100654 = {S8351, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; M703 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3100661 = {S8352, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; FW8937 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3100753 = {S8353, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; PTM805MCL Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3100866 = {S8354, 158, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - WaterProof Linux/3.0.13 Android/4.0.4 Release/02.23.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA3100961 = {S8355, 158, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; JT Hot Q/JT_Hot Q_V1.0; 480x800; APT/2.0) AppleWebkit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA3101059 = {S8356, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB972 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3101095 = {S8357, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; IHA-C0801 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3101287 = {S8358, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ar-eg; A330 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3101648 = {S8359, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-8220S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3101849 = {S8360, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; TOPTIL T900 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3101910 = {S8361, 61, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; vivo S7t Build/IMM76D)*/
const Device UA3102002 = {S8362, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PL7117 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102270 = {S8363, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AX8RACE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102365 = {S8364, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ANDY 3G 3.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102726 = {S8365, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Pentagram Ego Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102791 = {S8366, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A721J Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102871 = {S8367, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-; DC   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3103034 = {S8368, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; S4 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3103064 = {S8369, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; TR101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3103118 = {S8370, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fa-ir; DAYA A300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103148 = {S8371, 152, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; DL 3G MOBILE TG-M73_2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103153 = {S8372, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Icon 2G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103172 = {S8373, 210, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; zh-cn; sprd-GT8160/1.0 Android/4.0.1 Release/05.14.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3103260 = {S8374, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; DualDMax Build/DualDrive) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3103416 = {S8375, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IT-708C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103514 = {S8376, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; QM734 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3105073 = {S8377, 149, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; UPAD Go Smart G312 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3105075 = {S8378, 127, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; P91 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3105582 = {S8379, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V703 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3105841 = {S8380, 209, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; sprd-I8730/1.0 Android/4.0.4 Release/05.23.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3106225 = {S8381, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Axxion ATAB-701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3107186 = {S8382, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; PICOpadGCE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3107627 = {S8383, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; vi-vn; smart five Build/Android4.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3107814 = {S8384, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; OPAL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3107947 = {S8385, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; RC0709B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3107971 = {S8386, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; i-Call 300v2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108033 = {S8387, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N900 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108116 = {S8388, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108167 = {S8389, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; E4001 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108196 = {S8390, 153, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - N7500 Linux/3.0.13 Android/4.0.4 Release/07.12.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA3108391 = {S8391, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AX3Duo Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108637 = {S8392, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; PICOpad GGG Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3109137 = {S8393, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; msm8625 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3109138 = {S8394, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; Cloud Y5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3109189 = {S8395, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DT9138 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3113532 = {S8396, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; V8200plus Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3117731 = {S8397, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GFIVE A86 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3117886 = {S8398, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GN360 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3117893 = {S8399, 65, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Coolpad 7290 Build/IMM76D)*/
const Device UA3118037 = {S8400, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M5000 Build/1.0.697_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3118049 = {S8401, 142, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; BOWAY I7 Build/BOWAYA75P) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3118091 = {S8402, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZP111 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3118098 = {S8403, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAB466 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3118334 = {S8404, 65, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; K-Touch E688 Build/IMM76I)*/
const Device UA3118349 = {S8405, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; gs701b Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3118424 = {S8406, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; F-074 Build/V02R11A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3118567 = {S8407, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; TAB9 3G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3118761 = {S8408, 133, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MID7117CM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3118760 = {S8409, 151, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - A1G Linux/3.0.13 Android/4.0.4 Release/11.22.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA3118911 = {S8410, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; TP860A Build/ICS.g08refem618.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3118949 = {S8411, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; BF_G11 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3118958 = {S8412, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; android ics Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3119021 = {S8413, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TZX-cpt Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3119126 = {S8414, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AM736 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3119189 = {S8415, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; CTAB700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3119339 = {S8416, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; PMP3570C Build/PMP3570C_X_v1.0.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3119486 = {S8417, 140, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; Tablet Build/GT704-A13005-E01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3119670 = {S8418, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; MT0724B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3120429 = {S8419, 145, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Z6_WVGA_LC Build/Android4.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3120526 = {S8420, 148, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Connspeed T50 Build/ICS_IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3120781 = {S8421, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Tablet Rebel Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3120784 = {S8422, 57, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; 8720 Build/IML74K)*/
const Device UA3120850 = {S8423, 139, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; vi-vn; Masstel M12  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3120939 = {S8424, 128, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; VP11 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3120966 = {S8425, 157, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Aero A1-110 Build/ICS.bangho.gt8105.130+.all.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3120993 = {S8426, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; TB-100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121021 = {S8427, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; E721J Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121024 = {S8428, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; TAB7002 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3121088 = {S8429, 131, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; es-es; DC   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3121336 = {S8430, 134, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; N7800 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3121511 = {S8431, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Beta Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121569 = {S8432, 132, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MMD-P706 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3121771 = {S8433, 141, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; reeder A7S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3122630 = {S8434, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; AX540 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3122953 = {S8435, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; f761K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3124061 = {S8436, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; ChildPad Lazer Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3124602 = {S8437, 130, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ZTE U808 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3124607 = {S8438, 148, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Android Style one Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3124786 = {S8439, 129, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S72-A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3124889 = {S8440, 114, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-us; AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3124950 = {S8441, 144, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NOTE_CREATIVE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3124999 = {S8442, 136, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; S4001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125028 = {S8443, 135, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Oysters T37 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3125137 = {S8444, 143, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Masstel M500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125140 = {S8445, 159, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; Radiomarelli SevenQ Build/ICS_IMM76I_B511) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125437 = {S8446, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; CT0712 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125446 = {S8447, 138, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TSM-TP3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125654 = {S8448, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; MT7015 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3126477 = {S8449, 166, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.0.4;dev=edison;devm=MB865;man=motorola;brw=Android;shwd=540x960;shwa=540x923;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA3128008 = {S8450, 137, { &P15364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-us; A51 Build/MocorDroid4.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA459161 = {S8451, 139, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA692283 = {S8452, 142, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I535 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA707498 = {S8453, 139, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA807183 = {S8454, 139, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T999 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA827157 = {S8455, 61, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9300 Build/IMM76D)*/
const Device UA877589 = {S8456, 159, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9300/I9300BUALE6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA895362 = {S8457, 137, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SC-06D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA895894 = {S8458, 139, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9300 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899319 = {S8459, 140, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; GT-I9300T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899766 = {S8460, 159, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SAMSUNG GT-I9300/I9300XXALE8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899954 = {S8461, 140, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-T999V Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA900274 = {S8462, 158, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300-ORANGE/I9300BVALE4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA900455 = {S8463, 61, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-I535 Build/IMM76D)*/
const Device UA902246 = {S8464, 159, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9300/I9300BUALF1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA905196 = {S8465, 159, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9300/I9300XXALF2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA906312 = {S8466, 159, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9300/I9300BVALE4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA907053 = {S8467, 139, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I535 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA909895 = {S8468, 159, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9300/I9300XXALF6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA909986 = {S8469, 61, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-T999 Build/IMM76D)*/
const Device UA911145 = {S8470, 158, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300-ORANGE/I9300BVALF4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA912973 = {S8471, 159, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9300/I9300BVALF4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA925360 = {S8472, 159, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SAMSUNG GT-I9300/I9300XXBLFB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1959397 = {S8473, 139, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SCH-I939 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1965583 = {S8474, 137, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1976759 = {S8475, 130, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1990702 = {S8476, 62, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9300T Build/IMM76D)*/
const Device UA2001697 = {S8477, 158, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300-ORANGE/I9300BVBLG2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2003042 = {S8478, 159, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9300/I9300BUBLG3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2013274 = {S8479, 159, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9300/I9300BVBLG2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027059 = {S8480, 159, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; SAMSUNG GT-I9300/I9300XXALF5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027810 = {S8481, 125, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2028384 = {S8482, 130, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SCH-I939 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2030390 = {S8483, 59, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SC-06D Build/IMM76D)*/
const Device UA2030989 = {S8484, 159, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; SAMSUNG GT-I9300/I9300XXBLH1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032808 = {S8485, 62, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-T999V Build/IMM76D)*/
const Device UA2042026 = {S8486, 159, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SAMSUNG GT-I9300/I9300XXBLH3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2174393 = {S8487, 158, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300-ORANGE/I9300BVBLH2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2174873 = {S8488, 139, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2182306 = {S8489, 137, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SGH-T999 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2201485 = {S8490, 145, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA2203401 = {S8491, 159, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-I9300/I9300BVBLH2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2223412 = {S8492, 138, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-I9300T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2225885 = {S8493, 132, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257393 = {S8494, 136, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257450 = {S8495, 157, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG GT-I9300/I9300BUBLG3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257504 = {S8496, 130, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T999 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2257602 = {S8497, 145, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I535 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2257706 = {S8498, 159, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SAMSUNG GT-I9300/I9300XXALEF Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2352475 = {S8499, 132, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2352675 = {S8500, 140, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I93000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2355598 = {S8501, 145, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T999 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2375535 = {S8502, 137, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SCH-I535 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2395193 = {S8503, 125, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SCH-I939 Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2398263 = {S8504, 138, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SGH-T999V Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2402629 = {S8505, 139, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; GT-l9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2410620 = {S8506, 130, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2418340 = {S8507, 157, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG GT-I9300/I9300BVALF4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2420003 = {S8508, 136, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; it-it; I9300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2425632 = {S8509, 146, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9300 Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2486359 = {S8510, 156, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - GT-I9300 Linux/3.0.13 Android/4.1.9 Release/01.04.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2507300 = {S8511, 139, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-l9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2520796 = {S8512, 130, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SCH-I535 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2548355 = {S8513, 130, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; SCH-I939 Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2553596 = {S8514, 156, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-I9300-ORANGE/I9300BVBLH2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2577950 = {S8515, 136, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2588876 = {S8516, 142, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/RUISI.ORG) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2590532 = {S8517, 61, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-I939 Build/IMM76D)*/
const Device UA2594666 = {S8518, 137, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; es-es; I9300+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2641006 = {S8519, 61, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-I9300 Build/IML74K)*/
const Device UA2651870 = {S8520, 134, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.8; en-us; I9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652905 = {S8521, 146, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9300 Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2653527 = {S8522, 156, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-I9300-ORANGE/I9300BVBLG2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708211 = {S8523, 137, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-US; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2726311 = {S8524, 139, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/ZSALG5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2729489 = {S8525, 155, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - GT-9300 Linux/3.0.13 Android/4.1.9 Release/12.05.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2829905 = {S8526, 157, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG GT-I9300/I9300BVALE4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2832724 = {S8527, 157, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG GT-I9300/I9300BVBLG2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2841095 = {S8528, 58, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; I9300 Build/IMM76D)*/
const Device UA2897408 = {S8529, 153, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - I9300 Linux/3.0.13 Android/4.1.9 Release/05.21.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2936264 = {S8530, 161, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Samsung_I9300 Linux/3.0.13 Android/4.1.2 Release/03.19.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2937285 = {S8531, 148, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-I9300 Build/IceCreamSandwhich) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982250 = {S8532, 137, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; I9300i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984251 = {S8533, 139, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; GT-I9300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3072938 = {S8534, 134, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.6; en-us; I9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3073424 = {S8535, 160, { &P18283,&P18066,&P18062,&P18092 }};	/*Useragent - Samung_I9300 Linux/3.0.13 Android/4.1.1 Release/03.19.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA101992 = {S8536, 55, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Domnutch-Bot/Nutch-1.0 (Domnutch; http://www.Nutch.de/)*/
const Device UA106714 = {S8537, 82, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - KSCrawler/Nutch-1.0 (http://www.kindsight.net/en/kscrawler; crawler@kindsight.net)*/
const Device UA123571 = {S8538, 18, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - IS Alpha/Nutch-1.1*/
const Device UA144189 = {S8539, 90, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Attentio/Nutch-0.9-dev (Attentio's beta blog crawler; www.attentio.com; info@attentio.com)*/
const Device UA207256 = {S8540, 35, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Aghaven/Nutch-1.2 (www.aghaven.com)*/
const Device UA236802 = {S8541, 25, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - My Nutch Spider/Nutch-1.3*/
const Device UA253232 = {S8542, 118, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Mozilla/5.0 (compatible; OpenindexDeepSpider/Nutch-1.5-dev; +http://openindex.io/spider.html; systemsATopenindexDOTio)*/
const Device UA324167 = {S8543, 41, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Nutraspace/Nutch-1.2 (www.nutraspace.com)*/
const Device UA342117 = {S8544, 23, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Googlebot/Nutch-0.9-dev*/
const Device UA349531 = {S8545, 25, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - My Nutch Spider/Nutch-1.4*/
const Device UA351739 = {S8546, 22, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - COMODOSpider/Nutch-1.2*/
const Device UA380966 = {S8547, 75, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - MaxPointCrawler/Nutch-1.1 (maxpoint.crawler at maxpointinteractive dot com)*/
const Device UA381338 = {S8548, 19, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - nutch-1.4/Nutch-1.4*/
const Device UA381499 = {S8549, 21, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - HAZY.SPIDER/Nutch-1.4*/
const Device UA401250 = {S8550, 18, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - CloudACL/Nutch-1.4*/
const Device UA401466 = {S8551, 29, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - SemrushBot/Nutch-1.5-SNAPSHOT*/
const Device UA422377 = {S8552, 129, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Netluchs/Nutch-1.0 ( ; http://www.netluchs.de/bot.html; _do_not_spam_me___humans_please_use_info_at_netluchs.de_without_the_dash)*/
const Device UA440995 = {S8553, 106, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - spotinfluence/Nutch-1.4 (Spot Influence crawler; http://spotinfluence.com; hello at spotinfluence dot com)*/
const Device UA446434 = {S8554, 94, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - NutchCVS/0.7.2 (Nutch; http://lucene.apache.org/nutch/bot.html; nutch-agent@lucene.apache.org)*/
const Device UA454245 = {S8555, 43, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - RADaR-Bot/Nutch-1.4 (http://radar-bot.com/)*/
const Device UA457872 = {S8556, 19, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - googlebot/Nutch-1.4*/
const Device UA460558 = {S8557, 96, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - wttsearch/Nutch-1.0 (Crawler for SIFE (WTT); http://wtt.beecom.ch/wttsearch.html; wtt@beecom.ch)*/
const Device UA474244 = {S8558, 84, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - lmspider/Nutch-0.9-dev (For research purposes.; www.nuance.com; lmspider@nuance.com)*/
const Device UA523187 = {S8559, 61, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - emBot-GalaBuzz/Nutch-1.0 (http://emining.jp/; em@galabuzz.jp)*/
const Device UA534691 = {S8560, 80, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - lmspider/Nutch-0.9 (For research purposes.; www.nuance.com; lmspider@nuance.com)*/
const Device UA576226 = {S8561, 25, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - My Nutch Spider/Nutch-1.2*/
const Device UA593951 = {S8562, 138, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Nutch/Nutch-1.4 (A semantic crawler for a PhD thesis; http://issel.ee.auth.gr/doku.php/research; kvavliak at issel dot ee dot auth dot gr)*/
const Device UA603787 = {S8563, 39, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - ExactSeek Crawler (nutch 1.4)/Nutch-1.4*/
const Device UA604773 = {S8564, 23, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Kiodia Spider/Nutch-1.4*/
const Device UA679155 = {S8565, 96, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - ScholarUniverse/0.8 (Nutch; http://scholaruniverse.com/bot.jsp; fetch-agent@scholaruniverse.com)*/
const Device UA820486 = {S8566, 75, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - merocrawl/Nutch-1.4 (merobase crawler; www.merobase.com; info@merobase.com)*/
const Device UA831449 = {S8567, 16, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - wminer/Nutch-1.4*/
const Device UA878156 = {S8568, 19, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - My Spider/Nutch-1.4*/
const Device UA895648 = {S8569, 25, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - My Nutch Spider/Nutch-1.5*/
const Device UA896050 = {S8570, 56, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - WikiDo/Nutch-1.4 (http://wikido.com; crawler@wikido.com)*/
const Device UA899918 = {S8571, 17, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - crawler/Nutch-1.4*/
const Device UA901656 = {S8572, 119, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - TosCrawler/Nutch-1.4 (http://www.toshiba.co.jp/rdc/about/crawl_info.htm; 'Rdc-crawler at ml dot toshiba dot co dot jp')*/
const Device UA911775 = {S8573, 22, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - OpenWebIndex/Nutch-1.5*/
const Device UA916313 = {S8574, 134, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - KinglandSystemsCorp/Nutch-1.2 (Web crawler run by Kingland Systems Corp; http://www.kingland.com; kyle dot hansen at Kingland dot com)*/
const Device UA1965226 = {S8575, 32, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - HD nutch agent/Nutch-1.1 (Think)*/
const Device UA1991448 = {S8576, 27, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - My Nutch Spider/Nutch-1.5.1*/
const Device UA1998279 = {S8577, 47, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - spider/Nutch-1.5.1 (spider; http://www.xxx.com)*/
const Device UA2000675 = {S8578, 22, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - NexiSpider/Nutch-1.5.1*/
const Device UA2006157 = {S8579, 121, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - TosCrawler/Nutch-1.5.1 (http://www.toshiba.co.jp/rdc/about/crawl_info.htm; 'Rdc-crawler at ml dot toshiba dot co dot jp')*/
const Device UA2043052 = {S8580, 31, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Nutch/Nutch-2.0 (Nutch Crawler)*/
const Device UA2171249 = {S8581, 18, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - MSNBot/Nutch-1.5.1*/
const Device UA2201496 = {S8582, 31, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - NSIDC Libre Crawler/Nutch-1.5.1*/
const Device UA2205697 = {S8583, 88, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - NutchCVS (Nutch; http://lucene.apache.org/nutch/bot.html; nutch-agent@lucene.apache.org)*/
const Device UA2232230 = {S8584, 19, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Spiderman/Nutch-1.5*/
const Device UA2233450 = {S8585, 67, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - vik-robot/Nutch-1.0 (vikspider; http://vik.com; vikspider@163.com )*/
const Device UA2233801 = {S8586, 16, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Mysite/Nutch-2.0*/
const Device UA2234364 = {S8587, 30, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Parveen Nutch Spider/Nutch-1.5*/
const Device UA2237132 = {S8588, 33, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - WebDataCommonsCrawler/Nutch-1.5.1*/
const Device UA2238905 = {S8589, 27, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Mu Nutch Spider/Nutch-1.5.1*/
const Device UA2242206 = {S8590, 29, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - AstuteSocialCrawler/Nutch-1.4*/
const Device UA2242320 = {S8591, 18, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - hiva/Nutch-2.0-dev*/
const Device UA2242475 = {S8592, 98, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - MetricAlerts/Nutch-2.0 (Media monitoring; http://socialartisan.co.uk; crawler@socialartisan.co.uk)*/
const Device UA2341985 = {S8593, 16, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - spider/Nutch-2.0*/
const Device UA2355807 = {S8594, 73, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Matt_Crawler/Nutch-1.5.1 (Academic test crawl; research_crawler at yahoo)*/
const Device UA2367260 = {S8595, 28, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Affectv Robot v1.0/Nutch-2.1*/
const Device UA2402069 = {S8596, 31, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Ruilin's Nutch Spider/Nutch-1.5*/
const Device UA2408446 = {S8597, 119, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - TosCrawler/Nutch-1.6 (http://www.toshiba.co.jp/rdc/about/crawl_info.htm; 'Rdc-crawler at ml dot toshiba dot co dot jp')*/
const Device UA2421814 = {S8598, 30, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Nutch-Crawler-Manu/Nutch-1.5.1*/
const Device UA2426117 = {S8599, 85, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - ExactSeek Crawler (nutch 1.4)/Nutch-1.4 (ExactSeek Crawler; http://www.exactseek.com)*/
const Device UA2428525 = {S8600, 49, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - InboundScore/Nutch-1.4 (http://InboundScore.com/)*/
const Device UA2430616 = {S8601, 84, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - KSCrawler/Nutch-1.5.1 (http://www.kindsight.net/en/kscrawler; crawler@kindsight.net)*/
const Device UA2436184 = {S8602, 25, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - My Nutch Spider/Nutch-1.6*/
const Device UA2437579 = {S8603, 122, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - TosCrawler/Nutch-1.4 (http://www.toshiba.co.jp/rdc/about/crawl_info_en.htm; 'Rdc-crawler at ml dot toshiba dot co dot jp')*/
const Device UA2460570 = {S8604, 25, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - ReventIt Spider/Nutch-1.6*/
const Device UA2479105 = {S8605, 114, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - WhelanLabs Search/Nutch-1.4 (WhelanLabs Web Crawler Agent; searchengine@whelanlabs.com; http://www.whelanlabs.com)*/
const Device UA2486175 = {S8606, 72, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - LWNutch/Nutch-1.4 (another scientific bot - we accept your robots.txt! )*/
const Device UA2487455 = {S8607, 18, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - ReventIt/Nutch-1.6*/
const Device UA2487944 = {S8608, 21, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - BigBozz Bot/Nutch-1.4*/
const Device UA2488346 = {S8609, 76, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Mozilla/5.0 (compatible; BigBozzBot/1.1; +http://www.bigbozz.com/)/Nutch-1.4*/
const Device UA2490549 = {S8610, 22, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - OpenWebIndex/Nutch-1.6*/
const Device UA2493586 = {S8611, 104, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - NutchCVS/0.8-dev (Nutch running at UW; http://www.nutch.org/docs/en/bot.html; sycrawl@cs.washington.edu)*/
const Device UA2494416 = {S8612, 22, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Nutch Spider/Nutch-1.4*/
const Device UA2504867 = {S8613, 21, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Test Spider/Nutch-2.1*/
const Device UA2514006 = {S8614, 22, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Nutch Spider/Nutch-1.6*/
const Device UA2514353 = {S8615, 122, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - TosCrawler/Nutch-1.6 (http://www.toshiba.co.jp/rdc/about/crawl_info_en.htm; 'Rdc-crawler at ml dot toshiba dot co dot jp')*/
const Device UA2529662 = {S8616, 65, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - INLNutchSpider/Nutch-2.1 (Knowledge Indexer; http://www.inl.gov/)*/
const Device UA2534987 = {S8617, 28, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Affectv Robot v1.0/Nutch-1.6*/
const Device UA2554919 = {S8618, 16, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - spider/Nutch-2.1*/
const Device UA2559755 = {S8619, 22, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - MZone Spider/Nutch-1.6*/
const Device UA2561626 = {S8620, 28, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Super Nutch Spider/Nutch-1.6*/
const Device UA2565581 = {S8621, 82, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)/Nutch-2.1*/
const Device UA2571471 = {S8622, 17, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Crawler/Nutch-1.1*/
const Device UA2571516 = {S8623, 22, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - NutchCrawler/Nutch-1.1*/
const Device UA2577764 = {S8624, 27, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Your Nutch Spider/Nutch-2.1*/
const Device UA2586538 = {S8625, 148, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - spyder/Nutch-2.1 (just another internet crawler; http://www.paloaltonetworks.com/products/features/url-filtering.html; ghalevy@paloaltonetworks.com)*/
const Device UA2589738 = {S8626, 34, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - My Nutch Spider/Nutch-1.6-SNAPSHOT*/
const Device UA2591870 = {S8627, 106, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Googlebot/Nutch-1.4 (Prototype; http://en.wikipedia.org/wiki/Web_crawler; donotreply at prototype dot com)*/
const Device UA2601652 = {S8628, 22, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - NutchCrawler/Nutch-1.6*/
const Device UA2656501 = {S8629, 20, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - my crawler/Nutch-1.6*/
const Device UA2670826 = {S8630, 88, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Affectv Robot v1.0/Nutch-1.6 (http://www.affectv.co.uk; austin at affectv dot co dot uk)*/
const Device UA2679942 = {S8631, 19, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - KzjnetBot/Nutch-1.6*/
const Device UA2681154 = {S8632, 94, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - NutchCVS/0.7.1 (Nutch; http://lucene.apache.org/nutch/bot.html; nutch-agent@lucene.apache.org)*/
const Device UA2686356 = {S8633, 111, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - HPI-BI-Crawler/0.1(+http://www.hpi.uni-potsdam.de/meinel/forschung/web_30/blog_intelligence.html)/Nutch-2.0-dev*/
const Device UA2713421 = {S8634, 23, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - MyNutchSpider/Nutch-2.1*/
const Device UA2713980 = {S8635, 18, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Dorkobot/Nutch-1.6*/
const Device UA2715224 = {S8636, 155, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - "Mozilla/5.0"/Nutch-2.1 (just another internet crawler; http://www.paloaltonetworks.com/products/features/url-filtering.html; ghalevy@paloaltonetworks.com)*/
const Device UA2716095 = {S8637, 89, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - betasearch/Nutch-2.1 (Academic Beta Search; http://www.aramabeta.com; info@aramabeta.com)*/
const Device UA2721475 = {S8638, 64, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Ziba/Nutch-1.4 (Ziba Links Web Spider; http://www.zibalinks.com)*/
const Device UA2764041 = {S8639, 34, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - My Nutch Spider/Nutch-1.7-SNAPSHOT*/
const Device UA2784293 = {S8640, 21, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - MyNutchTest/Nutch-1.6*/
const Device UA2786794 = {S8641, 50, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Global-Crawler/Nutch-1.4 (Search platform crawler)*/
const Device UA2808745 = {S8642, 74, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - LinexBot/Nutch-1.0 (www.linexsystems.com; support at linexsystems dot com)*/
const Device UA2866594 = {S8643, 46, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Permaculture Super Coop Nutch Spider/Nutch-1.6*/
const Device UA2895371 = {S8644, 21, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - TsolCrawler/Nutch-1.4*/
const Device UA2896467 = {S8645, 105, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - ManziaBot/Nutch-2.1 (Web crawler from Manzia Corp; http://www.manzia.com/crawler; info at manzia dot com)*/
const Device UA2897828 = {S8646, 265, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Nimit is a web indexing system designed to focus crawl the web for educational content. the crawler is built on top of open-source nutch and is intended to to pocketize the web into domains eg content for engineering, medicine,law, humanities and business/Nutch-1.6*/
const Device UA2943992 = {S8647, 266, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Nimit is a web indexing system designed to focus crawl the web for educational content. the crawler is built on top of open-source nutch and is intended to to pocketize the web into domains eg content for engineering, medicine,law, humanities and business./Nutch-1.6*/
const Device UA2973985 = {S8648, 21, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - Acc Crawler/Nutch-1.7*/
const Device UA3057727 = {S8649, 24, { &P18254,&P7480,&P18245,&P18163 }};	/*Useragent - NutchCrawler/Nutch-2.2.1*/
const Device UA235494 = {S8650, 77, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; Edition Next; en) Presto/2.9.186 Version/12.00*/
const Device UA627041 = {S8651, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.289 Version/12.00*/
const Device UA899556 = {S8652, 101, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.289 Version/12.00,gzip(gfe) (via translate.google.com)*/
const Device UA899724 = {S8653, 94, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5328); ru) Presto/2.10.289 Version/12.00*/
const Device UA901401 = {S8654, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.289 Version/12.00*/
const Device UA902725 = {S8655, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.289 Version/12.00*/
const Device UA902833 = {S8656, 94, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5310); ru) Presto/2.10.289 Version/12.00*/
const Device UA903075 = {S8657, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5667); ru) Presto/2.10.289 Version/12.00*/
const Device UA903096 = {S8658, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4876); ru) Presto/2.10.289 Version/12.00*/
const Device UA903266 = {S8659, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.289 Version/12.00*/
const Device UA903388 = {S8660, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.289 Version/12.00*/
const Device UA903403 = {S8661, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.289 Version/12.00*/
const Device UA903371 = {S8662, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5660); ru) Presto/2.10.289 Version/12.00*/
const Device UA903543 = {S8663, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.289 Version/12.00*/
const Device UA903556 = {S8664, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.289 Version/12.00*/
const Device UA903560 = {S8665, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4947); ru) Presto/2.10.289 Version/12.00*/
const Device UA903708 = {S8666, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.289 Version/12.00*/
const Device UA904074 = {S8667, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.289 Version/12.00*/
const Device UA904428 = {S8668, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.289 Version/12.00*/
const Device UA904465 = {S8669, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.289 Version/12.00*/
const Device UA904651 = {S8670, 94, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5231); ru) Presto/2.10.289 Version/12.00*/
const Device UA904820 = {S8671, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.289 Version/12.00*/
const Device UA904866 = {S8672, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5218); ru) Presto/2.10.289 Version/12.00*/
const Device UA904884 = {S8673, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.289 Version/12.00*/
const Device UA905054 = {S8674, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.289 Version/12.00*/
const Device UA905112 = {S8675, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.289 Version/12.00*/
const Device UA905175 = {S8676, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.289 Version/12.00*/
const Device UA905286 = {S8677, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5650); ru) Presto/2.10.289 Version/12.00*/
const Device UA905343 = {S8678, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.289 Version/12.00*/
const Device UA905507 = {S8679, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.289 Version/12.00*/
const Device UA905723 = {S8680, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.10.289 Version/12.00*/
const Device UA905757 = {S8681, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.289 Version/12.00*/
const Device UA906118 = {S8682, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.289 Version/12.00*/
const Device UA906554 = {S8683, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.289 Version/12.00*/
const Device UA906831 = {S8684, 94, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5278); ru) Presto/2.10.289 Version/12.00*/
const Device UA906850 = {S8685, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.10.289 Version/12.00*/
const Device UA907022 = {S8686, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5676); ru) Presto/2.10.289 Version/12.00*/
const Device UA907090 = {S8687, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.289 Version/12.00*/
const Device UA907437 = {S8688, 79, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; de; rv:2.0) Gecko/X Firefox/4.0 Opera 12.00*/
const Device UA907451 = {S8689, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4930); ru) Presto/2.10.289 Version/12.00*/
const Device UA907556 = {S8690, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.289 Version/12.00*/
const Device UA907804 = {S8691, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5288); ru) Presto/2.10.289 Version/12.00*/
const Device UA907994 = {S8692, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.289 Version/12.00*/
const Device UA908619 = {S8693, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5299); ru) Presto/2.10.289 Version/12.00*/
const Device UA909479 = {S8694, 94, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5230); ru) Presto/2.10.289 Version/12.00*/
const Device UA909691 = {S8695, 94, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5336); ru) Presto/2.10.289 Version/12.00*/
const Device UA910732 = {S8696, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.289 Version/12.00*/
const Device UA925364 = {S8697, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.10.289 Version/12.00*/
const Device UA1898556 = {S8698, 66, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; ru) Opera 12.00*/
const Device UA1955857 = {S8699, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4110); ru) Presto/2.10.289 Version/12.00*/
const Device UA1956222 = {S8700, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.289 Version/12.00*/
const Device UA1968479 = {S8701, 71, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; ru) Presto/2.10.289 Version/12.01*/
const Device UA1984572 = {S8702, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.289 Version/12.00*/
const Device UA1988451 = {S8703, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5300); ru) Presto/2.10.289 Version/12.00*/
const Device UA1989374 = {S8704, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5217); ru) Presto/2.10.289 Version/12.00*/
const Device UA1994736 = {S8705, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.289 Version/12.00*/
const Device UA1997123 = {S8706, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5288); ru) Presto/2.10.289 Version/12.01*/
const Device UA1997128 = {S8707, 108, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; ru) Presto/2.10.289 Version/12.01,gzip(gfe) (via translate.google.com)*/
const Device UA1997804 = {S8708, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5831); ru) Presto/2.10.289 Version/12.01*/
const Device UA1997837 = {S8709, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5667); ru) Presto/2.10.289 Version/12.01*/
const Device UA1998537 = {S8710, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5704); ru) Presto/2.10.289 Version/12.01*/
const Device UA1999023 = {S8711, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4664); ru) Presto/2.10.289 Version/12.01*/
const Device UA1999042 = {S8712, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.289 Version/12.01*/
const Device UA1999478 = {S8713, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5328); ru) Presto/2.10.289 Version/12.01*/
const Device UA1999508 = {S8714, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.289 Version/12.01*/
const Device UA1999594 = {S8715, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.289 Version/12.01*/
const Device UA1999889 = {S8716, 94, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.7 (build 03797); ru) Presto/2.10.289 Version/12.01*/
const Device UA1999908 = {S8717, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5680); ru) Presto/2.10.289 Version/12.01*/
const Device UA1999913 = {S8718, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.289 Version/12.01*/
const Device UA1999915 = {S8719, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.289 Version/12.01*/
const Device UA2000823 = {S8720, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5966); ru) Presto/2.10.289 Version/12.01*/
const Device UA2001575 = {S8721, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.289 Version/12.01*/
const Device UA2001678 = {S8722, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5660); ru) Presto/2.10.289 Version/12.01*/
const Device UA2001966 = {S8723, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.289 Version/12.01*/
const Device UA2002743 = {S8724, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.289 Version/12.01*/
const Device UA2002942 = {S8725, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.289 Version/12.01*/
const Device UA2002987 = {S8726, 94, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5265); ru) Presto/2.10.289 Version/12.01*/
const Device UA2003524 = {S8727, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5676); ru) Presto/2.10.289 Version/12.01*/
const Device UA2003818 = {S8728, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.289 Version/12.01*/
const Device UA2003821 = {S8729, 94, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5218); ru) Presto/2.10.289 Version/12.01*/
const Device UA2003844 = {S8730, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.289 Version/12.01*/
const Device UA2003846 = {S8731, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.289 Version/12.01*/
const Device UA2003896 = {S8732, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.289 Version/12.01*/
const Device UA2004279 = {S8733, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.289 Version/12.01*/
const Device UA2004633 = {S8734, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.289 Version/12.01*/
const Device UA2004637 = {S8735, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.289 Version/12.01*/
const Device UA2004638 = {S8736, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4156); ru) Presto/2.10.289 Version/12.01*/
const Device UA2004880 = {S8737, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5754); ru) Presto/2.10.289 Version/12.01*/
const Device UA2004892 = {S8738, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.289 Version/12.01*/
const Device UA2006093 = {S8739, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.289 Version/12.01*/
const Device UA2006112 = {S8740, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.289 Version/12.01*/
const Device UA2007373 = {S8741, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4157); ru) Presto/2.10.289 Version/12.01*/
const Device UA2007382 = {S8742, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.289 Version/12.01*/
const Device UA2008901 = {S8743, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.289 Version/12.01*/
const Device UA2012029 = {S8744, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.289 Version/12.01*/
const Device UA2016143 = {S8745, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5970); ru) Presto/2.10.289 Version/12.01*/
const Device UA2041308 = {S8746, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.289 Version/12.00*/
const Device UA2154735 = {S8747, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.289 Version/12.00*/
const Device UA2156377 = {S8748, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.289 Version/12.00*/
const Device UA2161627 = {S8749, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4832); ru) Presto/2.10.289 Version/12.00*/
const Device UA2168257 = {S8750, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.289 Version/12.01*/
const Device UA2168911 = {S8751, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.289 Version/12.01*/
const Device UA2169139 = {S8752, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4930); ru) Presto/2.10.289 Version/12.01*/
const Device UA2169578 = {S8753, 94, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5230); ru) Presto/2.10.289 Version/12.01*/
const Device UA2171352 = {S8754, 94, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.7 (build 03773); ru) Presto/2.10.289 Version/12.01*/
const Device UA2171902 = {S8755, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03649); ru) Presto/2.10.289 Version/12.01*/
const Device UA2173872 = {S8756, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5650); ru) Presto/2.10.289 Version/12.01*/
const Device UA2175407 = {S8757, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.289 Version/12.01*/
const Device UA2185163 = {S8758, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5970); ru) Presto/2.10.289 Version/12.00*/
const Device UA2200353 = {S8759, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5966); ru) Presto/2.10.289 Version/12.00*/
const Device UA2204905 = {S8760, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5976); ru) Presto/2.10.289 Version/12.00*/
const Device UA2205751 = {S8761, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5976); ru) Presto/2.10.289 Version/12.01*/
const Device UA2262767 = {S8762, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03639); ru) Presto/2.10.289 Version/12.00*/
const Device UA2262771 = {S8763, 74, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.289 Version/12.01,gzip(gfe)*/
const Device UA2262774 = {S8764, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.289 Version/12.01*/
const Device UA2262864 = {S8765, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4661); ru) Presto/2.10.289 Version/12.01*/
const Device UA2318250 = {S8766, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03402); ru) Presto/2.10.289 Version/12.00*/
const Device UA2318429 = {S8767, 94, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5161); ru) Presto/2.10.289 Version/12.00*/
const Device UA2329551 = {S8768, 94, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.6 (build 03278); ru) Presto/2.10.289 Version/12.00*/
const Device UA2333528 = {S8769, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03780); ru) Presto/2.10.289 Version/12.00*/
const Device UA2339531 = {S8770, 94, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5299); ru) Presto/2.10.289 Version/12.01*/
const Device UA2340032 = {S8771, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5336); ru) Presto/2.10.289 Version/12.01*/
const Device UA2340043 = {S8772, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5325); ru) Presto/2.10.289 Version/12.01*/
const Device UA2340909 = {S8773, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.289 Version/12.01*/
const Device UA2363041 = {S8774, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.289 Version/12.01*/
const Device UA2369562 = {S8775, 63, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/12.0 (Windows NT 6.1; U; en) Presto/2.9.181 Version/12.00*/
const Device UA2375962 = {S8776, 94, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.6 (build 03278); ru) Presto/2.10.289 Version/12.01*/
const Device UA2384952 = {S8777, 74, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.289 Version/12.00,gzip(gfe)*/
const Device UA2385918 = {S8778, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.289 Version/12.00*/
const Device UA2386535 = {S8779, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 6001); ru) Presto/2.10.289 Version/12.00*/
const Device UA2395959 = {S8780, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5278); ru) Presto/2.10.289 Version/12.01*/
const Device UA2403423 = {S8781, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5997); ru) Presto/2.10.289 Version/12.01*/
const Device UA2403650 = {S8782, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5784); ru) Presto/2.10.289 Version/12.00*/
const Device UA2403777 = {S8783, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.289 Version/12.01*/
const Device UA2404634 = {S8784, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.289 Version/12.00*/
const Device UA2404837 = {S8785, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 6005); ru) Presto/2.10.289 Version/12.01*/
const Device UA2413959 = {S8786, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4110); ru) Presto/2.10.289 Version/12.01*/
const Device UA2425689 = {S8787, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5745); ru) Presto/2.10.289 Version/12.00*/
const Device UA2446608 = {S8788, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.289 Version/12.01*/
const Device UA2455925 = {S8789, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5784); ru) Presto/2.10.289 Version/12.01*/
const Device UA2460264 = {S8790, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.289 Version/12.00*/
const Device UA2469078 = {S8791, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 6015); ru) Presto/2.10.289 Version/12.01*/
const Device UA2502915 = {S8792, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5993); ru) Presto/2.10.289 Version/12.01*/
const Device UA2504781 = {S8793, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5880); ru) Presto/2.10.289 Version/12.00*/
const Device UA2526612 = {S8794, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5745); ru) Presto/2.10.289 Version/12.01*/
const Device UA2527163 = {S8795, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5997); ru) Presto/2.10.289 Version/12.00*/
const Device UA2556759 = {S8796, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5981); ru) Presto/2.10.289 Version/12.01*/
const Device UA2578130 = {S8797, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5993); ru) Presto/2.10.289 Version/12.00*/
const Device UA2578875 = {S8798, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5880); ru) Presto/2.10.289 Version/12.01*/
const Device UA2583789 = {S8799, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.289 Version/12.01*/
const Device UA2588227 = {S8800, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5325); ru) Presto/2.10.289 Version/12.00*/
const Device UA2600709 = {S8801, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.289 Version/12.00*/
const Device UA2665939 = {S8802, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.289 Version/12.01*/
const Device UA2697927 = {S8803, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5977); ru) Presto/2.10.289 Version/12.01*/
const Device UA2701970 = {S8804, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 6037); ru) Presto/2.10.289 Version/12.00*/
const Device UA2705196 = {S8805, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.289 Version/12.00*/
const Device UA2705782 = {S8806, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 6007); ru) Presto/2.10.289 Version/12.01*/
const Device UA2722875 = {S8807, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.289 Version/12.01*/
const Device UA2734200 = {S8808, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6081); ru) Presto/2.10.289 Version/12.01*/
const Device UA2749462 = {S8809, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 6003); ru) Presto/2.10.289 Version/12.01*/
const Device UA2750781 = {S8810, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6081); ru) Presto/2.10.289 Version/12.00*/
const Device UA2750985 = {S8811, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6089); ru) Presto/2.10.289 Version/12.00*/
const Device UA2760848 = {S8812, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.289 Version/12.01*/
const Device UA2775779 = {S8813, 94, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5217); ru) Presto/2.10.289 Version/12.01*/
const Device UA2792139 = {S8814, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6011); ru) Presto/2.10.289 Version/12.01*/
const Device UA2794698 = {S8815, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6089); ru) Presto/2.10.289 Version/12.01*/
const Device UA2809299 = {S8816, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4661); ru) Presto/2.10.289 Version/12.00*/
const Device UA2819518 = {S8817, 94, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.7 (build 03639); ru) Presto/2.10.289 Version/12.01*/
const Device UA2819900 = {S8818, 94, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5300); ru) Presto/2.10.289 Version/12.01*/
const Device UA2823848 = {S8819, 87, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5161); ru) Presto/2.10.289 Version/12.01*/
const Device UA2847952 = {S8820, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5981); ru) Presto/2.10.289 Version/12.00*/
const Device UA2850371 = {S8821, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5989); ru) Presto/2.10.289 Version/12.01*/
const Device UA2853966 = {S8822, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6048); ru) Presto/2.10.289 Version/12.01*/
const Device UA2855717 = {S8823, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 6054); ru) Presto/2.10.289 Version/12.00*/
const Device UA2859412 = {S8824, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 6008); ru) Presto/2.10.289 Version/12.00*/
const Device UA2859944 = {S8825, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 6008); ru) Presto/2.10.289 Version/12.01*/
const Device UA2862610 = {S8826, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6083); ru) Presto/2.10.289 Version/12.00*/
const Device UA2867972 = {S8827, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.289 Version/12.00*/
const Device UA2868095 = {S8828, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.289 Version/12.01*/
const Device UA2988256 = {S8829, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 6003); ru) Presto/2.10.289 Version/12.00*/
const Device UA2992161 = {S8830, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 6083); ru) Presto/2.10.289 Version/12.01*/
const Device UA3045755 = {S8831, 93, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 6007); ru) Presto/2.10.289 Version/12.00*/
const Device UA3099537 = {S8832, 86, { &P15364,&P7133,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6011); ru) Presto/2.10.289 Version/12.00*/
const Device UA2427893 = {S8833, 86, { &P17968,&P17979,&P18369,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/7.0.29952/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA368202 = {S8834, 104, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/18.6.872.0 Safari/535.8*/
const Device UA441158 = {S8835, 148, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA454773 = {S8836, 141, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA458294 = {S8837, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/1854; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA605640 = {S8838, 139, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19,gzip(gfe) (via translate.google.com)*/
const Device UA606640 = {S8839, 146, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19,gzip(gfe) (via translate.google.com)*/
const Device UA607524 = {S8840, 138, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/534.7 ZemanaAID/FFFF0061*/
const Device UA624761 = {S8841, 109, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19*/
const Device UA692168 = {S8842, 107, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/18.6.872.0 Safari/535.2  3gpp-gba*/
const Device UA844263 = {S8843, 109, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/180.1.1025.40 Safari/535.19*/
const Device UA895330 = {S8844, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/1895; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA903739 = {S8845, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/1911; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA909419 = {S8846, 145, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Client/1911; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA911039 = {S8847, 136, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Client/0; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA911529 = {S8848, 106, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.1634 Safari/535.19 YE*/
const Device UA1989409 = {S8849, 170, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19 AppEngine-Google; (+http://code.google.com/appengine; appid: s~cxX)*/
const Device UA1997105 = {S8850, 145, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Client/1928; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2005209 = {S8851, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/1950; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2007980 = {S8852, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/1949; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2043660 = {S8853, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/1957; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2043627 = {S8854, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/1960; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2167892 = {S8855, 177, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19 AppEngine-Google; (+http://code.google.com/appengine; appid: s~eachshadow)*/
const Device UA2193435 = {S8856, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2008; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2194222 = {S8857, 145, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Client/1960; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2197495 = {S8858, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/1999; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2203275 = {S8859, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2001; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2206102 = {S8860, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2004; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2206184 = {S8861, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2005; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2218672 = {S8862, 114, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1;) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.45 Safari/535.19*/
const Device UA2220282 = {S8863, 109, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; de) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/18.0 Safari/525.13*/
const Device UA2229354 = {S8864, 175, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19 AppEngine-Google; (+http://code.google.com/appengine; appid: s~momomich)*/
const Device UA2235766 = {S8865, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2026; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2235856 = {S8866, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2025; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2238275 = {S8867, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2009; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2260303 = {S8868, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2037; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2260369 = {S8869, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2035; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2260370 = {S8870, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2038; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2260422 = {S8871, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2040; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2340481 = {S8872, 145, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Client/1950; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2355605 = {S8873, 145, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Client/2026; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2358588 = {S8874, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2046; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2359533 = {S8875, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2048; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2363165 = {S8876, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2057; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2369027 = {S8877, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2082; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2388105 = {S8878, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2094; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2388223 = {S8879, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2092; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2390783 = {S8880, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2093; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2391464 = {S8881, 146, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Tenfoot/2095; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2391463 = {S8882, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2095; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2407514 = {S8883, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2100; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2408420 = {S8884, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2102; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2424228 = {S8885, 132, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/18.0 BIDUPlayerBrowser/2.x Safari/534.7*/
const Device UA2430534 = {S8886, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2109; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2431386 = {S8887, 146, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Tenfoot/2109; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2432955 = {S8888, 145, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Client/2109; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2432998 = {S8889, 146, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Tenfoot/2102; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2437338 = {S8890, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2106; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2455797 = {S8891, 112, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19,gzip(gfe)*/
const Device UA2461976 = {S8892, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2119; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2470518 = {S8893, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2122; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2472041 = {S8894, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/2125; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2490823 = {S8895, 146, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Tenfoot/2125; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2508001 = {S8896, 145, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Client/2125; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2535470 = {S8897, 143, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Tenfoot/X; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2545041 = {S8898, 119, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19,gzip(gfe)*/
const Device UA2566581 = {S8899, 142, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam Client/X; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2568263 = {S8900, 120, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/18.0 BIDUPlayerBrowser/2.x Safari/534.7*/
const Device UA2644116 = {S8901, 117, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19; Java/1.7.0_03*/
const Device UA2813906 = {S8902, 102, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/180.1.1025.40 Safari/535.19*/
const Device UA2947806 = {S8903, 150, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/X; ) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Safari/535.19*/
const Device UA2980326 = {S8904, 137, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; DOTA 2 Client/0; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2995438 = {S8905, 139, { &P15364,&P7133,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Half-Life/0; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2015623 = {S8906, 99, { &P15364,&P7133,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2015624 = {S8907, 106, { &P15364,&P7133,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2040558 = {S8908, 136, { &P15364,&P7133,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)*/
const Device UA2041086 = {S8909, 145, { &P15364,&P7133,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2041372 = {S8910, 138, { &P15364,&P7133,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2041730 = {S8911, 143, { &P15364,&P7133,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)*/
const Device UA2191014 = {S8912, 176, { &P15364,&P7133,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1 AppEngine-Google; (+http://code.google.com/appengine; appid: s~yunantongxue)*/
const Device UA2230758 = {S8913, 109, { &P15364,&P7133,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe)*/
const Device UA2233997 = {S8914, 116, { &P15364,&P7133,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe)*/
const Device UA2349259 = {S8915, 177, { &P15364,&P7133,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1 AppEngine-Google; (+http://code.google.com/appengine; appid: s~mintchipchoco)*/
const Device UA2551442 = {S8916, 111, { &P15364,&P7133,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1*/
const Device UA2603216 = {S8917, 174, { &P15364,&P7133,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1 AppEngine-Google; (+http://code.google.com/appengine; appid: s~text2solve)*/
const Device UA2843800 = {S8918, 105, { &P15364,&P7133,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA3101945 = {S8919, 119, { &P15364,&P7133,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US)  AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1*/
const Device UA353880 = {S8920, 100, { &P15364,&P7133,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.12 Safari/535.11*/
const Device UA401278 = {S8921, 107, { &P15364,&P7133,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA401829 = {S8922, 146, { &P15364,&P7133,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA427022 = {S8923, 111, { &P15364,&P7133,&P17830,&P18092 }};	/*Useragent - User-Agent:Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA591195 = {S8924, 136, { &P15364,&P7133,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/534.7 ZemanaAID/FFFF0061*/
const Device UA599939 = {S8925, 139, { &P15364,&P7133,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2166764 = {S8926, 162, { &P15364,&P7133,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.84 Safari/535.11 SE 2.X MetaSr 1.0,gzip(gfe) (via translate.google.com)*/
const Device UA2167524 = {S8927, 155, { &P15364,&P7133,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.84 Safari/535.11 SE 2.X MetaSr 1.0,gzip(gfe) (via translate.google.com)*/
const Device UA2259147 = {S8928, 128, { &P15364,&P7133,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.84 Safari/535.11 SE 2.X MetaSr 1.0,gzip(gfe)*/
const Device UA2259148 = {S8929, 135, { &P15364,&P7133,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.84 Safari/535.11 SE 2.X MetaSr 1.0,gzip(gfe)*/
const Device UA2355036 = {S8930, 184, { &P15364,&P7133,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.33 Safari/535.11 AppEngine-Google; (+http://code.google.com/appengine; appid: optimizelyedit)*/
const Device UA2364648 = {S8931, 107, { &P15364,&P7133,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML	 like Gecko) Chrome/17.0.963.38 Safari/535.11*/
const Device UA2367871 = {S8932, 160, { &P15364,&P7133,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/17.0.944.0 Safari/535.8 BitdefenderSafepay/0.2.0.0 (OBK; 17.0.944.0; r396.2011)*/
const Device UA2404559 = {S8933, 153, { &P15364,&P7133,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/17.0.944.0 Safari/535.8 BitdefenderSafepay/0.2.0.0 (OBK; 17.0.944.0; r396.2011)*/
const Device UA2597218 = {S8934, 99, { &P15364,&P7133,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML like Gecko) Chrome/17.0.963.46 Safari/535.11*/
const Device UA2598915 = {S8935, 113, { &P15364,&P7133,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.1.963.51 Safari/535.11 MRCHROME SOC*/
const Device UA2600320 = {S8936, 120, { &P15364,&P7133,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.1.963.51 Safari/535.11 MRCHROME SOC*/
const Device UA439415 = {S8937, 98, { &P18088,&P17017,&P17371,&P18092 }};	/*Useragent - Opera/10.61 (J2ME/MIDP; Opera Mini/5.1.21219/19.999; en-US; rv:1.9.3a5) WebKit/534.5 Presto/2.6.30*/
const Device UA893719 = {S8938, 86, { &P18088,&P17017,&P17371,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/5.18487/26.1670; U; es) Presto/2.8.119 Version/10.54*/
const Device UA2043650 = {S8939, 97, { &P18088,&P17017,&P17371,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/5.1.21219/19.999; en-US; rv:1.9.3a5) WebKit/534.5 Presto/2.6.30*/
const Device UA2389249 = {S8940, 58, { &P18088,&P17017,&P17371,&P18092 }};	/*Useragent - Opera/9.50 (J2ME/MIDP; Opera Mini/5.1.2169/28.3165; U; en)*/
const Device UA2431215 = {S8941, 88, { &P18088,&P17017,&P17371,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/5.1.24009/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2431325 = {S8942, 81, { &P18088,&P17017,&P17371,&P18092 }};	/*Useragent - Opera/9.80 (MTK; Opera Mini/5.1.3119/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2756420 = {S8943, 117, { &P15364,&P5690,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36*/
const Device UA2774359 = {S8944, 97, { &P15364,&P5690,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.1.0.0 Safari/537.36*/
const Device UA2800611 = {S8945, 104, { &P15364,&P5690,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.1.0.0 Safari/537.36*/
const Device UA2873610 = {S8946, 102, { &P15364,&P5690,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2873781 = {S8947, 109, { &P15364,&P5690,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2873936 = {S8948, 112, { &P15364,&P5690,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe)*/
const Device UA2874218 = {S8949, 102, { &P15364,&P5690,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2876292 = {S8950, 141, { &P15364,&P5690,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2876777 = {S8951, 109, { &P15364,&P5690,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2877028 = {S8952, 97, { &P15364,&P5690,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.2.0.0 Safari/537.36*/
const Device UA2893050 = {S8953, 124, { &P15364,&P5690,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36*/
const Device UA2938463 = {S8954, 97, { &P15364,&P5690,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.1.0.0 Safari/537.36*/
const Device UA2958465 = {S8955, 117, { &P15364,&P5690,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36*/
const Device UA2968905 = {S8956, 121, { &P15364,&P5690,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA3002706 = {S8957, 104, { &P15364,&P5690,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.2.0.0 Safari/537.36*/
const Device UA3052621 = {S8958, 176, { &P15364,&P5690,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36 AppEngine-Google; (+http://code.google.com/appengine; appid: s~nikajiyang)*/
const Device UA3080873 = {S8959, 110, { &P15364,&P5690,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.11.1453.94 Safari/537.36 QIPSurf*/
const Device UA5629 = {S8960, 117, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190*/
const Device UA5652 = {S8961, 117, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4*/
const Device UA5654 = {S8962, 106, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA5670 = {S8963, 117, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F191*/
const Device UA5688 = {S8964, 133, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 Safari/533.17.9*/
const Device UA5703 = {S8965, 117, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7*/
const Device UA5719 = {S8966, 117, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2*/
const Device UA5720 = {S8967, 106, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA5724 = {S8968, 133, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/533.17.9*/
const Device UA5702 = {S8969, 106, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA5740 = {S8970, 117, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3*/
const Device UA30063 = {S8971, 104, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA37154 = {S8972, 117, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1*/
const Device UA37161 = {S8973, 106, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA46553 = {S8974, 117, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2*/
const Device UA46557 = {S8975, 106, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA74148 = {S8976, 101, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0(iPhone;U;CPUiPhoneOS4_3_5likeMacOSX;ru-ru)AppleWebKit/533.17.9(KHTML,likeGecko)Mobile/8L1*/
const Device UA90486 = {S8977, 104, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en_us) AppleWebKit/525.18.1 (KHTML, like Gecko)*/
const Device UA90718 = {S8978, 106, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en_us) AppleWebKit/525.18.1 (KHTML, like Gecko)*/
const Device UA90720 = {S8979, 106, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en_us) AppleWebKit/525.18.1 (KHTML, like Gecko)*/
const Device UA176081 = {S8980, 127, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone Simulator; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F192*/
const Device UA181290 = {S8981, 117, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H8*/
const Device UA184912 = {S8982, 133, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190 Safari/533.17.9*/
const Device UA188253 = {S8983, 133, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 Safari/533.17.9*/
const Device UA197706 = {S8984, 119, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/9Z999*/
const Device UA226620 = {S8985, 133, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2 Safari/533.17.9*/
const Device UA234873 = {S8986, 133, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/533.17.9*/
const Device UA360711 = {S8987, 46, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Miu Ptt 1.5.2 (iPhone; iPhone OS 4.3.5; zh_TW)*/
const Device UA363556 = {S8988, 46, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Miu Ptt 1.5.3 (iPhone; iPhone OS 4.3.3; zh_TW)*/
const Device UA394205 = {S8989, 136, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - NaverCafe/1.0 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName*/
const Device UA420020 = {S8990, 133, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 Safari/533.17.9*/
const Device UA455658 = {S8991, 248, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.3.4;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA455812 = {S8992, 244, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.3;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA456054 = {S8993, 248, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone2,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.3.1;FBSS/1; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/1.0]*/
const Device UA541766 = {S8994, 119, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/9Z999*/
const Device UA547683 = {S8995, 119, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/9Z999*/
const Device UA553553 = {S8996, 117, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; zh-tw) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2*/
const Device UA624467 = {S8997, 248, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.3.2;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA631268 = {S8998, 248, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.3.3;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA663539 = {S8999, 248, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone2,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.3.5;FBSS/1; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/1.0]*/
const Device UA771061 = {S9000, 136, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - NaverCafe/1.0 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName*/
const Device UA2137030 = {S9001, 117, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (device; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2*/
const Device UA2284951 = {S9002, 111, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2*/
const Device UA2297081 = {S9003, 165, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper*/
const Device UA2411018 = {S9004, 145, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 iPad2,1 BingWeb/3.03.1428.X*/
const Device UA2419471 = {S9005, 144, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone 4_3_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPhone3,1/4.3.3)*/
const Device UA2643045 = {S9006, 145, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 iPad1,1 BingWeb/3.03.1428.X*/
const Device UA2681955 = {S9007, 114, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8K2*/
const Device UA2778645 = {S9008, 114, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; vi-vn) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8F190*/
const Device UA2835253 = {S9009, 142, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone 4_3_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPad2,2/4.3.3)*/
const Device UA2934148 = {S9010, 119, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; es-es) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148*/
const Device UA2977929 = {S9011, 114, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8J2*/
const Device UA3117433 = {S9012, 101, { &P12280,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko)*/
const Device UA3062784 = {S9013, 119, { &P17595,&P18207,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3062789 = {S9014, 119, { &P17595,&P18207,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3062793 = {S9015, 119, { &P17595,&P18207,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3062799 = {S9016, 119, { &P17595,&P18207,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3063505 = {S9017, 119, { &P17595,&P18207,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3065632 = {S9018, 129, { &P17595,&P18207,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36,gzip(gfe)*/
const Device UA3065809 = {S9019, 158, { &P17595,&P18207,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3093712 = {S9020, 129, { &P17595,&P18207,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36,gzip(gfe)*/
const Device UA3096518 = {S9021, 119, { &P17595,&P18207,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36*/
const Device UA3096691 = {S9022, 135, { &P17595,&P18207,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.36 (KHTML, like Gecko) Iron/29.0.1600.1 Chrome/29.0.1600.1 Safari/537.36*/
const Device UA3114033 = {S9023, 129, { &P17595,&P18207,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36,gzip(gfe)*/
const Device UA3114686 = {S9024, 158, { &P17595,&P18207,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2013535 = {S9025, 71, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; en) Presto/2.10.289 Version/12.02*/
const Device UA2040426 = {S9026, 94, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5328); ru) Presto/2.10.289 Version/12.02*/
const Device UA2040491 = {S9027, 79, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; en; rv:2.0) Gecko/X Firefox/4.0 Opera 12.02*/
const Device UA2040582 = {S9028, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.289 Version/12.02*/
const Device UA2040590 = {S9029, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.289 Version/12.02*/
const Device UA2040819 = {S9030, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4876); ru) Presto/2.10.289 Version/12.02*/
const Device UA2041075 = {S9031, 94, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.7 (build 03797); ru) Presto/2.10.289 Version/12.02*/
const Device UA2041251 = {S9032, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.289 Version/12.02*/
const Device UA2041258 = {S9033, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.289 Version/12.02*/
const Device UA2041334 = {S9034, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5972); ru) Presto/2.10.289 Version/12.02*/
const Device UA2041344 = {S9035, 101, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.289 Version/12.02,gzip(gfe) (via translate.google.com)*/
const Device UA2041465 = {S9036, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.289 Version/12.02*/
const Device UA2041705 = {S9037, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.289 Version/12.02*/
const Device UA2041748 = {S9038, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.289 Version/12.02*/
const Device UA2041782 = {S9039, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5711); ru) Presto/2.10.289 Version/12.02*/
const Device UA2041923 = {S9040, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.289 Version/12.02*/
const Device UA2042166 = {S9041, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5970); ru) Presto/2.10.289 Version/12.02*/
const Device UA2042355 = {S9042, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5680); ru) Presto/2.10.289 Version/12.02*/
const Device UA2042360 = {S9043, 94, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5310); ru) Presto/2.10.289 Version/12.02*/
const Device UA2042670 = {S9044, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.10.289 Version/12.02*/
const Device UA2042704 = {S9045, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.289 Version/12.02*/
const Device UA2042783 = {S9046, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5831); ru) Presto/2.10.289 Version/12.02*/
const Device UA2185198 = {S9047, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5676); ru) Presto/2.10.289 Version/12.02*/
const Device UA2185399 = {S9048, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.289 Version/12.02*/
const Device UA2186266 = {S9049, 94, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5218); ru) Presto/2.10.289 Version/12.02*/
const Device UA2186603 = {S9050, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.289 Version/12.02*/
const Device UA2186888 = {S9051, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.289 Version/12.02*/
const Device UA2187414 = {S9052, 73, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; en) Opera 12.02*/
const Device UA2187675 = {S9053, 94, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5308); ru) Presto/2.10.289 Version/12.02*/
const Device UA2188614 = {S9054, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.289 Version/12.02*/
const Device UA2188623 = {S9055, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.289 Version/12.02*/
const Device UA2189029 = {S9056, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5966); ru) Presto/2.10.289 Version/12.02*/
const Device UA2189920 = {S9057, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.289 Version/12.02*/
const Device UA2190370 = {S9058, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4933); ru) Presto/2.10.289 Version/12.02*/
const Device UA2190381 = {S9059, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.289 Version/12.02*/
const Device UA2190386 = {S9060, 94, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5230); ru) Presto/2.10.289 Version/12.02*/
const Device UA2192610 = {S9061, 94, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5300); ru) Presto/2.10.289 Version/12.02*/
const Device UA2193042 = {S9062, 94, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5282); ru) Presto/2.10.289 Version/12.02*/
const Device UA2193417 = {S9063, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.289 Version/12.02*/
const Device UA2195263 = {S9064, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.289 Version/12.02*/
const Device UA2195515 = {S9065, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.289 Version/12.02*/
const Device UA2195779 = {S9066, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5667); ru) Presto/2.10.289 Version/12.02*/
const Device UA2198367 = {S9067, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4930); ru) Presto/2.10.289 Version/12.02*/
const Device UA2198375 = {S9068, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4848); ru) Presto/2.10.289 Version/12.02*/
const Device UA2198392 = {S9069, 94, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5278); ru) Presto/2.10.289 Version/12.02*/
const Device UA2198394 = {S9070, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5660); ru) Presto/2.10.289 Version/12.02*/
const Device UA2199283 = {S9071, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5299); ru) Presto/2.10.289 Version/12.02*/
const Device UA2199394 = {S9072, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.289 Version/12.02*/
const Device UA2199571 = {S9073, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5650); ru) Presto/2.10.289 Version/12.02*/
const Device UA2200576 = {S9074, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.289 Version/12.02*/
const Device UA2201375 = {S9075, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.289 Version/12.02*/
const Device UA2202513 = {S9076, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.289 Version/12.02*/
const Device UA2203333 = {S9077, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03649); ru) Presto/2.10.289 Version/12.02*/
const Device UA2205192 = {S9078, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5288); ru) Presto/2.10.289 Version/12.02*/
const Device UA2205395 = {S9079, 94, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.7 (build 03653); ru) Presto/2.10.289 Version/12.02*/
const Device UA2205402 = {S9080, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5976); ru) Presto/2.10.289 Version/12.02*/
const Device UA2226614 = {S9081, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.289 Version/12.02*/
const Device UA2227947 = {S9082, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4110); ru) Presto/2.10.289 Version/12.02*/
const Device UA2227948 = {S9083, 94, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.7 (build 03639); ru) Presto/2.10.289 Version/12.02*/
const Device UA2229282 = {S9084, 94, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5217); ru) Presto/2.10.289 Version/12.02*/
const Device UA2229472 = {S9085, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03278); ru) Presto/2.10.289 Version/12.02*/
const Device UA2230804 = {S9086, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.289 Version/12.02*/
const Device UA2230812 = {S9087, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4832); ru) Presto/2.10.289 Version/12.02*/
const Device UA2230869 = {S9088, 94, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.5 (build 02842); ru) Presto/2.10.289 Version/12.02*/
const Device UA2230895 = {S9089, 94, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.6 (build 03402); ru) Presto/2.10.289 Version/12.02*/
const Device UA2232671 = {S9090, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5734); ru) Presto/2.10.289 Version/12.02*/
const Device UA2233710 = {S9091, 74, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.289 Version/12.02,gzip(gfe)*/
const Device UA2234661 = {S9092, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5745); ru) Presto/2.10.289 Version/12.02*/
const Device UA2262763 = {S9093, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03672); ru) Presto/2.10.289 Version/12.02*/
const Device UA2262764 = {S9094, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5325); ru) Presto/2.10.289 Version/12.02*/
const Device UA2262765 = {S9095, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5336); ru) Presto/2.10.289 Version/12.02*/
const Device UA2262769 = {S9096, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5993); ru) Presto/2.10.289 Version/12.02*/
const Device UA2262785 = {S9097, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.289 Version/12.02*/
const Device UA2262831 = {S9098, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5997); ru) Presto/2.10.289 Version/12.02*/
const Device UA2262836 = {S9099, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5784); ru) Presto/2.10.289 Version/12.02*/
const Device UA2262841 = {S9100, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5998); ru) Presto/2.10.289 Version/12.02*/
const Device UA2262816 = {S9101, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4661); ru) Presto/2.10.289 Version/12.02*/
const Device UA2346200 = {S9102, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03755); ru) Presto/2.10.289 Version/12.02*/
const Device UA2369693 = {S9103, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.289 Version/12.02*/
const Device UA2401256 = {S9104, 84, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; de; rv:2.0) Gecko/X Firefox/4.0 Opera 12.02*/
const Device UA2405874 = {S9105, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5880); ru) Presto/2.10.289 Version/12.02*/
const Device UA2428273 = {S9106, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5861); ru) Presto/2.10.289 Version/12.02*/
const Device UA2429555 = {S9107, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6011); ru) Presto/2.10.289 Version/12.02*/
const Device UA2431107 = {S9108, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.289 Version/12.02*/
const Device UA2435741 = {S9109, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5977); ru) Presto/2.10.289 Version/12.02*/
const Device UA2450068 = {S9110, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.289 Version/12.02*/
const Device UA2451384 = {S9111, 94, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5161); ru) Presto/2.10.289 Version/12.02*/
const Device UA2455493 = {S9112, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5730); ru) Presto/2.10.289 Version/12.02*/
const Device UA2484545 = {S9113, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5981); ru) Presto/2.10.289 Version/12.02*/
const Device UA2490836 = {S9114, 66, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/12.80 (Windows NT 6.1; U; en) Presto/22.12.289 Version/12.02*/
const Device UA2551805 = {S9115, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5309); ru) Presto/2.10.289 Version/12.02*/
const Device UA2560877 = {S9116, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6037); ru) Presto/2.10.289 Version/12.02*/
const Device UA2561846 = {S9117, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.289 Version/12.02*/
const Device UA2570218 = {S9118, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.289 Version/12.02*/
const Device UA2580227 = {S9119, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5989); ru) Presto/2.10.289 Version/12.02*/
const Device UA2599607 = {S9120, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 6007); ru) Presto/2.10.289 Version/12.02*/
const Device UA2600607 = {S9121, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 6054); ru) Presto/2.10.289 Version/12.02*/
const Device UA2603528 = {S9122, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 6003); ru) Presto/2.10.289 Version/12.02*/
const Device UA2663947 = {S9123, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6048); ru) Presto/2.10.289 Version/12.02*/
const Device UA2669067 = {S9124, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6081); ru) Presto/2.10.289 Version/12.02*/
const Device UA2704451 = {S9125, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6083); ru) Presto/2.10.289 Version/12.02*/
const Device UA2704524 = {S9126, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6042); ru) Presto/2.10.289 Version/12.02*/
const Device UA2708184 = {S9127, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 6008); ru) Presto/2.10.289 Version/12.02*/
const Device UA2711170 = {S9128, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6089); ru) Presto/2.10.289 Version/12.02*/
const Device UA2749200 = {S9129, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5999); ru) Presto/2.10.289 Version/12.02*/
const Device UA2760595 = {S9130, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03780); ru) Presto/2.10.289 Version/12.02*/
const Device UA2765940 = {S9131, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6080); ru) Presto/2.10.289 Version/12.02*/
const Device UA2770104 = {S9132, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.1 (build 6578); ru) Presto/2.10.289 Version/12.02*/
const Device UA2823770 = {S9133, 87, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03638); ru) Presto/2.10.289 Version/12.02*/
const Device UA2853109 = {S9134, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5990); ru) Presto/2.10.289 Version/12.02*/
const Device UA2855458 = {S9135, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 6044); ru) Presto/2.10.289 Version/12.02*/
const Device UA2856841 = {S9136, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6046); ru) Presto/2.10.289 Version/12.02*/
const Device UA2891958 = {S9137, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 6017); ru) Presto/2.10.289 Version/12.02*/
const Device UA2981491 = {S9138, 103, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; en) Presto/2.10.289 Version/12.02,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2998252 = {S9139, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 6019); ru) Presto/2.10.289 Version/12.02*/
const Device UA3001038 = {S9140, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.1 (build 6283); ru) Presto/2.10.289 Version/12.02*/
const Device UA3001420 = {S9141, 93, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.1 (build 6638); ru) Presto/2.10.289 Version/12.02*/
const Device UA3009869 = {S9142, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6639); ru) Presto/2.10.289 Version/12.02*/
const Device UA3092206 = {S9143, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.1 (build 6341); ru) Presto/2.10.289 Version/12.02*/
const Device UA3092744 = {S9144, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6683); ru) Presto/2.10.289 Version/12.02*/
const Device UA3124858 = {S9145, 86, { &P15364,&P7133,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6673); ru) Presto/2.10.289 Version/12.02*/
const Device UA2193450 = {S9146, 134, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/23.0.1271.17) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.17 Safari/537.11*/
const Device UA2202769 = {S9147, 134, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/23.0.1271.10) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.10 Safari/537.11*/
const Device UA2231007 = {S9148, 127, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/23.0.1271.26) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.26 Safari/537.11*/
const Device UA2232850 = {S9149, 134, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/23.0.1271.40) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.40 Safari/537.11*/
const Device UA2238126 = {S9150, 134, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/23.0.1271.26) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.26 Safari/537.11*/
const Device UA2259167 = {S9151, 134, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/23.0.1271.60) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.60 Safari/537.11*/
const Device UA2259319 = {S9152, 134, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/23.0.1271.52) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.52 Safari/537.11*/
const Device UA2357887 = {S9153, 134, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/23.0.1271.64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2358372 = {S9154, 127, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/23.0.1271.64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2361978 = {S9155, 207, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; GTB7.4; chromeframe/23.0.1271.64; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)*/
const Device UA2388054 = {S9156, 127, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/23.0.1271.91) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.91 Safari/537.11*/
const Device UA2388087 = {S9157, 134, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/23.0.1271.91) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.91 Safari/537.11*/
const Device UA2389934 = {S9158, 229, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/23.0.1271.91; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)*/
const Device UA2389948 = {S9159, 134, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/23.0.1271.95) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11*/
const Device UA2390007 = {S9160, 127, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/23.0.1271.95) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11*/
const Device UA2408300 = {S9161, 89, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/23.0.1271.97)*/
const Device UA2408304 = {S9162, 96, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/23.0.1271.97)*/
const Device UA2408355 = {S9163, 214, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/23.0.1271.97; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Tablet PC 2.0)*/
const Device UA2408361 = {S9164, 97, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/23.0.1271.97)*/
const Device UA2408370 = {S9165, 134, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/23.0.1271.97) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2408371 = {S9166, 127, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/23.0.1271.97) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2408387 = {S9167, 197, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/23.0.1271.97; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2408408 = {S9168, 199, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; chromeframe/23.0.1271.97)*/
const Device UA2408412 = {S9169, 183, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/23.0.1271.97; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2427659 = {S9170, 202, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; chromeframe/23.0.1271.97)*/
const Device UA2432060 = {S9171, 176, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/23.0.1271.97; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2432832 = {S9172, 194, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; chromeframe/23.0.1271.97)*/
const Device UA2432996 = {S9173, 111, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Avant Browser; chromeframe/23.0.1271.97)*/
const Device UA2433762 = {S9174, 183, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/23.0.1271.97; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2433892 = {S9175, 191, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/23.0.1271.97; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA2434198 = {S9176, 90, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/23.0.1271.97)*/
const Device UA2458417 = {S9177, 202, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/4.0; GTB7.4; chromeframe/23.0.1271.97; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)*/
const Device UA2458450 = {S9178, 104, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Avant Browser; chromeframe/23.0.1271.97)*/
const Device UA2458708 = {S9179, 99, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; .NET4.0C; chromeframe/23.0.1271.97)*/
const Device UA2654947 = {S9180, 282, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/23.0.1271.91; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA3043850 = {S9181, 226, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/23.0.1271.97; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Zune 4.7)*/
const Device UA3103969 = {S9182, 282, { &P15364,&P7133,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/23.0.1271.91; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6136; ms-office; MSOffice 14)*/
const Device UA6654 = {S9183, 90, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.1) Gecko/2008070208 Firefox/3.0.1*/
const Device UA446952 = {S9184, 79, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Mozilla/5.01 (Windows; U; Windows NT 5.1; ru; rv:1.9.0.1) Gecko/X Firefox/3.0.4*/
const Device UA448207 = {S9185, 77, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9) Gecko/X IceWeasel/3.0*/
const Device UA454483 = {S9186, 90, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; de; rv:1.9.0.1) Gecko/X Firefox/3.0.0	Firefox 3.0*/
const Device UA464023 = {S9187, 125, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 (.NET CLR 3.5.30729) WinNT-PAI 31.08.2009*/
const Device UA484253 = {S9188, 62, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2) Gecko/X Firefox/3.0.1*/
const Device UA514044 = {S9189, 125, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 (.NET CLR 3.5.30729) WinNT-A8I 02.01.2011*/
const Device UA549107 = {S9190, 79, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Mozilla/6.0 (Windows; U; Windows NT 5.1; en-US; rv:2.0.0.0) Gecko/X Firefox/3.0*/
const Device UA567633 = {S9191, 81, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - mozilla/5.0 (windows nt 5.2; u; ru; rv:1.9.1.6) gecko/X firefox/3.5.6 opera 10.70*/
const Device UA654788 = {S9192, 84, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9a1) Gecko/X GranParadiso/3.0a1*/
const Device UA696093 = {S9193, 83, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.13) Gecko/X firefox/3.6.18*/
const Device UA704319 = {S9194, 125, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 (.NET CLR 3.5.30729) WinNT-EVI 26.06.2010*/
const Device UA900457 = {S9195, 78, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.12) Gecko/X Firefox/3*/
const Device UA1865685 = {S9196, 99, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9) Gecko/X IceWeasel/3.0 ( .NET CLR 3.5.30729)*/
const Device UA2206872 = {S9197, 98, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9) Gecko/X IceWeasel/3.0 (.NET CLR 3.5.30729)*/
const Device UA2263139 = {S9198, 125, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 (.NET CLR 3.5.30729) WinNT-PAI 30.08.2009*/
const Device UA2270273 = {S9199, 101, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows; U; Windows NT 5.1; lt; rv:1.9.0.14) Gecko/X Firefox/3.0.14 (.NET CLR 3.5.30729)*/
const Device UA2437763 = {S9200, 88, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Windows; U; Windows NT 5.1; lt; rv:1.9.0.14) Gecko/X Firefox/3.0.14 (.NET CLR 3.5.30729)*/
const Device UA2511219 = {S9201, 82, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1 en-US; rv: 1.9.0.10)Gecko/X FireFox/3.0.16*/
const Device UA2545228 = {S9202, 78, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; EN; rv:1.9.0.6) Gecko/X Firefox/3.0.6*/
const Device UA2758031 = {S9203, 73, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows XP 5.2; en-US; rv:1.9) Gecko/Firefox/3.0*/
const Device UA2805936 = {S9204, 78, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; de; rv:1.9.0.8; Gecko/X Firefox/3.07)*/
const Device UA2806166 = {S9205, 133, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.14) Gecko/X Firefox/3.0.14 (.NET CLR 3.5.30729) WinNT-PAU 15.07.2009 FBSMTWB*/
const Device UA2948629 = {S9206, 109, { &P15364,&P5690,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9) Gecko/X IceWeasel/3.0 ( .NET CLR 3.5.30729; .NET4.0E)*/
const Device UA2756419 = {S9207, 124, { &P15364,&P16110,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36*/
const Device UA2770716 = {S9208, 117, { &P15364,&P16110,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36*/
const Device UA2783421 = {S9209, 104, { &P15364,&P16110,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.1.0.0 Safari/537.36*/
const Device UA2783440 = {S9210, 97, { &P15364,&P16110,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.1.0.0 Safari/537.36*/
const Device UA2873607 = {S9211, 109, { &P15364,&P16110,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2873677 = {S9212, 102, { &P15364,&P16110,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2874411 = {S9213, 119, { &P15364,&P16110,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe)*/
const Device UA2875806 = {S9214, 148, { &P15364,&P16110,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2877030 = {S9215, 104, { &P15364,&P16110,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.2.0.0 Safari/537.36*/
const Device UA2877032 = {S9216, 97, { &P15364,&P16110,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.2.0.0 Safari/537.36*/
const Device UA2877311 = {S9217, 112, { &P15364,&P16110,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe)*/
const Device UA2882358 = {S9218, 141, { &P15364,&P16110,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA102032 = {S9219, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA102260 = {S9220, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA102436 = {S9221, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA105221 = {S9222, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-br) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA105425 = {S9223, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA105898 = {S9224, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; it-it) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA106122 = {S9225, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sv-se) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA106409 = {S9226, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pl-pl) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA106679 = {S9227, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; da-dk) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA106762 = {S9228, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA107213 = {S9229, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA109871 = {S9230, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fi-fi) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA110614 = {S9231, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nb-no) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA110693 = {S9232, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-cn) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA114720 = {S9233, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ru-ru) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA119262 = {S9234, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-tw) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA119337 = {S9235, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA119734 = {S9236, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA119814 = {S9237, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA120508 = {S9238, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA121743 = {S9239, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA122727 = {S9240, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sv-se) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA122793 = {S9241, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ja-jp) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA123648 = {S9242, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; it-it) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA124695 = {S9243, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pl-pl) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA125857 = {S9244, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; da-dk) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA126293 = {S9245, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-br) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA129816 = {S9246, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fi-fi) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA130220 = {S9247, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ru-ru) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA131633 = {S9248, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-cn) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA132230 = {S9249, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA132245 = {S9250, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA132377 = {S9251, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA132509 = {S9252, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-tw) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA132724 = {S9253, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ru-ru) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA132884 = {S9254, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-gb) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA132928 = {S9255, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; it-it) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA133198 = {S9256, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nb-no) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA133347 = {S9257, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; da-dk) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA133775 = {S9258, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA133885 = {S9259, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ja-jp) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA134362 = {S9260, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nb-no) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA134376 = {S9261, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; el-gr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA135063 = {S9262, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-au) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA135239 = {S9263, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pl-pl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA135487 = {S9264, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tr-tr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA135640 = {S9265, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA136134 = {S9266, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-cn) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA136181 = {S9267, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-br) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA136821 = {S9268, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fi-fi) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA137220 = {S9269, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sv-se) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA138803 = {S9270, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ca) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA139627 = {S9271, 124, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ar) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA139642 = {S9272, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-xl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA146460 = {S9273, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; af-za) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA153553 = {S9274, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA153570 = {S9275, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA153654 = {S9276, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ja-jp) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA153662 = {S9277, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA154515 = {S9278, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA154975 = {S9279, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA155130 = {S9280, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ru-ru) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA155184 = {S9281, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-tw) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA155186 = {S9282, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; da-dk) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA155501 = {S9283, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-gb) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA156094 = {S9284, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; it-it) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA156269 = {S9285, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nb-no) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA156970 = {S9286, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; he-il) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA157006 = {S9287, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-cn) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA157035 = {S9288, 123, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko, Safari/5533.19.4) ADM/722*/
const Device UA157146 = {S9289, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pl-pl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA157189 = {S9290, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-ca) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA157209 = {S9291, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-au) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA157583 = {S9292, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sv-se) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA157730 = {S9293, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ko-kr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA157940 = {S9294, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; is-is) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA158265 = {S9295, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; uk-ua) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA158384 = {S9296, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fi-fi) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA158464 = {S9297, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-br) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA158500 = {S9298, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tr-tr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA158591 = {S9299, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-pt) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA158876 = {S9300, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; el-gr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA159036 = {S9301, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; cs-cz) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA159511 = {S9302, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-ch) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA159649 = {S9303, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; af-za) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA160656 = {S9304, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sk-sk) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA160934 = {S9305, 124, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA161035 = {S9306, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-xl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA173392 = {S9307, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-pt) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA179500 = {S9308, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180436 = {S9309, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sv-se) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180443 = {S9310, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; cs-cz) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180525 = {S9311, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180544 = {S9312, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180554 = {S9313, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ja-jp) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180599 = {S9314, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fi-fi) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180624 = {S9315, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180633 = {S9316, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-cn) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA181375 = {S9317, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-tw) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA181390 = {S9318, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-pt) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA181468 = {S9319, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; is-is) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA181514 = {S9320, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lt-lt) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA181516 = {S9321, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-gb) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA181526 = {S9322, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA181561 = {S9323, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA181568 = {S9324, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nb-no) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA181779 = {S9325, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-ca) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA182308 = {S9326, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pl-pl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA182669 = {S9327, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-au) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA182678 = {S9328, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ko-kr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA183201 = {S9329, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183203 = {S9330, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-xl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183219 = {S9331, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-gb) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183300 = {S9332, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; th-th) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA183364 = {S9333, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ca) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA183454 = {S9334, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183658 = {S9335, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183687 = {S9336, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-au) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183695 = {S9337, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183698 = {S9338, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; is-is) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183715 = {S9339, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-tw) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA183869 = {S9340, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-ch) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA183872 = {S9341, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ru-ru) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA183936 = {S9342, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-br) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA184096 = {S9343, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA184162 = {S9344, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sk-sk) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA184214 = {S9345, 123, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko, Safari/5533.21.1) ADM/722*/
const Device UA184386 = {S9346, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; cs-cz) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA184393 = {S9347, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; it-it) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA184419 = {S9348, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ru-ru) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA184721 = {S9349, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; da-dk) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA184846 = {S9350, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-ca) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA185409 = {S9351, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-br) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA185586 = {S9352, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; da-dk) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA185600 = {S9353, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fi-fi) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA186384 = {S9354, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lv-lv) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA186386 = {S9355, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sv-se) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA186417 = {S9356, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nb-no) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA186655 = {S9357, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ro-ro) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA187099 = {S9358, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pl-pl) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA187139 = {S9359, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tr-tr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA187216 = {S9360, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; el-gr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA187333 = {S9361, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; vi-vn) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA187583 = {S9362, 124, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ar) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA188057 = {S9363, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; el-gr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA188403 = {S9364, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-xl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA189205 = {S9365, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tr-tr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA189232 = {S9366, 124, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA190034 = {S9367, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hu-hu) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA190921 = {S9368, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; he-il) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA199565 = {S9369, 124, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; yi) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA228046 = {S9370, 120, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) WebClip/5534.49 Safari/5533.22.3*/
const Device UA228178 = {S9371, 117, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA228338 = {S9372, 117, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/5.0.6 Safari/533.22.3*/
const Device UA228941 = {S9373, 117, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA230014 = {S9374, 113, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA230164 = {S9375, 118, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA230362 = {S9376, 117, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA231050 = {S9377, 117, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA263005 = {S9378, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ja-jp) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA267029 = {S9379, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-ca) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA267500 = {S9380, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-pt) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA268256 = {S9381, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; el-gr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA269299 = {S9382, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; th-th) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA271764 = {S9383, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tr-tr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA271785 = {S9384, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-tw) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA272342 = {S9385, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tr-tr) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA272650 = {S9386, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-gb) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA274466 = {S9387, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-ca) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA276594 = {S9388, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-pt) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA276972 = {S9389, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; el-gr) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA277235 = {S9390, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; cs-cz) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA277393 = {S9391, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) WebClip/5527.2 Safari/5533.18.5*/
const Device UA278090 = {S9392, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ko-kr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA278377 = {S9393, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-ca) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA279051 = {S9394, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-pt) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA279869 = {S9395, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; th-th) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA281258 = {S9396, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; he-il) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA286722 = {S9397, 123, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.17.8 (KHTML, like Gecko, Safari/5533.17.8) ADM/722*/
const Device UA292091 = {S9398, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ca) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA292802 = {S9399, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sk-sk) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA293126 = {S9400, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hr-hr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA293778 = {S9401, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ca-es) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA293818 = {S9402, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; vi-vn) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA294262 = {S9403, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; th-th) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA295180 = {S9404, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-at) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA295247 = {S9405, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fa-ir) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA295373 = {S9406, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sl-si) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA296540 = {S9407, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lt-lt) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA297513 = {S9408, 124, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ar) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA300234 = {S9409, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; bg-bg) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA305723 = {S9410, 126, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA306231 = {S9411, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ca) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA307340 = {S9412, 125, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko, Safari/5533.20.27) ADM/722*/
const Device UA307535 = {S9413, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; he-il) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA308359 = {S9414, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lt-lt) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA308355 = {S9415, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; th-th) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA308401 = {S9416, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fa-ir) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA309297 = {S9417, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_8; ja-jp) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA309404 = {S9418, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ko-kr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA309563 = {S9419, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; et-ee) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA310973 = {S9420, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; af-za) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA311275 = {S9421, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_8; zh-cn) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA311687 = {S9422, 126, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ar) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA312263 = {S9423, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; af-za) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA313996 = {S9424, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ch) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA314115 = {S9425, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ca-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA314250 = {S9426, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hy-am) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA314739 = {S9427, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; bg-bg) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA314811 = {S9428, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sl-si) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA314792 = {S9429, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fa-ir) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA314837 = {S9430, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hr-hr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA315186 = {S9431, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; uk-ua) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA315854 = {S9432, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-at) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA317784 = {S9433, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lv-lv) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA319189 = {S9434, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; et-ee) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA372356 = {S9435, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lt-lt) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA372756 = {S9436, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; uk-ua) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA382865 = {S9437, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-xl) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA383187 = {S9438, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; he-il) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA383239 = {S9439, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ko-kr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA383624 = {S9440, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-au) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA384354 = {S9441, 124, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA384536 = {S9442, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; is-is) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA386261 = {S9443, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ro-ro) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA387089 = {S9444, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lv-lv) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA399355 = {S9445, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; th-th) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA414250 = {S9446, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; is-is) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA430463 = {S9447, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; cs-cz) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA430823 = {S9448, 125, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.16 (KHTML, like Gecko) WebClip/5527.2 Safari/5533.16*/
const Device UA431018 = {S9449, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-xl) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA435489 = {S9450, 124, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA448624 = {S9451, 124, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sa) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA468408 = {S9452, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fa-ir) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA469818 = {S9453, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ga-ie) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA470494 = {S9454, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hu-hu) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA472754 = {S9455, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fa-ir) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA473021 = {S9456, 124, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA473484 = {S9457, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; he-il) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA474423 = {S9458, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hu-hu) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA475117 = {S9459, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; et-ee) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA475703 = {S9460, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ca) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA476496 = {S9461, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ch) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA481327 = {S9462, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hr-hr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA492962 = {S9463, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ka-ge) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA497766 = {S9464, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hu-hu) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA500972 = {S9465, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ch) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA502025 = {S9466, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ca-es) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA519450 = {S9467, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; vi-vn) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA524215 = {S9468, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-ch) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA529273 = {S9469, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sk-sk) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA559078 = {S9470, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sl-si) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA566427 = {S9471, 123, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.5+ (KHTML, like Gecko) Version/5.0.6 Safari/533.5+*/
const Device UA662389 = {S9472, 118, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA681849 = {S9473, 118, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ar) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA701158 = {S9474, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sr-cs) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA704779 = {S9475, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; mt-mt) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA706638 = {S9476, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-au) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA734031 = {S9477, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ka-ge) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA734044 = {S9478, 124, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; la) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA736487 = {S9479, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ko-kr) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA737574 = {S9480, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; af-za) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA767945 = {S9481, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-ch) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA768687 = {S9482, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-be) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA777487 = {S9483, 124, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; yi) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA789376 = {S9484, 124, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; id) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA805037 = {S9485, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; af-za) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA823373 = {S9486, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ca-es) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA837961 = {S9487, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lt-lt) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA838341 = {S9488, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hu-hu) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA863776 = {S9489, 124, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA867609 = {S9490, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; uk-ua) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA889564 = {S9491, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sl-si) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA892706 = {S9492, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; vi-vn) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA895634 = {S9493, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ro-ro) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA899395 = {S9494, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ro-ro) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA913522 = {S9495, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; et-ee) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA1836388 = {S9496, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ca) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA1840548 = {S9497, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fa-ir) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA1845267 = {S9498, 124, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; id) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA1851370 = {S9499, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; et-ee) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA1860363 = {S9500, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-ch) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA1860415 = {S9501, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hy-am) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA1861755 = {S9502, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; mk-mk) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA1865071 = {S9503, 124, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tl) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2131205 = {S9504, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; is-is) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2131667 = {S9505, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ca-es) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA2134591 = {S9506, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ka-ge) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2207118 = {S9507, 124, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ar) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA2207757 = {S9508, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ro-ro) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2214872 = {S9509, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.6 Safari/533.22.3*/
const Device UA2258250 = {S9510, 129, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hr-hr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA2263027 = {S9511, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; vi-vn) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2266671 = {S9512, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; bg-bg) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2268391 = {S9513, 127, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; cs-cz) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA2269950 = {S9514, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lt-lt) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2276412 = {S9515, 121, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-ch) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2284764 = {S9516, 124, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; id) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2286994 = {S9517, 124, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; my) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2474920 = {S9518, 124, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ru) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2644228 = {S9519, 119, { &P17595,&P17295,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10\_5\_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/5.0.6 Safari/533.22.3*/
const Device UA24165 = {S9520, 142, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42324 = {S9521, 161, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; SAMSUNG GT-S5830/S5830BUKP6 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42338 = {S9522, 161, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; SAMSUNG GT-S5830/S5830BUKPD Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42361 = {S9523, 161, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SAMSUNG GT-S5830/S5830BUKPE Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42368 = {S9524, 160, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; GT-S5830-ORANGE/S5830BVKQ7 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48158 = {S9525, 143, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; GT-S5830B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49416 = {S9526, 143, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; GT-S5830L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58241 = {S9527, 161, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-nz; SAMSUNG GT-S5830/S5830BUKP5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58262 = {S9528, 142, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70292 = {S9529, 160, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5830-ORANGE/S5830BVKQ8 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76098 = {S9530, 150, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; GT-S5830D-parrot Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78479 = {S9531, 142, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79545 = {S9532, 162, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; SAMSUNG GT-S5830/S5830BUKPDT Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79572 = {S9533, 160, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5830-ORANGE/S5830BVKQA Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA211788 = {S9534, 66, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-S5830 Build/GINGERBREAD)*/
const Device UA235662 = {S9535, 66, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5830 Build/GINGERBREAD)*/
const Device UA240426 = {S9536, 143, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; GT-S5830D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA251918 = {S9537, 161, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; SAMSUNG GT-S5830/S5830BUKS1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA351559 = {S9538, 161, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; SAMSUNG GT-S5830/S5830BUKS2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA354673 = {S9539, 142, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA356599 = {S9540, 143, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; GT-S5830L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369598 = {S9541, 125, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5830 Build/GRWK74) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA413015 = {S9542, 160, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; GT-S5830-ORANGE/S5830BVKQ9 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627260 = {S9543, 160, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830-ORANGE/S5830BVKT4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627275 = {S9544, 66, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-S5830 Build/GINGERBREAD)*/
const Device UA627674 = {S9545, 143, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; GT-S5830T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA630524 = {S9546, 160, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-S5830-ORANGE/S5830BVKT3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA632308 = {S9547, 143, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-S5830M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA632553 = {S9548, 143, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-S5830C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA645503 = {S9549, 67, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5830L Build/GINGERBREAD)*/
const Device UA653512 = {S9550, 66, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5830 Build/GINGERBREAD)*/
const Device UA654354 = {S9551, 67, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5830D Build/GINGERBREAD)*/
const Device UA673419 = {S9552, 143, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; GT-S5830B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA683058 = {S9553, 143, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-S5830L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA693547 = {S9554, 136, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT-S5830 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA710869 = {S9555, 130, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA740845 = {S9556, 130, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-S5830 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA746383 = {S9557, 120, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; es-es; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA749686 = {S9558, 120, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA789452 = {S9559, 120, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; it-it; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA821756 = {S9560, 143, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; GT-S5830G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA855227 = {S9561, 120, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; de-de; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA903848 = {S9562, 135, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1903828 = {S9563, 120, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; pt-pt; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA1936846 = {S9564, 61, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; GT-S5830 Build/GRWK74)*/
const Device UA1937631 = {S9565, 67, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5830B Build/GINGERBREAD)*/
const Device UA1941790 = {S9566, 120, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; fr-fr; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA1973139 = {S9567, 67, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5830T Build/GINGERBREAD)*/
const Device UA1988473 = {S9568, 67, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5830M Build/GINGERBREAD)*/
const Device UA2019450 = {S9569, 136, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; GT-S5830 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021659 = {S9570, 168, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en_gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)*/
const Device UA2023804 = {S9571, 135, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2024353 = {S9572, 130, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2025911 = {S9573, 160, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT-S5830 Build/GRWK74; Hidric by Fernando F.) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026061 = {S9574, 136, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; he-il; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027455 = {S9575, 67, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-S5830L Build/GINGERBREAD)*/
const Device UA2027511 = {S9576, 154, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830 Build/OMEGA ROM by TeamCooper) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029982 = {S9577, 67, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5830L Build/GINGERBREAD)*/
const Device UA2030611 = {S9578, 67, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5830C Build/GINGERBREAD)*/
const Device UA2030870 = {S9579, 143, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-S5830B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031597 = {S9580, 154, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830 Build/Beats With Smooth Sense) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2176708 = {S9581, 143, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-S5830V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2256920 = {S9582, 140, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257104 = {S9583, 140, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257377 = {S9584, 140, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2258079 = {S9585, 120, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; nl-nl; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2362659 = {S9586, 161, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-S5830/S5830BUKT2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2364138 = {S9587, 140, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2385791 = {S9588, 143, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-S5830T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2403784 = {S9589, 160, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-S5830-ORANGE/S5830BVKT5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2413001 = {S9590, 130, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-S5830 Build/GRWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2414214 = {S9591, 137, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-S5830 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2423881 = {S9592, 142, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2463776 = {S9593, 135, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2474078 = {S9594, 120, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; de-at; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2496901 = {S9595, 141, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5830M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2518095 = {S9596, 136, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT-S5830 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2650034 = {S9597, 67, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5830V Build/GINGERBREAD)*/
const Device UA2693646 = {S9598, 141, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; GT-S5830T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2694881 = {S9599, 136, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830 Build/CCGH1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2751001 = {S9600, 160, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-S5830-ORANGE/S5830BVKT6 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2751420 = {S9601, 159, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5830 Build/The Ultimatum; The Ultimatum) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2765870 = {S9602, 150, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5830 Build/luoshaohao; IPHONE5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2786591 = {S9603, 67, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5830T Build/GINGERBREAD)*/
const Device UA2821349 = {S9604, 141, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; GT-S5830L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2836955 = {S9605, 142, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; GT-S5830 Build/S5830F1V102) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2842895 = {S9606, 201, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT-S5830 Build/Think Android Final by galaxy team; Think Android Final by galaxy team) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2889157 = {S9607, 142, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; GT-S5830 Build/S5830F1V100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2933683 = {S9608, 162, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5830 Build/love cry-A4; love cry-A4(?r?cH)) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2979395 = {S9609, 67, { &P14186,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5830G Build/GINGERBREAD)*/
const Device UA900931 = {S9610, 111, { &P17595,&P17297,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.15 (KHTML, like Gecko) Version/6.0 Safari/536.16*/
const Device UA905280 = {S9611, 111, { &P17595,&P17297,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.19 (KHTML, like Gecko) Version/6.0 Safari/536.19*/
const Device UA924456 = {S9612, 111, { &P17595,&P17297,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.25 (KHTML, like Gecko) Version/6.0 Safari/536.25*/
const Device UA925435 = {S9613, 118, { &P17595,&P17297,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) WebClip/7536.24 Safari/7536.25*/
const Device UA1997241 = {S9614, 115, { &P17595,&P17297,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) Version/6.0 Safari/534.56.5*/
const Device UA2258401 = {S9615, 123, { &P17595,&P17297,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) WebClip/7536.26.7 Safari/7536.26.17*/
const Device UA2258402 = {S9616, 119, { &P17595,&P17297,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/536.26.17 (KHTML, like Gecko) Version/6.0.2 Safari/536.26.17*/
const Device UA2387029 = {S9617, 117, { &P17595,&P17297,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/6.0.2 Safari/534.57.7*/
const Device UA2425268 = {S9618, 144, { &P17595,&P17297,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7.5) AppleWebKit/7534.57.7 (KHTML, like Gecko) Version/6.0.2 Safari/7536.26.17 Raven for Mac/0.7.14665*/
const Device UA2567817 = {S9619, 119, { &P17595,&P17297,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/536.28.10 (KHTML, like Gecko) Version/6.0.3 Safari/536.28.10*/
const Device UA2584765 = {S9620, 123, { &P17595,&P17297,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) WebClip/7536.28.3 Safari/7536.28.10*/
const Device UA2679317 = {S9621, 119, { &P17595,&P17297,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/536.29.13 (KHTML, like Gecko) Version/6.0.4 Safari/536.29.13*/
const Device UA2680707 = {S9622, 123, { &P17595,&P17297,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) WebClip/7536.29.3 Safari/7536.29.13*/
const Device UA2767904 = {S9623, 117, { &P17595,&P17297,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/536.30.1 (KHTML, like Gecko) Version/6.0.5 Safari/536.30.1*/
const Device UA2783265 = {S9624, 120, { &P17595,&P17297,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) WebClip/7536.30 Safari/7536.30.1*/
const Device UA3039541 = {S9625, 106, { &P15364,&P16110,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML,like Gecko) Chrome/29.0.1547.2 Safari/537.36*/
const Device UA3064404 = {S9626, 111, { &P15364,&P16110,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe)*/
const Device UA3065732 = {S9627, 147, { &P15364,&P16110,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3067629 = {S9628, 108, { &P15364,&P16110,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36*/
const Device UA3067740 = {S9629, 101, { &P15364,&P16110,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36*/
const Device UA3067952 = {S9630, 118, { &P15364,&P16110,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe)*/
const Device UA3082197 = {S9631, 124, { &P15364,&P16110,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/29.0.1600.1 Chrome/29.0.1600.1 Safari/537.36*/
const Device UA3082314 = {S9632, 128, { &P15364,&P16110,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe),gzip(gfe)*/
const Device UA3093893 = {S9633, 140, { &P15364,&P16110,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3094647 = {S9634, 117, { &P15364,&P16110,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Iron/29.0.1600.1 Chrome/29.0.1600.1 Safari/537.36*/
const Device UA53295 = {S9635, 77, { &P14523,&P5156,&P3519,&P18092 }};	/*Useragent - BlackBerry9800/6.0.0.600 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102*/
const Device UA60291 = {S9636, 129, { &P14523,&P5156,&P3519,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9800; en-US) AppleWebKit/534.8+ (KHTML, like Gecko) Version/6.0.0.600 Mobile Safari/534.8+*/
const Device UA2752738 = {S9637, 106, { &P15364,&P16110,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML like Gecko) Chrome/28.0.1469.0 Safari/537.36*/
const Device UA3024590 = {S9638, 128, { &P15364,&P16110,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe),gzip(gfe)*/
const Device UA3024742 = {S9639, 108, { &P15364,&P16110,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3024749 = {S9640, 101, { &P15364,&P16110,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3024880 = {S9641, 140, { &P15364,&P16110,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3028346 = {S9642, 118, { &P15364,&P16110,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe)*/
const Device UA3028930 = {S9643, 104, { &P15364,&P16110,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.1.0.0 Safari/537.36*/
const Device UA3029604 = {S9644, 97, { &P15364,&P16110,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.1.0.0 Safari/537.36*/
const Device UA3029651 = {S9645, 147, { &P15364,&P16110,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3029787 = {S9646, 111, { &P15364,&P16110,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe)*/
const Device UA2740608 = {S9647, 119, { &P17595,&P18207,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36*/
const Device UA2755513 = {S9648, 158, { &P17595,&P18207,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2756582 = {S9649, 129, { &P17595,&P18207,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36,gzip(gfe)*/
const Device UA2769172 = {S9650, 129, { &P17595,&P18207,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36,gzip(gfe)*/
const Device UA2871609 = {S9651, 135, { &P17595,&P18207,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36*/
const Device UA2873601 = {S9652, 120, { &P17595,&P18207,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2873702 = {S9653, 120, { &P17595,&P18207,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2873706 = {S9654, 120, { &P17595,&P18207,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2873924 = {S9655, 120, { &P17595,&P18207,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2876739 = {S9656, 130, { &P17595,&P18207,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe)*/
const Device UA2877508 = {S9657, 120, { &P17595,&P18207,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2885960 = {S9658, 159, { &P17595,&P18207,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2967542 = {S9659, 135, { &P17595,&P18207,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36*/
const Device UA3090145 = {S9660, 135, { &P17595,&P18207,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36*/
const Device UA3103775 = {S9661, 135, { &P17595,&P18207,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36*/
const Device UA3020112 = {S9662, 179, { &P15364,&P5690,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36 AppEngine-Google; (+http://code.google.com/appengine; appid: s~romantic-kimi)*/
const Device UA3024625 = {S9663, 140, { &P15364,&P5690,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3024741 = {S9664, 101, { &P15364,&P5690,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3024962 = {S9665, 108, { &P15364,&P5690,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3025368 = {S9666, 101, { &P15364,&P5690,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3025704 = {S9667, 111, { &P15364,&P5690,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe)*/
const Device UA3027259 = {S9668, 121, { &P15364,&P5690,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe),gzip(gfe)*/
const Device UA3028482 = {S9669, 97, { &P15364,&P5690,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.1.0.0 Safari/537.36*/
const Device UA3028772 = {S9670, 108, { &P15364,&P5690,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3030024 = {S9671, 104, { &P15364,&P5690,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.1.0.0 Safari/537.36*/
const Device UA3079068 = {S9672, 97, { &P15364,&P5690,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.1.0.0 Safari/537.36*/
const Device UA2387454 = {S9673, 97, { &P15364,&P5690,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.1.0.0 Safari/537.11*/
const Device UA2408291 = {S9674, 101, { &P15364,&P5690,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2408346 = {S9675, 101, { &P15364,&P5690,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2408364 = {S9676, 108, { &P15364,&P5690,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2408375 = {S9677, 111, { &P15364,&P5690,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe)*/
const Device UA2427005 = {S9678, 100, { &P15364,&P5690,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2428541 = {S9679, 97, { &P15364,&P5690,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.4.0.0 Safari/537.11*/
const Device UA2430980 = {S9680, 108, { &P15364,&P5690,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2435082 = {S9681, 140, { &P15364,&P5690,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2436332 = {S9682, 97, { &P15364,&P5690,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.2.0.0 Safari/537.11*/
const Device UA2437177 = {S9683, 97, { &P15364,&P5690,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.3.0.0 Safari/537.11*/
const Device UA2461205 = {S9684, 104, { &P15364,&P5690,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.4.0.0 Safari/537.11*/
const Device UA2486606 = {S9685, 251, { &P15364,&P5690,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11 AppEngine-Google; (+http://code.google.com/appengine; appid: s~youniyimo1) AppEngine-Google; (+http://code.google.com/appengine; appid: s~youniyimo0)*/
const Device UA2507430 = {S9686, 251, { &P15364,&P5690,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11 AppEngine-Google; (+http://code.google.com/appengine; appid: s~youniyimo1) AppEngine-Google; (+http://code.google.com/appengine; appid: s~youniyimo2)*/
const Device UA2532923 = {S9687, 251, { &P15364,&P5690,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11 AppEngine-Google; (+http://code.google.com/appengine; appid: s~youniyimo1) AppEngine-Google; (+http://code.google.com/appengine; appid: s~youniyimo1)*/
const Device UA2540594 = {S9688, 129, { &P15364,&P5690,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; ; NCLIENT50_AAP7150B86519C) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11*/
const Device UA2557207 = {S9689, 251, { &P15364,&P5690,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11 AppEngine-Google; (+http://code.google.com/appengine; appid: s~youniyimo1) AppEngine-Google; (+http://code.google.com/appengine; appid: s~youniyimo3)*/
const Device UA2847844 = {S9690, 97, { &P15364,&P5690,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.4.0.0 Safari/537.11*/
const Device UA2849740 = {S9691, 123, { &P15364,&P5690,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko GomezAgent 3.0) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2859864 = {S9692, 251, { &P15364,&P5690,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11 AppEngine-Google; (+http://code.google.com/appengine; appid: s~youniyimo1) AppEngine-Google; (+http://code.google.com/appengine; appid: s~youniyimo4)*/
const Device UA1994658 = {S9693, 123, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1207.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/22.0.1207.1 Safari/537.1*/
const Device UA1997673 = {S9694, 123, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1221.0) AppleWebKit/537.3 (KHTML, like Gecko) Chrome/22.0.1221.0 Safari/537.3*/
const Device UA2014095 = {S9695, 125, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1229.14) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.14 Safari/537.4*/
const Device UA2040420 = {S9696, 125, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1229.26) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.26 Safari/537.4*/
const Device UA2042910 = {S9697, 125, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1229.56) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.56 Safari/537.4*/
const Device UA2193785 = {S9698, 125, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1229.79) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4*/
const Device UA2196369 = {S9699, 125, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/22.0.1229.79) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4*/
const Device UA2199787 = {S9700, 125, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1229.91) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.91 Safari/537.4*/
const Device UA2200996 = {S9701, 125, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1229.64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.64 Safari/537.4*/
const Device UA2201630 = {S9702, 125, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1229.92) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.92 Safari/537.4*/
const Device UA2203778 = {S9703, 192, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/22.0.1229.94; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA2203785 = {S9704, 125, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1229.94) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203789 = {S9705, 159, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/22.0.1229.94; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2203794 = {S9706, 81, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/22.0.1229.94)*/
const Device UA2232103 = {S9707, 76, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/22.0.1229.94)*/
const Device UA2236195 = {S9708, 125, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1229.96) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.96 Safari/537.4*/
const Device UA2237984 = {S9709, 125, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/22.0.1229.94) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2238027 = {S9710, 132, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/22.0.1229.94) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2238112 = {S9711, 220, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/22.0.1229.94; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2238120 = {S9712, 210, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; chromeframe/22.0.1229.94; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8; .NET4.0C; .NET4.0E)*/
const Device UA2238209 = {S9713, 206, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/22.0.1229.94; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2238218 = {S9714, 238, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/22.0.1229.94; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2238231 = {S9715, 143, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; GTB7.4; chromeframe/22.0.1229.94; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)*/
const Device UA2238309 = {S9716, 140, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/22.0.1229.94; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)*/
const Device UA2249455 = {S9717, 182, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/22.0.1229.94; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)*/
const Device UA2253591 = {S9718, 180, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/22.0.1229.94; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA2350574 = {S9719, 132, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/22.0.1229.79) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4*/
const Device UA2354037 = {S9720, 278, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/22.0.1229.94; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2466144 = {S9721, 204, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/22.0.1229.94; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.1; .NET4.0C; .NET4.0E)*/
const Device UA2526926 = {S9722, 125, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/22.0.1229.95) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.95 Safari/537.4*/
const Device UA2578455 = {S9723, 179, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/22.0.1229.96; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA2695874 = {S9724, 125, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/22.0.1229.96) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.96 Safari/537.4*/
const Device UA2705949 = {S9725, 257, { &P15364,&P5690,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; chromeframe/22.0.1229.94; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2232879 = {S9726, 128, { &P15259,&P7106,&P15829,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9900; en) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.746 Mobile Safari/534.11+*/
const Device UA2362409 = {S9727, 77, { &P15259,&P7106,&P15829,&P18092 }};	/*Useragent - BlackBerry9900/7.1.0.694 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102*/
const Device UA913446 = {S9728, 126, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A5355d Safari/8536.25*/
const Device UA2012811 = {S9729, 126, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A5376e Safari/8536.25*/
const Device UA2042349 = {S9730, 124, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A403 Safari/8536.25*/
const Device UA2178044 = {S9731, 124, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A402 Safari/8536.25*/
const Device UA2202433 = {S9732, 123, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A403Safari/8536.25*/
const Device UA2203031 = {S9733, 123, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A403 Safari/8536.25*/
const Device UA2260597 = {S9734, 124, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A407 Safari/8536.25*/
const Device UA2260616 = {S9735, 124, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A406 Safari/8536.25*/
const Device UA2260650 = {S9736, 126, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2260669 = {S9737, 127, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A8424 Safari/8536.25*/
const Device UA2347220 = {S9738, 124, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A396 Safari/8536.25*/
const Device UA2356959 = {S9739, 127, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A8426 Safari/8536.25*/
const Device UA2360621 = {S9740, 125, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523Safari/8536.25*/
const Device UA2362282 = {S9741, 127, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A8425 Safari/8536.25*/
const Device UA2363433 = {S9742, 125, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2426291 = {S9743, 127, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A8421 Safari/8536.25*/
const Device UA2435908 = {S9744, 127, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A8500 Safari/8536.25*/
const Device UA2437561 = {S9745, 126, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A550 Safari/8536.25*/
const Device UA2466741 = {S9746, 125, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/6.0 Mobile/9B176 Safari/7534.48.3*/
const Device UA2693704 = {S9747, 125, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A8418 Safari/8536.25*/
const Device UA2787399 = {S9748, 124, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A391 Safari/8536.25*/
const Device UA2794403 = {S9749, 126, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146 Safari/8536.25*/
const Device UA2834999 = {S9750, 125, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A8416 Safari/8536.25*/
const Device UA2839510 = {S9751, 125, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A8406 Safari/8536.25*/
const Device UA2849675 = {S9752, 140, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0(iPad; U; CPU iPhone OS 60 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML like Gecko) Version/6.0 Mobile/10A5355d Safari/8536.25*/
const Device UA2987914 = {S9753, 126, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA2987967 = {S9754, 124, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA3042952 = {S9755, 126, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A520 Safari/8536.25*/
const Device UA3045350 = {S9756, 125, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/9Z999 Safari/8536.25*/
const Device UA3072565 = {S9757, 125, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/9Z999 Safari/8536.25*/
const Device UA3079914 = {S9758, 126, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A406 Safari/8536.25*/
const Device UA3098839 = {S9759, 127, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A8423 Safari/8536.25*/
const Device UA3108770 = {S9760, 126, { &P15767,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A406 Safari/8536.25*/
const Device UA2239304 = {S9761, 132, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/24.0.1297.0) AppleWebKit/537.16 (KHTML, like Gecko) Chrome/24.0.1297.0 Safari/537.16*/
const Device UA2353741 = {S9762, 132, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/24.0.1290.1) AppleWebKit/537.13 (KHTML, like Gecko) Chrome/24.0.1290.1 Safari/537.13*/
const Device UA2359339 = {S9763, 132, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/24.0.1312.5) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.5 Safari/537.17*/
const Device UA2361127 = {S9764, 125, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/24.0.1312.5) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.5 Safari/537.17*/
const Device UA2363187 = {S9765, 134, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/24.0.1312.14) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.14 Safari/537.17*/
const Device UA2368869 = {S9766, 127, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/24.0.1312.14) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.14 Safari/537.17*/
const Device UA2390991 = {S9767, 134, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/24.0.1312.27) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.27 Safari/537.17*/
const Device UA2408110 = {S9768, 134, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/24.0.1312.35) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.35 Safari/537.17*/
const Device UA2435718 = {S9769, 134, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/24.0.1312.40) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.40 Safari/537.17*/
const Device UA2436054 = {S9770, 134, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/24.0.1312.45) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.45 Safari/537.17*/
const Device UA2436737 = {S9771, 127, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/24.0.1312.45) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.45 Safari/537.17*/
const Device UA2460307 = {S9772, 127, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/24.0.1312.40) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.40 Safari/537.17*/
const Device UA2461957 = {S9773, 134, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/24.0.1312.52) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17*/
const Device UA2462194 = {S9774, 127, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/24.0.1312.52) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17*/
const Device UA2487801 = {S9775, 194, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; chromeframe/24.0.1312.57)*/
const Device UA2488064 = {S9776, 134, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/24.0.1312.56) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17*/
const Device UA2488202 = {S9777, 127, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/24.0.1312.56) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17*/
const Device UA2488807 = {S9778, 90, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/24.0.1312.57)*/
const Device UA2492180 = {S9779, 89, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/24.0.1312.57)*/
const Device UA2492190 = {S9780, 183, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/24.0.1312.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2492232 = {S9781, 96, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/24.0.1312.57)*/
const Device UA2492234 = {S9782, 97, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/24.0.1312.57)*/
const Device UA2492236 = {S9783, 214, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/24.0.1312.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Tablet PC 2.0)*/
const Device UA2492300 = {S9784, 199, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; chromeframe/24.0.1312.57)*/
const Device UA2492303 = {S9785, 216, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/24.0.1312.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E)*/
const Device UA2492329 = {S9786, 127, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/24.0.1312.57) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492376 = {S9787, 134, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/24.0.1312.57) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492464 = {S9788, 197, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/24.0.1312.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2510273 = {S9789, 209, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/24.0.1312.57; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2510290 = {S9790, 214, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/24.0.1312.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Tablet PC 2.0)*/
const Device UA2513196 = {S9791, 156, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/24.0.1312.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729)*/
const Device UA2557813 = {S9792, 183, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/24.0.1312.56; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2659515 = {S9793, 171, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/24.0.1312.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Tablet PC 2.0)*/
const Device UA2748914 = {S9794, 156, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/24.0.1312.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729)*/
const Device UA2779590 = {S9795, 270, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/24.0.1312.57;  SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2952720 = {S9796, 171, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/24.0.1312.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Tablet PC 2.0)*/
const Device UA2956853 = {S9797, 209, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/24.0.1312.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA3088137 = {S9798, 224, { &P15364,&P7133,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/24.0.1312.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Tablet PC 2.0; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2567775 = {S9799, 97, { &P15364,&P5690,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.2.0.0 Safari/537.22*/
const Device UA2567796 = {S9800, 102, { &P15364,&P5690,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2567865 = {S9801, 141, { &P15364,&P5690,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2567874 = {S9802, 109, { &P15364,&P5690,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2567881 = {S9803, 102, { &P15364,&P5690,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2568378 = {S9804, 112, { &P15364,&P5690,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)*/
const Device UA2569776 = {S9805, 97, { &P15364,&P5690,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.1.0.0 Safari/537.22*/
const Device UA2584646 = {S9806, 109, { &P15364,&P5690,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2605850 = {S9807, 104, { &P15364,&P5690,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.2.0.0 Safari/537.22*/
const Device UA2891466 = {S9808, 101, { &P15364,&P5690,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA3060213 = {S9809, 147, { &P15364,&P5690,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.7.1364.21027 Safari/537.22,gzip(gfe),gzip(gfe)*/
const Device UA3108741 = {S9810, 139, { &P15364,&P5690,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.7.1364.21027 (gnctrkcll) Safari/537.22*/
const Device UA3120871 = {S9811, 179, { &P15364,&P5690,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22 AppEngine-Google; (+http://code.google.com/appengine; appid: s~atmshowing01)*/
const Device UA3125125 = {S9812, 164, { &P15364,&P5690,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22 BitdefenderSafepay/17.14.0.0 (OBK; 25.0.1364.172; r1194.2013)*/
const Device UA37121 = {S9813, 107, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A5313e*/
const Device UA37141 = {S9814, 107, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A5274d*/
const Device UA37174 = {S9815, 107, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A5288d*/
const Device UA46471 = {S9816, 92, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)*/
const Device UA70722 = {S9817, 105, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334*/
const Device UA90980 = {S9818, 94, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)*/
const Device UA90981 = {S9819, 107, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405*/
const Device UA91000 = {S9820, 107, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A404*/
const Device UA358604 = {S9821, 107, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406*/
const Device UA360481 = {S9822, 44, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Miu Ptt 1.5.2 (iPhone; iPhone OS 5.0; zh_TW)*/
const Device UA575006 = {S9823, 153, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper*/
const Device UA575234 = {S9824, 134, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - NaverCafe/1.0 Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName*/
const Device UA616501 = {S9825, 106, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A40*/
const Device UA637320 = {S9826, 251, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,3;FBMD/iPhone;FBSN/iPhone OS;FBSV/5.0.1;FBSS/2; FBCR/Verizon;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA652612 = {S9827, 135, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 iPad2,1 BingWeb/3.02.1641.X*/
const Device UA691829 = {S9828, 155, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper*/
const Device UA692955 = {S9829, 244, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/5.0;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA725819 = {S9830, 136, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - NaverCafe/1.0 Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName*/
const Device UA792479 = {S9831, 107, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9Z999*/
const Device UA1935253 = {S9832, 101, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405*/
const Device UA2215059 = {S9833, 100, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A5302b*/
const Device UA2221249 = {S9834, 107, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176*/
const Device UA2304190 = {S9835, 166, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Safari/7534.48 Instapaper/4.0.1 (+http://www.instapaper.com/iphone)*/
const Device UA2395525 = {S9836, 166, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48 Instapaper/4.2.1 (+http://www.instapaper.com/iphone)*/
const Device UA2418787 = {S9837, 144, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone 5_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPhone3,1/5.0.1)*/
const Device UA2505723 = {S9838, 106, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523*/
const Device UA2520759 = {S9839, 125, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Safari/7534.48.3*/
const Device UA2597943 = {S9840, 46, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Miu Ptt 1.9.2 (iPhone; iPhone OS 5.0.1; zh_TW)*/
const Device UA2659318 = {S9841, 229, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 501 like Mac OS X) AppleWebKit/534.46 (KHTML like Gecko) Mobile/9A405 FBAN/FBIOS;FBAV/5.0;FBBV/40121;FBDV/iPhone31;FBMD/iPhone;FBSN/iPhone OS;FBSV/5.0.1;FBSS/2; FBCR/Sunrise;FBID/phone;FBLC/enUS*/
const Device UA2839104 = {S9842, 106, { &P12280,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403*/
const Device UA24754 = {S9843, 147, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_WildfireS_A510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA33155 = {S9844, 146, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - HTC_WildfireS_A510e/Mozilla/5.0(Linux; U; Android 2.3.3;it-it; Build/GRH78C) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43410 = {S9845, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-ch; HTC/WildfireS/1.33.166.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43418 = {S9846, 142, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_Wildfire_S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43433 = {S9847, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; el-gr; HTC/WildfireS/1.33.168.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43438 = {S9848, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.3; en-us; HTC_Wildfire_S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43439 = {S9849, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; HTC_Wildfire_S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43454 = {S9850, 142, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Wildfire S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43461 = {S9851, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;HTC_Wildfire_S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43499 = {S9852, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_Wildfire_S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA43505 = {S9853, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_Wildfire_SBuild/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59349 = {S9854, 148, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Wildfire S A510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59353 = {S9855, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC/WildfireS/1.33.161.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59354 = {S9856, 138, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_Marvel Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59364 = {S9857, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-at; HTC/WildfireS/1.33.169.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59367 = {S9858, 151, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_WildfireS-orange-LS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59394 = {S9859, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; HTC/WildfireS/1.33.163.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59465 = {S9860, 227, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTCA510e/1.0 Android/2.4 release/02.25.2011 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60873 = {S9861, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_WildfireS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70666 = {S9862, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; HTC/WildfireS/1.58.165.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72962 = {S9863, 146, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; el-gr; HTC_WildfireS_A510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA77048 = {S9864, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; HTC/WildfireS/1.64.192.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA193259 = {S9865, 72, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Wildfire S A510e Build/GRI40)*/
const Device UA251823 = {S9866, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; USCCADR6230US Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA347650 = {S9867, 137, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC-PG762 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA351685 = {S9868, 147, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_WildfireS_A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA352866 = {S9869, 148, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA352991 = {S9870, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/WildfireS/2.14.161.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA366344 = {S9871, 142, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Wildfire_S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368784 = {S9872, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Wildfire_S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA370010 = {S9873, 78, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.3; en_GB; HTC Wildfire S A510e Build/GRI40)*/
const Device UA371105 = {S9874, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-us; HTC Wildfire S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA380084 = {S9875, 142, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Wildfire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA421924 = {S9876, 136, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Wildfire S A510e Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA421983 = {S9877, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; HTC/WildfireS/2.14.165.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA422332 = {S9878, 151, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_WildfireS-orange-LS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA558034 = {S9879, 66, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Wildfire S Build/GRI40)*/
const Device UA582205 = {S9880, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-ch; HTC/WildfireS/2.14.166.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623290 = {S9881, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.5; en-us; HTC_Wildfire_S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA628411 = {S9882, 72, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Wildfire S A510e Build/GRJ90)*/
const Device UA632232 = {S9883, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_WildfireS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA641163 = {S9884, 66, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Wildfire S Build/GRJ90)*/
const Device UA643185 = {S9885, 147, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; HTC_WildfireS_A515c Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA663878 = {S9886, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC/WildfireS/2.14.163.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA664385 = {S9887, 125, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC A510e Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA673849 = {S9888, 61, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC-PG762 Build/GRJ22)*/
const Device UA673956 = {S9889, 136, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Wildfire S A510e Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA686835 = {S9890, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-at; HTC/WildfireS/2.14.169.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA705972 = {S9891, 145, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - HTC_WildfireS_A510e/Mozilla/5.0(Linux; U; Android 2.3.5;it-it;Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA731099 = {S9892, 137, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC A510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA737577 = {S9893, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us;HTC_Wildfire_S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA751952 = {S9894, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; el-gr; HTC/WildfireS/2.14.168.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA760384 = {S9895, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/WildfireS/1.33.161.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA786785 = {S9896, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.5; en-us; HTC_Wildfire_S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA806493 = {S9897, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Wildfire_SBuild/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA817141 = {S9898, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC/WildfireS/2.14.192.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA819433 = {S9899, 227, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTCA510e/1.0 Android/2.4 release/02.25.2011 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA848013 = {S9900, 125, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC A510e Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA897990 = {S9901, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/WildfireS/2.26.161.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA898830 = {S9902, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-at; HTC/WildfireS/2.26.169.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA898932 = {S9903, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC/WildfireS/2.26.163.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA899020 = {S9904, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; HTC/WildfireS/2.26.165.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA911440 = {S9905, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/WildfireS/2.26.161.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1923163 = {S9906, 148, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; HTC Wildfire S A515c Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1933452 = {S9907, 65, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; USCCADR6230US Build/GRJ22)*/
const Device UA1950388 = {S9908, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-ch; HTC/WildfireS/2.26.166.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1963658 = {S9909, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC/WildfireS/2.26.192.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1966402 = {S9910, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Wildfire S A510e Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1974071 = {S9911, 130, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC A510e Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1979284 = {S9912, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; el-gr; HTC/WildfireS/2.26.168.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1991129 = {S9913, 137, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019853 = {S9914, 61, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC A510e Build/GRI40)*/
const Device UA2022632 = {S9915, 130, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Wildfire S Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2024560 = {S9916, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Wildfire SBuild/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025337 = {S9917, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC WildfireS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025908 = {S9918, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Wildfire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA2027764 = {S9919, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Wildfire S A510e Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2030784 = {S9920, 130, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC A510e Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2030859 = {S9921, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us;HTC Wildfire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032037 = {S9922, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTCWildfire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034317 = {S9923, 66, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Wildfire S Build/GRI40)*/
const Device UA2362580 = {S9924, 152, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; nl-nl; HTC/WildfireS/2.26.161.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2495274 = {S9925, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.5; en-us; HTC Wildfire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2590520 = {S9926, 141, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Wildfire S A510e Build GRI40) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2748125 = {S9927, 153, { &P12871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Wildfire S A510e Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3115228 = {S9928, 125, { &P15364,&P7133,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Maxthon/4.2.0.800 Chrome/30.0.1551.0 Safari/537.36*/
const Device UA3115654 = {S9929, 108, { &P15364,&P7133,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3115655 = {S9930, 101, { &P15364,&P7133,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3128351 = {S9931, 147, { &P15364,&P7133,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3129791 = {S9932, 118, { &P15364,&P7133,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Maxthon/4.2.0.800 Chrome/30.0.1551.0 Safari/537.36*/
const Device UA3130084 = {S9933, 118, { &P15364,&P7133,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36,gzip(gfe)*/
const Device UA3130914 = {S9934, 111, { &P15364,&P7133,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36,gzip(gfe)*/
const Device UA28065 = {S9935, 88, { &P15364,&P5690,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.1.6) Gecko/20091201 Firefox/3.5.6*/
const Device UA478147 = {S9936, 81, { &P15364,&P5690,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; ru; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.50*/
const Device UA551914 = {S9937, 94, { &P15364,&P5690,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; Firefox/4.5; en; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.53*/
const Device UA589737 = {S9938, 101, { &P15364,&P5690,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.10) Gecko/X Firefox/3.5.10 WinTSI 21.04.2010*/
const Device UA590007 = {S9939, 102, { &P15364,&P5690,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.8) Gecko/X Firefox/3.5.8 WinNT-EVI 23.05.2010*/
const Device UA603068 = {S9940, 84, { &P15364,&P5690,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; en-US; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.70*/
const Device UA656827 = {S9941, 120, { &P15364,&P5690,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.5) Gecko/X Firefox/3.5.5 (.NET CLR 3.5.30729) WinTSI 30.01.2010*/
const Device UA713396 = {S9942, 94, { &P15364,&P5690,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; Firefox/3.5; en; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.53*/
const Device UA743732 = {S9943, 81, { &P15364,&P5690,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.63*/
const Device UA780774 = {S9944, 94, { &P15364,&P5690,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; Firefox/5.0; en; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.53*/
const Device UA791968 = {S9945, 81, { &P15364,&P5690,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 11.10*/
const Device UA809941 = {S9946, 70, { &P15364,&P5690,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; zh-cn; rv:1.8.1) Gecko/X Firefox/3.5.5*/
const Device UA823615 = {S9947, 84, { &P15364,&P5690,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; zh-cn; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.53*/
const Device UA890908 = {S9948, 81, { &P15364,&P5690,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; ru; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.60*/
const Device UA895980 = {S9949, 73, { &P15364,&P5690,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN; rv:1.9.1.2) Firefox/3.5.2*/
const Device UA2207376 = {S9950, 81, { &P15364,&P5690,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; ru; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.52*/
const Device UA2331957 = {S9951, 102, { &P15364,&P5690,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.8) Gecko/X Firefox/3.5.8 WinNT-EVU 13.01.2010*/
const Device UA2471955 = {S9952, 97, { &P15364,&P5690,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.1.0.0 Safari/537.17*/
const Device UA2488553 = {S9953, 140, { &P15364,&P5690,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2492149 = {S9954, 101, { &P15364,&P5690,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492187 = {S9955, 108, { &P15364,&P5690,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492272 = {S9956, 97, { &P15364,&P5690,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.2.0.0 Safari/537.17*/
const Device UA2492351 = {S9957, 101, { &P15364,&P5690,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492372 = {S9958, 111, { &P15364,&P5690,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)*/
const Device UA2510341 = {S9959, 104, { &P15364,&P5690,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.2.0.0 Safari/537.17*/
const Device UA2510497 = {S9960, 108, { &P15364,&P5690,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2540599 = {S9961, 129, { &P15364,&P5690,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; ; NCLIENT50_AAP7150B86519C) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17*/
const Device UA2559569 = {S9962, 100, { &P15364,&P5690,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML like Gecko) Chrome/24.0.1312.52 Safari/537.17*/
const Device UA2767450 = {S9963, 110, { &P15364,&P5690,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.10.1312.57 Safari/537.17 QIPSurf*/
const Device UA2850600 = {S9964, 107, { &P15364,&P5690,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.2.0.0 Safari/537.17,gzip(gfe)*/
const Device UA2852983 = {S9965, 111, { &P15364,&P5690,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)*/
const Device UA2012036 = {S9966, 135, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A5376e Safari/8536.25*/
const Device UA2012809 = {S9967, 133, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A383 Safari/8536.25*/
const Device UA2034642 = {S9968, 133, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A402 Safari/8536.25*/
const Device UA2034643 = {S9969, 133, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A403 Safari/8536.25*/
const Device UA2040848 = {S9970, 133, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A405 Safari/8536.25*/
const Device UA2201601 = {S9971, 132, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A403 Safari/8536.25*/
const Device UA2201792 = {S9972, 164, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10A403 Version/6.0 Safari/8536.25 Safari/6533.18.5*/
const Device UA2201942 = {S9973, 132, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A403Safari/8536.25*/
const Device UA2237046 = {S9974, 164, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10A405 Version/6.0 Safari/8536.25 Safari/6533.18.5*/
const Device UA2237417 = {S9975, 135, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A505 Safari/8536.25*/
const Device UA2261058 = {S9976, 135, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2261167 = {S9977, 135, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A525 Safari/8536.25*/
const Device UA2341377 = {S9978, 133, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A400 Safari/8536.25*/
const Device UA2341471 = {S9979, 133, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A396 Safari/8536.25*/
const Device UA2351831 = {S9980, 133, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A399 Safari/8536.25*/
const Device UA2356728 = {S9981, 134, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2356937 = {S9982, 134, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523Safari/8536.25*/
const Device UA2357077 = {S9983, 166, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10A523 Version/6.0 Safari/8536.25 Safari/6533.18.5*/
const Device UA2386639 = {S9984, 166, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10A525 Version/6.0 Safari/8536.25 Safari/6533.18.5*/
const Device UA2437557 = {S9985, 135, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A551 Safari/8536.25*/
const Device UA2456492 = {S9986, 134, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A525 Safari/8536.25*/
const Device UA2457213 = {S9987, 134, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A525Safari/8536.25*/
const Device UA2460879 = {S9988, 166, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10A551 Version/6.0 Safari/8536.25 Safari/6533.18.5*/
const Device UA2505647 = {S9989, 134, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A551 Safari/8536.25*/
const Device UA2507161 = {S9990, 134, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A551Safari/8536.25*/
const Device UA2525660 = {S9991, 128, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A5355d Safari/8536.25*/
const Device UA2528532 = {S9992, 133, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2538627 = {S9993, 135, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2595572 = {S9994, 134, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/9Z999 Safari/8536.25*/
const Device UA2653134 = {S9995, 135, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A524 Safari/8536.25*/
const Device UA2746852 = {S9996, 134, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/6.0 Mobile/9B176 Safari/7534.48.3*/
const Device UA2779429 = {S9997, 133, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B142 Safari/8536.25*/
const Device UA2779735 = {S9998, 135, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A551 Safari/8536.25*/
const Device UA2840908 = {S9999, 132, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/9Z999 Safari/8536.25*/
const Device UA2859540 = {S10000, 134, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/9Z999 Safari/8536.25*/
const Device UA2874248 = {S10001, 162, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/537.4 (KHTML, like Gecko; Google Page Speed Insights) Version/6.0 Mobile/10A525 Safari/8536.25*/
const Device UA2934848 = {S10002, 135, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA2952407 = {S10003, 135, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B143 Safari/8536.25*/
const Device UA2952633 = {S10004, 133, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA3042913 = {S10005, 132, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A405 Safari/8536.25*/
const Device UA3088004 = {S10006, 135, { &P12280,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329 Safari/8536.25*/
const Device UA3064751 = {S10007, 111, { &P15364,&P5690,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe)*/
const Device UA3064766 = {S10008, 108, { &P15364,&P5690,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36*/
const Device UA3065300 = {S10009, 140, { &P15364,&P5690,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3067631 = {S10010, 101, { &P15364,&P5690,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36*/
const Device UA3067806 = {S10011, 101, { &P15364,&P5690,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36*/
const Device UA3068074 = {S10012, 108, { &P15364,&P5690,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36*/
const Device UA3068230 = {S10013, 121, { &P15364,&P5690,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe),gzip(gfe)*/
const Device UA3082193 = {S10014, 117, { &P15364,&P5690,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Iron/29.0.1600.1 Chrome/29.0.1600.1 Safari/537.36*/
const Device UA2367618 = {S10015, 132, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1323.1) AppleWebKit/537.19 (KHTML, like Gecko) Chrome/25.0.1323.1 Safari/537.19*/
const Device UA2461095 = {S10016, 132, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1364.5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.5 Safari/537.22*/
const Device UA2470714 = {S10017, 134, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1364.36) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.36 Safari/537.22*/
const Device UA2470741 = {S10018, 134, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1364.29) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.29 Safari/537.22*/
const Device UA2490170 = {S10019, 127, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/25.0.1364.36) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.36 Safari/537.22*/
const Device UA2491062 = {S10020, 134, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1364.45) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.45 Safari/537.22*/
const Device UA2509316 = {S10021, 134, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1364.58) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.58 Safari/537.22*/
const Device UA2510939 = {S10022, 127, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/25.0.1364.58) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.58 Safari/537.22*/
const Device UA2514379 = {S10023, 134, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1364.68) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.68 Safari/537.22*/
const Device UA2532531 = {S10024, 127, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/25.0.1364.84) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.84 Safari/537.22*/
const Device UA2532543 = {S10025, 134, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1364.84) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.84 Safari/537.22*/
const Device UA2538398 = {S10026, 176, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/25.0.1364.97; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2538580 = {S10027, 134, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1364.97) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22*/
const Device UA2538587 = {S10028, 199, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; chromeframe/25.0.1364.97)*/
const Device UA2538924 = {S10029, 127, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/25.0.1364.97) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22*/
const Device UA2567810 = {S10030, 97, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/25.0.1364.172)*/
const Device UA2567820 = {S10031, 215, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/25.0.1364.172; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Tablet PC 2.0)*/
const Device UA2567859 = {S10032, 90, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/25.0.1364.172)*/
const Device UA2567876 = {S10033, 98, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/25.0.1364.172)*/
const Device UA2567879 = {S10034, 136, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1364.172) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2567997 = {S10035, 129, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/25.0.1364.172) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2568054 = {S10036, 205, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/25.0.1364.172; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2568143 = {S10037, 91, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/25.0.1364.172)*/
const Device UA2568258 = {S10038, 184, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/25.0.1364.172; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2568856 = {S10039, 205, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; chromeframe/25.0.1364.172)*/
const Device UA2568961 = {S10040, 136, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1364.152) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22*/
const Device UA2569530 = {S10041, 129, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/25.0.1364.152) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22*/
const Device UA2571052 = {S10042, 136, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/25.0.1364.160) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.160 Safari/537.22*/
const Device UA2571271 = {S10043, 129, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/25.0.1364.160) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.160 Safari/537.22*/
const Device UA2583954 = {S10044, 197, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/25.0.1364.172; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322)*/
const Device UA2584377 = {S10045, 184, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/25.0.1364.172; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2584663 = {S10046, 192, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/25.0.1364.172; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA2605732 = {S10047, 215, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/25.0.1364.172; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Tablet PC 2.0)*/
const Device UA2711308 = {S10048, 222, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/25.0.1364.172; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3)*/
const Device UA2855586 = {S10049, 226, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/25.0.1364.97; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Zune 4.7)*/
const Device UA2989760 = {S10050, 271, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/25.0.1364.172;  SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA3019521 = {S10051, 202, { &P15364,&P7133,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; chromeframe/25.0.1364.172; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322)*/
const Device UA30673 = {S10052, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA30740 = {S10053, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sv-se) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA91315 = {S10054, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; it-it) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA92398 = {S10055, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA92404 = {S10056, 123, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18*/
const Device UA92748 = {S10057, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA92936 = {S10058, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sv-se) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA93219 = {S10059, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ja-jp) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA93330 = {S10060, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA93470 = {S10061, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA93522 = {S10062, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA93536 = {S10063, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ja-jp) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA93642 = {S10064, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA93970 = {S10065, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA94153 = {S10066, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-br) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA94211 = {S10067, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA94596 = {S10068, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-cn) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA94735 = {S10069, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ru-ru) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA95248 = {S10070, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; it-it) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA95461 = {S10071, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nb-no) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA95505 = {S10072, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA95514 = {S10073, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/528.16 (KHTML, like Gecko) Version/4.0 Safari/528.16*/
const Device UA95743 = {S10074, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; cs-cz) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA97397 = {S10075, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ja-jp) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA97470 = {S10076, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nb-no) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA97481 = {S10077, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ja-jp) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA97670 = {S10078, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ru-ru) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA98097 = {S10079, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-br) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA98098 = {S10080, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fi-fi) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA98087 = {S10081, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA98432 = {S10082, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-gb) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA100270 = {S10083, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA100831 = {S10084, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-cn) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA101959 = {S10085, 123, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; fr-fr) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18*/
const Device UA103453 = {S10086, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; fr-fr) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA103542 = {S10087, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-xl) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA103784 = {S10088, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-cn) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA103818 = {S10089, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA103979 = {S10090, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA108981 = {S10091, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA113217 = {S10092, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ru-ru) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA113291 = {S10093, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA114181 = {S10094, 124, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19.2*/
const Device UA117205 = {S10095, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sv-se) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA117341 = {S10096, 123, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; nl-nl) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18*/
const Device UA120020 = {S10097, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nb-no) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA122475 = {S10098, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA123828 = {S10099, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA125163 = {S10100, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-ca) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA125893 = {S10101, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ru-ru) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA126495 = {S10102, 131, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/528.16 (KHTML, like Gecko) Version/4 Public Beta Safari/528.16*/
const Device UA126518 = {S10103, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA127088 = {S10104, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-ca) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA127122 = {S10105, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA127124 = {S10106, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; it-it) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA128154 = {S10107, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-br) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA128542 = {S10108, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-ca) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA129092 = {S10109, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA129613 = {S10110, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-ca) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA130645 = {S10111, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA132081 = {S10112, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sv-se) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA133016 = {S10113, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-pt) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA133084 = {S10114, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; it-it) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA133609 = {S10115, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-gb) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA133902 = {S10116, 123, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; de-de) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18*/
const Device UA133926 = {S10117, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; de-de) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA134027 = {S10118, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-tw) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA134673 = {S10119, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tr-tr) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA135055 = {S10120, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fi-fi) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA135214 = {S10121, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-au) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA136876 = {S10122, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; da-dk) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA136926 = {S10123, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; it-it) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA136944 = {S10124, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fi-fi) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA137768 = {S10125, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; de-de) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA138065 = {S10126, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-au) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA138949 = {S10127, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/531.5 (KHTML, like Gecko) Version/4.0.3 Safari/531.7*/
const Device UA139257 = {S10128, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA140759 = {S10129, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pl-pl) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA141212 = {S10130, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; es-es) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA143565 = {S10131, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fi-fi) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA145648 = {S10132, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; sv-se) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA147540 = {S10133, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nb-no) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA148877 = {S10134, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0 Safari/528.17*/
const Device UA151364 = {S10135, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; th-th) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA158954 = {S10136, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; de-de) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA163100 = {S10137, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-br) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA175491 = {S10138, 119, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; it-) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA190172 = {S10139, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-gb) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA228973 = {S10140, 118, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA229410 = {S10141, 117, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA229509 = {S10142, 114, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA230420 = {S10143, 115, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA231661 = {S10144, 113, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA234853 = {S10145, 115, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/4.0.1 Safari/530.18*/
const Device UA260024 = {S10146, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-pt) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA260317 = {S10147, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; el-gr) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA260633 = {S10148, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; da-dk) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA261108 = {S10149, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; da-dk) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA261580 = {S10150, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-gb) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA261583 = {S10151, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; da-dk) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA261871 = {S10152, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-gb) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA261886 = {S10153, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-au) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA262295 = {S10154, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-cn) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA263318 = {S10155, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pl-pl) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA269860 = {S10156, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; nl-nl) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA269977 = {S10157, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; fr-fr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA271028 = {S10158, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; nl-nl) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA276642 = {S10159, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-pt) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA277535 = {S10160, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pl-pl) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA277532 = {S10161, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pt-pt) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA278163 = {S10162, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-au) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA278789 = {S10163, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; nl-nl) AppleWebKit/528.16 (KHTML, like Gecko) Version/4.0 Safari/528.16*/
const Device UA279303 = {S10164, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; es-es) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA280364 = {S10165, 122, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ar) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA282090 = {S10166, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ko-kr) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA283846 = {S10167, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-tw) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA294068 = {S10168, 123, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/4.0 Safari/528.16*/
const Device UA296361 = {S10169, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; it-it) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA300262 = {S10170, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; el-gr) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA303811 = {S10171, 123, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; it-it) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18*/
const Device UA305934 = {S10172, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; es-es) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA306376 = {S10173, 120, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0(Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA312553 = {S10174, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.3 (KHTML, like Gecko) Version/4.0.3 Safari/533.3*/
const Device UA330779 = {S10175, 113, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/4.0 Safari/528.16*/
const Device UA371402 = {S10176, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; he-il) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA371577 = {S10177, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tr-tr) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA372178 = {S10178, 129, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) WebClip/5527.2 Safari/5531.22.7*/
const Device UA372234 = {S10179, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lt-lt) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA372548 = {S10180, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ko-kr) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA372550 = {S10181, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ca) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA372643 = {S10182, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; fr-fr) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA372725 = {S10183, 130, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/531.21.8 (KHTML, like Gecko) WebClip/5527.2 Safari/5531.21.10*/
const Device UA382374 = {S10184, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-gb) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA382609 = {S10185, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ar) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA382729 = {S10186, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.1 Safari/530.18*/
const Device UA383667 = {S10187, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; it-it) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA384131 = {S10188, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hr-hr) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA388347 = {S10189, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-ch) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA399196 = {S10190, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-tw) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA399455 = {S10191, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ca) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA399935 = {S10192, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; he-il) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA407964 = {S10193, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hu-hu) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA413473 = {S10194, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; el-gr) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA414611 = {S10195, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ca-es) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA415550 = {S10196, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ja-jp) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA415755 = {S10197, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; cs-cz) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA417398 = {S10198, 120, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7;en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA430241 = {S10199, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-tw) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA430456 = {S10200, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA431347 = {S10201, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ca) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA431990 = {S10202, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tr-tr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA438943 = {S10203, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; nl-nl) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA443774 = {S10204, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; af-za) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA444718 = {S10205, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ja-jp) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA448949 = {S10206, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; et-ee) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA459740 = {S10207, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; cs-cz) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA460373 = {S10208, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tr-tr) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA467101 = {S10209, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; af-za) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA468509 = {S10210, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; af-za) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA471841 = {S10211, 126, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-us) AppleWebKit/528.4+ (KHTML, like Gecko) Version/4.0dp1 Safari/526.11.2*/
const Device UA476599 = {S10212, 124, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ar) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA480796 = {S10213, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; uk-ua) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA482124 = {S10214, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; is-is) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA486288 = {S10215, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ru-ru) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA490167 = {S10216, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; uk-ua) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA495887 = {S10217, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; cs-cz) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA498059 = {S10218, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sl-si) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA498128 = {S10219, 124, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA500767 = {S10220, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ch) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA514068 = {S10221, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fa-ir) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA518742 = {S10222, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lt-lt) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA520246 = {S10223, 123, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; da-dk) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18*/
const Device UA521936 = {S10224, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-xl) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA522093 = {S10225, 122, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; tl) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA522321 = {S10226, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; he-il) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA526831 = {S10227, 133, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/528.18.1 (KHTML, like Gecko) Version/4 Public Beta Safari/528.17*/
const Device UA527154 = {S10228, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ca-es) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA528741 = {S10229, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; pt-br) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA624677 = {S10230, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; el-gr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA625673 = {S10231, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; is-is) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA637106 = {S10232, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; is-is) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA640741 = {S10233, 119, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_6; it-it) AppleWebKit/528.16 (KHTML, like Gecko) Version/4.0 Safari/528.16*/
const Device UA653527 = {S10234, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ko-kr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA656533 = {S10235, 113, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/4.0 Safari/528.17*/
const Device UA670997 = {S10236, 123, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-gb) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18*/
const Device UA693376 = {S10237, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; he-il) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA697181 = {S10238, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; vi-vn) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA697220 = {S10239, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-gb) AppleWebKit/528.10+ (KHTML, like Gecko) Version/4.0dp1 Safari/526.11.2*/
const Device UA707860 = {S10240, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ru-ru) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA709783 = {S10241, 120, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/5*/
const Device UA710059 = {S10242, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; zh-cn) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA721050 = {S10243, 119, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; fr-) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA725524 = {S10244, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; pt-br) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA730862 = {S10245, 123, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ru-ru) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18*/
const Device UA734186 = {S10246, 122, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA748105 = {S10247, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ca) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA751356 = {S10248, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; cs-cz) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA755814 = {S10249, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sk-sk) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA757600 = {S10250, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; th-th) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA761291 = {S10251, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ko-kr) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA765932 = {S10252, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lv-lv) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA766234 = {S10253, 119, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_7; en-us) AppleWebKit/528.16 (KHTML, like Gecko) Version/4.0 Safari/528.16*/
const Device UA770833 = {S10254, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; sv-se) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA773794 = {S10255, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lt-lt) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA782773 = {S10256, 123, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/531.9 (KHTML, like Gecko) WebClip/5527.2 Safari/5531.9*/
const Device UA785844 = {S10257, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; th-th) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA787730 = {S10258, 123, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ja-jp) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18*/
const Device UA804074 = {S10259, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-ch) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA806799 = {S10260, 126, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-gb) AppleWebKit/528.4+ (KHTML, like Gecko) Version/4.0dp1 Safari/526.11.2*/
const Device UA810826 = {S10261, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ro-ro) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA815875 = {S10262, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; th-th) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA838072 = {S10263, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-xl) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA852422 = {S10264, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ja-jp) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA869921 = {S10265, 118, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ar) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA870881 = {S10266, 123, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; zh-tw) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18*/
const Device UA870986 = {S10267, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA882782 = {S10268, 118, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA1837278 = {S10269, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; vi-vn) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA1837470 = {S10270, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; af-za) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA1852953 = {S10271, 123, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA1855050 = {S10272, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA1856921 = {S10273, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-xl) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA1861099 = {S10274, 131, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/528.16 (KHTML, like Gecko) Version/4 Public Beta Safari/528.16*/
const Device UA1863416 = {S10275, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sk-sk) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA1863993 = {S10276, 123, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-au) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18*/
const Device UA1865946 = {S10277, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; zh-tw) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA1867855 = {S10278, 120, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18*/
const Device UA1888600 = {S10279, 122, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sq) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA1930313 = {S10280, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_7; es-es) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18*/
const Device UA2128074 = {S10281, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ro-ro) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA2128573 = {S10282, 126, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/526.11 (KHTML, like Gecko) Version/4.0dp1 Safari/526.11.2*/
const Device UA2132170 = {S10283, 123, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; nb-no) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18*/
const Device UA2133718 = {S10284, 123, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; sv-se) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0.1 Safari/530.18*/
const Device UA2136244 = {S10285, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-au) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA2163014 = {S10286, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/528.14 (KHTML, like Gecko) Version/4.0 Safari/528.14*/
const Device UA2163959 = {S10287, 122, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.11+ (KHTML, like Gecko) Version/4.0 Safari/530.11*/
const Device UA2165476 = {S10288, 120, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/528.8+ (KHTML, like Gecko) Version/4.0 Safari/528.1*/
const Device UA2166204 = {S10289, 119, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.6 (KHTML, like Gecko) Version/4.0 Safari/530.5*/
const Device UA2167820 = {S10290, 136, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Intel Mac OS X 10_5_7; en-us; Quatius-TV/SONIQ_51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2208020 = {S10291, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ca-es) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA2209649 = {S10292, 118, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2216348 = {S10293, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; pl-pl) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA2217803 = {S10294, 136, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Intel Mac OS X 10_5_7; en-us; Quatius-TV/SONIQ_50) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2218101 = {S10295, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/528.12 (KHTML, like Gecko) Version/4.0 Safari/528.12*/
const Device UA2258256 = {S10296, 126, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/526.11 (KHTML, like Gecko) Version/4.0dp1 Safari/526.11.2*/
const Device UA2258323 = {S10297, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ru-ru) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2258353 = {S10298, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-gb) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2258518 = {S10299, 120, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.6+ (KHTML, like Gecko) Version/4.0 Safari/530.1*/
const Device UA2263100 = {S10300, 127, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-ch) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA2265094 = {S10301, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-ch) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA2265630 = {S10302, 129, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; pl-pl) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9 FBSMTWB*/
const Device UA2266121 = {S10303, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; nb-no) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA2266632 = {S10304, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; fi-fi) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA2267273 = {S10305, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; lt-lt) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2271366 = {S10306, 126, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; de-de) AppleWebKit/526.11 (KHTML, like Gecko) Version/4.0dp1 Safari/526.11.2*/
const Device UA2272094 = {S10307, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; da-dk) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA2280897 = {S10308, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; pl-pl) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA2294894 = {S10309, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; hr-hr) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA2307367 = {S10310, 122, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sw) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA2330677 = {S10311, 119, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.6 (KHTML, like Gecko) Version/4.0 Safari/530.6*/
const Device UA2333516 = {S10312, 117, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_4; en-us) AppleWebKit/528.1 (KHTML, like Gecko) Version/4.0 Safari/528.1*/
const Device UA2359735 = {S10313, 128, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; us-us) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA2416620 = {S10314, 136, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Intel Mac OS X 10_5_7; en-us; Quatius-TV/SONIQ_52) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2542008 = {S10315, 123, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_8; hu-hu) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA2542015 = {S10316, 121, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; pt-pt) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA2758933 = {S10317, 117, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_5; en-us) AppleWebKit/528.1 (KHTML, like Gecko) Version/4.0 Safari/528.1*/
const Device UA2988138 = {S10318, 136, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Intel Mac OS X 10_5_7; en-us; Quatius-TV/SONIQ_53) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA3069104 = {S10319, 125, { &P17595,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sl-si) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA40379 = {S10320, 126, { &P14929,&P5156,&P3519,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9780; en) AppleWebKit/534.8+ (KHTML, like Gecko) Version/6.0.0.666 Mobile Safari/534.8+*/
const Device UA52167 = {S10321, 77, { &P14929,&P5156,&P3519,&P18092 }};	/*Useragent - BlackBerry9780/6.0.0.448 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102*/
const Device UA2320656 = {S10322, 64, { &P14929,&P5156,&P3519,&P18092 }};	/*Useragent - BlackBerry9780/6.6.0.207 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2370869 = {S10323, 63, { &P14929,&P5156,&P3519,&P18092 }};	/*Useragent - BlackBerry9780/6.5.0.54 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA184782 = {S10324, 50, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.1)*/
const Device UA189422 = {S10325, 63, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/5.0)*/
const Device UA311898 = {S10326, 63, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/5.0)*/
const Device UA393313 = {S10327, 50, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1)*/
const Device UA632494 = {S10328, 97, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; AOL 9.7; AOLBuild 4343.19; Windows NT 5.1; WOW64; Trident/5.0)*/
const Device UA717331 = {S10329, 151, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA747610 = {S10330, 70, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.1; WOW64; Trident/5.0)*/
const Device UA1886606 = {S10331, 121, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/9.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1);)*/
const Device UA1944882 = {S10332, 178, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA1954988 = {S10333, 68, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; SV1)*/
const Device UA1957036 = {S10334, 68, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; SV1)*/
const Device UA1961346 = {S10335, 68, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; SV1)*/
const Device UA1996106 = {S10336, 76, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; MS-RTC LM 8)*/
const Device UA2151490 = {S10337, 132, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322)*/
const Device UA2159638 = {S10338, 68, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; SV1)*/
const Device UA2195395 = {S10339, 196, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C)*/
const Device UA2215519 = {S10340, 69, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; MSIE 9.0; Windows NT 5.1; de-DE) Trident/5.0*/
const Device UA2216892 = {S10341, 166, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)*/
const Device UA2336553 = {S10342, 166, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0)*/
const Device UA2417588 = {S10343, 90, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3230.2052; Windows XP 5.1; MSIE 9.0.8112.16421)*/
const Device UA2418330 = {S10344, 180, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/4.0; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2425790 = {S10345, 65, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; MSIE 9.0; Edition IBIS; Trident/5.0)*/
const Device UA2490689 = {S10346, 70, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.2; .NET CLR 1.0.3705;)*/
const Device UA2577046 = {S10347, 156, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)*/
const Device UA2692191 = {S10348, 214, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C)*/
const Device UA2775881 = {S10349, 73, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; SV1; SV1)*/
const Device UA2889176 = {S10350, 73, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; SV1; SV1)*/
const Device UA2980203 = {S10351, 73, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; SV1; SV1)*/
const Device UA2980957 = {S10352, 73, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; SV1; SV1)*/
const Device UA2981534 = {S10353, 110, { &P15364,&P5690,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/5.0; InfoPath.2; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA10835 = {S10354, 227, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTCA9191/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10851 = {S10355, 138, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; Desire HD Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA16614 = {S10356, 147, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_DesireHD_A9191 Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA16617 = {S10357, 147, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC_DesireHD_A9191 Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20899 = {S10358, 150, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_DesireHD-orange-LS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA23645 = {S10359, 137, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24291 = {S10360, 146, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_DesireHD_A9191 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24614 = {S10361, 153, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire HD 2.36.161.12 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA39563 = {S10362, 152, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire HD 2.50.161.3 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA39628 = {S10363, 146, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_DesireHD_A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA39648 = {S10364, 141, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire HD Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41071 = {S10365, 131, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-; 001HT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA45977 = {S10366, 146, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_DesireHD_A9191 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46061 = {S10367, 146, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_DesireHD_A9191 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52922 = {S10368, 152, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; HTC Desire HD 1.32.163.6 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52963 = {S10369, 141, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52987 = {S10370, 153, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; HTC Desire HD 2.36.163.13 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA53055 = {S10371, 150, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_DesireHD_A9191/1.0 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA53068 = {S10372, 152, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; HTC Desire HD 1.75.161.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA56535 = {S10373, 140, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_DesireHD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA56747 = {S10374, 133, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-jp; 001HT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58514 = {S10375, 146, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; HTC_DesireHD_A9192 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59547 = {S10376, 152, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire HD 2.50.161.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA69996 = {S10377, 147, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70271 = {S10378, 141, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Desire HD Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72065 = {S10379, 137, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Desire HD Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73256 = {S10380, 137, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; Desire HD Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73307 = {S10381, 152, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; HTC Desire HD 2.50.163.3 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76866 = {S10382, 138, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; iw-; HTC_DesireHD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78293 = {S10383, 144, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; iw-; HTC_DesireHD_A9191 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA179243 = {S10384, 138, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Desire HD Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA190147 = {S10385, 61, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; Desire HD Build/GRI40)*/
const Device UA202356 = {S10386, 137, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA210730 = {S10387, 137, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA249833 = {S10388, 137, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA347784 = {S10389, 152, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire HD 3.13.161.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA354635 = {S10390, 143, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Desire HD Build/unknown) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA359565 = {S10391, 152, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; HTC Desire HD 2.50.161.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369237 = {S10392, 67, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.3; en_US; Desire HD Build/GRI40)*/
const Device UA371083 = {S10393, 146, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; HTC_DesireHD_A9191 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA401105 = {S10394, 65, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Desire HD Build/GRJ90)*/
const Device UA535299 = {S10395, 135, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA551679 = {S10396, 57, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; 001HT Build/GRI40)*/
const Device UA570973 = {S10397, 125, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Desire HD Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA573527 = {S10398, 61, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; Desire HD Build/GRI40)*/
const Device UA622686 = {S10399, 61, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Desire HD Build/GRJ90)*/
const Device UA623971 = {S10400, 71, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Desire HD A9191 Build/GRJ90)*/
const Device UA625297 = {S10401, 146, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; HTC_DesireHD_A9192 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625664 = {S10402, 150, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_DesireHD-orange-LS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625998 = {S10403, 140, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_DesireHD Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA636627 = {S10404, 152, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC Desire HD 3.13.163.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA697047 = {S10405, 135, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire HD A9191 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA714029 = {S10406, 144, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Desire HD Build/GRI40) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA763977 = {S10407, 158, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire HD with Beats Audio Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1947570 = {S10408, 125, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Desire HD Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1987154 = {S10409, 130, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Desire HD Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1990052 = {S10410, 140, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire HD A9191 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2018317 = {S10411, 138, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-au; Desire HD Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019809 = {S10412, 61, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; Desire HD Build/GRJ22)*/
const Device UA2020194 = {S10413, 152, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; HTC Desire HD A9191 Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021581 = {S10414, 144, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Desire HD Build/Eva.MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021991 = {S10415, 137, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Desire HD Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022644 = {S10416, 144, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-; HTC_DesireHD_A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024946 = {S10417, 130, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Desire HD Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2026516 = {S10418, 134, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire HD Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2027704 = {S10419, 161, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7 base 280; en-gb; Desire HD Build/GRI40; LexBoosT DHD+) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029756 = {S10420, 152, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC Desire HD 2.50.163.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030523 = {S10421, 145, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Desire HD Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2031306 = {S10422, 140, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire HD A9191 Build GRJ90) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2031896 = {S10423, 145, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Desire HD Build/GRJ90) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2388808 = {S10424, 152, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire HD 3.13.161.5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2789163 = {S10425, 155, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30*/
const Device UA2808481 = {S10426, 145, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2814045 = {S10427, 133, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Desire HD Build MIUI) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2853451 = {S10428, 147, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; HTC-DESIRE HD Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2933892 = {S10429, 141, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-hk; HTC_Desire_HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3069739 = {S10430, 138, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Desire HD Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3110186 = {S10431, 162, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 2.3.5;dev=ace;devm=Desire HD;man=HTC;brw=Android;shwd=480x800;shwa=480x763;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA3117144 = {S10432, 141, { &P14225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; HTC Desire HD Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2567779 = {S10433, 104, { &P15364,&P16110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.2.0.0 Safari/537.22*/
const Device UA2567786 = {S10434, 97, { &P15364,&P16110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.2.0.0 Safari/537.22*/
const Device UA2567795 = {S10435, 109, { &P15364,&P16110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2567800 = {S10436, 102, { &P15364,&P16110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2568257 = {S10437, 119, { &P15364,&P16110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)*/
const Device UA2569381 = {S10438, 112, { &P15364,&P16110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)*/
const Device UA2569765 = {S10439, 104, { &P15364,&P16110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.1.0.0 Safari/537.22*/
const Device UA2569911 = {S10440, 97, { &P15364,&P16110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.1.0.0 Safari/537.22*/
const Device UA2584651 = {S10441, 148, { &P15364,&P16110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3027980 = {S10442, 171, { &P15364,&P16110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22 BitdefenderSafepay/17.14.0.0 (OBK; 25.0.1364.172; r1194.2013)*/
const Device UA3072840 = {S10443, 108, { &P15364,&P16110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.22 (KHTML like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2878888 = {S10444, 115, { &P17595,&P18207,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1083) AppleWebKit/537.36 (KHTML like Gecko) Chrome/28.0.1469.0 Safari/537.36*/
const Device UA2898317 = {S10445, 119, { &P17595,&P18207,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36*/
const Device UA2898330 = {S10446, 119, { &P17595,&P18207,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36*/
const Device UA2968285 = {S10447, 129, { &P17595,&P18207,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36,gzip(gfe)*/
const Device UA3024737 = {S10448, 119, { &P17595,&P18207,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3024752 = {S10449, 119, { &P17595,&P18207,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3024957 = {S10450, 119, { &P17595,&P18207,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3024988 = {S10451, 129, { &P17595,&P18207,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe)*/
const Device UA3028770 = {S10452, 119, { &P17595,&P18207,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3030408 = {S10453, 158, { &P17595,&P18207,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA56486 = {S10454, 77, { &P15722,&P5156,&P3519,&P18092 }};	/*Useragent - BlackBerry9300/6.0.0.526 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102*/
const Device UA75075 = {S10455, 126, { &P15722,&P5156,&P3519,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9300; en) AppleWebKit/534.8+ (KHTML, like Gecko) Version/6.0.0.668 Mobile Safari/534.8+*/
const Device UA925215 = {S10456, 139, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996825 = {S10457, 61, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-I9100 Build/JRO03H)*/
const Device UA2009558 = {S10458, 139, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9100 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029671 = {S10459, 139, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; GT-I9100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2198812 = {S10460, 139, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9100 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2226650 = {S10461, 140, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2234668 = {S10462, 139, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2242263 = {S10463, 61, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9100 Build/JZO54K)*/
const Device UA2257851 = {S10464, 139, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258125 = {S10465, 138, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/cMIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2383073 = {S10466, 140, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100G Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2401679 = {S10467, 136, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; GT-I9100 Build/1.9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2449694 = {S10468, 138, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100 Build/cMIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2456276 = {S10469, 132, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2502437 = {S10470, 62, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9100G Build/JZO54K)*/
const Device UA2503232 = {S10471, 130, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9100 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2504845 = {S10472, 140, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; GT-I9100M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2527161 = {S10473, 140, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; GT-I9100G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2527558 = {S10474, 137, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/M1UI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2553178 = {S10475, 139, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9100G Build/cMIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566850 = {S10476, 158, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I9100/I9100XWLSD Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2596941 = {S10477, 137, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; GT-I9100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2598165 = {S10478, 62, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9100M Build/JZO54K)*/
const Device UA2642158 = {S10479, 136, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.11; en-us; GT-I9100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2651864 = {S10480, 132, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2656308 = {S10481, 151, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2_????_??????; he-il; GT-I9100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658546 = {S10482, 140, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-I9100T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2661090 = {S10483, 62, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9100T Build/JZO54K)*/
const Device UA2666535 = {S10484, 139, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; GT-I9100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695879 = {S10485, 137, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-I9100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2701303 = {S10486, 140, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9100g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708574 = {S10487, 157, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100-ORANGE/I9100XWLSM Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2711362 = {S10488, 158, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I9100/I9100XWLSM Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2762782 = {S10489, 140, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9100G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2769469 = {S10490, 157, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-I9100-ORANGE/I9100XWLSU Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2783253 = {S10491, 158, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; SAMSUNG GT-I9100/I9100XWLSS Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2830244 = {S10492, 140, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100G Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2851251 = {S10493, 148, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT_I9100 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2860510 = {S10494, 143, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; I9100 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2861569 = {S10495, 143, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ar-ae; GT-I9100 Build/JZO54K_v12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934407 = {S10496, 131, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9100G Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2935446 = {S10497, 139, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9108 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937970 = {S10498, 157, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100-ORANGE/I9100XWLSY Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2946718 = {S10499, 158, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I9100/I9100XWLSY Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952880 = {S10500, 138, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-I9100G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2988387 = {S10501, 138, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; GT-I9100G Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2991945 = {S10502, 139, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; vi-vn; GT-I9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3070120 = {S10503, 130, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9100 Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3107484 = {S10504, 145, { &P15575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9100 Build/JZO54K.ASNET) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA91421 = {S10505, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pl-pl) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA91823 = {S10506, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; zh-cn) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA91978 = {S10507, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; da-dk) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA92177 = {S10508, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA92400 = {S10509, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; de-de) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA92760 = {S10510, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ja-jp) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA92875 = {S10511, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-us) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA93094 = {S10512, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; it-it) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA93240 = {S10513, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; es-es) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA93271 = {S10514, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ja-jp) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA93512 = {S10515, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-us) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA93520 = {S10516, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; it-it) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA93918 = {S10517, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fr-fr) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA94169 = {S10518, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; sv-se) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA94223 = {S10519, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ja-jp) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA94698 = {S10520, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; sv-se) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA94721 = {S10521, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-gb) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA95055 = {S10522, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-us) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA95182 = {S10523, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pt-br) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA95624 = {S10524, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; de-de) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA96470 = {S10525, 119, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/530.13 (KHTML, like Gecko) Version/4.0 Safari/530.15*/
const Device UA96598 = {S10526, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; it-it) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA96623 = {S10527, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; de-de) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA96634 = {S10528, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; pt-br) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA96856 = {S10529, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ko-kr) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA97492 = {S10530, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; es-es) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA98026 = {S10531, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nb-no) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA98101 = {S10532, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-gb) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA98133 = {S10533, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; de-de) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA98988 = {S10534, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-us) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA99015 = {S10535, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ru-ru) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA99299 = {S10536, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nb-no) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA99349 = {S10537, 119, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA100026 = {S10538, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; nl-nl) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA100414 = {S10539, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; nb-no) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA100609 = {S10540, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ja-jp) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA101315 = {S10541, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pl-pl) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA101585 = {S10542, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; nb-no) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA101631 = {S10543, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; fr-fr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA101769 = {S10544, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nl-nl) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA102733 = {S10545, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; es-es) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA104244 = {S10546, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; zh-cn) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA104234 = {S10547, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; fr-fr) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA104301 = {S10548, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; sv-se) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA106523 = {S10549, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; fr-fr) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA106592 = {S10550, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; de-de) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA106786 = {S10551, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; es-es) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA110001 = {S10552, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pt-pt) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA111275 = {S10553, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; de-at) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA114184 = {S10554, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; nl-nl) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA114255 = {S10555, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; zh-tw) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA114419 = {S10556, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; sv-se) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA116480 = {S10557, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; nl-nl) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA117397 = {S10558, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-gb) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA118222 = {S10559, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; es-es) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA118263 = {S10560, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; th-th) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA119173 = {S10561, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; fr-fr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA120375 = {S10562, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fi-fi) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA120649 = {S10563, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; zh-tw) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA122856 = {S10564, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA123936 = {S10565, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-au) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA125096 = {S10566, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA125276 = {S10567, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; sv-se) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA126893 = {S10568, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pt-pt) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA127757 = {S10569, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA127760 = {S10570, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; ja-jp) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA129378 = {S10571, 126, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ar) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA129733 = {S10572, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA130067 = {S10573, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-ca) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA130199 = {S10574, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; th-th) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA130383 = {S10575, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; de-de) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA130818 = {S10576, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; da-dk) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA131126 = {S10577, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fr-fr) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA131811 = {S10578, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pt-br) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA131986 = {S10579, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ru-ru) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA132896 = {S10580, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; zh-cn) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA133054 = {S10581, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; nb-no) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA133526 = {S10582, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-au) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA133822 = {S10583, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; sv-se) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA133953 = {S10584, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fa-ir) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA134126 = {S10585, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; zh-tw) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA134447 = {S10586, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA134629 = {S10587, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; it-it) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA137273 = {S10588, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; fi-fi) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA138110 = {S10589, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nl-nl) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA139688 = {S10590, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; nl-nl) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA145753 = {S10591, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; pt-br) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA151882 = {S10592, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; pt-br) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA152627 = {S10593, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; pt-br) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA152658 = {S10594, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; pt-pt) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA153284 = {S10595, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; nb-no) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA158659 = {S10596, 124, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA166900 = {S10597, 119, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/531.4 (KHTML, like Gecko) Version/4.0.3 Safari/531.4*/
const Device UA167291 = {S10598, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA168064 = {S10599, 124, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA177188 = {S10600, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA178210 = {S10601, 125, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA179291 = {S10602, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; es-xl) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA197888 = {S10603, 124, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA199545 = {S10604, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA201288 = {S10605, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA259534 = {S10606, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ja-jp) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA261431 = {S10607, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fi-fi) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA261875 = {S10608, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; it-it) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA262227 = {S10609, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-au) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA262499 = {S10610, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ru-ru) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA262744 = {S10611, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; zh-tw) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA262978 = {S10612, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-gb) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA263297 = {S10613, 124, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ar) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA263561 = {S10614, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; da-dk) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA263901 = {S10615, 125, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA264380 = {S10616, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; pt-pt) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA266595 = {S10617, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; es-es) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA270300 = {S10618, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-gb) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA271707 = {S10619, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; it-it) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA271872 = {S10620, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; af-za) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA274512 = {S10621, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ru-ru) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA274952 = {S10622, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; zh-cn) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA275510 = {S10623, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; fr-fr) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA277052 = {S10624, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; pl-pl) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA278095 = {S10625, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; th-th) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA278130 = {S10626, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ko-kr) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA278289 = {S10627, 119, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/531.5 (KHTML, like Gecko) Version/4.0.3 Safari/531.7*/
const Device UA278566 = {S10628, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; ru-ru) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA278963 = {S10629, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; tr-tr) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA279368 = {S10630, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; cs-cz) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA281346 = {S10631, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-gb) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA281519 = {S10632, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; el-gr) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA284721 = {S10633, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; tr-tr) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA289251 = {S10634, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; fi-fi) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA289408 = {S10635, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; pl-pl) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA291636 = {S10636, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; da-dk) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA292327 = {S10637, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA294887 = {S10638, 117, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/530.8 (KHTML, like Gecko) Version/4.0 Safari/530.8*/
const Device UA295141 = {S10639, 118, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; ar) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA295826 = {S10640, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; zh-tw) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA299160 = {S10641, 123, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko, Safari/6531.22.7) ADM/722*/
const Device UA301693 = {S10642, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA303472 = {S10643, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-au) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA311891 = {S10644, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA372395 = {S10645, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ru-ru) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA372965 = {S10646, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; da-dk) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA382652 = {S10647, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-ca) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA383701 = {S10648, 117, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/531.0 (KHTML, like Gecko) Version/4.0 Safari/531.0*/
const Device UA385310 = {S10649, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; fi-fi) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA390143 = {S10650, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; tr-tr) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA395031 = {S10651, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA408197 = {S10652, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; tr-tr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA409314 = {S10653, 124, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA412881 = {S10654, 134, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_6_1; en_GB, en_US) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA413430 = {S10655, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; he-il) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA415918 = {S10656, 126, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA416995 = {S10657, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; it-it) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA417274 = {S10658, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; lt-lt) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA420083 = {S10659, 124, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sv-se) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA441565 = {S10660, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ko-kr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA442071 = {S10661, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; vi-vn) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA442757 = {S10662, 122, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA446653 = {S10663, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; lt-lt) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA446944 = {S10664, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; vi-vn) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA451194 = {S10665, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA461859 = {S10666, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-au) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA465197 = {S10667, 122, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; nl-nl) AppleWebKit/533.16 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA465435 = {S10668, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; es-es) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA467005 = {S10669, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; fr-fr) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA472401 = {S10670, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; af-za) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA475179 = {S10671, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; zh-cn) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA477900 = {S10672, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; hu-hu) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA478075 = {S10673, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ro-ro) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA489056 = {S10674, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; cs-cz) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA492348 = {S10675, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; hr-hr) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA499565 = {S10676, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; is-is) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA501939 = {S10677, 123, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA503379 = {S10678, 119, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-us) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531*/
const Device UA506394 = {S10679, 115, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/531.9 (KHTML, like Gecko, Safari/6531.9) ADM/722*/
const Device UA511049 = {S10680, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; fi-fi) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA512299 = {S10681, 122, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA518154 = {S10682, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; el-gr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA531056 = {S10683, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ca-es) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA532536 = {S10684, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; de-de) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA533302 = {S10685, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; et-ee) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA533759 = {S10686, 124, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-us) AppleWebKit/531.21.8 (KHTML, like Gecko, Safari/6531.21.10) ADM/722*/
const Device UA534087 = {S10687, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; sk-sk) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA534934 = {S10688, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; sv-se) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA542003 = {S10689, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; pt-pt) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA556809 = {S10690, 124, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA558234 = {S10691, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA558339 = {S10692, 124, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA559915 = {S10693, 124, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA562837 = {S10694, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA565485 = {S10695, 124, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-fr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA599836 = {S10696, 125, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; zh-CN) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA635843 = {S10697, 117, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA639345 = {S10698, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; lt-lt) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA644968 = {S10699, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; he-il) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA645211 = {S10700, 126, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; id) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA647400 = {S10701, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; th-th) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA654486 = {S10702, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; vi-vn) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA658080 = {S10703, 120, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-us) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.*/
const Device UA659701 = {S10704, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0(Macintosh;U;Intel Mac OS X 10_6_3;en-us;V55 Build/MASTER)AppleWebKit/534.13(KHTML,like Gecko)Version4.0 Safari/534.13*/
const Device UA662539 = {S10705, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; ca-es) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA669579 = {S10706, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fr-ca) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA671199 = {S10707, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA676509 = {S10708, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA681920 = {S10709, 132, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/531.21.11 (KHTML, like Gecko) WebClip/6530.10 Safari/6531.21.10*/
const Device UA683331 = {S10710, 118, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA684347 = {S10711, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; ko-kr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA693053 = {S10712, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; de-ch) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA698281 = {S10713, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; pt-pt) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA701397 = {S10714, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fa-ir) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA707958 = {S10715, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; he-il) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA709581 = {S10716, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ca-es) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA709837 = {S10717, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; de-de) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA711916 = {S10718, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; tr-tr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA715819 = {S10719, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-ca) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA717956 = {S10720, 114, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA720888 = {S10721, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; fa-ir) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA722429 = {S10722, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ca-es) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA727047 = {S10723, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; fr-ca) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA729996 = {S10724, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; cs-cz) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA732612 = {S10725, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; el-gr) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA734050 = {S10726, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA734387 = {S10727, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; he-il) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA747894 = {S10728, 125, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ar) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA753900 = {S10729, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; es-xl) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA757306 = {S10730, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; el-gr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA759406 = {S10731, 124, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA760606 = {S10732, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ko-kr) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA762172 = {S10733, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; cs-cz) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA764464 = {S10734, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; es-xl) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA770994 = {S10735, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; pl-pl) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA775575 = {S10736, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; es-es) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA780243 = {S10737, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; de-de) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA780510 = {S10738, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA799670 = {S10739, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/533.21.1*/
const Device UA804528 = {S10740, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; th-th) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA806177 = {S10741, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-au) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA809616 = {S10742, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; zh-cn) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA845940 = {S10743, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; hu-hu) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA847541 = {S10744, 118, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ar) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA862144 = {S10745, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nl-nl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA877876 = {S10746, 117, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-us) AppleWebKit/531.9 (KHTML, like Gecko, Safari/6531.9) ADM/722*/
const Device UA1839225 = {S10747, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; vi-vn) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA1839560 = {S10748, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; nl-nl) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA1851099 = {S10749, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; nl-nl) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA1851881 = {S10750, 118, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/530.6+ (KHTML, like Gecko) Version/4.0 Safari/530.6*/
const Device UA1860599 = {S10751, 117, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-us) AppleWebKit/531.9 (KHTML, like Gecko, Safari/6531.9) ADM/722*/
const Device UA1861949 = {S10752, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; cs-cz) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA1864617 = {S10753, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; is-is) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA1865840 = {S10754, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; es-xl) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA1872863 = {S10755, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; bg-bg) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA1887217 = {S10756, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; us-us) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA1898682 = {S10757, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA2125908 = {S10758, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; he-il) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA2130782 = {S10759, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ro-ro) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA2131159 = {S10760, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; zh-tw) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA2132122 = {S10761, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; sk-sk) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA2136114 = {S10762, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; de-de) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA2142033 = {S10763, 124, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2143594 = {S10764, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA2207945 = {S10765, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; af-za) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2215597 = {S10766, 118, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2258251 = {S10767, 117, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/531.1 (KHTML, like Gecko) Version/4.0 Safari/531.1*/
const Device UA2258345 = {S10768, 119, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/530.10 (KHTML, like Gecko) Version/4.0 Safari/530.10*/
const Device UA2258773 = {S10769, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nb-no) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA2264924 = {S10770, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; lt-lt) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2265905 = {S10771, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-ca) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA2267238 = {S10772, 124, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fr-fr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2271705 = {S10773, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; da-dk) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA2274926 = {S10774, 125, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/526.9.1 (KHTML, like Gecko) Version/4.0dp1 Safari/526.10.1*/
const Device UA2275283 = {S10775, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; fa-ir) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2280200 = {S10776, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; it-it) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA2281783 = {S10777, 125, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; it-it) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2282753 = {S10778, 124, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; fr-fr) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2284080 = {S10779, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; fr-ca) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2285641 = {S10780, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA2289252 = {S10781, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; it-it) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA2293260 = {S10782, 109, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6) AppleWebKit/531.4 (KHTML, like Gecko) Version/4.0.3 Safari/531.4*/
const Device UA2294739 = {S10783, 124, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; nb-no) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2296510 = {S10784, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; de-ch) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2297871 = {S10785, 124, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pt-br) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2315785 = {S10786, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.5 Safari/533.19.4*/
const Device UA2316894 = {S10787, 136, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.6+ (KHTML, like Gecko) Version/4.0 Safari/528.16 Titanium/1.1.0*/
const Device UA2317642 = {S10788, 117, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/530.4 (KHTML, like Gecko) Version/4.0 Safari/530.4*/
const Device UA2359749 = {S10789, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; us-us) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2359757 = {S10790, 163, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; us-us) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10 Opera/9.50 (Windows NT 6.0; U; en)*/
const Device UA2369789 = {S10791, 129, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; de-ch) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA2370552 = {S10792, 182, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.17.8 (KHTML, like Gecko) Mozilla/5.0 - AppleWebKit/533.17.8 (KHTML, like Gecko) Version/4.0.0 Safari/533.17.8*/
const Device UA2438482 = {S10793, 125, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; es-es) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2493895 = {S10794, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; hi-in) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2597346 = {S10795, 118, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1061; en-us) AppleWebKit/531.9 (KHTML like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2597350 = {S10796, 123, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 106; en-us) AppleWebKit/531.22.7 (KHTML like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA2597438 = {S10797, 117, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 106; en-us) AppleWebKit/531.4 (KHTML like Gecko) Version/4.0.3 Safari/531.4*/
const Device UA2597687 = {S10798, 114, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 106; ar) AppleWebKit/531.9 (KHTML like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2597903 = {S10799, 117, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 106; en-us) AppleWebKit/531.5 (KHTML like Gecko) Version/4.0.3 Safari/531.7*/
const Device UA2599240 = {S10800, 117, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 106; en-us) AppleWebKit/530.13 (KHTML like Gecko) Version/4.0 Safari/530.15*/
const Device UA2720156 = {S10801, 127, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; sv-se) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA2720324 = {S10802, 121, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; hr-hr) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2742189 = {S10803, 124, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; pl-pl) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2759103 = {S10804, 136, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/(null) (KHTML, like Gecko) Version/4.0 Safari/528.16 Titanium/1.1.0*/
const Device UA2801328 = {S10805, 136, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-ca) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10 FBSMTWB*/
const Device UA2802781 = {S10806, 128, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-fr) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA2806116 = {S10807, 118, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; cy) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2981287 = {S10808, 135, { &P17595,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; U; PPC Mac OS X 10_6_7 rv:6.0; en-US) AppleWebKit/532.23.3 (KHTML, like Gecko) Version/4.0.2 Safari/532.23.3*/
const Device UA2567785 = {S10809, 120, { &P17595,&P18207,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2567787 = {S10810, 120, { &P17595,&P18207,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2567808 = {S10811, 120, { &P17595,&P18207,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2568041 = {S10812, 120, { &P17595,&P18207,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2569634 = {S10813, 130, { &P17595,&P18207,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)*/
const Device UA2584173 = {S10814, 130, { &P17595,&P18207,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)*/
const Device UA2584220 = {S10815, 159, { &P17595,&P18207,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2604389 = {S10816, 159, { &P17595,&P18207,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2734502 = {S10817, 120, { &P17595,&P18207,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA3071932 = {S10818, 120, { &P17595,&P18207,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA69916 = {S10819, 142, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA348645 = {S10820, 130, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA360495 = {S10821, 143, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-S5360L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA362910 = {S10822, 143, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-nz; GT-S5360T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA396967 = {S10823, 66, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-S5360 Build/GINGERBREAD)*/
const Device UA622962 = {S10824, 143, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; GT-S5360B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622991 = {S10825, 142, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625397 = {S10826, 161, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-S5369/S5369BUKL3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA628375 = {S10827, 161, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-S5369/S5369BULA1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA630199 = {S10828, 143, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-S5360B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633649 = {S10829, 66, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5360 Build/GINGERBREAD)*/
const Device UA639403 = {S10830, 151, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; SAMSUNG GT-S5360T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA642548 = {S10831, 130, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-S5360 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA648704 = {S10832, 160, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-S5369-ORANGE/S5369BVLC3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA680363 = {S10833, 143, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; GT-S5360L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA763287 = {S10834, 142, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SCH-i509 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA899772 = {S10835, 142, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SCH-i509 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1937832 = {S10836, 140, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1952216 = {S10837, 67, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-S5360L Build/GINGERBREAD)*/
const Device UA1959358 = {S10838, 67, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5360L Build/GINGERBREAD)*/
const Device UA1965478 = {S10839, 161, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ie; SAMSUNG GT-S5369/S5369BULD4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1971213 = {S10840, 159, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; SAMSUNG SAMSUNG GT-S5360T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1973017 = {S10841, 142, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; GT-S5369 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1985511 = {S10842, 130, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i509 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1988175 = {S10843, 160, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-S5369-ORANGE/S5369BVLE1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1999899 = {S10844, 161, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ie; SAMSUNG GT-S5369/S5369BULF2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021342 = {S10845, 66, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-i509 Build/GINGERBREAD)*/
const Device UA2021564 = {S10846, 130, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SCH-i509 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2023017 = {S10847, 140, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023383 = {S10848, 131, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; GT-S5360B Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2023416 = {S10849, 67, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5360B Build/GINGERBREAD)*/
const Device UA2023426 = {S10850, 67, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-S5360B Build/GINGERBREAD)*/
const Device UA2025116 = {S10851, 67, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5360T Build/GINGERBREAD)*/
const Device UA2026691 = {S10852, 66, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5369 Build/GINGERBREAD)*/
const Device UA2029546 = {S10853, 135, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i509 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2030858 = {S10854, 97, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.3.6;Zh_cn;GT-S5360;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA2030908 = {S10855, 160, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-S5369-ORANGE/S5369BVLF1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034034 = {S10856, 135, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5360 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2222926 = {S10857, 142, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; SCH-I509 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2367448 = {S10858, 161, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ie; SAMSUNG GT-S5369/S5369BULH1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2487236 = {S10859, 156, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.0 Version/4.0 Mobile Safari/530.17*/
const Device UA2487938 = {S10860, 161, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ie; SAMSUNG GT-S5369/S5369BULL1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2564262 = {S10861, 161, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-S5369/S5369BUMB1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2592761 = {S10862, 149, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SAMSUNG GT-S5360T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2650306 = {S10863, 83, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - ZingBrowser/3.1/Adr (Linux; U; 2.3.6; en; GT-S5360 Build/GINGERBREAD.DXLA1;240*320)*/
const Device UA2722907 = {S10864, 141, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5360L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2776150 = {S10865, 83, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - ZingBrowser/3.1/Adr (Linux; U; 2.3.6; en; GT-S5360 Build/GINGERBREAD.DXLE1;240*320)*/
const Device UA2781732 = {S10866, 160, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-S5369-ORANGE/S5369BVLL1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2807328 = {S10867, 140, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SCH-i509 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2816151 = {S10868, 167, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; SAMSUNG SAMSUNG SAMSUNG GT-S5360T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2828525 = {S10869, 83, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - ZingBrowser/3.1/Adr (Linux; U; 2.3.6; en; GT-S5360 Build/GINGERBREAD.DXKJ3;240*320)*/
const Device UA2981343 = {S10870, 83, { &P13123,&P5629,&P15760,&P18092 }};	/*Useragent - ZingBrowser/3.1/Adr (Linux; U; 2.3.6; en; GT-S5360 Build/GINGERBREAD.DXKK4;240*320)*/
const Device UA897296 = {S10871, 99, { &P15364,&P7133,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5*/
const Device UA897319 = {S10872, 136, { &P15364,&P7133,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5,gzip(gfe) (via translate.google.com)*/
const Device UA900904 = {S10873, 106, { &P15364,&P7133,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA901022 = {S10874, 143, { &P15364,&P7133,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5,gzip(gfe) (via translate.google.com)*/
const Device UA902276 = {S10875, 145, { &P15364,&P7133,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA903232 = {S10876, 138, { &P15364,&P7133,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA1952132 = {S10877, 137, { &P15364,&P7133,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/534.7 ZemanaAID/FFFF0059*/
const Device UA2355339 = {S10878, 116, { &P15364,&P7133,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5,gzip(gfe)*/
const Device UA2355558 = {S10879, 109, { &P15364,&P7133,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5,gzip(gfe)*/
const Device UA2363069 = {S10880, 125, { &P15364,&P7133,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5*/
const Device UA2363083 = {S10881, 132, { &P15364,&P7133,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5*/
const Device UA2428779 = {S10882, 135, { &P15364,&P7133,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5,gzip(gfe)*/
const Device UA2453103 = {S10883, 108, { &P15364,&P7133,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5*/
const Device UA2504924 = {S10884, 142, { &P15364,&P7133,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5410 Chrome/19.1.1084.5410 Safari/536.5,gzip(gfe)*/
const Device UA2640751 = {S10885, 118, { &P15364,&P7133,&P18167,&P18092 }};	/*Useragent - User-Agent: Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA2695166 = {S10886, 97, { &P15364,&P7133,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/19.77.34.5 Safari/537.1*/
const Device UA2357451 = {S10887, 119, { &P17595,&P17297,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2357471 = {S10888, 119, { &P17595,&P17297,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2358755 = {S10889, 119, { &P17595,&P17297,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2362059 = {S10890, 129, { &P17595,&P17297,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11,gzip(gfe)*/
const Device UA2389798 = {S10891, 119, { &P17595,&P17297,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11*/
const Device UA2435778 = {S10892, 193, { &P17595,&P17297,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/23.0.912.77 Safari/535.7 AppEngine-Google; (+http://code.google.com/appengine; appid: s~feedly-nikon)*/
const Device UA2437176 = {S10893, 120, { &P17595,&P17297,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11*/
const Device UA2437184 = {S10894, 120, { &P17595,&P17297,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11*/
const Device UA2437228 = {S10895, 130, { &P17595,&P17297,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11,gzip(gfe)*/
const Device UA2437581 = {S10896, 159, { &P17595,&P17297,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2490338 = {S10897, 132, { &P17595,&P17297,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11 SEMvisu/BOT*/
const Device UA2508045 = {S10898, 195, { &P17595,&P17297,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/23.0.912.77 Safari/535.7 AppEngine-Google; (+http://code.google.com/appengine; appid: s~feedly-hercule)*/
const Device UA2203725 = {S10899, 99, { &P15364,&P5690,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203797 = {S10900, 99, { &P15364,&P5690,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203805 = {S10901, 106, { &P15364,&P5690,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203982 = {S10902, 136, { &P15364,&P5690,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via translate.google.com)*/
const Device UA2231639 = {S10903, 106, { &P15364,&P5690,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2233051 = {S10904, 94, { &P15364,&P5690,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.<rndr(1000:1210*/
const Device UA2234779 = {S10905, 109, { &P15364,&P5690,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)*/
const Device UA2238054 = {S10906, 138, { &P15364,&P5690,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2259288 = {S10907, 174, { &P15364,&P5690,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 AppEngine-Google; (+http://code.google.com/appengine; appid: s~blassegg01)*/
const Device UA2355031 = {S10908, 108, { &P15364,&P5690,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; en-US) AppleWebKit/535.12 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/535.12*/
const Device UA2390838 = {S10909, 98, { &P15364,&P5690,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2534052 = {S10910, 130, { &P15364,&P5690,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (gtsr) Safari/537.4*/
const Device UA2535778 = {S10911, 130, { &P15364,&P5690,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (fnrb) Safari/537.4*/
const Device UA2540610 = {S10912, 127, { &P15364,&P5690,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; ; NCLIENT50_AAP7150B86519C) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2570712 = {S10913, 130, { &P15364,&P5690,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.520 YaBrowser/1.5.1105.520 (bskt) Safari/537.4*/
const Device UA2578140 = {S10914, 108, { &P15364,&P5690,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; zh-CN) AppleWebKit/535.12 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/535.12*/
const Device UA2711639 = {S10915, 100, { &P15364,&P5690,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.1.9999.433 Safari/537.4*/
const Device UA63272 = {S10916, 77, { &P11730,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry9700/5.0.0.714 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/100*/
const Device UA2585297 = {S10917, 64, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.15*/
const Device UA2585381 = {S10918, 57, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.15*/
const Device UA2668755 = {S10919, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.15*/
const Device UA2668947 = {S10920, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.15*/
const Device UA2670037 = {S10921, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.15*/
const Device UA2670776 = {S10922, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.15*/
const Device UA2670782 = {S10923, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.15*/
const Device UA2671025 = {S10924, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6080)) Presto/2.12.388 Version/12.15*/
const Device UA2671457 = {S10925, 69, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64) Opera 12.15*/
const Device UA2671670 = {S10926, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.15*/
const Device UA2671828 = {S10927, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.15*/
const Device UA2671839 = {S10928, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.15*/
const Device UA2672004 = {S10929, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.15*/
const Device UA2672092 = {S10930, 87, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.15*/
const Device UA2672094 = {S10931, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.15*/
const Device UA2672127 = {S10932, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.15*/
const Device UA2672204 = {S10933, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.15*/
const Device UA2672208 = {S10934, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.15*/
const Device UA2672318 = {S10935, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.15*/
const Device UA2672326 = {S10936, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.15*/
const Device UA2672344 = {S10937, 87, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.15*/
const Device UA2672347 = {S10938, 75, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.15*/
const Device UA2672543 = {S10939, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.15*/
const Device UA2672620 = {S10940, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.15*/
const Device UA2672646 = {S10941, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.15*/
const Device UA2672712 = {S10942, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.15*/
const Device UA2672813 = {S10943, 87, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.15*/
const Device UA2672812 = {S10944, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.15*/
const Device UA2672971 = {S10945, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.15*/
const Device UA2672980 = {S10946, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.15*/
const Device UA2673080 = {S10947, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.15*/
const Device UA2673162 = {S10948, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.15*/
const Device UA2673177 = {S10949, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.15*/
const Device UA2673248 = {S10950, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.15*/
const Device UA2673253 = {S10951, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.15*/
const Device UA2673363 = {S10952, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.15*/
const Device UA2673369 = {S10953, 67, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.15,gzip(gfe)*/
const Device UA2673547 = {S10954, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.15*/
const Device UA2673551 = {S10955, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.15*/
const Device UA2673823 = {S10956, 74, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.15 Yandex.Translate*/
const Device UA2674000 = {S10957, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.15*/
const Device UA2674180 = {S10958, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.15*/
const Device UA2674198 = {S10959, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.15*/
const Device UA2674199 = {S10960, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.15*/
const Device UA2674424 = {S10961, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.15*/
const Device UA2674435 = {S10962, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.15*/
const Device UA2674447 = {S10963, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.15*/
const Device UA2674437 = {S10964, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.15*/
const Device UA2674723 = {S10965, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.15*/
const Device UA2674741 = {S10966, 87, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.15*/
const Device UA2674831 = {S10967, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.15*/
const Device UA2675051 = {S10968, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.15*/
const Device UA2675067 = {S10969, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5997)) Presto/2.12.388 Version/12.15*/
const Device UA2675079 = {S10970, 87, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.15*/
const Device UA2675252 = {S10971, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.15*/
const Device UA2675266 = {S10972, 74, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.15,gzip(gfe)*/
const Device UA2675375 = {S10973, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.15*/
const Device UA2675385 = {S10974, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.15*/
const Device UA2675700 = {S10975, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.15*/
const Device UA2676068 = {S10976, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.15*/
const Device UA2676377 = {S10977, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4933)) Presto/2.12.388 Version/12.15*/
const Device UA2676637 = {S10978, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.15*/
const Device UA2676647 = {S10979, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.15*/
const Device UA2676660 = {S10980, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.15*/
const Device UA2676785 = {S10981, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.15*/
const Device UA2676788 = {S10982, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.15*/
const Device UA2676882 = {S10983, 87, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.15*/
const Device UA2677696 = {S10984, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.15*/
const Device UA2677698 = {S10985, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.15*/
const Device UA2677701 = {S10986, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.15*/
const Device UA2677711 = {S10987, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.15*/
const Device UA2680314 = {S10988, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.15*/
const Device UA2706645 = {S10989, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.15*/
const Device UA2707312 = {S10990, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5217)) Presto/2.12.388 Version/12.15*/
const Device UA2707472 = {S10991, 87, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5299)) Presto/2.12.388 Version/12.15*/
const Device UA2707583 = {S10992, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5288)) Presto/2.12.388 Version/12.15*/
const Device UA2707866 = {S10993, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5278)) Presto/2.12.388 Version/12.15*/
const Device UA2707914 = {S10994, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03658)) Presto/2.12.388 Version/12.15*/
const Device UA2707935 = {S10995, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.15*/
const Device UA2708716 = {S10996, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.15*/
const Device UA2708810 = {S10997, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6035)) Presto/2.12.388 Version/12.15*/
const Device UA2709425 = {S10998, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03789)) Presto/2.12.388 Version/12.15*/
const Device UA2710432 = {S10999, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.15*/
const Device UA2711217 = {S11000, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.15*/
const Device UA2711495 = {S11001, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.15*/
const Device UA2712287 = {S11002, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6046)) Presto/2.12.388 Version/12.15*/
const Device UA2712880 = {S11003, 103, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.15,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2713516 = {S11004, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.15*/
const Device UA2714167 = {S11005, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.15*/
const Device UA2714219 = {S11006, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5996)) Presto/2.12.388 Version/12.15*/
const Device UA2714229 = {S11007, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5660)) Presto/2.12.388 Version/12.15*/
const Device UA2714572 = {S11008, 87, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.15*/
const Device UA2714578 = {S11009, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.15*/
const Device UA2714577 = {S11010, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.15*/
const Device UA2714808 = {S11011, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6032)) Presto/2.12.388 Version/12.15*/
const Device UA2714809 = {S11012, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6044)) Presto/2.12.388 Version/12.15*/
const Device UA2716518 = {S11013, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5650)) Presto/2.12.388 Version/12.15*/
const Device UA2717105 = {S11014, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6014)) Presto/2.12.388 Version/12.15*/
const Device UA2734822 = {S11015, 96, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.15,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2734957 = {S11016, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5230)) Presto/2.12.388 Version/12.15*/
const Device UA2735119 = {S11017, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5734)) Presto/2.12.388 Version/12.15*/
const Device UA2735184 = {S11018, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5988)) Presto/2.12.388 Version/12.15*/
const Device UA2735318 = {S11019, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4661)) Presto/2.12.388 Version/12.15*/
const Device UA2735427 = {S11020, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4832)) Presto/2.12.388 Version/12.15*/
const Device UA2741510 = {S11021, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6561)) Presto/2.12.388 Version/12.15*/
const Device UA2751573 = {S11022, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4110)) Presto/2.12.388 Version/12.15*/
const Device UA2751832 = {S11023, 87, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5300)) Presto/2.12.388 Version/12.15*/
const Device UA2752095 = {S11024, 87, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03639)) Presto/2.12.388 Version/12.15*/
const Device UA2753666 = {S11025, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6573)) Presto/2.12.388 Version/12.15*/
const Device UA2754145 = {S11026, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6261)) Presto/2.12.388 Version/12.15*/
const Device UA2754544 = {S11027, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6277)) Presto/2.12.388 Version/12.15*/
const Device UA2754595 = {S11028, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.15*/
const Device UA2767248 = {S11029, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.15*/
const Device UA2767897 = {S11030, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.6 (build 03402)) Presto/2.12.388 Version/12.15*/
const Device UA2768558 = {S11031, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03755)) Presto/2.12.388 Version/12.15*/
const Device UA2773430 = {S11032, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6283)) Presto/2.12.388 Version/12.15*/
const Device UA2780922 = {S11033, 87, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03791)) Presto/2.12.388 Version/12.15*/
const Device UA2781166 = {S11034, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03672)) Presto/2.12.388 Version/12.15*/
const Device UA2784746 = {S11035, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6628)) Presto/2.12.388 Version/12.15*/
const Device UA2794897 = {S11036, 87, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5336)) Presto/2.12.388 Version/12.15*/
const Device UA2795206 = {S11037, 107, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03686); MRSPUTNIK OW 2, 3, 0, 244) Presto/2.12.388 Version/12.15*/
const Device UA2797237 = {S11038, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6638)) Presto/2.12.388 Version/12.15*/
const Device UA2799503 = {S11039, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6620)) Presto/2.12.388 Version/12.15*/
const Device UA2861479 = {S11040, 87, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5161)) Presto/2.12.388 Version/12.15*/
const Device UA2863194 = {S11041, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03780)) Presto/2.12.388 Version/12.15*/
const Device UA2864727 = {S11042, 80, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5325)) Presto/2.12.388 Version/12.15*/
const Device UA2865040 = {S11043, 87, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.15*/
const Device UA2873423 = {S11044, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6323)) Presto/2.12.388 Version/12.15*/
const Device UA2874783 = {S11045, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.15*/
const Device UA2875562 = {S11046, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6322)) Presto/2.12.388 Version/12.15*/
const Device UA2876120 = {S11047, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6648)) Presto/2.12.388 Version/12.15*/
const Device UA2883651 = {S11048, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6325)) Presto/2.12.388 Version/12.15*/
const Device UA2887746 = {S11049, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6657)) Presto/2.12.388 Version/12.15*/
const Device UA2893060 = {S11050, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6320)) Presto/2.12.388 Version/12.15*/
const Device UA2893869 = {S11051, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6645)) Presto/2.12.388 Version/12.15*/
const Device UA2894799 = {S11052, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6327)) Presto/2.12.388 Version/12.15*/
const Device UA2894829 = {S11053, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6324)) Presto/2.12.388 Version/12.15*/
const Device UA2946869 = {S11054, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6665)) Presto/2.12.388 Version/12.15*/
const Device UA2955601 = {S11055, 107, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.6 (build 03402); MRSPUTNIK OW 2, 3, 0, 104) Presto/2.12.388 Version/12.15*/
const Device UA2955758 = {S11056, 106, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.15*/
const Device UA2963847 = {S11057, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6336)) Presto/2.12.388 Version/12.15*/
const Device UA2965394 = {S11058, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6666)) Presto/2.12.388 Version/12.15*/
const Device UA2966502 = {S11059, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6337)) Presto/2.12.388 Version/12.15*/
const Device UA2971089 = {S11060, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6669)) Presto/2.12.388 Version/12.15*/
const Device UA2972381 = {S11061, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6673)) Presto/2.12.388 Version/12.15*/
const Device UA2991437 = {S11062, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5730)) Presto/2.12.388 Version/12.15*/
const Device UA3015697 = {S11063, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6661)) Presto/2.12.388 Version/12.15*/
const Device UA3016136 = {S11064, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6667)) Presto/2.12.388 Version/12.15*/
const Device UA3017781 = {S11065, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6341)) Presto/2.12.388 Version/12.15*/
const Device UA3023417 = {S11066, 84, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.15,gzip(gfe),gzip(gfe)*/
const Device UA3023759 = {S11067, 77, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.15,gzip(gfe),gzip(gfe)*/
const Device UA3024189 = {S11068, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6675)) Presto/2.12.388 Version/12.15*/
const Device UA3036657 = {S11069, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.15*/
const Device UA3039387 = {S11070, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6681)) Presto/2.12.388 Version/12.15*/
const Device UA3058517 = {S11071, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6676)) Presto/2.12.388 Version/12.15*/
const Device UA3059056 = {S11072, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6347)) Presto/2.12.388 Version/12.15*/
const Device UA3077748 = {S11073, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6668)) Presto/2.12.388 Version/12.15*/
const Device UA3113256 = {S11074, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.2 (build 7188)) Presto/2.12.388 Version/12.15*/
const Device UA3125458 = {S11075, 86, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6682)) Presto/2.12.388 Version/12.15*/
const Device UA3128610 = {S11076, 79, { &P15364,&P7133,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.2 (build 7194)) Presto/2.12.388 Version/12.15*/
const Device UA90485 = {S11077, 103, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3 like Mac OS X; en_us) AppleWebKit/525.18.1 (KHTML, like Gecko )*/
const Device UA90570 = {S11078, 116, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3_1 like Mac OS X; fr-fr) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/7D11*/
const Device UA167931 = {S11079, 95, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA180440 = {S11080, 108, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7*/
const Device UA180456 = {S11081, 97, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA180505 = {S11082, 108, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H8*/
const Device UA180773 = {S11083, 108, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190*/
const Device UA181257 = {S11084, 97, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA182631 = {S11085, 108, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F191*/
const Device UA182850 = {S11086, 108, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4*/
const Device UA190951 = {S11087, 108, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2*/
const Device UA191838 = {S11088, 97, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA191839 = {S11089, 108, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3*/
const Device UA191883 = {S11090, 124, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/533.17.9*/
const Device UA208630 = {S11091, 122, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/528.16*/
const Device UA222941 = {S11092, 97, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA222996 = {S11093, 108, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2*/
const Device UA231665 = {S11094, 108, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1*/
const Device UA231992 = {S11095, 97, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA236777 = {S11096, 124, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/533.17.9*/
const Device UA251288 = {S11097, 139, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - MQQBrowser/18 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3*/
const Device UA251687 = {S11098, 139, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - MQQBrowser/18 Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3*/
const Device UA257556 = {S11099, 142, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - MQQBrowser/1.9.1 Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3*/
const Device UA305721 = {S11100, 122, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190 Safari/528.16*/
const Device UA306595 = {S11101, 124, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F191 Safari/533.17.9*/
const Device UA310583 = {S11102, 108, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F192*/
const Device UA311456 = {S11103, 124, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 Safari/533.17.9*/
const Device UA312496 = {S11104, 124, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 Safari/533.17.9*/
const Device UA314737 = {S11105, 124, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 Safari/533.17.9*/
const Device UA317171 = {S11106, 122, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/528.16*/
const Device UA317644 = {S11107, 122, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 Safari/528.16*/
const Device UA329882 = {S11108, 124, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2 Safari/533.17.9*/
const Device UA455565 = {S11109, 239, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3_5 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.5;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]*/
const Device UA603324 = {S11110, 239, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3_2 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.2;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]*/
const Device UA603502 = {S11111, 239, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.1;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]*/
const Device UA603565 = {S11112, 235, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]*/
const Device UA607894 = {S11113, 239, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3_4 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.4;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]*/
const Device UA616497 = {S11114, 107, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H*/
const Device UA616498 = {S11115, 107, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K*/
const Device UA616499 = {S11116, 107, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L*/
const Device UA624832 = {S11117, 124, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H8 Safari/533.17.9*/
const Device UA629644 = {S11118, 239, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3_3 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.3;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]*/
const Device UA642476 = {S11119, 124, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190 Safari/533.17.9*/
const Device UA648090 = {S11120, 142, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - MQQBrowser/1.9.1 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3*/
const Device UA706596 = {S11121, 139, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - MQQBrowser/19 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3*/
const Device UA783942 = {S11122, 122, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2 Safari/528.16*/
const Device UA789378 = {S11123, 133, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 iPad2,1/K93AP Zite/1.3.3*/
const Device UA843014 = {S11124, 133, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 iPad2,1/K93AP Zite/1.3.3*/
const Device UA1876161 = {S11125, 122, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 Safari/528.16*/
const Device UA1922277 = {S11126, 139, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - MQQBrowser/19 Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3*/
const Device UA1990234 = {S11127, 133, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 iPad1,1/K48AP Zite/1.3.3*/
const Device UA2141367 = {S11128, 110, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/9Z999*/
const Device UA2161600 = {S11129, 133, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F191 iPad2,1/K93AP Zite/1.3.3*/
const Device UA2203758 = {S11130, 227, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.2;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]*/
const Device UA2229706 = {S11131, 225, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPad1,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]*/
const Device UA2260731 = {S11132, 227, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.4;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]*/
const Device UA2279677 = {S11133, 122, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 Safari/528.16*/
const Device UA2302976 = {S11134, 162, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 4_3 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48 Instapaper/4.0.1 (+http://www.instapaper.com/iphone)*/
const Device UA2436519 = {S11135, 227, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPad1,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.3;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]*/
const Device UA2459350 = {S11136, 227, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.1;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]*/
const Device UA2511438 = {S11137, 225, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 [FBAN/FBIOS;FBAV/5.4.2;FBBV/X;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.5;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]*/
const Device UA2514531 = {S11138, 225, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 [FBAN/FBIOS;FBAV/5.4.2;FBBV/X;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.3;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]*/
const Device UA2531206 = {S11139, 223, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F191 [FBAN/FBIOS;FBAV/5.4.2;FBBV/X;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]*/
const Device UA2582694 = {S11140, 232, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H8 [FBAN/FBIOS;FBAV/5.4.2;FBBV/X;FBDV/iPad2,3;FBMD/iPad;FBSN/iPhone OS;FBSV/4.3.2;FBSS/1; FBCR/Verizon;FBID/tablet;FBLC/en_US]*/
const Device UA2594266 = {S11141, 98, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 4_3_5 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176*/
const Device UA2652983 = {S11142, 98, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 4_3_3 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176*/
const Device UA2710649 = {S11143, 105, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 433 like Mac OS X; en-US) AppleWebKit/533.17.9 (KHTML like Gecko) Mobile/8J2*/
const Device UA2735123 = {S11144, 105, { &P15767,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 435 like Mac OS X; en-US) AppleWebKit/533.17.9 (KHTML like Gecko) Mobile/8L1*/
const Device UA251960 = {S11145, 76, { &P15364,&P7480,&P18245,&P18092 }};	/*Useragent - AppEngine-Google; (+http://code.google.com/appengine; appid: s~getfavicon27)*/
const Device UA571141 = {S11146, 78, { &P15364,&P7480,&P18245,&P18092 }};	/*Useragent - AppEngine-Google; (+http://code.google.com/appengine; appid: tmobile-internet)*/
const Device UA577771 = {S11147, 69, { &P15364,&P7480,&P18245,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; GoogleDocs; script; +http://docs.google.com)*/
const Device UA649669 = {S11148, 73, { &P15364,&P7480,&P18245,&P18092 }};	/*Useragent - AppEngine-Google; (+http://code.google.com/appengine; appid: tmobile-w4z)*/
const Device UA900504 = {S11149, 135, { &P15364,&P7480,&P18245,&P18092 }};	/*Useragent - Public Hub (+http://pubsubhubbub.appspot.com; 1 subscribers) AppEngine-Google; (+http://code.google.com/appengine; appid: pubsubhubbub)*/
const Device UA1896884 = {S11150, 71, { &P15364,&P7480,&P18245,&P18092 }};	/*Useragent - AppEngine-Google; (+http://code.google.com/appengine; appid: vi-mobile)*/
const Device UA1964023 = {S11151, 89, { &P15364,&P7480,&P18245,&P18092 }};	/*Useragent - AppEngine-Google; (+http://code.google.com/appengine; appid: s~google.com:mobile-checker)*/
const Device UA1999452 = {S11152, 38, { &P15364,&P7480,&P18245,&P18092 }};	/*Useragent - GoogleProducer; (+http://goo.gl/7y4SX)*/
const Device UA102389 = {S11153, 112, { &P15364,&P7133,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA119737 = {S11154, 118, { &P15364,&P7133,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA151542 = {S11155, 118, { &P15364,&P7133,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA179501 = {S11156, 118, { &P15364,&P7133,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180951 = {S11157, 119, { &P15364,&P7133,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA184095 = {S11158, 120, { &P15364,&P7133,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA2163512 = {S11159, 113, { &P15364,&P7133,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA2204462 = {S11160, 111, { &P15364,&P7133,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2235279 = {S11161, 106, { &P15364,&P7133,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.26.9 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA2386845 = {S11162, 106, { &P15364,&P7133,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2387069 = {S11163, 99, { &P15364,&P7133,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2389172 = {S11164, 106, { &P15364,&P7133,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA2404040 = {S11165, 99, { &P15364,&P7133,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA2404660 = {S11166, 107, { &P15364,&P7133,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA2423988 = {S11167, 95, { &P15364,&P7133,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2425606 = {S11168, 99, { &P15364,&P7133,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2426737 = {S11169, 102, { &P15364,&P7133,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2427648 = {S11170, 100, { &P15364,&P7133,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA2428668 = {S11171, 106, { &P15364,&P7133,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2467374 = {S11172, 99, { &P15364,&P7133,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA2357970 = {S11173, 64, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.11*/
const Device UA2357976 = {S11174, 57, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.11*/
const Device UA2367745 = {S11175, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.11*/
const Device UA2367752 = {S11176, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.11*/
const Device UA2367768 = {S11177, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.11*/
const Device UA2367779 = {S11178, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.11*/
const Device UA2367920 = {S11179, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.11*/
const Device UA2367919 = {S11180, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.11*/
const Device UA2367934 = {S11181, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.11*/
const Device UA2368116 = {S11182, 69, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64) Opera 12.11*/
const Device UA2368135 = {S11183, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.11*/
const Device UA2368258 = {S11184, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.11*/
const Device UA2368259 = {S11185, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.11*/
const Device UA2368261 = {S11186, 74, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.11,gzip(gfe)*/
const Device UA2368269 = {S11187, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.11*/
const Device UA2368272 = {S11188, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.11*/
const Device UA2368276 = {S11189, 75, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.11*/
const Device UA2368578 = {S11190, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.11*/
const Device UA2368580 = {S11191, 87, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.11*/
const Device UA2368594 = {S11192, 67, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.11,gzip(gfe)*/
const Device UA2368595 = {S11193, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.11*/
const Device UA2369740 = {S11194, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.11*/
const Device UA2387216 = {S11195, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.11*/
const Device UA2387347 = {S11196, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.11*/
const Device UA2387535 = {S11197, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.11*/
const Device UA2388148 = {S11198, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.11*/
const Device UA2388150 = {S11199, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.11*/
const Device UA2388152 = {S11200, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.11*/
const Device UA2388154 = {S11201, 87, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.11*/
const Device UA2388156 = {S11202, 87, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.11*/
const Device UA2388158 = {S11203, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.11*/
const Device UA2388160 = {S11204, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.11*/
const Device UA2388161 = {S11205, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.11*/
const Device UA2388296 = {S11206, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.11*/
const Device UA2388300 = {S11207, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.11*/
const Device UA2388301 = {S11208, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.11*/
const Device UA2388308 = {S11209, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.11*/
const Device UA2388506 = {S11210, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.11*/
const Device UA2388508 = {S11211, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.11*/
const Device UA2389356 = {S11212, 87, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.11*/
const Device UA2389478 = {S11213, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.11*/
const Device UA2389485 = {S11214, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.11*/
const Device UA2389777 = {S11215, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.11*/
const Device UA2390388 = {S11216, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.11*/
const Device UA2390857 = {S11217, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.11*/
const Device UA2404434 = {S11218, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.11*/
const Device UA2404524 = {S11219, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.11*/
const Device UA2404722 = {S11220, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.11*/
const Device UA2405092 = {S11221, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.11*/
const Device UA2405100 = {S11222, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.11*/
const Device UA2405101 = {S11223, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.11*/
const Device UA2405441 = {S11224, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4933)) Presto/2.12.388 Version/12.11*/
const Device UA2405624 = {S11225, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.11*/
const Device UA2405626 = {S11226, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.11*/
const Device UA2405680 = {S11227, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.11*/
const Device UA2405799 = {S11228, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.11*/
const Device UA2405886 = {S11229, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.11*/
const Device UA2422733 = {S11230, 87, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03639)) Presto/2.12.388 Version/12.11*/
const Device UA2422964 = {S11231, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.11*/
const Device UA2423866 = {S11232, 87, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5217)) Presto/2.12.388 Version/12.11*/
const Device UA2424110 = {S11233, 74, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.11 Yandex.Translate*/
const Device UA2426508 = {S11234, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03658)) Presto/2.12.388 Version/12.11*/
const Device UA2426513 = {S11235, 87, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5288)) Presto/2.12.388 Version/12.11*/
const Device UA2426612 = {S11236, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5650)) Presto/2.12.388 Version/12.11*/
const Device UA2426719 = {S11237, 87, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5299)) Presto/2.12.388 Version/12.11*/
const Device UA2426741 = {S11238, 87, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5300)) Presto/2.12.388 Version/12.11*/
const Device UA2428027 = {S11239, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5734)) Presto/2.12.388 Version/12.11*/
const Device UA2428108 = {S11240, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03789)) Presto/2.12.388 Version/12.11*/
const Device UA2428201 = {S11241, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.11*/
const Device UA2429412 = {S11242, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.11*/
const Device UA2429418 = {S11243, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5230)) Presto/2.12.388 Version/12.11*/
const Device UA2430650 = {S11244, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.11*/
const Device UA2431038 = {S11245, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.11*/
const Device UA2437657 = {S11246, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.11*/
const Device UA2457308 = {S11247, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4110)) Presto/2.12.388 Version/12.11*/
const Device UA2457377 = {S11248, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4832)) Presto/2.12.388 Version/12.11*/
const Device UA2457467 = {S11249, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4661)) Presto/2.12.388 Version/12.11*/
const Device UA2462054 = {S11250, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.11*/
const Device UA2483013 = {S11251, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5336)) Presto/2.12.388 Version/12.11*/
const Device UA2536568 = {S11252, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.11*/
const Device UA2554578 = {S11253, 87, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.11*/
const Device UA2554846 = {S11254, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03780)) Presto/2.12.388 Version/12.11*/
const Device UA2554878 = {S11255, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03791)) Presto/2.12.388 Version/12.11*/
const Device UA2559464 = {S11256, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.11*/
const Device UA2563053 = {S11257, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.11*/
const Device UA2565803 = {S11258, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.11*/
const Device UA2565975 = {S11259, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.11*/
const Device UA2570194 = {S11260, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.11*/
const Device UA2596318 = {S11261, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.11*/
const Device UA2596854 = {S11262, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5660)) Presto/2.12.388 Version/12.11*/
const Device UA2599869 = {S11263, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.11*/
const Device UA2654616 = {S11264, 87, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5278)) Presto/2.12.388 Version/12.11*/
const Device UA2668515 = {S11265, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.11*/
const Device UA2670762 = {S11266, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.11*/
const Device UA2672286 = {S11267, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.11*/
const Device UA2673509 = {S11268, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.11*/
const Device UA2708002 = {S11269, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.11*/
const Device UA2717469 = {S11270, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.11*/
const Device UA2727992 = {S11271, 114, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03686); MRSPUTNIK OW 2, 3, 0, 244) Presto/2.12.388 Version/12.11*/
const Device UA2730159 = {S11272, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.11*/
const Device UA2769479 = {S11273, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.11*/
const Device UA2843058 = {S11274, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.6 (build 03402)) Presto/2.12.388 Version/12.11*/
const Device UA2843215 = {S11275, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03755)) Presto/2.12.388 Version/12.11*/
const Device UA2843349 = {S11276, 80, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03672)) Presto/2.12.388 Version/12.11*/
const Device UA2843794 = {S11277, 78, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 198)) Presto/2.12.388 Version/12.11*/
const Device UA2859959 = {S11278, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6080)) Presto/2.12.388 Version/12.11*/
const Device UA2864458 = {S11279, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.11*/
const Device UA2865023 = {S11280, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.11*/
const Device UA3005096 = {S11281, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6638)) Presto/2.12.388 Version/12.11*/
const Device UA3006467 = {S11282, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6657)) Presto/2.12.388 Version/12.11*/
const Device UA3009229 = {S11283, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.11*/
const Device UA3021615 = {S11284, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6673)) Presto/2.12.388 Version/12.11*/
const Device UA3059568 = {S11285, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.11*/
const Device UA3078487 = {S11286, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6337)) Presto/2.12.388 Version/12.11*/
const Device UA3086724 = {S11287, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6046)) Presto/2.12.388 Version/12.11*/
const Device UA3103203 = {S11288, 79, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6573)) Presto/2.12.388 Version/12.11*/
const Device UA3103420 = {S11289, 86, { &P15364,&P7133,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6628)) Presto/2.12.388 Version/12.11*/
const Device UA622430 = {S11290, 143, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-N7000 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30*/
const Device UA622429 = {S11291, 145, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; GT-N7000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.1 Mobile Safari/534.30*/
const Device UA654182 = {S11292, 139, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-N7000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA680098 = {S11293, 61, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-N7000 Build/IML74K)*/
const Device UA715673 = {S11294, 139, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T879 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA722362 = {S11295, 139, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA854191 = {S11296, 140, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; SHV-E160S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA896018 = {S11297, 140, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; SHV-E160K Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA896360 = {S11298, 140, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; SHV-E160L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA898467 = {S11299, 139, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T879 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA907875 = {S11300, 158, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; SAMSUNG GT-N7000/N7000XXLQ3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA909553 = {S11301, 140, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; GT-N7000B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA923932 = {S11302, 61, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-N7000 Build/IMM76D)*/
const Device UA1960093 = {S11303, 134, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; N7000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1960962 = {S11304, 139, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-N7000 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1962066 = {S11305, 130, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-N7000 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1990033 = {S11306, 62, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SHV-E160K Build/IML74K)*/
const Device UA1991916 = {S11307, 62, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SHV-E160L Build/IML74K)*/
const Device UA1994113 = {S11308, 62, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SHV-E160S Build/IML74K)*/
const Device UA1994659 = {S11309, 133, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHW-M480K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1996488 = {S11310, 158, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; SAMSUNG GT-N7000/N7000BULP3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1997039 = {S11311, 158, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SAMSUNG GT-N7000/N7000XXLRI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1998753 = {S11312, 140, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; GT-N7000B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2008384 = {S11313, 140, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E160S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026091 = {S11314, 125, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-N7000 Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2026200 = {S11315, 137, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-N7000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027449 = {S11316, 130, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-N7000 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2027872 = {S11317, 133, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHW-M480W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032024 = {S11318, 139, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SCH-i889 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033069 = {S11319, 133, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; N7000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033102 = {S11320, 61, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-T879 Build/IMM76D)*/
const Device UA2040607 = {S11321, 140, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E160K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2041194 = {S11322, 137, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SC-05D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2159582 = {S11323, 133, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHW-M480S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2179085 = {S11324, 137, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2191883 = {S11325, 62, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHV-E160S Build/IMM76D)*/
const Device UA2201664 = {S11326, 140, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E160L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2203403 = {S11327, 145, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30*/
const Device UA2223962 = {S11328, 137, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N7000+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2242334 = {S11329, 62, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-N7000B Build/IMM76D)*/
const Device UA2346639 = {S11330, 62, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHV-E160K Build/IMM76D)*/
const Device UA2375166 = {S11331, 139, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-N7000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2400261 = {S11332, 62, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHV-E160L Build/IMM76D)*/
const Device UA2408017 = {S11333, 158, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SAMSUNG GT-N7000/N7000XXLRU Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426059 = {S11334, 158, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-ch; SAMSUNG GT-N7000/N7000BULR1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2449100 = {S11335, 134, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; N7000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2454855 = {S11336, 138, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-N7000B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2502358 = {S11337, 59, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SC-05D Build/IMM76D)*/
const Device UA2526181 = {S11338, 130, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SCH-i889 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2745159 = {S11339, 62, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHW-M480W Build/IMM76D)*/
const Device UA2775787 = {S11340, 145, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; vi-vn; N7000 Build/DragonFly_V006EnX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2829296 = {S11341, 145, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; vi-vn; N7000 Build/DragonFly_V005EnX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2834753 = {S11342, 146, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-N7000 Build/RocketMIUI v8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2849559 = {S11343, 145, { &P16356,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; N7000 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA353150 = {S11344, 86, { &P17968,&P17979,&P17373,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/6.5.27452/26.1305; U; en) Presto/2.8.119 Version/10.54*/
const Device UA2431158 = {S11345, 86, { &P17968,&P17979,&P17373,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/6.5.27452/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2708988 = {S11346, 88, { &P17968,&P17979,&P17373,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/6.5.29441/27.2314; U; en) Presto/2.8.119 480x800 Android*/
const Device UA2675556 = {S11347, 140, { &P15364,&P5690,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2676973 = {S11348, 108, { &P15364,&P5690,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2678356 = {S11349, 97, { &P15364,&P5690,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.1.3.0 Safari/537.31*/
const Device UA2678393 = {S11350, 101, { &P15364,&P5690,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2678676 = {S11351, 111, { &P15364,&P5690,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe)*/
const Device UA2679011 = {S11352, 108, { &P15364,&P5690,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2679137 = {S11353, 101, { &P15364,&P5690,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2679930 = {S11354, 97, { &P15364,&P5690,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.2.2.0 Safari/537.31*/
const Device UA2712675 = {S11355, 104, { &P15364,&P5690,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.2.2.0 Safari/537.31*/
const Device UA2719148 = {S11356, 108, { &P15364,&P5690,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; en-US) AppleWebKit/535.12 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/535.12*/
const Device UA2736901 = {S11357, 111, { &P15364,&P5690,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe)*/
const Device UA2752467 = {S11358, 97, { &P15364,&P5690,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.2.2.0 Safari/537.31*/
const Device UA2865760 = {S11359, 108, { &P15364,&P5690,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; zh-CN) AppleWebKit/535.12 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/535.12*/
const Device UA2674785 = {S11360, 158, { &P17595,&P18207,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2678583 = {S11361, 119, { &P17595,&P18207,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA2678584 = {S11362, 119, { &P17595,&P18207,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA2678597 = {S11363, 119, { &P17595,&P18207,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA2678736 = {S11364, 119, { &P17595,&P18207,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA2678740 = {S11365, 119, { &P17595,&P18207,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA2679582 = {S11366, 129, { &P17595,&P18207,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31,gzip(gfe)*/
const Device UA2679984 = {S11367, 129, { &P17595,&P18207,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31,gzip(gfe)*/
const Device UA2707368 = {S11368, 158, { &P17595,&P18207,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2936959 = {S11369, 118, { &P17595,&P18207,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.31 (KHTML like Gecko) Chrome/26.0.1410.63 Safari/537.31*/
const Device UA3075457 = {S11370, 119, { &P17595,&P18207,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA227992 = {S11371, 93, { &P15364,&P7133,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50*/
const Device UA228376 = {S11372, 100, { &P15364,&P7133,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50*/
const Device UA250865 = {S11373, 101, { &P15364,&P7133,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22*/
const Device UA251008 = {S11374, 108, { &P15364,&P7133,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22*/
const Device UA428869 = {S11375, 108, { &P15364,&P7133,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16*/
const Device UA428872 = {S11376, 101, { &P15364,&P7133,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16*/
const Device UA441021 = {S11377, 99, { &P15364,&P7133,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3*/
const Device UA441022 = {S11378, 106, { &P15364,&P7133,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3*/
const Device UA615809 = {S11379, 106, { &P15364,&P7133,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2*/
const Device UA615815 = {S11380, 99, { &P15364,&P7133,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2*/
const Device UA622530 = {S11381, 99, { &P15364,&P7133,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7*/
const Device UA622542 = {S11382, 106, { &P15364,&P7133,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7*/
const Device UA733034 = {S11383, 116, { &P15364,&P7133,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-us) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1 Safari/534.52.7*/
const Device UA775838 = {S11384, 118, { &P15364,&P7133,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-us) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.6 Safari/534.57.2*/
const Device UA795038 = {S11385, 112, { &P15364,&P7133,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-us) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50*/
const Device UA806464 = {S11386, 118, { &P15364,&P7133,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-us) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1 Safari/534.51.22*/
const Device UA1990921 = {S11387, 118, { &P15364,&P7133,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-us) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3*/
const Device UA2167510 = {S11388, 108, { &P15364,&P7133,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.1) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2*/
const Device UA2586541 = {S11389, 107, { &P15364,&P7133,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.3 Safari/534.53.10*/
const Device UA2586546 = {S11390, 100, { &P15364,&P7133,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.3 Safari/534.53.10*/
const Device UA389494 = {S11391, 158, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Superfeedr: Superparser bot/1.1 http://superfeedr.com - Please read this http://blog.superfeedr.com/publishers.html or get in touch if we are polling too hard*/
const Device UA418834 = {S11392, 45, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 (UNIX; I; amd64) DynoBot/0.8.2528*/
const Device UA454025 = {S11393, 82, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Dataprovider Site Explorer; http://www.dataprovider.com/)*/
const Device UA625506 = {S11394, 62, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - UniversalFeedParser/5.1 +https://code.google.com/p/feedparser/*/
const Device UA896684 = {S11395, 64, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - UniversalFeedParser/5.1.2 +https://code.google.com/p/feedparser/*/
const Device UA2005295 = {S11396, 102, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Xenu Link Sleuth/1.3.8 AppEngine-Google; (+http://code.google.com/appengine; appid: s~runningcheese01)*/
const Device UA2681873 = {S11397, 81, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; DynamicTemplates/2.0; http://www.sli-systems.com/) Gecko*/
const Device UA2898639 = {S11398, 82, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Superfeedr bot/2.0 http://superfeedr.com - Make your feeds realtime: get in touch!*/
const Device UA2944037 = {S11399, 63, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - GG PeekBot 2.0 ( http://gg.pl/ http://info.gadu-gadu.pl/praca )*/
const Device UA2946465 = {S11400, 20, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Ara.com.tr / Bot 1.0*/
const Device UA2947336 = {S11401, 46, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - PollBot v1.2 (http://besome1.info/robots.html)*/
const Device UA2948279 = {S11402, 77, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; linkdexbot/2.0; +http://www.linkdex.com/about/bots/)*/
const Device UA2949192 = {S11403, 19, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/xintellibot*/
const Device UA2950710 = {S11404, 64, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; scanbot/1.0; +http://dazzlepod.com/ip/)*/
const Device UA2955180 = {S11405, 39, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - MaxPoint Bot (+http://www.maxpoint.com)*/
const Device UA2957088 = {S11406, 25, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Sogou web spider/4.0?iVFA*/
const Device UA2957292 = {S11407, 63, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SafeLinks Bot; http://safelinkshq.com)*/
const Device UA2957422 = {S11408, 49, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Nitidum Tarsier/1.0 (+http://www.nitidum.com/bot)*/
const Device UA2957606 = {S11409, 54, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - DistSearchBot/dev (http://www.tkolb.de/distsearch.txt)*/
const Device UA2968016 = {S11410, 30, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/2.20*/
const Device UA2971954 = {S11411, 90, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; WebDataCentreBot/1.0; +http://WebDataCentre.com/;c2VvLWNyYXdsZXI=*/
const Device UA2972444 = {S11412, 30, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Screaming Frog SEO Spider/2,20*/
const Device UA2973637 = {S11413, 67, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; DonkeyBot/0.3a; +http://www.dasdonkey.com)*/
const Device UA2974205 = {S11414, 73, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SeznamBot/3.1-test1; +http://fulltext.sblog.cz/)*/
const Device UA2974855 = {S11415, 37, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - EhName Domain Registry Protection Bot*/
const Device UA2976151 = {S11416, 110, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - NCBot (http://netcomber.com?st=ba : tool for finding true domain owners) Queries/complaints: bot@netcomber.com*/
const Device UA2978568 = {S11417, 65, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - SocialOomphBot/1.0 (info@socialoomph.com http://socialoomph.com/)*/
const Device UA2979535 = {S11418, 44, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Trackybot/0.1 (+https://tracky.com/bot.html)*/
const Device UA2979774 = {S11419, 69, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Exabot/3.0;  http://www.exabot.com/go/robot)*/
const Device UA2982858 = {S11420, 48, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - OpenWebSpider/0.7 (http://www.openwebspider.org)*/
const Device UA2988101 = {S11421, 70, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Siteliner/1.0; +http://www.siteliner.com/bot)*/
const Device UA2989170 = {S11422, 82, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - AppEngine-Google; (+http://code.google.com/appengine; appid: s~socialmediacrawler)*/
const Device UA2990811 = {S11423, 73, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexMetrika/2.0; +http://yandex.com/bots mon2)*/
const Device UA2990870 = {S11424, 73, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexMetrika/2.0; +http://yandex.com/bots mon1)*/
const Device UA2994418 = {S11425, 28, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Clickagy Ad Intelligence Bot*/
const Device UA2999362 = {S11426, 49, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - datagnionbot (+http://www.datagnion.com/bot.html)*/
const Device UA2999792 = {S11427, 72, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Ephorusbot/undefined; +http://www.ephorus.com/)*/
const Device UA3000058 = {S11428, 79, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; PerfectMarketKWTbot/1.0 +http://www.perfectmarket.com)*/
const Device UA3003075 = {S11429, 73, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; equellaurlbot/1.0; +http://support.equella.com/)*/
const Device UA3004301 = {S11430, 93, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - A1 Website Download/4.2.4 (+http://www.microsystools.com/products/website-download/) miggibot*/
const Device UA3018310 = {S11431, 55, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - CatchBot/2.0; +http://www.catchbot.com;c2VvLWNyYXdsZXI=*/
const Device UA3018314 = {S11432, 85, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; seexie.com_bot/4.1; +http://www.seexie.com);c2VvLWNyYXdsZXI=*/
const Device UA3018315 = {S11433, 55, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - CatchBot/1.0; +http://www.catchbot.com;c2VvLWNyYXdsZXI=*/
const Device UA3018431 = {S11434, 114, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Tokenizer/1.1.19 (Shopping Search Engine; http://www.tokenizer.org/bot.html; agent@tokenizer.org);c2VvLWNyYXdsZXI=*/
const Device UA3018885 = {S11435, 63, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Snapbot/1.0 (Snap Shots, +http://www.snap.com);c2VvLWNyYXdsZXI=*/
const Device UA3019098 = {S11436, 66, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - PageBitesHyperBot/600 (http://www.pagebites.com/);c2VvLWNyYXdsZXI=*/
const Device UA3020458 = {S11437, 6, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - finbot*/
const Device UA3020663 = {S11438, 85, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; lookingforitbot/v1.0; http://www.lookingforit.net/bot.php?+)*/
const Device UA3023725 = {S11439, 8, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Abyssbot*/
const Device UA3024159 = {S11440, 70, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; DCPbot/1.5; +http://domains.checkparams.com/)*/
const Device UA3024233 = {S11441, 69, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Kzjnetbot/Kzjnetbot-0.1 (http://search.kzjnet.com; search@kzjnet.com)*/
const Device UA3027916 = {S11442, 63, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - SupremeSearch.net Spider-Bot/5.4 (http://www.supremesearch.net)*/
const Device UA3028864 = {S11443, 73, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - NCBot http://netcomber.com?st=ba2Tool for finding all their domain names.*/
const Device UA3028865 = {S11444, 11, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - ht2p-bot/v1*/
const Device UA3029963 = {S11445, 75, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - EhName Domain Portfolio Bot http://ehname.com/DOMAIN?st=ba robot@ehname.com*/
const Device UA3030203 = {S11446, 77, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SemrushBot/0.96.4; +http://www.semrush.com/bot.html)*/
const Device UA3032691 = {S11447, 117, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - NCBot (http://netcomber.com/DOMAIN?st=ba : tool for finding true domain owners) Queries/complaints: bot@netcomber.com*/
const Device UA3034061 = {S11448, 9, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - ManziaBot*/
const Device UA3034219 = {S11449, 12, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - TitleBot-1.0*/
const Device UA3034672 = {S11450, 74, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - NCBot http://netcomber.com?st=ba2 Tool for finding all their domain names.*/
const Device UA3036243 = {S11451, 17, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - ContextAd Bot 1.0*/
const Device UA3036300 = {S11452, 71, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MojeekBot/0.5; http://www.mojeek.com/bot.html)*/
const Device UA3039301 = {S11453, 63, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - SupremeSearch.net Spider-Bot/5.5 (http://www.supremesearch.net)*/
const Device UA3041946 = {S11454, 11, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Test spider*/
const Device UA3043784 = {S11455, 41, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; openstat.ru/Bot)*/
const Device UA3046598 = {S11456, 10, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Spider/Bot*/
const Device UA3047437 = {S11457, 25, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Sogou web spider/4.02525A*/
const Device UA3048320 = {S11458, 25, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Sogou web spider/4.05252A*/
const Device UA3051449 = {S11459, 47, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - semetrical (http://www.semetrical.com/bot.html)*/
const Device UA3052551 = {S11460, 64, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; seexie.com_bot/4.1; +http://seexie.com)*/
const Device UA3056936 = {S11461, 77, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - GozaikBot (www.gozaik.com;webmaster@gozaik.com;www.gozaik.com/gozaikbot.html)*/
const Device UA3057544 = {S11462, 101, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YodaoBot/1.0; http://www.yodao.com/help/webmaster/spider/;);c2VvLWNyYXdsZXI=*/
const Device UA3057678 = {S11463, 59, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Baiduspider+(+http://www.baidu.jp/spider/);c2VvLWNyYXdsZXI=*/
const Device UA3057677 = {S11464, 45, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Open Web Analytics Bot 1.2.4;c2VvLWNyYXdsZXI=*/
const Device UA3057926 = {S11465, 97, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; discobot/1.1; +http://discoveryengine.com/discobot.html;c2VvLWNyYXdsZXI=*/
const Device UA3057939 = {S11466, 95, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; 008/0.83; http://www.80legs.com/spider.html;) Gecko/X;c2VvLWNyYXdsZXI=*/
const Device UA3057946 = {S11467, 61, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - lobot/0.3 (http://www.lofoso.com/robot.html);c2VvLWNyYXdsZXI=*/
const Device UA3057950 = {S11468, 65, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - CCBot/1.0 (+http://www.commoncrawl.org/bot.html);c2VvLWNyYXdsZXI=*/
const Device UA3057954 = {S11469, 82, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - MoFuseBot (Feed Parser; http://bot.mofuse.com/; Allow like Gecko);c2VvLWNyYXdsZXI=*/
const Device UA3057958 = {S11470, 54, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - CatchBot/1.0; http://www.catchbot.com;c2VvLWNyYXdsZXI=*/
const Device UA3057960 = {S11471, 107, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - TrueKnowledgeBot (http://www.trueknowledge.com/tkbot/; tkbot -AT- trueknowledge _dot_ com);c2VvLWNyYXdsZXI=*/
const Device UA3057962 = {S11472, 100, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Baiduspider/2.0; +http://www.baidu.com/search/spider.html);c2VvLWNyYXdsZXI=*/
const Device UA3061687 = {S11473, 85, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - [('User-agent', 'Mozilla/5.0 (compatible; ZumBot/1.0; http://help.zum.com/inquiry)')]*/
const Device UA3062711 = {S11474, 26, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - ElisaBot 1.0 [u-f-g-table]*/
const Device UA3067506 = {S11475, 75, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SemrushBot/0.97; +http://www.semrush.com/bot.html)*/
const Device UA3069940 = {S11476, 68, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexMetrika/3.0; +http://yandex.com/bots)*/
const Device UA3070387 = {S11477, 22, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Contensive-Bot/3.0.150*/
const Device UA3070662 = {S11478, 72, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; archive-nz.com/1.1; +http://archive-nz.com/bot)*/
const Device UA3074499 = {S11479, 63, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ReplazBot/2.5; +http://www.replaz.com)*/
const Device UA3077696 = {S11480, 101, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Wepbot/3.0 (+http://www.nurago.com/index.php?id=210); Jakarta Commons-HttpClient/3.1;c2VvLWNyYXdsZXI=*/
const Device UA3077892 = {S11481, 51, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Nekstbot - http://www.ipipan.waw.pl/nekst/nekstbot/*/
const Device UA3078093 = {S11482, 58, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - SolomonoBot/1.01 (http://www.solomono.ru);c2VvLWNyYXdsZXI=*/
const Device UA3078098 = {S11483, 63, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - msnbot/1.1 (+http://search.msn.com/msnbot.htm);c2VvLWNyYXdsZXI=*/
const Device UA3078101 = {S11484, 40, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - EvaalSE - bot@evaal.com;c2VvLWNyYXdsZXI=*/
const Device UA3078114 = {S11485, 85, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Purebot/1.1; +http://www.puritysearch.net/);c2VvLWNyYXdsZXI=*/
const Device UA3078124 = {S11486, 40, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Checkbot/1.79 LWP/5.813;c2VvLWNyYXdsZXI=*/
const Device UA3078125 = {S11487, 64, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - msnbot/2.0b (+http://search.msn.com/msnbot.htm);c2VvLWNyYXdsZXI=*/
const Device UA3078162 = {S11488, 88, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; BuiltWith/0.3; +http://builtwith.com/bot.html);c2VvLWNyYXdsZXI=*/
const Device UA3080775 = {S11489, 101, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - searchdotsh/Search.sh-2.2.1 (search.sh spider ver.0.5; http://search.sh/; webmaster at search dot sh)*/
const Device UA3084001 = {S11490, 70, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YaDirectFetcher/1.0; +http://yandex.com/bots)*/
const Device UA3084033 = {S11491, 73, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; WMHelper/1.0; +http://www.wmhelper.com/bot.html)*/
const Device UA3084036 = {S11492, 91, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - searchdotsh/0.5.0 (search.sh spider ver.0.5; http://search.sh/; webmaster at search dot sh)*/
const Device UA3084707 = {S11493, 39, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; YandexBot/3.0)*/
const Device UA3091855 = {S11494, 86, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; seospyder/1.0; +http://www.mobiliodevelopment.com/seospyder/)*/
const Device UA3092622 = {S11495, 33, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Cliqz Bot (+http://www.cliqz.com)*/
const Device UA3093700 = {S11496, 70, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; spbot/4.0a; +http://www.seoprofiler.com/bot )*/
const Device UA3095033 = {S11497, 85, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; uMBot-IC/1.0; mailto: crawling@ubermetrics-technologies.com)*/
const Device UA3097580 = {S11498, 45, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - mediawords bot (http://cyber.law.harvard.edu)*/
const Device UA3097957 = {S11499, 7, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - gigabot*/
const Device UA3099696 = {S11500, 75, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Gaisbot/3.0 (robot@gais.cs.ccu.edu.tw; http://gais.cs.ccu.edu.tw/robot.php)*/
const Device UA3109293 = {S11501, 26, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - ElisaBot 1.0 unforgettable*/
const Device UA3109525 = {S11502, 56, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Josh Robot/2.0 (http://www.joshtsp.com; tim@timgill.com)*/
const Device UA3112831 = {S11503, 52, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Cliqzbot/0.1 (+http://cliqz.com +cliqzbot@cliqz.com)*/
const Device UA3114123 = {S11504, 89, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Please Name Your robot; +http://broadcart.com/websearch/bot.php)*/
const Device UA3116658 = {S11505, 72, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - HTTP/1.0 Mozilla/5.0+(compatible;+GetInfo.us+Bot;+http://getinfo.us/bot)*/
const Device UA3116884 = {S11506, 75, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Gulper Web Bot 0.2.4 (www.ecsl.cs.sunysb.edu/~maxim/cgi-bin/Link/GulperBot)*/
const Device UA3118473 = {S11507, 90, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Halebot (Mozilla/5.0 compatible; Halebot/2.1; http://www.tacitknowledge.com/halebot.shtml)*/
const Device UA3118826 = {S11508, 38, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - WebZIP/3.5 (http://www.spidersoft.com)*/
const Device UA3129745 = {S11509, 63, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; ReplazBot/2.6; +http://www.replaz.com)*/
const Device UA3130577 = {S11510, 70, { &P15364,&P7480,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; spbot/4.0b; +http://www.seoprofiler.com/bot )*/
const Device UA30021 = {S11511, 151, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5*/
const Device UA30054 = {S11512, 150, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA30062 = {S11513, 144, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA30071 = {S11514, 150, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9Z999 Safari/6533.18.5*/
const Device UA46547 = {S11515, 149, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C99 Safari/6533.18.5*/
const Device UA46614 = {S11516, 148, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5*/
const Device UA149907 = {S11517, 148, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2 like Mac OS X; nl-nl) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 Safari/6533.18.5*/
const Device UA169423 = {S11518, 150, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8E200 Safari/6533.18.5*/
const Device UA170449 = {S11519, 150, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8E128 Safari/6533.18.5*/
const Device UA174861 = {S11520, 135, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU OS 4_2_1 like Mac OS X) AppleWebKit/532.9 (KHTML, like Gecko) Version/5.0.3 Mobile/8B5097d Safari/6531.22.7*/
const Device UA180151 = {S11521, 150, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_7 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8E303 Safari/6533.18.5*/
const Device UA188854 = {S11522, 150, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9Z999 Safari/6533.18.5*/
const Device UA188909 = {S11523, 150, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8E401 Safari/6533.18.5*/
const Device UA222936 = {S11524, 150, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_9 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8E501 Safari/6533.18.5*/
const Device UA231610 = {S11525, 151, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_10 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8E600 Safari/6533.18.5*/
const Device UA302634 = {S11526, 150, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (device; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA603984 = {S11527, 137, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; 6533.17.9) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5*/
const Device UA630722 = {S11528, 136, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en_US) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5*/
const Device UA655628 = {S11529, 150, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9Z999 Safari/6533.18.5*/
const Device UA808878 = {S11530, 152, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en_us) AppleWebKit/6533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5*/
const Device UA1858267 = {S11531, 150, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; vi-vn) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/PV001 Safari/6533.18.5*/
const Device UA1871274 = {S11532, 144, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8E200 Safari/6533.18.5*/
const Device UA2157671 = {S11533, 152, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en_us) AppleWebKit/6533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5*/
const Device UA2303318 = {S11534, 131, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 4_2_1 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0 Mobile/8C148 Safari/7533.17.9*/
const Device UA2321067 = {S11535, 152, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en_us) AppleWebKit/6533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5*/
const Device UA2639581 = {S11536, 144, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone Simulator; U; CPU iPhone OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C134 Safari/6533.18.5*/
const Device UA2659751 = {S11537, 147, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 421 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA2663878 = {S11538, 147, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 426 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8E200 Safari/6533.18.5*/
const Device UA2698213 = {S11539, 148, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 421 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5*/
const Device UA2702567 = {S11540, 147, { &P12280,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 427 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8E303 Safari/6533.18.5*/
const Device UA247170 = {S11541, 99, { &P15364,&P7133,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.163 Safari/535.1*/
const Device UA249762 = {S11542, 106, { &P15364,&P7133,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA343072 = {S11543, 145, { &P15364,&P7133,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA343332 = {S11544, 138, { &P15364,&P7133,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA675304 = {S11545, 160, { &P15364,&P7133,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.802.30 Safari/535.1 SE 2.X MetaSr 1.0,gzip(gfe) (via translate.google.com)*/
const Device UA699364 = {S11546, 153, { &P15364,&P7133,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.802.30 Safari/535.1 SE 2.X MetaSr 1.0,gzip(gfe) (via translate.google.com)*/
const Device UA744854 = {S11547, 110, { &P15364,&P7133,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.94 Safari/535.1*/
const Device UA2152442 = {S11548, 85, { &P15364,&P7133,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535 (KHTML, like Gecko) Chrome/14 Safari/535*/
const Device UA2233988 = {S11549, 126, { &P15364,&P7133,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.802.30 Safari/535.1 SE 2.X MetaSr 1.0,gzip(gfe)*/
const Device UA2259197 = {S11550, 133, { &P15364,&P7133,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.802.30 Safari/535.1 SE 2.X MetaSr 1.0,gzip(gfe)*/
const Device UA2305499 = {S11551, 118, { &P15364,&P7133,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1*/
const Device UA2364699 = {S11552, 123, { &P15364,&P7133,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML	 like Gecko) Chrome/14.0.802.30 Safari/535.1 SE 2.X MetaSr 1.0*/
const Device UA2980507 = {S11553, 103, { &P15364,&P7133,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML like Gecko) Chrome/14.0.811.0 Safari/535.1*/
const Device UA2980698 = {S11554, 96, { &P15364,&P7133,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML like Gecko) Chrome/14.0.812.0 Safari/535.1*/
const Device UA5237 = {S11555, 139, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Desire_A8181 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA5247 = {S11556, 146, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Android 2.2; en-gb; Desire_A8181 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11207 = {S11557, 144, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC_Desire_A8181 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA12557 = {S11558, 147, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; de-at; HTC Desire 1.34.166.5 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA14656 = {S11559, 139, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Desire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20554 = {S11560, 136, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20863 = {S11561, 138, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; nl-nl; Desire_A8181 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA21355 = {S11562, 131, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; Desire_A8181 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA28420 = {S11563, 146, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC_Desire-orange-LS Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA29763 = {S11564, 225, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTCA8180/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA29780 = {S11565, 147, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; de-at; HTC Desire 2.33.163.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA29790 = {S11566, 147, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Desire 1.19.161.5 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA29810 = {S11567, 147, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Desire 2.10.161.2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA29843 = {S11568, 135, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Bravo Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA29849 = {S11569, 147, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; de-at; HTC Desire 2.10.163.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA29841 = {S11570, 136, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC_Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32112 = {S11571, 131, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-jp; X06HT Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32113 = {S11572, 134, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; HTCX06HT Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32122 = {S11573, 154, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-au; HTC_Desire_A8183 V1.16.841.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32125 = {S11574, 154, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-au; HTC_Desire_A8183 V1.25.841.5 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32132 = {S11575, 157, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; HTC_Desire_A8183 V2.26.841.2 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32124 = {S11576, 154, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-au; HTC_Desire_A8183 V2.26.841.2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA36939 = {S11577, 141, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; Desire_A8181 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA36942 = {S11578, 139, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; HTC Desire Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA54006 = {S11579, 137, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; HTC Bravo_C Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61816 = {S11580, 145, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC_Desire_A8181 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA63892 = {S11581, 147, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Desire 2.33.161.2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA64039 = {S11582, 142, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-hk; HTC_Desire_A8181 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67583 = {S11583, 168, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Desire Build/FRF91) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-20101109)*/
const Device UA122406 = {S11584, 60, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; HTC Desire Build/FRF91)*/
const Device UA140727 = {S11585, 138, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA154469 = {S11586, 143, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Desire CDMA Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA155768 = {S11587, 138, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Desire Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA160440 = {S11588, 137, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC bravo Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA359267 = {S11589, 150, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; de-at; HTC Desire 2.33.163.1 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA361242 = {S11590, 150, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; HTC Desire 2.33.161.2 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA396572 = {S11591, 63, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; HTC Desire Build/FRG83G)*/
const Device UA398041 = {S11592, 145, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; HTC_Desire_A8181 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA494895 = {S11593, 134, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA501433 = {S11594, 144, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Android 2.2; en-gb; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA572920 = {S11595, 124, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Desire Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA622989 = {S11596, 138, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; Desire_A8181 Build/FRF91) App3leWebKit/53.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA630302 = {S11597, 150, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; HTC Desire 2.33.161.6 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA630626 = {S11598, 138, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; HTC Bravo Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA649899 = {S11599, 139, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; HTC_Desire Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA701724 = {S11600, 150, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; de-at; HTC Desire 2.33.163.5 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA754088 = {S11601, 63, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; HTC Desire Build/FRG83D)*/
const Device UA1965244 = {S11602, 129, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Desire Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2018156 = {S11603, 55, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; X06HT Build/FRF91)*/
const Device UA2018253 = {S11604, 137, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; HTC Desire Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019408 = {S11605, 141, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; HTC Desire CDMA Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021381 = {S11606, 137, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-; HTC Desire Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024383 = {S11607, 127, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HTC Desire Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2425697 = {S11608, 150, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; nl-nl; HTC Desire 2.33.161.8 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2686813 = {S11609, 54, { &P11747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Android 2.2; zh-cn; HTC Desire)/GoBrowser*/
const Device UA92479 = {S11610, 71, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1; http://www.google.com/bot.html)*/
const Device UA93027 = {S11611, 72, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)*/
const Device UA94271 = {S11612, 9, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - googlebot*/
const Device UA97960 = {S11613, 72, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1;  http://www.google.com/bot.html)*/
const Device UA98129 = {S11614, 47, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot/2.1 (+http://www.google.com/bot.html)*/
const Device UA98993 = {S11615, 19, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot-Image/1.0*/
const Device UA100103 = {S11616, 72, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0+(compatible;+Googlebot/2.1;++http://www.google.com/bot.html)*/
const Device UA100442 = {S11617, 50, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot/2.1 (+http://www.googlebot.com/bot.html)*/
const Device UA103665 = {S11618, 49, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot/2.1 (http://www.googlebot.com/bot.html)*/
const Device UA145003 = {S11619, 58, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot-Image/1.0,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA154904 = {S11620, 53, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/4.0 (compatible; gallery_203.nasl; Googlebot)*/
const Device UA157679 = {S11621, 111, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html),gzip(gfe) (via docs.google.com/viewer)*/
const Device UA172279 = {S11622, 19, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot-Video/1.0*/
const Device UA188498 = {S11623, 22, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot-richsnippets*/
const Device UA191167 = {S11624, 86, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html) ASProxy/5.5b5*/
const Device UA227173 = {S11625, 111, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot-Image/1.0 AppEngine CirruxCache CoralWebPrxGooglebot-Image/1.0,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA250287 = {S11626, 99, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1; compatible; Googlebot/2.0; +http://www.google.com/bot.html)*/
const Device UA283944 = {S11627, 13, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot/2.1*/
const Device UA304178 = {S11628, 109, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html),gzip(gfe) (via translate.google.com)*/
const Device UA335355 = {S11629, 217, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html) AppEngine CirruxCache CoralWebPrxMozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html),gzip(gfe) (via docs.google.com/viewer)*/
const Device UA364282 = {S11630, 14, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot-News*/
const Device UA368607 = {S11631, 73, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - ?Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)*/
const Device UA389157 = {S11632, 86, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html) ASProxy/5.5b4*/
const Device UA418796 = {S11633, 69, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot/Googlebot 1.0 (GoogleBot; www.google.com; admin@google.com)*/
const Device UA430197 = {S11634, 50, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot/2.1 ( http://www.googlebot.com/bot.html)*/
const Device UA436156 = {S11635, 96, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html) [UsableNet Lift Mobile]*/
const Device UA459768 = {S11636, 56, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot-Image/1.0 ( http://www.googlebot.com/bot.html)*/
const Device UA503394 = {S11637, 71, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html*/
const Device UA506295 = {S11638, 47, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot/2.1+(+http://www.google.com/bot.html)*/
const Device UA509582 = {S11639, 47, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot/2.1 ( http://www.google.com/bot.html)*/
const Device UA547384 = {S11640, 33, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - http://www.googlebot.com/bot.html*/
const Device UA560316 = {S11641, 79, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla (compatible; Mediapartners-Google/2.1; +http://www.google.com/bot.html)*/
const Device UA565248 = {S11642, 133, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/4.0 (MobilePhone MM-8300/US/1.0) NetFront/3.1 MMP/2.0 (compatible; Mediapartners-Google/2.1; +http://www.google.com/bot.html)*/
const Device UA578585 = {S11643, 72, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.0; +http://www.google.com/bot.html)*/
const Device UA587170 = {S11644, 39, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; googlebot/2.1;*/
const Device UA635457 = {S11645, 86, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html) ASProxy/5.5b3*/
const Device UA649075 = {S11646, 46, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/4.0 (compatible; ScanAlert; Googlebot)*/
const Device UA673340 = {S11647, 61, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot/1.0 (googlebot@googlebot.com http://googlebot.com/)*/
const Device UA701618 = {S11648, 13, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot 1.0*/
const Device UA759852 = {S11649, 33, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot-Image/1.0 ASProxy/5.5b5*/
const Device UA773745 = {S11650, 39, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Locubot (compatible; Googlebot; msnbot)*/
const Device UA840014 = {S11651, 13, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot 2.1*/
const Device UA1888268 = {S11652, 46, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot/2.1 (+http://www.google.com/bot.html*/
const Device UA1902101 = {S11653, 70, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1;http://www.google.com/bot.html)*/
const Device UA1996471 = {S11654, 82, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla (compatible; Mediapartners-Googlebot/2.1;(+http://www.google.com/bot.html)*/
const Device UA2040890 = {S11655, 132, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/4.0 (compatible; +Baiduspider/2.0;++http://www.baidu.com/search/spider.html +Googlebot/2.1;++http://www.google.com/bot.html)*/
const Device UA2142249 = {S11656, 71, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1;+http://www.google.com/bot.html)*/
const Device UA2221254 = {S11657, 71, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozila/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)*/
const Device UA2268513 = {S11658, 49, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot/2.1(+http://www.googlebot.com/bot.html)*/
const Device UA2391766 = {S11659, 47, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot/2.1; +http://www.google.com/bot.html)*/
const Device UA2600242 = {S11660, 17, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - googlebot/bot.htm*/
const Device UA2638825 = {S11661, 46, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot/2.1 (http://www.google.com/bot.html)*/
const Device UA2677380 = {S11662, 33, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - GoogleBot (http://www.google.com)*/
const Device UA2696751 = {S11663, 76, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1; http://www.deepcrawl.co.uk/bot.html)*/
const Device UA2713624 = {S11664, 74, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - * Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)*/
const Device UA2718847 = {S11665, 91, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; GoogleDocs GoogleApps; script; +http://script.google.com/bot.html)*/
const Device UA2785785 = {S11666, 39, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1)*/
const Device UA2787177 = {S11667, 46, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot/2.1(+http://www.google.com/bot.html)*/
const Device UA2888521 = {S11668, 90, { &P18254,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; GoogleDocs GoogleApps script; +http://script.google.com/bot.html)*/
const Device UA2359517 = {S11669, 126, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B5105c Safari/8536.25*/
const Device UA2368110 = {S11670, 124, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B114 Safari/8536.25*/
const Device UA2391462 = {S11671, 126, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B5117b Safari/8536.25*/
const Device UA2437235 = {S11672, 126, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B5126b Safari/8536.25*/
const Device UA2470433 = {S11673, 124, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA2486558 = {S11674, 124, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B137 Safari/8536.25*/
const Device UA2506009 = {S11675, 124, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B135 Safari/8536.25*/
const Device UA2508284 = {S11676, 123, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA2515651 = {S11677, 126, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B311 Safari/8536.25*/
const Device UA2534896 = {S11678, 126, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B147 Safari/8536.25*/
const Device UA2535196 = {S11679, 123, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141Safari/8536.25*/
const Device UA2535472 = {S11680, 126, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146 Safari/8536.25*/
const Device UA2538935 = {S11681, 127, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B318a Safari/8536.25*/
const Device UA2560427 = {S11682, 123, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/9Z999 Safari/8536.25*/
const Device UA2561415 = {S11683, 125, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146 Safari/8536.25*/
const Device UA2561574 = {S11684, 125, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146Safari/8536.25*/
const Device UA2582269 = {S11685, 125, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/9Z999 Safari/8536.25*/
const Device UA2583703 = {S11686, 126, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329 Safari/8536.25*/
const Device UA2604533 = {S11687, 125, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329 Safari/8536.25*/
const Device UA2671983 = {S11688, 125, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329Safari/8536.25*/
const Device UA2706732 = {S11689, 126, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329 Safari/8536.25*/
const Device UA2749271 = {S11690, 124, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146 Safari/8536.25*/
const Device UA2790770 = {S11691, 124, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B136 Safari/8536.25*/
const Device UA2853632 = {S11692, 126, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B318 Safari/8536.25*/
const Device UA2879079 = {S11693, 126, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA2992077 = {S11694, 126, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA3035530 = {S11695, 126, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A406 Safari/8536.25*/
const Device UA3047919 = {S11696, 126, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A406 Safari/8536.25*/
const Device UA3059548 = {S11697, 124, { &P15767,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A406 Safari/8536.25*/
const Device UA2356450 = {S11698, 119, { &P17595,&P18207,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2357462 = {S11699, 119, { &P17595,&P18207,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2437159 = {S11700, 120, { &P17595,&P18207,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11*/
const Device UA2437170 = {S11701, 120, { &P17595,&P18207,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11*/
const Device UA2437410 = {S11702, 130, { &P17595,&P18207,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11,gzip(gfe)*/
const Device UA2437497 = {S11703, 159, { &P17595,&P18207,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2579974 = {S11704, 116, { &P17595,&P18207,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1082) AppleWebKit/537.11 (KHTML like Gecko) Chrome/23.0.1271.17 Safari/537.11*/
const Device UA2780948 = {S11705, 120, { &P17595,&P18207,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11*/
const Device UA3090481 = {S11706, 120, { &P17595,&P18207,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11*/
const Device UA911034 = {S11707, 147, { &P15364,&P7133,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA912380 = {S11708, 101, { &P15364,&P7133,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA912381 = {S11709, 108, { &P15364,&P7133,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA912536 = {S11710, 138, { &P15364,&P7133,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe) (via translate.google.com)*/
const Device UA912535 = {S11711, 145, { &P15364,&P7133,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe) (via translate.google.com)*/
const Device UA913181 = {S11712, 140, { &P15364,&P7133,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA924167 = {S11713, 137, { &P15364,&P7133,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/534.7 ZemanaAID/FFFF0061*/
const Device UA2355282 = {S11714, 118, { &P15364,&P7133,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe)*/
const Device UA2355527 = {S11715, 111, { &P15364,&P7133,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe)*/
const Device UA2389616 = {S11716, 220, { &P15364,&P7133,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; Zune 4.7) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11  Firefox/16.0*/
const Device UA2414586 = {S11717, 107, { &P15364,&P7133,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML like Gecko) Chrome/20.0.1132.47 Safari/536.11*/
const Device UA2746897 = {S11718, 183, { &P15364,&P7133,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11 AppEngine-Google; (+http://code.google.com/appengine; appid: s~zyfqwe5766)*/
const Device UA621838 = {S11719, 81, { &P12280,&P17779,&P18369,&P18092 }};	/*Useragent - Opera/9.80+(iPhone;+Opera+Mini/7.0.1/28.2051;+U;+en)+Presto/2.8.119+Version/11.10*/
const Device UA2431286 = {S11720, 81, { &P12280,&P17779,&P18369,&P18092 }};	/*Useragent - Opera/9.80 (iPhone; Opera Mini/7.0.5/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2199864 = {S11721, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Mercury3Free/6.0.1 CFNetwork/609 Darwin/13.0.0*/
const Device UA2200143 = {S11722, 39, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Feeds/2.2.1 CFNetwork/609 Darwin/13.0.0*/
const Device UA2200279 = {S11723, 39, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Terra/2.0.1 CFNetwork/609 Darwin/13.0.0*/
const Device UA2202547 = {S11724, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - HOT%20HD%20WPs/9.19.1 CFNetwork/609 Darwin/13.0.0*/
const Device UA2202602 = {S11725, 39, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MyPad/2.6.2 CFNetwork/609 Darwin/13.0.0*/
const Device UA2203153 = {S11726, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - WOW%20WPs/9.26.1 CFNetwork/609 Darwin/13.0.0*/
const Device UA2203308 = {S11727, 45, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Animations/9.30.1 CFNetwork/609 Darwin/13.0.0*/
const Device UA2203534 = {S11728, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - vBrowse%202.0/2.03 CFNetwork/609 Darwin/13.0.0*/
const Device UA2203767 = {S11729, 38, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Images/1.4 CFNetwork/609 Darwin/13.0.0*/
const Device UA2205983 = {S11730, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Mercury3/6.0.1 CFNetwork/609 Darwin/13.0.0*/
const Device UA2206082 = {S11731, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - GQMagazine/1.0 CFNetwork/609 Darwin/13.0.0*/
const Device UA2233041 = {S11732, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Web%20Feeds/1.2 CFNetwork/609 Darwin/13.0.0*/
const Device UA2233401 = {S11733, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - WEB%20to%20PDF/2.3 CFNetwork/609 Darwin/13.0.0*/
const Device UA2237624 = {S11734, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Prismatic/1.0.4 CFNetwork/609 Darwin/13.0.0*/
const Device UA2238363 = {S11735, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Qriket%20Game/3.1.5 CFNetwork/609 Darwin/13.0.0*/
const Device UA2242288 = {S11736, 40, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Digg/X.3.1.1 CFNetwork/609 Darwin/13.0.0*/
const Device UA2242353 = {S11737, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - CEBrowser/1.6.1 CFNetwork/609 Darwin/13.0.0*/
const Device UA2261756 = {S11738, 33, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - OWA/X CFNetwork/609 Darwin/13.0.0*/
const Device UA2348726 = {S11739, 37, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - JDate/101 CFNetwork/609 Darwin/13.0.0*/
const Device UA2350212 = {S11740, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - KaywaReader/1.21 CFNetwork/609 Darwin/13.0.0*/
const Device UA2351699 = {S11741, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - PhotoDiscover/2.1.1 CFNetwork/609 Darwin/13.0.0*/
const Device UA2351906 = {S11742, 38, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - iLoder/1.0 CFNetwork/609 Darwin/13.0.0*/
const Device UA2363175 = {S11743, 38, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ePrint/4.8 CFNetwork/609 Darwin/13.0.0*/
const Device UA2367311 = {S11744, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - NewsFree/1.1.1 CFNetwork/609 Darwin/13.0.0*/
const Device UA2368656 = {S11745, 38, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - TEE2/2.3.4 CFNetwork/609 Darwin/13.0.0*/
const Device UA2381120 = {S11746, 45, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Reddit%20Pics/1.6 CFNetwork/609 Darwin/13.0.0*/
const Device UA2381961 = {S11747, 41, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Scribbler/1.0 CFNetwork/609 Darwin/13.0.0*/
const Device UA2383237 = {S11748, 40, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - NewsMe/1.2.1 CFNetwork/609 Darwin/13.0.0*/
const Device UA2386866 = {S11749, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - TwitterFon/628 CFNetwork/609 Darwin/13.0.0*/
const Device UA2400547 = {S11750, 54, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MyPad%20for%20iPhone/2.6.2 CFNetwork/609 Darwin/13.0.0*/
const Device UA2400877 = {S11751, 40, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - NewsRack/254 CFNetwork/609 Darwin/13.0.0*/
const Device UA2402368 = {S11752, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - QihooBrowser/1.0 CFNetwork/609 Darwin/13.0.0*/
const Device UA2417711 = {S11753, 38, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Apsley/2.3 CFNetwork/609 Darwin/13.0.0*/
const Device UA2421475 = {S11754, 39, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - kikin/1.9.5 CFNetwork/609 Darwin/13.0.0*/
const Device UA2425082 = {S11755, 45, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - PhotoViewLite/2.4 CFNetwork/609 Darwin/13.0.0*/
const Device UA2425675 = {S11756, 45, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Readability/1.2.4 CFNetwork/609 Darwin/13.0.0*/
const Device UA2425825 = {S11757, 45, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - TwitterFonPro/628 CFNetwork/609 Darwin/13.0.0*/
const Device UA2426134 = {S11758, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - NewsifyFree/73 CFNetwork/609 Darwin/13.0.0*/
const Device UA2426274 = {S11759, 45, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - iSocializeFree/36 CFNetwork/609 Darwin/13.0.0*/
const Device UA2437256 = {S11760, 38, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Grokr/1508 CFNetwork/609 Darwin/13.0.0*/
const Device UA2454204 = {S11761, 41, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - TheFeed/1.0.2 CFNetwork/609 Darwin/13.0.0*/
const Device UA2454476 = {S11762, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ChromeSync%20Lite/1.2.1 CFNetwork/609 Darwin/13.0.0*/
const Device UA2455532 = {S11763, 41, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Celebrity/1.0 CFNetwork/609 Darwin/13.0.0*/
const Device UA2465480 = {S11764, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Demotivation/2.5.1 CFNetwork/609 Darwin/13.0.0*/
const Device UA2485392 = {S11765, 39, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Web/1.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2486165 = {S11766, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - AtomicBrowser/7.0.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2488881 = {S11767, 50, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Google/2.5.1.13455 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2489546 = {S11768, 48, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - AtomicLite/7.0.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2490322 = {S11769, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Clipish/9.29.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2490419 = {S11770, 45, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - browser/34122 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2490926 = {S11771, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Searchx/2.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2500863 = {S11772, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - TweetSpeaker/1.0.1 CFNetwork/609 Darwin/13.0.0*/
const Device UA2505657 = {S11773, 40, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Kik/6.1.0.20 CFNetwork/609 Darwin/13.0.0*/
const Device UA2507864 = {S11774, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - DaumApps/3.3.2 CFNetwork/609 Darwin/13.0.0*/
const Device UA2508662 = {S11775, 48, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Instapaper/4.2.7 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2508929 = {S11776, 57, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Celeb%20Wallpapers/9.13.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2509059 = {S11777, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - GeoServices/84 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2509673 = {S11778, 54, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ImageSearcherFreeS/1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2509851 = {S11779, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - SpringBoard/50 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2510216 = {S11780, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Settings/1.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2510720 = {S11781, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Extras4iMovie/10.01 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2511047 = {S11782, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - BrowserFree/1.2.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2511145 = {S11783, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - AirWatch/4.2.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2511508 = {S11784, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - 123people/3.1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2511928 = {S11785, 54, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Hot%20Wallpapers/10.02 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2512221 = {S11786, 50, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ImageSearcherS/3.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2513207 = {S11787, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Apolloplus/2.7.10 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2513221 = {S11788, 50, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - FeeddlerPro/1.12.5 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2514712 = {S11789, 55, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ImageSearcherFree/1.0.5 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2514720 = {S11790, 57, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Million%20HD%20WPs/8.80.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2515234 = {S11791, 61, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Free%20Video%20Downloader/1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2530722 = {S11792, 53, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - iAnnotate%20PDF/2.4.5 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2530725 = {S11793, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ImageDownloader/1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2532368 = {S11794, 55, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ImageSearcherProS/2.1.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2532960 = {S11795, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - AssistantServices/1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2533095 = {S11796, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ImageSearcher/3.1.3 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2535016 = {S11797, 48, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Y_Messenger/5467 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2535209 = {S11798, 45, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - PhotoFind/5.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2535210 = {S11799, 52, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Mobile%20Learn/3.1.4 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2535617 = {S11800, 59, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - securityd (unknown version) CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2535749 = {S11801, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - HideMyPhotos/9.29.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2535973 = {S11802, 50, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - LastPass%20Tab/2.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2536061 = {S11803, 64, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - HD%20&%20Retina%20Wallpapers/1.6 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2536060 = {S11804, 53, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - iDownloaderLite/1.0.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2536682 = {S11805, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - SecureBrowser/3.5.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2536812 = {S11806, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Holidays/10.01 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2536813 = {S11807, 54, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ImageSearcherPro/2.1.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2537057 = {S11808, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ImagePro/2.6 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2537058 = {S11809, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ImageHunter/2.5.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2537240 = {S11810, 53, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - EpicWallPaper2011/1.6 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2537377 = {S11811, 50, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - iDownloaderPro/2.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2537736 = {S11812, 63, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ApolloVideoDownloaderFree/1.2.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2558146 = {S11813, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Pulse/3.0.4 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2558183 = {S11814, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Love%20Pictures/3.7 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2558374 = {S11815, 63, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - syncdefaultsd (unknown version) CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2558373 = {S11816, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ubd/243.12.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2558402 = {S11817, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - PagesJaunes/5.3 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2558431 = {S11818, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Weather/1.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2558453 = {S11819, 40, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ZBar/2.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2558805 = {S11820, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Wallpapers/1.6 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2558937 = {S11821, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Funny%20Pics/2.93 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2559989 = {S11822, 55, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Love%20Messenger/9.16.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2560531 = {S11823, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - AdSheet/1.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2560755 = {S11824, 45, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Snipbase/2535 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2560871 = {S11825, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - mstreamd/2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2560968 = {S11826, 41, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - iPick/200 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2561332 = {S11827, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Download%20Free/2.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2561668 = {S11828, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - SSC%20Forums/X CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2563639 = {S11829, 39, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - tfctv/1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2566585 = {S11830, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Newsify/87 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2570433 = {S11831, 53, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - GoodReaderIPad/3.19.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2571393 = {S11832, 48, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - DiigoBrowser/4.7 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2577986 = {S11833, 36, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Azul/3.9 CFNetwork/609 Darwin/13.0.0*/
const Device UA2580317 = {S11834, 62, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Cool%20Retina%20Wallpapers/1.5 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2580534 = {S11835, 53, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Downloader%20copy/1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2580716 = {S11836, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - BarCodeScan/1.1.3 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2584203 = {S11837, 55, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - 2%20Million%20WPs/10.09 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2596323 = {S11838, 62, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - IMRemoteURLConnectionAgent/900 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2597196 = {S11839, 61, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - dataaccessd (unknown version) CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2597712 = {S11840, 62, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - itunesstored (unknown version) CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2597820 = {S11841, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - VideoDLFree/1.1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2597927 = {S11842, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - locationd/1491.92 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2597979 = {S11843, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - %20iSearch/1.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2598182 = {S11844, 41, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Cydia/0.9 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2598492 = {S11845, 62, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - mobileassetd (unknown version) CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2598499 = {S11846, 40, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - FDDM/1.3 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2599661 = {S11847, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Redd%20Alien/1630 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2599963 = {S11848, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - 360%20Browser/5.3.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2601595 = {S11849, 52, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - CLIPish%20Pro2/10.11 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2601728 = {S11850, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - iSocialize2/39 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2601928 = {S11851, 50, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - TubeDownloader/1.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2602944 = {S11852, 48, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Job%20Hopper/1.3 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2603803 = {S11853, 53, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Perfect%20Browser/9.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2604698 = {S11854, 60, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - NortonSafeBrowser/168.2.5.14 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2605198 = {S11855, 48, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - 360%20Lite/5.3.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2606384 = {S11856, 55, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - VideoDLPlusPlusFree/1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2659595 = {S11857, 48, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - DreamLeague/1.19 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2659597 = {S11858, 54, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - apsd (unknown version) CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2659884 = {S11859, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - VideoDLPro/1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2660134 = {S11860, 53, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Cool%20Wallpapers/1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2660284 = {S11861, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Symbaloo/1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2660384 = {S11862, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - IMTransferAgent/900 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2660588 = {S11863, 50, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ImageProHDFree/1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2660610 = {S11864, 60, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MusicDownloaderSSAFree/1.0.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2660709 = {S11865, 45, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ForumRunner/X CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2660723 = {S11866, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Instagram/X CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2661007 = {S11867, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Download/2.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2661103 = {S11868, 50, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ShoppingToday/2.04 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2661551 = {S11869, 48, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - iCandyWP1/9.15.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2662217 = {S11870, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Downloader/1.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2663848 = {S11871, 58, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MusicDownloaderBSOFree/1.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2665080 = {S11872, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - DL-Picture/1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2668287 = {S11873, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ScoopIt/111 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2672211 = {S11874, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Cooliris/71315 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2672580 = {S11875, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Reader/2.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2674185 = {S11876, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - vBrowse/3.7 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2675595 = {S11877, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ysearch2/3.0.4 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2676356 = {S11878, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Keeper/6.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2677174 = {S11879, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - SliceMobile/2.3.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2700393 = {S11880, 40, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Maps/1.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2700598 = {S11881, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ImageProHD/1.9 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2701183 = {S11882, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Snapchat/4.0.3 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2701272 = {S11883, 73, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - aosnotifyd/$%28CURRENT_PROJECT_VERSION%29 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2701287 = {S11884, 62, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - mediaserverd (unknown version) CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2701332 = {S11885, 57, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - configd (unknown version) CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2701508 = {S11886, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - iCarousel/1.2.7 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2701918 = {S11887, 55, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - gamed (unknown version) CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2702069 = {S11888, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - AppStore/1.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2702408 = {S11889, 41, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - xFeed/667 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2702553 = {S11890, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - VideoDownloader/1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2705421 = {S11891, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Lync%X/2.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2706971 = {S11892, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - iReader/5.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2707280 = {S11893, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - GoodReader/3.19.3 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2708051 = {S11894, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Epic%20Fail/2.8 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2709541 = {S11895, 61, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - CommCenterMobileHelper/1464.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2710520 = {S11896, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - UnlimitedSMS/4.4.23 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2712891 = {S11897, 53, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - UrlWebViewForIpad/1.8 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2715808 = {S11898, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MyPad_Premium/3.0.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2716074 = {S11899, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - YService/3.0.6 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2716333 = {S11900, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ImageSearch/1.5 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2716495 = {S11901, 48, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - wewsiphone/11710 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2716620 = {S11902, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - playable/3.6 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2726681 = {S11903, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Sigalert/1.0.1 CFNetwork/609 Darwin/13.0.0*/
const Device UA2730617 = {S11904, 50, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Pulse%20News/2.9.5 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2730926 = {S11905, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - FreeCell/2.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2731737 = {S11906, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - AWBrowser/2.0.460 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2732118 = {S11907, 54, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MusicDownloaderBSO/1.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2732393 = {S11908, 61, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ApolloGeneralDownloader/1.1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2736028 = {S11909, 48, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Silly%20Pics/1.3 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2738054 = {S11910, 50, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - BaDoink%20Lite/1.5 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2738226 = {S11911, 83, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Yellow%20Pages%20Local%20Search%20by%20Superpages/X CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2738610 = {S11912, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ImageMobile/1.9 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2748319 = {S11913, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Accounts/113 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2748388 = {S11914, 52, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Dual%20Browser/1.4.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2748418 = {S11915, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - iBuilder/1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2748762 = {S11916, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - iScanner/3.5 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2748842 = {S11917, 54, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ScrollingCredits/10.01 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2749113 = {S11918, 64, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - OTACrashCopier (unknown version) CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2749886 = {S11919, 54, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - TubeDownloaderLite/1.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2751152 = {S11920, 55, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - iDownloadsAppPlus/1.5.8 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2752898 = {S11921, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Yahoo!/9150 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2764166 = {S11922, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - WebSheet/1.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2764400 = {S11923, 50, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Codes%20Reader/2.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2764423 = {S11924, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Christmas/10.01 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2764680 = {S11925, 52, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ImageHunterPro/2.5.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2764717 = {S11926, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - LexPrint/3.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2764900 = {S11927, 41, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - NESN/1.03 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2765222 = {S11928, 40, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Flow/1.5 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2770001 = {S11929, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - PDF%20Printer/2.0.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2770621 = {S11930, 56, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - %EB%8B%A4%EC%9D%8C/4.0.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2773257 = {S11931, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Twitter/5.7 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2779249 = {S11932, 59, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ApolloVideoDownloader/1.2.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2780035 = {S11933, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Messenger/X CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2784775 = {S11934, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MxBrowser-iPhone/13 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2792260 = {S11935, 41, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Homes/1.5 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2793870 = {S11936, 55, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - VideoDownloaderLite/1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2795753 = {S11937, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - HDWallpaper/2.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2796877 = {S11938, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MyMedia/3.01 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2797380 = {S11939, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Filebox/1.4 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2799294 = {S11940, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Qrafter/7.6 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2799311 = {S11941, 41, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - iFunny/22 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2800706 = {S11942, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Guidebook/2.2.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2843159 = {S11943, 40, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Summly/2.0.2 CFNetwork/609 Darwin/13.0.0*/
const Device UA2850659 = {S11944, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Shazam/581 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2852056 = {S11945, 50, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Emoji%20PLUS/10.01 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2852221 = {S11946, 55, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Million%20Photos/8.60.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2852911 = {S11947, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - AppCake3/3.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2857925 = {S11948, 50, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - PimpYourPic/9.14.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2860056 = {S11949, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - VideoDLPlusPlus/1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2860241 = {S11950, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Pandora/4.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2860776 = {S11951, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - subwaysurfers/1.9.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2862131 = {S11952, 57, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - WordsWithFriendsFree/5.90 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2866584 = {S11953, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - FlashLight/3.6 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2868514 = {S11954, 52, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Talking%20Ginger/1.4 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2869349 = {S11955, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MessageMe/11 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2870006 = {S11956, 52, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Pic%20Collage/3.10.5 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2873838 = {S11957, 55, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MusicDownloaderLite/1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2874508 = {S11958, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - RoboForm/151 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2875305 = {S11959, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MobileSearch/5774 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2875374 = {S11960, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MusicDownloader/1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2878662 = {S11961, 60, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Accountable2You%20Mobile/1.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2879047 = {S11962, 45, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Ruzzle/1.4.10 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2879588 = {S11963, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - iPhone%20Free/X CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2885364 = {S11964, 45, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - whisper/2.9.8 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2886438 = {S11965, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MxBrowser-iPhone/14 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2889756 = {S11966, 57, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - backupd (unknown version) CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2890982 = {S11967, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - LinkedIn/6.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2892215 = {S11968, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - catalogTattoos/1.71 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2892620 = {S11969, 82, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - %E5%BF%AB%E6%92%AD%E5%A2%9E%E5%BC%BA%E7%89%88/1734 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2892997 = {S11970, 52, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - PDF%20PROvider/2.0.4 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2893158 = {S11971, 60, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ksmobilebrowser/1.2.4342.200 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2896092 = {S11972, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Qrafter%20Pro/7.6 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2897654 = {S11973, 91, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - %E5%A4%9A%E5%B1%8F%E4%BA%92%E5%8A%A8%E6%B5%8F%E8%A7%88/1976 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2897964 = {S11974, 48, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - SKOUT_SDK4/3.6.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2898637 = {S11975, 45, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - BaiduHD/2.2.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2898770 = {S11976, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - NaverSearch/4.7.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2899209 = {S11977, 48, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Tweetlogix/2.1.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2934649 = {S11978, 52, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - AppleIDAuthAgent/552 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2934989 = {S11979, 45, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - WallPaper/1.3 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2937046 = {S11980, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Florida/94 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2937199 = {S11981, 48, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Viber/3.0.0.4074 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2937482 = {S11982, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Thumb/3.8.8 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2937935 = {S11983, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Educreations/1243 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2938309 = {S11984, 59, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ColorLEDFlashlightPro/1.5.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2947003 = {S11985, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - YandexSearch/1480 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2952275 = {S11986, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - SkyGrid/2.6 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2955571 = {S11987, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - vBrowsePRO/3.7 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2956232 = {S11988, 45, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - DownMan/4.0.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2957034 = {S11989, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - NorthCarolina/291 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2957465 = {S11990, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Buffer/2.4 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2957482 = {S11991, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - logosQuiz/1.5.6 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2957949 = {S11992, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - NewJerseyFH/734 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2957955 = {S11993, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - SolitaireFree/3.8.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2958549 = {S11994, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Ourpad/1.6.6 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2958987 = {S11995, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Washington/208 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2959376 = {S11996, 40, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Voto/119 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2964449 = {S11997, 59, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - WEB%20to%20PDF%20Plus/2.3.3 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2965945 = {S11998, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - QRReader/3.6 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2968192 = {S11999, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - VZ%20Messages/1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2970468 = {S12000, 50, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - FastBrowser/10.3.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2971209 = {S12001, 45, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - GeorgiaFH/473 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2971210 = {S12002, 45, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - NewJersey/518 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2971211 = {S12003, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Louisiana/1291 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2974403 = {S12004, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - AlienBlueHD/2.2.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2975830 = {S12005, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - FreeDownload/2.35 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2985836 = {S12006, 41, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Clock/1.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2986759 = {S12007, 59, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - PerfectDownloaderLite/2.5.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2986922 = {S12008, 52, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - ChickOrDude-Lite/1.5 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2987076 = {S12009, 54, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Fun%20Animations/10.07 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2987394 = {S12010, 63, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - WallpaperGeneratorHD%20copy/4.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2987416 = {S12011, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - RSSReader/1.04 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2987722 = {S12012, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - SoundBooth/9.13.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2987830 = {S12013, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MobileStore/1.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2988908 = {S12014, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Photon/4.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2989254 = {S12015, 53, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Print%20Utility/2.9.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2989307 = {S12016, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - iBooks/1523 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2994430 = {S12017, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - DownloadAll/1.0.3 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2996153 = {S12018, 55, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MicroMessenger/4.5.0.54 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2996484 = {S12019, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Pennsylvania/1140 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2996490 = {S12020, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Delaware/253 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2997042 = {S12021, 45, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Accessbox/1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2997914 = {S12022, 57, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Ginger%20%F0%9F%8E%82/1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2998575 = {S12023, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Sudoku/1.4 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2998952 = {S12024, 50, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - BabyBumpFree/7.1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2999672 = {S12025, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Connecticut/718 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3002023 = {S12026, 48, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - IconPopQuiz/1.41 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3003737 = {S12027, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - WMV%20player/2.10 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3006180 = {S12028, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - To%20PDF/1.0.3 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3007081 = {S12029, 45, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Speedtest/3.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3008849 = {S12030, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Georgia/382 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3008941 = {S12031, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - SouthDakota/235 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3008946 = {S12032, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Missouri/100 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3009059 = {S12033, 40, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Maine/47 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3009961 = {S12034, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Songza/1433 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3011407 = {S12035, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - HP%20ePrint/5.2.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3013024 = {S12036, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Kentucky/121 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3017400 = {S12037, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - eBay/3.0.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3018675 = {S12038, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - CamFind/98 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3019375 = {S12039, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Springpad/4.0.6 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3019451 = {S12040, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - vine/1.3.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3019849 = {S12041, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Montana/258 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3019885 = {S12042, 45, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Tennessee/243 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3021295 = {S12043, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MyLifeRelease/312 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3021999 = {S12044, 54, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MusicDownloaderSSA/1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3024467 = {S12045, 48, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Incredimail/5649 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3028273 = {S12046, 51, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MxBrowser-iPhone/17 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3028848 = {S12047, 68, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Free%20Video%20Downloader%20Lite/1.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3034191 = {S12048, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Pinterest/2.6.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3044610 = {S12049, 41, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - passd/1.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3044663 = {S12050, 52, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Translate/1.3.1.2978 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3044747 = {S12051, 50, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MobileSlideShow/43 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3044860 = {S12052, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Glassboard/2041 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3045017 = {S12053, 45, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - iReddit/2.1.5 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3047722 = {S12054, 52, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Calculator%20Pro/2.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3048300 = {S12055, 55, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - VideoDownloaderFree/2.8 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3048671 = {S12056, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Virginia/169 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3048680 = {S12057, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Kansas/105 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3048769 = {S12058, 48, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - TextMe%202/2.2.5 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3049407 = {S12059, 50, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MilkTheCowFREE/2.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3049539 = {S12060, 48, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - OfficeJerk/1.9.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3051970 = {S12061, 40, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Scan/369 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3053650 = {S12062, 39, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Utah/74 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3054110 = {S12063, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - NebraskaFH/152 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3058270 = {S12064, 50, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - DespicableMe/1.1.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3058445 = {S12065, 62, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MusicDownloaderSokolovLite/1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3061319 = {S12066, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - 123peopleUS/3.4 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3061673 = {S12067, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - etracksapp/969 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3065789 = {S12068, 45, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - GroupMe/4.2.5 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3066656 = {S12069, 41, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Edmodo/12 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3071450 = {S12070, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - GWallPaper/1.7 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3071533 = {S12071, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - PhotoAge-Lite/1.5 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3071671 = {S12072, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Xmarks/1.5.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3073439 = {S12073, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Tap%20Forms/93 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3074196 = {S12074, 61, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Free%20Music%20Download/1.4.3 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3074198 = {S12075, 57, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - TotalDownloader%20Free/29 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3078319 = {S12076, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Solitaire/2.4.4 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3078356 = {S12077, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - food/2.3.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3082192 = {S12078, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Dashlane/45475 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3083265 = {S12079, 49, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Coast/1.0.2.62956 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3086492 = {S12080, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - PanelLED/1.3.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3086817 = {S12081, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - PZPlayer/1.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3088109 = {S12082, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Nebraska/398 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3088219 = {S12083, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - NewYork/184 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3092022 = {S12084, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MAKE19/1.8 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3092090 = {S12085, 53, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - classifieds_free/2232 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3093321 = {S12086, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - LINE/3.8.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3096629 = {S12087, 47, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - AlienBlue/2.8.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3100553 = {S12088, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - TalkTablet/6.9 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3101536 = {S12089, 63, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Barracuda_Safe_Browser/2.0.2.12 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3102452 = {S12090, 46, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - RhodeIsland/46 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3102932 = {S12091, 58, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - mp3musicdownloaderfree/5.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3103117 = {S12092, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - BejBlitz/608 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3104484 = {S12093, 39, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Iowa/50 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3107923 = {S12094, 58, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - MusicDownloaderSokolov/1.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3108600 = {S12095, 40, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Snip/1.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3108776 = {S12096, 43, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Dentist/1.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3124650 = {S12097, 44, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - JazzBird/7.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3125118 = {S12098, 42, { &P15364,&P18580,&P17470,&P18092 }};	/*Useragent - Daum/4.2.2 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA106150 = {S12099, 96, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 5.1; Microsoft Outlook 14.0.4760; Pro; ms-office; MSOffice 14)*/
const Device UA157776 = {S12100, 96, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 5.1; Microsoft Outlook 14.0.5128; Pro; ms-office; MSOffice 14)*/
const Device UA207952 = {S12101, 219, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA244781 = {S12102, 96, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 5.1; Microsoft Outlook 14.0.6106; Pro; ms-office; MSOffice 14)*/
const Device UA303021 = {S12103, 96, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 5.1; Microsoft Outlook 14.0.4734; Pro; ms-office; MSOffice 14)*/
const Device UA394775 = {S12104, 211, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.3; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA452085 = {S12105, 220, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; Schroders; Microsoft Outlook 14.0.6106; ms-office; MSOffice 14)*/
const Device UA542484 = {S12106, 199, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Outlook 14.0.5136; ms-office; MSOffice 14)*/
const Device UA555293 = {S12107, 72, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 5.1; Microsoft Outlook 14.0.6023; Pro)*/
const Device UA561236 = {S12108, 199, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.3; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA566411 = {S12109, 199, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.3; Microsoft Outlook 14.0.5138; ms-office; MSOffice 14)*/
const Device UA604988 = {S12110, 219, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA605178 = {S12111, 219, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA700181 = {S12112, 231, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA710712 = {S12113, 219, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA713578 = {S12114, 239, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA719990 = {S12115, 211, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.3; Microsoft Outlook 14.0.5128; ms-office; MSOffice 14)*/
const Device UA1904123 = {S12116, 199, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.3; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA1910616 = {S12117, 96, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 5.1; Microsoft Outlook 14.0.6112; Pro; ms-office; MSOffice 14)*/
const Device UA1981536 = {S12118, 207, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2010474 = {S12119, 72, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 5.1; Microsoft Outlook 14.0.6123; Pro)*/
const Device UA2170860 = {S12120, 259, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; GTB7.4; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.3; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2220881 = {S12121, 199, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Outlook 14.0.6114; ms-office; MSOffice 14)*/
const Device UA2301579 = {S12122, 172, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2362874 = {S12123, 219, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2390147 = {S12124, 219, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2433775 = {S12125, 72, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 5.1; Microsoft Outlook 14.0.6129; Pro)*/
const Device UA2456927 = {S12126, 251, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.3; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2482782 = {S12127, 207, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2516556 = {S12128, 219, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2524647 = {S12129, 155, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; Microsoft Outlook 14.0.6123; ms-office; MSOffice 14)*/
const Device UA2537918 = {S12130, 219, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2583465 = {S12131, 239, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2700415 = {S12132, 207, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2786643 = {S12133, 172, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2982212 = {S12134, 192, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.3; Microsoft Outlook 14.0.5138; ms-office; MSOffice 14)*/
const Device UA2987409 = {S12135, 238, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; ms-office; MSOffice 14) ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Outlook 14.0.6131)*/
const Device UA3021923 = {S12136, 72, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 5.1; Microsoft Outlook 14.0.7015; Pro)*/
const Device UA3023082 = {S12137, 219, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3023962 = {S12138, 232, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.3; MS-RTC LM 8; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3058363 = {S12139, 219, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3083629 = {S12140, 72, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 5.1; Microsoft Outlook 14.0.7106; Pro)*/
const Device UA3083814 = {S12141, 219, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3095547 = {S12142, 219, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3100285 = {S12143, 244, { &P15364,&P5690,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8; .NET4.0C; .NET4.0E; InfoPath.2; Microsoft Outlook 14.0.6133; ms-office; MSOffice 14)*/
const Device UA2567788 = {S12144, 120, { &P17595,&P17297,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2567819 = {S12145, 120, { &P17595,&P17297,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2567818 = {S12146, 120, { &P17595,&P17297,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2567937 = {S12147, 120, { &P17595,&P17297,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2567972 = {S12148, 120, { &P17595,&P17297,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2568216 = {S12149, 120, { &P17595,&P17297,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2584054 = {S12150, 130, { &P17595,&P17297,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)*/
const Device UA2604099 = {S12151, 159, { &P17595,&P17297,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA207812 = {S12152, 108, { &P15364,&P7133,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30*/
const Device UA207856 = {S12153, 101, { &P15364,&P7133,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30*/
const Device UA216184 = {S12154, 145, { &P15364,&P7133,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30,gzip(gfe) (via translate.google.com)*/
const Device UA320688 = {S12155, 137, { &P15364,&P7133,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.91 Safari/534.30,gzip(gfe) (via translate.google.com)*/
const Device UA578378 = {S12156, 106, { &P15364,&P7133,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; en-US) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.750.0 Safari/534.30*/
const Device UA602414 = {S12157, 132, { &P15364,&P7133,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/12.0.702.0 Safari/533.3 Awesomium/1.6.5*/
const Device UA1893121 = {S12158, 107, { &P15364,&P7133,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; AppleWebKit/534.24; KHTML; like Gecko; Chrome/12.0.702.0 Safari/534.24)*/
const Device UA1992915 = {S12159, 112, { &P15364,&P7133,&P17251,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Windows NT 6.1; AppleWebKit/534.24; KHTML, like Gecko; Chrome/12.0.702.0 Safari/534.24)*/
const Device UA2355291 = {S12160, 118, { &P15364,&P7133,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30,gzip(gfe)*/
const Device UA2375479 = {S12161, 120, { &P15364,&P7133,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.30 (KHTML	 like Gecko) Chrome/12.0.742.122 Safari/534.30 ChromePlus/1.6.3.1*/
const Device UA2409013 = {S12162, 111, { &P15364,&P7133,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30,gzip(gfe)*/
const Device UA2708510 = {S12163, 106, { &P15364,&P7133,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; en-CA) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.750.0 Safari/534.30*/
const Device UA2709374 = {S12164, 106, { &P15364,&P7133,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; es-US) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.750.0 Safari/534.30*/
const Device UA2709520 = {S12165, 106, { &P15364,&P7133,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; zh-CN) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.750.0 Safari/534.30*/
const Device UA2710210 = {S12166, 107, { &P15364,&P7133,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; haw-US) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.750.0 Safari/534.30*/
const Device UA2711329 = {S12167, 106, { &P15364,&P7133,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; fr-CA) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.750.0 Safari/534.30*/
const Device UA2803848 = {S12168, 147, { &P15364,&P7133,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2860757 = {S12169, 106, { &P15364,&P7133,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; tr-DE) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.750.0 Safari/534.30*/
const Device UA2878101 = {S12170, 105, { &P15364,&P7133,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.24 (KHTML like Gecko) Chrome/12.0.702.0 Safari/534.24*/
const Device UA2933590 = {S12171, 105, { &P15364,&P7133,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; en-US) AppleWebKit/534.30 (KHTML like Gecko) Chrome/12.0.750.0 Safari/534.30*/
const Device UA2979916 = {S12172, 119, { &P15364,&P7133,&P17251,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Windows NT 6.1; WOW64; AppleWebKit/534.24; KHTML, like Gecko; Chrome/12.0.702.0 Safari/534.24)*/
const Device UA2980045 = {S12173, 125, { &P15364,&P7133,&P17251,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Windows NT 6.1; AppleWebKit/534.24; Trident/4.0; KHTML, like Gecko; Chrome/12.0.702.0 Safari/534.24)*/
const Device UA2770105 = {S12174, 117, { &P15364,&P17285,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36*/
const Device UA2783478 = {S12175, 97, { &P15364,&P17285,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.1.0.0 Safari/537.36*/
const Device UA2783628 = {S12176, 104, { &P15364,&P17285,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.1.0.0 Safari/537.36*/
const Device UA2796498 = {S12177, 124, { &P15364,&P17285,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36*/
const Device UA2873648 = {S12178, 102, { &P15364,&P17285,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2873652 = {S12179, 109, { &P15364,&P17285,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2874649 = {S12180, 112, { &P15364,&P17285,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe)*/
const Device UA2876962 = {S12181, 141, { &P15364,&P17285,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2877031 = {S12182, 104, { &P15364,&P17285,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.2.0.0 Safari/537.36*/
const Device UA2877023 = {S12183, 97, { &P15364,&P17285,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.2.0.0 Safari/537.36*/
const Device UA2877382 = {S12184, 119, { &P15364,&P17285,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe)*/
const Device UA2887703 = {S12185, 148, { &P15364,&P17285,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2470409 = {S12186, 97, { &P15364,&P16110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.1.0.0 Safari/537.17*/
const Device UA2471961 = {S12187, 104, { &P15364,&P16110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.1.0.0 Safari/537.17*/
const Device UA2489208 = {S12188, 118, { &P15364,&P16110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)*/
const Device UA2490841 = {S12189, 111, { &P15364,&P16110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)*/
const Device UA2492153 = {S12190, 108, { &P15364,&P16110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492186 = {S12191, 101, { &P15364,&P16110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492503 = {S12192, 104, { &P15364,&P16110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.2.0.0 Safari/537.17*/
const Device UA2492509 = {S12193, 97, { &P15364,&P16110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.2.0.0 Safari/537.17*/
const Device UA2509887 = {S12194, 114, { &P15364,&P16110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; en-US) AppleWebKit/537.15 (KHTML, like Gecko) Chrome/24.0.1295.0 Safari/537.15*/
const Device UA2512538 = {S12195, 147, { &P15364,&P16110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2537886 = {S12196, 140, { &P15364,&P16110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2956188 = {S12197, 117, { &P15364,&P16110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.10.1312.57 Safari/537.17 QIPSurf*/
const Device UA120053 = {S12198, 62, { &P15364,&P6704,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.01; Windows 98; Alexa Toolbar)*/
const Device UA194787 = {S12199, 58, { &P15364,&P6704,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; Win 9x4.90)*/
const Device UA375501 = {S12200, 46, { &P15364,&P6704,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Windows 98)*/
const Device UA383549 = {S12201, 46, { &P15364,&P6704,&P17191,&P18092 }};	/*Useragent - Mozilla/4.5 (compatible; MSIE 6.0; Windows 98)*/
const Device UA418341 = {S12202, 46, { &P15364,&P6704,&P17191,&P18092 }};	/*Useragent - Mozilla/9.0 (compatible; MSIE 6.0; Windows 98)*/
const Device UA2294812 = {S12203, 47, { &P15364,&P6704,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.02; Windows 98)*/
const Device UA2791883 = {S12204, 60, { &P15364,&P6704,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Win 9x 4.90; Trident/4.0)*/
const Device UA2850035 = {S12205, 54, { &P15364,&P6704,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Win98; Trident/3.0)*/
const Device UA311915 = {S12206, 86, { &P18088,&P17017,&P17370,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/5.0.17443/24.746; U; en) Presto/2.5.25 Version/10.54*/
const Device UA813693 = {S12207, 68, { &P18088,&P17017,&P17370,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/5.0.3521/886; U; en) Presto/2.4.15*/
const Device UA2389965 = {S12208, 59, { &P18088,&P17017,&P17370,&P18092 }};	/*Useragent - Opera/9.50 (J2ME/MIDP; Opera Mini/5.0.21463/28.3165; U; en)*/
const Device UA2431122 = {S12209, 82, { &P18088,&P17017,&P17370,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/5.0/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2431229 = {S12210, 80, { &P18088,&P17017,&P17370,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/5/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2198530 = {S12211, 64, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.10*/
const Device UA2203706 = {S12212, 57, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.10*/
const Device UA2349502 = {S12213, 75, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.10*/
const Device UA2352707 = {S12214, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.10*/
const Device UA2353243 = {S12215, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.10*/
const Device UA2355299 = {S12216, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.10*/
const Device UA2355624 = {S12217, 74, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.10,gzip(gfe)*/
const Device UA2355880 = {S12218, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.10*/
const Device UA2356625 = {S12219, 67, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.10,gzip(gfe)*/
const Device UA2357319 = {S12220, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.10*/
const Device UA2357352 = {S12221, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.10*/
const Device UA2357357 = {S12222, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.10*/
const Device UA2357446 = {S12223, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.10*/
const Device UA2357453 = {S12224, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.10*/
const Device UA2357562 = {S12225, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.10*/
const Device UA2357680 = {S12226, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.10*/
const Device UA2357907 = {S12227, 80, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.10*/
const Device UA2358004 = {S12228, 80, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.10*/
const Device UA2358013 = {S12229, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.10*/
const Device UA2358071 = {S12230, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.10*/
const Device UA2358183 = {S12231, 80, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.10*/
const Device UA2358305 = {S12232, 69, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64) Opera 12.10*/
const Device UA2358406 = {S12233, 80, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.10*/
const Device UA2358413 = {S12234, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.10*/
const Device UA2358841 = {S12235, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.10*/
const Device UA2359082 = {S12236, 87, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.10*/
const Device UA2359104 = {S12237, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.10*/
const Device UA2359520 = {S12238, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.10*/
const Device UA2359524 = {S12239, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.10*/
const Device UA2359526 = {S12240, 80, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.10*/
const Device UA2360774 = {S12241, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.10*/
const Device UA2360882 = {S12242, 80, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.10*/
const Device UA2361333 = {S12243, 80, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.10*/
const Device UA2361321 = {S12244, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.10*/
const Device UA2361363 = {S12245, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.10*/
const Device UA2361404 = {S12246, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.10*/
const Device UA2361444 = {S12247, 80, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.10*/
const Device UA2361495 = {S12248, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.10*/
const Device UA2361636 = {S12249, 80, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.10*/
const Device UA2361638 = {S12250, 87, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.10*/
const Device UA2361826 = {S12251, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.10*/
const Device UA2362185 = {S12252, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.10*/
const Device UA2367492 = {S12253, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.10*/
const Device UA2367552 = {S12254, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.10*/
const Device UA2368335 = {S12255, 80, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.10*/
const Device UA2368440 = {S12256, 80, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.10*/
const Device UA2368494 = {S12257, 80, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.10*/
const Device UA2368784 = {S12258, 80, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.10*/
const Device UA2386708 = {S12259, 87, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5288)) Presto/2.12.388 Version/12.10*/
const Device UA2387025 = {S12260, 74, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.10 Yandex.Translate*/
const Device UA2387356 = {S12261, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.10*/
const Device UA2387362 = {S12262, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.10*/
const Device UA2387377 = {S12263, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.10*/
const Device UA2403524 = {S12264, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4933)) Presto/2.12.388 Version/12.10*/
const Device UA2403753 = {S12265, 80, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.10*/
const Device UA2403842 = {S12266, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.10*/
const Device UA2405104 = {S12267, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.10*/
const Device UA2422862 = {S12268, 87, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.10*/
const Device UA2422899 = {S12269, 80, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5230)) Presto/2.12.388 Version/12.10*/
const Device UA2423261 = {S12270, 80, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03789)) Presto/2.12.388 Version/12.10*/
const Device UA2425971 = {S12271, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.10*/
const Device UA2456841 = {S12272, 80, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.10*/
const Device UA2456868 = {S12273, 80, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.10*/
const Device UA2456993 = {S12274, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5660)) Presto/2.12.388 Version/12.10*/
const Device UA2457565 = {S12275, 87, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.10*/
const Device UA2459657 = {S12276, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.10*/
const Device UA2470161 = {S12277, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.10*/
const Device UA2507057 = {S12278, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.10*/
const Device UA2556504 = {S12279, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.10*/
const Device UA2578556 = {S12280, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4110)) Presto/2.12.388 Version/12.10*/
const Device UA2583072 = {S12281, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.10*/
const Device UA2653425 = {S12282, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.10*/
const Device UA2659347 = {S12283, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.10*/
const Device UA2666523 = {S12284, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.10*/
const Device UA2707018 = {S12285, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.10*/
const Device UA2767001 = {S12286, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.10*/
const Device UA2790414 = {S12287, 87, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5278)) Presto/2.12.388 Version/12.10*/
const Device UA2796493 = {S12288, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.10*/
const Device UA2840992 = {S12289, 87, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03658)) Presto/2.12.388 Version/12.10*/
const Device UA2841622 = {S12290, 87, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5299)) Presto/2.12.388 Version/12.10*/
const Device UA2841713 = {S12291, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4832)) Presto/2.12.388 Version/12.10*/
const Device UA2841785 = {S12292, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4661)) Presto/2.12.388 Version/12.10*/
const Device UA2842030 = {S12293, 87, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5300)) Presto/2.12.388 Version/12.10*/
const Device UA2855413 = {S12294, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.10*/
const Device UA2864533 = {S12295, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.10*/
const Device UA2891447 = {S12296, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.10*/
const Device UA2982869 = {S12297, 79, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5730)) Presto/2.12.388 Version/12.10*/
const Device UA3043279 = {S12298, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5650)) Presto/2.12.388 Version/12.10*/
const Device UA3045144 = {S12299, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.10*/
const Device UA3048236 = {S12300, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.10*/
const Device UA3073068 = {S12301, 86, { &P15364,&P7133,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.10*/
const Device UA438853 = {S12302, 62, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC One X Build/IML74K)*/
const Device UA438854 = {S12303, 140, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC One X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA455742 = {S12304, 151, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_X/1.26.161.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA457682 = {S12305, 151, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_X/1.26.110.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA608772 = {S12306, 149, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PJ83100/1.73.502.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA609609 = {S12307, 151, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC_One_X/1.26.163.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616671 = {S12308, 53, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - BBCBeacon/0.73 (HTC One X; Android 4.0.3) iplayer/1.3*/
const Device UA624882 = {S12309, 140, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA629752 = {S12310, 150, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_X-orange-LS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA652719 = {S12311, 151, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_X/1.28.161.9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA670489 = {S12312, 151, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC_One_X/1.26.166.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA712784 = {S12313, 149, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PJ83100/1.61.502.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA728771 = {S12314, 140, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC S720e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA731741 = {S12315, 151, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_X/1.28.110.7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA742163 = {S12316, 126, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC One X Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA770144 = {S12317, 151, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_One_X/1.26.169.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA779067 = {S12318, 151, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_One_X/1.26.111.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA788455 = {S12319, 152, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_One_X/1.28.111.10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA829164 = {S12320, 151, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_One_X/1.26.112.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA843964 = {S12321, 149, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PJ83100/1.82.502.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA879107 = {S12322, 141, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC EVA_UL Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA893382 = {S12323, 151, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC_One_X/1.28.163.9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899119 = {S12324, 149, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PJ83100/1.85.502.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA900471 = {S12325, 152, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_X/1.29.110.11 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA900572 = {S12326, 140, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA904118 = {S12327, 152, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_X/1.29.161.13 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA904980 = {S12328, 152, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_One_X/1.29.169.13 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA905057 = {S12329, 152, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC_One_X/1.29.166.13 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA906957 = {S12330, 140, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_One_X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA907881 = {S12331, 152, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC_One_X/1.29.163.13 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1953259 = {S12332, 152, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; HTC_One_X/1.28.118.11 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1968073 = {S12333, 151, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC_One_X/1.28.166.9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1992517 = {S12334, 62, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC One X Build/IMM76D)*/
const Device UA1996249 = {S12335, 149, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PJ83100/2.20.502.7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2001391 = {S12336, 149, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PJ83100/1.82.502.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2023757 = {S12337, 131, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC S720e Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2025457 = {S12338, 131, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC One X Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2026567 = {S12339, 149, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PJ83100/1.68.502.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028502 = {S12340, 136, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; One X Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031252 = {S12341, 126, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC S720e Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2031971 = {S12342, 149, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PJ83100/1.85.502.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032138 = {S12343, 152, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_One_X/1.29.111.11 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032994 = {S12344, 138, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; HTC One X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033519 = {S12345, 140, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC S720e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033658 = {S12346, 149, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PJ83100/2.20.502.5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033706 = {S12347, 151, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_One_X/1.28.169.9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2042983 = {S12348, 151, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_One_X/2.17.161.4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2166708 = {S12349, 150, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_One_X-orange-LS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2193309 = {S12350, 151, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HTC_One_X/2.17.111.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2197916 = {S12351, 151, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; HTC_One_X/2.17.163.4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2204929 = {S12352, 151, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; HTC_One_X/2.17.169.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2205851 = {S12353, 151, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-ch; HTC_One_X/2.17.166.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2206110 = {S12354, 141, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; HTC EVA_UL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2206151 = {S12355, 151, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_One_X/2.17.110.5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2235465 = {S12356, 151, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; HTC_One_X/2.17.112.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2341976 = {S12357, 136, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S720e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2351384 = {S12358, 151, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; HTC_One_X/2.17.118.4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2355838 = {S12359, 146, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2356264 = {S12360, 131, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC S720e Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2365347 = {S12361, 138, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2413784 = {S12362, 152, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_One_X/1.29.161.13 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2414316 = {S12363, 138, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-; HTC_One_X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2518752 = {S12364, 138, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; HTC_One_X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2588949 = {S12365, 149, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PJ83100/1.61.502.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2596618 = {S12366, 148, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneX/1.26.111.4 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597194 = {S12367, 149, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneX/1.29.166.13 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2599684 = {S12368, 148, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneX/1.26.110.5 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658125 = {S12369, 130, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC One X Build/IML74K) UC AppleWebKit/534.31 (KHTML like Gecko) Mobile Safari/534.31*/
const Device UA2658386 = {S12370, 148, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneX/1.28.110.7 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658567 = {S12371, 137, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCOneX Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658892 = {S12372, 149, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneX/1.28.111.10 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659078 = {S12373, 148, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneX/1.26.161.2 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659136 = {S12374, 148, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCOneX/2.17.161.4 Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659171 = {S12375, 147, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCOneX-orange-LS Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659203 = {S12376, 149, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneX/1.29.163.13 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659427 = {S12377, 148, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCOneX/2.17.110.5 Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659840 = {S12378, 149, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCOneX/1.29.161.13 Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2700184 = {S12379, 157, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - HTC One X Linux/3.0.13 Android/4.1.9 Release/11.28.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2814413 = {S12380, 138, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC One X Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2815495 = {S12381, 138, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.6; en-us; HTC One X Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2848956 = {S12382, 149, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneX/1.29.161.13 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2849882 = {S12383, 137, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneX Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2849959 = {S12384, 147, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneX-orange-LS Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3093562 = {S12385, 166, { &P18295,&P18066,&P18062,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.0.4;dev=evita;devm=HTC One X;man=HTC;brw=Android;shwd=720x1280;shwa=720x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA2673811 = {S12386, 111, { &P15364,&P16110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe)*/
const Device UA2676471 = {S12387, 147, { &P15364,&P16110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2677412 = {S12388, 104, { &P15364,&P16110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.1.3.0 Safari/537.31*/
const Device UA2678392 = {S12389, 108, { &P15364,&P16110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2678407 = {S12390, 101, { &P15364,&P16110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2678503 = {S12391, 97, { &P15364,&P16110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.1.3.0 Safari/537.31*/
const Device UA2678687 = {S12392, 118, { &P15364,&P16110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe)*/
const Device UA2679928 = {S12393, 104, { &P15364,&P16110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.2.2.0 Safari/537.31*/
const Device UA2679941 = {S12394, 97, { &P15364,&P16110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.2.2.0 Safari/537.31*/
const Device UA2717636 = {S12395, 140, { &P15364,&P16110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2870156 = {S12396, 104, { &P15364,&P16110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/26.9.9.9 Safari/537.36*/
const Device UA2969040 = {S12397, 140, { &P15364,&P17285,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2971083 = {S12398, 147, { &P15364,&P17285,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3024754 = {S12399, 101, { &P15364,&P17285,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3025065 = {S12400, 108, { &P15364,&P17285,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3026537 = {S12401, 121, { &P15364,&P17285,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe),gzip(gfe)*/
const Device UA3027176 = {S12402, 118, { &P15364,&P17285,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe)*/
const Device UA3028913 = {S12403, 97, { &P15364,&P17285,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.1.0.0 Safari/537.36*/
const Device UA3029231 = {S12404, 104, { &P15364,&P17285,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.1.0.0 Safari/537.36*/
const Device UA3030006 = {S12405, 111, { &P15364,&P17285,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe)*/
const Device UA623682 = {S12406, 67, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5830i Build/GINGERBREAD)*/
const Device UA625080 = {S12407, 143, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA632142 = {S12408, 131, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA702613 = {S12409, 162, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-S5839i-ORANGE/S5839iBVLC4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA765160 = {S12410, 162, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-S5839i-ORANGE/S5839iBVLC3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA781747 = {S12411, 143, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5839i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA849591 = {S12412, 163, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5839i/S5839iBULC1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA902807 = {S12413, 163, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; SAMSUNG GT-S5839i/S5839iBOLC5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA908911 = {S12414, 163, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; SAMSUNG GT-S5830i/S5830iXXLB2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA924870 = {S12415, 163, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5839i/S5839iBULE2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1973486 = {S12416, 67, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5839i Build/GINGERBREAD)*/
const Device UA1973509 = {S12417, 162, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-S5839i-ORANGE/S5839iBVLD3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1986611 = {S12418, 163, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-S5839i/S5839iBOLE1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1992580 = {S12419, 136, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1999146 = {S12420, 163, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-S5839i/S5839iBOLG1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027000 = {S12421, 141, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2232555 = {S12422, 162, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-S5839i-ORANGE/S5839iBVLH1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2234782 = {S12423, 163, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5839i/S5839iBULI1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2344623 = {S12424, 166, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/RazoDroiD v1.0 by (rajrocks)rishee) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2352125 = {S12425, 177, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6(RazoDroiD); en-gb; GT-S5830i Build/RazoDroiD v2.0 by (rajrocks)rishee) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2381119 = {S12426, 141, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5839i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2382876 = {S12427, 140, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-S5830i Build/AceCream) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2383849 = {S12428, 150, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/MindCr Rom V1.0.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2384029 = {S12429, 163, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; SAMSUNG GT-S5830i/S5830iXXLI2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2407255 = {S12430, 163, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; SAMSUNG GT-S5839i/S5839iBOLI1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2410842 = {S12431, 163, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; sr-rs; SAMSUNG GT-S5830i/S5830iXXLA4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2427601 = {S12432, 163, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5839i/S5839iBULK1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2431872 = {S12433, 163, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; SAMSUNG GT-S5839i/S5839iBOLK2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2446104 = {S12434, 150, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/Retribution_beta#1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2452756 = {S12435, 152, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/Retribution Beta 3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2453847 = {S12436, 150, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/Retribution Beta 4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2469169 = {S12437, 162, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-S5839i-ORANGE/S5839iBVLK2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2469232 = {S12438, 163, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5839i/S5839iBULL1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2503507 = {S12439, 158, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT-S5830i Build/GingerCream-Berk Semsioglu) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2519845 = {S12440, 136, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2526633 = {S12441, 138, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-S5830i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2557634 = {S12442, 163, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; SAMSUNG GT-S5830i/S5830iXXLK3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2697573 = {S12443, 163, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-S5830i/S5830iXXLH3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2748897 = {S12444, 142, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-S5830i Build/830iv1.5FX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2784957 = {S12445, 163, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; SAMSUNG GT-S5839i/S5839iBOMD1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2846319 = {S12446, 142, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; GT-S5830i Build/830iv1.4FX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2935222 = {S12447, 146, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-S5830i Build/830iv1.76513FX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2957632 = {S12448, 163, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; SAMSUNG GT-S5830i/S5830iXXMD1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2984504 = {S12449, 146, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/BoudZDroid v.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2996823 = {S12450, 146, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-S5830i Build/830iv1.96513FX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3034472 = {S12451, 163, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; SAMSUNG GT-S5839i/S5839iBOMG1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3085783 = {S12452, 145, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-S5830i Build/BoudZDroid V1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3115701 = {S12453, 163, { &P20134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5839i/S5839iBUMG1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2461905 = {S12454, 119, { &P17595,&P18207,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17*/
const Device UA2492189 = {S12455, 119, { &P17595,&P18207,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492219 = {S12456, 119, { &P17595,&P18207,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492326 = {S12457, 119, { &P17595,&P18207,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2508791 = {S12458, 158, { &P17595,&P18207,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2512847 = {S12459, 129, { &P17595,&P18207,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)*/
const Device UA2700845 = {S12460, 125, { &P17595,&P18207,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2; en-US) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1309.0 Safari/537.17*/
const Device UA2781999 = {S12461, 119, { &P17595,&P18207,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA3079355 = {S12462, 119, { &P17595,&P18207,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2509572 = {S12463, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.14*/
const Device UA2509590 = {S12464, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.14*/
const Device UA2509600 = {S12465, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.14*/
const Device UA2510164 = {S12466, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.14*/
const Device UA2510172 = {S12467, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.14*/
const Device UA2510171 = {S12468, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.14*/
const Device UA2510174 = {S12469, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.14*/
const Device UA2510178 = {S12470, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.14*/
const Device UA2510786 = {S12471, 74, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.14,gzip(gfe)*/
const Device UA2510789 = {S12472, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.14*/
const Device UA2510792 = {S12473, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.14*/
const Device UA2510809 = {S12474, 67, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.14,gzip(gfe)*/
const Device UA2511315 = {S12475, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.14*/
const Device UA2511316 = {S12476, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.14*/
const Device UA2511318 = {S12477, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.14*/
const Device UA2511323 = {S12478, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.14*/
const Device UA2511324 = {S12479, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.14*/
const Device UA2511328 = {S12480, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.14*/
const Device UA2512436 = {S12481, 87, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.14*/
const Device UA2512777 = {S12482, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.14*/
const Device UA2512788 = {S12483, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.14*/
const Device UA2512791 = {S12484, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.14*/
const Device UA2512793 = {S12485, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.14*/
const Device UA2512795 = {S12486, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.14*/
const Device UA2513067 = {S12487, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.14*/
const Device UA2513301 = {S12488, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.14*/
const Device UA2513307 = {S12489, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.14*/
const Device UA2513348 = {S12490, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.14*/
const Device UA2513356 = {S12491, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.14*/
const Device UA2513358 = {S12492, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.14*/
const Device UA2513360 = {S12493, 75, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.14*/
const Device UA2513882 = {S12494, 87, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.14*/
const Device UA2514040 = {S12495, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.14*/
const Device UA2514058 = {S12496, 69, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64) Opera 12.14*/
const Device UA2514383 = {S12497, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.14*/
const Device UA2514385 = {S12498, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.14*/
const Device UA2514407 = {S12499, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.14*/
const Device UA2514556 = {S12500, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6046)) Presto/2.12.388 Version/12.14*/
const Device UA2515374 = {S12501, 87, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.14*/
const Device UA2515527 = {S12502, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.14*/
const Device UA2515783 = {S12503, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.14*/
const Device UA2515812 = {S12504, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.14*/
const Device UA2516053 = {S12505, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.14*/
const Device UA2516142 = {S12506, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5997)) Presto/2.12.388 Version/12.14*/
const Device UA2516227 = {S12507, 57, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.14*/
const Device UA2516350 = {S12508, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.14*/
const Device UA2516392 = {S12509, 64, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.14*/
const Device UA2516441 = {S12510, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.14*/
const Device UA2530809 = {S12511, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.14*/
const Device UA2531279 = {S12512, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.14*/
const Device UA2532482 = {S12513, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.14*/
const Device UA2533233 = {S12514, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4933)) Presto/2.12.388 Version/12.14*/
const Device UA2533240 = {S12515, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.14*/
const Device UA2535448 = {S12516, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.14*/
const Device UA2536408 = {S12517, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6044)) Presto/2.12.388 Version/12.14*/
const Device UA2536646 = {S12518, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.14*/
const Device UA2536866 = {S12519, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5660)) Presto/2.12.388 Version/12.14*/
const Device UA2536960 = {S12520, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.14*/
const Device UA2537097 = {S12521, 87, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.14*/
const Device UA2537099 = {S12522, 87, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.14*/
const Device UA2537171 = {S12523, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.14*/
const Device UA2537290 = {S12524, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.14*/
const Device UA2537477 = {S12525, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.14*/
const Device UA2537575 = {S12526, 74, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.14 Yandex.Translate*/
const Device UA2537652 = {S12527, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.14*/
const Device UA2537731 = {S12528, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.14*/
const Device UA2537770 = {S12529, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.14*/
const Device UA2537774 = {S12530, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6035)) Presto/2.12.388 Version/12.14*/
const Device UA2537901 = {S12531, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.14*/
const Device UA2539484 = {S12532, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.14*/
const Device UA2539581 = {S12533, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.14*/
const Device UA2558599 = {S12534, 87, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.14*/
const Device UA2559077 = {S12535, 103, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.14,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2559079 = {S12536, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.14*/
const Device UA2559442 = {S12537, 87, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03658)) Presto/2.12.388 Version/12.14*/
const Device UA2560658 = {S12538, 87, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03789)) Presto/2.12.388 Version/12.14*/
const Device UA2561141 = {S12539, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5299)) Presto/2.12.388 Version/12.14*/
const Device UA2561496 = {S12540, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.14*/
const Device UA2562513 = {S12541, 87, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5217)) Presto/2.12.388 Version/12.14*/
const Device UA2563112 = {S12542, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4832)) Presto/2.12.388 Version/12.14*/
const Device UA2563420 = {S12543, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.14*/
const Device UA2563954 = {S12544, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5996)) Presto/2.12.388 Version/12.14*/
const Device UA2564084 = {S12545, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5288)) Presto/2.12.388 Version/12.14*/
const Device UA2564150 = {S12546, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.14*/
const Device UA2564154 = {S12547, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5650)) Presto/2.12.388 Version/12.14*/
const Device UA2564711 = {S12548, 96, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.14,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2568011 = {S12549, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.14*/
const Device UA2568194 = {S12550, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.14*/
const Device UA2571451 = {S12551, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.14*/
const Device UA2571561 = {S12552, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6032)) Presto/2.12.388 Version/12.14*/
const Device UA2572165 = {S12553, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5988)) Presto/2.12.388 Version/12.14*/
const Device UA2572189 = {S12554, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.14*/
const Device UA2580750 = {S12555, 87, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5230)) Presto/2.12.388 Version/12.14*/
const Device UA2581342 = {S12556, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4661)) Presto/2.12.388 Version/12.14*/
const Device UA2581429 = {S12557, 87, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5300)) Presto/2.12.388 Version/12.14*/
const Device UA2582146 = {S12558, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.14*/
const Device UA2585907 = {S12559, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6080)) Presto/2.12.388 Version/12.14*/
const Device UA2585982 = {S12560, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.14*/
const Device UA2598863 = {S12561, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4110)) Presto/2.12.388 Version/12.14*/
const Device UA2599138 = {S12562, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5336)) Presto/2.12.388 Version/12.14*/
const Device UA2599373 = {S12563, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5734)) Presto/2.12.388 Version/12.14*/
const Device UA2599672 = {S12564, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03791)) Presto/2.12.388 Version/12.14*/
const Device UA2599856 = {S12565, 87, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03639)) Presto/2.12.388 Version/12.14*/
const Device UA2600409 = {S12566, 87, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5325)) Presto/2.12.388 Version/12.14*/
const Device UA2602196 = {S12567, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.14*/
const Device UA2602454 = {S12568, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5278)) Presto/2.12.388 Version/12.14*/
const Device UA2606403 = {S12569, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.14*/
const Device UA2661589 = {S12570, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03672)) Presto/2.12.388 Version/12.14*/
const Device UA2661816 = {S12571, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.6 (build 03402)) Presto/2.12.388 Version/12.14*/
const Device UA2662887 = {S12572, 107, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03686); MRSPUTNIK OW 2, 3, 0, 244) Presto/2.12.388 Version/12.14*/
const Device UA2674370 = {S12573, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.14*/
const Device UA2699198 = {S12574, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03780)) Presto/2.12.388 Version/12.14*/
const Device UA2701230 = {S12575, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03638)) Presto/2.12.388 Version/12.14*/
const Device UA2703098 = {S12576, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03755)) Presto/2.12.388 Version/12.14*/
const Device UA2703618 = {S12577, 80, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.14*/
const Device UA2716078 = {S12578, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.14*/
const Device UA2718876 = {S12579, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.14*/
const Device UA2750007 = {S12580, 96, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.14,gzip(gfe)*/
const Device UA2754594 = {S12581, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.14*/
const Device UA2769053 = {S12582, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6561)) Presto/2.12.388 Version/12.14*/
const Device UA2792948 = {S12583, 113, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.14*/
const Device UA2858094 = {S12584, 87, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5161)) Presto/2.12.388 Version/12.14*/
const Device UA2863029 = {S12585, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 6014)) Presto/2.12.388 Version/12.14*/
const Device UA2867703 = {S12586, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6573)) Presto/2.12.388 Version/12.14*/
const Device UA2868308 = {S12587, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6283)) Presto/2.12.388 Version/12.14*/
const Device UA2868650 = {S12588, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.1 (build 6261)) Presto/2.12.388 Version/12.14*/
const Device UA2877257 = {S12589, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6638)) Presto/2.12.388 Version/12.14*/
const Device UA2895381 = {S12590, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6628)) Presto/2.12.388 Version/12.14*/
const Device UA2898123 = {S12591, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.14*/
const Device UA2944303 = {S12592, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6657)) Presto/2.12.388 Version/12.14*/
const Device UA2964939 = {S12593, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6665)) Presto/2.12.388 Version/12.14*/
const Device UA3003036 = {S12594, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.1 (build 6322)) Presto/2.12.388 Version/12.14*/
const Device UA3003248 = {S12595, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.1 (build 6323)) Presto/2.12.388 Version/12.14*/
const Device UA3015659 = {S12596, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.1 (build 6341)) Presto/2.12.388 Version/12.14*/
const Device UA3017229 = {S12597, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6648)) Presto/2.12.388 Version/12.14*/
const Device UA3020235 = {S12598, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6669)) Presto/2.12.388 Version/12.14*/
const Device UA3020499 = {S12599, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6673)) Presto/2.12.388 Version/12.14*/
const Device UA3023087 = {S12600, 84, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.14,gzip(gfe),gzip(gfe)*/
const Device UA3033266 = {S12601, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.14*/
const Device UA3058155 = {S12602, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6675)) Presto/2.12.388 Version/12.14*/
const Device UA3058247 = {S12603, 86, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6337)) Presto/2.12.388 Version/12.14*/
const Device UA3058276 = {S12604, 77, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.14,gzip(gfe),gzip(gfe)*/
const Device UA3105466 = {S12605, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.1 (build 6327)) Presto/2.12.388 Version/12.14*/
const Device UA3107924 = {S12606, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6676)) Presto/2.12.388 Version/12.14*/
const Device UA3125355 = {S12607, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6681)) Presto/2.12.388 Version/12.14*/
const Device UA3125471 = {S12608, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.1 (build 6347)) Presto/2.12.388 Version/12.14*/
const Device UA3128121 = {S12609, 79, { &P15364,&P7133,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.2 (build 7194)) Presto/2.12.388 Version/12.14*/
const Device UA2357458 = {S12610, 101, { &P15364,&P17285,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2357466 = {S12611, 108, { &P15364,&P17285,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2387483 = {S12612, 97, { &P15364,&P17285,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.1.0.0 Safari/537.11*/
const Device UA2387582 = {S12613, 104, { &P15364,&P17285,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.1.0.0 Safari/537.11*/
const Device UA2408432 = {S12614, 111, { &P15364,&P17285,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe)*/
const Device UA2427676 = {S12615, 147, { &P15364,&P17285,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2427812 = {S12616, 104, { &P15364,&P17285,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.2.0.0 Safari/537.11*/
const Device UA2429898 = {S12617, 97, { &P15364,&P17285,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.4.0.0 Safari/537.11*/
const Device UA2429917 = {S12618, 104, { &P15364,&P17285,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.4.0.0 Safari/537.11*/
const Device UA2432922 = {S12619, 118, { &P15364,&P17285,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe)*/
const Device UA2435494 = {S12620, 140, { &P15364,&P17285,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2436402 = {S12621, 97, { &P15364,&P17285,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.2.0.0 Safari/537.11*/
const Device UA2437232 = {S12622, 97, { &P15364,&P17285,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.3.0.0 Safari/537.11*/
const Device UA2437323 = {S12623, 104, { &P15364,&P17285,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.3.0.0 Safari/537.11*/
const Device UA2467070 = {S12624, 100, { &P15364,&P17285,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2957846 = {S12625, 106, { &P15364,&P17285,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) yi; AppleWebKit/X.12221 (KHTML, like Gecko) Chrome/23.0.1271.26 Safari/X.1221*/
const Device UA2873609 = {S12626, 120, { &P17595,&P17297,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2873693 = {S12627, 120, { &P17595,&P17297,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2873707 = {S12628, 120, { &P17595,&P17297,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2873758 = {S12629, 120, { &P17595,&P17297,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2873982 = {S12630, 120, { &P17595,&P17297,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2875349 = {S12631, 120, { &P17595,&P17297,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2876065 = {S12632, 130, { &P17595,&P17297,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe)*/
const Device UA2882382 = {S12633, 135, { &P17595,&P17297,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36*/
const Device UA2886195 = {S12634, 159, { &P17595,&P17297,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2357460 = {S12635, 101, { &P15364,&P16110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2387461 = {S12636, 97, { &P15364,&P16110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.1.0.0 Safari/537.11*/
const Device UA2387615 = {S12637, 104, { &P15364,&P16110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.1.0.0 Safari/537.11*/
const Device UA2408293 = {S12638, 108, { &P15364,&P16110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2428537 = {S12639, 104, { &P15364,&P16110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.4.0.0 Safari/537.11*/
const Device UA2429505 = {S12640, 97, { &P15364,&P16110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.2.0.0 Safari/537.11*/
const Device UA2430531 = {S12641, 97, { &P15364,&P16110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.4.0.0 Safari/537.11*/
const Device UA2433561 = {S12642, 147, { &P15364,&P16110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2434334 = {S12643, 118, { &P15364,&P16110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe)*/
const Device UA2435962 = {S12644, 111, { &P15364,&P16110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe)*/
const Device UA2436380 = {S12645, 104, { &P15364,&P16110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.2.0.0 Safari/537.11*/
const Device UA2437242 = {S12646, 104, { &P15364,&P16110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.3.0.0 Safari/537.11*/
const Device UA2437270 = {S12647, 97, { &P15364,&P16110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.3.0.0 Safari/537.11*/
const Device UA2460472 = {S12648, 140, { &P15364,&P16110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2556754 = {S12649, 107, { &P15364,&P16110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.11 (KHTML like Gecko) Chrome/23.0.1271.91 Safari/537.11*/
const Device UA102535 = {S12650, 119, { &P17595,&P17294,&P17335,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/533.16 (KHTML, like Gecko) Version/4.1 Safari/533.16*/
const Device UA120919 = {S12651, 125, { &P17595,&P17294,&P17335,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/4.1.1 Safari/533.17.8*/
const Device UA132795 = {S12652, 125, { &P17595,&P17294,&P17335,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/4.1.2 Safari/533.18.5*/
const Device UA153594 = {S12653, 125, { &P17595,&P17294,&P17335,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/4.1.3 Safari/533.19.4*/
const Device UA161747 = {S12654, 121, { &P17595,&P17294,&P17335,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko, Safari/4533.19.4) ADM/722*/
const Device UA2270068 = {S12655, 119, { &P17595,&P17294,&P17335,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; de) AppleWebKit/528.4+ (KHTML, like Gecko) Version/4.1 Safari/526.11.2*/
const Device UA2298220 = {S12656, 122, { &P17595,&P17294,&P17335,&P18092 }};	/*Useragent - Mozilla/5.0(Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/4.1.3 Safari/533.19.4*/
const Device UA2684326 = {S12657, 127, { &P17595,&P17294,&P17335,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10\_4\_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/4.1.3 Safari/533.19.4*/
const Device UA458589 = {S12658, 139, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA458595 = {S12659, 132, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA458600 = {S12660, 61, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-P5100 Build/IML74K)*/
const Device UA895400 = {S12661, 152, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; SAMSUNG GT-P5100/P5100BUALD5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA923928 = {S12662, 132, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1982028 = {S12663, 152, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SAMSUNG GT-P5100/P5100XWBLE4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026133 = {S12664, 61, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P5100 Build/IMM76D)*/
const Device UA2030093 = {S12665, 151, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P5100-ORANGE/P5100BVBLF3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2168046 = {S12666, 130, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2194711 = {S12667, 151, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P5100-ORANGE/P5100BVBLF5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2203363 = {S12668, 130, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2236364 = {S12669, 152, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; SAMSUNG GT-P5100/P5100BUBLH2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2353920 = {S12670, 151, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P5100-ORANGE/P5100BVBLH1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2361142 = {S12671, 132, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T779 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2608658 = {S12672, 152, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; sl-si; SAMSUNG GT-P5100/P5100BUALD4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2610977 = {S12673, 79, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/1.1.3 (Linux; U; Android 4.0.3; en-GB; GT-P5100 Build/IML74K.P5100ZSALD5)*/
const Device UA2612082 = {S12674, 130, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5100 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2613473 = {S12675, 92, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.youtube/3.5.5(Linux; U; Android 4.0.3; en_GB; GT-P5100 Build/IML74K) gzip*/
const Device UA2613502 = {S12676, 151, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100-ORANGE/P5100BVBLE1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2613736 = {S12677, 153, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5100 Build/IML74K; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2614176 = {S12678, 152, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hr-hr; SAMSUNG GT-P5100/P5100XWBLE2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2614597 = {S12679, 130, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5100 Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2614729 = {S12680, 130, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; xx-xx; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2614865 = {S12681, 130, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5100 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2615056 = {S12682, 79, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.0.3; en-GB; GT-P5100 Build/IML74K.P5100XWALD7)*/
const Device UA2615217 = {S12683, 152, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; SAMSUNG GT-P5100/P5100XXBLE1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615454 = {S12684, 153, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5100 Build/IMM76D; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2615715 = {S12685, 155, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SAMSUNG GT-P5100/P5100XWBLE2.DM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615838 = {S12686, 150, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG GT-P5100/P5100XWBLE4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615988 = {S12687, 150, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; SAMSUNG GT-P5100/P5100BUALD5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616484 = {S12688, 94, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.0.11 (Linux; U; Android 4.0.3; ru-RU; GT-P5100 Build/IML74K.P5100XXALD7)*/
const Device UA2616587 = {S12689, 151, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; GT-P5100-ORANGE/P5100BVALD1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616877 = {S12690, 152, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; SAMSUNG GT-P5100/P5100BUALC1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616949 = {S12691, 218, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-za; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.2.1-65 (Android 4.0.4; samsung GT-P5100 Build/IMM76D.P5100XWBLE4)*/
const Device UA2617061 = {S12692, 152, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; SAMSUNG GT-P5100/P5100BUBLF1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617069 = {S12693, 79, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.0.4; zh-HK; GT-P5100 Build/IMM76D.P5100ZSBLH3)*/
const Device UA2617143 = {S12694, 241, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 VoiceGO!(cybd08dd26fee94a5b4e5c27aa93e80fe56-0;Android,4.0.4,GT-P5100;3.6.0;1280x752,1.0;X;VoiceGo-TC;zh-TW)*/
const Device UA2617350 = {S12695, 145, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2617396 = {S12696, 93, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.1.2 (Linux; U; Android 4.0.3; en-US; GT-P5100 Build/IML74K.P5100XWALD2)*/
const Device UA2617643 = {S12697, 152, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; SAMSUNG GT-P5100/P5100XXBLI4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617668 = {S12698, 134, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5100 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2617740 = {S12699, 103, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 4.0.3; SAMSUNG GT-P5100 Build/ICE_CREAM_SANDWICH_MR1);*/
const Device UA2617781 = {S12700, 162, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ms-my; GT-P5100 Build/IML74K) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2618071 = {S12701, 155, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; SAMSUNG GT-P5100/P5100XWBLE4.DM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2618370 = {S12702, 134, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5100 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2618395 = {S12703, 80, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Biblia/2.2.3 (Linux; U; Android 4.0.4; es-US; GT-P5100 Build/IMM76D.P5100XXBLH4)*/
const Device UA2618417 = {S12704, 79, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Vyrso/2.2.3 (Linux; U; Android 4.0.3; en-GB; GT-P5100 Build/IML74K.P5100XWALD4)*/
const Device UA2618788 = {S12705, 83, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - MediaPlayer/2002/4.0.12 (Android 4.0.3; espresso10rf; GT-P5100; samsung; ; ; zh_CN)*/
const Device UA2619165 = {S12706, 80, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Biblia/2.2.3 (Linux; U; Android 4.0.3; es-ES; GT-P5100 Build/IML74K.P5100UBALD4)*/
const Device UA2619183 = {S12707, 273, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/1.9.10;FBDM/{density=1.0,width=1280,height=752};FBLC/en_GB;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P5100;FBSV/4.0.3;]*/
const Device UA2619370 = {S12708, 63, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.0; 12) GT-P5100 (Android 4.0.4, SDK 15)*/
const Device UA2619866 = {S12709, 139, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2619910 = {S12710, 80, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Biblia/2.1.2 (Linux; U; Android 4.0.3; es-US; GT-P5100 Build/IML74K.P5100UBALE1)*/
const Device UA2619912 = {S12711, 132, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2620054 = {S12712, 152, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; SAMSUNG GT-P5100/P5100BUBLG4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2620933 = {S12713, 137, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2621667 = {S12714, 280, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/1.6.4;FBPN/com.facebook.katana;FBDV/GT-P5100;FBSV/4.0.3;FBDM/{density=1.0,width=1280,height=752};FBCR/MY MAXIS;FBLC/en_GB;FB_FW/1;]*/
const Device UA2621711 = {S12715, 81, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.0.3; en-GB; GT-P5100 Build/IML74K.P5100XWALE1)*/
const Device UA2621914 = {S12716, 81, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-GB; GT-P5100 Build/IMM76D.P5100XWBLF1)*/
const Device UA2621924 = {S12717, 152, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; SAMSUNG GT-P5100/P5100XWBLH2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622101 = {S12718, 204, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Android_Gmarket Qoo10 MY_2.0.8_25(f6ce56dee1cbe648____X;GT-P5100;4.0.3)*/
const Device UA2622123 = {S12719, 82, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Biblia/2.2.8.1 (Linux; U; Android 4.0.4; es-US; GT-P5100 Build/IMM76D.P5100UBBLJ1)*/
const Device UA2622237 = {S12720, 81, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-GB; GT-P5100 Build/IMM76D.P5100XWBLE4)*/
const Device UA2622393 = {S12721, 144, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; GT-P5100 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2622404 = {S12722, 142, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K like Xoom) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622430 = {S12723, 150, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG GT-P5100/P5100BUBLH2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622920 = {S12724, 52, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - nApps (Android 4.0.3; GT-P5100; Android.Line; 2.5.4)*/
const Device UA2623494 = {S12725, 81, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.0.3; en-GB; GT-P5100 Build/IML74K.P5100XWALE2)*/
const Device UA2623525 = {S12726, 81, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-GB; GT-P5100 Build/IMM76D.P5100XXBLH5)*/
const Device UA2623561 = {S12727, 81, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-GB; GT-P5100 Build/IMM76D.P5100XWBLF3)*/
const Device UA2623743 = {S12728, 80, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - VKAndroidApp/2.7.5-62 (Android 4.0.4; SDK 15; armeabi-v7a; samsung GT-P5100; ru)*/
const Device UA2624075 = {S12729, 81, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.0.4; es-ES; GT-P5100 Build/IMM76D.P5100UBBLH2)*/
const Device UA2624232 = {S12730, 164, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; gbr-gbr; GT-P5100 Build/IML74K) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2624273 = {S12731, 132, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Groupon/2.0.2414 (Android 4.0.3; Samsung Espresso10rf / Samsung GT-P5100; )[preload=false;locale=en_GB;clientidbase=android-samsung]*/
const Device UA2624303 = {S12732, 81, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Vyrso/2.2.8.1 (Linux; U; Android 4.0.3; en-AU; GT-P5100 Build/IML74K.P5100XWALE3)*/
const Device UA2624692 = {S12733, 69, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Showtimezz@MY/1.4.8 (Android 4.0.3; samsung-GT-P5100%404.0.3 Build/15*/
const Device UA2625055 = {S12734, 77, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mrd/1.1.0 (Linux; U; Android 4.0.3; samsung GT-P5100) jp.co.paperboy.picto/37*/
const Device UA2625629 = {S12735, 54, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-GB; GT-P5100)*/
const Device UA2625894 = {S12736, 252, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 APPDIY/2.1.10 (zh-CN;800*1232*1.0;X/X/X) zhoujielun/2.2.0.4491 (Id:298,Date:2013-01-08 20:03:27,Channel:anzhuoshichang)*/
const Device UA2627153 = {S12737, 145, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2627556 = {S12738, 157, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-za; GT-P5100 Build/IMM76D) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 Whisk/3.8.1204.57*/
const Device UA2628274 = {S12739, 80, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.10 (Linux; U; Android 4.0.3; es-US; GT-P5100 Build/IML74K.P5100UBALD5)*/
const Device UA2628450 = {S12740, 152, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; SAMSUNG GT-P5100/P5100XWALD7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2628914 = {S12741, 162, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ph; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2629027 = {S12742, 144, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P5100 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2629194 = {S12743, 157, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IceCreamSandwich_mr1) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2630234 = {S12744, 133, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ph; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Vet-Encoding: gzip,deflate*/
const Device UA2630666 = {S12745, 218, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.2.1-65 (Android 4.0.3; samsung GT-P5100 Build/IML74K.P5100XWALD7)*/
const Device UA2631611 = {S12746, 132, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/IMM76D/534.30 (KHTML, like Gecko) Verse Gecko) Version/4.0 Safari/534.30*/
const Device UA2632159 = {S12747, 182, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K; samsung; mediatap) 752X1280 samsung GT-P5100 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2633334 = {S12748, 152, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; SAMSUNG GT-P5100/P5100XWALE2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2635513 = {S12749, 125, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2635844 = {S12750, 132, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5100 Build/IMM76D) AppleWebKit/534ment;      this.validation = val4.0 Safari/534.30*/
const Device UA2636979 = {S12751, 82, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - MediaPlayer/2002/4.5.0 (Android 4.0.3; espresso10rf; GT-P5100; samsung; ; ; en_GB)*/
const Device UA2637196 = {S12752, 137, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-za; GT-P5100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2637564 = {S12753, 92, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.youtube/3.5.5(Linux; U; Android 4.0.4; en_GB; GT-P5100 Build/IMM76D) gzip*/
const Device UA2637565 = {S12754, 151, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.2 Version/4.0 Mobile Safari/530.17*/
const Device UA2799940 = {S12755, 151, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P5100-ORANGE/P5100BVBLL5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2843659 = {S12756, 61, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-T779 Build/IMM76D)*/
const Device UA3031356 = {S12757, 152, { &P15876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SAMSUNG GT-P5100/P5100XXBLK2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3067633 = {S12758, 101, { &P15364,&P17285,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36*/
const Device UA3067756 = {S12759, 108, { &P15364,&P17285,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36*/
const Device UA3068154 = {S12760, 111, { &P15364,&P17285,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe)*/
const Device UA3082364 = {S12761, 118, { &P15364,&P17285,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe)*/
const Device UA3093913 = {S12762, 121, { &P15364,&P17285,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36,gzip(gfe),gzip(gfe)*/
const Device UA3094585 = {S12763, 117, { &P15364,&P17285,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Iron/29.0.1600.1 Chrome/29.0.1600.1 Safari/537.36*/
const Device UA3111001 = {S12764, 124, { &P15364,&P17285,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/29.0.1600.1 Chrome/29.0.1600.1 Safari/537.36*/
const Device UA3116375 = {S12765, 140, { &P15364,&P17285,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3127774 = {S12766, 147, { &P15364,&P17285,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2488883 = {S12767, 104, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144*/
const Device UA2489292 = {S12768, 104, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141*/
const Device UA2490032 = {S12769, 132, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 iPod4,1 BingWeb/3.03.1428.X*/
const Device UA2490736 = {S12770, 257, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 6_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.1;FBSS/2; FBCR/unavailable;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA2492712 = {S12771, 224, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 [FBAN/FBIOS;FBAV/5.4.2;FBBV/X;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]*/
const Device UA2509710 = {S12772, 224, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 [FBAN/FBIOS;FBAV/5.4.2;FBBV/X;FBDV/iPod5,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]*/
const Device UA2513850 = {S12773, 90, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2515660 = {S12774, 134, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 iPod5,1 BingWeb/3.03.1428.X*/
const Device UA2532509 = {S12775, 106, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146*/
const Device UA2533053 = {S12776, 134, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 iPod4,1 BingWeb/3.03.1428.X*/
const Device UA2539898 = {S12777, 233, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 [FBAN/FBIOS;FBAV/5.5;FBBV/X;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.1.2;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBOP/0]*/
const Device UA2561836 = {S12778, 136, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 iPod5,1 BingWeb/3.03.1428.X*/
const Device UA2565361 = {S12779, 92, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2565380 = {S12780, 261, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 6_1_2 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.1.2;FBSS/2; FBCR/unavailable;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA2583024 = {S12781, 135, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod4,1; iPhone; U; CPU OS 6_1_2 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N81AP (gzip)*/
const Device UA2585300 = {S12782, 106, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329*/
const Device UA2585643 = {S12783, 134, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 iPod4,1 BingWeb/3.03.1428.X*/
const Device UA2606989 = {S12784, 136, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 iPod5,1 BingWeb/3.03.1428.X*/
const Device UA2607368 = {S12785, 261, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 6_1_3 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.1.3;FBSS/2; FBCR/unavailable;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA2607382 = {S12786, 92, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2703838 = {S12787, 147, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 bline/1.06 (iPhone OS 6.1.3, iPod touch)*/
const Device UA2737341 = {S12788, 142, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone 6_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.8 (iPod4,1/6.1.3)*/
const Device UA2765175 = {S12789, 48, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - AppCake 3.8 (iPod touch; iPhone OS 6.1.2; en_US)*/
const Device UA2846475 = {S12790, 106, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B5126b*/
const Device UA2851294 = {S12791, 171, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Safari IPhoneAppVersion/2.5.2 IPhoneWebkitWrapper BillingSupported*/
const Device UA2936101 = {S12792, 121, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/8536.25*/
const Device UA2942935 = {S12793, 135, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod4,1; iPhone; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/N81AP (gzip)*/
const Device UA2946848 = {S12794, 233, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 [FBAN/FBIOS;FBAV/6.3;FBBV/X;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.1.3;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBOP/1]*/
const Device UA2952825 = {S12795, 46, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - AppCake 3.8 (iPod touch; iPhone OS 6.1; en_US)*/
const Device UA2961513 = {S12796, 135, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod5,1; iPhone; U; CPU OS 6_1_3 like Mac OS X; en_US) com.google.GooglePlus/19307 (KHTML, like Gecko) Mobile/N78AP (gzip)*/
const Device UA3046537 = {S12797, 173, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari IPhoneAppVersion/2.5.2 IPhoneWebkitWrapper BillingSupported*/
const Device UA3058305 = {S12798, 173, { &P17346,&P19826,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari IPhoneAppVersion/2.6.1 IPhoneWebkitWrapper BillingSupported*/
const Device UA3062783 = {S12799, 119, { &P17595,&P17297,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3062787 = {S12800, 119, { &P17595,&P17297,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3062791 = {S12801, 119, { &P17595,&P17297,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3062792 = {S12802, 119, { &P17595,&P17297,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3062798 = {S12803, 119, { &P17595,&P17297,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3065630 = {S12804, 119, { &P17595,&P17297,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3065635 = {S12805, 129, { &P17595,&P17297,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36,gzip(gfe)*/
const Device UA3081265 = {S12806, 158, { &P17595,&P17297,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3126582 = {S12807, 135, { &P17595,&P17297,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Iron/29.0.1600.1 Chrome/29.0.1600.1 Safari/537.36*/
const Device UA777007 = {S12808, 64, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.289 Version/12.00*/
const Device UA777889 = {S12809, 72, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; en; rv:2.0) Gecko/X Firefox/4.0 Opera 12.00*/
const Device UA808600 = {S12810, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.289 Version/12.00*/
const Device UA811427 = {S12811, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.10.289 Version/12.00*/
const Device UA848557 = {S12812, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.289 Version/12.00*/
const Device UA871689 = {S12813, 66, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; en) Opera 12.00*/
const Device UA899043 = {S12814, 64, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; en) Presto/2.10.289 Version/12.00*/
const Device UA899467 = {S12815, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5328); ru) Presto/2.10.289 Version/12.00*/
const Device UA899660 = {S12816, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.289 Version/12.00*/
const Device UA899839 = {S12817, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.289 Version/12.00*/
const Device UA903006 = {S12818, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.10.289 Version/12.00*/
const Device UA903142 = {S12819, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.10.289 Version/12.00*/
const Device UA903156 = {S12820, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.10.289 Version/12.00*/
const Device UA903339 = {S12821, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.289 Version/12.00*/
const Device UA903356 = {S12822, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.289 Version/12.00*/
const Device UA903387 = {S12823, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.289 Version/12.00*/
const Device UA903392 = {S12824, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.289 Version/12.00*/
const Device UA903414 = {S12825, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.289 Version/12.00*/
const Device UA903531 = {S12826, 101, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.289 Version/12.00,gzip(gfe) (via translate.google.com)*/
const Device UA903548 = {S12827, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.289 Version/12.00*/
const Device UA903566 = {S12828, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.289 Version/12.00*/
const Device UA903572 = {S12829, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.289 Version/12.00*/
const Device UA903595 = {S12830, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.289 Version/12.00*/
const Device UA903641 = {S12831, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.289 Version/12.00*/
const Device UA904048 = {S12832, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.289 Version/12.00*/
const Device UA904065 = {S12833, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4930); ru) Presto/2.10.289 Version/12.00*/
const Device UA904068 = {S12834, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.289 Version/12.00*/
const Device UA904117 = {S12835, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.289 Version/12.00*/
const Device UA904265 = {S12836, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.289 Version/12.00*/
const Device UA904533 = {S12837, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5667); ru) Presto/2.10.289 Version/12.00*/
const Device UA904638 = {S12838, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.289 Version/12.00*/
const Device UA904832 = {S12839, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.289 Version/12.00*/
const Device UA905065 = {S12840, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.289 Version/12.00*/
const Device UA905071 = {S12841, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5161); ru) Presto/2.10.289 Version/12.00*/
const Device UA905458 = {S12842, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.289 Version/12.00*/
const Device UA905466 = {S12843, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.289 Version/12.00*/
const Device UA905474 = {S12844, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5676); ru) Presto/2.10.289 Version/12.00*/
const Device UA905543 = {S12845, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.289 Version/12.00*/
const Device UA905569 = {S12846, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.289 Version/12.00*/
const Device UA905755 = {S12847, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.289 Version/12.00*/
const Device UA905975 = {S12848, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.289 Version/12.00*/
const Device UA906275 = {S12849, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.289 Version/12.00*/
const Device UA906360 = {S12850, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.289 Version/12.00*/
const Device UA906423 = {S12851, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5299); ru) Presto/2.10.289 Version/12.00*/
const Device UA906582 = {S12852, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5218); ru) Presto/2.10.289 Version/12.00*/
const Device UA906662 = {S12853, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5660); ru) Presto/2.10.289 Version/12.00*/
const Device UA906698 = {S12854, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02746); ru) Presto/2.10.289 Version/12.00*/
const Device UA906704 = {S12855, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5230); ru) Presto/2.10.289 Version/12.00*/
const Device UA906734 = {S12856, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.289 Version/12.00*/
const Device UA906960 = {S12857, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5650); ru) Presto/2.10.289 Version/12.00*/
const Device UA907095 = {S12858, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5217); ru) Presto/2.10.289 Version/12.00*/
const Device UA907118 = {S12859, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.289 Version/12.00*/
const Device UA907147 = {S12860, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.289 Version/12.00*/
const Device UA907269 = {S12861, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5336); ru) Presto/2.10.289 Version/12.00*/
const Device UA908263 = {S12862, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5288); ru) Presto/2.10.289 Version/12.00*/
const Device UA909605 = {S12863, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.289 Version/12.00*/
const Device UA911579 = {S12864, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.289 Version/12.00*/
const Device UA924850 = {S12865, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.10.289 Version/12.00*/
const Device UA1949738 = {S12866, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5300); ru) Presto/2.10.289 Version/12.00*/
const Device UA1954131 = {S12867, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.10.289 Version/12.00*/
const Device UA1957564 = {S12868, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.289 Version/12.00*/
const Device UA1959399 = {S12869, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.10.289 Version/12.00*/
const Device UA1965208 = {S12870, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4661); ru) Presto/2.10.289 Version/12.00*/
const Device UA1965881 = {S12871, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.10.289 Version/12.00*/
const Device UA1978745 = {S12872, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.10.289 Version/12.00*/
const Device UA1988598 = {S12873, 124, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.289 Version/12.00,gzip(gfe) (via translate.google.com)*/
const Device UA1993208 = {S12874, 64, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; en) Presto/2.10.289 Version/12.01*/
const Device UA1994953 = {S12875, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5278); ru) Presto/2.10.289 Version/12.00*/
const Device UA1997115 = {S12876, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.289 Version/12.01*/
const Device UA1997120 = {S12877, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.289 Version/12.01*/
const Device UA1997194 = {S12878, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.10.289 Version/12.01*/
const Device UA1997201 = {S12879, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4930); ru) Presto/2.10.289 Version/12.01*/
const Device UA1997807 = {S12880, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.289 Version/12.01*/
const Device UA1997813 = {S12881, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.289 Version/12.01*/
const Device UA1997819 = {S12882, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5650); ru) Presto/2.10.289 Version/12.01*/
const Device UA1997824 = {S12883, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.289 Version/12.01*/
const Device UA1997826 = {S12884, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.10.289 Version/12.01*/
const Device UA1997832 = {S12885, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.289 Version/12.01*/
const Device UA1998518 = {S12886, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.289 Version/12.01*/
const Device UA1998519 = {S12887, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.10.289 Version/12.01*/
const Device UA1998527 = {S12888, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.289 Version/12.01*/
const Device UA1998542 = {S12889, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.289 Version/12.01*/
const Device UA1998543 = {S12890, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.289 Version/12.01*/
const Device UA1999025 = {S12891, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.289 Version/12.01*/
const Device UA1999029 = {S12892, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.10.289 Version/12.01*/
const Device UA1999145 = {S12893, 64, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; en) Presto/2.10.289 Version/12.01*/
const Device UA1999464 = {S12894, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.289 Version/12.01*/
const Device UA1999465 = {S12895, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5667); ru) Presto/2.10.289 Version/12.01*/
const Device UA1999486 = {S12896, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.289 Version/12.01*/
const Device UA1999892 = {S12897, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.289 Version/12.01*/
const Device UA2000871 = {S12898, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.10.289 Version/12.01*/
const Device UA2000872 = {S12899, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.289 Version/12.01*/
const Device UA2000873 = {S12900, 101, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.289 Version/12.01,gzip(gfe) (via translate.google.com)*/
const Device UA2001745 = {S12901, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.289 Version/12.01*/
const Device UA2001953 = {S12902, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.10.289 Version/12.00*/
const Device UA2001993 = {S12903, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.289 Version/12.01*/
const Device UA2002418 = {S12904, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.289 Version/12.01*/
const Device UA2002622 = {S12905, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.289 Version/12.01*/
const Device UA2002932 = {S12906, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.289 Version/12.01*/
const Device UA2002934 = {S12907, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.289 Version/12.01*/
const Device UA2003271 = {S12908, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.289 Version/12.01*/
const Device UA2003820 = {S12909, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.289 Version/12.01*/
const Device UA2003838 = {S12910, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.289 Version/12.01*/
const Device UA2003841 = {S12911, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5218); ru) Presto/2.10.289 Version/12.01*/
const Device UA2004274 = {S12912, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.289 Version/12.01*/
const Device UA2004630 = {S12913, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5328); ru) Presto/2.10.289 Version/12.01*/
const Device UA2004640 = {S12914, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.289 Version/12.01*/
const Device UA2004660 = {S12915, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.289 Version/12.01*/
const Device UA2004672 = {S12916, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5676); ru) Presto/2.10.289 Version/12.01*/
const Device UA2004674 = {S12917, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.289 Version/12.01*/
const Device UA2004690 = {S12918, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.289 Version/12.01*/
const Device UA2007358 = {S12919, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.289 Version/12.01*/
const Device UA2007367 = {S12920, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5660); ru) Presto/2.10.289 Version/12.01*/
const Device UA2008879 = {S12921, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.10.289 Version/12.01*/
const Device UA2009019 = {S12922, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.10.289 Version/12.01*/
const Device UA2012018 = {S12923, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5288); ru) Presto/2.10.289 Version/12.01*/
const Device UA2012040 = {S12924, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.289 Version/12.01*/
const Device UA2012688 = {S12925, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5966); ru) Presto/2.10.289 Version/12.01*/
const Device UA2015305 = {S12926, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.289 Version/12.01*/
const Device UA2042858 = {S12927, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.289 Version/12.00*/
const Device UA2155273 = {S12928, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.289 Version/12.00*/
const Device UA2156418 = {S12929, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03672); ru) Presto/2.10.289 Version/12.00*/
const Device UA2168205 = {S12930, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.289 Version/12.01*/
const Device UA2168544 = {S12931, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5217); ru) Presto/2.10.289 Version/12.01*/
const Device UA2168700 = {S12932, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5278); ru) Presto/2.10.289 Version/12.01*/
const Device UA2169075 = {S12933, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.10.289 Version/12.01*/
const Device UA2169479 = {S12934, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.289 Version/12.01*/
const Device UA2169484 = {S12935, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5336); ru) Presto/2.10.289 Version/12.01*/
const Device UA2170769 = {S12936, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03672); ru) Presto/2.10.289 Version/12.01*/
const Device UA2171183 = {S12937, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.289 Version/12.01*/
const Device UA2171251 = {S12938, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5299); ru) Presto/2.10.289 Version/12.01*/
const Device UA2189904 = {S12939, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5966); ru) Presto/2.10.289 Version/12.00*/
const Device UA2206001 = {S12940, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.10.289 Version/12.01*/
const Device UA2206148 = {S12941, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.10.289 Version/12.00*/
const Device UA2219133 = {S12942, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5325); ru) Presto/2.10.289 Version/12.00*/
const Device UA2225537 = {S12943, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5300); ru) Presto/2.10.289 Version/12.01*/
const Device UA2226102 = {S12944, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.289 Version/12.01*/
const Device UA2226618 = {S12945, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5325); ru) Presto/2.10.289 Version/12.01*/
const Device UA2262390 = {S12946, 137, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; en) Presto/2.10.289 Version/12.01 AppEngine-Google; (+http://code.google.com/appengine; appid: schmittify)*/
const Device UA2262486 = {S12947, 74, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.289 Version/12.01,gzip(gfe)*/
const Device UA2328700 = {S12948, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02552); ru) Presto/2.10.289 Version/12.00*/
const Device UA2332704 = {S12949, 102, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; compatible; MSIE 7.0; Windows NT 6.0; bg) Presto/2.10.289 Version/12.00*/
const Device UA2332809 = {S12950, 124, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.289 Version/12.00,gzip(gfe) (via translate.google.com)*/
const Device UA2339697 = {S12951, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5230); ru) Presto/2.10.289 Version/12.01*/
const Device UA2340343 = {S12952, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.10.289 Version/12.01*/
const Device UA2355557 = {S12953, 74, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.289 Version/12.00,gzip(gfe)*/
const Device UA2362868 = {S12954, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.289 Version/12.01*/
const Device UA2362984 = {S12955, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.289 Version/12.00*/
const Device UA2385919 = {S12956, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.289 Version/12.01*/
const Device UA2386112 = {S12957, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5997); ru) Presto/2.10.289 Version/12.01*/
const Device UA2404223 = {S12958, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.289 Version/12.00*/
const Device UA2405243 = {S12959, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.289 Version/12.00*/
const Device UA2423746 = {S12960, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.289 Version/12.01*/
const Device UA2445998 = {S12961, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.289 Version/12.01*/
const Device UA2446043 = {S12962, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4661); ru) Presto/2.10.289 Version/12.01*/
const Device UA2455589 = {S12963, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5745); ru) Presto/2.10.289 Version/12.00*/
const Device UA2456101 = {S12964, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5997); ru) Presto/2.10.289 Version/12.00*/
const Device UA2460517 = {S12965, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.289 Version/12.01*/
const Device UA2468013 = {S12966, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.289 Version/12.00*/
const Device UA2480804 = {S12967, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5784); ru) Presto/2.10.289 Version/12.00*/
const Device UA2482495 = {S12968, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5880); ru) Presto/2.10.289 Version/12.00*/
const Device UA2482760 = {S12969, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5880); ru) Presto/2.10.289 Version/12.01*/
const Device UA2503190 = {S12970, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5784); ru) Presto/2.10.289 Version/12.01*/
const Device UA2503473 = {S12971, 60, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/12.0(Windows NT 5.1;U;en)Presto/22.9.168 Version/12.00*/
const Device UA2504387 = {S12972, 60, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/12.0(Windows NT 5.2;U;en)Presto/22.9.168 Version/12.00*/
const Device UA2529150 = {S12973, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6011); ru) Presto/2.10.289 Version/12.00*/
const Device UA2553530 = {S12974, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5993); ru) Presto/2.10.289 Version/12.00*/
const Device UA2583165 = {S12975, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.289 Version/12.01*/
const Device UA2603531 = {S12976, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.289 Version/12.00*/
const Device UA2644362 = {S12977, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02746); ru) Presto/2.10.289 Version/12.01*/
const Device UA2657799 = {S12978, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5981); ru) Presto/2.10.289 Version/12.00*/
const Device UA2684634 = {S12979, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03780); ru) Presto/2.10.289 Version/12.00*/
const Device UA2687088 = {S12980, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02552); ru) Presto/2.10.289 Version/12.01*/
const Device UA2695081 = {S12981, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5993); ru) Presto/2.10.289 Version/12.01*/
const Device UA2697525 = {S12982, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5977); ru) Presto/2.10.289 Version/12.01*/
const Device UA2705909 = {S12983, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.289 Version/12.01*/
const Device UA2710517 = {S12984, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6083); ru) Presto/2.10.289 Version/12.00*/
const Device UA2727097 = {S12985, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5745); ru) Presto/2.10.289 Version/12.01*/
const Device UA2728992 = {S12986, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5981); ru) Presto/2.10.289 Version/12.01*/
const Device UA2765161 = {S12987, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.289 Version/12.00*/
const Device UA2767825 = {S12988, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6089); ru) Presto/2.10.289 Version/12.00*/
const Device UA2767993 = {S12989, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6089); ru) Presto/2.10.289 Version/12.01*/
const Device UA2780386 = {S12990, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6081); ru) Presto/2.10.289 Version/12.00*/
const Device UA2810819 = {S12991, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02652); ru) Presto/2.10.289 Version/12.00*/
const Device UA2818226 = {S12992, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.10.289 Version/12.01*/
const Device UA2824751 = {S12993, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02652); ru) Presto/2.10.289 Version/12.01*/
const Device UA2846265 = {S12994, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6011); ru) Presto/2.10.289 Version/12.01*/
const Device UA2846499 = {S12995, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5977); ru) Presto/2.10.289 Version/12.00*/
const Device UA2852621 = {S12996, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6037); ru) Presto/2.10.289 Version/12.01*/
const Device UA2854680 = {S12997, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 6003); ru) Presto/2.10.289 Version/12.00*/
const Device UA2859628 = {S12998, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6081); ru) Presto/2.10.289 Version/12.01*/
const Device UA2867420 = {S12999, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.289 Version/12.00*/
const Device UA2868694 = {S13000, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.289 Version/12.01*/
const Device UA2933243 = {S13001, 87, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02560); ru) Presto/2.10.289 Version/12.00*/
const Device UA2935324 = {S13002, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 6007); ru) Presto/2.10.289 Version/12.00*/
const Device UA2988842 = {S13003, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 6007); ru) Presto/2.10.289 Version/12.01*/
const Device UA2990457 = {S13004, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 6008); ru) Presto/2.10.289 Version/12.01*/
const Device UA3045209 = {S13005, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6054); ru) Presto/2.10.289 Version/12.00*/
const Device UA3086442 = {S13006, 86, { &P15364,&P5690,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5989); ru) Presto/2.10.289 Version/12.01*/
const Device UA59574 = {S13007, 142, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA68742 = {S13008, 142, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70138 = {S13009, 143, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; GT-S5570L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA77821 = {S13010, 143, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; GT-S5570L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA355053 = {S13011, 142, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA356211 = {S13012, 160, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-S5570-ORANGE/S5570BVKT1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA371231 = {S13013, 131, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570I Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA395943 = {S13014, 66, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5570 Build/GINGERBREAD)*/
const Device UA397397 = {S13015, 143, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; GT-S5570B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA421628 = {S13016, 136, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5570 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA575429 = {S13017, 143, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; he-il; GT-S5570B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623057 = {S13018, 160, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5570-ORANGE/S5570BVKQ4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625040 = {S13019, 143, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626976 = {S13020, 161, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; SAMSUNG GT-S5570/S5570BUKS2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA631199 = {S13021, 66, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-S5570 Build/GINGERBREAD)*/
const Device UA635651 = {S13022, 143, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-S5570L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA638553 = {S13023, 66, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5570 Build/GINGERBREAD)*/
const Device UA641112 = {S13024, 142, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-nz; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA655277 = {S13025, 67, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5570I Build/GINGERBREAD)*/
const Device UA659148 = {S13026, 143, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-S5570B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA700838 = {S13027, 130, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA701306 = {S13028, 130, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5570 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA707370 = {S13029, 143, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA730032 = {S13030, 161, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; SAMSUNG GT-S5570/S5570BUKS1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA771021 = {S13031, 120, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-gb; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA1895357 = {S13032, 136, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; GT-S5570 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1987648 = {S13033, 67, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5570L Build/GINGERBREAD)*/
const Device UA2019264 = {S13034, 66, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-S5570 Build/GINGERBREAD)*/
const Device UA2020094 = {S13035, 67, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-S5570L Build/GINGERBREAD)*/
const Device UA2020707 = {S13036, 138, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7.1; vi-vn; GT-S5570 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020861 = {S13037, 60, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; GT-S5570 Build/GRI40)*/
const Device UA2021979 = {S13038, 136, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT-S5570 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022451 = {S13039, 67, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5570B Build/GINGERBREAD)*/
const Device UA2029141 = {S13040, 67, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5570B Build/GINGERBREAD)*/
const Device UA2030887 = {S13041, 159, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.8; en-us; GT-S5570 Build/GRI40; CyanMobile-eXperience) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033867 = {S13042, 135, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5570 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2149112 = {S13043, 120, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; es-es; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2215938 = {S13044, 140, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2234093 = {S13045, 160, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570-ORANGE/S5570BVKT4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257103 = {S13046, 141, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2331004 = {S13047, 140, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2362360 = {S13048, 161, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5570/S5570BUKT2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2362559 = {S13049, 160, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570-ORANGE/S5570BVKT5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2419103 = {S13050, 140, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2558296 = {S13051, 161, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-S5570/S5570MJKT3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2659111 = {S13052, 161, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5570/S5570BUKT4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2665679 = {S13053, 160, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570-ORANGE/S5570BVKT6 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2727962 = {S13054, 163, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5570 Build/GWK74; New Experience v6.0 FINAL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2996885 = {S13055, 159, { &P15649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT-S5570 Build/GINGERBREAD; JellyBread v2.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA924293 = {S13056, 152, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Full AOSP on Toro-VZW Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029098 = {S13057, 147, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Dark Jelly Belly Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029585 = {S13058, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; msm8960 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2199569 = {S13059, 148, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Dark JellyBellyG1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2201660 = {S13060, 163, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Toasted Marshmallow Build/Toasted Marshmallow) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2340055 = {S13061, 147, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.5; id-id; TREQ_A10_Pocket2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2342347 = {S13062, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; SKY Limo Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2352428 = {S13063, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC_ENRC2B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2417701 = {S13064, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N101 DUAL CORE2 V11 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2417725 = {S13065, 126, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; 4S Build/F88) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2419375 = {S13066, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; 9300 Build/F9AD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2419846 = {S13067, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; N101 DUAL CORE 2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2420744 = {S13068, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; X720D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2421777 = {S13069, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; S1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422058 = {S13070, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; RAPID5 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422263 = {S13071, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; POP9.7G2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422267 = {S13072, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HC801 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422372 = {S13073, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; msm8960 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422414 = {S13074, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; Pro 10 dual core Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422428 = {S13075, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; DPS Dream 9 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422938 = {S13076, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Q50 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2423959 = {S13077, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nb-no; 7024EP Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2425349 = {S13078, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nb-no; 9748EP Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2426413 = {S13079, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SP0718 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2426460 = {S13080, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; ALEXIS RX5 DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2427424 = {S13081, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; X2-IPS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2427764 = {S13082, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Crystal Build/JRO03C.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2428017 = {S13083, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Rikomagic MK802III Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2430621 = {S13084, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; RAPID5H Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2437281 = {S13085, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; SPICE IV Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2455115 = {S13086, 126, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; U1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2455188 = {S13087, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; es-es; 9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455467 = {S13088, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; POP 7.1HD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2455561 = {S13089, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ENERGY i10 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2457302 = {S13090, 153, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Full Android on PrimoC Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2457740 = {S13091, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-P600 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2457938 = {S13092, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Tab9.7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2458135 = {S13093, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nb-no; 8028EP Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2460272 = {S13094, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; M1001 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2461012 = {S13095, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; STUDIO Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2461166 = {S13096, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; AllviewCity Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2461303 = {S13097, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; RAPID5-H Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2466106 = {S13098, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; I9277 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2466730 = {S13099, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; imito am802 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2467277 = {S13100, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Q50R Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2467945 = {S13101, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; CAPTIVA PAD 10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2468427 = {S13102, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; i907c Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2468769 = {S13103, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Unique7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2480798 = {S13104, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ROCK X Build/ROCK X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2481692 = {S13105, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Fast Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2481750 = {S13106, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-uk; cyclone Titan Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2482875 = {S13107, 126, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; S2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2483091 = {S13108, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; M758A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483332 = {S13109, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; F8 SUP2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2483854 = {S13110, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; platinum Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2485117 = {S13111, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; Slick973V2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2486139 = {S13112, 153, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; E-Boda Supreme Dual Core X190 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2486646 = {S13113, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.4; en-us; Note3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2487013 = {S13114, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IMAGINE 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2503519 = {S13115, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; EPAD Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2504148 = {S13116, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Hold X Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2504538 = {S13117, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; vangogh Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2505298 = {S13118, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Tablet Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2505371 = {S13119, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TP7-1500DC-metal Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2505402 = {S13120, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; TA10100 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2505891 = {S13121, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; E74 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2506398 = {S13122, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TABLOID Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2506986 = {S13123, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; NBLT10-A4 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2507054 = {S13124, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; AQUILA 080-1008 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2507273 = {S13125, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; N10-2CPU Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2507673 = {S13126, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; U101 Build/FR-MX68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2507968 = {S13127, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; AQUILA 097-1006 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2510711 = {S13128, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; PC728 Build/PC728) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2523854 = {S13129, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Papyre Pad 972 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2525672 = {S13130, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TM-MID940 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2527247 = {S13131, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; P10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2527470 = {S13132, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N70HD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2527933 = {S13133, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; I9420 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2527982 = {S13134, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; M973W Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2528314 = {S13135, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; Tablet7525B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528365 = {S13136, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TAGIMID 140 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2528809 = {S13137, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; A10-2CPU Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2528908 = {S13138, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PUBLIC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2528938 = {S13139, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; P0V_TV-HDMI-KB-01 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2529315 = {S13140, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; ConCorde tab PLAY Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2529553 = {S13141, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Unique10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2529891 = {S13142, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TORNADO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2530127 = {S13143, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; Plug 10.1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2530280 = {S13144, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PULID F13 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2551789 = {S13145, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; TX85 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2552907 = {S13146, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; CTAB2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2554154 = {S13147, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; rk30 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2554325 = {S13148, 51, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1 Build/JRO03C)*/
const Device UA2554557 = {S13149, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; ATP7713 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555188 = {S13150, 126, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; G1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555533 = {S13151, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Royqueen 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555562 = {S13152, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; Spirit S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555580 = {S13153, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; U1 Pro Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555589 = {S13154, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-at; TABLO Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555856 = {S13155, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; TA8300 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555857 = {S13156, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 9706 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2556082 = {S13157, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 9300mini Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2556154 = {S13158, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; T-1006 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2556238 = {S13159, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Primux_Omega Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556486 = {S13160, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Tablet-PC-4 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2556498 = {S13161, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D725 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2556727 = {S13162, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A10-2cpu Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2557237 = {S13163, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; KTab7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2557254 = {S13164, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; DROIDZ Portal Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2557598 = {S13165, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; M972g Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2557611 = {S13166, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; MyAudio Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2557925 = {S13167, 147, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Mobiltab Sleek 5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558034 = {S13168, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; KTab9.7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2558351 = {S13169, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; B63 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2559739 = {S13170, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; PAD 9718DR Build/PAD 9718DR) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2560826 = {S13171, 124, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 97R Build/97R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2562371 = {S13172, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; S8+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2565241 = {S13173, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; mini STUDIO Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2577765 = {S13174, 165, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; androVM for VirtualBox ('Tablet' version) Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2578027 = {S13175, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; Ultimate 10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2578101 = {S13176, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; mfld_pr2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2578329 = {S13177, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EX'3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2578496 = {S13178, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Thrive Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2578670 = {S13179, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Cute  X Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2578717 = {S13180, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; RC9724C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2578724 = {S13181, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; U30GT mini Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2578895 = {S13182, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; D9702 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2579003 = {S13183, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Primux_Alpha Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2579825 = {S13184, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EX'Note ? Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580374 = {S13185, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; Classic103 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2580493 = {S13186, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; DIT7070 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2580692 = {S13187, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; B94M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2581842 = {S13188, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; RAPAX 070LE Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2582535 = {S13189, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SC-72 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2582761 = {S13190, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Android_53 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2582766 = {S13191, 145, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; prestige7L_4.1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2593605 = {S13192, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; Gpad Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2595275 = {S13193, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; TR736 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2595325 = {S13194, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; LETAB903 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2595581 = {S13195, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NINE T7800 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2595881 = {S13196, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; M100 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2596334 = {S13197, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; BRVTX11D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2597165 = {S13198, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Pandora 9.10 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597335 = {S13199, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; T10 3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2597526 = {S13200, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; GPLUS N930+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597541 = {S13201, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; iconBIT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2597640 = {S13202, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; VB100a Pro Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2598502 = {S13203, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; A07I-D15A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2598992 = {S13204, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S701-R2A-1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2599642 = {S13205, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NC-eLite Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2599754 = {S13206, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MID08S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2600167 = {S13207, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; Obreey SURFpad Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2600400 = {S13208, 146, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Mobiltab Sleek5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2600419 = {S13209, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; I699 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2600493 = {S13210, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Tab8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2603524 = {S13211, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2603632 = {S13212, 126, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; MID-801 Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2604384 = {S13213, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nb-no; 1038EP Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2604556 = {S13214, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2604945 = {S13215, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-uk; 777TPC Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2651295 = {S13216, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; Shogun 10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2653421 = {S13217, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; Ultimix99 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2653904 = {S13218, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Visture V4 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2654597 = {S13219, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; OV-EDUTAB Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2654603 = {S13220, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; CTP980 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2655397 = {S13221, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CW V8 Dual Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2655934 = {S13222, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Decontaminate V3.0 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2655937 = {S13223, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; TB7020+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2656344 = {S13224, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MID1002 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2656713 = {S13225, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; PC677 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2657627 = {S13226, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; Quicki-811 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2657754 = {S13227, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Teraware K977 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2657825 = {S13228, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; SAMURAI Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2658121 = {S13229, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; NEXO Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2658144 = {S13230, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N10-2cpu Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2658380 = {S13231, 145, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Cat Tablet CSG2 7.1J0 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2659109 = {S13232, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Tablet X2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2659239 = {S13233, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 9930 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659250 = {S13234, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; B9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659539 = {S13235, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; DIN09.97 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2659584 = {S13236, 126, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2659637 = {S13237, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HD470 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2660102 = {S13238, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; PC1002 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2660159 = {S13239, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; M74 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2660727 = {S13240, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Androtab 7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2661491 = {S13241, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; L9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2661625 = {S13242, 146, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Android Note II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2661792 = {S13243, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; Ak8462 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2662311 = {S13244, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; 10DCIX Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2662450 = {S13245, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; 9458DCB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2662839 = {S13246, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Phablet PII Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2663137 = {S13247, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; FCB tablet Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2663319 = {S13248, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; MMS-874.dualcore Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2663756 = {S13249, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Android_53DC Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665247 = {S13250, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Phablet PIII Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665368 = {S13251, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; M758C1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665555 = {S13252, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Neon9 Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666070 = {S13253, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; LUXURY Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666140 = {S13254, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ie; JB7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2666215 = {S13255, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; SELECT 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666245 = {S13256, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; CAPTIVA PAD 10.1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2666446 = {S13257, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; VS TOUCHTAB 9.7RT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2666734 = {S13258, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TAB700 NV Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2668325 = {S13259, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TAB7DUAL 8GB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2668473 = {S13260, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N9977 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2668479 = {S13261, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fa-ir; G4 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2668563 = {S13262, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; A9724 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2668752 = {S13263, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S71 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2670945 = {S13264, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2671069 = {S13265, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; EVO LTE Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2693043 = {S13266, 147, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Full Android on Grouper Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2693165 = {S13267, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HTC DLX_WLJ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695749 = {S13268, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; ConCorde tab NOVUM Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2696064 = {S13269, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; X817 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2696498 = {S13270, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-uk; SWIPE Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2696882 = {S13271, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; EVOLVE_FX420 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2697139 = {S13272, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; Q7pro-H10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2697292 = {S13273, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; bg-bg; M976 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2697468 = {S13274, 153, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; California Access TAB MM 1001 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2697682 = {S13275, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; sl-si; XP791 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2698476 = {S13276, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; EVOLVE_FX520 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2698785 = {S13277, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; FT-2008 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2698824 = {S13278, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MID9701 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2698972 = {S13279, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; TX12D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2699448 = {S13280, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; F1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2699517 = {S13281, 160, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_Y300C/Y300CV100R001C92B166; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA2699693 = {S13282, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-co; aigopad Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2699710 = {S13283, 125, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; - Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2699719 = {S13284, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; AT400 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2699733 = {S13285, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; PAN 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2700255 = {S13286, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A85-2cpu Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2700522 = {S13287, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; skyfire S+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2700599 = {S13288, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NID-7006A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2701150 = {S13289, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; RMD-1026 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2701265 = {S13290, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; MaxiIQ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2701404 = {S13291, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; PENTAGRAM TAB 8.3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2701684 = {S13292, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; TAB7B_3G Build/WEXLER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2702056 = {S13293, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Newsmy P72 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2702445 = {S13294, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; WB8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2702715 = {S13295, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; A8826 Build/ICS.A8826.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2702858 = {S13296, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2703272 = {S13297, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; M15GC702P Build/PD715Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2703278 = {S13298, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ENERGY s9 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2703640 = {S13299, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; GTD866 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2703726 = {S13300, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; Maxx II Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2703749 = {S13301, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; Diamond_Lite Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2703995 = {S13302, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; mid Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2704029 = {S13303, 152, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; GiONEE_Infinity_GN708 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2704572 = {S13304, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SC-75JB Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2705141 = {S13305, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 975TPCII Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2705169 = {S13306, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ACGM816KC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2705434 = {S13307, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; M7R4C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2705988 = {S13308, 146, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; Android Note II Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2706401 = {S13309, 126, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2706473 = {S13310, 126, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TBDB763 Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2706642 = {S13311, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; NTT 611 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2706843 = {S13312, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ph; T-746 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2706892 = {S13313, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TPC-EA901 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2707072 = {S13314, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; minipad Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2707245 = {S13315, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; DPS Dual 7 Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2707750 = {S13316, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; maTab LION Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2707782 = {S13317, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M1400 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2708057 = {S13318, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Xtab1081HD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2709498 = {S13319, 145, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; TELETALK_X110Q Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2712937 = {S13320, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MOMO20-AU Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2712953 = {S13321, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fa-ir; PC7558 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2713188 = {S13322, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; B943 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2713712 = {S13323, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M736 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2715279 = {S13324, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; da-dk; Ennovia TAB E309(V2) Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2717574 = {S13325, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MIDCD97 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2718466 = {S13326, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D7018 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2726805 = {S13327, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC DLX_WL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2727299 = {S13328, 146, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; YL-Coolpad 9960 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2727493 = {S13329, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; iM970 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2728007 = {S13330, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; msm8625 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2728056 = {S13331, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; da-dk; M972 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2729051 = {S13332, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; es209ra Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2729128 = {S13333, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; crane-evb Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2729160 = {S13334, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; AMOI M8228 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2729244 = {S13335, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Speed_MB-R2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2729563 = {S13336, 153, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PIRANHA BUSINESS III TAB 10.1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2729600 = {S13337, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ADM712KC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2729687 = {S13338, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; MP973G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2729733 = {S13339, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.0; zh-tw; HWA680 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2729834 = {S13340, 147, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.4; en-us; sanstar6577_ics2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2730592 = {S13341, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; Tablet Mint Max10 DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2730874 = {S13342, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; Kruger Matz 1010 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2731095 = {S13343, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; IPRO i5S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2731261 = {S13344, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Tab70s Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2731291 = {S13345, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; Max_X2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2732125 = {S13346, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Mercury X Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2732231 = {S13347, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M10+ Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2732316 = {S13348, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; SPC P2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2732401 = {S13349, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ENERGY s7 Dual Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2732438 = {S13350, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; RMD-877G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2732588 = {S13351, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Boost Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2732636 = {S13352, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; 1028DCS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2732648 = {S13353, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; JPAD730 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2732720 = {S13354, 181, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; androVM for VirtualBox ('Tablet' version with phone caps) Build/JRO03S) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2732748 = {S13355, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; TB7900 Build/TB7900) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2732766 = {S13356, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; M78A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2732802 = {S13357, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CT1006 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2732815 = {S13358, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; MITO t300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2732910 = {S13359, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; MT-801 Build/MT-801) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2733601 = {S13360, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; klu1506 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2733819 = {S13361, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; A9600 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2733890 = {S13362, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Primux_SonoraDual Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2733947 = {S13363, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 9706-T Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2733992 = {S13364, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Troy Duos 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2734397 = {S13365, 113, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; YL-Coolpad_5930/4.1.016.X.5930;540*960;CTC/2.0) CoolpadWebkit/534.30*/
const Device UA2734510 = {S13366, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MOMO20 Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2737617 = {S13367, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; TAB709D-3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2737698 = {S13368, 218, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MT6517-MT6517-H369X/1.0 Android/4.1.1 Release/04.02.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2737973 = {S13369, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; NPC001 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2738286 = {S13370, 126, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TBDB863 Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2738299 = {S13371, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; PC906T Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2738351 = {S13372, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; FUSION Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2746928 = {S13373, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; iTab971 Dual Core Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2747132 = {S13374, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; Shogun 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2747151 = {S13375, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; R10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2747836 = {S13376, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; ASK 780SP HD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2748100 = {S13377, 126, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; m6 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2748114 = {S13378, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NEO-X5 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2748255 = {S13379, 145, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; Padphone 3G-61 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2748276 = {S13380, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; V811 Core4 Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2748302 = {S13381, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MT8377 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2748628 = {S13382, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; CTAB101 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2749218 = {S13383, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; id-id; PICOpad_GFI Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2749225 = {S13384, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; RMP-600 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2749429 = {S13385, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; NetTAB SPACE II Plus Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2749580 = {S13386, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; N9zero Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2749554 = {S13387, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; D1002 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2750009 = {S13388, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MOBILTAB sleek 10.1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2750029 = {S13389, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; g08refem618 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2750043 = {S13390, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Pascal 2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2750304 = {S13391, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EZPAD 830DC Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2750392 = {S13392, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; PC98IPS Dual Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2750904 = {S13393, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; A8849 Build/ICS.A8849.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2751110 = {S13394, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; T-1005 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2751224 = {S13395, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; PRO9-A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2751226 = {S13396, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; TOUCHTAB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2751325 = {S13397, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TBD1083B Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2751501 = {S13398, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; Tab Play 002 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2751564 = {S13399, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; H2-9506CM Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2751908 = {S13400, 210, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MT-GT-T7100/1.0 Android/4.1.1 Release/04.02.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2752028 = {S13401, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MT6517 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2752367 = {S13402, 150, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Woxter Tablet PC98IPS DUAL Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2752498 = {S13403, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; CTP978 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2752579 = {S13404, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMID706 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2753188 = {S13405, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; TM903 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2762570 = {S13406, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; i708 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2762574 = {S13407, 55, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; S1 Build/JRO03H)*/
const Device UA2762921 = {S13408, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AMOI N820 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2763154 = {S13409, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; H-SP 9702 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2763212 = {S13410, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; WICKED-H Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2763256 = {S13411, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; SINA-1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2763609 = {S13412, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; avio_sen_S5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2763623 = {S13413, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; its2520 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2763789 = {S13414, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB-R3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2764246 = {S13415, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Ex_Note Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2764352 = {S13416, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; PENTAGRAM TAB 8.5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2764415 = {S13417, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.4; es-es; Note3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2764704 = {S13418, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; 10BPE0 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2764817 = {S13419, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; MID72C Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2764933 = {S13420, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; TY-Q8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2765045 = {S13421, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; HLV-T1001W Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2765051 = {S13422, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; OS988 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2765167 = {S13423, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N83-2cpu Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2765654 = {S13424, 147, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; KN_Mobile_QT_A08 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2766375 = {S13425, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; PT10.1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2766488 = {S13426, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; AEON Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2766503 = {S13427, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; ZP958 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2766779 = {S13428, 160, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_Y300C/Y300CV100R001C92B168; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA2766782 = {S13429, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; IMO S99 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2767300 = {S13430, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; VP70 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2767306 = {S13431, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; MR18 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2767626 = {S13432, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; PAD701 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2767813 = {S13433, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; GS3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768361 = {S13434, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; Pro dual 10 core Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2768493 = {S13435, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; Tab1030 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2768514 = {S13436, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fa-ir; TAX Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768727 = {S13437, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; CTAB743 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2768754 = {S13438, 149, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; i-mobile i-style 8 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768995 = {S13439, 146, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; Luna TAB10-150 Build/Luna TAB10-150) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2770261 = {S13440, 145, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Mitraveler 7D8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2772142 = {S13441, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMID720 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2777604 = {S13442, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; N70DC-S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2777732 = {S13443, 59, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SP0718 Build/JRO03H)*/
const Device UA2778590 = {S13444, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; REVO MAX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778823 = {S13445, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; B92M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2779386 = {S13446, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IT-M804RC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2779615 = {S13447, 162, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; L36h Build/10.1.A.1.350) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 BaiduBoxApp/3.6_130g*/
const Device UA2779786 = {S13448, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; L003 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2779906 = {S13449, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; Aveo NOTE II Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2780039 = {S13450, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; jpad802 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2780062 = {S13451, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; g08refem617 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2780197 = {S13452, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; MW0710 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2780256 = {S13453, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Tablet Planet II-v3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2780581 = {S13454, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; AML-MX REF Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2780817 = {S13455, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; S33D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2780840 = {S13456, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; M73 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2780906 = {S13457, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; TB1010 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2780991 = {S13458, 145, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; Connspeed IP71 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781287 = {S13459, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; RX3-DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2781662 = {S13460, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Wb7-L Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2781785 = {S13461, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M772 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2781878 = {S13462, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; PANDORA 9.20 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2782259 = {S13463, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 9D181 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2784113 = {S13464, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; EVO LTE Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2790415 = {S13465, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; ALEXIS-RX5DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2790854 = {S13466, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; QRD-8625DSDS Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2790987 = {S13467, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; x5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2791008 = {S13468, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; MIDM-N81S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2791087 = {S13469, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; N7Zero Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2791333 = {S13470, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; iTab1011 Dual Core Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2791447 = {S13471, 126, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; G1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2792214 = {S13472, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ACGM727KC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2792255 = {S13473, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; LA-Q1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2793749 = {S13474, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-uk; MP1041 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2793805 = {S13475, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IT-TITAN3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2793915 = {S13476, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 5930 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2794145 = {S13477, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; BTPC-1011-DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2794157 = {S13478, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; Nexo 10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2794283 = {S13479, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAB R76.2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2794490 = {S13480, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PICOpad GEW Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2794510 = {S13481, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; F918 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2794543 = {S13482, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; TAB9 3G V8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2794658 = {S13483, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; LA-M1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2794865 = {S13484, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; F600 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2794873 = {S13485, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MP7007 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2795125 = {S13486, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; QBA757 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2795284 = {S13487, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; HD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2795262 = {S13488, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; H9500+ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2795320 = {S13489, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Sweet M720 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2795404 = {S13490, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ACGM727KC_N Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2799199 = {S13491, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Energyi8Dual Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2799355 = {S13492, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; SP-120 DC Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2835187 = {S13493, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; aussiePAD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2836843 = {S13494, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MiSS Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2837417 = {S13495, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; 97S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2840086 = {S13496, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; mfld_pr3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2840426 = {S13497, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; G701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2843726 = {S13498, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; F220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2843854 = {S13499, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; OBEE_OS969 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2844148 = {S13500, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 97ST Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2844197 = {S13501, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; id-id; TREQ Turbo Build/FR-MX68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2844374 = {S13502, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 4u + Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2844764 = {S13503, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; DFP8007 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2844983 = {S13504, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CHUWI V99 Build/CHUWI V99) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2845003 = {S13505, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; zh-tw; GT-N7077 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2847202 = {S13506, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID-X4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2847593 = {S13507, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TLS-V4 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2847595 = {S13508, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; i90 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2847680 = {S13509, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; M704S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2847852 = {S13510, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AM818 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2848461 = {S13511, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 4U+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2848903 = {S13512, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-ca; U30GT MINI Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2848986 = {S13513, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; SSW-T7A41HD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2849235 = {S13514, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; N710 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2849456 = {S13515, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; PAD1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2849887 = {S13516, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Wanden.co.uk_N90 CZ Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2849958 = {S13517, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SX9722 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2850020 = {S13518, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MD-706S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2850104 = {S13519, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NEO-G4 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2850650 = {S13520, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; C3_77_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2850872 = {S13521, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GF-2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2851132 = {S13522, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; Core 10.1 3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2851178 = {S13523, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1_JELLYCUBE_V1; fr-ca; GT-N700 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2852038 = {S13524, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MID1003 Build/FR-MX68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2852088 = {S13525, 64, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ADM8000KP_A Build/JRO03H)*/
const Device UA2852225 = {S13526, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; Ultimix94 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2852827 = {S13527, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; VB80a Pro Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2855693 = {S13528, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TABLET 8.5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2855743 = {S13529, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; P102 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2855944 = {S13530, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; v1801mt77 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2856359 = {S13531, 151, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Woxter Tablet PC 85IPS Dual Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2857344 = {S13532, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; RC9730C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2858151 = {S13533, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID-709S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2858324 = {S13534, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; DINO7.17 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2858799 = {S13535, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Intellibook Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2858930 = {S13536, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XPAD-2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2858943 = {S13537, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; KTab10.1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2859333 = {S13538, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; QPAD9700 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2859403 = {S13539, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; CE15012-16 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2859453 = {S13540, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; DP-101 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2859522 = {S13541, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fa-ir; G4+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2859609 = {S13542, 152, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HUAWEI D2-0082 Build/HuaweiD2-0082) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2859635 = {S13543, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; MW0731 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2861800 = {S13544, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Greenstone Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2861961 = {S13545, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; E8050HG Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2862220 = {S13546, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; M84g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2862632 = {S13547, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; PER5274B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2862824 = {S13548, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MT12 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2863596 = {S13549, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Neon7 Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2864009 = {S13550, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; PREMIER Build/PREMIER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2864248 = {S13551, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AXPAD 7E01 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2864434 = {S13552, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MID1028 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2864643 = {S13553, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MID1001 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2864700 = {S13554, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ViewPad 100N Pro Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2864961 = {S13555, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Bryte Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2865334 = {S13556, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; Surfer 8_02 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2865372 = {S13557, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-de; M-12 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2865519 = {S13558, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; SIROCO 4 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2866002 = {S13559, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; KIRK3-A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2866146 = {S13560, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; ConCorde tab Speed Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2866301 = {S13561, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; FreeTAB1003 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2866364 = {S13562, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; da-dk; EN-TAB327 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2866474 = {S13563, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; T-763 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2866545 = {S13564, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; K47 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2866808 = {S13565, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; TM703 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867011 = {S13566, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MP717 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2867012 = {S13567, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N900 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867314 = {S13568, 158, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_D2-2010/D2-2010V100R001C92B132) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867474 = {S13569, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S750L Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2867929 = {S13570, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nitro 400D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2868019 = {S13571, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ID1018WTA Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2868147 = {S13572, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-; S2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2868273 = {S13573, 218, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MT6517-MT6517-H369X/1.0 Android/4.1.1 Release/05.09.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2869007 = {S13574, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; TAB700 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2869290 = {S13575, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; DITX Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2869637 = {S13576, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-I710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2869659 = {S13577, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; he-il; u_force_U7I Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2870784 = {S13578, 151, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; NGM_Forward_Infinity Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878334 = {S13579, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Q88 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878439 = {S13580, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I759 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878530 = {S13581, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MID703 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2878634 = {S13582, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MC7411B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878655 = {S13583, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Touchlet X10.dual Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2878653 = {S13584, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; BRVQ750D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2878906 = {S13585, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; ETL-T970 Build/ETL-T970) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2878918 = {S13586, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID82C Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878995 = {S13587, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; HFM7002KD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2879071 = {S13588, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MD700 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879279 = {S13589, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TechniPad 10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2879317 = {S13590, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; 10BPEOH Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2879502 = {S13591, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Grand DUOS  Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879524 = {S13592, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; Space Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2889471 = {S13593, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - MT6589_TD/V1 Linux/3.4.0 Android/4.1.2  Release/12.24.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2 System/Android 4.1.2;*/
const Device UA2889560 = {S13594, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Fast1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2890199 = {S13595, 160, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_Y300C/Y300CV100R001C92B165; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA2890297 = {S13596, 159, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Tablet-PC-4.1-Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ADM8000KP_A Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30,gzip(gfe)*/
const Device UA2890372 = {S13597, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Ex_Note6.0 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2890395 = {S13598, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-530M Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2890839 = {S13599, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; RMD_600 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2890887 = {S13600, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; th-th; jFone JS501 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2890919 = {S13601, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MA715 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2890971 = {S13602, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; ZTE N5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2891099 = {S13603, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; K-Touch U86 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2891107 = {S13604, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; Z520m-parrot Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2891167 = {S13605, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; ZP808 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2891341 = {S13606, 147, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; 820 Build/820-V1.0-2013.01.30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2891396 = {S13607, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; Gpad Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2891499 = {S13608, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-R530M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2891683 = {S13609, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; H3-8106L Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2891752 = {S13610, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; H100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2891805 = {S13611, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; TAB1052 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2891843 = {S13612, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A1020 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2891974 = {S13613, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; P9070 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892033 = {S13614, 148, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; Trevi_PHABLET_5_S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892040 = {S13615, 147, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GIONEE_INFINITY+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892093 = {S13616, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; FP-020-VE Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2892161 = {S13617, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; Q049 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892265 = {S13618, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HCAT-01ST Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2892331 = {S13619, 145, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SwipeHaloSpeed Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892446 = {S13620, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ro-ro;Nivo Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2892551 = {S13621, 151, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Tokio_Smartphone_1.0 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892656 = {S13622, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAB10-212 Build/Xenta TAB10-212) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2892737 = {S13623, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; GT10K-DC Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2892796 = {S13624, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; L36h Build/10.1.1.A.1.253) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892800 = {S13625, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; GT-N7I00 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892848 = {S13626, 146, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Walton Primo H1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892942 = {S13627, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; WT71 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2893052 = {S13628, 207, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MT-X5292/1.0 Android/4.1.1 Release/04.03.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2893110 = {S13629, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; T-SC2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2893386 = {S13630, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; NABIJR-NV5B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2893467 = {S13631, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; MIDCD10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2893882 = {S13632, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 9920 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2897611 = {S13633, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Q8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934373 = {S13634, 152, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; E-Boda Supreme X80 Dual Core Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2934521 = {S13635, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; TB750 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2934622 = {S13636, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; sunfire Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934625 = {S13637, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IFA10.1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2934626 = {S13638, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; OV-TB-O9C Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2934613 = {S13639, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MID9702 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2934787 = {S13640, 58, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; TY-Q8 Build/JRO03C)*/
const Device UA2934875 = {S13641, 152, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI D2-2010 Build/HuaweiD2-2010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934991 = {S13642, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Ex'Note S  Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2935139 = {S13643, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TAB700 4GB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2935221 = {S13644, 148, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Cat Tablet PHOENIX 8.1J0 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2935332 = {S13645, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; 8506IPS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2935526 = {S13646, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SV7612-RK2926 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2935794 = {S13647, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Tablet Ner?n Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2935796 = {S13648, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; RC0722C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2936121 = {S13649, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; LearnPad Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2936127 = {S13650, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; M75GC702G Build/PD700) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2936177 = {S13651, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; TAB852 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2936283 = {S13652, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; ZP908 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936331 = {S13653, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; xTAB-700dc Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2936416 = {S13654, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Orin S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936482 = {S13655, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; xTAB-70dci Build/w28ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2936678 = {S13656, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; M1003 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2936729 = {S13657, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; klu8506 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2936744 = {S13658, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CVD601 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2936957 = {S13659, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Alcatel 7030L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2937152 = {S13660, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; DTP-07HDAH Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2937245 = {S13661, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Kepler 2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2937306 = {S13662, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; FPT F12 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937433 = {S13663, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-uk; MAGPAD_MP1041 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2937466 = {S13664, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; cj-31s-jrrj Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937645 = {S13665, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; Infinity POP Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2937653 = {S13666, 126, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; S9 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2937754 = {S13667, 125, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; baoxue Build/F10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2937792 = {S13668, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; SKYTEX-SP723 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2937820 = {S13669, 145, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; TELETALK_X200Q Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937831 = {S13670, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; A10-QM Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2937866 = {S13671, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; QUANTUM65220 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937948 = {S13672, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; i-Call 503 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2938142 = {S13673, 147, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; AQUILA 097-1016 BT + 3G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2938160 = {S13674, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; TEACTAB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2938163 = {S13675, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; T277 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2938235 = {S13676, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Surfer 7.32 3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2938526 = {S13677, 151, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; i-mobile i-STYLE 7.2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2938711 = {S13678, 215, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1 Release/04.26.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2939136 = {S13679, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LXK901 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2939720 = {S13680, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 1018 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2940488 = {S13681, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; F7100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2940604 = {S13682, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Zeepad 9XN Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2941532 = {S13683, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; G926 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2942543 = {S13684, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; fnac 8 v2 Build/1.0.0_X-19:36) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2942546 = {S13685, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; M-pro Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950983 = {S13686, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; CRT97 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951103 = {S13687, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB-R2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951401 = {S13688, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; Speedstar S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951493 = {S13689, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; CTP988 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951594 = {S13690, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC S720t Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2951822 = {S13691, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Soaiy Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951815 = {S13692, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; MonsterPad Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951893 = {S13693, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; i926 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951965 = {S13694, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; A690 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2951976 = {S13695, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; HK-i7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952101 = {S13696, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; i6577 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952162 = {S13697, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Z770M Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952324 = {S13698, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S7300B Build/S7300B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2952347 = {S13699, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; Ultimix95 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2952473 = {S13700, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; TCL S820 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2952590 = {S13701, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Aprix D97+ Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2952722 = {S13702, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; StarPAD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2952745 = {S13703, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; U30GT2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2952807 = {S13704, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; Victorys_V8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952889 = {S13705, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ICOU10 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2952941 = {S13706, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Bambook S1 Build/NS4.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2953115 = {S13707, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; ARIA_Mini_wifi Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2953121 = {S13708, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hr-hr; MeanIT MG940A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2953298 = {S13709, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fa-ir; LD-459 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2953412 = {S13710, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; Sigma-X10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2953446 = {S13711, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SpeedStar S Pro Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2953519 = {S13712, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; V-PAD10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2953622 = {S13713, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; LETAB1006 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2953689 = {S13714, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ar-eg; G-Tab G100 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2953884 = {S13715, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; T750 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2954049 = {S13716, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; P700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2954414 = {S13717, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Primus 94HD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2954806 = {S13718, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; CTP818 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2954904 = {S13719, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AM818RC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2955056 = {S13720, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ATP7717 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2955143 = {S13721, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; moii E896 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955205 = {S13722, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; DTR-07HDAH Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2955241 = {S13723, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; W700 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955575 = {S13724, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; TAB9701 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2955596 = {S13725, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; D125 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2955787 = {S13726, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; MW0712 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2955949 = {S13727, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; NM890 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955964 = {S13728, 148, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HUAWEI Y300C Build/HuaweiY300C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955985 = {S13729, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAC-80031 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2956074 = {S13730, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; R830 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2956245 = {S13731, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-nz; CORE DIGITAL CD-7066 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2956312 = {S13732, 152, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; FreeTAB 9704 IPS2 X4 Build/unknown) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956376 = {S13733, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; NB032 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956611 = {S13734, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Surfer 7.31 3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2956700 = {S13735, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CrystalQuadCore Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2956730 = {S13736, 145, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; 3G7100i-PANELi Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956766 = {S13737, 147, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; KN_Mobile_QT_A06 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956864 = {S13738, 218, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MT6517-MT6517-H369X/1.0 Android/4.1.1 Release/04.08.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2956989 = {S13739, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CW-V88-QUAD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2957070 = {S13740, 113, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; YL-Coolpad_5930/4.1.025.X.5930;540*960;CTC/2.0) CoolpadWebkit/534.30*/
const Device UA2957074 = {S13741, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PRBT761 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957143 = {S13742, 146, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; I-Life wtab 702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957206 = {S13743, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; G-TiDE E6 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957257 = {S13744, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; T1400 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2957313 = {S13745, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; PAD 9716DR Build/PAD 9716DR) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2957321 = {S13746, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; TM-9748 3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2957495 = {S13747, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; QUEEN TAB 100 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2957592 = {S13748, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Cloudpad_800d Build/FSV03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2957637 = {S13749, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; XZPAD270G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2957672 = {S13750, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ar-eg; Q75S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957692 = {S13751, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SDT-737 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2957939 = {S13752, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; EHT7HD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2958036 = {S13753, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; i-mobile IQ1-1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958037 = {S13754, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EGS004 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2958044 = {S13755, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; sl-si; XPADK90 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2958185 = {S13756, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fa-ir; TB1000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2958268 = {S13757, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; MIDC470 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958352 = {S13758, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2958356 = {S13759, 152, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI MT1-T00 Build/HuaweiMT1-T00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958395 = {S13760, 219, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.1 Release/04.26.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2958537 = {S13761, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SKYTEX-SP705 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2958594 = {S13762, 122, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; CTA Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2958610 = {S13763, 218, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MT6517-MT6517-H369X/1.0 Android/4.1.1 Release/05.20.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2958628 = {S13764, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TP8-1500DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2958975 = {S13765, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A600 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2959346 = {S13766, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; CRXT 1125 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2959375 = {S13767, 219, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.1 Release/05.31.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2959619 = {S13768, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D7018 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2959620 = {S13769, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; Tab810 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2959755 = {S13770, 147, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Tablet-PC-4.1-Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; INM8002KP Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2960708 = {S13771, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Cherry Fusion Wind Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2961487 = {S13772, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; Tab760 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2962443 = {S13773, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; da-dk; TAB-E309(V3) Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2963655 = {S13774, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; da-dk; TAB-E419(V2) Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2964103 = {S13775, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; MTC 970 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2966006 = {S13776, 157, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 11_v77_gq2000_a41_6628_v20 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2968974 = {S13777, 145, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; TA-FONE A39 HD Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2972409 = {S13778, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; E58 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2982053 = {S13779, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; PL801 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2982839 = {S13780, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; N80HH Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2982874 = {S13781, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; N70DC-T Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2983156 = {S13782, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; NewPad Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2983206 = {S13783, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 7010S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2983385 = {S13784, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; 9706IPS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2983556 = {S13785, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nb-no; Tablet 9748DCB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2983806 = {S13786, 55, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; S2 Build/JRO03H)*/
const Device UA2983814 = {S13787, 153, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Mini Plus 4G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2983905 = {S13788, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; T5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984219 = {S13789, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Cayon X3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984331 = {S13790, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; TAB 177 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2984720 = {S13791, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; StarPAD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2984770 = {S13792, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 9709 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2984758 = {S13793, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Intki E96?? Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984814 = {S13794, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; F097 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2984825 = {S13795, 126, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; 8730 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2984999 = {S13796, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAB364 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2985161 = {S13797, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HD_3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985344 = {S13798, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; NetTAB SPACE II Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2985445 = {S13799, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CP101 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2985493 = {S13800, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; L106 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985587 = {S13801, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; U23GT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2985645 = {S13802, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 5930 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2985820 = {S13803, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; OUKI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985863 = {S13804, 59, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; NEO-X5 Build/JRO03H)*/
const Device UA2985886 = {S13805, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; V350 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985977 = {S13806, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; u12 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2986070 = {S13807, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; TAB R105BK Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2986496 = {S13808, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; HKPSG Max Pro Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986500 = {S13809, 156, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Motorola RaZR3Ds Build/9.8.2O-72_VZW-8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986580 = {S13810, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A86-2cpu Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2986583 = {S13811, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MEEG_101 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986658 = {S13812, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; I8000S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986682 = {S13813, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; F23 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986753 = {S13814, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI Y300C Build/HuaweiY300C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2986794 = {S13815, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; Evopad R9730 Build/Evopad R9730) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2986992 = {S13816, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; Tablet Mint 8x8i DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2987020 = {S13817, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Touchlet X10.dual+ Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2987023 = {S13818, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 070R9 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2987155 = {S13819, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; NX501 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987185 = {S13820, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; U8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987307 = {S13821, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; MT-901 Build/MT-901) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2987319 = {S13822, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Elite Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987324 = {S13823, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N86-2cpu Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2987347 = {S13824, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; Kruger Matz 0793 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2987424 = {S13825, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID8089D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2988328 = {S13826, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; OMEGA-MID9711 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2988581 = {S13827, 153, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MODECOM FreeTAB 8001 IPS X23G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2988749 = {S13828, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; Lenovo-A706/S100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2988819 = {S13829, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 7014 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2988839 = {S13830, 148, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolgen E69 NOTE2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2989011 = {S13831, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; ZTE V956 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2989108 = {S13832, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Superb Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2989137 = {S13833, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PC738 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2989139 = {S13834, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; 7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2989364 = {S13835, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Phablet P1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2989367 = {S13836, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; dual7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2989629 = {S13837, 63, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; AML-MX REF Build/JRO03C)*/
const Device UA2989870 = {S13838, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; AP-803 Build/AP-803) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2990000 = {S13839, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; K-Touch U86 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2990359 = {S13840, 145, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; POLYTRON_W7450 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2990545 = {S13841, 173, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; M83w Build/M83w-eng 4.1.1 JRO03H test-keys_OTA:MID_2.1.1.1_X.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2990621 = {S13842, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; ZTE U956 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2990736 = {S13843, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; NTT 617 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2990902 = {S13844, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; M101w Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2991694 = {S13845, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Mi-A531 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2991832 = {S13846, 57, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; H100 Build/JRO03C)*/
const Device UA2992209 = {S13847, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; NetTAB_Space_3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2992216 = {S13848, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; MW0931 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2992220 = {S13849, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Acme TB04 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2992257 = {S13850, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TZ100 plus Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2992362 = {S13851, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HR-Q10R Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2992449 = {S13852, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; V511M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2992542 = {S13853, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; GDC PegasusII Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2992660 = {S13854, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; CE15711-54 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2992751 = {S13855, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; ZTE N818 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2992805 = {S13856, 154, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; Bouygues Telecom Bs 351 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2992823 = {S13857, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Quickfone N1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2993008 = {S13858, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-gb; C3_77_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2993051 = {S13859, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; iB755C Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2993096 = {S13860, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; MID8128 Build/GreenDemon_v1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2993286 = {S13861, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; XC11-01A Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2993815 = {S13862, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; C1041IPS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2993870 = {S13863, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; V801 Core4 Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2993874 = {S13864, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; CyrusApel Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994007 = {S13865, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; H9600+ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994096 = {S13866, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; PC738 Build/PC738) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2994218 = {S13867, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; DTA-07FSBH Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994251 = {S13868, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; OPT-220 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994257 = {S13869, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; NBX-7012 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2994328 = {S13870, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; S4701 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994329 = {S13871, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; TB-308 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2994440 = {S13872, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; YOUPU U6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994566 = {S13873, 145, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Tablet PC szenio 2500 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2994623 = {S13874, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; LETAB907 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2994663 = {S13875, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZTE-Z998 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994750 = {S13876, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; RT3066BT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2994774 = {S13877, 124, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; U803D Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2994883 = {S13878, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; id-id; T753 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994899 = {S13879, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; S98 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994903 = {S13880, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; T9100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994987 = {S13881, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; T3188 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2995010 = {S13882, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; HS-8DTB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2995359 = {S13883, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Y88 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2995543 = {S13884, 171, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; androVM for VirtualBox ('Phone' version) Build/JRO03S) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2995768 = {S13885, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ET7-20C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2995881 = {S13886, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; TR-1010 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2996010 = {S13887, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; H38 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996212 = {S13888, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; XTAB-781+ Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2996222 = {S13889, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A9DualCore Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2996277 = {S13890, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; S508 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996325 = {S13891, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; H7000+ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2996339 = {S13892, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N80RK Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2996343 = {S13893, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; BEN7116N Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996383 = {S13894, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AX9Z_Race Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996452 = {S13895, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fa-ir; NX7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2996553 = {S13896, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HS-E956Q Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996721 = {S13897, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AMAZE-731 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2996725 = {S13898, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; NoVa Pad 80 C168D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2996746 = {S13899, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TY-Q88 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996755 = {S13900, 150, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ph; MyPhone A919 3D Duo Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996847 = {S13901, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ANS-01/8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2996893 = {S13902, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-pt; Boston 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996919 = {S13903, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; hu-hu; AKAIPHA_5800 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996944 = {S13904, 147, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; KN_Mobile_QT-A08 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997099 = {S13905, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A13 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997169 = {S13906, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; S2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997182 = {S13907, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; EMPEROR Build/EMPEROR) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2997215 = {S13908, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; uphone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2997245 = {S13909, 145, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI MT1-T00 Build/HuaweiMT1-T00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2997259 = {S13910, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; EUTB-730 ARGOS NAV Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2997303 = {S13911, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZTE-V72A Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Safari/534.30*/
const Device UA2997339 = {S13912, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; ETB8448 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2997392 = {S13913, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SV7612-RK2926 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2997431 = {S13914, 147, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; E-Boda Supreme XL200IPS Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2997434 = {S13915, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; M805 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2997448 = {S13916, 72, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; i-mobile i-STYLE 7A Build/JZO54K)*/
const Device UA2997480 = {S13917, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; C-501 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2997581 = {S13918, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; SZENIO 2500 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2997591 = {S13919, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ST9001 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2997627 = {S13920, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PLT7045K Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2997684 = {S13921, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; G402 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997691 = {S13922, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; J9500 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2997701 = {S13923, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.0; id-id; Venera Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997900 = {S13924, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID8080 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2997919 = {S13925, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; momo20-w2 Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997928 = {S13926, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; M15GC702P Build/PD717) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998026 = {S13927, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TBDC1093 Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 GSA/2.0.0.X*/
const Device UA2998114 = {S13928, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IN700 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998123 = {S13929, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; S32 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998155 = {S13930, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; PENTAGRAM TAB 7.5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2998158 = {S13931, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; G19 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2998168 = {S13932, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Fontab SFT071 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2998183 = {S13933, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; CAPHG10-01 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998283 = {S13934, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ET7183G-V13 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2998284 = {S13935, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; WTAB 705 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2998307 = {S13936, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; GPLUS_GN810 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998308 = {S13937, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; NVR-TAB8 S1 4GB Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2998408 = {S13938, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID-7089D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2998441 = {S13939, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; GT-N7I02 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998550 = {S13940, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ar-eg; Q75 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998562 = {S13941, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; M709 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2998577 = {S13942, 147, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; KINGO TAB 7 PLUS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998622 = {S13943, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; AS127 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2998651 = {S13944, 72, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; TBDC1093 Build/X)*/
const Device UA2998702 = {S13945, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; DSlide973 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2998820 = {S13946, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hr-hr; TPC-97150 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2998830 = {S13947, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A721 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2998986 = {S13948, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; BS101 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2998994 = {S13949, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; RMD-855 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2999014 = {S13950, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; i908 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2999049 = {S13951, 148, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; Xenta TAB10-212 Build/Xenta TAB10-212) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2999055 = {S13952, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; EverClassic Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999096 = {S13953, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A105 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2999243 = {S13954, 215, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1 Release/04.22.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2999248 = {S13955, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; N10QM Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2999323 = {S13956, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; es-es; QBA769 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999341 = {S13957, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; S4501M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999386 = {S13958, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; S5301M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999408 = {S13959, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; RoverPad 3W9.4 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2999469 = {S13960, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; Mio Pad A1001 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2999471 = {S13961, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; FONE 450 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999486 = {S13962, 71, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; i-mobile i-style 8 Build/JZO54K)*/
const Device UA2999553 = {S13963, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; J920 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2999586 = {S13964, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HTC_606w Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999687 = {S13965, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; VIVO X2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999764 = {S13966, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M71 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3000163 = {S13967, 147, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; TY-K-Touch_C966e Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3000242 = {S13968, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HS-E912S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3000431 = {S13969, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nitro 500D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3000433 = {S13970, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; Radiance Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3000521 = {S13971, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Sweet M1018 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3000577 = {S13972, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MIDCD10PR Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3000589 = {S13973, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.4; en-us; Luna TAB07-101 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3000807 = {S13974, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Y82 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3001277 = {S13975, 213, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SAMSUNG-GT-S7898_TD/1.0 Android/4.1.2 Release/11.15.2012 Browser/AppleWebKit534.30 Build/JRO03L) ApplelWebkit/534.30 (KHTML,like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3001405 = {S13976, 124, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; G7 Build/X06I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3001707 = {S13977, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; PAD 9719QR Build/PAD 9719QR) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3001710 = {S13978, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CRCB2094 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3001858 = {S13979, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; BEE_9500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3002046 = {S13980, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E455g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3002428 = {S13981, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; A8839 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3002578 = {S13982, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; IFQM712HCNF Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3002648 = {S13983, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IT-M747-RK Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3002660 = {S13984, 219, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.1 Release/05.28.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3002765 = {S13985, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMP3970B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3002889 = {S13986, 122, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; M6 Build/M6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3002983 = {S13987, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; M045 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3003143 = {S13988, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Flytouch XR Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3003266 = {S13989, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; CTA-E07-11Z Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3003535 = {S13990, 115, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; YL-Coolpad_5216S/4.1.022.X.5216S;480*800;CTC/2.0) CoolpadWebkit/534.30*/
const Device UA3004081 = {S13991, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GOCLM7000KD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3004391 = {S13992, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; N79 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3004782 = {S13993, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Tab701D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3004789 = {S13994, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Troy_Duos_S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3005093 = {S13995, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; SENSUELLE Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3005192 = {S13996, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; E1050HG Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3006371 = {S13997, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TechniPad 8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3006529 = {S13998, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; SL1066 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3006561 = {S13999, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; mini S4 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3006832 = {S14000, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fa-ir; G4_NFC Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3006897 = {S14001, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; TPC-840M Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3007094 = {S14002, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; CTA-E07-10Z Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3007216 = {S14003, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EX020D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3007311 = {S14004, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3007545 = {S14005, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PD200 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3007697 = {S14006, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; Thunderbird Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3008012 = {S14007, 145, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Primux_Alpha_2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3008276 = {S14008, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; F816 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3008376 = {S14009, 146, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Starmobile JUMP Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3008509 = {S14010, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; HIPSTREET Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3008554 = {S14011, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; XZPAD210G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3008683 = {S14012, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1 ;zh-TW; A4 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3009001 = {S14013, 148, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xenta-TAB07-210 Build/Xenta-TAB07-210) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3009885 = {S14014, 148, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Fusion Flare v1.0 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3009900 = {S14015, 124, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; baoxue Build/F8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3010165 = {S14016, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; TID-97062 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3010452 = {S14017, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; VENERA 907 Build/X06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3010763 = {S14018, 145, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Primux_Omega_2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3011117 = {S14019, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ENERGY i10 DUAL Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3011242 = {S14020, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; 5100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3011752 = {S14021, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; EverMagic Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3011920 = {S14022, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; A701D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3012615 = {S14023, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ST70216-1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3012659 = {S14024, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; PocketBook SURFpad2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3012703 = {S14025, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; P717A Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3012752 = {S14026, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SBM205SH Build/S0002) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3012753 = {S14027, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; SP401 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3012959 = {S14028, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; Orion7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3013082 = {S14029, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; busbi 7 dual v1.0 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3013132 = {S14030, 145, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; fnac 7 plus v2 Build/1.0.0_X-15:03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3013241 = {S14031, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Xtab785 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3013246 = {S14032, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; CTA-E07-13A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3013265 = {S14033, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XM721RK Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3013268 = {S14034, 147, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; G526-L22 Build/HuaweiG526-L22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3013604 = {S14035, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MacTab Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3013798 = {S14036, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; R7X Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3014146 = {S14037, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SBM205SH Build/S0004) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3014373 = {S14038, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; I110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3014395 = {S14039, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A77 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3014639 = {S14040, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PTAB1050 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3014731 = {S14041, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; PAD1001 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3015101 = {S14042, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; BTL-703 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3016014 = {S14043, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TAB9DUALC8GB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3016241 = {S14044, 151, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; i-mobile i-note 3 Build/JB_JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3016314 = {S14045, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; CORE 10.1 DUAL 3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3016396 = {S14046, 151, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Symphony Xplorer W71 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3016614 = {S14047, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; fnac 10 3G v2 Build/1.0.1_X-09:03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3016943 = {S14048, 147, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; Lanix Ilium_S115 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3016946 = {S14049, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; i-mobile IQ1-1A Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3017155 = {S14050, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TB1000 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3017523 = {S14051, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; G905 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3017846 = {S14052, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; VA75 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3018074 = {S14053, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E451g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3018115 = {S14054, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; 3G9728 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3018297 = {S14055, 146, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IMPERIUS TAB 10 MT7011 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3018434 = {S14056, 158, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_D2-2010/D2-2010V100R001C92B133) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3018905 = {S14057, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 10D8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3019065 = {S14058, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NB4-3/5788 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3019085 = {S14059, 150, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Woxter Tablet PC Nimbus 70 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3019116 = {S14060, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Tablet Biox Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3020243 = {S14061, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AM1006 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3020494 = {S14062, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MTV SLASH Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3020618 = {S14063, 215, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1 Release/05.27.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3020750 = {S14064, 219, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.2 Release/06.18.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3021050 = {S14065, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XOLO Play Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3021265 = {S14066, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; fnac 10 3G v2 Build/1.0.0_X-17:05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3021274 = {S14067, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TBQC1063 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3021331 = {S14068, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Mi-491 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3021526 = {S14069, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; VX70 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3022608 = {S14070, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; t520 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022883 = {S14071, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; CT0712 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3023608 = {S14072, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EGM002 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3024044 = {S14073, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A60 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3024130 = {S14074, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TechniPad_10-3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3024175 = {S14075, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; IS-T71 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3025564 = {S14076, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; M4TEL SS990 Build/JRO03C) AppleWebKit/534.70 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.70*/
const Device UA3026630 = {S14077, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LR102 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3027504 = {S14078, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; CnM-TP-7DC-16-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3027512 = {S14079, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Trio Stealth_8 Build/JR003C.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3029368 = {S14080, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; CnM-7DC-16-3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3031034 = {S14081, 126, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; F818 Build/F818) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3032057 = {S14082, 157, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U; Android 4.1.1;en-us;Digitac Ultratab;C755 Build/ICS) AppleWebKit/531.21.10(KTHML,like Gecko) Version/4.1.1 Mobile/7B367 Safari/531.21.10*/
const Device UA3033831 = {S14083, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; AS-AT-SEVEN-X2-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3033949 = {S14084, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MID1014 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3034860 = {S14085, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; INGO_TAB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3043124 = {S14086, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; N10one Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3044025 = {S14087, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A86 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3044130 = {S14088, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; D712 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3044389 = {S14089, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; OBLIO Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3044400 = {S14090, 126, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; M6 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3044510 = {S14091, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; MID-972R-3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3044557 = {S14092, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ctab1012c Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3044641 = {S14093, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; OS818 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044660 = {S14094, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PRBT742 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044686 = {S14095, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M3000 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044723 = {S14096, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; U23GT-S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3044780 = {S14097, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; I5P Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044838 = {S14098, 80, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Woxter Tablet PC 85IPS Dual Build/JRO03H)*/
const Device UA3044841 = {S14099, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TA2508-7BK Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3045010 = {S14100, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; INCA-Astro Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3045026 = {S14101, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; GF-1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3045056 = {S14102, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; i9900 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3045404 = {S14103, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; Ezcool Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3045445 = {S14104, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; N1010 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3045478 = {S14105, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; EUTB-708 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3045607 = {S14106, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; KYOTO SM16 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3045665 = {S14107, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HMIA5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3045814 = {S14108, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; 1006-IPS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3045884 = {S14109, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; U21GT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3045936 = {S14110, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3046088 = {S14111, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; T9510E Build/HuaweiT9510E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3047205 = {S14112, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; KATBL10A08E Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3047274 = {S14113, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; TLINK475 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3047528 = {S14114, 70, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HUAWEI Y300C Build/HuaweiY300C)*/
const Device UA3047585 = {S14115, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; Polypad 528HD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3048124 = {S14116, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; TB04 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3048153 = {S14117, 126, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; M7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3048244 = {S14118, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; sl-si; RC0817C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3048419 = {S14119, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; Padphone 71 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3048605 = {S14120, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; H700S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3048617 = {S14121, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; ilium Pad E10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3048622 = {S14122, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Disgo_Tablet_8400G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3048724 = {S14123, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Teraware K707G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3048728 = {S14124, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; RC9712C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3048791 = {S14125, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; X718 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3048834 = {S14126, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 97RQ Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3048854 = {S14127, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; SP0978 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3048885 = {S14128, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; MiniGpad Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3048909 = {S14129, 126, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; X7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3048911 = {S14130, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ET754-B13 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049001 = {S14131, 145, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SonyM35h Build/12.0.A.1.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049085 = {S14132, 146, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; E-boda essential smile Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3049104 = {S14133, 148, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; Xenta TAB09-211 Build/Xenta_TAB09-211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3049107 = {S14134, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A430 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049165 = {S14135, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A530HD Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049164 = {S14136, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; RM-19 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3049213 = {S14137, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; MT-101 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3049369 = {S14138, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; AN7IG3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3049656 = {S14139, 126, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; U3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3049697 = {S14140, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; 8BPE0H Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3049749 = {S14141, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; ETB7447 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3049802 = {S14142, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; UM-770 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049827 = {S14143, 124, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; M1032 Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3049850 = {S14144, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; HW-03E Build/HuaweiD2-6114) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049931 = {S14145, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MMDI9900 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049970 = {S14146, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; Elektra XL Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050108 = {S14147, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; DENARI 1029 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3050249 = {S14148, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; Explosion Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050251 = {S14149, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; KLYAN2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3050309 = {S14150, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; CTAB1013BT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3050652 = {S14151, 116, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; YL-Coolpad_7268/4.1.005.X.7268; 540*960; CUCC/2.0) CoolpadWebkit/534.30*/
const Device UA3050823 = {S14152, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ET-102 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3050864 = {S14153, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; Jet core doble-7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3051165 = {S14154, 219, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.1 Release/05.18.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3051399 = {S14155, 148, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Maxwell 2 Plus Build/1.0.0_X-15:51) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3051979 = {S14156, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TABLET 10-2X Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3052369 = {S14157, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; KID PLAY Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3052380 = {S14158, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; T6188 Build/MocorDroid4.1.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3053691 = {S14159, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; CT600 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3054479 = {S14160, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; 720M 3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3054706 = {S14161, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HTC 606w Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3055149 = {S14162, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; R452 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3055382 = {S14163, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; X10.Quad Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3055670 = {S14164, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HS-EG970 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3056304 = {S14165, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; fnac 7 v2 Build/1.0.0_X-20:13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3056558 = {S14166, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; U705W Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3056561 = {S14167, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; COROM8000KP Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3056631 = {S14168, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SKYTEX-SP910 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3056739 = {S14169, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; N451 F1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3056855 = {S14170, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 7038 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3056948 = {S14171, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; s4003 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057017 = {S14172, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; VOLT Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057052 = {S14173, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; M709 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3057070 = {S14174, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; Tab3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3057071 = {S14175, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; Andy A4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057256 = {S14176, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; MD-1008 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3057320 = {S14177, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Canvas 2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057511 = {S14178, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; IMO_Z9 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3057560 = {S14179, 166, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_G610-C00/G610-C00V100R001C92B120; 540*960; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA3057580 = {S14180, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; s7503 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057650 = {S14181, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; MT7A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3057718 = {S14182, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; Joyz Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057786 = {S14183, 219, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.1 Release/06.17.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3057818 = {S14184, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Polo Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057837 = {S14185, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; el-gr; iQTab 3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3057907 = {S14186, 219, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.2 Release/06.26.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3058014 = {S14187, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ADM8000KP Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3058099 = {S14188, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; UNUSUAL U7X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3058233 = {S14189, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Infinix X530 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058426 = {S14190, 126, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; R7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3058601 = {S14191, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; V7011 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3058755 = {S14192, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; fnac 8 3G v2 Build/1.0.0_X-14:45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3058854 = {S14193, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NX010HI8G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3058921 = {S14194, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Digital2 Platinum Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3058926 = {S14195, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; CROSS AT1G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058949 = {S14196, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; NOTE SMART Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3059027 = {S14197, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Trio Stealth_9 Build/JR003C.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3059038 = {S14198, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; id-; mbk77_twn_cu_jb Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3059299 = {S14199, 151, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; MTC 970H Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30 SVN/040HPG4*/
const Device UA3059494 = {S14200, 160, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_C8815/C8815V100R001C92B122; 540*960; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA3059626 = {S14201, 145, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Camelio Family tablet Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3059709 = {S14202, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Voxson Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3059744 = {S14203, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Andi4a Radium Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3059978 = {S14204, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; A7S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3060076 = {S14205, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SOLT JB Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3060639 = {S14206, 173, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Atrix HD Build/HoloBlur-X v0.9 PurePerformanceT Edition) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3060702 = {S14207, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; DTR10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3062316 = {S14208, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Z99 Dual Core Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3063783 = {S14209, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PRO7D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3064178 = {S14210, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; HN-M740 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3066122 = {S14211, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; AT-AS43D4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3070252 = {S14212, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Aoson M11 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3070511 = {S14213, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; X5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3071331 = {S14214, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MT887 Build/6.7.2_GC-384) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3071391 = {S14215, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 097R1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3071412 = {S14216, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; P76t??(M7H6) Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3071667 = {S14217, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; ovo 2.0 Build/FR-MX68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3071713 = {S14218, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; Koobe S7 Easy tablet Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3071777 = {S14219, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; id-id; Realpad P700 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3071848 = {S14220, 148, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; POV_TAB-PROTAB30IPS10-3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3071940 = {S14221, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; AT97C RK66WA Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3071984 = {S14222, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HOSIN V60 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3072041 = {S14223, 147, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 820 Build/820-V1.0-2013.01.18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3072205 = {S14224, 145, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; E-Boda Essential A160 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3072234 = {S14225, 152, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; JXD-S6600-M6 Build/S6600-M6-01-XCN) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3072452 = {S14226, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; QW TB-1517 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3072560 = {S14227, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; KYOTO SM23 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3072816 = {S14228, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 5216S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3072868 = {S14229, 153, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; GALAXY S2 HD LTE E120K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3073172 = {S14230, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TransBook TRA01 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3073569 = {S14231, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; ZEST D740 Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3073830 = {S14232, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; NeoTab H105 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3073905 = {S14233, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; TC55 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3073928 = {S14234, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ET-250 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3074120 = {S14235, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; Mini one Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3074147 = {S14236, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; vivo S7w Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3074153 = {S14237, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MT-719 SII Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3074250 = {S14238, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; MID-9108 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3074257 = {S14239, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; N810 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3074280 = {S14240, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Omega Q Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3074283 = {S14241, 146, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Z2_WVGA_QXH Build/Android4.1.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3074270 = {S14242, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; N10-QM Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3074393 = {S14243, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fi-fi; RD805 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3074552 = {S14244, 148, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Full AOSP on Leopard SOC Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3074723 = {S14245, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; H58 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3075192 = {S14246, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; jpad 723a Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3076737 = {S14247, 154, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xenta TAB08-201-3G Build/Xenta TAB08-201-3G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3077722 = {S14248, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-; Connspeed IP71 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3077827 = {S14249, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; el-gr; iQTab Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3077856 = {S14250, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A5ENZE Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3077882 = {S14251, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; QMobile_A950 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078148 = {S14252, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; aTAB7DCI Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3078207 = {S14253, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Platium Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3078215 = {S14254, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TU-6818E Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3078365 = {S14255, 219, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.1 Release/06.25.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3078376 = {S14256, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; CloudPad 700TV Build/FSV01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078511 = {S14257, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A29 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078562 = {S14258, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; ilium Pad E8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3078843 = {S14259, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; F815 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3078925 = {S14260, 147, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ph; Livall_TELPAD_D7_tablet Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3079105 = {S14261, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; 3G7271 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079195 = {S14262, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MID1047 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3079339 = {S14263, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LG-E410B/V10c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079651 = {S14264, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; V0714 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079824 = {S14265, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XO Learning tablet Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3080007 = {S14266, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MPQC974 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3080024 = {S14267, 146, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; YL-Coolpad_5950 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3080258 = {S14268, 163, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=m7;devm=HTC One;man=HTC;brw=Android;shwd=1080x1920;shwa=1080x1845;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA3080919 = {S14269, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; E9000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3081580 = {S14270, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D9018 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3081634 = {S14271, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; @TAB-EIGHT-X2-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3086113 = {S14272, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; CL500 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3086474 = {S14273, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; PB-S8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3086521 = {S14274, 147, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PIRANHA Pro II Tab 9.0" Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3086759 = {S14275, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; TAB7B Build/WEXLER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3086962 = {S14276, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 740T Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3087016 = {S14277, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HR-738R Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3087103 = {S14278, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; BOHP-P5A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3087224 = {S14279, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GIPO-T100F Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3087236 = {S14280, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; NeoTab B90 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3087306 = {S14281, 76, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; klu1506 Build/JRO03H)*/
const Device UA3087615 = {S14282, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; HTC_PN074 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3087698 = {S14283, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID-7000B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3087750 = {S14284, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HTC_801e Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3087853 = {S14285, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; da-dk; DSRV-9708C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3087922 = {S14286, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; S9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3087958 = {S14287, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MID-970DCHI Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3088134 = {S14288, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; samsung Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088149 = {S14289, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; ND-973 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3088261 = {S14290, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SC-97JB Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3088328 = {S14291, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; CTVISION Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3088616 = {S14292, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HELLOTAB2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3088669 = {S14293, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TPC1051 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3088764 = {S14294, 160, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; M35h Build/12.0.A.1.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 BaiduBoxApp/3.6_Xc*/
const Device UA3088788 = {S14295, 148, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xenta TAB08-200 Build/Xenta TAB08-200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3088923 = {S14296, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; SL966 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3088926 = {S14297, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; GiONEE_GN810 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3089385 = {S14298, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.X; de-de; Xelio10Pro Build/ODYS_Xelio) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3090062 = {S14299, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Commodore  IS-7107C Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3090380 = {S14300, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-S7898 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3091595 = {S14301, 148, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI C8815 Build/HuaweiC8815) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3091798 = {S14302, 60, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SBM205SH Build/S0004)*/
const Device UA3091814 = {S14303, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; A6S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3091850 = {S14304, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PLT8223G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3091865 = {S14305, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; G9 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3091976 = {S14306, 148, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; Trevi PHABLET 6 S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3091990 = {S14307, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; NaviPad TM-7045 3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3091999 = {S14308, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; smart_a17 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA3092036 = {S14309, 219, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.2 Release/06.17.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3092208 = {S14310, 125, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ICON Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092217 = {S14311, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; JAY-TECH Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092235 = {S14312, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; P886A Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3092274 = {S14313, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; TIGER 3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092320 = {S14314, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LT7035-F Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092339 = {S14315, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; Quadra 7 UltraSlim Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3092363 = {S14316, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; FUSION2IN1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3092443 = {S14317, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; TAB10IS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3092457 = {S14318, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 101B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3092525 = {S14319, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; AF9922 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092559 = {S14320, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; V100MD Build/V100MD.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3092592 = {S14321, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; KB706 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092662 = {S14322, 219, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.2 Release/07.02.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3092852 = {S14323, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; LIVE Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092907 = {S14324, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-nz; innos i3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092915 = {S14325, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; CTC07SO Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3093375 = {S14326, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GSSMID-7002S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3096206 = {S14327, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; FT970 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3099328 = {S14328, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; CW-V19 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3099852 = {S14329, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; C7/K7_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099878 = {S14330, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC_603e Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099983 = {S14331, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 7268 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3100206 = {S14332, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; Impact 2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3100283 = {S14333, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; 8730 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3100618 = {S14334, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; PAD-2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3100634 = {S14335, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Visture V5 HD Build/Visture V5 HD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3100812 = {S14336, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; U1203 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3101078 = {S14337, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; O+ 9.76 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3101328 = {S14338, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; Nettab Thor X Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3101365 = {S14339, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC 603e Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3101421 = {S14340, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; TOKS7G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3101448 = {S14341, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; K-Touch U90 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3101846 = {S14342, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I759 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3101975 = {S14343, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; N90FHDRK Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3102087 = {S14344, 150, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MT6517-H369X Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3102155 = {S14345, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; N800 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102183 = {S14346, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Titan6575-SA Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102340 = {S14347, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; B95+ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102617 = {S14348, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; RC0718C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3102788 = {S14349, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; ID-K97 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3102860 = {S14350, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Xtab1081HD+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3102926 = {S14351, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; M716D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102970 = {S14352, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; M-PAD Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3102975 = {S14353, 145, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; TY-K-Touch_E88 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102985 = {S14354, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; ZTE Kis Pro Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103124 = {S14355, 145, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Polaroid 40490 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103135 = {S14356, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GM80 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3103137 = {S14357, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; iTAB704j Build/PD715T) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103140 = {S14358, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; I9082 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3103184 = {S14359, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; ES802 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3103196 = {S14360, 147, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Device_Navigator Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103208 = {S14361, 150, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; TouchTab 13.3 DC Build/TouchTab 13.3 DC) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3103245 = {S14362, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; N9510 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103334 = {S14363, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 101P11C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3103349 = {S14364, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; APPTB102S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3103609 = {S14365, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; T06 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103725 = {S14366, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; NTT 528 Build/NTTX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3103941 = {S14367, 214, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MT6517-Rave-S89/1.0 Android/4.1.1 Release/05.06.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3104127 = {S14368, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; c410 Build/FSV04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3104293 = {S14369, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; LEOTEC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3104476 = {S14370, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; TA55 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3105643 = {S14371, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PROG400 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3105808 = {S14372, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HTC 608t Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3106094 = {S14373, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; TA-734 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3106492 = {S14374, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZTE_N9511 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3106902 = {S14375, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; SN97T41W Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3106980 = {S14376, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; TGA22 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3107411 = {S14377, 173, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; M83g Build/M83g-eng 4.1.1 JRO03H test-keys_OTA:MID_2.1.1.1_X.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3107460 = {S14378, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PRODG16 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3107474 = {S14379, 146, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ro-ro; ALLVIEW_P5_Qmax Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3107520 = {S14380, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Engage 8 Build/Engage 8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3107650 = {S14381, 115, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; YL-Coolpad_5879/4.1.002.X.5879; 480*854; CTC/2.0) CoolpadWebkit/534.30*/
const Device UA3107661 = {S14382, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZTE-Z995 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3107698 = {S14383, 150, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Huawei Y301A2 Build/HuaweiY301A2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3107790 = {S14384, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; TwinTAB-T7283GD1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3107806 = {S14385, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; DL HD EV 01 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3107845 = {S14386, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Zigo Eon6i Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3107891 = {S14387, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; u18 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3107946 = {S14388, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MIDC970 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3107944 = {S14389, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A820 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3107950 = {S14390, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; PMP7380D3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3107954 = {S14391, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; AIRIS_TM530 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3107964 = {S14392, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; Micromax A86 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108043 = {S14393, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Table PC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3108145 = {S14394, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; 5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108176 = {S14395, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-P712 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108238 = {S14396, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; P1001A Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3108284 = {S14397, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; P5100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108274 = {S14398, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; el-gr; Turbox-X G410 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108344 = {S14399, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; el-gr; TURBOX X-2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108386 = {S14400, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; LG-E415f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108421 = {S14401, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MPQC804HD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3108639 = {S14402, 146, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; YL-Coolpad_5891 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108657 = {S14403, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; Mi731 Build/Mi731) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3108662 = {S14404, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI C8815 Build/HuaweiC8815) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3108848 = {S14405, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HALO_VALUE+ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3108862 = {S14406, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; A53 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3109178 = {S14407, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; E812 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3109195 = {S14408, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAB10-400Build/NobleTAB10-400) AppleWebKit/534.30 (KHTML, likeGecko) Version/4.0 Safari/534.30*/
const Device UA3109278 = {S14409, 180, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=SGH-I717;devm=SAMSUNG-SGH-I717;man=samsung;brw=Android;shwd=800x1280;shwa=800x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA3109449 = {S14410, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xtreme-V30 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3110393 = {S14411, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Nimbus 100Q Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3111065 = {S14412, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NuclearSX-SP5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3111689 = {S14413, 149, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; i-mobile IQ5.1 Pro Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3114388 = {S14414, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GEM8902 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3114581 = {S14415, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; E821 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3118459 = {S14416, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; Pixus Play Two Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3118729 = {S14417, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; EBEST L5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3118877 = {S14418, 126, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; V8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3119035 = {S14419, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; T500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3119184 = {S14420, 148, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolgen E70 NOTES Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3119235 = {S14421, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; T29 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3119310 = {S14422, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; Ultimix77 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3119439 = {S14423, 56, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; S71 Build/JRO03H)*/
const Device UA3119644 = {S14424, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; ODEON TVTab-701 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3119655 = {S14425, 150, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; SUPREME IPS Dual Core X200 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3120138 = {S14426, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LINSAY COSMOS F-7HD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3120159 = {S14427, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SC-1007JB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3120180 = {S14428, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MyPhone A888 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3120384 = {S14429, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; RTB7.4DFUN Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3120386 = {S14430, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; INP97XP Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3120480 = {S14431, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; MTO_MK318 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3120537 = {S14432, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ARCHOS 97G10 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3120687 = {S14433, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; i1017 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3120783 = {S14434, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; XO Learning tablet Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3120865 = {S14435, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; SZENIO 2000 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3120978 = {S14436, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; ND-R726B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121027 = {S14437, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; EVERPAD SC-710 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3121029 = {S14438, 155, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; PENTAGRAM TAB Quadra 9.7 Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121038 = {S14439, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; SELECT7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121076 = {S14440, 219, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.1 Release/05.03.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3121086 = {S14441, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SNM-M35c Build/12.0.B.2.42) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121134 = {S14442, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; X920 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3121150 = {S14443, 146, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; TY-K-Touch_E616 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121163 = {S14444, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; I5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121187 = {S14445, 145, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; HTC-one Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3121493 = {S14446, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; ADT2.6 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3121554 = {S14447, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; F53s Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121579 = {S14448, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; H38 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121649 = {S14449, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; N70 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3121724 = {S14450, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; M720 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121807 = {S14451, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; K-Touch Q4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121816 = {S14452, 127, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; C22 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3122051 = {S14453, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; Space Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3122156 = {S14454, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; M-MP980S4 Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3122230 = {S14455, 219, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.1 Release/05.15.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3122362 = {S14456, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; LC1016C Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3122612 = {S14457, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; TB-7 v2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3122629 = {S14458, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; M5-3G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3122676 = {S14459, 219, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.1.1 Release/05.07.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3122898 = {S14460, 140, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EX_NOTE_? Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3124514 = {S14461, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; MID800 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3124522 = {S14462, 139, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; HRA_M710 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3124618 = {S14463, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; PAN7 HS Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3124653 = {S14464, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CLP 282 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3124677 = {S14465, 150, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hr-hr; 01_v77_yp1_y7800ntv Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3124777 = {S14466, 56, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; A7S Build/JZO54K)*/
const Device UA3124807 = {S14467, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; PrimuxZonda3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3124835 = {S14468, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; T8 3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3124841 = {S14469, 131, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TBDG773 Build/Zeki.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3124880 = {S14470, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; SPC P5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3124882 = {S14471, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; a5mid Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3124920 = {S14472, 129, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; RX4DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3124961 = {S14473, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AX8 NOTE II Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125017 = {S14474, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xolo QC800 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3125090 = {S14475, 151, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GPad 7.0 Excellent 4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125099 = {S14476, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; NBX-8012 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3125149 = {S14477, 138, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LGMS500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125177 = {S14478, 135, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ETAB123MKII Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3125183 = {S14479, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; DC-I9508 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3125241 = {S14480, 130, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HX-M02 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3125259 = {S14481, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; REVOMAX_2012 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125317 = {S14482, 148, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Maxwell 2 Plus Build/1.0.1_X-09:33) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3125331 = {S14483, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; KDI-I406 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3125357 = {S14484, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; KB708 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125371 = {S14485, 133, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; KBOOKPLUS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3125420 = {S14486, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; Combo Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125457 = {S14487, 143, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 5950 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125593 = {S14488, 144, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; fnac 10 3G v2 Build/1.0.2_X-09:39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3126355 = {S14489, 149, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; SM-T211-ORANGE/T211XXAMGC Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3126538 = {S14490, 141, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; id-id; CROSS-A20G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3126731 = {S14491, 137, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GV701S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3126934 = {S14492, 134, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; BioniQ 800 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3127008 = {S14493, 136, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; U680C Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3127073 = {S14494, 128, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; EHT9 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3127234 = {S14495, 132, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PLT9045K Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3127396 = {S14496, 142, { &P15364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; NABIJR-NV5B Build/JZO54L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2357296 = {S14497, 119, { &P17595,&P17296,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2437165 = {S14498, 120, { &P17595,&P17296,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11*/
const Device UA2437171 = {S14499, 120, { &P17595,&P17296,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11*/
const Device UA2437175 = {S14500, 120, { &P17595,&P17296,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11*/
const Device UA2437182 = {S14501, 120, { &P17595,&P17296,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11*/
const Device UA2437240 = {S14502, 120, { &P17595,&P17296,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11*/
const Device UA2437241 = {S14503, 120, { &P17595,&P17296,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11*/
const Device UA2437246 = {S14504, 120, { &P17595,&P17296,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11*/
const Device UA2437263 = {S14505, 120, { &P17595,&P17296,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11*/
const Device UA2437332 = {S14506, 130, { &P17595,&P17296,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11,gzip(gfe)*/
const Device UA2437477 = {S14507, 159, { &P17595,&P17296,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2597685 = {S14508, 116, { &P17595,&P17296,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1060) AppleWebKit/537.11 (KHTML like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2674646 = {S14509, 129, { &P17595,&P17296,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/23.0.648.205 Safari/534.16*/
const Device UA2674684 = {S14510, 129, { &P17595,&P17296,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/23.0.648.205 Safari/534.16*/
const Device UA2714921 = {S14511, 129, { &P17595,&P17296,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/23.0.648.205 Safari/534.16*/
const Device UA2434465 = {S14512, 141, { &P15767,&P18432,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A523 Safari/8536.25*/
const Device UA2434473 = {S14513, 142, { &P15767,&P18432,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A8426 Safari/8536.25*/
const Device UA2434668 = {S14514, 139, { &P15767,&P18432,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A403 Safari/8536.25*/
const Device UA2434669 = {S14515, 141, { &P15767,&P18432,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A550 Safari/8536.25*/
const Device UA2434691 = {S14516, 142, { &P15767,&P18432,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A8500 Safari/8536.25*/
const Device UA2435094 = {S14517, 139, { &P15767,&P18432,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A406 Safari/8536.25*/
const Device UA2435380 = {S14518, 139, { &P15767,&P18432,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A407 Safari/8536.25*/
const Device UA2492148 = {S14519, 119, { &P17595,&P17297,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492182 = {S14520, 119, { &P17595,&P17297,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492220 = {S14521, 119, { &P17595,&P17297,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492198 = {S14522, 119, { &P17595,&P17297,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492401 = {S14523, 119, { &P17595,&P17297,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492513 = {S14524, 119, { &P17595,&P17297,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492625 = {S14525, 129, { &P17595,&P17297,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)*/
const Device UA2511169 = {S14526, 158, { &P17595,&P17297,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2531609 = {S14527, 129, { &P17595,&P17297,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)*/
const Device UA2658725 = {S14528, 125, { &P17595,&P17297,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4; en-US) AppleWebKit/537.13 (KHTML, like Gecko) Chrome/24.0.1290.1 Safari/537.13*/
const Device UA47654 = {S14529, 161, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SAMSUNG GT-I9001/I9001BUKG1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47675 = {S14530, 142, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47686 = {S14531, 160, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; GT-I9001-ORANGE/I9001BVKP7 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60558 = {S14532, 142, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70202 = {S14533, 160, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; GT-I9001-ORANGE/I9001BVKP8 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71688 = {S14534, 160, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; GT-I9001-ORANGE/I9001BVKPA Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78402 = {S14535, 142, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; nl-nl; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90783 = {S14536, 161, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; SAMSUNG GT-I9001/I9001BUKE5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA220513 = {S14537, 66, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-I9001 Build/GINGERBREAD)*/
const Device UA355110 = {S14538, 66, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-I9001 Build/GINGERBREAD)*/
const Device UA575024 = {S14539, 120, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; de-de; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA603201 = {S14540, 161, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-I9001/I9001BUKP7 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622439 = {S14541, 148, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; cs-cz; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30*/
const Device UA624073 = {S14542, 66, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-I9001 Build/GINGERBREAD)*/
const Device UA627519 = {S14543, 142, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633320 = {S14544, 161, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; SAMSUNG GT-I9001/I9001BUKP2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA677359 = {S14545, 66, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9001 Build/GINGERBREAD)*/
const Device UA680393 = {S14546, 130, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9001 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA687305 = {S14547, 130, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9001 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA726575 = {S14548, 120, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-gb; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA735238 = {S14549, 120, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; es-es; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA767992 = {S14550, 120, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-ie; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA892031 = {S14551, 120, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; it-it; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA1906343 = {S14552, 120, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; ro-ro; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA1922094 = {S14553, 120, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA1968567 = {S14554, 120, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; de-at; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA1989688 = {S14555, 160, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-I9001-ORANGE/I9001BVKPC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1993055 = {S14556, 161, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-I9001/I9001BUKP9 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1993274 = {S14557, 135, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9001 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2021462 = {S14558, 130, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT-I9001 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2024218 = {S14559, 149, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; GT-I9001 Build/NitroX GingerBread) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028191 = {S14560, 160, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-I9001-ORANGE/I9001BVKPG Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031464 = {S14561, 140, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-I9001 Build/GB. BlazR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033077 = {S14562, 135, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9001 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2145722 = {S14563, 120, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; de-ch; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2215979 = {S14564, 140, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2221838 = {S14565, 120, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; pt-pt; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2232877 = {S14566, 161, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-I9001/I9001BOKQ4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2307375 = {S14567, 140, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2371110 = {S14568, 140, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2371354 = {S14569, 140, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2485244 = {S14570, 161, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-I9001/I9001BUKPC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2811178 = {S14571, 130, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fa-fa; GT-I9001 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2953185 = {S14572, 160, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-I9001-ORANGE/I9001BVKPK Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3069652 = {S14573, 120, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; fr-ch; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA3127262 = {S14574, 160, { &P17324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-I9001-ORANGE/I9001BVKPN Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2567777 = {S14575, 104, { &P15364,&P17285,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.2.0.0 Safari/537.22*/
const Device UA2567780 = {S14576, 97, { &P15364,&P17285,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.2.0.0 Safari/537.22*/
const Device UA2567791 = {S14577, 102, { &P15364,&P17285,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2567806 = {S14578, 109, { &P15364,&P17285,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2567938 = {S14579, 112, { &P15364,&P17285,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)*/
const Device UA2569767 = {S14580, 104, { &P15364,&P17285,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.1.0.0 Safari/537.22*/
const Device UA2570199 = {S14581, 97, { &P15364,&P17285,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.1.0.0 Safari/537.22*/
const Device UA2584247 = {S14582, 119, { &P15364,&P17285,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)*/
const Device UA2584752 = {S14583, 141, { &P15364,&P17285,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3086933 = {S14584, 100, { &P15364,&P17285,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.22 (KHTML like Gecko) Chrome/25.0.1364.97 Safari/537.22*/
const Device UA3107951 = {S14585, 164, { &P15364,&P17285,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22 BitdefenderSafepay/17.14.0.0 (OBK; 25.0.1364.172; r1194.2013)*/
const Device UA891535 = {S14586, 151, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonST25i Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA898411 = {S14587, 139, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; ST25i Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA900679 = {S14588, 151, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-ca; SonyEricssonST25a Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA902029 = {S14589, 153, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; SonyEricssonST25i-o Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA904153 = {S14590, 152, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-at; SonyEricssonST25iv Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA924100 = {S14591, 153, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonST25i-o Build/6.0.B.1.576) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1956580 = {S14592, 151, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-ie; SonyEricssonST25i Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1990415 = {S14593, 151, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; SonyEricssonST25a Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1997108 = {S14594, 151, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2003029 = {S14595, 139, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024572 = {S14596, 63, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; ST25a Build/6.0.B.1.564)*/
const Device UA2026824 = {S14597, 132, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ST25i Build/6.0.B.1.564) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2029101 = {S14598, 139, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-mx; ST25a Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030424 = {S14599, 132, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ST25i Build/6.0.B.3.184) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2030437 = {S14600, 63, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; ST25i Build/6.0.B.3.184)*/
const Device UA2033185 = {S14601, 139, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; ST25i Build/6.0.B.1.576) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2042952 = {S14602, 153, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonST25i-o Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2166814 = {S14603, 151, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonST25i Build/6.0.B.1.576) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2168316 = {S14604, 152, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-nz; SonyEricssonST25iv Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2205590 = {S14605, 151, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; SonyEricssonST25i Build/6.0.B.3.189) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2228159 = {S14606, 152, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-nz; SonyEricssonST25iv Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2237439 = {S14607, 151, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SonyEricssonST25i Build/6.0.B.3.188) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2340510 = {S14608, 139, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; ST25a Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2350834 = {S14609, 151, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-mx; SonyEricssonST25a Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2384581 = {S14610, 151, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SonyEricssonST25a Build/6.0.B.3.188) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2420620 = {S14611, 139, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ST25i Build/6.0.B.3.188) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2452223 = {S14612, 139, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; ST25i Build/6.0.B.3.189) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2455037 = {S14613, 139, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-mx; ST25a Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2528296 = {S14614, 139, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ST25a Build/6.0.B.3.188) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2688461 = {S14615, 139, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-ie; ST25i Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2759938 = {S14616, 149, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; iw-; SonyEricssonST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2815649 = {S14617, 158, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonST25i Build/KA02 Xperia SSpeed) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982737 = {S14618, 149, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; iw-; SonyEricssonST25a Build/6.0.B.3.188) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2988977 = {S14619, 151, { &P17625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonST25i Build/GB-ROCKS V3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2900225 = {S14620, 57, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.16*/
const Device UA2900254 = {S14621, 64, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.16*/
const Device UA2901023 = {S14622, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.16*/
const Device UA2901035 = {S14623, 74, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.16,gzip(gfe)*/
const Device UA2901129 = {S14624, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.16*/
const Device UA2901131 = {S14625, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.16*/
const Device UA2901141 = {S14626, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.16*/
const Device UA2901142 = {S14627, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.16*/
const Device UA2944658 = {S14628, 80, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.16*/
const Device UA2944660 = {S14629, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6327)) Presto/2.12.388 Version/12.16*/
const Device UA2944816 = {S14630, 67, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.16,gzip(gfe)*/
const Device UA2944818 = {S14631, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6561)) Presto/2.12.388 Version/12.16*/
const Device UA2944893 = {S14632, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6648)) Presto/2.12.388 Version/12.16*/
const Device UA2944959 = {S14633, 69, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64) Opera 12.16*/
const Device UA2944970 = {S14634, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.16*/
const Device UA2944979 = {S14635, 75, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.16*/
const Device UA2944983 = {S14636, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.16*/
const Device UA2945084 = {S14637, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.16*/
const Device UA2945089 = {S14638, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.16*/
const Device UA2945193 = {S14639, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6657)) Presto/2.12.388 Version/12.16*/
const Device UA2945211 = {S14640, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.16*/
const Device UA2945433 = {S14641, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.16*/
const Device UA2945441 = {S14642, 80, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.16*/
const Device UA2945443 = {S14643, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6628)) Presto/2.12.388 Version/12.16*/
const Device UA2945587 = {S14644, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.16*/
const Device UA2945614 = {S14645, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.16*/
const Device UA2945619 = {S14646, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.16*/
const Device UA2945628 = {S14647, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.16*/
const Device UA2945629 = {S14648, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.16*/
const Device UA2945630 = {S14649, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6322)) Presto/2.12.388 Version/12.16*/
const Device UA2945633 = {S14650, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.16*/
const Device UA2945672 = {S14651, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.16*/
const Device UA2945729 = {S14652, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.16*/
const Device UA2945730 = {S14653, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.16*/
const Device UA2945734 = {S14654, 87, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.16*/
const Device UA2945830 = {S14655, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.16*/
const Device UA2945841 = {S14656, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6323)) Presto/2.12.388 Version/12.16*/
const Device UA2945909 = {S14657, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.16*/
const Device UA2946061 = {S14658, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.16*/
const Device UA2946072 = {S14659, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.16*/
const Device UA2946077 = {S14660, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.16*/
const Device UA2946067 = {S14661, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6325)) Presto/2.12.388 Version/12.16*/
const Device UA2946456 = {S14662, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.1 (build 6283)) Presto/2.12.388 Version/12.16*/
const Device UA2946479 = {S14663, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.16*/
const Device UA2946480 = {S14664, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6638)) Presto/2.12.388 Version/12.16*/
const Device UA2947039 = {S14665, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.16*/
const Device UA2947239 = {S14666, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.16*/
const Device UA2947383 = {S14667, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.16*/
const Device UA2947385 = {S14668, 87, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.16*/
const Device UA2947389 = {S14669, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.16*/
const Device UA2948041 = {S14670, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.16*/
const Device UA2948453 = {S14671, 80, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.16*/
const Device UA2948529 = {S14672, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.16*/
const Device UA2948536 = {S14673, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6336)) Presto/2.12.388 Version/12.16*/
const Device UA2948540 = {S14674, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6665)) Presto/2.12.388 Version/12.16*/
const Device UA2963589 = {S14675, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.16*/
const Device UA2963792 = {S14676, 80, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.16*/
const Device UA2964935 = {S14677, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.16*/
const Device UA2964936 = {S14678, 80, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.16*/
const Device UA2964938 = {S14679, 87, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.16*/
const Device UA2964951 = {S14680, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.16*/
const Device UA2964952 = {S14681, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6573)) Presto/2.12.388 Version/12.16*/
const Device UA2965303 = {S14682, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.16*/
const Device UA2965305 = {S14683, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6261)) Presto/2.12.388 Version/12.16*/
const Device UA2965315 = {S14684, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.16*/
const Device UA2965321 = {S14685, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6666)) Presto/2.12.388 Version/12.16*/
const Device UA2965371 = {S14686, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.16*/
const Device UA2965377 = {S14687, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.16*/
const Device UA2965421 = {S14688, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5997)) Presto/2.12.388 Version/12.16*/
const Device UA2965551 = {S14689, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6277)) Presto/2.12.388 Version/12.16*/
const Device UA2965738 = {S14690, 80, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.16*/
const Device UA2965736 = {S14691, 80, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.16*/
const Device UA2965742 = {S14692, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.16*/
const Device UA2965923 = {S14693, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6645)) Presto/2.12.388 Version/12.16*/
const Device UA2966292 = {S14694, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6080)) Presto/2.12.388 Version/12.16*/
const Device UA2966320 = {S14695, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.16*/
const Device UA2966448 = {S14696, 80, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.16*/
const Device UA2966463 = {S14697, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.16*/
const Device UA2966472 = {S14698, 80, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.16*/
const Device UA2966978 = {S14699, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.16*/
const Device UA2967063 = {S14700, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.16*/
const Device UA2967505 = {S14701, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.16*/
const Device UA2967699 = {S14702, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6668)) Presto/2.12.388 Version/12.16*/
const Device UA2967826 = {S14703, 87, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.16*/
const Device UA2968188 = {S14704, 87, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.16*/
const Device UA2968226 = {S14705, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.16*/
const Device UA2968232 = {S14706, 80, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.16*/
const Device UA2968236 = {S14707, 87, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.16*/
const Device UA2968800 = {S14708, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6337)) Presto/2.12.388 Version/12.16*/
const Device UA2969405 = {S14709, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.16*/
const Device UA2969694 = {S14710, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6667)) Presto/2.12.388 Version/12.16*/
const Device UA2971237 = {S14711, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6341)) Presto/2.12.388 Version/12.16*/
const Device UA2971342 = {S14712, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6669)) Presto/2.12.388 Version/12.16*/
const Device UA2974567 = {S14713, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6673)) Presto/2.12.388 Version/12.16*/
const Device UA2974838 = {S14714, 84, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.16,gzip(gfe),gzip(gfe)*/
const Device UA3011035 = {S14715, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.16*/
const Device UA3011038 = {S14716, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6014)) Presto/2.12.388 Version/12.16*/
const Device UA3012297 = {S14717, 96, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.16,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3012405 = {S14718, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.16*/
const Device UA3012807 = {S14719, 80, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.16*/
const Device UA3012808 = {S14720, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.16*/
const Device UA3013165 = {S14721, 80, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03658)) Presto/2.12.388 Version/12.16*/
const Device UA3013174 = {S14722, 87, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5217)) Presto/2.12.388 Version/12.16*/
const Device UA3013311 = {S14723, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.16*/
const Device UA3013428 = {S14724, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6661)) Presto/2.12.388 Version/12.16*/
const Device UA3013782 = {S14725, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4933)) Presto/2.12.388 Version/12.16*/
const Device UA3013825 = {S14726, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.16*/
const Device UA3013944 = {S14727, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5996)) Presto/2.12.388 Version/12.16*/
const Device UA3014304 = {S14728, 87, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5300)) Presto/2.12.388 Version/12.16*/
const Device UA3014484 = {S14729, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.16*/
const Device UA3014706 = {S14730, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5660)) Presto/2.12.388 Version/12.16*/
const Device UA3014893 = {S14731, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4661)) Presto/2.12.388 Version/12.16*/
const Device UA3014896 = {S14732, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6620)) Presto/2.12.388 Version/12.16*/
const Device UA3015167 = {S14733, 80, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03789)) Presto/2.12.388 Version/12.16*/
const Device UA3015321 = {S14734, 87, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.16*/
const Device UA3015325 = {S14735, 87, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.16*/
const Device UA3015455 = {S14736, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6324)) Presto/2.12.388 Version/12.16*/
const Device UA3015456 = {S14737, 87, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5299)) Presto/2.12.388 Version/12.16*/
const Device UA3016129 = {S14738, 87, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03639)) Presto/2.12.388 Version/12.16*/
const Device UA3016453 = {S14739, 74, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.16 Yandex.Translate*/
const Device UA3016471 = {S14740, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.16*/
const Device UA3016599 = {S14741, 87, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.6 (build 03402)) Presto/2.12.388 Version/12.16*/
const Device UA3017065 = {S14742, 87, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5278)) Presto/2.12.388 Version/12.16*/
const Device UA3017789 = {S14743, 103, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.16,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3018247 = {S14744, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.16*/
const Device UA3018480 = {S14745, 80, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5230)) Presto/2.12.388 Version/12.16*/
const Device UA3018872 = {S14746, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.16*/
const Device UA3019110 = {S14747, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6035)) Presto/2.12.388 Version/12.16*/
const Device UA3019132 = {S14748, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6044)) Presto/2.12.388 Version/12.16*/
const Device UA3019671 = {S14749, 80, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.16*/
const Device UA3022743 = {S14750, 77, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.16,gzip(gfe),gzip(gfe)*/
const Device UA3024060 = {S14751, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6675)) Presto/2.12.388 Version/12.16*/
const Device UA3026868 = {S14752, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6676)) Presto/2.12.388 Version/12.16*/
const Device UA3030013 = {S14753, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6681)) Presto/2.12.388 Version/12.16*/
const Device UA3033329 = {S14754, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.16*/
const Device UA3034644 = {S14755, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6346)) Presto/2.12.388 Version/12.16*/
const Device UA3036342 = {S14756, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6682)) Presto/2.12.388 Version/12.16*/
const Device UA3039310 = {S14757, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6347)) Presto/2.12.388 Version/12.16*/
const Device UA3057014 = {S14758, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6320)) Presto/2.12.388 Version/12.16*/
const Device UA3057033 = {S14759, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6046)) Presto/2.12.388 Version/12.16*/
const Device UA3057060 = {S14760, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4110)) Presto/2.12.388 Version/12.16*/
const Device UA3057066 = {S14761, 87, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5336)) Presto/2.12.388 Version/12.16*/
const Device UA3058855 = {S14762, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6345)) Presto/2.12.388 Version/12.16*/
const Device UA3077434 = {S14763, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5988)) Presto/2.12.388 Version/12.16*/
const Device UA3077673 = {S14764, 80, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5288)) Presto/2.12.388 Version/12.16*/
const Device UA3077872 = {S14765, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6032)) Presto/2.12.388 Version/12.16*/
const Device UA3078077 = {S14766, 80, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.16*/
const Device UA3078460 = {S14767, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5734)) Presto/2.12.388 Version/12.16*/
const Device UA3082230 = {S14768, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.2 (build 7166)) Presto/2.12.388 Version/12.16*/
const Device UA3091858 = {S14769, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5650)) Presto/2.12.388 Version/12.16*/
const Device UA3095812 = {S14770, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.2 (build 7178)) Presto/2.12.388 Version/12.16*/
const Device UA3097280 = {S14771, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.2 (build 7188)) Presto/2.12.388 Version/12.16*/
const Device UA3107369 = {S14772, 80, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03755)) Presto/2.12.388 Version/12.16*/
const Device UA3107477 = {S14773, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.1 (build 6340)) Presto/2.12.388 Version/12.16*/
const Device UA3115500 = {S14774, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.2 (build 7194)) Presto/2.12.388 Version/12.16*/
const Device UA3124601 = {S14775, 79, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4832)) Presto/2.12.388 Version/12.16*/
const Device UA3124863 = {S14776, 86, { &P15364,&P7133,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6338)) Presto/2.12.388 Version/12.16*/
const Device UA2015524 = {S14777, 64, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.289 Version/12.02*/
const Device UA2040351 = {S14778, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.10.289 Version/12.02*/
const Device UA2040418 = {S14779, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.289 Version/12.02*/
const Device UA2040969 = {S14780, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.289 Version/12.02*/
const Device UA2040978 = {S14781, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.289 Version/12.02*/
const Device UA2041073 = {S14782, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.289 Version/12.02*/
const Device UA2041091 = {S14783, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5328); ru) Presto/2.10.289 Version/12.02*/
const Device UA2041207 = {S14784, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.10.289 Version/12.02*/
const Device UA2041234 = {S14785, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.10.289 Version/12.02*/
const Device UA2041456 = {S14786, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.10.289 Version/12.02*/
const Device UA2041492 = {S14787, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.289 Version/12.02*/
const Device UA2041796 = {S14788, 101, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.289 Version/12.02,gzip(gfe) (via translate.google.com)*/
const Device UA2041917 = {S14789, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.289 Version/12.02*/
const Device UA2041925 = {S14790, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.289 Version/12.02*/
const Device UA2041926 = {S14791, 71, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; WOW64; U; en) Presto/2.10.289 Version/12.02*/
const Device UA2041930 = {S14792, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.289 Version/12.02*/
const Device UA2042154 = {S14793, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.289 Version/12.02*/
const Device UA2042273 = {S14794, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.289 Version/12.02*/
const Device UA2042631 = {S14795, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.289 Version/12.02*/
const Device UA2042708 = {S14796, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.289 Version/12.02*/
const Device UA2042776 = {S14797, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.289 Version/12.02*/
const Device UA2181543 = {S14798, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5966); ru) Presto/2.10.289 Version/12.02*/
const Device UA2184015 = {S14799, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.10.289 Version/12.02*/
const Device UA2185062 = {S14800, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.289 Version/12.02*/
const Device UA2185979 = {S14801, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5218); ru) Presto/2.10.289 Version/12.02*/
const Device UA2185980 = {S14802, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.289 Version/12.02*/
const Device UA2186931 = {S14803, 72, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; en; rv:2.0) Gecko/X Firefox/4.0 Opera 12.02*/
const Device UA2187960 = {S14804, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.289 Version/12.02*/
const Device UA2188622 = {S14805, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.10.289 Version/12.02*/
const Device UA2189018 = {S14806, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.289 Version/12.02*/
const Device UA2189571 = {S14807, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.289 Version/12.02*/
const Device UA2189943 = {S14808, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5667); ru) Presto/2.10.289 Version/12.02*/
const Device UA2190366 = {S14809, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.289 Version/12.02*/
const Device UA2190795 = {S14810, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.289 Version/12.02*/
const Device UA2191113 = {S14811, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.289 Version/12.02*/
const Device UA2191303 = {S14812, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.289 Version/12.02*/
const Device UA2193040 = {S14813, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5676); ru) Presto/2.10.289 Version/12.02*/
const Device UA2193197 = {S14814, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.289 Version/12.02*/
const Device UA2193401 = {S14815, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.289 Version/12.02*/
const Device UA2193408 = {S14816, 66, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; en) Opera 12.02*/
const Device UA2194501 = {S14817, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.289 Version/12.02*/
const Device UA2194591 = {S14818, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.289 Version/12.02*/
const Device UA2194707 = {S14819, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.289 Version/12.02*/
const Device UA2194869 = {S14820, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.289 Version/12.02*/
const Device UA2195269 = {S14821, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5230); ru) Presto/2.10.289 Version/12.02*/
const Device UA2195654 = {S14822, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.289 Version/12.02*/
const Device UA2197320 = {S14823, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.289 Version/12.02*/
const Device UA2198382 = {S14824, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.289 Version/12.02*/
const Device UA2199119 = {S14825, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.289 Version/12.02*/
const Device UA2199275 = {S14826, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4930); ru) Presto/2.10.289 Version/12.02*/
const Device UA2199291 = {S14827, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.10.289 Version/12.02*/
const Device UA2199384 = {S14828, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.289 Version/12.02*/
const Device UA2200598 = {S14829, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5217); ru) Presto/2.10.289 Version/12.02*/
const Device UA2200760 = {S14830, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5278); ru) Presto/2.10.289 Version/12.02*/
const Device UA2200761 = {S14831, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5660); ru) Presto/2.10.289 Version/12.02*/
const Device UA2202512 = {S14832, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5288); ru) Presto/2.10.289 Version/12.02*/
const Device UA2202527 = {S14833, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.289 Version/12.02*/
const Device UA2202839 = {S14834, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.289 Version/12.02*/
const Device UA2205529 = {S14835, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.10.289 Version/12.02*/
const Device UA2225393 = {S14836, 102, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; compatible; MSIE 7.0; Windows NT 6.0; bg) Presto/2.10.289 Version/12.02*/
const Device UA2225595 = {S14837, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.289 Version/12.02*/
const Device UA2225594 = {S14838, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02746); ru) Presto/2.10.289 Version/12.02*/
const Device UA2226819 = {S14839, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.10.289 Version/12.02*/
const Device UA2226922 = {S14840, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5650); ru) Presto/2.10.289 Version/12.02*/
const Device UA2228095 = {S14841, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5299); ru) Presto/2.10.289 Version/12.02*/
const Device UA2228333 = {S14842, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5325); ru) Presto/2.10.289 Version/12.02*/
const Device UA2228837 = {S14843, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4661); ru) Presto/2.10.289 Version/12.02*/
const Device UA2228903 = {S14844, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5300); ru) Presto/2.10.289 Version/12.02*/
const Device UA2228900 = {S14845, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.10.289 Version/12.02*/
const Device UA2229002 = {S14846, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02552); ru) Presto/2.10.289 Version/12.02*/
const Device UA2230213 = {S14847, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03672); ru) Presto/2.10.289 Version/12.02*/
const Device UA2230482 = {S14848, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5336); ru) Presto/2.10.289 Version/12.02*/
const Device UA2230779 = {S14849, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.10.289 Version/12.02*/
const Device UA2230837 = {S14850, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.289 Version/12.02*/
const Device UA2233979 = {S14851, 74, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.289 Version/12.02,gzip(gfe)*/
const Device UA2235618 = {S14852, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5745); ru) Presto/2.10.289 Version/12.02*/
const Device UA2262414 = {S14853, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02560); ru) Presto/2.10.289 Version/12.02*/
const Device UA2262415 = {S14854, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03638); ru) Presto/2.10.289 Version/12.02*/
const Device UA2262435 = {S14855, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.289 Version/12.02*/
const Device UA2262436 = {S14856, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.289 Version/12.02*/
const Device UA2262451 = {S14857, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02564); ru) Presto/2.10.289 Version/12.02*/
const Device UA2262479 = {S14858, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5734); ru) Presto/2.10.289 Version/12.02*/
const Device UA2262480 = {S14859, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5784); ru) Presto/2.10.289 Version/12.02*/
const Device UA2262505 = {S14860, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5997); ru) Presto/2.10.289 Version/12.02*/
const Device UA2262516 = {S14861, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5993); ru) Presto/2.10.289 Version/12.02*/
const Device UA2345160 = {S14862, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02652); ru) Presto/2.10.289 Version/12.02*/
const Device UA2369755 = {S14863, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.289 Version/12.02*/
const Device UA2398389 = {S14864, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5161); ru) Presto/2.10.289 Version/12.02*/
const Device UA2404060 = {S14865, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5861); ru) Presto/2.10.289 Version/12.02*/
const Device UA2404525 = {S14866, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5880); ru) Presto/2.10.289 Version/12.02*/
const Device UA2424616 = {S14867, 65, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/12.80 (Windows NT 5.1; U; en) Presto/2.10.289 Version/12.02*/
const Device UA2427568 = {S14868, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6011); ru) Presto/2.10.289 Version/12.02*/
const Device UA2436541 = {S14869, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.289 Version/12.02*/
const Device UA2449712 = {S14870, 114, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03399); MRSPUTNIK OW 2, 3, 0, 104; ru) Presto/2.10.289 Version/12.02*/
const Device UA2449920 = {S14871, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5309); ru) Presto/2.10.289 Version/12.02*/
const Device UA2464777 = {S14872, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03649); ru) Presto/2.10.289 Version/12.02*/
const Device UA2471188 = {S14873, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5981); ru) Presto/2.10.289 Version/12.02*/
const Device UA2483559 = {S14874, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5977); ru) Presto/2.10.289 Version/12.02*/
const Device UA2536033 = {S14875, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6037); ru) Presto/2.10.289 Version/12.02*/
const Device UA2558944 = {S14876, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5989); ru) Presto/2.10.289 Version/12.02*/
const Device UA2559393 = {S14877, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6048); ru) Presto/2.10.289 Version/12.02*/
const Device UA2560814 = {S14878, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6054); ru) Presto/2.10.289 Version/12.02*/
const Device UA2561301 = {S14879, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.289 Version/12.02*/
const Device UA2571283 = {S14880, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.289 Version/12.02*/
const Device UA2581871 = {S14881, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5999); ru) Presto/2.10.289 Version/12.02*/
const Device UA2591774 = {S14882, 114, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03385); MRSPUTNIK OW 2, 3, 0, 104; ru) Presto/2.10.289 Version/12.02*/
const Device UA2603743 = {S14883, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 6007); ru) Presto/2.10.289 Version/12.02*/
const Device UA2665182 = {S14884, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6046); ru) Presto/2.10.289 Version/12.02*/
const Device UA2666231 = {S14885, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 6003); ru) Presto/2.10.289 Version/12.02*/
const Device UA2668835 = {S14886, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6081); ru) Presto/2.10.289 Version/12.02*/
const Device UA2693458 = {S14887, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5730); ru) Presto/2.10.289 Version/12.02*/
const Device UA2707546 = {S14888, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 6008); ru) Presto/2.10.289 Version/12.02*/
const Device UA2709249 = {S14889, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6083); ru) Presto/2.10.289 Version/12.02*/
const Device UA2712609 = {S14890, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6089); ru) Presto/2.10.289 Version/12.02*/
const Device UA2748818 = {S14891, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6042); ru) Presto/2.10.289 Version/12.02*/
const Device UA2774097 = {S14892, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.289 Version/12.02*/
const Device UA2777089 = {S14893, 103, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; en) Presto/2.10.289 Version/12.02,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2822715 = {S14894, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02830); ru) Presto/2.10.289 Version/12.02*/
const Device UA2825659 = {S14895, 87, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02620); ru) Presto/2.10.289 Version/12.02*/
const Device UA2834798 = {S14896, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 3925); ru) Presto/2.10.289 Version/12.02*/
const Device UA2956300 = {S14897, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 6017); ru) Presto/2.10.289 Version/12.02*/
const Device UA2994535 = {S14898, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 6019); ru) Presto/2.10.289 Version/12.02*/
const Device UA3034055 = {S14899, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6639); ru) Presto/2.10.289 Version/12.02*/
const Device UA3061093 = {S14900, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6683); ru) Presto/2.10.289 Version/12.02*/
const Device UA3078056 = {S14901, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6673); ru) Presto/2.10.289 Version/12.02*/
const Device UA3119113 = {S14902, 84, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.289 Version/12.02,gzip(gfe),gzip(gfe)*/
const Device UA3125064 = {S14903, 86, { &P15364,&P5690,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.1 (build 6341); ru) Presto/2.10.289 Version/12.02*/
const Device UA23819 = {S14904, 142, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38135 = {S14905, 140, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38136 = {S14906, 172, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android >>>>>CRISKELO-ROM Ver 54<<<<< 2.3.3; es-es; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40852 = {S14907, 143, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-I9000M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51749 = {S14908, 156, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android XXJVO 2.3.3 Powered; it-it; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51761 = {S14909, 154, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android GINGERBREAD 2.3.3; vi-vn; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51767 = {S14910, 172, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android >>>>>CRISKELO-ROM Ver 56<<<<< 2.3.3; fr-fr; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51769 = {S14911, 160, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9000-ORANGE/I9000BVJV7 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51780 = {S14912, 172, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android >>>>>CRISKELO-ROM Ver 59<<<<< 2.3.3; es-es; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51782 = {S14913, 172, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android >>>>>CRISKELO-ROM Ver 51<<<<< 2.3.3; es-es; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51751 = {S14914, 160, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; GT-I9000-ORANGE/I9000BVJV8 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA53166 = {S14915, 143, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; GT-I9000B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55667 = {S14916, 156, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Speed N Looks 2.3.3; en-us; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA69626 = {S14917, 144, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9000 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA180924 = {S14918, 142, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA182536 = {S14919, 66, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-I9000 Build/GINGERBREAD)*/
const Device UA204347 = {S14920, 142, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA207222 = {S14921, 66, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-I9000 Build/GINGERBREAD)*/
const Device UA227934 = {S14922, 141, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9000 Build/GalnetMIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA240438 = {S14923, 142, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA249712 = {S14924, 136, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-I9000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA300166 = {S14925, 136, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA339372 = {S14926, 66, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-I9000 Build/GINGERBREAD)*/
const Device UA352201 = {S14927, 142, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA355801 = {S14928, 66, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9000 Build/GINGERBREAD)*/
const Device UA436815 = {S14929, 67, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-I9000M Build/GINGERBREAD)*/
const Device UA624318 = {S14930, 160, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9000-ORANGE/I9000BVJVC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA660474 = {S14931, 138, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; iw-il; GT-I9000 Build/HebMIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA724581 = {S14932, 136, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA735929 = {S14933, 145, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6JW1; nl-nl; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1895863 = {S14934, 67, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-I9000B Build/GINGERBREAD)*/
const Device UA1908066 = {S14935, 66, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; GT-I9000 Build/GINGERBREAD)*/
const Device UA1931383 = {S14936, 130, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9000 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1940765 = {S14937, 130, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9000 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1971046 = {S14938, 145, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6JW4; nl-nl; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1992063 = {S14939, 237, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3;en-US;GT-I9000 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 AppEngine-Google; (+http://code.google.com/appengine; appid: s~google.com:mobile-checker)*/
const Device UA2018243 = {S14940, 134, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018781 = {S14941, 172, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android >>>>>CRISKELO-ROM Ver 69<<<<< 2.3.4; es-es; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019622 = {S14942, 135, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; GT-I9000 Build/MYUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019638 = {S14943, 130, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; fa-fa; GT-I9000 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2020448 = {S14944, 130, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9000 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2020893 = {S14945, 138, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; iw-il; GT-I9000 Build/HEBMIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022666 = {S14946, 142, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT-I9000 Build/Conspicuous) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023121 = {S14947, 144, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9000 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2023564 = {S14948, 135, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9000 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2023628 = {S14949, 148, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6 XXJVU; it-it; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025067 = {S14950, 144, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9000 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2025466 = {S14951, 135, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9000 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2027089 = {S14952, 143, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-I9000B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027729 = {S14953, 148, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6 XXJW4; it-it; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034055 = {S14954, 145, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6JVU; nl-nl; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257117 = {S14955, 140, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257130 = {S14956, 161, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-I9000/I9000BOJW6 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2542519 = {S14957, 140, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2545725 = {S14958, 135, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9000 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2545909 = {S14959, 135, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9000 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2734471 = {S14960, 160, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-I9000-ORANGE/I9000BVJVD Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2787230 = {S14961, 141, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-I9000M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2979872 = {S14962, 136, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042234 = {S14963, 142, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-cn; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3084693 = {S14964, 140, { &P14877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2223444 = {S14965, 194, { &P18436,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3)*/
const Device UA2335587 = {S14966, 205, { &P18436,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Tablet PC 2.0; Media Center PC 6.0; SLCC2)*/
const Device UA2340147 = {S14967, 193, { &P18436,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Media Center PC 6.0)*/
const Device UA2368178 = {S14968, 180, { &P18436,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; ASU2JS)*/
const Device UA2371994 = {S14969, 199, { &P18436,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Tablet PC 2.0; Zune 4.7; InfoPath.3)*/
const Device UA2386723 = {S14970, 165, { &P18436,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; ASU2JS)*/
const Device UA2387000 = {S14971, 185, { &P18436,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Tablet PC 2.0; ASU2JS)*/
const Device UA2403727 = {S14972, 173, { &P18436,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; MASMJS)*/
const Device UA2423450 = {S14973, 170, { &P18436,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; ASU2JS)*/
const Device UA2445615 = {S14974, 215, { &P18436,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Tablet PC 2.0; Media Center PC 6.0; SLCC2; Zune 4.7)*/
const Device UA2520391 = {S14975, 175, { &P18436,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; Touch; .NET4.0E; .NET4.0C; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7)*/
const Device UA2642529 = {S14976, 193, { &P18436,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Media Center PC 6.0; SLCC2)*/
const Device UA2813592 = {S14977, 203, { &P18436,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; Tablet PC 2.0; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7)*/
const Device UA2461903 = {S14978, 101, { &P15364,&P17285,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17*/
const Device UA2470417 = {S14979, 104, { &P15364,&P17285,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.1.0.0 Safari/537.17*/
const Device UA2471958 = {S14980, 97, { &P15364,&P17285,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.1.0.0 Safari/537.17*/
const Device UA2492179 = {S14981, 108, { &P15364,&P17285,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492501 = {S14982, 104, { &P15364,&P17285,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.2.0.0 Safari/537.17*/
const Device UA2492507 = {S14983, 97, { &P15364,&P17285,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.2.0.0 Safari/537.17*/
const Device UA2492609 = {S14984, 111, { &P15364,&P17285,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)*/
const Device UA2509076 = {S14985, 140, { &P15364,&P17285,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2510500 = {S14986, 118, { &P15364,&P17285,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)*/
const Device UA2535881 = {S14987, 147, { &P15364,&P17285,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2729815 = {S14988, 100, { &P15364,&P17285,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.17 (KHTML like Gecko) Chrome/24.0.1312.52 Safari/537.17*/
const Device UA91043 = {S14989, 106, { &P15364,&P7133,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA91052 = {S14990, 99, { &P15364,&P7133,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA252197 = {S14991, 145, { &P15364,&P7133,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.106 Safari/535.2,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA252342 = {S14992, 143, { &P15364,&P7133,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.106 Safari/535.2,gzip(gfe) (via translate.google.com)*/
const Device UA255824 = {S14993, 136, { &P15364,&P7133,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2,gzip(gfe) (via translate.google.com)*/
const Device UA255965 = {S14994, 137, { &P15364,&P7133,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/534.7 ZemanaAID/FFFF0061*/
const Device UA919567 = {S14995, 138, { &P15364,&P7133,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2362117 = {S14996, 109, { &P15364,&P7133,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2,gzip(gfe)*/
const Device UA2466367 = {S14997, 116, { &P15364,&P7133,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2,gzip(gfe)*/
const Device UA358778 = {S14998, 88, { &P18095,&P18109,&P17373,&P18092 }};	/*Useragent - Opera/9.80 (Series 60; Opera Mini/6.5.27309/26.1305; U; id) Presto/2.8.119 Version/10.54*/
const Device UA2431226 = {S14999, 88, { &P18095,&P18109,&P17373,&P18092 }};	/*Useragent - Opera/9.80 (Series 60; Opera Mini/6.5.27354/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA18614 = {S15000, 150, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC/DesireS/1.32.161.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18619 = {S15001, 150, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC/DesireS/1.32.161.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18620 = {S15002, 149, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_DesireS-orange-LS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA22345 = {S15003, 140, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA22344 = {S15004, 139, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_DesireS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA23179 = {S15005, 145, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_DesireS_S510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32779 = {S15006, 143, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - HTC_DesireS_S510e_Mozilla/5.0(Linux; U; Android 2.3.3;it-it; Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41950 = {S15007, 150, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-ch; HTC/DesireS/1.47.166.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42005 = {S15008, 145, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_DesireS_S510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42015 = {S15009, 143, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sl-; HTC_DesireS_S510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57807 = {S15010, 145, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC_DesireS_S510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57815 = {S15011, 150, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; HTC/DesireS/1.47.163.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57832 = {S15012, 150, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC/DesireS/1.47.161.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57856 = {S15013, 150, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; HTC/DesireS/1.32.163.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57866 = {S15014, 150, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-ch; HTC/DesireS/1.32.166.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59374 = {S15015, 145, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; th-th; HTC_DesireS_S510b Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA68822 = {S15016, 149, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; xx-xx; HTC/DesireS/1.07.163.1 Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA69930 = {S15017, 140, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70064 = {S15018, 140, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Desire S Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71061 = {S15019, 146, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC_Desire_S_S510e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71931 = {S15020, 146, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_Desire_S_S510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72627 = {S15021, 149, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_DesireS_S510e/1.0 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76379 = {S15022, 146, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC_DesireS_S510e Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA77054 = {S15023, 150, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; pt-pt; HTC/DesireS/1.32.168.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA190356 = {S15024, 64, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Desire S Build/GRI40)*/
const Device UA252248 = {S15025, 150, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/DesireS/2.10.161.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA367693 = {S15026, 139, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_DesireS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370895 = {S15027, 172, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh_tw; HTC Desire S Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)*/
const Device UA379928 = {S15028, 150, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-ch; HTC/DesireS/2.10.166.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA396176 = {S15029, 64, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Desire S Build/GRJ90)*/
const Device UA623039 = {S15030, 149, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_DesireS-orange-LS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA624362 = {S15031, 150, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-at; HTC/DesireS/2.10.163.5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA632768 = {S15032, 140, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA636793 = {S15033, 145, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-th; HTC_DesireS_S510b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA660240 = {S15034, 128, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire S Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA684966 = {S15035, 150, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/DesireS/1.47.161.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA686938 = {S15036, 128, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Desire S Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA698200 = {S15037, 146, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_Desire_S_S510e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA776276 = {S15038, 145, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC_DesireS_S510e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA860571 = {S15039, 142, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_Desire_SMS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA895468 = {S15040, 227, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTCS510e/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA908546 = {S15041, 133, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Desire S Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1935521 = {S15042, 150, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-pt; HTC/DesireS/2.10.168.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1986519 = {S15043, 133, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire S Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2021768 = {S15044, 143, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-; HTC_DesireS_S510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022784 = {S15045, 64, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Desire S Build/GRJ90)*/
const Device UA2023833 = {S15046, 140, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2215830 = {S15047, 227, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-hk; HTCS510e/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2320106 = {S15048, 132, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_DesireS_S510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile*/
const Device UA2362945 = {S15049, 150, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/DesireS/2.10.161.5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2445865 = {S15050, 133, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Desire S Build GRI40) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2494594 = {S15051, 133, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire S Build GRJ90) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2742510 = {S15052, 138, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3099737 = {S15053, 145, { &P14969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC_DesireS_S510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA359561 = {S15054, 37, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android 4.0.3)*/
const Device UA361776 = {S15055, 137, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3.0.2.0.1.0; en-us; Build/OPENMASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA366046 = {S15056, 129, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/IML75) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA397867 = {S15057, 129, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA422336 = {S15058, 122, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA437281 = {S15059, 123, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ITL41F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA441511 = {S15060, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA454097 = {S15061, 123, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/ITL41F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA454580 = {S15062, 122, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA457544 = {S15063, 138, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4.0.4.0.4; en-us; Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA583601 = {S15064, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA605743 = {S15065, 132, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; CHM805HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA607331 = {S15066, 140, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; STM712HCZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA607397 = {S15067, 128, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/4.03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA612771 = {S15068, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/OPENMASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616197 = {S15069, 123, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA616731 = {S15070, 37, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android 4.0.4)*/
const Device UA616730 = {S15071, 136, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ICS.f04ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA622363 = {S15072, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/IMM76K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30*/
const Device UA636202 = {S15073, 140, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3.0.2.0.1.0; en-us; Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA636739 = {S15074, 56, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; MID Build/IML74K)*/
const Device UA639418 = {S15075, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/IMM76K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA668157 = {S15076, 127, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA674640 = {S15077, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/IMM30B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA678639 = {S15078, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; STM805HCZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA684408 = {S15079, 144, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3.0.2.0.1.0; en-us; Build/OPENMASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA710537 = {S15080, 37, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android 4.0.2)*/
const Device UA725372 = {S15081, 127, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; 97D Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA729968 = {S15082, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; NAM805HCX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA731233 = {S15083, 37, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android 4.0.1)*/
const Device UA740625 = {S15084, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; CRYM1005HN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA749213 = {S15085, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/OPENMASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA767208 = {S15086, 156, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; e1108_v73_jbaol1g18flash8p4 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA796534 = {S15087, 135, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; TEACM1005HN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA799386 = {S15088, 128, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T-90 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA834487 = {S15089, 140, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3.0.2.0.1.0; en-us; Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA850320 = {S15090, 129, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; eeepc Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA851242 = {S15091, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA855144 = {S15092, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA888796 = {S15093, 127, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; pad Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA895005 = {S15094, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA896145 = {S15095, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; ST8001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA899214 = {S15096, 145, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_jbl1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899376 = {S15097, 138, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4.0.4.0.4; en-us; Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899413 = {S15098, 156, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Android SDK built for x86 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA900267 = {S15099, 145, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_hjy1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA900401 = {S15100, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ICS.f06ref848L.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA905564 = {S15101, 120, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/Xv2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA909834 = {S15102, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA910204 = {S15103, 148, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Cat Tablet Android 4.0.4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA910630 = {S15104, 128, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; NEXT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA911716 = {S15105, 138, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4.0.4.0.4; en-us; Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1948372 = {S15106, 51, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3 Build/IML74K)*/
const Device UA1952777 = {S15107, 132, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; PEM727HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1954526 = {S15108, 141, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; CHARM712HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1954811 = {S15109, 139, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PTM712HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1957654 = {S15110, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; DATAM805HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1959339 = {S15111, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; ARGM816HCX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1961975 = {S15112, 127, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1962483 = {S15113, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ITL41F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1965936 = {S15114, 132, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; ADM816HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1966676 = {S15115, 141, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; ARGM727HCX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1971377 = {S15116, 136, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; ARGM816HCX_A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1980618 = {S15117, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; CRYM805HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1981532 = {S15118, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ICS.g12refM1005.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1982568 = {S15119, 148, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; e1108_v73_jbaol1g18 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1989901 = {S15120, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; TR720F Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1990084 = {S15121, 141, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GPLUS S101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1993781 = {S15122, 141, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GOCLM712HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1994992 = {S15123, 137, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VegaCream RC2 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1995063 = {S15124, 129, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; eeepc Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1995594 = {S15125, 148, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; e1808_v75_hjy1a6x Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2018343 = {S15126, 139, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-us; generic Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2024294 = {S15127, 123, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16;*/
const Device UA2024482 = {S15128, 57, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3 Build/ICS.f04ref.X)*/
const Device UA2025430 = {S15129, 123, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2025854 = {S15130, 123, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2025912 = {S15131, 67, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; e1808_v75_jbl1 Build/IML74K)*/
const Device UA2026398 = {S15132, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/XGS2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027522 = {S15133, 145, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_jbl2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029075 = {S15134, 128, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; 720F Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2031011 = {S15135, 150, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_jbl1_5640 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031904 = {S15136, 140, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC_GOF_U Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032374 = {S15137, 127, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/OPENMASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033089 = {S15138, 131, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-1230 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033094 = {S15139, 142, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; %ModelName% Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033311 = {S15140, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LUBM816HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033433 = {S15141, 127, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; PAD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033633 = {S15142, 51, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3 Build/ITL41F)*/
const Device UA2034017 = {S15143, 147, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_jbaol1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034028 = {S15144, 129, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ICS.f11ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2034138 = {S15145, 140, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; EFM727HCF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034395 = {S15146, 120, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2175109 = {S15147, 135, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; DLHD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2176353 = {S15148, 150, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_hjy1_5647 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2186207 = {S15149, 145, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1808_v75_jbl2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2186510 = {S15150, 150, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1808_v75_jbl1_5640 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2203092 = {S15151, 129, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; PC936 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2203564 = {S15152, 128, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.2 Version/4.0 Mobile Safari/530.17*/
const Device UA2204931 = {S15153, 129, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; V1013 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2225413 = {S15154, 137, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; NEOBOX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2225868 = {S15155, 153, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; e1808_v75_hjy1a6x_5647 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2227170 = {S15156, 138, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TWD_MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2227739 = {S15157, 128, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.2 Version/4.0 Mobile Safari/530.17*/
const Device UA2238272 = {S15158, 140, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; HTC_POO_U Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2239227 = {S15159, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2242305 = {S15160, 56, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MID Build/IMM76D)*/
const Device UA2257220 = {S15161, 135, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.0; it-it; Android Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257258 = {S15162, 149, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_jbl1a535 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257291 = {S15163, 151, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_hjy1a6x_in Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257623 = {S15164, 129, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; P079R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2335517 = {S15165, 120, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2338912 = {S15166, 120, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn Build/S-2-20) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2342961 = {S15167, 145, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1911_v77_hjy1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2350374 = {S15168, 140, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Android T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2351700 = {S15169, 129, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/GB.AMLOGIC.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2353155 = {S15170, 136, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SPX_8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2354601 = {S15171, 137, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Tender Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2355270 = {S15172, 139, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EBM727KC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2356638 = {S15173, 158, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Telechips TCC8800 Evaluation Board Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2360203 = {S15174, 140, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Wxt Tab PC 65CXi Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2360365 = {S15175, 137, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Oysters T8 A4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2360444 = {S15176, 145, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VB100a Pro Build/ICS.g08refem611.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2360659 = {S15177, 136, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ICS.g04ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2360842 = {S15178, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MID72C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2361392 = {S15179, 132, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; PTM723MR Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2361986 = {S15180, 150, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - N1 Linux/3.0.13 Android/4.0.4 Release/08.15.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2362654 = {S15181, 159, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-gb; e1909c_v77_gq1008_9p017_6228 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2363138 = {S15182, 135, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; mi-book Mi1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2363892 = {S15183, 131, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID970T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2364440 = {S15184, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2365093 = {S15185, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Junior 4.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2365823 = {S15186, 135, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2365956 = {S15187, 141, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; CW V80 Build/ICS.g08refem618.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2366047 = {S15188, 137, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JY131C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2366192 = {S15189, 139, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Alpha GT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2366353 = {S15190, 137, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TRIO7C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2366371 = {S15191, 144, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Mercury_magiQ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2366635 = {S15192, 126, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; G1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2366691 = {S15193, 154, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HanZpad Build/master-ics-android-4.0.4_r1.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2366842 = {S15194, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i7S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2367071 = {S15195, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TM-MID732 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2367255 = {S15196, 107, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2367783 = {S15197, 131, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PLT1044 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2367999 = {S15198, 127, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; TB4-4/5580 Build/ITL41D) AppleWebKit/434.50 (KHTML, like Gecko) Version/4.0 Mobile*/
const Device UA2369540 = {S15199, 129, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; PC678 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2371628 = {S15200, 46, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3 Build/X)*/
const Device UA2372397 = {S15201, 128, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; 97FC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2372818 = {S15202, 136, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; MIDC907PR001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2373059 = {S15203, 136, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; IGUANAPAD M7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2373336 = {S15204, 129, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SANEI Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2373346 = {S15205, 129, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; M971B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2373535 = {S15206, 136, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TZ100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2374738 = {S15207, 127, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; M19 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2375150 = {S15208, 127, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; P90 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2375157 = {S15209, 148, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; samsung galaxy s2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2375409 = {S15210, 140, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; HFM752HCF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2375430 = {S15211, 128, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M746 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2375480 = {S15212, 129, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; TR976 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2375694 = {S15213, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2375744 = {S15214, 144, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; SXZ-PDX0-01 Build/SXZ-PDX0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2375917 = {S15215, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ICS.f06ref1024.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2375954 = {S15216, 129, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; MID08 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2375971 = {S15217, 136, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D701A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2376157 = {S15218, 132, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ATOM PAD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2376193 = {S15219, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; zh-cn; 9151 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2376203 = {S15220, 138, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; RC0710B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2376677 = {S15221, 129, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; M701F Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2376691 = {S15222, 147, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; INFINITY_PASSION Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2377039 = {S15223, 140, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; Europe_4G-4s Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2377227 = {S15224, 135, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LA-I Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2378160 = {S15225, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E140K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2378211 = {S15226, 209, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - MOT-XT685/1.0 Android/4.0.4 Release/9.15.2011 Browser/AppleWebKit534.30  Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT685 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2379517 = {S15227, 143, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Explay Informer 701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2379737 = {S15228, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; PC8005 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2380262 = {S15229, 131, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT7220S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2381023 = {S15230, 129, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; 18622 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2381724 = {S15231, 138, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ar-eg; ACCM700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2381970 = {S15232, 140, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; el-gr; IFQM712HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2382030 = {S15233, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; BOULM805ND Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2382121 = {S15234, 136, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M758A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2382320 = {S15235, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ASTRO+9.7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2382581 = {S15236, 131, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GEM9212 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2382997 = {S15237, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; EVOPAD R7010 Build/2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2383048 = {S15238, 132, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; YAN 7.1C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2383096 = {S15239, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; EXLP2R Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2383323 = {S15240, 128, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DOPO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2383594 = {S15241, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PTM712MCF Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2383632 = {S15242, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GX8404 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2383697 = {S15243, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MW0922 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2383731 = {S15244, 153, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; California Access TAB MM 1001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2384030 = {S15245, 142, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PICOpad_GEA Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2384431 = {S15246, 140, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ACGM727HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2384812 = {S15247, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; VORTEX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2384888 = {S15248, 132, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; KidsPad1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2384938 = {S15249, 129, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; FAR7 Build/V02R10A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2384987 = {S15250, 136, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Mi501 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2385107 = {S15251, 154, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; e1911_v77_gq2000_ov5647 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2385124 = {S15252, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JW101A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2385232 = {S15253, 136, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; ALEXIS-RX5DC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2385272 = {S15254, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; POM805HCF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2385320 = {S15255, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GASEM712MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2385514 = {S15256, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; WALMM752MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2386239 = {S15257, 145, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; INFINITY_POWER Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2386387 = {S15258, 141, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; HTC_H1000C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2386568 = {S15259, 136, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M4302 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2387152 = {S15260, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; UTAB71 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2387490 = {S15261, 146, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Tablet-PC-4 Build/ICS.g08refem618.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2387827 = {S15262, 128, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2388817 = {S15263, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MW0710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2390965 = {S15264, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HD7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2391287 = {S15265, 131, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; CTC07SO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2393367 = {S15266, 131, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; unknown Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2393868 = {S15267, 127, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; E71 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2394886 = {S15268, 131, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; TAB7012 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2395519 = {S15269, 128, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T03A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2395560 = {S15270, 128, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; M805 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2395726 = {S15271, 129, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; P-08D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2395735 = {S15272, 132, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; T9200 Build/HuaweiT9200) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2395842 = {S15273, 132, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; G15 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2396053 = {S15274, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; amd_brazos Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2396258 = {S15275, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; GASEM805HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2396340 = {S15276, 131, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; TAB9 3G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2396458 = {S15277, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; tablet_pc Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2397052 = {S15278, 135, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; PNDPP4MT9G2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2397279 = {S15279, 132, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; MID900SC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2397357 = {S15280, 138, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N101 DUAL CORE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2397362 = {S15281, 128, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; PC728 Build/PC728) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2397718 = {S15282, 128, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; MID14 Build/MID14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2397857 = {S15283, 128, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; 7018 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2397997 = {S15284, 152, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Zyrex_ZA987_OneScribe Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2398348 = {S15285, 131, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; AMOI N820 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2398612 = {S15286, 143, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/Android4.0.3 Release/08.22.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2398752 = {S15287, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; PA912B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2399058 = {S15288, 121, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - HUAWEI_T9200_TD/1.0 Android/4.0.3 (Linux; U; Android 4.0.3; zh-cn) Release/05.31.2012 Browser/WAP2.0 (AppleWebKit/534.30)*/
const Device UA2399510 = {S15289, 137, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CJ ThL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2399774 = {S15290, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; AN8BG3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2399844 = {S15291, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; XPRESS PRO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2400241 = {S15292, 136, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; EVOPad R8000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2400280 = {S15293, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; gs701b Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2400316 = {S15294, 129, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; XELIO Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2400317 = {S15295, 128, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; 101T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2400641 = {S15296, 139, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; sl-si; XP717 Build/ICS.f08refem32.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2400679 = {S15297, 135, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TAB7004GBBK Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2400720 = {S15298, 128, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I97D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2400750 = {S15299, 142, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - HUAWEI_T9200_TD/1.0 Android/4.0.3 (Linux; U; Android 4.0.3; zh-cn) Release/05.31.2012 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30*/
const Device UA2400922 = {S15300, 154, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-gb; e1909c_v77_gq1008_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2400938 = {S15301, 131, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID-001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2400969 = {S15302, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; TB512-16G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2401022 = {S15303, 135, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2401026 = {S15304, 138, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; kiwi k5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2401030 = {S15305, 144, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; NB017 / NB018 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2401016 = {S15306, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M1005D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2401043 = {S15307, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PX1345 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2401426 = {S15308, 135, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Build/ICS.Zoostorm_130.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2401433 = {S15309, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Q29 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2402190 = {S15310, 136, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N9977 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2402262 = {S15311, 139, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MHU001D Build/MHU001D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2402371 = {S15312, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; CTA Build/ICS.f32ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2402559 = {S15313, 150, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ro-ro; SUPREME IPS Dual Core X200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2403397 = {S15314, 138, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Swipe AllinOne Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2403951 = {S15315, 132, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Build/ICS.MID-701SC.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2404050 = {S15316, 136, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N10-4core Build/R13.4-rc2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2404708 = {S15317, 140, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Eclipse-X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2404866 = {S15318, 135, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; J-Q8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2404977 = {S15319, 143, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; SKYTEX-SP438 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2405227 = {S15320, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Pad Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2405928 = {S15321, 131, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; Velocity-V1 Build/T860) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2406676 = {S15322, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MANHATTAN Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2411269 = {S15323, 40, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Android 4.0.4; Application: com.e8tracks*/
const Device UA2411943 = {S15324, 131, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Android Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2413252 = {S15325, 152, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; MID ro.product.brand=Android Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2414021 = {S15326, 138, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; unknown Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2416027 = {S15327, 145, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1908_v77_jbl1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2417106 = {S15328, 138, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; Build/ICS.w17ref.Dualcore.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2420403 = {S15329, 150, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; e1911_v77_jbaol_g20 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2420681 = {S15330, 153, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-gb; e1909c_v77_gq1008_5647 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422376 = {S15331, 128, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; Build/ICS.Lazer.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2423861 = {S15332, 118, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-nz; Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2424074 = {S15333, 127, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; CTM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2426214 = {S15334, 143, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; yuanda75_ics Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2428363 = {S15335, 131, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; CT710FR Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2433439 = {S15336, 140, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; 116DN Build/ICS.g08refem611.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2437183 = {S15337, 132, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; POM727MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2437249 = {S15338, 150, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1808_v75_jbl2_6628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2437589 = {S15339, 141, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NUCLEAR Build/GT704-A13005-A04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2444913 = {S15340, 116, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-; Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2455593 = {S15341, 149, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1908_v77_jbl1a535 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2457261 = {S15342, 131, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; My Play Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2457440 = {S15343, 158, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1911_v77_mt6628_hjy1_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2458182 = {S15344, 158, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_hjy1_5640_maxwest Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2463608 = {S15345, 37, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - stagefright/1.1 (Linux;Android 4.0.3)*/
const Device UA2463814 = {S15346, 37, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - stagefright/1.1 (Linux;Android 4.0.4)*/
const Device UA2468387 = {S15347, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9999_custom Build/RMU) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2469312 = {S15348, 138, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IDOIPAD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2476316 = {S15349, 115, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn Build/S-2-20) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2480723 = {S15350, 132, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ICS.f03ref_NX.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2483665 = {S15351, 136, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; GLORY Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2484530 = {S15352, 144, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; hexing77_ics2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2485860 = {S15353, 135, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S150 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2494675 = {S15354, 135, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.0; en-us; Android Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2499705 = {S15355, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/ICS.f06ref848L.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2500853 = {S15356, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; S3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2501913 = {S15357, 60, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; TWD_MID Build/IMM76D)*/
const Device UA2502052 = {S15358, 126, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Build/ICS.VSA.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2502703 = {S15359, 158, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_hjy1_5647_maxwest Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2508170 = {S15360, 145, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1808_v75_jbl2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2522708 = {S15361, 139, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MSM7227A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2523323 = {S15362, 152, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_jbaol1_5647 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2525496 = {S15363, 148, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1908_v77_hjy1a6x Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2527660 = {S15364, 135, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iPhone5 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2527713 = {S15365, 136, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; HD II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528582 = {S15366, 156, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1808_v75_jbl1a535mt9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2552500 = {S15367, 143, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Pioneer E80w Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2554601 = {S15368, 135, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mini-2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2557638 = {S15369, 158, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; e1909c_v77_gq1008_5647_6228 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558597 = {S15370, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; WP7002 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2560590 = {S15371, 130, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; ST8002 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2561384 = {S15372, 124, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; MID Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2563068 = {S15373, 148, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Swipe Walk n Talk Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2565458 = {S15374, 159, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1809l_v75_gq1008_ov5647_v20 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2577728 = {S15375, 127, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 97D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2578014 = {S15376, 140, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; DOUBLE II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2578103 = {S15377, 150, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1808_v75_jbl1_5647 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2578145 = {S15378, 138, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-us; Smartphones Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2589705 = {S15379, 140, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; iPhone 4S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2590847 = {S15380, 142, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Build/Coolpad7728_V02.02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2592250 = {S15381, 129, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Build/ICS.f03ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2596597 = {S15382, 137, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NEOBOX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2598637 = {S15383, 139, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; S815_HZT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2600684 = {S15384, 158, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1808_v75_hjy1_5640_maxwest Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2641018 = {S15385, 131, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fi-fi; Build/ICS.pad10ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2645247 = {S15386, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Build/ICS.g12refM805.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2648682 = {S15387, 136, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.8; en-us; Android Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652167 = {S15388, 72, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; e1808_v75_jbl1_5640 Build/IML74K)*/
const Device UA2652271 = {S15389, 138, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Android Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2652628 = {S15390, 138, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; msm8x25 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2652689 = {S15391, 138, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; unknown Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2653174 = {S15392, 148, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1911_v77_jyuany1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2653954 = {S15393, 151, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1911_v77_hjy1_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654957 = {S15394, 124, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2655505 = {S15395, 152, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1911_v77_mt6628_hjy1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2656243 = {S15396, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; Build/ICS.ATABLET7306.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2656417 = {S15397, 135, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID Serials Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2659269 = {S15398, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ICS.g05_n1asref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2659552 = {S15399, 135, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T730 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659685 = {S15400, 143, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; HTC_PROMIN_U Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2663434 = {S15401, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; CTA Build/ICS.casper.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2666649 = {S15402, 144, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; hexing75_hspa   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2686225 = {S15403, 140, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Build/06.03.001.X.7728) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2691294 = {S15404, 60, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; unknown Build/IMM76D)*/
const Device UA2695073 = {S15405, 135, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Build/ICS.g05_n3aenref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2698377 = {S15406, 167, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1911_v77_jbaol_g20mt9p017fwvga_6628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2699134 = {S15407, 135, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Build/ICS.g05_n1benref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2703353 = {S15408, 134, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; S88 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2706708 = {S15409, 153, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1809l_v75_jbl_253_v20 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2721076 = {S15410, 148, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_nd1antv Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2722880 = {S15411, 158, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us Build/SmartDroid Berserk_V 2.3_DummyPlug_) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2729171 = {S15412, 135, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; ru-ru; iPhone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2734429 = {S15413, 159, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1808_v75_hjy1_9p017_maxwest Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2746499 = {S15414, 144, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; e1908_v77_nd3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2746718 = {S15415, 148, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1908_v77_jbl1ntv Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2764925 = {S15416, 140, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; HIGH QUALITY MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2768690 = {S15417, 139, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; S129 Build/Android4.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2775082 = {S15418, 80, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 4.0.9; e1108_v73_jbaol1g18flash8p4 Build/GRK39F)*/
const Device UA2778057 = {S15419, 137, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iphone 5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2778490 = {S15420, 131, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; TWD_MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2788376 = {S15421, 154, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1808_v75_jbl1_5640a522 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2790177 = {S15422, 136, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Build/HuaweiT9510E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2790773 = {S15423, 161, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1908_v77_mt6628_hjy1a6x_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2810121 = {S15424, 62, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; STM712HCZ Build/IML74K)*/
const Device UA2810503 = {S15425, 48, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3 Build/Xv2)*/
const Device UA2810902 = {S15426, 61, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3 Build/ICS.f06ref848L.X)*/
const Device UA2816971 = {S15427, 48, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - AndroidCvpPlayer(Linux; U; Android 4.0.4; en-us)*/
const Device UA2824259 = {S15428, 67, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; e1808_v75_jbl2 Build/IML74K)*/
const Device UA2827608 = {S15429, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Build/ICS.MBX.ENERGY.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2829209 = {S15430, 146, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Eclipse 4.3 MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2833662 = {S15431, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Build/ICS.g05_n3aref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2838134 = {S15432, 133, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Build/ICS.f06ref1024.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2848439 = {S15433, 137, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; J-Q88D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2849089 = {S15434, 119, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/G6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2850776 = {S15435, 129, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-nz; Build/ICS.OCTAVO.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2857420 = {S15436, 138, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; msm8x25 Build/JIEVIS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2863742 = {S15437, 143, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; hexing75_ics Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2876696 = {S15438, 132, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PHM727MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2878268 = {S15439, 57, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; T730 Build/IMM76D)*/
const Device UA2889532 = {S15440, 154, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; e1808_v75_hjy1a6x_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2889572 = {S15441, 151, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; e1808_v75_hjy1_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934214 = {S15442, 162, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1911_v77_jbaol_g20mt9p017_6628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936002 = {S15443, 142, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; hexing75_hspa   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2950319 = {S15444, 56, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; PAD Build/IMM76D)*/
const Device UA2950718 = {S15445, 57, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; 720F Build/IML74K)*/
const Device UA2950922 = {S15446, 136, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 4G LTE  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2957474 = {S15447, 129, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; Build/ICS.g04ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2980659 = {S15448, 59, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; TR720F Build/IML74K)*/
const Device UA2980818 = {S15449, 51, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3 Build/S-2-20)*/
const Device UA2980914 = {S15450, 142, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; %ModelName% Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2981017 = {S15451, 125, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2981208 = {S15452, 155, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; e1808_v75_jbl1_5640_cony Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2981615 = {S15453, 131, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; MID 940 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2985350 = {S15454, 152, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; e1908_v77_jbl1mt9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986681 = {S15455, 138, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987503 = {S15456, 152, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1808_v75_jbaol1_6628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987745 = {S15457, 137, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; iPhone Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2988049 = {S15458, 140, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; E60w Build/PioneerE60w) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3042235 = {S15459, 135, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.8; en-us; Android Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3043101 = {S15460, 157, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1911_v77_jbaol_g20mt9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3043285 = {S15461, 145, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Build/ICS.by Djzemych mod 21.09.12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3043344 = {S15462, 128, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; e101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3044338 = {S15463, 144, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Build/IRPMTD_6_02.47.00RPS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3048707 = {S15464, 145, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.0; vi-vn; Android Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3057536 = {S15465, 116, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3070133 = {S15466, 132, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ICS.BEAT.DUOS.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3070346 = {S15467, 59, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Tender Build/IMM76D)*/
const Device UA3099228 = {S15468, 151, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; e1911_v77_zlh1_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099622 = {S15469, 131, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MID 008 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3107503 = {S15470, 148, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; e1808_v75_hjy1a6x Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3117445 = {S15471, 132, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; in-id; e61 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3118533 = {S15472, 156, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; e1909c_v77_jbl_9p017_6628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3118682 = {S15473, 161, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1808_v75_mt6628_hjy1a6x_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3119518 = {S15474, 131, { &P17968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Unknown Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA157868 = {S15475, 73, { &P18088,&P17017,&P18410,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/4.0.13574/20.2485; U; en) Presto/2.5.25*/
const Device UA2431193 = {S15476, 88, { &P18088,&P17017,&P18410,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/4.0.10031/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2587622 = {S15477, 55, { &P18088,&P17017,&P18410,&P18092 }};	/*Useragent - Opera/9.50 (J2ME/MIDP; Opera Mini/4.0.10031/298; U; en)*/
const Device UA3022668 = {S15478, 80, { &P18088,&P17017,&P18410,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/4/30.3558; U; en) Presto/2.8.119 Version/11.10*/
const Device UA347151 = {S15479, 177, { &P15364,&P7133,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.36 Safari/535.7 AppEngine-Google; (+http://code.google.com/appengine; appid: s~quexdoo)*/
const Device UA351793 = {S15480, 142, { &P15364,&P7133,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7,gzip(gfe) (via translate.google.com)*/
const Device UA353541 = {S15481, 135, { &P15364,&P7133,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7,gzip(gfe) (via translate.google.com)*/
const Device UA366606 = {S15482, 144, { &P15364,&P7133,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA368129 = {S15483, 105, { &P15364,&P7133,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA368130 = {S15484, 98, { &P15364,&P7133,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA380155 = {S15485, 136, { &P15364,&P7133,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/534.7 ZemanaAID/FFFF0061*/
const Device UA2259351 = {S15486, 108, { &P15364,&P7133,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7,gzip(gfe)*/
const Device UA2362170 = {S15487, 115, { &P15364,&P7133,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7,gzip(gfe)*/
const Device UA2495397 = {S15488, 122, { &P15364,&P7133,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.910.20417 KChrome/3.2.5.20417  Safari/535.7*/
const Device UA2496800 = {S15489, 129, { &P15364,&P7133,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.910.20417 KChrome/3.2.5.20417  Safari/535.7*/
const Device UA2596284 = {S15490, 97, { &P15364,&P7133,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA2809748 = {S15491, 116, { &P15364,&P7133,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; JRCapture) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA2979839 = {S15492, 136, { &P15364,&P7133,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.904.0 Safari/535.7,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3101614 = {S15493, 93, { &P15364,&P7133,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows 7) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA2015619 = {S15494, 99, { &P15364,&P5690,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2016038 = {S15495, 99, { &P15364,&P5690,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2016043 = {S15496, 106, { &P15364,&P5690,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2042168 = {S15497, 136, { &P15364,&P5690,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)*/
const Device UA2042294 = {S15498, 138, { &P15364,&P5690,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2235271 = {S15499, 109, { &P15364,&P5690,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe)*/
const Device UA2540558 = {S15500, 121, { &P15364,&P5690,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; ; NCLIENT50_AAPXA934) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.79 Safari/537.1*/
const Device UA3094815 = {S15501, 135, { &P15364,&P5690,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.0 AOL/9.7 AOLBuild/4343.2039.US Safari/537.1*/
const Device UA916027 = {S15502, 86, { &P18088,&P17017,&P4719,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/4.3.25172/25.858; U; en) Presto/2.5.25 Version/10.54*/
const Device UA3033836 = {S15503, 88, { &P18088,&P17017,&P4719,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/4.3.25985/30.3697; U; en) Presto/2.8.119 Version/11.10*/
const Device UA3069079 = {S15504, 72, { &P18088,&P17017,&P4719,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/4.3.13057/19.836; U; ru) Presto/2.5.25*/
const Device UA2431171 = {S15505, 89, { &P18115,&P5764,&P18369,&P18092 }};	/*Useragent - Opera/9.80 (BlackBerry; Opera Mini/7.0.29990/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA347349 = {S15506, 88, { &P18088,&P17017,&P17373,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/6.5.26955/26.1283; U; en) Presto/2.8.119 Version/10.54*/
const Device UA2389188 = {S15507, 59, { &P18088,&P17017,&P17373,&P18092 }};	/*Useragent - Opera/9.50 (J2ME/MIDP; Opera Mini/6.5.28416/28.3165; U; en)*/
const Device UA2427614 = {S15508, 88, { &P18088,&P17017,&P17373,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/6.5.26955/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2431138 = {S15509, 90, { &P18088,&P17017,&P17373,&P18092 }};	/*Useragent - 0 Opera/9.80 (J2ME/MIDP; Opera Mini/6.5.27510/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA381323 = {S15510, 100, { &P15364,&P5690,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.46 Safari/535.11*/
const Device UA401369 = {S15511, 100, { &P15364,&P5690,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA401392 = {S15512, 107, { &P15364,&P5690,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA401925 = {S15513, 139, { &P15364,&P5690,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA601391 = {S15514, 136, { &P15364,&P5690,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/534.7 ZemanaAID/FFFF0061*/
const Device UA1951866 = {S15515, 118, { &P15364,&P5690,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.31 (KHTML, like Gecko) Chrome/17.0.558.0 Safari/534.31*/
const Device UA2161855 = {S15516, 177, { &P15364,&P5690,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11 AppEngine-Google; (+http://code.google.com/appengine; appid: s~huozhaiyuren)*/
const Device UA2166577 = {S15517, 155, { &P15364,&P5690,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.84 Safari/535.11 SE 2.X MetaSr 1.0,gzip(gfe) (via translate.google.com)*/
const Device UA2258965 = {S15518, 128, { &P15364,&P5690,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.84 Safari/535.11 SE 2.X MetaSr 1.0,gzip(gfe)*/
const Device UA2425902 = {S15519, 153, { &P15364,&P5690,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/17.0.944.0 Safari/535.8 BitdefenderSafepay/0.2.0.0 (OBK; 17.0.944.0; r396.2011)*/
const Device UA2590700 = {S15520, 125, { &P15364,&P5690,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.84 Safari/535.11 SE 2.X MetaSr 1.0*/
const Device UA2596237 = {S15521, 99, { &P15364,&P5690,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML like Gecko) Chrome/17.0.963.46 Safari/535.11*/
const Device UA2602558 = {S15522, 113, { &P15364,&P5690,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.1.963.51 Safari/535.11 MRCHROME SOC*/
const Device UA2817533 = {S15523, 116, { &P15364,&P5690,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; Alibrowser 1.8) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA2675447 = {S15524, 129, { &P17595,&P17297,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31,gzip(gfe)*/
const Device UA2678576 = {S15525, 119, { &P17595,&P17297,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA2678581 = {S15526, 119, { &P17595,&P17297,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA2678925 = {S15527, 119, { &P17595,&P17297,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA2678926 = {S15528, 119, { &P17595,&P17297,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA2678929 = {S15529, 119, { &P17595,&P17297,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA2679722 = {S15530, 119, { &P17595,&P17297,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA2679833 = {S15531, 158, { &P17595,&P17297,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2708301 = {S15532, 129, { &P17595,&P17297,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31,gzip(gfe)*/
const Device UA2567801 = {S15533, 120, { &P17595,&P17296,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2567811 = {S15534, 120, { &P17595,&P17296,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2567812 = {S15535, 120, { &P17595,&P17296,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2567789 = {S15536, 120, { &P17595,&P17296,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2567875 = {S15537, 120, { &P17595,&P17296,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2567882 = {S15538, 120, { &P17595,&P17296,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2567914 = {S15539, 120, { &P17595,&P17296,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2567936 = {S15540, 120, { &P17595,&P17296,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2568064 = {S15541, 120, { &P17595,&P17296,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2568248 = {S15542, 130, { &P17595,&P17296,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)*/
const Device UA2583823 = {S15543, 159, { &P17595,&P17296,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2674629 = {S15544, 129, { &P17595,&P17296,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/25.0.648.205 Safari/534.16*/
const Device UA2674861 = {S15545, 129, { &P17595,&P17296,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/25.0.648.205 Safari/534.16*/
const Device UA2714758 = {S15546, 129, { &P17595,&P17296,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/25.0.648.205 Safari/534.16*/
const Device UA221015 = {S15547, 106, { &P15364,&P7133,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1*/
const Device UA247980 = {S15548, 99, { &P15364,&P7133,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1*/
const Device UA333314 = {S15549, 137, { &P15364,&P7133,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/534.7 ZemanaAID/FFFF0061*/
const Device UA336442 = {S15550, 143, { &P15364,&P7133,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.218 Safari/535.1,gzip(gfe) (via translate.google.com)*/
const Device UA337799 = {S15551, 136, { &P15364,&P7133,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1,gzip(gfe) (via translate.google.com)*/
const Device UA451407 = {S15552, 118, { &P15364,&P7133,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; ru-RU) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/13.0.782.218 Safari/533.3*/
const Device UA669035 = {S15553, 174, { &P15364,&P7133,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1 AppEngine-Google; (+http://code.google.com/appengine; appid: s~pickieco10)*/
const Device UA2455907 = {S15554, 109, { &P15364,&P7133,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1,gzip(gfe)*/
const Device UA2639240 = {S15555, 116, { &P15364,&P7133,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1,gzip(gfe)*/
const Device UA2644326 = {S15556, 104, { &P15364,&P7133,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML like Gecko) Chrome/13.0.782.24 Safari/535.1*/
const Device UA3083309 = {S15557, 111, { &P15364,&P7133,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; compatible; AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1*/
const Device UA32810 = {S15558, 153, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT18i Build/4.0.1.A.0.119) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48638 = {S15559, 152, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT18i Build/4.0.1.A.1.17) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48745 = {S15560, 153, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; SonyEricssonLT18a Build/4.0.1.A.0.227) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49661 = {S15561, 153, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT18i Build/4.0.1.A.0.254) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49667 = {S15562, 153, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT18i Build/4.0.1.A.0.266) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49674 = {S15563, 153, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT18i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70788 = {S15564, 141, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; LT18i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72573 = {S15565, 152, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT18i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74196 = {S15566, 154, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT18iv Build/4.0.1.A.0.266) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA75802 = {S15567, 153, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT18i Build/4.0.1.A.0.284) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76395 = {S15568, 141, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LT18i Build/4.0.1.A.0.266) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78067 = {S15569, 155, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SonyEricssonLT18i-o Build/4.0.1.A.0.266) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78608 = {S15570, 153, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; SonyEricssonLT18a Build/4.0.1.A.0.165) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369507 = {S15571, 153, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SonyEricssonLT18iv Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623025 = {S15572, 154, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SonyEricssonLT18i-o Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623230 = {S15573, 64, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LT18i Build/4.0.2.A.0.62)*/
const Device UA623464 = {S15574, 152, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonLT18a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625057 = {S15575, 152, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; SonyEricssonLT18a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625344 = {S15576, 154, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SonyEricssonLT18i-o Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625414 = {S15577, 152, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627673 = {S15578, 140, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LT18i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA628507 = {S15579, 151, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT18i Build/4.0.2.C.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA629618 = {S15580, 152, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonLT18i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA630679 = {S15581, 153, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonLT18iv Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA635120 = {S15582, 140, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA636850 = {S15583, 128, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.42) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA671274 = {S15584, 128, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.58) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA674074 = {S15585, 151, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; SonyEricssonLT18a Build/4.0.2.D.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA677724 = {S15586, 128, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.62) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA678599 = {S15587, 140, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; LT18a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA683697 = {S15588, 139, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LT18i Build/4.0.2.C.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA695615 = {S15589, 152, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-hk; SonyEricssonLT18i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA722845 = {S15590, 151, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; SonyEricssonLT18a Build/4.0.2.D.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA725288 = {S15591, 153, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; SonyEricssonLT18a Build/4.0.1.A.0.266) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA740062 = {S15592, 140, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LT18a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA826728 = {S15593, 140, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; LT18i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA842379 = {S15594, 129, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.1.A.0.266) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA904642 = {S15595, 133, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.62) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1940167 = {S15596, 140, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1940730 = {S15597, 64, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LT18a Build/4.0.2.A.0.62)*/
const Device UA1941357 = {S15598, 141, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; LT18a Build/4.0.1.A.0.266) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1962458 = {S15599, 150, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-; SonyEricssonLT18a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1964322 = {S15600, 150, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-; SonyEricssonLT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1971854 = {S15601, 139, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; LT18a Build/4.0.2.D.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024942 = {S15602, 133, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.58) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2029749 = {S15603, 133, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.69) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2034202 = {S15604, 133, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.42) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2585365 = {S15605, 153, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonLT18iv Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2641708 = {S15606, 134, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.1.A.0.266) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2641739 = {S15607, 152, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonLT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2808954 = {S15608, 133, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build 4.0.2.A.0.62) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2981819 = {S15609, 140, { &P12798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1996293 = {S15610, 140, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC One X Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027795 = {S15611, 140, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2182288 = {S15612, 140, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC One X Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2231932 = {S15613, 140, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2238852 = {S15614, 136, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; One X Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257985 = {S15615, 140, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC One X Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258019 = {S15616, 136, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; One X Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2339118 = {S15617, 62, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC One X Build/JRO03C)*/
const Device UA2339384 = {S15618, 140, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.5; en-us; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2339913 = {S15619, 140, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2344312 = {S15620, 136, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; One X Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2355117 = {S15621, 136, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; One X Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2383135 = {S15622, 140, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; zh-cn; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2384857 = {S15623, 140, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; EndeavorU Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2416135 = {S15624, 131, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC One X Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2426155 = {S15625, 141, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; HTC EVA_UL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2430757 = {S15626, 150, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_X-orange-LS Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2437181 = {S15627, 152, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_X/3.14.161.27 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2457086 = {S15628, 62, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; EndeavorU Build/JRO03C)*/
const Device UA2459652 = {S15629, 152, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; HTC_One_X/3.14.163.27 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2460210 = {S15630, 152, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-ch; HTC_One_X/3.14.166.27 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2461987 = {S15631, 151, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_X/3.16.110.4 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2462122 = {S15632, 152, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_X/3.14.111.27 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467429 = {S15633, 140, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC S720e Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2468160 = {S15634, 151, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_X/2.17.161.4 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2482675 = {S15635, 62, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC One X Build/JZO54K)*/
const Device UA2487450 = {S15636, 152, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-at; HTC_One_X/3.14.169.27 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2506967 = {S15637, 152, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-at; HTC_One_X/3.14.112.27 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2507592 = {S15638, 151, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_X/2.17.111.3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2507775 = {S15639, 152, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; HTC_One_X/3.14.118.27 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528153 = {S15640, 131, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC S720e Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2530022 = {S15641, 151, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_X/2.17.110.5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2530660 = {S15642, 152, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; HTC_One_X/3.14.114.27 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2559114 = {S15643, 149, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PJ83100/3.18.502.6 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2596242 = {S15644, 130, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; HTC One X Build/IMM76D) UC AppleWebKit/534.31 (KHTML like Gecko) Mobile Safari/534.31*/
const Device UA2597011 = {S15645, 151, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; HTC_One_X/2.17.163.4 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597098 = {S15646, 130, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.5; en-us; HTC One X Build/IML74K) UC AppleWebKit/534.31 (KHTML like Gecko) Mobile Safari/534.31*/
const Device UA2598179 = {S15647, 139, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2653921 = {S15648, 58, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; One X Build/JZO54K)*/
const Device UA2655303 = {S15649, 138, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-; HTC_One_X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2655464 = {S15650, 138, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; HTC One X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2657401 = {S15651, 149, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PJ83100/2.20.502.7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2686686 = {S15652, 131, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.5; zh-cn; HTC One X Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2692667 = {S15653, 140, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC One X Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695347 = {S15654, 140, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2716090 = {S15655, 152, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_X/3.14.161.28 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2728046 = {S15656, 140, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; HTC One X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2763991 = {S15657, 151, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-ch; HTC_One_X/2.17.166.3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2818101 = {S15658, 140, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.5; zh-cn; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2823208 = {S15659, 131, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; zh-cn; HTC One X Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2847462 = {S15660, 62, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC S720e Build/JRO03C)*/
const Device UA2850239 = {S15661, 137, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTCOneX Build/JRO03C) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878254 = {S15662, 136, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S720e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879420 = {S15663, 149, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PJ83100/3.18.502.6 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934180 = {S15664, 140, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.5; en-us; HTC One X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2938111 = {S15665, 152, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_X/3.14.111.28 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2942259 = {S15666, 149, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PJ83100/3.18.502.6 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2982031 = {S15667, 140, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; vi-vn; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984714 = {S15668, 140, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; vi-vn; HTC One X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986784 = {S15669, 146, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_X Build/JB REMIX 3.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3064168 = {S15670, 166, { &P18295,&P18111,&P18112,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.1;dev=evita;devm=HTC One X;man=HTC;brw=Android;shwd=720x1280;shwa=720x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA1996319 = {S15671, 99, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.25 (KHTML, like Gecko) Sparrow/1081.53*/
const Device UA2041168 = {S15672, 102, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) QuickLook/4.0*/
const Device UA2164065 = {S15673, 99, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.25 (KHTML, like Gecko) Sparrow/1081.52*/
const Device UA2168906 = {S15674, 109, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en-US; rv:1.9.2.10) Gecko Expeditor 6.2.2.X-1115 Gecko/X-1115*/
const Device UA2195638 = {S15675, 101, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/1178*/
const Device UA2199899 = {S15676, 101, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/1164*/
const Device UA2201592 = {S15677, 101, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/1173*/
const Device UA2203340 = {S15678, 101, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/1172*/
const Device UA2203599 = {S15679, 104, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) NetNewsWire/3.3*/
const Device UA2205827 = {S15680, 52, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - QuickTime/7.7.1 (qtver=7.7.1;cpu=IA32;os=Mac 10.8.2)*/
const Device UA2205830 = {S15681, 101, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/1176*/
const Device UA2206141 = {S15682, 106, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) NetNewsWire/3.3.2*/
const Device UA2232623 = {S15683, 106, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Vienna/2.6.0.2601*/
const Device UA2232714 = {S15684, 98, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Gruml/1.2*/
const Device UA2237527 = {S15685, 107, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) NetNewsWire/3.2.15*/
const Device UA2258450 = {S15686, 101, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Safari/522.0*/
const Device UA2350057 = {S15687, 105, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Vienna/3.0b.2820*/
const Device UA2360619 = {S15688, 100, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Gruml/1.3.0*/
const Device UA2381147 = {S15689, 101, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Shrook/2.85l*/
const Device UA2383484 = {S15690, 96, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sunrise*/
const Device UA2384838 = {S15691, 43, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/4.0 (Mac OS X 10.8.2) Java/1.7.0_09*/
const Device UA2399594 = {S15692, 103, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/1143.6*/
const Device UA2415907 = {S15693, 106, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) NetNewsWire/3.3.1*/
const Device UA2421886 = {S15694, 111, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) RealPlayer/12.0.1.1750*/
const Device UA2425283 = {S15695, 122, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Vienna/3.0b.2821 Safari/536.26.17*/
const Device UA2454509 = {S15696, 97, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) iWeb/304*/
const Device UA2465596 = {S15697, 106, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) NetNewsWire/3.1.7*/
const Device UA2524819 = {S15698, 95, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) TubeTV*/
const Device UA2551002 = {S15699, 103, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/1043.1*/
const Device UA2577487 = {S15700, 106, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) NetNewsWire/3.2.7*/
const Device UA2578681 = {S15701, 52, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - QuickTime/7.7.1 (qtver=7.7.1;cpu=IA32;os=Mac 10.8.3)*/
const Device UA2593971 = {S15702, 100, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/904*/
const Device UA2652046 = {S15703, 101, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/1134*/
const Device UA2691526 = {S15704, 104, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/1081.32*/
const Device UA2691747 = {S15705, 101, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/1144*/
const Device UA2715456 = {S15706, 87, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko)*/
const Device UA2716668 = {S15707, 136, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) Vienna/3.0.0 Beta 11 :9d8d6f6: Safari/536.29.13*/
const Device UA2726535 = {S15708, 102, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Propane/1.2.5*/
const Device UA2735135 = {S15709, 120, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) Safari/536.28.10 webkit2png/dev*/
const Device UA2754180 = {S15710, 56, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.1 rv:172 (Macintosh; Mac OS X 10.8.3; en_US)*/
const Device UA2773083 = {S15711, 56, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.2 rv:175 (Macintosh; Mac OS X 10.8.3; en_US)*/
const Device UA2773673 = {S15712, 56, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.3 rv:176 (Macintosh; Mac OS X 10.8.3; en_US)*/
const Device UA2774171 = {S15713, 56, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.3 rv:176 (Macintosh; Mac OS X 10.8.4; en_US)*/
const Device UA2781404 = {S15714, 52, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - QuickTime/7.7.1 (qtver=7.7.1;cpu=IA32;os=Mac 10.8.4)*/
const Device UA2834482 = {S15715, 101, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Safari/417.8*/
const Device UA2840101 = {S15716, 99, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/536.11 (KHTML, like Gecko) Safari/536.11*/
const Device UA2844719 = {S15717, 103, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/1043.2*/
const Device UA2867961 = {S15718, 100, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) Safari/531.9*/
const Device UA2870017 = {S15719, 56, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.4 rv:183 (Macintosh; Mac OS X 10.8.4; en_US)*/
const Device UA2871601 = {S15720, 56, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.4 rv:183 (Macintosh; Mac OS X 10.8.4; en_AU)*/
const Device UA2873621 = {S15721, 56, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.4 rv:183 (Macintosh; Mac OS X 10.8.4; en_GB)*/
const Device UA2894297 = {S15722, 105, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) NetNewsWire/4.0.0*/
const Device UA2898250 = {S15723, 118, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) Paparazzi!/0.6.7 (like Safari)*/
const Device UA2943253 = {S15724, 118, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) Safari/536.30.1 webkit2png/dev*/
const Device UA2981700 = {S15725, 102, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Sparrow/959.4*/
const Device UA2999391 = {S15726, 97, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko),gzip(gfe)*/
const Device UA3005224 = {S15727, 99, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) NetNewsWire*/
const Device UA3006190 = {S15728, 100, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) Shrook/2.88l*/
const Device UA3030106 = {S15729, 56, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.5 rv:192 (Macintosh; Mac OS X 10.8.4; en_AU)*/
const Device UA3034870 = {S15730, 56, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.6 rv:194 (Macintosh; Mac OS X 10.8.4; en_US)*/
const Device UA3036000 = {S15731, 56, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.5 rv:192 (Macintosh; Mac OS X 10.8.4; en_US)*/
const Device UA3036006 = {S15732, 56, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.5 rv:192 (Macintosh; Mac OS X 10.8.4; en_GB)*/
const Device UA3078544 = {S15733, 56, { &P17595,&P18207,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.5 rv:192 (Macintosh; Mac OS X 10.8.5; en_US)*/
const Device UA11770 = {S15734, 77, { &P11730,&P5156,&P3519,&P18092 }};	/*Useragent - Blackberry9700/6.0.0.526 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102*/
const Device UA56051 = {S15735, 14, { &P11730,&P5156,&P3519,&P18092 }};	/*Useragent - BlackBerry9700*/
const Device UA63216 = {S15736, 129, { &P11730,&P5156,&P3519,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9700; en-US) AppleWebKit/534.8+ (KHTML, like Gecko) Version/6.0.0.448 Mobile Safari/534.8+*/
const Device UA2193628 = {S15737, 125, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/22.0.1229.79) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4*/
const Device UA2195896 = {S15738, 132, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/22.0.1229.79) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4*/
const Device UA2200313 = {S15739, 132, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/22.0.1229.91) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.91 Safari/537.4*/
const Device UA2201482 = {S15740, 125, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/22.0.1229.91) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.91 Safari/537.4*/
const Device UA2201649 = {S15741, 132, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/22.0.1229.92) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.92 Safari/537.4*/
const Device UA2201813 = {S15742, 125, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/22.0.1229.92) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.92 Safari/537.4*/
const Device UA2203738 = {S15743, 216, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/22.0.1229.94; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E)*/
const Device UA2203742 = {S15744, 89, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/22.0.1229.94)*/
const Device UA2203730 = {S15745, 96, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/22.0.1229.94)*/
const Device UA2203776 = {S15746, 125, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/22.0.1229.94) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203801 = {S15747, 132, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/22.0.1229.94) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203865 = {S15748, 214, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/22.0.1229.94; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Tablet PC 2.0)*/
const Device UA2203896 = {S15749, 197, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/22.0.1229.94; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2203981 = {S15750, 194, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; chromeframe/22.0.1229.94)*/
const Device UA2203988 = {S15751, 183, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/22.0.1229.94; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2205536 = {S15752, 125, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/22.0.1229.56) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.56 Safari/537.4*/
const Device UA2231614 = {S15753, 191, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/22.0.1229.94; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA2236199 = {S15754, 132, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/22.0.1229.96) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.96 Safari/537.4*/
const Device UA2237801 = {S15755, 176, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/22.0.1229.94; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2237847 = {S15756, 199, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; chromeframe/22.0.1229.94)*/
const Device UA2238251 = {S15757, 207, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; GTB7.4; chromeframe/22.0.1229.94; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)*/
const Device UA2238470 = {S15758, 202, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; chromeframe/22.0.1229.94)*/
const Device UA2246685 = {S15759, 166, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; chromeframe/22.0.1229.94)*/
const Device UA2259114 = {S15760, 125, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/22.0.1229.96) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.96 Safari/537.4*/
const Device UA2259653 = {S15761, 111, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Avant Browser; chromeframe/22.0.1229.94)*/
const Device UA2349273 = {S15762, 125, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/22.0.1229.26) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.26 Safari/537.4*/
const Device UA2350579 = {S15763, 132, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/22.0.1229.56) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.56 Safari/537.4*/
const Device UA2354395 = {S15764, 222, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; GTB7.4; chromeframe/22.0.1229.94; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C)*/
const Device UA2354521 = {S15765, 96, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/22.0.1229.94)*/
const Device UA2354628 = {S15766, 202, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/4.0; GTB7.4; chromeframe/22.0.1229.94; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)*/
const Device UA2355114 = {S15767, 226, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/22.0.1229.94; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Zune 4.7)*/
const Device UA2653915 = {S15768, 97, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/22.0.1229.94)*/
const Device UA2762807 = {S15769, 90, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/22.0.1229.94)*/
const Device UA2837782 = {S15770, 132, { &P15364,&P7133,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/22.0.1229.95) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.95 Safari/537.4*/
const Device UA2873598 = {S15771, 120, { &P17595,&P17296,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2873777 = {S15772, 120, { &P17595,&P17296,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2873779 = {S15773, 120, { &P17595,&P17296,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2873918 = {S15774, 120, { &P17595,&P17296,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2874011 = {S15775, 120, { &P17595,&P17296,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2874142 = {S15776, 120, { &P17595,&P17296,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2874325 = {S15777, 120, { &P17595,&P17296,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2874392 = {S15778, 120, { &P17595,&P17296,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2874393 = {S15779, 130, { &P17595,&P17296,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe)*/
const Device UA2875073 = {S15780, 120, { &P17595,&P17296,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2877330 = {S15781, 159, { &P17595,&P17296,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA909017 = {S15782, 141, { &P12968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT26i Build/6.1.A.0.452) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA924859 = {S15783, 141, { &P12968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT26i Build/6.1.A.0.453) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1953961 = {S15784, 132, { &P12968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26i Build/6.1.A.0.452) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1971335 = {S15785, 136, { &P12968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LT26i Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2007461 = {S15786, 141, { &P12968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT26i Build/6.1.A.0.454) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030399 = {S15787, 152, { &P12968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonLT26i Build/6.1.A.2.45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030559 = {S15788, 62, { &P12968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT26i Build/6.1.A.2.45)*/
const Device UA2030756 = {S15789, 140, { &P12968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT26i Build/6.1.A.2.45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2191499 = {S15790, 131, { &P12968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26i Build/6.1.A.2.45) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2232271 = {S15791, 140, { &P12968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT26i Build/6.1.A.2.50) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2236127 = {S15792, 140, { &P12968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; LT26i Build/6.1.A.1.58) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2338804 = {S15793, 132, { &P12968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26i Build/6.1.A.0.453) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2423869 = {S15794, 140, { &P12968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT26i Build/6.1.A.2.55) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2437197 = {S15795, 154, { &P12968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonLT26i-o Build/6.1.A.2.55) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2525545 = {S15796, 131, { &P12968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26i Build/6.1.A.2.50) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2528416 = {S15797, 131, { &P12968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26i Build/6.1.A.2.55) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2656751 = {S15798, 153, { &P12968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; SonyEricssonLT26iv Build/6.1.A.2.55) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2745653 = {S15799, 136, { &P12968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT26i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2746123 = {S15800, 138, { &P12968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; LT26i Build/6.1.A.2.45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2820619 = {S15801, 131, { &P12968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; LT26i Build 6.1.A.2.45) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA3069691 = {S15802, 58, { &P12968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; LT26i Build/IML74K)*/
const Device UA2667916 = {S15803, 137, { &P21648,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2670094 = {S15804, 164, { &P21648,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9505 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2709043 = {S15805, 177, { &P21648,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; fr-fr; SAMSUNG GT-I9505/I9505XXUAMDF Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2710359 = {S15806, 171, { &P21648,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; fr-fr; SAMSUNG GT-I9505-ORANGE Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2715756 = {S15807, 177, { &P21648,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; de-de; SAMSUNG GT-I9505/I9505XXUAMDC Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2716550 = {S15808, 165, { &P21648,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9505X Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2717847 = {S15809, 177, { &P21648,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9505/I9505XXUAMDD Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2737408 = {S15810, 138, { &P21648,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505X Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2740648 = {S15811, 177, { &P21648,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; de-de; SAMSUNG GT-I9505/I9505XXUAME2 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2885332 = {S15812, 177, { &P21648,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9505/I9505XXUBMEA Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2898307 = {S15813, 177, { &P21648,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; nl-nl; SAMSUNG GT-I9505/I9505XXUBMF4 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2998140 = {S15814, 156, { &P21648,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; GT-I9505 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3035056 = {S15815, 177, { &P21648,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; de-de; SAMSUNG GT-I9505/I9505XXUBMGC Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3036397 = {S15816, 177, { &P21648,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; nl-nl; SAMSUNG GT-I9505/I9505XXUBMGA Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3067929 = {S15817, 177, { &P21648,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9505/I9505XXUDMGG Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3081146 = {S15818, 177, { &P21648,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; nl-nl; SAMSUNG GT-I9505/I9505XXUDMH7 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3096455 = {S15819, 177, { &P21648,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; de-de; SAMSUNG GT-I9505/I9505XXUDMH9 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3114768 = {S15820, 177, { &P21648,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; de-de; SAMSUNG GT-I9505/I9505XXUDMI1 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3114769 = {S15821, 177, { &P21648,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; nl-nl; SAMSUNG GT-I9505/I9505XXUDMH6 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3062785 = {S15822, 119, { &P17595,&P17296,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3062786 = {S15823, 119, { &P17595,&P17296,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3062794 = {S15824, 119, { &P17595,&P17296,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3062796 = {S15825, 119, { &P17595,&P17296,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3062801 = {S15826, 119, { &P17595,&P17296,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3062802 = {S15827, 119, { &P17595,&P17296,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3062804 = {S15828, 119, { &P17595,&P17296,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3065628 = {S15829, 119, { &P17595,&P17296,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3065631 = {S15830, 119, { &P17595,&P17296,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3065634 = {S15831, 129, { &P17595,&P17296,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36,gzip(gfe)*/
const Device UA3080101 = {S15832, 158, { &P17595,&P17296,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2387788 = {S15833, 64, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.12*/
const Device UA2388354 = {S15834, 57, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.12*/
const Device UA2426258 = {S15835, 67, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.12,gzip(gfe)*/
const Device UA2426929 = {S15836, 87, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.12*/
const Device UA2426933 = {S15837, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.12*/
const Device UA2427089 = {S15838, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.12*/
const Device UA2427101 = {S15839, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.12*/
const Device UA2427110 = {S15840, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.12*/
const Device UA2427264 = {S15841, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.12*/
const Device UA2427328 = {S15842, 69, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64) Opera 12.12*/
const Device UA2427597 = {S15843, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.12*/
const Device UA2427612 = {S15844, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.12*/
const Device UA2427901 = {S15845, 80, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.12*/
const Device UA2427902 = {S15846, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.12*/
const Device UA2428167 = {S15847, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.12*/
const Device UA2429197 = {S15848, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.12*/
const Device UA2430540 = {S15849, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.12*/
const Device UA2430872 = {S15850, 87, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.12*/
const Device UA2432048 = {S15851, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.12*/
const Device UA2432114 = {S15852, 75, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.12*/
const Device UA2432402 = {S15853, 87, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.12*/
const Device UA2432655 = {S15854, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.12*/
const Device UA2434000 = {S15855, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.12*/
const Device UA2434088 = {S15856, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.12*/
const Device UA2436129 = {S15857, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.12*/
const Device UA2436429 = {S15858, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.12*/
const Device UA2437387 = {S15859, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.12*/
const Device UA2437424 = {S15860, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.12*/
const Device UA2437439 = {S15861, 80, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.12*/
const Device UA2437443 = {S15862, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.12*/
const Device UA2437444 = {S15863, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.12*/
const Device UA2437484 = {S15864, 80, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.12*/
const Device UA2437495 = {S15865, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.12*/
const Device UA2437513 = {S15866, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.12*/
const Device UA2437520 = {S15867, 87, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.12*/
const Device UA2437522 = {S15868, 74, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.12,gzip(gfe)*/
const Device UA2437525 = {S15869, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.12*/
const Device UA2437526 = {S15870, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.12*/
const Device UA2437527 = {S15871, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.12*/
const Device UA2437529 = {S15872, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.12*/
const Device UA2437533 = {S15873, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.12*/
const Device UA2437567 = {S15874, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.12*/
const Device UA2437569 = {S15875, 80, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.12*/
const Device UA2437573 = {S15876, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.12*/
const Device UA2437600 = {S15877, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.12*/
const Device UA2437605 = {S15878, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.12*/
const Device UA2437621 = {S15879, 80, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.12*/
const Device UA2437622 = {S15880, 87, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.12*/
const Device UA2437607 = {S15881, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.12*/
const Device UA2437647 = {S15882, 80, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.12*/
const Device UA2437660 = {S15883, 80, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.12*/
const Device UA2459469 = {S15884, 80, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.12*/
const Device UA2459846 = {S15885, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.12*/
const Device UA2460288 = {S15886, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.12*/
const Device UA2460416 = {S15887, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.12*/
const Device UA2460428 = {S15888, 80, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.12*/
const Device UA2460566 = {S15889, 74, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.12 Yandex.Translate*/
const Device UA2460588 = {S15890, 80, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.12*/
const Device UA2460592 = {S15891, 80, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.12*/
const Device UA2460614 = {S15892, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5660)) Presto/2.12.388 Version/12.12*/
const Device UA2460816 = {S15893, 87, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03639)) Presto/2.12.388 Version/12.12*/
const Device UA2460965 = {S15894, 87, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.12*/
const Device UA2461061 = {S15895, 80, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.12*/
const Device UA2461069 = {S15896, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4933)) Presto/2.12.388 Version/12.12*/
const Device UA2461387 = {S15897, 87, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.12*/
const Device UA2461798 = {S15898, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4661)) Presto/2.12.388 Version/12.12*/
const Device UA2467935 = {S15899, 87, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03789)) Presto/2.12.388 Version/12.12*/
const Device UA2468497 = {S15900, 80, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.12*/
const Device UA2468853 = {S15901, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.12*/
const Device UA2468888 = {S15902, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4832)) Presto/2.12.388 Version/12.12*/
const Device UA2468969 = {S15903, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.12*/
const Device UA2469400 = {S15904, 80, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5278)) Presto/2.12.388 Version/12.12*/
const Device UA2469665 = {S15905, 87, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5230)) Presto/2.12.388 Version/12.12*/
const Device UA2484145 = {S15906, 80, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03658)) Presto/2.12.388 Version/12.12*/
const Device UA2484176 = {S15907, 87, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5217)) Presto/2.12.388 Version/12.12*/
const Device UA2484895 = {S15908, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5650)) Presto/2.12.388 Version/12.12*/
const Device UA2485258 = {S15909, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4110)) Presto/2.12.388 Version/12.12*/
const Device UA2485259 = {S15910, 80, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5288)) Presto/2.12.388 Version/12.12*/
const Device UA2485322 = {S15911, 87, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5336)) Presto/2.12.388 Version/12.12*/
const Device UA2485851 = {S15912, 103, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.12,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2486103 = {S15913, 80, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5325)) Presto/2.12.388 Version/12.12*/
const Device UA2486781 = {S15914, 87, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5299)) Presto/2.12.388 Version/12.12*/
const Device UA2490658 = {S15915, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.12*/
const Device UA2507169 = {S15916, 87, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5300)) Presto/2.12.388 Version/12.12*/
const Device UA2508982 = {S15917, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.12*/
const Device UA2529500 = {S15918, 87, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03791)) Presto/2.12.388 Version/12.12*/
const Device UA2534895 = {S15919, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.12*/
const Device UA2537921 = {S15920, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.12*/
const Device UA2538123 = {S15921, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.12*/
const Device UA2539612 = {S15922, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.12*/
const Device UA2556211 = {S15923, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5734)) Presto/2.12.388 Version/12.12*/
const Device UA2558491 = {S15924, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5988)) Presto/2.12.388 Version/12.12*/
const Device UA2560081 = {S15925, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.12*/
const Device UA2561714 = {S15926, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.12*/
const Device UA2565019 = {S15927, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6035)) Presto/2.12.388 Version/12.12*/
const Device UA2569150 = {S15928, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.12*/
const Device UA2600448 = {S15929, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6046)) Presto/2.12.388 Version/12.12*/
const Device UA2604210 = {S15930, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.12*/
const Device UA2606116 = {S15931, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.12*/
const Device UA2660537 = {S15932, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6032)) Presto/2.12.388 Version/12.12*/
const Device UA2665393 = {S15933, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.12*/
const Device UA2669035 = {S15934, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.12*/
const Device UA2670010 = {S15935, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.12*/
const Device UA2699622 = {S15936, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6044)) Presto/2.12.388 Version/12.12*/
const Device UA2716817 = {S15937, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.12*/
const Device UA2747526 = {S15938, 80, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03780)) Presto/2.12.388 Version/12.12*/
const Device UA2752630 = {S15939, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.12*/
const Device UA2756725 = {S15940, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.12*/
const Device UA2763310 = {S15941, 96, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.12,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2763824 = {S15942, 106, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.12*/
const Device UA2767585 = {S15943, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.12*/
const Device UA2846908 = {S15944, 80, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03638)) Presto/2.12.388 Version/12.12*/
const Device UA2847074 = {S15945, 80, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.6 (build 03402)) Presto/2.12.388 Version/12.12*/
const Device UA2961130 = {S15946, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6638)) Presto/2.12.388 Version/12.12*/
const Device UA2969641 = {S15947, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.12*/
const Device UA2996590 = {S15948, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6261)) Presto/2.12.388 Version/12.12*/
const Device UA3076654 = {S15949, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6628)) Presto/2.12.388 Version/12.12*/
const Device UA3078648 = {S15950, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.12*/
const Device UA3088696 = {S15951, 79, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6573)) Presto/2.12.388 Version/12.12*/
const Device UA3092130 = {S15952, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.1 (build 6673)) Presto/2.12.388 Version/12.12*/
const Device UA3107406 = {S15953, 86, { &P15364,&P7133,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.1 (build 6337)) Presto/2.12.388 Version/12.12*/
const Device UA2434467 = {S15954, 150, { &P12280,&P18432,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A551 Safari/8536.25*/
const Device UA2434476 = {S15955, 150, { &P12280,&P18432,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A525 Safari/8536.25*/
const Device UA2434480 = {S15956, 150, { &P12280,&P18432,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A523 Safari/8536.25*/
const Device UA2434485 = {S15957, 148, { &P12280,&P18432,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A405 Safari/8536.25*/
const Device UA2434577 = {S15958, 148, { &P12280,&P18432,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A403 Safari/8536.25*/
const Device UA36878 = {S15959, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; Synergy Rom Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47479 = {S15960, 147, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Broodcomb Build/4.5.1_57_DX5-26) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49852 = {S15961, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; FIH-FB0 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70171 = {S15962, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Inspire4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA192386 = {S15963, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; zh-cn; m9 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA245456 = {S15964, 155, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Gummy Charged GBE 2.0 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250933 = {S15965, 154, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HUMBLE by Danalo1979 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA328035 = {S15966, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Generation X Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA332984 = {S15967, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; CSL-MI410 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA333644 = {S15968, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SX-SP700A Build/SKYPAD_Alpha) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA337113 = {S15969, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SHIFTAO5P Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA338087 = {S15970, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; VEGAn-TAB Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA339394 = {S15971, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; BLACKSOURC3 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA355673 = {S15972, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; AIRPAD Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA358091 = {S15973, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; X7G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA358968 = {S15974, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; sv-se; ARGM900HCA Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368729 = {S15975, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HS-ET919 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369287 = {S15976, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; HTC-Z510d/1.15.1401.1) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369774 = {S15977, 119, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; SO1 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA370029 = {S15978, 130, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; B5 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370072 = {S15979, 58, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; KPT A9 Build/GRJ22)*/
const Device UA370433 = {S15980, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 7005 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370499 = {S15981, 62, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.1; C905 Build/GINGERBREAD)*/
const Device UA370579 = {S15982, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SANSUI ETAB 700 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370616 = {S15983, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; ARGM900HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370848 = {S15984, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; GIONEE GN380 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA377864 = {S15985, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; SK-S150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA421797 = {S15986, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; AIR-910 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA421856 = {S15987, 147, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; PlayTabPro-GZ Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA430102 = {S15988, 154, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; Fly_IQ260; Build/ALPS.GB.FDD2.MP.V2.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA437113 = {S15989, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; iw-il; ALODM722HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA437167 = {S15990, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; hd-us; BlueStacks Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA437175 = {S15991, 155, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Gummy Charged GBE 2.1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA447133 = {S15992, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; md-us; VEGAn-TAB Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA450112 = {S15993, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; EM73_SIS_GC2M_V2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458775 = {S15994, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; MID1106 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458968 = {S15995, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; ru-ru; MID7025 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA550108 = {S15996, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; de-de; Cat StarGate Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA552186 = {S15997, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; th-th; WellcoM-A99 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA557432 = {S15998, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; Cat StarGate Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA567912 = {S15999, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; pt-br; SEBM912HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA570889 = {S16000, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; I10-50 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA571281 = {S16001, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; Wxt Tab PC Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA573820 = {S16002, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; LUBM912HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA578369 = {S16003, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; ko-kr; YRHG803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA578600 = {S16004, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; I10-80 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA581140 = {S16005, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Blade2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA581408 = {S16006, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; STM908HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623529 = {S16007, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; USCCADT71US Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA624101 = {S16008, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; UTV210 Build/GR144) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA634277 = {S16009, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; iw-il; CRYM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA637813 = {S16010, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; DGM912HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA639276 = {S16011, 90, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; BlackBerry Runtime for Android Apps Build/2.0.0.56)*/
const Device UA643767 = {S16012, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; fr-fr; ST8000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA644109 = {S16013, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SHIFTAO5P Build/FRG83D; CM-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA649727 = {S16014, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; KATBL10AND Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA650618 = {S16015, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Vision Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA651533 = {S16016, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; 97FC Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA655800 = {S16017, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; ZTE Blade 2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA656311 = {S16018, 128, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Cloud9/1.0.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA659876 = {S16019, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; SLIDEPAD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA661873 = {S16020, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Sensation 4g Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA662886 = {S16021, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; tr-tr; NAM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA667806 = {S16022, 163, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Bionix 2 - A TeamWhiskey Production Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA680613 = {S16023, 162, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Cydle M7 (v0004.06.03.02.en) Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA683703 = {S16024, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; pt-br; SWT_GT-7220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA687532 = {S16025, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; LUBM908HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA691179 = {S16026, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; PC677 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA692161 = {S16027, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; CHARM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA695397 = {S16028, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; md-us; m799ca Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA699197 = {S16029, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; t9199p Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA699701 = {S16030, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Gruesomes Rom Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA700501 = {S16031, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; in-id; PTM722HR Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA700601 = {S16032, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-us; GASEM722HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA701320 = {S16033, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Novo7Advanced Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA702078 = {S16034, 145, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; de-de; ODYS-Xpress Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA702081 = {S16035, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; KESAM726HN Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA702244 = {S16036, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; M70014 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA703271 = {S16037, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; CAT NOVA Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA707388 = {S16038, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; STM912HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA709908 = {S16039, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; One Pad Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA710998 = {S16040, 145, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; imito am801 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA711093 = {S16041, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; iw-il; CRYM908HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA713281 = {S16042, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; M778 Tablet PC Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA715005 = {S16043, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Precedent Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA715909 = {S16044, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; QW TB-1010 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA715924 = {S16045, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; LUBM900HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA718482 = {S16046, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; CreativESense Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA722092 = {S16047, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; B63M Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA725884 = {S16048, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; de-de; INM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA730296 = {S16049, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; MAPLM912HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA732354 = {S16050, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; I10 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA737336 = {S16051, 163, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-L95G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA737882 = {S16052, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; KESAM726HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA738429 = {S16053, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SP-80 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA744079 = {S16054, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; sr-rs; ZTE roamer Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA744838 = {S16055, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; A20 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA745410 = {S16056, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; T-112 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA746416 = {S16057, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Evo Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA748851 = {S16058, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Incredible 2 HD Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA752069 = {S16059, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; ZTE-Tureis Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA754704 = {S16060, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; eTouch 702et Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA758923 = {S16061, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; M865 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA759306 = {S16062, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HERO Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA759657 = {S16063, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; el-gr; PLAIM900HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA762429 = {S16064, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo P70/S100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA763548 = {S16065, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HUMBLE 4.5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA764636 = {S16066, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; in-id; TV pad K3410 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA765546 = {S16067, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; it-it; DATAM722HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA766125 = {S16068, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; hu-hu; xTAB-70 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA775253 = {S16069, 63, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; 7461-CI1100 Build/GRJ22)*/
const Device UA776169 = {S16070, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; el-gr; IQM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA777892 = {S16071, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; X1 Build/HCL ME Tablet X1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA780781 = {S16072, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; generic_vortex Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA781408 = {S16073, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; CIUI Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA788372 = {S16074, 130, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fa-ir; G1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA799574 = {S16075, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; ZTE V960 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA804169 = {S16076, 120, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; 7260 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA822541 = {S16077, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-tw; eye1828 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA844859 = {S16078, 85, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - ZTE-TU960s_TD/1.0 Linux/2.6.35 Android/2.3 Release/9.25.2011 Browser/AppleWebKit533.1*/
const Device UA853897 = {S16079, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; B63M Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA853935 = {S16080, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; I700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA886289 = {S16081, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; M865 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA897210 = {S16082, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; MS-N7Y1 Build/MS-N7Y1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA900470 = {S16083, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Mi701 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1879944 = {S16084, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; MID73_SIS_GC2M Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1891919 = {S16085, 153, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Full Android on A91 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1895725 = {S16086, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; COWON D3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1898089 = {S16087, 152, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; pl-pl; Android for YG A700 Board Build/A700) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1900235 = {S16088, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; iw-il; CRYM912HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1900413 = {S16089, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; S42HW Build/HuaweiS42HW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1901330 = {S16090, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; taurus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1906505 = {S16091, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; iw-il; MOM726HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1908141 = {S16092, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; MS-N0Y1 Build/MS-N0Y1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1928428 = {S16093, 130, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; B7 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1931404 = {S16094, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; zh-cn; Newpad P9 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA1934131 = {S16095, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; EIS01PT Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1934501 = {S16096, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; COBY KYROS 7024 Build/GR144) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1936864 = {S16097, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6330VWBuild/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1937521 = {S16098, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PGX Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1937802 = {S16099, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; mfld_pr3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1937930 = {S16100, 150, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Bulletproof_Doubleshot Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1938279 = {S16101, 154, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Gingerbread on Dream Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1938778 = {S16102, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; blitzrs8 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1940587 = {S16103, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; N10020 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1941384 = {S16104, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; A710 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1941418 = {S16105, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Precedent Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1945054 = {S16106, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; Mega Taicho Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1945499 = {S16107, 160, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Hannspadd Build/GRK39F; chrisd1a1 gingerpadd) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1949988 = {S16108, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; GASEM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1950980 = {S16109, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; KUTBL07ANDC Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1955759 = {S16110, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; I-TAB Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA1956923 = {S16111, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-in; MMI-CP11 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1957455 = {S16112, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; A9100D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1958544 = {S16113, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; MTS-SP150 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1961822 = {S16114, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; id-id; SpeedUp S1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1963337 = {S16115, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; STM908HDZ Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1967456 = {S16116, 145, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Hello Tab Build/0.0.1747.0094) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1967783 = {S16117, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; in-id; NX-A899 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1969486 = {S16118, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; MITO 898 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1969605 = {S16119, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; BLUM732HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1971073 = {S16120, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15 Build/GRH55; kuyaDROID) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1971094 = {S16121, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; I04S-3G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1971894 = {S16122, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LDK-ICK v1.4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1972117 = {S16123, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MSM Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1974017 = {S16124, 159, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; edifi (v0005.07.03.02.en) Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1975338 = {S16125, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; A20 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1975825 = {S16126, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Acquire Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1975956 = {S16127, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; COWON Z2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1976175 = {S16128, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; Fly_IQ285 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1978919 = {S16129, 152, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; HIGHSCREEN Jet Duo Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1981765 = {S16130, 154, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; SWTGT-7250 Build/gingerbread_house_r62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1982957 = {S16131, 147, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MikTouched 4G Slide Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1983765 = {S16132, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; HTC Inspire 3D Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1986920 = {S16133, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; X10S Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1987849 = {S16134, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; ja-jp; MVPM726HN Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1990464 = {S16135, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; android Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1991115 = {S16136, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; GASEM900HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1992121 = {S16137, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; HW-N80W Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1994907 = {S16138, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; el-gr; MLS IQTALK Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1994920 = {S16139, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; MC701A Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995687 = {S16140, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ADR555 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018678 = {S16141, 56, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.2 Build/GINGERBREAD)*/
const Device UA2018683 = {S16142, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; UTV210 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018812 = {S16143, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; Zpad Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019047 = {S16144, 68, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; EM73_SIS_HY30_V2 Build/GRI40)*/
const Device UA2019053 = {S16145, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; pl-pl; 701 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019115 = {S16146, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; PC687 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019407 = {S16147, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; in-id; Axioo-VIGO410 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019478 = {S16148, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Vision Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019694 = {S16149, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Shakira Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019716 = {S16150, 150, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn;Lenovo A1-32AB0 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019826 = {S16151, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; a7272_sphone Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019888 = {S16152, 147, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HUMBLE by Danalo1979 4G Build/2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019970 = {S16153, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; TX72 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019973 = {S16154, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; MP727 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019993 = {S16155, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; UP-8786 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020020 = {S16156, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MTAB701 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020048 = {S16157, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Gin-Tazz Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020071 = {S16158, 154, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Samsung EPIC SPH-700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020116 = {S16159, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; CLIQ XT Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020198 = {S16160, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; M703C Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020248 = {S16161, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME811 Build/SHDWR_X7_2.67.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020254 = {S16162, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; A01HW Build/HuaweiS7-901w) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020259 = {S16163, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; M778 Tablet PC Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020302 = {S16164, 65, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; Novo7Advanced Build/GRJ22)*/
const Device UA2020495 = {S16165, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; 7D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020697 = {S16166, 157, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Gingerbread on Sapphire Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020780 = {S16167, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; G Harmony v2.7.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020796 = {S16168, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; 97U Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020842 = {S16169, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; I04 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020846 = {S16170, 147, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; tr-tr; Polypad C7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020870 = {S16171, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; android Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020875 = {S16172, 150, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn;Lenovo S2-38AH0 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2020900 = {S16173, 150, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Htcclay's SuperLite 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020978 = {S16174, 130, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; X2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021001 = {S16175, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Cruz L47 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021010 = {S16176, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; H5000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021047 = {S16177, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; msm7627_surf Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021435 = {S16178, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; md-us; disgo7000 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021573 = {S16179, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; nl-nl; 97CU Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021620 = {S16180, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; BB EH8 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2021692 = {S16181, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; th-th; Gpad Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021726 = {S16182, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; SH8288U Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021802 = {S16183, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; pl-pl; TAB S73 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021866 = {S16184, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; el-gr; Passepartout 70 C124 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021906 = {S16185, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; ru-ru; DNSM912HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021909 = {S16186, 149, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; Geeksphone ZERO Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021935 = {S16187, 127, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Z510d) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021943 = {S16188, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; tr-tr; EFTAM722HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022107 = {S16189, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Octa Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022302 = {S16190, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; OnePAD 720 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022314 = {S16191, 121, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; T1301 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022315 = {S16192, 184, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; ETOOS 700C Build/R001(V0.17-JT-V961-113.GINGERBREAD.eng.chenxiang.X)) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022406 = {S16193, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; CA017AAC0G/ABC0G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022430 = {S16194, 128, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; zh-cn; Newpad Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022521 = {S16195, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME811 Build/SHDWR_X7_2.67.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022547 = {S16196, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; pl-pl; GOCLM908HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022575 = {S16197, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; iw-il; CRYM732HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022645 = {S16198, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; I100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022834 = {S16199, 184, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; openAOS Build of Android 2.3 (cyanogenmod edition) Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022854 = {S16200, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; zh-tw; POM726HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022892 = {S16201, 58, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; m799ca Build/GRI40)*/
const Device UA2022950 = {S16202, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; SH8298U Build/S8041) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023078 = {S16203, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; msm7627a_sku3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023100 = {S16204, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; H7-3G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023156 = {S16205, 60, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; CAT NOVA Build/GRI40)*/
const Device UA2023192 = {S16206, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; zh-cn; Newpad Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2023246 = {S16207, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; KATBL07AND Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023266 = {S16208, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; A10-MID Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023279 = {S16209, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; FT701 Build/FT701-005) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023392 = {S16210, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; EBEST W50L Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023528 = {S16211, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; cs-cz; GOCLM722HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023600 = {S16212, 130, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; V8 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023638 = {S16213, 56, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; A710 Build/GRJ22)*/
const Device UA2023692 = {S16214, 82, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.1; Full Android on SMDKV210 Build/GINGERBREAD)*/
const Device UA2023705 = {S16215, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; QB7211 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023775 = {S16216, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; FX2-PAD10 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023814 = {S16217, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; GIONEE GN106 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023883 = {S16218, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; SWT_GT_7205 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023985 = {S16219, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; nuoyaxin Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024002 = {S16220, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; NOVO7 Advanced Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024072 = {S16221, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GOOAPPLE V5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024144 = {S16222, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; GM800TW Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024156 = {S16223, 127, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; A301C Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2024179 = {S16224, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; 702et Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024321 = {S16225, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ChangHong V7 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024338 = {S16226, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; imx51_bbg Build/TouchSquid) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024428 = {S16227, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; minitab Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024470 = {S16228, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; world Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024471 = {S16229, 151, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HOSIN_U3 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024528 = {S16230, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; 898 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024545 = {S16231, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; GIONEE GN205 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024635 = {S16232, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; IQ240 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024761 = {S16233, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; TCL_A966 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024810 = {S16234, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Bifer A10 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024872 = {S16235, 157, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; Infinity Build/V0.19-QM.CPT-V961.114-GB-X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024876 = {S16236, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; ELIYA i5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024883 = {S16237, 147, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; Woxter Tablet PC 75 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024902 = {S16238, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; FIH-FB0 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024941 = {S16239, 126, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; EBEST W50L Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2024966 = {S16240, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; 7461-CI1100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024998 = {S16241, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; id-id; NX A7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024975 = {S16242, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; A10t(5DM3) Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025090 = {S16243, 159, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; edifi (v0004.07.03.02.en) Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025219 = {S16244, 151, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; VITA_V3S Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025263 = {S16245, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; A66 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025386 = {S16246, 56, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; 97FC Build/GRJ22)*/
const Device UA2025389 = {S16247, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; V13 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025445 = {S16248, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; tr-tr; ATIM908HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025483 = {S16249, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HUAWEI T8828 Build/HuaweiT8828) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2025514 = {S16250, 158, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo Lenovo A1-32AB0 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2025577 = {S16251, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; zh-cn; PAD Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025622 = {S16252, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; MP705C Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025637 = {S16253, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; a7272_sphone Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025650 = {S16254, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC-Z510d/1.16.1401.3) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025711 = {S16255, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; A106 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025718 = {S16256, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; KPT A9 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025741 = {S16257, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; zh-cn; Newpad Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025816 = {S16258, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; Changjiang P5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025837 = {S16259, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; A05 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025932 = {S16260, 127, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Q9 Build/Q9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025938 = {S16261, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; NEC-101T Build/X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025971 = {S16262, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; GIONEE GN109 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026095 = {S16263, 149, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; Q07CL01 Build/v00.12.04_Q07_CRA_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026115 = {S16264, 154, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; SWTGT-7250 Build/gingerbread_house_r58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026123 = {S16265, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; tetc Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026132 = {S16266, 149, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC-Z510d/1.13.1401.0 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026210 = {S16267, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; KUTBL08ANDC Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026343 = {S16268, 120, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; H701 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2026471 = {S16269, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; M019 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026535 = {S16270, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; IS701R Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026556 = {S16271, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; de-de; INM803HCZ Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026676 = {S16272, 145, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LD-ICS v1.2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026734 = {S16273, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; ru-ru; MVM900HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026752 = {S16274, 151, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; in-id; NX-A895 Build/ALPS.GB.FDD2.MP.V2.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026864 = {S16275, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ThL V7 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026898 = {S16276, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; mobiletek13_gb Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026979 = {S16277, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; H701 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026999 = {S16278, 130, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Z3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027119 = {S16279, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; SF68 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027128 = {S16280, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; tr-tr; EFTAM722HR Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027243 = {S16281, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; DOOV_D3 Build/DoovDOOV_D3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027360 = {S16282, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; ru-ru; MVM908HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027431 = {S16283, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; LT8029 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027553 = {S16284, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; Kyivstar Spark Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027600 = {S16285, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; B63 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027672 = {S16286, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DoubleShot Lite Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027682 = {S16287, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; CM008 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027680 = {S16288, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; H7-3G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027723 = {S16289, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A610 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027835 = {S16290, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; in-id; PTM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027849 = {S16291, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ar-eg; IZOOM-I8002 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027890 = {S16292, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; zh-cn; ARMM3V Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027948 = {S16293, 129, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; DOOV_D3 Build/DoovDOOV_D3) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2028008 = {S16294, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; Multilaser Diamond Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028017 = {S16295, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; zh-cn; M16 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028066 = {S16296, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT9326 Build/W326) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028187 = {S16297, 125, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; DOOV_D770 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2028223 = {S16298, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; I9350 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028232 = {S16299, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-AS840 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028462 = {S16300, 126, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GN205 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2028539 = {S16301, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; H700 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028577 = {S16302, 147, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; A006 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028584 = {S16303, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Mi400 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028588 = {S16304, 130, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; U9 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028780 = {S16305, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; XFactor Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029123 = {S16306, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; FWS610_EU Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029154 = {S16307, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; tr-tr; M778 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029209 = {S16308, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ko-kr; ITP-S110WD Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029359 = {S16309, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; W1350 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029451 = {S16310, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; G7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029553 = {S16311, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; H700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2029616 = {S16312, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; msm7627_sku5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029606 = {S16313, 162, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Full Android on x86 Emulator Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029636 = {S16314, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; th-th; 697 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029755 = {S16315, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; I04S Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029770 = {S16316, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; Mtag 352 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029832 = {S16317, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; C215 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029983 = {S16318, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ETON T800 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030007 = {S16319, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Philips W635 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030085 = {S16320, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Z3Z6Z7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030185 = {S16321, 154, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; SWTGT-7250 Build/gingerbread_house_r75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030192 = {S16322, 130, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ro-ro; P3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030294 = {S16323, 165, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; iXperiA 8s Build/iXperiA 8s v1.7; iXperiA 8s v1.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030329 = {S16324, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; T-111 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030509 = {S16325, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; l97D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030707 = {S16326, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; A301C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030823 = {S16327, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; Wxt Tab PC Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030980 = {S16328, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; FJJB091 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031156 = {S16329, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Z3A Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031195 = {S16330, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; ja-jp; MVPM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031449 = {S16331, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; H16 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031586 = {S16332, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; MS-N7Y1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031644 = {S16333, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; tr-tr; E-TAB Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031686 = {S16334, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; SUNVEK Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031918 = {S16335, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; es-es; CasiaTAB8 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031958 = {S16336, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-tw; APD-B705CC Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031961 = {S16337, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; 18FR Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032034 = {S16338, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; PAPYRE pad 712 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032043 = {S16339, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; M88 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032057 = {S16340, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; zh-tw; moii E598 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032105 = {S16341, 151, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Full Android on AD1000A Build/00.06) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032163 = {S16342, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; S820 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032230 = {S16343, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; FT701 Build/FT701-004) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032269 = {S16344, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE-N960/N960V1.0.0B03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032321 = {S16345, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; Axioo-VIGO410 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032532 = {S16346, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo A1-32AB0 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2032605 = {S16347, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; D23 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032632 = {S16348, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; M701 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032673 = {S16349, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; 4GS Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032799 = {S16350, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; TAB10-06 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032886 = {S16351, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; AS3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032979 = {S16352, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; OBEE-OS805 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032996 = {S16353, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; de-at; SLPad PA810G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2033012 = {S16354, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PAD7 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033088 = {S16355, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; ru-ru; BLADM722HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033352 = {S16356, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MORAL T16 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033373 = {S16357, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A621 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033525 = {S16358, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; A70HT3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033637 = {S16359, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; G20 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033652 = {S16360, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Philips_PI5000 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033655 = {S16361, 154, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; CushZero Build/FRG83G; zero.sholes.X.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033660 = {S16362, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; T1301 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033782 = {S16363, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ChangHongW3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033799 = {S16364, 125, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; W808 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2033992 = {S16365, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; TAB-7C-4GW-1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034084 = {S16366, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ar-eg; izoom-i8001 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034114 = {S16367, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Phone 4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034228 = {S16368, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; M730W Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034241 = {S16369, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; ru-ru; MVM900HWZ Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034311 = {S16370, 56, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; 7260 Build/GRJ90)*/
const Device UA2034392 = {S16371, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; wimo wf Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034400 = {S16372, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TCL A986 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034420 = {S16373, 145, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; MW0821_V2.0 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034537 = {S16374, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LT-722/M72T2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034545 = {S16375, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; TAB420 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034612 = {S16376, 155, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; HB-100 DASAN Networks, Inc. Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2167044 = {S16377, 158, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-ch; Orange Dublin Build/GB_P752V_OCHV1.0.13B04) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2221412 = {S16378, 151, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; MT791 Build/GINGERBREAD.b04MT1ref.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2222915 = {S16379, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hd-us; Andi4d Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2222983 = {S16380, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; C110 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2223431 = {S16381, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; GIONEE GN100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2223460 = {S16382, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; A8T Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2223536 = {S16383, 125, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8022 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2224922 = {S16384, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Precedent Build/AreaRom Test) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2227086 = {S16385, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; IMO X2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2227635 = {S16386, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; W5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2228533 = {S16387, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; ATLAS W Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2238801 = {S16388, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; 56021 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2242283 = {S16389, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Discovery Linux/2.6.35 Android/2.3.6 Release/07.30.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1*/
const Device UA2242321 = {S16390, 153, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - HUAWEI_T8828_TD/1.0 OPhone/2.6 (Linux; Android 2.3.7) Release/03.31.2012 Browser/WAP2.0 (AppleWebKit/533.1)Mobile Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2256928 = {S16391, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; R5830i Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2256933 = {S16392, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; el-gr; IQTALK Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2256955 = {S16393, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HOSIN_U2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2256956 = {S16394, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Skate4.3 H Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257074 = {S16395, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; AQ_4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257133 = {S16396, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; EverSmart Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257182 = {S16397, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; de-de; 970 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257189 = {S16398, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; USMART SL55 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2258446 = {S16399, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LT170 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2309973 = {S16400, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ChangHong-Z-ME Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2317724 = {S16401, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Sc0r3d Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2320330 = {S16402, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DRD-I Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2320558 = {S16403, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Zigo Eon 53 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2332538 = {S16404, 151, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MSI_Enjoy_10_Plus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2334581 = {S16405, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; G Harmony v2.8 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2334719 = {S16406, 32, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Pandora/1.5.18 Android/2.3.4 inc*/
const Device UA2335279 = {S16407, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; i50 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2337435 = {S16408, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; H500 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2337717 = {S16409, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; m7800h Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2337975 = {S16410, 149, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Changhong H5018 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2338600 = {S16411, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; DOOV_D2 Build/DOOV) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2339170 = {S16412, 154, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - MOT-MT680_TD/1.0 Android/2.3 (Linux; Android 2.3) Release/04.13.2012 Browser/WAP 2.0 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2339414 = {S16413, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; gaophone13_gb Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2339704 = {S16414, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; tr-tr; Parrot Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2340040 = {S16415, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; hu-hu; Atlas W Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2340182 = {S16416, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; V881 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2342111 = {S16417, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; magcomm73_gb Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2342357 = {S16418, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; nine i7400 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2343385 = {S16419, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-pt; GP-351 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2348759 = {S16420, 145, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; MW0821_V2.0 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2351951 = {S16421, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; IMO S78 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2352833 = {S16422, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; ATB-1108 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2356277 = {S16423, 32, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Pandora/1.5.17 Android/2.3.4 inc*/
const Device UA2410720 = {S16424, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Spice Mi 415 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2411284 = {S16425, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; MS-N0Y1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2412307 = {S16426, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; de-de; techTAB Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2412626 = {S16427, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; EBEST W58 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2412751 = {S16428, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; N60 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2412757 = {S16429, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; TAB360 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2412782 = {S16430, 126, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GN106 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2413401 = {S16431, 130, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Z510d Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2413956 = {S16432, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; M910A Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2413976 = {S16433, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; W2400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2414515 = {S16434, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; NB 2012 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2415269 = {S16435, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; Athena X2 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2418079 = {S16436, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A621W Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2418600 = {S16437, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Android_904 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2421369 = {S16438, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; S1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2421753 = {S16439, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.9; en-us; HAPPLE Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2422013 = {S16440, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GS1 Build/MocorDroid2.3.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2422754 = {S16441, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A651 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2423538 = {S16442, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; U880 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2424596 = {S16443, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Walton_Primo Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2439980 = {S16444, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; TCL_A919 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2441432 = {S16445, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; TCL_W939 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2442799 = {S16446, 117, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; YL-Coolpad_9900/2.3.018.P1.X.9900; 480*960; CTC/2.0) CoolpadWebkit/533.1*/
const Device UA2443074 = {S16447, 126, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MT680 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2443840 = {S16448, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; A109 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2443939 = {S16449, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Changjiang Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2444647 = {S16450, 125, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; V880 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2444732 = {S16451, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; N505 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2444849 = {S16452, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; 510_v2.3.8 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2445258 = {S16453, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; GPLUS-Z500 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2445289 = {S16454, 151, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ru-ru; MTC-962 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 SVN/01004*/
const Device UA2445561 = {S16455, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; B700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2446693 = {S16456, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HEDY_H701 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2452634 = {S16457, 155, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; iXperiA 8s Build/iXperiA 8s; iXperiA 8S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2455720 = {S16458, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Xfactor_mini Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2455886 = {S16459, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; in-id; A10GBM73 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2466801 = {S16460, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; JT Smart Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2468345 = {S16461, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - HUAWEI-HUAWEI-Y-220T/1.0 Linux/2.6.35.7 Android/2.3.5 Release/11.13.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2473813 = {S16462, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GOOAPPLE V5-3G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2474119 = {S16463, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; ru-ru; Q3M732HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2474220 = {S16464, 225, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; zh-cn; HTCA6390_TD/1.0 Android/2.3 release/4.1.2011 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2474777 = {S16465, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.9; it-it; B63M Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2475630 = {S16466, 149, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ALCATEL_one_touch_979 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2475676 = {S16467, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; BTPC-2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2475699 = {S16468, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; dkb Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2476398 = {S16469, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; DOOV_D3 Build/DoovDOOV_D3) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2476672 = {S16470, 156, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; USmart_SL88 Build/USmart_SL88USmart_SL88) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2479474 = {S16471, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; signal75_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2479690 = {S16472, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SS706 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2479973 = {S16473, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A66   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2480089 = {S16474, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; KS306 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2481257 = {S16475, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; B50 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2481267 = {S16476, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; B601 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2482044 = {S16477, 151, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-za; MOBICEL TITAN Build/MocorDroid2.3.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2483242 = {S16478, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; X40 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2483890 = {S16479, 129, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - WINGTECH-INCO/1.0 Linux/2.6.35.7 Android/2.3.5 Release/11.03.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2485172 = {S16480, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; U670C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2494565 = {S16481, 145, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; Qumo DotCom Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2494854 = {S16482, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; GIONEE GN210 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2494994 = {S16483, 158, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo Lenovo S2-38AH0 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2496181 = {S16484, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; BA014AAS0G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2496316 = {S16485, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I339 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2496837 = {S16486, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; Uno Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2497015 = {S16487, 83, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - ZTEU788_TD/1.0 Linux/2.6.35 Android/2.3 Release/12.25.2011 Browser/AppleWebKit533.1*/
const Device UA2498187 = {S16488, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; COWON Q7 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2500358 = {S16489, 116, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - V6500_ChinaUnicom/1.0 Linux/2.6.38.6(Android 2.3.6;) Release/08.06.2012 Browser/AppleWebKit533.1 Mobile Safari/533.1*/
const Device UA2501846 = {S16490, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; I9400NM Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2502215 = {S16491, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; I9400 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2502641 = {S16492, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; AL-6 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2502867 = {S16493, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ZEN U1 Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2503592 = {S16494, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; benrui15_6626_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2503695 = {S16495, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.9; en-us; G-TiDE Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2504804 = {S16496, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; BASE_Varia Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2505065 = {S16497, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; X35 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2505070 = {S16498, 129, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-yuanpeng/1.0 Linux/2.6.35.7 Android/2.3.5 Release/11.09.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2506154 = {S16499, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; Polytron W1320 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2518395 = {S16500, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; ADR21 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2518606 = {S16501, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; in-id; froyo_f698u202a Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2519173 = {S16502, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; ru-ru; MVM908HWZ Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2519941 = {S16503, 130, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Z1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2520292 = {S16504, 125, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8180 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2520301 = {S16505, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; imx53_smd Build/R10.3.3-r1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2520350 = {S16506, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; V850 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2520444 = {S16507, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Z3_Z6_Z7 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2520504 = {S16508, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; KPT A88 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2520549 = {S16509, 147, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MORAL_N01 Build/HuaqinMORAL_N01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2520651 = {S16510, 114, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; YL-Coolpad_5880/2.3.011.X.5880; 800*480; CTC/2.0) CoolpadWebkit/533.1*/
const Device UA2520825 = {S16511, 185, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; Lanix IliumS50Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LNX-Android-MMS-V1.1/1.2*/
const Device UA2520952 = {S16512, 153, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; XD-S6000 Build/msm7627a_sku1-eng2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2521110 = {S16513, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; MegaFon SP-A5 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2523817 = {S16514, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; AX-340 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2523968 = {S16515, 152, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ICE MOBILE Galaxy Prime Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2524226 = {S16516, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; Mtag 281 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2524273 = {S16517, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - KONKA_V926_TD/1.0 Android/2.3.5 MocorDroid/SpreadTrum Release/3.22.2012 Browser/AppleWebKit5333.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2525350 = {S16518, 158, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; sk-sk; Orange Dublin Build/GB_P752V_OSKV1.0.13B05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2525709 = {S16519, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; AND1  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2526430 = {S16520, 91, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - CoolPad8060_CMCC_TD/1.0 Linux/2.6.35 Android/2.3 Release/8.30.2012 Browser/AppleWebkit533.1*/
const Device UA2526479 = {S16521, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; i5830 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2527199 = {S16522, 145, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; MOBO BT Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2527730 = {S16523, 124, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android 2.3.6;zh-tw;E650 Build/GRK39F)AppleWebkit/533.1(KHTML,like Gecko)Version/4.0 Mobile Safari/533.1*/
const Device UA2542834 = {S16524, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GN205 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2544096 = {S16525, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; FireBird Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2544400 = {S16526, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; imx51_bbg Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2545250 = {S16527, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; A66 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2545625 = {S16528, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; mx1002 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2545654 = {S16529, 129, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MAC B200/V400) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2545685 = {S16530, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GN106 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2545756 = {S16531, 126, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT536 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2545861 = {S16532, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; id-id; Q7079W Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2546022 = {S16533, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LT S800 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2546023 = {S16534, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; PULID F7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2546182 = {S16535, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 5010 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2546405 = {S16536, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HUAWEI T8828 Build/HuaweiT8828) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2547069 = {S16537, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; pl-pl; MID02 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2547362 = {S16538, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U8667 Build/HuaweiU8667) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2547625 = {S16539, 130, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; U6 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2547750 = {S16540, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8022 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2547800 = {S16541, 172, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Android 2.3 GingerBread; it-it; Android for AVANTIS WEBBY5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2547879 = {S16542, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; MID8128 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2548218 = {S16543, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; pt-br; TAB 800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2548413 = {S16544, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Android Smart Phone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2548597 = {S16545, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; G_7C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2548885 = {S16546, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-mx; Andy 3G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2549253 = {S16547, 129, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE U885 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2549817 = {S16548, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; DOOV_D600 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2550670 = {S16549, 82, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - ZTEU885_TD/1.0 Linux/2.6.35 Android/2.3 Release/2.25.2012 Browser/AppleWebKit533.1*/
const Device UA2550973 = {S16550, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; verykool S757 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2551228 = {S16551, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; epade A360S Build/alpsepade) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2552579 = {S16552, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; tr-tr; U8816 Build/HuaweiU8816) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2552583 = {S16553, 145, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; Movistar Express Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2552787 = {S16554, 150, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; ALCATEL ONE TOUCH 928D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2553115 = {S16555, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ms-my; MI360 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2553488 = {S16556, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; iBall 3G 7325 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2553890 = {S16557, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; C208 Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2554032 = {S16558, 150, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; GT7250S Build/gingerbread_qrd_r209) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2554335 = {S16559, 158, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; sk-sk; Orange Dublin Build/GB_P752V_OSKV1.0.13B06) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2554412 = {S16560, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; n903e Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2554888 = {S16561, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A702_Andy Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2555122 = {S16562, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - HUAWEI-HUAWEI-Y-220T/1.0 Linux/2.6.35.7 Android/2.3.5 Release/11.01.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2555367 = {S16563, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; P1 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2555383 = {S16564, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; X501_USA_RS Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2555486 = {S16565, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; AND1   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2556032 = {S16566, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - HUAWEI-HUAWEI-Y-220T/1.0 Linux/2.6.35.7 Android/2.3.5 Release/12.15.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2556077 = {S16567, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - HUAWEI-HUAWEI-Y-220T/1.0 Linux/2.6.35.7 Android/2.3.5 Release/11.28.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2556231 = {S16568, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; AQ 4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2556574 = {S16569, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - null Linux/2.6.35 Android/4.1.2 Release/null Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1 Android 2.3.5;*/
const Device UA2556981 = {S16570, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; C3100 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2557513 = {S16571, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; G3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2558112 = {S16572, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; msm7627a_i300 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2574293 = {S16573, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2574339 = {S16574, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Zedd Clan Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2574391 = {S16575, 149, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; Bliss A70 Build/LCAQF20.1.0_W1230) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2574468 = {S16576, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; VINCI Tab II VS-3001 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2574644 = {S16577, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; DOOV_D770 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2574707 = {S16578, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; GIONEE GN777 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2574863 = {S16579, 149, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - CoolPad8026_CMCC_TD/1.0 Linux/2.6.32 Android/2.3 Release/03.15.2012 Browser/WAP2.0 (AppleWebKit/533.1) Mobile Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2576593 = {S16580, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A7013 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2577200 = {S16581, 114, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_5110/2.3.007.X.5110; 320*480; CTC/2.0) CoolpadWebkit/533.1*/
const Device UA2577215 = {S16582, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HT-I617 Build/MocorDroid2.3.5) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2577632 = {S16583, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Lephone_TD8208_TD/1.0 Android/2.3.5 MocorDroid/SpreadTrum Release/3.22.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2578547 = {S16584, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; RPSPG3501 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2579031 = {S16585, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PLUS Pro Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2579609 = {S16586, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; A20 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2579953 = {S16587, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; SPC Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2580294 = {S16588, 124, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-B51/1.0 Linux/2.6.35.7 Android/2.3.6 Release/12.29.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2580303 = {S16589, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - HUAWEI-HUAWEI-Y-220T/1.0 Linux/2.6.35.7 Android/2.3.5 Release/12.25.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2588189 = {S16590, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; LINGWIN U820S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2588555 = {S16591, 208, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - MOT-XT536/1.0 Android/2.3.7 Release/2.28.2012 Browser/AppleWebKit533.1 Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MOT-XT536 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2589451 = {S16592, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; W808 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2589587 = {S16593, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Mi-A303 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2589627 = {S16594, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; ru-ru; AP-801 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2589883 = {S16595, 162, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HC-HE-N620E/HE-N620E-S003-CT; 320*480; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2589946 = {S16596, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GN777 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2590213 = {S16597, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; IQ236 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2590431 = {S16598, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Bambook S1 Build/EA1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2591892 = {S16599, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; cs-cz; TA7200 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2592405 = {S16600, 152, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Oplus A16 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2592858 = {S16601, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; E5680 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2593766 = {S16602, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Epade S3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2594005 = {S16603, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; A7500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2594191 = {S16604, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE N790 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2594291 = {S16605, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-RU; PAP4000 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2594421 = {S16606, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A326/S100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2594935 = {S16607, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Z9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2595408 = {S16608, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; B710 Build/2.0.1083_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2595613 = {S16609, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DTC Build/UNKNOWN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2595711 = {S16610, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; Vega x Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2596172 = {S16611, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/2.3.5 Release/11.09.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2596760 = {S16612, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; YPY_S350 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2596847 = {S16613, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; EverStar Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2598167 = {S16614, 157, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; TCL-D662/EG27_FLID_V0.8; 480*800; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2599083 = {S16615, 130, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-Byond-B54/1.0 Linux/2.6.35.7 Android/2.3.6 Release/01.07.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2599760 = {S16616, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; Slim Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2600232 = {S16617, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; 31TL04 Build/01.03.000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2600677 = {S16618, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPARK Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2601758 = {S16619, 93, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-US;) AppleWebKit/999+ (KHTML, like Gecko) Safari/999.9*/
const Device UA2640621 = {S16620, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; W6800 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2641108 = {S16621, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SunPhone-G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2641340 = {S16622, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-gb; S16 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2641348 = {S16623, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; KIRA N10021 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2641449 = {S16624, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo S2-38AH0 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2641516 = {S16625, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DB50 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2642458 = {S16626, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DIGICEL Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2642782 = {S16627, 113, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - T-smart D08_TD/1.0 Android/2.3 Release/3.20.2011 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2643003 = {S16628, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE-T U812 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2643131 = {S16629, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; TCL A919 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2643241 = {S16630, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SunTab-G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2643332 = {S16631, 167, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; MBX DVBT reference board (c03ref) Build/3010B.X.V28) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2643414 = {S16632, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; M900HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2643786 = {S16633, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; G5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2643796 = {S16634, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo A1-32AB0 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2643857 = {S16635, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; KSP8000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2644008 = {S16636, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; W58SX Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2644129 = {S16637, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; EBEST V6 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2644194 = {S16638, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Mytel Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2644423 = {S16639, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; RSD-V1 Build/alpsRSD-V1) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2644562 = {S16640, 153, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; V880 Build/GWK74; MoKeeOS-v0.3.Xalpha) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2644681 = {S16641, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ar-eg; imden A9106 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2645667 = {S16642, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; TCLW989 Build/REL_2.3.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2645702 = {S16643, 36, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - KakaoTalkAndroid/3.3.3 Android/2.3.4*/
const Device UA2646365 = {S16644, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; th-th; GT0001ALTH Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2647240 = {S16645, 161, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; CHL-S3000/Hualu_S3000_V1.0; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2650060 = {S16646, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; JC-S9220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2650841 = {S16647, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; T9987 Build/MocorDroid2.3.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2650904 = {S16648, 125, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8020 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2650934 = {S16649, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; Etab 0208 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2651205 = {S16650, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; ZTE V881 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2651811 = {S16651, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; epade A360S Build/alpsepade) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652049 = {S16652, 145, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HT-I617 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652266 = {S16653, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MID-Q5 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652332 = {S16654, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ar-eg; UR-A38 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652358 = {S16655, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; i66 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652458 = {S16656, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; gb92236a Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652463 = {S16657, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; C7500 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652472 = {S16658, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; imx51_bbg Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652650 = {S16659, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPSSON iMO1000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652732 = {S16660, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ru-ru; SP-A20i Build/MF_GB_01.02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652954 = {S16661, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; YT521 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2653204 = {S16662, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; Helsinki Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2653869 = {S16663, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; LUX Build/LUX01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2653895 = {S16664, 89, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - MASTONE G3_TD/1.0 Android/2.3 Release/4.20.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0*/
const Device UA2654142 = {S16665, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; RPSPB4001 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2654328 = {S16666, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; W381 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2654766 = {S16667, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad 5216 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2655074 = {S16668, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; nl-nl; VALENCIA Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2656026 = {S16669, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-tw; eye1828 Build/R10.3.2_3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2656819 = {S16670, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE N790S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2657274 = {S16671, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; W05 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2657991 = {S16672, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; S7054 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2658082 = {S16673, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Ofone Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2659087 = {S16674, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; PROA680 Build/V1.0.09) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2659302 = {S16675, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; AirTab M70G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2659585 = {S16676, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SP100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2660318 = {S16677, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; Star TV Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2660960 = {S16678, 96, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Telego_G902_TD/G902_V2.00 Linux/2.6.35 Android/2.3.5 Release/07.14.2012 Browser/AppleWebKit533.1*/
const Device UA2661244 = {S16679, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; X350 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2661398 = {S16680, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; KDI-i360   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2661956 = {S16681, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - CoolPad8070_CMCC_TD/1.08 Linux/2.6.35 Android/2.3.5 Release/03.29.2012 Mozilla/5.0 AppleWebKit/533.1 Version/4.0 Mobile Safari/533.1*/
const Device UA2662309 = {S16682, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A69 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2663210 = {S16683, 125, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-B51+/1.0 Linux/2.6.35.7 Android/2.3.6 Release/01.25.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2664504 = {S16684, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; el-gr; iQTalk Crystal Build/V1.0.27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2665780 = {S16685, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; SVP Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2666604 = {S16686, 129, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-yuanpeng/1.0 Linux/2.6.35.7 Android/2.3.5 Release/01.21.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2666984 = {S16687, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SL-S1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2667080 = {S16688, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - HUAWEI-HUAWEI-Y-220T/1.0 Linux/2.6.35.7 Android/2.3.5 Release/01.16.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2668922 = {S16689, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; W105 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2682731 = {S16690, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; TR719 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2682960 = {S16691, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; sdkDemo Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2683374 = {S16692, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; TAB-PROTAB2XL Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2683939 = {S16693, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; acsip13_gb Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2683997 = {S16694, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; EBEST N4 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2684180 = {S16695, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; E one Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2684383 = {S16696, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; Ypy 7 - TB07STA Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2684632 = {S16697, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; G.fone G1345 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2684697 = {S16698, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; I9000 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2684705 = {S16699, 160, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ALCATEL_one_touch_979_HelloKitty Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2685020 = {S16700, 178, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 (Mobile; afma-sdk-a-v4.1.1)*/
const Device UA2685060 = {S16701, 172, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Android 2.3 GingerBread; it-it; Android for AVANTIS WEBBY5 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2685827 = {S16702, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; XT760 Build/02.3C.08RPS) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2685883 = {S16703, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; E5838 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2685971 = {S16704, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; 9000H Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2685975 = {S16705, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-EG870 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2686179 = {S16706, 130, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Android 2.3.5;zh-cn;GO M2 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2686192 = {S16707, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BA014AAM0G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2686361 = {S16708, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; E909B+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2686369 = {S16709, 129, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HOSIN_U2 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2686413 = {S16710, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Bambook S1 Build/EA1.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2686441 = {S16711, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; leepoo i500 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2686456 = {S16712, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; K-Touch E619 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2686499 = {S16713, 84, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - ZTEU960s2_TD/1.0 Linux/2.6.35 Android/2.3 Release/3.21.2012 Browser/AppleWebKit533.1*/
const Device UA2686642 = {S16714, 125, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.7; zh-cn; 8022 Build GWK74) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2686676 = {S16715, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; G'HONGT628 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2686848 = {S16716, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; moii-E796 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2686851 = {S16717, 152, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-XT553 Build/AGSPIN_B_02.01.35RPD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2686865 = {S16718, 152, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-XT553 Build/AGSPIN_B_02.01.38RPD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2686893 = {S16719, 151, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; AMOI_N89 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2687374 = {S16720, 85, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - ZTE-TU805_TD/1.0 Linux/2.6.35 Android/2.3 Release/10.08.2011 Browser/AppleWebKit533.1*/
const Device UA2687950 = {S16721, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; JC-A107 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2688454 = {S16722, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; D118 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2689913 = {S16723, 161, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BRS-BROR W60/BROR_W60_V1.0; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2690133 = {S16724, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SIENDON P1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2690814 = {S16725, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; sv-se; Doro PhoneEasy 740 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2690956 = {S16726, 145, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; M.T.T. Tablet Build/R10.3.2_3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2691766 = {S16727, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Seals TS3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2692351 = {S16728, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XLTEL X800 Build/MocorDroid2.3.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2693387 = {S16729, 162, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Cydle M7 (v0005.07.03.01.en) Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2693430 = {S16730, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Android Multimedia  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2693525 = {S16731, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; 4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2693687 = {S16732, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; MITO T100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2693999 = {S16733, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; NODIS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2694440 = {S16734, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; EF68 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2695309 = {S16735, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; AX525 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2695398 = {S16736, 58, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; U670C Build/GRK39F)*/
const Device UA2695534 = {S16737, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SunTab-C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2697675 = {S16738, 149, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Lephone_C03 Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2698687 = {S16739, 157, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; BERAQ NB-1000 Build/VENUS_00.03.165.H1096) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2699073 = {S16740, 150, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LNV-Lenovo_A365e Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2699868 = {S16741, 150, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB626 Build/5.5.1Q-172-DEFY2-CCI-6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2702371 = {S16742, 175, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Haier_HT-I600_TD/I600_MocorDroid2.2_W11.41_V1.00 Release/06.26.2012 Mozilla/5.0 (Linux; U; Android 2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2704956 = {S16743, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; 31TL04 Build/01.02.000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2706899 = {S16744, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MAGNUS FANTASY G3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2707676 = {S16745, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; Dublin Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2708278 = {S16746, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; S900 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2717903 = {S16747, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; V8110 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2721161 = {S16748, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; tr-tr; NEOI 588-734 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2721391 = {S16749, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; Gvon 990 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2721700 = {S16750, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME811 Build/SHDWR_X7_2.74.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2722095 = {S16751, 130, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; X1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2722360 = {S16752, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; TCL A968 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2722378 = {S16753, 149, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; HTC_IncredibleE_S715e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2722525 = {S16754, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-za; ZTE V10 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2722641 = {S16755, 96, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SHP-SH330T/V03.03; 320*480; CTC/2.0) eZBrowser/1.00*/
const Device UA2722749 = {S16756, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPSSON iMO850 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2723057 = {S16757, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; 610 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2723327 = {S16758, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Haier HW-N86W Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2723746 = {S16759, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ANTAI Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2723989 = {S16760, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; N620 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2724463 = {S16761, 38, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4)*/
const Device UA2724946 = {S16762, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; CS40G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2725564 = {S16763, 168, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BW-BenWee 5100S/BenWee_5100S_V1.1; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2726569 = {S16764, 149, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Binatone iHomePhone 3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2726944 = {S16765, 168, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GIO-GiONEE_C600/CBD75B04_01_T9161 ; 320*480; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2726968 = {S16766, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; maxwest Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2727083 = {S16767, 151, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 5.5.5(Android 2.3.6); en-us; MC919LL Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2727149 = {S16768, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; H943 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2727180 = {S16769, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; Athena Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2727483 = {S16770, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; tr-tr; MC603CH Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2727645 = {S16771, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; N855D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2728179 = {S16772, 149, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; YL-Coolpad 5216 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2728205 = {S16773, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - KONKA_V913_TD/1.0 Release/7.13.2012 Mozilla/5.0(Linux;U;Android 2.3.5) AppleWebKit/533.1(KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2729227 = {S16774, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; WING W35 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2729376 = {S16775, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; M1507 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2729757 = {S16776, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; xpndr_imh3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2729811 = {S16777, 168, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BW-BenWee 5100S/BenWee_5100S_V1.5; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2731544 = {S16778, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; PROA680 Build/V1.0.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2731631 = {S16779, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Malata I5 Build/MalataI5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2732412 = {S16780, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; S30 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2733112 = {S16781, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; SM55 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Versao/4.0 Mobile Safari/533.1*/
const Device UA2733497 = {S16782, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 5211 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2734618 = {S16783, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; L1-Forst Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2734772 = {S16784, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; P8 Build/MocorDroid2.3.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2737577 = {S16785, 126, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-S7100/1.0 Linux/2.6.35.7 Android/2.3.5 Release/04.01.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2742670 = {S16786, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; I60M Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2742803 = {S16787, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; gaoxinqi Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2742839 = {S16788, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; B920 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2743023 = {S16789, 108, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Lenovo_S2-38AT0_TD/3.0 LeOS/3.0 (Linux; Android 2.3) Release/10.01.2011 Browser/WAP 2.0 (AppleWebKit/530.17)*/
const Device UA2743266 = {S16790, 102, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - PhilipsW635/1.0 Linux/2.6 Android/2.3 Browser/AppleWebKit533.1 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA2743531 = {S16791, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MT680 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2743711 = {S16792, 125, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; C600 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2743724 = {S16793, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; TCL_A988 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2743847 = {S16794, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; DH U890 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2745467 = {S16795, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ar-eg; N76WIraq Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2745738 = {S16796, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; BF_A11 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2747010 = {S16797, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; GC500SF Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2747059 = {S16798, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Callbar A1 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2748152 = {S16799, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ar-eg; Lawzi Group SGC101 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2748366 = {S16800, 154, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TY-K-Touch_E619+ Build/TBEX_8405_V0101) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2749516 = {S16801, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; Hi-Class Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2749815 = {S16802, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; OPS-60 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2751740 = {S16803, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; P10 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2758376 = {S16804, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; el-gr; IQM722HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2759216 = {S16805, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; S800_ares4 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2759505 = {S16806, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; N62 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2759812 = {S16807, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; hexing15_6626_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2760605 = {S16808, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ET1 Build/01-X-2272-0100-00-D1-X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2761011 = {S16809, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; EBEST N4 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2761024 = {S16810, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; AOLE Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2762129 = {S16811, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; ZTE N960 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2762554 = {S16812, 169, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BLF-lephone 1800/lephone_1800_V1.3; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2763262 = {S16813, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; AT-5915 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2763470 = {S16814, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - HUAWEI-HUAWEI-Y-220T/1.0 Linux/2.6.35.7 Android/2.3.5 Release/10.20.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2763568 = {S16815, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; U710 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2763754 = {S16816, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HD6 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2764405 = {S16817, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; el-gr; iQTalk Crystal Build/V1.0.25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2764541 = {S16818, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; F32 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2764746 = {S16819, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; W337 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2765252 = {S16820, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ZJT-H1 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2765364 = {S16821, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; M.T.T. SmartFun Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2765613 = {S16822, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; AW 550D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2769645 = {S16823, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; 31TL04 Build/01.05.000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2775176 = {S16824, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; 91001E Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2775183 = {S16825, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; IncredibleS Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2775283 = {S16826, 127, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Q7 Build/Q7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2775769 = {S16827, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; I800B+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2775812 = {S16828, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; HTC-EVO-3G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2776883 = {S16829, 169, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BLF-lephone 1800/lephone_1800_V1.1; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2777317 = {S16830, 151, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 5.5.3(Android 2.3.6); en-us; MC919LL Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2777614 = {S16831, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; BF-A10 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2777724 = {S16832, 145, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; Telenor_Smart_Pro Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2777889 = {S16833, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; OS323 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2777964 = {S16834, 114, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_5230/2.3.004.X.5230; 320*480; CTC/2.0) CoolpadWebkit/533.1*/
const Device UA2778150 = {S16835, 157, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; BERAQ NB-1000 Build/VENUS_00.03.151.H1096) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2779764 = {S16836, 168, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad 5211/2.3.007.P2.X.5211; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2779937 = {S16837, 152, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ICE_MOBILE_APOLLO_TOUCH Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2779957 = {S16838, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; E619 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2780699 = {S16839, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; Mi230 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2781045 = {S16840, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fa-ir; GLX Luster Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2781255 = {S16841, 85, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - ZTEU793_TD/1.0 Linux/2.6.35 Android/2.3.5 Release/01.04.2013 Browser/AppleWebKit533.1*/
const Device UA2781494 = {S16842, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; WX06K Build/114.1.0029) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2781786 = {S16843, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HUAWEI G7500 Build/HUAWEIG7500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2786619 = {S16844, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MALATA Z100A Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2786699 = {S16845, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; GIONEE GN170 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2786747 = {S16846, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; H5500 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2786877 = {S16847, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; A51 Build/A51Gizoom-i8004) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2787397 = {S16848, 145, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TW-TONEWIN_S8310 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2787418 = {S16849, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE-T U960 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2787650 = {S16850, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; MegaFon SIM+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2787783 = {S16851, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ru-ru; G23 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2788501 = {S16852, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; W9001A Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2788772 = {S16853, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; moii-E501 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2789102 = {S16854, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; EBEST W70 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2789392 = {S16855, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; D500 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2789542 = {S16856, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; th-th; NEC-101S Build/X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2789610 = {S16857, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ZTE-U V760 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2789701 = {S16858, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; NOAIN A901 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2789928 = {S16859, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Saihon I91 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2790109 = {S16860, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.9; en-us; FDT G505 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2790155 = {S16861, 147, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Snexian Mi320 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2790192 = {S16862, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; sr-rs; Telenor Smart Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2790779 = {S16863, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Z3M Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2792147 = {S16864, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; V880 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2792773 = {S16865, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; TCA200 Build/R10.3.3-r1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2793155 = {S16866, 150, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ALCATEL one touch D662 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2793768 = {S16867, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; i508d Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2794665 = {S16868, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; N790 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2795928 = {S16869, 147, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; fa-ir; GLX Mobile Build/GLX-2013.03.18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2806565 = {S16870, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; Amin Rom Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2807591 = {S16871, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; T60 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2808285 = {S16872, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; pt-br; MYSM722HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2808548 = {S16873, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ru-ru; S700 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2809370 = {S16874, 145, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; KIRA N1002x Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2810039 = {S16875, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; S710D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2810229 = {S16876, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; YUSUN - W 800 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2810413 = {S16877, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; 5300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2810433 = {S16878, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3;es-co; ZTE-U V760 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2810520 = {S16879, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; EngelDroid EN1001 Build/X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2810662 = {S16880, 147, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; AndroTabGIN Build/0.0.1758.0257) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2810769 = {S16881, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; minitab7 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2811049 = {S16882, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; Z909 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2811307 = {S16883, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ru-ru; ImPAD1311 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2811549 = {S16884, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MORAL_N01 Build/HuaqinMORAL_N01) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2811947 = {S16885, 160, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3 YunOs 1.0.0.3; zh-cn; K-Touch W658 Build/AliyunOs-2012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2812009 = {S16886, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ET1 Build/02-X-1371-0601-00-M1-X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2812557 = {S16887, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; P4D ORION Build/GB.b12ref.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2812642 = {S16888, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; GM800 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2813707 = {S16889, 153, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Solar Pad Build/gingerbread_house_r59) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2814232 = {S16890, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Hosin_U7 Build/HosinU7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2814457 = {S16891, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Z180 Build/MALATAZ180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2815054 = {S16892, 124, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HOSIN_U2 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2815060 = {S16893, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Z710e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2815261 = {S16894, 126, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; S710D Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2815276 = {S16895, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo S2-38AT0 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2815403 = {S16896, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; EBEST W66A Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2815729 = {S16897, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; HTC-EVO-3G Build/chinhko) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2815732 = {S16898, 130, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-us; G75 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2815829 = {S16899, 165, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ZXY-ZTE-C_X500/ZTE-C X500MMB_31_Z4_C_TS9CF313B_04) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2815912 = {S16900, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; E810 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2815954 = {S16901, 124, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; TCL A968 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2816026 = {S16902, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; G888 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2816199 = {S16903, 168, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; innos a35 Build/A35_UMTS_V3_2.3_6190_V005_11b_AFRICA) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2816473 = {S16904, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HC-HE-E617 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2816515 = {S16905, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MTW20 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2816554 = {S16906, 204, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - MOT-XT536/1.0 Android/2.3.7 Release/2.28.2012 Browser/AppleWebKit533.1 Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT536 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2816592 = {S16907, 168, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GIO-GiONEE_C600/CBD75B04_01_T9133 ; 320*480; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2816736 = {S16908, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; G9C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2816769 = {S16909, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Changhong H5018 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2816842 = {S16910, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Android Smart Phone Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2816990 = {S16911, 127, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ThL V7 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2817035 = {S16912, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8180 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2817054 = {S16913, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME811 Build/SHDWR_X7_2.71.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2817136 = {S16914, 105, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; TCL-C995/AT02_CTCN_V2.0; 480*800; CTC/2.0) TAPAS BROWSER/2.0*/
const Device UA2817164 = {S16915, 168, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GIO-GiONEE_C500/CBE73A04_02_T9096 ; 320*480; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2817195 = {S16916, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; HD7+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2817269 = {S16917, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; E5670 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2817348 = {S16918, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; W708&W706 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2817419 = {S16919, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; T710 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2817470 = {S16920, 130, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Haier HW-N86W Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2817570 = {S16921, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; G910+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2817589 = {S16922, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.9; ar-ye; TCL C990 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2817611 = {S16923, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; U910 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2817612 = {S16924, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TCL W969 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2818022 = {S16925, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; M35 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2818173 = {S16926, 154, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LeWa_OS Build/v1.3.0; LeWaOS-HappyFrog) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2819274 = {S16927, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; K-Touch_W680 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2821626 = {S16928, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; CHER A91 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2821721 = {S16929, 130, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; DOOV_D770 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2822302 = {S16930, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; RSD-V1 Build/alpsRSD-V1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2822422 = {S16931, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; AM968 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2822793 = {S16932, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; G9C_BSX Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2822814 = {S16933, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; MITO 677 DEVICE Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2823224 = {S16934, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; K-Touch E619 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2823293 = {S16935, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; 5110 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2823336 = {S16936, 181, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;AliyunOS 1.5.1.16-RT-X.X; zh-cn; K-Touch E619 Build/AliyunOs-2012;Android 2.3 Compatible) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2823590 = {S16937, 165, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Optimus Net Build/Sony HD 2.0; Xperia by AD Bros.) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2825570 = {S16938, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; A209 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2826043 = {S16939, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; X3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2826245 = {S16940, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE U960s2 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2826551 = {S16941, 120, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; 7230 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2826587 = {S16942, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; V.LAND EX350 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2827010 = {S16943, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GN160T Build/MocorDroid2.3.5) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2828149 = {S16944, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HEDY_S803 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2830374 = {S16945, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; MCT666 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2830548 = {S16946, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; WPF-W9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2832996 = {S16947, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SC160 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2833158 = {S16948, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; MPM-i616 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2833535 = {S16949, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; iBall Andi4.3J Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2833596 = {S16950, 151, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Galaxy Precedent Build/AreaRom Test) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2833903 = {S16951, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-by; MTC Viva Build/HuaweiU8816) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2834312 = {S16952, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; nb-no; AKS-0085SM Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2834744 = {S16953, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; BF_A15 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2834771 = {S16954, 122, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; M910A Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2835655 = {S16955, 147, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Android Multimedia Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2835938 = {S16956, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; KN69 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2836139 = {S16957, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; F999 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2836768 = {S16958, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; MOX-PAD752 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2837377 = {S16959, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; D888 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2838439 = {S16960, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; GSTAR P12 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2838918 = {S16961, 130, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Epade S3 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2838940 = {S16962, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OKA13 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2839776 = {S16963, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; imx53_smd Build/R10.3.2_3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2839999 = {S16964, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; pt-br; TAB 810 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2840120 = {S16965, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; 596 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2841845 = {S16966, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N960 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2842941 = {S16967, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8060 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2843381 = {S16968, 63, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; U8667 Build/HuaweiU8667)*/
const Device UA2843408 = {S16969, 149, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-ph; ALCATEL_one_touch_978 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2843981 = {S16970, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; FLY IQ256 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2844196 = {S16971, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SmartWorld Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2844902 = {S16972, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Oyster500 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2846050 = {S16973, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; CR888 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2846609 = {S16974, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-B.S-mobile/1.0 Linux/Apps_2.6.32.9-perf Android/2.3.9 Release/11.19.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2848055 = {S16975, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; BF_A8 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2848546 = {S16976, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; C918 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2849176 = {S16977, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; C9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2850413 = {S16978, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ug-; LT S800 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2850973 = {S16979, 129, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; C01 Build/MocorDroid2.3.5) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2853546 = {S16980, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; gaophone15_6628_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2855909 = {S16981, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; SH-Saihon_I97 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2856167 = {S16982, 145, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; id-id; Andriod Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2856713 = {S16983, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; X20 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2858726 = {S16984, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ICE_MOBILE_APOLLO Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2860968 = {S16985, 145, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - CoolPadCoolpad-8056_TD/1.0 Android/2.3.5(Linux; Android 2.3.5) Release/10.30.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2865848 = {S16986, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; XUNHU S900 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2866739 = {S16987, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Colors X17 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2869203 = {S16988, 221, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.9; zh-cn; MTK6515M-MTK6515M-H003/1.0 Android/2.3.9 Release/04.15.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2869698 = {S16989, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; BLU Tigo Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2878748 = {S16990, 167, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Konka E5660S/1.0 Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Konka E5660S Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2888930 = {S16991, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; XD-S6000 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2889055 = {S16992, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; YEPEN P666 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2889081 = {S16993, 57, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GN106 Build/GRJ22)*/
const Device UA2889099 = {S16994, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; A811 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2889182 = {S16995, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Mi_A200 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2889219 = {S16996, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A370/S101) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2891130 = {S16997, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; OT-990C Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2891757 = {S16998, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ET1 Build/01-X-2602-0100-00-D1-X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2891980 = {S16999, 147, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; WHAM WS35 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2892210 = {S17000, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE U793 Build/MocorDroid2.3.5) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2892257 = {S17001, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Leopard MF800 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2892963 = {S17002, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; AS123 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2933263 = {S17003, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ?1??O-I727 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2933441 = {S17004, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; epade Q3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2933607 = {S17005, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; S306 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2933713 = {S17006, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; MD Smart Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2933724 = {S17007, 161, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; CHL-S3500/Hualu_S3500_V1.0; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2933765 = {S17008, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; GPLUS-W628 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2933917 = {S17009, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT_P5100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2933959 = {S17010, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PLUS Stil Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2934141 = {S17011, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; E908 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2934384 = {S17012, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; TD505 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2934771 = {S17013, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; SK-W15 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2935055 = {S17014, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; Dmobile A8 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2936391 = {S17015, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A105 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2936553 = {S17016, 81, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; Ilium_AVVIO_760 Build/a200Ilium_AVVIO_760)*/
const Device UA2936779 = {S17017, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A2_Classic Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2936991 = {S17018, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; PROA680 Build/V1.0.11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2937222 = {S17019, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; GSM25 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2937257 = {S17020, 225, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; FIREFLY_MOBILE-FIREFLY_S60/1.0 Android/2.3.7 Release/04.03.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2937986 = {S17021, 208, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; sprd-B51+/1.0 Android/2.3.6 Release/04.08.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2941951 = {S17022, 128, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - FLY-FLY-F351/1.0 Linux/2.6.35.7 Android/2.3.5 Release 05.22.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2949516 = {S17023, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; VOTO W5300 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2950082 = {S17024, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; yusun W702 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2950110 = {S17025, 84, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - ZTE-Tu960_TD/1.0 Linux/2.6.35 Android/2.3 Release/8.25.2011 Browser/AppleWebKit533.1*/
const Device UA2950269 = {S17026, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TETC-G19 Build/alpsTETC_G19) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2950504 = {S17027, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8020 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2950526 = {S17028, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lovme_A_18 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2950536 = {S17029, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; Action Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2950582 = {S17030, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; A330 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2950713 = {S17031, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; DROID3 Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2951334 = {S17032, 153, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Solar Pad Build/gingerbread_house_r66) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2951660 = {S17033, 161, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; CHL-S3000/Hualu_S3000_V1.5; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2951828 = {S17034, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; GO_Live_S1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2952039 = {S17035, 157, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; BERAQ NB-929 Build/VENUS_00.03.165.H1100A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2952634 = {S17036, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 7011 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2952996 = {S17037, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; POLYTRON S2350 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2953089 = {S17038, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; EF68A Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2953107 = {S17039, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; ZTE V8110 Build/GRK39F) AppleWebKit/533.1(KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2953278 = {S17040, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-nz; innos a35 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2953458 = {S17041, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-; Mi240   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2954274 = {S17042, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Kricket S1 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2954481 = {S17043, 157, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; Ilium_AVVIO_760 Build/a200Ilium_AVVIO_760) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2955115 = {S17044, 145, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; LAVORUM Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2956542 = {S17045, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; BP306 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2957120 = {S17046, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; G-TiDE E60 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2957361 = {S17047, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-E912 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2957463 = {S17048, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; X15 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2958218 = {S17049, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; MeuAN200 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2958283 = {S17050, 150, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Mtech DEZIRE Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2958593 = {S17051, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; Shadow Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2958914 = {S17052, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; B305A Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2959790 = {S17053, 160, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Konka E5660S/1.0 Mozilla/5.0 (Linux; U; Android 2.3.5; en-nz; T640S Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2963572 = {S17054, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; AX520 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2979161 = {S17055, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; LUBM908HW Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2979442 = {S17056, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pl-pl; NavRoad MOBIO Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2979482 = {S17057, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-cn; HTC_S715e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2979740 = {S17058, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; M203 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2980146 = {S17059, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HYUNDAI H11 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2980354 = {S17060, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ouboxin Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2980367 = {S17061, 127, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; TCLW989 Build/REL_2.3.7) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2980548 = {S17062, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; LINGWIN U880 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2980668 = {S17063, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; fulinpengda Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2980693 = {S17064, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; TCL A860 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2980837 = {S17065, 149, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; BESTA UNO BSP01 Release/2.24.2011) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2980843 = {S17066, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; innos i5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2981054 = {S17067, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; N98 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2981099 = {S17068, 165, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-XPERIA S Build/MiniXperiaMod; MiniXperiaMod-V5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2981278 = {S17069, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; AXF-01B Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2981327 = {S17070, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GN160T Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2981361 = {S17071, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; BF9200 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2981415 = {S17072, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; G_7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2981443 = {S17073, 154, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SWTGT-7250 Build/gingerbread_house_r79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2981469 = {S17074, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; HD7-PRO Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2981486 = {S17075, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; Z2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2981549 = {S17076, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; VB808 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2981558 = {S17077, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; N619 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2981563 = {S17078, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; C9220 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2981639 = {S17079, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; RSD_V2 Build/HUAQINRSD_V2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2981664 = {S17080, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Ouki_A18 Build/MocorDroid2.3.5) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2981988 = {S17081, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; W6100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982033 = {S17082, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; GiONEE GN108 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982046 = {S17083, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SOP-G928 Build/alpsSOP_G928) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982103 = {S17084, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZJ105 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982365 = {S17085, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; TP703 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982398 = {S17086, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HEDY_H712 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982424 = {S17087, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; CETRIX CS1216 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982433 = {S17088, 122, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; 7230-B Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2982472 = {S17089, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPSSON iMO888 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982783 = {S17090, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; STM753HD Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982795 = {S17091, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; NOAIN A902 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982809 = {S17092, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; IM-A730K Build/Bcola) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982825 = {S17093, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; G5C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982979 = {S17094, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lovme X3 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982981 = {S17095, 147, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; BT Home Smart Phone Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2983068 = {S17096, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; V3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2983204 = {S17097, 147, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; Niutek-3G-4.0 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2983344 = {S17098, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; TCL A909 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2983410 = {S17099, 164, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; X402 Build/X402-Android-2.3.6-TecSync-2012/10/10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2983721 = {S17100, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; nToBeBox 2.3.4; tr-tr; ITS10BR Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2984035 = {S17101, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; sl-si; B705A Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2984145 = {S17102, 168, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BW-BenWee 5100S/BenWee_5100S_V1.2; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2984258 = {S17103, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Apollo one Build/MocorDroid2.3.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2984574 = {S17104, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; QY-S10 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2986061 = {S17105, 167, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BLW-VE600/VE600_D11_V1.00_V1.4_X; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2986230 = {S17106, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; Ilium S50 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2986666 = {S17107, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZY-FLY_N9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2986954 = {S17108, 182, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Lephone_TD505_TD/1.0 Android/2.3.5 (Linux; Android 2.3.5) Release/8.9.2012 Mozilla/5.0 (Linux; U; Android 2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2989553 = {S17109, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; B51+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2989589 = {S17110, 161, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Konka E5660S/1.0 Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; ZA-966 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2993631 = {S17111, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; el-gr; iQTalk Crystal Build/V1.0.31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2995658 = {S17112, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; Newman A17  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2995805 = {S17113, 109, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - HS-T912_TD/1.0 Android/2.3 Release/1.31.2013 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2996012 = {S17114, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; el-gr; iQTalk Crystal Build/V1.0.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2996804 = {S17115, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DTC GT7 Build/GT822) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2996842 = {S17116, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; M2 Elegante Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2996907 = {S17117, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; Avvio 701 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2997057 = {S17118, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; th-th; A301 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2997447 = {S17119, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; M39 Sawy Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2997599 = {S17120, 212, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-AS-45/57/1.0 Android/2.3.5 Release/05.08.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2998137 = {S17121, 60, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; Z10 Build/10.1.0.103)*/
const Device UA2998267 = {S17122, 226, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; FIREFLY_MOBILE-FIREFLY_A530/1.0 Android/2.3.7 Release/04.26.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2998474 = {S17123, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-E820 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2998674 = {S17124, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A703_Andy Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2999236 = {S17125, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-nz; innos a35plus Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2999787 = {S17126, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-bo; Bmobile_AX525 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3000771 = {S17127, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; BY2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3001096 = {S17128, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; 12018 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3001187 = {S17129, 145, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; Essentielb-Pixis Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3001703 = {S17130, 215, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-Android-351/1.0 Android/2.3.5 Release/04.24.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3001705 = {S17131, 215, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-Android-401/1.0 Android/2.3.5 Release/04.18.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3002475 = {S17132, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ar-eg; msm7627a_sku1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3002649 = {S17133, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SL-S10 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3003203 = {S17134, 225, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; FIREFLY_MOBILE-FIREFLY_S70/1.0 Android/2.3.7 Release/05.06.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3004773 = {S17135, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Masstel M05 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3005891 = {S17136, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; AS126 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3006116 = {S17137, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; nl-nl; SMP35-100 Build/3.0.1398_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3006674 = {S17138, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; C-1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3008774 = {S17139, 188, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; Lanix Ilium_S105 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LNX-Android-MMS-V1.1/1.2*/
const Device UA3009408 = {S17140, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; V1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3014490 = {S17141, 147, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LIYANG Z9 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3016339 = {S17142, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Q1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3016386 = {S17143, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - HUAWEI-HUAWEI-Y-220T/1.0 Linux/2.6.35.7 Android/2.3.5 Release/03.20.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3016697 = {S17144, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-pt; M8402 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3019115 = {S17145, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; N8303 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3020403 = {S17146, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Magic-35 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3021446 = {S17147, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-pt; M8403 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3026748 = {S17148, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; iNote mini Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3030112 = {S17149, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A3* Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3041633 = {S17150, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 5880 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042042 = {S17151, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ar-eg; izoom-i8001 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042205 = {S17152, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; BRS-BROR_S9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042215 = {S17153, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ChangHongV7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042219 = {S17154, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; YOORD D599 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042237 = {S17155, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; SOP-G998 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042293 = {S17156, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; U10 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042325 = {S17157, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; DK P170 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042335 = {S17158, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; DK P120 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042345 = {S17159, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; NEC909e Build/X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042411 = {S17160, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; C868 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042454 = {S17161, 147, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; Bliss Pad Q7011 Build/1.0.540_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042463 = {S17162, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; E9005 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042469 = {S17163, 124, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; KPT A88 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA3042471 = {S17164, 151, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; COOCO_A6 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042498 = {S17165, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SHT-H539 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042513 = {S17166, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TETC G16 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042525 = {S17167, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TAIXING Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042639 = {S17168, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; I3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042769 = {S17169, 111, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - KONKA V920_TD/1.0 Android/2.3 Release/2.2.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA3042814 = {S17170, 131, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; T9108 Build/MocorDroid2.3.5) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA3042858 = {S17171, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ET1 Build/03-X-1852-0602-00-M1-X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042985 = {S17172, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; zh-ch; zh-tw; Aigo D1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3043004 = {S17173, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; cs-cz; Kis Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3043030 = {S17174, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Karbonn_A5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3043212 = {S17175, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; ZTE V960 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3043335 = {S17176, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad8050 Build/MocorDroid2.3.5) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3043662 = {S17177, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; C3668 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3043832 = {S17178, 168, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BW-BenWee 5100S/BenWee_5100S_V1.3; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3043883 = {S17179, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; KPT A55 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3043923 = {S17180, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; U16 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3044214 = {S17181, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; JDD-F99 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3044501 = {S17182, 163, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YCT-TE690/ctyon_TE690-1_V1.6; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3044590 = {S17183, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Opal 800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3045066 = {S17184, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZY-FLY_S1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3045386 = {S17185, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; FORTUNE Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3045569 = {S17186, 147, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ginwave15_6626_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3048094 = {S17187, 106, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - ZTEU887_TD/1.0 Linux/2.6.35.7 Android/2.3.5 Release/01.01.2012 Browser/AppleWebKit533.1 Mozilla/5.0 Mobile*/
const Device UA3048821 = {S17188, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; FDT E18 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3049092 = {S17189, 218, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MT6517-WINDS-NOTE-W/1.0 Android/2.3.5 Release/04.16.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3049486 = {S17190, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; XUNHU S990 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3049682 = {S17191, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; AS135 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3049688 = {S17192, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE U793 Build MocorDroid2.3.5) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA3057034 = {S17193, 221, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.9; zh-cn; MTK6515M-MTK6515M-H003/1.0 Android/2.3.9 Release/05.16.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3057051 = {S17194, 208, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; sprd-B51+/1.0 Android/2.3.6 Release/05.29.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3057210 = {S17195, 212, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-yuanpeng/1.0 Android/2.3.5 Release/06.13.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3057252 = {S17196, 155, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; cs-cz; ALIGATOR S4000Duo Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3058022 = {S17197, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Trinity-T3 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3058433 = {S17198, 214, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-WINDS-NOTE/1.0 Android/2.3.5 Release/07.13.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3058563 = {S17199, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; AMAZE S250 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3058856 = {S17200, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; SPC S1 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3059220 = {S17201, 217, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-UltraPhone-40/1.0 Android/2.3.5 Release/07.31.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3059707 = {S17202, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; TLINK351 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3069733 = {S17203, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ru-ru; HTC_Incredible_A6300 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3070081 = {S17204, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; A208 Build/UNKNOWN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3070158 = {S17205, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TCL A919 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3070194 = {S17206, 130, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; T868 Build/MocorDroid2.3.5) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA3070216 = {S17207, 157, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; EBEST X1 Build/BC40_NIECHE_X1W_HW1.1_V001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3070228 = {S17208, 163, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BRS-BROR W60+/BROR_W60+_V1.0; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3070630 = {S17209, 125, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8060 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3070644 = {S17210, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; AE808 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3070691 = {S17211, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; i5300 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3070848 = {S17212, 58, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; 7230-B Build/GRJ90)*/
const Device UA3071124 = {S17213, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Nexus_One Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3071127 = {S17214, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; INCO Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3071206 = {S17215, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; BF_A12P Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3071264 = {S17216, 149, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad8050 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3071346 = {S17217, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; AX3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3071417 = {S17218, 160, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; HYPERION 8 Mod by echcon04 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3071484 = {S17219, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; HANEL E21 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3071828 = {S17220, 150, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 8056 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3072231 = {S17221, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; TCL J210C Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3072699 = {S17222, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; G-TiDE E9i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3072966 = {S17223, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE U887 Build/MocorDroid2.3.5) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3073669 = {S17224, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; OPS-51   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3074202 = {S17225, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; telego Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3074235 = {S17226, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; S712 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3074367 = {S17227, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; MOBO BOSSY Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3074460 = {S17228, 209, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-S7100/1.0 Android/2.3.5 Release/05.04.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3074570 = {S17229, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; GN107 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3076444 = {S17230, 215, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad-8076_TD/1.0 Android/2.3.5 Release/03.11.2012 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3078282 = {S17231, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hr-hr; RACER III mini Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3079363 = {S17232, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; U35X Build/V1.0.09) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3079825 = {S17233, 150, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; iNote beyond Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3084937 = {S17234, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; EB-WX1GJ Build/2.01d) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3084997 = {S17235, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; de-de; 1370 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3085224 = {S17236, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; VITA_V4W Build/HuaqinA51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3085469 = {S17237, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; S8022 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3085699 = {S17238, 161, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; CHL-S3000/Hualu_S3000_V1.2; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3085990 = {S17239, 67, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; YL-Coolpad 5216 Build/GRJ90)*/
const Device UA3086198 = {S17240, 90, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - DESAY_TS908_CMCC_TD/1.0 Android/2.3.7 (LinuxOS 2.6.35.7) Release/04.13.2012 Browser/WAP2.0*/
const Device UA3086816 = {S17241, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; th-th; UPAD U30 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3088165 = {S17242, 219, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/2.3.5 Release/04.15.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3088201 = {S17243, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; V8405 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3088505 = {S17244, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; m190s Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3088861 = {S17245, 216, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; sprd-GaLaX-X5570/1.0 Android/2.3.5 Release/03.01.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3091414 = {S17246, 170, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LNV-Lenovo_A370e/Lenovo A370e_S110_X; 480*800; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3091617 = {S17247, 211, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MT-NOTE-LIFE/1.0 Android/2.3.5 Release/05.24.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3091822 = {S17248, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.8; en-us; V-S3500 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3092292 = {S17249, 200, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; T780+/TBTX_8990_V0101 Android/2.3.5 Release/05.10.2013 Browser/AppleWebKit533.1 Build/IMM76D)ApplelWebkit/533.1 (KHTML,like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3092409 = {S17250, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; VENERA Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3097794 = {S17251, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; 2GO Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3097892 = {S17252, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME811 Build/SHDWR_X7_2.71.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3098329 = {S17253, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; TCLW989 Build/REL_2.3.3.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3098360 = {S17254, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; kailitong Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3098395 = {S17255, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; C1000 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3098466 = {S17256, 124, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.7; zh-cn; T92 Build GWK74) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA3098527 = {S17257, 161, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BLW-VE600/VLAND_VE600_V1.3; 480*800;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3098558 = {S17258, 169, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BLF-lephone 1800/lephone_1800_V1.0; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3098598 = {S17259, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; shiyu Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3098631 = {S17260, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; KliTON I608 Build/KliTONKliTON) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3098655 = {S17261, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; tr-tr; E-TAB_GOV Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3098674 = {S17262, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; DESAY TS908 Build/MocorDroid2.3.5) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA3098766 = {S17263, 151, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 5.3.4(Android 2.3.6); zh-cn; MC919LL Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3098774 = {S17264, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; T232 Build/MocorDroid2.3.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3098835 = {S17265, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; k9p Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3098929 = {S17266, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A370 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3099048 = {S17267, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; BOWAY_U3 Build/alpsBOWAY-) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3099108 = {S17268, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A326 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3099172 = {S17269, 55, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; S9 Build/GRK39F)*/
const Device UA3099759 = {S17270, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; Nuqleo-Fusion Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3099957 = {S17271, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.8; in-id; PGM 205 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3100028 = {S17272, 67, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; epade A360S Build/alpsepade)*/
const Device UA3100061 = {S17273, 132, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Z990 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3100127 = {S17274, 133, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-cr; AX520 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3100395 = {S17275, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; Mi240   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3100508 = {S17276, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-EG876 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3101054 = {S17277, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; IHS-C250 Build/IHS-C250-V001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3101956 = {S17278, 149, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; Sansui SA41 Build/MocorDroid2.3.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3101988 = {S17279, 157, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; BERAQ NB-939 Build/VENUS_00.03.170.H1101A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3102362 = {S17280, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; F307 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3102675 = {S17281, 216, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lephone_TD9229_TD/1.0 Android/2.3.5 Release/8.9.2012 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5 ) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3102789 = {S17282, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ro-ro; Evotech Cyano  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3103031 = {S17283, 125, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; %s; OlivePad) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA3103637 = {S17284, 215, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-A1-INFINITY/1.0 Android/2.3.5 Release/04.22.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3104027 = {S17285, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-EG901 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3107061 = {S17286, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LNV-Lenovo A370e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3107509 = {S17287, 207, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; sprd-B54/1.0 Android/2.3.6 Release/05.10.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3107665 = {S17288, 165, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; pt-br; Multilaser Orion Build/Multilaser-Orion-V00.00.01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3107820 = {S17289, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; ZA966 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3108074 = {S17290, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; FIREFLY_S80 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3108142 = {S17291, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; starTIM1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3108611 = {S17292, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; S350 Build/S350_VK_MX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3108745 = {S17293, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; AX353 Build/MocorDroid2.3.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3112161 = {S17294, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; ONE TOUCH 4007D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3112677 = {S17295, 141, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; T1+ Build/MocorDroid2.3.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3117265 = {S17296, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; FWS610 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3117295 = {S17297, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-us; QBEN QB700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3117370 = {S17298, 135, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; mx1001 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3117440 = {S17299, 136, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ouqi_key Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3117524 = {S17300, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Z1i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3117635 = {S17301, 139, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; kliton I30 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3117691 = {S17302, 58, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; M910A Build/GRK39F)*/
const Device UA3117871 = {S17303, 140, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZIPO-W919 F Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3118251 = {S17304, 134, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; T9555 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3118667 = {S17305, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; USMART SL33 Build/USMARTSL33) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3118739 = {S17306, 148, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ET1 Build/04-X-2399-0601-00-M1-X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3119091 = {S17307, 138, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; epade V05 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3121323 = {S17308, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ar-eg; K99+ Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3121338 = {S17309, 207, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; sprd-B51/1.0 Android/2.3.6 Release/04.08.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3123565 = {S17310, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; sr-rs; Telenor Smart 2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3124109 = {S17311, 153, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; itel iNote mini Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3124868 = {S17312, 219, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/2.3.5 Release/05.09.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3124879 = {S17313, 142, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; MOBO Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3124942 = {S17314, 146, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; FLY F40+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3124992 = {S17315, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; ONE TOUCH 4007A Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3125214 = {S17316, 144, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; OPS-41 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3125312 = {S17317, 143, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-S5831i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3125414 = {S17318, 216, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-Greentel-G11/1.0 Android/2.3.5 Release/06.19.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3126007 = {S17319, 180, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 2.3.6;dev=SGH-I717;devm=SAMSUNG-SGH-I717;man=samsung;brw=Android;shwd=800x1280;shwa=800x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA3127389 = {S17320, 137, { &P15364,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; AT-AS35D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA217860 = {S17321, 97, { &P15364,&P5690,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.813.0 Safari/535.1*/
const Device UA248984 = {S17322, 99, { &P15364,&P5690,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.187 Safari/535.1*/
const Device UA249773 = {S17323, 106, { &P15364,&P5690,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA615627 = {S17324, 120, { &P15364,&P5690,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/534.13*/
const Device UA630284 = {S17325, 153, { &P15364,&P5690,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.802.30 Safari/535.1 SE 2.X MetaSr 1.0,gzip(gfe) (via translate.google.com)*/
const Device UA860563 = {S17326, 138, { &P15364,&P5690,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2234481 = {S17327, 126, { &P15364,&P5690,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.802.30 Safari/535.1 SE 2.X MetaSr 1.0,gzip(gfe)*/
const Device UA2298999 = {S17328, 106, { &P15364,&P5690,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1*/
const Device UA2818200 = {S17329, 96, { &P15364,&P5690,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.1 (KHTML like Gecko) Chrome/14.0.794.0 Safari/535.1*/
const Device UA2980424 = {S17330, 96, { &P15364,&P5690,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML like Gecko) Chrome/14.0.809.0 Safari/535.1*/
const Device UA3041828 = {S17331, 110, { &P15364,&P5690,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.94 Safari/535.1*/
const Device UA2885398 = {S17332, 99, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A4400f*/
const Device UA2886774 = {S17333, 219, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A4400f [FBAN/FBIOS;FBAV/6.2;FBBV/X;FBDV/iPad2,5;FBMD/iPad;FBSN/iPhone OS;FBSV/7.0;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBOP/1]*/
const Device UA2947242 = {S17334, 99, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Mobile/11A4414e*/
const Device UA2948092 = {S17335, 219, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Mobile/11A4414e [FBAN/FBIOS;FBAV/6.3;FBBV/X;FBDV/iPad2,5;FBMD/iPad;FBSN/iPhone OS;FBSV/7.0;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBOP/1]*/
const Device UA2965151 = {S17336, 83, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko)*/
const Device UA3023745 = {S17337, 101, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4435d*/
const Device UA3023768 = {S17338, 85, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko)*/
const Device UA3028155 = {S17339, 221, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4435d [FBAN/FBIOS;FBAV/6.3;FBBV/X;FBDV/iPad2,5;FBMD/iPad;FBSN/iPhone OS;FBSV/7.0;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBOP/1]*/
const Device UA3033020 = {S17340, 101, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449a*/
const Device UA3034134 = {S17341, 101, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d*/
const Device UA3034766 = {S17342, 221, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d [FBAN/FBIOS;FBAV/6.4;FBBV/X;FBDV/iPad2,5;FBMD/iPad;FBSN/iPhone OS;FBSV/7.0;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBOP/1]*/
const Device UA3038094 = {S17343, 221, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449a [FBAN/FBIOS;FBAV/6.4;FBBV/X;FBDV/iPad2,5;FBMD/iPad;FBSN/iPhone OS;FBSV/7.0;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBOP/1]*/
const Device UA3059073 = {S17344, 101, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449b*/
const Device UA3080191 = {S17345, 99, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465*/
const Device UA3096922 = {S17346, 219, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 [FBAN/FBIOS;FBAV/6.5;FBBV/X;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/7.0;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBOP/1]*/
const Device UA3109692 = {S17347, 143, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari/8536.25 (AirWatch Browser v2.7.1427)*/
const Device UA3111776 = {S17348, 101, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501*/
const Device UA3111837 = {S17349, 124, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 iPad2,1/K93AP Zite/2.4.1*/
const Device UA3113622 = {S17350, 129, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari/8536.26.7 DiigoBrowser*/
const Device UA3114594 = {S17351, 130, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad3,1; iPad; U; CPU OS 7_0 like Mac OS X; en_US) com.google.GooglePlus/23341 (KHTML, like Gecko) Mobile/J1AP (gzip)*/
const Device UA3114948 = {S17352, 246, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 7_0 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/7.0;FBSS/1; FBCR/unavailable;FBID/tablet;FBLC/en_US;FBSF/1.0]*/
const Device UA3115083 = {S17353, 131, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad2,1; iPad; U; CPU OS 7_0 like Mac OS X; en_US) com.google.GooglePlus/23341 (KHTML, like Gecko) Mobile/K93AP (gzip)*/
const Device UA3115388 = {S17354, 87, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko)*/
const Device UA3115458 = {S17355, 225, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 [FBAN/FBIOS;FBAV/6.5.1;FBBV/X;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/7.0.2;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBOP/1]*/
const Device UA3115791 = {S17356, 110, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) BaiduHD 2.3.4 AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176*/
const Device UA3127084 = {S17357, 132, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad3,4; iPad; U; CPU OS 7_0 like Mac OS X; en_US) com.google.GooglePlus/23341 (KHTML, like Gecko) Mobile/P101AP (gzip)*/
const Device UA3128099 = {S17358, 40, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - QQ???HD 3.3 (iPad; iPhone OS 7.0; zh_CN)*/
const Device UA3129324 = {S17359, 126, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 iPad2,1/K93AP Zite/2.4.1*/
const Device UA3130235 = {S17360, 131, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Safari/8536.26.7 DiigoBrowser*/
const Device UA3130253 = {S17361, 112, { &P15767,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) BaiduHD 2.3.4 AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176*/
const Device UA102566 = {S17362, 112, { &P15364,&P17285,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA121493 = {S17363, 118, { &P15364,&P17285,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA133494 = {S17364, 118, { &P15364,&P17285,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA174917 = {S17365, 120, { &P15364,&P17285,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA180169 = {S17366, 118, { &P15364,&P17285,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA180949 = {S17367, 119, { &P15364,&P17285,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA2258945 = {S17368, 106, { &P15364,&P17285,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.26.9 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA2352382 = {S17369, 102, { &P15364,&P17285,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.26.9 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2386824 = {S17370, 99, { &P15364,&P17285,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2404642 = {S17371, 99, { &P15364,&P17285,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA2406788 = {S17372, 99, { &P15364,&P17285,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2421981 = {S17373, 106, { &P15364,&P17285,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA2423080 = {S17374, 106, { &P15364,&P17285,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2423257 = {S17375, 95, { &P15364,&P17285,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2425354 = {S17376, 99, { &P15364,&P17285,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA2428661 = {S17377, 100, { &P15364,&P17285,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA2457429 = {S17378, 106, { &P15364,&P17285,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2457879 = {S17379, 107, { &P15364,&P17285,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA181055 = {S17380, 120, { &P15364,&P7133,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA521054 = {S17381, 157, { &P15364,&P7133,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.127 Safari/534.16,gzip(gfe) (via translate.google.com)*/
const Device UA756869 = {S17382, 119, { &P15364,&P7133,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; ru-RU) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.11 Safari/534.16*/
const Device UA822507 = {S17383, 118, { &P15364,&P7133,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; de-DE) AppleWebKit/534.17 (KHTML, like Gecko) Chrome/10.0.649.0 Safari/534.17*/
const Device UA1870337 = {S17384, 159, { &P15364,&P7133,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2151221 = {S17385, 127, { &P15364,&P7133,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/10.0 baidubrowser/1.x Safari/534.7*/
const Device UA2159887 = {S17386, 120, { &P15364,&P7133,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; ru-RU; AppleWebKit/534.16; KHTML; like Gecko; Chrome/10.0.648.11;Safari/534.16)*/
const Device UA2480491 = {S17387, 130, { &P15364,&P7133,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.127 Safari/534.16,gzip(gfe)*/
const Device UA2674459 = {S17388, 99, { &P15364,&P7133,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/10.0.782.112 Safari/535.1*/
const Device UA2674483 = {S17389, 108, { &P15364,&P7133,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.64 Safari/537.31*/
const Device UA2933586 = {S17390, 117, { &P15364,&P7133,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; de-DE) AppleWebKit/534.17 (KHTML like Gecko) Chrome/10.0.649.0 Safari/534.17*/
const Device UA2958512 = {S17391, 120, { &P15364,&P7133,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0+(Windows;+U;+Windows+NT+6.1;+en-US)+AppleWebKit/534.16+(KHTML,+like+Gecko)+Chrome/10.0.648.204+Safari/534.16*/
const Device UA2973290 = {S17392, 121, { &P15364,&P7133,&P17249,&P18092 }};	/*Useragent - =Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA429157 = {S17393, 131, { &P15259,&P6374,&P4418,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9900; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.576 Mobile Safari/534.11+*/
const Device UA429222 = {S17394, 77, { &P15259,&P6374,&P4418,&P18092 }};	/*Useragent - BlackBerry9900/7.0.0.576 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102*/
const Device UA368200 = {S17395, 97, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/18.6.872.0 Safari/535.8*/
const Device UA453892 = {S17396, 141, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA587348 = {S17397, 155, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/535.12 (KHTML, like Gecko) Chrome/18.0.966.0 Safari/535.12,gzip(gfe) (via translate.google.com)*/
const Device UA605856 = {S17398, 109, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19*/
const Device UA606088 = {S17399, 139, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19,gzip(gfe) (via translate.google.com)*/
const Device UA614977 = {S17400, 102, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA625454 = {S17401, 109, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA627121 = {S17402, 118, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/535.12 (KHTML, like Gecko) Chrome/18.0.966.0 Safari/535.12*/
const Device UA636143 = {S17403, 97, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/18.6.872.0 Safari/535.8*/
const Device UA666977 = {S17404, 104, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/18.6.872.0 Safari/535.8*/
const Device UA904573 = {S17405, 150, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay/1911; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA907270 = {S17406, 106, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.1634 Safari/535.19 YI*/
const Device UA1965280 = {S17407, 136, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Client/0; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA1979300 = {S17408, 118, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/535.12 (KHTML, like Gecko) Chrome/18.0.966.0 Safari/535.12*/
const Device UA1998571 = {S17409, 150, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay/1928; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2015114 = {S17410, 150, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay/1950; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2043695 = {S17411, 150, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay/1960; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2167448 = {S17412, 106, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; en-US) AppleWebKit/535.12 (KHTML, like Gecko) Chrome/18.0.966.0 Safari/535.12*/
const Device UA2206187 = {S17413, 150, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay/2005; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2235770 = {S17414, 150, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay/2026; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2259102 = {S17415, 112, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19,gzip(gfe)*/
const Device UA2355667 = {S17416, 128, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/535.12 (KHTML, like Gecko) Chrome/18.0.966.0 Safari/535.12,gzip(gfe)*/
const Device UA2371856 = {S17417, 114, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2;) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.45 Safari/535.19*/
const Device UA2374532 = {S17418, 114, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1;) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.45 Safari/535.19*/
const Device UA2407662 = {S17419, 150, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay/2095; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2411737 = {S17420, 109, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; de) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/18.0 Safari/525.13*/
const Device UA2415828 = {S17421, 106, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; zh-CN) AppleWebKit/535.12 (KHTML, like Gecko) Chrome/18.0.966.0 Safari/535.12*/
const Device UA2424224 = {S17422, 132, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/18.0 BIDUPlayerBrowser/2.x Safari/534.7*/
const Device UA2430857 = {S17423, 150, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay/2109; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2433144 = {S17424, 150, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay/2102; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2472065 = {S17425, 150, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay/2125; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2535471 = {S17426, 147, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay/X; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2567537 = {S17427, 120, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/18.0 BIDUPlayerBrowser/2.x Safari/534.7*/
const Device UA2939577 = {S17428, 112, { &P15364,&P5690,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; en-US; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025 Safari/535.19*/
const Device UA366542 = {S17429, 136, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B5141a Safari/7534.48.3*/
const Device UA398172 = {S17430, 134, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B148 Safari/7534.48.3*/
const Device UA421881 = {S17431, 134, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3*/
const Device UA422323 = {S17432, 134, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7534.48.3*/
const Device UA422380 = {S17433, 134, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B179 Safari/7534.48.3*/
const Device UA454799 = {S17434, 134, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A406 Safari/7534.48.3*/
const Device UA585740 = {S17435, 134, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B150 Safari/7534.48.3*/
const Device UA587468 = {S17436, 134, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B157 Safari/7534.48.3*/
const Device UA587511 = {S17437, 134, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B155 Safari/7534.48.3*/
const Device UA589698 = {S17438, 134, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B172 Safari/7534.48.3*/
const Device UA590180 = {S17439, 134, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B174 Safari/7534.48.3*/
const Device UA616123 = {S17440, 136, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA636688 = {S17441, 134, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3*/
const Device UA777976 = {S17442, 136, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9Z999 Safari/7534.48.3*/
const Device UA793768 = {S17443, 136, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3*/
const Device UA888147 = {S17444, 136, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A406 Safari/7534.48.3*/
const Device UA895623 = {S17445, 127, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.46*/
const Device UA899035 = {S17446, 136, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B208 Safari/7534.48.3*/
const Device UA902895 = {S17447, 130, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA908452 = {S17448, 126, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari Version/5.1*/
const Device UA910038 = {S17449, 124, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari Version/5.1*/
const Device UA1976479 = {S17450, 124, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari Version/5.1*/
const Device UA1986550 = {S17451, 125, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7534.46*/
const Device UA1992144 = {S17452, 134, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9Z999 Safari/7534.48.3*/
const Device UA2154562 = {S17453, 136, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7534.48.3*/
const Device UA2156230 = {S17454, 126, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari Version/5.1*/
const Device UA2222173 = {S17455, 136, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (device; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA2222203 = {S17456, 135, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206Safari/7534.48.3*/
const Device UA2323292 = {S17457, 146, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3*/
const Device UA2352535 = {S17458, 165, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9B206 Version/5.1 Safari/8536.25 Safari/6533.18.5*/
const Device UA2382300 = {S17459, 137, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/10A403 Safari/7534.48.3*/
const Device UA2442872 = {S17460, 135, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA2554634 = {S17461, 137, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/10A523 Safari/7534.48.3*/
const Device UA3043061 = {S17462, 136, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3*/
const Device UA3047024 = {S17463, 137, { &P12280,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/10B329 Safari/7534.48.3*/
const Device UA246719 = {S17464, 63, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; Trident/6.0)*/
const Device UA248188 = {S17465, 75, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; Win64; x64; Trident/6.0)*/
const Device UA330420 = {S17466, 50, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2)*/
const Device UA437187 = {S17467, 70, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0)*/
const Device UA1988497 = {S17468, 64, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.10; Windows NT 6.2; Trident/5.0)*/
const Device UA2158414 = {S17469, 76, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.10; Windows NT 6.2; Windows Live Writer 1.0)*/
const Device UA2232420 = {S17470, 84, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3230.2052; Windows 6.2; MSIE 9.10.8400.0)*/
const Device UA2238372 = {S17471, 88, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3230.2052; Windows 6.2; MSIE 9.10.9200.16420)*/
const Device UA2362965 = {S17472, 88, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3230.2052; Windows 6.2; MSIE 9.10.9200.16433)*/
const Device UA2374652 = {S17473, 80, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; Trident/5.0; XBLWP7; ZuneWP7)*/
const Device UA2402882 = {S17474, 171, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA2429191 = {S17475, 88, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.2; MSIE 9.10.9200.16453)*/
const Device UA2429562 = {S17476, 88, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.2; MSIE 9.10.9200.16384)*/
const Device UA2455989 = {S17477, 198, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0)*/
const Device UA2503062 = {S17478, 183, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2)*/
const Device UA2516444 = {S17479, 88, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.2; MSIE 9.10.9200.16484)*/
const Device UA2528658 = {S17480, 173, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; ASU2JS)*/
const Device UA2568506 = {S17481, 88, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.2; MSIE 9.10.9200.16519)*/
const Device UA2676911 = {S17482, 88, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.2; MSIE 9.10.9200.16540)*/
const Device UA2695884 = {S17483, 169, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; SE 2.X MetaSr 1.0)*/
const Device UA2721499 = {S17484, 51, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.10; Windows NT 6.2)*/
const Device UA2741521 = {S17485, 88, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.4.3607.2246; Windows 6.2; MSIE 9.10.9200.16580)*/
const Device UA2818256 = {S17486, 143, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729)*/
const Device UA2849242 = {S17487, 178, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Tablet PC 2.0; ASU2JS)*/
const Device UA2871624 = {S17488, 88, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.5.4209.2358; Windows 6.2; MSIE 9.10.9200.16599)*/
const Device UA2943864 = {S17489, 64, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.11; Windows NT 6.2; Trident/5.0)*/
const Device UA2947522 = {S17490, 88, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.5.4209.2358; Windows 6.2; MSIE 9.10.9200.16635)*/
const Device UA3034757 = {S17491, 88, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.5.4413.1752; Windows 6.2; MSIE 9.10.9200.16660)*/
const Device UA3048586 = {S17492, 96, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.10; Windows NT 6.2; WOW64; Trident/6.0; SiteKiosk 7.8 Build 332)*/
const Device UA3083879 = {S17493, 88, { &P15364,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; GoogleToolbar 7.5.4413.1752; Windows 6.2; MSIE 9.10.9200.16688)*/
const Device UA1968751 = {S17494, 254, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4128; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA1976172 = {S17495, 244, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4128; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA1994850 = {S17496, 220, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA1995072 = {S17497, 203, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA2154482 = {S17498, 72, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Microsoft Office/15.0 (Windows NT 6.2; Microsoft Outlook 15.0.4128; Pro)*/
const Device UA2160260 = {S17499, 259, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4128; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA2167311 = {S17500, 259, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 15.0.4128; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA2167895 = {S17501, 269, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4128; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA2168119 = {S17502, 230, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA2183028 = {S17503, 249, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4128; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA2223269 = {S17504, 240, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA2245691 = {S17505, 244, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2245754 = {S17506, 220, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2245784 = {S17507, 215, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2245786 = {S17508, 230, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA2338109 = {S17509, 213, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA2342208 = {S17510, 249, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2345329 = {S17511, 225, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2346180 = {S17512, 254, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2350893 = {S17513, 72, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Microsoft Office/15.0 (Windows NT 6.2; Microsoft Outlook 15.0.4420; Pro)*/
const Device UA2363218 = {S17514, 230, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2366447 = {S17515, 224, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA2367414 = {S17516, 248, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2367613 = {S17517, 263, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2375967 = {S17518, 196, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA2381071 = {S17519, 230, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2383202 = {S17520, 259, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2384830 = {S17521, 208, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2384860 = {S17522, 259, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2400835 = {S17523, 269, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2421718 = {S17524, 277, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2422282 = {S17525, 273, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2454904 = {S17526, 72, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Microsoft Office/15.0 (Windows NT 6.2; Microsoft Outlook 15.0.4433; Pro)*/
const Device UA2456270 = {S17527, 236, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Media Center PC 6.0; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2468611 = {S17528, 244, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4433; Microsoft Outlook 15.0.4433; ms-office; MSOffice 15)*/
const Device UA2481391 = {S17529, 175, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2481644 = {S17530, 292, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2481927 = {S17531, 215, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2482180 = {S17532, 72, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Microsoft Office/15.0 (Windows NT 6.2; Microsoft Outlook 15.0.4454; Pro)*/
const Device UA2483246 = {S17533, 244, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4454; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2485931 = {S17534, 254, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4433; Microsoft Outlook 15.0.4433; ms-office; MSOffice 15)*/
const Device UA2502995 = {S17535, 265, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Media Center PC 6.0; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2507623 = {S17536, 236, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2507631 = {S17537, 220, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2527670 = {S17538, 254, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4454; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2530815 = {S17539, 230, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2553566 = {S17540, 302, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2553951 = {S17541, 204, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2554267 = {S17542, 253, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 15.0.4128; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA2554879 = {S17543, 237, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2556373 = {S17544, 225, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2557457 = {S17545, 259, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4454; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2559141 = {S17546, 196, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2560056 = {S17547, 241, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2568526 = {S17548, 220, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2569103 = {S17549, 215, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2569620 = {S17550, 244, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2569992 = {S17551, 249, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2579641 = {S17552, 225, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4433; ms-office; MSOffice 15)*/
const Device UA2580175 = {S17553, 72, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Microsoft Office/15.0 (Windows NT 6.2; Microsoft Outlook 15.0.4481; Pro)*/
const Device UA2580327 = {S17554, 249, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4454; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2580375 = {S17555, 253, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 15.0.4454; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2583555 = {S17556, 263, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2584257 = {S17557, 248, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2584314 = {S17558, 224, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2585059 = {S17559, 230, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2593833 = {S17560, 240, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2595363 = {S17561, 259, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4433; Microsoft Outlook 15.0.4433; ms-office; MSOffice 15)*/
const Device UA2595508 = {S17562, 251, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2599988 = {S17563, 225, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2602023 = {S17564, 230, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2605557 = {S17565, 196, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2606460 = {S17566, 264, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2653326 = {S17567, 236, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2659266 = {S17568, 269, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4454; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2666980 = {S17569, 253, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2667079 = {S17570, 292, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2667869 = {S17571, 235, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2668115 = {S17572, 277, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2668538 = {S17573, 237, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2668694 = {S17574, 273, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2668831 = {S17575, 254, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2668851 = {S17576, 259, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2669546 = {S17577, 259, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2703000 = {S17578, 221, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; HPNTDFJS; Tablet PC 2.0; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2704120 = {S17579, 239, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Media Center PC 6.0; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2707218 = {S17580, 273, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2711307 = {S17581, 278, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2727154 = {S17582, 288, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2727201 = {S17583, 278, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2727499 = {S17584, 273, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2729717 = {S17585, 203, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 15.0.4433; ms-office; MSOffice 15)*/
const Device UA2732968 = {S17586, 244, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2733220 = {S17587, 251, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2733221 = {S17588, 251, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Media Center PC 6.0; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2733252 = {S17589, 220, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2733470 = {S17590, 236, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2733702 = {S17591, 269, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2733905 = {S17592, 215, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2736042 = {S17593, 249, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2740042 = {S17594, 230, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2740048 = {S17595, 224, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2741529 = {S17596, 208, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2741617 = {S17597, 263, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2750061 = {S17598, 72, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Microsoft Office/15.0 (Windows NT 6.2; Microsoft Outlook 15.0.4505; Pro)*/
const Device UA2750460 = {S17599, 225, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2753019 = {S17600, 241, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2753257 = {S17601, 235, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2753372 = {S17602, 230, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2756548 = {S17603, 259, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2770865 = {S17604, 277, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2771320 = {S17605, 292, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2782455 = {S17606, 272, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Tablet PC 2.0; InfoPath.3; MALCJS; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2782448 = {S17607, 237, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2782547 = {S17608, 259, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2782789 = {S17609, 253, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2791455 = {S17610, 259, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4454; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2795231 = {S17611, 254, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2796463 = {S17612, 269, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2796490 = {S17613, 288, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2797018 = {S17614, 251, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2800145 = {S17615, 240, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2817891 = {S17616, 241, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA2824355 = {S17617, 235, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA2840601 = {S17618, 258, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2843831 = {S17619, 223, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2844726 = {S17620, 240, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2851399 = {S17621, 230, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2852174 = {S17622, 263, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2856659 = {S17623, 273, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4454; Microsoft Outlook 15.0.4454; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2858467 = {S17624, 268, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Media Center PC 6.0; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2866833 = {S17625, 236, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2867099 = {S17626, 273, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2868440 = {S17627, 278, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2868582 = {S17628, 273, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2868672 = {S17629, 258, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2868744 = {S17630, 219, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; MASMJS; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2868912 = {S17631, 239, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2870186 = {S17632, 124, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2892432 = {S17633, 153, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2947152 = {S17634, 208, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2947176 = {S17635, 244, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2947355 = {S17636, 220, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2948032 = {S17637, 72, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Microsoft Office/15.0 (Windows NT 6.2; Microsoft Outlook 15.0.4517; Pro)*/
const Device UA2948288 = {S17638, 248, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2948297 = {S17639, 249, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2948320 = {S17640, 224, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2948324 = {S17641, 215, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2966215 = {S17642, 235, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2966910 = {S17643, 213, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2967029 = {S17644, 230, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2967563 = {S17645, 263, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2968796 = {S17646, 277, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2969646 = {S17647, 230, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2970563 = {S17648, 264, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2970920 = {S17649, 259, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2982318 = {S17650, 317, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA3014239 = {S17651, 253, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3015213 = {S17652, 292, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3015750 = {S17653, 251, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3016710 = {S17654, 219, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; MASMJS; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3017691 = {S17655, 240, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3017714 = {S17656, 236, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3018109 = {S17657, 254, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3018298 = {S17658, 273, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3018440 = {S17659, 269, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3018925 = {S17660, 237, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3020093 = {S17661, 239, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3020882 = {S17662, 259, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3021186 = {S17663, 278, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3043376 = {S17664, 307, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA3057380 = {S17665, 258, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3057622 = {S17666, 124, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3057801 = {S17667, 153, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3077878 = {S17668, 273, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3078174 = {S17669, 288, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3083967 = {S17670, 203, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA3087835 = {S17671, 288, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA3088067 = {S17672, 240, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA3088397 = {S17673, 302, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA3094720 = {S17674, 220, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA3094874 = {S17675, 226, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; ASU2JS; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA3096126 = {S17676, 224, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA3096146 = {S17677, 208, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA3107576 = {S17678, 307, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3107672 = {S17679, 158, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Win64; x64; Trident/6.0; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3109443 = {S17680, 241, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA3109774 = {S17681, 259, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/7.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4535; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA3110639 = {S17682, 72, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Microsoft Office/15.0 (Windows NT 6.2; Microsoft Outlook 15.0.4535; Pro)*/
const Device UA3112218 = {S17683, 244, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4535; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA3115468 = {S17684, 249, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4535; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA3124809 = {S17685, 302, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3126643 = {S17686, 213, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA3126775 = {S17687, 263, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA3127712 = {S17688, 235, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA3128581 = {S17689, 230, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA3130285 = {S17690, 272, { &P15364,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Tablet PC 2.0; InfoPath.3; MALCJS; Microsoft Outlook 15.0.4535; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA174912 = {S17691, 118, { &P15364,&P7133,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.20 (KHTML, like Gecko) Chrome/11.0.672.2 Safari/534.20*/
const Device UA181109 = {S17692, 108, { &P15364,&P7133,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.1) AppleWebKit/534.23 (KHTML, like Gecko) Chrome/11.0.686.3 Safari/534.23*/
const Device UA195382 = {S17693, 100, { &P15364,&P7133,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA205402 = {S17694, 107, { &P15364,&P7133,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24*/
const Device UA319554 = {S17695, 137, { &P15364,&P7133,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.60 Safari/534.24,gzip(gfe) (via translate.google.com)*/
const Device UA673659 = {S17696, 144, { &P15364,&P7133,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24,gzip(gfe) (via translate.google.com)*/
const Device UA771860 = {S17697, 155, { &P15364,&P7133,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.20 (KHTML, like Gecko) Chrome/11.0.672.2 Safari/534.20,gzip(gfe) (via translate.google.com)*/
const Device UA1863875 = {S17698, 111, { &P15364,&P7133,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1) AppleWebKit/534.21 (KHTML, like Gecko) Chrome/11.0.682.0 Safari/534.21*/
const Device UA2146238 = {S17699, 199, { &P15364,&P7133,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.697.0 Safari/534.24Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA2213679 = {S17700, 118, { &P15364,&P7133,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; de-DE) AppleWebKit/534.19 (KHTML, like Gecko) Chrome/11.0.661.0 Safari/534.19*/
const Device UA2456018 = {S17701, 110, { &P15364,&P7133,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.60 Safari/534.24,gzip(gfe)*/
const Device UA2527248 = {S17702, 117, { &P15364,&P7133,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24,gzip(gfe)*/
const Device UA2596285 = {S17703, 99, { &P15364,&P7133,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML like Gecko) Chrome/11.0.696.57 Safari/534.24*/
const Device UA2348567 = {S17704, 132, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N8000 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2399898 = {S17705, 61, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-N8000 Build/JRO03C)*/
const Device UA2401575 = {S17706, 132, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; GT-N8020 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2404682 = {S17707, 152, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; SAMSUNG GT-N8000/N8000XXBLJ9 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422603 = {S17708, 132, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N8000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2424681 = {S17709, 152, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; SAMSUNG GT-N8020/N8020XXALJD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2456979 = {S17710, 61, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N8000 Build/JZO54K)*/
const Device UA2458928 = {S17711, 152, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; SAMSUNG GT-N8020/N8020XDALK1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2491399 = {S17712, 152, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-N8020/N8020XXBMA3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2530228 = {S17713, 132, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N8020 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2532555 = {S17714, 132, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; tr-tr; GT-N8005 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2538105 = {S17715, 152, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; SAMSUNG GT-N8000/N8000XXCMA1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2578868 = {S17716, 151, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GT-N8020-ORANGE/N8020XDALK1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2596905 = {S17717, 61, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-N8020 Build/JRO03C)*/
const Device UA2598147 = {S17718, 61, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N8020 Build/JZO54K)*/
const Device UA2604413 = {S17719, 151, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-N8020-ORANGE/N8020XDBMA2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2605401 = {S17720, 152, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; SAMSUNG GT-N8020/N8020XDBMA4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2659489 = {S17721, 152, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-at; SAMSUNG GT-N8020/N8020XDALK5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2659548 = {S17722, 152, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; SAMSUNG GT-N8020/N8020XXALK3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2661450 = {S17723, 152, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-at; SAMSUNG GT-N8020/N8020XDBMA2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2857682 = {S17724, 61, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N8005 Build/JZO54K)*/
const Device UA2972860 = {S17725, 152, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-N8020/N8020XXBMF2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2983236 = {S17726, 139, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N8000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986841 = {S17727, 130, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-N8000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3019200 = {S17728, 152, { &P18437,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-pt; SAMSUNG GT-N8000/N8000XXCMD2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2205896 = {S17729, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; es-ES) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M*/
const Device UA2205908 = {S17730, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; fr-fr) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M*/
const Device UA2205911 = {S17731, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-gb) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M*/
const Device UA2206027 = {S17732, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; fa-fa) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M*/
const Device UA2206039 = {S17733, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; fa-ir) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M*/
const Device UA2206054 = {S17734, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; zh-HK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M*/
const Device UA2206118 = {S17735, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-SG) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M*/
const Device UA2236461 = {S17736, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-za) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M*/
const Device UA2236463 = {S17737, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-ae) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M*/
const Device UA2237040 = {S17738, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; ar-EG) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M*/
const Device UA2237043 = {S17739, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; fr-ca) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M*/
const Device UA2237045 = {S17740, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; ar-KW) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M*/
const Device UA2237699 = {S17741, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; de-AT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M*/
const Device UA2238742 = {S17742, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-IN) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M*/
const Device UA2238748 = {S17743, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-PK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M*/
const Device UA2238806 = {S17744, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; ar-BH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M*/
const Device UA2238963 = {S17745, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-AS) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M*/
const Device UA2260082 = {S17746, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; nl-nl) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6838M*/
const Device UA2260316 = {S17747, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-us) AppleWebKit/534.35 (KHTML	 like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.4250M*/
const Device UA2353119 = {S17748, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; ar-OM) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6569M*/
const Device UA2357742 = {S17749, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-au) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6885M*/
const Device UA2357966 = {S17750, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; fa-IR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6885M*/
const Device UA2358168 = {S17751, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-ie) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6885M*/
const Device UA2358297 = {S17752, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; ru-ru) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6885M*/
const Device UA2358315 = {S17753, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; zh-hk) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6885M*/
const Device UA2361201 = {S17754, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; de-de) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6953M*/
const Device UA2368854 = {S17755, 128, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; it-it) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7055*/
const Device UA2369029 = {S17756, 128, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-sg) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6993*/
const Device UA2387135 = {S17757, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; sv-se) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6953M*/
const Device UA2406023 = {S17758, 128, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; ar-AE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.6993*/
const Device UA2431938 = {S17759, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-ca) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7310M*/
const Device UA2432016 = {S17760, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; ko-KR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7310M*/
const Device UA2432144 = {S17761, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; th-TH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7310M*/
const Device UA2432251 = {S17762, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; ro-RO) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7310M*/
const Device UA2432502 = {S17763, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; ar-SA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2432503 = {S17764, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-AU) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2432504 = {S17765, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; fr-FR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2432506 = {S17766, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; es-MX) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2432508 = {S17767, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; ja-JP) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2432509 = {S17768, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; zh-TW) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2432510 = {S17769, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; ar-ae) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2432511 = {S17770, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; nl-NL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2432512 = {S17771, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; de-DE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2432507 = {S17772, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; ko-kr) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2432814 = {S17773, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; pt-br) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7310M*/
const Device UA2432928 = {S17774, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; vi-vn) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7310M*/
const Device UA2433199 = {S17775, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; es-CL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7310M*/
const Device UA2433305 = {S17776, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; ru-RU) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2433307 = {S17777, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; pt-PT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2433310 = {S17778, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; pt-BR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2433596 = {S17779, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; es-es) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2433598 = {S17780, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-GB) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2433597 = {S17781, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2433595 = {S17782, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; it-IT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2433803 = {S17783, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; tr-TR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2433804 = {S17784, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-CA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2435844 = {S17785, 128, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; es-mx) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7193*/
const Device UA2460996 = {S17786, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-HK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2461180 = {S17787, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; th-th) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2461181 = {S17788, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-IE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2461182 = {S17789, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; nb-NO) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2461186 = {S17790, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; fr-CA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2461275 = {S17791, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; fr-CH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7310M*/
const Device UA2461927 = {S17792, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; he-IL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2461928 = {S17793, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; es-CO) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2461929 = {S17794, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; es-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2461931 = {S17795, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; zh-CN) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2461932 = {S17796, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; es-AR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2461934 = {S17797, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; de-CH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2461935 = {S17798, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-PH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2461936 = {S17799, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-NZ) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2461939 = {S17800, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; nl-BE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2461940 = {S17801, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; pl-PL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2461941 = {S17802, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; vi-VN) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2461942 = {S17803, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; da-DK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2461943 = {S17804, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; sv-SE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2461944 = {S17805, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; el-GR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2469256 = {S17806, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; es-VE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2469257 = {S17807, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-ZA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2469258 = {S17808, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; fr-BE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2469259 = {S17809, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; hr-HR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2469367 = {S17810, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; ro-ro) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2469370 = {S17811, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; de-at) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2469373 = {S17812, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; es-PE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2469889 = {S17813, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; fi-FI) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2470073 = {S17814, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; ar-eg) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2470074 = {S17815, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; ru-UA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2470083 = {S17816, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; hu-hu) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2485674 = {S17817, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; es-CR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2485802 = {S17818, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; id-ID) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2486941 = {S17819, 129, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; ms-MY) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M*/
const Device UA2513313 = {S17820, 128, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7812*/
const Device UA2513324 = {S17821, 128, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; tr-tr) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7812*/
const Device UA2513857 = {S17822, 128, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-in) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7812*/
const Device UA2514333 = {S17823, 128, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; es-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7812*/
const Device UA2514338 = {S17824, 128, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; zh-cn) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7812*/
const Device UA2516374 = {S17825, 128, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-nz) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7812*/
const Device UA2537699 = {S17826, 128, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; iw-il) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7812*/
const Device UA2538907 = {S17827, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-nz) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7910*/
const Device UA2538911 = {S17828, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; zh-hk) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7910*/
const Device UA2538955 = {S17829, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; tr-tr) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7910*/
const Device UA2538982 = {S17830, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; vi-VN) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7910*/
const Device UA2538984 = {S17831, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; nb-no) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7910*/
const Device UA2539527 = {S17832, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; el-GR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7910*/
const Device UA2539659 = {S17833, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; nl-be) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7910*/
const Device UA2561304 = {S17834, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ar-eg) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7910*/
const Device UA2561850 = {S17835, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; de-at) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7910*/
const Device UA2566065 = {S17836, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-GB) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2566066 = {S17837, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fa-IR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2566064 = {S17838, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2566068 = {S17839, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; zh-TW) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2566277 = {S17840, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fr-CA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2566276 = {S17841, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-AU) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2566283 = {S17842, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-MX) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2566295 = {S17843, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; sv-SE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2566563 = {S17844, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ar-ae) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2566567 = {S17845, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; he-IL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2566574 = {S17846, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-CA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2566577 = {S17847, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-CO) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2566579 = {S17848, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; it-IT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2567036 = {S17849, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; de-DE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2567040 = {S17850, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; th-TH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2567041 = {S17851, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fr-FR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2567042 = {S17852, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ar-SA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2567043 = {S17853, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-ES) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2567045 = {S17854, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ko-KR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2567047 = {S17855, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ja-JP) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2567354 = {S17856, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; tr-TR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2567359 = {S17857, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; zh-HK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2567595 = {S17858, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; vi-vn) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2567599 = {S17859, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; nl-NL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2567745 = {S17860, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-NZ) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2567756 = {S17861, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-PH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2568035 = {S17862, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2568034 = {S17863, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; nl-BE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2568176 = {S17864, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-SG) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2568185 = {S17865, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-ZA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2568193 = {S17866, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-IN) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2568203 = {S17867, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ru-RU) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2568394 = {S17868, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; pl-pl) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2568396 = {S17869, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-PK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2568410 = {S17870, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-HK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2568412 = {S17871, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; da-DK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2568765 = {S17872, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-IE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2569200 = {S17873, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; nb-NO) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2569442 = {S17874, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; pt-PT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2569560 = {S17875, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; zh-CN) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2569896 = {S17876, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ar-EG) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2570201 = {S17877, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; pt-BR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2570212 = {S17878, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; el-gr) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2583047 = {S17879, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-CL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2583051 = {S17880, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fr-CH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2583082 = {S17881, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-AR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2583138 = {S17882, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; de-CH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2583152 = {S17883, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ms-MY) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2583303 = {S17884, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; de-AT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2583379 = {S17885, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ro-RO) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2585478 = {S17886, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fr-BE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2585703 = {S17887, 127, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2600472 = {S17888, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ar-KW) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2601013 = {S17889, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; cs-CZ) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2601473 = {S17890, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-PE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2601523 = {S17891, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; bg-BG) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2602648 = {S17892, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ru-UA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2603149 = {S17893, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-VE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2603589 = {S17894, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fi-FI) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2603654 = {S17895, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; hu-HU) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2603777 = {S17896, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-EC) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2604211 = {S17897, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; iw-il) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2604834 = {S17898, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; id-ID) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2666706 = {S17899, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ca-ES) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2666707 = {S17900, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; sl-SI) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2666772 = {S17901, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-PR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2666836 = {S17902, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-CR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2666898 = {S17903, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; hr-HR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2666981 = {S17904, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; sk-SK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2667179 = {S17905, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; in-id) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2667276 = {S17906, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-BE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2667327 = {S17907, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-GT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2667670 = {S17908, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-DO) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2668035 = {S17909, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; it-CH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2668067 = {S17910, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; zh-MO) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2668075 = {S17911, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ar-BH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2668508 = {S17912, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-AS) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2668977 = {S17913, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ru-KZ) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2674277 = {S17914, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; it-it) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8506*/
const Device UA2676530 = {S17915, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-au) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8506*/
const Device UA2677368 = {S17916, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-es) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8506*/
const Device UA2702723 = {S17917, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-PA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2703257 = {S17918, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-VI) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2704530 = {S17919, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-MT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2705113 = {S17920, 128, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2705325 = {S17921, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-HN) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2705596 = {S17922, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ar-MA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2705873 = {S17923, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-SV) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2705903 = {S17924, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ar-OM) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2707102 = {S17925, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ar-QA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2707435 = {S17926, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-TT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2715197 = {S17927, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fr-fr) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8777*/
const Device UA2715354 = {S17928, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-ca) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8777*/
const Device UA2717840 = {S17929, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; pl-PL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8777*/
const Device UA2719362 = {S17930, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-gb) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8909*/
const Device UA2732736 = {S17931, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; uk-UA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2732827 = {S17932, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ar-IQ) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2732834 = {S17933, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-BB) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2732868 = {S17934, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ur-PK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2740215 = {S17935, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; zh-tw) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8909*/
const Device UA2749770 = {S17936, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; et-EE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2753225 = {S17937, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ru-ru) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.9039*/
const Device UA2765982 = {S17938, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-dz) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2770333 = {S17939, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.9133*/
const Device UA2793324 = {S17940, 136, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US-POSIX) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2855415 = {S17941, 148, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ja-JP@calendar=japanese) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2855619 = {S17942, 148, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US@calendar=buddhist) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2857031 = {S17943, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; bn-BD) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2857084 = {S17944, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-UM) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2857331 = {S17945, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; lt-LT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2858837 = {S17946, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-NG) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2859778 = {S17947, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; is-IS) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2863641 = {S17948, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ko-kr) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181*/
const Device UA2863755 = {S17949, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; de-de) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181*/
const Device UA2864045 = {S17950, 147, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fa-IR@calendar=persian) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2864232 = {S17951, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fa-fa) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181*/
const Device UA2864519 = {S17952, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fa-ir) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181*/
const Device UA2865175 = {S17953, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-tw) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961*/
const Device UA2878946 = {S17954, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181*/
const Device UA2878981 = {S17955, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-ae) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181*/
const Device UA2891935 = {S17956, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; pt-br) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181*/
const Device UA2891957 = {S17957, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; pt-pt) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181*/
const Device UA2956226 = {S17958, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-in) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181*/
const Device UA2995280 = {S17959, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; zh-cn) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181*/
const Device UA3001727 = {S17960, 128, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fa-) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181*/
const Device UA3006140 = {S17961, 140, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181,gzip(gfe)*/
const Device UA3020014 = {S17962, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-bh) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181*/
const Device UA3106260 = {S17963, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-za) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181*/
const Device UA3106988 = {S17964, 130, { &P15364,&P18110,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-jo) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181*/
const Device UA2470640 = {S17965, 111, { &P15364,&P18110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17*/
const Device UA2488024 = {S17966, 102, { &P15364,&P18110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17*/
const Device UA2510992 = {S17967, 114, { &P15364,&P18110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17,gzip(gfe)*/
const Device UA2515588 = {S17968, 104, { &P15364,&P18110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.70 Safari/537.17*/
const Device UA2530328 = {S17969, 112, { &P15364,&P18110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17,gzip(gfe)*/
const Device UA2602219 = {S17970, 114, { &P15364,&P18110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Unix; en-US) AppleWebKit/537.15 (KHTML, like Gecko) Chrome/24.0.1295.0 Safari/537.15 Surf/0.6*/
const Device UA2674608 = {S17971, 106, { &P15364,&P18110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/24.0.1410.63 Safari/537.31*/
const Device UA2674632 = {S17972, 109, { &P15364,&P18110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/24.0.1410.63 Safari/537.31*/
const Device UA2674652 = {S17973, 106, { &P15364,&P18110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/24.0.1410.63 Safari/537.31*/
const Device UA2674876 = {S17974, 106, { &P15364,&P18110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/24.0.1410.63 Safari/537.31*/
const Device UA2675208 = {S17975, 109, { &P15364,&P18110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/24.0.1410.63 Safari/537.31*/
const Device UA2715171 = {S17976, 109, { &P15364,&P18110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/24.0.1410.63 Safari/537.31*/
const Device UA2716576 = {S17977, 106, { &P15364,&P18110,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/24.0.1410.63 Safari/537.31*/
const Device UA438819 = {S17978, 140, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC One S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA438881 = {S17979, 140, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_One_S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA588577 = {S17980, 140, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC VLE_U Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA603888 = {S17981, 151, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_S/1.53.161.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA609222 = {S17982, 151, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_S/1.53.110.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA622307 = {S17983, 146, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC One S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.1 Mobile Safari/534.30*/
const Device UA625159 = {S17984, 150, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_S-orange-LS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA630586 = {S17985, 62, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC One S Build/IML74K)*/
const Device UA657844 = {S17986, 151, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_One_S/1.77.111.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA665228 = {S17987, 151, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC_One_S/1.53.166.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA667428 = {S17988, 62, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC VLE_U Build/IML74K)*/
const Device UA720815 = {S17989, 151, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_One_S/1.53.169.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA766397 = {S17990, 151, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_One_S/1.53.111.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA810569 = {S17991, 136, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; Z520m Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA863937 = {S17992, 151, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_S/1.78.161.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA895792 = {S17993, 151, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; HTC_One_S/1.53.114.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899052 = {S17994, 151, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_One_S/1.77.111.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA903867 = {S17995, 140, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC_VLE_U Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1954336 = {S17996, 140, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_One_S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1955234 = {S17997, 151, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC_One_S/1.53.163.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1960868 = {S17998, 151, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_One_S/1.53.112.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1970361 = {S17999, 151, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC_One_S/1.78.166.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1973927 = {S18000, 151, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_One_S/1.78.169.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1988147 = {S18001, 151, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_S/1.77.110.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1993205 = {S18002, 131, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC One S Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1996699 = {S18003, 140, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC One S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028664 = {S18004, 141, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; HTC One xs Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029184 = {S18005, 62, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC One S Build/IMM76D)*/
const Device UA2167961 = {S18006, 140, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC VLE_U Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2182443 = {S18007, 136, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; Z520m Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2192579 = {S18008, 151, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; HTC_One_S/2.38.169.6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2195947 = {S18009, 151, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-ch; HTC_One_S/2.38.166.6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2199434 = {S18010, 62, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC VLE_U Build/IMM76D)*/
const Device UA2199843 = {S18011, 150, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_One_S-orange-LS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2201861 = {S18012, 151, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_One_S/2.38.110.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2205763 = {S18013, 151, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_One_S/2.38.161.6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2218806 = {S18014, 151, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; HTC_One_S/1.77.114.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2234075 = {S18015, 161, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_One_S_pj40200/1.11.169.132 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2235191 = {S18016, 161, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_One_S_pj40200/1.11.161.133 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257443 = {S18017, 152, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HTC_One_S/2.38.111.10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2356305 = {S18018, 152, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; HTC_One_S/2.38.114.10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2386137 = {S18019, 152, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; HTC_One_S/2.38.112.10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2416989 = {S18020, 131, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC One S Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2497306 = {S18021, 153, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; HTC_One_S/1.11.118.122 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2527453 = {S18022, 147, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; HTC_One_SC_T528d Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2545584 = {S18023, 131, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; HTC One S Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2597163 = {S18024, 148, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneS/1.53.112.4 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597338 = {S18025, 137, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCOneS Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597686 = {S18026, 139, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC ONE S Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597920 = {S18027, 148, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneS/1.78.161.2 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2598150 = {S18028, 149, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCOneS/2.38.111.10 Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2598495 = {S18029, 148, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneS/1.77.111.4 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658396 = {S18030, 148, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneS/1.77.110.5 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658979 = {S18031, 148, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneS/1.53.110.3 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659156 = {S18032, 148, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCOneS/2.38.161.6 Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659452 = {S18033, 147, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCOneS-orange-LS Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2709163 = {S18034, 149, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; es-US; HTC_One_S/1.53.161.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2777227 = {S18035, 153, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-cz; HTC_One_S/1.11.113.123 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2809988 = {S18036, 138, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; HTC_One_S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2828801 = {S18037, 151, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; HTC_One_S/2.38.163.7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2849897 = {S18038, 148, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCOneS/2.38.166.6 Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2850482 = {S18039, 148, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneS/1.53.161.3 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2850665 = {S18040, 137, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneS Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057005 = {S18041, 141, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; HTC_One_SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3118885 = {S18042, 140, { &P18280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_One_S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2482854 = {S18043, 113, { &P17595,&P17296,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 Macintosh Intel Mac OS X 10_6_8 AppleWebKit/537.17 KHTML, like Gecko Chrome/24.0.1312.5 Safari/537.17*/
const Device UA2492151 = {S18044, 119, { &P17595,&P17296,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492193 = {S18045, 119, { &P17595,&P17296,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492214 = {S18046, 119, { &P17595,&P17296,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492218 = {S18047, 119, { &P17595,&P17296,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492311 = {S18048, 119, { &P17595,&P17296,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492337 = {S18049, 119, { &P17595,&P17296,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492362 = {S18050, 119, { &P17595,&P17296,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492393 = {S18051, 119, { &P17595,&P17296,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492559 = {S18052, 119, { &P17595,&P17296,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2510200 = {S18053, 158, { &P17595,&P17296,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2511641 = {S18054, 129, { &P17595,&P17296,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17,gzip(gfe)*/
const Device UA2674464 = {S18055, 129, { &P17595,&P17296,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/24.0.648.205 Safari/534.16*/
const Device UA2674659 = {S18056, 129, { &P17595,&P17296,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/24.0.648.205 Safari/534.16*/
const Device UA2715423 = {S18057, 129, { &P17595,&P17296,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/24.0.648.205 Safari/534.16*/
const Device UA3024559 = {S18058, 119, { &P17595,&P17297,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3024762 = {S18059, 119, { &P17595,&P17297,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3024965 = {S18060, 119, { &P17595,&P17297,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3025084 = {S18061, 119, { &P17595,&P17297,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3025367 = {S18062, 119, { &P17595,&P17297,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3027150 = {S18063, 119, { &P17595,&P17297,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3027410 = {S18064, 158, { &P17595,&P17297,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3029689 = {S18065, 129, { &P17595,&P17297,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe)*/
const Device UA366619 = {S18066, 127, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B5141a Safari/7534.48.3*/
const Device UA422343 = {S18067, 125, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7534.48.3*/
const Device UA428114 = {S18068, 125, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3*/
const Device UA587481 = {S18069, 125, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B157 Safari/7534.48.3*/
const Device UA588689 = {S18070, 125, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B169 Safari/7534.48.3*/
const Device UA590248 = {S18071, 125, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B174 Safari/7534.48.3*/
const Device UA599983 = {S18072, 123, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7534.46*/
const Device UA609260 = {S18073, 127, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA672158 = {S18074, 125, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.46*/
const Device UA804160 = {S18075, 127, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3*/
const Device UA899810 = {S18076, 127, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9Z999 Safari/7534.48.3*/
const Device UA907928 = {S18077, 117, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari Version/5.1*/
const Device UA909003 = {S18078, 115, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari Version/5.1*/
const Device UA1949593 = {S18079, 125, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B155 Safari/7534.48.3*/
const Device UA2161539 = {S18080, 125, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA2260489 = {S18081, 125, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B172 Safari/7534.48.3*/
const Device UA2260522 = {S18082, 125, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B179 Safari/7534.48.3*/
const Device UA2260573 = {S18083, 125, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Version/5.1 Safari/8536.25*/
const Device UA2331983 = {S18084, 127, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B202 Safari/7534.48.3*/
const Device UA2354934 = {S18085, 137, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Version/5.1 Safari/7534.48.3 Kikin/1.9.5*/
const Device UA2361916 = {S18086, 152, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X; zh-cn Model:iPad3,1) UC AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7543.48.3*/
const Device UA2475023 = {S18087, 127, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3*/
const Device UA2504798 = {S18088, 128, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/10A523 Safari/7534.48.3*/
const Device UA2577715 = {S18089, 123, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Version/5.1 Safari/8536.25*/
const Device UA2692476 = {S18090, 128, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/10A403 Safari/7534.48.3*/
const Device UA2817920 = {S18091, 127, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B208 Safari/7534.48.3*/
const Device UA2979502 = {S18092, 127, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7534.48.3*/
const Device UA3072501 = {S18093, 128, { &P15767,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/10B329 Safari/7534.48.3*/
const Device UA897299 = {S18094, 99, { &P15364,&P5690,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5*/
const Device UA897344 = {S18095, 106, { &P15364,&P5690,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5*/
const Device UA898193 = {S18096, 138, { &P15364,&P5690,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA900906 = {S18097, 99, { &P15364,&P5690,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA901082 = {S18098, 136, { &P15364,&P5690,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5,gzip(gfe) (via translate.google.com)*/
const Device UA905648 = {S18099, 106, { &P15364,&P5690,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA2259071 = {S18100, 109, { &P15364,&P5690,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5,gzip(gfe)*/
const Device UA2363058 = {S18101, 125, { &P15364,&P5690,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5*/
const Device UA2387120 = {S18102, 132, { &P15364,&P5690,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5*/
const Device UA2400994 = {S18103, 120, { &P15364,&P5690,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Windows NT 5.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/19.0.1084.36 Safari/535.19*/
const Device UA2423625 = {S18104, 125, { &P15364,&P5690,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5*/
const Device UA2424043 = {S18105, 135, { &P15364,&P5690,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5,gzip(gfe)*/
const Device UA54620 = {S18106, 141, { &P15767,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.10*/
const Device UA63028 = {S18107, 140, { &P15767,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA89789 = {S18108, 146, { &P15767,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0(iPad; U; CPU iPhone OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B314 Safari/531.21.10*/
const Device UA90113 = {S18109, 142, { &P15767,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2_1 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B405 Safari/531.21.10*/
const Device UA124466 = {S18110, 141, { &P15767,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7W367a Safari/531.21.10*/
const Device UA126344 = {S18111, 142, { &P15767,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B500 Safari/531.21.10*/
const Device UA279616 = {S18112, 139, { &P15767,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/7B367 Safari/6531.22.7*/
const Device UA291888 = {S18113, 153, { &P15767,&P17141,&P4201,&P18092 }};	/*Useragent - UserAgent: Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B405 Safari/531.21.10*/
const Device UA623410 = {S18114, 237, { &P15767,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176,Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA675428 = {S18115, 135, { &P15767,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B500 Safari/53*/
const Device UA692881 = {S18116, 142, { &P15767,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/9Z999 Safari/531.21.10*/
const Device UA742583 = {S18117, 155, { &P15767,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.10 Safari/525.20*/
const Device UA747262 = {S18118, 239, { &P15767,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206,Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2139544 = {S18119, 140, { &P15767,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; es-es) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B360 Safari/531.21.10*/
const Device UA2222148 = {S18120, 155, { &P15767,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.X-10-16 20:23:50*/
const Device UA2260100 = {S18121, 140, { &P15767,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0(iPad; U; CPU iPhone OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B314 Safari/123*/
const Device UA2413352 = {S18122, 143, { &P15767,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.10*/
const Device UA2512896 = {S18123, 141, { &P15767,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21 (KHTML, like Gecko) Version/4.0.4 Mobile/7B314 Safari/531.21*/
const Device UA2662517 = {S18124, 139, { &P15767,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 322 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML like Gecko) Version/4.0.4 Mobile/7B500 Safari/531.21.10*/
const Device UA2804633 = {S18125, 147, { &P15767,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.102.3.3)*/
const Device UA2807438 = {S18126, 156, { &P15767,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.1021.10gin_lib.cc*/
const Device UA2977867 = {S18127, 130, { &P15767,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B314*/
const Device UA73936 = {S18128, 142, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA358813 = {S18129, 160, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-N7000-ORANGE/N7000XXKKA Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA361477 = {S18130, 143, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHV-E160K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA580178 = {S18131, 67, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHV-E160K Build/GINGERBREAD)*/
const Device UA589996 = {S18132, 161, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-N7000/N7000XXKL7 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622973 = {S18133, 142, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA624176 = {S18134, 140, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ja-jp; SC-05D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA624898 = {S18135, 143, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-N7000B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625023 = {S18136, 143, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHV-E160S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625992 = {S18137, 66, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-N7000 Build/GINGERBREAD)*/
const Device UA630848 = {S18138, 161, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; SAMSUNG GT-N7000/N7000XXLC1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA631959 = {S18139, 161, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ie; SAMSUNG GT-N7000/N7000XXLB1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA639113 = {S18140, 161, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-N7000/N7000XXKKA Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA641691 = {S18141, 143, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHV-E160L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA644305 = {S18142, 66, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-N7000 Build/GINGERBREAD)*/
const Device UA650795 = {S18143, 67, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHV-E160L Build/GINGERBREAD)*/
const Device UA654723 = {S18144, 67, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHV-E160S Build/GINGERBREAD)*/
const Device UA655905 = {S18145, 130, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-N7000 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA675507 = {S18146, 142, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i889 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA687253 = {S18147, 154, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6-mobile-star; iw-il; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA702203 = {S18148, 161, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-N7000/N7000XXLA6 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA716762 = {S18149, 130, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-N7000 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA737979 = {S18150, 140, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA795155 = {S18151, 64, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SC-05D Build/GINGERBREAD)*/
const Device UA796091 = {S18152, 120, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; fr-fr; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA840442 = {S18153, 160, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-N7000-ORANGE/N7000XXLA6 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA905216 = {S18154, 135, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-N7000 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1952780 = {S18155, 135, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-N7000 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1961076 = {S18156, 67, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-N7000B Build/GINGERBREAD)*/
const Device UA1961505 = {S18157, 120, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-ie; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA1974250 = {S18158, 120, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; de-de; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA1994075 = {S18159, 120, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; it-it; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2026532 = {S18160, 135, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; GT-N7000 Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2030118 = {S18161, 66, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-i889 Build/GINGERBREAD)*/
const Device UA2031318 = {S18162, 160, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-N7000-ORANGE/N7000XXLC1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031654 = {S18163, 135, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i889 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2031687 = {S18164, 135, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; GT-N7000 Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2032863 = {S18165, 142, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033832 = {S18166, 60, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-N7000 Build/GRJ22)*/
const Device UA2160749 = {S18167, 120, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-gb; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2174877 = {S18168, 136, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT-N7000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2493974 = {S18169, 140, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2545486 = {S18170, 136, { &P16356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SHV-E160L Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2675137 = {S18171, 111, { &P15364,&P17285,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe)*/
const Device UA2675749 = {S18172, 140, { &P15364,&P17285,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2678094 = {S18173, 118, { &P15364,&P17285,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe)*/
const Device UA2678363 = {S18174, 97, { &P15364,&P17285,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.1.3.0 Safari/537.31*/
const Device UA2678370 = {S18175, 104, { &P15364,&P17285,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.1.3.0 Safari/537.31*/
const Device UA2678397 = {S18176, 101, { &P15364,&P17285,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2678526 = {S18177, 108, { &P15364,&P17285,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2679097 = {S18178, 147, { &P15364,&P17285,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2679937 = {S18179, 104, { &P15364,&P17285,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.2.2.0 Safari/537.31*/
const Device UA2679938 = {S18180, 97, { &P15364,&P17285,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.2.2.0 Safari/537.31*/
const Device UA396884 = {S18181, 97, { &P15364,&P18110,&P20806,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; C) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.8.0 Safari/533.3*/
const Device UA628885 = {S18182, 93, { &P15364,&P18110,&P20806,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.1 Safari/534.34*/
const Device UA2387489 = {S18183, 79, { &P15364,&P18110,&P20806,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.2*/
const Device UA2473962 = {S18184, 93, { &P15364,&P18110,&P20806,&P18092 }};	/*Useragent - Mozilla/5.0 (QtEmbedded; Linux) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.0 Safari/534.34*/
const Device UA2482732 = {S18185, 97, { &P15364,&P18110,&P20806,&P18092 }};	/*Useragent - Mozilla/5.0 (QtEmbedded; Linux; en) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.1 Safari/534.34*/
const Device UA2555660 = {S18186, 97, { &P15364,&P18110,&P20806,&P18092 }};	/*Useragent - Mozilla/5.0 (QtEmbedded; Linux; po) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.1 Safari/534.34*/
const Device UA2695284 = {S18187, 97, { &P15364,&P18110,&P20806,&P18092 }};	/*Useragent - Mozilla/5.0 (QtEmbedded; Linux; de) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.1 Safari/534.34*/
const Device UA2729801 = {S18188, 93, { &P15364,&P18110,&P20806,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.21 (KHTML, like Gecko) Qt/4.8.4 Safari/537.21*/
const Device UA2751054 = {S18189, 99, { &P15364,&P18110,&P20806,&P18092 }};	/*Useragent - Mozilla/5.0 (QtEmbedded; Linux; en-US) AppleWebKit/537.4 (KHTML, like Gecko) Qt/4.8.3 Safari/534.34*/
const Device UA2997092 = {S18190, 99, { &P15364,&P18110,&P20806,&P18092 }};	/*Useragent - Mozilla/5.0 (QtEmbedded; Linux; en-GB) AppleWebKit/537.4 (KHTML, like Gecko) Qt/4.8.3 Safari/534.34*/
const Device UA3099686 = {S18191, 97, { &P15364,&P18110,&P20806,&P18092 }};	/*Useragent - Mozilla/5.0 (QtEmbedded; Linux; fr) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.1 Safari/534.34*/
const Device UA2898310 = {S18192, 102, { &P15364,&P18110,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36*/
const Device UA2898311 = {S18193, 104, { &P15364,&P18110,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36*/
const Device UA2898326 = {S18194, 111, { &P15364,&P18110,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36*/
const Device UA2966445 = {S18195, 114, { &P15364,&P18110,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36,gzip(gfe)*/
const Device UA3011791 = {S18196, 112, { &P15364,&P18110,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36,gzip(gfe)*/
const Device UA3015162 = {S18197, 143, { &P15364,&P18110,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA608744 = {S18198, 38, { &P18254,&P7480,&P18245,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Blog Search;)*/
const Device UA1950569 = {S18199, 129, { &P18254,&P7480,&P18245,&P18092 }};	/*Useragent - NetworkedBlogs (+http://www.networkedblogs.com;) AppEngine-Google; (+http://code.google.com/appengine; appid: s~networkedblogshr)*/
const Device UA2355804 = {S18200, 79, { &P18254,&P7480,&P18245,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; LinkChecker/8.2; +http://linkchecker.sourceforge.net/)*/
const Device UA2359488 = {S18201, 41, { &P18254,&P7480,&P18245,&P18092 }};	/*Useragent - OppO 1.0 ( http://www.inboundscore.com/ )*/
const Device UA2408160 = {S18202, 58, { &P18254,&P7480,&P18245,&P18092 }};	/*Useragent - Open Web Indexer (+http://www.icedteapowered.com/openweb/)*/
const Device UA2414160 = {S18203, 41, { &P18254,&P7480,&P18245,&P18092 }};	/*Useragent - BellPagesCA/1.0; +http://www.BellPages.ca*/
const Device UA2429900 = {S18204, 63, { &P18254,&P7480,&P18245,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Allrati/1.2; +http://www.allrati.com/)*/
const Device UA2461827 = {S18205, 64, { &P18254,&P7480,&P18245,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Allrati/1.21; +http://www.allrati.com/)*/
const Device UA2492471 = {S18206, 98, { &P18254,&P7480,&P18245,&P18092 }};	/*Useragent - PulseParser/0.1 AppEngine-Google; (+http://code.google.com/appengine; appid: s~hr-pulsesubscriber)*/
const Device UA2511521 = {S18207, 42, { &P18254,&P7480,&P18245,&P18092 }};	/*Useragent - SiteExplorer/1.0 http://siteexplorer.info/*/
const Device UA2513073 = {S18208, 98, { &P18254,&P7480,&P18245,&P18092 }};	/*Useragent - PulseParser/0.3 AppEngine-Google; (+http://code.google.com/appengine; appid: s~hr-pulsesubscriber)*/
const Device UA2515521 = {S18209, 71, { &P18254,&P7480,&P18245,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; SiteExplorer/1.0b; +http://siteexplorer.info/)*/
const Device UA2516720 = {S18210, 61, { &P18254,&P7480,&P18245,&P18092 }};	/*Useragent - RockmeltEmbedder (http://rockmelt.com; feedback@rockmelt.com)*/
const Device UA2657814 = {S18211, 44, { &P18254,&P7480,&P18245,&P18092 }};	/*Useragent - BUbiNG (+http://law.di.unimi.it/BUbiNG.html)*/
const Device UA2679672 = {S18212, 62, { &P18254,&P7480,&P18245,&P18092 }};	/*Useragent - A6-Indexer/1.0 (http://www.a6corp.com/a6-web-scraping-policy/)*/
const Device UA2708739 = {S18213, 84, { &P18254,&P7480,&P18245,&P18092 }};	/*Useragent - Feed-Fetcher AppEngine-Google; (+http://code.google.com/appengine; appid: s~ziin-me)*/
const Device UA2732342 = {S18214, 68, { &P18254,&P7480,&P18245,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Web-Monitoring/1.0; +http://monoid.nic.ru/)*/
const Device UA2737635 = {S18215, 36, { &P18254,&P7480,&P18245,&P18092 }};	/*Useragent - VSE/1.0 (John.Macartney@cengage.com)*/
const Device UA2784785 = {S18216, 34, { &P18254,&P7480,&P18245,&P18092 }};	/*Useragent - Scrapy/0.16.5 (+http://scrapy.org)*/
const Device UA2674473 = {S18217, 129, { &P17595,&P17296,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/26.0.648.205 Safari/534.16*/
const Device UA2674628 = {S18218, 129, { &P17595,&P17296,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/26.0.648.205 Safari/534.16*/
const Device UA2674925 = {S18219, 129, { &P17595,&P17296,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31,gzip(gfe)*/
const Device UA2677328 = {S18220, 158, { &P17595,&P17296,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2678586 = {S18221, 119, { &P17595,&P17296,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA2678592 = {S18222, 119, { &P17595,&P17296,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA2678593 = {S18223, 119, { &P17595,&P17296,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA2678730 = {S18224, 119, { &P17595,&P17296,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA2678733 = {S18225, 119, { &P17595,&P17296,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA2678744 = {S18226, 119, { &P17595,&P17296,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA2679176 = {S18227, 119, { &P17595,&P17296,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA2679340 = {S18228, 119, { &P17595,&P17296,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA2679527 = {S18229, 119, { &P17595,&P17296,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA2718745 = {S18230, 129, { &P17595,&P17296,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/26.0.648.205 Safari/534.16*/
const Device UA199106 = {S18231, 100, { &P15364,&P5690,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.91 Safari/534.30*/
const Device UA204611 = {S18232, 138, { &P15364,&P5690,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30,gzip(gfe) (via translate.google.com)*/
const Device UA210469 = {S18233, 101, { &P15364,&P5690,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30*/
const Device UA219919 = {S18234, 108, { &P15364,&P5690,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA663952 = {S18235, 140, { &P15364,&P5690,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA776353 = {S18236, 120, { &P15364,&P5690,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.724.100 Safari/534.30*/
const Device UA1887323 = {S18237, 109, { &P15364,&P5690,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.27 (KHTML, like Gecko) Chrome/12.0.712.0 (Chromium Portable 9.2)*/
const Device UA1993352 = {S18238, 132, { &P15364,&P5690,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/12.0.702.0 Safari/533.3 Awesomium/1.6.5*/
const Device UA2292628 = {S18239, 114, { &P15364,&P5690,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30, SurfCop 1.5*/
const Device UA2340403 = {S18240, 107, { &P15364,&P5690,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.30 (KHTML	 like Gecko) Chrome/12.0.742.91 Safari/534.30*/
const Device UA2355681 = {S18241, 111, { &P15364,&P5690,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30,gzip(gfe)*/
const Device UA2374372 = {S18242, 109, { &P15364,&P5690,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; de) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/12.0 Safari/525.19*/
const Device UA2597904 = {S18243, 99, { &P15364,&P5690,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML like Gecko) Chrome/12.0.742.53 Safari/534.30*/
const Device UA2673267 = {S18244, 120, { &P15364,&P5690,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-CA) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.724.100 Safari/534.30*/
const Device UA2709381 = {S18245, 120, { &P15364,&P5690,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; es-US) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.724.100 Safari/534.30*/
const Device UA2709512 = {S18246, 120, { &P15364,&P5690,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; fr-CA) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.724.100 Safari/534.30*/
const Device UA2709514 = {S18247, 121, { &P15364,&P5690,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; haw-US) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.724.100 Safari/534.30*/
const Device UA2712119 = {S18248, 120, { &P15364,&P5690,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; br-FR) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.724.100 Safari/534.30*/
const Device UA2751232 = {S18249, 120, { &P15364,&P5690,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.724.100 Safari/534.30*/
const Device UA2758853 = {S18250, 115, { &P15364,&P5690,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; Alibrowser 1.0) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.9 Safari/534.30*/
const Device UA2759015 = {S18251, 109, { &P15364,&P5690,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.27 (KHTML, like Gecko) Chrome/12.0.712.0 (Chromium Portable 9.2)*/
const Device UA2980695 = {S18252, 119, { &P15364,&P5690,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.30 (KHTML like Gecko) Chrome/12.0.724.100 Safari/534.30*/
const Device UA2203743 = {S18253, 102, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203777 = {S18254, 100, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2233625 = {S18255, 119, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko; Google Web Preview) Chrome/22.0.1229 Safari/537.4*/
const Device UA2235704 = {S18256, 109, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229 Safari/537.4*/
const Device UA2238620 = {S18257, 109, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2239325 = {S18258, 127, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko; Google Page Speed Insights) Chrome/22.0.1229 Safari/537.4*/
const Device UA2239371 = {S18259, 158, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko; Google Web Preview) Chrome/22.0.1229 Safari/537.4,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2260199 = {S18260, 110, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)*/
const Device UA2352127 = {S18261, 137, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4,gzip(gfe) (via translate.google.com)*/
const Device UA2355584 = {S18262, 112, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)*/
const Device UA2403502 = {S18263, 101, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML like Gecko) Chrome/22.0.1229.56 Safari/537.4*/
const Device UA2530116 = {S18264, 102, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux armv6l) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2596664 = {S18265, 93, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4*/
const Device UA2674626 = {S18266, 109, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/22.0.1410.63 Safari/537.31*/
const Device UA2674644 = {S18267, 106, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/22.0.1410.63 Safari/537.31*/
const Device UA2674654 = {S18268, 106, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/22.0.1410.63 Safari/537.31*/
const Device UA2675185 = {S18269, 109, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/22.0.1410.63 Safari/537.31*/
const Device UA2675191 = {S18270, 106, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/22.0.1410.63 Safari/537.31*/
const Device UA2716514 = {S18271, 109, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/22.0.1410.63 Safari/537.31*/
const Device UA2716820 = {S18272, 106, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/22.0.1410.63 Safari/537.31*/
const Device UA2983065 = {S18273, 129, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko; Google Web Preview) Chrome/22.0.1229 Safari/537.4,gzip(gfe)*/
const Device UA2983899 = {S18274, 103, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i586) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2994543 = {S18275, 103, { &P15364,&P18110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4,gzip(gfe)*/
const Device UA2240792 = {S18276, 139, { &P18595,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I8190 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2406873 = {S18277, 61, { &P18595,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-I8190 Build/JRO03H)*/
const Device UA2459479 = {S18278, 159, { &P18595,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; SAMSUNG GT-I8190/I8190XXALK6 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2459527 = {S18279, 159, { &P18595,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; SAMSUNG GT-I8190/I8190XXALJL Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2461034 = {S18280, 159, { &P18595,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; SAMSUNG GT-I8190/I8190XXALL2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2487324 = {S18281, 139, { &P18595,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8190 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2504871 = {S18282, 151, { &P18595,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1_????-??????; he-il; GT-I8190 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2505880 = {S18283, 137, { &P18595,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-I8190 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2506984 = {S18284, 61, { &P18595,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8190 Build/JZO54K)*/
const Device UA2512818 = {S18285, 159, { &P18595,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; SAMSUNG GT-I8190/I8190XXALK1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2571599 = {S18286, 159, { &P18595,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I8190/I8190XXAMB3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2706101 = {S18287, 137, { &P18595,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-I8190 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2707796 = {S18288, 137, { &P18595,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I8190 Build/JRO03H) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2709502 = {S18289, 137, { &P18595,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I8190 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2887168 = {S18290, 159, { &P18595,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I8190/I8190XXAME1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2938234 = {S18291, 137, { &P18595,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I8190 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2963152 = {S18292, 159, { &P18595,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I8190/I8190XXAMD3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996707 = {S18293, 139, { &P18595,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I8190 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998807 = {S18294, 135, { &P18595,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-I8190 Build/JRO03H) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3023910 = {S18295, 159, { &P18595,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-pt; SAMSUNG GT-I8190/I8190XXAMG1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3047682 = {S18296, 134, { &P18595,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; I8190 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3087841 = {S18297, 159, { &P18595,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; el-gr; SAMSUNG GT-I8190/I8190XXAMB2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA892469 = {S18298, 132, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA902475 = {S18299, 132, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1972524 = {S18300, 61, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P5110 Build/IMM76D)*/
const Device UA1987741 = {S18301, 61, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-P5110 Build/IML74K)*/
const Device UA2257301 = {S18302, 130, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257843 = {S18303, 130, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2573575 = {S18304, 139, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2611383 = {S18305, 153, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5110 Build/IML74K; 1280*752) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2611690 = {S18306, 130, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5110 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2614124 = {S18307, 69, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - BBC iPlayer (AndroidApp; 1.3; 13014) GT-P5110 (Android 4.0.4, SDK 15)*/
const Device UA2614914 = {S18308, 80, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Biblia/2.1.2 (Linux; U; Android 4.0.3; es-US; GT-P5110 Build/IML74K.P5110XXALD6)*/
const Device UA2615032 = {S18309, 139, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2615096 = {S18310, 93, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.1.2 (Linux; U; Android 4.0.4; en-GB; GT-P5110 Build/IMM76D.P5110XXBLH3)*/
const Device UA2615285 = {S18311, 93, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.youtube/4.1.23(Linux; U; Android 4.0.3; en_AU; GT-P5110 Build/IML74K) gzip*/
const Device UA2615680 = {S18312, 80, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Biblia/2.1.2 (Linux; U; Android 4.0.3; es-ES; GT-P5110 Build/IML74K.P5110XWALD4)*/
const Device UA2615939 = {S18313, 130, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5110 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2616025 = {S18314, 63, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.0; 12) GT-P5110 (Android 4.0.4, SDK 15)*/
const Device UA2616559 = {S18315, 218, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.2.1-65 (Android 4.0.3; samsung GT-P5110 Build/IML74K.P5110XWALD2)*/
const Device UA2616650 = {S18316, 103, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 4.0.4; SAMSUNG GT-P5110 Build/ICE_CREAM_SANDWICH_MR1);*/
const Device UA2616900 = {S18317, 79, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.0.4; nl-BE; GT-P5110 Build/IMM76D.P5110XXBLI1)*/
const Device UA2617001 = {S18318, 145, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2617948 = {S18319, 153, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5110 Build/IMM76D; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2618543 = {S18320, 79, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Vyrso/2.2.3 (Linux; U; Android 4.0.3; en-AU; GT-P5110 Build/IML74K.P5110XWALE2)*/
const Device UA2618549 = {S18321, 79, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.3; de-DE; GT-P5110 Build/IML74K.P5110XWALD2)*/
const Device UA2618899 = {S18322, 162, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5110 Build/IML74K) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2619171 = {S18323, 79, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.3; en-NZ; GT-P5110 Build/IML74K.P5110XWALE1)*/
const Device UA2619600 = {S18324, 63, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.1; 17) GT-P5110 (Android 4.0.4, SDK 15)*/
const Device UA2620142 = {S18325, 145, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hr-hr; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30*/
const Device UA2620332 = {S18326, 134, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Groupon/2.1.2483 (Android 4.0.3; Samsung Espresso10wifi / Samsung GT-P5110; )[preload=false;locale=es_US;clientidbase=android-samsung]*/
const Device UA2620553 = {S18327, 94, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.apps.plus/X (Linux; U; Android 4.0.3; zh_TW; GT-P5110; Build/IML74K) (gzip)*/
const Device UA2620651 = {S18328, 94, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.apps.plus/X (Linux; U; Android 4.0.4; en_US; GT-P5110; Build/IMM76D) (gzip)*/
const Device UA2620900 = {S18329, 264, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/1.9.6;FBDM/{density=1.0,width=1280,height=752};FBLC/es_US;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P5110;FBSV/4.0.3;]*/
const Device UA2621086 = {S18330, 111, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.4; de-DE; GT-P5110 Build/cm_p5110-userdebug 4.0.4 IMM76L eng..X.X test-keys)*/
const Device UA2621836 = {S18331, 223, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 sessionId/e4eb0725-5a8d-403c-b7ef-2f571f6a5400 tiantian(securitypay) tiantian(tenpay)*/
const Device UA2622516 = {S18332, 265, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5110 Build/IML74K) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 baidubrowser/061_1.41.3.2_diordna_2321_008/gnusmas_51_3.0.4_0115P-TG/1200a/C88EF6E4BA2CB8DA86308DB47B04845C%7C0/1*/
const Device UA2622638 = {S18333, 134, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5110 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2622917 = {S18334, 80, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - VKAndroidApp/2.7.6-64 (Android 4.0.3; SDK 15; armeabi-v7a; samsung GT-P5110; ru)*/
const Device UA2623379 = {S18335, 163, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/5.0.2 Version/4.0 Safari/534.30 Safari/6533.18.5*/
const Device UA2623541 = {S18336, 132, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2623573 = {S18337, 221, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko)6.X.1.1.utmcsr=google|utmccn=(organic)|utmcmd=organic|utmctr=sofas; _msuuid_5620s53616=BDA191D0-2F2A-43DC-B131-0BAX*/
const Device UA2623575 = {S18338, 183, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5110 Buil=X; __utma=X.X.X.X.X.1; __utmb=X.12.10.X; __utmc=X; __utmz=X.X.1.1.utmcsr=google|utmccn=(organic)|utmcmd=organic|utmctr=sofas*/
const Device UA2623684 = {S18339, 134, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5110 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2624229 = {S18340, 93, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.youtube/4.0.23(Linux; U; Android 4.0.4; en_GB; GT-P5110 Build/IMM76D) gzip*/
const Device UA2625543 = {S18341, 94, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.2.10 (Linux; U; Android 4.0.4; nb-NO; GT-P5110 Build/IMM76D.P5110XXBLH4)*/
const Device UA2625895 = {S18342, 130, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5110 Build/IMM76L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2627322 = {S18343, 218, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.3.5-95 (Android 4.0.4; samsung GT-P5110 Build/IMM76D.P5110XXBLH4)*/
const Device UA2627503 = {S18344, 257, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5110 Build/IMM76D) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 baidubrowser/061_1.41.3.2_diordna_257_0821/gnusmas_51_4.0.4_0115P-TG/Xf/40C92E4CB5D423DXF0E2C882FDC%7C0/1*/
const Device UA2628699 = {S18345, 144, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; GT-P5110 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2629665 = {S18346, 151, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; GT-P5110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MantanoReader*/
const Device UA2633895 = {S18347, 288, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 NAVER(inapp; cafe; 100; 2.9.10) com.nhn.android.navercafe/2.9.10 (Linux; U; Android 4.0.3; ko-kr; GT-P5110 Build/IML74K; samsung) 800X1232 samsung GT-P5110*/
const Device UA2635162 = {S18348, 62, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.1; 9) GT-P5110 (Android 4.0.3, SDK 15)*/
const Device UA2635589 = {S18349, 464, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5110urniture.co.uk%2pple26ei%3DaNlEUajNGsSJ7Ab3r4HYDQ%26usg%3DAFQjCNEKOKin3_wddMYo8ZfbgmAjmxb4jg%26bvm%3Dbv.X%2Cd.ZGU%22%2C%22landing%22%3A%22http%3A//www.harveysfurniture.co.uk/mobile/home.aspx%22%2C%22time%22%3AX%7D%5D%2C%22sessionLandingPage%22%3A%22http%3A//www.harveysfurniture.co.uk/mobile/home.aspx%22%2C%22__v%22%3A%7B%7D%7D; _qubitTracker=X.X; _qst=%5B2%2C0%5D; qb_etc_status=%7B%22c%22%3A%7B%7D%7D; qb_ss_*/
const Device UA2635588 = {S18350, 474, { &P15882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5110 Build/IMM76D) AppleWebKit/534.UajNGsSJ7Ab3r4HYGeck6usg%3DAFQjCNEKOKin3_wddMYo8ZfbgmAjmxb4jg%26bvm%3Dbv.X%2Cd.ZGU%22%2C%22landing%22%3A%22http%3A//www.harveysfurniture.furniture.co.uk%e.as%26ei%3DaNlEUajNGsSJ7Ab3r4HYDQ%26usg%3DAFQjCNEKOKin3_wddMYo8ZfbgmAjmxb4jg%26bvm%3Dbv.X%2Cd.ZGU%22%2C%22landing%22%3A%22http%3A//www.harveysfurniture.co.uk/mobile/home.aspx%22%2C%22time%22%3AX%7D%5D%2C%22sessionLandingPage%22%3A%22htt*/
const Device UA3111107 = {S18351, 119, { &P17595,&P18207,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3115650 = {S18352, 119, { &P17595,&P18207,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3115674 = {S18353, 119, { &P17595,&P18207,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3129769 = {S18354, 119, { &P17595,&P18207,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3129957 = {S18355, 119, { &P17595,&P18207,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3130850 = {S18356, 119, { &P17595,&P18207,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3131176 = {S18357, 129, { &P17595,&P18207,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36,gzip(gfe)*/
const Device UA93333 = {S18358, 65, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.00*/
const Device UA93491 = {S18359, 34, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Opera/8.00 (Windows NT 5.1; U; en)*/
const Device UA98415 = {S18360, 27, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Opera/8.01 (Windows NT 5.1)*/
const Device UA98994 = {S18361, 65, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; ru) Opera 8.01*/
const Device UA99143 = {S18362, 46, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; en) Opera 8.01*/
const Device UA99199 = {S18363, 46, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; en) Opera 8.00*/
const Device UA99897 = {S18364, 64, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.0*/
const Device UA120887 = {S18365, 34, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Opera/8.50 (Windows NT 5.1; U; en)*/
const Device UA130902 = {S18366, 65, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.02*/
const Device UA153193 = {S18367, 34, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Opera/8.xx (Windows NT 5.1; U; en)*/
const Device UA192045 = {S18368, 34, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Opera/8.53 (Windows NT 5.1; U; de)*/
const Device UA301332 = {S18369, 34, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Opera/8.52 (Windows NT 5.1; U; en)*/
const Device UA312642 = {S18370, 34, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Opera/8.51 (Windows NT 5.1; U; en)*/
const Device UA321516 = {S18371, 34, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Opera/8.54 (Windows NT 5.1; U; ru)*/
const Device UA462391 = {S18372, 34, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Opera/8.10 (Windows NT 5.1; U; en)*/
const Device UA473115 = {S18373, 34, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Opera/8.01 (Windows NT 5.1; U; en)*/
const Device UA500283 = {S18374, 34, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Opera/8.02 (Windows NT 5.1; U; en)*/
const Device UA599786 = {S18375, 74, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.0	Opera 8.0*/
const Device UA812686 = {S18376, 33, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Opera/8.0 (Windows NT 5.1; U; en)*/
const Device UA828650 = {S18377, 46, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; en) Opera 8.02*/
const Device UA848297 = {S18378, 34, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Opera/8.53 (Windows NT 5.2; U; en)*/
const Device UA915853 = {S18379, 45, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; en) Opera 8.0*/
const Device UA1909434 = {S18380, 47, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Opera/8.40 (Windows NT 5.1; U; pl) Version/10.8*/
const Device UA2140616 = {S18381, 47, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Opera/8.20 (Windows NT 5.1; U; pl) Version/10.8*/
const Device UA2147832 = {S18382, 62, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Opera/8.20 (Windows NT 5.1; U; pl) Presto/2.2.15 Version/10.10*/
const Device UA2150131 = {S18383, 62, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Opera/8.40 (Windows NT 5.1; U; pl) Presto/2.4.15 Version/10.10*/
const Device UA2262265 = {S18384, 47, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Opera/8.10 (Windows NT 5.1; U; pl) Version/10.8*/
const Device UA2276922 = {S18385, 47, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Opera/8.30 (Windows NT 5.1; U; pl) Version/10.8*/
const Device UA2305447 = {S18386, 62, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Opera/8.10 (Windows NT 5.1; U; pl) Presto/2.1.15 Version/10.10*/
const Device UA2409780 = {S18387, 62, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Opera/8.30 (Windows NT 5.1; U; pl) Presto/2.3.15 Version/10.10*/
const Device UA2502474 = {S18388, 93, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - ('Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.0', 'Opera 8.0, Windows XP')*/
const Device UA2801596 = {S18389, 34, { &P15364,&P5690,&P17213,&P18092 }};	/*Useragent - Opera/8.50 (Windows NT 5.2; U; ru)*/
const Device UA97723 = {S18390, 59, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Maxthon)*/
const Device UA100803 = {S18391, 78, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Maxthon; .NET CLR 1.1.4322)*/
const Device UA120854 = {S18392, 72, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Maxthon)*/
const Device UA139276 = {S18393, 115, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Maxthon; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.1)*/
const Device UA205701 = {S18394, 82, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322) Maxthon*/
const Device UA236529 = {S18395, 213, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Maxthon; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Embedded Web Browser from: http://bsalsa.com/; .NET CLR 1.1.4322; FDM; .NET CLR 2.0.50727)*/
const Device UA288183 = {S18396, 109, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; CNCDialer; Maxthon; .NET CLR 1.1.4322; .NET CLR 2.0.50727)*/
const Device UA293710 = {S18397, 83, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; MAXTHON)*/
const Device UA383635 = {S18398, 103, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; SV1; Maxthon; .NET CLR 1.1.4322; .NET CLR 2.0.50727)*/
const Device UA416679 = {S18399, 155, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MAXTHON)*/
const Device UA444960 = {S18400, 78, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Maxthon; .NET CLR 1.1.4322)*/
const Device UA520529 = {S18401, 64, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; SV1; MAXTHON)*/
const Device UA698255 = {S18402, 64, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; MAXTHON)*/
const Device UA702681 = {S18403, 150, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Maxthon; .NET CLR 2.0.50727)*/
const Device UA723474 = {S18404, 72, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; MAXTHON)*/
const Device UA765288 = {S18405, 175, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; MAXTHON)*/
const Device UA772342 = {S18406, 166, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Maxthon; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30)*/
const Device UA804720 = {S18407, 73, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Maxthon; Maxthon)*/
const Device UA1863295 = {S18408, 195, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Maxthon; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA1941325 = {S18409, 64, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.1; SV1; MAXTHON)*/
const Device UA1942731 = {S18410, 72, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/4.0; MAXTHON)*/
const Device UA1945353 = {S18411, 106, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)*/
const Device UA1946477 = {S18412, 160, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAXTHON)*/
const Device UA1948245 = {S18413, 160, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAXTHON)*/
const Device UA1989339 = {S18414, 72, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 5.1; Maxthon)*/
const Device UA1992686 = {S18415, 72, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Maxthon)*/
const Device UA1994657 = {S18416, 72, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; Maxthon)*/
const Device UA2141168 = {S18417, 213, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Maxthon; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2151308 = {S18418, 106, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)*/
const Device UA2154750 = {S18419, 72, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; Maxthon)*/
const Device UA2160336 = {S18420, 72, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; Maxthon)*/
const Device UA2162665 = {S18421, 72, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; Maxthon)*/
const Device UA2163669 = {S18422, 72, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Maxthon)*/
const Device UA2165670 = {S18423, 72, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; Maxthon)*/
const Device UA2167461 = {S18424, 72, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 5.2; Maxthon)*/
const Device UA2217074 = {S18425, 160, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAXTHON)*/
const Device UA2321142 = {S18426, 175, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; MAXTHON)*/
const Device UA2326416 = {S18427, 187, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MAXTHON)*/
const Device UA2330030 = {S18428, 187, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MAXTHON)*/
const Device UA2335464 = {S18429, 71, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; SV1; MAXTHON)*/
const Device UA2371003 = {S18430, 77, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; Maxthon)*/
const Device UA2408503 = {S18431, 164, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Maxthon; Maxthon; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2410022 = {S18432, 106, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)*/
const Device UA2591048 = {S18433, 97, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0(compatible; MSIE 8.0; Windows NT 5.2; Maxthon; .NET CLR 1.1.4322; .NET CLR 2.0.50727)*/
const Device UA2682171 = {S18434, 59, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 5.1; Maxthon)*/
const Device UA2686478 = {S18435, 87, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; Maxthon; Avant Browser)*/
const Device UA2686972 = {S18436, 77, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; SV1; Maxthon)*/
const Device UA2741804 = {S18437, 128, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Avant Browser; Avant Browser; Maxthon; .NET CLR 2.0.50727; .NET CLR 1.1.4322)*/
const Device UA2741888 = {S18438, 71, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; SV1; MAXTHON)*/
const Device UA2743537 = {S18439, 77, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Maxthon; SV1)*/
const Device UA2743655 = {S18440, 259, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; EmbeddedWB 14.52 from: http://www.bsalsa.com/ EmbeddedWB 14.52; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C; .NET4.0E; MAXTHON)*/
const Device UA2745100 = {S18441, 81, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; Maxthon; Maxthon)*/
const Device UA2746685 = {S18442, 106, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)*/
const Device UA2759270 = {S18443, 106, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)*/
const Device UA2787235 = {S18444, 77, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Maxthon; SV1)*/
const Device UA2796625 = {S18445, 135, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0   (compatible;   MSIE   7.0;   Windows   NT   5.1;   SV1;   Maxthon;   .NET   CLR   2.0.50727;   .NET   CLR   3.0.04506.30)*/
const Device UA2812062 = {S18446, 187, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MAXTHON)*/
const Device UA2813965 = {S18447, 71, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; WOW64; SV1; Maxthon)*/
const Device UA2815035 = {S18448, 81, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; Maxthon; Maxthon)*/
const Device UA2817002 = {S18449, 87, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; Avant Browser; Maxthon)*/
const Device UA2828636 = {S18450, 191, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; WebSaver; Maxthon; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022)*/
const Device UA2829270 = {S18451, 191, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; WebSaver; Maxthon; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022)*/
const Device UA2829788 = {S18452, 87, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; Maxthon; Avant Browser)*/
const Device UA2832934 = {S18453, 87, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; Avant Browser; Maxthon)*/
const Device UA2848088 = {S18454, 77, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; Maxthon; SV1)*/
const Device UA2888891 = {S18455, 77, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; SV1; Maxthon)*/
const Device UA2889048 = {S18456, 87, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; Maxthon; Avant Browser)*/
const Device UA2933383 = {S18457, 81, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Maxthon; Maxthon)*/
const Device UA2933523 = {S18458, 87, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Maxthon; Avant Browser)*/
const Device UA2950187 = {S18459, 87, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; Maxthon; Avant Browser)*/
const Device UA2950219 = {S18460, 77, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; SV1; Maxthon)*/
const Device UA2950432 = {S18461, 81, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; Maxthon; Maxthon)*/
const Device UA2950485 = {S18462, 81, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Maxthon; Maxthon)*/
const Device UA2950685 = {S18463, 81, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; Maxthon; Maxthon)*/
const Device UA2979664 = {S18464, 155, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MAXTHON)*/
const Device UA2981084 = {S18465, 87, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Avant Browser; Maxthon)*/
const Device UA2981636 = {S18466, 81, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; Maxthon; Maxthon)*/
const Device UA2985479 = {S18467, 77, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; SV1; Maxthon)*/
const Device UA3069668 = {S18468, 171, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Maxthon; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; Avant Browser)*/
const Device UA3098448 = {S18469, 81, { &P15364,&P5690,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; Maxthon; Maxthon)*/
const Device UA116217 = {S18470, 17, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - W3C_Validator/1.1*/
const Device UA140199 = {S18471, 24, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - W3C_Validator/1.432.2.22*/
const Device UA171632 = {S18472, 17, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - W3C_Validator/1.2*/
const Device UA428867 = {S18473, 17, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - W3C_Validator/1.3*/
const Device UA622974 = {S18474, 100, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - Python-urllib/2.7 AppEngine-Google; (+http://code.google.com/appengine; appid: s~hr-pulsesubscriber)*/
const Device UA756179 = {S18475, 89, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - Python-urllib/2.5 AppEngine-Google; (+http://code.google.com/appengine; appid: ggghelper)*/
const Device UA2428172 = {S18476, 50, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - Readability/e86230 - http://readability.com/about/*/
const Device UA2462203 = {S18477, 50, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - Readability/b27ecf - http://readability.com/about/*/
const Device UA2490556 = {S18478, 50, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - Readability/348c87 - http://readability.com/about/*/
const Device UA2492141 = {S18479, 50, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - Readability/b32b87 - http://readability.com/about/*/
const Device UA2510040 = {S18480, 101, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - Python-urllib/1.17 AppEngine-Google; (+http://code.google.com/appengine; appid: s~hr-pulsesubscriber)*/
const Device UA2537366 = {S18481, 120, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - heimoicom/1.0 (+http://heimoicom.appspot.com/) AppEngine-Google; (+http://code.google.com/appengine; appid: s~heimoisrv)*/
const Device UA2585292 = {S18482, 50, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - W3C_Validator/1.3 http://validator.w3.org/services*/
const Device UA2605189 = {S18483, 50, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - Readability/4824a0 - http://readability.com/about/*/
const Device UA2680549 = {S18484, 83, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 AppEngine-Google; (+http://code.google.com/appengine; appid: s~ziin-me)*/
const Device UA2712992 = {S18485, 50, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - Readability/bdc18f - http://readability.com/about/*/
const Device UA2740612 = {S18486, 50, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - Readability/4aa255 - http://readability.com/about/*/
const Device UA2874394 = {S18487, 50, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - Readability/36a234 - http://readability.com/about/*/
const Device UA2888687 = {S18488, 50, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - Readability/dbf201 - http://readability.com/about/*/
const Device UA2899268 = {S18489, 50, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - Readability/744cab - http://readability.com/about/*/
const Device UA2948415 = {S18490, 50, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - Readability/95e8c2 - http://readability.com/about/*/
const Device UA3020562 = {S18491, 38, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - Tiny Tiny RSS/1.9 (http://tt-rss.org/)*/
const Device UA3025453 = {S18492, 50, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - Readability/38b3ab - http://readability.com/about/*/
const Device UA3034093 = {S18493, 50, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - Readability/b5bf8e - http://readability.com/about/*/
const Device UA3036994 = {S18494, 50, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - Readability/73c8c6 - http://readability.com/about/*/
const Device UA3110826 = {S18495, 39, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - Tiny Tiny RSS/1.10 (http://tt-rss.org/)*/
const Device UA3115586 = {S18496, 50, { &P15364,&P7480,&P17470,&P18092 }};	/*Useragent - Readability/7a48f3 - http://readability.com/about/*/
const Device UA2426056 = {S18497, 140, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_PN071 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2458991 = {S18498, 62, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC PN071 Build/JZO54K)*/
const Device UA2533303 = {S18499, 60, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC One Build/JZO54K)*/
const Device UA2559658 = {S18500, 138, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2600020 = {S18501, 137, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTCONE Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2605190 = {S18502, 149, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; HTC_One/1.28.111.6 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2605853 = {S18503, 149, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC_One/1.28.161.7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659795 = {S18504, 59, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTCONE Build/JZO54K)*/
const Device UA2662110 = {S18505, 62, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC_PN071 Build/JZO54K)*/
const Device UA2662640 = {S18506, 143, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; HTC One 801e Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665857 = {S18507, 65, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC One 801e Build/JZO54K)*/
const Device UA2666179 = {S18508, 138, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666876 = {S18509, 153, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_PN07120/1.26.502.6 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2669209 = {S18510, 154, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_PN07120/1.26.502.10 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2709557 = {S18511, 150, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; HTC_One/1.29.163.11 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2710496 = {S18512, 149, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; HTC_One/1.28.163.3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2711654 = {S18513, 150, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC_One/1.29.161.11 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2712399 = {S18514, 160, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; HTC_PN071-orange-LS/1.28.73.7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719031 = {S18515, 150, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; HTC_One/1.29.111.11 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2751469 = {S18516, 160, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; HTC_PN071-orange-LS/1.28.75.8 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2773434 = {S18517, 154, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_PN07120/1.26.502.12 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2773995 = {S18518, 150, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; HTC_One/1.29.111.16 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2780854 = {S18519, 136, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; HTC One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2796879 = {S18520, 150, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC_One/1.29.161.16 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2877341 = {S18521, 150, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; HTC_One/1.29.161.17 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879633 = {S18522, 161, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; HTC_PN071-orange-LS/1.28.73.10 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879760 = {S18523, 161, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; HTC_PN071-orange-LS/1.28.75.10 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2890640 = {S18524, 153, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_PN07120/1.21.502.1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2901194 = {S18525, 161, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; HTC_PN071-orange-LS/1.28.75.11 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956604 = {S18526, 160, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; HTC_PN071-orange-LS/1.28.69.8 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2961455 = {S18527, 150, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; HTC_One/1.29.163.16 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3001514 = {S18528, 161, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; HTC_PN071-orange-LS/1.28.69.10 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3021352 = {S18529, 154, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_PN07120/1.26.502.15 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3048526 = {S18530, 135, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; HTCONE Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103362 = {S18531, 141, { &P20705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; One Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA53786 = {S18532, 77, { &P15722,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry9300/5.0.0.955 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102*/
const Device UA3062788 = {S18533, 102, { &P15364,&P18110,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3062790 = {S18534, 104, { &P15364,&P18110,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3062797 = {S18535, 111, { &P15364,&P18110,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3093646 = {S18536, 112, { &P15364,&P18110,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36,gzip(gfe)*/
const Device UA3094726 = {S18537, 114, { &P15364,&P18110,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36,gzip(gfe)*/
const Device UA3115325 = {S18538, 120, { &P15364,&P18110,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/29.0.1600.1 Chrome/29.0.1600.1 Safari/537.36*/
const Device UA3127323 = {S18539, 104, { &P15364,&P18110,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3127561 = {S18540, 118, { &P15364,&P18110,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Iron/29.0.1600.1 Chrome/29.0.1600.1 Safari/537.36*/
const Device UA37122 = {S18541, 136, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5313e Safari/7534.48.3*/
const Device UA37142 = {S18542, 136, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5274d Safari/7534.48.3*/
const Device UA37173 = {S18543, 136, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5288d Safari/7534.48.3*/
const Device UA37178 = {S18544, 136, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5302b Safari/7534.48.3*/
const Device UA70719 = {S18545, 134, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3*/
const Device UA91013 = {S18546, 136, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A404 Safari/7534.48.3*/
const Device UA90997 = {S18547, 136, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A402 Safari/7534.48.3*/
const Device UA231283 = {S18548, 136, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5220p Safari/7534.48.3*/
const Device UA355444 = {S18549, 136, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A406 Safari/7534.48.3*/
const Device UA452068 = {S18550, 136, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5248d Safari/7534.48.3*/
const Device UA572872 = {S18551, 134, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A332 Safari/7534.48.3*/
const Device UA622532 = {S18552, 136, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3*/
const Device UA653796 = {S18553, 136, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9Z999 Safari/7534.48.3*/
const Device UA653873 = {S18554, 127, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.46*/
const Device UA662582 = {S18555, 136, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7534.48.3*/
const Device UA725368 = {S18556, 134, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3*/
const Device UA754129 = {S18557, 130, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3*/
const Device UA772298 = {S18558, 134, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9Z999 Safari/7534.48.3*/
const Device UA815095 = {S18559, 136, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA1960125 = {S18560, 126, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari Version/5.1*/
const Device UA1968906 = {S18561, 135, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.32*/
const Device UA2153389 = {S18562, 124, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Safari Version/5.1*/
const Device UA2305618 = {S18563, 136, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3*/
const Device UA2379956 = {S18564, 107, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; iPhone OS 5_0) AppleWebKit/535.7 (KHTML, like Gecko) Version/5.1 Mobile Safari/7354.48*/
const Device UA2421851 = {S18565, 138, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X; en) AppleWebKit/528.18 (KHTML, like Gecko) Version/5.1 Mobile/7A341 Safari/528.16*/
const Device UA2440443 = {S18566, 140, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/by alpden Safari/7534.48.3*/
const Device UA2811125 = {S18567, 136, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B208 Safari/7534.48.3*/
const Device UA2864109 = {S18568, 125, { &P12280,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safar*/
const Device UA2966255 = {S18569, 158, { &P17595,&P17296,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3024560 = {S18570, 119, { &P17595,&P17296,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3024942 = {S18571, 119, { &P17595,&P17296,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3025070 = {S18572, 119, { &P17595,&P17296,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3025543 = {S18573, 119, { &P17595,&P17296,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3025724 = {S18574, 119, { &P17595,&P17296,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3025915 = {S18575, 119, { &P17595,&P17296,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3026186 = {S18576, 119, { &P17595,&P17296,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3026652 = {S18577, 119, { &P17595,&P17296,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3027268 = {S18578, 129, { &P17595,&P17296,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36,gzip(gfe)*/
const Device UA3027403 = {S18579, 119, { &P17595,&P17296,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA101848 = {S18580, 112, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA132847 = {S18581, 118, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA134075 = {S18582, 118, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA153623 = {S18583, 118, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA181409 = {S18584, 118, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA183317 = {S18585, 118, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA184104 = {S18586, 120, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA252700 = {S18587, 124, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; BOLT/2.800) AppleWebKit/534.6 (KHTML, like Gecko) Version/5.0 Safari/534.6.3*/
const Device UA279209 = {S18588, 118, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA306936 = {S18589, 120, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA647453 = {S18590, 118, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA668399 = {S18591, 124, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; BOLT/2.802) AppleWebKit/534.6 (KHTML, like Gecko) Version/5.0 Safari/534.6.3*/
const Device UA693283 = {S18592, 118, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA694206 = {S18593, 112, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/535.3 (KHTML, like Gecko) Version/5.0.1 Safari/535.3*/
const Device UA727619 = {S18594, 107, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA841709 = {S18595, 112, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA1912960 = {S18596, 114, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.12 (KHTML, like Gecko) Version/5.0.1 Safari/534.12*/
const Device UA1941362 = {S18597, 102, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2164705 = {S18598, 105, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.34 (KHTML, like Gecko) Version/5.0 Dooble/1.31 Safari/534.34*/
const Device UA2164994 = {S18599, 106, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2227006 = {S18600, 106, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA2351813 = {S18601, 102, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; en-US) AppleWebKit/535.12 (KHTML, like Gecko) Version/5.0.1 Safari/535.12*/
const Device UA2386757 = {S18602, 99, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2394450 = {S18603, 106, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2404532 = {S18604, 99, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA2422334 = {S18605, 95, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2422329 = {S18606, 99, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2423462 = {S18607, 99, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA2426637 = {S18608, 100, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA2445250 = {S18609, 107, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA3033983 = {S18610, 122, { &P15364,&P5690,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/531.1 (KHTML, like Gecko) FlyFlow/2.3 Version/5.0 Safari/531.1*/
const Device UA605190 = {S18611, 172, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; LG-E400-Orange/V10d Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA631445 = {S18612, 165, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; LG-E400/V10g Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA636044 = {S18613, 160, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; LG-E400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA656664 = {S18614, 172, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; LG-E400-Orange/V10g Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA675144 = {S18615, 165, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; LG-E400/V10e Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA700307 = {S18616, 165, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - LG-E400 Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; E400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA760849 = {S18617, 60, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-E400 Build/GRK39F)*/
const Device UA764830 = {S18618, 161, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; LG-E400f Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA806167 = {S18619, 165, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; LG-E400/V10d Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA893658 = {S18620, 161, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-sa; LG-E400g Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA901535 = {S18621, 166, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; LG-E400f V10b Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA912342 = {S18622, 172, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; LG-E400-Orange/V10k Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA1953613 = {S18623, 165, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; sk-sk; LG-E400/V10c Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA1980118 = {S18624, 165, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; LG-E400/V10h Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA1994668 = {S18625, 165, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; LG-E400/V10k Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA2032444 = {S18626, 61, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-E400f Build/GRK39F)*/
const Device UA2034222 = {S18627, 165, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hr-hr; LG-E400/V10b Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA2041378 = {S18628, 166, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; LG-E400f V10d Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA2163521 = {S18629, 161, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; LG-E400R Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA2190388 = {S18630, 172, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; LG-E400-Orange/V10m Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA2203068 = {S18631, 165, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; LG-E400/V10i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA2229905 = {S18632, 165, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; el-gr; LG-E400/V10f Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA2236058 = {S18633, 165, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; LG-E400/V10m Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA2257070 = {S18634, 165, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; LG-E400/V10n Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA2389484 = {S18635, 165, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; LG-E400/V10j Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA2422211 = {S18636, 168, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; LG-E400b-parrot Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA2445630 = {S18637, 61, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-E400g Build/GRK39F)*/
const Device UA2462003 = {S18638, 165, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; LG-E400/V10p Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA2486892 = {S18639, 172, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; LG-E400-Orange/V10p Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA2694809 = {S18640, 135, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; LG-E400g Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2723287 = {S18641, 161, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; LG-E400b Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA2787081 = {S18642, 172, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; LG-E400-Orange/V10i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA3117829 = {S18643, 157, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en; LG-E400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA3117955 = {S18644, 61, { &P18241,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-E400R Build/GRK39F)*/
const Device UA347781 = {S18645, 98, { &P15364,&P5690,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA351710 = {S18646, 105, { &P15364,&P5690,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA351784 = {S18647, 98, { &P15364,&P5690,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA355648 = {S18648, 116, { &P15364,&P5690,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/535.3 (KHTML, like Gecko) Chrome/16.0.883.0 Safari/535.3*/
const Device UA368194 = {S18649, 135, { &P15364,&P5690,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7,gzip(gfe) (via translate.google.com)*/
const Device UA398266 = {S18650, 105, { &P15364,&P5690,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7*/
const Device UA622888 = {S18651, 93, { &P15364,&P5690,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 Windows NT 5.1 AppleWebKit/535.7 KHTML, like Gecko Chrome/16.0.912.0 Safari/535.7*/
const Device UA690160 = {S18652, 153, { &P15364,&P5690,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/535.3 (KHTML, like Gecko) Chrome/16.0.883.0 Safari/535.3,gzip(gfe) (via translate.google.com)*/
const Device UA755439 = {S18653, 171, { &P15364,&P5690,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.904.0 Safari/535.7 AppEngine-Google; (+http://code.google.com/appengine; appid: s~daisyw110)*/
const Device UA789687 = {S18654, 116, { &P15364,&P5690,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/535.3 (KHTML, like Gecko) Chrome/16.0.883.0 Safari/535.3*/
const Device UA2159030 = {S18655, 122, { &P15364,&P5690,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.910.20417 KChrome/3.2.5.20417  Safari/535.7*/
const Device UA2258980 = {S18656, 108, { &P15364,&P5690,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7,gzip(gfe)*/
const Device UA2413288 = {S18657, 97, { &P15364,&P5690,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.6 (KHTML	 like Gecko) Chrome/16.0.899.0 Safari/535.6*/
const Device UA2543053 = {S18658, 98, { &P15364,&P5690,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0+(Windows+NT+5.1)+AppleWebKit/535.7+(KHTML,+like+Gecko)+Chrome/16.0.912.77+Safari/535.7*/
const Device UA2597901 = {S18659, 114, { &P15364,&P5690,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML like Gecko) Chrome/16.0.912.75 Safari/535.7 CoolNovo/2.0.0.9*/
const Device UA3100681 = {S18660, 136, { &P15364,&P5690,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.904.0 Safari/535.7,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3118120 = {S18661, 126, { &P15364,&P5690,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/535.3 (KHTML, like Gecko) Chrome/16.0.883.0 Safari/535.3,gzip(gfe)*/
const Device UA2359229 = {S18662, 141, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Galaxy Nexus Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2362759 = {S18663, 63, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2; Galaxy Nexus Build/JOP40C)*/
const Device UA2369730 = {S18664, 143, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2386304 = {S18665, 147, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2/4.1.2; en-us; Galaxy Nexus Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2388147 = {S18666, 65, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; Galaxy Nexus Build/JOP40D)*/
const Device UA2388416 = {S18667, 143, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2389111 = {S18668, 141, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Galaxy Nexus Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2406159 = {S18669, 143, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2407202 = {S18670, 134, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Galaxy Nexus Build/JOP40D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2470271 = {S18671, 143, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JOQ10A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2492651 = {S18672, 142, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2505251 = {S18673, 143, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JB_MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2511780 = {S18674, 143, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2512909 = {S18675, 142, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2514558 = {S18676, 64, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; Galaxy Nexus Build/JDQ39)*/
const Device UA2535466 = {S18677, 143, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2563823 = {S18678, 133, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; Galaxy Nexus Build/JDQ39) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2596091 = {S18679, 143, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654855 = {S18680, 141, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-; Galaxy Nexus Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2663007 = {S18681, 140, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-; Galaxy Nexus Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2705768 = {S18682, 143, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2792465 = {S18683, 145, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JB_MR1.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2862021 = {S18684, 142, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JER09) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2966990 = {S18685, 143, { &P16292,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JWR66N) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA18168 = {S18686, 118, { &P15364,&P5690,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.19 (KHTML, like Gecko) Chrome/11.0.661.0 Safari/534.19*/
const Device UA180127 = {S18687, 108, { &P15364,&P5690,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 5.1) AppleWebKit/534.23 (KHTML, like Gecko) Chrome/11.0.686.3 Safari/534.23*/
const Device UA193039 = {S18688, 137, { &P15364,&P5690,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.60 Safari/534.24,gzip(gfe) (via translate.google.com)*/
const Device UA193157 = {S18689, 107, { &P15364,&P5690,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.60 Safari/534.24*/
const Device UA195670 = {S18690, 100, { &P15364,&P5690,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA195977 = {S18691, 100, { &P15364,&P5690,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA518696 = {S18692, 108, { &P15364,&P5690,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 5.2) AppleWebKit/534.23 (KHTML, like Gecko) Chrome/11.0.686.3 Safari/534.23*/
const Device UA560633 = {S18693, 112, { &P15364,&P5690,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko; PTST 2.284) Chrome/11.0.696.68 Safari/534.24*/
const Device UA653223 = {S18694, 118, { &P15364,&P5690,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.17 (KHTML, like Gecko) Chrome/11.0.652.0 Safari/534.17*/
const Device UA1865225 = {S18695, 145, { &P15364,&P5690,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 5.1) AppleWebKit/534.23 (KHTML, like Gecko) Chrome/11.0.686.3 Safari/534.23,gzip(gfe) (via translate.google.com)*/
const Device UA2135187 = {S18696, 139, { &P15364,&P5690,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.60 Safari/534.24,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2258593 = {S18697, 113, { &P15364,&P5690,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.60 Safari/534.24, SurfCop 1.5*/
const Device UA2258627 = {S18698, 110, { &P15364,&P5690,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.14 (Chromium Portable 9.2)*/
const Device UA2335201 = {S18699, 100, { &P15364,&P5690,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML	 like Gecko) Chrome/11.0.696.77 Safari/534.24*/
const Device UA2362364 = {S18700, 110, { &P15364,&P5690,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.60 Safari/534.24,gzip(gfe)*/
const Device UA2371338 = {S18701, 118, { &P15364,&P5690,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; de-DE) AppleWebKit/534.19 (KHTML, like Gecko) Chrome/11.0.661.0 Safari/534.19*/
const Device UA2580394 = {S18702, 99, { &P15364,&P5690,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML like Gecko) Chrome/11.0.696.71 Safari/534.24*/
const Device UA2743910 = {S18703, 112, { &P15364,&P5690,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko; PTST 2.280) Chrome/11.0.696.65 Safari/534.24*/
const Device UA2980382 = {S18704, 117, { &P15364,&P5690,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.17 (KHTML like Gecko) Chrome/11.0.652.0 Safari/534.17*/
const Device UA912384 = {S18705, 101, { &P15364,&P5690,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA912398 = {S18706, 101, { &P15364,&P5690,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA912410 = {S18707, 108, { &P15364,&P5690,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA912664 = {S18708, 138, { &P15364,&P5690,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe) (via translate.google.com)*/
const Device UA913240 = {S18709, 140, { &P15364,&P5690,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA923864 = {S18710, 108, { &P15364,&P5690,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA2235720 = {S18711, 111, { &P15364,&P5690,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe)*/
const Device UA3099264 = {S18712, 100, { &P15364,&P5690,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.11 (KHTML like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA5634 = {S18713, 146, { &P17346,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8G4 Safari/6533.18.5*/
const Device UA5662 = {S18714, 146, { &P17346,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F190 Safari/6533.18.5*/
const Device UA5718 = {S18715, 146, { &P17346,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5*/
const Device UA46556 = {S18716, 146, { &P17346,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8K2 Safari/6533.18.5*/
const Device UA181413 = {S18717, 146, { &P17346,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5*/
const Device UA232715 = {S18718, 146, { &P17346,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA567890 = {S18719, 148, { &P17346,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9Z999 Safari/6533.18.5*/
const Device UA1891347 = {S18720, 148, { &P17346,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA2518024 = {S18721, 146, { &P17346,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA2597356 = {S18722, 143, { &P17346,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 433 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5*/
const Device UA2600104 = {S18723, 143, { &P17346,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 432 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5*/
const Device UA2600525 = {S18724, 144, { &P17346,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 43 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8F190 Safari/6533.18.5*/
const Device UA2600548 = {S18725, 143, { &P17346,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 435 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA2640125 = {S18726, 144, { &P17346,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5*/
const Device UA2658438 = {S18727, 143, { &P17346,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 431 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8G4 Safari/6533.18.5*/
const Device UA2659146 = {S18728, 143, { &P17346,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 434 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8K2 Safari/6533.18.5*/
const Device UA2949900 = {S18729, 148, { &P17346,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_1 like Mac OS X; es-es) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA3070048 = {S18730, 148, { &P17346,&P17090,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA1994645 = {S18731, 248, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA1995075 = {S18732, 231, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA2154993 = {S18733, 72, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Microsoft Office/15.0 (Windows NT 6.1; Microsoft Outlook 15.0.4128; Pro)*/
const Device UA2338911 = {S18734, 210, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA2339317 = {S18735, 260, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4128; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA2356148 = {S18736, 243, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2367297 = {S18737, 248, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2367250 = {S18738, 263, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2377406 = {S18739, 72, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Microsoft Office/15.0 (Windows NT 6.1; Microsoft Outlook 15.0.4420; Pro)*/
const Device UA2382056 = {S18740, 277, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2384858 = {S18741, 292, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2386388 = {S18742, 272, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2428619 = {S18743, 236, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2491521 = {S18744, 233, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2503486 = {S18745, 72, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Microsoft Office/15.0 (Windows NT 6.1; Microsoft Outlook 15.0.4454; Pro)*/
const Device UA2531299 = {S18746, 253, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2533365 = {S18747, 238, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2548084 = {S18748, 214, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA2553711 = {S18749, 251, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2553876 = {S18750, 265, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2557965 = {S18751, 226, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2562041 = {S18752, 268, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MDDRJS; Microsoft Outlook 15.0.4454; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2580018 = {S18753, 72, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Microsoft Office/15.0 (Windows NT 6.1; Microsoft Outlook 15.0.4481; Pro)*/
const Device UA2583616 = {S18754, 226, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2583919 = {S18755, 272, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2583943 = {S18756, 277, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2584022 = {S18757, 243, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2584281 = {S18758, 248, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2589588 = {S18759, 263, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA2596993 = {S18760, 72, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Microsoft Office/15.0 (Windows NT 6.1; Microsoft Outlook 15.0.4433; Pro)*/
const Device UA2605629 = {S18761, 263, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2659166 = {S18762, 241, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4454; ms-office; MSOffice 15)*/
const Device UA2666990 = {S18763, 241, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2667765 = {S18764, 292, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2695098 = {S18765, 280, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2695371 = {S18766, 227, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2695758 = {S18767, 251, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2695980 = {S18768, 282, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2705433 = {S18769, 255, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2705714 = {S18770, 222, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2706959 = {S18771, 253, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2708892 = {S18772, 251, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2712871 = {S18773, 227, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2713336 = {S18774, 280, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4481; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2727829 = {S18775, 222, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2732999 = {S18776, 124, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2739172 = {S18777, 263, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2740008 = {S18778, 260, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2740460 = {S18779, 226, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2741535 = {S18780, 231, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2741612 = {S18781, 248, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2750239 = {S18782, 123, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Microsoft Outlook 15.0.4481; ms-office; MSOffice 15)*/
const Device UA2753193 = {S18783, 292, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2753393 = {S18784, 72, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Microsoft Office/15.0 (Windows NT 6.1; Microsoft Outlook 15.0.4505; Pro)*/
const Device UA2754676 = {S18785, 277, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2769996 = {S18786, 253, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2770379 = {S18787, 255, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2796802 = {S18788, 124, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2797158 = {S18789, 229, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2816292 = {S18790, 277, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4128; Microsoft Outlook 15.0.4128; ms-office; MSOffice 15)*/
const Device UA2842632 = {S18791, 301, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2855643 = {S18792, 231, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4433; ms-office; MSOffice 15)*/
const Device UA2868487 = {S18793, 210, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2879090 = {S18794, 251, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2879195 = {S18795, 270, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2879251 = {S18796, 227, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2937602 = {S18797, 153, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2938122 = {S18798, 282, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2946040 = {S18799, 248, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2947171 = {S18800, 263, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2947520 = {S18801, 72, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Microsoft Office/15.0 (Windows NT 6.1; Microsoft Outlook 15.0.4517; Pro)*/
const Device UA2947525 = {S18802, 226, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2947853 = {S18803, 272, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2948031 = {S18804, 231, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2957394 = {S18805, 152, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2958021 = {S18806, 289, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2966223 = {S18807, 210, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2966398 = {S18808, 241, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2967152 = {S18809, 124, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2967793 = {S18810, 277, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2969736 = {S18811, 292, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA2982882 = {S18812, 256, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2985336 = {S18813, 152, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Microsoft Outlook 15.0.4420; Microsoft Outlook 15.0.4420; ms-office; MSOffice 15)*/
const Device UA2997910 = {S18814, 273, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2998700 = {S18815, 260, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4433; Microsoft Outlook 15.0.4433; ms-office; MSOffice 15)*/
const Device UA3014386 = {S18816, 265, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3014928 = {S18817, 301, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3015042 = {S18818, 253, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3016041 = {S18819, 251, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3017748 = {S18820, 215, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3019868 = {S18821, 153, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3020596 = {S18822, 270, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3057453 = {S18823, 117, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3057483 = {S18824, 282, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3057643 = {S18825, 158, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Win64; x64; Trident/6.0; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3077866 = {S18826, 232, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3078340 = {S18827, 152, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3078420 = {S18828, 129, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Win64; x64; Trident/6.0; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3083964 = {S18829, 231, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA3092082 = {S18830, 306, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3094999 = {S18831, 72, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Microsoft Office/15.0 (Windows NT 6.1; Microsoft Outlook 15.0.4535; Pro)*/
const Device UA3096187 = {S18832, 224, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA3096214 = {S18833, 248, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA3096247 = {S18834, 260, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Microsoft Outlook 15.0.4535; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA3107229 = {S18835, 258, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3107455 = {S18836, 273, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3107955 = {S18837, 146, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3109444 = {S18838, 263, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA3110705 = {S18839, 241, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA3114862 = {S18840, 277, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4535; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA3121257 = {S18841, 258, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; InfoPath.3; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA3125033 = {S18842, 330, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; Microsoft Outlook 15.0.4517; ms-office; MSOffice 15)*/
const Device UA3126685 = {S18843, 124, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA3127542 = {S18844, 292, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; InfoPath.3; Microsoft Outlook 15.0.4535; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA3127706 = {S18845, 265, { &P15364,&P7133,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; Microsoft Outlook 15.0.4535; Microsoft Outlook 15.0.4535; ms-office; MSOffice 15)*/
const Device UA29888 = {S18846, 119, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C5101c*/
const Device UA29982 = {S18847, 117, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C134*/
const Device UA29997 = {S18848, 136, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/X.X (KHTML, like Gecko) Version/X.X Mobile/XXXX Safari/X.X*/
const Device UA30003 = {S18849, 119, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148*/
const Device UA30005 = {S18850, 106, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA30029 = {S18851, 120, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148a*/
const Device UA90033 = {S18852, 104, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2 like Mac OS X; en_us) AppleWebKit/525.18.1 (KHTML, like Gecko)*/
const Device UA90716 = {S18853, 106, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_7 like Mac OS X; en_us) AppleWebKit/525.18.1 (KHTML, like Gecko)*/
const Device UA144293 = {S18854, 104, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA157476 = {S18855, 135, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/533.17.9*/
const Device UA158477 = {S18856, 136, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148a Safari/533.17.9*/
const Device UA170702 = {S18857, 106, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA171157 = {S18858, 119, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E200*/
const Device UA171455 = {S18859, 106, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA171760 = {S18860, 119, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E128*/
const Device UA173187 = {S18861, 135, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E200 Safari/533.17.9*/
const Device UA180446 = {S18862, 119, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_7 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E303*/
const Device UA181455 = {S18863, 106, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_7 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA190871 = {S18864, 106, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA190943 = {S18865, 119, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E401*/
const Device UA197165 = {S18866, 135, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E401 Safari/533.17.9*/
const Device UA211401 = {S18867, 136, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_10 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E600 Safari/533.17.9*/
const Device UA222987 = {S18868, 106, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_9 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA223912 = {S18869, 119, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_9 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E501*/
const Device UA231618 = {S18870, 120, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_10 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E600*/
const Device UA232025 = {S18871, 107, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_10 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA300736 = {S18872, 119, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/9Z999*/
const Device UA303256 = {S18873, 113, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148*/
const Device UA363568 = {S18874, 46, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Miu Ptt 1.5.3 (iPhone; iPhone OS 4.2.1; zh_TW)*/
const Device UA455731 = {S18875, 251, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,3;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.2.6;FBSS/2; FBCR/Verizon;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA503464 = {S18876, 116, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8C148*/
const Device UA605750 = {S18877, 251, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_5 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,3;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.2.5;FBSS/2; FBCR/Verizon;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA624742 = {S18878, 253, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_10 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,3;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.2.10;FBSS/2; FBCR/Verizon;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA625402 = {S18879, 251, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,3;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.2.8;FBSS/2; FBCR/Verizon;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA658340 = {S18880, 248, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone2,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.2.1;FBSS/1; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/1.0]*/
const Device UA687339 = {S18881, 251, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_7 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,3;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.2.7;FBSS/2; FBCR/Verizon;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA699271 = {S18882, 136, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - NaverCafe/1.0 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; ko-kr) AppleWebKit/528.18 (KHTML, like Gecko) myApplicationName*/
const Device UA760530 = {S18883, 119, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (device; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148*/
const Device UA785438 = {S18884, 251, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_9 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,3;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.2.9;FBSS/2; FBCR/Verizon;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA1981459 = {S18885, 147, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 iPad1,1 BingWeb/3.03.1428.X*/
const Device UA1983957 = {S18886, 166, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 4_2 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/8C148 Safari/7534.48 Instapaper/4.0.4 (+http://www.instapaper.com/iphone)*/
const Device UA2207731 = {S18887, 118, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C134b*/
const Device UA2222354 = {S18888, 244, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2 like Mac OS X; nl_NL) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.2;FBSS/2; FBCR/vfnl;FBID/phone;FBLC/nl_NL;FBSF/2.0]*/
const Device UA2418662 = {S18889, 144, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone 4_2_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPhone1,2/4.2.1)*/
const Device UA2462838 = {S18890, 167, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper*/
const Device UA2762044 = {S18891, 142, { &P12280,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone 4_2_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPad1,1/4.2.1)*/
const Device UA636753 = {S18892, 142, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA780313 = {S18893, 66, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9070 Build/GINGERBREAD)*/
const Device UA811083 = {S18894, 161, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-I9070/I9070BULD1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA895172 = {S18895, 143, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; GT-I9070P Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA901725 = {S18896, 162, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070P-ORANGE/I9070PBVLD3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA907008 = {S18897, 161, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-I9070/I9070BULE2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA909254 = {S18898, 160, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070P-TMUK/I9070PBVLD3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1994601 = {S18899, 163, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-I9070P/I9070PBOLE2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025125 = {S18900, 135, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9070 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2029011 = {S18901, 162, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070P-ORANGE/I9070PBVLF1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031393 = {S18902, 161, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; SAMSUNG GT-I9070/I9070BOLE1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032930 = {S18903, 67, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9070P Build/GINGERBREAD)*/
const Device UA2163060 = {S18904, 120, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2185667 = {S18905, 161, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; SAMSUNG GT-I9070/I9070XXLF4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2222204 = {S18906, 140, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2228018 = {S18907, 154, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6_mobile-star; iw-il; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2321933 = {S18908, 120, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; es-es; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2353872 = {S18909, 162, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-I9070P-ORANGE/I9070PBVLG1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2407252 = {S18910, 163, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-I9070P/I9070PBOLK1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2408218 = {S18911, 161, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; SAMSUNG GT-I9070/I9070BOLK1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2413817 = {S18912, 120, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; it-it; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2428228 = {S18913, 161, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-I9070/I9070BULK1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2433273 = {S18914, 161, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; SAMSUNG GT-I9070/I9070XXLK1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2435164 = {S18915, 161, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; SAMSUNG GT-I9070/I9070XXLK2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2480016 = {S18916, 160, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070P-TMUK/I9070PBVLF1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2482952 = {S18917, 162, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-I9070P-ORANGE/I9070PBVLK2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2546384 = {S18918, 120, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; de-de; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2981910 = {S18919, 120, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; de-ch; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA3042917 = {S18920, 120, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; fr-ch; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA3099839 = {S18921, 152, { &P18304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6_mobile-star; iw-; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2387755 = {S18922, 149, { &P12280,&P19826,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.91 Mobile/10B5105c Safari/8536.25*/
const Device UA2408171 = {S18923, 149, { &P12280,&P19826,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.91 Mobile/10B5117b Safari/8536.25*/
const Device UA2435229 = {S18924, 150, { &P12280,&P19826,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B5126b Safari/8536.25*/
const Device UA2488893 = {S18925, 148, { &P12280,&P19826,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B144 Safari/8536.25*/
const Device UA2489112 = {S18926, 148, { &P12280,&P19826,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B141 Safari/8536.25*/
const Device UA2492638 = {S18927, 148, { &P12280,&P19826,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B142 Safari/8536.25*/
const Device UA2492636 = {S18928, 148, { &P12280,&P19826,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B143 Safari/8536.25*/
const Device UA2516523 = {S18929, 150, { &P12280,&P19826,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B145 Safari/8536.25*/
const Device UA2532607 = {S18930, 150, { &P12280,&P19826,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B146 Safari/8536.25*/
const Device UA2585701 = {S18931, 150, { &P12280,&P19826,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B329 Safari/8536.25*/
const Device UA2712031 = {S18932, 150, { &P12280,&P19826,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B350 Safari/8536.25*/
const Device UA802072 = {S18933, 147, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA810716 = {S18934, 139, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-L710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA890774 = {S18935, 140, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E210K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA897563 = {S18936, 140, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I747M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA898912 = {S18937, 140, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E210S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA903210 = {S18938, 140, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E210L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA908297 = {S18939, 61, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SPH-L710 Build/IMM76D)*/
const Device UA910716 = {S18940, 140, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-R530U Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA911768 = {S18941, 69, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D)*/
const Device UA1950991 = {S18942, 147, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I747 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1983686 = {S18943, 62, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-R530U Build/IMM76D)*/
const Device UA1988481 = {S18944, 62, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I747M Build/IMM76D)*/
const Device UA2027832 = {S18945, 62, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHV-E210L Build/IMM76D)*/
const Device UA2030225 = {S18946, 146, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I747M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2033761 = {S18947, 62, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHV-E210S Build/IMM76D)*/
const Device UA2167625 = {S18948, 137, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SPH-L710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2193855 = {S18949, 142, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-L710 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2222767 = {S18950, 138, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SGH-I747M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2223051 = {S18951, 62, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHV-E210K Build/IMM76D)*/
const Device UA2227672 = {S18952, 138, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I747 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2257672 = {S18953, 145, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-L710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2319327 = {S18954, 145, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2339880 = {S18955, 131, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I747M Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2356155 = {S18956, 153, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2364988 = {S18957, 143, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Galaxy S III Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2367460 = {S18958, 140, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-R530C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2457794 = {S18959, 62, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-R530C Build/IMM76D)*/
const Device UA2521098 = {S18960, 131, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SHV-E210L Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2822226 = {S18961, 131, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SHV-E210S Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3066990 = {S18962, 177, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.0.4;dev=d2att;devm=SAMSUNG-SGH-I747;man=samsung;brw=Android;shwd=720x1280;shwa=720x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA3069981 = {S18963, 139, { &P18645,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I747 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA438851 = {S18964, 140, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC One V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA438855 = {S18965, 62, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC One V Build/IML74K)*/
const Device UA662880 = {S18966, 136, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; T320a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA663200 = {S18967, 151, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_V/1.65.161.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA789288 = {S18968, 126, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC One V Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA900425 = {S18969, 144, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC-ADR6290US Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA900459 = {S18970, 143, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; T320a-parrot Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA906972 = {S18971, 151, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_V/2.09.161.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA910003 = {S18972, 150, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; HTC_One_V-orange-LS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA910605 = {S18973, 140, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_T120C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA923888 = {S18974, 140, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_One_V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1958521 = {S18975, 151, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; HTC_One_V/1.61.113.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1961810 = {S18976, 151, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_One_V/1.61.111.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1968002 = {S18977, 151, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC_One_V/2.09.166.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1973101 = {S18978, 131, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC One V Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1992321 = {S18979, 151, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; HTC_One_V/1.61.114.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1995018 = {S18980, 151, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; HTC_One_V/2.09.114.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027536 = {S18981, 151, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; HTC_One_V/2.09.113.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030456 = {S18982, 151, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_One_V/2.09.111.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032999 = {S18983, 151, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC_One_V/1.65.166.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2180548 = {S18984, 151, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; HTC_One_V/2.22.114.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2181920 = {S18985, 151, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC_One_V/2.22.166.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2188972 = {S18986, 151, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_One_V/2.22.111.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2226447 = {S18987, 151, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; HTC_One_V/2.22.113.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2355665 = {S18988, 140, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_One_V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2362585 = {S18989, 151, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_V/2.22.161.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2402424 = {S18990, 151, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_One_V/2.22.161.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2410257 = {S18991, 138, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; HTC_One_V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2546521 = {S18992, 140, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-mx; HTC ONE V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659145 = {S18993, 139, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC One V Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2686912 = {S18994, 138, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; HTC One V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2706436 = {S18995, 140, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_One_V Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2848966 = {S18996, 137, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCOneV Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957390 = {S18997, 151, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; HTC_One_V/2.22.161.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3006053 = {S18998, 151, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_One_V/2.22.111.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3046670 = {S18999, 140, { &P18296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC One V Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA18196 = {S19000, 139, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; LT15i Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18215 = {S19001, 152, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; SonyEricssonLT15iv Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18221 = {S19002, 150, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; sv-se; SonyEricssonLT15i Build/3.0.A.2.52) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18281 = {S19003, 151, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; SonyEricssonLT15i Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18306 = {S19004, 151, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; SonyEricssonLT15i Build/3.0.A.0.326) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18310 = {S19005, 153, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; SonyEricssonLT15i-o Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18319 = {S19006, 151, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; sv-se; SonyEricssonLT15i Build/3.0.A.2.182) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18324 = {S19007, 151, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; sv-se; SonyEricssonLT15i Build/3.0.A.2.179) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18330 = {S19008, 150, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; sv-se; SonyEricssonLT15i Build/3.0.A.2.91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18290 = {S19009, 139, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; LT15i Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24592 = {S19010, 151, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; SonyEricssonLT15i Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32920 = {S19011, 153, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonLT15i Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41648 = {S19012, 141, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; LT15i Build/3.0.1.A.0.150) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41664 = {S19013, 155, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonLT15i-o Build/3.0.1.A.0.150) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41754 = {S19014, 153, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; SonyEricssonLT15i Build/3.0.1.A.0.151) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41759 = {S19015, 141, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; LT15i Build/3.0.1.A.0.151) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47095 = {S19016, 153, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT15i Build/4.0.1.A.0.239) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47105 = {S19017, 139, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; LT15i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47106 = {S19018, 153, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonLT15i Build/4.0.1.A.0.227) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47110 = {S19019, 151, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonLT15i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47144 = {S19020, 153, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT15i Build/4.0.1.A.0.203) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47175 = {S19021, 152, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ie; SonyEricssonLT15iv Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47188 = {S19022, 151, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonLT15i Build/4.0.A.2.377) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47190 = {S19023, 149, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ar-; SonyEricssonLT15i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47148 = {S19024, 153, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT15i Build/4.0.1.A.0.150) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48191 = {S19025, 154, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-za; SonyEricssonLT15 Build/3.0.A.2.181_R2A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49642 = {S19026, 153, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT15i Build/4.0.1.A.0.266) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49633 = {S19027, 153, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT15i Build/4.0.1.A.0.254) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49645 = {S19028, 153, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT15i Build/4.0.1.A.0.277) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49648 = {S19029, 151, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT15i Build/4.0.2.A.0.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57366 = {S19030, 154, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonLT15iv Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57369 = {S19031, 153, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonLT15i Build/3.0.1.A.0.100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57399 = {S19032, 153, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; pl-pl; SonyEricssonLT15i-o Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57410 = {S19033, 141, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; LT15i Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57480 = {S19034, 141, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; LT15i Build/3.0.1.A.0.146) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57498 = {S19035, 149, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; sv-se; SonyEricssonLT15i Build/3.0.J.2.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57500 = {S19036, 151, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonLT15i Build/4.0.A.0.263) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57507 = {S19037, 155, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonLT15i-o Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57512 = {S19038, 154, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; SonyEricssonLT15iv Build/3.0.1.A.0.146) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57535 = {S19039, 153, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; SonyEricssonLT15i Build/3.0.1.A.0.146) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57544 = {S19040, 151, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonLT15i Build/4.0.A.2.100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59575 = {S19041, 151, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; sv-se; SonyEricssonLT15i Build/3.0.A.2.119) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60132 = {S19042, 151, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonLT15i Build/4.0.A.2.357) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60134 = {S19043, 151, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonLT15i Build/4.0.A.2.362) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60161 = {S19044, 151, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonLT15i Build/4.0.A.2.305) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60194 = {S19045, 147, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT15i Build/private) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61197 = {S19046, 154, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; SonyEricssonLT15 Build/3.0.A.2.181_R1A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70095 = {S19047, 153, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT15i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70349 = {S19048, 153, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT15i Build/4.0.1.A.0.217) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70371 = {S19049, 153, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT15i Build/4.0.1.A.0.284) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70715 = {S19050, 152, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; SonyEricssonLT15iv Build/3.0.A.0.326) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70982 = {S19051, 141, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LT15i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71665 = {S19052, 151, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; SonyEricssonLT15i Build/4.0.A.2.257) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74367 = {S19053, 151, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT15i Build/4.0.2.A.0.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74912 = {S19054, 151, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; es-es; SonyEricssonLT15i Build/3.0.A.2.118) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA75483 = {S19055, 152, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonLT15i Build/4.0.2.A.0.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76260 = {S19056, 152, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT15i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76348 = {S19057, 151, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonLT15i Build/4.0.A.2.371) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79386 = {S19058, 153, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ro-ro; SonyEricssonLT15i Build/3.0.1.A.0.150) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90574 = {S19059, 150, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonLT15i Build/4.0.A.0.55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA189799 = {S19060, 63, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.2; LT15i Build/3.0.A.2.181)*/
const Device UA196186 = {S19061, 65, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; LT15i Build/3.0.1.A.0.145)*/
const Device UA252502 = {S19062, 140, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LT15i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368893 = {S19063, 154, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT15i-o Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368927 = {S19064, 153, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT15iv Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369198 = {S19065, 152, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA371303 = {S19066, 154, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; SonyEricssonLT15 Build/3.0.A.2.181_R3B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA457081 = {S19067, 152, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonLT15i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA593783 = {S19068, 128, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT15i Build/4.0.2.A.0.62) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA624805 = {S19069, 153, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; SonyEricssonLT15iv Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA637574 = {S19070, 154, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonLT15i-o Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA658345 = {S19071, 140, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LT15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA659041 = {S19072, 64, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LT15i Build/4.0.2.A.0.62)*/
const Device UA660964 = {S19073, 140, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LT15i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA690937 = {S19074, 140, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; LT15i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA819756 = {S19075, 129, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; LT15i Build/3.0.1.A.0.145) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1911093 = {S19076, 128, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT15i Build/4.0.2.A.0.42) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1970059 = {S19077, 133, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT15i Build/4.0.2.A.0.62) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2021384 = {S19078, 140, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LT15i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022066 = {S19079, 127, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; LT15i Build/4.0.A.2.368) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022813 = {S19080, 128, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT15i Build/4.0.2.A.0.58) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2027210 = {S19081, 133, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT15i Build/4.0.2.A.0.42) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2028837 = {S19082, 150, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-; SonyEricssonLT15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028841 = {S19083, 134, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; LT15i Build/3.0.1.A.0.145) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2032546 = {S19084, 141, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; nl-nl; LT15i Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034494 = {S19085, 139, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Xperia Arc Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2643892 = {S19086, 152, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ko-kr; SonyEricssonLT15i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2811852 = {S19087, 144, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Sony Xperia Arc Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2979075 = {S19088, 152, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; SonyEricssonLT15i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2988778 = {S19089, 153, { &P14993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonLT15iv Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA91033 = {S19090, 99, { &P15364,&P5690,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA252368 = {S19091, 136, { &P15364,&P5690,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.106 Safari/535.2,gzip(gfe) (via translate.google.com)*/
const Device UA255968 = {S19092, 138, { &P15364,&P5690,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA587062 = {S19093, 61, { &P15364,&P5690,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) Chrome/15.0.874.121 Safari/535.2*/
const Device UA626319 = {S19094, 99, { &P15364,&P5690,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA629036 = {S19095, 106, { &P15364,&P5690,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA1925220 = {S19096, 118, { &P15364,&P5690,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA2148216 = {S19097, 106, { &P15364,&P5690,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA2185632 = {S19098, 174, { &P15364,&P5690,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2 AppEngine-Google; (+http://code.google.com/appengine; appid: s~qiquwanlu1)*/
const Device UA2355358 = {S19099, 109, { &P15364,&P5690,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2,gzip(gfe)*/
const Device UA2807633 = {S19100, 96, { &P15364,&P5690,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/150.1.2.3 Safari/535.1*/
const Device UA2950197 = {S19101, 96, { &P15364,&P5690,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.2 (KHTML like Gecko) Chrome/15.0.872.0 Safari/535.2*/
const Device UA180105 = {S19102, 120, { &P15364,&P5690,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA180276 = {S19103, 120, { &P15364,&P5690,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA311997 = {S19104, 157, { &P15364,&P5690,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16,gzip(gfe) (via translate.google.com)*/
const Device UA623246 = {S19105, 110, { &P15364,&P5690,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/10.0 Safari/534.7*/
const Device UA2280597 = {S19106, 159, { &P15364,&P5690,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2385201 = {S19107, 130, { &P15364,&P5690,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16,gzip(gfe)*/
const Device UA2673357 = {S19108, 119, { &P15364,&P5690,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA2675008 = {S19109, 101, { &P15364,&P5690,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.64 Safari/537.31*/
const Device UA2675942 = {S19110, 120, { &P15364,&P5690,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; haw-US) AppleWebKit/534.16 (KHTML like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA2708347 = {S19111, 119, { &P15364,&P5690,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-CA) AppleWebKit/534.16 (KHTML like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA2709330 = {S19112, 119, { &P15364,&P5690,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; es-US) AppleWebKit/534.16 (KHTML like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA2710979 = {S19113, 119, { &P15364,&P5690,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; fr-CA) AppleWebKit/534.16 (KHTML like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA2716830 = {S19114, 101, { &P15364,&P5690,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.64 Safari/537.31*/
const Device UA2735056 = {S19115, 119, { &P15364,&P5690,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; oc-FR) AppleWebKit/534.16 (KHTML like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA2768027 = {S19116, 119, { &P15364,&P5690,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/534.16 (KHTML like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA2840680 = {S19117, 146, { &P15364,&P5690,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.133 Safari/534.16 (via babelfish.yahoo.com)*/
const Device UA2538246 = {S19118, 104, { &P15364,&P18110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22*/
const Device UA2567793 = {S19119, 103, { &P15364,&P18110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2584662 = {S19120, 112, { &P15364,&P18110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2597995 = {S19121, 186, { &P15364,&P18110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.45 Safari/537.22 AppEngine-Google; (+http://code.google.com/appengine; appid: s~google.com:pssics)*/
const Device UA2674500 = {S19122, 109, { &P15364,&P18110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/25.0.1410.63 Safari/537.31*/
const Device UA2674533 = {S19123, 106, { &P15364,&P18110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/25.0.1410.63 Safari/537.31*/
const Device UA2674615 = {S19124, 106, { &P15364,&P18110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/25.0.1410.63 Safari/537.31*/
const Device UA2674645 = {S19125, 106, { &P15364,&P18110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/25.0.1410.63 Safari/537.31*/
const Device UA2675000 = {S19126, 109, { &P15364,&P18110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/25.0.1410.63 Safari/537.31*/
const Device UA2707495 = {S19127, 109, { &P15364,&P18110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/25.0.1410.63 Safari/537.31*/
const Device UA2714965 = {S19128, 95, { &P15364,&P18110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; en-US) AppleWebKit/534.6 (KHTML, like Gecko) Chrome/25.0.500.0 Safari/534.6*/
const Device UA2716828 = {S19129, 106, { &P15364,&P18110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/25.0.1410.63 Safari/537.31*/
const Device UA2750747 = {S19130, 162, { &P15364,&P18110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.32 (KHTML, like Gecko) Chromium/25.0.1349.2 Chrome/25.0.1349.2 Safari/537.32 Debian/7.0 (3.7.91-1) Epiphany/3.7.91*/
const Device UA2766154 = {S19131, 115, { &P15364,&P18110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22,gzip(gfe)*/
const Device UA2792593 = {S19132, 112, { &P15364,&P18110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22,gzip(gfe)*/
const Device UA2958243 = {S19133, 165, { &P15364,&P18110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.32 (KHTML, like Gecko) Chromium/25.0.1349.2 Chrome/25.0.1349.2 Safari/537.32 Debian/unstable (3.8.2-1) Epiphany/3.8.2*/
const Device UA2999634 = {S19134, 136, { &P15364,&P18110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Jolicloud Linux i686) AppleWebKit/537.22 (KHTML, like Gecko) Chromium/25.0.1364.160 Chrome/25.0.1364.160 Safari/537.22*/
const Device UA3049422 = {S19135, 162, { &P15364,&P18110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.32 (KHTML, like Gecko) Chromium/25.0.1349.2 Chrome/25.0.1349.2 Safari/537.32 Debian/7.0 (3.4.2-2.1) Epiphany/3.4.2*/
const Device UA3088579 = {S19136, 138, { &P15364,&P18110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Jolicloud Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Chromium/25.0.1364.160 Chrome/25.0.1364.160 Safari/537.22*/
const Device UA3108925 = {S19137, 165, { &P15364,&P18110,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.32 (KHTML, like Gecko) Chromium/25.0.1349.2 Chrome/25.0.1349.2 Safari/537.32 Debian/unstable (3.8.2-3) Epiphany/3.8.2*/
const Device UA53601 = {S19138, 46, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.1; Windows XP)*/
const Device UA92819 = {S19139, 42, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/3.0 (x86 [en] Windows NT 5.1; Sun)*/
const Device UA99983 = {S19140, 50, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.5; Windows NT 5.1)*/
const Device UA110283 = {S19141, 40, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1)*/
const Device UA118643 = {S19142, 48, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Windows XP 5.1; MSIE 6)*/
const Device UA122481 = {S19143, 63, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - FeedDemon/3.1 (http://www.feeddemon.com/; Microsoft Windows XP)*/
const Device UA124653 = {S19144, 60, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.14)*/
const Device UA127479 = {S19145, 74, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0+(compatible;+MSIE+6.0;+Windows+NT+5.1;+SV1;+.NET+CLR+1.1.4322)*/
const Device UA130255 = {S19146, 60, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla 5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.0.11)*/
const Device UA131033 = {S19147, 59, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN; rv:1.9.1.2)*/
const Device UA140010 = {S19148, 63, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - FeedDemon/2.7 (http://www.newsgator.com/; Microsoft Windows XP)*/
const Device UA142069 = {S19149, 40, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Internet Explorer 6 (MSIE 6; Windows XP)*/
const Device UA154340 = {S19150, 203, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; ArgClrInt/1.0; Windows NT 5.1; Trident/4.0; GTB6.4; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA160645 = {S19151, 47, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US)*/
const Device UA162461 = {S19152, 63, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - FeedDemon/4.0 (http://www.feeddemon.com/; Microsoft Windows XP)*/
const Device UA163670 = {S19153, 84, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.17pre) Gecko K-Meleon/1.6.0*/
const Device UA173848 = {S19154, 42, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows XP 5.1) Java/1.6.0_24*/
const Device UA195402 = {S19155, 63, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-gb; SiteCon/8.8.14)*/
const Device UA210384 = {S19156, 60, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows; U; Windows NT 5.1; zh-TW; rv:1.9.0.11)*/
const Device UA261763 = {S19157, 89, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0+(compatible;+MSIE+7.0;+Windows+NT+5.2;+.NET+CLR+1.1.4322;+.NET+CLR+2.0.50727)*/
const Device UA265384 = {S19158, 26, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Sunrise/0.42g (Windows XP)*/
const Device UA382343 = {S19159, 63, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - FeedDemon/3.0 (http://www.feeddemon.com/; Microsoft Windows XP)*/
const Device UA417799 = {S19160, 51, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MyApp 1.0; Windows NT 5.1)*/
const Device UA473746 = {S19161, 84, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533+ (KHTML, like Gecko)*/
const Device UA563926 = {S19162, 28, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1)*/
const Device UA687485 = {S19163, 69, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.57.2 (KHTML, like Gecko)*/
const Device UA892470 = {S19164, 63, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - FeedDemon/4.1 (http://www.feeddemon.com/; Microsoft Windows XP)*/
const Device UA895729 = {S19165, 83, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; AppleWebKit/535.19 (KHTML, like Gecko; Windows NT 5.1)*/
const Device UA1898404 = {S19166, 79, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Windows; U; Windows NT 5.1; en-US; rv:1.8.1.2) Gecko/X*/
const Device UA1989759 = {S19167, 52, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - IBM-LC-Base-Notes_8.5.3.X_0921-Windows XP Java/1.6.0*/
const Device UA2263423 = {S19168, 53, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; en-US; rv:1.7.6) Gecko/X*/
const Device UA2355171 = {S19169, 42, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows XP 5.1) Java/1.7.0_09*/
const Device UA2368740 = {S19170, 60, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows; U; Windows NT 5.1; en-GB; rv:1.9.2.20)*/
const Device UA2583012 = {S19171, 64, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; EN; rv:1.9.0.6) Gecko/2*/
const Device UA2682180 = {S19172, 60, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.13)*/
const Device UA2883127 = {S19173, 63, { &P15364,&P5690,&P17470,&P18092 }};	/*Useragent - FeedDemon/4.5 (http://www.feeddemon.com/; Microsoft Windows XP)*/
const Device UA2357689 = {S19174, 102, { &P15364,&P18110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2360473 = {S19175, 142, { &P15364,&P18110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Jolicloud Linux i686) AppleWebKit/537.6 (KHTML, like Gecko) Joli OS/1.2 Chromium/23.0.1240.0 Chrome/23.0.1240.0 Safari/537.6*/
const Device UA2408297 = {S19176, 104, { &P15364,&P18110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2433229 = {S19177, 111, { &P15364,&P18110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2458711 = {S19178, 114, { &P15364,&P18110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe)*/
const Device UA2459143 = {S19179, 112, { &P15364,&P18110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe)*/
const Device UA2674653 = {S19180, 109, { &P15364,&P18110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/23.0.1410.63 Safari/537.31*/
const Device UA2674705 = {S19181, 106, { &P15364,&P18110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/23.0.1410.63 Safari/537.31*/
const Device UA2674822 = {S19182, 109, { &P15364,&P18110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/23.0.1410.63 Safari/537.31*/
const Device UA2675199 = {S19183, 106, { &P15364,&P18110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/23.0.1410.63 Safari/537.31*/
const Device UA2675712 = {S19184, 106, { &P15364,&P18110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/23.0.1410.63 Safari/537.31*/
const Device UA2707506 = {S19185, 106, { &P15364,&P18110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/23.0.1410.63 Safari/537.31*/
const Device UA2707710 = {S19186, 109, { &P15364,&P18110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/23.0.1410.63 Safari/537.31*/
const Device UA2844939 = {S19187, 141, { &P15364,&P18110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2952272 = {S19188, 159, { &P15364,&P18110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/23.0.1271.95 Chrome/23.0.1271.95 Safari/537.6+ SUSE/12.3 (3.6.1) Epiphany/3.6.1*/
const Device UA3009899 = {S19189, 141, { &P15364,&P18110,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/23.0.1271.95 Chrome/23.0.1271.95 Safari/537.6+ SugarLabs/0.98*/
const Device UA227236 = {S19190, 93, { &P15364,&P17285,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50*/
const Device UA228047 = {S19191, 100, { &P15364,&P17285,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50*/
const Device UA250884 = {S19192, 101, { &P15364,&P17285,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22*/
const Device UA250914 = {S19193, 108, { &P15364,&P17285,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22*/
const Device UA346839 = {S19194, 106, { &P15364,&P17285,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7*/
const Device UA428868 = {S19195, 101, { &P15364,&P17285,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16*/
const Device UA428875 = {S19196, 108, { &P15364,&P17285,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16*/
const Device UA440731 = {S19197, 106, { &P15364,&P17285,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3*/
const Device UA440766 = {S19198, 99, { &P15364,&P17285,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3*/
const Device UA615814 = {S19199, 99, { &P15364,&P17285,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2*/
const Device UA615816 = {S19200, 106, { &P15364,&P17285,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2*/
const Device UA622552 = {S19201, 99, { &P15364,&P17285,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7*/
const Device UA2586539 = {S19202, 100, { &P15364,&P17285,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.3 Safari/534.53.10*/
const Device UA2586544 = {S19203, 107, { &P15364,&P17285,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.3 Safari/534.53.10*/
const Device UA458623 = {S19204, 139, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA458627 = {S19205, 132, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA899198 = {S19206, 152, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; SAMSUNG GT-P3100/P3100BUALD5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA899632 = {S19207, 132, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1959477 = {S19208, 133, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; GT-P3100B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1964502 = {S19209, 61, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P3100 Build/IMM76D)*/
const Device UA1965389 = {S19210, 61, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-P3100 Build/IML74K)*/
const Device UA2195658 = {S19211, 130, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2198946 = {S19212, 152, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; SAMSUNG GT-P3100/P3100BUBLH3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2230926 = {S19213, 130, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2356901 = {S19214, 133, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; GT-P3100B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2413456 = {S19215, 130, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2460652 = {S19216, 151, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P3100-ORANGE/P3100BVBLH1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2507200 = {S19217, 132, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2546898 = {S19218, 130, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3100 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2608286 = {S19219, 62, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-P3100B Build/IML74K)*/
const Device UA2608972 = {S19220, 134, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P3100 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2609883 = {S19221, 125, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3100 Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2610660 = {S19222, 92, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.youtube/3.5.5(Linux; U; Android 4.0.3; en_US; GT-P3100 Build/IML74K) gzip*/
const Device UA2611053 = {S19223, 152, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3100 Build/IML74K; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2611440 = {S19224, 125, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2611661 = {S19225, 152, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; hr-hr; SAMSUNG GT-P3100/P3100BUALD1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2612182 = {S19226, 139, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2614222 = {S19227, 145, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.7 Mobile Safari/534.30*/
const Device UA2614314 = {S19228, 130, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3100 Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2614667 = {S19229, 79, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Vyrso/2.1.2 (Linux; U; Android 4.0.4; en-PH; GT-P3100 Build/IMM76D.P3100XXBLE3)*/
const Device UA2614906 = {S19230, 93, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.youtube/4.0.23(Linux; U; Android 4.0.4; en_GB; GT-P3100 Build/IMM76D) gzip*/
const Device UA2615190 = {S19231, 152, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build/IMM76D; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2615279 = {S19232, 162, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; GT-P3100 Build/IMM76D) An-ph; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615328 = {S19233, 145, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA2615894 = {S19234, 218, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.2.1-65 (Android 4.0.4; samsung GT-P3100 Build/IMM76D.P3100DDBLF3)*/
const Device UA2616014 = {S19235, 130, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2616202 = {S19236, 79, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.0.4; en-GB; GT-P3100 Build/IMM76D.P3100DDBLF3)*/
const Device UA2616217 = {S19237, 69, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - BBC iPlayer (AndroidApp; 1.3; 13014) GT-P3100 (Android 4.0.3, SDK 15)*/
const Device UA2616294 = {S19238, 131, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/XXLQ1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616705 = {S19239, 79, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Vyrso/2.1.2 (Linux; U; Android 4.0.4; en-GB; GT-P3100 Build/IMM76D.P3100XWBLF1)*/
const Device UA2617236 = {S19240, 79, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.4; pt-BR; GT-P3100 Build/IMM76D.P3100UBBLJ1)*/
const Device UA2617397 = {S19241, 233, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 VoiceGO!(cyb42f2f9766a5b045ced0d29bX-0;Android,4.0.4,GT-P3100;3.6.0;1024x552,1.0;X;VoiceGo-TC;zh-TW)*/
const Device UA2617458 = {S19242, 134, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3100 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2617490 = {S19243, 151, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P3100-ORANGE/P3100BVBLF6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617845 = {S19244, 155, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-P3100/P3100BUBLF1.DM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617883 = {S19245, 155, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-P3100/P3100BUBLH3.DM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617929 = {S19246, 103, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 4.0.4; SAMSUNG GT-P3100 Build/ICE_CREAM_SANDWICH_MR1);*/
const Device UA2618075 = {S19247, 79, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.3; en-US; GT-P3100 Build/IML74K.P3100UBALD3)*/
const Device UA2618320 = {S19248, 143, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2618449 = {S19249, 62, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P3100B Build/IMM76D)*/
const Device UA2618463 = {S19250, 79, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.4; en-GB; GT-P3100 Build/IMM76D.P3100DDBLH3)*/
const Device UA2618637 = {S19251, 79, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.4; en-PH; GT-P3100 Build/IMM76D.P3100XXBLH2)*/
const Device UA2618803 = {S19252, 79, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.3; en-US; GT-P3100 Build/IML74K.P3100XXALD4)*/
const Device UA2619295 = {S19253, 103, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 4.0.3; SAMSUNG GT-P3100 Build/ICE_CREAM_SANDWICH_MR1);*/
const Device UA2619447 = {S19254, 62, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.3; 4) GT-P3100 (Android 4.0.3, SDK 15)*/
const Device UA2619513 = {S19255, 69, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Showtimezz@MY/1.4.8 (Android 4.0.4; samsung-GT-P3100%404.0.4 Build/15*/
const Device UA2619725 = {S19256, 45, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Scanner Radio/3.8.5 (Android 4.0.4, GT-P3100)*/
const Device UA2620237 = {S19257, 79, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.3; en-GB; GT-P3100 Build/IML74K.P3100XWALD2)*/
const Device UA2620335 = {S19258, 80, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Biblia/2.2.7 (Linux; U; Android 4.0.4; es-ES; GT-P3100 Build/IMM76D.P3100XWBLF3)*/
const Device UA2620637 = {S19259, 279, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ms-my; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/1.7.1;FBPN/com.facebook.katana;FBDV/GT-P3100;FBSV/4.0.3;FBDM/{density=1.0,width=600,height=976};FBCR/MY MAXIS;FBLC/ms_MY;FB_FW/1;]*/
const Device UA2620810 = {S19260, 81, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - MediaPlayer/2002/4.0.12 (Android 4.0.4; espressorf; GT-P3100; samsung; ; ; en_GB)*/
const Device UA2621067 = {S19261, 131, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100B Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2621211 = {S19262, 135, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3100B Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2621514 = {S19263, 280, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/1.6.4;FBPN/com.facebook.katana;FBDV/GT-P3100;FBSV/4.0.4;FBDM/{density=1.0,width=1024,height=552};FBCR/MY MAXIS;FBLC/en_GB;FB_FW/1;]*/
const Device UA2621620 = {S19264, 110, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - AndroidNative/2.0.30 (Linux; U; Android 4.0.4; en-us; SAMSUNG GT-P3100; Build/IMM76D) tablet; 1024x552; FilmOn*/
const Device UA2621651 = {S19265, 81, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - MediaPlayer/2004/4.0.12 (Android 4.0.4; espressorf; GT-P3100; samsung; ; ; en_US)*/
const Device UA2621969 = {S19266, 155, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sr-rs; SAMSUNG GT-P3100/P3100BUBLF2.DM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622096 = {S19267, 162, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2622339 = {S19268, 162, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; GT-P3100 Buioid 4.0.4; en-ph; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622464 = {S19269, 50, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - BEETmobile Hotspot/1.0.2 (Android 4.0.4, GT-P3100)*/
const Device UA2622595 = {S19270, 175, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG GT-P3100/Sungsonic??? RC-01 Built By Adi??? Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622671 = {S19271, 137, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2622910 = {S19272, 81, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.0.4; zh-HK; GT-P3100 Build/IMM76D.P3100ZSBLH4)*/
const Device UA2623245 = {S19273, 80, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - VKAndroidApp/2.7.6-64 (Android 4.0.4; SDK 15; armeabi-v7a; samsung GT-P3100; ru)*/
const Device UA2623553 = {S19274, 265, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2623579 = {S19275, 151, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P3100-ORANGE/P3100BVBLF4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2624072 = {S19276, 171, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG GT-P3100/Sungsonic" RC-01 Built By Adi" Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2624202 = {S19277, 82, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Biblia/2.2.8.1 (Linux; U; Android 4.0.3; es-US; GT-P3100 Build/IML74K.P3100UBALD4)*/
const Device UA2624645 = {S19278, 171, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG GT-P3100/SungsonicT RC-01 Built By AdiT Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2624704 = {S19279, 81, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.0.3; zh-HK; GT-P3100 Build/IML74K.P3100ZSALD6)*/
const Device UA2625126 = {S19280, 102, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3100 Build/IMM76D) Appleko) Version/4.0 Safari/534.30*/
const Device UA2625242 = {S19281, 80, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.10 (Linux; U; Android 4.0.4; en-GB; GT-P3100 Build/IMM76D.P3100XWBLJ1)*/
const Device UA2626324 = {S19282, 62, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.3; 4) GT-P3100 (Android 4.0.4, SDK 15)*/
const Device UA2626790 = {S19283, 130, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Groupon/2.2.2561 (Android 4.0.4; Samsung Espressorf / Samsung GT-P3100; )[preload=false;locale=it_IT;clientidbase=android-samsung]*/
const Device UA2627065 = {S19284, 153, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SAMSUNG GT-P3100/P3100BUALD5T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2627579 = {S19285, 137, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; GT-P3100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2627820 = {S19286, 102, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KSafari/534.30*/
const Device UA2628147 = {S19287, 137, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2628481 = {S19288, 131, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-; GT-P3100B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2629085 = {S19289, 162, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; GT-P3100 Build/ 4.0.4; en-ph; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2629245 = {S19290, 170, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; GT-P3100 Build/IML74K   http://stubes.tk) AppleWebKit/534.30 (KHTML, like Gecko   http://stubes.tk) Version/4.0 Safari/534.30*/
const Device UA2630265 = {S19291, 132, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2630351 = {S19292, 139, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2631150 = {S19293, 131, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-P3100B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2632367 = {S19294, 150, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; SAMSUNG GT-P3100/P3100BUALD5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2633943 = {S19295, 151, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; in-id; GT-P3100 Build/IML74K) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.1 Version/4.0 Mobile Safari/530.17*/
const Device UA2634310 = {S19296, 59, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - nApps (Android 4.0.4; GT-P3100; androidapp.lineplay; 1.2.1)*/
const Device UA2634967 = {S19297, 157, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3100 Build/Smartfren.andromaxtab7.0.v.37.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2635059 = {S19298, 152, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SAMSUNG GT-P3100/P3100BUBLF1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2635158 = {S19299, 137, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; se-se; GT-P3100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2635632 = {S19300, 149, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P3100 Build/IMM76D) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.2 Version/4.0 Mobile Safari/530.17*/
const Device UA2636319 = {S19301, 183, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P3100 Build/IMM76D) [FBAN/Orca-Android;FBAV/2.3.2-release;FBLC/ms_MY;FBBV/X;FBCR/MY_MAXIS;FBMF/samsung;FBBD/samsung;FBDV/GT-P3100;FBSV/4.0.4]*/
const Device UA3020373 = {S19302, 151, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P3100-ORANGE/P3100BVBLL4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3056920 = {S19303, 151, { &P15919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; in-id; GT-P3100 Build/IMM76D) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.1 Version/4.0 Mobile Safari/530.17*/
const Device UA2203729 = {S19304, 117, { &P17595,&P17297,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203733 = {S19305, 117, { &P17595,&P17297,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203782 = {S19306, 117, { &P17595,&P17297,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203784 = {S19307, 117, { &P17595,&P17297,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203861 = {S19308, 117, { &P17595,&P17297,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2204087 = {S19309, 117, { &P17595,&P17297,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2234465 = {S19310, 127, { &P17595,&P17297,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)*/
const Device UA2238409 = {S19311, 154, { &P17595,&P17297,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via translate.google.com)*/
const Device UA2238520 = {S19312, 154, { &P17595,&P17297,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via translate.google.com)*/
const Device UA2258348 = {S19313, 127, { &P17595,&P17297,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)*/
const Device UA2258376 = {S19314, 156, { &P17595,&P17297,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2354653 = {S19315, 156, { &P17595,&P17297,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2436909 = {S19316, 193, { &P17595,&P17297,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 AppEngine-Google; (+http://code.google.com/appengine; appid: s~inqarticles)*/
const Device UA2536309 = {S19317, 188, { &P17595,&P17297,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 AppEngine-Google; (+http://code.google.com/appengine; appid: s~inqimg)*/
const Device UA2567857 = {S19318, 188, { &P17595,&P17297,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 AppEngine-Google; (+http://code.google.com/appengine; appid: s~inqimx)*/
const Device UA2597513 = {S19319, 114, { &P17595,&P17297,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1070) AppleWebKit/537.4 (KHTML like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2749240 = {S19320, 148, { &P17595,&P17297,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.510 YaBrowser/1.5.1105.510 (gtsr) Safari/537.4*/
const Device UA2857825 = {S19321, 148, { &P17595,&P17297,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.510 YaBrowser/1.5.1105.510 (fnrb) Safari/537.4*/
const Device UA3046142 = {S19322, 148, { &P17595,&P17297,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.519 YaBrowser/1.5.1105.519 (bskt) Safari/537.4*/
const Device UA2203724 = {S19323, 99, { &P15364,&P17285,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203731 = {S19324, 106, { &P15364,&P17285,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2204096 = {S19325, 136, { &P15364,&P17285,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via translate.google.com)*/
const Device UA2231147 = {S19326, 143, { &P15364,&P17285,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via translate.google.com)*/
const Device UA2233981 = {S19327, 116, { &P15364,&P17285,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)*/
const Device UA2234791 = {S19328, 109, { &P15364,&P17285,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)*/
const Device UA2238591 = {S19329, 138, { &P15364,&P17285,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2457137 = {S19330, 98, { &P15364,&P17285,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2535454 = {S19331, 130, { &P15364,&P17285,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (fnrb) Safari/537.4*/
const Device UA2535456 = {S19332, 130, { &P15364,&P17285,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (gtsr) Safari/537.4*/
const Device UA2583062 = {S19333, 130, { &P15364,&P17285,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.520 YaBrowser/1.5.1105.520 (bskt) Safari/537.4*/
const Device UA2752488 = {S19334, 100, { &P15364,&P17285,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.1.9999.433 Safari/537.4*/
const Device UA2854556 = {S19335, 137, { &P15364,&P17285,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (gtsr) Safari/537.4*/
const Device UA625199 = {S19336, 131, { &P15250,&P7106,&P15829,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9360; en-GB) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.336 Mobile Safari/534.11+*/
const Device UA2402710 = {S19337, 77, { &P15250,&P7106,&P15829,&P18092 }};	/*Useragent - BlackBerry9360/7.1.0.694 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102*/
const Device UA910390 = {S19338, 125, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/21.0.1180.15) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.15 Safari/537.1*/
const Device UA913163 = {S19339, 125, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/21.0.1180.41) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.41 Safari/537.1*/
const Device UA924811 = {S19340, 125, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/21.0.1180.49) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.49 Safari/537.1*/
const Device UA925309 = {S19341, 125, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/21.0.1180.60) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.60 Safari/537.1*/
const Device UA1996987 = {S19342, 125, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/21.0.1180.57) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.57 Safari/537.1*/
const Device UA1997134 = {S19343, 132, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/21.0.1180.60) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.60 Safari/537.1*/
const Device UA1997290 = {S19344, 180, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/21.0.1180.60; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA2001065 = {S19345, 125, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/21.0.1180.83) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1*/
const Device UA2001277 = {S19346, 199, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/21.0.1180.83; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2003026 = {S19347, 125, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/21.0.1180.75) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1*/
const Device UA2005478 = {S19348, 125, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/21.0.1180.79) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.79 Safari/537.1*/
const Device UA2005610 = {S19349, 125, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/21.0.1180.75) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1*/
const Device UA2007819 = {S19350, 212, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/21.0.1180.83; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2010166 = {S19351, 125, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/21.0.1180.77) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.77 Safari/537.1*/
const Device UA2010185 = {S19352, 125, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/21.0.1180.79) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.79 Safari/537.1*/
const Device UA2010996 = {S19353, 125, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/21.0.1180.83) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1*/
const Device UA2016031 = {S19354, 125, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/21.0.1180.89) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2041065 = {S19355, 192, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/21.0.1180.89; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA2041080 = {S19356, 76, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/21.0.1180.89)*/
const Device UA2041324 = {S19357, 125, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/21.0.1180.89) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2041746 = {S19358, 140, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/21.0.1180.89; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)*/
const Device UA2041989 = {S19359, 226, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/21.0.1180.89; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA2194155 = {S19360, 132, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/21.0.1180.89) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2343127 = {S19361, 198, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; chromeframe/21.0.1180.89; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E)*/
const Device UA2344387 = {S19362, 219, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; chromeframe/21.0.1180.89; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2367759 = {S19363, 185, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; MS-RTC LM 8; InfoPath.1; chromeframe/21.0.1180.83)*/
const Device UA2380600 = {S19364, 204, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.2; chromeframe/21.0.1180.89)*/
const Device UA2384518 = {S19365, 132, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/21.0.1180.79) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.79 Safari/537.1*/
const Device UA2416770 = {S19366, 196, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/21.0.1180.89; InfoPath.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA2591335 = {S19367, 238, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/21.0.1180.89; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Tablet PC 1.7; .NET CLR 2.0.50727; InfoPath.1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA2824109 = {S19368, 182, { &P15364,&P5690,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/21.0.1180.89; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7)*/
const Device UA913451 = {S19369, 73, { &P15364,&P7133,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:17.0) Gecko/17.0 Firefox/17.0*/
const Device UA913457 = {S19370, 68, { &P15364,&P7133,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:17.0) Gecko/17.0 Firefox/17.0*/
const Device UA2390796 = {S19371, 121, { &P15364,&P7133,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2) Gecko/X MRA 5.6 (build 03278) Firefox/17.0 (.NET CLR 3.5.30729)*/
const Device UA2467029 = {S19372, 82, { &P15364,&P7133,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.1; Win64; x64; rv:17.0) Gecko/17.0 Firefox/17.0*/
const Device UA92871 = {S19373, 34, { &P15364,&P5690,&P4811,&P18092 }};	/*Useragent - Opera/9.00 (Windows NT 5.1; U; en)*/
const Device UA93298 = {S19374, 64, { &P15364,&P5690,&P4811,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 9.0*/
const Device UA93315 = {S19375, 33, { &P15364,&P5690,&P4811,&P18092 }};	/*Useragent - Opera/9.0 (Windows NT 5.1; U; en)*/
const Device UA101675 = {S19376, 34, { &P15364,&P5690,&P4811,&P18092 }};	/*Useragent - Opera/9.01 (Windows NT 5.1; U; en)*/
const Device UA125223 = {S19377, 34, { &P15364,&P5690,&P4811,&P18092 }};	/*Useragent - Opera/9.02 (Windows NT 5.1; U; en)*/
const Device UA129522 = {S19378, 34, { &P15364,&P5690,&P4811,&P18092 }};	/*Useragent - Opera/9.02 (Windows NT 5.2; U; en)*/
const Device UA173601 = {S19379, 34, { &P15364,&P5690,&P4811,&P18092 }};	/*Useragent - Opera/9.00 (Windows NT 5.2; U; en)*/
const Device UA748325 = {S19380, 30, { &P15364,&P5690,&P4811,&P18092 }};	/*Useragent - Opera/9.02 (Windows XP; U; ru)*/
const Device UA787290 = {S19381, 34, { &P15364,&P5690,&P4811,&P18092 }};	/*Useragent - Opera/9.01 (Windows NT 5.2; U; en)*/
const Device UA1898638 = {S19382, 27, { &P15364,&P5690,&P4811,&P18092 }};	/*Useragent - Opera/9.01 (Windows NT 5.1)*/
const Device UA2391637 = {S19383, 42, { &P15364,&P5690,&P4811,&P18092 }};	/*Useragent - Opera - Opera/9.00 (Windows NT 5.1; U; en)*/
const Device UA2502470 = {S19384, 63, { &P15364,&P5690,&P4811,&P18092 }};	/*Useragent - ('Opera/9.00 (Windows NT 5.1; U; en)', 'Opera 9.0, Windows XP')*/
const Device UA2184856 = {S19385, 131, { &P15232,&P7106,&P15829,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9810; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.694 Mobile Safari/534.11+*/
const Device UA2229993 = {S19386, 77, { &P15232,&P7106,&P15829,&P18092 }};	/*Useragent - BlackBerry9810/7.1.0.694 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102*/
const Device UA604539 = {S19387, 246, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Zune 4.7; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA622649 = {S19388, 224, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA816049 = {S19389, 220, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA895783 = {S19390, 225, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Zune 4.7; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA900163 = {S19391, 203, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA1904101 = {S19392, 96, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.2; Microsoft Outlook 14.0.4760; Pro; ms-office; MSOffice 14)*/
const Device UA1966799 = {S19393, 230, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA1975611 = {S19394, 240, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Tablet PC 2.0; Zune 4.7; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2160083 = {S19395, 248, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; SLCC2; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2164421 = {S19396, 208, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2167025 = {S19397, 215, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Microsoft Outlook 14.0.6122; ms-office; MSOffice 14)*/
const Device UA2222696 = {S19398, 218, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2223541 = {S19399, 263, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2242565 = {S19400, 72, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.2; Microsoft Outlook 14.0.6123; Pro)*/
const Device UA2242621 = {S19401, 72, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.2; Microsoft Outlook 14.0.6112; Pro)*/
const Device UA2245660 = {S19402, 224, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2245725 = {S19403, 236, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Media Center PC 6.0; SLCC2; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2337538 = {S19404, 225, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Zune 4.7; Microsoft Outlook 14.0.6122; ms-office; MSOffice 14)*/
const Device UA2339236 = {S19405, 251, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2359843 = {S19406, 203, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2361616 = {S19407, 203, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2362813 = {S19408, 224, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2362824 = {S19409, 220, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2368305 = {S19410, 196, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2368416 = {S19411, 218, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2387978 = {S19412, 248, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; SLCC2; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2390209 = {S19413, 263, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2391959 = {S19414, 72, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.2; Microsoft Outlook 14.0.6023; Pro)*/
const Device UA2392149 = {S19415, 208, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2404134 = {S19416, 224, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2404776 = {S19417, 251, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2425620 = {S19418, 239, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Media Center PC 6.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2428138 = {S19419, 225, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Zune 4.7; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2432111 = {S19420, 72, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.2; Microsoft Outlook 14.0.6129; Pro)*/
const Device UA2441055 = {S19421, 96, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.2; Microsoft Outlook 14.0.4734; Pro; ms-office; MSOffice 14)*/
const Device UA2445490 = {S19422, 136, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2466520 = {S19423, 213, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; HPNTDFJS; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA2482029 = {S19424, 124, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2482175 = {S19425, 230, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2482453 = {S19426, 203, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2482991 = {S19427, 223, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Tablet PC 2.0; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2514000 = {S19428, 220, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2516160 = {S19429, 224, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2516161 = {S19430, 203, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2530094 = {S19431, 218, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2531386 = {S19432, 263, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2532180 = {S19433, 248, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; SLCC2; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2537813 = {S19434, 196, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2560336 = {S19435, 239, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Media Center PC 6.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2560721 = {S19436, 124, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2562718 = {S19437, 225, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2563798 = {S19438, 236, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Media Center PC 6.0; SLCC2; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2566876 = {S19439, 251, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2581228 = {S19440, 230, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2582450 = {S19441, 203, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2599630 = {S19442, 235, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2654324 = {S19443, 226, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; MALCJS; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA2664345 = {S19444, 129, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Win64; x64; Trident/6.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2695804 = {S19445, 258, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; SLCC2; Zune 4.7; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2700688 = {S19446, 246, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2765060 = {S19447, 219, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; MASMJS; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2765660 = {S19448, 273, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2793222 = {S19449, 218, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2802348 = {S19450, 96, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.2; Microsoft Outlook 14.0.5128; Pro; ms-office; MSOffice 14)*/
const Device UA2811650 = {S19451, 235, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2811800 = {S19452, 273, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Zune 4.7; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2837890 = {S19453, 218, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2853592 = {S19454, 228, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2859236 = {S19455, 234, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA3021738 = {S19456, 203, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3023182 = {S19457, 229, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3023411 = {S19458, 208, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3023465 = {S19459, 218, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3023560 = {S19460, 224, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3023762 = {S19461, 72, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.2; Microsoft Outlook 14.0.7015; Pro)*/
const Device UA3025165 = {S19462, 208, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3025428 = {S19463, 263, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3078088 = {S19464, 213, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3078468 = {S19465, 224, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3081715 = {S19466, 224, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3082919 = {S19467, 203, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3083105 = {S19468, 72, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.2; Microsoft Outlook 14.0.7106; Pro)*/
const Device UA3083568 = {S19469, 226, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; MALCJS; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3083683 = {S19470, 220, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3083739 = {S19471, 208, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; InfoPath.3; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3094738 = {S19472, 263, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; InfoPath.3; Media Center PC 6.0; SLCC2; Tablet PC 2.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3095829 = {S19473, 213, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3110972 = {S19474, 124, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3110979 = {S19475, 239, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3111673 = {S19476, 236, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; SLCC2; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3126796 = {S19477, 236, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Media Center PC 6.0; SLCC2; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3127368 = {S19478, 223, { &P15364,&P16110,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA885420 = {S19479, 128, { &P18669,&P7106,&P15829,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9320; en) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.398 Mobile Safari/534.11+*/
const Device UA2004600 = {S19480, 145, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST21i Build/11.0.A.0.16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032131 = {S19481, 141, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST21i Build/11.0.A.0.16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033324 = {S19482, 147, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; SonyST21i-o Build/11.0.A.0.16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2035149 = {S19483, 145, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; SonyST21i Build/11.0.A.0.25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2035157 = {S19484, 145, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyST21i Build/2.1.J.3.8.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2035153 = {S19485, 65, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - SonyST21i Build/11.0.A.0.16 stagefright/1.2 (Linux;Android 4.0.4)*/
const Device UA2035234 = {S19486, 145, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-mx; SonyST21a Build/11.0.A.0.16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2191571 = {S19487, 145, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; SonyST21i2 Build/11.0.A.1.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2193192 = {S19488, 145, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SonyST21i Build/11.0.A.0.26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2205960 = {S19489, 145, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; SonyST21a2 Build/11.0.A.1.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2229728 = {S19490, 141, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; ST21i2 Build/11.0.A.1.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2238877 = {S19491, 147, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SonyST21i-o Build/11.0.A.0.26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2342379 = {S19492, 146, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hr-hr; SonyST21iv Build/11.0.A.0.16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2347305 = {S19493, 141, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; ST21a2 Build/11.0.A.1.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2349646 = {S19494, 146, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SonyST21iv Build/11.0.A.0.26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2382496 = {S19495, 141, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-mx; ST21a Build/11.0.A.0.16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2388503 = {S19496, 145, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST21i Build/11.0.A.4.22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2401677 = {S19497, 141, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ST21i Build/11.0.A.0.26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2417159 = {S19498, 145, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyST21a Build/11.0.A.0.30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2419902 = {S19499, 63, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST21i2 Build/11.0.A.1.8)*/
const Device UA2424662 = {S19500, 146, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyST21a2 Build/11.0.A.1.12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426004 = {S19501, 141, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST21i Build/11.0.A.4.22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426425 = {S19502, 63, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST21i Build/11.0.A.4.22)*/
const Device UA2430029 = {S19503, 147, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; SonyST21i-o Build/11.0.A.4.22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2459310 = {S19504, 145, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyST21i2 Build/11.0.A.6.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2480436 = {S19505, 145, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; SonyST21i Build/11.0.A.0.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483825 = {S19506, 145, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyST21a Build/11.0.A.4.22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2485355 = {S19507, 145, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SonyST21i Build/11.0.A.4.25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2500336 = {S19508, 63, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST21a Build/11.0.A.0.16)*/
const Device UA2505274 = {S19509, 146, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hr-hr; SonyST21iv Build/11.0.A.4.22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2507127 = {S19510, 141, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST21i2 Build/11.0.A.6.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2508002 = {S19511, 146, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SonyST21iv Build/11.0.A.4.25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2554453 = {S19512, 142, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST21a2 Build/11.0.A.1.12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2568417 = {S19513, 145, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; SonyST21i Build/11.0.A.4.27) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2579063 = {S19514, 141, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST21a Build/11.0.A.4.22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580339 = {S19515, 141, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ST21i Build/11.0.A.4.25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583297 = {S19516, 145, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; SonyST21i2 Build/11.0.A.6.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2668326 = {S19517, 141, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; ST21i Build/11.0.A.4.27) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2668753 = {S19518, 141, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; ST21i2 Build/11.0.A.6.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695942 = {S19519, 65, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - SonyST21i Build/11.0.A.4.22 stagefright/1.2 (Linux;Android 4.0.4)*/
const Device UA2729059 = {S19520, 143, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; iw-; SonyST21i Build/11.0.A.4.22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2730097 = {S19521, 145, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyST21a2 Build/11.0.A.6.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2832793 = {S19522, 141, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST21a Build/11.0.A.0.30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985601 = {S19523, 143, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fa-; SonyST21i2 Build/11.0.A.6.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2995744 = {S19524, 141, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST21a2 Build/11.0.A.6.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3070269 = {S19525, 63, { &P15989,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST21a2 Build/11.0.A.1.8)*/
const Device UA252116 = {S19526, 125, { &P15364,&P5690,&P21901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/15.0.874.106) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.106 Safari/535.2*/
const Device UA252360 = {S19527, 76, { &P15364,&P5690,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/15.0.874.106)*/
const Device UA253620 = {S19528, 125, { &P15364,&P5690,&P21901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/15.0.874.120) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.120 Safari/535.2*/
const Device UA255083 = {S19529, 159, { &P15364,&P5690,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/15.0.874.121; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA255778 = {S19530, 125, { &P15364,&P5690,&P21901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/15.0.874.121) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA453173 = {S19531, 245, { &P15364,&P5690,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/15.0.874.121; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; Zurich Financial Services; Zurich Financial Services)*/
const Device UA454034 = {S19532, 177, { &P15364,&P5690,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; chromeframe/15.0.874.121; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8)*/
const Device UA577209 = {S19533, 125, { &P15364,&P5690,&P21901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/15.0.874.121) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA660262 = {S19534, 186, { &P15364,&P5690,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/15.0.874.121; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; InfoPath.1)*/
const Device UA703143 = {S19535, 192, { &P15364,&P5690,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/15.0.874.121; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; .NET4.0C; .NET4.0E)*/
const Device UA789719 = {S19536, 279, { &P15364,&P5690,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/15.0.874.121; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322; .NET CLR 1.0.3705; Tablet PC 1.7; .NET4.0C; .NET4.0E; Zurich Financial Services; Zurich Financial Services)*/
const Device UA862958 = {S19537, 223, { &P15364,&P5690,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/15.0.874.121; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.2)*/
const Device UA1895711 = {S19538, 121, { &P15364,&P5690,&P21901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/15.0.854.0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.854.0 Safari/535.2*/
const Device UA1905568 = {S19539, 125, { &P15364,&P5690,&P21901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/15.0.874.102) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2*/
const Device UA1930079 = {S19540, 220, { &P15364,&P5690,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/15.0.874.121; GTB7.2; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2141913 = {S19541, 121, { &P15364,&P5690,&P21901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/15.0.849.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/15.0.849.0 Safari/535.1*/
const Device UA2231563 = {S19542, 209, { &P15364,&P5690,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; SV1; chromeframe/15.0.874.121; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.1; MS-RTC LM 8)*/
const Device UA2335744 = {S19543, 217, { &P15364,&P5690,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/15.0.874.121; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.2; MS-RTC LM 8)*/
const Device UA2786843 = {S19544, 281, { &P15364,&P5690,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/15.0.874.121; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.2)*/
const Device UA2840703 = {S19545, 126, { &P15364,&P5690,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; chromeframe/15.0.874.106; .NET CLR 2.0.50727; InfoPath.2)*/
const Device UA615842 = {S19546, 64, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.229 Version/11.64*/
const Device UA624560 = {S19547, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.64*/
const Device UA624652 = {S19548, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.229 Version/11.64*/
const Device UA628606 = {S19549, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.229 Version/11.64*/
const Device UA630890 = {S19550, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.229 Version/11.64*/
const Device UA632044 = {S19551, 94, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5328); ru) Presto/2.10.229 Version/11.64*/
const Device UA634018 = {S19552, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.229 Version/11.64*/
const Device UA647524 = {S19553, 94, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5278); ru) Presto/2.10.229 Version/11.64*/
const Device UA648999 = {S19554, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.64*/
const Device UA656923 = {S19555, 94, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5218); ru) Presto/2.10.229 Version/11.64*/
const Device UA657559 = {S19556, 94, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.64*/
const Device UA659293 = {S19557, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.229 Version/11.64*/
const Device UA665096 = {S19558, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.229 Version/11.64*/
const Device UA669198 = {S19559, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.229 Version/11.64*/
const Device UA670056 = {S19560, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.229 Version/11.64*/
const Device UA671603 = {S19561, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.229 Version/11.64*/
const Device UA674960 = {S19562, 94, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5267); ru) Presto/2.10.229 Version/11.64*/
const Device UA679654 = {S19563, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5300); ru) Presto/2.10.229 Version/11.64*/
const Device UA682020 = {S19564, 93, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4664); ru) Presto/2.10.229 Version/11.64*/
const Device UA686703 = {S19565, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.229 Version/11.64*/
const Device UA686915 = {S19566, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.229 Version/11.64*/
const Device UA688559 = {S19567, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.229 Version/11.64*/
const Device UA694012 = {S19568, 101, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; en) Presto/2.10.229 Version/11.64,gzip(gfe) (via translate.google.com)*/
const Device UA708332 = {S19569, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.229 Version/11.64*/
const Device UA728862 = {S19570, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.229 Version/11.64*/
const Device UA755427 = {S19571, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5299); ru) Presto/2.10.229 Version/11.64*/
const Device UA767843 = {S19572, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5217); ru) Presto/2.10.229 Version/11.64*/
const Device UA770772 = {S19573, 93, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4947); ru) Presto/2.10.229 Version/11.64*/
const Device UA771310 = {S19574, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.229 Version/11.64*/
const Device UA771464 = {S19575, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.229 Version/11.64*/
const Device UA772608 = {S19576, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5230); ru) Presto/2.10.229 Version/11.64*/
const Device UA774528 = {S19577, 93, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4848); ru) Presto/2.10.229 Version/11.64*/
const Device UA777399 = {S19578, 94, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5288); ru) Presto/2.10.229 Version/11.64*/
const Device UA779555 = {S19579, 79, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; ru; rv:2.0) Gecko/X Firefox/4.0 Opera 11.64*/
const Device UA783067 = {S19580, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5336); ru) Presto/2.10.229 Version/11.64*/
const Device UA784799 = {S19581, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03780); ru) Presto/2.10.229 Version/11.64*/
const Device UA799899 = {S19582, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.64*/
const Device UA807159 = {S19583, 93, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4110); ru) Presto/2.10.229 Version/11.64*/
const Device UA814748 = {S19584, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.229 Version/11.64*/
const Device UA818989 = {S19585, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.229 Version/11.64*/
const Device UA828652 = {S19586, 93, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4157); ru) Presto/2.10.229 Version/11.64*/
const Device UA829873 = {S19587, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03278); ru) Presto/2.10.229 Version/11.64*/
const Device UA834309 = {S19588, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.64*/
const Device UA835418 = {S19589, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.229 Version/11.64*/
const Device UA875748 = {S19590, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.229 Version/11.64*/
const Device UA880635 = {S19591, 93, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4930); ru) Presto/2.10.229 Version/11.64*/
const Device UA886673 = {S19592, 93, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4871); ru) Presto/2.10.229 Version/11.64*/
const Device UA900169 = {S19593, 93, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5667); ru) Presto/2.10.229 Version/11.64*/
const Device UA900272 = {S19594, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5650); ru) Presto/2.10.229 Version/11.64*/
const Device UA900313 = {S19595, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5660); ru) Presto/2.10.229 Version/11.64*/
const Device UA900508 = {S19596, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5676); ru) Presto/2.10.229 Version/11.64*/
const Device UA900590 = {S19597, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.229 Version/11.64*/
const Device UA904827 = {S19598, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.229 Version/11.64*/
const Device UA1952858 = {S19599, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.229 Version/11.64*/
const Device UA1983718 = {S19600, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.229 Version/11.64*/
const Device UA1995650 = {S19601, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.10.229 Version/11.64*/
const Device UA2159898 = {S19602, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.229 Version/11.64*/
const Device UA2160091 = {S19603, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.229 Version/11.64*/
const Device UA2161749 = {S19604, 66, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; ru) Opera 11.64*/
const Device UA2181398 = {S19605, 93, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5970); ru) Presto/2.10.229 Version/11.64*/
const Device UA2189514 = {S19606, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.229 Version/11.64*/
const Device UA2218437 = {S19607, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.229 Version/11.64*/
const Device UA2220628 = {S19608, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5161); ru) Presto/2.10.229 Version/11.64*/
const Device UA2230718 = {S19609, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.10.229 Version/11.64*/
const Device UA2335008 = {S19610, 87, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5325); ru) Presto/2.10.229 Version/11.64*/
const Device UA2385239 = {S19611, 74, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.229 Version/11.64,gzip(gfe)*/
const Device UA2386550 = {S19612, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.229 Version/11.64*/
const Device UA2422384 = {S19613, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.229 Version/11.64*/
const Device UA2465960 = {S19614, 93, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5966); ru) Presto/2.10.229 Version/11.64*/
const Device UA2486813 = {S19615, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.229 Version/11.64*/
const Device UA2527289 = {S19616, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.229 Version/11.64*/
const Device UA2578207 = {S19617, 93, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5784); ru) Presto/2.10.229 Version/11.64*/
const Device UA2654504 = {S19618, 93, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5880); ru) Presto/2.10.229 Version/11.64*/
const Device UA2666949 = {S19619, 93, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 6068); ru) Presto/2.10.229 Version/11.64*/
const Device UA2684924 = {S19620, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4661); ru) Presto/2.10.229 Version/11.64*/
const Device UA2707268 = {S19621, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.229 Version/11.64*/
const Device UA2727122 = {S19622, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5745); ru) Presto/2.10.229 Version/11.64*/
const Device UA2839343 = {S19623, 93, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5997); ru) Presto/2.10.229 Version/11.64*/
const Device UA2867975 = {S19624, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.229 Version/11.64*/
const Device UA2984109 = {S19625, 93, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5977); ru) Presto/2.10.229 Version/11.64*/
const Device UA3072579 = {S19626, 86, { &P15364,&P7133,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6083); ru) Presto/2.10.229 Version/11.64*/
const Device UA2717086 = {S19627, 142, { &P12280,&P19826,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B329 Safari/8536.25*/
const Device UA2717095 = {S19628, 142, { &P12280,&P19826,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B146 Safari/8536.25*/
const Device UA2717564 = {S19629, 140, { &P12280,&P19826,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B144 Safari/8536.25*/
const Device UA2717570 = {S19630, 142, { &P12280,&P19826,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B145 Safari/8536.25*/
const Device UA2717576 = {S19631, 140, { &P12280,&P19826,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B142 Safari/8536.25*/
const Device UA2717568 = {S19632, 140, { &P12280,&P19826,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B143 Safari/8536.25*/
const Device UA2717871 = {S19633, 140, { &P12280,&P19826,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B141 Safari/8536.25*/
const Device UA2718652 = {S19634, 142, { &P12280,&P19826,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B350 Safari/8536.25*/
const Device UA369126 = {S19635, 147, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Sensation 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA369381 = {S19636, 140, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA369915 = {S19637, 150, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; zh-tw; HTC Sensation Z710e Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA371206 = {S19638, 86, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Sensation 4G with Beats Audio Build/IML74K)*/
const Device UA371230 = {S19639, 143, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sensation_4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA453341 = {S19640, 150, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA588605 = {S19641, 146, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Sensation_Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA589253 = {S19642, 164, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Sensation 4G with Beats Audio Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616154 = {S19643, 147, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_Sensation_4G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA636921 = {S19644, 147, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_Sensation_4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA646722 = {S19645, 150, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-sg; HTC_Sensation_Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA648881 = {S19646, 72, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Sensation Z710e Build/IML74K)*/
const Device UA680278 = {S19647, 136, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation Z710e Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA775639 = {S19648, 150, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Sensation_Z710e Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA824689 = {S19649, 69, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Sensation 4G Build/IML74K)*/
const Device UA833249 = {S19650, 167, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Sensation 4G XE with Beats Audio Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA897004 = {S19651, 143, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sensation 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1950951 = {S19652, 141, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation Z710e Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1955903 = {S19653, 146, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Sensation Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2023652 = {S19654, 131, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Z710e Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2026921 = {S19655, 148, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; HTC_Sensation_Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026965 = {S19656, 65, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Sensation 4G Build/IML74K)*/
const Device UA2030201 = {S19657, 126, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Z710e Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2032100 = {S19658, 147, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_Sensation_4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033505 = {S19659, 143, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Sensation 4G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2360160 = {S19660, 150, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Sensation_Z710e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2403189 = {S19661, 147, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_Sensation_4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2443752 = {S19662, 148, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; HTC Sensation Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2444443 = {S19663, 150, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Sensation_Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2457730 = {S19664, 150, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Sensation Z710e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2464041 = {S19665, 147, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Sensation 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2484480 = {S19666, 150, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Sensation Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2520067 = {S19667, 72, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Sensation Z710E Build/IMM76D)*/
const Device UA2579908 = {S19668, 147, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Sensation 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2694241 = {S19669, 72, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Sensation Z710e Build/IML74K)*/
const Device UA2811949 = {S19670, 145, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; HTC Sensation 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2813011 = {S19671, 144, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; Sensation_Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2816774 = {S19672, 68, { &P15201,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Sensation Z710e Build/IML74K)*/
const Device UA893321 = {S19673, 142, { &P18507,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1954205 = {S19674, 143, { &P18507,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-I8160L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1959647 = {S19675, 161, { &P18507,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; SAMSUNG GT-I8160/I8160XXLD8 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1967084 = {S19676, 66, { &P18507,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I8160 Build/GINGERBREAD)*/
const Device UA1988216 = {S19677, 161, { &P18507,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-at; SAMSUNG GT-I8160/I8160BOLE1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029950 = {S19678, 161, { &P18507,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pl-pl; SAMSUNG GT-I8160/I8160BOLF2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2230154 = {S19679, 143, { &P18507,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-I8160P Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2347455 = {S19680, 140, { &P18507,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2360229 = {S19681, 135, { &P18507,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8160 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2373099 = {S19682, 67, { &P18507,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I8160L Build/GINGERBREAD)*/
const Device UA2388232 = {S19683, 161, { &P18507,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; SAMSUNG GT-I8160/I8160BOLK2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2434259 = {S19684, 162, { &P18507,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-I8160P-ORANGE/I8160PBVLK3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2494939 = {S19685, 67, { &P18507,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I8160P Build/GINGERBREAD)*/
const Device UA2536648 = {S19686, 161, { &P18507,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; SAMSUNG GT-I8160/I8160XXLL2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2579132 = {S19687, 161, { &P18507,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; SAMSUNG GT-I8160/I8160XXLK6 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2765994 = {S19688, 129, { &P18507,&P5629,&P15760,&P18092 }};	/*Useragent - samsung-I8160/1.0 Linux/2.6.35.7 Android/2.3.5 Release/01.05.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA911381 = {S19689, 128, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; Build/JRN84D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1955782 = {S19690, 37, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android 4.1.1)*/
const Device UA2029463 = {S19691, 139, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; VegaBean Beta 1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2229157 = {S19692, 127, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2229551 = {S19693, 139, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; VegaBean Beta 3 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2230021 = {S19694, 156, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Android SDK built for x86 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2235176 = {S19695, 37, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android 4.1.2)*/
const Device UA2361305 = {S19696, 131, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; ATP7717 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2362262 = {S19697, 129, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; PC936 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2362528 = {S19698, 126, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; U1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2366440 = {S19699, 133, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; 9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2366654 = {S19700, 143, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; N101 DUAL CORE2 V11 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2367166 = {S19701, 129, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HC801 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2367394 = {S19702, 139, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; VegaBean Beta 6 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2368130 = {S19703, 133, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ACM3066-8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2375788 = {S19704, 147, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.5; en-us; TREQ_A10_Pocket2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2380215 = {S19705, 142, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Express Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2381673 = {S19706, 130, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; SHOGUN Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2382941 = {S19707, 138, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; Tablet 9748DCB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2383519 = {S19708, 136, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; X720D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2383538 = {S19709, 136, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; zh-tw; N7077 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2385158 = {S19710, 130, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; SP0718 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2385416 = {S19711, 132, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; Slick973 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2385995 = {S19712, 133, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2391430 = {S19713, 129, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TABLO Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2399494 = {S19714, 128, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.2 Version/4.0 Mobile Safari/530.17*/
const Device UA2401678 = {S19715, 139, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; VegaBean Beta 5 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2402539 = {S19716, 136, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; EVOPad R9720 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2403281 = {S19717, 129, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M1001 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2404384 = {S19718, 130, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; X2-IPS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2418952 = {S19719, 149, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Android SDK built for x86 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2419770 = {S19720, 123, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422398 = {S19721, 137, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; cm_tenderloin Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2423760 = {S19722, 126, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2425184 = {S19723, 118, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2426954 = {S19724, 153, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1911_v77_gq2000_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2429877 = {S19725, 167, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; e1909c_v77_gq1008_9p017_sui_a41_6628 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2432781 = {S19726, 131, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Android Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2434386 = {S19727, 158, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1911_v77_gq2000_9p017_6628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2451198 = {S19728, 141, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; VegaBean Beta 2.2 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2459108 = {S19729, 154, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_jbla668_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2460424 = {S19730, 132, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ADM816KC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2466826 = {S19731, 153, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_gq2002_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467256 = {S19732, 144, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; unknown Build/Mobiltab Sleek 10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2468147 = {S19733, 151, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; e1901_v77_jdt1_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469223 = {S19734, 160, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_jbla668_9p017_fwvga Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2481520 = {S19735, 66, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; cm_tenderloin Build/JZO54K)*/
const Device UA2481871 = {S19736, 157, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_gq2002_5647_notv Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2482898 = {S19737, 138, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.5; en-us; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2484369 = {S19738, 156, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1908_v77_jbl1_9p017_6628 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2485374 = {S19739, 136, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TAGI MID 950 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2485549 = {S19740, 156, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_gq2002_9p017_fg Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2486238 = {S19741, 130, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; PUBLIC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2487323 = {S19742, 151, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_jbl1_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2488403 = {S19743, 160, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1911_v77_gq2000_a41_6628_v20 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2504455 = {S19744, 145, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_jbl1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2507381 = {S19745, 159, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1911_v77_gq2000_ov5647_6628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2507992 = {S19746, 158, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Motorola Galaxy s3 Build/9.8.2O-72_VZW-8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2509594 = {S19747, 149, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Tablet-PC-4.1-Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ADM8000KP_A Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2510710 = {S19748, 136, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; da-dk; EN-TAB309 Build/IML74K..X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2516571 = {S19749, 126, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; V2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2518502 = {S19750, 130, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2527335 = {S19751, 134, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HD3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528232 = {S19752, 162, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_jbla668_9p017_nozhrtw Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528611 = {S19753, 151, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1908_v77_jbl1_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2530251 = {S19754, 154, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1920_v77_zlh_9p017_asx Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2531403 = {S19755, 129, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; V1013 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2532277 = {S19756, 154, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_jbla668_fwvga Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2534662 = {S19757, 156, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1909l_v77_gq1008_a41_v20 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556758 = {S19758, 148, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_cxq_asx Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2557456 = {S19759, 131, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; unknown Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2558069 = {S19760, 145, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1921_v77_cxq1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558305 = {S19761, 127, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; CTM Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2558477 = {S19762, 136, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TY-Q8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2561098 = {S19763, 157, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; e1901_v77_jdt1_9p017_fwvga Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2561731 = {S19764, 149, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Tablet-PC-4.1-Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ADM8000KP_B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2563067 = {S19765, 157, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1909c_v77_gq1008_a41_6628 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2578827 = {S19766, 138, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.5; vi-vn; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2582669 = {S19767, 137, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ss_tenderloin Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2582860 = {S19768, 134, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AML-MX REF Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2596926 = {S19769, 68, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; Build/JRO03C)*/
const Device UA2597872 = {S19770, 147, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_gq2002 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597880 = {S19771, 161, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1911_v77_mt6628_hjy1_9p017_v2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2598415 = {S19772, 154, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_gq2008_ov8825 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2599187 = {S19773, 128, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MR18 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2601526 = {S19774, 137, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; vi-vn; iphone Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654304 = {S19775, 56, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; MID Build/JRO03H)*/
const Device UA2655399 = {S19776, 130, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; RK736H Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2657282 = {S19777, 148, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_cxq_asx Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658455 = {S19778, 137, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.5; en-us; iPhone Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658575 = {S19779, 158, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1911_v77_mt6628_hjy1_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665715 = {S19780, 149, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_gq2002tc Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666264 = {S19781, 144, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_cxq Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2667173 = {S19782, 150, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1920_v77_zlh_fwvga Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2667858 = {S19783, 132, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; MID-709S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2673826 = {S19784, 135, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; H100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2697023 = {S19785, 135, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; iPhone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2697518 = {S19786, 127, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MID Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2699404 = {S19787, 138, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; MAX-SGS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2699514 = {S19788, 135, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; GF-2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2704589 = {S19789, 148, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; e2001_v89_gq2008s Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2707019 = {S19790, 156, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1920_v77_zlh_9p017_fwvga Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2707205 = {S19791, 147, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_gq2008 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708415 = {S19792, 76, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1901_v77_jbla668_fwvga Build/JRO03C)*/
const Device UA2713594 = {S19793, 162, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1911_v77_mt6628_hjy1_9p017_v20 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2728187 = {S19794, 138, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.5; tr-tr; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2728702 = {S19795, 153, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_gq2002_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2729077 = {S19796, 82, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1901_v77_jbla668_9p017_fwvga Build/JRO03C)*/
const Device UA2730066 = {S19797, 150, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; e1901_v77_gq2002_fg Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2730854 = {S19798, 82, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1911_v77_gq2000_a41_6628_v20 Build/JRO03C)*/
const Device UA2731289 = {S19799, 160, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; e1909c_v77_jbl_9p017_a41_6628 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2733222 = {S19800, 138, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; unknown Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734196 = {S19801, 151, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_jbl_hd_rtw Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734799 = {S19802, 138, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734931 = {S19803, 134, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; S98 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747447 = {S19804, 155, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1908_v77_jbl1a535_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747718 = {S19805, 78, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1908_v77_jbl1_9p017_6628 Build/JRO03C)*/
const Device UA2762938 = {S19806, 136, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; iphone5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2765424 = {S19807, 138, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; VegaBean Final Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2779392 = {S19808, 78, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1909l_v77_gq1008_a41_v20 Build/JRO03C)*/
const Device UA2779532 = {S19809, 136, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; iPhone5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2779617 = {S19810, 149, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; e2001_v89_jbl1a698 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2779783 = {S19811, 127, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MID Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2782002 = {S19812, 132, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; CTP838BT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2791446 = {S19813, 158, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1920_v77_gq2006_9p017_32p8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2794679 = {S19814, 123, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2797137 = {S19815, 133, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; HFM8000KP Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2798955 = {S19816, 140, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; HTC_POO_U Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2799056 = {S19817, 132, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; HRA_M801 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2843931 = {S19818, 151, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v15_hy_mt9t113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2845913 = {S19819, 138, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; unknown Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2848353 = {S19820, 60, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; unknown Build/JZO54K)*/
const Device UA2848880 = {S19821, 155, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.4; en-us; e1908_v77_nd6_6628_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2850144 = {S19822, 136, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.3; en-us; unknown Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2854699 = {S19823, 151, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1921_v77_mt6628_hjy Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2859362 = {S19824, 138, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; samsung Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2862332 = {S19825, 147, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1901_v77_jbl_hd Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867518 = {S19826, 147, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; eidolon15_nand_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2869041 = {S19827, 135, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; GASEM7000KD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2878631 = {S19828, 147, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; e1901_v77_gq2002 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2890715 = {S19829, 157, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; e1920_v77_zlh_ov5647_fwvga Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934440 = {S19830, 73, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1901_v77_jdt1_9p017 Build/JRO03C)*/
const Device UA2934900 = {S19831, 67, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1901_v77_jbl1 Build/JRO03C)*/
const Device UA2937360 = {S19832, 149, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; e1901_v77_gq2002gn Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952466 = {S19833, 127, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2956127 = {S19834, 156, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us;LC0720C Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 4.1.1; cs-cz; Build/JRO03CSafari/534.30*/
const Device UA2965356 = {S19835, 153, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 09l_v77_gq1008_a41_v20 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984326 = {S19836, 155, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; e1908_v77_nd3_6628_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986782 = {S19837, 37, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - stagefright/1.1 (Linux;Android 4.1.2)*/
const Device UA2987467 = {S19838, 76, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1920_v77_zlh_9p017_asx Build/JRO03C)*/
const Device UA2992957 = {S19839, 79, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1901_v77_jdt1_9p017_fwvga Build/JRO03C)*/
const Device UA2996313 = {S19840, 157, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; e1901_v77_jdt1_9p017_fwvga Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999382 = {S19841, 68, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android stagefright/1.2 (Linux;Android 4.1.2)*/
const Device UA3000687 = {S19842, 134, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MID Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3011163 = {S19843, 140, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; CNM-TP-10DC-16-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3012148 = {S19844, 149, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; e1901_v77_gq3002gn Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3023334 = {S19845, 139, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; CnM-TP-8DC-16-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3027018 = {S19846, 138, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; CnM-TP-7DC-8-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3043981 = {S19847, 76, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1901_v77_jbla668_9p017 Build/JRO03C)*/
const Device UA3044395 = {S19848, 135, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; iPhone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3046471 = {S19849, 72, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1920_v77_zlh_fwvga Build/JRO03C)*/
const Device UA3050298 = {S19850, 148, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1902_v77_jblw005 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3072906 = {S19851, 134, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078552 = {S19852, 123, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Build/Zeki.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3086327 = {S19853, 73, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1901_v77_jbl1_9p017 Build/JRO03C)*/
const Device UA3088792 = {S19854, 71, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 4.1.1; eidolon15_nand_gb2 Build/GRK39F)*/
const Device UA3090540 = {S19855, 138, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Android Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099184 = {S19856, 48, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - AndroidCvpPlayer(Linux; U; Android 4.1.2; en-us)*/
const Device UA3100693 = {S19857, 161, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; e1908_v77_mt6628_hjy1a6x_9p017 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102330 = {S19858, 143, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1910_v77_yz Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108101 = {S19859, 130, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3120818 = {S19860, 71, { &P17968,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; e1901_v77_gq2002tc Build/JRO03C)*/
const Device UA2674471 = {S19861, 106, { &P15364,&P18110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31*/
const Device UA2674501 = {S19862, 109, { &P15364,&P18110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31*/
const Device UA2674798 = {S19863, 106, { &P15364,&P18110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31*/
const Device UA2674811 = {S19864, 106, { &P15364,&P18110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31*/
const Device UA2675705 = {S19865, 109, { &P15364,&P18110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31*/
const Device UA2675736 = {S19866, 111, { &P15364,&P18110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31*/
const Device UA2678403 = {S19867, 104, { &P15364,&P18110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31*/
const Device UA2678521 = {S19868, 102, { &P15364,&P18110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31*/
const Device UA2707701 = {S19869, 109, { &P15364,&P18110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31*/
const Device UA2708293 = {S19870, 114, { &P15364,&P18110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31,gzip(gfe)*/
const Device UA2713777 = {S19871, 112, { &P15364,&P18110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31,gzip(gfe)*/
const Device UA2716386 = {S19872, 106, { &P15364,&P18110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31*/
const Device UA2795054 = {S19873, 143, { &P15364,&P18110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3047932 = {S19874, 106, { &P15364,&P18110,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31*/
const Device UA305956 = {S19875, 95, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7; en-us) AppleWebKit/534.20.8 (KHTML, like Gecko)*/
const Device UA344605 = {S19876, 97, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/535.1 (KHTML, like Gecko) Safari/535.1*/
const Device UA456195 = {S19877, 46, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3)*/
const Device UA626960 = {S19878, 52, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - QuickTime/7.7.1 (qtver=7.7.1;cpu=IA32;os=Mac 10.7.3)*/
const Device UA628061 = {S19879, 109, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.2.10) Gecko Expeditor 6.2.2.X-1115 Gecko/X-1115*/
const Device UA633122 = {S19880, 52, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - QuickTime/7.7.1 (qtver=7.7.1;cpu=IA32;os=Mac 10.7.4)*/
const Device UA670197 = {S19881, 101, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.53.11 (KHTML, like Gecko) Sparrow/1043*/
const Device UA798296 = {S19882, 122, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.53.11 (KHTML, like Gecko) LittleSnapper/12900 Safari/525.13*/
const Device UA896162 = {S19883, 103, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) Sparrow/1081.38*/
const Device UA896273 = {S19884, 103, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) Sparrow/1081.32*/
const Device UA896907 = {S19885, 87, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3; AppleWebKit/534.55.3 (KHTML, like Gecko)*/
const Device UA897364 = {S19886, 103, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) Sparrow/1081.51*/
const Device UA903162 = {S19887, 43, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/4.0 (Mac OS X 10.7.4) Java/1.6.0_33*/
const Device UA911705 = {S19888, 100, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) Sparrow/1144*/
const Device UA1960202 = {S19889, 105, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) NetNewsWire/3.2.7*/
const Device UA1996703 = {S19890, 105, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) NetNewsWire/3.3.1*/
const Device UA2039630 = {S19891, 87, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko)*/
const Device UA2200121 = {S19892, 105, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) NetNewsWire/3.3.2*/
const Device UA2201590 = {S19893, 100, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/1176*/
const Device UA2201596 = {S19894, 100, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/1172*/
const Device UA2202480 = {S19895, 52, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - QuickTime/7.7.1 (qtver=7.7.1;cpu=IA32;os=Mac 10.7.5)*/
const Device UA2230732 = {S19896, 100, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Safari/531.9*/
const Device UA2232262 = {S19897, 100, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/1178*/
const Device UA2237163 = {S19898, 103, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) NetNewsWire/3.3*/
const Device UA2237603 = {S19899, 100, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/1173*/
const Device UA2314456 = {S19900, 106, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) NetNewsWire/3.2.11*/
const Device UA2332594 = {S19901, 209, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/534.52.7 (KHTML	 like Gecko) Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_2; en) AppleWebKit/534.52.7 (KHTML	 like Gecko) &quot;wKiosk for Mac&quot;*/
const Device UA2349781 = {S19902, 105, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Vienna/2.6.0.2601*/
const Device UA2349842 = {S19903, 100, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Safari/522.0*/
const Device UA2350213 = {S19904, 102, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/1143.6*/
const Device UA2352522 = {S19905, 102, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/1043.1*/
const Device UA2353955 = {S19906, 104, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Vienna/3.0b.2820*/
const Device UA2366174 = {S19907, 106, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) NetNewsWire/3.2.15*/
const Device UA2371905 = {S19908, 73, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebune 4.7; .NET4.0C)*/
const Device UA2401306 = {S19909, 118, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Paparazzi!/0.6.5 (like Safari)*/
const Device UA2401617 = {S19910, 74, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - CalendarStore/5.0.3 (1204.2); iCal/5.0.3 (1605.4); Mac OS X/10.7.5 (11G63)*/
const Device UA2454658 = {S19911, 103, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/1081.52*/
const Device UA2454975 = {S19912, 110, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) RealPlayer/12.0.1.1750*/
const Device UA2455508 = {S19913, 75, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mac OS X/10.7.5 (11G63); ExchangeWebServices/2.2 (127); iCal/5.0.3 (1605.4)*/
const Device UA2466030 = {S19914, 95, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sunrise*/
const Device UA2480161 = {S19915, 100, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Safari/417.8*/
const Device UA2481503 = {S19916, 99, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Gruml/1.3.0*/
const Device UA2500385 = {S19917, 96, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) iWeb/304*/
const Device UA2525675 = {S19918, 100, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/1164*/
const Device UA2551112 = {S19919, 99, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/904*/
const Device UA2553208 = {S19920, 110, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) RealPlayer/12.0.0.1725*/
const Device UA2577740 = {S19921, 103, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/1081.53*/
const Device UA2578308 = {S19922, 98, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/536.26.17 (KHTML, like Gecko),gzip(gfe)*/
const Device UA2595018 = {S19923, 121, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Vienna/3.0b.2821 Safari/536.26.17*/
const Device UA2652743 = {S19924, 88, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mac OS X/10.7.5 (11G63); ExchangeWebServices/2.2 (127); AddressBookSourceSync/1.4 (1091)*/
const Device UA2695018 = {S19925, 101, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/959.4*/
const Device UA2726763 = {S19926, 110, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) RealPlayer/11.1.0.1116*/
const Device UA2746437 = {S19927, 102, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/1043.2*/
const Device UA2773259 = {S19928, 56, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.2 rv:175 (Macintosh; Mac OS X 10.7.5; en_US)*/
const Device UA2783677 = {S19929, 56, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.3 rv:176 (Macintosh; Mac OS X 10.7.5; en_US)*/
const Device UA2833185 = {S19930, 43, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/4.0 (Mac OS X 10.7.5) Java/1.7.0_09*/
const Device UA2835010 = {S19931, 94, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) TubeTV*/
const Device UA2857242 = {S19932, 135, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Vienna/3.0.0 Beta 11 :9d8d6f6: Safari/536.28.10*/
const Device UA2865257 = {S19933, 135, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Vienna/3.0.0 Beta 11 :9d8d6f6: Safari/536.29.13*/
const Device UA2871615 = {S19934, 56, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.4 rv:183 (Macintosh; Mac OS X 10.7.5; en_US)*/
const Device UA2893570 = {S19935, 56, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.4 rv:183 (Macintosh; Mac OS X 10.7.5; en_GB)*/
const Device UA2941953 = {S19936, 56, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.4 rv:183 (Macintosh; Mac OS X 10.7.5; de_DE)*/
const Device UA2981886 = {S19937, 103, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Sparrow/1081.27*/
const Device UA2981960 = {S19938, 105, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) NetNewsWire/3.1.7*/
const Device UA3007022 = {S19939, 105, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) NetNewsWire/4.0.0*/
const Device UA3014843 = {S19940, 116, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Vienna/3.0.0 Safari/536.30.1*/
const Device UA3030107 = {S19941, 56, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.5 rv:192 (Macintosh; Mac OS X 10.7.5; en_US)*/
const Device UA3030148 = {S19942, 56, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.5 rv:192 (Macintosh; Mac OS X 10.7.5; en_GB)*/
const Device UA3041727 = {S19943, 86, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.1.9) Gecko/X Mozilla/2.0*/
const Device UA3070632 = {S19944, 100, { &P17595,&P17297,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Safari/412.0*/
const Device UA174118 = {S19945, 97, { &P15364,&P18110,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; C) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.2 Safari/533.3*/
const Device UA379781 = {S19946, 93, { &P15364,&P18110,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.7.4 Safari/534.34*/
const Device UA396507 = {S19947, 95, { &P15364,&P18110,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.7.4 Safari/534.34*/
const Device UA591401 = {S19948, 101, { &P15364,&P18110,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Qt/4.7.4 Safari/534.3*/
const Device UA797230 = {S19949, 97, { &P15364,&P18110,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (QtEmbedded; U; Linux; C) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.1 Safari/533.3*/
const Device UA1894574 = {S19950, 103, { &P15364,&P18110,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Qt/4.7.1 Safari/534.10*/
const Device UA2333385 = {S19951, 93, { &P15364,&P18110,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.30 (KHTML, like Gecko) Qt/4.7.1 Safari/534.30*/
const Device UA2594472 = {S19952, 101, { &P15364,&P18110,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (QtEmbedded; U; Linux; fr-FR) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.2 Safari/533.3*/
const Device UA2726530 = {S19953, 101, { &P15364,&P18110,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (QtEmbedded; U; Linux; ar-SA) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.2 Safari/533.3*/
const Device UA2759448 = {S19954, 93, { &P15364,&P18110,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (QtEmbedded; Linux) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.7.1 Safari/534.34*/
const Device UA2843871 = {S19955, 101, { &P15364,&P18110,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (QtEmbedded; U; Linux; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.2 Safari/533.3*/
const Device UA182835 = {S19956, 133, { &P15364,&P7133,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.107 Safari/534.13 BuiltWith/1.2*/
const Device UA301306 = {S19957, 155, { &P15364,&P7133,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.84 Safari/534.13,gzip(gfe) (via translate.google.com)*/
const Device UA525282 = {S19958, 132, { &P15364,&P7133,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13 SunChrome/9.0*/
const Device UA2288553 = {S19959, 131, { &P15364,&P7133,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13, SurfCop 1.5*/
const Device UA2423767 = {S19960, 96, { &P15364,&P7133,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597 Safari/534.13*/
const Device UA2480467 = {S19961, 128, { &P15364,&P7133,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.84 Safari/534.13,gzip(gfe)*/
const Device UA2580020 = {S19962, 117, { &P15364,&P7133,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-us) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13*/
const Device UA2673185 = {S19963, 117, { &P15364,&P7133,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-CA) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13*/
const Device UA2678042 = {S19964, 117, { &P15364,&P7133,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; es-US) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13*/
const Device UA2708354 = {S19965, 117, { &P15364,&P7133,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; de-DE) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13*/
const Device UA2708666 = {S19966, 117, { &P15364,&P7133,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; fr-FR) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13*/
const Device UA2709337 = {S19967, 117, { &P15364,&P7133,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; fr-CA) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13*/
const Device UA2709526 = {S19968, 118, { &P15364,&P7133,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; haw-US) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13*/
const Device UA2735051 = {S19969, 117, { &P15364,&P7133,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; zh-CN) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13*/
const Device UA2585392 = {S19970, 57, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.15*/
const Device UA2669626 = {S19971, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.15*/
const Device UA2670007 = {S19972, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.15*/
const Device UA2670053 = {S19973, 67, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.15,gzip(gfe)*/
const Device UA2670079 = {S19974, 57, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2) Presto/2.12.388 Version/12.15*/
const Device UA2670783 = {S19975, 62, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1) Opera 12.15*/
const Device UA2670775 = {S19976, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.15*/
const Device UA2671749 = {S19977, 78, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; Edition Campaign 21) Presto/2.12.388 Version/12.15*/
const Device UA2671812 = {S19978, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.15*/
const Device UA2671851 = {S19979, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.15*/
const Device UA2671885 = {S19980, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.15*/
const Device UA2672090 = {S19981, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.15*/
const Device UA2672176 = {S19982, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.15*/
const Device UA2672298 = {S19983, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.15*/
const Device UA2672345 = {S19984, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.15*/
const Device UA2672449 = {S19985, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.15*/
const Device UA2672466 = {S19986, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.15*/
const Device UA2672523 = {S19987, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.15*/
const Device UA2672524 = {S19988, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.15*/
const Device UA2672671 = {S19989, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.15*/
const Device UA2672703 = {S19990, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.15*/
const Device UA2672808 = {S19991, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.15*/
const Device UA2672814 = {S19992, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.15*/
const Device UA2672911 = {S19993, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.15*/
const Device UA2672968 = {S19994, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.15*/
const Device UA2672985 = {S19995, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.15*/
const Device UA2672999 = {S19996, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.15*/
const Device UA2673032 = {S19997, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.15*/
const Device UA2673079 = {S19998, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.15*/
const Device UA2673094 = {S19999, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.15*/
const Device UA2673225 = {S20000, 74, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.15 Yandex.Translate*/
const Device UA2673266 = {S20001, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.15*/
const Device UA2673539 = {S20002, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.15*/
const Device UA2673544 = {S20003, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.15*/
const Device UA2673577 = {S20004, 68, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:2.0) Gecko/X Firefox/4.0 Opera 12.15*/
const Device UA2673632 = {S20005, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.15*/
const Device UA2673879 = {S20006, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.15*/
const Device UA2673967 = {S20007, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.15*/
const Device UA2674052 = {S20008, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.15*/
const Device UA2674183 = {S20009, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.15*/
const Device UA2674200 = {S20010, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.15*/
const Device UA2674213 = {S20011, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.15*/
const Device UA2674225 = {S20012, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.15*/
const Device UA2674722 = {S20013, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.15*/
const Device UA2674729 = {S20014, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.15*/
const Device UA2674952 = {S20015, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.15*/
const Device UA2674957 = {S20016, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.15*/
const Device UA2674964 = {S20017, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.15*/
const Device UA2674968 = {S20018, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6080)) Presto/2.12.388 Version/12.15*/
const Device UA2675316 = {S20019, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.15*/
const Device UA2675522 = {S20020, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.15*/
const Device UA2675566 = {S20021, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4933)) Presto/2.12.388 Version/12.15*/
const Device UA2675591 = {S20022, 64, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; WOW64) Presto/2.12.388 Version/12.15*/
const Device UA2675677 = {S20023, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.15*/
const Device UA2675679 = {S20024, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.15*/
const Device UA2675775 = {S20025, 106, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.15*/
const Device UA2675833 = {S20026, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.15*/
const Device UA2675911 = {S20027, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.15*/
const Device UA2675955 = {S20028, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.15*/
const Device UA2676369 = {S20029, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.15*/
const Device UA2676383 = {S20030, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.15*/
const Device UA2676387 = {S20031, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.15*/
const Device UA2676648 = {S20032, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.15*/
const Device UA2676662 = {S20033, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.15*/
const Device UA2676761 = {S20034, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.15*/
const Device UA2676859 = {S20035, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.15*/
const Device UA2676864 = {S20036, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.15*/
const Device UA2676867 = {S20037, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.15*/
const Device UA2677047 = {S20038, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.15*/
const Device UA2677066 = {S20039, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.15*/
const Device UA2677227 = {S20040, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.15*/
const Device UA2677232 = {S20041, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.15*/
const Device UA2677238 = {S20042, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.15*/
const Device UA2677703 = {S20043, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.15*/
const Device UA2679811 = {S20044, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.15*/
const Device UA2706350 = {S20045, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5650)) Presto/2.12.388 Version/12.15*/
const Device UA2708229 = {S20046, 107, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03686); MRSPUTNIK OW 2, 3, 0, 244) Presto/2.12.388 Version/12.15*/
const Device UA2708714 = {S20047, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03639)) Presto/2.12.388 Version/12.15*/
const Device UA2708959 = {S20048, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6035)) Presto/2.12.388 Version/12.15*/
const Device UA2709021 = {S20049, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.15*/
const Device UA2709036 = {S20050, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03402)) Presto/2.12.388 Version/12.15*/
const Device UA2709152 = {S20051, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6044)) Presto/2.12.388 Version/12.15*/
const Device UA2709412 = {S20052, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5230)) Presto/2.12.388 Version/12.15*/
const Device UA2710431 = {S20053, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.15*/
const Device UA2711688 = {S20054, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.15*/
const Device UA2712531 = {S20055, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.15*/
const Device UA2713241 = {S20056, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.15*/
const Device UA2713871 = {S20057, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.15*/
const Device UA2713910 = {S20058, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; Edition Yx) Presto/2.12.388 Version/12.15,gzip(gfe)*/
const Device UA2714071 = {S20059, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6046)) Presto/2.12.388 Version/12.15*/
const Device UA2714159 = {S20060, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.15*/
const Device UA2714224 = {S20061, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03658)) Presto/2.12.388 Version/12.15*/
const Device UA2714419 = {S20062, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.15*/
const Device UA2714580 = {S20063, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.15*/
const Device UA2714804 = {S20064, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5278)) Presto/2.12.388 Version/12.15*/
const Device UA2714810 = {S20065, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03755)) Presto/2.12.388 Version/12.15*/
const Device UA2715337 = {S20066, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03789)) Presto/2.12.388 Version/12.15*/
const Device UA2716072 = {S20067, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.15*/
const Device UA2717720 = {S20068, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6014)) Presto/2.12.388 Version/12.15*/
const Device UA2719097 = {S20069, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.15*/
const Device UA2734413 = {S20070, 107, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03402); MRSPUTNIK OW 2, 3, 0, 104) Presto/2.12.388 Version/12.15*/
const Device UA2734807 = {S20071, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.3 (build 02552)) Presto/2.12.388 Version/12.15*/
const Device UA2734813 = {S20072, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.3 (build 02560)) Presto/2.12.388 Version/12.15*/
const Device UA2735002 = {S20073, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4110)) Presto/2.12.388 Version/12.15*/
const Device UA2735591 = {S20074, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03672)) Presto/2.12.388 Version/12.15*/
const Device UA2735694 = {S20075, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5660)) Presto/2.12.388 Version/12.15*/
const Device UA2735718 = {S20076, 107, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03757); MRSPUTNIK OW 2, 3, 0, 288) Presto/2.12.388 Version/12.15*/
const Device UA2735910 = {S20077, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5336)) Presto/2.12.388 Version/12.15*/
const Device UA2736093 = {S20078, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5217)) Presto/2.12.388 Version/12.15*/
const Device UA2736218 = {S20079, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5288)) Presto/2.12.388 Version/12.15*/
const Device UA2740478 = {S20080, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6561)) Presto/2.12.388 Version/12.15*/
const Device UA2750857 = {S20081, 96, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.15,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2751498 = {S20082, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6032)) Presto/2.12.388 Version/12.15*/
const Device UA2751726 = {S20083, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4832)) Presto/2.12.388 Version/12.15*/
const Device UA2754304 = {S20084, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6573)) Presto/2.12.388 Version/12.15*/
const Device UA2755250 = {S20085, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6261)) Presto/2.12.388 Version/12.15*/
const Device UA2756718 = {S20086, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.15*/
const Device UA2767253 = {S20087, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.3 (build 02564)) Presto/2.12.388 Version/12.15*/
const Device UA2767380 = {S20088, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03791)) Presto/2.12.388 Version/12.15*/
const Device UA2767474 = {S20089, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5300)) Presto/2.12.388 Version/12.15*/
const Device UA2768359 = {S20090, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4661)) Presto/2.12.388 Version/12.15*/
const Device UA2768672 = {S20091, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5299)) Presto/2.12.388 Version/12.15*/
const Device UA2772193 = {S20092, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6277)) Presto/2.12.388 Version/12.15*/
const Device UA2780842 = {S20093, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5996)) Presto/2.12.388 Version/12.15*/
const Device UA2780924 = {S20094, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.4 (build 02652)) Presto/2.12.388 Version/12.15*/
const Device UA2782961 = {S20095, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6283)) Presto/2.12.388 Version/12.15*/
const Device UA2794122 = {S20096, 71, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.15 WebWasher 3.4*/
const Device UA2795233 = {S20097, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02746)) Presto/2.12.388 Version/12.15*/
const Device UA2795478 = {S20098, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5325)) Presto/2.12.388 Version/12.15*/
const Device UA2796811 = {S20099, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6628)) Presto/2.12.388 Version/12.15*/
const Device UA2860986 = {S20100, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.4 (build 02620)) Presto/2.12.388 Version/12.15*/
const Device UA2863242 = {S20101, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5734)) Presto/2.12.388 Version/12.15*/
const Device UA2870116 = {S20102, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6638)) Presto/2.12.388 Version/12.15*/
const Device UA2871958 = {S20103, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6620)) Presto/2.12.388 Version/12.15*/
const Device UA2876504 = {S20104, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.15*/
const Device UA2886066 = {S20105, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6323)) Presto/2.12.388 Version/12.15*/
const Device UA2887170 = {S20106, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6657)) Presto/2.12.388 Version/12.15*/
const Device UA2895071 = {S20107, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6327)) Presto/2.12.388 Version/12.15*/
const Device UA2896122 = {S20108, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6648)) Presto/2.12.388 Version/12.15*/
const Device UA2897402 = {S20109, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6322)) Presto/2.12.388 Version/12.15*/
const Device UA2936091 = {S20110, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5161)) Presto/2.12.388 Version/12.15*/
const Device UA2936428 = {S20111, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03399)) Presto/2.12.388 Version/12.15*/
const Device UA2955876 = {S20112, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.4 (build 02625)) Presto/2.12.388 Version/12.15*/
const Device UA2966140 = {S20113, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6665)) Presto/2.12.388 Version/12.15*/
const Device UA2966498 = {S20114, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6666)) Presto/2.12.388 Version/12.15*/
const Device UA2970268 = {S20115, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6341)) Presto/2.12.388 Version/12.15*/
const Device UA2971505 = {S20116, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6669)) Presto/2.12.388 Version/12.15*/
const Device UA2971584 = {S20117, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6673)) Presto/2.12.388 Version/12.15*/
const Device UA2992720 = {S20118, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03780)) Presto/2.12.388 Version/12.15*/
const Device UA2992881 = {S20119, 107, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03658); MRSPUTNIK OW 2, 3, 0, 216) Presto/2.12.388 Version/12.15*/
const Device UA2993164 = {S20120, 80, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03638)) Presto/2.12.388 Version/12.15*/
const Device UA3005898 = {S20121, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6325)) Presto/2.12.388 Version/12.15*/
const Device UA3013095 = {S20122, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6337)) Presto/2.12.388 Version/12.15*/
const Device UA3014686 = {S20123, 87, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/10.00 (Windows NT 5.1; U; Edition Englend Local; en) Presto/2.9.168 Version/12.15*/
const Device UA3023262 = {S20124, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6675)) Presto/2.12.388 Version/12.15*/
const Device UA3023770 = {S20125, 77, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.15,gzip(gfe),gzip(gfe)*/
const Device UA3028323 = {S20126, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6681)) Presto/2.12.388 Version/12.15*/
const Device UA3030075 = {S20127, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6676)) Presto/2.12.388 Version/12.15*/
const Device UA3034688 = {S20128, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.15*/
const Device UA3057101 = {S20129, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6336)) Presto/2.12.388 Version/12.15*/
const Device UA3060133 = {S20130, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6347)) Presto/2.12.388 Version/12.15*/
const Device UA3101781 = {S20131, 106, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02842); MRSPUTNIK OW 1, 0, 0, 25) Presto/2.12.388 Version/12.15*/
const Device UA3107199 = {S20132, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6667)) Presto/2.12.388 Version/12.15*/
const Device UA3125254 = {S20133, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6682)) Presto/2.12.388 Version/12.15*/
const Device UA3126903 = {S20134, 79, { &P15364,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.2 (build 7188)) Presto/2.12.388 Version/12.15*/
const Device UA3026507 = {S20135, 142, { &P12280,&P19826,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B329 Safari/8536.25*/
const Device UA3026508 = {S20136, 140, { &P12280,&P19826,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B142 Safari/8536.25*/
const Device UA3026513 = {S20137, 142, { &P12280,&P19826,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B350 Safari/8536.25*/
const Device UA3026578 = {S20138, 142, { &P12280,&P19826,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B145 Safari/8536.25*/
const Device UA3026590 = {S20139, 142, { &P12280,&P19826,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B146 Safari/8536.25*/
const Device UA3026972 = {S20140, 140, { &P12280,&P19826,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B143 Safari/8536.25*/
const Device UA3027045 = {S20141, 140, { &P12280,&P19826,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B144 Safari/8536.25*/
const Device UA3028984 = {S20142, 140, { &P12280,&P19826,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B141 Safari/8536.25*/
const Device UA3092408 = {S20143, 141, { &P12280,&P19826,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9Z999 Safari/8536.25*/
const Device UA625021 = {S20144, 90, { &P17595,&P17294,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.4; en-GB; rv:1.9.2.28) Gecko/X Firefox/3.6.28*/
const Device UA925689 = {S20145, 138, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2228514 = {S20146, 138, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2228657 = {S20147, 138, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SPH-L900 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2229272 = {S20148, 138, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2235432 = {S20149, 146, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I317 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2235488 = {S20150, 139, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SHV-E250S Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2237769 = {S20151, 139, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SHV-E250K Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261540 = {S20152, 139, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I317M Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261633 = {S20153, 139, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SHV-E250L Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261666 = {S20154, 138, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-R950 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2348449 = {S20155, 138, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I317 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2355080 = {S20156, 139, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889V Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2367199 = {S20157, 138, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2391289 = {S20158, 136, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SC-02E Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2400854 = {S20159, 139, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105T Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2403851 = {S20160, 138, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2408178 = {S20161, 146, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2423572 = {S20162, 138, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2423661 = {S20163, 139, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100T Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2425151 = {S20164, 138, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2426359 = {S20165, 138, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2426451 = {S20166, 132, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2427435 = {S20167, 139, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2427936 = {S20168, 139, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2431894 = {S20169, 139, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2434770 = {S20170, 139, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2484235 = {S20171, 143, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2 DLL1; SGH-T889 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2504557 = {S20172, 138, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; GT-N7100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2505922 = {S20173, 138, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-N719 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2506709 = {S20174, 138, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R950 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2535257 = {S20175, 139, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889V Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2562066 = {S20176, 139, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2579675 = {S20177, 148, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1 V2.1 ALK8; SGH-T889 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2603552 = {S20178, 139, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2848202 = {S20179, 132, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2983903 = {S20180, 132, { &P18442,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA49560 = {S20181, 128, { &P15250,&P6374,&P4418,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9360; en) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.319 Mobile Safari/534.11+*/
const Device UA896320 = {S20182, 77, { &P15250,&P6374,&P4418,&P18092 }};	/*Useragent - BlackBerry9360/7.0.0.586 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102*/
const Device UA806936 = {S20183, 86, { &P18088,&P17017,&P18525,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/6.0.24093/24.746; U; en) Presto/2.5.25 Version/10.54*/
const Device UA2431212 = {S20184, 88, { &P18088,&P17017,&P18525,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/6.0.24093/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2203728 = {S20185, 117, { &P17595,&P18207,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203732 = {S20186, 117, { &P17595,&P18207,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203793 = {S20187, 117, { &P17595,&P18207,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2231057 = {S20188, 154, { &P17595,&P18207,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via translate.google.com)*/
const Device UA2232666 = {S20189, 156, { &P17595,&P18207,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2235257 = {S20190, 127, { &P17595,&P18207,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)*/
const Device UA2402656 = {S20191, 116, { &P17595,&P18207,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.4 (KHTML like Gecko) Chrome/22.0.1229.79 Safari/537.4*/
const Device UA2553570 = {S20192, 117, { &P17595,&P18207,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2597519 = {S20193, 114, { &P17595,&P18207,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1082) AppleWebKit/537.4 (KHTML like Gecko) Chrome/22.0.1229.92 Safari/537.4*/
const Device UA2598527 = {S20194, 114, { &P17595,&P18207,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1081) AppleWebKit/537.4 (KHTML like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2708257 = {S20195, 148, { &P17595,&P18207,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.510 YaBrowser/1.5.1105.510 (gtsr) Safari/537.4*/
const Device UA2731187 = {S20196, 148, { &P17595,&P18207,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.510 YaBrowser/1.5.1105.510 (gtsr) Safari/537.4*/
const Device UA2734405 = {S20197, 148, { &P17595,&P18207,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.519 YaBrowser/1.5.1105.519 (bskt) Safari/537.4*/
const Device UA2781083 = {S20198, 117, { &P17595,&P18207,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA3091616 = {S20199, 117, { &P17595,&P18207,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA91069 = {S20200, 147, { &P15364,&P7133,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/1742; ) AppleWebKit/534.1 (KHTML, like Gecko) Chrome/6.0.444.0 Safari/534.1*/
const Device UA133678 = {S20201, 134, { &P15364,&P7133,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.33 Safari/534.3 SE 2.X MetaSr 1.0*/
const Device UA137809 = {S20202, 171, { &P15364,&P7133,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.33 Safari/534.3 SE 2.X MetaSr 1.0,gzip(gfe) (via translate.google.com)*/
const Device UA252691 = {S20203, 147, { &P15364,&P7133,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay/1738; ) AppleWebKit/534.1 (KHTML, like Gecko) Chrome/6.0.444.0 Safari/534.1*/
const Device UA634410 = {S20204, 140, { &P15364,&P7133,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Source Client/0; ) AppleWebKit/534.1 (KHTML, like Gecko) Chrome/6.0.444.0 Safari/534.1*/
const Device UA675202 = {S20205, 142, { &P15364,&P7133,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay; ) AppleWebKit/534.1 (KHTML, like Gecko) Chrome/6.0.444.0 Safari/534.1*/
const Device UA824059 = {S20206, 115, { &P15364,&P7133,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-GB) AppleWebKit/534.1 (KHTML, like Gecko) Chrome/6.0.428.0 Safari/534.1*/
const Device UA2316217 = {S20207, 115, { &P15364,&P7133,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.3 (KHTML	 like Gecko) Chrome/6.0.458.1 Safari/534.3*/
const Device UA2362173 = {S20208, 125, { &P15364,&P7133,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.464.0 Safari/534.3,gzip(gfe)*/
const Device UA32897 = {S20209, 151, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonST18a Build/4.0.A.2.299) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47520 = {S20210, 153, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonST18i Build/4.0.1.A.0.215) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47549 = {S20211, 153, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonST18i Build/4.0.1.A.0.165) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47550 = {S20212, 151, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonST18i Build/4.0.A.2.357) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47530 = {S20213, 151, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonST18i Build/4.0.A.2.335) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47582 = {S20214, 151, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonST18i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47583 = {S20215, 139, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; ST18i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48740 = {S20216, 151, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonSO-03C Build/4.0.D.2.61) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49769 = {S20217, 153, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonST18i Build/4.0.1.A.0.277) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49884 = {S20218, 153, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonST18a Build/4.0.1.A.0.284) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60441 = {S20219, 151, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonST18i Build/4.0.A.2.136) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70755 = {S20220, 153, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST18i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70768 = {S20221, 141, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; ST18i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA75261 = {S20222, 140, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; ST18i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76713 = {S20223, 152, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ie; SonyEricssonST18iv Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76749 = {S20224, 152, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonST18i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA77202 = {S20225, 152, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; SonyEricssonST18av Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79448 = {S20226, 152, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; SonyEricssonST18i Build/4.0.1.A.2.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA351166 = {S20227, 151, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-mx; SonyEricssonST18a Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA424862 = {S20228, 152, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonST18i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA454711 = {S20229, 153, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-03C Build/4.0.1.C.1.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623841 = {S20230, 153, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; SonyEricssonST18a Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA624994 = {S20231, 152, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonST18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627006 = {S20232, 140, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; ST18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA628727 = {S20233, 153, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonST18iv Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA630847 = {S20234, 64, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; ST18i Build/4.0.2.A.0.62)*/
const Device UA634124 = {S20235, 153, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; SonyEricssonST18av Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA637949 = {S20236, 152, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-03C Build/4.0.1.C.1.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA639759 = {S20237, 152, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-hk; SonyEricssonST18i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA656919 = {S20238, 141, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; ST18a Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA659141 = {S20239, 153, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonST18iv Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA659188 = {S20240, 140, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; ST18i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA665981 = {S20241, 63, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; ST18i Build/4.0.A.2.368)*/
const Device UA681961 = {S20242, 128, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ST18i Build/4.0.2.A.0.62) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA696484 = {S20243, 152, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonST18a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA755631 = {S20244, 140, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; ST18a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA838582 = {S20245, 139, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-mx; ST18a Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA856926 = {S20246, 140, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; ST18a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1932163 = {S20247, 151, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; SonyEricssonST18i Build/4.0.A.2.362) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1963584 = {S20248, 133, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ST18i Build/4.0.2.A.0.62) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2020919 = {S20249, 139, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ST18i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021650 = {S20250, 140, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-hk; ST18i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022460 = {S20251, 63, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; ST18a Build/4.0.A.2.368)*/
const Device UA2022597 = {S20252, 128, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ST18i Build/4.0.2.A.0.58) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022577 = {S20253, 128, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ST18i Build/4.0.2.A.0.42) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022757 = {S20254, 127, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; ST18i Build/4.0.A.2.368) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2032655 = {S20255, 152, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-nz; SonyEricssonST18a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2357303 = {S20256, 153, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-03C Build/4.0.1.C.1.31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2441644 = {S20257, 133, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ST18i Build/4.0.2.A.0.42) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2641274 = {S20258, 150, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-; SonyEricssonST18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2733320 = {S20259, 153, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonST18iv Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2758286 = {S20260, 64, { &P15397,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; ST18a Build/4.0.2.A.0.62)*/
const Device UA164272 = {S20261, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.7.62 Version/11.00*/
const Device UA165629 = {S20262, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.7.62 Version/11.00*/
const Device UA172406 = {S20263, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.7.62 Version/11.01*/
const Device UA172632 = {S20264, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.7.62 Version/11.01*/
const Device UA186235 = {S20265, 65, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; en-US) Presto/2.7.62 Version/11.01*/
const Device UA296858 = {S20266, 99, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.7.62 Version/11.00,gzip(gfe) (via translate.google.com)*/
const Device UA298902 = {S20267, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.7.62 Version/11.00*/
const Device UA299354 = {S20268, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.7.62 Version/11.00*/
const Device UA300751 = {S20269, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.7.62 Version/11.01*/
const Device UA301324 = {S20270, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.7.62 Version/11.01*/
const Device UA301600 = {S20271, 99, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.7.62 Version/11.01,gzip(gfe) (via translate.google.com)*/
const Device UA303280 = {S20272, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.7.62 Version/11.01*/
const Device UA328459 = {S20273, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.7.62 Version/11.01*/
const Device UA379003 = {S20274, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.7.62 Version/11.01*/
const Device UA408797 = {S20275, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.7.62 Version/11.01*/
const Device UA439282 = {S20276, 62, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.7.39 Version/11.00*/
const Device UA445991 = {S20277, 81, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; U; de; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 11.01*/
const Device UA446387 = {S20278, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.7.62 Version/11.01*/
const Device UA452891 = {S20279, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.7.62 Version/11.01*/
const Device UA498773 = {S20280, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.7.62 Version/11.00*/
const Device UA523507 = {S20281, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03639); ru) Presto/2.7.62 Version/11.01*/
const Device UA524447 = {S20282, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03278); ru) Presto/2.7.62 Version/11.01*/
const Device UA532254 = {S20283, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.7.62 Version/11.01*/
const Device UA536123 = {S20284, 66, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; de) Opera 11.01*/
const Device UA551781 = {S20285, 134, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.13) Gecko/X Opera/9.80 (Windows NT 6.1; U; zh-tw) Presto/2.7.62 Version/11.01*/
const Device UA557680 = {S20286, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.7.62 Version/11.00*/
const Device UA558133 = {S20287, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.7.62 Version/11.00*/
const Device UA566197 = {S20288, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.7.62 Version/11.00*/
const Device UA576422 = {S20289, 66, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1 x64; U; en) Presto/2.7.62 Version/11.00*/
const Device UA583914 = {S20290, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.7.62 Version/11.00*/
const Device UA607795 = {S20291, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.7.62 Version/11.00*/
const Device UA688739 = {S20292, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.7.62 Version/11.01*/
const Device UA705715 = {S20293, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.7.62 Version/11.01*/
const Device UA705721 = {S20294, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.7.62 Version/11.00*/
const Device UA717859 = {S20295, 66, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; fr) Opera 11.00*/
const Device UA722275 = {S20296, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.7.62 Version/11.01*/
const Device UA733376 = {S20297, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.7.62 Version/11.00*/
const Device UA754706 = {S20298, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.7.62 Version/11.00*/
const Device UA759503 = {S20299, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.7.62 Version/11.00*/
const Device UA789094 = {S20300, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.7.62 Version/11.01*/
const Device UA798250 = {S20301, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.7.62 Version/11.00*/
const Device UA845078 = {S20302, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.7.62 Version/11.01*/
const Device UA892001 = {S20303, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.7.62 Version/11.01*/
const Device UA895178 = {S20304, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.7.62 Version/11.00*/
const Device UA906614 = {S20305, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.7.62 Version/11.00*/
const Device UA1842603 = {S20306, 81, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; U; ru; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 11.00*/
const Device UA1878621 = {S20307, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4110); ru) Presto/2.7.62 Version/11.01*/
const Device UA1882670 = {S20308, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.7.62 Version/11.01*/
const Device UA1904324 = {S20309, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4848); ru) Presto/2.7.62 Version/11.00*/
const Device UA1955722 = {S20310, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5218); ru) Presto/2.7.62 Version/11.00*/
const Device UA1966275 = {S20311, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.7.62 Version/11.00*/
const Device UA1974341 = {S20312, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.7.62 Version/11.01*/
const Device UA1980938 = {S20313, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.7.62 Version/11.01*/
const Device UA1990353 = {S20314, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.7.62 Version/11.01*/
const Device UA2146185 = {S20315, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4848); ru) Presto/2.7.62 Version/11.01*/
const Device UA2158672 = {S20316, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.7.62 Version/11.01*/
const Device UA2159612 = {S20317, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.7.62 Version/11.00*/
const Device UA2220805 = {S20318, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.7.62 Version/11.01*/
const Device UA2221017 = {S20319, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5265); ru) Presto/2.7.62 Version/11.00*/
const Device UA2221264 = {S20320, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.7.62 Version/11.00*/
const Device UA2222231 = {S20321, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.7.62 Version/11.00*/
const Device UA2228787 = {S20322, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.7.62 Version/11.00*/
const Device UA2232277 = {S20323, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.7.62 Version/11.00*/
const Device UA2262847 = {S20324, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.7.62 Version/11.01*/
const Device UA2271059 = {S20325, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.7.62 Version/11.00*/
const Device UA2285640 = {S20326, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.7.62 Version/11.00*/
const Device UA2319200 = {S20327, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5196); ru) Presto/2.7.62 Version/11.00*/
const Device UA2337921 = {S20328, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.7.62 Version/11.00*/
const Device UA2347938 = {S20329, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.7.62 Version/11.00*/
const Device UA2395983 = {S20330, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.7.62 Version/11.01*/
const Device UA2397878 = {S20331, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.7.62 Version/11.01*/
const Device UA2400051 = {S20332, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.7.62 Version/11.01*/
const Device UA2426276 = {S20333, 72, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.7.62 Version/11.01,gzip(gfe)*/
const Device UA2444142 = {S20334, 85, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5267); ru) Presto/2.7.62 Version/11.00*/
const Device UA2455742 = {S20335, 72, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.7.62 Version/11.00,gzip(gfe)*/
const Device UA2456869 = {S20336, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.7.62 Version/11.01*/
const Device UA2457108 = {S20337, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.7.62 Version/11.00*/
const Device UA2518724 = {S20338, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5650); ru) Presto/2.7.62 Version/11.00*/
const Device UA2529425 = {S20339, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.7.62 Version/11.00*/
const Device UA2556777 = {S20340, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.7.62 Version/11.01*/
const Device UA2654212 = {S20341, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.7.62 Version/11.01*/
const Device UA2694391 = {S20342, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.7.62 Version/11.00*/
const Device UA2695145 = {S20343, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.7.62 Version/11.01*/
const Device UA2707954 = {S20344, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.7.62 Version/11.00*/
const Device UA2727611 = {S20345, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.7.62 Version/11.00*/
const Device UA2732089 = {S20346, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6060); ru) Presto/2.7.62 Version/11.00*/
const Device UA2750008 = {S20347, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.7.62 Version/11.01*/
const Device UA2829558 = {S20348, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5966); ru) Presto/2.7.62 Version/11.00*/
const Device UA3048063 = {S20349, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6089); ru) Presto/2.7.62 Version/11.00*/
const Device UA3086263 = {S20350, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5981); ru) Presto/2.7.62 Version/11.01*/
const Device UA3103677 = {S20351, 84, { &P15364,&P7133,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6578); ru) Presto/2.7.62 Version/11.01*/
const Device UA59258 = {S20352, 151, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Bonsai/Kernel_v5.0.0-B13; en-us Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70994 = {S20353, 128, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky/1.6.1 Safari/533.1*/
const Device UA74109 = {S20354, 128, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv_se) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky/1.5.0 Safari/533.1*/
const Device UA171684 = {S20355, 132, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA183426 = {S20356, 37, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - stagefright/1.1 (Linux;Android 2.3.3)*/
const Device UA189717 = {S20357, 37, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - stagefright/1.1 (Linux;Android 2.3.4)*/
const Device UA249682 = {S20358, 138, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; generic) AppleWebKit/534.51 (KHTML, like Gecko; Google Web Preview) Version/4.0 Mobile Safari/534.51*/
const Device UA316467 = {S20359, 174, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Android for Telechips TCC8900 Evaluation Board Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368713 = {S20360, 153, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1101_v73_klx1ctp_noalsps Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370416 = {S20361, 139, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; generic Build/4.4.1_214) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA389288 = {S20362, 132, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA409428 = {S20363, 37, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - stagefright/1.1 (Linux;Android 2.3.5)*/
const Device UA421293 = {S20364, 92, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us) AppleWebKit/999+ (KHTML, like Gecko) Safari/999.9*/
const Device UA433615 = {S20365, 132, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; de-de Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA434029 = {S20366, 131, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA440316 = {S20367, 114, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA452708 = {S20368, 171, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; Android for Telechips M801 Evaluation Board Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA535343 = {S20369, 128, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; generic) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA540533 = {S20370, 131, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA542800 = {S20371, 131, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA572695 = {S20372, 37, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - stagefright/1.1 (Linux;Android 2.3.6)*/
const Device UA609396 = {S20373, 136, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; generic) AppleWebKit/535.1 (KHTML, like Gecko; Google Web Preview) Version/4.0 Mobile Safari/535.1*/
const Device UA625586 = {S20374, 138, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; generic) AppleWebKit/534.24 (KHTML, like Gecko; Google Web Preview) Version/4.0 Mobile Safari/534.24*/
const Device UA628100 = {S20375, 136, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-tw; eye 2828 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA629959 = {S20376, 37, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - stagefright/1.1 (Linux;Android 2.3.1)*/
const Device UA635431 = {S20377, 145, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_hjy2a7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA643949 = {S20378, 136, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; generic) AppleWebKit/536.8 (KHTML, like Gecko; Google Web Preview) Version/4.0 Mobile Safari/536.8*/
const Device UA653583 = {S20379, 129, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; MID Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA655163 = {S20380, 135, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Android Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA656723 = {S20381, 138, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; iPhone 4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA657007 = {S20382, 146, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1102_v73_gq1001c Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA663057 = {S20383, 143, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_hjy1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA666616 = {S20384, 147, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; in-id; e1107_v73_jhgg1_imo Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA666726 = {S20385, 174, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; Android for Telechips TCC8900 Evaluation Board Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA682181 = {S20386, 155, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; huaqin73_gb Build/ALPS.GB.FDD2.MP.V2.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA682534 = {S20387, 37, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - stagefright/1.1 (Linux;Android 2.3.7)*/
const Device UA684818 = {S20388, 130, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; 3G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA693441 = {S20389, 140, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; youwave_custom Build/YWV) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA702147 = {S20390, 148, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1109_v73_gq1002_ctp Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA719884 = {S20391, 135, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Android Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA751829 = {S20392, 136, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Android Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA751956 = {S20393, 149, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1109_v73_gq1002_ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA752158 = {S20394, 132, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; nl-nl Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA753331 = {S20395, 132, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; es-es Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA753950 = {S20396, 146, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Full Android Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA754466 = {S20397, 131, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MID Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA757777 = {S20398, 148, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; e1107_v73_jhgg1_imo Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA762339 = {S20399, 145, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_hjy2a6 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA762398 = {S20400, 133, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; hd-us; eeepc Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA765518 = {S20401, 131, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA767316 = {S20402, 136, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Android Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA781589 = {S20403, 45, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Windowshop/1.0.42 (Android/2.3.4/Kindle Fire)*/
const Device UA795236 = {S20404, 151, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1101_v73_jbl3ctp_1208 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA801241 = {S20405, 147, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1103_v13_jhgg1ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA804976 = {S20406, 150, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1101_v73_jbl3ctp_1208 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA826132 = {S20407, 143, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1109_v73_jxlc1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA837171 = {S20408, 148, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1109_v73_gq1003_rtp Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA847341 = {S20409, 149, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1109_v73_gq1003_rtp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA847572 = {S20410, 142, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1107_v73_ddb1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA849935 = {S20411, 153, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1101_v73_klx1ctp_capella Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA853237 = {S20412, 148, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1107_v73_jhgg1_ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA854360 = {S20413, 147, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1103_v73_jhgg1ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA856263 = {S20414, 143, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_jbl1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA871693 = {S20415, 144, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1109_v73_jxlc1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA891951 = {S20416, 135, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; generic Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA892372 = {S20417, 146, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1103_v73_hjy1ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA896046 = {S20418, 151, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_jbl1flash8p4 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA899120 = {S20419, 156, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; e1101_v73_jbl3ctp_1208_inco Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1863883 = {S20420, 132, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-au Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1938642 = {S20421, 148, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1109_v13_gq1002_ctp Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1953921 = {S20422, 138, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; S150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1958309 = {S20423, 53, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3; MID Build/GRH55)*/
const Device UA1966630 = {S20424, 145, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_hjy1a6 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1967087 = {S20425, 151, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1101_v73_jbl2ctp_a519 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1975766 = {S20426, 154, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; SWTGT-7250 Build/gingerbread_house_r79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1978019 = {S20427, 134, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Parrot Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1984574 = {S20428, 131, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; M5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1993183 = {S20429, 135, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; ADR555 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1994322 = {S20430, 150, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_hjy1maxwest Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2004903 = {S20431, 138, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; i9988_custom Build/RMO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018345 = {S20432, 132, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018559 = {S20433, 132, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; fr-fr Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019054 = {S20434, 171, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Android for Telechips M801 Evaluation Board Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019159 = {S20435, 95, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; Android for Telechips M801 Evaluation Board Build/GRI40)*/
const Device UA2019751 = {S20436, 98, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.1; Android for Telechips TCC8900 Evaluation Board Build/GRH78)*/
const Device UA2020323 = {S20437, 134, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; Android Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021570 = {S20438, 129, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; MID Build/MID) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021718 = {S20439, 123, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Android Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2021961 = {S20440, 59, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; Android Build/GRJ22)*/
const Device UA2022774 = {S20441, 63, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; huaqin73_gb Build/GRJ22)*/
const Device UA2023252 = {S20442, 87, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; MID reference device (b01ref) Build/GB.b01ref.X)*/
const Device UA2023272 = {S20443, 149, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1109_v13_gq1003_rtp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023333 = {S20444, 150, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1101_v73_jbl2ctp_a519 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023624 = {S20445, 55, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; MID Build/GRJ22)*/
const Device UA2024459 = {S20446, 11, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Android/2.3*/
const Device UA2024830 = {S20447, 145, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1102_v73_gq1001c Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025196 = {S20448, 148, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_jbaol1g18 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025201 = {S20449, 146, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1109c_v73_gq1008 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025330 = {S20450, 50, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 2.3.6)*/
const Device UA2025661 = {S20451, 145, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_hjy2a5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026015 = {S20452, 145, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1107_v73_ht_ntv Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026100 = {S20453, 144, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1101_v73_nd1ctp Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026325 = {S20454, 130, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; zh-cn; PAD Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2026745 = {S20455, 145, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1107_v73_hjy_ctp Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027155 = {S20456, 149, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1105_v73_gq1006_rtp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027157 = {S20457, 133, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; T7303 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027180 = {S20458, 140, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; VITA_V4W Build/HuaqinA51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027183 = {S20459, 151, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1101_v73_jbl1ctp_a515 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028993 = {S20460, 125, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; 7260 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2029116 = {S20461, 139, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HKV MID 2S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029262 = {S20462, 146, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1107_v73_hjy_ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029882 = {S20463, 139, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; unknown Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029992 = {S20464, 145, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1103_v73_hjy1ctp Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030193 = {S20465, 143, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1107_v73_ddb1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030464 = {S20466, 137, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; generic Build/unknown) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030499 = {S20467, 59, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Android Build/GRJ90)*/
const Device UA2031629 = {S20468, 147, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; e1107_v73_jhgg1_mox Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032330 = {S20469, 141, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1107_v73_ht Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032411 = {S20470, 143, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1102_v73_jbl1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032434 = {S20471, 147, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1109_v73_jxlc1nps Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032547 = {S20472, 145, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1107_v73_ht_84g Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032912 = {S20473, 159, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1101_v73_jbl3ctp_1208_newboot Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033417 = {S20474, 148, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1103_v73_jxlc1btctp Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033653 = {S20475, 152, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1107_v73_jhgg1_ctp_84g Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033717 = {S20476, 149, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1109_v13_gq1002_ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034148 = {S20477, 150, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1103_v13_jhgg1ctpflb Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034265 = {S20478, 134, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; iDroid Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034307 = {S20479, 142, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1102_v73_jbl1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034440 = {S20480, 136, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Android Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2167110 = {S20481, 121, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - MSM7627A/1.0 Android/2.3 gingerbread/2.3.5 Release/11.16.2011 Browser/WebKit533.1 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA2183474 = {S20482, 144, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - null Linux/2.6.35 Android/2.3.6 Release/null Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1 Android 2.3.5;*/
const Device UA2200212 = {S20483, 37, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Pandora/1.6 Android/2.3.3 cdma_shadow*/
const Device UA2202520 = {S20484, 128, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.2 Version/4.0 Mobile Safari/530.17*/
const Device UA2222438 = {S20485, 139, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; acsip13_gb Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2223218 = {S20486, 152, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; O+_8.7_Android Build/OplusOplus_A100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2233814 = {S20487, 136, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; generic) AppleWebKit/537.4 (KHTML, like Gecko; Google Web Preview) Version/4.0 Mobile Safari/537.4*/
const Device UA2234416 = {S20488, 136, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Pioneer Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2256971 = {S20489, 128, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.2 Version/4.0 Mobile Safari/530.17*/
const Device UA2257034 = {S20490, 147, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1101_v73_jhgg1ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257116 = {S20491, 135, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; POCKET Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2319579 = {S20492, 59, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; generic Build/GRJ22)*/
const Device UA2340944 = {S20493, 135, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; iPhone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2343746 = {S20494, 128, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.2 Version/4.0 Mobile Safari/530.17*/
const Device UA2346327 = {S20495, 151, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1109l_jyuany2_6622_nd Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2352158 = {S20496, 143, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; iDroid S Build/iDroidiDroid) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2352843 = {S20497, 129, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; MID Build/MID) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2360389 = {S20498, 128, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.2 Version/4.0 Mobile Safari/530.17*/
const Device UA2361541 = {S20499, 141, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; generic Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2361996 = {S20500, 139, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; U8667 Build/HuaweiU8667) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2362287 = {S20501, 138, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; FLY IQ256 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2364729 = {S20502, 160, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_5010/5010_V1.10; 240*320; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2365331 = {S20503, 134, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; W1350 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2365457 = {S20504, 135, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3;en-us; ZTE-U V760 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2365608 = {S20505, 128, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.2 Version/4.0 Mobile Safari/530.17*/
const Device UA2366364 = {S20506, 91, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; BlackBerry Runtime for Android Apps Build/2.1.0.131)*/
const Device UA2366569 = {S20507, 140, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-pt; tmn smart a8 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2372389 = {S20508, 142, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Changjiang P5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2373419 = {S20509, 138, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; SAGETEL G3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2373651 = {S20510, 134, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ThL V8 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2373818 = {S20511, 164, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; LG-su760 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2374285 = {S20512, 144, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; da-dk; CHARM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2375151 = {S20513, 157, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_nd1gti9220flash8p4 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2375348 = {S20514, 134, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U670C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2375498 = {S20515, 138, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-mx; M4TEL SS550 Build/GRJ90) AppleWebKit/533.1(KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2375875 = {S20516, 134, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; UR-A38 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2376043 = {S20517, 136, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; TWZ TA2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2376067 = {S20518, 134, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; W2400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2377224 = {S20519, 168, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; BW-BenWee 5100S/BenWee_5100S_V1.0; 320*480;  CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2378040 = {S20520, 134, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; P9100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2378106 = {S20521, 142, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; MTAG 351 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2378636 = {S20522, 133, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; H500 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2380172 = {S20523, 131, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; 4U Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2380688 = {S20524, 134, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; GP-351 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2380844 = {S20525, 148, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; Telenor_One_Touch_C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2380895 = {S20526, 138, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; WX06K Build/113.0.0027) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2381676 = {S20527, 136, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; MTC 960 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2381972 = {S20528, 147, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; e1109c_v73_gq1008a Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2382985 = {S20529, 132, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; x603 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2383238 = {S20530, 143, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; COOLPIX S800c Build/CP01_WW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2383581 = {S20531, 140, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; izoom i7001 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2383794 = {S20532, 148, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; MegaFon_SP-A10 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2383914 = {S20533, 132, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A99 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2384175 = {S20534, 145, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; sr-rs; Telenor_Smart_Pro Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2384789 = {S20535, 154, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; iBall-Andi4.3J Build/ALPS.GB2.MP.V1.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2385046 = {S20536, 153, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; Ilium_S100 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2385908 = {S20537, 150, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; GT7250S Build/gingerbread_qrd_r209) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2387284 = {S20538, 137, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; IMO S900 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2387655 = {S20539, 140, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; X501_USA_RS Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2393453 = {S20540, 168, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GIO-GiONEE_C600/CBD75B04_01_T9151 ; 320*480; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2393454 = {S20541, 140, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Q7079W Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2394494 = {S20542, 132, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; M019 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2395361 = {S20543, 137, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Mystique Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2395813 = {S20544, 141, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; A51 Build/A51Gizoom-i8004) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2395821 = {S20545, 140, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; yuanda73_gb Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2395862 = {S20546, 142, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-B9120 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2397578 = {S20547, 136, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HENGXUN Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2398019 = {S20548, 147, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; Snexian Mi320 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2398211 = {S20549, 131, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; X3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2400800 = {S20550, 134, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; B4000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2401265 = {S20551, 138, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; pt-br; GP-431 Build/REL_2.3.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2402494 = {S20552, 134, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MI360 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2403007 = {S20553, 136, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; maxwest Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2404244 = {S20554, 131, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; S9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2404725 = {S20555, 136, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; X40 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2412583 = {S20556, 145, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; e1101_v13_hjy1ctp Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2417764 = {S20557, 156, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; e1108_v73_jbl1flash_extreme Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2419006 = {S20558, 137, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; M20 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2424425 = {S20559, 147, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_jbl1a535 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2426518 = {S20560, 133, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; E9771 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2428576 = {S20561, 139, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; SmartTouch Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2441964 = {S20562, 142, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; e1109_v73_jy1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2444324 = {S20563, 145, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1103_v73_nd2ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2458037 = {S20564, 150, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_jbl1mt9p017 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2467205 = {S20565, 142, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SICT Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2481127 = {S20566, 148, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1109l_jyuany2_6622 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2496337 = {S20567, 145, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_hjy1a5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2503921 = {S20568, 153, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1815_v20_hy1qixian_6622 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2545462 = {S20569, 143, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1117_v13_jhgg Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2546732 = {S20570, 145, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; e1109c_v73_i9220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2547652 = {S20571, 48, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - AndroidCvpPlayer(Linux; U; Android 2.3.5; en-us)*/
const Device UA2555309 = {S20572, 139, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; SmartWorld Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2577522 = {S20573, 149, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1117_v13_jhggtv_flb Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2588138 = {S20574, 147, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; e1117_v73_jhgg_mox Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2594571 = {S20575, 146, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; e1117_v73_zxd2ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2602608 = {S20576, 157, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; Ilium_AVVIO_760 Build/a200Ilium_AVVIO_760) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2642876 = {S20577, 136, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Android Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2653013 = {S20578, 148, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1109c_v73_gq1008ov Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2663376 = {S20579, 125, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-F200/1.0 Linux/2.6.35.7 Android/2.3.6 Release/01.21.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2687569 = {S20580, 145, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1103_v73_nd1ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2697320 = {S20581, 146, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 5.1.1; th-th; iPhone Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2698267 = {S20582, 146, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 5.1.1; en-us; iPhone Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2698301 = {S20583, 140, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; O+8.1Android Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2723745 = {S20584, 60, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; Pioneer Build/GRK39F)*/
const Device UA2729767 = {S20585, 133, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; T730 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2743494 = {S20586, 143, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; e1102_v73_geg1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2744596 = {S20587, 146, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1108_v73_hjy1a6w Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2747044 = {S20588, 154, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; e1108_v73_jbl1a535mt9p017 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2758422 = {S20589, 62, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; S150 Build/GINGERBREAD)*/
const Device UA2758589 = {S20590, 73, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; e1109_v73_gq1002_ctp Build/GRK39F)*/
const Device UA2764141 = {S20591, 138, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PhydeuxROM Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2766851 = {S20592, 137, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; gb92236a Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2777558 = {S20593, 59, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; POCKET Build/GRK39F)*/
const Device UA2782062 = {S20594, 134, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; GK-A1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2782932 = {S20595, 149, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de;  Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 TOLINO_BROWSER_1.1.1*/
const Device UA2786923 = {S20596, 145, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; e1109c_v73_gxqg3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2788631 = {S20597, 136, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Build/Android_V01.01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2808094 = {S20598, 72, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; e1109_v73_gq1002_ctp Build/GRJ22)*/
const Device UA2809397 = {S20599, 143, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; e1117_v73_jhgg Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2810241 = {S20600, 58, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Parrot Build/GRJ90)*/
const Device UA2811416 = {S20601, 152, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; e1108_v73_jbl1custom8p4 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2811434 = {S20602, 137, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; MID Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2812773 = {S20603, 128, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Android Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2813365 = {S20604, 40, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Android 2.3.6; Application: com.e8tracks*/
const Device UA2819158 = {S20605, 177, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - TCL-E906_TD/1.0 Android/2.3 (Linux; U; Android 2.3.7; zh-cn) Release/12.09.2011 Browser/WAP 2.0 Mozilla/5.0 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2826867 = {S20606, 148, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; e1815_v20_lbx2_6622 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2837467 = {S20607, 137, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Phone 5s Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2838934 = {S20608, 143, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1109l_jyuany2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2840502 = {S20609, 131, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pl-pl; LUX Build/LUX01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2851876 = {S20610, 144, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.9; es-es; HAPPLE Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2933696 = {S20611, 146, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; e1117_v73_zxd3ctp Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2933897 = {S20612, 144, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1107_v13_jhgg1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2948906 = {S20613, 132, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; tr-tr Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2949400 = {S20614, 149, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1102_v73_jbl1_lt18i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2966883 = {S20615, 130, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Android-Cloud-X2/1.0 Linux/2.6.35.7 Android/2.3 Release/06.17.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2979640 = {S20616, 148, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; iPhone 4S (MC605CH) Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2981625 = {S20617, 145, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; e1109c_v73_jhgg1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982155 = {S20618, 150, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; e1101_v73_jhgg1ctp_vi Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3008506 = {S20619, 142, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-; hexing75_hspa   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3043355 = {S20620, 153, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; el-gr; e1117_v73_zxd3ctp_turbox Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3044926 = {S20621, 146, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn;Iphone 4S Build/Iphone 4S V1.0.0) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3103155 = {S20622, 144, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; hexing75_hspa   Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3117990 = {S20623, 148, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; e1108_v73_jbl1_top5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3118180 = {S20624, 149, { &P17968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; e1815_hy1qixian_i667 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2400853 = {S20625, 132, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2428594 = {S20626, 61, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P3110 Build/JRO03C)*/
const Device UA2466575 = {S20627, 139, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2506390 = {S20628, 130, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2578598 = {S20629, 61, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P3110 Build/JZO54K)*/
const Device UA2614891 = {S20630, 139, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GT-P3110 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2615092 = {S20631, 139, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2617685 = {S20632, 145, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; GT-P3110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2620150 = {S20633, 139, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2622301 = {S20634, 162, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2622655 = {S20635, 163, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/5.0.2 Version/4.0 Safari/534.30 Safari/6533.18.5*/
const Device UA2623227 = {S20636, 218, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.3.4-94 (Android 4.1.1; samsung GT-P3110 Build/JRO03C.P3110XXCLK7)*/
const Device UA2623681 = {S20637, 139, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2624741 = {S20638, 81, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.1.1; en-GB; GT-P3110 Build/JRO03C.P3110XXCLK7)*/
const Device UA2625222 = {S20639, 71, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - BBC iPlayer (AndroidApp; 1.5.0.666; X) GT-P3110 (Android 4.1.1, SDK 16)*/
const Device UA2625710 = {S20640, 82, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Biblia/2.2.8.1 (Linux; U; Android 4.1.1; es-MX; GT-P3110 Build/JRO03C.P3110XXCLK5)*/
const Device UA2625937 = {S20641, 130, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3110 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2629600 = {S20642, 254, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3110 Build/JRO03C) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.4 Version/4.0 Mobile Safari/530.17 baidubrowser/061_1.8.4.2_diordna_679_006/gnusmas_61_1.1.4_0113P-TG/Xy/92E1087FBD8BB71D23F46C0EFX%7C0/1*/
const Device UA2629990 = {S20643, 63, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - BBC News (AndroidApp; 2.1; 30) GT-P3110 (Android 4.1.1, SDK 16)*/
const Device UA2633087 = {S20644, 71, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - BBC iPlayer (AndroidApp; 1.5.2.170; X) GT-P3110 (Android 4.1.1, SDK 16)*/
const Device UA2633920 = {S20645, 92, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - com.google.android.youtube/4.3.9(Linux; U; Android 4.1.1; en_GB; GT-P3110 Build/JRO03C) gzip*/
const Device UA2634598 = {S20646, 179, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P3110 Build/JRO03C) [FBAN/Orca-Android;FBAV/2.3.2-release;FBLC/en_GB;FBBV/X;FBCR/null;FBMF/samsung;FBBD/samsung;FBDV/GT-P3110;FBSV/4.1.1]*/
const Device UA2635391 = {S20647, 132, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; GT-P3110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2635767 = {S20648, 131, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 4.1.1; fr-fr; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2635853 = {S20649, 145, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30*/
const Device UA2636023 = {S20650, 75, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - GoogleAnalytics/2.0 (Linux; U; Android 4.1.2; en-gb; GT-P3110 Build/JZO54K)*/
const Device UA2636198 = {S20651, 175, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 GoogleGoggles-Android/1.9; gzip*/
const Device UA2636312 = {S20652, 130, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P3110 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2636741 = {S20653, 132, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Bui Android 4.1.1; en-gb; GT-P3110 HTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2636833 = {S20654, 175, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3110 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 GoogleGoggles-Android/1.9; gzip*/
const Device UA2637053 = {S20655, 167, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWn-gb; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2637168 = {S20656, 123, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like 6236.8.10.X; __utmc4.30*/
const Device UA2637170 = {S20657, 330, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JR5D; _qsst=X; qb_etc_ likus=%7B%22c%22%3A%7B%7D%7D; __utma=X.X.X.X.X.1; __utmb=X.14.10.X; __utmc=X; __utmz=X.X.1.1.utmgclid=cjsardbeprycfunmtaodplkaxa|utmccn=brand%20pure%20(pc)%20-%20exact|utmcmd=ppc|utmctr=harveys; qb_ss_status=%7B%22c%22%3A%7B%7D%2C%22cg%22%3A0.X%7D*/
const Device UA2637585 = {S20658, 459, { &P15932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRedetails%3Dital%2520white%22%2C%22time%22%3AX%7D%5D%2C%22sessionLandingPage%22%3A%22http%3A//www.harveysfurniture.co.uk/mobile/products/dining-room/categories/display-cabinets.aspx%3Frangedetails%3Dital%2520white%22%2C%22__v%22%3A%7B%7D%7D; _qubitTracker=X.X; _qPageNum_harveysfurniture=3; _qst=%5B1%2C0%5D; _qsst=X; qb_etc_status=%7B%22c%22%3A%7B%7D%7D; qb_ss_status=%7B%22c%22%3A%7B%7D%2C%22cg%2*/
const Device UA456754 = {S20659, 98, { &P15364,&P18110,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Ultimate Edition; X11; Linux x86; rv:8.0) Gecko/X Ultimate Edition / 3.2 Firefox/10.0*/
const Device UA2368607 = {S20660, 67, { &P15364,&P18110,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:10.0.11) Gecko/X Firefox/10.0.11*/
const Device UA2740409 = {S20661, 97, { &P15364,&P18110,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:10.0.12) Gecko/X Firefox/10.0.12 Iceweasel/10.0.12 Snapper/1.0*/
const Device UA2850849 = {S20662, 69, { &P15364,&P18110,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:10.0.12) Gecko/X Iceweasel/10.0.12*/
const Device UA2205341 = {S20663, 142, { &P15767,&P18432,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A403 Safari/7534.48.3*/
const Device UA2260587 = {S20664, 142, { &P15767,&P18432,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A406 Safari/7534.48.3*/
const Device UA2260666 = {S20665, 142, { &P15767,&P18432,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A407 Safari/7534.48.3*/
const Device UA2260670 = {S20666, 144, { &P15767,&P18432,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A523 Safari/7534.48.3*/
const Device UA2362347 = {S20667, 145, { &P15767,&P18432,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A8426 Safari/7534.48.3*/
const Device UA2437691 = {S20668, 144, { &P15767,&P18432,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A550 Safari/7534.48.3*/
const Device UA2460932 = {S20669, 145, { &P15767,&P18432,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A8500 Safari/7534.48.3*/
const Device UA421980 = {S20670, 39, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Web/1.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA428203 = {S20671, 57, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Million%20HD%20WPs/8.80.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA428398 = {S20672, 56, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - 2%20Million%20WPs/9.16.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA454600 = {S20673, 43, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Feeds/2.2.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA454850 = {S20674, 48, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - WOW%20WPs/9.26.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA529033 = {S20675, 44, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ABCNews/1.4 CFNetwork/485.13.9 Darwin/11.0.0*/
const Device UA535773 = {S20676, 60, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - locationd (unknown version) CFNetwork/485.13.9 Darwin/11.0.0*/
const Device UA545521 = {S20677, 54, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ChromeSync%20Lite/1.2 CFNetwork/485.13.9 Darwin/11.0.0*/
const Device UA546004 = {S20678, 51, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - SexyNFunWPs/8.50.1 CFNetwork/485.13.9 Darwin/11.0.0*/
const Device UA609028 = {S20679, 47, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - NaverCafe/1.1.9 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA613264 = {S20680, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Clipish/9.29.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA613277 = {S20681, 51, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - HideMyPhotos/9.29.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA635132 = {S20682, 40, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ZBar/2.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA636790 = {S20683, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - GQMagazine/1.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA641394 = {S20684, 53, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - EpicWallPaper2011/1.6 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA661332 = {S20685, 53, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - HOT%20HD%20WPs/9.19.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA662306 = {S20686, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Animations/9.30.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA665122 = {S20687, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - BrowserFree/1.2.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA671450 = {S20688, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - MobileFilter/3.01 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA676719 = {S20689, 47, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Web%20Feeds/1.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA676927 = {S20690, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - SpringBoard/50 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA686116 = {S20691, 50, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - TweetSpeaker/1.0.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA687424 = {S20692, 57, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Celeb%20Wallpapers/9.13.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA688583 = {S20693, 50, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - vBrowse%202.0/2.03 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA696041 = {S20694, 44, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - iScanner/1.4 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA698859 = {S20695, 44, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Settings/1.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA722396 = {S20696, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - VanillaSurf/2.9.6 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA725442 = {S20697, 52, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ImageSearcherS/2.1.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA746441 = {S20698, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - DL-Picture/1.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA749842 = {S20699, 51, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Diigo%20Browser/2.6 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA755694 = {S20700, 43, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - kikin/1.9.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA773212 = {S20701, 55, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Love%20Messenger/9.16.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA784061 = {S20702, 50, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Codes%20Reader/2.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA787889 = {S20703, 47, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Pinterest/1.5.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA801121 = {S20704, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Dot%20Lock/2.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA811153 = {S20705, 40, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Fera/5.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA832830 = {S20706, 52, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Dual%20Browser/1.4.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA836759 = {S20707, 45, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - TheFeed/1.0.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA858487 = {S20708, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ForumRunner/41 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA869275 = {S20709, 42, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - iBuilder/1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA872053 = {S20710, 50, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - SexyShelves/9.02.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA891318 = {S20711, 51, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Love%20Pictures/3.6 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA892010 = {S20712, 47, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ImageSearch/1.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA892173 = {S20713, 48, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - iCandyWP1/9.15.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA893371 = {S20714, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - BarCodeScan/1.1.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA895782 = {S20715, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Epic%20Fail/2.7.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA898501 = {S20716, 50, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - PimpYourPic/9.14.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA899607 = {S20717, 53, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Downloader%20copy/1.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA901089 = {S20718, 55, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ImageSearcherProS/2.1.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA901095 = {S20719, 55, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ImageSearcherFree/1.0.5 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA901999 = {S20720, 54, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ImageSearcherPro/2.1.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA902079 = {S20721, 42, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Images/1.4 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA903596 = {S20722, 51, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ImageSearcher/3.1.3 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA903844 = {S20723, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Mercury3/6.0.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA904139 = {S20724, 56, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ImageSearcherFreeS/1.0.5 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA905635 = {S20725, 50, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Mercury3Free/6.0.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA907762 = {S20726, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - 360%20Browser/5.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA912382 = {S20727, 52, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - CFKnottsBerryFarm/61 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA912395 = {S20728, 53, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - CFCanadaWonderland/48 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA912429 = {S20729, 48, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - CFWorldsOfFun/56 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA913172 = {S20730, 45, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - browser/31759 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA1937220 = {S20731, 51, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - InstapaperPro/4.0.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA1949655 = {S20732, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - RedLaser/3.3.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA1953017 = {S20733, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - NetNewsWire/1.0.5 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA1955875 = {S20734, 42, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Reader/2.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA1958144 = {S20735, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - iSocialize2/31 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA1958371 = {S20736, 47, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - FastBrowser/1.7 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA1958671 = {S20737, 44, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - NewsMe/1.2.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA1971726 = {S20738, 43, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - xFeed/2.0.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA1978124 = {S20739, 57, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - VideoDownloaderLite/2.0.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA1984138 = {S20740, 44, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - LexPrint/3.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA1988638 = {S20741, 50, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - QihooBrowser/2.5.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA1988976 = {S20742, 48, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Tweetlogix/1.9.5 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA1990950 = {S20743, 44, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - MyMedia/2.41 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA1991053 = {S20744, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Readability/1.1.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA1991061 = {S20745, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - 360%20Lite/5.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA1991250 = {S20746, 50, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - LastPass%20Tab/2.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA1992113 = {S20747, 41, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - JDate/101 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA1994178 = {S20748, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Corus/3.0.3832 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA1995138 = {S20749, 45, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Browser/1.2.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA1996299 = {S20750, 48, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Dolphin%20HD/5.3 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA1997002 = {S20751, 45, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - PhotoFind/5.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA1997051 = {S20752, 50, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Pulse%20News/2.9.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2005276 = {S20753, 43, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Terra/2.0.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2007534 = {S20754, 57, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Evanov-CidcRadio/2.0.5369 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2009060 = {S20755, 43, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - MyPad/2.6.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2153675 = {S20756, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - MobileLearn/3.0.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2156652 = {S20757, 53, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - $%7BPRODUCT_NAME%7D/1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2157862 = {S20758, 41, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - FML/2.3.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2159186 = {S20759, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Reddit%20Pics/1.6 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2160573 = {S20760, 50, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Demotivation/2.5.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2160789 = {S20761, 58, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - MyPad%20for%20iPhone/2.6.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2161973 = {S20762, 48, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - FindMyiPhone/1.4 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2162668 = {S20763, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ImageProHD/1.9 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2163256 = {S20764, 50, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ImageProHDFree/1.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2163551 = {S20765, 50, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Silly%20Pics/1.1.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2163625 = {S20766, 41, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Cydia/0.9 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2164124 = {S20767, 53, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - SpeakText%20FREE/12.4 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2164520 = {S20768, 45, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Scribbler/1.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2165464 = {S20769, 39, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Scan/38 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2166758 = {S20770, 40, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Azul/3.6 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2167535 = {S20771, 53, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - iAnnotate%20PDF/2.2.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2168023 = {S20772, 60, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - myPDFReader_old_design/1.0.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2169799 = {S20773, 42, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ePrint/4.8 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2172966 = {S20774, 57, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Photo%20Gallery%20Pro/2.6 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2175685 = {S20775, 48, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - KaywaReader/1.21 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2186854 = {S20776, 51, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Download%20Free/2.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2189543 = {S20777, 52, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Extras4iMovie/9.30.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2202840 = {S20778, 44, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Digg/X.3.0.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2203475 = {S20779, 51, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - AtomicBrowser/7.0.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2205545 = {S20780, 51, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - MyPad_Premium/2.6.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2205637 = {S20781, 47, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - 123people/3.1.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2205682 = {S20782, 43, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Qrafter/5.6 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2221456 = {S20783, 47, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - match.com/2.1.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2222365 = {S20784, 52, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - FanballNews/1.1.2316 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2223442 = {S20785, 44, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - BejBlitz/588 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2223748 = {S20786, 44, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Download/2.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2223786 = {S20787, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Slotomania/1.12.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2233898 = {S20788, 50, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - iDownloaderPro/2.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2234887 = {S20789, 50, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - NFL_US_Premium/247 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2235141 = {S20790, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ImageHunter/2.4.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2235813 = {S20791, 48, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - DiigoBrowser/4.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2237433 = {S20792, 53, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Pic%20Collage/2.21.14 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2238347 = {S20793, 51, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Qriket%20Game/3.1.5 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2242244 = {S20794, 53, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - AngryBirdsFree/1.2.1 CFNetwork/485.13.9 Darwin/11.0.0*/
const Device UA2242458 = {S20795, 40, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Maps/1.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2260345 = {S20796, 41, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Netflix/X CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2260409 = {S20797, 51, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - MusicDownloader/2.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2262002 = {S20798, 57, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - WordsWithFriendsFree/4.13 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2262007 = {S20799, 53, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Perfect%20Browser/8.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2262120 = {S20800, 42, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ScoopIt/55 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2262176 = {S20801, 47, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - SmartSync/4.5.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2262894 = {S20802, 50, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - WEB%20to%20PDF/2.3 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2306025 = {S20803, 48, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - SuperProber/3.0 CFNetwork/485.13.9 Darwin/11.0.0*/
const Device UA2313786 = {S20804, 48, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Job%20Hopper/1.3 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2316208 = {S20805, 47, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - iCarousel/1.0.5 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2316493 = {S20806, 44, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - FreeCell/2.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2319251 = {S20807, 55, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Fun%20Animations/9.29.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2323973 = {S20808, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - GoodReader/3.14.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2324358 = {S20809, 45, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ESCPR01/2.2.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2325090 = {S20810, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Wallpapers/1.6 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2331326 = {S20811, 55, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Million%20Photos/8.60.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2333975 = {S20812, 43, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Weather/1.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2337519 = {S20813, 62, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - CommCenterMobileHelper/1285.21 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2338369 = {S20814, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - The_Boys/4.1.6 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2339476 = {S20815, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Downloader/1.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2340037 = {S20816, 44, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Amazon/2.0.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2342936 = {S20817, 44, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - AppCake3/3.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2348493 = {S20818, 50, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - PDF%20PROvider/2.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2348788 = {S20819, 48, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Y_Messenger/5467 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2349037 = {S20820, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - SliceMobile/2.0.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2353963 = {S20821, 57, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - MusicDownloaderLite/2.2.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2356443 = {S20822, 50, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - FeeddlerPro/1.12.5 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2359352 = {S20823, 50, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Google/2.5.1.13455 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2359979 = {S20824, 47, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - AtBat.Full/1497 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2364431 = {S20825, 52, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Action%20Movie/1.2.7 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2367072 = {S20826, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Cooliris/68643 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2367649 = {S20827, 47, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - CEBrowser/1.6.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2369861 = {S20828, 57, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - adlibd (unknown version) CFNetwork/485.13.9 Darwin/11.0.0*/
const Device UA2372042 = {S20829, 45, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - GroupMe/3.3.4 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2379184 = {S20830, 42, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Apsley/2.3 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2381964 = {S20831, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - %20iSearch/1.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2383414 = {S20832, 51, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - PDF%20Printer/2.0.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2386228 = {S20833, 44, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - vBrowse/2.04 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2389475 = {S20834, 48, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - AtomicLite/7.0.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2403292 = {S20835, 53, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - GoodReaderIPad/3.18.6 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2403458 = {S20836, 42, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Keeper/5.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2410225 = {S20837, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Textfree/5.0.5 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2410363 = {S20838, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - FindMyFriends/1.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2411407 = {S20839, 47, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ScoreCenter/322 CFNetwork/548.0.4 Darwin/11.0.0*/
const Device UA2411835 = {S20840, 63, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - syncdefaultsd (unknown version) CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2411948 = {S20841, 45, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - iReddit/2.1.5 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2422325 = {S20842, 61, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Free%20Video%20Downloader/1.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2426024 = {S20843, 48, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Instapaper/4.2.7 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2428117 = {S20844, 42, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - TEE2/2.3.4 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2441898 = {S20845, 45, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - textPlus/6698 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2447659 = {S20846, 50, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ZillowMap/5.3.0.86 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2455585 = {S20847, 48, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - FindMyiPhone/1.4 CFNetwork/548.0.4 Darwin/11.0.0*/
const Device UA2456906 = {S20848, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Apolloplus/2.7.10 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2460286 = {S20849, 41, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - MSN/1.1.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2463093 = {S20850, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - BarcodeFree/1.0.7 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2463325 = {S20851, 59, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - securityd (unknown version) CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2463390 = {S20852, 45, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Crackle/2.4.8 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2463464 = {S20853, 62, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - itunesstored (unknown version) CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2463565 = {S20854, 47, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - iDownloader/2.5 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2463566 = {S20855, 43, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Chess/2.4.5 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2466124 = {S20856, 48, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Mega%20Run/1.2.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2473924 = {S20857, 40, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - iPhone/X CFNetwork/548.0.4 Darwin/11.0.0*/
const Device UA2474960 = {S20858, 44, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Installous/1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2476368 = {S20859, 50, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ChasingYello/1.0.4 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2476464 = {S20860, 45, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - SportsTap/305 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2480886 = {S20861, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Funny%20Pics/2.93 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2481802 = {S20862, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ysearch2/3.0.3 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2486641 = {S20863, 43, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Searchx/2.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2487391 = {S20864, 54, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Hot%20Wallpapers/10.02 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2496000 = {S20865, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - FastFood/4.0.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2502707 = {S20866, 59, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - BubbleBreakerPro%20Free/1.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2518244 = {S20867, 60, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ScrambleWithFriendsFree/4.15 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2518688 = {S20868, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - TwitterFon/583 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2519059 = {S20869, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - SolitaireFree/3.6 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2519633 = {S20870, 52, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Solitaire%20Free/1.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2544577 = {S20871, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - midomi-free/5.0.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2551524 = {S20872, 47, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Prismatic/1.0.4 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2556561 = {S20873, 63, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ApolloVideoDownloaderFree/1.2.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2574551 = {S20874, 50, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - urbanspin/2.1.0.17 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2578117 = {S20875, 41, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - iPick/200 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2580183 = {S20876, 47, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ImageMobile/1.8 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2584892 = {S20877, 39, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - tfctv/1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2588373 = {S20878, 41, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Homes/1.5 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2596384 = {S20879, 64, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - HD%20&%20Retina%20Wallpapers/1.6 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2640624 = {S20880, 56, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - MONOPOLY%20Hotels/1.0.28 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2640862 = {S20881, 54, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - PicFrame_ver_1.0.0/1.3 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2641514 = {S20882, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - NYTimes/9886.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2642046 = {S20883, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - LineRunner/1.9 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2642977 = {S20884, 42, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - eBay/2.1.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2644055 = {S20885, 60, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ScrambleWithFriendsPaid/4.17 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2644675 = {S20886, 52, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - UberSocial/1.0.0.251 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2652902 = {S20887, 53, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - VideoDownloader/2.2.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2659019 = {S20888, 44, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - playable/3.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2663667 = {S20889, 44, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ImagePro/2.6 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2668654 = {S20890, 42, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Newsify/87 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2670702 = {S20891, 45, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - BaiduHD/2.0.3 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2671909 = {S20892, 52, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - CLIPish%20Pro2/10.11 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2683926 = {S20893, 47, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - AlienBlue/2.6.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2684313 = {S20894, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Pandora/3.1.17 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2684887 = {S20895, 38, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Us/1.3 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2685058 = {S20896, 47, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - HootSuite/2.4.7 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2686203 = {S20897, 53, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - EditorsChoice/10451.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2686408 = {S20898, 60, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - MatchingWithFriendsFree/4.12 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2691410 = {S20899, 44, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - iGmail/9.6.3 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2693116 = {S20900, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Qrafter%20Pro/5.6 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2697951 = {S20901, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Holidays/10.01 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2721573 = {S20902, 45, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ooVoo/1.1.2.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2730974 = {S20903, 58, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - SecureBrowser/3.6.0%20Beta CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2791217 = {S20904, 53, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - iDownloaderLite/1.0.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2809074 = {S20905, 43, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Groupon/953 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2809134 = {S20906, 54, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - TubeDownloaderLite/2.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2811290 = {S20907, 43, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - AdSheet/1.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2813208 = {S20908, 51, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - OregonSettler/1.0.7 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2816535 = {S20909, 61, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ApolloGeneralDownloader/1.1.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2817784 = {S20910, 51, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - BarcodeQRFree/1.0.5 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2835011 = {S20911, 52, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - TubeDownloader/2.2.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2837309 = {S20912, 50, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Downloader-Pro/2.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2842285 = {S20913, 48, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - DreamLeague/1.19 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2843485 = {S20914, 62, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Cool%20Retina%20Wallpapers/1.5 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2844163 = {S20915, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - AirWatch/4.2.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2847350 = {S20916, 51, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - ImageDownloader/1.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2850603 = {S20917, 52, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Mobile%20Learn/3.1.4 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2852695 = {S20918, 58, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - MusicDownloaderBSOFree/1.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2859520 = {S20919, 55, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - VideoDLPlusPlusFree/1.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2863644 = {S20920, 60, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - NortonSafeBrowser/168.2.5.14 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2888953 = {S20921, 43, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - MSNBC/2.6.3 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2889070 = {S20922, 48, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - HuffPost/2.9.2.7 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2933528 = {S20923, 47, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - KakaoTalk/3.0.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2937122 = {S20924, 48, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - wewsiphone/11710 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2941466 = {S20925, 41, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - iFunny/22 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2950081 = {S20926, 40, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Flow/1.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2950316 = {S20927, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - mallstreet/627 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2952028 = {S20928, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - VideoDLFree/1.1.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2958423 = {S20929, 52, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Talking%20Ginger/1.4 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2964930 = {S20930, 51, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - MxBrowser-iPhone/14 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2964932 = {S20931, 82, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - %E5%BF%AB%E6%92%AD%E5%A2%9E%E5%BC%BA%E7%89%88/1977 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2980053 = {S20932, 45, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Twitter/4.1.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2980505 = {S20933, 54, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - HuffingtonPost/2.8.5.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2985893 = {S20934, 60, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - MusicDownloaderSSAFree/1.0.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2993585 = {S20935, 51, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - UnlimitedSMS/4.4.23 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA3020952 = {S20936, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - AlienBlueHD/2.2.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA3028593 = {S20937, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - LinkedIn/6.1.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA3041614 = {S20938, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - RageComics/3.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA3042967 = {S20939, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Smurfs-IPad/1.2.5 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA3045537 = {S20940, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - SSC%20Forums/X CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA3056951 = {S20941, 43, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Florida/110 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA3058568 = {S20942, 51, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - MxBrowser-iPhone/17 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA3077511 = {S20943, 91, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - %E5%A4%9A%E5%B1%8F%E4%BA%92%E5%8A%A8%E6%B5%8F%E8%A7%88/1976 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA3088730 = {S20944, 47, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - logosQuiz/1.5.6 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA3088920 = {S20945, 46, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Washington/208 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA3090506 = {S20946, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Pennsylvania/1153 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA3098036 = {S20947, 47, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Instagram/2.4.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA3098422 = {S20948, 51, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - subwaysurfers/1.2.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA3102457 = {S20949, 43, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Georgia/376 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA3103317 = {S20950, 48, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - LetsGolf3/1.1.0 CFNetwork/485.13.9 Darwin/11.0.0*/
const Device UA3106587 = {S20951, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - NorthCarolina/296 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA3118028 = {S20952, 47, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Differences/1.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA3121632 = {S20953, 51, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - MxBrowser-iPhone/13 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA3128608 = {S20954, 49, { &P15364,&P18570,&P17470,&P18092 }};	/*Useragent - Coast/1.0.1.62563 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA464929 = {S20955, 81, { &P15364,&P7133,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; U; ru; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.60*/
const Device UA479434 = {S20956, 81, { &P15364,&P7133,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; U; ru; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.50*/
const Device UA599677 = {S20957, 107, { &P15364,&P7133,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; pl; rv:1.9.1) Gecko/X Firefox/3.5 (.NET CLR 3.5.30729)	Firefox 3.5*/
const Device UA739610 = {S20958, 91, { &P15364,&P7133,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; U; YB/3.5.1; ru; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.63*/
const Device UA780071 = {S20959, 84, { &P15364,&P7133,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; U; en-GB; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.51*/
const Device UA864219 = {S20960, 81, { &P15364,&P7133,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; U; en; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 11.10*/
const Device UA875505 = {S20961, 81, { &P15364,&P7133,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; U; ru; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.70*/
const Device UA896464 = {S20962, 81, { &P15364,&P7133,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; U; ru; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.52*/
const Device UA899281 = {S20963, 81, { &P15364,&P7133,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; U; ru; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.53*/
const Device UA1980351 = {S20964, 78, { &P15364,&P7133,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 /Windows; U; Windows NT 6.1; de; rv:1.9.1.5) Gecko/X Firefox/3.5.5*/
const Device UA2166594 = {S20965, 72, { &P15364,&P7133,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; U; zh-cn; rv:1.9.1.6) Gecko/X Firefox/3.5.6*/
const Device UA2206894 = {S20966, 81, { &P15364,&P7133,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en_US; rv:1.9.1.9) Gecko/X Firefox/3.5.9*/
const Device UA3114129 = {S20967, 125, { &P15364,&P16110,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Maxthon/4.2.0.800 Chrome/30.0.1551.0 Safari/537.36*/
const Device UA3115648 = {S20968, 101, { &P15364,&P16110,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3115676 = {S20969, 108, { &P15364,&P16110,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3129240 = {S20970, 118, { &P15364,&P16110,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36,gzip(gfe)*/
const Device UA3129996 = {S20971, 118, { &P15364,&P16110,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Maxthon/4.2.0.800 Chrome/30.0.1551.0 Safari/537.36*/
const Device UA32708 = {S20972, 118, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U;MSIE9.0; Windows NT 9.0; it-IT) AppleWebKit/525 (KHTML, like Gecko) BrowserNG/7.2.6.9 3gpp-gba*/
const Device UA92229 = {S20973, 55, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/3.0 (compatible; WebCapture 2.0; Auto; Windows)*/
const Device UA94197 = {S20974, 27, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/0.91 Beta (Windows)*/
const Device UA98177 = {S20975, 53, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Lotus-Notes/6.0; Windows-NT)*/
const Device UA98606 = {S20976, 33, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Windows-Media-Player/9.00.00.4503*/
const Device UA99071 = {S20977, 35, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Windows-Media-Player/11.0.5721.5268*/
const Device UA102232 = {S20978, 34, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Windows-Media-Player/10.00.00.xxxx*/
const Device UA104753 = {S20979, 53, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Lotus-Notes/5.0; Windows-NT)*/
const Device UA123060 = {S20980, 27, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - LinkExaminer/1.01 (Windows)*/
const Device UA137559 = {S20981, 49, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; WebCapture 3.0; Windows)*/
const Device UA145107 = {S20982, 35, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Windows-Media-Player/11.0.5721.5280*/
const Device UA147276 = {S20983, 34, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Windows-Media-Player/10.00.00.4081*/
const Device UA147474 = {S20984, 36, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Windows-Media-Player/12.0.7600.16667*/
const Device UA149062 = {S20985, 36, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Windows-Media-Player/11.0.6002.18311*/
const Device UA149450 = {S20986, 33, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Windows-Media-Player/9.00.00.4510*/
const Device UA152092 = {S20987, 60, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - FeedDemon/4.0 (http://www.feeddemon.com/; Microsoft Windows)*/
const Device UA159202 = {S20988, 53, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE5.0; Windows NT; DigExt)*/
const Device UA164170 = {S20989, 51, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Check&Get 3.4; Windows NT)*/
const Device UA187680 = {S20990, 82, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.6*/
const Device UA195396 = {S20991, 82, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.7*/
const Device UA222600 = {S20992, 84, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.7.1*/
const Device UA245248 = {S20993, 85, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.0*/
const Device UA264605 = {S20994, 85, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/526.9+ (KHTML, like Gecko) AdobeAIR/1.5.3*/
const Device UA289999 = {S20995, 84, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.5.1*/
const Device UA362802 = {S20996, 46, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.1; Windows NT)*/
const Device UA382956 = {S20997, 84, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.0.2*/
const Device UA391342 = {S20998, 68, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - curl/7.19.2 (i386-pc-win32) libcurl/7.19.2 OpenSSL/0.9.8i zlib/1.2.3*/
const Device UA431020 = {S20999, 84, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.0.3*/
const Device UA484213 = {S21000, 82, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.5*/
const Device UA511680 = {S21001, 69, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - curl/7.21.0 (amd64-pc-win32) libcurl/7.21.0 OpenSSL/0.9.8o zlib/1.2.3*/
const Device UA525615 = {S21002, 60, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - FeedDemon/3.0 (http://www.feeddemon.com/; Microsoft Windows)*/
const Device UA535340 = {S21003, 69, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - curl/7.21.6 (amd64-pc-win32) libcurl/7.21.6 OpenSSL/0.9.8r zlib/1.2.5*/
const Device UA545987 = {S21004, 107, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/523+ (KHTML, like Gecko) AdobeAIR/1.5 Community Help Client 1.0*/
const Device UA576210 = {S21005, 85, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.1*/
const Device UA629900 = {S21006, 85, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.2*/
const Device UA715686 = {S21007, 45, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - curl/7.10.4 (win32) libcurl/7.10.4 zlib/1.1.4*/
const Device UA729928 = {S21008, 53, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - curl/7.22.0 (i386-pc-win32) libcurl/7.22.0 zlib/1.2.5*/
const Device UA758886 = {S21009, 31, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0(Windows; U) Gecko/X*/
const Device UA761563 = {S21010, 83, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/526.9+ (KHTML, like Gecko) AdobeAIR/1.5*/
const Device UA768060 = {S21011, 53, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - curl/7.21.1 (i386-pc-win32) libcurl/7.21.1 zlib/1.2.5*/
const Device UA776779 = {S21012, 94, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.2 X/Yammer*/
const Device UA820634 = {S21013, 84, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.0.4*/
const Device UA834328 = {S21014, 82, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/531.9 (KHTML, like Gecko) AdobeAIR/2.0*/
const Device UA854134 = {S21015, 85, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.3*/
const Device UA887325 = {S21016, 60, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - FeedDemon/4.1 (http://www.feeddemon.com/; Microsoft Windows)*/
const Device UA923808 = {S21017, 49, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - curl/7.10.8 (win32) libcurl/7.10.8 OpenSSL/0.9.7c*/
const Device UA1851002 = {S21018, 81, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/523+ (KHTML, like Gecko) AdobeAIR/1.1*/
const Device UA1884772 = {S21019, 85, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/526.9+ (KHTML, like Gecko) AdobeAIR/1.5.2*/
const Device UA1902189 = {S21020, 53, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - curl/7.19.7 (i386-pc-win32) libcurl/7.19.7 zlib/1.2.3*/
const Device UA1949138 = {S21021, 70, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - curl/7.23.1 (x86_64-pc-win32) libcurl/7.23.1 OpenSSL/0.9.8r zlib/1.2.5*/
const Device UA1996990 = {S21022, 85, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.4*/
const Device UA2268406 = {S21023, 85, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/526.9+ (KHTML, like Gecko) AdobeAIR/1.5.1*/
const Device UA2308484 = {S21024, 44, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - curl/7.23.1 (x86_64-pc-win32) libcurl/7.23.1*/
const Device UA2355155 = {S21025, 85, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.5*/
const Device UA2502696 = {S21026, 115, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.4 Zoodles/3.0.5 Safari/525.27.1*/
const Device UA2506262 = {S21027, 85, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.6*/
const Device UA2517005 = {S21028, 57, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - curl/7.12.1 (i386-pc-win32) libcurl/7.12.1 OpenSSL/0.9.7d*/
const Device UA2588144 = {S21029, 68, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U;) AppleWebKit/532.0 (KHTML, like Gecko) Iron*/
const Device UA2668110 = {S21030, 85, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.7*/
const Device UA2871486 = {S21031, 60, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - FeedDemon/4.5 (http://www.feeddemon.com/; Microsoft Windows)*/
const Device UA2956978 = {S21032, 85, { &P15364,&P17350,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.8*/
const Device UA2029858 = {S21033, 139, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7000 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032474 = {S21034, 139, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7000 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257949 = {S21035, 139, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257991 = {S21036, 139, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7000 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258008 = {S21037, 139, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7000 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2362535 = {S21038, 61, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N7000 Build/JZO54K)*/
const Device UA2385422 = {S21039, 61, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-N7000 Build/JRO03C)*/
const Device UA2435740 = {S21040, 133, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHW-M480W Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2459363 = {S21041, 133, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHW-M480S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2505611 = {S21042, 133, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHW-M480K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2528514 = {S21043, 140, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E160L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2530196 = {S21044, 140, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E160K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2531049 = {S21045, 140, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E160S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2577895 = {S21046, 130, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7000 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2595618 = {S21047, 137, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-N7000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2664085 = {S21048, 62, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E160L Build/JZO54K)*/
const Device UA2666236 = {S21049, 62, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E160S Build/JZO54K)*/
const Device UA2670056 = {S21050, 147, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E160K/KKJMC3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2680845 = {S21051, 158, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-N7000/N7000XXLSZ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2702936 = {S21052, 62, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E160K Build/JZO54K)*/
const Device UA2762991 = {S21053, 140, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-N7000B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2767839 = {S21054, 62, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N7000B Build/JZO54K)*/
const Device UA2885307 = {S21055, 158, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; SAMSUNG GT-N7000/N7000XXLT5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958148 = {S21056, 139, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-i889 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2974412 = {S21057, 158, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-N7000/N7000XXLT9 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984891 = {S21058, 139, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985199 = {S21059, 137, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SC-05D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3015178 = {S21060, 157, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000-ORANGE/N7000XXLT9 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044654 = {S21061, 62, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHW-M480W Build/JZO54K)*/
const Device UA3049727 = {S21062, 157, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000-ORANGE/N7000XXLSZ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3062694 = {S21063, 147, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E160K/KKJMH1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3091998 = {S21064, 140, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N7000 Build/VeTW3.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3096070 = {S21065, 158, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-N7000/N7000XXLTA Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3100000 = {S21066, 59, { &P16356,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SC-05D Build/JZO54K)*/
const Device UA370797 = {S21067, 143, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-S6500T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA675784 = {S21068, 161, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-S6500/S6500BULC5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA763414 = {S21069, 142, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; GT-S6500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA785669 = {S21070, 143, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6500D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA885362 = {S21071, 67, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S6500D Build/GINGERBREAD)*/
const Device UA904042 = {S21072, 161, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S6500/S6500BVLD2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA910211 = {S21073, 160, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6500-ORANGE/S6500BVLD2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1981402 = {S21074, 143, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-S6500L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1994024 = {S21075, 163, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hr-hr; SAMSUNG GT-S6500D/S6500DXXLD3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1994337 = {S21076, 160, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; GT-S6500-ORANGE/S6500BVLC5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995316 = {S21077, 160, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6500-ORANGE/S6500BVLE3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1996244 = {S21078, 161, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S6500/S6500BVLE3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027873 = {S21079, 161, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pl-pl; SAMSUNG GT-S6500/S6500XXLD1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028996 = {S21080, 66, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S6500 Build/GINGERBREAD)*/
const Device UA2029444 = {S21081, 163, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-S6500D/S6500DXXLE2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029503 = {S21082, 161, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-at; SAMSUNG GT-S6500/S6500XXLE1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031706 = {S21083, 163, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; SAMSUNG GT-S6500D/S6500DXXLC4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2180614 = {S21084, 161, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S6500/S6500BVLG2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2202844 = {S21085, 160, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6500-ORANGE/S6500BVLG2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2238800 = {S21086, 161, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-S6500/S6500BULE3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257142 = {S21087, 163, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hr-hr; SAMSUNG GT-S6500D/S6500DXXLH2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2384387 = {S21088, 141, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S6500D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2393208 = {S21089, 120, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; it-it; GT-S6500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2456597 = {S21090, 161, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-at; SAMSUNG GT-S6500/S6500XXLH2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2509692 = {S21091, 161, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-S6500/S6500BULK2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2534747 = {S21092, 163, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hr-hr; SAMSUNG GT-S6500D/S6500DXXLK2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2547137 = {S21093, 67, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S6500L Build/GINGERBREAD)*/
const Device UA2561466 = {S21094, 160, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-S6500-ORANGE/S6500BVLK2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2569312 = {S21095, 161, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; cs-cz; SAMSUNG GT-S6500/S6500XXLK2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2692289 = {S21096, 67, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S6500T Build/GINGERBREAD)*/
const Device UA2751235 = {S21097, 161, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S6500/S6500BVLK2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2818064 = {S21098, 136, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6500D Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2855989 = {S21099, 120, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; es-es; GT-S6500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2870206 = {S21100, 161, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-S6500/S6500BUMD1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2889150 = {S21101, 120, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; el-gr; GT-S6500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2980372 = {S21102, 120, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; hu-hu; GT-S6500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2980804 = {S21103, 120, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; cs-cz; GT-S6500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2981972 = {S21104, 141, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S6500T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3056577 = {S21105, 163, { &P18213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hr-hr; SAMSUNG GT-S6500D/S6500DXXMF1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA92395 = {S21106, 35, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.11 (Windows NT 5.1; U) [en]*/
const Device UA92397 = {S21107, 66, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.54 [en]*/
const Device UA92752 = {S21108, 36, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.54 (Windows NT 5.1; U)  [pl]*/
const Device UA92999 = {S21109, 76, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.1) Opera 7.01 [en]*/
const Device UA99208 = {S21110, 62, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.60 (Windows NT 5.2; U)  [en] (IBM EVV/3.0/EAK01AG9/LE)*/
const Device UA137732 = {S21111, 65, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 7.60*/
const Device UA153869 = {S21112, 34, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.x (Windows NT 5.1; U) [en]*/
const Device UA178436 = {S21113, 26, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.50 (Windows XP; U)*/
const Device UA240570 = {S21114, 35, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.50 (Windows NT 5.1; U) [en]*/
const Device UA259531 = {S21115, 35, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.51 (Windows NT 5.1; U) [en]*/
const Device UA304963 = {S21116, 76, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.1) Opera 7.02 [en]*/
const Device UA335604 = {S21117, 35, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.22 (Windows NT 5.1; U) [en]*/
const Device UA339040 = {S21118, 35, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.10 (Windows NT 5.1; U) [en]*/
const Device UA531362 = {S21119, 35, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.20 (Windows NT 5.1; U) [en]*/
const Device UA599362 = {S21120, 66, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.20 [de]*/
const Device UA755894 = {S21121, 66, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.10 [en]*/
const Device UA814329 = {S21122, 66, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.51 [en]*/
const Device UA820517 = {S21123, 71, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows XP) Opera 7.0 [en]*/
const Device UA887989 = {S21124, 76, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.1) Opera 7.04 [de]*/
const Device UA916620 = {S21125, 35, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.02 (Windows NT 5.1; U) [fr]*/
const Device UA1879157 = {S21126, 47, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.20 (Windows NT 5.1; U; pl) Version/10.7*/
const Device UA1884194 = {S21127, 47, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.40 (Windows NT 5.1; U; pl) Version/10.7*/
const Device UA1888513 = {S21128, 62, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.20 (Windows NT 5.1; U; pl) Presto/2.2.15 Version/10.10*/
const Device UA1891037 = {S21129, 47, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.30 (Windows NT 5.1; U; pl) Version/10.7*/
const Device UA1902905 = {S21130, 47, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.10 (Windows NT 5.1; U; pl) Version/10.7*/
const Device UA2132169 = {S21131, 62, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.30 (Windows NT 5.1; U; pl) Presto/2.3.15 Version/10.10*/
const Device UA2208230 = {S21132, 66, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.23 [ru]*/
const Device UA2215057 = {S21133, 62, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.40 (Windows NT 5.1; U; pl) Presto/2.4.15 Version/10.10*/
const Device UA2216367 = {S21134, 62, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.10 (Windows NT 5.1; U; pl) Presto/2.1.15 Version/10.10*/
const Device UA2271411 = {S21135, 66, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.52 [en]*/
const Device UA2275719 = {S21136, 35, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.03 (Windows NT 5.1; U) [en]*/
const Device UA2295396 = {S21137, 66, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.11 [en]*/
const Device UA2408689 = {S21138, 36, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.23 (Windows NT 5.1; U)  [en]*/
const Device UA2472172 = {S21139, 76, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.1) Opera 7.03 [ru]*/
const Device UA2483034 = {S21140, 46, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera 7.5 (Win XP): Opera/7.50 (Windows XP; U)*/
const Device UA2502465 = {S21141, 106, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - ('Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.1) Opera 7.02 [en]', 'Opera 7.02, Windows XP')*/
const Device UA2505316 = {S21142, 51, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera 7.51 (Win XP): Opera/7.51 (Windows NT 5.1; U)*/
const Device UA2681101 = {S21143, 92, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.60 [en] (IBM EVV/3.0/EAK01AG9/LE)*/
const Device UA2681625 = {S21144, 66, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.53 [ru]*/
const Device UA2683464 = {S21145, 66, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.21 [ru]*/
const Device UA2685292 = {S21146, 35, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.52 (Windows NT 5.1; U) [ru]*/
const Device UA2686740 = {S21147, 47, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Mozilla/3.0 (Windows NT 5.1; U) Opera 7.23 [ru]*/
const Device UA2806003 = {S21148, 35, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.0 (Windows NT 5.1; U; [en])*/
const Device UA2932911 = {S21149, 47, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U) Opera 7.54 [ru]*/
const Device UA2977889 = {S21150, 35, { &P15364,&P5690,&P4810,&P18092 }};	/*Useragent - Opera/7.01 (Windows NT 5.1; U) [ru]*/
const Device UA2407483 = {S21151, 125, { &P15364,&P7133,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA2407485 = {S21152, 118, { &P15364,&P7133,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA2460072 = {S21153, 128, { &P15364,&P7133,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1,gzip(gfe)*/
const Device UA2460397 = {S21154, 135, { &P15364,&P7133,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1,gzip(gfe)*/
const Device UA2603276 = {S21155, 99, { &P15364,&P7133,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.4.1000  Safari/537.1*/
const Device UA2713488 = {S21156, 118, { &P15364,&P7133,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1*/
const Device UA2714031 = {S21157, 125, { &P15364,&P7133,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1*/
const Device UA2754024 = {S21158, 128, { &P15364,&P7133,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1,gzip(gfe)*/
const Device UA2770383 = {S21159, 135, { &P15364,&P7133,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1,gzip(gfe)*/
const Device UA2956399 = {S21160, 124, { &P15364,&P7133,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA23661 = {S21161, 103, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA180921 = {S21162, 110, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.21 (KHTML, like Gecko) Chrome/11.0.678.0 Safari/534.21*/
const Device UA186744 = {S21163, 144, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Jolicloud Linux i686) AppleWebKit/534.24 (KHTML, like Gecko) Joli OS/1.2 Chromium/11.0.696.14 Chrome/11.0.696.14 Safari/534.24*/
const Device UA434389 = {S21164, 100, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.23 (KHTML, like Gecko) Chrome/11.0.686.3 Safari/534.23*/
const Device UA544266 = {S21165, 110, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24*/
const Device UA624210 = {S21166, 120, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko; Google Web Preview) Chrome/11.0.696 Safari/534.24*/
const Device UA664893 = {S21167, 106, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux; en-us) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA757718 = {S21168, 110, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; en_us) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA778911 = {S21169, 115, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 Slackware/13.37 (X11; U; Linux x86_64; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/11.0.696.50*/
const Device UA805231 = {S21170, 119, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US) AppleWebKit/534.18 (KHTML, like Gecko) Chrome/11.0.660.0 Safari/534.18*/
const Device UA858708 = {S21171, 130, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux x86_64;en-us;I10a Build/IML74K) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA891540 = {S21172, 134, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; en-us; TB782B Build/Ice Cream) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA895145 = {S21173, 140, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24,gzip(gfe) (via translate.google.com)*/
const Device UA895554 = {S21174, 117, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; Z520m; en-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA900920 = {S21175, 117, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; T320a; en-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA1960306 = {S21176, 117, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; T320a; fr-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA1961101 = {S21177, 117, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; Z520m; fr-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA1997111 = {S21178, 117, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; A320a; en-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA1997834 = {S21179, 117, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; A320a; fr-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2163474 = {S21180, 110, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; en_gb) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2168627 = {S21181, 130, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux x86_64;en-us;GA20 Build/IML74K) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2168628 = {S21182, 132, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux x86_64;en-us;I10-12 Build/IML74K) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2179635 = {S21183, 129, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux x86_64;en-us;I7a Build/IML74K) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2190911 = {S21184, 134, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; en-us; TB1082 Build/Ice Cream) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2234459 = {S21185, 113, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24,gzip(gfe)*/
const Device UA2258151 = {S21186, 129, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux x86_64;en-us;i7c Build/IML74K) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2259958 = {S21187, 103, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Safari/534.24*/
const Device UA2260215 = {S21188, 119, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; en-us; CMP741E) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2275929 = {S21189, 103, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686) AppleWebKit/534.22 (KHTML, like Gecko) Chrome/11.0.683.0 Safari/534.22*/
const Device UA2339616 = {S21190, 103, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (i7a; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2346416 = {S21191, 112, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; LT26i-o) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2356158 = {S21192, 91, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; ) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2360335 = {S21193, 117, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; A320a; zh-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2413758 = {S21194, 117, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; Z520m; es-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2415083 = {S21195, 117, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; A320a; es-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2434869 = {S21196, 115, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Tablet-PC-4 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2457673 = {S21197, 117, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; X525a; en-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2477379 = {S21198, 146, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24; 360browser(securitypay,securityinstalled)*/
const Device UA2485445 = {S21199, 114, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - VB100a Pro Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2486784 = {S21200, 115, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - ELEMENT10 1 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2495493 = {S21201, 117, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; Z520m; zh-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2505436 = {S21202, 119, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - OV-SteelCore(B) Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2531211 = {S21203, 113, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - ST80208-1 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2544161 = {S21204, 110, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; de_de) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2545985 = {S21205, 142, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2552636 = {S21206, 102, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11;Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2552936 = {S21207, 136, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_DesireHD_Beats_X315e; en-gb) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2556891 = {S21208, 104, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (110A; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2557563 = {S21209, 128, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Cat Tablet PHOENIX 8.1J0 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2582828 = {S21210, 112, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - TBD1083B Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2598015 = {S21211, 111, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - TBDB763 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2598177 = {S21212, 113, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - ST80216-1 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2654104 = {S21213, 110, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; vi_vn) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2674525 = {S21214, 109, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/11.0.1410.63 Safari/537.31*/
const Device UA2674667 = {S21215, 106, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/11.0.1410.63 Safari/537.31*/
const Device UA2674857 = {S21216, 106, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/11.0.1410.63 Safari/537.31*/
const Device UA2674865 = {S21217, 106, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/11.0.1410.63 Safari/537.31*/
const Device UA2675211 = {S21218, 109, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/11.0.1410.63 Safari/537.31*/
const Device UA2683611 = {S21219, 110, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; it_it) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2699294 = {S21220, 110, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - ICOU10 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2714923 = {S21221, 106, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/11.0.1410.63 Safari/537.31*/
const Device UA2717843 = {S21222, 111, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - TBDB863 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2719083 = {S21223, 109, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/11.0.1410.63 Safari/537.31*/
const Device UA2748429 = {S21224, 111, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - MID7500 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2758684 = {S21225, 110, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; es_us) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2763975 = {S21226, 117, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; X525a; fr-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2809213 = {S21227, 110, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; nl_nl) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2813018 = {S21228, 133, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux x86_64; en-us; I7a Build/IML74K) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2815273 = {S21229, 110, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; fr_fr) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2843190 = {S21230, 101, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; ) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24,gzip(gfe)*/
const Device UA2875314 = {S21231, 110, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - TBD753 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2897643 = {S21232, 111, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - TBD1083 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2934489 = {S21233, 117, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; X525a; en-us) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2956945 = {S21234, 120, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Tabloid_SW70-01A Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2966987 = {S21235, 111, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - TBDG773 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2985574 = {S21236, 115, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - g08refem618 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2998709 = {S21237, 108, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - 8JC1 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2999483 = {S21238, 111, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - DINO762 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA3058138 = {S21239, 123, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24,gzip(gfe),gzip(gfe)*/
const Device UA3071793 = {S21240, 108, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - TABX Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA3073645 = {S21241, 109, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - TM903 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA3079020 = {S21242, 124, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; ja-jp;) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24 TT/4.1.2-X.X*/
const Device UA3091621 = {S21243, 114, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - AXPAD 7I05 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA3100396 = {S21244, 111, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - NTT 611 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA3102887 = {S21245, 110, { &P15364,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; en-us) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA603679 = {S21246, 141, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; LT18i Build/4.1.A.0.562) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616665 = {S21247, 153, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SonyEricssonLT18i Build/4.1.A.0.562) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA675217 = {S21248, 63, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; LT18i Build/4.1.A.0.562)*/
const Device UA778459 = {S21249, 139, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; LT18i Build/4.1.K.0.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA902255 = {S21250, 141, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT18i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA907862 = {S21251, 139, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT18i Build/4.1.L.0.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1980540 = {S21252, 141, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; LT18a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1986024 = {S21253, 132, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT18i Build/4.1.B.0.431) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2014761 = {S21254, 141, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT18i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2024914 = {S21255, 132, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; LT18i Build/4.1.A.0.562) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2027173 = {S21256, 141, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LT18a Build/4.1.A.0.562) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029316 = {S21257, 139, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; LT18i Build/4.1.C.0.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030066 = {S21258, 153, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyEricssonLT18a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030981 = {S21259, 154, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SonyEricssonLT18iv Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033428 = {S21260, 63, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT18i Build/4.1.B.0.587)*/
const Device UA2224490 = {S21261, 141, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT18a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2235918 = {S21262, 132, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT18i Build/4.1.B.0.587) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2375501 = {S21263, 155, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-ch; SonyEricssonLT18i-o Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2529913 = {S21264, 140, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; LT18i Build/4.1.B.1.13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2826382 = {S21265, 63, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT18a Build/4.1.B.0.587)*/
const Device UA2837957 = {S21266, 156, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; LT18i Build/ThilinaC@xda_pureness_13.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2845608 = {S21267, 156, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; LT18i Build/ThilinaC@xda_pureness_14.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2935399 = {S21268, 155, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT18i Build/Xperia Ultimate HDT 3.0.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3042468 = {S21269, 139, { &P12798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-; LT18i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA503707 = {S21270, 107, { &P15364,&P17285,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.1b4) Gecko/X Firefox/3.5b4 GTB5 (.NET CLR 3.5.30729)*/
const Device UA590134 = {S21271, 123, { &P15364,&P17285,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.1.2) Gecko/X Firefox/3.5.2 (.NET CLR 3.5.30729) WinNT-PAU 24.08.2009*/
const Device UA676266 = {S21272, 120, { &P15364,&P17285,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.1.1) Gecko/X Firefox/3.5.1 (.NET CLR 3.5.30729) WinTSI 09.02.2010*/
const Device UA680822 = {S21273, 125, { &P15364,&P17285,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.1.10) Gecko/X Firefox/3.5.10 (.NET CLR 3.5.30729) WinNT-EVI 25.04.2010*/
const Device UA690122 = {S21274, 144, { &P15364,&P17285,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; rv:1.8.1) Gecko/X Firefox/3.5.0Mozilla/5.0 (Windows; U; Windows NT 6.1; rv:1.9.2) Gecko/X Firefox/3.5.0*/
const Device UA2203270 = {S21275, 69, { &P15364,&P17285,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; U; ja; rv:1.9.1.6) Gecko/X Firefox/3.5.6*/
const Device UA2803812 = {S21276, 121, { &P15364,&P17285,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-GB; rv:1.9.1.3) Gecko/X Firefox/3.5.3 ( .NET CLR 3.5.30729) WinTSI 30.10.2009*/
const Device UA2770999 = {S21277, 114, { &P15364,&P18110,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36,gzip(gfe)*/
const Device UA2774181 = {S21278, 105, { &P15364,&P18110,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36*/
const Device UA2774192 = {S21279, 103, { &P15364,&P18110,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36*/
const Device UA2784162 = {S21280, 112, { &P15364,&P18110,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36*/
const Device UA2784248 = {S21281, 120, { &P15364,&P18110,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36*/
const Device UA2784690 = {S21282, 118, { &P15364,&P18110,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36*/
const Device UA2898624 = {S21283, 113, { &P15364,&P18110,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36,gzip(gfe)*/
const Device UA64575 = {S21284, 77, { &P15257,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry8530/5.0.0.973 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/105*/
const Device UA48650 = {S21285, 77, { &P15232,&P6374,&P4418,&P18092 }};	/*Useragent - BlackBerry9810/7.0.0.261 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102*/
const Device UA230174 = {S21286, 131, { &P15232,&P6374,&P4418,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9810; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.261 Mobile Safari/534.11+*/
const Device UA429224 = {S21287, 87, { &P15232,&P6374,&P4418,&P18092 }};	/*Useragent - TwitterForBlackBerry/2.1.0.28 (BlackBerry; U; BlackBerry 9810; en-US) Version/7.0.0.261*/
const Device UA615829 = {S21288, 64, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.229 Version/11.64*/
const Device UA622998 = {S21289, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.229 Version/11.64*/
const Device UA623239 = {S21290, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.229 Version/11.64*/
const Device UA623342 = {S21291, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.229 Version/11.64*/
const Device UA626448 = {S21292, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.229 Version/11.64*/
const Device UA628168 = {S21293, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.64*/
const Device UA629010 = {S21294, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.229 Version/11.64*/
const Device UA630283 = {S21295, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.229 Version/11.64*/
const Device UA630633 = {S21296, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5230); ru) Presto/2.10.229 Version/11.64*/
const Device UA632258 = {S21297, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.229 Version/11.64*/
const Device UA635379 = {S21298, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.229 Version/11.64*/
const Device UA641495 = {S21299, 72, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.64*/
const Device UA642959 = {S21300, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.229 Version/11.64*/
const Device UA646342 = {S21301, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.64*/
const Device UA647938 = {S21302, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.229 Version/11.64*/
const Device UA654142 = {S21303, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.229 Version/11.64*/
const Device UA663104 = {S21304, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.10.229 Version/11.64*/
const Device UA663124 = {S21305, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5328); ru) Presto/2.10.229 Version/11.64*/
const Device UA668266 = {S21306, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5218); ru) Presto/2.10.229 Version/11.64*/
const Device UA671473 = {S21307, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.229 Version/11.64*/
const Device UA673838 = {S21308, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.229 Version/11.64*/
const Device UA675091 = {S21309, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5299); ru) Presto/2.10.229 Version/11.64*/
const Device UA676517 = {S21310, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5309); ru) Presto/2.10.229 Version/11.64*/
const Device UA678541 = {S21311, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.229 Version/11.64*/
const Device UA685431 = {S21312, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.229 Version/11.64*/
const Device UA687461 = {S21313, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.64*/
const Device UA688695 = {S21314, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.10.229 Version/11.64*/
const Device UA691565 = {S21315, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.10.229 Version/11.64*/
const Device UA699583 = {S21316, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.229 Version/11.64*/
const Device UA703487 = {S21317, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.229 Version/11.64*/
const Device UA709809 = {S21318, 101, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.229 Version/11.64,gzip(gfe) (via translate.google.com)*/
const Device UA716558 = {S21319, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5300); ru) Presto/2.10.229 Version/11.64*/
const Device UA740413 = {S21320, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.229 Version/11.64*/
const Device UA749178 = {S21321, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.229 Version/11.64*/
const Device UA755384 = {S21322, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5161); ru) Presto/2.10.229 Version/11.64*/
const Device UA765217 = {S21323, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.229 Version/11.64*/
const Device UA767715 = {S21324, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.64*/
const Device UA768493 = {S21325, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.229 Version/11.64*/
const Device UA768792 = {S21326, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.64*/
const Device UA770591 = {S21327, 124, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.229 Version/11.64,gzip(gfe) (via translate.google.com)*/
const Device UA773601 = {S21328, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.10.229 Version/11.64*/
const Device UA775960 = {S21329, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5288); ru) Presto/2.10.229 Version/11.64*/
const Device UA783882 = {S21330, 64, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; en) Presto/2.10.229 Version/11.64*/
const Device UA787942 = {S21331, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5217); ru) Presto/2.10.229 Version/11.64*/
const Device UA789899 = {S21332, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.229 Version/11.64*/
const Device UA790436 = {S21333, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.229 Version/11.64*/
const Device UA791294 = {S21334, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.229 Version/11.64*/
const Device UA796420 = {S21335, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02652); ru) Presto/2.10.229 Version/11.64*/
const Device UA804209 = {S21336, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5278); ru) Presto/2.10.229 Version/11.64*/
const Device UA815237 = {S21337, 66, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; ru) Opera 11.64*/
const Device UA815471 = {S21338, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.229 Version/11.64*/
const Device UA817816 = {S21339, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.229 Version/11.64*/
const Device UA820175 = {S21340, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5336); ru) Presto/2.10.229 Version/11.64*/
const Device UA830462 = {S21341, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.10.229 Version/11.64*/
const Device UA841732 = {S21342, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4930); ru) Presto/2.10.229 Version/11.64*/
const Device UA849578 = {S21343, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.229 Version/11.64*/
const Device UA853839 = {S21344, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.10.229 Version/11.64*/
const Device UA860777 = {S21345, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03780); ru) Presto/2.10.229 Version/11.64*/
const Device UA866160 = {S21346, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.10.229 Version/11.64*/
const Device UA866343 = {S21347, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.229 Version/11.64*/
const Device UA899889 = {S21348, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5650); ru) Presto/2.10.229 Version/11.64*/
const Device UA900566 = {S21349, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.229 Version/11.64*/
const Device UA901316 = {S21350, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5667); ru) Presto/2.10.229 Version/11.64*/
const Device UA901483 = {S21351, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5660); ru) Presto/2.10.229 Version/11.64*/
const Device UA904829 = {S21352, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.229 Version/11.64*/
const Device UA906288 = {S21353, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.229 Version/11.64*/
const Device UA911181 = {S21354, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.229 Version/11.64*/
const Device UA1974216 = {S21355, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5325); ru) Presto/2.10.229 Version/11.64*/
const Device UA1995355 = {S21356, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.10.229 Version/11.64*/
const Device UA2157249 = {S21357, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03672); ru) Presto/2.10.229 Version/11.64*/
const Device UA2161244 = {S21358, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5676); ru) Presto/2.10.229 Version/11.64*/
const Device UA2162153 = {S21359, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4661); ru) Presto/2.10.229 Version/11.64*/
const Device UA2183879 = {S21360, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.10.229 Version/11.64*/
const Device UA2184211 = {S21361, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.229 Version/11.64*/
const Device UA2229596 = {S21362, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.10.229 Version/11.64*/
const Device UA2325643 = {S21363, 87, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02560); ru) Presto/2.10.229 Version/11.64*/
const Device UA2355292 = {S21364, 74, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.229 Version/11.64,gzip(gfe)*/
const Device UA2377704 = {S21365, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5966); ru) Presto/2.10.229 Version/11.64*/
const Device UA2386168 = {S21366, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.229 Version/11.64*/
const Device UA2427771 = {S21367, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.229 Version/11.64*/
const Device UA2456593 = {S21368, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.229 Version/11.64*/
const Device UA2485471 = {S21369, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.229 Version/11.64*/
const Device UA2503052 = {S21370, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5784); ru) Presto/2.10.229 Version/11.64*/
const Device UA2503610 = {S21371, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5997); ru) Presto/2.10.229 Version/11.64*/
const Device UA2667524 = {S21372, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.229 Version/11.64*/
const Device UA2693957 = {S21373, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5745); ru) Presto/2.10.229 Version/11.64*/
const Device UA2700819 = {S21374, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5981); ru) Presto/2.10.229 Version/11.64*/
const Device UA2704124 = {S21375, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.229 Version/11.64*/
const Device UA2728120 = {S21376, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5880); ru) Presto/2.10.229 Version/11.64*/
const Device UA2751636 = {S21377, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6089); ru) Presto/2.10.229 Version/11.64*/
const Device UA2857892 = {S21378, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 6003); ru) Presto/2.10.229 Version/11.64*/
const Device UA2893178 = {S21379, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.229 Version/11.64*/
const Device UA3118843 = {S21380, 86, { &P15364,&P5690,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5989); ru) Presto/2.10.229 Version/11.64*/
const Device UA627598 = {S21381, 142, { &P18214,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA636086 = {S21382, 143, { &P18214,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-S7500L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA672831 = {S21383, 161, { &P18214,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S7500/S7500BULB1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA677118 = {S21384, 130, { &P18214,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S7500 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA703827 = {S21385, 66, { &P18214,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S7500 Build/GINGERBREAD)*/
const Device UA899194 = {S21386, 161, { &P18214,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-S7500/S7500BULC1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA924838 = {S21387, 161, { &P18214,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S7500/S7500BULE2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1954208 = {S21388, 143, { &P18214,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S7500W Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1990268 = {S21389, 120, { &P18214,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-gb; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA1991190 = {S21390, 154, { &P18214,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6_mobile-star; iw-il; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025186 = {S21391, 135, { &P18214,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S7500 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2230594 = {S21392, 140, { &P18214,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2256938 = {S21393, 143, { &P18214,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-S7500T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2333089 = {S21394, 67, { &P18214,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S7500L Build/GINGERBREAD)*/
const Device UA2361956 = {S21395, 173, { &P18214,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6_mobile-star; iw-il; SAMSUNG GT-S7500/S7500BULE2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2364782 = {S21396, 120, { &P18214,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; it-it; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2394733 = {S21397, 120, { &P18214,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; de-de; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2470657 = {S21398, 161, { &P18214,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-S7500/S7500BULK1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2527614 = {S21399, 67, { &P18214,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S7500T Build/GINGERBREAD)*/
const Device UA2674766 = {S21400, 161, { &P18214,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-S7500/S7500BUMB1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2722281 = {S21401, 120, { &P18214,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; ro-ro; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2758819 = {S21402, 120, { &P18214,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; pt-pt; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2015112 = {S21403, 117, { &P17595,&P18207,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.82 Safari/537.1*/
const Device UA2015617 = {S21404, 117, { &P17595,&P18207,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2015621 = {S21405, 117, { &P17595,&P18207,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2041484 = {S21406, 154, { &P17595,&P18207,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)*/
const Device UA2200548 = {S21407, 154, { &P17595,&P18207,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)*/
const Device UA2343924 = {S21408, 154, { &P17595,&P18207,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)*/
const Device UA2549005 = {S21409, 117, { &P17595,&P18207,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2600427 = {S21410, 114, { &P17595,&P18207,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1080) AppleWebKit/537.1 (KHTML like Gecko) Chrome/21.0.1180.57 Safari/537.1*/
const Device UA2600762 = {S21411, 114, { &P17595,&P18207,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1082) AppleWebKit/537.1 (KHTML like Gecko) Chrome/21.0.1180.82 Safari/537.1*/
const Device UA2603748 = {S21412, 114, { &P17595,&P18207,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1081) AppleWebKit/537.1 (KHTML like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2781230 = {S21413, 117, { &P17595,&P18207,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA3091584 = {S21414, 117, { &P17595,&P18207,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA3025634 = {S21415, 135, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A442 Safari/9537.53*/
const Device UA3027670 = {S21416, 135, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A441 Safari/9537.53*/
const Device UA3027724 = {S21417, 136, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A446a Safari/9537.53*/
const Device UA3030456 = {S21418, 135, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A451 Safari/9537.53*/
const Device UA3030768 = {S21419, 135, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A445 Safari/9537.53*/
const Device UA3031522 = {S21420, 137, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A4449d Safari/9537.53*/
const Device UA3031947 = {S21421, 137, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A4449a Safari/9537.53*/
const Device UA3032043 = {S21422, 135, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A449 Safari/9537.53*/
const Device UA3032347 = {S21423, 135, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A452 Safari/9537.53*/
const Device UA3033301 = {S21424, 135, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A462 Safari/9537.53*/
const Device UA3033394 = {S21425, 135, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A450 Safari/9537.53*/
const Device UA3034094 = {S21426, 135, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A457 Safari/9537.53*/
const Device UA3035600 = {S21427, 135, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A464 Safari/9537.53*/
const Device UA3036016 = {S21428, 135, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A448 Safari/9537.53*/
const Device UA3036157 = {S21429, 135, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A463 Safari/9537.53*/
const Device UA3037104 = {S21430, 135, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A459 Safari/9537.53*/
const Device UA3037136 = {S21431, 135, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A456 Safari/9537.53*/
const Device UA3037942 = {S21432, 135, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A453 Safari/9537.53*/
const Device UA3038196 = {S21433, 135, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A458 Safari/9537.53*/
const Device UA3039484 = {S21434, 135, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A465 Safari/9537.53*/
const Device UA3062577 = {S21435, 137, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B472 Safari/9537.53*/
const Device UA3066113 = {S21436, 137, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B473 Safari/9537.53*/
const Device UA3066825 = {S21437, 135, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A466 Safari/9537.53*/
const Device UA3079750 = {S21438, 137, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B470 Safari/9537.53*/
const Device UA3080184 = {S21439, 137, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B475 Safari/9537.53*/
const Device UA3082217 = {S21440, 137, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B481 Safari/9537.53*/
const Device UA3083827 = {S21441, 137, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B478 Safari/9537.53*/
const Device UA3092809 = {S21442, 137, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A4449b Safari/9537.53*/
const Device UA3094865 = {S21443, 138, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A470a Safari/9537.53*/
const Device UA3108865 = {S21444, 135, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A444 Safari/9537.53*/
const Device UA3109625 = {S21445, 137, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B479 Safari/9537.53*/
const Device UA3110239 = {S21446, 135, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A468 Safari/9537.53*/
const Device UA3113755 = {S21447, 137, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B488 Safari/9537.53*/
const Device UA3114148 = {S21448, 134, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A465 Safari/9537.53*/
const Device UA3114193 = {S21449, 134, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A465Safari/9537.53*/
const Device UA3115138 = {S21450, 137, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A501 Safari/9537.53*/
const Device UA3115741 = {S21451, 137, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B480 Safari/9537.53*/
const Device UA3115759 = {S21452, 137, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_3 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B491 Safari/9537.53*/
const Device UA3126891 = {S21453, 137, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B487 Safari/9537.53*/
const Device UA3127049 = {S21454, 137, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A470 Safari/9537.53*/
const Device UA3128010 = {S21455, 137, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_3 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B494 Safari/9537.53*/
const Device UA3128273 = {S21456, 137, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B489 Safari/9537.53*/
const Device UA3128780 = {S21457, 136, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A501Safari/9537.53*/
const Device UA3128981 = {S21458, 138, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_3 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B498a Safari/9537.53*/
const Device UA3129083 = {S21459, 137, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_3 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B497 Safari/9537.53*/
const Device UA3130811 = {S21460, 137, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_3 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B495 Safari/9537.53*/
const Device UA3130982 = {S21461, 136, { &P12280,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A501 Safari/9537.53*/
const Device UA909175 = {S21462, 141, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; Galaxy Nexus Build/JRN84D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA909928 = {S21463, 127, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; Galaxy Nexus Build/JRN84D) AppleWebKit/535.7 (KHTML, like Gecko) Mobile Safari/535.7*/
const Device UA910024 = {S21464, 63, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1; Galaxy Nexus Build/JRN84D)*/
const Device UA911827 = {S21465, 143, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA912404 = {S21466, 65, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Galaxy Nexus Build/JRO03C)*/
const Device UA913190 = {S21467, 143, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRN84D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA923784 = {S21468, 143, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA923809 = {S21469, 143, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA925182 = {S21470, 143, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1950439 = {S21471, 129, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Galaxy Nexus Build/JRO03C) AppleWebKit/535.7 (KHTML, like Gecko) Mobile Safari/535.7*/
const Device UA1950813 = {S21472, 141, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; Galaxy Nexus Build/IMM76K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1975914 = {S21473, 143, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1980067 = {S21474, 143, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1997253 = {S21475, 143, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JR003H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1999573 = {S21476, 143, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2000589 = {S21477, 143, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03O) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028650 = {S21478, 134, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Galaxy Nexus Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2028677 = {S21479, 143, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033333 = {S21480, 149, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2042683 = {S21481, 143, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03U) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2204700 = {S21482, 143, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2206125 = {S21483, 143, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Galaxy Nexus Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2231623 = {S21484, 143, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Galaxy Nexus Build/JZ054K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2232476 = {S21485, 143, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Galaxy Nexus Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2237121 = {S21486, 65, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Galaxy Nexus Build/JZO54K)*/
const Device UA2257887 = {S21487, 143, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Galaxy Nexus Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2354931 = {S21488, 134, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Galaxy Nexus Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2367187 = {S21489, 149, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03O) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30*/
const Device UA2387297 = {S21490, 154, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Galaxy Nexus Build/JZO54K/4.2 Hybrid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2405455 = {S21491, 139, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; Galaxy X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2420671 = {S21492, 129, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03O) AppleWebKit/535.7 (KHTML, like Gecko) Mobile Safari/535.7*/
const Device UA2445538 = {S21493, 141, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; Galaxy Nexus Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2466310 = {S21494, 143, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Galaxy Nexus Build/JR003H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2759871 = {S21495, 148, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; Galaxy Nexus Build/WWW.ROMUP.COM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2824517 = {S21496, 159, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Galaxy Nexus Build/Wind-Galaxy Nexus-V1.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2983291 = {S21497, 61, { &P16292,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Galaxy X Build/JRO03C)*/
const Device UA95955 = {S21498, 98, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/531.21.11 (KHTML, like Gecko)*/
const Device UA96565 = {S21499, 92, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6; en-us) AppleWebKit/531.9 (KHTML, like Gecko)*/
const Device UA96798 = {S21500, 94, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-us) AppleWebKit/531.9 (KHTML, like Gecko)*/
const Device UA99207 = {S21501, 94, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-us) AppleWebKit/531.9 (KHTML, like Gecko)*/
const Device UA133506 = {S21502, 97, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.18.1 (KHTML, like Gecko)*/
const Device UA149622 = {S21503, 88, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - User-Agent:Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.18.1*/
const Device UA154473 = {S21504, 97, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko)*/
const Device UA166808 = {S21505, 97, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko)*/
const Device UA181410 = {S21506, 97, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko)*/
const Device UA191138 = {S21507, 97, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko)*/
const Device UA195228 = {S21508, 85, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/534.24 (KHTML, like Gecko)*/
const Device UA351743 = {S21509, 100, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.52.7 (KHTML, like Gecko) Sparrow/1043*/
const Device UA391553 = {S21510, 209, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.17.8 (KHTML, like Gecko) Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en) AppleWebKit/533.17.8 (KHTML, like Gecko) "wKiosk for Mac"*/
const Device UA394251 = {S21511, 119, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/531.21.8+(KHTML, like Gecko, Safari/528.16) (null)*/
const Device UA409326 = {S21512, 211, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; it-it) AppleWebKit/533.20.25 (KHTML, like Gecko) Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en) AppleWebKit/533.20.25 (KHTML, like Gecko) "wKiosk for Mac"*/
const Device UA420409 = {S21513, 128, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Paparazzi!/0.5b5 (like Safari)*/
const Device UA421311 = {S21514, 75, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mac OS X/10.6.8 (10K549); ExchangeWebServices/1.3 (61); iCal/4.0.4 (1395.7)*/
const Device UA543950 = {S21515, 53, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en)*/
const Device UA553091 = {S21516, 128, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) WebClip/6530.10 Safari/6530.10*/
const Device UA568006 = {S21517, 209, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en) AppleWebKit/531.22.7 (KHTML, like Gecko) "wKiosk for Mac"*/
const Device UA575512 = {S21518, 102, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.51.22 (KHTML, like Gecko) Sparrow/959.1*/
const Device UA649795 = {S21519, 44, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - MacOutlook/14.12.0.X (Intel Mac OS X 10.6.8)*/
const Device UA678963 = {S21520, 116, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) NetNewsWire/3.2.15*/
const Device UA685079 = {S21521, 102, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1043.1*/
const Device UA743351 = {S21522, 115, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) NetNewsWire/3.2.7*/
const Device UA767332 = {S21523, 209, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en) AppleWebKit/533.21.1 (KHTML, like Gecko) "wKiosk for Mac"*/
const Device UA773725 = {S21524, 106, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) NetNewsWire/3.2.15*/
const Device UA788004 = {S21525, 103, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) NetNewsWire/3.3*/
const Device UA792158 = {S21526, 109, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.2.10) Gecko Expeditor 6.2.2.X-1115 Gecko/X-1115*/
const Device UA793126 = {S21527, 105, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Vienna/2.6.0.2601*/
const Device UA812178 = {S21528, 116, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) NetNewsWire/3.2.15*/
const Device UA873473 = {S21529, 100, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Safari/531.9*/
const Device UA879901 = {S21530, 110, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Safari/531.9*/
const Device UA896321 = {S21531, 103, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1081.27*/
const Device UA898945 = {S21532, 103, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1081.51*/
const Device UA901824 = {S21533, 103, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1081.52*/
const Device UA911691 = {S21534, 100, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1144*/
const Device UA1876287 = {S21535, 115, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) NetNewsWire/3.2.7*/
const Device UA1879419 = {S21536, 110, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Safari/522.0*/
const Device UA1887466 = {S21537, 110, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Safari/531.9*/
const Device UA1890997 = {S21538, 110, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Safari/522.0*/
const Device UA1950500 = {S21539, 102, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1043.2*/
const Device UA1955861 = {S21540, 105, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) NetNewsWire/3.2.7*/
const Device UA1956606 = {S21541, 95, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sunrise*/
const Device UA1960335 = {S21542, 102, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1143.6*/
const Device UA1988811 = {S21543, 103, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1081.53*/
const Device UA1988842 = {S21544, 100, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Safari/522.0*/
const Device UA1995540 = {S21545, 100, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1164*/
const Device UA1998521 = {S21546, 105, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) NetNewsWire/3.3.2*/
const Device UA2010015 = {S21547, 100, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1172*/
const Device UA2137930 = {S21548, 131, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) LittleSnapper/12474 Safari/525.13*/
const Device UA2149815 = {S21549, 112, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Sparrow/1043.1*/
const Device UA2156469 = {S21550, 103, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1081.28*/
const Device UA2162002 = {S21551, 100, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Safari/417.8*/
const Device UA2166434 = {S21552, 110, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) RealPlayer/12.0.0.1725*/
const Device UA2167614 = {S21553, 105, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) NetNewsWire/3.3.1*/
const Device UA2179027 = {S21554, 100, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1173*/
const Device UA2202995 = {S21555, 100, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1176*/
const Device UA2217815 = {S21556, 99, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/904*/
const Device UA2222225 = {S21557, 105, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) NetNewsWire/3.1.7*/
const Device UA2234955 = {S21558, 100, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/1178*/
const Device UA2258677 = {S21559, 126, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) FluidApp/1.1 Safari/533.21.1*/
const Device UA2291891 = {S21560, 112, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) (null) FBSMTWB*/
const Device UA2292043 = {S21561, 131, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) LittleSnapper/12474 Safari/525.13*/
const Device UA2308341 = {S21562, 200, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.51.22 (KHTML, like Gecko) Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en) AppleWebKit/533.21.1 (KHTML, like Gecko) "wKiosk for Mac"*/
const Device UA2335930 = {S21563, 106, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) NetNewsWire/3.2.11*/
const Device UA2359997 = {S21564, 96, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) iWeb/304*/
const Device UA2364276 = {S21565, 79, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.1.7) Zimbra/7.2.0*/
const Device UA2402360 = {S21566, 118, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Paparazzi!/0.6.5 (like Safari)*/
const Device UA2410727 = {S21567, 110, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) RealPlayer/11.1.0.1116*/
const Device UA2412832 = {S21568, 105, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Vienna/2.5.0.2501*/
const Device UA2413432 = {S21569, 94, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) TubeTV*/
const Device UA2438320 = {S21570, 134, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) LittleSnapper/12267 Safari/531.21.10*/
const Device UA2451709 = {S21571, 118, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/(null) Safari/533.21.1*/
const Device UA2456311 = {S21572, 138, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Safari/531.2+ Miro/5.0.4 (http://www.getmiro.com/)*/
const Device UA2473549 = {S21573, 97, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) KidZui K2*/
const Device UA2493953 = {S21574, 158, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Mozilla/4.0; Mozilla/5.0; Mozilla/6.0; Safari/431.7; Macintosh; U; PPC Mac OS X 10.6 Leopard; AppleWebKit/421.9 (KHTML, like Gecko) )*/
const Device UA2500801 = {S21575, 110, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) RealPlayer/12.0.1.1750*/
const Device UA2520316 = {S21576, 110, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) RealPlayer/12.0.0.1569*/
const Device UA2546174 = {S21577, 102, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) (null) FBSMTWB*/
const Device UA2553614 = {S21578, 128, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/(null) Safari/533.21.1*/
const Device UA2569179 = {S21579, 120, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/534.24 (KHTML, like Gecko) Contact: feedback@getprismatic.com*/
const Device UA2588241 = {S21580, 80, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - MacAppStore/1.0.2 (Macintosh; U; Intel Mac OS X 10.6.8; en) AppleWebKit/534.57.2*/
const Device UA2588493 = {S21581, 105, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Vienna/2.5.1.2502*/
const Device UA2681812 = {S21582, 59, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - iPhoto/8.1.2 (Macintosh; U; Intel Mac OS X 10.6.8; English)*/
const Device UA2726383 = {S21583, 110, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Sparrow/1176*/
const Device UA2757852 = {S21584, 96, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - DAVKit/4.0.3 (732.2); CalendarStore/4.0.4 (997.7); iCal/4.0.4 (1395.7); Mac OS X/10.6.8 (10K549)*/
const Device UA2803994 = {S21585, 80, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - MacAppStore/1.0.2 (Macintosh; U; Intel Mac OS X 10.6.8; en) AppleWebKit/533.21.1*/
const Device UA2814801 = {S21586, 101, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Sparrow/959.4*/
const Device UA2820587 = {S21587, 103, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.55.3 (KHTML, like Gecko) Safari/534.55.3*/
const Device UA2859249 = {S21588, 55, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us*/
const Device UA2859961 = {S21589, 126, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/533.16 (KHTML, like Gecko) OceanBrowser/2.0 Safari/533.16*/
const Device UA3013536 = {S21590, 43, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/4.0 (Mac OS X 10.6.8) Java/1.6.0_51*/
const Device UA3018199 = {S21591, 116, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.59.8 (KHTML, like Gecko) Vienna/3.0.0 Safari/534.59.8*/
const Device UA3087872 = {S21592, 113, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.59.8 (KHTML, like Gecko) Elluminate Live! Web Tour*/
const Device UA3087963 = {S21593, 80, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - MacAppStore/1.0.2 (Macintosh; U; Intel Mac OS X 10.6.8; en) AppleWebKit/534.59.8*/
const Device UA3100721 = {S21594, 41, { &P17595,&P17296,&P18108,&P18092 }};	/*Useragent - Entourage/12.36.0 (Intel Mac OS X 10.6.8)*/
const Device UA2257969 = {S21595, 140, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One S Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258053 = {S21596, 140, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2356091 = {S21597, 140, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2421673 = {S21598, 140, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC One S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426427 = {S21599, 62, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC One S Build/JRO03C)*/
const Device UA2428998 = {S21600, 151, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_S/3.16.110.3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2429924 = {S21601, 151, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_S/3.16.161.9 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2460083 = {S21602, 140, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC_One_S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2461155 = {S21603, 151, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-ch; HTC_One_S/3.16.166.8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467901 = {S21604, 156, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC_One_S_Special_Edition Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2468333 = {S21605, 136, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; Z520m Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2480335 = {S21606, 62, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC One S Build/JZO54K)*/
const Device UA2491522 = {S21607, 150, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_S-orange-LS Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2509090 = {S21608, 151, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-at; HTC_One_S/3.16.169.8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2511292 = {S21609, 151, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_S/3.16.111.5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2531831 = {S21610, 151, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; HTC_One_S/3.16.114.5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567408 = {S21611, 151, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_S/3.16.111.7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658122 = {S21612, 151, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; HTC_One_S/2.38.161.6 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2660433 = {S21613, 153, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; HTC_One_S/3.17.113.102 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2661243 = {S21614, 151, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-at; HTC_One_S/3.16.112.5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2675587 = {S21615, 161, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_S_pj40200/3.19.161.101 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2699756 = {S21616, 131, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC One S Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2700354 = {S21617, 151, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; HTC_One_S/3.16.163.8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2707475 = {S21618, 161, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-at; HTC_One_S_pj40200/3.19.169.101 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708167 = {S21619, 140, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC VLE_U Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2714684 = {S21620, 161, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_S_pj40200/3.19.161.102 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2716915 = {S21621, 140, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; HTC_VLE_U Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719281 = {S21622, 152, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_S/3.16.161.10 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719294 = {S21623, 62, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC VLE_U Build/JRO03C)*/
const Device UA2778166 = {S21624, 78, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC One S Special Edition Build/JRO03C)*/
const Device UA2840065 = {S21625, 156, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One S Special Edition Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2849204 = {S21626, 138, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; HTC One S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2851597 = {S21627, 151, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-ch; HTC_One_S/2.38.166.6 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2852388 = {S21628, 153, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; HTC_One_S/3.17.118.102 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2857639 = {S21629, 152, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_S/2.38.111.10 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2893043 = {S21630, 152, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_S/3.16.111.10 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985380 = {S21631, 138, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-; HTC_One_S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3009913 = {S21632, 140, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_One_S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044235 = {S21633, 151, { &P18280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_S/2.38.110.2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2436067 = {S21634, 141, { &P15767,&P19826,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B5126b Safari/8536.25*/
const Device UA2492643 = {S21635, 139, { &P15767,&P19826,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B141 Safari/8536.25*/
const Device UA2532510 = {S21636, 141, { &P15767,&P19826,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B147 Safari/8536.25*/
const Device UA2532511 = {S21637, 141, { &P15767,&P19826,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B146 Safari/8536.25*/
const Device UA2585475 = {S21638, 141, { &P15767,&P19826,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B329 Safari/8536.25*/
const Device UA221371 = {S21639, 137, { &P15364,&P5690,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/534.7 ZemanaAID/FFFF0061*/
const Device UA247030 = {S21640, 99, { &P15364,&P5690,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1*/
const Device UA247991 = {S21641, 136, { &P15364,&P5690,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1,gzip(gfe) (via translate.google.com)*/
const Device UA333717 = {S21642, 99, { &P15364,&P5690,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1*/
const Device UA636101 = {S21643, 105, { &P15364,&P5690,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1*/
const Device UA2259089 = {S21644, 106, { &P15364,&P5690,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1*/
const Device UA2259262 = {S21645, 110, { &P15364,&P5690,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/13 Safari/525.13*/
const Device UA2295269 = {S21646, 112, { &P15364,&P5690,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1, SurfCop 1.5*/
const Device UA2297218 = {S21647, 138, { &P15364,&P5690,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2355516 = {S21648, 109, { &P15364,&P5690,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1,gzip(gfe)*/
const Device UA2817795 = {S21649, 104, { &P15364,&P5690,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.1 (KHTML like Gecko) Chrome/13.0.782.41 Safari/535.1*/
const Device UA90799 = {S21650, 127, { &P15767,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5288d Safari/7534.48.3*/
const Device UA90801 = {S21651, 127, { &P15767,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5313e Safari/7534.48.3*/
const Device UA90803 = {S21652, 127, { &P15767,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5302b Safari/7534.48.3*/
const Device UA90806 = {S21653, 125, { &P15767,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3*/
const Device UA90844 = {S21654, 123, { &P15767,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.46*/
const Device UA90871 = {S21655, 127, { &P15767,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A402 Safari/7534.48.3*/
const Device UA257274 = {S21656, 125, { &P15767,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.46*/
const Device UA358439 = {S21657, 108, { &P15767,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334*/
const Device UA622533 = {S21658, 127, { &P15767,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3*/
const Device UA624555 = {S21659, 127, { &P15767,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5274d Safari/7534.48.3*/
const Device UA637556 = {S21660, 127, { &P15767,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9Z999 Safari/7534.48.3*/
const Device UA676909 = {S21661, 127, { &P15767,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7534.48.3*/
const Device UA708363 = {S21662, 134, { &P15767,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A404 Safari/7534.48.3*/
const Device UA1892083 = {S21663, 127, { &P15767,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5220p Safari/7534.48.3*/
const Device UA1956435 = {S21664, 117, { &P15767,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari Version/5.1*/
const Device UA2153358 = {S21665, 127, { &P15767,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA2319193 = {S21666, 125, { &P15767,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3*/
const Device UA2354061 = {S21667, 131, { &P15767,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU iPhone OS 501 like Mac OS X) AppleWebKit/534.46 (KHTML like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3*/
const Device UA2355738 = {S21668, 144, { &P15767,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/537.4 (KHTML, like Gecko; Google Web Preview) Version/5.1 Mobile/9A334 Safari/7534.48.3*/
const Device UA2386616 = {S21669, 150, { &P15767,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5 like Mac OS X; zh-cn Model:iPad2,1) UC AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7543.48.3*/
const Device UA2664565 = {S21670, 125, { &P15767,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA623420 = {S21671, 146, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC_Explorer_A310b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625641 = {S21672, 146, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC_Explorer_A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626849 = {S21673, 146, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC Explorer A310b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627553 = {S21674, 146, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA631732 = {S21675, 140, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_Explorer Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633625 = {S21676, 140, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Explorer Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA644895 = {S21677, 151, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/Explorer/1.17.161.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA692381 = {S21678, 150, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_Explorer-orange-LS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA694741 = {S21679, 151, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; HTC/Explorer/1.29.161.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA700664 = {S21680, 70, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Explorer A310e Build/GRJ90)*/
const Device UA716789 = {S21681, 151, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-ch; HTC/Explorer/1.29.166.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA737197 = {S21682, 134, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Explorer A310e Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA841615 = {S21683, 151, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC/Explorer/1.29.163.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA860550 = {S21684, 151, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC/Explorer/1.17.163.5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA924768 = {S21685, 151, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/Explorer/1.41.161.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1934389 = {S21686, 151, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-ch; HTC/Explorer/1.17.166.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1942911 = {S21687, 64, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Explorer Build/GRJ90)*/
const Device UA1996115 = {S21688, 151, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-ch; HTC/Explorer/1.41.166.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1996633 = {S21689, 151, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC/Explorer/1.41.163.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023685 = {S21690, 125, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC A310e Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2025331 = {S21691, 70, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Explorer A310b Build/GRJ90)*/
const Device UA2028082 = {S21692, 137, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-cn; HTC_A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2444638 = {S21693, 130, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC A310e Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2545151 = {S21694, 137, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2759251 = {S21695, 139, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Explorer A310e Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2985444 = {S21696, 151, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; HTC/Explorer/1.41.161.6 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3070415 = {S21697, 151, { &P15798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC/Explorer/1.41.192.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2205336 = {S21698, 151, { &P12280,&P18432,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A405 Safari/7534.48.3*/
const Device UA2205345 = {S21699, 151, { &P12280,&P18432,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A403 Safari/7534.48.3*/
const Device UA2261168 = {S21700, 153, { &P12280,&P18432,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A523 Safari/7534.48.3*/
const Device UA2261169 = {S21701, 153, { &P12280,&P18432,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A525 Safari/7534.48.3*/
const Device UA2437635 = {S21702, 153, { &P12280,&P18432,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A551 Safari/7534.48.3*/
const Device UA2193923 = {S21703, 117, { &P17595,&P17296,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4*/
const Device UA2203727 = {S21704, 117, { &P17595,&P17296,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203736 = {S21705, 117, { &P17595,&P17296,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203740 = {S21706, 117, { &P17595,&P17296,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203791 = {S21707, 117, { &P17595,&P17296,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203884 = {S21708, 117, { &P17595,&P17296,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203894 = {S21709, 117, { &P17595,&P17296,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203895 = {S21710, 117, { &P17595,&P17296,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2204010 = {S21711, 117, { &P17595,&P17296,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2231336 = {S21712, 154, { &P17595,&P17296,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via translate.google.com)*/
const Device UA2235714 = {S21713, 127, { &P17595,&P17296,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)*/
const Device UA2258267 = {S21714, 156, { &P17595,&P17296,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2597219 = {S21715, 114, { &P17595,&P17296,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1060) AppleWebKit/537.4 (KHTML like Gecko) Chrome/22.0.1229.79 Safari/537.4*/
const Device UA2674482 = {S21716, 129, { &P17595,&P17296,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/22.0.648.205 Safari/534.16*/
const Device UA2674793 = {S21717, 129, { &P17595,&P17296,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/22.0.648.205 Safari/534.16*/
const Device UA2714928 = {S21718, 129, { &P17595,&P17296,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/22.0.648.205 Safari/534.16*/
const Device UA2857581 = {S21719, 148, { &P17595,&P17296,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.510 YaBrowser/1.5.1105.510 (gtsr) Safari/537.4*/
const Device UA2989126 = {S21720, 148, { &P17595,&P17296,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.510 YaBrowser/1.5.1105.510 (fnrb) Safari/537.4*/
const Device UA516766 = {S21721, 115, { &P15364,&P18110,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (GTmetrix http://gtmetrix.com/; X11; U; Linux i686 (x86_64); en-US; rv:1.9.2.13) Gecko/X Firefox/3.6.13*/
const Device UA575592 = {S21722, 74, { &P15364,&P18110,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11;Arch Linux i686; en-US; rv:1.9.2.8) Gecko/X Firefox/3.6.8*/
const Device UA778904 = {S21723, 96, { &P15364,&P18110,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; de; rv:1.9.2.12) Gecko/X Iceweasel/3.6.12 (like Firefox/3.6.12)*/
const Device UA825791 = {S21724, 98, { &P15364,&P18110,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-AU; rv:1.9.2.26) Gecko/X Firefox/2.0.0.3 (like Firefox/3.6.26)*/
const Device UA2013048 = {S21725, 94, { &P15364,&P18110,&P17558,&P18092 }};	/*Useragent - OMozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.9) Gecko/X Red Hat/3.6.9-2.el6 Firefox/3.6.9*/
const Device UA2136305 = {S21726, 96, { &P15364,&P18110,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.17) Gecko/X Firefox/8.04 (hardy) Firefox/3.6.17*/
const Device UA2157021 = {S21727, 85, { &P15364,&P18110,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 on x86_64; rv:5.0) Gecko/X Firefox/3.6.17 Firefox/3.6.17*/
const Device UA224547 = {S21728, 93, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50*/
const Device UA227260 = {S21729, 93, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50*/
const Device UA228719 = {S21730, 100, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50*/
const Device UA232652 = {S21731, 100, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50*/
const Device UA250935 = {S21732, 108, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22*/
const Device UA251022 = {S21733, 101, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22*/
const Device UA251075 = {S21734, 108, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22*/
const Device UA344505 = {S21735, 101, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22*/
const Device UA347421 = {S21736, 106, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7*/
const Device UA350562 = {S21737, 106, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7*/
const Device UA428873 = {S21738, 101, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16*/
const Device UA428884 = {S21739, 108, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16*/
const Device UA440932 = {S21740, 99, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3*/
const Device UA615810 = {S21741, 99, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2*/
const Device UA622555 = {S21742, 99, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7*/
const Device UA626982 = {S21743, 106, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3*/
const Device UA641419 = {S21744, 99, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3*/
const Device UA647853 = {S21745, 106, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2*/
const Device UA688410 = {S21746, 108, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16*/
const Device UA691358 = {S21747, 99, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7*/
const Device UA714921 = {S21748, 106, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3*/
const Device UA800802 = {S21749, 106, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2*/
const Device UA805496 = {S21750, 99, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2*/
const Device UA857337 = {S21751, 101, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16*/
const Device UA2158936 = {S21752, 109, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.34 (KHTML, like Gecko) SlimBoat/1.0.12 Version/5.1 Safari/534.34*/
const Device UA2586542 = {S21753, 100, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.3 Safari/534.53.10*/
const Device UA2586545 = {S21754, 107, { &P15364,&P5690,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.3 Safari/534.53.10*/
const Device UA616429 = {S21755, 257, { &P17346,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 6_0 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.0;FBSS/2; FBCR/unavailable;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA1999586 = {S21756, 106, { &P17346,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A5376e*/
const Device UA2043647 = {S21757, 104, { &P17346,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403*/
const Device UA2166928 = {S21758, 90, { &P17346,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2201959 = {S21759, 132, { &P17346,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 iPod4,1 BingWeb/3.03.1428.X*/
const Device UA2202159 = {S21760, 226, { &P17346,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.0;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]*/
const Device UA2232219 = {S21761, 137, { &P17346,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPodTouch4G; iPhone; U; CPU OS 6_0 like Mac OS X; en_US) com.google.GooglePlus/10352 (KHTML, like Gecko) Mobile/N81AP (gzip)*/
const Device UA2238381 = {S21762, 104, { &P17346,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A406*/
const Device UA2261375 = {S21763, 261, { &P17346,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 6_0_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.0.1;FBSS/2; FBCR/unavailable;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA2261389 = {S21764, 106, { &P17346,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523*/
const Device UA2261391 = {S21765, 134, { &P17346,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 iPod4,1 BingWeb/3.03.1428.X*/
const Device UA2357216 = {S21766, 92, { &P17346,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2403960 = {S21767, 136, { &P17346,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 iPod5,1 BingWeb/3.03.1428.X*/
const Device UA2404202 = {S21768, 147, { &P17346,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 bline/1.03 (iPhone OS 6.0.1, iPod touch)*/
const Device UA2425080 = {S21769, 142, { &P17346,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone 6_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPod4,1/6.0.1)*/
const Device UA2428899 = {S21770, 230, { &P17346,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.0.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]*/
const Device UA2434132 = {S21771, 226, { &P17346,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A406 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPod5,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/6.0;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]*/
const Device UA2481544 = {S21772, 154, { &P17346,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper*/
const Device UA2484561 = {S21773, 135, { &P17346,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod4,1; iPhone; U; CPU OS 6_0_1 like Mac OS X; en_US) com.google.GooglePlus/13196 (KHTML, like Gecko) Mobile/N81AP (gzip)*/
const Device UA2484678 = {S21774, 225, { &P17346,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPad3,4;FBMD/iPad;FBSN/iPhone OS;FBSV/6.0.1;FBSS/2; FBCR/;FBID/tablet;FBLC/en_US]*/
const Device UA2578413 = {S21775, 123, { &P17346,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3*/
const Device UA3070206 = {S21776, 152, { &P17346,&P18432,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper*/
const Device UA90290 = {S21777, 141, { &P15767,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA91110 = {S21778, 127, { &P15767,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en_US) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5*/
const Device UA147443 = {S21779, 139, { &P15767,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 Safari/6533.18.5*/
const Device UA153184 = {S21780, 140, { &P15767,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134b Safari/6533.18.5*/
const Device UA305118 = {S21781, 141, { &P15767,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9Z999 Safari/6533.18.5*/
const Device UA688801 = {S21782, 140, { &P15767,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en_us) AppleWebKit/6533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA719898 = {S21783, 153, { &P15767,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; zh-cn Model:iPad1,1) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA880508 = {S21784, 141, { &P15767,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C5115c Safari/6533.18.5*/
const Device UA2143717 = {S21785, 141, { &P15767,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.3*/
const Device UA2207243 = {S21786, 141, { &P15767,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C5101c Safari/6533.18.5*/
const Device UA2642827 = {S21787, 125, { &P15767,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C134 Safari/6533.18.5*/
const Device UA2642831 = {S21788, 138, { &P15767,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.1*/
const Device UA2711755 = {S21789, 139, { &P15767,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 421 like Mac OS X; haw-US) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA12930 = {S21790, 117, { &P15364,&P7133,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.5 (KHTML, likeGecko) Chrome/4.1.249.1045 Safari/532.5*/
const Device UA113190 = {S21791, 115, { &P15364,&P7133,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Chrome/4.0.221.7 Safari/532.2*/
const Device UA130501 = {S21792, 118, { &P15364,&P7133,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.1.249.1045 Safari/532.5*/
const Device UA2259889 = {S21793, 165, { &P15364,&P7133,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; AppleWebKit/540.0 (KHTML, like Gecko) ChromePlus/4.0.222.3 Chrome/4.0.222.3 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA2332393 = {S21794, 143, { &P15364,&P7133,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US Alexa Toolbar) AppleWebKit/532.5 (KHTML	 like Gecko Alexa Toolbar) Chrome/4.0.249.0 Safari/532.5*/
const Device UA2786641 = {S21795, 118, { &P15364,&P7133,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; AppleWebKit/532.2; KHTML?? like Gecko; Chrome/4.0.222.12 Safari/532.2)*/
const Device UA2801409 = {S21796, 116, { &P15364,&P7133,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; it-IT) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.25 Safari/532.5*/
const Device UA2509571 = {S21797, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.14*/
const Device UA2509581 = {S21798, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.14*/
const Device UA2509582 = {S21799, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.14*/
const Device UA2509598 = {S21800, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.14*/
const Device UA2510161 = {S21801, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.14*/
const Device UA2510167 = {S21802, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.14*/
const Device UA2510187 = {S21803, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.14*/
const Device UA2510162 = {S21804, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.14*/
const Device UA2510781 = {S21805, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.14*/
const Device UA2510787 = {S21806, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.14*/
const Device UA2510793 = {S21807, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.14*/
const Device UA2510794 = {S21808, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.14*/
const Device UA2510814 = {S21809, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.14*/
const Device UA2511304 = {S21810, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.14*/
const Device UA2511317 = {S21811, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.14*/
const Device UA2512444 = {S21812, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.14*/
const Device UA2512454 = {S21813, 64, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; WOW64) Presto/2.12.388 Version/12.14*/
const Device UA2512779 = {S21814, 57, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2) Presto/2.12.388 Version/12.14*/
const Device UA2512786 = {S21815, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.14*/
const Device UA2512798 = {S21816, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.14*/
const Device UA2512805 = {S21817, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.14*/
const Device UA2512807 = {S21818, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.14*/
const Device UA2512810 = {S21819, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.14*/
const Device UA2512812 = {S21820, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.14*/
const Device UA2512813 = {S21821, 78, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; Edition Campaign 21) Presto/2.12.388 Version/12.14*/
const Device UA2512892 = {S21822, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.14*/
const Device UA2513058 = {S21823, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.14*/
const Device UA2513064 = {S21824, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.14*/
const Device UA2513066 = {S21825, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.14*/
const Device UA2513302 = {S21826, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.14*/
const Device UA2513362 = {S21827, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.14*/
const Device UA2513862 = {S21828, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.14*/
const Device UA2514039 = {S21829, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.14*/
const Device UA2514047 = {S21830, 67, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.14,gzip(gfe)*/
const Device UA2514048 = {S21831, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.14*/
const Device UA2514054 = {S21832, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.14*/
const Device UA2514061 = {S21833, 62, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1) Opera 12.14*/
const Device UA2514357 = {S21834, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.14*/
const Device UA2514391 = {S21835, 68, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:2.0) Gecko/X Firefox/4.0 Opera 12.14*/
const Device UA2514410 = {S21836, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.14*/
const Device UA2514411 = {S21837, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.14*/
const Device UA2514550 = {S21838, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6046)) Presto/2.12.388 Version/12.14*/
const Device UA2515198 = {S21839, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.14*/
const Device UA2516056 = {S21840, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.14*/
const Device UA2516386 = {S21841, 57, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.14*/
const Device UA2516588 = {S21842, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.14*/
const Device UA2530745 = {S21843, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03658)) Presto/2.12.388 Version/12.14*/
const Device UA2530810 = {S21844, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.14*/
const Device UA2530828 = {S21845, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.14*/
const Device UA2531051 = {S21846, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.14*/
const Device UA2532320 = {S21847, 74, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.14 Yandex.Translate*/
const Device UA2534830 = {S21848, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.14*/
const Device UA2535229 = {S21849, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.14*/
const Device UA2535288 = {S21850, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03789)) Presto/2.12.388 Version/12.14*/
const Device UA2536099 = {S21851, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6044)) Presto/2.12.388 Version/12.14*/
const Device UA2536156 = {S21852, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.14*/
const Device UA2536647 = {S21853, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.14*/
const Device UA2536764 = {S21854, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.14*/
const Device UA2536859 = {S21855, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.14*/
const Device UA2536860 = {S21856, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.14*/
const Device UA2536968 = {S21857, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.14*/
const Device UA2537279 = {S21858, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.14*/
const Device UA2537280 = {S21859, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.14*/
const Device UA2537283 = {S21860, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.14*/
const Device UA2537577 = {S21861, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.14*/
const Device UA2537656 = {S21862, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.14*/
const Device UA2537693 = {S21863, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.14*/
const Device UA2537730 = {S21864, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.14*/
const Device UA2537761 = {S21865, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.14*/
const Device UA2537773 = {S21866, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4933)) Presto/2.12.388 Version/12.14*/
const Device UA2537788 = {S21867, 106, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.14*/
const Device UA2537974 = {S21868, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.14*/
const Device UA2539407 = {S21869, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.14*/
const Device UA2539412 = {S21870, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.14*/
const Device UA2558314 = {S21871, 107, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03402); MRSPUTNIK OW 2, 3, 0, 104) Presto/2.12.388 Version/12.14*/
const Device UA2558360 = {S21872, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6035)) Presto/2.12.388 Version/12.14*/
const Device UA2558409 = {S21873, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.14*/
const Device UA2558531 = {S21874, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5278)) Presto/2.12.388 Version/12.14*/
const Device UA2558535 = {S21875, 107, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03686); MRSPUTNIK OW 2, 3, 0, 244) Presto/2.12.388 Version/12.14*/
const Device UA2558747 = {S21876, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5230)) Presto/2.12.388 Version/12.14*/
const Device UA2559075 = {S21877, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.14*/
const Device UA2559129 = {S21878, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.14*/
const Device UA2559602 = {S21879, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.14*/
const Device UA2559757 = {S21880, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.14*/
const Device UA2560774 = {S21881, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5288)) Presto/2.12.388 Version/12.14*/
const Device UA2560783 = {S21882, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03402)) Presto/2.12.388 Version/12.14*/
const Device UA2562822 = {S21883, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.14*/
const Device UA2563845 = {S21884, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; Edition Yx) Presto/2.12.388 Version/12.14,gzip(gfe)*/
const Device UA2568184 = {S21885, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.14*/
const Device UA2568389 = {S21886, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.14*/
const Device UA2570694 = {S21887, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6032)) Presto/2.12.388 Version/12.14*/
const Device UA2580653 = {S21888, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5650)) Presto/2.12.388 Version/12.14*/
const Device UA2580654 = {S21889, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.14*/
const Device UA2581196 = {S21890, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.3 (build 02552)) Presto/2.12.388 Version/12.14*/
const Device UA2581373 = {S21891, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03639)) Presto/2.12.388 Version/12.14*/
const Device UA2581763 = {S21892, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5299)) Presto/2.12.388 Version/12.14*/
const Device UA2581936 = {S21893, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4110)) Presto/2.12.388 Version/12.14*/
const Device UA2582200 = {S21894, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5336)) Presto/2.12.388 Version/12.14*/
const Device UA2584490 = {S21895, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5996)) Presto/2.12.388 Version/12.14*/
const Device UA2585909 = {S21896, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6080)) Presto/2.12.388 Version/12.14*/
const Device UA2585985 = {S21897, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.14*/
const Device UA2597056 = {S21898, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4661)) Presto/2.12.388 Version/12.14*/
const Device UA2598260 = {S21899, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5734)) Presto/2.12.388 Version/12.14*/
const Device UA2599340 = {S21900, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4832)) Presto/2.12.388 Version/12.14*/
const Device UA2599811 = {S21901, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03672)) Presto/2.12.388 Version/12.14*/
const Device UA2600581 = {S21902, 107, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03757); MRSPUTNIK OW 2, 3, 0, 288) Presto/2.12.388 Version/12.14*/
const Device UA2601476 = {S21903, 96, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.14,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2603055 = {S21904, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5660)) Presto/2.12.388 Version/12.14*/
const Device UA2603075 = {S21905, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5217)) Presto/2.12.388 Version/12.14*/
const Device UA2603426 = {S21906, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03755)) Presto/2.12.388 Version/12.14*/
const Device UA2603437 = {S21907, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03791)) Presto/2.12.388 Version/12.14*/
const Device UA2606402 = {S21908, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.14*/
const Device UA2661038 = {S21909, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5325)) Presto/2.12.388 Version/12.14*/
const Device UA2661141 = {S21910, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5300)) Presto/2.12.388 Version/12.14*/
const Device UA2662299 = {S21911, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.3 (build 02560)) Presto/2.12.388 Version/12.14*/
const Device UA2662759 = {S21912, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.3 (build 02564)) Presto/2.12.388 Version/12.14*/
const Device UA2672750 = {S21913, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.14*/
const Device UA2674266 = {S21914, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.14*/
const Device UA2702201 = {S21915, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5161)) Presto/2.12.388 Version/12.14*/
const Device UA2712617 = {S21916, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.14*/
const Device UA2730145 = {S21917, 107, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02842); MRSPUTNIK OW 2, 3, 0, 104) Presto/2.12.388 Version/12.14*/
const Device UA2737640 = {S21918, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.14*/
const Device UA2748641 = {S21919, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03649)) Presto/2.12.388 Version/12.14*/
const Device UA2749268 = {S21920, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02830)) Presto/2.12.388 Version/12.14*/
const Device UA2756735 = {S21921, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.14*/
const Device UA2772521 = {S21922, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6561)) Presto/2.12.388 Version/12.14*/
const Device UA2779389 = {S21923, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02743)) Presto/2.12.388 Version/12.14*/
const Device UA2792663 = {S21924, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03638)) Presto/2.12.388 Version/12.14*/
const Device UA2852683 = {S21925, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02746)) Presto/2.12.388 Version/12.14*/
const Device UA2855489 = {S21926, 107, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03399); MRSPUTNIK OW 2, 3, 0, 104) Presto/2.12.388 Version/12.14*/
const Device UA2859057 = {S21927, 80, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03653)) Presto/2.12.388 Version/12.14*/
const Device UA2867243 = {S21928, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6573)) Presto/2.12.388 Version/12.14*/
const Device UA2886752 = {S21929, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.14*/
const Device UA2934830 = {S21930, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5988)) Presto/2.12.388 Version/12.14*/
const Device UA2944453 = {S21931, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6657)) Presto/2.12.388 Version/12.14*/
const Device UA2958046 = {S21932, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6283)) Presto/2.12.388 Version/12.14*/
const Device UA2958599 = {S21933, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6638)) Presto/2.12.388 Version/12.14*/
const Device UA3017517 = {S21934, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6341)) Presto/2.12.388 Version/12.14*/
const Device UA3021524 = {S21935, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6673)) Presto/2.12.388 Version/12.14*/
const Device UA3024110 = {S21936, 77, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.14,gzip(gfe),gzip(gfe)*/
const Device UA3050189 = {S21937, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6628)) Presto/2.12.388 Version/12.14*/
const Device UA3056539 = {S21938, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6327)) Presto/2.12.388 Version/12.14*/
const Device UA3062127 = {S21939, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.14*/
const Device UA3077527 = {S21940, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6337)) Presto/2.12.388 Version/12.14*/
const Device UA3100182 = {S21941, 107, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03796); MRSPUTNIK OW 2, 3, 0, 104) Presto/2.12.388 Version/12.14*/
const Device UA3106332 = {S21942, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6648)) Presto/2.12.388 Version/12.14*/
const Device UA3108065 = {S21943, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6675)) Presto/2.12.388 Version/12.14*/
const Device UA3108104 = {S21944, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6681)) Presto/2.12.388 Version/12.14*/
const Device UA3108292 = {S21945, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6347)) Presto/2.12.388 Version/12.14*/
const Device UA3126738 = {S21946, 79, { &P15364,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.2 (build 7166)) Presto/2.12.388 Version/12.14*/
const Device UA444637 = {S21947, 75, { &P15364,&P18110,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.5) Gecko/X iceweasel/3.0.5*/
const Device UA449503 = {S21948, 93, { &P15364,&P18110,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i586; en-US; rv:1.9.0.1) Gecko/X OLPC/Firefox-6 (XO) Firefox/3.0.1*/
const Device UA459863 = {S21949, 88, { &P15364,&P18110,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9) Gecko/X Iceweasel/3.0 (Debian-3.0~rc2-2)*/
const Device UA491329 = {S21950, 92, { &P15364,&P18110,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; pl-PL; rv:1.9.1.3) Gecko/X Iceweasel/3.5.3 (Debian-3.5.3-1)*/
const Device UA502561 = {S21951, 88, { &P15364,&P18110,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; tr-TR; rv:1.9b5) Gecko/X SUSE/2.9.95-25.1 Firefox/3.0b5*/
const Device UA646022 = {S21952, 91, { &P15364,&P18110,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.16) Gecko/X firefox/3.0.6 (Debian-3.0.6-3)*/
const Device UA721940 = {S21953, 95, { &P15364,&P18110,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.8.1.1) Gecko/X Firefox/3.0 (Debian-2.0.0.1+dfsg-2)*/
const Device UA776435 = {S21954, 95, { &P15364,&P18110,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.2) Gecko/X Iceweasel/3.5.2 (Zenwalk GNU Linux)*/
const Device UA844328 = {S21955, 93, { &P15364,&P18110,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.14) Gecko/X Firefox/3.0.14 (Debian-3.0.14-1)*/
const Device UA877165 = {S21956, 93, { &P15364,&P18110,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.0.16) Gecko/X Firefox/3.0.6 (Windows NT 5.1)*/
const Device UA913469 = {S21957, 78, { &P15364,&P18110,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.11) Gecko GranParadiso/3.0.11*/
const Device UA2288860 = {S21958, 88, { &P15364,&P18110,&P3409,&P18092 }};	/*Useragent - mozilla/5.0 (x11; u; linux i686; sv-se; rv:1.9.0.3) gecko/X suse/3.0.3-1.1 firefox/3.0.3*/
const Device UA2311003 = {S21959, 73, { &P15364,&P18110,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux ppc; en-US; rv:1.9a8) Gecko/X GranParadiso/3.1*/
const Device UA2681753 = {S21960, 80, { &P15364,&P18110,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.0.8) Gecko/X GranParadiso/3.0.8*/
const Device UA2742146 = {S21961, 84, { &P15364,&P18110,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.0.5) Gecko/X Gentoo Iceweasel/3.0.5*/
const Device UA3109239 = {S21962, 59, { &P18088,&P17017,&P18549,&P18092 }};	/*Useragent - Opera/9.50 (J2ME/MIDP; Opera Mini/4.5.33867/29.3757; U; en)*/
const Device UA3115160 = {S21963, 88, { &P18088,&P17017,&P18549,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/4.5.33867/31.1475; U; en) Presto/2.8.119 Version/11.10*/
const Device UA20217 = {S21964, 151, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonSK17i Build/4.0.A.0.263) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46965 = {S21965, 151, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonSK17i Build/4.0.A.2.335) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46976 = {S21966, 151, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonSK17i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46992 = {S21967, 152, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonSK17iv Build/4.0.A.2.335) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46993 = {S21968, 139, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SK17i Build/4.0.A.2.335) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47008 = {S21969, 151, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonSK17i Build/4.0.A.2.364) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47045 = {S21970, 153, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonSK17i-o Build/4.0.A.2.335) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47047 = {S21971, 139, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SK17i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60094 = {S21972, 151, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonSK17i Build/4.0.A.2.204) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70243 = {S21973, 153, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonSK17i-o Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70818 = {S21974, 151, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonSK17i Build/4.0.A.2.377) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73384 = {S21975, 139, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SK17i Build/4.0.A.2.377) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76089 = {S21976, 140, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SK17i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA378452 = {S21977, 63, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SK17i Build/4.0.A.2.335)*/
const Device UA458431 = {S21978, 152, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonSK17i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA576614 = {S21979, 152, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonSK17i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623666 = {S21980, 154, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; SonyEricssonSK17i-o Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA624246 = {S21981, 152, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; SonyEricssonSK17a Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625061 = {S21982, 140, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SK17i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625216 = {S21983, 152, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonSK17i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA628140 = {S21984, 64, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SK17i Build/4.0.2.A.0.62)*/
const Device UA629244 = {S21985, 152, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonSK17i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633660 = {S21986, 152, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonSK17a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA653817 = {S21987, 151, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-ar; SonyEricssonSK17a Build/4.0.A.2.335) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA662783 = {S21988, 154, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonSK17i-o Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA665443 = {S21989, 153, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonSK17iv Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA669890 = {S21990, 140, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SK17i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA703381 = {S21991, 154, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonSK17i-o Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA708427 = {S21992, 153, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; SonyEricssonSK17iv Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA711374 = {S21993, 63, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SK17a Build/4.0.A.2.335)*/
const Device UA728579 = {S21994, 140, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SK17i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA745137 = {S21995, 140, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SK17a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA761634 = {S21996, 64, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SK17a Build/4.0.2.A.0.58)*/
const Device UA802572 = {S21997, 140, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; SK17a Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1938540 = {S21998, 139, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-ar; SK17a Build/4.0.A.2.335) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1947321 = {S21999, 128, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; SK17i Build/4.0.2.A.0.62) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2021161 = {S22000, 153, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-pt; SonyEricssonSK17iv Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023731 = {S22001, 128, { &P15047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; SK17i Build/4.0.2.A.0.58) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA111898 = {S22002, 62, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.6.30 Version/10.60*/
const Device UA126209 = {S22003, 62, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.6.30 Version/10.61*/
const Device UA133153 = {S22004, 62, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.6.30 Version/10.62*/
const Device UA133377 = {S22005, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.6.30 Version/10.61*/
const Device UA144160 = {S22006, 62, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.6.30 Version/10.63*/
const Device UA145176 = {S22007, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.6.30 Version/10.63*/
const Device UA148905 = {S22008, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.6.30 Version/10.63*/
const Device UA152198 = {S22009, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.6.30 Version/10.63*/
const Device UA156014 = {S22010, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.6.30 Version/10.63*/
const Device UA157721 = {S22011, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.6.30 Version/10.63*/
const Device UA160456 = {S22012, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.6.30 Version/10.63*/
const Device UA168699 = {S22013, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.6.30 Version/10.63*/
const Device UA267665 = {S22014, 62, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; en) Presto/2.6.30 Version/10.60*/
const Device UA277463 = {S22015, 62, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; en) Presto/2.6.30 Version/10.62*/
const Device UA277961 = {S22016, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.6.30 Version/10.62*/
const Device UA279422 = {S22017, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.6.30 Version/10.62*/
const Device UA282951 = {S22018, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.6.30 Version/10.61*/
const Device UA292634 = {S22019, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.6.30 Version/10.63*/
const Device UA292973 = {S22020, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.6.30 Version/10.63*/
const Device UA297923 = {S22021, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.6.30 Version/10.63*/
const Device UA300769 = {S22022, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.6.30 Version/10.63*/
const Device UA391599 = {S22023, 112, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); MRSPUTNIK OW 2, 3, 0, 104; ru) Presto/2.6.30 Version/10.63*/
const Device UA400800 = {S22024, 62, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; ru) Presto/2.6.30 Version/10.61*/
const Device UA414434 = {S22025, 62, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; en) Presto/2.6.30 Version/10.63*/
const Device UA415315 = {S22026, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.6.30 Version/10.63*/
const Device UA416079 = {S22027, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.6.30 Version/10.62*/
const Device UA442314 = {S22028, 99, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.6.30 Version/10.62,gzip(gfe) (via translate.google.com)*/
const Device UA444100 = {S22029, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03638); ru) Presto/2.6.30 Version/10.63*/
const Device UA446292 = {S22030, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.6.30 Version/10.61*/
const Device UA463690 = {S22031, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.6.30 Version/10.60*/
const Device UA470439 = {S22032, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.6.30 Version/10.61*/
const Device UA494090 = {S22033, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.6.30 Version/10.63*/
const Device UA513245 = {S22034, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.6.30 Version/10.60*/
const Device UA522482 = {S22035, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03672); ru) Presto/2.6.30 Version/10.63*/
const Device UA529163 = {S22036, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.6.30 Version/10.61*/
const Device UA529377 = {S22037, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.6.30 Version/10.60*/
const Device UA556300 = {S22038, 84, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.6.30 Version/10.63*/
const Device UA565461 = {S22039, 84, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.6.30 Version/10.63*/
const Device UA693788 = {S22040, 84, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.6.30 Version/10.63*/
const Device UA750702 = {S22041, 84, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.6.30 Version/10.63*/
const Device UA763353 = {S22042, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.6.30 Version/10.61*/
const Device UA764998 = {S22043, 66, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/10.60 (Windows NT 5.1; U; en-US) Presto/2.6.30 Version/10.60*/
const Device UA860100 = {S22044, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.6.30 Version/10.62*/
const Device UA862112 = {S22045, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.6.30 Version/10.63*/
const Device UA865124 = {S22046, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.6.30 Version/10.62*/
const Device UA1842582 = {S22047, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.6.30 Version/10.60*/
const Device UA1854565 = {S22048, 99, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; en) Presto/2.6.30 Version/10.63,gzip(gfe) (via translate.google.com)*/
const Device UA1871801 = {S22049, 47, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/6.30 (Windows NT 5.1; U; pl) Version/10.6*/
const Device UA1873291 = {S22050, 99, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; en) Presto/2.6.30 Version/10.60,gzip(gfe) (via translate.google.com)*/
const Device UA1890814 = {S22051, 84, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.6.30 Version/10.63*/
const Device UA1899573 = {S22052, 47, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/6.40 (Windows NT 5.1; U; pl) Version/10.6*/
const Device UA1941838 = {S22053, 84, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.6.30 Version/10.60*/
const Device UA1945090 = {S22054, 84, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.6.30 Version/10.61*/
const Device UA2146805 = {S22055, 84, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.6.30 Version/10.61*/
const Device UA2148780 = {S22056, 47, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/6.10 (Windows NT 5.1; U; pl) Version/10.6*/
const Device UA2151767 = {S22057, 84, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.6.30 Version/10.62*/
const Device UA2208773 = {S22058, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.6.30 Version/10.61*/
const Device UA2209439 = {S22059, 47, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/6.20 (Windows NT 5.1; U; pl) Version/10.6*/
const Device UA2219063 = {S22060, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.6.30 Version/10.63*/
const Device UA2297071 = {S22061, 84, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.6.30 Version/10.63*/
const Device UA2334228 = {S22062, 84, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.6.30 Version/10.63*/
const Device UA2410890 = {S22063, 84, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.6.30 Version/10.62*/
const Device UA2419721 = {S22064, 84, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.6.30 Version/10.63*/
const Device UA2455850 = {S22065, 72, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.6.30 Version/10.63,gzip(gfe)*/
const Device UA2518562 = {S22066, 85, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.6.30 Version/10.62*/
const Device UA2523806 = {S22067, 84, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.6.30 Version/10.61*/
const Device UA2684605 = {S22068, 84, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.6.30 Version/10.63*/
const Device UA2688239 = {S22069, 84, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.6.30 Version/10.63*/
const Device UA2982786 = {S22070, 72, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; en) Presto/2.6.30 Version/10.60,gzip(gfe)*/
const Device UA3086953 = {S22071, 84, { &P15364,&P5690,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.6.30 Version/10.63*/
const Device UA3092272 = {S22072, 174, { &P15364,&P5690,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1568.2 Safari/537.36 AppEngine-Google; (+http://code.google.com/appengine; appid: s~unalin001)*/
const Device UA3115646 = {S22073, 101, { &P15364,&P5690,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3129164 = {S22074, 111, { &P15364,&P5690,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36,gzip(gfe)*/
const Device UA3129180 = {S22075, 118, { &P15364,&P5690,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Maxthon/4.2.0.800 Chrome/30.0.1551.0 Safari/537.36*/
const Device UA3130150 = {S22076, 108, { &P15364,&P5690,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3131113 = {S22077, 101, { &P15364,&P5690,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3132747 = {S22078, 140, { &P15364,&P5690,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2359222 = {S22079, 57, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.11*/
const Device UA2367471 = {S22080, 64, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; WOW64) Presto/2.12.388 Version/12.11*/
const Device UA2367743 = {S22081, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.11*/
const Device UA2367874 = {S22082, 69, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; Edition Yx) Presto/2.12.388 Version/12.11*/
const Device UA2367914 = {S22083, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.11*/
const Device UA2367916 = {S22084, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.11*/
const Device UA2367917 = {S22085, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.11*/
const Device UA2367924 = {S22086, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.11*/
const Device UA2367940 = {S22087, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.11*/
const Device UA2368288 = {S22088, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.11*/
const Device UA2368396 = {S22089, 62, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1) Opera 12.11*/
const Device UA2368576 = {S22090, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.11*/
const Device UA2368582 = {S22091, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.11*/
const Device UA2368584 = {S22092, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.11*/
const Device UA2368586 = {S22093, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.11*/
const Device UA2368603 = {S22094, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.11*/
const Device UA2368767 = {S22095, 57, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2) Presto/2.12.388 Version/12.11*/
const Device UA2368885 = {S22096, 68, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:2.0) Gecko/X Firefox/4.0 Opera 12.11*/
const Device UA2369691 = {S22097, 67, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.11,gzip(gfe)*/
const Device UA2369733 = {S22098, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.11*/
const Device UA2369736 = {S22099, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.11*/
const Device UA2387100 = {S22100, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.11*/
const Device UA2387208 = {S22101, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.11*/
const Device UA2387340 = {S22102, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.11*/
const Device UA2387342 = {S22103, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.11*/
const Device UA2387344 = {S22104, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.11*/
const Device UA2387528 = {S22105, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.11*/
const Device UA2387533 = {S22106, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.11*/
const Device UA2387880 = {S22107, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4933)) Presto/2.12.388 Version/12.11*/
const Device UA2387888 = {S22108, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.11*/
const Device UA2388151 = {S22109, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.11*/
const Device UA2388293 = {S22110, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.11*/
const Device UA2388306 = {S22111, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.11*/
const Device UA2388311 = {S22112, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.11*/
const Device UA2388499 = {S22113, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.11*/
const Device UA2388500 = {S22114, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.11*/
const Device UA2388502 = {S22115, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.11*/
const Device UA2388516 = {S22116, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.11*/
const Device UA2389464 = {S22117, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.11*/
const Device UA2389473 = {S22118, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.11*/
const Device UA2390381 = {S22119, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.11*/
const Device UA2390389 = {S22120, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.11*/
const Device UA2390393 = {S22121, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.11*/
const Device UA2390397 = {S22122, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.11*/
const Device UA2390402 = {S22123, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.11*/
const Device UA2390864 = {S22124, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.11*/
const Device UA2391483 = {S22125, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.11*/
const Device UA2404523 = {S22126, 74, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.11 Yandex.Translate*/
const Device UA2404588 = {S22127, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.11*/
const Device UA2404591 = {S22128, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.11*/
const Device UA2405091 = {S22129, 71, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.11 WebWasher 3.4*/
const Device UA2405094 = {S22130, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.11*/
const Device UA2405138 = {S22131, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.11*/
const Device UA2405394 = {S22132, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03658)) Presto/2.12.388 Version/12.11*/
const Device UA2405516 = {S22133, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.11*/
const Device UA2405621 = {S22134, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.11*/
const Device UA2405622 = {S22135, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.11*/
const Device UA2405627 = {S22136, 106, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.11*/
const Device UA2405629 = {S22137, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5288)) Presto/2.12.388 Version/12.11*/
const Device UA2405678 = {S22138, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.11*/
const Device UA2405682 = {S22139, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.11*/
const Device UA2405794 = {S22140, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.11*/
const Device UA2405838 = {S22141, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.11*/
const Device UA2405839 = {S22142, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.11*/
const Device UA2405872 = {S22143, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.11*/
const Device UA2405885 = {S22144, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; Edition Yx) Presto/2.12.388 Version/12.11,gzip(gfe)*/
const Device UA2406622 = {S22145, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03789)) Presto/2.12.388 Version/12.11*/
const Device UA2422130 = {S22146, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03402)) Presto/2.12.388 Version/12.11*/
const Device UA2425286 = {S22147, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5299)) Presto/2.12.388 Version/12.11*/
const Device UA2425415 = {S22148, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03639)) Presto/2.12.388 Version/12.11*/
const Device UA2426735 = {S22149, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.11*/
const Device UA2427777 = {S22150, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5230)) Presto/2.12.388 Version/12.11*/
const Device UA2427783 = {S22151, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.11*/
const Device UA2427784 = {S22152, 107, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03686); MRSPUTNIK OW 2, 3, 0, 244) Presto/2.12.388 Version/12.11*/
const Device UA2428024 = {S22153, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5660)) Presto/2.12.388 Version/12.11*/
const Device UA2428318 = {S22154, 107, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03402); MRSPUTNIK OW 2, 3, 0, 104) Presto/2.12.388 Version/12.11*/
const Device UA2429963 = {S22155, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.11*/
const Device UA2430048 = {S22156, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.11*/
const Device UA2431868 = {S22157, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.3 (build 02552)) Presto/2.12.388 Version/12.11*/
const Device UA2437652 = {S22158, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.11*/
const Device UA2457074 = {S22159, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5336)) Presto/2.12.388 Version/12.11*/
const Device UA2457166 = {S22160, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5650)) Presto/2.12.388 Version/12.11*/
const Device UA2457313 = {S22161, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4832)) Presto/2.12.388 Version/12.11*/
const Device UA2466975 = {S22162, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5278)) Presto/2.12.388 Version/12.11*/
const Device UA2467010 = {S22163, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5217)) Presto/2.12.388 Version/12.11*/
const Device UA2469311 = {S22164, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.11*/
const Device UA2482353 = {S22165, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5734)) Presto/2.12.388 Version/12.11*/
const Device UA2482745 = {S22166, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4110)) Presto/2.12.388 Version/12.11*/
const Device UA2482915 = {S22167, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5300)) Presto/2.12.388 Version/12.11*/
const Device UA2505804 = {S22168, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03672)) Presto/2.12.388 Version/12.11*/
const Device UA2534942 = {S22169, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.11*/
const Device UA2561579 = {S22170, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.11*/
const Device UA2566356 = {S22171, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.11*/
const Device UA2571607 = {S22172, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.11*/
const Device UA2581545 = {S22173, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.11*/
const Device UA2595057 = {S22174, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5325)) Presto/2.12.388 Version/12.11*/
const Device UA2595179 = {S22175, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4661)) Presto/2.12.388 Version/12.11*/
const Device UA2602001 = {S22176, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.11*/
const Device UA2603744 = {S22177, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.11*/
const Device UA2653762 = {S22178, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03755)) Presto/2.12.388 Version/12.11*/
const Device UA2654388 = {S22179, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03791)) Presto/2.12.388 Version/12.11*/
const Device UA2655284 = {S22180, 96, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2655376 = {S22181, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03638)) Presto/2.12.388 Version/12.11*/
const Device UA2659507 = {S22182, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.11*/
const Device UA2669089 = {S22183, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.11*/
const Device UA2706006 = {S22184, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.11*/
const Device UA2707828 = {S22185, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.11*/
const Device UA2708821 = {S22186, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.11*/
const Device UA2734383 = {S22187, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.11*/
const Device UA2768719 = {S22188, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.11*/
const Device UA2770096 = {S22189, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.11*/
const Device UA2790524 = {S22190, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.3 (build 02564)) Presto/2.12.388 Version/12.11*/
const Device UA2843114 = {S22191, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.4 (build 02625)) Presto/2.12.388 Version/12.11*/
const Device UA2845478 = {S22192, 80, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03780)) Presto/2.12.388 Version/12.11*/
const Device UA2851459 = {S22193, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6035)) Presto/2.12.388 Version/12.11*/
const Device UA2952711 = {S22194, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.11*/
const Device UA2953617 = {S22195, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6080)) Presto/2.12.388 Version/12.11*/
const Device UA2986286 = {S22196, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.11*/
const Device UA3003614 = {S22197, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.11*/
const Device UA3020451 = {S22198, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6673)) Presto/2.12.388 Version/12.11*/
const Device UA3087940 = {S22199, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.11*/
const Device UA3092603 = {S22200, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.11*/
const Device UA3103389 = {S22201, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6638)) Presto/2.12.388 Version/12.11*/
const Device UA3103811 = {S22202, 79, { &P15364,&P5690,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6573)) Presto/2.12.388 Version/12.11*/
const Device UA106509 = {S22203, 62, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.6.30 Version/10.60*/
const Device UA119491 = {S22204, 62, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; en) Presto/2.6.30 Version/10.61*/
const Device UA134759 = {S22205, 62, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.6.30 Version/10.62*/
const Device UA143468 = {S22206, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.6.30 Version/10.62*/
const Device UA152229 = {S22207, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.6.30 Version/10.63*/
const Device UA153396 = {S22208, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.6.30 Version/10.63*/
const Device UA155476 = {S22209, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.6.30 Version/10.63*/
const Device UA162740 = {S22210, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03755); ru) Presto/2.6.30 Version/10.63*/
const Device UA167475 = {S22211, 72, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; YB/3.5.1; ru) Presto/2.6.30 Version/10.63*/
const Device UA295363 = {S22212, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.6.30 Version/10.63*/
const Device UA298906 = {S22213, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.6.30 Version/10.63*/
const Device UA302863 = {S22214, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.6.30 Version/10.62*/
const Device UA306301 = {S22215, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.6.30 Version/10.60*/
const Device UA433701 = {S22216, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.6.30 Version/10.62*/
const Device UA446537 = {S22217, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.6.30 Version/10.60*/
const Device UA475552 = {S22218, 99, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.6.30 Version/10.62,gzip(gfe) (via translate.google.com)*/
const Device UA490611 = {S22219, 99, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.6.30 Version/10.61,gzip(gfe) (via translate.google.com)*/
const Device UA496135 = {S22220, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03639); ru) Presto/2.6.30 Version/10.63*/
const Device UA504753 = {S22221, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.6.30 Version/10.63*/
const Device UA508262 = {S22222, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.6.30 Version/10.63*/
const Device UA529192 = {S22223, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.6.30 Version/10.60*/
const Device UA532780 = {S22224, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.6.30 Version/10.61*/
const Device UA562274 = {S22225, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.6.30 Version/10.63*/
const Device UA627104 = {S22226, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.6.30 Version/10.63*/
const Device UA653855 = {S22227, 99, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.6.30 Version/10.63,gzip(gfe) (via translate.google.com)*/
const Device UA655528 = {S22228, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.6.30 Version/10.61*/
const Device UA774934 = {S22229, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.6.30 Version/10.63*/
const Device UA809030 = {S22230, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.6.30 Version/10.63*/
const Device UA816174 = {S22231, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.6.30 Version/10.61*/
const Device UA1880344 = {S22232, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.6.30 Version/10.63*/
const Device UA1888026 = {S22233, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.6.30 Version/10.63*/
const Device UA1918069 = {S22234, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.6.30 Version/10.63*/
const Device UA1946578 = {S22235, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.6.30 Version/10.60*/
const Device UA1952751 = {S22236, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.6.30 Version/10.62*/
const Device UA1958378 = {S22237, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.6.30 Version/10.61*/
const Device UA2132844 = {S22238, 63, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/10.60 (Windows NT 6.1; U; ru) Presto/2.6.30 Version/10.60*/
const Device UA2143703 = {S22239, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.6.30 Version/10.63*/
const Device UA2156085 = {S22240, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.6.30 Version/10.63*/
const Device UA2206988 = {S22241, 99, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.6.30 Version/10.60,gzip(gfe) (via translate.google.com)*/
const Device UA2219207 = {S22242, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.6.30 Version/10.63*/
const Device UA2275081 = {S22243, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.6.30 Version/10.61*/
const Device UA2288377 = {S22244, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.6.30 Version/10.63*/
const Device UA2292672 = {S22245, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.6.30 Version/10.61*/
const Device UA2302171 = {S22246, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.6.30 Version/10.62*/
const Device UA2303632 = {S22247, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.6.30 Version/10.60*/
const Device UA2323418 = {S22248, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.6.30 Version/10.61*/
const Device UA2337895 = {S22249, 93, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; en) Presto/2.6.30 Version/10.60 CSVT/1V8DCX7NLW-4LW48EQOX(ENU)*/
const Device UA2340918 = {S22250, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.6.30 Version/10.63*/
const Device UA2555039 = {S22251, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.6.30 Version/10.63*/
const Device UA2592852 = {S22252, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.6.30 Version/10.63*/
const Device UA2594303 = {S22253, 72, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.6.30 Version/10.63,gzip(gfe)*/
const Device UA2642135 = {S22254, 85, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.6.30 Version/10.62*/
const Device UA2725094 = {S22255, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.6.30 Version/10.62*/
const Device UA2777830 = {S22256, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.6.30 Version/10.63*/
const Device UA2809070 = {S22257, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.6.30 Version/10.63*/
const Device UA2811917 = {S22258, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.6.30 Version/10.63*/
const Device UA2831394 = {S22259, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.6.30 Version/10.61*/
const Device UA2835362 = {S22260, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.6.30 Version/10.63*/
const Device UA2837981 = {S22261, 84, { &P15364,&P7133,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.6.30 Version/10.62*/
const Device UA2203735 = {S22262, 106, { &P15364,&P16110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203779 = {S22263, 99, { &P15364,&P16110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2231135 = {S22264, 143, { &P15364,&P16110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe) (via translate.google.com)*/
const Device UA2234471 = {S22265, 109, { &P15364,&P16110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)*/
const Device UA2259293 = {S22266, 116, { &P15364,&P16110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)*/
const Device UA2534065 = {S22267, 130, { &P15364,&P16110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (gtsr) Safari/537.4*/
const Device UA2534066 = {S22268, 137, { &P15364,&P16110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (gtsr) Safari/537.4*/
const Device UA2534901 = {S22269, 137, { &P15364,&P16110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (fnrb) Safari/537.4*/
const Device UA2535464 = {S22270, 130, { &P15364,&P16110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.511 YaBrowser/1.5.1105.511 (fnrb) Safari/537.4*/
const Device UA2568097 = {S22271, 137, { &P15364,&P16110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.520 YaBrowser/1.5.1105.520 (bskt) Safari/537.4*/
const Device UA2603780 = {S22272, 130, { &P15364,&P16110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1105.520 YaBrowser/1.5.1105.520 (bskt) Safari/537.4*/
const Device UA2718789 = {S22273, 107, { &P15364,&P16110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.1.9999.433 Safari/537.4*/
const Device UA2737693 = {S22274, 100, { &P15364,&P16110,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.1.9999.433 Safari/537.4*/
const Device UA102823 = {S22275, 60, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - FyberSpider/1.3 (http://www.fybersearch.com/fyberspider.php)*/
const Device UA127374 = {S22276, 78, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - SpokeSpider/1.0 (http://support.spoke.com/webspider/) Mozilla/5.0 (not really)*/
const Device UA197012 = {S22277, 54, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - OpenWebSpider v0.1.2.B (http://www.openwebspider.org/)*/
const Device UA233395 = {S22278, 78, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - Mozilla/5.0 (compatible; WangIDSpider/1.0; +http://www.wangid.com/spider.html)*/
const Device UA255977 = {S22279, 73, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - Mozilla/5.0 (compatible; JikeSpider; +http://shoulu.jike.com/spider.html)*/
const Device UA270817 = {S22280, 55, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - Mozilla/4.0 (compatible; BOTW Spider; +http://botw.org)*/
const Device UA300188 = {S22281, 86, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - Mozilla/5.0 (compatible;WI Job Roboter Spider Version 3;+http://www.webintegration.at)*/
const Device UA306058 = {S22282, 66, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - DRKSpider - Website link validator - http://www.drk.com.ar/spider/*/
const Device UA355481 = {S22283, 73, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - Mozilla/5.0 (compatible; YRSpider; +http://www.yunrang.com/yrspider.html)*/
const Device UA373453 = {S22284, 52, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - OpenWebSpider v0.1.4 (http://www.openwebspider.org/)*/
const Device UA416010 = {S22285, 87, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - http://www.activesearchresults.com/addwebsite.php ASR Ranking Technology/Spider/Crawler*/
const Device UA416740 = {S22286, 63, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - SupremeSearch.net Spider-Bot/2.0 (http://www.supremesearch.net)*/
const Device UA452935 = {S22287, 16, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - OrbitSpiderBot/1*/
const Device UA912670 = {S22288, 93, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - Mozilla/5.0 (compatible; OpenindexSpider; +http://www.openindex.io/en/webmasters/spider.html)*/
const Device UA914773 = {S22289, 85, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1; Sycara +http://www.sycara.com/SpiderInfo.php)*/
const Device UA2229032 = {S22290, 47, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - Wspider-1.0;http://www.w3456.com/ PrimarySpider*/
const Device UA2368136 = {S22291, 72, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - Mozilla/5.0 (compatible; EasouSpider; +http://www.easou.com/spider.html)*/
const Device UA2404163 = {S22292, 22, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - Content Crawler Spider*/
const Device UA2408276 = {S22293, 79, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - Mozilla/5.0 (compatible; EasouSpider; +http://www.easou.com/search/spider.html)*/
const Device UA2431873 = {S22294, 77, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - Mozilla/5.0 (compatible; SecMindSpider; +http://securemymind.com/spider.html)*/
const Device UA2433552 = {S22295, 81, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - Mozilla/5.0 (compatible; SecMindSpider; +http://www.securemymind.com/spider.html)*/
const Device UA2461225 = {S22296, 75, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - Mozilla/5.0 (compatible; SouthWestSpider; +http://dianlake.com/spider.html)*/
const Device UA2461226 = {S22297, 84, { &P18254,&P7480,&P18245,&P19831 }};	/*Useragent - Mozilla/5.0 (compatible; GreatChinaSpider; +http://www.learnatchina.com/spider.html)*/
const Device UA32842 = {S22298, 63, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; rv:2.2) Gecko/20110201*/
const Device UA162117 = {S22299, 47, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - QuickTime/7.6.9 (qtver=7.6.9;os=Windows NT 6.1)*/
const Device UA170980 = {S22300, 59, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; SuperCleaner 2.96; Windows NT 6.1)*/
const Device UA216227 = {S22301, 57, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - QuickTime/7.7 (qtver=7.7;os=Windows NT 6.1Service Pack 1)*/
const Device UA243035 = {S22302, 142, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE/9.0; Windows NT 6.1; Trident/5.0; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.1.4322)*/
const Device UA262838 = {S22303, 47, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US)*/
const Device UA297378 = {S22304, 50, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv*/
const Device UA351176 = {S22305, 162, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; ArgClrInt/1.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA366310 = {S22306, 89, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534+ (KHTML, like Gecko) BingPreview/1.0b*/
const Device UA366314 = {S22307, 162, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; ArgClrInt/1.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET4.0C; .NET4.0E; .NET CLR 3.5.30729; .NET CLR 3.0.30729)*/
const Device UA437930 = {S22308, 41, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows 7 6.1) Java/1.6.0_31*/
const Device UA554339 = {S22309, 87, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Java 1.6.0_17; Windows 7 6.1 amd64; en_US) ICEbrowser/v6_0_2 Java/1.6.0_17*/
const Device UA705584 = {S22310, 57, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; fr; rv:1.8.1.20)*/
const Device UA705705 = {S22311, 56, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; fr; rv:1.9.0.6)*/
const Device UA729254 = {S22312, 84, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; zh-CN) AppleWebKit/533+ (KHTML, like Gecko)*/
const Device UA743315 = {S22313, 40, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1)*/
const Device UA767229 = {S22314, 28, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - InetURL/1.0 (Windows NT 6.1)*/
const Device UA805664 = {S22315, 61, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Custo 3 (Netwu.com); Windows NT 6.1)*/
const Device UA830801 = {S22316, 51, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - IBM-LC-Base-Notes_8.5.3.X_0921-Windows 7 Java/1.6.0*/
const Device UA840846 = {S22317, 76, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko)*/
const Device UA881316 = {S22318, 69, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.57.2 (KHTML, like Gecko)*/
const Device UA1979270 = {S22319, 128, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534+ (KHTML, like Gecko) BingPreview/1.0b,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA1996413 = {S22320, 57, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.0) Gecko/ /12.0*/
const Device UA2042714 = {S22321, 41, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows 7 6.1) Java/1.7.0_07*/
const Device UA2152326 = {S22322, 107, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.9 (KHTML, like Gecko) Boxee/1.0.1 bxapi/7.1*/
const Device UA2202911 = {S22323, 48, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1) Gecko/X*/
const Device UA2473236 = {S22324, 28, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1)*/
const Device UA2519863 = {S22325, 88, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534+ (KHTML, like Gecko) OpenWebKitSharp*/
const Device UA2742259 = {S22326, 68, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - WinM RSS Reader/1.2 (Windows; U; Windows NT 6.1; en-GB; rv:1.9.2.13)*/
const Device UA2774170 = {S22327, 46, { &P15364,&P7133,&P17470,&P18092 }};	/*Useragent - User-Agent:Mozilla/5.0 (Windows NT 6.1; WOW64)*/
const Device UA1181 = {S22328, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-my) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA1184 = {S22329, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-in) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA4074 = {S22330, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; pt-br) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA4081 = {S22331, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; el-gr) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA12124 = {S22332, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; da-dk) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA12125 = {S22333, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-no) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA12136 = {S22334, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; fr-fr) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA12139 = {S22335, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; cs-cz) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA17087 = {S22336, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-gb) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA17093 = {S22337, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; nl-nl) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA17097 = {S22338, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; de-de) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA17120 = {S22339, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-sg) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA23541 = {S22340, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; sv-se) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA23552 = {S22341, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-tw) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA23559 = {S22342, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; zh-tw) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA23561 = {S22343, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-za) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA23601 = {S22344, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-ca) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA36354 = {S22345, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-ph) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA61805 = {S22346, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-se) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA66312 = {S22347, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-cz) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA66420 = {S22348, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; es-ve) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA75809 = {S22349, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-pl) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA95312 = {S22350, 121, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-us) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13*/
const Device UA97667 = {S22351, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA98490 = {S22352, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA98990 = {S22353, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA99692 = {S22354, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; de-de) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA99787 = {S22355, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20*/
const Device UA100066 = {S22356, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18*/
const Device UA110853 = {S22357, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; nl-nl) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA115803 = {S22358, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; nl-nl) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18*/
const Device UA116555 = {S22359, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; nl-nl) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA117764 = {S22360, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; es-es) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA118027 = {S22361, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; it-it) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA119450 = {S22362, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; es-es) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA122249 = {S22363, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; fr-fr) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA122297 = {S22364, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; fr-fr) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20*/
const Device UA122681 = {S22365, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; es-es) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA125585 = {S22366, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_4; fr-fr) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA131212 = {S22367, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA132955 = {S22368, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; de-de) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA133236 = {S22369, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-au) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA133336 = {S22370, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; it-it) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA133865 = {S22371, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; fr-fr) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18*/
const Device UA135873 = {S22372, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; de-de) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18*/
const Device UA137076 = {S22373, 119, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3;) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA137374 = {S22374, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; ru-ru) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA138046 = {S22375, 117, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3;) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.22*/
const Device UA138608 = {S22376, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; pt-br) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA141686 = {S22377, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; it-it) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA143415 = {S22378, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; ru-ru) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA147234 = {S22379, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; sv-se) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA149968 = {S22380, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; de-de) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20*/
const Device UA157230 = {S22381, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; nb-no) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA165428 = {S22382, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; hu-hu) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20*/
const Device UA166175 = {S22383, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; nb-no) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA183930 = {S22384, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/3.1.1 Safari/525.18*/
const Device UA228965 = {S22385, 117, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA230416 = {S22386, 115, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/3.1.1 Safari/525.18*/
const Device UA260736 = {S22387, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-gb) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA268036 = {S22388, 117, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3;) AppleWebKit/525.19 (KHTML, like Gecko) Version/3.1.2 Safari/525.21*/
const Device UA271998 = {S22389, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; ru-ru) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA274786 = {S22390, 121, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; de-de) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13*/
const Device UA276124 = {S22391, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; sv-se) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA283438 = {S22392, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; nb-no) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA297836 = {S22393, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; ja-jp) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA303065 = {S22394, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-gb) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA329260 = {S22395, 113, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/3.1 Safari/525.13*/
const Device UA371711 = {S22396, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.1 Safari/525.13*/
const Device UA373268 = {S22397, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; ja-jp) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA384717 = {S22398, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; pt-pt) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA388908 = {S22399, 121, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; cs-cz) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13*/
const Device UA389183 = {S22400, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; fi-fi) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA390827 = {S22401, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; fi-fi) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA414561 = {S22402, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; zh-tw) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA418332 = {S22403, 121, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; it-it) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13*/
const Device UA432599 = {S22404, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; ja-jp) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA439785 = {S22405, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; pt-br) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA447100 = {S22406, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; pl-pl) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA462249 = {S22407, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA474782 = {S22408, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; ja-jp) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18*/
const Device UA475445 = {S22409, 121, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; es-es) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13*/
const Device UA481642 = {S22410, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; ru-ru) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18*/
const Device UA482086 = {S22411, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; ru-ru) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20*/
const Device UA483302 = {S22412, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-au) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA483773 = {S22413, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; ko-kr) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA486311 = {S22414, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; pl-pl) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA491948 = {S22415, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; ja-jp) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20*/
const Device UA497840 = {S22416, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; cs-cz) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA506094 = {S22417, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-au) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA511292 = {S22418, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; es-es) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20*/
const Device UA523786 = {S22419, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; el-gr) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA543428 = {S22420, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; it-it) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18*/
const Device UA577709 = {S22421, 121, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-us) AppleWebKit/525.12 (KHTML, like Gecko) Version/3.1 Safari/525.12*/
const Device UA582586 = {S22422, 120, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; it-it) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.1*/
const Device UA624904 = {S22423, 116, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en) AppleWebKit/525.9 (KHTML, like Gecko) Version/3.1 Safari/525.9*/
const Device UA661732 = {S22424, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (WebTubeHD; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA710933 = {S22425, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; fi-fi) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA711680 = {S22426, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; nl-nl) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20*/
const Device UA713158 = {S22427, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; tr-tr) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA714758 = {S22428, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; zh-cn) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA726642 = {S22429, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; it-it) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20*/
const Device UA731287 = {S22430, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; da-dk) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA736130 = {S22431, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; zh-cn) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA750723 = {S22432, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; es-es) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18*/
const Device UA756801 = {S22433, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; nb-no) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20*/
const Device UA768591 = {S22434, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; en-gb) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20*/
const Device UA782372 = {S22435, 122, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; ar) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA784876 = {S22436, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA791022 = {S22437, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; pl-pl) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA809651 = {S22438, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; es-xl) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA836012 = {S22439, 116, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_2; en-gb) AppleWebKit/526+ (KHTML, like Gecko) Version/3.1 Safari/525.9*/
const Device UA866191 = {S22440, 121, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; fr-fr) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13*/
const Device UA868820 = {S22441, 118, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_2; en) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18*/
const Device UA892985 = {S22442, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; en-ca) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20*/
const Device UA893369 = {S22443, 119, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_4; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1 Safari/525.13*/
const Device UA899694 = {S22444, 118, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_3; en) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20*/
const Device UA1836250 = {S22445, 121, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; nl-nl) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13*/
const Device UA1836797 = {S22446, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA1840508 = {S22447, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; zh-cn) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA1841845 = {S22448, 124, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; ar) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA1855793 = {S22449, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; zh-tw) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA1865962 = {S22450, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; pt-pt) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA1873624 = {S22451, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; pt-br) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18*/
const Device UA1891636 = {S22452, 119, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; fr-fr) AppleWebKit/525.9 (KHTML, like Gecko) Version/3.1 Safari/525.9*/
const Device UA2126769 = {S22453, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA2128842 = {S22454, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; da-dk) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA2131781 = {S22455, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; tr-tr) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA2135098 = {S22456, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; sv-se) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA2212191 = {S22457, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-gb) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18*/
const Device UA2264256 = {S22458, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA2267858 = {S22459, 121, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; ru-ru) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13*/
const Device UA2274793 = {S22460, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; th-th) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA2274915 = {S22461, 121, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; ja-jp) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13*/
const Device UA2290710 = {S22462, 119, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-us) AppleWebKit/525.9 (KHTML, like Gecko) Version/3.1 Safari/525.9*/
const Device UA2300329 = {S22463, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; he-il) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA2437721 = {S22464, 121, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13*/
const Device UA2438254 = {S22465, 127, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA2439501 = {S22466, 119, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-us) AppleWebKit/525.7 (KHTML, like Gecko) Version/3.1 Safari/525.7*/
const Device UA2774607 = {S22467, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; de-de) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20*/
const Device UA2801900 = {S22468, 123, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; pl-pl) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20*/
const Device UA2802437 = {S22469, 121, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_3; sv-se) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20*/
const Device UA2948841 = {S22470, 125, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; es-xl) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA3069173 = {S22471, 121, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; pt-br) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13*/
const Device UA3069248 = {S22472, 121, { &P17595,&P17295,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-gb) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13*/
const Device UA924777 = {S22473, 138, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1999352 = {S22474, 138, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2012765 = {S22475, 138, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-I535 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2225717 = {S22476, 138, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-i9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2231791 = {S22477, 138, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2238137 = {S22478, 138, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2238242 = {S22479, 138, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2367295 = {S22480, 139, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I93000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2386283 = {S22481, 139, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1;  GT-I9300 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA2386313 = {S22482, 139, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300T Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2402015 = {S22483, 132, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2427844 = {S22484, 138, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; GT-I9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2430899 = {S22485, 139, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999V Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2455764 = {S22486, 136, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SC-06D Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2456144 = {S22487, 135, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; I9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2482583 = {S22488, 136, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; I9300i Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2483429 = {S22489, 139, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2653309 = {S22490, 133, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; i9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2748317 = {S22491, 139, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2833140 = {S22492, 132, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2854972 = {S22493, 150, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2_????_??????; GT-I9300 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2943986 = {S22494, 139, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999N Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2947860 = {S22495, 139, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999V Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2987078 = {S22496, 138, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I939 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3118717 = {S22497, 138, { &P18283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-I939 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA92165 = {S22498, 46, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64(Windows NT 5.1; U; en) Presto/2.1.1*/
const Device UA92826 = {S22499, 47, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.2; U; en) Presto/2.1.1*/
const Device UA94016 = {S22500, 47, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 5.1; U; ru) Presto/2.1.1*/
const Device UA94622 = {S22501, 47, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.60 (Windows NT 5.1; U; ru) Presto/2.1.1*/
const Device UA110302 = {S22502, 70, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.1.1*/
const Device UA124957 = {S22503, 47, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.61 (Windows NT 5.1; U; en) Presto/2.1.1*/
const Device UA134094 = {S22504, 47, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.62 (Windows NT 5.1; U; ru) Presto/2.1.1*/
const Device UA270866 = {S22505, 47, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 5.2; U; en) Presto/2.1.1*/
const Device UA305230 = {S22506, 70, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.1.1*/
const Device UA386685 = {S22507, 70, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.1.1*/
const Device UA416080 = {S22508, 47, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.62 (Windows NT 5.2; U; en) Presto/2.1.1*/
const Device UA442580 = {S22509, 70, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.1.1*/
const Device UA446487 = {S22510, 70, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.60 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.1.1*/
const Device UA446495 = {S22511, 70, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.1.1*/
const Device UA468966 = {S22512, 47, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.60 (Windows NT 5.2; U; en) Presto/2.1.1*/
const Device UA504118 = {S22513, 70, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.1.1*/
const Device UA509017 = {S22514, 70, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.62 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.1.1*/
const Device UA512183 = {S22515, 47, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.61 (Windows NT 5.2; U; en) Presto/2.1.1*/
const Device UA517902 = {S22516, 97, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 5.7 (build 03686); MRSPUTNIK OW 2, 3, 0, 244; ru) Presto/2.1.1*/
const Device UA522055 = {S22517, 70, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.60 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.1.1*/
const Device UA524190 = {S22518, 70, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.60 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.1.1*/
const Device UA529880 = {S22519, 70, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.1.1*/
const Device UA534949 = {S22520, 97, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 5.1; U; MRA 5.7 (build 03686); MRSPUTNIK OW 2, 3, 0, 244; ru) Presto/2.1.1*/
const Device UA563269 = {S22521, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.1.1*/
const Device UA753177 = {S22522, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.1.1*/
const Device UA767957 = {S22523, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.1.1*/
const Device UA803633 = {S22524, 78, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 9.63*/
const Device UA807017 = {S22525, 65, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 9.64*/
const Device UA814933 = {S22526, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.1.1*/
const Device UA816906 = {S22527, 70, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.1.1*/
const Device UA852068 = {S22528, 81, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; en-GB; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 9.61*/
const Device UA1861409 = {S22529, 70, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.62 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.1.1*/
const Device UA1864597 = {S22530, 70, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.1.1*/
const Device UA1895378 = {S22531, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.1.1*/
const Device UA1906774 = {S22532, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.1.1*/
const Device UA1994267 = {S22533, 70, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.1.1*/
const Device UA2127434 = {S22534, 65, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 9.63*/
const Device UA2131644 = {S22535, 70, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.1.1*/
const Device UA2151935 = {S22536, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.62 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.1.1*/
const Device UA2157284 = {S22537, 70, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.1.1*/
const Device UA2216559 = {S22538, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.60 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.1.1*/
const Device UA2262349 = {S22539, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.1.1*/
const Device UA2262348 = {S22540, 70, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.1.1*/
const Device UA2266357 = {S22541, 78, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; ru; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 9.64*/
const Device UA2274703 = {S22542, 70, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.62 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.1.1*/
const Device UA2279125 = {S22543, 96, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 5.1; U; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94; ru) Presto/2.1.1*/
const Device UA2284240 = {S22544, 70, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.62 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.1.1*/
const Device UA2301478 = {S22545, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.1.1*/
const Device UA2305376 = {S22546, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.1.1*/
const Device UA2373754 = {S22547, 70, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.1.1*/
const Device UA2379880 = {S22548, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.1.1*/
const Device UA2520704 = {S22549, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.1.1*/
const Device UA2548643 = {S22550, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.1.1*/
const Device UA2587159 = {S22551, 78, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 9.62*/
const Device UA2642107 = {S22552, 70, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.62 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.1.1*/
const Device UA2652672 = {S22553, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.1.1*/
const Device UA2655050 = {S22554, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.1.1*/
const Device UA2694849 = {S22555, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.1.1*/
const Device UA2712142 = {S22556, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.1.1*/
const Device UA2722842 = {S22557, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.1.1*/
const Device UA2727337 = {S22558, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.1.1*/
const Device UA2747940 = {S22559, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.1.1*/
const Device UA2761647 = {S22560, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.1.1*/
const Device UA2763744 = {S22561, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.1.1*/
const Device UA2775709 = {S22562, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.1.1*/
const Device UA2786992 = {S22563, 69, { &P15364,&P5690,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.1.1*/
const Device UA159877 = {S22564, 119, { &P15364,&P7133,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10*/
const Device UA294746 = {S22565, 156, { &P15364,&P7133,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10,gzip(gfe) (via translate.google.com)*/
const Device UA836985 = {S22566, 119, { &P15364,&P7133,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; de-DE) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10*/
const Device UA2375315 = {S22567, 135, { &P15364,&P7133,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.10 (KHTML	 like Gecko) Chrome/8.0.552.237 Safari/534.10 control/6.02164*/
const Device UA2384955 = {S22568, 129, { &P15364,&P7133,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10,gzip(gfe)*/
const Device UA171512 = {S22569, 118, { &P15364,&P5690,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.84 Safari/534.13*/
const Device UA172087 = {S22570, 118, { &P15364,&P5690,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13*/
const Device UA491557 = {S22571, 154, { &P15364,&P5690,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.12 (KHTML, like Gecko) Chrome/9.0.587.0 Safari/534.12,gzip(gfe) (via translate.google.com)*/
const Device UA2276782 = {S22572, 132, { &P15364,&P5690,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13 SunChrome/9.0*/
const Device UA2323161 = {S22573, 118, { &P15364,&P5690,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.13 (KHTML	 like Gecko) Chrome/9.0.597.19 Safari/534.13*/
const Device UA2385099 = {S22574, 128, { &P15364,&P5690,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13,gzip(gfe)*/
const Device UA2673264 = {S22575, 116, { &P15364,&P5690,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-CA) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.0 Safari/534.13*/
const Device UA2673341 = {S22576, 116, { &P15364,&P5690,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.12 (KHTML like Gecko) Chrome/9.0.576.0 Safari/534.12*/
const Device UA2678279 = {S22577, 116, { &P15364,&P5690,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; fr-CA) AppleWebKit/534.12 (KHTML like Gecko) Chrome/9.0.576.0 Safari/534.12*/
const Device UA2680941 = {S22578, 117, { &P15364,&P5690,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; es-US) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13*/
const Device UA2680942 = {S22579, 117, { &P15364,&P5690,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; haw-US) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.0 Safari/534.13*/
const Device UA2708865 = {S22580, 116, { &P15364,&P5690,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; fr-FR) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.0 Safari/534.13*/
const Device UA2709771 = {S22581, 116, { &P15364,&P5690,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; da-DE) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.0 Safari/534.13*/
const Device UA2710999 = {S22582, 117, { &P15364,&P5690,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13*/
const Device UA2735095 = {S22583, 117, { &P15364,&P5690,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; ku-DE) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13*/
const Device UA2736283 = {S22584, 117, { &P15364,&P5690,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; ru-DE) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.98 Safari/534.13*/
const Device UA2806189 = {S22585, 117, { &P15364,&P5690,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/534.12 (KHTML, like Gecko) Chrome/9.0.576.0 Safari/534.12*/
const Device UA2860251 = {S22586, 116, { &P15364,&P5690,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; de-DE) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.0 Safari/534.13*/
const Device UA2865518 = {S22587, 116, { &P15364,&P5690,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; br-FR) AppleWebKit/534.12 (KHTML like Gecko) Chrome/9.0.576.0 Safari/534.12*/
const Device UA2865585 = {S22588, 116, { &P15364,&P5690,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; fy-DE) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.0 Safari/534.13*/
const Device UA2865666 = {S22589, 116, { &P15364,&P5690,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; oc-FR) AppleWebKit/534.13 (KHTML like Gecko) Chrome/9.0.597.0 Safari/534.13*/
const Device UA3119784 = {S22590, 114, { &P15364,&P5690,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows U Windows NT 5.1 en-US) AppleWebKit/534.12 (KHTML, like Gecko) Chrome/9.0.583.0 Safari/534.12*/
const Device UA47373 = {S22591, 142, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61575 = {S22592, 142, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76678 = {S22593, 142, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA252239 = {S22594, 143, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5660V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA342466 = {S22595, 143, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; GT-S5660L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA395079 = {S22596, 66, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-S5660 Build/GINGERBREAD)*/
const Device UA452669 = {S22597, 66, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5660 Build/GINGERBREAD)*/
const Device UA586782 = {S22598, 142, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633743 = {S22599, 163, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; SAMSUNG GT-S5660V/S5660VXXKQE Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA661241 = {S22600, 163, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; SAMSUNG GT-S5660V/S5660VXXKQK Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA676832 = {S22601, 66, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5660 Build/GINGERBREAD)*/
const Device UA738430 = {S22602, 130, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5660 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA764991 = {S22603, 67, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5660V Build/GINGERBREAD)*/
const Device UA894859 = {S22604, 143, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-S5660L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA902843 = {S22605, 163, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-S5660V/S5660VXXKT4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1939186 = {S22606, 66, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-S5660 Build/GINGERBREAD)*/
const Device UA1965925 = {S22607, 135, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5660 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2019763 = {S22608, 67, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5660L Build/GINGERBREAD)*/
const Device UA2022555 = {S22609, 130, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; fa-fa; GT-S5660 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2028867 = {S22610, 135, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5660 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2364262 = {S22611, 140, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2413208 = {S22612, 140, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2523718 = {S22613, 136, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5660 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2545143 = {S22614, 61, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; GT-S5660 Build/GRWK74)*/
const Device UA2545639 = {S22615, 130, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fa-fa; GT-S5660 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2660386 = {S22616, 163, { &P15060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; SAMSUNG GT-S5660V/S5660VXXKT8 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2357770 = {S22617, 127, { &P15364,&P17285,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/23.0.1271.64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2358090 = {S22618, 134, { &P15364,&P17285,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/23.0.1271.64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2388035 = {S22619, 134, { &P15364,&P17285,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/23.0.1271.91) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.91 Safari/537.11*/
const Device UA2388326 = {S22620, 127, { &P15364,&P17285,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/23.0.1271.91) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.91 Safari/537.11*/
const Device UA2389899 = {S22621, 127, { &P15364,&P17285,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/23.0.1271.95) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11*/
const Device UA2390013 = {S22622, 134, { &P15364,&P17285,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/23.0.1271.95) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11*/
const Device UA2408222 = {S22623, 182, { &P15364,&P17285,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/23.0.1271.97; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2408331 = {S22624, 89, { &P15364,&P17285,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/23.0.1271.97)*/
const Device UA2408336 = {S22625, 96, { &P15364,&P17285,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/23.0.1271.97)*/
const Device UA2408330 = {S22626, 127, { &P15364,&P17285,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/23.0.1271.97) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2430854 = {S22627, 194, { &P15364,&P17285,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; chromeframe/23.0.1271.97; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2431961 = {S22628, 134, { &P15364,&P17285,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/23.0.1271.97) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2432633 = {S22629, 195, { &P15364,&P17285,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; chromeframe/23.0.1271.97; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2432708 = {S22630, 189, { &P15364,&P17285,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; chromeframe/23.0.1271.97; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2434537 = {S22631, 161, { &P15364,&P17285,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/23.0.1271.97; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2436390 = {S22632, 229, { &P15364,&P17285,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/23.0.1271.97; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C)*/
const Device UA2460545 = {S22633, 210, { &P15364,&P17285,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; chromeframe/23.0.1271.97; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA232840 = {S22634, 66, { &P18254,&P7480,&P18245,&P19827 }};	/*Useragent - Mozilla/5.0 (compatible; AhrefsBot/1.0; +http://ahrefs.com/robot/)*/
const Device UA255195 = {S22635, 66, { &P18254,&P7480,&P18245,&P19827 }};	/*Useragent - Mozilla/5.0 (compatible; AhrefsBot/2.0; +http://ahrefs.com/robot/)*/
const Device UA632126 = {S22636, 105, { &P18254,&P7480,&P18245,&P19827 }};	/*Useragent - Mozilla/5.0 (compatible; AhrefsBot/2.0; +http://ahrefs.com/robot/),gzip(gfe) (via docs.google.com/viewer)*/
const Device UA909506 = {S22637, 66, { &P18254,&P7480,&P18245,&P19827 }};	/*Useragent - Mozilla/5.0 (compatible; AhrefsBot/3.1; +http://ahrefs.com/robot/)*/
const Device UA2193293 = {S22638, 66, { &P18254,&P7480,&P18245,&P19827 }};	/*Useragent - Mozilla/5.0 (compatible; AhrefsBot/4.0; +http://ahrefs.com/robot/)*/
const Device UA2567484 = {S22639, 40, { &P18254,&P7480,&P18245,&P19827 }};	/*Useragent - AhrefsBot.Feeds v0.1; http://ahrefs.com/*/
const Device UA2569817 = {S22640, 77, { &P18254,&P7480,&P18245,&P19827 }};	/*Useragent - Mozilla/5.0 (compatible; AhrefsBot.FreshPages/0.1; +http://ahrefs.com/robot/)*/
const Device UA2947237 = {S22641, 66, { &P18254,&P7480,&P18245,&P19827 }};	/*Useragent - Mozilla/5.0 (compatible; AhrefsBot/5.0; +http://ahrefs.com/robot/)*/
const Device UA609371 = {S22642, 102, { &P17595,&P17296,&P17558,&P18092 }};	/*Useragent - Mozilla/5.2 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.2.3) Gecko/X Firefox/3.6.3 Java/1.6.0_31*/
const Device UA627059 = {S22643, 87, { &P17595,&P17296,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6;en-US; rv:1.9.2.9) Gecko/X Firefox/3.6.9*/
const Device UA894271 = {S22644, 77, { &P17595,&P17296,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US) Gecko/X Firefox/3.6.11*/
const Device UA1961684 = {S22645, 103, { &P17595,&P17296,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.2.28) Gecko/X (CK-Firefox) Firefox/3.6.28*/
const Device UA2144116 = {S22646, 84, { &P17595,&P17296,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.2) Gecko/X Firefox 3.6*/
const Device UA2000889 = {S22647, 102, { &P15364,&P18110,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1*/
const Device UA2015636 = {S22648, 100, { &P15364,&P18110,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2041221 = {S22649, 109, { &P15364,&P18110,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2197565 = {S22650, 144, { &P15364,&P18110,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Jolicloud Linux i686) AppleWebKit/537.1 (KHTML, like Gecko) Joli OS/1.2 Chromium/21.0.1180.79 Chrome/21.0.1180.79 Safari/537.1*/
const Device UA2200776 = {S22651, 139, { &P15364,&P18110,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)*/
const Device UA2204330 = {S22652, 137, { &P15364,&P18110,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)*/
const Device UA2354642 = {S22653, 104, { &P15364,&P18110,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 \(X11; Linux x8664\) AppleWebKit/537.1 \(KHTML like Gecko\) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2466346 = {S22654, 110, { &P15364,&P18110,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe)*/
const Device UA2674630 = {S22655, 106, { &P15364,&P18110,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/21.0.1410.63 Safari/537.31*/
const Device UA2674640 = {S22656, 109, { &P15364,&P18110,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/21.0.1410.63 Safari/537.31*/
const Device UA2674809 = {S22657, 106, { &P15364,&P18110,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/21.0.1410.63 Safari/537.31*/
const Device UA2674873 = {S22658, 109, { &P15364,&P18110,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/21.0.1410.63 Safari/537.31*/
const Device UA2675009 = {S22659, 106, { &P15364,&P18110,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/21.0.1410.63 Safari/537.31*/
const Device UA2718192 = {S22660, 109, { &P15364,&P18110,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/21.0.1410.63 Safari/537.31*/
const Device UA2719087 = {S22661, 106, { &P15364,&P18110,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/21.0.1410.63 Safari/537.31*/
const Device UA353872 = {S22662, 103, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.12 Safari/535.11*/
const Device UA401267 = {S22663, 101, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA602871 = {S22664, 110, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.83 Safari/535.11*/
const Device UA656196 = {S22665, 144, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Jolicloud Linux i686) AppleWebKit/535.11 (KHTML, like Gecko) Joli OS/1.2 Chromium/17.0.963.46 Chrome/17.0.963.46 Safari/535.11*/
const Device UA683121 = {S22666, 167, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/unstable (3.4.1-1) Epiphany/3.4.1*/
const Device UA897923 = {S22667, 165, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/unstable (3.4.2-1) Epiphany/3.4.2*/
const Device UA899834 = {S22668, 167, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/unstable (3.4.2-1) Epiphany/3.4.2*/
const Device UA1972346 = {S22669, 138, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11,gzip(gfe) (via translate.google.com)*/
const Device UA1989965 = {S22670, 175, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/sid/experimental (3.2.1-2) Epiphany/3.2.1*/
const Device UA2000165 = {S22671, 172, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/unstable (3.4.2-1+build1) Epiphany/3.4.2*/
const Device UA2039629 = {S22672, 174, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/unstable (3.4.2-1+build1) Epiphany/3.4.2*/
const Device UA2204043 = {S22673, 190, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+; (+http://immediatenet.com/thumbnail_api.html; free thumbnails)*/
const Device UA2236464 = {S22674, 167, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/unstable (3.4.2-2) Epiphany/3.4.2*/
const Device UA2260064 = {S22675, 165, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/unstable (3.4.2-2) Epiphany/3.4.2*/
const Device UA2335268 = {S22676, 119, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+*/
const Device UA2335822 = {S22677, 165, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/unstable (3.4.1-1) Epiphany/3.4.1*/
const Device UA2374236 = {S22678, 173, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/sid/experimental (3.2.1-2) Epiphany/3.2.1*/
const Device UA2496676 = {S22679, 126, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux armv6l) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+*/
const Device UA2534850 = {S22680, 164, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/7.0 (3.4.2-2.1) Epiphany/3.4.2*/
const Device UA2599534 = {S22681, 162, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Debian/7.0 (3.4.2-2.1) Epiphany/3.4.2*/
const Device UA2674477 = {S22682, 106, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/17.0.1410.63 Safari/537.31*/
const Device UA2674613 = {S22683, 109, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/17.0.1410.63 Safari/537.31*/
const Device UA2674792 = {S22684, 106, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/17.0.1410.63 Safari/537.31*/
const Device UA2674874 = {S22685, 106, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/17.0.1410.63 Safari/537.31*/
const Device UA2676617 = {S22686, 109, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/17.0.1410.63 Safari/537.31*/
const Device UA2715577 = {S22687, 106, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/17.0.1410.63 Safari/537.31*/
const Device UA2718861 = {S22688, 109, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/17.0.1410.63 Safari/537.31*/
const Device UA2752635 = {S22689, 139, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/537.6+ SugarLabs/0.98*/
const Device UA3117746 = {S22690, 157, { &P15364,&P18110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ SUSE/12.2 (3.4.2) Epiphany/3.4.2*/
const Device UA16772 = {S22691, 147, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; E15a Build/2.0.2.A.0.24) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA16776 = {S22692, 160, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; SonyEricssonE15av Build/2.0.2.A.0.24) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA18662 = {S22693, 159, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-ar; SonyEricssonE15a Build/2.0.1.A.0.47) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA19745 = {S22694, 161, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; SonyEricssonE15i-o Build/2.1.1.A.0.16) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA19789 = {S22695, 158, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; SonyEricssonE15i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA19812 = {S22696, 160, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1 - JIT Enabled; vi-vn; E15i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA42523 = {S22697, 99, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.1-update1;Zh_cn;E15i;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA43920 = {S22698, 147, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; E15i Build/2.0.1.A.0.47) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA43940 = {S22699, 160, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; SonyEricssonE15iv Build/2.0.1.A.0.47) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA43944 = {S22700, 147, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; E15i Build/2.0.2.A.0.24) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA58417 = {S22701, 147, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; E15i Build/2.1.1.A.0.16) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA58430 = {S22702, 146, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; E15i Build/2.1.1.C.0.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA58939 = {S22703, 146, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-mx; E15a Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA70022 = {S22704, 144, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; E15i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA71849 = {S22705, 147, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; iw-il; E15a Build/2.1.1.A.0.16) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA73004 = {S22706, 157, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ph; SonyEricssonE15iBuild/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA76427 = {S22707, 93, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Android (Linux; U; Android 2.1-update1; zh-cn; E15i) AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA172400 = {S22708, 147, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-mx; E15a Build/2.0.1.A.0.47) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA173592 = {S22709, 68, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; E15i Build/2.1.1.A.0.6)*/
const Device UA179836 = {S22710, 146, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; E15i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA793846 = {S22711, 132, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fa-ir; E15i Build/2.1.1.A.0.6) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1876782 = {S22712, 69, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - PVPlayer E15i CORE/7.507.1.1 OpenCORE/2.05 (Linux;Android2.1-update1)*/
const Device UA1897011 = {S22713, 68, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; E15a Build/2.1.1.A.0.6)*/
const Device UA1906633 = {S22714, 133, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; E15i Build/2.1.1.A.0.16) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2020850 = {S22715, 133, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; E15i Build/2.0.1.A.0.47) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2023292 = {S22716, 132, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fa-ir; E15i Build/2.1.1.C.0.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2028740 = {S22717, 137, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; E15i Build/2.1.1.A.0.6) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2684156 = {S22718, 138, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; E15i Build/2.1.1.A.0.16) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2720655 = {S22719, 146, { &P14202,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; E15i Build/2.1.1.B.0.5) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2677726 = {S22720, 109, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4008.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1480.0 Safari/537.36*/
const Device UA2678854 = {S22721, 111, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4008.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1480.0 Safari/537.36*/
const Device UA2709269 = {S22722, 109, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4068.1.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1494.8 Safari/537.36*/
const Device UA2709277 = {S22723, 111, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4068.1.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1494.8 Safari/537.36*/
const Device UA2710751 = {S22724, 109, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4068.2.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1494.8 Safari/537.36*/
const Device UA2714042 = {S22725, 111, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4050.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1489.4 Safari/537.36*/
const Device UA2714346 = {S22726, 112, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4028.0.2013) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1484.1 Safari/537.36*/
const Device UA2715357 = {S22727, 111, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4068.1.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1494.8 Safari/537.36*/
const Device UA2716720 = {S22728, 109, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4035.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1485.1 Safari/537.36*/
const Device UA2716724 = {S22729, 111, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4035.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1485.1 Safari/537.36*/
const Device UA2716903 = {S22730, 111, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4035.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1485.1 Safari/537.36*/
const Device UA2719366 = {S22731, 111, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4100.1.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.4 Safari/537.36*/
const Device UA2719386 = {S22732, 111, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4100.1.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.4 Safari/537.36*/
const Device UA2719478 = {S22733, 109, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4100.2.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.4 Safari/537.36*/
const Device UA2740187 = {S22734, 112, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4100.7.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.11 Safari/537.36*/
const Device UA2740301 = {S22735, 112, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4100.7.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.11 Safari/537.36*/
const Device UA2740352 = {S22736, 110, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4100.7.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.11 Safari/537.36*/
const Device UA2740506 = {S22737, 109, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4100.1.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.4 Safari/537.36*/
const Device UA2754187 = {S22738, 113, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4100.17.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.20 Safari/537.36*/
const Device UA2754343 = {S22739, 111, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4100.17.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.20 Safari/537.36*/
const Device UA2754489 = {S22740, 113, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4100.17.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.20 Safari/537.36*/
const Device UA2784144 = {S22741, 113, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4100.38.5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.35 Safari/537.36*/
const Device UA2784294 = {S22742, 111, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4100.38.4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.35 Safari/537.36*/
const Device UA2784296 = {S22743, 113, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4100.38.3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.35 Safari/537.36*/
const Device UA2784302 = {S22744, 111, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4100.38.3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.35 Safari/537.36*/
const Device UA2870198 = {S22745, 111, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4100.44.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.47 Safari/537.36*/
const Device UA2870199 = {S22746, 113, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4100.44.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.47 Safari/537.36*/
const Device UA2870201 = {S22747, 113, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4100.44.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.47 Safari/537.36*/
const Device UA2875692 = {S22748, 111, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4100.48.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.53 Safari/537.36*/
const Device UA2875696 = {S22749, 113, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4100.48.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.53 Safari/537.36*/
const Device UA2875698 = {S22750, 113, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4100.48.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.53 Safari/537.36*/
const Device UA2888596 = {S22751, 111, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4100.53.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.61 Safari/537.36*/
const Device UA2888601 = {S22752, 113, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4100.53.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.61 Safari/537.36*/
const Device UA2888629 = {S22753, 113, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4100.53.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.61 Safari/537.36*/
const Device UA2888656 = {S22754, 111, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4100.53.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.66 Safari/537.36*/
const Device UA2888657 = {S22755, 113, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4100.53.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.66 Safari/537.36*/
const Device UA2888658 = {S22756, 113, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4100.53.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.66 Safari/537.36*/
const Device UA2888659 = {S22757, 111, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4100.53.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.66 Safari/537.36*/
const Device UA2947719 = {S22758, 113, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4100.68.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.68 Safari/537.36*/
const Device UA2947991 = {S22759, 113, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4100.68.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.68 Safari/537.36*/
const Device UA2947995 = {S22760, 111, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4100.78.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.68 Safari/537.36*/
const Device UA2947999 = {S22761, 111, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4100.68.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.68 Safari/537.36*/
const Device UA2955656 = {S22762, 112, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3927.0.2013) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1456.1 Safari/537.36*/
const Device UA3024944 = {S22763, 111, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4100.86.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3024966 = {S22764, 113, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4100.86.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3025186 = {S22765, 113, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4100.86.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3036654 = {S22766, 113, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4100.89.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3037935 = {S22767, 113, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4100.89.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3038880 = {S22768, 111, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4100.89.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3092484 = {S22769, 114, { &P15364,&P6596,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4100.53.15) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2015618 = {S22770, 117, { &P17595,&P17297,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2015629 = {S22771, 117, { &P17595,&P17297,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2015637 = {S22772, 117, { &P17595,&P17297,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2016034 = {S22773, 117, { &P17595,&P17297,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2017888 = {S22774, 117, { &P17595,&P17297,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2041734 = {S22775, 154, { &P17595,&P17297,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)*/
const Device UA2042650 = {S22776, 117, { &P17595,&P17297,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2196404 = {S22777, 156, { &P17595,&P17297,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2416777 = {S22778, 132, { &P23773,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N8010 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2427439 = {S22779, 61, { &P23773,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-N8010 Build/JRO03C)*/
const Device UA2484831 = {S22780, 132, { &P23773,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N8010 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2507451 = {S22781, 61, { &P23773,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N8010 Build/JZO54K)*/
const Device UA2855725 = {S22782, 130, { &P23773,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-N8010 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2986063 = {S22783, 139, { &P23773,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N8010 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA4921 = {S22784, 145, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I9000/I9000ZSJPD Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA4947 = {S22785, 136, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; hr-hr; GT-I9000 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA7461 = {S22786, 136, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-I9000 Build/XWJS5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA7494 = {S22787, 134, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA7456 = {S22788, 142, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA7498 = {S22789, 136, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; en-gb; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA7501 = {S22790, 144, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-I9000 Build/Vibrace V.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA14869 = {S22791, 135, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; pt-br; GT-I9000B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51763 = {S22792, 138, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; hr-hr; GT-I9000 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA51799 = {S22793, 132, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA63150 = {S22794, 134, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA64246 = {S22795, 136, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA134153 = {S22796, 134, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I9000 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA140777 = {S22797, 58, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I9000 Build/FROYO)*/
const Device UA159404 = {S22798, 135, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-I9000M Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA159572 = {S22799, 60, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-I9000 Build/FROYO)*/
const Device UA169714 = {S22800, 137, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; GT-I9000B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA292975 = {S22801, 135, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I9000 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA548459 = {S22802, 59, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I9000M Build/FROYO)*/
const Device UA644719 = {S22803, 124, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-I9000 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA791467 = {S22804, 122, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-I9000 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2018533 = {S22805, 59, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I9000B Build/FROYO)*/
const Device UA2033690 = {S22806, 129, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-I9000 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2441436 = {S22807, 127, { &P14877,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-I9000 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2359326 = {S22808, 129, { &P18385,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Nexus 7 Build/JOP40C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2368596 = {S22809, 131, { &P18385,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 7 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2467553 = {S22810, 137, { &P18385,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 7 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2512911 = {S22811, 130, { &P18385,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2708723 = {S22812, 134, { &P18385,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; en-US; Nexus 7 Build/JOP40C) AppleWebKit/535.19 (KHTML like Gecko) Chrome/18.0.1025.166 Safari/535.19*/
const Device UA2711116 = {S22813, 137, { &P18385,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; haw-US; Nexus 7 Build/JOP40D) AppleWebKit/535.19 (KHTML like Gecko) Chrome/18.0.1025.166 Safari/535.19*/
const Device UA2846580 = {S22814, 134, { &P18385,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 7 3G Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2206158 = {S22815, 88, { &P15364,&P7133,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:16.0) Gecko/X Firefox/16.0 AlexaToolbar/alxf-2.17*/
const Device UA2259009 = {S22816, 70, { &P15364,&P7133,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:16.0) Gecko/X Firefox/16.0*/
const Device UA2259142 = {S22817, 66, { &P15364,&P7133,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:16.0) Gecko/X Firefox/16.00*/
const Device UA2386525 = {S22818, 82, { &P15364,&P7133,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.1; Win64; x64; rv:16.0) Gecko/16.0 Firefox/16.0*/
const Device UA2469082 = {S22819, 69, { &P15364,&P7133,&P18428,&P18092 }};	/*Useragent - Mozilla/6.0 (Windows NT 6.1; WOW64; rv:16.0.1) Gecko/X Firefox/16.0.1*/
const Device UA2527216 = {S22820, 61, { &P15364,&P7133,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 Windows NT 6.1 WOW64 rv 16.0 Gecko/X Firefox/16.0*/
const Device UA2995703 = {S22821, 74, { &P15364,&P7133,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; rv:16.0.1) Gecko/X Firefox/16.0.1*/
const Device UA475249 = {S22822, 131, { &P15364,&P17285,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Windows; U; Windows NT 6.0; en-US; rv:1.9.0.7; .NET CLR 1.1.4322; .NET CLR 1.0.3705) Gecko/X Firefox/3.0.7*/
const Device UA480274 = {S22823, 58, { &P15364,&P17285,&P3409,&P18092 }};	/*Useragent - Mozilla /5.0 (Windows NT 6.0; U; en) Gecko/X Firefox/3.0.1*/
const Device UA553594 = {S22824, 140, { &P15364,&P17285,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; rv:1.8.1) Gecko/X Firefox/3.0Mozilla/5.0 (Windows; U; Windows NT 6.1; rv:1.9.2) Gecko/X Firefox/3.0*/
const Device UA700724 = {S22825, 149, { &P15364,&P17285,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.14) Gecko/X YFF3 Firefox/3.0.14 ( .NET CLR 3.5.30729) WinNT-PAI 08.08.2009 SearchToolbar/1.2*/
const Device UA764288 = {S22826, 125, { &P15364,&P17285,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 (.NET CLR 3.5.30729) WinNT-EVI 01.05.2010*/
const Device UA779754 = {S22827, 133, { &P15364,&P17285,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 (.NET CLR 3.5.30729) WinNT-EVI 30.04.2010 FBSMTWB*/
const Device UA794576 = {S22828, 133, { &P15364,&P17285,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.14) Gecko/X Firefox/3.0.14 (.NET CLR 3.5.30729) WinNT-PAI 17.08.2009 FBSMTWB*/
const Device UA872256 = {S22829, 76, { &P15364,&P17285,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; de; rv:1.9b5) Gecko/X Firefox/3.0b5*/
const Device UA2141957 = {S22830, 65, { &P15364,&P17285,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; U; Windows NT 6.0) Gecko/X Firefox/3.0.3*/
const Device UA2221274 = {S22831, 152, { &P15364,&P17285,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.11) Gecko/X YFF3 Firefox/3.0.11 (.NET CLR 3.5.30729) WinNT-PAI 18.08.2009 SearchToolbar FBSMTWB*/
const Device UA2235373 = {S22832, 81, { &P15364,&P17285,&P3409,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.3) Gecko/X Firefox/3.0.3*/
const Device UA2259883 = {S22833, 104, { &P15364,&P17285,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 WinNT-PAI 30.06.2009*/
const Device UA2438506 = {S22834, 133, { &P15364,&P17285,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.16) Gecko/X Firefox/3.0.16 (.NET CLR 3.5.30729) WinNT-PAI 13.09.2009 FBSMTWB*/
const Device UA2753401 = {S22835, 85, { &P15364,&P17285,&P3409,&P18092 }};	/*Useragent - Mozi    lla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.3) Gecko/X Firefox/3.0.3*/
const Device UA3097770 = {S22836, 135, { &P15364,&P17285,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.17) Gecko/X Firefox/3.0.17 GTB6 (.NET CLR 3.5.30729) WinTSI 24.12.2009 FBSMTWB*/
const Device UA1951688 = {S22837, 145, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; LG-E610-Orange Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1961300 = {S22838, 138, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; LG-E612 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1974408 = {S22839, 138, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; LG-E610 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1978913 = {S22840, 139, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-mx; LG-E612f Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1980055 = {S22841, 143, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-E610/V10d Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1993433 = {S22842, 144, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; LG-E610v/V10c Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2006929 = {S22843, 144, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; LG-E610v/V10d Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025774 = {S22844, 143, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; LG-E610/V10a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026844 = {S22845, 139, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-sa; LG-E612g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028076 = {S22846, 60, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-E610 Build/IML74K)*/
const Device UA2028675 = {S22847, 135, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; et; LG-E610 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029845 = {S22848, 143, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-E610/V10e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2222426 = {S22849, 60, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-E612 Build/IML74K)*/
const Device UA2224638 = {S22850, 139, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; LG-E610v Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2341731 = {S22851, 143, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; LG-E610/V10f Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2375958 = {S22852, 61, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-E612f Build/IML74K)*/
const Device UA2390360 = {S22853, 143, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-E610/V10l Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2433359 = {S22854, 144, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; LG-E610v/V10f Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2451123 = {S22855, 61, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-E610v Build/IML74K)*/
const Device UA2454181 = {S22856, 61, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-E612g Build/IML74K)*/
const Device UA2578188 = {S22857, 143, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; LG-E610/V10c Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2584493 = {S22858, 143, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; LG-E610/V10b Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2586003 = {S22859, 144, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; LG-E610v/V10g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2694039 = {S22860, 143, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-E610/V10j Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2824939 = {S22861, 143, { &P18621,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-E610/V10g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA8528 = {S22862, 136, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA14293 = {S22863, 137, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18073 = {S22864, 137, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; GT-P1000L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38499 = {S22865, 96, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.2;Zh_cn;GT-P1000;600*1024;)AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA40952 = {S22866, 137, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-us; GT-P1000N Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA43728 = {S22867, 135, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-P1000R Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48981 = {S22868, 135, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-au; GT-P1000T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA53684 = {S22869, 135, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; pt-br; GT-P1000L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA64024 = {S22870, 134, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA64136 = {S22871, 135, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67044 = {S22872, 135, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-P1000M Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67819 = {S22873, 141, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Tablet Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA75312 = {S22874, 120, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.2;en-ie;GT-P1000Build/FROYO)AppleWebKit/533.1(KHTML,likeGecko)Version/4.0MobileSafari/533.1*/
const Device UA157650 = {S22875, 58, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; GT-P1000 Build/FROYO)*/
const Device UA178704 = {S22876, 59, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; GT-P1000T Build/FROYO)*/
const Device UA370762 = {S22877, 133, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370984 = {S22878, 70, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.2; en_GB; GT-P1000 Build/FROYO.XWJJ7)*/
const Device UA392080 = {S22879, 60, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-P1000 Build/FROYO)*/
const Device UA444386 = {S22880, 59, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; GT-P1000L Build/FROYO)*/
const Device UA543119 = {S22881, 59, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; GT-P1000N Build/FROYO)*/
const Device UA1858710 = {S22882, 59, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; GT-P1000M Build/FROYO)*/
const Device UA1926324 = {S22883, 124, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2018777 = {S22884, 132, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019201 = {S22885, 59, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; GT-P1000R Build/FROYO)*/
const Device UA2019383 = {S22886, 122, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; fa-fa; GT-P1000 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022332 = {S22887, 61, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-P1000N Build/FROYO)*/
const Device UA2026639 = {S22888, 127, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2028230 = {S22889, 129, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2402572 = {S22890, 148, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en2; SAMSUNG GT-P1000 Tablet Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2607572 = {S22891, 136, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2607583 = {S22892, 132, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.21.0; en-gb; GT-P1000) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2607593 = {S22893, 130, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux U Android 2.2 es-es GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2607631 = {S22894, 61, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-P1000L Build/FROYO)*/
const Device UA2607637 = {S22895, 133, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-; GT-P1000L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2607675 = {S22896, 149, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; SAMSUNG GT-P1000 Tablet Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2607682 = {S22897, 133, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2; en-au; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2607683 = {S22898, 70, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/0.9.1 (Linux; U; Android 2.2; en_GB; GT-P1000 Build/FROYO.JVJM5)*/
const Device UA2607800 = {S22899, 59, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.0; 1) GT-P1000 (Android 2.2, SDK 8)*/
const Device UA2607847 = {S22900, 160, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en_gb; GT-P1000 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)*/
const Device UA2607913 = {S22901, 133, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-; GT-P1000T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2607961 = {S22902, 70, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/0.9.3 (Linux; U; Android 2.2; en_GB; GT-P1000 Build/FROYO.DXJK2)*/
const Device UA2607969 = {S22903, 150, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO; 600*1024) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2608331 = {S22904, 132, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-P1000 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2609007 = {S22905, 142, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; nl-nl; GT-P1000 Build/Android.Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2609810 = {S22906, 60, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.0; 1) GT-P1000M (Android 2.2, SDK 8)*/
const Device UA2609988 = {S22907, 127, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build FROYO) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2610230 = {S22908, 129, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build FROYO) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2610318 = {S22909, 137, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-us; GT-P1000L Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2610534 = {S22910, 142, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; SAMSUNG GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2610999 = {S22911, 118, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-P1000) AntivirusAM/2.5 (KHTML, like Gecko) Version/Version 2.5/Antivirus*/
const Device UA2611263 = {S22912, 152, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO; 1024*600) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2611606 = {S22913, 280, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 [FBAN/FB4A;FBAV/1.9.3;FBDM/{density=1.5,width=600,height=1024};FBLC/en_GB;FB_FW/2;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P1000;FBSV/2.2;]*/
const Device UA2611645 = {S22914, 442, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/53uaprof/GT-P1000.xml, Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/53YS00NmI1LWE5ZjctZTQwY2EzYmYzOTQ00; ASP.NET_SessionId=aky0bl55ihlfv0aiiwp5ul45; __atuvc=3|20; language=en-GB; pay=9,-1,54,0,0; __utma=X.X.X.X.X.1; __utmz=X.X.1.1.utmcsr=google|utmccn=(organic)|utmcmd=organic|utmctr=r59%20truck%20spares; _em_hl=1; _em_vt=533b1d8000*/
const Device UA2612747 = {S22915, 74, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - BookShelf/XX-XXXXXXXXXXXX/Android 2.2/X-60:A1:0A:94:41:04/samsung GT-P1000*/
const Device UA2614066 = {S22916, 118, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mob*/
const Device UA2614172 = {S22917, 162, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-P1000N Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2614634 = {S22918, 379, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-au; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile bKit/533.1 (KHTML, like Gecko) VMOUS=RgJeycPJzQEkAAAANjc4NDk5MmEtMzYwNi00YmM5LTkwM2ItZjBhOGRkYjBiZTM40; language=en-US; WT_FPC=id=1.42.30.87-X.X:lv=X:ss=X; __utma=X.X.X.X.X.2; __utmb=X.5.10.X; __utmc=X; __utmz=X.X.2.2.utmcsr=careersearch.colesgroup.com.au|u*/
const Device UA2614754 = {S22919, 70, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Vyrso/2.1.2 (Linux; U; Android 2.2; en-GB; GT-P1000 Build/FROYO.JPJM5)*/
const Device UA2614771 = {S22920, 128, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-au; GT-P1000T Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2615256 = {S22921, 72, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Biblia/2.1.2 (Linux; U; Android 2.2; es-US; GT-P1000L Build/FROYO.UTJM3)*/
const Device UA2615677 = {S22922, 70, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Vyrso/2.1.2 (Linux; U; Android 2.2; en-GB; GT-P1000 Build/FROYO.ZHJK1)*/
const Device UA2615731 = {S22923, 135, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2615929 = {S22924, 72, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Biblia/2.1.2 (Linux; U; Android 2.2; es-US; GT-P1000L Build/FROYO.UMJK1)*/
const Device UA2616299 = {S22925, 72, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Biblia/2.1.2 (Linux; U; Android 2.2; es-US; GT-P1000L Build/FROYO.UTJM2)*/
const Device UA2616440 = {S22926, 72, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Biblia/2.1.2 (Linux; U; Android 2.2; es-US; GT-P1000L Build/FROYO.UMJJ3)*/
const Device UA2617175 = {S22927, 84, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.1.2 (Linux; U; Android 2.2; en-AU; GT-P1000 Build/FROYO.XWJJ4)*/
const Device UA2617215 = {S22928, 70, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 2.2; cs-CZ; GT-P1000 Build/FROYO.XWJMI)*/
const Device UA2617648 = {S22929, 134, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; pt-br; GT-P1000 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2617644 = {S22930, 74, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Biblia/2.2.3 (Linux; U; Android 2.2.1; es-US; GT-P1000N Build/FROYO.UBJM7)*/
const Device UA2617698 = {S22931, 72, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Biblia/1.0.3 (Linux; U; Android 2.2; es-US; GT-P1000L Build/FROYO.UTJJ5)*/
const Device UA2618073 = {S22932, 72, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Biblia/2.2.3 (Linux; U; Android 2.2; es-US; GT-P1000N Build/FROYO.VFJM2)*/
const Device UA2618081 = {S22933, 72, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Biblia/2.2.3 (Linux; U; Android 2.2; es-US; GT-P1000N Build/FROYO.UBJJ5)*/
const Device UA2618222 = {S22934, 133, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; de-; GT-P1000R Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2618368 = {S22935, 133, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-P1000L Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2618392 = {S22936, 133, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2618624 = {S22937, 134, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2618677 = {S22938, 71, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 2.2; en-CA; GT-P1000R Build/FROYO.UXJJ6)*/
const Device UA2618696 = {S22939, 73, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - GoogleAnalytics/2.0 (Linux; U; Android 2.2; en-ca; GT-P1000R Build/FROYO)*/
const Device UA2618814 = {S22940, 133, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-P1000N Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2618822 = {S22941, 133, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-P1000N) antivirusHttpClient/2.0 (KHTML, like Gecko) Version/Version 2.3/Antivirus pro*/
const Device UA2618885 = {S22942, 160, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-P1000M Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2618977 = {S22943, 103, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - com.google.android.youtube/2.4.4(Linux; U; Android 2.2; en_CA; GT-P1000M Build/FROYO) Experiment/1 gzip*/
const Device UA2618989 = {S22944, 79, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) Hotels.com/3.0*/
const Device UA2619066 = {S22945, 70, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Vyrso/2.2.3 (Linux; U; Android 2.2; en-GB; GT-P1000 Build/FROYO.JVJJ1)*/
const Device UA2619076 = {S22946, 77, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - MediaPlayer/2002/4.0.12 (Android 2.2; GT-P1000; GT-P1000; samsung; ; ; en_GB)*/
const Device UA2619155 = {S22947, 163, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Rdio.Net; Android/2.2.8; Android/2.2; DM=samsung; DN=GT-P1000M; DC=samsung; DF=samsung/GT-P1000M/GT-P1000M/GT-P1000M:2.2/FROYO/UGJJ7:user/release-keys; DH=GT-P1000*/
const Device UA2619722 = {S22948, 92, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - com.google.android.apps.plus/X (Linux; U; Android 2.2; en_CA; GT-P1000M; Build/FROYO) (gzip)*/
const Device UA2620008 = {S22949, 128, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Groupon/2.1.2483 (Android 2.2; Samsung GT-P1000R / Samsung GT-P1000R; )[preload=false;locale=en_CA;clientidbase=android-samsung]*/
const Device UA2620059 = {S22950, 144, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SAMSUNG GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2620978 = {S22951, 80, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; SAMSUNG GT-P1000 Tablet Build/MASTER)*/
const Device UA2621289 = {S22952, 143, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-ca; SAMSUNG GT-P1000M Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2621457 = {S22953, 280, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 [FBAN/FB4A;FBAV/1.6.4;FBPN/com.facebook.katana;FBDV/GT-P1000;FBSV/2.2;FBDM/{density=1.5,width=600,height=1024};FBCR/MY MAXIS;FBLC/en_GB;FB_FW/1;]*/
const Device UA2621562 = {S22954, 59, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.3; 4) GT-P1000 (Android 2.2, SDK 8)*/
const Device UA2621656 = {S22955, 272, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 [FBAN/FB4A;FBAV/1.9.7;FBDM/{density=1.5,width=600,height=1024};FBLC/en_GB;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P1000;FBSV/2.2;]*/
const Device UA2621705 = {S22956, 168, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (innerActive-ANDROID-2061)*/
const Device UA2622065 = {S22957, 143, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-ca; SAMSUNG GT-P1000R Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2622196 = {S22958, 161, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-P1000 Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2622351 = {S22959, 133, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; GT-P1000Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2622492 = {S22960, 72, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 2.2; en-GB; GT-P1000 Build/FROYO.XWJJ3)*/
const Device UA2623297 = {S22961, 86, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.2.8.1 (Linux; U; Android 2.2; en-GB; GT-P1000 Build/FROYO.DXKM2)*/
const Device UA2623347 = {S22962, 66, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-P1000R) BBC News/1.3*/
const Device UA2623348 = {S22963, 60, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.3; 4) GT-P1000R (Android 2.2, SDK 8)*/
const Device UA2623774 = {S22964, 178, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ar-ae; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2624698 = {S22965, 74, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 2.2.1; en-GB; GT-P1000 Build/FROYO.XWJMP)*/
const Device UA2624757 = {S22966, 44, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Scanner Radio/3.9.1 (Android 2.2, GT-P1000M)*/
const Device UA2625230 = {S22967, 143, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-au; SAMSUNG GT-P1000T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2626133 = {S22968, 96, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik 2.05.12114; (Linux; U; Android 2.2; GT-P1000R; ON) 9774d56d682e549c.KfLAGZ2j; 400x682x1.5*/
const Device UA2626216 = {S22969, 248, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 baidubrowser/042_1.41.3.2_diordna_006_4201/gnusmas_8_1.2.2_0001P-TG/Xb/A4EXF7CAB4F1C11C4FF0%7CX/1*/
const Device UA2626677 = {S22970, 161, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; pt_br; GT-P1000L Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)*/
const Device UA2626766 = {S22971, 68, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-P1000M) CBC News/1.0.5*/
const Device UA2626912 = {S22972, 152, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.Xc0b415a654c29b6e84*/
const Device UA2626919 = {S22973, 152, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.16255c1d48804bXf166*/
const Device UA2626959 = {S22974, 73, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - GoogleAnalytics/2.0 (Linux; U; Android 2.2; es-us; GT-P1000N Build/FROYO)*/
const Device UA2627193 = {S22975, 275, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Versionie: ASP.NET_SessionId=1ogopqx44t0mmbuajquc3iha; __utmmobile=0x6BD61FDEXB5; __utma=X.X.X.X.X.2; __utmc=X; __utmz=X.X.1.1.utmcsr=(direct)|utmccn=(direct)|utmcmd=(none)*/
const Device UA2627557 = {S22976, 149, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.177b4abXd4e8f030*/
const Device UA2627788 = {S22977, 49, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Android 2.2 samsung GT-P1000 FROYO.DXJK2;600X1024*/
const Device UA2629010 = {S22978, 79, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.01; 2) GT-P1000M (Android 2.2, SDK 8) flash:11.1.102.59*/
const Device UA2629202 = {S22979, 160, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-au; GT-P1000T Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2629215 = {S22980, 133, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-P1000T Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2631471 = {S22981, 44, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Scanner Radio/3.9.3 (Android 2.2, GT-P1000M)*/
const Device UA2632506 = {S22982, 151, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - OneBrowser/4.0.2/Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2632572 = {S22983, 160, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-P1000R Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2632694 = {S22984, 133, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; fr-fr;GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2633382 = {S22985, 57, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - nApps (Android 2.2; GT-P1000; androidapp.lineplay; 1.2.1)*/
const Device UA2633451 = {S22986, 134, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWt/533.1 (KHTML, like Gecko) VersVersion/4.0 Mobile Safari/533.1*/
const Device UA2635809 = {S22987, 99, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.2.1;Zh_cn;GT-P1000N;600*1024;)AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA2636368 = {S22988, 158, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-; GT-P1000M Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2636558 = {S22989, 150, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-US; SAMSUNG GT-P1000 Tablet Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2637863 = {S22990, 77, { &P13452,&P5096,&P3508,&P18092 }};	/*Useragent - APKXDL (Linux; U; Android 2.2;en_CA; GT-P1000M/FROYO)com.capcom.smurfsandroid*/
const Device UA136551 = {S22991, 70, { &P18254,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0 +http://www.bing.com/bingbot.htm)*/
const Device UA145981 = {S22992, 71, { &P18254,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm)*/
const Device UA224042 = {S22993, 69, { &P18254,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible;bingbot/2.0;+http://www.bing.com/bingbot.htm)*/
const Device UA430093 = {S22994, 143, { &P18254,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: flyproxy0)*/
const Device UA724207 = {S22995, 110, { &P18254,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm),gzip(gfe) (via docs.google.com/viewer)*/
const Device UA1927502 = {S22996, 215, { &P18254,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine CirruxCache CoralWebPrxMozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm),gzip(gfe) (via docs.google.com/viewer)*/
const Device UA1934168 = {S22997, 144, { &P18254,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: proxydisk9)*/
const Device UA2159157 = {S22998, 143, { &P18254,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: buttonses)*/
const Device UA2258664 = {S22999, 84, { &P18254,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (seoanalyzer; compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm)*/
const Device UA2443031 = {S23000, 144, { &P18254,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: proxyfile2)*/
const Device UA2487768 = {S23001, 70, { &P18254,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; Bingbot/2.0;+http://www.bing.com/bingbot.htm)*/
const Device UA2537523 = {S23002, 84, { &P18254,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) SitemapProbe*/
const Device UA2763518 = {S23003, 70, { &P18254,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0  http://www.bing.com/bingbot.htm)*/
const Device UA2810601 = {S23004, 145, { &P18254,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: scroll-down)*/
const Device UA900588 = {S23005, 58, { &P15364,&P7133,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:14.0) Gecko/X Firefox/14.0*/
const Device UA925274 = {S23006, 72, { &P15364,&P7133,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:14.0) Gecko/X Firefox/14.0.1*/
const Device UA925365 = {S23007, 82, { &P15364,&P7133,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:14.0) Gecko/X Firefox/14.0.1 Firefox/14.0.1*/
const Device UA1996626 = {S23008, 113, { &P15364,&P7133,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; WOW64; en-US; rv:2.0.4) Gecko/X AskTbAVR-IDW/3.12.5.17700 Firefox/14.0.1*/
const Device UA2686374 = {S23009, 81, { &P15364,&P7133,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.1; Win64; x64; rv:14.0) Gecko/X Firefox/14.0.1*/
const Device UA92387 = {S23010, 70, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - magpie-crawler/1.1 (U; Linux amd64; en-GB; +http://www.brandwatch.net)*/
const Device UA103869 = {S23011, 69, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - magpie-crawler/1.1 (U; Linux i386; en-GB; +http://www.brandwatch.net)*/
const Device UA248551 = {S23012, 74, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Linux; Socialradarbot/2.0; en-US; bot@infegy.com)*/
const Device UA309972 = {S23013, 105, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 2.6.32-5-amd64; java 1.6.0_18; Europe/de) http://yacy.net/bot.html*/
const Device UA349333 = {S23014, 106, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 2.6.32-5-amd64; java 1.6.0_18; America/en) http://yacy.net/bot.html*/
const Device UA351557 = {S23015, 103, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; i386 Linux 2.6.32-5-686; java 1.6.0_18; America/en) http://yacy.net/bot.html*/
const Device UA375721 = {S23016, 105, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 2.6.32-5-amd64; java 1.6.0_18; Europe/en) http://yacy.net/bot.html*/
const Device UA453333 = {S23017, 136, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) eyeVerifyBot (Linux x86_64) Webkit/Apple Safari/1.5 Safari/534.34*/
const Device UA512009 = {S23018, 82, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.8) nxsCrawler-09 http://null32.de*/
const Device UA554588 = {S23019, 105, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 2.6.32-5-amd64; java 1.6.0_26; Europe/en) http://yacy.net/bot.html*/
const Device UA588090 = {S23020, 105, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (webportal/global; amd64 Linux 2.6.32-5-amd64; java 1.6.0_18; Europe/en) http://yacy.net/bot.html*/
const Device UA625473 = {S23021, 104, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 2.6.32-custom; java 1.6.0_26; Europe/en) http://yacy.net/bot.html*/
const Device UA691328 = {S23022, 105, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 2.6.26-2-amd64; java 1.6.0_18; Europe/de) http://yacy.net/bot.html*/
const Device UA693585 = {S23023, 102, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; i386 Linux 2.6.32-5-686; java 1.6.0_18; Europe/en) http://yacy.net/bot.html*/
const Device UA700693 = {S23024, 102, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; i386 Linux 2.6.32-5-686; java 1.6.0_18; Europe/de) http://yacy.net/bot.html*/
const Device UA701548 = {S23025, 102, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; i386 Linux 2.6.32-5-686; java 1.6.0_18; Europe/fr) http://yacy.net/bot.html*/
const Device UA897262 = {S23026, 105, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld-global; amd64 Linux 2.6.32-5-amd64; java 1.6.0_18; Europe/en) http://yacy.net/bot.html*/
const Device UA920784 = {S23027, 112, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 2.6.32-5-openvz-amd64; java 1.6.0_18; Europe/en) http://yacy.net/bot.html*/
const Device UA1915289 = {S23028, 105, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (webportal/global; amd64 Linux 2.6.26-2-amd64; java 1.6.0_18; Europe/en) http://yacy.net/bot.html*/
const Device UA2164214 = {S23029, 107, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.2.0-23-generic; java 1.6.0_24; Europe/en) http://yacy.net/bot.html*/
const Device UA2167908 = {S23030, 110, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.1.10-1.16-desktop; java 1.6.0_24; Europe/de) http://yacy.net/bot.html*/
const Device UA2215976 = {S23031, 105, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 2.6.26-2-amd64; java 1.6.0_18; Europe/en) http://yacy.net/bot.html*/
const Device UA2262003 = {S23032, 107, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.2.0-31-generic; java 1.6.0_24; Europe/en) http://yacy.net/bot.html*/
const Device UA2336015 = {S23033, 107, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.2.0-23-generic; java 1.6.0_24; Europe/de) http://yacy.net/bot.html*/
const Device UA2354831 = {S23034, 107, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.2.0-32-generic; java 1.6.0_24; Europe/en) http://yacy.net/bot.html*/
const Device UA2367078 = {S23035, 103, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.5.7-gentoo; java 1.7.0_09; Europe/de) http://yacy.net/bot.html*/
const Device UA2367082 = {S23036, 110, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.1.10-1.16-default; java 1.6.0_24; Europe/de) http://yacy.net/bot.html*/
const Device UA2403699 = {S23037, 110, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.4.11-2.16-desktop; java 1.7.0_09; Europe/nl) http://yacy.net/bot.html*/
const Device UA2427832 = {S23038, 104, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; i386 Linux 3.1.10-slacko_4gA; java 1.7.0; Europe/de) http://yacy.net/bot.html*/
const Device UA2460571 = {S23039, 104, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld-global; amd64 Linux 3.2.0-4-amd64; java 1.6.0_24; Europe/en) http://yacy.net/bot.html*/
const Device UA2460579 = {S23040, 102, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (torworld/any; amd64 Linux 3.4.0-5-generic; java 1.7.0_09; Europe/en) http://yacy.net/bot.html*/
const Device UA2461050 = {S23041, 104, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.6.11-gentoo; java 1.7.0_09; Europe/de) http://yacy.net/bot.html*/
const Device UA2461351 = {S23042, 105, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.2.35-minimal; java 1.7.0_10; Europe/en) http://yacy.net/bot.html*/
const Device UA2471621 = {S23043, 107, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld-global; amd64 Linux 3.2.0-35-generic; java 1.7.0_09; Europe/en) http://yacy.net/bot.html*/
const Device UA2503313 = {S23044, 104, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.2.0-4-amd64; java 1.6.0_24; Europe/en) http://yacy.net/bot.html*/
const Device UA2554253 = {S23045, 110, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.4.11-2.16-desktop; java 1.7.0_09; Europe/de) http://yacy.net/bot.html*/
const Device UA2558265 = {S23046, 85, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (Linux 3.0.0-13-virtual x86_64) NettRapport Crawler (post@nettrapport.no)*/
const Device UA2594766 = {S23047, 104, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) RMSS_WEBKIT_CRAWLER Safari/534.34*/
const Device UA2596956 = {S23048, 104, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.2.0-4-amd64; java 1.7.0_03; Europe/en) http://yacy.net/bot.html*/
const Device UA2599551 = {S23049, 110, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.1.10-1.16-default; java 1.6.0_27; Europe/de) http://yacy.net/bot.html*/
const Device UA2668982 = {S23050, 104, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.7.10-gentoo; java 1.7.0_17; Europe/de) http://yacy.net/bot.html*/
const Device UA2673827 = {S23051, 85, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Linux i686; Yandex.Gazeta Bot/1.0; +http://gazeta.yandex.ru)*/
const Device UA2708921 = {S23052, 105, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (webportal-global; amd64 Linux 2.6.32-5-amd64; java 1.6.0_45; Europe/en) http://yacy.net/bot.html*/
const Device UA2708932 = {S23053, 107, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.7.10-gentoo-r1; java 1.7.0_17; Europe/de) http://yacy.net/bot.html*/
const Device UA2732743 = {S23054, 104, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.2.0-4-amd64; java 1.6.0_27; Europe/en) http://yacy.net/bot.html*/
const Device UA2738818 = {S23055, 87, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Linux x86_64; Mail.RU_Bot/2.0; +http://go.mail.ru/help/robots)*/
const Device UA2768615 = {S23056, 110, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.1.10-1.19-desktop; java 1.6.0_27; Europe/de) http://yacy.net/bot.html*/
const Device UA2769835 = {S23057, 108, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.8.0-22-generic; java 1.6.0_27; Pacific/en) http://yacy.net/bot.html*/
const Device UA2770765 = {S23058, 104, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.8.13-gentoo; java 1.7.0_17; Europe/de) http://yacy.net/bot.html*/
const Device UA2796387 = {S23059, 104, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.8.13-gentoo; java 1.7.0_21; Europe/de) http://yacy.net/bot.html*/
const Device UA2858795 = {S23060, 108, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.5.0-23-generic; java 1.6.0_27; America/en) http://yacy.net/bot.html*/
const Device UA2872331 = {S23061, 108, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.8.0-23-generic; java 1.6.0_27; Pacific/en) http://yacy.net/bot.html*/
const Device UA2872554 = {S23062, 103, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.7.9-gentoo; java 1.7.0_21; Europe/pl) http://yacy.net/bot.html*/
const Device UA2877432 = {S23063, 107, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 2.6.32-47-server; java 1.6.0_26; Europe/de) http://yacy.net/bot.html*/
const Device UA2878780 = {S23064, 104, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.2.0-4-amd64; java 1.6.0_27; Europe/de) http://yacy.net/bot.html*/
const Device UA2894928 = {S23065, 110, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.1.10-1.29-default; java 1.6.0_27; Europe/de) http://yacy.net/bot.html*/
const Device UA2895417 = {S23066, 107, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.7.10-gentoo-r1; java 1.7.0_21; Europe/pl) http://yacy.net/bot.html*/
const Device UA2898411 = {S23067, 107, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.8.0-25-generic; java 1.7.0_21; Europe/de) http://yacy.net/bot.html*/
const Device UA2901360 = {S23068, 77, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Linux i686; en-US; URLfilterDB-crawler/1.0) ufdb/1.0*/
const Device UA2933112 = {S23069, 51, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - WIRE/0.21 (Linux; x86_64; Bot,Robot,Spider,Crawler)*/
const Device UA2938336 = {S23070, 100, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.2.0-4-amd64; java 1.7.0_03; US/en) http://yacy.net/bot.html*/
const Device UA2945732 = {S23071, 107, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 2.6.32-49-server; java 1.6.0_27; Europe/en) http://yacy.net/bot.html*/
const Device UA2958517 = {S23072, 101, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.2.0-4-amd64; java 1.7.0_03; Etc/de) http://yacy.net/bot.html*/
const Device UA2971341 = {S23073, 96, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.9.7; java 1.7.0_25; Europe/en) http://yacy.net/bot.html*/
const Device UA2974904 = {S23074, 100, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.2.0-4-amd64; java 1.7.0_25; US/en) http://yacy.net/bot.html*/
const Device UA2974898 = {S23075, 105, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 2.6.32-5-amd64; java 1.7.0_21; Europe/en) http://yacy.net/bot.html*/
const Device UA3011893 = {S23076, 104, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld-global; amd64 Linux 3.2.0-4-amd64; java 1.6.0_27; Europe/de) http://yacy.net/bot.html*/
const Device UA3013002 = {S23077, 105, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; i386 Linux 3.2.0-4-686-pae; java 1.6.0_27; Europe/de) http://yacy.net/bot.html*/
const Device UA3021356 = {S23078, 103, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.5.0-23-generic; java 1.6.0_27; US/en) http://yacy.net/bot.html*/
const Device UA3023311 = {S23079, 104, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.8.13-gentoo; java 1.7.0_40; Europe/de) http://yacy.net/bot.html*/
const Device UA3024556 = {S23080, 107, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.8.0-27-generic; java 1.7.0_25; Europe/en) http://yacy.net/bot.html*/
const Device UA3025960 = {S23081, 101, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.10.1-pf; java 1.8.0-ea; America/en) http://yacy.net/bot.html*/
const Device UA3033454 = {S23082, 107, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.5.0-27-generic; java 1.7.0_25; Europe/en) http://yacy.net/bot.html*/
const Device UA3047613 = {S23083, 105, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; i386 Linux 3.2.0-4-686-pae; java 1.6.0_27; Europe/en) http://yacy.net/bot.html*/
const Device UA3057784 = {S23084, 104, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.2.0-4-amd64; java 1.7.0_25; Europe/en) http://yacy.net/bot.html*/
const Device UA3057917 = {S23085, 105, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (amd64 Linux 2.6.31-19-server; java 1.6.0_0; Europe/en) http://yacy.net/bot.html;c2VvLWNyYXdsZXI=*/
const Device UA3057936 = {S23086, 102, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (amd64 Linux 2.6.26-2-amd64; java 1.6.0_12; Zulu/en) http://yacy.net/bot.html;c2VvLWNyYXdsZXI=*/
const Device UA3058598 = {S23087, 107, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 2.6.32-50-server; java 1.6.0_26; Europe/de) http://yacy.net/bot.html*/
const Device UA3063889 = {S23088, 104, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.10.7-gentoo; java 1.7.0_40; Europe/de) http://yacy.net/bot.html*/
const Device UA3064214 = {S23089, 105, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (superfi/global; amd64 Linux 2.6.32-042stab079.5; java 1.6.0_27; Etc/en) http://yacy.net/bot.html*/
const Device UA3066649 = {S23090, 110, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; i386 Linux 3.2.41-042stab078.27; java 1.6.0_27; Europe/de) http://yacy.net/bot.html*/
const Device UA3083911 = {S23091, 110, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 2.6.32-028stab092.1; java 1.6.0_27; Europe/en) http://yacy.net/bot.html*/
const Device UA3092124 = {S23092, 104, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (amd64 Linux 2.6.28-18-generic; java 1.6.0_19; GMT/en) http://yacy.net/bot.html;c2VvLWNyYXdsZXI=*/
const Device UA3108060 = {S23093, 104, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (webportal-global; amd64 Linux 3.2.0-4-amd64; java 1.7.0_03; Europe/en) http://yacy.net/bot.html*/
const Device UA3108844 = {S23094, 110, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.7.10-1.16-desktop; java 1.7.0_40; Europe/de) http://yacy.net/bot.html*/
const Device UA3111764 = {S23095, 105, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (superfi/global; amd64 Linux 2.6.32-042stab079.6; java 1.6.0_27; Etc/en) http://yacy.net/bot.html*/
const Device UA3114960 = {S23096, 104, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (webportal-global; amd64 Linux 3.2.0-4-amd64; java 1.7.0_25; Europe/en) http://yacy.net/bot.html*/
const Device UA3120202 = {S23097, 104, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (webportal-global; amd64 Linux 3.2.0-4-amd64; java 1.6.0_27; Europe/en) http://yacy.net/bot.html*/
const Device UA3131723 = {S23098, 101, { &P18254,&P18110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Linux 3.2.0-4-amd64; java 1.7.0_25; Etc/de) http://yacy.net/bot.html*/
const Device UA614144 = {S23099, 75, { &P15364,&P7133,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:15.0) Gecko/15.0 Firefox/15.0a1*/
const Device UA900943 = {S23100, 60, { &P15364,&P7133,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:15.0) Gecko/X Firefox/15.0a2*/
const Device UA2043596 = {S23101, 82, { &P15364,&P7133,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X Firefox/15.0.1 Firefox/15.0.1*/
const Device UA2366165 = {S23102, 83, { &P15364,&P7133,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:15.0) Gecko/X Firefox/15.0 Firefox/15.0*/
const Device UA2383459 = {S23103, 80, { &P15364,&P7133,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X Firefox/15.0.1*/
const Device UA3023302 = {S23104, 72, { &P15364,&P7133,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; rv:15.0) Gecko/X Firefox/15.0.1*/
const Device UA454823 = {S23105, 139, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA608842 = {S23106, 152, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/NOCTURNAL VENUM Rom) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA624153 = {S23107, 139, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SPH-D710 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA686981 = {S23108, 139, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA698634 = {S23109, 61, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SPH-D710 Build/IML74K)*/
const Device UA797917 = {S23110, 61, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SPH-D710 Build/IMM76I)*/
const Device UA845445 = {S23111, 139, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1962729 = {S23112, 138, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SPH-D710 Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1972418 = {S23113, 147, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/SPH.D710.SNROM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1981099 = {S23114, 147, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/HyDrA_Rom_v0.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1984436 = {S23115, 139, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025031 = {S23116, 144, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/Slim-n-Trim) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030307 = {S23117, 139, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2039701 = {S23118, 142, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710BST Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2231970 = {S23119, 64, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SPH-D710BST Build/IMM76I)*/
const Device UA2257858 = {S23120, 145, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2365630 = {S23121, 143, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710VMUB Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2463874 = {S23122, 137, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SPH-D710 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2466696 = {S23123, 202, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 androidAppVersion/2.5.16 AndroidWebkitWrapper BillingSupported*/
const Device UA2467307 = {S23124, 65, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SPH-D710VMUB Build/IMM76I)*/
const Device UA2820935 = {S23125, 130, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SPH-D710 Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2846919 = {S23126, 134, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SPH-D710 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 ??Safari/534.30*/
const Device UA3109333 = {S23127, 170, { &P16362,&P18066,&P18062,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.0.4;dev=SPH-D710;devm=SPH-D710;man=samsung;brw=Android;shwd=480x800;shwa=480x763;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA427334 = {S23128, 103, { &P17346,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176*/
const Device UA441534 = {S23129, 246, { &P17346,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 5_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/5.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA615680 = {S23130, 92, { &P17346,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)*/
const Device UA615681 = {S23131, 250, { &P17346,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 5_1_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/5.1.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA615687 = {S23132, 105, { &P17346,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206*/
const Device UA616215 = {S23133, 131, { &P17346,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 iPod4,1 BingWeb/3.03.1428.X*/
const Device UA625788 = {S23134, 90, { &P17346,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)*/
const Device UA691048 = {S23135, 133, { &P17346,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 iPod3,1 BingWeb/3.03.1428.X*/
const Device UA1986631 = {S23136, 153, { &P17346,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper*/
const Device UA2041678 = {S23137, 229, { &P17346,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/5.1.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]*/
const Device UA2178193 = {S23138, 138, { &P17346,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPodTouch4G; iPhone; U; CPU OS 5_1_1 like Mac OS X; en_US) com.google.GooglePlus/8654 (KHTML, like Gecko) Mobile/N81AP (gzip)*/
const Device UA2202492 = {S23139, 225, { &P17346,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/5.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]*/
const Device UA2218255 = {S23140, 103, { &P17346,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405*/
const Device UA2325040 = {S23141, 151, { &P17346,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper*/
const Device UA2419408 = {S23142, 142, { &P17346,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone 5_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPod3,1/5.1.1)*/
const Device UA2544037 = {S23143, 122, { &P17346,&P18117,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA3100884 = {S23144, 50, { &P17346,&P18117,&P17596,&P18092 }};	/*Useragent - Zeusmos 2.6.9 (iPod touch; iPhone OS 5.1.1; en_US)*/
const Device UA24508 = {S23145, 144, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Sprint APA9292KT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24509 = {S23146, 135, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PC36100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48588 = {S23147, 144, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Sprint APA9292KT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49851 = {S23148, 142, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; -us; Sprint APA9292KT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70806 = {S23149, 135, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; PC36100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73839 = {S23150, 144, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Sprint APA9292KT Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA169749 = {S23151, 135, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; PC36100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA180323 = {S23152, 135, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PC36100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA196842 = {S23153, 59, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; PC36100 Build/GRI40)*/
const Device UA199101 = {S23154, 135, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PC36100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA236619 = {S23155, 135, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA248972 = {S23156, 135, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PC36100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA298603 = {S23157, 135, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; PC36100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA314548 = {S23158, 135, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PC36100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368919 = {S23159, 133, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; PC36100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369050 = {S23160, 142, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/GRJ90) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA369328 = {S23161, 70, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.5; en_US; PC36100 Build/4.67.651.3)*/
const Device UA398288 = {S23162, 144, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Sprint APA9292KT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622587 = {S23163, 144, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Sprint APA9292KT Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622941 = {S23164, 135, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627479 = {S23165, 59, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; PC36100 Build/GRJ90)*/
const Device UA651029 = {S23166, 59, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; PC36100 Build/GRI40)*/
const Device UA653162 = {S23167, 135, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA710339 = {S23168, 59, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; PC36100 Build/GRJ22)*/
const Device UA842734 = {S23169, 142, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; Sprint APA9292KT Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA879738 = {S23170, 142, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; -us; Sprint APA9292KT Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1886375 = {S23171, 142, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PC36100 Build/GRI40) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA1892071 = {S23172, 133, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; PC36100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1950326 = {S23173, 139, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; EVO 4G Plus Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2004801 = {S23174, 141, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; EVO Build/GRI40; CIUI-HTC) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020660 = {S23175, 123, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; PC36100 Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022007 = {S23176, 167, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en_us; PC36100 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)*/
const Device UA2024781 = {S23177, 128, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; PC36100 Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2031789 = {S23178, 133, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; PC36100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2235793 = {S23179, 143, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/GRJ90) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30*/
const Device UA2582129 = {S23180, 160, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; PC36100 Build/thundershock; ThunderShock v.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2597720 = {S23181, 143, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PC36100-HTC EVO Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2706588 = {S23182, 160, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PC36100 Build/Sense and AOSP; SensuAOSP 2.55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2743601 = {S23183, 138, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ar-eg; HTC EVO 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2751912 = {S23184, 160, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Gingerbread 2.3.7; en-us; PC36100 Build/GRI40; Unity 0.0.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2759291 = {S23185, 166, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ar-eg; EVO 4G Build/GRI40; Dream Rise_EVO4G_2.3.7_X_ANZHI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3117506 = {S23186, 129, { &P13644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; EVO 4G Build/unknown) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2202662 = {S23187, 57, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.10*/
const Device UA2351873 = {S23188, 68, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:2.0) Gecko/X Firefox/4.0 Opera 12.10*/
const Device UA2353562 = {S23189, 57, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2) Presto/2.12.388 Version/12.10*/
const Device UA2354139 = {S23190, 69, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; Edition Yx) Presto/2.12.388 Version/12.10*/
const Device UA2354459 = {S23191, 64, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; WOW64) Presto/2.12.388 Version/12.10*/
const Device UA2355037 = {S23192, 62, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1) Opera 12.10*/
const Device UA2355359 = {S23193, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.10*/
const Device UA2355501 = {S23194, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.10*/
const Device UA2356404 = {S23195, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.10*/
const Device UA2356627 = {S23196, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.10*/
const Device UA2357289 = {S23197, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.10*/
const Device UA2357290 = {S23198, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.10*/
const Device UA2357306 = {S23199, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.10*/
const Device UA2357524 = {S23200, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.10*/
const Device UA2357538 = {S23201, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.10*/
const Device UA2357697 = {S23202, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.10*/
const Device UA2357915 = {S23203, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.10*/
const Device UA2357997 = {S23204, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.10*/
const Device UA2358000 = {S23205, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.10*/
const Device UA2358022 = {S23206, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.10*/
const Device UA2358207 = {S23207, 67, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.10,gzip(gfe)*/
const Device UA2358362 = {S23208, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.10*/
const Device UA2358382 = {S23209, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.10*/
const Device UA2358589 = {S23210, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.10*/
const Device UA2358831 = {S23211, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.10*/
const Device UA2359087 = {S23212, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.10*/
const Device UA2359089 = {S23213, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.10*/
const Device UA2359091 = {S23214, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.10*/
const Device UA2359096 = {S23215, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.10*/
const Device UA2359106 = {S23216, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.10*/
const Device UA2359529 = {S23217, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.10*/
const Device UA2360796 = {S23218, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.10*/
const Device UA2360825 = {S23219, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.10*/
const Device UA2360847 = {S23220, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.10*/
const Device UA2361026 = {S23221, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.10*/
const Device UA2361109 = {S23222, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.10*/
const Device UA2361129 = {S23223, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03789)) Presto/2.12.388 Version/12.10*/
const Device UA2361299 = {S23224, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.10*/
const Device UA2361322 = {S23225, 74, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.10 Yandex.Translate*/
const Device UA2361345 = {S23226, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.10*/
const Device UA2361376 = {S23227, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.10*/
const Device UA2361490 = {S23228, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.10*/
const Device UA2361558 = {S23229, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.10*/
const Device UA2361644 = {S23230, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.10*/
const Device UA2361752 = {S23231, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.10*/
const Device UA2361973 = {S23232, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.10*/
const Device UA2362093 = {S23233, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.10*/
const Device UA2367443 = {S23234, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.10*/
const Device UA2367576 = {S23235, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.10*/
const Device UA2368004 = {S23236, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.10*/
const Device UA2368218 = {S23237, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.10*/
const Device UA2368354 = {S23238, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.10*/
const Device UA2368476 = {S23239, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.10*/
const Device UA2368485 = {S23240, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.10*/
const Device UA2368530 = {S23241, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.10*/
const Device UA2369696 = {S23242, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.10*/
const Device UA2386648 = {S23243, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.10*/
const Device UA2386821 = {S23244, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4933)) Presto/2.12.388 Version/12.10*/
const Device UA2386857 = {S23245, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.10*/
const Device UA2386980 = {S23246, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.10*/
const Device UA2387039 = {S23247, 88, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; Edition Campaign 21) Presto/2.12.388 Version/12.10,gzip(gfe)*/
const Device UA2403763 = {S23248, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5230)) Presto/2.12.388 Version/12.10*/
const Device UA2403827 = {S23249, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.10*/
const Device UA2404094 = {S23250, 106, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.10*/
const Device UA2404242 = {S23251, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03658)) Presto/2.12.388 Version/12.10*/
const Device UA2424076 = {S23252, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5288)) Presto/2.12.388 Version/12.10*/
const Device UA2426609 = {S23253, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.10*/
const Device UA2437304 = {S23254, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.10*/
const Device UA2456403 = {S23255, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5650)) Presto/2.12.388 Version/12.10*/
const Device UA2456474 = {S23256, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.10*/
const Device UA2468438 = {S23257, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.10*/
const Device UA2484220 = {S23258, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.10*/
const Device UA2554240 = {S23259, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.10*/
const Device UA2596548 = {S23260, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.10*/
const Device UA2601198 = {S23261, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.10*/
const Device UA2654318 = {S23262, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4110)) Presto/2.12.388 Version/12.10*/
const Device UA2665091 = {S23263, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.10*/
const Device UA2695083 = {S23264, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5278)) Presto/2.12.388 Version/12.10*/
const Device UA2704657 = {S23265, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.10*/
const Device UA2708283 = {S23266, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.10*/
const Device UA2727399 = {S23267, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03402)) Presto/2.12.388 Version/12.10*/
const Device UA2749444 = {S23268, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.10*/
const Device UA2790372 = {S23269, 80, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03639)) Presto/2.12.388 Version/12.10*/
const Device UA2791992 = {S23270, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.10*/
const Device UA2800516 = {S23271, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.10*/
const Device UA2840853 = {S23272, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5660)) Presto/2.12.388 Version/12.10*/
const Device UA2841407 = {S23273, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4832)) Presto/2.12.388 Version/12.10*/
const Device UA2841473 = {S23274, 107, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03402); MRSPUTNIK OW 2, 3, 0, 104) Presto/2.12.388 Version/12.10*/
const Device UA2852613 = {S23275, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.10*/
const Device UA2934610 = {S23276, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.10*/
const Device UA2991627 = {S23277, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.10*/
const Device UA2991907 = {S23278, 79, { &P15364,&P5690,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.10*/
const Device UA156132 = {S23279, 58, { &P15364,&P17350,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; MSIE 9.0; WIndows NT 9.0; en-US))*/
const Device UA160310 = {S23280, 57, { &P15364,&P17350,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; MSIE 9.0; Windows NT 9.0; en-US)*/
const Device UA172970 = {S23281, 54, { &P15364,&P17350,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Win32; Trident/5.0)*/
const Device UA196096 = {S23282, 63, { &P15364,&P17350,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 7.1; Trident/5.0)*/
const Device UA389424 = {S23283, 54, { &P15364,&P17350,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Win64; Trident/5.0)*/
const Device UA440401 = {S23284, 50, { &P15364,&P17350,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; MSIE 9.0; Windows NT 9.0)*/
const Device UA685677 = {S23285, 57, { &P15364,&P17350,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 9.0; en-US)*/
const Device UA2330134 = {S23286, 120, { &P15364,&P17350,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; MSIE 9.0; Windows NT 9.0; en-US)Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 7.1; Trident/5.0)*/
const Device UA2440193 = {S23287, 60, { &P15364,&P17350,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; U; MSIE 9.0; WIndows NT 9.0; en-US)*/
const Device UA2502322 = {S23288, 58, { &P15364,&P17350,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; MSIE 9.0; WIndows NT 9.0; de-DE))*/
const Device UA2509636 = {S23289, 52, { &P15364,&P17350,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; 68K; Trident/3.0)*/
const Device UA2639346 = {S23290, 57, { &P15364,&P17350,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; MSIE 9.0; WIndows NT 9.0; en-GB)*/
const Device UA2639671 = {S23291, 57, { &P15364,&P17350,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; MSIE 9.0; WIndows NT 9.0; de-DE)*/
const Device UA2639816 = {S23292, 57, { &P15364,&P17350,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; MSIE 9.0; WIndows NT 9.0; en-CA)*/
const Device UA2699907 = {S23293, 59, { &P15364,&P17350,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; 68K; WOW64; Trident/3.0)*/
const Device UA2726845 = {S23294, 45, { &P15364,&P17350,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (MSIE 9.0; Windows NT 9.0; en-US)*/
const Device UA2727596 = {S23295, 57, { &P15364,&P17350,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; MSIE 9.0; WIndows NT 9.0; zh-cn)*/
const Device UA2791751 = {S23296, 64, { &P15364,&P17350,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; PPC; Win64; x64; Trident/1.0)*/
const Device UA2792165 = {S23297, 57, { &P15364,&P17350,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; 68K; SV1; Trident/5.0)*/
const Device UA2792792 = {S23298, 59, { &P15364,&P17350,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; PPC; WOW64; Trident/3.0)*/
const Device UA2849286 = {S23299, 62, { &P15364,&P17350,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; PPC; .NET CLR; Trident/5.0)*/
const Device UA29953 = {S23300, 147, { &P12280,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A400 Safari/6531.22.7*/
const Device UA29971 = {S23301, 147, { &P12280,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A306 Safari/6531.22.7*/
const Device UA29975 = {S23302, 148, { &P12280,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.4 Mobile/8A260b Safari/531.21.10*/
const Device UA64003 = {S23303, 145, { &P12280,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293 Safari/6531.22.7*/
const Device UA131611 = {S23304, 148, { &P12280,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A400a Safari/6531.22.7*/
const Device UA171339 = {S23305, 128, { &P12280,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293*/
const Device UA300808 = {S23306, 145, { &P12280,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/9Z999 Safari/6531.22.7*/
const Device UA303167 = {S23307, 147, { &P12280,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/9Z999 Safari/6531.22.7*/
const Device UA372610 = {S23308, 141, { &P12280,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/8A293 Safari/528.16*/
const Device UA433956 = {S23309, 144, { &P12280,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293 Safari/531.22.7*/
const Device UA482639 = {S23310, 148, { &P12280,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293 Safari/6531.22.7*/
const Device UA485364 = {S23311, 144, { &P12280,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.05 Mobile/8A293 Safari/6531.22.7*/
const Device UA677659 = {S23312, 147, { &P12280,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/9Z999 Safari/6531.22.7*/
const Device UA789651 = {S23313, 141, { &P12280,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA1851711 = {S23314, 147, { &P12280,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA1861354 = {S23315, 145, { &P12280,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA1865063 = {S23316, 157, { &P12280,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/525.13 (KHTML, like Gecko; Google Web Preview) Version/4.0.5 Mobile Safari/525.13*/
const Device UA1983369 = {S23317, 144, { &P12280,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X: en-us) AppleWebKit/532.9  (KHTML, like Gecko) Version/4.0 Mobile/8A293 Safari/6531.22.7*/
const Device UA1995365 = {S23318, 175, { &P12280,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; en-us) AppleWebKit/536.8 (KHTML, like Gecko; Google Page Speed Insights) Version/4.0.5 Mobile/8A306 Safari/6531.22.7*/
const Device UA2234759 = {S23319, 175, { &P12280,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; en-us) AppleWebKit/537.4 (KHTML, like Gecko; Google Page Speed Insights) Version/4.0.5 Mobile/8A306 Safari/6531.22.7*/
const Device UA2662010 = {S23320, 143, { &P12280,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 40 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML like Gecko) Version/4.0.5 Mobile/8A293 Safari/6531.22.7*/
const Device UA2664353 = {S23321, 144, { &P12280,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 401 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML like Gecko) Version/4.0.5 Mobile/8A306 Safari/6531.22.7*/
const Device UA605594 = {S23322, 139, { &P16349,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sony Tablet S Build/TISU0077) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA899219 = {S23323, 139, { &P16349,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Sony Tablet S Build/TISU0084) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA909312 = {S23324, 139, { &P16349,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Sony Tablet S Build/TISU0093) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2003512 = {S23325, 141, { &P16349,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Sony Tablet S Build/TISU0R0110) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032272 = {S23326, 146, { &P16349,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sony Tablet S Build/TISU0077) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033884 = {S23327, 139, { &P16349,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Sony Tablet S Build/TISU0085) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2198533 = {S23328, 139, { &P16349,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sony Tablet S Build/TISU0124) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2229370 = {S23329, 148, { &P16349,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; Turbo-X Tablet Spice III Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2406597 = {S23330, 139, { &P16349,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sony Tablet S Build/TISU0143) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2434023 = {S23331, 68, { &P16349,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Sony Tablet S Build/TISU0143)*/
const Device UA2461272 = {S23332, 137, { &P16349,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Tablet Scooby Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2845833 = {S23333, 146, { &P16349,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sony Tablet S Build/TISU0143) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2951464 = {S23334, 137, { &P16349,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; Sony Tablet S Build/TISU0143) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2568259 = {S23335, 143, { &P12280,&P19826,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B146 Safari/8536.25*/
const Device UA2568318 = {S23336, 141, { &P12280,&P19826,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B144 Safari/8536.25*/
const Device UA2568325 = {S23337, 141, { &P12280,&P19826,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B142 Safari/8536.25*/
const Device UA2568329 = {S23338, 141, { &P12280,&P19826,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B143 Safari/8536.25*/
const Device UA2568359 = {S23339, 143, { &P12280,&P19826,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B145 Safari/8536.25*/
const Device UA2568365 = {S23340, 141, { &P12280,&P19826,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B141 Safari/8536.25*/
const Device UA2585372 = {S23341, 143, { &P12280,&P19826,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B329 Safari/8536.25*/
const Device UA2605882 = {S23342, 143, { &P12280,&P19826,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B318 Safari/8536.25*/
const Device UA2709284 = {S23343, 143, { &P12280,&P19826,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B350 Safari/8536.25*/
const Device UA2437337 = {S23344, 133, { &P17346,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B5126b Safari/8536.25*/
const Device UA2456836 = {S23345, 133, { &P17346,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B5105c Safari/8536.25*/
const Device UA2487506 = {S23346, 131, { &P17346,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA2490677 = {S23347, 131, { &P17346,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B144 Safari/8536.25*/
const Device UA2532516 = {S23348, 133, { &P17346,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146 Safari/8536.25*/
const Device UA2583799 = {S23349, 133, { &P17346,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329 Safari/8536.25*/
const Device UA2891464 = {S23350, 133, { &P17346,&P19826,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B329 Safari/8536.25*/
const Device UA117949 = {S23351, 117, { &P15364,&P7133,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.125 Safari/533.4*/
const Device UA2125535 = {S23352, 154, { &P15364,&P7133,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.125 Safari/533.4,gzip(gfe) (via translate.google.com)*/
const Device UA2392920 = {S23353, 115, { &P15364,&P7133,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-us) AppleWebKit/533.2 (KHTML, like Gecko) Chrome/5.0.342.7 Safari/533.2*/
const Device UA2457870 = {S23354, 90, { &P15364,&P7133,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/533 (KHTML, like Gecko) Chrome/5.0.375 Safari/533*/
const Device UA2503314 = {S23355, 126, { &P15364,&P7133,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.86 Safari/533.4,gzip(gfe)*/
const Device UA2812848 = {S23356, 117, { &P15364,&P7133,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; AppleWebKit/533.3; KHTML?? like Gecko; Chrome/5.0.354.0 Safari/533.3)*/
const Device UA426715 = {S23357, 132, { &P12972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T989 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA454294 = {S23358, 147, { &P12972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SAMSUNG-SGH-T989 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA456605 = {S23359, 132, { &P12972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T989 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA652475 = {S23360, 139, { &P12972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T989 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA663944 = {S23361, 61, { &P12972,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SGH-T989 Build/IML74K)*/
const Device UA680628 = {S23362, 131, { &P12972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T989 Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA868116 = {S23363, 139, { &P12972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T989 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA893905 = {S23364, 139, { &P12972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T989 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA895984 = {S23365, 147, { &P12972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-T989 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA901643 = {S23366, 140, { &P12972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; SGH-T989D Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1981476 = {S23367, 62, { &P12972,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SGH-T989D Build/IML74K)*/
const Device UA1993197 = {S23368, 147, { &P12972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-T989D-parrot Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1998926 = {S23369, 140, { &P12972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-T989D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025231 = {S23370, 132, { &P12972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T989 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2025467 = {S23371, 69, { &P12972,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SAMSUNG-SGH-T989 Build/IMM76L)*/
const Device UA2027699 = {S23372, 136, { &P12972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T989 Build/5.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031615 = {S23373, 145, { &P12972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T989 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2034352 = {S23374, 61, { &P12972,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-T989 Build/IMM76D)*/
const Device UA2390654 = {S23375, 62, { &P12972,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-T989D Build/IMM76D)*/
const Device UA2552175 = {S23376, 137, { &P12972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SGH-T989 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2589308 = {S23377, 137, { &P12972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; SGH-T989 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2693336 = {S23378, 145, { &P12972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SGH-T989D-parrot Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2770342 = {S23379, 111, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4191.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1520.2 Safari/537.36*/
const Device UA2773272 = {S23380, 109, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4191.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1520.2 Safari/537.36*/
const Device UA2773281 = {S23381, 111, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4191.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1520.2 Safari/537.36*/
const Device UA2870901 = {S23382, 109, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4287.1.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1541.3 Safari/537.36*/
const Device UA2875715 = {S23383, 111, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4287.1.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1541.3 Safari/537.36*/
const Device UA2876111 = {S23384, 109, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4287.2.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1541.3 Safari/537.36*/
const Device UA2876597 = {S23385, 111, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4287.1.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1541.3 Safari/537.36*/
const Device UA2885335 = {S23386, 109, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4319.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.2 Safari/537.36*/
const Device UA2885347 = {S23387, 111, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4319.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.2 Safari/537.36*/
const Device UA2887917 = {S23388, 111, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4319.0.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.2 Safari/537.36*/
const Device UA2888358 = {S23389, 111, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4319.3.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.2 Safari/537.36*/
const Device UA2888661 = {S23390, 109, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4319.3.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.2 Safari/537.36*/
const Device UA2946045 = {S23391, 111, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4319.16.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.16 Safari/537.36*/
const Device UA2948036 = {S23392, 113, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4319.16.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.16 Safari/537.36*/
const Device UA2948317 = {S23393, 113, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4319.16.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.16 Safari/537.36*/
const Device UA2972563 = {S23394, 113, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4319.28.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.22 Safari/537.36*/
const Device UA2972562 = {S23395, 111, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4319.28.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.22 Safari/537.36*/
const Device UA2972564 = {S23396, 113, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4319.28.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.22 Safari/537.36*/
const Device UA2973785 = {S23397, 111, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4319.32.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.23 Safari/537.36*/
const Device UA2974738 = {S23398, 113, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4319.32.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.23 Safari/537.36*/
const Device UA2974741 = {S23399, 113, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4319.32.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.23 Safari/537.36*/
const Device UA2975893 = {S23400, 113, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4319.39.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.32 Safari/537.36*/
const Device UA2975903 = {S23401, 113, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4319.39.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.32 Safari/537.36*/
const Device UA2975909 = {S23402, 111, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4319.39.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.32 Safari/537.36*/
const Device UA3028224 = {S23403, 113, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4319.52.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.41 Safari/537.36*/
const Device UA3028233 = {S23404, 111, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4319.52.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.41 Safari/537.36*/
const Device UA3030752 = {S23405, 113, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4319.52.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.41 Safari/537.36*/
const Device UA3033003 = {S23406, 113, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4319.74.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36*/
const Device UA3033180 = {S23407, 113, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4319.74.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36*/
const Device UA3033438 = {S23408, 111, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4319.74.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36*/
const Device UA3036829 = {S23409, 113, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4319.59.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.49 Safari/537.36*/
const Device UA3036834 = {S23410, 113, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4319.59.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.49 Safari/537.36*/
const Device UA3036957 = {S23411, 111, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4319.59.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.49 Safari/537.36*/
const Device UA3037090 = {S23412, 113, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4319.65.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.56 Safari/537.36*/
const Device UA3037091 = {S23413, 113, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4319.65.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.56 Safari/537.36*/
const Device UA3037102 = {S23414, 111, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4319.65.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.56 Safari/537.36*/
const Device UA3063569 = {S23415, 113, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4319.79.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.70 Safari/537.36*/
const Device UA3063570 = {S23416, 111, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4319.79.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.70 Safari/537.36*/
const Device UA3063578 = {S23417, 113, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4319.79.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.70 Safari/537.36*/
const Device UA3095672 = {S23418, 113, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 4319.96.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.74 Safari/537.36*/
const Device UA3095673 = {S23419, 111, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 4319.96.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.74 Safari/537.36*/
const Device UA3095674 = {S23420, 113, { &P15364,&P6596,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 4319.96.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.74 Safari/537.36*/
const Device UA33283 = {S23421, 130, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500R Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA33386 = {S23422, 130, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500D Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA48529 = {S23423, 148, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; SAMSUNG-GT-P7500/P7500BUKG7 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA48535 = {S23424, 148, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; de-de; SAMSUNG-GT-P7500/P7500BUKG5 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA48543 = {S23425, 148, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; de-de; SAMSUNG-GT-P7500/P7500BUKH1 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA61565 = {S23426, 129, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA68979 = {S23427, 130, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500M Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA73731 = {S23428, 148, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; SAMSUNG-GT-P7500/P7500BUKI1 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA207950 = {S23429, 58, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; GT-P7500 Build/HMJ37)*/
const Device UA396193 = {S23430, 136, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA569299 = {S23431, 59, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; GT-P7500R Build/HMJ37)*/
const Device UA2019872 = {S23432, 59, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; GT-P7500M Build/HMJ37)*/
const Device UA2022237 = {S23433, 150, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37; 1280*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2215833 = {S23434, 127, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2607598 = {S23435, 152, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; cs-cz; SAMSUNG-GT-P7500/P7500BUKEE Build/HONEYCOMB) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2607973 = {S23436, 128, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-; GT-P7500R Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2607995 = {S23437, 59, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; GT-P7500D Build/HMJ37)*/
const Device UA2608574 = {S23438, 148, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; el-gr; SAMSUNG-GT-P7500/P7500BUKH4 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2609429 = {S23439, 134, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; se-se; GT-P7500 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2609723 = {S23440, 122, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2610169 = {S23441, 159, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2610235 = {S23442, 128, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2610907 = {S23443, 122, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500) antivirusAM/2.4.5 (KHTML, like Gecko) Version/Version 2.4.5/antivirus*/
const Device UA2611029 = {S23444, 127, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2611482 = {S23445, 55, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Android/3.9.1 (Linux; U; Android 3.1; en_CA; GT-P7500R)*/
const Device UA2613895 = {S23446, 65, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Logos/2.0.11 (Linux; U; Android 3.1; en-GB; GT-P7500 Build/HMJ37)*/
const Device UA2615666 = {S23447, 60, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.0; 1) GT-P7500 (Android 3.1, SDK 12)*/
const Device UA2616207 = {S23448, 70, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 3.1; es-ES; GT-P7500 Build/HMJ37.UBKG9)*/
const Device UA2616257 = {S23449, 200, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; nl-be; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 TwonkyBeamBrowser/3.2.0-57 (Android 3.1; samsung GT-P7500 Build/HMJ37)*/
const Device UA2616393 = {S23450, 148, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; SAMSUNG-GT-P7500/P7500BUKEG Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2617357 = {S23451, 136, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2617748 = {S23452, 141, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; sv-se; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version /4.0 Safari/534.13 AndroidApp*/
const Device UA2618058 = {S23453, 92, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 3.1; SAMSUNG GT-P7500 Build/HONEYCOMB_MR1);*/
const Device UA2618482 = {S23454, 129, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2618535 = {S23455, 139, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500 Build/HMJ37 like Xoom) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2618797 = {S23456, 73, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - GoogleAnalytics/2.0 (Linux; U; Android 3.1; en-ca; GT-P7500R Build/HMJ37)*/
const Device UA2619019 = {S23457, 66, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500M) BBC News/1.0*/
const Device UA2619020 = {S23458, 61, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.0; 5) GT-P7500M (Android 3.1, SDK 12)*/
const Device UA2619026 = {S23459, 275, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.6.3;FBPN/com.facebook.katana;FBDV/GT-P7500;FBSV/3.1;FBDM/{density=1.0,width=1280,height=800};FBCR/MY MAXIS;FBLC/en_US;FB_FW/1;]*/
const Device UA2619069 = {S23460, 77, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - MediaPlayer/2002/4.0.12 (Android 3.1; GT-P7500; GT-P7500; samsung; ; ; en_GB)*/
const Device UA2619114 = {S23461, 70, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 3.1; en-US; GT-P7500 Build/HMJ37.ZSKH2)*/
const Device UA2619289 = {S23462, 77, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - MediaPlayer/2002/4.0.12 (Android 3.1; GT-P7500; GT-P7500; samsung; ; ; ms_MY)*/
const Device UA2619428 = {S23463, 71, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Logos/2.0.11 (Linux; U; Android 3.1; pt-BR; GT-P7500 Build/HMJ37.UBKH1)*/
const Device UA2619445 = {S23464, 70, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 3.1; en-US; GT-P7500 Build/HMJ37.UBKG6)*/
const Device UA2619502 = {S23465, 126, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Groupon/2.1.2483 (Android 3.1; Samsung GT-P7500 / Samsung GT-P7500; )[preload=false;locale=en_US;clientidbase=android-samsung]*/
const Device UA2619526 = {S23466, 132, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2619563 = {S23467, 71, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 3.1; en-CA; GT-P7500R Build/HMJ37.UXKH4)*/
const Device UA2619787 = {S23468, 259, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2620086 = {S23469, 92, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - com.google.android.apps.plus/X (Linux; U; Android 3.1; en_CA; GT-P7500R; Build/HMJ37) (gzip)*/
const Device UA2620266 = {S23470, 78, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - VKAndroidApp/2.7.5-60 (Android 3.1; SDK 12; armeabi-v7a; samsung GT-P7500; uk)*/
const Device UA2620751 = {S23471, 148, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; hr-hr; SAMSUNG-GT-P7500/P7500BUKG2 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2621213 = {S23472, 92, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - com.google.android.apps.plus/X (Linux; U; Android 3.1; en_CA; GT-P7500M; Build/HMJ37) (gzip)*/
const Device UA2621904 = {S23473, 197, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; ms-my; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 ZyngaBundleIdentifier/com.zynga.zjayakashi ZyngaBundleVersion/1.8.0*/
const Device UA2622509 = {S23474, 160, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500R Build/HMJ37) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2622844 = {S23475, 268, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.12;FBDM/{density=1.0,width=1280,height=800};FBLC/en_US;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P7500;FBSV/3.1;]*/
const Device UA2623104 = {S23476, 273, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/2.0;FBDM/{density=1.0,width=1280,height=800};FBLC/en_US;FB_FW/2;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P7500;FBSV/3.1;]*/
const Device UA2623687 = {S23477, 81, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500M Build/HMJ37) Globe News/1.10*/
const Device UA2623854 = {S23478, 273, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500R Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/2.0;FBDM/{density=1.0,width=1280,height=800};FBLC/en_CA;FB_FW/2;FBCR/ROGERS;FBPN/com.facebook.katana;FBDV/GT-P7500R;FBSV/3.1;]*/
const Device UA2624394 = {S23479, 120, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Android 3.1; en-us; GT-P7500 Build/HMJ37) FreeWheelAdManager/5.2.2-r9326-X;com.cnn.mobile.android.phone/109*/
const Device UA2625213 = {S23480, 199, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Android_Gmarket Qoo10 MY_2.0.9_26(4a90ee6f1a06a162____X;GT-P7500;3.1)*/
const Device UA2625476 = {S23481, 140, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500R Build/HMJ37 like Xoom) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2626979 = {S23482, 93, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 3.1; SAMSUNG GT-P7500R Build/HONEYCOMB_MR1);*/
const Device UA2627521 = {S23483, 99, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500 Build/HMJ37) AppleWo) Version/4.0 Safari/534.13*/
const Device UA2627627 = {S23484, 76, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - MediaPlayer/2002/4.1.2 (Android 3.1; GT-P7500; GT-P7500; samsung; ; ; en_US)*/
const Device UA2627834 = {S23485, 193, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500R Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 androidAppVersion/2.5.17 AndroidWebkitWrapper BillingSupported*/
const Device UA2628919 = {S23486, 142, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500D Build/HMJ37) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2629058 = {S23487, 142, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500M Build/HMJ37) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2630505 = {S23488, 148, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; SAMSUNG-GT-P7500/P7500BUKF6 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2630781 = {S23489, 67, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - DinnerSpinnerFree/2.1.2 (Android 3.1; en-us; GT-P7500R/HMJ37.UXKH4)*/
const Device UA2633410 = {S23490, 67, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - BBC iPlayer (AndroidApp; 1.3; 13014) GT-P7500 (Android 3.1, SDK 12)*/
const Device UA2636627 = {S23491, 78, { &P17337,&P6546,&P4459,&P18092 }};	/*Useragent - VKAndroidApp/2.7.6-64 (Android 3.1; SDK 12; armeabi-v7a; samsung GT-P7500; ru)*/
const Device UA812305 = {S23492, 143, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_Desire_C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA901752 = {S23493, 156, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_Desire_C_N/1.45.161.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA904209 = {S23494, 143, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Desire C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA909699 = {S23495, 156, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_Desire_C_N/1.45.161.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA924454 = {S23496, 136, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; A320a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1953407 = {S23497, 65, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Desire C Build/IML74K)*/
const Device UA1965616 = {S23498, 154, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; HTC_Desire_C/1.45.162.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1978732 = {S23499, 154, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC_Desire_C/1.45.163.6 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1989286 = {S23500, 154, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC_Desire_C/1.45.163.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1991182 = {S23501, 154, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_Desire_C/1.45.162.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1993485 = {S23502, 154, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_Desire_C/1.45.169.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029843 = {S23503, 154, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; HTC_Desire_C/1.46.113.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030147 = {S23504, 154, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_Desire_C/1.46.112.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033032 = {S23505, 154, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_Desire_C/1.45.169.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2166875 = {S23506, 154, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_Desire_C/1.46.111.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2168698 = {S23507, 154, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; HTC_Desire_C/1.46.114.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2180275 = {S23508, 154, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_Desire_C/1.46.123.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2181253 = {S23509, 153, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC_Desire_C-orange-LS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2365762 = {S23510, 143, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Desire C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2368151 = {S23511, 154, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; HTC_Desire_C/1.45.162.7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2383153 = {S23512, 154, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; el-gr; HTC_Desire_C/1.46.124.7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2405867 = {S23513, 156, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_Desire_C_N/2.00.161.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2407300 = {S23514, 154, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; HTC_Desire_C/2.00.114.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2424723 = {S23515, 143, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_Desire_C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426324 = {S23516, 154, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_Desire_C/2.00.162.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2429705 = {S23517, 154, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_Desire_C/2.00.111.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2458088 = {S23518, 156, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC_Desire_C_N/2.00.163.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2468963 = {S23519, 156, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_Desire_C_N/2.00.169.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483759 = {S23520, 154, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_Desire_C/2.00.123.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2485256 = {S23521, 154, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; HTC_Desire_C/2.00.113.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2506051 = {S23522, 154, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; el-gr; HTC_Desire_C/2.00.124.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2527537 = {S23523, 65, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Desire C Build/IMM76D)*/
const Device UA2528593 = {S23524, 154, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-at; HTC_Desire_C/2.00.112.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2738204 = {S23525, 154, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; HTC_Desire_C/2.00.162.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2980110 = {S23526, 141, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; HTC Desire C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088801 = {S23527, 154, { &P18382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_Desire_C/2.00.111.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA890622 = {S23528, 86, { &P17968,&P17979,&P17371,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/5.1.25169/26.1670; U; es) Presto/2.8.119 Version/10.54*/
const Device UA2797855 = {S23529, 86, { &P17968,&P17979,&P17371,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/5.1.24639/30.3214; U; en) Presto/2.8.119 Version/11.10*/
const Device UA11695 = {S23530, 133, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MB860 Build/4.5.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40091 = {S23531, 144, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; MB860 Build/4.5.2A-51_OLL-17) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40116 = {S23532, 144, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; MB860 Build/4.5.2A-51_OLL-37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40128 = {S23533, 141, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; MB860 Build/4.5.2A-SKT-75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40151 = {S23534, 144, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; MB860 Build/4.5.2A-51_OLL-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40159 = {S23535, 137, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/4.5.1-174) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40164 = {S23536, 134, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/4.5.70) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40177 = {S23537, 143, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/CherryBlur v0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40485 = {S23538, 137, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/4.5.1-178) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40483 = {S23539, 134, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MB860 Build/4.5.47) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48038 = {S23540, 144, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB860 Build/4.5.2A-74_OLE-26) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48046 = {S23541, 144, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB860 Build/4.5.2A-74_OLE-19) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48050 = {S23542, 144, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB860 Build/4.5.2A-74_OLE-21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48061 = {S23543, 143, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB860 Build/4.5.2A-11_OLE-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48069 = {S23544, 144, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/4.5.2A-74_OLE-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49186 = {S23545, 150, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/Navalynt Test v1 Alpha) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49187 = {S23546, 144, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/CherryBlur v1.4a) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55410 = {S23547, 134, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MB860 Build/4.5.31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55709 = {S23548, 134, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/4.5.78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55717 = {S23549, 134, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/4.5.91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55719 = {S23550, 142, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/4.5.2A-5_OLH-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55722 = {S23551, 137, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/4.5.1_134) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55723 = {S23552, 134, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/4.5.88) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA65908 = {S23553, 136, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MB860 Build/4.5.1_74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70322 = {S23554, 144, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; MB860 Build/4.5.2A-74_OLE-20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70941 = {S23555, 135, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB860 Build/4.5.130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72017 = {S23556, 144, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB860 Build/4.5.2A-74_OLE-29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73665 = {S23557, 136, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/4.5.9153) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74486 = {S23558, 144, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/Home Base Orange) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA75158 = {S23559, 135, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB860 Build/4.5.131) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76885 = {S23560, 135, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB860 Build/4.5.132) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA224642 = {S23561, 134, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB860 Build/4.5.91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA351277 = {S23562, 153, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB860 Build/4.5.91; RedPill Hybrid V1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA352967 = {S23563, 146, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; MB860 Build/4.5.2A-51_OLL-17.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368815 = {S23564, 144, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; MB860 Build/4.5.2A-51_OLL-50) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370541 = {S23565, 144, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; MB860 Build/4.5.2A-51_OLL-23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622442 = {S23566, 142, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB860 Build/4.5.91) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.1 Mobile Safari/534.30*/
const Device UA625028 = {S23567, 135, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB860 Build/4.5.141) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627485 = {S23568, 144, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB860 Build/4.5.2A-74_OLE-31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA629248 = {S23569, 146, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB860 Build/4.5.2A-74_OLE-31.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA630094 = {S23570, 144, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; MB860 Build/4.5.2A-51_OLL-52) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA632186 = {S23571, 144, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MB860 Build/4.5.2A-51_OLL-48) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA661217 = {S23572, 59, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; MB860 Build/4.5.141)*/
const Device UA663111 = {S23573, 144, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; he-il; MB860 Build/4.5.2A-74_OLE-33) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA674981 = {S23574, 144, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MB860 Build/4.5.2A-51_OLL-49) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA678831 = {S23575, 142, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; MB860 Build/4.5.2A-SKT-167) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA679906 = {S23576, 135, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB860 Build/4.5.133) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA688169 = {S23577, 135, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB860 Build/4.5.140) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA708609 = {S23578, 146, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB860 Build/4.5.2A-74_OLE-31.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA722299 = {S23579, 147, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; MB860 Build/4.5.2A-74_OLE-31.20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA730049 = {S23580, 147, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; MB860 Build/4.5.2A-74_OLE-31.11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA742829 = {S23581, 144, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; MB860 Build/4.5.2A-51_OLL-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA751246 = {S23582, 68, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; MB860 Build/4.5.2A-74_OLE-31)*/
const Device UA794113 = {S23583, 144, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; MB860 Build/4.5.2A-74_OLE-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA893652 = {S23584, 146, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MB860 Build/4.5.2A-51_OLL-51.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1948521 = {S23585, 144, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB860 Build/4.5.2A-74_OLE-27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025336 = {S23586, 66, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; MB860 Build/4.5.2A-SKT-167)*/
const Device UA2026675 = {S23587, 146, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; MB860 Build/4.5.2A-51_OLL-35.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031912 = {S23588, 133, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-; MB860 Build/4.5.141) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034388 = {S23589, 59, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; MB860 Build/4.5.141)*/
const Device UA2206024 = {S23590, 143, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB860 Build/4.5.141; MROM-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2356044 = {S23591, 143, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB860 Build/4.5.141) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2511347 = {S23592, 146, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB860 Build/4.5.1A-129_OLY-145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2836076 = {S23593, 142, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-; MB860 Build/4.5.2A-74_OLE-31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2994684 = {S23594, 125, { &P17283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB860 Build/4.5.141) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Sa*/
const Device UA105255 = {S23595, 116, { &P15364,&P5690,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.89 Safari/532.5*/
const Device UA135692 = {S23596, 118, { &P15364,&P5690,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.1.249.1064 Safari/532.5*/
const Device UA461980 = {S23597, 118, { &P15364,&P5690,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.1.249.1064 Safari/532.5*/
const Device UA628176 = {S23598, 115, { &P15364,&P5690,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Chrome/4.0.223.2 Safari/532.2*/
const Device UA784888 = {S23599, 77, { &P15364,&P5690,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; de-DE) Chrome/4.0.223.3 Safari/532.2*/
const Device UA2740248 = {S23600, 110, { &P15364,&P5690,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1) AppleWebKit/532.6 (KHTML, like Gecko) Chrome/4.0.573.150 Safari/532.6*/
const Device UA2602785 = {S23601, 127, { &P15364,&P17285,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/26.0.1410.43) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/537.31*/
const Device UA2605385 = {S23602, 134, { &P15364,&P17285,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/26.0.1410.43) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/537.31*/
const Device UA2674037 = {S23603, 161, { &P15364,&P17285,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/26.0.1410.64; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2674114 = {S23604, 195, { &P15364,&P17285,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; chromeframe/26.0.1410.64; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2674546 = {S23605, 189, { &P15364,&P17285,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; chromeframe/26.0.1410.64; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2675371 = {S23606, 202, { &P15364,&P17285,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.4; chromeframe/26.0.1410.64; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2677095 = {S23607, 134, { &P15364,&P17285,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/26.0.1410.64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2678527 = {S23608, 89, { &P15364,&P17285,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/26.0.1410.64)*/
const Device UA2678528 = {S23609, 96, { &P15364,&P17285,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/26.0.1410.64)*/
const Device UA2678737 = {S23610, 229, { &P15364,&P17285,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/26.0.1410.64; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C)*/
const Device UA2679194 = {S23611, 182, { &P15364,&P17285,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/26.0.1410.64; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2679372 = {S23612, 127, { &P15364,&P17285,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/26.0.1410.64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2717243 = {S23613, 219, { &P15364,&P17285,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/26.0.1410.64; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2717246 = {S23614, 210, { &P15364,&P17285,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; chromeframe/26.0.1410.64; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2719122 = {S23615, 240, { &P15364,&P17285,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/26.0.1410.64; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2736824 = {S23616, 253, { &P15364,&P17285,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; chromeframe/26.0.1410.64; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA95127 = {S23617, 109, { &P15364,&P18110,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/532.9 (KHTML, like Gecko) Chrome/5.0.307.7 Safari/532.9*/
const Device UA124171 = {S23618, 108, { &P15364,&P18110,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/532.9 (KHTML, like Gecko) Chrome/5.0.307.11 Safari/532.9*/
const Device UA258117 = {S23619, 119, { &P15364,&P18110,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Jolicloud Linux i686; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.125 Safari/533.4*/
const Device UA712733 = {S23620, 107, { &P15364,&P18110,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i586; en-US) AppleWebKit/533.2 (KHTML, like Gecko) Chrome/5.0.342.1 Safari/533.2*/
const Device UA896817 = {S23621, 116, { &P15364,&P18110,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; x86_64 Linux; en_GB, en_US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/5.0.358.0 Safari/533.3*/
const Device UA2323179 = {S23622, 109, { &P15364,&P18110,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; x86_64 Linux; en_US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/5.0.358.0 Safari/533.3*/
const Device UA382023 = {S23623, 88, { &P18088,&P17017,&P17780,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/6.1.25378/26.1424; U; ru) Presto/2.8.119 Version/10.54*/
const Device UA2389887 = {S23624, 59, { &P18088,&P17017,&P17780,&P18092 }};	/*Useragent - Opera/9.50 (J2ME/MIDP; Opera Mini/6.1.27786/28.3165; U; en)*/
const Device UA2431125 = {S23625, 88, { &P18088,&P17017,&P17780,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/6.1.25378/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA72886 = {S23626, 152, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMT11i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73056 = {S23627, 153, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMT11i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78285 = {S23628, 155, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SonyEricssonMT11i-o Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA254012 = {S23629, 154, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SonyEricssonMT11iv Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA354611 = {S23630, 152, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-hk; SonyEricssonMT11i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369401 = {S23631, 153, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SonyEricssonMT11iv Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370954 = {S23632, 140, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MT11i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA371201 = {S23633, 139, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; MT11a Build/4.0.2.D.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA453126 = {S23634, 153, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-za; SonyEricssonMT11iv Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625821 = {S23635, 64, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; MT11i Build/4.0.2.A.0.62)*/
const Device UA625803 = {S23636, 154, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SonyEricssonMT11i-o Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA628101 = {S23637, 152, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633473 = {S23638, 152, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonMT11i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA637141 = {S23639, 154, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SonyEricssonMT11i-o Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA641442 = {S23640, 151, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; SonyEricssonMT11a Build/4.0.2.D.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA672467 = {S23641, 140, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; MT11i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA673708 = {S23642, 141, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; MT11i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA674056 = {S23643, 151, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; SonyEricssonMT11a Build/4.0.2.D.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA686840 = {S23644, 140, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA698336 = {S23645, 128, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; MT11i Build/4.0.2.A.0.62) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA706557 = {S23646, 141, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; MT11a Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA734234 = {S23647, 153, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; SonyEricssonMT11a Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA761450 = {S23648, 152, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonMT11a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA830951 = {S23649, 140, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; MT11i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1958953 = {S23650, 140, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MT11a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1979878 = {S23651, 152, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-ar; SonyEricssonMT11a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021058 = {S23652, 128, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MT11i Build/4.0.2.A.0.42) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022762 = {S23653, 140, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-hk; MT11i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025562 = {S23654, 139, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; MT11a Build/4.0.2.D.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027353 = {S23655, 140, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-ar; MT11a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028378 = {S23656, 133, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MT11i Build/4.0.2.A.0.62) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2543413 = {S23657, 64, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; MT11a Build/4.0.2.A.0.62)*/
const Device UA2888893 = {S23658, 150, { &P13018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ar-; SonyEricssonMT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79950 = {S23659, 82, { &P15364,&P17285,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/13*/
const Device UA221016 = {S23660, 99, { &P15364,&P17285,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1*/
const Device UA247033 = {S23661, 106, { &P15364,&P17285,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1*/
const Device UA727292 = {S23662, 118, { &P15364,&P17285,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1*/
const Device UA2375845 = {S23663, 99, { &P15364,&P17285,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML	 like Gecko) Chrome/13.0.782.220 Safari/535.1*/
const Device UA2567658 = {S23664, 116, { &P15364,&P17285,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1,gzip(gfe)*/
const Device UA2672975 = {S23665, 118, { &P15364,&P17285,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-CA) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1*/
const Device UA2678793 = {S23666, 118, { &P15364,&P17285,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; es-US) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1*/
const Device UA2708164 = {S23667, 118, { &P15364,&P17285,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; fr-FR) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1*/
const Device UA2709427 = {S23668, 119, { &P15364,&P17285,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; haw-US) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1*/
const Device UA2710717 = {S23669, 118, { &P15364,&P17285,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; zh-CN) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1*/
const Device UA2711746 = {S23670, 118, { &P15364,&P17285,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; fr-CA) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1*/
const Device UA2750956 = {S23671, 118, { &P15364,&P17285,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; ku-DE) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1*/
const Device UA2787084 = {S23672, 104, { &P15364,&P17285,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML like Gecko) Chrome/13.0.782.41 Safari/535.1*/
const Device UA2860022 = {S23673, 118, { &P15364,&P17285,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; de-DE) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1*/
const Device UA2362787 = {S23674, 131, { &P15236,&P7106,&P15829,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9930; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.755 Mobile Safari/534.11+*/
const Device UA2573332 = {S23675, 77, { &P15236,&P7106,&P15829,&P18092 }};	/*Useragent - BlackBerry9930/7.1.0.163 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/105*/
const Device UA616634 = {S23676, 141, { &P17172,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Transformer TF101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA661161 = {S23677, 70, { &P17172,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Transformer TF101 Build/IML74K)*/
const Device UA743685 = {S23678, 148, { &P17172,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Transformer TF101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA806401 = {S23679, 158, { &P17172,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3 FROZN by jcarrz1; en-us; Transformer TF101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA874170 = {S23680, 141, { &P17172,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Transformer TF101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA901293 = {S23681, 129, { &P17172,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; TF101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032639 = {S23682, 141, { &P17172,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Transformer TF101 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033294 = {S23683, 139, { &P17172,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; Transformer TF101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2814131 = {S23684, 162, { &P17172,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Transformer_TF101 Build/IML74K; 1280*752) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2839185 = {S23685, 154, { &P17172,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Transformer TF101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA92806 = {S23686, 73, { &P18254,&P7480,&P18245,&P18093 }};	/*Useragent - facebookexternalhit/1.0 (+http://www.facebook.com/externalhit_uatext.php)*/
const Device UA94199 = {S23687, 54, { &P18254,&P7480,&P18245,&P18093 }};	/*Useragent - facebookplatform/1.0 (+http://developers.facebook.com)*/
const Device UA106430 = {S23688, 73, { &P18254,&P7480,&P18245,&P18093 }};	/*Useragent - facebookexternalhit/1.1 (+http://www.facebook.com/externalhit_uatext.php)*/
const Device UA263547 = {S23689, 144, { &P18254,&P7480,&P18245,&P18093 }};	/*Useragent - facebookexternalhit/1.0 (+http://www.facebook.com/externalhit_uatext.php) AppEngine-Google; (+http://code.google.com/appengine; appid: rtweetme)*/
const Device UA688372 = {S23690, 90, { &P18254,&P7480,&P18245,&P18093 }};	/*Useragent - facebookexternalhit/1.1 (+http://www.facebook.com/externalhit_uatext.php) Yandex.Translate*/
const Device UA2163398 = {S23691, 23, { &P18254,&P7480,&P18245,&P18093 }};	/*Useragent - facebookexternalhit/1.1*/
const Device UA2281212 = {S23692, 112, { &P18254,&P7480,&P18245,&P18093 }};	/*Useragent - facebookexternalhit/1.1 (+http://www.facebook.com/externalhit_uatext.php),gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2516853 = {S23693, 19, { &P18254,&P7480,&P18245,&P18093 }};	/*Useragent - facebookexternalhit*/
const Device UA2387800 = {S23694, 57, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.12*/
const Device UA2424674 = {S23695, 64, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; WOW64) Presto/2.12.388 Version/12.12*/
const Device UA2425840 = {S23696, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.12*/
const Device UA2426925 = {S23697, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.12*/
const Device UA2426940 = {S23698, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.12*/
const Device UA2426941 = {S23699, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.12*/
const Device UA2426938 = {S23700, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.12*/
const Device UA2427077 = {S23701, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.12*/
const Device UA2427082 = {S23702, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.12*/
const Device UA2427083 = {S23703, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.12*/
const Device UA2427088 = {S23704, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.12*/
const Device UA2427094 = {S23705, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.12*/
const Device UA2427104 = {S23706, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.12*/
const Device UA2427084 = {S23707, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.12*/
const Device UA2427470 = {S23708, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.12*/
const Device UA2427609 = {S23709, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.12*/
const Device UA2428164 = {S23710, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.12*/
const Device UA2428178 = {S23711, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.12*/
const Device UA2428193 = {S23712, 68, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:2.0) Gecko/X Firefox/4.0 Opera 12.12*/
const Device UA2428341 = {S23713, 62, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1) Opera 12.12*/
const Device UA2428517 = {S23714, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.12*/
const Device UA2430713 = {S23715, 69, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; Edition Yx) Presto/2.12.388 Version/12.12*/
const Device UA2431062 = {S23716, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.12*/
const Device UA2431101 = {S23717, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.12*/
const Device UA2432811 = {S23718, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.12*/
const Device UA2433717 = {S23719, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.12*/
const Device UA2433784 = {S23720, 57, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2) Presto/2.12.388 Version/12.12*/
const Device UA2436443 = {S23721, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.12*/
const Device UA2436470 = {S23722, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.12*/
const Device UA2437340 = {S23723, 67, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.12,gzip(gfe)*/
const Device UA2437348 = {S23724, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.12*/
const Device UA2437403 = {S23725, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.12*/
const Device UA2437420 = {S23726, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.12*/
const Device UA2437398 = {S23727, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.12*/
const Device UA2437436 = {S23728, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.12*/
const Device UA2437438 = {S23729, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.12*/
const Device UA2437442 = {S23730, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.12*/
const Device UA2437489 = {S23731, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.12*/
const Device UA2437492 = {S23732, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.12*/
const Device UA2437506 = {S23733, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.12*/
const Device UA2437507 = {S23734, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.12*/
const Device UA2437514 = {S23735, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.12*/
const Device UA2437519 = {S23736, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.12*/
const Device UA2437528 = {S23737, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.12*/
const Device UA2437558 = {S23738, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.12*/
const Device UA2437563 = {S23739, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.12*/
const Device UA2437603 = {S23740, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.12*/
const Device UA2437654 = {S23741, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.12*/
const Device UA2437664 = {S23742, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.12*/
const Device UA2437669 = {S23743, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.12*/
const Device UA2437670 = {S23744, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.12*/
const Device UA2437674 = {S23745, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.12*/
const Device UA2437684 = {S23746, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.12*/
const Device UA2437689 = {S23747, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.12*/
const Device UA2437666 = {S23748, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.12*/
const Device UA2459847 = {S23749, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.12*/
const Device UA2460590 = {S23750, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.12*/
const Device UA2460593 = {S23751, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.12*/
const Device UA2460601 = {S23752, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.12*/
const Device UA2460720 = {S23753, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.12*/
const Device UA2460857 = {S23754, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.12*/
const Device UA2460892 = {S23755, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4933)) Presto/2.12.388 Version/12.12*/
const Device UA2460899 = {S23756, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.12*/
const Device UA2460966 = {S23757, 106, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.12*/
const Device UA2461025 = {S23758, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03658)) Presto/2.12.388 Version/12.12*/
const Device UA2461045 = {S23759, 74, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.12 Yandex.Translate*/
const Device UA2467356 = {S23760, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5278)) Presto/2.12.388 Version/12.12*/
const Device UA2467510 = {S23761, 88, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; Edition Campaign 21) Presto/2.12.388 Version/12.12,gzip(gfe)*/
const Device UA2468366 = {S23762, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03402)) Presto/2.12.388 Version/12.12*/
const Device UA2468515 = {S23763, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.12*/
const Device UA2468995 = {S23764, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5650)) Presto/2.12.388 Version/12.12*/
const Device UA2469175 = {S23765, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.12*/
const Device UA2469179 = {S23766, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5660)) Presto/2.12.388 Version/12.12*/
const Device UA2469181 = {S23767, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03789)) Presto/2.12.388 Version/12.12*/
const Device UA2469932 = {S23768, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03639)) Presto/2.12.388 Version/12.12*/
const Device UA2483698 = {S23769, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.12*/
const Device UA2484139 = {S23770, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4110)) Presto/2.12.388 Version/12.12*/
const Device UA2484213 = {S23771, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4661)) Presto/2.12.388 Version/12.12*/
const Device UA2484223 = {S23772, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5336)) Presto/2.12.388 Version/12.12*/
const Device UA2485099 = {S23773, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5230)) Presto/2.12.388 Version/12.12*/
const Device UA2485550 = {S23774, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4832)) Presto/2.12.388 Version/12.12*/
const Device UA2486168 = {S23775, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5734)) Presto/2.12.388 Version/12.12*/
const Device UA2486814 = {S23776, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.3 (build 02564)) Presto/2.12.388 Version/12.12*/
const Device UA2486875 = {S23777, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5288)) Presto/2.12.388 Version/12.12*/
const Device UA2487214 = {S23778, 107, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03686); MRSPUTNIK OW 2, 3, 0, 244) Presto/2.12.388 Version/12.12*/
const Device UA2490628 = {S23779, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.12*/
const Device UA2506177 = {S23780, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03755)) Presto/2.12.388 Version/12.12*/
const Device UA2506426 = {S23781, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5325)) Presto/2.12.388 Version/12.12*/
const Device UA2506677 = {S23782, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5299)) Presto/2.12.388 Version/12.12*/
const Device UA2506694 = {S23783, 96, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.12,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2507461 = {S23784, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5217)) Presto/2.12.388 Version/12.12*/
const Device UA2511652 = {S23785, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.12*/
const Device UA2513000 = {S23786, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.12*/
const Device UA2529387 = {S23787, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.3 (build 02552)) Presto/2.12.388 Version/12.12*/
const Device UA2529424 = {S23788, 107, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03402); MRSPUTNIK OW 2, 3, 0, 104) Presto/2.12.388 Version/12.12*/
const Device UA2534069 = {S23789, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.12*/
const Device UA2534834 = {S23790, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.12*/
const Device UA2537481 = {S23791, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6044)) Presto/2.12.388 Version/12.12*/
const Device UA2538006 = {S23792, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.12*/
const Device UA2538528 = {S23793, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.12*/
const Device UA2539513 = {S23794, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.12*/
const Device UA2557586 = {S23795, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.4 (build 02652)) Presto/2.12.388 Version/12.12*/
const Device UA2558517 = {S23796, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6035)) Presto/2.12.388 Version/12.12*/
const Device UA2565456 = {S23797, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6046)) Presto/2.12.388 Version/12.12*/
const Device UA2565762 = {S23798, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.12*/
const Device UA2569360 = {S23799, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.12*/
const Device UA2579147 = {S23800, 71, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.12 WebWasher 3.4*/
const Device UA2583071 = {S23801, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.12*/
const Device UA2595913 = {S23802, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5161)) Presto/2.12.388 Version/12.12*/
const Device UA2596830 = {S23803, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5300)) Presto/2.12.388 Version/12.12*/
const Device UA2603949 = {S23804, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.12*/
const Device UA2669107 = {S23805, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.12*/
const Device UA2699768 = {S23806, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5988)) Presto/2.12.388 Version/12.12*/
const Device UA2707628 = {S23807, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.12*/
const Device UA2709253 = {S23808, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.12*/
const Device UA2728813 = {S23809, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03672)) Presto/2.12.388 Version/12.12*/
const Device UA2733749 = {S23810, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6080)) Presto/2.12.388 Version/12.12*/
const Device UA2747783 = {S23811, 107, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03396); MRSPUTNIK OW 2, 3, 0, 104) Presto/2.12.388 Version/12.12*/
const Device UA2754603 = {S23812, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.12*/
const Device UA2764235 = {S23813, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6032)) Presto/2.12.388 Version/12.12*/
const Device UA2791482 = {S23814, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03791)) Presto/2.12.388 Version/12.12*/
const Device UA2846524 = {S23815, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03780)) Presto/2.12.388 Version/12.12*/
const Device UA2846560 = {S23816, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02746)) Presto/2.12.388 Version/12.12*/
const Device UA2847119 = {S23817, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.3 (build 02560)) Presto/2.12.388 Version/12.12*/
const Device UA2937007 = {S23818, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.12*/
const Device UA2955873 = {S23819, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.12*/
const Device UA2963700 = {S23820, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.12*/
const Device UA2984464 = {S23821, 80, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03638)) Presto/2.12.388 Version/12.12*/
const Device UA3020241 = {S23822, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6673)) Presto/2.12.388 Version/12.12*/
const Device UA3058044 = {S23823, 77, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.12,gzip(gfe),gzip(gfe)*/
const Device UA3079132 = {S23824, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.12*/
const Device UA3102954 = {S23825, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6573)) Presto/2.12.388 Version/12.12*/
const Device UA3124875 = {S23826, 79, { &P15364,&P5690,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.1 (build 6341)) Presto/2.12.388 Version/12.12*/
const Device UA32951 = {S23827, 138, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_ChaCha Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46794 = {S23828, 149, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; el-gr; HTC/ChaCha/1.19.168.3 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46811 = {S23829, 149, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; HTC/ChaCha/1.19.163.4 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49474 = {S23830, 147, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; HTC_ChaChaCha_A810e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59899 = {S23831, 144, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_ChaCha_A810e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59903 = {S23832, 144, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC ChaCha A810e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59906 = {S23833, 148, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_ChaCha-orange-LS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59907 = {S23834, 144, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC_ChaCha_A810b Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59914 = {S23835, 149, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-at; HTC/ChaCha/1.19.169.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59911 = {S23836, 149, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC/ChaCha/1.19.161.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70384 = {S23837, 142, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-; HTC_ChaCha_A810e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73082 = {S23838, 144, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_ChaCha_A810e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79425 = {S23839, 149, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-nz; HTC/ChaCha/1.27.179.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90728 = {S23840, 143, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - HTC_ChaCha_A810e/Mozilla/5.0(Linux; U; Android 2.3.3;it-it; Build/GRH78C) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA195336 = {S23841, 144, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC ChaCha A810b Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368880 = {S23842, 147, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; HTC ChaChaCha A810e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369495 = {S23843, 151, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; HTC_ChaChaCha-orange-LS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA378822 = {S23844, 68, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC ChaCha A810e Build/GRI40)*/
const Device UA452260 = {S23845, 68, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC ChaCha A810e Build/GRJ90)*/
const Device UA584235 = {S23846, 144, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC ChaCha A810b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA584929 = {S23847, 144, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC_ChaCha_A810b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA624936 = {S23848, 147, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; HTC_ChaChaCha_A810e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625591 = {S23849, 144, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC ChaCha A810e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627664 = {S23850, 149, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/ChaCha/1.57.161.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA657084 = {S23851, 138, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_ChaCha Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA662291 = {S23852, 149, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC/ChaCha/1.57.163.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA691718 = {S23853, 148, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_ChaCha-orange-LS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA699442 = {S23854, 149, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-at; HTC/ChaCha/1.57.169.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA703637 = {S23855, 147, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; HTC ChaChaCha A810e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA736514 = {S23856, 149, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-nz; HTC/ChaCha/1.57.179.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA780344 = {S23857, 143, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - HTC_ChaCha_A810e/Mozilla/5.0(Linux; U; Android 2.3.5;it-it; Build/GRH78C) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1917784 = {S23858, 137, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC A810e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1924394 = {S23859, 132, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC ChaCha A810e Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1934557 = {S23860, 149, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/ChaCha/1.19.161.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1940735 = {S23861, 125, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC A810e Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1950123 = {S23862, 149, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/ChaCha/1.57.161.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1951207 = {S23863, 136, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-cn; HTCA810e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1962931 = {S23864, 149, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC/ChaCha/1.57.163.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023966 = {S23865, 132, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC ChaCha A810e Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2026129 = {S23866, 149, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC/ChaCha/1.53.192.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027321 = {S23867, 130, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC A810e Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2030133 = {S23868, 137, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC ChaCha A810e Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2033752 = {S23869, 149, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-pt; HTC/ChaCha/1.57.168.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2445543 = {S23870, 161, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC ChaCha A810e Build/GRJ90; SuperOSR-chacha) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2496372 = {S23871, 137, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC ChaCha A810e Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2697219 = {S23872, 149, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; nl-nl; HTC/ChaCha/1.57.161.4 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2722927 = {S23873, 149, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-at; HTC/ChaCha/1.57.169.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2840988 = {S23874, 149, { &P15483,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-nz; HTC/ChaCha/1.57.179.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA782823 = {S23875, 147, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC EVO 3D X515m Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA856312 = {S23876, 151, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_EVO_3D_GSM_X515m Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA858913 = {S23877, 69, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC EVO 3D X515m Build/IML74K)*/
const Device UA873557 = {S23878, 140, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_X515C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899375 = {S23879, 142, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; EVO3D_X515m Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA911683 = {S23880, 146, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; HTC_EVO3D_X515a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA911760 = {S23881, 147, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; HTC EVO 3D X515a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA912462 = {S23882, 146, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-sg; HTC_EVO3D_X515m Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1992209 = {S23883, 138, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC EVO 3D X515m Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1992400 = {S23884, 140, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-kr; HTC_X515E Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028465 = {S23885, 131, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC X515d Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2029332 = {S23886, 138, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; HTC EVO 3D X515m Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2031545 = {S23887, 62, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC_X515E Build/IML74K)*/
const Device UA2033677 = {S23888, 69, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC EVO 3D X515a Build/IML74K)*/
const Device UA2222782 = {S23889, 142, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Kingdom Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2235168 = {S23890, 152, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC_X515a V3.29.841.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2445086 = {S23891, 141, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-X515d/2.21.1401.3) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2586604 = {S23892, 273, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC EVO 3D X515m Build/IML74K) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 baidubrowser/042_1.41.3.2_diordna_069_045/CTH_51_3.0.4_m515X-D3-OVE-CTH/Xa/0A4794C9CE42717CF87A1F2472B68FFA%7CX/1*/
const Device UA2642311 = {S23893, 129, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC X515d) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778199 = {S23894, 140, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; HTC X515e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2851377 = {S23895, 142, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; EVO3D_X515m Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934429 = {S23896, 152, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC_X515a V1.24.841.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2980362 = {S23897, 140, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-; EVO3D_X515m Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3069945 = {S23898, 140, { &P12657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC X515m Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA712055 = {S23899, 69, { &P18435,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; ARM; Trident/6.0)*/
const Device UA857353 = {S23900, 76, { &P18435,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; ARM; Trident/6.0; Touch)*/
const Device UA2190085 = {S23901, 84, { &P18435,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; ARM; Trident/6.0; Touch; ARMBJS)*/
const Device UA2859404 = {S23902, 77, { &P18435,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; ARM; Trident/6.0; ARMBJS)*/
const Device UA3099627 = {S23903, 124, { &P18435,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; ARM; Trident/6.0; Touch; .NET4.0E; .NET4.0C; Tablet PC 2.0; WebView/1.0)*/
const Device UA2228224 = {S23904, 132, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2391339 = {S23905, 152, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; SAMSUNG GT-P3100/P3100BUCLK1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2419879 = {S23906, 130, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2421871 = {S23907, 61, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P3100 Build/JRO03C)*/
const Device UA2483329 = {S23908, 132, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2508087 = {S23909, 133, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; GT-P3100B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2527428 = {S23910, 61, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P3100 Build/JZO54K)*/
const Device UA2578085 = {S23911, 139, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2615385 = {S23912, 139, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3100 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2615572 = {S23913, 139, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3100 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2616077 = {S23914, 93, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.1.2 (Linux; U; Android 4.1.1; en-GB; GT-P3100 Build/JRO03C.P3100DDBLI1)*/
const Device UA2616762 = {S23915, 145, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2621802 = {S23916, 130, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3100 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2622149 = {S23917, 136, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/TELL// X/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2623109 = {S23918, 81, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.1.1; en-US; GT-P3100 Build/JRO03C.P3100DDCLL1)*/
const Device UA2623971 = {S23919, 155, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; SAMSUNG GT-P3100/P3100BUCLJ1.DM Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2625357 = {S23920, 94, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.2.10 (Linux; U; Android 4.1.1; en-US; GT-P3100 Build/JRO03C.P3100BUCLK1)*/
const Device UA2626195 = {S23921, 134, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3100 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2626833 = {S23922, 139, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2626857 = {S23923, 244, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 VoiceGO!(cyb471cfd6517cc855a94a1c1c91fc15bc7-0;Android,4.1.1,GT-P3100;3.8.0;600x895,1.0;X;VoiceGo-TC;zh-TW;X;X)*/
const Device UA2627084 = {S23924, 139, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2628386 = {S23925, 150, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; SAMSUNG GT-P3100/P3100BUCLK1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2628474 = {S23926, 136, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3100 Build/TELL// X/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2628520 = {S23927, 102, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) Applcko) Version/4.0 Safari/534.30*/
const Device UA2629093 = {S23928, 62, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P3100B Build/JRO03C)*/
const Device UA2630106 = {S23929, 130, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3100 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2631067 = {S23930, 151, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.1 Version/4.0 Mobile Safari/530.17*/
const Device UA2631148 = {S23931, 145, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30*/
const Device UA2631544 = {S23932, 130, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2631699 = {S23933, 218, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.4.0-99 (Android 4.1.1; samsung GT-P3100 Build/JRO03C.P3100DDCLL1)*/
const Device UA2631902 = {S23934, 149, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C.P3100DDCLL1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2632302 = {S23935, 131, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-; GT-P3100B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2633239 = {S23936, 137, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2633719 = {S23937, 137, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; GT-P3100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2633795 = {S23938, 131, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 4.1.2; fr-fr; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2633889 = {S23939, 144, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-gb; GT-P3100 Build/JellyBean) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2634974 = {S23940, 130, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P3100 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2634994 = {S23941, 130, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3100 Build JRO03C) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2635847 = {S23942, 140, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3100B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2636706 = {S23943, 151, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3100 Build/JZO54K) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.2 Version/4.0 Mobile Safari/530.17*/
const Device UA2637154 = {S23944, 93, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - com.google.android.youtube/4.2.16(Linux; U; Android 4.1.2; en_GB; GT-P3100 Build/JZO54K) gzip*/
const Device UA2637446 = {S23945, 152, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; SAMSUNG GT-P3100/P3100BUDMB2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2637740 = {S23946, 175, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 GoogleGoggles-Android/1.9; gzip*/
const Device UA2638018 = {S23947, 143, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; GT-P3100 Build/JRO03C) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2879099 = {S23948, 151, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-P3100-ORANGE/P3100BVDMD1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3020700 = {S23949, 151, { &P15919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-P3100-ORANGE/P3100BVDMF1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA94041 = {S23950, 113, { &P17595,&P18120,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/523.12 (KHTML, like Gecko) Version/3.0.4 Safari/523.12*/
const Device UA98159 = {S23951, 118, { &P17595,&P18120,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-us) AppleWebKit/523.15.1 (KHTML, like Gecko) Version/3.0.4 Safari/523.15*/
const Device UA105267 = {S23952, 118, { &P17595,&P18120,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-us) AppleWebKit/523.10.3 (KHTML, like Gecko) Version/3.0.4 Safari/523.10*/
const Device UA154073 = {S23953, 115, { &P17595,&P18120,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/522.11.1 (KHTML, like Gecko) Version/3.0.3 Safari/522.12.1*/
const Device UA176153 = {S23954, 113, { &P17595,&P18120,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/522.10.1 (KHTML, like Gecko) Version/3.0 Safari/522.11*/
const Device UA279063 = {S23955, 120, { &P17595,&P18120,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-us) AppleWebKit/523.10.5 (KHTML, like Gecko) Version/3.0.4 Safari/523.13.2*/
const Device UA408574 = {S23956, 112, { &P17595,&P18120,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/523.6 (KHTML, like Gecko) Version/3.0.3 Safari/523.6*/
const Device UA1864606 = {S23957, 112, { &P17595,&P18120,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/523.1 (KHTML, like Gecko) Version/3.0.3 Safari/523.1*/
const Device UA2271683 = {S23958, 136, { &P17595,&P18120,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en_US) AppleWebKit/522.7 (KHTML, like Gecko) Dreamweaver/10.0.0.4117 Version/3.0 Safari/522.7*/
const Device UA2335529 = {S23959, 137, { &P17595,&P18120,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-us) AppleWebKit/523.15.1 (KHTML, like Gecko) Version/3.0.4 Safari/99999 MultiSafari/5523.15*/
const Device UA118071 = {S23960, 117, { &P15364,&P5690,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.125 Safari/533.4*/
const Device UA266563 = {S23961, 116, { &P15364,&P5690,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.99 Safari/533.4*/
const Device UA715726 = {S23962, 154, { &P15364,&P5690,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.125 Safari/533.4,gzip(gfe) (via translate.google.com)*/
const Device UA829313 = {S23963, 115, { &P15364,&P5690,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/531.0 (KHTML, like Gecko) Chrome/5.0.195.0 Safari/531.0*/
const Device UA2455889 = {S23964, 126, { &P15364,&P5690,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.86 Safari/533.4,gzip(gfe)*/
const Device UA2785747 = {S23965, 191, { &P15364,&P5690,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.70 Safari/533.4 AppEngine-Google; (+http://code.google.com/appengine; appid: maipurokushi)*/
const Device UA924887 = {S23966, 61, { &P15364,&P5690,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:17.0) Gecko/17.0 Firefox/17.0*/
const Device UA1995103 = {S23967, 73, { &P15364,&P5690,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:17.0) Gecko/17.0 Firefox/17.0*/
const Device UA1995360 = {S23968, 61, { &P15364,&P5690,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:17.0) Gecko/17.0 Firefox/17.0*/
const Device UA2391314 = {S23969, 70, { &P15364,&P5690,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0(Windows; U; Windows NT 5.2; rv:1.9.2) Gecko/X Firefox/17.0*/
const Device UA2391317 = {S23970, 105, { &P15364,&P5690,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.1.7) Gecko/X Firefox/17.0 (.NET CLR 3.0.04506.648)*/
const Device UA2538617 = {S23971, 80, { &P15364,&P5690,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:17.0) Gecko/X Firefox/17.0ms1*/
const Device UA64788 = {S23972, 77, { &P15216,&P6217,&P3418,&P18092 }};	/*Useragent - BlackBerry8520/4.6.1.314 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/600*/
const Device UA2717087 = {S23973, 133, { &P15767,&P19826,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B146 Safari/8536.25*/
const Device UA2717088 = {S23974, 131, { &P15767,&P19826,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B141 Safari/8536.25*/
const Device UA2717567 = {S23975, 133, { &P15767,&P19826,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B147 Safari/8536.25*/
const Device UA2719068 = {S23976, 133, { &P15767,&P19826,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B329 Safari/8536.25*/
const Device UA2877410 = {S23977, 141, { &P15364,&P7133,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100 (Edition Next)*/
const Device UA2884699 = {S23978, 134, { &P15364,&P7133,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100 (Edition Next)*/
const Device UA2946165 = {S23979, 137, { &P15364,&P7133,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.130 (Edition Rambler)*/
const Device UA3011523 = {S23980, 141, { &P15364,&P7133,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.130 (Edition Campaign 06)*/
const Device UA3015626 = {S23981, 148, { &P15364,&P7133,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.130 (Edition Campaign 06)*/
const Device UA3018599 = {S23982, 136, { &P15364,&P7133,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.130 (Edition Seznam)*/
const Device UA3025465 = {S23983, 139, { &P15364,&P7133,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Yx)*/
const Device UA3027316 = {S23984, 126, { &P15364,&P7133,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153*/
const Device UA3027550 = {S23985, 119, { &P15364,&P7133,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153*/
const Device UA3028018 = {S23986, 136, { &P15364,&P7133,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153,gzip(gfe)*/
const Device UA3028071 = {S23987, 143, { &P15364,&P7133,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Seznam)*/
const Device UA3028486 = {S23988, 148, { &P15364,&P7133,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Campaign 21)*/
const Device UA3029223 = {S23989, 144, { &P15364,&P7133,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Rambler)*/
const Device UA3029640 = {S23990, 129, { &P15364,&P7133,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153,gzip(gfe)*/
const Device UA3029914 = {S23991, 132, { &P15364,&P7133,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Yx)*/
const Device UA3030061 = {S23992, 141, { &P15364,&P7133,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Campaign 21)*/
const Device UA3034257 = {S23993, 146, { &P15364,&P7133,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153,gzip(gfe),gzip(gfe)*/
const Device UA149271 = {S23994, 116, { &P15364,&P7133,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA193635 = {S23995, 105, { &P15364,&P7133,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.28 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.28*/
const Device UA196247 = {S23996, 98, { &P15364,&P7133,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.28 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.28*/
const Device UA291932 = {S23997, 130, { &P15364,&P7133,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.50 Safari/534.7 SunChrome/7.0*/
const Device UA546467 = {S23998, 150, { &P15364,&P7133,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.16; QQBrowser/5.1.6829.201 (webkit)*/
const Device UA627461 = {S23999, 117, { &P15364,&P7133,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; de-DE) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/7.0.540.0 Safari/534.10*/
const Device UA666630 = {S24000, 150, { &P15364,&P7133,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.16; QQBrowser/5.1.7061.201 (webkit)*/
const Device UA709645 = {S24001, 126, { &P15364,&P7133,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0 baidubrowser/1.x Safari/534.7*/
const Device UA849046 = {S24002, 154, { &P15364,&P7133,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/7.0.544.0 Safari/534.10,gzip(gfe) (via translate.google.com)*/
const Device UA2289711 = {S24003, 150, { &P15364,&P7133,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.16; QQBrowser/5.1.7257.201 (webkit)*/
const Device UA2421187 = {S24004, 126, { &P15364,&P7133,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.41 Safari/534.7,gzip(gfe)*/
const Device UA3064639 = {S24005, 127, { &P18385,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus 7 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2205354 = {S24006, 143, { &P15767,&P18117,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9B206 Safari/7534.48.3*/
const Device UA2205487 = {S24007, 141, { &P15767,&P18117,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9B176 Safari/7534.48.3*/
const Device UA2260495 = {S24008, 141, { &P15767,&P18117,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9A405 Safari/7534.48.3*/
const Device UA2502116 = {S24009, 143, { &P15767,&P18117,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9Z999 Safari/7534.48.3*/
const Device UA97892 = {S24010, 64, { &P15364,&P17350,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; DigExt; DTS Agent*/
const Device UA134093 = {S24011, 47, { &P15364,&P17350,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01; Windows NT)*/
const Device UA520385 = {S24012, 47, { &P15364,&P17350,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.02; Windows NT)*/
const Device UA630752 = {S24013, 41, { &P15364,&P17350,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Win32)*/
const Device UA2302286 = {S24014, 59, { &P15364,&P17350,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible;MSIE 5.0;  Windows NT 7.0; LOVEYOU)*/
const Device UA2306690 = {S24015, 60, { &P15364,&P17350,&P4551,&P18092 }};	/*Useragent - Mozilla/6.0 (compatible;MSIE 5.0;  Windows NT 7.0; QQLOADOW)*/
const Device UA2510263 = {S24016, 52, { &P15364,&P17350,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; 68K; Trident/1.0)*/
const Device UA2581090 = {S24017, 59, { &P15364,&P17350,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; 68K; WOW64; Trident/4.0)*/
const Device UA2658810 = {S24018, 64, { &P15364,&P17350,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; PPC; Win64; x64; Trident/5.0)*/
const Device UA2658869 = {S24019, 57, { &P15364,&P17350,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; 68K; SV1; Trident/4.0)*/
const Device UA2659583 = {S24020, 62, { &P15364,&P17350,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; PPC; .NET CLR; Trident/5.0)*/
const Device UA2659774 = {S24021, 59, { &P15364,&P17350,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; PPC; WOW64; Trident/2.0)*/
const Device UA2792870 = {S24022, 64, { &P15364,&P17350,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; 68K; Win64; x64; Trident/1.0)*/
const Device UA45825 = {S24023, 147, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Sensation Z710e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA45982 = {S24024, 147, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Sensation Z710e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA45983 = {S24025, 147, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC_Sensation_Z710e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46057 = {S24026, 227, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTCZ710e/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46168 = {S24027, 147, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; HTC Sensation Z710e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46173 = {S24028, 147, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Sensation_Z710e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48507 = {S24029, 143, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; HTC Sensation 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48513 = {S24030, 143, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;HTC_Sensation_4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48515 = {S24031, 143, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_Sensation_4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA48517 = {S24032, 143, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; HTC_Sensation_4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48519 = {S24033, 143, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; HTC_Sensation_4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48523 = {S24034, 143, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_Sensation_4GBuild/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48526 = {S24035, 144, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Sensation 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48525 = {S24036, 143, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_Sensation_4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA48528 = {S24037, 144, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_Sensation_4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59692 = {S24038, 147, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Sensation Z710e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59723 = {S24039, 147, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_Sensation_Z710e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61529 = {S24040, 143, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Sensation4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61531 = {S24041, 144, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Sensation 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61532 = {S24042, 144, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_Sensation_4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61533 = {S24043, 143, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;HTC Sensation 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61537 = {S24044, 143, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Sensation 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA61538 = {S24045, 143, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Sensation 4GBuild/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61540 = {S24046, 143, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.3; en-us; HTC_Sensation_4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61549 = {S24047, 143, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.3; en-us; HTC Sensation 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61550 = {S24048, 143, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;HTC_Sensation_4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA69893 = {S24049, 143, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Sensation 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA69904 = {S24050, 143, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Sensation 4GBuild/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA69911 = {S24051, 143, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;HTC Sensation 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70360 = {S24052, 143, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_Sensation_4GBuild/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72084 = {S24053, 147, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Sensation_Z710e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72799 = {S24054, 147, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Sensation Z710e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73138 = {S24055, 144, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Htc Sensation 4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73906 = {S24056, 144, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Sensation_4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA75502 = {S24057, 142, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; vi-; HTC_Sensation_4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA191301 = {S24058, 134, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Sensation_4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA196863 = {S24059, 71, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Sensation Z710e Build/GRI40)*/
const Device UA197243 = {S24060, 133, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; Sensation_4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA204853 = {S24061, 133, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.3; en-us; Sensation_4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA210007 = {S24062, 133, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Sensation_4GBuild/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA217997 = {S24063, 133, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;Sensation_4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA227938 = {S24064, 71, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Sensation Z710e Build/GRJ22)*/
const Device UA236691 = {S24065, 68, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Sensation 4G Build/GRJ22)*/
const Device UA325896 = {S24066, 68, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Sensation 4G Build/GRI40)*/
const Device UA354047 = {S24067, 74, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.3 (Linux; U; Android 2.3.4; en_US; HTC Sensation 4G Build/GRJ22)*/
const Device UA379536 = {S24068, 143, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; HTC Sensation 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA421199 = {S24069, 143, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTCSensation 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA567157 = {S24070, 143, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Sensation4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA582795 = {S24071, 164, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Sensation 4G XE with Beats Audio Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA651744 = {S24072, 135, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Sensation Z710e Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA738209 = {S24073, 135, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Sensation Z710e Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA740284 = {S24074, 71, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Sensation Z710e Build/GRJ90)*/
const Device UA741035 = {S24075, 125, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Z710e Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1900766 = {S24076, 144, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Sensation 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1933986 = {S24077, 137, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Z710e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1948975 = {S24078, 135, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Sensation Z710e Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1962412 = {S24079, 140, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Sensation Z710e Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1993427 = {S24080, 140, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Sensation Z710e Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2018993 = {S24081, 143, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; HTC Sensation 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019413 = {S24082, 151, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Sensation 4G Build/GRJ22) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2020118 = {S24083, 145, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-; HTC Sensation Z710e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020626 = {S24084, 155, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Htcclay's SuperSensation 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021172 = {S24085, 147, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Sensation Z710e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021606 = {S24086, 132, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Sensation 4G Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2021611 = {S24087, 142, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; HTC Sensation 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022042 = {S24088, 164, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Sensation Z710e with Beats Audio Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032816 = {S24089, 140, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Sensation Z710e Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2543400 = {S24090, 130, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Z710e Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2815875 = {S24091, 157, { &P15201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Sensation Z710e with Beats Audio Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2675175 = {S24092, 127, { &P15364,&P17285,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/27.0.1453.47) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.47 Safari/537.36*/
const Device UA2740509 = {S24093, 127, { &P15364,&P17285,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/27.0.1453.81) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.81 Safari/537.36*/
const Device UA2741425 = {S24094, 127, { &P15364,&P17285,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/27.0.1453.93) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36*/
const Device UA2753532 = {S24095, 161, { &P15364,&P17285,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/27.0.1453.94; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2753670 = {S24096, 134, { &P15364,&P17285,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/27.0.1453.94) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36*/
const Device UA2755302 = {S24097, 134, { &P15364,&P17285,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/27.0.1453.93) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36*/
const Device UA2757172 = {S24098, 127, { &P15364,&P17285,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/27.0.1453.94) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36*/
const Device UA2774275 = {S24099, 129, { &P15364,&P17285,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/27.0.1453.110) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36*/
const Device UA2783724 = {S24100, 203, { &P15364,&P17285,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.4; chromeframe/27.0.1453.110; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2784394 = {S24101, 136, { &P15364,&P17285,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/27.0.1453.110) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36*/
const Device UA2784509 = {S24102, 183, { &P15364,&P17285,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/27.0.1453.110; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2784643 = {S24103, 196, { &P15364,&P17285,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; chromeframe/27.0.1453.110; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2784654 = {S24104, 190, { &P15364,&P17285,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; chromeframe/27.0.1453.110; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2869733 = {S24105, 241, { &P15364,&P17285,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/27.0.1453.110; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2873766 = {S24106, 90, { &P15364,&P17285,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/27.0.1453.116)*/
const Device UA2873987 = {S24107, 97, { &P15364,&P17285,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/27.0.1453.116)*/
const Device UA2874464 = {S24108, 129, { &P15364,&P17285,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/27.0.1453.116) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2874539 = {S24109, 136, { &P15364,&P17285,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/27.0.1453.116) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2880450 = {S24110, 177, { &P15364,&P17285,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/27.0.1453.116; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2942148 = {S24111, 211, { &P15364,&P17285,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; chromeframe/27.0.1453.116; GTB7.5; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA29917 = {S24112, 145, { &P12280,&P17084,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA29976 = {S24113, 145, { &P12280,&P17084,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293 Safari/6531.22.7*/
const Device UA90029 = {S24114, 147, { &P12280,&P17084,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B5097d Safari/6531.22.7*/
const Device UA170944 = {S24115, 145, { &P12280,&P17084,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/9Z999 Safari/6531.22.7*/
const Device UA1973741 = {S24116, 128, { &P12280,&P17084,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293*/
const Device UA2127761 = {S24117, 141, { &P12280,&P17084,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2128933 = {S24118, 145, { &P12280,&P17084,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; vi-vn) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/PV004 Safari/6531.22.7*/
const Device UA2268112 = {S24119, 145, { &P12280,&P17084,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; vi-vn) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/PV001 Safari/6531.22.7*/
const Device UA2282174 = {S24120, 147, { &P12280,&P17084,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/ 8B117  Safari/6531.22.7*/
const Device UA2659362 = {S24121, 143, { &P12280,&P17084,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 41 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA5644 = {S24122, 104, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA5708 = {S24123, 115, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7*/
const Device UA5721 = {S24124, 115, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2*/
const Device UA90489 = {S24125, 102, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en_us) AppleWebKit/525.18.1 (KHTML, like Gecko)*/
const Device UA90726 = {S24126, 104, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en_us) AppleWebKit/525.18.1 (KHTML, like Gecko)*/
const Device UA169279 = {S24127, 102, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA180441 = {S24128, 104, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA180794 = {S24129, 115, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190*/
const Device UA182051 = {S24130, 115, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4*/
const Device UA191904 = {S24131, 104, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA223927 = {S24132, 115, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2*/
const Device UA223926 = {S24133, 104, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA231624 = {S24134, 115, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1*/
const Device UA232029 = {S24135, 104, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA232610 = {S24136, 131, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/533.17.9*/
const Device UA455871 = {S24137, 250, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_3_2 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3.2;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA456291 = {S24138, 246, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_3 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA457075 = {S24139, 250, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_3_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA625089 = {S24140, 250, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_3_5 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3.5;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA626086 = {S24141, 250, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_3_3 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3.3;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA628349 = {S24142, 143, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 iPod4,1 BingWeb/3.03.1428.X*/
const Device UA636912 = {S24143, 131, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/533.17.9*/
const Device UA749873 = {S24144, 250, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_3_4 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3.4;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA750916 = {S24145, 143, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 iPod4,1 BingWeb/3.03.1428.X*/
const Device UA1952147 = {S24146, 143, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 iPod4,1 BingWeb/3.03.1428.X*/
const Device UA1984432 = {S24147, 143, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 iPod4,1 BingWeb/3.03.1428.X*/
const Device UA2202423 = {S24148, 239, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3.5;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]*/
const Device UA2203920 = {S24149, 237, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]*/
const Device UA2204245 = {S24150, 239, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3.2;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]*/
const Device UA2205609 = {S24151, 239, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]*/
const Device UA2229990 = {S24152, 239, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3.4;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]*/
const Device UA2336874 = {S24153, 143, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190 iPod4,1 BingWeb/3.03.1428.X*/
const Device UA2393500 = {S24154, 143, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2 iPod4,1 BingWeb/3.03.1428.X*/
const Device UA2432745 = {S24155, 239, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 [FBAN/FBIOS;FBAV/5.3;FBBV/89182;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.3.3;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]*/
const Device UA2708156 = {S24156, 113, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 43 like Mac OS X; en-US) AppleWebKit/533.17.9 (KHTML like Gecko) Mobile/8F190*/
const Device UA2709089 = {S24157, 112, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 435 like Mac OS X; en-US) AppleWebKit/533.17.9 (KHTML like Gecko) Mobile/8L1*/
const Device UA2711123 = {S24158, 112, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 433 like Mac OS X; es-US) AppleWebKit/533.17.9 (KHTML like Gecko) Mobile/8J2*/
const Device UA2726625 = {S24159, 142, { &P17346,&P17090,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone 4_3_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPod4,1/4.3.3)*/
const Device UA2973268 = {S24160, 137, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973464 = {S24161, 138, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973665 = {S24162, 138, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973899 = {S24163, 138, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973908 = {S24164, 145, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I317 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973939 = {S24165, 137, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973955 = {S24166, 138, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974022 = {S24167, 145, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974164 = {S24168, 137, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974306 = {S24169, 130, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974319 = {S24170, 135, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SC-02E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974482 = {S24171, 137, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974645 = {S24172, 137, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SPH-L900 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974728 = {S24173, 137, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974793 = {S24174, 137, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974873 = {S24175, 138, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974880 = {S24176, 138, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I317M Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974910 = {S24177, 137, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974922 = {S24178, 137, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974928 = {S24179, 138, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975031 = {S24180, 137, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975084 = {S24181, 138, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975575 = {S24182, 138, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021093 = {S24183, 137, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-N719 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021566 = {S24184, 130, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021671 = {S24185, 138, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SHV-E250S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022801 = {S24186, 138, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889V Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023114 = {S24187, 137, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-R950 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024222 = {S24188, 130, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3058047 = {S24189, 137, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; GT-N7100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3077963 = {S24190, 137, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-N719 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3107838 = {S24191, 138, { &P18442,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SHV-E250K Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA94067 = {S24192, 65, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; de) Opera 9.21*/
const Device UA96734 = {S24193, 34, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Opera/9.23 (Windows NT 5.1; U; ru)*/
const Device UA97845 = {S24194, 34, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Opera/9.27 (Windows NT 5.1; U; ru)*/
const Device UA100011 = {S24195, 34, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Opera/9.24 (Windows NT 5.1; U; ru)*/
const Device UA101072 = {S24196, 34, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Opera/9.20 (Windows NT 5.1; U; de)*/
const Device UA103409 = {S24197, 34, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Opera/9.21 (Windows NT 5.1; U; en)*/
const Device UA103576 = {S24198, 34, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Opera/9.22 (Windows NT 5.1; U; en)*/
const Device UA145867 = {S24199, 34, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Opera/9.26 (Windows NT 5.1; U; ru)*/
const Device UA146340 = {S24200, 34, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Opera/9.25 (Windows NT 5.1; U; ru)*/
const Device UA188724 = {S24201, 34, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Opera/9.21 (Windows NT 5.2; U; ru)*/
const Device UA190612 = {S24202, 34, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - opera/9.26 (windows nt 5.2; u; de)*/
const Device UA200780 = {S24203, 34, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Opera/9.27 (Windows NT 5.2; U; ru)*/
const Device UA234757 = {S24204, 34, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Opera/9.23 (Windows NT 5.2; U; en)*/
const Device UA274033 = {S24205, 34, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Opera/9.20 (Windows NT 5.2; U; en)*/
const Device UA415733 = {S24206, 65, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 9.24*/
const Device UA536306 = {S24207, 57, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Opera/9.27 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru)*/
const Device UA660651 = {S24208, 65, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; en) Opera 9.27*/
const Device UA757269 = {S24209, 81, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; es-la; rv:1.8.0) Gecko/X Firefox/1.5.0 Opera 9.27*/
const Device UA768119 = {S24210, 65, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 9.26*/
const Device UA811048 = {S24211, 78, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.8.0) Gecko/X Firefox/1.5.0 Opera 9.24*/
const Device UA826642 = {S24212, 34, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Opera/9.25 (Windows NT 5.2; U; en)*/
const Device UA855429 = {S24213, 78, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.8.0) Gecko/X Firefox/1.5.0 Opera 9.22*/
const Device UA862298 = {S24214, 78, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.8.0) Gecko/X Firefox/1.5.0 Opera 9.26*/
const Device UA872910 = {S24215, 78, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; U; en; rv:1.8.0) Gecko/X Firefox/1.5.0 Opera 9.27*/
const Device UA1899283 = {S24216, 47, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Opera/9.20 (Windows NT 5.1; U; pl) Version/10.9*/
const Device UA2126142 = {S24217, 65, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 9.22*/
const Device UA2131929 = {S24218, 62, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Opera/9.20 (Windows NT 5.1; U; pl) Presto/2.2.15 Version/10.10*/
const Device UA2266394 = {S24219, 86, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; MEGAUPLOAD 1.0; ru) Opera 9.23*/
const Device UA2275311 = {S24220, 57, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Opera/9.27 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru)*/
const Device UA2310552 = {S24221, 56, { &P15364,&P5690,&P18405,&P18092 }};	/*Useragent - Opera/9.27 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru)*/
const Device UA2001942 = {S24222, 141, { &P18223,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT22i Build/6.1.B.0.544) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2008883 = {S24223, 153, { &P18223,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonLT22i Build/6.1.B.0.544) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2235285 = {S24224, 141, { &P18223,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; LT22i Build/6.1.B.0.550) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257624 = {S24225, 132, { &P18223,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT22i Build/6.1.B.0.544) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2403649 = {S24226, 64, { &P18223,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT22i Build/6.1.1.B.1.54)*/
const Device UA2422631 = {S24227, 156, { &P18223,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SonyEricssonLT22i-o Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2424491 = {S24228, 142, { &P18223,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT22i Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455992 = {S24229, 141, { &P18223,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; LT22i Build/6.1.B.0.559) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2505162 = {S24230, 133, { &P18223,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT22i Build/6.1.1.B.1.54) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2530863 = {S24231, 142, { &P18223,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; LT22i Build/6.1.1.B.1.75) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2202888 = {S24232, 73, { &P15364,&P7133,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:19.0) Gecko/19.0 Firefox/19.0*/
const Device UA2471326 = {S24233, 88, { &P15364,&P7133,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:19.0) Gecko/X Firefox/19.0 AlexaToolbar/alxf-2.17*/
const Device UA2750104 = {S24234, 82, { &P15364,&P7133,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.1; Win64; x64; rv:19.0) Gecko/19.0 Firefox/19.0*/
const Device UA2783296 = {S24235, 110, { &P15364,&P7133,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; WOW64; en-US; rv:19.0) Gecko/X AskTbAVR-IDW/3.12.5.17700 Firefox/19.0*/
const Device UA456136 = {S24236, 101, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.9 Safari/536.5*/
const Device UA610013 = {S24237, 64, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) Chrome/19.0.1042.0 Safari/535.21*/
const Device UA626808 = {S24238, 122, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.8 (KHTML, like Gecko; Google Web Preview) Chrome/19.0.1084.36 Safari/536.8*/
const Device UA899229 = {S24239, 137, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5,gzip(gfe) (via translate.google.com)*/
const Device UA900667 = {S24240, 112, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/536.8 (KHTML, like Gecko) Chrome/19.0.1084.36 Safari/536.8*/
const Device UA900841 = {S24241, 100, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA901772 = {S24242, 161, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.8 (KHTML, like Gecko; Google Web Preview) Chrome/19.0.1084.36 Safari/536.8,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA902115 = {S24243, 109, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA924490 = {S24244, 130, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.8 (KHTML, like Gecko; Google Page Speed Insights) Chrome/19.0.1084.36 Safari/536.8*/
const Device UA1976846 = {S24245, 106, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - "Mozilla/5.0 (X11; gzOS Linux i686) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.9 Safari/536.5"*/
const Device UA2333086 = {S24246, 103, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.20 (KHTML	 like Gecko) Chrome/19.0.1036.7 Safari/535.20*/
const Device UA2333410 = {S24247, 101, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.20 (KHTML	 like Gecko) Chrome/19.0.1036.7 Safari/535.20*/
const Device UA2377590 = {S24248, 101, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.9 Safari/536.5*/
const Device UA2674475 = {S24249, 106, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/19.0.1410.63 Safari/537.31*/
const Device UA2674514 = {S24250, 106, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/19.0.1410.63 Safari/537.31*/
const Device UA2674794 = {S24251, 109, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/19.0.1410.63 Safari/537.31*/
const Device UA2674994 = {S24252, 106, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/19.0.1410.63 Safari/537.31*/
const Device UA2675708 = {S24253, 109, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/19.0.1410.63 Safari/537.31*/
const Device UA2714590 = {S24254, 109, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/19.0.1410.63 Safari/537.31*/
const Device UA2717981 = {S24255, 106, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/19.0.1410.63 Safari/537.31*/
const Device UA3032779 = {S24256, 89, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.9 Safari/536.5*/
const Device UA3043679 = {S24257, 98, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/19.77.34.5 Safari/537.1*/
const Device UA3070916 = {S24258, 100, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/19.77.34.5 Safari/537.1*/
const Device UA3070931 = {S24259, 101, { &P15364,&P18110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i586) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/19.77.34.5 Safari/537.1*/
const Device UA589781 = {S24260, 84, { &P15364,&P7133,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:10.0.1) Gecko/X Firefox/10.0.1 Firefox/10.0.1*/
const Device UA591073 = {S24261, 74, { &P15364,&P7133,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:10.0.2) Gecko/X Firefox/10.0.2*/
const Device UA2323120 = {S24262, 67, { &P15364,&P7133,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.1; rv:10.0) Gecko/X Firefox/10.0*/
const Device UA2461500 = {S24263, 71, { &P15364,&P7133,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:10.0.12) Gecko/X Firefox/10.0.12*/
const Device UA2673033 = {S24264, 103, { &P15364,&P7133,&P18099,&P18092 }};	/*Useragent - Mozilla/10 (Windows; U; Windows NT 6.1; en-US;     rv:1.9.2.13) Gecko/X Firefox/10 (.NET CLR 3.5.30729)*/
const Device UA2006229 = {S24265, 146, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2006679 = {S24266, 138, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SPH-L710 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2177336 = {S24267, 138, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2186176 = {S24268, 139, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-R530U Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2196742 = {S24269, 139, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747M Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2204667 = {S24270, 138, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2205857 = {S24271, 139, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305T Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2235175 = {S24272, 139, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2237768 = {S24273, 138, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2237933 = {S24274, 146, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2238057 = {S24275, 138, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2353656 = {S24276, 139, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305N Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2356504 = {S24277, 139, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SHV-E210K Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2357104 = {S24278, 139, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SHV-E210S Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2366802 = {S24279, 139, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SHV-E210L Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2427234 = {S24280, 138, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2430970 = {S24281, 139, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2432035 = {S24282, 139, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2436416 = {S24283, 139, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2529640 = {S24284, 139, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2658023 = {S24285, 139, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530C Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2676853 = {S24286, 139, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305N Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2878150 = {S24287, 139, { &P18645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747m Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA624777 = {S24288, 88, { &P17595,&P17295,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.5; en-GB; rv:1.9.2.28) Gecko/X Firefox/3.6.28*/
const Device UA3064663 = {S24289, 137, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3064884 = {S24290, 137, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3064893 = {S24291, 138, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3064989 = {S24292, 137, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065079 = {S24293, 145, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065086 = {S24294, 137, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065252 = {S24295, 138, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065381 = {S24296, 138, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065582 = {S24297, 137, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065612 = {S24298, 138, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065743 = {S24299, 137, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066532 = {S24300, 135, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SC-02E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066878 = {S24301, 137, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067197 = {S24302, 138, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067893 = {S24303, 145, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I317 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068061 = {S24304, 137, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068214 = {S24305, 138, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080077 = {S24306, 138, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080127 = {S24307, 130, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3081030 = {S24308, 137, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SPH-L900 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081185 = {S24309, 138, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081278 = {S24310, 138, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I317M Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083293 = {S24311, 137, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094436 = {S24312, 138, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SHV-E250S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111553 = {S24313, 137, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-R950 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114555 = {S24314, 138, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889V Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126465 = {S24315, 137, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-N719 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126963 = {S24316, 130, { &P18442,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA136461 = {S24317, 116, { &P15364,&P5690,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA136829 = {S24318, 153, { &P15364,&P5690,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3,gzip(gfe) (via translate.google.com)*/
const Device UA300911 = {S24319, 115, { &P15364,&P5690,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.4 (KHTML, like Gecko) Chrome/6.0.481.0 Safari/534.4*/
const Device UA395655 = {S24320, 140, { &P15364,&P5690,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Source Client/0; ) AppleWebKit/534.1 (KHTML, like Gecko) Chrome/6.0.444.0 Safari/534.1*/
const Device UA885822 = {S24321, 142, { &P15364,&P5690,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Steam GameOverlay; ) AppleWebKit/534.1 (KHTML, like Gecko) Chrome/6.0.444.0 Safari/534.1*/
const Device UA2314555 = {S24322, 116, { &P15364,&P5690,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.3 (KHTML	 like Gecko) Chrome/6.0.472.53 Safari/534.3*/
const Device UA2594555 = {S24323, 126, { &P15364,&P5690,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3,gzip(gfe)*/
const Device UA2937963 = {S24324, 16, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - Sapere/Nutch-2.2*/
const Device UA2944422 = {S24325, 25, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - My Nutch Spider/Nutch-1.7*/
const Device UA2965378 = {S24326, 17, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - guoming/Nutch-1.6*/
const Device UA2967122 = {S24327, 26, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - NIS Nutch Spider/Nutch-1.7*/
const Device UA2972141 = {S24328, 21, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - MyNutchTest/Nutch-1.7*/
const Device UA2973810 = {S24329, 24, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - ccsnutchspider/Nutch-2.2*/
const Device UA2973980 = {S24330, 22, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - Focus Spider/Nutch-1.6*/
const Device UA2983585 = {S24331, 15, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - crawl/Nutch-2.0*/
const Device UA2985796 = {S24332, 144, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - linguatools-bot/Nutch-1.6 (searching for translated pages; http://www.linguatools.de/linguatoolsbot.html; peter dot kolb at linguatools dot org)*/
const Device UA3010435 = {S24333, 21, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - NutchSpider/Nutch-1.6*/
const Device UA3012175 = {S24334, 15, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - grank/Nutch-1.6*/
const Device UA3015093 = {S24335, 15, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - shiva/Nutch-1.4*/
const Device UA3018312 = {S24336, 111, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - NutchCVS/0.7.1 (Nutch; http://lucene.apache.org/nutch/bot.html; nutch-agent@lucene.apache.org);c2VvLWNyYXdsZXI=*/
const Device UA3032714 = {S24337, 70, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - My Nutch Spider/Nutch-1.7 (http://wiki.creativecommons.org/DiscoverEd)*/
const Device UA3033391 = {S24338, 22, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - tomit spider/Nutch-1.7*/
const Device UA3033935 = {S24339, 22, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - zctech agent/Nutch-2.1*/
const Device UA3036045 = {S24340, 62, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - CCResearchBot/1.0 commoncrawl.org/research//Nutch-1.7-SNAPSHOT*/
const Device UA3064045 = {S24341, 18, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - Mysite/Nutch-2.2.1*/
const Device UA3067625 = {S24342, 27, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - YourNutchSpider/Nutch-2.2.1*/
const Device UA3072684 = {S24343, 51, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - PulsePoint bot/Nutch-1.6 (www.pulsepoint.com; info)*/
const Device UA3077851 = {S24344, 21, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - foresealife/Nutch-1.6*/
const Device UA3078179 = {S24345, 72, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - Domnutch-Bot/Nutch-1.0 (Domnutch; http://www.Nutch.de/);c2VvLWNyYXdsZXI=*/
const Device UA3084034 = {S24346, 12, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - CB/Nutch-1.7*/
const Device UA3096271 = {S24347, 19, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - Oculus360/Nutch-1.7*/
const Device UA3107363 = {S24348, 27, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - My Nutch Spider/Nutch-2.2.1*/
const Device UA3107402 = {S24349, 32, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - nutch-solr-integration/Nutch-1.7*/
const Device UA3111761 = {S24350, 20, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - gorgor/Nutch-2.0-dev*/
const Device UA3114275 = {S24351, 17, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - Nutch12/Nutch-1.2*/
const Device UA3119028 = {S24352, 119, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - LWNutch/Nutch-1.4 (another scientific bot - we check your robots.txt! ; contact by mail: abuse _at- languageweaver.com)*/
const Device UA3129560 = {S24353, 37, { &P15364,&P7480,&P18245,&P18163 }};	/*Useragent - Telecom/Nutch-1.6 (warning@gmail.com)*/
const Device UA350550 = {S24354, 131, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-ca; GT-P7500R Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA360835 = {S24355, 59, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7500 Build/HTJ85B)*/
const Device UA586661 = {S24356, 149, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; SAMSUNG-GT-P7500/P7500BUKL1 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA623046 = {S24357, 130, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA689991 = {S24358, 131, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fr-ca; GT-P7500V Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA755617 = {S24359, 60, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7500R Build/HTJ85B)*/
const Device UA771734 = {S24360, 149, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7500/P7500BOKK1 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA838526 = {S24361, 148, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7500-ORANGE/P7500BVKK2 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA877602 = {S24362, 148, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; es-es; GT-P7500-ORANGE/P7500BVLA1 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2024364 = {S24363, 128, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2026106 = {S24364, 137, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2026665 = {S24365, 149, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7500/P7500BOLA1 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2607643 = {S24366, 149, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; SAMSUNG-GT-P7500/P7500BUKJ1 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2607823 = {S24367, 138, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-ca; GT-P7500R Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2607953 = {S24368, 72, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 3.2; en_CA; GT-P7500R Build/HTJ85B.UXKK2)*/
const Device UA2608080 = {S24369, 148, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7500-ORANGE/P7500BVKK1 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2609219 = {S24370, 60, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7500V Build/HTJ85B)*/
const Device UA2609281 = {S24371, 123, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7500 Build/HTJ85B) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2610879 = {S24372, 148, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7500-ORANGE/P7500BVKJ5 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2611344 = {S24373, 128, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7500 Build/HTJ85B) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2611315 = {S24374, 118, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7500) AntivirusAM/2.5 (KHTML, like Gecko) Version/Version 2.5/Antivirus*/
const Device UA2611391 = {S24375, 149, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7500-ORANGE/P7500BVLA1T Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2612178 = {S24376, 92, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 3.2; SAMSUNG GT-P7500 Build/HONEYCOMB_MR2);*/
const Device UA2613128 = {S24377, 129, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; es-; GT-P7500R Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2613896 = {S24378, 153, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7500 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2615362 = {S24379, 115, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Android 3.2; en-gb; GT-P7500 Build/HTJ85B) FreeWheelAdManager/5.0.3-r9054-X;com.gotv.crackle.handset/X*/
const Device UA2615831 = {S24380, 91, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - com.google.android.youtube/4.1.23(Linux; U; Android 3.2; en_GB; GT-P7500 Build/HTJ85B) gzip*/
const Device UA2615835 = {S24381, 128, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 3.2; zh-cn; GT-P7500 Build HTJ85B) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2616037 = {S24382, 129, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P7500V Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2616347 = {S24383, 147, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-; SAMSUNG-GT-P7500/P7500BUKL1 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2616387 = {S24384, 138, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en_ph-; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2616597 = {S24385, 71, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 3.2; pt-BR; GT-P7500 Build/HTJ85B.UBKL3)*/
const Device UA2616971 = {S24386, 60, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.0; 1) GT-P7500 (Android 3.2, SDK 13)*/
const Device UA2617011 = {S24387, 137, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2617137 = {S24388, 71, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 3.2; es-US; GT-P7500 Build/HTJ85B.UBKJ1)*/
const Device UA2617341 = {S24389, 266, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.11;FBDM/{density=1.0,width=1280,height=752};FBLC/en_US;FBCR/Smart;FBPN/com.facebook.katana;FBDV/GT-P7500;FBSV/3.2;]*/
const Device UA2617506 = {S24390, 141, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7500 Build/HTJ85B) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2617599 = {S24391, 72, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Biblia/2.2.3 (Linux; U; Android 3.2; es-ES; GT-P7500 Build/HTJ85B.UBKL1)*/
const Device UA2617810 = {S24392, 71, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 3.2; pt-BR; GT-P7500 Build/HTJ85B.UBLB2)*/
const Device UA2618039 = {S24393, 160, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ms-my; GT-P7500 Build/HTJ85B) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2618329 = {S24394, 132, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7500 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2618420 = {S24395, 277, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-ca; GT-P7500R Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.11;FBDM/{density=1.0,width=1280,height=752};FBLC/en_CA;FB_FW/2;FBCR/ROGERS;FBPN/com.facebook.katana;FBDV/GT-P7500R;FBSV/3.2;]*/
const Device UA2618706 = {S24396, 93, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 3.2; SAMSUNG GT-P7500R Build/HONEYCOMB_MR2);*/
const Device UA2618743 = {S24397, 269, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.11;FBDM/{density=1.0,width=1280,height=752};FBLC/en_GB;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P7500;FBSV/3.2;]*/
const Device UA2618868 = {S24398, 77, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - MediaPlayer/2002/4.0.12 (Android 3.2; GT-P7500; GT-P7500; samsung; ; ; en_GB)*/
const Device UA2619938 = {S24399, 276, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.6.0;FBPN/com.facebook.katana;FBDV/GT-P7500;FBSV/3.2;FBDM/{density=1.0,width=1280,height=752};FBCR/MY MAXIS;FBLC/en_GB;FB_FW/1;]*/
const Device UA2620346 = {S24400, 74, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - GoogleAnalytics/2.0 (Linux; U; Android 3.2; en-ca; GT-P7500R Build/HTJ85B)*/
const Device UA2620966 = {S24401, 130, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7500 Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2621519 = {S24402, 73, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - GroupMe-Android/3.3.5 (Android 3.2; en_US; samsung GT-P7500 Build/HTJ85B)*/
const Device UA2622381 = {S24403, 274, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/2.0;FBDM/{density=1.0,width=1280,height=752};FBLC/en_US;FB_FW/2;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P7500;FBSV/3.2;]*/
const Device UA2622786 = {S24404, 76, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - MediaPlayer/2002/4.1.0 (Android 3.2; GT-P7500; GT-P7500; samsung; ; ; en_US)*/
const Device UA2622884 = {S24405, 89, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Android (Linux; U; Android 3.2; fa-fa; GT-P7500) AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA2624468 = {S24406, 132, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; GT-P7500 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2624818 = {S24407, 135, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7500 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2626625 = {S24408, 76, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - MediaPlayer/2002/4.2.0 (Android 3.2; GT-P7500; GT-P7500; samsung; ; ; ms_MY)*/
const Device UA2627078 = {S24409, 88, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik 2.05.12114; (Linux; U; Android 3.2; GT-P7500R; ON) 372b581e949b1424; 1280x752x1.0*/
const Device UA2627709 = {S24410, 150, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7500/P7500BUKL1T Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2628022 = {S24411, 143, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-ca; GT-P7500R Build/HTJ85B) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2628400 = {S24412, 78, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - VKAndroidApp/2.7.6-64 (Android 3.2; SDK 13; armeabi-v7a; samsung GT-P7500; en)*/
const Device UA2629087 = {S24413, 61, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.0; 5) GT-P7500R (Android 3.2, SDK 13)*/
const Device UA2629088 = {S24414, 68, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-ca; GT-P7500R) CBC News/1.0.5*/
const Device UA2630279 = {S24415, 134, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Groupon/2.0.2414 (Android 3.2; Samsung GT-P7500R / Samsung GT-P7500R; ROGERS)[preload=false;locale=en_CA;clientidbase=android-samsung]*/
const Device UA2633838 = {S24416, 84, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fr-ca; GT-P7500V Build/HTJ85B) GameCenter/3.0206*/
const Device UA2634302 = {S24417, 154, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; HTJ85B; samsung; GT-P7500R; en-us; dream) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2635781 = {S24418, 61, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.1; 9) GT-P7500R (Android 3.2, SDK 13)*/
const Device UA2636056 = {S24419, 177, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7500R Build/HTJ85B) [FBAN/Orca-Android;FBAV/2.3.2-release;FBLC/en_CA;FBBV/X;FBCR/null;FBMF/samsung;FBBD/samsung;FBDV/GT-P7500R;FBSV/3.2]*/
const Device UA2636127 = {S24420, 122, { &P17337,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3026496 = {S24421, 133, { &P15767,&P19826,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B329 Safari/8536.25*/
const Device UA3026845 = {S24422, 133, { &P15767,&P19826,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B146 Safari/8536.25*/
const Device UA3026859 = {S24423, 131, { &P15767,&P19826,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B141 Safari/8536.25*/
const Device UA3027404 = {S24424, 133, { &P15767,&P19826,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B147 Safari/8536.25*/
const Device UA32766 = {S24425, 143, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-P1000M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA33414 = {S24426, 142, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38584 = {S24427, 146, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000 (Seetah) Build/Seetah) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40772 = {S24428, 143, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-P1000R Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA56620 = {S24429, 143, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000N Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA56763 = {S24430, 143, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; GT-P1000T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74468 = {S24431, 143, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; GT-P1000L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA195755 = {S24432, 66, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-P1000 Build/GINGERBREAD)*/
const Device UA370111 = {S24433, 143, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-P1000L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA586255 = {S24434, 143, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-P1000N Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625244 = {S24435, 66, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-P1000 Build/GINGERBREAD)*/
const Device UA671631 = {S24436, 142, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA713869 = {S24437, 136, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000 Build/GR144) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA721655 = {S24438, 142, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA725649 = {S24439, 136, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA755463 = {S24440, 120, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; es-es; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA812743 = {S24441, 67, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-P1000N Build/GINGERBREAD)*/
const Device UA1898694 = {S24442, 120, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; it-it; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA1943553 = {S24443, 140, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020311 = {S24444, 130, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2020549 = {S24445, 67, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-P1000T Build/GINGERBREAD)*/
const Device UA2020577 = {S24446, 67, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-P1000M Build/GINGERBREAD)*/
const Device UA2021052 = {S24447, 67, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-P1000R Build/GINGERBREAD)*/
const Device UA2022944 = {S24448, 140, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028195 = {S24449, 130, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-P1000 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2029340 = {S24450, 142, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2221822 = {S24451, 120, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; de-de; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2310439 = {S24452, 120, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; pt-pt; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2361423 = {S24453, 161, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-P1000/P1000BOJU3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2411442 = {S24454, 120, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; fr-fr; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2411843 = {S24455, 142, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2428312 = {S24456, 143, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-P1000T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2442797 = {S24457, 135, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-P1000 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2445774 = {S24458, 120, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2607568 = {S24459, 66, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.1; GT-P1000 Build/GINGERBREAD)*/
const Device UA2607591 = {S24460, 136, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-P1000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2607619 = {S24461, 67, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-P1000L Build/GINGERBREAD)*/
const Device UA2607632 = {S24462, 98, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.3.3;Zh_cn;GT-P1000;600*1024;)AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA2607825 = {S24463, 140, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2607884 = {S24464, 135, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-ca; GT-P1000 Build/BOCA) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2607910 = {S24465, 168, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en_gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)*/
const Device UA2607921 = {S24466, 136, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT-P1000 Build/GR144) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2607960 = {S24467, 141, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.3; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2607970 = {S24468, 141, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-P1000T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2608208 = {S24469, 66, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; GT-P1000 Build/GINGERBREAD)*/
const Device UA2608273 = {S24470, 135, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-P1000 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2608490 = {S24471, 136, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2608637 = {S24472, 130, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-P1000 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2608670 = {S24473, 85, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.3; en_GB; GT-P1000 Build/Genesis 3.0 update 2.1.1)*/
const Device UA2608802 = {S24474, 136, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; GT-P1000 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2608817 = {S24475, 34, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Android/1.0 (GT-P1000 GINGERBREAD)*/
const Device UA2608842 = {S24476, 66, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-P1000 Build/GINGERBREAD)*/
const Device UA2608851 = {S24477, 141, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2609224 = {S24478, 167, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-P1000 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2609752 = {S24479, 135, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.3; zh-cn; GT-P1000 Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2609767 = {S24480, 147, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2610252 = {S24481, 67, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-P1000N Build/GINGERBREAD)*/
const Device UA2610291 = {S24482, 67, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-P1000L Build/GINGERBREAD)*/
const Device UA2610393 = {S24483, 66, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-P1000 Build/GINGERBREAD)*/
const Device UA2610650 = {S24484, 142, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2610979 = {S24485, 134, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2611323 = {S24486, 150, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SAMSUNG GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2611469 = {S24487, 135, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-P1000 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2611379 = {S24488, 290, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 [FBAN/FB4A;FBAV/1.9.3;FBDM/{density=1.5,width=600,height=1024};FBLC/en_GB;FB_FW/2;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P1000;FBSV/2.3.3;]*/
const Device UA2612184 = {S24489, 158, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-P1000 Build/GINGERBREAD; 600*1024) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2612814 = {S24490, 62, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.0; 1) GT-P1000 (Android 2.3.3, SDK 10)*/
const Device UA2613055 = {S24491, 135, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-P1000 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2615296 = {S24492, 151, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000T Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA2615623 = {S24493, 78, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 2.3.3; en-GB; GT-P1000 Build/GINGERBREAD.DXJPJ)*/
const Device UA2615683 = {S24494, 135, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; GT-P1000 Build/BOCA) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2615949 = {S24495, 78, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 2.3.3; en-ZA; GT-P1000 Build/GINGERBREAD.XXJQ1)*/
const Device UA2616065 = {S24496, 140, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2616119 = {S24497, 136, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; GT-P1000T Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2616163 = {S24498, 80, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Biblia/2.1.2 (Linux; U; Android 2.3.3; es-US; GT-P1000N Build/GINGERBREAD.UBJP7)*/
const Device UA2616196 = {S24499, 141, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-P1000 Build/GalnetMIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2616394 = {S24500, 78, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 2.3.3; es-ES; GT-P1000 Build/GINGERBREAD.XWJPA)*/
const Device UA2616441 = {S24501, 141, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2616452 = {S24502, 78, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 2.3.3; en-ZA; GT-P1000 Build/GINGERBREAD.XWJPE)*/
const Device UA2616459 = {S24503, 78, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Vyrso/2.1.2 (Linux; U; Android 2.3.3; en-US; GT-P1000 Build/GINGERBREAD.XWJP9)*/
const Device UA2616471 = {S24504, 144, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en_us; GT-P1000 Build/NFP91) AppleWebKit/534.15.12 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.15.12*/
const Device UA2616520 = {S24505, 141, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA2616618 = {S24506, 124, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000) antivirusAM/2.4.5 (KHTML, like Gecko) Version/Version 2.4.5/antivirus*/
const Device UA2616984 = {S24507, 78, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 2.3.3; en-GB; GT-P1000 Build/GINGERBREAD.JPJPD)*/
const Device UA2616983 = {S24508, 150, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2617224 = {S24509, 78, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 2.3.3; en-GB; GT-P1000 Build/GINGERBREAD.DXJPE)*/
const Device UA2617338 = {S24510, 78, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 2.3.3; zh-HK; GT-P1000 Build/GINGERBREAD.ZHJP7)*/
const Device UA2617437 = {S24511, 141, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-P1000L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2617465 = {S24512, 67, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-P1000 Build/7G21C0)*/
const Device UA2617715 = {S24513, 140, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2618273 = {S24514, 78, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Vyrso/2.1.2 (Linux; U; Android 2.3.3; en-AU; GT-P1000 Build/GINGERBREAD.XWJPJ)*/
const Device UA2618274 = {S24515, 151, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-us; GT-P1000L Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2618299 = {S24516, 79, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - MediaPlayer/2002/4.0.12 (Android 2.3.3; GT-P1000; GT-P1000; samsung; ; ; en_US)*/
const Device UA2618321 = {S24517, 148, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android BadBoyBunny; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2618542 = {S24518, 65, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.3.3; 15) GT-P1000 (Android 2.3.3, SDK 10)*/
const Device UA2618664 = {S24519, 85, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Vyrso/2.2.3 (Linux; U; Android 2.3.6; en-GB; GT-P1000 Build/Overcome 7 Series v4.1.0)*/
const Device UA2618858 = {S24520, 150, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2618942 = {S24521, 79, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/2.0.11 (Linux; U; Android 2.3.6; en-AU; GT-P1000 Build/GINGERBREAD.XWJU5)*/
const Device UA2618951 = {S24522, 81, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - GoogleAnalytics/2.0 (Linux; U; Android 2.3.3; en-us; GT-P1000N Build/GINGERBREAD)*/
const Device UA2618959 = {S24523, 97, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - com.google.android.youtube/2.3.4(Linux; U; Android 2.3.3; en_US; GT-P1000 Build/GINGERBREAD) gzip*/
const Device UA2619198 = {S24524, 167, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ms-my; GT-P1000 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2619292 = {S24525, 290, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 [FBAN/FB4A;FBAV/1.6.3;FBPN/com.facebook.katana;FBDV/GT-P1000;FBSV/2.3.3;FBDM/{density=1.5,width=600,height=1024};FBCR/MY MAXIS;FBLC/en_GB;FB_FW/1;]*/
const Device UA2619793 = {S24526, 78, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; Galaxy-Tab-GT-P1000 Build/RP1VR6)*/
const Device UA2619937 = {S24527, 78, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Vyrso/0.9.4 (Linux; U; Android 2.3.3; en_PH; GT-P1000 Build/GINGERBREAD.DXJPI)*/
const Device UA2620134 = {S24528, 63, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.3; 4) GT-P1000T (Android 2.3.3, SDK 10)*/
const Device UA2620380 = {S24529, 128, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-P1000) antivirusHttpClient/2.0 (KHTML, like Gecko) Version/Version 2.1/antivirus*/
const Device UA2620673 = {S24530, 137, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2620737 = {S24531, 154, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Rdio.Net; Android/2.4.2; Android/4.1.2; DM=samsung; DN=GT-P1000; DC=samsung; DF=samsung/GT-P1000/GT-P1000:2.3.5/GINGERBREAD/XXJVT:user/release-keys; DH=p1*/
const Device UA2620761 = {S24532, 134, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-P1000 Build/GFAN_ROM_X) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2620799 = {S24533, 136, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2620856 = {S24534, 78, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/2.2.7 (Linux; U; Android 2.3.3; zh-TW; GT-P1000 Build/GINGERBREAD.ZUJP8)*/
const Device UA2620866 = {S24535, 81, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.01; 2) GT-P1000 (Android 2.3.3, SDK 10) flash:11.1.102.59*/
const Device UA2621068 = {S24536, 80, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Biblia/2.2.3 (Linux; U; Android 2.3.6; es-US; GT-P1000N Build/GINGERBREAD.UBJR4)*/
const Device UA2621323 = {S24537, 78, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/2.2.7 (Linux; U; Android 2.3.6; zh-HK; GT-P1000 Build/GINGERBREAD.ZHJQ5)*/
const Device UA2621913 = {S24538, 71, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Biblia/2.2.7 (Linux; U; Android 2.3.3; es-US; GT-P1000L Build/HumberOS)*/
const Device UA2622168 = {S24539, 130, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-P1000 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2622235 = {S24540, 148, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; GT-P1000 Build/GINGERBREAD.XWJQ8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2622387 = {S24541, 81, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - GoogleAnalytics/2.0 (Linux; U; Android 2.3.6; es-us; GT-P1000N Build/GINGERBREAD)*/
const Device UA2622397 = {S24542, 82, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Biblia/2.2.8.1 (Linux; U; Android 2.3.3; es-US; GT-P1000N Build/GINGERBREAD.VFJP6)*/
const Device UA2622495 = {S24543, 68, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-P1000R) BBC News/1.3*/
const Device UA2622496 = {S24544, 63, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.3; 4) GT-P1000R (Android 2.3.3, SDK 10)*/
const Device UA2622501 = {S24545, 67, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-P1000T Build/GINGERBREAD)*/
const Device UA2622993 = {S24546, 76, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - MediaPlayer/0000/3.0 (Android 2.3.3; GT-P1000; GT-P1000; samsung; ; ; en_GB)*/
const Device UA2623019 = {S24547, 136, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; GT-P1000 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2623125 = {S24548, 80, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 2.3.6; de-DE; GT-P1000 Build/GINGERBREAD.XWJQ8)*/
const Device UA2623351 = {S24549, 105, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGke Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2624005 = {S24550, 80, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - VKAndroidApp/2.7.6-64 (Android 2.3.6; SDK 10; armeabi-v7a; samsung GT-P1000; ru)*/
const Device UA2624057 = {S24551, 63, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - BBC News (AndroidApp; 2.0; 19) GT-P1000 (Android 2.3.3, SDK 10)*/
const Device UA2624123 = {S24552, 95, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.2.8.1 (Linux; U; Android 2.3.3; en-CA; GT-P1000R Build/GINGERBREAD.UXKG1)*/
const Device UA2624216 = {S24553, 141, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-P1000M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2624669 = {S24554, 128, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Groupon/2.2.2561 (Android 2.3.3; Samsung GT-P1000 / Samsung GT-P1000; )[preload=false;locale=en_GB;clientidbase=android-samsung]*/
const Device UA2625262 = {S24555, 130, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-P1000 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2625433 = {S24556, 85, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 2.3.3; en-US; GT-P1000 Build/Overcome v3.1.0-KRATOS)*/
const Device UA2625434 = {S24557, 135, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000T Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2625514 = {S24558, 38, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Android/2.3.3; samsung GT-P1000; 2.3.2*/
const Device UA2625611 = {S24559, 141, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-P1000L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2625729 = {S24560, 96, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-P1000 Build/GINGERBREAD) Daily Mail Online/1.8.3*/
const Device UA2626147 = {S24561, 169, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en_us; GT-P1000N Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)*/
const Device UA2626288 = {S24562, 63, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.0; 1) GT-P1000R (Android 2.3.3, SDK 10)*/
const Device UA2626374 = {S24563, 99, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - com.google.android.youtube/4.2.16(Linux; U; Android 2.3.3; en_CA; GT-P1000R Build/GINGERBREAD) gzip*/
const Device UA2626526 = {S24564, 156, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.0 Version/4.0 Mobile Safari/530.17*/
const Device UA2626687 = {S24565, 131, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2626708 = {S24566, 121, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000N) AntivirusAM/2.5 (KHTML, like Gecko) Version/Version 2.5/Antivirus*/
const Device UA2626772 = {S24567, 80, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - VKAndroidApp/2.7.6-64 (Android 2.3.6; SDK 10; armeabi-v7a; samsung GT-P1000; en)*/
const Device UA2626830 = {S24568, 79, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/2.2.10 (Linux; U; Android 2.3.3; en-US; GT-P1000 Build/GINGERBREAD.ZTJP8)*/
const Device UA2626975 = {S24569, 157, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000T Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.0 Version/4.0 Mobile Safari/530.17*/
const Device UA2627136 = {S24570, 143, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; cs-cz; GT-P1000L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2627241 = {S24571, 96, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - nApps (Android 2.3.3; GT-P1000R; %EB%84%A4%EC%9D%B4%EB%B2%84+%EC%B9%B4%EB%A9%94%EB%9D%BC; 1.8.2)*/
const Device UA2627247 = {S24572, 161, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hr-hr; SAMSUNG GT-P1000/P1000BOJU2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2627486 = {S24573, 267, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 baidubrowser/042_1.41.3.2_diordna_4201_006/gnusmas_01_4.3.2_0001P-TG/Xb/59E3D8A74D6B206FBB45E5A5C4C356D6%7CX/1*/
const Device UA2628010 = {S24574, 70, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-P1000M) CBC News/1.0.5*/
const Device UA2628024 = {S24575, 150, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-P1000R Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2629196 = {S24576, 150, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.7 Mobile Safari/534.30*/
const Device UA2629232 = {S24577, 135, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; GT-P1000 Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2630840 = {S24578, 88, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik 2.11.12449; (Linux; U; Android 2.3.3; GT-P1000; ON) c1b9a046dccce90a; 400x682x1.5*/
const Device UA2631073 = {S24579, 141, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-; GT-P1000N Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2631322 = {S24580, 148, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; SAMSUNG GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2633163 = {S24581, 187, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-P1000L Build/GINGERBREAD) [FBAN/Orca-Android;FBAV/2.3.1-release;FBLC/en_US;FBBV/X;FBCR/chatr;FBMF/samsung;FBBD/samsung;FBDV/GT-P1000L;FBSV/2.3.3]*/
const Device UA2633376 = {S24582, 140, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 INTSafari/534.30*/
const Device UA2634547 = {S24583, 136, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; GT-P1000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2635201 = {S24584, 175, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6 E-3F 'A1H4G; en-us; GT-P1000  by mohsen afrousheh Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2635215 = {S24585, 141, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-; GT-P1000N Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2637435 = {S24586, 125, { &P13452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-P1000 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA363591 = {S24587, 164, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; HTC Sensation XE with Beats Audio Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA363657 = {S24588, 170, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-gb; HTC Sensation XE with Beats Audio Z715e Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA369196 = {S24589, 154, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SensationXE_Beats_Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA369499 = {S24590, 86, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Sensation XE with Beats Audio Build/IML74K)*/
const Device UA369548 = {S24591, 140, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616654 = {S24592, 170, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616708 = {S24593, 158, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-sg; HTC_SensationXE_Beats_Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA628070 = {S24594, 92, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Sensation XE with Beats Audio Z715e Build/IML74K)*/
const Device UA637681 = {S24595, 152, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_SensationXE_Beats Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA682395 = {S24596, 156, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XE with Beats Audio Z715e Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA728189 = {S24597, 164, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA896738 = {S24598, 158, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC_SensationXE_Beats_Z715a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA896852 = {S24599, 170, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC Sensation XE with Beats Audio Z715a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA908436 = {S24600, 161, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XE with Beats Audio Z715e Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1966044 = {S24601, 164, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation XE Build/SmartDroid Berserk Line) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2023759 = {S24602, 166, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2024161 = {S24603, 156, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; HTC_SensationXE_Beats_Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026386 = {S24604, 92, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Sensation XE with Beats Audio Z715a Build/IML74K)*/
const Device UA2028064 = {S24605, 140, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC_Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034003 = {S24606, 156, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_SensationXE_Beats/1.0 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2444736 = {S24607, 131, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Z715e Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2473824 = {S24608, 161, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XE with Beats Audio Z715e Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2484822 = {S24609, 158, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_SensationXE_Beats_Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2487448 = {S24610, 170, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555793 = {S24611, 170, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Sensation_XE_with_Beats_Audio_Z715e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654233 = {S24612, 92, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Sensation XE with Beats Audio Z715e Build/IML74K)*/
const Device UA2727141 = {S24613, 158, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_SensationXE_Beats_Z715e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2982811 = {S24614, 168, { &P13157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2008915 = {S24615, 68, { &P15364,&P7133,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:18.0) Gecko/18.0 Firefox/18.0*/
const Device UA2469408 = {S24616, 70, { &P15364,&P7133,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:18.0) Gecko/X Firefox/18.0*/
const Device UA2530169 = {S24617, 82, { &P15364,&P7133,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.1; Win64; x64; rv:18.0) Gecko/18.0 Firefox/18.0*/
const Device UA2570290 = {S24618, 70, { &P15364,&P7133,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; rv:11.0) Gecko/X Firefox/18.0*/
const Device UA2566453 = {S24619, 138, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566490 = {S24620, 138, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566518 = {S24621, 139, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566628 = {S24622, 138, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566680 = {S24623, 138, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566695 = {S24624, 146, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566710 = {S24625, 138, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566759 = {S24626, 138, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566797 = {S24627, 139, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105T Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566822 = {S24628, 131, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566931 = {S24629, 138, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SPH-L900 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566962 = {S24630, 131, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567094 = {S24631, 131, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567095 = {S24632, 139, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I317M Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567091 = {S24633, 139, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889V Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567098 = {S24634, 132, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105T Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567127 = {S24635, 138, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567133 = {S24636, 138, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-R950 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567201 = {S24637, 139, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567253 = {S24638, 139, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567268 = {S24639, 146, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I317 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567258 = {S24640, 132, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567327 = {S24641, 136, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SC-02E Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567334 = {S24642, 131, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567344 = {S24643, 131, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567384 = {S24644, 138, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R950 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567509 = {S24645, 139, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567495 = {S24646, 131, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567699 = {S24647, 138, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583792 = {S24648, 139, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2584164 = {S24649, 131, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2584866 = {S24650, 139, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2600345 = {S24651, 139, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100T Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602497 = {S24652, 139, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2603894 = {S24653, 131, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R950 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2666999 = {S24654, 132, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667177 = {S24655, 138, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-N719 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667401 = {S24656, 139, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889V Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667472 = {S24657, 131, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SPH-L900 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667837 = {S24658, 139, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I317 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2668774 = {S24659, 132, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889V Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2732962 = {S24660, 139, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SHV-E250S Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2749910 = {S24661, 139, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SHV-E250K Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2793527 = {S24662, 138, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; GT-N7100 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2988586 = {S24663, 132, { &P18442,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I317M Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA90800 = {S24664, 98, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A5313e*/
const Device UA90802 = {S24665, 98, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A5288d*/
const Device UA90804 = {S24666, 98, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A5302b*/
const Device UA90807 = {S24667, 96, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334*/
const Device UA90866 = {S24668, 104, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPad OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334*/
const Device UA90864 = {S24669, 127, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - MQQBrowser/18 Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Safari/7534.48.3*/
const Device UA90872 = {S24670, 85, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)*/
const Device UA90874 = {S24671, 98, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405*/
const Device UA200885 = {S24672, 83, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)*/
const Device UA255937 = {S24673, 132, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - MQQBrowser/1.9.1 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA256417 = {S24674, 112, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/525.20*/
const Device UA257838 = {S24675, 129, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - MQQBrowser/19 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA359887 = {S24676, 237, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 5_0_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1;FBBV/4100.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/5.0.1;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]*/
const Device UA361065 = {S24677, 233, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 5_0 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1;FBBV/4100.0;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/5.0;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]*/
const Device UA616495 = {S24678, 97, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A40*/
const Device UA671880 = {S24679, 129, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - MQQBrowser/18 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA740191 = {S24680, 164, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48 Instapaper/4.1.1 (+http://www.instapaper.com/iphone)*/
const Device UA890789 = {S24681, 98, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9Z999*/
const Device UA1916276 = {S24682, 130, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - MQQBrowser/1.9.1 Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Safari/7534.48.3*/
const Device UA1947089 = {S24683, 112, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - MQQBrowser/10 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405*/
const Device UA2202538 = {S24684, 217, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/5.0.1;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]*/
const Device UA2205102 = {S24685, 213, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPad2,1;FBMD/iPad;FBSN/iPhone OS;FBSV/5.0;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US]*/
const Device UA2403613 = {S24686, 135, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 iPad2,1__weibo__3.1.1__ipad__os5.0.1*/
const Device UA2422832 = {S24687, 128, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/8536.26.7 DiigoBrowser*/
const Device UA2444161 = {S24688, 98, { &P15767,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176*/
const Device UA89270 = {S24689, 56, { &P18254,&P17017,&P18546,&P18092 }};	/*Useragent - Lynx/2.8.6rel.4 libwww-FM/2.14 SSL-MM/1.4.1 GNUTLS/2.0.4*/
const Device UA104629 = {S24690, 43, { &P18254,&P17017,&P18546,&P18092 }};	/*Useragent - Lynx/2.8.7dev.9 libwww-FM/2.14 SSL-MM/1.4.1*/
const Device UA109864 = {S24691, 43, { &P18254,&P17017,&P18546,&P18092 }};	/*Useragent - Lynx/2.8.8dev.2 libwww-FM/2.14 SSL-MM/1.4.1*/
const Device UA127660 = {S24692, 69, { &P18254,&P17017,&P18546,&P18092 }};	/*Useragent - Lynx/2.8.5rel.1 libwww-FM/2.14 SSL-MM/1.4.1 OpenSSL/0.9.8e-fips-rhel5*/
const Device UA141959 = {S24693, 57, { &P18254,&P17017,&P18546,&P18092 }};	/*Useragent - Lynx/2.8.3dev.9 libwww-FM/2.14 SSL-MM/1.4.1 OpenSSL/0.9.6*/
const Device UA157442 = {S24694, 58, { &P18254,&P17017,&P18546,&P18092 }};	/*Useragent - Lynx/2.8.6rel.4 libwww-FM/2.14 SSL-MM/1.4.1 OpenSSL/0.9.8g*/
const Device UA282179 = {S24695, 56, { &P18254,&P17017,&P18546,&P18092 }};	/*Useragent - Lynx/2.8.8dev.5 libwww-FM/2.14 SSL-MM/1.4.1 GNUTLS/2.8.6*/
const Device UA292521 = {S24696, 43, { &P18254,&P17017,&P18546,&P18092 }};	/*Useragent - Lynx/2.8.8dev.3 libwww-FM/2.14 SSL-MM/1.4.1*/
const Device UA450617 = {S24697, 57, { &P18254,&P17017,&P18546,&P18092 }};	/*Useragent - Lynx/2.8.8dev.9 libwww-FM/2.14 SSL-MM/1.4.1 GNUTLS/2.10.5*/
const Device UA473691 = {S24698, 64, { &P18254,&P17017,&P18546,&P18092 }};	/*Useragent - Lynx/2.8.5rel.1 libwww-FM/2.14FM SSL-MM/1.4.1 OpenSSL/0.9.7d-dev*/
const Device UA567502 = {S24699, 62, { &P18254,&P17017,&P18546,&P18092 }};	/*Useragent - Lynx/2.8.6rel.5 libwww-FM/2.14 SSL-MM/1.4.1 OpenSSL/1.0.0-fips*/
const Device UA890148 = {S24700, 58, { &P18254,&P17017,&P18546,&P18092 }};	/*Useragent - Lynx/2.8.7rel.1 libwww-FM/2.14 SSL-MM/1.4.1 OpenSSL/1.0.1c*/
const Device UA2042981 = {S24701, 65, { &P18254,&P17017,&P18546,&P18092 }};	/*Useragent - Lynx/2.8.5rel.1 libwww-FM/2.15FC SSL-MM/1.4.1c OpenSSL/0.9.7e-dev*/
const Device UA2371741 = {S24702, 58, { &P18254,&P17017,&P18546,&P18092 }};	/*Useragent - Lynx/2.8.8dev.9 libwww-FM/2.14 SSL-MM/1.4.1 GNUTLS/2.12.14*/
const Device UA2408647 = {S24703, 58, { &P18254,&P17017,&P18546,&P18092 }};	/*Useragent - Lynx/2.8.6rel.5 libwww-FM/2.14 SSL-MM/1.4.1 OpenSSL/0.9.8h*/
const Device UA2566442 = {S24704, 138, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566511 = {S24705, 139, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566531 = {S24706, 138, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-I535 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566532 = {S24707, 131, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-I535 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566664 = {S24708, 138, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; GT-I9300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2566704 = {S24709, 138, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566795 = {S24710, 138, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566807 = {S24711, 138, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566809 = {S24712, 131, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566888 = {S24713, 139, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999V Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566908 = {S24714, 131, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567107 = {S24715, 131, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567193 = {S24716, 139, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300T Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567385 = {S24717, 138, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567497 = {S24718, 138, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; GT-I9300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583274 = {S24719, 132, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583278 = {S24720, 131, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583801 = {S24721, 139, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I93000 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583962 = {S24722, 132, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999V Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2584095 = {S24723, 138, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-i9300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2732916 = {S24724, 135, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; I9300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2733412 = {S24725, 131, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2863840 = {S24726, 133, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; i9300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2878435 = {S24727, 136, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SC-06D Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2967614 = {S24728, 139, { &P18283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999V Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2015620 = {S24729, 117, { &P17595,&P17296,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2015638 = {S24730, 117, { &P17595,&P17296,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2016035 = {S24731, 117, { &P17595,&P17296,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2016040 = {S24732, 117, { &P17595,&P17296,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2016042 = {S24733, 117, { &P17595,&P17296,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2016046 = {S24734, 117, { &P17595,&P17296,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2016047 = {S24735, 117, { &P17595,&P17296,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2016060 = {S24736, 117, { &P17595,&P17296,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2042270 = {S24737, 154, { &P17595,&P17296,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)*/
const Device UA2199228 = {S24738, 117, { &P17595,&P17296,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.90 Safari/537.1*/
const Device UA2199297 = {S24739, 156, { &P17595,&P17296,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2597352 = {S24740, 114, { &P17595,&P17296,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1060) AppleWebKit/537.1 (KHTML like Gecko) Chrome/21.0.1180.79 Safari/537.1*/
const Device UA2674656 = {S24741, 129, { &P17595,&P17296,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/21.0.648.205 Safari/534.16*/
const Device UA2674675 = {S24742, 129, { &P17595,&P17296,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/21.0.648.205 Safari/534.16*/
const Device UA2719075 = {S24743, 129, { &P17595,&P17296,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/21.0.648.205 Safari/534.16*/
const Device UA180523 = {S24744, 129, { &P17595,&P17296,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16*/
const Device UA181420 = {S24745, 129, { &P17595,&P17296,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16*/
const Device UA198133 = {S24746, 129, { &P17595,&P17296,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA305316 = {S24747, 129, { &P17595,&P17296,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.127 Safari/534.16*/
const Device UA309224 = {S24748, 129, { &P17595,&P17296,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.133 Safari/534.16*/
const Device UA311563 = {S24749, 129, { &P17595,&P17296,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16*/
const Device UA390757 = {S24750, 129, { &P17595,&P17296,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.133 Safari/534.16*/
const Device UA416388 = {S24751, 127, { &P17595,&P17296,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.639.0 Safari/534.16*/
const Device UA859921 = {S24752, 128, { &P17595,&P17296,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.11 Safari/534.16*/
const Device UA2303443 = {S24753, 134, { &P17595,&P17296,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X  10_6_8;  fr_US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.3 idtz*/
const Device UA2318350 = {S24754, 134, { &P17595,&P17296,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X  10_6_4;  fr_US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/10.0.648.134 Safari/534.3 idtz*/
const Device UA2330815 = {S24755, 129, { &P17595,&P17296,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; de-DE) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA2433593 = {S24756, 192, { &P17595,&P17296,&P17249,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; en) Presto/2.7.62 Version/11.00Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16*/
const Device UA440504 = {S24757, 138, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; LT26i Build/6.0.A.3.67) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA440864 = {S24758, 152, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonLT26i-o Build/6.0.A.3.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA603584 = {S24759, 152, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonLT26i-o Build/6.0.A.3.73) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA624787 = {S24760, 150, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonLT26i Build/6.0.A.3.67) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA639557 = {S24761, 138, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; LT26i Build/6.0.A.3.73) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA664232 = {S24762, 152, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; SonyEricssonLT26i-o Build/6.0.A.3.67) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA725916 = {S24763, 62, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; LT26i Build/6.0.A.3.62)*/
const Device UA732306 = {S24764, 150, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; SonyEricssonLT26i Build/6.0.A.3.73) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA849327 = {S24765, 126, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT26i Build/6.0.A.3.62) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA862608 = {S24766, 150, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-mx; SonyEricssonLT26i Build/6.0.A.3.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA895651 = {S24767, 138, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-mx; LT26i Build/6.0.A.3.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA900145 = {S24768, 150, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; SonyEricssonLT26i Build/6.0.A.3.76) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA900175 = {S24769, 138, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; LT26i Build/6.0.A.3.76) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA900413 = {S24770, 150, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonLT26i Build/6.0.A.3.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA900429 = {S24771, 138, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; LT26i Build/6.0.A.3.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA902589 = {S24772, 152, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; SonyEricssonLT26i-o Build/6.0.A.3.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1956912 = {S24773, 131, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT26i Build/6.0.A.3.75) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1977958 = {S24774, 131, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT26i Build/6.0.A.3.73) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2024516 = {S24775, 151, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; SonyEricssonLT26i Build/6.0.A.1.249) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024571 = {S24776, 131, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT26i Build/6.0.A.3.67) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2025182 = {S24777, 151, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; fr-be; SonyEricssonLT26iv Build/6.0.A.3.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028933 = {S24778, 131, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT26i Build/6.0.A.3.62) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2032691 = {S24779, 126, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT26i Build/6.0.A.3.67) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2032990 = {S24780, 126, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT26i Build/6.0.A.3.73) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2033464 = {S24781, 131, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT26i Build/6.0.A.3.76) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2811559 = {S24782, 139, { &P12968,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; LT26i Build/6.0.A.1.249) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2196727 = {S24783, 131, { &P13062,&P7106,&P15829,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9790; en-GB) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.714 Mobile Safari/534.11+*/
const Device UA2976830 = {S24784, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fr-BE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9737IT*/
const Device UA3022434 = {S24785, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; zh-CN) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9737IT*/
const Device UA3022570 = {S24786, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; zh-HK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9737IT*/
const Device UA3022729 = {S24787, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fr-CH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9737IT*/
const Device UA3022749 = {S24788, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-ZA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9737IT*/
const Device UA3023083 = {S24789, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-TT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9737IT*/
const Device UA3023178 = {S24790, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-BE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9737IT*/
const Device UA3023185 = {S24791, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; id-ID) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9737IT*/
const Device UA3024119 = {S24792, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-NG) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9737IT*/
const Device UA3031165 = {S24793, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; de-AT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3032928 = {S24794, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; zh-TW) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3032930 = {S24795, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; de-DE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3032931 = {S24796, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; it-IT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3032939 = {S24797, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ru-RU) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3032943 = {S24798, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; pl-PL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3032953 = {S24799, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; vi-VN) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IP*/
const Device UA3033489 = {S24800, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; tr-TR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3033496 = {S24801, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; nl-NL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3033497 = {S24802, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-ES) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3033507 = {S24803, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ja-JP) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3033491 = {S24804, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; pt-BR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3033986 = {S24805, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fr-FR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3033991 = {S24806, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-IE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3033999 = {S24807, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-CA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3034112 = {S24808, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-MX) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3034365 = {S24809, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-NZ) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3034679 = {S24810, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3035715 = {S24811, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-AU) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3035725 = {S24812, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; he-IL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3035730 = {S24813, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; sv-SE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3036109 = {S24814, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fr-CA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3036132 = {S24815, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-SG) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3036122 = {S24816, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-HK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3036413 = {S24817, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-GB) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3036421 = {S24818, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; nl-BE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3036508 = {S24819, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-IN) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3036510 = {S24820, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; th-TH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3036519 = {S24821, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-CO) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3036526 = {S24822, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; el-GR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3036525 = {S24823, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; pt-PT) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3036827 = {S24824, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ko-KR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3037083 = {S24825, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; da-DK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3037085 = {S24826, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-PH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3037100 = {S24827, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3037099 = {S24828, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; de-CH) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3037103 = {S24829, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fa-IR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3037106 = {S24830, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; nb-NO) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3038046 = {S24831, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ar-SA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3059922 = {S24832, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fi-FI) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3059931 = {S24833, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ru-UA) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3060554 = {S24834, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ar-KW) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IP*/
const Device UA3060555 = {S24835, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-EC) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3060839 = {S24836, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-PK) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IP*/
const Device UA3061297 = {S24837, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-VE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3061298 = {S24838, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-AR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3061301 = {S24839, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-PR) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3061430 = {S24840, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ms-MY) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3062280 = {S24841, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-CL) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3078750 = {S24842, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-PE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3079178 = {S24843, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ca-ES) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3079502 = {S24844, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ar-AE) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3079623 = {S24845, 127, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3079662 = {S24846, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-DO) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT*/
const Device UA3079915 = {S24847, 130, { &P15364,&P18110,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-AS) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IP*/
const Device UA11403 = {S24848, 142, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; et-ee; LG-P500 Build/FRG83) AppleWebKit/528.5+ (KHTML, nagu Gecko) Versioon/3.1.2 Mobile Safari/525.20,1*/
const Device UA11430 = {S24849, 161, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; pt-; LG-P500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA11434 = {S24850, 168, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P500/v10q Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA11443 = {S24851, 168, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; LG-P500/v10p Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA11471 = {S24852, 168, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; it-it; LG-P500/V10b Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA16334 = {S24853, 164, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; LG-P500h Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA16338 = {S24854, 162, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ko-; LG-P500h Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA16340 = {S24855, 171, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; LG-P500h-parrot Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA23677 = {S24856, 163, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA26556 = {S24857, 166, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; LG-P500/v10a Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA26568 = {S24858, 139, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ru-ru; LG-P500 Build/FRF91) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA26588 = {S24859, 170, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; LG-P500-Bouygues/Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA26595 = {S24860, 166, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; it-it; LG-P500/V10a Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA26596 = {S24861, 141, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; LG-P500 Build/FRG83) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA40248 = {S24862, 94, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.2;Zh_cn;LG-P500;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA40253 = {S24863, 161, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pt-; LG-P500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA40257 = {S24864, 173, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; LG-P500-Bouygues/ Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA40288 = {S24865, 96, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.2.2;Zh_cn;LG-P500;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA40295 = {S24866, 163, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; LG-P500 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA41034 = {S24867, 164, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P500h Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA41041 = {S24868, 169, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pt-; LG-P500h-parrot Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA41042 = {S24869, 169, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-ca; LG-P500h-parrot Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA48911 = {S24870, 160, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ko-; LG-P500h Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA53984 = {S24871, 162, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-sa; LG-P500h Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA55493 = {S24872, 145, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; LG Optimus One P500 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA56718 = {S24873, 137, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; LG-P500h Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA62858 = {S24874, 163, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; LG-P500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA64582 = {S24875, 161, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; LG-P500 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA90484 = {S24876, 143, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; LG-P500h Build/FRG83) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA166966 = {S24877, 57, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; LG-P500 Build/FRF91)*/
const Device UA173147 = {S24878, 59, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; LG-P500 Build/FRG83)*/
const Device UA182029 = {S24879, 59, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P500 Build/FRG83)*/
const Device UA524261 = {S24880, 58, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; LG-P500h Build/FRF91)*/
const Device UA541439 = {S24881, 60, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; LG-P500h Build/FRG83)*/
const Device UA668853 = {S24882, 123, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; LG-P500 Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA708648 = {S24883, 123, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-in; LG-P500 Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2034577 = {S24884, 131, { &P14967,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0(Linux; U; Android 2.2; en-gb; LG-P500 Build/FRF91) AppleWebKit/533.0 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.*/
const Device UA804125 = {S24885, 132, { &P18437,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N8000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2024676 = {S24886, 61, { &P18437,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-N8000 Build/IMM76D)*/
const Device UA2183547 = {S24887, 139, { &P18437,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-N8000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2204049 = {S24888, 130, { &P18437,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-N8000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2224970 = {S24889, 132, { &P18437,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; GT-N8005 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2351433 = {S24890, 152, { &P18437,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SAMSUNG GT-N8000/N8000XWALG9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2525349 = {S24891, 130, { &P18437,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-N8000 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA29928 = {S24892, 142, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16*/
const Device UA29938 = {S24893, 141, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7C144 Safari/528.16*/
const Device UA62189 = {S24894, 142, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7D11 Safari/528.16*/
const Device UA88729 = {S24895, 142, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7C116a Safari/528.16*/
const Device UA89125 = {S24896, 143, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7C145 Safari/528.16*/
const Device UA117957 = {S24897, 142, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (device; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16*/
const Device UA305254 = {S24898, 143, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/9Z999 Safari/528.16*/
const Device UA372027 = {S24899, 139, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/PV02 Safari/528.16*/
const Device UA396961 = {S24900, 152, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16*/
const Device UA513137 = {S24901, 147, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA575206 = {S24902, 149, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16*/
const Device UA701583 = {S24903, 152, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; de-de) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16*/
const Device UA716962 = {S24904, 142, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (device; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7D11 Safari/528.16*/
const Device UA726261 = {S24905, 152, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; it-it) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16*/
const Device UA745695 = {S24906, 152, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; es-es) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16*/
const Device UA766163 = {S24907, 141, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7C97d Safari/528.16*/
const Device UA845189 = {S24908, 143, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/9Z999 Safari/528.16*/
const Device UA1853694 = {S24909, 139, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/PV01 Safari/528.16*/
const Device UA1938183 = {S24910, 152, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; fr-fr) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16*/
const Device UA1939472 = {S24911, 152, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; nl-nl) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16*/
const Device UA2149891 = {S24912, 152, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; ru-ru) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16*/
const Device UA2265865 = {S24913, 142, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7D11 Safari/528.16*/
const Device UA2662877 = {S24914, 139, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 312 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML like Gecko) Version/4.0 Mobile/7D11 Safari/528.16*/
const Device UA2701312 = {S24915, 139, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 313 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML like Gecko) Version/4.0 Mobile/7E18 Safari/528.16*/
const Device UA2810919 = {S24916, 145, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (heaveniphone; U; CPU iPhone OS 3_1_3 like Mac OS X; en) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16*/
const Device UA3042115 = {S24917, 149, { &P12280,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; ar) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16*/
const Device UA612892 = {S24918, 158, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-tw; HTC_SensationXL_Beats_X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA683959 = {S24919, 92, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Sensation XL with Beats Audio X315e Build/IML74K)*/
const Device UA842188 = {S24920, 154, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SensationXL_Beats_X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA875437 = {S24921, 170, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA896220 = {S24922, 158, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC_SensationXL_Beats_X315b Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA896698 = {S24923, 170, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC Sensation XL with Beats Audio X315b Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899665 = {S24924, 163, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC/SensationXL_Beats/2.32.161.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA903172 = {S24925, 163, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_SensationXL_Beats/2.31.111.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1983240 = {S24926, 162, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC_SensationXL_Beats-orange-LS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996417 = {S24927, 163, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC_SensationXL_Beats/2.36.111.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025024 = {S24928, 160, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; Sensation XL with Beats Audio Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026558 = {S24929, 161, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XL with Beats Audio X315e Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2026737 = {S24930, 140, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Sensation XL Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029727 = {S24931, 161, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC Sensation XL with Beats Audio X315b Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2030787 = {S24932, 92, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Sensation XL with Beats Audio X315b Build/IML74K)*/
const Device UA2034376 = {S24933, 156, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XL with Beats Audio X315e Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2171376 = {S24934, 149, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_DesireHD_X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2227331 = {S24935, 136, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2340264 = {S24936, 196, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation XL with Beats Audio X315e Build/Vertigo's Total Senseless v4.1.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2400129 = {S24937, 154, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_TelusHD_Beats_X315e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2413157 = {S24938, 140, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC_X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2496013 = {S24939, 156, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; HTC_SensationXL_Beats_X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2519098 = {S24940, 152, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; SensationXL_Beats_X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2693238 = {S24941, 154, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SensationXL_Beats_X315e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2813898 = {S24942, 161, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XL with Beats Audio X315e Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2817331 = {S24943, 168, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950189 = {S24944, 140, { &P15794,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3111121 = {S24945, 102, { &P15364,&P18110,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3115678 = {S24946, 104, { &P15364,&P18110,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3132827 = {S24947, 111, { &P15364,&P18110,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA2769463 = {S24948, 142, { &P12280,&P19826,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B350 Safari/8536.25*/
const Device UA2769668 = {S24949, 142, { &P12280,&P19826,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B145 Safari/8536.25*/
const Device UA2770263 = {S24950, 142, { &P12280,&P19826,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B329 Safari/8536.25*/
const Device UA2771206 = {S24951, 142, { &P12280,&P19826,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B146 Safari/8536.25*/
const Device UA2772837 = {S24952, 140, { &P12280,&P19826,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B143 Safari/8536.25*/
const Device UA2772845 = {S24953, 140, { &P12280,&P19826,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B142 Safari/8536.25*/
const Device UA2773048 = {S24954, 140, { &P12280,&P19826,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B144 Safari/8536.25*/
const Device UA2773063 = {S24955, 140, { &P12280,&P19826,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B141 Safari/8536.25*/
const Device UA2937721 = {S24956, 141, { &P12280,&P19826,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9Z999 Safari/8536.25*/
const Device UA249006 = {S24957, 142, { &P16436,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA575337 = {S24958, 66, { &P16436,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-I8150 Build/GINGERBREAD)*/
const Device UA622994 = {S24959, 142, { &P16436,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626796 = {S24960, 143, { &P16436,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-I8150T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA638865 = {S24961, 66, { &P16436,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I8150 Build/GINGERBREAD)*/
const Device UA653101 = {S24962, 143, { &P16436,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; GT-I8150B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA660007 = {S24963, 130, { &P16436,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA730340 = {S24964, 143, { &P16436,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-I8150B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1957700 = {S24965, 135, { &P16436,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2033714 = {S24966, 67, { &P16436,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I8150T Build/GINGERBREAD)*/
const Device UA2033922 = {S24967, 135, { &P16436,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I8150 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2218844 = {S24968, 140, { &P16436,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2518826 = {S24969, 135, { &P16436,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2380718 = {S24970, 132, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2480083 = {S24971, 61, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P5100 Build/JRO03C)*/
const Device UA2487853 = {S24972, 152, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; SAMSUNG GT-P5100/P5100BUCLL1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2595596 = {S24973, 132, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P5100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2597780 = {S24974, 132, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T779 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615389 = {S24975, 132, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5100 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616134 = {S24976, 132, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5100 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616622 = {S24977, 61, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P5100 Build/JZO54K)*/
const Device UA2616868 = {S24978, 139, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; GT-P5100 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2617156 = {S24979, 152, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; SAMSUNG GT-P5100/P5100BUCLJ1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2623067 = {S24980, 132, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; GT-P5100 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2625843 = {S24981, 130, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P5100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2625916 = {S24982, 94, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.2.10 (Linux; U; Android 4.1.1; en-ZA; GT-P5100 Build/JRO03C.P5100ZSCLL1)*/
const Device UA2626671 = {S24983, 93, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - com.google.android.youtube/4.1.47(Linux; U; Android 4.1.1; en_GB; GT-P5100 Build/JRO03C) gzip*/
const Device UA2627779 = {S24984, 130, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-P5100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2630104 = {S24985, 139, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2632112 = {S24986, 162, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5100 Build/JRO03C) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2632831 = {S24987, 152, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; sl-si; SAMSUNG GT-P5100/P5100BUCLJ2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2633244 = {S24988, 130, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P5100 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2634421 = {S24989, 151, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5100 Build/JRO03C) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.2 Version/4.0 Mobile Safari/530.17*/
const Device UA2635578 = {S24990, 145, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; GT-P5100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30*/
const Device UA2636372 = {S24991, 155, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-P5100/P5100BUDMA2.DM Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2636700 = {S24992, 59, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - nApps (Android 4.1.1; GT-P5100; androidapp.lineplay; 1.2.2)*/
const Device UA2636826 = {S24993, 134, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5100 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2770358 = {S24994, 152, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; SAMSUNG GT-P5100/P5100BUDMD1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2879070 = {S24995, 151, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-P5100-ORANGE/P5100BVDMD1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2936071 = {S24996, 61, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T779 Build/JZO54K)*/
const Device UA3027014 = {S24997, 151, { &P15876,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-P5100-ORANGE/P5100BVDMF1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA368853 = {S24998, 132, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P7500 Build/HTJ85B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA605609 = {S24999, 132, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA775084 = {S25000, 132, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P7500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1971155 = {S25001, 132, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1997100 = {S25002, 133, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2030210 = {S25003, 61, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7500 Build/IMM76D)*/
const Device UA2030244 = {S25004, 133, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2031724 = {S25005, 133, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2230037 = {S25006, 62, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7500R Build/IMM76D)*/
const Device UA2230699 = {S25007, 130, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2236855 = {S25008, 151, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; SAMSUNG GT-P7500/P7500BULP5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2423437 = {S25009, 151, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SAMSUNG GT-P7500/P7500XWLPB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2608090 = {S25010, 61, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-P7500 Build/HTJ85B)*/
const Device UA2609551 = {S25011, 133, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; GT-P7500R Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2609564 = {S25012, 133, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; GT-P7500D Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2609798 = {S25013, 151, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SAMSUNG GT-P7500/P7500BULP1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2613139 = {S25014, 153, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7500 Build/IMM76D; 1280*752) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2613884 = {S25015, 62, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7500M Build/IMM76D)*/
const Device UA2614029 = {S25016, 140, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2614368 = {S25017, 62, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7500D Build/IMM76D)*/
const Device UA2614642 = {S25018, 132, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2614913 = {S25019, 145, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-P7500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.5 Mobile Safari/534.30*/
const Device UA2614916 = {S25020, 139, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2615148 = {S25021, 75, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.0.4; en-CA; GT-P7500R Build/IMM76D.RUXLH2)*/
const Device UA2615278 = {S25022, 131, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/IMM76) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615518 = {S25023, 145, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA2615812 = {S25024, 139, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2615991 = {S25025, 146, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA2615999 = {S25026, 134, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P7500 Build/TISU0093) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616116 = {S25027, 131, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-; GT-P7500R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616171 = {S25028, 143, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA2616240 = {S25029, 73, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Vyrso/2.1.2 (Linux; U; Android 4.0.4; zh-CN; GT-P7500 Build/IMM76D.ZSLP8)*/
const Device UA2616309 = {S25030, 212, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.2.1-65 (Android 4.0.4; samsung GT-P7500 Build/IMM76D.XXLQ8)*/
const Device UA2616993 = {S25031, 151, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sr-rs; SAMSUNG GT-P7500/P7500XWLP5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617154 = {S25032, 73, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.0.4; en-US; GT-P7500 Build/IMM76D.XWLP5)*/
const Device UA2617471 = {S25033, 93, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.youtube/4.0.23(Linux; U; Android 4.0.4; en_GB; GT-P7500 Build/IMM76D) gzip*/
const Device UA2617628 = {S25034, 132, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500) antivirusHttpClient/2.0 (KHTML, like Gecko) Version/Version 2.3/Antivirus pro*/
const Device UA2617654 = {S25035, 52, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - nApps (Android 4.0.4; GT-P7500; Android.Line; 3.1.3)*/
const Device UA2617663 = {S25036, 73, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.0.4; nl-NL; GT-P7500 Build/IMM76D.XWLP6)*/
const Device UA2617817 = {S25037, 151, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SAMSUNG GT-P7500/P7500BULP3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617890 = {S25038, 73, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.4; en-GB; GT-P7500 Build/IMM76D.XXLQ8)*/
const Device UA2618087 = {S25039, 140, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2618135 = {S25040, 103, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 4.0.4; SAMSUNG GT-P7500 Build/ICE_CREAM_SANDWICH_MR1);*/
const Device UA2618601 = {S25041, 150, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P7500-ORANGE/P7500BVLP6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2618891 = {S25042, 76, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - GoogleAnalytics/2.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500R Build/IMM76D)*/
const Device UA2618893 = {S25043, 75, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - GoogleAnalytics/2.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/IML74K)*/
const Device UA2618908 = {S25044, 152, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500R Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MantanoReader*/
const Device UA2618956 = {S25045, 76, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - GoogleAnalytics/2.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500D Build/IMM76D)*/
const Device UA2619059 = {S25046, 73, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.0.4; en-US; GT-P7500 Build/IMM76D.JPLPC)*/
const Device UA2619661 = {S25047, 95, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.apps.plus/X (Linux; U; Android 4.0.4; en_CA; GT-P7500R; Build/IMM76D) (gzip)*/
const Device UA2619798 = {S25048, 134, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2620047 = {S25049, 160, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P7500 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2620248 = {S25050, 151, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; SAMSUNG GT-P7500/P7500XWLP6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2620404 = {S25051, 75, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.4; en-CA; GT-P7500D Build/IMM76D.DTLLH1)*/
const Device UA2620473 = {S25052, 104, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 4.0.4; SAMSUNG GT-P7500R Build/ICE_CREAM_SANDWICH_MR1);*/
const Device UA2620474 = {S25053, 149, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG GT-P7500/P7500BULP5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2620497 = {S25054, 95, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.apps.plus/X (Linux; U; Android 4.0.4; en_CA; GT-P7500D; Build/IMM76D) (gzip)*/
const Device UA2620563 = {S25055, 137, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2620990 = {S25056, 139, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; GT-P7500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2621669 = {S25057, 162, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2621809 = {S25058, 155, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 4G Build/6.7.2-223_DBN_M4-23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2621901 = {S25059, 78, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - MediaPlayer/2002/4.1.0 (Android 4.0.4; GT-P7500; GT-P7500; samsung; ; ; en_GB)*/
const Device UA2622363 = {S25060, 132, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; GT-P7500 Build/HTJ85B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622453 = {S25061, 52, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Scanner Radio Pro/3.9.0.2 (Android 4.0.4, GT-P7500D)*/
const Device UA2622503 = {S25062, 134, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; GT-P7500 Build/TISU0124) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622564 = {S25063, 140, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2622909 = {S25064, 145, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500R Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2623000 = {S25065, 75, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-GB; GT-P7500 Build/IMM76D.XWLP9)*/
const Device UA2623527 = {S25066, 49, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Hotmail/7.8.2.8 (Android 4.0.4; samsungGT-P7500R)*/
const Device UA2623976 = {S25067, 85, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik 2.02.11978; (Linux; U; Android 4.0.4; GT-P7500) 3efd9aa46fac3720; 1280x752x1.0*/
const Device UA2624038 = {S25068, 161, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P7500R Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2624143 = {S25069, 50, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Scanner Radio Pro/3.9.1 (Android 4.0.4, GT-P7500D)*/
const Device UA2624242 = {S25070, 150, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P7500-ORANGE/P7500BVLP8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2624251 = {S25071, 89, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.2.8.1 (Linux; U; Android 4.0.4; en-US; GT-P7500 Build/IMM76D.DXLP9)*/
const Device UA2624756 = {S25072, 150, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500-ORANGE/P7500BVLP3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2625258 = {S25073, 135, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P7500 Build/R13.4-rc2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2625294 = {S25074, 76, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.10 (Linux; U; Android 4.0.4; en-US; GT-P7500M Build/IMM76D.MUGLH4)*/
const Device UA2625462 = {S25075, 57, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Android/3.7.3 (Linux; U; Android 4.0.4; en_CA; GT-P7500R)*/
const Device UA2626058 = {S25076, 143, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P7500 wI-fI & 3g Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2626077 = {S25077, 129, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P7500 Build/Xqm) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2626945 = {S25078, 125, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2627126 = {S25079, 55, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Android/4.7 (Linux; U; Android 4.0.4; en_CA; GT-P7500M)*/
const Device UA2628012 = {S25080, 151, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-P7500 Build/IMM76D) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.0 Version/4.0 Mobile Safari/530.17*/
const Device UA2628390 = {S25081, 94, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.youtube/4.1.47(Linux; U; Android 4.0.4; en_CA; GT-P7500M Build/IMM76D) gzip*/
const Device UA2628449 = {S25082, 130, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7500 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2628688 = {S25083, 102, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) Appleko) Version/4.0 Safari/534.30*/
const Device UA2630735 = {S25084, 140, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P7500 WIFI_3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2630833 = {S25085, 130, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7500 Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2632046 = {S25086, 285, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ms-my; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/2.2;FBBV/X;FBDM/{density=1.0,width=1280,height=752};FBLC/ms_MY;FB_FW/2;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P7500;FBSV/4.0.4;]*/
const Device UA2632156 = {S25087, 134, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P7500 Build/TISU0143) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2632449 = {S25088, 158, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SAMSUNG GT-P7500/P7500BULP1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2632746 = {S25089, 104, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Android 4.0.4; en-ca; GT-P7500R Build/IMM76D) FreeWheelAdManager/5.3.0-r9362-X;com.vevo/442*/
const Device UA2632777 = {S25090, 63, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.1; 9) GT-P7500M (Android 4.0.4, SDK 15)*/
const Device UA2633254 = {S25091, 154, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG GT-P7500/P7500BULP5.DM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2633738 = {S25092, 152, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500M Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MantanoReader*/
const Device UA2633857 = {S25093, 80, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - VKAndroidApp/2.7.6-64 (Android 4.0.4; SDK 15; armeabi-v7a; samsung GT-P7500; ru)*/
const Device UA2635351 = {S25094, 145, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500D Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2637182 = {S25095, 70, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - APKXDL (Linux; U; Android 4.0.4;en_CA; GT-P7500R/IMM76D)com.sygic.aura*/
const Device UA2637355 = {S25096, 128, { &P17337,&P18066,&P18062,&P18092 }};	/*Useragent - Groupon/2.3.2787 (Android 4.0.4; Samsung GT-P7500 / Samsung GT-P7500; )[preload=false;locale=en_US;clientidbase=android-samsung]*/
const Device UA874001 = {S25097, 132, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1951244 = {S25098, 61, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-P3110 Build/IML74K)*/
const Device UA2000167 = {S25099, 132, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2189292 = {S25100, 61, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P3110 Build/IMM76D)*/
const Device UA2227396 = {S25101, 130, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2367028 = {S25102, 130, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2608368 = {S25103, 130, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3110 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2611276 = {S25104, 152, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3110 Build/IML74K; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2611349 = {S25105, 69, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - BBC iPlayer (AndroidApp; 1.3; 13014) GT-P3110 (Android 4.0.3, SDK 15)*/
const Device UA2613477 = {S25106, 145, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2614483 = {S25107, 65, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.3.3; 15) GT-P3110 (Android 4.0.4, SDK 15)*/
const Device UA2614969 = {S25108, 69, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - BBC iPlayer (AndroidApp; 1.3; 13014) GT-P3110 (Android 4.0.4, SDK 15)*/
const Device UA2615634 = {S25109, 139, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2615807 = {S25110, 65, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.3.3; 15) GT-P3110 (Android 4.0.3, SDK 15)*/
const Device UA2615885 = {S25111, 145, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30*/
const Device UA2616562 = {S25112, 93, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.1.2 (Linux; U; Android 4.0.3; en-PH; GT-P3110 Build/IML74K.P3110XXALD4)*/
const Device UA2616988 = {S25113, 218, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.2.1-65 (Android 4.0.4; samsung GT-P3110 Build/IMM76D.P3110XXBLH2)*/
const Device UA2617102 = {S25114, 212, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3110C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; 360 Aphone Browser (Version 2.6.3); 360browser(securitypay,securityinstalled)*/
const Device UA2617421 = {S25115, 79, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.3; en-AU; GT-P3110 Build/IML74K.P3110ZSALD5)*/
const Device UA2617725 = {S25116, 79, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.3; en-AU; GT-P3110 Build/IML74K.P3110XWALD2)*/
const Device UA2618093 = {S25117, 93, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.2.3 (Linux; U; Android 4.0.4; en-GB; GT-P3110 Build/IMM76D.P3110XXBLH2)*/
const Device UA2618975 = {S25118, 162, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ph; GT-P3110 Build/IML74K) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2620328 = {S25119, 152, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3110 Build/IMM76D; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2621053 = {S25120, 132, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM7ategoryId":"Search","Click":0,"Carted":0,"Purchased":0,"ProfileC4.30*/
const Device UA2621498 = {S25121, 260, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3110 Build/IMM76D) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 baidubrowser/061_1.6.3.2_diordna_679_006/gnusmas_51_4.0.4_0113P-TG/Xa/7C3794EBF80E2CB90F3BA3BA4B6BD2CD%7C0/1*/
const Device UA2622000 = {S25122, 130, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3110 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2622335 = {S25123, 125, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3110 Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2622354 = {S25124, 139, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2622424 = {S25125, 92, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.youtube/3.5.5(Linux; U; Android 4.0.3; en_GB; GT-P3110 Build/IML74K) gzip*/
const Device UA2622553 = {S25126, 218, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.3.4-94 (Android 4.0.3; samsung GT-P3110 Build/IML74K.P3110XXALD4)*/
const Device UA2623547 = {S25127, 278, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 parameters/{scr=552_1024,cm=1,ql=l,imei=94:63:D1:FD:C1:AA,bm=1,eng=adrmini15,mdm=1,mdl=GT-P3110,ps=1} tiantian(securitypay) tiantian(tenpay)*/
const Device UA2623604 = {S25128, 211, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, likX.X.X.X.1; __utmb=X.4.10.X; __utmc=X; __utmz=X.X.1.1.utmcsr=google|utmccn=(organic)|utmcmd=organic|utmctr=harveys*/
const Device UA2623944 = {S25129, 93, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.youtube/4.0.23(Linux; U; Android 4.0.4; en_GB; GT-P3110 Build/IMM76D) gzip*/
const Device UA2624882 = {S25130, 80, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - VKAndroidApp/2.7.6-64 (Android 4.0.4; SDK 15; armeabi-v7a; samsung GT-P3110; ru)*/
const Device UA2624954 = {S25131, 163, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/5.0.2 Version/4.0 Safari/534.30 Safari/6533.18.5*/
const Device UA2625515 = {S25132, 71, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - BBC iPlayer (AndroidApp; 1.5.0.666; X) GT-P3110 (Android 4.0.4, SDK 15)*/
const Device UA2625677 = {S25133, 137, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2627289 = {S25134, 102, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; GT-P3110 Build/IMM76D) Appecko) Version/4.0 Safari/534.30*/
const Device UA2629288 = {S25135, 144, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fi-fi; GT-P3110 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2629782 = {S25136, 71, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - BBC iPlayer (AndroidApp; 1.5.0.666; X) GT-P3110 (Android 4.0.3, SDK 15)*/
const Device UA2634489 = {S25137, 132, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IML76D) appleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2637814 = {S25138, 132, { &P15932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1995245 = {S25139, 84, { &P15364,&P18118,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:14.0) Gecko/X Firefox/14.0.1 Firefox/14.0.1*/
const Device UA2260102 = {S25140, 80, { &P15364,&P18118,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:14.0; ips-agent) Gecko/X Firefox/14.0.1*/
const Device UA2361722 = {S25141, 73, { &P15364,&P7133,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA2680762 = {S25142, 88, { &P15364,&P7133,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:20.0) Gecko/X Firefox/20.0 AlexaToolbar/alxf-2.18*/
const Device UA2751192 = {S25143, 82, { &P15364,&P7133,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.1; Win64; x64; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA2756668 = {S25144, 79, { &P15364,&P7133,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.1*/
const Device UA2756691 = {S25145, 88, { &P15364,&P7133,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.1-x64*/
const Device UA2861105 = {S25146, 70, { &P15364,&P7133,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.1; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA2993391 = {S25147, 49, { &P15364,&P7133,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) Gecko/X Firefox/20.0*/
const Device UA30679 = {S25148, 88, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; MAXTHON 2.0)*/
const Device UA95620 = {S25149, 160, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; MAXTHON 2.0)*/
const Device UA100486 = {S25150, 63, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Maxthon 2.0)*/
const Device UA122427 = {S25151, 83, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; MAXTHON 2.0)*/
const Device UA135688 = {S25152, 87, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; Maxthon 2.0)*/
const Device UA259545 = {S25153, 146, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; Maxthon 2.0)*/
const Device UA260539 = {S25154, 126, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; MAXTHON 2.0)*/
const Device UA279086 = {S25155, 154, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; Maxthon 2.0)*/
const Device UA285185 = {S25156, 96, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; MAXTHON 2.0)*/
const Device UA285834 = {S25157, 76, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Maxthon 2.0)*/
const Device UA289031 = {S25158, 107, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; MAXTHON 2.0)*/
const Device UA416557 = {S25159, 76, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; MAXTHON 2.0)*/
const Device UA442653 = {S25160, 105, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Maxthon; .NET CLR 2.0.50727; Maxthon 2.0)*/
const Device UA450803 = {S25161, 137, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; Trident/4.0; Media Center PC 4.0; SLCC1; .NET CLR 3.0.04320; Maxthon 2.0)*/
const Device UA484953 = {S25162, 98, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Avant Browser; .NET CLR 2.0.50727; MAXTHON 2.0)*/
const Device UA496902 = {S25163, 159, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Maxthon 2.0)*/
const Device UA551102 = {S25164, 173, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; MAXTHON 2.0)*/
const Device UA624244 = {S25165, 159, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MAXTHON 2.0)*/
const Device UA659856 = {S25166, 149, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Maxthon; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MAXTHON 2.0)*/
const Device UA675101 = {S25167, 87, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; MAXTHON 2.0)*/
const Device UA716510 = {S25168, 174, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Maxthon 2; MAXTHON 2.0)*/
const Device UA730133 = {S25169, 134, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Maxthon 2.0)*/
const Device UA804182 = {S25170, 77, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Maxthon; MAXTHON 2.0)*/
const Device UA1887577 = {S25171, 97, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Maxthon; .NET CLR 2.0.50727; Maxthon 2.0)*/
const Device UA1937851 = {S25172, 81, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; Maxthon 2.0)*/
const Device UA2266700 = {S25173, 146, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MAXTHON 2.0)*/
const Device UA2272833 = {S25174, 135, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; Maxthon 2.0)*/
const Device UA2306926 = {S25175, 145, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MAXTHON 2.0)*/
const Device UA2801397 = {S25176, 156, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; IEMB3; IEMB3; MAXTHON 2.0)*/
const Device UA2948940 = {S25177, 81, { &P15364,&P5690,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; SV1; Maxthon 2.0)*/
const Device UA2769907 = {S25178, 127, { &P15364,&P17285,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/28.0.1500.29) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.29 Safari/537.36*/
const Device UA2877378 = {S25179, 127, { &P15364,&P17285,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/28.0.1500.52) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36*/
const Device UA2888698 = {S25180, 127, { &P15364,&P17285,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/28.0.1500.63) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36*/
const Device UA2898716 = {S25181, 127, { &P15364,&P17285,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/28.0.1500.71) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36*/
const Device UA2945461 = {S25182, 134, { &P15364,&P17285,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/28.0.1500.71) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36*/
const Device UA2947878 = {S25183, 134, { &P15364,&P17285,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/28.0.1500.72) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36*/
const Device UA2947887 = {S25184, 127, { &P15364,&P17285,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/28.0.1500.72) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36*/
const Device UA3025188 = {S25185, 89, { &P15364,&P17285,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/28.0.1500.95)*/
const Device UA3025322 = {S25186, 202, { &P15364,&P17285,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.5; chromeframe/28.0.1500.95; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA3026086 = {S25187, 134, { &P15364,&P17285,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/28.0.1500.95) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3026098 = {S25188, 96, { &P15364,&P17285,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/28.0.1500.95)*/
const Device UA3026389 = {S25189, 127, { &P15364,&P17285,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/28.0.1500.95) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3027211 = {S25190, 164, { &P15364,&P17285,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/28.0.1500.95; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)*/
const Device UA3028415 = {S25191, 229, { &P15364,&P17285,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/28.0.1500.95; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C)*/
const Device UA3029155 = {S25192, 161, { &P15364,&P17285,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.5; chromeframe/28.0.1500.95; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA3029269 = {S25193, 195, { &P15364,&P17285,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.5; chromeframe/28.0.1500.95; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA3029517 = {S25194, 189, { &P15364,&P17285,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.5; chromeframe/28.0.1500.95; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA3029886 = {S25195, 219, { &P15364,&P17285,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.5; chromeframe/28.0.1500.95; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA3058614 = {S25196, 210, { &P15364,&P17285,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.5; chromeframe/28.0.1500.95; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2401947 = {S25197, 132, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2420752 = {S25198, 61, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P5110 Build/JRO03C)*/
const Device UA2459883 = {S25199, 130, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2594697 = {S25200, 61, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P5110 Build/JZO54K)*/
const Device UA2597291 = {S25201, 139, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2615587 = {S25202, 132, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5110 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617029 = {S25203, 132, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-P5110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617707 = {S25204, 139, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-P5110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2618555 = {S25205, 132, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5110 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2621988 = {S25206, 134, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5110 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2622946 = {S25207, 145, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2623591 = {S25208, 93, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - com.google.android.youtube/4.1.23(Linux; U; Android 4.1.1; en_GB; GT-P5110 Build/JRO03C) gzip*/
const Device UA2624049 = {S25209, 130, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P5110 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2624726 = {S25210, 80, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.2.10 (Linux; U; Android 4.1.1; en-GB; GT-P5110 Build/JRO03C.P5110XXCLK7)*/
const Device UA2625490 = {S25211, 223, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P5110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 sessionId/c36b5d49-0d14-4b82-84d9-6b0a5f965fb8 tiantian(securitypay) tiantian(tenpay)*/
const Device UA2626187 = {S25212, 94, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - com.google.android.apps.plus/X (Linux; U; Android 4.1.1; fr_FR; GT-P5110; Build/JRO03C) (gzip)*/
const Device UA2627717 = {S25213, 257, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P5110 Build/JRO03C) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 baidubrowser/061_1.41.3.2_diordna_2321_008/gnusmas_61_1.1.4_0115P-TG/Xk/B0AAF0DDXED091EBB86749AA4CD%7C0/1*/
const Device UA2627812 = {S25214, 476, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/5%2C%22landing%22%3A%22http%3A//www.harveysfurniture.co.uk/mobile/home.aspx%22%2C%22time%22%3AX%7D%2C%7B%22url%22%3A%22direct%22%2C%22landing%22%3A%22http%3A//www.harveysfurniture.co.uk/mobile/error.aspx%3Faspxerrorpath%3D%252fmobile%252fcategory.aspx%22%2C%22time%22%3AX%7D%5D%2C%22sessionLandingPage%22%3A%22http%3A//www.harveysfurniture.co.uk/mobile/error.aspx%3Faspxerrorpath%3D%252fmobile%25*/
const Device UA2628175 = {S25215, 216, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Ve2C%22sessionLandingPage%22%3A%22http%3A//www.harveysfurniturept-Charset: utf-8, iso-8859-1, utf-16, *;q=0.7*/
const Device UA2628385 = {S25216, 132, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P511GT-P5110 Build/JRO03C) AppleWebK30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2628587 = {S25217, 144, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2629786 = {S25218, 349, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 BuiPC=id=Xa7d1b1aa785dX:lv=X:ss=X; _qubitTracker=X.X; _qPageNum_harveysfurniture=7; _qst=%5B1%2C0%5D; _qsst=X; qb_etc_status=%7B%22c%22%3A%7B%7D%7D; __utma=X.X.X.X.X.1; __utmb=X.8.10.X; __utmc=X; __utmz=X.X.1.1.utmgclid=cog4sav0l7ucffhjtaoddnqaog|utmccn=brand%20pure%20(pc)%20-%20exact|utmcmd=pp*/
const Device UA2630533 = {S25219, 235, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 VoiceGO!(cyb9bXa78aXf3d91aea03735e-0;Android,4.1.1,GT-P5110;3.8.0;1280x696,1.0;X;VoiceGo-TC;zh-TW;X;X)*/
const Device UA2631468 = {S25220, 54, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Android/4.8 (Linux; U; Android 4.1.1; en_GB; GT-P5110)*/
const Device UA2633652 = {S25221, 132, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; GT-P5110 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2634856 = {S25222, 130, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P5110 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2635496 = {S25223, 242, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P5110 Build/JZO54K) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.4 Version/4.0 Mobile Safari/530.17 baidubrowser/061_1.8.4.2_diordna_2321_008/gnusmas_61_2.1.4_0115P-TG/Xz/BBXDB6CC5BXE8%7C0/1*/
const Device UA2635521 = {S25224, 63, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - BBC News (AndroidApp; 2.1; 33) GT-P5110 (Android 4.1.1, SDK 16)*/
const Device UA2635543 = {S25225, 153, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P5110 Build/JZO54K; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2635681 = {S25226, 130, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-P5110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2636855 = {S25227, 483, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWe52fcategory.aspx%22%2C%22timko) Version/4.0 4566%7D%2C%7B%22url%22%3A%22direct%22%2C%22landing%22%3A%22http%3A//www.harveysfurniture.co.uk/mobile/error.aspx%3Faspxerrorpath%3D%252fmobile%252fcategory.aspx%22%2C%22time%22%3AX%7D%5D%2C%22sessionLandingPage%22%3A%22http%3A//www.harveysfurniture.co.uk/mobile/error.aspx%3Faspxerrorpath%3D%252fmobile%252fcategory.aspx%22%2C%22__v%22%3A%7B%7D%7D; _qubitTracker=X*/
const Device UA2636863 = {S25228, 132, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO0 en-gb; GT-P5110 Build/JRO03C) Aike Gecko) Version/4.0 Safari/534.30*/
const Device UA2637668 = {S25229, 125, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Bu33655; __utmc=X; __utmz=7KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2637996 = {S25230, 62, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.0; 5) GT-P5110 (Android 4.1.2, SDK 16)*/
const Device UA2638003 = {S25231, 153, { &P15882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P5110 Build/JRO03C; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2223521 = {S25232, 139, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257717 = {S25233, 132, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2366528 = {S25234, 139, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2367390 = {S25235, 139, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3113 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455167 = {S25236, 61, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P3113 Build/JRO03C)*/
const Device UA2470131 = {S25237, 130, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2546755 = {S25238, 139, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2612307 = {S25239, 120, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.0.8 (Linux; U; Android 4.1.1; en-US; GT-P3113 Build/cm_p3113-userdebug 4.1.1 JRO03L eng.codeworkx.X.X test-keys)*/
const Device UA2614509 = {S25240, 139, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2615681 = {S25241, 145, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA2622766 = {S25242, 61, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P3113 Build/JZO54K)*/
const Device UA2623952 = {S25243, 139, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2625385 = {S25244, 80, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.2.10 (Linux; U; Android 4.1.1; en-US; GT-P3113 Build/JRO03C.P3113UECLK7)*/
const Device UA2625836 = {S25245, 218, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.3.4-94 (Android 4.1.1; samsung GT-P3113 Build/JRO03C.P3113UECLK7)*/
const Device UA2625906 = {S25246, 62, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.3; 4) GT-P3113 (Android 4.1.1, SDK 16)*/
const Device UA2625991 = {S25247, 163, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/5.0.2 Version/4.0 Safari/534.30 Safari/6533.18.5*/
const Device UA2626545 = {S25248, 62, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - nApps (Android 4.1.1; GT-P3113; line-antivirus-android; 1.0.6)*/
const Device UA2627026 = {S25249, 151, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.0 Version/4.0 Mobile Safari/530.17*/
const Device UA2627083 = {S25250, 93, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - com.google.android.youtube/4.1.23(Linux; U; Android 4.1.1; en_US; GT-P3113 Build/JRO03C) gzip*/
const Device UA2627239 = {S25251, 87, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 2.6); (Linux; U; Android 4.1.1; SAMSUNG GT-P3113 Build/JELLY_BEAN);*/
const Device UA2627350 = {S25252, 142, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; GT-P3113 Build/JRO03C like Xoom) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2627457 = {S25253, 130, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2627512 = {S25254, 49, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Scanner Radio Pro/3.9.2 (Android 4.1.1, GT-P3113)*/
const Device UA2628156 = {S25255, 94, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - com.google.android.apps.plus/X (Linux; U; Android 4.1.1; en_US; GT-P3113; Build/JRO03C) (gzip)*/
const Device UA2628427 = {S25256, 144, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2628605 = {S25257, 103, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Android 4.1.1; en-ca; GT-P3113 Build/JRO03C) FreeWheelAdManager/5.3.0-r9362-X;com.vevo/442*/
const Device UA2628701 = {S25258, 195, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 androidAppVersion/2.5.17 AndroidWebkitWrapper BillingSupported*/
const Device UA2629673 = {S25259, 89, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik 2.06.12230; (Linux; U; Android 4.1.1; GT-P3113; ON) 659c8688ccdde2dd; 1024x552x1.0*/
const Device UA2629718 = {S25260, 54, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Android/4.7 (Linux; U; Android 4.1.1; en_CA; GT-P3113)*/
const Device UA2630111 = {S25261, 137, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2631397 = {S25262, 85, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - TwonkyBeamBrowser/3.4.0-99 (Android 4.1.1; samsung GT-P3113 Build/JRO03C.P3113UECLK7)*/
const Device UA2631501 = {S25263, 131, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Groupon/2.2.2561 (Android 4.1.1; Samsung Espressowifi / Samsung GT-P3113; )[preload=true;locale=en_US;clientidbase=android-samsung]*/
const Device UA2631557 = {S25264, 82, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - MediaPlayer/2002/4.2.3 (Android 4.1.1; espressowifi; GT-P3113; samsung; ; ; en_CA)*/
const Device UA2632924 = {S25265, 49, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Scanner Radio Pro/3.8.3 (Android 4.1.1, GT-P3113)*/
const Device UA2633272 = {S25266, 151, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; GT-P3113 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MantanoReader*/
const Device UA2634177 = {S25267, 143, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en_CA; GT-P3113 BUILD/JRO03C) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2634846 = {S25268, 59, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - nApps (Android 4.1.1; GT-P3113; androidapp.lineplay; 1.2.1)*/
const Device UA2635191 = {S25269, 175, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; GT-P3113 Build/JRO03C) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 GoogleGoggles-Android/1.9; gzip*/
const Device UA2635212 = {S25270, 179, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P3113 Build/JRO03C) [FBAN/Orca-Android;FBAV/2.3.2-release;FBLC/en_US;FBBV/X;FBCR/null;FBMF/samsung;FBBD/samsung;FBDV/GT-P3113;FBSV/4.1.1]*/
const Device UA2635327 = {S25271, 124, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2635466 = {S25272, 63, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - BBC News (AndroidApp; 2.2; 26) GT-P3113 (Android 4.1.1, SDK 16)*/
const Device UA2636028 = {S25273, 59, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - nApps (Android 4.1.1; GT-P3113; androidapp.lineplay; 1.2.3)*/
const Device UA2636217 = {S25274, 137, { &P22037,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1953383 = {S25275, 74, { &P15364,&P6704,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; MSN Optimized)*/
const Device UA1953780 = {S25276, 64, { &P15364,&P6704,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; SV1)*/
const Device UA1958812 = {S25277, 64, { &P15364,&P6704,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; SV1)*/
const Device UA1992509 = {S25278, 72, { &P15364,&P6704,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; MS-RTC LM 8)*/
const Device UA2220660 = {S25279, 69, { &P15364,&P6704,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; SV1; SV1)*/
const Device UA2248159 = {S25280, 182, { &P15364,&P6704,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows 98; Win 9x 4.90; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)*/
const Device UA2334989 = {S25281, 69, { &P15364,&P6704,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; SV1; SV1)*/
const Device UA2658939 = {S25282, 54, { &P15364,&P6704,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Win98; Trident/5.0)*/
const Device UA2850904 = {S25283, 60, { &P15364,&P6704,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Win 9x 4.90; Trident/5.0)*/
const Device UA2389741 = {S25284, 127, { &P15364,&P17285,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/24.0.1312.14) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.14 Safari/537.17*/
const Device UA2459890 = {S25285, 127, { &P15364,&P17285,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/24.0.1312.45) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.45 Safari/537.17*/
const Device UA2462323 = {S25286, 127, { &P15364,&P17285,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/24.0.1312.52) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17*/
const Device UA2469541 = {S25287, 134, { &P15364,&P17285,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/24.0.1312.52) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17*/
const Device UA2487604 = {S25288, 134, { &P15364,&P17285,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/24.0.1312.57) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2488089 = {S25289, 161, { &P15364,&P17285,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/24.0.1312.57; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2488144 = {S25290, 127, { &P15364,&P17285,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/24.0.1312.56) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17*/
const Device UA2489333 = {S25291, 134, { &P15364,&P17285,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/24.0.1312.56) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17*/
const Device UA2490848 = {S25292, 174, { &P15364,&P17285,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; chromeframe/24.0.1312.57; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2492184 = {S25293, 89, { &P15364,&P17285,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/24.0.1312.57)*/
const Device UA2492404 = {S25294, 182, { &P15364,&P17285,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/24.0.1312.57; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2492460 = {S25295, 127, { &P15364,&P17285,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/24.0.1312.57) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2492555 = {S25296, 96, { &P15364,&P17285,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/24.0.1312.57)*/
const Device UA2492547 = {S25297, 195, { &P15364,&P17285,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; chromeframe/24.0.1312.57; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2510086 = {S25298, 202, { &P15364,&P17285,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.4; chromeframe/24.0.1312.57; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2511093 = {S25299, 189, { &P15364,&P17285,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; chromeframe/24.0.1312.57; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2511493 = {S25300, 210, { &P15364,&P17285,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; chromeframe/24.0.1312.57; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2537826 = {S25301, 240, { &P15364,&P17285,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/24.0.1312.57; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2362445 = {S25302, 132, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2384080 = {S25303, 132, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2421892 = {S25304, 132, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P5113 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2454027 = {S25305, 61, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P5113 Build/JRO03C)*/
const Device UA2469497 = {S25306, 130, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2553165 = {S25307, 139, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2613144 = {S25308, 139, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2615847 = {S25309, 132, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616182 = {S25310, 125, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.1.2 (Linux; U; Android 4.1.1; en-US; GT-P5113 Build/cm_p5113-userdebug 4.1.1 JRO03R eng..X.X test-keys)*/
const Device UA2618346 = {S25311, 111, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.1.2; en-US; GT-P5113 Build/cm_p5113-userdebug 4.1.2 JZO54K eng..X.X test-keys)*/
const Device UA2618756 = {S25312, 61, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P5113 Build/JZO54K)*/
const Device UA2621262 = {S25313, 132, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2623627 = {S25314, 81, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.1.1; en-US; GT-P5113 Build/JRO03C.P5113UECLK7)*/
const Device UA2625020 = {S25315, 134, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Groupon/2.1.2486 (Android 4.1.1; Samsung Espresso10wifi / Samsung GT-P5113; )[preload=false;locale=en_CA;clientidbase=android-samsung]*/
const Device UA2625378 = {S25316, 94, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - com.google.android.apps.plus/X (Linux; U; Android 4.1.1; en_CA; GT-P5113; Build/JRO03C) (gzip)*/
const Device UA2625987 = {S25317, 62, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.1; 9) GT-P5113 (Android 4.1.1, SDK 16)*/
const Device UA2626046 = {S25318, 89, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik 2.06.12230; (Linux; U; Android 4.1.1; GT-P5113; ON) 6df6e536c10c2ec6; 1280x752x1.0*/
const Device UA2626446 = {S25319, 117, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Android 4.1.1; en-za; GT-P5113 Build/JRO03C) FreeWheelAdManager/5.2.3-r9361-X;com.gotv.crackle.handset/X*/
const Device UA2626612 = {S25320, 54, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Android/4.7 (Linux; U; Android 4.1.1; en_CA; GT-P5113)*/
const Device UA2626745 = {S25321, 46, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - rad.io/1.1.12 (Android 4.1.1) samsung/GT-P5113*/
const Device UA2627147 = {S25322, 134, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2627549 = {S25323, 137, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; GT-P5113 Build/JRO03C) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2627621 = {S25324, 144, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; GT-P5113 Build/JRO03C) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2627950 = {S25325, 163, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/5.0.2 Version/4.0 Safari/534.30 Safari/6533.18.5*/
const Device UA2628310 = {S25326, 93, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - com.google.android.youtube/4.1.23(Linux; U; Android 4.1.1; en_US; GT-P5113 Build/JRO03C) gzip*/
const Device UA2629064 = {S25327, 88, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 4.1.1; SAMSUNG GT-P5113 Build/Unknown);*/
const Device UA2630417 = {S25328, 69, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Gospel_Library 2.3.1.3 / Android 4.1.1 P5113UECLK7 / samsung GT-P5113*/
const Device UA2631042 = {S25329, 112, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Android 4.1.1; en_US; GT-P5113/espresso10wifi; com.getsetgames.megajump 35 1.6; Kiip/Java 11 1.0.14-2 2012-05-24*/
const Device UA2631510 = {S25330, 155, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; JRO03C; samsung; GT-P5113; en-us; dream) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2631559 = {S25331, 130, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P5113 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2632136 = {S25332, 195, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 androidAppVersion/2.5.17 AndroidWebkitWrapper BillingSupported*/
const Device UA2632242 = {S25333, 145, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2632772 = {S25334, 59, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - nApps (Android 4.1.1; GT-P5113; androidapp.lineplay; 1.2.1)*/
const Device UA2632895 = {S25335, 151, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-ca; GT-P5113 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MantanoReader*/
const Device UA2633223 = {S25336, 62, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.0; 5) GT-P5113 (Android 4.1.1, SDK 16)*/
const Device UA2633431 = {S25337, 80, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - VKAndroidApp/2.7.6-64 (Android 4.1.1; SDK 16; armeabi-v7a; samsung GT-P5113; en)*/
const Device UA2633476 = {S25338, 270, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/2.1;FBDM/{density=1.0,width=1280,height=752};FBLC/es_US;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P5113;FBSV/4.1.1;]*/
const Device UA2633586 = {S25339, 51, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - ZillowMap/5.2.194 (Android 4.1.1, samsung GT-P5113)*/
const Device UA2634993 = {S25340, 219, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.4.1-100 (Android 4.1.1; samsung GT-P5113 Build/JRO03C.P5113UECLK7)*/
const Device UA2635128 = {S25341, 95, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - nApps:(Android 4.1.1; GT-P5113; %EB%84%A4%EC%9D%B4%EB%B2%84+%EC%BA%98%EB%A6%B0%EB%8D%94; 1.9.6)*/
const Device UA2635640 = {S25342, 142, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-ca; GT-P5113 Build/JRO03C like Xoom) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2636359 = {S25343, 143, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr_TR; GT-P5113 BUILD/JRO03C) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2637471 = {S25344, 175, { &P22043,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 GoogleGoggles-Android/1.9; gzip*/
const Device UA2043019 = {S25345, 67, { &P15364,&P5690,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:15.0) Gecko/X Firefox/15.0.1*/
const Device UA2043354 = {S25346, 75, { &P15364,&P5690,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:15.0) Gecko/X Firefox/15.0.1 Firefox/15.0.1*/
const Device UA2043622 = {S25347, 67, { &P15364,&P5690,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64; rv:15.0) Gecko/X Firefox/15.0.1*/
const Device UA2200364 = {S25348, 70, { &P15364,&P5690,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:15.0) Gecko/X Firefox/15.0*/
const Device UA2206072 = {S25349, 84, { &P15364,&P5690,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.3) Gecko/X Firefox/15.0.0.1*/
const Device UA2350043 = {S25350, 70, { &P15364,&P5690,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0(Windows; U; Windows NT 5.2; rv:1.9.2) Gecko/X Firefox/15.0*/
const Device UA2384247 = {S25351, 105, { &P15364,&P5690,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.1.7) Gecko/X Firefox/15.0 (.NET CLR 3.0.04506.648)*/
const Device UA64094 = {S25352, 77, { &P12141,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry8900/5.0.0.900 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/100*/
const Device UA1953337 = {S25353, 74, { &P15364,&P6704,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Alexa Toolbar)*/
const Device UA1965080 = {S25354, 64, { &P15364,&P6704,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; SV1)*/
const Device UA1987526 = {S25355, 66, { &P15364,&P6704,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; MyIE2)*/
const Device UA1990843 = {S25356, 64, { &P15364,&P6704,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; SV1)*/
const Device UA2256648 = {S25357, 69, { &P15364,&P6704,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; SV1; SV1)*/
const Device UA2413425 = {S25358, 69, { &P15364,&P6704,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; SV1; SV1)*/
const Device UA2596336 = {S25359, 54, { &P15364,&P6704,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Win98; Trident/4.0)*/
const Device UA2791704 = {S25360, 60, { &P15364,&P6704,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Win 9x 4.90; Trident/2.0)*/
const Device UA614775 = {S25361, 70, { &P15364,&P7133,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:12.0) Gecko/X Firefox/12.0*/
const Device UA615133 = {S25362, 92, { &P15364,&P7133,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:12.0; StumbleUpon; noc@stumbleupon.com) Gecko/X Firefox/12.0*/
const Device UA631236 = {S25363, 78, { &P15364,&P7133,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.0) Gecko/X Firefox/12.0 Firefox/12.0*/
const Device UA2163024 = {S25364, 67, { &P15364,&P7133,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.1; rv:12.0) Gecko/X Firefox/12.0*/
const Device UA2568302 = {S25365, 134, { &P15767,&P19826,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B146 Safari/8536.25*/
const Device UA2568306 = {S25366, 134, { &P15767,&P19826,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B147 Safari/8536.25*/
const Device UA2569937 = {S25367, 132, { &P15767,&P19826,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B141 Safari/8536.25*/
const Device UA2585916 = {S25368, 134, { &P15767,&P19826,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B329 Safari/8536.25*/
const Device UA22117 = {S25369, 46, { &P15364,&P6704,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows 98)*/
const Device UA690912 = {S25370, 62, { &P15364,&P6704,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.50; Windows 98; SiteKiosk 4.8)*/
const Device UA2505180 = {S25371, 78, { &P15364,&P6704,&P17387,&P18092 }};	/*Useragent - MSIE 5.5 (Win ME): Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; Win 9x 4.90)*/
const Device UA2713298 = {S25372, 88, { &P15364,&P7133,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:21.0) Gecko/X Firefox/21.0 AlexaToolbar/alxf-2.18*/
const Device UA2784778 = {S25373, 74, { &P15364,&P7133,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:16.0.1) Gecko/X Firefox/21.0.1*/
const Device UA2784853 = {S25374, 85, { &P15364,&P7133,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:21.0) Gecko/X Firefox/21.0 IceDragon/21.0.1.15*/
const Device UA2999475 = {S25375, 70, { &P15364,&P7133,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.1; rv:21.0) Gecko/21.0 Firefox/21.0*/
const Device UA3049232 = {S25376, 82, { &P15364,&P7133,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.1; Win64; x64; rv:21.0) Gecko/21.0 Firefox/21.0*/
const Device UA2040911 = {S25377, 109, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2876.0.0) AppleWebKit/537.10 (KHTML, like Gecko) Chrome/23.0.1262.2 Safari/537.10*/
const Device UA2042103 = {S25378, 111, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2876.0.0) AppleWebKit/537.10 (KHTML, like Gecko) Chrome/23.0.1262.2 Safari/537.10*/
const Device UA2195618 = {S25379, 113, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2913.49.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.19 Safari/537.11*/
const Device UA2197564 = {S25380, 111, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2913.49.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.19 Safari/537.11*/
const Device UA2197568 = {S25381, 111, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2913.50.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.19 Safari/537.11*/
const Device UA2198088 = {S25382, 110, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2913.21.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.8 Safari/537.11*/
const Device UA2201777 = {S25383, 111, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2913.27.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.11 Safari/537.11*/
const Device UA2202167 = {S25384, 113, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2913.27.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.11 Safari/537.11*/
const Device UA2202389 = {S25385, 111, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2913.3.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.2 Safari/537.11*/
const Device UA2202674 = {S25386, 109, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2913.3.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.2 Safari/537.11*/
const Device UA2206181 = {S25387, 111, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2913.41.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.16 Safari/537.11*/
const Device UA2232209 = {S25388, 111, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2913.73.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.30 Safari/537.11*/
const Device UA2232228 = {S25389, 113, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2913.73.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.30 Safari/537.11*/
const Device UA2232848 = {S25390, 114, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2913.106.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.41 Safari/537.11*/
const Device UA2232862 = {S25391, 112, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2913.106.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.41 Safari/537.11*/
const Device UA2235705 = {S25392, 114, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 2913.84.10) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.49 Safari/537.11*/
const Device UA2237251 = {S25393, 114, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2913.135.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.53 Safari/537.11*/
const Device UA2237252 = {S25394, 112, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2913.135.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.53 Safari/537.11*/
const Device UA2259953 = {S25395, 114, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 2913.157.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.83 Safari/537.11*/
const Device UA2259957 = {S25396, 112, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2913.156.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.83 Safari/537.11*/
const Device UA2259960 = {S25397, 114, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2913.156.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.83 Safari/537.11*/
const Device UA2352653 = {S25398, 113, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2913.41.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.16 Safari/537.11*/
const Device UA2356025 = {S25399, 114, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 2913.135.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.53 Safari/537.11*/
const Device UA2357473 = {S25400, 114, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2913.166.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.84 Safari/537.11*/
const Device UA2357759 = {S25401, 112, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2913.166.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.84 Safari/537.11*/
const Device UA2359379 = {S25402, 114, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 2913.168.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.84 Safari/537.11*/
const Device UA2359541 = {S25403, 114, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2913.187.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.89 Safari/537.11*/
const Device UA2359542 = {S25404, 112, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2913.187.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.89 Safari/537.11*/
const Device UA2362484 = {S25405, 112, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2913.156.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.83 Safari/537.11*/
const Device UA2362765 = {S25406, 114, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 2913.187.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.89 Safari/537.11*/
const Device UA2368393 = {S25407, 114, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2913.181.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.87 Safari/537.11*/
const Device UA2385917 = {S25408, 113, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 2913.53.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.20 Safari/537.11*/
const Device UA2388495 = {S25409, 114, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 2913.224.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.94 Safari/537.11*/
const Device UA2390678 = {S25410, 114, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2913.224.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.94 Safari/537.11*/
const Device UA2390681 = {S25411, 112, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2913.224.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.94 Safari/537.11*/
const Device UA2403262 = {S25412, 112, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2913.120.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.49 Safari/537.11*/
const Device UA2404939 = {S25413, 112, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2913.200.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.92 Safari/537.11*/
const Device UA2407504 = {S25414, 114, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 2913.251.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2408231 = {S25415, 114, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2913.260.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.99 Safari/537.11*/
const Device UA2408232 = {S25416, 112, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2913.260.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.99 Safari/537.11*/
const Device UA2427303 = {S25417, 113, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2913.276.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.110 Safari/537.11*/
const Device UA2427561 = {S25418, 114, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 2913.260.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.99 Safari/537.11*/
const Device UA2437226 = {S25419, 115, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2913.270.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.110 Safari/537.11*/
const Device UA2437273 = {S25420, 114, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 2913.212.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.93 Safari/537.11*/
const Device UA2437288 = {S25421, 115, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2913.278.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.110 Safari/537.11*/
const Device UA2459285 = {S25422, 114, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2913.225.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.94 Safari/537.11*/
const Device UA2462374 = {S25423, 115, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2913.331.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.111 Safari/537.11*/
const Device UA2470137 = {S25424, 113, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2913.331.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.111 Safari/537.11*/
const Device UA2470139 = {S25425, 115, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 2913.331.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.111 Safari/537.11*/
const Device UA2530753 = {S25426, 113, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2913.345.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.113 Safari/537.11*/
const Device UA2558943 = {S25427, 115, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 2913.345.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.113 Safari/537.11*/
const Device UA2731110 = {S25428, 114, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2913.187.6) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.89 Safari/537.11*/
const Device UA2842301 = {S25429, 114, { &P15364,&P6596,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2913.187.3) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.89 Safari/537.11*/
const Device UA454813 = {S25430, 132, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA605591 = {S25431, 132, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA630601 = {S25432, 132, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P7510 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA648028 = {S25433, 132, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P7510 Build/HTJ85B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1994205 = {S25434, 132, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T859 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1995465 = {S25435, 61, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7510 Build/IMM76D)*/
const Device UA2227890 = {S25436, 130, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2456411 = {S25437, 150, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P7510-ORANGE/P7510BVLP5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2498927 = {S25438, 61, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-T859 Build/IMM76D)*/
const Device UA2557659 = {S25439, 150, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P7510-ORANGE/P7510BVLP7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2608327 = {S25440, 61, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-P7510 Build/IML74K)*/
const Device UA2608834 = {S25441, 119, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/1.1.2 (Linux; U; Android 4.0.4; en-CA; GT-P7510 Build/cm_p4wifi-userdebug 4.0.4 IMM76L eng.jenkins.X.X test-keys)*/
const Device UA2611806 = {S25442, 139, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2612945 = {S25443, 145, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2613189 = {S25444, 130, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7510 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2613317 = {S25445, 153, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7510 Build/IMM76D; 1280*752) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2613617 = {S25446, 103, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Android 4.0.4; en-us; GT-P7510 Build/IMM76D) FreeWheelAdManager/4.9.2-r8793-X;com.vevo/435*/
const Device UA2614670 = {S25447, 73, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.0.4; en-US; GT-P7510 Build/IMM76D.UELPL)*/
const Device UA2614926 = {S25448, 137, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2615919 = {S25449, 69, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - BBC iPlayer (AndroidApp; 1.3; 13014) GT-P7510 (Android 4.0.4, SDK 15)*/
const Device UA2616137 = {S25450, 296, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7510 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 parameters/{scr=752_1280,cm=1,ql=l,imei=BC:47:60:F7:DE:1A,bm=1,eng=adrmini15,mdm=1,mdl=GT-P7510,ps=1} tiantian(securitypay,securityinstalled) tiantian(tenpay)*/
const Device UA2616434 = {S25451, 75, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Biblia/2.1.2 (Linux; U; Android 4.0.4; es-US; GT-P7510 Build/Task650's ICS)*/
const Device UA2616585 = {S25452, 122, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2616756 = {S25453, 139, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2616927 = {S25454, 73, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.0.4; en-GB; GT-P7510 Build/IMM76D.XWLP6)*/
const Device UA2617240 = {S25455, 73, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.4; en-US; GT-P7510 Build/IMM76D.ZSLP8)*/
const Device UA2617393 = {S25456, 212, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.2.1-65 (Android 4.0.4; samsung GT-P7510 Build/IMM76D.UELPL)*/
const Device UA2617411 = {S25457, 196, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 androidAppVersion/2.2.2.1 AndroidWebkitWrapper BillingSupported*/
const Device UA2618980 = {S25458, 150, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P7510-ORANGE/P7510BVLP1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2619166 = {S25459, 87, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.2.3 (Linux; U; Android 4.0.4; en-GB; GT-P7510 Build/IMM76D.XWLP5)*/
const Device UA2619654 = {S25460, 94, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.apps.plus/X (Linux; U; Android 4.0.4; en_US; GT-P7510; Build/IMM76D) (gzip)*/
const Device UA2619887 = {S25461, 103, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 4.0.4; SAMSUNG GT-P7510 Build/ICE_CREAM_SANDWICH_MR1);*/
const Device UA2620788 = {S25462, 128, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Groupon/2.1.2483 (Android 4.0.4; Samsung GT-P7510 / Samsung GT-P7510; )[preload=false;locale=en_US;clientidbase=android-samsung]*/
const Device UA2620844 = {S25463, 137, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2621092 = {S25464, 78, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - MediaPlayer/2002/4.1.0 (Android 4.0.4; GT-P7510; GT-P7510; samsung; ; ; en_US)*/
const Device UA2622139 = {S25465, 144, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2622202 = {S25466, 47, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Scanner Radio/3.9.0.2 (Android 4.0.4, GT-P7510)*/
const Device UA2622260 = {S25467, 139, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; GT-P7510 Build/HTJ85B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2622484 = {S25468, 132, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4, en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2623377 = {S25469, 75, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-GB; GT-P7510 Build/IMM76D.XXLPH)*/
const Device UA2624151 = {S25470, 93, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.youtube/4.2.16(Linux; U; Android 4.0.4; en_US; GT-P7510 Build/IMM76D) gzip*/
const Device UA2624193 = {S25471, 45, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Scanner Radio/3.9.1 (Android 4.0.4, GT-P7510)*/
const Device UA2624627 = {S25472, 47, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Scanner Radio/3.9.1.1 (Android 4.0.4, GT-P7510)*/
const Device UA2624706 = {S25473, 75, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-GB; GT-P7510 Build/IMM76D.VLLH3)*/
const Device UA2624803 = {S25474, 88, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.2.10 (Linux; U; Android 4.0.4; en-US; GT-P7510 Build/IMM76D.XWLPC)*/
const Device UA2625411 = {S25475, 137, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; it-it; GT-P7510 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2625532 = {S25476, 85, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik 2.02.11978; (Linux; U; Android 4.0.4; GT-P7510) 5a5d79363c64d681; 1280x752x1.0*/
const Device UA2626113 = {S25477, 131, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.04; en-us; GT-P7510 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2626123 = {S25478, 51, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - ZillowMap/5.2.194 (Android 4.0.4, samsung GT-P7510)*/
const Device UA2626224 = {S25479, 62, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.0; 5) GT-P7510 (Android 4.0.4, SDK 15)*/
const Device UA2626405 = {S25480, 63, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - BBC News (AndroidApp; 2.1; 33) GT-P7510 (Android 4.0.4, SDK 15)*/
const Device UA2626434 = {S25481, 45, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Scanner Radio/3.9.2 (Android 4.0.4, GT-P7510)*/
const Device UA2629524 = {S25482, 62, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.1; 9) GT-P7510 (Android 4.0.4, SDK 15)*/
const Device UA2630341 = {S25483, 148, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-; GT-P7510-ORANGE/P7510BVLP5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2631237 = {S25484, 87, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - ????/2.12.24 (Linux; U; Android 4.0.4; zh-cn; GT-P7510 Build/IMM76D) AppleWebKit/528.5+*/
const Device UA2631736 = {S25485, 45, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Scanner Radio/3.9.3 (Android 4.0.4, GT-P7510)*/
const Device UA2631759 = {S25486, 80, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - VKAndroidApp/2.7.6-64 (Android 4.0.4; SDK 15; armeabi-v7a; samsung GT-P7510; en)*/
const Device UA2631760 = {S25487, 270, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/2.1;FBDM/{density=1.0,width=1280,height=752};FBLC/en_US;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P7510;FBSV/4.0.4;]*/
const Device UA2632878 = {S25488, 56, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Android/4.8.1 (Linux; U; Android 4.0.4; en_US; GT-P7510)*/
const Device UA2633386 = {S25489, 195, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 androidAppVersion/2.5.17 AndroidWebkitWrapper BillingSupported*/
const Device UA2633550 = {S25490, 134, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2634864 = {S25491, 200, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 ZyngaBundleIdentifier/com.zynga.zjayakashi ZyngaBundleVersion/2.0.0*/
const Device UA2635594 = {S25492, 144, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7510 Build/IMM76L) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2635861 = {S25493, 45, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - Scanner Radio/3.9.4 (Android 4.0.4, GT-P7510)*/
const Device UA2636467 = {S25494, 46, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - rad.io/1.1.19 (Android 4.0.4) samsung/GT-P7510*/
const Device UA2636977 = {S25495, 63, { &P12879,&P18066,&P18062,&P18092 }};	/*Useragent - BBC News (AndroidApp; 2.2; 29) GT-P7510 (Android 4.0.4, SDK 15)*/
const Device UA174921 = {S25496, 105, { &P15364,&P7133,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; ) AppleWebKit/533.9 (KHTML, like Gecko) Maxthon/3.0 Safari/533.9*/
const Device UA190766 = {S25497, 107, { &P15364,&P7133,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; ) AppleWebKit/534.12 (KHTML, like Gecko) Maxthon/3.0 Safari/534.12*/
const Device UA209291 = {S25498, 214, { &P15364,&P7133,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; ) AppleWebKit/534.12 (KHTML, like Gecko) Maxthon/3.0 Safari/534.12Mozilla/5.0 (Windows; U; Windows NT 6.1; ) AppleWebKit/534.12 (KHTML, like Gecko) Maxthon/3.0 Safari/534.12*/
const Device UA494751 = {S25499, 115, { &P15364,&P7133,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Maxthon/3.0.6.27 Safari/532.4*/
const Device UA694949 = {S25500, 115, { &P15364,&P7133,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.1 (KHTML, like Gecko) Maxthon/3.0.11.8 Safari/533.1*/
const Device UA783740 = {S25501, 118, { &P15364,&P7133,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12*/
const Device UA787751 = {S25502, 111, { &P15364,&P7133,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12*/
const Device UA2127749 = {S25503, 110, { &P15364,&P7133,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/530.6 (KHTML, like Gecko) Maxthon/3.0 Safari/530.6*/
const Device UA2195943 = {S25504, 148, { &P15364,&P7133,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12,gzip(gfe) (via translate.google.com)*/
const Device UA2200602 = {S25505, 111, { &P15364,&P7133,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA2200615 = {S25506, 118, { &P15364,&P7133,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA2205752 = {S25507, 155, { &P15364,&P7133,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12,gzip(gfe) (via translate.google.com)*/
const Device UA2217804 = {S25508, 130, { &P15364,&P7133,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12*/
const Device UA2235731 = {S25509, 128, { &P15364,&P7133,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12,gzip(gfe)*/
const Device UA2259140 = {S25510, 121, { &P15364,&P7133,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12,gzip(gfe)*/
const Device UA2426463 = {S25511, 128, { &P15364,&P7133,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1,gzip(gfe)*/
const Device UA2466330 = {S25512, 121, { &P15364,&P7133,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1,gzip(gfe)*/
const Device UA2950703 = {S25513, 157, { &P15364,&P7133,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3118103 = {S25514, 150, { &P15364,&P7133,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA153895 = {S25515, 117, { &P15364,&P5690,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.558.0 Safari/534.10*/
const Device UA168649 = {S25516, 119, { &P15364,&P5690,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10*/
const Device UA294760 = {S25517, 156, { &P15364,&P5690,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10,gzip(gfe) (via translate.google.com)*/
const Device UA2272590 = {S25518, 158, { &P15364,&P5690,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2360478 = {S25519, 114, { &P15364,&P5690,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; vi) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.558.0 Safari/534.10*/
const Device UA2366729 = {S25520, 129, { &P15364,&P5690,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10,gzip(gfe)*/
const Device UA2402816 = {S25521, 114, { &P15364,&P5690,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; id) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.558.0 Safari/534.10*/
const Device UA2413666 = {S25522, 117, { &P15364,&P5690,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; Pt-br) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.558.0 Safari/534.10*/
const Device UA2483159 = {S25523, 117, { &P15364,&P5690,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; es-la) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.558.0 Safari/534.10*/
const Device UA2651853 = {S25524, 117, { &P15364,&P5690,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; fr-fr) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.558.0 Safari/534.10*/
const Device UA2680943 = {S25525, 119, { &P15364,&P5690,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; haw-US) AppleWebKit/534.10 (KHTML like Gecko) Chrome/8.0.552.224 Safari/534.10*/
const Device UA2708265 = {S25526, 118, { &P15364,&P5690,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.10 (KHTML like Gecko) Chrome/8.0.552.224 Safari/534.10*/
const Device UA2708345 = {S25527, 118, { &P15364,&P5690,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-CA) AppleWebKit/534.10 (KHTML like Gecko) Chrome/8.0.552.224 Safari/534.10*/
const Device UA2709332 = {S25528, 118, { &P15364,&P5690,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; fr-CA) AppleWebKit/534.10 (KHTML like Gecko) Chrome/8.0.552.224 Safari/534.10*/
const Device UA2709447 = {S25529, 118, { &P15364,&P5690,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/534.10 (KHTML like Gecko) Chrome/8.0.552.224 Safari/534.10*/
const Device UA2710985 = {S25530, 118, { &P15364,&P5690,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; es-US) AppleWebKit/534.10 (KHTML like Gecko) Chrome/8.0.552.224 Safari/534.10*/
const Device UA2735202 = {S25531, 118, { &P15364,&P5690,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; br-FR) AppleWebKit/534.10 (KHTML like Gecko) Chrome/8.0.552.224 Safari/534.10*/
const Device UA2735971 = {S25532, 118, { &P15364,&P5690,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; pl-DE) AppleWebKit/534.10 (KHTML like Gecko) Chrome/8.0.552.224 Safari/534.10*/
const Device UA2859969 = {S25533, 118, { &P15364,&P5690,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; de-DE) AppleWebKit/534.10 (KHTML like Gecko) Chrome/8.0.552.224 Safari/534.10*/
const Device UA2860382 = {S25534, 118, { &P15364,&P5690,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; ru-DE) AppleWebKit/534.10 (KHTML like Gecko) Chrome/8.0.552.224 Safari/534.10*/
const Device UA349878 = {S25535, 128, { &P13062,&P6374,&P4418,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9790; en) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.528 Mobile Safari/534.11+*/
const Device UA93316 = {S25536, 47, { &P15364,&P6704,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.00; Windows 98)*/
const Device UA104475 = {S25537, 53, { &P15364,&P6704,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0(compatible; MSIE 5.0; Windows 98; DigExt)*/
const Device UA121204 = {S25538, 54, { &P15364,&P6704,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows 98; DigExt)*/
const Device UA130421 = {S25539, 47, { &P15364,&P6704,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01; Windows 98)*/
const Device UA382278 = {S25540, 44, { &P15364,&P6704,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0(compatible;MSIE 5.00;Windows 98)*/
const Device UA496319 = {S25541, 47, { &P15364,&P6704,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.02; Windows 98)*/
const Device UA538597 = {S25542, 119, { &P15364,&P6704,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01; Windows 98; Linux 2.4.2-mvista_01) [Netgem; 4.6.23c; i-Player; netbox; btdigitaltv]*/
const Device UA877597 = {S25543, 66, { &P15364,&P6704,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.05; Windows 98; .NET CLR 1.1.4322)*/
const Device UA2658840 = {S25544, 60, { &P15364,&P6704,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; Win 9x 4.90; Trident/4.0)*/
const Device UA2851957 = {S25545, 54, { &P15364,&P6704,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; Win98; Trident/5.0)*/
const Device UA3041939 = {S25546, 76, { &P15364,&P6704,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows 98; Avant Browser; Avant Browser)*/
const Device UA441536 = {S25547, 142, { &P18321,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA899483 = {S25548, 143, { &P18321,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-S5300B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1966905 = {S25549, 66, { &P18321,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5300 Build/GINGERBREAD)*/
const Device UA2025572 = {S25550, 130, { &P18321,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2028913 = {S25551, 143, { &P18321,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-S5300L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2168267 = {S25552, 140, { &P18321,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2441602 = {S25553, 67, { &P18321,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5300B Build/GINGERBREAD)*/
const Device UA888152 = {S25554, 86, { &P17968,&P17979,&P17780,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/6.1.27762/26.1670; U; es) Presto/2.8.119 Version/10.54*/
const Device UA2431421 = {S25555, 86, { &P17968,&P17979,&P17780,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/6.1.25577/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2008999 = {S25556, 117, { &P17595,&P17295,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.57 Safari/537.1*/
const Device UA2205475 = {S25557, 117, { &P17595,&P17295,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.90 Safari/537.1*/
const Device UA2205606 = {S25558, 117, { &P17595,&P17295,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.90 Safari/537.1*/
const Device UA2205611 = {S25559, 117, { &P17595,&P17295,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.90 Safari/537.1*/
const Device UA2205794 = {S25560, 117, { &P17595,&P17295,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_5) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.90 Safari/537.1*/
const Device UA2205803 = {S25561, 117, { &P17595,&P17295,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.90 Safari/537.1*/
const Device UA2205891 = {S25562, 154, { &P17595,&P17295,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.90 Safari/537.1,gzip(gfe) (via translate.google.com)*/
const Device UA2231571 = {S25563, 117, { &P17595,&P17295,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2258179 = {S25564, 127, { &P17595,&P17295,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.90 Safari/537.1,gzip(gfe)*/
const Device UA2417012 = {S25565, 156, { &P17595,&P17295,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.90 Safari/537.1,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2647444 = {S25566, 117, { &P17595,&P17295,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.79 Safari/537.1*/
const Device UA2674624 = {S25567, 129, { &P17595,&P17295,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/21.0.648.205 Safari/534.16*/
const Device UA2716431 = {S25568, 129, { &P17595,&P17295,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/21.0.648.205 Safari/534.16*/
const Device UA2948474 = {S25569, 137, { &P18283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; GT-I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36*/
const Device UA2973667 = {S25570, 137, { &P18283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973948 = {S25571, 138, { &P18283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973968 = {S25572, 137, { &P18283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974039 = {S25573, 137, { &P18283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974281 = {S25574, 137, { &P18283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974288 = {S25575, 137, { &P18283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974370 = {S25576, 138, { &P18283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974800 = {S25577, 138, { &P18283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974903 = {S25578, 135, { &P18283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SC-06D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975036 = {S25579, 137, { &P18283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-I535 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975533 = {S25580, 138, { &P18283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999V Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975737 = {S25581, 137, { &P18283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; GT-I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021354 = {S25582, 138, { &P18283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022191 = {S25583, 137, { &P18283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-i9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022272 = {S25584, 138, { &P18283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022510 = {S25585, 137, { &P18283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I939 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023020 = {S25586, 134, { &P18283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023268 = {S25587, 134, { &P18283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023548 = {S25588, 138, { &P18283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I93000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA453557 = {S25589, 77, { &P15364,&P5690,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; I; Windows NT 5.1; en; rv:1.9.2.13) Gecko/X Firefox/4.0*/
const Device UA696574 = {S25590, 93, { &P15364,&P5690,&P17203,&P18092 }};	/*Useragent - User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.0.6) Gecko/X Firefox/4.0.1*/
const Device UA710768 = {S25591, 68, { &P15364,&P5690,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:2.0b13pre) Gecko/X Firefox/4.0b13pre*/
const Device UA894284 = {S25592, 50, { &P15364,&P5690,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) Gecko/X Firefox/4.0.1*/
const Device UA2139943 = {S25593, 54, { &P15364,&P5690,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0(Windows NT 5.1;rv:2.0) Gecko/X Firefox/4.0*/
const Device UA2235240 = {S25594, 70, { &P15364,&P5690,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:2.0.1) Gecko/X Firefox/4.0.1,gzip(gfe)*/
const Device UA2260110 = {S25595, 50, { &P15364,&P5690,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0(WindowsNT5.1;rv:2.0)Gecko/XFirefox/4.0*/
const Device UA2505830 = {S25596, 77, { &P15364,&P5690,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.1) Gecko/X Firefox/4*/
const Device UA47824 = {S25597, 144, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-137) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47825 = {S25598, 144, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-163) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48328 = {S25599, 149, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-154_MR-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48329 = {S25600, 144, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-154) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48333 = {S25601, 146, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-154_4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48334 = {S25602, 144, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48336 = {S25603, 149, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-154_MR-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48337 = {S25604, 146, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-154_5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48330 = {S25605, 144, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-143) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48862 = {S25606, 160, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Motorola Electrify Build/4.5.1A_SUN_USC_15.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49986 = {S25607, 160, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Motorola Electrify Build/4.5.1A_SUN_USC_16.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61289 = {S25608, 144, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-122) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61290 = {S25609, 144, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-133) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61293 = {S25610, 144, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MB855 Build/4.5.1A-1_SUN-101) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70088 = {S25611, 146, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-154_6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70680 = {S25612, 146, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-198_5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71808 = {S25613, 146, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-198_6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72473 = {S25614, 150, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-198_GSM-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369238 = {S25615, 76, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.4; en_US; MB855 Build/4.5.1A-1_SUN-198_6)*/
const Device UA396941 = {S25616, 70, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; MB855 Build/4.5.1A-1_SUN-198_6)*/
const Device UA451505 = {S25617, 146, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-198_7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622882 = {S25618, 160, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Motorola Electrify Build/4.5.1A_SUN_USC_19.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623949 = {S25619, 84, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Motorola Electrify Build/4.5.1A_SUN_USC_19.0)*/
const Device UA641356 = {S25620, 84, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; Motorola Electrify Build/4.5.1A_SUN_USC_16.7)*/
const Device UA868405 = {S25621, 147, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.1A-1_SUN-254_12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA911994 = {S25622, 147, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA923959 = {S25623, 71, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; MB855 Build/4.5.1A-1_SUN-254_13)*/
const Device UA1936427 = {S25624, 152, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.1A-1_SUN-254_4_GSM-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1940356 = {S25625, 146, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.1A-1_SUN-254_6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1943303 = {S25626, 152, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.1A-1_SUN-254_4_GSM-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022058 = {S25627, 167, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Motorola Electrify Build/4.5.1A_SUN_USC_19.0) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2033222 = {S25628, 168, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Motorola Electrify Build/4.5.1A_SUN_USC_19.0) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2453301 = {S25629, 153, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Slim Photon Build/4.5.1A-1_SUN-254_13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2453981 = {S25630, 155, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30*/
const Device UA2520515 = {S25631, 145, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2796086 = {S25632, 147, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.0B-1_May-265_13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2934904 = {S25633, 169, { &P12731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.8; en-us; MB855 Build/4.5.1A-1_SUN-198_6; CyanMobile-eXperience) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2168603 = {S25634, 139, { &P18673,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2237671 = {S25635, 140, { &P18673,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT-S7562L Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2242279 = {S25636, 61, { &P18673,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-S7562 Build/IMM76I)*/
const Device UA2257518 = {S25637, 137, { &P18673,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-S7562 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2400672 = {S25638, 130, { &P18673,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562 Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2420901 = {S25639, 140, { &P18673,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562i Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455808 = {S25640, 131, { &P18673,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562i Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2553003 = {S25641, 62, { &P18673,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-S7562L Build/IMM76I)*/
const Device UA2555694 = {S25642, 62, { &P18673,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-S7562i Build/IMM76I)*/
const Device UA2939582 = {S25643, 144, { &P18673,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S7562 Build/862_F1_V2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2983091 = {S25644, 125, { &P18673,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/IMM76I) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2989303 = {S25645, 139, { &P18673,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3016611 = {S25646, 151, { &P18673,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S7562 Build/862_F1_V2.4_S501UL) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3020507 = {S25647, 150, { &P18673,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S7562 Build/862_F1_V2.3_S501U) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3048861 = {S25648, 149, { &P18673,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S7562 Build/IMM761.S7562ZCALK2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3050435 = {S25649, 144, { &P18673,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; GT-S7562 Build/862_F1_V2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103180 = {S25650, 136, { &P18673,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/320) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3107707 = {S25651, 136, { &P18673,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/340) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA149564 = {S25652, 116, { &P15364,&P5690,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA205628 = {S25653, 98, { &P15364,&P5690,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.28 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.28*/
const Device UA283365 = {S25654, 116, { &P15364,&P5690,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.41 Safari/534.7*/
const Device UA291993 = {S25655, 153, { &P15364,&P5690,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7,gzip(gfe) (via translate.google.com)*/
const Device UA432405 = {S25656, 130, { &P15364,&P5690,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.50 Safari/534.7 SunChrome/7.0*/
const Device UA646519 = {S25657, 126, { &P15364,&P5690,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0 baidubrowser/1.x Safari/534.7*/
const Device UA1886168 = {S25658, 150, { &P15364,&P5690,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.16; QQBrowser/5.1.7061.201 (webkit)*/
const Device UA2287533 = {S25659, 150, { &P15364,&P5690,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.16; QQBrowser/5.1.6829.201 (webkit)*/
const Device UA2503458 = {S25660, 126, { &P15364,&P5690,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.41 Safari/534.7,gzip(gfe)*/
const Device UA2674212 = {S25661, 114, { &P15364,&P5690,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.6 (KHTML like Gecko) Chrome/7.0.503.0 Safari/534.6*/
const Device UA2675828 = {S25662, 114, { &P15364,&P5690,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-CA) AppleWebKit/534.6 (KHTML like Gecko) Chrome/7.0.503.0 Safari/534.6*/
const Device UA2678290 = {S25663, 114, { &P15364,&P5690,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; fr-CA) AppleWebKit/534.6 (KHTML like Gecko) Chrome/7.0.503.0 Safari/534.6*/
const Device UA2708511 = {S25664, 114, { &P15364,&P5690,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; fr-FR) AppleWebKit/534.6 (KHTML like Gecko) Chrome/7.0.503.0 Safari/534.6*/
const Device UA2709455 = {S25665, 114, { &P15364,&P5690,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; es-US) AppleWebKit/534.6 (KHTML like Gecko) Chrome/7.0.503.0 Safari/534.6*/
const Device UA2711118 = {S25666, 115, { &P15364,&P5690,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; haw-US) AppleWebKit/534.6 (KHTML like Gecko) Chrome/7.0.503.0 Safari/534.6*/
const Device UA2735133 = {S25667, 114, { &P15364,&P5690,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/534.6 (KHTML like Gecko) Chrome/7.0.503.0 Safari/534.6*/
const Device UA2735698 = {S25668, 114, { &P15364,&P5690,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; ku-DE) AppleWebKit/534.6 (KHTML like Gecko) Chrome/7.0.503.0 Safari/534.6*/
const Device UA2811158 = {S25669, 143, { &P15364,&P5690,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; AppleWebKit/534.7; KHTML?? like Gecko; Chrome/7.0.517.41 Safari/534.7 ChromePlus/1.5.0.0alpha1)*/
const Device UA2566450 = {S25670, 138, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566510 = {S25671, 138, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SPH-L710 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566526 = {S25672, 139, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747M Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566536 = {S25673, 139, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-R530U Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566636 = {S25674, 131, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566640 = {S25675, 146, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566634 = {S25676, 138, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566772 = {S25677, 139, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567008 = {S25678, 139, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567115 = {S25679, 131, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SPH-L710 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567196 = {S25680, 131, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567204 = {S25681, 138, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567213 = {S25682, 139, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567216 = {S25683, 132, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-R530U Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567221 = {S25684, 139, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305T Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567259 = {S25685, 132, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747M Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567333 = {S25686, 139, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305N Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567571 = {S25687, 139, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567698 = {S25688, 146, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2571470 = {S25689, 132, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583105 = {S25690, 132, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583207 = {S25691, 139, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2584098 = {S25692, 138, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2584102 = {S25693, 139, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2585167 = {S25694, 131, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2602039 = {S25695, 138, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2603479 = {S25696, 139, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667084 = {S25697, 139, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305N Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667184 = {S25698, 132, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305T Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2673780 = {S25699, 139, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530C Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2675918 = {S25700, 132, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530C Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2878422 = {S25701, 139, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747M Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2899413 = {S25702, 132, { &P18645,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747M Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA440917 = {S25703, 71, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; en) Presto/2.10.229 Version/11.62*/
const Device UA453569 = {S25704, 87, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.62*/
const Device UA453573 = {S25705, 94, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5230); ru) Presto/2.10.229 Version/11.62*/
const Device UA453705 = {S25706, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.229 Version/11.62*/
const Device UA453736 = {S25707, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.229 Version/11.62*/
const Device UA453883 = {S25708, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.229 Version/11.62*/
const Device UA453900 = {S25709, 87, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5288); ru) Presto/2.10.229 Version/11.62*/
const Device UA453910 = {S25710, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.229 Version/11.62*/
const Device UA454328 = {S25711, 94, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5218); ru) Presto/2.10.229 Version/11.62*/
const Device UA454380 = {S25712, 79, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.62*/
const Device UA454821 = {S25713, 87, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.62*/
const Device UA454883 = {S25714, 94, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5299); ru) Presto/2.10.229 Version/11.62*/
const Device UA603120 = {S25715, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.229 Version/11.62*/
const Device UA603125 = {S25716, 66, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; ru) Opera 11.62*/
const Device UA603126 = {S25717, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.229 Version/11.62*/
const Device UA603219 = {S25718, 93, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4871); ru) Presto/2.10.229 Version/11.62*/
const Device UA603331 = {S25719, 94, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5300); ru) Presto/2.10.229 Version/11.62*/
const Device UA603370 = {S25720, 94, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5161); ru) Presto/2.10.229 Version/11.62*/
const Device UA603528 = {S25721, 93, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4661); ru) Presto/2.10.229 Version/11.62*/
const Device UA603967 = {S25722, 87, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5309); ru) Presto/2.10.229 Version/11.62*/
const Device UA604229 = {S25723, 94, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5308); ru) Presto/2.10.229 Version/11.62*/
const Device UA607124 = {S25724, 87, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.229 Version/11.62*/
const Device UA623588 = {S25725, 87, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.229 Version/11.62*/
const Device UA628810 = {S25726, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.62*/
const Device UA630229 = {S25727, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.229 Version/11.62*/
const Device UA630840 = {S25728, 94, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.62*/
const Device UA630896 = {S25729, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.229 Version/11.62*/
const Device UA632525 = {S25730, 87, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.229 Version/11.62*/
const Device UA634763 = {S25731, 87, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.62*/
const Device UA635603 = {S25732, 87, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.229 Version/11.62*/
const Device UA649991 = {S25733, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.229 Version/11.62*/
const Device UA656703 = {S25734, 87, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.229 Version/11.62*/
const Device UA657140 = {S25735, 87, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.229 Version/11.62*/
const Device UA661778 = {S25736, 93, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4664); ru) Presto/2.10.229 Version/11.62*/
const Device UA663717 = {S25737, 87, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.229 Version/11.62*/
const Device UA671924 = {S25738, 87, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.229 Version/11.62*/
const Device UA676314 = {S25739, 87, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5217); ru) Presto/2.10.229 Version/11.62*/
const Device UA690785 = {S25740, 94, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5328); ru) Presto/2.10.229 Version/11.62*/
const Device UA696447 = {S25741, 108, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; en) Presto/2.10.229 Version/11.62,gzip(gfe) (via translate.google.com)*/
const Device UA731738 = {S25742, 94, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.7 (build 03789); ru) Presto/2.10.229 Version/11.62*/
const Device UA739866 = {S25743, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.229 Version/11.62*/
const Device UA748401 = {S25744, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.229 Version/11.62*/
const Device UA786691 = {S25745, 87, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5278); ru) Presto/2.10.229 Version/11.62*/
const Device UA787868 = {S25746, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.229 Version/11.62*/
const Device UA789257 = {S25747, 87, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.229 Version/11.62*/
const Device UA819794 = {S25748, 93, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.9 (build 4930); ru) Presto/2.10.229 Version/11.62*/
const Device UA856051 = {S25749, 87, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.229 Version/11.62*/
const Device UA896005 = {S25750, 87, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.229 Version/11.62*/
const Device UA901768 = {S25751, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.229 Version/11.62*/
const Device UA1954167 = {S25752, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.229 Version/11.62*/
const Device UA1985295 = {S25753, 87, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.229 Version/11.62*/
const Device UA1987398 = {S25754, 93, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.8 (build 4110); ru) Presto/2.10.229 Version/11.62*/
const Device UA1987719 = {S25755, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.229 Version/11.62*/
const Device UA1989075 = {S25756, 93, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5754); ru) Presto/2.10.229 Version/11.62*/
const Device UA1990266 = {S25757, 87, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03278); ru) Presto/2.10.229 Version/11.62*/
const Device UA1990365 = {S25758, 94, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.7 (build 03658); ru) Presto/2.10.229 Version/11.62*/
const Device UA2159077 = {S25759, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5667); ru) Presto/2.10.229 Version/11.62*/
const Device UA2164650 = {S25760, 87, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03780); ru) Presto/2.10.229 Version/11.62*/
const Device UA2168077 = {S25761, 93, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5831); ru) Presto/2.10.229 Version/11.62*/
const Device UA2177167 = {S25762, 93, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5970); ru) Presto/2.10.229 Version/11.62*/
const Device UA2178684 = {S25763, 93, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5972); ru) Presto/2.10.229 Version/11.62*/
const Device UA2333578 = {S25764, 94, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.7 (build 03639); ru) Presto/2.10.229 Version/11.62*/
const Device UA2333788 = {S25765, 94, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 5.10 (build 5325); ru) Presto/2.10.229 Version/11.62*/
const Device UA2369164 = {S25766, 72, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/11.62 (Windows NT 6.1; WOW64; U; en) Presto/2.10.229 Version/11.62*/
const Device UA2381773 = {S25767, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.10.229 Version/11.62*/
const Device UA2402620 = {S25768, 74, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.229 Version/11.62,gzip(gfe)*/
const Device UA2424045 = {S25769, 93, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 6005); ru) Presto/2.10.229 Version/11.62*/
const Device UA2456357 = {S25770, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.229 Version/11.62*/
const Device UA2485354 = {S25771, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.229 Version/11.62*/
const Device UA2554012 = {S25772, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.229 Version/11.62*/
const Device UA2579990 = {S25773, 57, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.10.229 Version/11.62*/
const Device UA2588961 = {S25774, 87, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5336); ru) Presto/2.10.229 Version/11.62*/
const Device UA2653681 = {S25775, 93, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 8.0 (build 5784); ru) Presto/2.10.229 Version/11.62*/
const Device UA2667124 = {S25776, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.229 Version/11.62*/
const Device UA2745931 = {S25777, 93, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 5966); ru) Presto/2.10.229 Version/11.62*/
const Device UA2860917 = {S25778, 93, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; MRA 6.0 (build 6081); ru) Presto/2.10.229 Version/11.62*/
const Device UA2958141 = {S25779, 86, { &P15364,&P7133,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.229 Version/11.62*/
const Device UA199107 = {S25780, 118, { &P17595,&P17296,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.91 Safari/534.30*/
const Device UA207671 = {S25781, 119, { &P17595,&P17296,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30*/
const Device UA208889 = {S25782, 119, { &P17595,&P17296,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30*/
const Device UA211060 = {S25783, 119, { &P17595,&P17296,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30*/
const Device UA216813 = {S25784, 119, { &P17595,&P17296,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA219929 = {S25785, 119, { &P17595,&P17296,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA323074 = {S25786, 119, { &P17595,&P17296,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30*/
const Device UA327004 = {S25787, 119, { &P17595,&P17296,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA648475 = {S25788, 119, { &P17595,&P17296,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30*/
const Device UA786195 = {S25789, 156, { &P17595,&P17296,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30,gzip(gfe) (via translate.google.com)*/
const Device UA2319351 = {S25790, 129, { &P17595,&P17296,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.16*/
const Device UA2674816 = {S25791, 129, { &P17595,&P17296,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/12.0.648.205 Safari/534.16*/
const Device UA2674863 = {S25792, 129, { &P17595,&P17296,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/12.0.648.205 Safari/534.16*/
const Device UA2718186 = {S25793, 129, { &P17595,&P17296,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/12.0.648.205 Safari/534.16*/
const Device UA352836 = {S25794, 142, { &P15364,&P17285,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7,gzip(gfe) (via translate.google.com)*/
const Device UA352839 = {S25795, 135, { &P15364,&P17285,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7,gzip(gfe) (via translate.google.com)*/
const Device UA366169 = {S25796, 98, { &P15364,&P17285,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7*/
const Device UA366199 = {S25797, 105, { &P15364,&P17285,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7*/
const Device UA588252 = {S25798, 136, { &P15364,&P17285,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/534.7 ZemanaAID/FFFF0061*/
const Device UA2360186 = {S25799, 105, { &P15364,&P17285,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.7 (KHTML	 like Gecko) Chrome/16.0.912.36 Safari/535.7*/
const Device UA2375182 = {S25800, 98, { &P15364,&P17285,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.8 (KHTML	 like Gecko) Chrome/16.0.912.63 Safari/535.8*/
const Device UA3061615 = {S25801, 137, { &P18283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; GT-I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3064655 = {S25802, 137, { &P18283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3064660 = {S25803, 137, { &P18283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065156 = {S25804, 138, { &P18283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065334 = {S25805, 137, { &P18283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065380 = {S25806, 137, { &P18283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065733 = {S25807, 138, { &P18283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065974 = {S25808, 138, { &P18283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066224 = {S25809, 138, { &P18283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066896 = {S25810, 137, { &P18283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-I535 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067293 = {S25811, 135, { &P18283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SC-06D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067472 = {S25812, 137, { &P18283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081049 = {S25813, 138, { &P18283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999V Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083155 = {S25814, 137, { &P18283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; GT-I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093669 = {S25815, 137, { &P18283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-i9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094976 = {S25816, 138, { &P18283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110906 = {S25817, 137, { &P18283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I939 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3113521 = {S25818, 138, { &P18283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I93000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA73026 = {S25819, 140, { &P15364,&P7133,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (Mozilla/5.0; Windows NT 6.1 WOW64; 6.0 ) AppleWebKit/533.4 (KHTML, like Gecko) Firefox/6.0/7.4.1.8 Mobile Safari/533.4 3gpp-gba*/
const Device UA452659 = {S25820, 102, { &P15364,&P7133,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.3a3pre) Gecko/X Firefox6.0 (.NET CLR 3.5.30729)*/
const Device UA681100 = {S25821, 81, { &P15364,&P7133,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:6.0.2) Gecko/X Firefox/6.0.2 Firefox/6.0.2*/
const Device UA1902480 = {S25822, 72, { &P15364,&P7133,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:6.0.2) Gecko/X Firefox/6.0.2*/
const Device UA1990643 = {S25823, 57, { &P15364,&P7133,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:6.0) Gecko/X Firefox/6.00*/
const Device UA2569803 = {S25824, 111, { &P15364,&P7133,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:6.0) Gecko/X Firefox/6.0 Google (+https://developers.google.com/+/web/snippet/)*/
const Device UA2574149 = {S25825, 72, { &P15364,&P7133,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; rv:6.0a2) Gecko/X Firefox/6.0a2*/
const Device UA2748154 = {S25826, 77, { &P15364,&P7133,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:6.0) Gecko/X Firefox/6.0 Safari/5.05 Pico/3.2*/
const Device UA2240795 = {S25827, 140, { &P23771,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ph; GT-I8190N Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467037 = {S25828, 140, { &P23771,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8190N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2490691 = {S25829, 161, { &P23771,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I8190N/I8190NXXALL4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2507427 = {S25830, 62, { &P23771,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8190N Build/JZO54K)*/
const Device UA2563850 = {S25831, 160, { &P23771,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-I8190N-ORANGE/I8190NXXALL5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2675041 = {S25832, 160, { &P23771,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-I8190N-ORANGE/I8190NXXAMB1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2737899 = {S25833, 161, { &P23771,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I8190N/I8190NXXAMB2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2801161 = {S25834, 161, { &P23771,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I8190N/I8190NXXAMD3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2855264 = {S25835, 138, { &P23771,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-I8190N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2975789 = {S25836, 161, { &P23771,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; SAMSUNG GT-I8190N/I8190NXXAMG6 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2976020 = {S25837, 161, { &P23771,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-I8190N/I8190NXXAMG3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3032051 = {S25838, 128, { &P15767,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A4449a Safari/9537.53*/
const Device UA3037887 = {S25839, 128, { &P15767,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A4449d Safari/9537.53*/
const Device UA3059874 = {S25840, 126, { &P15767,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A458 Safari/9537.53*/
const Device UA3060921 = {S25841, 126, { &P15767,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A465 Safari/9537.53*/
const Device UA3062565 = {S25842, 126, { &P15767,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A462 Safari/9537.53*/
const Device UA3093799 = {S25843, 128, { &P15767,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B475 Safari/9537.53*/
const Device UA3094919 = {S25844, 128, { &P15767,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11B478 Safari/9537.53*/
const Device UA3109324 = {S25845, 128, { &P15767,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A4449b Safari/9537.53*/
const Device UA3109887 = {S25846, 119, { &P15767,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A465 Safari/*/
const Device UA3113150 = {S25847, 126, { &P15767,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Version/7.0 Safari/8536.25*/
const Device UA3114763 = {S25848, 128, { &P15767,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A501 Safari/9537.53*/
const Device UA3129516 = {S25849, 125, { &P15767,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A465 Safari/9537.53*/
const Device UA3130090 = {S25850, 128, { &P15767,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Version/7.0 Safari/8536.25*/
const Device UA3130246 = {S25851, 121, { &P15767,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A501 Safari/*/
const Device UA17984 = {S25852, 153, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonMT15i Build/3.0.1.A.0.108) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18002 = {S25853, 139, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; zh-cn; MT15i Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18017 = {S25854, 153, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/3.0.1.A.0.135) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18023 = {S25855, 152, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/3.0.1.A.0.64) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18035 = {S25856, 153, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/3.0.1.A.0.142) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18047 = {S25857, 153, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/3.0.1.A.0.143) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24565 = {S25858, 151, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; SonyEricssonMT15i Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32995 = {S25859, 151, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/4.0.A.2.286) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41433 = {S25860, 139, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; es-es; MT15i Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41417 = {S25861, 141, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; MT15i Build/3.0.1.A.0.146) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41464 = {S25862, 154, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; SonyEricssonMT15iv Build/3.0.1.A.0.146) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41443 = {S25863, 155, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonMT15i-o Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41483 = {S25864, 151, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; SonyEricssonMT15i Build/3.0.A.0.326) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41505 = {S25865, 139, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; MT15i Build/3.0.A.0.326) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47198 = {S25866, 152, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ie; SonyEricssonMT15iv Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47248 = {S25867, 151, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonMT15i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49686 = {S25868, 151, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonMT15i Build/4.0.2.A.0.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49688 = {S25869, 153, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonMT15i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49694 = {S25870, 152, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonMT15i Build/4.0.1.A.2.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49696 = {S25871, 153, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonMT15i Build/4.0.1.A.0.277) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA53618 = {S25872, 150, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; sv-se; SonyEricssonMT15i Build/3.0.A.2.15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57163 = {S25873, 150, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; sv-se; SonyEricssonMT15i Build/3.0.A.2.79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57170 = {S25874, 154, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonMT15iv Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57181 = {S25875, 153, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; SonyEricssonMT15i Build/3.0.1.A.0.146) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57188 = {S25876, 150, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; SonyEricssonMT15i Build/3.0.A.2.52) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57190 = {S25877, 153, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/3.0.1.A.0.149) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57196 = {S25878, 153, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/3.0.1.A.0.150) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57198 = {S25879, 151, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; pt-pt; SonyEricssonMT15i Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57215 = {S25880, 141, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; MT15i Build/3.0.1.A.0.135) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57217 = {S25881, 153, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; es-es; SonyEricssonMT15i-o Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57224 = {S25882, 151, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; sv-se; SonyEricssonMT15i Build/3.0.A.2.182) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57232 = {S25883, 141, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; MT15i Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57239 = {S25884, 153, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonMT15i Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60209 = {S25885, 151, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/4.0.A.2.262) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60225 = {S25886, 151, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/4.0.A.2.204) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60210 = {S25887, 139, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; MT15i Build/4.0.A.2.240) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60228 = {S25888, 151, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/4.0.A.2.330) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60240 = {S25889, 151, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/4.0.A.2.305) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60249 = {S25890, 151, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/4.0.A.2.364) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60273 = {S25891, 153, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonMT15i Build/4.0.1.A.0.119) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60272 = {S25892, 151, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15i Build/4.0.A.2.362) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70330 = {S25893, 155, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; SonyEricssonMT15i-o Build/3.0.1.A.0.150) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70421 = {S25894, 153, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonMT15i Build/4.0.1.A.0.284) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72477 = {S25895, 141, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MT15i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72834 = {S25896, 153, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMT15i Build/4.0.1.A.0.227) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74836 = {S25897, 141, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; MT15i Build/3.0.1.A.0.150) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA75460 = {S25898, 139, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; MT15i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76037 = {S25899, 88, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Android (Linux; U; Android 2.3.3; zh-cn; MT15i) AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA90548 = {S25900, 152, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; es-es; SonyEricssonMT15iv Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA196530 = {S25901, 65, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; MT15i Build/3.0.1.A.0.145)*/
const Device UA252017 = {S25902, 152, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMT15i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA379931 = {S25903, 140, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MT15i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA398611 = {S25904, 153, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMT15iv Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625300 = {S25905, 154, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMT15i-o Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626496 = {S25906, 154, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMT15i-o Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA632236 = {S25907, 152, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMT15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633716 = {S25908, 140, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MT15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA656628 = {S25909, 64, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; MT15i Build/4.0.2.A.0.62)*/
const Device UA662124 = {S25910, 152, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonMT15i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA673468 = {S25911, 153, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-za; SonyEricssonMT15iv Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA675141 = {S25912, 128, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MT15i Build/4.0.2.A.0.42) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA705496 = {S25913, 140, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; MT15i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA734056 = {S25914, 128, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MT15i Build/4.0.2.A.0.62) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1902211 = {S25915, 152, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonMT15i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1993510 = {S25916, 133, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MT15i Build/4.0.2.A.0.62) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2020287 = {S25917, 129, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; MT15i Build/3.0.1.A.0.145) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022936 = {S25918, 127, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; MT15i Build/4.0.A.2.368) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2030190 = {S25919, 134, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; MT15i Build/3.0.1.A.0.145) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2030253 = {S25920, 133, { &P15517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MT15i Build/4.0.2.A.0.42) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA363918 = {S25921, 64, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.229 Version/11.61*/
const Device UA380071 = {S25922, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.61*/
const Device UA380835 = {S25923, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.229 Version/11.61*/
const Device UA380986 = {S25924, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.229 Version/11.61*/
const Device UA381096 = {S25925, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.229 Version/11.61*/
const Device UA381097 = {S25926, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.229 Version/11.61*/
const Device UA382148 = {S25927, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.229 Version/11.61*/
const Device UA398590 = {S25928, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5217); ru) Presto/2.10.229 Version/11.61*/
const Device UA398610 = {S25929, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5218); ru) Presto/2.10.229 Version/11.61*/
const Device UA402564 = {S25930, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5161); ru) Presto/2.10.229 Version/11.61*/
const Device UA402570 = {S25931, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.229 Version/11.61*/
const Device UA402567 = {S25932, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5230); ru) Presto/2.10.229 Version/11.61*/
const Device UA422198 = {S25933, 101, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.229 Version/11.61,gzip(gfe) (via translate.google.com)*/
const Device UA422238 = {S25934, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4661); ru) Presto/2.10.229 Version/11.61*/
const Device UA422245 = {S25935, 66, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; ru) Opera 11.61*/
const Device UA423758 = {S25936, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.229 Version/11.61*/
const Device UA437311 = {S25937, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.229 Version/11.61*/
const Device UA458458 = {S25938, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.229 Version/11.61*/
const Device UA587638 = {S25939, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.61*/
const Device UA587859 = {S25940, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.229 Version/11.61*/
const Device UA587924 = {S25941, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03278); ru) Presto/2.10.229 Version/11.61*/
const Device UA588031 = {S25942, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.229 Version/11.61*/
const Device UA588099 = {S25943, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4110); ru) Presto/2.10.229 Version/11.61*/
const Device UA603755 = {S25944, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.229 Version/11.61*/
const Device UA608797 = {S25945, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.229 Version/11.61*/
const Device UA625253 = {S25946, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.229 Version/11.61*/
const Device UA636113 = {S25947, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.61*/
const Device UA642178 = {S25948, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4930); ru) Presto/2.10.229 Version/11.61*/
const Device UA645615 = {S25949, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.229 Version/11.61*/
const Device UA647744 = {S25950, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.229 Version/11.61*/
const Device UA649117 = {S25951, 72, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.61*/
const Device UA651832 = {S25952, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.229 Version/11.61*/
const Device UA653628 = {S25953, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.229 Version/11.61*/
const Device UA656165 = {S25954, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.229 Version/11.61*/
const Device UA661976 = {S25955, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5328); ru) Presto/2.10.229 Version/11.61*/
const Device UA682907 = {S25956, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.229 Version/11.61*/
const Device UA688327 = {S25957, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.61*/
const Device UA691929 = {S25958, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.229 Version/11.61*/
const Device UA708928 = {S25959, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.10.229 Version/11.61*/
const Device UA715463 = {S25960, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.229 Version/11.61*/
const Device UA715970 = {S25961, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.229 Version/11.61*/
const Device UA716130 = {S25962, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.61*/
const Device UA718386 = {S25963, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.229 Version/11.61*/
const Device UA726974 = {S25964, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.229 Version/11.61*/
const Device UA762276 = {S25965, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03780); ru) Presto/2.10.229 Version/11.61*/
const Device UA764506 = {S25966, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.229 Version/11.61*/
const Device UA813480 = {S25967, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5278); ru) Presto/2.10.229 Version/11.61*/
const Device UA813482 = {S25968, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.229 Version/11.61*/
const Device UA895876 = {S25969, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.229 Version/11.61*/
const Device UA902351 = {S25970, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.229 Version/11.61*/
const Device UA912351 = {S25971, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.229 Version/11.61*/
const Device UA1949194 = {S25972, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03755); ru) Presto/2.10.229 Version/11.61*/
const Device UA1958244 = {S25973, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03639); ru) Presto/2.10.229 Version/11.61*/
const Device UA1969906 = {S25974, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5288); ru) Presto/2.10.229 Version/11.61*/
const Device UA1983972 = {S25975, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.229 Version/11.61*/
const Device UA1989791 = {S25976, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5299); ru) Presto/2.10.229 Version/11.61*/
const Device UA1995241 = {S25977, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.10.229 Version/11.61*/
const Device UA2164042 = {S25978, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5667); ru) Presto/2.10.229 Version/11.61*/
const Device UA2177294 = {S25979, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.229 Version/11.61*/
const Device UA2186095 = {S25980, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.10.229 Version/11.61*/
const Device UA2206003 = {S25981, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.10.229 Version/11.61*/
const Device UA2262795 = {S25982, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.229 Version/11.61*/
const Device UA2324946 = {S25983, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03649); ru) Presto/2.10.229 Version/11.61*/
const Device UA2367446 = {S25984, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.229 Version/11.61*/
const Device UA2385145 = {S25985, 74, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.229 Version/11.61,gzip(gfe)*/
const Device UA2386033 = {S25986, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.229 Version/11.61*/
const Device UA2405109 = {S25987, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.229 Version/11.61*/
const Device UA2410034 = {S25988, 87, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03402); ru) Presto/2.10.229 Version/11.61*/
const Device UA2455893 = {S25989, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5993); ru) Presto/2.10.229 Version/11.61*/
const Device UA2460429 = {S25990, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.229 Version/11.61*/
const Device UA2501501 = {S25991, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5966); ru) Presto/2.10.229 Version/11.61*/
const Device UA2596937 = {S25992, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6011); ru) Presto/2.10.229 Version/11.61*/
const Device UA2600100 = {S25993, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.229 Version/11.61*/
const Device UA2666528 = {S25994, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.229 Version/11.61*/
const Device UA2673296 = {S25995, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6081); ru) Presto/2.10.229 Version/11.61*/
const Device UA2686081 = {S25996, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5676); ru) Presto/2.10.229 Version/11.61*/
const Device UA2695400 = {S25997, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5880); ru) Presto/2.10.229 Version/11.61*/
const Device UA2712905 = {S25998, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6083); ru) Presto/2.10.229 Version/11.61*/
const Device UA2727165 = {S25999, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5784); ru) Presto/2.10.229 Version/11.61*/
const Device UA2736848 = {S26000, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6089); ru) Presto/2.10.229 Version/11.61*/
const Device UA2747166 = {S26001, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5977); ru) Presto/2.10.229 Version/11.61*/
const Device UA2748093 = {S26002, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5989); ru) Presto/2.10.229 Version/11.61*/
const Device UA2748495 = {S26003, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6037); ru) Presto/2.10.229 Version/11.61*/
const Device UA2749367 = {S26004, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6054); ru) Presto/2.10.229 Version/11.61*/
const Device UA2784445 = {S26005, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.229 Version/11.61*/
const Device UA2848404 = {S26006, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5981); ru) Presto/2.10.229 Version/11.61*/
const Device UA2857845 = {S26007, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6048); ru) Presto/2.10.229 Version/11.61*/
const Device UA2864153 = {S26008, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 6008); ru) Presto/2.10.229 Version/11.61*/
const Device UA2934804 = {S26009, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5990); ru) Presto/2.10.229 Version/11.61*/
const Device UA3080023 = {S26010, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6683); ru) Presto/2.10.229 Version/11.61*/
const Device UA3104214 = {S26011, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6638); ru) Presto/2.10.229 Version/11.61*/
const Device UA3122184 = {S26012, 86, { &P15364,&P7133,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6639); ru) Presto/2.10.229 Version/11.61*/
const Device UA50156 = {S26013, 87, { &P11639,&P5035,&P3418,&P18092 }};	/*Useragent - BlackBerry9000/4.6.0.167 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102 ips-agent*/
const Device UA2431221 = {S26014, 79, { &P15767,&P17779,&P18369,&P18092 }};	/*Useragent - Opera/9.80 (iPad; Opera Mini/7.0.5/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA412432 = {S26015, 104, { &P15364,&P18110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.45 Safari/535.19*/
const Device UA604616 = {S26016, 112, { &P15364,&P18110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19*/
const Device UA605809 = {S26017, 103, { &P15364,&P18110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19*/
const Device UA607012 = {S26018, 140, { &P15364,&P18110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19,gzip(gfe) (via translate.google.com)*/
const Device UA613225 = {S26019, 148, { &P15364,&P18110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Jolicloud Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Joli OS/1.2 Chromium/18.0.1025.162 Chrome/18.0.1025.162 Safari/535.19*/
const Device UA2217506 = {S26020, 115, { &P15364,&P18110,&P18102,&P18092 }};	/*Useragent - User-Agent: Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19*/
const Device UA2220484 = {S26021, 141, { &P15364,&P18110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.45 Safari/535.19,gzip(gfe) (via translate.google.com)*/
const Device UA2366680 = {S26022, 114, { &P15364,&P18110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.45 Safari/535.19,gzip(gfe)*/
const Device UA2674694 = {S26023, 109, { &P15364,&P18110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/18.0.1410.63 Safari/537.31*/
const Device UA2674803 = {S26024, 106, { &P15364,&P18110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/18.0.1410.63 Safari/537.31*/
const Device UA2674815 = {S26025, 106, { &P15364,&P18110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/18.0.1410.63 Safari/537.31*/
const Device UA2674871 = {S26026, 109, { &P15364,&P18110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/18.0.1410.63 Safari/537.31*/
const Device UA2674886 = {S26027, 106, { &P15364,&P18110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/18.0.1410.63 Safari/537.31*/
const Device UA2714753 = {S26028, 109, { &P15364,&P18110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/18.0.1410.63 Safari/537.31*/
const Device UA2717994 = {S26029, 106, { &P15364,&P18110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/18.0.1410.63 Safari/537.31*/
const Device UA2956932 = {S26030, 111, { &P15364,&P18110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; en-us) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.45 Safari/535.19*/
const Device UA2989010 = {S26031, 123, { &P15364,&P18110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; X11; en-US; Valve Client/0; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA33084 = {S26032, 137, { &P12847,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; M865 Build/HuaweiM865) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48825 = {S26033, 137, { &P12847,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; M865 Build/HuaweiM865) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73090 = {S26034, 137, { &P12847,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; M865 Build/HuaweiM865) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA366079 = {S26035, 144, { &P12847,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; USCCADR3305 Build/HuaweiM865) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368779 = {S26036, 144, { &P12847,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; USCCADR3305 Build/HuaweiM865) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369332 = {S26037, 151, { &P12847,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; USCCADR3305 Build/HuaweiM865) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA369785 = {S26038, 199, { &P12847,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; M865 Build/HuaweiM865) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 androidAppVersion/2.3.1 AndroidWebkitWrapper BillingSupported*/
const Device UA378694 = {S26039, 61, { &P12847,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; M865 Build/HuaweiM865)*/
const Device UA412422 = {S26040, 137, { &P12847,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; M865 Build/HuaweiM865) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA653736 = {S26041, 61, { &P12847,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; M865 Build/HuaweiM865)*/
const Device UA680713 = {S26042, 61, { &P12847,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; M865 Build/HuaweiM865)*/
const Device UA711591 = {S26043, 139, { &P12847,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; M865C Build/HuaweiM865C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA879991 = {S26044, 63, { &P12847,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; M865C Build/HuaweiM865C)*/
const Device UA1946324 = {S26045, 68, { &P12847,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; USCCADR3305 Build/HuaweiM865)*/
const Device UA1992361 = {S26046, 68, { &P12847,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; USCCADR3305 Build/HuaweiM865)*/
const Device UA2019977 = {S26047, 74, { &P12847,&P5629,&P15760,&P18092 }};	/*Useragent - HuaweiM865/M865C153B826 CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 2.3.3)*/
const Device UA3085101 = {S26048, 71, { &P12847,&P5629,&P15760,&P18092 }};	/*Useragent - HuaweiM865C/C378B876 CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 2.3.6)*/
const Device UA1996126 = {S26049, 151, { &P18317,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ASUS Transformer Pad TF300T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2010500 = {S26050, 80, { &P18317,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ASUS Transformer Pad TF300T Build/JRO03C)*/
const Device UA2205669 = {S26051, 152, { &P18317,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ASUS Transformer Pad TF300TG Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2352787 = {S26052, 81, { &P18317,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ASUS Transformer Pad TF300TG Build/JRO03C)*/
const Device UA2384728 = {S26053, 139, { &P18317,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; ASUS Pad TF300T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2464749 = {S26054, 152, { &P18317,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ASUS Transformer Pad TF300TL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2477329 = {S26055, 158, { &P18317,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ASUS Transformer Pad TF300T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695238 = {S26056, 146, { &P18317,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Transformer Pad TF300T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2762454 = {S26057, 81, { &P18317,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ASUS Transformer Pad TF300TL Build/JRO03C)*/
const Device UA2852703 = {S26058, 161, { &P18317,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ASUS Transformer Pad TF300T Build/BlueAndroid@V3.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3044150 = {S26059, 154, { &P18317,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ASUS Transformer Pad TF300T Build/JB@OC_V03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2241438 = {S26060, 91, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - com.google.android.apps.plus/X (Linux; U; Android 4.2; en_US; Nexus 7; Build/JOP23E) (gzip)*/
const Device UA2241439 = {S26061, 88, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - com.google.android.videos/2.2.2(Linux; U; Android 4.2; en_US; Nexus 7 Build/JOP23E) gzip*/
const Device UA2241668 = {S26062, 140, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 7 Build/JOP03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 GSA/2.0.6.X*/
const Device UA2241699 = {S26063, 129, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 7 Build/JOP08B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2241731 = {S26064, 129, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 7 Build/JOP11B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2241738 = {S26065, 129, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 7 Build/JOP09D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2242011 = {S26066, 129, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 7 Build/JVP15S) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2359438 = {S26067, 129, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 7 Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2362748 = {S26068, 58, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2; Nexus 7 Build/JOP40C)*/
const Device UA2387681 = {S26069, 60, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; Nexus 7 Build/JOP40D)*/
const Device UA2390878 = {S26070, 131, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 7 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2404860 = {S26071, 127, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-; Nexus 7 Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2426880 = {S26072, 129, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-; Nexus 7 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2467059 = {S26073, 138, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 7 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467426 = {S26074, 129, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Nexus 7 Build/JOP40D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2513180 = {S26075, 131, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 7 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2515541 = {S26076, 130, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2516750 = {S26077, 59, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; Nexus 7 Build/JDQ39)*/
const Device UA2563304 = {S26078, 137, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2584343 = {S26079, 128, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-; Nexus 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2704049 = {S26080, 131, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2768678 = {S26081, 131, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3079853 = {S26082, 131, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 7 Build/JRN84D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3087050 = {S26083, 133, { &P18385,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 3G Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA401374 = {S26084, 107, { &P15364,&P17285,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA401556 = {S26085, 137, { &P15364,&P17285,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11,gzip(gfe) (via translate.google.com)*/
const Device UA591033 = {S26086, 139, { &P15364,&P17285,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2010435 = {S26087, 100, { &P15364,&P17285,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.70 Safari/535.11*/
const Device UA2259260 = {S26088, 97, { &P15364,&P17285,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.8 (KHTML	 like Gecko) Chrome/17.0.932.0 Safari/535.8*/
const Device UA2425059 = {S26089, 153, { &P15364,&P17285,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/17.0.944.0 Safari/535.8 BitdefenderSafepay/0.2.0.0 (OBK; 17.0.944.0; r396.2011)*/
const Device UA2545367 = {S26090, 135, { &P15364,&P17285,&P17830,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.12 Safari/535.11*/
const Device UA2778003 = {S26091, 160, { &P15364,&P17285,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/17.0.944.0 Safari/535.8 BitdefenderSafepay/0.2.0.0 (OBK; 17.0.944.0; r396.2011)*/
const Device UA2790798 = {S26092, 128, { &P15364,&P17285,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.84 Safari/535.11 SE 2.X MetaSr 1.0,gzip(gfe)*/
const Device UA412585 = {S26093, 119, { &P17595,&P17297,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.45 Safari/535.19*/
const Device UA427946 = {S26094, 119, { &P17595,&P17297,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.54 Safari/535.19*/
const Device UA457869 = {S26095, 120, { &P17595,&P17297,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/180.1.1025.40 Safari/535.19*/
const Device UA614928 = {S26096, 120, { &P17595,&P17297,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA614992 = {S26097, 120, { &P17595,&P17297,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA623181 = {S26098, 120, { &P17595,&P17297,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA849232 = {S26099, 120, { &P17595,&P17297,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/180.1.1025.40 Safari/535.19*/
const Device UA2202054 = {S26100, 120, { &P17595,&P17297,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA2477229 = {S26101, 120, { &P17595,&P17297,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/180.1.1025.40 Safari/535.19*/
const Device UA399121 = {S26102, 118, { &P17595,&P17297,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.20 (KHTML, like Gecko) Chrome/19.0.1036.7 Safari/535.20*/
const Device UA820591 = {S26103, 118, { &P17595,&P17297,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.24 (KHTML, like Gecko) Chrome/19.0.1055.1 Safari/535.24*/
const Device UA897618 = {S26104, 117, { &P17595,&P17297,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5*/
const Device UA900842 = {S26105, 117, { &P17595,&P17297,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA901793 = {S26106, 117, { &P17595,&P17297,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA907953 = {S26107, 156, { &P17595,&P17297,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2153121 = {S26108, 117, { &P17595,&P17297,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA2360764 = {S26109, 143, { &P17595,&P17297,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5*/
const Device UA2391579 = {S26110, 143, { &P17595,&P17297,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5*/
const Device UA2422303 = {S26111, 143, { &P17595,&P17297,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5*/
const Device UA24001 = {S26112, 137, { &P15649,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; GT-S5570L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42455 = {S26113, 154, { &P15649,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; GT-S5570-ORANGE/S5570BVKE1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42456 = {S26114, 154, { &P15649,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5570-ORANGE/S5570BVKE2 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42471 = {S26115, 154, { &P15649,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pl-pl; GT-S5570-ORANGE/S5570BVKC4 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47987 = {S26116, 137, { &P15649,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; GT-S5570B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58338 = {S26117, 154, { &P15649,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ro-ro; GT-S5570-ORANGE/S5570BVKC3 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60926 = {S26118, 137, { &P15649,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; GT-S5570B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA64730 = {S26119, 136, { &P15649,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA181432 = {S26120, 60, { &P15649,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-S5570 Build/FROYO)*/
const Device UA397328 = {S26121, 124, { &P15649,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5570 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA450355 = {S26122, 61, { &P15649,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-S5570L Build/FROYO)*/
const Device UA1894041 = {S26123, 61, { &P15649,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-S5570B Build/FROYO)*/
const Device UA2019401 = {S26124, 97, { &P15649,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.2.1;Zh_cn;GT-S5570;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA2019673 = {S26125, 134, { &P15649,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020711 = {S26126, 137, { &P15649,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; GT-S5570L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027607 = {S26127, 129, { &P15649,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-S5570 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA252520 = {S26128, 88, { &P18095,&P18109,&P17780,&P18092 }};	/*Useragent - Opera/9.80 (Series 60; Opera Mini/6.1.26266/26.1069; U; en) Presto/2.8.119 Version/10.54*/
const Device UA2431344 = {S26129, 88, { &P18095,&P18109,&P17780,&P18092 }};	/*Useragent - Opera/9.80 (Series 60; Opera Mini/6.1.25570/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2387789 = {S26130, 69, { &P15364,&P18118,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:17.0) Gecko/X Firefox/17.0*/
const Device UA2018034 = {S26131, 64, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.10.289 Version/12.02*/
const Device UA2041196 = {S26132, 87, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.10.289 Version/12.02*/
const Device UA2184133 = {S26133, 87, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.10.289 Version/12.02*/
const Device UA2186586 = {S26134, 123, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0); en) Presto/2.10.289 Version/12.02*/
const Device UA2186893 = {S26135, 86, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5972); ru) Presto/2.10.289 Version/12.02*/
const Device UA2192618 = {S26136, 86, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5970); ru) Presto/2.10.289 Version/12.02*/
const Device UA2193414 = {S26137, 116, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0); en) Presto/2.10.289 Version/12.02*/
const Device UA2196265 = {S26138, 86, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4953); ru) Presto/2.10.289 Version/12.02*/
const Device UA2196702 = {S26139, 87, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5310); ru) Presto/2.10.289 Version/12.02*/
const Device UA2197879 = {S26140, 87, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.10.289 Version/12.02*/
const Device UA2197877 = {S26141, 87, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5339); ru) Presto/2.10.289 Version/12.02*/
const Device UA2199278 = {S26142, 86, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5831); ru) Presto/2.10.289 Version/12.02*/
const Device UA2200587 = {S26143, 86, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5754); ru) Presto/2.10.289 Version/12.02*/
const Device UA2201559 = {S26144, 101, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; pl) Presto/2.10.289 Version/12.02,gzip(gfe) (via translate.google.com)*/
const Device UA2201680 = {S26145, 87, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.4 (build 02647); ru) Presto/2.10.289 Version/12.02*/
const Device UA2205984 = {S26146, 86, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5976); ru) Presto/2.10.289 Version/12.02*/
const Device UA2226835 = {S26147, 86, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5711); ru) Presto/2.10.289 Version/12.02*/
const Device UA2229325 = {S26148, 66, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; en) Opera 12.02*/
const Device UA2230471 = {S26149, 86, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5680); ru) Presto/2.10.289 Version/12.02*/
const Device UA2258853 = {S26150, 72, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; en; rv:2.0) Gecko/X Firefox/4.0 Opera 12.02*/
const Device UA2262560 = {S26151, 87, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.6 (build 03278); ru) Presto/2.10.289 Version/12.02*/
const Device UA2262588 = {S26152, 86, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4157); ru) Presto/2.10.289 Version/12.02*/
const Device UA2262619 = {S26153, 87, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03686); ru) Presto/2.10.289 Version/12.02*/
const Device UA2341635 = {S26154, 102, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; compatible; MSIE 7.0; Windows NT 6.0; bg) Presto/2.10.289 Version/12.02*/
const Device UA2342394 = {S26155, 87, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03790); ru) Presto/2.10.289 Version/12.02*/
const Device UA2349884 = {S26156, 86, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4133); ru) Presto/2.10.289 Version/12.02*/
const Device UA2355364 = {S26157, 74, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; pl) Presto/2.10.289 Version/12.02,gzip(gfe)*/
const Device UA2362640 = {S26158, 86, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 6001); ru) Presto/2.10.289 Version/12.02*/
const Device UA2369295 = {S26159, 65, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/12.02 (Windows NT 6.0; U; en) Presto/2.10.289 Version/12.02*/
const Device UA2415274 = {S26160, 86, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4876); ru) Presto/2.10.289 Version/12.02*/
const Device UA2456248 = {S26161, 86, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5998); ru) Presto/2.10.289 Version/12.02*/
const Device UA2526481 = {S26162, 86, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 8.0 (build 5745); ru) Presto/2.10.289 Version/12.02*/
const Device UA2555138 = {S26163, 86, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 6005); ru) Presto/2.10.289 Version/12.02*/
const Device UA2592915 = {S26164, 87, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5244); ru) Presto/2.10.289 Version/12.02*/
const Device UA2693481 = {S26165, 86, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4598); ru) Presto/2.10.289 Version/12.02*/
const Device UA2729238 = {S26166, 86, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 6015); ru) Presto/2.10.289 Version/12.02*/
const Device UA2744841 = {S26167, 86, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4664); ru) Presto/2.10.289 Version/12.02*/
const Device UA2749957 = {S26168, 86, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 6068); ru) Presto/2.10.289 Version/12.02*/
const Device UA2820410 = {S26169, 87, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5328); ru) Presto/2.10.289 Version/12.02*/
const Device UA2822074 = {S26170, 87, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.3 (build 02564); ru) Presto/2.10.289 Version/12.02*/
const Device UA2824684 = {S26171, 87, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5231); ru) Presto/2.10.289 Version/12.02*/
const Device UA2831257 = {S26172, 87, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.6 (build 03402); ru) Presto/2.10.289 Version/12.02*/
const Device UA2981279 = {S26173, 87, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.4 (build 02652); ru) Presto/2.10.289 Version/12.02*/
const Device UA3070526 = {S26174, 86, { &P15364,&P17285,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5993); ru) Presto/2.10.289 Version/12.02*/
const Device UA1995071 = {S26175, 58, { &P15364,&P16110,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; rv:16.0) Gecko/X Firefox/16.0*/
const Device UA2260188 = {S26176, 69, { &P15364,&P16110,&P18428,&P18092 }};	/*Useragent - Mozilla/6.0 (Windows NT 6.2; WOW64; rv:16.0.1) Gecko/X Firefox/16.0.1*/
const Device UA2367594 = {S26177, 74, { &P15364,&P16110,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:16.0.1) Gecko/X Firefox/16.0.1*/
const Device UA4389 = {S26178, 63, { &P15364,&P7133,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:2.0) Gecko/20100101 Firefox/4.0*/
const Device UA552132 = {S26179, 81, { &P15364,&P7133,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:2.0b10) Gecko/X Firefox/4.0b10*/
const Device UA702743 = {S26180, 72, { &P15364,&P7133,&P17203,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows NT 6.1); Gecko/X Firefox/4.0.1; 10; Windows NT 6.1)*/
const Device UA729544 = {S26181, 68, { &P15364,&P7133,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.0) Gecko/X Firefox/4.0*/
const Device UA783644 = {S26182, 97, { &P15364,&P7133,&P17203,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows NT 6.1); Gecko/X Firefox/4.0.1; 10; Windows NT 6.1; Trident/4.0; InfoPath.2)*/
const Device UA813095 = {S26183, 81, { &P15364,&P7133,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0.1) Gecko/X Firefox/4.0.1 Firefox/4.0.1*/
const Device UA2409395 = {S26184, 85, { &P15364,&P7133,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1); Gecko/X Firefox/4.0.1; 10; Windows NT 6.1; Trident/6.0)*/
const Device UA2807393 = {S26185, 48, { &P15364,&P7133,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) Gecko/X Firefox/4.0*/
const Device UA2979704 = {S26186, 91, { &P15364,&P7133,&P17203,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Windows NT 6.1; Win64; x64; rv:2.0b8pre; Gecko/X Firefox/4.0b8pre)*/
const Device UA2015625 = {S26187, 99, { &P15364,&P17285,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2015628 = {S26188, 106, { &P15364,&P17285,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2040569 = {S26189, 136, { &P15364,&P17285,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)*/
const Device UA2203080 = {S26190, 138, { &P15364,&P17285,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2204425 = {S26191, 145, { &P15364,&P17285,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2385086 = {S26192, 109, { &P15364,&P17285,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe)*/
const Device UA2725799 = {S26193, 112, { &P15364,&P17285,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Neustar WPM) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1*/
const Device UA2889119 = {S26194, 118, { &P15364,&P17285,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/21.0.597.0 Safari/534.13*/
const Device UA2740995 = {S26195, 88, { &P15364,&P7133,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:22.0) Gecko/X Firefox/22.0 AlexaToolbar/alxf-2.18*/
const Device UA2947716 = {S26196, 84, { &P15364,&P7133,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:22.0) Gecko/X Firefox/22.0 IceDragon/22.0.0.1*/
const Device UA3112320 = {S26197, 79, { &P15364,&P7133,&P22170,&P18092 }};	/*Useragent - <![CDATA[Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:22.0) Gecko/X Firefox/22.0*/
const Device UA459272 = {S26198, 66, { &P15991,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S6102 Build/GINGERBREAD)*/
const Device UA459292 = {S26199, 130, { &P15991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA459299 = {S26200, 142, { &P15991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA459304 = {S26201, 143, { &P15991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-S6102B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027496 = {S26202, 143, { &P15991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6102E Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029972 = {S26203, 67, { &P15991,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S6102B Build/GINGERBREAD)*/
const Device UA2164745 = {S26204, 140, { &P15991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2520897 = {S26205, 136, { &P15991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6102E Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2814827 = {S26206, 135, { &P15991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA23736 = {S26207, 78, { &P11639,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry9000/5.0.0.1067 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102*/
const Device UA2888648 = {S26208, 88, { &P15364,&P7133,&P22171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:23.0) Gecko/X Firefox/23.0 AlexaToolbar/alxf-2.18*/
const Device UA3093221 = {S26209, 82, { &P15364,&P7133,&P22171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.1; Win64; x64; rv:23.0) Gecko/23.0 Firefox/23.0*/
const Device UA3111795 = {S26210, 79, { &P15364,&P7133,&P22171,&P18092 }};	/*Useragent - <![CDATA[Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:23.0) Gecko/X Firefox/23.0*/
const Device UA76072 = {S26211, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.3; en-us; C771 Build/C771M070) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76074 = {S26212, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M070) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA247755 = {S26213, 135, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M070) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA371018 = {S26214, 68, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.3; en_US; C771 Build/GINGERBREAD)*/
const Device UA382237 = {S26215, 135, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA613038 = {S26216, 135, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA631650 = {S26217, 135, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M080) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA857810 = {S26218, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.3; en-us; C771 Build/C771M100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA891877 = {S26219, 142, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M110) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA925298 = {S26220, 135, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M120) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1904227 = {S26221, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;C771 Build/C771M070) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1954742 = {S26222, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;C771 Build/C771M100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1961232 = {S26223, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771Build/C771M100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1969502 = {S26224, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; C771 Build/C771M100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020268 = {S26225, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771Build/C771M070) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021336 = {S26226, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; C771 Build/C771M080) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022488 = {S26227, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; C771 Build/C771M070) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025940 = {S26228, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; C771 Build/C771M110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026827 = {S26229, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771Build/C771M110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028460 = {S26230, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA2029201 = {S26231, 142, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M120) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2029477 = {S26232, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;C771 Build/C771M120) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029923 = {S26233, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.3; en-us; C771 Build/C771M120) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030500 = {S26234, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA2030702 = {S26235, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;C771 Build/C771M110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030723 = {S26236, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.3; en-us; C771 Build/C771M110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2235593 = {S26237, 135, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2238916 = {S26238, 59, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; C771 Build/C771M130)*/
const Device UA2402919 = {S26239, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;C771 Build/C771M130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2480576 = {S26240, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771Build/C771M130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2480752 = {S26241, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; C771 Build/C771M130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2526934 = {S26242, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.3; en-us; C771 Build/C771M130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2554124 = {S26243, 134, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA3127083 = {S26244, 135, { &P17927,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; C771 Build/C771M140) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369566 = {S26245, 149, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; XT910 Build/6.7.1_SPU-ICS-DUMMY) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA912496 = {S26246, 150, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MOT-XT910 Build/6.7.3-94_SPI-324) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1972754 = {S26247, 158, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MOT-XT910 Build/6.7.2-180_SPU-19-TA-11.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1979455 = {S26248, 154, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MOT-XT910 Build/6.7.2-180_SLC-29_R01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1987253 = {S26249, 146, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT910 Build/6.7.3-94_SPI-324) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1990601 = {S26250, 150, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MOT-XT910 Build/6.7.2-180_SLC-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1998538 = {S26251, 150, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MOT-XT910 Build/6.7.3-94_SPI-328) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2002953 = {S26252, 146, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT910 Build/6.7.3-94_SPI-328) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2006926 = {S26253, 154, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2009035 = {S26254, 154, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; MOT-XT910 Build/6.7.2-180_SLC-35_R02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2009718 = {S26255, 158, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOT-XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025089 = {S26256, 141, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT910 Build/6.7.2_GC-145-SPDU-10) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2026610 = {S26257, 154, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MOT-XT910 Build/6.7.2_GC-145-SPDU-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027329 = {S26258, 146, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT910 Build/6.7.2-180_SLC-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027366 = {S26259, 150, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MOT-XT910 Build/6.7.2-180_SLC-34) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027639 = {S26260, 155, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOT-XT910 Build/6.7.6-226_SPU-DE-TA-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028342 = {S26261, 151, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XT910 Build/6.7.6-226_SPU-DE-TA-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029112 = {S26262, 149, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; MOT-XT910K Build/6.7.4-27_SA-17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030374 = {S26263, 155, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOT-XT910 Build/6.7.2-180_SPU-19-TA-5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030533 = {S26264, 150, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; XT910 Build/6.7.2-180_SLC-35_R02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032294 = {S26265, 150, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MOT-XT910 Build/6.7.2-180_SLC-29) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033695 = {S26266, 154, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT910 Build/6.7.2-180_SPU-19-TA-11.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034298 = {S26267, 149, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; MOT-XT910S Build/6.7.4-27_SA-17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2177626 = {S26268, 146, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT910 Build/6.7.2-180_SLC-35) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2177850 = {S26269, 76, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; XT910 Build/6.7.2-180_SPU-19-TA-11.6)*/
const Device UA2187783 = {S26270, 150, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MOT-XT910 Build/6.7.2-180_SLC-35) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2218797 = {S26271, 151, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XT910 Build/6.7.2-180_SPU-19-TA-5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2220850 = {S26272, 146, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT910 Build/6.7.2-180_SLC-34) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257439 = {S26273, 167, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MOT-XT910 Build/ULTIMOBILE-RAZRME-4.0.4-MTP-X-2.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257471 = {S26274, 156, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; MOT-XT910 Build/6.7.2-180_SPU-19TA11.9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2343705 = {S26275, 154, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MOT-XT910 Build/6.7.2-180_SLC-34_R01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2345893 = {S26276, 154, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MOT-XT910 Build/6.7.2-180_SLC-35_R01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2349982 = {S26277, 153, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MOT-XT910 Build/6.7.2_GC-213-SPDU-6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2351667 = {S26278, 154, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MOT-XT910 Build/6.7.2-180_SLC-29_R03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2355189 = {S26279, 154, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MOT-XT910 Build/6.7.2-180_SLC-31_R01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2355452 = {S26280, 150, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; MOT-XT910 Build/6.7.3-94_SPI-329) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2365205 = {S26281, 150, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; XT910 Build/6.7.2-180_SLC-29_R01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2397594 = {S26282, 137, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT910 Build/6.7.3-94_SPI-328) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2401584 = {S26283, 152, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; XT910 Build/6.7.2-180_SPU-19TA11.9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2413882 = {S26284, 156, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MOT-XT910 Build/6.7.2_GC-145-SPDUF-910) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2414057 = {S26285, 136, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT910 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2419258 = {S26286, 165, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MOT-XT910 Build/ULTIMOBILE-RAZR-4.0.4-VIA-X-1.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423413 = {S26287, 149, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; MOT-XT910K Build/6.7.4-27_SA-46) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2475750 = {S26288, 143, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT910 Build/6.7.2_GC-145-SPDUF-910) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2477202 = {S26289, 146, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; XT910 Build/6.7.3-94_SPI-329) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2477934 = {S26290, 140, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT910 Build/6.7.2_GC-213-SPDU-6) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2479092 = {S26291, 156, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MOT-XT910 Build/6.7.2_GC-213-SPDUF-906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2481009 = {S26292, 149, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; MOT-XT910S Build/6.7.4-27_SA-51) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2502532 = {S26293, 150, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT910 Build/6.7.2-180_SLC-35_R01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2552984 = {S26294, 150, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT910 Build/6.7.2-180_SLC-29_R03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2575642 = {S26295, 142, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT910 Build/6.7.2-180_SPU-19-TA-5) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2641916 = {S26296, 150, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOT-XT910 Build/6.7.2-152_SPU-17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2642855 = {S26297, 136, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; XT910 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2651624 = {S26298, 163, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT910 Build/ULTIMOBILE-RAZRME-4.0.4-MTP-X-2.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2693848 = {S26299, 145, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; XT910K Build/6.7.4-27_SA-46) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2694317 = {S26300, 150, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT910 Build/6.7.2-180_SLC-34_R01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2727109 = {S26301, 150, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; XT910 Build/6.7.2-180_SLC-31_R01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2744894 = {S26302, 149, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT910 Build/6.7.2_GC-213-SPDU-6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2790500 = {S26303, 161, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT910 Build/ULTIMOBILE-RAZR-4.0.4-VIA-X-1.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2816972 = {S26304, 146, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT910 Build/6.7.2-180_SLC-29) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2821772 = {S26305, 149, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOT-XT910 Build/6.7.3-94_SPI-32) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2827253 = {S26306, 152, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT910 Build/6.7.2_GC-213-SPDUF-906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2831255 = {S26307, 143, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT910 Build/6.7.2_GC-213-SPDUF-906) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2889073 = {S26308, 158, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MOT-XT910 Build/6.7.2-180_SPU-19-TA-11.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950586 = {S26309, 156, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-; MOT-XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2980646 = {S26310, 152, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT910 Build/6.7.2_GC-145-SPDUF-910) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2988754 = {S26311, 165, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MOT-XT910 Build/ULTIMOBILE-RAZR-4.0.4-VIA-X-1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3042368 = {S26312, 150, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; MOT-XT910 Build/6.7.3-94_SPI-326) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3085457 = {S26313, 165, { &P15877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MOT-XT910 Build/ULTIMOBILE-RAZR-4.0.4-MTP-X-1.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA161957 = {S26314, 62, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.7.62 Version/11.00*/
const Device UA164985 = {S26315, 99, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.7.62 Version/11.00,gzip(gfe) (via translate.google.com)*/
const Device UA165324 = {S26316, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.7.62 Version/11.00*/
const Device UA166167 = {S26317, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.7.62 Version/11.00*/
const Device UA166285 = {S26318, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.7.62 Version/11.00*/
const Device UA169550 = {S26319, 62, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; en) Presto/2.7.62 Version/11.01*/
const Device UA170385 = {S26320, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.7.62 Version/11.01*/
const Device UA171296 = {S26321, 99, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.7.62 Version/11.01,gzip(gfe) (via translate.google.com)*/
const Device UA171428 = {S26322, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.7.62 Version/11.01*/
const Device UA232513 = {S26323, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.7.62 Version/11.00*/
const Device UA296671 = {S26324, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.7.62 Version/11.00*/
const Device UA298371 = {S26325, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.7.62 Version/11.00*/
const Device UA301217 = {S26326, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.7.62 Version/11.01*/
const Device UA301220 = {S26327, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.7.62 Version/11.01*/
const Device UA302059 = {S26328, 62, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; ru) Presto/2.7.62 Version/11.01*/
const Device UA302156 = {S26329, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.7.62 Version/11.01*/
const Device UA303877 = {S26330, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.7.62 Version/11.01*/
const Device UA304452 = {S26331, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.7.62 Version/11.01*/
const Device UA304497 = {S26332, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.7.62 Version/11.01*/
const Device UA329510 = {S26333, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.7.62 Version/11.00*/
const Device UA338075 = {S26334, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.7.62 Version/11.01*/
const Device UA374109 = {S26335, 62, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; ru) Presto/2.7.62 Version/11.00*/
const Device UA380650 = {S26336, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.7.62 Version/11.00*/
const Device UA388811 = {S26337, 66, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; en) Opera 11.01*/
const Device UA394012 = {S26338, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.7.62 Version/11.01*/
const Device UA396886 = {S26339, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.7.62 Version/11.00*/
const Device UA396924 = {S26340, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.7.62 Version/11.00*/
const Device UA409265 = {S26341, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.7.62 Version/11.01*/
const Device UA432778 = {S26342, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.7.62 Version/11.00*/
const Device UA435467 = {S26343, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.7.62 Version/11.00*/
const Device UA444603 = {S26344, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.7.62 Version/11.00*/
const Device UA444667 = {S26345, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.7.62 Version/11.00*/
const Device UA515289 = {S26346, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.7.62 Version/11.01*/
const Device UA518459 = {S26347, 111, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94; ru) Presto/2.7.62 Version/11.01*/
const Device UA519153 = {S26348, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 3797); ru) Presto/2.7.62 Version/11.01*/
const Device UA532786 = {S26349, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.7.62 Version/11.01*/
const Device UA536601 = {S26350, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.7.62 Version/11.00*/
const Device UA541662 = {S26351, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.7.62 Version/11.00*/
const Device UA546650 = {S26352, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.7.62 Version/11.00*/
const Device UA558359 = {S26353, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.7.62 Version/11.01*/
const Device UA607483 = {S26354, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.7.62 Version/11.00*/
const Device UA609375 = {S26355, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.7.62 Version/11.01*/
const Device UA632595 = {S26356, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.7.62 Version/11.01*/
const Device UA649935 = {S26357, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.7.62 Version/11.00*/
const Device UA693386 = {S26358, 81, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; de; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 11.00*/
const Device UA694817 = {S26359, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.7.62 Version/11.01*/
const Device UA726446 = {S26360, 66, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; pl) Opera 11.00*/
const Device UA741888 = {S26361, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.7.62 Version/11.01*/
const Device UA750800 = {S26362, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.7.62 Version/11.01*/
const Device UA759109 = {S26363, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.7.62 Version/11.01*/
const Device UA794018 = {S26364, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.7.62 Version/11.00*/
const Device UA794491 = {S26365, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.7.62 Version/11.01*/
const Device UA817402 = {S26366, 81, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 11.01*/
const Device UA820048 = {S26367, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.7.62 Version/11.01*/
const Device UA837635 = {S26368, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.7.62 Version/11.01*/
const Device UA868981 = {S26369, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.7.62 Version/11.00*/
const Device UA869078 = {S26370, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.7.62 Version/11.00*/
const Device UA896818 = {S26371, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.7.62 Version/11.01*/
const Device UA1857147 = {S26372, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.7.62 Version/11.00*/
const Device UA1857398 = {S26373, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.7.62 Version/11.00*/
const Device UA1864393 = {S26374, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.7.62 Version/11.01*/
const Device UA1905394 = {S26375, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.7.62 Version/11.01*/
const Device UA1956806 = {S26376, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5218); ru) Presto/2.7.62 Version/11.01*/
const Device UA1960182 = {S26377, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.7.62 Version/11.00*/
const Device UA1962134 = {S26378, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.7.62 Version/11.00*/
const Device UA1963895 = {S26379, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.7.62 Version/11.01*/
const Device UA1989050 = {S26380, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.7.62 Version/11.00*/
const Device UA2146816 = {S26381, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4841); ru) Presto/2.7.62 Version/11.00*/
const Device UA2161418 = {S26382, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.7.62 Version/11.01*/
const Device UA2163152 = {S26383, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.7.62 Version/11.00*/
const Device UA2167540 = {S26384, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.7.62 Version/11.00*/
const Device UA2167915 = {S26385, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.7.62 Version/11.00*/
const Device UA2184855 = {S26386, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.7.62 Version/11.00*/
const Device UA2215499 = {S26387, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4871); ru) Presto/2.7.62 Version/11.00*/
const Device UA2219740 = {S26388, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5328); ru) Presto/2.7.62 Version/11.00*/
const Device UA2220259 = {S26389, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.7.62 Version/11.01*/
const Device UA2230449 = {S26390, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.7.62 Version/11.01*/
const Device UA2262421 = {S26391, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.7.62 Version/11.00*/
const Device UA2262507 = {S26392, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.7.62 Version/11.00*/
const Device UA2273095 = {S26393, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03791); ru) Presto/2.7.62 Version/11.00*/
const Device UA2332514 = {S26394, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5267); ru) Presto/2.7.62 Version/11.00*/
const Device UA2333296 = {S26395, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5328); ru) Presto/2.7.62 Version/11.01*/
const Device UA2338120 = {S26396, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.7.62 Version/11.01*/
const Device UA2403029 = {S26397, 72, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.7.62 Version/11.00,gzip(gfe)*/
const Device UA2417190 = {S26398, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.7.62 Version/11.01*/
const Device UA2441574 = {S26399, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.7.62 Version/11.01*/
const Device UA2443534 = {S26400, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.7.62 Version/11.01*/
const Device UA2454119 = {S26401, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.7.62 Version/11.01*/
const Device UA2466938 = {S26402, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.7.62 Version/11.00*/
const Device UA2480276 = {S26403, 72, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.7.62 Version/11.01,gzip(gfe)*/
const Device UA2518297 = {S26404, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5267); ru) Presto/2.7.62 Version/11.01*/
const Device UA2528168 = {S26405, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.7.62 Version/11.00*/
const Device UA2554007 = {S26406, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.7.62 Version/11.00*/
const Device UA2557657 = {S26407, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.7.62 Version/11.00*/
const Device UA2574097 = {S26408, 85, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5282); ru) Presto/2.7.62 Version/11.00*/
const Device UA2653272 = {S26409, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.7.62 Version/11.01*/
const Device UA2702827 = {S26410, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.7.62 Version/11.01*/
const Device UA2732779 = {S26411, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.7.62 Version/11.00*/
const Device UA2809881 = {S26412, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5667); ru) Presto/2.7.62 Version/11.00*/
const Device UA2844011 = {S26413, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.7.62 Version/11.01*/
const Device UA2846735 = {S26414, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.7.62 Version/11.01*/
const Device UA2993734 = {S26415, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6089); ru) Presto/2.7.62 Version/11.00*/
const Device UA3121679 = {S26416, 84, { &P15364,&P5690,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6578); ru) Presto/2.7.62 Version/11.00*/
const Device UA454542 = {S26417, 95, { &P15364,&P5690,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-GB; rv:1.8.1.6) Gecko/X Firefox/2.0	Firefox 2.0.0.6*/
const Device UA512019 = {S26418, 81, { &P15364,&P5690,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.8.1a2) Gecko/X BonEcho/2.0a2*/
const Device UA638291 = {S26419, 134, { &P15364,&P5690,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.20) Gecko/X Firefox/2.0.0.20 GTB7.0 (.NET CLR 3.5.30729) WinNT-PAI 21.07.2009*/
const Device UA691182 = {S26420, 81, { &P15364,&P5690,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; WindowsNT 5.1; en-US; rv1.8.1.6) Gecko/X Firefox/2.0.0.6*/
const Device UA725625 = {S26421, 64, { &P15364,&P5690,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1) Gecko/X Firefox/2.0.0.3*/
const Device UA736531 = {S26422, 127, { &P15364,&P5690,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.20) Gecko/X Firefox/2.0.0.20 (.NET CLR 3.5.30729) WinNT-EVI 18.05.2010*/
const Device UA842757 = {S26423, 115, { &P15364,&P5690,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U;XMPP Tiscali Communicator v.10.0.1; Windows NT 5.1; it; rv:1.8.1.3) Gecko/X Firefox/2.0.0.3*/
const Device UA871562 = {S26424, 85, { &P15364,&P5690,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-GB; rv:1.8.1.13) Gecko/X Firefox/2.0.0.13*/
const Device UA1893049 = {S26425, 81, { &P15364,&P5690,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1b2) Gecko/X BonEcho/2.0b2*/
const Device UA2208164 = {S26426, 105, { &P15364,&P5690,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (http://vx.netlux.org/ Windows; U; Windows NT 5.1; en-US; rv:1.8.1.9) Gecko/X Firefox/2.0.0.9*/
const Device UA2219458 = {S26427, 82, { &P15364,&P5690,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0+(Windows;+U;+Windows+NT+5.1;en-US;+rv:1.8.1.6)+Gecko/X+Firefox/2.0.0.6*/
const Device UA2473717 = {S26428, 58, { &P15364,&P5690,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:9.0.1) Gecko/X Firefox/2.0*/
const Device UA2502473 = {S26429, 114, { &P15364,&P5690,&P17202,&P18092 }};	/*Useragent - ('Mozilla/5.0 (Windows; U; Windows NT 5.1; en-GB; rv:1.8.1.6) Gecko/X Firefox/2.0.0.6', 'Firefox 2.0, Windows XP')*/
const Device UA2956194 = {S26430, 83, { &P15364,&P5690,&P17202,&P18092 }};	/*Useragent - Mozilla/6.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.1) Gecko/X Firefox/2.0.0.1*/
const Device UA2766203 = {S26431, 59, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC One Build/JDQ39)*/
const Device UA2784912 = {S26432, 137, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC_One Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2799388 = {S26433, 137, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC One Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2870618 = {S26434, 141, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC_PN07120 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879421 = {S26435, 138, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892385 = {S26436, 138, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; HTC_One Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2895720 = {S26437, 142, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-tw; HTC One 801e Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937554 = {S26438, 64, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC One 801e Build/JDQ39)*/
const Device UA2957874 = {S26439, 139, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; HTC_PN071 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957931 = {S26440, 153, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC_PN07120/1.26.502.10 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958372 = {S26441, 56, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; One Build/JZO54K)*/
const Device UA2972874 = {S26442, 148, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC_One/2.24.161.1 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2974943 = {S26443, 159, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; HTC_PN071-orange-LS/2.24.73.1 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999622 = {S26444, 61, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC_PN071 Build/JDQ39)*/
const Device UA3001961 = {S26445, 153, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC_PN07120/1.26.502.12 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3013435 = {S26446, 134, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022090 = {S26447, 159, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; HTC_PN071-orange-LS/2.24.75.1 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022153 = {S26448, 159, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; pl-pl; HTC_PN071-orange-LS/2.24.69.1 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022228 = {S26449, 148, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; HTC_One/2.24.163.2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3034169 = {S26450, 159, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; HTC_PN071-orange-LS/2.24.73.3 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3036909 = {S26451, 148, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; de-de; HTC_One/2.24.111.3 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050187 = {S26452, 135, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-; HTC One Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3090035 = {S26453, 134, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3094136 = {S26454, 159, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; HTC_PN071-orange-LS/2.24.73.8 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3124892 = {S26455, 146, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ru-ru; HTC One dual sim Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3126246 = {S26456, 138, { &P20705,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; HTC One Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2191109 = {S26457, 142, { &P18582,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT28at Build/6.1.C.1.105) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2193409 = {S26458, 154, { &P18582,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyEricssonLT28at Build/6.1.C.1.105) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2203499 = {S26459, 152, { &P18582,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SonyEricssonLT28i Build/6.1.E.1.19) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2223515 = {S26460, 141, { &P18582,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT28h Build/6.1.E.0.233) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2223690 = {S26461, 152, { &P18582,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyEricssonLT28h Build/6.1.E.1.20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2235692 = {S26462, 140, { &P18582,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LT28i Build/6.1.E.1.19) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2239158 = {S26463, 64, { &P18582,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT28at Build/6.1.C.1.105)*/
const Device UA2352366 = {S26464, 140, { &P18582,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; LT28at Build/6.1.C.1.105) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2353919 = {S26465, 140, { &P18582,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT28h Build/6.1.E.1.20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2458885 = {S26466, 61, { &P18582,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT28h Build/6.1.E.3.7)*/
const Device UA2504392 = {S26467, 140, { &P18582,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; LT28h Build/6.1.E.2.68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2506636 = {S26468, 139, { &P18582,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT28h Build/6.1.E.3.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2526502 = {S26469, 132, { &P18582,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT28h Build/6.1.E.0.233) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2527925 = {S26470, 140, { &P18582,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LT28i Build/6.1.E.2.68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2551858 = {S26471, 131, { &P18582,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT28h Build/6.1.E.1.20) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2656727 = {S26472, 61, { &P18582,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT28i Build/6.1.E.3.7)*/
const Device UA2697373 = {S26473, 139, { &P18582,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LT28i Build/6.1.E.3.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2698477 = {S26474, 130, { &P18582,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT28h Build/6.1.E.3.7) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2700424 = {S26475, 139, { &P18582,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT28i Build/6.1.E.3.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708471 = {S26476, 142, { &P18582,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT28at Build/6.1.C.1.111) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3109289 = {S26477, 174, { &P18582,&P18066,&P18062,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.0.4;dev=LT28at;devm=LT28at;man=Sony Ericsson;brw=Android;shwd=720x1280;shwa=720x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA79941 = {S26478, 82, { &P15364,&P17285,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/12*/
const Device UA198098 = {S26479, 107, { &P15364,&P17285,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.91 Safari/534.30*/
const Device UA219868 = {S26480, 101, { &P15364,&P17285,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA327433 = {S26481, 138, { &P15364,&P17285,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30,gzip(gfe) (via translate.google.com)*/
const Device UA2168230 = {S26482, 127, { &P15364,&P17285,&P17251,&P18092 }};	/*Useragent - MMozilla/5.0 (Windows NT 6.0) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30 ChromePlus/1.6.3.0alpha4*/
const Device UA2259158 = {S26483, 105, { &P15364,&P17285,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.27 (KHTML, like Gecko) Chrome/12.0.712.0 (Chromium Portable)*/
const Device UA2373167 = {S26484, 132, { &P15364,&P17285,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/12.0.702.0 Safari/533.3 Awesomium/1.6.5*/
const Device UA2374281 = {S26485, 108, { &P15364,&P17285,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.30 (KHTML	 like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA2409601 = {S26486, 131, { &P15364,&P17285,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30; Windows NT 6.0; Trident/5.0)*/
const Device UA2411205 = {S26487, 109, { &P15364,&P17285,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; de) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/12.0 Safari/525.19*/
const Device UA2805957 = {S26488, 146, { &P15364,&P17285,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.91 Safari/534.30,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2821719 = {S26489, 100, { &P15364,&P17285,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.30 (KHTML like Gecko) Chrome/12.0.742.100 Safari/534.30*/
const Device UA3072438 = {S26490, 131, { &P15364,&P17285,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30; Windows NT 6.0; Trident/6.0)*/
const Device UA351412 = {S26491, 136, { &P14574,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-gb; Nexus S Build/IML63B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA925476 = {S26492, 138, { &P14574,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S Build/JRO03E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1982477 = {S26493, 138, { &P14574,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1995168 = {S26494, 136, { &P14574,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; Nexus S Build/JRN83C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1995937 = {S26495, 60, { &P14574,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Nexus S Build/JRO03E)*/
const Device UA1996254 = {S26496, 138, { &P14574,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2003523 = {S26497, 138, { &P14574,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Nexus S Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028602 = {S26498, 129, { &P14574,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Nexus S Build/JRO03E) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2028639 = {S26499, 153, { &P14574,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Full Android on Crespo Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2204866 = {S26500, 138, { &P14574,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2234212 = {S26501, 60, { &P14574,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Nexus S Build/JZO54K)*/
const Device UA2238145 = {S26502, 138, { &P14574,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2360552 = {S26503, 129, { &P14574,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Nexus S Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2384881 = {S26504, 136, { &P14574,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; Nexus S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA924836 = {S26505, 133, { &P17346,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A5355d Safari/8536.25*/
const Device UA2012817 = {S26506, 133, { &P17346,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A5376e Safari/8536.25*/
const Device UA2042766 = {S26507, 131, { &P17346,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A403 Safari/8536.25*/
const Device UA2200241 = {S26508, 131, { &P17346,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A406 Safari/8536.25*/
const Device UA2261403 = {S26509, 133, { &P17346,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2425009 = {S26510, 164, { &P17346,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10A523 Version/6.0 Safari/8536.25 Safari/6533.18.5*/
const Device UA2552136 = {S26511, 162, { &P17346,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10A403 Version/6.0 Safari/8536.25 Safari/6533.18.5*/
const Device UA2594137 = {S26512, 129, { &P17346,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 60 like Mac OS X) AppleWebKit/536.26 (KHTML like Gecko) Version/6.0 Mobile/10A403 Safari/8536.25*/
const Device UA2596476 = {S26513, 129, { &P17346,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 60 like Mac OS X) AppleWebKit/536.26 (KHTML like Gecko) Version/6.0 Mobile/10A406 Safari/8536.25*/
const Device UA2597881 = {S26514, 130, { &P17346,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 601 like Mac OS X) AppleWebKit/536.26 (KHTML like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2598078 = {S26515, 131, { &P17346,&P18432,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 60 like Mac OS X) AppleWebKit/536.26 (KHTML like Gecko) Version/6.0 Mobile/10A5376e Safari/8536.25*/
const Device UA2973661 = {S26516, 138, { &P18645,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973894 = {S26517, 137, { &P18645,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973953 = {S26518, 137, { &P18645,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974161 = {S26519, 145, { &P18645,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974247 = {S26520, 137, { &P18645,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974264 = {S26521, 138, { &P18645,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974269 = {S26522, 137, { &P18645,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974285 = {S26523, 138, { &P18645,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530C Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974324 = {S26524, 138, { &P18645,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974377 = {S26525, 138, { &P18645,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974387 = {S26526, 138, { &P18645,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747M Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974445 = {S26527, 138, { &P18645,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974715 = {S26528, 137, { &P18645,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SPH-L710 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975526 = {S26529, 145, { &P18645,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975929 = {S26530, 138, { &P18645,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976001 = {S26531, 138, { &P18645,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976014 = {S26532, 138, { &P18645,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-R530U Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022993 = {S26533, 138, { &P18645,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023436 = {S26534, 137, { &P18645,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3107824 = {S26535, 138, { &P18645,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305N Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2538337 = {S26536, 127, { &P15364,&P17285,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/25.0.1364.97) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22*/
const Device UA2538822 = {S26537, 134, { &P15364,&P17285,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/25.0.1364.97) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22*/
const Device UA2565726 = {S26538, 136, { &P15364,&P17285,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/25.0.1364.160) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.160 Safari/537.22*/
const Device UA2567925 = {S26539, 90, { &P15364,&P17285,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/25.0.1364.172)*/
const Device UA2567991 = {S26540, 97, { &P15364,&P17285,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/25.0.1364.172)*/
const Device UA2568124 = {S26541, 196, { &P15364,&P17285,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; chromeframe/25.0.1364.172; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2568163 = {S26542, 129, { &P15364,&P17285,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/25.0.1364.172) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2568160 = {S26543, 162, { &P15364,&P17285,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/25.0.1364.172; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2568369 = {S26544, 183, { &P15364,&P17285,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/25.0.1364.172; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2568518 = {S26545, 136, { &P15364,&P17285,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/25.0.1364.172) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2569088 = {S26546, 129, { &P15364,&P17285,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/25.0.1364.152) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22*/
const Device UA2569473 = {S26547, 136, { &P15364,&P17285,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/25.0.1364.152) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22*/
const Device UA2571048 = {S26548, 129, { &P15364,&P17285,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/25.0.1364.160) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.160 Safari/537.22*/
const Device UA2584520 = {S26549, 203, { &P15364,&P17285,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.4; chromeframe/25.0.1364.172; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)*/
const Device UA2584870 = {S26550, 230, { &P15364,&P17285,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/25.0.1364.172; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C)*/
const Device UA2584943 = {S26551, 190, { &P15364,&P17285,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; chromeframe/25.0.1364.172; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2605847 = {S26552, 211, { &P15364,&P17285,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; chromeframe/25.0.1364.172; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2860457 = {S26553, 241, { &P15364,&P17285,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/25.0.1364.172; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2407659 = {S26554, 69, { &P15364,&P18118,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:20.0) Gecko/X Firefox/20.0*/
const Device UA197530 = {S26555, 213, { &P12280,&P17084,&P4201,&P18089 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7 (compatible; Googlebot-Mobile/2.1; +http://www.google.com/bot.html)*/
const Device UA1996470 = {S26556, 217, { &P12280,&P17084,&P4201,&P18089 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7 (compatible; Mediapartners-Google/2.1; +http://www.google.com/bot.html)*/
const Device UA3023901 = {S26557, 127, { &P15364,&P17285,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/29.0.1547.32) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.32 Safari/537.36*/
const Device UA3031493 = {S26558, 127, { &P15364,&P17285,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/29.0.1547.49) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.49 Safari/537.36*/
const Device UA3033597 = {S26559, 127, { &P15364,&P17285,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/29.0.1547.57) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36*/
const Device UA3036469 = {S26560, 195, { &P15364,&P17285,&P22769,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/29.0.1547.57; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 1.1.4322; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA3036980 = {S26561, 134, { &P15364,&P17285,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/29.0.1547.57) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36*/
const Device UA3062349 = {S26562, 134, { &P15364,&P17285,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/29.0.1547.62) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.62 Safari/537.36*/
const Device UA3064084 = {S26563, 127, { &P15364,&P17285,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/29.0.1547.66) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36*/
const Device UA3064841 = {S26564, 182, { &P15364,&P17285,&P22769,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.5; chromeframe/29.0.1547.66; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA3065052 = {S26565, 134, { &P15364,&P17285,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/29.0.1547.66) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36*/
const Device UA3065849 = {S26566, 127, { &P15364,&P17285,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/29.0.1547.62) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.62 Safari/537.36*/
const Device UA3067260 = {S26567, 195, { &P15364,&P17285,&P22769,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.5; chromeframe/29.0.1547.66; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA3067272 = {S26568, 161, { &P15364,&P17285,&P22769,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.5; chromeframe/29.0.1547.66; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA3067850 = {S26569, 96, { &P15364,&P17285,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/29.0.1547.66)*/
const Device UA3067884 = {S26570, 89, { &P15364,&P17285,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/29.0.1547.66)*/
const Device UA3068226 = {S26571, 189, { &P15364,&P17285,&P22769,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.5; chromeframe/29.0.1547.66; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA3080738 = {S26572, 202, { &P15364,&P17285,&P22769,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.5; chromeframe/29.0.1547.66; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA3096676 = {S26573, 127, { &P15364,&P17285,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/29.0.1547.76) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36*/
const Device UA3110355 = {S26574, 134, { &P15364,&P17285,&P22769,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/29.0.1547.76) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36*/
const Device UA2205408 = {S26575, 152, { &P12280,&P18117,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9B206 Safari/7534.48.3*/
const Device UA2205474 = {S26576, 152, { &P12280,&P18117,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9B208 Safari/7534.48.3*/
const Device UA2205554 = {S26577, 150, { &P12280,&P18117,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9B176 Safari/7534.48.3*/
const Device UA2205717 = {S26578, 150, { &P12280,&P18117,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9B179 Safari/7534.48.3*/
const Device UA2382694 = {S26579, 152, { &P12280,&P18117,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9Z999 Safari/7534.48.3*/
const Device UA2746533 = {S26580, 150, { &P12280,&P18117,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9A405 Safari/7534.48.3*/
const Device UA3115649 = {S26581, 119, { &P17595,&P17297,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3128317 = {S26582, 119, { &P17595,&P17297,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3128881 = {S26583, 119, { &P17595,&P17297,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3129331 = {S26584, 119, { &P17595,&P17297,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3130085 = {S26585, 119, { &P17595,&P17297,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3131356 = {S26586, 119, { &P17595,&P17297,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA2570439 = {S26587, 106, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.777 Mobile Safari/537.10+*/
const Device UA2570876 = {S26588, 106, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.837 Mobile Safari/537.10+*/
const Device UA2584178 = {S26589, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1002 Mobile Safari/537.10+*/
const Device UA2585562 = {S26590, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1020 Mobile Safari/537.10+*/
const Device UA2604055 = {S26591, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1212 Mobile Safari/537.10+*/
const Device UA2604102 = {S26592, 106, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.936 Mobile Safari/537.10+*/
const Device UA2604147 = {S26593, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1110 Mobile Safari/537.10+*/
const Device UA2670013 = {S26594, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1248 Mobile Safari/537.10+*/
const Device UA2671187 = {S26595, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1284 Mobile Safari/537.10+*/
const Device UA2672617 = {S26596, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1392 Mobile Safari/537.10+*/
const Device UA2674136 = {S26597, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1465 Mobile Safari/537.10+*/
const Device UA2674316 = {S26598, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1320 Mobile Safari/537.10+*/
const Device UA2674382 = {S26599, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1471 Mobile Safari/537.10+*/
const Device UA2675774 = {S26600, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1411 Mobile Safari/537.10+*/
const Device UA2677612 = {S26601, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1507 Mobile Safari/537.10+*/
const Device UA2677756 = {S26602, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1447 Mobile Safari/537.10+*/
const Device UA2677968 = {S26603, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1429 Mobile Safari/537.10+*/
const Device UA2709286 = {S26604, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1720 Mobile Safari/537.10+*/
const Device UA2712710 = {S26605, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1552 Mobile Safari/537.10+*/
const Device UA2714588 = {S26606, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1483 Mobile Safari/537.10+*/
const Device UA2715196 = {S26607, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1696 Mobile Safari/537.10+*/
const Device UA2716485 = {S26608, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1756 Mobile Safari/537.10+*/
const Device UA2716716 = {S26609, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1597 Mobile Safari/537.10+*/
const Device UA2718728 = {S26610, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1666 Mobile Safari/537.10+*/
const Device UA2718780 = {S26611, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1609 Mobile Safari/537.10+*/
const Device UA2719290 = {S26612, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1627 Mobile Safari/537.10+*/
const Device UA2737358 = {S26613, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1485 Mobile Safari/537.10+*/
const Device UA2739195 = {S26614, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1895 Mobile Safari/537.10+*/
const Device UA2739607 = {S26615, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1762 Mobile Safari/537.10+*/
const Device UA2739848 = {S26616, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1832 Mobile Safari/537.10+*/
const Device UA2741622 = {S26617, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1916 Mobile Safari/537.10+*/
const Device UA2741722 = {S26618, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1880 Mobile Safari/537.10+*/
const Device UA2753358 = {S26619, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2342 Mobile Safari/537.10+*/
const Device UA2754166 = {S26620, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2336 Mobile Safari/537.10+*/
const Device UA2755390 = {S26621, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2324 Mobile Safari/537.10+*/
const Device UA2756707 = {S26622, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2009 Mobile Safari/537.10+*/
const Device UA2769827 = {S26623, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2354 Mobile Safari/537.10+*/
const Device UA2769910 = {S26624, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4297 Mobile Safari/537.10+*/
const Device UA2770448 = {S26625, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2390 Mobile Safari/537.10+*/
const Device UA2784158 = {S26626, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2420 Mobile Safari/537.10+*/
const Device UA2784822 = {S26627, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2050 Mobile Safari/537.10+*/
const Device UA2784949 = {S26628, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4333 Mobile Safari/537.10+*/
const Device UA2799421 = {S26629, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2025 Mobile Safari/537.10+*/
const Device UA2800690 = {S26630, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4369 Mobile Safari/537.10+*/
const Device UA2861613 = {S26631, 106, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.987 Mobile Safari/537.10+*/
const Device UA2875856 = {S26632, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4429 Mobile Safari/537.10+*/
const Device UA2876437 = {S26633, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2414 Mobile Safari/537.10+*/
const Device UA2877612 = {S26634, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4393 Mobile Safari/537.10+*/
const Device UA2886367 = {S26635, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4453 Mobile Safari/537.10+*/
const Device UA2894110 = {S26636, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2062 Mobile Safari/537.10+*/
const Device UA2897660 = {S26637, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4537 Mobile Safari/537.10+*/
const Device UA2899420 = {S26638, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4543 Mobile Safari/537.10+*/
const Device UA2943480 = {S26639, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2074 Mobile Safari/537.10+*/
const Device UA2944162 = {S26640, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4477 Mobile Safari/537.10+*/
const Device UA2966888 = {S26641, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2151 Mobile Safari/537.10+*/
const Device UA2968446 = {S26642, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4597 Mobile Safari/537.10+*/
const Device UA2971913 = {S26643, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4633 Mobile Safari/537.10+*/
const Device UA2974001 = {S26644, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4651 Mobile Safari/537.10+*/
const Device UA2974758 = {S26645, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4687 Mobile Safari/537.10+*/
const Device UA2998618 = {S26646, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1768 Mobile Safari/537.10+*/
const Device UA2999770 = {S26647, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4261 Mobile Safari/537.10+*/
const Device UA3012706 = {S26648, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4492 Mobile Safari/537.10+*/
const Device UA3021046 = {S26649, 112, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - LinkedInBlackBerry/10.1.4 (1) Z10 STL100-2/10.1.0.4633 (Research In Motion;RIM BlackBerry Device;Z10;STL100-2;1)*/
const Device UA3021402 = {S26650, 112, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - LinkedInBlackBerry/10.1.4 (1) Z10 STL100-3/10.1.0.4633 (Research In Motion;RIM BlackBerry Device;Z10;STL100-3;1)*/
const Device UA3024367 = {S26651, 112, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - LinkedInBlackBerry/10.1.4 (1) Z10 STL100-4/10.1.0.2151 (Research In Motion;RIM BlackBerry Device;Z10;STL100-4;1)*/
const Device UA3031256 = {S26652, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4780 Mobile Safari/537.10+*/
const Device UA3039089 = {S26653, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4828 Mobile Safari/537.10+*/
const Device UA3057715 = {S26654, 112, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - LinkedInBlackBerry/10.1.4 (1) Z10 STL100-1/10.1.0.4633 (Research In Motion;RIM BlackBerry Device;Z10;STL100-1;1)*/
const Device UA3088645 = {S26655, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4246 Mobile Safari/537.10+*/
const Device UA3107705 = {S26656, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4657 Mobile Safari/537.10+*/
const Device UA3121351 = {S26657, 107, { &P19794,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4339 Mobile Safari/537.10+*/
const Device UA94674 = {S26658, 50, { &P15364,&P7127,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0)*/
const Device UA99212 = {S26659, 69, { &P15364,&P7127,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; .NET CLR 1.1.4322)*/
const Device UA134562 = {S26660, 51, { &P15364,&P7127,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0 )*/
const Device UA2128942 = {S26661, 89, { &P15364,&P7127,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (Compatible; MSIE 5.5; Windows NT5.0; QX; SV1; .NET CLR 1.1.4322; InfoPath.2)*/
const Device UA2258012 = {S26662, 50, { &P15364,&P7127,&P17387,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.5; Windows NT 5.0)*/
const Device UA2504903 = {S26663, 72, { &P15364,&P7127,&P17387,&P18092 }};	/*Useragent - MSIE 5.5 (Win 2000): Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0 )*/
const Device UA2585284 = {S26664, 69, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; Win64; x64) Presto/2.12.388 Version/12.15*/
const Device UA2607172 = {S26665, 57, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2) Presto/2.12.388 Version/12.15*/
const Device UA2672085 = {S26666, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.15*/
const Device UA2672311 = {S26667, 74, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.15,gzip(gfe)*/
const Device UA2672616 = {S26668, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.15*/
const Device UA2672976 = {S26669, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.15*/
const Device UA2674750 = {S26670, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.15*/
const Device UA2674973 = {S26671, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.15*/
const Device UA2675259 = {S26672, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.15*/
const Device UA2675376 = {S26673, 75, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.15*/
const Device UA2675908 = {S26674, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.15*/
const Device UA2676636 = {S26675, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.15*/
const Device UA2676767 = {S26676, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.15*/
const Device UA2677035 = {S26677, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.15*/
const Device UA2677329 = {S26678, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.15*/
const Device UA2707731 = {S26679, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.15*/
const Device UA2707732 = {S26680, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.15*/
const Device UA2708797 = {S26681, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.15*/
const Device UA2708864 = {S26682, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.15*/
const Device UA2709031 = {S26683, 67, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2) Presto/2.12.388 Version/12.15,gzip(gfe)*/
const Device UA2710638 = {S26684, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.15*/
const Device UA2710646 = {S26685, 69, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64) Opera 12.15*/
const Device UA2713512 = {S26686, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.15*/
const Device UA2714300 = {S26687, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.15*/
const Device UA2714680 = {S26688, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.15*/
const Device UA2717008 = {S26689, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.15*/
const Device UA2734784 = {S26690, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.15*/
const Device UA2735299 = {S26691, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.15*/
const Device UA2735472 = {S26692, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.15*/
const Device UA2735524 = {S26693, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6080)) Presto/2.12.388 Version/12.15*/
const Device UA2736400 = {S26694, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.15*/
const Device UA2736593 = {S26695, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.15*/
const Device UA2751103 = {S26696, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.15*/
const Device UA2751202 = {S26697, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.15*/
const Device UA2751678 = {S26698, 87, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.15*/
const Device UA2752631 = {S26699, 142, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; Win64; x64) Presto/2.12.388 Version/12.15 AppEngine-Google; (+http://code.google.com/appengine; appid: schmittify)*/
const Device UA2753359 = {S26700, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.15*/
const Device UA2766473 = {S26701, 80, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.15*/
const Device UA2769307 = {S26702, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6561)) Presto/2.12.388 Version/12.15*/
const Device UA2771849 = {S26703, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6573)) Presto/2.12.388 Version/12.15*/
const Device UA2794830 = {S26704, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.15*/
const Device UA2798859 = {S26705, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6283)) Presto/2.12.388 Version/12.15*/
const Device UA2799415 = {S26706, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6261)) Presto/2.12.388 Version/12.15*/
const Device UA2860942 = {S26707, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.15*/
const Device UA2861051 = {S26708, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.15*/
const Device UA2861172 = {S26709, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.15*/
const Device UA2863165 = {S26710, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.15*/
const Device UA2869590 = {S26711, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6638)) Presto/2.12.388 Version/12.15*/
const Device UA2875387 = {S26712, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6628)) Presto/2.12.388 Version/12.15*/
const Device UA2878792 = {S26713, 87, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 5.10 (build 5300)) Presto/2.12.388 Version/12.15*/
const Device UA2880208 = {S26714, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6322)) Presto/2.12.388 Version/12.15*/
const Device UA2883398 = {S26715, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.15*/
const Device UA2891730 = {S26716, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.15*/
const Device UA2893721 = {S26717, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6648)) Presto/2.12.388 Version/12.15*/
const Device UA2895369 = {S26718, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6657)) Presto/2.12.388 Version/12.15*/
const Device UA2895522 = {S26719, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6323)) Presto/2.12.388 Version/12.15*/
const Device UA2898788 = {S26720, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6327)) Presto/2.12.388 Version/12.15*/
const Device UA2901077 = {S26721, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6620)) Presto/2.12.388 Version/12.15*/
const Device UA2957572 = {S26722, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6277)) Presto/2.12.388 Version/12.15*/
const Device UA2959167 = {S26723, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6325)) Presto/2.12.388 Version/12.15*/
const Device UA3018452 = {S26724, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6337)) Presto/2.12.388 Version/12.15*/
const Device UA3020885 = {S26725, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6673)) Presto/2.12.388 Version/12.15*/
const Device UA3057466 = {S26726, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6669)) Presto/2.12.388 Version/12.15*/
const Device UA3059489 = {S26727, 86, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.15*/
const Device UA3072933 = {S26728, 87, { &P15364,&P16110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.15*/
const Device UA615668 = {S26729, 59, { &P15364,&P5690,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:6.0) Gecko/X Firefox/10.0.2*/
const Device UA640231 = {S26730, 71, { &P15364,&P5690,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:10.0) Gecko/X Firefox/10.0 Firefox/10.0*/
const Device UA2356388 = {S26731, 71, { &P15364,&P5690,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:10.0.10) Gecko/X Firefox/10.0.10*/
const Device UA2436517 = {S26732, 86, { &P15364,&P5690,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:10.0.11) Gecko/X Firefox/10.0.11ms1*/
const Device UA20746 = {S26733, 151, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; sv-se; SonyEricssonR800i Build/3.0.A.2.179) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20756 = {S26734, 139, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; R800i Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20765 = {S26735, 152, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonR800i Build/3.0.1.A.0.64) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20772 = {S26736, 139, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; R800i Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA23503 = {S26737, 151, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; SonyEricssonR800i Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24561 = {S26738, 151, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; SonyEricssonR800i Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43084 = {S26739, 151, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ar-; SonyEricssonR800i Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43092 = {S26740, 153, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonR800i Build/3.0.1.A.0.150) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43119 = {S26741, 153, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonR800i Build/3.0.1.A.0.146) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43138 = {S26742, 141, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; R800i Build/3.0.1.A.0.150) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48221 = {S26743, 153, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-za; SonyEricssonR800i Build/4.0.1.A.0.165) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48223 = {S26744, 151, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonR800i Build/4.0.A.2.257) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48229 = {S26745, 153, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonR800i Build/4.0.1.A.0.215) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48234 = {S26746, 153, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonR800i Build/4.0.1.A.0.247) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48242 = {S26747, 139, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; R800i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49806 = {S26748, 151, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonR800i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49864 = {S26749, 153, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonR800i Build/4.0.1.A.0.277) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49869 = {S26750, 151, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonR800i Build/4.0.2.A.0.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49868 = {S26751, 152, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonR800i Build/4.0.1.A.2.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58993 = {S26752, 152, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonR800i Build/3.0.1.A.0.79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59015 = {S26753, 141, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; R800i Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59029 = {S26754, 153, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonR800i Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59056 = {S26755, 151, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; sv-se; SonyEricssonR800i Build/3.0.A.2.119) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59067 = {S26756, 141, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; R800i Build/3.0.1.A.0.146) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59049 = {S26757, 153, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonR800i Build/3.0.1.A.0.151) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59091 = {S26758, 152, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonR800i Build/3.0.1.A.0.95) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61201 = {S26759, 151, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonR800i Build/4.0.A.2.286) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61207 = {S26760, 151, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonR800i Build/4.0.A.2.364) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70587 = {S26761, 152, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonR800i Build/GSX (Beta 2)) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70998 = {S26762, 153, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonR800i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74481 = {S26763, 152, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonR800i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA251810 = {S26764, 140, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; R800i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA253985 = {S26765, 152, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonR800i Build/4.0.2.A.0.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA316571 = {S26766, 65, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; R800i Build/3.0.1.A.0.145)*/
const Device UA368781 = {S26767, 152, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonR800i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA428408 = {S26768, 152, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonR800i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA534850 = {S26769, 63, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.2; R800i Build/3.0.A.2.184)*/
const Device UA625620 = {S26770, 64, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; R800i Build/4.0.2.A.0.42)*/
const Device UA688825 = {S26771, 152, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonR800i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA751518 = {S26772, 150, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ar-; SonyEricssonR800i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA814158 = {S26773, 140, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; R800i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA844135 = {S26774, 140, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; R800i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1938272 = {S26775, 141, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; R800i Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1963421 = {S26776, 150, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonR800i Build/6.0.A.3.67) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025143 = {S26777, 140, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; R800i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025462 = {S26778, 152, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonR800i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027154 = {S26779, 133, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; R800i Build/4.0.2.A.0.42) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2027940 = {S26780, 133, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; R800i Build/4.0.2.A.0.58) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2028061 = {S26781, 128, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; R800i Build/4.0.2.A.0.42) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2317331 = {S26782, 150, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-mx; SonyEricssonR800i Build/6.0.A.3.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2364779 = {S26783, 140, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; R800i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2425970 = {S26784, 140, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; R800i Build/4.0.2.A.0.79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2520583 = {S26785, 138, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-mx; R800i Build/6.0.A.3.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2936996 = {S26786, 140, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; R800i Build/4.0.2.A.0.79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2957686 = {S26787, 152, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonR800i Build/4.0.2.A.0.79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2980478 = {S26788, 64, { &P23919,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; R800i Build/4.0.2.A.0.69)*/
const Device UA3111109 = {S26789, 101, { &P15364,&P17285,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3111108 = {S26790, 108, { &P15364,&P17285,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3132678 = {S26791, 111, { &P15364,&P17285,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36,gzip(gfe)*/
const Device UA22353 = {S26792, 165, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; LG-P970h Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA33235 = {S26793, 164, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P970 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA46868 = {S26794, 169, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P970/V10c Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA47450 = {S26795, 165, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-mx; LG-P970h Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA49231 = {S26796, 165, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; LG-P970g Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA54114 = {S26797, 164, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P970 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA54401 = {S26798, 171, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P970-Orange Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA58376 = {S26799, 165, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - LG-P970/ Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA58378 = {S26800, 171, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P970-Orange Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA58380 = {S26801, 166, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; de; LG-P970/V10b Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA59947 = {S26802, 169, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-au; LG-P970/V10a Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA59948 = {S26803, 169, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P970/V10b Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA61060 = {S26804, 169, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; LG-P970/V11b Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA70373 = {S26805, 169, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; nl-nl; LG-P970/V10e Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA71015 = {S26806, 162, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ru; LG-P970h Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA72666 = {S26807, 166, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; de; LG-P970/V10c Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA72732 = {S26808, 169, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P970/V10d Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA79480 = {S26809, 169, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; LG-P970/V11c Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA321454 = {S26810, 60, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P970 Build/FRG83G)*/
const Device UA660585 = {S26811, 61, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P970h Build/FRG83G)*/
const Device UA755477 = {S26812, 124, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; LG-P970 Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1948881 = {S26813, 161, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; sr; LG-P970 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA1979494 = {S26814, 129, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; LG-P970 Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2019992 = {S26815, 61, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P970g Build/FRG83G)*/
const Device UA2020289 = {S26816, 166, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; de; LG-P970/V10d Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2026354 = {S26817, 134, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; LG-P970 Build/Black Pearl) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2933143 = {S26818, 137, { &P14123,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-co; LG-P970h Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Versi?n/4.0 Mobile Safari/533.1*/
const Device UA3064775 = {S26819, 137, { &P18645,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3064880 = {S26820, 145, { &P18645,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3064881 = {S26821, 137, { &P18645,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3064982 = {S26822, 138, { &P18645,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065137 = {S26823, 138, { &P18645,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530C Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065368 = {S26824, 145, { &P18645,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065441 = {S26825, 137, { &P18645,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3065460 = {S26826, 138, { &P18645,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065465 = {S26827, 137, { &P18645,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065479 = {S26828, 138, { &P18645,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065588 = {S26829, 138, { &P18645,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066530 = {S26830, 138, { &P18645,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747M Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066880 = {S26831, 137, { &P18645,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SPH-L710 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067178 = {S26832, 138, { &P18645,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067192 = {S26833, 138, { &P18645,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067907 = {S26834, 138, { &P18645,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081031 = {S26835, 137, { &P18645,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081344 = {S26836, 138, { &P18645,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3082222 = {S26837, 138, { &P18645,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-R530U Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126840 = {S26838, 138, { &P18645,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305N Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA22396 = {S26839, 178, { &P12449,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE5-00u/043.115; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.7.4 3gpp-gba*/
const Device UA56457 = {S26840, 193, { &P12449,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE5-00.2/071.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.26 Mobile Safari/533.4 3gpp-gba*/
const Device UA56456 = {S26841, 179, { &P12449,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE5-00.2/046.001; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.2 3gpp-gba*/
const Device UA64571 = {S26842, 177, { &P12449,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE5-00/042.007; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.2 3gpp-gba*/
const Device UA67247 = {S26843, 10, { &P12449,&P5890,&P3439,&P18092 }};	/*Useragent - NOKIAE5-00*/
const Device UA70155 = {S26844, 192, { &P12449,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE5-00u/072.001; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.26 Mobile Safari/533.4 3gpp-gba*/
const Device UA76951 = {S26845, 190, { &P12449,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2NokiaE5-00/071.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.26 Mobile Safari/533.4 3gpp-gba*/
const Device UA812115 = {S26846, 12, { &P12449,&P5890,&P3439,&P18092 }};	/*Useragent - NOKIAE5-00.2*/
const Device UA925472 = {S26847, 191, { &P12449,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE5-00/101.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.35 Mobile Safari/533.4 3gpp-gba*/
const Device UA2272478 = {S26848, 49, { &P12449,&P5890,&P3439,&P18092 }};	/*Useragent - (C)NokiaE5-00/SymbianOS/9.1 Series60/3.0 3gpp-gba*/
const Device UA2448700 = {S26849, 82, { &P12449,&P5890,&P3439,&P18092 }};	/*Useragent - 3gpp-gba Mozilla/4.0 (compatible; Nokia Podcasting; SymbianOS; NokiaE5-00/101.003)*/
const Device UA2752544 = {S26850, 106, { &P12449,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NOKIAE5-00*/
const Device UA23041 = {S26851, 36, { &P15364,&P7127,&P17470,&P18092 }};	/*Useragent - Mozilla/4.8 [en] (Windows NT 5.0; U)*/
const Device UA93049 = {S26852, 37, { &P15364,&P7127,&P17470,&P18092 }};	/*Useragent - Mozilla/4.76 [en] (Windows NT 5.0; U)*/
const Device UA94198 = {S26853, 37, { &P15364,&P7127,&P17470,&P18092 }};	/*Useragent - Mozilla/4.79 [en] (Windows NT 5.0; U)*/
const Device UA106338 = {S26854, 34, { &P15364,&P7127,&P17470,&P18092 }};	/*Useragent - Mozilla/4.73 [en] (Windows NT 5.0)*/
const Device UA115618 = {S26855, 34, { &P15364,&P7127,&P17470,&P18092 }};	/*Useragent - Mozilla/4.79 [en] (Windows NT 5.0)*/
const Device UA118078 = {S26856, 34, { &P15364,&P7127,&P17470,&P18092 }};	/*Useragent - Mozilla/4.77 [en] (Windows NT 5.0)*/
const Device UA623136 = {S26857, 64, { &P15364,&P7127,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.5b) Gecko/X*/
const Device UA625752 = {S26858, 28, { &P15364,&P7127,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows NT 5.0)*/
const Device UA181064 = {S26859, 112, { &P15364,&P18110,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA183468 = {S26860, 154, { &P15364,&P18110,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Jolicloud Linux i686; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Joli OS/1.2 Chromium/10.0.648.82 Chrome/10.0.648.82 Safari/534.16*/
const Device UA184489 = {S26861, 141, { &P15364,&P18110,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.16 (KHTML, like Gecko, Google Keyword Suggestion) Chrome/10.0.648.127 Safari/534.16*/
const Device UA300109 = {S26862, 128, { &P15364,&P18110,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.16 SUSE/10.0.626.0 (KHTML, like Gecko) Chrome/10.0.626.0 Safari/534.16*/
const Device UA581750 = {S26863, 114, { &P15364,&P18110,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux armv7l; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA759205 = {S26864, 119, { &P15364,&P18110,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.634.0 Safari/534.16*/
const Device UA2323913 = {S26865, 114, { &P15364,&P18110,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (U; compatible; Linux; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA2674649 = {S26866, 106, { &P15364,&P18110,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.63 Safari/537.31*/
const Device UA2674700 = {S26867, 109, { &P15364,&P18110,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.63 Safari/537.31*/
const Device UA2674715 = {S26868, 106, { &P15364,&P18110,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.63 Safari/537.31*/
const Device UA2675018 = {S26869, 109, { &P15364,&P18110,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.63 Safari/537.31*/
const Device UA2675721 = {S26870, 106, { &P15364,&P18110,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.63 Safari/537.31*/
const Device UA2718409 = {S26871, 109, { &P15364,&P18110,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.63 Safari/537.31*/
const Device UA2718770 = {S26872, 106, { &P15364,&P18110,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.63 Safari/537.31*/
const Device UA47465 = {S26873, 143, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDXBuild/4.5.1_57_DX5-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47468 = {S26874, 143, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47469 = {S26875, 144, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47480 = {S26876, 144, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47484 = {S26877, 144, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-33) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47493 = {S26878, 143, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47496 = {S26879, 143, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;DROIDX Build/4.5.1_57_DX5-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60407 = {S26880, 144, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-26) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60414 = {S26881, 144, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60418 = {S26882, 143, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDXBuild/4.5.1_57_DX5-26) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70911 = {S26883, 143, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA71173 = {S26884, 143, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;DROIDX Build/4.5.1_57_DX5-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71325 = {S26885, 143, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71334 = {S26886, 143, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDXBuild/4.5.1_57_DX5-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72058 = {S26887, 143, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA179420 = {S26888, 144, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA183565 = {S26889, 143, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA193950 = {S26890, 144, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA220038 = {S26891, 135, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROIDX Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA234946 = {S26892, 132, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROIDX Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA245069 = {S26893, 68, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; DROIDX Build/4.5.1_57_DX5-35)*/
const Device UA249909 = {S26894, 132, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROIDX Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA313876 = {S26895, 144, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA324302 = {S26896, 132, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA576432 = {S26897, 151, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-35) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA623004 = {S26898, 144, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623318 = {S26899, 144, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX Build/4.5.1_57_DX5-21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623590 = {S26900, 68, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; DROIDX Build/4.5.1_57_DX8-51)*/
const Device UA673897 = {S26901, 143, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA680551 = {S26902, 143, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA687996 = {S26903, 143, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA691270 = {S26904, 143, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA692229 = {S26905, 143, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDXBuild/4.5.1_57_DX8-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA758821 = {S26906, 135, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA851465 = {S26907, 135, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROIDX Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA887657 = {S26908, 141, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROIDX Build/GRH78C; 4000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1903059 = {S26909, 56, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; DROIDX Build/VZW)*/
const Device UA1940585 = {S26910, 145, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Gingercomb; en-us; DroidX Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1982552 = {S26911, 151, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2019182 = {S26912, 56, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; DROIDX Build/VZW)*/
const Device UA2019777 = {S26913, 135, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROIDX Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022350 = {S26914, 144, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/4.5.1_57_DX8-50) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030700 = {S26915, 142, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2382605 = {S26916, 152, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30*/
const Device UA3079646 = {S26917, 144, { &P17308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/4.5.1_57_DX9-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2603688 = {S26918, 143, { &P18713,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; LG-P760/V20a Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2668500 = {S26919, 146, { &P18713,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P760/P76020b Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2679310 = {S26920, 138, { &P18713,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; LG-P760 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2679669 = {S26921, 138, { &P18713,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-P768 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2697265 = {S26922, 138, { &P18713,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P769 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2712881 = {S26923, 60, { &P18713,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P760 Build/JZO54K)*/
const Device UA2716877 = {S26924, 146, { &P18713,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P760/P76020c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2717252 = {S26925, 138, { &P18713,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; sk-sk; LG-P760 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2739600 = {S26926, 143, { &P18713,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; LG-P760/V20b Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752441 = {S26927, 60, { &P18713,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P768 Build/JZO54K)*/
const Device UA2755303 = {S26928, 145, { &P18713,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-P760-Orange Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2797208 = {S26929, 146, { &P18713,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; LG-P760/P76020b Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2860725 = {S26930, 138, { &P18713,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LGMS769 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937253 = {S26931, 138, { &P18713,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P769 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937955 = {S26932, 60, { &P18713,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P769 Build/JZO54K)*/
const Device UA2994487 = {S26933, 135, { &P18713,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; et; LG-P760 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3003524 = {S26934, 60, { &P18713,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LGMS769 Build/JZO54K)*/
const Device UA3080017 = {S26935, 143, { &P18713,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; LG-P760/V20c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3081669 = {S26936, 146, { &P18713,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P760/P76020i Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3091983 = {S26937, 146, { &P18713,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; LG-P760/P76020a Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125250 = {S26938, 146, { &P18713,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P760/P76020h Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2490859 = {S26939, 82, { &P15364,&P21460,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (IE 11.0; Windows NT 6.3; Win64; x64; Trident/7.0; rv:11.0) like Gecko*/
const Device UA2490871 = {S26940, 70, { &P15364,&P21460,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (IE 11.0; Windows NT 6.3; Trident/7.0; rv:11.0) like Gecko*/
const Device UA2533320 = {S26941, 77, { &P15364,&P21460,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (IE 11.0; Windows NT 6.3; WOW64; Trident/7.0; rv:11.0) like Gecko*/
const Device UA2604059 = {S26942, 73, { &P15364,&P21460,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.3; Win64; x64; Trident/7.0; rv:11.0) like Gecko*/
const Device UA2670982 = {S26943, 68, { &P15364,&P21460,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.3; WOW64; Trident/7.0; rv:11.0) like Gecko*/
const Device UA2989666 = {S26944, 90, { &P15364,&P21460,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (IE 11.0; Windows NT 6.3; Trident/7.0; .NET4.0E; .NET4.0C; rv:11.0) like Gecko*/
const Device UA30030 = {S26945, 148, { &P17346,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA516452 = {S26946, 148, { &P17346,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9Z999 Safari/6533.18.5*/
const Device UA2600152 = {S26947, 145, { &P17346,&P17145,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 421 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA48116 = {S26948, 152, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonWT19i Build/4.0.1.A.0.55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49551 = {S26949, 153, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonWT19i Build/4.0.1.A.0.271) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49558 = {S26950, 153, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonWT19i Build/4.0.1.A.0.258) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA77800 = {S26951, 141, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; WT19i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78038 = {S26952, 153, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonWT19i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA350411 = {S26953, 151, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonWT19i Build/4.0.2.C.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368795 = {S26954, 140, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; WT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368878 = {S26955, 128, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; WT19i Build/4.0.2.A.0.62) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA370849 = {S26956, 150, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-; SonyEricssonWT19i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA630394 = {S26957, 152, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SonyEricssonWT19i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA631750 = {S26958, 154, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pl-pl; SonyEricssonWT19i-o Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633062 = {S26959, 140, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; WT19i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633299 = {S26960, 64, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; WT19i Build/4.0.2.A.0.62)*/
const Device UA633731 = {S26961, 152, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; SonyEricssonWT19i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA658749 = {S26962, 139, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; WT19i Build/4.0.2.C.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA662982 = {S26963, 152, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonWT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA670306 = {S26964, 153, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SonyEricssonWT19iv Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA686175 = {S26965, 153, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-za; SonyEricssonWT19iv Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA705438 = {S26966, 128, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; WT19i Build/4.0.2.A.0.58) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA714738 = {S26967, 152, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonWT19i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA742125 = {S26968, 154, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-nz; SonyEricssonWT19iv Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA779582 = {S26969, 154, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; SonyEricssonWT19i-o Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1976891 = {S26970, 140, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; WT19i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1992957 = {S26971, 153, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-be; SonyEricssonWT19iv Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029268 = {S26972, 133, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; WT19i Build/4.0.2.A.0.62) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2743586 = {S26973, 134, { &P12616,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; WT19i Build/4.0.1.A.0.283) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2369712 = {S26974, 109, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3196.1.0) AppleWebKit/537.19 (KHTML, like Gecko) Chrome/25.0.1324.1 Safari/537.19*/
const Device UA2369717 = {S26975, 111, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3196.1.0) AppleWebKit/537.19 (KHTML, like Gecko) Chrome/25.0.1324.1 Safari/537.19*/
const Device UA2387542 = {S26976, 111, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3196.2.0) AppleWebKit/537.19 (KHTML, like Gecko) Chrome/25.0.1324.1 Safari/537.19*/
const Device UA2388222 = {S26977, 111, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3196.4.0) AppleWebKit/537.19 (KHTML, like Gecko) Chrome/25.0.1324.1 Safari/537.19*/
const Device UA2405679 = {S26978, 111, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3196.3.0) AppleWebKit/537.19 (KHTML, like Gecko) Chrome/25.0.1324.1 Safari/537.19*/
const Device UA2406786 = {S26979, 109, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3337.1.0) AppleWebKit/537.21 (KHTML, like Gecko) Chrome/25.0.1347.5 Safari/537.21*/
const Device UA2407512 = {S26980, 111, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3337.1.0) AppleWebKit/537.21 (KHTML, like Gecko) Chrome/25.0.1347.5 Safari/537.21*/
const Device UA2407516 = {S26981, 111, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3337.1.0) AppleWebKit/537.21 (KHTML, like Gecko) Chrome/25.0.1347.5 Safari/537.21*/
const Device UA2429847 = {S26982, 111, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3379.2.0) AppleWebKit/537.21 (KHTML, like Gecko) Chrome/25.0.1354.2 Safari/537.21*/
const Device UA2430180 = {S26983, 111, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3379.2.0) AppleWebKit/537.21 (KHTML, like Gecko) Chrome/25.0.1354.2 Safari/537.21*/
const Device UA2430941 = {S26984, 109, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3379.2.0) AppleWebKit/537.21 (KHTML, like Gecko) Chrome/25.0.1354.2 Safari/537.21*/
const Device UA2434671 = {S26985, 111, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3428.0.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.2 Safari/537.22*/
const Device UA2435233 = {S26986, 111, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3428.0.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.2 Safari/537.22*/
const Device UA2435896 = {S26987, 109, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3428.0.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.2 Safari/537.22*/
const Device UA2437451 = {S26988, 112, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3417.0.2012) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1363.0 Safari/537.22*/
const Device UA2461909 = {S26989, 113, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3428.49.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.26 Safari/537.22*/
const Device UA2461912 = {S26990, 111, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3428.49.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.26 Safari/537.22*/
const Device UA2461978 = {S26991, 113, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3428.49.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.26 Safari/537.22*/
const Device UA2470855 = {S26992, 111, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3428.65.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.33 Safari/537.22*/
const Device UA2471620 = {S26993, 113, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3428.65.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.33 Safari/537.22*/
const Device UA2471635 = {S26994, 113, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3428.65.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.33 Safari/537.22*/
const Device UA2487417 = {S26995, 111, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3428.92.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.46 Safari/537.22*/
const Device UA2487420 = {S26996, 113, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3428.92.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.46 Safari/537.22*/
const Device UA2487511 = {S26997, 113, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3428.94.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.49 Safari/537.22*/
const Device UA2489170 = {S26998, 113, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3428.77.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.40 Safari/537.22*/
const Device UA2490860 = {S26999, 114, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3428.114.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.63 Safari/537.22*/
const Device UA2508979 = {S27000, 114, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3428.110.2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.57 Safari/537.22*/
const Device UA2509103 = {S27001, 114, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3428.127.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.68 Safari/537.22*/
const Device UA2513071 = {S27002, 112, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3428.127.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.68 Safari/537.22*/
const Device UA2513075 = {S27003, 114, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3428.127.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.68 Safari/537.22*/
const Device UA2515894 = {S27004, 114, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3428.149.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.87 Safari/537.22*/
const Device UA2516060 = {S27005, 114, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3428.131.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.74 Safari/537.22*/
const Device UA2516762 = {S27006, 112, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3428.149.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.87 Safari/537.22*/
const Device UA2516763 = {S27007, 114, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3428.149.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.87 Safari/537.22*/
const Device UA2532810 = {S27008, 114, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3428.167.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.91 Safari/537.22*/
const Device UA2533069 = {S27009, 114, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3428.149.2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.87 Safari/537.22*/
const Device UA2568078 = {S27010, 113, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3428.193.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.126 Safari/537.22*/
const Device UA2568095 = {S27011, 115, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3428.193.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.126 Safari/537.22*/
const Device UA2568100 = {S27012, 115, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3428.193.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.126 Safari/537.22*/
const Device UA2569186 = {S27013, 115, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3428.210.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.173 Safari/537.22*/
const Device UA2572244 = {S27014, 113, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3428.210.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.173 Safari/537.22*/
const Device UA2572245 = {S27015, 115, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3428.210.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.173 Safari/537.22*/
const Device UA2732840 = {S27016, 114, { &P15364,&P6596,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3428.183.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22*/
const Device UA924164 = {S27017, 151, { &P18449,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonST27i Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1996395 = {S27018, 139, { &P18449,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; ST27i Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2000851 = {S27019, 151, { &P18449,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonST27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030498 = {S27020, 139, { &P18449,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; ST27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2185596 = {S27021, 63, { &P18449,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; ST27i Build/6.0.B.3.184)*/
const Device UA2257173 = {S27022, 151, { &P18449,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-mx; SonyEricssonST27a Build/6.0.B.3.191) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2342982 = {S27023, 151, { &P18449,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SonyEricssonST27a Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2384890 = {S27024, 139, { &P18449,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-mx; ST27a Build/6.0.B.3.191) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2404213 = {S27025, 152, { &P18449,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-za; SonyEricssonST27iv Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2468999 = {S27026, 134, { &P18449,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ST27I Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2594910 = {S27027, 63, { &P18449,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; ST27a Build/6.0.B.3.191)*/
const Device UA2645308 = {S27028, 132, { &P18449,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ST27i Build/6.0.B.3.184) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2982176 = {S27029, 149, { &P18449,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; iw-; SonyEricssonST27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3117991 = {S27030, 139, { &P18449,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; th-th; ST27a Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2434558 = {S27031, 142, { &P15767,&P18117,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9B206 Safari/7534.48.3*/
const Device UA2435248 = {S27032, 140, { &P15767,&P18117,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9B176 Safari/7534.48.3*/
const Device UA2469133 = {S27033, 140, { &P15767,&P18117,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9A405 Safari/7534.48.3*/
const Device UA2489787 = {S27034, 109, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3635.0.0) AppleWebKit/537.28 (KHTML, like Gecko) Chrome/26.0.1398.0 Safari/537.28*/
const Device UA2489788 = {S27035, 111, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3635.0.0) AppleWebKit/537.28 (KHTML, like Gecko) Chrome/26.0.1398.0 Safari/537.28*/
const Device UA2489795 = {S27036, 111, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3635.0.0) AppleWebKit/537.28 (KHTML, like Gecko) Chrome/26.0.1398.0 Safari/537.28*/
const Device UA2510399 = {S27037, 111, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3658.0.0) AppleWebKit/537.30 (KHTML, like Gecko) Chrome/26.0.1403.0 Safari/537.30*/
const Device UA2510407 = {S27038, 109, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3658.0.0) AppleWebKit/537.30 (KHTML, like Gecko) Chrome/26.0.1403.0 Safari/537.30*/
const Device UA2512453 = {S27039, 111, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3658.0.0) AppleWebKit/537.30 (KHTML, like Gecko) Chrome/26.0.1403.0 Safari/537.30*/
const Device UA2513389 = {S27040, 109, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3701.0.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.4 Safari/537.31*/
const Device UA2516469 = {S27041, 111, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3701.0.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.4 Safari/537.31*/
const Device UA2536141 = {S27042, 111, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3701.0.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.4 Safari/537.31*/
const Device UA2537297 = {S27043, 111, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3701.30.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.15 Safari/537.31*/
const Device UA2537299 = {S27044, 113, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3701.30.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.15 Safari/537.31*/
const Device UA2537306 = {S27045, 113, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3701.30.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.15 Safari/537.31*/
const Device UA2539128 = {S27046, 113, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3701.23.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.12 Safari/537.31*/
const Device UA2539455 = {S27047, 113, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3701.23.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.12 Safari/537.31*/
const Device UA2539617 = {S27048, 111, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3701.23.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.12 Safari/537.31*/
const Device UA2570203 = {S27049, 111, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3701.42.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.28 Safari/537.31*/
const Device UA2570207 = {S27050, 113, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3701.42.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.28 Safari/537.31*/
const Device UA2570224 = {S27051, 113, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3701.42.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.28 Safari/537.31*/
const Device UA2570278 = {S27052, 113, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3701.30.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.15 Safari/537.31*/
const Device UA2571013 = {S27053, 111, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3701.46.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.30 Safari/537.31*/
const Device UA2571021 = {S27054, 113, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3701.46.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.30 Safari/537.31*/
const Device UA2571014 = {S27055, 113, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3701.46.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.30 Safari/537.31*/
const Device UA2585777 = {S27056, 111, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3701.62.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Safari/537.31*/
const Device UA2585787 = {S27057, 113, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3701.62.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Safari/537.31*/
const Device UA2585790 = {S27058, 113, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3701.62.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Safari/537.31*/
const Device UA2602797 = {S27059, 111, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3701.76.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.51 Safari/537.31*/
const Device UA2605201 = {S27060, 113, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3701.76.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.51 Safari/537.31*/
const Device UA2605244 = {S27061, 113, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3701.76.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.51 Safari/537.31*/
const Device UA2673520 = {S27062, 113, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3701.81.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.57 Safari/537.31*/
const Device UA2673832 = {S27063, 111, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3701.81.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.57 Safari/537.31*/
const Device UA2675967 = {S27064, 111, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3701.81.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.57 Safari/537.31*/
const Device UA2675970 = {S27065, 113, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3701.81.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.57 Safari/537.31*/
const Device UA2675959 = {S27066, 113, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3701.81.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.57 Safari/537.31*/
const Device UA2680304 = {S27067, 113, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3701.81.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.57 Safari/537.31*/
const Device UA2738235 = {S27068, 113, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3701.90.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31*/
const Device UA2738827 = {S27069, 114, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3701.111.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.66 Safari/537.31*/
const Device UA2781865 = {S27070, 114, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3701.109.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA2782310 = {S27071, 114, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3701.112.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.67 Safari/537.31*/
const Device UA2861491 = {S27072, 121, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3701.81.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.57 Safari/537.31,gzip(gfe)*/
const Device UA2865746 = {S27073, 152, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3701.81.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.57 Safari/537.31,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2866095 = {S27074, 123, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3701.81.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.57 Safari/537.31,gzip(gfe)*/
const Device UA2952050 = {S27075, 112, { &P15364,&P6596,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3545.0.2013) AppleWebKit/537.26 (KHTML, like Gecko) Chrome/26.0.1382.0 Safari/537.26*/
const Device UA2009439 = {S27076, 105, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.1+ (KHTML, like Gecko) Version/10.0.0.1337 Mobile Safari/537.1+*/
const Device UA2173916 = {S27077, 105, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.3+ (KHTML, like Gecko) Version/10.0.0.1337 Mobile Safari/537.3+*/
const Device UA2235232 = {S27078, 104, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.3+ (KHTML, like Gecko) Version/10.0.9.388 Mobile Safari/537.3+*/
const Device UA2256932 = {S27079, 106, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.785 Mobile Safari/537.10+*/
const Device UA2256961 = {S27080, 106, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.902 Mobile Safari/537.10+*/
const Device UA2354062 = {S27081, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.0.1337 Mobile Safari/537.10+*/
const Device UA2357847 = {S27082, 106, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.945 Mobile Safari/537.10+*/
const Device UA2359510 = {S27083, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1077 Mobile Safari/537.10+*/
const Device UA2368256 = {S27084, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1287 Mobile Safari/537.10+*/
const Device UA2368536 = {S27085, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1179 Mobile Safari/537.10+*/
const Device UA2387673 = {S27086, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1407 Mobile Safari/537.10+*/
const Device UA2387796 = {S27087, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1473 Mobile Safari/537.10+*/
const Device UA2404502 = {S27088, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1425 Mobile Safari/537.10+*/
const Device UA2406156 = {S27089, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1103 Mobile Safari/537.10+*/
const Device UA2406953 = {S27090, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1595 Mobile Safari/537.10+*/
const Device UA2408244 = {S27091, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1673 Mobile Safari/537.10+*/
const Device UA2426348 = {S27092, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1655 Mobile Safari/537.10+*/
const Device UA2429926 = {S27093, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1994 Mobile Safari/537.10+*/
const Device UA2431750 = {S27094, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1757 Mobile Safari/537.10+*/
const Device UA2431818 = {S27095, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1675 Mobile Safari/537.10+*/
const Device UA2432846 = {S27096, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2108 Mobile Safari/537.10+*/
const Device UA2433985 = {S27097, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1904 Mobile Safari/537.10+*/
const Device UA2461862 = {S27098, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2342 Mobile Safari/537.10+*/
const Device UA2461993 = {S27099, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2264 Mobile Safari/537.10+*/
const Device UA2462131 = {S27100, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2222 Mobile Safari/537.10+*/
const Device UA2470227 = {S27101, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2318 Mobile Safari/537.10+*/
const Device UA2470801 = {S27102, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2360 Mobile Safari/537.10+*/
const Device UA2470953 = {S27103, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2372 Mobile Safari/537.10+*/
const Device UA2471706 = {S27104, 106, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.75 Mobile Safari/537.10+*/
const Device UA2487981 = {S27105, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2320 Mobile Safari/537.10+*/
const Device UA2488283 = {S27106, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.261 Mobile Safari/537.10+*/
const Device UA2490609 = {S27107, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.252 Mobile Safari/537.10+*/
const Device UA2492599 = {S27108, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.225 Mobile Safari/537.10+*/
const Device UA2492616 = {S27109, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.263 Mobile Safari/537.10+*/
const Device UA2508510 = {S27110, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2466 Mobile Safari/537.10+*/
const Device UA2509154 = {S27111, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.357 Mobile Safari/537.10+*/
const Device UA2512188 = {S27112, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.243 Mobile Safari/537.10+*/
const Device UA2513068 = {S27113, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.462 Mobile Safari/537.10+*/
const Device UA2515428 = {S27114, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.552 Mobile Safari/537.10+*/
const Device UA2531886 = {S27115, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.576 Mobile Safari/537.10+*/
const Device UA2535174 = {S27116, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.672 Mobile Safari/537.10+*/
const Device UA2536154 = {S27117, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.588 Mobile Safari/537.10+*/
const Device UA2539847 = {S27118, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.648 Mobile Safari/537.10+*/
const Device UA2564732 = {S27119, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2709 Mobile Safari/537.10+*/
const Device UA2565236 = {S27120, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.690 Mobile Safari/537.10+*/
const Device UA2571382 = {S27121, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.684 Mobile Safari/537.10+*/
const Device UA2580818 = {S27122, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2598 Mobile Safari/537.10+*/
const Device UA2583058 = {S27123, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.738 Mobile Safari/537.10+*/
const Device UA2583116 = {S27124, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.702 Mobile Safari/537.10+*/
const Device UA2604451 = {S27125, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2743 Mobile Safari/537.10+*/
const Device UA2657963 = {S27126, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.1754 Mobile Safari/537.10+*/
const Device UA2667234 = {S27127, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.768 Mobile Safari/537.10+*/
const Device UA2667886 = {S27128, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.822 Mobile Safari/537.10+*/
const Device UA2692131 = {S27129, 104, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.3+ (KHTML, like Gecko) Version/10.0.9.386 Mobile Safari/537.3+*/
const Device UA2706450 = {S27130, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.696 Mobile Safari/537.10+*/
const Device UA2837108 = {S27131, 104, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.3+ (KHTML, like Gecko) Version/10.0.9.443 Mobile Safari/537.3+*/
const Device UA2853237 = {S27132, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.606 Mobile Safari/537.10+*/
const Device UA2867847 = {S27133, 112, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - LinkedInBlackBerry/10.0.3 (4) Z10 STL100-3/10.0.10.672 (Research In Motion;RIM BlackBerry Device;Z10;STL100-3;1)*/
const Device UA2879660 = {S27134, 112, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - LinkedInBlackBerry/10.0.3 (4) Z10 STL100-4/10.0.9.2743 (Research In Motion;RIM BlackBerry Device;Z10;STL100-4;1)*/
const Device UA2957373 = {S27135, 112, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - LinkedInBlackBerry/10.0.3 (4) Z10 STL100-1/10.0.10.648 (Research In Motion;RIM BlackBerry Device;Z10;STL100-1;1)*/
const Device UA3044529 = {S27136, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.9.2442 Mobile Safari/537.10+*/
const Device UA3049562 = {S27137, 112, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - LinkedInBlackBerry/10.0.3 (4) Z10 STL100-2/10.0.10.690 (Research In Motion;RIM BlackBerry Device;Z10;STL100-2;1)*/
const Device UA3118980 = {S27138, 107, { &P19794,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.10.291 Mobile Safari/537.10+*/
const Device UA196656 = {S27139, 142, { &P16757,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA229384 = {S27140, 142, { &P16757,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA249245 = {S27141, 136, { &P16757,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SCH-I500 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA249849 = {S27142, 142, { &P16757,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA316054 = {S27143, 136, { &P16757,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-I500 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA329487 = {S27144, 136, { &P16757,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I500 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA354668 = {S27145, 141, { &P16757,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.5; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA355381 = {S27146, 141, { &P16757,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I500Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA355609 = {S27147, 140, { &P16757,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ko-; SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369329 = {S27148, 77, { &P16757,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.5; en_US; SCH-I500 Build/GINGERBREAD.EI20)*/
const Device UA451687 = {S27149, 66, { &P16757,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SCH-I500 Build/GINGERBREAD)*/
const Device UA576353 = {S27150, 142, { &P16757,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA637636 = {S27151, 66, { &P16757,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SCH-I500 Build/GINGERBREAD)*/
const Device UA639081 = {S27152, 141, { &P16757,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.5; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA644003 = {S27153, 141, { &P16757,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us;SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA693738 = {S27154, 141, { &P16757,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA900376 = {S27155, 154, { &P16757,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 CSOUTH-i500*/
const Device UA2021066 = {S27156, 149, { &P16757,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2373212 = {S27157, 66, { &P16757,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-I500 Build/GINGERBREAD)*/
const Device UA2588110 = {S27158, 170, { &P16757,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android BeigePhone Fascinate v3.0 - 2.3.5; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627034 = {S27159, 131, { &P13144,&P7106,&P15829,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9860; en-GB) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.342 Mobile Safari/534.11+*/
const Device UA2425873 = {S27160, 77, { &P13144,&P7106,&P15829,&P18092 }};	/*Useragent - BlackBerry9860/7.1.0.694 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102*/
const Device UA92392 = {S27161, 119, { &P17595,&P17294,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA92821 = {S27162, 117, { &P17595,&P17294,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/528.16 (KHTML, like Gecko) Version/4.0 Safari/528.16*/
const Device UA93032 = {S27163, 125, { &P17595,&P17294,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA98985 = {S27164, 126, { &P17595,&P17294,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA130672 = {S27165, 121, { &P17595,&P17294,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19*/
const Device UA130899 = {S27166, 117, { &P17595,&P17294,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA273497 = {S27167, 119, { &P17595,&P17294,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/528.18.1 (KHTML, like Gecko) Version/4.0 Safari/528.17*/
const Device UA488917 = {S27168, 129, { &P17595,&P17294,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/528.16 (KHTML, like Gecko) Version/4 Public Beta Safari/528.16*/
const Device UA534759 = {S27169, 122, { &P17595,&P17294,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/528.4+ (KHTML, like Gecko) Version/4.0dp1 Safari/526.11.2*/
const Device UA2568007 = {S27170, 111, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3797.0.0) AppleWebKit/537.33 (KHTML, like Gecko) Chrome/27.0.1430.0 Safari/537.33*/
const Device UA2568021 = {S27171, 109, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3797.0.0) AppleWebKit/537.33 (KHTML, like Gecko) Chrome/27.0.1430.0 Safari/537.33*/
const Device UA2568191 = {S27172, 111, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3797.0.0) AppleWebKit/537.33 (KHTML, like Gecko) Chrome/27.0.1430.0 Safari/537.33*/
const Device UA2569177 = {S27173, 111, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3833.0.0) AppleWebKit/537.33 (KHTML, like Gecko) Chrome/27.0.1438.8 Safari/537.33*/
const Device UA2569175 = {S27174, 111, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3833.0.0) AppleWebKit/537.33 (KHTML, like Gecko) Chrome/27.0.1438.8 Safari/537.33*/
const Device UA2569185 = {S27175, 109, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3833.0.0) AppleWebKit/537.33 (KHTML, like Gecko) Chrome/27.0.1438.8 Safari/537.33*/
const Device UA2586566 = {S27176, 111, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3881.4.0) AppleWebKit/537.35 (KHTML, like Gecko) Chrome/27.0.1446.5 Safari/537.35*/
const Device UA2601743 = {S27177, 109, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3881.4.0) AppleWebKit/537.35 (KHTML, like Gecko) Chrome/27.0.1446.5 Safari/537.35*/
const Device UA2604366 = {S27178, 111, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3912.4.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.6 Safari/537.36*/
const Device UA2604419 = {S27179, 109, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3912.4.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.6 Safari/537.36*/
const Device UA2604527 = {S27180, 111, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3912.4.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.6 Safari/537.36*/
const Device UA2606189 = {S27181, 111, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3881.4.0) AppleWebKit/537.35 (KHTML, like Gecko) Chrome/27.0.1446.5 Safari/537.35*/
const Device UA2606444 = {S27182, 112, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3887.0.2013) AppleWebKit/537.35 (KHTML, like Gecko) Chrome/27.0.1448.0 Safari/537.35*/
const Device UA2673956 = {S27183, 113, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3912.10.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.36 Safari/537.36*/
const Device UA2674427 = {S27184, 113, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3912.10.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.36 Safari/537.36*/
const Device UA2675500 = {S27185, 111, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3912.10.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.36 Safari/537.36*/
const Device UA2678942 = {S27186, 113, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3912.23.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.46 Safari/537.36*/
const Device UA2679179 = {S27187, 111, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3912.23.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.46 Safari/537.36*/
const Device UA2679459 = {S27188, 113, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3912.33.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.56 Safari/537.36*/
const Device UA2679462 = {S27189, 113, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3912.33.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.56 Safari/537.36*/
const Device UA2679464 = {S27190, 111, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3912.33.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.56 Safari/537.36*/
const Device UA2679531 = {S27191, 113, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3912.23.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.46 Safari/537.36*/
const Device UA2710745 = {S27192, 113, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3912.52.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.76 Safari/537.36*/
const Device UA2710750 = {S27193, 113, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3912.52.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.76 Safari/537.36*/
const Device UA2712036 = {S27194, 111, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3912.52.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.76 Safari/537.36*/
const Device UA2714979 = {S27195, 113, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3912.43.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.69 Safari/537.36*/
const Device UA2714981 = {S27196, 111, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3912.43.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.69 Safari/537.36*/
const Device UA2714970 = {S27197, 113, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3912.43.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.69 Safari/537.36*/
const Device UA2719387 = {S27198, 113, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3912.63.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.83 Safari/537.36*/
const Device UA2719484 = {S27199, 111, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3912.63.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.83 Safari/537.36*/
const Device UA2719486 = {S27200, 113, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3912.63.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.83 Safari/537.36*/
const Device UA2741411 = {S27201, 112, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3912.79.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.103 Safari/537.36*/
const Device UA2741412 = {S27202, 114, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3912.79.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.103 Safari/537.36*/
const Device UA2741413 = {S27203, 114, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3912.79.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.103 Safari/537.36*/
const Device UA2855816 = {S27204, 112, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3733.0.2013) AppleWebKit/537.32 (KHTML, like Gecko) Chrome/27.0.1417.2 Safari/537.32*/
const Device UA2869864 = {S27205, 114, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3912.93.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.112 Safari/537.36*/
const Device UA2873810 = {S27206, 115, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS armv7l 3912.101.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2873926 = {S27207, 113, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3912.101.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2874095 = {S27208, 115, { &P15364,&P6596,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3912.101.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA49256 = {S27209, 142, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49258 = {S27210, 142, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA50020 = {S27211, 143, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; SGH-T989D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79282 = {S27212, 141, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us;SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79294 = {S27213, 141, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.5; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79283 = {S27214, 141, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T989Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA359103 = {S27215, 66, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-T989 Build/GINGERBREAD)*/
const Device UA359354 = {S27216, 141, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us;SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA363753 = {S27217, 141, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA369220 = {S27218, 38, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Android/2.3.6; samsung SGH-T989; 2.2.0*/
const Device UA369556 = {S27219, 97, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - com.google.android.youtube/2.3.4(Linux; U; Android 2.3.6; en_US; SGH-T989 Build/GINGERBREAD) gzip*/
const Device UA452929 = {S27220, 141, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA580910 = {S27221, 130, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA616494 = {S27222, 67, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SGH-T989D Build/GINGERBREAD)*/
const Device UA622536 = {S27223, 142, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633506 = {S27224, 66, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SGH-T989 Build/GINGERBREAD)*/
const Device UA649723 = {S27225, 141, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.6; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA658014 = {S27226, 148, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD 2.3.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA682976 = {S27227, 141, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA704335 = {S27228, 140, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA737247 = {S27229, 155, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/Hercules Latest Greatest) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021377 = {S27230, 141, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.5; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023559 = {S27231, 149, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2024967 = {S27232, 34, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Android/1.0 (SGH-T989 GINGERBREAD)*/
const Device UA2418617 = {S27233, 150, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30*/
const Device UA2789765 = {S27234, 135, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3084868 = {S27235, 141, { &P12972,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; SGH-T989D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA412897 = {S27236, 118, { &P17595,&P17297,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.65 Safari/535.11*/
const Device UA412941 = {S27237, 118, { &P17595,&P17297,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11*/
const Device UA412963 = {S27238, 118, { &P17595,&P17297,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11*/
const Device UA428601 = {S27239, 118, { &P17595,&P17297,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11*/
const Device UA613342 = {S27240, 129, { &P17595,&P17297,&P17830,&P18092 }};	/*Useragent - User-Agent:Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.83 Safari/535.11*/
const Device UA634223 = {S27241, 118, { &P17595,&P17297,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA825498 = {S27242, 138, { &P17595,&P17297,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.10 (KHTML, like Gecko) Chrome/17.0.925.0 QQBrowser/1.0.0.1096 Safari/535.10*/
const Device UA899135 = {S27243, 139, { &P17595,&P17297,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.3.0.1804 Safari/535.11*/
const Device UA904085 = {S27244, 139, { &P17595,&P17297,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.4.0.2117 Safari/535.11*/
const Device UA1979307 = {S27245, 138, { &P17595,&P17297,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.10 (KHTML, like Gecko) Chrome/17.0.925.0 QQBrowser/1.0.0.1096 Safari/535.10*/
const Device UA1980022 = {S27246, 138, { &P17595,&P17297,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.10 (KHTML, like Gecko) Chrome/17.0.925.0 QQBrowser/1.1.0.1475 Safari/535.10*/
const Device UA2200563 = {S27247, 118, { &P17595,&P17297,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA2761137 = {S27248, 139, { &P17595,&P17297,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.4.0.2117 Safari/535.11*/
const Device UA2878148 = {S27249, 139, { &P17595,&P17297,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.3.0.1804 Safari/535.11*/
const Device UA1890034 = {S27250, 91, { &P18182,&P5448,&P17371,&P18092 }};	/*Useragent - Opera/9.80 (Windows Mobile; Opera Mini/5.1.21595/25.657; U; en) Presto/2.5.25 Version/10.54*/
const Device UA2431169 = {S27251, 93, { &P18182,&P5448,&P17371,&P18092 }};	/*Useragent - Opera/9.80 (Windows Mobile; Opera Mini/5.1.21594/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA177025 = {S27252, 143, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; SHW-M250S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA178990 = {S27253, 143, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; SHW-M250L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA192368 = {S27254, 143, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; SHW-M250K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA206994 = {S27255, 67, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SHW-M250S Build/GINGERBREAD)*/
const Device UA208995 = {S27256, 143, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; SHW-M250L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA231072 = {S27257, 143, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; SHW-M250S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA231271 = {S27258, 143, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; SHW-M250K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA242284 = {S27259, 143, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; SHW-M250S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA249425 = {S27260, 143, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; SHW-M250L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250420 = {S27261, 143, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; SHW-M250K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA349345 = {S27262, 143, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHW-M250K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA356090 = {S27263, 143, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHW-M250S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA356427 = {S27264, 67, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHW-M250S Build/GINGERBREAD)*/
const Device UA368666 = {S27265, 141, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; vi-; SHW-M250L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA561449 = {S27266, 67, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M250S Build/GINGERBREAD)*/
const Device UA572557 = {S27267, 67, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SHW-M250S Build/GINGERBREAD)*/
const Device UA574159 = {S27268, 67, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SHW-M250K Build/GINGERBREAD)*/
const Device UA666018 = {S27269, 143, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHW-M250L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA692480 = {S27270, 67, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHW-M250K Build/GINGERBREAD)*/
const Device UA704846 = {S27271, 67, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SHW-M250L Build/GINGERBREAD)*/
const Device UA711653 = {S27272, 67, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SHW-M250K Build/GINGERBREAD)*/
const Device UA732137 = {S27273, 143, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SHW-M240S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1893636 = {S27274, 67, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M250L Build/GINGERBREAD)*/
const Device UA1903186 = {S27275, 67, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M250K Build/GINGERBREAD)*/
const Device UA2020163 = {S27276, 67, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M240S Build/GINGERBREAD)*/
const Device UA2020817 = {S27277, 67, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SHW-M250L Build/GINGERBREAD)*/
const Device UA2021745 = {S27278, 67, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHW-M250L Build/GINGERBREAD)*/
const Device UA2033449 = {S27279, 143, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M240S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2838005 = {S27280, 64, { &P14854,&P5629,&P15760,&P18092 }};	/*Useragent - DaumMobileApp (SHW-M250K; U; Android 2.3.6|10; ko-kr) DaumCafe/6*/
const Device UA152480 = {S27281, 106, { &P15364,&P18110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-us) AppleWebKit/531.2+ (KHTML, like Gecko) Version/5.0 Safari/531.2+*/
const Device UA255683 = {S27282, 105, { &P15364,&P18110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Intel X86; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA453159 = {S27283, 103, { &P15364,&P18110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux; ko-KR) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+*/
const Device UA572693 = {S27284, 105, { &P15364,&P18110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Grid OS 1.0; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Version/5.0 Safari/533.1*/
const Device UA623356 = {S27285, 92, { &P15364,&P18110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.7 (KHTML, like Gecko) Version/5.0 Safari/534.7*/
const Device UA626753 = {S27286, 101, { &P15364,&P18110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA715741 = {S27287, 96, { &P15364,&P18110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.4+ (KHTML, like Gecko) Version/5.0 Safari/535.4+*/
const Device UA902913 = {S27288, 98, { &P15364,&P18110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux; ko-KR) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+*/
const Device UA2209453 = {S27289, 122, { &P15364,&P18110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-us) AppleWebKit/534.16+ (KHTML, like Gecko) Version/5.0 Safari/534.16+ Epiphany/2.30.6*/
const Device UA2308308 = {S27290, 109, { &P15364,&P18110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.16+ (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA2656852 = {S27291, 96, { &P15364,&P18110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.20+ (KHTML, like Gecko) Version/5.0 Safari/535.20+*/
const Device UA2762263 = {S27292, 118, { &P15364,&P18110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux; en-us) AppleWebKit/535.22 (KHTML, like Gecko) Version/5.0 Safari/535.22 LinutopKiosk/5.0.2*/
const Device UA2833886 = {S27293, 90, { &P15364,&P18110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA925325 = {S27294, 125, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/21.0.1180.60) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.60 Safari/537.1*/
const Device UA925328 = {S27295, 132, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/21.0.1180.60) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.60 Safari/537.1*/
const Device UA1995372 = {S27296, 132, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/21.0.1180.49) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.49 Safari/537.1*/
const Device UA1997541 = {S27297, 132, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/21.0.1180.77) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.77 Safari/537.1*/
const Device UA1998448 = {S27298, 125, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/21.0.1180.77) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.77 Safari/537.1*/
const Device UA2000911 = {S27299, 132, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/21.0.1180.75) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1*/
const Device UA2003527 = {S27300, 125, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/21.0.1180.75) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1*/
const Device UA2003718 = {S27301, 132, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/21.0.1180.83) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1*/
const Device UA2005283 = {S27302, 125, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/21.0.1180.79) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.79 Safari/537.1*/
const Device UA2005399 = {S27303, 132, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/21.0.1180.79) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.79 Safari/537.1*/
const Device UA2007127 = {S27304, 125, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/21.0.1180.83) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1*/
const Device UA2008820 = {S27305, 197, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/21.0.1180.79; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2009818 = {S27306, 163, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/21.0.1180.79; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729)*/
const Device UA2010180 = {S27307, 176, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/21.0.1180.79; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2015626 = {S27308, 96, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/21.0.1180.89)*/
const Device UA2015631 = {S27309, 89, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/21.0.1180.89)*/
const Device UA2016032 = {S27310, 132, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/21.0.1180.89) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2016051 = {S27311, 125, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/21.0.1180.89) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2189938 = {S27312, 191, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/21.0.1180.89; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA2201688 = {S27313, 166, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; chromeframe/21.0.1180.89)*/
const Device UA2343990 = {S27314, 183, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/21.0.1180.89; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2345592 = {S27315, 197, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/21.0.1180.79; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2403119 = {S27316, 242, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/21.0.1180.89; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.2; MS-RTC EA 2; MS-RTC LM 8)*/
const Device UA2470507 = {S27317, 316, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/21.0.1180.83; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; MRA 5.7 (build 03796); SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.2)*/
const Device UA2500878 = {S27318, 227, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; chromeframe/21.0.1180.83; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; .NET4.0C; .NET4.0E)*/
const Device UA2525449 = {S27319, 242, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SV1; chromeframe/21.0.1180.83; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA2649121 = {S27320, 209, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/21.0.1180.89; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2657255 = {S27321, 97, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/21.0.1180.89)*/
const Device UA2731473 = {S27322, 237, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/21.0.1180.83; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8; Tablet PC 2.0)*/
const Device UA2845981 = {S27323, 90, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/21.0.1180.89)*/
const Device UA2952516 = {S27324, 235, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/21.0.1180.75; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3)*/
const Device UA3086371 = {S27325, 227, { &P15364,&P7133,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SV1; chromeframe/21.0.1180.83; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; .NET4.0C; .NET4.0E)*/
const Device UA7907 = {S27326, 143, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB525 Build/3.4.2-107_JDN-4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA7968 = {S27327, 139, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; ME525 Build/JDGC_1.11.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA7969 = {S27328, 143, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB525 Build/3.4.2-107_JDN-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA7976 = {S27329, 143, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; MB525 Build/3.4.2-74_JST-84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA8007 = {S27330, 138, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME525 Build/JDGC_2.6.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA8009 = {S27331, 142, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; MB525 Build/3.4.2-107_JDN-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA9509 = {S27332, 136, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB525 Build/3.4.2-90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA9533 = {S27333, 139, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; ME525 Build/JDGC_0.26.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10480 = {S27334, 137, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MB525 Build/3.4.2-117) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10486 = {S27335, 136, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; MB525 Build/3.4.3-11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10490 = {S27336, 135, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ru-; MB525 Build/3.4.2-155) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10506 = {S27337, 137, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MB525 Build/3.4.2-145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10510 = {S27338, 137, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB525 Build/3.4.2-131) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10548 = {S27339, 137, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; MB525 Build/3.4.2-164) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10557 = {S27340, 198, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-MB525/Blur_Version.34.4.706.MB525.TELUS.en.CA Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; MB525 Build/3.4.2-107_JDNL-6_R02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10560 = {S27341, 137, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB525 Build/3.4.2-177) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10561 = {S27342, 137, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-au; MB525 Build/3.4.2-145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10580 = {S27343, 201, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-MB525/Blur_Version.34.4.709.MB525.AmericaMovil.en.01 Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; MB525 Build/3.4.2-107_JDNL-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10575 = {S27344, 201, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-MB525/Blur_Version.34.4.707.MB525.Personal.en.AR Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; MB525 Build/3.4.2-107_JDNL-7_R02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11638 = {S27345, 143, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB525 Build/3.4.2-107_JDN-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA39297 = {S27346, 198, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-MB525/Blur_Version.34.4.706.MB525.TELUS.en.CA Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; MB525 Build/3.4.2-107_JDNL-6_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA39411 = {S27347, 144, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; he-il; MB525 Build/JEM_3.4.3-36-1.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA39421 = {S27348, 135, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; cs-; MB525 Build/3.4.2-179) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52101 = {S27349, 142, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB525Build/3.4.2-107_JDN-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52116 = {S27350, 138, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME525 Build/JDGC_2.5.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52119 = {S27351, 142, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.1; en-us; MB525 Build/3.4.2-107_JDN-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52133 = {S27352, 139, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME525 Build/JDGC_2.10.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52135 = {S27353, 142, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB525 Build/3.4.2-107_JDN-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA52136 = {S27354, 142, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;MB525 Build/3.4.2-107_JDN-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52612 = {S27355, 144, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB525 Build/JEM_3.4.3-36-1.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52626 = {S27356, 141, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; MotoMB525 Build/3.4.2-179) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52649 = {S27357, 140, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB525 Build/JEM_3.4.3-21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52661 = {S27358, 142, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB525 Build/JEM_3.4.3-36-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52664 = {S27359, 145, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB525 Build/goapk_cg_3.4.3-11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52633 = {S27360, 195, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-MB525/Blur_Version.34.4.729.MB525.Latam.en.01 Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; MB525 Build/3.4.2-107_JDNL-29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52669 = {S27361, 135, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MB525 Build/3.4.3-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52670 = {S27362, 137, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB525 Build/3.4.2-179) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52672 = {S27363, 142, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB525 Build/JEM_3.4.3-33-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52717 = {S27364, 137, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; MB525 Build/3.4.2-155) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52725 = {S27365, 140, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB525 Build/JEM_3.4.3-27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61249 = {S27366, 144, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB525 Build/JEM_3.4.3-36-1.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70581 = {S27367, 140, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB525 Build/JEM_3.4.3-20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70638 = {S27368, 205, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-MB525/Blur_Version.34.4.802.MB525.AmericaMovil.en.01 Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; MB525 Build/3.4.2-108_JDNL-2_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71590 = {S27369, 201, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-MB525/Blur_Version.34.4.802.MB525.Movistar.en.01 Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; MB525 Build/3.4.2-108_JDNL-2_R02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA376468 = {S27370, 67, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; MB525 Build/3.4.2-107_JDN-9)*/
const Device UA395372 = {S27371, 61, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; MB525 Build/3.4.2-177)*/
const Device UA693981 = {S27372, 130, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB525 Build/JEM_3.4.3-33-1) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA695461 = {S27373, 125, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MB525 Build/3.4.2-117) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA713419 = {S27374, 127, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME525 Build/JDGC_2.10.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA721163 = {S27375, 127, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME525 Build/JDGC_2.13.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA744383 = {S27376, 139, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME525 Build/JDGC_2.13.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA748307 = {S27377, 194, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-MB525/Blur_Version.34.4.806.MB525.Latam.en.01 Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB525 Build/3.4.2-108_JDNL-6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA851015 = {S27378, 201, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-MB525/Blur_Version.34.4.805.MB525.AmericaMovil.en.01 Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; MB525 Build/3.4.2-108_JDNL-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA857872 = {S27379, 131, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB525 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA858094 = {S27380, 132, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB525 Build/JEM_3.4.3-36-1.7) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1936487 = {S27381, 127, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; ME525 Build/JDGC_1.11.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1960500 = {S27382, 132, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME525 Build/JDGC_2.10.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1983269 = {S27383, 201, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-MB525/Blur_Version.34.4.807.MB525.AmericaMovil.en.BR Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; MB525 Build/3.4.2-108_JDNL-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020406 = {S27384, 125, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB525 Build/3.4.2-177) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2021344 = {S27385, 126, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME525 Build/JDGC_2.6.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2021668 = {S27386, 127, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; ME525 Build/JDGC_0.26.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022170 = {S27387, 123, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MB525 Build/3.4.3-3) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2024913 = {S27388, 132, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME525 Build/JDGC_2.13.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2026829 = {S27389, 137, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB525 Build/JEM_3.4.3-36-1.7) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2027104 = {S27390, 130, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MB525 Build/3.4.2-117) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2314532 = {S27391, 132, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; ME525 Build/JDGC_1.11.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2394721 = {S27392, 130, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB525 Build/3.4.2-177) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2495465 = {S27393, 135, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB525 Build/JEM_3.4.3-33-1) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2786837 = {S27394, 139, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME525 Build/JDGC_2.12.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2810153 = {S27395, 131, { &P13505,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME525 Build/JDGC_2.6.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA249317 = {S27396, 130, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; SGH-T859 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA361182 = {S27397, 59, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; SGH-T859 Build/HTJ85B)*/
const Device UA622893 = {S27398, 130, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA624999 = {S27399, 130, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7510 Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA634534 = {S27400, 59, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7510 Build/HTJ85B)*/
const Device UA691292 = {S27401, 137, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA825066 = {S27402, 148, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7510-ORANGE/P7510BVKL2 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2021731 = {S27403, 128, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2025654 = {S27404, 151, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7510 Build/HTJ85B; 1280*752) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2608138 = {S27405, 142, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7510 Build/HTJ85B) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2611230 = {S27406, 128, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7510 Build/HTJ85B) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2611343 = {S27407, 123, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7510 Build/HTJ85B) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2611679 = {S27408, 132, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7510 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2611994 = {S27409, 135, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2613165 = {S27410, 60, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.0; 5) GT-P7510 (Android 3.2, SDK 13)*/
const Device UA2613732 = {S27411, 72, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/2.0.11 (Linux; U; Android 3.2; en-US; GT-P7510 Build/HTJ85B.UEKMP)*/
const Device UA2615339 = {S27412, 115, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Android 3.2; en-us; GT-P7510 Build/HTJ85B) FreeWheelAdManager/5.0.3-r9054-X;com.gotv.crackle.handset/X*/
const Device UA2616063 = {S27413, 268, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.7.2;FBPN/com.facebook.katana;FBDV/GT-P7510;FBSV/3.2;FBDM/{density=1.0,width=1280,height=752};FBCR/;FBLC/en_US;FB_FW/1;]*/
const Device UA2616125 = {S27414, 71, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Vyrso/2.1.2 (Linux; U; Android 3.2; en-US; GT-P7510 Build/HTJ85B.VLKL2)*/
const Device UA2616700 = {S27415, 92, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 3.2; SAMSUNG GT-P7510 Build/HONEYCOMB_MR2);*/
const Device UA2616955 = {S27416, 72, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Biblia/2.1.2 (Linux; U; Android 3.2; es-ES; GT-P7510 Build/HTJ85B.UBKL2)*/
const Device UA2617184 = {S27417, 269, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.11;FBDM/{density=1.0,width=1280,height=752};FBLC/en_US;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P7510;FBSV/3.2;]*/
const Device UA2619881 = {S27418, 126, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Groupon/2.1.2483 (Android 3.2; Samsung GT-P7510 / Samsung GT-P7510; )[preload=false;locale=en_US;clientidbase=android-samsung]*/
const Device UA2622026 = {S27419, 276, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7510 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 parameters/{scr=752_1280,cm=1,ql=l,imei=94:63:D1:F7:7E:E5,bm=1,eng=adrmini13,mdm=1,mdl=GT-P7510,ps=1} tiantian(securitypay) tiantian(tenpay)*/
const Device UA2622588 = {S27420, 73, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 3.2; en-US; GT-P7510 Build/HTJ85B.UEKMM)*/
const Device UA2623466 = {S27421, 388, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, likeb_etc_status=%7B%22c%22%3A%7B%7D%7D; _qubitTracker=X.X; _qPageNum_harveysfu2o5t2hw55r0pl5155; X-Mapping-ihjijfgc=6021E6A5F9C100D0B84AE856E8CBBCFE; HomestyleBasket=X; HViewID=c4e40b3a-65b1-4217-a490-af7710fdf0a3; _qubitTracker_s=X.X; WT_FPC=id=216e37d2f1d84b71cX:lv=X:ss=X; __utma=X.X.X.X.135*/
const Device UA2626499 = {S27422, 91, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - com.google.android.youtube/4.2.16(Linux; U; Android 3.2; en_US; GT-P7510 Build/HTJ85B) gzip*/
const Device UA2628524 = {S27423, 52, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Android/4.7 (Linux; U; Android 3.2; en_US; GT-P7510)*/
const Device UA2629419 = {S27424, 60, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.1; 9) GT-P7510 (Android 3.2, SDK 13)*/
const Device UA2629663 = {S27425, 78, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - VKAndroidApp/2.7.6-64 (Android 3.2; SDK 13; armeabi-v7a; samsung GT-P7510; ru)*/
const Device UA2809036 = {S27426, 137, { &P12879,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; SGH-T859 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA114773 = {S27427, 77, { &P15364,&P7127,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.0; DigExt; .NET CLR 1.1.4322)*/
const Device UA157551 = {S27428, 147, { &P15364,&P7127,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.0; Trident/4.0; InfoPath.1; SV1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 3.0.04506.30)*/
const Device UA399866 = {S27429, 50, { &P15364,&P7127,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.0)*/
const Device UA487097 = {S27430, 183, { &P15364,&P7127,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; InfoPath.2; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET CLR 3.0.30618)*/
const Device UA1994379 = {S27431, 66, { &P15364,&P7127,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0;GB)*/
const Device UA2165629 = {S27432, 68, { &P15364,&P7127,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; SV1)*/
const Device UA2441722 = {S27433, 97, { &P15364,&P7127,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)*/
const Device UA2687007 = {S27434, 178, { &P15364,&P7127,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2722730 = {S27435, 62, { &P15364,&P7127,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.00; Windows NT 5.0; MyIE 3.01)*/
const Device UA2787265 = {S27436, 73, { &P15364,&P7127,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; SV1; SV1)*/
const Device UA3042405 = {S27437, 73, { &P15364,&P7127,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; SV1; SV1)*/
const Device UA19561 = {S27438, 143, { &P14186,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5830-ORANGE Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA22996 = {S27439, 137, { &P14186,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; GT-S5830T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24095 = {S27440, 136, { &P14186,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24289 = {S27441, 137, { &P14186,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; GT-S5830L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43349 = {S27442, 137, { &P14186,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; GT-S5830L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48157 = {S27443, 137, { &P14186,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; GT-S5830B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61149 = {S27444, 137, { &P14186,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; GT-S5830B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90375 = {S27445, 131, { &P14186,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux U Android 2.2.1 en-gb GT-S5830 Build/FROYO) AppleWebKit/533.1(KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA189800 = {S27446, 60, { &P14186,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-S5830 Build/FROYO)*/
const Device UA450857 = {S27447, 61, { &P14186,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-S5830T Build/FROYO)*/
const Device UA566730 = {S27448, 61, { &P14186,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; GT-S5830L Build/FROYO)*/
const Device UA681566 = {S27449, 124, { &P14186,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-S5830 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA736063 = {S27450, 61, { &P14186,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-S5830L Build/FROYO)*/
const Device UA1969291 = {S27451, 129, { &P14186,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-S5830 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2019625 = {S27452, 134, { &P14186,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019701 = {S27453, 61, { &P14186,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-S5830B Build/FROYO)*/
const Device UA2743093 = {S27454, 134, { &P14186,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-S5830 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA92881 = {S27455, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA93275 = {S27456, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA94275 = {S27457, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA94403 = {S27458, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; es-es) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA94837 = {S27459, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; ja-jp) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA95948 = {S27460, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; nb-no) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA97812 = {S27461, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; fr-fr) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA98956 = {S27462, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA100810 = {S27463, 126, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.12*/
const Device UA106757 = {S27464, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-gb) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA110937 = {S27465, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; nl-nl) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA116641 = {S27466, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; nl-nl) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA117988 = {S27467, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; de-de) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA121407 = {S27468, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; fr-fr) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA121726 = {S27469, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; de-de) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA129486 = {S27470, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; es-es) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA135473 = {S27471, 123, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; en-us) AppleWebKit/525.19 (KHTML, like Gecko) Version/3.2.1 Safari/525.19*/
const Device UA136837 = {S27472, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; it-it) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA136845 = {S27473, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA142117 = {S27474, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; it-it) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA156086 = {S27475, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; nb-no) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA157723 = {S27476, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; sv-se) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA172046 = {S27477, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; pt-br) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA195618 = {S27478, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; pt-br) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA220048 = {S27479, 116, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.12*/
const Device UA228298 = {S27480, 117, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA229412 = {S27481, 117, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA260425 = {S27482, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ja-jp) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA261914 = {S27483, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; sv-se) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA264759 = {S27484, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA273271 = {S27485, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; zh-cn) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA275430 = {S27486, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; nl-nl) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA277130 = {S27487, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; de-de) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA277493 = {S27488, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; da-dk) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA278993 = {S27489, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ru-ru) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA282806 = {S27490, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; ru-ru) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA294895 = {S27491, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; fi-fi) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA299519 = {S27492, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; pt-br) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA399793 = {S27493, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; cs-cz) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA430979 = {S27494, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; ko-kr) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA431449 = {S27495, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; zh-cn) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA431515 = {S27496, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; pl-pl) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA431948 = {S27497, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-au) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA442064 = {S27498, 126, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; de-de) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.12*/
const Device UA442137 = {S27499, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; fr-fr) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA444304 = {S27500, 126, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; fr-fr) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.12*/
const Device UA469065 = {S27501, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; zh-tw) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA482646 = {S27502, 124, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; ar) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA490572 = {S27503, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; he-il) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA494802 = {S27504, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA496329 = {S27505, 126, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/3.2 Safari/525.26.12*/
const Device UA523078 = {S27506, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; ja-jp) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA538322 = {S27507, 126, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; sv-se) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.12*/
const Device UA599068 = {S27508, 125, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_8; ja-jp) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA626924 = {S27509, 121, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.25 (KHTML, like Gecko) Version/3.2 Safari/525.25*/
const Device UA683246 = {S27510, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; af-za) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA720620 = {S27511, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; fi-fi) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA724057 = {S27512, 123, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA729878 = {S27513, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; pl-pl) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA731210 = {S27514, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; pt-pt) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA743825 = {S27515, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; da-dk) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA761672 = {S27516, 125, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_8; ja-jp) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA764902 = {S27517, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; zh-tw) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA777144 = {S27518, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-au) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA786537 = {S27519, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-gb) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA790278 = {S27520, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; el-gr) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA820667 = {S27521, 125, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; hr-hr) AppleWebKit/530.1+ (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA822158 = {S27522, 126, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; ja-jp) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.12*/
const Device UA824301 = {S27523, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; th-th) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA846050 = {S27524, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; es-es) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA856991 = {S27525, 124, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ar) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA867462 = {S27526, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA880976 = {S27527, 124, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_5; fi-fi) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.12*/
const Device UA1840692 = {S27528, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; tr-tr) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA1841198 = {S27529, 123, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/525.19 (KHTML, like Gecko) Version/3.2.1 Safari/525/19*/
const Device UA1850576 = {S27530, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nl-nl) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA1856811 = {S27531, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; el-gr) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA2125730 = {S27532, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; af-za) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA2126736 = {S27533, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-ca) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA2128215 = {S27534, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; fr-fr) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA2134937 = {S27535, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; pt-pt) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA2137904 = {S27536, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; it-it) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA2275063 = {S27537, 126, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; it-it) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.12*/
const Device UA2282123 = {S27538, 127, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; de-de) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA2284593 = {S27539, 124, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA2720845 = {S27540, 104, { &P17595,&P17295,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.0.6) Gecko/X Version/3.2.1 Safari/525.27.1*/
const Device UA205656 = {S27541, 102, { &P15364,&P18110,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30*/
const Device UA218555 = {S27542, 104, { &P15364,&P18110,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.124 Safari/534.30*/
const Device UA249559 = {S27543, 120, { &P15364,&P18110,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.51 (KHTML, like Gecko; Google Web Preview) Chrome/12.0.742 Safari/534.51*/
const Device UA560071 = {S27544, 111, { &P15364,&P18110,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.124 Safari/534.30*/
const Device UA623812 = {S27545, 124, { &P15364,&P18110,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 ArchLinux (X11; U; Linux x86_64; en-US) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30*/
const Device UA1886748 = {S27546, 103, { &P15364,&P18110,&P17251,&P18092 }};	/*Useragent - 'Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.91 Safari/534.30'*/
const Device UA2674467 = {S27547, 109, { &P15364,&P18110,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/12.0.1410.63 Safari/537.31*/
const Device UA2674616 = {S27548, 106, { &P15364,&P18110,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/12.0.1410.63 Safari/537.31*/
const Device UA2674691 = {S27549, 106, { &P15364,&P18110,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/12.0.1410.63 Safari/537.31*/
const Device UA2675210 = {S27550, 109, { &P15364,&P18110,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/12.0.1410.63 Safari/537.31*/
const Device UA2675714 = {S27551, 106, { &P15364,&P18110,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/12.0.1410.63 Safari/537.31*/
const Device UA2714601 = {S27552, 109, { &P15364,&P18110,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/12.0.1410.63 Safari/537.31*/
const Device UA2751946 = {S27553, 106, { &P15364,&P18110,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/12.0.1410.63 Safari/537.31*/
const Device UA2983761 = {S27554, 174, { &P15364,&P18110,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30 AppEngine-Google; (+http://code.google.com/appengine; appid: s~croossa)*/
const Device UA2567832 = {S27555, 151, { &P18317,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ASUS Transformer Pad TF300T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2567842 = {S27556, 80, { &P18317,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; ASUS Transformer Pad TF300T Build/JOP40D)*/
const Device UA2714685 = {S27557, 81, { &P18317,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; ASUS Transformer Pad TF300TG Build/JOP40D)*/
const Device UA2717085 = {S27558, 152, { &P18317,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ASUS Transformer Pad TF300TG Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2738350 = {S27559, 152, { &P18317,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; ASUS Transformer Pad TF300TL Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2781268 = {S27560, 139, { &P18317,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; ja-jp; ASUS Pad TF300T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2850491 = {S27561, 146, { &P18317,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Transformer Pad TF300T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2856790 = {S27562, 145, { &P18317,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Transformer Pad TF300T Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2989171 = {S27563, 158, { &P18317,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ASUS Transformer Pad TF300T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049619 = {S27564, 81, { &P18317,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; ASUS Transformer Pad TF300TL Build/JOP40D)*/
const Device UA3115659 = {S27565, 119, { &P17595,&P17296,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3128891 = {S27566, 119, { &P17595,&P17296,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3129477 = {S27567, 129, { &P17595,&P17296,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36,gzip(gfe)*/
const Device UA3129478 = {S27568, 119, { &P17595,&P17296,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3129597 = {S27569, 119, { &P17595,&P17296,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3130082 = {S27570, 119, { &P17595,&P17296,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3130469 = {S27571, 119, { &P17595,&P17296,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3130656 = {S27572, 119, { &P17595,&P17296,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3130857 = {S27573, 119, { &P17595,&P17296,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA3130859 = {S27574, 119, { &P17595,&P17296,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA2740348 = {S27575, 130, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.85 Safari/537.36*/
const Device UA2741596 = {S27576, 137, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2741696 = {S27577, 137, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2741701 = {S27578, 137, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2741726 = {S27579, 145, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2741733 = {S27580, 137, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2752948 = {S27581, 138, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I317M Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2752945 = {S27582, 138, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753445 = {S27583, 135, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SC-02E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753849 = {S27584, 138, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753873 = {S27585, 138, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754244 = {S27586, 145, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I317 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754335 = {S27587, 137, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SPH-L900 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754575 = {S27588, 137, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754720 = {S27589, 138, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754936 = {S27590, 137, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754951 = {S27591, 138, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755085 = {S27592, 138, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755059 = {S27593, 138, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755338 = {S27594, 137, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755353 = {S27595, 137, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755551 = {S27596, 138, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2756008 = {S27597, 137, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773713 = {S27598, 130, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2773843 = {S27599, 137, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-R950 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2800513 = {S27600, 138, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889V Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869309 = {S27601, 130, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2877176 = {S27602, 137, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-N719 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877301 = {S27603, 138, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SHV-E250S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879320 = {S27604, 130, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2892619 = {S27605, 137, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-N719 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893499 = {S27606, 130, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2937747 = {S27607, 138, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SHV-E250K Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2957347 = {S27608, 137, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; GT-N7100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3073982 = {S27609, 134, { &P18442,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; N7100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA911573 = {S27610, 156, { &P17595,&P17297,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11,gzip(gfe) (via translate.google.com)*/
const Device UA912386 = {S27611, 119, { &P17595,&P17297,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA912402 = {S27612, 119, { &P17595,&P17297,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA912412 = {S27613, 119, { &P17595,&P17297,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA912423 = {S27614, 119, { &P17595,&P17297,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA912510 = {S27615, 119, { &P17595,&P17297,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA923999 = {S27616, 119, { &P17595,&P17297,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA1987545 = {S27617, 158, { &P17595,&P17297,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2010801 = {S27618, 114, { &P17595,&P17297,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 Macintosh Intel Mac OS X 10_7_4 AppleWebKit/536.11 KHTML, like Gecko Chrome/20.0.1132.57 Safari/536.11*/
const Device UA2597963 = {S27619, 116, { &P17595,&P17297,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1070) AppleWebKit/536.11 (KHTML like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA368220 = {S27620, 144, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Sensation Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA441155 = {S27621, 156, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC/Sensation/3.32.161.11 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA441272 = {S27622, 156, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC/Sensation/3.32.162.11 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA456151 = {S27623, 144, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Sensation Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA609382 = {S27624, 156, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC/Sensation/3.32.166.11 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA609644 = {S27625, 156, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC/Sensation/3.32.163.12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616639 = {S27626, 154, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_Sensation-orange-LS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA685169 = {S27627, 156, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; HTC/Sensation/3.32.165.11 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA697417 = {S27628, 66, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Sensation Build/IML74K)*/
const Device UA715055 = {S27629, 140, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Sensation Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA729562 = {S27630, 140, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Sensation Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA742544 = {S27631, 140, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; HTC-Z710a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA767686 = {S27632, 155, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC/Sensation/1.45.161.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA864970 = {S27633, 140, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Sensation Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA892926 = {S27634, 150, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC_Sensation_Z710a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA895691 = {S27635, 155, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC/Sensation/1.50.161.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA897773 = {S27636, 150, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC Sensation Z710a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA909163 = {S27637, 144, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_Sensation Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA909562 = {S27638, 153, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC_Z710a V3.32.841.15 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1941599 = {S27639, 144, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-gb; HTC Sensation Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1959577 = {S27640, 152, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC_Z710a V1.45.841.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1967891 = {S27641, 140, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Sensation Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1970585 = {S27642, 156, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; HTC/Sensation/3.32.187.11 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1986460 = {S27643, 156, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-pt; HTC/Sensation/3.32.168.11 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1987432 = {S27644, 144, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Sensation Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1994101 = {S27645, 72, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Sensation Z710a Build/IML74K)*/
const Device UA2025011 = {S27646, 155, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC/Sensation/1.45.161.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025859 = {S27647, 155, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Sensation Build/HyperNonSense 2.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029400 = {S27648, 62, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Sensation Build/IMM76I)*/
const Device UA2031531 = {S27649, 62, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC-Z710a Build/IML74K)*/
const Device UA2032246 = {S27650, 149, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation Z710 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032498 = {S27651, 155, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC/Sensation/1.45.162.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033671 = {S27652, 66, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Sensation Build/IMM76L)*/
const Device UA2034509 = {S27653, 155, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC/Sensation/1.50.162.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2323110 = {S27654, 155, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC/Sensation/1.45.163.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2359364 = {S27655, 156, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC/Sensation/3.32.161.52 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2362689 = {S27656, 156, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC/Sensation/3.32.162.52 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2369537 = {S27657, 156, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC/Sensation/3.32.166.52 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2404537 = {S27658, 156, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; HTC/Sensation/3.32.165.52 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2405730 = {S27659, 156, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC/Sensation/3.32.163.53 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2445472 = {S27660, 141, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation Z710a Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2578219 = {S27661, 156, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-pt; HTC/Sensation/3.32.168.52 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659402 = {S27662, 156, { &P15438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; HTC/Sensation/3.32.187.52 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2946083 = {S27663, 125, { &P15364,&P7133,&P22763,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1*/
const Device UA2947935 = {S27664, 118, { &P15364,&P7133,&P22763,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1*/
const Device UA3015160 = {S27665, 135, { &P15364,&P7133,&P22763,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1,gzip(gfe)*/
const Device UA3015891 = {S27666, 128, { &P15364,&P7133,&P22763,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1,gzip(gfe)*/
const Device UA2737801 = {S27667, 147, { &P12968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyLT26i-o Build/6.2.B.0.197) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2774320 = {S27668, 63, { &P12968,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LT26i Build/6.2.B.0.200)*/
const Device UA2774322 = {S27669, 145, { &P12968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyLT26i Build/6.2.B.0.200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2783705 = {S27670, 141, { &P12968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LT26i Build/6.2.B.0.200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2784809 = {S27671, 147, { &P12968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyLT26i-o Build/6.2.B.0.200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2869865 = {S27672, 145, { &P12968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT26i Build/6.2.B.0.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2877251 = {S27673, 141, { &P12968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT26i Build/6.2.B.0.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956796 = {S27674, 141, { &P12968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LT26i Build/6.2.B.0.197) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2959640 = {S27675, 65, { &P12968,&P18111,&P18112,&P18092 }};	/*Useragent - SonyLT26i Build/6.2.B.0.200 stagefright/1.2 (Linux;Android 4.1.2)*/
const Device UA3000021 = {S27676, 146, { &P12968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-be; SonyLT26iv Build/6.2.B.0.200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3061767 = {S27677, 144, { &P12968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyLT26i Build/6.2.B.1.96) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3064440 = {S27678, 140, { &P12968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT26i Build/6.2.B.1.96) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3107187 = {S27679, 156, { &P12968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT26i Build/Trinity Vengeance RaptorXL) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA190416 = {S27680, 118, { &P17595,&P17296,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24*/
const Device UA195664 = {S27681, 118, { &P17595,&P17296,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA195668 = {S27682, 118, { &P17595,&P17296,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA195672 = {S27683, 118, { &P17595,&P17296,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA309393 = {S27684, 117, { &P17595,&P17296,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.698.0 Safari/534.24*/
const Device UA315805 = {S27685, 118, { &P17595,&P17296,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA445813 = {S27686, 127, { &P17595,&P17296,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/534.18 (KHTML, like Gecko) Chrome/11.0.660.0 Safari/534.18*/
const Device UA449216 = {S27687, 118, { &P17595,&P17296,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA632920 = {S27688, 118, { &P17595,&P17296,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA638423 = {S27689, 127, { &P17595,&P17296,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.17 (KHTML, like Gecko) Chrome/11.0.655.0 Safari/534.17*/
const Device UA873899 = {S27690, 118, { &P17595,&P17296,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24*/
const Device UA1863832 = {S27691, 127, { &P17595,&P17296,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/534.20 (KHTML, like Gecko) Chrome/11.0.672.2 Safari/534.20*/
const Device UA2258700 = {S27692, 127, { &P17595,&P17296,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/534.20 (KHTML, like Gecko) Chrome/11.0.672.2 Safari/534.20*/
const Device UA2293941 = {S27693, 127, { &P17595,&P17296,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/534.18 (KHTML, like Gecko) Chrome/11.0.660.0 Safari/534.18*/
const Device UA2393245 = {S27694, 118, { &P17595,&P17296,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/534.24 (KHTML	 like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA2674621 = {S27695, 129, { &P17595,&P17296,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/11.0.648.205 Safari/534.16*/
const Device UA2674677 = {S27696, 129, { &P17595,&P17296,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/11.0.648.205 Safari/534.16*/
const Device UA2714757 = {S27697, 129, { &P17595,&P17296,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/11.0.648.205 Safari/534.16*/
const Device UA2043996 = {S27698, 141, { &P18564,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One X+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2043999 = {S27699, 63, { &P18564,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC One X+ Build/JRO03C)*/
const Device UA2257673 = {S27700, 143, { &P18564,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_OneXplus Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2362378 = {S27701, 149, { &P18564,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PM63100/1.15.502.9 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2363034 = {S27702, 155, { &P18564,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_OneXplus/1.14.161.13 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2368090 = {S27703, 155, { &P18564,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_OneXplus/1.14.161.18 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2401470 = {S27704, 136, { &P18564,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; X525a Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2482020 = {S27705, 143, { &P18564,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; HTC EVARE_UL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2488651 = {S27706, 154, { &P18564,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_OneXplus/1.17.161.7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2594401 = {S27707, 149, { &P18564,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PM63100/1.08.502.1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2596344 = {S27708, 141, { &P18564,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTCOneXplus Build/JRO03C) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2671198 = {S27709, 149, { &P18564,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PM63100/1.19.502.1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2691109 = {S27710, 65, { &P18564,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC EVARE_UL Build/JRO03C)*/
const Device UA2713473 = {S27711, 154, { &P18564,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_OneXplus/1.17.161.8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2763608 = {S27712, 139, { &P18564,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; HTC One X+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2933669 = {S27713, 143, { &P18564,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HTC_EVARE_UL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985035 = {S27714, 152, { &P18564,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One X+ Build/QX Senseless v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044355 = {S27715, 160, { &P18564,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PM63100/1.15.502.9 Build/QX Senseless v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA11209 = {S27716, 138, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Desire Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA23750 = {S27717, 140, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Desire_A8181 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24566 = {S27718, 144, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_Desire_A8181 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA45927 = {S27719, 148, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; HTC_Desire-orange-LS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46381 = {S27720, 149, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; HTC Desire 2.33.161.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51020 = {S27721, 156, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC_Desire_A8183 V2.26.841.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA53413 = {S27722, 134, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; Desire Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA69895 = {S27723, 138, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72588 = {S27724, 144, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_Desire_A8181 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA77631 = {S27725, 142, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; iw-; HTC_Desire_A8181 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA180013 = {S27726, 138, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA188073 = {S27727, 143, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Desire CDMA Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA189262 = {S27728, 62, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Desire Build/GRI40)*/
const Device UA194231 = {S27729, 138, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Desire Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA194357 = {S27730, 138, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA211461 = {S27731, 140, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ERIS-ADR6200 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA211518 = {S27732, 138, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA219506 = {S27733, 150, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire Build/GINGERVillain-2.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA248856 = {S27734, 143, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Desire CDMA Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA249838 = {S27735, 139, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250025 = {S27736, 138, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA252073 = {S27737, 140, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Desire_A8181 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA255813 = {S27738, 138, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA312096 = {S27739, 140, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Eris ADR6200 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA315276 = {S27740, 143, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Desire CDMA Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA321858 = {S27741, 143, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Desire CDMA Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA327077 = {S27742, 135, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6200 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA335101 = {S27743, 143, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Desire CDMA Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA344515 = {S27744, 150, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire Build/GINGERVillain-3.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA362718 = {S27745, 67, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Desire CDMA Build/GRI40)*/
const Device UA368837 = {S27746, 139, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; HTC Desire Build/MvMiui) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370757 = {S27747, 151, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; HTC Desire Build/Villain_AuDT 3.2U1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370883 = {S27748, 151, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; HTC Desire Build/Villain_AuPC 3.2U1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA371275 = {S27749, 126, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Desire Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA392877 = {S27750, 138, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Desire Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA397201 = {S27751, 62, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Desire Build/GRI40)*/
const Device UA417097 = {S27752, 138, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire Build/GRI54) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA419958 = {S27753, 150, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Desire Build/GINGERVillain-2.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA550824 = {S27754, 150, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Desire Build/GINGERVillain-2.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA557977 = {S27755, 149, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; HTC Desire Build/Villain_AuPC 2.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622471 = {S27756, 149, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Desire CDMA Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30*/
const Device UA637851 = {S27757, 62, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Desire Build/GRJ22)*/
const Device UA655679 = {S27758, 143, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Desire CDMA Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA680280 = {S27759, 126, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Desire Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA682605 = {S27760, 147, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; HTC Desire Build/Villain_Au 3.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA686408 = {S27761, 62, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Desire Build/GRJ90)*/
const Device UA690655 = {S27762, 138, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA715971 = {S27763, 144, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA725303 = {S27764, 143, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Desire CDMA Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA738201 = {S27765, 150, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire Build/GINGERVillain-3.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA750944 = {S27766, 144, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Desire CDMA Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA764063 = {S27767, 149, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; HTC Desire Build/Villain_AuPC 3.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA789634 = {S27768, 127, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;u;Android 2.3.7;zh-cn;HTC Desire Build) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA805792 = {S27769, 138, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA915540 = {S27770, 140, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6200 ERIS Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018211 = {S27771, 150, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; HTC Desire Build/GINGERVillain-0.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018284 = {S27772, 143, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC Desire CDMA Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018491 = {S27773, 150, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire Build/GINGERVillain-1.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018570 = {S27774, 150, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire Build/GINGERVillain-1.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018693 = {S27775, 150, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Desire Build/GINGERVillain-2.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018949 = {S27776, 150, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; HTC Desire Build/GINGERVillain-2.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019260 = {S27777, 149, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Desire CDMA Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019531 = {S27778, 150, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC Desire Build/Villain_AuDT 2.8b) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019640 = {S27779, 150, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC Desire Build/Villain_AuPC 2.8b) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019714 = {S27780, 126, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Desire Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2020183 = {S27781, 138, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fa-ir; HTC Desire Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020832 = {S27782, 149, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; HTC Desire Build/Villain_AuDT 3.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022136 = {S27783, 126, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Desire Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2025412 = {S27784, 134, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; Desire Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027036 = {S27785, 158, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; HTC_Desire_A8181 Build/Mod Lonas Sense 0.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027425 = {S27786, 183, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Gingerbread 2.3.7; en-gb; HTC Desire Build/Port and Mod by RMG2; MokeeOS v0.5 RC1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030443 = {S27787, 171, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire Build/Port and Mod by RMG2; MokeeOS v0.6 RC2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031196 = {S27788, 131, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Desire Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2032909 = {S27789, 141, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; HTC Desire Build/SoulMIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2336154 = {S27790, 138, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; HTC Desire Build/xMIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2394120 = {S27791, 131, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Desire Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2505734 = {S27792, 138, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2519409 = {S27793, 133, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; X06HT Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2683026 = {S27794, 141, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ru-ru; HTC Desire Build/FCR MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2775249 = {S27795, 140, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; HTC Desire Build/Siberia) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2778121 = {S27796, 130, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Android-Desire/1.0 Linux/2.6.35.7 Android/2.3.6 Release/10.29.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2810303 = {S27797, 158, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; HTC_Desire_A8181 Build/Mod Lonas Sense 0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2846207 = {S27798, 130, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Android-Desire/1.0 Linux/2.6.35.7 Android/2.3.6 Release/10.30.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3041737 = {S27799, 146, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ru-ru; HTC Desire Build/NMR 2.2.30RC4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3087381 = {S27800, 145, { &P11747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; HTC Desire Build/Miui_Pecha2k) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA68406 = {S27801, 73, { &P14888,&P5156,&P3519,&P18092 }};	/*Useragent - BlackBerry9650/6.0.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/105*/
const Device UA380967 = {S27802, 129, { &P14888,&P5156,&P3519,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9650; en-US) AppleWebKit/534.8+ (KHTML, like Gecko) Version/6.0.0.719 Mobile Safari/534.8+*/
const Device UA2540494 = {S27803, 138, { &P21261,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9500 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2669054 = {S27804, 164, { &P21261,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9500 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2766307 = {S27805, 137, { &P21261,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA924839 = {S27806, 73, { &P15364,&P17285,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:17.0) Gecko/17.0 Firefox/17.0*/
const Device UA924840 = {S27807, 61, { &P15364,&P17285,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:17.0) Gecko/17.0 Firefox/17.0*/
const Device UA2390798 = {S27808, 119, { &P15364,&P17285,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.3) Gecko/X Firefox/17.0 GTB7.0 ( .NET CLR 3.5.30729; .NET4.0E)*/
const Device UA2472026 = {S27809, 57, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.13*/
const Device UA2488290 = {S27810, 75, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.13*/
const Device UA2490178 = {S27811, 64, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.13*/
const Device UA2491896 = {S27812, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.13*/
const Device UA2491899 = {S27813, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.13*/
const Device UA2491904 = {S27814, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.13*/
const Device UA2491908 = {S27815, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.13*/
const Device UA2491897 = {S27816, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.13*/
const Device UA2492123 = {S27817, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.13*/
const Device UA2492129 = {S27818, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.13*/
const Device UA2492131 = {S27819, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.13*/
const Device UA2492132 = {S27820, 74, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.13,gzip(gfe)*/
const Device UA2492133 = {S27821, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.13*/
const Device UA2492139 = {S27822, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.13*/
const Device UA2492178 = {S27823, 67, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.13,gzip(gfe)*/
const Device UA2492196 = {S27824, 80, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.13*/
const Device UA2492197 = {S27825, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.13*/
const Device UA2492360 = {S27826, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.13*/
const Device UA2492445 = {S27827, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.13*/
const Device UA2508043 = {S27828, 80, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.13*/
const Device UA2508097 = {S27829, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.13*/
const Device UA2508202 = {S27830, 62, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1) Opera 12.13*/
const Device UA2508250 = {S27831, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.13*/
const Device UA2508769 = {S27832, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.13*/
const Device UA2508853 = {S27833, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6035)) Presto/2.12.388 Version/12.13*/
const Device UA2508873 = {S27834, 87, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.13*/
const Device UA2508919 = {S27835, 80, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.13*/
const Device UA2509806 = {S27836, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.13*/
const Device UA2510449 = {S27837, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.13*/
const Device UA2510518 = {S27838, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.13*/
const Device UA2510934 = {S27839, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.13*/
const Device UA2511029 = {S27840, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.13*/
const Device UA2511408 = {S27841, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.13*/
const Device UA2511414 = {S27842, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.13*/
const Device UA2511996 = {S27843, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.13*/
const Device UA2512012 = {S27844, 80, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.13*/
const Device UA2512839 = {S27845, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.13*/
const Device UA2512982 = {S27846, 87, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.13*/
const Device UA2515320 = {S27847, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.13*/
const Device UA2530451 = {S27848, 87, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.13*/
const Device UA2531908 = {S27849, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.13*/
const Device UA2536574 = {S27850, 87, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.13*/
const Device UA2537582 = {S27851, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.13*/
const Device UA2537729 = {S27852, 80, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.13*/
const Device UA2538107 = {S27853, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.13*/
const Device UA2559261 = {S27854, 80, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.13*/
const Device UA2563795 = {S27855, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.13*/
const Device UA2564606 = {S27856, 87, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.13*/
const Device UA2566035 = {S27857, 80, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.13*/
const Device UA2569760 = {S27858, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.13*/
const Device UA2580527 = {S27859, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.13*/
const Device UA2581942 = {S27860, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.13*/
const Device UA2599752 = {S27861, 87, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.13*/
const Device UA2660147 = {S27862, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.13*/
const Device UA2662037 = {S27863, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.13*/
const Device UA2665805 = {S27864, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.13*/
const Device UA2667436 = {S27865, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.13*/
const Device UA2699105 = {S27866, 80, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.13*/
const Device UA2699840 = {S27867, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.13*/
const Device UA2701799 = {S27868, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.13*/
const Device UA2702192 = {S27869, 87, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.13*/
const Device UA2702671 = {S27870, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6032)) Presto/2.12.388 Version/12.13*/
const Device UA2702729 = {S27871, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.13*/
const Device UA2703149 = {S27872, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.13*/
const Device UA2703473 = {S27873, 87, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.13*/
const Device UA2705535 = {S27874, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.13*/
const Device UA2730548 = {S27875, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6046)) Presto/2.12.388 Version/12.13*/
const Device UA2767553 = {S27876, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.13*/
const Device UA2780785 = {S27877, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.13*/
const Device UA2783525 = {S27878, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.13*/
const Device UA2849479 = {S27879, 80, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.13*/
const Device UA2850502 = {S27880, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.13*/
const Device UA2851108 = {S27881, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4947)) Presto/2.12.388 Version/12.13*/
const Device UA2851550 = {S27882, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 5.9 (build 4871)) Presto/2.12.388 Version/12.13*/
const Device UA2861786 = {S27883, 86, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.13*/
const Device UA2952304 = {S27884, 79, { &P15364,&P7133,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; MRA 6.0 (build 5676)) Presto/2.12.388 Version/12.13*/
const Device UA205669 = {S27885, 115, { &P17595,&P17296,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.813.0 Safari/535.1*/
const Device UA249210 = {S27886, 117, { &P17595,&P17296,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA249765 = {S27887, 117, { &P17595,&P17296,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA250085 = {S27888, 117, { &P17595,&P17296,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA250096 = {S27889, 117, { &P17595,&P17296,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA250126 = {S27890, 117, { &P17595,&P17296,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA250166 = {S27891, 117, { &P17595,&P17296,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA250161 = {S27892, 117, { &P17595,&P17296,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA251311 = {S27893, 113, { &P17595,&P17296,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10_6_7) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.790.0 Safari/535.1*/
const Device UA341243 = {S27894, 117, { &P17595,&P17296,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1*/
const Device UA398321 = {S27895, 123, { &P17595,&P17296,&P17253,&P18092 }};	/*Useragent - Ruby, Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA2258016 = {S27896, 117, { &P17595,&P17296,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/535.1 (KHTML	 like Gecko) Chrome/14.0.835.186 Safari/535.1*/
const Device UA2337415 = {S27897, 117, { &P17595,&P17296,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/535.1 (KHTML	 like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA2374270 = {S27898, 117, { &P17595,&P17296,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.1 (KHTML	 like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA2674708 = {S27899, 129, { &P17595,&P17296,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/14.0.648.205 Safari/534.16*/
const Device UA2674858 = {S27900, 129, { &P17595,&P17296,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/14.0.648.205 Safari/534.16*/
const Device UA2715169 = {S27901, 129, { &P17595,&P17296,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/14.0.648.205 Safari/534.16*/
const Device UA222587 = {S27902, 144, { &P15364,&P18110,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Jolicloud Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Joli OS/1.2 Chromium/13.0.782.107 Chrome/13.0.782.107 Safari/535.1*/
const Device UA237619 = {S27903, 102, { &P15364,&P18110,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1*/
const Device UA246367 = {S27904, 100, { &P15364,&P18110,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1*/
const Device UA379362 = {S27905, 114, { &P15364,&P18110,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 Slackware/13.37 (X11; U; Linux x86_64; en-US) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41*/
const Device UA421570 = {S27906, 101, { &P15364,&P18110,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux amd64) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/13.0.766.0 Safari/534.36*/
const Device UA539327 = {S27907, 109, { &P15364,&P18110,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/13.0.765.0 Safari/534.36*/
const Device UA2674462 = {S27908, 109, { &P15364,&P18110,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/13.0.1410.63 Safari/537.31*/
const Device UA2674683 = {S27909, 109, { &P15364,&P18110,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/13.0.1410.63 Safari/537.31*/
const Device UA2674877 = {S27910, 106, { &P15364,&P18110,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/13.0.1410.63 Safari/537.31*/
const Device UA2675187 = {S27911, 106, { &P15364,&P18110,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/13.0.1410.63 Safari/537.31*/
const Device UA2675194 = {S27912, 106, { &P15364,&P18110,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/13.0.1410.63 Safari/537.31*/
const Device UA2714932 = {S27913, 109, { &P15364,&P18110,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/13.0.1410.63 Safari/537.31*/
const Device UA2735611 = {S27914, 106, { &P15364,&P18110,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/13.0.1410.63 Safari/537.31*/
const Device UA22231 = {S27915, 146, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonX10i Build/private) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA45508 = {S27916, 151, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonX10i Build/3.0.1.G.0.78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA45513 = {S27917, 158, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonX10i Build/3.0.1.G.0.75 Ultima) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA45587 = {S27918, 152, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonX10iv Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA45662 = {S27919, 151, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonX10i Build/3.0.1.G.0.76) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA45677 = {S27920, 153, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonX10i-o Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA45730 = {S27921, 150, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; vi-vn; SonyEricssonX10i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49332 = {S27922, 139, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA50027 = {S27923, 151, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonX10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59592 = {S27924, 151, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonX10i Build/3.0.1.G.0.68) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59607 = {S27925, 151, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonX10i Build/3.0.1.G.0.35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59595 = {S27926, 151, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonX10i Build/3.0.1.G.0.70) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73704 = {S27927, 87, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Android (Linux; U; Android 2.3.3; fa-ir; X10i) AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA74381 = {S27928, 93, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.3.3;Zh_cn;X10i;480*854;)AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA77922 = {S27929, 165, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5 RRQ ilove BMG; es-es; SonyEricssonX10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA208934 = {S27930, 63, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; X10i Build/3.0.1.G.0.75)*/
const Device UA623254 = {S27931, 127, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; X10i Build/3.0.1.G.0.75) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA709956 = {S27932, 141, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7 pRiMe X; en-gb; X10i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA873074 = {S27933, 151, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonX10i Build/6.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1905408 = {S27934, 139, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1965406 = {S27935, 139, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; X10i Build/6.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1977494 = {S27936, 132, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; X10i Build/3.0.1.G.0.75) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2021380 = {S27937, 151, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonX10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021597 = {S27938, 63, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; X10i Build/3.0.1.G.0.75)*/
const Device UA2022607 = {S27939, 149, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ar-; SonyEricssonX10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022817 = {S27940, 141, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7 pRiMe 7; en-gb; X10i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026451 = {S27941, 143, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.9; en-gb; Xperia X10 Build/6.0.A.3.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028087 = {S27942, 146, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7 V6.0 MM TEAM; en-us; X10i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029985 = {S27943, 151, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonX10S Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034329 = {S27944, 144, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7 V7 MM TEAM; en-us; X10i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2145076 = {S27945, 139, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2474882 = {S27946, 143, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.9; en-us; Xperia X10 Build/6.0.A.3.73) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2642764 = {S27947, 150, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; SonyEricssonX10 Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2828193 = {S27948, 144, { &P11950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; Xperia X10 Build/6.0.B.0.743) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2015616 = {S27949, 69, { &P15364,&P18118,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:16.0) Gecko/X Firefox/16.0*/
const Device UA2601354 = {S27950, 76, { &P15364,&P18118,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (X11: Ubuntu; Linux x86_64; rv:16.0; Rover) Gecko/X Firefox/16.0*/
const Device UA43175 = {S27951, 147, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID X2 Build/4.5.1A-DTN-117-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA43180 = {S27952, 147, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; DROID X2 Build/4.5.1A-DTN-117-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43188 = {S27953, 148, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID X2 Build/4.5.1A-DTN-150-26) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43189 = {S27954, 147, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;DROID X2 Build/4.5.1A-DTN-117-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43195 = {S27955, 147, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDX2 Build/4.5.1A-DTN-117-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43200 = {S27956, 148, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID X2 Build/4.5.1A-DTN-150-30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43201 = {S27957, 148, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID X2 Build/4.5.1A-DTN-150-14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59116 = {S27958, 148, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID X2 Build/4.5.1A-DTN-117-14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59126 = {S27959, 139, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID X2 Build/4.5.1_93) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59128 = {S27960, 148, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID X2 Build/4.5.1A-DTN-117-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59129 = {S27961, 148, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID X2 Build/4.5.1A-DTN-150-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59136 = {S27962, 147, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID X2 Build/4.5.1A-DTN-117-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59131 = {S27963, 148, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID X2 Build/4.5.1A-DTN-150-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59140 = {S27964, 147, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID X2 Build/4.5.1A-DTN-117-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59142 = {S27965, 148, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID X2 Build/4.5.1A-DTN-117-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70848 = {S27966, 137, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID X2 Build/4.5.91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72122 = {S27967, 148, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID X2 Build/4.5.1A-DTN-150-27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76662 = {S27968, 147, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; DROID X2 Build/4.5.1A-DTN-150-30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA77499 = {S27969, 147, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID X2 Build/4.5.1A-DTN-117-8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA77847 = {S27970, 147, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;DROID X2 Build/4.5.1A-DTN-150-30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78219 = {S27971, 147, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX2 Build/4.5.1A-DTN-150-30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78269 = {S27972, 147, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; DROID X2 Build/4.5.1A-DTN-150-30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78435 = {S27973, 147, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID X2 Build/4.5.1A-DTN-150-30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA78436 = {S27974, 147, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID X2Build/4.5.1A-DTN-150-30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA212818 = {S27975, 72, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; DROID X2 Build/4.5.1A-DTN-117-15)*/
const Device UA250527 = {S27976, 72, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; DROID X2 Build/4.5.1A-DTN-150-30)*/
const Device UA580642 = {S27977, 66, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; Milestone X2 Build/45.0.43)*/
const Device UA622924 = {S27978, 142, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Milestone X2 Build/45.0.43) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623267 = {S27979, 148, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA641087 = {S27980, 142, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Milestone X2 Build/45.0.25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA646543 = {S27981, 148, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA655302 = {S27982, 72, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; DROID X2 Build/4.5.1A-DTN-200-18)*/
const Device UA678974 = {S27983, 147, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA682258 = {S27984, 148, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA750931 = {S27985, 147, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROIDX2 Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA799906 = {S27986, 147, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA863645 = {S27987, 147, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID X2Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA869934 = {S27988, 147, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA876728 = {S27989, 147, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us;DROID X2 Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019422 = {S27990, 142, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Milestone X2 Build/45.0.18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021237 = {S27991, 66, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Milestone X2 Build/45.0.25)*/
const Device UA2024433 = {S27992, 69, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; MB870 Build/4.5.1A-DTN-150-30)*/
const Device UA2025874 = {S27993, 145, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB870 Build/4.5.1A-DTN-150-30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2684427 = {S27994, 146, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; DROID X2 Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2813924 = {S27995, 141, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; DROID X2 Build/4.5.1A-DTN-150-30) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2832519 = {S27996, 156, { &P16092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-18) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA174898 = {S27997, 105, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; ) AppleWebKit/533.9 (KHTML, like Gecko) Maxthon/3.0 Safari/533.9*/
const Device UA192678 = {S27998, 107, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; ) AppleWebKit/534.12 (KHTML, like Gecko) Maxthon/3.0 Safari/534.12*/
const Device UA285232 = {S27999, 110, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/530.6 (KHTML, like Gecko) Maxthon/3.0 Safari/530.6*/
const Device UA623678 = {S28000, 110, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/535.3 (KHTML, like Gecko) Maxthon/3.0 Safari/535.3*/
const Device UA640487 = {S28001, 130, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12*/
const Device UA671668 = {S28002, 105, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; ) AppleWebKit/533.9 (KHTML, like Gecko) Maxthon/3.0 Safari/533.9*/
const Device UA672904 = {S28003, 130, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12*/
const Device UA719639 = {S28004, 115, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.1 (KHTML, like Gecko) Maxthon/3.0.9.10 Safari/533.1*/
const Device UA741835 = {S28005, 106, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; ) AppleWebKit/534.12 (KHTML,like Gecko) Maxthon/3.0 Safari/534.12*/
const Device UA816636 = {S28006, 111, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12*/
const Device UA2009042 = {S28007, 132, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/19.0.1084.52 Safari/535.12*/
const Device UA2168277 = {S28008, 118, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12*/
const Device UA2193489 = {S28009, 118, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12*/
const Device UA2200611 = {S28010, 111, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA2205234 = {S28011, 111, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12*/
const Device UA2206004 = {S28012, 148, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12,gzip(gfe) (via translate.google.com)*/
const Device UA2234011 = {S28013, 121, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12,gzip(gfe)*/
const Device UA2402836 = {S28014, 121, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1,gzip(gfe)*/
const Device UA2424317 = {S28015, 118, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA2424827 = {S28016, 111, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA2426421 = {S28017, 118, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA2675171 = {S28018, 113, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/26.0.1410.43 Safari/535.12*/
const Device UA2994686 = {S28019, 123, { &P15364,&P5690,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/26.0.1410.43 Safari/535.12,gzip(gfe)*/
const Device UA3035165 = {S28020, 91, { &P18088,&P22710,&P17369,&P18092 }};	/*Useragent - Opera/9.80 (MAUI Runtime; Opera Mini/4.4.31762/30.3697; U; en) Presto/2.8.119 Version/11.10*/
const Device UA18738 = {S28021, 115, { &P11854,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE71-2;Mozilla/5.0(symbianos/9.2;u;nokiae71-2/100.07.76 profile/midp-2.0 Configuration/cldc-1.1)applewebkit/413*/
const Device UA30369 = {S28022, 156, { &P11854,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE71x/ATT.03.28 Mozilla/5.0 SymbianOS/9.3; U; [en]; Series60/3.2; Profile/MIDP-2.1 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA33036 = {S28023, 141, { &P11854,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3 U; Series60/3.2 NokiaE71x Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA33369 = {S28024, 155, { &P11854,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE71-1/510.21.009; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA34523 = {S28025, 92, { &P11854,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE71/1.0 (510.21.009) SymbianOS/9.2 Series60/3.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA35377 = {S28026, 185, { &P11854,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE71-1/200.21.118; 60/3.1 NokiaE71-1/200.21.118; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA62692 = {S28027, 155, { &P11854,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE71-2/500.21.009; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA64092 = {S28028, 155, { &P11854,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE71-3/300.21.012; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA67029 = {S28029, 8, { &P11854,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE71*/
const Device UA68963 = {S28030, 77, { &P11854,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Nokia Podcasting; SymbianOS; NokiaE71-1/500.21.009;)*/
const Device UA68991 = {S28031, 10, { &P11854,&P5890,&P3439,&P18092 }};	/*Useragent - nokiae71-1*/
const Device UA500801 = {S28032, 145, { &P11854,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NOKIAE71/1.0; Profile/MIDP-2.0 Configuration/CLDC-1.1) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA719967 = {S28033, 291, { &P11854,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NOKIAE71/1.0; Profile/MIDP-2.0 Configuration/CLDC-1.1) AppleWebKit/413 (KHTML, like Gecko) Safari/413 Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NOKIAE71/1.0; Profile/MIDP-2.0 Configuration/CLDC-1.1) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA910376 = {S28034, 102, { &P15364,&P18110,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11*/
const Device UA912442 = {S28035, 104, { &P15364,&P18110,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA924096 = {S28036, 111, { &P15364,&P18110,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA2163689 = {S28037, 139, { &P15364,&P18110,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11,gzip(gfe) (via translate.google.com)*/
const Device UA2260004 = {S28038, 104, { &P15364,&P18110,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux armv6l) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA2674502 = {S28039, 109, { &P15364,&P18110,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/20.0.1410.63 Safari/537.31*/
const Device UA2674527 = {S28040, 106, { &P15364,&P18110,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/20.0.1410.63 Safari/537.31*/
const Device UA2674870 = {S28041, 109, { &P15364,&P18110,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/20.0.1410.63 Safari/537.31*/
const Device UA2674884 = {S28042, 106, { &P15364,&P18110,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/20.0.1410.63 Safari/537.31*/
const Device UA2675012 = {S28043, 106, { &P15364,&P18110,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/20.0.1410.63 Safari/537.31*/
const Device UA2717995 = {S28044, 109, { &P15364,&P18110,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/20.0.1410.63 Safari/537.31*/
const Device UA2718531 = {S28045, 106, { &P15364,&P18110,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/20.0.1410.63 Safari/537.31*/
const Device UA2738779 = {S28046, 133, { &P15364,&P18110,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11 (larbin2.6.3@unspecified.mail)*/
const Device UA2832767 = {S28047, 112, { &P15364,&P18110,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe)*/
const Device UA3043671 = {S28048, 102, { &P15364,&P18110,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i586) AppleWebKit/536.6 (KHTML, like Gecko) Chrome/20.0.1092.0 Safari/536.6*/
const Device UA42554 = {S28049, 137, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.3; en-us; pcdadr6350 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42557 = {S28050, 137, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; pcdadr6350 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA42561 = {S28051, 137, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;pcdadr6350 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42567 = {S28052, 138, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; pcdadr6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42569 = {S28053, 136, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-; pcdadr6350 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42578 = {S28054, 138, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; pcdadr6350 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58452 = {S28055, 137, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; pcdadr6350Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58455 = {S28056, 137, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; pcdadr6350 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58470 = {S28057, 138, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; pcdadr6350 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59718 = {S28058, 135, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ADR6350 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70586 = {S28059, 135, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70876 = {S28060, 137, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; pcdadr6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA71130 = {S28061, 134, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; ADR6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71129 = {S28062, 134, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA71375 = {S28063, 137, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;pcdadr6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71516 = {S28064, 137, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; pcdadr6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71553 = {S28065, 137, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; pcdadr6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72222 = {S28066, 137, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; pcdadr6350Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79167 = {S28067, 136, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-; pcdadr6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA220998 = {S28068, 59, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; ADR6350 Build/GRI40)*/
const Device UA241930 = {S28069, 59, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; ADR6350 Build/GRJ22)*/
const Device UA369240 = {S28070, 65, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.4; en_US; ADR6350 Build/GRJ22)*/
const Device UA371112 = {S28071, 134, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6350Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA700195 = {S28072, 135, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ADR6350 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA851004 = {S28073, 138, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; pcdadr6350 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1919381 = {S28074, 138, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; pcdadr6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1932824 = {S28075, 123, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; ADR6350 Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1946893 = {S28076, 134, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;ADR6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020109 = {S28077, 142, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6350 Build/GRJ22) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2020137 = {S28078, 134, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; ADR6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023294 = {S28079, 133, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; ADR6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023424 = {S28080, 59, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; ADR6350 Build/GRJ90)*/
const Device UA2444993 = {S28081, 128, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; ADR6350 Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2454051 = {S28082, 143, { &P14207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6350 Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30*/
const Device UA2436093 = {S28083, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/546.10 (KHTML, like Gecko) Version/6.0 Mobile/7E18WD Safari/8536.25*/
const Device UA2772326 = {S28084, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A381 Safari/8536.25*/
const Device UA2796726 = {S28085, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2796934 = {S28086, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A394 Safari/8536.25*/
const Device UA2797352 = {S28087, 135, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A4372q Safari/8536.25*/
const Device UA2799012 = {S28088, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A386 Safari/8536.25*/
const Device UA2799172 = {S28089, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A396 Safari/8536.25*/
const Device UA2800686 = {S28090, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A392 Safari/8536.25*/
const Device UA2869298 = {S28091, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A383 Safari/8536.25*/
const Device UA2870027 = {S28092, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A400 Safari/8536.25*/
const Device UA2870899 = {S28093, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.44 (KHTML, like Gecko) Version/6.0 Mobile/11A404 Safari/8536.25*/
const Device UA2872528 = {S28094, 135, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A4372r Safari/8536.25*/
const Device UA2874345 = {S28095, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A401 Safari/8536.25*/
const Device UA2875691 = {S28096, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.44 (KHTML, like Gecko) Version/6.0 Mobile/11A403 Safari/8536.25*/
const Device UA2883660 = {S28097, 135, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A4400f Safari/8536.25*/
const Device UA2886762 = {S28098, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.44 (KHTML, like Gecko) Version/6.0 Mobile/11A407 Safari/8536.25*/
const Device UA2887918 = {S28099, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.44 (KHTML, like Gecko) Version/6.0 Mobile/11A409 Safari/8536.25*/
const Device UA2888524 = {S28100, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.44 (KHTML, like Gecko) Version/6.0 Mobile/11A408 Safari/8536.25*/
const Device UA2897121 = {S28101, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.44 (KHTML, like Gecko) Version/6.0 Mobile/11A410 Safari/8536.25*/
const Device UA2898329 = {S28102, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11A416 Safari/8536.25*/
const Device UA2899277 = {S28103, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11A415 Safari/8536.25*/
const Device UA2945794 = {S28104, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11A414 Safari/8536.25*/
const Device UA2947000 = {S28105, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2947240 = {S28106, 135, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11A4414e Safari/8536.25*/
const Device UA2947238 = {S28107, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11A421 Safari/8536.25*/
const Device UA2963955 = {S28108, 135, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11A4414c Safari/8536.25*/
const Device UA2972863 = {S28109, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11A428 Safari/8536.25*/
const Device UA2974029 = {S28110, 135, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.49.1 (KHTML, like Gecko) Version/6.0 Mobile/11A432 Safari/8536.25*/
const Device UA2974950 = {S28111, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11A430 Safari/8536.25*/
const Device UA2975964 = {S28112, 135, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/11A435 Safari/8536.25*/
const Device UA3022935 = {S28113, 135, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/11A436 Safari/8536.25*/
const Device UA3023647 = {S28114, 135, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/11A438 Safari/8536.25*/
const Device UA3024241 = {S28115, 137, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/11A4435d Safari/8536.25*/
const Device UA3024283 = {S28116, 135, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/11A439 Safari/8536.25*/
const Device UA3026314 = {S28117, 135, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA3074675 = {S28118, 133, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA3110075 = {S28119, 137, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA3115618 = {S28120, 137, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA3128305 = {S28121, 135, { &P12280,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA428558 = {S28122, 78, { &P15364,&P7133,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Avant Browser)*/
const Device UA625067 = {S28123, 85, { &P15364,&P7133,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Avant Browser)*/
const Device UA649398 = {S28124, 90, { &P15364,&P7133,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; Avant Browser)*/
const Device UA2158249 = {S28125, 78, { &P15364,&P7133,&P22732,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; Avant Browser)*/
const Device UA2259331 = {S28126, 77, { &P15364,&P7133,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; AvantBrowser)*/
const Device UA2775657 = {S28127, 83, { &P15364,&P7133,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; SV1; Avant Browser)*/
const Device UA2824353 = {S28128, 93, { &P15364,&P7133,&P22732,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; Avant Browser; Avant Browser)*/
const Device UA2828026 = {S28129, 93, { &P15364,&P7133,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; Avant Browser; Avant Browser)*/
const Device UA2836484 = {S28130, 183, { &P15364,&P7133,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Avant Browser)*/
const Device UA2982161 = {S28131, 83, { &P15364,&P7133,&P22732,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; SV1; Avant Browser)*/
const Device UA3117926 = {S28132, 186, { &P15364,&P7133,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Avant Browser)*/
const Device UA191298 = {S28133, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.8.131 Version/11.11*/
const Device UA191429 = {S28134, 63, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; en) Presto/2.8.131 Version/11.10*/
const Device UA191938 = {S28135, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.8.131 Version/11.10*/
const Device UA196407 = {S28136, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.8.131 Version/11.11*/
const Device UA197113 = {S28137, 63, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; en) Presto/2.8.131 Version/11.11*/
const Device UA202415 = {S28138, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.8.131 Version/11.11*/
const Device UA310272 = {S28139, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.8.131 Version/11.10*/
const Device UA310394 = {S28140, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.8.131 Version/11.10*/
const Device UA310632 = {S28141, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.8.131 Version/11.10*/
const Device UA311111 = {S28142, 100, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; en) Presto/2.8.131 Version/11.10,gzip(gfe) (via translate.google.com)*/
const Device UA311130 = {S28143, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.8.131 Version/11.10*/
const Device UA311491 = {S28144, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.8.131 Version/11.10*/
const Device UA311603 = {S28145, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.8.131 Version/11.10*/
const Device UA312870 = {S28146, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.8.131 Version/11.10*/
const Device UA315942 = {S28147, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4110); ru) Presto/2.8.131 Version/11.11*/
const Device UA316265 = {S28148, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4110); ru) Presto/2.8.131 Version/11.10*/
const Device UA316389 = {S28149, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.8.131 Version/11.11*/
const Device UA319059 = {S28150, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.8.131 Version/11.11*/
const Device UA319098 = {S28151, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.8.131 Version/11.11*/
const Device UA319419 = {S28152, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.8.131 Version/11.11*/
const Device UA319504 = {S28153, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.8.131 Version/11.11*/
const Device UA320126 = {S28154, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.8.131 Version/11.11*/
const Device UA379702 = {S28155, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.8.131 Version/11.11*/
const Device UA382094 = {S28156, 34, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/11.1 (Windows NT 6.1: U: en)*/
const Device UA419567 = {S28157, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.8.131 Version/11.10*/
const Device UA542763 = {S28158, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.8.131 Version/11.10*/
const Device UA543359 = {S28159, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.8.131 Version/11.10*/
const Device UA552621 = {S28160, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.8.131 Version/11.10*/
const Device UA562521 = {S28161, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.8.131 Version/11.11*/
const Device UA569091 = {S28162, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.8.131 Version/11.10*/
const Device UA582589 = {S28163, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.8.131 Version/11.10*/
const Device UA652590 = {S28164, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.8.131 Version/11.10*/
const Device UA685004 = {S28165, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.8.131 Version/11.10*/
const Device UA694807 = {S28166, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.8.131 Version/11.11*/
const Device UA712626 = {S28167, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.8.131 Version/11.11*/
const Device UA725017 = {S28168, 100, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.8.131 Version/11.11,gzip(gfe) (via translate.google.com)*/
const Device UA730701 = {S28169, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.8.131 Version/11.11*/
const Device UA759349 = {S28170, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.8.131 Version/11.10*/
const Device UA781764 = {S28171, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.8.131 Version/11.11*/
const Device UA791628 = {S28172, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.8.131 Version/11.11*/
const Device UA869396 = {S28173, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.8.131 Version/11.11*/
const Device UA899968 = {S28174, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.8.131 Version/11.10*/
const Device UA1974821 = {S28175, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.8.131 Version/11.11*/
const Device UA1977825 = {S28176, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.8.131 Version/11.10*/
const Device UA1994936 = {S28177, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.8.131 Version/11.11*/
const Device UA2137573 = {S28178, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.8.131 Version/11.11*/
const Device UA2154909 = {S28179, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.8.131 Version/11.10*/
const Device UA2262773 = {S28180, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.8.131 Version/11.10*/
const Device UA2285249 = {S28181, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.8.131 Version/11.11*/
const Device UA2314345 = {S28182, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5196); ru) Presto/2.8.131 Version/11.11*/
const Device UA2316428 = {S28183, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.8.131 Version/11.11*/
const Device UA2328808 = {S28184, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5265); ru) Presto/2.8.131 Version/11.10*/
const Device UA2346335 = {S28185, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.8.131 Version/11.11*/
const Device UA2348847 = {S28186, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.8.131 Version/11.11*/
const Device UA2369581 = {S28187, 34, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/11.1 (Windows NT 6.1; U; en)*/
const Device UA2375778 = {S28188, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.8.131 Version/11.11*/
const Device UA2381887 = {S28189, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.8.131 Version/11.11*/
const Device UA2454263 = {S28190, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.8.131 Version/11.10*/
const Device UA2454737 = {S28191, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.8.131 Version/11.10*/
const Device UA2456511 = {S28192, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.8.131 Version/11.11*/
const Device UA2463115 = {S28193, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.8.131 Version/11.10*/
const Device UA2469166 = {S28194, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.8.131 Version/11.11*/
const Device UA2494640 = {S28195, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.8.131 Version/11.11*/
const Device UA2505215 = {S28196, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.8.131 Version/11.11*/
const Device UA2553338 = {S28197, 73, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.8.131 Version/11.11,gzip(gfe)*/
const Device UA2553612 = {S28198, 73, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.8.131 Version/11.10,gzip(gfe)*/
const Device UA2554164 = {S28199, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.8.131 Version/11.11*/
const Device UA2554569 = {S28200, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.8.131 Version/11.10*/
const Device UA2573913 = {S28201, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5308); ru) Presto/2.8.131 Version/11.11*/
const Device UA2685808 = {S28202, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5328); ru) Presto/2.8.131 Version/11.11*/
const Device UA2733232 = {S28203, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.8.131 Version/11.11*/
const Device UA2759217 = {S28204, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.8.131 Version/11.10*/
const Device UA2780012 = {S28205, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.8.131 Version/11.10*/
const Device UA2813001 = {S28206, 86, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.8.131 Version/11.10*/
const Device UA2949649 = {S28207, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.8.131 Version/11.11*/
const Device UA3044342 = {S28208, 85, { &P15364,&P7133,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.8.131 Version/11.10*/
const Device UA245262 = {S28209, 142, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369265 = {S28210, 149, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T679 Build/BPEARromICB.RC.3.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369333 = {S28211, 148, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T679 Build/Peach.Sunrise.1.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370394 = {S28212, 142, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA635148 = {S28213, 143, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; SGH-T679M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA639159 = {S28214, 66, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SGH-T679 Build/GINGERBREAD)*/
const Device UA723370 = {S28215, 141, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us;SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA723442 = {S28216, 141, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.5; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA742266 = {S28217, 141, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.5; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA760360 = {S28218, 141, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T679Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA762989 = {S28219, 141, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA863364 = {S28220, 149, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T679 Build/Peach.Sunrise.1.2a) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA900437 = {S28221, 66, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-T679 Build/GINGERBREAD)*/
const Device UA1936686 = {S28222, 67, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SGH-T679M Build/GINGERBREAD)*/
const Device UA1959727 = {S28223, 141, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.6; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1960839 = {S28224, 141, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us;SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1964448 = {S28225, 141, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA1976144 = {S28226, 141, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T679Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995335 = {S28227, 141, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030057 = {S28228, 150, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2475002 = {S28229, 149, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2546621 = {S28230, 141, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T679 Build/ExHibition) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2643248 = {S28231, 148, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T679 Build/Galaxy ExHibition) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2684874 = {S28232, 140, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2686382 = {S28233, 141, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; SGH-T679M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3117739 = {S28234, 135, { &P16273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SGH-T679 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA347708 = {S28235, 116, { &P17595,&P17297,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA366217 = {S28236, 116, { &P17595,&P17297,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7*/
const Device UA366218 = {S28237, 116, { &P17595,&P17297,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7*/
const Device UA366326 = {S28238, 116, { &P17595,&P17297,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7*/
const Device UA399007 = {S28239, 194, { &P17595,&P17297,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7 AppEngine-Google; (+http://code.google.com/appengine; appid: s~feedly-social)*/
const Device UA663800 = {S28240, 116, { &P17595,&P17297,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7*/
const Device UA899758 = {S28241, 195, { &P17595,&P17297,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7 AppEngine-Google; (+http://code.google.com/appengine; appid: s~feedly-twitter)*/
const Device UA2168214 = {S28242, 116, { &P17595,&P17297,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA2357846 = {S28243, 194, { &P17595,&P17297,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7 AppEngine-Google; (+http://code.google.com/appengine; appid: s~feedly-images)*/
const Device UA2844082 = {S28244, 118, { &P17595,&P17297,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10\_7\_2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.59 Safari/535.7*/
const Device UA2540638 = {S28245, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.458 Mobile Safari/537.10+*/
const Device UA2540640 = {S28246, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.413 Mobile Safari/537.10+*/
const Device UA2540639 = {S28247, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.479 Mobile Safari/537.10+*/
const Device UA2540641 = {S28248, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.693 Mobile Safari/537.10+*/
const Device UA2540642 = {S28249, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.278 Mobile Safari/537.10+*/
const Device UA2540643 = {S28250, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.212 Mobile Safari/537.10+*/
const Device UA2540644 = {S28251, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.416 Mobile Safari/537.10+*/
const Device UA2540645 = {S28252, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.422 Mobile Safari/537.10+*/
const Device UA2540647 = {S28253, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.642 Mobile Safari/537.10+*/
const Device UA2540646 = {S28254, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.542 Mobile Safari/537.10+*/
const Device UA2540648 = {S28255, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.512 Mobile Safari/537.10+*/
const Device UA2540649 = {S28256, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.440 Mobile Safari/537.10+*/
const Device UA2540650 = {S28257, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.206 Mobile Safari/537.10+*/
const Device UA2540651 = {S28258, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.311 Mobile Safari/537.10+*/
const Device UA2540652 = {S28259, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.362 Mobile Safari/537.10+*/
const Device UA2540653 = {S28260, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.582 Mobile Safari/537.10+*/
const Device UA2540654 = {S28261, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.401 Mobile Safari/537.10+*/
const Device UA2540655 = {S28262, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.395 Mobile Safari/537.10+*/
const Device UA2540656 = {S28263, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.155 Mobile Safari/537.10+*/
const Device UA2540657 = {S28264, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.297 Mobile Safari/537.10+*/
const Device UA2540658 = {S28265, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.461 Mobile Safari/537.10+*/
const Device UA2540659 = {S28266, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.455 Mobile Safari/537.10+*/
const Device UA2540660 = {S28267, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.786 Mobile Safari/537.10+*/
const Device UA2540661 = {S28268, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.777 Mobile Safari/537.10+*/
const Device UA2540662 = {S28269, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.269 Mobile Safari/537.10+*/
const Device UA2540664 = {S28270, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.527 Mobile Safari/537.10+*/
const Device UA2540666 = {S28271, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.254 Mobile Safari/537.10+*/
const Device UA2540667 = {S28272, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.392 Mobile Safari/537.10+*/
const Device UA2540668 = {S28273, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.368 Mobile Safari/537.10+*/
const Device UA2540669 = {S28274, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.383 Mobile Safari/537.10+*/
const Device UA2540670 = {S28275, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.305 Mobile Safari/537.10+*/
const Device UA2540671 = {S28276, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.699 Mobile Safari/537.10+*/
const Device UA2540672 = {S28277, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.152 Mobile Safari/537.10+*/
const Device UA2540673 = {S28278, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.230 Mobile Safari/537.10+*/
const Device UA2540674 = {S28279, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.491 Mobile Safari/537.10+*/
const Device UA2540675 = {S28280, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.594 Mobile Safari/537.10+*/
const Device UA2540676 = {S28281, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.789 Mobile Safari/537.10+*/
const Device UA2540677 = {S28282, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.684 Mobile Safari/537.10+*/
const Device UA2540678 = {S28283, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.476 Mobile Safari/537.10+*/
const Device UA2540679 = {S28284, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.795 Mobile Safari/537.10+*/
const Device UA2540680 = {S28285, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.792 Mobile Safari/537.10+*/
const Device UA2540681 = {S28286, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.224 Mobile Safari/537.10+*/
const Device UA2540682 = {S28287, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.816 Mobile Safari/537.10+*/
const Device UA2540663 = {S28288, 103, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.42 Mobile Safari/537.10+*/
const Device UA2567689 = {S28289, 104, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.837 Mobile Safari/537.10+*/
const Device UA2585424 = {S28290, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1002 Mobile Safari/537.10+*/
const Device UA2601990 = {S28291, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1110 Mobile Safari/537.10+*/
const Device UA2606642 = {S28292, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1020 Mobile Safari/537.10+*/
const Device UA2674997 = {S28293, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1471 Mobile Safari/537.10+*/
const Device UA2675420 = {S28294, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1320 Mobile Safari/537.10+*/
const Device UA2676135 = {S28295, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1483 Mobile Safari/537.10+*/
const Device UA2680697 = {S28296, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1507 Mobile Safari/537.10+*/
const Device UA2710033 = {S28297, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1720 Mobile Safari/537.10+*/
const Device UA2713392 = {S28298, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1429 Mobile Safari/537.10+*/
const Device UA2715239 = {S28299, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1447 Mobile Safari/537.10+*/
const Device UA2716489 = {S28300, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1756 Mobile Safari/537.10+*/
const Device UA2716710 = {S28301, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1597 Mobile Safari/537.10+*/
const Device UA2718787 = {S28302, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1609 Mobile Safari/537.10+*/
const Device UA2719278 = {S28303, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1627 Mobile Safari/537.10+*/
const Device UA2738158 = {S28304, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1666 Mobile Safari/537.10+*/
const Device UA2739855 = {S28305, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1832 Mobile Safari/537.10+*/
const Device UA2740960 = {S28306, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1916 Mobile Safari/537.10+*/
const Device UA2754672 = {S28307, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1895 Mobile Safari/537.10+*/
const Device UA2754824 = {S28308, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2336 Mobile Safari/537.10+*/
const Device UA2769718 = {S28309, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2342 Mobile Safari/537.10+*/
const Device UA2781468 = {S28310, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1485 Mobile Safari/537.10+*/
const Device UA2781659 = {S28311, 112, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - LinkedInBlackBerry/10.1.2 (5) Q10 SQN100-3/10.1.0.1483 (Research In Motion;RIM BlackBerry Device;Q10;SQN100-3;1)*/
const Device UA2783703 = {S28312, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2390 Mobile Safari/537.10+*/
const Device UA2784040 = {S28313, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2038 Mobile Safari/537.10+*/
const Device UA2784161 = {S28314, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2420 Mobile Safari/537.10+*/
const Device UA2796902 = {S28315, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4285 Mobile Safari/537.10+*/
const Device UA2797562 = {S28316, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4333 Mobile Safari/537.10+*/
const Device UA2800591 = {S28317, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2062 Mobile Safari/537.10+*/
const Device UA2800694 = {S28318, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4369 Mobile Safari/537.10+*/
const Device UA2863948 = {S28319, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1552 Mobile Safari/537.10+*/
const Device UA2865199 = {S28320, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1411 Mobile Safari/537.10+*/
const Device UA2869765 = {S28321, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1910 Mobile Safari/537.10+*/
const Device UA2870164 = {S28322, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2354 Mobile Safari/537.10+*/
const Device UA2872527 = {S28323, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2025 Mobile Safari/537.10+*/
const Device UA2873632 = {S28324, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4429 Mobile Safari/537.10+*/
const Device UA2876352 = {S28325, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2009 Mobile Safari/537.10+*/
const Device UA2885078 = {S28326, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4453 Mobile Safari/537.10+*/
const Device UA2894630 = {S28327, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2121 Mobile Safari/537.10+*/
const Device UA2899272 = {S28328, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4537 Mobile Safari/537.10+*/
const Device UA2943973 = {S28329, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4477 Mobile Safari/537.10+*/
const Device UA2946877 = {S28330, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4543 Mobile Safari/537.10+*/
const Device UA2957129 = {S28331, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1200 Mobile Safari/537.10+*/
const Device UA2957316 = {S28332, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.1880 Mobile Safari/537.10+*/
const Device UA2960361 = {S28333, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4393 Mobile Safari/537.10+*/
const Device UA2966191 = {S28334, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4597 Mobile Safari/537.10+*/
const Device UA2971097 = {S28335, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4633 Mobile Safari/537.10+*/
const Device UA2971146 = {S28336, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4561 Mobile Safari/537.10+*/
const Device UA2972574 = {S28337, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4651 Mobile Safari/537.10+*/
const Device UA2976080 = {S28338, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4687 Mobile Safari/537.10+*/
const Device UA2997801 = {S28339, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.2324 Mobile Safari/537.10+*/
const Device UA3005448 = {S28340, 112, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - LinkedInBlackBerry/10.1.2 (5) Q10 SQN100-2/10.1.0.2038 (Research In Motion;RIM BlackBerry Device;Q10;SQN100-2;1)*/
const Device UA3022366 = {S28341, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4699 Mobile Safari/537.10+*/
const Device UA3033717 = {S28342, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4780 Mobile Safari/537.10+*/
const Device UA3037398 = {S28343, 105, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.1.0.4828 Mobile Safari/537.10+*/
const Device UA3078571 = {S28344, 112, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - LinkedInBlackBerry/10.1.4 (1) Q10 SQN100-1/10.1.0.4633 (Research In Motion;RIM BlackBerry Device;Q10;SQN100-1;1)*/
const Device UA3092291 = {S28345, 112, { &P21285,&P22468,&P22813,&P18092 }};	/*Useragent - LinkedInBlackBerry/10.1.4 (1) Q10 SQN100-5/10.1.0.1720 (Research In Motion;RIM BlackBerry Device;Q10;SQN100-5;1)*/
const Device UA48821 = {S28346, 131, { &P13144,&P6374,&P4418,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9860; en-GB) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.296 Mobile Safari/534.11+*/
const Device UA853292 = {S28347, 77, { &P13144,&P6374,&P4418,&P18092 }};	/*Useragent - BlackBerry9860/7.0.0.576 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102*/
const Device UA2042135 = {S28348, 65, { &P15364,&P5690,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:16.0) Gecko/X Firefox/16.0*/
const Device UA2234789 = {S28349, 68, { &P15364,&P5690,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:16.0) Gecko/X Firefox/16.0,gzip(gfe)*/
const Device UA2356557 = {S28350, 70, { &P15364,&P5690,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:16.0) Gecko/X Firefox/16.0*/
const Device UA3057443 = {S28351, 74, { &P15364,&P5690,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; rv:16.0.1) Gecko/X Firefox/16.0.1*/
const Device UA368840 = {S28352, 147, { &P13042,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Incredible S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA903319 = {S28353, 230, { &P13042,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTCS710e/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA912281 = {S28354, 148, { &P13042,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; IncredibleS_S710e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1956396 = {S28355, 138, { &P13042,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Incredible S Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1989558 = {S28356, 152, { &P13042,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_IncredibleS_S710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1995450 = {S28357, 69, { &P13042,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Incredible S Build/IMM76D)*/
const Device UA1998767 = {S28358, 147, { &P13042,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; HTC Incredible S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2010373 = {S28359, 152, { &P13042,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; HTC_IncredibleS_S710e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2024979 = {S28360, 138, { &P13042,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Incredible S Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2025683 = {S28361, 69, { &P13042,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Incredible S Build/IML74K)*/
const Device UA2028874 = {S28362, 133, { &P13042,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Incredible S Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2385904 = {S28363, 139, { &P13042,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCS710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2545906 = {S28364, 146, { &P13042,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-; IncredibleS_S710e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2546445 = {S28365, 138, { &P13042,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Incredible S Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2654070 = {S28366, 147, { &P13042,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Incredible S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2727143 = {S28367, 147, { &P13042,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Incredible_S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2759077 = {S28368, 145, { &P13042,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-; HTC Incredible S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2765539 = {S28369, 158, { &P13042,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_IncredibleS_S710e Build/2.15.401.100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2993638 = {S28370, 152, { &P13042,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_IncredibleS_S710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2195394 = {S28371, 139, { &P19861,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I605 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2382015 = {S28372, 61, { &P19861,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SCH-I605 Build/JRO03C)*/
const Device UA2430864 = {S28373, 139, { &P19861,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I605 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2508189 = {S28374, 139, { &P19861,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I605 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2508302 = {S28375, 139, { &P19861,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I605 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2655383 = {S28376, 137, { &P19861,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; SCH-I605 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2696881 = {S28377, 61, { &P19861,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-I605 Build/JZO54K)*/
const Device UA2879046 = {S28378, 137, { &P19861,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SCH-I605 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108961 = {S28379, 172, { &P19861,&P18111,&P18112,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=t0ltevzw;devm=SCH-I605;man=samsung;brw=Android;shwd=720x1280;shwa=720x1230;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA615617 = {S28380, 132, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA879228 = {S28381, 61, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-P5113 Build/IML74K)*/
const Device UA906492 = {S28382, 132, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1957310 = {S28383, 61, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P5113 Build/IMM76D)*/
const Device UA2167200 = {S28384, 130, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2608355 = {S28385, 139, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2609433 = {S28386, 139, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2610465 = {S28387, 130, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-P5113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2610787 = {S28388, 79, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/1.1.3 (Linux; U; Android 4.0.4; en-CA; GT-P5113 Build/IMM76D.P5113UEBLE4)*/
const Device UA2611550 = {S28389, 92, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.youtube/3.5.5(Linux; U; Android 4.0.3; en_US; GT-P5113 Build/IML74K) gzip*/
const Device UA2614566 = {S28390, 93, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.youtube/4.0.23(Linux; U; Android 4.0.4; en_US; GT-P5113 Build/IMM76D) gzip*/
const Device UA2614889 = {S28391, 79, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.0.4; en-US; GT-P5113 Build/IMM76D.P5113UEBLG5)*/
const Device UA2615692 = {S28392, 117, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Android 4.0.3; en-us; GT-P5113 Build/IML74K) FreeWheelAdManager/4.9.2-r8793-X;com.gotv.crackle.handset/X*/
const Device UA2615695 = {S28393, 117, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Android 4.0.4; en-us; GT-P5113 Build/IMM76D) FreeWheelAdManager/5.0.0-r8951-X;com.gotv.crackle.handset/X*/
const Device UA2615809 = {S28394, 79, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.0.4; en-US; GT-P5113 Build/IMM76D.P5113UEBLG2)*/
const Device UA2615854 = {S28395, 56, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Android/4.4.1 (Linux; U; Android 4.0.4; en_US; GT-P5113)*/
const Device UA2615986 = {S28396, 145, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA2616095 = {S28397, 163, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/5.0.2 Version/4.0 Safari/534.30 Safari/6533.18.5*/
const Device UA2616476 = {S28398, 145, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2617209 = {S28399, 153, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5113 Build/IMM76D; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2617370 = {S28400, 144, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2617517 = {S28401, 130, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5113 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2617924 = {S28402, 79, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.3; en-US; GT-P5113 Build/IML74K.P5113UEALD7)*/
const Device UA2618138 = {S28403, 273, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/1.9.11;FBDM/{density=1.0,width=800,height=1232};FBLC/zh_TW;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P5113;FBSV/4.0.4;]*/
const Device UA2618194 = {S28404, 103, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 4.0.4; SAMSUNG GT-P5113 Build/ICE_CREAM_SANDWICH_MR1);*/
const Device UA2618887 = {S28405, 62, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.1; 9) GT-P5113 (Android 4.0.4, SDK 15)*/
const Device UA2618967 = {S28406, 56, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Android/4.4.1 (Linux; U; Android 4.0.3; en_US; GT-P5113)*/
const Device UA2619537 = {S28407, 134, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2620910 = {S28408, 142, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P5113 Build/IMM76D like Xoom) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2621218 = {S28409, 94, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.apps.plus/X (Linux; U; Android 4.0.4; en_US; GT-P5113; Build/IMM76D) (gzip)*/
const Device UA2621440 = {S28410, 139, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2621849 = {S28411, 195, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 androidAppVersion/2.5.16 AndroidWebkitWrapper BillingSupported*/
const Device UA2621880 = {S28412, 81, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-CA; GT-P5113 Build/IMM76D.P5113UEBLH3)*/
const Device UA2623729 = {S28413, 134, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Groupon/2.1.2486 (Android 4.0.4; Samsung Espresso10wifi / Samsung GT-P5113; )[preload=false;locale=en_CA;clientidbase=android-samsung]*/
const Device UA2624807 = {S28414, 88, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik 2.04.12078; (Linux; U; Android 4.0.4; GT-P5113; ON) de4a3718b1a6296; 1280x752x1.0*/
const Device UA2625362 = {S28415, 125, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; GT-P5113 Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2625525 = {S28416, 130, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5113 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2628625 = {S28417, 151, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MantanoReader*/
const Device UA2629244 = {S28418, 170, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D   http://stubes.tk) AppleWebKit/534.30 (KHTML, like Gecko   http://stubes.tk) Version/4.0 Safari/534.30*/
const Device UA2629459 = {S28419, 134, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5113 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2630446 = {S28420, 137, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2631348 = {S28421, 80, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - VKAndroidApp/2.7.6-64 (Android 4.0.4; SDK 15; armeabi-v7a; samsung GT-P5113; ru)*/
const Device UA2636556 = {S28422, 162, { &P22043,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5113 Build/IML74K) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA626062 = {S28423, 161, { &P18349,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5690/S5690BVLC5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626837 = {S28424, 142, { &P18349,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5690 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA657250 = {S28425, 143, { &P18349,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; GT-S5690M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA660594 = {S28426, 143, { &P18349,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-S5690L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA665091 = {S28427, 66, { &P18349,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5690 Build/GINGERBREAD)*/
const Device UA745874 = {S28428, 143, { &P18349,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; GT-S5690R Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA899427 = {S28429, 161, { &P18349,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5690/S5690BVLD1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1983219 = {S28430, 160, { &P18349,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5690-ORANGE/S5690BVLD1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1986452 = {S28431, 161, { &P18349,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-S5690/S5690BOLE2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028643 = {S28432, 161, { &P18349,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5690/S5690BVLF2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2164121 = {S28433, 160, { &P18349,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5690-ORANGE/S5690BVLF2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2444633 = {S28434, 160, { &P18349,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-S5690-ORANGE/S5690BVLC5 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2490011 = {S28435, 161, { &P18349,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-S5690/S5690BOLL1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2518664 = {S28436, 135, { &P18349,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5690 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2519925 = {S28437, 67, { &P18349,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5690L Build/GINGERBREAD)*/
const Device UA2691480 = {S28438, 140, { &P18349,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5690 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2721835 = {S28439, 67, { &P18349,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5690R Build/GINGERBREAD)*/
const Device UA2749943 = {S28440, 161, { &P18349,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SAMSUNG GT-S5690/S5690BVLL2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2816628 = {S28441, 154, { &P18349,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6_UOU?-O??O??; iw-il; GT-S5690 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2933506 = {S28441, 154, { &P18349,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6_UOU?-O??O??; iw-il; GT-S5690 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2991336 = {S28442, 160, { &P18349,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ro-ro; GT-S5690-ORANGE/S5690BVLL2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2972407 = {S28443, 131, { &P15364,&P18118,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/28.0.1500.71 Chrome/28.0.1500.71 Safari/537.36*/
const Device UA2973597 = {S28444, 133, { &P15364,&P18118,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/28.0.1500.71 Chrome/28.0.1500.71 Safari/537.36*/
const Device UA3021704 = {S28445, 133, { &P15364,&P18118,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/28.0.1500.71 Chrome/28.0.1500.71 Safari/537.36*/
const Device UA3025618 = {S28446, 141, { &P15364,&P18118,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/28.0.1500.71 Chrome/28.0.1500.71 Safari/537.36,gzip(gfe)*/
const Device UA3058258 = {S28447, 172, { &P15364,&P18118,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/28.0.1500.71 Chrome/28.0.1500.71 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3058263 = {S28448, 143, { &P15364,&P18118,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/28.0.1500.71 Chrome/28.0.1500.71 Safari/537.36,gzip(gfe)*/
const Device UA3078236 = {S28449, 170, { &P15364,&P18118,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/28.0.1500.71 Chrome/28.0.1500.71 Safari/537.36,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA456586 = {S28450, 65, { &P15364,&P16110,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:12.0) Gecko/X Firefox/12.0*/
const Device UA614772 = {S28451, 70, { &P15364,&P16110,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:12.0) Gecko/X Firefox/12.0*/
const Device UA686124 = {S28452, 96, { &P15364,&P16110,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Windows; U; Windows NT 6.2; WOW64; en-US; rv:12.0) Gecko/X Firefox/12.0*/
const Device UA899615 = {S28453, 89, { &P15364,&P16110,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Windows; U; Windows NT 6.2; en-US; rv:12.0) Gecko/X Firefox/12.0*/
const Device UA614762 = {S28454, 159, { &P23905,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100P-ORANGE/I9100PBVLP6 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA657042 = {S28455, 140, { &P23905,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100P Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA903237 = {S28456, 159, { &P23905,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100P-ORANGE/I9100PBVLP8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA911833 = {S28457, 159, { &P23905,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100P-ORANGE/I9100PBVLPA Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1962240 = {S28458, 160, { &P23905,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; SAMSUNG GT-I9100P/I9100PXXLPS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1965114 = {S28459, 62, { &P23905,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-I9100P Build/IML74K)*/
const Device UA2031421 = {S28460, 160, { &P23905,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; SAMSUNG GT-I9100P/I9100PXXLPP Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2041860 = {S28461, 140, { &P23905,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; GT-I9100P Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2205823 = {S28462, 159, { &P23905,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100P-ORANGE/I9100PBVLPC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2242280 = {S28463, 62, { &P23905,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9100P Build/IMM76D)*/
const Device UA2471649 = {S28464, 160, { &P23905,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SAMSUNG GT-I9100P/I9100PXXLQ8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2584962 = {S28465, 159, { &P23905,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100P-ORANGE/I9100PBVLPE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2762516 = {S28466, 138, { &P23905,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-I9100P Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA623533 = {S28467, 128, { &P18285,&P7106,&P15829,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9220; en) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.337 Mobile Safari/534.11+*/
const Device UA92866 = {S28468, 114, { &P15364,&P17285,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9.1*/
const Device UA98832 = {S28469, 112, { &P15364,&P17285,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Safari/528.17*/
const Device UA99182 = {S28470, 119, { &P15364,&P17285,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA103014 = {S28471, 118, { &P15364,&P17285,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19.1*/
const Device UA128177 = {S28472, 118, { &P15364,&P17285,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA131284 = {S28473, 112, { &P15364,&P17285,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA414607 = {S28474, 112, { &P15364,&P17285,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; ru-RU) AppleWebKit/528.16 (KHTML, like Gecko) Version/4.0 Safari/528.16*/
const Device UA2258993 = {S28475, 106, { &P15364,&P17285,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.26.9 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA2424559 = {S28476, 99, { &P15364,&P17285,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA2427406 = {S28477, 100, { &P15364,&P17285,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA2467397 = {S28478, 107, { &P15364,&P17285,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA2670615 = {S28479, 138, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2670605 = {S28480, 135, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SC-02E Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2670737 = {S28481, 131, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2671179 = {S28482, 145, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I317 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2671635 = {S28483, 138, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2672198 = {S28484, 130, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2673947 = {S28485, 138, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2674420 = {S28486, 138, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675691 = {S28487, 130, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676032 = {S28488, 137, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676040 = {S28489, 137, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676075 = {S28490, 137, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676077 = {S28491, 137, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676100 = {S28492, 137, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676184 = {S28493, 137, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676200 = {S28494, 145, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676242 = {S28495, 138, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676283 = {S28496, 138, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105T Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676424 = {S28497, 137, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SPH-L900 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676425 = {S28498, 130, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676672 = {S28499, 131, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105T Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676682 = {S28500, 138, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889V Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676684 = {S28501, 130, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676731 = {S28502, 138, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I317M Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676780 = {S28503, 131, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676829 = {S28504, 137, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R950 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677157 = {S28505, 137, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677160 = {S28506, 137, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677162 = {S28507, 130, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2677287 = {S28508, 130, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2677295 = {S28509, 138, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708038 = {S28510, 130, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2709101 = {S28511, 131, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889V Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2709363 = {S28512, 137, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-R950 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2710429 = {S28513, 130, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R950 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2713247 = {S28514, 138, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734786 = {S28515, 137, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-N719 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734900 = {S28516, 130, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SPH-L900 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2734983 = {S28517, 137, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; GT-N7100 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735057 = {S28518, 138, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I317 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735337 = {S28519, 138, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T889V Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2736251 = {S28520, 137, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-N719 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2736435 = {S28521, 130, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2864569 = {S28522, 138, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SHV-E250S Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3047652 = {S28523, 131, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3047676 = {S28524, 131, { &P18442,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA19875 = {S28525, 141, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC-A9192/1.0 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA23646 = {S28526, 138, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Inspire 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42587 = {S28527, 140, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; HTC-A9192/1.0 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42595 = {S28528, 140, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.3; en-us; HTC-A9192/1.0 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42599 = {S28529, 140, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;HTC-A9192/1.0 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA45979 = {S28530, 143, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Inspire" 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48571 = {S28531, 137, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC-A9192 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48573 = {S28532, 137, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC-A9192 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61586 = {S28533, 142, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_Inspire_4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA69905 = {S28534, 138, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Inspire 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70392 = {S28535, 141, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC-A9192/1.0 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70756 = {S28536, 142, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Inspire 4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72930 = {S28537, 141, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC-A9192/1.0 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA319042 = {S28538, 139, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Inspire 4G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA321260 = {S28539, 138, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Inspire 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA332345 = {S28540, 138, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Inspire 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA453128 = {S28541, 62, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Inspire 4G Build/GRJ90)*/
const Device UA575067 = {S28542, 138, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Inspire 4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA576647 = {S28543, 141, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC-A9192/1.0 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA714871 = {S28544, 62, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; Inspire 4G Build/GRI40)*/
const Device UA715216 = {S28545, 155, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Inspire 4G Build/GRI40; Absolution v3.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA765541 = {S28546, 152, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Inspire 4G Build/Eva.MIUI-2.1.13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA776941 = {S28547, 137, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC-A9192 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA849252 = {S28548, 145, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Inspire 4G Build/Eva.MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021049 = {S28549, 140, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC-A9192/1.0 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA2021190 = {S28550, 140, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC-A9192/1.0Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021531 = {S28551, 153, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Inspire 4G Build/Eva.MIUI.1.12.16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021742 = {S28552, 141, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC-A9192/1.0 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022549 = {S28553, 66, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Inspire 4G Build/GRJ90)*/
const Device UA2868557 = {S28554, 142, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC-A9192/1.0 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3102384 = {S28555, 125, { &P17108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC-A9192/1.0 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mob*/
const Device UA35443 = {S28556, 47, { &P15364,&P5690,&P17215,&P18092 }};	/*Useragent - Opera/9.5  (Windows NT 5.1; U; ko) Presto/2.0.0*/
const Device UA38527 = {S28557, 93, { &P15364,&P5690,&P17215,&P18092 }};	/*Useragent - Opera/9.50 (Windows NT 5.1; MDA_Compact_V/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1; U; de)*/
const Device UA92924 = {S28558, 34, { &P15364,&P5690,&P17215,&P18092 }};	/*Useragent - Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA95069 = {S28559, 34, { &P15364,&P5690,&P17215,&P18092 }};	/*Useragent - Opera/9.51 (Windows NT 5.1; U; ru)*/
const Device UA101047 = {S28560, 34, { &P15364,&P5690,&P17215,&P18092 }};	/*Useragent - Opera/9.52 (Windows NT 5.1; U; ru)*/
const Device UA119714 = {S28561, 34, { &P15364,&P5690,&P17215,&P18092 }};	/*Useragent - Opera/9.51 (Windows NT 5.2; U; en)*/
const Device UA131416 = {S28562, 33, { &P15364,&P5690,&P17215,&P18092 }};	/*Useragent - Opera/9.5 (Windows NT 5.1; U; en)*/
const Device UA238580 = {S28563, 37, { &P15364,&P5690,&P17215,&P18092 }};	/*Useragent - Opera/9.50 (Windows NT 5.2; U; zh-cn)*/
const Device UA414453 = {S28564, 65, { &P15364,&P5690,&P17215,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; ru) Opera 9.52*/
const Device UA442459 = {S28565, 57, { &P15364,&P5690,&P17215,&P18092 }};	/*Useragent - Opera/9.52 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru)*/
const Device UA445647 = {S28566, 57, { &P15364,&P5690,&P17215,&P18092 }};	/*Useragent - Opera/9.52 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru)*/
const Device UA476142 = {S28567, 57, { &P15364,&P5690,&P17215,&P18092 }};	/*Useragent - Opera/9.51 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru)*/
const Device UA627323 = {S28568, 34, { &P15364,&P5690,&P17215,&P18092 }};	/*Useragent - Opera/9.53 (Windows NT 5.1; U; ru)*/
const Device UA799673 = {S28569, 65, { &P15364,&P5690,&P17215,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; en) Opera 9.50*/
const Device UA893794 = {S28570, 65, { &P15364,&P5690,&P17215,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; ru) Opera 9.51*/
const Device UA2212265 = {S28571, 34, { &P15364,&P5690,&P17215,&P18092 }};	/*Useragent - Opera/9.52 (Windows NT 5.2; U; ru)*/
const Device UA2801332 = {S28572, 57, { &P15364,&P5690,&P17215,&P18092 }};	/*Useragent - Opera/9.50 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru)*/
const Device UA1949163 = {S28573, 75, { &P15364,&P18110,&P17216,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; en; rv:2.0) Gecko/X Firefox/4.0 Opera 12.00*/
const Device UA1995944 = {S28574, 55, { &P15364,&P18110,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Linux; U; en) Presto/2.10.254 Version/12.00*/
const Device UA1996402 = {S28575, 65, { &P15364,&P18110,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; en) Presto/2.10.289 Version/12.01*/
const Device UA2391203 = {S28576, 60, { &P15364,&P18110,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux x86_64) Presto/2.12.423 Version/12.00*/
const Device UA2714735 = {S28577, 75, { &P15364,&P18110,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Linux i686; U; fr) Presto/2.10.287 Version/12.00 ; SC/IHD92 STB*/
const Device UA3052405 = {S28578, 88, { &P15364,&P18110,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; U;  CE-HTML/1.0 NETTV/4.1.1; en) Presto/2.10.287 Version/12.00*/
const Device UA24231 = {S28579, 123, { &P14635,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA26961 = {S28580, 128, { &P14635,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaC3-00.1/5.0(ATT.03.61) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA31053 = {S28581, 73, { &P14635,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (04.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac3-00*/
const Device UA2984997 = {S28582, 186, { &P14635,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (08.71) Profile/MIDP-2.1 Configuration/CLDC-1.1 NokiaC3-00/5.0 (08.71) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA73116 = {S28583, 146, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; MOT-XT910 Build/6.5.1-73_SPU-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA350183 = {S28584, 147, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; MOT-XT910 Build/6.5.1_73_SLC-25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA351497 = {S28585, 147, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; MOT-XT910 Build/6.5.1-73_SLC-24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA366016 = {S28586, 148, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; XT910 Build/6.5.1-73_SPU-11-M1-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370696 = {S28587, 144, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; XT910 Build/6.5.1-167_SPU-14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA428551 = {S28588, 148, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; XT910 Build/6.5.1-167-SPD-IRD-40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA575690 = {S28589, 146, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; MOT-XT910 Build/6.5.1-73_SPU-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA579674 = {S28590, 135, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; XT910 Build/6.5.1_73-spd-ird-12) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA606630 = {S28591, 151, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; MOT-XT910 Build/6.5.1-167_SLC-M33.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA608845 = {S28592, 147, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; XT910 Build/6.5.1-167_SLC-M33.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622474 = {S28593, 153, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; XT910 Build/6.5.1-167_SLC-M33.1) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30*/
const Device UA624353 = {S28594, 143, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; XT910 Build/6.5.1_73_SLC-24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA624971 = {S28595, 153, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MOT-XT910 Build/6.5.1-167_SPU-15-M2-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627056 = {S28596, 151, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MOT-XT910 Build/6.5.1-167_SPU-15-M2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA628083 = {S28597, 151, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; MOT-XT910 Build/6.5.1_73-SPD-IRD-21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA632172 = {S28598, 147, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; MOT-XT910 Build/6.5.1_73_SLC-24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA637400 = {S28599, 147, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; XT910 Build/6.5.1_73-SPD-IRD-21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA653490 = {S28600, 152, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; MOT-XT910 Build/6.5.1-73_SPU-11-M1-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA658907 = {S28601, 152, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MOT-XT910 Build/6.5.1-167-SPD-IRD-40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA659536 = {S28602, 151, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; MOT-XT910 Build/6.5.1-73_SPU-9-TA-6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA662643 = {S28603, 156, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; MOT-XT910 Build/6.5.1-167_SPU-15-M2-1-YO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA667392 = {S28604, 156, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MOT-XT910 Build/6.5.1-167_SPU-15-M2-1-DE) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA668200 = {S28605, 143, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; XT910 Build/6.5.1-73_SLC-24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA669937 = {S28606, 149, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MOT-XT910 Build/6.5.1-167_SLC-M37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA672695 = {S28607, 147, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; XT910 Build/6.5.1-167_SPU-15-M2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA684828 = {S28608, 149, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; XT910 Build/6.5.1-167_SPU-15-M2-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA685345 = {S28609, 151, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MOT-XT910 Build/6.5.1_73-spd-ird-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA686450 = {S28610, 149, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; MOT-XT910 Build/6.5.1_73_SLC-8.3M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA703260 = {S28611, 142, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; XT910 Build/6.5.1-73_SPU-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA708401 = {S28612, 147, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; XT910 Build/6.5.1_73-spd-ird-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA710599 = {S28613, 150, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; MOT-XT910 Build/6.5.1-73_SLC-ME6.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA718486 = {S28614, 151, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; MOT-XT910 Build/6.5.1-73_SPU-9-TA-4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA729228 = {S28615, 150, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; XT910K Build/6.5.1-8_SDK-31_LE-106) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA736566 = {S28616, 72, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; XT910 Build/6.5.1-167-SPD-IRD-40)*/
const Device UA739622 = {S28617, 143, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; XT910 Build/6.5.1_73_SLC-25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA747634 = {S28618, 152, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; XT910 Build/6.5.1-167_SPU-15-M2-1-DE) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA751236 = {S28619, 149, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; MOT-XT910 Build/6.5.1_73_SLC-8.2M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA752766 = {S28620, 151, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MOT-XT910 Build/6.5.1_73-spd-ird-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA764969 = {S28621, 151, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; MOT-XT910 Build/6.5.1-73_SPU-9-TA-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA767050 = {S28622, 149, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; XT910K Build/6.5.1-8_SDK-31_LE-58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA774737 = {S28623, 155, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; MOT-XT910 Build/6.5.1-73_SPU-11-M1-2-DE) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA781261 = {S28624, 147, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MOT-XT910 Build/6.5.1-73_SPU-11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA785751 = {S28625, 153, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MOT-XT910 Build/6.5.1-167_SLC-M36_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA786539 = {S28626, 146, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; XT910 Build/6.5.1-73_SLC-ME6.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA813551 = {S28627, 147, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; he-il; XT910 Build/6.5.1_73-spd-ird-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA822273 = {S28628, 156, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; MOT-XT910 Build/6.5.1-73_SPU-11-M1-2-SFR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA824013 = {S28629, 149, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; MOT-XT910 Build/6.5.1_73_SLC-8.1M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA828330 = {S28630, 145, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT910 Build/6.5.1-167_SLC-M37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA829887 = {S28631, 149, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT910 Build/6.5.1-167_SLC-M36_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA833500 = {S28632, 148, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT910 Build/6.5.1_GC-136-SPDU-13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA834859 = {S28633, 152, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; XT910 Build/6.5.1-167_SPU-15-M2-1-YO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA848978 = {S28634, 150, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; XT910S Build/6.5.1-8_SDS-29_LE-106) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA853887 = {S28635, 156, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MOT-XT910 Build/6.5.1-167_SPU-15-M2-1-FR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA895035 = {S28636, 153, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; MOT-XT910 Build/6.5.1-167_SPU-15-M2-4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1930977 = {S28637, 147, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT910 Build/6.5.1_GC-88-SPDU-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1932343 = {S28638, 149, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; XT910S Build/6.5.1-8_SDS-29_LE-58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1936728 = {S28639, 148, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MOT-XT910 Build/6.5.1-150_SPU-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1939417 = {S28640, 151, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; MOT-XT910 Build/6.5.1-73_SPU-9-TA-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1942282 = {S28641, 147, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; XT910 Build/6.5.1-73_SPU-9-TA-6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1945780 = {S28642, 152, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; XT910 Build/6.5.1-73_SPU-11-M1-2-SFR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1953069 = {S28643, 145, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; XT910 Build/6.5.1_73_SLC-8.3M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1977191 = {S28644, 157, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MOT-XT910 Build/6.5.1-167_SPU-15-M2-1-CEE) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1987399 = {S28645, 153, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MOT-XT910 Build/6.5.1-167_SLC-M36_R02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1989717 = {S28646, 136, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT910 Build/6.5.1-167-SPD-IRD-40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1992015 = {S28647, 71, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; XT910 Build/6.5.1_73-SPD-IRD-21)*/
const Device UA1994438 = {S28648, 153, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MOT-XT910 Build/6.5.1-167_SLC-M34_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995447 = {S28649, 152, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; he-il; MOT-XT910 Build/6.5.1-167-SPD-IRD-39) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021461 = {S28650, 144, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; XT910 Build/6.5.1-150_SPU-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021710 = {S28651, 148, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MOT-XT910 Build/6.5.1-167_SPU-13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022035 = {S28652, 148, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MOT-XT910 Build/6.5.1-144_SPU-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022145 = {S28653, 151, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; XT910 Build/6.5.1-73_SPU-11-M1-2-DE) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022997 = {S28654, 151, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; el-gr; MOT-XT910 Build/6.5.1-73_SPU-9-TA-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023899 = {S28655, 153, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; da-dk; XT910 Build/6.5.1-167_SPU-15-M2-1-CEE) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024242 = {S28656, 148, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MOT-XT910 Build/6.5.1-167_SPU-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024262 = {S28657, 147, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT910 Build/6.5.1_GC-136-SPDU-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024821 = {S28658, 145, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; XT910 Build/6.5.1_73_SLC-8.1M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024947 = {S28659, 145, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; XT910 Build/6.5.1_73_SLC-8.2M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025468 = {S28660, 149, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT910 Build/6.5.1-167_SLC-M34_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025780 = {S28661, 152, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; XT910 Build/6.5.1-167_SPU-15-M2-1-FR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027774 = {S28662, 141, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT910 Build/6.5.1-167-SPD-IRD-40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2030786 = {S28663, 155, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; MOT-XT910 Build/6.5.1-73_SPU-11-M1-2-RT) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031838 = {S28664, 152, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; MOT-XT910 Build/6.5.1-167-SPD-IRD-37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032162 = {S28665, 149, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; MOT-XT910 Build/6.5.1-167_SLC-M33) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032572 = {S28666, 136, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT910 Build/6.5.1_GC-136-SPDU-13) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2033810 = {S28667, 135, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; XT910 Build/6.5.1_73-SPD-IRD-21) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2364807 = {S28668, 140, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; XT910 Build/6.5.1_73-spd-ird-12) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2684927 = {S28669, 149, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT910 Build/6.5.1-167_SLC-M36_R02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2761486 = {S28670, 153, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MOT-XT910 Build/6.5.1-167_SLC-M36_R03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2980312 = {S28671, 148, { &P15877,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MOT-XT910 Build/6.5.1-167_SPU-16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48748 = {S28672, 137, { &P12803,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-I405 4G Build/EGB) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48749 = {S28673, 137, { &P12803,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I405 4G Build/EH2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48750 = {S28674, 137, { &P12803,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-I405 4G Build/EH2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73012 = {S28675, 137, { &P12803,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I405 4G Build/EI2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA77827 = {S28676, 142, { &P12803,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I405 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA576289 = {S28677, 66, { &P12803,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SCH-I405 Build/GINGERBREAD)*/
const Device UA1918252 = {S28678, 149, { &P12803,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I405 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA1958293 = {S28679, 142, { &P12803,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I405 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1960843 = {S28680, 137, { &P12803,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I405 4G Build/FC1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1987276 = {S28681, 154, { &P12803,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I405U Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 USCC-I405U*/
const Device UA1995553 = {S28682, 137, { &P12803,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I405 4G Build/FF1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023996 = {S28683, 34, { &P12803,&P5629,&P15760,&P18092 }};	/*Useragent - Android/1.0 (SCH-I405 GINGERBREAD)*/
const Device UA2031132 = {S28684, 66, { &P12803,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-I405 Build/GINGERBREAD)*/
const Device UA2523025 = {S28685, 67, { &P12803,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-I405U Build/GINGERBREAD)*/
const Device UA2652966 = {S28686, 149, { &P12803,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I405 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2712697 = {S28687, 137, { &P12803,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I405 4G Build/GC1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2865625 = {S28688, 135, { &P12803,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SCH-I405 4G Build/GC1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2035159 = {S28689, 142, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; SonyST23 Build/2.3.J.3.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2035164 = {S28690, 142, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyST23 Build/2.3.J.2.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2035173 = {S28691, 144, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; SonyST23i Build/11.0.A.0.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2035185 = {S28692, 144, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyST23i Build/11.0.A.1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2035200 = {S28693, 145, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-mx; SonyST23a Build/11.0.A.2.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2035238 = {S28694, 148, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyST23i Build/2.3.X.2.8010.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2035214 = {S28695, 146, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyST23i Build/2.3.J.2.10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2203371 = {S28696, 145, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST23i Build/11.0.A.2.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2229090 = {S28697, 141, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST23i Build/11.0.A.2.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2425176 = {S28698, 141, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST23i Build/11.0.A.2.16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426063 = {S28699, 145, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST23i Build/11.0.A.2.19) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426432 = {S28700, 62, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST23i Build/11.0.A.5.5)*/
const Device UA2428093 = {S28701, 144, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST23i Build/11.0.A.5.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2428140 = {S28702, 145, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST23i Build/11.0.A.2.16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2432100 = {S28703, 145, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-mx; SonyST23a Build/11.0.A.2.16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455684 = {S28704, 146, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-be; SonyST23iv Build/11.0.A.2.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2458154 = {S28705, 141, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST23i Build/11.0.A.2.19) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2459261 = {S28706, 140, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST23i Build/11.0.A.5.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2484755 = {S28707, 144, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyST23a Build/11.0.A.5.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2487948 = {S28708, 144, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST23i Build/11.0.A.5.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2509137 = {S28709, 140, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST23i Build/11.0.A.5.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528216 = {S28710, 141, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-mx; ST23a Build/11.0.A.2.16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580244 = {S28711, 145, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-be; SonyST23iv Build/11.0.A.5.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2653937 = {S28712, 141, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-mx; ST23a Build/11.0.A.2.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2698701 = {S28713, 140, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST23a Build/11.0.A.5.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2728993 = {S28714, 63, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST23a Build/11.0.A.2.16)*/
const Device UA2781778 = {S28715, 144, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-mx; SonyST23a Build/11.0.A.5.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2841404 = {S28716, 65, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - SonyST23i Build/11.0.A.2.10 stagefright/1.2 (Linux;Android 4.0.4)*/
const Device UA2846227 = {S28717, 64, { &P18450,&P18066,&P18062,&P18092 }};	/*Useragent - SonyST23i Build/11.0.A.5.5 stagefright/1.2 (Linux;Android 4.0.4)*/
const Device UA93069 = {S28718, 114, { &P15364,&P5690,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9.1*/
const Device UA99070 = {S28719, 119, { &P15364,&P5690,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA99982 = {S28720, 112, { &P15364,&P5690,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Safari/528.17*/
const Device UA100101 = {S28721, 112, { &P15364,&P5690,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA125166 = {S28722, 118, { &P15364,&P5690,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA132212 = {S28723, 112, { &P15364,&P5690,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/528.16 (KHTML, like Gecko) Version/4.0 Safari/528.16*/
const Device UA149797 = {S28724, 110, { &P15364,&P5690,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en) AppleWebKit/526.9 (KHTML, like Gecko) Version/4.0dp1 Safari/526.8*/
const Device UA165161 = {S28725, 118, { &P15364,&P5690,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19.1*/
const Device UA382474 = {S28726, 118, { &P15364,&P5690,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA659326 = {S28727, 100, { &P15364,&P5690,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA671516 = {S28728, 99, { &P15364,&P5690,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA730043 = {S28729, 118, { &P15364,&P5690,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; zh-CN) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19.1*/
const Device UA791698 = {S28730, 114, { &P15364,&P5690,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9.1*/
const Device UA872517 = {S28731, 112, { &P15364,&P5690,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA2207371 = {S28732, 117, { &P15364,&P5690,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/526.11 (KHTML, like Gecko) Version/4.0dp1 Safari/526.12.2*/
const Device UA2409113 = {S28733, 132, { &P15364,&P5690,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; BOLT/2.340) AppleWebKit/530+ (KHTML, like Gecko) Version/4.0 Safari/530.17  3gpp-gba*/
const Device UA3042319 = {S28734, 118, { &P15364,&P5690,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/531.21.8 (KHTML like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA900928 = {S28735, 99, { &P15364,&P16110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA901572 = {S28736, 106, { &P15364,&P16110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA2169146 = {S28737, 104, { &P15364,&P16110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/19.77.34.5 Safari/537.1*/
const Device UA2360573 = {S28738, 125, { &P15364,&P16110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5*/
const Device UA2363182 = {S28739, 132, { &P15364,&P16110,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5*/
const Device UA2888356 = {S28740, 58, { &P15364,&P5690,&P22171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:23.0) Gecko/X Firefox/23.0*/
const Device UA2897123 = {S28741, 65, { &P15364,&P5690,&P22171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64; rv:23.0) Gecko/X Firefox/23.0*/
const Device UA90792 = {S28742, 101, { &P15775,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 800)*/
const Device UA90791 = {S28743, 109, { &P15775,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 800; Orange)*/
const Device UA399133 = {S28744, 115, { &P15775,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 800; 800 BOUYGUES)*/
const Device UA399151 = {S28745, 111, { &P15775,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 800; Vodafone)*/
const Device UA399152 = {S28746, 106, { &P15775,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 800; SFR)*/
const Device UA399163 = {S28747, 111, { &P15775,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 800; T-Mobile)*/
const Device UA661750 = {S28748, 102, { &P15775,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Nokia 800C)*/
const Device UA681407 = {S28749, 109, { &P15775,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 800; Meteor)*/
const Device UA761163 = {S28750, 101, { &P15775,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Nokia 800)*/
const Device UA925778 = {S28751, 110, { &P15775,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 800; Optimus)*/
const Device UA925802 = {S28752, 115, { &P15775,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 610; 800 BOUYGUES)*/
const Device UA925854 = {S28753, 96, { &P15775,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Nokia; 800C)*/
const Device UA2812109 = {S28754, 111, { &P15775,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 800),gzip(gfe)*/
const Device UA2981938 = {S28755, 109, { &P15775,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA_blocked; Lumia 800)*/
const Device UA3085032 = {S28756, 140, { &P15775,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 800),gzip(gfe) (via docs.google.com/viewer)*/
const Device UA79932 = {S28757, 82, { &P15364,&P17285,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/11*/
const Device UA174924 = {S28758, 118, { &P15364,&P17285,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.20 (KHTML, like Gecko) Chrome/11.0.672.2 Safari/534.20*/
const Device UA181119 = {S28759, 108, { &P15364,&P17285,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.0) AppleWebKit/534.23 (KHTML, like Gecko) Chrome/11.0.686.3 Safari/534.23*/
const Device UA207039 = {S28760, 100, { &P15364,&P17285,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24*/
const Device UA311465 = {S28761, 107, { &P15364,&P17285,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.16 Safari/534.24*/
const Device UA315052 = {S28762, 137, { &P15364,&P17285,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.65 Safari/534.24,gzip(gfe) (via translate.google.com)*/
const Device UA2333127 = {S28763, 100, { &P15364,&P17285,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.24 (KHTML	 like Gecko) Chrome/11.0.696.65 Safari/534.24*/
const Device UA2338098 = {S28764, 107, { &P15364,&P17285,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.24 (KHTML	 like Gecko) Chrome/11.0.696.71 Safari/534.24*/
const Device UA2933592 = {S28765, 117, { &P15364,&P17285,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.20 (KHTML like Gecko) Chrome/11.0.672.2 Safari/534.20*/
const Device UA2980383 = {S28766, 98, { &P15364,&P17285,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.24 (KHTML like Gecko) Chrome/11.0.696.3 Safari/534.24*/
const Device UA3042316 = {S28767, 106, { &P15364,&P17285,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.24 (KHTML like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA49620 = {S28768, 150, { &P13064,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SAMSUNG-SGH-I727 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49621 = {S28769, 150, { &P13064,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SAMSUNG-SGH-I727 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622899 = {S28770, 74, { &P13064,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SAMSUNG-SGH-I727 Build/GINGERBREAD)*/
const Device UA622978 = {S28771, 150, { &P13064,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I727 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626472 = {S28772, 74, { &P13064,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SAMSUNG-SGH-I727 Build/GINGERBREAD)*/
const Device UA627480 = {S28773, 168, { &P13064,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; SAMSUNG-SGH-I727 Build/Sky Rocket Ice Cream Sandwich) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA630517 = {S28774, 148, { &P13064,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SAMSUNG-SGH-I727 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA725595 = {S28775, 150, { &P13064,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; SAMSUNG-SGH-I727 Build/DJ ICS Plus) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA781507 = {S28776, 148, { &P13064,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; SAMSUNG-SGH-I727 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021399 = {S28777, 142, { &P13064,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; SGH-I727 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025417 = {S28778, 34, { &P13064,&P5629,&P15760,&P18092 }};	/*Useragent - Android/1.0 (SGH-I727 GINGERBREAD)*/
const Device UA3109448 = {S28779, 178, { &P13064,&P5629,&P15760,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 2.3.6;dev=SGH-I727;devm=SAMSUNG-SGH-I727;man=samsung;brw=Android;shwd=480x800;shwa=480x763;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA459178 = {S28780, 64, { &P17835,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; Nook Tablet Build/GRK39F)*/
const Device UA459172 = {S28781, 69, { &P17835,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; Nook Tablet Build/GINGERBREAD)*/
const Device UA622584 = {S28782, 146, { &P17835,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK BNTV250 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622889 = {S28783, 134, { &P17835,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; BNTV250 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA623734 = {S28784, 147, { &P17835,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK BNTV250A Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625257 = {S28785, 135, { &P17835,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; BNTV250A Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA700496 = {S28786, 65, { &P17835,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; BNTV250 Build/GINGERBREAD)*/
const Device UA902967 = {S28787, 66, { &P17835,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; BNTV250A Build/GINGERBREAD)*/
const Device UA909468 = {S28788, 146, { &P17835,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK BNTV250A Build/GINGERBREAD 1.4.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA910595 = {S28789, 145, { &P17835,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK BNTV250 Build/GINGERBREAD 1.4.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA910705 = {S28790, 153, { &P17835,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK BNTV250A Build/GINGERBREAD 1.4.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA911610 = {S28791, 152, { &P17835,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK BNTV250 Build/GINGERBREAD 1.4.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA609167 = {S28792, 120, { &P17595,&P17296,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.163 Safari/535.19*/
const Device UA614905 = {S28793, 120, { &P17595,&P17296,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA614911 = {S28794, 120, { &P17595,&P17296,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA614915 = {S28795, 120, { &P17595,&P17296,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA614926 = {S28796, 120, { &P17595,&P17296,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA614924 = {S28797, 120, { &P17595,&P17296,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA614952 = {S28798, 120, { &P17595,&P17296,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA614991 = {S28799, 120, { &P17595,&P17296,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA615007 = {S28800, 120, { &P17595,&P17296,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA830712 = {S28801, 120, { &P17595,&P17296,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/180.1.1025.40 Safari/535.19*/
const Device UA2596294 = {S28802, 117, { &P17595,&P17296,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1060) AppleWebKit/535.19 (KHTML like Gecko) Chrome/18.0.1025.142 Safari/535.19*/
const Device UA2674641 = {S28803, 129, { &P17595,&P17296,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/18.0.648.205 Safari/534.16*/
const Device UA2674851 = {S28804, 129, { &P17595,&P17296,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/18.0.648.205 Safari/534.16*/
const Device UA2714920 = {S28805, 129, { &P17595,&P17296,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/18.0.648.205 Safari/534.16*/
const Device UA3031566 = {S28806, 125, { &P17595,&P17296,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/534.1 (KHTML, like Gecko) Chrome/18.0.427.0 Safari/534.1*/
const Device UA33367 = {S28807, 129, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA200904 = {S28808, 129, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; SGH-T859 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA316369 = {S28809, 133, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7510 Build/HONEYCOMB) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA323666 = {S28810, 58, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; GT-P7510 Build/HMJ37)*/
const Device UA369342 = {S28811, 70, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 3.1; en_US; GT-P7510 Build/HMJ37.UEKME)*/
const Device UA659431 = {S28812, 136, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2021614 = {S28813, 150, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7510 Build/HMJ37; 1280*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2462997 = {S28814, 127, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2607626 = {S28815, 141, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7510 Build/HMJ37) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2607778 = {S28816, 70, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Logos/0.9.3 (Linux; U; Android 3.1; en_US; GT-P7510 Build/HMJ37.UEKMB)*/
const Device UA2607791 = {S28817, 52, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en_US; GT-P7510)*/
const Device UA2608261 = {S28818, 127, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7510 Build/HMJ37) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2608737 = {S28819, 70, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 3.1; en_US; GT-P7510 Build/HMJ37.ZSKI1)*/
const Device UA2609607 = {S28820, 129, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7510 Build/HMJ29) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2610013 = {S28821, 231, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7510 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 sessionId/X-e3a9-4022-aca6-7c84cc7392f7 tiantian(securitypay,securityinstalled) tiantian(tenpay)*/
const Device UA2611221 = {S28822, 70, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Logos/1.1.3 (Linux; U; Android 3.1; es-US; GT-P7510 Build/HMJ37.UBKH1)*/
const Device UA2615192 = {S28823, 268, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.10;FBDM/{density=1.0,width=1280,height=800};FBLC/en_US;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P7510;FBSV/3.1;]*/
const Device UA2615360 = {S28824, 114, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Android 3.1; en-us; GT-P7510 Build/HMJ37) FreeWheelAdManager/5.0.3-r9054-X;com.gotv.crackle.handset/X*/
const Device UA2615589 = {S28825, 67, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - BBC iPlayer (AndroidApp; 1.3; 13014) GT-P7510 (Android 3.1, SDK 12)*/
const Device UA2615911 = {S28826, 267, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; es-es; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.8.4;FBDM/{density=1.0,width=1280,height=800};FBLC/es_ES;FB_FW/1;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P7510;FBSV/3.1;]*/
const Device UA2615921 = {S28827, 70, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 3.1; en-GB; GT-P7510 Build/HMJ37.XXKH3)*/
const Device UA2616469 = {S28828, 90, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 2.2.1); (Linux; U; Android 3.1; SAMSUNG GT-P7510 Build/HONEYCOMB_MR1);*/
const Device UA2616806 = {S28829, 90, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - com.google.android.youtube/4.1.23(Linux; U; Android 3.1; en_US; GT-P7510 Build/HMJ37) gzip*/
const Device UA2617171 = {S28830, 71, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Biblia/2.1.2 (Linux; U; Android 3.1; es-ES; GT-P7510 Build/HMJ37.UBKG6)*/
const Device UA2619838 = {S28831, 70, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 3.1; en-US; GT-P7510 Build/HMJ37.UEKF3)*/
const Device UA2620159 = {S28832, 197, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 fb4a androidAppVersion/2.5.16 AndroidWebkitWrapper BillingSupported*/
const Device UA2621923 = {S28833, 60, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.1; 9) GT-P7510 (Android 3.1, SDK 12)*/
const Device UA2626131 = {S28834, 126, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Groupon/2.1.2486 (Android 3.1; Samsung GT-P7510 / Samsung GT-P7510; )[preload=false;locale=en_US;clientidbase=android-samsung]*/
const Device UA2629417 = {S28835, 76, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - MediaPlayer/2002/4.2.0 (Android 3.1; GT-P7510; GT-P7510; samsung; ; ; fr_FR)*/
const Device UA2630578 = {S28836, 128, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 3.1; fr-fr; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2631683 = {S28837, 157, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-; GT-P7510 Build/HMJ37) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2632460 = {S28838, 60, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.0; 5) GT-P7510 (Android 3.1, SDK 12)*/
const Device UA2636335 = {S28839, 61, { &P12879,&P6546,&P4459,&P18092 }};	/*Useragent - Gospel_Library 2.4.2.7 / Android 3.1 XXKG6 / samsung GT-P7510*/
const Device UA29929 = {S28840, 140, { &P17346,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16*/
const Device UA89009 = {S28841, 141, { &P17346,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7C145 Safari/528.16*/
const Device UA89810 = {S28842, 140, { &P17346,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7D11 Safari/528.16*/
const Device UA93500 = {S28843, 139, { &P17346,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7C144 Safari/528.16*/
const Device UA94072 = {S28844, 141, { &P17346,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7C146 Safari/528.16*/
const Device UA147391 = {S28845, 142, { &P17346,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU OS 3_1_3 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA396888 = {S28846, 150, { &P17346,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (whited00r iPod; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16*/
const Device UA1937948 = {S28847, 150, { &P17346,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (whited00r iPod; U; CPU iPhone OS 3_1_3 like Mac OS X; es-es) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16*/
const Device UA2216017 = {S28848, 150, { &P17346,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (whited00r iPod; U; CPU iPhone OS 3_1_3 like Mac OS X; en-gb) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16*/
const Device UA2596211 = {S28849, 137, { &P17346,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 312 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML like Gecko) Version/4.0 Mobile/7D11 Safari/528.16*/
const Device UA2596538 = {S28850, 139, { &P17346,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3 1 2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML like Gecko) Version/4.0 Mobile/7D11 Safari/528.16*/
const Device UA2658890 = {S28851, 138, { &P17346,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 311 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML like Gecko) Version/4.0 Mobile/7C145 Safari/528.16*/
const Device UA2711888 = {S28852, 138, { &P17346,&P17137,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 313 like Mac OS X; haw-US) AppleWebKit/528.18 (KHTML like Gecko) Version/4.0 Mobile/7E18 Safari/528.16*/
const Device UA607444 = {S28853, 132, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P3113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA754286 = {S28854, 61, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-P3113 Build/IML74K)*/
const Device UA912626 = {S28855, 132, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1991360 = {S28856, 139, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1991439 = {S28857, 61, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P3113 Build/IMM76D)*/
const Device UA2182255 = {S28858, 130, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2411430 = {S28859, 139, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P3113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2420481 = {S28860, 130, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-P3113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2608295 = {S28861, 79, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/1.1.2 (Linux; U; Android 4.0.3; en-US; GT-P3113 Build/IML74K.P3113UEALD3)*/
const Device UA2608955 = {S28862, 120, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/1.1.2 (Linux; U; Android 4.0.4; en-US; GT-P3113 Build/cm_p3113-userdebug 4.0.4 IMM76L eng.codeworkx.X.X test-keys)*/
const Device UA2610394 = {S28863, 125, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/1.1.2 (Linux; U; Android 4.0.4; en-US; GT-P3113 Build/cm_espressowifi-userdebug 4.0.4 IMM76I eng.xoomdev.X.X test-keys)*/
const Device UA2610512 = {S28864, 92, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.youtube/3.3.0(Linux; U; Android 4.0.3; en_US; GT-P3113 Build/IML74K) gzip*/
const Device UA2610518 = {S28865, 132, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2610692 = {S28866, 139, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2611775 = {S28867, 117, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Android 4.0.3; en-us; GT-P3113 Build/IML74K) FreeWheelAdManager/4.7.0-r8234-X;com.gotv.crackle.handset/X*/
const Device UA2612911 = {S28868, 125, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3113 Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2614715 = {S28869, 93, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.youtube/4.0.23(Linux; U; Android 4.0.4; en_US; GT-P3113 Build/IMM76D) gzip*/
const Device UA2615693 = {S28870, 117, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Android 4.0.4; en-us; GT-P3113 Build/IMM76D) FreeWheelAdManager/5.0.3-r9054-X;com.gotv.crackle.handset/X*/
const Device UA2615729 = {S28871, 145, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA2616092 = {S28872, 163, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/5.0.2 Version/4.0 Safari/534.30 Safari/6533.18.5*/
const Device UA2616704 = {S28873, 130, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3113 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2617538 = {S28874, 79, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.4; en-US; GT-P3113 Build/IMM76D.P3113UEBLG2)*/
const Device UA2618747 = {S28875, 271, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P3113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/1.9.6;FBDM/{density=1.0,width=600,height=976};FBLC/en_US;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P3113;FBSV/4.0.3;]*/
const Device UA2618831 = {S28876, 111, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.4; en-US; GT-P3113 Build/cm_p3113-userdebug 4.0.4 IMM76L eng..X.X test-keys)*/
const Device UA2618877 = {S28877, 132, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Groupon/2.1.2483 (Android 4.0.4; Samsung Espressowifi / Samsung GT-P3113; )[preload=false;locale=en_US;clientidbase=android-samsung]*/
const Device UA2619003 = {S28878, 145, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; GT-P3113 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.7 Mobile Safari/534.30*/
const Device UA2619325 = {S28879, 62, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.3; 4) GT-P3113 (Android 4.0.4, SDK 15)*/
const Device UA2620354 = {S28880, 79, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.3; en-US; GT-P3113 Build/IML74K.P3113UEALCB)*/
const Device UA2620372 = {S28881, 94, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.apps.plus/X (Linux; U; Android 4.0.4; en_US; GT-P3113; Build/IMM76D) (gzip)*/
const Device UA2620478 = {S28882, 45, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - rad.io/1.1.9 (Android 4.0.4) samsung/GT-P3113*/
const Device UA2621388 = {S28883, 192, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/4.0 (Compatible, Aggregation, Twonky, 7.1-beta-X-Android ) TwonkyMedia-NMC DLNADOC/1.50 mobileview TwonkyBeamBrowser/3.3.1-89 (Android 4.0.4; samsung GT-P3113 Build/IMM76D.P3113UEBLH2)*/
const Device UA2621725 = {S28884, 137, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76D) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2622146 = {S28885, 101, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 2.2.1); (Linux; U; Android 4.0.4; SAMSUNG GT-P3113 Build/ICE_CREAM_SANDWICH_MR1);*/
const Device UA2622367 = {S28886, 144, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2622406 = {S28887, 139, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2622418 = {S28888, 120, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-US; GT-P3113 Build/cm_p3113-userdebug 4.0.4 IMM76L eng.jenkins.X.X test-keys)*/
const Device UA2622498 = {S28889, 56, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Android/4.6.1 (Linux; U; Android 4.0.4; en_US; GT-P3113)*/
const Device UA2622653 = {S28890, 85, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - nApps (Android 4.0.4; GT-P3113; %EB%84%A4%EC%9D%B4%EB%B2%84%EC%9B%B9%ED%88%B0; 1.2.7)*/
const Device UA2623176 = {S28891, 81, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-US; GT-P3113 Build/IMM76D.P3113UEBLH2)*/
const Device UA2623977 = {S28892, 84, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik 2.02.11978; (Linux; U; Android 4.0.4; GT-P3113) f3ba87a5eae70a74; 600x976x1.0*/
const Device UA2624266 = {S28893, 132, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Groupon/2.2.2561 (Android 4.0.3; Samsung Espressowifi / Samsung GT-P3113; )[preload=false;locale=en_US;clientidbase=android-samsung]*/
const Device UA2624863 = {S28894, 144, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; az-az; GT-P3113 Build/IML74K) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2625353 = {S28895, 75, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - GroupMe-Android/3.3.9 (Android 4.0.3; en_US; samsung GT-P3113 Build/IML74K)*/
const Device UA2626481 = {S28896, 94, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.2.10 (Linux; U; Android 4.0.4; en-US; GT-P3113 Build/IMM76D.P3113UEBLG5)*/
const Device UA2627223 = {S28897, 131, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P3113Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2627848 = {S28898, 195, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 androidAppVersion/2.5.17 AndroidWebkitWrapper BillingSupported*/
const Device UA2629500 = {S28899, 132, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2629959 = {S28900, 143, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en_CA; GT-P3113 BUILD/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2631007 = {S28901, 132, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-ca; GT-P3113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2631023 = {S28902, 145, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P3113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2635242 = {S28903, 68, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Archie Comics/1.04 (Android 4.0.4/GT-P3113 [samsung espressowifiue])*/
const Device UA2637256 = {S28904, 124, { &P22037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P3113) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA94027 = {S28905, 51, { &P15364,&P7127,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0)*/
const Device UA112254 = {S28906, 50, { &P15364,&P7127,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 5.0)*/
const Device UA153192 = {S28907, 71, { &P15364,&P7127,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; NetCaptor 6.5.0RC1)*/
const Device UA271074 = {S28908, 65, { &P15364,&P7127,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 5.0; Alexa Toolbar)*/
const Device UA470982 = {S28909, 51, { &P15364,&P7127,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.01; Windows NT 5.0)*/
const Device UA772306 = {S28910, 59, { &P15364,&P7127,&P4551,&P18092 }};	/*Useragent - Mozilla/3.0 (compatible;MSIE 5.0;  Windows NT 5.0; 360safe)*/
const Device UA2136802 = {S28911, 56, { &P15364,&P7127,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; SV1)*/
const Device UA2137941 = {S28912, 51, { &P15364,&P7127,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.05; Windows NT 5.0)*/
const Device UA2144200 = {S28913, 59, { &P15364,&P7127,&P4551,&P18092 }};	/*Useragent - Mozilla/6.0 (compatible;MSIE 5.0;  Windows NT 5.0; 360safe)*/
const Device UA2267601 = {S28914, 49, { &P15364,&P7127,&P4551,&P18092 }};	/*Useragent - EmailsGet (compatible; MSIE 5.01; Windows NT 5.0)*/
const Device UA2524051 = {S28915, 63, { &P15364,&P7127,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; Windows NT 5.0; Trident/4.0)*/
const Device UA225831 = {S28916, 142, { &P14433,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA345828 = {S28917, 142, { &P14433,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369256 = {S28918, 143, { &P14433,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-I9003L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA564046 = {S28919, 66, { &P14433,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-I9003 Build/GINGERBREAD)*/
const Device UA630476 = {S28920, 66, { &P14433,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9003 Build/GINGERBREAD)*/
const Device UA633969 = {S28921, 142, { &P14433,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA717818 = {S28922, 66, { &P14433,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-I9003 Build/GINGERBREAD)*/
const Device UA773533 = {S28923, 130, { &P14433,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9003 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA902903 = {S28924, 130, { &P14433,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9003 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2025362 = {S28925, 135, { &P14433,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9003 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2033723 = {S28926, 136, { &P14433,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-I9003 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033868 = {S28927, 135, { &P14433,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9003 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2256976 = {S28928, 140, { &P14433,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2409654 = {S28929, 140, { &P14433,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2806711 = {S28930, 143, { &P14433,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; GT-I9003L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA410843 = {S28931, 128, { &P16598,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA639536 = {S28932, 137, { &P16598,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Acer Iconia A500 Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA651000 = {S28933, 57, { &P16598,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; A500 Build/IML74K)*/
const Device UA663296 = {S28934, 135, { &P16598,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Iconia A500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1956969 = {S28935, 135, { &P16598,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1979781 = {S28936, 128, { &P16598,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2031596 = {S28937, 127, { &P16598,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; A500 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033733 = {S28938, 128, { &P16598,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A500 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2395510 = {S28939, 126, { &P16598,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; A500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2530774 = {S28940, 135, { &P16598,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A500 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556018 = {S28941, 144, { &P16598,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A500 Build/ICS_IMM76I_B230) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987254 = {S28942, 57, { &P16598,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; A500 Build/IMM76I)*/
const Device UA251635 = {S28943, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.9.168 Version/11.52*/
const Device UA251642 = {S28944, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.9.168 Version/11.52*/
const Device UA251650 = {S28945, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.9.168 Version/11.52*/
const Device UA251639 = {S28946, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.9.168 Version/11.52*/
const Device UA251751 = {S28947, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.9.168 Version/11.52*/
const Device UA251790 = {S28948, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.9.168 Version/11.52*/
const Device UA251801 = {S28949, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.9.168 Version/11.52*/
const Device UA251805 = {S28950, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.9.168 Version/11.52*/
const Device UA251834 = {S28951, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.9.168 Version/11.52*/
const Device UA251836 = {S28952, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.9.168 Version/11.52*/
const Device UA251842 = {S28953, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4871); ru) Presto/2.9.168 Version/11.52*/
const Device UA251860 = {S28954, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4832); ru) Presto/2.9.168 Version/11.52*/
const Device UA251874 = {S28955, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.9.168 Version/11.52*/
const Device UA251880 = {S28956, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4848); ru) Presto/2.9.168 Version/11.52*/
const Device UA251890 = {S28957, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.9.168 Version/11.52*/
const Device UA251897 = {S28958, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.9.168 Version/11.52*/
const Device UA252000 = {S28959, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.9.168 Version/11.52*/
const Device UA344956 = {S28960, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4841); ru) Presto/2.9.168 Version/11.52*/
const Device UA345838 = {S28961, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4110); ru) Presto/2.9.168 Version/11.52*/
const Device UA421920 = {S28962, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.9.168 Version/11.52*/
const Device UA437105 = {S28963, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.9.168 Version/11.52*/
const Device UA451330 = {S28964, 123, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; Edition Ukraine Local; ru) Presto/2.9.168 Version/11.52,gzip(gfe) (via translate.google.com)*/
const Device UA574808 = {S28965, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03278); ru) Presto/2.9.168 Version/11.52*/
const Device UA591703 = {S28966, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.9.168 Version/11.52*/
const Device UA603066 = {S28967, 63, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; en) Presto/2.8.131 Version/11.52*/
const Device UA649869 = {S28968, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.9.168 Version/11.52*/
const Device UA703514 = {S28969, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4930); ru) Presto/2.9.168 Version/11.52*/
const Device UA731077 = {S28970, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.9.168 Version/11.52*/
const Device UA743411 = {S28971, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4933); ru) Presto/2.9.168 Version/11.52*/
const Device UA771905 = {S28972, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.9.168 Version/11.52*/
const Device UA783652 = {S28973, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03639); ru) Presto/2.9.168 Version/11.52*/
const Device UA849386 = {S28974, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5265); ru) Presto/2.9.168 Version/11.52*/
const Device UA854017 = {S28975, 72, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; ru; rv:2.0) Gecko/X Firefox/4.0 Opera 11.52*/
const Device UA882744 = {S28976, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03755); ru) Presto/2.9.168 Version/11.52*/
const Device UA894446 = {S28977, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.9.168 Version/11.52*/
const Device UA1902975 = {S28978, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4661); ru) Presto/2.9.168 Version/11.52*/
const Device UA1903046 = {S28979, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03402); ru) Presto/2.9.168 Version/11.52*/
const Device UA1903839 = {S28980, 66, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; ru) Opera 11.52*/
const Device UA1954905 = {S28981, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5218); ru) Presto/2.9.168 Version/11.52*/
const Device UA1960692 = {S28982, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5196); ru) Presto/2.9.168 Version/11.52*/
const Device UA1977161 = {S28983, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.9.168 Version/11.52*/
const Device UA1981527 = {S28984, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.9.168 Version/11.52*/
const Device UA2155405 = {S28985, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.9.168 Version/11.52*/
const Device UA2168081 = {S28986, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.9.168 Version/11.52*/
const Device UA2190655 = {S28987, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.9.168 Version/11.52*/
const Device UA2226660 = {S28988, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.9.168 Version/11.52*/
const Device UA2299943 = {S28989, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03780); ru) Presto/2.9.168 Version/11.52*/
const Device UA2308824 = {S28990, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4947); ru) Presto/2.9.168 Version/11.52*/
const Device UA2315900 = {S28991, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5328); ru) Presto/2.9.168 Version/11.52*/
const Device UA2332961 = {S28992, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5267); ru) Presto/2.9.168 Version/11.52*/
const Device UA2352409 = {S28993, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.9.168 Version/11.52*/
const Device UA2467249 = {S28994, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.9.168 Version/11.52*/
const Device UA2481015 = {S28995, 73, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.9.168 Version/11.52,gzip(gfe)*/
const Device UA2481332 = {S28996, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.9.168 Version/11.52*/
const Device UA2486461 = {S28997, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.9.168 Version/11.52*/
const Device UA2496386 = {S28998, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.9.168 Version/11.52*/
const Device UA2654224 = {S28999, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.9.168 Version/11.52*/
const Device UA2667283 = {S29000, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.9.168 Version/11.52*/
const Device UA2703486 = {S29001, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6060); ru) Presto/2.9.168 Version/11.52*/
const Device UA2786863 = {S29002, 86, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5308); ru) Presto/2.9.168 Version/11.52*/
const Device UA2826294 = {S29003, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5966); ru) Presto/2.9.168 Version/11.52*/
const Device UA2867044 = {S29004, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6578); ru) Presto/2.9.168 Version/11.52*/
const Device UA2936485 = {S29005, 85, { &P15364,&P7133,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6089); ru) Presto/2.9.168 Version/11.52*/
const Device UA900525 = {S29006, 70, { &P15364,&P7133,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:13.0) Gecko/X Firefox/13.0*/
const Device UA902623 = {S29007, 67, { &P15364,&P7133,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:13.0) Gecko/X Firefox/13.0.1*/
const Device UA905899 = {S29008, 82, { &P15364,&P7133,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:13.0) Gecko/X Firefox/13.0.1 Firefox/13.0.1*/
const Device UA1959794 = {S29009, 69, { &P15364,&P7133,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.1; rv:13.0) Gecko/X Firefox/13.0.1*/
const Device UA2314425 = {S29010, 83, { &P15364,&P7133,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:13.0) Gecko/X Firefox/13.0 Firefox/13.0*/
const Device UA2786705 = {S29011, 81, { &P15364,&P7133,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.1; Win64; x64; rv:13.0) Gecko/X Firefox/13.0.1*/
const Device UA2367465 = {S29012, 127, { &P12280,&P19826,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B5105c Twitter for iPhone*/
const Device UA2405715 = {S29013, 127, { &P12280,&P19826,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B5117b Twitter for iPhone*/
const Device UA2437385 = {S29014, 127, { &P12280,&P19826,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B5126b Twitter for iPhone*/
const Device UA2487363 = {S29015, 125, { &P12280,&P19826,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Twitter for iPhone*/
const Device UA2487694 = {S29016, 125, { &P12280,&P19826,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Twitter for iPhone*/
const Device UA2489078 = {S29017, 125, { &P12280,&P19826,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Twitter for iPhone*/
const Device UA2492473 = {S29018, 125, { &P12280,&P19826,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 Twitter for iPhone*/
const Device UA2516525 = {S29019, 127, { &P12280,&P19826,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145 Twitter for iPhone*/
const Device UA2532515 = {S29020, 127, { &P12280,&P19826,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Twitter for iPhone*/
const Device UA2585303 = {S29021, 127, { &P12280,&P19826,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Twitter for iPhone*/
const Device UA2666225 = {S29022, 127, { &P12280,&P19826,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B318 Twitter for iPhone*/
const Device UA2709272 = {S29023, 127, { &P12280,&P19826,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Twitter for iPhone*/
const Device UA912274 = {S29024, 78, { &P15364,&P7133,&P22578,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; Edition Next; ru) Presto/2.11.310 Version/12.50*/
const Device UA2400968 = {S29025, 132, { &P18702,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N8013 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2469026 = {S29026, 132, { &P18702,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N8013 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2482472 = {S29027, 61, { &P18702,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N8013 Build/JZO54K)*/
const Device UA2560017 = {S29028, 139, { &P18702,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N8013 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2657534 = {S29029, 61, { &P18702,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-N8013 Build/JRO03C)*/
const Device UA2850166 = {S29030, 130, { &P18702,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-N8013 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA10337 = {S29031, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; T- Mobile G2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43291 = {S29032, 159, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; ZTE-C_N606/N606_QQV1.0.0B01; 240*320; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49973 = {S29033, 129, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; pt-pt; V9c Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52097 = {S29034, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB611Build/BGN_1.1.30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57937 = {S29035, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; asadullah Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59867 = {S29036, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-T839Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA75505 = {S29037, 160, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; ZTE-C_N606/N606QQ_OV1.0.0B01; 240*320; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79533 = {S29038, 140, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; th-th; WellcoM-A89 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA161384 = {S29039, 131, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A32 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA167113 = {S29040, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; FIH-F0X Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA167477 = {S29041, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; UPC300-2.2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA168366 = {S29042, 131, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A28 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA178438 = {S29043, 166, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; MoDaCo Custom ROM for the Advent Vega r8 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA227282 = {S29044, 131, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MSM Build/FRX07) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA258097 = {S29045, 159, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; TOSHIBA_FOLIO_AND_A Build/TOSHIBA_FOLIO_AND_A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA299166 = {S29046, 145, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; nonsensikal froyo Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA301176 = {S29047, 141, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Vanilla Tazz Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA301489 = {S29048, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; meizu_m9 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA304523 = {S29049, 148, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; VEGAn-TAB-v1.0.0b5.1.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA305141 = {S29050, 132, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; A7-040 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA312111 = {S29051, 132, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; wwe10 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA316573 = {S29052, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; CSL-MI410 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA324639 = {S29053, 139, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; TC970 (Wi-Fi) Build/s3394) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA324644 = {S29054, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAWEE10 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA331408 = {S29055, 131, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; P041 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA332045 = {S29056, 132, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; cius Build/9.2.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA334402 = {S29057, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; cius Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA351130 = {S29058, 132, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; DroidBuild/FRK76) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA362192 = {S29059, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; NABI-A Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368938 = {S29060, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-es; ADR-2100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370116 = {S29061, 150, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; EBEST W16 Build/BQ3_W16_HW1.1_V004) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370269 = {S29062, 144, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - ua:Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; America Build/W800_V01.05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370603 = {S29063, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; FrankenTAB Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370971 = {S29064, 160, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad_W706+/Coolpad_W706+_V4.60; 320*480) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA388470 = {S29065, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; nl-nl; sec_smdkv210 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA393458 = {S29066, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; IDEOS Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA396089 = {S29067, 151, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; Hikari-iFrame/WDPF-701ME Build/2.0.0E) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA396127 = {S29068, 132, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; pt-br; NB002 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA396945 = {S29069, 140, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; nl-nl; DMEDIA_ATM3470 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA400130 = {S29070, 130, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; CometBuild/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA420311 = {S29071, 141, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; CRICKET DROID-X Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA435044 = {S29072, 149, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Edge ERMINE-0.9; Android 2.2.1; en-us; edge Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA436260 = {S29073, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; de-de; ODYS Xtreme Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA450258 = {S29074, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DroniX Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA451303 = {S29075, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; TPC800W Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA451379 = {S29076, 65, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - QuickPlayer2.0 CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 2.2.2)*/
const Device UA458649 = {S29077, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; MID7020 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458740 = {S29078, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; MID8020 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458921 = {S29079, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MID7008 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA491459 = {S29080, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; UPC300-2.2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA505668 = {S29081, 130, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; X2 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA509344 = {S29082, 140, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; hd-us; generic_x86 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA523354 = {S29083, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; JellyBean Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA525469 = {S29084, 146, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; vi-vn; simcom16_bu1_a10y Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA536251 = {S29085, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; tr-tr; EASYPAD Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA536304 = {S29086, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; apanda_A60 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA539757 = {S29087, 130, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MD70 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA542117 = {S29088, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; maple Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA543740 = {S29089, 131, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Mi320 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA547860 = {S29090, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; CSL-MI410 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA549520 = {S29091, 148, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2-update1; en-us; Next CEWS10 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA550313 = {S29092, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; pt-br; FS-M74TG Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA551894 = {S29093, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; pl-pl; NaviTablet Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA552759 = {S29094, 132, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; it-it; Andy Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA555553 = {S29095, 149, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; lepad_001n Build/PQXU100.4.0073.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA557790 = {S29096, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; MID-71 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA558093 = {S29097, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MID-2 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA561533 = {S29098, 154, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; Android-for-Rockchip-2818 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA562067 = {S29099, 142, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; NEXT NXTTAB10BL Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA562219 = {S29100, 140, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; BEBOOK070I00 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA565684 = {S29101, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ru-ru; Beeline M2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA566294 = {S29102, 150, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; id-id; NX-A891 Build/FRG83G)         AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA570966 = {S29103, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; md-us; H900 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA571024 = {S29104, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; NX-A891 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA573296 = {S29105, 140, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; nl-nl; DMEDIA ATM3470 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA575107 = {S29106, 132, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; lynx Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA632119 = {S29107, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SP01 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA632416 = {S29108, 132, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-; SF4Y6 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA638088 = {S29109, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; P0106RG Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA639542 = {S29110, 131, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; M799 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA641053 = {S29111, 154, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; Lenovo 3GC101/S-5-02;320*533;CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA648924 = {S29112, 141, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ASP320Q_ANDi Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA651048 = {S29113, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; tablet_p2 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA656076 = {S29114, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; in-id; NX-A712 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA661917 = {S29115, 139, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; one touch 906 Build/FSR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA669189 = {S29116, 146, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; VEGAn-TAB-v1.0.0B5.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA670832 = {S29117, 129, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; i7 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA675039 = {S29118, 123, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; C8500S Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA675045 = {S29119, 142, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; FMT-NT7A22-01 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA675431 = {S29120, 141, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; A7-004 Build/QAJ3000.3.0018) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA677703 = {S29121, 131, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; vm Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA678265 = {S29122, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; xam5001 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA678504 = {S29123, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; CIUS-7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA683706 = {S29124, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; TEL7J, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA686187 = {S29125, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; nl-nl; QWTB9010 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA686871 = {S29126, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; in-id; CROSS-A1 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA688242 = {S29127, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; P4D SIRIUS Build/MID) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA692415 = {S29128, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; NBPC724 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA695724 = {S29129, 113, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; YL-Coolpad_E239/1.01.019.X.E239; 320*480; CTC/2.0) CoolpadWebkit/533.1*/
const Device UA696426 = {S29130, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Livall Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA699141 = {S29131, 140, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; nl-nl; Los Angeles Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA699477 = {S29132, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; YENO100, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA712077 = {S29133, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; OP070 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA712125 = {S29134, 132, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; venus Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA715882 = {S29135, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Mi-A300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA718727 = {S29136, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; AND01 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA719125 = {S29137, 132, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; WM8650 Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA719899 = {S29138, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ko-kr; ILT-MX100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA721087 = {S29139, 143, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; TOSHIBA_AC_AND_AZ Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA723770 = {S29140, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; pt-pt; Slidepad Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA724599 = {S29141, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; MFC150ES, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA725869 = {S29142, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; TO101 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA732141 = {S29143, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; P801W Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA732981 = {S29144, 142, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; GTablet-TnT-Lite Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA734821 = {S29145, 152, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; Hikari-iFrame/WDPF-703TI Build/1.04.00) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA735809 = {S29146, 162, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-us; EF101 Build/Kaissen_I7003G_BOE_OV2_TV7.2_X_WCDMA) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA735861 = {S29147, 141, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MFC150GB_02, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA736758 = {S29148, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; SLIDEPAD Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA736786 = {S29149, 160, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-C_N880S/N880SV1.0.0B03; 480*800; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA739051 = {S29150, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; MID, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA740316 = {S29151, 154, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; Lenovo 3GC101/S-5-03;320*533;CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA740850 = {S29152, 131, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; X2G Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA741227 = {S29153, 131, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; V9e Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA750332 = {S29154, 155, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; in-id; AD350 Build/ATHENA_00.03.68.10.45.H1007) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA754103 = {S29155, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; ZTE V880 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA754514 = {S29156, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; tr-tr; A2000 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA755650 = {S29157, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; IVIO_DE38 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA760183 = {S29158, 160, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Z999 Build/I700T_P7_T13_TSZY_FT_S42_R_0_01_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA763399 = {S29159, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; de-de; NB_10_HD Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA763738 = {S29160, 152, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ZiiLABS ZiiO Shenbi  Build/v2.01.01-FROYO) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile Safari/528.16*/
const Device UA771328 = {S29161, 153, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ZiiLABS ViewBook 730  Build/v1.00.08-FROYO) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile Safari/528.16*/
const Device UA772364 = {S29162, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; ST1001 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA775496 = {S29163, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; SP-60 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA813561 = {S29164, 142, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; Movistar Prime Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA817830 = {S29165, 114, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.1; zh-cn; YL-Coolpad_5855/1.02.001.X.5855; 480*320; CTC/2.0) CoolpadWebkit/533.1*/
const Device UA824725 = {S29166, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; de-de; U0101HA Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA825627 = {S29167, 149, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; lepad_001n Build/PQXU100.4.0091.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA829778 = {S29168, 140, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; nl-nl; TAB-TEGRA-10-1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA849084 = {S29169, 151, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; id-id; SpeedUp S1 Build/SpeedUp_S1_Ver01_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA851298 = {S29170, 152, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; Lenovo Lenovo S1-37AH0 Build/FRG83) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA875224 = {S29171, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; CIUS-7-AT Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA910261 = {S29172, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; KM7J2, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1862725 = {S29173, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; md-us; avlite Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1867060 = {S29174, 140, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; WellcoM-A99 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1870754 = {S29175, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ld-us; AD1000 Build/H6.1-S1.6.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1876827 = {S29176, 113, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; YL-Coolpad_N930/1.01.012.X.N930; 480*800; CTC/2.0) CoolpadWebkit/533.1*/
const Device UA1877274 = {S29177, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; FIH-FM6 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1880774 = {S29178, 140, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; tr-tr; V9-MUL_F680 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1883563 = {S29179, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; A602 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1883843 = {S29180, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SH72x8U Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1889287 = {S29181, 131, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A55 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1893517 = {S29182, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-; sec_smdkv210 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1893632 = {S29183, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; pt-br; FS-M73T Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1893825 = {S29184, 132, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-es; AIRIS Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1894729 = {S29185, 131, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-jp; S31HW Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1898092 = {S29186, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; SP-40 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1898542 = {S29187, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; COSMO DUO Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1899173 = {S29188, 130, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; i7 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1899368 = {S29189, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; SF4Y6 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1903033 = {S29190, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; MoFing_701 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1903543 = {S29191, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; DroniX Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1903811 = {S29192, 131, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; G3 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1905771 = {S29193, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; AM728 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1907223 = {S29194, 132, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; el-gr; Carl Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1924054 = {S29195, 131, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; M718G Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1924176 = {S29196, 132, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; m87vg Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1931208 = {S29197, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; A602 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1931236 = {S29198, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Dk1031 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1932547 = {S29199, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; L601 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1932876 = {S29200, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; I-L1 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1934107 = {S29201, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; s710i Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1934361 = {S29202, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; M778R Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1935064 = {S29203, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; A611 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1935964 = {S29204, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-id; EVO_Tab Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1937188 = {S29205, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; A607 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1940455 = {S29206, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; nl-nl; TR718D, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1941387 = {S29207, 147, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; D000-X-S01 Build/00.20.32-02.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1942497 = {S29208, 162, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; V-T200 Build/I700T_P6_T08_FLOL_FT_S42_R_0_05_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1951540 = {S29209, 149, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; lepad_001n Build/PQXU100.4.0097.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1952075 = {S29210, 150, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; SWTGT-7250 Build/froyo_almond_r245) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1955978 = {S29211, 139, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; md-us; DW-UBT7+.A Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1960170 = {S29212, 143, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MTAG7 Build/V.MTS.MTAG7.B09) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1961186 = {S29213, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; A612 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1961949 = {S29214, 160, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad_W706+/Coolpad_W706+_V4.70; 320*480) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1962261 = {S29215, 152, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; Hikari-iFrame/WDPF-703TI Build/2.02.00) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1965064 = {S29216, 131, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-es; Tairu Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1968269 = {S29217, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; A615 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1976155 = {S29218, 145, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ro-ro; Vancouver_Orange Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1979179 = {S29219, 148, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; 4.3 Internet Tablet Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018082 = {S29220, 141, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Chromatic Dream Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018124 = {S29221, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; GW620 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018240 = {S29222, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; de-de; smartpad Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018250 = {S29223, 149, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; firetrap TwoNights2.2 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018310 = {S29224, 145, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; ME811 Build/SHDWR_X6_2.200.25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018364 = {S29225, 139, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; TnT G-Harmony Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018523 = {S29226, 60, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; imx51_bbg Build/FRF85B)*/
const Device UA2018547 = {S29227, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; tr-tr; I7500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018616 = {S29228, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; TR718D Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018628 = {S29229, 143, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; Geeksphone ZERO Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018702 = {S29230, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Prince-I688 Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018800 = {S29231, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SH8158U Build/S3181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018840 = {S29232, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; Corvus5 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018868 = {S29233, 58, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; SAWEE10 Build/FRF85B)*/
const Device UA2018892 = {S29234, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; it-it; 4Geek Horus Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018951 = {S29235, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Corvus5 SP1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018963 = {S29236, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Android TV Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019177 = {S29237, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-es; MID-10 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019207 = {S29238, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; earlgrey Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019211 = {S29239, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Onida_i9 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019234 = {S29240, 145, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME811 Build/SHDWR_X6_2.200.33) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019284 = {S29241, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pt-pt; u8220 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019302 = {S29242, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; VX780A Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019430 = {S29243, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; TCL A906 Build/FSR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019440 = {S29244, 130, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ms-my; Mi320 Build/FRF91) AppleWebKit/533.1 (KHTML, sperti Gecko) Versi/4.0 Mobil Safari/533.1*/
const Device UA2019445 = {S29245, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Zoompad Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019541 = {S29246, 139, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ro-ro; ALLVIEW_P1 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019597 = {S29247, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; T720 SE, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019696 = {S29248, 131, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; th-th; G8 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019697 = {S29249, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; FIH-F0X Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019721 = {S29250, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; tr-tr; ST070I00 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019768 = {S29251, 53, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; i7 Build/FRF85B)*/
const Device UA2020021 = {S29252, 143, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ru-ru; MTC 950 Build/HuaweiMTC_950) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020039 = {S29253, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; M721 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020055 = {S29254, 143, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; MTC 950 Build/HuaweiMTC_950) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020097 = {S29255, 113, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; YL-Coolpad_9930/1.02.117.X.9930; 960*480; CTC/2.0) CoolpadWebkit/533.1*/
const Device UA2020238 = {S29256, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; LT-NA7F Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020264 = {S29257, 141, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; tr-tr; Polypad C7000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020273 = {S29258, 131, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; 2GO Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020468 = {S29259, 156, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - CoolPad8809_CMCC_TD/1.0 OPhone/2.5 (Linux; Android 2.2) Release/04.28.2011 Browser/WAP2.0 (AppleWebKit/533.1) Mobile Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2020548 = {S29260, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pt-pt; TB-002 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020575 = {S29261, 145, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ru-by; MTC Mini Build/HuaweiMTC_Mini) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020603 = {S29262, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; id-id; MOVI M1 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020763 = {S29263, 154, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; i808 Build/ATHENA_00.03.66.03.07.H1001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020827 = {S29264, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; ME811 Build/SHDWR_X6_2.200.25) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2020841 = {S29265, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; saturn Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020879 = {S29266, 121, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; E239 Build/MASTER) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2020930 = {S29267, 126, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-C N606 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2021065 = {S29268, 132, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-us; agave Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021076 = {S29269, 155, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; in-id; AD350 Build/ATHENA_00.03.68.10.40.H1007) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021094 = {S29270, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; M701, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021414 = {S29271, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; CAP6_US Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021600 = {S29272, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; SN10T2 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021601 = {S29273, 139, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; M31 Build/1.0.4506.0251) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021663 = {S29274, 160, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; TY-K-Touch_E800/X_7950_V0713; 480*800; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021766 = {S29275, 141, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; BL-R78242-EW Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021818 = {S29276, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; 5855 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021865 = {S29277, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; R81P4 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021880 = {S29278, 131, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; A35 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021976 = {S29279, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; VX580A Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022019 = {S29280, 130, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad W706+ Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022031 = {S29281, 171, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ZeusPAD ZP-712 Build/I700T_P7_T11_SPWE_CY_R_0_04_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022135 = {S29282, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; CFITD, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022187 = {S29283, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; TCL A990 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022199 = {S29284, 141, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; A7-004 Build/QAJ3000.3.0017) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022209 = {S29285, 148, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; DNP-725Lucky Build/1.0.5478.0317) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022235 = {S29286, 122, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; meizu_m9 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022256 = {S29287, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; U8S_MID, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022304 = {S29288, 139, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; in-id; F698-DG30M Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022346 = {S29289, 139, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; Entel-N700 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022354 = {S29290, 139, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; in-id; Twilight_7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022450 = {S29291, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; in-id; Mito 9800 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022520 = {S29292, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MF8604 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022554 = {S29293, 141, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; G.fone G1315 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022649 = {S29294, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; TR718DC, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022738 = {S29295, 139, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Ritroid T1 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022781 = {S29296, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; nl-nl; TAB410 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022832 = {S29297, 143, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; id-id; AndroTab Build/1.0.5478.0317)AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022904 = {S29298, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-es; PC7000 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022920 = {S29299, 131, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; Lenovo S1-37AH0 Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022931 = {S29300, 165, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2_v1.5.3; hd-gb; WM8650 Build/FRG83D_v1.5.3 Android Froyo 2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022971 = {S29301, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-es; Qpad Q760 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023015 = {S29302, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; HWA505 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023084 = {S29303, 145, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Galaxy Pad Build/froyo_almond) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023184 = {S29304, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; public Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023213 = {S29305, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; A603 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023293 = {S29306, 153, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - ua:Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; AX_W_RTP Build/AX_W_VER13_P3_RTP_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023353 = {S29307, 122, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; 3GC101 Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2023532 = {S29308, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; MID-71, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023695 = {S29309, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; rk2818 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023912 = {S29310, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME811 Build/SHDWR_X6_2.200.33) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2024132 = {S29311, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SH8268U Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024177 = {S29312, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; LEXIBOOK, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024210 = {S29313, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; MUSN-M500 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024493 = {S29314, 162, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; CTG701 Build/I700T_P7_T04_TSCL_FT_S42_R_0_01_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024579 = {S29315, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MBI i7 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024757 = {S29316, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GN200 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024896 = {S29317, 166, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad_W706/Coolpad_W706_V2.41; 320*480;CTC/2.0) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2025111 = {S29318, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; OIDZ Glee Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025117 = {S29319, 155, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Precision Tablet ERP-A1-38 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025252 = {S29320, 154, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ZiiLABS ZiiO WaWaYaYa  Build/v2.00.12-Froyo) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile Safari/528.16*/
const Device UA2025481 = {S29321, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; MI5 Build/JOYUI-3G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025599 = {S29322, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; M721, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025620 = {S29323, 142, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad W706+ Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025687 = {S29324, 166, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad_W706/Coolpad_W706_V3.10; 320*480;CTC/2.0) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2025882 = {S29325, 128, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; C8500S Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2026624 = {S29326, 147, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; D000-X-S25 Build/00.21.29-01.23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026634 = {S29327, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MI355 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026834 = {S29328, 113, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; YL-Coolpad_9100/1.02.009.X.9100; 800*480; CTC/2.0) CoolpadWebkit/533.1*/
const Device UA2026861 = {S29329, 139, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-ve; C8512 Build/HuaweiC8512) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026983 = {S29330, 125, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; T92 Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2027088 = {S29331, 151, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; QB57 Build/QB57L25.YC1.00.B25.54.03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027279 = {S29332, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; tikx2 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027384 = {S29333, 130, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; M302 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027456 = {S29334, 127, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; 3GC101 Build/FRG83) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2027681 = {S29335, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-es; Vexia 7600 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027773 = {S29336, 150, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Galaxy Pad Build/froyo_almond_r232) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027846 = {S29337, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ko-kr; ESP_E201V7 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027961 = {S29338, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; pt-br; AMOSTRA Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028086 = {S29339, 141, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; A7-004 Build/QAJ3000.3.0022) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028170 = {S29340, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad W706+ Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2028745 = {S29341, 146, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; Wizlet Rise T-308 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029552 = {S29342, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; W606 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029839 = {S29343, 152, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; Hikari-iFrame/WDPF-703TI Build/2.03.00) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029999 = {S29344, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; Fly_IQ250; Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031474 = {S29345, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; NBPC824 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031682 = {S29346, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MT620 Build/THTTD_N_01.01.36I) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2032427 = {S29347, 114, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; YL-Coolpad_5899/2.2.007.X.5899; 480*320; CTC/2.0) CoolpadWebkit/533.2*/
const Device UA2032433 = {S29348, 120, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; T92 Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2032745 = {S29349, 159, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad_W706/Coolpad_W706+_V2.00; 320*480) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2032933 = {S29350, 141, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; pnx6715_e150b Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2033079 = {S29351, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; AX 8.6 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033558 = {S29352, 162, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-us; EF101 Build/Kaissen_I7003G_BOE_OV2_TV7.6_X_WCDMA) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033939 = {S29353, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; NBPC724A Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033968 = {S29354, 132, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-es; m77vg Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034204 = {S29355, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; SM07 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034563 = {S29356, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; TEL7JB, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034635 = {S29357, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HW-HUAWEI_S8520/S8520V100R001C92B011; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2223172 = {S29358, 139, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Fusion Tab Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2256901 = {S29359, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-co; ADR1105 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2256954 = {S29360, 140, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ru-ru; Explay A320 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2256982 = {S29361, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; WM8650-mid Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2262150 = {S29362, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; M4301 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2338967 = {S29363, 140, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; in-id; A850 Build/SDB-P2_1.3.20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2339920 = {S29364, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; NBPC824A Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2409897 = {S29365, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; A35DM Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2410796 = {S29366, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; FFS070 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2411675 = {S29367, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MT620 Build/THTTD_N_01.01.36I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2413619 = {S29368, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; in-id; MG-251 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2419652 = {S29369, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; tr-tr; WM8650-pid Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2439670 = {S29370, 139, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; MFC150DEX, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2442470 = {S29371, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; A2000 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2443432 = {S29372, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Mi280 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2444594 = {S29373, 140, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Z100 Build/SDB-P2_1.3.20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2448658 = {S29374, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Mypad Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2466370 = {S29375, 174, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Coolpad8050_CMCC_TD/1.0 Linux/2.6.35 Android/2.3.5 Release/7.16.2012 Mozilla/5.0 (Linux; U; Android 2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2475667 = {S29376, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; CTG701 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2478499 = {S29377, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; GFIVE A57 Build/S21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2479402 = {S29378, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; IB-Q6 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2494343 = {S29379, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A70H2 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2495626 = {S29380, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2-X; ru-ru; Inch OS Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2502446 = {S29381, 139, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; in-id; ZTE-U V859 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2502482 = {S29382, 131, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; A60 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2505635 = {S29383, 132, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; V871 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2520752 = {S29384, 141, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; NIU TEK N109 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2542503 = {S29385, 132, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-; maple Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2544380 = {S29386, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Lynx Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2545823 = {S29387, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; A605 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2572956 = {S29388, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; vi-vn; LY-F528 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2573376 = {S29389, 144, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Lemon P1 Build/1.0.6008.0254) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2588216 = {S29390, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; Lenovo S1-37AH0 Build/FRG83) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2588276 = {S29391, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; yap01 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2588966 = {S29392, 142, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; BizSmartPhone Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2589467 = {S29393, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; Energy 63 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2589761 = {S29394, 151, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; fa-ir; GLX Mobile Build/A2-V1.0-2012.04.26) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2589930 = {S29395, 131, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; ZP02 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2594779 = {S29396, 147, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; vi-vn; ALCATEL_one_touch_927 Build/FSR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2596364 = {S29397, 146, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; EX 3mini Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2596416 = {S29398, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; EVOTab Build/FRG83G) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2597531 = {S29399, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; EVO Tab Build/FRG83G) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2600649 = {S29400, 159, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Aedupac Xtab Light 100 Build/froyoalmondr238) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2639347 = {S29401, 63, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - QuickPlayer2.0 CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 2.2)*/
const Device UA2639850 = {S29402, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-in; Sansui_A11 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2640033 = {S29403, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; V8502 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2640681 = {S29404, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; T7033 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2640961 = {S29405, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; yezz_Andy Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2642288 = {S29406, 160, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad_W706+/Coolpad_W706+_V5.50; 320*480) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2644259 = {S29407, 132, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; S700 Build/B1326) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2658480 = {S29408, 143, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HelloTab Build/1.0.4992.0138) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2659093 = {S29409, 142, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; AndroTab Build/1.0.4383.0267)AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2684983 = {S29410, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HW-HUAWEI_C8550/C8550V100R001C92B665; 240*320; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2685707 = {S29411, 160, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-C_N880S/N880SV1.0.0B04; 480*800; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2720998 = {S29412, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Magnum 2X Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2721474 = {S29413, 134, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-id; EVO_7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2742471 = {S29414, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ru-ru; Inch_os Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2743235 = {S29415, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; th-th; G.net A6 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2743302 = {S29416, 62, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; TC970 (Wi-Fi) Build/4749)*/
const Device UA2762405 = {S29417, 148, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; epade A300 Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2774998 = {S29418, 141, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad W706 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2774956 = {S29419, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGT-SMA2 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2775342 = {S29420, 132, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; TIK Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2786556 = {S29421, 149, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - ua:Mozilla/5.0 (Linux; U; Android 2.2.2; tr-tr; Neoi-585 Build/Neoi-585_V01.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2786844 = {S29422, 140, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-ph; ICE+ Build/CloudfoneICE+) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2789493 = {S29423, 139, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; one touch 906 Build/FSR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2804393 = {S29424, 133, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; real3d Build/FRG22D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2809352 = {S29425, 132, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; in-id; A6T Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2810331 = {S29426, 141, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; ahong16_a10y Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2811330 = {S29427, 153, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; Full Android on Sapphire Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2811927 = {S29428, 139, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; in-id; ZTE-U V859 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2812165 = {S29429, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; AM728 3G Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2812963 = {S29430, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; xpndr_imh Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2812979 = {S29431, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME811 Build/SHDWR_X6_2.200.33) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2816930 = {S29432, 138, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; zh-cn; HW-HUAWEI_S8520/S8520V100R001C92B011; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2818013 = {S29433, 139, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SYTAB10ST-DS Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2832152 = {S29434, 137, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; aura 7.0 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2863812 = {S29435, 125, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; X2 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/*/
const Device UA2889085 = {S29436, 66, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; Coolpad W706+ Build/FRG83G)*/
const Device UA2949302 = {S29437, 61, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; CROSS-A1 Build/MASTER)*/
const Device UA2978151 = {S29438, 130, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; GS30 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2980346 = {S29439, 147, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; D000-X-S01 Build/00.21.29-02.20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982972 = {S29440, 135, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; rlg L8 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2988623 = {S29441, 140, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Z100 Build/SDB-P1_1.3.28) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042537 = {S29442, 149, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HomeChoiceE360 Alpha Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3085207 = {S29443, 167, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; msm7227_ffa Build/I700T_P6_T12_FLMO_FT_S42_R_0_02_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3098267 = {S29444, 136, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; it-it; MID701, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3098903 = {S29445, 147, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; Vodafone 858 Build/DeBranded_r0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3117234 = {S29446, 140, { &P15364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; P1-S Build/1.0.6008.0254) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA900885 = {S29447, 58, { &P15364,&P5690,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:14.0) Gecko/X Firefox/14.0*/
const Device UA901506 = {S29448, 58, { &P15364,&P5690,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:14.0) Gecko/X Firefox/14.0*/
const Device UA924860 = {S29449, 75, { &P15364,&P5690,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:14.0) Gecko/X Firefox/14.0.1 Firefox/14.0.1*/
const Device UA1996343 = {S29450, 72, { &P15364,&P5690,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:14.0) Gecko/X Firefox/14.0.1*/
const Device UA2413030 = {S29451, 69, { &P15364,&P5690,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 5.1; rv:14.0) Gecko/X Firefox/14.0.1*/
const Device UA2503007 = {S29452, 60, { &P15364,&P5690,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) Gecko/X Firefox/14.0 Opera/12.0*/
const Device UA2773075 = {S29453, 113, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.43.7 (KHTML, like Gecko) Version/7.0 Safari/537.43.7*/
const Device UA2799020 = {S29454, 109, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.44 (KHTML, like Gecko) Version/7.0 Safari/537.44*/
const Device UA2869954 = {S29455, 109, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.45 (KHTML, like Gecko) Version/7.0 Safari/537.45*/
const Device UA2884688 = {S29456, 113, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.46.5 (KHTML, like Gecko) Version/7.0 Safari/537.46.5*/
const Device UA2898147 = {S29457, 109, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.47 (KHTML, like Gecko) Version/7.0 Safari/537.47*/
const Device UA2948546 = {S29458, 113, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.48.3 (KHTML, like Gecko) Version/7.0 Safari/537.48.3*/
const Device UA2968948 = {S29459, 109, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.50 (KHTML, like Gecko) Version/7.0 Safari/537.50*/
const Device UA2975790 = {S29460, 113, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.51.3 (KHTML, like Gecko) Version/7.0 Safari/537.51.3*/
const Device UA3024219 = {S29461, 109, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.52 (KHTML, like Gecko) Version/7.0 Safari/537.52*/
const Device UA3024279 = {S29462, 109, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.53 (KHTML, like Gecko) Version/7.0 Safari/537.53*/
const Device UA3031448 = {S29463, 109, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.58 (KHTML, like Gecko) Version/7.0 Safari/537.58*/
const Device UA3033287 = {S29464, 109, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.56 (KHTML, like Gecko) Version/7.0 Safari/537.56*/
const Device UA3033622 = {S29465, 109, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.59 (KHTML, like Gecko) Version/7.0 Safari/537.59*/
const Device UA3034251 = {S29466, 113, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.54.1 (KHTML, like Gecko) Version/7.0 Safari/537.54.1*/
const Device UA3036984 = {S29467, 109, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.55 (KHTML, like Gecko) Version/7.0 Safari/537.55*/
const Device UA3056871 = {S29468, 107, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/538.1 (KHTML, like Gecko) Version/7.0 Safari/538.1*/
const Device UA3062139 = {S29469, 109, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.63 (KHTML, like Gecko) Version/7.0 Safari/537.63*/
const Device UA3063749 = {S29470, 109, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.64 (KHTML, like Gecko) Version/7.0 Safari/537.64*/
const Device UA3064189 = {S29471, 109, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.65 (KHTML, like Gecko) Version/7.0 Safari/537.65*/
const Device UA3065072 = {S29472, 109, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.61 (KHTML, like Gecko) Version/7.0 Safari/537.61*/
const Device UA3066111 = {S29473, 109, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.66 (KHTML, like Gecko) Version/7.0 Safari/537.66*/
const Device UA3080926 = {S29474, 109, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.68 (KHTML, like Gecko) Version/7.0 Safari/537.68*/
const Device UA3083312 = {S29475, 109, { &P17595,&P21721,&P24209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.71 (KHTML, like Gecko) Version/7.0 Safari/537.71*/
const Device UA368173 = {S29476, 97, { &P15364,&P17285,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/18.6.872.0 Safari/535.8*/
const Device UA368213 = {S29477, 104, { &P15364,&P17285,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/18.6.872.0 Safari/535.8*/
const Device UA412431 = {S29478, 108, { &P15364,&P17285,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.45 Safari/535.19*/
const Device UA457772 = {S29479, 139, { &P15364,&P17285,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19,gzip(gfe) (via translate.google.com)*/
const Device UA605617 = {S29480, 102, { &P15364,&P17285,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19*/
const Device UA631047 = {S29481, 146, { &P15364,&P17285,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19,gzip(gfe) (via translate.google.com)*/
const Device UA1964876 = {S29482, 136, { &P15364,&P17285,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Client/0; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2005704 = {S29483, 120, { &P15364,&P17285,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.19 (KHTML,             like Gecko) Chrome/18.0.1025.45 Safari/535.19*/
const Device UA2043425 = {S29484, 150, { &P15364,&P17285,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam GameOverlay/1960; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2206621 = {S29485, 150, { &P15364,&P17285,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam GameOverlay/2005; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2233630 = {S29486, 150, { &P15364,&P17285,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam GameOverlay/2009; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2236175 = {S29487, 150, { &P15364,&P17285,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam GameOverlay/2026; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2360235 = {S29488, 109, { &P15364,&P17285,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; de) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/18.0 Safari/525.13*/
const Device UA2405708 = {S29489, 150, { &P15364,&P17285,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam GameOverlay/2095; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2431095 = {S29490, 150, { &P15364,&P17285,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam GameOverlay/2109; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2433581 = {S29491, 150, { &P15364,&P17285,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam GameOverlay/2102; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2472059 = {S29492, 150, { &P15364,&P17285,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam GameOverlay/2125; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2510339 = {S29493, 147, { &P15364,&P17285,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam GameOverlay/X; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2528717 = {S29494, 132, { &P15364,&P17285,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/18.0 BIDUPlayerBrowser/2.x Safari/534.7*/
const Device UA2666722 = {S29495, 120, { &P15364,&P17285,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/18.0 BIDUPlayerBrowser/2.x Safari/534.7*/
const Device UA2967076 = {S29496, 116, { &P15364,&P17285,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.19 (KHTML,         like Gecko) Chrome/18.0.1025.45 Safari/535.19*/
const Device UA3028154 = {S29497, 145, { &P15364,&P17285,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam Client/X; ) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Safari/535.19*/
const Device UA2042162 = {S29498, 70, { &P15364,&P17285,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:15.0) Gecko/X Firefox/15.0*/
const Device UA2043619 = {S29499, 67, { &P15364,&P17285,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:15.0) Gecko/X Firefox/15.0.1*/
const Device UA2189649 = {S29500, 75, { &P15364,&P17285,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:15.0) Gecko/X Firefox/15.0.1 Firefox/15.0.1*/
const Device UA3057442 = {S29501, 72, { &P15364,&P17285,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; rv:15.0) Gecko/X Firefox/15.0.2*/
const Device UA92716 = {S29502, 79, { &P15364,&P7127,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.0) LinkCheck by Siteimprove.com*/
const Device UA149322 = {S29503, 154, { &P15364,&P7127,&P17193,&P18092 }};	/*Useragent - Mozilla/4.79 [en] (compatible; MSIE 7.0; Windows NT 5.0; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 1.1.4322; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648)*/
const Device UA162534 = {S29504, 51, { &P15364,&P7127,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.01; Windows NT 5.0)*/
const Device UA196402 = {S29505, 159, { &P15364,&P7127,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 5.0; Trident/4.0; FBSMTWB; .NET CLR 2.0.34861; .NET CLR 3.0.3746.3218; .NET CLR 3.5.33652; msn OptimizedIE8;ENUS)*/
const Device UA389321 = {S29506, 55, { &P15364,&P7127,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.0; SV1)*/
const Device UA543800 = {S29507, 50, { &P15364,&P7127,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 5.0)*/
const Device UA652531 = {S29508, 62, { &P15364,&P7127,&P17193,&P18092 }};	/*Useragent - Mozilla/6.0 (compatible;MSIE 7.0;  Windows NT 5.0; QQDownload)*/
const Device UA715450 = {S29509, 59, { &P15364,&P7127,&P17193,&P18092 }};	/*Useragent - Mozilla/3.0 (compatible;MSIE 7.0;  Windows NT 5.0; LOVEYOU)*/
const Device UA1961958 = {S29510, 68, { &P15364,&P7127,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 5.0; SV1)*/
const Device UA2359701 = {S29511, 98, { &P15364,&P7127,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; Mozilla/4.0 (compatible; MSIE 7.0; Win32; 1&1))*/
const Device UA2721566 = {S29512, 97, { &P15364,&P7127,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)*/
const Device UA2790163 = {S29513, 79, { &P15364,&P7127,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.0; SEOlyzer 1.0 by SOFTPAE.com)*/
const Device UA3028033 = {S29514, 135, { &P15364,&P7127,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; FDM)*/
const Device UA3069347 = {S29515, 51, { &P15364,&P7127,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.01; Windows NT 5.0)*/
const Device UA340629 = {S29516, 130, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA371147 = {S29517, 123, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA647175 = {S29518, 59, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P6200 Build/HTJ85B)*/
const Device UA665608 = {S29519, 131, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; pt-br; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1945425 = {S29520, 60, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P6200L Build/HTJ85B)*/
const Device UA2025030 = {S29521, 151, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B; 1024*552) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2027422 = {S29522, 128, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2034042 = {S29523, 137, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2216330 = {S29524, 128, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2608819 = {S29525, 151, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200L Build/HTJ85B; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2609719 = {S29526, 128, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build HTJ85B) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2609781 = {S29527, 122, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6200) antivirusAM/2.4.5 (KHTML, like Gecko) Version/Version 2.4.5/antivirus*/
const Device UA2609981 = {S29528, 132, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6200 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2611888 = {S29529, 275, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.3;FBDM/{density=1.0,width=600,height=976};FBLC/en_GB;FB_FW/2;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P6200;FBSV/3.2;]*/
const Device UA2612702 = {S29530, 60, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.3; 4) GT-P6200 (Android 3.2, SDK 13)*/
const Device UA2612772 = {S29531, 138, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2613185 = {S29532, 92, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 3.2; SAMSUNG GT-P6200 Build/HONEYCOMB_MR2);*/
const Device UA2613949 = {S29533, 129, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200L Build/HTJ85B) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2614869 = {S29534, 160, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2614967 = {S29535, 270, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; pt-br; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.10;FBDM/{density=1.0,width=600,height=976};FBLC/pt_BR;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P6200L;FBSV/3.2;]*/
const Device UA2615368 = {S29536, 129, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2615701 = {S29537, 135, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2616569 = {S29538, 137, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2616745 = {S29539, 100, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-ph; GT-P6200 B3 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2617367 = {S29540, 133, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6200L Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2617594 = {S29541, 265, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.11;FBDM/{density=1.0,width=600,height=976};FBLC/en_GB;FBCR/Smart;FBPN/com.facebook.katana;FBDV/GT-P6200;FBSV/3.2;]*/
const Device UA2618044 = {S29542, 198, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 ZyngaBundleIdentifier/com.zynga.zjayakashi ZyngaBundleVersion/1.7.0*/
const Device UA2618250 = {S29543, 140, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B like Xoom) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2618424 = {S29544, 77, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - MediaPlayer/2002/4.0.12 (Android 3.2; GT-P6200; GT-P6200; samsung; ; ; en_US)*/
const Device UA2618632 = {S29545, 71, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 3.2; en-US; GT-P6200 Build/HTJ85B.XXKL7)*/
const Device UA2619082 = {S29546, 261, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2619150 = {S29547, 79, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) Expedia/2.0.1*/
const Device UA2619193 = {S29548, 269, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ms-my; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.10;FBDM/{density=1.0,width=1024,height=552};FBLC/ms_MY;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P6200;FBSV/3.2;]*/
const Device UA2619716 = {S29549, 261, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ms-my; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.11;FBDM/{density=1.0,width=1024,height=552};FBLC/ms_MY;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P6200;FBSV/3.2;]*/
const Device UA2620397 = {S29550, 77, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - MediaPlayer/2002/4.0.12 (Android 3.2; GT-P6200; GT-P6200; samsung; ; ; en_GB)*/
const Device UA2621046 = {S29551, 265, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-ph; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.12;FBDM/{density=1.0,width=600,height=976};FBLC/en_PH;FBCR/SMART;FBPN/com.facebook.katana;FBDV/GT-P6200;FBSV/3.2;]*/
const Device UA2621532 = {S29552, 276, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.8.3;FBDM/{density=1.0,width=1024,height=552};FBLC/en_US;FB_FW/1;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P6200;FBSV/3.2;]*/
const Device UA2621903 = {S29553, 73, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 3.2; en-GB; GT-P6200 Build/HTJ85B.XXKK6)*/
const Device UA2623580 = {S29554, 131, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Groupon/2.1.2486 (Android 3.2; Samsung GT-P6200 / Samsung GT-P6200; MAXIS)[preload=false;locale=en_US;clientidbase=android-samsung]*/
const Device UA2624263 = {S29555, 193, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-ph; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 androidAppVersion/2.5.16 AndroidWebkitWrapper BillingSupported*/
const Device UA2624553 = {S29556, 198, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 ZyngaBundleIdentifier/com.zynga.zjayakashi ZyngaBundleVersion/1.8.0*/
const Device UA2626520 = {S29557, 73, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/2.2.10 (Linux; U; Android 3.2; es-ES; GT-P6200L Build/HTJ85B.UBLA1)*/
const Device UA2628630 = {S29558, 92, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - com.google.android.youtube/4.0.23(Linux; U; Android 3.2; en_US; GT-P6200L Build/HTJ85B) gzip*/
const Device UA2629012 = {S29559, 131, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like GeckAccept-Encoding: gzip,deflate*/
const Device UA2630040 = {S29560, 198, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 ZyngaBundleIdentifier/com.zynga.zjayakashi ZyngaBundleVersion/1.9.0*/
const Device UA2633267 = {S29561, 45, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Scanner Radio/3.9.1.1 (Android 3.2, GT-P6200)*/
const Device UA2634217 = {S29562, 57, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - nApps (Android 3.2; GT-P6200; androidapp.lineplay; 1.2.1)*/
const Device UA2635618 = {S29563, 136, { &P15985,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; se-se; GT-P6200L Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2739856 = {S29564, 130, { &P18283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.85 Safari/537.36*/
const Device UA2741707 = {S29565, 137, { &P18283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2741708 = {S29566, 137, { &P18283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2741718 = {S29567, 137, { &P18283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754410 = {S29568, 137, { &P18283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755125 = {S29569, 137, { &P18283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-I535 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755127 = {S29570, 138, { &P18283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999V Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755355 = {S29571, 138, { &P18283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755358 = {S29572, 137, { &P18283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755435 = {S29573, 137, { &P18283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; GT-I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2757162 = {S29574, 138, { &P18283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769746 = {S29575, 138, { &P18283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769784 = {S29576, 137, { &P18283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-i9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783527 = {S29577, 138, { &P18283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I93000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799343 = {S29578, 137, { &P18283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I939 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2866731 = {S29579, 130, { &P18283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.74 Safari/537.36*/
const Device UA2868576 = {S29580, 135, { &P18283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SC-06D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2875469 = {S29581, 134, { &P18283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; i9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892789 = {S29582, 134, { &P18283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2948290 = {S29583, 138, { &P18283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2043356 = {S29584, 69, { &P15364,&P18118,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:15.0) Gecko/X Firefox/15.0.1*/
const Device UA2202075 = {S29585, 86, { &P15364,&P18118,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:15.0) Gecko/X Firefox/15.0.1 Firefox/15.0.1*/
const Device UA363926 = {S29586, 64, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.229 Version/11.61*/
const Device UA368124 = {S29587, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.61*/
const Device UA380709 = {S29588, 64, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; en) Presto/2.10.229 Version/11.61*/
const Device UA380764 = {S29589, 101, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.229 Version/11.61,gzip(gfe) (via translate.google.com)*/
const Device UA380834 = {S29590, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.229 Version/11.61*/
const Device UA380837 = {S29591, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.229 Version/11.61*/
const Device UA381037 = {S29592, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.229 Version/11.61*/
const Device UA381095 = {S29593, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.229 Version/11.61*/
const Device UA381142 = {S29594, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.229 Version/11.61*/
const Device UA382089 = {S29595, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.10.229 Version/11.61*/
const Device UA382150 = {S29596, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.229 Version/11.61*/
const Device UA398352 = {S29597, 66, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; en) Opera 11.61*/
const Device UA402571 = {S29598, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.229 Version/11.61*/
const Device UA440927 = {S29599, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.229 Version/11.61*/
const Device UA587680 = {S29600, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.61*/
const Device UA587918 = {S29601, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03780); ru) Presto/2.10.229 Version/11.61*/
const Device UA588209 = {S29602, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.229 Version/11.61*/
const Device UA588567 = {S29603, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.10.229 Version/11.61*/
const Device UA588657 = {S29604, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5161); ru) Presto/2.10.229 Version/11.61*/
const Device UA588707 = {S29605, 72, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; ru; rv:2.0) Gecko/X Firefox/4.0 Opera 11.61*/
const Device UA589377 = {S29606, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5218); ru) Presto/2.10.229 Version/11.61*/
const Device UA590023 = {S29607, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5230); ru) Presto/2.10.229 Version/11.61*/
const Device UA600098 = {S29608, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.229 Version/11.61*/
const Device UA607428 = {S29609, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.229 Version/11.61*/
const Device UA626876 = {S29610, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.61*/
const Device UA628776 = {S29611, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.229 Version/11.61*/
const Device UA630441 = {S29612, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.229 Version/11.61*/
const Device UA661644 = {S29613, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5217); ru) Presto/2.10.229 Version/11.61*/
const Device UA680603 = {S29614, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.229 Version/11.61*/
const Device UA681239 = {S29615, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.61*/
const Device UA683846 = {S29616, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.229 Version/11.61*/
const Device UA684861 = {S29617, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.229 Version/11.61*/
const Device UA688060 = {S29618, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.229 Version/11.61*/
const Device UA688481 = {S29619, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.61*/
const Device UA693161 = {S29620, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.10.229 Version/11.61*/
const Device UA696400 = {S29621, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.229 Version/11.61*/
const Device UA697446 = {S29622, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.229 Version/11.61*/
const Device UA704422 = {S29623, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.229 Version/11.61*/
const Device UA705313 = {S29624, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4930); ru) Presto/2.10.229 Version/11.61*/
const Device UA709799 = {S29625, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.229 Version/11.61*/
const Device UA715444 = {S29626, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.229 Version/11.61*/
const Device UA727556 = {S29627, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.10.229 Version/11.61*/
const Device UA727591 = {S29628, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.229 Version/11.61*/
const Device UA731455 = {S29629, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.229 Version/11.61*/
const Device UA736420 = {S29630, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.10.229 Version/11.61*/
const Device UA739376 = {S29631, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4661); ru) Presto/2.10.229 Version/11.61*/
const Device UA746366 = {S29632, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02652); ru) Presto/2.10.229 Version/11.61*/
const Device UA770773 = {S29633, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.229 Version/11.61*/
const Device UA790501 = {S29634, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.10.229 Version/11.61*/
const Device UA805712 = {S29635, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.229 Version/11.61*/
const Device UA853413 = {S29636, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5278); ru) Presto/2.10.229 Version/11.61*/
const Device UA862095 = {S29637, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.229 Version/11.61*/
const Device UA869558 = {S29638, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.229 Version/11.61*/
const Device UA901849 = {S29639, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.229 Version/11.61*/
const Device UA1950889 = {S29640, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03672); ru) Presto/2.10.229 Version/11.61*/
const Device UA1951441 = {S29641, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03638); ru) Presto/2.10.229 Version/11.61*/
const Device UA1952176 = {S29642, 114, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); MRSPUTNIK OW 2, 3, 0, 104; ru) Presto/2.10.229 Version/11.61*/
const Device UA1953425 = {S29643, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02560); ru) Presto/2.10.229 Version/11.61*/
const Device UA1956013 = {S29644, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5288); ru) Presto/2.10.229 Version/11.61*/
const Device UA1957819 = {S29645, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.229 Version/11.61*/
const Device UA1964774 = {S29646, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.229 Version/11.61*/
const Device UA1965515 = {S29647, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02552); ru) Presto/2.10.229 Version/11.61*/
const Device UA1975958 = {S29648, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5328); ru) Presto/2.10.229 Version/11.61*/
const Device UA1995263 = {S29649, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.10.229 Version/11.61*/
const Device UA2153238 = {S29650, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5667); ru) Presto/2.10.229 Version/11.61*/
const Device UA2174743 = {S29651, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.10.229 Version/11.61*/
const Device UA2186173 = {S29652, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.229 Version/11.61*/
const Device UA2235281 = {S29653, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.10.229 Version/11.61*/
const Device UA2323268 = {S29654, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02564); ru) Presto/2.10.229 Version/11.61*/
const Device UA2333678 = {S29655, 87, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5299); ru) Presto/2.10.229 Version/11.61*/
const Device UA2386689 = {S29656, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.229 Version/11.61*/
const Device UA2403924 = {S29657, 74, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.229 Version/11.61,gzip(gfe)*/
const Device UA2414030 = {S29658, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5966); ru) Presto/2.10.229 Version/11.61*/
const Device UA2422612 = {S29659, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.229 Version/11.61*/
const Device UA2441526 = {S29660, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.229 Version/11.61*/
const Device UA2456103 = {S29661, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.229 Version/11.61*/
const Device UA2468065 = {S29662, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.229 Version/11.61*/
const Device UA2600812 = {S29663, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.229 Version/11.61*/
const Device UA2717787 = {S29664, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6089); ru) Presto/2.10.229 Version/11.61*/
const Device UA2777674 = {S29665, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5993); ru) Presto/2.10.229 Version/11.61*/
const Device UA2842921 = {S29666, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5880); ru) Presto/2.10.229 Version/11.61*/
const Device UA2844982 = {S29667, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6011); ru) Presto/2.10.229 Version/11.61*/
const Device UA2854866 = {S29668, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.229 Version/11.61*/
const Device UA2857317 = {S29669, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6054); ru) Presto/2.10.229 Version/11.61*/
const Device UA2861195 = {S29670, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6083); ru) Presto/2.10.229 Version/11.61*/
const Device UA2868350 = {S29671, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.229 Version/11.61*/
const Device UA2991077 = {S29672, 86, { &P15364,&P5690,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6081); ru) Presto/2.10.229 Version/11.61*/
const Device UA21930 = {S29673, 168, { &P14748,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE72-2/031.023; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.3.1*/
const Device UA43668 = {S29674, 170, { &P14748,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE72/021.024 (SymbianOS/9.3; U; Series60/3.2 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.1.19424*/
const Device UA44509 = {S29675, 191, { &P14748,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE72-2/071.004; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.26 Mobile Safari/533.4 3gpp-gba*/
const Device UA64680 = {S29676, 168, { &P14748,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE72-1/031.023; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.3.1*/
const Device UA67764 = {S29677, 37, { &P14748,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE72-1/SymbianOS/9.1 Series60/3.0*/
const Device UA891840 = {S29678, 191, { &P14748,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE72-1/091.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.34 Mobile Safari/533.4 3gpp-gba*/
const Device UA422388 = {S29679, 132, { &P17346,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7534.48.3*/
const Device UA623152 = {S29680, 134, { &P17346,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA896333 = {S29681, 132, { &P17346,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3*/
const Device UA2394751 = {S29682, 134, { &P17346,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9Z999 Safari/7534.48.3*/
const Device UA2525751 = {S29683, 163, { &P17346,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9B206 Version/5.1 Safari/8536.25 Safari/6533.18.5*/
const Device UA2598075 = {S29684, 131, { &P17346,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 511 like Mac OS X) AppleWebKit/534.46 (KHTML like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA2685364 = {S29685, 134, { &P17346,&P18117,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3*/
const Device UA213927 = {S29686, 138, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-I510 4G Build/EP1F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA223886 = {S29687, 142, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-I510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA244397 = {S29688, 137, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I510 4G Build/EP3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250507 = {S29689, 142, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA254311 = {S29690, 135, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 4G Build/0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA338505 = {S29691, 137, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-I510 4G Build/EP1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA350903 = {S29692, 66, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-I510 Build/GINGERBREAD)*/
const Device UA562114 = {S29693, 138, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-I510 4G Build/EP1H) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA571732 = {S29694, 139, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-I510 4G Build/B 4.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA576037 = {S29695, 137, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 4G Build/EP4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622550 = {S29696, 142, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623706 = {S29697, 137, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 4G Build/FP1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1889694 = {S29698, 137, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SCH-I510 4G Build/EG2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1997527 = {S29699, 137, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 4G Build/FP5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022060 = {S29700, 140, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SCH-I510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024671 = {S29701, 137, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 4G Build/FP3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024704 = {S29702, 34, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - Android/1.0 (SCH-I510 GINGERBREAD)*/
const Device UA2031884 = {S29703, 150, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2439834 = {S29704, 149, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2570291 = {S29705, 137, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 4G Build/FP8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2576274 = {S29706, 137, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 4G Build/FP6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2588737 = {S29707, 97, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - com.google.android.youtube/2.4.4(Linux; U; Android 2.3.6; en_US; SCH-I510 Build/GINGERBREAD) gzip*/
const Device UA2668683 = {S29708, 135, { &P17149,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SCH-I510 4G Build/FP8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA902622 = {S29709, 60, { &P15364,&P5690,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:13.0) Gecko/X Firefox/13.0.1*/
const Device UA903472 = {S29710, 60, { &P15364,&P5690,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:13.0) Gecko/X Firefox/13.0.1*/
const Device UA903802 = {S29711, 75, { &P15364,&P5690,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:13.0) Gecko/X Firefox/13.0.1 Firefox/13.0.1*/
const Device UA628298 = {S29712, 138, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MOT-XT615 Build/V1.52A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA631050 = {S29713, 138, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; MOT-XT615 Build/V1.540) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA649823 = {S29714, 134, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; XT615 Build/V1.37H) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA665245 = {S29715, 134, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; XT615 Build/V1.540) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA742103 = {S29716, 144, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - MOT-XT615/Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT615 Build/V1.62D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA768122 = {S29717, 138, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MOT-XT615 Build/V1.48C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA831195 = {S29718, 134, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; XT615 Build/V1.52A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA908073 = {S29719, 134, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT615 Build/V1.590) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1958815 = {S29720, 134, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; XT615 Build/V1.48C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1964904 = {S29721, 134, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT615 Build/V1.600) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1965820 = {S29722, 137, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MOT-XT389 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1991178 = {S29723, 134, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT615 Build/V1.59A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1994361 = {S29724, 134, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT615 Build/V1.60B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1994821 = {S29725, 134, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT615 Build/V1.54B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1994989 = {S29726, 138, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; MOT-XT615 Build/V1.67C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995364 = {S29727, 143, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MOT-XT389 Build/06.13.03R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995404 = {S29728, 138, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MOT-XT615 Build/V1.670) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2012020 = {S29729, 143, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MOT-XT389 Build/0B.10.00R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023904 = {S29730, 134, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; XT615 Build/V1.54C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024740 = {S29731, 122, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT615 Build/V1.540) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2025254 = {S29732, 134, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT615 Build/V1.37I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027589 = {S29733, 127, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT615 Build/V1.540) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2029952 = {S29734, 140, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; MOT-XT389 Build/0A.17.02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030910 = {S29735, 134, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; XT615 Build/V1.67C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031804 = {S29736, 134, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; XT615 Build/V1.37L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034184 = {S29737, 58, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; XT615 Build/V1.540)*/
const Device UA2180048 = {S29738, 139, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; XT389 Build/06.13.03R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2204381 = {S29739, 143, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MOT-XT389 Build/06.1A.00R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2256999 = {S29740, 143, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MOT-XT389 Build/0B.13.00R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2348924 = {S29741, 139, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; XT389 Build/0B.10.00R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2356353 = {S29742, 144, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - MOT-XT615/Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT615 Build/V1.74D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2372572 = {S29743, 127, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT615 Build/V1.54B) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2381816 = {S29744, 143, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; MOT-XT389 Build/0A.26.00R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2382965 = {S29745, 139, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; XT389 Build/0C.03.05R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2395967 = {S29746, 134, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; XT615 Build/V1.670) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2413525 = {S29747, 139, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT389 Build/07.1F.04R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2421507 = {S29748, 139, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; XT389 Build/0B.13.00R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2480738 = {S29749, 134, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT615 Build/V1.60E) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2510695 = {S29750, 144, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - MOT-XT615/Mozilla/5.0 (Linux; U; Android 2.3.7; fr-ca; XT615 Build/V1.74F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2589496 = {S29751, 138, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ro-ro; MOT-XT615 Build/V1.67A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2644256 = {S29752, 127, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT615 Build/V1.37I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2709016 = {S29753, 134, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT615 Build/V1.60G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2746764 = {S29754, 143, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MOT-XT389 Build/0A.1A.06R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2810529 = {S29755, 127, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT615 Build/V1.37H) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2838995 = {S29756, 139, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; XT389 Build/0A.17.02R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982058 = {S29757, 139, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; XT389 Build/06.1A.00R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2983242 = {S29758, 139, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT389 Build/07.1F.1CR_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3117453 = {S29759, 134, { &P18252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT615 Build/V1.60C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA897314 = {S29760, 106, { &P15364,&P17285,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5*/
const Device UA897521 = {S29761, 136, { &P15364,&P17285,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5,gzip(gfe) (via translate.google.com)*/
const Device UA900839 = {S29762, 99, { &P15364,&P17285,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA2363159 = {S29763, 125, { &P15364,&P17285,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5*/
const Device UA2391107 = {S29764, 132, { &P15364,&P17285,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5*/
const Device UA460748 = {S29765, 212, { &P15364,&P7133,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; OfficeLiveConnector.1.4; OfficeLivePatch.1.3)*/
const Device UA656327 = {S29766, 183, { &P15364,&P7133,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Avant Browser)*/
const Device UA768713 = {S29767, 191, { &P15364,&P7133,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Avant Browser)*/
const Device UA775588 = {S29768, 193, { &P15364,&P7133,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Avant Browser)*/
const Device UA803519 = {S29769, 162, { &P15364,&P7133,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Avant Browser)*/
const Device UA837218 = {S29770, 198, { &P15364,&P7133,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Avant Browser)*/
const Device UA1961008 = {S29771, 78, { &P15364,&P7133,&P22730,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 6.1; Avant Browser)*/
const Device UA1971245 = {S29772, 196, { &P15364,&P7133,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; Avant Browser)*/
const Device UA1988975 = {S29773, 155, { &P15364,&P7133,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Avant Browser)*/
const Device UA2222381 = {S29774, 198, { &P15364,&P7133,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Avant Browser)*/
const Device UA2372582 = {S29775, 193, { &P15364,&P7133,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Zune 4.7; Avant Browser)*/
const Device UA2394961 = {S29776, 160, { &P15364,&P7133,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Avant Browser)*/
const Device UA2685149 = {S29777, 167, { &P15364,&P7133,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Avant Browser)*/
const Device UA2947822 = {S29778, 225, { &P15364,&P7133,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Win64; x64; Trident/6.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0; MAAU; NP08; Avant Browser)*/
const Device UA23674 = {S29779, 145, { &P13503,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_DesireZ_A7272 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38818 = {S29780, 151, { &P13503,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-ch; HTC Desire Z 1.82.166.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38819 = {S29781, 152, { &P13503,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-ch; HTC Desire Z  2.42.166.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38848 = {S29782, 151, { &P13503,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire Z 2.42.161.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38924 = {S29783, 151, { &P13503,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire Z 1.82.161.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38928 = {S29784, 151, { &P13503,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-ch; HTC Desire Z 2.42.166.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA39215 = {S29785, 151, { &P13503,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC Desire Z 1.85.861.3 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40572 = {S29786, 137, { &P13503,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; HTC-A7275 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46215 = {S29787, 145, { &P13503,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_DesireZ_A7272 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52561 = {S29788, 151, { &P13503,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC Desire Z 2.42.861.6 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA898876 = {S29789, 152, { &P13503,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire Z 2.42.161.20 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1996937 = {S29790, 151, { &P13503,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC Desire Z 2.42.861.8 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2337606 = {S29791, 151, { &P13503,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-at; HTC Desire Z 2.42.166.4 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2476014 = {S29792, 152, { &P13503,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-at; HTC Desire Z  2.42.166.4 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2505000 = {S29793, 152, { &P13503,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; nl-nl; HTC Desire Z 2.42.161.22 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3064641 = {S29794, 134, { &P18635,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus 4 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2016039 = {S29795, 99, { &P15364,&P16110,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2016030 = {S29796, 106, { &P15364,&P16110,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2186224 = {S29797, 136, { &P15364,&P16110,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe) (via translate.google.com)*/
const Device UA2553892 = {S29798, 116, { &P15364,&P16110,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe)*/
const Device UA2842242 = {S29799, 109, { &P15364,&P16110,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1,gzip(gfe)*/
const Device UA909110 = {S29800, 140, { &P16292,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; Galaxy Nexus Build/JRN84D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA911957 = {S29801, 142, { &P16292,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Galaxy Nexus Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1955698 = {S29802, 141, { &P16292,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1;  Galaxy Nexus Build/JRN84D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA1996708 = {S29803, 136, { &P16292,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Galaxy Nexus Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2197579 = {S29804, 142, { &P16292,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2233629 = {S29805, 136, { &P16292,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2237076 = {S29806, 143, { &P16292,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1;  Galaxy Nexus Build/JRO03O) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA2405860 = {S29807, 138, { &P16292,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Galaxy X Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2652627 = {S29808, 143, { &P16292,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2;  Galaxy Nexus Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA401283 = {S29809, 118, { &P17595,&P17296,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA401290 = {S29810, 118, { &P17595,&P17296,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA401320 = {S29811, 118, { &P17595,&P17296,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA401390 = {S29812, 118, { &P17595,&P17296,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA401423 = {S29813, 118, { &P17595,&P17296,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA412895 = {S29814, 118, { &P17595,&P17296,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.65 Safari/535.11*/
const Device UA412968 = {S29815, 118, { &P17595,&P17296,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11*/
const Device UA428564 = {S29816, 118, { &P17595,&P17296,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11*/
const Device UA602319 = {S29817, 118, { &P17595,&P17296,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.83 Safari/535.11*/
const Device UA891105 = {S29818, 138, { &P17595,&P17296,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.10 (KHTML, like Gecko) Chrome/17.0.925.0 QQBrowser/1.0.0.1096 Safari/535.10*/
const Device UA2223860 = {S29819, 130, { &P17595,&P17296,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) UC AppleWebKit/534.31 (KHTML, like Gecko) Chrome/17.0.558.0 Safari/534.31*/
const Device UA2374246 = {S29820, 118, { &P17595,&P17296,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/535.11 (KHTML	 like Gecko) Chrome/17.0.963.46 Safari/535.11*/
const Device UA2473535 = {S29821, 139, { &P17595,&P17296,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.4.0.2117 Safari/535.11*/
const Device UA2597176 = {S29822, 115, { &P17595,&P17296,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1061) AppleWebKit/535.11 (KHTML like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA2600202 = {S29823, 115, { &P17595,&P17296,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1060) AppleWebKit/535.11 (KHTML like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA2674611 = {S29824, 129, { &P17595,&P17296,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/17.0.648.205 Safari/534.16*/
const Device UA2674807 = {S29825, 129, { &P17595,&P17296,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/17.0.648.205 Safari/534.16*/
const Device UA2716430 = {S29826, 129, { &P17595,&P17296,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/17.0.648.205 Safari/534.16*/
const Device UA603067 = {S29827, 79, { &P15364,&P5690,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; I; Windows NT 5.1; en; rv:1.9.2.13) Gecko/X Firefox/5.0.1*/
const Device UA783223 = {S29828, 74, { &P15364,&P5690,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:5.0.1) Gecko/X Firefox/5.0.1 Firefox/5.0.1*/
const Device UA894653 = {S29829, 48, { &P15364,&P5690,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) Gecko/X Firefox/5.0*/
const Device UA1888395 = {S29830, 93, { &P15364,&P5690,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:5.0) Gecko/X Firefox/5.0,gzip(gfe) (via translate.google.com)*/
const Device UA2233999 = {S29831, 66, { &P15364,&P5690,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:5.0) Gecko/X Firefox/5.0,gzip(gfe)*/
const Device UA2978569 = {S29832, 75, { &P15364,&P5690,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:5.0) Gecko/X Firefox/5.0*/
const Device UA101099 = {S29833, 78, { &P15364,&P5690,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Avant Browser)*/
const Device UA270101 = {S29834, 93, { &P15364,&P5690,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Avant Browser; Avant Browser)*/
const Device UA786273 = {S29835, 136, { &P15364,&P5690,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Avant Browser)*/
const Device UA2164083 = {S29836, 78, { &P15364,&P5690,&P22731,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; Avant Browser)*/
const Device UA2166540 = {S29837, 78, { &P15364,&P5690,&P22731,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; Avant Browser)*/
const Device UA2166649 = {S29838, 78, { &P15364,&P5690,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; Avant Browser)*/
const Device UA2315079 = {S29839, 215, { &P15364,&P5690,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Avant Browser; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Avant Browser)*/
const Device UA2330667 = {S29840, 216, { &P15364,&P5690,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Avant Browser; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Avant Browser)*/
const Device UA2374704 = {S29841, 308, { &P15364,&P5690,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Avant Browser; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; MSN Optimized;FR; .NET CLR 2.0.50727; FDM; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSN Optimized;FR)*/
const Device UA2546383 = {S29842, 93, { &P15364,&P5690,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; Avant Browser; Avant Browser)*/
const Device UA2722526 = {S29843, 93, { &P15364,&P5690,&P22731,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; Avant Browser; Avant Browser)*/
const Device UA2725129 = {S29844, 83, { &P15364,&P5690,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; Avant Browser; SV1)*/
const Device UA2727933 = {S29845, 183, { &P15364,&P5690,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Avant Browser)*/
const Device UA2745291 = {S29846, 93, { &P15364,&P5690,&P22731,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; Avant Browser; Avant Browser)*/
const Device UA2758789 = {S29847, 83, { &P15364,&P5690,&P22731,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.1; SV1; Avant Browser)*/
const Device UA2775858 = {S29848, 83, { &P15364,&P5690,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; SV1; Avant Browser)*/
const Device UA2787282 = {S29849, 83, { &P15364,&P5690,&P22731,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.2; SV1; Avant Browser)*/
const Device UA2950505 = {S29850, 201, { &P15364,&P5690,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Avant Browser)*/
const Device UA3041863 = {S29851, 108, { &P15364,&P5690,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Avant Browser; Avant Browser; Avant Browser)*/
const Device UA2456415 = {S29852, 139, { &P20146,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2529544 = {S29853, 61, { &P20146,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9082 Build/JZO54K)*/
const Device UA2559218 = {S29854, 140, { &P20146,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9082L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2600023 = {S29855, 137, { &P20146,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708238 = {S29856, 151, { &P20146,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.3 Version/4.0 Mobile Safari/530.17*/
const Device UA2708372 = {S29857, 62, { &P20146,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9082L Build/JZO54K)*/
const Device UA2896088 = {S29858, 140, { &P20146,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9082i Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952513 = {S29859, 130, { &P20146,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9082 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3106722 = {S29860, 215, { &P20146,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Samsung-GT-I9082/1.0 Android/4.1.1 Release/05.24.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA608876 = {S29861, 141, { &P15397,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ST18i Build/4.1.A.0.562) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616507 = {S29862, 141, { &P15397,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; Xperia Ray Build/ITL41F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616521 = {S29863, 146, { &P15397,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Xperia ray Build/4.1.A.0.562) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA685195 = {S29864, 63, { &P15397,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; ST18i Build/4.1.A.0.562)*/
const Device UA899600 = {S29865, 141, { &P15397,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST18i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA907616 = {S29866, 153, { &P15397,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SonyEricssonST18a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1969834 = {S29867, 141, { &P15397,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; ST18a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2008428 = {S29868, 153, { &P15397,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonST18i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029180 = {S29869, 63, { &P15397,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST18i Build/4.1.B.0.587)*/
const Device UA2030753 = {S29870, 141, { &P15397,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST18i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032980 = {S29871, 132, { &P15397,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ST18i Build/4.1.B.0.431) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2183235 = {S29872, 154, { &P15397,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonST18iv Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2230387 = {S29873, 154, { &P15397,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; SonyEricssonST18av Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2383899 = {S29874, 132, { &P15397,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ST18i Build/4.1.B.0.587) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2498474 = {S29875, 141, { &P15397,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST18a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2848553 = {S29876, 140, { &P15397,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; ST18i Build/4.1.B.1.13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3042332 = {S29877, 63, { &P15397,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST18a Build/4.1.B.0.587)*/
const Device UA47401 = {S29878, 153, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST15i Build/4.0.1.A.0.169) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47423 = {S29879, 139, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; ST15i Build/4.0.A.2.364) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49513 = {S29880, 151, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonST15i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49761 = {S29881, 153, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST15i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49764 = {S29882, 152, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST15i Build/4.0.1.A.2.24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60342 = {S29883, 151, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonST15i Build/4.0.A.2.330) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60345 = {S29884, 151, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonST15i Build/4.0.A.2.319) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60349 = {S29885, 151, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonST15i Build/4.0.A.2.364) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60353 = {S29886, 151, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonST15i Build/4.0.A.2.329) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60378 = {S29887, 151, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonST15i Build/4.0.A.2.254) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60383 = {S29888, 151, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonST15i Build/4.0.A.2.240) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71193 = {S29889, 139, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ST15i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71856 = {S29890, 151, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ie; SonyEricssonST15i Build/4.0.A.2.377) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72964 = {S29891, 141, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; ST15i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA75398 = {S29892, 152, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST15i Build/4.0.2.A.0.45) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA571468 = {S29893, 63, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; ST15i Build/4.0.A.2.377)*/
const Device UA590243 = {S29894, 128, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; ST15i Build/4.0.2.A.0.62) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA622926 = {S29895, 64, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; ST15i Build/4.0.2.A.0.62)*/
const Device UA630101 = {S29896, 152, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonST15i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA630539 = {S29897, 152, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonST15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA632610 = {S29898, 140, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; ST15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA649392 = {S29899, 152, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ie; SonyEricssonST15i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA680126 = {S29900, 151, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-mx; SonyEricssonST15a Build/4.0.A.2.377) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA710703 = {S29901, 140, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; ST15i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA743061 = {S29902, 139, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-mx; ST15a Build/4.0.A.2.377) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA817786 = {S29903, 140, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; ST15i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA852551 = {S29904, 152, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-cr; SonyEricssonST15a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1948411 = {S29905, 153, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; hr-hr; SonyEricssonST15iv Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1964460 = {S29906, 152, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; SonyEricssonST15a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023687 = {S29907, 63, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; ST15a Build/4.0.A.2.377)*/
const Device UA2507725 = {S29908, 152, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; SonyEricssonST15a Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2512815 = {S29909, 152, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonST15i Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2558575 = {S29910, 153, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; hr-hr; SonyEricssonST15iv Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2559454 = {S29911, 140, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; ST15i Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2642366 = {S29912, 140, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-cr; ST15a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2764305 = {S29913, 140, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; ST15a Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2807809 = {S29914, 153, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; hr-hr; SonyEricssonST15iv Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2813544 = {S29915, 140, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; ST15a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2878035 = {S29916, 139, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; ST15i Build/4.0.A.2.377) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3117500 = {S29917, 150, { &P15147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fa-; SonyEricssonST15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2237434 = {S29918, 136, { &P18113,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XT912 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257931 = {S29919, 136, { &P18113,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT912 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2430979 = {S29920, 150, { &P18113,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2486585 = {S29921, 148, { &P18113,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/Fat-Free Bean) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2531769 = {S29922, 151, { &P18113,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2570879 = {S29923, 73, { &P18113,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16)*/
const Device UA2601059 = {S29924, 144, { &P18113,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2658313 = {S29925, 143, { &P18113,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/BATAKANG) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659335 = {S29926, 151, { &P18113,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2667917 = {S29927, 149, { &P18113,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2733206 = {S29928, 157, { &P18113,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30*/
const Device UA2992131 = {S29929, 163, { &P18113,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/Breakneck Blur 98.72.16 (r7)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3020092 = {S29930, 144, { &P18113,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/Blurry JB) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2770268 = {S29931, 133, { &P15767,&P19826,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B329 Safari/8536.25*/
const Device UA2772835 = {S29932, 133, { &P15767,&P19826,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B146 Safari/8536.25*/
const Device UA2772841 = {S29933, 131, { &P15767,&P19826,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B141 Safari/8536.25*/
const Device UA2772842 = {S29934, 133, { &P15767,&P19826,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B147 Safari/8536.25*/
const Device UA2670730 = {S29935, 138, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300T Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673067 = {S29936, 137, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-i9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2674743 = {S29937, 130, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2675066 = {S29938, 138, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I93000 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675840 = {S29939, 130, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-I535 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676014 = {S29940, 137, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676023 = {S29941, 137, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676090 = {S29942, 130, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676102 = {S29943, 137, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676244 = {S29944, 137, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676269 = {S29945, 137, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676285 = {S29946, 130, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676427 = {S29947, 138, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676673 = {S29948, 130, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676677 = {S29949, 137, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-I535 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676839 = {S29950, 138, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999V Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677223 = {S29951, 131, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2678413 = {S29952, 137, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; GT-I9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708119 = {S29953, 131, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999V Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2713808 = {S29954, 134, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; I9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735620 = {S29955, 130, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2781605 = {S29956, 134, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; I9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2794944 = {S29957, 132, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; i9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2970917 = {S29958, 138, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999V Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2991510 = {S29959, 135, { &P18283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SC-06D Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2431784 = {S29960, 86, { &P17968,&P17979,&P18525,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/6.0.24556/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA210151 = {S29961, 142, { &P16479,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-R720 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA337067 = {S29962, 66, { &P16479,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SCH-R720 Build/GINGERBREAD)*/
const Device UA369686 = {S29963, 77, { &P16479,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.4; en_US; SCH-R720 Build/GINGERBREAD.EH02)*/
const Device UA571327 = {S29964, 149, { &P16479,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-R720 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA634154 = {S29965, 142, { &P16479,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-R720 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA677472 = {S29966, 142, { &P16479,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SCH-R720 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA806349 = {S29967, 140, { &P16479,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; -us; SCH-R720 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023414 = {S29968, 66, { &P16479,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-R720 Build/GINGERBREAD)*/
const Device UA2024167 = {S29969, 168, { &P16479,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en_us; SCH-R720 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)*/
const Device UA2024802 = {S29970, 130, { &P16479,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-R720 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2025770 = {S29971, 34, { &P16479,&P5629,&P15760,&P18092 }};	/*Useragent - Android/1.0 (SCH-R720 GINGERBREAD)*/
const Device UA2027287 = {S29972, 145, { &P16479,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-R720 Build/V6_1.2.60412.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2521521 = {S29973, 140, { &P16479,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-R720 Build/Viperdink) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2834499 = {S29974, 140, { &P16479,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; -us; SCH-R720 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2989156 = {S29975, 144, { &P16479,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-R720 Build/Chicken-CTMod) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3070184 = {S29976, 222, { &P16479,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-R720 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Android MetroPCS androidAppVersion/2.5.10 AndroidWebkitWrapper BillingSupported*/
const Device UA2434557 = {S29977, 151, { &P12280,&P18117,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9B206 Safari/7534.48.3*/
const Device UA2435112 = {S29978, 149, { &P12280,&P18117,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9B176 Safari/7534.48.3*/
const Device UA2436052 = {S29979, 151, { &P12280,&P18117,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9B208 Safari/7534.48.3*/
const Device UA2436268 = {S29980, 149, { &P12280,&P18117,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9B179 Safari/7534.48.3*/
const Device UA2557405 = {S29981, 149, { &P12280,&P18117,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9A405 Safari/7534.48.3*/
const Device UA97809 = {S29982, 118, { &P15364,&P5690,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.2.149.29 Safari/525.13*/
const Device UA111090 = {S29983, 117, { &P15364,&P5690,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.3.154.0 Safari/525.19*/
const Device UA122529 = {S29984, 115, { &P15364,&P5690,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.A.B.C Safari/525.13*/
const Device UA259516 = {S29985, 118, { &P15364,&P5690,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.4.154.29 Safari/525.19*/
const Device UA382471 = {S29986, 118, { &P15364,&P5690,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.2.149.27 Safari/525.13*/
const Device UA416447 = {S29987, 111, { &P15364,&P5690,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.2.149.27 Safari/525.13*/
const Device UA857298 = {S29988, 117, { &P15364,&P5690,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.3.154.6 Safari/525.19*/
const Device UA1867460 = {S29989, 118, { &P15364,&P5690,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.4.154.23 Safari/525.19*/
const Device UA1886119 = {S29990, 96, { &P15364,&P5690,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/0.0.0.0 Safari/534.24*/
const Device UA2131467 = {S29991, 115, { &P15364,&P5690,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; ru-RU) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.A.B.C Safari/525.13*/
const Device UA2139314 = {S29992, 115, { &P15364,&P5690,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; de) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.4.154.29 Safari/525.19*/
const Device UA2212028 = {S29993, 115, { &P15364,&P5690,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; de) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.2.149.27 Safari/525.13*/
const Device UA2278325 = {S29994, 115, { &P15364,&P5690,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/0.0.0.0 Safari/534.16*/
const Device UA2316793 = {S29995, 115, { &P15364,&P5690,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; es-ES) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.A.B.C Safari/525.13*/
const Device UA2332897 = {S29996, 115, { &P15364,&P5690,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; fr-FR) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.A.B.C Safari/525.13*/
const Device UA2337372 = {S29997, 115, { &P15364,&P5690,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.A.B.C Safari/525.13*/
const Device UA2441229 = {S29998, 191, { &P15364,&P5690,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.A.B.C Safari/525.13 AppEngine-Google; (+http://code.google.com/appengine; appid: s~cookn-cloud)*/
const Device UA2775357 = {S29999, 120, { &P15364,&P5690,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; AppleWebKit/525.13; KHTML?? like Gecko; Chrome/0.2.149.27 Safari/525.13)*/
const Device UA2815321 = {S30000, 109, { &P15364,&P5690,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1;) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.A.B.C Safari/525.13*/
const Device UA33312 = {S30001, 163, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LG-P999 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA42268 = {S30002, 162, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;LG-P999 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA42271 = {S30003, 162, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LG-P999 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA42273 = {S30004, 162, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; LG-P999 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA42275 = {S30005, 162, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LG-P999Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA42278 = {S30006, 162, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.3; en-us; LG-P999 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA68332 = {S30007, 136, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P999 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA69268 = {S30008, 136, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P999 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA238248 = {S30009, 59, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; LG-P999 Build/GRI40)*/
const Device UA242229 = {S30010, 135, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P999 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA248859 = {S30011, 135, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P999 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA252013 = {S30012, 136, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P999 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA348469 = {S30013, 163, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P999 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA369175 = {S30014, 157, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P999 Build/FRG83G; LG-P990 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA545235 = {S30015, 136, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LG-P999 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA565590 = {S30016, 142, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LG-P999 Build/GRI40) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA623504 = {S30017, 59, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P999 Build/GRJ22)*/
const Device UA626084 = {S30018, 163, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P999 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA658722 = {S30019, 59, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; LG-P999 Build/GRI40)*/
const Device UA670981 = {S30020, 138, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P999 Build/SINGH-UI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA685245 = {S30021, 162, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P999 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA687096 = {S30022, 162, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;LG-P999 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA787955 = {S30023, 162, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; LG-P999 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA852401 = {S30024, 162, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; LG-P999 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA878566 = {S30025, 162, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P999Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2022230 = {S30026, 165, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P999 Build/GRJB54C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2023006 = {S30027, 133, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; LG-P999 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025140 = {S30028, 134, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P999 Build/RG2x) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034449 = {S30029, 90, { &P12896,&P5629,&P15760,&P18092 }};	/*Useragent - com.google.android.youtube/2.4.4(Linux; U; Android 2.3.4; en_US; LG-P999 Build/GRJ22) gzip*/
const Device UA219051 = {S30030, 117, { &P17595,&P17296,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1*/
const Device UA219625 = {S30031, 117, { &P17595,&P17296,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1*/
const Device UA221034 = {S30032, 117, { &P17595,&P17296,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1*/
const Device UA247982 = {S30033, 117, { &P17595,&P17296,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1*/
const Device UA247985 = {S30034, 117, { &P17595,&P17296,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1*/
const Device UA331939 = {S30035, 116, { &P17595,&P17296,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1*/
const Device UA333715 = {S30036, 117, { &P17595,&P17296,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1*/
const Device UA334625 = {S30037, 116, { &P17595,&P17296,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1*/
const Device UA409433 = {S30038, 115, { &P17595,&P17296,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1*/
const Device UA715235 = {S30039, 116, { &P17595,&P17296,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1*/
const Device UA2258255 = {S30040, 117, { &P17595,&P17296,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.1 (KHTML	 like Gecko) Chrome/13.0.782.107 Safari/535.1*/
const Device UA2674790 = {S30041, 129, { &P17595,&P17296,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/13.0.648.205 Safari/534.16*/
const Device UA2674808 = {S30042, 129, { &P17595,&P17296,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/13.0.648.205 Safari/534.16*/
const Device UA2718690 = {S30043, 129, { &P17595,&P17296,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/13.0.648.205 Safari/534.16*/
const Device UA603392 = {S30044, 155, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID RAZR Build/6.7.1-68_DHD-15_M4-7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA608689 = {S30045, 140, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDRAZR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA611387 = {S30046, 159, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID RAZR 4G Build/6.7.1-68_DHD-15_M4-11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616711 = {S30047, 158, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID RAZR 4G Build/6.7.1-68_DHD-15_M4-7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA714456 = {S30048, 159, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID RAZR 4G Build/6.7.1-68_DHD-15_M4-17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA740640 = {S30049, 156, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID RAZR Build/6.7.1-68_DHD-15_M4-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA748241 = {S30050, 141, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Droid Razr Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA790359 = {S30051, 159, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID RAZR 4G Build/6.7.1-68_DHD-15_M4-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA790907 = {S30052, 136, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT912 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA843898 = {S30053, 141, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID RAZR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA855908 = {S30054, 156, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID RAZR Build/6.7.1-68_DHD-15_M4-11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA860433 = {S30055, 136, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT912 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA895986 = {S30056, 160, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR 4G Build/6.7.2-180_DHD-16_M4-26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA902170 = {S30057, 160, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR 4G Build/6.7.2-180_DHD-16_M4-29) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA903178 = {S30058, 160, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR 4G Build/6.7.2-180_DHD-16_M4-30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA906688 = {S30059, 157, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA907471 = {S30060, 160, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR 4G Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA908049 = {S30061, 79, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; DROID RAZR Build/6.7.2-180_DHD-16_M4-31)*/
const Device UA910083 = {S30062, 150, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA912478 = {S30063, 160, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR 4G Build/6.7.2-180_DHD-16_M4-34) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA924225 = {S30064, 157, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-35) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA924756 = {S30065, 160, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR 4G Build/6.7.2-180_DHD-16_M4-35) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1954759 = {S30066, 155, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1955079 = {S30067, 146, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/Arctic Beta) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1991377 = {S30068, 157, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-34) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028569 = {S30069, 77, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; DROID RAZR Build/6.7.1-68_DHD-15_M4-7)*/
const Device UA2029150 = {S30070, 157, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-29) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030265 = {S30071, 157, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031462 = {S30072, 160, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR 4G Build/6.7.2-180_DHD-16_M4-28) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2201969 = {S30073, 163, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30*/
const Device UA2223439 = {S30074, 160, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR 4G Build/6.7.2-180_DHD-16_M4-36) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2372499 = {S30075, 160, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR 4G Build/6.7.2-180_DHD-16_M4-24) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2412681 = {S30076, 106, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 4.0.4; MOTOROLA DROID RAZR Build/ICE_CREAM_SANDWICH_MR1);*/
const Device UA2463773 = {S30077, 158, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; DROID RAZR 4G Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2809178 = {S30078, 176, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - MOT-DROID RAZR/ Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) 1617 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2829232 = {S30079, 132, { &P18113,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT912 Build/B4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA237082 = {S30080, 117, { &P17595,&P17297,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.215 Safari/535.1*/
const Device UA237917 = {S30081, 117, { &P17595,&P17297,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.215 Safari/535.1*/
const Device UA247041 = {S30082, 117, { &P17595,&P17297,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1*/
const Device UA381940 = {S30083, 117, { &P17595,&P17297,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1*/
const Device UA734789 = {S30084, 117, { &P17595,&P17297,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1*/
const Device UA2200131 = {S30085, 117, { &P17595,&P17297,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1*/
const Device UA86114 = {S30086, 15, { &P18254,&P7480,&P17470,&P18091 }};	/*Useragent - ArchitextSpider*/
const Device UA93982 = {S30087, 22, { &P18254,&P7480,&P17470,&P18091 }};	/*Useragent - Spider.TerraNautic.net*/
const Device UA95423 = {S30088, 34, { &P18254,&P7480,&P17470,&P18091 }};	/*Useragent - ZoomSpider - wrensoft.com [ZSEBOT]*/
const Device UA98145 = {S30089, 26, { &P18254,&P7480,&P17470,&P18091 }};	/*Useragent - Comodo-Certificates-Spider*/
const Device UA102180 = {S30090, 16, { &P18254,&P7480,&P17470,&P18091 }};	/*Useragent - Xaldon WebSpider*/
const Device UA105113 = {S30091, 14, { &P18254,&P7480,&P17470,&P18091 }};	/*Useragent - JOC Web Spider*/
const Device UA121213 = {S30092, 71, { &P18254,&P7480,&P17470,&P18091 }};	/*Useragent - Sogou Pic Spider/3.0(+http://www.sogou.com/docs/help/webmasters.htm#07)*/
const Device UA121820 = {S30093, 39, { &P18254,&P7480,&P17470,&P18091 }};	/*Useragent - Liquida Spider 1.0 +http://liquida.com/*/
const Device UA125672 = {S30094, 15, { &P18254,&P7480,&P17470,&P18091 }};	/*Useragent - Test Spider 0.2*/
const Device UA166235 = {S30095, 14, { &P18254,&P7480,&P17470,&P18091 }};	/*Useragent - QBikSpider/2.0*/
const Device UA169682 = {S30096, 14, { &P18254,&P7480,&P17470,&P18091 }};	/*Useragent - WCH Web Spider*/
const Device UA169935 = {S30097, 47, { &P18254,&P7480,&P17470,&P18091 }};	/*Useragent - DaylifeFeedFetcher/1.0 (http://www.daylife.com)*/
const Device UA194613 = {S30098, 16, { &P18254,&P7480,&P17470,&P18091 }};	/*Useragent - NodejsSpider/1.0*/
const Device UA201731 = {S30099, 14, { &P18254,&P7480,&P17470,&P18091 }};	/*Useragent - DLE_Spider.exe*/
const Device UA299873 = {S30100, 10, { &P18254,&P7480,&P17470,&P18091 }};	/*Useragent - XYZ Spider*/
const Device UA347547 = {S30101, 109, { &P18254,&P7480,&P17470,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; SpiderLing (a SPIDER for LINGustic research); http://nlp.fi.muni.cz/projects/biwec/)*/
const Device UA601625 = {S30102, 34, { &P18254,&P7480,&P17470,&P18091 }};	/*Useragent - Blogshares Spiders (Renewed V1.65)*/
const Device UA627003 = {S30103, 72, { &P18254,&P7480,&P17470,&P18091 }};	/*Useragent - Sogou News Spider/4.0(+http://www.sogou.com/docs/help/webmasters.htm#07)*/
const Device UA771679 = {S30104, 45, { &P18254,&P7480,&P17470,&P18091 }};	/*Useragent - Sogou Mobile Spider1.0 (http://wap.sogou.com)*/
const Device UA1898729 = {S30105, 16, { &P18254,&P7480,&P17470,&P18091 }};	/*Useragent - Sogou Web Spider*/
const Device UA2197102 = {S30106, 11, { &P18254,&P7480,&P17470,&P18091 }};	/*Useragent - EasouSpider*/
const Device UA2367472 = {S30107, 144, { &P15767,&P19826,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B5105c Safari/7534.48.3*/
const Device UA2488894 = {S30108, 142, { &P15767,&P19826,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B141 Safari/7534.48.3*/
const Device UA2532751 = {S30109, 144, { &P15767,&P19826,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B146 Safari/7534.48.3*/
const Device UA2565783 = {S30110, 144, { &P15767,&P19826,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B147 Safari/7534.48.3*/
const Device UA2585299 = {S30111, 144, { &P15767,&P19826,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B329 Safari/7534.48.3*/
const Device UA238806 = {S30112, 63, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; en) Presto/2.9.168 Version/11.51*/
const Device UA242512 = {S30113, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.9.168 Version/11.51*/
const Device UA242846 = {S30114, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.9.168 Version/11.51*/
const Device UA242851 = {S30115, 86, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.9.168 Version/11.51*/
const Device UA242952 = {S30116, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.9.168 Version/11.51*/
const Device UA243252 = {S30117, 86, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.9.168 Version/11.51*/
const Device UA243741 = {S30118, 86, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.9.168 Version/11.51*/
const Device UA243748 = {S30119, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.9.168 Version/11.51*/
const Device UA243911 = {S30120, 86, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.9.168 Version/11.51*/
const Device UA243963 = {S30121, 100, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.9.168 Version/11.51,gzip(gfe) (via translate.google.com)*/
const Device UA244128 = {S30122, 86, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.9.168 Version/11.51*/
const Device UA244155 = {S30123, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.9.168 Version/11.51*/
const Device UA247704 = {S30124, 66, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; en) Opera 11.51*/
const Device UA248359 = {S30125, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4841); ru) Presto/2.9.168 Version/11.51*/
const Device UA250051 = {S30126, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4848); ru) Presto/2.9.168 Version/11.51*/
const Device UA336403 = {S30127, 86, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.9.168 Version/11.51*/
const Device UA336554 = {S30128, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.9.168 Version/11.51*/
const Device UA336578 = {S30129, 86, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.9.168 Version/11.51*/
const Device UA336647 = {S30130, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.9.168 Version/11.51*/
const Device UA337596 = {S30131, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4661); ru) Presto/2.9.168 Version/11.51*/
const Device UA338060 = {S30132, 86, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03278); ru) Presto/2.9.168 Version/11.51*/
const Device UA338172 = {S30133, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4110); ru) Presto/2.9.168 Version/11.51*/
const Device UA338192 = {S30134, 86, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.9.168 Version/11.51*/
const Device UA342120 = {S30135, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4832); ru) Presto/2.9.168 Version/11.51*/
const Device UA344211 = {S30136, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4871); ru) Presto/2.9.168 Version/11.51*/
const Device UA344512 = {S30137, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.9.168 Version/11.51*/
const Device UA567833 = {S30138, 86, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03638); ru) Presto/2.9.168 Version/11.51*/
const Device UA608713 = {S30139, 86, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.9.168 Version/11.51*/
const Device UA665744 = {S30140, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.9.168 Version/11.51*/
const Device UA852351 = {S30141, 86, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03780); ru) Presto/2.9.168 Version/11.51*/
const Device UA868563 = {S30142, 86, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.9.168 Version/11.51*/
const Device UA1962889 = {S30143, 86, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.9.168 Version/11.51*/
const Device UA1964173 = {S30144, 86, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.9.168 Version/11.51*/
const Device UA2143426 = {S30145, 72, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; ru; rv:2.0) Gecko/X Firefox/4.0 Opera 11.51*/
const Device UA2166968 = {S30146, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.9.168 Version/11.51*/
const Device UA2229357 = {S30147, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.9.168 Version/11.51*/
const Device UA2262786 = {S30148, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.9.168 Version/11.51*/
const Device UA2262806 = {S30149, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.9.168 Version/11.51*/
const Device UA2325292 = {S30150, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.9.168 Version/11.51*/
const Device UA2332895 = {S30151, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.9.168 Version/11.51*/
const Device UA2353349 = {S30152, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.9.168 Version/11.51*/
const Device UA2393355 = {S30153, 86, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5267); ru) Presto/2.9.168 Version/11.51*/
const Device UA2411540 = {S30154, 86, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5265); ru) Presto/2.9.168 Version/11.51*/
const Device UA2482157 = {S30155, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.9.168 Version/11.51*/
const Device UA2529722 = {S30156, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.9.168 Version/11.51*/
const Device UA2652925 = {S30157, 73, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.9.168 Version/11.51,gzip(gfe)*/
const Device UA2694955 = {S30158, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.9.168 Version/11.51*/
const Device UA2709197 = {S30159, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.9.168 Version/11.51*/
const Device UA2727228 = {S30160, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.9.168 Version/11.51*/
const Device UA2839103 = {S30161, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5966); ru) Presto/2.9.168 Version/11.51*/
const Device UA2869820 = {S30162, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6578); ru) Presto/2.9.168 Version/11.51*/
const Device UA2994176 = {S30163, 85, { &P15364,&P7133,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6089); ru) Presto/2.9.168 Version/11.51*/
const Device UA4227 = {S30164, 155, { &P14358,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE63-3/200.21.012; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA24510 = {S30165, 155, { &P14358,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE63-1/510.21.010; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA33984 = {S30166, 185, { &P14358,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE63-1/500U; Series60/3.1 NokiaE63-1/500.21.009; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA33989 = {S30167, 185, { &P14358,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokianOS/9.2; U; Series60/3.1 NokiaE63-1/500.21.009; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA61667 = {S30168, 155, { &P14358,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE63-2/200.21.012; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA61988 = {S30169, 153, { &P14358,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE63/200.21.005 (SymbianOS/9.2; U; Series60/3.1 Mozilla/5.0; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA68478 = {S30170, 33, { &P14358,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE63/UCBrowser7.7.0.81/28/800*/
const Device UA103350 = {S30171, 8, { &P14358,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE63*/
const Device UA2193631 = {S30172, 125, { &P15364,&P17285,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/22.0.1229.79) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4*/
const Device UA2194064 = {S30173, 132, { &P15364,&P17285,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/22.0.1229.79) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4*/
const Device UA2201648 = {S30174, 125, { &P15364,&P17285,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/22.0.1229.92) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.92 Safari/537.4*/
const Device UA2201753 = {S30175, 132, { &P15364,&P17285,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/22.0.1229.92) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.92 Safari/537.4*/
const Device UA2203737 = {S30176, 125, { &P15364,&P17285,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/22.0.1229.94) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2203804 = {S30177, 89, { &P15364,&P17285,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/22.0.1229.94)*/
const Device UA2203859 = {S30178, 96, { &P15364,&P17285,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/22.0.1229.94)*/
const Device UA2203915 = {S30179, 182, { &P15364,&P17285,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/22.0.1229.94; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2203990 = {S30180, 161, { &P15364,&P17285,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/22.0.1229.94; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2203995 = {S30181, 132, { &P15364,&P17285,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/22.0.1229.94) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2231775 = {S30182, 202, { &P15364,&P17285,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.4; chromeframe/22.0.1229.94; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2232320 = {S30183, 189, { &P15364,&P17285,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; chromeframe/22.0.1229.94; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2238339 = {S30184, 195, { &P15364,&P17285,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; chromeframe/22.0.1229.94; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2238614 = {S30185, 174, { &P15364,&P17285,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.4; chromeframe/22.0.1229.94; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2245123 = {S30186, 240, { &P15364,&P17285,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.4; chromeframe/22.0.1229.94; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2258889 = {S30187, 125, { &P15364,&P17285,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/22.0.1229.96) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.96 Safari/537.4*/
const Device UA3032600 = {S30188, 71, { &P15364,&P7133,&P19806,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:17.0) Gecko/X Thunderbird/17.0.8*/
const Device UA99082 = {S30189, 116, { &P15364,&P5690,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.38 Safari/532.0*/
const Device UA259524 = {S30190, 116, { &P15364,&P5690,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.27 Safari/532.0*/
const Device UA1874668 = {S30191, 113, { &P15364,&P5690,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.8 (KHTML, like Gecko) Chrome/3.1.0.0 Safari/532.8*/
const Device UA2293966 = {S30192, 102, { &P15364,&P5690,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.0 (KHTML,like Gecko) Chrome/3.0.195.27*/
const Device UA2741856 = {S30193, 139, { &P15364,&P5690,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; Valve Source Client; ) AppleWebKit/532.1 (KHTML, like Gecko) Chrome/3.0.195.24 Safari/532.1*/
const Device UA99890 = {S30194, 11, { &P15364,&P17017,&P18245,&P18092 }};	/*Useragent - LongURL API*/
const Device UA538851 = {S30195, 59, { &P15364,&P17017,&P18245,&P18092 }};	/*Useragent - UniversalFeedParser/4.2-pre-316-svn +http://feedparser.org/*/
const Device UA603334 = {S30196, 64, { &P15364,&P17017,&P18245,&P18092 }};	/*Useragent - UniversalFeedParser/5.1.1 +https://code.google.com/p/feedparser/*/
const Device UA754831 = {S30197, 86, { &P15364,&P17017,&P18245,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; grub-client-0.3.x; Crawl your own stuff with http://grub.org)*/
const Device UA884543 = {S30198, 91, { &P15364,&P17017,&P18245,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; Ask Jeeves/Teoma; http://sp.ask.com/docs/about/tech_crawling.html)*/
const Device UA901728 = {S30199, 72, { &P15364,&P17017,&P18245,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Genieo/1.0 http://www.genieo.com/webfilter.html*/
const Device UA902620 = {S30200, 73, { &P15364,&P17017,&P18245,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Genieo/1.0 http://www.genieo.com/webfilter.html)*/
const Device UA1896268 = {S30201, 74, { &P15364,&P17017,&P18245,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Crawling jpeg; http://www.yama.info.waseda.ac.jp)*/
const Device UA1897671 = {S30202, 105, { &P15364,&P17017,&P18245,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; zermelo +http://www.powerset.com) [email:paul@page-store.com,crawl@powerset.com]*/
const Device UA1897837 = {S30203, 82, { &P15364,&P17017,&P18245,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; grub-client-0.2.x; Crawl your stuff with http://grub.org)*/
const Device UA2391466 = {S30204, 42, { &P15364,&P17017,&P18245,&P18092 }};	/*Useragent - RockmeltEmbedService (http://rockmelt.com)*/
const Device UA445847 = {S30205, 69, { &P15364,&P18110,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:2.0b12pre) Gecko/X Firefox/4.0b12pre*/
const Device UA2009050 = {S30206, 94, { &P15364,&P18110,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:2.0.1) Gecko/X Firefox/4.0.1 Iceweasel/4.0.1 WebMoney Advisor*/
const Device UA2641801 = {S30207, 59, { &P15364,&P18110,&P17203,&P18092 }};	/*Useragent - a/5.0 (X11; Linux x86_64; rv:2.0b11) Gecko/X Firefox/4.0b11*/
const Device UA2891565 = {S30208, 90, { &P15364,&P18110,&P17203,&P18092 }};	/*Useragent - User-Agent: Mozilla/5.0 (X11; U; Linux x86_64; en-GB; rv:1.9.2.3) Gecko/X Gentoo Firefox/4*/
const Device UA2390622 = {S30209, 67, { &P15364,&P18118,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:18.0) Gecko/X Firefox/18.0*/
const Device UA640621 = {S30210, 63, { &P15364,&P7133,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:11.0) Gecko Firefox/11.0*/
const Device UA726116 = {S30211, 78, { &P15364,&P7133,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:11.0) Gecko/X Firefox/11.0 Firefox/11.0*/
const Device UA1962880 = {S30212, 70, { &P15364,&P7133,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; rv:11.0) Gecko/X Firefox/11.0*/
const Device UA1967348 = {S30213, 79, { &P15364,&P7133,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.1; Win64; x64; rv:11.0) Gecko/X Firefox/11.0*/
const Device UA2700456 = {S30214, 77, { &P15364,&P7133,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 Panguso Transcoder (Windows NT 6.1; rv:11.0) Gecko/X Firefox/11.0*/
const Device UA92919 = {S30215, 169, { &P11628,&P17484,&P3625,&P18089 }};	/*Useragent - SAMSUNG-SGH-E250/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0 (compatible; Googlebot-Mobile/2.1; +http://www.google.com/bot.html)*/
const Device UA3092125 = {S30216, 167, { &P11628,&P17484,&P3625,&P18089 }};	/*Useragent - SAMSUNG-SGH-E250/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0 (compatible; Googlebot-Mobile/2.1; +http://www.g;c2VvLWNyYXdsZXI=*/
const Device UA3100019 = {S30217, 169, { &P11628,&P17484,&P3625,&P18089 }};	/*Useragent - SAMSUNG-SGH-E250/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0 (compatible; Googlebot-Mobile/2.1;  http://www.google.com/bot.html)*/
const Device UA79923 = {S30218, 82, { &P15364,&P17285,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/10*/
const Device UA174063 = {S30219, 120, { &P15364,&P17285,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.151 Safari/534.16*/
const Device UA640847 = {S30220, 157, { &P15364,&P17285,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16,gzip(gfe) (via translate.google.com)*/
const Device UA2674806 = {S30221, 108, { &P15364,&P17285,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.64 Safari/537.31*/
const Device UA2675005 = {S30222, 101, { &P15364,&P17285,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.64 Safari/537.31*/
const Device UA2740592 = {S30223, 130, { &P18645,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.85 Safari/537.36*/
const Device UA2741598 = {S30224, 145, { &P18645,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2741702 = {S30225, 137, { &P18645,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2741700 = {S30226, 138, { &P18645,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747M Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2741730 = {S30227, 137, { &P18645,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2752817 = {S30228, 138, { &P18645,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.74 Safari/537.36*/
const Device UA2753005 = {S30229, 138, { &P18645,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753270 = {S30230, 138, { &P18645,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753279 = {S30231, 137, { &P18645,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753280 = {S30232, 138, { &P18645,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753331 = {S30233, 138, { &P18645,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530C Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754079 = {S30234, 137, { &P18645,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SPH-L710 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754641 = {S30235, 137, { &P18645,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754691 = {S30236, 138, { &P18645,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754853 = {S30237, 138, { &P18645,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-R530U Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755048 = {S30238, 138, { &P18645,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755388 = {S30239, 145, { &P18645,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769039 = {S30240, 137, { &P18645,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769543 = {S30241, 138, { &P18645,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769691 = {S30242, 138, { &P18645,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305N Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2770041 = {S30243, 138, { &P18645,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879265 = {S30244, 138, { &P18645,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA93036 = {S30245, 87, { &P18254,&P7480,&P18245,&P18090 }};	/*Useragent - Mozilla/5.0 (compatible; Yahoo! Slurp/3.0; http://help.yahoo.com/help/us/ysearch/slurp)*/
const Device UA98151 = {S30246, 83, { &P18254,&P7480,&P18245,&P18090 }};	/*Useragent - Mozilla/5.0 (compatible; Yahoo! Slurp; http://help.yahoo.com/help/us/ysearch/slurp)*/
const Device UA191038 = {S30247, 82, { &P18254,&P7480,&P18245,&P18090 }};	/*Useragent - Mozilla/5.0 (compatible; Yahoo! Slurp;http://help.yahoo.com/help/us/ysearch/slurp)*/
const Device UA382339 = {S30248, 88, { &P18254,&P7480,&P18245,&P18090 }};	/*Useragent - Mozilla/5.0 (compatible; Yahoo! Slurp/3.0 ; http://help.yahoo.com/help/us/ysearch/slurp)*/
const Device UA444555 = {S30249, 92, { &P18254,&P7480,&P18245,&P18090 }};	/*Useragent - Mozilla/5.0 (compatible; Yahoo! Slurp/3.0 Asia; http://help.yahoo.com/help/us/ysearch/slurp)*/
const Device UA459926 = {S30250, 86, { &P18254,&P7480,&P18245,&P18090 }};	/*Useragent - Mozilla/5.0 (compatible; Yahoo! DE Slurp; http://help.yahoo.com/help/us/ysearch/slurp)*/
const Device UA501861 = {S30251, 88, { &P18254,&P7480,&P18245,&P18090 }};	/*Useragent - Mozilla/5.0 (compatible; Yahoo! Slurp Asia; http://help.yahoo.com/help/us/ysearch/slurp)*/
const Device UA517646 = {S30252, 76, { &P18254,&P7480,&P18245,&P18090 }};	/*Useragent - Mozilla/5.0 (Slurp/si; slurp@inktomi.com; http://www.inktomi.com/slurp.html)*/
const Device UA543528 = {S30253, 77, { &P18254,&P7480,&P18245,&P18090 }};	/*Useragent - Mozilla/5.0 (Slurp/cat; slurp@inktomi.com; http://www.inktomi.com/slurp.html)*/
const Device UA902791 = {S30254, 82, { &P18254,&P7480,&P18245,&P18090 }};	/*Useragent - Mozilla/5.0 (compatible; Yahoo! Nano; http://help.yahoo.com/help/us/ysearch/slurp)*/
const Device UA1893600 = {S30255, 80, { &P18254,&P7480,&P18245,&P18090 }};	/*Useragent - Mozilla/3.0 (Slurp.so/Goo; slurp@inktomi.com; http://www.inktomi.com/slurp.html)*/
const Device UA1895382 = {S30256, 76, { &P18254,&P7480,&P18245,&P18090 }};	/*Useragent - Mozilla/3.0 (Slurp/si; slurp@inktomi.com; http://www.inktomi.com/slurp.html)*/
const Device UA1896911 = {S30257, 77, { &P18254,&P7480,&P18245,&P18090 }};	/*Useragent - Mozilla/3.0 (Slurp/cat; slurp@inktomi.com; http://www.inktomi.com/slurp.html)*/
const Device UA2126829 = {S30258, 126, { &P18254,&P7480,&P18245,&P18090 }};	/*Useragent - Mozilla/5.0 (compatible; Yahoo! Slurp/3.0; http://help.yahoo.com/help/us/ysearch/slurp),gzip(gfe) (via docs.google.com/viewer)*/
const Device UA616548 = {S30259, 141, { &P14993,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xperia Arc Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616578 = {S30260, 141, { &P14993,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; Xperia Arc Build/ITL41F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616603 = {S30261, 141, { &P14993,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Xperia Arc Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616606 = {S30262, 158, { &P14993,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; SonyEricssonXperia Arc Build/4.1.A.0.562) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA902484 = {S30263, 141, { &P14993,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT15i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2007706 = {S30264, 153, { &P14993,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyEricssonLT15i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030400 = {S30265, 63, { &P14993,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT15i Build/4.1.B.0.587)*/
const Device UA2031060 = {S30266, 154, { &P14993,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonLT15iv Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031610 = {S30267, 141, { &P14993,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT15i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2699165 = {S30268, 140, { &P14993,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; LT15i Build/4.1.B.1.13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2717090 = {S30269, 142, { &P12280,&P18432,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A551 Safari/8536.25*/
const Device UA2717092 = {S30270, 140, { &P12280,&P18432,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A405 Safari/8536.25*/
const Device UA2717572 = {S30271, 140, { &P12280,&P18432,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A403 Safari/8536.25*/
const Device UA2717579 = {S30272, 142, { &P12280,&P18432,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A523 Safari/8536.25*/
const Device UA2717583 = {S30273, 142, { &P12280,&P18432,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A525 Safari/8536.25*/
const Device UA2708567 = {S30274, 131, { &P15767,&P18432,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A406 Safari/8536.25*/
const Device UA2717093 = {S30275, 131, { &P15767,&P18432,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A403 Safari/8536.25*/
const Device UA2717091 = {S30276, 133, { &P15767,&P18432,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A523 Safari/8536.25*/
const Device UA2717569 = {S30277, 131, { &P15767,&P18432,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A407 Safari/8536.25*/
const Device UA2717575 = {S30278, 134, { &P15767,&P18432,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A8426 Safari/8536.25*/
const Device UA2717876 = {S30279, 133, { &P15767,&P18432,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A550 Safari/8536.25*/
const Device UA2717865 = {S30280, 134, { &P15767,&P18432,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A8500 Safari/8536.25*/
const Device UA2029082 = {S30281, 147, { &P18114,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer Prime TF201 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033054 = {S30282, 147, { &P18114,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer Prime TF201 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2201487 = {S30283, 76, { &P18114,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Transformer Prime TF201 Build/JRO03C)*/
const Device UA2386402 = {S30284, 147, { &P18114,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Transformer Prime TF201 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422459 = {S30285, 129, { &P18114,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; TF201 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3085599 = {S30286, 145, { &P18114,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; Transformer Prime TF201 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA348823 = {S30287, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Galaxy Nexus Build/ITL41F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA350024 = {S30288, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; Galaxy Nexus Build/ICL53C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA351190 = {S30289, 154, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Galaxy Nexus Build/I9250XWKK5_ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA352437 = {S30290, 65, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.1; Galaxy Nexus Build/ITL41F)*/
const Device UA352977 = {S30291, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; Galaxy Nexus Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA357484 = {S30292, 65, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.2; Galaxy Nexus Build/ICL53F)*/
const Device UA359050 = {S30293, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Galaxy Nexus Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA359180 = {S30294, 65, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Galaxy Nexus Build/IML74K)*/
const Device UA359256 = {S30295, 142, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Galaxy Nexus Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA359331 = {S30296, 142, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Galaxy Nexus Build/IML75) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA359619 = {S30297, 129, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Galaxy Nexus Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) Mobile Safari/535.7*/
const Device UA361782 = {S30298, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Galaxy Nexus Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA368369 = {S30299, 142, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GalaxyNexus Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA368939 = {S30300, 142, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA369680 = {S30301, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM30B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA370415 = {S30302, 96, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.youtube/3.3.0(Linux; U; Android 4.0.2; en_US; Galaxy Nexus Build/ICL53F) gzip*/
const Device UA397851 = {S30303, 142, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GalaxyNexus Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA399049 = {S30304, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM30D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA399045 = {S30305, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM30C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA428362 = {S30306, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM62B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA440681 = {S30307, 129, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76B) AppleWebKit/535.7 (KHTML, like Gecko) Mobile Safari/535.7*/
const Device UA454568 = {S30308, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA603301 = {S30309, 129, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76F) AppleWebKit/535.7 (KHTML, like Gecko) Mobile Safari/535.7*/
const Device UA609353 = {S30310, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA610057 = {S30311, 129, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76I) AppleWebKit/535.7 (KHTML, like Gecko) Mobile Safari/535.7*/
const Device UA616717 = {S30312, 141, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-us; Galaxy Nexus Build/IFL10B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616703 = {S30313, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616741 = {S30314, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616745 = {S30315, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA622482 = {S30316, 149, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Galaxy Nexus Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.1 Mobile Safari/534.30*/
const Device UA633291 = {S30317, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Galaxy Nexus Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA638662 = {S30318, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Galaxy Nexus Build/ITL41F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA656177 = {S30319, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA687777 = {S30320, 139, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.2; pt-br; Galaxy X Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA711841 = {S30321, 65, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Galaxy Nexus Build/IMM76K)*/
const Device UA745940 = {S30322, 140, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Galaxy Nexus Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA761323 = {S30323, 146, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Galaxy Nexus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA812010 = {S30324, 155, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Sprint Galaxy Nexus CDMA Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA812379 = {S30325, 150, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4 IMM76D; en-us; Galaxy Nexus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA826320 = {S30326, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/ITL41F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA843467 = {S30327, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Galaxy Nexus Build/IMM30B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA863535 = {S30328, 142, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM62) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899812 = {S30329, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM72B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA900098 = {S30330, 129, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76K) AppleWebKit/535.7 (KHTML, like Gecko) Mobile Safari/535.7*/
const Device UA904585 = {S30331, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA924360 = {S30332, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1936189 = {S30333, 146, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; GT-I9250_EUR_XX Build/ICL23D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1940967 = {S30334, 129, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; zh-cn; Galaxy Nexus Build/ITL41F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1952381 = {S30335, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1956564 = {S30336, 129, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.2; zh-cn; Galaxy Nexus Build/ICL53F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1995011 = {S30337, 134, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Galaxy Nexus Build/IMM76K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2017885 = {S30338, 139, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; Galaxy X Build/IMM76K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2020083 = {S30339, 141, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-us; Galaxy Nexus Build/IFL10C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2020675 = {S30340, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Galaxy Nexus Build/ICL25B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2020999 = {S30341, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Galaxy Nexus Build/ITL39B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2022477 = {S30342, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Galaxy Nexus Build/IML77B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2024098 = {S30343, 134, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Galaxy Nexus Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2030671 = {S30344, 129, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Galaxy Nexus Build/IMM76I) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2033271 = {S30345, 149, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2034497 = {S30346, 134, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.2; zh-cn; Galaxy Nexus Build/ICL53F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2227195 = {S30347, 146, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2383446 = {S30348, 151, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Samsung Galaxy Nexus Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2384677 = {S30349, 135, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; Galaxy Nexus Build/M7511) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2395327 = {S30350, 134, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Galaxy Nexus Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2396054 = {S30351, 141, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; Galaxy Nexus Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2413804 = {S30352, 133, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2421996 = {S30353, 133, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; Galaxy Nexus Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2444592 = {S30354, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.2; ja-jp; Galaxy Nexus Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2475088 = {S30355, 143, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2652909 = {S30356, 135, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; el-gr; Galaxy Nexus Build/M7513) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2815038 = {S30357, 142, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; Galaxy Nexus Build/IMM48) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950138 = {S30358, 136, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Galaxy Nexus Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2982908 = {S30359, 138, { &P16292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; Galaxy Nexus Build/TB27TGPS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA64674 = {S30360, 77, { &P14533,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry9630/5.0.0.975 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/105*/
const Device UA3114782 = {S30361, 89, { &P18088,&P22637,&P17369,&P18092 }};	/*Useragent - Opera/9.80 (SpreadTrum; Opera Mini/4.4.31492/31.1475; U; en) Presto/2.8.119 Version/11.10*/
const Device UA193980 = {S30362, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.0; U; en-CA) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/233.70 Safari/534.6 TouchPad/1.0*/
const Device UA202914 = {S30363, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.0; U; en-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/233.70 Safari/534.6 TouchPad/1.0*/
const Device UA208562 = {S30364, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.0; U; de-DE) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/233.70 Safari/534.6 TouchPad/1.0*/
const Device UA209153 = {S30365, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.0; U; en-GB) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/233.70 Safari/534.6 TouchPad/1.0*/
const Device UA212798 = {S30366, 137, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; en-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0*/
const Device UA212845 = {S30367, 137, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; en-GB) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0*/
const Device UA212849 = {S30368, 137, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; en-CA) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0*/
const Device UA214206 = {S30369, 137, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; de-DE) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0*/
const Device UA218939 = {S30370, 137, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.3; U; en-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.3 Safari/534.6 TouchPad/1.0*/
const Device UA220456 = {S30371, 137, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; fr-FR) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0*/
const Device UA224528 = {S30372, 137, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; en-IE) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0*/
const Device UA227921 = {S30373, 137, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; en-AU) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0*/
const Device UA250437 = {S30374, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; en-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0*/
const Device UA251537 = {S30375, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; fr-FR) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0*/
const Device UA251540 = {S30376, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; de-DE) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0*/
const Device UA251550 = {S30377, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; en-GB) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0*/
const Device UA251551 = {S30378, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; en-IE) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0*/
const Device UA251568 = {S30379, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; en-CA) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0*/
const Device UA251570 = {S30380, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; en-NZ) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0*/
const Device UA251573 = {S30381, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; es-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0*/
const Device UA251712 = {S30382, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; fr-CA) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0*/
const Device UA251715 = {S30383, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; en-AU) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0*/
const Device UA251881 = {S30384, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; en-SG) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0*/
const Device UA332348 = {S30385, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.1; U; en-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/233.72 Safari/534.6 TouchPad/1.0*/
const Device UA333320 = {S30386, 137, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; es-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0*/
const Device UA333666 = {S30387, 137, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; en-SG) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0*/
const Device UA334105 = {S30388, 137, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; en-NZ) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0*/
const Device UA334405 = {S30389, 137, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; fr-CA) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0*/
const Device UA366793 = {S30390, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; en-GB) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0*/
const Device UA366809 = {S30391, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; de-DE) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0*/
const Device UA367111 = {S30392, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; en-CA) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0*/
const Device UA367889 = {S30393, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; fr-CA) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0*/
const Device UA368266 = {S30394, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; es-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0*/
const Device UA368281 = {S30395, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; en-IE) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0*/
const Device UA380415 = {S30396, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; en-SG) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0*/
const Device UA396900 = {S30397, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; es-MX) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0*/
const Device UA396934 = {S30398, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; en-HK) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0*/
const Device UA622538 = {S30399, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; en-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0*/
const Device UA629021 = {S30400, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; fr-FR) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0*/
const Device UA629613 = {S30401, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.0; U; es-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/233.70 Safari/534.6 TouchPad/1.0*/
const Device UA630484 = {S30402, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; en-AU) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0*/
const Device UA636164 = {S30403, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; es-ES) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0*/
const Device UA647300 = {S30404, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; es-ES) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0*/
const Device UA679908 = {S30405, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; es-MX) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0*/
const Device UA681064 = {S30406, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; en-NZ) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0*/
const Device UA698256 = {S30407, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; en-MX) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0*/
const Device UA725134 = {S30408, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.4; U; zh-CN) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.76 Safari/534.6 TouchPad/1.0*/
const Device UA727050 = {S30409, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.0; U; fr-FR) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/233.70 Safari/534.6 TouchPad/1.0*/
const Device UA756818 = {S30410, 137, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; es-ES) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0*/
const Device UA764189 = {S30411, 137, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; es-MX) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0*/
const Device UA768906 = {S30412, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; it-IT) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0*/
const Device UA769118 = {S30413, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.6; U; en-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0*/
const Device UA780868 = {S30414, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.0; U; en-IE) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/233.70 Safari/534.6 TouchPad/1.0*/
const Device UA788602 = {S30415, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; en-HK) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0*/
const Device UA2217049 = {S30416, 135, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.5; U; zh-CN) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.83 Safari/534.6 TouchPad/1.0*/
const Device UA2260481 = {S30417, 137, { &P16044,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.2; U; zh-CN) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/234.40.1 Safari/534.6 TouchPad/1.0*/
const Device UA805076 = {S30418, 140, { &P18009,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_C715c Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA895311 = {S30419, 145, { &P18009,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCEVODesign4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1973828 = {S30420, 67, { &P18009,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTCEVODesign4G Build/IML74K)*/
const Device UA1995019 = {S30421, 147, { &P18009,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sprint APC715CKT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1995317 = {S30422, 138, { &P18009,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PH44100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1997193 = {S30423, 60, { &P18009,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; PH44100 Build/IML74K)*/
const Device UA1995051 = {S30424, 132, { &P23773,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N8010 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2197315 = {S30425, 61, { &P23773,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-N8010 Build/IMM76D)*/
const Device UA2257654 = {S30426, 130, { &P23773,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-N8010 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2510802 = {S30427, 141, { &P18447,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LT30p Build/9.1.A.0.489) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2510920 = {S30428, 145, { &P18447,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyLT30p Build/9.1.A.0.489) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583003 = {S30429, 145, { &P18447,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-ca; SonyLT30p Build/9.1.A.0.492) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2675824 = {S30430, 141, { &P18447,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LT30p Build/9.1.A.1.140) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2678944 = {S30431, 145, { &P18447,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyLT30p Build/9.1.A.1.140) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2732387 = {S30432, 141, { &P18447,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; LT30p Build/9.1.A.0.492) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2800687 = {S30433, 147, { &P18447,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyLT30p-o Build/9.1.A.1.141) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2863501 = {S30434, 63, { &P18447,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LT30p Build/9.1.A.1.141)*/
const Device UA2865632 = {S30435, 145, { &P18447,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyLT30p Build/9.1.A.1.141) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2872323 = {S30436, 141, { &P18447,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LT30p Build/9.1.A.1.141) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957695 = {S30437, 145, { &P18447,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; SonyLT30p Build/9.1.A.1.142) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2974967 = {S30438, 147, { &P18447,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyLT30p-o Build/9.1.A.1.145) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2995810 = {S30439, 65, { &P18447,&P18111,&P18112,&P18092 }};	/*Useragent - SonyLT30p Build/9.1.A.1.140 stagefright/1.2 (Linux;Android 4.1.2)*/
const Device UA3023951 = {S30440, 141, { &P18447,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LT30p Build/9.1.A.1.145) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3089205 = {S30441, 65, { &P18447,&P18111,&P18112,&P18092 }};	/*Useragent - SonyLT30p Build/9.1.A.1.141 stagefright/1.2 (Linux;Android 4.1.2)*/
const Device UA3107116 = {S30442, 143, { &P18447,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fa-; SonyLT30p Build/9.1.A.1.141) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2360625 = {S30443, 146, { &P18472,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; DROID RAZR HD Build/9.8.1Q_39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2423987 = {S30444, 151, { &P18472,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XT925 Build/9.8.0Q-97-XT925_VQU-19) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2483997 = {S30445, 151, { &P18472,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; RAZR HD Build/9.8.2Q-8-XT925_VQL-6) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2485006 = {S30446, 152, { &P18472,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT925 Build/9.8.2Q-8-XT925_VQL-12.2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2535459 = {S30447, 155, { &P18472,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA584016 = {S30448, 66, { &P24502,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SCH-I110 Build/GINGERBREAD)*/
const Device UA622943 = {S30449, 142, { &P24502,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I110 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA906733 = {S30450, 134, { &P24502,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I110 Build/FE2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA909678 = {S30451, 142, { &P24502,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I110 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023877 = {S30452, 66, { &P24502,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-I110 Build/GINGERBREAD)*/
const Device UA2043473 = {S30453, 134, { &P24502,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I110 Build/FH1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2432358 = {S30454, 134, { &P24502,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I110 Build/FJ1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2600323 = {S30455, 134, { &P24502,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I110 Build/GA1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2642917 = {S30456, 34, { &P24502,&P5629,&P15760,&P18092 }};	/*Useragent - Android/1.0 (SCH-I110 GINGERBREAD)*/
const Device UA89023 = {S30457, 100, { &P17737,&P17658,&P18392,&P18089 }};	/*Useragent - DoCoMo/2.0 N905i(c100;TB;W24H16) (compatible; Googlebot-Mobile/2.1; +http://www.google.com/bot.html)*/
const Device UA2981631 = {S30458, 100, { &P17737,&P17658,&P18392,&P18089 }};	/*Useragent - DoCoMo/2.0 N905i(c100;TB;W24H16) (compatible; Googlebot-Mobile/2.1;  http://www.google.com/bot.html)*/
const Device UA634615 = {S30459, 131, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19 AvantBrowser/Tri-Core*/
const Device UA738231 = {S30460, 124, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19 AvantBrowser/Tri-Core*/
const Device UA896537 = {S30461, 128, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 AvantBrowser/Tri-Core*/
const Device UA898701 = {S30462, 121, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 AvantBrowser/Tri-Core*/
const Device UA912014 = {S30463, 130, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11 AvantBrowser/Tri-Core*/
const Device UA925347 = {S30464, 123, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11 AvantBrowser/Tri-Core*/
const Device UA2008926 = {S30465, 128, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 AvantBrowser/Tri-Core*/
const Device UA2017931 = {S30466, 121, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 AvantBrowser/Tri-Core*/
const Device UA2199101 = {S30467, 114, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Avant TriCore) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2199445 = {S30468, 121, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; Avant TriCore) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2359260 = {S30469, 121, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; Avant TriCore) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.96 Safari/537.4*/
const Device UA2359274 = {S30470, 114, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Avant TriCore) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.96 Safari/537.4*/
const Device UA2431874 = {S30471, 123, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; Avant TriCore) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2431879 = {S30472, 116, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Avant TriCore) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2533975 = {S30473, 123, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; Avant TriCore) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2536983 = {S30474, 116, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Avant TriCore) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2605222 = {S30475, 117, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Avant TriCore) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2605246 = {S30476, 124, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; Avant TriCore) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2713969 = {S30477, 123, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; Avant TriCore) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2718668 = {S30478, 116, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Avant TriCore) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2877632 = {S30479, 117, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36*/
const Device UA2877690 = {S30480, 124, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36*/
const Device UA2946739 = {S30481, 116, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36*/
const Device UA2947233 = {S30482, 123, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36*/
const Device UA3083972 = {S30483, 123, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36*/
const Device UA3111763 = {S30484, 116, { &P15364,&P7133,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36*/
const Device UA616625 = {S30485, 128, { &P18282,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA683353 = {S30486, 57, { &P18282,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; A200 Build/IML74K)*/
const Device UA2364738 = {S30487, 135, { &P18282,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2444473 = {S30488, 126, { &P18282,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; A200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA453581 = {S30489, 64, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; en) Presto/2.10.229 Version/11.62*/
const Device UA453693 = {S30490, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.229 Version/11.62*/
const Device UA453698 = {S30491, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.229 Version/11.62*/
const Device UA453720 = {S30492, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.229 Version/11.62*/
const Device UA453740 = {S30493, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.229 Version/11.62*/
const Device UA453752 = {S30494, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.10.229 Version/11.62*/
const Device UA453819 = {S30495, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5288); ru) Presto/2.10.229 Version/11.62*/
const Device UA453916 = {S30496, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.229 Version/11.62*/
const Device UA453920 = {S30497, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4661); ru) Presto/2.10.229 Version/11.62*/
const Device UA453849 = {S30498, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.229 Version/11.62*/
const Device UA453961 = {S30499, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.229 Version/11.62*/
const Device UA453965 = {S30500, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.229 Version/11.62*/
const Device UA454011 = {S30501, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4930); ru) Presto/2.10.229 Version/11.62*/
const Device UA453921 = {S30502, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.229 Version/11.62*/
const Device UA454709 = {S30503, 72, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.62*/
const Device UA454718 = {S30504, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5217); ru) Presto/2.10.229 Version/11.62*/
const Device UA454934 = {S30505, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5299); ru) Presto/2.10.229 Version/11.62*/
const Device UA458076 = {S30506, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.229 Version/11.62*/
const Device UA603078 = {S30507, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.229 Version/11.62*/
const Device UA603148 = {S30508, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.229 Version/11.62*/
const Device UA603340 = {S30509, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5161); ru) Presto/2.10.229 Version/11.62*/
const Device UA604841 = {S30510, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5309); ru) Presto/2.10.229 Version/11.62*/
const Device UA605866 = {S30511, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.229 Version/11.62*/
const Device UA614872 = {S30512, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5328); ru) Presto/2.10.229 Version/11.62*/
const Device UA623013 = {S30513, 64, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.229 Version/11.62*/
const Device UA623598 = {S30514, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.229 Version/11.62*/
const Device UA623653 = {S30515, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.229 Version/11.62*/
const Device UA624234 = {S30516, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.62*/
const Device UA625505 = {S30517, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.229 Version/11.62*/
const Device UA625762 = {S30518, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.62*/
const Device UA626411 = {S30519, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.62*/
const Device UA626987 = {S30520, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.62*/
const Device UA630008 = {S30521, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5278); ru) Presto/2.10.229 Version/11.62*/
const Device UA631745 = {S30522, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.229 Version/11.62*/
const Device UA631999 = {S30523, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.229 Version/11.62*/
const Device UA633229 = {S30524, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.229 Version/11.62*/
const Device UA638499 = {S30525, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.229 Version/11.62*/
const Device UA642781 = {S30526, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.62*/
const Device UA654317 = {S30527, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.229 Version/11.62*/
const Device UA658346 = {S30528, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5218); ru) Presto/2.10.229 Version/11.62*/
const Device UA670978 = {S30529, 101, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; en) Presto/2.10.229 Version/11.62,gzip(gfe) (via translate.google.com)*/
const Device UA672798 = {S30530, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.229 Version/11.62*/
const Device UA674876 = {S30531, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.229 Version/11.62*/
const Device UA675914 = {S30532, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.229 Version/11.62*/
const Device UA679870 = {S30533, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.10.229 Version/11.62*/
const Device UA688869 = {S30534, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.229 Version/11.62*/
const Device UA699437 = {S30535, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.229 Version/11.62*/
const Device UA755506 = {S30536, 66, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; en) Opera 11.62*/
const Device UA766714 = {S30537, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.10.229 Version/11.62*/
const Device UA775158 = {S30538, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5230); ru) Presto/2.10.229 Version/11.62*/
const Device UA778507 = {S30539, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5300); ru) Presto/2.10.229 Version/11.62*/
const Device UA780385 = {S30540, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.229 Version/11.62*/
const Device UA822277 = {S30541, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.10.229 Version/11.62*/
const Device UA888052 = {S30542, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.229 Version/11.62*/
const Device UA901588 = {S30543, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.229 Version/11.62*/
const Device UA906591 = {S30544, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.229 Version/11.62*/
const Device UA1959515 = {S30545, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.229 Version/11.62*/
const Device UA1970520 = {S30546, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5667); ru) Presto/2.10.229 Version/11.62*/
const Device UA1975571 = {S30547, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.10.229 Version/11.62*/
const Device UA1984835 = {S30548, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.10.229 Version/11.62*/
const Device UA1990748 = {S30549, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03672); ru) Presto/2.10.229 Version/11.62*/
const Device UA1991149 = {S30550, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.10.229 Version/11.62*/
const Device UA1996329 = {S30551, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.10.229 Version/11.62*/
const Device UA2184217 = {S30552, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.229 Version/11.62*/
const Device UA2262504 = {S30553, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.10.229 Version/11.62*/
const Device UA2262519 = {S30554, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.10.229 Version/11.62*/
const Device UA2315337 = {S30555, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02746); ru) Presto/2.10.229 Version/11.62*/
const Device UA2332908 = {S30556, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03399); ru) Presto/2.10.229 Version/11.62*/
const Device UA2402810 = {S30557, 74, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.229 Version/11.62,gzip(gfe)*/
const Device UA2412154 = {S30558, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02552); ru) Presto/2.10.229 Version/11.62*/
const Device UA2443878 = {S30559, 87, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02652); ru) Presto/2.10.229 Version/11.62*/
const Device UA2456323 = {S30560, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.229 Version/11.62*/
const Device UA2459142 = {S30561, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6011); ru) Presto/2.10.229 Version/11.62*/
const Device UA2482059 = {S30562, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.229 Version/11.62*/
const Device UA2506048 = {S30563, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.229 Version/11.62*/
const Device UA2594330 = {S30564, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.229 Version/11.62*/
const Device UA2654316 = {S30565, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5880); ru) Presto/2.10.229 Version/11.62*/
const Device UA2706127 = {S30566, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.229 Version/11.62*/
const Device UA2957711 = {S30567, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.229 Version/11.62*/
const Device UA3043069 = {S30568, 86, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5784); ru) Presto/2.10.229 Version/11.62*/
const Device UA3043512 = {S30569, 133, { &P15364,&P5690,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; Edition IBIS; Trident/4.0; zh-cn) Presto/2.10.229 Version/11.62; zh-cn) Presto/2.10.229 Version/11.62)*/
const Device UA3095658 = {S30570, 144, { &P12280,&P22882,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A4449d Safari/8536.25*/
const Device UA3095930 = {S30571, 142, { &P12280,&P22882,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A465 Safari/8536.25*/
const Device UA3109641 = {S30572, 144, { &P12280,&P22882,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A4449a Safari/8536.25*/
const Device UA3115190 = {S30573, 142, { &P12280,&P22882,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A466 Safari/8536.25*/
const Device UA3115355 = {S30574, 144, { &P12280,&P22882,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A501 Safari/8536.25*/
const Device UA3115405 = {S30575, 145, { &P12280,&P22882,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A470a Safari/8536.25*/
const Device UA2675386 = {S30576, 65, { &P15364,&P5690,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64; rv:21.0) Gecko/X Firefox/21.0*/
const Device UA2676388 = {S30577, 65, { &P15364,&P5690,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:21.0) Gecko/X Firefox/21.0*/
const Device UA2784862 = {S30578, 78, { &P15364,&P5690,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:21.0) Gecko/X Firefox/21.0 IceDragon/21.0.1.15*/
const Device UA3047271 = {S30579, 70, { &P15364,&P5690,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 5.1; rv:21.0) Gecko/21.0 Firefox/21.0*/
const Device UA2408255 = {S30580, 118, { &P15364,&P16110,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA2533375 = {S30581, 125, { &P15364,&P16110,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.3.6000 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA2714027 = {S30582, 125, { &P15364,&P16110,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1*/
const Device UA2717129 = {S30583, 118, { &P15364,&P16110,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1*/
const Device UA2869980 = {S30584, 135, { &P15364,&P16110,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1,gzip(gfe)*/
const Device UA3118454 = {S30585, 135, { &P15364,&P16110,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1,gzip(gfe)*/
const Device UA197459 = {S30586, 64, { &P15364,&P18110,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; en) Presto/2.8.131 Version/11.11*/
const Device UA398789 = {S30587, 64, { &P15364,&P18110,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2425647 = {S30588, 66, { &P15364,&P18110,&P17374,&P18092 }};	/*Useragent - 0 Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2353732 = {S30589, 73, { &P15364,&P5690,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:19.0) Gecko/19.0 Firefox/19.0*/
const Device UA2354171 = {S30590, 61, { &P15364,&P5690,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:19.0) Gecko/19.0 Firefox/19.0*/
const Device UA2471040 = {S30591, 68, { &P15364,&P5690,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:19.0) Gecko/X Firefox/19.0,gzip(gfe)*/
const Device UA2563788 = {S30592, 70, { &P15364,&P5690,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0(Windows; U; Windows NT 5.2; rv:1.9.2) Gecko/X Firefox/19.0*/
const Device UA2784893 = {S30593, 120, { &P15364,&P5690,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:19.0) Gecko/X MRA 5.5 (build 02842) Firefox/19.0 (.NET CLR 3.5.30729)*/
const Device UA2799518 = {S30594, 77, { &P15364,&P5690,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:19.0) Gecko/X Firefox/19.0*/
const Device UA902608 = {S30595, 70, { &P15364,&P17285,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:13.0) Gecko/X Firefox/13.0*/
const Device UA902626 = {S30596, 60, { &P15364,&P17285,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:13.0) Gecko/X Firefox/13.0.1*/
const Device UA908167 = {S30597, 75, { &P15364,&P17285,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:13.0) Gecko/X Firefox/13.0.1 Firefox/13.0.1*/
const Device UA614796 = {S30598, 67, { &P15364,&P18118,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:13.0) Gecko/X Firefox/13.0*/
const Device UA2742727 = {S30599, 86, { &P15364,&P18118,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:13.0) Gecko/X Firefox/13.0.1 Firefox/13.0.1*/
const Device UA8684 = {S30600, 137, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Nexus One Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24486 = {S30601, 137, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Nexus One Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA169577 = {S30602, 138, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; Nexus One Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA190780 = {S30603, 61, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; Nexus One Build/GRJ22)*/
const Device UA209005 = {S30604, 137, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Nexus One Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA228715 = {S30605, 137, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Nexus One Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA242593 = {S30606, 138, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Nexus One Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250375 = {S30607, 138, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus One Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA296926 = {S30608, 143, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Nexus One Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA301254 = {S30609, 137, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Nexus One Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA304882 = {S30610, 61, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; Nexus One Build/GRI40)*/
const Device UA341696 = {S30611, 62, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; Nexus One Build/GRK39F)*/
const Device UA459201 = {S30612, 135, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.4; en-gb; Nexus One Build/GRI05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA459199 = {S30613, 135, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.4; en-us; Nexus One Build/GRH84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA491477 = {S30614, 135, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; Nexus One Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA544543 = {S30615, 135, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; Nexus One Build/MOG) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA569707 = {S30616, 136, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Android 2.3; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA603597 = {S30617, 137, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en_gb; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626546 = {S30618, 137, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus One Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA697428 = {S30619, 62, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; Nexus One Build/GRK39F)*/
const Device UA743403 = {S30620, 138, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Nexus One Build/GRH70B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA786902 = {S30621, 137, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Nexus One Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1868082 = {S30622, 137, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Nexus One Build/GRI54) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1895545 = {S30623, 61, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Nexus One Build/GRJ90)*/
const Device UA1906451 = {S30624, 136, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; nl-; Nexus One Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1950845 = {S30625, 137, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en_us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020650 = {S30626, 137, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Nexus One Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA2020821 = {S30627, 126, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Nexus One Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022440 = {S30628, 139, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7p0; en-us; Nexus One Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032766 = {S30629, 135, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; Nexus One Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2428463 = {S30630, 202, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1(compatible; goggles-proxy; +http://www.google.com/mobile/goggles/)*/
const Device UA2603796 = {S30631, 137, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; Nexus One Build/GRH87C) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2659190 = {S30632, 136, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2659506 = {S30633, 136, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2849702 = {S30634, 136, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus One Build/GRJ22) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2849825 = {S30635, 136, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3097683 = {S30636, 143, { &P16036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Nexus One Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2505936 = {S30637, 142, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Galaxy ACE Build/JELLYBEAN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2511267 = {S30638, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; i777 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2701389 = {S30639, 137, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; msm8960 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2701585 = {S30640, 147, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; sanstar6577_ics2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2746930 = {S30641, 145, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Full Android on Grouper Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2750577 = {S30642, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-l900 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2751290 = {S30643, 129, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; mk808b Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2751663 = {S30644, 152, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; iTouch infinity-HD89+ Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752239 = {S30645, 142, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; v89_gq2008s Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768683 = {S30646, 141, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; v89_zlh_hd Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2780182 = {S30647, 140, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; redhookbay Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781864 = {S30648, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Q508 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2791557 = {S30649, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; 2330 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2793896 = {S30650, 141, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; MAX III HD Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2794511 = {S30651, 133, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; Z5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2799314 = {S30652, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; 7102 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2799356 = {S30653, 144, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; Aevo NOTE II+ Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2858349 = {S30654, 137, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; msm8974 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2859250 = {S30655, 129, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; p4wifi Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2859890 = {S30656, 145, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; Android Ativ S Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2860038 = {S30657, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; GF-3 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2863746 = {S30658, 141, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; zh-tw; ASK SP581 HD Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2864133 = {S30659, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; HD-7 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865537 = {S30660, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - OPPO_R809T/1.0 Linux/3.4.0 Android/4.2.1  Release/12.24.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2 System/Android 4.2.1;*/
const Device UA2865626 = {S30661, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; A19S Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2866869 = {S30662, 210, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; zh-cn; MT-GT-T9500/1.0 Android/4.2.0 Release/05.03.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2866891 = {S30663, 142, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; NOTE II PRO Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2868030 = {S30664, 51, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2 Build/JDQ39E)*/
const Device UA2868461 = {S30665, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; X508 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2869119 = {S30666, 210, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; zh-cn; MT-GT-T9500/1.0 Android/4.2.0 Release/05.15.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2869289 = {S30667, 131, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; nl-nl; MIDC490 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2878942 = {S30668, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; B943 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878976 = {S30669, 128, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; mk808 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2892397 = {S30670, 142, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; Colossus II Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2893109 = {S30671, 140, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1;zh-cn; ZTE V967S Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30;*/
const Device UA2936358 = {S30672, 140, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; MTO MK319 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936699 = {S30673, 134, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Phone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2937069 = {S30674, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; GF-4 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937158 = {S30675, 133, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; Z2 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937564 = {S30676, 142, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; v89_gq3008s Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937568 = {S30677, 133, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; K5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937783 = {S30678, 142, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; v89_jdt1qhd Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937838 = {S30679, 143, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; v89_jbl1a668 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2938596 = {S30680, 127, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; JB7+ Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2944711 = {S30681, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-M919G Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955905 = {S30682, 150, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GFIVE President(G9) Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955952 = {S30683, 140, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; BLAZE 2.0 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956290 = {S30684, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; H3088 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956736 = {S30685, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; HX-9299A Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957128 = {S30686, 144, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; MC002 Build/Retail_WGMC002) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957183 = {S30687, 149, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; v89_jbl1a668_fwvga Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957449 = {S30688, 126, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; P7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2957694 = {S30689, 143, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; v89_gq3002sc Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958161 = {S30690, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; Orion IV Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958188 = {S30691, 146, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Z2_WVGA_ADM Build/Android4.2.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2958190 = {S30692, 134, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; RKM MK802IV Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2958228 = {S30693, 207, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; zh-cn; MT-H9082/1.0 Android/4.2.0 Release/04.26.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2958482 = {S30694, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SM-T217A Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2958603 = {S30695, 144, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; Aveo NOTE PRO Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958983 = {S30696, 146, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; v89_jbla768_asx Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2960972 = {S30697, 143, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - HUAWEI_P6-T00_TD/1.0 Android/4.2.2 (Linux; U; Android 4.2.2; zh-cn) Release/03.20.2013 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30*/
const Device UA2961526 = {S30698, 152, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; 06v20_v89_jbla768_asx Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2962938 = {S30699, 125, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; M6 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2962954 = {S30700, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; IMO S89 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2964650 = {S30701, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; W150 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2969464 = {S30702, 133, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; V973 Core4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2986895 = {S30703, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; n7100 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2989164 = {S30704, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; YOUPU U6 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2989416 = {S30705, 140, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; VirtualBox Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2990059 = {S30706, 144, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; th-th; i-mobile IQ 9 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2993693 = {S30707, 147, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Transformer Pad Infinity Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2993808 = {S30708, 143, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; HOSIN V70 Build/HOSINA728) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994038 = {S30709, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; R809T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994551 = {S30710, 141, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; GPLUS N939 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2995355 = {S30711, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; H920 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2995836 = {S30712, 141, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; zh-tw; ASK SP553 HD Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996018 = {S30713, 154, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; HUAWEI G700-U00 Build/HuaweiG700-U00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996235 = {S30714, 133, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; G1 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996579 = {S30715, 145, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Z6_WVGA_LC Build/Android4.2.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2996654 = {S30716, 141, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; itouch_HD5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996781 = {S30717, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1;zh-cn; Lenovo_K860i/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2.1 Mobile Safari/534.30*/
const Device UA2996978 = {S30718, 131, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-P5220 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2997060 = {S30719, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; A670 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997328 = {S30720, 134, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; TS4 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997468 = {S30721, 141, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1;en-us; Lenovo_S920_ROW/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2.1 Mobile Safari/534.30*/
const Device UA2997598 = {S30722, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; X509 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997662 = {S30723, 140, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; EX_NOTE_? Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997828 = {S30724, 128, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; GN708T Build/JOP40D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2997843 = {S30725, 129, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; HD-89 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2998050 = {S30726, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; ru-ru; HD Quad Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998175 = {S30727, 146, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Andi4.7G COBALT Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998371 = {S30728, 141, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; EX'Note IV Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998384 = {S30729, 142, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; v89_gq2002s Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998729 = {S30730, 132, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; N-06E Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998740 = {S30731, 141, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1;en-us; Lenovo_S820_ROW/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2.1 Mobile Safari/534.30*/
const Device UA2998976 = {S30732, 151, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-ph; MyPhone Agua Iceberg Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999028 = {S30733, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; A55HD Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999051 = {S30734, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; xt875 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999159 = {S30735, 129, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; th-th; SP0728 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2999304 = {S30736, 137, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1;zh-cn; Lenovo_S820/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2.1 Mobile Safari/534.30*/
const Device UA2999358 = {S30737, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; DOOV S1 Build/DOOVS1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999365 = {S30738, 134, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; W89 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999554 = {S30739, 137, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; VOW V5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3000152 = {S30740, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; vi-vn; VDM-HD4 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3000795 = {S30741, 129, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Tablet Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3000836 = {S30742, 142, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; Gmate G2000 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3002372 = {S30743, 149, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; 06_v89_jbla768_asx Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3002947 = {S30744, 144, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; v89_jdt1fwvga Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3002970 = {S30745, 132, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; AT10LE-A Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3003186 = {S30746, 128, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MK908 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3003273 = {S30747, 210, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; zh-cn; MT-GT-T9500/1.0 Android/4.2.0 Release/05.16.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3003749 = {S30748, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Primo-G3 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3004772 = {S30749, 134, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; I30 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3005843 = {S30750, 137, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; HY5001 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3006953 = {S30751, 210, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; zh-cn; MT-GT-T9500/1.0 Android/4.2.0 Release/05.24.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3007202 = {S30752, 144, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; Forward_Prime Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3007715 = {S30753, 133, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; S4 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3009095 = {S30754, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; i4000 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3010505 = {S30755, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; X130_HD Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3011301 = {S30756, 150, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HUAWEI P6-T00 Build/HuaweiP6-T00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3011591 = {S30757, 162, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; 06_v89_jbla768_asx_s4lockscreen Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3011889 = {S30758, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; T600 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3012155 = {S30759, 130, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; STB9097 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3012480 = {S30760, 148, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Cherry thunder2.0 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3013023 = {S30761, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; 202K Build/100.2.2840) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3014957 = {S30762, 154, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; HUAWEI G610-U00 Build/HuaweiG610-U00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3015181 = {S30763, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Atrix HD Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3015781 = {S30764, 140, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; S39h Build/16.0.A.0.36) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3015862 = {S30765, 160, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HW-HUAWEI_P6-C00/P6-C00V100R001C92B112SP01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3016171 = {S30766, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; I9389 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3016258 = {S30767, 150, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HUAWEI P6-C00 Build/HuaweiP6-C00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3017720 = {S30768, 131, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; EveryPad Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3018694 = {S30769, 141, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; LESPHONE01 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3019214 = {S30770, 137, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; en-us; MINI-S4 Build/MINI-S4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3020941 = {S30771, 160, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HW-HUAWEI_P6-C00/P6-C00V100R001C92B115SP01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3021240 = {S30772, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; Glory 2 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3021249 = {S30773, 141, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; SMARTLET 2 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022500 = {S30774, 140, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; vivo X510t Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022999 = {S30775, 137, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - OPPO_R821T/V1 Linux/3.4.5 Android/4.2.2 Release/03.26.2013 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2 System/Android 4.2.2;*/
const Device UA3023509 = {S30776, 144, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.3; en-us; v89_jdt1fwvga Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3024945 = {S30777, 128, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; nl-nl; 1023Q Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3027012 = {S30778, 140, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; M-PP2G530 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3027496 = {S30779, 131, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; M470BSE Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3030149 = {S30780, 144, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Symphony W65 Build/UNKNOWN) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3031118 = {S30781, 133, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; A24 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3034255 = {S30782, 146, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; AUXUS Nuclea N1 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3034561 = {S30783, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; ZTE N986 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3046391 = {S30784, 147, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Transformer Prime Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3046446 = {S30785, 151, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Full JellyBean on Mako Build/LGEP19) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3047692 = {S30786, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; OS988 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3047895 = {S30787, 134, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; HD5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3048522 = {S30788, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; i9480 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3048683 = {S30789, 141, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; AMOI M8448 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3048957 = {S30790, 141, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; ELIYA S898 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049298 = {S30791, 137, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; H9700+ Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049338 = {S30792, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-LS980 Build/JDQ39B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2 Mobile Safari/534.30*/
const Device UA3050105 = {S30793, 137, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; H9500+ Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050186 = {S30794, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; de-de; Q880_XK Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050233 = {S30795, 137, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; UMI_S1 Build/UMI_S1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050245 = {S30796, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; WA520 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050302 = {S30797, 133, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; G6 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050381 = {S30798, 149, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; 01_v89_jbla728_asx Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050453 = {S30799, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; H7189 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050456 = {S30800, 133, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Q5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050512 = {S30801, 141, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; itouch HD5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050932 = {S30802, 134, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ThL Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3051571 = {S30803, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGT-P470 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3053315 = {S30804, 134, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; CW-V88-QUAD Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3054191 = {S30805, 137, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; es-es; MINI-S4 Build/MINI-S4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3054505 = {S30806, 133, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; S9 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3054762 = {S30807, 147, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; 11_gq2000s_fwvga Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3054899 = {S30808, 133, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Z1 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3056882 = {S30809, 72, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; HUAWEI P6-C00 Build/HuaweiP6-C00)*/
const Device UA3056992 = {S30810, 137, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1;zh-cn; Lenovo_K860/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2.1 Mobile Safari/534.30*/
const Device UA3057174 = {S30811, 127, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; A70X Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3057235 = {S30812, 140, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Zeepad 9XN Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057291 = {S30813, 124, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; _ Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3057701 = {S30814, 129, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; S1 pro Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3057765 = {S30815, 145, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Primo_NX Build/WALTONWALTON) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057769 = {S30816, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; NX40X Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057824 = {S30817, 145, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; Mlais MX58 Air Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057849 = {S30818, 143, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Cherry Razor Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057869 = {S30819, 141, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Meitu Kiss Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058005 = {S30820, 143, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Symphony ZII Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058109 = {S30821, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC_PN072 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058201 = {S30822, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; i6000 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058396 = {S30823, 137, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ThL W9 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058502 = {S30824, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; it-it; CENTURION Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058742 = {S30825, 131, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; de-de; TPC-9000 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3058745 = {S30826, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; W200 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058800 = {S30827, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; S5-F Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058862 = {S30828, 152, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Symphony Xplorer W32 Build/UNKNOWN) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058981 = {S30829, 134, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; W8s Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3059015 = {S30830, 151, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Vandroid Build/S5-V1.0-2013.07.18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3059204 = {S30831, 144, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; BLU Life View Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3059454 = {S30832, 132, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; CRXT 1131 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3059508 = {S30833, 124, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; PX2 Build/PX2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3059516 = {S30834, 129, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; M9 pro Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3060112 = {S30835, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; en-gb; MID-756 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3060798 = {S30836, 131, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; AP-7S118 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3061759 = {S30837, 133, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; it-it; X50D Build/X50D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3061982 = {S30838, 128, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; C1059 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3067252 = {S30839, 182, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.2.2;dev=jflteatt;devm=SAMSUNG-SGH-I337;man=samsung;brw=Android;shwd=1080x1920;shwa=1080x1845;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA3072823 = {S30840, 137, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; GN708T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3073411 = {S30841, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; V976 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3073576 = {S30842, 143, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.9; zh-tw; ASK SP554 HD Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3073759 = {S30843, 147, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; APOLLO ONE K6589 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3073991 = {S30844, 143, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; ITOUCH HD89S Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3074105 = {S30845, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; HTC_PO582 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3074124 = {S30846, 146, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; en-us; GT-T9500 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3074183 = {S30847, 133, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; vi-vn; V813 Core4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3074262 = {S30848, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; X120Q Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3074242 = {S30849, 133, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; vi-vn; V811 Core4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3074307 = {S30850, 130, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SM-T215 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3075767 = {S30851, 134, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; x909 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3077626 = {S30852, 148, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; 01v21_v89_gq2008s Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3077765 = {S30853, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; TCL S950 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2 Mobile Safari/534.30*/
const Device UA3078047 = {S30854, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; ND500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078200 = {S30855, 141, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HUAWEI P6-C00 Build/HuaweiP6-C00) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3078206 = {S30856, 207, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; zh-cn; MT-H9082/1.0 Android/4.2.0 Release/05.24.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3078385 = {S30857, 130, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; K1001L1 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3078405 = {S30858, 130, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; rk30mtk Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3078423 = {S30859, 133, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; C3 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078517 = {S30860, 129, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; CX-919 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3078823 = {S30861, 137, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ru-ru; X-basic Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078826 = {S30862, 128, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; U55GT Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3078876 = {S30863, 131, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-au; M470BSS Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3078941 = {S30864, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ViewPad 100Q Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3079047 = {S30865, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; Stylo Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079117 = {S30866, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ko-kr; IM-A880S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079151 = {S30867, 151, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; th-th; i-mobile i-STYLE 7.3A Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079306 = {S30868, 137, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Etel i4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079507 = {S30869, 142, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Symphony W68 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079637 = {S30870, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; POMP W89 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079779 = {S30871, 160, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HW-HUAWEI_P6-C00/P6-C00V100R001C92B116SP01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079781 = {S30872, 145, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - HUAWEI_G610-T00_TD/1.0 Android/4.2.1 (Linux; U; Android 4.2.1; zh-cn) Release/05.15.2013 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30*/
const Device UA3079863 = {S30873, 130, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; MID9724 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3079864 = {S30874, 137, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Q1000S Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3080251 = {S30875, 140, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ASP-5000H Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3080397 = {S30876, 144, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-in; Micromax A240 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3083239 = {S30877, 142, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Symphony W82 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3083722 = {S30878, 161, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Full AOSP on Toroplus Build/PHONELAB_JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3086515 = {S30879, 144, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Maximus V.1.2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3087314 = {S30880, 132, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; AT10PE-A Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3088053 = {S30881, 142, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Lenovo S820 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088183 = {S30882, 130, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ME302KL Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3088958 = {S30883, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; ALWAY-A8 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3089047 = {S30884, 143, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; DROIDZ Race X Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3089055 = {S30885, 137, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Silver Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3090816 = {S30886, 134, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-tw; S510 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3091681 = {S30887, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; RM-25 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3091764 = {S30888, 129, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; B1-711 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Safari/534.30*/
const Device UA3091856 = {S30889, 126, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; S1S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3091896 = {S30890, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; 9960 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092002 = {S30891, 131, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-tw; Mini one Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3092021 = {S30892, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; 9400 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092148 = {S30893, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; fr-fr; STAIRWAY Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092179 = {S30894, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; ro-ro; V1_Viper Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092182 = {S30895, 131, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SM-T217S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3092189 = {S30896, 125, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; U8 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3092228 = {S30897, 140, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; COSMOS X2 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092265 = {S30898, 141, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-uk; CYCLONE VOYAGER-BT Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3092357 = {S30899, 152, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; 01v21_v89_jbl1a698_2g Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092389 = {S30900, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; PMID906/PMID910 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3092400 = {S30901, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; de-de; PAD 9719QR Build/PAD 9719QR) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3092490 = {S30902, 144, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.3; es-es; v89_jdt1fwvga Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092642 = {S30903, 140, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; M-PP2S500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092780 = {S30904, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; HS-U970 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092925 = {S30905, 130, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; DT mini Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3093060 = {S30906, 143, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Symphony W92 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3093462 = {S30907, 132, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; it-it; M-MP720GO Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3093498 = {S30908, 126, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; X81 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3093608 = {S30909, 143, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; L36h Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3094129 = {S30910, 163, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nocturnal Google Edition CDMA 5.0 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3095943 = {S30911, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; ARNOVA 90 G4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3101763 = {S30912, 145, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; SOWA S8800 Pro Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102315 = {S30913, 146, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; G9589/N7102 PAD Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102355 = {S30914, 140, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; K47/C9_89 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102781 = {S30915, 150, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Matricom G-Box Midnight MX2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3102809 = {S30916, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; neken N5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103015 = {S30917, 141, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; imatch i26 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103384 = {S30918, 142, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nitro 450QD Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103394 = {S30919, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; i9501 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103638 = {S30920, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; mini pad Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3104879 = {S30921, 148, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; VOVO_VII Build/MocorDroid4.2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3105020 = {S30922, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; EX_S4 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3105227 = {S30923, 129, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; K1001L Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3106102 = {S30924, 146, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Wammy Passion Z Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3106168 = {S30925, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; it-it; MID9035C Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3106308 = {S30926, 142, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; Pandora 9.21 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3106881 = {S30927, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Akai A8 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3107091 = {S30928, 145, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; th-th; i-mobile IQ 9A Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3107383 = {S30929, 76, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; HUAWEI G610-U00 Build/HuaweiG610-U00)*/
const Device UA3107560 = {S30930, 130, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; V97 HD Build/V97 HD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3107755 = {S30931, 130, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; sv-se; 1048QCS Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3107850 = {S30932, 130, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ifiveX2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3107837 = {S30933, 72, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; HUAWEI P6-T00 Build/HuaweiP6-T00)*/
const Device UA3108075 = {S30934, 148, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; 01v21_v89_gq3008s Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108105 = {S30935, 210, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; zh-cn; MT-GT-A9500/1.0 Android/4.2.0 Release/07.25.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3108107 = {S30936, 161, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Kinder-Tablet-1.0-Weltbild-Mozilla/5.0 (Linux; U; Android 4.2.2; de-DE; PEDI_PLUS_W Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3108131 = {S30937, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Cosmos X Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108158 = {S30938, 134, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; V70 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108163 = {S30939, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; ru-ru; Five Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108175 = {S30940, 140, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; de-de; AVUS A-24 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108244 = {S30941, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; UM-A90 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108250 = {S30942, 144, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Andi5h quadro Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108484 = {S30943, 134, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; NEO-X7-216A Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3108526 = {S30944, 140, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; Tablet PC 2008 DC Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3108534 = {S30945, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I7105 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108539 = {S30946, 146, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ANDROIDGMAILAPP Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108709 = {S30947, 142, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; nl-nl; POV_TAB-P1025(v1.0) Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3108801 = {S30948, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; N900D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108913 = {S30949, 133, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; DLX Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108935 = {S30950, 134, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; OZZY Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108997 = {S30951, 151, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; GT-P5200-ORANGE/P5200XXUAMFC Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3109063 = {S30952, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; it-it; LUXURY 3 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3109166 = {S30953, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; fr-fr; DARKSIDE Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3109264 = {S30954, 132, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; P2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3110001 = {S30955, 142, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Symphony W85 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3110217 = {S30956, 133, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; vi-vn; F13 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3110909 = {S30957, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; vi-vn; Nova E Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3111686 = {S30958, 140, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Karbonn A8 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3112568 = {S30959, 150, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; th-th; i-mobile i-STYLE 7.3 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3120239 = {S30960, 143, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn;GiONEE-E3/ E3 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3120250 = {S30961, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; K-one Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3120624 = {S30962, 140, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Newman K1 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3120716 = {S30963, 131, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-p6210 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3120918 = {S30964, 143, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; ASK SP460 HD Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121201 = {S30965, 143, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; v89_gq2002sc Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121387 = {S30966, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; ZP958 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121421 = {S30967, 140, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; ru-ru; ZTE V880H Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121609 = {S30968, 133, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; A13 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3123979 = {S30969, 153, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Full AOSP on Maguro Build/4.2.2.632) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3124260 = {S30970, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; TCL J620 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Versio/4.0 Mobile Safari/534.30*/
const Device UA3124814 = {S30971, 136, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; PL7117 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3124916 = {S30972, 134, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; pt-pt; ALEXISRX5DC Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3125079 = {S30973, 142, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; Funker X504 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125100 = {S30974, 133, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; vi-vn; F19 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125157 = {S30975, 149, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; 06v20_v89_gq3009hd Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125273 = {S30976, 146, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Luna TAB09-100 Build/Luna TAB09-100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3125320 = {S30977, 141, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Z10 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3125333 = {S30978, 142, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; 03_v89_jysj_f668_sx Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3125384 = {S30979, 149, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; 06_v89_gq3009fwvga Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125441 = {S30980, 133, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GoTab-GTD9 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3125478 = {S30981, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; it-it; F9192 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125485 = {S30982, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9158 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125489 = {S30983, 132, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; MOMO20-W2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3125501 = {S30984, 142, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GCFONE 570Q Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125541 = {S30985, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2;zh-cn; ZTE V818 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30;*/
const Device UA3125552 = {S30986, 144, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Kurio4S Build/C13200US102a) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125585 = {S30987, 146, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Lenovo S820_ROW Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125650 = {S30988, 131, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MID0928 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3125667 = {S30989, 150, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; th-th; i-mobile i-STYLE 8.1 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125701 = {S30990, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; en-au; MID-756 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125716 = {S30991, 137, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; de-de; NO1_S6 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125734 = {S30992, 139, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; vivo X3t Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125744 = {S30993, 133, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; S6 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125778 = {S30994, 140, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; Revo Lead Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125877 = {S30995, 137, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Y9190   Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125894 = {S30996, 137, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Mi-510 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125926 = {S30997, 141, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; vivo S7i(t) Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3126051 = {S30998, 131, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R960 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3126103 = {S30999, 140, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; id-id; CROSS A88 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3126233 = {S31000, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; Unusual_U10Z Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3126294 = {S31001, 133, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; X40 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3126432 = {S31002, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; fnac 3.5 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3127064 = {S31003, 147, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-ca; LG-D803/D803B10a Build/JDQ39B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2 Mobile Safari/534.30*/
const Device UA3127095 = {S31004, 135, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Numy 3G AX1 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3129255 = {S31005, 138, { &P15364,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; TECNO P5 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2362197 = {S31006, 135, { &P18635,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Nexus 4 Build/JOP40C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2389104 = {S31007, 137, { &P18635,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 4 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2483444 = {S31008, 131, { &P18635,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 4 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2515525 = {S31009, 136, { &P18635,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2533186 = {S31010, 130, { &P18635,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2709329 = {S31011, 143, { &P18635,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; es-US; Nexus 4 Build/JOP40D) AppleWebKit/535.19 (KHTML like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2509686 = {S31012, 64, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; WOW64) Presto/2.12.388 Version/12.14*/
const Device UA2510173 = {S31013, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.14*/
const Device UA2512032 = {S31014, 57, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.14*/
const Device UA2512803 = {S31015, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.14*/
const Device UA2512804 = {S31016, 67, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.14,gzip(gfe)*/
const Device UA2514393 = {S31017, 80, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.14*/
const Device UA2531127 = {S31018, 68, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:2.0) Gecko/X Firefox/4.0 Opera 12.14*/
const Device UA2531413 = {S31019, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.14*/
const Device UA2532325 = {S31020, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.14*/
const Device UA2535671 = {S31021, 80, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.14*/
const Device UA2536759 = {S31022, 80, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.14*/
const Device UA2537183 = {S31023, 62, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0) Opera 12.14*/
const Device UA2537473 = {S31024, 80, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.14*/
const Device UA2537641 = {S31025, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.14*/
const Device UA2537759 = {S31026, 80, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.14*/
const Device UA2537771 = {S31027, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.14*/
const Device UA2561250 = {S31028, 106, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.14*/
const Device UA2561928 = {S31029, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.14*/
const Device UA2563486 = {S31030, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.14*/
const Device UA2564613 = {S31031, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.14*/
const Device UA2564705 = {S31032, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.14*/
const Device UA2571427 = {S31033, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.14*/
const Device UA2572186 = {S31034, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.14*/
const Device UA2572214 = {S31035, 116, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)) Presto/2.12.388 Version/12.14*/
const Device UA2580591 = {S31036, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.14*/
const Device UA2580596 = {S31037, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.14*/
const Device UA2580614 = {S31038, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.14*/
const Device UA2581909 = {S31039, 80, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.14*/
const Device UA2582632 = {S31040, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.14*/
const Device UA2597851 = {S31041, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.14*/
const Device UA2599300 = {S31042, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.14*/
const Device UA2599730 = {S31043, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.14*/
const Device UA2599967 = {S31044, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.14*/
const Device UA2600481 = {S31045, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.14*/
const Device UA2602402 = {S31046, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.14*/
const Device UA2603108 = {S31047, 80, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.14*/
const Device UA2603706 = {S31048, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.14*/
const Device UA2661102 = {S31049, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.14*/
const Device UA2661330 = {S31050, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.14*/
const Device UA2661410 = {S31051, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.14*/
const Device UA2663098 = {S31052, 88, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; Edition Campaign 21) Presto/2.12.388 Version/12.14,gzip(gfe)*/
const Device UA2663992 = {S31053, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.14*/
const Device UA2664975 = {S31054, 80, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.14*/
const Device UA2672288 = {S31055, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.14*/
const Device UA2700874 = {S31056, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.14*/
const Device UA2701849 = {S31057, 95, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; compatible; MSIE 7.0; Windows NT 6.0) Presto/2.12.388 Version/12.14*/
const Device UA2708201 = {S31058, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.14*/
const Device UA2730115 = {S31059, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.14*/
const Device UA2730405 = {S31060, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.14*/
const Device UA2730936 = {S31061, 80, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.14*/
const Device UA2749009 = {S31062, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.14*/
const Device UA2858582 = {S31063, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.14*/
const Device UA2995024 = {S31064, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.14*/
const Device UA2998681 = {S31065, 79, { &P15364,&P17285,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.14*/
const Device UA92699 = {S31066, 50, { &P15364,&P17188,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 4.0)*/
const Device UA97981 = {S31067, 69, { &P15364,&P17188,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 4.0; .NET CLR 1.0.2914)*/
const Device UA459860 = {S31068, 114, { &P15364,&P17188,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 4.0; Mozilla/4.0 (compatable; MSIE 4.0; Windows NT; Windows 95); TX)*/
const Device UA766989 = {S31069, 157, { &P15364,&P16110,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; TmstmpExt)*/
const Device UA1900285 = {S31070, 150, { &P15364,&P16110,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; TmstmpExt)*/
const Device UA1901864 = {S31071, 167, { &P15364,&P16110,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; TmstmpExt)*/
const Device UA2261907 = {S31072, 172, { &P15364,&P16110,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; TmstmpExt)*/
const Device UA2262058 = {S31073, 178, { &P15364,&P16110,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; TmstmpExt)*/
const Device UA2375045 = {S31074, 193, { &P15364,&P16110,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Media Center PC 6.0; TmstmpExt)*/
const Device UA2395492 = {S31075, 171, { &P15364,&P16110,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; TmstmpExt)*/
const Device UA2447588 = {S31076, 160, { &P15364,&P16110,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; TmstmpExt)*/
const Device UA2475975 = {S31077, 193, { &P15364,&P16110,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; SLCC2; Tablet PC 2.0; TmstmpExt)*/
const Device UA2527145 = {S31078, 188, { &P15364,&P16110,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Zune 4.7; TmstmpExt)*/
const Device UA2547309 = {S31079, 206, { &P15364,&P16110,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 5.0; Media Center PC 6.0; SLCC1; SLCC2; TmstmpExt)*/
const Device UA2553846 = {S31080, 157, { &P15364,&P16110,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; TmstmpExt)*/
const Device UA2694836 = {S31081, 172, { &P15364,&P16110,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Avant Browser; TmstmpExt)*/
const Device UA2696060 = {S31082, 221, { &P15364,&P16110,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 5.0; Media Center PC 6.0; SLCC1; SLCC2; Tablet PC 2.0; TmstmpExt)*/
const Device UA2727703 = {S31083, 178, { &P15364,&P16110,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; TmstmpExt)*/
const Device UA2777379 = {S31084, 194, { &P15364,&P16110,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Zune 4.7; InfoPath.3; TmstmpExt)*/
const Device UA2842373 = {S31085, 173, { &P15364,&P16110,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; MASMJS; TmstmpExt)*/
const Device UA2936759 = {S31086, 168, { &P15364,&P16110,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; HPDTDFJS; TmstmpExt)*/
const Device UA3117353 = {S31087, 181, { &P15364,&P16110,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Zune 4.7; TmstmpExt)*/
const Device UA2408280 = {S31088, 92, { &P15364,&P7133,&P22775,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SlimBrowser/6.01.091)*/
const Device UA2432320 = {S31089, 85, { &P15364,&P7133,&P22775,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SlimBrowser/6.01.091)*/
const Device UA2434026 = {S31090, 107, { &P15364,&P7133,&P22775,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Avant Browser; SlimBrowser/6.01.091)*/
const Device UA2436046 = {S31091, 100, { &P15364,&P7133,&P22775,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Avant Browser; SlimBrowser/6.01.091)*/
const Device UA2486330 = {S31092, 92, { &P15364,&P7133,&P22775,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SlimBrowser/6.01.091)*/
const Device UA2583912 = {S31093, 93, { &P15364,&P7133,&P22775,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SlimBrowser/6.01.101)*/
const Device UA2583997 = {S31094, 86, { &P15364,&P7133,&P22775,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; SlimBrowser/6.01.101)*/
const Device UA2602715 = {S31095, 108, { &P15364,&P7133,&P22775,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Avant Browser; SlimBrowser/6.01.101)*/
const Device UA2668225 = {S31096, 101, { &P15364,&P7133,&P22775,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Avant Browser; SlimBrowser/6.01.101)*/
const Device UA2794512 = {S31097, 173, { &P15364,&P7133,&P22775,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; SlimBrowser/6.01.101)*/
const Device UA613269 = {S31098, 61, { &P15364,&P18110,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:13.0) Gecko/X Firefox/13.0*/
const Device UA904774 = {S31099, 80, { &P15364,&P18110,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:13.0) Gecko/X Firefox/13.0.1 Iceweasel/13.0.1*/
const Device UA2354687 = {S31100, 61, { &P15364,&P18110,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:13.0) Gecko/X Iceweasel/13.0*/
const Device UA2684683 = {S31101, 76, { &P15364,&P18110,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:13.0) Gecko/X Firefox/13.0.1 Firefox/13.0.1*/
const Device UA2356973 = {S31102, 153, { &P12280,&P19826,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B5095f Safari/7534.48.3*/
const Device UA2362594 = {S31103, 153, { &P12280,&P19826,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B5105c Safari/7534.48.3*/
const Device UA2489523 = {S31104, 151, { &P12280,&P19826,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B144 Safari/7534.48.3*/
const Device UA2489667 = {S31105, 151, { &P12280,&P19826,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B143 Safari/7534.48.3*/
const Device UA2489950 = {S31106, 151, { &P12280,&P19826,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B142 Safari/7534.48.3*/
const Device UA2490030 = {S31107, 151, { &P12280,&P19826,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B141 Safari/7534.48.3*/
const Device UA2512869 = {S31108, 153, { &P12280,&P19826,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B145 Safari/7534.48.3*/
const Device UA2532871 = {S31109, 153, { &P12280,&P19826,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B146 Safari/7534.48.3*/
const Device UA2585566 = {S31110, 153, { &P12280,&P19826,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B329 Safari/7534.48.3*/
const Device UA2710754 = {S31111, 153, { &P12280,&P19826,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B350 Safari/7534.48.3*/
const Device UA2670753 = {S31112, 130, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2670889 = {S31113, 138, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2671033 = {S31114, 138, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-R530U Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2671856 = {S31115, 137, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673085 = {S31116, 138, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673353 = {S31117, 138, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673537 = {S31118, 138, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305T Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673565 = {S31119, 130, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SPH-L710 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2675042 = {S31120, 138, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530C Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675528 = {S31121, 137, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675844 = {S31122, 131, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676029 = {S31123, 138, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747M Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676084 = {S31124, 137, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676095 = {S31125, 137, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676170 = {S31126, 138, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676197 = {S31127, 137, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SPH-L710 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676240 = {S31128, 138, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305N Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676258 = {S31129, 131, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747M Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676484 = {S31130, 145, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676573 = {S31131, 130, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676594 = {S31132, 138, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676730 = {S31133, 131, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676734 = {S31134, 145, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676736 = {S31135, 138, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676778 = {S31136, 130, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2677229 = {S31137, 138, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305N Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677304 = {S31138, 137, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677976 = {S31139, 131, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530C Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2714575 = {S31140, 131, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-R530U Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735487 = {S31141, 138, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2861445 = {S31142, 131, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305N Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2879049 = {S31143, 138, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3013243 = {S31144, 131, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3087667 = {S31145, 131, { &P18645,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305T Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA402898 = {S31146, 126, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; Xoom Build/IML34B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA402907 = {S31147, 126, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; Xoom Build/IML41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA402908 = {S31148, 126, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; Xoom Build/IML41B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA402915 = {S31149, 126, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; Xoom Build/IML52C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA402973 = {S31150, 126, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; Xoom Build/IML48B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA402978 = {S31151, 125, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; Xoom Build/IML46) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA403045 = {S31152, 126, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; Xoom Build/IML62B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA911868 = {S31153, 128, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xoom Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA924834 = {S31154, 128, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xoom Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1997242 = {S31155, 57, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Xoom Build/JRO03H)*/
const Device UA2006201 = {S31156, 128, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xoom Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2025440 = {S31157, 128, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xoom Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2225972 = {S31158, 128, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xoom Build/JRO03O) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2237050 = {S31159, 128, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xoom Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2237793 = {S31160, 57, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Xoom Build/JZO54K)*/
const Device UA2393248 = {S31161, 125, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; Xoom Build/JRN82) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2526635 = {S31162, 135, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xoom Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2549926 = {S31163, 128, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xoom Build/JRO03T) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2652660 = {S31164, 128, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xoom Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2658378 = {S31165, 125, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; Xoom Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.1 Safari/534.13*/
const Device UA2694044 = {S31166, 126, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; Xoom Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2994021 = {S31167, 128, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xoom Build/JZO54L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3024136 = {S31168, 128, { &P15869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xoom Build/JZO54M) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA382090 = {S31169, 129, { &P15234,&P5156,&P3519,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9330; en-US) AppleWebKit/534.8+ (KHTML, like Gecko) Version/6.0.0.719 Mobile Safari/534.8+*/
const Device UA2035305 = {S31170, 112, { &P18445,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 920)*/
const Device UA2362456 = {S31171, 120, { &P18445,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 920; ANZ821)*/
const Device UA2367170 = {S31172, 122, { &P18445,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 920; Vodafone)*/
const Device UA2401811 = {S31173, 120, { &P18445,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 920; Orange)*/
const Device UA2469505 = {S31174, 112, { &P18445,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia 920)*/
const Device UA2481528 = {S31175, 113, { &P18445,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; Nokia; Nokia 920T)*/
const Device UA3061789 = {S31176, 120, { &P18445,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 920; 1326-2)*/
const Device UA8955 = {S31177, 157, { &P11684,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Nokia5800d-1b /11.2.008; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA32486 = {S31178, 154, { &P11684,&P5129,&P3690,&P18092 }};	/*Useragent - Nokia5800/30.0.011.1 (SymbianOS/9.4; U; Series60/5.0 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA35000 = {S31179, 185, { &P11684,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Nokia5800d-1; U; Series60/5.0 Nokia5800d-1/11.0.008; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA62756 = {S31180, 152, { &P11684,&P5129,&P3690,&P18092 }};	/*Useragent - Nokia5800i/10.3.007 (SymbianOS/9.4; U; Series60/5.0; Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/413 (KHTML,like Gecko) Safari/413*/
const Device UA64696 = {S31181, 181, { &P11684,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5800d-1b/51.2.007; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.5.2 3gpp-gba*/
const Device UA64765 = {S31182, 180, { &P11684,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5800d-1/52.0.007; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.9 3gpp-gba*/
const Device UA251814 = {S31183, 194, { &P11684,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5800d-1/60.0.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 Mobile Safari/533.4 3gpp-gba*/
const Device UA722387 = {S31184, 161, { &P11684,&P5129,&P3690,&P18092 }};	/*Useragent - Nokia5800w/52.0.003 (SymbianOS/9.4; U; Series60/5.0; Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/413 (KHTML,like Gecko) Safari/413 3gpp-gba*/
const Device UA3043681 = {S31185, 170, { &P11684,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5800/30.0.004; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 3gpp-gba*/
const Device UA487972 = {S31186, 132, { &P15364,&P18110,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; es-AR; rv:1.9.1.6) Gecko/X Firefox/3.6 Iceweasel/3.5.6 (like Firefox/3.5.6; Debian-3.5.6-1~bpo50+1)*/
const Device UA523223 = {S31187, 96, { &P15364,&P18110,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-ES; rv:1.9.1.8) Gecko/X Firefox/3.0.6 (like Firefox/3.5.8)*/
const Device UA654929 = {S31188, 77, { &P15364,&P18110,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux i686; U; en; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.51*/
const Device UA677206 = {S31189, 112, { &P15364,&P18110,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; pt-BR; rv:1.9.1.6) Gecko/X Iceweasel/3.5.6 (like Firefox/3.5.6; Debian-3.5.6-1)*/
const Device UA733211 = {S31190, 98, { &P15364,&P18110,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; it; rv:1.9.1.16) Gecko/X Iceweasel/3.5.16 (like Firefox/3.5.16)*/
const Device UA781454 = {S31191, 70, { &P15364,&P18110,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; fr; rv:1.9.1.1) Gecko/X Firefox/3.5.1*/
const Device UA891798 = {S31192, 118, { &P15364,&P18110,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.6) Gecko/X Firefox/3.5.6 (like Firefox/3.5.6; Debian-3.5.6-1~bpo50+1)*/
const Device UA1953513 = {S31193, 98, { &P15364,&P18110,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.1.12) Gecko/X Firefox/2.0.0.3 (like Firefox/3.5.12)*/
const Device UA2128905 = {S31194, 108, { &P15364,&P18110,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.3) Gecko/X Firefox/2.0 (like Firefox/3.5.5; Debian-3.5.5-1)*/
const Device UA2139346 = {S31195, 63, { &P15364,&P18110,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; pt-BR; rv:1.9.1.1) Gecko/X Firefox/3.5.1*/
const Device UA2149956 = {S31196, 72, { &P15364,&P18110,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Linux X86_64; en-US; rv:1.9.1) Gecko/X Firefox/3.5*/
const Device UA2223369 = {S31197, 97, { &P15364,&P18110,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.16) Gecko/X Firefox/3.5.16 (like Firefox/3.5.16)*/
const Device UA2848541 = {S31198, 98, { &P15364,&P18110,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1.16) Gecko/X firefox/3.0.6 (like Firefox/3.5.16)*/
const Device UA49211 = {S31199, 163, { &P20603,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-C800 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA357614 = {S31200, 164, { &P20603,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; LG-C800G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA369482 = {S31201, 142, { &P20603,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-C800 Build/GRJ22) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA646302 = {S31202, 59, { &P20603,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-C800 Build/GRJ22)*/
const Device UA678509 = {S31203, 162, { &P20603,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; LG-C800 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA719740 = {S31204, 162, { &P20603,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-C800Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA738605 = {S31205, 162, { &P20603,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;LG-C800 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA751565 = {S31206, 162, { &P20603,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-C800 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA754276 = {S31207, 162, { &P20603,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; LG-C800 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA1995466 = {S31208, 164, { &P20603,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-C800 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA1998769 = {S31209, 60, { &P20603,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-C800 Build/GRK39F)*/
const Device UA2029198 = {S31210, 163, { &P20603,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; LG-C800 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2030740 = {S31211, 163, { &P20603,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us;LG-C800 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2032127 = {S31212, 163, { &P20603,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.6; en-us; LG-C800 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2033034 = {S31213, 163, { &P20603,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-C800 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2033470 = {S31214, 163, { &P20603,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-C800Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2033528 = {S31215, 60, { &P20603,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-C800G Build/GRJ22)*/
const Device UA2042027 = {S31216, 165, { &P20603,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; LG-C800G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2549247 = {S31217, 61, { &P20603,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-C800G Build/GRK39F)*/
const Device UA2408254 = {S31218, 118, { &P15364,&P5690,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA2434550 = {S31219, 128, { &P15364,&P5690,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1,gzip(gfe)*/
const Device UA2458914 = {S31220, 125, { &P15364,&P5690,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA2582676 = {S31221, 118, { &P15364,&P5690,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.3.6000 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA2604430 = {S31222, 125, { &P15364,&P5690,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.3.6000 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA2712024 = {S31223, 118, { &P15364,&P5690,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1*/
const Device UA2752712 = {S31224, 128, { &P15364,&P5690,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1,gzip(gfe)*/
const Device UA2753369 = {S31225, 125, { &P15364,&P5690,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1*/
const Device UA2754393 = {S31226, 125, { &P15364,&P5690,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1*/
const Device UA2768922 = {S31227, 118, { &P15364,&P5690,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1*/
const Device UA3129179 = {S31228, 112, { &P15364,&P5690,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Maxthon/4.0 Chrome/30.0.1551.0 Safari/537.36*/
const Device UA39416 = {S31229, 137, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB526 Build/4.5.1_128) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48838 = {S31230, 144, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB526 Build/4.5.1-134_DFP-67) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48841 = {S31231, 144, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB526 Build/4.5.1-134_DFP-82) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48843 = {S31232, 144, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; MB526 Build/4.5.1-134_DFP-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72752 = {S31233, 144, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; MB526 Build/4.5.1-134_DFP-74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA363505 = {S31234, 145, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MB526 Build/4.5.1-134_DFP-132) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368669 = {S31235, 149, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MotoMB526 Build/4.5.1-134_DFP-188) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370827 = {S31236, 155, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MB526 Build/4.5.3-134_DFP-132-anzhi.com) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA421956 = {S31237, 145, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; he-il; MB526 Build/4.5.1-134_DFP-161) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623147 = {S31238, 145, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; sv-se; MB526 Build/4.5.1-134_DFP-137) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625025 = {S31239, 145, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; MB526 Build/4.5.1-134_DFP-891) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625160 = {S31240, 146, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MB526 Build/4.5.1-134_DFP-1881) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA628719 = {S31241, 146, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; MB526 Build/4.5.1-134_DFP-1321) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA639473 = {S31242, 145, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; MB526 Build/4.5.1-134_DFP-125) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA662394 = {S31243, 145, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MB526 Build/4.5.1-134_DFP-231) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA677960 = {S31244, 69, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; MB526 Build/4.5.1-134_DFP-231)*/
const Device UA694488 = {S31245, 143, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; MB526 Build/4.5.2-51_DFL-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA698868 = {S31246, 144, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; MB526 Build/4.5.1-134_DFP-89) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA708303 = {S31247, 143, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; MB526 Build/4.5.2-51_DFL-50) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA719219 = {S31248, 143, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; MB526 Build/4.5.2-51_DFL-39) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA725765 = {S31249, 150, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; MotoMB526 Build/4.5.1-134_DFP-1251) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA726148 = {S31250, 145, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; MB526 Build/4.5.1-134_DFP-823) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA747286 = {S31251, 134, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MB526 Build/4.5.1-134_DFP-1321) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA754687 = {S31252, 145, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; MB526 Build/4.5.1-134_DFP-821) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA771938 = {S31253, 149, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MotoMB526 Build/4.5.1-134_DFP-231) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA788142 = {S31254, 150, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MotoMB526 Build/4.5.1-134_DFP-2312) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA793804 = {S31255, 145, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; MB526 Build/4.5.1-134_DFP-822) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA802215 = {S31256, 146, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MB526 Build/4.5.1-134_DFP-2311) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1936919 = {S31257, 145, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MB526 Build/4.5.1-134_DFP-139) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1937437 = {S31258, 145, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MB526 Build/4.5.1-134_DFP-188) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1952819 = {S31259, 146, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; MB526 Build/4.5.1-134_DFP-1252) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021109 = {S31260, 144, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB526 Build/4.5.1-134_DFP-24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021458 = {S31261, 133, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MB526 Build/4.5.1-134_DFP-137) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2023180 = {S31262, 69, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; MB526 Build/4.5.1-134_DFP-891)*/
const Device UA2023196 = {S31263, 144, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB526 Build/4.5.1-134_DFP-31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025819 = {S31264, 139, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MB526 Build/4.5.1-134_DFP-1321) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2026403 = {S31265, 133, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MB526 Build/4.5.1-134_DFP-231) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2026704 = {S31266, 138, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MB526 Build/4.5.1-134_DFP-231) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2027345 = {S31267, 145, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; MB526 Build/4.5.1-134_DFP-146) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028091 = {S31268, 146, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MB526 Build/4.5.1-134_DFP-2312) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031295 = {S31269, 144, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB526 Build/4.5.2-109_DHT-25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031488 = {S31270, 121, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MB526 Build/GWK74) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2393649 = {S31271, 67, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; MB526 Build/4.5.2-51_DFL-50)*/
const Device UA2590167 = {S31272, 135, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MB526 Build/WajkIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2642048 = {S31273, 126, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MB526 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2685559 = {S31274, 146, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; MB526 Build/4.5.1-134_DFP-1251) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2979836 = {S31275, 145, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; MB526 Build/4.5.1-134_DFP-120) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3041454 = {S31276, 57, { &P12848,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; MB526 Build/GWK74)*/
const Device UA2405151 = {S31277, 56, { &P18088,&P17017,&P18332,&P18092 }};	/*Useragent - Opera/9.50 (J2ME/MIDP; Opera Mini/2.1197/28.3165; U; en)*/
const Device UA2709945 = {S31278, 79, { &P18088,&P17017,&P18332,&P18092 }};	/*Useragent - Opera/9.80 (MTK; Opera Mini/2.1198/29.3417; U; en) Presto/2.8.119 Version/11.10*/
const Device UA238581 = {S31279, 63, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; en) Presto/2.9.168 Version/11.51*/
const Device UA242476 = {S31280, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.9.168 Version/11.51*/
const Device UA242828 = {S31281, 63, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; en) Presto/2.9.168 Version/11.51*/
const Device UA242847 = {S31282, 86, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.9.168 Version/11.51*/
const Device UA242848 = {S31283, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.9.168 Version/11.51*/
const Device UA242998 = {S31284, 86, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.9.168 Version/11.51*/
const Device UA243133 = {S31285, 100, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.9.168 Version/11.51,gzip(gfe) (via translate.google.com)*/
const Device UA243698 = {S31286, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.9.168 Version/11.51*/
const Device UA243702 = {S31287, 86, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.9.168 Version/11.51*/
const Device UA243917 = {S31288, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.9.168 Version/11.51*/
const Device UA243925 = {S31289, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.9.168 Version/11.51*/
const Device UA243936 = {S31290, 86, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.9.168 Version/11.51*/
const Device UA243968 = {S31291, 86, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.9.168 Version/11.51*/
const Device UA243993 = {S31292, 86, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.9.168 Version/11.51*/
const Device UA244627 = {S31293, 86, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.9.168 Version/11.51*/
const Device UA244630 = {S31294, 112, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94; ru) Presto/2.9.168 Version/11.51*/
const Device UA248777 = {S31295, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4841); ru) Presto/2.9.168 Version/11.51*/
const Device UA250297 = {S31296, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.9.168 Version/11.51*/
const Device UA336648 = {S31297, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.9.168 Version/11.51*/
const Device UA336671 = {S31298, 86, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.9.168 Version/11.51*/
const Device UA336817 = {S31299, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.9.168 Version/11.51*/
const Device UA337236 = {S31300, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.9.168 Version/11.51*/
const Device UA337235 = {S31301, 86, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.9.168 Version/11.51*/
const Device UA337588 = {S31302, 86, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.9.168 Version/11.51*/
const Device UA342229 = {S31303, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4832); ru) Presto/2.9.168 Version/11.51*/
const Device UA379733 = {S31304, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4871); ru) Presto/2.9.168 Version/11.51*/
const Device UA421374 = {S31305, 86, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.9.168 Version/11.51*/
const Device UA580517 = {S31306, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.9.168 Version/11.51*/
const Device UA608986 = {S31307, 86, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.9.168 Version/11.51*/
const Device UA665995 = {S31308, 86, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.9.168 Version/11.51*/
const Device UA720359 = {S31309, 66, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; ru) Opera 11.51*/
const Device UA760242 = {S31310, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4661); ru) Presto/2.9.168 Version/11.51*/
const Device UA769408 = {S31311, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.9.168 Version/11.51*/
const Device UA836389 = {S31312, 72, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; ru; rv:2.0) Gecko/X Firefox/4.0 Opera 11.51*/
const Device UA1898143 = {S31313, 86, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03791); ru) Presto/2.9.168 Version/11.51*/
const Device UA1898952 = {S31314, 86, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.9.168 Version/11.51*/
const Device UA1977848 = {S31315, 86, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.9.168 Version/11.51*/
const Device UA2154941 = {S31316, 86, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.9.168 Version/11.51*/
const Device UA2156398 = {S31317, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.9.168 Version/11.51*/
const Device UA2298205 = {S31318, 86, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.9.168 Version/11.51*/
const Device UA2307787 = {S31319, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.9.168 Version/11.51*/
const Device UA2312978 = {S31320, 86, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.9.168 Version/11.51*/
const Device UA2375244 = {S31321, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.9.168 Version/11.51*/
const Device UA2380853 = {S31322, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.9.168 Version/11.51*/
const Device UA2381063 = {S31323, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.9.168 Version/11.51*/
const Device UA2466354 = {S31324, 73, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.9.168 Version/11.51,gzip(gfe)*/
const Device UA2551254 = {S31325, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.9.168 Version/11.51*/
const Device UA2573470 = {S31326, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.9.168 Version/11.51*/
const Device UA2656570 = {S31327, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.9.168 Version/11.51*/
const Device UA2758563 = {S31328, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.9.168 Version/11.51*/
const Device UA2790398 = {S31329, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.9.168 Version/11.51*/
const Device UA2843093 = {S31330, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.9.168 Version/11.51*/
const Device UA2858081 = {S31331, 85, { &P15364,&P5690,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.9.168 Version/11.51*/
const Device UA2043051 = {S31332, 80, { &P15364,&P18110,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:15.0) Gecko/X Firefox/15.0.1 Iceweasel/15.0.1*/
const Device UA2043085 = {S31333, 63, { &P15364,&P18110,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:15.0) Gecko/X Firefox/15.0.1*/
const Device UA2258522 = {S31334, 100, { &P15364,&P18110,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Ultimate Edition; X11; Linux x86; rv:8.0) Gecko/X Ultimate Edition / 3.5 Firefox/15.0.1*/
const Device UA2451322 = {S31335, 76, { &P15364,&P18110,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:15.0) Gecko/X Firefox/15.0.1 Firefox/15.0.1*/
const Device UA2537633 = {S31336, 73, { &P15364,&P18110,&P18345,&P18092 }};	/*Useragent - Mozilla/1.2 (Mozilla/5.0 (X11; Linux i686; rv:15.0) Gecko/X Firefox/15.0)*/
const Device UA2595484 = {S31337, 71, { &P15364,&P18110,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:15.0) Gecko/X Debian Iceweasel/15.02*/
const Device UA2008913 = {S31338, 61, { &P15364,&P5690,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:18.0) Gecko/18.0 Firefox/18.0*/
const Device UA2186799 = {S31339, 73, { &P15364,&P5690,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:18.0) Gecko/18.0 Firefox/18.0*/
const Device UA2390623 = {S31340, 65, { &P15364,&P5690,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:18.0) Gecko/X Firefox/18.0*/
const Device UA2532961 = {S31341, 107, { &P15364,&P5690,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.1) Gecko/X Firefox/18.0.0.1[en]C-CCK-MCD DOJ3jx7bf*/
const Device UA347237 = {S31342, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.60*/
const Device UA347492 = {S31343, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4930); ru) Presto/2.10.229 Version/11.60*/
const Device UA347573 = {S31344, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.10.229 Version/11.60*/
const Device UA347706 = {S31345, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.229 Version/11.60*/
const Device UA347733 = {S31346, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.229 Version/11.60*/
const Device UA347761 = {S31347, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.229 Version/11.60*/
const Device UA347769 = {S31348, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.229 Version/11.60*/
const Device UA351436 = {S31349, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.229 Version/11.60*/
const Device UA351489 = {S31350, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.229 Version/11.60*/
const Device UA351536 = {S31351, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.229 Version/11.60*/
const Device UA351680 = {S31352, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.229 Version/11.60*/
const Device UA351712 = {S31353, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.229 Version/11.60*/
const Device UA351715 = {S31354, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.229 Version/11.60*/
const Device UA351717 = {S31355, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.60*/
const Device UA351726 = {S31356, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.229 Version/11.60*/
const Device UA351750 = {S31357, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4110); ru) Presto/2.10.229 Version/11.60*/
const Device UA351767 = {S31358, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.229 Version/11.60*/
const Device UA351785 = {S31359, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.229 Version/11.60*/
const Device UA352779 = {S31360, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4661); ru) Presto/2.10.229 Version/11.60*/
const Device UA353503 = {S31361, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.60*/
const Device UA355815 = {S31362, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03649); ru) Presto/2.10.229 Version/11.60*/
const Device UA356545 = {S31363, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03638); ru) Presto/2.10.229 Version/11.60*/
const Device UA358816 = {S31364, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.229 Version/11.60*/
const Device UA363983 = {S31365, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.60*/
const Device UA454796 = {S31366, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.229 Version/11.60*/
const Device UA605814 = {S31367, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.229 Version/11.60*/
const Device UA622547 = {S31368, 64, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.229 Version/11.60*/
const Device UA628989 = {S31369, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.229 Version/11.60*/
const Device UA630830 = {S31370, 101, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.229 Version/11.60,gzip(gfe) (via translate.google.com)*/
const Device UA641621 = {S31371, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.60*/
const Device UA642788 = {S31372, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.229 Version/11.60*/
const Device UA644935 = {S31373, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.229 Version/11.60*/
const Device UA655377 = {S31374, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.229 Version/11.60*/
const Device UA667566 = {S31375, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03755); ru) Presto/2.10.229 Version/11.60*/
const Device UA671465 = {S31376, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.229 Version/11.60*/
const Device UA681738 = {S31377, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.229 Version/11.60*/
const Device UA704558 = {S31378, 66, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; ru) Opera 11.60*/
const Device UA715980 = {S31379, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03278); ru) Presto/2.10.229 Version/11.60*/
const Device UA742697 = {S31380, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03639); ru) Presto/2.10.229 Version/11.60*/
const Device UA896820 = {S31381, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.229 Version/11.60*/
const Device UA901902 = {S31382, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.229 Version/11.60*/
const Device UA1934622 = {S31383, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03672); ru) Presto/2.10.229 Version/11.60*/
const Device UA1934995 = {S31384, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.6 (build 03402); ru) Presto/2.10.229 Version/11.60*/
const Device UA1935221 = {S31385, 72, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; ru; rv:2.0) Gecko/X Firefox/4.0 Opera 11.60*/
const Device UA1935534 = {S31386, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.229 Version/11.60*/
const Device UA1951523 = {S31387, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.229 Version/11.60*/
const Device UA1952670 = {S31388, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.229 Version/11.60*/
const Device UA1956320 = {S31389, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5218); ru) Presto/2.10.229 Version/11.60*/
const Device UA1971426 = {S31390, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5328); ru) Presto/2.10.229 Version/11.60*/
const Device UA1978336 = {S31391, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.229 Version/11.60*/
const Device UA1990229 = {S31392, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.229 Version/11.60*/
const Device UA2167059 = {S31393, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.10.229 Version/11.60*/
const Device UA2178345 = {S31394, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.10.229 Version/11.60*/
const Device UA2184561 = {S31395, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.229 Version/11.60*/
const Device UA2232463 = {S31396, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.10.229 Version/11.60*/
const Device UA2306804 = {S31397, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03780); ru) Presto/2.10.229 Version/11.60*/
const Device UA2318260 = {S31398, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5667); ru) Presto/2.10.229 Version/11.60*/
const Device UA2318401 = {S31399, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5230); ru) Presto/2.10.229 Version/11.60*/
const Device UA2334907 = {S31400, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.229 Version/11.60*/
const Device UA2366890 = {S31401, 74, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.10.229 Version/11.60,gzip(gfe)*/
const Device UA2385687 = {S31402, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5966); ru) Presto/2.10.229 Version/11.60*/
const Device UA2386178 = {S31403, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.229 Version/11.60*/
const Device UA2403370 = {S31404, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.229 Version/11.60*/
const Device UA2422147 = {S31405, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.229 Version/11.60*/
const Device UA2468149 = {S31406, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.229 Version/11.60*/
const Device UA2494877 = {S31407, 87, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5282); ru) Presto/2.10.229 Version/11.60*/
const Device UA2589544 = {S31408, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.229 Version/11.60*/
const Device UA2594532 = {S31409, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5745); ru) Presto/2.10.229 Version/11.60*/
const Device UA2595128 = {S31410, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5880); ru) Presto/2.10.229 Version/11.60*/
const Device UA2600041 = {S31411, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.229 Version/11.60*/
const Device UA2653210 = {S31412, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5784); ru) Presto/2.10.229 Version/11.60*/
const Device UA2705128 = {S31413, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.229 Version/11.60*/
const Device UA2722992 = {S31414, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5676); ru) Presto/2.10.229 Version/11.60*/
const Device UA2737186 = {S31415, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6089); ru) Presto/2.10.229 Version/11.60*/
const Device UA2747538 = {S31416, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6011); ru) Presto/2.10.229 Version/11.60*/
const Device UA2764478 = {S31417, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6054); ru) Presto/2.10.229 Version/11.60*/
const Device UA2846310 = {S31418, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5977); ru) Presto/2.10.229 Version/11.60*/
const Device UA2860390 = {S31419, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6083); ru) Presto/2.10.229 Version/11.60*/
const Device UA2867497 = {S31420, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.229 Version/11.60*/
const Device UA2953943 = {S31421, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6081); ru) Presto/2.10.229 Version/11.60*/
const Device UA3071063 = {S31422, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5981); ru) Presto/2.10.229 Version/11.60*/
const Device UA3101168 = {S31423, 86, { &P15364,&P7133,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 6008); ru) Presto/2.10.229 Version/11.60*/
const Device UA79968 = {S31424, 82, { &P15364,&P17285,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/15*/
const Device UA91034 = {S31425, 106, { &P15364,&P17285,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA91044 = {S31426, 99, { &P15364,&P17285,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA252543 = {S31427, 143, { &P15364,&P17285,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.106 Safari/535.2,gzip(gfe) (via translate.google.com)*/
const Device UA254953 = {S31428, 136, { &P15364,&P17285,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2,gzip(gfe) (via translate.google.com)*/
const Device UA688582 = {S31429, 114, { &P15364,&P17285,&P17254,&P18092 }};	/*Useragent - Chrome/15.0.860.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/15.0.860.0*/
const Device UA2673769 = {S31430, 114, { &P15364,&P17285,&P17254,&P18092 }};	/*Useragent - Chrome/15.0.860.0 (Windows; U; Windows NT 6.0; es-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/15.0.860.0*/
const Device UA2708160 = {S31431, 114, { &P15364,&P17285,&P17254,&P18092 }};	/*Useragent - Chrome/15.0.860.0 (Windows; U; Windows NT 6.0; fr-FR) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/15.0.860.0*/
const Device UA2708861 = {S31432, 114, { &P15364,&P17285,&P17254,&P18092 }};	/*Useragent - Chrome/15.0.860.0 (Windows; U; Windows NT 6.0; en-CA) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/15.0.860.0*/
const Device UA2709384 = {S31433, 115, { &P15364,&P17285,&P17254,&P18092 }};	/*Useragent - Chrome/15.0.860.0 (Windows; U; Windows NT 6.0; haw-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/15.0.860.0*/
const Device UA2735319 = {S31434, 114, { &P15364,&P17285,&P17254,&P18092 }};	/*Useragent - Chrome/15.0.860.0 (Windows; U; Windows NT 6.0; fr-CA) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/15.0.860.0*/
const Device UA2768130 = {S31435, 114, { &P15364,&P17285,&P17254,&P18092 }};	/*Useragent - Chrome/15.0.860.0 (Windows; U; Windows NT 6.0; zh-CN) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/15.0.860.0*/
const Device UA2818063 = {S31436, 209, { &P15364,&P17285,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.2 (KHTML like Gecko) Chrome/15.0.860.0 Safari/535.2Chrome/15.0.860.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.20.25 (KHTML like Gecko) Version/15.0.860.0*/
const Device UA2860027 = {S31437, 114, { &P15364,&P17285,&P17254,&P18092 }};	/*Useragent - Chrome/15.0.860.0 (Windows; U; Windows NT 6.0; pl-DE) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/15.0.860.0*/
const Device UA1989077 = {S31438, 90, { &P17595,&P18207,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en-US; rv:1.9.2.28) Gecko/X Firefox/3.6.28*/
const Device UA40204 = {S31439, 168, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; LG-P500/V20d Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA40221 = {S31440, 139, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; uk-ua; LG-P500 Build/GRI40) AppleWebKit/528.5+ (KHTML, O: Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA40275 = {S31441, 165, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; LG-P500 Build/AGA-ROM) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA40294 = {S31442, 181, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; LG-P500 Build/openOptimus build 2.202) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA40273 = {S31443, 175, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3 by Snegovik; ru-ru; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA41026 = {S31444, 175, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-sa; LG-P500h/v20a Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA55474 = {S31445, 168, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; LG-P500/v20g Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA55487 = {S31446, 169, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; LG-P500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA57939 = {S31447, 163, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA68331 = {S31448, 141, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA77845 = {S31449, 168, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; LG-P500/v20h Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA219623 = {S31450, 135, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA251011 = {S31451, 135, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA396110 = {S31452, 59, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; LG-P500 Build/GRI40)*/
const Device UA578485 = {S31453, 150, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P500 Build/GRK39F; Oxygen-2.3.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622712 = {S31454, 164, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; LG-P500h Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA632913 = {S31455, 175, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-mx; LG-P500h/v10a Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA634177 = {S31456, 60, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; LG-P500h Build/GRI40)*/
const Device UA651872 = {S31457, 169, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; LG-P500h-parrot Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA660649 = {S31458, 171, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; LG-P500h-parrot Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA685378 = {S31459, 59, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; LG-P500 Build/GRI40)*/
const Device UA700167 = {S31460, 123, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-in; LG-P500 Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA714613 = {S31461, 168, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; LG-P500/V20i Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA735941 = {S31462, 162, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; LG-P500h Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2018757 = {S31463, 65, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P500 Build/GINGERBREAD)*/
const Device UA2020054 = {S31464, 165, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; LG Optimus One P500 Build/openOptimus build 2.202) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020534 = {S31465, 173, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; LG-P500-Bouygues/ Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2021589 = {S31466, 59, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; LG-P500 Build/GRI40)*/
const Device UA2024034 = {S31467, 155, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P500 Build/GINGERBREAD; Oxygen-2.3.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2933190 = {S31468, 136, { &P14967,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-mx; LG-P500h Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Versi?n/4.0 Mobile Safari/533.1*/
const Device UA2670194 = {S31469, 57, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.15*/
const Device UA2671998 = {S31470, 80, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.15*/
const Device UA2672069 = {S31471, 80, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.15*/
const Device UA2672596 = {S31472, 67, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.15,gzip(gfe)*/
const Device UA2673897 = {S31473, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.15*/
const Device UA2674171 = {S31474, 64, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; WOW64) Presto/2.12.388 Version/12.15*/
const Device UA2674751 = {S31475, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.15*/
const Device UA2674836 = {S31476, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.15*/
const Device UA2676384 = {S31477, 68, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:2.0) Gecko/X Firefox/4.0 Opera 12.15*/
const Device UA2707306 = {S31478, 80, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.15*/
const Device UA2708036 = {S31479, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.15*/
const Device UA2708387 = {S31480, 116, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)) Presto/2.12.388 Version/12.15*/
const Device UA2709110 = {S31481, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.15*/
const Device UA2709162 = {S31482, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.15*/
const Device UA2709488 = {S31483, 80, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.15*/
const Device UA2710430 = {S31484, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.15*/
const Device UA2710832 = {S31485, 80, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.15*/
const Device UA2711011 = {S31486, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.15*/
const Device UA2711496 = {S31487, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.15*/
const Device UA2711810 = {S31488, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.15*/
const Device UA2711924 = {S31489, 80, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.15*/
const Device UA2711932 = {S31490, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.15*/
const Device UA2712376 = {S31491, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.15*/
const Device UA2712940 = {S31492, 88, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; Edition Campaign 21) Presto/2.12.388 Version/12.15,gzip(gfe)*/
const Device UA2713666 = {S31493, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.15*/
const Device UA2713667 = {S31494, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.15*/
const Device UA2716664 = {S31495, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.15*/
const Device UA2734705 = {S31496, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.15*/
const Device UA2734793 = {S31497, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.15*/
const Device UA2734960 = {S31498, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.15*/
const Device UA2735031 = {S31499, 62, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0) Opera 12.15*/
const Device UA2735548 = {S31500, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.15*/
const Device UA2735621 = {S31501, 80, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.15*/
const Device UA2737433 = {S31502, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.15*/
const Device UA2738146 = {S31503, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.15*/
const Device UA2750803 = {S31504, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.15*/
const Device UA2750894 = {S31505, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.15*/
const Device UA2750962 = {S31506, 106, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.15*/
const Device UA2751167 = {S31507, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.15*/
const Device UA2767280 = {S31508, 80, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.15*/
const Device UA2767921 = {S31509, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5997)) Presto/2.12.388 Version/12.15*/
const Device UA2768492 = {S31510, 95, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; compatible; MSIE 7.0; Windows NT 6.0) Presto/2.12.388 Version/12.15*/
const Device UA2774129 = {S31511, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.15*/
const Device UA2780994 = {S31512, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.15*/
const Device UA2793941 = {S31513, 80, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.15*/
const Device UA2795187 = {S31514, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.15*/
const Device UA2795291 = {S31515, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.15*/
const Device UA2795789 = {S31516, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.15*/
const Device UA2860421 = {S31517, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.15*/
const Device UA2861859 = {S31518, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.15*/
const Device UA2864032 = {S31519, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.15*/
const Device UA2864463 = {S31520, 80, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.15*/
const Device UA2891710 = {S31521, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.15*/
const Device UA2936287 = {S31522, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.15*/
const Device UA2955704 = {S31523, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.15*/
const Device UA2993475 = {S31524, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.15*/
const Device UA2995176 = {S31525, 80, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.15*/
const Device UA3014091 = {S31526, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.15*/
const Device UA3072859 = {S31527, 79, { &P15364,&P17285,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.15*/
const Device UA138441 = {S31528, 168, { &P15364,&P17285,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; TmstmpExt)*/
const Device UA151983 = {S31529, 147, { &P15364,&P17285,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; TmstmpExt)*/
const Device UA164085 = {S31530, 175, { &P15364,&P17285,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; TmstmpExt)*/
const Device UA439167 = {S31531, 215, { &P15364,&P17285,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.0; Trident/4.0; Mozilla/4.0 ; MSIE 6.0; Windows NT 5.1; SV1; ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; TmstmpExt)*/
const Device UA536746 = {S31532, 196, { &P15364,&P17285,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; TmstmpExt)*/
const Device UA555306 = {S31533, 178, { &P15364,&P17285,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7; TmstmpExt)*/
const Device UA1881759 = {S31534, 157, { &P15364,&P17285,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Zune 4.7; TmstmpExt)*/
const Device UA1947394 = {S31535, 107, { &P15364,&P17285,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; .NET4.0C; .NET CLR 3.5.30729; TmstmpExt)*/
const Device UA2140485 = {S31536, 183, { &P15364,&P17285,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Tablet PC 2.0; TmstmpExt)*/
const Device UA2354596 = {S31537, 142, { &P17625,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST25i Build/6.1.1.B.1.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2354661 = {S31538, 154, { &P17625,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyEricssonST25a Build/6.1.1.B.1.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2362932 = {S31539, 64, { &P17625,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST25i Build/6.1.1.B.1.10)*/
const Device UA2404526 = {S31540, 154, { &P17625,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonST25i Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2429485 = {S31541, 155, { &P17625,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; SonyEricssonST25iv Build/6.1.1.B.1.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2430905 = {S31542, 156, { &P17625,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SonyEricssonST25i-o Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2459601 = {S31543, 142, { &P17625,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST25i Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2480126 = {S31544, 64, { &P17625,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST25a Build/6.1.1.B.1.10)*/
const Device UA2480308 = {S31545, 142, { &P17625,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST25a Build/6.1.1.B.1.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2696694 = {S31546, 142, { &P17625,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; ST25a Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747220 = {S31547, 141, { &P17625,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST25i Build/6.1.1.C.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768566 = {S31548, 142, { &P17625,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; ST25i Build/6.1.1.B.1.89) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3056818 = {S31549, 143, { &P17625,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ST25i Build/6.1.1.B.1.100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2035223 = {S31550, 144, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST26i Build/11.0.A.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257505 = {S31551, 145, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST26i Build/11.0.A.3.18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2362680 = {S31552, 141, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST26i Build/11.0.A.3.18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2368790 = {S31553, 145, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST26i Build/11.0.A.3.28) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2385431 = {S31554, 147, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; SonyST26i-o Build/11.0.A.3.18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2385978 = {S31555, 145, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SonyST26i Build/11.0.A.3.22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2404039 = {S31556, 145, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; SonyST26a Build/11.0.A.3.18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2427015 = {S31557, 146, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; SonyST26iv Build/11.0.A.3.28) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2427649 = {S31558, 141, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST26i Build/11.0.A.3.28) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2437268 = {S31559, 144, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST26i Build/11.0.A.7.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2457097 = {S31560, 141, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; ST26i Build/11.0.A.3.22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2459034 = {S31561, 62, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST26i Build/11.0.A.7.5)*/
const Device UA2461027 = {S31562, 140, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST26i Build/11.0.A.7.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2485046 = {S31563, 146, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; SonyST26i-o Build/11.0.A.7.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2507301 = {S31564, 144, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; SonyST26a Build/11.0.A.7.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2510658 = {S31565, 144, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyST26i Build/11.0.A.7.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2531148 = {S31566, 140, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST26i Build/11.0.A.7.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2534867 = {S31567, 145, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; SonyST26iv Build/11.0.A.7.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2553289 = {S31568, 146, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; SonyST26iv Build/11.0.A.3.18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2579549 = {S31569, 145, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; iw-il; SonyST26a Build/11.0.A.3.28) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2670296 = {S31570, 145, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-mx; SonyST26a Build/11.0.A.7.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2732012 = {S31571, 140, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; ST26a Build/11.0.A.7.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2791266 = {S31572, 62, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST26a Build/11.0.A.7.5)*/
const Device UA2795539 = {S31573, 141, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-mx; ST26a Build/11.0.A.7.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2844894 = {S31574, 141, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; ST26a Build/11.0.A.3.28) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986316 = {S31575, 64, { &P18448,&P18066,&P18062,&P18092 }};	/*Useragent - SonyST26i Build/11.0.A.7.5 stagefright/1.2 (Linux;Android 4.0.4)*/
const Device UA5093 = {S31576, 153, { &P14864,&P5267,&P3834,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8500M/S8500MUGJF5; U; Bada/1.0; en-ca) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA53805 = {S31577, 145, { &P14864,&P5267,&P3834,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8500B/1.0; U; Bada/1.0; pt-br) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA105148 = {S31578, 151, { &P14864,&P5267,&P3834,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8500/S8500XXJEE; U; Bada/1.0; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA2510812 = {S31579, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.14*/
const Device UA2516401 = {S31580, 57, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2) Presto/2.12.388 Version/12.14*/
const Device UA2516517 = {S31581, 64, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.14*/
const Device UA2531656 = {S31582, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.14*/
const Device UA2531761 = {S31583, 74, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.14,gzip(gfe)*/
const Device UA2533241 = {S31584, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.14*/
const Device UA2535475 = {S31585, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.14*/
const Device UA2537095 = {S31586, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.14*/
const Device UA2537275 = {S31587, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.14*/
const Device UA2558480 = {S31588, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.14*/
const Device UA2561080 = {S31589, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.14*/
const Device UA2561577 = {S31590, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.14*/
const Device UA2562518 = {S31591, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.14*/
const Device UA2563855 = {S31592, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.14*/
const Device UA2569920 = {S31593, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.14*/
const Device UA2570877 = {S31594, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.14*/
const Device UA2571058 = {S31595, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.14*/
const Device UA2571455 = {S31596, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.14*/
const Device UA2572155 = {S31597, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.14*/
const Device UA2580561 = {S31598, 67, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2) Presto/2.12.388 Version/12.14,gzip(gfe)*/
const Device UA2581512 = {S31599, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.14*/
const Device UA2581742 = {S31600, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.14*/
const Device UA2598879 = {S31601, 75, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.14*/
const Device UA2599844 = {S31602, 69, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64) Opera 12.14*/
const Device UA2601604 = {S31603, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.14*/
const Device UA2603273 = {S31604, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.14*/
const Device UA2662209 = {S31605, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.14*/
const Device UA2662416 = {S31606, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.14*/
const Device UA2663693 = {S31607, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.14*/
const Device UA2668756 = {S31608, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.14*/
const Device UA2670009 = {S31609, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.14*/
const Device UA2699724 = {S31610, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.14*/
const Device UA2700643 = {S31611, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.14*/
const Device UA2702323 = {S31612, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.14*/
const Device UA2705929 = {S31613, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.14*/
const Device UA2716656 = {S31614, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.14*/
const Device UA2730562 = {S31615, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5997)) Presto/2.12.388 Version/12.14*/
const Device UA2733505 = {S31616, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6080)) Presto/2.12.388 Version/12.14*/
const Device UA2792345 = {S31617, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.14*/
const Device UA2800514 = {S31618, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.14*/
const Device UA2853299 = {S31619, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5861)) Presto/2.12.388 Version/12.14*/
const Device UA2854020 = {S31620, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6035)) Presto/2.12.388 Version/12.14*/
const Device UA2856777 = {S31621, 87, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.14*/
const Device UA3048845 = {S31622, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.14*/
const Device UA3125442 = {S31623, 86, { &P15364,&P16110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.14*/
const Device UA2890986 = {S31624, 64, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.16*/
const Device UA2900488 = {S31625, 57, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2) Presto/2.12.388 Version/12.16*/
const Device UA2944825 = {S31626, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.16*/
const Device UA2946472 = {S31627, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.16*/
const Device UA2946725 = {S31628, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.16*/
const Device UA2946726 = {S31629, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.16*/
const Device UA2948048 = {S31630, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.16*/
const Device UA2964039 = {S31631, 74, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.16,gzip(gfe)*/
const Device UA2964204 = {S31632, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6657)) Presto/2.12.388 Version/12.16*/
const Device UA2964303 = {S31633, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.16*/
const Device UA2964528 = {S31634, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.16*/
const Device UA2964945 = {S31635, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6283)) Presto/2.12.388 Version/12.16*/
const Device UA2965358 = {S31636, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.16*/
const Device UA2965361 = {S31637, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6322)) Presto/2.12.388 Version/12.16*/
const Device UA2966136 = {S31638, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.16*/
const Device UA2966144 = {S31639, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.16*/
const Device UA2966316 = {S31640, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.16*/
const Device UA2968242 = {S31641, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6665)) Presto/2.12.388 Version/12.16*/
const Device UA3010771 = {S31642, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.16*/
const Device UA3011453 = {S31643, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6638)) Presto/2.12.388 Version/12.16*/
const Device UA3012810 = {S31644, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.16*/
const Device UA3012979 = {S31645, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.16*/
const Device UA3013031 = {S31646, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.16*/
const Device UA3013170 = {S31647, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6573)) Presto/2.12.388 Version/12.16*/
const Device UA3013200 = {S31648, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6327)) Presto/2.12.388 Version/12.16*/
const Device UA3013551 = {S31649, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.16*/
const Device UA3014154 = {S31650, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6648)) Presto/2.12.388 Version/12.16*/
const Device UA3014184 = {S31651, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.16*/
const Device UA3014703 = {S31652, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.16*/
const Device UA3016191 = {S31653, 75, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.16*/
const Device UA3016601 = {S31654, 67, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2) Presto/2.12.388 Version/12.16,gzip(gfe)*/
const Device UA3016742 = {S31655, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6337)) Presto/2.12.388 Version/12.16*/
const Device UA3017253 = {S31656, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6628)) Presto/2.12.388 Version/12.16*/
const Device UA3017575 = {S31657, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.16*/
const Device UA3017794 = {S31658, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6323)) Presto/2.12.388 Version/12.16*/
const Device UA3017812 = {S31659, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.16*/
const Device UA3018904 = {S31660, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6341)) Presto/2.12.388 Version/12.16*/
const Device UA3019135 = {S31661, 69, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64) Opera 12.16*/
const Device UA3020244 = {S31662, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6673)) Presto/2.12.388 Version/12.16*/
const Device UA3030200 = {S31663, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6675)) Presto/2.12.388 Version/12.16*/
const Device UA3033269 = {S31664, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.16*/
const Device UA3056648 = {S31665, 87, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.16*/
const Device UA3056966 = {S31666, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6325)) Presto/2.12.388 Version/12.16*/
const Device UA3056997 = {S31667, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.16*/
const Device UA3057018 = {S31668, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.16*/
const Device UA3057180 = {S31669, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.16*/
const Device UA3057307 = {S31670, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.16*/
const Device UA3057387 = {S31671, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6620)) Presto/2.12.388 Version/12.16*/
const Device UA3057561 = {S31672, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6669)) Presto/2.12.388 Version/12.16*/
const Device UA3058126 = {S31673, 84, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.16,gzip(gfe),gzip(gfe)*/
const Device UA3077329 = {S31674, 80, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.16*/
const Device UA3077873 = {S31675, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6261)) Presto/2.12.388 Version/12.16*/
const Device UA3078412 = {S31676, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6676)) Presto/2.12.388 Version/12.16*/
const Device UA3078493 = {S31677, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6681)) Presto/2.12.388 Version/12.16*/
const Device UA3078818 = {S31678, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6347)) Presto/2.12.388 Version/12.16*/
const Device UA3091819 = {S31679, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.16*/
const Device UA3091853 = {S31680, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.16*/
const Device UA3091866 = {S31681, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.16*/
const Device UA3092088 = {S31682, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.16*/
const Device UA3106981 = {S31683, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.16*/
const Device UA3107085 = {S31684, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.1 (build 6336)) Presto/2.12.388 Version/12.16*/
const Device UA3107108 = {S31685, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.16*/
const Device UA3107105 = {S31686, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6561)) Presto/2.12.388 Version/12.16*/
const Device UA3107701 = {S31687, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.16*/
const Device UA3127282 = {S31688, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.2 (build 7188)) Presto/2.12.388 Version/12.16*/
const Device UA3127401 = {S31689, 86, { &P15364,&P16110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.2 (build 7194)) Presto/2.12.388 Version/12.16*/
const Device UA49896 = {S31690, 143, { &P23905,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9100P Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623021 = {S31691, 162, { &P23905,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9100P-ORANGE/I9100PBVKI3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623080 = {S31692, 143, { &P23905,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100P Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626871 = {S31693, 162, { &P23905,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9100P-ORANGE/I9100PBVKL2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA631717 = {S31694, 67, { &P23905,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-I9100P Build/GINGERBREAD)*/
const Device UA633605 = {S31695, 143, { &P23905,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-I9100P Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA649569 = {S31696, 67, { &P23905,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9100P Build/GINGERBREAD)*/
const Device UA717781 = {S31697, 67, { &P23905,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-I9100P Build/GINGERBREAD)*/
const Device UA2809650 = {S31698, 141, { &P23905,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-I9100P Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2601063 = {S31699, 104, { &P18254,&P18110,&P21712,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.9.0 Safari/534.34*/
const Device UA2602311 = {S31700, 102, { &P18254,&P18110,&P21712,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.9.0 Safari/534.34*/
const Device UA722313 = {S31701, 75, { &P15364,&P16110,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:15.0) Gecko/15.0 Firefox/15.0a1*/
const Device UA2043686 = {S31702, 67, { &P15364,&P16110,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:15.0) Gecko/X Firefox/15.0.1*/
const Device UA6916 = {S31703, 159, { &P12484,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1 Community 2; es-es; HTC_Wildfire_A3333 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51152 = {S31704, 145, { &P12484,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pt-; HTC_Wildfire_A3333 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA63610 = {S31705, 147, { &P12484,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC_Wildfire_A3333 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA94086 = {S31706, 46, { &P15364,&P6729,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows 95)*/
const Device UA2273839 = {S31707, 62, { &P15364,&P6729,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.50; Windows 95; SiteKiosk 4.8)*/
const Device UA2261667 = {S31708, 140, { &P18471,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XT907 Build/9.8.1Q_27-2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2508767 = {S31709, 144, { &P18471,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT905 Build/9.8.2Q_SMUIRD-7) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2670034 = {S31710, 138, { &P18471,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-66) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3115868 = {S31711, 91, { &P18088,&P22710,&P17780,&P18092 }};	/*Useragent - Opera/9.80 (MAUI Runtime; Opera Mini/6.1.27412/31.1475; U; en) Presto/2.8.119 Version/11.10*/
const Device UA403353 = {S31712, 139, { &P18224,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U8815 Build/HuaweiU8815) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA607803 = {S31713, 146, { &P18224,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; U8815 Build/HuaweiU8815C02B882) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA734376 = {S31714, 146, { &P18224,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; U8815 Build/HuaweiU8815C02B885) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA895852 = {S31715, 139, { &P18224,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U8818 Build/HuaweiU8818) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA896830 = {S31716, 146, { &P18224,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-nz; U8815 Build/HuaweiU8815C02B884) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA899931 = {S31717, 146, { &P18224,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ie; U8815 Build/HuaweiU8815C02B886) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA910345 = {S31718, 148, { &P18224,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; U8815N Build/HuaweiU8815NC02B891) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA924477 = {S31719, 146, { &P18224,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; U8815 Build/HuaweiU8815C02B892) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1961420 = {S31720, 147, { &P18224,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; U8815 Build/HuaweiU8815C346B893) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995213 = {S31721, 153, { &P18224,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; HUAWEI_Ascend G 300 Build/HuaweiU8815) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026962 = {S31722, 147, { &P18224,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; U8815 Build/r0_de-branded_u8815) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030535 = {S31723, 146, { &P18224,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ie; U8815 Build/HuaweiU8815C02B893) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032740 = {S31724, 132, { &P18224,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; U8818 Build/HuaweiU8818) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2032964 = {S31725, 63, { &P18224,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; U8815 Build/HuaweiU8815)*/
const Device UA2034118 = {S31726, 152, { &P18224,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; U8815 Build/AtomicMod BX (By Tillaz)) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2200536 = {S31727, 148, { &P18224,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; U8815N Build/HuaweiU8815NC02B894) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2225811 = {S31728, 146, { &P18224,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; U8815 Build/HuaweiU8815C02B895) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2256984 = {S31729, 147, { &P18224,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; U8815 Build/HuaweiU8815C346B900) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2564320 = {S31730, 147, { &P18224,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; U8815 Build/HuaweiU8815C346B801) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2825414 = {S31731, 63, { &P18224,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; U8818 Build/HuaweiU8818)*/
const Device UA2461923 = {S31732, 67, { &P15364,&P18118,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:19.0) Gecko/X Firefox/19.0*/
const Device UA57116 = {S31733, 142, { &P13974,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA394949 = {S31734, 66, { &P13974,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5670 Build/GINGERBREAD)*/
const Device UA397223 = {S31735, 143, { &P13974,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT-S5670L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA578440 = {S31736, 142, { &P13974,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; hr-hr; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA637157 = {S31737, 142, { &P13974,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA651501 = {S31738, 66, { &P13974,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5670 Build/GINGERBREAD)*/
const Device UA664370 = {S31739, 143, { &P13974,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; GT-S5670L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA692362 = {S31740, 130, { &P13974,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5670 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA711265 = {S31741, 143, { &P13974,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-S5670L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022833 = {S31742, 66, { &P13974,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-S5670 Build/GINGERBREAD)*/
const Device UA2024964 = {S31743, 130, { &P13974,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5670 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2026042 = {S31744, 143, { &P13974,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-S5670B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026792 = {S31745, 153, { &P13974,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6(RazoDroiD); en-gb; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032081 = {S31746, 67, { &P13974,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5670L Build/GINGERBREAD)*/
const Device UA2034481 = {S31747, 135, { &P13974,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5670 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2364161 = {S31748, 140, { &P13974,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2410076 = {S31749, 140, { &P13974,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2747670 = {S31750, 141, { &P13974,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2787062 = {S31751, 67, { &P13974,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5670L Build/GINGERBREAD)*/
const Device UA635016 = {S31752, 139, { &P15802,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Amaze_4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA673979 = {S31753, 143, { &P15802,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_Amaze_4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA800603 = {S31754, 65, { &P15802,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC_Amaze_4G Build/IML74K)*/
const Device UA878270 = {S31755, 139, { &P15802,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; HTC Ruby Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA900157 = {S31756, 139, { &P15802,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; HTC_Ruby Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA900427 = {S31757, 143, { &P15802,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; HTC Amaze 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2023958 = {S31758, 61, { &P15802,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Ruby Build/IML74K)*/
const Device UA2031848 = {S31759, 65, { &P15802,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Amaze 4G Build/IML74K)*/
const Device UA2194218 = {S31760, 146, { &P15802,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Ruby Build/Speed Rom 7.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2200111 = {S31761, 162, { &P15802,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Ruby Build/Speed Rom 7.8 By EclipzeRemix) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2341150 = {S31762, 152, { &P15802,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_Amaze_4G Build/JellybeanFusion) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2356303 = {S31763, 162, { &P15802,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Ruby Build/Speed Rom 8.0 By EclipzeRemix) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2361761 = {S31764, 157, { &P15802,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Amaze 4G Build/Fourth Bar: Infusion) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2383234 = {S31765, 146, { &P15802,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Ruby Build/Speed Rom 7.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2445712 = {S31766, 136, { &P15802,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Amaze Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2527617 = {S31767, 143, { &P15802,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_Amaze_4g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2653464 = {S31768, 168, { &P15802,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Ruby Build/Speed Rom 9.0 Beta1 By EclipzeRemix) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708142 = {S31769, 143, { &P15802,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Amaze 4g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2759507 = {S31770, 136, { &P15802,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; es-es; Amaze_4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2796426 = {S31771, 148, { &P15802,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Amaze S400 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3007221 = {S31772, 162, { &P15802,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_Amaze_4G Build/Kaioken Rom by SuperAfnan) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027879 = {S31773, 138, { &P18713,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P769 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2387606 = {S31774, 143, { &P18713,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; LG-P760/V10a Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2404105 = {S31775, 138, { &P18713,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; LG-P760 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2416234 = {S31776, 138, { &P18713,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-P768 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423736 = {S31777, 143, { &P18713,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; LG-P760/V10c Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2425890 = {S31778, 60, { &P18713,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P769 Build/IMM76L)*/
const Device UA2427123 = {S31779, 146, { &P18713,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; LG-P760/P76010e Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2457709 = {S31780, 60, { &P18713,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P760 Build/IMM76L)*/
const Device UA2509698 = {S31781, 143, { &P18713,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; LG-P760/V10f Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2527326 = {S31782, 146, { &P18713,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; LG-P760/P76010a Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528035 = {S31783, 60, { &P18713,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P768 Build/IMM76L)*/
const Device UA2529533 = {S31784, 143, { &P18713,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; LG-P760/V10b Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2537178 = {S31785, 146, { &P18713,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; LG-P760/P76010h Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2595980 = {S31786, 135, { &P18713,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sr; LG-P760 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2850122 = {S31787, 135, { &P18713,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it; LG-P769 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2948181 = {S31788, 145, { &P18472,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; DROID RAZR HD Build/9.8.1Q_39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36*/
const Device UA2974146 = {S31789, 145, { &P18472,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-79) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975944 = {S31790, 151, { &P18472,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT925 Build/9.8.2Q-8-XT925_VQL-12.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021949 = {S31791, 150, { &P18472,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XT925 Build/9.8.0Q-97-XT925_VQU-19) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA93141 = {S31792, 61, { &P18254,&P7480,&P18245,&P18189 }};	/*Useragent - FeedFetcher-Google; (+http://www.google.com/feedfetcher.html)*/
const Device UA97816 = {S31793, 86, { &P18254,&P7480,&P18245,&P18189 }};	/*Useragent - Mozilla/5.0 (compatible) Feedfetcher-Google; (+http://www.google.com/feedfetcher.html)*/
const Device UA438315 = {S31794, 85, { &P18254,&P7480,&P18245,&P18189 }};	/*Useragent - Mozilla/5.0 (compatible) Feedfetcher-Google;(+http://www.google.com/feedfetcher.html)*/
const Device UA532550 = {S31795, 87, { &P18254,&P7480,&P18245,&P18189 }};	/*Useragent - Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 2 subscribers; feed-id=X)*/
const Device UA624558 = {S31796, 88, { &P18254,&P7480,&P18245,&P18189 }};	/*Useragent - Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 16 subscribers; feed-id=X)*/
const Device UA629843 = {S31797, 88, { &P18254,&P7480,&P18245,&P18189 }};	/*Useragent - Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 11 subscribers; feed-id=X)*/
const Device UA633735 = {S31798, 88, { &P18254,&P7480,&P18245,&P18189 }};	/*Useragent - Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 61 subscribers; feed-id=X)*/
const Device UA634261 = {S31799, 88, { &P18254,&P7480,&P18245,&P18189 }};	/*Useragent - Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 12 subscribers; feed-id=X)*/
const Device UA642563 = {S31800, 89, { &P18254,&P7480,&P18245,&P18189 }};	/*Useragent - Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 150 subscribers; feed-id=X)*/
const Device UA1992450 = {S31801, 88, { &P18254,&P7480,&P18245,&P18189 }};	/*Useragent - Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 39 subscribers; feed-id=X)*/
const Device UA2039624 = {S31802, 89, { &P18254,&P7480,&P18245,&P18189 }};	/*Useragent - Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 203 subscribers; feed-id=X)*/
const Device UA2168320 = {S31803, 88, { &P18254,&P7480,&P18245,&P18189 }};	/*Useragent - Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 70 subscribers; feed-id=X)*/
const Device UA2172767 = {S31804, 88, { &P18254,&P7480,&P18245,&P18189 }};	/*Useragent - Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 14 subscribers; feed-id=X)*/
const Device UA2203643 = {S31805, 88, { &P18254,&P7480,&P18245,&P18189 }};	/*Useragent - Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 59 subscribers; feed-id=X)*/
const Device UA2242296 = {S31806, 161, { &P18254,&P7480,&P18245,&P18189 }};	/*Useragent - Feedfetcher-Google; (+http://www.google.com/feedfetcher.html; 2 subscribers; feed-id=X) AppEngine-Google; (+http://code.google.com/appengine; appid: voxelshader)*/
const Device UA2774168 = {S31807, 73, { &P18254,&P7480,&P18245,&P18189 }};	/*Useragent - Feedly/1.0 (+http://www.feedly.com/fetcher.html; like FeedFetcher-Google)*/
const Device UA2462418 = {S31808, 140, { &P19777,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9105P Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2462425 = {S31809, 161, { &P19777,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I9105P/I9105PXXALK4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2462427 = {S31810, 139, { &P19777,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9105 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2462432 = {S31811, 140, { &P19777,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9105P Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2462430 = {S31812, 62, { &P19777,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9105P Build/JZO54K)*/
const Device UA2734182 = {S31813, 61, { &P19777,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9105 Build/JZO54K)*/
const Device UA2737565 = {S31814, 161, { &P19777,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-at; SAMSUNG GT-I9105P/I9105PXXAMC2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2766792 = {S31815, 161, { &P19777,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I9105P/I9105PXXAMB6 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768739 = {S31816, 160, { &P19777,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; GT-I9105P-ORANGE/I9105PXXAMBA Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2992543 = {S31817, 161, { &P19777,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; sl-si; SAMSUNG GT-I9105P/I9105PXXAMA2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3010485 = {S31818, 160, { &P19777,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ro-ro; GT-I9105P-ORANGE/I9105PXXAMD1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103183 = {S31819, 143, { &P19777,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2_M.S; he-il; GT-I9105 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA368812 = {S31820, 59, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P970 Build/GRJ22)*/
const Device UA369919 = {S31821, 160, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; is; LG-P970 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA626497 = {S31822, 171, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; LG-P970g-parrot Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA626533 = {S31823, 170, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; LG-P970-Orange Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA640950 = {S31824, 163, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LG-P970 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA655750 = {S31825, 164, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P970h Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA675153 = {S31826, 168, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; LG-P970/V20o Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA744467 = {S31827, 164, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; LG-P970g Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA797571 = {S31828, 175, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; LG-P970-Orange/V20g Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA810535 = {S31829, 123, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LG-P970 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA864541 = {S31830, 168, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LG-P970/V20b Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA1975754 = {S31831, 168, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; LG-P970/V20c Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA1977205 = {S31832, 172, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; LG-P970-Bouygues Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA1987400 = {S31833, 128, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LG-P970 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2023943 = {S31834, 174, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; LG-P970/$(Version) Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2032061 = {S31835, 168, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-pt; LG-P970/V20a Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2033825 = {S31836, 60, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P970g Build/GRJ22)*/
const Device UA2034583 = {S31837, 60, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P970h Build/GRJ22)*/
const Device UA2233809 = {S31838, 175, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; LG-P970-Orange/V20h Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2528429 = {S31839, 168, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; LG-P970/V20e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2578039 = {S31840, 164, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - LG-P970/ Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2692780 = {S31841, 146, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P970 Build/GWK74; CiCi-v3.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2933231 = {S31842, 136, { &P14123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; LG-P970h Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Versi?n/4.0 Mobile Safari/533.1*/
const Device UA100478 = {S31843, 62, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.2.15 Version/10.10*/
const Device UA101174 = {S31844, 62, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; en) Presto/2.2.15 Version/10.10*/
const Device UA161764 = {S31845, 85, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.2.15 Version/10.10*/
const Device UA166983 = {S31846, 85, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.2.15 Version/10.10*/
const Device UA169152 = {S31847, 85, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.2.15 Version/10.10*/
const Device UA287039 = {S31848, 85, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.2.15 Version/10.10*/
const Device UA292720 = {S31849, 99, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.2.15 Version/10.10,gzip(gfe) (via translate.google.com)*/
const Device UA435399 = {S31850, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.2.15 Version/10.10*/
const Device UA436623 = {S31851, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.2.15 Version/10.10*/
const Device UA490898 = {S31852, 85, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03638); ru) Presto/2.2.15 Version/10.10*/
const Device UA528525 = {S31853, 85, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.2.15 Version/10.10*/
const Device UA551305 = {S31854, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.2.15 Version/10.10*/
const Device UA571007 = {S31855, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.2.15 Version/10.10*/
const Device UA576593 = {S31856, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.2.15 Version/10.10*/
const Device UA591018 = {S31857, 85, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.2.15 Version/10.10*/
const Device UA609888 = {S31858, 85, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.2.15 Version/10.10*/
const Device UA655399 = {S31859, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.2.15 Version/10.10*/
const Device UA705164 = {S31860, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.2.15 Version/10.10*/
const Device UA808761 = {S31861, 66, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; ru) Opera 10.10*/
const Device UA829965 = {S31862, 85, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.2.15 Version/10.10*/
const Device UA840529 = {S31863, 89, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; YB/3.5.1; ru; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 10.10*/
const Device UA896211 = {S31864, 85, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.2.15 Version/10.10*/
const Device UA1868261 = {S31865, 85, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.2.15 Version/10.10*/
const Device UA1887872 = {S31866, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.2.15 Version/10.10*/
const Device UA1953412 = {S31867, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.2.15 Version/10.10*/
const Device UA1955237 = {S31868, 85, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5267); ru) Presto/2.2.15 Version/10.10*/
const Device UA1964729 = {S31869, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.2.15 Version/10.10*/
const Device UA1973514 = {S31870, 85, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5328); ru) Presto/2.2.15 Version/10.10*/
const Device UA1984710 = {S31871, 85, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5308); ru) Presto/2.2.15 Version/10.10*/
const Device UA1990595 = {S31872, 85, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5288); ru) Presto/2.2.15 Version/10.10*/
const Device UA2145647 = {S31873, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.2.15 Version/10.10*/
const Device UA2155752 = {S31874, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.2.15 Version/10.10*/
const Device UA2166959 = {S31875, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.2.15 Version/10.10*/
const Device UA2208746 = {S31876, 85, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.2.15 Version/10.10*/
const Device UA2217444 = {S31877, 85, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.2.15 Version/10.10*/
const Device UA2223702 = {S31878, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.2.15 Version/10.10*/
const Device UA2227745 = {S31879, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.2.15 Version/10.10*/
const Device UA2262424 = {S31880, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.2.15 Version/10.10*/
const Device UA2278768 = {S31881, 85, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.2.15 Version/10.10*/
const Device UA2281558 = {S31882, 85, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.2.15 Version/10.10*/
const Device UA2304167 = {S31883, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4832); ru) Presto/2.2.15 Version/10.10*/
const Device UA2306521 = {S31884, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.2.15 Version/10.10*/
const Device UA2307665 = {S31885, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4930); ru) Presto/2.2.15 Version/10.10*/
const Device UA2330878 = {S31886, 85, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.2.15 Version/10.10*/
const Device UA2402726 = {S31887, 72, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.2.15 Version/10.10,gzip(gfe)*/
const Device UA2422779 = {S31888, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.2.15 Version/10.10*/
const Device UA2423620 = {S31889, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.2.15 Version/10.10*/
const Device UA2486958 = {S31890, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.2.15 Version/10.10*/
const Device UA2575775 = {S31891, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5966); ru) Presto/2.2.15 Version/10.10*/
const Device UA2594693 = {S31892, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.2.15 Version/10.10*/
const Device UA2706222 = {S31893, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.2.15 Version/10.10*/
const Device UA2993886 = {S31894, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6089); ru) Presto/2.2.15 Version/10.10*/
const Device UA3074094 = {S31895, 84, { &P15364,&P5690,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6578); ru) Presto/2.2.15 Version/10.10*/
const Device UA2001937 = {S31896, 65, { &P15364,&P17285,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:16.0) Gecko/X Firefox/16.0*/
const Device UA2258896 = {S31897, 70, { &P15364,&P17285,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:16.0) Gecko/X Firefox/16.0*/
const Device UA3048820 = {S31898, 74, { &P15364,&P17285,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; rv:16.0.1) Gecko/X Firefox/16.0.1*/
const Device UA901002 = {S31899, 117, { &P17595,&P17296,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA901043 = {S31900, 117, { &P17595,&P17296,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA901098 = {S31901, 117, { &P17595,&P17296,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA901229 = {S31902, 117, { &P17595,&P17296,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA901762 = {S31903, 154, { &P17595,&P17296,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5,gzip(gfe) (via translate.google.com)*/
const Device UA901817 = {S31904, 117, { &P17595,&P17296,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA901943 = {S31905, 117, { &P17595,&P17296,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA902198 = {S31906, 117, { &P17595,&P17296,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA902321 = {S31907, 117, { &P17595,&P17296,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA902479 = {S31908, 117, { &P17595,&P17296,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA2367487 = {S31909, 143, { &P17595,&P17296,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5*/
const Device UA2597307 = {S31910, 114, { &P17595,&P17296,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1060) AppleWebKit/536.5 (KHTML like Gecko) Chrome/19.0.1084.53 Safari/536.5*/
const Device UA2674460 = {S31911, 129, { &P17595,&P17296,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/19.0.648.205 Safari/534.16*/
const Device UA2674998 = {S31912, 129, { &P17595,&P17296,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/19.0.648.205 Safari/534.16*/
const Device UA2719145 = {S31913, 129, { &P17595,&P17296,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/19.0.648.205 Safari/534.16*/
const Device UA2567109 = {S31914, 145, { &P18448,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyST26i Build/11.2.A.0.21) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583268 = {S31915, 141, { &P18448,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; ST26i Build/11.2.A.0.21) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2667957 = {S31916, 146, { &P18448,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-at; SonyST26iv Build/11.2.A.0.21) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2712993 = {S31917, 145, { &P18448,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyST26a Build/11.2.A.0.21) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2795453 = {S31918, 141, { &P18448,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; ST26a Build/11.2.A.0.21) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2796303 = {S31919, 63, { &P18448,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; ST26i Build/11.2.A.0.31)*/
const Device UA2796304 = {S31920, 145, { &P18448,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyST26i Build/11.2.A.0.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2869295 = {S31921, 147, { &P18448,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; SonyST26i-o Build/11.2.A.0.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2874503 = {S31922, 146, { &P18448,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-at; SonyST26iv Build/11.2.A.0.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2875372 = {S31923, 141, { &P18448,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; ST26i Build/11.2.A.0.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2891145 = {S31924, 65, { &P18448,&P18111,&P18112,&P18092 }};	/*Useragent - SonyST26i Build/11.2.A.0.21 stagefright/1.2 (Linux;Android 4.1.2)*/
const Device UA2900421 = {S31925, 145, { &P18448,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyST26a Build/11.2.A.0.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2953756 = {S31926, 63, { &P18448,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; ST26a Build/11.2.A.0.21)*/
const Device UA2998692 = {S31927, 65, { &P18448,&P18111,&P18112,&P18092 }};	/*Useragent - SonyST26i Build/11.2.A.0.31 stagefright/1.2 (Linux;Android 4.1.2)*/
const Device UA3009608 = {S31928, 141, { &P18448,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ST26a Build/11.2.A.0.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078948 = {S31929, 145, { &P18448,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyST26i Build/11.2.A.0.33) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3082214 = {S31930, 145, { &P18448,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyST26a Build/11.2.A.0.33) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3111951 = {S31931, 141, { &P18448,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; ST26a Build/11.2.A.0.33) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125486 = {S31932, 141, { &P18448,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ST26i Build/11.2.A.0.33) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA368306 = {S31933, 136, { &P17968,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; cm_tenderloin Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA412260 = {S31934, 137, { &P17968,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; cm_tenderloin Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA441485 = {S31935, 137, { &P17968,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; cm_tenderloin Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA604799 = {S31936, 152, { &P17968,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; cm_tenderloin Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA605724 = {S31937, 137, { &P17968,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; cm_tenderloin Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA615683 = {S31938, 152, { &P17968,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; cm_tenderloin Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA616734 = {S31939, 65, { &P17968,&P18149,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; cm_tenderloin Build/GWK74)*/
const Device UA901035 = {S31940, 152, { &P17968,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; cm_tenderloin Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA908903 = {S31941, 66, { &P17968,&P18149,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; cm_tenderloin Build/IMM76L)*/
const Device UA1986254 = {S31942, 168, { &P17968,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/IMM76I; CyanogenMod-Dark Unicorn Resurrected) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2023876 = {S31943, 143, { &P17968,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; cm_tenderloin Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2685377 = {S31944, 159, { &P17968,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; cm_tenderloin Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA710412 = {S31945, 151, { &P18303,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SonyEricssonMT27i Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA899381 = {S31946, 139, { &P18303,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; MT27i Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1966106 = {S31947, 151, { &P18303,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SonyEricssonMT27i Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995115 = {S31948, 151, { &P18303,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; SonyEricssonMT27i Build/6.0.B.1.576) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2007381 = {S31949, 151, { &P18303,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SonyEricssonMT27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024984 = {S31950, 132, { &P18303,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MT27i Build/6.0.B.1.564) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2030311 = {S31951, 139, { &P18303,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MT27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030339 = {S31952, 63, { &P18303,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; MT27i Build/6.0.B.3.184)*/
const Device UA2030824 = {S31953, 139, { &P18303,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; MT27i Build/6.0.B.1.576) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2172778 = {S31954, 151, { &P18303,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-sg; SonyEricssonMT27i Build/6.0.B.3.188) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257193 = {S31955, 132, { &P18303,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MT27i Build/6.0.B.3.184) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2480290 = {S31956, 149, { &P18303,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; iw-; SonyEricssonMT27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2519923 = {S31957, 139, { &P18303,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; in-id; MT27i Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2648208 = {S31958, 139, { &P18303,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-sg; MT27i Build/6.0.B.3.188) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2568312 = {S31959, 134, { &P15767,&P18432,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A523 Safari/8536.25*/
const Device UA2568313 = {S31960, 132, { &P15767,&P18432,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A403 Safari/8536.25*/
const Device UA2568321 = {S31961, 135, { &P15767,&P18432,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A8426 Safari/8536.25*/
const Device UA2568376 = {S31962, 134, { &P15767,&P18432,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A550 Safari/8536.25*/
const Device UA2568866 = {S31963, 132, { &P15767,&P18432,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A406 Safari/8536.25*/
const Device UA2569104 = {S31964, 135, { &P15767,&P18432,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A8500 Safari/8536.25*/
const Device UA2569109 = {S31965, 132, { &P15767,&P18432,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A407 Safari/8536.25*/
const Device UA2561580 = {S31966, 84, { &P18436,&P21460,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (IE 11.0; Windows NT 6.3; WOW64; Trident/7.0; Touch; rv:11.0) like Gecko*/
const Device UA2562923 = {S31967, 89, { &P18436,&P21460,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (IE 11.0; Windows NT 6.3; Win64; x64; Trident/7.0; Touch; rv:11.0) like Gecko*/
const Device UA2599741 = {S31968, 77, { &P18436,&P21460,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (IE 11.0; Windows NT 6.3; Trident/7.0; Touch; rv:11.0) like Gecko*/
const Device UA2670402 = {S31969, 75, { &P18436,&P21460,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.3; WOW64; Trident/7.0; Touch; rv:11.0) like Gecko*/
const Device UA2670419 = {S31970, 80, { &P18436,&P21460,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.3; Win64; x64; Trident/7.0; Touch; rv:11.0) like Gecko*/
const Device UA2566438 = {S31971, 129, { &P18385,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566439 = {S31972, 128, { &P18385,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Nexus 7 Build/JOP40C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566706 = {S31973, 130, { &P18385,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 7 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2034653 = {S31974, 65, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Desire X Build/IMM76D)*/
const Device UA2226870 = {S31975, 143, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Desire_X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257438 = {S31976, 154, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; HTC_Desire_X/1.14.169.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257565 = {S31977, 163, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; HTC_Desire_X-orange-LS/1.14.75.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257604 = {S31978, 155, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Desire_X/1.14.161.11 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2352651 = {S31979, 143, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Desire X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2362181 = {S31980, 154, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HTC_Desire_X/1.14.111.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2405105 = {S31981, 154, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; HTC_Desire_X/1.14.114.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2421852 = {S31982, 154, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; HTC_Desire_X/1.14.113.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2427692 = {S31983, 154, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; HTC_Desire_X/1.14.118.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2428143 = {S31984, 154, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Desire_X/1.18.161.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2430826 = {S31985, 154, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; HTC_Desire_X/1.18.169.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2531900 = {S31986, 154, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; HTC_Desire_X/1.18.114.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2532813 = {S31987, 154, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Desire_X/1.26.161.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2557517 = {S31988, 154, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; HTC_Desire_X/1.18.118.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2560593 = {S31989, 154, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; HTC_Desire_X/1.18.114.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2560601 = {S31990, 154, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HTC_Desire_X/1.18.111.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2563475 = {S31991, 154, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; HTC_Desire_X/1.26.169.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2570341 = {S31992, 154, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; HTC_Desire_X/1.18.118.4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583136 = {S31993, 154, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; HTC_Desire_X/1.18.113.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2663350 = {S31994, 154, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; HTC_Desire_X/1.25.123.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2698240 = {S31995, 163, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ch; HTC_Desire_X-orange-LS/1.14.65.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2733926 = {S31996, 141, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_Desire_X Build/SVHD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2864531 = {S31997, 154, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; HTC_Desire_X/1.25.112.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3073138 = {S31998, 144, { &P18462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Desire_XS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA687945 = {S31999, 128, { &P12825,&P7106,&P15829,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9380; en) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.336 Mobile Safari/534.11+*/
const Device UA9720 = {S32000, 147, { &P12444,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; U20i Build/2.0.2.A.0.24) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA9786 = {S32001, 146, { &P12444,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; U20i Build/2.0.A.0.504) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA19441 = {S32002, 159, { &P12444,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; SonyEricssonU20iv Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA19491 = {S32003, 146, { &P12444,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; U20i Build/2.1.1.B.0.5) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA21994 = {S32004, 146, { &P12444,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; U20i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA58138 = {S32005, 147, { &P12444,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; pl-pl; U20i Build/2.1.1.A.0.16) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA58159 = {S32006, 146, { &P12444,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; U20i Build/2.1.1.C.0.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA58190 = {S32007, 147, { &P12444,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; U20i Build/2.1.1.A.0.19) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA63628 = {S32008, 158, { &P12444,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; SonyEricssonU20i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA70212 = {S32009, 147, { &P12444,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-kr; U20i Build/2.1.1.A.0.28) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA189798 = {S32010, 68, { &P12444,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; U20i Build/2.1.1.A.0.6)*/
const Device UA719964 = {S32011, 132, { &P12444,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; U20i Build/2.1.1.A.0.6) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022566 = {S32012, 133, { &P12444,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; U20i Build/2.0.2.A.0.24) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA92445 = {S32013, 116, { &P15364,&P5690,&P17237,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/530.5 (KHTML, like Gecko) Chrome/2.0.172.43 Safari/530.5*/
const Device UA430162 = {S32014, 116, { &P15364,&P5690,&P17237,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/530.5 (KHTML, like Gecko) Chrome/2.0.172.43 Safari/530.5*/
const Device UA539928 = {S32015, 109, { &P15364,&P5690,&P17237,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1;) AppleWebKit/530.1 (KHTML, like Gecko) Chrome/2.0.169.1 Safari/530.1*/
const Device UA667467 = {S32016, 112, { &P15364,&P5690,&P17237,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; eu) AppleWebKit/530.4 (KHTML, like Gecko) Chrome/2.0.172.0 Safari/530.4*/
const Device UA1963867 = {S32017, 189, { &P15364,&P5690,&P17237,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/530.5 (KHTML, like Gecko) Chrome/2.0.172.33 Safari/530.5 AppEngine-Google; (+http://code.google.com/appengine; appid: metafabric)*/
const Device UA2176721 = {S32018, 96, { &P15364,&P5690,&P17237,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.10 (KHTML, like Gecko) Chrome/2.0.1.3 Safari/536.10*/
const Device UA2705801 = {S32019, 119, { &P15364,&P5690,&P17237,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/2.4.2.86 Safari/537.22 BIDUPlayerBrowser/2.x*/
const Device UA30012 = {S32020, 104, { &P17346,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA90488 = {S32021, 104, { &P17346,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en_us) AppleWebKit/525.18.1 (KHTML, like Gecko)*/
const Device UA157622 = {S32022, 117, { &P17346,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148*/
const Device UA158830 = {S32023, 133, { &P17346,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/533.17.9*/
const Device UA421307 = {S32024, 165, { &P17346,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper*/
const Device UA628325 = {S32025, 250, { &P17346,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_2_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod2,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.2.1;FBSS/1; FBCR/;FBID/phone;FBLC/en_US;FBSF/1.0]*/
const Device UA747502 = {S32026, 145, { &P17346,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 iPod2,1 BingWeb/3.03.1428.X*/
const Device UA2419409 = {S32027, 142, { &P17346,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone 4_2_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPod2,1/4.2.1)*/
const Device UA2712573 = {S32028, 141, { &P17346,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 421 like Mac OS X; es-US) AppleWebKit/533.17.9 (KHTML like Gecko) Mobile/8C148 iPod21 BingWeb/3.02.1641.X*/
const Device UA2808889 = {S32029, 52, { &P17346,&P17145,&P17596,&P18092 }};	/*Useragent - Instagram 2.5.0 (iPod touch; iPhone OS 4.2.1; en_US)*/
const Device UA93088 = {S32030, 116, { &P15364,&P7133,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.27 Safari/532.0*/
const Device UA765406 = {S32031, 113, { &P15364,&P7133,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.8 (KHTML, like Gecko) Chrome/3.1.0.0 Safari/532.8*/
const Device UA766260 = {S32032, 143, { &P15364,&P7133,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Steam GameOverlay; ) AppleWebKit/532.1 (KHTML, like Gecko) Chrome/3.0.195.24 Safari/532.1*/
const Device UA1884554 = {S32033, 139, { &P15364,&P7133,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; Valve Source Client; ) AppleWebKit/532.1 (KHTML, like Gecko) Chrome/3.0.195.24 Safari/532.1*/
const Device UA2333332 = {S32034, 115, { &P15364,&P7133,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/531.0 (KHTML	 like Gecko) Chrome/3.0.190.1 Safari/531.0*/
const Device UA683825 = {S32035, 139, { &P18687,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; LG-P705g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA703184 = {S32036, 138, { &P18687,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; LG-P705 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899720 = {S32037, 139, { &P18687,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-mx; LG-P708g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1969988 = {S32038, 139, { &P18687,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; LG-P705f Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034343 = {S32039, 60, { &P18687,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-P705 Build/IML74K)*/
const Device UA2222941 = {S32040, 129, { &P18687,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; LG-P705 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2375184 = {S32041, 61, { &P18687,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-P708g Build/IML74K)*/
const Device UA2411490 = {S32042, 61, { &P18687,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-P705g Build/IML74K)*/
const Device UA2456017 = {S32043, 143, { &P18687,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-nz; LG-P705/V10g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2499918 = {S32044, 143, { &P18687,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-nz; LG-P705/V10c Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2551109 = {S32045, 143, { &P18687,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-nz; LG-P705/V10e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2640987 = {S32046, 61, { &P18687,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-P705f Build/IML74K)*/
const Device UA2817071 = {S32047, 124, { &P18687,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; LG-P705 Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2352371 = {S32048, 64, { &P18449,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST27i Build/6.1.1.B.1.10)*/
const Device UA2367593 = {S32049, 142, { &P18449,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST27i Build/6.1.1.B.1.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2427262 = {S32050, 154, { &P18449,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SonyEricssonST27a Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2428603 = {S32051, 154, { &P18449,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonST27i Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2468258 = {S32052, 142, { &P18449,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST27i Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2468287 = {S32053, 142, { &P18449,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; ST27a Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528054 = {S32054, 155, { &P18449,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-za; SonyEricssonST27iv Build/6.1.1.B.1.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695095 = {S32055, 142, { &P18449,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST27a Build/6.1.1.B.1.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2790403 = {S32056, 64, { &P18449,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST27a Build/6.1.1.B.1.10)*/
const Device UA578817 = {S32057, 138, { &P12788,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PG86100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA647996 = {S32058, 147, { &P12788,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sprint APX515CKT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA911285 = {S32059, 152, { &P12788,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; HTC/EVO_3D/3.28.161.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1961492 = {S32060, 141, { &P12788,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Evo 3D Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1961756 = {S32061, 60, { &P12788,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; PG86100 Build/IML74K)*/
const Device UA1975833 = {S32062, 152, { &P12788,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC/EVO_3D/3.28.166.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1990359 = {S32063, 152, { &P12788,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; HTC/EVO_3D/3.28.165.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2022907 = {S32064, 137, { &P12788,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PG86100 Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028222 = {S32065, 152, { &P12788,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; HTC/EVO_3D/3.28.163.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2164644 = {S32066, 138, { &P12788,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ISW12HT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2224201 = {S32067, 138, { &P12788,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PG86100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2351837 = {S32068, 144, { &P12788,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PG86100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30*/
const Device UA2444992 = {S32069, 138, { &P12788,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PG86100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2480919 = {S32070, 147, { &P12788,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Sprint APX515CKT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2502385 = {S32071, 60, { &P12788,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; ISW12HT Build/IML74K)*/
const Device UA2525618 = {S32072, 147, { &P12788,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Sprint APX515CKT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2546330 = {S32073, 136, { &P12788,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; PG86100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2822351 = {S32074, 152, { &P12788,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; HTC/EVO_3D/1.21.161.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2981927 = {S32075, 145, { &P12788,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; -us; Sprint APX515CKT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3070446 = {S32076, 145, { &P12788,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sprint APX515CKT Build/MyUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA910759 = {S32077, 129, { &P18385,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; Nexus 7 Build/JRN84D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA913421 = {S32078, 131, { &P18385,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus 7 Build/JRO03D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2014819 = {S32079, 137, { &P18385,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus 7 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2197587 = {S32080, 131, { &P18385,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2224898 = {S32081, 138, { &P18385,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Google Nexus 7 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2240862 = {S32082, 135, { &P18385,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; KRAKD Nexus 7 Build/JRN84D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2240917 = {S32083, 146, { &P18385,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Ainovo Flame (Nexus 7) Build/JRO03D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2241915 = {S32084, 118, { &P18385,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; Nexus 7) AppleWebKit/537.7 (KHTML, like Gecko) Chrome/18.0.1025.308 Safari/537.1*/
const Device UA2242109 = {S32085, 135, { &P18385,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus 7 16G Build/JRO03D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2242113 = {S32086, 137, { &P18385,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2768363 = {S32087, 135, { &P18385,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; en-US; Nexus 7 Build/JRN84D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Safari/535.19*/
const Device UA638727 = {S32088, 138, { &P18240,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; LG-P700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1974726 = {S32089, 143, { &P18240,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; LG-P700/V10c Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1992629 = {S32090, 60, { &P18240,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-P700 Build/IML74K)*/
const Device UA1992666 = {S32091, 145, { &P18240,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; LG-P700-Orange Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996739 = {S32092, 143, { &P18240,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-P700/V10g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029197 = {S32093, 143, { &P18240,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; LG-P700/V10f Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2222890 = {S32094, 143, { &P18240,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; LG-P700/V10a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2315955 = {S32095, 135, { &P18240,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; bs; LG-P700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2353270 = {S32096, 143, { &P18240,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; LG-P700/V10h Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2391311 = {S32097, 143, { &P18240,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; LG-P700/V10d Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2487373 = {S32098, 143, { &P18240,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; LG-P700/V10i Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2490359 = {S32099, 143, { &P18240,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-P700/V10k Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2696206 = {S32100, 143, { &P18240,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; LG-P700/V10e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616601 = {S32101, 148, { &P13018,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Sony Xperia Neo V Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616681 = {S32102, 141, { &P13018,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MT11i Build/4.1.A.0.562) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616748 = {S32103, 63, { &P13018,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; MT11i Build/4.1.A.0.562)*/
const Device UA900309 = {S32104, 141, { &P13018,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MT11i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034209 = {S32105, 141, { &P13018,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MT11a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2191570 = {S32106, 153, { &P13018,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyEricssonMT11a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257481 = {S32107, 141, { &P13018,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; MT11i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257486 = {S32108, 153, { &P13018,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; SonyEricssonMT11i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257566 = {S32109, 154, { &P13018,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SonyEricssonMT11iv Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2344565 = {S32110, 63, { &P13018,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MT11i Build/4.1.B.0.587)*/
const Device UA2454933 = {S32111, 156, { &P13018,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MT11i Build/ThilinaC@xda_pureness_13.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2552594 = {S32112, 141, { &P13018,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MT11a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555167 = {S32113, 156, { &P13018,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MT11i Build/ThilinaC@xda_pureness_14.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2693182 = {S32114, 63, { &P13018,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MT11a Build/4.1.B.0.587)*/
const Device UA20046 = {S32115, 163, { &P15028,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-sa; LG-P350 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA20048 = {S32116, 170, { &P15028,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-P350-Orange Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA20073 = {S32117, 170, { &P15028,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P350-Orange Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA20074 = {S32118, 162, { &P15028,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P350/V10a/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA21644 = {S32119, 163, { &P15028,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P350 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA42656 = {S32120, 168, { &P15028,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P350/V10d Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA49490 = {S32121, 165, { &P15028,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-mx; LG-P350g Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA58559 = {S32122, 157, { &P15028,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; LG-P350/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA58550 = {S32123, 162, { &P15028,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ro-ro; LG-P350/V10b/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA58579 = {S32124, 164, { &P15028,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; LG-P350 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA61027 = {S32125, 165, { &P15028,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-mx; LG-P350f Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA72728 = {S32126, 168, { &P15028,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; hu-hu; LG-P350/V10f Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA76800 = {S32127, 160, { &P15028,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; iw; LG-P350 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA314768 = {S32128, 59, { &P15028,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P350 Build/FRG83)*/
const Device UA664434 = {S32129, 168, { &P15028,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; hr-hr; LG-P350/V10g Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA702472 = {S32130, 123, { &P15028,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; LG-P350 Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA777870 = {S32131, 61, { &P15028,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P350f Build/FRG83G)*/
const Device UA786430 = {S32132, 61, { &P15028,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P350g Build/FRG83G)*/
const Device UA1952549 = {S32133, 168, { &P15028,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; hr-hr; LG-P350/V10i Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2019013 = {S32134, 59, { &P15028,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; LG-P350 Build/FRG83)*/
const Device UA2023531 = {S32135, 123, { &P15028,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; LG-P350 Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA3043984 = {S32136, 168, { &P15028,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; nl-nl; LG-P350/V10e Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA5789 = {S32137, 156, { &P11929,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0Nokia5230-c/20.2.098; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) BrowserNG/7.2.3.1*/
const Device UA8987 = {S32138, 169, { &P11929,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5230-1b/20.2.005; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.3*/
const Device UA14864 = {S32139, 169, { &P11929,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5230-1d/20.4.005; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.3*/
const Device UA37718 = {S32140, 190, { &P11929,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0Nokia5230/50.0.001; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.25 Mobile Safari/533.4 3gpp-gba*/
const Device UA43512 = {S32141, 194, { &P11929,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5230-1d/50.4.001; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.25 Mobile Safari/533.4 3gpp-gba*/
const Device UA63284 = {S32142, 157, { &P11929,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5230-c/20.2.098; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) BrowserNG/7.2.3.1*/
const Device UA64620 = {S32143, 166, { &P11929,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5230/20.0.005; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.3*/
const Device UA64758 = {S32144, 169, { &P11929,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5230-1c/20.8.007; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.3*/
const Device UA68166 = {S32145, 34, { &P11929,&P5129,&P3690,&P18092 }};	/*Useragent - Nokia5230/UCBrowser7.7.0.81/50/999*/
const Device UA69521 = {S32146, 9, { &P11929,&P5129,&P3690,&P18092 }};	/*Useragent - Nokia5230*/
const Device UA346872 = {S32147, 191, { &P11929,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5230/51.0.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 Mobile Safari/533.4 3gpp-gba*/
const Device UA651011 = {S32148, 194, { &P11929,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5230-1b/51.2.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 Mobile Safari/533.4 3gpp-gba*/
const Device UA92740 = {S32149, 62, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.2.15 Version/10.00*/
const Device UA92822 = {S32150, 62, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; en) Presto/2.2.15 Version/10.00*/
const Device UA94700 = {S32151, 62, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; en) Presto/2.2.15 Version/10.01*/
const Device UA98891 = {S32152, 48, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/10.00 (Windows NT 5.1; U; en) Presto/2.2.0*/
const Device UA172121 = {S32153, 85, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.2.15 Version/10.00*/
const Device UA183854 = {S32154, 85, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02603); ru) Presto/2.2.15 Version/10.00*/
const Device UA421862 = {S32155, 84, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.2.15 Version/10.00*/
const Device UA432549 = {S32156, 85, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.2.15 Version/10.00*/
const Device UA448410 = {S32157, 85, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.2.15 Version/10.01*/
const Device UA459845 = {S32158, 62, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; en) Presto/2.2.15 Version/10.01*/
const Device UA477939 = {S32159, 79, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; ru; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 10.00*/
const Device UA480594 = {S32160, 85, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.2.15 Version/10.00*/
const Device UA552753 = {S32161, 84, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.2.15 Version/10.00*/
const Device UA565790 = {S32162, 84, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.2.15 Version/10.00*/
const Device UA591520 = {S32163, 85, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.2.15 Version/10.00*/
const Device UA669852 = {S32164, 84, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.2.15 Version/10.00*/
const Device UA684381 = {S32165, 85, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.2.15 Version/10.00*/
const Device UA753268 = {S32166, 84, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.2.15 Version/10.00*/
const Device UA813457 = {S32167, 99, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; en) Presto/2.2.15 Version/10.00,gzip(gfe) (via translate.google.com)*/
const Device UA851995 = {S32168, 79, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; ru; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 10.01*/
const Device UA855006 = {S32169, 66, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; ru) Opera 10.00*/
const Device UA1840954 = {S32170, 66, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; ru) Opera 10.01*/
const Device UA1868165 = {S32171, 85, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.2.15 Version/10.01*/
const Device UA1870011 = {S32172, 85, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.2.15 Version/10.00*/
const Device UA1974171 = {S32173, 85, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.2.15 Version/10.00*/
const Device UA1978790 = {S32174, 84, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.2.15 Version/10.00*/
const Device UA2131372 = {S32175, 85, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.2.15 Version/10.00*/
const Device UA2137749 = {S32176, 84, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.2.15 Version/10.00*/
const Device UA2219213 = {S32177, 85, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.2.15 Version/10.00*/
const Device UA2223650 = {S32178, 84, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.2.15 Version/10.00*/
const Device UA2262495 = {S32179, 85, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.2.15 Version/10.00*/
const Device UA2341564 = {S32180, 84, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.2.15 Version/10.00*/
const Device UA2369480 = {S32181, 66, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/10.02 (Windows NT 5.1; U; en) Presto/22.10.289 Version/10.02*/
const Device UA2369548 = {S32182, 71, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/10.0 (Windows NT 5.2; WOW64; U; en)Presto/22.10.168 Version/10.00*/
const Device UA2372922 = {S32183, 84, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.2.15 Version/10.00*/
const Device UA2400049 = {S32184, 84, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.2.15 Version/10.00*/
const Device UA2480800 = {S32185, 72, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.2.15 Version/10.00,gzip(gfe)*/
const Device UA2506652 = {S32186, 84, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.2.15 Version/10.00*/
const Device UA2527698 = {S32187, 84, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.2.15 Version/10.00*/
const Device UA2547714 = {S32188, 84, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.2.15 Version/10.00*/
const Device UA2653347 = {S32189, 84, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.2.15 Version/10.00*/
const Device UA2703026 = {S32190, 84, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.2.15 Version/10.00*/
const Device UA2762449 = {S32191, 84, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.2.15 Version/10.00*/
const Device UA2802152 = {S32192, 30, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/10.0 (Windows NT 5.1; U)*/
const Device UA2815490 = {S32193, 84, { &P15364,&P5690,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.2.15 Version/10.00*/
const Device UA924483 = {S32194, 68, { &P15364,&P16110,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:17.0) Gecko/17.0 Firefox/17.0*/
const Device UA924890 = {S32195, 73, { &P15364,&P16110,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:17.0) Gecko/17.0 Firefox/17.0*/
const Device UA900441 = {S32196, 139, { &P18509,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-MS770 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027619 = {S32197, 61, { &P18509,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-MS770 Build/IMM76I)*/
const Device UA2554079 = {S32198, 219, { &P18509,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-MS770 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Android MetroPCS androidAppVersion/2.5.10 AndroidWebkitWrapper BillingSupported*/
const Device UA2655139 = {S32199, 137, { &P18509,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; LG-MS770 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3015874 = {S32200, 184, { &P18509,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-MS770 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 androidAppVersion/2.6.8 AndroidWebkitWrapper*/
const Device UA3093854 = {S32201, 160, { &P18509,&P18066,&P18062,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.0.4;dev=l0;devm=LG-MS770;man=lge;brw=Android;shwd=320x480;shwa=320x455;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA90788 = {S32202, 95, { &P15795,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; 710)*/
const Device UA90795 = {S32203, 101, { &P15795,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 710)*/
const Device UA371366 = {S32204, 111, { &P15795,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 710; T-Mobile)*/
const Device UA399158 = {S32205, 111, { &P15795,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 710; Vodafone)*/
const Device UA399162 = {S32206, 109, { &P15795,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 710; Orange)*/
const Device UA399175 = {S32207, 100, { &P15795,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0;Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 710)*/
const Device UA678546 = {S32208, 109, { &P15795,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 710; Meteor)*/
const Device UA761318 = {S32209, 101, { &P15795,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Nokia 710)*/
const Device UA925855 = {S32210, 110, { &P15795,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 710; Optimus)*/
const Device UA2545942 = {S32211, 109, { &P15795,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Nokia_blocked; Lumia 710)*/
const Device UA2979432 = {S32212, 111, { &P15795,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 710),gzip(gfe)*/
const Device UA2982202 = {S32213, 140, { &P15795,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 710),gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2568316 = {S32214, 143, { &P12280,&P18432,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A523 Safari/8536.25*/
const Device UA2568358 = {S32215, 141, { &P12280,&P18432,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A403 Safari/8536.25*/
const Device UA2568361 = {S32216, 143, { &P12280,&P18432,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A525 Safari/8536.25*/
const Device UA2568371 = {S32217, 143, { &P12280,&P18432,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A551 Safari/8536.25*/
const Device UA2568372 = {S32218, 141, { &P12280,&P18432,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A405 Safari/8536.25*/
const Device UA2436529 = {S32219, 113, { &P13209,&P17658,&P18392,&P18168 }};	/*Useragent - DoCoMo/2.0 P900i(c100;TB;W24H11) (compatible; ichiro/mobile goo;+http://search.goo.ne.jp/option/use/sub4/sub4-1/)*/
const Device UA2042947 = {S32220, 76, { &P17595,&P17296,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:15.0) Gecko/X Firefox/15.0.1*/
const Device UA2194223 = {S32221, 91, { &P17595,&P17296,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:15.0) Gecko/X Firefox/15.0.1 Firefox/15.0.1*/
const Device UA2470285 = {S32222, 69, { &P15364,&P18118,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:21.0) Gecko/X Firefox/21.0*/
const Device UA120214 = {S32223, 94, { &P15364,&P18110,&P20804,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Qt/4.6.3 Safari/532.4*/
const Device UA261776 = {S32224, 94, { &P15364,&P18110,&P20804,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux; en-US) AppleWebKit/527+ (KHTML, like Gecko, Safari/419.3) Qt/4.5.2*/
const Device UA273604 = {S32225, 96, { &P15364,&P18110,&P20804,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux; en-US) AppleWebKit/523.15 (KHTML, like Gecko, Safari/419.3) Qt/4.4.3*/
const Device UA444934 = {S32226, 103, { &P15364,&P18110,&P20804,&P18092 }};	/*Useragent - Mozilla/5.0 (QtEmbedded; U; Linux; en-US) AppleWebKit/523.15 (KHTML, like Gecko, Safari/419.3) Qt/4.4.0*/
const Device UA576666 = {S32227, 101, { &P15364,&P18110,&P20804,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Qt/4.6.2 Safari/534.3*/
const Device UA756643 = {S32228, 99, { &P15364,&P18110,&P20804,&P18092 }};	/*Useragent - Mozilla/5.0 (QtEmbedded; U; Linux; C -) AppleWebKit/532.4 (KHTML, like Gecko) Qt/4.6.2 Safari/532.4*/
const Device UA3125583 = {S32229, 97, { &P15364,&P18110,&P20804,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; C) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.6.2 Safari/533.3*/
const Device UA93620 = {S32230, 95, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.18 (KHTML, like Gecko)*/
const Device UA97585 = {S32231, 97, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/525.28.3 (KHTML, like Gecko)*/
const Device UA99226 = {S32232, 95, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-us) AppleWebKit/525.18 (KHTML, like Gecko)*/
const Device UA121656 = {S32233, 95, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-us) AppleWebKit/525.18 (KHTML, like Gecko)*/
const Device UA140006 = {S32234, 97, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/525.27.1 (KHTML, like Gecko)*/
const Device UA175777 = {S32235, 56, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US)*/
const Device UA178778 = {S32236, 97, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko)*/
const Device UA228806 = {S32237, 87, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko)*/
const Device UA233428 = {S32238, 106, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) NetNewsWire/3.2.15*/
const Device UA289354 = {S32239, 127, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) WebClip/5527.2 Safari/5530.19*/
const Device UA374524 = {S32240, 127, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.18 (KHTML, like Gecko) WebClip/5525.7 Safari/5525.20.1*/
const Device UA388386 = {S32241, 126, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) WebClip/5527.2 Safari/5527.2*/
const Device UA416465 = {S32242, 129, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Sunrise/1.7.1 like Safari/5525.18*/
const Device UA449270 = {S32243, 112, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) (null) FBSMTWB*/
const Device UA472115 = {S32244, 123, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko, Safari/5525.20.1) ADM/559*/
const Device UA565211 = {S32245, 106, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) NetNewsWire/3.2.11*/
const Device UA664105 = {S32246, 43, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/4.0 (Mac OS X 10.5.8) Java/1.5.0_30*/
const Device UA697515 = {S32247, 105, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) NetNewsWire/3.1.7*/
const Device UA719560 = {S32248, 63, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; fr; rv:1.9.0.3)*/
const Device UA792398 = {S32249, 95, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; en-us) AppleWebKit/525.18 (KHTML, like Gecko)*/
const Device UA849139 = {S32250, 102, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) (null) FBSMTWB*/
const Device UA897050 = {S32251, 90, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.0.6) Gecko/X Safari/525.27.1*/
const Device UA1849399 = {S32252, 119, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/531.21.8+(KHTML, like Gecko, Safari/528.16) (null)*/
const Device UA1893165 = {S32253, 109, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) RealPlayer/10.1.0.412*/
const Device UA1895366 = {S32254, 100, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Safari/531.9*/
const Device UA1896378 = {S32255, 100, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Safari/417.8*/
const Device UA1896840 = {S32256, 100, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Safari/522.0*/
const Device UA1902995 = {S32257, 105, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Shiira Safari/125*/
const Device UA1903427 = {S32258, 95, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Sunrise*/
const Device UA1904327 = {S32259, 109, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) RealPlayer/11.0.1.949*/
const Device UA2125993 = {S32260, 131, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/525.28.3 (KHTML, like Gecko) WebClip/5525.7.1 Safari/5525.28.3*/
const Device UA2140450 = {S32261, 131, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/525.27.1 (KHTML, like Gecko) WebClip/5525.7.1 Safari/5525.27.1*/
const Device UA2145229 = {S32262, 105, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) NetNewsWire/3.2.7*/
const Device UA2166581 = {S32263, 62, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) TheDancingQueen*/
const Device UA2208861 = {S32264, 131, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; ja-jp) AppleWebKit/525.18 (KHTML, like Gecko) Sunrise/1.7.5 like Safari/5525.20.1*/
const Device UA2258287 = {S32265, 102, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.18 (KHTML, like Gecko) TubeTV*/
const Device UA2258644 = {S32266, 113, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_5; en-us) AppleWebKit/525.27.1 (KHTML, like Gecko) Shiira Safari/125*/
const Device UA2293352 = {S32267, 105, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Vienna/2.5.0.2501*/
const Device UA2293937 = {S32268, 97, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) KidZui K2*/
const Device UA2296147 = {S32269, 103, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) NetNewsWire/3.2*/
const Device UA2299387 = {S32270, 115, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Sunrise/2.1.5 Safari/533.22.3*/
const Device UA2310777 = {S32271, 109, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.2.10) Gecko Expeditor 6.2.2.X-1115 Gecko/X-1115*/
const Device UA2316948 = {S32272, 110, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) RealPlayer/12.0.0.1725*/
const Device UA2339792 = {S32273, 128, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.18 (KHTML, like Gecko) LittleSnapper/9020 Safari/525.13*/
const Device UA2370376 = {S32274, 110, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) RealPlayer/11.1.0.1116*/
const Device UA2440011 = {S32275, 109, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) RealPlayer/10.1.0.503*/
const Device UA2554234 = {S32276, 112, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko, Safari/534.50.2) ADM/737*/
const Device UA2639806 = {S32277, 94, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) TubeTV*/
const Device UA2694571 = {S32278, 110, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) RealPlayer/12.0.1.1750*/
const Device UA2720380 = {S32279, 110, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) RealPlayer/12.0.0.1569*/
const Device UA2757532 = {S32280, 110, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Safari/531.9*/
const Device UA2806593 = {S32281, 110, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Safari/417.8*/
const Device UA2978831 = {S32282, 33, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Galaxy/1.0 [en] (Mac OS X 10.5.6)*/
const Device UA2979309 = {S32283, 40, { &P17595,&P17295,&P18108,&P18092 }};	/*Useragent - Galaxy/1.0 [en] (Mac OS X 10.5.6; U; en)*/
const Device UA352711 = {S32284, 142, { &P13122,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-B5510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625900 = {S32285, 142, { &P13122,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-B5510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627629 = {S32286, 161, { &P13122,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; SAMSUNG GT-B5510/B5510ATLA1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633401 = {S32287, 143, { &P13122,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; GT-B5510L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA660264 = {S32288, 160, { &P13122,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-B5510-ORANGE/B5510BVKJ2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA664378 = {S32289, 143, { &P13122,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-B5510L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA671464 = {S32290, 143, { &P13122,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; GT-B5510B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA684085 = {S32291, 161, { &P13122,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; SAMSUNG GT-B5510/B5510ATKK2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA700442 = {S32292, 160, { &P13122,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-B5510-ORANGE/B5510BVLB1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA766235 = {S32293, 66, { &P13122,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-B5510 Build/GINGERBREAD)*/
const Device UA1964975 = {S32294, 160, { &P13122,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-B5510-ORANGE/B5510BVLD1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1967930 = {S32295, 161, { &P13122,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-B5510/B5510XXLE1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1981928 = {S32296, 143, { &P13122,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-B5510B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022818 = {S32297, 66, { &P13122,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-B5510 Build/GINGERBREAD)*/
const Device UA2034582 = {S32298, 67, { &P13122,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-B5510L Build/GINGERBREAD)*/
const Device UA2197110 = {S32299, 161, { &P13122,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-B5510/B5510XXLH4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2229581 = {S32300, 160, { &P13122,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-B5510-ORANGE/B5510BVLF1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257114 = {S32301, 160, { &P13122,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-B5510-ORANGE/B5510BVLH1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2488406 = {S32302, 161, { &P13122,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; SAMSUNG GT-B5510/B5510XXLL1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2544531 = {S32303, 67, { &P13122,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-B5510L Build/GINGERBREAD)*/
const Device UA2725994 = {S32304, 140, { &P13122,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-B5510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2865444 = {S32305, 160, { &P13122,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-B5510-ORANGE/B5510BVLK1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA412969 = {S32306, 100, { &P15364,&P16110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11*/
const Device UA2191264 = {S32307, 107, { &P15364,&P16110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.70 Safari/535.11*/
const Device UA2424037 = {S32308, 160, { &P15364,&P16110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/17.0.944.0 Safari/535.8 BitdefenderSafepay/0.2.0.0 (OBK; 17.0.944.0; r396.2011)*/
const Device UA2554586 = {S32309, 153, { &P15364,&P16110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/17.0.944.0 Safari/535.8 BitdefenderSafepay/0.2.0.0 (OBK; 17.0.944.0; r396.2011)*/
const Device UA2654403 = {S32310, 135, { &P15364,&P16110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.84 Safari/535.11 SE 2.X MetaSr 1.0,gzip(gfe)*/
const Device UA2766454 = {S32311, 120, { &P15364,&P16110,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.1.963.51 Safari/535.11 MRCHROME SOC*/
const Device UA2538236 = {S32312, 58, { &P15364,&P5690,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:20.0) Gecko/X Firefox/20.0*/
const Device UA2539804 = {S32313, 65, { &P15364,&P5690,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64; rv:20.0) Gecko/X Firefox/20.0*/
const Device UA2703950 = {S32314, 70, { &P15364,&P5690,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 5.1; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA2755180 = {S32315, 72, { &P15364,&P5690,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.1*/
const Device UA2772569 = {S32316, 79, { &P15364,&P5690,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.1*/
const Device UA33430 = {S32317, 50, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 2.2.2)*/
const Device UA40052 = {S32318, 124, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; pt-pt Build/FINAL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA145708 = {S32319, 48, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 2.2)*/
const Device UA146667 = {S32320, 37, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - stagefright/1.0 (Linux;Android 2.2.1)*/
const Device UA163054 = {S32321, 50, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 2.2.1)*/
const Device UA163928 = {S32322, 136, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; imx51_bbg Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA166914 = {S32323, 105, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA173314 = {S32324, 136, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Android Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA175115 = {S32325, 133, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; generic Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA175969 = {S32326, 138, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA177589 = {S32327, 105, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA184927 = {S32328, 141, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Full Android Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA188928 = {S32329, 136, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; ST070I00 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA242060 = {S32330, 134, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; mercury Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA258269 = {S32331, 145, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; nl-nl; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA258597 = {S32332, 147, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA299025 = {S32333, 130, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ld-us; MID Build/JIEVIS) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA309400 = {S32334, 143, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; TSB_CLOUD_COMPANION;TOSHIBA_AC_AND_AZ) AppleWebkit/533.1(KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA309793 = {S32335, 137, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; MID_Serails Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA315249 = {S32336, 37, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - stagefright/1.0 (Linux;Android 2.2.2)*/
const Device UA321305 = {S32337, 141, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; generic Build/Uberoid.Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA383357 = {S32338, 35, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - stagefright/1.0 (Linux;Android 2.2)*/
const Device UA394663 = {S32339, 60, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; Android Build/MASTER)*/
const Device UA396156 = {S32340, 136, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; FIH-F0X Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA451497 = {S32341, 168, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; msm7227_ffa Build/I700T_P7_T07_NEXX_FT_R_0_01_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA525906 = {S32342, 138, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; md-us; sec_smdkv210 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA541859 = {S32343, 57, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; generic Build/Froyo)*/
const Device UA558925 = {S32344, 167, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; id-id; msm7227_ffa Build/I700T_P4_10_AC73_RE_R_0_02_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA560373 = {S32345, 140, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; FUSIONideos Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA574814 = {S32346, 139, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; ckt16_a10y Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA624157 = {S32347, 140, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; it; TSB_CLOUD_COMPANION;TOSHIBA_AC_AND_AZ) AppleWebkit/533.1(KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA624889 = {S32348, 130, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MID Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626581 = {S32349, 147, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; D000-X-S01 Build/00.21.29-02.20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA632239 = {S32350, 134, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; generic/sourceidnull) AppleWebKit/533.1(KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA682978 = {S32351, 168, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; msm7227_ffa Build/I700T_P6_T06_AC86A_FT_H44_R_0_01_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA707339 = {S32352, 132, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; MID Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA711002 = {S32353, 134, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; hd-us; eeepc Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA725750 = {S32354, 167, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; msm7227_ffa Build/I700T_P6_T06_AC79_FT_H44_R_0_01_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA730933 = {S32355, 167, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; msm7227_ffa Build/I700T_P7_T07_NEXX_FT_H44_R_0_02_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA777476 = {S32356, 167, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; msm7227_ffa Build/I700T_P7_T01_TSMT_FT_S42_R_0_01_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA813356 = {S32357, 131, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Android 2.2; Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA859428 = {S32358, 134, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; eeepc Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1895742 = {S32359, 168, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; msm7227_ffa Build/I700T_P6_T02_NEXX_FT_R_0_03_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1911120 = {S32360, 58, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; mercury Build/FRF85B)*/
const Device UA1924922 = {S32361, 168, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; msm7227_ffa Build/I700T_P6_T02_NEXX_FT_R_0_04_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1931011 = {S32362, 37, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - stagefright/1.0 (Linux;Android 2.2.3)*/
const Device UA1972098 = {S32363, 135, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pl-pl; MID 01 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1988041 = {S32364, 136, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; hr-hr; ARIZONA Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1992164 = {S32365, 121, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; generic Build/Froyo) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2018189 = {S32366, 139, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2-update1; en-us; MID Build/froyo) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018685 = {S32367, 61, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; MID_Serails Build/FRF91)*/
const Device UA2018795 = {S32368, 139, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-mx; FUSIONideos Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018884 = {S32369, 65, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; Full Android Build/MASTER)*/
const Device UA2018897 = {S32370, 167, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; msm7227_ffa Build/I700T_P4_10_AC73_RE_R_0_05_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019056 = {S32371, 114, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019594 = {S32372, 167, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; msm7227_ffa Build/I700T_P4_10_AC74_RE_R_0_01_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019629 = {S32373, 136, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; vi-vn; IPHONE4 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019914 = {S32374, 178, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Android for Telechips TCC8900 Evaluation Board (US) Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020133 = {S32375, 169, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; msm7227_ffa Build/I700T_P4_11A_TSLSB_FT_R_0_01_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020284 = {S32376, 168, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; msm7227_ffa Build/I700T_P6_T02_AC01_FT4_H44_R_0_02_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020382 = {S32377, 132, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MID Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2020500 = {S32378, 183, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; th-th; msm7227_ffa Build/InoteLite_P4_17_FLSE_FT_R_0_07_1010_TH009_X) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2020630 = {S32379, 56, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 2.2) SW:V10e*/
const Device UA2021286 = {S32380, 136, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; hr-hr; ARIZONA Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021426 = {S32381, 116, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2022203 = {S32382, 139, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; vi-vn; Iphone A4S Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024234 = {S32383, 168, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; msm7227_ffa Build/I700T_P6_T06_AC80D_FT_H44_R_0_01_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024508 = {S32384, 168, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; msm7227_ffa Build/I700T_P6_T10_GSXX_FT_R_0_01_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024513 = {S32385, 135, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-; ANDROID MID Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027815 = {S32386, 168, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; msm7227_ffa Build/I700T_P7_T04_TSCDC_FT_S42_R_0_02_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031492 = {S32387, 125, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ko-kr; Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2371809 = {S32388, 135, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; St-A77 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2372381 = {S32389, 136, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; EVO_Tab Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2372729 = {S32390, 162, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Aedupac Xtab Light 100 Build/froyo_almond_r238) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2374895 = {S32391, 170, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-us; EF101 Build/Kaissen_neutral_I7003G_BOE_OV2_TV9.1_X_WCDMA) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2379102 = {S32392, 133, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; A35DE Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2392792 = {S32393, 135, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; ILT-MX100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2393396 = {S32394, 138, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; WDPF-703SE-E Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2395495 = {S32395, 133, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; A505 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2743801 = {S32396, 142, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - HuaweiT8500_TD/1.0 (Linux; U; Android 2.2.2; zh-cn) Release/7.27.2011 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 AppleWebKit/533.1*/
const Device UA2787040 = {S32397, 133, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; unknown Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2801486 = {S32398, 11, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Android 2.2*/
const Device UA2809312 = {S32399, 127, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; device Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2822410 = {S32400, 128, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.2 Version/4.0 Mobile Safari/530.17*/
const Device UA2949051 = {S32401, 137, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; ANDROID MID Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2980134 = {S32402, 167, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; msm7227_ffa Build/I700T_P6_T12_FLMO_FT_S42_R_0_02_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042485 = {S32403, 135, { &P17968,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; unknown Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2386026 = {S32404, 74, { &P17595,&P17296,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:17.0) Gecko/X Firefox/17.0*/
const Device UA359301 = {S32405, 127, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA363778 = {S32406, 128, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/ITL41F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA368834 = {S32407, 127, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA369276 = {S32408, 127, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA369321 = {S32409, 62, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 4.0.3; en_US; Xoom Build/IML77)*/
const Device UA369481 = {S32410, 128, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM30B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA401270 = {S32411, 128, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM30C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA402797 = {S32412, 125, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-us; Xoom Build/IRK77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA402844 = {S32413, 128, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Xoom Build/IRL09B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA402889 = {S32414, 127, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Xoom Build/IRL20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA403007 = {S32415, 127, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/IML75) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA403009 = {S32416, 128, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA403022 = {S32417, 128, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/IML74B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA403039 = {S32418, 127, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/IML81) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA403058 = {S32419, 127, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA403059 = {S32420, 114, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/IML77) AppleWebKit/535.7 (KHTML, like Gecko)  Safari/535.7*/
const Device UA403065 = {S32421, 127, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/IML70) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA403082 = {S32422, 127, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA403062 = {S32423, 128, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/HTK75D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA403129 = {S32424, 127, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM19) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA403144 = {S32425, 56, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Xoom Build/IML77)*/
const Device UA403165 = {S32426, 127, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA403175 = {S32427, 127, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA403190 = {S32428, 128, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/HLK75F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA403192 = {S32429, 134, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA403292 = {S32430, 127, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM48) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA616642 = {S32431, 127, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM76) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA710583 = {S32432, 128, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA760456 = {S32433, 128, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA830013 = {S32434, 128, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA901670 = {S32435, 57, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Xoom Build/IMM76L)*/
const Device UA2023611 = {S32436, 135, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025777 = {S32437, 134, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM76) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257601 = {S32438, 133, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MZ505 Build/Moto_MZ505) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2336971 = {S32439, 125, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; cn-zh; Xoom Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2653283 = {S32440, 62, { &P15869,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MZ505 Build/Moto_MZ505)*/
const Device UA910766 = {S32441, 136, { &P15364,&P17816,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86; %lang_code%) adbeat.com/policy Gecko/X Ubuntu/10.04 (lucid) Firefox/3.6.3 AppleWebKit/532.4 Safari/532.4*/
const Device UA2429717 = {S32442, 157, { &P18283,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-T999 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2431753 = {S32443, 157, { &P18283,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-I535 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2433908 = {S32444, 157, { &P18283,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-I9300 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2512378 = {S32445, 157, { &P18283,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-T999 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2513264 = {S32446, 157, { &P18283,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-I535 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2516387 = {S32447, 157, { &P18283,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-I9300 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2534903 = {S32448, 156, { &P18283,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2537048 = {S32449, 156, { &P18283,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T999 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2537055 = {S32450, 156, { &P18283,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I535 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2679675 = {S32451, 157, { &P18283,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9300 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2705715 = {S32452, 155, { &P18283,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I535 Build/MROM; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867466 = {S32453, 155, { &P18283,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/4.60; CyanogenMod-4.60) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2897967 = {S32454, 155, { &P18283,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/4.70; CyanogenMod-4.70) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2948412 = {S32455, 164, { &P18283,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T999 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/d2tmo*/
const Device UA2965087 = {S32456, 160, { &P18283,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/4.80) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/4.80/i9300*/
const Device UA2968143 = {S32457, 164, { &P18283,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I535 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/d2vzw*/
const Device UA79838 = {S32458, 34, { &P15364,&P17017,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0;*/
const Device UA97279 = {S32459, 8, { &P15364,&P17017,&P17191,&P18092 }};	/*Useragent - MSIE 6.0*/
const Device UA130753 = {S32460, 42, { &P15364,&P17017,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; ) Gecko*/
const Device UA151381 = {S32461, 33, { &P15364,&P17017,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0i(compatible;MSIE 6.0)*/
const Device UA166332 = {S32462, 77, { &P15364,&P17017,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (X11; MSIE 6.0; i686; .NET CLR 1.1.4322; .NET CLR 2.0.50727; FDM)*/
const Device UA413360 = {S32463, 32, { &P15364,&P17017,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible;MSIE6.0)*/
const Device UA481533 = {S32464, 35, { &P15364,&P17017,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (MSIE 6.0); dtaglim:128*/
const Device UA546515 = {S32465, 36, { &P15364,&P17017,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.00;)*/
const Device UA698557 = {S32466, 34, { &P15364,&P17017,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible: MSIE 6.0)*/
const Device UA1842988 = {S32467, 22, { &P15364,&P17017,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (MSIE 6.0)*/
const Device UA26972 = {S32468, 160, { &P13659,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2; en-us; LG-P509 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA26975 = {S32469, 160, { &P13659,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us;LG-P509 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA26979 = {S32470, 160, { &P13659,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-P509Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA26984 = {S32471, 140, { &P13659,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-P509 Build/FRF91) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA30138 = {S32472, 160, { &P13659,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-P509 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA30139 = {S32473, 160, { &P13659,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2; en-us; LG-P509 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA40519 = {S32474, 163, { &P13659,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P509Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA40524 = {S32475, 163, { &P13659,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.2; en-us; LG-P509 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA40533 = {S32476, 159, { &P13659,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ru-; LG-P509 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA40546 = {S32477, 163, { &P13659,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P509 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA55808 = {S32478, 163, { &P13659,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.2; en-us; LG-P509 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA55812 = {S32479, 163, { &P13659,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;LG-P509 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA55830 = {S32480, 164, { &P13659,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P509 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA63412 = {S32481, 161, { &P13659,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-P509 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA170013 = {S32482, 57, { &P13659,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; LG-P509 Build/FRF91)*/
const Device UA395963 = {S32483, 60, { &P13659,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P509 Build/FRG83G)*/
const Device UA659960 = {S32484, 163, { &P13659,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P509 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA3026582 = {S32485, 133, { &P15767,&P18432,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A523 Safari/8536.25*/
const Device UA3026721 = {S32486, 131, { &P15767,&P18432,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A403 Safari/8536.25*/
const Device UA3027081 = {S32487, 131, { &P15767,&P18432,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A407 Safari/8536.25*/
const Device UA3027137 = {S32488, 133, { &P15767,&P18432,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A550 Safari/8536.25*/
const Device UA3027332 = {S32489, 134, { &P15767,&P18432,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A8426 Safari/8536.25*/
const Device UA3028827 = {S32490, 131, { &P15767,&P18432,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A406 Safari/8536.25*/
const Device UA3030163 = {S32491, 134, { &P15767,&P18432,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A8500 Safari/8536.25*/
const Device UA672020 = {S32492, 136, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT875 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA694980 = {S32493, 136, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; XT875 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA762431 = {S32494, 143, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID BIONIC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA823786 = {S32495, 145, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; XT875 Build/5.5.1_84_DBN-71) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA908826 = {S32496, 146, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2_223) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA908908 = {S32497, 158, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2-223_DBN_M4-3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA909240 = {S32498, 149, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2_223) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA909429 = {S32499, 155, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2-223_DBN_M4-3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA911679 = {S32500, 158, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2-223_DBN_M4-7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA912480 = {S32501, 158, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2-223_DBN_M4-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA923868 = {S32502, 155, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2-223_DBN_M4-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1938805 = {S32503, 145, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; XT875 Build/5.5.1_84_D3G-55) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1972845 = {S32504, 155, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2-223_DBN_M4-6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1979285 = {S32505, 155, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2-223_DBN_M4-7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1993555 = {S32506, 158, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2-223_DBN_M4-6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996935 = {S32507, 159, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2-223_DBN_M4-12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029590 = {S32508, 159, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2-223_DBN_M4-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031032 = {S32509, 159, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2-223_DBN_M4-15) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032022 = {S32510, 157, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/LIQUIDBLUR-V1.0-JMOD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2230427 = {S32511, 159, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2-223_DBN_M4-23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2232261 = {S32512, 156, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2-223_DBN_M4-24) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2234000 = {S32513, 78, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; DROID BIONIC Build/6.7.2-223_DBN_M4-23)*/
const Device UA2236425 = {S32514, 156, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2237051 = {S32515, 159, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2-223_DBN_M4-24) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257639 = {S32516, 162, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2257666 = {S32517, 149, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2339720 = {S32518, 156, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2-223_DBN_M4-12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2349276 = {S32519, 159, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2-223_DBN_M4-19) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2386287 = {S32520, 143, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2403695 = {S32521, 140, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2478881 = {S32522, 159, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/6.7.2-223_DBN_M4-21) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2502754 = {S32523, 143, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2553432 = {S32524, 140, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2562377 = {S32525, 152, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/Icarus HD 1.7.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2563813 = {S32526, 155, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/Icarus HD 1.7.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2577852 = {S32527, 154, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2602462 = {S32528, 161, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC 4G Build/Icarus HD 1.8.3 ReMiX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2703144 = {S32529, 158, { &P12605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/Icarus HD 1.8.3 ReMiX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2431219 = {S32530, 89, { &P18115,&P5764,&P17373,&P18092 }};	/*Useragent - Opera/9.80 (BlackBerry; Opera Mini/6.5.27548/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2974270 = {S32531, 137, { &P18471,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-78) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974665 = {S32532, 143, { &P18471,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT905 Build/9.8.2Q_SMUIRD-7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974735 = {S32533, 139, { &P18471,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XT907 Build/9.8.1Q_27-2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2043785 = {S32534, 124, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC)*/
const Device UA2043787 = {S32535, 107, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Accord)*/
const Device UA2197873 = {S32536, 108, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; PM23300)*/
const Device UA2423680 = {S32537, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 1.00.183.02)*/
const Device UA2427212 = {S32538, 132, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; Orange)*/
const Device UA2467250 = {S32539, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 1.00.166.02)*/
const Device UA2507679 = {S32540, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.166.01)*/
const Device UA2507684 = {S32541, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.161.01)*/
const Device UA2508185 = {S32542, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.166.02)*/
const Device UA2509941 = {S32543, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.163.02)*/
const Device UA2528128 = {S32544, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 1.00.162.02)*/
const Device UA2528266 = {S32545, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 1.00.163.03)*/
const Device UA2530977 = {S32546, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.172.02)*/
const Device UA2555508 = {S32547, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 1.00.161.02)*/
const Device UA2557848 = {S32548, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.163.01)*/
const Device UA2559050 = {S32549, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.161.02)*/
const Device UA2559595 = {S32550, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.163.04)*/
const Device UA2560740 = {S32551, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.161.04)*/
const Device UA2563045 = {S32552, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.172.04)*/
const Device UA2563410 = {S32553, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.166.04)*/
const Device UA2582912 = {S32554, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.162.04)*/
const Device UA2597037 = {S32555, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.164.02)*/
const Device UA2599587 = {S32556, 106, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; C620e)*/
const Device UA2656770 = {S32557, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 1.00.172.02)*/
const Device UA2658028 = {S32558, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.162.01)*/
const Device UA2662078 = {S32559, 112, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; C620t; CMCC)*/
const Device UA2663097 = {S32560, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.183.04)*/
const Device UA2673479 = {S32561, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.01.163.00)*/
const Device UA2675646 = {S32562, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.01.161.00)*/
const Device UA2676460 = {S32563, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.01.166.00)*/
const Device UA2697698 = {S32564, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.172.01)*/
const Device UA2713476 = {S32565, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.01.172.00)*/
const Device UA2717005 = {S32566, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.01.162.00)*/
const Device UA2727243 = {S32567, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 1.00.164.02)*/
const Device UA2728521 = {S32568, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.183.01)*/
const Device UA2767554 = {S32569, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.01.164.00)*/
const Device UA2781507 = {S32570, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.01.183.00)*/
const Device UA2844472 = {S32571, 120, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; C620d; ChinaTelecom)*/
const Device UA2854845 = {S32572, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 2.00.164.04)*/
const Device UA3036027 = {S32573, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 3.41.161.02)*/
const Device UA3061542 = {S32574, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 3.41.162.02)*/
const Device UA3066788 = {S32575, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 3.41.166.02)*/
const Device UA3082588 = {S32576, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 3.41.163.02)*/
const Device UA3093329 = {S32577, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 3.41.164.02)*/
const Device UA3126262 = {S32578, 137, { &P18502,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8X by HTC; 3.41.183.02)*/
const Device UA48012 = {S32579, 153, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-52) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48014 = {S32580, 153, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48015 = {S32581, 150, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-52) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48016 = {S32582, 153, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-50) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48017 = {S32583, 150, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48019 = {S32584, 163, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-91-LaunchKit) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60964 = {S32585, 150, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-44) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60965 = {S32586, 150, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-47) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60966 = {S32587, 150, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-46) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60968 = {S32588, 150, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70226 = {S32589, 153, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-56) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70350 = {S32590, 160, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-91-LaunchKit) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70378 = {S32591, 153, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-57) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71053 = {S32592, 153, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-59) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71369 = {S32593, 153, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74123 = {S32594, 153, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-61) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74916 = {S32595, 150, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-61) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA77900 = {S32596, 153, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79307 = {S32597, 153, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-60) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA252229 = {S32598, 150, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA252428 = {S32599, 159, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-62_MR-11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA357880 = {S32600, 153, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-70) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA362332 = {S32601, 150, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-70) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA605229 = {S32602, 153, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-73) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA609306 = {S32603, 150, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-73) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623091 = {S32604, 153, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-71) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623098 = {S32605, 150, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-71) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA641467 = {S32606, 156, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-62_MR-11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA651752 = {S32607, 158, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-62_MR-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA676766 = {S32608, 155, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-62_MR-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA896127 = {S32609, 153, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA896381 = {S32610, 150, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA901297 = {S32611, 74, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; DROID BIONIC Build/5.5.1_84_DBN-74)*/
const Device UA1905475 = {S32612, 158, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-62_MR-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020798 = {S32613, 153, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-68) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028546 = {S32614, 152, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC 4G Build/5.5.1_84_DBN-74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA2201776 = {S32615, 158, { &P12605,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-74) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA3068965 = {S32616, 131, { &P15767,&P22882,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1590.0 Mobile/12E55 Safari/8536.25*/
const Device UA3096147 = {S32617, 133, { &P15767,&P22882,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A465 Safari/8536.25*/
const Device UA3096615 = {S32618, 135, { &P15767,&P22882,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A4449d Safari/8536.25*/
const Device UA3109921 = {S32619, 135, { &P15767,&P22882,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A4449a Safari/8536.25*/
const Device UA3115351 = {S32620, 135, { &P15767,&P22882,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A501 Safari/8536.25*/
const Device UA91045 = {S32621, 117, { &P17595,&P17297,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA252004 = {S32622, 117, { &P17595,&P17297,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2*/
const Device UA379236 = {S32623, 115, { &P17595,&P17297,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.861.0 Safari/535.2*/
const Device UA628844 = {S32624, 117, { &P17595,&P17297,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2*/
const Device UA669395 = {S32625, 117, { &P17595,&P17297,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2*/
const Device UA2202361 = {S32626, 117, { &P17595,&P17297,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2*/
const Device UA49320 = {S32627, 142, { &P12987,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SPH-M930 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70118 = {S32628, 145, { &P12987,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SPH-M930BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA615791 = {S32629, 145, { &P12987,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M930BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA624336 = {S32630, 66, { &P12987,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SPH-M930 Build/GINGERBREAD)*/
const Device UA624817 = {S32631, 69, { &P12987,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SPH-M930BST Build/GINGERBREAD)*/
const Device UA627326 = {S32632, 69, { &P12987,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SPH-M930BST Build/GINGERBREAD)*/
const Device UA632767 = {S32633, 142, { &P12987,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M930 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA673864 = {S32634, 66, { &P12987,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SPH-M930 Build/GINGERBREAD)*/
const Device UA2020057 = {S32635, 152, { &P12987,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SPH-M930BST Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2698107 = {S32636, 149, { &P12987,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M930BST Build/ROM by Bloodawn) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2698397 = {S32637, 146, { &P12987,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M930 Build/ROM by Bloodawn) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3098325 = {S32638, 143, { &P12987,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SPH-M930BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2740941 = {S32639, 58, { &P15364,&P5690,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:22.0) Gecko/X Firefox/22.0*/
const Device UA2740959 = {S32640, 65, { &P15364,&P5690,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64; rv:22.0) Gecko/X Firefox/22.0*/
const Device UA2947722 = {S32641, 77, { &P15364,&P5690,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:22.0) Gecko/X Firefox/22.0 IceDragon/22.0.0.1*/
const Device UA3057191 = {S32642, 84, { &P15364,&P5690,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:22.0) Gecko/X Firefox/22.0 IceDragon/22.0.0.1*/
const Device UA3089415 = {S32643, 70, { &P15364,&P5690,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 5.1; rv:22.0) Gecko/22.0 Firefox/22.0*/
const Device UA2569938 = {S32644, 133, { &P15364,&P18118,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu Chromium/25.0.1364.160 Chrome/25.0.1364.160 Safari/537.22*/
const Device UA2571463 = {S32645, 135, { &P15364,&P18118,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu Chromium/25.0.1364.160 Chrome/25.0.1364.160 Safari/537.22*/
const Device UA2667590 = {S32646, 145, { &P15364,&P18118,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu Chromium/25.0.1364.160 Chrome/25.0.1364.160 Safari/537.22,gzip(gfe)*/
const Device UA2668142 = {S32647, 143, { &P15364,&P18118,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu Chromium/25.0.1364.160 Chrome/25.0.1364.160 Safari/537.22,gzip(gfe)*/
const Device UA2672763 = {S32648, 135, { &P15364,&P18118,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu Chromium/25.0.1364.160 Chrome/25.0.1364.160 Safari/537.22*/
const Device UA2704609 = {S32649, 174, { &P15364,&P18118,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu Chromium/25.0.1364.160 Chrome/25.0.1364.160 Safari/537.22,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2707783 = {S32650, 172, { &P15364,&P18118,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu Chromium/25.0.1364.160 Chrome/25.0.1364.160 Safari/537.22,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2993952 = {S32651, 132, { &P15364,&P18118,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.22 (KHTML like Gecko) Ubuntu Chromium/25.0.1364.160 Chrome/25.0.1364.160 Safari/537.22*/
const Device UA29886 = {S32652, 143, { &P17346,&P17084,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B118 Safari/6531.22.7*/
const Device UA29896 = {S32653, 143, { &P17346,&P17084,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA417205 = {S32654, 143, { &P17346,&P17084,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/9Z999 Safari/6531.22.7*/
const Device UA2437890 = {S32655, 145, { &P17346,&P17084,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2658954 = {S32656, 141, { &P17346,&P17084,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 41 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA2659433 = {S32657, 141, { &P17346,&P17084,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 41 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML like Gecko) Version/4.0.5 Mobile/8B118 Safari/6531.22.7*/
const Device UA2358595 = {S32658, 57, { &P15364,&P16110,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2) Presto/2.12.388 Version/12.11*/
const Device UA2362130 = {S32659, 64, { &P15364,&P16110,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.11*/
const Device UA2406140 = {S32660, 86, { &P15364,&P16110,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.11*/
const Device UA2424629 = {S32661, 86, { &P15364,&P16110,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.11*/
const Device UA2426866 = {S32662, 86, { &P15364,&P16110,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.11*/
const Device UA2427414 = {S32663, 86, { &P15364,&P16110,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.11*/
const Device UA2429778 = {S32664, 74, { &P15364,&P16110,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.11,gzip(gfe)*/
const Device UA2457518 = {S32665, 86, { &P15364,&P16110,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.11*/
const Device UA2482436 = {S32666, 86, { &P15364,&P16110,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.11*/
const Device UA2506695 = {S32667, 86, { &P15364,&P16110,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.11*/
const Device UA2793214 = {S32668, 86, { &P15364,&P16110,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.11*/
const Device UA2993887 = {S32669, 86, { &P15364,&P16110,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.11*/
const Device UA3074172 = {S32670, 86, { &P15364,&P16110,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.11*/
const Device UA2390865 = {S32671, 74, { &P17595,&P18207,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:17.0) Gecko/X Firefox/17.0*/
const Device UA2797840 = {S32672, 76, { &P18088,&P22781,&P4732,&P18092 }};	/*Useragent - Opera/9.80 (VRE; Opera Mini/4.2/30.3214; U; en) Presto/2.8.119 Version/11.10*/
const Device UA32982 = {S32673, 150, { &P12814,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SAMSUNG-SGH-I777 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79247 = {S32674, 150, { &P12814,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SAMSUNG-SGH-I777 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250528 = {S32675, 136, { &P12814,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-I777 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA360013 = {S32676, 142, { &P12814,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-I777 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA361170 = {S32677, 74, { &P12814,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SAMSUNG-SGH-I777 Build/GINGERBREAD)*/
const Device UA369318 = {S32678, 86, { &P12814,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.6; en_US; SAMSUNG-SGH-I777 Build/GINGERBREAD.UCKK6)*/
const Device UA409545 = {S32679, 74, { &P12814,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SAMSUNG-SGH-I777 Build/GINGERBREAD)*/
const Device UA622582 = {S32680, 150, { &P12814,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I777 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1938678 = {S32681, 148, { &P12814,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SAMSUNG-SGH-I777 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022456 = {S32682, 60, { &P12814,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; SGH-I777 Build/GWK74)*/
const Device UA2032380 = {S32683, 139, { &P12814,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SGH-I777 Build/DoomZday) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA347239 = {S32684, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.60*/
const Device UA347283 = {S32685, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.10.229 Version/11.60*/
const Device UA347241 = {S32686, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.60*/
const Device UA347335 = {S32687, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.10.229 Version/11.60*/
const Device UA347491 = {S32688, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.10.229 Version/11.60*/
const Device UA347528 = {S32689, 101, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.229 Version/11.60,gzip(gfe) (via translate.google.com)*/
const Device UA347773 = {S32690, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.60*/
const Device UA349338 = {S32691, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.10.229 Version/11.60*/
const Device UA351485 = {S32692, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.10.229 Version/11.60*/
const Device UA351518 = {S32693, 66, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; en) Opera 11.60*/
const Device UA351532 = {S32694, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.10.229 Version/11.60*/
const Device UA351571 = {S32695, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.60*/
const Device UA351640 = {S32696, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.10.229 Version/11.60*/
const Device UA351673 = {S32697, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4930); ru) Presto/2.10.229 Version/11.60*/
const Device UA351707 = {S32698, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4871); ru) Presto/2.10.229 Version/11.60*/
const Device UA351734 = {S32699, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.10.229 Version/11.60*/
const Device UA351782 = {S32700, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.10.229 Version/11.60*/
const Device UA351745 = {S32701, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.10.229 Version/11.60*/
const Device UA351787 = {S32702, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.10.229 Version/11.60*/
const Device UA352097 = {S32703, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.10.229 Version/11.60*/
const Device UA352110 = {S32704, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.10.229 Version/11.60*/
const Device UA352602 = {S32705, 113, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94; ru) Presto/2.10.229 Version/11.60*/
const Device UA352713 = {S32706, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.10.229 Version/11.60*/
const Device UA353241 = {S32707, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4661); ru) Presto/2.10.229 Version/11.60*/
const Device UA355424 = {S32708, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03672); ru) Presto/2.10.229 Version/11.60*/
const Device UA364119 = {S32709, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.60*/
const Device UA437851 = {S32710, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.10.229 Version/11.60*/
const Device UA576103 = {S32711, 64, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; en) Presto/2.10.229 Version/11.60*/
const Device UA588642 = {S32712, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.10.229 Version/11.60*/
const Device UA599689 = {S32713, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.10.229 Version/11.60*/
const Device UA600150 = {S32714, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5267); ru) Presto/2.10.229 Version/11.60*/
const Device UA607798 = {S32715, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.10.229 Version/11.60*/
const Device UA640237 = {S32716, 64, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; en) Presto/2.10.229 Version/11.60*/
const Device UA642054 = {S32717, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.10.229 Version/11.60*/
const Device UA648935 = {S32718, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4947); ru) Presto/2.10.229 Version/11.60*/
const Device UA656910 = {S32719, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03780); ru) Presto/2.10.229 Version/11.60*/
const Device UA662193 = {S32720, 72, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.60*/
const Device UA685705 = {S32721, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4832); ru) Presto/2.10.229 Version/11.60*/
const Device UA688973 = {S32722, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4841); ru) Presto/2.10.229 Version/11.60*/
const Device UA698299 = {S32723, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.10.229 Version/11.60*/
const Device UA704084 = {S32724, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.10.229 Version/11.60*/
const Device UA708154 = {S32725, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.10.229 Version/11.60*/
const Device UA708903 = {S32726, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.10.229 Version/11.60*/
const Device UA711276 = {S32727, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03791); ru) Presto/2.10.229 Version/11.60*/
const Device UA726054 = {S32728, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.10.229 Version/11.60*/
const Device UA730600 = {S32729, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02564); ru) Presto/2.10.229 Version/11.60*/
const Device UA790903 = {S32730, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.10.229 Version/11.60*/
const Device UA826276 = {S32731, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5218); ru) Presto/2.10.229 Version/11.60*/
const Device UA895650 = {S32732, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.10.229 Version/11.60*/
const Device UA903355 = {S32733, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.10.229 Version/11.60*/
const Device UA2156530 = {S32734, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.10.229 Version/11.60*/
const Device UA2159362 = {S32735, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.10.229 Version/11.60*/
const Device UA2164830 = {S32736, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5328); ru) Presto/2.10.229 Version/11.60*/
const Device UA2166987 = {S32737, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.10.229 Version/11.60*/
const Device UA2191497 = {S32738, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.10.229 Version/11.60*/
const Device UA2227154 = {S32739, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.10.229 Version/11.60*/
const Device UA2229759 = {S32740, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.10.229 Version/11.60*/
const Device UA2308940 = {S32741, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02560); ru) Presto/2.10.229 Version/11.60*/
const Device UA2317443 = {S32742, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5704); ru) Presto/2.10.229 Version/11.60*/
const Device UA2319066 = {S32743, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5230); ru) Presto/2.10.229 Version/11.60*/
const Device UA2322463 = {S32744, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5667); ru) Presto/2.10.229 Version/11.60*/
const Device UA2325826 = {S32745, 87, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5308); ru) Presto/2.10.229 Version/11.60*/
const Device UA2385166 = {S32746, 74, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.10.229 Version/11.60,gzip(gfe)*/
const Device UA2424035 = {S32747, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.10.229 Version/11.60*/
const Device UA2428623 = {S32748, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.10.229 Version/11.60*/
const Device UA2460925 = {S32749, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.10.229 Version/11.60*/
const Device UA2480733 = {S32750, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5784); ru) Presto/2.10.229 Version/11.60*/
const Device UA2480954 = {S32751, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.10.229 Version/11.60*/
const Device UA2645475 = {S32752, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5966); ru) Presto/2.10.229 Version/11.60*/
const Device UA2704212 = {S32753, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.10.229 Version/11.60*/
const Device UA2734867 = {S32754, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6083); ru) Presto/2.10.229 Version/11.60*/
const Device UA2748511 = {S32755, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6037); ru) Presto/2.10.229 Version/11.60*/
const Device UA2751005 = {S32756, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6089); ru) Presto/2.10.229 Version/11.60*/
const Device UA2766048 = {S32757, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6081); ru) Presto/2.10.229 Version/11.60*/
const Device UA2855506 = {S32758, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6060); ru) Presto/2.10.229 Version/11.60*/
const Device UA2868090 = {S32759, 86, { &P15364,&P5690,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6578); ru) Presto/2.10.229 Version/11.60*/
const Device UA1996415 = {S32760, 135, { &P19904,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C5170 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029189 = {S32761, 57, { &P19904,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; C5170 Build/IML77)*/
const Device UA2481680 = {S32762, 136, { &P19904,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Hydro Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2661822 = {S32763, 58, { &P19904,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Hydro Build/IMM76I)*/
const Device UA3044540 = {S32764, 133, { &P19904,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; C5170 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3046814 = {S32765, 198, { &P19904,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C5170 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 androidAppVersion/2.5.22 AndroidWebkitWrapper BillingSupported*/
const Device UA3092025 = {S32766, 180, { &P19904,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C5170 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 androidAppVersion/2.6.8 AndroidWebkitWrapper*/
const Device UA686772 = {S32767, 64, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.10.289 Version/12.00*/
const Device UA856006 = {S32768, 86, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4953); ru) Presto/2.10.289 Version/12.00*/
const Device UA904234 = {S32769, 87, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5310); ru) Presto/2.10.289 Version/12.00*/
const Device UA904595 = {S32770, 87, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5339); ru) Presto/2.10.289 Version/12.00*/
const Device UA905182 = {S32771, 87, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.10.289 Version/12.00*/
const Device UA905764 = {S32772, 87, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.10.289 Version/12.00*/
const Device UA908537 = {S32773, 123, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; U; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0); en) Presto/2.10.289 Version/12.00*/
const Device UA910949 = {S32774, 86, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5711); ru) Presto/2.10.289 Version/12.00*/
const Device UA1954736 = {S32775, 87, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.4 (build 02647); ru) Presto/2.10.289 Version/12.00*/
const Device UA1959677 = {S32776, 116, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0); en) Presto/2.10.289 Version/12.00*/
const Device UA1961373 = {S32777, 102, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; compatible; MSIE 7.0; Windows NT 6.0; bg) Presto/2.10.289 Version/12.00*/
const Device UA1964347 = {S32778, 86, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5680); ru) Presto/2.10.289 Version/12.00*/
const Device UA1975977 = {S32779, 86, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5754); ru) Presto/2.10.289 Version/12.00*/
const Device UA1986560 = {S32780, 87, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.10.289 Version/12.00*/
const Device UA1991883 = {S32781, 101, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; de) Presto/2.10.289 Version/12.00,gzip(gfe) (via translate.google.com)*/
const Device UA1995930 = {S32782, 64, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; en) Presto/2.10.289 Version/12.01*/
const Device UA1999022 = {S32783, 87, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.10.289 Version/12.01*/
const Device UA1999484 = {S32784, 87, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.10.289 Version/12.01*/
const Device UA2002926 = {S32785, 87, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5339); ru) Presto/2.10.289 Version/12.01*/
const Device UA2003694 = {S32786, 87, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5310); ru) Presto/2.10.289 Version/12.01*/
const Device UA2004648 = {S32787, 116, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0); en) Presto/2.10.289 Version/12.01*/
const Device UA2004663 = {S32788, 86, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5831); ru) Presto/2.10.289 Version/12.01*/
const Device UA2007357 = {S32789, 86, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4953); ru) Presto/2.10.289 Version/12.01*/
const Device UA2164067 = {S32790, 87, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03686); ru) Presto/2.10.289 Version/12.00*/
const Device UA2168329 = {S32791, 86, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5680); ru) Presto/2.10.289 Version/12.01*/
const Device UA2168791 = {S32792, 87, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.10.289 Version/12.01*/
const Device UA2169803 = {S32793, 87, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03686); ru) Presto/2.10.289 Version/12.01*/
const Device UA2177124 = {S32794, 86, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5972); ru) Presto/2.10.289 Version/12.01*/
const Device UA2224091 = {S32795, 101, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.10.289 Version/12.01,gzip(gfe) (via translate.google.com)*/
const Device UA2262589 = {S32796, 86, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5831); ru) Presto/2.10.289 Version/12.00*/
const Device UA2306921 = {S32797, 72, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; en; rv:2.0) Gecko/X Firefox/4.0 Opera 12.00*/
const Device UA2369095 = {S32798, 63, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/12.0 (Windows NT 6.0; U; en)Presto/22.9.168 Version/12.00*/
const Device UA2548227 = {S32799, 86, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5754); ru) Presto/2.10.289 Version/12.01*/
const Device UA2641627 = {S32800, 86, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4598); ru) Presto/2.10.289 Version/12.00*/
const Device UA2776394 = {S32801, 86, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5970); ru) Presto/2.10.289 Version/12.01*/
const Device UA2809298 = {S32802, 86, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4133); ru) Presto/2.10.289 Version/12.00*/
const Device UA2809323 = {S32803, 113, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94; ru) Presto/2.10.289 Version/12.00*/
const Device UA2811380 = {S32804, 86, { &P15364,&P17285,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4139); ru) Presto/2.10.289 Version/12.00*/
const Device UA22166 = {S32805, 136, { &P15389,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-nz; IDEOS X5 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA45290 = {S32806, 142, { &P15389,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-za; IDEOS X5 Build/HuaweiU8800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49879 = {S32807, 142, { &P15389,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-ve; U8800 Build/HuaweiU8800-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70297 = {S32808, 149, { &P15389,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Huawei U8800-51 Build/HWU8800B635) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70551 = {S32809, 149, { &P15389,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Huawei U8800-51 Build/HWU8800B630) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90546 = {S32810, 133, { &P15389,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; u8800 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA192688 = {S32811, 139, { &P15389,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; u8800 Build/HuaweiU8800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA232356 = {S32812, 142, { &P15389,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; tr-tr; IDEOS X5 Build/HuaweiU8800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368899 = {S32813, 140, { &P15389,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-; IDEOS X5 Build/HuaweiU8800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA417663 = {S32814, 57, { &P15389,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; u8800 Build/FRG83)*/
const Device UA534697 = {S32815, 60, { &P15389,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; IDEOS X5 Build/FRG83)*/
const Device UA672990 = {S32816, 73, { &P15389,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; Huawei U8800-51 Build/HWU8800B635)*/
const Device UA673778 = {S32817, 139, { &P15389,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; nb-no; u8800 Build/HuaweiU8800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA737503 = {S32818, 145, { &P15389,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; U8800-51 Build/HuaweiU8800-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1994524 = {S32819, 152, { &P15389,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Huawei U8800-51 Build/HuaweiU8800-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020462 = {S32820, 139, { &P15389,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ru-uz; Ucell Build/HuaweiU8800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021215 = {S32821, 141, { &P15389,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ru-by; MTC Neo Build/HuaweiU8800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2189300 = {S32822, 149, { &P15389,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Huawei U8800-51 Build/HWU8800B661) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2362709 = {S32823, 79, { &P15364,&P7133,&P22733,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Avant Browser)*/
const Device UA2362714 = {S32824, 86, { &P15364,&P7133,&P22733,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Avant Browser)*/
const Device UA2426030 = {S32825, 91, { &P15364,&P7133,&P22733,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Win64; x64; Trident/6.0; Avant Browser)*/
const Device UA609305 = {S32826, 139, { &P15575,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  GT-I9100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA609311 = {S32827, 139, { &P15575,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  GT-I9100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA610180 = {S32828, 133, { &P15575,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  GT-I9100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA611525 = {S32829, 140, { &P15575,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  GT-I9100T Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA615498 = {S32830, 140, { &P15575,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  GT-I9100M Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA900639 = {S32831, 138, { &P15575,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA900683 = {S32832, 138, { &P15575,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA900743 = {S32833, 139, { &P15575,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100M Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA900956 = {S32834, 139, { &P15575,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100T Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA904660 = {S32835, 139, { &P15575,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1954680 = {S32836, 132, { &P15575,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1992745 = {S32837, 140, { &P15575,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  GT-I9100G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA2005034 = {S32838, 139, { &P15575,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2027092 = {S32839, 149, { &P15575,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Galaxy SII T-mobile Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2030129 = {S32840, 132, { &P15575,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2261465 = {S32841, 139, { &P15575,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100T Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA51086 = {S32842, 151, { &P14985,&P5520,&P3797,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8530/S8530JPKA1; U; Bada/1.2; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.2 Mobile WVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA68456 = {S32843, 96, { &P14985,&P5520,&P3797,&P18092 }};	/*Useragent - SAMSUNG-GT-S8530/1.0 SHP/VPP/R5 Dolfin/2.2 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA1996354 = {S32844, 139, { &P16362,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-D710 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2198944 = {S32845, 139, { &P16362,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-D710 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2205049 = {S32846, 139, { &P16362,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-D710 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257930 = {S32847, 139, { &P16362,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-D710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2416217 = {S32848, 61, { &P16362,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SPH-D710 Build/JRO03L)*/
const Device UA2466471 = {S32849, 61, { &P16362,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SPH-D710 Build/JZO54K)*/
const Device UA2580123 = {S32850, 143, { &P16362,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-D710VMUB Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2672411 = {S32851, 142, { &P16362,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-D710BST Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2703975 = {S32852, 65, { &P16362,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SPH-D710VMUB Build/JZO54K)*/
const Device UA2767543 = {S32853, 64, { &P16362,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SPH-D710BST Build/JZO54K)*/
const Device UA2985461 = {S32854, 137, { &P16362,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SPH-D710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3004157 = {S32855, 139, { &P16362,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ar-ye; SPH-D710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 ?????? Safari/534.30*/
const Device UA924442 = {S32856, 127, { &P12280,&P18432,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A5355d Twitter for iPhone*/
const Device UA2041377 = {S32857, 125, { &P12280,&P18432,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Twitter for iPhone*/
const Device UA2202710 = {S32858, 125, { &P12280,&P18432,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Twitter for iPhone*/
const Device UA2261146 = {S32859, 127, { &P12280,&P18432,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Twitter for iPhone*/
const Device UA2261148 = {S32860, 127, { &P12280,&P18432,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Twitter for iPhone*/
const Device UA2437560 = {S32861, 127, { &P12280,&P18432,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Twitter for iPhone*/
const Device UA79959 = {S32862, 82, { &P15364,&P17285,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/14*/
const Device UA249763 = {S32863, 106, { &P15364,&P17285,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA250072 = {S32864, 99, { &P15364,&P17285,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA250269 = {S32865, 136, { &P15364,&P17285,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1,gzip(gfe) (via translate.google.com)*/
const Device UA2885383 = {S32866, 141, { &P15364,&P16110,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100 (Edition Next)*/
const Device UA2885392 = {S32867, 134, { &P15364,&P16110,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100 (Edition Next)*/
const Device UA3027539 = {S32868, 126, { &P15364,&P16110,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153*/
const Device UA3027765 = {S32869, 119, { &P15364,&P16110,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153*/
const Device UA3028083 = {S32870, 148, { &P15364,&P16110,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Campaign 21)*/
const Device UA3029032 = {S32871, 144, { &P15364,&P16110,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Rambler)*/
const Device UA3029084 = {S32872, 136, { &P15364,&P16110,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153,gzip(gfe)*/
const Device UA3029911 = {S32873, 139, { &P15364,&P16110,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Yx)*/
const Device UA3030634 = {S32874, 141, { &P15364,&P16110,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Campaign 21)*/
const Device UA3059048 = {S32875, 148, { &P15364,&P16110,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Campaign 06)*/
const Device UA2713770 = {S32876, 141, { &P18223,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT22i Build/6.2.A.0.400) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2754471 = {S32877, 141, { &P18223,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; LT22i Build/6.2.A.0.402) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781911 = {S32878, 141, { &P18223,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-at; LT22i Build/6.2.A.0.404) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2875565 = {S32879, 145, { &P18223,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyLT22i Build/6.2.A.1.100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2877418 = {S32880, 63, { &P18223,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LT22i Build/6.2.A.1.100)*/
const Device UA2896091 = {S32881, 141, { &P18223,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LT22i Build/6.2.A.1.100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3020270 = {S32882, 147, { &P18223,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; SonyLT22i-o Build/6.2.A.1.100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3073352 = {S32883, 65, { &P18223,&P18111,&P18112,&P18092 }};	/*Useragent - SonyLT22i Build/6.2.A.0.400 stagefright/1.2 (Linux;Android 4.1.2)*/
const Device UA3124484 = {S32884, 65, { &P18223,&P18111,&P18112,&P18092 }};	/*Useragent - SonyLT22i Build/6.2.A.1.100 stagefright/1.2 (Linux;Android 4.1.2)*/
const Device UA369241 = {S32885, 151, { &P18223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; sv-se; SonyEricssonLT22i Build/6.0.B.0.639) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA902854 = {S32886, 153, { &P18223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; SonyEricssonLT22i-o Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA905080 = {S32887, 151, { &P18223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonLT22i Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1950977 = {S32888, 132, { &P18223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT22i Build/6.0.B.1.564) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1959852 = {S32889, 139, { &P18223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; LT22i Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1961391 = {S32890, 63, { &P18223,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; LT22i Build/6.0.B.1.564)*/
const Device UA1982662 = {S32891, 151, { &P18223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; SonyEricssonLT22i Build/6.0.B.1.576) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1987386 = {S32892, 153, { &P18223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; SonyEricssonLT22i-o Build/6.0.B.1.576) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1991913 = {S32893, 151, { &P18223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-mx; SonyEricssonLT22i Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2004688 = {S32894, 139, { &P18223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LT22i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2007380 = {S32895, 151, { &P18223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; nl-nl; SonyEricssonLT22i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2009137 = {S32896, 153, { &P18223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; SonyEricssonLT22i-o Build/6.0.B.3.187) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2016153 = {S32897, 151, { &P18223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-sg; SonyEricssonLT22i Build/6.0.B.3.188) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029623 = {S32898, 127, { &P18223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT22i Build/6.0.B.1.564) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2033150 = {S32899, 151, { &P18223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; sv-se; SonyEricssonLT22i Build/6.0.B.3.115) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2187374 = {S32900, 151, { &P18223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-at; SonyEricssonLT22i Build/6.0.B.3.187) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2198337 = {S32901, 153, { &P18223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; SonyEricssonLT22i-o Build/6.0.B.3.162) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2223389 = {S32902, 139, { &P18223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-mx; LT22i Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2349124 = {S32903, 139, { &P18223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; LT22i Build/6.0.B.3.187) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2413871 = {S32904, 132, { &P18223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT22i Build/6.0.B.3.184) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2686979 = {S32905, 139, { &P18223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; LT22i Build/6.0.B.1.576) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2043016 = {S32906, 76, { &P17595,&P17297,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:15.0) Gecko/X Firefox/15.0.1*/
const Device UA2228157 = {S32907, 91, { &P17595,&P17297,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:15.0) Gecko/X Firefox/15.0.1 Firefox/15.0.1*/
const Device UA16537 = {S32908, 140, { &P13889,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-P505/V10b Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA16538 = {S32909, 140, { &P13889,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-P505/V10e Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA16536 = {S32910, 138, { &P13889,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-P505/V09c Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA16541 = {S32911, 140, { &P13889,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-P505/V11e Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA56855 = {S32912, 139, { &P13889,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; LG-P505/V11e Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA56864 = {S32913, 141, { &P13889,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P505/V11h Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA56865 = {S32914, 136, { &P13889,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P505 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA56866 = {S32915, 141, { &P13889,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P505/V10i Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA56867 = {S32916, 135, { &P13889,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-P505 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60395 = {S32917, 137, { &P13889,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; LG-P505R Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76835 = {S32918, 65, { &P13889,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/0.9.1 (Linux; U; Android 2.2.1; en_US; LG-P505 Build/FRG83)*/
const Device UA440349 = {S32919, 138, { &P13889,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; LG-P505CH Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA656199 = {S32920, 140, { &P13889,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-P505/V11e Build/FRG85) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA668359 = {S32921, 60, { &P13889,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P505 Build/FRG83G)*/
const Device UA694061 = {S32922, 59, { &P13889,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; LG-P505 Build/FRG83)*/
const Device UA1941177 = {S32923, 61, { &P13889,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P505R Build/FRG83G)*/
const Device UA2021989 = {S32924, 136, { &P13889,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; LG-P505R Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022864 = {S32925, 62, { &P13889,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P505CH Build/FRG83G)*/
const Device UA911700 = {S32926, 140, { &P15364,&P17285,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA912388 = {S32927, 101, { &P15364,&P17285,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA912390 = {S32928, 108, { &P15364,&P17285,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA912650 = {S32929, 138, { &P15364,&P17285,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe) (via translate.google.com)*/
const Device UA2672459 = {S32930, 136, { &P18471,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT905 Build/9.8.2Q_SMUIRD-7) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676038 = {S32931, 139, { &P18471,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XT907 Build/9.8.1Q_27-2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676578 = {S32932, 132, { &P18471,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XT907 Build/9.8.1Q_27-2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676849 = {S32933, 143, { &P18471,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT905 Build/9.8.2Q_SMUIRD-7) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2937452 = {S32934, 137, { &P18471,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-78) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2946180 = {S32935, 130, { &P18471,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-78) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA158693 = {S32936, 62, { &P15364,&P17285,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; en) Presto/2.7.62 Version/11.00*/
const Device UA172354 = {S32937, 62, { &P15364,&P17285,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; pl) Presto/2.7.62 Version/11.01*/
const Device UA304520 = {S32938, 85, { &P15364,&P17285,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.7.62 Version/11.01*/
const Device UA305127 = {S32939, 85, { &P15364,&P17285,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.7.62 Version/11.01*/
const Device UA502316 = {S32940, 66, { &P15364,&P17285,&P17217,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; en) Opera 11.00*/
const Device UA516191 = {S32941, 85, { &P15364,&P17285,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.7.62 Version/11.01*/
const Device UA558192 = {S32942, 81, { &P15364,&P17285,&P17217,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; U; ja; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 11.00*/
const Device UA2129077 = {S32943, 85, { &P15364,&P17285,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.7.62 Version/11.00*/
const Device UA2359567 = {S32944, 140, { &P16292,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Galaxy Nexus Build/JOP40C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2368289 = {S32945, 142, { &P16292,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2423288 = {S32946, 136, { &P16292,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2515540 = {S32947, 141, { &P16292,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2986551 = {S32948, 135, { &P16292,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3008329 = {S32949, 168, { &P16292,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG Galaxy Nexus Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA459195 = {S32950, 162, { &P14114,&P17296,&P18161,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us; Silk/1.0.13.328_X) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16 Silk-Accelerated=true*/
const Device UA628749 = {S32951, 158, { &P14114,&P17296,&P18161,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us; Silk/1.1.0-84) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16 Silk-Accelerated=true*/
const Device UA29936 = {S32952, 143, { &P12280,&P17134,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A400 Safari/528.16*/
const Device UA88631 = {S32953, 141, { &P12280,&P17134,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2406366 = {S32954, 145, { &P12280,&P17134,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293 Safari/6531.22.7*/
const Device UA2659528 = {S32955, 139, { &P12280,&P17134,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 30 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA616632 = {S32956, 147, { &P13064,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SAMSUNG-SGH-I727 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA685256 = {S32957, 145, { &P13064,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; SAMSUNG-SGH-I727 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA762805 = {S32958, 69, { &P13064,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SAMSUNG-SGH-I727 Build/IML74K)*/
const Device UA874092 = {S32959, 147, { &P13064,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA879053 = {S32960, 69, { &P13064,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SAMSUNG-SGH-I727 Build/IMM76D)*/
const Device UA889075 = {S32961, 145, { &P13064,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA896574 = {S32962, 139, { &P13064,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I727 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030727 = {S32963, 155, { &P13064,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SAMSUNG-SGH-I727 Build/Stock Revamped) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031427 = {S32964, 153, { &P13064,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2033980 = {S32965, 147, { &P13064,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I727 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3046052 = {S32966, 125, { &P13064,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.*/
const Device UA3093726 = {S32967, 178, { &P13064,&P18066,&P18062,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.0.4;dev=SGH-I727;devm=SAMSUNG-SGH-I727;man=samsung;brw=Android;shwd=480x800;shwa=480x763;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA6560 = {S32968, 152, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; HTC Wildfire 1.29.163.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA6603 = {S32969, 152, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; HTC Wildfire 1.14.162.3 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA6629 = {S32970, 152, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; de-at; HTC Wildfire 2.31.163.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA6921 = {S32971, 148, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - HTC_Wildfire_Mozilla/5.0 (Linux; U; Android 2.2; it-it;HTC Buzz Built/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA9314 = {S32972, 141, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32320 = {S32973, 152, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; HTC Wildfire 2.24.164.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32321 = {S32974, 152, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Wildfire 2.24.161.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32323 = {S32975, 152, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; nl-nl; HTC Wildfire 2.24.172.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32331 = {S32976, 152, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Wildfire 1.14.161.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32337 = {S32977, 152, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; HTC Wildfire 2.24.163.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32341 = {S32978, 152, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; HTC Wildfire 1.36.164.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32349 = {S32979, 152, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; HTC Wildfire 2.24.162.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32353 = {S32980, 152, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; HTC Wildfire 1.25.162.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32354 = {S32981, 152, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Wildfire 1.26.161.3 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32371 = {S32982, 152, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; HTC Wildfire 1.13.163.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32376 = {S32983, 152, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Wildfire 2.24.161.2 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32446 = {S32984, 151, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; HTC_Wildfire-orange-LS Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32449 = {S32985, 152, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; nl-nl; HTC Wildfire 1.14.172.2 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA37777 = {S32986, 139, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; hr-; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA37785 = {S32987, 152, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; HTC Wildfire 1.17.164.3 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA44269 = {S32988, 140, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Wildfire Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51175 = {S32989, 154, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1 Orange Froyo; es-es; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA165295 = {S32990, 65, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; HTC Wildfire Build/FRG83D)*/
const Device UA684430 = {S32991, 129, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; HTC Wildfire Build/FRG83D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA899227 = {S32992, 152, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Wildfire 2.24.161.4 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1864649 = {S32993, 138, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Wildfire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018982 = {S32994, 153, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1 Community 2; es-es; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019827 = {S32995, 126, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Wildfire Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2025366 = {S32996, 134, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; HTC Wildfire Build/FRG83D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2028516 = {S32997, 152, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; HTC Wildfire 2.24.163.3 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034276 = {S32998, 152, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; HTC Wildfire 2.24.164.3 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2375695 = {S32999, 152, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; HTC Wildfire 2.24.162.3 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2440913 = {S33000, 131, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Wildfire Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2847191 = {S33001, 152, { &P13815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; nl-nl; HTC Wildfire 2.24.172.5 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA895053 = {S33002, 142, { &P18513,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1953941 = {S33003, 143, { &P18513,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-S6802B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030613 = {S33004, 66, { &P18513,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S6802 Build/GINGERBREAD)*/
const Device UA2205732 = {S33005, 140, { &P18513,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2223310 = {S33006, 142, { &P18513,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6352 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2338392 = {S33007, 130, { &P18513,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6802 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2644306 = {S33008, 67, { &P18513,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S6802B Build/GINGERBREAD)*/
const Device UA2822894 = {S33009, 135, { &P18513,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6352 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2830961 = {S33010, 66, { &P18513,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S6352 Build/GINGERBREAD)*/
const Device UA204767 = {S33011, 63, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.9.168 Version/11.50*/
const Device UA205949 = {S33012, 63, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; en) Presto/2.9.168 Version/11.50*/
const Device UA210042 = {S33013, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.9.168 Version/11.50*/
const Device UA210058 = {S33014, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.9.168 Version/11.50*/
const Device UA210194 = {S33015, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.9.168 Version/11.50*/
const Device UA211695 = {S33016, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.9.168 Version/11.50*/
const Device UA216376 = {S33017, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.9.168 Version/11.50*/
const Device UA219314 = {S33018, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.9.168 Version/11.50*/
const Device UA219706 = {S33019, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.9.168 Version/11.50*/
const Device UA227990 = {S33020, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.9.168 Version/11.50*/
const Device UA230410 = {S33021, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.9.168 Version/11.50*/
const Device UA241986 = {S33022, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.9.168 Version/11.50*/
const Device UA324754 = {S33023, 66, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; en) Opera 11.50*/
const Device UA328761 = {S33024, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.9.168 Version/11.50*/
const Device UA329905 = {S33025, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.9.168 Version/11.50*/
const Device UA332548 = {S33026, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.9.168 Version/11.50*/
const Device UA334193 = {S33027, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.9.168 Version/11.50*/
const Device UA335313 = {S33028, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4661); ru) Presto/2.9.168 Version/11.50*/
const Device UA394600 = {S33029, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.9.168 Version/11.50*/
const Device UA395011 = {S33030, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.9.168 Version/11.50*/
const Device UA452064 = {S33031, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); en) Presto/2.9.168 Version/11.50*/
const Device UA556922 = {S33032, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02746); ru) Presto/2.9.168 Version/11.50*/
const Device UA558838 = {S33033, 72, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; ru; rv:2.0) Gecko/X Firefox/4.0 Opera 11.50*/
const Device UA585597 = {S33034, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.9.168 Version/11.50*/
const Device UA639981 = {S33035, 103, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; es-ES) Presto/2.9.168 Version/11.50,gzip(gfe) (via translate.google.com)*/
const Device UA647240 = {S33036, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.9.168 Version/11.50*/
const Device UA647916 = {S33037, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.9.168 Version/11.50*/
const Device UA648699 = {S33038, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.9.168 Version/11.50*/
const Device UA667531 = {S33039, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.9.168 Version/11.50*/
const Device UA698788 = {S33040, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.9.168 Version/11.50*/
const Device UA704872 = {S33041, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.9.168 Version/11.50*/
const Device UA730947 = {S33042, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.9.168 Version/11.50*/
const Device UA734136 = {S33043, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.9.168 Version/11.50*/
const Device UA774973 = {S33044, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.9.168 Version/11.50*/
const Device UA776820 = {S33045, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.9.168 Version/11.50*/
const Device UA782598 = {S33046, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.9.168 Version/11.50*/
const Device UA805591 = {S33047, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.9.168 Version/11.50*/
const Device UA880698 = {S33048, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03791); ru) Presto/2.9.168 Version/11.50*/
const Device UA897927 = {S33049, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.9.168 Version/11.50*/
const Device UA1890840 = {S33050, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03780); ru) Presto/2.9.168 Version/11.50*/
const Device UA1937422 = {S33051, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4930); ru) Presto/2.9.168 Version/11.50*/
const Device UA2145583 = {S33052, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4841); ru) Presto/2.9.168 Version/11.50*/
const Device UA2154536 = {S33053, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.9.168 Version/11.50*/
const Device UA2156421 = {S33054, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.9.168 Version/11.50*/
const Device UA2160655 = {S33055, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.9.168 Version/11.50*/
const Device UA2167736 = {S33056, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.9.168 Version/11.50*/
const Device UA2262402 = {S33057, 86, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.9.168 Version/11.50*/
const Device UA2262741 = {S33058, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.9.168 Version/11.50*/
const Device UA2351756 = {S33059, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.9.168 Version/11.50*/
const Device UA2352638 = {S33060, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.9.168 Version/11.50*/
const Device UA2421319 = {S33061, 73, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.9.168 Version/11.50,gzip(gfe)*/
const Device UA2457169 = {S33062, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.9.168 Version/11.50*/
const Device UA2481305 = {S33063, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.9.168 Version/11.50*/
const Device UA2485038 = {S33064, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.9.168 Version/11.50*/
const Device UA2654153 = {S33065, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.9.168 Version/11.50*/
const Device UA2667246 = {S33066, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.9.168 Version/11.50*/
const Device UA2796343 = {S33067, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6578); ru) Presto/2.9.168 Version/11.50*/
const Device UA2834648 = {S33068, 85, { &P15364,&P5690,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5745); ru) Presto/2.9.168 Version/11.50*/
const Device UA92506 = {S33069, 62, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.5.24 Version/10.52*/
const Device UA96838 = {S33070, 62, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.5.22 Version/10.50*/
const Device UA106300 = {S33071, 62, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.5.24 Version/10.54*/
const Device UA134846 = {S33072, 62, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.5.22 Version/10.51*/
const Device UA139715 = {S33073, 62, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; de) Presto/2.5.24 Version/10.53*/
const Device UA299310 = {S33074, 85, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.5.24 Version/10.53*/
const Device UA400121 = {S33075, 85, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.5.24 Version/10.53*/
const Device UA414744 = {S33076, 99, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.5.24 Version/10.54,gzip(gfe) (via translate.google.com)*/
const Device UA416160 = {S33077, 85, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.5.24 Version/10.54*/
const Device UA434391 = {S33078, 85, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.5.22 Version/10.50*/
const Device UA459498 = {S33079, 62, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; en) Presto/2.5.24 Version/10.53*/
const Device UA460292 = {S33080, 85, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.5.24 Version/10.53*/
const Device UA477326 = {S33081, 85, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.5.24 Version/10.53*/
const Device UA485824 = {S33082, 62, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; en) Presto/2.5.22 Version/10.50*/
const Device UA535037 = {S33083, 85, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.5.22 Version/10.51*/
const Device UA538787 = {S33084, 62, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; ru) Presto/2.5.22 Version/10.51*/
const Device UA553968 = {S33085, 84, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.5.24 Version/10.53*/
const Device UA1863372 = {S33086, 85, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.5.22 Version/10.51*/
const Device UA1866166 = {S33087, 85, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.5.24 Version/10.53*/
const Device UA1888679 = {S33088, 84, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.5.24 Version/10.53*/
const Device UA2150664 = {S33089, 84, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.5.24 Version/10.53*/
const Device UA2215411 = {S33090, 84, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.5.24 Version/10.53*/
const Device UA2262411 = {S33091, 85, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.5.24 Version/10.53*/
const Device UA2277099 = {S33092, 85, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.5.22 Version/10.51*/
const Device UA2281561 = {S33093, 85, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.5.22 Version/10.51*/
const Device UA2297296 = {S33094, 84, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.5.24 Version/10.53*/
const Device UA2359954 = {S33095, 84, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.5.22 Version/10.51*/
const Device UA2481020 = {S33096, 72, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.5.24 Version/10.54,gzip(gfe)*/
const Device UA2518472 = {S33097, 84, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.5.24 Version/10.53*/
const Device UA2649513 = {S33098, 84, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.5.24 Version/10.53*/
const Device UA2685481 = {S33099, 85, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.5.24 Version/10.53*/
const Device UA2824260 = {S33100, 84, { &P15364,&P5690,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.5.24 Version/10.53*/
const Device UA624918 = {S33101, 148, { &P19891,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SAMSUNG-SGH-I717 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625007 = {S33102, 150, { &P19891,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I717 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633725 = {S33103, 74, { &P19891,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SAMSUNG-SGH-I717 Build/GINGERBREAD)*/
const Device UA1992073 = {S33104, 155, { &P19891,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I717 Build/CollectiveCustom) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027300 = {S33105, 143, { &P19891,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SAMSUNG-SGH-I717 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA47291 = {S33106, 148, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3Build/5.5.1_84_D3G-20_TA-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47298 = {S33107, 144, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-52) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47299 = {S33108, 148, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-20_TA-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA47302 = {S33109, 144, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-19) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47307 = {S33110, 149, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-54_MR-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47309 = {S33111, 144, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-54) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47311 = {S33112, 148, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-20_TA-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47325 = {S33113, 144, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60296 = {S33114, 144, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID3 Build/5.5.1_53_D3G-11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60297 = {S33115, 137, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_80) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60303 = {S33116, 150, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-20_TA-99) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60306 = {S33117, 149, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-20_TA-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70562 = {S33118, 143, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72232 = {S33119, 143, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA73594 = {S33120, 143, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;DROID3 Build/5.5.1_84_D3G-55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74124 = {S33121, 143, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3Build/5.5.1_84_D3G-55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74136 = {S33122, 149, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-55_MR-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA75958 = {S33123, 143, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76851 = {S33124, 144, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID3 Build/5.5.1_84_D3G-71) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78594 = {S33125, 144, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID3 Build/5.5.1_84_D3G-74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA451226 = {S33126, 147, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 4G Build/5.5.1_84_D3G-55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA585059 = {S33127, 144, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID3 Build/5.5.1_84_D3G-65) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA588530 = {S33128, 149, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-66_M2-6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622447 = {S33129, 150, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID3 Build/5.5.1_84_D3G-65) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30*/
const Device UA625747 = {S33130, 150, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA651300 = {S33131, 144, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-64) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA680767 = {S33132, 74, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; DROID3 Build/5.5.1_84_D3G-66_M2-10)*/
const Device UA791405 = {S33133, 149, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-66_M2-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1957053 = {S33134, 149, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA1981857 = {S33135, 149, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1983264 = {S33136, 149, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;DROID3 Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1985538 = {S33137, 149, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1987595 = {S33138, 153, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 4G Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1992919 = {S33139, 149, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022625 = {S33140, 133, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2023639 = {S33141, 145, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK DROID3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031266 = {S33142, 141, { &P15911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; XT862 Build/SLNCT-57.1.60) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA91032 = {S33143, 117, { &P17595,&P17296,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA91111 = {S33144, 117, { &P17595,&P17296,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA91112 = {S33145, 117, { &P17595,&P17296,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA251867 = {S33146, 117, { &P17595,&P17296,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2*/
const Device UA251971 = {S33147, 117, { &P17595,&P17296,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2*/
const Device UA255821 = {S33148, 117, { &P17595,&P17296,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA345507 = {S33149, 117, { &P17595,&P17296,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2*/
const Device UA345509 = {S33150, 117, { &P17595,&P17296,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2*/
const Device UA629108 = {S33151, 117, { &P17595,&P17296,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA2410142 = {S33152, 119, { &P17595,&P17296,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10\_6\_8) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA2674696 = {S33153, 129, { &P17595,&P17296,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/15.0.648.205 Safari/534.16*/
const Device UA2675007 = {S33154, 129, { &P17595,&P17296,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/15.0.648.205 Safari/534.16*/
const Device UA2715740 = {S33155, 129, { &P17595,&P17296,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/15.0.648.205 Safari/534.16*/
const Device UA64151 = {S33156, 77, { &P12141,&P6217,&P3418,&P18092 }};	/*Useragent - BlackBerry8900/4.6.1.231 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100*/
const Device UA32851 = {S33157, 126, { &P16598,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; A500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA73345 = {S33158, 133, { &P16598,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; A500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA253484 = {S33159, 128, { &P16598,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; A500 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA331832 = {S33160, 55, { &P16598,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; A500 Build/HTJ85B)*/
const Device UA369249 = {S33161, 84, { &P16598,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 3.2.1; en_US; A500 Build/Acer_A500_7.014.01_COM_GEN2)*/
const Device UA369896 = {S33162, 126, { &P16598,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; cs-; A500 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA647180 = {S33163, 57, { &P16598,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; A500 Build/HTK55D)*/
const Device UA838541 = {S33164, 135, { &P16598,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; A500 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA1938295 = {S33165, 128, { &P16598,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; A500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2020926 = {S33166, 127, { &P16598,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; A500 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA783217 = {S33167, 144, { &P18496,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6410LVW 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA827851 = {S33168, 141, { &P18496,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6410LVW Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1950871 = {S33169, 63, { &P18496,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; ADR6410LVW Build/IML74K)*/
const Device UA2028638 = {S33170, 144, { &P18496,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6410LVW 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2187998 = {S33171, 141, { &P18496,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6410LRA Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2223008 = {S33172, 141, { &P18496,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6410LVW Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2237418 = {S33173, 63, { &P18496,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ADR6410LVW Build/IMM76D)*/
const Device UA2484015 = {S33174, 140, { &P18496,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-pr; ADR6410OM Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2599139 = {S33175, 141, { &P18496,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6410LRA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA475195 = {S33176, 104, { &P15364,&P18110,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.13) Gecko/X Firefox/2.0 (Debian-3.0.12-1~backports50+1)*/
const Device UA540018 = {S33177, 65, { &P15364,&P18110,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11;U;Linux i686;en-US;rv:1.8.1) Gecko/X Firefox/2.0*/
const Device UA541243 = {S33178, 101, { &P15364,&P18110,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686 (x86_64); pl-PL; rv:1.8.1) Gecko/X Firefox/2.0-1.fc4.remi Firefox/2.0*/
const Device UA672267 = {S33179, 69, { &P15364,&P18110,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.11) Firefox/2.0.0.11*/
const Device UA692430 = {S33180, 104, { &P15364,&P18110,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.15) Gecko/X Iceweasel/2.0.0.15 (Debian-2.0.0.15-0etch1)*/
const Device UA825302 = {S33181, 94, { &P15364,&P18110,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; ja; rv:1.8.1.11) Gecko/X Firefox/2.0.0.11 (Debian-2.0.0.11-1)*/
const Device UA896097 = {S33182, 77, { &P15364,&P18110,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Windows NT i686; fr; rv:1.9.0.1) Gecko/X Firefox/2.0.0.8*/
const Device UA915485 = {S33183, 73, { &P15364,&P18110,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1b2) Gecko/X BonEcho/2.0b2*/
const Device UA1893648 = {S33184, 97, { &P15364,&P18110,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; nl; rv:1.8.1b2) Gecko/X BonEcho/2.0b2 (Debian-1.99+2.0b2+dfsg-1)*/
const Device UA2291685 = {S33185, 133, { &P15364,&P18110,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; pl; rv:1.8.1.1) Gecko/X Mozilla/5.0 (X11; U; Linux i686; fr; rv:1.8.1) Gecko/X Firefox/2.0b2*/
const Device UA2387369 = {S33186, 74, { &P17595,&P17297,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:17.0) Gecko/X Firefox/17.0*/
const Device UA2973891 = {S33187, 127, { &P18385,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus 7 Build/JWR66V) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3029694 = {S33188, 134, { &P18385,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Google Nexus 7 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA448372 = {S33189, 116, { &P15364,&P7133,&P3409,&P18092 }};	/*Useragent - index.boo.ly/1.0 (windows; u; windows nt 6.1; en-gb; rv:1.9.2.15) gecko/X firefox/3.6.15 gtb7.1 (.net clr 3.5.30729)*/
const Device UA729752 = {S33190, 84, { &P15364,&P7133,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9a8) Gecko/X GranParadiso/3.0a8*/
const Device UA808347 = {S33191, 104, { &P15364,&P7133,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-GB; rv:1.9.0.10) Gecko/X Firefox/3.0.10 (.NET CLR 3.5.30729)*/
const Device UA2132867 = {S33192, 83, { &P15364,&P7133,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.1.18) Gecko/X FireFox/3.6.13*/
const Device UA7509 = {S33193, 172, { &P13324,&P5055,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.2NokiaN8-00/011.012; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.7.2 3gpp-gba*/
const Device UA24683 = {S33194, 174, { &P13324,&P5055,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaN8-00/014.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.8.10 3gpp-gba*/
const Device UA33463 = {S33195, 187, { &P13324,&P5055,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaN8-00/022.014; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.37 Mobile Safari/533.4 3gpp-gba*/
const Device UA89926 = {S33196, 172, { &P13324,&P5055,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.2 Nokia N8-00/010.020; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6 3gpp-gba*/
const Device UA166238 = {S33197, 10, { &P13324,&P5055,&P3804,&P18092 }};	/*Useragent - NokiaN8-00*/
const Device UA92754 = {S33198, 27, { &P15364,&P6704,&P17470,&P18092 }};	/*Useragent - Mozilla/4.5 [en] (Win98; I)*/
const Device UA98999 = {S33199, 50, { &P15364,&P6704,&P17470,&P18092 }};	/*Useragent - Mozilla/4.5 (compatible; HTTrack 3.0x; Windows 98)*/
const Device UA111232 = {S33200, 25, { &P15364,&P6704,&P17470,&P18092 }};	/*Useragent - Mozilla/4.72 [en] (Win98)*/
const Device UA117500 = {S33201, 28, { &P15364,&P6704,&P17470,&P18092 }};	/*Useragent - Mozilla/4.73 [en] (Win98; U)*/
const Device UA120456 = {S33202, 25, { &P15364,&P6704,&P17470,&P18092 }};	/*Useragent - Mozilla/4.76 [en] (Win98)*/
const Device UA121206 = {S33203, 28, { &P15364,&P6704,&P17470,&P18092 }};	/*Useragent - Mozilla/4.76 [en] (Win98; U)*/
const Device UA121444 = {S33204, 28, { &P15364,&P6704,&P17470,&P18092 }};	/*Useragent - Mozilla/4.75 [en] (Win98; U)*/
const Device UA131221 = {S33205, 39, { &P15364,&P6704,&P17470,&P18092 }};	/*Useragent - Mozilla/3.0 [en] (compatible; Win98; U)*/
const Device UA138926 = {S33206, 28, { &P15364,&P6704,&P17470,&P18092 }};	/*Useragent - Mozilla/4.61 [en] (Win98; I)*/
const Device UA141962 = {S33207, 63, { &P15364,&P6704,&P17470,&P18092 }};	/*Useragent - Mozilla/4.7 (compatible; OffByOne; Windows 98) Webster Pro V3.2*/
const Device UA151970 = {S33208, 47, { &P15364,&P6704,&P17470,&P18092 }};	/*Useragent - Mozilla/4.5 [en]C-CCK-MCD {RuralNet} (Win98; I)*/
const Device UA273674 = {S33209, 27, { &P15364,&P6704,&P17470,&P18092 }};	/*Useragent - Mozilla/4.7 [en] (Win98; I)*/
const Device UA459873 = {S33210, 39, { &P15364,&P6704,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0(Windows;N;Win98;m18)Gecko/X*/
const Device UA878926 = {S33211, 69, { &P15364,&P6704,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.5) Gecko/X K-Meleon/0.8.2*/
const Device UA904540 = {S33212, 56, { &P15364,&P6704,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows; U; Win98; en-US; rv:1.7.2) Gecko/X*/
const Device UA1840080 = {S33213, 75, { &P15364,&P6704,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win 9x 4.90; en-US; rv:1.7.5) Gecko/X K-Meleon/0.9*/
const Device UA9590 = {S33214, 144, { &P11950,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; X10i Build/2.0.A.0.504) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA9601 = {S33215, 147, { &P11950,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; X10i Build/2.0.2.A.0.24) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA9644 = {S33216, 183, { &P11950,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-Eternity|build003 Sempiternal@aioboards.com; en-gb; X10i Build/2.0.2.A.0.24) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA9670 = {S33217, 158, { &P11950,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; SonyEricssonX10i Build/2.0.A.0.504) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA19154 = {S33218, 159, { &P11950,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; SonyEricssonX10iv Build/2.1.A.0.435) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA42085 = {S33219, 93, { &P11950,&P5477,&P4861,&P18092 }};	/*Useragent - Android (Linux; U; Android 2.1-update1; fa-ir; X10i) AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA43766 = {S33220, 146, { &P11950,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; X10i Build/2.0.A.0.504) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA44146 = {S33221, 150, { &P11950,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-rooted-zdz; en-us; X10i Build/2.0.2.A.0.24) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA57980 = {S33222, 147, { &P11950,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; X10i Build/2.1.1.A.0.28) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA58048 = {S33223, 99, { &P11950,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.1-update1;Zh_cn;X10i;480*854;)AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA58076 = {S33224, 144, { &P11950,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; X10i Build/2.1.B.0.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA63076 = {S33225, 146, { &P11950,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; X10i Build/2.1.A.0.435) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA174261 = {S33226, 68, { &P11950,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; X10i Build/2.1.A.0.435)*/
const Device UA1874528 = {S33227, 69, { &P11950,&P5477,&P4861,&P18092 }};	/*Useragent - PVPlayer X10i CORE/7.507.1.1 OpenCORE/2.05 (Linux;Android2.1-update1)*/
const Device UA2020319 = {S33228, 132, { &P11950,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fa-ir; X10i Build/2.1.A.0.435) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2021105 = {S33229, 146, { &P11950,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; X10i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2024758 = {S33230, 132, { &P11950,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; X10i Build/2.0.A.0.504) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2026777 = {S33231, 147, { &P11950,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; X10i Build/2.1.1.A.0.16) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2032576 = {S33232, 137, { &P11950,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; X10i Build/2.1.A.0.435) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA41598 = {S33233, 163, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LG-P990 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA342039 = {S33234, 139, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Optimus 2X Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368630 = {S33235, 160, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en; LG-P990 Build/GRJ23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA368965 = {S33236, 59, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P990 Build/GRJ23)*/
const Device UA369184 = {S33237, 165, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; LG-P990hN Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA370770 = {S33238, 136, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; Optimus 2X Build/GRJ23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA395738 = {S33239, 139, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Optimus 2X Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA624645 = {S33240, 168, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; LG-P990/V20e Build/GRJ23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA654382 = {S33241, 163, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; LG-P990 Build/GRJ23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA662720 = {S33242, 62, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; Optimus 2X Build/GRJ22)*/
const Device UA726689 = {S33243, 168, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; LG-P990/V20d Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA750155 = {S33244, 139, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; Optimus 2X Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA771806 = {S33245, 160, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Optimus 2X Build/FRG83G; LG-P990 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA795986 = {S33246, 164, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; LG-P990h Build/GRJ23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA849011 = {S33247, 137, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Optimus 2X Build/WIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA849619 = {S33248, 164, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-sa; LG-P990h Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA1944739 = {S33249, 168, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-ch; LG-P990/V20c Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2022466 = {S33250, 123, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LG-P990 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022504 = {S33251, 127, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Optimus 2X Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2028925 = {S33252, 61, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P990hN Build/GRJ22)*/
const Device UA2031778 = {S33253, 128, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LG-P990 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2033704 = {S33254, 132, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Optimus 2X Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2473589 = {S33255, 162, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en; LG-P990hN Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2502290 = {S33256, 168, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-ch; LG-P990/V20f Build/GRJ23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2645548 = {S33257, 165, { &P14017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es; LG-P990/V20e Build/GRJ23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA326845 = {S33258, 49, { &P15364,&P18110,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) Gecko Firefox/5.0*/
const Device UA793369 = {S33259, 81, { &P15364,&P18110,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 on x86_64; rv:5.0) Gecko/X Firefox/5.0 Iceweasel/5.0*/
const Device UA24584 = {S33260, 174, { &P14983,&P5055,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaC7-00/014.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.8.10 3gpp-gba*/
const Device UA48257 = {S33261, 188, { &P14983,&P5055,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaC7-00s/022.014; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.37 Mobile Safari/533.4 3gpp-gba*/
const Device UA50045 = {S33262, 187, { &P14983,&P5055,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaC7-00/022.014; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.37 Mobile Safari/533.4 3gpp-gba*/
const Device UA622916 = {S33263, 189, { &P14983,&P5055,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaC7-00.1/024.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.37 Mobile Safari/533.4 3gpp-gba*/
const Device UA712832 = {S33264, 188, { &P14983,&P5055,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.2NokiaC7-00.1/024.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.37 Mobile Safari/533.4 3gpp-gba*/
const Device UA49787 = {S33265, 143, { &P15809,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC_Bliss_S510b Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58614 = {S33266, 149, { &P15809,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC_S510b V1.36.841.3 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70149 = {S33267, 143, { &P15809,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_Rhyme_S510b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70333 = {S33268, 149, { &P15809,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC_S510b V1.36.841.5 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70545 = {S33269, 143, { &P15809,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_Bliss_S510b Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79088 = {S33270, 143, { &P15809,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_Bliss_S510b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA422367 = {S33271, 149, { &P15809,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC_S510b V2.14.841.4 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA569703 = {S33272, 143, { &P15809,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Rhyme S510b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA678764 = {S33273, 143, { &P15809,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Bliss_S510b Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA720932 = {S33274, 67, { &P15809,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Rhyme S510b Build/GRJ90)*/
const Device UA1912873 = {S33275, 131, { &P15809,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Rhyme S510b Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1948187 = {S33276, 149, { &P15809,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC_S510b V1.36.841.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1965680 = {S33277, 137, { &P15809,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC_S510b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023625 = {S33278, 137, { &P15809,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC S510b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023825 = {S33279, 149, { &P15809,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC_S510b V1.36.841.5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024969 = {S33280, 125, { &P15809,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC S510b Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2030255 = {S33281, 136, { &P15809,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Rhyme S510b Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2375441 = {S33282, 130, { &P15809,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC S510b Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2933451 = {S33283, 141, { &P15809,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-; HTC_Rhyme_S510b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2390065 = {S33284, 65, { &P15364,&P17285,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:18.0) Gecko/X Firefox/18.0*/
const Device UA2490671 = {S33285, 70, { &P15364,&P17285,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:18.0) Gecko/X Firefox/18.0*/
const Device UA41099 = {S33286, 194, { &P14033,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaC5-03.2/20.1.024; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.25 Mobile Safari/533.4 3gpp-gba*/
const Device UA41101 = {S33287, 180, { &P14033,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaC5-03.2/12.1.023; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.9 3gpp-gba*/
const Device UA64170 = {S33288, 178, { &P14033,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaC5-03/11.0.024; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.9 3gpp-gba*/
const Device UA623709 = {S33289, 192, { &P14033,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaC5-03/22.0.007; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 Mobile Safari/533.4 3gpp-gba*/
const Device UA93507 = {S33290, 41, { &P15364,&P17350,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Win32)*/
const Device UA113414 = {S33291, 54, { &P15364,&P17350,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Win64; Trident/4.0)*/
const Device UA221335 = {S33292, 58, { &P15364,&P17350,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; MSIE 8.0; WIndows NT 9.0; en-US))*/
const Device UA687601 = {S33293, 43, { &P15364,&P17350,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows)*/
const Device UA795955 = {S33294, 46, { &P15364,&P17350,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT)*/
const Device UA2509458 = {S33295, 52, { &P15364,&P17350,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; 68K; Trident/4.0)*/
const Device UA2658875 = {S33296, 64, { &P15364,&P17350,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; 68K; Win64; x64; Trident/5.0)*/
const Device UA2660686 = {S33297, 57, { &P15364,&P17350,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; 68K; SV1; Trident/5.0)*/
const Device UA2704563 = {S33298, 59, { &P15364,&P17350,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; PPC; WOW64; Trident/1.0)*/
const Device UA2791762 = {S33299, 62, { &P15364,&P17350,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; PPC; .NET CLR; Trident/2.0)*/
const Device UA2792811 = {S33300, 59, { &P15364,&P17350,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; 68K; WOW64; Trident/5.0)*/
const Device UA2792899 = {S33301, 65, { &P15364,&P17350,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; PPC; Win64; IA64; Trident/2.0)*/
const Device UA33320 = {S33302, 191, { &P12193,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaC5-00/071.005; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.26 Mobile Safari/533.4 3gpp-gba*/
const Device UA36815 = {S33303, 193, { &P12193,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaC5-00.2/071.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.26 Mobile Safari/533.4 3gpp-gba*/
const Device UA41228 = {S33304, 193, { &P12193,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaC5-00.1/071.005; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.26 Mobile Safari/533.4 3gpp-gba*/
const Device UA51106 = {S33305, 170, { &P12193,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaC5-00.1/032.010; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.5.2*/
const Device UA64548 = {S33306, 168, { &P12193,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaC5-00/032.010; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.5.2*/
const Device UA72649 = {S33307, 177, { &P12193,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaC5-00i/071.003 (SymbianOS/9.3; U; Series60/3.2 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/533.4 (KHTML, like Gecko) Version/3.0 Safari/533.4 3gpp-gba*/
const Device UA648333 = {S33308, 193, { &P12193,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaC5-00.3/071.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.26 Mobile Safari/533.4 3gpp-gba*/
const Device UA2367932 = {S33309, 73, { &P15364,&P17285,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA2487933 = {S33310, 65, { &P15364,&P17285,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:20.0) Gecko/X Firefox/20.0*/
const Device UA2756610 = {S33311, 72, { &P15364,&P17285,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.1*/
const Device UA2770013 = {S33312, 88, { &P15364,&P17285,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.1-x64*/
const Device UA2870012 = {S33313, 141, { &P16292,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JWR66N) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2969480 = {S33314, 141, { &P16292,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3023346 = {S33315, 141, { &P16292,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3030461 = {S33316, 141, { &P16292,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JWR66X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3034335 = {S33317, 141, { &P16292,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3036443 = {S33318, 63, { &P16292,&P21637,&P23997,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.3; Galaxy Nexus Build/JWR66Y)*/
const Device UA3064910 = {S33319, 141, { &P16292,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108576 = {S33320, 139, { &P16292,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-; Galaxy Nexus Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3114342 = {S33321, 141, { &P16292,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3115731 = {S33322, 141, { &P16292,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA200253 = {S33323, 66, { &P15364,&P18110,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux x86_64; U; en) Presto/2.9.168 Version/11.50*/
const Device UA660734 = {S33324, 61, { &P15364,&P18110,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; U; en) Presto/2.9.201 Version/11.50*/
const Device UA743784 = {S33325, 73, { &P15364,&P18110,&P17366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.50*/
const Device UA2178393 = {S33326, 62, { &P20076,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT26w Build/6.1.A.2.45)*/
const Device UA2178440 = {S33327, 152, { &P20076,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; SonyEricssonLT26w Build/6.1.A.2.45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2228732 = {S33328, 140, { &P20076,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT26w Build/6.1.A.2.45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257836 = {S33329, 140, { &P20076,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT26w Build/6.1.A.2.50) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2478800 = {S33330, 131, { &P20076,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26w Build/6.1.A.2.45) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2500220 = {S33331, 140, { &P20076,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT26w Build/6.1.A.1.58) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2505482 = {S33332, 140, { &P20076,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT26w Build/6.1.A.2.55) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028804 = {S33333, 138, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; IdeaTab A2107A-H Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2257311 = {S33334, 140, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; IdeaTab A2107A-H Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2356515 = {S33335, 140, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; IdeaTab A2107A-F Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2389495 = {S33336, 155, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb, SmartTabII7 Build/A2107A_A404_010_002_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2404391 = {S33337, 155, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ro-ro, SmartTabII7 Build/A2107A_A404_004_043_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2414104 = {S33338, 69, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; IdeaTab A2107A-H Build/IML74K)*/
const Device UA2423191 = {S33339, 155, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ie, SmartTabII7 Build/A2107A_A404_004_043_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2499324 = {S33340, 132, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A2107A-H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2504758 = {S33341, 69, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; IdeaTab A2107A-F Build/IML74K)*/
const Device UA2505155 = {S33342, 155, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr, SmartTabII7 Build/A2107A_A404_004_043_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2554796 = {S33343, 155, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt, SmartTabII7 Build/A2107A_A404_010_002_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2558159 = {S33344, 155, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es, SmartTabII7 Build/A2107A_A404_010_002_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2578906 = {S33345, 135, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SmartTabII7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2583039 = {S33346, 155, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it, SmartTabII7 Build/A2107A_A404_107_055_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2600118 = {S33347, 155, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de, SmartTabII7 Build/A2107A_A404_107_055_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2657740 = {S33348, 155, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-be, SmartTabII7 Build/A2107A_A404_004_043_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2697202 = {S33349, 155, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr, SmartTabII7 Build/A2107A_A404_004_043_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2727862 = {S33350, 155, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu, SmartTabII7 Build/A2107A_A404_004_043_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2728826 = {S33351, 155, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz, SmartTabII7 Build/A2107A_A404_004_043_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2766479 = {S33352, 155, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl, SmartTabII7 Build/A2107A_A404_108_057_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2779943 = {S33353, 155, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us, SmartTabII7 Build/A2107A_A404_107_055_X_VODA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2983112 = {S33354, 64, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SmartTabII7 Build/IMM76D)*/
const Device UA3070625 = {S33355, 61, { &P18724,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; A2107A-H Build/IMM76D)*/
const Device UA4894 = {S33356, 143, { &P14877,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; GT-I9000 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA7473 = {S33357, 134, { &P14877,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; fr-fr; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA14866 = {S33358, 146, { &P14877,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; GT-I9000B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA50457 = {S33359, 146, { &P14877,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; GT-I9000M Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA54215 = {S33360, 145, { &P14877,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I9000 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA56627 = {S33361, 172, { &P14877,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; GT-I9000M Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.1;CBCAndroidNewsAppUserAgent*/
const Device UA138502 = {S33362, 67, { &P14877,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I9000 Build/ECLAIR)*/
const Device UA505776 = {S33363, 68, { &P14877,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I9000M Build/ECLAIR)*/
const Device UA2022095 = {S33364, 131, { &P14877,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I9000 Build/ECLAIR) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022461 = {S33365, 143, { &P14877,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn;GT-I9000Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2029630 = {S33366, 157, { &P14877,&P5477,&P4861,&P18092 }};	/*Useragent - 2.1 GT-I9000---Mozilla/5.0 (Linux; U; Android 2.1; xx-xx; GT-I9000 Build/ECLAIR) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2031328 = {S33367, 153, { &P14877,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1.1; pl-pl; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 WykopDroid*/
const Device UA2361062 = {S33368, 131, { &P15263,&P7106,&P15829,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9850; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.755 Mobile Safari/534.11+*/
const Device UA3064783 = {S33369, 145, { &P18472,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-79) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066529 = {S33370, 151, { &P18472,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT925 Build/9.8.2Q-8-XT925_VQL-12.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081233 = {S33371, 145, { &P18472,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; DROID RAZR HD Build/9.8.1Q_39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3113184 = {S33372, 150, { &P18472,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XT925 Build/9.8.0Q-97-XT925_VQU-19) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3022353 = {S33373, 142, { &P12280,&P18432,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A525 Safari/8536.25*/
const Device UA3026586 = {S33374, 140, { &P12280,&P18432,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A405 Safari/8536.25*/
const Device UA3026843 = {S33375, 140, { &P12280,&P18432,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A403 Safari/8536.25*/
const Device UA3026850 = {S33376, 142, { &P12280,&P18432,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A523 Safari/8536.25*/
const Device UA3026977 = {S33377, 142, { &P12280,&P18432,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A551 Safari/8536.25*/
const Device UA2508113 = {S33378, 152, { &P15767,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_1 like Mac OS X; zh-cn Model:iPad3,4) UC AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7543.48.3*/
const Device UA2509122 = {S33379, 116, { &P15767,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari Version/5.1*/
const Device UA2510525 = {S33380, 124, { &P15767,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B141 Safari/7536.26*/
const Device UA2511761 = {S33381, 137, { &P15767,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Version/5.1 Safari/7534.48.3 Kikin/1.9.5*/
const Device UA2515079 = {S33382, 159, { &P15767,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Version/5.1 Safari/7534.48.3 NAVER(inapp; search; 103; 4.3.0)*/
const Device UA2532763 = {S33383, 127, { &P15767,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA2533054 = {S33384, 161, { &P15767,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Version/5.1 Safari/7534.48.3 NAVER(inapp; search; 103; 4.3.0)*/
const Device UA2533075 = {S33385, 126, { &P15767,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B146 Safari/7536.26*/
const Device UA2537401 = {S33386, 118, { &P15767,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari Version/5.1*/
const Device UA2538610 = {S33387, 125, { &P15767,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA2560906 = {S33388, 118, { &P15767,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B147 Safari Version/5.1*/
const Device UA2583065 = {S33389, 127, { &P15767,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA2585555 = {S33390, 118, { &P15767,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari Version/5.1*/
const Device UA2606184 = {S33391, 126, { &P15767,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B329 Safari/7536.26*/
const Device UA2749305 = {S33392, 161, { &P15767,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B147 Version/5.1 Safari/7534.48.3 NAVER(inapp; search; 103; 4.3.0)*/
const Device UA2854422 = {S33393, 126, { &P15767,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B147 Safari/7536.26*/
const Device UA2887164 = {S33394, 161, { &P15767,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Version/5.1 Safari/7534.48.3 NAVER(inapp; search; 310; 4.7.1)*/
const Device UA2535785 = {S33395, 146, { &P22394,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyC6603 Build/10.1.A.1.350) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2563396 = {S33396, 142, { &P22394,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C6603 Build/10.1.A.1.350) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2604382 = {S33397, 64, { &P22394,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; C6603 Build/10.1.A.1.434)*/
const Device UA2669377 = {S33398, 142, { &P22394,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C6603 Build/10.1.A.1.434) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2673421 = {S33399, 146, { &P22394,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyC6603 Build/10.1.A.1.434) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2741648 = {S33400, 148, { &P22394,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyC6603 Build/10.1.1.A.1.253) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752781 = {S33401, 144, { &P22394,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; C6603 Build/10.1.1.A.1.253) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2799159 = {S33402, 148, { &P22394,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyC6603 Build/10.1.1.A.1.307) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2860016 = {S33403, 66, { &P22394,&P18111,&P18112,&P18092 }};	/*Useragent - SonyC6603 Build/10.1.A.1.434 stagefright/1.2 (Linux;Android 4.1.2)*/
const Device UA2871388 = {S33404, 144, { &P22394,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C6603 Build/10.1.1.A.1.307) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957751 = {S33405, 68, { &P22394,&P18111,&P18112,&P18092 }};	/*Useragent - SonyC6603 Build/10.1.1.A.1.253 stagefright/1.2 (Linux;Android 4.1.2)*/
const Device UA3125893 = {S33406, 136, { &P22394,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; C6603 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2965806 = {S33407, 129, { &P18385,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JWR66N) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2974402 = {S33408, 129, { &P18385,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2975961 = {S33409, 129, { &P18385,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3033185 = {S33410, 58, { &P18385,&P21637,&P23997,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.3; Nexus 7 Build/JWR66Y)*/
const Device UA3033189 = {S33411, 129, { &P18385,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3034386 = {S33412, 129, { &P18385,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3080898 = {S33413, 127, { &P18385,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-; Nexus 7 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3093396 = {S33414, 129, { &P18385,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3093705 = {S33415, 129, { &P18385,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JSS15R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3128244 = {S33416, 136, { &P18385,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2676282 = {S33417, 147, { &P18472,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676823 = {S33418, 145, { &P18472,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; DROID RAZR HD Build/9.8.1Q_39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676930 = {S33419, 151, { &P18472,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT925 Build/9.8.2Q-8-XT925_VQL-12.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677001 = {S33420, 150, { &P18472,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; RAZR HD Build/9.8.2Q-8-XT925_VQL-6) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677068 = {S33421, 144, { &P18472,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT925 Build/9.8.2Q-8-XT925_VQL-12.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2678420 = {S33422, 138, { &P18472,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; DROID RAZR HD Build/9.8.1Q_39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2707088 = {S33423, 150, { &P18472,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XT925 Build/9.8.0Q-97-XT925_VQU-19) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2937702 = {S33424, 145, { &P18472,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-79) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA197223 = {S33425, 63, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; en) Presto/2.9.168 Version/11.50*/
const Device UA208522 = {S33426, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03790); ru) Presto/2.9.168 Version/11.50*/
const Device UA210854 = {S33427, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.9.168 Version/11.50*/
const Device UA216896 = {S33428, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.9.168 Version/11.50*/
const Device UA217164 = {S33429, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03789); ru) Presto/2.9.168 Version/11.50*/
const Device UA218262 = {S33430, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.9.168 Version/11.50*/
const Device UA218452 = {S33431, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.9.168 Version/11.50*/
const Device UA223227 = {S33432, 100, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.9.168 Version/11.50,gzip(gfe) (via translate.google.com)*/
const Device UA230301 = {S33433, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.9.168 Version/11.50*/
const Device UA239522 = {S33434, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03773); ru) Presto/2.9.168 Version/11.50*/
const Device UA323494 = {S33435, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4139); ru) Presto/2.9.168 Version/11.50*/
const Device UA324246 = {S33436, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4110); ru) Presto/2.9.168 Version/11.50*/
const Device UA324896 = {S33437, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4156); ru) Presto/2.9.168 Version/11.50*/
const Device UA325613 = {S33438, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.9.168 Version/11.50*/
const Device UA328370 = {S33439, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.9.168 Version/11.50*/
const Device UA334707 = {S33440, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03757); ru) Presto/2.9.168 Version/11.50*/
const Device UA337978 = {S33441, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.9.168 Version/11.50*/
const Device UA454559 = {S33442, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.9.168 Version/11.50*/
const Device UA554829 = {S33443, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03658); ru) Presto/2.9.168 Version/11.50*/
const Device UA567201 = {S33444, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4661); ru) Presto/2.9.168 Version/11.50*/
const Device UA574429 = {S33445, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4848); ru) Presto/2.9.168 Version/11.50*/
const Device UA607657 = {S33446, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.9.168 Version/11.50*/
const Device UA689548 = {S33447, 72, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.50*/
const Device UA702095 = {S33448, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.9.168 Version/11.50*/
const Device UA703776 = {S33449, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.9.168 Version/11.50*/
const Device UA718745 = {S33450, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.9.168 Version/11.50*/
const Device UA833811 = {S33451, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.9.168 Version/11.50*/
const Device UA881972 = {S33452, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03639); ru) Presto/2.9.168 Version/11.50*/
const Device UA898453 = {S33453, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.9.168 Version/11.50*/
const Device UA1888918 = {S33454, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03672); ru) Presto/2.9.168 Version/11.50*/
const Device UA1889326 = {S33455, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03791); ru) Presto/2.9.168 Version/11.50*/
const Device UA1982453 = {S33456, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.9.168 Version/11.50*/
const Device UA2140079 = {S33457, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03638); ru) Presto/2.9.168 Version/11.50*/
const Device UA2147638 = {S33458, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4871); ru) Presto/2.9.168 Version/11.50*/
const Device UA2161111 = {S33459, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.9.168 Version/11.50*/
const Device UA2163432 = {S33460, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5265); ru) Presto/2.9.168 Version/11.50*/
const Device UA2166052 = {S33461, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5711); ru) Presto/2.9.168 Version/11.50*/
const Device UA2167619 = {S33462, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.9.168 Version/11.50*/
const Device UA2226244 = {S33463, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.9.168 Version/11.50*/
const Device UA2227747 = {S33464, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.9.168 Version/11.50*/
const Device UA2237519 = {S33465, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.9.168 Version/11.50*/
const Device UA2262380 = {S33466, 58, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows 7; U; en) Presto/2.9.168 Version/11.50*/
const Device UA2290420 = {S33467, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03755); ru) Presto/2.9.168 Version/11.50*/
const Device UA2291749 = {S33468, 66, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; ru) Opera 11.50*/
const Device UA2320639 = {S33469, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5196); ru) Presto/2.9.168 Version/11.50*/
const Device UA2325403 = {S33470, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5267); ru) Presto/2.9.168 Version/11.50*/
const Device UA2395040 = {S33471, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5328); ru) Presto/2.9.168 Version/11.50*/
const Device UA2422534 = {S33472, 73, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.9.168 Version/11.50,gzip(gfe)*/
const Device UA2444721 = {S33473, 86, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5308); ru) Presto/2.9.168 Version/11.50*/
const Device UA2457275 = {S33474, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.9.168 Version/11.50*/
const Device UA2466449 = {S33475, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5998); ru) Presto/2.9.168 Version/11.50*/
const Device UA2486703 = {S33476, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.9.168 Version/11.50*/
const Device UA2503784 = {S33477, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.9.168 Version/11.50*/
const Device UA2667554 = {S33478, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6068); ru) Presto/2.9.168 Version/11.50*/
const Device UA2690902 = {S33479, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5966); ru) Presto/2.9.168 Version/11.50*/
const Device UA2705470 = {S33480, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6060); ru) Presto/2.9.168 Version/11.50*/
const Device UA2869049 = {S33481, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.1 (build 6578); ru) Presto/2.9.168 Version/11.50*/
const Device UA2936438 = {S33482, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6089); ru) Presto/2.9.168 Version/11.50*/
const Device UA3043211 = {S33483, 85, { &P15364,&P7133,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 8.0 (build 5784); ru) Presto/2.9.168 Version/11.50*/
const Device UA2489020 = {S33484, 112, { &P15803,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia200/11.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA47331 = {S33485, 153, { &P23914,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonR800x Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA68466 = {S33486, 150, { &P23914,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; SonyEricssonR800x Build/3.0.E.2.89) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA205338 = {S33487, 138, { &P23914,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; R800x Build/3.0.E.2.89) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA223130 = {S33488, 152, { &P23914,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonR800x Build/3.0.1.E.0.88) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA248664 = {S33489, 140, { &P23914,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; R800x Build/3.0.1.E.0.88) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA253905 = {S33490, 150, { &P23914,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; SonyEricssonR800x Build/3.0.E.2.52) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA554286 = {S33491, 62, { &P23914,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.2; R800x Build/3.0.E.2.89)*/
const Device UA625027 = {S33492, 152, { &P23914,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonR800x Build/4.0.2.E.0.57) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA632014 = {S33493, 200, { &P23914,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonR800xhttp://www.sonyericsson.com/UAprof/R800xR301.xml Build/3.0.1.E.1.44) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA637336 = {S33494, 140, { &P23914,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; R800x Build/4.0.2.E.0.57) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA644889 = {S33495, 64, { &P23914,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; R800x Build/3.0.1.E.0.88)*/
const Device UA675641 = {S33496, 140, { &P23914,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; R800x Build/3.0.1.E.1.44) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA681229 = {S33497, 64, { &P23914,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; R800x Build/4.0.2.E.0.57)*/
const Device UA1889213 = {S33498, 152, { &P23914,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonR800x Build/3.0.1.E.0.82) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020067 = {S33499, 147, { &P23914,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; R800x Build/3.0.1.E.0.88) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2024216 = {S33500, 200, { &P23914,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonR800xhttp://www.sonyericsson.com/UAprof/R800xR301.xml Build/4.0.2.E.0.63) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026057 = {S33501, 151, { &P23914,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonR800x Build/4.0.2.E.0.57) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA2027869 = {S33502, 151, { &P23914,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonR800xBuild/4.0.2.E.0.57) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032577 = {S33503, 151, { &P23914,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; SonyEricssonR800x Build/4.0.2.E.0.57) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2442199 = {S33504, 151, { &P23914,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;SonyEricssonR800x Build/4.0.2.E.0.57) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2759074 = {S33505, 140, { &P23914,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; R800x Build/4.0.2.E.0.63) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2457650 = {S33506, 140, { &P16292,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.20 (KHTML, like Gecko) Chrome/25.0.1338.0 Mobile Safari/537.20*/
const Device UA2566120 = {S33507, 141, { &P16292,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566545 = {S33508, 142, { &P16292,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567017 = {S33509, 134, { &P16292,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567104 = {S33510, 140, { &P16292,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Galaxy Nexus Build/JOP40C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567242 = {S33511, 135, { &P16292,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA633886 = {S33512, 58, { &P15364,&P5690,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko/X Firefox/12.0*/
const Device UA698125 = {S33513, 71, { &P15364,&P5690,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:12.0) Gecko/X Firefox/12.0 Firefox/12.0*/
const Device UA2690750 = {S33514, 48, { &P15364,&P5690,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0WindowsNT5.1rv12.0Gecko/XFirefox/12.0*/
const Device UA2703763 = {S33515, 78, { &P15364,&P5690,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; es-ES; rv:1.9.2) Gecko/X Firefox/12.0*/
const Device UA2713369 = {S33516, 73, { &P15364,&P5690,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Firefox/12.0; Windows NT 5.1;) vAutoBrowser/1304*/
const Device UA2719050 = {S33517, 61, { &P15364,&P5690,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:12.0) Gecko/12.0 Firefox/12.0*/
const Device UA2470101 = {S33518, 70, { &P15364,&P17285,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:21.0) Gecko/X Firefox/21.0*/
const Device UA2567473 = {S33519, 65, { &P15364,&P17285,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:21.0) Gecko/X Firefox/21.0*/
const Device UA2784928 = {S33520, 78, { &P15364,&P17285,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:21.0) Gecko/X Firefox/21.0 IceDragon/21.0.1.15*/
const Device UA2944520 = {S33521, 77, { &P15364,&P17285,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; fr; rv:1.9.0.1) Gecko/X Firefox/21.0*/
const Device UA177679 = {S33522, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.8.131 Version/11.10*/
const Device UA178958 = {S33523, 100, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.8.131 Version/11.10,gzip(gfe) (via translate.google.com)*/
const Device UA178965 = {S33524, 63, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; en) Presto/2.8.131 Version/11.10*/
const Device UA179428 = {S33525, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.8.131 Version/11.10*/
const Device UA190157 = {S33526, 63, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; en) Presto/2.8.131 Version/11.10*/
const Device UA191025 = {S33527, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.8.131 Version/11.10*/
const Device UA191120 = {S33528, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.8.131 Version/11.10*/
const Device UA197239 = {S33529, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.8.131 Version/11.11*/
const Device UA197744 = {S33530, 63, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; en) Presto/2.8.131 Version/11.11*/
const Device UA198980 = {S33531, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.8.131 Version/11.11*/
const Device UA199704 = {S33532, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.8.131 Version/11.11*/
const Device UA202360 = {S33533, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.8.131 Version/11.11*/
const Device UA202537 = {S33534, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.8.131 Version/11.11*/
const Device UA203459 = {S33535, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.8.131 Version/11.11*/
const Device UA203468 = {S33536, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.8.131 Version/11.11*/
const Device UA310663 = {S33537, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.8.131 Version/11.10*/
const Device UA310819 = {S33538, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.8.131 Version/11.10*/
const Device UA310829 = {S33539, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.8.131 Version/11.10*/
const Device UA311303 = {S33540, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.8.131 Version/11.10*/
const Device UA312739 = {S33541, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.8.131 Version/11.10*/
const Device UA314009 = {S33542, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.8.131 Version/11.10*/
const Device UA314986 = {S33543, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.8.131 Version/11.11*/
const Device UA315262 = {S33544, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.8.131 Version/11.11*/
const Device UA315545 = {S33545, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.8.131 Version/11.10*/
const Device UA315780 = {S33546, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.8.131 Version/11.11*/
const Device UA315941 = {S33547, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.8.131 Version/11.11*/
const Device UA319172 = {S33548, 63, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; en) Presto/2.8.131 Version/11.11*/
const Device UA319174 = {S33549, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.8.131 Version/11.11*/
const Device UA319648 = {S33550, 100, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.8.131 Version/11.11,gzip(gfe) (via translate.google.com)*/
const Device UA319768 = {S33551, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.8.131 Version/11.11*/
const Device UA320318 = {S33552, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.8.131 Version/11.10*/
const Device UA322640 = {S33553, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.8.131 Version/11.11*/
const Device UA328791 = {S33554, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.8.131 Version/11.11*/
const Device UA359433 = {S33555, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.8.131 Version/11.11*/
const Device UA374856 = {S33556, 34, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/11.1 (Windows NT 5.1: U: en)*/
const Device UA376735 = {S33557, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.8.131 Version/11.10*/
const Device UA401100 = {S33558, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.8.131 Version/11.10*/
const Device UA419425 = {S33559, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.8.131 Version/11.11*/
const Device UA449581 = {S33560, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.8.131 Version/11.11*/
const Device UA537728 = {S33561, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.8.131 Version/11.10*/
const Device UA599784 = {S33562, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.8.131 Version/11.11*/
const Device UA608805 = {S33563, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.8.131 Version/11.10*/
const Device UA609094 = {S33564, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.8.131 Version/11.11*/
const Device UA669072 = {S33565, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.8.131 Version/11.11*/
const Device UA694385 = {S33566, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.8.131 Version/11.10*/
const Device UA704778 = {S33567, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.8.131 Version/11.11*/
const Device UA713053 = {S33568, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.8.131 Version/11.11*/
const Device UA758798 = {S33569, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.8.131 Version/11.11*/
const Device UA769628 = {S33570, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.8.131 Version/11.10*/
const Device UA772583 = {S33571, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.8.131 Version/11.10*/
const Device UA792949 = {S33572, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.8.131 Version/11.10*/
const Device UA810811 = {S33573, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.8.131 Version/11.10*/
const Device UA896899 = {S33574, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.8.131 Version/11.10*/
const Device UA1874380 = {S33575, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.8.131 Version/11.10*/
const Device UA1875171 = {S33576, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.8.131 Version/11.10*/
const Device UA1880510 = {S33577, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.8.131 Version/11.11*/
const Device UA1888219 = {S33578, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.8.131 Version/11.10*/
const Device UA1951843 = {S33579, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.8.131 Version/11.11*/
const Device UA1963317 = {S33580, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.8.131 Version/11.11*/
const Device UA1980232 = {S33581, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.8.131 Version/11.11*/
const Device UA2153470 = {S33582, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.8.131 Version/11.11*/
const Device UA2159171 = {S33583, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.8.131 Version/11.11*/
const Device UA2167022 = {S33584, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.8.131 Version/11.11*/
const Device UA2231890 = {S33585, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.8.131 Version/11.10*/
const Device UA2262423 = {S33586, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.8.131 Version/11.11*/
const Device UA2262506 = {S33587, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.8.131 Version/11.11*/
const Device UA2283510 = {S33588, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.8.131 Version/11.10*/
const Device UA2285896 = {S33589, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); ru) Presto/2.8.131 Version/11.11*/
const Device UA2302585 = {S33590, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.8.131 Version/11.10*/
const Device UA2302967 = {S33591, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4871); ru) Presto/2.8.131 Version/11.11*/
const Device UA2317499 = {S33592, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.8.131 Version/11.11*/
const Device UA2330810 = {S33593, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.8.131 Version/11.10*/
const Device UA2339147 = {S33594, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.8.131 Version/11.10*/
const Device UA2342183 = {S33595, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.8.131 Version/11.11*/
const Device UA2344764 = {S33596, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.8.131 Version/11.10*/
const Device UA2369253 = {S33597, 34, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/11.1 (Windows NT 5.1; U; en)*/
const Device UA2411624 = {S33598, 86, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.8.131 Version/11.10*/
const Device UA2421263 = {S33599, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.8.131 Version/11.11*/
const Device UA2463245 = {S33600, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.8.131 Version/11.10*/
const Device UA2490372 = {S33601, 34, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/11.1 (Windows NT 5.2: U; en)*/
const Device UA2503244 = {S33602, 73, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.8.131 Version/11.10,gzip(gfe)*/
const Device UA2529049 = {S33603, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.8.131 Version/11.11*/
const Device UA2594727 = {S33604, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.8.131 Version/11.11*/
const Device UA2595013 = {S33605, 73, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.8.131 Version/11.11,gzip(gfe)*/
const Device UA2667671 = {S33606, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.8.131 Version/11.11*/
const Device UA2697214 = {S33607, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.8.131 Version/11.10*/
const Device UA2727393 = {S33608, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.8.131 Version/11.11*/
const Device UA2728011 = {S33609, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.8.131 Version/11.10*/
const Device UA2746846 = {S33610, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.8.131 Version/11.10*/
const Device UA2758985 = {S33611, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.8.131 Version/11.10*/
const Device UA2760882 = {S33612, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.8.131 Version/11.10*/
const Device UA2839621 = {S33613, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.8.131 Version/11.11*/
const Device UA2855776 = {S33614, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6060); ru) Presto/2.8.131 Version/11.11*/
const Device UA2983311 = {S33615, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 8.0 (build 5880); ru) Presto/2.8.131 Version/11.11*/
const Device UA3045674 = {S33616, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.8.131 Version/11.10*/
const Device UA3085768 = {S33617, 85, { &P15364,&P5690,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.8.131 Version/11.10*/
const Device UA779826 = {S33618, 131, { &P17687,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MZ604 Build/I.7.1-14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA868674 = {S33619, 131, { &P17687,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; MZ604 Build/I.7.1-34) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA898546 = {S33620, 138, { &P17687,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; MZ604 Build/I.7.1-34_KDDI-6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA911121 = {S33621, 131, { &P17687,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; MZ604 Build/I.7.1-42) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1959598 = {S33622, 60, { &P17687,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; MZ604 Build/I.7.1-34)*/
const Device UA1994697 = {S33623, 131, { &P17687,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MZ604 Build/I.7.1-39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1996959 = {S33624, 131, { &P17687,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MZ604 Build/I.7.1-45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1997082 = {S33625, 60, { &P17687,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MZ604 Build/I.7.1-45)*/
const Device UA370031 = {S33626, 151, { &P12841,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; HTC_PH39100/3.15.0.0 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA601816 = {S33627, 142, { &P12841,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC PH39100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA627502 = {S33628, 154, { &P12841,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_PH39100/3.26.502.56 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA659464 = {S33629, 64, { &P12841,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC PH39100 Build/IML74K)*/
const Device UA811802 = {S33630, 154, { &P12841,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_PH39100/3.26.502.52 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA829987 = {S33631, 154, { &P12841,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_PH39100/3.26.502.54 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA863010 = {S33632, 153, { &P12841,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_PH39100/1.63.502.4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2218534 = {S33633, 152, { &P12841,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; HTC_PH39100/3.26.502.56 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2686186 = {S33634, 140, { &P12841,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; HTC PH39100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708159 = {S33635, 152, { &P12841,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTCPH39100/3.26.502.56 Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3046787 = {S33636, 142, { &P12841,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC PH39100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA910066 = {S33637, 119, { &P17595,&P17296,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11*/
const Device UA910429 = {S33638, 119, { &P17595,&P17296,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11*/
const Device UA912385 = {S33639, 119, { &P17595,&P17296,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA912392 = {S33640, 119, { &P17595,&P17296,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA912405 = {S33641, 119, { &P17595,&P17296,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA912543 = {S33642, 119, { &P17595,&P17296,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA912655 = {S33643, 119, { &P17595,&P17296,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA912765 = {S33644, 119, { &P17595,&P17296,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA913280 = {S33645, 119, { &P17595,&P17296,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA1968912 = {S33646, 158, { &P17595,&P17296,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2580298 = {S33647, 116, { &P17595,&P17296,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1060) AppleWebKit/536.11 (KHTML like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA2674679 = {S33648, 129, { &P17595,&P17296,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/20.0.648.205 Safari/534.16*/
const Device UA2674818 = {S33649, 129, { &P17595,&P17296,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/20.0.648.205 Safari/534.16*/
const Device UA2714925 = {S33650, 129, { &P17595,&P17296,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/20.0.648.205 Safari/534.16*/
const Device UA1999712 = {S33651, 74, { &P17595,&P17296,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:16.0) Gecko/X Firefox/16.0*/
const Device UA3086229 = {S33652, 74, { &P17595,&P17296,&P18428,&P18092 }};	/*Useragent - Mozilla/6.0 (Macintosh; Intel Mac OS X 10.6; rv:16.0) Gecko/X Firefox/16.0*/
const Device UA251312 = {S33653, 101, { &P15364,&P16110,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22*/
const Device UA251376 = {S33654, 108, { &P15364,&P16110,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22*/
const Device UA350831 = {S33655, 106, { &P15364,&P16110,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7*/
const Device UA351428 = {S33656, 99, { &P15364,&P16110,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7*/
const Device UA409450 = {S33657, 100, { &P15364,&P16110,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50*/
const Device UA630000 = {S33658, 106, { &P15364,&P16110,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2*/
const Device UA630343 = {S33659, 106, { &P15364,&P16110,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3*/
const Device UA680181 = {S33660, 101, { &P15364,&P16110,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16*/
const Device UA685963 = {S33661, 99, { &P15364,&P16110,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3*/
const Device UA751785 = {S33662, 99, { &P15364,&P16110,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2*/
const Device UA778923 = {S33663, 108, { &P15364,&P16110,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16*/
const Device UA803329 = {S33664, 93, { &P15364,&P16110,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50*/
const Device UA2586540 = {S33665, 100, { &P15364,&P16110,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.3 Safari/534.53.10*/
const Device UA2586543 = {S33666, 107, { &P15364,&P16110,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.3 Safari/534.53.10*/
const Device UA100327 = {S33667, 13, { &P15364,&P17017,&P18418,&P18092 }};	/*Useragent - PycURL/7.19.5*/
const Device UA127037 = {S33668, 32, { &P15364,&P17017,&P18418,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; pycurl)*/
const Device UA140389 = {S33669, 13, { &P15364,&P17017,&P18418,&P18092 }};	/*Useragent - PycURL/7.19.7*/
const Device UA226575 = {S33670, 13, { &P15364,&P17017,&P18418,&P18092 }};	/*Useragent - PycURL/7.21.3*/
const Device UA393681 = {S33671, 13, { &P15364,&P17017,&P18418,&P18092 }};	/*Useragent - PycURL/7.21.2*/
const Device UA408216 = {S33672, 13, { &P15364,&P17017,&P18418,&P18092 }};	/*Useragent - PycURL/7.15.5*/
const Device UA608631 = {S33673, 13, { &P15364,&P17017,&P18418,&P18092 }};	/*Useragent - PycURL/7.25.0*/
const Device UA612870 = {S33674, 13, { &P15364,&P17017,&P18418,&P18092 }};	/*Useragent - PycURL/7.23.1*/
const Device UA622668 = {S33675, 13, { &P15364,&P17017,&P18418,&P18092 }};	/*Useragent - PycURL/7.18.2*/
const Device UA645408 = {S33676, 13, { &P15364,&P17017,&P18418,&P18092 }};	/*Useragent - PycURL/7.19.4*/
const Device UA672224 = {S33677, 13, { &P15364,&P17017,&P18418,&P18092 }};	/*Useragent - PycURL/7.21.6*/
const Device UA705198 = {S33678, 13, { &P15364,&P17017,&P18418,&P18092 }};	/*Useragent - PycURL/7.21.0*/
const Device UA891582 = {S33679, 13, { &P15364,&P17017,&P18418,&P18092 }};	/*Useragent - PycURL/7.22.0*/
const Device UA900542 = {S33680, 13, { &P15364,&P17017,&P18418,&P18092 }};	/*Useragent - PycURL/7.21.4*/
const Device UA901327 = {S33681, 13, { &P15364,&P17017,&P18418,&P18092 }};	/*Useragent - PycURL/7.24.0*/
const Device UA54266 = {S33682, 133, { &P14161,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; VM670 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57925 = {S33683, 152, { &P14161,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; VM670 Build/Aphrodite Extra Pure 1.x) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57927 = {S33684, 134, { &P14161,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; VM670 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70727 = {S33685, 131, { &P14161,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-; VM670 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90483 = {S33686, 138, { &P14161,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-; VM670 Build/FRG83) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA188557 = {S33687, 57, { &P14161,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; VM670 Build/FRG83)*/
const Device UA378547 = {S33688, 58, { &P14161,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; VM670 Build/FRG83G)*/
const Device UA1891715 = {S33689, 134, { &P14161,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-VM670 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018397 = {S33690, 140, { &P14161,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; VM670 Build/FRG83) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2020962 = {S33691, 157, { &P14161,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en_us; VM670 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)*/
const Device UA2020986 = {S33692, 141, { &P14161,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; VM670 Build/FRG83G) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2022240 = {S33693, 132, { &P14161,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-; VM670 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024217 = {S33694, 137, { &P14161,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; VM670 Build/OverDrive) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2643062 = {S33695, 135, { &P14161,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; VM670 Build/StockOV) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA350391 = {S33696, 131, { &P12825,&P6374,&P4418,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9380; en-GB) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.513 Mobile Safari/534.11+*/
const Device UA2973474 = {S33697, 127, { &P18385,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Nexus 7 Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2973946 = {S33698, 128, { &P18385,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974788 = {S33699, 129, { &P18385,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 7 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA12872 = {S33700, 155, { &P13669,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1 Gingerlike Joneidy; en-us; LG-MS690 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA34311 = {S33701, 134, { &P13669,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-MS690 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA34314 = {S33702, 136, { &P13669,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-MS690 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40620 = {S33703, 164, { &P13669,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1 Sense Rom Joneidy 7/18/2011; en-us; LG-MS690 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40622 = {S33704, 137, { &P13669,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-MS690 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55940 = {S33705, 132, { &P13669,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-; LG-MS690 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55942 = {S33706, 154, { &P13669,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1 Sense Rom Joneidy; en-us; LG-MS690 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90518 = {S33707, 143, { &P13669,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-MS690 Build/FRG83) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA187935 = {S33708, 60, { &P13669,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; LG-MS690 Build/FRG83)*/
const Device UA300575 = {S33709, 58, { &P13669,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; LG-MS690 Build/FRF91)*/
const Device UA401103 = {S33710, 61, { &P13669,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-MS690 Build/FRG83G)*/
const Device UA2020666 = {S33711, 134, { &P13669,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-; LG-MS690 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022631 = {S33712, 135, { &P13669,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-; LG-MS690 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA899808 = {S33713, 77, { &P18436,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; Touch)*/
const Device UA1957679 = {S33714, 82, { &P18436,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; Win64; x64; Trident/6.0; Touch)*/
const Device UA2155306 = {S33715, 70, { &P18436,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; Trident/6.0; Touch)*/
const Device UA2419525 = {S33716, 172, { &P18436,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0)*/
const Device UA2655275 = {S33717, 185, { &P18436,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; Win64; x64; Trident/6.0; Touch; .NET4.0E; .NET4.0C; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; MDDCJS)*/
const Device UA170397 = {S33718, 189, { &P15364,&P5690,&P21926,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E) chromeframe/8.0.552.224*/
const Device UA185636 = {S33719, 168, { &P15364,&P5690,&P21926,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 3.0.04506.30; .NET4.0C; .NET4.0E) chromeframe/8.0.552.224*/
const Device UA298751 = {S33720, 143, { &P15364,&P5690,&P21926,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10 chromeframe/8.0.552.224*/
const Device UA395430 = {S33721, 173, { &P15364,&P5690,&P21926,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 3.0.04506.30; .NET4.0C; .NET4.0E) chromeframe/8.0.552.224*/
const Device UA451522 = {S33722, 143, { &P15364,&P5690,&P21926,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10 chromeframe/8.0.552.224*/
const Device UA495300 = {S33723, 74, { &P15364,&P5690,&P21926,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1) chromeframe/8.0.552.224*/
const Device UA718322 = {S33724, 220, { &P15364,&P5690,&P21926,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; GX755 XPUSV1.X; SVCVersion 7.30; SVCVersion 8.11; SVCVersion 8.14; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; InfoPath.1) chromeframe/8.0.552.224*/
const Device UA850442 = {S33725, 87, { &P15364,&P5690,&P21926,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0) chromeframe/8.0.552.237*/
const Device UA2720465 = {S33726, 161, { &P15364,&P5690,&P21926,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; Zune 4.7; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729) chromeframe/8.0.552.237*/
const Device UA2706598 = {S33727, 138, { &P21648,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9505 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2712285 = {S33728, 60, { &P21648,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9505 Build/JDQ39)*/
const Device UA2783059 = {S33729, 139, { &P21648,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9505 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2795200 = {S33730, 136, { &P21648,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-; GT-I9505 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2938469 = {S33731, 139, { &P21648,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9505G Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3076974 = {S33732, 61, { &P21648,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9505G Build/JDQ39)*/
const Device UA3089052 = {S33733, 139, { &P21648,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9505 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA18394 = {S33734, 149, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC_IncredibleS_S710e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41790 = {S33735, 227, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTCS710e/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43169 = {S33736, 144, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; HTC Incredible S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57595 = {S33737, 145, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Incredible S Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57611 = {S33738, 147, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ru-; HTC_IncredibleS_S710e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71481 = {S33739, 144, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Incredible S Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA191783 = {S33740, 68, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Incredible S Build/GRI40)*/
const Device UA567953 = {S33741, 132, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Incredible S Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA623159 = {S33742, 149, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC_IncredibleS_S710e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA624877 = {S33743, 227, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTCS710e/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA636756 = {S33744, 144, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC Incredible S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA658321 = {S33745, 68, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Incredible S Build/GRJ90)*/
const Device UA674327 = {S33746, 157, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Incredible S Build/Nik's TriNity v4.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA716481 = {S33747, 132, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Incredible S Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA754665 = {S33748, 140, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; Incredible S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA802938 = {S33749, 162, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_IncredibleS_S710e Build/Nik's TriNity v4.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA821257 = {S33750, 147, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-; HTC_IncredibleS_S710e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA903384 = {S33751, 137, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Incredible S Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1898555 = {S33752, 137, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-kr; HTC_S710E Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1961688 = {S33753, 137, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-kr; HTC_S710E Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1984973 = {S33754, 137, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Incredible S Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2021471 = {S33755, 64, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; Incredible S Build/GRI40)*/
const Device UA2034611 = {S33756, 137, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Incredible S Build GRJ90) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2544428 = {S33757, 137, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Incredible S Build GRI40) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2977894 = {S33758, 140, { &P13042,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; Incredible S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2797002 = {S33759, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; iw-il) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9174AT*/
const Device UA3022577 = {S33760, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; de-de) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9737AP*/
const Device UA3023296 = {S33761, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fr-fr) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9737AT*/
const Device UA3024328 = {S33762, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; in-id) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9737AP*/
const Device UA3030976 = {S33763, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-gb) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP*/
const Device UA3031629 = {S33764, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AT*/
const Device UA3031718 = {S33765, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ko-kr) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP*/
const Device UA3032927 = {S33766, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; vi-vn) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP*/
const Device UA3032933 = {S33767, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; tr-tr) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP*/
const Device UA3032944 = {S33768, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fa-ir) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP*/
const Device UA3032949 = {S33769, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; pl-pl) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AT*/
const Device UA3033504 = {S33770, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; zh-tw) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP*/
const Device UA3033976 = {S33771, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ar-ae) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP*/
const Device UA3033977 = {S33772, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-ca) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP*/
const Device UA3033993 = {S33773, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fa-fa) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP*/
const Device UA3034127 = {S33774, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP*/
const Device UA3034760 = {S33775, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-in) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP*/
const Device UA3035716 = {S33776, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-au) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AT*/
const Device UA3035729 = {S33777, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; it-it) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AT*/
const Device UA3036112 = {S33778, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-es) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AT*/
const Device UA3036126 = {S33779, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; pt-br) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP*/
const Device UA3036131 = {S33780, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; ru-ru) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AT*/
const Device UA3059780 = {S33781, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-dz) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP*/
const Device UA3059925 = {S33782, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; nl-nl) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AT*/
const Device UA3059929 = {S33783, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-sg) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP*/
const Device UA3060844 = {S33784, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; es-mx) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP*/
const Device UA3061105 = {S33785, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-ph) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP*/
const Device UA3062781 = {S33786, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-ae) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP*/
const Device UA3079427 = {S33787, 128, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fa-) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP*/
const Device UA3079659 = {S33788, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-nz) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AT*/
const Device UA3079703 = {S33789, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; pt-pt) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AT*/
const Device UA3079833 = {S33790, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; zh-cn) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP*/
const Device UA3080230 = {S33791, 130, { &P15364,&P18110,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-ie) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP*/
const Device UA48281 = {S33792, 144, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 Build/4.5.1_57_DR2-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48284 = {S33793, 143, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2Build/4.5.1_57_DR2-31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48289 = {S33794, 143, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 Build/4.5.1_57_DR2-31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA48292 = {S33795, 143, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; DROID2 Build/4.5.1_57_DR2-31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48299 = {S33796, 144, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 Build/4.5.1_57_DR2-31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48285 = {S33797, 143, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;DROID2 Build/4.5.1_57_DR2-31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61262 = {S33798, 144, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 Build/4.5.1_57_DR2-29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79454 = {S33799, 166, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en_us; DROID2 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.2 Mobile (Dragonfruit)*/
const Device UA186325 = {S33800, 133, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 Build/STAB) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA246970 = {S33801, 68, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; DROID2 Build/4.5.1_57_DR2-31)*/
const Device UA249911 = {S33802, 132, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID2 Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA321413 = {S33803, 132, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID2 Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA329459 = {S33804, 132, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID2 Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA379152 = {S33805, 135, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID2 Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA451104 = {S33806, 136, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Droid 2 Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA570930 = {S33807, 151, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 Build/4.5.1_57_DR2-31) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA609100 = {S33808, 68, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; DROID2 Build/4.5.1_57_DR4-51)*/
const Device UA609273 = {S33809, 143, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; DROID2 Build/4.5.1_57_DR4-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA611269 = {S33810, 143, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID2Build/4.5.1_57_DR4-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623232 = {S33811, 144, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID2 Build/4.5.1_57_DR4-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623422 = {S33812, 144, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID2 Build/4.5.1_57_DR4-52) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA650597 = {S33813, 143, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;DROID2 Build/4.5.1_57_DR4-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA773810 = {S33814, 143, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID2 Build/4.5.1_57_DR4-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA854613 = {S33815, 143, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; DROID2 Build/4.5.1_57_DR4-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022642 = {S33816, 56, { &P13286,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; DROID2 Build/VZW)*/
const Device UA92928 = {S33817, 115, { &P15364,&P17350,&P17384,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Digital AlphaServer 1000A 4/233; Windows NT; Powered By 64-Bit Alpha Processor)*/
const Device UA111824 = {S33818, 41, { &P15364,&P17350,&P17384,&P18092 }};	/*Useragent - Mozilla/3.0 (compatible; MSIE 4.0; Win32)*/
const Device UA143996 = {S33819, 46, { &P15364,&P17350,&P17384,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.0; Windows NT)*/
const Device UA492419 = {S33820, 54, { &P15364,&P17350,&P17384,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows; U; 32bit)*/
const Device UA630906 = {S33821, 41, { &P15364,&P17350,&P17384,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.0; Win32)*/
const Device UA2127266 = {S33822, 47, { &P15364,&P17350,&P17384,&P18092 }};	/*Useragent - Mozilla/4.5 (compatible; MSIE 4.01; Windows NT)*/
const Device UA2509650 = {S33823, 52, { &P15364,&P17350,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; 68K; Trident/5.0)*/
const Device UA2660071 = {S33824, 59, { &P15364,&P17350,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; 68K; WOW64; Trident/5.0)*/
const Device UA2660760 = {S33825, 65, { &P15364,&P17350,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; 68K; Win64; IA64; Trident/4.0)*/
const Device UA2791908 = {S33826, 57, { &P15364,&P17350,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; 68K; SV1; Trident/2.0)*/
const Device UA603896 = {S33827, 145, { &P19891,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; SAMSUNG-SGH-I717 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA658754 = {S33828, 147, { &P19891,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SAMSUNG-SGH-I717 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA892561 = {S33829, 147, { &P19891,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA896727 = {S33830, 69, { &P19891,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D)*/
const Device UA898903 = {S33831, 145, { &P19891,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA924201 = {S33832, 147, { &P19891,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I717 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027689 = {S33833, 69, { &P19891,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SAMSUNG-SGH-I717 Build/IML74K)*/
const Device UA2033253 = {S33834, 153, { &P19891,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA85208 = {S33835, 16, { &P18088,&P17017,&P18246,&P18092 }};	/*Useragent - MAUI WAP Browser*/
const Device UA87307 = {S33836, 16, { &P18088,&P17017,&P18246,&P18092 }};	/*Useragent - MAUI_WAP_Browser*/
const Device UA87736 = {S33837, 34, { &P18088,&P17017,&P18246,&P18092 }};	/*Useragent - MAUI WAP Browser, MAUI WAP Browser*/
const Device UA134008 = {S33838, 12, { &P18088,&P17017,&P18246,&P18092 }};	/*Useragent - Maui Browser*/
const Device UA136021 = {S33839, 43, { &P18088,&P17017,&P18246,&P18092 }};	/*Useragent - MAUI WAP Browser/WAP2.0 Profile/MT6253/QCIF*/
const Device UA144750 = {S33840, 43, { &P18088,&P17017,&P18246,&P18092 }};	/*Useragent - MAUI WAP Browser/WAP2.0 Profile/MT6253/QVGA*/
const Device UA623034 = {S33841, 52, { &P18088,&P17017,&P18246,&P18092 }};	/*Useragent - M70/Q03C MAUI-browser/MIDP2.0/CLDC1.1/Screen-240X400*/
const Device UA658006 = {S33842, 26, { &P18088,&P17017,&P18246,&P18092 }};	/*Useragent - Maui Browser, Maui Browser*/
const Device UA891929 = {S33843, 36, { &P18088,&P17017,&P18246,&P18092 }};	/*Useragent - D66/Q03C MAUI-browser/Screen-240X320*/
const Device UA2383642 = {S33844, 194, { &P18088,&P17017,&P18246,&P18092 }};	/*Useragent - TELSDA55_11B_HW (MRE\2.3.00(2048) resolution\X chipset\MT6255 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) G68_LS_F2_EN_X Release/2012.08.30 WAP Browser/MAUI Profile/  Q03C1-2.40 en-US*/
const Device UA2415865 = {S33845, 199, { &P18088,&P17017,&P18246,&P18092 }};	/*Useragent - TELSDA55_11B_HW (MRE\2.3.00(2048) resolution\X chipset\MT6255 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) G68_LS_WIFI_TV_EN_X Release/2012.08.16 WAP Browser/MAUI Profile/  Q03C1-2.40 en-US*/
const Device UA2419109 = {S33846, 191, { &P18088,&P17017,&P18246,&P18092 }};	/*Useragent - TELSDA55_11B_HW (MRE\2.3.00(2048) resolution\X chipset\MT6255 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) G68_LS_EN_X Release/2012.08.06 WAP Browser/MAUI Profile/  Q03C1-2.40 en-US*/
const Device UA2443261 = {S33847, 16, { &P18088,&P17017,&P18246,&P18092 }};	/*Useragent - maui TRUSTED/1.0*/
const Device UA2598136 = {S33848, 26, { &P18088,&P17017,&P18246,&P18092 }};	/*Useragent - SYM_FT04i_MAUI WAP Browser*/
const Device UA2601044 = {S33849, 226, { &P18088,&P17017,&P18246,&P18092 }};	/*Useragent - CKT50_COSMOS_11B_HW (MRE\2.5.00(2500) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) SLT-S04A_UTL_L2EN_202_X Release/2013.01.06 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US*/
const Device UA2658771 = {S33850, 225, { &P18088,&P17017,&P18246,&P18092 }};	/*Useragent - TINNO50_6464_W1224_11B_HW (MRE\2.5.00(800) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\1 gsensor\0 keyboard\normal) F7013B_PP_F2_PK_V8 Release/2012.11.29 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US*/
const Device UA2767105 = {S33851, 394, { &P18088,&P17017,&P18246,&P18092 }};	/*Useragent - DEWAV50_COSMOS_SLIM_11B_HW (MRE/3.0.00(2500);MAUI/_DL32571_INTEX-PLAYER_L101;BDATE/2013/03/19 17:43;LCD/X;CHIP/MT6250;KEY/Reduced;TOUCH/1;CAMERA/0;SENSOR/0;DEV/DEWAV50_COSMOS_SLIM_11B_HW;WAP Browser/MAUI (HTTP PGDL;HTTPS);GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/005;) L3257_DL32571_INTEX-PLAYER_L101 Release/2013.03.19 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US*/
const Device UA2769683 = {S33852, 226, { &P18088,&P17017,&P18246,&P18092 }};	/*Useragent - CKT50_COSMOS_11B_HW (MRE\2.5.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) SLT-S04A_UTL_L2EN_206_X Release/2013.04.10 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US*/
const Device UA243526 = {S33853, 101, { &P15851,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e)*/
const Device UA247409 = {S33854, 98, { &P15851,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar 4G)*/
const Device UA250779 = {S33855, 103, { &P15851,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar; Orange)*/
const Device UA364185 = {S33856, 114, { &P15851,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e; 2.01.161.03)*/
const Device UA625489 = {S33857, 95, { &P15851,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar)*/
const Device UA636518 = {S33858, 114, { &P15851,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e; 2.05.161.01)*/
const Device UA639840 = {S33859, 114, { &P15851,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e; 1.08.164.02)*/
const Device UA667019 = {S33860, 114, { &P15851,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e; 2.01.164.03)*/
const Device UA697810 = {S33861, 114, { &P15851,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e; 1.08.161.01)*/
const Device UA718072 = {S33862, 114, { &P15851,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e; 1.08.172.01)*/
const Device UA925827 = {S33863, 114, { &P15851,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e; 4.06.164.01)*/
const Device UA925866 = {S33864, 114, { &P15851,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e; 4.06.166.01)*/
const Device UA925879 = {S33865, 114, { &P15851,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e; 4.06.172.01)*/
const Device UA925889 = {S33866, 114, { &P15851,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e; 4.06.161.02)*/
const Device UA2418370 = {S33867, 114, { &P15851,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radar C110e; 4.06.168.01)*/
const Device UA98533 = {S33868, 119, { &P17595,&P17294,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.18*/
const Device UA101400 = {S33869, 117, { &P17595,&P17294,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13*/
const Device UA127183 = {S33870, 119, { &P17595,&P17294,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.22*/
const Device UA431148 = {S33871, 121, { &P17595,&P17294,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_7; en-us) AppleWebKit/526.1+ (KHTML, like Gecko) Version/3.1 Safari/525.13*/
const Device UA12635 = {S33872, 154, { &P13163,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaN97-1/22.0.110; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) BrowserNG/7.1.4*/
const Device UA26717 = {S33873, 152, { &P13163,&P5129,&P3690,&P18092 }};	/*Useragent - NokiaN97/22.1.112 (SymbianOS/9.4; Series60/5.0 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebkit/525 (KHTML, like Gecko) BrowserNG/7.1.4*/
const Device UA33470 = {S33874, 171, { &P13163,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaN97-4/30.0.004; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.28 3gpp-gba*/
const Device UA46099 = {S33875, 171, { &P13163,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaN97-5/30.2.004; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.28 3gpp-gba*/
const Device UA49141 = {S33876, 153, { &P13163,&P5129,&P3690,&P18092 }};	/*Useragent - NokiaN97i/22.0.110 (SymbianOS/9.4; Series60/5.0 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebkit/525 (KHTML, like Gecko) BrowserNG/7.1.4*/
const Device UA51229 = {S33877, 74, { &P13163,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Nokia Podcasting; SymbianOS; NokiaN97-1/22.0.110)*/
const Device UA51382 = {S33878, 169, { &P13163,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaN97-1/22.0.110; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) NokiaBrowser/7.3.1.25 3gpp-gba*/
const Device UA64228 = {S33879, 154, { &P13163,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaN97-5/11.2.102; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) BrowserNG/7.1.4*/
const Device UA64229 = {S33880, 154, { &P13163,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaN97-3/22.2.110; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) BrowserNG/7.1.4*/
const Device UA64826 = {S33881, 154, { &P13163,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaN97-4/12.0.110; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) BrowserNG/7.1.4*/
const Device UA69526 = {S33882, 8, { &P13163,&P5129,&P3690,&P18092 }};	/*Useragent - NokiaN97*/
const Device UA2437380 = {S33883, 103, { &P15364,&P7480,&P17260,&P18090 }};	/*Useragent - Mozilla/5.0 (compatible; Yahoo! Slurp/3.0; http://help.yahoo.com/help/us/ysearch/slurp) NOT Firefox/3.5*/
const Device UA2468369 = {S33884, 142, { &P15364,&P7480,&P17260,&P18090 }};	/*Useragent - Mozilla/5.0 (compatible; Yahoo! Slurp/3.0; http://help.yahoo.com/help/us/ysearch/slurp) NOT Firefox/3.5,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA97901 = {S33885, 46, { &P15364,&P17350,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows NT)*/
const Device UA118329 = {S33886, 41, { &P15364,&P17350,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Win32)*/
const Device UA125120 = {S33887, 97, { &P15364,&P17350,&P17387,&P18092 }};	/*Useragent - LeapTag (compatible; Mozilla 4.0; MSIE 5.5; http://beta.leaptag.com/?p=win32&v=0.8.4.trunk.r5774)*/
const Device UA503676 = {S33888, 77, { &P15364,&P17350,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.50; Windows NT; SiteKiosk 4.8; SiteCoach 1.0)*/
const Device UA653657 = {S33889, 46, { &P15364,&P17350,&P17387,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.5; Windows NT)*/
const Device UA2741750 = {S33890, 43, { &P15364,&P17350,&P17387,&P18092 }};	/*Useragent - Mozilla/5.0(compatible;MSIE 5.5;Windows NT)*/
const Device UA398155 = {S33891, 154, { &P16384,&P17342,&P17454,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5380K/S5380KDDKK6; U; Bada/2.0; en-us) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile HVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA753653 = {S33892, 146, { &P16384,&P17342,&P17454,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5380B/1.0; U; Bada/2.0; pt-br) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile HVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA864432 = {S33893, 152, { &P16384,&P17342,&P17454,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5380/S5380AELB3; U; Bada/2.0; fr-fr) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile HVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA911551 = {S33894, 154, { &P16384,&P17342,&P17454,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5380D/S5380DGTLE1; U; Bada/2.0; en-us) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile HVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA1957320 = {S33895, 143, { &P16384,&P17342,&P17454,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5380L/1.0; U; Bada/2.0; es) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile HVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA2891911 = {S33896, 57, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.16*/
const Device UA2900683 = {S33897, 64, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; WOW64) Presto/2.12.388 Version/12.16*/
const Device UA2965372 = {S33898, 80, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.16*/
const Device UA2965419 = {S33899, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.16*/
const Device UA2966622 = {S33900, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.16*/
const Device UA2966628 = {S33901, 67, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.16,gzip(gfe)*/
const Device UA2967508 = {S33902, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.16*/
const Device UA2967511 = {S33903, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.16*/
const Device UA3012806 = {S33904, 116, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)) Presto/2.12.388 Version/12.16*/
const Device UA3013556 = {S33905, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.16*/
const Device UA3014486 = {S33906, 80, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.16*/
const Device UA3015100 = {S33907, 68, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:2.0) Gecko/X Firefox/4.0 Opera 12.16*/
const Device UA3016190 = {S33908, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.1 (build 6639)) Presto/2.12.388 Version/12.16*/
const Device UA3016871 = {S33909, 80, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.16*/
const Device UA3017037 = {S33910, 80, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.16*/
const Device UA3017165 = {S33911, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.16*/
const Device UA3017434 = {S33912, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.16*/
const Device UA3017561 = {S33913, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.16*/
const Device UA3018080 = {S33914, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6019)) Presto/2.12.388 Version/12.16*/
const Device UA3018103 = {S33915, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.16*/
const Device UA3018368 = {S33916, 80, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.16*/
const Device UA3018605 = {S33917, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6017)) Presto/2.12.388 Version/12.16*/
const Device UA3018769 = {S33918, 80, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.16*/
const Device UA3056942 = {S33919, 88, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; Edition Campaign 21) Presto/2.12.388 Version/12.16,gzip(gfe)*/
const Device UA3056991 = {S33920, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.16*/
const Device UA3056998 = {S33921, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.16*/
const Device UA3057137 = {S33922, 80, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.16*/
const Device UA3057153 = {S33923, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.16*/
const Device UA3057168 = {S33924, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.1 (build 6337)) Presto/2.12.388 Version/12.16*/
const Device UA3057225 = {S33925, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5999)) Presto/2.12.388 Version/12.16*/
const Device UA3057319 = {S33926, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.16*/
const Device UA3058286 = {S33927, 77, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.16,gzip(gfe),gzip(gfe)*/
const Device UA3060349 = {S33928, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.1 (build 6683)) Presto/2.12.388 Version/12.16*/
const Device UA3077720 = {S33929, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.16*/
const Device UA3077759 = {S33930, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.16*/
const Device UA3078160 = {S33931, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.16*/
const Device UA3078375 = {S33932, 106, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.16*/
const Device UA3078418 = {S33933, 62, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0) Opera 12.16*/
const Device UA3078433 = {S33934, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.16*/
const Device UA3091732 = {S33935, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.16*/
const Device UA3091772 = {S33936, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.16*/
const Device UA3091931 = {S33937, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.16*/
const Device UA3106777 = {S33938, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.16*/
const Device UA3106961 = {S33939, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.16*/
const Device UA3107077 = {S33940, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.16*/
const Device UA3107157 = {S33941, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.16*/
const Device UA3124521 = {S33942, 80, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.16*/
const Device UA3124664 = {S33943, 79, { &P15364,&P17285,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.16*/
const Device UA2388411 = {S33944, 58, { &P15364,&P16110,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; rv:18.0) Gecko/X Firefox/18.0*/
const Device UA2471006 = {S33945, 70, { &P15364,&P16110,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:18.0) Gecko/X Firefox/18.0*/
const Device UA256763 = {S33946, 70, { &P14913,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SPH-M820-BST Build/GINGERBREAD)*/
const Device UA596772 = {S33947, 70, { &P14913,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SPH-M820-BST Build/GINGERBREAD)*/
const Device UA623087 = {S33948, 146, { &P14913,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623913 = {S33949, 146, { &P14913,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SPH-M820-BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA644062 = {S33950, 147, { &P14913,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SPH-M820-BST Build/Chicken Tits) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA750440 = {S33951, 144, { &P14913,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; -us; SPH-M820-BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA869542 = {S33952, 159, { &P14913,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/Chicken Tits by Bloodawn) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1937212 = {S33953, 144, { &P14913,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; -us; SPH-M820-BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1963186 = {S33954, 159, { &P14913,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/Chicken-Tits by Bloodawn) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1970918 = {S33955, 147, { &P14913,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/Chicken Tits) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1986541 = {S33956, 145, { &P14913,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SPH-M820-BST Build/HONEYBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029382 = {S33957, 153, { &P14913,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2406815 = {S33958, 150, { &P14913,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/ROM by Bloodawn) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2589477 = {S33959, 150, { &P14913,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/Beats by Dr.Dre) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA924948 = {S33960, 62, { &P15364,&P18110,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:17.0) Gecko/17.0 Firefox/17.0*/
const Device UA3096409 = {S33961, 78, { &P15364,&P18110,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:17.0) Gecko/X Firefox/17.0 Iceweasel/17.0.9*/
const Device UA370813 = {S33962, 157, { &P13054,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; HTC Mecha Build/TeamVicious-MIUI-ICECREAM) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA616688 = {S33963, 144, { &P13054,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6425LVW 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA655786 = {S33964, 144, { &P13054,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA685392 = {S33965, 141, { &P13054,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA694203 = {S33966, 141, { &P13054,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6425LVW Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899047 = {S33967, 141, { &P13054,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899415 = {S33968, 144, { &P13054,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1948356 = {S33969, 141, { &P13054,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; ADR6425LVW Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1973551 = {S33970, 63, { &P13054,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; ADR6425LVW Build/IML74K)*/
const Device UA2025128 = {S33971, 63, { &P13054,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ADR6425LVW Build/IMM76D)*/
const Device UA2031636 = {S33972, 144, { &P13054,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6425LVW 4G Build/IMM30B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2185746 = {S33973, 145, { &P13054,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW 4G Build/Rezound) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2235580 = {S33974, 147, { &P13054,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30*/
const Device UA2257277 = {S33975, 142, { &P13054,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW Build/Rezound) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2499333 = {S33976, 141, { &P13054,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6425LVW Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2507648 = {S33977, 155, { &P13054,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW Build/EclipticRez-Revamped) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2508034 = {S33978, 158, { &P13054,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW 4G Build/EclipticRez-Revamped) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2685781 = {S33979, 139, { &P13054,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; ADR6425LVW Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2688828 = {S33980, 142, { &P13054,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; -us; ADR6425LVW 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2217845 = {S33981, 44, { &P15364,&P7133,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:11.0)*/
const Device UA2349591 = {S33982, 57, { &P15364,&P7133,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; rv:11.0) Gecko/X*/
const Device UA2795765 = {S33983, 68, { &P15364,&P7133,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko*/
const Device UA2869166 = {S33984, 73, { &P15364,&P7133,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; Trident/7.0; rv:11.0) like Gecko*/
const Device UA3023635 = {S33985, 50, { &P15364,&P7133,&P21461,&P18092 }};	/*Useragent - Windows-RSS-Platform/2.0 (IE 11.0; Windows NT 6.1)*/
const Device UA3092345 = {S33986, 90, { &P15364,&P7133,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; Touch; Tablet PC 2.0; rv:11.0) like Gecko*/
const Device UA3125236 = {S33987, 97, { &P15364,&P7133,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (AOL 9.7; AOLBuild 4343.1028; Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko*/
const Device UA597039 = {S33988, 165, { &P13070,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-E739 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/V1.2*/
const Device UA616152 = {S33989, 60, { &P13070,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-E739 Build/GRK39F)*/
const Device UA622922 = {S33990, 164, { &P13070,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-E739 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/V1.2*/
const Device UA625014 = {S33991, 59, { &P13070,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-E739 Build/GRJ22)*/
const Device UA691172 = {S33992, 163, { &P13070,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-E739Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/V1.2*/
const Device UA697104 = {S33993, 163, { &P13070,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;LG-E739 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/V1.2*/
const Device UA722618 = {S33994, 164, { &P13070,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-E739Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/V1.2*/
const Device UA725958 = {S33995, 163, { &P13070,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; LG-E739 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/V1.2*/
const Device UA766102 = {S33996, 164, { &P13070,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us;LG-E739 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/V1.2*/
const Device UA773252 = {S33997, 163, { &P13070,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; LG-E739 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/V1.2*/
const Device UA774279 = {S33998, 163, { &P13070,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-E739 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 MMS/LG-Android-MMS-V1.0/V1.2*/
const Device UA808064 = {S33999, 164, { &P13070,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.6; en-us; LG-E739 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/V1.2*/
const Device UA825384 = {S34000, 164, { &P13070,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; LG-E739 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/V1.2*/
const Device UA842810 = {S34001, 164, { &P13070,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-E739 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 MMS/LG-Android-MMS-V1.0/V1.2*/
const Device UA2933188 = {S34002, 135, { &P13070,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; LG-E739 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Versi?n/4.0 Mobile Safari/533.1*/
const Device UA2933296 = {S34003, 136, { &P13070,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; LG-E739 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Versi?n/4.0 Mobile Safari/533.1*/
const Device UA2985162 = {S34004, 151, { &P13070,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-E739 Build/GRJ22; JellyBred-v1.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2888666 = {S34005, 135, { &P18635,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36*/
const Device UA2974071 = {S34006, 134, { &P18635,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Nexus 4 Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975007 = {S34007, 136, { &P18635,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 4 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975581 = {S34008, 128, { &P18635,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2715816 = {S34009, 92, { &P15364,&P7133,&P22774,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SlimBrowser/7.00.024)*/
const Device UA2718723 = {S34010, 85, { &P15364,&P7133,&P22774,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SlimBrowser/7.00.024)*/
const Device UA2796941 = {S34011, 93, { &P15364,&P7133,&P22774,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SlimBrowser/7.00.035)*/
const Device UA2796946 = {S34012, 86, { &P15364,&P7133,&P22774,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; SlimBrowser/7.00.035)*/
const Device UA2865757 = {S34013, 100, { &P15364,&P7133,&P22774,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Avant Browser; SlimBrowser/7.00.024)*/
const Device UA2875571 = {S34014, 108, { &P15364,&P7133,&P22774,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Avant Browser; SlimBrowser/7.00.035)*/
const Device UA2959174 = {S34015, 101, { &P15364,&P7133,&P22774,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Avant Browser; SlimBrowser/7.00.035)*/
const Device UA2959401 = {S34016, 107, { &P15364,&P7133,&P22774,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Avant Browser; SlimBrowser/7.00.035)*/
const Device UA3010120 = {S34017, 92, { &P15364,&P7133,&P22774,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SlimBrowser/7.00.035)*/
const Device UA2945949 = {S34018, 80, { &P18057,&P5567,&P17371,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/5.1/27.2338; U; en) Presto/2.8.119 400X240 LG VN271*/
const Device UA2538136 = {S34019, 69, { &P15364,&P18118,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:22.0) Gecko/X Firefox/22.0*/
const Device UA582504 = {S34020, 140, { &P14854,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; SHW-M250K Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616749 = {S34021, 140, { &P14854,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; SHW-M250S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA628892 = {S34022, 62, { &P14854,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SHW-M250S Build/IML74K)*/
const Device UA738997 = {S34023, 140, { &P14854,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; SHW-M250L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1962781 = {S34024, 62, { &P14854,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SHW-M250L Build/IML74K)*/
const Device UA1968698 = {S34025, 62, { &P14854,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SHW-M250K Build/IML74K)*/
const Device UA2002920 = {S34026, 140, { &P14854,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHW-M250S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027957 = {S34027, 140, { &P14854,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHW-M250K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030282 = {S34028, 62, { &P14854,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHW-M250S Build/IMM76D)*/
const Device UA2030994 = {S34029, 140, { &P14854,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHW-M250L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2230761 = {S34030, 62, { &P14854,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHW-M250K Build/IMM76D)*/
const Device UA2449787 = {S34031, 62, { &P14854,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHW-M250L Build/IMM76D)*/
const Device UA3042571 = {S34032, 145, { &P14854,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; SHW-M250S Build/LPNAN_1.3.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2362952 = {S34033, 130, { &P18639,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Nexus 10 Build/JVP15P) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2391597 = {S34034, 132, { &P18639,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 10 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2515537 = {S34035, 131, { &P18639,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 10 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2708114 = {S34036, 137, { &P18639,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; en-US; Nexus 10 Build/JOP40D) AppleWebKit/535.19 (KHTML like Gecko) Chrome/18.0.1025.166 Safari/535.19*/
const Device UA2566443 = {S34037, 136, { &P18635,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566645 = {S34038, 129, { &P18635,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566820 = {S34039, 137, { &P18635,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 4 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567225 = {S34040, 135, { &P18635,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Nexus 4 Build/JOP40C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602446 = {S34041, 130, { &P18635,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 4 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2485704 = {S34042, 52, { &P23144,&P18580,&P17596,&P18092 }};	/*Useragent - MobileSafari/8536.25 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2387453 = {S34043, 74, { &P17595,&P17296,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:18.0) Gecko/X Firefox/18.0*/
const Device UA1954443 = {S34044, 129, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M1001 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1975589 = {S34045, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; QW TB-1207 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1985125 = {S34046, 143, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SXZ-PDX0-01 Build/SXZ-PDX0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1992756 = {S34047, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; DFP7005 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1995035 = {S34048, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VegaCream RC2 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2025343 = {S34049, 140, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VegaCream Beta 3 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2025782 = {S34050, 140, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Wxt Tab PC 65CXi Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2026146 = {S34051, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A721 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2026928 = {S34052, 128, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M805 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2027177 = {S34053, 129, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; CT802 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2027354 = {S34054, 131, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-7200 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2027424 = {S34055, 127, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M11 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2028092 = {S34056, 128, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; 97FC Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2028263 = {S34057, 140, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Android B1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2028264 = {S34058, 129, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M1001 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2028359 = {S34059, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A710 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2028487 = {S34060, 129, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; XELIO Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2028681 = {S34061, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Play Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2028800 = {S34062, 129, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M1005 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2029241 = {S34063, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MP907C Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2029289 = {S34064, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TC-971 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2030267 = {S34065, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID970 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2030622 = {S34066, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; QW TB-1217 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2032179 = {S34067, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; DFP7005B Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2032450 = {S34068, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; vivow Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2033616 = {S34069, 128, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; EPAD Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2222033 = {S34070, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BC1077 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2228403 = {S34071, 128, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 101T Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2228458 = {S34072, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TC-7120 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2229172 = {S34073, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; cyclone Titan Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2261472 = {S34074, 142, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ALLVIEWSPEED Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261506 = {S34075, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JW101A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2261531 = {S34076, 129, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VIG07 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2261582 = {S34077, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; f7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261583 = {S34078, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; F721 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2314082 = {S34079, 139, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID-WCDMA Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2326439 = {S34080, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID970 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2329618 = {S34081, 127, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M91 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2338663 = {S34082, 128, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M805 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2338733 = {S34083, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID9107CM Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2339339 = {S34084, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A088 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2340117 = {S34085, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Ray Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2344004 = {S34086, 128, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 930i Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2344183 = {S34087, 129, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PC936 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2344271 = {S34088, 131, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TC-973D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2345482 = {S34089, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID900SC Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2345539 = {S34090, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N900 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2411005 = {S34091, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; QW TB-1210 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2413462 = {S34092, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TAB-9701 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2413638 = {S34093, 139, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SX-SP435A Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2414635 = {S34094, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TC-971 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2415553 = {S34095, 128, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MiSS Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2416985 = {S34096, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CW V8 Dual Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2417710 = {S34097, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2419233 = {S34098, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M1005D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2419456 = {S34099, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID-001 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2421345 = {S34100, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Q8-A13 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2422355 = {S34101, 142, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BASE_Lutea_3 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2422470 = {S34102, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; 070V2 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2424477 = {S34103, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GOTAB Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2425046 = {S34104, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TOUCH90B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2426093 = {S34105, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M9701THW Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2427501 = {S34106, 140, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST9 Marvel Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2428123 = {S34107, 139, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; maxtouuch Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2428730 = {S34108, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; UM-A91 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2442885 = {S34109, 128, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; 3C97 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2445442 = {S34110, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TAB971 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2445769 = {S34111, 128, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 3C97 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2449924 = {S34112, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Alpha GT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2452438 = {S34113, 142, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Smartab MT12 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2452850 = {S34114, 144, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Milagrow Tab Top 7.4 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2454602 = {S34115, 131, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID970T Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2455000 = {S34116, 143, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Exagerate XZPAD970H Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2455310 = {S34117, 139, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CRANE EVB Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2455577 = {S34118, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Q29 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2455605 = {S34119, 146, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KN_MOBILE_QT_A06 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2455707 = {S34120, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 9700DN Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2456584 = {S34121, 146, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; INFINITY_PASSION Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2457698 = {S34122, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMURAI Build/SAMURAI) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2457929 = {S34123, 140, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ELEMENT7V2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2458050 = {S34124, 131, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M9702TH Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2458705 = {S34125, 129, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CT710 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2465176 = {S34126, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M901 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2465535 = {S34127, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Q8 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2465743 = {S34128, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ASTRO+9.7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2466490 = {S34129, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; QWTB-1280BL Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2466979 = {S34130, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N9977 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2467349 = {S34131, 131, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CT710FR Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2467398 = {S34132, 129, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AW101 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2467493 = {S34133, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; disgo9000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2467835 = {S34134, 126, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; G1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2468633 = {S34135, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; UM-A90 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2476553 = {S34136, 142, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Broncho N802 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2476817 = {S34137, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MIRA-2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2477523 = {S34138, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID-GSM Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2477817 = {S34139, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2478214 = {S34140, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A750 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2478307 = {S34141, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Acqua Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2478430 = {S34142, 129, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Pacha Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2479456 = {S34143, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; J1000 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2480028 = {S34144, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JSA-MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2480281 = {S34145, 144, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Polaroid 40140 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2480636 = {S34146, 131, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID1107 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2481672 = {S34147, 141, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PICOpad_GEA Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2481678 = {S34148, 143, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MTC Viva Build/HuaweiU8816) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2481736 = {S34149, 167, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; E-Boda Supreme X80 Dual Core Build/ICS.g12refM806A1YBD.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2481829 = {S34150, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Q75 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2481896 = {S34151, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Swipe AllinOne Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2482058 = {S34152, 142, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; yuanda75_ics Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2482309 = {S34153, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Q88 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2482507 = {S34154, 148, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Exagerate XZPAD700 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2482661 = {S34155, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; 740RS Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2483477 = {S34156, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TPC0752 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2484432 = {S34157, 131, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAD1001 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2485667 = {S34158, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB 170 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2486980 = {S34159, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID7040T-3G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2487247 = {S34160, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AN13FP Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2497163 = {S34161, 129, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M971B Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2499558 = {S34162, 143, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; treq A10G Duo Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2499767 = {S34163, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Z7H Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2500799 = {S34164, 128, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PC728 Build/PC728) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2501508 = {S34165, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Tablet PC 2000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2501586 = {S34166, 142, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Touch Kem430 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2502284 = {S34167, 139, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; RYOMA M1S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2502516 = {S34168, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; X12 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2502525 = {S34169, 139, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WOHTC G36 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2502572 = {S34170, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MING_SII Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2502603 = {S34171, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V12 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2502583 = {S34172, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; P10MAN R2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2502791 = {S34173, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i10 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2502904 = {S34174, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A21plus Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2503143 = {S34175, 128, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 101D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2503345 = {S34176, 146, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Gadmei ICS Build/E8HD-T364-1280x768) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2503754 = {S34177, 143, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SP-A20i Build/MF_ICS_02.24) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2504100 = {S34178, 139, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JXD-S6600 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2504303 = {S34179, 175, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M972w Build/M972w-eng 4.0.4 IMM76D test-keys_OTA:MID_2.1.1.1_X.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2504310 = {S34180, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M9702T Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2504408 = {S34181, 146, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Tablet-PC-4 Build/ICS.g08refem618.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2504506 = {S34182, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MD-0696B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2504662 = {S34183, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAD 1001 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2504734 = {S34184, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; F1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2504967 = {S34185, 129, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V800H Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2504987 = {S34186, 144, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; iliumPAD Build/GS.NN2R3N.421SGL.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2505056 = {S34187, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; myTab 7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2505233 = {S34188, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Q88B-5S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2505355 = {S34189, 149, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V9-1688-CJ-0308-31S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2505468 = {S34190, 140, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Netbook PC 12700 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2505676 = {S34191, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; P6 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2506030 = {S34192, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TOUCH78 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2506648 = {S34193, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KY-901 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2519891 = {S34194, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GN868 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2520248 = {S34195, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; P077A Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2520629 = {S34196, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; My Audio Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2521951 = {S34197, 139, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AMOI N820 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2524427 = {S34198, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MAG101 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2524532 = {S34199, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; V12 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2524795 = {S34200, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; B928A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2525131 = {S34201, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Z5 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2526546 = {S34202, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPX_8 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2527068 = {S34203, 150, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  Artes D708 Build/ICS.g12refM708A1AST.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2527436 = {S34204, 140, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; DL HD-72 X Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2527591 = {S34205, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NEO-G4 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2527644 = {S34206, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Q38 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2527683 = {S34207, 128, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; NT-2750 Build/MID) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2527815 = {S34208, 131, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MW0821+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2527825 = {S34209, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JY139 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2527796 = {S34210, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; I5 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2527887 = {S34211, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; J-Q8B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2527907 = {S34212, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; OVO 2.0 Build/FR-M768) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2528103 = {S34213, 143, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; W27(Dualcore) Build/ICS.w27ref.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2528409 = {S34214, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; POM805HCF Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2528562 = {S34215, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T271 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2529261 = {S34216, 159, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V8200-Memphis-orange-Slovakia Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2529572 = {S34217, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TLF TPC701 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2529754 = {S34218, 129, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V100D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2529918 = {S34219, 152, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHIRU SHOGUN(QuadCore) Build/SHIRU_SHOGUN) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2530670 = {S34220, 143, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tmn smart a18 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2535017 = {S34221, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BT210 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2545240 = {S34222, 131, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID-A10 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2545651 = {S34223, 128, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; 9701 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2549500 = {S34224, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; aigopad Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2551560 = {S34225, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; mfld_dv10 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2553317 = {S34226, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; iB750C Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2554220 = {S34227, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SIROCO 3 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2554410 = {S34228, 143, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ratech77_ics2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2556376 = {S34229, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SXZ-PDX0-05M Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2556577 = {S34230, 131, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LXT1001 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2557012 = {S34231, 131, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; myTab10 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2557526 = {S34232, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Planet II v2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2558162 = {S34233, 142, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Royqueen 702 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2558179 = {S34234, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Ezcool Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2558404 = {S34235, 147, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 1688-CJ-C30Q8-31S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2559305 = {S34236, 139, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Explosion Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2574695 = {S34237, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N70 DUAL CORE Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2578389 = {S34238, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JT-Smart PC01 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2578420 = {S34239, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; J-88 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2578506 = {S34240, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; G1 SKY Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2580039 = {S34241, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M9100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2589373 = {S34242, 139, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SZENIO IPS 9700 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2592709 = {S34243, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Mini Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2593496 = {S34244, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; RMD-973G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2593834 = {S34245, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Z500 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2594016 = {S34246, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; DFP7006 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2594639 = {S34247, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N10-4core Build/R13.4-rc2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2595111 = {S34248, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TZ100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2595547 = {S34249, 139, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MTC_968 Build/ICECREAM) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2596083 = {S34250, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GNET Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2596175 = {S34251, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Eliya S6 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2596632 = {S34252, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S5A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2596729 = {S34253, 143, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; hexing77_ics2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2596920 = {S34254, 129, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V800A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2597859 = {S34255, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PICOpad GJT Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2644267 = {S34256, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IS801C Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2644403 = {S34257, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; X2 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2647560 = {S34258, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Telus HD Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2648411 = {S34259, 131, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WT-M768 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2651944 = {S34260, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CW-V9-DUAL Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2652133 = {S34261, 142, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GIONEE-GN700 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2655791 = {S34262, 128, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A106 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2655882 = {S34263, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A088 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2656407 = {S34264, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A721 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2656820 = {S34265, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; J-Q88D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2658441 = {S34266, 154, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MegaFon_SP-A10 Build/IceCreamSandwich) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2659387 = {S34267, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Alpha GT Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2660449 = {S34268, 142, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC PROMIN_U Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2662350 = {S34269, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; reedPAD4N Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2664875 = {S34270, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB 170N Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2668141 = {S34271, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; EG950 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2691569 = {S34272, 131, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GEM9212 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2693480 = {S34273, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T291 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2694428 = {S34274, 143, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; NB017 / NB018 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2696065 = {S34275, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Informer 921 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2696463 = {S34276, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V700N Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2698096 = {S34277, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M702VM Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2698442 = {S34278, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JXD-P300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2701618 = {S34279, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M5000 Build/1.0.528_X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2705142 = {S34280, 139, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB 0170N Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2705633 = {S34281, 131, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB 180 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2721551 = {S34282, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AllviewSpeed2+ Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2727267 = {S34283, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; monga M8 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2727781 = {S34284, 131, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; My play Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2728877 = {S34285, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB 0170 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2729131 = {S34286, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Z1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2729370 = {S34287, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NOVO NV7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2729874 = {S34288, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; I110 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2729965 = {S34289, 141, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; P102G Build/GS.NN2R3N.421S3D.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2730157 = {S34290, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; reedPAD4+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2733933 = {S34291, 131, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB 280 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2746065 = {S34292, 131, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; lt1002b Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2748031 = {S34293, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Quadro Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2749517 = {S34294, 144, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Condor Tab SMB-B9705 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2758359 = {S34295, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; T736B Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2761851 = {S34296, 139, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC POO_U Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2762293 = {S34297, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; XPRESS PRO Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2762656 = {S34298, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A10-4core Build/R13.4-rc2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2762812 = {S34299, 144, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NetTAB SPACE II Plus Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2762923 = {S34300, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; P4i Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2763031 = {S34301, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A930 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2763339 = {S34302, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Mio Pad A1001 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2765038 = {S34303, 143, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 1688-XHB-2357 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2765152 = {S34304, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C800 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2775653 = {S34305, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Neura 3G Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2778648 = {S34306, 139, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; EverGlory Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2779201 = {S34307, 128, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V71B Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2790727 = {S34308, 139, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; StarPAD 7 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2794702 = {S34309, 140, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HISENSE F0 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2796530 = {S34310, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; UM-A13B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2812527 = {S34311, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TB782B Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2814049 = {S34312, 128, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PC728 Build/PC728) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2815867 = {S34313, 139, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Herpa Pad c5900 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2815925 = {S34314, 131, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A10-MID Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2816118 = {S34315, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; EB-A71GJ Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2816943 = {S34316, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID971 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2817080 = {S34317, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; f7 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2818004 = {S34318, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; NVSBL DRAGON Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2820068 = {S34319, 139, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; JXD-S9100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2822148 = {S34320, 140, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; OCOCCI_MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2822338 = {S34321, 129, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID08 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2823679 = {S34322, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MB1003 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2823857 = {S34323, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID06N Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2824380 = {S34324, 128, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; I97D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2825388 = {S34325, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; P4 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2830420 = {S34326, 128, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; D101 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2830689 = {S34327, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HLV-T702 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2831056 = {S34328, 143, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S5/C2_77_5647 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2832208 = {S34329, 142, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GSmart G1342 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2832263 = {S34330, 129, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; 097Q2 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2833025 = {S34331, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TAB468 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2833901 = {S34332, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT685 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2834366 = {S34333, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M750 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2834597 = {S34334, 139, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Pentagram TAB80 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2836376 = {S34335, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; X18 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2837132 = {S34336, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; H8000 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2837138 = {S34337, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MW0922 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2838541 = {S34338, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HLV-T9701 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2839229 = {S34339, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AirTab P71g Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2839600 = {S34340, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALEXIS-RX5DC Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2839673 = {S34341, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; platinum Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2840248 = {S34342, 157, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Pad 7208HD Build/ICS.polypad.7208HD.Dualcore.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2843089 = {S34343, 129, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; novo7 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2844533 = {S34344, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DLR_MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2844606 = {S34345, 140, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C3_77_5647 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2845271 = {S34346, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TPC0952 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2845342 = {S34347, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SmartPadv10 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2845468 = {S34348, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TRIO7C Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2845717 = {S34349, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Tablet Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2848484 = {S34350, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S3 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2848688 = {S34351, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Q75S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2849012 = {S34352, 143, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NetTAB Pocket Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2849443 = {S34353, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MD-0651 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2854058 = {S34354, 128, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; mw07 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2855384 = {S34355, 139, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N8010_APT Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2892329 = {S34356, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HS-U909 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2892755 = {S34357, 142, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N8000_USA_RS Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2895730 = {S34358, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PHM727MC Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2934038 = {S34359, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ARIA Build/ICS.w22ref.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2936216 = {S34360, 129, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XM908 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2936853 = {S34361, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Swipe 9X Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2951862 = {S34362, 141, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PICOpad_GFI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2951911 = {S34363, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NE-103T Build/X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2956569 = {S34364, 143, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GEM7032G-rev2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2956837 = {S34365, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Quince Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2957759 = {S34366, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB 270 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2959599 = {S34367, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB 171 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2982178 = {S34368, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Penta IS907C Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2982848 = {S34369, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E140K Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2982999 = {S34370, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i7S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2983313 = {S34371, 130, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; 1040RS Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2983764 = {S34372, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; YPY_S400 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2984430 = {S34373, 128, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; NT-2755 Build/MID) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2984654 = {S34374, 142, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; amity 8GB-V2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2984699 = {S34375, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; F8G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2985400 = {S34376, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB 143 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2985925 = {S34377, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; W13PRO Build/ICS.g04wXref.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2991895 = {S34378, 144, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CA-201L Build/CA201LU325_00) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2998248 = {S34379, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BT230 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2999303 = {S34380, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NT-3902S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2999310 = {S34381, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; FLY Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2999779 = {S34382, 129, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T7040 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3006436 = {S34383, 140, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GR-TB10S Build/ICS.GR-TB10S.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3007773 = {S34384, 140, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KINGTOP915 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3009397 = {S34385, 140, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenco CARTAB-920 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3009966 = {S34386, 140, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KINGTOP977 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3016270 = {S34387, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Calipso4 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3019379 = {S34388, 137, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAP5430 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3043304 = {S34389, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T5500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3043844 = {S34390, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT97I-RK66WA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3044391 = {S34391, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; W76 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3044649 = {S34392, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HD II Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3045036 = {S34393, 142, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Philips W737 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3045226 = {S34394, 143, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; DL DroidTV DR-T71 Build/tablet-c) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3049758 = {S34395, 139, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Axxion ATAB-701 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3070512 = {S34396, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; monga M5 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3071778 = {S34397, 132, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB 0180 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3073824 = {S34398, 139, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DNS S3502 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3077675 = {S34399, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; W5510 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3085531 = {S34400, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LGL96G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3086196 = {S34401, 136, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Sygnus Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3088874 = {S34402, 143, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tmn smart a60 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3099952 = {S34403, 139, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IHA-C0707 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3100181 = {S34404, 135, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; W2500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3108300 = {S34405, 134, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; X35E Build/RheaTK) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3118812 = {S34406, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Sky Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3125341 = {S34407, 138, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NEC-NE-201A1A Build/X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3126573 = {S34408, 133, { &P15364,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C-4 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA221341 = {S34409, 138, { &P16321,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; X500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA624935 = {S34410, 136, { &P16321,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ZTE-X500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA632131 = {S34411, 62, { &P16321,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; X500 Build/GINGERBREAD)*/
const Device UA783709 = {S34412, 60, { &P16321,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; ZTE-X500 Build/FRG83)*/
const Device UA2423611 = {S34413, 145, { &P16321,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; X500_USA_General Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2807896 = {S34414, 136, { &P16321,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; X500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982447 = {S34415, 148, { &P16321,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; X500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, como Gecko) versi?n/4.0 Mobile Safari/533.1,gzip(gfe)*/
const Device UA912427 = {S34416, 119, { &P17595,&P18207,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA2017696 = {S34417, 119, { &P17595,&P18207,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA2039636 = {S34418, 119, { &P17595,&P18207,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA2558119 = {S34419, 119, { &P17595,&P18207,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA2597902 = {S34420, 116, { &P17595,&P18207,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1080) AppleWebKit/536.11 (KHTML like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA2781345 = {S34421, 119, { &P17595,&P18207,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA3097026 = {S34422, 119, { &P17595,&P18207,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA92878 = {S34423, 116, { &P15364,&P7133,&P17237,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/530.5 (KHTML, like Gecko) Chrome/2.0.172.43 Safari/530.5*/
const Device UA2181594 = {S34424, 103, { &P15364,&P7133,&P17237,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.10 (KHTML, like Gecko) Chrome/2.0.1.3 Safari/536.10*/
const Device UA2183528 = {S34425, 96, { &P15364,&P7133,&P17237,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.10 (KHTML, like Gecko) Chrome/2.0.1.3 Safari/536.10*/
const Device UA2350520 = {S34426, 103, { &P15364,&P7133,&P17237,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; en-US) AppleWebKit/528.8 (KHTML, like Gecko) Chrome/2.0.156.0 Safari/528.8*/
const Device UA2429428 = {S34427, 157, { &P15575,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9100 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2460342 = {S34428, 158, { &P15575,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9100G Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2514519 = {S34429, 157, { &P15575,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-I9100 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2537632 = {S34430, 156, { &P15575,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9100 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567435 = {S34431, 157, { &P15575,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9100G Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2678068 = {S34432, 157, { &P15575,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9100 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2731043 = {S34433, 158, { &P15575,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9100G Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2898414 = {S34434, 155, { &P15575,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9100 Build/4.75; CyanogenMod-4.75) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998289 = {S34435, 155, { &P15575,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9100 Build/4.60; CyanogenMod-4.60) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3011184 = {S34436, 160, { &P15575,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9100 Build/4.80) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/4.80/i9100*/
const Device UA3016685 = {S34437, 166, { &P15575,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9100G Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/i9100g*/
const Device UA3121181 = {S34438, 155, { &P15575,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9100 Build/4.50; CyanogenMod-4.50) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA49458 = {S34439, 129, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA567070 = {S34440, 133, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7300 Build/HONEYCOMB) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA575508 = {S34441, 130, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; th-th; GT-P7300B Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA643304 = {S34442, 58, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; GT-P7300 Build/HMJ37)*/
const Device UA734501 = {S34443, 148, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; cs-cz; SAMSUNG-GT-P7300/P7300BOKH5 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1902916 = {S34444, 136, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2311635 = {S34445, 127, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2608116 = {S34446, 127, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7300 Build/HMJ37) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2608616 = {S34447, 59, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; GT-P7300B Build/HMJ37)*/
const Device UA2609454 = {S34448, 122, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; fa-fa; GT-P7300 Build/HMJ37) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2609795 = {S34449, 150, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7300 Build/HMJ37; 1280*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2609975 = {S34450, 134, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; se-se; GT-P7300 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2612604 = {S34451, 148, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; hr-hr; SAMSUNG-GT-P7300/P7300BOKH4 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2615743 = {S34452, 64, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Vyrso/2.1.2 (Linux; U; Android 3.1; en-GB; GT-P7300 Build/HMJ37)*/
const Device UA2615987 = {S34453, 271, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; pt-br; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.10;FBDM/{density=1.0,width=800,height=1280};FBLC/pt_BR;FB_FW/2;FBCR/TIM;FBPN/com.facebook.katana;FBDV/GT-P7300;FBSV/3.1;]*/
const Device UA2617780 = {S34454, 230, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-tw; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 VoiceGO!(cybb6c19fd1eXfcdbce5cdddb0245-0;Android,3.1,GT-P7300;3.6.0;1280x752,1.0;X;VoiceGo-TC;zh-TW)*/
const Device UA2617974 = {S34455, 118, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7300) AntivirusAM/2.5 (KHTML, like Gecko) Version/Version 2.5/Antivirus*/
const Device UA2618937 = {S34456, 71, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Biblia/2.2.3 (Linux; U; Android 3.1; es-US; GT-P7300 Build/HMJ37.UTKH6)*/
const Device UA2619299 = {S34457, 268, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.11;FBDM/{density=1.0,width=1280,height=800};FBLC/en_US;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P7300;FBSV/3.1;]*/
const Device UA2619706 = {S34458, 168, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7300 Build/HMJ37) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (innerActive-ANDROID-2061)*/
const Device UA2619925 = {S34459, 71, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Biblia/2.2.3 (Linux; U; Android 3.1; es-ES; GT-P7300 Build/HMJ37.UTKI2)*/
const Device UA2620754 = {S34460, 92, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 3.1; SAMSUNG GT-P7300 Build/HONEYCOMB_MR1);*/
const Device UA2623790 = {S34461, 77, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - MediaPlayer/2002/4.0.12 (Android 3.1; GT-P7300; GT-P7300; samsung; ; ; en_GB)*/
const Device UA2624386 = {S34462, 78, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - VKAndroidApp/2.7.6-64 (Android 3.1; SDK 12; armeabi-v7a; samsung GT-P7300; ru)*/
const Device UA2624564 = {S34463, 67, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 3.1; en-US; GT-P7300B Build/HMJ37)*/
const Device UA2624602 = {S34464, 128, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; fr-fr;GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2631584 = {S34465, 128, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 3.1; fr-fr; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2632807 = {S34466, 278, { &P12889,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; th-th; GT-P7300B Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/2.2;FBBV/X;FBDM/{density=1.0,width=800,height=1280};FBLC/th_TH;FB_FW/2;FBCR/DTAC;FBPN/com.facebook.katana;FBDV/GT-P7300B;FBSV/3.1;]*/
const Device UA2739897 = {S34467, 70, { &P15364,&P7133,&P22772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:24.0) Gecko/X Firefox/24.0*/
const Device UA3066943 = {S34468, 88, { &P15364,&P7133,&P22772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:24.0) Gecko/X Firefox/24.0 AlexaToolbar/alxf-2.19*/
const Device UA3095669 = {S34469, 86, { &P15364,&P7133,&P22772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:24.0) Gecko/X Firefox/24.0 PaleMoon/24.0.1*/
const Device UA3096350 = {S34470, 81, { &P15364,&P7133,&P22772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:24.0) Gecko/X Firefox/24.0 PaleMoon/24.0.1*/
const Device UA2240800 = {S34471, 135, { &P18635,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 4 Build/JOP14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2240801 = {S34472, 136, { &P18635,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 4 Build/JVP15D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2240802 = {S34473, 136, { &P18635,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 4 Build/JVP15B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2240803 = {S34474, 136, { &P18635,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 4 Build/JOP19B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2240804 = {S34475, 135, { &P18635,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 4 Build/JOP22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2240805 = {S34476, 136, { &P18635,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 4 Build/JOP24B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2240806 = {S34477, 136, { &P18635,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 4 Build/JOP24G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2240817 = {S34478, 128, { &P18635,&P18598,&P19802,&P18092 }};	/*Useragent - Currents/2.0.0 (Linux; U; Android 4.2; en-us; phone/Nexus 4 Build/JOP14; Density/320; gzip) com.google.android.apps.currents/234*/
const Device UA2362351 = {S34479, 136, { &P18635,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 4 Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2387832 = {S34480, 58, { &P18635,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2; Nexus 4 Build/JOP40C)*/
const Device UA2391181 = {S34481, 138, { &P18635,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 4 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2405366 = {S34482, 60, { &P18635,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; Nexus 4 Build/JOP40D)*/
const Device UA2509948 = {S34483, 137, { &P18635,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2511518 = {S34484, 138, { &P18635,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 4 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2513989 = {S34485, 137, { &P18635,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2516462 = {S34486, 59, { &P18635,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; Nexus 4 Build/JDQ39)*/
const Device UA2662841 = {S34487, 135, { &P18635,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-; Nexus 4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2707543 = {S34488, 138, { &P18635,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2429918 = {S34489, 61, { &P18283,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-I9300 Build/JOP40D)*/
const Device UA2459861 = {S34490, 139, { &P18283,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-I9300 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2460053 = {S34491, 139, { &P18283,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-I535 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467770 = {S34492, 139, { &P18283,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-T999 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2515531 = {S34493, 139, { &P18283,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-I9300 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2537618 = {S34494, 138, { &P18283,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I535 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2537630 = {S34495, 60, { &P18283,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9300 Build/JDQ39)*/
const Device UA2537696 = {S34496, 139, { &P18283,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-I535 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2539853 = {S34497, 138, { &P18283,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2561461 = {S34498, 138, { &P18283,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T999 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2655536 = {S34499, 139, { &P18283,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; GT-i9300 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2714182 = {S34500, 139, { &P18283,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I535 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2717580 = {S34501, 139, { &P18283,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752428 = {S34502, 61, { &P18283,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SCH-I535 Build/JDQ39E)*/
const Device UA2768825 = {S34503, 139, { &P18283,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2776853 = {S34504, 139, { &P18283,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781519 = {S34505, 61, { &P18283,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGH-T999 Build/JDQ39E)*/
const Device UA2845183 = {S34506, 61, { &P18283,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; SCH-I535 Build/JOP40D)*/
const Device UA2889488 = {S34507, 139, { &P18283,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; it-it; GT-i9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892897 = {S34508, 131, { &P18283,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9300 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2985625 = {S34509, 61, { &P18283,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; SGH-T999 Build/JOP40D)*/
const Device UA2739863 = {S34510, 65, { &P15364,&P16110,&P22772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:24.0) Gecko/X Firefox/24.0*/
const Device UA2740031 = {S34511, 70, { &P15364,&P16110,&P22772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:24.0) Gecko/X Firefox/24.0*/
const Device UA3096295 = {S34512, 86, { &P15364,&P16110,&P22772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:24.0) Gecko/X Firefox/24.0 PaleMoon/24.0.1*/
const Device UA3096617 = {S34513, 81, { &P15364,&P16110,&P22772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:24.0) Gecko/X Firefox/24.0 PaleMoon/24.0.1*/
const Device UA403345 = {S34514, 101, { &P18218,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 610)*/
const Device UA896822 = {S34515, 101, { &P18218,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Nokia 610)*/
const Device UA898349 = {S34516, 111, { &P18218,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 610; Vodafone)*/
const Device UA925750 = {S34517, 115, { &P18218,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 610; 610 BOUYGUES)*/
const Device UA925795 = {S34518, 109, { &P18218,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 610; Orange)*/
const Device UA925797 = {S34519, 105, { &P18218,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 610 NFC)*/
const Device UA925799 = {S34520, 109, { &P18218,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 610; Meteor)*/
const Device UA925829 = {S34521, 112, { &P18218,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 610; NRJMobile)*/
const Device UA925885 = {S34522, 106, { &P18218,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 610; SFR)*/
const Device UA925924 = {S34523, 113, { &P18218,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 610 NFC; Orange)*/
const Device UA2463809 = {S34524, 102, { &P18218,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Nokia 610C)*/
const Device UA2730651 = {S34525, 97, { &P18218,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Nokia; Cliff)*/
const Device UA91534 = {S34526, 27, { &P15364,&P6729,&P17470,&P18092 }};	/*Useragent - Mozilla/3.01Gold (Win95; I)*/
const Device UA94064 = {S34527, 46, { &P15364,&P6729,&P17470,&P18092 }};	/*Useragent - Microsoft Internet Explorer/4.0b1 (Windows 95)*/
const Device UA94162 = {S34528, 27, { &P15364,&P6729,&P17470,&P18092 }};	/*Useragent - Mozilla/2.02Gold (Win95; I)*/
const Device UA94817 = {S34529, 71, { &P15364,&P6729,&P17470,&P18092 }};	/*Useragent - Mozilla/4.5 [en] (Win95; I)(compatible; Powermarks/3.0; Windows 95/NT4)*/
const Device UA99357 = {S34530, 63, { &P15364,&P6729,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Powermarks/3.5; Windows 95/98/2000/NT)*/
const Device UA121513 = {S34531, 22, { &P15364,&P6729,&P17470,&P18092 }};	/*Useragent - Mozilla/3.0 (Win95; I)*/
const Device UA173917 = {S34532, 28, { &P15364,&P6729,&P17470,&P18092 }};	/*Useragent - Mozilla/4.05 [en] (Win95; I)*/
const Device UA175171 = {S34533, 27, { &P15364,&P6729,&P17470,&P18092 }};	/*Useragent - Mozilla/4.5 [en] (Win95; I)*/
const Device UA181104 = {S34534, 28, { &P15364,&P6729,&P17470,&P18092 }};	/*Useragent - NCSA Mosaic/3.0 (Windows 95)*/
const Device UA186628 = {S34535, 27, { &P15364,&P6729,&P17470,&P18092 }};	/*Useragent - Mozilla/4.7 [en] (Win95; U)*/
const Device UA269559 = {S34536, 24, { &P15364,&P6729,&P17470,&P18092 }};	/*Useragent - Mozilla/2.02E (Win95; U)*/
const Device UA2972134 = {S34537, 142, { &P12280,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.12 Mobile/11A4372q Safari/8536.25*/
const Device UA2972147 = {S34538, 142, { &P12280,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.12 Mobile/11A4400f Safari/8536.25*/
const Device UA2974951 = {S34539, 142, { &P12280,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.12 Mobile/11A4414e Safari/8536.25*/
const Device UA3026723 = {S34540, 142, { &P12280,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A4435d Safari/8536.25*/
const Device UA3033030 = {S34541, 142, { &P12280,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A4449a Safari/8536.25*/
const Device UA3034364 = {S34542, 142, { &P12280,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A4449d Safari/8536.25*/
const Device UA3083921 = {S34543, 140, { &P12280,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A465 Safari/8536.25*/
const Device UA3115363 = {S34544, 142, { &P12280,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A501 Safari/8536.25*/
const Device UA3115474 = {S34545, 140, { &P12280,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A466 Safari/8536.25*/
const Device UA3115528 = {S34546, 143, { &P12280,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A470a Safari/8536.25*/
const Device UA1997510 = {S34547, 161, { &P18703,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Vodafone Smart II Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 SVN/01016*/
const Device UA2242278 = {S34548, 75, { &P18703,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; Vodafone Smart II Build/GINGERBREAD)*/
const Device UA2426671 = {S34549, 152, { &P18703,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; tr-tr; Vodafone 861 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 SVN/010 06*/
const Device UA2790853 = {S34550, 165, { &P18703,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android AG3N 4.0; es-es; Vodafone Smart II Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 SVN/0104.0*/
const Device UA458633 = {S34551, 151, { &P18317,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ASUS Transformer Pad TF300T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA458636 = {S34552, 80, { &P18317,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; ASUS Transformer Pad TF300T Build/IML74K)*/
const Device UA899864 = {S34553, 152, { &P18317,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ASUS Transformer Pad TF300TG Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1965185 = {S34554, 139, { &P18317,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ASUS Pad TF300T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2025301 = {S34555, 81, { &P18317,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; ASUS Transformer Pad TF300TG Build/IML74K)*/
const Device UA2030619 = {S34556, 146, { &P18317,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Transformer Pad TF300T Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2327682 = {S34557, 152, { &P18317,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ASUS Transformer Pad TF300TL Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2496085 = {S34558, 158, { &P18317,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ASUS Transformer Pad TF300T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3064866 = {S34559, 137, { &P18471,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-78) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067664 = {S34560, 143, { &P18471,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT905 Build/9.8.2Q_SMUIRD-7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068412 = {S34561, 139, { &P18471,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XT907 Build/9.8.1Q_27-2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA92396 = {S34562, 13, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/2.7*/
const Device UA92877 = {S34563, 15, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/2.7.1*/
const Device UA93939 = {S34564, 59, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress.com mShots; http://support.wordpress.com/contact/*/
const Device UA176633 = {S34565, 15, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/2.6.2*/
const Device UA238826 = {S34566, 24, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - ImgPress - WordPress.com*/
const Device UA258765 = {S34567, 49, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.1.3; http://politics.zonkeyblog.co.uk*/
const Device UA258862 = {S34568, 41, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.1; http://isaveholidays.co.uk*/
const Device UA258874 = {S34569, 51, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/2.9.2; http://polinlawoffice.shikshik.org*/
const Device UA258903 = {S34570, 46, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/2.9.2; http://molicyber.shikshik.org*/
const Device UA258921 = {S34571, 47, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.0.4; http://fixhealthcarepolicy.com*/
const Device UA259090 = {S34572, 53, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.2.1; http://abcnews.go.com/blogs/politics*/
const Device UA290681 = {S34573, 28, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/x.x.x.x PHP/4.x.xx*/
const Device UA366336 = {S34574, 63, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - Automattic Analytics Crawler/0.1; http://wordpress.com/crawler/*/
const Device UA430021 = {S34575, 45, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/2.9.2; http://aceflyer.shikshik.org*/
const Device UA454838 = {S34576, 31, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.3.1; http://x.X.com*/
const Device UA512041 = {S34577, 43, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.0.4; http://testwp.co.uk/canvas*/
const Device UA532232 = {S34578, 45, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/MU; http://chartroose.wordpress.com*/
const Device UA564899 = {S34579, 49, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.2.1; http://thefirsttimebuyerclub.com*/
const Device UA581063 = {S34580, 33, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.3; http://gpcnews.com*/
const Device UA582803 = {S34581, 38, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.3.1; http://screwcable.com*/
const Device UA621852 = {S34582, 45, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.3.1; http://horsesdolphins.x10.mx*/
const Device UA666252 = {S34583, 43, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.0.4; http://ismysiteindexed.com*/
const Device UA700764 = {S34584, 33, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.2; http://xindxr.info*/
const Device UA854447 = {S34585, 49, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.3.1; http://celebritynews.fansite.biz*/
const Device UA1885562 = {S34586, 37, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.1; http://yellowfield.net*/
const Device UA2194203 = {S34587, 71, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5-alpha-21535; http://thefashionloverblogging.wordpress.com*/
const Device UA2205034 = {S34588, 63, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5-alpha-21535; http://myfavoritelooks.wordpress.com*/
const Device UA2231033 = {S34589, 53, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5-alpha-21989; http://apps2.wordpress.com*/
const Device UA2231035 = {S34590, 52, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5-alpha-21989; http://p2wp.wordpress.com*/
const Device UA2237039 = {S34591, 38, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.4.2; http://www.isitwp.com*/
const Device UA2238090 = {S34592, 26, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5-alpha-21989;*/
const Device UA2238117 = {S34593, 48, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5-alpha-21989; http://thelightnc.com*/
const Device UA2238449 = {S34594, 52, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5-alpha-21989; http://ipowerrichmond.com*/
const Device UA2238561 = {S34595, 46, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5-alpha-21989; http://hiphopnc.com*/
const Device UA2262908 = {S34596, 54, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5-alpha-21989; http://news.stylecaster.com*/
const Device UA2262910 = {S34597, 52, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5-alpha-21989; http://praiserichmond.com*/
const Device UA2350080 = {S34598, 63, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5-alpha-21535; http://lastnightidrank.wordpress.com*/
const Device UA2350479 = {S34599, 51, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5-alpha-21535; http://kymx.cbslocal.com*/
const Device UA2356281 = {S34600, 58, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5-alpha-21989; http://annaleonie.wordpress.com*/
const Device UA2422110 = {S34601, 37, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.4.2; http://infoegypt.com*/
const Device UA2492143 = {S34602, 26, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.6-alpha-23334;*/
const Device UA2510937 = {S34603, 20, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress.com Reader*/
const Device UA2532806 = {S34604, 56, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.6-alpha-23334; http://goldblum.wordpress.com*/
const Device UA2562523 = {S34605, 47, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.6-alpha-23334; http://wordpress.com*/
const Device UA2581796 = {S34606, 55, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPressPollBot v1.2 (http://besome1.info/robots.html)*/
const Device UA2603820 = {S34607, 80, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5.1; http://ppt.4istudent.com,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2659448 = {S34608, 35, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.4.2; http://patrikaa.in*/
const Device UA2659508 = {S34609, 43, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5.1; http://localhost/wordpress*/
const Device UA2659628 = {S34610, 39, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.4.2; http://newindianews.in*/
const Device UA2659702 = {S34611, 35, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.4.2; http://morenews.in*/
const Device UA2659704 = {S34612, 37, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5; http://indianreport.in*/
const Device UA2659706 = {S34613, 41, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5; http://indiadailyreport.in*/
const Device UA2659707 = {S34614, 38, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5; http://theindiannews.in*/
const Device UA2659708 = {S34615, 46, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.4.2; http://indiabusinessreport.in*/
const Device UA2659709 = {S34616, 50, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.4.2; http://indiabusinessandfinance.in*/
const Device UA2659710 = {S34617, 37, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5; http://channelindia.in*/
const Device UA2659871 = {S34618, 43, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5; http://goodmorningindia.asia*/
const Device UA2660373 = {S34619, 48, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.6-alpha-23334; http://elcidharth.com*/
const Device UA2665186 = {S34620, 38, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5; http://bharatvaartaa.in*/
const Device UA2679933 = {S34621, 26, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.6-beta1-24023;*/
const Device UA2700071 = {S34622, 39, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5; http://indiamedianews.in*/
const Device UA2700072 = {S34623, 45, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.4.2; http://indianbusinessnews.in*/
const Device UA2700074 = {S34624, 38, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5; http://indianewsfeed.in*/
const Device UA2700075 = {S34625, 47, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.4.2; http://mumbaicommercialnews.in*/
const Device UA2700076 = {S34626, 41, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5; http://indianmarketnews.in*/
const Device UA2700120 = {S34627, 37, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5; http://indiatoday.asia*/
const Device UA2700121 = {S34628, 40, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.4.2; http://indiatechnews.in*/
const Device UA2700122 = {S34629, 44, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.4.2; http://indiamarketreport.in*/
const Device UA2700123 = {S34630, 39, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.4.2; http://indiainsider.in*/
const Device UA2700124 = {S34631, 46, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5; http://theindianbusinessline.in*/
const Device UA2700125 = {S34632, 42, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.4.2; http://indiaglobalnews.in*/
const Device UA2700126 = {S34633, 44, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.4.2; http://theindianreporter.in*/
const Device UA2700127 = {S34634, 40, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5; http://thepagesofindia.in*/
const Device UA2700128 = {S34635, 39, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5; http://hindustandaily.in*/
const Device UA2700129 = {S34636, 38, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5; http://vaartaapatram.in*/
const Device UA2701311 = {S34637, 46, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5; http://theindiancommunicator.in*/
const Device UA2701310 = {S34638, 39, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5; http://theindiandaily.in*/
const Device UA2705146 = {S34639, 41, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.4.2; http://indiabroadcast.in*/
const Device UA2733122 = {S34640, 84, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5.1; http://e-books.4istudent.com,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2740232 = {S34641, 20, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.6-beta3;*/
const Device UA2753247 = {S34642, 26, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.6-beta3-24330;*/
const Device UA2771290 = {S34643, 26, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.6-beta3-24375;*/
const Device UA2844523 = {S34644, 46, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5; http://www.colombotecnologia.co*/
const Device UA2844797 = {S34645, 43, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5; http://noticiasdeempresas.us*/
const Device UA2845493 = {S34646, 54, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.4.2; http://couponalert.blog.mindspark.com*/
const Device UA2854681 = {S34647, 38, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5; http://venezuelanews.ws*/
const Device UA2870912 = {S34648, 20, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.6-beta4;*/
const Device UA2879687 = {S34649, 94, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5.1; http://free-pdf-download.4istudent.com,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2888689 = {S34650, 26, { &P18254,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.6-beta4-24523;*/
const Device UA23429 = {S34651, 137, { &P13526,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; myTouch4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA39721 = {S34652, 136, { &P13526,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; myTouch4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA39724 = {S34653, 136, { &P13526,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; myTouch4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA39726 = {S34654, 136, { &P13526,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;myTouch4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA39749 = {S34655, 136, { &P13526,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; myTouch4GBuild/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA39750 = {S34656, 136, { &P13526,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; myTouch4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA39753 = {S34657, 137, { &P13526,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; myTouch4G Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59678 = {S34658, 137, { &P13526,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; myTouch4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73627 = {S34659, 141, { &P13526,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_myTouch4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA640074 = {S34660, 137, { &P13526,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; myTouch4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2974021 = {S34661, 140, { &P16292,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974283 = {S34662, 141, { &P16292,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020968 = {S34663, 139, { &P16292,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Galaxy Nexus Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021561 = {S34664, 133, { &P16292,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA33171 = {S34665, 149, { &P12710,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC_A510b V1.52.841.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA33172 = {S34666, 148, { &P12710,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC Wildfire S A510b Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48868 = {S34667, 149, { &P12710,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC_A510b V1.52.841.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59398 = {S34668, 147, { &P12710,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-nz; HTC_WildfireS_A510b Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61400 = {S34669, 137, { &P12710,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; HTC-A510a Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA251922 = {S34670, 137, { &P12710,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_A510c Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA455359 = {S34671, 147, { &P12710,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-nz; HTC_WildfireS_A510b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA654370 = {S34672, 72, { &P12710,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Wildfire S A510b Build/GRI40)*/
const Device UA680457 = {S34673, 61, { &P12710,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC_A510c Build/GRJ22)*/
const Device UA1901668 = {S34674, 61, { &P12710,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC-A510a Build/GRI40)*/
const Device UA1997071 = {S34675, 149, { &P12710,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC_A510b V2.26.841.4 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1997684 = {S34676, 148, { &P12710,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC Wildfire S A510b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019957 = {S34677, 139, { &P12710,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC-A510c/1.06.1401.4) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022824 = {S34678, 127, { &P12710,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC A510c) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026700 = {S34679, 72, { &P12710,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Wildfire S A510b Build/GRJ90)*/
const Device UA2028310 = {S34680, 125, { &P12710,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC A510c Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2032198 = {S34681, 150, { &P12710,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_A510c Build/TeamMarvelUSC2.3.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2220898 = {S34682, 149, { &P12710,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC_A510b V2.26.841.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2221630 = {S34683, 149, { &P12710,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC_A510b V1.52.841.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2475206 = {S34684, 130, { &P12710,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC A510c Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA186134 = {S34685, 115, { &P15364,&P17017,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (en-us) AppleWebKit/534.14 (KHTML, like Gecko; Google Wireless Transcoder) Chrome/9.0.597 Safari/534.14*/
const Device UA253783 = {S34686, 8, { &P15364,&P17017,&P17248,&P18092 }};	/*Useragent - Chrome/9*/
const Device UA393013 = {S34687, 152, { &P15364,&P17017,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (en-us) AppleWebKit/534.14 (KHTML, like Gecko; Google Wireless Transcoder) Chrome/9.0.597 Safari/534.14,gzip(gfe) (via translate.google.com)*/
const Device UA93324 = {S34688, 65, { &P15364,&P5690,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; ru) Opera 8.50*/
const Device UA133694 = {S34689, 65, { &P15364,&P5690,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.51*/
const Device UA260170 = {S34690, 65, { &P15364,&P5690,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.53*/
const Device UA272325 = {S34691, 65, { &P15364,&P5690,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.54*/
const Device UA434587 = {S34692, 46, { &P15364,&P5690,&P18098,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; en) Opera 8.50*/
const Device UA599643 = {S34693, 75, { &P15364,&P5690,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.50	Opera 8.5*/
const Device UA736632 = {S34694, 65, { &P15364,&P5690,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.52*/
const Device UA813162 = {S34695, 46, { &P15364,&P5690,&P18098,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; en) Opera 8.51*/
const Device UA857547 = {S34696, 46, { &P15364,&P5690,&P18098,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; pl) Opera 8.54*/
const Device UA1899382 = {S34697, 46, { &P15364,&P5690,&P18098,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; en) Opera 8.53*/
const Device UA2298885 = {S34698, 46, { &P15364,&P5690,&P18098,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; en) Opera 8.52*/
const Device UA2502471 = {S34699, 94, { &P15364,&P5690,&P18098,&P18092 }};	/*Useragent - ('Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.50', 'Opera 8.5, Windows XP')*/
const Device UA20572 = {S34700, 125, { &P14540,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaX2-01.1/5.0 (07.22) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA33426 = {S34701, 123, { &P14540,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA57001 = {S34702, 73, { &P14540,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-01*/
const Device UA2951466 = {S34703, 186, { &P14540,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA2534697 = {S34704, 70, { &P15364,&P16110,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:22.0) Gecko/X Firefox/22.0*/
const Device UA2535165 = {S34705, 58, { &P15364,&P16110,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; rv:22.0) Gecko/X Firefox/22.0*/
const Device UA2946392 = {S34706, 84, { &P15364,&P16110,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:22.0) Gecko/X Firefox/22.0 IceDragon/22.0.0.1*/
const Device UA123136 = {S34707, 96, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.0; Microsoft Outlook 14.0.4760; Pro; ms-office; MSOffice 14)*/
const Device UA161736 = {S34708, 96, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.0; Microsoft Outlook 14.0.5128; Pro; ms-office; MSOffice 14)*/
const Device UA182932 = {S34709, 96, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.0; Microsoft Outlook 14.0.4734; Pro; ms-office; MSOffice 14)*/
const Device UA237388 = {S34710, 226, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA246552 = {S34711, 96, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.0; Microsoft Outlook 14.0.6106; Pro; ms-office; MSOffice 14)*/
const Device UA346280 = {S34712, 226, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA452290 = {S34713, 72, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.0; Microsoft Outlook 14.0.6112; Pro)*/
const Device UA603859 = {S34714, 214, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA604456 = {S34715, 203, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA604497 = {S34716, 233, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA654039 = {S34717, 221, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA708034 = {S34718, 203, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA1969750 = {S34719, 210, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2154360 = {S34720, 238, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2176671 = {S34721, 72, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.0; Microsoft Outlook 14.0.6123; Pro)*/
const Device UA2247098 = {S34722, 210, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA2315731 = {S34723, 236, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Zune 4.7; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2318076 = {S34724, 116, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Microsoft Outlook 14.0.6117; ms-office; MSOffice 14)*/
const Device UA2363059 = {S34725, 226, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2368788 = {S34726, 193, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2368871 = {S34727, 233, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2482523 = {S34728, 210, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2483118 = {S34729, 72, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.0; Microsoft Outlook 14.0.6129; Pro)*/
const Device UA2516673 = {S34730, 226, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2531505 = {S34731, 205, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2565389 = {S34732, 233, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2664237 = {S34733, 116, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2665602 = {S34734, 210, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2704461 = {S34735, 123, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2720540 = {S34736, 203, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2793694 = {S34737, 238, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2852797 = {S34738, 274, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2855560 = {S34739, 236, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Zune 4.7; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2856930 = {S34740, 215, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC1; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2933690 = {S34741, 235, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA3023772 = {S34742, 226, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3059085 = {S34743, 203, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7012; ms-office; MSOffice 14)*/
const Device UA3083819 = {S34744, 226, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3095875 = {S34745, 205, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3110480 = {S34746, 210, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA3111005 = {S34747, 72, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Microsoft Office/14.0 (Windows NT 6.0; Microsoft Outlook 14.0.7106; Pro)*/
const Device UA3114649 = {S34748, 233, { &P15364,&P17285,&P18568,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; InfoPath.3; Microsoft Outlook 14.0.7105; ms-office; MSOffice 14)*/
const Device UA93084 = {S34749, 65, { &P15364,&P5690,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Avant Browser)*/
const Device UA128926 = {S34750, 182, { &P15364,&P5690,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Avant Browser; Avant Browser; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30)*/
const Device UA226776 = {S34751, 98, { &P15364,&P5690,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; CHWIE_AT71; SV1; Avant Browser; MEGAUPLOAD 1.0)*/
const Device UA385148 = {S34752, 93, { &P15364,&P5690,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Avant Browser; Avant Browser)*/
const Device UA808211 = {S34753, 224, { &P15364,&P5690,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Avant Browser; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Alexa Toolbar; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648)*/
const Device UA1957909 = {S34754, 78, { &P15364,&P5690,&P22730,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 5.1; Avant Browser)*/
const Device UA2153148 = {S34755, 78, { &P15364,&P5690,&P22730,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 5.2; Avant Browser)*/
const Device UA2157374 = {S34756, 215, { &P15364,&P5690,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Avant Browser; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Avant Browser)*/
const Device UA2166919 = {S34757, 78, { &P15364,&P5690,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 5.2; Avant Browser)*/
const Device UA2338763 = {S34758, 191, { &P15364,&P5690,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Avant Browser; Avant Browser; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Avant Browser)*/
const Device UA2774058 = {S34759, 141, { &P15364,&P5690,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2;  SV1; Avant Browser; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; Media Center PC 6.0; FDM)*/
const Device UA2824095 = {S34760, 112, { &P15364,&P5690,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; Avant Browser; .NET CLR 1.1.4322; Avant Browser)*/
const Device UA3020629 = {S34761, 156, { &P15364,&P5690,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1;  SV1; Avant Browser; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; FDM)*/
const Device UA3027927 = {S34762, 154, { &P15364,&P5690,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Avant Browser; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; Media Center PC 6.0; FDM)*/
const Device UA347338 = {S34763, 116, { &P17595,&P17296,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA347785 = {S34764, 116, { &P17595,&P17296,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA347786 = {S34765, 116, { &P17595,&P17296,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA351801 = {S34766, 116, { &P17595,&P17296,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA351901 = {S34767, 116, { &P17595,&P17296,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA366180 = {S34768, 116, { &P17595,&P17296,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7*/
const Device UA366278 = {S34769, 116, { &P17595,&P17296,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7*/
const Device UA368139 = {S34770, 116, { &P17595,&P17296,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA368163 = {S34771, 116, { &P17595,&P17296,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA710852 = {S34772, 153, { &P17595,&P17296,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7,gzip(gfe) (via translate.google.com)*/
const Device UA2332597 = {S34773, 116, { &P17595,&P17296,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.7 (KHTML	 like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA2339103 = {S34774, 116, { &P17595,&P17296,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/535.7 (KHTML	 like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA2339853 = {S34775, 116, { &P17595,&P17296,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/535.7 (KHTML	 like Gecko) Chrome/16.0.912.75 Safari/535.7*/
const Device UA2674517 = {S34776, 129, { &P17595,&P17296,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/16.0.648.205 Safari/534.16*/
const Device UA2674610 = {S34777, 129, { &P17595,&P17296,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/16.0.648.205 Safari/534.16*/
const Device UA2718950 = {S34778, 129, { &P17595,&P17296,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/16.0.648.205 Safari/534.16*/
const Device UA56959 = {S34779, 143, { &P16553,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59542 = {S34780, 143, { &P16553,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71456 = {S34781, 143, { &P16553,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369228 = {S34782, 150, { &P16553,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T959V Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA369311 = {S34783, 143, { &P16553,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622891 = {S34784, 143, { &P16553,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA634396 = {S34785, 67, { &P16553,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-T959V Build/GINGERBREAD)*/
const Device UA639310 = {S34786, 142, { &P16553,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.6; en-us; SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA683678 = {S34787, 142, { &P16553,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA691051 = {S34788, 142, { &P16553,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA705845 = {S34789, 142, { &P16553,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T959VBuild/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA784404 = {S34790, 142, { &P16553,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us;SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019018 = {S34791, 67, { &P16553,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SGH-T959V Build/GINGERBREAD)*/
const Device UA2020746 = {S34792, 67, { &P16553,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SGH-T959V Build/GINGERBREAD)*/
const Device UA2020883 = {S34793, 67, { &P16553,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SGH-T959V Build/GINGERBREAD)*/
const Device UA90886 = {S34794, 132, { &P17346,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3*/
const Device UA333557 = {S34795, 134, { &P17346,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5274d Safari/7534.48.3*/
const Device UA622534 = {S34796, 134, { &P17346,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3*/
const Device UA784178 = {S34797, 134, { &P17346,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9Z999 Safari/7534.48.3*/
const Device UA1892772 = {S34798, 134, { &P17346,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A5220p Safari/7534.48.3*/
const Device UA2597517 = {S34799, 130, { &P17346,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 50 like Mac OS X) AppleWebKit/534.46 (KHTML like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3*/
const Device UA2658424 = {S34800, 131, { &P17346,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 501 like Mac OS X) AppleWebKit/534.46 (KHTML like Gecko) Version/5.1 Mobile/9B176 Safari/7534.48.3*/
const Device UA2659282 = {S34801, 131, { &P17346,&P17146,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 501 like Mac OS X) AppleWebKit/534.46 (KHTML like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3*/
const Device UA2566433 = {S34802, 74, { &P17595,&P17297,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:22.0) Gecko/X Firefox/22.0*/
const Device UA902621 = {S34803, 76, { &P17595,&P17297,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:13.0) Gecko/X Firefox/13.0.1*/
const Device UA2227280 = {S34804, 144, { &P18656,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; USCCADR3310 Build/HuaweiM866) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2230705 = {S34805, 139, { &P18656,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; H866C Build/HuaweiH866C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2366786 = {S34806, 63, { &P18656,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; H866C Build/HuaweiH866C)*/
const Device UA2411406 = {S34807, 144, { &P18656,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HUAWEI M866 Build/HuaweiM866) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2526601 = {S34808, 138, { &P18656,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; H866CBuild/HuaweiH866C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2556438 = {S34809, 137, { &P18656,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; M866 Build/HuaweiM866) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2694253 = {S34810, 68, { &P18656,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; USCCADR3310 Build/HuaweiM866)*/
const Device UA2842322 = {S34811, 138, { &P18656,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.6; en-us; H866C Build/HuaweiH866C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2843635 = {S34812, 138, { &P18656,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; H866C Build/HuaweiH866C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA3086398 = {S34813, 137, { &P18656,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; -us; H866C Build/HuaweiH866C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2388398 = {S34814, 74, { &P17595,&P18207,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:18.0) Gecko/X Firefox/18.0*/
const Device UA857790 = {S34815, 138, { &P18243,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; LG-P880 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2024669 = {S34816, 60, { &P18243,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-P880 Build/IML74K)*/
const Device UA2026780 = {S34817, 143, { &P18243,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-P880/V10b Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027978 = {S34818, 143, { &P18243,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-P880/V10a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028959 = {S34819, 143, { &P18243,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-P880/V10d Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2191112 = {S34820, 143, { &P18243,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-P880/V10f Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2366205 = {S34821, 139, { &P18243,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; LG-P880g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2413488 = {S34822, 129, { &P18243,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; LG-P880 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2427036 = {S34823, 143, { &P18243,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; LG-P880/V10h Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2646769 = {S34824, 135, { &P18243,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; bs; LG-P880 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2951169 = {S34825, 61, { &P18243,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-P880g Build/IML74K)*/
const Device UA924973 = {S34826, 119, { &P15364,&P7133,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA925433 = {S34827, 126, { &P15364,&P7133,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA2422609 = {S34828, 129, { &P15364,&P7133,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7,gzip(gfe)*/
const Device UA2423160 = {S34829, 136, { &P15364,&P7133,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7,gzip(gfe)*/
const Device UA2168169 = {S34830, 144, { &P20289,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC6435LVW 4G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2366065 = {S34831, 141, { &P20289,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC6435LVW Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2405883 = {S34832, 63, { &P20289,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC6435LVW Build/JRO03C)*/
const Device UA2585336 = {S34833, 144, { &P20289,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC6435LVW 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2605942 = {S34834, 141, { &P20289,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC6435LVW Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2765598 = {S34835, 63, { &P20289,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC6435LVW Build/JZO54K)*/
const Device UA2985264 = {S34836, 139, { &P20289,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; HTC6435LVW Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA22340 = {S34837, 164, { &P15184,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; LG-P920 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA46519 = {S34838, 163, { &P15184,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P920/V10f/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA46543 = {S34839, 165, { &P15184,&P5096,&P3508,&P18092 }};	/*Useragent - LG-P920/ Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA46593 = {S34840, 170, { &P15184,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; LG-P920/V10d  Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA46595 = {S34841, 163, { &P15184,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P920/V10d/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA59827 = {S34842, 170, { &P15184,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; LG-P920/V10c  Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA59829 = {S34843, 173, { &P15184,&P5096,&P3508,&P18092 }};	/*Useragent - LG-P920 Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P920; Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA59837 = {S34844, 164, { &P15184,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P920 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA59821 = {S34845, 171, { &P15184,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P920-Orange Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA70142 = {S34846, 170, { &P15184,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; LG-P920/V10f  Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA225273 = {S34847, 60, { &P15184,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P920 Build/FRG83G)*/
const Device UA379544 = {S34848, 165, { &P15184,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-mx; LG-P920h Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA576414 = {S34849, 163, { &P15184,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P920/V10k/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA623671 = {S34850, 170, { &P15184,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; LG-P920/V10l  Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2020002 = {S34851, 61, { &P15184,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P920h Build/FRG83G)*/
const Device UA2022261 = {S34852, 170, { &P15184,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; hr-hr; LG-P920/V10b  Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2642563 = {S34853, 129, { &P15184,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; LG-P920 Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2807661 = {S34854, 163, { &P15184,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; el-gr; LG-P920/V10c/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA92927 = {S34855, 48, { &P15364,&P17350,&P18068,&P18092 }};	/*Useragent - Mozilla/1.22 (compatible; MSIE 2.0d; Windows NT)*/
const Device UA116727 = {S34856, 47, { &P15364,&P17350,&P18068,&P18092 }};	/*Useragent - Mozilla/1.22 (compatible; MSIE 1.5; Windows NT)*/
const Device UA702350 = {S34857, 37, { &P15364,&P17350,&P18068,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE; Win32)*/
const Device UA2418373 = {S34858, 44, { &P15364,&P17350,&P18068,&P18092 }};	/*Useragent - lla/1.22 (compatible; MSIE 2.0d; Windows NT)*/
const Device UA2509432 = {S34859, 52, { &P15364,&P17350,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; 68K; Trident/3.0)*/
const Device UA2509457 = {S34860, 52, { &P15364,&P17350,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; 68K; Trident/2.0)*/
const Device UA2597826 = {S34861, 64, { &P15364,&P17350,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; PPC; Win64; x64; Trident/1.0)*/
const Device UA2661228 = {S34862, 62, { &P15364,&P17350,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; PPC; .NET CLR; Trident/2.0)*/
const Device UA2729562 = {S34863, 64, { &P15364,&P17350,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; 68K; Win64; x64; Trident/3.0)*/
const Device UA2791770 = {S34864, 59, { &P15364,&P17350,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; PPC; WOW64; Trident/1.0)*/
const Device UA2791862 = {S34865, 64, { &P15364,&P17350,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; 68K; Win64; x64; Trident/5.0)*/
const Device UA2792525 = {S34866, 59, { &P15364,&P17350,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; PPC; WOW64; Trident/4.0)*/
const Device UA2792566 = {S34867, 57, { &P15364,&P17350,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; 68K; SV1; Trident/1.0)*/
const Device UA2792600 = {S34868, 62, { &P15364,&P17350,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; PPC; .NET CLR; Trident/3.0)*/
const Device UA2793090 = {S34869, 59, { &P15364,&P17350,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; 68K; WOW64; Trident/2.0)*/
const Device UA2849927 = {S34870, 57, { &P15364,&P17350,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; 68K; SV1; Trident/4.0)*/
const Device UA2850043 = {S34871, 59, { &P15364,&P17350,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; 68K; WOW64; Trident/1.0)*/
const Device UA2850881 = {S34872, 64, { &P15364,&P17350,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; PPC; Win64; x64; Trident/3.0)*/
const Device UA2977606 = {S34873, 39, { &P15364,&P17350,&P18068,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE; Windows)*/
const Device UA2540495 = {S34874, 61, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-I9500 Build/JOP40D)*/
const Device UA2711437 = {S34875, 138, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2712496 = {S34876, 60, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9500 Build/JDQ39)*/
const Device UA2769748 = {S34877, 139, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2771208 = {S34878, 137, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9500 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2782110 = {S34879, 139, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892476 = {S34880, 139, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2893498 = {S34881, 136, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.3; en-us; I9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2893581 = {S34882, 138, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.9; en-gb; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2895906 = {S34883, 136, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.3; es-es; I9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2898295 = {S34884, 138, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.9; en-us; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937230 = {S34885, 136, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958458 = {S34886, 138, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.9; es-es; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2960692 = {S34887, 139, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996595 = {S34888, 139, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; GT-i9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998046 = {S34889, 136, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; vi-vn; I9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999150 = {S34890, 138, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.9; vi-vn; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999258 = {S34891, 139, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; vi-vn; GT-i9500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999309 = {S34892, 136, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.3; it-it; I9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3000929 = {S34893, 138, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.9; it-it; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3004939 = {S34894, 138, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.9; nl-nl; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3006048 = {S34895, 136, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.3; en-gb; I9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3006644 = {S34896, 136, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.3; nl-nl; I9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3008718 = {S34897, 133, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; I9500 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3013803 = {S34898, 149, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Samsung Galaxy S 4 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3015060 = {S34899, 151, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.9; en-us; GT-I9500 Build/JDQ39.I9500ZSUAMDF) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3019687 = {S34900, 139, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.3; en-us; GT-I9500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050620 = {S34901, 138, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.9; ar-eg; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3055169 = {S34902, 146, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9500 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3074326 = {S34903, 138, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.9; fr-fr; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092454 = {S34904, 181, { &P21261,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9500 Build/JDQ39) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 T5/1.0 baiduboxapp/4.5.1 (Baidu; P1 4.2.2)*/
const Device UA79886 = {S34905, 81, { &P15364,&P17285,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/4*/
const Device UA103329 = {S34906, 116, { &P15364,&P17285,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.78 Safari/532.5*/
const Device UA124918 = {S34907, 118, { &P15364,&P17285,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.1.249.1025 Safari/532.5*/
const Device UA602209 = {S34908, 124, { &P15364,&P17285,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0 (x86_64); de-DE) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/4.0.202.2 Safari/532.0*/
const Device UA611384 = {S34909, 116, { &P17595,&P18207,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1063.0 Safari/536.3*/
const Device UA2006325 = {S34910, 117, { &P17595,&P18207,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA2200646 = {S34911, 117, { &P17595,&P18207,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA2234199 = {S34912, 158, { &P17595,&P18207,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1063.0 Safari/536.3 (2.3.0.1, ruby 1.9.2 (2012-05-01) [java])*/
const Device UA2360766 = {S34913, 143, { &P17595,&P18207,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5*/
const Device UA2584142 = {S34914, 117, { &P17595,&P18207,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA2796573 = {S34915, 117, { &P17595,&P18207,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA2800466 = {S34916, 149, { &P17595,&P18207,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1063.0 Safari/536.3 (2.3.3, ruby 1.9.3 (2013-02-22))*/
const Device UA3094416 = {S34917, 117, { &P17595,&P18207,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA3131148 = {S34918, 151, { &P17595,&P18207,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1063.0 Safari/536.3 (2.3.4.1, ruby 2.0.0 (2013-06-27))*/
const Device UA2359489 = {S34919, 141, { &P18471,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XT907 Build/9.8.1Q_27-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2362482 = {S34920, 63, { &P18471,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; XT907 Build/9.8.1Q_27-2)*/
const Device UA2421587 = {S34921, 139, { &P18471,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XT907 Build/9.8.1Q_25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426166 = {S34922, 70, { &P18471,&P18111,&P18112,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android 4.1.1;motorola XT907 Build/9.8.1Q_27-2)*/
const Device UA2536542 = {S34923, 145, { &P18471,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; XT905 Build/9.8.2Q_SMUIRD-7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2670761 = {S34924, 139, { &P18471,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-66) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2731141 = {S34925, 67, { &P18471,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT905 Build/9.8.2Q_SMUIRD-7)*/
const Device UA2746892 = {S34926, 155, { &P18471,&P18111,&P18112,&P18092 }};	/*Useragent - MOT-XT907/ Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XT907 Build/9.8.1Q_27-2) 1617 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2767706 = {S34927, 141, { &P18471,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-66-1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2844831 = {S34928, 139, { &P18471,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; XT907 Build/9.8.1Q_27-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2861664 = {S34929, 153, { &P18471,&P18111,&P18112,&P18092 }};	/*Useragent - MOT-XT907/ Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-66) 1617 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2937873 = {S34930, 139, { &P18471,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-78) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2942547 = {S34931, 141, { &P18471,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-78-1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955856 = {S34932, 137, { &P18471,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; XT907 Build/9.8.1Q-66) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2968324 = {S34933, 61, { &P18471,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT907 Build/9.8.1Q-78)*/
const Device UA2969839 = {S34934, 68, { &P18471,&P18111,&P18112,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android 4.1.2;motorola XT907 Build/9.8.1Q-78)*/
const Device UA3057706 = {S34935, 153, { &P18471,&P18111,&P18112,&P18092 }};	/*Useragent - MOT-XT907/ Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-78) 1617 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3109002 = {S34936, 173, { &P18471,&P18111,&P18112,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=scorpion_mini;devm=XT907;man=motorola;brw=Android;shwd=540x888;shwa=540x851;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA3124785 = {S34937, 155, { &P18471,&P18111,&P18112,&P18092 }};	/*Useragent - MOT-XT907/ Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-78-1) 1617 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA36858 = {S34938, 134, { &P12216,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA36860 = {S34939, 134, { &P12216,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6300Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA36862 = {S34940, 134, { &P12216,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; ADR6300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA36866 = {S34941, 134, { &P12216,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; ADR6300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA36867 = {S34942, 134, { &P12216,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;ADR6300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA36871 = {S34943, 135, { &P12216,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59858 = {S34944, 135, { &P12216,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ADR6300 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73145 = {S34945, 135, { &P12216,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ADR6300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA171212 = {S34946, 135, { &P12216,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA181120 = {S34947, 135, { &P12216,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA190862 = {S34948, 135, { &P12216,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA247694 = {S34949, 59, { &P12216,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; ADR6300 Build/GRJ22)*/
const Device UA248913 = {S34950, 135, { &P12216,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA329495 = {S34951, 135, { &P12216,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369216 = {S34952, 65, { &P12216,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.4; en_US; ADR6300 Build/GRJ22)*/
const Device UA370407 = {S34953, 33, { &P12216,&P5629,&P15760,&P18092 }};	/*Useragent - Android/2.3.4; HTC ADR6300; 2.2.0*/
const Device UA684985 = {S34954, 59, { &P12216,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; ADR6300 Build/FRF91)*/
const Device UA1898309 = {S34955, 142, { &P12216,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6300 Build/GRJ22) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2032727 = {S34956, 59, { &P12216,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; ADR6300 Build/GRJ22)*/
const Device UA2272383 = {S34957, 135, { &P12216,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2573622 = {S34958, 133, { &P12216,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; -us; ADR6300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2797949 = {S34959, 105, { &P18323,&P17484,&P17373,&P18092 }};	/*Useragent - SAMSUNG-GT-S5222 Opera/9.80 (J2ME/MIDP; Opera Mini/6.5.29806/30.3214; U; en) Presto/2.8.119 Version/11.10*/
const Device UA214630 = {S34960, 107, { &P15364,&P18110,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.785.0 Safari/535.1*/
const Device UA250157 = {S34961, 100, { &P15364,&P18110,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA250761 = {S34962, 144, { &P15364,&P18110,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Jolicloud Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Joli OS/1.2 Chromium/14.0.835.126 Chrome/14.0.835.126 Safari/535.1*/
const Device UA640778 = {S34963, 100, { &P15364,&P18110,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.824.0 Safari/535.1*/
const Device UA838983 = {S34964, 138, { &P15364,&P18110,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1 (ru-RU,ru;q=0.8,en-US;q=0.6,en;q=0.4)*/
const Device UA1980136 = {S34965, 104, { &P15364,&P18110,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.94 Safari/535.1*/
const Device UA2301089 = {S34966, 137, { &P15364,&P18110,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1,gzip(gfe) (via translate.google.com)*/
const Device UA2333737 = {S34967, 98, { &P15364,&P18110,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML	 like Gecko) Chrome/14.0.803.0 Safari/535.1*/
const Device UA2674670 = {S34968, 109, { &P15364,&P18110,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/14.0.1410.63 Safari/537.31*/
const Device UA2674697 = {S34969, 106, { &P15364,&P18110,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/14.0.1410.63 Safari/537.31*/
const Device UA2674707 = {S34970, 106, { &P15364,&P18110,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/14.0.1410.63 Safari/537.31*/
const Device UA2674883 = {S34971, 106, { &P15364,&P18110,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/14.0.1410.63 Safari/537.31*/
const Device UA2676613 = {S34972, 109, { &P15364,&P18110,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/14.0.1410.63 Safari/537.31*/
const Device UA2715754 = {S34973, 109, { &P15364,&P18110,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/14.0.1410.63 Safari/537.31*/
const Device UA2716836 = {S34974, 106, { &P15364,&P18110,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/14.0.1410.63 Safari/537.31*/
const Device UA1958746 = {S34975, 153, { &P15047,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyEricssonSK17a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1974511 = {S34976, 141, { &P15047,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SK17i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2203290 = {S34977, 153, { &P15047,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonSK17i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2229039 = {S34978, 63, { &P15047,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SK17i Build/4.1.B.0.587)*/
const Device UA2231515 = {S34979, 141, { &P15047,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SK17i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2232424 = {S34980, 154, { &P15047,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SonyEricssonSK17iv Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257781 = {S34981, 141, { &P15047,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SK17a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2727200 = {S34982, 141, { &P15047,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SK17a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2758343 = {S34983, 63, { &P15047,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SK17a Build/4.1.B.0.431)*/
const Device UA79883 = {S34984, 81, { &P15364,&P17285,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/3*/
const Device UA93305 = {S34985, 116, { &P15364,&P17285,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.25 Safari/532.0*/
const Device UA2043012 = {S34986, 76, { &P17595,&P18207,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:15.0) Gecko/X Firefox/15.0.1*/
const Device UA2693180 = {S34987, 78, { &P17595,&P18207,&P18345,&P18092 }};	/*Useragent - Mosssszilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:15.0) Gecko/X Firefox/15.0*/
const Device UA257425 = {S34988, 121, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/17.0.942.0) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/17.0.942.0 Safari/535.8*/
const Device UA366503 = {S34989, 125, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/17.0.963.26) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.26 Safari/535.11*/
const Device UA381228 = {S34990, 125, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/17.0.963.46) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.46 Safari/535.11*/
const Device UA401375 = {S34991, 125, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/17.0.963.56) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA401611 = {S34992, 229, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/17.0.963.56; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA409671 = {S34993, 158, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/17.0.963.56; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA412898 = {S34994, 125, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/17.0.963.65) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.65 Safari/535.11*/
const Device UA412973 = {S34995, 125, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/17.0.963.66) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11*/
const Device UA427658 = {S34996, 125, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/17.0.963.78) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.78 Safari/535.11*/
const Device UA428576 = {S34997, 125, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/17.0.963.79) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11*/
const Device UA428808 = {S34998, 159, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/17.0.963.79; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA458303 = {S34999, 125, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/17.0.963.83) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.83 Safari/535.11*/
const Device UA458321 = {S35000, 238, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/17.0.963.83; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA583467 = {S35001, 125, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/17.0.963.33) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.33 Safari/535.11*/
const Device UA586914 = {S35002, 125, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/17.0.963.38) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.38 Safari/535.11*/
const Device UA590438 = {S35003, 205, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/17.0.963.56; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA590572 = {S35004, 125, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/17.0.963.56) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA591166 = {S35005, 198, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/17.0.963.56; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA597006 = {S35006, 212, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; chromeframe/17.0.963.46; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MuW; InfoPath.2; .NET4.0C; .NET4.0E; MuW 64; MuW 64; QG Managed;)*/
const Device UA692365 = {S35007, 195, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/17.0.963.78; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.3)*/
const Device UA865996 = {S35008, 191, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/17.0.963.79; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA899053 = {S35009, 75, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/17.0.963.79)*/
const Device UA2332935 = {S35010, 250, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/17.0.963.46; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; AskTbFWV5/5.14.0.19709)*/
const Device UA2375884 = {S35011, 255, { &P15364,&P5690,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.2; chromeframe/17.0.963.46; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; Windows-Media-Player/10.00.00.3990; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729;*/
const Device UA2043017 = {S35012, 76, { &P17595,&P17295,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:15.0) Gecko/X Firefox/15.0.1*/
const Device UA19626 = {S35013, 160, { &P14842,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; SonyEricssonE10i-o Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA19638 = {S35014, 159, { &P14842,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; SonyEricssonE10iv Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA19660 = {S35015, 146, { &P14842,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; E10i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA30312 = {S35016, 146, { &P14842,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-kr; E10i Build/2.0.A.0.497) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA34196 = {S35017, 146, { &P14842,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; E10i Build/2.0.A.0.504) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA40591 = {S35018, 99, { &P14842,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.1-update1;Zh_cn;E10i;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA42388 = {S35019, 160, { &P14842,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1 - JIT Enabled; sv-se; E10i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA42417 = {S35020, 93, { &P14842,&P5477,&P4861,&P18092 }};	/*Useragent - Android (Linux; U; Android 2.1-update1; fa-ir; E10i) AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA58277 = {S35021, 146, { &P14842,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; E10i Build/2.1.1.C.0.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA58283 = {S35022, 147, { &P14842,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; pl-pl; E10i Build/2.1.1.A.0.16) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA62540 = {S35023, 158, { &P14842,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; SonyEricssonE10i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA77356 = {S35024, 147, { &P14842,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; E10i Build/2.1.1.A.0.19) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA189229 = {S35025, 68, { &P14842,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; E10i Build/2.1.1.A.0.6)*/
const Device UA769274 = {S35026, 147, { &P14842,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; E10i Build/2.0.2.A.0.24) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1902318 = {S35027, 132, { &P14842,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fa-ir; E10i Build/2.1.1.A.0.6) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2025092 = {S35028, 132, { &P14842,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fa-ir; E10i Build/2.1.1.C.0.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA454563 = {S35029, 139, { &P12814,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I777 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA690858 = {S35030, 139, { &P12814,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-I777 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA722734 = {S35031, 147, { &P12814,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SAMSUNG-SGH-I777 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA758729 = {S35032, 139, { &P12814,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I777 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA765592 = {S35033, 69, { &P12814,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SAMSUNG-SGH-I777 Build/IML74K)*/
const Device UA895184 = {S35034, 141, { &P12814,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; Galaxy S II Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1991544 = {S35035, 61, { &P12814,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SGH-I777 Build/IML74K)*/
const Device UA2031283 = {S35036, 138, { &P12814,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I777 Build/cMIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2393565 = {S35037, 139, { &P12814,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I777 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2413402 = {S35038, 61, { &P12814,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I777 Build/IMM76L)*/
const Device UA2416754 = {S35039, 147, { &P12814,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I777 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2433702 = {S35040, 69, { &P12814,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SAMSUNG-SGH-I777 Build/IMM76D)*/
const Device UA696981 = {S35041, 98, { &P15364,&P16110,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.6 (KHTML, like Gecko) Chrome/20.0.1090.0 Safari/536.6*/
const Device UA912420 = {S35042, 108, { &P15364,&P16110,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA1951419 = {S35043, 138, { &P15364,&P16110,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11,gzip(gfe) (via translate.google.com)*/
const Device UA2675769 = {S35044, 98, { &P15364,&P16110,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/20 Safari/537.17*/
const Device UA2489338 = {S35045, 114, { &P15228,&P5043,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-03/07.48; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2034046 = {S35046, 139, { &P18301,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VS840 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2355343 = {S35047, 61, { &P18301,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; VS840 4G Build/IMM76D)*/
const Device UA2738338 = {S35048, 138, { &P18301,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VS840PP Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2868573 = {S35049, 60, { &P18301,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; VS840PP Build/IMM76D)*/
const Device UA2984007 = {S35050, 137, { &P18301,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; VS840 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA32986 = {S35051, 141, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Nexus S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA64115 = {S35052, 135, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; Nexus S Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67198 = {S35053, 133, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; Nexus S Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67354 = {S35054, 135, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Nexus S Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67525 = {S35055, 136, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; Nexus S Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67734 = {S35056, 135, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Nexus S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA180570 = {S35057, 135, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Nexus S Build/GRI54) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA188401 = {S35058, 135, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Nexus S Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA190831 = {S35059, 59, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; Nexus S Build/GRJ22)*/
const Device UA196516 = {S35060, 135, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Nexus S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA207069 = {S35061, 135, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Nexus S Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA237796 = {S35062, 135, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Nexus S Build/GTJ61) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA244159 = {S35063, 136, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Nexus S Build/GRK39C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA244472 = {S35064, 60, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; Nexus S Build/GRK39F)*/
const Device UA245041 = {S35065, 136, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Nexus S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA249840 = {S35066, 135, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus S Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250070 = {S35067, 135, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus S Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA342442 = {S35068, 136, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA391537 = {S35069, 59, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; Nexus S Build/GRI54)*/
const Device UA459208 = {S35070, 134, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.4; en-us; Nexus S Build/GRI16B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA459217 = {S35071, 133, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.4; en-gb; Nexus S Build/GRH84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA459218 = {S35072, 134, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.4; en-us; Nexus S Build/GRI06B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA459227 = {S35073, 139, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.4; en-us; Nexus S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA459237 = {S35074, 154, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.4; en-us; Full Android on Crespo Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA517888 = {S35075, 135, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Nexus S Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA576456 = {S35076, 59, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; Nexus S Build/GWK74)*/
const Device UA631011 = {S35077, 135, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Nexus S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA679669 = {S35078, 124, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Nexus S Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1860244 = {S35079, 59, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.1; Nexus S Build/GRH78)*/
const Device UA2018233 = {S35080, 143, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Samsung Nexus S Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018400 = {S35081, 133, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; iw-; Nexus S Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018498 = {S35082, 134, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; xx-xx; Nexus S Build/GRH41B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018577 = {S35083, 133, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; Nexus S Build/GKH77) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020208 = {S35084, 137, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Nexus S Build/WS0.5.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021789 = {S35085, 135, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021858 = {S35086, 146, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK Nexus S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023013 = {S35087, 123, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Nexus S Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2024583 = {S35088, 137, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7p0; en-us; Nexus S Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2440560 = {S35089, 134, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Nexus S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2446101 = {S35090, 129, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Nexus S Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2720092 = {S35091, 135, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; Nexus S Build/GRI20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2758269 = {S35092, 141, { &P14574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; Nexus S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41403 = {S35093, 136, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ADR6400L Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41406 = {S35094, 139, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ADR6400L 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41409 = {S35095, 139, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ADR6400L 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41410 = {S35096, 139, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ADR6400L 4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57143 = {S35097, 136, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6400L Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57145 = {S35098, 139, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6400L 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74024 = {S35099, 139, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ADR6400L 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA197222 = {S35100, 60, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; ADR6400L Build/GRJ22)*/
const Device UA203470 = {S35101, 140, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ThunderBolt Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA209210 = {S35102, 139, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Thunderbolt Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA210289 = {S35103, 140, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Thunderbolt Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA220052 = {S35104, 137, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ADR6400L Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA249270 = {S35105, 137, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6400L Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250723 = {S35106, 143, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC ThunderBolt Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA333623 = {S35107, 144, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC ThunderBolt Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368936 = {S35108, 61, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; ADR6400L Build/GRK39F)*/
const Device UA369343 = {S35109, 66, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.4; en_US; ADR6400L Build/GRJ22)*/
const Device UA563536 = {S35110, 143, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC ThunderBolt Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA646517 = {S35111, 161, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC ThunderBolt Build/GRJ22; liquid.mecha.X.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA681724 = {S35112, 138, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6400L 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA690936 = {S35113, 140, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ADR6400L 4G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA692532 = {S35114, 138, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6400L4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA745977 = {S35115, 138, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6400L 4GBuild/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA769400 = {S35116, 138, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; ADR6400L 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA771426 = {S35117, 138, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; ADR6400L 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA774966 = {S35118, 138, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;ADR6400L 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA775636 = {S35119, 137, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-; ADR6400L 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA800926 = {S35120, 143, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; A05PTH3ORYB0LT Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA821788 = {S35121, 137, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ADR6400L Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1905379 = {S35122, 143, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6400L Build/GRJ22) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA1940240 = {S35123, 67, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC ThunderBolt Build/GRJ22)*/
const Device UA1996790 = {S35124, 152, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6400L Build/GRJ22; JELLYBLUR-V1.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019099 = {S35125, 143, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC ThunderBolt Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020391 = {S35126, 134, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; ADR6400L Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020903 = {S35127, 124, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ADR6400L Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2031830 = {S35128, 129, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ADR6400L Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2355952 = {S35129, 144, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6400L Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2411758 = {S35130, 60, { &P13991,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; ADR6400L Build/GRJ22)*/
const Device UA18152 = {S35131, 169, { &P14017,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; nl-nl; LG-P990/V10a Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA18157 = {S35132, 169, { &P14017,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; nl-nl; LG-P990/V10b Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA22200 = {S35133, 169, { &P14017,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; LG-P990/V10d Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA22201 = {S35134, 169, { &P14017,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; LG-P990/V10c Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA24650 = {S35135, 164, { &P14017,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P990 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA24700 = {S35136, 161, { &P14017,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en; LG-P990 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA45309 = {S35137, 169, { &P14017,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; LG-P990/V10f Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA48566 = {S35138, 165, { &P14017,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-sa; LG-P990H Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA49440 = {S35139, 166, { &P14017,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; LG-P990hN Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA49680 = {S35140, 165, { &P14017,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-mx; LG-P990h Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA53735 = {S35141, 163, { &P14017,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; LG-P990 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA59557 = {S35142, 169, { &P14017,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; LG-P990/V10e Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA190102 = {S35143, 60, { &P14017,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P990 Build/FRG83G)*/
const Device UA396563 = {S35144, 144, { &P14017,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-P990 Build/HannsTitan-0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1924258 = {S35145, 124, { &P14017,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; LG-P990 Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2020101 = {S35146, 62, { &P14017,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P990hN Build/FRG83G)*/
const Device UA2020942 = {S35147, 61, { &P14017,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P990h Build/FRG83G)*/
const Device UA2027638 = {S35148, 129, { &P14017,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; LG-P990 Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2034663 = {S35149, 144, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT30p Build/7.0.A.3.64) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034664 = {S35150, 145, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT30p Build/7.0.A.3.108) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034671 = {S35151, 165, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyLT30p Build/TESTFEATURE-SUPERSTAMINA-X-0705) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034719 = {S35152, 145, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyLT30p Build/7.0.A.1.303) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034744 = {S35153, 145, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyLT30p Build/7.0.A.3.116) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034756 = {S35154, 165, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT30p Build/TESTFEATURE-SUPERSTAMINA-X-0541) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034771 = {S35155, 165, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyLT30p Build/TESTFEATURE-SUPERSTAMINA-X-0856) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034821 = {S35156, 145, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT30p Build/7.0.A.1.300) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034833 = {S35157, 158, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT30p Build/BLUE1.0-DECOUPLED-X-1600) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034855 = {S35158, 146, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyLT30p Build/7.0.X.0.2303) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034942 = {S35159, 145, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT30p Build/7.0.A.1.129) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034952 = {S35160, 144, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT30p Build/7.0.A.3.35) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034963 = {S35161, 145, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT30p Build/7.0.A.1.269) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2035034 = {S35162, 143, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; SonyLT30p Build/7.0.A.2.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2035106 = {S35163, 141, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT30p Build/private) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2204862 = {S35164, 141, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT30p Build/7.0.A.1.303) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2239094 = {S35165, 147, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyLT30p-o Build/7.0.A.1.303) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2356210 = {S35166, 145, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyLT30p Build/7.0.A.3.195) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2362789 = {S35167, 141, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT30p Build/7.0.A.3.195) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2387499 = {S35168, 63, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT30p Build/7.0.A.3.195)*/
const Device UA2388282 = {S35169, 147, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyLT30p-o Build/7.0.A.3.195) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2425595 = {S35170, 145, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; SonyLT30p Build/7.0.A.3.197) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2430084 = {S35171, 145, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyLT30p Build/7.0.A.3.223) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2461202 = {S35172, 141, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT30p Build/7.0.A.3.223) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2652229 = {S35173, 145, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT30p Build/7.0.A.3.193) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665464 = {S35174, 145, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyLT30p Build/7.0.A.3.226) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2982881 = {S35175, 65, { &P18447,&P18066,&P18062,&P18092 }};	/*Useragent - SonyLT30p Build/7.0.A.3.195 stagefright/1.2 (Linux;Android 4.0.4)*/
const Device UA821082 = {S35176, 109, { &P15364,&P18110,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.8.0.7) Gecko/X Firefox/1.5.0.7 (Debian-1.5.dfsg+1.5.0.7-1~bpo.1)*/
const Device UA896020 = {S35177, 83, { &P15364,&P18110,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; pl; rv:1.8.0.7) Gecko/X Firefox/1.5.0.7 (Swiftfox)*/
const Device UA90223 = {S35178, 61, { &P18088,&P17017,&P17368,&P18092 }};	/*Useragent - Opera/8.01 (J2ME/MIDP; Opera Mini/3.1.10423/1724; en; U; ssr)*/
const Device UA452998 = {S35179, 130, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA628347 = {S35180, 59, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P6800 Build/HTJ85B)*/
const Device UA2024811 = {S35181, 123, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2031626 = {S35182, 137, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2031847 = {S35183, 128, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2032623 = {S35184, 151, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2034530 = {S35185, 128, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2609910 = {S35186, 130, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6800) antivirusHttpClient/2.0 (KHTML, like Gecko) Version/Version 2.3/Antivirus pro*/
const Device UA2611859 = {S35187, 128, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build HTJ85B) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2612444 = {S35188, 260, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 parameters/{scr=1232_800,cm=1,ql=l,imei=X,bm=1,eng=adrmini13,mdm=1,mdl=GT-P6800,ps=1} tiantian(securitypay) tiantian(tenpay)*/
const Device UA2617258 = {S35189, 243, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-tw; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 MobileSafari/534.13 VoiceGO!(cyb61ee75964e97b054ae89afa2366b5238-0;Android,3.2,GT-P6800;3.6.0;1280x752,1.0;X;VoiceGo-TC;zh-TW)*/
const Device UA2617417 = {S35190, 132, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6800 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2617578 = {S35191, 92, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 3.2; SAMSUNG GT-P6800 Build/HONEYCOMB_MR2);*/
const Device UA2618367 = {S35192, 160, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6800 Build/HTJ85B) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2618672 = {S35193, 65, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 3.2; cs-CZ; GT-P6800 Build/HTJ85B)*/
const Device UA2619319 = {S35194, 77, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - MediaPlayer/2002/4.0.12 (Android 3.2; GT-P6800; GT-P6800; samsung; ; ; en_GB)*/
const Device UA2619622 = {S35195, 71, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 3.2; en-US; GT-P6800 Build/HTJ85B.ZSLA2)*/
const Device UA2620613 = {S35196, 92, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - com.google.android.apps.plus/X (Linux; U; Android 3.2; en_GB; GT-P6800; Build/HTJ85B) (gzip)*/
const Device UA2622603 = {S35197, 282, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13 [FBAN/FB4A;FBAV/1.9.3;FBDM/{density=1.0,width=545,height=320};FBLC/en_GB;FB_FW/2;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P6800;FBSV/3.2;]*/
const Device UA2622912 = {S35198, 274, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/2.0;FBDM/{density=1.0,width=800,height=1232};FBLC/en_GB;FB_FW/2;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P6800;FBSV/3.2;]*/
const Device UA2623964 = {S35199, 75, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Mrd/1.1.0 (Linux; U; Android 3.2; samsung GT-P6800) jp.co.paperboy.picto/37*/
const Device UA2624620 = {S35200, 73, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 3.2; pt-BR; GT-P6800 Build/HTJ85B.UBLA2)*/
const Device UA2625157 = {S35201, 129, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6800) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2625716 = {S35202, 78, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - VKAndroidApp/2.7.6-64 (Android 3.2; SDK 13; armeabi-v7a; samsung GT-P6800; ru)*/
const Device UA2629354 = {S35203, 269, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.10;FBDM/{density=1.0,width=1280,height=752};FBLC/en_US;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P6800;FBSV/3.2;]*/
const Device UA2630493 = {S35204, 129, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb;GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2632147 = {S35205, 125, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Groupon/2.2.2572 (Android 3.2; Samsung GT-P6800 / Samsung GT-P6800; )[preload=true;locale=en_US;clientidbase=android-samsung]*/
const Device UA2633940 = {S35206, 64, { &P17323,&P6437,&P16179,&P18092 }};	/*Useragent - Android 3.2;AppleWebKit/533.1;Build/HTJ85B;GT-P6800 Build/HTJ85B*/
const Device UA428277 = {S35207, 127, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/18.0.1025.54) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.54 Safari/535.19*/
const Device UA440790 = {S35208, 129, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/18.0.1025.142) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19*/
const Device UA453713 = {S35209, 129, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/18.0.1025.142) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19*/
const Device UA454055 = {S35210, 231, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.3; chromeframe/18.0.1025.142; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C)*/
const Device UA457629 = {S35211, 129, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/18.0.1025.151) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19*/
const Device UA588774 = {S35212, 125, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/18.0.1025.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.1 Safari/535.19*/
const Device UA603604 = {S35213, 129, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/18.0.1025.152) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.152 Safari/535.19*/
const Device UA605657 = {S35214, 129, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/18.0.1025.162) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19*/
const Device UA605773 = {S35215, 129, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/18.0.1025.162) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19*/
const Device UA605885 = {S35216, 207, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/18.0.1025.162; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA606282 = {S35217, 77, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/18.0.1025.162)*/
const Device UA606524 = {S35218, 181, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/18.0.1025.162; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA606766 = {S35219, 136, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/18.0.1025.162) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19*/
const Device UA609325 = {S35220, 251, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/18.0.1025.162; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA614133 = {S35221, 129, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/18.0.1025.168) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA614921 = {S35222, 129, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/18.0.1025.168) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA643693 = {S35223, 205, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/18.0.1025.168; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA1994639 = {S35224, 136, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/18.0.1025.168) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA2332444 = {S35225, 180, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/18.0.1025.168; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA2520290 = {S35226, 227, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Master Desktop XP-FR-AF V02.00.00.00.00  R3; SVCVersion 8.14; chromeframe/18.0.1025.152; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2524323 = {S35227, 272, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/18.0.1025.162; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA2543653 = {S35228, 185, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/18.0.1025.168; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.1)*/
const Device UA2731214 = {S35229, 325, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; chromeframe/18.0.1025.162; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6109; ms-office; MSOffice 14)*/
const Device UA2747802 = {S35230, 306, { &P15364,&P5690,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; MathPlayer 2.20; chromeframe/18.0.1025.168; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.3; IPH 1.1.21.4019)*/
const Device UA457905 = {S35231, 125, { &P15364,&P5690,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/19.0.1084.15) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.15 Safari/536.5*/
const Device UA607458 = {S35232, 125, { &P15364,&P5690,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/19.0.1084.24) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.24 Safari/536.5*/
const Device UA615366 = {S35233, 125, { &P15364,&P5690,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/19.0.1084.41) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.41 Safari/536.5*/
const Device UA688272 = {S35234, 125, { &P15364,&P5690,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/19.0.1084.46) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5*/
const Device UA881010 = {S35235, 125, { &P15364,&P5690,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/19.0.1084.46) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5*/
const Device UA897382 = {S35236, 125, { &P15364,&P5690,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/19.0.1084.52) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5*/
const Device UA897534 = {S35237, 159, { &P15364,&P5690,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/19.0.1084.52; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA897561 = {S35238, 164, { &P15364,&P5690,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/19.0.1084.52; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA897623 = {S35239, 148, { &P15364,&P5690,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/19.0.1084.52; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)*/
const Device UA897636 = {S35240, 125, { &P15364,&P5690,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/19.0.1084.52) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5*/
const Device UA897865 = {S35241, 230, { &P15364,&P5690,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/19.0.1084.52; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA897997 = {S35242, 76, { &P15364,&P5690,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/19.0.1084.52)*/
const Device UA898100 = {S35243, 194, { &P15364,&P5690,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/19.0.1084.52; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; InfoPath.2)*/
const Device UA899452 = {S35244, 204, { &P15364,&P5690,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/19.0.1084.52; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA901109 = {S35245, 125, { &P15364,&P5690,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/19.0.1084.56) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA901950 = {S35246, 125, { &P15364,&P5690,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/19.0.1084.56) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA904671 = {S35247, 132, { &P15364,&P5690,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/19.0.1084.56) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA1985551 = {S35248, 204, { &P15364,&P5690,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; chromeframe/19.0.1084.52; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.3)*/
const Device UA2495155 = {S35249, 226, { &P15364,&P5690,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/19.0.1084.52; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA2574188 = {S35250, 204, { &P15364,&P5690,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/19.0.1084.52; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.1)*/
const Device UA2726886 = {S35251, 104, { &P15364,&P5690,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; chromeframe/19.0.1084.56; .NET4.0C)*/
const Device UA2386448 = {S35252, 147, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; DROID RAZR HD Build/9.8.1Q_37) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2404343 = {S35253, 147, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; DROID RAZR HD Build/9.8.1Q_39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2406899 = {S35254, 69, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; DROID RAZR HD Build/9.8.1Q_39)*/
const Device UA2425156 = {S35255, 151, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; XT925 Build/9.8.2Q-8-XT925_VQU-13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467535 = {S35256, 76, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android 4.1.1;motorola DROID RAZR HD Build/9.8.1Q_39)*/
const Device UA2509469 = {S35257, 153, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; XT925 Build/9.8.2Q-8-XT925_VQL-12.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2582138 = {S35258, 156, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2585725 = {S35259, 78, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2)*/
const Device UA2658409 = {S35260, 154, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; XT925 Build/9.8.2Q-8-XT925_VQUL-1601) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659785 = {S35261, 75, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT925 Build/9.8.2Q-8-XT925_VQL-12.2)*/
const Device UA2697312 = {S35262, 146, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; DROID RAZR HD Build/BATAKANG) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2703635 = {S35263, 151, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; XT925 Build/9.8.2Q-8-XT925_VQU-25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2732984 = {S35264, 151, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; XT925 Build/9.8.2Q-8-XT925_VQU-27) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2738273 = {S35265, 153, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT925 Build/9.8.2Q-50-XT925_VQLM-21) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2750558 = {S35266, 154, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2750975 = {S35267, 156, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778957 = {S35268, 152, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; XT925 Build/9.8.2Q-8-XT925_VQUL-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2795228 = {S35269, 153, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT925 Build/9.8.2Q-50-XT925_VQLM-20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2844020 = {S35270, 152, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; XT925 Build/9.8.0Q-97-XT925_VQU-19) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892451 = {S35271, 150, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/RAGE HD v1.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2938082 = {S35272, 147, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-79) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2946173 = {S35273, 149, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-79-1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2966044 = {S35274, 153, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT925 Build/9.8.2Q-50-XT925_VQLM-24) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2971766 = {S35275, 76, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android 4.1.2;motorola DROID RAZR HD Build/9.8.1Q-79)*/
const Device UA3093304 = {S35276, 178, { &P18472,&P18111,&P18112,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=vanquish;devm=DROID RAZR HD;man=motorola;brw=Android;shwd=720x1184;shwa=720x1134;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA894930 = {S35277, 127, { &P15364,&P5690,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/20.0.1132.11) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.11 Safari/536.11*/
const Device UA898218 = {S35278, 127, { &P15364,&P5690,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/20.0.1132.17) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.17 Safari/536.11*/
const Device UA900739 = {S35279, 127, { &P15364,&P5690,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/20.0.1132.27) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.27 Safari/536.11*/
const Device UA908315 = {S35280, 127, { &P15364,&P5690,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/20.0.1132.43) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.43 Safari/536.11*/
const Device UA910385 = {S35281, 127, { &P15364,&P5690,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/20.0.1132.47) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11*/
const Device UA911099 = {S35282, 127, { &P15364,&P5690,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/20.0.1132.47) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11*/
const Device UA911482 = {S35283, 220, { &P15364,&P5690,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.3; chromeframe/20.0.1132.47; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA911509 = {S35284, 159, { &P15364,&P5690,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/20.0.1132.47; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA912521 = {S35285, 127, { &P15364,&P5690,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/20.0.1132.57) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA912677 = {S35286, 140, { &P15364,&P5690,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/20.0.1132.57; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)*/
const Device UA912816 = {S35287, 127, { &P15364,&P5690,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/20.0.1132.57) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA924026 = {S35288, 206, { &P15364,&P5690,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/20.0.1132.57; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA1964545 = {S35289, 76, { &P15364,&P5690,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/20.0.1132.57)*/
const Device UA1983016 = {S35290, 134, { &P15364,&P5690,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/20.0.1132.57) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA2154983 = {S35291, 147, { &P15364,&P5690,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/20.0.1132.57; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) )*/
const Device UA2159008 = {S35292, 171, { &P15364,&P5690,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/20.0.1132.47; InfoPath.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2220230 = {S35293, 180, { &P15364,&P5690,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/20.0.1132.57; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA2545805 = {S35294, 195, { &P15364,&P5690,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/20.0.1132.57; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2573494 = {S35295, 252, { &P15364,&P5690,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/20.0.1132.47; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; MS-RTC LM 8)*/
const Device UA2786931 = {S35296, 231, { &P15364,&P5690,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/20.0.1132.57; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2994946 = {S35297, 362, { &P15364,&P5690,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; FBViewer-6.0.2.0; chromeframe/20.0.1132.57; FBViewer-6.0.5.0; FBViewer-5.0.1.23; FBViewer-6.0.2.0; FBViewer-6.0.5.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; FBViewer-6.0.2.0; FBViewer-6.0.5.0; FBViewer-6.0.2.0; FBViewer-6.0.5.0)*/
const Device UA70158 = {S35298, 150, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1_73_DHD-4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA77105 = {S35299, 151, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1-73_DHD-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78763 = {S35300, 148, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR Build/6.5.1-73_DHD-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78901 = {S35301, 156, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1-73_DHD-10_PS-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA361817 = {S35302, 77, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; DROID RAZR Build/6.5.1-73_DHD-11_M1-2)*/
const Device UA368963 = {S35303, 157, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1-73_DHD-11_M1-29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369161 = {S35304, 157, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M3-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA380007 = {S35305, 157, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M2-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA398287 = {S35306, 154, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR Build/6.5.1-167_DHD-14_M2-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA575539 = {S35307, 156, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1-73_DHD-11_TA-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA576933 = {S35308, 156, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1-73_DHD-11_TA-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA583608 = {S35309, 157, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M2-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622464 = {S35310, 160, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR Build/6.5.1-167_DHD-14_M2-5) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30*/
const Device UA622543 = {S35311, 156, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1-73_DHD-11_M1-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623011 = {S35312, 157, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M3-8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626102 = {S35313, 154, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR Build/6.5.1-167_DHD-14_M3-8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA629133 = {S35314, 153, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR Build/6.5.1-73_DHD-11_M1-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA634695 = {S35315, 153, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR Build/6.5.1-73_DHD-11_TA-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA649293 = {S35316, 78, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; DROID RAZR Build/6.5.1-167_DHD-14_M3-8)*/
const Device UA661363 = {S35317, 156, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1-73_DHD-90_TA-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA665382 = {S35318, 156, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1-73_DHD-11_M1-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA701894 = {S35319, 151, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1-73_DHD-91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA780221 = {S35320, 157, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1-73_DHD-11_TA-33) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1936617 = {S35321, 157, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M2-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1941202 = {S35322, 157, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M2-4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022250 = {S35323, 152, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022465 = {S35324, 153, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR Build/6.5.1-73_DHD-11_M1-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022943 = {S35325, 152, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023068 = {S35326, 153, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR Build/6.5.1-73_DHD-90_TA-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023459 = {S35327, 156, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR 4G Build/6.5.1-73_DHD-11_TA-4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024928 = {S35328, 161, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR Build/6.5.1-167_DHD-14_M3-8) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2025112 = {S35329, 157, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M3-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025491 = {S35330, 156, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us;DROID RAZR 4G Build/6.5.1-167_DHD-14_M3-8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028050 = {S35331, 156, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M3-8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029144 = {S35332, 156, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M3-8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA2029591 = {S35333, 156, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M3-8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030460 = {S35334, 157, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M3-4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030666 = {S35335, 156, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M2-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031188 = {S35336, 156, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M2-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031249 = {S35337, 156, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROIDRAZR 4G Build/6.5.1-167_DHD-14_M3-8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031824 = {S35338, 156, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M2-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA2032097 = {S35339, 156, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4GBuild/6.5.1-167_DHD-14_M2-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033559 = {S35340, 157, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR 4G Build/6.5.1-167_DHD-14_M3-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034515 = {S35341, 154, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR Build/6.5.1-167_DHD-14_M3-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2684294 = {S35342, 148, { &P18113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID RAZR Build/6.5.1-73_DHD-91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2717629 = {S35343, 143, { &P18462,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_Desire_X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719020 = {S35344, 143, { &P18462,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC Desire X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2729326 = {S35345, 65, { &P18462,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC Desire X Build/JRO03C)*/
const Device UA2755228 = {S35346, 163, { &P18462,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; HTC_Desire_X-orange-LS/2.22.75.4 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2770150 = {S35347, 154, { &P18462,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; HTC_Desire_X/2.23.114.1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2770164 = {S35348, 154, { &P18462,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; HTC_Desire_X/2.23.113.2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2784996 = {S35349, 155, { &P18462,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_Desire_X/2.20.161.11 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2797462 = {S35350, 154, { &P18462,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-at; HTC_Desire_X/2.20.169.7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2799220 = {S35351, 154, { &P18462,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; HTC_Desire_X/2.23.118.2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867396 = {S35352, 154, { &P18462,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-at; HTC_Desire_X/2.23.123.2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2869999 = {S35353, 154, { &P18462,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_Desire_X/2.23.111.4 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2893877 = {S35354, 152, { &P18462,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; HTC_Desire_X_dual_sim Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2970549 = {S35355, 154, { &P18462,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; HTC_Desire_X/2.23.114.2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2973547 = {S35356, 153, { &P18462,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; HTC_Desire_XC_dual_sim Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3012751 = {S35357, 152, { &P18462,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; HTC Desire X dual sim Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050029 = {S35358, 154, { &P18462,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-at; HTC_Desire_X/2.23.112.2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050729 = {S35359, 163, { &P18462,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; HTC_Desire_X-orange-LS/1.14.75.1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3106821 = {S35360, 153, { &P18462,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; HTC Desire XC dual sim Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3132769 = {S35361, 163, { &P18462,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; HTC_Desire_X-orange-LS/2.22.75.8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2388421 = {S35362, 74, { &P17595,&P17297,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:18.0) Gecko/X Firefox/18.0*/
const Device UA427339 = {S35363, 16, { &P18254,&P17017,&P18245,&P18092 }};	/*Useragent - RSS Scout 0.11.2*/
const Device UA427524 = {S35364, 40, { &P18254,&P17017,&P18245,&P18092 }};	/*Useragent - OppO 1.0 ( http://www.inboundscore.com )*/
const Device UA441484 = {S35365, 17, { &P18254,&P17017,&P18245,&P18092 }};	/*Useragent - news.me extractor*/
const Device UA903622 = {S35366, 54, { &P18254,&P17017,&P18245,&P18092 }};	/*Useragent - LinkChecker/7.9 (+http://linkchecker.sourceforge.net/)*/
const Device UA923815 = {S35367, 46, { &P18254,&P17017,&P18245,&P18092 }};	/*Useragent - StatusNet/1.1.0-release (Fight for Your Right)*/
const Device UA923826 = {S35368, 56, { &P18254,&P17017,&P18245,&P18092 }};	/*Useragent - SSL Labs (https://www.ssllabs.com/about/assessment.html)*/
const Device UA924815 = {S35369, 62, { &P18254,&P17017,&P18245,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; LinkJumper/0.1.2; +Someday ya skazhu)*/
const Device UA1996221 = {S35370, 84, { &P18254,&P17017,&P18245,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; IODC-Internet Engineering Odysseus 001; +http://iodc.co.uk)*/
const Device UA2041314 = {S35371, 48, { &P18254,&P17017,&P18245,&P18092 }};	/*Useragent - http://www.americanexpress.com [amex:prodSearch]*/
const Device UA2155834 = {S35372, 42, { &P18254,&P17017,&P18245,&P18092 }};	/*Useragent - check_http/v1.4.16 (nagios-plugins 1.4.16)*/
const Device UA2751996 = {S35373, 19, { &P18254,&P17017,&P18245,&P18092 }};	/*Useragent - Tumblr/14.0.835.186*/
const Device UA2752389 = {S35374, 75, { &P18254,&P17017,&P18245,&P18092 }};	/*Useragent - Mechanize/2.6.0 Ruby/1.9.2p290 (http://github.com/sparklemotion/mechanize/)*/
const Device UA347712 = {S35375, 101, { &P15364,&P18110,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA359703 = {S35376, 136, { &P15364,&P18110,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7,gzip(gfe) (via translate.google.com)*/
const Device UA366231 = {S35377, 99, { &P15364,&P18110,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7*/
const Device UA366317 = {S35378, 142, { &P15364,&P18110,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Jolicloud Linux i686) AppleWebKit/535.7 (KHTML, like Gecko) Joli OS/1.2 Chromium/16.0.912.63 Chrome/16.0.912.63 Safari/535.7*/
const Device UA687688 = {S35379, 108, { &P15364,&P18110,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA912393 = {S35380, 184, { &P15364,&P18110,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7 Safari/532.4*/
const Device UA2315638 = {S35381, 99, { &P15364,&P18110,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.7 (KHTML	 like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA2315732 = {S35382, 142, { &P15364,&P18110,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Jolicloud Linux i686) AppleWebKit/535.7 (KHTML	 like Gecko) Joli OS/1.2 Chromium/16.0.912.63 Chrome/16.0.912.63 Safari/535.7*/
const Device UA2331547 = {S35383, 94, { &P15364,&P18110,&P17400,&P18092 }};	/*Useragent - "Mozilla/5.0(X11;Linuxx86_64)AppleWebKit/535.7(KHTML,likeGecko)Chrome/16.0.912.77Safari/535.7"*/
const Device UA2421227 = {S35384, 89, { &P15364,&P18110,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA2674510 = {S35385, 106, { &P15364,&P18110,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/16.0.1410.63 Safari/537.31*/
const Device UA2674623 = {S35386, 109, { &P15364,&P18110,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/16.0.1410.63 Safari/537.31*/
const Device UA2674795 = {S35387, 106, { &P15364,&P18110,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/16.0.1410.63 Safari/537.31*/
const Device UA2675189 = {S35388, 109, { &P15364,&P18110,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/16.0.1410.63 Safari/537.31*/
const Device UA2675207 = {S35389, 106, { &P15364,&P18110,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/16.0.1410.63 Safari/537.31*/
const Device UA2714941 = {S35390, 109, { &P15364,&P18110,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/16.0.1410.63 Safari/537.31*/
const Device UA2718189 = {S35391, 106, { &P15364,&P18110,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/16.0.1410.63 Safari/537.31*/
const Device UA90889 = {S35392, 105, { &P17346,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405*/
const Device UA90894 = {S35393, 103, { &P17346,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334*/
const Device UA90888 = {S35394, 92, { &P17346,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)*/
const Device UA234031 = {S35395, 90, { &P17346,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko)*/
const Device UA361455 = {S35396, 50, { &P17346,&P17146,&P17596,&P18092 }};	/*Useragent - Miu Ptt 1.5.2 (iPod touch; iPhone OS 5.0.1; zh_TW)*/
const Device UA456692 = {S35397, 246, { &P17346,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 5_0 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/5.0;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA623197 = {S35398, 250, { &P17346,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 5_0_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/5.0.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA793759 = {S35399, 133, { &P17346,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 iPod4,1 BingWeb/3.03.1428.X*/
const Device UA1948935 = {S35400, 131, { &P17346,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 iPod4,1 BingWeb/3.02.1641.X*/
const Device UA2202395 = {S35401, 229, { &P17346,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/5.0.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]*/
const Device UA2203268 = {S35402, 225, { &P17346,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 [FBAN/FBIOS;FBAV/5.0;FBBV/47423;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/5.0;FBSS/2; FBCR/;FBID/phone;FBLC/en_US]*/
const Device UA2261365 = {S35403, 153, { &P17346,&P17146,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari IPhoneAppVersion/2.1 IPhoneWebkitWrapper*/
const Device UA2387479 = {S35404, 74, { &P17595,&P17297,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:19.0) Gecko/X Firefox/19.0*/
const Device UA582080 = {S35405, 152, { &P16366,&P17342,&P17454,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8600/S8600DDKK4; U; Bada/2.0; en-us) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile WVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA623183 = {S35406, 142, { &P18300,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T769 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA694176 = {S35407, 66, { &P18300,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-T769 Build/GINGERBREAD)*/
const Device UA1957039 = {S35408, 141, { &P18300,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T769Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1979224 = {S35409, 141, { &P18300,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.6; en-us; SGH-T769 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1983093 = {S35410, 141, { &P18300,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us;SGH-T769 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1989318 = {S35411, 141, { &P18300,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; SGH-T769 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1992392 = {S35412, 141, { &P18300,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T769 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA2475978 = {S35413, 149, { &P18300,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T769 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2979892 = {S35414, 140, { &P18300,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SGH-T769 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20 = {S35415, 152, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-SGH-I897/I897UCJK1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA4936 = {S35416, 134, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-I897 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA7070 = {S35417, 152, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-SGH-I897/I897UCKB1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA7192 = {S35418, 152, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-SGH-I897/I897UCJL2 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA7463 = {S35419, 150, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SAMSUNG-SGH-I897 Build/FROYO XWJS5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11661 = {S35420, 140, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1 JS3; en-us; SGH-I897 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11663 = {S35421, 140, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1 JS5; en-us; SGH-I897 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA26887 = {S35422, 142, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-SGH-I897 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA26894 = {S35423, 144, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SAMSUNG-SGH-I897 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA26895 = {S35424, 136, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-I897 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32528 = {S35425, 152, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-SGH-I897/I897UCJL1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32725 = {S35426, 152, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-SGH-I897/I897UCJI6 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32731 = {S35427, 152, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-SGH-I897/I897UCJJ4 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA37851 = {S35428, 151, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2; en-us; SAMSUNG-SGH-I897/I897UCKB1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA37853 = {S35429, 151, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-SGH-I897/I897UCKB1Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA37862 = {S35430, 151, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2; en-us; SAMSUNG-SGH-I897/I897UCKB1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA37850 = {S35431, 151, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us;SAMSUNG-SGH-I897/I897UCKB1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51398 = {S35432, 150, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ko-; SAMSUNG-SGH-I897/I897UCKB1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA68054 = {S35433, 148, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1 JS8; en-gb; SAMSUNG-SGH-I897 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73541 = {S35434, 78, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/0.9.1 (Linux; U; Android 2.2; en_US; SAMSUNG-SGH-I897 Build/FROYO.UCKB2)*/
const Device UA74260 = {S35435, 152, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-SGH-I897/I897UCJJ3 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78090 = {S35436, 141, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-SGH-I897Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA153505 = {S35437, 66, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; SAMSUNG-SGH-I897 Build/FROYO)*/
const Device UA448561 = {S35438, 134, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-I897 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA780277 = {S35439, 149, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SAMSUNG-SGH-I897 Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA1861993 = {S35440, 68, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SAMSUNG-SGH-I897 Build/FROYO)*/
const Device UA2018564 = {S35441, 148, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1 JS7; en-us; SAMSUNG-SGH-I897 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019125 = {S35442, 138, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SGH-I897 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2021647 = {S35443, 140, { &P12496,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-; SAMSUNG-SGH-I897 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2661615 = {S35444, 132, { &P22606,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N5110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2709873 = {S35445, 61, { &P22606,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N5110 Build/JZO54K)*/
const Device UA79890 = {S35446, 81, { &P15364,&P17285,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/5*/
const Device UA95405 = {S35447, 116, { &P15364,&P17285,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.55 Safari/533.4*/
const Device UA252009 = {S35448, 140, { &P18195,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; YP-G70 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA371284 = {S35449, 76, { &P18195,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.5; en_US; YP-G70 Build/GINGERBREAD.UEKI8)*/
const Device UA693411 = {S35450, 64, { &P18195,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; YP-G70 Build/GINGERBREAD)*/
const Device UA750645 = {S35451, 140, { &P18195,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; YP-G70 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023468 = {S35452, 64, { &P18195,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; YP-G70 Build/GINGERBREAD)*/
const Device UA2256934 = {S35453, 138, { &P18195,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; YP-G70 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1959492 = {S35454, 139, { &P15880,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB865 Build/6.7.2_169) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1995900 = {S35455, 147, { &P15880,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB865 Build/6.7.2-226-EDICS-8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2193363 = {S35456, 143, { &P15880,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MB865 Build/6.7.2_EDEM-18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2193478 = {S35457, 148, { &P15880,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2236521 = {S35458, 148, { &P15880,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ME865 Build/6.7.2_GC-220-EDS-3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2239269 = {S35459, 70, { &P15880,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10)*/
const Device UA2365119 = {S35460, 149, { &P15880,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ME865 Build/6.7.2_GC-180-EDS-20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2384782 = {S35461, 141, { &P15880,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2418429 = {S35462, 151, { &P15880,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB865 Build/67.21.125 by timmytim) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2431109 = {S35463, 143, { &P15880,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MB865 Build/6.7.2_EDEM-20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2452645 = {S35464, 140, { &P15880,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ME865 Build/6.7.2_GC-180-EDS-20) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2552569 = {S35465, 70, { &P15880,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ME865 Build/6.7.2_GC-220-EDS-3)*/
const Device UA2653073 = {S35466, 146, { &P15880,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2727425 = {S35467, 143, { &P15880,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; MB865 Build/6.7.2_EDEM-17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3117821 = {S35468, 138, { &P15880,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MB865 Build/6.7.2-226-EDICS-8) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3117935 = {S35469, 162, { &P15880,&P18066,&P18062,&P18092 }};	/*Useragent - MOT-MB865/ Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB865 Build/6.7.2-226-EDICS-10) 1617 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA63776 = {S35470, 78, { &P14888,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry9650/5.0.0.1006 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/105*/
const Device UA2404852 = {S35471, 57, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.12*/
const Device UA2427090 = {S35472, 80, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.12*/
const Device UA2427892 = {S35473, 79, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.12*/
const Device UA2429901 = {S35474, 80, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.12*/
const Device UA2430884 = {S35475, 79, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.12*/
const Device UA2431988 = {S35476, 67, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.12,gzip(gfe)*/
const Device UA2434684 = {S35477, 79, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.12*/
const Device UA2437153 = {S35478, 64, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; WOW64) Presto/2.12.388 Version/12.12*/
const Device UA2437490 = {S35479, 79, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.12*/
const Device UA2459570 = {S35480, 79, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.12*/
const Device UA2460781 = {S35481, 79, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.12*/
const Device UA2460963 = {S35482, 109, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)) Presto/2.12.388 Version/12.12*/
const Device UA2460974 = {S35483, 80, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.12*/
const Device UA2461111 = {S35484, 79, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.12*/
const Device UA2461217 = {S35485, 80, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.12*/
const Device UA2467649 = {S35486, 62, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0) Opera 12.12*/
const Device UA2468364 = {S35487, 79, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.12*/
const Device UA2468637 = {S35488, 80, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.12*/
const Device UA2468792 = {S35489, 79, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.12*/
const Device UA2468814 = {S35490, 116, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)) Presto/2.12.388 Version/12.12*/
const Device UA2468928 = {S35491, 102, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; compatible; MSIE 7.0; Windows NT 6.0) Presto/2.12.388 Version/12.12*/
const Device UA2469074 = {S35492, 80, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.12*/
const Device UA2469184 = {S35493, 79, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.12*/
const Device UA2484781 = {S35494, 79, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.12*/
const Device UA2484846 = {S35495, 79, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.12*/
const Device UA2486137 = {S35496, 68, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:2.0) Gecko/X Firefox/4.0 Opera 12.12*/
const Device UA2486279 = {S35497, 79, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.12*/
const Device UA2486398 = {S35498, 79, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.12*/
const Device UA2506488 = {S35499, 79, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.12*/
const Device UA2506813 = {S35500, 95, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; compatible; MSIE 7.0; Windows NT 6.0) Presto/2.12.388 Version/12.12*/
const Device UA2507150 = {S35501, 79, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.12*/
const Device UA2507288 = {S35502, 106, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.12*/
const Device UA2656854 = {S35503, 79, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.12*/
const Device UA2697651 = {S35504, 79, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.12*/
const Device UA2729033 = {S35505, 80, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.12*/
const Device UA2778536 = {S35506, 80, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.12*/
const Device UA2846215 = {S35507, 79, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.12*/
const Device UA2846773 = {S35508, 79, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.12*/
const Device UA2846898 = {S35509, 80, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.12*/
const Device UA2847664 = {S35510, 80, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.12*/
const Device UA2866630 = {S35511, 79, { &P15364,&P17285,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.12*/
const Device UA459194 = {S35512, 155, { &P14114,&P5629,&P18161,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Silk/1.0.13.328_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Silk-Accelerated=true*/
const Device UA615114 = {S35513, 154, { &P14114,&P5629,&P18161,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Silk/1.0.22.79_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Silk-Accelerated=true*/
const Device UA623418 = {S35514, 151, { &P14114,&P5629,&P18161,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Silk/1.1.0-84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Silk-Accelerated=true*/
const Device UA626608 = {S35515, 151, { &P14114,&P5629,&P18161,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Silk/1.1.0-80) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Silk-Accelerated=true*/
const Device UA1948885 = {S35516, 191, { &P14114,&P5629,&P18161,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Silk/1.0.13.81_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Silk-Accelerated=true,gzip(gfe) (via translate.google.com)*/
const Device UA2408357 = {S35517, 155, { &P14114,&P5629,&P18161,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Silk/1.0.22.153_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Silk-Accelerated=true*/
const Device UA1999169 = {S35518, 74, { &P17595,&P17297,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:16.0) Gecko/X Firefox/16.0*/
const Device UA47711 = {S35519, 146, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB865 Build/5.5.1-175_EDIFW2-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47717 = {S35520, 137, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB865 Build/5.5.1-134) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47712 = {S35521, 137, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB865 Build/5.5.1-149) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60606 = {S35522, 137, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB865 Build/5.5.1_100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70405 = {S35523, 147, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB865 Build/5.5.1-175_EDFFW1-13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71659 = {S35524, 147, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB865 Build/5.5.1-175_EDFFW1-16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73332 = {S35525, 147, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB865 Build/5.5.1-175_EDFFW1-17) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73861 = {S35526, 146, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB865 Build/5.5.1-175_EDFFW1-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74946 = {S35527, 147, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB865 Build/5.5.1-175_EDFFW1-11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78782 = {S35528, 145, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB865 Build/5.5.1-175_EDMR1.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369819 = {S35529, 141, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; ME865 Build/5.5.1-1_GC-89) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370533 = {S35530, 76, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.6; en_US; MB865 Build/5.5.1-175_EDMR1.25)*/
const Device UA371288 = {S35531, 70, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; MB865 Build/5.5.1-175_EDMR1.25)*/
const Device UA622368 = {S35532, 154, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB865 Build/5.5.1-175_EDMR1.25) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.1 Mobile Safari/534.30*/
const Device UA622945 = {S35533, 146, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB865 Build/5.5.1-175_EDMR1.25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA635750 = {S35534, 71, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; MB865 Build/5.5.1-175_EDFFW1-16)*/
const Device UA653533 = {S35535, 146, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB865 Build/5.5.1-175_EDMR1.22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA701303 = {S35536, 141, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MB865 Build/5.5.1-EDEM-28) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA715376 = {S35537, 141, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MB865 Build/5.5.1-EDEM-27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA860020 = {S35538, 144, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MB865 Build/5.5.1-EDEM-28-37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA886014 = {S35539, 144, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MB865 Build/5.5.1-256_EDL-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1906139 = {S35540, 146, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB865 Build/5.5.1-175_EDMR1.15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1940302 = {S35541, 150, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME865 Build/5.5.1-1_GC-90_EDISON-6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1952624 = {S35542, 139, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME865 Build/5.5.1-1_GC-90_EDISON-11) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1953504 = {S35543, 142, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ME865 Build/5.5.1-1_GC-109) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1955378 = {S35544, 144, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB865 Build/5.5.1-256_EDL-27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1993629 = {S35545, 144, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MB865 Build/5.5.1-256_EDL-34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020089 = {S35546, 145, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB865 Build/5.5.1-175_EDMR1.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022359 = {S35547, 150, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ME865 Build/5.5.1-1_GC-86_EDSCHN-6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022543 = {S35548, 151, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ME865 Build/5.5.1-1_GC-90_EDISON-11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022695 = {S35549, 151, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ME865 Build/5.5.1-1_GC-90_EDSHKTW-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031325 = {S35550, 130, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME865 Build/5.5.1-1_GC-109) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2033153 = {S35551, 135, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME865 Build/5.5.1-1_GC-109) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2033458 = {S35552, 139, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME865 Build/5.5.1-1_GC-90_EDSHKTW-2) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2811853 = {S35553, 144, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME865 Build/5.5.1-1_GC-90_EDISON-11) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3070530 = {S35554, 146, { &P15880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MB865 Build/5.5.1-256_EDL-34.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA366198 = {S35555, 142, { &P15650,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M580 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369130 = {S35556, 152, { &P15650,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M580BST Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA623037 = {S35557, 145, { &P15650,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M580BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA660995 = {S35558, 69, { &P15650,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SPH-M580BST Build/GINGERBREAD)*/
const Device UA758582 = {S35559, 66, { &P15650,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SPH-M580 Build/GINGERBREAD)*/
const Device UA2023836 = {S35560, 143, { &P15650,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; -us; SPH-M580BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024467 = {S35561, 149, { &P15650,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M580 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2026130 = {S35562, 140, { &P15650,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; -us; SPH-M580 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA29887 = {S35563, 127, { &P12280,&P17084,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B117 Safari/532.9*/
const Device UA29919 = {S35564, 101, { &P12280,&P17084,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko)*/
const Device UA29920 = {S35565, 114, { &P12280,&P17084,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B117*/
const Device UA29978 = {S35566, 134, { &P12280,&P17084,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/X.X (KHTML, like Gecko) Version/X.X Mobile/XXXX Safari/X.X*/
const Device UA301127 = {S35567, 114, { &P12280,&P17084,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/9Z999*/
const Device UA455736 = {S35568, 244, { &P12280,&P17084,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.1;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA696287 = {S35569, 112, { &P12280,&P17084,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8J2*/
const Device UA2553533 = {S35570, 140, { &P12280,&P17084,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone 4_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Grindr/1.8.5 (iPhone2,1/4.1)*/
const Device UA24403 = {S35571, 142, { &P14872,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA33224 = {S35572, 162, { &P14872,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Bonsai/Kernel_v5.0.0-EF02; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA33295 = {S35573, 142, { &P14872,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49975 = {S35574, 152, { &P14872,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android RandomROM 2.3.4; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA257670 = {S35575, 66, { &P14872,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SPH-D700 Build/GINGERBREAD)*/
const Device UA257799 = {S35576, 160, { &P14872,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Samurai-2.6.35_gen1.0.2; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA351195 = {S35577, 154, { &P14872,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Gingerbread 2.3.5; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368854 = {S35578, 149, { &P14872,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA381715 = {S35579, 66, { &P14872,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SPH-D700 Build/GINGERBREAD)*/
const Device UA420751 = {S35580, 66, { &P14872,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SPH-D700 Build/GINGERBREAD)*/
const Device UA622578 = {S35581, 142, { &P14872,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633899 = {S35582, 142, { &P14872,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA689292 = {S35583, 159, { &P14872,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android nubernel-2.6.35_v0.0.1; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA748206 = {S35584, 169, { &P14872,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android EI22_IAP_Samurai-2.6.35_gen1.0.4; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019049 = {S35585, 66, { &P14872,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SPH-D700 Build/GINGERBREAD)*/
const Device UA2021210 = {S35586, 159, { &P14872,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android nubernel-2.6.35_v0.0.0; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021216 = {S35587, 168, { &P14872,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en_us; SPH-D700 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)*/
const Device UA2022206 = {S35588, 66, { &P14872,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; SPH-D700 Build/GINGERBREAD)*/
const Device UA2024341 = {S35589, 140, { &P14872,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027676 = {S35590, 145, { &P14872,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-D700 Build/Mortify ROM v2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA360380 = {S35591, 137, { &P13644,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PC36100 Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA368851 = {S35592, 150, { &P13644,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3 slow churned; en-us; PC36100 Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA656477 = {S35593, 140, { &P13644,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Evo 4G Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA686097 = {S35594, 138, { &P13644,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PC36100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA749044 = {S35595, 138, { &P13644,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PC36100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA770238 = {S35596, 134, { &P13644,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; EVO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA802947 = {S35597, 137, { &P13644,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PC36100 Build/IBZ40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA804195 = {S35598, 138, { &P13644,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PC36100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA862569 = {S35599, 140, { &P13644,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Evo 4G Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA898773 = {S35600, 56, { &P13644,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; EVO Build/IML74K)*/
const Device UA907965 = {S35601, 138, { &P13644,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PC36100 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1930546 = {S35602, 137, { &P13644,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; PC36100 Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1979215 = {S35603, 134, { &P13644,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EVO Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1994370 = {S35604, 138, { &P13644,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PC36100 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030617 = {S35605, 147, { &P13644,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sprint APA9292KT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031054 = {S35606, 56, { &P13644,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; EVO Build/IMM76D)*/
const Device UA2031238 = {S35607, 60, { &P13644,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; PC36100 Build/IMM76I)*/
const Device UA2355790 = {S35608, 140, { &P13644,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EVO Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2496777 = {S35609, 128, { &P13644,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; DEVO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2570289 = {S35610, 134, { &P13644,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; ODYS-EVO Build/ODYS-EVO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2746751 = {S35611, 140, { &P13644,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; EVO Build/ElegenciaRom) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2759237 = {S35612, 143, { &P13644,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.2; zh-cn; HTC EVO 4G+ For Sprint Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA901537 = {S35613, 86, { &P18095,&P18109,&P18525,&P18092 }};	/*Useragent - Opera/9.80 (Series 60; Opera Mini/6.0.24095/24.760; U; en) Presto/2.5.25 Version/10.54*/
const Device UA2431331 = {S35614, 88, { &P18095,&P18109,&P18525,&P18092 }};	/*Useragent - Opera/9.80 (Series 60; Opera Mini/6.0.24455/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA272193 = {S35615, 96, { &P18254,&P7480,&P18245,&P18165 }};	/*Useragent - Mozilla/5.0 (compatible; cdlwas_bot; heritrix/1.14.1 +http://webarchives.cdlib.org/p/webmasters)*/
const Device UA331849 = {S35616, 89, { &P18254,&P7480,&P18245,&P18165 }};	/*Useragent - webcrawler (compatible; heritrix/1.14.4 ++http://www.onb.ac.at/about/webarchivierung.htm)*/
const Device UA388382 = {S35617, 82, { &P18254,&P7480,&P18245,&P18165 }};	/*Useragent - Arquivo-web-crawler  (compatible; heritrix/1.14.3 +http://arquivo.pt/faq-crawling)*/
const Device UA407886 = {S35618, 67, { &P18254,&P7480,&P18245,&P18165 }};	/*Useragent - Mozilla/5.0 (compatible; heritrix/1.14.3 +http://www.accelobot.com)*/
const Device UA436661 = {S35619, 71, { &P18254,&P7480,&P18245,&P18165 }};	/*Useragent - Mozilla/5.0 (compatible; heritrix/3.1.0-RC1 +http://vefsafn.is/crawler)*/
const Device UA615293 = {S35620, 75, { &P18254,&P7480,&P18245,&P18165 }};	/*Useragent - Mozilla/5.0 (compatible; heritrix/1.14.4 +http://netarkivet.dk/webcrawler/)*/
const Device UA1896760 = {S35621, 95, { &P18254,&P7480,&P18245,&P18165 }};	/*Useragent - Mozilla/5.0 (compatible;archive.org_bot/heritrix-1.9.0-X +http://pandora.nla.gov.au/crawl.html)*/
const Device UA1897085 = {S35622, 66, { &P18254,&P7480,&P18245,&P18165 }};	/*Useragent - Mozilla/5.0 (compatible; heritrix/1.x.x +http://www.accelobot.com)*/
const Device UA1897384 = {S35623, 69, { &P18254,&P7480,&P18245,&P18165 }};	/*Useragent - Mozilla/5.0 (compatible; worio bot heritrix/1.10.0 +http://worio.com)*/
const Device UA1940826 = {S35624, 67, { &P18254,&P7480,&P18245,&P18165 }};	/*Useragent - Mozilla/5.0 (compatible; heritrix/1.3.0 http://archive.crawler.org)*/
const Device UA2236558 = {S35625, 97, { &P18254,&P7480,&P18245,&P18165 }};	/*Useragent - Mozilla/5.0 (compatible; heritrix/3.1.2-SNAPSHOT-X.X +http://archive.org/details/archive.org_bot)*/
const Device UA2309586 = {S35626, 65, { &P18254,&P7480,&P18245,&P18165 }};	/*Useragent - HopperBot/1.1 (compatible; heritrix/3.1.0 +http://www.hopper.com)*/
const Device UA2471067 = {S35627, 101, { &P18254,&P7480,&P18245,&P18165 }};	/*Useragent - Mozilla/5.0 (compatible; heritrix/3.1.2-SNAPSHOT-X.X +http://www.archive.org/details/archive.org_bot)*/
const Device UA2563403 = {S35628, 62, { &P18254,&P7480,&P18245,&P18165 }};	/*Useragent - Mozilla/5.0 (compatible; heritrix/3.1.1 +http://www.baidu.com)*/
const Device UA2710368 = {S35629, 72, { &P18254,&P7480,&P18245,&P18165 }};	/*Useragent - citeseerxbot (compatible; heritrix/1.14.4 +http://citeseerx.ist.psu.edu)*/
const Device UA2472035 = {S35630, 57, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.13*/
const Device UA2487359 = {S35631, 68, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:2.0) Gecko/X Firefox/4.0 Opera 12.13*/
const Device UA2487652 = {S35632, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.13*/
const Device UA2487856 = {S35633, 80, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.13*/
const Device UA2488266 = {S35634, 64, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; WOW64) Presto/2.12.388 Version/12.13*/
const Device UA2488277 = {S35635, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.13*/
const Device UA2490067 = {S35636, 57, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2) Presto/2.12.388 Version/12.13*/
const Device UA2491889 = {S35637, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.13*/
const Device UA2491909 = {S35638, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.13*/
const Device UA2491922 = {S35639, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.13*/
const Device UA2492124 = {S35640, 67, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.13,gzip(gfe)*/
const Device UA2492126 = {S35641, 80, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.13*/
const Device UA2492130 = {S35642, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.13*/
const Device UA2492366 = {S35643, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.13*/
const Device UA2492632 = {S35644, 78, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; Edition Campaign 21) Presto/2.12.388 Version/12.13*/
const Device UA2508162 = {S35645, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.13*/
const Device UA2508293 = {S35646, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.13*/
const Device UA2508583 = {S35647, 80, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.13*/
const Device UA2508591 = {S35648, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4848)) Presto/2.12.388 Version/12.13*/
const Device UA2508708 = {S35649, 80, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.13*/
const Device UA2508709 = {S35650, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4139)) Presto/2.12.388 Version/12.13*/
const Device UA2508914 = {S35651, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5831)) Presto/2.12.388 Version/12.13*/
const Device UA2509497 = {S35652, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.13*/
const Device UA2510094 = {S35653, 80, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.13*/
const Device UA2510288 = {S35654, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.13*/
const Device UA2510463 = {S35655, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5997)) Presto/2.12.388 Version/12.13*/
const Device UA2510655 = {S35656, 80, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5231)) Presto/2.12.388 Version/12.13*/
const Device UA2510770 = {S35657, 80, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5265)) Presto/2.12.388 Version/12.13*/
const Device UA2510931 = {S35658, 80, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03773)) Presto/2.12.388 Version/12.13*/
const Device UA2511028 = {S35659, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.13*/
const Device UA2511043 = {S35660, 62, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1) Opera 12.13*/
const Device UA2511259 = {S35661, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.13*/
const Device UA2511334 = {S35662, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.13*/
const Device UA2511404 = {S35663, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5966)) Presto/2.12.388 Version/12.13*/
const Device UA2511466 = {S35664, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.13*/
const Device UA2511632 = {S35665, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5711)) Presto/2.12.388 Version/12.13*/
const Device UA2511736 = {S35666, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.13*/
const Device UA2511817 = {S35667, 80, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.13*/
const Device UA2511835 = {S35668, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.13*/
const Device UA2512088 = {S35669, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.13*/
const Device UA2537021 = {S35670, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.13*/
const Device UA2537044 = {S35671, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4156)) Presto/2.12.388 Version/12.13*/
const Device UA2537270 = {S35672, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5745)) Presto/2.12.388 Version/12.13*/
const Device UA2537938 = {S35673, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.13*/
const Device UA2560528 = {S35674, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.13*/
const Device UA2561198 = {S35675, 80, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5218)) Presto/2.12.388 Version/12.13*/
const Device UA2563072 = {S35676, 80, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5328)) Presto/2.12.388 Version/12.13*/
const Device UA2563200 = {S35677, 74, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1) Presto/2.12.388 Version/12.13 Yandex.Translate*/
const Device UA2580511 = {S35678, 80, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.13*/
const Device UA2580571 = {S35679, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4133)) Presto/2.12.388 Version/12.13*/
const Device UA2583059 = {S35680, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.13*/
const Device UA2597106 = {S35681, 80, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5244)) Presto/2.12.388 Version/12.13*/
const Device UA2598675 = {S35682, 80, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.13*/
const Device UA2599686 = {S35683, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6035)) Presto/2.12.388 Version/12.13*/
const Device UA2599998 = {S35684, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.13*/
const Device UA2659536 = {S35685, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.8 (build 4120)) Presto/2.12.388 Version/12.13*/
const Device UA2659565 = {S35686, 106, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.13*/
const Device UA2659716 = {S35687, 80, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5267)) Presto/2.12.388 Version/12.13*/
const Device UA2660056 = {S35688, 80, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5308)) Presto/2.12.388 Version/12.13*/
const Device UA2660436 = {S35689, 80, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5196)) Presto/2.12.388 Version/12.13*/
const Device UA2699647 = {S35690, 80, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03757)) Presto/2.12.388 Version/12.13*/
const Device UA2700929 = {S35691, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6048)) Presto/2.12.388 Version/12.13*/
const Device UA2701991 = {S35692, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4930)) Presto/2.12.388 Version/12.13*/
const Device UA2702852 = {S35693, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5667)) Presto/2.12.388 Version/12.13*/
const Device UA2706754 = {S35694, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 5990)) Presto/2.12.388 Version/12.13*/
const Device UA2748613 = {S35695, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6032)) Presto/2.12.388 Version/12.13*/
const Device UA2751266 = {S35696, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6089)) Presto/2.12.388 Version/12.13*/
const Device UA2800572 = {S35697, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.13*/
const Device UA2850521 = {S35698, 80, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.7 (build 03790)) Presto/2.12.388 Version/12.13*/
const Device UA2850604 = {S35699, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5704)) Presto/2.12.388 Version/12.13*/
const Device UA2850768 = {S35700, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 5993)) Presto/2.12.388 Version/12.13*/
const Device UA2851214 = {S35701, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.9 (build 4841)) Presto/2.12.388 Version/12.13*/
const Device UA2851727 = {S35702, 80, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 5.10 (build 5282)) Presto/2.12.388 Version/12.13*/
const Device UA2855585 = {S35703, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6003)) Presto/2.12.388 Version/12.13*/
const Device UA2855593 = {S35704, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6054)) Presto/2.12.388 Version/12.13*/
const Device UA2858177 = {S35705, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6042)) Presto/2.12.388 Version/12.13*/
const Device UA2859145 = {S35706, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6081)) Presto/2.12.388 Version/12.13*/
const Device UA2990056 = {S35707, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6007)) Presto/2.12.388 Version/12.13*/
const Device UA3047287 = {S35708, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.13*/
const Device UA3071504 = {S35709, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 6.0 (build 6044)) Presto/2.12.388 Version/12.13*/
const Device UA3101515 = {S35710, 79, { &P15364,&P5690,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; MRA 8.0 (build 6008)) Presto/2.12.388 Version/12.13*/
const Device UA33157 = {S35711, 139, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; U8650 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA69978 = {S35712, 152, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-tw; HUAWEI_IDEOS_U8650 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA239653 = {S35713, 146, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HUAWEI SONIC Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369124 = {S35714, 76, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GGR Sonic Next Gen Build/HuaweiU8650)*/
const Device UA369093 = {S35715, 152, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-hr; GGR Sonic Next Gen Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA396968 = {S35716, 143, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; U8650 Build/HuaweiU8650B836) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA397779 = {S35717, 139, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; U8650 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA654665 = {S35718, 127, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SONIC Build/HuaweiU8660) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA655721 = {S35719, 139, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; U8650 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA667557 = {S35720, 63, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; U8650 Build/HuaweiU8650)*/
const Device UA677284 = {S35721, 147, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; U8650 Build/HuaweiU8650B835SP01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA689114 = {S35722, 139, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; el-gr; U8650 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA713080 = {S35723, 141, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-pt; U8650-1 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA722861 = {S35724, 145, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HuaweiU8650 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA751033 = {S35725, 139, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-cn; SONIC Build/HuaweiU8660) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1930011 = {S35726, 139, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-cn; SONIC Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1940106 = {S35727, 152, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; HUAWEI_IDEOS_U8650 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1941029 = {S35728, 147, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; Kyivstar Aqua Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1944902 = {S35729, 141, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; MTC 955 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1947037 = {S35730, 63, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; U8650 Build/HuaweiU8650)*/
const Device UA1975911 = {S35731, 143, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; U8650 Build/HuaweiU8650B882) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020224 = {S35732, 141, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; MTC 955 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021571 = {S35733, 67, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; U8650 Build/HuaweiU8650B836)*/
const Device UA2022593 = {S35734, 141, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; tr-tr; U8650-1 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022733 = {S35735, 70, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HUAWEI SONIC Build/HuaweiU8650)*/
const Device UA2023112 = {S35736, 127, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SONIC Build/HuaweiU8650) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2023220 = {S35737, 135, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; u8650 Build/benru89) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027551 = {S35738, 132, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SONIC Build/HuaweiU8660) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2032611 = {S35739, 63, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SONIC Build/HuaweiU8660)*/
const Device UA2034360 = {S35740, 141, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; U8650-1 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2375811 = {S35741, 143, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; U8650 Build/HuaweiU8650B836) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2410047 = {S35742, 145, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-in; HuaweiU8650 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2544104 = {S35743, 132, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SONIC Build/HuaweiU8650) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2577034 = {S35744, 159, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; Huawei u8650 Build/Lunar Prime; Lunar Prime) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2684005 = {S35745, 63, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; U8650 Build/HuaweiU8650)*/
const Device UA2720619 = {S35746, 133, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; U8650 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2797063 = {S35747, 143, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; U8650 Build/HuaweiU8650B888) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2818034 = {S35748, 161, { &P12935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; Lucky Ultra Sonic U8650 Build/HuaweiU8650B836) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA96718 = {S35749, 125, { &P17595,&P17296,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.55 Safari/533.4*/
const Device UA167372 = {S35750, 126, { &P17595,&P17296,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.127 Safari/533.4*/
const Device UA204513 = {S35751, 124, { &P17595,&P17296,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/533.2 (KHTML, like Gecko) Chrome/5.0.342.9 Safari/533.2*/
const Device UA272945 = {S35752, 126, { &P17595,&P17296,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.127 Safari/533.4*/
const Device UA285779 = {S35753, 124, { &P17595,&P17296,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/533.2 (KHTML, like Gecko) Chrome/5.0.342.9 Safari/533.2*/
const Device UA413444 = {S35754, 125, { &P17595,&P17296,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.86 Safari/533.4*/
const Device UA428761 = {S35755, 126, { &P17595,&P17296,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; fr-FR) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.126 Safari/533.4*/
const Device UA525941 = {S35756, 126, { &P17595,&P17296,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.127 Safari/533.4*/
const Device UA893330 = {S35757, 125, { &P17595,&P17296,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.86 Safari/533.4*/
const Device UA2720732 = {S35758, 124, { &P17595,&P17296,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/5.0.354.0 Safari/533.3*/
const Device UA8720 = {S35759, 154, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; DL DesireHD v2.0 internal Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10680 = {S35760, 228, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; HTCA9191/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10690 = {S35761, 142, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; iw-; HTC_DesireHD_A9191 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10780 = {S35762, 151, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC_DesireHD-orange-LS Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10787 = {S35763, 146, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-tw; HTC_DesireHD_A9191 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA14611 = {S35764, 153, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Desire HD 1.39.161.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA14637 = {S35765, 150, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; HTC Desire HD 1.32.163.6 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA14661 = {S35766, 153, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; HTC Desire HD 1.75.163.2 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA14682 = {S35767, 142, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Desire HD Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA14691 = {S35768, 153, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Desire HD 1.75.161.2 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA14725 = {S35769, 153, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; HTC Desire HD 1.32.163.6 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA14756 = {S35770, 153, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; HTC Desire HD 1.32.163.5 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA14785 = {S35771, 150, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Desire HD 1.39.161.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA14787 = {S35772, 150, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; HTC Desire HD 1.32.163.5 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA14791 = {S35773, 148, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC_DesireHD-orange-LS Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA39586 = {S35774, 227, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; HTCA9191/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA39668 = {S35775, 141, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-mx; HTC_DesireHD Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42603 = {S35776, 137, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;Desire HD Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA44091 = {S35777, 138, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC_DesireHD Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA44093 = {S35778, 135, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; Desire HD Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48938 = {S35779, 131, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; 001HT Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA54417 = {S35780, 138, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Desire HD Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA54475 = {S35781, 147, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; HTC_DesireHD_A9192 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61748 = {S35782, 144, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC_DesireHD_A9191 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61750 = {S35783, 147, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC_DesireHD_A9191 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA150419 = {S35784, 59, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; Desire HD Build/FRF91)*/
const Device UA165362 = {S35785, 62, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; Desire HD Build/FRG83D)*/
const Device UA644392 = {S35786, 123, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; Desire HD Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA652312 = {S35787, 126, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; Desire HD Build/FRG83D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2018292 = {S35788, 55, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; 001HT Build/FRF91)*/
const Device UA2019120 = {S35789, 145, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Desire HD Build/FRG83D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2033664 = {S35790, 128, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; Desire HD Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2336421 = {S35791, 131, { &P14225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; Desire HD Build/FRG83D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA92212 = {S35792, 119, { &P15364,&P7133,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA104964 = {S35793, 112, { &P15364,&P7133,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA106373 = {S35794, 112, { &P15364,&P7133,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/528.16 (KHTML, like Gecko) Version/4.0 Safari/528.16*/
const Device UA125200 = {S35795, 118, { &P15364,&P7133,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA127071 = {S35796, 114, { &P15364,&P7133,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9.1*/
const Device UA344870 = {S35797, 101, { &P15364,&P7133,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA408500 = {S35798, 114, { &P15364,&P7133,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532+ (KHTML, like Gecko) Version/4.0.2 Safari/530.19.1*/
const Device UA693511 = {S35799, 112, { &P15364,&P7133,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Safari/528.17*/
const Device UA1875853 = {S35800, 117, { &P15364,&P7133,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/526.11 (KHTML, like Gecko) Version/4.0dp1 Safari/526.12.2*/
const Device UA2424088 = {S35801, 106, { &P15364,&P7133,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA2458246 = {S35802, 107, { &P15364,&P7133,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA2466757 = {S35803, 99, { &P15364,&P7133,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA24506 = {S35804, 144, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Sprint APX515CKT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA33250 = {S35805, 135, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PG86100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48382 = {S35806, 149, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; HTC/EVO_3D/1.21.161.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48399 = {S35807, 149, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-ch; HTC/EVO_3D/1.21.166.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48404 = {S35808, 149, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; HTC/EVO_3D/1.21.163.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49600 = {S35809, 144, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; Sprint APX515CKT Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49604 = {S35810, 135, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PG86100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49605 = {S35811, 144, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Sprint APX515CKT Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA197498 = {S35812, 59, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; PG86100 Build/GRI40)*/
const Device UA234721 = {S35813, 59, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; PG86100 Build/GRJ22)*/
const Device UA247163 = {S35814, 135, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PG86100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369290 = {S35815, 65, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.4; en_US; PG86100 Build/GRJ22)*/
const Device UA437137 = {S35816, 149, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-ch; HTC/EVO_3D/1.21.166.5 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA437162 = {S35817, 135, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; ISW12HT Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622444 = {S35818, 141, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; ISW12HT Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30*/
const Device UA640815 = {S35819, 144, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Sprint APX515CKT Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA667825 = {S35820, 149, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; HTC/EVO_3D/1.21.161.5 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA693908 = {S35821, 135, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PG86100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA702970 = {S35822, 149, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; HTC/EVO_3D/1.24.165.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA788381 = {S35823, 149, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; HTC/EVO_3D/1.21.163.4 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019382 = {S35824, 142, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PG86100 Build/GRJ22) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2020944 = {S35825, 133, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; PG86100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021302 = {S35826, 59, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; ISW12HT Build/GRJ22)*/
const Device UA2021782 = {S35827, 149, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; HTC/EVO_3D/1.24.165.2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2258589 = {S35828, 137, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; APX515CKT Build/GRJ22) AppleWebKit/533.1 (KHTML	 like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2816785 = {S35829, 128, { &P12788,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; PG86100 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA18166 = {S35830, 135, { &P14799,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; pt-br; GT-I5500B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA23187 = {S35831, 137, { &P14799,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; GT-I5500L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67319 = {S35832, 135, { &P14799,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-I5500L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67853 = {S35833, 134, { &P14799,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I5500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72792 = {S35834, 137, { &P14799,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; GT-I5500L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA310483 = {S35835, 58, { &P14799,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I5500 Build/FROYO)*/
const Device UA547306 = {S35836, 59, { &P14799,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I5500L Build/FROYO)*/
const Device UA555780 = {S35837, 136, { &P14799,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; hr-hr; GT-I5500 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA565813 = {S35838, 61, { &P14799,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-I5500L Build/FROYO)*/
const Device UA634407 = {S35839, 59, { &P14799,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I5500B Build/FROYO)*/
const Device UA638602 = {S35840, 137, { &P14799,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; GT-I5500B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1881298 = {S35841, 135, { &P14799,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-I5500M Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1958724 = {S35842, 134, { &P14799,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2;es-es;GT-I5500 Build/ FROYO) AppleWebKit/533.1 (KHTML, like Geccko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019811 = {S35843, 61, { &P14799,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; GT-I5500L Build/FROYO)*/
const Device UA2019919 = {S35844, 134, { &P14799,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-I5500 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023581 = {S35845, 61, { &P14799,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; GT-I5500B Build/FROYO)*/
const Device UA2025210 = {S35846, 136, { &P14799,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; GT-I5500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2409697 = {S35847, 132, { &P14799,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-; GT-I5500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033438 = {S35848, 132, { &P18454,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGPT12 Build/TISU0105) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2195939 = {S35849, 132, { &P18454,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGPT12 Build/TISU0122) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2238552 = {S35850, 131, { &P18454,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGPT12 Build/TID0129) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2362165 = {S35851, 131, { &P18454,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGPT12 Build/TID0136) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2426848 = {S35852, 131, { &P18454,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGPT13 Build/TID0129) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2429775 = {S35853, 131, { &P18454,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGPT12 Build/TID0142) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2437350 = {S35854, 131, { &P18454,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGPT13 Build/TID0142) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2451931 = {S35855, 132, { &P18454,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; SGPT12 Build/TISU0118) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2460240 = {S35856, 60, { &P18454,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SGPT12 Build/TID0142)*/
const Device UA2656473 = {S35857, 60, { &P18454,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SGPT13 Build/TID0142)*/
const Device UA2730770 = {S35858, 131, { &P18454,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; SGPT13 Build/TID0136) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA369148 = {S35859, 145, { &P18661,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N860 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA572642 = {S35860, 138, { &P18661,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N860 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA636268 = {S35861, 62, { &P18661,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; N860 Build/GINGERBREAD)*/
const Device UA822702 = {S35862, 140, { &P18661,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N860 Build/WP7 ROM 2.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2385007 = {S35863, 140, { &P18661,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N860 Build/Shinjitsu Rom) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2463556 = {S35864, 136, { &P18661,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; N860 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2508194 = {S35865, 135, { &P18661,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N860 Build/n00b Rom) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2656732 = {S35866, 142, { &P18661,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N860 Build/ROM by Bloodawn) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA62640 = {S35867, 137, { &P14433,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; GT-I9003L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA64777 = {S35868, 136, { &P14433,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA187906 = {S35869, 60, { &P14433,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-I9003 Build/FROYO)*/
const Device UA575569 = {S35870, 124, { &P14433,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-I9003 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA814280 = {S35871, 61, { &P14433,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-I9003L Build/FROYO)*/
const Device UA2034155 = {S35872, 129, { &P14433,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-I9003 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2370615 = {S35873, 134, { &P14433,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA885419 = {S35874, 139, { &P18283,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  GT-I9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA900391 = {S35875, 138, { &P18283,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA901532 = {S35876, 139, { &P18283,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300T Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA901773 = {S35877, 138, { &P18283,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9300 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA903838 = {S35878, 138, { &P18283,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA911274 = {S35879, 138, { &P18283,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I535 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA911353 = {S35880, 139, { &P18283,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999V Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA911555 = {S35881, 136, { &P18283,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-06D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2023573 = {S35882, 140, { &P18283,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  GT-I9300T Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA2375154 = {S35883, 132, { &P18283,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2380406 = {S35884, 132, { &P18283,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2385056 = {S35885, 135, { &P18283,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2394260 = {S35886, 138, { &P18283,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I939 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2466540 = {S35887, 139, { &P18283,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I93000 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2503817 = {S35888, 138, { &P18283,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; GT-l9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2760452 = {S35889, 136, { &P18283,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; I9300+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2837112 = {S35890, 135, { &P18283,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; I9300 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2948084 = {S35891, 136, { &P18283,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-06D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari 535.19*/
const Device UA3044939 = {S35892, 138, { &P18283,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-l9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA181069 = {S35893, 145, { &P15364,&P5690,&P21923,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16 chromeframe/10.0.648.204*/
const Device UA208036 = {S35894, 204, { &P15364,&P5690,&P21923,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/10.0.648.204; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322; InfoPath.2; .NET4.0C; .NET4.0E; MS-RTC LM 8)*/
const Device UA242073 = {S35895, 209, { &P15364,&P5690,&P21923,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; SV1; chromeframe/10.0.648.205; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.1; MS-RTC LM 8)*/
const Device UA375734 = {S35896, 164, { &P15364,&P5690,&P21923,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/10.0.648.151; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA421054 = {S35897, 170, { &P15364,&P5690,&P21923,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; chromeframe/10.0.648.151; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C)*/
const Device UA634790 = {S35898, 145, { &P15364,&P5690,&P21923,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16 chromeframe/10.0.648.205*/
const Device UA704011 = {S35899, 217, { &P15364,&P5690,&P21923,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/10.0.648.204; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322; InfoPath.2; .NET4.0C; .NET4.0E; MS-RTC LM 8)*/
const Device UA708387 = {S35900, 177, { &P15364,&P5690,&P21923,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/10.0.648.205; SV1; MS-RTC LM 8; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA3082795 = {S35901, 140, { &P12280,&P19826,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B143 Safari/8536.25*/
const Device UA3082797 = {S35902, 142, { &P12280,&P19826,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B350 Safari/8536.25*/
const Device UA3082798 = {S35903, 142, { &P12280,&P19826,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B329 Safari/8536.25*/
const Device UA3082988 = {S35904, 142, { &P12280,&P19826,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B146 Safari/8536.25*/
const Device UA3082998 = {S35905, 140, { &P12280,&P19826,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B144 Safari/8536.25*/
const Device UA3083003 = {S35906, 142, { &P12280,&P19826,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B145 Safari/8536.25*/
const Device UA3083012 = {S35907, 140, { &P12280,&P19826,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B142 Safari/8536.25*/
const Device UA3083422 = {S35908, 140, { &P12280,&P19826,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B141 Safari/8536.25*/
const Device UA403354 = {S35909, 136, { &P18225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U9200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA895466 = {S35910, 141, { &P18225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U9200 Build/HuaweiU9200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1961417 = {S35911, 143, { &P18225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; GS03 Build/HuaweiU9200-92) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028096 = {S35912, 132, { &P18225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; U9200 Build/HuaweiU9200) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2031743 = {S35913, 63, { &P18225,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; U9200 Build/HuaweiU9200)*/
const Device UA2187896 = {S35914, 148, { &P18225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; U9200 Build/HuaweiU9200C02B013) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257181 = {S35915, 148, { &P18225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; U9200 Build/HuaweiU9200C02B021) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2342037 = {S35916, 145, { &P18225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; U9202L-1 Build/HuaweiU9202L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2384524 = {S35917, 143, { &P18225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; U9200E Build/HuaweiU9200E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2428302 = {S35918, 148, { &P18225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; U9200 Build/HuaweiU9200C02B023) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2551000 = {S35919, 148, { &P18225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; U9200 Build/HuaweiU9200C02B009) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2578804 = {S35920, 151, { &P18225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; U9200 Build/HuaweiU9200C02B013SP3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2981916 = {S35921, 145, { &P18225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; U9202L-3 Build/HuaweiU9202L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3070521 = {S35922, 65, { &P18225,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; U9200E Build/HuaweiU9200E)*/
const Device UA3118964 = {S35923, 145, { &P18225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; U9200E Build/HuaweiU9200E) AppleWebKit/534.30 (KHTML, like Gecko) Phi?n b?n/4.0 Mobile Safari/534.30*/
const Device UA2035304 = {S35924, 112, { &P18444,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 820)*/
const Device UA2382013 = {S35925, 122, { &P18444,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 820; Vodafone)*/
const Device UA2387511 = {S35926, 120, { &P18444,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 820; ANZ825)*/
const Device UA2506074 = {S35927, 120, { &P18444,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 820; Orange)*/
const Device UA2506381 = {S35928, 112, { &P18444,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia 820)*/
const Device UA3064039 = {S35929, 120, { &P18444,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 820; 1328-3)*/
const Device UA625169 = {S35930, 130, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA687481 = {S35931, 148, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7300-ORANGE/P7300BVKJ5 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA706477 = {S35932, 131, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; th-th; GT-P7300B Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA774655 = {S35933, 59, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7300 Build/HTJ85B)*/
const Device UA1953540 = {S35934, 148, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7300-ORANGE/P7300BVLC2 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2370865 = {S35935, 128, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2444216 = {S35936, 137, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2463559 = {S35937, 149, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; cs-cz; SAMSUNG-GT-P7300/P7300BOLC1 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2608794 = {S35938, 60, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7300B Build/HTJ85B)*/
const Device UA2609177 = {S35939, 151, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7300 Build/HTJ85B; 1280*752) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2609516 = {S35940, 118, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7300) AntivirusAM/2.5 (KHTML, like Gecko) Version/Version 2.5/Antivirus*/
const Device UA2610035 = {S35941, 136, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2613574 = {S35942, 128, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7300 Build/HTJ85B) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2616553 = {S35943, 202, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; nl-nl; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 TwonkyBeamBrowser/3.2.1-65 (Android 3.2; samsung GT-P7300 Build/HTJ85B)*/
const Device UA2618226 = {S35944, 92, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 3.2; SAMSUNG GT-P7300 Build/HONEYCOMB_MR2);*/
const Device UA2618559 = {S35945, 65, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 3.2; en-GB; GT-P7300 Build/HTJ85B)*/
const Device UA2619160 = {S35946, 160, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7300 Build/HTJ85B) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2619232 = {S35947, 132, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7300 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2619330 = {S35948, 140, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7300 Build/HTJ85B like Xoom) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2624070 = {S35949, 148, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7300-ORANGE/P7300BVKT5 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2624103 = {S35950, 274, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/2.0;FBDM/{density=1.0,width=1280,height=752};FBLC/en_US;FB_FW/2;FBCR/MY MAXIS;FBPN/com.facebook.katana;FBDV/GT-P7300;FBSV/3.2;]*/
const Device UA2630043 = {S35951, 126, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Groupon/2.2.2572 (Android 3.2; Samsung GT-P7300 / Samsung GT-P7300; )[preload=false;locale=en_US;clientidbase=android-samsung]*/
const Device UA2632272 = {S35952, 34, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Android 3.2; 13; samsung; GT-P7300*/
const Device UA2635280 = {S35953, 147, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fr-fr;GT-P7300-ORANGE/P7300BVLC2 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2635806 = {S35954, 123, { &P12889,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fa-fa; GT-P7300 Build/HTJ85B) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA98156 = {S35955, 151, { &P15364,&P16110,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA386929 = {S35956, 50, { &P15364,&P16110,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2)*/
const Device UA2148688 = {S35957, 150, { &P15364,&P16110,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2; WOW64; Trident/4.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729)*/
const Device UA2335492 = {S35958, 172, { &P15364,&P16110,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2; WOW64; Trident/4.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; InfoPath.3; Zune 4.7)*/
const Device UA2527346 = {S35959, 171, { &P15364,&P16110,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2; WOW64; Trident/4.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA2597452 = {S35960, 63, { &P15364,&P16110,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.2; Trident/4.0)*/
const Device UA2693738 = {S35961, 165, { &P15364,&P16110,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2; WOW64; Trident/4.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0)*/
const Device UA2773972 = {S35962, 151, { &P15364,&P16110,&P17190,&P18092 }};	/*Useragent - 'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center C 6.0)*/
const Device UA3099295 = {S35963, 155, { &P15364,&P16110,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2; Win64; x64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727)*/
const Device UA77514 = {S35964, 164, { &P12980,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; LG-E510f Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA647234 = {S35965, 59, { &P12980,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-E510 Build/GRJ22)*/
const Device UA671867 = {S35966, 164, { &P12980,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-sa; LG-E510g Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA686451 = {S35967, 181, { &P12980,&P5629,&P15760,&P18092 }};	/*Useragent - LG-E510/V10c Mozilla/5.0 (Linux; U; Android 2.3.6; el-gr; LG-E510/V10c Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA703341 = {S35968, 60, { &P12980,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-E510f Build/GRJ22)*/
const Device UA710573 = {S35969, 163, { &P12980,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-E510 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA711366 = {S35970, 163, { &P12980,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; LG-E510 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA1983055 = {S35971, 164, { &P12980,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; LG-E510f Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2024466 = {S35972, 174, { &P12980,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; LG-E510/${Version} Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2029380 = {S35973, 59, { &P12980,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-E510 Build/GRJ22)*/
const Device UA2033876 = {S35974, 60, { &P12980,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-E510g Build/GRJ22)*/
const Device UA2588497 = {S35975, 174, { &P12980,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; LG-E510/${Version} Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2684278 = {S35976, 181, { &P12980,&P5629,&P15760,&P18092 }};	/*Useragent - LG-E510/V10d Mozilla/5.0 (Linux; U; Android 2.3.6; el-gr; LG-E510/V10d Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2811199 = {S35977, 59, { &P12980,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; LG-E510 Build/GRJ22)*/
const Device UA63659 = {S35978, 167, { &P13552,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaC6-00.1/20.2.042; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) BrowserNG/7.2.6.9 3gpp-gba*/
const Device UA64747 = {S35979, 165, { &P13552,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaC6-00/20.0.042; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) BrowserNG/7.2.6.9 3gpp-gba*/
const Device UA626646 = {S35980, 171, { &P13552,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaC6-00/42.0.004; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 3gpp-gba*/
const Device UA734284 = {S35981, 173, { &P13552,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaC6-00.1/42.2.004; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 3gpp-gba*/
const Device UA2566541 = {S35982, 146, { &P18472,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; DROID RAZR HD Build/9.8.1Q_39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566812 = {S35983, 152, { &P18472,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT925 Build/9.8.2Q-8-XT925_VQL-12.2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567086 = {S35984, 139, { &P18472,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; DROID RAZR HD Build/9.8.1Q_39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583066 = {S35985, 155, { &P18472,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583194 = {S35986, 151, { &P18472,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; RAZR HD Build/9.8.2Q-8-XT925_VQL-6) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583969 = {S35987, 145, { &P18472,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT925 Build/9.8.2Q-8-XT925_VQL-12.2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2584850 = {S35988, 148, { &P18472,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2602016 = {S35989, 151, { &P18472,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XT925 Build/9.8.0Q-97-XT925_VQU-19) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2029013 = {S35990, 141, { &P18302,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; MT25i Build/4.1.B.0.479) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2179512 = {S35991, 141, { &P18302,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; MT25i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2368008 = {S35992, 141, { &P18302,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; MT25i Build/4.1.B.0.626) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2427737 = {S35993, 153, { &P18302,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; SonyEricssonMT25i Build/4.1.B.0.631) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2434303 = {S35994, 141, { &P18302,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; MT25i Build/4.1.B.0.631) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2506083 = {S35995, 63, { &P18302,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MT25i Build/4.1.B.0.631)*/
const Device UA2991930 = {S35996, 139, { &P18302,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fa-; MT25i Build/4.1.B.0.631) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3096169 = {S35997, 142, { &P12280,&P19826,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B329 Safari/8536.25*/
const Device UA3096438 = {S35998, 142, { &P12280,&P19826,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B350 Safari/8536.25*/
const Device UA3096506 = {S35999, 142, { &P12280,&P19826,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B146 Safari/8536.25*/
const Device UA3096515 = {S36000, 140, { &P12280,&P19826,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B144 Safari/8536.25*/
const Device UA3096547 = {S36001, 140, { &P12280,&P19826,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B142 Safari/8536.25*/
const Device UA3096551 = {S36002, 142, { &P12280,&P19826,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B145 Safari/8536.25*/
const Device UA3096558 = {S36003, 140, { &P12280,&P19826,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B143 Safari/8536.25*/
const Device UA3096565 = {S36004, 140, { &P12280,&P19826,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B141 Safari/8536.25*/
const Device UA452921 = {S36005, 150, { &P18231,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ALCATEL_one_touch_918M Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA689588 = {S36006, 150, { &P18231,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; ALCATEL_one_touch_918D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA711152 = {S36007, 150, { &P18231,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; ALCATEL ONE TOUCH 918D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA770741 = {S36008, 150, { &P18231,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; ALCATEL ONE TOUCH 918 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA895533 = {S36009, 151, { &P18231,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ALCATEL ONE TOUCH 918S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA896688 = {S36010, 151, { &P18231,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ALCATEL ONE TOUCH 918A Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1986469 = {S36011, 151, { &P18231,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; ALCATEL ONE TOUCH 918D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030014 = {S36012, 151, { &P18231,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; iw-il; ALCATEL_ONE_TOUCH_918M Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2336355 = {S36013, 143, { &P18231,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; nl-nl; ALCATEL_OT_918D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2347492 = {S36014, 157, { &P18231,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ro-ro; Alcatel_one_touch_918_Orange Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2546019 = {S36015, 150, { &P18231,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; ALCATEL_one_touch_918 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2810246 = {S36016, 74, { &P18231,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; ALCATEL ONE TOUCH 918D Build/GRJ90)*/
const Device UA2830820 = {S36017, 149, { &P18231,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-; ALCATEL ONE TOUCH 918A Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2797909 = {S36018, 86, { &P17968,&P17979,&P21454,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/14.0.1074/30.3214; U; en) Presto/2.8.119 Version/11.10*/
const Device UA399168 = {S36019, 101, { &P18176,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 900)*/
const Device UA896472 = {S36020, 109, { &P18176,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 900; Orange)*/
const Device UA925777 = {S36021, 101, { &P18176,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Nokia 900)*/
const Device UA925793 = {S36022, 111, { &P18176,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 900; Vodafone)*/
const Device UA2528737 = {S36023, 107, { &P18176,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 900; None)*/
const Device UA2790074 = {S36024, 140, { &P18176,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 900),gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2005200 = {S36025, 139, { &P13991,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6400L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2011785 = {S36026, 142, { &P13991,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6400L 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027864 = {S36027, 139, { &P13991,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6400L 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2042734 = {S36028, 142, { &P13991,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ThunderBolt Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257529 = {S36029, 142, { &P13991,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6400L 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257536 = {S36030, 139, { &P13991,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6400L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2496280 = {S36031, 139, { &P13991,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6400L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2496772 = {S36032, 142, { &P13991,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6400L 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2506827 = {S36033, 61, { &P13991,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ADR6400L Build/IMM76D)*/
const Device UA2525797 = {S36034, 64, { &P13991,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ThunderBolt Build/IML74K)*/
const Device UA2689143 = {S36035, 61, { &P13991,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; ADR6400L Build/IML74K)*/
const Device UA2704091 = {S36036, 137, { &P13991,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; ADR6400L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2733418 = {S36037, 156, { &P13991,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6400L Build/MaceksMod Mecha Rebirth) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2856789 = {S36038, 148, { &P13991,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6400L 4G Build/ICE-ICE Baby) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2741699 = {S36039, 128, { &P18385,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754380 = {S36040, 127, { &P18385,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Nexus 7 Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754719 = {S36041, 129, { &P18385,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 7 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2873886 = {S36042, 135, { &P18385,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA503941 = {S36043, 64, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows 98; Maxthon; Maxthon)*/
const Device UA664959 = {S36044, 55, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; Maxthon)*/
const Device UA836933 = {S36045, 70, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows 98; Maxthon; Avant Browser)*/
const Device UA1951376 = {S36046, 68, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Maxthon)*/
const Device UA1951391 = {S36047, 68, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; Maxthon)*/
const Device UA1954985 = {S36048, 68, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; Maxthon)*/
const Device UA1960062 = {S36049, 68, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Maxthon)*/
const Device UA1964937 = {S36050, 68, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Maxthon)*/
const Device UA1984913 = {S36051, 68, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Maxthon)*/
const Device UA2222587 = {S36052, 73, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; SV1; Maxthon)*/
const Device UA2243036 = {S36053, 73, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; SV1; Maxthon)*/
const Device UA2247787 = {S36054, 77, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Maxthon; Maxthon)*/
const Device UA2257995 = {S36055, 83, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; Maxthon; Avant Browser)*/
const Device UA2334857 = {S36056, 73, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Maxthon; SV1)*/
const Device UA2374426 = {S36057, 77, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Maxthon; Maxthon)*/
const Device UA2374972 = {S36058, 77, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Maxthon; Maxthon)*/
const Device UA2376046 = {S36059, 83, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Avant Browser; Maxthon)*/
const Device UA2394547 = {S36060, 73, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Maxthon; SV1)*/
const Device UA2395336 = {S36061, 83, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Avant Browser; Maxthon)*/
const Device UA2413247 = {S36062, 83, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Avant Browser; Maxthon)*/
const Device UA2413304 = {S36063, 83, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Maxthon; Avant Browser)*/
const Device UA2413682 = {S36064, 73, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; SV1; Maxthon)*/
const Device UA2413732 = {S36065, 73, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Maxthon; SV1)*/
const Device UA2413806 = {S36066, 77, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Maxthon; Maxthon)*/
const Device UA3098975 = {S36067, 88, { &P15364,&P6704,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; SV1; Avant Browser; Maxthon)*/
const Device UA2677406 = {S36068, 87, { &P15364,&P7133,&P24342,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; QQBrowser/7.3.8126.400)*/
const Device UA2679635 = {S36069, 94, { &P15364,&P7133,&P24342,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; QQBrowser/7.3.8126.400)*/
const Device UA2901013 = {S36070, 89, { &P15364,&P7133,&P24342,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; QQBrowser/7.3.11251.400)*/
const Device UA2946027 = {S36071, 96, { &P15364,&P7133,&P24342,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; QQBrowser/7.3.11251.400)*/
const Device UA2946073 = {S36072, 176, { &P15364,&P7133,&P24342,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/7.3.11251.400)*/
const Device UA2964036 = {S36073, 183, { &P15364,&P7133,&P24342,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/7.3.11251.400)*/
const Device UA2965981 = {S36074, 155, { &P15364,&P7133,&P24342,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; QQBrowser/7.3.11251.400)*/
const Device UA2966835 = {S36075, 162, { &P15364,&P7133,&P24342,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; QQBrowser/7.3.11251.400)*/
const Device UA3013779 = {S36076, 191, { &P15364,&P7133,&P24342,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; QQBrowser/7.3.11251.400)*/
const Device UA3014708 = {S36077, 193, { &P15364,&P7133,&P24342,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; QQBrowser/7.3.11251.400)*/
const Device UA3016749 = {S36078, 186, { &P15364,&P7133,&P24342,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; QQBrowser/7.3.11251.400)*/
const Device UA3019515 = {S36079, 165, { &P15364,&P7133,&P24342,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; QQBrowser/7.3.11251.400)*/
const Device UA1391 = {S36080, 89, { &P15364,&P7127,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.0) Opera 7.02 Bork-edition [en]*/
const Device UA336662 = {S36081, 35, { &P15364,&P7127,&P4810,&P18092 }};	/*Useragent - Opera/7.54 (Windows NT 5.0; U) [de]*/
const Device UA408995 = {S36082, 47, { &P15364,&P7127,&P4810,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; U) Opera 7.01 [en]*/
const Device UA526346 = {S36083, 35, { &P15364,&P7127,&P4810,&P18092 }};	/*Useragent - Opera/7.01 (Windows NT 5.0; U) [en]*/
const Device UA631938 = {S36084, 47, { &P15364,&P7127,&P4810,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; U) Opera 7.11 [en]*/
const Device UA659482 = {S36085, 35, { &P15364,&P7127,&P4810,&P18092 }};	/*Useragent - Opera/7.03 (Windows NT 5.0; U) [en]*/
const Device UA756827 = {S36086, 47, { &P15364,&P7127,&P4810,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; U) Opera 7.21 [en]*/
const Device UA843860 = {S36087, 76, { &P15364,&P7127,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.0) Opera 7.01 [en]*/
const Device UA860671 = {S36088, 47, { &P15364,&P7127,&P4810,&P18092 }};	/*Useragent - Mozilla/3.0 (Windows NT 5.0; U) Opera 7.10 [de]*/
const Device UA875438 = {S36089, 76, { &P15364,&P7127,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.0) Opera 7.03 [en]*/
const Device UA1969245 = {S36090, 63, { &P15364,&P7127,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) Opera 7.54u1*/
const Device UA2242299 = {S36091, 66, { &P15364,&P7127,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) Opera 7.11 [en]*/
const Device UA2307880 = {S36092, 35, { &P15364,&P7127,&P4810,&P18092 }};	/*Useragent - Opera/7.10 (Windows NT 5.0; U) [en]*/
const Device UA2505005 = {S36093, 36, { &P15364,&P7127,&P4810,&P18092 }};	/*Useragent - Opera/7.23 (Windows NT 5.0; U)  [ja]*/
const Device UA2506947 = {S36094, 62, { &P15364,&P7127,&P4810,&P18092 }};	/*Useragent - Opera/7.60 (Windows NT 5.0; U)  [en] (IBM EVV/3.0/EAK01AG9/LE)*/
const Device UA2540707 = {S36095, 66, { &P15364,&P7127,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) Opera 7.22 [en]*/
const Device UA2681103 = {S36096, 66, { &P15364,&P7127,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) Opera 7.20 [en]*/
const Device UA2681820 = {S36097, 66, { &P15364,&P7127,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) Opera 7.21 [ru]*/
const Device UA2682301 = {S36098, 66, { &P15364,&P7127,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) Opera 7.53 [ru]*/
const Device UA2682370 = {S36099, 35, { &P15364,&P7127,&P4810,&P18092 }};	/*Useragent - Opera/7.53 (Windows NT 5.0; U) [ru]*/
const Device UA2682382 = {S36100, 66, { &P15364,&P7127,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) Opera 7.23 [ru]*/
const Device UA2844940 = {S36101, 36, { &P15364,&P7127,&P4810,&P18092 }};	/*Useragent - Opera/7.52 (Windows NT 5.0; U)  [en]*/
const Device UA249611 = {S36102, 117, { &P17595,&P17297,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1*/
const Device UA249794 = {S36103, 117, { &P17595,&P17297,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1*/
const Device UA322347 = {S36104, 115, { &P17595,&P17297,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.803.0 Safari/535.1*/
const Device UA628042 = {S36105, 117, { &P17595,&P17297,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA756856 = {S36106, 117, { &P17595,&P17297,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA2203305 = {S36107, 117, { &P17595,&P17297,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA2028860 = {S36108, 138, { &P13644,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PC36100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029523 = {S36109, 138, { &P13644,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PC36100 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2201681 = {S36110, 134, { &P13644,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EVO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2355742 = {S36111, 138, { &P13644,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PC36100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426728 = {S36112, 134, { &P13644,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EVO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2429216 = {S36113, 56, { &P13644,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; EVO Build/JRO03C)*/
const Device UA2502710 = {S36114, 56, { &P13644,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; EVO Build/IMM76D)*/
const Device UA2528070 = {S36115, 141, { &P13644,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EVO 4G LTE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2652545 = {S36116, 147, { &P13644,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PC36100 Build/Androxide 3.1.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2657311 = {S36117, 132, { &P13644,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; EVO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2696895 = {S36118, 134, { &P13644,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; EVO Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994630 = {S36119, 137, { &P13644,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PC36100 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2431546 = {S36120, 105, { &P14824,&P17484,&P17369,&P18092 }};	/*Useragent - SAMSUNG-GT-E2252 Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.29595/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA3116442 = {S36121, 105, { &P14824,&P17484,&P17369,&P18092 }};	/*Useragent - SAMSUNG GT-E2252 Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.29060/31.1475; U; en) Presto/2.8.119 Version/11.10*/
const Device UA907631 = {S36122, 138, { &P18645,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-L710 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA907678 = {S36123, 146, { &P18645,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA909522 = {S36124, 139, { &P18645,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I747M Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA913187 = {S36125, 139, { &P18645,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530U Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1961797 = {S36126, 139, { &P18645,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1995467 = {S36127, 139, { &P18645,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210K Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2030751 = {S36128, 139, { &P18645,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2367557 = {S36129, 139, { &P18645,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530C Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2415342 = {S36130, 142, { &P18645,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Galaxy S III Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA924306 = {S36131, 137, { &P15364,&P17820,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.11 (KHTML, like Gecko) Ubuntu/12.04 Chromium/20.0.1132.47 Chrome/20.0.1132.47 Safari/536.11*/
const Device UA924753 = {S36132, 139, { &P15364,&P17820,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.11 (KHTML, like Gecko) Ubuntu/12.04 Chromium/20.0.1132.47 Chrome/20.0.1132.47 Safari/536.11*/
const Device UA2260096 = {S36133, 147, { &P15364,&P17820,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.11 (KHTML, like Gecko) Ubuntu/12.04 Chromium/20.0.1132.47 Chrome/20.0.1132.47 Safari/536.11,gzip(gfe)*/
const Device UA2348606 = {S36134, 176, { &P15364,&P17820,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.11 (KHTML, like Gecko) Ubuntu/12.04 Chromium/20.0.1132.47 Chrome/20.0.1132.47 Safari/536.11,gzip(gfe) (via translate.google.com)*/
const Device UA2355495 = {S36135, 149, { &P15364,&P17820,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.11 (KHTML, like Gecko) Ubuntu/12.04 Chromium/20.0.1132.47 Chrome/20.0.1132.47 Safari/536.11,gzip(gfe)*/
const Device UA2400651 = {S36136, 139, { &P15364,&P17820,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/536.11 (KHTML, like Gecko) Ubuntu/12.04 Chromium/20.0.1132.47 Chrome/20.0.1132.47 Safari/536.11*/
const Device UA2830304 = {S36137, 176, { &P15364,&P17820,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.11 (KHTML, like Gecko) Ubuntu/12.04 Chromium/20.0.1132.47 Chrome/20.0.1132.47 Safari/536.11,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3020745 = {S36138, 132, { &P18466,&P18110,&P24334,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Safari/535.19 Silk-Accelerated=true*/
const Device UA453591 = {S36139, 178, { &P13324,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.3 NokiaN8-00; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.4.2.6 Mobile Safari/533.4 3gpp-gba*/
const Device UA2017988 = {S36140, 191, { &P13324,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.3 NokiaN8-00/111.040.1511; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.1.4 Mobile Safari/535.1 3gpp-gba*/
const Device UA2008893 = {S36141, 64, { &P15364,&P16110,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; U; en) Presto/2.10.289 Version/12.02*/
const Device UA2262857 = {S36142, 93, { &P15364,&P16110,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; U; MRA 6.0 (build 5970); ru) Presto/2.10.289 Version/12.02*/
const Device UA2348413 = {S36143, 93, { &P15364,&P16110,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; U; MRA 6.0 (build 5831); ru) Presto/2.10.289 Version/12.02*/
const Device UA2350248 = {S36144, 93, { &P15364,&P16110,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; U; MRA 6.0 (build 5976); ru) Presto/2.10.289 Version/12.02*/
const Device UA2653229 = {S36145, 93, { &P15364,&P16110,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; U; MRA 6.0 (build 5998); ru) Presto/2.10.289 Version/12.02*/
const Device UA2763358 = {S36146, 93, { &P15364,&P16110,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; U; MRA 6.0 (build 6015); ru) Presto/2.10.289 Version/12.02*/
const Device UA3099244 = {S36147, 93, { &P15364,&P16110,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; U; MRA 6.0 (build 6005); ru) Presto/2.10.289 Version/12.02*/
const Device UA1993987 = {S36148, 129, { &P17926,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AT100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1998766 = {S36149, 58, { &P17926,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; AT100 Build/IMM76D)*/
const Device UA2449408 = {S36150, 136, { &P17926,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AT100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2692272 = {S36151, 127, { &P17926,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; AT100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2833725 = {S36152, 142, { &P17926,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AT100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA171098 = {S36153, 136, { &P16757,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SCH-I500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA173931 = {S36154, 60, { &P16757,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SCH-I500 Build/FROYO)*/
const Device UA211507 = {S36155, 60, { &P16757,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; SCH-I500 Build/FROYO)*/
const Device UA236463 = {S36156, 136, { &P16757,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SCH-I500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA556470 = {S36157, 143, { &P16757,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SCH-I500 Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA685969 = {S36158, 135, { &P16757,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;SCH-I500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA703468 = {S36159, 134, { &P16757,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ko-; SCH-I500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1883286 = {S36160, 135, { &P16757,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.1; en-us; SCH-I500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018622 = {S36161, 135, { &P16757,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;SCH-I500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018874 = {S36162, 135, { &P16757,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SCH-I500Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018938 = {S36163, 135, { &P16757,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; SCH-I500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019157 = {S36164, 135, { &P16757,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SCH-I500Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019513 = {S36165, 135, { &P16757,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.2; en-us; SCH-I500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2143291 = {S36166, 135, { &P16757,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.2; en-us; SCH-I500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2495070 = {S36167, 134, { &P16757,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ru-; SCH-I500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA382213 = {S36168, 52, { &P18254,&P7480,&P18245,&P18168 }};	/*Useragent - ichiro/3.0 (http://help.goo.ne.jp/help/article/1142)*/
const Device UA460465 = {S36169, 52, { &P18254,&P7480,&P18245,&P18168 }};	/*Useragent - ichiro/3.0 (http://help.goo.ne.jp/door/crawler.html)*/
const Device UA1977501 = {S36170, 60, { &P18254,&P7480,&P18245,&P18168 }};	/*Useragent - ichiro/3.0 (http://search.goo.ne.jp/option/use/sub4/sub4-1/)*/
const Device UA232723 = {S36171, 53, { &P17491,&P18377,&P17750,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo 3DS; U; ; en) Version/1.7455.US*/
const Device UA235398 = {S36172, 53, { &P17491,&P18377,&P17750,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo 3DS; U; ; en) Version/1.7455.EU*/
const Device UA609249 = {S36173, 53, { &P17491,&P18377,&P17750,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo 3DS; U; ; en) Version/1.7498.EU*/
const Device UA613278 = {S36174, 53, { &P17491,&P18377,&P17750,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo 3DS; U; ; en) Version/1.7498.US*/
const Device UA613284 = {S36175, 53, { &P17491,&P18377,&P17750,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo 3DS; U; ; es) Version/1.7498.US*/
const Device UA613349 = {S36176, 53, { &P17491,&P18377,&P17750,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo 3DS; U; ; nl) Version/1.7498.EU*/
const Device UA613350 = {S36177, 53, { &P17491,&P18377,&P17750,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo 3DS; U; ; de) Version/1.7498.EU*/
const Device UA613595 = {S36178, 53, { &P17491,&P18377,&P17750,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo 3DS; U; ; fr) Version/1.7498.EU*/
const Device UA2601137 = {S36179, 53, { &P17491,&P18377,&P17750,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo 3DS; U; ; en) Version/1.7552.EU*/
const Device UA2602051 = {S36180, 53, { &P17491,&P18377,&P17750,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo 3DS; U; ; en) Version/1.7552.US*/
const Device UA2602312 = {S36181, 53, { &P17491,&P18377,&P17750,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo 3DS; U; ; ja) Version/1.7552.JP*/
const Device UA2603280 = {S36182, 53, { &P17491,&P18377,&P17750,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo 3DS; U; ; nl) Version/1.7552.EU*/
const Device UA2603842 = {S36183, 53, { &P17491,&P18377,&P17750,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo 3DS; U; ; fr) Version/1.7552.EU*/
const Device UA2603844 = {S36184, 53, { &P17491,&P18377,&P17750,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo 3DS; U; ; de) Version/1.7552.EU*/
const Device UA2604153 = {S36185, 53, { &P17491,&P18377,&P17750,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo 3DS; U; ; es) Version/1.7552.US*/
const Device UA2014443 = {S36186, 139, { &P12972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T989 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029758 = {S36187, 139, { &P12972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T989 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2196131 = {S36188, 139, { &P12972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T989 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2201661 = {S36189, 139, { &P12972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T989 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2238444 = {S36190, 139, { &P12972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T989 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2355433 = {S36191, 147, { &P12972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-T989 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2364021 = {S36192, 132, { &P12972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T989 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2425081 = {S36193, 139, { &P12972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T989 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2427834 = {S36194, 69, { &P12972,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-T989 Build/JZO54K)*/
const Device UA2447282 = {S36195, 147, { &P12972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-T989 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2487453 = {S36196, 152, { &P12972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-T989 Build/TweakWiz V2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2652932 = {S36197, 61, { &P12972,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T989 Build/JZO54K)*/
const Device UA2700877 = {S36198, 147, { &P12972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-T989D-parrot Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2716120 = {S36199, 140, { &P12972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-T989D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2763668 = {S36200, 162, { &P12972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-T989 Build/Slim Bean All-Star V5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2767889 = {S36201, 62, { &P12972,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T989D Build/JZO54K)*/
const Device UA2983895 = {S36202, 137, { &P12972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SGH-T989 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA902639 = {S36203, 76, { &P17595,&P17296,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:13.0) Gecko/X Firefox/13.0.1*/
const Device UA905311 = {S36204, 91, { &P17595,&P17296,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:13.0) Gecko/X Firefox/13.0.1 Firefox/13.0.1*/
const Device UA622965 = {S36205, 57, { &P15364,&P7133,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:5.0) Gecko/X Firefox/5.02*/
const Device UA627791 = {S36206, 75, { &P15364,&P7133,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:5.0) Gecko/X Firefox/5.02011-10-16 20:21:42*/
const Device UA1990766 = {S36207, 60, { &P15364,&P7133,&P17207,&P18092 }};	/*Useragent - ,ozilla/3.0 (Windows NT 6.1; rv:2.0.1) Gecko/X Firefox/5.0.1*/
const Device UA2148218 = {S36208, 64, { &P15364,&P7133,&P17207,&P18092 }};	/*Useragent - =Mozilla/5.0 (Windows NT 6.1; WOW64; rv:5.0) Gecko/X Firefox/5.0*/
const Device UA2314464 = {S36209, 77, { &P15364,&P7133,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2) Gecko/X Firefox/5.0*/
const Device UA22362 = {S36210, 137, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; HTC-Z710a Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA33057 = {S36211, 141, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_Sensation Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA33060 = {S36212, 141, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Sensation Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA45823 = {S36213, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; pt-pt; HTC/Sensation/1.35.168.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA45890 = {S36214, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-ch; HTC/Sensation/1.45.166.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA45891 = {S36215, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; HTC/Sensation/1.45.162.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA45981 = {S36216, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC/Sensation/1.45.161.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46134 = {S36217, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-ch; HTC/Sensation/1.35.166.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46163 = {S36218, 147, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; HTC_Sensation_Z710a Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46660 = {S36219, 137, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; HTC-Z710a Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48126 = {S36220, 149, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; HTC_Z710a V1.34.841.6 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59623 = {S36221, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-ch; HTC/Sensation/1.35.166.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59626 = {S36222, 147, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC_Sensation_Z710a Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59634 = {S36223, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC/Sensation/1.35.161.2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59635 = {S36224, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; HTC/Sensation/1.35.163.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59644 = {S36225, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; HTC/Sensation/1.35.162.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59666 = {S36226, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC/Sensation/1.24.161.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59642 = {S36227, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; HTC/Sensation/1.24.162.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59688 = {S36228, 151, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_Sensation-orange-LS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59693 = {S36229, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC/Sensation/1.27.161.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59705 = {S36230, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; HTC/Sensation/1.27.163.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59707 = {S36231, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; HTC/Sensation/1.27.162.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59709 = {S36232, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ru-ru; HTC/Sensation/1.27.187.5 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59714 = {S36233, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-ch; HTC/Sensation/1.27.166.5 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61087 = {S36234, 147, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC Sensation Z710a Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61088 = {S36235, 149, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC_Z710a V1.34.841.6 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70197 = {S36236, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC/Sensation/1.35.161.2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76499 = {S36237, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; HTC/Sensation/1.35.163.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA77031 = {S36238, 151, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC_Sensation-orange-LS Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90779 = {S36239, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; HTC/Sensation/1.17.162.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250055 = {S36240, 147, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; HTC Sensation Z710a Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250424 = {S36241, 137, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Sensation Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250883 = {S36242, 137, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Sensation Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA251908 = {S36243, 141, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC_Sensation Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA251994 = {S36244, 141, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Sensation Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA252032 = {S36245, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; HTC/Sensation/1.45.163.4 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA252534 = {S36246, 149, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; HTC_Z710a V1.45.841.5 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA339251 = {S36247, 137, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Sensation Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA350210 = {S36248, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC/Sensation/1.50.161.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370868 = {S36249, 150, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; xx-xx; HTC/Sensation/1.17.162.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA451235 = {S36250, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC/Sensation/1.45.161.2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA576525 = {S36251, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; el-gr; HTC/Sensation/1.45.168.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA624781 = {S36252, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; HTC/Sensation/1.45.165.2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626446 = {S36253, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; HTC/Sensation/1.45.162.2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA630523 = {S36254, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; HTC/Sensation/1.50.162.2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA635509 = {S36255, 65, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Sensation Build/GRJ22)*/
const Device UA708058 = {S36256, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-ch; HTC/Sensation/1.50.166.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA743371 = {S36257, 61, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC-Z710a Build/GRI40)*/
const Device UA759192 = {S36258, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; HTC/Sensation/1.45.165.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA760285 = {S36259, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; HTC/Sensation/1.50.163.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1913935 = {S36260, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-ch; HTC/Sensation/1.45.166.3 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1929956 = {S36261, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; HTC/Sensation/1.34.165.6 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019297 = {S36262, 71, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Sensation Z710a Build/GRI40)*/
const Device UA2019333 = {S36263, 65, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Sensation Build/GRI40)*/
const Device UA2019779 = {S36264, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; el-gr; HTC/Sensation/1.27.168.5 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020464 = {S36265, 61, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC-Z710a Build/GRJ22)*/
const Device UA2020776 = {S36266, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; HTC/Sensation/1.45.187.3 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021343 = {S36267, 61, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; Sensation Build/GRJ22)*/
const Device UA2021730 = {S36268, 165, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; HTC Sensation kingdroid v11 sense 3.5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022880 = {S36269, 152, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; HTC/Sensation/1.50.165.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022900 = {S36270, 135, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Sensation Z710a Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2024173 = {S36271, 71, { &P15438,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Sensation Z710a Build/GRJ22)*/
const Device UA2566623 = {S36272, 140, { &P18471,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XT907 Build/9.8.1Q_27-2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566694 = {S36273, 133, { &P18471,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XT907 Build/9.8.1Q_27-2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567000 = {S36274, 144, { &P18471,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT905 Build/9.8.2Q_SMUIRD-7) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602015 = {S36275, 137, { &P18471,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT905 Build/9.8.2Q_SMUIRD-7) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2670206 = {S36276, 138, { &P18471,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-66) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2670218 = {S36277, 131, { &P18471,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-66) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA6665 = {S36278, 76, { &P13485,&P5026,&P3410,&P18092 }};	/*Useragent - BlackBerry8330/4.5.0.77 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105*/
const Device UA43980 = {S36279, 78, { &P13485,&P5026,&P3410,&P18092 }};	/*Useragent - BlackBerry8330m/4.5.0.186 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/104*/
const Device UA47759 = {S36280, 151, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 GLOBAL Build/4.5.1_57_D2G-37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47765 = {S36281, 151, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 GLOBAL Build/4.5.1_57_D2G-36) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60661 = {S36282, 151, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 GLOBAL Build/4.5.1_57_D2G-30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70214 = {S36283, 151, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 GLOBAL Build/4.5.1_57_D2G-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73838 = {S36284, 151, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 GLOBAL Build/4.5.1_57_D2G-38) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79764 = {S36285, 150, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2GLOBAL Build/4.5.1_57_D2G-38) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA212825 = {S36286, 140, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID2 GLOBAL Build/S273) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA251989 = {S36287, 75, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; DROID2 GLOBAL Build/4.5.1_57_D2G-38)*/
const Device UA379587 = {S36288, 140, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID2 GLOBAL Build/S273) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA412735 = {S36289, 152, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID2 GLOBAL Build/4.5.1_57_D2GA-59) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA427508 = {S36290, 151, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID2 GLOBALBuild/4.5.1_57_D2GA-59) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA451498 = {S36291, 150, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.3; en-us; DROID2 GLOBAL Build/4.5.1_57_D2G-38) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA576358 = {S36292, 150, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 GLOBALBuild/4.5.1_57_D2G-38) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA577463 = {S36293, 150, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;DROID2 GLOBAL Build/4.5.1_57_D2G-38) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA599843 = {S36294, 151, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; DROID2 GLOBAL Build/4.5.1_57_D2GA-59) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA649282 = {S36295, 76, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; DROID2 GLOBAL Build/4.5.1_57_D2GA-59)*/
const Device UA656227 = {S36296, 151, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID2GLOBAL Build/4.5.1_57_D2GA-59) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA692006 = {S36297, 150, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; DROID2 GLOBAL Build/4.5.1_57_D2G-38) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA700795 = {S36298, 150, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID2 GLOBAL Build/4.5.1_57_D2G-38) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA776286 = {S36299, 151, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID2 GLOBAL Build/4.5.1_57_D2GA-59) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA798876 = {S36300, 151, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;DROID2 GLOBAL Build/4.5.1_57_D2GA-59) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA852234 = {S36301, 151, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; DROID2 GLOBAL Build/4.5.1_57_D2GA-59) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1926258 = {S36302, 160, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID2 GLOBAL Build/VZW; liquid.droid2we.X.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024003 = {S36303, 141, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID2 GLOBAL Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027524 = {S36304, 139, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; DROID2 GLOBAL Build/4.5.1_57_D2G-38) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2028751 = {S36305, 144, { &P12562,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; DROID2 GLOBAL Build/4.5.1_57_D2G-38) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA50453 = {S36306, 137, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Moto Droid Build/FRG01B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA105435 = {S36307, 131, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Droid Build/FRF57) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA107927 = {S36308, 131, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Droid Build/FRFxx) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA111457 = {S36309, 132, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Droid Build/FRF84B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA125431 = {S36310, 132, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Droid Build/FRG01B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA125785 = {S36311, 131, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; droid Build/FRF83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA127629 = {S36312, 131, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Droid Build/FRG22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA129892 = {S36313, 132, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Droid Build/FRG22D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA142494 = {S36314, 133, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Droid Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA142847 = {S36315, 134, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Droid Build/FRG22D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA158843 = {S36316, 134, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Droid Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA171942 = {S36317, 134, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Droid Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA188493 = {S36318, 58, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; Droid Build/FRG83G)*/
const Device UA268200 = {S36319, 132, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Droid Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA297306 = {S36320, 58, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; Droid Build/FRG83D)*/
const Device UA302666 = {S36321, 134, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Droid Build/FRG83S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369299 = {S36322, 63, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.2.3; en_US; Droid Build/FRK76)*/
const Device UA576299 = {S36323, 133, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; Droid Build/FRK76) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633481 = {S36324, 57, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.3; Droid Build/FRK76)*/
const Device UA650666 = {S36325, 131, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Droid Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA662827 = {S36326, 132, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; en-us;Droid Build/FRK76) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA746472 = {S36327, 132, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; Droid Build/FRK76) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA750411 = {S36328, 132, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.3; en-us; Droid Build/FRK76) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA786989 = {S36329, 132, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.3; en-us; Droid Build/FRK76) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA896626 = {S36330, 133, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; sholes/sourceidnull) AppleWebKit/533.1(KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018092 = {S36331, 56, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; Droid Build/FRG22D)*/
const Device UA2018480 = {S36332, 131, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Droid Build/FRG01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019733 = {S36333, 112, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Droid Build/FRG22D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0*/
const Device UA2021003 = {S36334, 131, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; ko-; Droid Build/FRK76) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021401 = {S36335, 140, { &P13257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; Droid Build/FRK76) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA135427 = {S36336, 95, { &P15767,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA155627 = {S36337, 110, { &P15767,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148*/
const Device UA156915 = {S36338, 97, { &P15767,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko)*/
const Device UA289940 = {S36339, 109, { &P15767,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C134b*/
const Device UA293066 = {S36340, 126, { &P15767,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/533.17.9*/
const Device UA455810 = {S36341, 239, { &P15767,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 4_2_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad1,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.2.1;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]*/
const Device UA603572 = {S36342, 135, { &P15767,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 iPad1,1/K48AP Zite/1.3.3*/
const Device UA658888 = {S36343, 124, { &P15767,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/528.16*/
const Device UA715247 = {S36344, 108, { &P15767,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; ru-ru) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C134*/
const Device UA730532 = {S36345, 144, { &P15767,&P17145,&P17596,&P18092 }};	/*Useragent - MQQBrowser/1.9.1 Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/7534.48.3*/
const Device UA803675 = {S36346, 108, { &P15767,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/7B367*/
const Device UA1903211 = {S36347, 141, { &P15767,&P17145,&P17596,&P18092 }};	/*Useragent - MQQBrowser/18 Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/7534.48.3*/
const Device UA2317888 = {S36348, 235, { &P15767,&P17145,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 4_2 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPad1,1;FBMD/iPad;FBSN/iPhone OS;FBSV/4.2;FBSS/1; FBCR/;FBID/tablet;FBLC/en_US;FBSF/1.0]*/
const Device UA2581159 = {S36349, 147, { &P18125,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2607234 = {S36350, 69, { &P18125,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; DROID4 Build/9.8.2O-72_VZW-18)*/
const Device UA2702562 = {S36351, 140, { &P18125,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2751448 = {S36352, 149, { &P18125,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID4 Build/9.8.2O-72_VZW-18-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3046250 = {S36353, 145, { &P18125,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA716728 = {S36354, 139, { &P18495,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Prism Build/HuaweiU8651) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA765224 = {S36355, 140, { &P18495,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U8651T Build/HuaweiU8651) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA895394 = {S36356, 63, { &P18495,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; Prism Build/HuaweiU8651)*/
const Device UA1958171 = {S36357, 138, { &P18495,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; PrismBuild/HuaweiU8651) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1979232 = {S36358, 138, { &P18495,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.6; en-us; Prism Build/HuaweiU8651) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1992264 = {S36359, 138, { &P18495,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Prism Build/HuaweiU8651) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA2023604 = {S36360, 138, { &P18495,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; Prism Build/HuaweiU8651) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027724 = {S36361, 138, { &P18495,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us;Prism Build/HuaweiU8651) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2933371 = {S36362, 64, { &P18495,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; U8651T Build/HuaweiU8651)*/
const Device UA91269 = {S36363, 28, { &P18254,&P7480,&P18390,&P18389 }};	/*Useragent - Google-Site-Verification/1.0*/
const Device UA2584170 = {S36364, 103, { &P18254,&P7480,&P18390,&P18389 }};	/*Useragent - Mozilla 5.0 (compatible; Google-Site-Verification/1.0; +http://www.google.com/webmasters/verification/)*/
const Device UA2604483 = {S36365, 103, { &P18254,&P7480,&P18390,&P18389 }};	/*Useragent - Mozilla/5.0 (compatible; Google-Site-Verification/1.0; +http://www.google.com/webmasters/verification/)*/
const Device UA2604530 = {S36366, 54, { &P18254,&P7480,&P18390,&P18389 }};	/*Useragent - Mozilla/5.0 (compatible; Google-Site-Verification/1.0)*/
const Device UA30727 = {S36367, 51, { &P18254,&P7480,&P18245,&P22924 }};	/*Useragent - Yeti/1.0 (NHN Corp.; http://help.naver.com/robots/)*/
const Device UA1880304 = {S36368, 88, { &P18254,&P7480,&P18245,&P22924 }};	/*Useragent - Yeti/1.0 (NHN Corp.; http://help.naver.com/robots/),gzip(gfe) (via translate.google.com)*/
const Device UA2362166 = {S36369, 51, { &P18254,&P7480,&P18245,&P22924 }};	/*Useragent - Yeti/1.1 (NHN Corp.; http://help.naver.com/robots/)*/
const Device UA2715376 = {S36370, 128, { &P18635,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.74 Safari/537.36*/
const Device UA2741715 = {S36371, 135, { &P18635,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753068 = {S36372, 134, { &P18635,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Nexus 4 Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754712 = {S36373, 136, { &P18635,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 4 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA344202 = {S36374, 83, { &P15364,&P7133,&P22765,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Maxthon/3.0)*/
const Device UA576322 = {S36375, 76, { &P15364,&P7133,&P22765,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Maxthon/3.0)*/
const Device UA628868 = {S36376, 181, { &P15364,&P7133,&P22765,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Maxthon/3.0)*/
const Device UA664852 = {S36377, 174, { &P15364,&P7133,&P22765,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Maxthon/3.0)*/
const Device UA690314 = {S36378, 189, { &P15364,&P7133,&P22765,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Maxthon/3.0)*/
const Device UA782314 = {S36379, 153, { &P15364,&P7133,&P22765,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Maxthon/3.0)*/
const Device UA1907941 = {S36380, 170, { &P15364,&P7133,&P22765,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Maxthon/3.0)*/
const Device UA2148894 = {S36381, 164, { &P15364,&P7133,&P22765,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Maxthon/3.0)*/
const Device UA2161825 = {S36382, 98, { &P15364,&P7133,&P22765,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Avant Browser; Maxthon/3.0)*/
const Device UA2162630 = {S36383, 91, { &P15364,&P7133,&P22765,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Avant Browser; Maxthon/3.0)*/
const Device UA2330649 = {S36384, 191, { &P15364,&P7133,&P22765,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Maxthon/3.0)*/
const Device UA2423108 = {S36385, 84, { &P15364,&P7133,&P22765,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Maxthon/3.0)*/
const Device UA2517719 = {S36386, 201, { &P15364,&P7133,&P22765,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Zune 4.7; Maxthon/3.0)*/
const Device UA2528649 = {S36387, 77, { &P15364,&P7133,&P22765,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Maxthon/3.0)*/
const Device UA2842451 = {S36388, 99, { &P15364,&P7133,&P22765,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Avant Browser; Maxthon/3.0)*/
const Device UA3070440 = {S36389, 211, { &P15364,&P7133,&P22765,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.1; Tablet PC 2.0; .NET4.0C; .NET4.0E; Maxthon/3.0)*/
const Device UA11337 = {S36390, 143, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; MB860 Build/OLYLA_U4_0.37.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11339 = {S36391, 173, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.4.1.16.MB860.ATT.en.US Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11358 = {S36392, 143, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; MB860 Build/OLSKT_U4_1.14.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11360 = {S36393, 154, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-MB860/ Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; MB860 Build/OLYLA_U4_0.50.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11361 = {S36394, 176, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB860 Build/Blur_Version.4.1.31.MB860.Orange.en.GB Flex/P025) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11363 = {S36395, 172, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/Blur_Version.0.0.1.MB860.ATT.en.US Flex/P013) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11365 = {S36396, 178, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; MB860 Build/Blur_Version.4.0.240.MB860.Telstra.en.AU Flex/P003) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11366 = {S36397, 182, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; MB860 Build/Blur_Version.0.37.4.MB860.BellMobility.en.CA Flex/P025) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11369 = {S36398, 142, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/OLYFR_U4_1.5.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11693 = {S36399, 142, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/OLYFR_U4_1.8.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11694 = {S36400, 173, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/Blur_Version.4.1.83.MB860.ATT.en.US Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11696 = {S36401, 173, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.4.1.52.MB860.ATT.en.US Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11697 = {S36402, 173, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.4.1.18.MB860.ATT.en.US Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11744 = {S36403, 173, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.4.1.57.MB860.ATT.en.US Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11745 = {S36404, 142, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/OLYFR_U4_1.5.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11746 = {S36405, 175, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.4.0.1960.MB860.ATT.en.US Flex/P015) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11747 = {S36406, 173, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.4.1.23.MB860.ATT.en.US Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11748 = {S36407, 172, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.0.0.1.MB860.ATT.en.US Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11749 = {S36408, 142, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/OLYFR_U4_1.2.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11751 = {S36409, 176, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.4.0.19915.MB860.ATT.en.US Flex/P009) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11752 = {S36410, 173, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/Blur_Version.4.1.75.MB860.ATT.en.US Flex/P013) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24224 = {S36411, 173, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/Blur_Version.4.1.83.MB860.ATT.en.US Flex/P013) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA26928 = {S36412, 162, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.0.0.1.MB860.ATT.en.US) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA26932 = {S36413, 173, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.4.1.26.MB860.ATT.en.US Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40093 = {S36414, 182, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; MB860 Build/Blur_Version.0.37.4.MB860.BellMobility.en.CA Flex/P024) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40095 = {S36415, 142, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-au; MB860 Build/OLYEM_U4_1.4.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40118 = {S36416, 177, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/Blur_Version.4.0.530.MB860.Retail.en.GB Flex/P022) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40119 = {S36417, 154, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-MB860/ Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; MB860 Build/OLYLA_U4_0.54.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48030 = {S36418, 142, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-sg; MB860 Build/OLYEM_U4_3.1.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48037 = {S36419, 178, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/Blur_Version.4.3.10.MB860.MERetail.en.06 Flex/P008) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48042 = {S36420, 142, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; MB860 Build/OLYEM_U4_2.1.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48044 = {S36421, 142, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB860 Build/OLYEM_U4_4.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48052 = {S36422, 180, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-sg; MB860 Build/Blur_Version.4.3.13.MB860.AsiaRetail.en.03 Flex/P021) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48053 = {S36423, 174, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; cs-; MB860 Build/Blur_Version.4.4.20.MB860.Retail.en.DE Flex/P023) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48064 = {S36424, 177, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; he-il; MB860 Build/Blur_Version.4.4.80.MB860.Partner.en.IL Flex/P013) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48070 = {S36425, 176, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB860 Build/Blur_Version.4.4.35.MB860.Retail.en.GB Flex/P026) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48072 = {S36426, 176, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; MB860 Build/Blur_Version.4.2.13.MB860.Retail.en.FR Flex/P016) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48075 = {S36427, 176, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; MB860 Build/Blur_Version.4.4.20.MB860.Retail.en.DE Flex/P023) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48079 = {S36428, 177, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-sg; MB860 Build/Blur_Version.4.3.12.MB860.Starhub.en.SG Flex/P007) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48080 = {S36429, 178, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; MB860 Build/Blur_Version.4.4.23.MB860.Vodafone.en.EU Flex/P016) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48086 = {S36430, 142, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/OLYEM_U4_3.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55329 = {S36431, 144, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; MB860 Build/OLYLA_U4_0.37.23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55330 = {S36432, 158, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-MB860/ Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; MB860 Build/OLYLA_U4_0.50.0_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55334 = {S36433, 183, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; MB860 Build/Blur_Version.0.37.23.MB860.BellMobility.en.CA Flex/P026) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55335 = {S36434, 177, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-au; MB860 Build/Blur_Version.4.1.42.MB860.Telstra.en.AU Flex/P029) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55338 = {S36435, 174, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; MB860 Build/Blur_Version.4.1.200.MB860.SKT.en.KR Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55348 = {S36436, 158, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-MB860/ Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; MB860 Build/OLYLA_U4_0.50.0_R03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55353 = {S36437, 182, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.4.1.120.ME860.ChinaRetail.en.CN Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55369 = {S36438, 143, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; MB860 Build/OLSKT_U4_1.20.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55374 = {S36439, 142, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-au; MB860 Build/OLYEM_U4_1.4.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55384 = {S36440, 142, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; MB860 Build/OLYEM_U4_1.3.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55389 = {S36441, 177, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-au; MB860 Build/Blur_Version.4.1.46.MB860.Telstra.en.AU Flex/P032) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55399 = {S36442, 174, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/Blur_Version.4.1.140.MB860.SKT.en.KR Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55406 = {S36443, 158, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-MB860/ Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; MB860 Build/OLYLA_U4_0.43.0_R03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55414 = {S36444, 181, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; MB860 Build/Blur_Version.4.2.11.MB860.Retail - Blurred. Flex/P014) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55371 = {S36445, 173, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/Blur_Version.4.1.84.MB860.ATT.en.US Flex/P014) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55425 = {S36446, 142, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB860 Build/OLYEM_U4_1.3.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55426 = {S36447, 176, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; MB860 Build/Blur_Version.4.1.33.MB860.Orange.en.FR Flex/P034) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55421 = {S36448, 174, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/Blur_Version.4.1.170.MB860.SKT.en.KR Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55705 = {S36449, 173, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB860 Build/Blur_Version.4.1.57.MB860.ATT.en.US Flex/P013) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55710 = {S36450, 177, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-au; MB860 Build/Blur_Version.4.1.47.MB860.Telstra.en.AU Flex/P033) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55713 = {S36451, 158, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-MB860/ Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; MB860 Build/OLYLA_U4_0.54.0_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60997 = {S36452, 180, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-sg; MB860 Build/Blur_Version.4.3.10.MB860.AsiaRetail.en.03 Flex/P020) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60998 = {S36453, 178, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB860 Build/Blur_Version.4.2.23.MB860.T-Mobile.en.GB Flex/P021) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61000 = {S36454, 142, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB860 Build/OLYEM_U4_2.2.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61003 = {S36455, 142, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB860 Build/OLYEM_U4_4.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61004 = {S36456, 176, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB860 Build/Blur_Version.4.4.20.MB860.Orange.en.GB Flex/P035) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61006 = {S36457, 176, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; MB860 Build/Blur_Version.4.4.20.MB860.Orange.en.FR Flex/P038) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61007 = {S36458, 142, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-sg; MB860 Build/OLYEM_U4_3.1.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61010 = {S36459, 172, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; MB860 Build/Blur_Version.4.4.20.MB860.O2.en.DE Flex/P020) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70873 = {S36460, 176, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/Blur_Version.4.2.11.MB860.Retail.en.FR Flex/P014) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70929 = {S36461, 158, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-MB860/ Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; MB860 Build/OLYLA_U4_0.54.0_R05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72287 = {S36462, 182, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB860 Build/Blur_Version.4.1.120.ME860.ChinaRetail.en.CN Flex/P014) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73338 = {S36463, 173, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; MB860 Build/Blur_Version.4.4.20.MB860.TIM.en.IT Flex/P017) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA77991 = {S36464, 154, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-MB860/ Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB860 Build/OLYLA_U4_0.56.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79122 = {S36465, 178, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB860 Build/Blur_Version.4.4.80.MB860.MERetail.en.06 Flex/P013) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90529 = {S36466, 177, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; MB860 Build/Blur_Version.4.0.300.MB860.Orange.en.GB Flex/P009) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA174874 = {S36467, 66, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; MB860 Build/OLYFR_U4_1.8.3)*/
const Device UA371073 = {S36468, 176, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB860 Build/Blur_Version.4.4.35.MB860.Retail.en.GB Flex/P023) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA524056 = {S36469, 66, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; MB860 Build/OLYFR_U4_1.2.6)*/
const Device UA784300 = {S36470, 130, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB860 Build/OLYEM_U4_2.1.1) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1908705 = {S36471, 142, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; he-il; MB860 Build/OLYEM_U4_4.8.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1938602 = {S36472, 158, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-MB860/ Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; MB860 Build/OLYLA_U4_0.54.0_R06) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019446 = {S36473, 131, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB860 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026442 = {S36474, 135, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB860 Build/OLYEM_U4_2.1.1) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2573315 = {S36475, 142, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB860 Build/OLYEM_U4_2.1.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2699672 = {S36476, 176, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MB860 Build/Blur_Version.4.4.33.MB860.Retail.en.ES Flex/P029) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2787428 = {S36477, 135, { &P17283,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB860 Build/OLYEM_U4_3.1.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2431189 = {S36478, 105, { &P15988,&P17484,&P17373,&P18092 }};	/*Useragent - SAMSUNG-GT-C3312 Opera/9.80 (J2ME/MIDP; Opera Mini/6.5.29390/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2974024 = {S36479, 136, { &P20705,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974287 = {S36480, 135, { &P20705,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTCONE Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974614 = {S36481, 138, { &P20705,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC_PN071 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975079 = {S36482, 141, { &P20705,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One 801e Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022220 = {S36483, 145, { &P20705,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One dual sim Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA245285 = {S36484, 139, { &P16349,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Sony Tablet S Build/THMAS11000) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA252039 = {S36485, 140, { &P16349,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; Sony Tablet S Build/THMASS11004) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA359315 = {S36486, 69, { &P16349,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; Sony Tablet S Build/THMAS0042)*/
const Device UA451883 = {S36487, 139, { &P16349,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; Sony Tablet S Build/THMAS11002) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA571271 = {S36488, 68, { &P16349,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; Sony Tablet S Build/THMAS11000)*/
const Device UA629583 = {S36489, 141, { &P16349,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Sony Tablet S Build/THMASU0035) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA632024 = {S36490, 140, { &P16349,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; es-es; Sony Tablet S Build/THMAS0050) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA633048 = {S36491, 140, { &P16349,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Sony Tablet S Build/THMAS0042) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA658385 = {S36492, 139, { &P16349,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; Sony Tablet S Build/THMAS11001) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA826308 = {S36493, 140, { &P16349,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; Sony Tablet S Build/THMA3G11005) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA841687 = {S36494, 141, { &P16349,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; de-de; Sony Tablet S Build/THMAST0066) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1937752 = {S36495, 138, { &P16349,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; Sony Tablet S Build/THMD01900) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2020747 = {S36496, 140, { &P16349,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; zh-cn; Sony Tablet S Build/THMAS0030) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2020843 = {S36497, 141, { &P16349,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; tr-tr; Sony Tablet S Build/THMASU0032) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2205968 = {S36498, 107, { &P17595,&P18207,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/537.9 (KHTML, like Gecko) Version/6.1 Safari/537.9*/
const Device UA2238808 = {S36499, 111, { &P17595,&P18207,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/537.12 (KHTML, like Gecko) Version/6.1 Safari/537.12.2*/
const Device UA2258462 = {S36500, 113, { &P17595,&P18207,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/537.15.1 (KHTML, like Gecko) Version/6.1 Safari/537.15.1*/
const Device UA2349203 = {S36501, 111, { &P17595,&P18207,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.11 (KHTML, like Gecko) Version/6.1 Safari/537.11*/
const Device UA2357078 = {S36502, 109, { &P17595,&P18207,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/537.17 (KHTML, like Gecko) Version/6.1 Safari/537.17*/
const Device UA2388425 = {S36503, 113, { &P17595,&P18207,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/537.18.1 (KHTML, like Gecko) Version/6.1 Safari/537.18.1*/
const Device UA2416944 = {S36504, 109, { &P17595,&P18207,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.7 (KHTML, like Gecko) Version/6.1 Safari/537.7*/
const Device UA2425162 = {S36505, 113, { &P17595,&P18207,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/537.20.3 (KHTML, like Gecko) Version/6.1 Safari/537.20.3*/
const Device UA2429536 = {S36506, 109, { &P17595,&P18207,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/537.22 (KHTML, like Gecko) Version/6.1 Safari/537.22*/
const Device UA2437498 = {S36507, 109, { &P17595,&P18207,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/537.22 (KHTML, like Gecko) Version/6.1 Safari/537.23*/
const Device UA2561601 = {S36508, 115, { &P17595,&P18207,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.28.1 (KHTML, like Gecko) Version/6.1 Safari/537.28.1*/
const Device UA2796394 = {S36509, 111, { &P17595,&P18207,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.41 (KHTML, like Gecko) Version/6.1 Safari/537.41*/
const Device UA2797280 = {S36510, 117, { &P17595,&P18207,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.43.58 (KHTML, like Gecko) Version/6.1 Safari/537.43.58*/
const Device UA2855392 = {S36511, 115, { &P17595,&P18207,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.31.2 (KHTML, like Gecko) Version/6.1 Safari/537.31.2*/
const Device UA2886770 = {S36512, 115, { &P17595,&P18207,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.46.3 (KHTML, like Gecko) Version/6.1 Safari/537.46.3*/
const Device UA2947262 = {S36513, 115, { &P17595,&P18207,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.48.2 (KHTML, like Gecko) Version/6.1 Safari/537.48.2*/
const Device UA2974580 = {S36514, 115, { &P17595,&P18207,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.51.3 (KHTML, like Gecko) Version/6.1 Safari/537.51.3*/
const Device UA3033899 = {S36515, 111, { &P17595,&P18207,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.59 (KHTML, like Gecko) Version/6.1 Safari/537.59*/
const Device UA3036990 = {S36516, 115, { &P17595,&P18207,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.54.1 (KHTML, like Gecko) Version/6.1 Safari/537.54.1*/
const Device UA3067788 = {S36517, 111, { &P17595,&P18207,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.66 (KHTML, like Gecko) Version/6.1 Safari/537.66*/
const Device UA3094884 = {S36518, 111, { &P17595,&P18207,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.71 (KHTML, like Gecko) Version/6.1 Safari/537.71*/
const Device UA2198235 = {S36519, 144, { &P15767,&P17090,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8G4 Safari/7534.48.3*/
const Device UA2205500 = {S36520, 144, { &P15767,&P17090,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8J2 Safari/7534.48.3*/
const Device UA2205615 = {S36521, 144, { &P15767,&P17090,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8L1 Safari/7534.48.3*/
const Device UA2205691 = {S36522, 144, { &P15767,&P17090,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8J3 Safari/7534.48.3*/
const Device UA2205814 = {S36523, 144, { &P15767,&P17090,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8H7 Safari/7534.48.3*/
const Device UA2234291 = {S36524, 144, { &P15767,&P17090,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8F190 Safari/7534.48.3*/
const Device UA2235348 = {S36525, 144, { &P15767,&P17090,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8F191 Safari/7534.48.3*/
const Device UA2239218 = {S36526, 144, { &P15767,&P17090,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8K2 Safari/7534.48.3*/
const Device UA2366865 = {S36527, 144, { &P15767,&P17090,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8H8 Safari/7534.48.3*/
const Device UA8493 = {S36528, 73, { &P14714,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax3-02*/
const Device UA52267 = {S36529, 79, { &P14714,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaX3-02/5.0 (05.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 NokiaX3-02/05.60*/
const Device UA64355 = {S36530, 123, { &P14714,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA397350 = {S36531, 125, { &P14714,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaX3-02.5/5.0 (06.75) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA603351 = {S36532, 130, { &P15364,&P7133,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/19.0.1077.3) AppleWebKit/536.4 (KHTML, like Gecko) Chrome/19.0.1077.3 Safari/536.4*/
const Device UA610740 = {S36533, 132, { &P15364,&P7133,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/19.0.1084.30) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.30 Safari/536.5*/
const Device UA623189 = {S36534, 132, { &P15364,&P7133,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/19.0.1084.46) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5*/
const Device UA749703 = {S36535, 125, { &P15364,&P7133,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/19.0.1084.46) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5*/
const Device UA897380 = {S36536, 125, { &P15364,&P7133,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/19.0.1084.52) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5*/
const Device UA897515 = {S36537, 132, { &P15364,&P7133,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/19.0.1084.52) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5*/
const Device UA897851 = {S36538, 197, { &P15364,&P7133,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/19.0.1084.52; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA897882 = {S36539, 212, { &P15364,&P7133,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/19.0.1084.52; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA898098 = {S36540, 197, { &P15364,&P7133,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/19.0.1084.52; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA898446 = {S36541, 176, { &P15364,&P7133,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/19.0.1084.52; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA900838 = {S36542, 96, { &P15364,&P7133,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/19.0.1084.56)*/
const Device UA900958 = {S36543, 229, { &P15364,&P7133,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/19.0.1084.56; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8)*/
const Device UA901054 = {S36544, 132, { &P15364,&P7133,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/19.0.1084.56) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA901206 = {S36545, 125, { &P15364,&P7133,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/19.0.1084.56) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA901540 = {S36546, 89, { &P15364,&P7133,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/19.0.1084.56)*/
const Device UA1968232 = {S36547, 192, { &P15364,&P7133,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/19.0.1084.52; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)*/
const Device UA1977139 = {S36548, 191, { &P15364,&P7133,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/19.0.1084.52; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA2166076 = {S36549, 229, { &P15364,&P7133,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/19.0.1084.56; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8)*/
const Device UA2334366 = {S36550, 176, { &P15364,&P7133,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/19.0.1084.52; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MS-RTC LM 8)*/
const Device UA2643341 = {S36551, 282, { &P15364,&P7133,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/19.0.1084.56; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2766987 = {S36552, 97, { &P15364,&P7133,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/19.0.1084.56)*/
const Device UA2773044 = {S36553, 142, { &P12280,&P18432,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A523 Safari/8536.25*/
const Device UA2773047 = {S36554, 140, { &P12280,&P18432,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A405 Safari/8536.25*/
const Device UA2773059 = {S36555, 142, { &P12280,&P18432,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A551 Safari/8536.25*/
const Device UA2773068 = {S36556, 140, { &P12280,&P18432,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A403 Safari/8536.25*/
const Device UA2773076 = {S36557, 142, { &P12280,&P18432,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A525 Safari/8536.25*/
const Device UA2741595 = {S36558, 141, { &P16292,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2741703 = {S36559, 140, { &P16292,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769782 = {S36560, 133, { &P16292,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2773548 = {S36561, 139, { &P16292,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Galaxy Nexus Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2947601 = {S36562, 118, { &P15364,&P16110,&P22763,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1*/
const Device UA2948535 = {S36563, 125, { &P15364,&P16110,&P22763,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1*/
const Device UA3017685 = {S36564, 135, { &P15364,&P16110,&P22763,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1,gzip(gfe)*/
const Device UA32838 = {S36565, 152, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17a Build/4.0.1.A.0.32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47727 = {S36566, 153, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.1.A.0.233) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47726 = {S36567, 153, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.1.A.0.215) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47744 = {S36568, 153, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.1.A.0.204) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47723 = {S36569, 153, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.1.A.0.247) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49576 = {S36570, 153, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.1.A.0.263) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49577 = {S36571, 153, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.1.A.0.271) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49580 = {S36572, 153, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.1.A.0.284) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49584 = {S36573, 152, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonST17i Build/4.0.1.A.2.24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60621 = {S36574, 152, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.1.A.0.57) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70695 = {S36575, 152, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.2.A.0.19) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73132 = {S36576, 152, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.2.A.0.35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74523 = {S36577, 151, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.2.A.0.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA254004 = {S36578, 153, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-nz; SonyEricssonST17a Build/4.0.1.A.0.284) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368697 = {S36579, 140, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; ST17i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368703 = {S36580, 141, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-nz; ST17a Build/4.0.1.A.0.284) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA624956 = {S36581, 152, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA636279 = {S36582, 152, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; nb-no; SonyEricssonST17i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA658219 = {S36583, 152, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonST17i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA659052 = {S36584, 64, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; ST17i Build/4.0.2.A.0.62)*/
const Device UA709218 = {S36585, 153, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-za; SonyEricssonST17iv Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1945022 = {S36586, 152, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-cl; SonyEricssonST17a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1966662 = {S36587, 152, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-nz; SonyEricssonST17a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1972801 = {S36588, 140, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; ST17i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019667 = {S36589, 152, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonST17i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021525 = {S36590, 140, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ST17i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022712 = {S36591, 141, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ST17i Build/4.0.1.A.0.284) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024646 = {S36592, 140, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ST17a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2512119 = {S36593, 152, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonST17i Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2662882 = {S36594, 152, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonST17a Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2730594 = {S36595, 140, { &P15354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; ST17i Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA589371 = {S36596, 71, { &P15364,&P5690,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko/X Firefox/11.0 Firefox/11.0*/
const Device UA606110 = {S36597, 71, { &P15364,&P5690,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko/X Firefox/11.0*/
const Device UA2167890 = {S36598, 56, { &P15364,&P5690,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko Firefox/11.0*/
const Device UA2219433 = {S36599, 70, { &P15364,&P5690,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; rv:11.0) Gecko/X Firefox/11.0*/
const Device UA2466776 = {S36600, 78, { &P15364,&P5690,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2) Gecko/X Firefox/11.0*/
const Device UA2717894 = {S36601, 61, { &P15364,&P5690,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:11.0) Gecko/11.0 Firefox/11.0*/
const Device UA2772848 = {S36602, 131, { &P15767,&P18432,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A403 Safari/8536.25*/
const Device UA2773045 = {S36603, 133, { &P15767,&P18432,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A523 Safari/8536.25*/
const Device UA2773079 = {S36604, 133, { &P15767,&P18432,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A550 Safari/8536.25*/
const Device UA2773060 = {S36605, 134, { &P15767,&P18432,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A8426 Safari/8536.25*/
const Device UA2773488 = {S36606, 131, { &P15767,&P18432,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A407 Safari/8536.25*/
const Device UA2773502 = {S36607, 134, { &P15767,&P18432,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A8500 Safari/8536.25*/
const Device UA2783678 = {S36608, 131, { &P15767,&P18432,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A406 Safari/8536.25*/
const Device UA630042 = {S36609, 74, { &P15364,&P18110,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:11.0) Gecko/X Firefox/11.0 Iceweasel/11.0*/
const Device UA845269 = {S36610, 79, { &P15364,&P18110,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.16) Gecko/X Gecko Firefox/11.0*/
const Device UA2331607 = {S36611, 72, { &P15364,&P18110,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:11.0) Gecko/X Firefox/11.0 Firefox/11.0*/
const Device UA2879467 = {S36612, 132, { &P18466,&P18110,&P23483,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Safari/535.19 Silk-Accelerated=true*/
const Device UA889783 = {S36613, 133, { &P20934,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; 5860E Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031038 = {S36614, 133, { &P20934,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; 5860e Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2225669 = {S36615, 57, { &P20934,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; 5860E Build/GWK74)*/
const Device UA2693873 = {S36616, 213, { &P20934,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; 5860E Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Android MetroPCS androidAppVersion/2.5.10 AndroidWebkitWrapper BillingSupported*/
const Device UA2696358 = {S36617, 131, { &P20934,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-; 5860e Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3124833 = {S36618, 178, { &P20934,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; 5860E Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 androidAppVersion/2.6.8 AndroidWebkitWrapper*/
const Device UA24586 = {S36619, 144, { &P14183,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42968 = {S36620, 143, { &P14183,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA70070 = {S36621, 143, { &P14183,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SAMSUNG-SGH-I997Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71178 = {S36622, 80, { &P14183,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/0.9.0 (Linux; U; Android 2.2.1; en_US; SAMSUNG-SGH-I997 Build/FROYO.UCKE3)*/
const Device UA72867 = {S36623, 146, { &P14183,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA74427 = {S36624, 143, { &P14183,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA207076 = {S36625, 68, { &P14183,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SAMSUNG-SGH-I997 Build/FROYO)*/
const Device UA369308 = {S36626, 80, { &P14183,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.2.1; en_US; SAMSUNG-SGH-I997 Build/FROYO.UCKH1)*/
const Device UA371309 = {S36627, 132, { &P14183,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SAMSUNG-SGH-I997 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA633332 = {S36628, 142, { &P14183,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1907228 = {S36629, 143, { &P14183,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019381 = {S36630, 151, { &P14183,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2022395 = {S36631, 143, { &P14183,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.1; en-us; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2405234 = {S36632, 124, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC)*/
const Device UA2421257 = {S36633, 106, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; A620b)*/
const Device UA2429527 = {S36634, 106, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; A620e)*/
const Device UA2432021 = {S36635, 132, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; Orange)*/
const Device UA2433520 = {S36636, 106, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; A620m)*/
const Device UA2486660 = {S36637, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 1.04.163.03)*/
const Device UA2506089 = {S36638, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 1.04.161.01)*/
const Device UA2506158 = {S36639, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 1.04.166.01)*/
const Device UA2507162 = {S36640, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 1.04.165.01)*/
const Device UA2529874 = {S36641, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 1.04.169.01)*/
const Device UA2530161 = {S36642, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 1.04.162.01)*/
const Device UA2556076 = {S36643, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 1.04.172.01)*/
const Device UA2604449 = {S36644, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 2.00.161.01)*/
const Device UA2659820 = {S36645, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 1.04.183.01)*/
const Device UA2669878 = {S36646, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 2.00.165.01)*/
const Device UA2670103 = {S36647, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 2.00.166.01)*/
const Device UA2671014 = {S36648, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 2.00.163.02)*/
const Device UA2704404 = {S36649, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 2.00.172.01)*/
const Device UA2705396 = {S36650, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 2.00.169.01)*/
const Device UA2705758 = {S36651, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 2.00.162.01)*/
const Device UA2729253 = {S36652, 112, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; A620t; CMCC)*/
const Device UA2738031 = {S36653, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 2.00.162.02)*/
const Device UA2752155 = {S36654, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 2.00.172.02)*/
const Device UA2866612 = {S36655, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 2.00.183.01)*/
const Device UA2987993 = {S36656, 120, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; A620d; ChinaTelecom)*/
const Device UA3030059 = {S36657, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 3.29.163.01)*/
const Device UA3030327 = {S36658, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 3.29.161.01)*/
const Device UA3036017 = {S36659, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 3.29.166.01)*/
const Device UA3058590 = {S36660, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 3.29.162.01)*/
const Device UA3058694 = {S36661, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 3.29.172.01)*/
const Device UA3058762 = {S36662, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 3.29.169.01)*/
const Device UA3079565 = {S36663, 137, { &P18503,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; Windows Phone 8S by HTC; 3.29.165.01)*/
const Device UA2489319 = {S36664, 114, { &P18267,&P5043,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-02/11.79; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2185219 = {S36665, 69, { &P15364,&P18110,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 on x86_64; rv:16.0) Gecko/X Firefox/16.0*/
const Device UA2260216 = {S36666, 78, { &P15364,&P18110,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:16.0) Gecko/X Firefox/16.0 Iceweasel/16.0.2*/
const Device UA2368156 = {S36667, 60, { &P15364,&P18110,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:16.0) Gecko/X Abrowser/16.0*/
const Device UA2387296 = {S36668, 62, { &P15364,&P18110,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:16.0) Gecko/X Abrowser/16.0*/
const Device UA18535 = {S36669, 151, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-ca; SonyEricssonLT15a Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA33237 = {S36670, 151, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonLT15a Build/4.0.A.2.286) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41909 = {S36671, 151, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; SonyEricssonLT15a Build/3.0.A.2.118) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41919 = {S36672, 151, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; es-cl; SonyEricssonLT15a Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57775 = {S36673, 151, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; SonyEricssonLT15a Build/3.0.A.0.326) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57776 = {S36674, 141, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; LT15a Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57791 = {S36675, 153, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; SonyEricssonLT15a Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73469 = {S36676, 151, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-mx; SonyEricssonLT15a Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73619 = {S36677, 154, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; SonyEricssonLT15av Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA348362 = {S36678, 140, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; LT15a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370236 = {S36679, 152, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonLT15a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA556673 = {S36680, 65, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; LT15a Build/3.0.1.A.0.145)*/
const Device UA622953 = {S36681, 152, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; SonyEricssonLT15a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625377 = {S36682, 151, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; SonyEricssonLT15a Build/4.0.2.D.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA637929 = {S36683, 151, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; SonyEricssonLT15a Build/4.0.2.D.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA662432 = {S36684, 153, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; SonyEricssonLT15av Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA690043 = {S36685, 140, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LT15a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA698453 = {S36686, 64, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LT15a Build/4.0.2.A.0.42)*/
const Device UA728796 = {S36687, 153, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; SonyEricssonLT15av Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA754653 = {S36688, 139, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; iw-il; LT15a Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1987530 = {S36689, 139, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; LT15a Build/4.0.2.D.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1990468 = {S36690, 152, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-cl; SonyEricssonLT15a Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023601 = {S36691, 139, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; es-cl; LT15a Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034606 = {S36692, 139, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; LT15a Build/4.0.2.D.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2360090 = {S36693, 139, { &P14111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; es-co; LT15a Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA637010 = {S36694, 74, { &P15364,&P5690,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:9.0.1) Gecko/X Firefox/9.0.1 Firefox/9.0.1*/
const Device UA651778 = {S36695, 56, { &P15364,&P5690,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:9.0) Gecko/X Firefox/9.0*/
const Device UA878847 = {S36696, 69, { &P15364,&P5690,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 5.1; rv:9.0.1) Gecko/X Firefox/9.0.1*/
const Device UA894385 = {S36697, 50, { &P15364,&P5690,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) Gecko/X Firefox/9.0.1*/
const Device UA2234785 = {S36698, 70, { &P15364,&P5690,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:9.0.1) Gecko/X Firefox/9.0.1,gzip(gfe)*/
const Device UA2199530 = {S36699, 58, { &P15364,&P18110,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux zbov) Presto/2.11.355 Version/12.10*/
const Device UA2355630 = {S36700, 68, { &P15364,&P18110,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux zbov) Presto/2.11.355 Version/12.10,gzip(gfe)*/
const Device UA2387056 = {S36701, 63, { &P15364,&P18110,&P21951,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; X11; Linux i686) Opera 12.10*/
const Device UA2424520 = {S36702, 71, { &P15364,&P18110,&P21951,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.10*/
const Device UA2387786 = {S36703, 57, { &P15364,&P16110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2) Presto/2.12.388 Version/12.12*/
const Device UA2405336 = {S36704, 64, { &P15364,&P16110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.12*/
const Device UA2427297 = {S36705, 86, { &P15364,&P16110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5977)) Presto/2.12.388 Version/12.12*/
const Device UA2427900 = {S36706, 86, { &P15364,&P16110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.12*/
const Device UA2437488 = {S36707, 86, { &P15364,&P16110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.12*/
const Device UA2437540 = {S36708, 86, { &P15364,&P16110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.12*/
const Device UA2461250 = {S36709, 86, { &P15364,&P16110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6011)) Presto/2.12.388 Version/12.12*/
const Device UA2461320 = {S36710, 74, { &P15364,&P16110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.12,gzip(gfe)*/
const Device UA2468909 = {S36711, 86, { &P15364,&P16110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.12*/
const Device UA2469206 = {S36712, 86, { &P15364,&P16110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5981)) Presto/2.12.388 Version/12.12*/
const Device UA2485182 = {S36713, 86, { &P15364,&P16110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.12*/
const Device UA2485552 = {S36714, 69, { &P15364,&P16110,&P20412,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64) Opera 12.12*/
const Device UA2486311 = {S36715, 67, { &P15364,&P16110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2) Presto/2.12.388 Version/12.12,gzip(gfe)*/
const Device UA2528821 = {S36716, 86, { &P15364,&P16110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.12*/
const Device UA2557907 = {S36717, 86, { &P15364,&P16110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.12*/
const Device UA2579786 = {S36718, 86, { &P15364,&P16110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.12*/
const Device UA2659547 = {S36719, 86, { &P15364,&P16110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.12*/
const Device UA2667110 = {S36720, 86, { &P15364,&P16110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.12*/
const Device UA2697589 = {S36721, 86, { &P15364,&P16110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.12*/
const Device UA2704163 = {S36722, 86, { &P15364,&P16110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6060)) Presto/2.12.388 Version/12.12*/
const Device UA2852500 = {S36723, 86, { &P15364,&P16110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.12*/
const Device UA2998472 = {S36724, 86, { &P15364,&P16110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.1 (build 6578)) Presto/2.12.388 Version/12.12*/
const Device UA2886409 = {S36725, 142, { &P22394,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; C6603 Build/10.3.A.0.423) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2895702 = {S36726, 146, { &P22394,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SonyC6603 Build/10.3.A.0.423) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2964867 = {S36727, 66, { &P22394,&P18598,&P19802,&P18092 }};	/*Useragent - SonyC6603 Build/10.3.A.0.423 stagefright/1.2 (Linux;Android 4.2.2)*/
const Device UA3019761 = {S36728, 66, { &P22394,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; C6603 Build/10.3.1.A.0.244)*/
const Device UA3026330 = {S36729, 148, { &P22394,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SonyC6603 Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3026489 = {S36730, 144, { &P22394,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; C6603 Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058441 = {S36731, 68, { &P22394,&P18598,&P19802,&P18092 }};	/*Useragent - SonyC6603 Build/10.3.1.A.0.244 stagefright/1.2 (Linux;Android 4.2.2)*/
const Device UA3061103 = {S36732, 143, { &P22394,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; C6603 Build/10.3.1.A.1.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125248 = {S36733, 142, { &P22394,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-; C6603 Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3126553 = {S36734, 143, { &P22394,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; C6603 Build/10.3.1.A.2.67) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3129231 = {S36735, 147, { &P22394,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SonyC6603 Build/10.3.1.A.2.67) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA863688 = {S36736, 131, { &P17686,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MZ601 Build/I.7.1-23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA899770 = {S36737, 131, { &P17686,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; MZ601 Build/I.7.1-42) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1949527 = {S36738, 129, { &P17686,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; MZ601 Build/ITL41F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1961295 = {S36739, 131, { &P17686,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MZ601 Build/I.7.1-45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029008 = {S36740, 60, { &P17686,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MZ601 Build/I.7.1-45)*/
const Device UA2029501 = {S36741, 131, { &P17686,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MZ601 Build/I.7.1-21) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2034093 = {S36742, 131, { &P17686,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MZ601 Build/I.7.1-17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2395368 = {S36743, 131, { &P17686,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MZ601 Build/I.7.1-34) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2200464 = {S36744, 117, { &P17595,&P17297,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA2201536 = {S36745, 117, { &P17595,&P17297,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2203302 = {S36746, 113, { &P17595,&P17297,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2203575 = {S36747, 118, { &P17595,&P17297,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA2203770 = {S36748, 117, { &P17595,&P17297,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2205095 = {S36749, 117, { &P17595,&P17297,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA2285918 = {S36750, 126, { &P17595,&P17297,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7; en-us) AppleWebKit/534.20.8 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA2301762 = {S36751, 127, { &P17595,&P17297,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_2; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2316908 = {S36752, 127, { &P17595,&P17297,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_3; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2419577 = {S36753, 111, { &P17595,&P17297,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5 Safari/533.16*/
const Device UA2642933 = {S36754, 127, { &P17595,&P17297,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA3042797 = {S36755, 127, { &P17595,&P17297,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA29948 = {S36756, 101, { &P12280,&P17144,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko)*/
const Device UA29950 = {S36757, 114, { &P12280,&P17144,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A293*/
const Device UA29952 = {S36758, 116, { &P12280,&P17144,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A400*/
const Device UA29970 = {S36759, 116, { &P12280,&P17144,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A306*/
const Device UA29972 = {S36760, 103, { &P12280,&P17144,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko)*/
const Device UA29957 = {S36761, 103, { &P12280,&P17144,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko)*/
const Device UA89803 = {S36762, 113, { &P12280,&P17144,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/7D11*/
const Device UA143505 = {S36763, 129, { &P12280,&P17144,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A400 Safari/532.9*/
const Device UA489826 = {S36764, 129, { &P12280,&P17144,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A306 Safari/532.9*/
const Device UA605710 = {S36765, 248, { &P12280,&P17144,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_2 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.0.2;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA635228 = {S36766, 248, { &P12280,&P17144,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone3,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.0.1;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA639238 = {S36767, 127, { &P12280,&P17144,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A293 Safari/532.9*/
const Device UA708612 = {S36768, 244, { &P12280,&P17144,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPhone2,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/4.0;FBSS/1; FBCR/AT&T;FBID/phone;FBLC/en_US;FBSF/1.0]*/
const Device UA412975 = {S36769, 70, { &P18254,&P17350,&P17470,&P18092 }};	/*Useragent - OSSProxy 1.3.333.307 (Build 333.307 Win32 en-us)(Feb 22 2012 18:20:37)*/
const Device UA878074 = {S36770, 70, { &P18254,&P17350,&P17470,&P18092 }};	/*Useragent - OSSProxy 1.3.326.329 (Build 326.329 Win32 en-us)(Apr 15 2010 15:40:21)*/
const Device UA900507 = {S36771, 70, { &P18254,&P17350,&P17470,&P18092 }};	/*Useragent - OSSProxy 1.3.333.310 (Build 333.310 Win32 en-us)(May 24 2012 17:45:09)*/
const Device UA2043122 = {S36772, 70, { &P18254,&P17350,&P17470,&P18092 }};	/*Useragent - OSSProxy 1.3.334.331 (Build 334.331 Win32 en-us)(Aug 31 2012 15:09:13)*/
const Device UA2262882 = {S36773, 70, { &P18254,&P17350,&P17470,&P18092 }};	/*Useragent - OSSProxy 1.3.335.312 (Build 335.312 Win32 en-us)(Oct 25 2012 23:16:24)*/
const Device UA2462197 = {S36774, 70, { &P18254,&P17350,&P17470,&P18092 }};	/*Useragent - OSSProxy 1.3.335.325 (Build 335.325 Win32 en-us)(Dec 20 2012 17:03:03)*/
const Device UA2585460 = {S36775, 70, { &P18254,&P17350,&P17470,&P18092 }};	/*Useragent - OSSProxy 1.3.335.327 (Build 335.327 Win32 en-us)(Mar  6 2013 12:10:47)*/
const Device UA2601875 = {S36776, 70, { &P18254,&P17350,&P17470,&P18092 }};	/*Useragent - OSSProxy 1.3.335.329 (Build 335.329 Win32 en-us)(Mar 28 2013 11:13:41)*/
const Device UA2678753 = {S36777, 70, { &P18254,&P17350,&P17470,&P18092 }};	/*Useragent - OSSProxy 1.3.335.337 (Build 335.337 Win32 en-us)(Apr  4 2013 16:51:42)*/
const Device UA2888619 = {S36778, 70, { &P18254,&P17350,&P17470,&P18092 }};	/*Useragent - OSSProxy 1.3.336.313 (Build 336.313 Win32 en-us)(Jun 12 2013 18:51:39)*/
const Device UA2974394 = {S36779, 70, { &P18254,&P17350,&P17470,&P18092 }};	/*Useragent - OSSProxy 1.3.336.314 (Build 336.314 Win32 en-us)(Jul 11 2013 11:28:10)*/
const Device UA2992864 = {S36780, 69, { &P18254,&P17350,&P17470,&P18092 }};	/*Useragent - OSSProxy 1.3.335.337 (Build 335.337 Win32 en-us)(Apr 4 2013 16:51:42)*/
const Device UA3027132 = {S36781, 70, { &P18254,&P17350,&P17470,&P18092 }};	/*Useragent - OSSProxy 1.3.336.317 (Build 336.317 Win32 en-us)(Jul 26 2013 13:44:52)*/
const Device UA3030581 = {S36782, 70, { &P18254,&P17350,&P17470,&P18092 }};	/*Useragent - OSSProxy 1.3.336.318 (Build 336.318 Win32 en-us)(Aug  6 2013 18:31:22)*/
const Device UA3035775 = {S36783, 70, { &P18254,&P17350,&P17470,&P18092 }};	/*Useragent - OSSProxy 1.3.336.320 (Build 336.320 Win32 en-us)(Aug 16 2013 17:38:43)*/
const Device UA2674173 = {S36784, 131, { &P18454,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGPT12 Build/TJS0166) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2707885 = {S36785, 131, { &P18454,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGPT13 Build/TJS0166) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2712166 = {S36786, 60, { &P18454,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SGPT13 Build/TJS0166)*/
const Device UA2719433 = {S36787, 60, { &P18454,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SGPT12 Build/TJS0166)*/
const Device UA3017534 = {S36788, 132, { &P18454,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGPT12 Build/TJDS0170) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3019367 = {S36789, 132, { &P18454,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGPT13 Build/TJDS0170) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2679255 = {S36790, 13, { &P15364,&P17017,&P18589,&P18092 }};	/*Useragent - Java/1.7.0_21*/
const Device UA2203300 = {S36791, 57, { &P15364,&P17285,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.10*/
const Device UA2353740 = {S36792, 64, { &P15364,&P17285,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; WOW64) Presto/2.12.388 Version/12.10*/
const Device UA2357461 = {S36793, 80, { &P15364,&P17285,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.10*/
const Device UA2357693 = {S36794, 79, { &P15364,&P17285,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.10*/
const Device UA2357737 = {S36795, 80, { &P15364,&P17285,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.10*/
const Device UA2358548 = {S36796, 79, { &P15364,&P17285,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.10*/
const Device UA2360986 = {S36797, 79, { &P15364,&P17285,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.10*/
const Device UA2362602 = {S36798, 67, { &P15364,&P17285,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.10,gzip(gfe)*/
const Device UA2368244 = {S36799, 116, { &P15364,&P17285,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)) Presto/2.12.388 Version/12.10*/
const Device UA2386395 = {S36800, 79, { &P15364,&P17285,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.10*/
const Device UA2403878 = {S36801, 68, { &P15364,&P17285,&P21951,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:2.0) Gecko/X Firefox/4.0 Opera 12.10*/
const Device UA2424602 = {S36802, 80, { &P15364,&P17285,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.10*/
const Device UA2426306 = {S36803, 79, { &P15364,&P17285,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.10*/
const Device UA2456617 = {S36804, 80, { &P15364,&P17285,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.10*/
const Device UA2456801 = {S36805, 80, { &P15364,&P17285,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.10*/
const Device UA2457566 = {S36806, 62, { &P15364,&P17285,&P21951,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0) Opera 12.10*/
const Device UA2841097 = {S36807, 79, { &P15364,&P17285,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.10*/
const Device UA2841454 = {S36808, 80, { &P15364,&P17285,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.6 (build 03278)) Presto/2.12.388 Version/12.10*/
const Device UA2982871 = {S36809, 79, { &P15364,&P17285,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.10*/
const Device UA2230312 = {S36810, 151, { &P18472,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT925 Build/7.7.1Q-144_VQL_S3-49.2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2233992 = {S36811, 160, { &P18472,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR HD Build/7.7.1Q-144_VQW_S33425-2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA250308 = {S36812, 143, { &P13141,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; SHV-E110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA251317 = {S36813, 143, { &P13141,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; SGH-I727R Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA616493 = {S36814, 67, { &P13141,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SGH-I727R Build/GINGERBREAD)*/
const Device UA622339 = {S36815, 149, { &P13141,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; SGH-I727R Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30*/
const Device UA627227 = {S36816, 67, { &P13141,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHV-E110S Build/GINGERBREAD)*/
const Device UA648082 = {S36817, 143, { &P13141,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHV-E110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA750778 = {S36818, 142, { &P13141,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9210 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1906325 = {S36819, 67, { &P13141,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SHV-E110S Build/GINGERBREAD)*/
const Device UA2021491 = {S36820, 141, { &P13141,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; SGH-I727R Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028742 = {S36821, 66, { &P13141,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9210 Build/GINGERBREAD)*/
const Device UA2222659 = {S36822, 161, { &P13141,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-I9210/I9210XXLC2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2684513 = {S36823, 161, { &P13141,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; SAMSUNG GT-I9210/I9210XXLD1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3106564 = {S36824, 73, { &P13141,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHV-E110S Build/GINGERBREAD)*/
const Device UA3033106 = {S36825, 86, { &P17968,&P17979,&P24211,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/15.0.1162/30.3697; U; en) Presto/2.8.119 Version/11.10*/
const Device UA46847 = {S36826, 138, { &P15489,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ADR8995 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70838 = {S36827, 135, { &P15489,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ADR8995 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA660825 = {S36828, 59, { &P15489,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; ADR8995 Build/GRI40)*/
const Device UA856505 = {S36829, 143, { &P15489,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ADR8995 4G Build/Broken Out) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1973855 = {S36830, 140, { &P15489,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ADR8995 Build/Broken Out) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021199 = {S36831, 136, { &P15489,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-; ADR8995 4G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA7903 = {S36832, 150, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525Build/JORDN_U3_6.36.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA7911 = {S36833, 150, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.1-update1; en-us; MB525 Build/JORDN_U3_6.36.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA7965 = {S36834, 150, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JORDN_U3_6.36.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 MobileSafari/530.17*/
const Device UA7966 = {S36835, 150, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.1-update1; en-us; MB525 Build/JORDN_U3_6.36.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA7977 = {S36836, 152, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JDSKT_U3_02.18.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA7997 = {S36837, 151, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JORDN_U3_6.35.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA8012 = {S36838, 151, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JORDN_U3_6.17.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA8016 = {S36839, 150, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us;MB525 Build/JORDN_U3_6.36.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA9530 = {S36840, 152, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-kr; MB525 Build/JDSKT_U3_02.20.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10471 = {S36841, 150, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; MB525 Build/JRDNEM_U3_2.51.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10512 = {S36842, 197, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-MB525/Blur_Version.6.36.1.MB525.TIM.en.BR Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; MB525 Build/JORLA_U3_6.36.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10532 = {S36843, 150, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-; MB525 Build/JRDNEM_U3_2.51.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10484 = {S36844, 152, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JRDNEM_U3_2.34.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10515 = {S36845, 152, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-sg; MB525 Build/JRDNEM_U3_2.49.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10579 = {S36846, 152, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JRDNEM_U3_2.54.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA11637 = {S36847, 151, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JORDN_U3_6.34.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA12798 = {S36848, 152, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB525 Build/JRDNEM_U3_2.21.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA12803 = {S36849, 210, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-MB525/Blur_Version.6.14.5.MB525.AmericaMovil.en.BR Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; MB525 Build/JORLA_U3_6.14.5_R01) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA12806 = {S36850, 202, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-MB525/Blur_Version.6.14.7.MB525.Personal.en.AR Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; MB525 Build/JORLA_U3_6.14.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA13568 = {S36851, 152, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; MB525 Build/JRDNEM_U3_2.24.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA13572 = {S36852, 152, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB525 Build/JRDNEM_U3_2.22.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA18687 = {S36853, 152, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JRDNEM_U3_2.59.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA26828 = {S36854, 151, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JORDN_U3_6.36.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA26832 = {S36855, 151, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JORDN_U3_6.19.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA44277 = {S36856, 152, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; MB525 Build/JRDNEM_U3_2.26.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA52132 = {S36857, 151, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JORDN_U3_6.29.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA52147 = {S36858, 151, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JORDN_U3_6.28.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA52644 = {S36859, 210, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-MB525/Blur_Version.6.36.1.MB525.AmericaMovil.en.MX Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; MB525 Build/JORLA_U3_6.36.1_R01) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA52726 = {S36860, 152, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-za; MB525 Build/JRDNEM_U3_2.51.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA54240 = {S36861, 152, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB525 Build/JRDNEM_U3_2.51.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA62990 = {S36862, 152, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB525 Build/JRDNEM_U3_2.51.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA62991 = {S36863, 152, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB525 Build/JRDNEM_U3_2.21.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA90093 = {S36864, 151, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB525 Build/JORDN_U3_6.11.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA636036 = {S36865, 74, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; MB525 Build/JRDNEM_U3_2.59.0)*/
const Device UA2023920 = {S36866, 138, { &P13505,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; MB525 Build/JRDNEM_U3_2.51.1) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA6495 = {S36867, 133, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2; en-us; SGH-T959 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA6500 = {S36868, 133, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T959 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA6506 = {S36869, 151, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.Awesome.ButNot.1; en-us; SGH-T959 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA6508 = {S36870, 132, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ru-; SGH-T959 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA6511 = {S36871, 136, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; SGH-T959 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA6528 = {S36872, 137, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T959 Build/SKRILLER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24602 = {S36873, 137, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; SGH-T959P Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38126 = {S36874, 135, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; GT-T959 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51065 = {S36875, 160, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en_us; SGH-T959 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.2 Mobile (Dragonfruit)*/
const Device UA51074 = {S36876, 142, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2 - UVKB5; en-us; SGH-T959 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51118 = {S36877, 133, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us;SGH-T959 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51119 = {S36878, 139, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T959 Build/TOXICFROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51120 = {S36879, 133, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2; en-us; SGH-T959 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51123 = {S36880, 133, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T959Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51273 = {S36881, 136, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-T959 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA64128 = {S36882, 134, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T959 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA165609 = {S36883, 135, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-ca; SGH-T959D Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA299494 = {S36884, 58, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; SGH-T959 Build/FROYO)*/
const Device UA473848 = {S36885, 134, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T959 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA679850 = {S36886, 59, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; SGH-T959D Build/FROYO)*/
const Device UA1888260 = {S36887, 61, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SGH-T959P Build/FROYO)*/
const Device UA2019179 = {S36888, 60, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SGH-T959 Build/FROYO)*/
const Device UA2021331 = {S36889, 122, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; SGH-T959 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022670 = {S36890, 138, { &P12482,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SGH-T959 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA93317 = {S36891, 8, { &P15364,&P17017,&P17193,&P18092 }};	/*Useragent - MSIE 7.0*/
const Device UA127438 = {S36892, 34, { &P15364,&P17017,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0)*/
const Device UA140405 = {S36893, 13, { &P15364,&P17017,&P17193,&P18092 }};	/*Useragent - mAgent MSIE 7*/
const Device UA271683 = {S36894, 48, { &P15364,&P17017,&P17193,&P18092 }};	/*Useragent - Mozilla/6.0 (compatible; MSIE 7.01; Mac_PowerPC)*/
const Device UA444808 = {S36895, 36, { &P15364,&P17017,&P17193,&P18092 }};	/*Useragent - Fitel5Browser(compatible; MSIE 7.0;)*/
const Device UA791832 = {S36896, 231, { &P15364,&P17017,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0 +http://internetmemory.org/en/); crawling on behalf of The National Archives UK Government Web Archive: http://www.nationalarchives.gov.uk/webarchive/; email: webarchive@nationalarchives.gsi.gov.uk*/
const Device UA822914 = {S36897, 77, { &P15364,&P17017,&P17193,&P18092 }};	/*Useragent - CitiStreet.com/1.0+(compatible;Mozilla/5.0;MSIE 7.0;++http://citistreet.com/)*/
const Device UA2509376 = {S36898, 52, { &P15364,&P17017,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; PPC; Trident/5.0)*/
const Device UA2852000 = {S36899, 57, { &P15364,&P17017,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; PPC; SV1; Trident/1.0)*/
const Device UA90339 = {S36900, 139, { &P16053,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Vortex Build/FRF91) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA90349 = {S36901, 141, { &P16053,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Vortex Build/FRG83) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA156291 = {S36902, 132, { &P16053,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Vortex Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA167940 = {S36903, 134, { &P16053,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Vortex Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA175510 = {S36904, 135, { &P16053,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Vortex Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA178969 = {S36905, 59, { &P16053,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; Vortex Build/FRG83G)*/
const Device UA294928 = {S36906, 56, { &P16053,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; Vortex Build/FRF91)*/
const Device UA301554 = {S36907, 58, { &P16053,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; Vortex Build/FRG83)*/
const Device UA369531 = {S36908, 65, { &P16053,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.2.2; en_US; Vortex Build/FRG83G)*/
const Device UA418402 = {S36909, 134, { &P16053,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;Vortex Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA435941 = {S36910, 134, { &P16053,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Vortex Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA539744 = {S36911, 134, { &P16053,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.2; en-us; Vortex Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA543997 = {S36912, 134, { &P16053,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; VortexBuild/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA681539 = {S36913, 134, { &P16053,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.2; en-us; Vortex Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1878754 = {S36914, 142, { &P16053,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Vortex Build/FRG83G) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2018905 = {S36915, 135, { &P16053,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG Vortex Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2474175 = {S36916, 133, { &P16053,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; -us; Vortex Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA91490 = {S36917, 34, { &P15364,&P17285,&P18405,&P18092 }};	/*Useragent - Opera/9.20 (Windows NT 6.0; U; en)*/
const Device UA98014 = {S36918, 34, { &P15364,&P17285,&P18405,&P18092 }};	/*Useragent - Opera/9.25 (Windows NT 6.0; U; en)*/
const Device UA107926 = {S36919, 34, { &P15364,&P17285,&P18405,&P18092 }};	/*Useragent - Opera/9.26 (Windows NT 6.0; U; en)*/
const Device UA131194 = {S36920, 34, { &P15364,&P17285,&P18405,&P18092 }};	/*Useragent - Opera/9.24 (Windows NT 6.0; U; en)*/
const Device UA138956 = {S36921, 34, { &P15364,&P17285,&P18405,&P18092 }};	/*Useragent - Opera/9.23 (Windows NT 6.0; U; en)*/
const Device UA148561 = {S36922, 34, { &P15364,&P17285,&P18405,&P18092 }};	/*Useragent - Opera/9.21 (Windows NT 6.0; U; en)*/
const Device UA172220 = {S36923, 34, { &P15364,&P17285,&P18405,&P18092 }};	/*Useragent - Opera/9.27 (Windows NT 6.0; U; ru)*/
const Device UA512895 = {S36924, 34, { &P15364,&P17285,&P18405,&P18092 }};	/*Useragent - Opera/9.22 (Windows NT 6.0; U; ru)*/
const Device UA712993 = {S36925, 65, { &P15364,&P17285,&P18405,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.0; en) Opera 9.26*/
const Device UA2502472 = {S36926, 66, { &P15364,&P17285,&P18405,&P18092 }};	/*Useragent - ('Opera/9.20 (Windows NT 6.0; U; en)', 'Opera 9.2, Windows Vista')*/
const Device UA194004 = {S36927, 126, { &P17926,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; AT100 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA218007 = {S36928, 55, { &P17926,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; AT100 Build/HMJ37)*/
const Device UA354864 = {S36929, 71, { &P17926,&P6546,&P4459,&P18092 }};	/*Useragent - Logos/0.9.3 (Linux; U; Android 3.1; en_US; AT100 Build/HMJ37.31.5.0011)*/
const Device UA369512 = {S36930, 71, { &P17926,&P6546,&P4459,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 3.1; en_US; AT100 Build/HMJ37.01.5.0032)*/
const Device UA2975068 = {S36931, 134, { &P18635,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus 4 Build/JWR66V) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA401284 = {S36932, 118, { &P17595,&P17295,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA401319 = {S36933, 118, { &P17595,&P17295,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_5) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA401625 = {S36934, 118, { &P17595,&P17295,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA412942 = {S36935, 118, { &P17595,&P17295,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11*/
const Device UA426966 = {S36936, 118, { &P17595,&P17295,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.65 Safari/535.11*/
const Device UA427048 = {S36937, 118, { &P17595,&P17295,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_2) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.65 Safari/535.11*/
const Device UA666073 = {S36938, 118, { &P17595,&P17295,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.65 Safari/535.11*/
const Device UA865264 = {S36939, 118, { &P17595,&P17295,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.65 Safari/535.11*/
const Device UA2674856 = {S36940, 129, { &P17595,&P17295,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/17.0.648.205 Safari/534.16*/
const Device UA2736731 = {S36941, 129, { &P17595,&P17295,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/17.0.648.205 Safari/534.16*/
const Device UA2738853 = {S36942, 70, { &P15364,&P17285,&P22772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:24.0) Gecko/X Firefox/24.0*/
const Device UA2740050 = {S36943, 58, { &P15364,&P17285,&P22772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:24.0) Gecko/X Firefox/24.0*/
const Device UA3096292 = {S36944, 74, { &P15364,&P17285,&P22772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:24.0) Gecko/X Firefox/24.0 PaleMoon/24.0.1*/
const Device UA3114402 = {S36945, 86, { &P15364,&P17285,&P22772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:24.0) Gecko/X Firefox/24.0 PaleMoon/24.0.1*/
const Device UA105771 = {S36946, 134, { &P15364,&P5690,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; TmstmpExt)*/
const Device UA262743 = {S36947, 62, { &P15364,&P5690,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 6.0; Windows NT 5.1; SV1; TmstmpExt)*/
const Device UA264302 = {S36948, 193, { &P15364,&P5690,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; TmstmpExt)*/
const Device UA414060 = {S36949, 181, { &P15364,&P5690,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 ; MSIE 6.0; Windows NT 5.1; SV1; ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; TmstmpExt)*/
const Device UA505457 = {S36950, 187, { &P15364,&P5690,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 5.1; Trident/4.0; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; TmstmpExt)*/
const Device UA515053 = {S36951, 144, { &P15364,&P5690,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Zune 4.7; TmstmpExt)*/
const Device UA2210124 = {S36952, 121, { &P15364,&P5690,&P20800,&P18092 }};	/*Useragent - Outlook-Express/7.0 (MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; TmstmpExt)*/
const Device UA650310 = {S36953, 163, { &P18281,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; LG-P940 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA680725 = {S36954, 137, { &P18281,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ko-kr; LG-KU5400 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA686758 = {S36955, 168, { &P18281,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; LG-P940/V10h Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA699136 = {S36956, 169, { &P18281,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; LG-P940-Orange Build/V10h) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA711421 = {S36957, 59, { &P18281,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; LG-P940 Build/GWK74)*/
const Device UA723080 = {S36958, 136, { &P18281,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ko-kr; LG-SU540 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA769533 = {S36959, 164, { &P18281,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; LG-P940h Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA775194 = {S36960, 169, { &P18281,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; LG-P940-Orange Build/V10n) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA830312 = {S36961, 168, { &P18281,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; LG-P940/V10n Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2019414 = {S36962, 163, { &P18281,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; LG-P940 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2022992 = {S36963, 60, { &P18281,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; LG-SU540 Build/GWK74)*/
const Device UA2024727 = {S36964, 61, { &P18281,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; LG-KU5400 Build/GWK74)*/
const Device UA2032522 = {S36965, 160, { &P18281,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; sl; LG-P940 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2222005 = {S36966, 168, { &P18281,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; LG-P940/V10o Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2475853 = {S36967, 165, { &P18281,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P940h/ Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2496724 = {S36968, 128, { &P18281,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LG-P940 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2686223 = {S36969, 168, { &P18281,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; LG-P940/V10k Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2811103 = {S36970, 168, { &P18281,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; LG-P940/V10f Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2741600 = {S36971, 154, { &P18472,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753609 = {S36972, 145, { &P18472,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; DROID RAZR HD Build/9.8.1Q_39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753888 = {S36973, 151, { &P18472,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT925 Build/9.8.2Q-8-XT925_VQL-12.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774085 = {S36974, 150, { &P18472,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XT925 Build/9.8.0Q-97-XT925_VQU-19) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2884696 = {S36975, 134, { &P15364,&P5690,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100 (Edition Next)*/
const Device UA3027536 = {S36976, 119, { &P15364,&P5690,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153*/
const Device UA3028062 = {S36977, 137, { &P15364,&P5690,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Rambler)*/
const Device UA3028481 = {S36978, 126, { &P15364,&P5690,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153*/
const Device UA3028933 = {S36979, 141, { &P15364,&P5690,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Campaign 21)*/
const Device UA3029036 = {S36980, 119, { &P15364,&P5690,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153*/
const Device UA3029735 = {S36981, 129, { &P15364,&P5690,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153,gzip(gfe)*/
const Device UA3030041 = {S36982, 132, { &P15364,&P5690,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Yx)*/
const Device UA3030056 = {S36983, 136, { &P15364,&P5690,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Seznam)*/
const Device UA3030550 = {S36984, 141, { &P15364,&P5690,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Campaign 06)*/
const Device UA2676104 = {S36985, 140, { &P16292,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676247 = {S36986, 141, { &P16292,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676750 = {S36987, 133, { &P16292,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676842 = {S36988, 134, { &P16292,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2677296 = {S36989, 139, { &P16292,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Galaxy Nexus Build/JOP40C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2006108 = {S36990, 134, { &P19791,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT556 Build/V1.58R) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2007370 = {S36991, 134, { &P19791,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT557 Build/V1.65G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2156425 = {S36992, 134, { &P19791,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT556 Build/V1.70B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2389501 = {S36993, 134, { &P19791,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT556 Build/V1.58S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2429876 = {S36994, 134, { &P19791,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT557 Build/V1.65H) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2483797 = {S36995, 58, { &P19791,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; XT556 Build/V1.58R)*/
const Device UA2489158 = {S36996, 134, { &P19791,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT557 Build/V1.65K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2561597 = {S36997, 58, { &P19791,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; XT557 Build/V1.65K)*/
const Device UA2406512 = {S36998, 152, { &P18505,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422714 = {S36999, 74, { &P18505,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20)*/
const Device UA2430927 = {S37000, 151, { &P18505,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; MB886 Build/9.8.2Q-8_MB886_FFW-11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2503307 = {S37001, 150, { &P18505,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB886 Build/9.8.0Q-97_MB886_LE-7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2507330 = {S37002, 81, { &P18505,&P18111,&P18112,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android 4.1.1;motorola MB886 Build/9.8.0Q-97_MB886_FFW-20)*/
const Device UA2791170 = {S37003, 150, { &P18505,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2846474 = {S37004, 73, { &P18505,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; MB886 Build/9.8.2Q-8_MB886_FFW-11)*/
const Device UA3044107 = {S37005, 166, { &P18505,&P18111,&P18112,&P18092 }};	/*Useragent - MOT-MB886/ Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB886 Build/9.8.0Q-97_MB886_FFW-20) 1617 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3048966 = {S37006, 154, { &P18505,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; MB886 Build/9.8.2Q-50_MB886_NII_TA-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3062174 = {S37007, 168, { &P18505,&P18111,&P18112,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.1;dev=qinara;devm=MB886;man=motorola;brw=Android;shwd=720x1184;shwa=720x1134;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA16785 = {S37008, 136, { &P16553,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.1; en-us; SGH-T959V Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA16828 = {S37009, 136, { &P16553,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-T959V Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA16829 = {S37010, 136, { &P16553,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;SGH-T959V Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA16835 = {S37011, 136, { &P16553,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; SGH-T959V Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA16839 = {S37012, 136, { &P16553,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-T959VBuild/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA16847 = {S37013, 135, { &P16553,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T959V Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA16849 = {S37014, 135, { &P16553,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-; SGH-T959V Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA64810 = {S37015, 137, { &P16553,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-T959V Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74662 = {S37016, 182, { &P16553,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-T959V Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 androidAppVersion/2.0.2 AndroidWebkitWrapper*/
const Device UA174605 = {S37017, 61, { &P16553,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SGH-T959V Build/FROYO)*/
const Device UA369305 = {S37018, 74, { &P16553,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.2.1; en_US; SGH-T959V Build/FROYO.VUVKD1)*/
const Device UA521831 = {S37019, 144, { &P16553,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-T959V Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA369323 = {S37020, 74, { &P17926,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 3.2.1; en_US; AT100 Build/HTK55D.31.5.0002)*/
const Device UA578130 = {S37021, 58, { &P17926,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; AT100 Build/HTK55D)*/
const Device UA622549 = {S37022, 129, { &P17926,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; AT100 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA624095 = {S37023, 136, { &P17926,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; AT100 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA899021 = {S37024, 128, { &P17926,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; at100 Build/v_1.2) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1958155 = {S37025, 128, { &P17926,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; AT100 Build/v_1.3) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1961648 = {S37026, 128, { &P17926,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; AT100 Build/v_1.1) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2022436 = {S37027, 127, { &P17926,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-; AT100 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA910530 = {S37028, 153, { &P18505,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB886 Build/7.7.1Q-115_MB886_FFW-14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996617 = {S37029, 158, { &P18505,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; MB886 Build/7.7.1Q-115_MB886_BELL_FFW-12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028045 = {S37030, 153, { &P18505,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB886 Build/7.7.1Q-144_MB886_MR1-11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031353 = {S37031, 153, { &P18505,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB886 Build/7.7.1Q-144_MB886_MR1-22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2204018 = {S37032, 75, { &P18505,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MB886 Build/7.7.1Q-144_MB886_MR1-22)*/
const Device UA2364714 = {S37033, 158, { &P18505,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; MB886 Build/7.7.1Q-115_MB886_BELL_FFW-11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2463499 = {S37034, 152, { &P18505,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB886 Build/7.7.1Q-115_MB886_FFW-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2499730 = {S37035, 82, { &P18505,&P18066,&P18062,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android 4.0.4;motorola MB886 Build/7.7.1Q-144_MB886_MR1-22)*/
const Device UA2004621 = {S37036, 95, { &P15364,&P22186,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.3) Gecko/X Fedora/3.0.15-1.fc10 Firefox/3.0.15*/
const Device UA43623 = {S37037, 109, { &P12137,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-GT-S5620B/1.0 SHP/VPP/R5 Dolfin/1.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA53457 = {S37038, 109, { &P12137,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-GT-S5620L/1.0 SHP/VPP/R5 Dolfin/1.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA55065 = {S37039, 108, { &P12137,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-GT-S5620/1.0 SHP/VPP/R5 Dolfin/1.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA2798055 = {S37040, 105, { &P18657,&P17484,&P18369,&P18092 }};	/*Useragent - SAMSUNG-GT-C3262 Opera/9.80 (J2ME/MIDP; Opera Mini/7.0.30281/30.3214; U; en) Presto/2.8.119 Version/11.10*/
const Device UA924173 = {S37041, 131, { &P18668,&P7106,&P15829,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9310; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.477 Mobile Safari/534.11+*/
const Device UA599835 = {S37042, 98, { &P15364,&P7133,&P18333,&P18092 }};	/*Useragent - iTunes/10.6 (Windows; Microsoft Windows 7 Home Premium Edition (Build 7600)) AppleWebKit/534.54.16*/
const Device UA901230 = {S37043, 118, { &P15364,&P7133,&P18333,&P18092 }};	/*Useragent - iTunes/10.6.3 (Windows; Microsoft Windows 7 x64 Home Premium Edition Service Pack 1 (Build 7601)) AppleWebKit/534.57.2*/
const Device UA1951216 = {S37044, 94, { &P15364,&P7133,&P18333,&P18092 }};	/*Useragent - iTunes/10.6.3 (Windows; Microsoft Windows 7 Starter Edition (Build 7600)) AppleWebKit/534.57.2*/
const Device UA2040394 = {S37045, 112, { &P15364,&P7133,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows 7 x64 Business Edition Service Pack 1 (Build 7601)) AppleWebKit/536.26.9*/
const Device UA2041301 = {S37046, 108, { &P15364,&P7133,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows 7 Business Edition Service Pack 1 (Build 7601)) AppleWebKit/536.26.9*/
const Device UA2041430 = {S37047, 112, { &P15364,&P7133,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows 7 x64 Ultimate Edition Service Pack 1 (Build 7601)) AppleWebKit/536.26.9*/
const Device UA2042107 = {S37048, 112, { &P15364,&P7133,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows 7 Home Premium Edition Service Pack 1 (Build 7601)) AppleWebKit/536.26.9*/
const Device UA2043648 = {S37049, 108, { &P15364,&P7133,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows 7 Ultimate Edition Service Pack 1 (Build 7601)) AppleWebKit/536.26.9*/
const Device UA2203601 = {S37050, 101, { &P15364,&P7133,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows 7 x64 Home Premium Edition (Build 7600)) AppleWebKit/536.26.9*/
const Device UA2204029 = {S37051, 97, { &P15364,&P7133,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows 7 x64 Ultimate Edition (Build 7600)) AppleWebKit/536.26.9*/
const Device UA2204031 = {S37052, 107, { &P15364,&P7133,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows 7 Starter Edition Service Pack 1 (Build 7601)) AppleWebKit/536.26.9*/
const Device UA2204145 = {S37053, 97, { &P15364,&P7133,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows 7 x64 Business Edition (Build 7600)) AppleWebKit/536.26.9*/
const Device UA2204146 = {S37054, 110, { &P15364,&P7133,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows 7 Enterprise Edition Service Pack 1 (Build 7601)) AppleWebKit/536.26.9*/
const Device UA2204300 = {S37055, 114, { &P15364,&P7133,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows 7 x64 Enterprise Edition Service Pack 1 (Build 7601)) AppleWebKit/536.26.9*/
const Device UA2204812 = {S37056, 93, { &P15364,&P7133,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows 7 Business Edition (Build 7600)) AppleWebKit/536.26.9*/
const Device UA2205401 = {S37057, 93, { &P15364,&P7133,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows 7 Ultimate Edition (Build 7600)) AppleWebKit/536.26.9*/
const Device UA2242325 = {S37058, 99, { &P15364,&P7133,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows 7 x64 Enterprise Edition (Build 7600)) AppleWebKit/536.26.9*/
const Device UA2242361 = {S37059, 114, { &P15364,&P7133,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows 7 x64 Home Basic Edition Service Pack 1 (Build 7601)) AppleWebKit/536.26.9*/
const Device UA2346946 = {S37060, 95, { &P15364,&P7133,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows 7 x64 Service Pack 1 (Build 7601)) AppleWebKit/536.26.9*/
const Device UA2347195 = {S37061, 91, { &P15364,&P7133,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows 7 Service Pack 1 (Build 7601)) AppleWebKit/536.26.9*/
const Device UA2383843 = {S37062, 95, { &P15364,&P7133,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows 7 Enterprise Edition (Build 7600)) AppleWebKit/536.26.9*/
const Device UA423471 = {S37063, 128, { &P12765,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA616696 = {S37064, 57, { &P12765,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; A100 Build/IML74K)*/
const Device UA2401047 = {S37065, 128, { &P12765,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; A100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2414444 = {S37066, 150, { &P12765,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ro-ro; IMPRESSPE3D Essential A100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2574348 = {S37067, 135, { &P12765,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2686479 = {S37068, 126, { &P12765,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-; A100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2839469 = {S37069, 57, { &P12765,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; A100 Build/IMM76D)*/
const Device UA2856889 = {S37070, 149, { &P12765,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; A-BELL A100 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2203611 = {S37071, 112, { &P15803,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia200/11.56; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2388146 = {S37072, 111, { &P15803,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia200/10.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5*/
const Device UA724918 = {S37073, 150, { &P18476,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I577 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA892436 = {S37074, 148, { &P18476,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SAMSUNG-SGH-I577 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032153 = {S37075, 74, { &P18476,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SAMSUNG-SGH-I577 Build/GINGERBREAD)*/
const Device UA3125925 = {S37076, 178, { &P18476,&P5629,&P15760,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 2.3.6;dev=SGH-I577;devm=SAMSUNG-SGH-I577;man=samsung;brw=Android;shwd=480x800;shwa=480x763;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA2669826 = {S37077, 164, { &P22582,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2750373 = {S37078, 145, { &P22582,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2869700 = {S37079, 172, { &P22582,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2879525 = {S37080, 156, { &P22582,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SGH-I337 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3050001 = {S37081, 164, { &P22582,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SGH-I337 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA64675 = {S37082, 177, { &P14188,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5233/40.1.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.7.4 3gpp-gba*/
const Device UA68394 = {S37083, 34, { &P14188,&P5129,&P3690,&P18092 }};	/*Useragent - Nokia5233/UCBrowser7.7.0.81/50/800*/
const Device UA69525 = {S37084, 9, { &P14188,&P5129,&P3690,&P18092 }};	/*Useragent - Nokia5233*/
const Device UA346964 = {S37085, 191, { &P14188,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5233/51.1.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 Mobile Safari/533.4 3gpp-gba*/
const Device UA2493771 = {S37086, 10, { &P14188,&P5129,&P3690,&P18092 }};	/*Useragent - Nokia 5233*/
const Device UA224647 = {S37087, 136, { &P17158,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; VS910 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA601769 = {S37088, 61, { &P17158,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; VS910 4G Build/GRK39F)*/
const Device UA625035 = {S37089, 137, { &P17158,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; VS910 4G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA644580 = {S37090, 60, { &P17158,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; VS910 4G Build/GRJ22)*/
const Device UA2385799 = {S37091, 142, { &P17158,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; VS910 4G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA93343 = {S37092, 164, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Maxthon 2.0)*/
const Device UA93907 = {S37093, 164, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAXTHON 2.0)*/
const Device UA106785 = {S37094, 222, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Maxthon 2.0)*/
const Device UA117422 = {S37095, 179, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; Maxthon 2.0)*/
const Device UA138324 = {S37096, 191, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MAXTHON 2.0)*/
const Device UA141352 = {S37097, 181, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Maxthon 2.0)*/
const Device UA154565 = {S37098, 174, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Maxthon 2.0)*/
const Device UA176058 = {S37099, 153, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Maxthon 2.0)*/
const Device UA261044 = {S37100, 191, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Tablet PC 2.0; MAXTHON 2.0)*/
const Device UA270570 = {S37101, 179, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; MAXTHON 2.0)*/
const Device UA382398 = {S37102, 171, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Maxthon 2.0)*/
const Device UA392954 = {S37103, 83, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; MAXTHON 2.0)*/
const Device UA414692 = {S37104, 158, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Tablet PC 2.0; Maxthon 2.0)*/
const Device UA445789 = {S37105, 237, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; Maxthon 2.0)*/
const Device UA461330 = {S37106, 191, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Tablet PC 2.0; Maxthon 2.0)*/
const Device UA511543 = {S37107, 193, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; MAXTHON 2.0)*/
const Device UA538124 = {S37108, 160, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Maxthon 2.0)*/
const Device UA552008 = {S37109, 184, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAXTHON 2.0)*/
const Device UA559163 = {S37110, 164, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; MAXTHON 2.0)*/
const Device UA692877 = {S37111, 177, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; WOW64; Trident/5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0; Maxthon 2.0)*/
const Device UA713835 = {S37112, 153, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Maxthon 2.0)*/
const Device UA728159 = {S37113, 165, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2; .NET4.0C; MAXTHON 2.0)*/
const Device UA758502 = {S37114, 239, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Zune 4.0; InfoPath.3; MS-RTC LM 8; .NET4.0C; .NET4.0E; Maxthon 2.0)*/
const Device UA779848 = {S37115, 76, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; MAXTHON 2.0)*/
const Device UA855298 = {S37116, 150, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Maxthon 2.0)*/
const Device UA2126627 = {S37117, 170, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; MAXTHON 2.0)*/
const Device UA2209431 = {S37118, 87, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; SV1; .NET CLR 1.1.4322; MAXTHON 2.0)*/
const Device UA2265344 = {S37119, 239, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Maxthon 2.0)*/
const Device UA2303944 = {S37120, 222, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 3.0.04506; Media Center PC 5.0; SLCC1; .NET4.0C; Maxthon 2.0)*/
const Device UA2989210 = {S37121, 84, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; MAXTHON 2.0)*/
const Device UA3087580 = {S37122, 77, { &P15364,&P7133,&P22372,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; MAXTHON 2.0)*/
const Device UA348168 = {S37123, 104, { &P15364,&P7133,&P22374,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.3 (KHTML, like Gecko) Maxthon/3.3.1.600 Safari/535.3*/
const Device UA348863 = {S37124, 97, { &P15364,&P7133,&P22374,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.3 (KHTML, like Gecko) Maxthon/3.3.1.600 Safari/535.3*/
const Device UA366259 = {S37125, 123, { &P15364,&P7133,&P22374,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.3 (KHTML, like Gecko) Maxthon/3.3.3.1000 Chrome/16.0.883.0 Safari/535.3*/
const Device UA366327 = {S37126, 116, { &P15364,&P7133,&P22374,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.3 (KHTML, like Gecko) Maxthon/3.3.3.1000 Chrome/16.0.883.0 Safari/535.3*/
const Device UA900244 = {S37127, 125, { &P15364,&P7133,&P22374,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.3.9.1000 Chrome/18.0.966.0 Safari/535.12*/
const Device UA900759 = {S37128, 118, { &P15364,&P7133,&P22374,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.3.9.1000 Chrome/18.0.966.0 Safari/535.12*/
const Device UA2597175 = {S37129, 117, { &P15364,&P7133,&P22374,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.12 (KHTML like Gecko) Maxthon/3.3.4.4000 Chrome/18.0.966.0 Safari/535.12*/
const Device UA2489705 = {S37130, 112, { &P18643,&P5043,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia311/03.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA29926 = {S37131, 116, { &P12280,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18*/
const Device UA29927 = {S37132, 104, { &P12280,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko)*/
const Device UA29934 = {S37133, 116, { &P12280,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7D11*/
const Device UA29937 = {S37134, 104, { &P12280,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko)*/
const Device UA29941 = {S37135, 115, { &P12280,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7C144*/
const Device UA29944 = {S37136, 102, { &P12280,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko)*/
const Device UA29981 = {S37137, 130, { &P12280,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7D11 Safari/528.16*/
const Device UA89170 = {S37138, 106, { &P12280,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko)*/
const Device UA285019 = {S37139, 130, { &P12280,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Safari/528.18*/
const Device UA505035 = {S37140, 130, { &P12280,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7D11 Safari/528.18*/
const Device UA545180 = {S37141, 129, { &P12280,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7C144 Safari/528.18*/
const Device UA681494 = {S37142, 114, { &P12280,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko)*/
const Device UA696704 = {S37143, 126, { &P12280,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18*/
const Device UA808273 = {S37144, 126, { &P12280,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; es-es) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18*/
const Device UA915840 = {S37145, 130, { &P12280,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Safari/528.16*/
const Device UA1837163 = {S37146, 116, { &P12280,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (device; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18*/
const Device UA2392318 = {S37147, 123, { &P12280,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (whited00r iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18*/
const Device UA2890744 = {S37148, 116, { &P12280,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3 1 2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7D11*/
const Device UA658467 = {S37149, 147, { &P16263,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HUAWEI MediaPad Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2031894 = {S37150, 143, { &P16263,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SpringBoard Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033907 = {S37151, 76, { &P16263,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HUAWEI MediaPad Build/HuaweiMediaPad)*/
const Device UA2257677 = {S37152, 147, { &P16263,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MediaPad 7 Lite Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2503268 = {S37153, 76, { &P16263,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; MediaPad 7 Lite Build/HuaweiMediaPad)*/
const Device UA2684877 = {S37154, 136, { &P16263,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; GT01 Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2686151 = {S37155, 155, { &P16263,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HUAWEI OsiMood MediaPad Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2729438 = {S37156, 148, { &P16263,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MediaPad 10 LINK Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2788154 = {S37157, 72, { &P16263,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SpringBoard Build/HuaweiMediaPad)*/
const Device UA3495 = {S37158, 165, { &P11905,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia6120c/4.21 (SymbianOS/9.2; U; Series60/3.1 Nokia6120c/4.21; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) Mozilla/5.0 AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA63348 = {S37159, 150, { &P11905,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Nokia6120ci/7.02; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA64433 = {S37160, 167, { &P11905,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia6120ci/7.10 (SymbianOS/9.2; U; Series60/3.1 Nokia6120ci/7.10; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) Mozilla/5.0 AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA64842 = {S37161, 149, { &P11905,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Nokia6120c/7.10; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA67070 = {S37162, 44, { &P11905,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia6120ciB11.04/SymbianOS/9.1 Series60/3.0*/
const Device UA67220 = {S37163, 56, { &P11905,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia6120c/3.83; Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA69480 = {S37164, 45, { &P11905,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia6120c(B20.03)/SymbianOS/9.1 Series60/3.0*/
const Device UA73289 = {S37165, 119, { &P11905,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Nokia6120guration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA490413 = {S37166, 11, { &P11905,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia6120ci*/
const Device UA739919 = {S37167, 122, { &P15841,&P24181,&P24417,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; RIM Tablet OS 2.1.0; en-US) AppleWebKit/536.2+ (KHTML, like Gecko) Version/7.2.1.0 Safari/536.2+*/
const Device UA349364 = {S37168, 129, { &P15745,&P5156,&P3519,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9670; en-US) AppleWebKit/534.8+ (KHTML, like Gecko) Version/6.0.0.706 Mobile Safari/534.8+*/
const Device UA624958 = {S37169, 121, { &P15241,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia300/5.0 (07.03) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA913462 = {S37170, 76, { &P17595,&P17297,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:14.0) Gecko/X Firefox/14.0.1*/
const Device UA1996320 = {S37171, 91, { &P17595,&P17297,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:14.0) Gecko/X Firefox/14.0.1 Firefox/14.0.1*/
const Device UA826482 = {S37172, 74, { &P15364,&P17285,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:10.0.1) Gecko/X Firefox/10.0.1*/
const Device UA836189 = {S37173, 77, { &P15364,&P17285,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:10.0.2) Gecko/X Firefox/10.0.2 Firefox/10.0.2*/
const Device UA2260035 = {S37174, 81, { &P15364,&P17285,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:10.0.2) Gecko/X Firefox/10.0.2*/
const Device UA2462064 = {S37175, 64, { &P15364,&P17285,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:10.0.12) Gecko/X Firefox/10.0.12*/
const Device UA2779672 = {S37176, 62, { &P15364,&P17285,&P18099,&P18092 }};	/*Useragent - Mozilla/6.0 (Windows NT 6.0; rv:10.0.3) Gecko/X Firefox/10.0.2*/
const Device UA2361587 = {S37177, 57, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.11*/
const Device UA2367473 = {S37178, 64, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; WOW64) Presto/2.12.388 Version/12.11*/
const Device UA2387884 = {S37179, 79, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.11*/
const Device UA2388519 = {S37180, 80, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03796)) Presto/2.12.388 Version/12.11*/
const Device UA2389465 = {S37181, 80, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.11*/
const Device UA2390401 = {S37182, 79, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.11*/
const Device UA2391457 = {S37183, 67, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.11,gzip(gfe)*/
const Device UA2404022 = {S37184, 62, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0) Opera 12.11*/
const Device UA2404599 = {S37185, 79, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.11*/
const Device UA2404751 = {S37186, 116, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)) Presto/2.12.388 Version/12.11*/
const Device UA2405403 = {S37187, 80, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.11*/
const Device UA2405633 = {S37188, 79, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.11*/
const Device UA2405796 = {S37189, 79, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.11*/
const Device UA2422222 = {S37190, 109, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)) Presto/2.12.388 Version/12.11*/
const Device UA2426608 = {S37191, 95, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; compatible; MSIE 7.0; Windows NT 6.0) Presto/2.12.388 Version/12.11*/
const Device UA2426859 = {S37192, 79, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5754)) Presto/2.12.388 Version/12.11*/
const Device UA2427013 = {S37193, 68, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:2.0) Gecko/X Firefox/4.0 Opera 12.11*/
const Device UA2427463 = {S37194, 79, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.11*/
const Device UA2428327 = {S37195, 80, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.11*/
const Device UA2428485 = {S37196, 102, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; compatible; MSIE 7.0; Windows NT 6.0) Presto/2.12.388 Version/12.11*/
const Device UA2457164 = {S37197, 79, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5970)) Presto/2.12.388 Version/12.11*/
const Device UA2457371 = {S37198, 106, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94) Presto/2.12.388 Version/12.11*/
const Device UA2458063 = {S37199, 80, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5339)) Presto/2.12.388 Version/12.11*/
const Device UA2504949 = {S37200, 79, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4157)) Presto/2.12.388 Version/12.11*/
const Device UA2528049 = {S37201, 80, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.4 (build 02647)) Presto/2.12.388 Version/12.11*/
const Device UA2529211 = {S37202, 79, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.11*/
const Device UA2656018 = {S37203, 79, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 8.0 (build 5880)) Presto/2.12.388 Version/12.11*/
const Device UA2790471 = {S37204, 79, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4664)) Presto/2.12.388 Version/12.11*/
const Device UA2795176 = {S37205, 79, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6068)) Presto/2.12.388 Version/12.11*/
const Device UA2843033 = {S37206, 80, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03686)) Presto/2.12.388 Version/12.11*/
const Device UA2843317 = {S37207, 79, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.8 (build 4598)) Presto/2.12.388 Version/12.11*/
const Device UA2843366 = {S37208, 79, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5680)) Presto/2.12.388 Version/12.11*/
const Device UA2843480 = {S37209, 79, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4876)) Presto/2.12.388 Version/12.11*/
const Device UA3048579 = {S37210, 79, { &P15364,&P17285,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6083)) Presto/2.12.388 Version/12.11*/
const Device UA24745 = {S37211, 133, { &P13685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LS670 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA69100 = {S37212, 142, { &P13685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LS670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA69502 = {S37213, 132, { &P13685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-LS670 Build/0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA180114 = {S37214, 139, { &P13685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LS670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA248619 = {S37215, 57, { &P13685,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; LS670 Build/GRI40)*/
const Device UA349410 = {S37216, 134, { &P13685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-LS670 Build/106) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA353621 = {S37217, 134, { &P13685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-LS670 Build/107) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370211 = {S37218, 134, { &P13685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-LS670 Build/109) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA564662 = {S37219, 141, { &P13685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-LS670 Build/STABLE-001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA576537 = {S37220, 140, { &P13685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LS670 Build/GRI40) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA704654 = {S37221, 131, { &P13685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; LS670 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019299 = {S37222, 136, { &P13685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-LS670 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019553 = {S37223, 142, { &P13685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-LS670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1982211 = {S37224, 135, { &P19907,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C5155 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2242221 = {S37225, 57, { &P19907,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; C5155 Build/IML77)*/
const Device UA2348179 = {S37226, 135, { &P19907,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; Rise Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2694131 = {S37227, 57, { &P19907,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Rise Build/IMM76I)*/
const Device UA2437262 = {S37228, 165, { &P18645,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SAMSUNG-SGH-I747 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2437282 = {S37229, 157, { &P18645,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-L710 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2468948 = {S37230, 158, { &P18645,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-R530U Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2512077 = {S37231, 165, { &P18645,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SAMSUNG-SGH-I747 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2513567 = {S37232, 157, { &P18645,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-L710 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2536598 = {S37233, 156, { &P18645,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L710 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2536612 = {S37234, 164, { &P18645,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I747 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2559171 = {S37235, 158, { &P18645,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-R530U Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2565784 = {S37236, 157, { &P18645,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530U Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2600691 = {S37237, 156, { &P18645,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; GT-I9305 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666490 = {S37238, 157, { &P18645,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530C Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708062 = {S37239, 157, { &P18645,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; GT-I9305 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2716496 = {S37240, 167, { &P18645,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I747 Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2733573 = {S37241, 155, { &P18645,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L710 Build/MROM; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2766673 = {S37242, 163, { &P18645,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I747 Build/MROM; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2947399 = {S37243, 164, { &P18645,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L710 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/d2spr*/
const Device UA3017134 = {S37244, 165, { &P18645,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530U Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/d2usc*/
const Device UA3017512 = {S37245, 165, { &P18645,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530C Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/d2cri*/
const Device UA3056428 = {S37246, 160, { &P18645,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; GT-I9305 Build/4.80) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/4.80/i9305*/
const Device UA2205676 = {S37247, 80, { &P18318,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ASUS Transformer Pad TF700T Build/JRO03C)*/
const Device UA2205843 = {S37248, 151, { &P18318,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ASUS Transformer Pad TF700T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2504042 = {S37249, 139, { &P18318,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; ASUS Pad TF700T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2556108 = {S37250, 152, { &P18318,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; da-dk; ASUS Transformer Pad TF700KL Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2577867 = {S37251, 158, { &P18318,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ASUS Transformer Pad TF700T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2940993 = {S37252, 136, { &P18635,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JWR66N) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2970406 = {S37253, 136, { &P18635,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3023357 = {S37254, 136, { &P18635,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3031259 = {S37255, 136, { &P18635,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JWR66X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3033988 = {S37256, 136, { &P18635,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3034075 = {S37257, 58, { &P18635,&P21637,&P23997,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.3; Nexus 4 Build/JWR66Y)*/
const Device UA3061072 = {S37258, 134, { &P18635,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-; Nexus 4 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3064719 = {S37259, 136, { &P18635,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3114340 = {S37260, 136, { &P18635,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3115765 = {S37261, 136, { &P18635,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA590294 = {S37262, 93, { &P17595,&P17297,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:10.0.2) Gecko/X Firefox/10.0.2 Firefox/10.0.2*/
const Device UA613227 = {S37263, 78, { &P17595,&P17297,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:10.0.4) Gecko/X Firefox/10.0.4*/
const Device UA656346 = {S37264, 80, { &P17595,&P17297,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2; rv:10.0.1) Gecko/X Firefox/10.0.1*/
const Device UA4320 = {S37265, 147, { &P12110,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Legend 2.05.161.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA4324 = {S37266, 147, { &P12110,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; HTC Legend 1.32.163.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA4332 = {S37267, 134, { &P12110,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ko-; HTC Legend Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24851 = {S37268, 136, { &P12110,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-au; HTC Legend Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24865 = {S37269, 147, { &P12110,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Legend 1.32.161.4 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24904 = {S37270, 147, { &P12110,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; de-at; HTC Legend 3.14.177.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24977 = {S37271, 147, { &P12110,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Legend 1.23.161.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA25341 = {S37272, 147, { &P12110,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Legend 3.14.161.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA45162 = {S37273, 147, { &P12110,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; HTC Legend 2.05.163.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA54800 = {S37274, 147, { &P12110,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; HTC Legend 3.14.163.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA163687 = {S37275, 60, { &P12110,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; HTC Legend Build/FRF91)*/
const Device UA784331 = {S37276, 124, { &P12110,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Legend Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1958158 = {S37277, 147, { &P12110,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Legend 3.14.161.2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018098 = {S37278, 132, { &P12110,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; Legend Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018104 = {S37279, 134, { &P12110,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; Legend Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027506 = {S37280, 129, { &P12110,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Legend Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA163141 = {S37281, 128, { &P17595,&P17296,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.231 Safari/534.10*/
const Device UA163149 = {S37282, 128, { &P17595,&P17296,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.231 Safari/534.10*/
const Device UA164661 = {S37283, 128, { &P17595,&P17296,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.231 Safari/534.10*/
const Device UA176885 = {S37284, 128, { &P17595,&P17296,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.231 Safari/534.10*/
const Device UA187068 = {S37285, 147, { &P17595,&P17296,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/525.13 (KHTML, like Gecko; Google Page Speed) Chrome/8.0.552.237 Safari/525.13*/
const Device UA199360 = {S37286, 128, { &P17595,&P17296,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.231 Safari/534.10*/
const Device UA199862 = {S37287, 223, { &P17595,&P17296,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-us) AppleWebKit/533.17.8 (KHTML, like Gecko) Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en) AppleWebKit/533.17.8 (KHTML, like Gecko) Chrome/8.1.0.0 Safari/533.17.8*/
const Device UA443951 = {S37288, 128, { &P17595,&P17296,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.210 Safari/534.10*/
const Device UA684159 = {S37289, 128, { &P17595,&P17296,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10*/
const Device UA763156 = {S37290, 128, { &P17595,&P17296,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.215 Safari/534.10*/
const Device UA882303 = {S37291, 128, { &P17595,&P17296,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.231 Safari/534.10*/
const Device UA2673263 = {S37292, 144, { &P17595,&P17296,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1064; en-CA) AppleWebKit/525.13 (KHTML like Gecko; Google Page Speed) Chrome/8.0.552.237 Safari/525.13*/
const Device UA2673773 = {S37293, 144, { &P17595,&P17296,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1064; es-US) AppleWebKit/525.13 (KHTML like Gecko; Google Page Speed) Chrome/8.0.552.237 Safari/525.13*/
const Device UA2674436 = {S37294, 144, { &P17595,&P17296,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1064; en-US) AppleWebKit/525.13 (KHTML like Gecko; Google Page Speed) Chrome/8.0.552.237 Safari/525.13*/
const Device UA2678789 = {S37295, 145, { &P17595,&P17296,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1064; haw-US) AppleWebKit/525.13 (KHTML like Gecko; Google Page Speed) Chrome/8.0.552.237 Safari/525.13*/
const Device UA2680938 = {S37296, 144, { &P17595,&P17296,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1064; fr-CA) AppleWebKit/525.13 (KHTML like Gecko; Google Page Speed) Chrome/8.0.552.237 Safari/525.13*/
const Device UA2735271 = {S37297, 144, { &P17595,&P17296,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1064; zh-CN) AppleWebKit/525.13 (KHTML like Gecko; Google Page Speed) Chrome/8.0.552.237 Safari/525.13*/
const Device UA2860198 = {S37298, 144, { &P17595,&P17296,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 1064; fr-FR) AppleWebKit/525.13 (KHTML like Gecko; Google Page Speed) Chrome/8.0.552.237 Safari/525.13*/
const Device UA2040997 = {S37299, 67, { &P15364,&P18110,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux x86_64; U; ru) Presto/2.10.289 Version/12.02*/
const Device UA2383390 = {S37300, 73, { &P15364,&P18110,&P19961,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; en; rv:2.0) Gecko/X Firefox/4.0 Opera 12.02*/
const Device UA2819669 = {S37301, 67, { &P15364,&P18110,&P19961,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; X11; Linux i686; en) Opera 12.02*/
const Device UA23039 = {S37302, 80, { &P15364,&P5690,&P20918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; rv:1.7.3) Gecko/20041001 Firefox/0.10.1*/
const Device UA442985 = {S37303, 77, { &P15364,&P5690,&P20918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT5.1; en-US; rv:1.7) Gecko/X Firefox/0.8.0+*/
const Device UA650332 = {S37304, 68, { &P15364,&P5690,&P20918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; ?; Windows NT 5.1; *rv:*) Gecko/* Firefox/0.9**/
const Device UA680809 = {S37305, 92, { &P15364,&P5690,&P20918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.6) Gecko/X Firefox/0.8 Mnenhy/0.6.0.103*/
const Device UA23352 = {S37306, 161, { &P14916,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C3303/C3303DDJG4 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1,SAMSUNG-GT-C3303/C3303DDJG4 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24647 = {S37307, 74, { &P14916,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C3303i/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA55097 = {S37308, 80, { &P14916,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C3303/C3303DDJG4 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61337 = {S37309, 165, { &P14916,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C3303i/C3303iDDKB2 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1,SAMSUNG-GT-C3303i/C3303iDDKB2 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64236 = {S37310, 74, { &P14916,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C3303K/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA697591 = {S37311, 143, { &P18581,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA896418 = {S37312, 67, { &P18581,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-S720C Build/GINGERBREAD)*/
const Device UA2030661 = {S37313, 142, { &P18581,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us;SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2375127 = {S37314, 142, { &P18581,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-S720CBuild/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2446746 = {S37315, 142, { &P18581,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.6; en-us; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2545494 = {S37316, 142, { &P18581,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA2546096 = {S37317, 142, { &P18581,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2816417 = {S37318, 141, { &P18581,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2999983 = {S37319, 143, { &P18581,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3117759 = {S37320, 35, { &P18581,&P5629,&P15760,&P18092 }};	/*Useragent - Android/1.0 (SCH-S720C GINGERBREAD)*/
const Device UA3039307 = {S37321, 65, { &P15364,&P5690,&P22772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64; rv:24.0) Gecko/X Firefox/24.0*/
const Device UA3039308 = {S37322, 58, { &P15364,&P5690,&P22772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:24.0) Gecko/X Firefox/24.0*/
const Device UA3096348 = {S37323, 74, { &P15364,&P5690,&P22772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:24.0) Gecko/X Firefox/24.0 PaleMoon/24.0.1*/
const Device UA1950209 = {S37324, 139, { &P12681,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; Nexus S 4G Build/JRN84D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996756 = {S37325, 141, { &P12681,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S 4G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2010072 = {S37326, 141, { &P12681,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S 4G Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030213 = {S37327, 155, { &P12681,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Full Android on Crespo4G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2040604 = {S37328, 141, { &P12681,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S 4G Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2202078 = {S37329, 63, { &P12681,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Nexus S 4G Build/JRO03R)*/
const Device UA2225215 = {S37330, 164, { &P12681,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Full Android on Crespo4G Build/PHONELAB_JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2237048 = {S37331, 141, { &P12681,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus S 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2236371 = {S37332, 73, { &P15364,&P17285,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:19.0) Gecko/19.0 Firefox/19.0*/
const Device UA2487975 = {S37333, 81, { &P15364,&P17285,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:19.0) Gecko/X Firefox/19.0 AlexaToolbar/alxf-2.17*/
const Device UA2774280 = {S37334, 82, { &P15364,&P17285,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:19.0) Gecko/X Firefox/19.0 GTB6*/
const Device UA48691 = {S37335, 148, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; Orange Monte Carlo Build/OUK_B05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48693 = {S37336, 148, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; Orange Monte Carlo Build/OUK_B03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48698 = {S37337, 148, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-at; Orange Monte Carlo Build/OAT_B05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73277 = {S37338, 137, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; ZTE-Skate Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76337 = {S37339, 140, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; BASE Lutea 2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA77924 = {S37340, 148, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sk-sk; Orange Monte Carlo Build/OSK_B04) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA359942 = {S37341, 148, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Orange Monte Carlo Build/OUK_B05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369189 = {S37342, 148, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-at; Orange Monte Carlo Build/OAT_B07) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA609175 = {S37343, 148, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Orange Monte Carlo Build/OUK_B08) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA629942 = {S37344, 141, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; ZTE Skate Build/V1.0.0B03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA636963 = {S37345, 148, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-at; Orange Monte Carlo Build/OAT_B06) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA644344 = {S37346, 148, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; Orange Monte Carlo Build/OSP_B02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA646489 = {S37347, 148, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; Orange Monte Carlo Build/OSP_B03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA656000 = {S37348, 137, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; ZTE-SKATE Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA696122 = {S37349, 139, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; ZTE Skate Build/OUK_B05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA705550 = {S37350, 137, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; ZTE-SKATE Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA707538 = {S37351, 148, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; sk-sk; Orange Monte Carlo Build/OSK_B05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA808616 = {S37352, 141, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; ZTE Skate Build/V1.0.0B05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA887349 = {S37353, 137, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; ZTE Skate Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1903859 = {S37354, 72, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; Orange Monte Carlo Build/OUK_B05)*/
const Device UA1967565 = {S37355, 130, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; Skate Build/R0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020687 = {S37356, 61, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; ZTE-SKATE Build/GRJ22)*/
const Device UA2021335 = {S37357, 57, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; Skate Build/GRJ22)*/
const Device UA2022474 = {S37358, 137, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; ZTE-SKATE Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025675 = {S37359, 148, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-at; Orange Monte Carlo Build/OAT_B08) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034292 = {S37360, 148, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; sk-sk; Orange Monte Carlo Build/OSK_B06) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2324166 = {S37361, 72, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Orange Monte Carlo Build/OSP_B03)*/
const Device UA2397573 = {S37362, 148, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Orange Monte Carlo Build/OUK_B07) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2474087 = {S37363, 140, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; BASE Lutea 2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2482226 = {S37364, 139, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; ZTE Skate Build/OUK_B08) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2494753 = {S37365, 137, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; ZTE-Skate Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2520807 = {S37366, 138, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-mx; ZTE-U V960 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2551629 = {S37367, 155, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; Skate Build/GRJ22; The Dark ICS By Faia) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2582975 = {S37368, 148, { &P12784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; Orange Monte Carlo Build/OSP_B04) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2672602 = {S37369, 134, { &P18635,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Nexus 4 Build/JOP40C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676011 = {S37370, 135, { &P18635,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676293 = {S37371, 128, { &P18635,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676450 = {S37372, 136, { &P18635,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 4 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708422 = {S37373, 129, { &P18635,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 4 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2865708 = {S37374, 127, { &P18635,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Nexus 4 Build/JOP40C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA33251 = {S37375, 144, { &P15995,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Sprint APA7373KT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370998 = {S37376, 144, { &P15995,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Sprint APA7373KT Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2444510 = {S37377, 144, { &P15995,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Sprint APA7373KT Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA197739 = {S37378, 93, { &P13868,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; HD7)*/
const Device UA210654 = {S37379, 95, { &P13868,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; T9296)*/
const Device UA223423 = {S37380, 99, { &P13868,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; HD7 T9292)*/
const Device UA640235 = {S37381, 99, { &P13868,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.2; Trident/5.0; IEMobile/9.0; HTC; HD7 T9292)*/
const Device UA805702 = {S37382, 94, { &P13868,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Gold)*/
const Device UA925699 = {S37383, 102, { &P13868,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Touch-IT HD7)*/
const Device UA925744 = {S37384, 106, { &P13868,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Mersad HD7 T9292)*/
const Device UA925816 = {S37385, 102, { &P13868,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; HD7 Infinity)*/
const Device UA925864 = {S37386, 99, { &P13868,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; GOLD-7PRO)*/
const Device UA2857970 = {S37387, 114, { &P13868,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; WpsLondonTest; HTC; HD7 T9292)*/
const Device UA901557 = {S37388, 78, { &P17595,&P17296,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:10.0.5) Gecko/X Firefox/10.0.5*/
const Device UA1971182 = {S37389, 93, { &P17595,&P17296,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:10.0.2) Gecko/X Firefox/10.0.2 Firefox/10.0.2*/
const Device UA2222191 = {S37390, 80, { &P17595,&P17296,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2; rv:10.0.1) Gecko/X Firefox/10.0.1*/
const Device UA438883 = {S37391, 142, { &P18297,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I8530 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033060 = {S37392, 66, { &P18297,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I8530 Build/GINGERBREAD)*/
const Device UA2234604 = {S37393, 161, { &P18297,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; SAMSUNG GT-I8530/I8530BULE3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257151 = {S37394, 161, { &P18297,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; SAMSUNG GT-I8530/I8530BULH1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2502274 = {S37395, 140, { &P18297,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-I8530 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2590145 = {S37396, 135, { &P18297,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8530 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA452960 = {S37397, 59, { &P17168,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7310 Build/HTJ85B)*/
const Device UA623810 = {S37398, 130, { &P17168,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7310 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA626405 = {S37399, 131, { &P17168,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHW-M305W Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA662134 = {S37400, 130, { &P17168,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7310 Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA685316 = {S37401, 131, { &P17168,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHW-M305W Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2463603 = {S37402, 137, { &P17168,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7310 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2609387 = {S37403, 137, { &P17168,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7310 Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2610060 = {S37404, 160, { &P17168,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7310 Build/HTJ85B) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2614815 = {S37405, 128, { &P17168,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P7310 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2616537 = {S37406, 66, { &P17168,&P6437,&P16179,&P18092 }};	/*Useragent - Biblia/2.1.2 (Linux; U; Android 3.2; es-ES; GT-P7310 Build/HTJ85B)*/
const Device UA2617910 = {S37407, 129, { &P17168,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7310 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2618023 = {S37408, 71, { &P17168,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 3.2; en-US; GT-P7310 Build/HTJ85B.UELA3)*/
const Device UA2619980 = {S37409, 71, { &P17168,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 3.2; en-US; GT-P7310 Build/HTJ85B.UEKJ5)*/
const Device UA2623940 = {S37410, 87, { &P17168,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 3.2; en-US; GT-P7310 Build/Overcome 8.9 Series v2.2.0)*/
const Device UA2625641 = {S37411, 269, { &P17168,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7310 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.12;FBDM/{density=1.0,width=800,height=1232};FBLC/en_US;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P7310;FBSV/3.2;]*/
const Device UA2626311 = {S37412, 132, { &P17168,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7310 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2626835 = {S37413, 128, { &P17168,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7310 Build/HTJ85B) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2488670 = {S37414, 112, { &P18221,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia302/14.26; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA623495 = {S37415, 141, { &P15364,&P17820,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/12.04 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19*/
const Device UA625694 = {S37416, 139, { &P15364,&P17820,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/12.04 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19*/
const Device UA785480 = {S37417, 141, { &P15364,&P17820,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/12.04 Chromium/18.0.1025.151 Chrome/18.0.1025.151 Safari/535.19*/
const Device UA1959884 = {S37418, 176, { &P15364,&P17820,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/12.04 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19,gzip(gfe) (via translate.google.com)*/
const Device UA1960722 = {S37419, 178, { &P15364,&P17820,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/12.04 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19,gzip(gfe) (via translate.google.com)*/
const Device UA2836231 = {S37420, 149, { &P15364,&P17820,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/12.04 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19,gzip(gfe)*/
const Device UA2001048 = {S37421, 60, { &P15364,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress 3.5-alpha-21535 Feed Client Mozilla/5.0 Compatible*/
const Device UA2204099 = {S37422, 60, { &P15364,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress 3.5-alpha-21989 Feed Client Mozilla/5.0 Compatible*/
const Device UA2408266 = {S37423, 58, { &P15364,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress 3.5-RC6-23166 Feed Client Mozilla/5.0 Compatible*/
const Device UA2492134 = {S37424, 60, { &P15364,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress 3.6-alpha-23334 Feed Client Mozilla/5.0 Compatible*/
const Device UA2771287 = {S37425, 60, { &P15364,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress 3.6-beta3-24375 Feed Client Mozilla/5.0 Compatible*/
const Device UA2870910 = {S37426, 54, { &P15364,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress 3.6-beta4 Feed Client Mozilla/5.0 Compatible*/
const Device UA2888682 = {S37427, 60, { &P15364,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress 3.6-beta4-24523 Feed Client Mozilla/5.0 Compatible*/
const Device UA2974878 = {S37428, 58, { &P15364,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress 3.6-RC1-24757 Feed Client Mozilla/5.0 Compatible*/
const Device UA2975509 = {S37429, 52, { &P15364,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress 3.6-RC2 Feed Client Mozilla/5.0 Compatible*/
const Device UA2997796 = {S37430, 37, { &P15364,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5.1; http://news.money.ca*/
const Device UA3017855 = {S37431, 38, { &P15364,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5.2; http://www.isitwp.com*/
const Device UA3020168 = {S37432, 72, { &P15364,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5.2; http://boocu.com,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3020916 = {S37433, 43, { &P15364,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5.2; http://localhost/wordpress*/
const Device UA3025461 = {S37434, 48, { &P15364,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress 3.6 Feed Client Mozilla/5.0 Compatible*/
const Device UA3029777 = {S37435, 61, { &P15364,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.6-RC2-24920; http://ioneglobalgrind.wordpress.com*/
const Device UA3032607 = {S37436, 60, { &P15364,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress 3.7-alpha-24954 Feed Client Mozilla/5.0 Compatible*/
const Device UA3037712 = {S37437, 60, { &P15364,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress 3.7-alpha-25000 Feed Client Mozilla/5.0 Compatible*/
const Device UA3037753 = {S37438, 26, { &P15364,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.7-alpha-25000;*/
const Device UA3068049 = {S37439, 60, { &P15364,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress 3.7-alpha-25157 Feed Client Mozilla/5.0 Compatible*/
const Device UA3080592 = {S37440, 26, { &P15364,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.7-alpha-25157;*/
const Device UA3088392 = {S37441, 49, { &P15364,&P7480,&P18245,&P18156 }};	/*Useragent - WordPress/3.5.1; http://tommyscostaricadreams.com*/
const Device UA2527273 = {S37442, 141, { &P22394,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C6603 Build/10.1.A.0.287) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2582906 = {S37443, 141, { &P22394,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6603 Build/10.1.A.1.434) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2490096 = {S37444, 112, { &P15785,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia201/11.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA900259 = {S37445, 141, { &P15517,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MT15i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2196253 = {S37446, 153, { &P15517,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonMT15i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2205588 = {S37447, 154, { &P15517,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonMT15iv Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2349639 = {S37448, 63, { &P15517,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MT15i Build/4.1.B.0.587)*/
const Device UA2375516 = {S37449, 132, { &P15517,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MT15i Build/4.1.B.0.431) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2400676 = {S37450, 141, { &P15517,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; MT15i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2935064 = {S37451, 140, { &P15517,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; MT15i Build/4.1.B.1.13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2435621 = {S37452, 145, { &P15767,&P17090,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8J2 Safari/6533.18.5*/
const Device UA2435644 = {S37453, 145, { &P15767,&P17090,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8L1 Safari/6533.18.5*/
const Device UA2436073 = {S37454, 145, { &P15767,&P17090,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8G4 Safari/6533.18.5*/
const Device UA2436277 = {S37455, 145, { &P15767,&P17090,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8J3 Safari/6533.18.5*/
const Device UA2436452 = {S37456, 145, { &P15767,&P17090,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8H7 Safari/6533.18.5*/
const Device UA2437314 = {S37457, 145, { &P15767,&P17090,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8F190 Safari/6533.18.5*/
const Device UA2461209 = {S37458, 145, { &P15767,&P17090,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8K2 Safari/6533.18.5*/
const Device UA2468538 = {S37459, 145, { &P15767,&P17090,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8F191 Safari/6533.18.5*/
const Device UA2485749 = {S37460, 145, { &P15767,&P17090,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8H8 Safari/6533.18.5*/
const Device UA32793 = {S37461, 136, { &P15594,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SGH-T589 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32797 = {S37462, 135, { &P15594,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.2; en-us; SGH-T589 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32798 = {S37463, 135, { &P15594,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;SGH-T589 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32800 = {S37464, 135, { &P15594,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.2; en-us; SGH-T589 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47634 = {S37465, 137, { &P15594,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; SGH-T589R Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48595 = {S37466, 135, { &P15594,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SGH-T589Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48602 = {S37467, 135, { &P15594,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SGH-T589 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA378652 = {S37468, 60, { &P15594,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; SGH-T589 Build/FROYO)*/
const Device UA828033 = {S37469, 61, { &P15594,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; SGH-T589R Build/FROYO)*/
const Device UA8663 = {S37470, 139, { &P13437,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; T-Mobile G2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38685 = {S37471, 138, { &P13437,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; T-Mobile G2Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38688 = {S37472, 138, { &P13437,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; T-Mobile G2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38690 = {S37473, 138, { &P13437,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; T-Mobile G2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA38696 = {S37474, 138, { &P13437,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; T-Mobile G2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA38702 = {S37475, 138, { &P13437,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; T-MobileG2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38718 = {S37476, 138, { &P13437,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;T-Mobile G2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38719 = {S37477, 138, { &P13437,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; T-Mobile G2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52467 = {S37478, 138, { &P13437,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;T-Mobile G2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52479 = {S37479, 138, { &P13437,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.3; en-us; T-Mobile G2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52481 = {S37480, 138, { &P13437,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; T-MobileG2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA69774 = {S37481, 139, { &P13437,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; T-Mobile G2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74887 = {S37482, 137, { &P13437,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; T-Mobile G2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79305 = {S37483, 138, { &P13437,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; T-Mobile G2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA213435 = {S37484, 63, { &P13437,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; T-Mobile G2 Build/GRJ22)*/
const Device UA378242 = {S37485, 63, { &P13437,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; T-Mobile G2 Build/GRI40)*/
const Device UA570773 = {S37486, 146, { &P13437,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; T-Mobile G2 Build/GRJ22) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA622448 = {S37487, 145, { &P13437,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; T-Mobile G2 Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30*/
const Device UA768014 = {S37488, 139, { &P13437,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; T-Mobile G2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA652658 = {S37489, 140, { &P18324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/ZV4.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1997083 = {S37490, 64, { &P18324,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; LG-VM696 Build/ZV5.GWK74)*/
const Device UA1997135 = {S37491, 140, { &P18324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/ZV5.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2555375 = {S37492, 145, { &P18324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/Sleipnir 2.0.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2594660 = {S37493, 140, { &P18324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/ICS Elite) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2726392 = {S37494, 138, { &P18324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-; LG-VM696 Build/ZV5.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2779710 = {S37495, 143, { &P18324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/TwIsT3D v1.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2841350 = {S37496, 150, { &P18324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/Tagged 1.0.5 (Full)) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982890 = {S37497, 143, { &P18324,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/Tagged 1.0.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24594 = {S37498, 123, { &P14482,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaC3-01/5.0 (06.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA44114 = {S37499, 73, { &P14482,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaC3-01/5.0 (05.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac3-01*/
const Device UA622986 = {S37500, 125, { &P14482,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaC3-01.5/5.0 (06.75) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA2565147 = {S37501, 140, { &P18736,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; HTC_K2_UL Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597404 = {S37502, 140, { &P18736,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; HTC_OneSV Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666904 = {S37503, 63, { &P18736,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC One SV Build/JZO54K)*/
const Device UA2677987 = {S37504, 151, { &P18736,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC_OneSV/2.14.161.7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2679252 = {S37505, 140, { &P18736,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC_C525u Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2712412 = {S37506, 151, { &P18736,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-ch; HTC_OneSV/2.14.166.7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2737042 = {S37507, 151, { &P18736,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-at; HTC_OneSV/2.14.169.7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2737439 = {S37508, 153, { &P18736,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; HTC_OneSV/2.14.164.107 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752819 = {S37509, 151, { &P18736,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; HTC_OneSV/2.14.162.8 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778830 = {S37510, 141, { &P18736,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One SV Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867782 = {S37511, 151, { &P18736,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; HTC_OneSV/2.15.118.2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879630 = {S37512, 151, { &P18736,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; HTC_OneSV/2.16.111.3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2960666 = {S37513, 160, { &P18736,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; HTC_C525u-orange-LS/2.16.73.3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049121 = {S37514, 151, { &P18736,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; cs-cz; HTC_OneSV/2.15.113.2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3064656 = {S37515, 136, { &P21648,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068411 = {S37516, 137, { &P21648,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505G Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126374 = {S37517, 137, { &P21648,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505X Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA6515 = {S37518, 143, { &P12482,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; SGH-T959 Build/ECLAIR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32202 = {S37519, 144, { &P12482,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.1-update1; en-us; SGH-T959 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32213 = {S37520, 144, { &P12482,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SGH-T959 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 MobileSafari/530.17*/
const Device UA32216 = {S37521, 144, { &P12482,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SGH-T959Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32226 = {S37522, 144, { &P12482,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us;SGH-T959 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32228 = {S37523, 144, { &P12482,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.1-update1; en-us; SGH-T959 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA62489 = {S37524, 145, { &P12482,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SGH-T959 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA71922 = {S37525, 143, { &P12482,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; SGH-T959 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA89977 = {S37526, 150, { &P12482,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SGH-T959 Build/ECLAIR) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA146796 = {S37527, 67, { &P12482,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SGH-T959 Build/ECLAIR)*/
const Device UA152845 = {S37528, 146, { &P12482,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; SGH-T959D Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018858 = {S37529, 68, { &P12482,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SGH-T959D Build/ECLAIR)*/
const Device UA251763 = {S37530, 81, { &P17595,&P17296,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en) Presto/2.9.168 Version/11.52*/
const Device UA2721054 = {S37531, 120, { &P17595,&P17296,&P22542,&P18092 }};	/*Useragent - Opera 11.52(Intel Mac OS X 10.6.8): Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en-US) Presto/2.9.168 Version/11.52*/
const Device UA2434553 = {S37532, 148, { &P17346,&P18432,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A523 Safari/8536.25*/
const Device UA2434938 = {S37533, 146, { &P17346,&P18432,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A403 Safari/8536.25*/
const Device UA2437098 = {S37534, 146, { &P17346,&P18432,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10A406 Safari/8536.25*/
const Device UA2598109 = {S37535, 144, { &P17346,&P18432,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 601 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML like Gecko) CriOS/23.0.1271.96 Mobile/10A523 Safari/8536.25*/
const Device UA3061042 = {S37536, 139, { &P16292,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Galaxy Nexus Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3064768 = {S37537, 141, { &P16292,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065464 = {S37538, 140, { &P16292,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2561251 = {S37539, 137, { &P20705,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2659429 = {S37540, 136, { &P20705,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTCONE Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2665460 = {S37541, 139, { &P20705,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC_PN071 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2667561 = {S37542, 142, { &P20705,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One 801e Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3000388 = {S37543, 146, { &P20705,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One dual sim Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA8126 = {S37544, 82, { &P11724,&P5026,&P3410,&P18092 }};	/*Useragent - BlackBerry8310_CMCC/4.5.0.142 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/241*/
const Device UA63673 = {S37545, 77, { &P11724,&P5026,&P3410,&P18092 }};	/*Useragent - BlackBerry8310/4.5.0.182 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102*/
const Device UA2462482 = {S37546, 130, { &P19782,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; B1-A71 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Safari/534.30*/
const Device UA2462483 = {S37547, 59, { &P19782,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; B1-A71 Build/JZO54K)*/
const Device UA2561769 = {S37548, 130, { &P19782,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; B1-A71 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2205607 = {S37549, 152, { &P12280,&P17146,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9A406 Safari/7534.48.3*/
const Device UA2205626 = {S37550, 152, { &P12280,&P17146,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9A405 Safari/7534.48.3*/
const Device UA2205791 = {S37551, 150, { &P12280,&P17146,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9A334 Safari/7534.48.3*/
const Device UA48675 = {S37552, 126, { &P15602,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; A501 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA74002 = {S37553, 128, { &P15602,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; A501 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA569266 = {S37554, 55, { &P15602,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; A501 Build/HTJ85B)*/
const Device UA636644 = {S37555, 57, { &P15602,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; A501 Build/HTK55D)*/
const Device UA68133 = {S37556, 136, { &P14908,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA178773 = {S37557, 60, { &P14908,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-P1010 Build/FROYO)*/
const Device UA216218 = {S37558, 136, { &P14908,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2607755 = {S37559, 134, { &P14908,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2607805 = {S37560, 124, { &P14908,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1010 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2607923 = {S37561, 60, { &P14908,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; GT-P1010 Build/FROYO)*/
const Device UA2609510 = {S37562, 124, { &P14908,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; GT-P1010 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2610683 = {S37563, 129, { &P14908,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1010 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2611044 = {S37564, 129, { &P14908,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; GT-P1010 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2615848 = {S37565, 73, { &P14908,&P5096,&P3508,&P18092 }};	/*Useragent - Biblia/2.1.2 (Linux; U; Android 2.2.1; es-US; GT-P1010 Build/FROYO.UJKB4)*/
const Device UA2615868 = {S37566, 72, { &P14908,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 2.2.1; en-GB; GT-P1010 Build/FROYO.XWKB8)*/
const Device UA2617205 = {S37567, 54, { &P14908,&P5096,&P3508,&P18092 }};	/*Useragent - Android/4.5 (Linux; U; Android 2.2.1; en_US; GT-P1010)*/
const Device UA2618014 = {S37568, 72, { &P14908,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 2.2.1; de-DE; GT-P1010 Build/FROYO.XWKM2)*/
const Device UA2620356 = {S37569, 73, { &P14908,&P5096,&P3508,&P18092 }};	/*Useragent - Biblia/2.2.3 (Linux; U; Android 2.2.1; es-ES; GT-P1010 Build/FROYO.UJKM1)*/
const Device UA2620938 = {S37570, 72, { &P14908,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 2.2.1; en-US; GT-P1010 Build/FROYO.JPKC4)*/
const Device UA2622391 = {S37571, 74, { &P14908,&P5096,&P3508,&P18092 }};	/*Useragent - Vyrso/2.2.8.1 (Linux; U; Android 2.2.2; pt-BR; GT-P1010 Build/FROYO.UJKM5)*/
const Device UA2623378 = {S37572, 74, { &P14908,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 2.2.1; en-US; GT-P1010 Build/FROYO.UEKC6)*/
const Device UA2625890 = {S37573, 134, { &P14908,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-P1010 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2625940 = {S37574, 255, { &P14908,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1010 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 baidubrowser/042_1.41.3.2_diordna_4201_006/gnusmas_8_1.2.2_0101P-TG/Xy/6EC45499CXCBDFFE6D53AE8C2B0%7C0/1*/
const Device UA2628278 = {S37575, 87, { &P14908,&P5096,&P3508,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.2.10 (Linux; U; Android 2.2.1; en-GB; GT-P1010 Build/FROYO.XWKC1)*/
const Device UA2631274 = {S37576, 134, { &P14908,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2632305 = {S37577, 35, { &P14908,&P5096,&P3508,&P18092 }};	/*Useragent - Android 2.2.1; 8; samsung; GT-P1010*/
const Device UA2469473 = {S37578, 104, { &P18254,&P18110,&P19818,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.8.1 Safari/534.34*/
const Device UA2567833 = {S37579, 102, { &P18254,&P18110,&P19818,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.8.2 Safari/534.34*/
const Device UA2488272 = {S37580, 116, { &P15767,&P19826,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Twitter for iPhone*/
const Device UA2532517 = {S37581, 118, { &P15767,&P19826,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Twitter for iPhone*/
const Device UA2533253 = {S37582, 118, { &P15767,&P19826,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B147 Twitter for iPhone*/
const Device UA2585370 = {S37583, 118, { &P15767,&P19826,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Twitter for iPhone*/
const Device UA2025241 = {S37584, 146, { &P18653,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P870/P87009l Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2166626 = {S37585, 138, { &P18653,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P870 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2204799 = {S37586, 146, { &P18653,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P870/P87010c Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2237741 = {S37587, 146, { &P18653,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P870/P87010f Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2420237 = {S37588, 60, { &P18653,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P870 Build/IMM76I)*/
const Device UA2437374 = {S37589, 146, { &P18653,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P870/P87010i Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA48897 = {S37590, 150, { &P12942,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SAMSUNG-SGH-I927 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48898 = {S37591, 150, { &P12942,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SAMSUNG-SGH-I927 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA360159 = {S37592, 74, { &P12942,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SAMSUNG-SGH-I927 Build/GINGERBREAD)*/
const Device UA362638 = {S37593, 150, { &P12942,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I927 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA363640 = {S37594, 74, { &P12942,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SAMSUNG-SGH-I927 Build/GINGERBREAD)*/
const Device UA2032987 = {S37595, 148, { &P12942,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SAMSUNG-SGH-I927 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA220906 = {S37596, 139, { &P15364,&P17818,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.30 (KHTML, like Gecko) Ubuntu/11.04 Chromium/12.0.742.112 Chrome/12.0.742.112 Safari/534.30*/
const Device UA224953 = {S37597, 137, { &P15364,&P17818,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.30 (KHTML, like Gecko) Ubuntu/11.04 Chromium/12.0.742.112 Chrome/12.0.742.112 Safari/534.30*/
const Device UA43389 = {S37598, 135, { &P17932,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SGH-T499Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43393 = {S37599, 135, { &P17932,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SGH-T499 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA49827 = {S37600, 137, { &P17932,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; fr-ca; SGH-T499V Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59328 = {S37601, 135, { &P17932,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.2; en-us; SGH-T499 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59329 = {S37602, 135, { &P17932,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.2; en-us; SGH-T499 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59335 = {S37603, 136, { &P17932,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SGH-T499 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59332 = {S37604, 135, { &P17932,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;SGH-T499 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA249868 = {S37605, 137, { &P17932,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; SGH-T499Y Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369017 = {S37606, 143, { &P17932,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SGH-T499 Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA557538 = {S37607, 60, { &P17932,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; SGH-T499 Build/FROYO)*/
const Device UA2023098 = {S37608, 124, { &P17932,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SGH-T499 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2023773 = {S37609, 61, { &P17932,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; SGH-T499Y Build/FROYO)*/
const Device UA2039836 = {S37610, 128, { &P18466,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2193488 = {S37611, 57, { &P18466,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; KFTT Build/IML74K)*/
const Device UA2236174 = {S37612, 135, { &P18466,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; KFTT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2523784 = {S37613, 126, { &P18466,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; KFTT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2892714 = {S37614, 131, { &P22228,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SM-T210 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2899414 = {S37615, 132, { &P22228,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SM-T210R Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2965001 = {S37616, 61, { &P22228,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SM-T210R Build/JZO54K)*/
const Device UA3006481 = {S37617, 60, { &P22228,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SM-T210 Build/JZO54K)*/
const Device UA2000086 = {S37618, 153, { &P12616,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; SonyEricssonWT19i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026619 = {S37619, 139, { &P12616,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; WT19i Build/4.1.L.0.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030232 = {S37620, 141, { &P12616,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; WT19i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031409 = {S37621, 63, { &P12616,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; WT19i Build/4.1.B.0.587)*/
const Device UA2033916 = {S37622, 141, { &P12616,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; WT19i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2353638 = {S37623, 154, { &P12616,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SonyEricssonWT19iv Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA579190 = {S37624, 61, { &P18497,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; M886 Build/HuaweiM886)*/
const Device UA622574 = {S37625, 137, { &P18497,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; M886 Build/HuaweiM886) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623016 = {S37626, 137, { &P18497,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; M886 Build/HuaweiM886) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA630641 = {S37627, 61, { &P18497,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; M886 Build/HuaweiM886)*/
const Device UA868797 = {S37628, 146, { &P18497,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6-Oriax123; en-us; M886 Build/HuaweiM886) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1993228 = {S37629, 157, { &P18497,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6 - iCe - xXNubCakeXx; en-us; M886 Build/HuaweiM886) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2702246 = {S37630, 156, { &P18497,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6_TheTrooperRom_v2.0; en-us; M886 Build/HuaweiM886) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA252238 = {S37631, 145, { &P15885,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; XT603 Build/5.5.1Q-117_PAX-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA628900 = {S37632, 145, { &P15885,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; XT603 Build/5.5.1Q-117_PAX-33) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995871 = {S37633, 145, { &P15885,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; XT603 Build/5.5.1Q-117_PAX-63) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032476 = {S37634, 145, { &P15885,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; XT603 Build/5.5.1Q-117_PAX-58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2235252 = {S37635, 145, { &P15885,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; XT603 Build/5.5.1Q-117_PAX-69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2469782 = {S37636, 145, { &P15885,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; XT603 Build/5.5.1Q-117_PAX-79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2659194 = {S37637, 69, { &P15885,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; XT603 Build/5.5.1Q-117_PAX-79)*/
const Device UA907321 = {S37638, 146, { &P15767,&P18117,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9B206 Safari/7534.48.3*/
const Device UA908454 = {S37639, 144, { &P15767,&P18117,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9B176 Safari/7534.48.3*/
const Device UA1960665 = {S37640, 144, { &P15767,&P18117,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9A405 Safari/7534.48.3*/
const Device UA2531341 = {S37641, 132, { &P20160,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N5100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2704189 = {S37642, 61, { &P20160,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N5100 Build/JZO54K)*/
const Device UA2995472 = {S37643, 130, { &P20160,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-N5100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA61348 = {S37644, 148, { &P15065,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID Pro Build/4.5.1-110-VNS-11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61349 = {S37645, 148, { &P15065,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID Pro Build/4.5.1-110-VNS-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79490 = {S37646, 147, { &P15065,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;DROID Pro Build/4.5.1-110-VNS-22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79725 = {S37647, 147, { &P15065,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; DROID Pro Build/4.5.1-110-VNS-22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250019 = {S37648, 148, { &P15065,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID Pro Build/4.5.1-110-VNS-22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA440700 = {S37649, 148, { &P15065,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID Pro Build/4.5.1-110-VNS-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA575056 = {S37650, 72, { &P15065,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; DROID Pro Build/4.5.1-110-VNS-22)*/
const Device UA658437 = {S37651, 147, { &P15065,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID Pro Build/4.5.1-110-VNS-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA740859 = {S37652, 147, { &P15065,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT610 Build/4.5.2-51_V2L-28_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA744757 = {S37653, 147, { &P15065,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID ProBuild/4.5.1-110-VNS-22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA753384 = {S37654, 147, { &P15065,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROIDPro Build/4.5.1-110-VNS-22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA768280 = {S37655, 147, { &P15065,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; DROID Pro Build/4.5.1-110-VNS-22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA846680 = {S37656, 147, { &P15065,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDPro Build/4.5.1-110-VNS-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA850987 = {S37657, 147, { &P15065,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;DROID Pro Build/4.5.1-110-VNS-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA854655 = {S37658, 147, { &P15065,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID ProBuild/4.5.1-110-VNS-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA862576 = {S37659, 147, { &P15065,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; DROID Pro Build/4.5.1-110-VNS-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1907227 = {S37660, 147, { &P15065,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.3; en-us; DROID Pro Build/4.5.1-110-VNS-22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1992394 = {S37661, 147, { &P15065,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; DROID Pro Build/4.5.1-110-VNS-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019808 = {S37662, 136, { &P15065,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID Pro Build/S273) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2841497 = {S37663, 123, { &P15065,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; DROID Pro Build/4.5.1-110-VNS-35) com.google.android.apps.books/1.1.5 (1105) Safari/*/
const Device UA18477 = {S37664, 151, { &P23918,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-ca; SonyEricssonR800a Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18481 = {S37665, 153, { &P23918,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonR800a Build/3.0.1.A.0.126) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18482 = {S37666, 153, { &P23918,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonR800a Build/3.0.1.A.0.135) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41853 = {S37667, 141, { &P23918,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-mx; R800a Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48353 = {S37668, 151, { &P23918,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; iw-il; SonyEricssonR800a Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57706 = {S37669, 139, { &P23918,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-ca; R800a Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57710 = {S37670, 151, { &P23918,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; es-mx; SonyEricssonR800a Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57720 = {S37671, 153, { &P23918,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-mx; SonyEricssonR800a Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57716 = {S37672, 139, { &P23918,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; es-mx; R800a Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72434 = {S37673, 151, { &P23918,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; SonyEricssonR800a Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA616492 = {S37674, 64, { &P23918,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; R800a Build/4.0.2.A.0.42)*/
const Device UA632672 = {S37675, 152, { &P23918,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; SonyEricssonR800a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA655942 = {S37676, 140, { &P23918,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; R800a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA680353 = {S37677, 153, { &P23918,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-cr; SonyEricssonR800a Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA706909 = {S37678, 65, { &P23918,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; R800a Build/3.0.1.A.0.145)*/
const Device UA2018539 = {S37679, 63, { &P23918,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.2; R800a Build/3.0.A.2.181)*/
const Device UA2025878 = {S37680, 153, { &P23918,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonR800a Build/3.0.1.A.0.148) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033619 = {S37681, 139, { &P23918,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; iw-il; R800a Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA452888 = {S37682, 167, { &P15184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es;LG-P920/V20a Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA634071 = {S37683, 163, { &P15184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; LG-P920 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA661548 = {S37684, 167, { &P15184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es;LG-P920/V20c Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA694827 = {S37685, 59, { &P15184,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; LG-P920 Build/GRJ90)*/
const Device UA752524 = {S37686, 168, { &P15184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; LG-P920/V20a Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA781741 = {S37687, 168, { &P15184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; nl-nl; LG-P920/V21b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA787823 = {S37688, 165, { &P15184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-sa; LG-P920h Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA870719 = {S37689, 164, { &P15184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-sa; LG-P920h Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA1996326 = {S37690, 167, { &P15184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es;LG-P920/V20e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2025835 = {S37691, 167, { &P15184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-nz;LG-P920/V20b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2027856 = {S37692, 180, { &P15184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6 www.htcspain.com; es-es; LG-P920 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2028219 = {S37693, 175, { &P15184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6/AcuraTeam; es-sa; LG-P920h Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2029605 = {S37694, 168, { &P15184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; LG-P920/V21a Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2029775 = {S37695, 168, { &P15184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; LG-P920/V20b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2033366 = {S37696, 141, { &P15184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ar-ar; LG-P920 Build/GRJ90) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2033596 = {S37697, 184, { &P15184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P920 Build/PromethiusProject v1.5 21E) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2222201 = {S37698, 170, { &P15184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; LG-P920-Orange Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2441798 = {S37699, 128, { &P15184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LG-P920 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2789193 = {S37700, 167, { &P15184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-ch;LG-P920/V20f Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2789260 = {S37701, 184, { &P15184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; LG-P920 Build/PromethiusProject v1.5 20F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA3098890 = {S37702, 168, { &P15184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; nl-nl; LG-P920/V21e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA616718 = {S37703, 129, { &P16976,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AT300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1952595 = {S37704, 58, { &P16976,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; AT300 Build/IML74K)*/
const Device UA2652341 = {S37705, 136, { &P16976,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AT300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2599845 = {S37706, 132, { &P18466,&P18110,&P21708,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Safari/535.19 Silk-Accelerated=true*/
const Device UA2675976 = {S37707, 128, { &P18385,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676013 = {S37708, 129, { &P18385,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 7 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676241 = {S37709, 127, { &P18385,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Nexus 7 Build/JOP40C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2707174 = {S37710, 135, { &P18385,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA605514 = {S37711, 112, { &P15803,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia200/11.56; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6*/
const Device UA2184010 = {S37712, 113, { &P15803,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia200/11.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA511515 = {S37713, 84, { &P17595,&P17296,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:2.0b11pre) Gecko/X Firefox/4.0b11pre*/
const Device UA552365 = {S37714, 78, { &P17595,&P17296,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6.8; en; rv:2.0) Gecko/X Firefox/4.0*/
const Device UA1988273 = {S37715, 78, { &P17595,&P17296,&P17203,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Intel Mac OS X 10.6; rv:2.0b8) Gecko/X Firefox/4.0b8)*/
const Device UA1996093 = {S37716, 151, { &P18317,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300T Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2191012 = {S37717, 152, { &P18317,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300TG Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2261533 = {S37718, 152, { &P18317,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300TL Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2261563 = {S37719, 139, { &P18317,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Pad TF300T Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2261654 = {S37720, 146, { &P18317,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Transformer Pad TF300T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2351962 = {S37721, 146, { &P18317,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Transformer Pad TF300T Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA32853 = {S37722, 80, { &P15364,&P5690,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; ru; rv:1.8.1.12) Gecko/20080201 Firefox*/
const Device UA116011 = {S37723, 55, { &P15364,&P5690,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Firefox Addon; Windows XP 5.1)*/
const Device UA372617 = {S37724, 37, { &P15364,&P5690,&P18107,&P18092 }};	/*Useragent - Mozilla/4.0 (Firefox; Windows NT 5.1)*/
const Device UA454150 = {S37725, 171, { &P15364,&P5690,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.25) Gecko/X AskTbFWV5/3.14.0.19709  Firefox[xSP_2:f862ceb31b6660fXb111b35a7e_208]  X  (.NET CLR 3.5.30729) FBSMTWB*/
const Device UA504528 = {S37726, 85, { &P15364,&P5690,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.13) Gecko/X  Firefox  GTB7.1*/
const Device UA636922 = {S37727, 114, { &P15364,&P5690,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.25) Gecko/X  Firefoxcontrol/5.00294  X  SearchToolbar/1.2*/
const Device UA656261 = {S37728, 99, { &P15364,&P5690,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.25) Gecko/X  Firefox  (.NET CLR 3.5.30729)*/
const Device UA660195 = {S37729, 117, { &P15364,&P5690,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.12) Gecko/X AskTbX-SD/3.14.1.20007  FirefoxBuild/13.00287  X*/
const Device UA876002 = {S37730, 100, { &P15364,&P5690,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.13) Gecko/X  Firefox  ( .NET CLR 3.5.30729)*/
const Device UA915269 = {S37731, 84, { &P15364,&P5690,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-GB; rv:1.8.1.6) Gecko/X Firefox /2.0.0.6*/
const Device UA1989401 = {S37732, 81, { &P15364,&P5690,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.7) Gecko/X Firefox_123/0.9.3*/
const Device UA2232421 = {S37733, 82, { &P15364,&P5690,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.452) Gecko/X Mnenhy/0.6.0.104*/
const Device UA2413349 = {S37734, 76, { &P15364,&P5690,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:2.0) Gecko/X Firefox,##ProjectID10##2071####*/
const Device UA2413832 = {S37735, 125, { &P15364,&P5690,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; de; rv:1.9.2.6) Gecko/X Firefox/ Anonymisiert durch AlMiSoft Browser-Anonymisierer X*/
const Device UA2986178 = {S37736, 82, { &P15364,&P5690,&P18107,&P18092 }};	/*Useragent - Mozilla/3.0 (Windows; U; Windows NT 5.1; en-US; rv:1.452) Gecko/X Mnenhy/0.6.0.104*/
const Device UA2657943 = {S37737, 80, { &P18318,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; ASUS Transformer Pad TF700T Build/JOP40D)*/
const Device UA2675421 = {S37738, 151, { &P18318,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ASUS Transformer Pad TF700T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3048812 = {S37739, 145, { &P18318,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Transformer Pad TF700T Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3049211 = {S37740, 139, { &P18318,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; ja-jp; ASUS Pad TF700T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA351703 = {S37741, 123, { &P15364,&P5690,&P21902,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/16.0.912.63) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA353591 = {S37742, 211, { &P15364,&P5690,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/16.0.912.63; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA357841 = {S37743, 210, { &P15364,&P5690,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/16.0.912.63; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA366234 = {S37744, 123, { &P15364,&P5690,&P21902,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/16.0.912.75) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7*/
const Device UA366463 = {S37745, 139, { &P15364,&P5690,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/16.0.912.75; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA368189 = {S37746, 123, { &P15364,&P5690,&P21902,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/16.0.912.77) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA397518 = {S37747, 215, { &P15364,&P5690,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/16.0.912.75; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA397882 = {S37748, 75, { &P15364,&P5690,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/16.0.912.75)*/
const Device UA582399 = {S37749, 207, { &P15364,&P5690,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/16.0.912.75; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA583779 = {S37750, 123, { &P15364,&P5690,&P21902,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/16.0.912.75) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7*/
const Device UA585266 = {S37751, 178, { &P15364,&P5690,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/16.0.912.75; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA843498 = {S37752, 238, { &P15364,&P5690,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/16.0.912.75; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA1936775 = {S37753, 123, { &P15364,&P5690,&P21902,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/16.0.912.63) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA380160 = {S37754, 140, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Desire Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA454869 = {S37755, 141, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Desire Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA622434 = {S37756, 147, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; HTC Desire Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.1 Mobile Safari/534.30*/
const Device UA854462 = {S37757, 141, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Desire Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA906299 = {S37758, 141, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Desire Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1960497 = {S37759, 143, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC Desire V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1993387 = {S37760, 143, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC_Desire_V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996092 = {S37761, 150, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; HTC_Desire_VC_T328d Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2023808 = {S37762, 134, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Desire Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2024214 = {S37763, 63, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Desire Build/IMM76L)*/
const Device UA2028641 = {S37764, 144, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-id; HTC Desire VC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031105 = {S37765, 144, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC_Desire_VC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034214 = {S37766, 137, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Desire Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2242218 = {S37767, 65, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Desire V Build/IML74K)*/
const Device UA2338511 = {S37768, 150, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; HTC Desire VC T328d Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2465292 = {S37769, 141, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ar-; HTC_Desire_V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2521317 = {S37770, 66, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Desire VC Build/IML74K)*/
const Device UA2729676 = {S37771, 147, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Desire Build/mariusa v2.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2750915 = {S37772, 143, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; HTC_Desire_P Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2856392 = {S37773, 147, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Desire Build/mariusa v3.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2859633 = {S37774, 143, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; HTC_Desire_Q Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057590 = {S37775, 145, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; HTC_Desire_200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3098578 = {S37776, 72, { &P11747,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Desire VC T328d Build/IML74K)*/
const Device UA199127 = {S37777, 183, { &P15364,&P7133,&P21897,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/12.0.742.91; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA205718 = {S37778, 134, { &P15364,&P7133,&P21897,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/12.0.742.100) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30*/
const Device UA320418 = {S37779, 88, { &P15364,&P7133,&P21897,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/12.0.742.91)*/
const Device UA326144 = {S37780, 96, { &P15364,&P7133,&P21897,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/12.0.742.112)*/
const Device UA426977 = {S37781, 174, { &P15364,&P7133,&P21897,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 6.1; chromeframe/12.0.742.100; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)*/
const Device UA652294 = {S37782, 127, { &P15364,&P7133,&P21897,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/12.0.742.122) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA665234 = {S37783, 134, { &P15364,&P7133,&P21897,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/12.0.742.122) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA737906 = {S37784, 162, { &P15364,&P7133,&P21897,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/12.0.742.91; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729)*/
const Device UA758812 = {S37785, 174, { &P15364,&P7133,&P21897,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; chromeframe/12.0.742.100)*/
const Device UA1887520 = {S37786, 134, { &P15364,&P7133,&P21897,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/12.0.742.112) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30*/
const Device UA2137638 = {S37787, 132, { &P15364,&P7133,&P21897,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/12.0.742.91) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.91 Safari/534.30*/
const Device UA2139075 = {S37788, 127, { &P15364,&P7133,&P21897,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/12.0.742.112) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30*/
const Device UA2211416 = {S37789, 196, { &P15364,&P7133,&P21897,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/12.0.742.91; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2258982 = {S37790, 127, { &P15364,&P7133,&P21897,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/12.0.742.100) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30*/
const Device UA2681934 = {S37791, 224, { &P15364,&P7133,&P21897,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/12.0.742.122; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA3045677 = {S37792, 174, { &P15364,&P7133,&P21897,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; chromeframe/12.0.742.100; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)*/
const Device UA3094398 = {S37793, 133, { &P15767,&P19826,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B329 Safari/8536.25*/
const Device UA3096505 = {S37794, 133, { &P15767,&P19826,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B146 Safari/8536.25*/
const Device UA3096511 = {S37795, 131, { &P15767,&P19826,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B141 Safari/8536.25*/
const Device UA3096528 = {S37796, 133, { &P15767,&P19826,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B147 Safari/8536.25*/
const Device UA2455563 = {S37797, 147, { &P18626,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I547 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2530259 = {S37798, 69, { &P18626,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SAMSUNG-SGH-I547 Build/JRO03L)*/
const Device UA2605309 = {S37799, 140, { &P18626,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I547C Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2751060 = {S37800, 62, { &P18626,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I547C Build/JZO54K)*/
const Device UA2891343 = {S37801, 147, { &P18626,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I547 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2945537 = {S37802, 69, { &P18626,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-I547 Build/JZO54K)*/
const Device UA3094024 = {S37803, 181, { &P18626,&P18111,&P18112,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=comancheatt;devm=SAMSUNG-SGH-I547;man=samsung;brw=Android;shwd=480x800;shwa=480x763;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA2669602 = {S37804, 164, { &P22597,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SCH-I545 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2874774 = {S37805, 167, { &P22597,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SCH-I545 4G Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2935762 = {S37806, 137, { &P22597,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3074496 = {S37807, 165, { &P22597,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SCH-I545L Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA251746 = {S37808, 135, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; sdk Build/ICS_MR0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA358871 = {S37809, 135, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; sdk Build/ICS_MR0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA359993 = {S37810, 131, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; sdk Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA604839 = {S37811, 131, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; sdk Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA781221 = {S37812, 151, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Full AOSP on Rk29sdk Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA868803 = {S37813, 144, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Full AOSP on Rk29sdk Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1951043 = {S37814, 131, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; rk30sdk Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1968864 = {S37815, 144, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Full AOSP on Rk29sdk Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2021728 = {S37816, 124, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; sdk Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2021922 = {S37817, 142, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; google_sdk Build/ICS_MR0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2024930 = {S37818, 60, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; google_sdk Build/MR1)*/
const Device UA2025071 = {S37819, 53, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; sdk Build/MR1)*/
const Device UA2028827 = {S37820, 124, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; sdk Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2031274 = {S37821, 138, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; google_sdk Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034409 = {S37822, 73, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Full AOSP on Rk29sdk Build/IML74K)*/
const Device UA2150171 = {S37823, 138, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; google_sdk Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2363073 = {S37824, 161, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 BaDoinkDownloader_Android_2.0*/
const Device UA2455567 = {S37825, 164, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 BaDoinkDownloader_AndroidTab_2.0*/
const Device UA2474388 = {S37826, 57, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.2; sdk Build/ICS_MR0)*/
const Device UA2476424 = {S37827, 73, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Full AOSP on Rk29sdk Build/IMM76D)*/
const Device UA2502118 = {S37828, 145, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; mmi_apps_sdk Build/6.7.1_22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2758817 = {S37829, 138, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; rk29sdk Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2761885 = {S37830, 128, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; sdk Build/ICS_MR0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2835748 = {S37831, 60, { &P18061,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; rk30sdk Build/IMM76D)*/
const Device UA2670980 = {S37832, 164, { &P22594,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SGH-M919 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2966271 = {S37833, 156, { &P22594,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SGH-M919 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2361020 = {S37834, 128, { &P19775,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A210 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2427401 = {S37835, 57, { &P19775,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; A210 Build/JRO03H)*/
const Device UA2696183 = {S37836, 47, { &P19775,&P18111,&P18112,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android 4.1.1 acer_A210)*/
const Device UA3048906 = {S37837, 130, { &P19775,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A210 Build/JRO4HDb1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2714132 = {S37838, 132, { &P18466,&P18110,&P22588,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Safari/535.19 Silk-Accelerated=true*/
const Device UA99000 = {S37839, 118, { &P15364,&P5690,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/1.0.154.36 Safari/525.19*/
const Device UA193880 = {S37840, 116, { &P15364,&P5690,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/1, 6, 0, 4 Safari/533.3*/
const Device UA249848 = {S37841, 116, { &P15364,&P5690,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/1, 6, 2, 0 Safari/533.3*/
const Device UA589330 = {S37842, 118, { &P15364,&P5690,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; cs-CZ) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/1.0.154.48 Safari/525.19*/
const Device UA1840978 = {S37843, 118, { &P15364,&P5690,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/1.0.154.48 Safari/525.19*/
const Device UA1883927 = {S37844, 116, { &P15364,&P5690,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/1, 6, 0, 6 Safari/533.3*/
const Device UA2334506 = {S37845, 118, { &P15364,&P5690,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.19 (KHTML	 like Gecko) Chrome/1.0.154.59 Safari/525.19*/
const Device UA2644046 = {S37846, 97, { &P15364,&P5690,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.3 (KHTML, like Gecko) Chrome/1, 6, 0, 4 Safari/535.3*/
const Device UA2775662 = {S37847, 96, { &P15364,&P5690,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/1.0.862.0 Safari/535.2*/
const Device UA2433749 = {S37848, 157, { &P14877,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9000 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2514120 = {S37849, 157, { &P14877,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9000 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2536595 = {S37850, 156, { &P14877,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9000 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2971386 = {S37851, 169, { &P14877,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9000 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/galaxysmtd*/
const Device UA909452 = {S37852, 156, { &P18125,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 4G Build/6.7.2-180_DR4-16_M2-33) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1970028 = {S37853, 156, { &P18125,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 4G Build/6.7.2-180_DR4-16_M2-20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1970159 = {S37854, 137, { &P18125,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1993185 = {S37855, 156, { &P18125,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 4G Build/6.7.2-180_DR4-16_M2-35) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1997264 = {S37856, 153, { &P18125,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 Build/6.7.2-180_DR4-16_M2-37) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2002769 = {S37857, 156, { &P18125,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 4G Build/6.7.2-180_DR4-16_M2-37) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2017588 = {S37858, 75, { &P18125,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; DROID4 Build/6.7.2-180_DR4-16_M2-37)*/
const Device UA2026149 = {S37859, 153, { &P18125,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 Build/6.7.2-180_DR4-16_M2-33) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026268 = {S37860, 156, { &P18125,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 4G Build/6.7.2-180_DR4-16_M2-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026826 = {S37861, 156, { &P18125,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 4G Build/6.7.2-180_DR4-16_M2-26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027379 = {S37862, 156, { &P18125,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 4G Build/6.7.2-180_DR4-16_M2-32) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027763 = {S37863, 156, { &P18125,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 4G Build/6.7.2-180_DR4-16_M2-28) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2189397 = {S37864, 146, { &P18125,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 Build/6.7.2-180_DR4-16_M2-37) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2395658 = {S37865, 153, { &P18125,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 Build/6.7.2-180_DR4-16_M2-35) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2553085 = {S37866, 159, { &P18125,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 Build/6.7.2-180_DR4-16_M2-37) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.8 Mobile Safari/534.30*/
const Device UA2879013 = {S37867, 164, { &P23521,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9195 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2945572 = {S37868, 177, { &P23521,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9195/I9195XXUAMF6 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3022539 = {S37869, 177, { &P23521,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; es-es; SAMSUNG GT-I9195/I9195XXUAMG1 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3057761 = {S37870, 165, { &P23521,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9195X Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3058691 = {S37871, 177, { &P23521,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; fr-fr; SAMSUNG GT-I9195/I9195XXUAMG6 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3059232 = {S37872, 165, { &P23521,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; es-us; SAMSUNG GT-I9195L Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3062034 = {S37873, 177, { &P23521,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; fr-fr; SAMSUNG GT-I9195/I9195XXUAMG5 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3074239 = {S37874, 165, { &P23521,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-au; SAMSUNG GT-I9195T Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3114897 = {S37875, 171, { &P23521,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; es-es; SAMSUNG GT-I9195-ORANGE Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA913432 = {S37876, 76, { &P17595,&P17296,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:14.0) Gecko/X Firefox/14.0.1*/
const Device UA1996940 = {S37877, 91, { &P17595,&P17296,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:14.0) Gecko/X Firefox/14.0.1 Firefox/14.0.1*/
const Device UA15015 = {S37878, 180, { &P12088,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaX6-00.1/30.2.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.6 3gpp-gba*/
const Device UA21712 = {S37879, 163, { &P12088,&P5129,&P3690,&P18092 }};	/*Useragent - NokiaX6-00m/21.1.004 (SymbianOS/9.4; U; Series60/5.0 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Safari/525 3gpp-gba*/
const Device UA64694 = {S37880, 178, { &P12088,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaX6-00/21.0.004; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.5.2 3gpp-gba*/
const Device UA575992 = {S37881, 192, { &P12088,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaX6-00/40.0.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 Mobile Safari/533.4 3gpp-gba*/
const Device UA737105 = {S37882, 194, { &P12088,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaX6-00.1/40.2.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 Mobile Safari/533.4 3gpp-gba*/
const Device UA899418 = {S37883, 154, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1809c_v75_gq1008_9p017 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1994439 = {S37884, 149, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1809c_v75_jbl_253 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1997244 = {S37885, 155, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1809c_v75_gq1008_ov5647 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030471 = {S37886, 162, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1809c_v75_gq1008_ov5647_exnote Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031757 = {S37887, 153, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-gb; e1809c_v77_gq1008_5647 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034371 = {S37888, 76, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; e1809c_v75_gq1008_9p017 Build/IML74K)*/
const Device UA2190014 = {S37889, 150, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1809c_v75_jbl_5647 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2229288 = {S37890, 150, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1809c_v75_jbl_5647 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2235421 = {S37891, 149, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1809c_v75_jbl_253 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257286 = {S37892, 153, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1809c_v75_gq1008_5640 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2342886 = {S37893, 159, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; e1809c_v75_gq1008_ov5647_uta Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2395848 = {S37894, 77, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; e1809c_v75_gq1008_ov5647 Build/IML74K)*/
const Device UA2426007 = {S37895, 154, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1809c_v75_jbl_253_6628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2445854 = {S37896, 146, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1809c_v75_jdt1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2457522 = {S37897, 151, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1809c_v75_jbl_9p017 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2461253 = {S37898, 154, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1809c_v75_gq1008_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469272 = {S37899, 159, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1809c_v75_gq1008_9p017_6628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2501190 = {S37900, 150, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1809c_v15_zlh_l328 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2507967 = {S37901, 158, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1809c_v75_gq1008_5647_6628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2554196 = {S37902, 163, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; el-gr; e1809c_v75_gq1008_ov5647_turbo_x Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597911 = {S37903, 156, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1809c_v75_jbl_9p017_6628 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2645518 = {S37904, 154, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; e1809c_v77_gq1008_9p017 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2652394 = {S37905, 156, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; e1809c_v75_jbl_253_tronix Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2685873 = {S37906, 152, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1809c_v75_jhgg_9p017 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2686148 = {S37907, 150, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; e1809c_v77_jbl_5647 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2746976 = {S37908, 150, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; e1809c_v75_jbl_5640 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2763457 = {S37909, 152, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; e1809c_v75_hrtm1quest Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2790725 = {S37910, 151, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; e1809c_v75_hlx1_5647 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2817585 = {S37911, 146, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1809c_v75_hlx1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2833240 = {S37912, 75, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; e1809c_v75_gq1008_5640 Build/IML74K)*/
const Device UA3042088 = {S37913, 146, { &P18706,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1809c_v75_jbs1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA251567 = {S37914, 63, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; en) Presto/2.9.168 Version/11.52*/
const Device UA251600 = {S37915, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.9.168 Version/11.52*/
const Device UA251614 = {S37916, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4661); ru) Presto/2.9.168 Version/11.52*/
const Device UA251638 = {S37917, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4876); ru) Presto/2.9.168 Version/11.52*/
const Device UA251646 = {S37918, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03797); ru) Presto/2.9.168 Version/11.52*/
const Device UA251652 = {S37919, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4664); ru) Presto/2.9.168 Version/11.52*/
const Device UA251667 = {S37920, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4139); ru) Presto/2.9.168 Version/11.52*/
const Device UA251670 = {S37921, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4598); ru) Presto/2.9.168 Version/11.52*/
const Device UA251680 = {S37922, 63, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.2; U; ru) Presto/2.9.168 Version/11.52*/
const Device UA251672 = {S37923, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03686); ru) Presto/2.9.168 Version/11.52*/
const Device UA251767 = {S37924, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03796); ru) Presto/2.9.168 Version/11.52*/
const Device UA251791 = {S37925, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4120); ru) Presto/2.9.168 Version/11.52*/
const Device UA251793 = {S37926, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03790); ru) Presto/2.9.168 Version/11.52*/
const Device UA251828 = {S37927, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.9.168 Version/11.52*/
const Device UA251843 = {S37928, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03278); ru) Presto/2.9.168 Version/11.52*/
const Device UA251854 = {S37929, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4832); ru) Presto/2.9.168 Version/11.52*/
const Device UA251876 = {S37930, 100, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.9.168 Version/11.52,gzip(gfe) (via translate.google.com)*/
const Device UA251896 = {S37931, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03757); ru) Presto/2.9.168 Version/11.52*/
const Device UA251983 = {S37932, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4157); ru) Presto/2.9.168 Version/11.52*/
const Device UA251984 = {S37933, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4156); ru) Presto/2.9.168 Version/11.52*/
const Device UA251985 = {S37934, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4841); ru) Presto/2.9.168 Version/11.52*/
const Device UA251995 = {S37935, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4133); ru) Presto/2.9.168 Version/11.52*/
const Device UA251996 = {S37936, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4871); ru) Presto/2.9.168 Version/11.52*/
const Device UA252018 = {S37937, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03773); ru) Presto/2.9.168 Version/11.52*/
const Device UA344511 = {S37938, 66, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; en) Opera 11.52*/
const Device UA344863 = {S37939, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03791); ru) Presto/2.9.168 Version/11.52*/
const Device UA344829 = {S37940, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.4 (build 02647); ru) Presto/2.9.168 Version/11.52*/
const Device UA345262 = {S37941, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03789); ru) Presto/2.9.168 Version/11.52*/
const Device UA345834 = {S37942, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03658); ru) Presto/2.9.168 Version/11.52*/
const Device UA347479 = {S37943, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4930); ru) Presto/2.9.168 Version/11.52*/
const Device UA351728 = {S37944, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4933); ru) Presto/2.9.168 Version/11.52*/
const Device UA590215 = {S37945, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5231); ru) Presto/2.9.168 Version/11.52*/
const Device UA654843 = {S37946, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03755); ru) Presto/2.9.168 Version/11.52*/
const Device UA699673 = {S37947, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4953); ru) Presto/2.9.168 Version/11.52*/
const Device UA737612 = {S37948, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.8 (build 4110); ru) Presto/2.9.168 Version/11.52*/
const Device UA772158 = {S37949, 72, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.52*/
const Device UA789435 = {S37950, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5310); ru) Presto/2.9.168 Version/11.52*/
const Device UA790891 = {S37951, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03672); ru) Presto/2.9.168 Version/11.52*/
const Device UA802322 = {S37952, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03649); ru) Presto/2.9.168 Version/11.52*/
const Device UA807454 = {S37953, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03638); ru) Presto/2.9.168 Version/11.52*/
const Device UA825976 = {S37954, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.6 (build 03402); ru) Presto/2.9.168 Version/11.52*/
const Device UA838451 = {S37955, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5244); ru) Presto/2.9.168 Version/11.52*/
const Device UA895064 = {S37956, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5339); ru) Presto/2.9.168 Version/11.52*/
const Device UA1955060 = {S37957, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5196); ru) Presto/2.9.168 Version/11.52*/
const Device UA1972469 = {S37958, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5680); ru) Presto/2.9.168 Version/11.52*/
const Device UA1988869 = {S37959, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5265); ru) Presto/2.9.168 Version/11.52*/
const Device UA2154722 = {S37960, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5711); ru) Presto/2.9.168 Version/11.52*/
const Device UA2215155 = {S37961, 113, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.7 (build 03639); MRSPUTNIK OW 2, 3, 0, 213; ru) Presto/2.9.168 Version/11.52*/
const Device UA2220073 = {S37962, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5754); ru) Presto/2.9.168 Version/11.52*/
const Device UA2226844 = {S37963, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5972); ru) Presto/2.9.168 Version/11.52*/
const Device UA2303686 = {S37964, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.3 (build 02552); ru) Presto/2.9.168 Version/11.52*/
const Device UA2351694 = {S37965, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5976); ru) Presto/2.9.168 Version/11.52*/
const Device UA2364465 = {S37966, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5282); ru) Presto/2.9.168 Version/11.52*/
const Device UA2403569 = {S37967, 73, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.9.168 Version/11.52,gzip(gfe)*/
const Device UA2474887 = {S37968, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5308); ru) Presto/2.9.168 Version/11.52*/
const Device UA2496666 = {S37969, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5831); ru) Presto/2.9.168 Version/11.52*/
const Device UA2503424 = {S37970, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6001); ru) Presto/2.9.168 Version/11.52*/
const Device UA2524062 = {S37971, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5970); ru) Presto/2.9.168 Version/11.52*/
const Device UA2527747 = {S37972, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6005); ru) Presto/2.9.168 Version/11.52*/
const Device UA2528924 = {S37973, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6015); ru) Presto/2.9.168 Version/11.52*/
const Device UA2667486 = {S37974, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 6068); ru) Presto/2.9.168 Version/11.52*/
const Device UA2762467 = {S37975, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.0 (build 5998); ru) Presto/2.9.168 Version/11.52*/
const Device UA2814138 = {S37976, 86, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.10 (build 5328); ru) Presto/2.9.168 Version/11.52*/
const Device UA2957699 = {S37977, 85, { &P15364,&P5690,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 6.1 (build 6578); ru) Presto/2.9.168 Version/11.52*/
const Device UA20381 = {S37978, 151, { &P15079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonMK16i Build/4.0.A.0.261) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46935 = {S37979, 153, { &P15079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonMK16i Build/4.0.1.A.0.169) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46946 = {S37980, 151, { &P15079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonMK16i Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49747 = {S37981, 153, { &P15079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; SonyEricssonMK16i Build/4.0.1.A.0.277) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60059 = {S37982, 151, { &P15079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; SonyEricssonMK16i Build/4.0.A.2.335) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60068 = {S37983, 151, { &P15079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMK16i Build/4.0.A.2.364) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73564 = {S37984, 153, { &P15079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMK16i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74930 = {S37985, 141, { &P15079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MK16i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA603129 = {S37986, 140, { &P15079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; MK16i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA632830 = {S37987, 152, { &P15079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMK16i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA692432 = {S37988, 64, { &P15079,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; MK16i Build/4.0.2.A.0.62)*/
const Device UA704714 = {S37989, 152, { &P15079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonMK16i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA706033 = {S37990, 152, { &P15079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonMK16i Build/4.0.2.A.0.69) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA712647 = {S37991, 140, { &P15079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MK16i Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA835012 = {S37992, 140, { &P15079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MK16i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1939312 = {S37993, 153, { &P15079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-za; SonyEricssonMK16iv Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024036 = {S37994, 152, { &P15079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonMK16i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024220 = {S37995, 128, { &P15079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MK16i Build/4.0.2.A.0.62) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA623140 = {S37996, 139, { &P15364,&P17819,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/11.10 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19*/
const Device UA624969 = {S37997, 141, { &P15364,&P17819,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/11.10 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19*/
const Device UA1968656 = {S37998, 176, { &P15364,&P17819,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/11.10 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19,gzip(gfe) (via translate.google.com)*/
const Device UA2421839 = {S37999, 149, { &P15364,&P17819,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/11.10 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19,gzip(gfe)*/
const Device UA616723 = {S38000, 132, { &P17168,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7310 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1992886 = {S38001, 132, { &P17168,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7310 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2166767 = {S38002, 61, { &P17168,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7310 Build/IMM76D)*/
const Device UA2381548 = {S38003, 133, { &P17168,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHW-M305W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2592751 = {S38004, 130, { &P17168,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P7310 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2608427 = {S38005, 132, { &P17168,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; GT-P7310 Build/HTJ85B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2608724 = {S38006, 132, { &P17168,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P7310 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2611576 = {S38007, 138, { &P17168,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 4.0.3.0.2.0.1.0; de_DE; GT-P7310 Build/p5wifi-userdebug 4.0.3.0.2.0.1.0 OPENMASTER eng.cainm.X.X test-keys)*/
const Device UA2617021 = {S38008, 273, { &P17168,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; GT-P7310 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/1.9.11;FBDM/{density=1.0,width=1280,height=752};FBLC/de_AT;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P7310;FBSV/4.0.4;]*/
const Device UA2617065 = {S38009, 93, { &P17168,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.youtube/4.1.23(Linux; U; Android 4.0.4; en_US; GT-P7310 Build/IMM76D) gzip*/
const Device UA2617598 = {S38010, 73, { &P17168,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.4; en-US; GT-P7310 Build/IMM76D.UELPB)*/
const Device UA2619824 = {S38011, 100, { &P17168,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.youtube/3.5.5(Linux; U; Android 4.0.4.0.4.0.4; nl_NL; GT-P7310 Build/IMM76D) gzip*/
const Device UA2622573 = {S38012, 134, { &P17168,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7310 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2624338 = {S38013, 75, { &P17168,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-GB; GT-P7310 Build/IMM76D.XXLPO)*/
const Device UA2624414 = {S38014, 130, { &P17168,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7310 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2625130 = {S38015, 132, { &P17168,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7310 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2626623 = {S38016, 128, { &P17168,&P18066,&P18062,&P18092 }};	/*Useragent - Groupon/2.2.2561 (Android 4.0.4; Samsung GT-P7310 / Samsung GT-P7310; )[preload=false;locale=en_US;clientidbase=android-samsung]*/
const Device UA2628236 = {S38017, 139, { &P17168,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-P7310 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2629982 = {S38018, 153, { &P17168,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7310 Build/IMM76D; 1280*752) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2632153 = {S38019, 78, { &P17168,&P18066,&P18062,&P18092 }};	/*Useragent - MediaPlayer/2002/4.2.3 (Android 4.0.4; GT-P7310; GT-P7310; samsung; ; ; en_GB)*/
const Device UA2635662 = {S38020, 145, { &P17168,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7310 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2193295 = {S38021, 141, { &P15767,&P17146,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9A334 Safari/7534.48.3*/
const Device UA2205517 = {S38022, 143, { &P15767,&P17146,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9A405 Safari/7534.48.3*/
const Device UA97133 = {S38023, 166, { &P15364,&P7133,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA818704 = {S38024, 188, { &P15364,&P7133,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Avant Browser)*/
const Device UA829223 = {S38025, 181, { &P15364,&P7133,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Avant Browser)*/
const Device UA1989170 = {S38026, 183, { &P15364,&P7133,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Avant Browser)*/
const Device UA2158045 = {S38027, 78, { &P15364,&P7133,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; Avant Browser)*/
const Device UA2165659 = {S38028, 196, { &P15364,&P7133,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; Avant Browser)*/
const Device UA2166210 = {S38029, 78, { &P15364,&P7133,&P22731,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; Avant Browser)*/
const Device UA2222589 = {S38030, 213, { &P15364,&P7133,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; Avant Browser; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Avant Browser)*/
const Device UA2574352 = {S38031, 218, { &P15364,&P7133,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; Avant Browser; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0; Avant Browser)*/
const Device UA2574638 = {S38032, 160, { &P15364,&P7133,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Avant Browser)*/
const Device UA2641358 = {S38033, 198, { &P15364,&P7133,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Avant Browser)*/
const Device UA2685467 = {S38034, 187, { &P15364,&P7133,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; Avant Browser; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; Avant Browser)*/
const Device UA2692844 = {S38035, 83, { &P15364,&P7133,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; Avant Browser; SV1)*/
const Device UA2722509 = {S38036, 83, { &P15364,&P7133,&P22731,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; SV1; Avant Browser)*/
const Device UA2812271 = {S38037, 93, { &P15364,&P7133,&P22731,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; Avant Browser; Avant Browser)*/
const Device UA2950101 = {S38038, 201, { &P15364,&P7133,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C; .NET4.0E; Avant Browser)*/
const Device UA2951033 = {S38039, 216, { &P15364,&P7133,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MAEM; .NET4.0C; Zune 4.7; Media Center PC 6.0; BOIE9;ENUS; Avant Browser)*/
const Device UA2429872 = {S38040, 61, { &P15575,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-I9100 Build/JOP40D)*/
const Device UA2437312 = {S38041, 139, { &P15575,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9100 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2505109 = {S38042, 140, { &P15575,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9100G Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2510785 = {S38043, 139, { &P15575,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-I9100 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2537078 = {S38044, 60, { &P15575,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9100 Build/JDQ39)*/
const Device UA2562925 = {S38045, 138, { &P15575,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9100 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665744 = {S38046, 139, { &P15575,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9100G Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2717074 = {S38047, 139, { &P15575,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9100 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768785 = {S38048, 140, { &P15575,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9100G Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879217 = {S38049, 131, { &P15575,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9100 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2956219 = {S38050, 62, { &P15575,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9100G Build/JDQ39E)*/
const Device UA3121474 = {S38051, 173, { &P15575,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9100 Build/ThunderBolt ROM V8 By ALHussein Mohammed) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA902368 = {S38052, 70, { &P15364,&P16110,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:13.0) Gecko/X Firefox/13.0*/
const Device UA902776 = {S38053, 67, { &P15364,&P16110,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:13.0) Gecko/X Firefox/13.0.1*/
const Device UA2237704 = {S38054, 57, { &P15364,&P16110,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2) Presto/2.12.388 Version/12.10*/
const Device UA2238941 = {S38055, 64, { &P15364,&P16110,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.10*/
const Device UA2361082 = {S38056, 86, { &P15364,&P16110,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5998)) Presto/2.12.388 Version/12.10*/
const Device UA2424522 = {S38057, 86, { &P15364,&P16110,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5784)) Presto/2.12.388 Version/12.10*/
const Device UA2456515 = {S38058, 86, { &P15364,&P16110,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.10*/
const Device UA2594895 = {S38059, 86, { &P15364,&P16110,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 5976)) Presto/2.12.388 Version/12.10*/
const Device UA2841334 = {S38060, 74, { &P15364,&P16110,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.10,gzip(gfe)*/
const Device UA1948425 = {S38061, 56, { &P15364,&P18346,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.10; SV1)*/
const Device UA1955073 = {S38062, 64, { &P15364,&P18346,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.10; Trident/4.0)*/
const Device UA1993167 = {S38063, 78, { &P15364,&P18346,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Alexa Toolbar)*/
const Device UA2155329 = {S38064, 68, { &P15364,&P18346,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; SV1)*/
const Device UA2162410 = {S38065, 70, { &P15364,&P18346,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; IEMB3)*/
const Device UA2167790 = {S38066, 68, { &P15364,&P18346,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; SV1)*/
const Device UA2440464 = {S38067, 98, { &P15364,&P18346,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.10; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)*/
const Device UA2475116 = {S38068, 152, { &P15364,&P18346,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.10; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA2822427 = {S38069, 73, { &P15364,&P18346,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; SV1; SV1)*/
const Device UA2834566 = {S38070, 179, { &P15364,&P18346,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.10; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2950302 = {S38071, 73, { &P15364,&P18346,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; SV1; SV1)*/
const Device UA1961206 = {S38072, 149, { &P18228,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; ALCATEL ONE TOUCH 991 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1961626 = {S38073, 150, { &P18228,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; ALCATEL ONE TOUCH 991D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2204153 = {S38074, 156, { &P18228,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; ALCATEL_ONE_TOUCH_991_Orange Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257057 = {S38075, 150, { &P18228,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ALCATEL ONE TOUCH 991S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2413426 = {S38076, 149, { &P18228,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; ALCATEL_one_touch_991 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2600655 = {S38077, 150, { &P18228,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; ALCATEL ONE TOUCH 991T Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2750712 = {S38078, 150, { &P18228,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; ALCATEL_ONE_TOUCH_991A Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2821016 = {S38079, 150, { &P18228,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; ALCATEL ONE TOUCH 991A Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3119349 = {S38080, 149, { &P18228,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; ALCATEL_ONE_TOUCH_991 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3016 = {S38081, 112, { &P15518,&P17484,&P3636,&P18092 }};	/*Useragent - SAMSUNG-GT-S5230/1.0 SHP/VPP/R5 Jasmine/0.8 NexPlayer/2.12 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA60281 = {S38082, 125, { &P15518,&P17484,&P3636,&P18092 }};	/*Useragent - SAMSUNG-GT-S5230N-ORANGE/S5230NBVKD1 SHP/VPP/R5 Jasmine/0.8 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA162364 = {S38083, 145, { &P15364,&P17817,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Ubuntu/10.10 Chromium/8.0.552.224 Chrome/8.0.552.224 Safari/534.10*/
const Device UA434497 = {S38084, 120, { &P15364,&P17817,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/540.0 (KHTML, like Gecko) Ubuntu/10.10 Chrome/8.1.0.0 Safari/540.0*/
const Device UA507293 = {S38085, 147, { &P15364,&P17817,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Ubuntu/10.10 Chromium/8.0.552.237 Chrome/8.0.552.237 Safari/534.10*/
const Device UA577780 = {S38086, 60, { &P15364,&P17285,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:8.0.1) Gecko/X Firefox/8.0.1*/
const Device UA582134 = {S38087, 100, { &P15364,&P17285,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.3) Gecko/X Firefox/8.0 (.NET CLR 3.5.30729)*/
const Device UA731105 = {S38088, 72, { &P15364,&P17285,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:8.0.1) Gecko/X Firefox/8.0.1*/
const Device UA795454 = {S38089, 68, { &P15364,&P17285,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:8.0) Gecko/X Firefox/8.0 Firefox/8.0*/
const Device UA2423778 = {S38090, 72, { &P15364,&P5690,&P19806,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:17.0) Gecko/17.0 Thunderbird/17.0*/
const Device UA3032610 = {S38091, 64, { &P15364,&P5690,&P19806,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:17.0) Gecko/X Thunderbird/17.0.8*/
const Device UA138777 = {S38092, 170, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Maxthon; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)*/
const Device UA417059 = {S38093, 139, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Maxthon; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729)*/
const Device UA469062 = {S38094, 175, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Maxthon; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)*/
const Device UA528583 = {S38095, 83, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; SV1; .NET CLR 1.1.4322; MAXTHON)*/
const Device UA680327 = {S38096, 84, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Maxthon; Win64; x64; Trident/4.0)*/
const Device UA696409 = {S38097, 64, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; SV1; Maxthon)*/
const Device UA706143 = {S38098, 160, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Maxthon; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA817071 = {S38099, 64, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; SV1; MAXTHON)*/
const Device UA1858185 = {S38100, 156, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; Maxthon; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)*/
const Device UA1869959 = {S38101, 83, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; SV1; .NET CLR 1.1.4322; MAXTHON)*/
const Device UA1941860 = {S38102, 72, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; MAXTHON)*/
const Device UA1947050 = {S38103, 64, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; SV1; MAXTHON)*/
const Device UA1960064 = {S38104, 72, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; Maxthon)*/
const Device UA1993799 = {S38105, 72, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; Maxthon)*/
const Device UA2150421 = {S38106, 160, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAXTHON)*/
const Device UA2152631 = {S38107, 92, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; .NET CLR 2.0.50727; MAXTHON)*/
const Device UA2154343 = {S38108, 72, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 6.1; Maxthon)*/
const Device UA2157535 = {S38109, 72, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; Maxthon)*/
const Device UA2166470 = {S38110, 72, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; Maxthon)*/
const Device UA2218998 = {S38111, 106, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)*/
const Device UA2256763 = {S38112, 160, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAXTHON)*/
const Device UA2309998 = {S38113, 72, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; Trident/4.0; Maxthon)*/
const Device UA2443606 = {S38114, 187, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MAXTHON)*/
const Device UA2494270 = {S38115, 91, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; .NET CLR 1.1.4322; MAXTHON)*/
const Device UA2520353 = {S38116, 167, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAXTHON)*/
const Device UA2553953 = {S38117, 106, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)*/
const Device UA2555060 = {S38118, 107, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)*/
const Device UA2594817 = {S38119, 72, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; MAXTHON)*/
const Device UA2686966 = {S38120, 87, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; Avant Browser; Maxthon)*/
const Device UA2723497 = {S38121, 77, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; SV1; Maxthon)*/
const Device UA2741632 = {S38122, 94, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/en-us  Safari/537.1*/
const Device UA2743314 = {S38123, 87, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; Avant Browser; Maxthon)*/
const Device UA2746889 = {S38124, 65, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.1; SV1; MAXTHON)*/
const Device UA2786909 = {S38125, 85, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; SV1; TencentTraveler 4.0; Maxthon)*/
const Device UA2787743 = {S38126, 81, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; Maxthon; Maxthon)*/
const Device UA2790196 = {S38127, 160, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAXTHON)*/
const Device UA2826232 = {S38128, 283, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; Maxthon; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; EmbeddedWB 14.52 from: http://www.bsalsa.com/ EmbeddedWB 14.52; .NET CLR 2.0.50727; .NET4.0C; .NET4.0E; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; KBX)*/
const Device UA2830140 = {S38129, 77, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; Maxthon; SV1)*/
const Device UA2837723 = {S38130, 73, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; MAXTHON)*/
const Device UA2840895 = {S38131, 187, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MAXTHON)*/
const Device UA2933476 = {S38132, 81, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; Maxthon; Maxthon)*/
const Device UA2933801 = {S38133, 81, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; Maxthon; Maxthon)*/
const Device UA2949704 = {S38134, 213, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; GTB6.6; Maxthon; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C; MASN; AskTB5.6)*/
const Device UA2950234 = {S38135, 87, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; Avant Browser; Maxthon)*/
const Device UA2950428 = {S38136, 77, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.1; SV1; Maxthon)*/
const Device UA2980869 = {S38137, 87, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; Avant Browser; Maxthon)*/
const Device UA2981511 = {S38138, 81, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.1; Maxthon; Maxthon)*/
const Device UA3116635 = {S38139, 179, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; Maxthon; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3)*/
const Device UA3118082 = {S38140, 152, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MALN; MAXTHON)*/
const Device UA3118393 = {S38141, 188, { &P15364,&P7133,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MAXTHON)*/
const Device UA401560 = {S38142, 166, { &P17968,&P5629,&P15760,&P18091 }};	/*Useragent - Mozilla/5.0 (Linux;u;Android 2.3.7;zh-cn;) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Mobile Safari/533.1 (compatible; +http://www.baidu.com/search/spider.html)*/
const Device UA94676 = {S38143, 47, { &P15364,&P17285,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 6.0; U; en) Presto/2.1.1*/
const Device UA101660 = {S38144, 47, { &P15364,&P17285,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 6.0; U; en) Presto/2.1.1*/
const Device UA101896 = {S38145, 47, { &P15364,&P17285,&P18337,&P18092 }};	/*Useragent - Opera/9.61 (Windows NT 6.0; U; en) Presto/2.1.1*/
const Device UA126674 = {S38146, 47, { &P15364,&P17285,&P18337,&P18092 }};	/*Useragent - Opera/9.62 (Windows NT 6.0; U; en) Presto/2.1.1*/
const Device UA549232 = {S38147, 70, { &P15364,&P17285,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.1.1*/
const Device UA841647 = {S38148, 47, { &P15364,&P17285,&P18337,&P18092 }};	/*Useragent - Opera/9.60 (Windows NT 6.0; U; bg) Presto/2.1.1*/
const Device UA1872763 = {S38149, 70, { &P15364,&P17285,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.1.1*/
const Device UA70648 = {S38150, 141, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Runnymede Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73340 = {S38151, 141, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Runnymede Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA251967 = {S38152, 167, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Sensation XL with Beats Audio X315e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA371320 = {S38153, 165, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; tr-; HTC Sensation XL with Beats Audio X315e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA379597 = {S38154, 141, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Runnymede Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA575028 = {S38155, 141, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Runnymede Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622955 = {S38156, 155, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC_SensationXL_Beats_X315b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA629343 = {S38157, 167, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC Sensation XL with Beats Audio X315b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA631605 = {S38158, 155, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC_SensationXL_Beats_X315e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633575 = {S38159, 160, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC/SensationXL_Beats/1.05.161.4 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA634166 = {S38160, 91, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Sensation XL with Beats Audio X315e Build/GRJ90)*/
const Device UA644520 = {S38161, 149, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; HTC_SensationXL_Beats Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA775229 = {S38162, 159, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC_SensationXL_Beats-orange-LS Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA800172 = {S38163, 155, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Sensation XL with Beats Audio X315e Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA800695 = {S38164, 137, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-kr; HTC-X315E Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA829364 = {S38165, 157, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; Sensation XL with Beats Audio Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1942372 = {S38166, 91, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Sensation XL with Beats Audio X315b Build/GRJ90)*/
const Device UA1988637 = {S38167, 160, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Sensation XL with Beats Audio X315e Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2020143 = {S38168, 65, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Runnymede Build/GRJ90)*/
const Device UA2031767 = {S38169, 155, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; HTC Sensation XL with Beats Audio X315b Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2033418 = {S38170, 137, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC_X315e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2643255 = {S38171, 160, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Sensation XL with Beats Audio X315b Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2684666 = {S38172, 61, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC-X315E Build/GRJ90)*/
const Device UA2721394 = {S38173, 130, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC X315e Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2786479 = {S38174, 137, { &P15794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC X315e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3064644 = {S38175, 128, { &P18639,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus 10 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2741711 = {S38176, 137, { &P18471,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-66) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753567 = {S38177, 143, { &P18471,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT905 Build/9.8.2Q_SMUIRD-7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755135 = {S38178, 139, { &P18471,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XT907 Build/9.8.1Q_27-2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA94043 = {S38179, 63, { &P15364,&P21445,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows ME) Opera 7.11  [en]*/
const Device UA306386 = {S38180, 31, { &P15364,&P21445,&P4810,&P18092 }};	/*Useragent - Opera/7.50 (Windows ME; U) [en]*/
const Device UA492839 = {S38181, 62, { &P15364,&P21445,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows ME) Opera 7.54 [en]*/
const Device UA630389 = {S38182, 72, { &P15364,&P21445,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows ME) Opera 7.03 [de]*/
const Device UA888487 = {S38183, 62, { &P15364,&P21445,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows ME) Opera 7.53 [en]*/
const Device UA2275259 = {S38184, 72, { &P15364,&P21445,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows ME) Opera 7.02 [en]*/
const Device UA2504961 = {S38185, 46, { &P15364,&P21445,&P4810,&P18092 }};	/*Useragent - Opera 7.5 (Win ME): Opera/7.50 (Windows ME; U)*/
const Device UA2682627 = {S38186, 62, { &P15364,&P21445,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows ME) Opera 7.23 [ru]*/
const Device UA2844845 = {S38187, 32, { &P15364,&P21445,&P4810,&P18092 }};	/*Useragent - Opera/7.22 (Windows ME; U)  [en]*/
const Device UA436211 = {S38188, 108, { &P15364,&P16110,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30*/
const Device UA724802 = {S38189, 96, { &P15364,&P16110,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows 8) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30*/
const Device UA784362 = {S38190, 99, { &P15364,&P16110,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.27 (KHTML, like Gecko) Chrome/12.0.712.0 Safari/534.27*/
const Device UA2589351 = {S38191, 132, { &P15364,&P16110,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/12.0.702.0 Safari/533.3 Awesomium/1.6.5*/
const Device UA2933591 = {S38192, 95, { &P15364,&P16110,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows 8) AppleWebKit/534.30 (KHTML like Gecko) Chrome/12.0.742.112 Safari/534.30*/
const Device UA198504 = {S38193, 143, { &P17952,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA208023 = {S38194, 67, { &P17952,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M110S Build/GINGERBREAD)*/
const Device UA235932 = {S38195, 143, { &P17952,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA350301 = {S38196, 143, { &P17952,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA354715 = {S38197, 67, { &P17952,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHW-M110S Build/GINGERBREAD)*/
const Device UA393281 = {S38198, 67, { &P17952,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SHW-M110S Build/GINGERBREAD)*/
const Device UA397352 = {S38199, 143, { &P17952,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA643195 = {S38200, 67, { &P17952,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SHW-M110S Build/GINGERBREAD)*/
const Device UA1970550 = {S38201, 145, { &P17952,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SHW-M110S Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018970 = {S38202, 145, { &P17952,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; SHW-M110S Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019346 = {S38203, 145, { &P17952,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; SHW-M110S Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2024958 = {S38204, 145, { &P17952,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SHW-M110S Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2026260 = {S38205, 131, { &P17952,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; SHW-M110S Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2445176 = {S38206, 136, { &P17952,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; SHW-M110S Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2445297 = {S38207, 141, { &P17952,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2474671 = {S38208, 136, { &P17952,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; SHW-M110S Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2573548 = {S38209, 141, { &P17952,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2722070 = {S38210, 136, { &P17952,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SHW-M110S Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2779687 = {S38211, 142, { &P17952,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M110S Build/??????????) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2935039 = {S38211, 142, { &P17952,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M110S Build/??????????) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2958441 = {S38212, 143, { &P17952,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2232042 = {S38213, 136, { &P12605,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XT875 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2355403 = {S38214, 136, { &P12605,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT875 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555576 = {S38215, 143, { &P12605,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID BIONIC Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2670773 = {S38216, 153, { &P12605,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2679626 = {S38217, 75, { &P12605,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22)*/
const Device UA2717663 = {S38218, 146, { &P12605,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2795704 = {S38219, 151, { &P12605,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2866689 = {S38220, 146, { &P12605,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID BIONIC Build/Blurry JB) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA349263 = {S38221, 152, { &P18599,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-R680 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 USCC-R680*/
const Device UA636228 = {S38222, 142, { &P18599,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-R680 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA787460 = {S38223, 66, { &P18599,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SCH-R680 Build/GINGERBREAD)*/
const Device UA2024323 = {S38224, 66, { &P18599,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-R680 Build/GINGERBREAD)*/
const Device UA600539 = {S38225, 74, { &P17595,&P17296,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:12.0) Gecko/X Firefox/12.0*/
const Device UA810750 = {S38226, 87, { &P17595,&P17296,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:12.0) Gecko/X Firefox/12.0 Firefox/12.0*/
const Device UA22167 = {S38227, 141, { &P15390,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P925/V08g Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA33221 = {S38228, 141, { &P15390,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P925/V10a Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA45291 = {S38229, 136, { &P15390,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P925 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA45292 = {S38230, 141, { &P15390,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P925/V10d Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA45294 = {S38231, 141, { &P15390,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P925/V10e Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA379434 = {S38232, 60, { &P15390,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P925 Build/FRG83G)*/
const Device UA2019037 = {S38233, 141, { &P15390,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P925/V09c Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021720 = {S38234, 139, { &P15390,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-; LG-P925/V10e Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2809781 = {S38235, 68, { &P15390,&P5096,&P3508,&P18092 }};	/*Useragent - CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 2.2) lge LG-P925 SW:V20p*/
const Device UA2717565 = {S38236, 140, { &P17346,&P19826,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B329 Safari/8536.25*/
const Device UA2717566 = {S38237, 140, { &P17346,&P19826,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B146 Safari/8536.25*/
const Device UA2717585 = {S38238, 138, { &P17346,&P19826,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B144 Safari/8536.25*/
const Device UA2717864 = {S38239, 138, { &P17346,&P19826,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10B141 Safari/8536.25*/
const Device UA94190 = {S38240, 47, { &P15364,&P6729,&P18068,&P18092 }};	/*Useragent - Mozilla/1.22 (compatible; MSIE 2.0; Windows 95)*/
const Device UA2659005 = {S38241, 54, { &P15364,&P6729,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; Win95; Trident/2.0)*/
const Device UA2792797 = {S38242, 54, { &P15364,&P6729,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; Win95; Trident/5.0)*/
const Device UA623083 = {S38243, 67, { &P15364,&P18118,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:11.0) Gecko/X Firefox/11.0*/
const Device UA860000 = {S38244, 80, { &P15364,&P18118,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:11.0) Gecko/X Firefox/11.0 Firefox/11.0*/
const Device UA2509011 = {S38245, 145, { &P18453,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; SonyLT25i Build/9.1.A.0.490) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2582361 = {S38246, 141, { &P18453,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT25i Build/9.1.A.0.490) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2604425 = {S38247, 145, { &P18453,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; SonyLT25i Build/9.1.A.0.492) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2673699 = {S38248, 63, { &P18453,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LT25i Build/9.1.A.1.140)*/
const Device UA2675958 = {S38249, 145, { &P18453,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; SonyLT25i Build/9.1.A.1.140) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2706589 = {S38250, 141, { &P18453,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; LT25i Build/9.1.A.1.140) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2770288 = {S38251, 145, { &P18453,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; SonyLT25i Build/9.1.A.1.142) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2853365 = {S38252, 141, { &P18453,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; LT25i Build/9.1.A.0.492) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2944945 = {S38253, 145, { &P18453,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-sg; SonyLT25i Build/9.1.A.1.145) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998314 = {S38254, 141, { &P18453,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; LT25i Build/9.1.A.1.142) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3019545 = {S38255, 141, { &P18453,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-sg; LT25i Build/9.1.A.1.145) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3073250 = {S38256, 65, { &P18453,&P18111,&P18112,&P18092 }};	/*Useragent - SonyLT25i Build/9.1.A.1.140 stagefright/1.2 (Linux;Android 4.1.2)*/
const Device UA614980 = {S38257, 120, { &P17595,&P18207,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA2002720 = {S38258, 120, { &P17595,&P18207,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.163 Safari/535.19*/
const Device UA2201425 = {S38259, 120, { &P17595,&P18207,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA2381793 = {S38260, 120, { &P17595,&P18207,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/180.1.1025.40 Safari/535.19*/
const Device UA2580104 = {S38261, 120, { &P17595,&P18207,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA2695207 = {S38262, 99, { &P17595,&P18207,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; AMD Mac OS X 10_8_2) AppleWebKit/535.22 (KHTML, like Gecko) Chrome/18.6.872*/
const Device UA2750182 = {S38263, 120, { &P17595,&P18207,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/180.1.1025.40 Safari/535.19*/
const Device UA2797422 = {S38264, 120, { &P17595,&P18207,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA3107582 = {S38265, 120, { &P17595,&P18207,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA2717096 = {S38266, 134, { &P15767,&P18117,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9B206 Safari/7534.48.3*/
const Device UA2717863 = {S38267, 132, { &P15767,&P18117,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9B176 Safari/7534.48.3*/
const Device UA2769957 = {S38268, 132, { &P15767,&P18117,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9A405 Safari/7534.48.3*/
const Device UA49301 = {S38269, 142, { &P14959,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA77401 = {S38270, 142, { &P14959,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA636718 = {S38271, 66, { &P14959,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-I9103 Build/GINGERBREAD)*/
const Device UA677302 = {S38272, 66, { &P14959,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-I9103 Build/GINGERBREAD)*/
const Device UA748978 = {S38273, 142, { &P14959,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA769512 = {S38274, 130, { &P14959,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9103 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1904221 = {S38275, 66, { &P14959,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9103 Build/GINGERBREAD)*/
const Device UA1942969 = {S38276, 130, { &P14959,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9103 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1958064 = {S38277, 135, { &P14959,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9103 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2027328 = {S38278, 135, { &P14959,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9103 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2028726 = {S38279, 135, { &P14959,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9103 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2463017 = {S38280, 140, { &P14959,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1996307 = {S38281, 131, { &P12628,&P7106,&P15829,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9350; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.580 Mobile Safari/534.11+*/
const Device UA1989619 = {S38282, 132, { &P18702,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-N8013 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2030069 = {S38283, 61, { &P18702,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-N8013 Build/IMM76D)*/
const Device UA2347450 = {S38284, 130, { &P18702,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-N8013 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA908646 = {S38285, 132, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2034571 = {S38286, 61, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P6800 Build/IMM76D)*/
const Device UA2383596 = {S38287, 130, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2608329 = {S38288, 153, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2609163 = {S38289, 132, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; GT-P6800 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2612386 = {S38290, 130, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2614578 = {S38291, 118, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.0.4; cs-CZ; GT-P6800 Build/cm_p6800-userdebug 4.0.4 IMM76L eng.locerra.X.X test-keys)*/
const Device UA2615290 = {S38292, 73, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.0.4; en-US; GT-P6800 Build/IMM76D.DXLP7)*/
const Device UA2615892 = {S38293, 130, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2616350 = {S38294, 139, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2616382 = {S38295, 94, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.apps.plus/X (Linux; U; Android 4.0.4; en_US; GT-P6800; Build/IMM76D) (gzip)*/
const Device UA2616579 = {S38296, 145, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2616996 = {S38297, 149, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D, compatible iPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617778 = {S38298, 132, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6800) antivirusHttpClient/2.0 (KHTML, like Gecko) Version/Version 2.3/Antivirus pro*/
const Device UA2618188 = {S38299, 73, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.4; en-GB; GT-P6800 Build/IMM76D.XXLQ1)*/
const Device UA2618650 = {S38300, 125, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2618651 = {S38301, 134, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.41.0; en-gb; GT-P6800) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2619124 = {S38302, 103, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 4.0.4; SAMSUNG GT-P6800 Build/ICE_CREAM_SANDWICH_MR1);*/
const Device UA2619151 = {S38303, 73, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.4; pt-BR; GT-P6800 Build/IMM76D.UBLP5)*/
const Device UA2619567 = {S38304, 162, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6800 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2619816 = {S38305, 73, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.4; zh-HK; GT-P6800 Build/IMM76D.ZSLP7)*/
const Device UA2621336 = {S38306, 50, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - TapSonic 1.4.0 (Android 4.0.4 , GT-P6800, samsung)*/
const Device UA2621350 = {S38307, 91, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6800 Build/IMM76D) Daily Mail Online/1.8.3*/
const Device UA2621573 = {S38308, 92, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Vyrso/2.2.7 (Linux; U; Android 4.0.4; en-GB; GT-P6800 Build/SkyHigh ROM P6800DXLP8 ICS v3.5)*/
const Device UA2622893 = {S38309, 75, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-US; GT-P6800 Build/IMM76D.DDLP5)*/
const Device UA2624192 = {S38310, 128, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Groupon/2.1.2486 (Android 4.0.4; Samsung GT-P6800 / Samsung GT-P6800; )[preload=false;locale=en_GB;clientidbase=android-samsung]*/
const Device UA2624289 = {S38311, 75, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-GB; GT-P6800 Build/IMM76D.DXLP8)*/
const Device UA2626246 = {S38312, 262, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 parameters/{scr=1232_800,cm=1,ql=l,imei=X,bm=1,eng=adrmini15,mdm=1,mdl=GT-P6800,ps=1} tiantian(securitypay) tiantian(tenpay)*/
const Device UA2628138 = {S38313, 258, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GT-P6800 Build/IMM76D) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 baidubrowser/061_0.41.3.2_diordna_2321_008/gnusmas_51_4.0.4_0086P-TG/1092a/E46AA9836BAFXF4290E099E6A%7CX/1*/
const Device UA2629036 = {S38314, 129, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P6800 Build/Xqm) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2635917 = {S38315, 288, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 NAVER(inapp; cafe; 100; 2.9.10) com.nhn.android.navercafe/2.9.10 (Linux; U; Android 4.0.4; ko-kr; GT-P6800 Build/IMM76D; samsung) 800X1232 samsung GT-P6800*/
const Device UA2636069 = {S38316, 134, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6800 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2636818 = {S38317, 59, { &P17323,&P18066,&P18062,&P18092 }};	/*Useragent - nApps (Android 4.0.4; GT-P6800; androidapp.lineplay; 1.2.0)*/
const Device UA23020 = {S38318, 144, { &P15555,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; myTouch_4G_Slide Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47352 = {S38319, 143, { &P15555,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; myTouch_4G_Slide Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA47353 = {S38320, 143, { &P15555,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; myTouch_4G_Slide Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47358 = {S38321, 143, { &P15555,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;myTouch_4G_Slide Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47359 = {S38322, 143, { &P15555,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; myTouch_4G_Slide Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47363 = {S38323, 143, { &P15555,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; myTouch_4G_SlideBuild/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60326 = {S38324, 144, { &P15555,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; myTouch_4G_Slide Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369325 = {S38325, 74, { &P15555,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.4; en_US; myTouch_4G_Slide Build/GRJ22)*/
const Device UA378427 = {S38326, 68, { &P15555,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; myTouch_4G_Slide Build/GRJ22)*/
const Device UA693795 = {S38327, 144, { &P15555,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; myTouch_4G_Slide Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019362 = {S38328, 151, { &P15555,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; myTouch_4G_Slide Build/GRJ22) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2019950 = {S38329, 142, { &P15555,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; myTouch_4G_Slide Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2205347 = {S38330, 149, { &P17346,&P18432,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A403 Safari/7534.48.3*/
const Device UA2238046 = {S38331, 149, { &P17346,&P18432,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A406 Safari/7534.48.3*/
const Device UA2261382 = {S38332, 151, { &P17346,&P18432,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10A523 Safari/7534.48.3*/
const Device UA2597838 = {S38333, 148, { &P17346,&P18432,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 601 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/21.0.1180.82 Mobile/10A523 Safari/7534.48.3*/
const Device UA2600524 = {S38334, 147, { &P17346,&P18432,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 60 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/21.0.1180.82 Mobile/10A403 Safari/7534.48.3*/
const Device UA601782 = {S38335, 58, { &P15364,&P7127,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:12.0) Gecko/X Firefox/12.0*/
const Device UA2364787 = {S38336, 71, { &P15364,&P7127,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:12.0) Gecko/X Firefox/12.0 Firefox/12.0*/
const Device UA33314 = {S38337, 123, { &P14711,&P5204,&P4030,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA2371208 = {S38338, 73, { &P14711,&P5204,&P4030,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (11.20) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-01*/
const Device UA3099520 = {S38339, 186, { &P14711,&P5204,&P4030,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (11.20) Profile/MIDP-2.1 Configuration/CLDC-1.1 NokiaC2-01/5.0 (11.20) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA11502 = {S38340, 152, { &P16331,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HUAWEI-M860 Build/ERE27) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA26635 = {S38341, 147, { &P16331,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HUAWEI-M860 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA39977 = {S38342, 185, { &P16331,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; M860 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 androidAppVersion/2.0.3 AndroidWebkitWrapper*/
const Device UA53517 = {S38343, 92, { &P16331,&P5477,&P4861,&P18092 }};	/*Useragent - Huawei-M860/M860V100R001C153B251SP03 Player/OpenCORE-Player/2.02 (Linux;Android 2.1-update1)*/
const Device UA53519 = {S38344, 138, { &P16331,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; M860 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA55522 = {S38345, 95, { &P16331,&P5477,&P4861,&P18092 }};	/*Useragent - Huawei-HUAWEI-M860/M860V100R001C177B261 Player/OpenCORE-Player/2.02 (Linux;Android 2.1-update1)*/
const Device UA62383 = {S38346, 140, { &P16331,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; M860 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA79300 = {S38347, 183, { &P16331,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; M860 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 androidAppVersion/2.1 AndroidWebkitWrapper*/
const Device UA162766 = {S38348, 62, { &P16331,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; M860 Build/ERE27)*/
const Device UA165234 = {S38349, 69, { &P16331,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; HUAWEI-M860 Build/ERE27)*/
const Device UA2018266 = {S38350, 145, { &P16331,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; M860 Build/ERE27) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA12693 = {S38351, 134, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; ADR6200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 MobileSafari/530.17*/
const Device UA12697 = {S38352, 135, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; ADR6200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA12702 = {S38353, 143, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; ADR6200 Build/Flash2Talk V3) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA28165 = {S38354, 148, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Desire_A8181 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA28170 = {S38355, 152, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-tw; HTC_Desire_A8181 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA28244 = {S38356, 148, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; desire_A8181 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA29107 = {S38357, 134, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us;ADR6200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA29803 = {S38358, 157, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Desire 1.24.161.3 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA29807 = {S38359, 157, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Desire 1.15.161.4 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA29808 = {S38360, 146, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC_Desire Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA29809 = {S38361, 157, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; HTC Desire 1.19.161.5 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA29813 = {S38362, 145, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Bravo Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA29848 = {S38363, 157, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; de-at; HTC Desire 1.21.163.2 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA31259 = {S38364, 146, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Desire Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32114 = {S38365, 144, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; HTCX06HT Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32118 = {S38366, 164, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; HTC_Desire_A8183 V1.16.841.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32119 = {S38367, 165, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; HTC_Desire_A8183 V1.15.841.14 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32120 = {S38368, 164, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; HTC_Desire_A8183 V1.25.841.5 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32128 = {S38369, 165, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; HTC_Desire_A8183 V1.15.841.13 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA37101 = {S38370, 157, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Desire 1.15.161.3 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA45692 = {S38371, 137, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-gb; HTC Bravo Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA54005 = {S38372, 147, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HTC Bravo_C Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA64323 = {S38373, 156, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC_Desire-orange-LS Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA66546 = {S38374, 134, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; ADR6200Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA66549 = {S38375, 134, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.1; en-us; ADR6200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA66554 = {S38376, 133, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; -us; ADR6200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA66544 = {S38377, 134, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.1; en-us; ADR6200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA93547 = {S38378, 68, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; HTC Desire Build/ERE27)*/
const Device UA2018717 = {S38379, 138, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; HTC Desire Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2025294 = {S38380, 155, { &P11747,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; HTC Desire 1.19.161.5 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.*/
const Device UA456572 = {S38381, 74, { &P17595,&P17295,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:12.0) Gecko/X Firefox/12.0*/
const Device UA2415906 = {S38382, 131, { &P16976,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AT300SE Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2425399 = {S38383, 129, { &P16976,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AT300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2483622 = {S38384, 60, { &P16976,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; AT300SE Build/JRO03C)*/
const Device UA2506870 = {S38385, 58, { &P16976,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; AT300 Build/JRO03C)*/
const Device UA2257685 = {S38386, 143, { &P18504,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LS840 Build/ZVI.IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2516541 = {S38387, 143, { &P18504,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LS840 Build/ZVK.IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2536621 = {S38388, 65, { &P18504,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-LS840 Build/ZVK.IMM76D)*/
const Device UA2659422 = {S38389, 143, { &P18504,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LS840 Build/ZVJ.IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2991524 = {S38390, 143, { &P18504,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LS840 Build/ZVM.IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2429873 = {S38391, 161, { &P16292,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2515261 = {S38392, 161, { &P16292,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2534068 = {S38393, 160, { &P16292,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2717932 = {S38394, 163, { &P16292,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996094 = {S38395, 142, { &P20408,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ZTE V768 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2236031 = {S38396, 66, { &P20408,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; ZTE V768 Build/GINGERBREAD)*/
const Device UA2415492 = {S38397, 141, { &P20408,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ZTE V768 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA2450928 = {S38398, 141, { &P20408,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us;ZTE V768 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2453651 = {S38399, 141, { &P20408,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.7; en-us; ZTE V768 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2465974 = {S38400, 141, { &P20408,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ZTEV768 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2478949 = {S38401, 141, { &P20408,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.7; en-us; ZTE V768 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2500937 = {S38402, 141, { &P20408,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ZTE V768Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2973941 = {S38403, 136, { &P21648,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974795 = {S38404, 137, { &P21648,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505G Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA797245 = {S38405, 62, { &P15364,&P7133,&P18374,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; en) Presto/2.6.31 Version/10.70*/
const Device UA381219 = {S38406, 27, { &P18254,&P7480,&P18245,&P18116 }};	/*Useragent - Acoon v4.9.5 (www.acoon.de)*/
const Device UA398811 = {S38407, 28, { &P18254,&P7480,&P18245,&P18116 }};	/*Useragent - Acoon v4.10.1 (www.acoon.de)*/
const Device UA895510 = {S38408, 73, { &P18254,&P7480,&P18245,&P18116 }};	/*Useragent - Mozilla/5.0 (compatible; AcoonBot/4.11.0; +http://www.acoon.de/robot.asp)*/
const Device UA902644 = {S38409, 73, { &P18254,&P7480,&P18245,&P18116 }};	/*Useragent - Mozilla/5.0 (compatible; AcoonBot/4.11.1; +http://www.acoon.de/robot.asp)*/
const Device UA187413 = {S38410, 147, { &P15364,&P17817,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Ubuntu/10.10 Chromium/10.0.648.133 Chrome/10.0.648.133 Safari/534.16*/
const Device UA639395 = {S38411, 149, { &P15364,&P17817,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Ubuntu/10.10 Chromium/10.0.648.127 Chrome/10.0.648.127 Safari/534.16*/
const Device UA1876285 = {S38412, 149, { &P15364,&P17817,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux armv7l; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Ubuntu/10.10 Chromium/10.0.648.205 Chrome/10.0.648.205 Safari/534.16*/
const Device UA252011 = {S38413, 102, { &P15364,&P18110,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2*/
const Device UA252559 = {S38414, 109, { &P15364,&P18110,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.106 Safari/535.2*/
const Device UA252834 = {S38415, 144, { &P15364,&P18110,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Jolicloud Linux i686) AppleWebKit/535.2 (KHTML, like Gecko) Joli OS/1.2 Chromium/15.0.874.106 Chrome/15.0.874.106 Safari/535.2*/
const Device UA255777 = {S38416, 100, { &P15364,&P18110,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA2674508 = {S38417, 106, { &P15364,&P18110,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/15.0.1410.63 Safari/537.31*/
const Device UA2674511 = {S38418, 106, { &P15364,&P18110,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/15.0.1410.63 Safari/537.31*/
const Device UA2674524 = {S38419, 109, { &P15364,&P18110,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/15.0.1410.63 Safari/537.31*/
const Device UA2674651 = {S38420, 109, { &P15364,&P18110,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/15.0.1410.63 Safari/537.31*/
const Device UA2674864 = {S38421, 106, { &P15364,&P18110,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/15.0.1410.63 Safari/537.31*/
const Device UA2714741 = {S38422, 109, { &P15364,&P18110,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/15.0.1410.63 Safari/537.31*/
const Device UA2717979 = {S38423, 106, { &P15364,&P18110,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86; 5.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/15.0.1410.63 Safari/537.31*/
const Device UA92399 = {S38424, 65, { &P15364,&P21442,&P17470,&P18092 }};	/*Useragent - Mozilla/4.7 (compatible; OffByOne; Windows 2000) Webster Pro V3.4*/
const Device UA105711 = {S38425, 51, { &P15364,&P21442,&P17470,&P18092 }};	/*Useragent - ICE Browser/5.05 (Java 1.4.0; Windows 2000 5.0 x86)*/
const Device UA261698 = {S38426, 48, { &P15364,&P21442,&P17470,&P18092 }};	/*Useragent - Mozilla/4.7 (compatible; OffByOne; Windows 2000)*/
const Device UA49969 = {S38427, 146, { &P13124,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PantechP9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA421760 = {S38428, 140, { &P13124,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PantechP9070 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA743013 = {S38429, 64, { &P13124,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; PantechP9070 Build/GRJ90)*/
const Device UA2758868 = {S38430, 138, { &P13124,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; PantechP9070 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3111068 = {S38431, 172, { &P13124,&P5629,&P15760,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 2.3.5;dev=presto;devm=PantechP9070;man=PANTECH;brw=Android;shwd=480x800;shwa=480x763;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA1990582 = {S38432, 141, { &P15147,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST15i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2175184 = {S38433, 153, { &P15147,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyEricssonST15i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2234083 = {S38434, 141, { &P15147,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST15i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2242255 = {S38435, 63, { &P15147,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST15i Build/4.1.B.0.587)*/
const Device UA2257613 = {S38436, 153, { &P15147,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyEricssonST15a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2551536 = {S38437, 154, { &P15147,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hr-hr; SonyEricssonST15iv Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2693213 = {S38438, 141, { &P15147,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST15a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2818947 = {S38439, 141, { &P15147,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; ST15a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA627125 = {S38440, 83, { &P15364,&P7127,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9.2.28) Gecko/X Firefox/3.6.28*/
const Device UA91023 = {S38441, 145, { &P17262,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Kindle Fire Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370804 = {S38442, 161, { &P17262,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Kindle_Fire Build/GINGERBREAD; 1024*600) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA622380 = {S38443, 151, { &P17262,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Kindle Fire Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30*/
const Device UA629797 = {S38444, 69, { &P17262,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; Kindle Fire Build/GINGERBREAD)*/
const Device UA697711 = {S38445, 143, { &P17262,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; Kindle Fire Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1938722 = {S38446, 64, { &P17262,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; Kindle Fire Build/GRK39F)*/
const Device UA2023384 = {S38447, 140, { &P17262,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; md-us; Kindle Fire Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA152045 = {S38448, 51, { &P15364,&P17188,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0b; Windows NT 4.0)*/
const Device UA491952 = {S38449, 70, { &P15364,&P17188,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0b; Windows NT 4.0; .NET CLR 1.0.2914)*/
const Device UA2221075 = {S38450, 55, { &P15364,&P17188,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.0; SV1)*/
const Device UA2573316 = {S38451, 97, { &P15364,&P17188,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)*/
const Device UA2660630 = {S38452, 57, { &P15364,&P17188,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; WinNT4.0; Trident/3.0)*/
const Device UA2808460 = {S38453, 151, { &P15364,&P17188,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.0; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA3098524 = {S38454, 178, { &P15364,&P17188,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2708059 = {S38455, 74, { &P17595,&P18577,&P17415,&P18092 }};	/*Useragent - Safari/6534.59.8 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro7%2C1)*/
const Device UA2716840 = {S38456, 69, { &P17595,&P18577,&P17415,&P18092 }};	/*Useragent - Safari/6534.59.8 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (iMac10%2C1)*/
const Device UA2736532 = {S38457, 71, { &P17595,&P18577,&P17415,&P18092 }};	/*Useragent - Safari/6534.59.8 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBook4%2C1)*/
const Device UA2752446 = {S38458, 71, { &P17595,&P18577,&P17415,&P18092 }};	/*Useragent - Safari/6534.59.8 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (Macmini3%2C1)*/
const Device UA2767609 = {S38459, 70, { &P17595,&P18577,&P17415,&P18092 }};	/*Useragent - Safari/6534.59.8 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacPro1%2C1)*/
const Device UA2794422 = {S38460, 74, { &P17595,&P18577,&P17415,&P18092 }};	/*Useragent - Safari/6534.59.8 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookAir3%2C2)*/
const Device UA897367 = {S38461, 117, { &P17595,&P17295,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5*/
const Device UA899976 = {S38462, 117, { &P17595,&P17295,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.53 Safari/536.5*/
const Device UA900115 = {S38463, 117, { &P17595,&P17295,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.53 Safari/536.5*/
const Device UA900161 = {S38464, 117, { &P17595,&P17295,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.53 Safari/536.5*/
const Device UA900278 = {S38465, 117, { &P17595,&P17295,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_2) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.53 Safari/536.5*/
const Device UA901555 = {S38466, 117, { &P17595,&P17295,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA901648 = {S38467, 117, { &P17595,&P17295,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA902329 = {S38468, 117, { &P17595,&P17295,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_5) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA2674869 = {S38469, 129, { &P17595,&P17295,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/19.0.648.205 Safari/534.16*/
const Device UA2718867 = {S38470, 129, { &P17595,&P17295,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/19.0.648.205 Safari/534.16*/
const Device UA2489038 = {S38471, 112, { &P18506,&P5043,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia305/03.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA3045784 = {S38472, 173, { &P18506,&P5043,&P19812,&P18092 }};	/*Useragent - Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Series40; Nokia305/03.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA1995357 = {S38473, 139, { &P18295,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2013244 = {S38474, 135, { &P18295,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; One X Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2231011 = {S38475, 139, { &P18295,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One X Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2237831 = {S38476, 135, { &P18295,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; One X Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2354545 = {S38477, 139, { &P18295,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EndeavorU Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2456167 = {S38478, 139, { &P18295,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; HTC One X Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2482799 = {S38479, 139, { &P18295,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; EndeavorU Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2691199 = {S38480, 139, { &P18295,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.5; HTC One X Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2727866 = {S38481, 133, { &P18295,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One X Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2791932 = {S38482, 140, { &P18295,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1;  HTC One X Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA2840850 = {S38483, 129, { &P18295,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; One X Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2947936 = {S38484, 118, { &P15364,&P5690,&P22763,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1*/
const Device UA3016441 = {S38485, 118, { &P15364,&P5690,&P22763,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1*/
const Device UA3017738 = {S38486, 128, { &P15364,&P5690,&P22763,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1,gzip(gfe)*/
const Device UA3018937 = {S38487, 125, { &P15364,&P5690,&P22763,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1*/
const Device UA3020360 = {S38488, 125, { &P15364,&P5690,&P22763,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1*/
const Device UA378899 = {S38489, 104, { &P15364,&P16110,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.803.0 Safari/535.1*/
const Device UA789669 = {S38490, 97, { &P15364,&P16110,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.814.0 Safari/535.1*/
const Device UA93882 = {S38491, 98, { &P15364,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1;  http://www.changedetection.com/bot.html )*/
const Device UA125189 = {S38492, 83, { &P15364,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; CollapsarWEB qihoobot@qihoo.net)*/
const Device UA161457 = {S38493, 146, { &P15364,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; CrystalSemanticsBot http://www.crystalsemantics.com/service-navigation/imprint/useragent/)*/
const Device UA167157 = {S38494, 83, { &P15364,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; QihooBot 1.0 qihoobot@qihoo.net)*/
const Device UA191860 = {S38495, 60, { &P15364,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/6.0 (MSIE 6.0; Windows NT 5.1; Searcharoo.NET Robot)*/
const Device UA193567 = {S38496, 61, { &P15364,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/6.0 (MSIE 6.0; Windows NT 5.1; Searcharoo.NET; robot)*/
const Device UA276107 = {S38497, 70, { &P15364,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; maxamine.com-robot)*/
const Device UA657596 = {S38498, 82, { &P15364,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; .NET CLR 1.1.4322; AdSonar Bot)*/
const Device UA736014 = {S38499, 99, { &P15364,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727) yolinkBot*/
const Device UA783455 = {S38500, 97, { &P15364,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; http://www.changedetection.com/bot.html )*/
const Device UA2271051 = {S38501, 84, { &P15364,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; QihooBot 1.0  qihoobot@qihoo.net)*/
const Device UA2437543 = {S38502, 120, { &P15364,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; CrystalSemanticsBot http://www.crystalsemantics.com/user-agent/)*/
const Device UA349001 = {S38503, 140, { &P13074,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P930/V10f Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA351681 = {S38504, 135, { &P13074,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P930 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625196 = {S38505, 140, { &P13074,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P930/V10j Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA640949 = {S38506, 140, { &P13074,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; LG-P930/V11a Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA698895 = {S38507, 59, { &P13074,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; LG-P930 Build/GRJ90)*/
const Device UA189204 = {S38508, 141, { &P16168,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LogicPD Zoom2 Build/ERD79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA302484 = {S38509, 146, { &P16168,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Nook Color (zoom2) Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA448474 = {S38510, 135, { &P16168,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; NookColor Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622910 = {S38511, 141, { &P16168,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; NOOK BNRV200 Build/ERD79) Apple WebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA693435 = {S38512, 60, { &P16168,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; BNRV200 Build/MASTER)*/
const Device UA910158 = {S38513, 147, { &P16168,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; NOOK BNRV200 Build/ERD79 1.4.3) Apple WebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1881384 = {S38514, 62, { &P16168,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; NOOKcolor Build/MASTER)*/
const Device UA2043700 = {S38515, 146, { &P16168,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; NOOK BNRV200Build/ERD79 1.4.3) Apple WebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2043701 = {S38516, 146, { &P16168,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.1; en-us; NOOK BNRV200 Build/ERD79 1.4.3) Apple WebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2043708 = {S38517, 161, { &P16168,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; BNRV200 Build/MASTER) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2043709 = {S38518, 146, { &P16168,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; NOOK BNRV200 Build/ERD79 1.4.3) Apple WebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA2043727 = {S38519, 140, { &P16168,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; NOOK BNRV200 Build/ERD79) Apple WebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA2043739 = {S38520, 91, { &P16168,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; BNRV200 Build/MASTER) WWReader/1.5.0 build 800*/
const Device UA2043742 = {S38521, 146, { &P16168,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; NOOK BNRV200 Build/ERD791.4.3) Apple WebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2043741 = {S38522, 146, { &P16168,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; NOOK BNRV200 Build/ERD79 1.4.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA347137 = {S38523, 150, { &P15978,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Huawei-U8652 Build/HuaweiU8652B840) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627308 = {S38524, 150, { &P15978,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Huawei-U8652 Build/HuaweiU8652B855) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA644983 = {S38525, 145, { &P15978,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-nz; U8652-51 Build/HuaweiU8652-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA768246 = {S38526, 145, { &P15978,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-mx; U8652-51 Build/HuaweiU8652-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1980888 = {S38527, 74, { &P15978,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Huawei-U8652 Build/HuaweiU8652B855)*/
const Device UA2020518 = {S38528, 139, { &P15978,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; U8652 Build/HuaweiU8652) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022740 = {S38529, 74, { &P15978,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; Huawei-U8652 Build/HuaweiU8652B840)*/
const Device UA2234645 = {S38530, 146, { &P15978,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Huawei-U8652 Build/HuaweiU8652) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2593660 = {S38531, 145, { &P15978,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U8652-51 Build/HuaweiU8652-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2837906 = {S38532, 70, { &P15978,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; Huawei-U8652 Build/HuaweiU8652)*/
const Device UA2201600 = {S38533, 141, { &P18582,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT28at Build/6.1.C.1.105) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2368189 = {S38534, 139, { &P18582,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT28i Build/6.1.E.2.68) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2424670 = {S38535, 138, { &P18582,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT28h Build/6.1.E.3.7) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA366680 = {S38536, 65, { &P15364,&P5690,&P17190,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.1) KomodiaBot/1.0*/
const Device UA451570 = {S38537, 117, { &P15364,&P5690,&P17190,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Banca Caboto s.p.a.; InfoPath.1; Banca Caboto s.p.a.)*/
const Device UA493950 = {S38538, 160, { &P15364,&P5690,&P17190,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.1; Banca Caboto s.p.a.)*/
const Device UA531913 = {S38539, 116, { &P15364,&P5690,&P17190,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; InfoPath.1; Banca Caboto s.p.a.)*/
const Device UA564794 = {S38540, 96, { &P15364,&P5690,&P17190,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Banca Caboto s.p.a.; InfoPath.1)*/
const Device UA2248555 = {S38541, 170, { &P15364,&P5690,&P17190,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; botw; .NET CLR 2.0.50727; .NET CLR 1.1.4322; InfoPath.3; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; botw)*/
const Device UA33457 = {S38542, 142, { &P12595,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-T759 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47995 = {S38543, 141, { &P12595,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us;SGH-T759 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47997 = {S38544, 141, { &P12595,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-T759 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA48001 = {S38545, 141, { &P12595,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.3; en-us; SGH-T759 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60932 = {S38546, 141, { &P12595,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.3; en-us; SGH-T759 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60934 = {S38547, 141, { &P12595,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-T759Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60937 = {S38548, 142, { &P12595,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; SGH-T759 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76235 = {S38549, 187, { &P12595,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-T759 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 androidAppVersion/2.0.3 AndroidWebkitWrapper*/
const Device UA191108 = {S38550, 66, { &P12595,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SGH-T759 Build/GINGERBREAD)*/
const Device UA369608 = {S38551, 78, { &P12595,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.3; en_US; SGH-T759 Build/GINGERBREAD.UVKE8)*/
const Device UA654849 = {S38552, 142, { &P12595,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SGH-T759 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020349 = {S38553, 149, { &P12595,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-T759 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA93048 = {S38554, 100, { &P15364,&P7127,&P22728,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; Avant Browser [avantbrowser.com]; Hotbar 4.4.5.0)*/
const Device UA169003 = {S38555, 80, { &P15364,&P7127,&P22728,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; Avant Browser; Avant Browser)*/
const Device UA89546 = {S38556, 115, { &P15767,&P17141,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7D11*/
const Device UA89819 = {S38557, 109, { &P15767,&P17141,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B367*/
const Device UA96252 = {S38558, 96, { &P15767,&P17141,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko)*/
const Device UA114545 = {S38559, 111, { &P15767,&P17141,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2_1 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B405*/
const Device UA116545 = {S38560, 98, { &P15767,&P17141,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2_1 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko)*/
const Device UA126502 = {S38561, 98, { &P15767,&P17141,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko)*/
const Device UA126905 = {S38562, 111, { &P15767,&P17141,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B500*/
const Device UA135240 = {S38563, 140, { &P15767,&P17141,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B367 Safari/528.16 NightBrowser/1.0*/
const Device UA275642 = {S38564, 126, { &P15767,&P17141,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B367 Safari/531.21.10*/
const Device UA388863 = {S38565, 128, { &P15767,&P17141,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2_1 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B405 Safari/531.21.10*/
const Device UA484853 = {S38566, 128, { &P15767,&P17141,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B500 Safari/531.21.10*/
const Device UA522601 = {S38567, 77, { &P15767,&P17141,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0(iPad; U; iPhone OS 3_2; en-us) AppleWebKit/531.21.10 Mobile/7B314*/
const Device UA50066 = {S38568, 50, { &P15364,&P17188,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 4.0)*/
const Device UA104540 = {S38569, 51, { &P15364,&P17188,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 4.0)*/
const Device UA114581 = {S38570, 65, { &P15364,&P17188,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 4.0; Alexa Toolbar)*/
const Device UA1886572 = {S38571, 59, { &P15364,&P17188,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible;MSIE 5.0;  Windows NT 4.0; 360safe)*/
const Device UA2143265 = {S38572, 51, { &P15364,&P17188,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.05; Windows NT 4.0)*/
const Device UA2296564 = {S38573, 59, { &P15364,&P17188,&P4551,&P18092 }};	/*Useragent - Mozilla/3.0 (compatible;MSIE 5.0;  Windows NT 4.0; 360safe)*/
const Device UA2471720 = {S38574, 51, { &P15364,&P17188,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.01; Windows NT 4.0)*/
const Device UA2791829 = {S38575, 57, { &P15364,&P17188,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; WinNT4.0; Trident/3.0)*/
const Device UA1994996 = {S38576, 152, { &P18746,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; T-Mobile myTouch Q Build/HuaweiU8730) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029479 = {S38577, 76, { &P18746,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; T-Mobile myTouch Q Build/HuaweiU8730)*/
const Device UA2378696 = {S38578, 151, { &P18746,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; T-Mobile myTouch Q Build/HuaweiU8730) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2378859 = {S38579, 151, { &P18746,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.6; en-us; T-Mobile myTouch Q Build/HuaweiU8730) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2378890 = {S38580, 151, { &P18746,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; T-Mobile myTouch Q Build/HuaweiU8730) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA2400197 = {S38581, 151, { &P18746,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; T-MobilemyTouch Q Build/HuaweiU8730) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2414714 = {S38582, 151, { &P18746,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us;T-Mobile myTouch Q Build/HuaweiU8730) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2419806 = {S38583, 151, { &P18746,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; T-Mobile myTouch QBuild/HuaweiU8730) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2447398 = {S38584, 151, { &P18746,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; T-Mobile myTouchQ Build/HuaweiU8730) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51207 = {S38585, 142, { &P12496,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-I897 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51399 = {S38586, 150, { &P12496,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SAMSUNG-SGH-I897 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51406 = {S38587, 150, { &P12496,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SAMSUNG-SGH-I897 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74622 = {S38588, 150, { &P12496,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SAMSUNG-SGH-I897 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA190327 = {S38589, 136, { &P12496,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SGH-I897 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA195793 = {S38590, 142, { &P12496,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SGH-I897 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA236647 = {S38591, 136, { &P12496,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-I897 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA248864 = {S38592, 136, { &P12496,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-I897 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA341046 = {S38593, 142, { &P12496,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-I897 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA560209 = {S38594, 74, { &P12496,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SAMSUNG-SGH-I897 Build/GINGERBREAD)*/
const Device UA634450 = {S38595, 142, { &P12496,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-I897 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA669993 = {S38596, 74, { &P12496,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SAMSUNG-SGH-I897 Build/GINGERBREAD)*/
const Device UA737160 = {S38597, 136, { &P12496,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-I897 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1886988 = {S38598, 60, { &P12496,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SGH-I897 Build/GRJ22)*/
const Device UA1889460 = {S38599, 74, { &P12496,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SAMSUNG-SGH-I897 Build/GINGERBREAD)*/
const Device UA1933840 = {S38600, 66, { &P12496,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; SGH-I897 Build/GINGERBREAD)*/
const Device UA2018976 = {S38601, 66, { &P12496,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SGH-I897 Build/GINGERBREAD)*/
const Device UA2019424 = {S38602, 60, { &P12496,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SGH-I897 Build/GRJ22)*/
const Device UA2022723 = {S38603, 141, { &P12496,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-I897 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA726405 = {S38604, 97, { &P15364,&P5690,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:7.0.1) Gecko/X Firefox/7.0.1,gzip(gfe) (via translate.google.com)*/
const Device UA894648 = {S38605, 50, { &P15364,&P5690,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) Gecko/X Firefox/7.0.1*/
const Device UA2234466 = {S38606, 70, { &P15364,&P5690,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:7.0.1) Gecko/X Firefox/7.0.1,gzip(gfe)*/
const Device UA2474053 = {S38607, 79, { &P15364,&P5690,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:7.0a1) Gecko/X Firefox/7.0a1*/
const Device UA2806102 = {S38608, 79, { &P15364,&P5690,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:7.0a1) Gecko/X Firefox/7.0a1*/
const Device UA840072 = {S38609, 140, { &P18582,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LT28at Build/6.0.C.1.257) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA877313 = {S38610, 152, { &P18582,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SonyEricssonLT28at Build/6.0.C.1.257) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA899930 = {S38611, 150, { &P18582,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-; SonyEricssonLT28at Build/6.0.C.1.257) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1971282 = {S38612, 138, { &P18582,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-; LT28at Build/6.0.C.1.257) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1981934 = {S38613, 64, { &P18582,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; LT28at Build/6.0.C.1.257)*/
const Device UA1993874 = {S38614, 150, { &P18582,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; SonyEricssonLT28i Build/6.0.F.0.72) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1994183 = {S38615, 151, { &P18582,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-ca; SonyEricssonLT28i Build/6.0.F.0.111) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028094 = {S38616, 63, { &P18582,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; LT28i Build/6.0.F.0.111)*/
const Device UA2029446 = {S38617, 139, { &P18582,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-ca; LT28i Build/6.0.F.0.111) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032261 = {S38618, 138, { &P18582,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; LT28i Build/6.0.F.0.72) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2496697 = {S38619, 132, { &P18582,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT28i Build/6.0.F.0.111) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1972512 = {S38620, 148, { &P18224,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HUAWEI U8818 Build/HuaweiU8818) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029100 = {S38621, 148, { &P18224,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HUAWEI U8815 Build/HuaweiU8815) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2178616 = {S38622, 155, { &P18224,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HUAWEI U8815 Build/HuaweiU8815C02B936) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2236060 = {S38623, 157, { &P18224,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; HUAWEI U8815N Build/HuaweiU8815NC02B936) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2239018 = {S38624, 155, { &P18224,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; HUAWEI U8815 Build/HuaweiU8815C02B937) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2390979 = {S38625, 155, { &P18224,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HUAWEI U8815 Build/HuaweiU8815C02B952) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2446309 = {S38626, 70, { &P18224,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HUAWEI U8818 Build/HuaweiU8818)*/
const Device UA2469224 = {S38627, 157, { &P18224,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; HUAWEI U8815N Build/HuaweiU8815NC02B942) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2475884 = {S38628, 70, { &P18224,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HUAWEI U8815 Build/HuaweiU8815)*/
const Device UA2503249 = {S38629, 155, { &P18224,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; HUAWEI Ascend G 300 Build/HuaweiU8815) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2520610 = {S38630, 139, { &P18224,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI U8818 Build/HuaweiU8818) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2558972 = {S38631, 155, { &P18224,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HUAWEI U8815 Build/HuaweiU8815C02B960) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567749 = {S38632, 157, { &P18224,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; HUAWEI U8815N Build/HuaweiU8815NC02B943) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2693959 = {S38633, 147, { &P18224,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Ascend G300 Build/HuaweiU8815) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2763468 = {S38634, 154, { &P18224,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Ascend G300 Build/HuaweiU8815C02B952) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3071463 = {S38635, 79, { &P18224,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HUAWEI U8815N Build/HuaweiU8815NC02B942)*/
const Device UA3121364 = {S38636, 144, { &P18224,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; U8815-51 Build/HuaweiU8815) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA63015 = {S38637, 77, { &P13341,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry9105/5.0.0.884 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/603*/
const Device UA440746 = {S38638, 109, { &P15364,&P16110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19*/
const Device UA440826 = {S38639, 102, { &P15364,&P16110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19*/
const Device UA2260039 = {S38640, 150, { &P15364,&P16110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; Valve Steam GameOverlay/2026; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2353563 = {S38641, 150, { &P15364,&P16110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; Valve Steam GameOverlay/2005; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2375861 = {S38642, 136, { &P15364,&P16110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; Valve Client/0; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2408168 = {S38643, 150, { &P15364,&P16110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; Valve Steam GameOverlay/2095; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2430877 = {S38644, 150, { &P15364,&P16110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; Valve Steam GameOverlay/2109; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2433270 = {S38645, 150, { &P15364,&P16110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; Valve Steam GameOverlay/2102; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2470754 = {S38646, 150, { &P15364,&P16110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; Valve Steam GameOverlay/2125; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2557262 = {S38647, 132, { &P15364,&P16110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/18.0 BIDUPlayerBrowser/2.x Safari/534.7*/
const Device UA2600484 = {S38648, 120, { &P15364,&P16110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; zh_CN) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/18.0 BIDUPlayerBrowser/2.x Safari/534.7*/
const Device UA2875379 = {S38649, 143, { &P15364,&P16110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; Valve Steam Tenfoot/X; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA2945806 = {S38650, 150, { &P15364,&P16110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; Valve Steam GameOverlay/X; ) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Safari/535.19*/
const Device UA3024380 = {S38651, 145, { &P15364,&P16110,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; Valve Steam Client/X; ) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Safari/535.19*/
const Device UA2240794 = {S38652, 138, { &P18595,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2467603 = {S38653, 138, { &P18595,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2483362 = {S38654, 150, { &P18595,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1_????-??????; GT-I8190 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA98998 = {S38655, 75, { &P15364,&P17188,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 4.0) Opera 7.0 [en]*/
const Device UA600324 = {S38656, 34, { &P15364,&P17188,&P4810,&P18092 }};	/*Useragent - Opera/7.0 (Windows NT 4.0; U) [de]*/
const Device UA714898 = {S38657, 66, { &P15364,&P17188,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.0) Opera 7.23 [de]*/
const Device UA916114 = {S38658, 35, { &P15364,&P17188,&P4810,&P18092 }};	/*Useragent - Opera/7.10 (Windows NT 4.0; U) [de]*/
const Device UA2295092 = {S38659, 66, { &P15364,&P17188,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.0) Opera 7.11 [de]*/
const Device UA369023 = {S38660, 60, { &P18498,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-LG855 Build/GRJ22)*/
const Device UA623055 = {S38661, 136, { &P18498,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LG855 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029490 = {S38662, 142, { &P18498,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LG855 Build/CTMod 3.6.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031655 = {S38663, 141, { &P18498,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LG855 Build/CTMod 3.68) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2467276 = {S38664, 147, { &P18498,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LG855 Build/CTMod 3.75 Final) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2734805 = {S38665, 141, { &P18498,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LG855 Build/CTMod 3.80) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA910095 = {S38666, 119, { &P17595,&P17295,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11*/
const Device UA910232 = {S38667, 119, { &P17595,&P17295,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11*/
const Device UA910549 = {S38668, 119, { &P17595,&P17295,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_5) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11*/
const Device UA910815 = {S38669, 119, { &P17595,&P17295,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11*/
const Device UA910999 = {S38670, 119, { &P17595,&P17295,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11*/
const Device UA911270 = {S38671, 119, { &P17595,&P17295,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11*/
const Device UA911561 = {S38672, 156, { &P17595,&P17295,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11,gzip(gfe) (via translate.google.com)*/
const Device UA911728 = {S38673, 119, { &P17595,&P17295,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_2) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11*/
const Device UA1990708 = {S38674, 119, { &P17595,&P17295,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.43 Safari/536.11*/
const Device UA2674474 = {S38675, 129, { &P17595,&P17295,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/20.0.648.205 Safari/534.16*/
const Device UA2752109 = {S38676, 129, { &P17595,&P17295,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/20.0.648.205 Safari/534.16*/
const Device UA3082793 = {S38677, 133, { &P15767,&P19826,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B329 Safari/8536.25*/
const Device UA3082987 = {S38678, 133, { &P15767,&P19826,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B146 Safari/8536.25*/
const Device UA3082992 = {S38679, 131, { &P15767,&P19826,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B141 Safari/8536.25*/
const Device UA3083403 = {S38680, 133, { &P15767,&P19826,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B147 Safari/8536.25*/
const Device UA169210 = {S38681, 133, { &P13222,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Ally Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA169737 = {S38682, 57, { &P13222,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; Ally Build/FRG83D)*/
const Device UA191892 = {S38683, 57, { &P13222,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; Ally Build/FRG83G)*/
const Device UA192446 = {S38684, 133, { &P13222,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Ally Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA196332 = {S38685, 133, { &P13222,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Ally Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA532221 = {S38686, 132, { &P13222,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; Ally Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA532280 = {S38687, 132, { &P13222,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.2; en-us; Ally Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA536849 = {S38688, 132, { &P13222,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;Ally Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA548311 = {S38689, 132, { &P13222,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Ally Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA555770 = {S38690, 132, { &P13222,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.2; en-us; Ally Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA557381 = {S38691, 132, { &P13222,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; AllyBuild/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA557437 = {S38692, 132, { &P13222,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Ally Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA797604 = {S38693, 132, { &P13222,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;Ally Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1877115 = {S38694, 132, { &P13222,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; AllyBuild/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018638 = {S38695, 132, { &P13222,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.1; en-us; Ally Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019492 = {S38696, 140, { &P13222,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Ally Build/FRG83G) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA67904 = {S38697, 133, { &P13746,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; vi-vn; GT540 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA68368 = {S38698, 133, { &P13746,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT540 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA809935 = {S38699, 133, { &P13746,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT540 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1901935 = {S38700, 133, { &P13746,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; GT540 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021363 = {S38701, 57, { &P13746,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; GT540 Build/GRI40)*/
const Device UA2165057 = {S38702, 127, { &P13746,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LGL35G/V100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2238811 = {S38703, 135, { &P13746,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LGL35G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2384732 = {S38704, 59, { &P13746,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LGL35G Build/GRK39F)*/
const Device UA2697467 = {S38705, 124, { &P13746,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en; LGL35G/V100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA114783 = {S38706, 62, { &P15364,&P17285,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.6.30 Version/10.60*/
const Device UA128218 = {S38707, 62, { &P15364,&P17285,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.6.30 Version/10.61*/
const Device UA138337 = {S38708, 62, { &P15364,&P17285,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.6.30 Version/10.62*/
const Device UA147978 = {S38709, 62, { &P15364,&P17285,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.6.30 Version/10.63*/
const Device UA161663 = {S38710, 85, { &P15364,&P17285,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03686); ru) Presto/2.6.30 Version/10.63*/
const Device UA385975 = {S38711, 85, { &P15364,&P17285,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.6.30 Version/10.63*/
const Device UA1863992 = {S38712, 85, { &P15364,&P17285,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.6.30 Version/10.63*/
const Device UA2262570 = {S38713, 85, { &P15364,&P17285,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.6.30 Version/10.63*/
const Device UA2265817 = {S38714, 85, { &P15364,&P17285,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.6.30 Version/10.62*/
const Device UA211431 = {S38715, 142, { &P14849,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-I800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250337 = {S38716, 142, { &P14849,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA354148 = {S38717, 77, { &P14849,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.3 (Linux; U; Android 2.3.5; en_US; SCH-I800 Build/GINGERBREAD.EI04)*/
const Device UA355701 = {S38718, 140, { &P14849,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ko-; SCH-I800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368849 = {S38719, 97, { &P14849,&P5629,&P15760,&P18092 }};	/*Useragent - com.google.android.youtube/2.3.4(Linux; U; Android 2.3.5; en_US; SCH-I800 Build/GINGERBREAD) gzip*/
const Device UA557020 = {S38720, 142, { &P14849,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SCH-I800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622458 = {S38721, 148, { &P14849,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I800 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30*/
const Device UA635584 = {S38722, 66, { &P14849,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SCH-I800 Build/GINGERBREAD)*/
const Device UA737590 = {S38723, 141, { &P14849,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.5; en-us; SCH-I800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA754968 = {S38724, 141, { &P14849,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA806152 = {S38725, 141, { &P14849,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.5; en-us; SCH-I800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA817559 = {S38726, 141, { &P14849,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I800Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA845543 = {S38727, 141, { &P14849,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us;SCH-I800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA93068 = {S38728, 63, { &P15364,&P21442,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows 2000) Opera 6.0 [en]*/
const Device UA261742 = {S38729, 28, { &P15364,&P21442,&P17466,&P18092 }};	/*Useragent - Opera/6.05 (Windows 2000; U)*/
const Device UA414499 = {S38730, 64, { &P15364,&P21442,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows 2000) Opera 6.03 [en]*/
const Device UA599409 = {S38731, 45, { &P15364,&P21442,&P17466,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows 2000; U) Opera 6.03 [en]*/
const Device UA656617 = {S38732, 32, { &P15364,&P21442,&P17466,&P18092 }};	/*Useragent - Opera/6.0 (Windows 2000; U) [fr]*/
const Device UA857773 = {S38733, 46, { &P15364,&P21442,&P17466,&P18092 }};	/*Useragent - Mozilla/4.78 (Windows 2000; U) Opera 6.01 [en]*/
const Device UA1941923 = {S38734, 33, { &P15364,&P21442,&P17466,&P18092 }};	/*Useragent - Opera/6.01 (Windows 2000; U) [de]*/
const Device UA2803840 = {S38735, 34, { &P15364,&P21442,&P17466,&P18092 }};	/*Useragent - Opera/6.04 (Windows 2000; U)  [en]*/
const Device UA58863 = {S38736, 150, { &P14183,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SAMSUNG-SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA69508 = {S38737, 150, { &P14183,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SAMSUNG-SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70927 = {S38738, 150, { &P14183,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SAMSUNG-SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA241241 = {S38739, 142, { &P14183,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA568094 = {S38740, 136, { &P14183,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-I997 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA572758 = {S38741, 142, { &P14183,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA576445 = {S38742, 150, { &P14183,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623201 = {S38743, 74, { &P14183,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SAMSUNG-SGH-I997 Build/GINGERBREAD)*/
const Device UA723894 = {S38744, 156, { &P14183,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6 UCLB3; en-us; SAMSUNG-SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA740161 = {S38745, 142, { &P14183,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA801068 = {S38746, 142, { &P14183,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1916144 = {S38747, 148, { &P14183,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android XXJVT 2.3.5; en-us; SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019461 = {S38748, 74, { &P14183,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SAMSUNG-SGH-I997 Build/GINGERBREAD)*/
const Device UA2023312 = {S38749, 74, { &P14183,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SAMSUNG-SGH-I997 Build/GINGERBREAD)*/
const Device UA2024123 = {S38750, 66, { &P14183,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; SGH-I997 Build/GINGERBREAD)*/
const Device UA2545117 = {S38751, 143, { &P14183,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SAMSUNG-SGH-I997 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2812788 = {S38752, 148, { &P14183,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SAMSUNG-SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2204908 = {S38753, 150, { &P18642,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Huawei-U8665 Build/HuaweiU8665B037) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2242366 = {S38754, 74, { &P18642,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; Huawei-U8665 Build/HuaweiU8665B037)*/
const Device UA2986869 = {S38755, 148, { &P18642,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; Huawei-U8665 Build/HuaweiU8665B037) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3093449 = {S38756, 172, { &P18642,&P5629,&P15760,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 2.3.6;dev=hwu8665;devm=Huawei-U8665;man=HUAWEI;brw=Android;shwd=320x480;shwa=320x455;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA100371 = {S38757, 110, { &P15364,&P5690,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; fr; rv:1.8.1) VoilaBot BETA 1.2 (support.voilabot@orange-ftgroup.com)*/
const Device UA416021 = {S38758, 104, { &P15364,&P5690,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1;fr;rv:1.8.1) VoilaBotCollector BETA 0.1  (http://www.voila.com/)*/
const Device UA2945951 = {S38759, 90, { &P19886,&P5567,&P18525,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/6.0.3/27.2338; U; en) Presto/2.8.119 320X240 Samsung SCH-U485*/
const Device UA2939186 = {S38760, 131, { &P23955,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P5210 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2969665 = {S38761, 60, { &P23955,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P5210 Build/JDQ39)*/
const Device UA3014933 = {S38762, 134, { &P23955,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-P5210XD1 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA603713 = {S38763, 80, { &P15364,&P7133,&P22340,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:11.0) Gecko/X Firefox/11.0 CometBird/11.0*/
const Device UA736236 = {S38764, 83, { &P15364,&P7133,&P22340,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:11.0) Gecko/X Firefox/11.0 CometBrowser/11.0*/
const Device UA3064642 = {S38765, 135, { &P20705,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065616 = {S38766, 137, { &P20705,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC_PN071 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066225 = {S38767, 140, { &P20705,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One 801e Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080593 = {S38768, 132, { &P20705,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; One Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083375 = {S38769, 144, { &P20705,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One dual sim Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094526 = {S38770, 136, { &P20705,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; HTC_One Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109622 = {S38771, 134, { &P20705,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTCONE Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2566499 = {S38772, 138, { &P15575,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566517 = {S38773, 138, { &P15575,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566685 = {S38774, 139, { &P15575,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100T Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566899 = {S38775, 131, { &P15575,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567101 = {S38776, 139, { &P15575,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100G Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567120 = {S38777, 139, { &P15575,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100M Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567117 = {S38778, 139, { &P15575,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100G Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567564 = {S38779, 139, { &P15575,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100T Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583211 = {S38780, 131, { &P15575,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2427434 = {S38781, 140, { &P18295,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; HTC One X Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2560457 = {S38782, 139, { &P18295,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC One X Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2560799 = {S38783, 135, { &P18295,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One X Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2704246 = {S38784, 61, { &P18295,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC One X Build/JDQ39)*/
const Device UA2731504 = {S38785, 57, { &P18295,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; One X Build/JDQ39)*/
const Device UA2765165 = {S38786, 140, { &P18295,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; HTC One X Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781897 = {S38787, 140, { &P18295,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC One X Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956547 = {S38788, 136, { &P18295,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One X Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3031048 = {S38789, 139, { &P18295,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC_One_X Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3060397 = {S38790, 140, { &P18295,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; HTC EVA_UL Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3063153 = {S38791, 150, { &P18295,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; de-ch; HTC_One_X/4.17.166.2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3064033 = {S38792, 150, { &P18295,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC_One_X/4.17.161.2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3067497 = {S38793, 150, { &P18295,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; de-at; HTC_One_X/4.17.169.2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3114753 = {S38794, 139, { &P18295,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HTC S720e Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3115740 = {S38795, 150, { &P18295,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; HTC_One_X/4.17.163.3 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2417619 = {S38796, 140, { &P19845,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZTE N9120 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2504054 = {S38797, 62, { &P19845,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ZTE N9120 Build/IMM76I)*/
const Device UA3071614 = {S38798, 220, { &P19845,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZTE N9120 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Android MetroPCS androidAppVersion/2.5.10 AndroidWebkitWrapper BillingSupported*/
const Device UA3091946 = {S38799, 185, { &P19845,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZTE N9120 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 androidAppVersion/2.6.8 AndroidWebkitWrapper*/
const Device UA604931 = {S38800, 154, { &P15575,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100 Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA615650 = {S38801, 154, { &P15575,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA895628 = {S38802, 165, { &P15575,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy SII T-mobile Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA904969 = {S38803, 165, { &P15575,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy SII T-mobile Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA906057 = {S38804, 155, { &P15575,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100G Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027500 = {S38805, 147, { &P15575,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2028418 = {S38806, 165, { &P15575,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy SII T-mobile Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030984 = {S38807, 158, { &P15575,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031599 = {S38808, 155, { &P15575,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100G Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA124098 = {S38809, 47, { &P15364,&P17285,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows; MSIE 6.0; Windows NT 6.0)*/
const Device UA135396 = {S38810, 63, { &P15364,&P17285,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows; U; Windows NT 6.0;)*/
const Device UA167268 = {S38811, 51, { &P15364,&P17285,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.01; Windows NT 6.0)*/
const Device UA275538 = {S38812, 50, { &P15364,&P17285,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.0)*/
const Device UA813443 = {S38813, 63, { &P15364,&P17285,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 6.0; Trident/5.0)*/
const Device UA2805313 = {S38814, 57, { &P15364,&P17285,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; MSIE 6.0; Windows NT 6.0; en-US)*/
const Device UA2937977 = {S38815, 120, { &P15364,&P17285,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.0; SV1; MRA 5.8 (build 4157); .NET CLR 2.0.50727; AskTbPTV/5.11.3.15590)*/
const Device UA2984875 = {S38816, 166, { &P15364,&P17285,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.0; WOW64; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; Media Center PC 5.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618)*/
const Device UA622958 = {S38817, 99, { &P15364,&P17817,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.24) Gecko/X Ubuntu/10.10 (maverick) Firefox/3.6.24*/
const Device UA2971588 = {S38818, 133, { &P15767,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.12 Mobile/11A4400f Safari/8536.25*/
const Device UA2972128 = {S38819, 133, { &P15767,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.12 Mobile/11A4414e Safari/8536.25*/
const Device UA3026581 = {S38820, 133, { &P15767,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A4435d Safari/8536.25*/
const Device UA3033031 = {S38821, 133, { &P15767,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A4449a Safari/8536.25*/
const Device UA3034358 = {S38822, 133, { &P15767,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A4449d Safari/8536.25*/
const Device UA3083242 = {S38823, 131, { &P15767,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A465 Safari/8536.25*/
const Device UA3115385 = {S38824, 133, { &P15767,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A501 Safari/8536.25*/
const Device UA33231 = {S38825, 139, { &P12622,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; C5120 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625442 = {S38826, 139, { &P12622,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; C5121 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020022 = {S38827, 63, { &P12622,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; C5120 Build/GINGERBREAD)*/
const Device UA2588078 = {S38828, 63, { &P12622,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; C5121 Build/GINGERBREAD)*/
const Device UA2595227 = {S38829, 134, { &P12622,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; C5120 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA609400 = {S38830, 147, { &P12972,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  SAMSUNG-SGH-T989 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA609494 = {S38831, 133, { &P12972,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  SGH-T989 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA804685 = {S38832, 133, { &P12972,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  SGH-T989 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA901894 = {S38833, 138, { &P12972,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA901946 = {S38834, 138, { &P12972,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA901979 = {S38835, 146, { &P12972,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-T989 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA902710 = {S38836, 146, { &P12972,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-T989 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA904331 = {S38837, 139, { &P12972,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989D Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1979614 = {S38838, 132, { &P12972,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2001438 = {S38839, 139, { &P12972,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2026412 = {S38840, 132, { &P12972,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2477838 = {S38841, 139, { &P20141,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S5301 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666410 = {S38842, 159, { &P20141,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; SAMSUNG GT-S5301/S5301XXAMA3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2698685 = {S38843, 61, { &P20141,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-S5301 Build/IMM76D)*/
const Device UA2700207 = {S38844, 140, { &P20141,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; GT-S5301L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2712703 = {S38845, 140, { &P20141,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT-S5301B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952337 = {S38846, 137, { &P20141,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-S5301 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA47929 = {S38847, 153, { &P16991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-fj; Vodafone 858 Build/Vodafone858C02B623) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47930 = {S38848, 153, { &P16991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-al; Vodafone 858 Build/Vodafone858C02B622) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48456 = {S38849, 157, { &P16991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ro-ro; Vodafone 858 Build/Vodafone858C02B617SP01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48458 = {S38850, 153, { &P16991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-mt; Vodafone 858 Build/Vodafone858C02B621) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61413 = {S38851, 153, { &P16991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; Vodafone 858 Build/Vodafone858C02B617) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61415 = {S38852, 153, { &P16991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-za; Vodafone 858 Build/Vodafone858C02B620) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61422 = {S38853, 153, { &P16991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; lt-lt; Vodafone 858 Build/Vodafone858C02B619) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78318 = {S38854, 153, { &P16991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gh; Vodafone 858 Build/Vodafone858C02B625) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78878 = {S38855, 153, { &P16991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; Vodafone 858 Build/Vodafone858C02B650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA575469 = {S38856, 157, { &P16991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-au; Vodafone 858 Build/Vodafone858C02B638SP01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA575537 = {S38857, 77, { &P16991,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; Vodafone 858 Build/Vodafone858C02B650)*/
const Device UA632102 = {S38858, 77, { &P16991,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; Vodafone 858 Build/Vodafone858C02B617)*/
const Device UA645454 = {S38859, 153, { &P16991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; Vodafone 858 Build/Vodafone858C02B652) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA646076 = {S38860, 153, { &P16991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; Vodafone 858 Build/Vodafone858C02B626) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA775186 = {S38861, 153, { &P16991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-qa; Vodafone 858 Build/Vodafone858C02B637) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA801211 = {S38862, 153, { &P16991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; tr-tr; Vodafone 858 Build/Vodafone858C02B663) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA820924 = {S38863, 153, { &P16991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-za; Vodafone 858 Build/Vodafone858C02B627) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1940616 = {S38864, 153, { &P16991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; hr-hr; Vodafone 858 Build/Vodafone858C02B636) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1965106 = {S38865, 157, { &P16991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; el-gr; Vodafone 858 Build/Vodafone858C02B652SP01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020532 = {S38866, 151, { &P16991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; SmartLucky Build/Vodafone858C02B625) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2475092 = {S38867, 157, { &P16991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; Vodafone 858 Build/Vodafone858C02B636SP01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2644394 = {S38868, 153, { &P16991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; tr-tr; Vodafone 858 Build/Vodafone858C02B678) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA603608 = {S38869, 81, { &P18254,&P7480,&P18245,&P18344 }};	/*Useragent - Mozilla/5.0 (compatible; Blekkobot; ScoutJet; +http://blekko.com/about/blekkobot)*/
const Device UA2372249 = {S38870, 164, { &P20202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-L38C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2504767 = {S38871, 60, { &P20202,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-L38C Build/GRK39F)*/
const Device UA2391214 = {S38872, 140, { &P19849,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-S7560M Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2391247 = {S38873, 147, { &P19849,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-S7560M-parrot Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2505151 = {S38874, 62, { &P19849,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-S7560M Build/IMM76I)*/
const Device UA3044165 = {S38875, 138, { &P19849,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-S7560M Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2405835 = {S38876, 62, { &P17968,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA2508091 = {S38877, 68, { &P17968,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:20.0a2) Gecko/20.0a2 Firefox/20.0a2*/
const Device UA2680224 = {S38878, 68, { &P17968,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:20.0.1) Gecko/20.0.1 Firefox/20.0.1*/
const Device UA2936012 = {S38879, 64, { &P17968,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:20.0) Gecko/20.0 OAFirefox/20.0*/
const Device UA3058400 = {S38880, 80, { &P17968,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile: SAMSUNG-SGH-I337; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA130331 = {S38881, 152, { &P15364,&P18110,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Jolicloud Linux i686; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Jolicloud/1.0 Chromium/6.0.472.36 Chrome/6.0.472.36 Safari/534.3*/
const Device UA136561 = {S38882, 110, { &P15364,&P18110,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA137972 = {S38883, 108, { &P15364,&P18110,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA140716 = {S38884, 117, { &P15364,&P18110,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA166625 = {S38885, 112, { &P15364,&P18110,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux 2.6.29; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.62 Safari/534.3*/
const Device UA2324713 = {S38886, 110, { &P15364,&P18110,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux armv6l; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA3026653 = {S38887, 134, { &P15767,&P18117,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9B206 Safari/7534.48.3*/
const Device UA3027473 = {S38888, 132, { &P15767,&P18117,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9B176 Safari/7534.48.3*/
const Device UA3092435 = {S38889, 132, { &P15767,&P18117,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9A405 Safari/7534.48.3*/
const Device UA251584 = {S38890, 64, { &P15364,&P18110,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; en) Presto/2.9.168 Version/11.52*/
const Device UA255559 = {S38891, 106, { &P15364,&P16110,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA568834 = {S38892, 97, { &P15364,&P16110,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/15.0.849.0 Safari/535.1*/
const Device UA441216 = {S38893, 120, { &P17595,&P17295,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_5) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19*/
const Device UA457608 = {S38894, 120, { &P17595,&P17295,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19*/
const Device UA457786 = {S38895, 120, { &P17595,&P17295,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19*/
const Device UA603889 = {S38896, 120, { &P17595,&P17295,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19*/
const Device UA614960 = {S38897, 120, { &P17595,&P17295,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA615021 = {S38898, 120, { &P17595,&P17295,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA712220 = {S38899, 120, { &P17595,&P17295,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA2222119 = {S38900, 120, { &P17595,&P17295,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA2674674 = {S38901, 129, { &P17595,&P17295,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/18.0.648.205 Safari/534.16*/
const Device UA2717910 = {S38902, 129, { &P17595,&P17295,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/18.0.648.205 Safari/534.16*/
const Device UA2706648 = {S38903, 157, { &P22584,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-ca; SGH-I337M Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2712918 = {S38904, 165, { &P22584,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; es-us; SAMSUNG SGH-I337M Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2738237 = {S38905, 164, { &P22584,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-ca; SGH-I337M-parrot Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2672761 = {S38906, 145, { &P18640,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA609399 = {S38907, 139, { &P16356,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  GT-N7000 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA609455 = {S38908, 139, { &P16356,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  GT-N7000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA900615 = {S38909, 138, { &P16356,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-N7000 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA901929 = {S38910, 139, { &P16356,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SHV-E160L Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA902820 = {S38911, 138, { &P16356,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1954188 = {S38912, 139, { &P16356,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SHV-E160K Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1959558 = {S38913, 139, { &P16356,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SHV-E160S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1972136 = {S38914, 132, { &P16356,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1989599 = {S38915, 139, { &P16356,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-N7000B Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1996095 = {S38916, 138, { &P16356,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T879 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1998768 = {S38917, 139, { &P16356,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2031420 = {S38918, 139, { &P16356,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2204151 = {S38919, 139, { &P16356,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160K Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2205461 = {S38920, 136, { &P16356,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-05D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2230363 = {S38921, 133, { &P16356,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M480W Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2234376 = {S38922, 139, { &P16356,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2382825 = {S38923, 133, { &P16356,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M480S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2451887 = {S38924, 136, { &P16356,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N7000+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA17227 = {S38925, 161, { &P11907,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE52/033.003 (SymbianOS/9.3; U; Series60/3.2 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525*/
const Device UA24768 = {S38926, 177, { &P11907,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE52-1/054.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.2 3gpp-gba*/
const Device UA44207 = {S38927, 166, { &P11907,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE52-2/034.001; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.1.5*/
const Device UA615640 = {S38928, 191, { &P11907,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE52-1/091.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.34 Mobile Safari/533.4 3gpp-gba*/
const Device UA2360258 = {S38929, 191, { &P11907,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE52-2/091.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.34 Mobile Safari/533.4 3gpp-gba*/
const Device UA62208 = {S38930, 77, { &P14384,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry9520/5.0.0.713 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/125*/
const Device UA64684 = {S38931, 78, { &P12140,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry9550/5.0.0.1015 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/105*/
const Device UA600501 = {S38932, 74, { &P17595,&P17297,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:12.0) Gecko/X Firefox/12.0*/
const Device UA770501 = {S38933, 87, { &P17595,&P17297,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:12.0) Gecko/X Firefox/12.0 Firefox/12.0*/
const Device UA22286 = {S38934, 138, { &P15759,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Vision Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA168229 = {S38935, 138, { &P15759,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; HTC Vision Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA182399 = {S38936, 138, { &P15759,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Vision Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA192857 = {S38937, 138, { &P15759,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Vision Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA195922 = {S38938, 62, { &P15759,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Vision Build/GRI40)*/
const Device UA197132 = {S38939, 138, { &P15759,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Vision Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA235488 = {S38940, 138, { &P15759,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Vision Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250023 = {S38941, 138, { &P15759,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Vision Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA302859 = {S38942, 138, { &P15759,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC Vision Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA558514 = {S38943, 62, { &P15759,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Vision Build/FRF91)*/
const Device UA565773 = {S38944, 138, { &P15759,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Vision Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633691 = {S38945, 138, { &P15759,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Vision Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA660788 = {S38946, 62, { &P15759,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Vision Build/GRI40)*/
const Device UA767013 = {S38947, 138, { &P15759,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Vision Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1938683 = {S38948, 126, { &P15759,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Vision Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2545590 = {S38949, 131, { &P15759,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Vision Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA150847 = {S38950, 131, { &P14255,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Ideos Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA165888 = {S38951, 134, { &P14255,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; it-it; Ideos Build/B827SP01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA196577 = {S38952, 133, { &P14255,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-ke; Ideos Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA315071 = {S38953, 134, { &P14255,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; it-it; Ideos Build/B827SP03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA373591 = {S38954, 55, { &P14255,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; Ideos Build/FRF91)*/
const Device UA537154 = {S38955, 57, { &P14255,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; Ideos Build/FRG83)*/
const Device UA819990 = {S38956, 134, { &P14255,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-nz; Ideos Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020093 = {S38957, 121, { &P14255,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-ke; Ideos Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA925386 = {S38958, 89, { &P18254,&P7480,&P18245,&P19829 }};	/*Useragent - Mozilla/5.0 (compatible; discoverybot/2.0; +http://discoveryengine.com/discoverybot.html)*/
const Device UA2356191 = {S38959, 128, { &P18254,&P7480,&P18245,&P19829 }};	/*Useragent - Mozilla/5.0 (compatible; discoverybot/2.0; +http://discoveryengine.com/discoverybot.html),gzip(gfe) (via docs.google.com/viewer)*/
const Device UA573742 = {S38960, 50, { &P15364,&P7127,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 5.0)*/
const Device UA1984664 = {S38961, 66, { &P15364,&P7127,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0;GB)*/
const Device UA1993764 = {S38962, 78, { &P15364,&P7127,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Alexa Toolbar)*/
const Device UA2154196 = {S38963, 68, { &P15364,&P7127,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; SV1)*/
const Device UA2158791 = {S38964, 68, { &P15364,&P7127,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; SV1)*/
const Device UA2761776 = {S38965, 73, { &P15364,&P7127,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; SV1; SV1)*/
const Device UA2775876 = {S38966, 73, { &P15364,&P7127,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; SV1; SV1)*/
const Device UA2753431 = {S38967, 50, { &P15364,&P18580,&P18157,&P18092 }};	/*Useragent - Puffin%20Free/9090 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2754548 = {S38968, 43, { &P15364,&P18580,&P18157,&P18092 }};	/*Useragent - Puffin/9073 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA53631 = {S38969, 46, { &P15364,&P17350,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT)*/
const Device UA99075 = {S38970, 82, { &P15364,&P17350,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; MS Web Services Client Protocol 2.0.50727.3053)*/
const Device UA123356 = {S38971, 41, { &P15364,&P17350,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Win32)*/
const Device UA2509626 = {S38972, 52, { &P15364,&P17350,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; 68K; Trident/5.0)*/
const Device UA2580052 = {S38973, 65, { &P15364,&P17350,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; 68K; Win64; IA64; Trident/2.0)*/
const Device UA2660247 = {S38974, 59, { &P15364,&P17350,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; 68K; WOW64; Trident/2.0)*/
const Device UA2791715 = {S38975, 57, { &P15364,&P17350,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; 68K; SV1; Trident/4.0)*/
const Device UA2949960 = {S38976, 46, { &P15364,&P17350,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Windows NT)*/
const Device UA2424637 = {S38977, 142, { &P18304,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2482924 = {S38978, 139, { &P18304,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9070 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556358 = {S38979, 61, { &P18304,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9070 Build/JZO54K)*/
const Device UA2712224 = {S38980, 140, { &P18304,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; GT-I9070P Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2868526 = {S38981, 158, { &P18304,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; SAMSUNG GT-I9070/I9070XXLQE Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936766 = {S38982, 62, { &P18304,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9070P Build/JZO54K)*/
const Device UA2988515 = {S38983, 137, { &P18304,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-I9070 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA902394 = {S38984, 147, { &P12557,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB612 Build/4.5.1-KNS-29-BST-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA924896 = {S38985, 144, { &P12557,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB612 Build/4.5.1-110-KNS-46) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2242338 = {S38986, 68, { &P12557,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; MB612 Build/4.5.1-110-KNS-46)*/
const Device UA902635 = {S38987, 76, { &P17595,&P17295,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:13.0) Gecko/X Firefox/13.0.1*/
const Device UA146527 = {S38988, 86, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.6)*/
const Device UA190023 = {S38989, 86, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.7)*/
const Device UA241774 = {S38990, 86, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.8)*/
const Device UA242077 = {S38991, 79, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.8)*/
const Device UA315910 = {S38992, 177, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.7)*/
const Device UA317337 = {S38993, 79, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.7)*/
const Device UA349581 = {S38994, 86, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.9)*/
const Device UA408297 = {S38995, 79, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.6)*/
const Device UA497510 = {S38996, 192, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C; Sleipnir/2.9.6)*/
const Device UA538553 = {S38997, 79, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.3)*/
const Device UA563774 = {S38998, 192, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; YTB730; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.7)*/
const Device UA571741 = {S38999, 177, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.8)*/
const Device UA613787 = {S39000, 87, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.10)*/
const Device UA613899 = {S39001, 80, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.10)*/
const Device UA629870 = {S39002, 79, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.9)*/
const Device UA662747 = {S39003, 185, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.10)*/
const Device UA704473 = {S39004, 86, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.3)*/
const Device UA736116 = {S39005, 177, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.9)*/
const Device UA742906 = {S39006, 86, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.4)*/
const Device UA851886 = {S39007, 194, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Sleipnir/2.9.9)*/
const Device UA924875 = {S39008, 87, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.11)*/
const Device UA925180 = {S39009, 80, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.11)*/
const Device UA1842394 = {S39010, 79, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.4)*/
const Device UA1854685 = {S39011, 177, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.6)*/
const Device UA1996956 = {S39012, 185, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.11)*/
const Device UA2008977 = {S39013, 87, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.12)*/
const Device UA2041329 = {S39014, 80, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.12)*/
const Device UA2144524 = {S39015, 86, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.1)*/
const Device UA2205104 = {S39016, 80, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.14)*/
const Device UA2205106 = {S39017, 87, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.14)*/
const Device UA2226054 = {S39018, 185, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.12)*/
const Device UA2232824 = {S39019, 185, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.14)*/
const Device UA2337271 = {S39020, 168, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Sleipnir/2.9.10)*/
const Device UA2338659 = {S39021, 178, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.11)*/
const Device UA2370565 = {S39022, 191, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; BRI/2; Sleipnir/2.9.8)*/
const Device UA2384369 = {S39023, 168, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Sleipnir/2.9.14)*/
const Device UA2435935 = {S39024, 80, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.15)*/
const Device UA2435944 = {S39025, 87, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.15)*/
const Device UA2436150 = {S39026, 185, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.15)*/
const Device UA2492761 = {S39027, 186, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; Sleipnir/2.9.6)*/
const Device UA2528646 = {S39028, 178, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.15)*/
const Device UA2528665 = {S39029, 88, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/2.9.15)*/
const Device UA2579429 = {S39030, 81, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Sleipnir/2.9.15)*/
const Device UA2596257 = {S39031, 88, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/2.9.14)*/
const Device UA2675878 = {S39032, 87, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.16)*/
const Device UA2676201 = {S39033, 80, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.16)*/
const Device UA2676955 = {S39034, 88, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/2.9.16)*/
const Device UA2679970 = {S39035, 81, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Sleipnir/2.9.16)*/
const Device UA2795311 = {S39036, 195, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Sleipnir/2.9.16)*/
const Device UA2799192 = {S39037, 88, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/2.9.17)*/
const Device UA2799287 = {S39038, 87, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.17)*/
const Device UA2799307 = {S39039, 81, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Sleipnir/2.9.17)*/
const Device UA2835479 = {S39040, 185, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Sleipnir/2.9.14)*/
const Device UA2842357 = {S39041, 81, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Sleipnir/2.9.14)*/
const Device UA2869438 = {S39042, 80, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.17)*/
const Device UA2952309 = {S39043, 88, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/2.9.10)*/
const Device UA2954654 = {S39044, 87, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/2.9.7)*/
const Device UA2957025 = {S39045, 87, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/2.9.9)*/
const Device UA2992458 = {S39046, 88, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/2.9.12)*/
const Device UA3023806 = {S39047, 81, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Sleipnir/2.9.18)*/
const Device UA3023817 = {S39048, 88, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/2.9.18)*/
const Device UA3023820 = {S39049, 87, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/2.9.18)*/
const Device UA3028161 = {S39050, 80, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/2.9.18)*/
const Device UA3071163 = {S39051, 80, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Sleipnir/2.9.9)*/
const Device UA3099472 = {S39052, 87, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/2.9.3)*/
const Device UA3101870 = {S39053, 81, { &P15364,&P7133,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Sleipnir/2.9.10)*/
const Device UA609302 = {S39054, 130, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA609398 = {S39055, 124, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  Build/ITL41F) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA634523 = {S39056, 124, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA902236 = {S39057, 123, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA902337 = {S39058, 133, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA903935 = {S39059, 129, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA908274 = {S39060, 137, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4.0.4.0.4; Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA909961 = {S39061, 129, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1950409 = {S39062, 127, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1954505 = {S39063, 127, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1957419 = {S39064, 139, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3.0.2.0.1.0; Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1982131 = {S39065, 144, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbl1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2023472 = {S39066, 133, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; STM712HCZ Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2027896 = {S39067, 147, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_hjy1a6x Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2028426 = {S39068, 144, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_hjy1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2030459 = {S39069, 133, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; STM805HCZ Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2034623 = {S39070, 146, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbaol1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2194369 = {S39071, 149, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbl1_5640 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2227018 = {S39072, 149, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1808_v75_jbl1_5640 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2227158 = {S39073, 137, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TWD_MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2230336 = {S39074, 129, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; V1013 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2261463 = {S39075, 144, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1808_v75_jbl2 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261486 = {S39076, 144, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbl2 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261529 = {S39077, 133, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2342262 = {S39078, 148, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbl1a535 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2349213 = {S39079, 129, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Build/ICS.g04ref.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2360246 = {S39080, 133, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; disgo9104 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2366561 = {S39081, 149, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_hjy1_5647 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2366732 = {S39082, 142, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Penta WS703C Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2373753 = {S39083, 128, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M711 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2376026 = {S39084, 128, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; T03A Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2377578 = {S39085, 139, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MAGPAD7.1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2379730 = {S39086, 137, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; jsa_MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2380271 = {S39087, 138, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Infinity Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2380577 = {S39088, 128, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M71G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2382027 = {S39089, 137, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; DFP7004 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2382339 = {S39090, 135, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M758A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2383265 = {S39091, 136, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; 756TPC Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2383398 = {S39092, 149, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1911_v77_jbaol_g20 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2383562 = {S39093, 135, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; X9015 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2383589 = {S39094, 134, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; classic101 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2384445 = {S39095, 136, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Penta WS702C Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2385500 = {S39096, 158, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; e1909c_v77_gq1008_9p017_6228 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2390313 = {S39097, 135, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A10??(8DM1) Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2393630 = {S39098, 138, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Tablet PC 2000 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2399290 = {S39099, 134, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; S5-N Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2399621 = {S39100, 138, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N101 DUAL CORE Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2400751 = {S39101, 136, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CJ ThL Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2401582 = {S39102, 144, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1911_v77_hjy1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2401681 = {S39103, 134, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T272 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2403589 = {S39104, 129, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Clear Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2404346 = {S39105, 136, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; UM-A13 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2412772 = {S39106, 131, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; unknown Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2413246 = {S39107, 134, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ARGM727HCX Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2415873 = {S39108, 137, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; unknown Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2418219 = {S39109, 152, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_hjy1a6x_5647 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2421622 = {S39110, 152, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; e1909c_v77_gq1008_5647 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2423801 = {S39111, 136, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Tender Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2424073 = {S39112, 153, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; e1911_v77_gq2000_ov5647 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2425367 = {S39113, 144, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; INFINITY_POWER Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2425311 = {S39114, 132, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; POM727MC Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2451002 = {S39115, 128, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; 720F Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2452277 = {S39116, 144, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1908_v77_jbl1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2453266 = {S39117, 131, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID 008 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2455675 = {S39118, 153, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; e1909c_v77_gq1008_9p017 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2457818 = {S39119, 133, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Pad Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2469254 = {S39120, 149, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1808_v75_jbl2_6628 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2481828 = {S39121, 132, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; EBM727KC Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2505111 = {S39122, 157, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1911_v77_mt6628_hjy1_9p017 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2506708 = {S39123, 134, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S150 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2523998 = {S39124, 136, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; NEOBOX Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2526217 = {S39125, 148, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1908_v77_jbl1a535 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2526607 = {S39126, 153, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbl1_5640a522 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2532955 = {S39127, 157, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; e1909c_v77_gq1008_5647_6228 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2553734 = {S39128, 129, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; FAR7 Build/V02R10A) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2589859 = {S39129, 118, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19*/
const Device UA2596658 = {S39130, 130, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TR720F Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2647559 = {S39131, 150, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_hjy1a6x_in Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2746640 = {S39132, 156, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1911_v77_jbaol_g20mt9p017 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2747812 = {S39133, 136, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NEOBOX Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2761955 = {S39134, 151, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbaol1_5647 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2837553 = {S39135, 151, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1908_v77_jbl1a535ntv Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2839729 = {S39136, 125, { &P17968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; X Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2197983 = {S39137, 135, { &P18113,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XT912 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2236723 = {S39138, 135, { &P18113,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT912 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2560426 = {S39139, 150, { &P18113,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2668941 = {S39140, 144, { &P18113,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA345259 = {S39141, 152, { &P13677,&P5267,&P3834,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5253/S5253DDKJ2; U; Bada/1.0; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA2197919 = {S39142, 147, { &P18614,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XT890 Build/8.7.1I-110_IFW-29) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2205922 = {S39143, 147, { &P18614,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XT890 Build/8.7.1I-110_IFW-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2237428 = {S39144, 150, { &P18614,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; XT890 Build/8.7.1I-110_IFW-DE-32) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257936 = {S39145, 147, { &P18614,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; XT890 Build/8.7.1I-110_SML-12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2355145 = {S39146, 147, { &P18614,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT890 Build/8.7.1I-110_SML-13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2366514 = {S39147, 150, { &P18614,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; XT890 Build/8.7.1I-110_IFW-DE-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2390975 = {S39148, 147, { &P18614,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XT890 Build/8.7.1I-110_IFW-39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2404129 = {S39149, 150, { &P18614,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; XT890 Build/8.7.1I-110_IFW-DE-39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422429 = {S39150, 150, { &P18614,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; XT890 Build/8.7.1I-110_IFW-DE-40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2425817 = {S39151, 149, { &P18614,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT890 Build/8.7.1I-110_SML-16.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2428248 = {S39152, 147, { &P18614,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT890 Build/8.7.1I-110_SML-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2456009 = {S39153, 149, { &P18614,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT890 Build/8.7.1I-110_SML-16.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483774 = {S39154, 69, { &P18614,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; XT890 Build/8.7.1I-110_IFW-39)*/
const Device UA2526286 = {S39155, 146, { &P18614,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; XT890 Build/8.7.1I-110_SML-8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2531358 = {S39156, 147, { &P18614,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT890 Build/8.7.1I-110_SML-20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2764309 = {S39157, 135, { &P18614,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XT890 Build/unknown) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2557844 = {S39158, 139, { &P18458,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I200 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2569520 = {S39159, 61, { &P18458,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-I200 Build/JZO54K)*/
const Device UA2988601 = {S39160, 137, { &P18458,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SCH-I200 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3126739 = {S39161, 141, { &P18458,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I200PP Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422651 = {S39162, 140, { &P23772,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; GT-I8190L Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2513335 = {S39163, 140, { &P23772,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-I8190L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555690 = {S39164, 62, { &P23772,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-I8190L Build/JRO03H)*/
const Device UA2598750 = {S39165, 62, { &P23772,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8190L Build/JZO54K)*/
const Device UA573071 = {S39166, 104, { &P15364,&P18110,&P19773,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.9) Gecko/X Mandriva/2.0.4-69.1mib2010.0 SeaMonkey/2.0.4*/
const Device UA2493126 = {S39167, 79, { &P15364,&P18110,&P19773,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1.17) Gecko/X SeaMonkey/2.0.12*/
const Device UA647887 = {S39168, 154, { &P11747,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; sv-se; HTC Desire Build/GRI40; Elelinux-CM7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA760382 = {S39169, 168, { &P11747,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MildWild for Desire Build/GWK 74 MildWild for Desire) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA784168 = {S39170, 158, { &P11747,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire Build/GWK74-MildWild for Desire) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA872638 = {S39171, 162, { &P11747,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Desire CDMA Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA907991 = {S39172, 155, { &P11747,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire Build/GRI40; MildWild CM-5.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA908380 = {S39173, 159, { &P11747,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire Build/GRI40; CyanogenMod-7.2.0.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026783 = {S39174, 147, { &P11747,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MildWild for Desire Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2685741 = {S39175, 82, { &P11747,&P18698,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Desire Build/GWK74-MildWild for Desire)*/
const Device UA22944 = {S39176, 139, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-tw; HTC_Glacier Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA39710 = {S39177, 138, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA39711 = {S39178, 138, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTCGlacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA39714 = {S39179, 138, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59684 = {S39180, 139, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Glacier Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59710 = {S39181, 139, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA69871 = {S39182, 138, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA70300 = {S39183, 138, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73140 = {S39184, 139, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Glacier Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA170486 = {S39185, 139, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC Glacier Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA180087 = {S39186, 139, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Glacier Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA193951 = {S39187, 139, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Glacier Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA209142 = {S39188, 139, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA234988 = {S39189, 63, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Glacier Build/GRJ22)*/
const Device UA249889 = {S39190, 139, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369269 = {S39191, 69, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.4; en_US; HTC Glacier Build/GRJ22)*/
const Device UA636002 = {S39192, 138, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC GlacierBuild/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA642856 = {S39193, 139, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Glacier Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1860891 = {S39194, 139, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; HTC Glacier Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019068 = {S39195, 63, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Glacier Build/GRI40)*/
const Device UA2019491 = {S39196, 139, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Glacier Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020078 = {S39197, 146, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Glacier Build/GRJ22) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2020380 = {S39198, 63, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Glacier Build/GRJ22)*/
const Device UA2021038 = {S39199, 137, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021277 = {S39200, 63, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Glacier Build/GRJ90)*/
const Device UA2023768 = {S39201, 152, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Glacier Build/GRI40; Absolution v3.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026456 = {S39202, 143, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027821 = {S39203, 139, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Glacier Build/DarKsyS 3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031710 = {S39204, 147, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Glacier Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2032324 = {S39205, 152, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Glacier Build/GRJ90; CyanUX V3.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2815896 = {S39206, 132, { &P15539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Glacier Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA330340 = {S39207, 107, { &P15364,&P16110,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; ) AppleWebKit/534.12 (KHTML, like Gecko) Maxthon/3.0 Safari/534.12*/
const Device UA2039853 = {S39208, 118, { &P15364,&P16110,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12*/
const Device UA2164342 = {S39209, 111, { &P15364,&P16110,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12*/
const Device UA2202182 = {S39210, 118, { &P15364,&P16110,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA2232349 = {S39211, 111, { &P15364,&P16110,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA2455849 = {S39212, 128, { &P15364,&P16110,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12,gzip(gfe)*/
const Device UA2462661 = {S39213, 105, { &P15364,&P16110,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; ) AppleWebKit/533.9 (KHTML, like Gecko) Maxthon/3.0 Safari/533.9*/
const Device UA2526715 = {S39214, 121, { &P15364,&P16110,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12,gzip(gfe)*/
const Device UA2844293 = {S39215, 128, { &P15364,&P16110,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1,gzip(gfe)*/
const Device UA2488849 = {S39216, 112, { &P12989,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia303/13.47; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA623085 = {S39217, 140, { &P18184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; YP-GS1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1941675 = {S39218, 64, { &P18184,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; YP-GS1 Build/GINGERBREAD)*/
const Device UA2691546 = {S39219, 138, { &P18184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; YP-GS1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2489609 = {S39220, 112, { &P18219,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia202/20.28; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2490638 = {S39221, 112, { &P18219,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia203/20.36; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA92707 = {S39222, 62, { &P15364,&P7133,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.5.24 Version/10.53*/
const Device UA101524 = {S39223, 62, { &P15364,&P7133,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.5.22 Version/10.50*/
const Device UA106744 = {S39224, 62, { &P15364,&P7133,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.5.24 Version/10.54*/
const Device UA130575 = {S39225, 62, { &P15364,&P7133,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.5.22 Version/10.51*/
const Device UA141474 = {S39226, 62, { &P15364,&P7133,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.5.24 Version/10.52*/
const Device UA446456 = {S39227, 99, { &P15364,&P7133,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.5.24 Version/10.53,gzip(gfe) (via translate.google.com)*/
const Device UA508477 = {S39228, 85, { &P15364,&P7133,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.5.22 Version/10.50*/
const Device UA1866976 = {S39229, 85, { &P15364,&P7133,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.5.24 Version/10.53*/
const Device UA2222178 = {S39230, 85, { &P15364,&P7133,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.5.24 Version/10.53*/
const Device UA2262794 = {S39231, 85, { &P15364,&P7133,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.5.24 Version/10.53*/
const Device UA2278916 = {S39232, 85, { &P15364,&P7133,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.5.22 Version/10.51*/
const Device UA2313710 = {S39233, 84, { &P15364,&P7133,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.5.24 Version/10.53*/
const Device UA2803589 = {S39234, 84, { &P15364,&P7133,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4120); ru) Presto/2.5.24 Version/10.53*/
const Device UA2815705 = {S39235, 85, { &P15364,&P7133,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5231); ru) Presto/2.5.24 Version/10.52*/
const Device UA2506898 = {S39236, 32, { &P15364,&P6729,&P4810,&P18092 }};	/*Useragent - Opera/7.03 (Windows 95; U)  [en]*/
const Device UA2541913 = {S39237, 72, { &P15364,&P6729,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows 95) Opera 7.03 [de]*/
const Device UA22267 = {S39238, 137, { &P15060,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; GT-S5660L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24608 = {S39239, 136, { &P15060,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5660 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42794 = {S39240, 97, { &P15060,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.2.1;Zh_cn;GT-S5660;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA178718 = {S39241, 60, { &P15060,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-S5660 Build/FROYO)*/
const Device UA758386 = {S39242, 124, { &P15060,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-S5660 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1967149 = {S39243, 129, { &P15060,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-S5660 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2462926 = {S39244, 134, { &P15060,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-; GT-S5660 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA616364 = {S39245, 129, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; Nexus 7 Build/JFN53N) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA912948 = {S39246, 131, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA924882 = {S39247, 131, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA925481 = {S39248, 60, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Nexus 7 Build/JRO03D)*/
const Device UA1961796 = {S39249, 129, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; Nexus 7 Build/JRN84D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1996856 = {S39250, 131, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2006928 = {S39251, 131, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2028347 = {S39252, 58, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1; Nexus 7 Build/JRN84D)*/
const Device UA2029133 = {S39253, 129, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Nexus 7 Build/JRO03D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2192583 = {S39254, 131, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03S) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2201623 = {S39255, 131, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2201774 = {S39256, 144, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA2203710 = {S39257, 60, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Nexus 7 Build/JZO54K)*/
const Device UA2205089 = {S39258, 131, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2238804 = {S39259, 146, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Ainovo Flame (Nexus 7) Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2241047 = {S39260, 129, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; Nexus 7 Build/JRO03S) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2241067 = {S39261, 138, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Google Nexus 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2241103 = {S39262, 138, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Google Nexus 7 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2241191 = {S39263, 138, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03S) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2241221 = {S39264, 158, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Nexus 7 Build/JRO03L by Team Shogun v1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2241230 = {S39265, 144, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA2241241 = {S39266, 142, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/CodenameSammy-1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2241273 = {S39267, 144, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03S) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA2241063 = {S39268, 138, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2241287 = {S39269, 144, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA2241289 = {S39270, 201, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Nexus 7 Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.2.1-65 (Android 4.1.1; asus Nexus 7 Build/JRO03D)*/
const Device UA2241295 = {S39271, 138, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2241311 = {S39272, 161, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2241348 = {S39273, 114, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.0.11 (Linux; U; Android 4.1.1; en-US; Nexus 7 Build/cm_elf2-userdebug 4.1.1 JRO03L eng.alan.X.X test-keys)*/
const Device UA2241352 = {S39274, 120, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.1.1; en-US; Nexus 7 Build/pa_grouper-userdebug 4.1.1 JRO03R eng.agascoig.X.X test-keys)*/
const Device UA2241353 = {S39275, 77, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.1.1; en-US; Nexus 7 Build/Glazed ICS V1.1.1)*/
const Device UA2241360 = {S39276, 66, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.1.1; en-US; Nexus 7 Build/JRO03R)*/
const Device UA2241367 = {S39277, 120, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.1.1; en-US; Nexus 7 Build/cm_grouper-userdebug 4.1.1 JRO03L eng.winner00.X.X test-keys)*/
const Device UA2241370 = {S39278, 113, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.1.2; en-US; Nexus 7 Build/cm_elf2-userdebug 4.1.2 JZO54K eng.alan.X.X test-keys)*/
const Device UA2241371 = {S39279, 120, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.1.2; en-US; Nexus 7 Build/pa_grouper-userdebug 4.1.2 JZO54K eng.agascoig.X.X test-keys)*/
const Device UA2241389 = {S39280, 131, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.0.11 (Linux; U; Android 4.1.1; en-US; Nexus 7 Build/pa_grouper-userdebug 4.1.1 JRO03L eng.root.X.X test-keys)*/
const Device UA2241416 = {S39281, 74, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - GoogleAnalytics/2.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03D)*/
const Device UA2241430 = {S39282, 93, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - com.google.android.apps.plus/X (Linux; U; Android 4.1.2; en_US; Nexus 7; Build/JZO54K) (gzip)*/
const Device UA2241434 = {S39283, 93, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - com.google.android.apps.plus/X (Linux; U; Android 4.1.1; en_US; Nexus 7; Build/JRO03D) (gzip)*/
const Device UA2241436 = {S39284, 91, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - com.google.android.videos/2.0.10(Linux; U; Android 4.1.2; en_US; Nexus 7 Build/JZO54K) gzip*/
const Device UA2241444 = {S39285, 91, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - com.google.android.youtube/4.0.8(Linux; U; Android 4.1.1; en_US; Nexus 7 Build/JRO03S) gzip*/
const Device UA2241457 = {S39286, 53, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Bible 4.0.9 (asus Nexus 7 nakasi;Android 4.1.1;en_US)*/
const Device UA2241461 = {S39287, 68, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - BBC iPlayer (AndroidApp; 1.3; 13014) Nexus 7 (Android 4.1.2, SDK 16)*/
const Device UA2241466 = {S39288, 55, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - BBCBeacon/0.73 (Nexus 7; Android 4.1.2) iplayer/1.4.2.3*/
const Device UA2241468 = {S39289, 61, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.0; 8) Nexus 7 (Android 4.1.2, SDK 16)*/
const Device UA2241482 = {S39290, 67, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Google Nexus 7 Build/JRO03H)*/
const Device UA2241483 = {S39291, 75, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Ainovo Flame (Nexus 7) Build/JRO03D)*/
const Device UA2241489 = {S39292, 159, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Ainovo Flame (Nexus 7) Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2241490 = {S39293, 144, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2241494 = {S39294, 128, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; Nexus 7 Build/JRN87) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2241465 = {S39295, 53, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Bible 4.0.7 (asus Nexus 7 nakasi;Android 4.1.1;en_US)*/
const Device UA2241540 = {S39296, 293, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Nexus 7 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 parameters/{scr=553_962,cm=1,ql=l,imei=10:bf:48:cb:dd:cb,bm=1,eng=adrmini16,mdm=1,mdl=Nexus 7,ps=1} tiantian(securitypay,securityinstalled) tiantian(tenpay)*/
const Device UA2241560 = {S39297, 142, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 GSA/2.0.0.X*/
const Device UA2241598 = {S39298, 127, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; Nexus 7 Build/JRN84D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2241613 = {S39299, 129, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Nexus 7 Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2241706 = {S39300, 147, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; Nexus 7 16G Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 GSA/2.0.5.X*/
const Device UA2241729 = {S39301, 138, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2241740 = {S39302, 131, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZ054K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2241748 = {S39303, 201, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.2.1-65 (Android 4.1.2; asus Nexus 7 Build/JZO54K)*/
const Device UA2241750 = {S39304, 129, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Nexus 7 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2241774 = {S39305, 129, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Nexus 7 Build/JRO03H) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2241789 = {S39306, 131, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2241820 = {S39307, 129, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; Nexus 7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2241893 = {S39308, 275, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/1.9.10;FBDM/{density=1.33125,width=800,height=1205};FBLC/en_US;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/Nexus 7;FBSV/4.1.2;]*/
const Device UA2241922 = {S39309, 144, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2241929 = {S39310, 132, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; Nexus 7 Build/ADEPT01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2241937 = {S39311, 158, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Nexus 7 Build/JZO54K by Team Shogun v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2241964 = {S39312, 136, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-gb; Nexus 7 Build/JRN84D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2241996 = {S39313, 143, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO54K) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2242070 = {S39314, 116, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Android 4.1.1; en-us; Nexus 7 Build/JRO03D) FreeWheelAdManager/5.0.2-r9017-X;com.gotv.crackle.handset/X*/
const Device UA2242072 = {S39315, 82, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 2.2.1); (Linux; U; Android 4.1.2; ASUS Nexus 7 Build/Unknown);*/
const Device UA2242080 = {S39316, 44, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Scanner Radio/3.8.3 (Android 4.1.1, Nexus 7)*/
const Device UA2851521 = {S39317, 157, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1_Generick_V6_by_darkrickpr; ru-ru; Nexus 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3073472 = {S39318, 130, { &P18385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/V5 HD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA50116 = {S39319, 47, { &P15364,&P22310,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows 3.1)*/
const Device UA521288 = {S39320, 55, { &P15364,&P22310,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Win3.1; ATHMWWW1.1;)*/
const Device UA2849889 = {S39321, 56, { &P15364,&P22310,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; Win3.11; Trident/4.0)*/
const Device UA2356692 = {S39322, 98, { &P15364,&P7133,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.1.8244(Windows;Microsoft Windows (6.1.7601.1);en;kml:2.2;client:Free;type:default)*/
const Device UA2386408 = {S39323, 98, { &P15364,&P7133,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.1.8244(Windows;Microsoft Windows (6.1.7601.1);de;kml:2.2;client:Free;type:default)*/
const Device UA2426926 = {S39324, 98, { &P15364,&P7133,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.2.8415(Windows;Microsoft Windows (6.1.7600.0);en;kml:2.2;client:Free;type:default)*/
const Device UA2428162 = {S39325, 98, { &P15364,&P7133,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.2.8415(Windows;Microsoft Windows (6.1.7601.1);de;kml:2.2;client:Free;type:default)*/
const Device UA2437267 = {S39326, 98, { &P15364,&P7133,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.2.8415(Windows;Microsoft Windows (6.1.7601.1);en;kml:2.2;client:Free;type:default)*/
const Device UA2566835 = {S39327, 98, { &P15364,&P7133,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.1.7601.1);ru;kml:2.2;client:Free;type:default)*/
const Device UA2567661 = {S39328, 122, { &P15364,&P7133,&P20429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/7.0.3.8542 Safari/532.4*/
const Device UA2567665 = {S39329, 98, { &P15364,&P7133,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.1.7601.1);en;kml:2.2;client:Free;type:default)*/
const Device UA2570277 = {S39330, 98, { &P15364,&P7133,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.1.7600.0);en;kml:2.2;client:Free;type:default)*/
const Device UA2570286 = {S39331, 98, { &P15364,&P7133,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.1.7601.1);de;kml:2.2;client:Free;type:default)*/
const Device UA2582958 = {S39332, 102, { &P15364,&P7133,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.1.7601.1);es-419;kml:2.2;client:Free;type:default)*/
const Device UA2601983 = {S39333, 98, { &P15364,&P7133,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.1.7601.1);nl;kml:2.2;client:Free;type:default)*/
const Device UA2603560 = {S39334, 98, { &P15364,&P7133,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.1.7601.1);pt;kml:2.2;client:Free;type:default)*/
const Device UA2603679 = {S39335, 98, { &P15364,&P7133,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.1.7601.1);fr;kml:2.2;client:Free;type:default)*/
const Device UA2604379 = {S39336, 98, { &P15364,&P7133,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.1.7601.1);it;kml:2.2;client:Free;type:default)*/
const Device UA2666593 = {S39337, 98, { &P15364,&P7133,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.1.7601.1);es;kml:2.2;client:Free;type:default)*/
const Device UA22736 = {S39338, 61, { &P18088,&P5448,&P22273,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320)*/
const Device UA80309 = {S39339, 68, { &P18088,&P5448,&P22273,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; Smartphone; 176x220)*/
const Device UA132948 = {S39340, 56, { &P18088,&P5448,&P22273,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; 240x320)*/
const Device UA378709 = {S39341, 184, { &P15364,&P5690,&P18147,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322; MS-RTC LM 8; InfoPath.2)*/
const Device UA516663 = {S39342, 217, { &P15364,&P5690,&P18147,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; chromeframe; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA551401 = {S39343, 245, { &P15364,&P5690,&P18147,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; OfficeLiveConnector.1.4; OfficeLivePatch.1.3)*/
const Device UA1841031 = {S39344, 134, { &P15364,&P5690,&P18147,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; chromeframe)*/
const Device UA2244155 = {S39345, 96, { &P15364,&P5690,&P18147,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; chromeframe; .NET CLR 2.0.50727)*/
const Device UA403388 = {S39346, 134, { &P18251,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; XT320 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1987960 = {S39347, 134, { &P18251,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; XT321 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1997046 = {S39348, 140, { &P18251,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; MOT-XT320 Build/0A.1F.3E) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027440 = {S39349, 122, { &P18251,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT320 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2029637 = {S39350, 58, { &P18251,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; XT320 Build/GRK39F)*/
const Device UA2223374 = {S39351, 140, { &P18251,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hr-hr; MOT-XT320 Build/0A.1F.3A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2226621 = {S39352, 140, { &P18251,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MOT-XT320 Build/0A.21.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257040 = {S39353, 140, { &P18251,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MOT-XT320 Build/0A.1F.37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2344327 = {S39354, 140, { &P18251,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; MOT-XT320 Build/0A.1F.49) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2393165 = {S39355, 133, { &P18251,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; XT320 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Versao/4.0 Mobile Safari/533.1*/
const Device UA2393949 = {S39356, 133, { &P18251,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; XT321 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Versao/4.0 Mobile Safari/533.1*/
const Device UA2414309 = {S39357, 140, { &P18251,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; MOT-XT320 Build/0A.1D.36) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2445648 = {S39358, 140, { &P18251,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; MOT-XT320 Build/0A.21.15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2501984 = {S39359, 140, { &P18251,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; MOT-XT320 Build/0A.1D.38) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2549789 = {S39360, 140, { &P18251,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; MOT-XT320 Build/0A.21.2C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2578189 = {S39361, 140, { &P18251,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pl-pl; MOT-XT320 Build/0A.21.2D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2722829 = {S39362, 139, { &P18251,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-pt; MOT-XT320 Build/0A.21.21) AppleWebKit/533.1 (KHTML, like Gecko) Versao/4.0 Mobile Safari/533.1*/
const Device UA2980277 = {S39363, 58, { &P18251,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; XT321 Build/GRK39F)*/
const Device UA99372 = {S39364, 41, { &P15364,&P17350,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Win32)*/
const Device UA104805 = {S39365, 46, { &P15364,&P17350,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT)*/
const Device UA117999 = {S39366, 47, { &P15364,&P17350,&P17193,&P18092 }};	/*Useragent - Mozilla/6.0 (compatible; MSIE 7.01; Windows NT)*/
const Device UA179288 = {S39367, 50, { &P15364,&P17350,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.5)*/
const Device UA257537 = {S39368, 98, { &P15364,&P17350,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0 Win32 +http://www.internetmemory.org, +http://www.parliament.uk)*/
const Device UA655599 = {S39369, 173, { &P15364,&P17350,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 7.1; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C)*/
const Device UA691868 = {S39370, 59, { &P15364,&P17350,&P17193,&P18092 }};	/*Useragent - Mozilla/6.0 (compatible;MSIE 7.0;  Windows NT 7.0; 360safe)*/
const Device UA698261 = {S39371, 58, { &P15364,&P17350,&P17193,&P18092 }};	/*Useragent - Mozilla/3.0 (compatible;MSIE 7.0;  Windows NT 7.0;  MyIE2)*/
const Device UA728367 = {S39372, 59, { &P15364,&P17350,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible;MSIE 7.0;  Windows NT 7.0; 360safe)*/
const Device UA808734 = {S39373, 59, { &P15364,&P17350,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible;MSIE 7.0;  Windows NT 7.0; 360safe)*/
const Device UA2216416 = {S39374, 43, { &P15364,&P17350,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows)*/
const Device UA2509883 = {S39375, 52, { &P15364,&P17350,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; 68K; Trident/4.0)*/
const Device UA2580074 = {S39376, 57, { &P15364,&P17350,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; 68K; SV1; Trident/2.0)*/
const Device UA2658860 = {S39377, 64, { &P15364,&P17350,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; 68K; Win64; x64; Trident/4.0)*/
const Device UA2658927 = {S39378, 59, { &P15364,&P17350,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; PPC; WOW64; Trident/3.0)*/
const Device UA2791673 = {S39379, 59, { &P15364,&P17350,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; 68K; WOW64; Trident/1.0)*/
const Device UA2791769 = {S39380, 62, { &P15364,&P17350,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; PPC; .NET CLR; Trident/5.0)*/
const Device UA2791828 = {S39381, 64, { &P15364,&P17350,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; PPC; Win64; x64; Trident/3.0)*/
const Device UA2486402 = {S39382, 156, { &P20192,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2532562 = {S39383, 150, { &P20192,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Orange Daytona Build/ESPC224B164) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558010 = {S39384, 147, { &P20192,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI G510-0010 Build/HuaweiG510-0010) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2659072 = {S39385, 154, { &P20192,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Orange Daytona Build/HuaweiG510-0200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2705961 = {S39386, 156, { &P20192,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HUAWEI G510-0200 Build/HuaweiG510-0200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2707944 = {S39387, 148, { &P20192,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HUAWEI G510-0100 Build/C02B171) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2729611 = {S39388, 78, { &P20192,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HUAWEI G510-0010 Build/HuaweiG510-0010)*/
const Device UA2738152 = {S39389, 148, { &P20192,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; HUAWEI G510-0100 Build/C02B170) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2748467 = {S39390, 156, { &P20192,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HUAWEI G510-0100 Build/HuaweiG510-0100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2765789 = {S39391, 76, { &P20192,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Orange Daytona Build/HuaweiG510-0200)*/
const Device UA2773648 = {S39392, 147, { &P20192,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Orange Daytona Build/C224B179) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2796873 = {S39393, 78, { &P20192,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HUAWEI G510-0200 Build/HuaweiG510-0200)*/
const Device UA2864026 = {S39394, 156, { &P20192,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; HUAWEI G510-0251 Build/HuaweiG510-0251) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2866735 = {S39395, 78, { &P20192,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HUAWEI G510-0100 Build/HuaweiG510-0100)*/
const Device UA2998269 = {S39396, 158, { &P20192,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HUAWEI Ascend G510 Build/HuaweiG510-0200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3005375 = {S39397, 78, { &P20192,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HUAWEI G510-0251 Build/HuaweiG510-0251)*/
const Device UA903803 = {S39398, 125, { &P15364,&P7133,&P22375,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.4.1.1000 Chrome/18.0.966.0 Safari/535.12*/
const Device UA904054 = {S39399, 118, { &P15364,&P7133,&P22375,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.4.1.1000 Chrome/18.0.966.0 Safari/535.12*/
const Device UA2595820 = {S39400, 125, { &P15364,&P7133,&P22375,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.4.5.2000 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA2984016 = {S39401, 118, { &P15364,&P7133,&P22375,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.4.5.2000 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA352905 = {S39402, 130, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/16.0.912.63) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA353166 = {S39403, 182, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/16.0.912.63; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA357618 = {S39404, 203, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/16.0.912.63; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; Tablet PC 2.0)*/
const Device UA366210 = {S39405, 130, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/16.0.912.75) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7*/
const Device UA366255 = {S39406, 123, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/16.0.912.75) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7*/
const Device UA366264 = {S39407, 88, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/16.0.912.75)*/
const Device UA368158 = {S39408, 130, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/16.0.912.77) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA368161 = {S39409, 123, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/16.0.912.77) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA368185 = {S39410, 95, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/16.0.912.77)*/
const Device UA397857 = {S39411, 198, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/16.0.912.75; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.2)*/
const Device UA437320 = {S39412, 155, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/16.0.912.75; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729)*/
const Device UA586729 = {S39413, 215, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/16.0.912.75; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3)*/
const Device UA587476 = {S39414, 196, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; chromeframe/16.0.912.77)*/
const Device UA638802 = {S39415, 123, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/16.0.912.63) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA754044 = {S39416, 170, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/16.0.912.75; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Tablet PC 2.0)*/
const Device UA828763 = {S39417, 216, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/16.0.912.77;  SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E)*/
const Device UA850704 = {S39418, 155, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/16.0.912.75; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729)*/
const Device UA2440904 = {S39419, 208, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/16.0.912.75; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA2474308 = {S39420, 170, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/16.0.912.75; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Tablet PC 2.0)*/
const Device UA2599808 = {S39421, 269, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/16.0.912.77;  SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA2747595 = {S39422, 96, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/16.0.912.77)*/
const Device UA2845336 = {S39423, 269, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/16.0.912.77;  SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; Microsoft Outlook 14.0.6126; ms-office; MSOffice 14)*/
const Device UA2989784 = {S39424, 89, { &P15364,&P7133,&P21902,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/16.0.912.75)*/
const Device UA175917 = {S39425, 7, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - desktop*/
const Device UA2282367 = {S39426, 5, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - Linux*/
const Device UA2933194 = {S39427, 5, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - D?xO?*/
const Device UA2933298 = {S39428, 2, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - -1*/
const Device UA2934093 = {S39429, 4, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - S728*/
const Device UA2937739 = {S39430, 6, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - wp.exe*/
const Device UA2949495 = {S39431, 13, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - -1 or 107=107*/
const Device UA2949633 = {S39432, 15, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - -1" or "70"="70*/
const Device UA2949721 = {S39433, 14, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - -1" or "39"="0*/
const Device UA2949852 = {S39434, 14, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - -1" or "77"="0*/
const Device UA2949889 = {S39435, 10, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - -1 or 58=0*/
const Device UA2950923 = {S39436, 6, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - 1.11.4*/
const Device UA2953771 = {S39437, 3, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - UMT*/
const Device UA2977347 = {S39438, 7, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - Toshiba*/
const Device UA2977415 = {S39439, 5, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - Flash*/
const Device UA2977675 = {S39440, 3, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - IGX*/
const Device UA2979320 = {S39441, 14, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - -1" or "27"="0*/
const Device UA2979607 = {S39442, 4, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - jack*/
const Device UA2979773 = {S39443, 14, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - -1" or "16"="0*/
const Device UA2979850 = {S39444, 14, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - -1' or '30'='0*/
const Device UA3008505 = {S39445, 4, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - S732*/
const Device UA3040714 = {S39446, 7, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - testing*/
const Device UA3058935 = {S39447, 7, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - fliptop*/
const Device UA3084855 = {S39448, 6, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - SN55TV*/
const Device UA3108219 = {S39449, 7, { &P15364,&P18588,&P18589,&P18092 }};	/*Useragent - Newsify*/
const Device UA2316959 = {S39450, 142, { &P18735,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-R820 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2459327 = {S39451, 66, { &P18735,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-R820 Build/GINGERBREAD)*/
const Device UA2952191 = {S39452, 140, { &P18735,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SCH-R820 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA575478 = {S39453, 76, { &P17595,&P17296,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:9.0a2) Gecko/X Firefox/9.0a2*/
const Device UA584909 = {S39454, 90, { &P17595,&P17296,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:9.0.1) Gecko/X Firefox/9.0.1 Firefox/9.0.1*/
const Device UA2643081 = {S39455, 90, { &P17595,&P17296,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:9.0a2) Gecko/X Firefox/9.0a2 Firefox 9.0.1*/
const Device UA68256 = {S39456, 102, { &P14802,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; SGH-i917)*/
const Device UA318681 = {S39457, 103, { &P14802,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; SGH-i917.)*/
const Device UA925798 = {S39458, 99, { &P14802,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; CETUS)*/
const Device UA925845 = {S39459, 102, { &P14802,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible: MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; SGH-i917)*/
const Device UA2765130 = {S39460, 117, { &P14802,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; WpsLondonTest; SAMSUNG; SGH-i917)*/
const Device UA2407288 = {S39461, 118, { &P15364,&P17285,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA2430138 = {S39462, 125, { &P15364,&P17285,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA2717810 = {S39463, 118, { &P15364,&P17285,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1*/
const Device UA2738264 = {S39464, 125, { &P15364,&P17285,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.6.2000 Chrome/26.0.1410.43 Safari/537.1*/
const Device UA33145 = {S39465, 142, { &P12867,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-MS910 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA640761 = {S39466, 66, { &P12867,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-MS910 Build/GINGERBREAD)*/
const Device UA1993195 = {S39467, 143, { &P12867,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-MS910 Build/GINGERBREAD+) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021182 = {S39468, 140, { &P12867,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; LG-MS910 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030237 = {S39469, 150, { &P12867,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-MS910 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2414691 = {S39470, 142, { &P12867,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-MS910 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA459241 = {S39471, 61, { &P15983,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-MS695 Build/GRK39F)*/
const Device UA459240 = {S39472, 137, { &P15983,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-MS695 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2719288 = {S39473, 67, { &P15364,&P18118,&P22171,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:23.0) Gecko/X Firefox/23.0*/
const Device UA627503 = {S39474, 141, { &P18685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; U8655-1 Build/HuaweiU8655) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA899470 = {S39475, 146, { &P18685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; STARTRAIL II Build/HuaweiU8655) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1996943 = {S39476, 142, { &P18685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; U8655-51 Build/HuaweiU8655) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032617 = {S39477, 65, { &P18685,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; U8655-1 Build/HuaweiU8655)*/
const Device UA2170014 = {S39478, 139, { &P18685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; U8655 Build/HuaweiU8655) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257184 = {S39479, 138, { &P18685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_C8655/C8655V100R001C92B818; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2388198 = {S39480, 152, { &P18685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; HUAWEI IDEOS Y 200 Build/HuaweiU8655) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2419445 = {S39481, 146, { &P18685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HUAWEI C8655 Build/HuaweiC8655) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2448011 = {S39482, 142, { &P18685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_C8655/C8655V100R001C92B816SP01; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2574430 = {S39483, 138, { &P18685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_C8655/C8655V100R001C92B816; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2701834 = {S39484, 142, { &P18685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; Etisalat Build/HuaweiU8655) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2727840 = {S39485, 141, { &P18685,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; MTC 965 Build/HuaweiU8655) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042767 = {S39486, 66, { &P18685,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; U8655-51 Build/HuaweiU8655)*/
const Device UA94062 = {S39487, 63, { &P15364,&P6729,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows 95) Opera 6.01  [en]*/
const Device UA412870 = {S39488, 62, { &P15364,&P6729,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows 95) Opera 6.02 [de]*/
const Device UA327165 = {S39489, 81, { &P17595,&P17296,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en) Presto/2.8.131 Version/11.11*/
const Device UA663602 = {S39490, 81, { &P17595,&P17296,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.7; U; en) Presto/2.8.131 Version/11.10*/
const Device UA1960331 = {S39491, 139, { &P18174,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VS920 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2362786 = {S39492, 61, { &P18174,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; VS920 4G Build/IMM76D)*/
const Device UA239128 = {S39493, 179, { &P15364,&P5690,&P21898,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/13.0.782.220; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA255761 = {S39494, 252, { &P15364,&P5690,&P21898,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/13.0.782.218; chromeframe; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA329715 = {S39495, 164, { &P15364,&P5690,&P21898,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/13.0.782.107; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA330814 = {S39496, 125, { &P15364,&P5690,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/13.0.782.107) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1*/
const Device UA335605 = {S39497, 125, { &P15364,&P5690,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/13.0.782.215) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.215 Safari/535.1*/
const Device UA336007 = {S39498, 125, { &P15364,&P5690,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/13.0.782.218) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.218 Safari/535.1*/
const Device UA337787 = {S39499, 125, { &P15364,&P5690,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/13.0.782.220) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1*/
const Device UA436980 = {S39500, 217, { &P15364,&P5690,&P21898,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/13.0.782.112; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8; .NET4.0C; .NET4.0E)*/
const Device UA437044 = {S39501, 192, { &P15364,&P5690,&P21898,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/13.0.782.220; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA572391 = {S39502, 128, { &P15364,&P5690,&P21898,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; Trident/4.0; chromeframe/13.0.782.112; .NET CLR 1.1.4322; .NET CLR 2.0.50727)*/
const Device UA574553 = {S39503, 179, { &P15364,&P5690,&P21898,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; chromeframe/13.0.782.215; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MS-RTC LM 8)*/
const Device UA738792 = {S39504, 125, { &P15364,&P5690,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/13.0.782.112) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1*/
const Device UA882955 = {S39505, 212, { &P15364,&P5690,&P21898,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/13.0.782.220; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA1897604 = {S39506, 76, { &P15364,&P5690,&P21898,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/13.0.782.220)*/
const Device UA2258818 = {S39507, 123, { &P15364,&P5690,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/13.0.767.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/13.0.767.1 Safari/534.36*/
const Device UA2258881 = {S39508, 123, { &P15364,&P5690,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/13.0.782.55) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.55 Safari/535.1*/
const Device UA2308915 = {S39509, 256, { &P15364,&P5690,&P21898,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; SearchToolbar 1.2; chromeframe/13.0.782.220; GTB7.2; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; AV3; .NET CLR 1.1*/
const Device UA2353875 = {S39510, 238, { &P15364,&P5690,&P21898,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/13.0.782.215; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Tablet PC 1.7; .NET CLR 2.0.50727; InfoPath.1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA2639169 = {S39511, 132, { &P15364,&P5690,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/13.0.782.112) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1*/
const Device UA2774613 = {S39512, 123, { &P15364,&P5690,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/13.0.782.32) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.32 Safari/535.1*/
const Device UA1995015 = {S39513, 132, { &P19780,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Large Screen Safari/534.24 GoogleTV/S67254*/
const Device UA2153100 = {S39514, 134, { &P19780,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.20 (KHTML, like Gecko) Chrome/11.0.672.2 Large Screen Safari/534.20 GoogleTV/X*/
const Device UA2232677 = {S39515, 130, { &P19780,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Large Screen Safari/534.24 GoogleTV/b70199*/
const Device UA2359980 = {S39516, 132, { &P19780,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; GoogleTV 3.2; LG Google TV Build/MASTER) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Safari/534.24*/
const Device UA2506325 = {S39517, 126, { &P19780,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; GoogleTV 3.2; VAP430 Build/MASTER) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Safari/534.24*/
const Device UA2558022 = {S39518, 132, { &P19780,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; GoogleTV 3.2; NSZ-GS7/GX70 Build/MASTER) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Safari/534.24*/
const Device UA2600698 = {S39519, 135, { &P19780,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; GoogleTV 3.2; LG Google TV G3 Build/MASTER) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Safari/534.24*/
const Device UA2708324 = {S39520, 126, { &P19780,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; GoogleTV 3.2; GTV100 Build/MASTER) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Safari/534.24*/
const Device UA2732522 = {S39521, 135, { &P19780,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; GoogleTV 3.2; hisense_gx1200v Build/MASTER) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Safari/534.24*/
const Device UA2752430 = {S39522, 136, { &P19780,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; GoogleTV 3.2; asus_google_cube Build/MASTER) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Safari/534.24*/
const Device UA3087384 = {S39523, 128, { &P19780,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; GoogleTV 3.2; TI320-DU Build/MASTER) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Safari/534.24*/
const Device UA2195951 = {S39524, 71, { &P18254,&P7480,&P18245,&P19830 }};	/*Useragent - Mozilla/5.0 (compatible; Plukkie/1.5; http://www.botje.com/plukkie.htm)*/
const Device UA2158799 = {S39525, 135, { &P18665,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N861 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2382773 = {S39526, 57, { &P18665,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; N861 Build/IMM76D)*/
const Device UA169529 = {S39527, 71, { &P15364,&P18110,&P20803,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/4.3; Linux) KHTML/4.3.5 (like Gecko)*/
const Device UA2332361 = {S39528, 145, { &P17989,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; X501_USA_Cricket Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2483166 = {S39529, 69, { &P17989,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; X501_USA_Cricket Build/GRK39F)*/
const Device UA2859213 = {S39530, 208, { &P17989,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; X501_USA_Cricket Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 androidAppVersion/2.5.22 AndroidWebkitWrapper BillingSupported*/
const Device UA3092091 = {S39531, 190, { &P17989,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; X501_USA_Cricket Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 androidAppVersion/2.6.8 AndroidWebkitWrapper*/
const Device UA49952 = {S39532, 131, { &P15236,&P6374,&P4418,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9930; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.254 Mobile Safari/534.11+*/
const Device UA429130 = {S39533, 73, { &P15236,&P6374,&P4418,&P18092 }};	/*Useragent - BlackBerry9930/7.0.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/105*/
const Device UA900701 = {S39534, 139, { &P18295,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One X Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA907798 = {S39535, 139, { &P18295,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One X Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1996121 = {S39536, 135, { &P18295,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; One X Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261487 = {S39537, 137, { &P18295,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTCOneX Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA2549785 = {S39538, 135, { &P18295,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S720e Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2658630 = {S39539, 139, { &P18295,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  HTC One X Build/IMM76D) AppleWebKit/535.19 (KHTML like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA2659416 = {S39540, 139, { &P18295,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  HTC One X Build/IML74K) AppleWebKit/535.19 (KHTML like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA913454 = {S39541, 76, { &P17595,&P17295,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:14.0) Gecko/X Firefox/14.0.1*/
const Device UA93227 = {S39542, 163, { &P12280,&P17134,&P18245,&P18091 }};	/*Useragent - AdsBot-Google-Mobile (+http://www.google.com/mobile/adsbot.html) Mozilla (iPhone; U; CPU iPhone OS 3 0 like Mac OS X) AppleWebKit (KHTML, like Gecko) Mobile Safari*/
const Device UA2741568 = {S39543, 118, { &P12280,&P17134,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us; EasouSpider; +http://www.easou.com/search/spider.html)*/
const Device UA2784854 = {S39544, 89, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro9%2C2)*/
const Device UA2796515 = {S39545, 86, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - ThumbnailService/39600.038 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro10%2C1)*/
const Device UA2800140 = {S39546, 89, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookAir5%2C2)*/
const Device UA2800651 = {S39547, 84, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (iMac13%2C1)*/
const Device UA2869494 = {S39548, 79, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - WebProcess/8536.30.1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro9%2C2)*/
const Device UA2875076 = {S39549, 86, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (Macmini6%2C2)*/
const Device UA2879590 = {S39550, 85, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - ThumbnailService/39600.038 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookAir5%2C2)*/
const Device UA2897082 = {S39551, 79, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - WebProcess/8536.30.1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookAir5%2C2)*/
const Device UA2938424 = {S39552, 89, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - Norton%20Identity%20Safe/1.03 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro10%2C1)*/
const Device UA2938820 = {S39553, 80, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - ThumbnailService/39600.038 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (iMac12%2C2)*/
const Device UA2958664 = {S39554, 69, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - Flint/165 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro10%2C1)*/
const Device UA2958848 = {S39555, 74, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - WebProcess/8536.30.1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (iMac13%2C2)*/
const Device UA2999863 = {S39556, 58, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - Miro/5.0.4 (http://www.getmiro.com/; Darwin 12.4.0 x86_64)*/
const Device UA3001184 = {S39557, 86, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBook7%2C1)*/
const Device UA3001465 = {S39558, 74, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - CoverScout%203/3.5.7 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (iMac12%2C2)*/
const Device UA3006890 = {S39559, 92, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - XProtectUpdater (unknown version) CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro9%2C2)*/
const Device UA3008984 = {S39560, 74, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - Socialite/7848 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro10%2C1)*/
const Device UA3050430 = {S39561, 85, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacPro5%2C1)*/
const Device UA3050455 = {S39562, 82, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - ThumbnailService/39600.038 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (Macmini6%2C2)*/
const Device UA3050627 = {S39563, 82, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - ThumbnailService/39600.038 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBook5%2C1)*/
const Device UA3050863 = {S39564, 79, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - CoverScout%203/3.5.7 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro9%2C2)*/
const Device UA3063000 = {S39565, 68, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - POP/0.24 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro10%2C1)*/
const Device UA3077880 = {S39566, 76, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - RSS%20Notifier/7 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro10%2C1)*/
const Device UA3081717 = {S39567, 67, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - POP/0.24 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookAir5%2C2)*/
const Device UA3088828 = {S39568, 76, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - WebProcess/8536.30.1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBook5%2C1)*/
const Device UA3114087 = {S39569, 62, { &P17595,&P22878,&P17470,&P18092 }};	/*Useragent - POP/0.24 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (iMac12%2C2)*/
const Device UA249906 = {S39570, 101, { &P15842,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e)*/
const Device UA356975 = {S39571, 114, { &P15842,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e; 2.01.161.04)*/
const Device UA421815 = {S39572, 114, { &P15842,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e; 1.08.161.02)*/
const Device UA552133 = {S39573, 97, { &P15842,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; PI39100)*/
const Device UA641968 = {S39574, 114, { &P15842,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e; 2.05.161.01)*/
const Device UA732745 = {S39575, 114, { &P15842,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e; 1.08.165.02)*/
const Device UA764392 = {S39576, 95, { &P15842,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; X310e)*/
const Device UA772485 = {S39577, 114, { &P15842,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e; 2.05.162.01)*/
const Device UA774953 = {S39578, 114, { &P15842,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e; 1.08.162.02)*/
const Device UA925733 = {S39579, 95, { &P15842,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Titan)*/
const Device UA925826 = {S39580, 114, { &P15842,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e; 4.06.162.01)*/
const Device UA925886 = {S39581, 114, { &P15842,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e; 4.06.163.03)*/
const Device UA925905 = {S39582, 114, { &P15842,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e; 4.06.165.02)*/
const Device UA925906 = {S39583, 114, { &P15842,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e; 4.06.161.01)*/
const Device UA2768512 = {S39584, 114, { &P15842,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; TITAN X310e; 4.07.162.03)*/
const Device UA94654 = {S39585, 34, { &P15364,&P17188,&P4811,&P18092 }};	/*Useragent - Opera/9.00 (Windows NT 4.0; U; en)*/
const Device UA2202803 = {S39586, 114, { &P15228,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-03/07.48; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2368286 = {S39587, 113, { &P15228,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-03/06.97; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5*/
const Device UA2672788 = {S39588, 62, { &P17968,&P17979,&P24344,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:23.0) Gecko/23.0 Firefox/23.0*/
const Device UA2735125 = {S39589, 68, { &P17968,&P17979,&P24344,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:23.0a1) Gecko/23.0a1 Firefox/23.0a1*/
const Device UA2799198 = {S39590, 68, { &P17968,&P17979,&P24344,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:23.0a2) Gecko/23.0a2 Firefox/23.0a2*/
const Device UA3057799 = {S39591, 101, { &P17968,&P17979,&P24344,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:23.0) Gecko/23.0 Firefox/23.0,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3092512 = {S39592, 72, { &P17968,&P17979,&P24344,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:23.0) Gecko/23.0 Firefox/23.0,gzip(gfe)*/
const Device UA378401 = {S39593, 81, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader/1569 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (iMac11%2C2)*/
const Device UA451249 = {S39594, 71, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - Spector%20Pro/3950 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (iMac10%2C1)*/
const Device UA552811 = {S39595, 78, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - CoverScout%203/3.4.3 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro7%2C1)*/
const Device UA557798 = {S39596, 86, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader/1444 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro7%2C1)*/
const Device UA560276 = {S39597, 83, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader/1569 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBook7%2C1)*/
const Device UA564211 = {S39598, 86, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader/1569 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookAir3%2C2)*/
const Device UA640825 = {S39599, 69, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - NewsFire/84 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro5%2C3)*/
const Device UA894646 = {S39600, 78, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - WebProcess/6534.57.2 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro7%2C1)*/
const Device UA895073 = {S39601, 75, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - WebProcess/6534.57.2 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBook7%2C1)*/
const Device UA1898931 = {S39602, 76, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - Spector%20Pro/3731 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro5%2C5)*/
const Device UA1965418 = {S39603, 73, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - WebProcess/6534.57.2 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (iMac10%2C1)*/
const Device UA2214681 = {S39604, 98, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - Google%20Quick%20Search%20Box/2.0.0.1447 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro5%2C5)*/
const Device UA2262875 = {S39605, 75, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - WebProcess/6534.57.2 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (Macmini3%2C1)*/
const Device UA2291692 = {S39606, 91, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - XProtectUpdater (unknown version) CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro7%2C1)*/
const Device UA2295255 = {S39607, 93, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - Google%20Quick%20Search%20Box/2.0.0.1447 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (iMac11%2C2)*/
const Device UA2313394 = {S39608, 66, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - NewsFire/85 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (Macmini1%2C1)*/
const Device UA2337692 = {S39609, 78, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - WebProcess/6534.57.2 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookAir3%2C2)*/
const Device UA2353708 = {S39610, 79, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - URL%20Extractor/3.4.1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro5%2C2)*/
const Device UA2361077 = {S39611, 90, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/454.12.4 Darwin/10.8.0 (x86_64) (MacBookPro8%2C1)*/
const Device UA2421561 = {S39612, 83, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/454.12.4 Darwin/10.8.0 (i386) (iMac11%2C2)*/
const Device UA2444249 = {S39613, 74, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - WebProcess/6534.57.2 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacPro1%2C1)*/
const Device UA2462888 = {S39614, 81, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - ScreenSaverEngine/3.0.3 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro6%2C2)*/
const Device UA2499680 = {S39615, 85, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBook7%2C1)*/
const Device UA2587272 = {S39616, 93, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - Software%20Update (unknown version) CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro7%2C1)*/
const Device UA2593966 = {S39617, 88, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookAir3%2C2)*/
const Device UA2638889 = {S39618, 88, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - XProtectUpdater (unknown version) CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBook7%2C1)*/
const Device UA2742104 = {S39619, 91, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - XProtectUpdater (unknown version) CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookAir3%2C2)*/
const Device UA2805998 = {S39620, 70, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - sysagent/1.0 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro7%2C1)*/
const Device UA2806370 = {S39621, 79, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - Software%20Update/270 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro7%2C1)*/
const Device UA2806974 = {S39622, 81, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - ocspd (unknown version) CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro6%2C2)*/
const Device UA2888855 = {S39623, 65, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - sysagent/1.0 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (iMac11%2C3)*/
const Device UA2933149 = {S39624, 90, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - Software%20Update (unknown version) CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBook7%2C1)*/
const Device UA2951369 = {S39625, 85, { &P17595,&P18577,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/454.12.4 Darwin/10.8.0 (i386) (Macmini4%2C1)*/
const Device UA2237666 = {S39626, 62, { &P17968,&P17979,&P23216,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:19.0) Gecko/19.0 Firefox/19.0*/
const Device UA2428052 = {S39627, 68, { &P17968,&P17979,&P23216,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:19.0a2) Gecko/19.0a2 Firefox/19.0a2*/
const Device UA2570874 = {S39628, 68, { &P17968,&P17979,&P23216,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:19.0.2) Gecko/19.0.2 Firefox/19.0.2*/
const Device UA2603592 = {S39629, 68, { &P17968,&P17979,&P23216,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:19.0.0) Gecko/19.0.0 Firefox/19.0.0*/
const Device UA2166077 = {S39630, 139, { &P18679,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-B5330 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2366980 = {S39631, 137, { &P18679,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-B5330 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2400330 = {S39632, 61, { &P18679,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-B5330 Build/IMM76D)*/
const Device UA2417891 = {S39633, 140, { &P18679,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; GT-B5330L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455691 = {S39634, 159, { &P18679,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; SAMSUNG GT-B5330/B5330XWALI4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2551489 = {S39635, 140, { &P18679,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT-B5330B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2652989 = {S39636, 159, { &P18679,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; SAMSUNG GT-B5330/B5330XWALI2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2833159 = {S39637, 159, { &P18679,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; SAMSUNG GT-B5330/B5330XWALH6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3085827 = {S39638, 62, { &P18679,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-B5330L Build/IMM76D)*/
const Device UA168768 = {S39639, 114, { &P15364,&P17285,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.1 (KHTML, like Gecko) Maxthon/3.0.8.2 Safari/533.1*/
const Device UA175399 = {S39640, 105, { &P15364,&P17285,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; ) AppleWebKit/533.9 (KHTML, like Gecko) Maxthon/3.0 Safari/533.9*/
const Device UA193118 = {S39641, 107, { &P15364,&P17285,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; ) AppleWebKit/534.12 (KHTML, like Gecko) Maxthon/3.0 Safari/534.12*/
const Device UA2039614 = {S39642, 111, { &P15364,&P17285,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12*/
const Device UA2152645 = {S39643, 118, { &P15364,&P17285,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12*/
const Device UA2238849 = {S39644, 111, { &P15364,&P17285,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA2259002 = {S39645, 118, { &P15364,&P17285,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA2367535 = {S39646, 121, { &P15364,&P17285,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12,gzip(gfe)*/
const Device UA2367973 = {S39647, 139, { &P18450,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST23i Build/11.0.A.5.5) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2432689 = {S39648, 140, { &P18450,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST23a Build/11.0.A.2.16) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA98458 = {S39649, 84, { &P12280,&P17779,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko)*/
const Device UA385278 = {S39650, 23, { &P12280,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPhone3C1/801.293*/
const Device UA2133285 = {S39651, 83, { &P12280,&P17779,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko)*/
const Device UA2199975 = {S39652, 24, { &P12280,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPhone4C1/1001.403*/
const Device UA2356521 = {S39653, 24, { &P12280,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPhone4C1/1001.523*/
const Device UA2356708 = {S39654, 24, { &P12280,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPhone5C2/1001.525*/
const Device UA2356717 = {S39655, 24, { &P12280,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPhone3C1/1001.523*/
const Device UA2356971 = {S39656, 24, { &P12280,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPhone5C1/1001.525*/
const Device UA2437577 = {S39657, 24, { &P12280,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPhone5C2/1001.551*/
const Device UA2532698 = {S39658, 24, { &P12280,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPhone5C2/1002.146*/
const Device UA2532752 = {S39659, 24, { &P12280,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPhone4C1/1002.146*/
const Device UA2605947 = {S39660, 24, { &P12280,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPhone5C2/1002.329*/
const Device UA2606858 = {S39661, 24, { &P12280,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPhone3C3/1002.329*/
const Device UA2606958 = {S39662, 24, { &P12280,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPhone3C1/1002.329*/
const Device UA2607374 = {S39663, 24, { &P12280,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPhone4C1/1002.329*/
const Device UA2709258 = {S39664, 61, { &P12280,&P17779,&P17596,&P18092 }};	/*Useragent - YellowPages-Intl/3.8.0-3349 (iPhone; iPhone OS; 6.1.3; en_CA)*/
const Device UA2710762 = {S39665, 24, { &P12280,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPhone5C1/1002.350*/
const Device UA2712549 = {S39666, 24, { &P12280,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPhone5C2/1002.350*/
const Device UA609355 = {S39667, 140, { &P18280,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  HTC One S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA611160 = {S39668, 140, { &P18280,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  HTC VLE_U Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA900711 = {S39669, 139, { &P18280,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC VLE_U Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA901723 = {S39670, 139, { &P18280,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1992588 = {S39671, 139, { &P18280,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2193494 = {S39672, 139, { &P18280,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC VLE_U Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2375481 = {S39673, 140, { &P18280,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  HTC One S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA2551777 = {S39674, 140, { &P18280,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  HTC VLE_U Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA2744530 = {S39675, 140, { &P18280,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One xs Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA549330 = {S39676, 90, { &P15364,&P17817,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux x86_64; U; Ubuntu/10.10 (maverick); en) Presto/2.7.62 Version/11.01*/
const Device UA682778 = {S39677, 97, { &P15364,&P16110,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.1 Safari/535.1*/
const Device UA2554849 = {S39678, 105, { &P15364,&P16110,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.4 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.4*/
const Device UA3064999 = {S39679, 128, { &P18385,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3065329 = {S39680, 129, { &P18385,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 7 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3065599 = {S39681, 127, { &P18385,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Nexus 7 Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2168188 = {S39682, 139, { &P18300,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T769 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2383822 = {S39683, 61, { &P18300,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-T769 Build/IMM76D)*/
const Device UA622884 = {S39684, 159, { &P14114,&P17296,&P18166,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us; Silk/1.1.0-84) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16 Silk-Accelerated=false*/
const Device UA2408424 = {S39685, 163, { &P14114,&P17296,&P18166,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us; Silk/1.0.22.153_X) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16 Silk-Accelerated=false*/
const Device UA615568 = {S39686, 139, { &P15364,&P17816,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/10.04 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19*/
const Device UA690855 = {S39687, 141, { &P15364,&P17816,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/10.04 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19*/
const Device UA1954849 = {S39688, 176, { &P15364,&P17816,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/10.04 Chromium/18.0.1025.151 Chrome/18.0.1025.151 Safari/535.19,gzip(gfe) (via translate.google.com)*/
const Device UA2840654 = {S39689, 149, { &P15364,&P17816,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/10.04 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19,gzip(gfe)*/
const Device UA2406372 = {S39690, 83, { &P15364,&P7133,&P22367,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.3) Gecko/X Firefox/15.3.2 PaleMoon/15.3.2*/
const Device UA2408214 = {S39691, 96, { &P15364,&P7133,&P22367,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:15.3) Gecko/X Firefox/15.3.2-x64 PaleMoon/15.3.2-x64*/
const Device UA924883 = {S39692, 76, { &P17595,&P18207,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:14.0) Gecko/X Firefox/14.0.1*/
const Device UA2169425 = {S39693, 91, { &P17595,&P18207,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:14.0) Gecko/X Firefox/14.0.1 Firefox/14.0.1*/
const Device UA3026561 = {S39694, 140, { &P17346,&P19826,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B329 Safari/8536.25*/
const Device UA3026970 = {S39695, 140, { &P17346,&P19826,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B146 Safari/8536.25*/
const Device UA3028777 = {S39696, 138, { &P17346,&P19826,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B141 Safari/8536.25*/
const Device UA3030236 = {S39697, 138, { &P17346,&P19826,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10B144 Safari/8536.25*/
const Device UA633041 = {S39698, 89, { &P17595,&P18120,&P18099,&P18092 }};	/*Useragent - Mozilla/6.0 (Macintosh; I; Intel Mac OS X 11_7_9; de-LI; rv:1.9b4) Gecko/X Firefox/10.0a4*/
const Device UA635342 = {S39699, 89, { &P17595,&P18120,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; I; Intel Mac OS X 11_7_9; de-LI; rv:1.9b4) Gecko/X Firefox/10.0a4*/
const Device UA601674 = {S39700, 67, { &P15364,&P18118,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:12.0) Gecko/X Firefox/12.0*/
const Device UA784741 = {S39701, 82, { &P15364,&P18118,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:12.0) Gecko/X Firefox/12.0 Firefox/12.0*/
const Device UA2014758 = {S39702, 152, { &P18552,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT897 Build/7.7.1Q-6_SPR-89_ASA-36) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2203346 = {S39703, 153, { &P18552,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT897 Build/7.7.1Q-6_SPR-125_CSP-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257412 = {S39704, 153, { &P18552,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT897 Build/7.7.1Q-6_SPR-125_ASA-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2368166 = {S39705, 153, { &P18552,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT897 Build/7.7.1Q-6_SPR-125_CSP-14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469277 = {S39706, 153, { &P18552,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT897 Build/7.7.1Q-6_SPR-125_ASA-14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2507971 = {S39707, 75, { &P18552,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; XT897 Build/7.7.1Q-6_SPR-125_ASA-14)*/
const Device UA2035269 = {S39708, 153, { &P18451,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyEricssonLT26ii Build/6.1.A.2.45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2035281 = {S39709, 140, { &P18451,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT26ii Build/6.1.A.2.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2035283 = {S39710, 132, { &P18451,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26ii Build/6.1.A.2.45) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2354334 = {S39711, 63, { &P18451,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT26ii Build/6.1.A.2.45)*/
const Device UA2416119 = {S39712, 141, { &P18451,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT26ii Build/6.1.A.2.45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455558 = {S39713, 141, { &P18451,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT26ii Build/6.1.A.2.50) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2530626 = {S39714, 141, { &P18451,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT26ii Build/6.1.A.2.55) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA4 = {S39715, 161, { &P12496,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-; SAMSUNG-SGH-I897/I897UCJH7 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA11 = {S39716, 163, { &P12496,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SAMSUNG-SGH-I897/I897UCJF1 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA14 = {S39717, 163, { &P12496,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SAMSUNG-SGH-I897/I897UCJE5 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA7068 = {S39718, 161, { &P12496,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-; SAMSUNG-SGH-I897/I897UCJF6 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA7084 = {S39719, 163, { &P12496,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SAMSUNG-SGH-I897/I897UCJE1 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32527 = {S39720, 153, { &P12496,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SAMSUNG-SGH-I897 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32722 = {S39721, 163, { &P12496,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SAMSUNG-SGH-I897/I897UCJF6 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32723 = {S39722, 155, { &P12496,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; SAMSUNG-SGH-I897/I897UCJF6 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32727 = {S39723, 163, { &P12496,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SAMSUNG-SGH-I897/I897UCJH7 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32728 = {S39724, 163, { &P12496,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SAMSUNG-SGH-I897/I897UCJH3 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32729 = {S39725, 163, { &P12496,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SAMSUNG-SGH-I897/I897UCJH2 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA37867 = {S39726, 162, { &P12496,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.1-update1; en-us; SAMSUNG-SGH-I897/I897UCJH7 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA37870 = {S39727, 162, { &P12496,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SAMSUNG-SGH-I897/I897UCJH7 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 MobileSafari/530.17*/
const Device UA51306 = {S39728, 162, { &P12496,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.1-update1; en-us; SAMSUNG-SGH-I897/I897UCJH7 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA76912 = {S39729, 81, { &P12496,&P5477,&P4861,&P18092 }};	/*Useragent - Logos/0.9.1 (Linux; U; Android 2.1-update1; en_US; SAMSUNG-SGH-I897 Build/ECLAIR)*/
const Device UA90021 = {S39730, 158, { &P12496,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SAMSUNG-SGH-I897 Build/ECLAIR) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA157340 = {S39731, 75, { &P12496,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SAMSUNG-SGH-I897 Build/ECLAIR)*/
const Device UA895509 = {S39732, 85, { &P15364,&P18375,&P17470,&P18092 }};	/*Useragent - HTTP_Request2/2.1.1 (http://pear.php.net/package/http_request2) PHP/5.3.2-1ubuntu4.15*/
const Device UA2010426 = {S39733, 85, { &P15364,&P18375,&P17470,&P18092 }};	/*Useragent - HTTP_Request2/2.1.1 (http://pear.php.net/package/http_request2) PHP/5.3.2-1ubuntu4.17*/
const Device UA2197911 = {S39734, 85, { &P15364,&P18375,&P17470,&P18092 }};	/*Useragent - HTTP_Request2/2.1.1 (http://pear.php.net/package/http_request2) PHP/5.3.2-1ubuntu4.18*/
const Device UA2757160 = {S39735, 85, { &P15364,&P18375,&P17470,&P18092 }};	/*Useragent - HTTP_Request2/2.1.1 (http://pear.php.net/package/http_request2) PHP/5.3.2-1ubuntu4.19*/
const Device UA99997 = {S39736, 124, { &P17595,&P17294,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.12*/
const Device UA100500 = {S39737, 125, { &P17595,&P17294,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA130896 = {S39738, 123, { &P17595,&P17294,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.28.3*/
const Device UA2652975 = {S39739, 128, { &P17595,&P17294,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10\_4\_11; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA2503000 = {S39740, 147, { &P19867,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I437 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597332 = {S39741, 147, { &P19867,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I437 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2669324 = {S39742, 69, { &P19867,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-I437 Build/JZO54K)*/
const Device UA2794164 = {S39743, 148, { &P19867,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I437Z Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2886363 = {S39744, 148, { &P19867,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I437P Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3016544 = {S39745, 70, { &P19867,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-I437P Build/JZO54K)*/
const Device UA3126277 = {S39746, 180, { &P19867,&P18111,&P18112,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=expressatt;devm=SAMSUNG-SGH-I437;man=samsung;brw=Android;shwd=480x800;shwa=480x763;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA2513852 = {S39747, 146, { &P18614,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; XT890 Build/9.8.2I-50_SMI-26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2561610 = {S39748, 146, { &P18614,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT890 Build/9.8.2I-50_SML-20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580844 = {S39749, 68, { &P18614,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT890 Build/9.8.2I-50_SMI-26)*/
const Device UA2581631 = {S39750, 146, { &P18614,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; XT890 Build/9.8.2I-50_SMI-29) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2679319 = {S39751, 146, { &P18614,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT890 Build/9.8.2I-50_SML-25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2713322 = {S39752, 146, { &P18614,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT890 Build/9.8.2I-50_SML-23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2849024 = {S39753, 146, { &P18614,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; XT890 Build/9.8.2I-50_SMI-23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2895050 = {S39754, 146, { &P18614,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT890 Build/9.8.2I-50_SML-28) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022748 = {S39755, 146, { &P18614,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT890 Build/9.8.2I-50_SML-29) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2535169 = {S39756, 62, { &P17968,&P17979,&P23653,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:22.0) Gecko/22.0 Firefox/22.0*/
const Device UA2735038 = {S39757, 68, { &P17968,&P17979,&P23653,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:22.0a2) Gecko/22.0a2 Firefox/22.0a2*/
const Device UA2749427 = {S39758, 68, { &P17968,&P17979,&P23653,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:22.0a1) Gecko/22.0a1 Firefox/22.0a1*/
const Device UA2957801 = {S39759, 101, { &P17968,&P17979,&P23653,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:22.0) Gecko/22.0 Firefox/22.0,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3009965 = {S39760, 72, { &P17968,&P17979,&P23653,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:22.0) Gecko/22.0 Firefox/22.0,gzip(gfe)*/
const Device UA2469414 = {S39761, 62, { &P17968,&P17979,&P23665,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:21.0) Gecko/21.0 Firefox/21.0*/
const Device UA2665490 = {S39762, 68, { &P17968,&P17979,&P23665,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:21.0a2) Gecko/21.0a2 Firefox/21.0a2*/
const Device UA2997572 = {S39763, 101, { &P17968,&P17979,&P23665,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:21.0) Gecko/21.0 Firefox/21.0,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2458158 = {S39764, 112, { &P18718,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 620)*/
const Device UA2516062 = {S39765, 122, { &P18718,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 620; Vodafone)*/
const Device UA2530254 = {S39766, 112, { &P18718,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia 620)*/
const Device UA2531600 = {S39767, 120, { &P18718,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 620; ANZ846)*/
const Device UA2716809 = {S39768, 120, { &P18718,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 620; Meteor)*/
const Device UA49995 = {S39769, 137, { &P18003,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6330VW Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA640914 = {S39770, 61, { &P18003,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; ADR6330VW Build/GRJ22)*/
const Device UA759725 = {S39771, 136, { &P18003,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;ADR6330VW Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA792228 = {S39772, 136, { &P18003,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; ADR6330VW Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA861149 = {S39773, 136, { &P18003,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6330VW Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA1907709 = {S39774, 136, { &P18003,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; ADR6330VW Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA34161 = {S39775, 136, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; MB525 Build/4.5.1_85) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48840 = {S39776, 137, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME525+ Build/4.5.3-66) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48845 = {S39777, 144, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB525 Build/4.5.1-134_DFP-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52706 = {S39778, 137, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; MB525 Build/4.5.1_112) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70366 = {S39779, 144, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME525+ Build/4.5.3-109_DPP-6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78839 = {S39780, 145, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; ME525+ Build/4.5.2-109_DHT-17) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA232243 = {S39781, 133, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB525 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250796 = {S39782, 133, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB525 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA322143 = {S39783, 133, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; MB525 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA421986 = {S39784, 145, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; ME525+ Build/4.5.2-109_DHT-25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622389 = {S39785, 139, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; MB525 Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30*/
const Device UA685549 = {S39786, 57, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; MB525 Build/GWK74)*/
const Device UA697549 = {S39787, 121, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MB525 Build/GWK74) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA700884 = {S39788, 135, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MB525 Build/WajkIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA700885 = {S39789, 133, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME525+ Build/4.5.3-109_DPP-14) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA701421 = {S39790, 145, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME525+ Build/4.5.3-109_DPP-11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA735875 = {S39791, 133, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME525+ Build/4.5.3-109_DPP-11) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA761630 = {S39792, 145, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME525+ Build/4.5.3-109_DPP-14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA787750 = {S39793, 132, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME525+ Build/4.5.3-109_DPP-6) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1907853 = {S39794, 121, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MB525 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1935335 = {S39795, 145, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; ME525+ Build/4.5.2-109_DHT-22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1975261 = {S39796, 137, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME525+ Build/4.5.3-109_DPP-6) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1982306 = {S39797, 126, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MB525 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1983794 = {S39798, 138, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME525+ Build/4.5.3-109_DPP-11) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1994443 = {S39799, 145, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME525+ Build/4.5.3-109_DPP-17) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995394 = {S39800, 138, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME525+ Build/4.5.3-109_DPP-14) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2018881 = {S39801, 134, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; MB525 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018908 = {S39802, 57, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; MB525 Build/GRJ22)*/
const Device UA2020099 = {S39803, 134, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MB525 Build/VIPFXY) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020185 = {S39804, 57, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; MB525 Build/GRJ90)*/
const Device UA2020244 = {S39805, 134, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MB525 Build/VIPFXY) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020368 = {S39806, 122, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MB525 Build/VIPFXY) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2020492 = {S39807, 144, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB525 Build/4.5.2-109_DHT-17) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022782 = {S39808, 135, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; iw-il; MB525 Build/HebMIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022787 = {S39809, 133, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME525+ Build/4.5.2-109_DHT-17) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2024324 = {S39810, 69, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; ME525+ Build/4.5.2-109_DHT-25)*/
const Device UA2028300 = {S39811, 133, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME525+ Build/4.5.3-109_DPP-17) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2028832 = {S39812, 144, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MB525 Build/4.5.2-109_DHT-25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033401 = {S39813, 138, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME525+ Build/4.5.3-109_DPP-17) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2223599 = {S39814, 135, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT555C Build/V1.67D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2393583 = {S39815, 138, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME525+ Build/4.5.2-109_DHT-25) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2503172 = {S39816, 59, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; XT555C Build/V1.67D)*/
const Device UA2722493 = {S39817, 125, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.7; zh-cn; MB525 Build MIUI) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2758858 = {S39818, 68, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; ME525+ Build/4.5.3-109_DPP-6)*/
const Device UA2812514 = {S39819, 145, { &P13505,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; MB525 Build/4.5.1-134_DFP-231) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2973885 = {S39820, 137, { &P18283,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T999 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974152 = {S39821, 136, { &P18283,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974271 = {S39822, 137, { &P18283,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I535 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975981 = {S39823, 137, { &P18283,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9300 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020931 = {S39824, 129, { &P18283,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022969 = {S39825, 137, { &P18283,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-T999 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA32208 = {S39826, 121, { &P12280,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1A543 Safari/419.3*/
const Device UA84799 = {S39827, 122, { &P12280,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1A538a Safari/419.3*/
const Device UA84801 = {S39828, 120, { &P12280,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C25 Safari/419.3*/
const Device UA84800 = {S39829, 122, { &P12280,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1A543a Safari/419.3*/
const Device UA84802 = {S39830, 120, { &P12280,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C28 Safari/419.3*/
const Device UA84803 = {S39831, 122, { &P12280,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/3B48b Safari/419.3*/
const Device UA84804 = {S39832, 122, { &P12280,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/3B48a Safari/419.3*/
const Device UA98239 = {S39833, 123, { &P12280,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/3A109a Safari/419.3*/
const Device UA127203 = {S39834, 121, { &P12280,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; XXXXX like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/241 Safari/419.3*/
const Device UA167067 = {S39835, 122, { &P12280,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/4A102 Safari/419.3*/
const Device UA169573 = {S39836, 121, { &P12280,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/4A93 Safari/419.3*/
const Device UA261599 = {S39837, 122, { &P12280,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1A537a Safari/419.3*/
const Device UA289897 = {S39838, 123, { &P12280,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/3A110a Safari/419.3*/
const Device UA294588 = {S39839, 108, { &P12280,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/8B117*/
const Device UA522804 = {S39840, 122, { &P12280,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1A498b Safari/419.3*/
const Device UA643393 = {S39841, 136, { &P12280,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/4A102 Safari/419 (United States)*/
const Device UA770081 = {S39842, 134, { &P12280,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C25 Safari/419.3 Safari/525.20*/
const Device UA2265285 = {S39843, 122, { &P12280,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1A535b Safari/419.3*/
const Device UA2373422 = {S39844, 115, { &P12280,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/u15 Safari/5*/
const Device UA2439697 = {S39845, 118, { &P12280,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/A2DUs2 Safari/5*/
const Device UA2720792 = {S39846, 121, { &P12280,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1A498b Safari/4.03*/
const Device UA48357 = {S39847, 146, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; XT860 Build/5.5.1-112_SLU-61.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48362 = {S39848, 146, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; XT860 Build/5.5.1-112_SLU-54.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61325 = {S39849, 144, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; XT860 Build/5.5.1-112_SLU-60) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370929 = {S39850, 145, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT860 Build/5.5.1-112_SLU-57M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA635581 = {S39851, 147, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; XT860 Build/5.5.1-112_SLU-47.6M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA640151 = {S39852, 145, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT860 Build/5.5.1-112_SLU-56M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA651027 = {S39853, 142, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; ME863 Build/SLNCU-60.2.200) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA656437 = {S39854, 144, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; XT860 Build/5.5.1-112_SLU-89) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA662165 = {S39855, 144, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT860 Build/5.5.1-112_SLU-79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA692575 = {S39856, 144, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT860 Build/5.5.1-112_SLU-88) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA701496 = {S39857, 142, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; ME863 Build/SLNCU-60.1.103) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA701880 = {S39858, 145, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; XT860 Build/5.5.1-112_SLU-44M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA743248 = {S39859, 145, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; XT860 Build/5.5.1-112_SLU-39M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA803819 = {S39860, 145, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; XT860 Build/5.5.1-112_SLU-42M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020279 = {S39861, 146, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; XT860 Build/5.5.1-112_SLU-54.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022371 = {S39862, 71, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; XT860 Build/5.5.1-112_SLU-47.6M)*/
const Device UA2024065 = {S39863, 142, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ME863 Build/SLNCU-60.1.101) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027205 = {S39864, 142, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ME863 Build/SLNCU-60.1.102) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031575 = {S39865, 144, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT860 Build/5.5.1-112_SLU-87) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2197719 = {S39866, 150, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MILESTONE3 Build/5.5.1.1-84_D3G-25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2762654 = {S39867, 74, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; MILESTONE3 Build/5.5.1.1-84_D3G-25)*/
const Device UA2775203 = {S39868, 66, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; ME863 Build/SLNCU-60.2.200)*/
const Device UA2809589 = {S39869, 149, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; XT860 Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2823086 = {S39870, 135, { &P20969,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; ME863 Build SLNCU-60.1.102) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA363327 = {S39871, 142, { &P18150,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5363 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA576662 = {S39872, 142, { &P18150,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-S5363 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA765740 = {S39873, 66, { &P18150,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5363 Build/GINGERBREAD)*/
const Device UA1940584 = {S39874, 66, { &P18150,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-S5363 Build/GINGERBREAD)*/
const Device UA2399936 = {S39875, 154, { &P18150,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6_mobile-star; iw-il; GT-S5363 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2418802 = {S39876, 140, { &P18150,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5363 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2717934 = {S39877, 128, { &P15767,&P18432,&P22902,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.4.2 Mobile/10A523 Safari/8536.25*/
const Device UA2752825 = {S39878, 126, { &P15767,&P18432,&P22902,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.4.2 Mobile/10A403 Safari/8536.25*/
const Device UA2945959 = {S39879, 88, { &P18518,&P5567,&P17370,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/5.0/27.2338; U; en) Presto/2.8.119 320X240 Samsung SCH-U380*/
const Device UA2042398 = {S39880, 116, { &P15767,&P18432,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Twitter for iPhone*/
const Device UA2260679 = {S39881, 118, { &P15767,&P18432,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Twitter for iPhone*/
const Device UA2356710 = {S39882, 116, { &P15767,&P18432,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A407 Twitter for iPhone*/
const Device UA2356730 = {S39883, 116, { &P15767,&P18432,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A406 Twitter for iPhone*/
const Device UA2369748 = {S39884, 119, { &P15767,&P18432,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8426 Twitter for iPhone*/
const Device UA2427593 = {S39885, 119, { &P15767,&P18432,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8500 Twitter for iPhone*/
const Device UA2437619 = {S39886, 118, { &P15767,&P18432,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550 Twitter for iPhone*/
const Device UA1983664 = {S39887, 68, { &P15364,&P18346,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; SV1)*/
const Device UA1990877 = {S39888, 78, { &P15364,&P18346,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; MSN Optimized)*/
const Device UA2167259 = {S39889, 68, { &P15364,&P18346,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; SV1)*/
const Device UA2167653 = {S39890, 76, { &P15364,&P18346,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; MS-RTC LM 8)*/
const Device UA2813815 = {S39891, 73, { &P15364,&P18346,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; SV1; SV1)*/
const Device UA2817000 = {S39892, 73, { &P15364,&P18346,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; SV1; SV1)*/
const Device UA2173875 = {S39893, 148, { &P17989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V8000_USA_Cricket Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2420898 = {S39894, 70, { &P17989,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; V8000_USA_Cricket Build/IMM76D)*/
const Device UA2554316 = {S39895, 148, { &P17989,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N8000_USA_Cricket Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2992782 = {S39896, 70, { &P17989,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; N8000_USA_Cricket Build/IMM76I)*/
const Device UA2717935 = {S39897, 137, { &P12280,&P18432,&P22902,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.4.2 Mobile/10A523 Safari/8536.25*/
const Device UA2753576 = {S39898, 135, { &P12280,&P18432,&P22902,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.4.2 Mobile/10A403 Safari/8536.25*/
const Device UA2602576 = {S39899, 149, { &P15877,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2668207 = {S39900, 71, { &P15877,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14)*/
const Device UA2669044 = {S39901, 148, { &P15877,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; XT910 Build/9.8.2O-124_SPUL-18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2700286 = {S39902, 153, { &P15877,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MOTOROLA RAZR Build/9.8.2O-72_VZW-8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2715165 = {S39903, 148, { &P15877,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT910 Build/9.8.2O-124_SPUL-17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2729731 = {S39904, 146, { &P15877,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MOT-XT910 Build/6.7.2_GC-384) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2729768 = {S39905, 155, { &P15877,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MOT-XT910 Build/6.7.2_GC-384-SPDU-801) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2838385 = {S39906, 136, { &P15877,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT910 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2850592 = {S39907, 142, { &P15877,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; XT910 Build/6.7.2_GC-384) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2853963 = {S39908, 151, { &P15877,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; XT910 Build/6.7.2_GC-384-SPDU-801) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3119454 = {S39909, 146, { &P15877,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT910 Build/9.8.2O-72_VZW-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708579 = {S39910, 141, { &P12280,&P18117,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9B176 Safari/7534.48.3*/
const Device UA2717577 = {S39911, 143, { &P12280,&P18117,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9B208 Safari/7534.48.3*/
const Device UA2717582 = {S39912, 141, { &P12280,&P18117,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9B179 Safari/7534.48.3*/
const Device UA2717586 = {S39913, 143, { &P12280,&P18117,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9B206 Safari/7534.48.3*/
const Device UA2996396 = {S39914, 143, { &P12280,&P18117,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9Z999 Safari/7534.48.3*/
const Device UA633519 = {S39915, 136, { &P17160,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Slider SL101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA764195 = {S39916, 65, { &P17160,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Slider SL101 Build/IML74K)*/
const Device UA683350 = {S39917, 70, { &P17595,&P17295,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3;) Gecko/X Firefox/3.0*/
const Device UA726708 = {S39918, 82, { &P17595,&P17295,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6 ; nl; rv:1.9) Gecko/X Firefox/3.0*/
const Device UA2350954 = {S39919, 164, { &P17595,&P17295,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.2a1pre) Gecko/X Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.5; en-US; rv:1.9.0.7) Gecko/X Firefox/3.0.7*/
const Device UA2541430 = {S39920, 86, { &P17595,&P17295,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.1b1) Gecko/X Firefox/3.0*/
const Device UA92402 = {S39921, 60, { &P18254,&P6729,&P4551,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01; Windows 95; MSIECrawler)*/
const Device UA1898149 = {S39922, 60, { &P18254,&P6729,&P4551,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows 95) VoilaBot; 1.6*/
const Device UA1898659 = {S39923, 88, { &P18254,&P6729,&P4551,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows 95) VoilaBot BETA 1.2 (http://www.voila.com/)*/
const Device UA427345 = {S39924, 124, { &P12280,&P18117,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Twitter for iPhone*/
const Device UA427356 = {S39925, 124, { &P12280,&P18117,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Twitter for iPhone*/
const Device UA615511 = {S39926, 126, { &P12280,&P18117,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Twitter for iPhone*/
const Device UA808977 = {S39927, 124, { &P12280,&P18117,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Twitter for iPhone*/
const Device UA898906 = {S39928, 126, { &P12280,&P18117,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Twitter for iPhone*/
const Device UA2165316 = {S39929, 124, { &P12280,&P18117,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Twitter for iPhone*/
const Device UA2712353 = {S39930, 164, { &P22585,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SPH-L720 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA626404 = {S39931, 142, { &P19854,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-R760 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA755219 = {S39932, 66, { &P19854,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-R760 Build/GINGERBREAD)*/
const Device UA720487 = {S39933, 86, { &P15364,&P16110,&P22733,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Avant Browser)*/
const Device UA789296 = {S39934, 79, { &P15364,&P16110,&P22733,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; Avant Browser)*/
const Device UA1987345 = {S39935, 91, { &P15364,&P16110,&P22733,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Win64; x64; Trident/6.0; Avant Browser)*/
const Device UA2676768 = {S39936, 137, { &P18713,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2678955 = {S39937, 137, { &P18713,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P768 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2701576 = {S39938, 137, { &P18713,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P769 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2872562 = {S39939, 137, { &P18713,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LGMS769 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2974042 = {S39940, 137, { &P15575,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974441 = {S39941, 138, { &P15575,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974602 = {S39942, 138, { &P15575,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975060 = {S39943, 138, { &P15575,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022511 = {S39944, 137, { &P15575,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9100 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3058134 = {S39945, 130, { &P15575,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA144083 = {S39946, 111, { &P15364,&P7480,&P18107,&P18143 }};	/*Useragent - Mozilla/5.0 (compatible; Firefox compatible; MS IE compatible; +http://search.daum.net/) Daumoa-feedfetcher/2.0*/
const Device UA593573 = {S39947, 132, { &P15364,&P7480,&P18107,&P18143 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE or Firefox mutant; not on Windows server; + http://tab.search.daum.net/aboutWebSearch.html) Daumoa/3.0*/
const Device UA21782 = {S39948, 156, { &P12400,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN95_8GB-3;Mozilla/5.0 SymbianOS/9.2;U; Series60/3.1 NokiaN95_8GB-3/11.2.011 Profile/MIDP-2.0 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML, like Gecko)*/
const Device UA21784 = {S39949, 155, { &P12400,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaN95_8GB-3/1.2.011 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA31123 = {S39950, 155, { &P12400,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaN95_8GB/20.0.016; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA453554 = {S39951, 132, { &P18712,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Next8P12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1960286 = {S39952, 61, { &P18712,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Next8P12 Build/IML74K)*/
const Device UA2202798 = {S39953, 130, { &P18712,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; NEXT8P Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2362337 = {S39954, 132, { &P18712,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Next8P12 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2596258 = {S39955, 61, { &P18712,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Next8P12 Build/IMM76I)*/
const Device UA2777277 = {S39956, 132, { &P18712,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NEXT8P12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2406916 = {S39957, 69, { &P15364,&P18110,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 on x86_64; rv:18.0) Gecko/X Firefox/18.0*/
const Device UA2490676 = {S39958, 76, { &P15364,&P18110,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:18.0) Gecko/X Firefox/18.0 Iceweasel/18.0.1*/
const Device UA2702159 = {S39959, 98, { &P15364,&P18110,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (Ultimate Edition; X11; Linux x64; rv:8.0) Gecko/X Ultimate Edition / 3.4 Firefox/18.0*/
const Device UA32562 = {S39960, 153, { &P12004,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaN95-5/22.1.002; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA64226 = {S39961, 152, { &P12004,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaN95-3/20.2.011 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA64460 = {S39962, 151, { &P12004,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaN95/35.0.002; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA81129 = {S39963, 8, { &P12004,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN95*/
const Device UA120962 = {S39964, 12, { &P12004,&P5890,&P3439,&P18092 }};	/*Useragent - NOKIAN95 8GB*/
const Device UA2892295 = {S39965, 89, { &P12004,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN95/1.0(15.1.002)SymbianOS/9.2 Series60/3.01 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2003032 = {S39966, 116, { &P14635,&P5843,&P17438,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-00/so6.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15*/
const Device UA3033024 = {S39967, 141, { &P17346,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A4449a Safari/9537.53*/
const Device UA3034369 = {S39968, 141, { &P17346,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A4449d Safari/9537.53*/
const Device UA3083302 = {S39969, 139, { &P17346,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A465 Safari/9537.53*/
const Device UA3115365 = {S39970, 141, { &P17346,&P22882,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A501 Safari/9537.53*/
const Device UA26442 = {S39971, 137, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Nexus One Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38517 = {S39972, 138, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Nexus One Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA54621 = {S39973, 135, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA54669 = {S39974, 137, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; NexusOne Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA54670 = {S39975, 134, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; NexusOne Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA89871 = {S39976, 135, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; Nexus One Build/FRF50) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA109685 = {S39977, 137, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus One Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA273462 = {S39978, 135, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus One Build/FRG33) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA291895 = {S39979, 138, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Nexus One Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA292902 = {S39980, 62, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; Nexus One Build/FRG83D)*/
const Device UA299609 = {S39981, 136, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Nexus One Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA462860 = {S39982, 136, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus One Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA481792 = {S39983, 135, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus One Build/FRF83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA491528 = {S39984, 155, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Nexus One Build/RoDrIgUeZsTyLe MIUI-MOD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA678712 = {S39985, 137, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en_us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA687404 = {S39986, 159, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; PocketEdge FROYO; Android 2.2; en-us; Nexus One Build/FRG83) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA1875247 = {S39987, 134, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en; Nexus One Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018069 = {S39988, 59, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; Nexus One Build/FRF91)*/
const Device UA2018142 = {S39989, 135, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus One Build/FRF51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018279 = {S39990, 135, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus One Build/FRF58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023942 = {S39991, 130, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android 2.2.1;en-us;Nexus One Build.FRG83) AppleWebKit/553.1(KHTML,like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2381303 = {S39992, 136, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2600766 = {S39993, 136, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2659089 = {S39994, 135, { &P16036,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus One Build/ FRF91) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA403357 = {S39995, 133, { &P18230,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; A1_07 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA718427 = {S39996, 57, { &P18230,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; A1_07 Build/GRJ22)*/
const Device UA767164 = {S39997, 121, { &P18230,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; A1_07 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2020731 = {S39998, 149, { &P18230,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; A1_07 Build/GRJ22; 600*1024) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2032145 = {S39999, 126, { &P18230,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; A1_07 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA781007 = {S40000, 150, { &P19787,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; T-Mobile myTouch Build/HuaweiU8680) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2190057 = {S40001, 74, { &P19787,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; T-Mobile myTouch Build/HuaweiU8680)*/
const Device UA2416311 = {S40002, 149, { &P19787,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us;T-Mobile myTouch Build/HuaweiU8680) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2419175 = {S40003, 149, { &P19787,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.6; en-us; T-Mobile myTouch Build/HuaweiU8680) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2451363 = {S40004, 149, { &P19787,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; T-Mobile myTouch Build/HuaweiU8680) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA2452284 = {S40005, 149, { &P19787,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.6; en-us; T-Mobile myTouch Build/HuaweiU8680) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2465078 = {S40006, 149, { &P19787,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; T-MobilemyTouch Build/HuaweiU8680) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2465818 = {S40007, 149, { &P19787,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; T-Mobile myTouchBuild/HuaweiU8680) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2709570 = {S40008, 141, { &P18449,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; ST27i Build/6.2.A.0.400) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867731 = {S40009, 141, { &P18449,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-nz; ST27a Build/6.2.A.0.400) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879919 = {S40010, 145, { &P18449,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyST27i Build/6.2.A.1.100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2880206 = {S40011, 63, { &P18449,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; ST27i Build/6.2.A.1.100)*/
const Device UA2884703 = {S40012, 141, { &P18449,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; ST27i Build/6.2.A.1.100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2964172 = {S40013, 145, { &P18449,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyST27a Build/6.2.A.1.100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3013494 = {S40014, 63, { &P18449,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; ST27a Build/6.2.A.1.100)*/
const Device UA3017221 = {S40015, 146, { &P18449,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-za; SonyST27iv Build/6.2.A.1.100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3075319 = {S40016, 141, { &P18449,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; ST27a Build/6.2.A.1.100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3034961 = {S40017, 106, { &P19957,&P17484,&P17369,&P18092 }};	/*Useragent - SAMSUNG-GT-E1282T Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.31524/30.3697; U; en) Presto/2.8.119 Version/11.10*/
const Device UA3020905 = {S40018, 132, { &P18465,&P18110,&P24334,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Safari/535.19 Silk-Accelerated=true*/
const Device UA48383 = {S40019, 144, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC EVO 3D X515m Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48400 = {S40020, 143, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC_EVO3D_X515m Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49260 = {S40021, 143, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; HTC_EVO3D_X515a Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49293 = {S40022, 139, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_Kingdom Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60594 = {S40023, 139, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; vi-vn; HTC_Kingdom Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61182 = {S40024, 137, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; HTC_X515E Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72024 = {S40025, 149, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; HTC_X515a V1.24.841.2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA236722 = {S40026, 144, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; HTC EVO 3D X515a Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369660 = {S40027, 62, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; HTC_X515E Build/GRK39F)*/
const Device UA370498 = {S40028, 138, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-kr; HTC_X515E Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370767 = {S40029, 127, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HTC X515d) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA566456 = {S40030, 61, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC_X515E Build/GRJ22)*/
const Device UA622337 = {S40031, 150, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; HTC EVO 3D X515a Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30*/
const Device UA649458 = {S40032, 68, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC EVO 3D X515m Build/GRJ22)*/
const Device UA652587 = {S40033, 68, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC EVO 3D X515a Build/GRJ22)*/
const Device UA663500 = {S40034, 132, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC EVO 3D X515m Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA713424 = {S40035, 149, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; HTC_X515a V1.24.841.3 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA902769 = {S40036, 137, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC EVO 3D X515m Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2022384 = {S40037, 125, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC X515m Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2024405 = {S40038, 137, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC X515m Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024566 = {S40039, 137, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC EVO 3D X515m Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2024871 = {S40040, 140, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HTC-X515d/1.09.1401.13) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026085 = {S40041, 130, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC X515m Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2026795 = {S40042, 132, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC EVO 3D X515m Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2027728 = {S40043, 137, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.4; zh-cn; HTC EVO 3D X515m Build GRJ22) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2033083 = {S40044, 144, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC EVO 3D X515m Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2815608 = {S40045, 131, { &P12657,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HTC X515d Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA61670 = {S40046, 161, { &P15373,&P5129,&P3690,&P18092 }};	/*Useragent - Nokia5530/32.0.007 (SymbianOS/9.4; U; Series60/5.0 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Safari/525 3gpp-gba*/
const Device UA64708 = {S40047, 180, { &P15373,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5530c-2/32.0.007; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.9 3gpp-gba*/
const Device UA69532 = {S40048, 9, { &P15373,&P5129,&P3690,&P18092 }};	/*Useragent - Nokia5530*/
const Device UA251973 = {S40049, 194, { &P15373,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5530c-2/40.0.003; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 Mobile Safari/533.4 3gpp-gba*/
const Device UA614780 = {S40050, 64, { &P17968,&P17979,&P18628,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:14.0) Gecko/14.0 Firefox/14.0a2*/
const Device UA821828 = {S40051, 64, { &P17968,&P17979,&P18628,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:14.0) Gecko/14.0 Firefox/14.0a1*/
const Device UA876431 = {S40052, 62, { &P17968,&P17979,&P18628,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:14.0) Gecko/14.0 Firefox/14.0*/
const Device UA905456 = {S40053, 61, { &P17968,&P17979,&P18628,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:14.0) Gecko/14.0 Fennec/14.0*/
const Device UA913433 = {S40054, 64, { &P17968,&P17979,&P18628,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:14.0) Gecko/14.0 Firefox/14.0.1*/
const Device UA2001872 = {S40055, 64, { &P17968,&P17979,&P18628,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:14.0) Gecko/14.0 Firefox/14.0.2*/
const Device UA29954 = {S40056, 145, { &P17346,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A400 Safari/6531.22.7*/
const Device UA106201 = {S40057, 143, { &P17346,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293 Safari/6531.22.7*/
const Device UA2658516 = {S40058, 141, { &P17346,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 40 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML like Gecko) Version/4.0.5 Mobile/8A293 Safari/6531.22.7*/
const Device UA2659486 = {S40059, 142, { &P17346,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 402 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML like Gecko) Version/4.0.5 Mobile/8A400 Safari/6531.22.7*/
const Device UA3062795 = {S40060, 119, { &P17595,&P21721,&P24142,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.65 Safari/537.36*/
const Device UA3064758 = {S40061, 137, { &P15575,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3064876 = {S40062, 138, { &P15575,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065737 = {S40063, 138, { &P15575,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067645 = {S40064, 138, { &P15575,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3112024 = {S40065, 137, { &P15575,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9100 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2489374 = {S40066, 146, { &P17346,&P19826,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B144 Safari/8536.25*/
const Device UA2489524 = {S40067, 146, { &P17346,&P19826,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B141 Safari/8536.25*/
const Device UA2532640 = {S40068, 148, { &P17346,&P19826,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B146 Safari/8536.25*/
const Device UA2585608 = {S40069, 148, { &P17346,&P19826,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B329 Safari/8536.25*/
const Device UA925371 = {S40070, 138, { &P15575,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9100 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2032492 = {S40071, 139, { &P15575,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9100G Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2232490 = {S40072, 139, { &P15575,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2237761 = {S40073, 138, { &P15575,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2384744 = {S40074, 132, { &P15575,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2466118 = {S40075, 132, { &P15575,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9100 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2506123 = {S40076, 139, { &P15575,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100M Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2508515 = {S40077, 139, { &P15575,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2595086 = {S40078, 139, { &P15575,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2;  GT-I9100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA2558416 = {S40079, 146, { &P21935,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E980/E98008e Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2702351 = {S40080, 138, { &P21935,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E980 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2709361 = {S40081, 146, { &P21935,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E980/E98010g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2766954 = {S40082, 60, { &P21935,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E980 Build/JZO54K)*/
const Device UA2779801 = {S40083, 146, { &P21935,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E980/E98008g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2940411 = {S40084, 146, { &P21935,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E980/E98010k Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997246 = {S40085, 146, { &P21935,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E980/E98010i Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3109141 = {S40086, 167, { &P21935,&P18111,&P18112,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=geefhd;devm=LG-E980;man=LGE;brw=Android;shwd=1080x1920;shwa=1080x1845;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA746957 = {S40087, 137, { &P20935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ZTE-Z990G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1958527 = {S40088, 61, { &P20935,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; ZTE-Z990G Build/GRJ22)*/
const Device UA2833413 = {S40089, 135, { &P20935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; -us; ZTE-Z990G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA854887 = {S40090, 74, { &P17595,&P18207,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:10.0) Gecko/X Firefox/10.0*/
const Device UA1971834 = {S40091, 138, { &P18509,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2041190 = {S40092, 78, { &P17595,&P17295,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:10.0.7) Gecko/X Firefox/10.0.7*/
const Device UA2797348 = {S40093, 112, { &P17346,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A4372q*/
const Device UA2867768 = {S40094, 96, { &P17346,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko)*/
const Device UA2875862 = {S40095, 237, { &P17346,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A4372q [FBAN/FBIOS;FBAV/6.2;FBBV/X;FBDV/iPod5,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/7.0;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBOP/1]*/
const Device UA2885402 = {S40096, 112, { &P17346,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A4400f*/
const Device UA2900021 = {S40097, 237, { &P17346,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A4400f [FBAN/FBIOS;FBAV/6.2;FBBV/X;FBDV/iPod5,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/7.0;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBOP/1]*/
const Device UA2947246 = {S40098, 112, { &P17346,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Mobile/11A4414e*/
const Device UA2948104 = {S40099, 237, { &P17346,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Mobile/11A4414e [FBAN/FBIOS;FBAV/6.3;FBBV/X;FBDV/iPod5,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/7.0;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBOP/1]*/
const Device UA3025932 = {S40100, 239, { &P17346,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4435d [FBAN/FBIOS;FBAV/6.3;FBBV/X;FBDV/iPod5,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/7.0;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBOP/1]*/
const Device UA3026320 = {S40101, 114, { &P17346,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4435d*/
const Device UA3033029 = {S40102, 239, { &P17346,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449a [FBAN/FBIOS;FBAV/6.3;FBBV/X;FBDV/iPod5,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/7.0;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBOP/1]*/
const Device UA3033379 = {S40103, 114, { &P17346,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449a*/
const Device UA3034137 = {S40104, 114, { &P17346,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d*/
const Device UA3034375 = {S40105, 239, { &P17346,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d [FBAN/FBIOS;FBAV/6.4;FBBV/X;FBDV/iPod5,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/7.0;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBOP/1]*/
const Device UA3083310 = {S40106, 112, { &P17346,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465*/
const Device UA3096870 = {S40107, 237, { &P17346,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 [FBAN/FBIOS;FBAV/6.5;FBBV/X;FBDV/iPod5,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/7.0;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBOP/5]*/
const Device UA3108088 = {S40108, 98, { &P17346,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko)*/
const Device UA3115087 = {S40109, 139, { &P17346,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 iPod5,1 BingWeb/4.1.1502.X*/
const Device UA3115354 = {S40110, 114, { &P17346,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501*/
const Device UA3115450 = {S40111, 243, { &P17346,&P22882,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 [FBAN/FBIOS;FBAV/6.5.1;FBBV/X;FBDV/iPod5,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/7.0.2;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBOP/5]*/
const Device UA22383 = {S40112, 136, { &P15448,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-T839 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59866 = {S40113, 135, { &P15448,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;SGH-T839 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59868 = {S40114, 135, { &P15448,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-T839 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA59871 = {S40115, 135, { &P15448,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; SGH-T839 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59873 = {S40116, 135, { &P15448,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.1; en-us; SGH-T839 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59880 = {S40117, 143, { &P15448,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-T839 Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA315100 = {S40118, 60, { &P15448,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SGH-T839 Build/FROYO)*/
const Device UA2031605 = {S40119, 137, { &P15448,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SGH-T839 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034408 = {S40120, 136, { &P15448,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SGH-T839 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA903954 = {S40121, 107, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2430.0.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1172.0 Safari/537.1*/
const Device UA910776 = {S40122, 109, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2465.30.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.18 Safari/537.1*/
const Device UA912899 = {S40123, 109, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2465.62.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.41 Safari/537.1*/
const Device UA924878 = {S40124, 111, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2465.82.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.50 Safari/537.1*/
const Device UA924881 = {S40125, 109, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2465.82.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.50 Safari/537.1*/
const Device UA925090 = {S40126, 109, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2465.97.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.55 Safari/537.1*/
const Device UA1997319 = {S40127, 110, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2465.105.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.68 Safari/537.1*/
const Device UA1997549 = {S40128, 112, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2465.116.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.78 Safari/537.1*/
const Device UA1998276 = {S40129, 112, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2465.105.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.68 Safari/537.1*/
const Device UA2007339 = {S40130, 112, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2465.127.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1*/
const Device UA2007353 = {S40131, 110, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2465.127.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1*/
const Device UA2010374 = {S40132, 110, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2465.121.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.79 Safari/537.1*/
const Device UA2010480 = {S40133, 112, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2465.121.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.79 Safari/537.1*/
const Device UA2010797 = {S40134, 110, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2465.116.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.78 Safari/537.1*/
const Device UA2041666 = {S40135, 110, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2465.163.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.91 Safari/537.1*/
const Device UA2042018 = {S40136, 112, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2465.142.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2042908 = {S40137, 112, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2465.148.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2043323 = {S40138, 110, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2465.142.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2201944 = {S40139, 112, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2465.163.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.91 Safari/537.1*/
const Device UA2232322 = {S40140, 112, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2465.211.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.92 Safari/537.1*/
const Device UA2232329 = {S40141, 110, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2465.211.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.92 Safari/537.1*/
const Device UA2233786 = {S40142, 112, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2465.173.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.91 Safari/537.1*/
const Device UA2236181 = {S40143, 110, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2465.227.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.92 Safari/537.1*/
const Device UA2236182 = {S40144, 112, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2465.228.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.92 Safari/537.1*/
const Device UA2236185 = {S40145, 112, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2465.227.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.92 Safari/537.1*/
const Device UA2386915 = {S40146, 112, { &P15364,&P6596,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2465.238.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.92 Safari/537.1*/
const Device UA2030778 = {S40147, 62, { &P17968,&P17979,&P18522,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:18.0) Gecko/18.0 Firefox/18.0*/
const Device UA2355057 = {S40148, 68, { &P17968,&P17979,&P18522,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:18.0a2) Gecko/18.0a2 Firefox/18.0a2*/
const Device UA2450213 = {S40149, 68, { &P17968,&P17979,&P18522,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:18.0a1) Gecko/18.0a1 Firefox/18.0a1*/
const Device UA2516372 = {S40150, 68, { &P17968,&P17979,&P18522,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:18.0.2) Gecko/18.0.2 Firefox/18.0.2*/
const Device UA2557182 = {S40151, 101, { &P17968,&P17979,&P18522,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:18.0) Gecko/18.0 Firefox/18.0,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2974040 = {S40152, 140, { &P22394,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6603 Build/10.3.A.0.423) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3026076 = {S40153, 127, { &P22394,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6603 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA24480 = {S40154, 137, { &P13974,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; GT-S5670L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41372 = {S40155, 97, { &P13974,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.2.1;Zh_cn;GT-S5670;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA60607 = {S40156, 137, { &P13974,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; GT-S5670B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA64524 = {S40157, 136, { &P13974,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5670 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90712 = {S40158, 143, { &P13974,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ar-ae; GT-S5670 Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA176035 = {S40159, 60, { &P13974,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-S5670 Build/FROYO)*/
const Device UA379901 = {S40160, 124, { &P13974,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5670 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA551148 = {S40161, 61, { &P13974,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-S5670L Build/FROYO)*/
const Device UA2306462 = {S40162, 134, { &P13974,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-; GT-S5670 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2544472 = {S40163, 129, { &P13974,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-S5670 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA763325 = {S40164, 124, { &P15364,&P5690,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19 AvantBrowser/Tri-Core*/
const Device UA906684 = {S40165, 121, { &P15364,&P5690,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5 AvantBrowser/Tri-Core*/
const Device UA925354 = {S40166, 123, { &P15364,&P5690,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11 AvantBrowser/Tri-Core*/
const Device UA2008929 = {S40167, 121, { &P15364,&P5690,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 AvantBrowser/Tri-Core*/
const Device UA2198999 = {S40168, 114, { &P15364,&P5690,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; Avant TriCore) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2359480 = {S40169, 114, { &P15364,&P5690,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; Avant TriCore) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.96 Safari/537.4*/
const Device UA2431877 = {S40170, 116, { &P15364,&P5690,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; Avant TriCore) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2531328 = {S40171, 116, { &P15364,&P5690,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; Avant TriCore) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2605179 = {S40172, 117, { &P15364,&P5690,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; Avant TriCore) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2717852 = {S40173, 116, { &P15364,&P5690,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; Avant TriCore) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2877646 = {S40174, 117, { &P15364,&P5690,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36*/
const Device UA2946106 = {S40175, 116, { &P15364,&P5690,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36*/
const Device UA3111767 = {S40176, 116, { &P15364,&P5690,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36*/
const Device UA632575 = {S40177, 83, { &P15364,&P16110,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.2.28) Gecko/X Firefox/3.6.28*/
const Device UA894280 = {S40178, 66, { &P17595,&P17296,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6) Gecko/X Firefox/5.0.1*/
const Device UA1890991 = {S40179, 92, { &P17595,&P17296,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:5.0.1) Gecko/X Firefox/5.0.1 SmartLinksAddon*/
const Device UA90401 = {S40180, 131, { &P13251,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; 001DL Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA143239 = {S40181, 137, { &P13251,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Dell Streak Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA161431 = {S40182, 61, { &P13251,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; Dell Streak Build/FRF91)*/
const Device UA165887 = {S40183, 135, { &P13251,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-; Dell Streak Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA181495 = {S40184, 139, { &P13251,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Dell Streak Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA182608 = {S40185, 140, { &P13251,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Dell Streak Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA189864 = {S40186, 138, { &P13251,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-; Dell Streak Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA236014 = {S40187, 134, { &P13251,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; 001DL Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA308013 = {S40188, 64, { &P13251,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; Dell Streak Build/FRG83G)*/
const Device UA308876 = {S40189, 133, { &P13251,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; 001DL Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019147 = {S40190, 63, { &P13251,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; Dell Streak Build/FRG83)*/
const Device UA2019293 = {S40191, 58, { &P13251,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; 001DL Build/FRG83G)*/
const Device UA2021916 = {S40192, 128, { &P13251,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Dell Streak Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022584 = {S40193, 125, { &P13251,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; Dell Streak Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2574460 = {S40194, 133, { &P13251,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Dell Streak Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA16430 = {S40195, 134, { &P14334,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA22314 = {S40196, 136, { &P14334,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46504 = {S40197, 134, { &P14334,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA54637 = {S40198, 136, { &P14334,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA205495 = {S40199, 60, { &P14334,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; SPH-M910 Build/FROYO)*/
const Device UA370174 = {S40200, 71, { &P14334,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.2.2; en_US; SPH-M910 Build/FROYO.EC07)*/
const Device UA374532 = {S40201, 60, { &P14334,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SPH-M910 Build/FROYO)*/
const Device UA625925 = {S40202, 136, { &P14334,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633252 = {S40203, 60, { &P14334,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.3; SPH-M910 Build/FROYO)*/
const Device UA2032265 = {S40204, 134, { &P14334,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; en-; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2427458 = {S40205, 139, { &P18448,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST26i Build/11.0.A.7.5) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2506505 = {S40206, 139, { &P18448,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST26a Build/11.0.A.7.5) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA902928 = {S40207, 155, { &P18016,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ThinkPad Tablet Build/ThinkPadTablet_A400_03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1992551 = {S40208, 84, { &P18016,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; ThinkPad Tablet Build/ThinkPadTablet_A400_03)*/
const Device UA2708115 = {S40209, 152, { &P18016,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; ThinkPad Tablet Build/ThinkPadTabletA40003) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Safari/534.30*/
const Device UA2726781 = {S40210, 149, { &P18016,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ThinkPad Tablet Build/waynetss_A400_03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2261507 = {S40211, 137, { &P18713,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P769 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261534 = {S40212, 137, { &P18713,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P760 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2379958 = {S40213, 137, { &P18713,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P768 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA250073 = {S40214, 117, { &P17595,&P17295,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA250091 = {S40215, 117, { &P17595,&P17295,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA250172 = {S40216, 117, { &P17595,&P17295,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA250275 = {S40217, 117, { &P17595,&P17295,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA250295 = {S40218, 117, { &P17595,&P17295,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA250441 = {S40219, 117, { &P17595,&P17295,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA343727 = {S40220, 117, { &P17595,&P17295,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA2258220 = {S40221, 116, { &P17595,&P17295,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_5) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.35 Safari/535.1*/
const Device UA2372038 = {S40222, 115, { &P17595,&P17295,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.1 (KHTML	 like Gecko) Chrome/14.0.803.0 Safari/535.1*/
const Device UA2674472 = {S40223, 129, { &P17595,&P17295,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/14.0.648.205 Safari/534.16*/
const Device UA2751811 = {S40224, 129, { &P17595,&P17295,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/14.0.648.205 Safari/534.16*/
const Device UA629727 = {S40225, 78, { &P15364,&P17285,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Avant Browser)*/
const Device UA658799 = {S40226, 85, { &P15364,&P17285,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Avant Browser)*/
const Device UA873271 = {S40227, 90, { &P15364,&P17285,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Win64; x64; Trident/5.0; Avant Browser)*/
const Device UA2163880 = {S40228, 78, { &P15364,&P17285,&P22732,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; Avant Browser)*/
const Device UA2727140 = {S40229, 93, { &P15364,&P17285,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; Avant Browser; Avant Browser)*/
const Device UA2743591 = {S40230, 83, { &P15364,&P17285,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; Avant Browser; SV1)*/
const Device UA2831172 = {S40231, 83, { &P15364,&P17285,&P22732,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; SV1; Avant Browser)*/
const Device UA2950016 = {S40232, 93, { &P15364,&P17285,&P22732,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; Avant Browser; Avant Browser)*/
const Device UA52064 = {S40233, 144, { &P14799,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I5500 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA53769 = {S40234, 145, { &P14799,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; GT-I5500B Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA67311 = {S40235, 145, { &P14799,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; GT-I5500L Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA67332 = {S40236, 146, { &P14799,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; GT-I5500M Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA67809 = {S40237, 142, { &P14799,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; GT-I5500 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA164840 = {S40238, 66, { &P14799,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I5500 Build/ERE27)*/
const Device UA566387 = {S40239, 67, { &P14799,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I5500B Build/ERE27)*/
const Device UA648674 = {S40240, 67, { &P14799,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I5500L Build/ERE27)*/
const Device UA749854 = {S40241, 68, { &P14799,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I5500M Build/ECLAIR)*/
const Device UA625279 = {S40242, 142, { &P18500,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-R920 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA641812 = {S40243, 66, { &P18500,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-R920 Build/GINGERBREAD)*/
const Device UA2237744 = {S40244, 135, { &P15364,&P19822,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/12.10 Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2238894 = {S40245, 137, { &P15364,&P19822,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/12.10 Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2402626 = {S40246, 137, { &P15364,&P19822,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/12.10 Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2426400 = {S40247, 145, { &P15364,&P19822,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/12.10 Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/537.4,gzip(gfe)*/
const Device UA442961 = {S40248, 66, { &P15364,&P7133,&P19905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; en) Opera 10.62*/
const Device UA2568301 = {S40249, 141, { &P17346,&P19826,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B146 Safari/8536.25*/
const Device UA2568366 = {S40250, 139, { &P17346,&P19826,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B144 Safari/8536.25*/
const Device UA2568367 = {S40251, 139, { &P17346,&P19826,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B141 Safari/8536.25*/
const Device UA2585385 = {S40252, 141, { &P17346,&P19826,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10B329 Safari/8536.25*/
const Device UA2741720 = {S40253, 136, { &P20705,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2752943 = {S40254, 141, { &P20705,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One 801e Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753227 = {S40255, 138, { &P20705,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC_PN071 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755120 = {S40256, 135, { &P20705,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTCONE Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA221057 = {S40257, 117, { &P17595,&P17295,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1*/
const Device UA222605 = {S40258, 117, { &P17595,&P17295,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1*/
const Device UA338236 = {S40259, 117, { &P17595,&P17295,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1*/
const Device UA653338 = {S40260, 117, { &P17595,&P17295,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1*/
const Device UA717279 = {S40261, 117, { &P17595,&P17295,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.31 (KHTML, like Gecko) Chrome/13.0.748.0 Safari/534.31*/
const Device UA778501 = {S40262, 117, { &P17595,&P17295,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1*/
const Device UA1897047 = {S40263, 117, { &P17595,&P17295,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.215 Safari/535.1*/
const Device UA2332663 = {S40264, 115, { &P17595,&P17295,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_5) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.772.0 Safari/535.1*/
const Device UA2674867 = {S40265, 129, { &P17595,&P17295,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/13.0.648.205 Safari/534.16*/
const Device UA2735836 = {S40266, 129, { &P17595,&P17295,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/13.0.648.205 Safari/534.16*/
const Device UA2568308 = {S40267, 135, { &P15767,&P18117,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9B206 Safari/7534.48.3*/
const Device UA2568524 = {S40268, 133, { &P15767,&P18117,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9B176 Safari/7534.48.3*/
const Device UA422242 = {S40269, 152, { &P14864,&P17342,&P17454,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8500/S8500XXLA1; U; Bada/2.0; en-us) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile WVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA325564 = {S40270, 138, { &P12532,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Flyer_P510e; ru-ru) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA430005 = {S40271, 137, { &P12532,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Flyer_P512; nl-nl) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA430108 = {S40272, 140, { &P12532,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Flyer_P512_NA; en-ca) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2020583 = {S40273, 222, { &P12532,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTCP510e/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1; zh-cn) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA898772 = {S40274, 145, { &P18508,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HUAWEI-M920 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1952280 = {S40275, 69, { &P18508,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; HUAWEI-M920 Build/GINGERBREAD)*/
const Device UA2759708 = {S40276, 143, { &P18508,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; -us; HUAWEI-M920 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2235639 = {S40277, 62, { &P15364,&P5690,&P20761,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:16.0) Gecko/X SeaMonkey/2.13.1*/
const Device UA2258860 = {S40278, 75, { &P15364,&P5690,&P20761,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:16.0) Gecko/X Firefox/16.0 SeaMonkey/2.13.2*/
const Device UA2356349 = {S40279, 82, { &P15364,&P5690,&P20761,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:16.0) Gecko/X Firefox/16.0 SeaMonkey/2.13.2*/
const Device UA2880203 = {S40280, 132, { &P18465,&P18110,&P23483,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Safari/535.19 Silk-Accelerated=true*/
const Device UA488822 = {S40281, 108, { &P15364,&P17813,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.0.3) Gecko/X Ubuntu/8.04 (hardy) Firefox/3.0.3 (Linux Mint)*/
const Device UA796098 = {S40282, 94, { &P15364,&P17813,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.2) Gecko/X Ubuntu/1.4.0 (hardy) Firefox/3.0.2*/
const Device UA881555 = {S40283, 89, { &P15364,&P17813,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; fr; rv:1.9.0.3) Gecko/X Ubuntu/8.04 (hardy) Firefox/3.03*/
const Device UA2263935 = {S40284, 92, { &P15364,&P17813,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; de; rv:1.9.0.13) Gecko/X Ubuntu/8.04 (hardy) Firefox/3.0.13*/
const Device UA2758084 = {S40285, 106, { &P15364,&P17813,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.1) Gecko/X Ubuntu/8.04 (hardy) Firefox/3.0.3 (Linux Mint)*/
const Device UA616049 = {S40286, 64, { &P15364,&P17285,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.10.229 Version/11.64*/
const Device UA630376 = {S40287, 87, { &P15364,&P17285,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.64*/
const Device UA637430 = {S40288, 87, { &P15364,&P17285,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5310); ru) Presto/2.10.229 Version/11.64*/
const Device UA681344 = {S40289, 87, { &P15364,&P17285,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.4 (build 02647); ru) Presto/2.10.229 Version/11.64*/
const Device UA723717 = {S40290, 116, { &P15364,&P17285,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0); en) Presto/2.10.229 Version/11.64*/
const Device UA775105 = {S40291, 86, { &P15364,&P17285,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.64*/
const Device UA802291 = {S40292, 116, { &P15364,&P17285,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0); en) Presto/2.10.229 Version/11.64*/
const Device UA806640 = {S40293, 87, { &P15364,&P17285,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.64*/
const Device UA862615 = {S40294, 87, { &P15364,&P17285,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.64*/
const Device UA891844 = {S40295, 87, { &P15364,&P17285,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5339); ru) Presto/2.10.229 Version/11.64*/
const Device UA1985626 = {S40296, 66, { &P15364,&P17285,&P18347,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; en) Opera 11.64*/
const Device UA2158834 = {S40297, 87, { &P15364,&P17285,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.64*/
const Device UA2320281 = {S40298, 86, { &P15364,&P17285,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 6.0 (build 5680); ru) Presto/2.10.229 Version/11.64*/
const Device UA2393499 = {S40299, 87, { &P15364,&P17285,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5265); ru) Presto/2.10.229 Version/11.64*/
const Device UA2798175 = {S40300, 89, { &P18115,&P5764,&P17369,&P18092 }};	/*Useragent - Opera/9.80 (BlackBerry; Opera Mini/4.4.31892/30.3214; U; en) Presto/2.8.119 Version/11.10*/
const Device UA125931 = {S40301, 69, { &P15364,&P18110,&P21955,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9) Gecko Minefield/3.0*/
const Device UA636800 = {S40302, 75, { &P15364,&P18110,&P21955,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.6) Gecko/X Minefield/3.0.6*/
const Device UA678184 = {S40303, 65, { &P15364,&P18110,&P21955,&P18092 }};	/*Useragent - Mozilla/5.0(X11;U;Linuxi686;en-US;rv:1.9a1)Gecko/XMinefield/3.0a1*/
const Device UA904079 = {S40304, 76, { &P17595,&P18207,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:13.0) Gecko/X Firefox/13.0.1*/
const Device UA2257596 = {S40305, 142, { &P18303,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MT27i Build/6.1.1.B.1.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423630 = {S40306, 64, { &P18303,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MT27i Build/6.1.1.B.1.54)*/
const Device UA2425527 = {S40307, 154, { &P18303,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; SonyEricssonMT27i Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467602 = {S40308, 142, { &P18303,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MT27i Build/6.1.1.B.1.54) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3065247 = {S40309, 136, { &P18713,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065257 = {S40310, 136, { &P18713,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P768 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065367 = {S40311, 136, { &P18713,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065723 = {S40312, 136, { &P18713,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LGMS769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA246517 = {S40313, 134, { &P17713,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LGL55C Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370702 = {S40314, 141, { &P17713,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; LGL55C Build/GRJ22) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA633996 = {S40315, 58, { &P17713,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LGL55C Build/GRJ22)*/
const Device UA1982033 = {S40316, 142, { &P17713,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LGL55C Build/AreaRomQ v2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1985348 = {S40317, 142, { &P17713,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LGL55C Build/AreaRomQ v1.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026169 = {S40318, 140, { &P17713,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LGL55c Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028420 = {S40319, 142, { &P17713,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LGL55C Build/AreaRomQ v1.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033261 = {S40320, 142, { &P17713,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LGL55C Build/AreaRomQ v2.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3065142 = {S40321, 139, { &P16292,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Galaxy Nexus Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3064984 = {S40322, 136, { &P22597,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080310 = {S40323, 137, { &P22597,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545L Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA23964 = {S40324, 129, { &P15223,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Zio Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA177912 = {S40325, 131, { &P15223,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Zio Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA195227 = {S40326, 55, { &P15223,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; Zio Build/FRG83)*/
const Device UA376768 = {S40327, 53, { &P15223,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; Zio Build/FRF91)*/
const Device UA2654152 = {S40328, 129, { &P15223,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Zio Build/ERD79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA198114 = {S40329, 178, { &P15364,&P5690,&P21897,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/12.0.742.91; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA200583 = {S40330, 125, { &P15364,&P5690,&P21897,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/12.0.742.91) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.91 Safari/534.30*/
const Device UA205313 = {S40331, 127, { &P15364,&P5690,&P21897,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/12.0.742.100) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30*/
const Device UA218583 = {S40332, 172, { &P15364,&P5690,&P21897,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/12.0.742.122; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA218643 = {S40333, 127, { &P15364,&P5690,&P21897,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/12.0.742.122) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA327763 = {S40334, 76, { &P15364,&P5690,&P21897,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/12.0.742.122)*/
const Device UA329217 = {S40335, 127, { &P15364,&P5690,&P21897,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/12.0.742.112) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30*/
const Device UA637160 = {S40336, 174, { &P15364,&P5690,&P21897,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/12.0.742.122; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C)*/
const Device UA2290964 = {S40337, 127, { &P15364,&P5690,&P21897,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; chromeframe/12.0.742.122) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA2291564 = {S40338, 140, { &P15364,&P5690,&P21897,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/12.0.742.122; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)*/
const Device UA2372241 = {S40339, 125, { &P15364,&P5690,&P21897,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/12.0.742.30) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.30 Safari/534.30*/
const Device UA2677127 = {S40340, 130, { &P21769,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ME301T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2714279 = {S40341, 59, { &P21769,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; ME301T Build/JOP40D)*/
const Device UA2567714 = {S40342, 90, { &P15364,&P7133,&P22369,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:19.0) Gecko/X Firefox/19.0 PaleMoon/19.0.2-x64*/
const Device UA2567943 = {S40343, 81, { &P15364,&P7133,&P22369,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:19.0) Gecko/X Firefox/19.0 PaleMoon/19.0.2*/
const Device UA33002 = {S40344, 139, { &P12824,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; U8180 Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA33004 = {S40345, 143, { &P12824,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; Cataclysm Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48773 = {S40346, 143, { &P12824,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; Starshine Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70021 = {S40347, 144, { &P12824,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; DroniX-0.5 Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70767 = {S40348, 146, { &P12824,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-ph; Netphone 501 Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70888 = {S40349, 142, { &P12824,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; cs-cz; IDEOS X1 Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79198 = {S40350, 143, { &P12824,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; nl-be; Stockholm Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368765 = {S40351, 127, { &P12824,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; U8180 Build/HuaweiU8180) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA420455 = {S40352, 63, { &P12824,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; U8180 Build/HuaweiU8180)*/
const Device UA645263 = {S40353, 149, { &P12824,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; Huawei Ideos X1 Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2574024 = {S40354, 147, { &P12824,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ru-ru; MegaFon U8180 Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA785520 = {S40355, 155, { &P18233,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ALCATEL_one_touch_995 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA924295 = {S40356, 156, { &P18233,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ALCATEL_one_touch_995S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032086 = {S40357, 156, { &P18233,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ALCATEL_one_touch_995A Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2940548 = {S40358, 127, { &P12280,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A4372q Twitter for iPhone*/
const Device UA2947248 = {S40359, 127, { &P12280,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A4400f Twitter for iPhone*/
const Device UA2947251 = {S40360, 127, { &P12280,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Mobile/11A4414e Twitter for iPhone*/
const Device UA3026317 = {S40361, 129, { &P12280,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4435d Twitter for iPhone*/
const Device UA3033019 = {S40362, 129, { &P12280,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449a Twitter for iPhone*/
const Device UA3034361 = {S40363, 129, { &P12280,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d Twitter for iPhone*/
const Device UA3079813 = {S40364, 127, { &P12280,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Twitter for iPhone*/
const Device UA3111574 = {S40365, 127, { &P12280,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A466 Twitter for iPhone*/
const Device UA3112851 = {S40366, 130, { &P12280,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A470a Twitter for iPhone*/
const Device UA3115352 = {S40367, 129, { &P12280,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Twitter for iPhone*/
const Device UA715667 = {S40368, 139, { &P20011,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Next7P12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2386104 = {S40369, 132, { &P20011,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Next7P12 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2430926 = {S40370, 135, { &P20011,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Next7P12-8G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2656277 = {S40371, 61, { &P20011,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Next7P12 Build/IMM76I)*/
const Device UA2656909 = {S40372, 64, { &P20011,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Next7P12-8G Build/IMM76I)*/
const Device UA652016 = {S40373, 142, { &P18084,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Transformer TF101G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA893861 = {S40374, 71, { &P18084,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Transformer TF101G Build/IML74K)*/
const Device UA456203 = {S40375, 61, { &P15364,&P18110,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:12.0) Gecko/X Firefox/12.0*/
const Device UA630268 = {S40376, 74, { &P15364,&P18110,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:12.0) Gecko/X Firefox/12.0 Iceweasel/12.0*/
const Device UA1983045 = {S40377, 72, { &P15364,&P18110,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:12.0) Gecko/X Firefox/12.0 Firefox/12.0*/
const Device UA809591 = {S40378, 140, { &P20086,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-LS696 Build/ZV9.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257183 = {S40379, 140, { &P20086,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-LS696 Build/ZVF.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2403155 = {S40380, 64, { &P20086,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; LG-LS696 Build/ZVF.GWK74)*/
const Device UA2939580 = {S40381, 140, { &P20086,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-LS696 Build/ZVG.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2163472 = {S40382, 142, { &P18672,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2225902 = {S40383, 143, { &P18672,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-S5302B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2236482 = {S40384, 140, { &P18672,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2415361 = {S40385, 66, { &P18672,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5302 Build/GINGERBREAD)*/
const Device UA2416891 = {S40386, 130, { &P18672,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5302 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2790038 = {S40387, 67, { &P18672,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5302B Build/GINGERBREAD)*/
const Device UA88549 = {S40388, 143, { &P17346,&P17131,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 2_2 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5G77a Safari/525.20*/
const Device UA88610 = {S40389, 145, { &P17346,&P17131,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 2_2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5H11a Safari/525.20*/
const Device UA125879 = {S40390, 144, { &P17346,&P17131,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 2_2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5H11 Safari/525.20*/
const Device UA165226 = {S40391, 142, { &P17346,&P17131,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 2_2 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5G77 Safari/525.20*/
const Device UA2658249 = {S40392, 141, { &P17346,&P17131,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 221 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML like Gecko) Version/3.1.1 Mobile/5H11 Safari/525.20*/
const Device UA2659375 = {S40393, 142, { &P17346,&P17131,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 221 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML like Gecko) Version/3.1.1 Mobile/5H11a Safari/525.20*/
const Device UA2225654 = {S40394, 5, { &P15364,&P18588,&P17200,&P18092 }};	/*Useragent - MSIE9*/
const Device UA33226 = {S40395, 154, { &P23916,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonR800at Build/3.0.1.B.0.131) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43544 = {S40396, 142, { &P23916,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; R800at Build/3.0.1.B.0.270) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43538 = {S40397, 154, { &P23916,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonR800at Build/3.0.1.B.0.270) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59526 = {S40398, 154, { &P23916,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonR800at Build/3.0.1.B.0.148) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74703 = {S40399, 154, { &P23916,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonR800at Build/3.0.1.B.0.273) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA609187 = {S40400, 142, { &P23916,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; R800at Build/3.0.1.B.0.285) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626178 = {S40401, 154, { &P23916,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SonyEricssonR800at Build/3.0.1.B.0.285) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1966039 = {S40402, 66, { &P23916,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; R800at Build/3.0.1.B.0.285)*/
const Device UA211548 = {S40403, 87, { &P15364,&P7127,&P17200,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.0;.NET CLR 1.0.3705; ContextAd Bot 1.0)*/
const Device UA574449 = {S40404, 130, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6210 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA641466 = {S40405, 59, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P6210 Build/HTJ85B)*/
const Device UA2022212 = {S40406, 137, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6210 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2608125 = {S40407, 128, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6210 Build/HTJ85B) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2608760 = {S40408, 60, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - BBC News (AndroidApp; 1.3; 4) GT-P6210 (Android 3.2, SDK 13)*/
const Device UA2609244 = {S40409, 128, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P6210 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2614208 = {S40410, 71, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 3.2; en_US; GT-P6210 Build/HTJ85B.UELA4)*/
const Device UA2616316 = {S40411, 65, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 3.2; en-US; GT-P6210 Build/HTJ85B)*/
const Device UA2616826 = {S40412, 71, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 3.2; pt-BR; GT-P6210 Build/HTJ85B.UBLA1)*/
const Device UA2617261 = {S40413, 71, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - Vyrso/2.1.2 (Linux; U; Android 3.2; en-US; GT-P6210 Build/HTJ85B.UEKL1)*/
const Device UA2618086 = {S40414, 72, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/2.0.11 (Linux; U; Android 3.2; es-US; GT-P6210 Build/HTJ85B.UBKL2)*/
const Device UA2619324 = {S40415, 151, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6210 Build/HTJ85B; 1024*552) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2619734 = {S40416, 71, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 3.2; en-US; GT-P6210 Build/HTJ85B.UELA3)*/
const Device UA2620596 = {S40417, 149, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6210 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MantanoReader*/
const Device UA2620863 = {S40418, 92, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0(YAdSDK; 1.9.2.1); (Linux; U; Android 3.2; SAMSUNG GT-P6210 Build/HONEYCOMB_MR2);*/
const Device UA2620927 = {S40419, 92, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - com.google.android.apps.plus/X (Linux; U; Android 3.2; en_US; GT-P6210; Build/HTJ85B) (gzip)*/
const Device UA2621984 = {S40420, 125, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - Groupon/2.1.2486 (Android 3.2; Samsung GT-P6210 / Samsung GT-P6210; )[preload=true;locale=en_US;clientidbase=android-samsung]*/
const Device UA2622312 = {S40421, 198, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6210 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 ZyngaBundleIdentifier/com.zynga.zjmontopia ZyngaBundleVersion/1.8.1*/
const Device UA2623800 = {S40422, 67, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - BBC iPlayer (AndroidApp; 1.3; 13014) GT-P6210 (Android 3.2, SDK 13)*/
const Device UA2625905 = {S40423, 65, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6210) BBC News/1.3*/
const Device UA2628359 = {S40424, 268, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6210 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.12;FBDM/{density=1.0,width=600,height=976};FBLC/en_US;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P6210;FBSV/3.2;]*/
const Device UA2628717 = {S40425, 61, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - BBC News (AndroidApp; 2.1; 30) GT-P6210 (Android 3.2, SDK 13)*/
const Device UA2636687 = {S40426, 198, { &P14908,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6210 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 ZyngaBundleIdentifier/com.zynga.zjayakashi ZyngaBundleVersion/2.0.0*/
const Device UA3035295 = {S40427, 136, { &P20705,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3036275 = {S40428, 138, { &P20705,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC_PN071 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3064765 = {S40429, 135, { &P20705,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTCONE Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3079099 = {S40430, 145, { &P20705,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One dual sim Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3080655 = {S40431, 141, { &P20705,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One 801e Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA575081 = {S40432, 143, { &P18072,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHV-E120S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA643543 = {S40433, 143, { &P18072,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHV-E120L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA654621 = {S40434, 67, { &P18072,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHV-E120L Build/GINGERBREAD)*/
const Device UA663891 = {S40435, 67, { &P18072,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHV-E120S Build/GINGERBREAD)*/
const Device UA669737 = {S40436, 143, { &P18072,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHV-E120K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025041 = {S40437, 131, { &P18072,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SHV-E120L Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2030180 = {S40438, 67, { &P18072,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHV-E120K Build/GINGERBREAD)*/
const Device UA2034503 = {S40439, 136, { &P18072,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SHV-E120L Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2809512 = {S40440, 136, { &P18072,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SHV-E120S Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1955896 = {S40441, 75, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - Read%20Later/133 CFNetwork/520.4.3 Darwin/11.4.0 (x86_64) (MacBookAir4%2C2)*/
const Device UA1959323 = {S40442, 67, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - NewsFire/84 CFNetwork/520.4.3 Darwin/11.4.0 (x86_64) (MacBook5%2C1)*/
const Device UA2242286 = {S40443, 89, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)*/
const Device UA2338188 = {S40444, 99, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - Google%20Quick%20Search%20Box/2.0.0.1447 CFNetwork/520.4.3 Darwin/11.4.0 (x86_64) (MacBookPro8%2C2)*/
const Device UA2341334 = {S40445, 58, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - Miro/5.0.3 (http://www.getmiro.com/; Darwin 11.4.0 x86_64)*/
const Device UA2350694 = {S40446, 89, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookAir4%2C1)*/
const Device UA2351879 = {S40447, 84, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (iMac12%2C1)*/
const Device UA2383286 = {S40448, 87, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader/1725 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)*/
const Device UA2386599 = {S40449, 80, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - WebProcess/7536.26.17 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)*/
const Device UA2425002 = {S40450, 75, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - WebProcess/7536.26.17 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (iMac12%2C1)*/
const Device UA2427209 = {S40451, 80, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - WebProcess/7536.26.17 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookAir4%2C2)*/
const Device UA2455024 = {S40452, 85, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - ThumbnailService/39600.038 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)*/
const Device UA2525754 = {S40453, 85, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - ThumbnailService/39600.038 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookAir4%2C2)*/
const Device UA2552522 = {S40454, 82, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - ocspd (unknown version) CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)*/
const Device UA2593535 = {S40455, 92, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - XProtectUpdater (unknown version) CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)*/
const Device UA2593598 = {S40456, 82, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - ocspd (unknown version) CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookAir4%2C2)*/
const Device UA2727431 = {S40457, 86, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBook5%2C1)*/
const Device UA2761427 = {S40458, 92, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - XProtectUpdater (unknown version) CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookAir4%2C2)*/
const Device UA2762076 = {S40459, 82, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader/1725 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (iMac12%2C1)*/
const Device UA2836250 = {S40460, 87, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader/1725 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookAir4%2C2)*/
const Device UA2839587 = {S40461, 80, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - ThumbnailService/39600.038 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (iMac12%2C1)*/
const Device UA2843308 = {S40462, 77, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - WebProcess/7536.26.17 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBook5%2C1)*/
const Device UA2848307 = {S40463, 77, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - Spector%20Pro/4821 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)*/
const Device UA2951938 = {S40464, 72, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - Spector%20Pro/4821 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (iMac12%2C1)*/
const Device UA2981622 = {S40465, 94, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - Software%20Update (unknown version) CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)*/
const Device UA2981712 = {S40466, 86, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (Macmini5%2C1)*/
const Device UA3042681 = {S40467, 79, { &P17595,&P18578,&P17470,&P18092 }};	/*Useragent - ThumbnailService/39600.038 CFNetwork/520.5.1 Darwin/11.4.2 (i386) (MacPro1%2C1)*/
const Device UA2403383 = {S40468, 132, { &P18466,&P18110,&P19804,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Safari/535.19 Silk-Accelerated=true*/
const Device UA2202609 = {S40469, 108, { &P18634,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; id102)*/
const Device UA2232263 = {S40470, 112, { &P18634,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 822)*/
const Device UA683828 = {S40471, 114, { &P18267,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-02/11.57; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6*/
const Device UA902418 = {S40472, 115, { &P18267,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-02/10.91; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA2462468 = {S40473, 145, { &P18719,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; SonyC1505 Build/11.3.A.0.26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2462469 = {S40474, 141, { &P18719,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; C1505 Build/11.3.A.0.26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2462470 = {S40475, 145, { &P18719,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SonyC1505 Build/11.3.A.0.12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2601133 = {S40476, 145, { &P18719,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; SonyC1505 Build/11.3.A.0.47) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2672555 = {S40477, 141, { &P18719,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; C1505 Build/11.3.A.0.47) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734000 = {S40478, 63, { &P18719,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; C1505 Build/11.3.A.0.47)*/
const Device UA2734961 = {S40479, 147, { &P18719,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; SonyC1505-o Build/11.3.A.0.47) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752405 = {S40480, 145, { &P18719,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; SonyC1505 Build/11.3.A.0.56) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2939890 = {S40481, 145, { &P18719,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; SonyC1505 Build/11.3.A.0.59) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994509 = {S40482, 146, { &P18719,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-be; SonyC1505v Build/11.3.A.0.47) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994979 = {S40483, 146, { &P18719,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-be; SonyC1505v Build/11.3.A.0.56) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3013033 = {S40484, 145, { &P18719,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; SonyC1505 Build/11.3.A.2.13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3015283 = {S40485, 141, { &P18719,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; C1505 Build/11.3.A.0.59) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3021332 = {S40486, 141, { &P18719,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; C1505 Build/11.3.A.2.13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3024512 = {S40487, 147, { &P18719,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; SonyC1505-o Build/11.3.A.2.13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3047296 = {S40488, 141, { &P18719,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; C1505 Build/11.3.A.0.56) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3083447 = {S40489, 145, { &P18719,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; SonyC1505 Build/11.3.A.2.23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3093648 = {S40490, 141, { &P18719,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; C1505 Build/11.3.A.2.23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2434812 = {S40491, 151, { &P12280,&P17146,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9A405 Safari/7534.48.3*/
const Device UA2436260 = {S40492, 151, { &P12280,&P17146,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9A406 Safari/7534.48.3*/
const Device UA2436471 = {S40493, 149, { &P12280,&P17146,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9A334 Safari/7534.48.3*/
const Device UA26477 = {S40494, 117, { &P15518,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-S5230G/S523GBKJG1 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA56654 = {S40495, 116, { &P15518,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-S5230/S5230XPKB1 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA603259 = {S40496, 139, { &P14969,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Desire S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA662932 = {S40497, 148, { &P14969,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_DesireS_S510e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA854989 = {S40498, 143, { &P14969,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Desire S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026071 = {S40499, 134, { &P14969,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Desire S Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2026277 = {S40500, 230, { &P14969,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTCS510e/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029022 = {S40501, 144, { &P14969,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; DesireS_S510e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030800 = {S40502, 65, { &P14969,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Desire S Build/IMM76D)*/
const Device UA2031001 = {S40503, 148, { &P14969,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_DesireS_S510e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032335 = {S40504, 134, { &P14969,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Desire S Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2032698 = {S40505, 129, { &P14969,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Desire S Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2033114 = {S40506, 143, { &P14969,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Desire S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2403482 = {S40507, 143, { &P14969,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; HTC_DesireSV Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2424511 = {S40508, 143, { &P14969,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Desire_S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2457315 = {S40509, 144, { &P14969,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; HTC Desire SV Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2653088 = {S40510, 143, { &P14969,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Desire S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2696048 = {S40511, 66, { &P14969,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Desire SV Build/IMM76D)*/
const Device UA2699779 = {S40512, 139, { &P14969,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Desire_S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2819440 = {S40513, 65, { &P14969,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Desire S Build/IML74K)*/
const Device UA2834511 = {S40514, 144, { &P14969,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; HTC Desire S Build/miui.es) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2992651 = {S40515, 141, { &P14969,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-; HTC_DesireSV Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3086613 = {S40516, 148, { &P14969,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; HTC_DesireS_S510e Build/qsense) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA111528 = {S40517, 36, { &P15364,&P17285,&P17470,&P18092 }};	/*Useragent - Mozilla/4.8 [en] (Windows NT 6.0; U)*/
const Device UA131567 = {S40518, 119, { &P15364,&P17285,&P17470,&P18092 }};	/*Useragent - Sean's Agent/1.0 (compatible; SA 1.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.04506; .NET CLR 1.1.4322;*/
const Device UA157171 = {S40519, 175, { &P15364,&P17285,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; ArgClrInt/1.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA164715 = {S40520, 61, { &P15364,&P17285,&P17470,&P18092 }};	/*Useragent - QuickTime/7.6.9 (qtver=7.6.9;os=Windows NT 6.0Service Pack 1)*/
const Device UA296602 = {S40521, 61, { &P15364,&P17285,&P17470,&P18092 }};	/*Useragent - QuickTime/7.6.9 (qtver=7.6.9;os=Windows NT 6.0Service Pack 2)*/
const Device UA416807 = {S40522, 91, { &P15364,&P17285,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; <a href=http://www.reget.com>ReGet Deluxe 5.1</a>; Windows NT 6.0)*/
const Device UA657557 = {S40523, 76, { &P15364,&P17285,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko)*/
const Device UA716727 = {S40524, 56, { &P15364,&P17285,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; fr; rv:1.9.0.1)*/
const Device UA770758 = {S40525, 69, { &P15364,&P17285,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.57.2 (KHTML, like Gecko)*/
const Device UA1872609 = {S40526, 59, { &P15364,&P17285,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; SuperCleaner 2.96; Windows NT 6.0)*/
const Device UA2167926 = {S40527, 50, { &P15364,&P17285,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:12.0) Gecko/ /12.0*/
const Device UA2384799 = {S40528, 45, { &P15364,&P17285,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows Vista 6.0) Java/1.7.0_09*/
const Device UA2456978 = {S40529, 61, { &P15364,&P17285,&P17470,&P18092 }};	/*Useragent - QuickTime/7.7.3 (qtver=7.7.3;os=Windows NT 6.0Service Pack 2)*/
const Device UA2480735 = {S40530, 45, { &P15364,&P17285,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows Vista 6.0) Java/1.6.0_37*/
const Device UA2555492 = {S40531, 68, { &P15364,&P17285,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.11) Gecko/X*/
const Device UA2774851 = {S40532, 59, { &P15364,&P17285,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.6)*/
const Device UA2981460 = {S40533, 60, { &P15364,&P17285,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.11)*/
const Device UA455257 = {S40534, 105, { &P17595,&P17294,&P22505,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.4; en; rv:1.9.2.28) Gecko/X Camino/2.1.2 (like Firefox/3.6.28)*/
const Device UA3085917 = {S40535, 103, { &P17595,&P17294,&P22505,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.4; en; rv:1.9.2.28) Gecko/X Camino/2.1.2 (like Firefox/13.0)*/
const Device UA225320 = {S40536, 129, { &P17168,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7310 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA556637 = {S40537, 133, { &P17168,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7310 Build/HONEYCOMB) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA625018 = {S40538, 136, { &P17168,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7310 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA691499 = {S40539, 58, { &P17168,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; GT-P7310 Build/HMJ37)*/
const Device UA2607718 = {S40540, 159, { &P17168,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7310 Build/HMJ37) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2607919 = {S40541, 150, { &P17168,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7310 Build/HMJ37; 1280*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2610344 = {S40542, 70, { &P17168,&P6546,&P4459,&P18092 }};	/*Useragent - Logos/1.1.3 (Linux; U; Android 3.1; en-US; GT-P7310 Build/HMJ37.UEKHA)*/
const Device UA2612272 = {S40543, 127, { &P17168,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7310 Build/HMJ37) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2615480 = {S40544, 114, { &P17168,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Android 3.1; en-us; GT-P7310 Build/HMJ37) FreeWheelAdManager/5.0.3-r9054-X;com.gotv.crackle.handset/X*/
const Device UA2615674 = {S40545, 268, { &P17168,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7310 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.10;FBDM/{density=1.0,width=1280,height=800};FBLC/en_GB;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P7310;FBSV/3.1;]*/
const Device UA2621611 = {S40546, 66, { &P17168,&P6546,&P4459,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 3.1; en-GB; GT-P7310 Build/HMJ37)*/
const Device UA2627522 = {S40547, 127, { &P17168,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-; GT-P7310 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2629189 = {S40548, 87, { &P17168,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik 2.06.12230; (Linux; U; Android 3.1; GT-P7310; ON) b0a76adfbda439e1; 1280x800x1.0*/
const Device UA428368 = {S40549, 132, { &P17482,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ARCHOS 80G9 Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA848517 = {S40550, 61, { &P17482,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; ARCHOS 80G9 Build/MR1)*/
const Device UA2257638 = {S40551, 135, { &P17482,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ARCHOS 80G9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2424188 = {S40552, 64, { &P17482,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ARCHOS 80G9 Build/IMM76D)*/
const Device UA2242177 = {S40553, 141, { &P18592,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC One VX Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2403696 = {S40554, 149, { &P18592,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PM36100/1.17.502.5 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2505158 = {S40555, 63, { &P18592,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC One VX Build/IMM76I)*/
const Device UA2983281 = {S40556, 149, { &P18592,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PM36100/1.17.502.4 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3027865 = {S40557, 149, { &P18592,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PM36100/1.17.502.9 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1987751 = {S40558, 141, { &P20937,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR910L 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1994157 = {S40559, 138, { &P20937,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR910L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2178596 = {S40560, 60, { &P20937,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; ADR910L Build/IML74K)*/
const Device UA2546914 = {S40561, 141, { &P20937,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR910L 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558773 = {S40562, 138, { &P20937,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR910L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2705891 = {S40563, 60, { &P20937,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ADR910L Build/IMM76D)*/
const Device UA16509 = {S40564, 207, { &P12849,&P5267,&P3834,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S7230E/S723EDTKB1; U; Bada/1.0; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 NexPlayer/3.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B*/
const Device UA2205427 = {S40565, 150, { &P17346,&P18117,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9B206 Safari/7534.48.3*/
const Device UA2205812 = {S40566, 148, { &P17346,&P18117,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9B176 Safari/7534.48.3*/
const Device UA2597214 = {S40567, 147, { &P17346,&P18117,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 511 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/21.0.1180.82 Mobile/9B206 Safari/7534.48.3*/
const Device UA2850060 = {S40568, 146, { &P17346,&P18117,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 51 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/21.0.1180.82 Mobile/9B176 Safari/7534.48.3*/
const Device UA604635 = {S40569, 141, { &P15364,&P17818,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/11.04 Chromium/18.0.1025.151 Chrome/18.0.1025.151 Safari/535.19*/
const Device UA605027 = {S40570, 139, { &P15364,&P17818,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/11.04 Chromium/18.0.1025.151 Chrome/18.0.1025.151 Safari/535.19*/
const Device UA2315354 = {S40571, 176, { &P15364,&P17818,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/11.04 Chromium/18.0.1025.151 Chrome/18.0.1025.151 Safari/535.19,gzip(gfe) (via translate.google.com)*/
const Device UA2594510 = {S40572, 149, { &P15364,&P17818,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/11.04 Chromium/18.0.1025.151 Chrome/18.0.1025.151 Safari/535.19,gzip(gfe)*/
const Device UA49398 = {S40573, 136, { &P13011,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-VS700 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA659175 = {S40574, 60, { &P13011,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-VS700 Build/GRJ22)*/
const Device UA678912 = {S40575, 135, { &P13011,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;LG-VS700 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA683451 = {S40576, 135, { &P13011,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-VS700 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA697816 = {S40577, 135, { &P13011,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; LG-VS700 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA753540 = {S40578, 135, { &P13011,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; LG-VS700 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA825975 = {S40579, 135, { &P13011,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-VS700Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1903143 = {S40580, 143, { &P13011,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-VS700 Build/GRJ22) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2742330 = {S40581, 134, { &P13011,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; LG-VS700 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA19374 = {S40582, 146, { &P12082,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; X10a Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA26333 = {S40583, 146, { &P12082,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-mx; X10a Build/2.0.A.0.504) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA33402 = {S40584, 158, { &P12082,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SonyEricssonX10a Build/2.1.A.0.492) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA42162 = {S40585, 147, { &P12082,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; X10a Build/2.1.1.A.0.16) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA42178 = {S40586, 93, { &P12082,&P5477,&P4861,&P18092 }};	/*Useragent - Android (Linux; U; Android 2.1-update1; fa-ir; X10a) AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA53512 = {S40587, 147, { &P12082,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-co; X10a Build/2.0.2.A.0.24) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA54545 = {S40588, 146, { &P12082,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; X10a Build/2.1.A.0.435) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA58633 = {S40589, 146, { &P12082,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; X10a Build/2.1.A.0.492) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA705813 = {S40590, 68, { &P12082,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; X10a Build/2.0.A.0.504)*/
const Device UA2018720 = {S40591, 69, { &P12082,&P5477,&P4861,&P18092 }};	/*Useragent - PVPlayer X10a CORE/7.507.1.1 OpenCORE/2.05 (Linux;Android2.1-update1)*/
const Device UA50248 = {S40592, 128, { &P13687,&P17603,&P4732,&P18092 }};	/*Useragent - PantechP7040/JFUS03152010F; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.16479; U; en-US) Opera 9.50*/
const Device UA56117 = {S40593, 125, { &P13687,&P17603,&P4732,&P18092 }};	/*Useragent - PantechP7040/JFUS03152010F; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.16479; U; es) Opera 9.50*/
const Device UA67321 = {S40594, 127, { &P13687,&P17603,&P4732,&P18092 }};	/*Useragent - PantechP7040/JLUS05242010; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.16479; U; en-US) Opera 9.50*/
const Device UA68863 = {S40595, 127, { &P13687,&P17603,&P4732,&P18092 }};	/*Useragent - PantechP7040/JLUS04042011; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.16479; U; en-US) Opera 9.50*/
const Device UA2146935 = {S40596, 117, { &P13687,&P17603,&P4732,&P18092 }};	/*Useragent - PantechP7040/JLUSX; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.16479; U; fr) Opera 9.50*/
const Device UA2977759 = {S40597, 118, { &P13687,&P17603,&P4732,&P18092 }};	/*Useragent - PantechP7040/JFUSXF; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.16479; U; fr) Opera 9.50*/
const Device UA2669490 = {S40598, 75, { &P17595,&P18207,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.8.3) Presto/2.12.388 Version/12.15*/
const Device UA287937 = {S40599, 110, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.1 (KHTML, like Gecko) CEF/0.0.0.0 Safari/534.1*/
const Device UA419909 = {S40600, 112, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) VIDEODOWNLOAD Safari/533.3*/
const Device UA2199041 = {S40601, 108, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.3 (KHTML, like Gecko) Lunascape/6.8.0.25815 Safari/535.3*/
const Device UA2232157 = {S40602, 101, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.3 (KHTML, like Gecko) Lunascape/6.8.0.25815 Safari/535.3*/
const Device UA2258979 = {S40603, 99, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) iLivid/1.0 Safari/534.34*/
const Device UA2355829 = {S40604, 92, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.34 (KHTML, like Gecko) iLivid/1.0 Safari/534.34*/
const Device UA2362774 = {S40605, 97, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (N; Windows NT 6.1; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) Raptr Safari/534.34*/
const Device UA2386154 = {S40606, 90, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (N; Windows NT 6.1) AppleWebKit/534.34 (KHTML, like Gecko) Raptr Safari/534.34*/
const Device UA2468961 = {S40607, 96, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.11 (KHTML, like Gecko) SuperBird/23.0 Safari/537.11*/
const Device UA2485074 = {S40608, 103, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) SuperBird/23.0 Safari/537.11*/
const Device UA2532575 = {S40609, 111, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Superbird/24.0.1312.57 Safari/537.17*/
const Device UA2536738 = {S40610, 104, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.17 (KHTML, like Gecko) Superbird/24.0.1312.57 Safari/537.17*/
const Device UA2653030 = {S40611, 99, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.14 (KHTML, like Gecko) JavaFX/2.2 Safari/535.14*/
const Device UA2672957 = {S40612, 104, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML, like Gecko) Superbird/25.0.1364.97 Safari/537.22*/
const Device UA2691466 = {S40613, 101, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, Like Gecko) 20.0.1132.47 Safari/536.11*/
const Device UA2709229 = {S40614, 103, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Superbird/27.0.1450.0 Safari/537.36*/
const Device UA2726666 = {S40615, 94, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.11 (KHTML, Like Gecko) 20.0.1132.47 Safari/536.11*/
const Device UA2731977 = {S40616, 94, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.15 (KHTML, like Gecko) Steam Safari/535.15*/
const Device UA2738724 = {S40617, 110, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-GB) AppleWebKit/534.3 (KHTML, like Gecko) MBMANDR/2.2 Safari/534.3*/
const Device UA2781846 = {S40618, 112, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-GB) AppleWebKit/537.11 (KHTML, like Gecko) MBMANDR/2.2 Safari/537.11*/
const Device UA2784910 = {S40619, 111, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Superbird/27.0.1454.94 Safari/537.22*/
const Device UA2844076 = {S40620, 93, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/ Safari/*/
const Device UA3023748 = {S40621, 111, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Superbird/28.0.1500.71 Safari/537.36*/
const Device UA3091438 = {S40622, 94, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Steam Safari/535.19*/
const Device UA3094465 = {S40623, 95, { &P15364,&P7133,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.21 (KHTML, like Gecko) Banana Safari/537.21*/
const Device UA2801209 = {S40624, 42, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Passbook/1.0 CFNetwork/646.4 Darwin/14.0.0*/
const Device UA3093246 = {S40625, 51, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - SiriViewService/1.0 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3094895 = {S40626, 39, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Web/1.0 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3095026 = {S40627, 51, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - AtomicBrowser/7.0.1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3095893 = {S40628, 49, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Coast/1.0.2.62956 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3096742 = {S40629, 46, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Instapaper/5.0 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3096842 = {S40630, 46, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Wallpapers/1.6 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3096902 = {S40631, 43, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Newsify/115 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3096920 = {S40632, 50, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Google/2.5.1.13455 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3096932 = {S40633, 47, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - ImageMobile/2.0 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3109538 = {S40634, 49, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - AlienBlueHD/2.3.0 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3109699 = {S40635, 47, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Guidebook/2.2.6 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3109768 = {S40636, 62, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - itunesstored (unknown version) CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3109770 = {S40637, 46, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - locationd/1613 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3110011 = {S40638, 55, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - MusicDownloaderLite/1.1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3110080 = {S40639, 41, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Edmodo/14 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3110185 = {S40640, 48, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Apolloplus/2.8.1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3110279 = {S40641, 63, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - IMRemoteURLConnectionAgent/1000 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3110357 = {S40642, 45, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - GroupMe/4.2.5 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3110421 = {S40643, 53, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Perfect%20Browser/9.1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3110447 = {S40644, 60, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - NortonSafeBrowser/168.2.5.14 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3110478 = {S40645, 62, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - itunescloudd (unknown version) CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3110499 = {S40646, 44, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Keeper/7.0.4 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3110525 = {S40647, 48, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - AtomicLite/7.0.1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3110686 = {S40648, 57, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - assetsd (unknown version) CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3110715 = {S40649, 54, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Hot%20Wallpapers/10.02 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3111031 = {S40650, 50, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - GeoServices/702.14 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3111327 = {S40651, 46, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Dashlane/45475 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3111340 = {S40652, 45, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - browser/40887 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3111610 = {S40653, 44, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - ImagePro/2.6 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3111774 = {S40654, 47, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - AlienBlue/2.8.2 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3111779 = {S40655, 56, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - ImageSearcherFreeS/1.1.1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3112316 = {S40656, 49, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - NaverSearch/4.7.6 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3112522 = {S40657, 55, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - 2%20Million%20WPs/10.09 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3112788 = {S40658, 63, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - syncdefaultsd (unknown version) CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3112989 = {S40659, 61, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - dataaccessd (unknown version) CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3113116 = {S40660, 46, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - SpringBoard/50 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3113337 = {S40661, 59, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - securityd (unknown version) CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3113417 = {S40662, 51, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - AssistantServices/1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3113989 = {S40663, 43, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Weather/1.0 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3114230 = {S40664, 42, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Lync%X/2.2 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3114625 = {S40665, 52, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - IMTransferAgent/1000 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3114630 = {S40666, 52, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - CLIPish%20Pro2/10.11 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3114631 = {S40667, 46, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Clipish/9.29.1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3114634 = {S40668, 49, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - YandexSearch/1480 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3114639 = {S40669, 48, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Incredimail/6361 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3114640 = {S40670, 51, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - MusicDownloader/1.1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3114664 = {S40671, 91, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - %E5%A4%9A%E5%B1%8F%E4%BA%92%E5%8A%A8%E6%B5%8F%E8%A7%88/1976 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3114800 = {S40672, 45, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Qrafter/7.7.1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3114905 = {S40673, 50, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - TubeDownloader/1.3 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3114974 = {S40674, 47, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Prismatic/1.1.4 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3115246 = {S40675, 47, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Pinterest/2.6.2 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3115566 = {S40676, 42, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - mstreamd/2 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3115781 = {S40677, 44, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - LinkedIn/6.2 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3116188 = {S40678, 52, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Pic%20Collage/3.15.7 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3126067 = {S40679, 49, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Readability/1.3.0 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3127028 = {S40680, 53, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - GoodReaderIPad/3.19.4 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3127418 = {S40681, 43, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - ScoopIt/111 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3127449 = {S40682, 49, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - BrowserFree/1.2.1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3127511 = {S40683, 41, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - ubd/289.1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3127533 = {S40684, 54, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - MusicDownloaderSSA/1.1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3127579 = {S40685, 46, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - ysearch2/3.0.4 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3127603 = {S40686, 57, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Million%20HD%20WPs/8.80.1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3127605 = {S40687, 45, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - 123people/3.4 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3127650 = {S40688, 48, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - SKOUT_SDK4/3.8.0 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3127718 = {S40689, 49, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - VZ%20Messages/1.1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3127753 = {S40690, 55, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - ImageSearcherFree/1.0.6 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3127955 = {S40691, 45, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - BaiduHD/2.3.1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3128020 = {S40692, 43, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Searchx/2.0 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3128062 = {S40693, 43, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Pulse/3.2.1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3128075 = {S40694, 58, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - MusicDownloaderSSAFree/1.1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3128101 = {S40695, 55, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - VideoDLPlusPlusFree/1.1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3128112 = {S40696, 43, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - AdSheet/1.0 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3128180 = {S40697, 51, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - iDownloaderLite/1.2 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3128196 = {S40698, 51, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - FreeRSSReader/1.9.5 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3128848 = {S40699, 50, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - FastBrowser/10.3.4 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3129225 = {S40700, 49, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - VideoDLFree/1.1.1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3130199 = {S40701, 61, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - Free%20Video%20Downloader/1.5 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3130219 = {S40702, 49, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - ImageHunter/2.5.1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3130681 = {S40703, 46, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - whisper/2.9.94 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3131230 = {S40704, 43, { &P15364,&P23096,&P17470,&P18092 }};	/*Useragent - MyPad/3.1.1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA2433594 = {S40705, 192, { &P17595,&P17296,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0)*/
const Device UA2557762 = {S40706, 139, { &P19856,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-LS970 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2606348 = {S40707, 61, { &P19856,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-LS970 Build/JZO54K)*/
const Device UA207329 = {S40708, 106, { &P14738,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; 7 Mozart; Orange)*/
const Device UA217497 = {S40709, 104, { &P14738,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; 7 Mozart T8698)*/
const Device UA224857 = {S40710, 98, { &P14738,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; 7 Mozart)*/
const Device UA228764 = {S40711, 95, { &P14738,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; T8697)*/
const Device UA658511 = {S40712, 96, { &P14738,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Mozart)*/
const Device UA925696 = {S40713, 97, { &P14738,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; PD67100)*/
const Device UA925703 = {S40714, 111, { &P14738,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Mersad 7 Mozart T8698)*/
const Device UA925918 = {S40715, 102, { &P14738,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Mozart T8698)*/
const Device UA1996146 = {S40716, 62, { &P17968,&P17979,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:17.0) Gecko/17.0 Firefox/17.0*/
const Device UA2028335 = {S40717, 68, { &P17968,&P17979,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:17.0a1) Gecko/17.0a1 Firefox/17.0a1*/
const Device UA2256931 = {S40718, 68, { &P17968,&P17979,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:17.0a2) Gecko/17.0a2 Firefox/17.0a2*/
const Device UA3043578 = {S40719, 64, { &P17968,&P17979,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:17.0) Gecko/17.0 OAFirefox/17.0*/
const Device UA904380 = {S40720, 161, { &P21023,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; LG-E405f Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA2027912 = {S40721, 160, { &P21023,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; LG-E405 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA2242223 = {S40722, 60, { &P21023,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-E405 Build/GRK39F)*/
const Device UA2743509 = {S40723, 61, { &P21023,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-E405f Build/GRK39F)*/
const Device UA19546 = {S40724, 163, { &P12896,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P999 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA19551 = {S40725, 163, { &P12896,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.2; en-us; LG-P999 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA42257 = {S40726, 161, { &P12896,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en; LG-P999 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA42272 = {S40727, 164, { &P12896,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-P999; Build/FRE83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA58222 = {S40728, 163, { &P12896,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P999Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA58228 = {S40729, 163, { &P12896,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.2; en-us; LG-P999 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA58229 = {S40730, 163, { &P12896,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;LG-P999 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA63802 = {S40731, 164, { &P12896,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P999 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA191212 = {S40732, 60, { &P12896,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P999 Build/FRG83G)*/
const Device UA2018830 = {S40733, 133, { &P12896,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-P999 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA54767 = {S40734, 150, { &P11950,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; SonyEricssonX10i Build/R2BA026) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA119857 = {S40735, 151, { &P11950,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; SonyEricssonX10iv Build/R2BA024) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA589374 = {S40736, 66, { &P15364,&P7133,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.1; rv:10.0) Gecko/X Firefox/9.0*/
const Device UA2233711 = {S40737, 70, { &P15364,&P7133,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:9.0.1) Gecko/X Firefox/9.0.1,gzip(gfe)*/
const Device UA2394167 = {S40738, 84, { &P15364,&P7133,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:6.0.1) Gecko/X myibrow/6.0.1 Firefox/9.0*/
const Device UA2767575 = {S40739, 64, { &P15364,&P7133,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows:Windows NT 6.1;rv:10.0) Gecko/X Firefox/9.0*/
const Device UA13234 = {S40740, 174, { &P12588,&P5055,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaC6-01/014.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.8.10 3gpp-gba*/
const Device UA24719 = {S40741, 176, { &P12588,&P5055,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaC6-01.3/014.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.8.10 3gpp-gba*/
const Device UA49566 = {S40742, 189, { &P12588,&P5055,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaC6-01.3/022.014; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.37 Mobile Safari/533.4 3gpp-gba*/
const Device UA50060 = {S40743, 187, { &P12588,&P5055,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaC6-01/022.014; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.37 Mobile Safari/533.4 3gpp-gba*/
const Device UA377133 = {S40744, 10, { &P12588,&P5055,&P3804,&P18092 }};	/*Useragent - NokiaC6-01*/
const Device UA150227 = {S40745, 18, { &P18254,&P7480,&P17470,&P18090 }};	/*Useragent - Yahoo! Slurp China*/
const Device UA1996810 = {S40746, 104, { &P18254,&P18110,&P18519,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.6.1 Safari/534.34*/
const Device UA2221377 = {S40747, 102, { &P18254,&P18110,&P18519,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.6.0 Safari/534.34*/
const Device UA225905 = {S40748, 134, { &P18195,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; YP-G70 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA379005 = {S40749, 58, { &P18195,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; YP-G70 Build/FROYO)*/
const Device UA2809321 = {S40750, 132, { &P18195,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-; YP-G70 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2704812 = {S40751, 169, { &P22596,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SCH-R970 USCC Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2736164 = {S40752, 165, { &P22596,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SCH-R970C Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2937684 = {S40753, 165, { &P22596,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SCH-R970X Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3101042 = {S40754, 176, { &P22596,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SCH-R970 USCC-R970 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0.X Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA630734 = {S40755, 119, { &P17595,&P17295,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; nb-no) AppleWebKit/531.21.8 (KHTML, like Gecko) Gecko/X Firefox/3.5.5*/
const Device UA717434 = {S40756, 83, { &P17595,&P17295,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.5; de; rv:1.9.1.7) Gecko/X Firefox/3.5.7*/
const Device UA2974027 = {S40757, 136, { &P22597,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2229755 = {S40758, 138, { &P16292,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; Galaxy Nexus Build/IMM96K) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/25.0.1207.1 Mobile Safari/8536.25*/
const Device UA2566616 = {S40759, 142, { &P16292,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Galaxy Nexus Build/JRO03O) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566749 = {S40760, 135, { &P16292,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Galaxy Nexus Build/JRO03O) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566906 = {S40761, 142, { &P16292,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567338 = {S40762, 135, { &P16292,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2569931 = {S40763, 140, { &P16292,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; Galaxy Nexus Build/JRN84D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583373 = {S40764, 138, { &P16292,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Galaxy X Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA896609 = {S40765, 140, { &P18072,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; SHV-E120K Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899191 = {S40766, 140, { &P18072,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SHV-E120L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899217 = {S40767, 140, { &P18072,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SHV-E120S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA924236 = {S40768, 140, { &P18072,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E120S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1961508 = {S40769, 140, { &P18072,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E120K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027918 = {S40770, 62, { &P18072,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHV-E120S Build/IMM76D)*/
const Device UA2032868 = {S40771, 140, { &P18072,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E120L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033912 = {S40772, 62, { &P18072,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SHV-E120L Build/IML74K)*/
const Device UA2419751 = {S40773, 144, { &P18072,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; SHV-E120L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?i Safari/534.30*/
const Device UA2463852 = {S40774, 62, { &P18072,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHV-E120K Build/IMM76D)*/
const Device UA2466080 = {S40775, 136, { &P18072,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; vi-vn; E120L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2478642 = {S40776, 62, { &P18072,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHV-E120L Build/IMM76D)*/
const Device UA2815694 = {S40777, 138, { &P18072,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; zh-cn; SHV-E120L Build/Mander) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2830174 = {S40778, 131, { &P18072,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SHV-E120L Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3043603 = {S40779, 144, { &P18072,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; SHV-E120K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?i Safari/534.30*/
const Device UA441461 = {S40780, 125, { &P15364,&P5690,&P24195,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.12.12934.201*/
const Device UA686796 = {S40781, 123, { &P15364,&P5690,&P24195,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.7 (KHTML, like Gecko) Chrome/20.0.1099.0 Safari/536.7 QQBrowser/6.12.13631.201*/
const Device UA733236 = {S40782, 88, { &P15364,&P5690,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0) QQBrowser/6.12.12934.201*/
const Device UA1951570 = {S40783, 79, { &P15364,&P5690,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; QQBrowser/6.12)*/
const Device UA1952655 = {S40784, 71, { &P15364,&P5690,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QQBrowser/6.12)*/
const Device UA1962979 = {S40785, 71, { &P15364,&P5690,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; SV1; QQBrowser/6.12)*/
const Device UA1969676 = {S40786, 79, { &P15364,&P5690,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; QQBrowser/6.12)*/
const Device UA2242681 = {S40787, 167, { &P15364,&P5690,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/6.12)*/
const Device UA2253412 = {S40788, 167, { &P15364,&P5690,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/6.12)*/
const Device UA2364689 = {S40789, 71, { &P15364,&P5690,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; QQBrowser/6.12)*/
const Device UA2395302 = {S40790, 113, { &P15364,&P5690,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)*/
const Device UA2412839 = {S40791, 113, { &P15364,&P5690,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)*/
const Device UA2497039 = {S40792, 184, { &P15364,&P5690,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; QQBrowser/6.12)*/
const Device UA2519915 = {S40793, 125, { &P15364,&P5690,&P24195,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.12.12934.201*/
const Device UA2547025 = {S40794, 184, { &P15364,&P5690,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; QQBrowser/6.12)*/
const Device UA2574776 = {S40795, 113, { &P15364,&P5690,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)*/
const Device UA2815805 = {S40796, 192, { &P15364,&P5690,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C; QQBrowser/6.12)*/
const Device UA2817858 = {S40797, 174, { &P15364,&P5690,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/6.12)*/
const Device UA2933710 = {S40798, 113, { &P15364,&P5690,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)*/
const Device UA2934971 = {S40799, 113, { &P15364,&P5690,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)*/
const Device UA16590 = {S40800, 73, { &P14826,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (04.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac1-01*/
const Device UA24533 = {S40801, 62, { &P14826,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (04.40) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA41167 = {S40802, 67, { &P14826,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaC1-01_CMCC/2.0 (05.40) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA61525 = {S40803, 65, { &P14826,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaC1-01.1/2.0 (04.40a) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2983154 = {S40804, 125, { &P14826,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA94393 = {S40805, 47, { &P15364,&P7133,&P18337,&P18092 }};	/*Useragent - Opera/9.62 (Windows NT 6.1; U; en) Presto/2.1.1*/
const Device UA99987 = {S40806, 47, { &P15364,&P7133,&P18337,&P18092 }};	/*Useragent - Opera/9.60 (Windows NT 6.1; U; en) Presto/2.1.1*/
const Device UA101994 = {S40807, 47, { &P15364,&P7133,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 6.1; U; en) Presto/2.1.1*/
const Device UA123598 = {S40808, 47, { &P15364,&P7133,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 6.1; U; en) Presto/2.1.1*/
const Device UA271134 = {S40809, 47, { &P15364,&P7133,&P18337,&P18092 }};	/*Useragent - Opera/9.61 (Windows NT 6.1; U; en) Presto/2.1.1*/
const Device UA443111 = {S40810, 88, { &P15364,&P7133,&P18337,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; U; YB/3.5.1; ru; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 9.64*/
const Device UA503251 = {S40811, 70, { &P15364,&P7133,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.1.1*/
const Device UA2263380 = {S40812, 75, { &P15364,&P7133,&P18337,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; YB/3.5.1; ru) Opera 9.64*/
const Device UA2639906 = {S40813, 78, { &P15364,&P7133,&P18337,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; U; en; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 9.63*/
const Device UA626428 = {S40814, 137, { &P18493,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-MS840 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA632273 = {S40815, 61, { &P18493,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-MS840 Build/GRK39F)*/
const Device UA2023205 = {S40816, 136, { &P18493,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-MS840 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2759050 = {S40817, 135, { &P18493,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; LG-MS840 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2973938 = {S40818, 136, { &P18713,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975016 = {S40819, 136, { &P18713,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LGMS769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975022 = {S40820, 136, { &P18713,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P768 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975951 = {S40821, 136, { &P18713,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2481611 = {S40822, 140, { &P19902,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2554876 = {S40823, 62, { &P19902,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-R530M Build/JZO54K)*/
const Device UA2935245 = {S40824, 138, { &P19902,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SCH-R530M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3070497 = {S40825, 140, { &P19902,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-R530M Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA24014 = {S40826, 116, { &P15655,&P17484,&P3834,&P18092 }};	/*Useragent - SAMSUNG-GT-S3850/1.0 SHP/VPP/R5 Dolfin/2.0 NexPlayer/3.0 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B*/
const Device UA622454 = {S40827, 142, { &P13074,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P930 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30*/
const Device UA853027 = {S40828, 143, { &P13074,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LG-P930/V18f Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1989731 = {S40829, 138, { &P13074,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LG-P930 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1991986 = {S40830, 138, { &P13074,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P930 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1993227 = {S40831, 143, { &P13074,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P930/V20c Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2017136 = {S40832, 143, { &P13074,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-P930/V20e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2225536 = {S40833, 60, { &P13074,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P930 Build/IMM76D)*/
const Device UA2387779 = {S40834, 143, { &P13074,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-P930/V20f Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2043504 = {S40835, 123, { &P15364,&P7133,&P24194,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.7 (KHTML, like Gecko) Chrome/20.0.1099.0 Safari/536.7 QQBrowser/6.14.15493.201*/
const Device UA2161430 = {S40836, 201, { &P15364,&P7133,&P24194,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C) QQBrowser/6.14.14517.201*/
const Device UA2205979 = {S40837, 186, { &P15364,&P7133,&P24194,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.14.15493.201*/
const Device UA2206137 = {S40838, 193, { &P15364,&P7133,&P24194,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.14.15493.201*/
const Device UA2229595 = {S40839, 165, { &P15364,&P7133,&P24194,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C) QQBrowser/6.14.15493.201*/
const Device UA2259669 = {S40840, 95, { &P15364,&P7133,&P24194,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0) QQBrowser/6.14.15493.201*/
const Device UA2362031 = {S40841, 162, { &P15364,&P7133,&P24194,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729) QQBrowser/6.14.15493.201*/
const Device UA2382064 = {S40842, 193, { &P15364,&P7133,&P24194,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.14.15493.201*/
const Device UA2402813 = {S40843, 133, { &P15364,&P7133,&P24194,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.7 (KHTML, like Gecko) Chrome/20.0.1099.0 Safari/536.7 QQBrowser/6.14.15493.201,gzip(gfe)*/
const Device UA2410454 = {S40844, 186, { &P15364,&P7133,&P24194,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.14.14517.201*/
const Device UA2500276 = {S40845, 88, { &P15364,&P7133,&P24194,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0) QQBrowser/6.14.15493.201*/
const Device UA2790209 = {S40846, 249, { &P15364,&P7133,&P24194,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Alexa Toolbar) QQBrowser/6.14.15493.201*/
const Device UA3118084 = {S40847, 182, { &P15364,&P7133,&P24194,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; qdesk 2.5.1270.201; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; BRI/2) QQBrowser/6.14.15493.201*/
const Device UA10338 = {S40848, 106, { &P15364,&P18110,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/540.0 (KHTML,like Gecko) Chrome/9.1.0.0 Safari/540.0*/
const Device UA173281 = {S40849, 110, { &P15364,&P18110,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13*/
const Device UA315969 = {S40850, 119, { &P15364,&P18110,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.14 (KHTML, like Gecko; Google Web Preview) Chrome/9.0.597 Safari/534.14*/
const Device UA494419 = {S40851, 111, { &P15364,&P18110,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.0 Safari/534.13*/
const Device UA753120 = {S40852, 111, { &P15364,&P18110,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Windows NT 6; en-US) AppleWebKit/534.12 (KHTML, like Gecko) Chrome/9.0.587.0 Safari/534.12*/
const Device UA2144315 = {S40853, 118, { &P15364,&P18110,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US) AppleWebKit/534.12 (KHTML, like Gecko) Chrome/9.0.576.0 Safari/534.12*/
const Device UA456174 = {S40854, 70, { &P15364,&P17285,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:11.0) Gecko/X Firefox/11.0*/
const Device UA600530 = {S40855, 71, { &P15364,&P17285,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:11.0) Gecko/X Firefox/11.0 Firefox/11.0*/
const Device UA622956 = {S40856, 65, { &P15364,&P17285,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:11.0) Gecko/X Firefox/11.0*/
const Device UA94076 = {S40857, 62, { &P15364,&P7133,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; en) Presto/2.2.15 Version/10.00*/
const Device UA94664 = {S40858, 62, { &P15364,&P7133,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; en) Presto/2.2.15 Version/10.01*/
const Device UA102024 = {S40859, 48, { &P15364,&P7133,&P4812,&P18092 }};	/*Useragent - Opera/10.00 (Windows NT 6.1; U; en) Presto/2.2.0*/
const Device UA509724 = {S40860, 85, { &P15364,&P7133,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.2.15 Version/10.00*/
const Device UA562039 = {S40861, 84, { &P15364,&P7133,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.2.15 Version/10.00*/
const Device UA648839 = {S40862, 85, { &P15364,&P7133,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.5 (build 02842); ru) Presto/2.2.15 Version/10.00*/
const Device UA736626 = {S40863, 99, { &P15364,&P7133,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.2.15 Version/10.00,gzip(gfe) (via translate.google.com)*/
const Device UA770855 = {S40864, 84, { &P15364,&P7133,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.2.15 Version/10.01*/
const Device UA1841829 = {S40865, 79, { &P15364,&P7133,&P4812,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; U; ru; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 10.00*/
const Device UA1875173 = {S40866, 85, { &P15364,&P7133,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.2.15 Version/10.00*/
const Device UA2150617 = {S40867, 84, { &P15364,&P7133,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.2.15 Version/10.00*/
const Device UA2266723 = {S40868, 79, { &P15364,&P7133,&P4812,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; U; ru; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 10.01*/
const Device UA2281416 = {S40869, 85, { &P15364,&P7133,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.2.15 Version/10.01*/
const Device UA2442227 = {S40870, 85, { &P15364,&P7133,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.2.15 Version/10.00*/
const Device UA2450735 = {S40871, 84, { &P15364,&P7133,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.2.15 Version/10.00*/
const Device UA2490364 = {S40872, 66, { &P15364,&P7133,&P4812,&P18092 }};	/*Useragent - Opera/10.02 (Windows NT 6.1; U; en) Presto/22.10.289 Version/10.02*/
const Device UA2640922 = {S40873, 84, { &P15364,&P7133,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5754); ru) Presto/2.2.15 Version/10.00*/
const Device UA2684789 = {S40874, 85, { &P15364,&P7133,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.2.15 Version/10.00*/
const Device UA2746201 = {S40875, 84, { &P15364,&P7133,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.2.15 Version/10.00*/
const Device UA2833617 = {S40876, 84, { &P15364,&P7133,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.2.15 Version/10.00*/
const Device UA90066 = {S40877, 150, { &P16757,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SCH-I500 Build/ECLAIR) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA132392 = {S40878, 145, { &P16757,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SCH-I500 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA139982 = {S40879, 67, { &P16757,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SCH-I500 Build/ECLAIR)*/
const Device UA152149 = {S40880, 143, { &P16757,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-; SCH-I500 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA418173 = {S40881, 144, { &P16757,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.1-update1; en-us; SCH-I500 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA538765 = {S40882, 144, { &P16757,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.1-update1; en-us; SCH-I500 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1879885 = {S40883, 144, { &P16757,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SCH-I500Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018421 = {S40884, 144, { &P16757,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us;SCH-I500 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018532 = {S40885, 144, { &P16757,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SCH-I500 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 MobileSafari/530.17*/
const Device UA2569396 = {S40886, 89, { &P18254,&P5690,&P17381,&P19831 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN; rv:1.8.0.11)  Firefox/1.5.0.11; 360Spider*/
const Device UA2882037 = {S40887, 88, { &P18254,&P5690,&P17381,&P19831 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN; rv:1.8.0.11) Firefox/1.5.0.11 360Spider;*/
const Device UA30678 = {S40888, 95, { &P17595,&P17296,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.1.7) Gecko/20091221 Firefox/3.5.7*/
const Device UA625087 = {S40889, 87, { &P17595,&P17296,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6.8; U; en; rv:1.9.1.6) Gecko/X Firefox/3.5.6*/
const Device UA222049 = {S40890, 137, { &P15364,&P17817,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.30 (KHTML, like Gecko) Ubuntu/10.10 Chromium/12.0.742.112 Chrome/12.0.742.112 Safari/534.30*/
const Device UA232116 = {S40891, 139, { &P15364,&P17817,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.30 (KHTML, like Gecko) Ubuntu/10.10 Chromium/12.0.742.112 Chrome/12.0.742.112 Safari/534.30*/
const Device UA2319595 = {S40892, 144, { &P15364,&P17817,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 (x86_64)) AppleWebKit/534.30 (KHTML, like Gecko) Ubuntu/10.10 Chromium/12.0.742.91 Chrome/12.0.742.91 Safari/534.30*/
const Device UA49878 = {S40893, 142, { &P12572,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SPH-D600 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60741 = {S40894, 142, { &P12572,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SPH-D600 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623731 = {S40895, 142, { &P12572,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-D600 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA649772 = {S40896, 66, { &P12572,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SPH-D600 Build/GINGERBREAD)*/
const Device UA653278 = {S40897, 66, { &P12572,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SPH-D600 Build/GINGERBREAD)*/
const Device UA2685330 = {S40898, 140, { &P12572,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SPH-D600 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2346129 = {S40899, 132, { &P18437,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8000 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2366804 = {S40900, 132, { &P18437,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8020 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2402127 = {S40901, 132, { &P18437,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8000 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2432778 = {S40902, 132, { &P18437,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8020 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2535551 = {S40903, 132, { &P18437,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8005 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2659341 = {S40904, 133, { &P18437,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2;  GT-N8000 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA627378 = {S40905, 152, { &P13025,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; SonyEricssonWT19a Build/4.0.2.A.0.61) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA659892 = {S40906, 153, { &P13025,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; SonyEricssonWT19a Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA701239 = {S40907, 152, { &P13025,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-cl; SonyEricssonWT19a Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA778968 = {S40908, 141, { &P13025,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; WT19a Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA852213 = {S40909, 140, { &P13025,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-cl; WT19a Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA893480 = {S40910, 152, { &P13025,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-ar; SonyEricssonWT19a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA895587 = {S40911, 152, { &P13025,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-gt; SonyEricssonWT19a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1993671 = {S40912, 140, { &P13025,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-ar; WT19a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024305 = {S40913, 65, { &P13025,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; WT19a Build/4.0.1.A.0.283)*/
const Device UA2029073 = {S40914, 151, { &P13025,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; SonyEricssonWT19a Build/4.0.2.D.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2684693 = {S40915, 140, { &P13025,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-gt; WT19a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2725056 = {S40916, 139, { &P13025,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; WT19a Build/4.0.2.D.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2820597 = {S40917, 140, { &P13025,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; WT19a Build/4.0.2.A.0.61) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA503356 = {S40918, 83, { &P15364,&P5690,&P17381,&P18092 }};	/*Useragent - mozilla/5.0 (windows; u; windows nt 5.1; es-ar; rv:1.8.0.7) gecko/X firefox/1.5.0.7*/
const Device UA516789 = {S40919, 80, { &P15364,&P5690,&P17381,&P18092 }};	/*Useragent - mozilla/5.0 (windows; u; windows nt 5.1; it; rv:1.8.0.7) gecko/X firefox/1.5.0.7*/
const Device UA632804 = {S40920, 67, { &P15364,&P5690,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; tr; rv:1.8.0) Gecko/X Firefox/1.5.0*/
const Device UA826638 = {S40921, 83, { &P15364,&P5690,&P17381,&P18092 }};	/*Useragent - mozilla/5.0 (windows; u; windows nt 5.1; pt-br; rv:1.8.0.7) gecko/X firefox/1.5.0.7*/
const Device UA1870416 = {S40922, 65, { &P15364,&P5690,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1) Gecko/X Firefox/1.5.0.12*/
const Device UA1885377 = {S40923, 83, { &P15364,&P5690,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.8.0.8) Gecko/X Firefox/1.5.0.8*/
const Device UA2139235 = {S40924, 80, { &P15364,&P5690,&P17381,&P18092 }};	/*Useragent - mozilla/5.0 (windows; u; windows nt 5.1; ja; rv:1.8.0.7) gecko/X firefox/1.5.0.7*/
const Device UA2720213 = {S40925, 80, { &P15364,&P5690,&P17381,&P18092 }};	/*Useragent - mozilla/5.0 (windows; u; windows nt 5.1; sl; rv:1.8.0.7) gecko/X firefox/1.5.0.7*/
const Device UA2768298 = {S40926, 78, { &P15364,&P5690,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN; rv:1.8.0.11) Firefox/1.5.0.11;*/
const Device UA3116865 = {S40927, 80, { &P15364,&P5690,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0+(Windows;+U;+Windows+NT+5.1;+de;+rv:1.8.0.1)+Gecko/X+Firefox/1.5.0.1*/
const Device UA2505210 = {S40928, 154, { &P21545,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HUAWEI Y210-0010 Build/HuaweiY210-0010) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2530694 = {S40929, 146, { &P21545,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_Y210-2010/Y210-2010V100R001C92B823; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2555440 = {S40930, 154, { &P21545,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HUAWEI Y210-0100 Build/HuaweiY210-0100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2555713 = {S40931, 146, { &P21545,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_Y210-2010/Y210-2010V100R001C92B817; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2583033 = {S40932, 154, { &P21545,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HUAWEI Y210-0200 Build/HuaweiY210-0200) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2655591 = {S40933, 146, { &P21545,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_Y210-2010/Y210-2010V100R001C92B821; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2658604 = {S40934, 138, { &P21545,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_Y210S/Y210SV100R001C92B823; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2668074 = {S40935, 154, { &P21545,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; HUAWEI Y210-0151 Build/HuaweiY210-0151) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2707078 = {S40936, 146, { &P21545,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_Y210-2010/Y210-2010V100R001C92B825; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2734707 = {S40937, 146, { &P21545,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_Y210-2010/Y210-2010V100R001C92B826; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2748864 = {S40938, 146, { &P21545,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_Y210-2010/Y210-2010V100R001C92B824; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2751430 = {S40939, 154, { &P21545,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; HUAWEI Y210-0251 Build/HuaweiY210-0251) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2752626 = {S40940, 150, { &P21545,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-ca; NATCOM N8302 Build/HuaweiY210-0200) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2797367 = {S40941, 146, { &P21545,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_Y210-2010/Y210-2010V100R001C92B828; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2934993 = {S40942, 157, { &P21545,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; HUAWEI Ascend Y 210 Build/HuaweiY210-0100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2987469 = {S40943, 78, { &P21545,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; HUAWEI Y210-0100 Build/HuaweiY210-0100)*/
const Device UA2992589 = {S40944, 158, { &P21545,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; HUAWEI Ascend Y 210D Build/HuaweiY210-0200) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3057520 = {S40945, 154, { &P21545,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; HUAWEI Y210-2010 Build/HuaweiY210-2010) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2969748 = {S40946, 128, { &P18385,&P21637,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus 7 Build/JWR66V) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.111 Safari/537.36*/
const Device UA2674928 = {S40947, 82, { &P15364,&P7133,&P22225,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:20.0) Gecko/X Firefox/20.0 SeaMonkey/2.17.1*/
const Device UA2719119 = {S40948, 69, { &P15364,&P7133,&P22225,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:20.0) Gecko/X SeaMonkey/2.17.1*/
const Device UA2860995 = {S40949, 87, { &P15364,&P7133,&P22225,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:20.0) Gecko/X Firefox/20.0 SeaMonkey/2.17.1*/
const Device UA5631 = {S40950, 136, { &P12280,&P17090,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190 Twitter for iPhone*/
const Device UA5651 = {S40951, 136, { &P12280,&P17090,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 Twitter for iPhone*/
const Device UA5684 = {S40952, 136, { &P12280,&P17090,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 Twitter for iPhone*/
const Device UA5729 = {S40953, 136, { &P12280,&P17090,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Twitter for iPhone*/
const Device UA37167 = {S40954, 136, { &P12280,&P17090,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Twitter for iPhone*/
const Device UA46568 = {S40955, 136, { &P12280,&P17090,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2 Twitter for iPhone*/
const Device UA2300909 = {S40956, 138, { &P12280,&P17090,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/9Z999 Twitter for iPhone*/
const Device UA21277 = {S40957, 136, { &P15178,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-LW690 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59223 = {S40958, 137, { &P15178,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-LW690 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA197080 = {S40959, 60, { &P15178,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; LG-LW690 Build/FRG83)*/
const Device UA666124 = {S40960, 61, { &P15178,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-LW690 Build/FRG83G)*/
const Device UA895670 = {S40961, 112, { &P15785,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia201/11.56; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6*/
const Device UA2234647 = {S40962, 113, { &P15785,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia201/11.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA49190 = {S40963, 150, { &P13031,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; tr-tr; GM Ultimate Slim Build/HuaweiU8510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49207 = {S40964, 139, { &P13031,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; S41HW Build/HuaweiU8510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49965 = {S40965, 139, { &P13031,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; U8510 Build/HuaweiU8510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA554393 = {S40966, 63, { &P13031,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; U8510 Build/HuaweiU8510)*/
const Device UA623982 = {S40967, 139, { &P13031,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; U8510 Build/HuaweiU8510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA635255 = {S40968, 139, { &P13031,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; U8510 Build/HuaweiU8510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA699249 = {S40969, 63, { &P13031,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; U8510 Build/HuaweiU8510)*/
const Device UA2024174 = {S40970, 63, { &P13031,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; U8510 Build/HuaweiU8510)*/
const Device UA2024252 = {S40971, 145, { &P13031,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HuaweiU8510 Build/HuaweiU8510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257155 = {S40972, 143, { &P13031,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-by; MTC Bravo Build/HuaweiU8510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2396018 = {S40973, 139, { &P13031,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U8510 Build/HuaweiU8510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2010613 = {S40974, 153, { &P14111,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SonyEricssonLT15a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029321 = {S40975, 154, { &P14111,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; SonyEricssonLT15av Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030836 = {S40976, 141, { &P14111,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LT15a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2360405 = {S40977, 127, { &P14111,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Arc Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2397980 = {S40978, 63, { &P14111,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT15a Build/4.1.B.0.587)*/
const Device UA2404228 = {S40979, 125, { &P14111,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-; Arc Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2479307 = {S40980, 56, { &P14111,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Arc Build/IMM76L)*/
const Device UA2500199 = {S40981, 141, { &P14111,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; LT15a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2190347 = {S40982, 139, { &P18641,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-M830 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2351376 = {S40983, 61, { &P18641,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SPH-M830 Build/IMM76I)*/
const Device UA2479969 = {S40984, 137, { &P18641,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SPH-M830 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA428547 = {S40985, 133, { &P17211,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ARCHOS 101G9 Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA874264 = {S40986, 62, { &P17211,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; ARCHOS 101G9 Build/MR1)*/
const Device UA2257808 = {S40987, 136, { &P17211,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ARCHOS 101G9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2424426 = {S40988, 65, { &P17211,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ARCHOS 101G9 Build/IMM76D)*/
const Device UA1996991 = {S40989, 155, { &P15575,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2015559 = {S40990, 155, { &P15575,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033039 = {S40991, 156, { &P15575,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100G Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2192560 = {S40992, 155, { &P15575,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2231716 = {S40993, 155, { &P15575,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9100 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2385308 = {S40994, 160, { &P15575,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9100G Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2825230 = {S40995, 180, { &P15575,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100G Build/SlimTW5-jelly-Bean-v3.0#_; CyanogenMod-SlimTW5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2829792 = {S40996, 152, { &P15575,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100 Build/1.5; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3043465 = {S40997, 148, { &P15575,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9100 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA343871 = {S40998, 104, { &P15364,&P16110,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.904.0 Safari/535.7*/
const Device UA401196 = {S40999, 97, { &P15364,&P16110,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.904.0 Safari/535.7*/
const Device UA3087990 = {S41000, 129, { &P15364,&P16110,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.910.20417 KChrome/3.2.5.20417  Safari/535.7*/
const Device UA3064997 = {S41001, 144, { &P22582,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083366 = {S41002, 137, { &P22582,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2675605 = {S41003, 62, { &P17976,&P17979,&P24344,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:23.0) Gecko/23.0 Firefox/23.0*/
const Device UA2799413 = {S41004, 68, { &P17976,&P17979,&P24344,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:23.0a2) Gecko/23.0a2 Firefox/23.0a2*/
const Device UA3011519 = {S41005, 72, { &P17976,&P17979,&P24344,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:23.0) Gecko/23.0 Firefox/23.0,gzip(gfe)*/
const Device UA3047704 = {S41006, 101, { &P17976,&P17979,&P24344,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:23.0) Gecko/23.0 Firefox/23.0,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3125343 = {S41007, 82, { &P17976,&P17979,&P24344,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:23.0) Gecko/23.0 Firefox/23.0,gzip(gfe),gzip(gfe)*/
const Device UA335066 = {S41008, 138, { &P17691,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; SAMSUNG-SGH-I957 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA352393 = {S41009, 131, { &P17691,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-ca; SGH-I957M Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA369494 = {S41010, 78, { &P17691,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 3.2; en_US; SAMSUNG-SGH-I957 Build/MASTER.CKJ7)*/
const Device UA659299 = {S41011, 67, { &P17691,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; SAMSUNG-SGH-I957 Build/MASTER)*/
const Device UA681491 = {S41012, 131, { &P17691,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-ca; SGH-I957R Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA720698 = {S41013, 131, { &P17691,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-ca; SGH-I957D Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2721745 = {S41014, 60, { &P17691,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; SGH-I957R Build/MASTER)*/
const Device UA2807929 = {S41015, 60, { &P17691,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; SGH-I957M Build/MASTER)*/
const Device UA2949244 = {S41016, 145, { &P17691,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; SAMSUNG-SGH-I957 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA61113 = {S41017, 144, { &P12634,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HUAWEI-M835 Build/HuaweiM835) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76561 = {S41018, 142, { &P12634,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-; HUAWEI-M835 Build/HuaweiM835) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78103 = {S41019, 204, { &P12634,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HUAWEI-M835 Build/HuaweiM835) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Android MetroPCS androidAppVersion/2.1 AndroidWebkitWrapper*/
const Device UA378520 = {S41020, 68, { &P12634,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; HUAWEI-M835 Build/HuaweiM835)*/
const Device UA2019495 = {S41021, 151, { &P12634,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HUAWEI-M835 Build/HuaweiM835) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA3117796 = {S41022, 224, { &P12634,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HUAWEI-M835 Build/HuaweiM835) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Android MetroPCS androidAppVersion/2.5.10 AndroidWebkitWrapper BillingSupported*/
const Device UA251468 = {S41023, 129, { &P17687,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; MZ604 Build/H.6.6-23) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA397411 = {S41024, 129, { &P17687,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ604 Build/H.6.6-27) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA578636 = {S41025, 129, { &P17687,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; MZ604 Build/H.6.6-29) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA632669 = {S41026, 58, { &P17687,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; MZ604 Build/H.6.6-23)*/
const Device UA634687 = {S41027, 129, { &P17687,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-ca; MZ604 Build/H.6.6-30) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2601557 = {S41028, 132, { &P18465,&P18110,&P21708,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Safari/535.19 Silk-Accelerated=true*/
const Device UA77759 = {S41029, 190, { &P12671,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.3 Nokia700/111.020.0308; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.4.1.14 Mobile Safari/533.4 3gpp-gba*/
const Device UA605148 = {S41030, 190, { &P12671,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.4 Nokia700/112.010.1404; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.2.1.20 Mobile Safari/535.1 3gpp-gba*/
const Device UA661843 = {S41031, 177, { &P12671,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.3 Nokia700; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.4.1.14 Mobile Safari/533.4 3gpp-gba*/
const Device UA2205914 = {S41032, 190, { &P12671,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.5 Nokia700/113.010.1506; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.2.21 Mobile Safari/535.1 3gpp-gba*/
const Device UA2566805 = {S41033, 150, { &P18113,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567202 = {S41034, 143, { &P18113,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2666889 = {S41035, 135, { &P18113,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT912 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2257394 = {S41036, 146, { &P19911,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-E970/E97010l Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257418 = {S41037, 146, { &P19911,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-E970/E97010o Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2338503 = {S41038, 138, { &P19911,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-E970 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2425911 = {S41039, 146, { &P19911,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-E970/E97011c Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426783 = {S41040, 60, { &P19911,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-E970 Build/IMM76L)*/
const Device UA3065570 = {S41041, 135, { &P18635,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081112 = {S41042, 136, { &P18635,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 4 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081674 = {S41043, 134, { &P18635,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Nexus 4 Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3113445 = {S41044, 128, { &P18635,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA32223 = {S41045, 134, { &P12482,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; SGH-T959 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49733 = {S41046, 143, { &P12482,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; SGH-T959D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA194749 = {S41047, 136, { &P12482,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SGH-T959 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA209786 = {S41048, 136, { &P12482,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T959 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA241922 = {S41049, 143, { &P12482,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; SGH-T959P Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250007 = {S41050, 136, { &P12482,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-T959 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA327166 = {S41051, 142, { &P12482,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SGH-T959 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA456742 = {S41052, 143, { &P12482,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; SGH-T959W Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA572821 = {S41053, 135, { &P12482,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; T959 Build/ReVaMpEd) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA584290 = {S41054, 142, { &P12482,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T959 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633188 = {S41055, 142, { &P12482,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T959 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1900605 = {S41056, 67, { &P12482,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SGH-T959P Build/GINGERBREAD)*/
const Device UA1901353 = {S41057, 60, { &P12482,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; SGH-T959 Build/GRJ22)*/
const Device UA1905204 = {S41058, 67, { &P12482,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SGH-T959D Build/GINGERBREAD)*/
const Device UA2018550 = {S41059, 136, { &P12482,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-T959 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032327 = {S41060, 67, { &P12482,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-T959W Build/GINGERBREAD)*/
const Device UA3041646 = {S41061, 138, { &P12482,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; T959 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2730124 = {S41062, 139, { &P18641,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-M830 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2755097 = {S41063, 61, { &P18641,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SPH-M830 Build/JZO54K)*/
const Device UA2997288 = {S41064, 137, { &P18641,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SPH-M830 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA637775 = {S41065, 96, { &P15364,&P17815,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.5) Gecko/X Ubuntu/9.04 (intrepid) Firefox/3.0.5*/
const Device UA435672 = {S41066, 142, { &P14353,&P5267,&P3834,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5250/1.0; U; bada/1.0; es) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA625424 = {S41067, 206, { &P14353,&P5267,&P3834,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5250/S5250XEKJ3; U; Bada/1.0; ru-ru) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 NexPlayer/3.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B*/
const Device UA395437 = {S41068, 102, { &P18254,&P7133,&P17200,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; FDM; MSIECrawler; Media Center PC 5.0)*/
const Device UA2204335 = {S41069, 76, { &P18254,&P7133,&P17200,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; MSIECrawler)*/
const Device UA2539846 = {S41070, 113, { &P18254,&P7133,&P17200,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; CrawlDaddy v0.1.0 abot v1.1.0.0 http://code.google.com/p/abot)*/
const Device UA2751917 = {S41071, 95, { &P18254,&P7133,&P17200,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; abot v1.1.1.0 http://code.google.com/p/abot)*/
const Device UA627393 = {S41072, 90, { &P17595,&P17297,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.2.25) Gecko/X Firefox/3.6.25*/
const Device UA2566485 = {S41073, 139, { &P18295,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566760 = {S41074, 139, { &P18295,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EndeavorU Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566803 = {S41075, 132, { &P18295,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566923 = {S41076, 135, { &P18295,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; One X Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567581 = {S41077, 139, { &P18295,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One X Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2865275 = {S41078, 128, { &P18295,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; One X Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA172538 = {S41079, 136, { &P16561,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SCH-R910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA175499 = {S41080, 60, { &P16561,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SCH-R910 Build/FROYO)*/
const Device UA250782 = {S41081, 136, { &P16561,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SCH-R910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA654499 = {S41082, 60, { &P16561,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; SCH-R910 Build/FROYO)*/
const Device UA870090 = {S41083, 66, { &P16561,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SCH-R910 Build/FROYO)*/
const Device UA2680908 = {S41084, 88, { &P15364,&P16110,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:20.0) Gecko/X Firefox/20.0 AlexaToolbar/alxf-2.18*/
const Device UA2737585 = {S41085, 62, { &P15364,&P16110,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64;) Gecko/X Firefox/20.0*/
const Device UA2756987 = {S41086, 88, { &P15364,&P16110,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.1-x64*/
const Device UA2757121 = {S41087, 79, { &P15364,&P16110,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.1*/
const Device UA2492060 = {S41088, 70, { &P12280,&P19826,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPhone4,1; iPhone OS 6.1; en_US; en) AppleWebKit/420+*/
const Device UA2512868 = {S41089, 72, { &P12280,&P19826,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPhone4,1; iPhone OS 6.1.1; en_US; en) AppleWebKit/420+*/
const Device UA2532926 = {S41090, 72, { &P12280,&P19826,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPhone4,1; iPhone OS 6.1.2; en_US; en) AppleWebKit/420+*/
const Device UA2582603 = {S41091, 72, { &P12280,&P19826,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPhone4,1; iPhone OS 6.1.3; en_US; en) AppleWebKit/420+*/
const Device UA2738246 = {S41092, 72, { &P12280,&P19826,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPhone5,2; iPhone OS 6.1.4; en_US; en) AppleWebKit/420+*/
const Device UA2202058 = {S41093, 116, { &P15767,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari Version/5.1*/
const Device UA2205254 = {S41094, 159, { &P15767,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Version/5.1 Safari/7534.48.3 NAVER(inapp; search; 103; 3.8.2)*/
const Device UA2205543 = {S41095, 124, { &P15767,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10A403 Safari/7536.26*/
const Device UA2233841 = {S41096, 135, { &P15767,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Version/5.1 Safari/7534.48.3 Kikin/1.9.5*/
const Device UA2357093 = {S41097, 126, { &P15767,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10A523 Safari/7536.26*/
const Device UA2357528 = {S41098, 118, { &P15767,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari Version/5.1*/
const Device UA2404734 = {S41099, 150, { &P15767,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6 like Mac OS X; zh-cn Model:iPad3,4) UC AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7543.48.3*/
const Device UA2462079 = {S41100, 161, { &P15767,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Version/5.1 Safari/7534.48.3 NAVER(inapp; search; 103; 4.2.6)*/
const Device UA2470048 = {S41101, 161, { &P15767,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550 Version/5.1 Safari/7534.48.3 NAVER(inapp; search; 103; 4.2.6)*/
const Device UA2483742 = {S41102, 119, { &P15767,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8426 Safari Version/5.1*/
const Device UA2538611 = {S41103, 127, { &P15767,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA2539850 = {S41104, 125, { &P15767,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA2539852 = {S41105, 127, { &P15767,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA2657081 = {S41106, 119, { &P15767,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8500 Safari Version/5.1*/
const Device UA2763529 = {S41107, 118, { &P15767,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550 Safari Version/5.1*/
const Device UA2933677 = {S41108, 125, { &P15767,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3*/
const Device UA3042633 = {S41109, 126, { &P15767,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/10A403 Safari/7534.48.3*/
const Device UA3085513 = {S41110, 125, { &P15767,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA3099169 = {S41111, 127, { &P15767,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA3099885 = {S41112, 116, { &P15767,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A407 Safari Version/5.1*/
const Device UA662024 = {S41113, 74, { &P15364,&P17350,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT x.y; Win64; x64; rv:10.0.1) Gecko/X Firefox/10.0.1*/
const Device UA862093 = {S41114, 140, { &P13141,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SHV-E110S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA904998 = {S41115, 140, { &P13141,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I727R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA924207 = {S41116, 140, { &P13141,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E110S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033880 = {S41117, 62, { &P13141,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I727R Build/IMM76D)*/
const Device UA2257488 = {S41118, 139, { &P13141,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; GT-I9210 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2445838 = {S41119, 62, { &P13141,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHV-E110S Build/IMM76D)*/
const Device UA2520901 = {S41120, 138, { &P13141,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SGH-I727R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2670247 = {S41121, 139, { &P13141,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-hk; GT-I9210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2727297 = {S41122, 147, { &P13141,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SHV-E110S-parrot Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2786543 = {S41123, 62, { &P13141,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SHV-E110S Build/IML74K)*/
const Device UA3099167 = {S41124, 61, { &P13141,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9210 Build/IML74K)*/
const Device UA834237 = {S41125, 127, { &P17595,&P17297,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_3; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA1969040 = {S41126, 127, { &P17595,&P17297,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA2200474 = {S41127, 117, { &P17595,&P17297,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA2203666 = {S41128, 114, { &P17595,&P17297,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/4.0.3 Safari/531.9*/
const Device UA2203926 = {S41129, 118, { &P17595,&P17297,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA2416448 = {S41130, 127, { &P17595,&P17297,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA2516623 = {S41131, 67, { &P17595,&P17297,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_8 en-us) Version/4.0*/
const Device UA2509637 = {S41132, 59, { &P15364,&P18110,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; Linux i686; Trident/3.0)*/
const Device UA2531840 = {S41133, 61, { &P15364,&P18110,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; Linux x86_64; Trident/5.0)*/
const Device UA2580045 = {S41134, 71, { &P15364,&P18110,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; Linux i686; Win64; x64; Trident/2.0)*/
const Device UA2658838 = {S41135, 70, { &P15364,&P18110,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; Linux i686; Tablet PC; Trident/2.0)*/
const Device UA2659569 = {S41136, 64, { &P15364,&P18110,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; Linux i686; SV1; Trident/1.0)*/
const Device UA2700271 = {S41137, 66, { &P15364,&P18110,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; Linux i686; WOW64; Trident/5.0)*/
const Device UA2730388 = {S41138, 72, { &P15364,&P18110,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; Linux x86_64; Tablet PC; Trident/5.0)*/
const Device UA2791844 = {S41139, 74, { &P15364,&P18110,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; Linux x86_64; Win64; IA64; Trident/1.0)*/
const Device UA2791884 = {S41140, 66, { &P15364,&P18110,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; Linux x86_64; SV1; Trident/5.0)*/
const Device UA2792885 = {S41141, 68, { &P15364,&P18110,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; Linux x86_64; WOW64; Trident/3.0)*/
const Device UA1957695 = {S41142, 139, { &P18459,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VS950 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2242227 = {S41143, 61, { &P18459,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; VS950 4G Build/IMM76D)*/
const Device UA79914 = {S41144, 81, { &P15364,&P17285,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/9*/
const Device UA173140 = {S41145, 118, { &P15364,&P17285,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13*/
const Device UA3080567 = {S41146, 202, { &P15364,&P17285,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U;                             Windows NT 6.0; en-US)                             AppleWebKit/534.13 (KHTML, like Gecko)                             Chrome/9.0.597.98 Safari/534.13*/
const Device UA2035302 = {S41147, 101, { &P18445,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 920)*/
const Device UA2043790 = {S41148, 102, { &P18445,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Juggernaut; Alpha)*/
const Device UA2401664 = {S41149, 109, { &P18445,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 920; ANZ821)*/
const Device UA2404374 = {S41150, 111, { &P18445,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 920; Vodafone)*/
const Device UA2428559 = {S41151, 109, { &P18445,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 920; Orange)*/
const Device UA2652832 = {S41152, 101, { &P18445,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Nokia; Nokia 920)*/
const Device UA2698357 = {S41153, 102, { &P18445,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Nokia 920T)*/
const Device UA3080867 = {S41154, 109, { &P18445,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 920; 1326-2)*/
const Device UA368763 = {S41155, 67, { &P12532,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; HTC Flyer P512 Build/HTK75C)*/
const Device UA398309 = {S41156, 137, { &P12532,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us;HTC_Flyer_P512 Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA398482 = {S41157, 141, { &P12532,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; HTC_Flyer_P512_NA Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA430085 = {S41158, 137, { &P12532,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; HTC Flyer P510e Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA430090 = {S41159, 133, { &P12532,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; HTC_Flyer Build/HTK75C) AppleWebkit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA430096 = {S41160, 137, { &P12532,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-; HTC Flyer P510e Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA430103 = {S41161, 133, { &P12532,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; de-de; HTC Flyer Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA430104 = {S41162, 132, { &P12532,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; de-de;HTC_Flyer Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA430147 = {S41163, 70, { &P12532,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; HTC_Flyer_P512_NA Build/HTK75C)*/
const Device UA622947 = {S41164, 139, { &P12532,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-gb; HTC Flyer P510e Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA634128 = {S41165, 138, { &P12532,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-gb; HTC Flyer P512 Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA679029 = {S41166, 138, { &P12532,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-gb;HTC_Flyer_P510e Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA699854 = {S41167, 68, { &P12532,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; HTC Flyer P510e Build/HTK75C)*/
const Device UA2024706 = {S41168, 140, { &P12532,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-ca;HTC_Flyer_P512_NA Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA3648 = {S41169, 159, { &P13644,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Sprint APA9292KT Build/OMJ_EVO_2.2_Froyo_v3.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10149 = {S41170, 143, { &P13644,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Cricket APA9292KT Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA16404 = {S41171, 145, { &P13644,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Sprint APA9292KT Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA16405 = {S41172, 140, { &P13644,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; -us; Sprint APA9292KT Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40035 = {S41173, 144, { &P13644,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Sprint APA9292KT Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40036 = {S41174, 135, { &P13644,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; PC36100 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48918 = {S41175, 133, { &P13644,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; PC36100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA62748 = {S41176, 142, { &P13644,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Sprint APA9292KT Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA137313 = {S41177, 57, { &P13644,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; PC36100 Build/FRF91)*/
const Device UA149736 = {S41178, 135, { &P13644,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; PC36100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA300031 = {S41179, 134, { &P13644,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; PC36100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA832843 = {S41180, 132, { &P13644,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; PC36100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1903389 = {S41181, 121, { &P13644,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; PC36100 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2023707 = {S41182, 126, { &P13644,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; PC36100 Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2508021 = {S41183, 133, { &P15364,&P18118,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.17 (KHTML, like Gecko) Ubuntu Chromium/24.0.1312.56 Chrome/24.0.1312.56 Safari/537.17*/
const Device UA2510275 = {S41184, 131, { &P15364,&P18118,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.17 (KHTML, like Gecko) Ubuntu Chromium/24.0.1312.56 Chrome/24.0.1312.56 Safari/537.17*/
const Device UA2731007 = {S41185, 141, { &P15364,&P18118,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.17 (KHTML, like Gecko) Ubuntu Chromium/24.0.1312.56 Chrome/24.0.1312.56 Safari/537.17,gzip(gfe)*/
const Device UA2851562 = {S41186, 143, { &P15364,&P18118,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.17 (KHTML, like Gecko) Ubuntu Chromium/24.0.1312.56 Chrome/24.0.1312.56 Safari/537.17,gzip(gfe)*/
const Device UA2621685 = {S41187, 132, { &P22037,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3113 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2636054 = {S41188, 139, { &P22037,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3113 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3055564 = {S41189, 131, { &P22037,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3113 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3064038 = {S41190, 60, { &P22037,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P3113 Build/JDQ39)*/
const Device UA79899 = {S41191, 81, { &P15364,&P17285,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/7*/
const Device UA149275 = {S41192, 116, { &P15364,&P17285,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA195592 = {S41193, 98, { &P15364,&P17285,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.28 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.28*/
const Device UA378476 = {S41194, 105, { &P15364,&P17285,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.28 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.28*/
const Device UA254472 = {S41195, 126, { &P18017,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; K1 Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA674676 = {S41196, 55, { &P18017,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; K1 Build/HTK75C)*/
const Device UA47462 = {S41197, 149, { &P14911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Milestone X Build/4.5.1_57_MX2-27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47482 = {S41198, 149, { &P14911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Milestone X Build/4.5.1_57_MX2-20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA252545 = {S41199, 149, { &P14911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Milestone X Build/4.5.1_57_MX2-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA359548 = {S41200, 73, { &P14911,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Milestone X Build/4.5.1_57_MX2-34)*/
const Device UA370623 = {S41201, 79, { &P14911,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.5; en_US; Milestone X Build/4.5.1_57_MX2-32)*/
const Device UA622969 = {S41202, 149, { &P14911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Milestone X Build/4.5.1_57_MX2-34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA901909 = {S41203, 138, { &P16362,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA903044 = {S41204, 138, { &P16362,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SPH-D710 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2198145 = {S41205, 141, { &P16362,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710BST Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2391449 = {S41206, 142, { &P16362,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710VMUB Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2883658 = {S41207, 126, { &P15767,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A4400f Safari/8536.25*/
const Device UA2947245 = {S41208, 126, { &P15767,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11A4414e Safari/8536.25*/
const Device UA2970831 = {S41209, 126, { &P15767,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11A4414c Safari/8536.25*/
const Device UA3023477 = {S41210, 128, { &P15767,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/11A4435d Safari/8536.25*/
const Device UA3096317 = {S41211, 149, { &P15767,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/11A465 Safari/8536.25 YaBrowser/2.1.1364.172*/
const Device UA3108013 = {S41212, 124, { &P15767,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA3109234 = {S41213, 124, { &P15767,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A406 Safari/8536.25*/
const Device UA3115392 = {S41214, 151, { &P15767,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/11A501 Safari/8536.25 YaBrowser/2.1.1364.172*/
const Device UA3130646 = {S41215, 126, { &P15767,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA2947739 = {S41216, 90, { &P15364,&P7133,&P23655,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:20.2) Gecko/X Firefox/20.2 PaleMoon/20.2.1-x64*/
const Device UA2947738 = {S41217, 81, { &P15364,&P7133,&P23655,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:20.2) Gecko/X Firefox/20.2 PaleMoon/20.2.1*/
const Device UA901456 = {S41218, 132, { &P15876,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P5100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1954458 = {S41219, 132, { &P15876,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P5100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1972383 = {S41220, 133, { &P15876,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  GT-P5100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2402552 = {S41221, 132, { &P15876,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T779 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2610554 = {S41222, 133, { &P15876,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  GT-P5100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA1996982 = {S41223, 132, { &P20453,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; NABI2-NV7A Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029587 = {S41224, 134, { &P20453,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NABI2-NV7A Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2353723 = {S41225, 137, { &P20453,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; NABI2-NV7A-UK Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2423983 = {S41226, 137, { &P20453,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ie; NABI2-NV7A-IE Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2463953 = {S41227, 134, { &P20453,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NABI2-NV7A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2501679 = {S41228, 63, { &P20453,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; NABI2-NV7A Build/IMM76L)*/
const Device UA901543 = {S41229, 64, { &P17968,&P17979,&P24193,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:16.0) Gecko/16.0 Firefox/16.0a1*/
const Device UA909770 = {S41230, 62, { &P17968,&P17979,&P24193,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:16.0) Gecko/16.0 Firefox/16.0*/
const Device UA1995689 = {S41231, 68, { &P17968,&P17979,&P24193,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:16.0a2) Gecko/16.0a2 Firefox/16.0a2*/
const Device UA2204453 = {S41232, 68, { &P17968,&P17979,&P24193,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:16.0.1) Gecko/16.0.1 Firefox/16.0.1*/
const Device UA2236866 = {S41233, 68, { &P17968,&P17979,&P24193,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:16.0.2) Gecko/16.0.2 Firefox/16.0.2*/
const Device UA2982084 = {S41234, 64, { &P17968,&P17979,&P24193,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:16.0) Gecko/16.0 OAFirefox/16.0*/
const Device UA910752 = {S41235, 132, { &P18061,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; sdk Build/JRN83C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996203 = {S41236, 134, { &P18061,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; sdk Build/JRO03E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2222980 = {S41237, 141, { &P18061,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; google_sdk Build/JRO03E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2229200 = {S41238, 131, { &P18061,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; rk30sdk Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257953 = {S41239, 134, { &P18061,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; sdk Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2420776 = {S41240, 60, { &P18061,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; rk30sdk Build/JRO03H)*/
const Device UA2422772 = {S41241, 127, { &P18061,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; sdk Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2425779 = {S41242, 161, { &P18061,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 BaDoinkDownloader_Android_2.0*/
const Device UA2480157 = {S41243, 141, { &P18061,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; google_sdk Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2526804 = {S41244, 161, { &P18061,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 BaDoinkDownloader_Android_2.0*/
const Device UA2706043 = {S41245, 140, { &P18061,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; rk2928sdk Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2729222 = {S41246, 144, { &P18061,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; Full AOSP on Rk29sdk Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2760614 = {S41247, 144, { &P18061,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.0; en-us; Full AOSP on Rk29sdk Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2763994 = {S41248, 133, { &P18061,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; rk2928sdk Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2764987 = {S41249, 140, { &P18061,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; rk2926sdk Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2817285 = {S41250, 127, { &P18061,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; sdk Build/JRO03E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2851794 = {S41251, 62, { &P18061,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; rk2928sdk Build/JRO03H)*/
const Device UA2863825 = {S41252, 140, { &P18061,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; rk292Xsdk Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3077116 = {S41253, 138, { &P18061,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; rk30sdk Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556773 = {S41254, 91, { &P15364,&P5690,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; AOL 9.7; AOLBuild 4343.55; Windows NT 5.1; Trident/6.0)*/
const Device UA2660029 = {S41255, 64, { &P15364,&P5690,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 5.2; Trident/2.0)*/
const Device UA2714668 = {S41256, 100, { &P15364,&P5690,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; AOL 9.7; AOLBuild 4343.1028; Windows NT 5.1; WOW64; Trident/6.0)*/
const Device UA2470791 = {S41257, 76, { &P15364,&P18110,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:19.0) Gecko/X Firefox/19.0 Iceweasel/19.0*/
const Device UA2489010 = {S41258, 69, { &P15364,&P18110,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 on x86_64; rv:19.0) Gecko/X Firefox/19.0*/
const Device UA3063052 = {S41259, 129, { &P18283,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Safari/537.36*/
const Device UA3064868 = {S41260, 136, { &P18283,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065993 = {S41261, 137, { &P18283,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I535 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066001 = {S41262, 137, { &P18283,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T999 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080414 = {S41263, 137, { &P18283,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9300 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA43314 = {S41264, 25, { &P17968,&P17979,&P17470,&P18092 }};	/*Useragent - MobileZone v0.1 (Android)*/
const Device UA61870 = {S41265, 22, { &P17968,&P17979,&P17470,&P18092 }};	/*Useragent - AndroidDownloadManager*/
const Device UA88719 = {S41266, 30, { &P17968,&P17979,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android*/
const Device UA130235 = {S41267, 15, { &P17968,&P17979,&P17470,&P18092 }};	/*Useragent - Android-EAS/0.1*/
const Device UA164716 = {S41268, 11, { &P17968,&P17979,&P17470,&P18092 }};	/*Useragent - Android/0.3*/
const Device UA637275 = {S41269, 15, { &P17968,&P17979,&P17470,&P18092 }};	/*Useragent - SAMSUNG-Android*/
const Device UA2682867 = {S41270, 82, { &P17968,&P17979,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; U; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.1*/
const Device UA2713497 = {S41271, 132, { &P18465,&P18110,&P22588,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Safari/535.19 Silk-Accelerated=true*/
const Device UA896548 = {S41272, 140, { &P18185,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; YP-GI1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024158 = {S41273, 64, { &P18185,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; YP-GI1 Build/GINGERBREAD)*/
const Device UA2830287 = {S41274, 138, { &P18185,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; YP-GI1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2557763 = {S41275, 139, { &P18438,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F200S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2570057 = {S41276, 139, { &P18438,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F200K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2581996 = {S41277, 138, { &P18438,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LG-E973 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2584172 = {S41278, 146, { &P18438,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LG-E973/E97320h Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2596315 = {S41279, 139, { &P18438,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F200L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2702254 = {S41280, 60, { &P18438,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E973 Build/JZO54K)*/
const Device UA2766462 = {S41281, 61, { &P18438,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F200S Build/JZO54K)*/
const Device UA2793135 = {S41282, 61, { &P18438,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F200K Build/JZO54K)*/
const Device UA2861020 = {S41283, 61, { &P18438,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F200L Build/JZO54K)*/
const Device UA2871487 = {S41284, 143, { &P18438,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F200K/20d Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2993171 = {S41285, 145, { &P18438,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LG-E973-parrot Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2489604 = {S41286, 114, { &P13035,&P5043,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-02/06.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2560691 = {S41287, 116, { &P13035,&P5043,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-02.1/07.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA603592 = {S41288, 152, { &P14985,&P17342,&P17454,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8530/S8530DDLC2; U; Bada/2.0; en-us) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile WVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA2360583 = {S41289, 130, { &P18639,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 10 Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2390938 = {S41290, 132, { &P18639,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 10 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2405787 = {S41291, 61, { &P18639,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; Nexus 10 Build/JOP40D)*/
const Device UA2429693 = {S41292, 130, { &P18639,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 10 Build/JVP15P) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2507722 = {S41293, 132, { &P18639,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 10 Build/JOP40F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2515589 = {S41294, 131, { &P18639,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 10 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2527900 = {S41295, 59, { &P18639,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2; Nexus 10 Build/JVP15P)*/
const Device UA2532086 = {S41296, 60, { &P18639,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; Nexus 10 Build/JDQ39)*/
const Device UA2536167 = {S41297, 132, { &P18639,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 10 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2769457 = {S41298, 132, { &P18639,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 10 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2982207 = {S41299, 130, { &P18639,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 10 Build/JVP15I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA48704 = {S41300, 150, { &P17320,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; ALCATEL_one_touch_990 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48706 = {S41301, 150, { &P17320,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; ALCATEL one touch 990 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49881 = {S41302, 151, { &P17320,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ALCATEL_one_touch_990A Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59318 = {S41303, 151, { &P17320,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Alcatel one touch 990S Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70647 = {S41304, 157, { &P17320,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; Alcatel one touch 990 Orange Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90564 = {S41305, 151, { &P17320,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ALCATEL_one_touch_990S Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1895745 = {S41306, 74, { &P17320,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; ALCATEL_one_touch_990 Build/FRG83G)*/
const Device UA2020251 = {S41307, 74, { &P17320,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; ALCATEL one touch 990 Build/FRG83G)*/
const Device UA2023513 = {S41308, 75, { &P17320,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; Alcatel one touch 990S Build/FRG83G)*/
const Device UA2023895 = {S41309, 75, { &P17320,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; ALCATEL_one_touch_990A Build/FRG83G)*/
const Device UA2492647 = {S41310, 156, { &P12280,&P19826,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 Safari/7534.48.3 TeamStream (iPhone4,1 306 2.3.1)*/
const Device UA2509951 = {S41311, 156, { &P12280,&P19826,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Safari/7534.48.3 TeamStream (iPhone5,1 306 2.3.1)*/
const Device UA2510222 = {S41312, 156, { &P12280,&P19826,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/7534.48.3 TeamStream (iPhone3,3 306 2.3.1)*/
const Device UA2511952 = {S41313, 156, { &P12280,&P19826,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145 Safari/7534.48.3 TeamStream (iPhone4,1 313 2.4)*/
const Device UA2532865 = {S41314, 156, { &P12280,&P19826,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 TeamStream (iPhone4,1 313 2.4)*/
const Device UA2536100 = {S41315, 154, { &P12280,&P19826,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Safari/7534.48.3 TeamStream (iPhone3,1 313 2.4)*/
const Device UA2601989 = {S41316, 158, { &P12280,&P19826,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 TeamStream (iPhone4,1 336 2.5.1)*/
const Device UA3026891 = {S41317, 158, { &P12280,&P19826,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3 TeamStream (iPhone5,2 379 2.7.2)*/
const Device UA2781933 = {S41318, 76, { &P17595,&P22878,&P18430,&P18092 }};	/*Useragent - Safari/8536.30.1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro10%2C1)*/
const Device UA2797460 = {S41319, 70, { &P17595,&P22878,&P18430,&P18092 }};	/*Useragent - Safari/8536.30.1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (iMac12%2C2)*/
const Device UA2799670 = {S41320, 75, { &P17595,&P22878,&P18430,&P18092 }};	/*Useragent - Safari/8536.30.1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookAir5%2C2)*/
const Device UA2870367 = {S41321, 71, { &P17595,&P22878,&P18430,&P18092 }};	/*Useragent - Safari/8536.30.1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacPro3%2C1)*/
const Device UA2871062 = {S41322, 72, { &P17595,&P22878,&P18430,&P18092 }};	/*Useragent - Safari/8536.30.1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBook5%2C1)*/
const Device UA2879406 = {S41323, 72, { &P17595,&P22878,&P18430,&P18092 }};	/*Useragent - Safari/8536.30.1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (Macmini6%2C2)*/
const Device UA2434058 = {S41324, 128, { &P18239,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A510 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2459546 = {S41325, 73, { &P18239,&P18111,&P18112,&P18092 }};	/*Useragent - AndroidDownloadManager/4.1.2 (Linux; U; Android 4.1.2; A510 Build/JZO54K)*/
const Device UA2653378 = {S41326, 128, { &P18239,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; A510 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2205249 = {S41327, 123, { &P15364,&P5690,&P24194,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.7 (KHTML, like Gecko) Chrome/20.0.1099.0 Safari/536.7 QQBrowser/6.14.15493.201*/
const Device UA2205540 = {S41328, 88, { &P15364,&P5690,&P24194,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0) QQBrowser/6.14.15493.201*/
const Device UA2205971 = {S41329, 80, { &P15364,&P5690,&P24194,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) QQBrowser/6.14.15493.201*/
const Device UA2525056 = {S41330, 123, { &P15364,&P5690,&P24194,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/536.7 (KHTML, like Gecko) Chrome/20.0.1099.0 Safari/536.7 QQBrowser/6.14.15493.201*/
const Device UA2651643 = {S41331, 75, { &P15364,&P5690,&P24194,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1) QQBrowser/6.14.15493.201*/
const Device UA2834302 = {S41332, 133, { &P15364,&P5690,&P24194,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.7 (KHTML, like Gecko) Chrome/20.0.1099.0 Safari/536.7 QQBrowser/6.14.15493.201,gzip(gfe)*/
const Device UA2981949 = {S41333, 154, { &P15364,&P5690,&P24194,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QQDownload 718; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ) QQBrowser/6.14.15493.201*/
const Device UA3117807 = {S41334, 191, { &P15364,&P5690,&P24194,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Microsoft Windows Media Center PC 6.0) QQBrowser/6.14.15493.201*/
const Device UA3118010 = {S41335, 189, { &P15364,&P5690,&P24194,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; SV1; QQDownload 718; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; BRI/2; u9dnfsh) QQBrowser/6.14.14517.201*/
const Device UA88722 = {S41336, 114, { &P17346,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7C106c*/
const Device UA92935 = {S41337, 102, { &P17346,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko)*/
const Device UA93503 = {S41338, 102, { &P17346,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko)*/
const Device UA93948 = {S41339, 102, { &P17346,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko)*/
const Device UA93974 = {S41340, 114, { &P17346,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7D11*/
const Device UA96009 = {S41341, 115, { &P17346,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7C146*/
const Device UA104796 = {S41342, 113, { &P17346,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7C144*/
const Device UA121158 = {S41343, 115, { &P17346,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7C145*/
const Device UA124941 = {S41344, 114, { &P17346,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18*/
const Device UA157927 = {S41345, 100, { &P17346,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko)*/
const Device UA414893 = {S41346, 128, { &P17346,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Safari/528.18*/
const Device UA417678 = {S41347, 128, { &P17346,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7D11 Safari/528.18*/
const Device UA2145998 = {S41348, 124, { &P17346,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (whited00r iPod; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18*/
const Device UA2393042 = {S41349, 124, { &P17346,&P17137,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (whited00r iPod; U; CPU iPhone OS 3_1_3 like Mac OS X; es-es) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18*/
const Device UA2257686 = {S41350, 133, { &P18556,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NOOK BNTV600 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2712930 = {S41351, 131, { &P18556,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BNTV600 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2739715 = {S41352, 60, { &P18556,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; BNTV600 Build/IMM76L)*/
const Device UA598726 = {S41353, 121, { &P18221,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia302/5.0 (14.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA2431371 = {S41354, 156, { &P18635,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 4 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2510121 = {S41355, 156, { &P18635,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 4 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2534070 = {S41356, 155, { &P18635,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2948414 = {S41357, 162, { &P18635,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/mako*/
const Device UA3011784 = {S41358, 154, { &P18635,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/4.75; CyanogenMod-4.75) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA725237 = {S41359, 134, { &P19873,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LGL75C Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029645 = {S41360, 58, { &P19873,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LGL75C Build/GRJ22)*/
const Device UA2973471 = {S41361, 137, { &P22582,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974142 = {S41362, 144, { &P22582,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA1964504 = {S41363, 142, { &P18458,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I200 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2041775 = {S41364, 139, { &P18458,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2197816 = {S41365, 61, { &P18458,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-I200 Build/IMM76D)*/
const Device UA2505733 = {S41366, 138, { &P20226,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E975 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2599444 = {S41367, 60, { &P20226,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E975 Build/JZO54K)*/
const Device UA2667488 = {S41368, 143, { &P20226,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; LG-E975/V10b Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2713493 = {S41369, 143, { &P20226,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; LG-E975/V10c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734579 = {S41370, 143, { &P20226,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; LG-E975/V10d Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2736608 = {S41371, 160, { &P20226,&P18111,&P18112,&P18092 }};	/*Useragent - LG-E975K-AB/E975K10d Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; LG-E975K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752519 = {S41372, 145, { &P20226,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-E975-Orange Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2780617 = {S41373, 61, { &P20226,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E975K Build/JZO54K)*/
const Device UA2784411 = {S41374, 143, { &P20226,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-E975/V10f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2866182 = {S41375, 146, { &P20226,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-E975/E97510d Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2954545 = {S41376, 143, { &P20226,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-E975/V10a Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3035738 = {S41377, 143, { &P20226,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-E975/V10g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA5020 = {S41378, 87, { &P15364,&P5690,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0 (windows; U; Windows NT 5.1; zh-CN; rv:1.7.12) Gecko/20050919 Firefox/1.0.7*/
const Device UA542539 = {S41379, 117, { &P15364,&P5690,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0 (windows; U; Windows NT 5.1; zh-CN; rv:1.7.12) Gecko/X Firefox/1.0.7,gzip(gfe) (via translate.google.com)*/
const Device UA546303 = {S41380, 79, { &P15364,&P5690,&P17282,&P18092 }};	/*Useragent - Mozilla/6.0 (Windows; U; Windows NT 5.1; en-US; rv:1.7.7) Gecko/X Firefox/1.0.3*/
const Device UA757163 = {S41381, 79, { &P15364,&P5690,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0   (Windows; U; Windows NT 5.1; nl-NL; rv:1.7.5) Gecko/X Firefox/1.0*/
const Device UA869339 = {S41382, 77, { &P15364,&P5690,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-TW; rv:1.7.5) Gecko/X Firefox/1.0*/
const Device UA890733 = {S41383, 76, { &P15364,&P5690,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT5.1; en; rv:1.7.10) Gecko/X Firefox/1.0.5*/
const Device UA2484240 = {S41384, 75, { &P15364,&P5690,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; zh-TW; rv:1.7) Gecko/X Firefox/1.0*/
const Device UA3027749 = {S41385, 71, { &P15364,&P16110,&P19806,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:17.0) Gecko/X Thunderbird/17.0.8*/
const Device UA2703919 = {S41386, 146, { &P22582,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2714463 = {S41387, 68, { &P22582,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39)*/
const Device UA2783621 = {S41388, 138, { &P22582,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I337 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2895249 = {S41389, 139, { &P22582,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I337 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956545 = {S41390, 144, { &P22582,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3002796 = {S41391, 61, { &P22582,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGH-I337 Build/JDQ39E)*/
const Device UA2489910 = {S41392, 132, { &P18466,&P18110,&P19837,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Safari/535.19 Silk-Accelerated=true*/
const Device UA105724 = {S41393, 112, { &P15364,&P5690,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/523.15 (KHTML, like Gecko) Version/3.0 Safari/523.15*/
const Device UA132796 = {S41394, 115, { &P15364,&P5690,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en) AppleWebKit/522.13.1 (KHTML, like Gecko) Version/3.0.2 Safari/522.13.1*/
const Device UA139470 = {S41395, 113, { &P15364,&P5690,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en) AppleWebKit/522.11.3 (KHTML, like Gecko) Version/3.0 Safari/522.11.3*/
const Device UA142512 = {S41396, 116, { &P15364,&P5690,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/523.12.9 (KHTML, like Gecko) Version/3.0 Safari/523.12.9*/
const Device UA264952 = {S41397, 115, { &P15364,&P5690,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en) AppleWebKit/522.15.5 (KHTML, like Gecko) Version/3.0.3 Safari/522.15.5*/
const Device UA783790 = {S41398, 113, { &P15364,&P5690,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; nl) AppleWebKit/522.11.3 (KHTML, like Gecko) Version/3.0 Safari/522.11.3*/
const Device UA807953 = {S41399, 115, { &P15364,&P5690,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en) AppleWebKit/522.13.1 (KHTML, like Gecko) Version/3.0.2 Safari/522.13.1*/
const Device UA842705 = {S41400, 115, { &P15364,&P5690,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en) AppleWebKit/522.12.1 (KHTML, like Gecko) Version/3.0.1 Safari/522.12.2*/
const Device UA165655 = {S41401, 108, { &P17595,&P17294,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Safari/419.3*/
const Device UA305057 = {S41402, 106, { &P17595,&P17294,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Safari/312.6*/
const Device UA375007 = {S41403, 108, { &P17595,&P17294,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Safari/417.8*/
const Device UA389629 = {S41404, 106, { &P17595,&P17294,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Safari/412.5*/
const Device UA500089 = {S41405, 127, { &P17595,&P17294,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/525.18 (KHTML, like Gecko) Sunrise/1.7.4 like Safari/4525.22*/
const Device UA511272 = {S41406, 96, { &P17595,&P17294,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; nl-nl) AppleWebKit/533.19.4 (KHTML, like Gecko)*/
const Device UA518529 = {S41407, 104, { &P17595,&P17294,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Safari/412*/
const Device UA756617 = {S41408, 107, { &P17595,&P17294,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Safari/416.12*/
const Device UA757122 = {S41409, 106, { &P17595,&P17294,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Safari/412.2*/
const Device UA873114 = {S41410, 108, { &P17595,&P17294,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Safari/417.9.2*/
const Device UA2137958 = {S41411, 124, { &P17595,&P17294,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/(null) Safari/533.19.4*/
const Device UA2208249 = {S41412, 44, { &P17595,&P17294,&P18108,&P18092 }};	/*Useragent - Mozilla/4.0 (Mac OS X 10.4.11) Java/1.5.0_19*/
const Device UA2313649 = {S41413, 116, { &P17595,&P17294,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) RealPlayer/12.0.0.1725*/
const Device UA2774728 = {S41414, 117, { &P17595,&P17294,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) RealPlayer/10.1.0.412*/
const Device UA2815698 = {S41415, 115, { &P17595,&P17294,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) RealPlayer/10.1.0.503*/
const Device UA2978138 = {S41416, 115, { &P17595,&P17294,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) RealPlayer/10.0.0.331*/
const Device UA2005268 = {S41417, 155, { &P14877,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9000 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031108 = {S41418, 172, { &P14877,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9000 Build/JRO03H; CyanogenMod-TAJB//MKalter\\V7.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2199446 = {S41419, 155, { &P14877,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9000 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2361652 = {S41420, 159, { &P14877,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9000 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747169 = {S41421, 160, { &P14877,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; GT-I9000B Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422697 = {S41422, 139, { &P18316,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F160K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423715 = {S41423, 139, { &P18316,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F160L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2424092 = {S41424, 139, { &P18316,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F160S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2487344 = {S41425, 140, { &P18316,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F160LV Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2527445 = {S41426, 61, { &P18316,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F160L Build/JZO54K)*/
const Device UA2555357 = {S41427, 61, { &P18316,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F160K Build/JZO54K)*/
const Device UA2698627 = {S41428, 61, { &P18316,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F160S Build/JZO54K)*/
const Device UA2707488 = {S41429, 148, { &P18316,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F160K/F160K20f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734368 = {S41430, 142, { &P18316,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; LG-F160L/LV Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2764398 = {S41431, 141, { &P18316,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; LG-F160S/K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2769730 = {S41432, 148, { &P18316,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F160K/F160K20g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2846977 = {S41433, 62, { &P18316,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F160LV Build/JZO54K)*/
const Device UA3002959 = {S41434, 138, { &P18316,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; LG-F160 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3020348 = {S41435, 148, { &P18316,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F160K/F160K20h Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3026971 = {S41436, 141, { &P12280,&P18117,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9B176 Safari/7534.48.3*/
const Device UA3027076 = {S41437, 143, { &P12280,&P18117,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9B206 Safari/7534.48.3*/
const Device UA3029282 = {S41438, 143, { &P12280,&P18117,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9B208 Safari/7534.48.3*/
const Device UA3029509 = {S41439, 141, { &P12280,&P18117,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9B179 Safari/7534.48.3*/
const Device UA152242 = {S41440, 45, { &P15364,&P18110,&P18407,&P18092 }};	/*Useragent - Opera/9.70 (Linux ppc64 ; U; en) Presto/2.2.1*/
const Device UA645229 = {S41441, 62, { &P15364,&P18110,&P18407,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Linux i686 ; en) Opera 9.70*/
const Device UA707252 = {S41442, 47, { &P15364,&P18110,&P18407,&P18092 }};	/*Useragent - Opera/9.70 (Linux i686 ; U; zh-cn) Presto/2.2.0*/
const Device UA760367 = {S41443, 126, { &P15364,&P18110,&P18407,&P18092 }};	/*Useragent - Opera/9.70 (Linux armv6l ; U; CE-HTML/1.0 (<profilelist><ui_profile name="PHILIPS_OLS_2010"/></profilelist>); en) Presto/2.2.1*/
const Device UA902552 = {S41444, 76, { &P15364,&P18110,&P18407,&P18092 }};	/*Useragent - Opera/9.70 (Linux armv7l ; U; HomeManagerV1.5/TSV3.1/1.09M; en) Presto/2.2.1*/
const Device UA1854381 = {S41445, 75, { &P15364,&P18110,&P18407,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux i686 ; U; en; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 9.70*/
const Device UA2008445 = {S41446, 72, { &P15364,&P18110,&P18407,&P18092 }};	/*Useragent - Opera/9.70 (Linux armv6l ; U; HomeManager/HSV5.0/1.15C; en) Presto/2.2.1*/
const Device UA2262433 = {S41447, 50, { &P15364,&P18110,&P18407,&P18092 }};	/*Useragent - Opera/9.70 (Linux 7405d0-smp ; U; en) Presto/2.2.0*/
const Device UA2327132 = {S41448, 126, { &P15364,&P18110,&P18407,&P18092 }};	/*Useragent - Opera/9.70 (Linux x86_64 ; U; CE-HTML/1.0 (<profilelist><ui_profile name="PHILIPS_OLS_2008"/></profilelist>); en) Presto/2.2.1*/
const Device UA3064867 = {S41449, 130, { &P18437,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3064894 = {S41450, 130, { &P18437,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8020 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3064975 = {S41451, 130, { &P18437,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8020 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3065474 = {S41452, 130, { &P18437,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080753 = {S41453, 130, { &P18437,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8005 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA924469 = {S41454, 71, { &P15364,&P18110,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 on x86_64; rv:14.0) Gecko/X Firefox/14.0.1*/
const Device UA925473 = {S41455, 78, { &P15364,&P18110,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:14.0) Gecko/X Firefox/14.0.1 Iceweasel/14.0.1*/
const Device UA2168068 = {S41456, 76, { &P15364,&P18110,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:14.0) Gecko/X Firefox/14.0.1 Firefox/14.0.1*/
const Device UA2481384 = {S41457, 100, { &P15364,&P18110,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Ultimate Edition; X11; Linux x86; rv:8.0) Gecko/X Ultimate Edition / 3.5 Firefox/14.0.1*/
const Device UA33074 = {S41458, 82, { &P14912,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C3222W/C3222WJVKC2 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62723 = {S41459, 73, { &P14912,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C3222/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2002858 = {S41460, 191, { &P14983,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.3 NokiaC7-00/111.040.1511; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.1.4 Mobile Safari/535.1 3gpp-gba*/
const Device UA2348310 = {S41461, 192, { &P14983,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.3 NokiaC7-00s/111.040.1511; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.1.4 Mobile Safari/535.1 3gpp-gba*/
const Device UA49839 = {S41462, 131, { &P15263,&P6374,&P4418,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9850; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.261 Mobile Safari/534.11+*/
const Device UA2238338 = {S41463, 139, { &P18280,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2413850 = {S41464, 139, { &P18280,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2421524 = {S41465, 155, { &P18280,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Special Edition Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2694118 = {S41466, 139, { &P18280,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC VLE_U Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA346965 = {S41467, 152, { &P14114,&P5629,&P18166,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Silk/1.1.0-84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Silk-Accelerated=false*/
const Device UA615349 = {S41468, 155, { &P14114,&P5629,&P18166,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Silk/1.0.22.79_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Silk-Accelerated=false*/
const Device UA641214 = {S41469, 152, { &P14114,&P5629,&P18166,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Silk/1.1.0-80) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Silk-Accelerated=false*/
const Device UA658271 = {S41470, 156, { &P14114,&P5629,&P18166,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Silk/1.0.13.328_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Silk-Accelerated=false*/
const Device UA2408431 = {S41471, 156, { &P14114,&P5629,&P18166,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Silk/1.0.22.153_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Silk-Accelerated=false*/
const Device UA2566971 = {S41472, 131, { &P19861,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-I605 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567712 = {S41473, 138, { &P19861,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2571157 = {S41474, 138, { &P19861,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-I605 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2706129 = {S41475, 131, { &P19861,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566452 = {S41476, 138, { &P15575,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566957 = {S41477, 131, { &P15575,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567241 = {S41478, 139, { &P15575,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567250 = {S41479, 139, { &P15575,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100M Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567494 = {S41480, 139, { &P15575,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2604718 = {S41481, 138, { &P15575,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9100 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667967 = {S41482, 132, { &P15575,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100M Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2706736 = {S41483, 132, { &P15575,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2779961 = {S41484, 132, { &P15575,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2352887 = {S41485, 133, { &P18554,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NOOK BNTV400 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2712913 = {S41486, 131, { &P18554,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BNTV400 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2740175 = {S41487, 60, { &P18554,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; BNTV400 Build/IMM76L)*/
const Device UA626025 = {S41488, 115, { &P15228,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-03/06.51; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.62.10*/
const Device UA2230726 = {S41489, 114, { &P15228,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-03/07.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6*/
const Device UA2001674 = {S41490, 139, { &P12968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26i Build/6.1.A.2.45) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2031595 = {S41491, 135, { &P12968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LT26i Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2822109 = {S41492, 140, { &P12968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  LT26i Build/6.1.A.2.45) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA316195 = {S41493, 136, { &P17863,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SCH-I400 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA421688 = {S41494, 136, { &P17863,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SCH-I400 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA454655 = {S41495, 60, { &P17863,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; SCH-I400 Build/FROYO)*/
const Device UA1954005 = {S41496, 135, { &P17863,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SCH-I400Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024776 = {S41497, 135, { &P17863,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.2; en-us; SCH-I400 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031814 = {S41498, 135, { &P17863,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.2; en-us; SCH-I400 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033027 = {S41499, 135, { &P17863,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;SCH-I400 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2534213 = {S41500, 62, { &P17976,&P17979,&P23653,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:22.0) Gecko/22.0 Firefox/22.0*/
const Device UA2767099 = {S41501, 68, { &P17976,&P17979,&P23653,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:22.0a2) Gecko/22.0a2 Firefox/22.0a2*/
const Device UA2879405 = {S41502, 72, { &P17976,&P17979,&P23653,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:22.0) Gecko/22.0 Firefox/22.0,gzip(gfe)*/
const Device UA2997916 = {S41503, 101, { &P17976,&P17979,&P23653,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:22.0) Gecko/22.0 Firefox/22.0,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA3130869 = {S41504, 127, { &P18385,&P21637,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus 7 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3132981 = {S41505, 129, { &P18385,&P21637,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3.1; Nexus 7 Build/JLS36I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA2401912 = {S41506, 138, { &P18447,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xperia T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2490597 = {S41507, 140, { &P18447,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.0.489) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2973951 = {S41508, 138, { &P18295,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974616 = {S41509, 134, { &P18295,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; One X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976011 = {S41510, 138, { &P18295,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EndeavorU Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022712 = {S41511, 138, { &P18295,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3058369 = {S41512, 138, { &P18295,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; HTC One X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA73462 = {S41513, 147, { &P17172,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA236488 = {S41514, 139, { &P17172,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA246453 = {S41515, 140, { &P17172,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA248217 = {S41516, 69, { &P17172,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; Transformer TF101 Build/HTK75)*/
const Device UA330416 = {S41517, 68, { &P17172,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; Transformer TF101 Build/HTJ85B)*/
const Device UA701648 = {S41518, 157, { &P17172,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1 KRAKD by jcarrz1; en-us; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2024524 = {S41519, 134, { &P17172,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; ja-jp; TF101-WiMAX Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA3064767 = {S41520, 137, { &P18595,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065590 = {S41521, 137, { &P18595,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2405702 = {S41522, 151, { &P18505,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2428085 = {S41523, 150, { &P18505,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MB886 Build/9.8.2Q-8_MB886_FFW-11) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA159755 = {S41524, 113, { &P15364,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.215 Safari/534.10*/
const Device UA168658 = {S41525, 111, { &P15364,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10*/
const Device UA185182 = {S41526, 106, { &P15364,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; en-US) AppleWebKit/533.1 (KHTML, like Gecko) Chrome/8.0 Safari/533.1*/
const Device UA258239 = {S41527, 156, { &P15364,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Jolicloud Linux i686; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Jolicloud/1.1 Chromium/8.0.552.237 Chrome/8.0.552.237 Safari/534.10*/
const Device UA437226 = {S41528, 126, { &P15364,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; en-us; EPAD Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA509543 = {S41529, 120, { &P15364,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10*/
const Device UA767765 = {S41530, 126, { &P15364,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; en-gb; EPAD Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA880228 = {S41531, 111, { &P15364,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; ru-RU) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10*/
const Device UA2305840 = {S41532, 126, { &P15364,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; en-us; A500 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA3117602 = {S41533, 130, { &P15364,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; en-us; ETBW11AA Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA2672405 = {S41534, 159, { &P21350,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ie; SAMSUNG GT-I8730/I8730XWAMB8 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2710087 = {S41535, 158, { &P21350,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-I8730-ORANGE/I8730XWAMC2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2764969 = {S41536, 139, { &P21350,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-I8730 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2767239 = {S41537, 61, { &P21350,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8730 Build/JZO54K)*/
const Device UA2783714 = {S41538, 140, { &P21350,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-I8730T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3004786 = {S41539, 62, { &P21350,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8730T Build/JZO54K)*/
const Device UA3016528 = {S41540, 158, { &P21350,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-I8730-ORANGE/I8730XWAMF1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022029 = {S41541, 159, { &P21350,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I8730/I8730XWAMG1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050516 = {S41542, 159, { &P21350,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SAMSUNG GT-I8730/I8730XWAMD7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058181 = {S41543, 159, { &P21350,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; SAMSUNG GT-I8730/I8730XWAMD2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3111747 = {S41544, 159, { &P21350,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ie; SAMSUNG GT-I8730/I8730XWAMG6 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029620 = {S41545, 95, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - HTCS720t_TD/1.0 Android/4.0 release/2012 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2413660 = {S41546, 148, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 MSM7627A/1.0 Linux/3.0.21 Android/4.0.4 Release/11.16.2011 Browser/AppleWebKit533.1 Profile/MIDP-1.0 Configuration/CLDC-1.0 Mobile/4.0.4*/
const Device UA2418488 = {S41547, 135, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - 9220 Linux/2.6.35 Android/4.0.3 Release/08.24.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1*/
const Device UA2466304 = {S41548, 91, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - CoolPad8190_CMCC_TD/1.0 Linux/3.0.8 Android/4.0 Release/10.15.2012 Browser/AppleWebkit534.3*/
const Device UA2483453 = {S41549, 123, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - Z15A-Z15A/1.0 Linux/2.6.35.7 Android/4.0 Release/10.31.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2596217 = {S41550, 134, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-C3100/1.0 Linux/2.6.35.7 Android/android 4.0.3 Release/12.25.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2596296 = {S41551, 125, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-S560/1.0 Linux/2.6.35.7 Android/4.0.3 Release/11.16.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2601689 = {S41552, 134, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-C3100/1.0 Linux/2.6.35.7 Android/android 4.0.3 Release/12.27.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2604549 = {S41553, 123, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-M2/1.0 Linux/2.6.35.7 Android/4.0.4 Release/01.28.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2659133 = {S41554, 123, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - Z15A-Z15A/1.0 Linux/2.6.35.7 Android/4.0 Release/12.26.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2660864 = {S41555, 136, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/4.0.4 Release/12.18.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2667235 = {S41556, 136, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/4.0.2 Release/02.21.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2708896 = {S41557, 134, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-C6200/1.0 Linux/2.6.35.7 Android/android 4.0.3 Release/01.09.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2735357 = {S41558, 136, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/4.0.4 Release/02.22.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2736571 = {S41559, 124, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-M6A/1.0 Linux/2.6.35.7 Android/4.0.3 Release/03.21.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2737995 = {S41560, 136, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/4.0.2 Release/03.23.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2746815 = {S41561, 111, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - Lenovo-A630t_TD/S100 Linux/3.0.13 Android/4.0 Release/10.08.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30*/
const Device UA2747640 = {S41562, 85, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - ZTEU880F1_TD/1.0 Linux/3.0.13 Android/4.0 Release/7.10.2012 Browser/AppleWebKit534.30*/
const Device UA2764982 = {S41563, 84, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - ZTEU817_TD/1.0 Linux/3.0.13 Android/4.0 Release/11.20.2012 Browser/AppleWebKit534.30*/
const Device UA2768465 = {S41564, 91, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - CoolPad8720_CMCC_TD/1.0 Linux/3.0.8 Android/4.0 Release/03.31.2013 Browser/AppleWebkit534.3*/
const Device UA2768501 = {S41565, 125, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - MTK-SKING/1.0 Linux/2.6.35.7 Android/4.0.3 Release/03.27.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2790844 = {S41566, 85, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - ZTEU960s3_TD/1.0 Linux/2.6.39 Android/4.0 Release/6.10.2012 Browser/AppleWebKit534.30*/
const Device UA2829921 = {S41567, 84, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - HuaweiT8808D_TD/1.0 Android/4.0 Release/02.01.2012 Browser/WAP2.0 appleWebkit/534.30*/
const Device UA2851106 = {S41568, 113, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - HS-T950_TD/1.0 Android/4.0.3 Release/28.10.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2864527 = {S41569, 129, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-GT-N9300/1.0 Linux/2.6.35.7 Android/4.0.3 Release/03.26.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2990140 = {S41570, 85, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - ZTE U880F_TD/1.0 Linux/2.6.39 Android/4.0 Release/5.24.2012 Browser/AppleWebKit534.30*/
const Device UA3071831 = {S41571, 113, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - HS-T909_TD/1.0 Android/4.0.3 Release/28.10.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA3103200 = {S41572, 127, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - HUAWEI_G606-T00_TD/1.0 Linux/3.0.8 Android/4.0.3 Release/01.06.2013 Browser/Webkit54731 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA3107238 = {S41573, 118, { &P15364,&P17979,&P18062,&P18092 }};	/*Useragent - ZTE U808_TD/1.0 Linux/3.0.8 Android/4.0 Release/3.6.2013 Browser/AppleWebKit533.1(KHTML,like Gecko) Mozilla/5.0 Mobile*/
const Device UA2387887 = {S41574, 62, { &P17976,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA2531340 = {S41575, 68, { &P17976,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:20.0a2) Gecko/20.0a2 Firefox/20.0a2*/
const Device UA2657720 = {S41576, 72, { &P17976,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:20.0) Gecko/20.0 Firefox/20.0,gzip(gfe)*/
const Device UA2679840 = {S41577, 68, { &P17976,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:20.0.1) Gecko/20.0.1 Firefox/20.0.1*/
const Device UA2799073 = {S41578, 85, { &P17595,&P21721,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.43.7 (KHTML, like Gecko)*/
const Device UA2870016 = {S41579, 56, { &P17595,&P21721,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.4 rv:183 (Macintosh; Mac OS X 10.9.0; en_US)*/
const Device UA2873841 = {S41580, 56, { &P17595,&P21721,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.3 rv:176 (Macintosh; Mac OS X 10.9.0; en_US)*/
const Device UA2882475 = {S41581, 56, { &P17595,&P21721,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.4 rv:184 (Macintosh; Mac OS X 10.9.0; en_US)*/
const Device UA2938571 = {S41582, 56, { &P17595,&P21721,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.4 rv:183 (Macintosh; Mac OS X 10.9.0; en_GB)*/
const Device UA2972380 = {S41583, 56, { &P17595,&P21721,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.5 rv:189 (Macintosh; Mac OS X 10.9.0; en_US)*/
const Device UA2975529 = {S41584, 106, { &P17595,&P21721,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.51.3 (KHTML, like Gecko) Version/1.0 Safari/1*/
const Device UA3022682 = {S41585, 99, { &P17595,&P21721,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.51.3 (KHTML, like Gecko) QuickLook/5.0*/
const Device UA3031020 = {S41586, 56, { &P17595,&P21721,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.5 rv:192 (Macintosh; Mac OS X 10.9.0; en_GB)*/
const Device UA3036007 = {S41587, 56, { &P17595,&P21721,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.5 rv:192 (Macintosh; Mac OS X 10.9.0; en_US)*/
const Device UA3036752 = {S41588, 56, { &P17595,&P21721,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.6 rv:194 (Macintosh; Mac OS X 10.9.0; en_US)*/
const Device UA3058876 = {S41589, 56, { &P17595,&P21721,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.6 rv:193 (Macintosh; Mac OS X 10.9.0; en_US)*/
const Device UA3078882 = {S41590, 56, { &P17595,&P21721,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.5 rv:192 (Macintosh; Mac OS X 10.9.0; en_AU)*/
const Device UA3094704 = {S41591, 54, { &P17595,&P21721,&P18108,&P18092 }};	/*Useragent - Airmail 1.1 rv:201 (Macintosh; Mac OS X 10.9.0; en_US)*/
const Device UA3112842 = {S41592, 96, { &P17595,&P21721,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.71 (KHTML, like Gecko) Sparrow/1178*/
const Device UA3125539 = {S41593, 56, { &P17595,&P21721,&P18108,&P18092 }};	/*Useragent - Airmail 1.0.5 rv:192 (Macintosh; Mac OS X 10.9.0; de_DE)*/
const Device UA2262368 = {S41594, 85, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - ThumbnailService/39600.038 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookAir4%2C2)*/
const Device UA2262559 = {S41595, 80, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - ThumbnailService/39600.038 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (iMac12%2C2)*/
const Device UA2348791 = {S41596, 70, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - NewsFire/84 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookPro5%2C5)*/
const Device UA2367536 = {S41597, 73, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - Socialite/7825 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookPro8%2C2)*/
const Device UA2368403 = {S41598, 89, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (MacBookPro9%2C2)*/
const Device UA2382243 = {S41599, 79, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - CoverScout%203/3.4.3 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookPro8%2C1)*/
const Device UA2387431 = {S41600, 86, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - ThumbnailService/39600.038 CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (MacBookPro10%2C1)*/
const Device UA2400891 = {S41601, 70, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - NewsFire/84 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookAir4%2C2)*/
const Device UA2421157 = {S41602, 74, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - CoverScout%203/3.5.6 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (iMac11%2C3)*/
const Device UA2424555 = {S41603, 80, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - WebProcess/8536.26.17 CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (MacBookPro9%2C2)*/
const Device UA2426167 = {S41604, 89, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (MacBookAir5%2C2)*/
const Device UA2457283 = {S41605, 80, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - WebProcess/8536.26.17 CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (MacBookAir5%2C2)*/
const Device UA2458014 = {S41606, 75, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - WebProcess/8536.26.17 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (iMac12%2C1)*/
const Device UA2466656 = {S41607, 77, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - WebProcess/8536.26.17 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBook5%2C1)*/
const Device UA2479617 = {S41608, 87, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader/1725 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookPro8%2C1)*/
const Device UA2479978 = {S41609, 82, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - ThumbnailService/39600.038 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBook5%2C1)*/
const Device UA2481434 = {S41610, 75, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - Email%20Marketing/4.0 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (iMac12%2C1)*/
const Device UA2483588 = {S41611, 84, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (iMac13%2C1)*/
const Device UA2525988 = {S41612, 81, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - ThumbnailService/39600.038 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacPro3%2C1)*/
const Device UA2527886 = {S41613, 77, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - Caffeinated/1.3.2 CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (MacBookPro10%2C1)*/
const Device UA2552661 = {S41614, 82, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - ThumbnailService/39600.038 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (Macmini5%2C1)*/
const Device UA2556788 = {S41615, 82, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - Cookie%20Stumbler/1631 CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (MacBookPro10%2C1)*/
const Device UA2691875 = {S41616, 92, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - XProtectUpdater (unknown version) CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookPro8%2C1)*/
const Device UA2835023 = {S41617, 86, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBook5%2C1)*/
const Device UA2844159 = {S41618, 76, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - Caffeinated/1.3.2 CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (MacBookAir5%2C2)*/
const Device UA2983285 = {S41619, 86, { &P17595,&P19835,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (Macmini6%2C1)*/
const Device UA2703793 = {S41620, 145, { &P22582,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2715000 = {S41621, 138, { &P22582,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA3033012 = {S41622, 64, { &P15364,&P17285,&P19806,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:17.0) Gecko/X Thunderbird/17.0.8*/
const Device UA2568320 = {S41623, 144, { &P12280,&P18117,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9B206 Safari/7534.48.3*/
const Device UA2568373 = {S41624, 144, { &P12280,&P18117,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9B208 Safari/7534.48.3*/
const Device UA2569094 = {S41625, 142, { &P12280,&P18117,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9B179 Safari/7534.48.3*/
const Device UA2569382 = {S41626, 142, { &P12280,&P18117,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9B176 Safari/7534.48.3*/
const Device UA2223255 = {S41627, 128, { &P18248,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A700 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2402025 = {S41628, 57, { &P18248,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; A700 Build/JRO03H)*/
const Device UA2540134 = {S41629, 76, { &P15364,&P18110,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:20.0) Gecko/X Firefox/20.0 Iceweasel/20.0*/
const Device UA2565873 = {S41630, 69, { &P15364,&P18110,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 on x86_64; rv:20.0) Gecko/X Firefox/20.0*/
const Device UA2998337 = {S41631, 100, { &P15364,&P18110,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Ultimate Edition; X11; Linux x64; rv:8.0) Gecko/X Ultimate Edition / 3.4.1 Firefox/20.0*/
const Device UA366742 = {S41632, 147, { &P18114,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Transformer Prime TF201 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA644844 = {S41633, 76, { &P18114,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Transformer Prime TF201 Build/IML74K)*/
const Device UA657170 = {S41634, 129, { &P18114,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; TF201 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA888195 = {S41635, 147, { &P18114,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Transformer Prime TF201 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1945779 = {S41636, 154, { &P18114,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Transformer Prime TF201 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028218 = {S41637, 145, { &P18114,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; Transformer Prime TF201 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA224673 = {S41638, 131, { &P15364,&P17818,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/11.04 Chromium/14.0.825.0 Chrome/14.0.825.0 Safari/535.1*/
const Device UA250319 = {S41639, 137, { &P15364,&P17818,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/11.04 Chromium/14.0.835.202 Chrome/14.0.835.202 Safari/535.1*/
const Device UA1903498 = {S41640, 172, { &P15364,&P17818,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/11.04 Chromium/14.0.835.202 Chrome/14.0.835.202 Safari/535.1,gzip(gfe) (via translate.google.com)*/
const Device UA2374023 = {S41641, 131, { &P15364,&P17818,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML	 like Gecko) Ubuntu/11.04 Chromium/14.0.803.0 Chrome/14.0.803.0 Safari/535.1*/
const Device UA459249 = {S41642, 73, { &P15988,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-C3312/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2654039 = {S41643, 138, { &P15988,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-C3312/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2798012 = {S41644, 162, { &P15988,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-C3312/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (J2ME/MIDP; Opera Mini/6.5.27510/30.3214; U; en) Presto/2.8.119 Version/11.10*/
const Device UA137414 = {S41645, 62, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.2.15 Version/10.10*/
const Device UA306219 = {S41646, 85, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03797); ru) Presto/2.2.15 Version/10.10*/
const Device UA374785 = {S41647, 85, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03796); ru) Presto/2.2.15 Version/10.10*/
const Device UA443698 = {S41648, 85, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.7 (build 03686); ru) Presto/2.2.15 Version/10.10*/
const Device UA574284 = {S41649, 84, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4848); ru) Presto/2.2.15 Version/10.10*/
const Device UA801309 = {S41650, 84, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4876); ru) Presto/2.2.15 Version/10.10*/
const Device UA835014 = {S41651, 66, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; ru) Opera 10.10*/
const Device UA852154 = {S41652, 84, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.9 (build 4953); ru) Presto/2.2.15 Version/10.10*/
const Device UA1894417 = {S41653, 84, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4598); ru) Presto/2.2.15 Version/10.10*/
const Device UA1957062 = {S41654, 85, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5310); ru) Presto/2.2.15 Version/10.10*/
const Device UA2154591 = {S41655, 85, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5339); ru) Presto/2.2.15 Version/10.10*/
const Device UA2262768 = {S41656, 84, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4664); ru) Presto/2.2.15 Version/10.10*/
const Device UA2262783 = {S41657, 84, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4157); ru) Presto/2.2.15 Version/10.10*/
const Device UA2285643 = {S41658, 84, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.8 (build 4133); ru) Presto/2.2.15 Version/10.10*/
const Device UA2324186 = {S41659, 85, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 5.10 (build 5244); ru) Presto/2.2.15 Version/10.10*/
const Device UA2366351 = {S41660, 84, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5976); ru) Presto/2.2.15 Version/10.10*/
const Device UA2383645 = {S41661, 84, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5972); ru) Presto/2.2.15 Version/10.10*/
const Device UA2398722 = {S41662, 84, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5970); ru) Presto/2.2.15 Version/10.10*/
const Device UA2442566 = {S41663, 84, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5680); ru) Presto/2.2.15 Version/10.10*/
const Device UA2497253 = {S41664, 84, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 5831); ru) Presto/2.2.15 Version/10.10*/
const Device UA2696507 = {S41665, 84, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6005); ru) Presto/2.2.15 Version/10.10*/
const Device UA2697341 = {S41666, 84, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6015); ru) Presto/2.2.15 Version/10.10*/
const Device UA3043324 = {S41667, 84, { &P15364,&P7133,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; MRA 6.0 (build 6001); ru) Presto/2.2.15 Version/10.10*/
const Device UA2213020 = {S41668, 5, { &P18254,&P7480,&P18589,&P18090 }};	/*Useragent - Slurp*/
const Device UA2738806 = {S41669, 80, { &P18254,&P7480,&P18589,&P18090 }};	/*Useragent - *Mozilla/5.0 (compatible; Yahoo! SlurpChina; http://misc.yahoo.com.cn/help.html)*/
const Device UA2458614 = {S41670, 67, { &P15364,&P7133,&P20456,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:18.0) Gecko/X SeaMonkey/2.15*/
const Device UA2515433 = {S41671, 82, { &P15364,&P7133,&P20456,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:18.0) Gecko/X Firefox/18.0 SeaMonkey/2.15.2*/
const Device UA2755126 = {S41672, 142, { &P22394,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6603 Build/10.1.1.A.1.253) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3468 = {S41673, 105, { &P11653,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-F480/1.0 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA20518 = {S41674, 106, { &P11653,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-F480T/1.0 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA20576 = {S41675, 74, { &P11653,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-F480L/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53368 = {S41676, 118, { &P11653,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-F480-ORANGE/F480ABIB1 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA63807 = {S41677, 120, { &P11653,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-F480-Vodafone/F480BUHK1 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA2706403 = {S41678, 137, { &P22594,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2715710 = {S41679, 130, { &P22594,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA230577 = {S41680, 134, { &P17715,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LGL45C Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA656649 = {S41681, 58, { &P17715,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LGL45C Build/GRJ22)*/
const Device UA89072 = {S41682, 113, { &P16941,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.3.1; U; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Pixi/1.0*/
const Device UA89706 = {S41683, 115, { &P16941,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.3.5.1; U; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Pixi/1.1*/
const Device UA100825 = {S41684, 107, { &P16941,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.0; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1*/
const Device UA125552 = {S41685, 107, { &P16941,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.0*/
const Device UA130269 = {S41686, 107, { &P16941,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1*/
const Device UA141816 = {S41687, 107, { &P16941,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; de-DE) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1*/
const Device UA145271 = {S41688, 107, { &P16941,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; en-GB) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1*/
const Device UA202354 = {S41689, 109, { &P16941,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5.1; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1*/
const Device UA207089 = {S41690, 109, { &P16941,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5.1; U; es-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1*/
const Device UA262529 = {S41691, 109, { &P16941,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.1.1; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1*/
const Device UA281714 = {S41692, 107, { &P16941,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; es-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.0*/
const Device UA284887 = {S41693, 107, { &P16941,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; fr-FR) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1*/
const Device UA286740 = {S41694, 107, { &P16941,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; es-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1*/
const Device UA506986 = {S41695, 107, { &P16941,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; es-ES) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1*/
const Device UA748310 = {S41696, 107, { &P16941,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.3; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1*/
const Device UA811062 = {S41697, 115, { &P16941,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.2.9.1; U; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Pixi/1.0*/
const Device UA2285966 = {S41698, 107, { &P16941,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.0; U; es-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1*/
const Device UA2299571 = {S41699, 108, { &P16941,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0(webOS/1.4.5.1; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pixi/1.1*/
const Device UA2784929 = {S41700, 41, { &P17968,&P17979,&P23030,&P18092 }};	/*Useragent - Dolphin http client/10.0.0(222) (Android)*/
const Device UA2800313 = {S41701, 41, { &P17968,&P17979,&P23030,&P18092 }};	/*Useragent - Dolphin http client/10.0.0(223) (Android)*/
const Device UA2875641 = {S41702, 41, { &P17968,&P17979,&P23030,&P18092 }};	/*Useragent - Dolphin http client/10.0.0(224) (Android)*/
const Device UA2876989 = {S41703, 41, { &P17968,&P17979,&P23030,&P18092 }};	/*Useragent - Dolphin http client/10.0.0(225) (Android)*/
const Device UA2947396 = {S41704, 45, { &P17968,&P17979,&P23030,&P18092 }};	/*Useragent - Dolphin http client/10.0.1 INT(228) (Android)*/
const Device UA2963964 = {S41705, 41, { &P17968,&P17979,&P23030,&P18092 }};	/*Useragent - Dolphin http client/10.0.0(226) (Android)*/
const Device UA2973593 = {S41706, 47, { &P17968,&P17979,&P23030,&P18092 }};	/*Useragent - Dolphin http client/10.0.1 Beta2(229) (Android)*/
const Device UA2974689 = {S41707, 41, { &P17968,&P17979,&P23030,&P18092 }};	/*Useragent - Dolphin http client/10.0.1(230) (Android)*/
const Device UA3024035 = {S41708, 41, { &P17968,&P17979,&P23030,&P18092 }};	/*Useragent - Dolphin http client/10.0.1(231) (Android)*/
const Device UA3027643 = {S41709, 47, { &P17968,&P17979,&P23030,&P18092 }};	/*Useragent - Dolphin http client/10.0.2 beta1(232) (Android)*/
const Device UA3031941 = {S41710, 47, { &P17968,&P17979,&P23030,&P18092 }};	/*Useragent - Dolphin http client/10.0.2 beta2(233) (Android)*/
const Device UA3061130 = {S41711, 46, { &P17968,&P17979,&P23030,&P18092 }};	/*Useragent - Dolphin http client/10.0.3 Beta(235) (Android)*/
const Device UA3062642 = {S41712, 47, { &P17968,&P17979,&P23030,&P18092 }};	/*Useragent - Dolphin http client/10.0.3 Beta2(236) (Android)*/
const Device UA3066917 = {S41713, 41, { &P17968,&P17979,&P23030,&P18092 }};	/*Useragent - Dolphin http client/10.0.3(238) (Android)*/
const Device UA2537061 = {S41714, 132, { &P18466,&P18110,&P21167,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Safari/535.19 Silk-Accelerated=true*/
const Device UA90787 = {S41715, 125, { &P15869,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA223144 = {S41716, 131, { &P15869,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; MZ505 Build/Moto_MZ505) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402615 = {S41717, 126, { &P15869,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMI87C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402616 = {S41718, 125, { &P15869,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ01) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402618 = {S41719, 125, { &P15869,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ13) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402620 = {S41720, 125, { &P15869,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ19) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402624 = {S41721, 125, { &P15869,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ27) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402630 = {S41722, 126, { &P15869,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMI80B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402649 = {S41723, 125, { &P15869,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402650 = {S41724, 125, { &P15869,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ29) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402651 = {S41725, 126, { &P15869,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ28B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402661 = {S41726, 126, { &P15869,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ04B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402673 = {S41727, 126, { &P15869,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMI84D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402675 = {S41728, 126, { &P15869,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ07B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402686 = {S41729, 123, { &P15869,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; ru-; Xoom Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402763 = {S41730, 125, { &P15869,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ33) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402780 = {S41731, 125, { &P15869,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMI73) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA403279 = {S41732, 125, { &P15869,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMI90) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA776491 = {S41733, 60, { &P15869,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; MZ505 Build/Moto_MZ505)*/
const Device UA62832 = {S41734, 92, { &P14454,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE66/1.0 (410.21.010) SymbianOS/9.2 Series60/3.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64669 = {S41735, 155, { &P14454,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE66-1/500.21.009; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA69530 = {S41736, 8, { &P14454,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE66*/
const Device UA2354720 = {S41737, 62, { &P17976,&P17979,&P23216,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:19.0) Gecko/19.0 Firefox/19.0*/
const Device UA2424482 = {S41738, 68, { &P17976,&P17979,&P23216,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:19.0a2) Gecko/19.0a2 Firefox/19.0a2*/
const Device UA2529374 = {S41739, 72, { &P17976,&P17979,&P23216,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:19.0) Gecko/19.0 Firefox/19.0,gzip(gfe)*/
const Device UA2570873 = {S41740, 68, { &P17976,&P17979,&P23216,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:19.0.2) Gecko/19.0.2 Firefox/19.0.2*/
const Device UA2733300 = {S41741, 68, { &P17976,&P17979,&P23216,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:19.0.0) Gecko/19.0.0 Firefox/19.0.0*/
const Device UA359879 = {S41742, 144, { &P14877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA366552 = {S41743, 139, { &P14877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA622465 = {S41744, 143, { &P14877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-I9000 Build/IMM76K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30*/
const Device UA656587 = {S41745, 139, { &P14877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA671142 = {S41746, 139, { &P14877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-I9000 Build/IMM76K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA684302 = {S41747, 139, { &P14877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA686753 = {S41748, 61, { &P14877,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-I9000 Build/IML74K)*/
const Device UA777389 = {S41749, 139, { &P14877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9000 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA901802 = {S41750, 139, { &P14877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9000 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1963720 = {S41751, 61, { &P14877,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9000 Build/IMM76L)*/
const Device UA2022709 = {S41752, 144, { &P14877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; es-es; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2024444 = {S41753, 147, { &P14877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9000 Build/MyICS Ultimate) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2024840 = {S41754, 154, { &P14877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9000 Build/MyICS Ultimate Fusion) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025308 = {S41755, 145, { &P14877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-I9000B Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027941 = {S41756, 140, { &P14877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT-I9000B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029305 = {S41757, 139, { &P14877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9000 Build/IMM30B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032053 = {S41758, 138, { &P14877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-I9000 Build/IMM26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032171 = {S41759, 138, { &P14877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-I9000 Build/I9000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033312 = {S41760, 140, { &P14877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-I9000B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2436477 = {S41761, 126, { &P15767,&P18432,&P22900,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.2 Mobile/10A523 Safari/8536.25*/
const Device UA2436539 = {S41762, 124, { &P15767,&P18432,&P22900,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.2 Mobile/10A403 Safari/8536.25*/
const Device UA401986 = {S41763, 118, { &P17595,&P18207,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA2005691 = {S41764, 118, { &P17595,&P18207,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11*/
const Device UA2199850 = {S41765, 118, { &P17595,&P18207,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA2600228 = {S41766, 118, { &P17595,&P18207,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA2797185 = {S41767, 118, { &P17595,&P18207,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA3096185 = {S41768, 118, { &P17595,&P18207,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA5921 = {S41769, 141, { &P14842,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; es-es; E10i Build/1.1.A.0.12) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA30516 = {S41770, 155, { &P14842,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; SonyEricssonE10iv Build/1.2.A.1.174) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA30527 = {S41771, 156, { &P14842,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; SonyEricssonE10i-o Build/1.2.A.1.174) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA30535 = {S41772, 141, { &P14842,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-za; E10i Build/1.0.A.1.37) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA30553 = {S41773, 141, { &P14842,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; E10i Build/1.0.A.1.36) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA30570 = {S41774, 142, { &P14842,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; E10i Build/1.2.A.1.174) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA30603 = {S41775, 140, { &P14842,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; fr-fr; E10i Build/1.1.A.0.8) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA36526 = {S41776, 141, { &P14842,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; E10i Build/1.0.A.0.13) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA62023 = {S41777, 153, { &P14842,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; SonyEricssonE10i Build/1.0.A.1.36) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA67605 = {S41778, 65, { &P14842,&P5062,&P3441,&P18092 }};	/*Useragent - SonyEricssonE10i CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 1.6)*/
const Device UA70985 = {S41779, 141, { &P14842,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; E10i Build/1.0.A.1.38) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA609292 = {S41780, 142, { &P17172,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  Transformer TF101 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA609338 = {S41781, 142, { &P17172,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  Transformer TF101 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA758646 = {S41782, 141, { &P17172,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA902829 = {S41783, 141, { &P17172,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Transformer TF101 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA911286 = {S41784, 135, { &P17172,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Transformer Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2031151 = {S41785, 129, { &P17172,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TF101 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2033201 = {S41786, 135, { &P17281,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I905 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2041812 = {S41787, 132, { &P17281,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I905 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2206140 = {S41788, 61, { &P17281,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-I905 Build/IMM76D)*/
const Device UA2741725 = {S41789, 136, { &P18283,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754404 = {S41790, 137, { &P18283,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I535 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755131 = {S41791, 137, { &P18283,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T999 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755535 = {S41792, 137, { &P18283,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9300 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2772262 = {S41793, 129, { &P18283,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2783590 = {S41794, 137, { &P18283,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-T999 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2800378 = {S41795, 137, { &P18283,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SCH-I535 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893097 = {S41796, 134, { &P18283,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SC-06D Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA42991 = {S41797, 167, { &P15106,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_Salsa_C510e Build/LibertySense_GINGERBREAD_V0.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58886 = {S41798, 143, { &P15106,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Salsa C510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58888 = {S41799, 143, { &P15106,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_Salsa_C510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58889 = {S41800, 143, { &P15106,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC_Salsa_C510b Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58892 = {S41801, 143, { &P15106,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC Salsa C510b Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA371308 = {S41802, 67, { &P15106,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Salsa C510b Build/GRI40)*/
const Device UA565836 = {S41803, 67, { &P15106,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Salsa C510e Build/GRI40)*/
const Device UA1914615 = {S41804, 137, { &P15106,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC C510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1947273 = {S41805, 137, { &P15106,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC C510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019825 = {S41806, 136, { &P15106,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTCC510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021251 = {S41807, 131, { &P15106,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Salsa C510e Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2023175 = {S41808, 125, { &P15106,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC C510e Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2025244 = {S41809, 136, { &P15106,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-cn; HTCC510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025378 = {S41810, 125, { &P15106,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC C510e Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2393554 = {S41811, 130, { &P15106,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC C510e Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2413111 = {S41812, 136, { &P15106,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Salsa C510e Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2496802 = {S41813, 130, { &P15106,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC C510e Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA187689 = {S41814, 50, { &P15364,&P18110,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux; en) Presto/2.7.81 Version/11.00*/
const Device UA300157 = {S41815, 65, { &P15364,&P18110,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux x86_64; U; pl) Presto/2.7.62 Version/11.00*/
const Device UA306180 = {S41816, 63, { &P15364,&P18110,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; ja) Presto/2.7.62 Version/11.01*/
const Device UA797367 = {S41817, 69, { &P15364,&P18110,&P17217,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; X11; Linux x86_64; pl) Opera 11.00*/
const Device UA901128 = {S41818, 58, { &P15364,&P17285,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:14.0) Gecko/X Firefox/14.0*/
const Device UA1995938 = {S41819, 75, { &P15364,&P17285,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:14.0) Gecko/X Firefox/14.0.1 Firefox/14.0.1*/
const Device UA1996050 = {S41820, 72, { &P15364,&P17285,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:14.0) Gecko/X Firefox/14.0.1*/
const Device UA79895 = {S41821, 79, { &P15364,&P17285,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0 AppleWebKit (KHTML, like Gecko Chrome/6*/
const Device UA137332 = {S41822, 116, { &P15364,&P17285,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA254585 = {S41823, 147, { &P15364,&P17285,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam GameOverlay/1742; ) AppleWebKit/534.1 (KHTML, like Gecko) Chrome/6.0.444.0 Safari/534.1*/
const Device UA562508 = {S41824, 140, { &P15364,&P17285,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Source Client/0; ) AppleWebKit/534.1 (KHTML, like Gecko) Chrome/6.0.444.0 Safari/534.1*/
const Device UA720780 = {S41825, 153, { &P15364,&P17285,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3,gzip(gfe) (via translate.google.com)*/
const Device UA882912 = {S41826, 142, { &P15364,&P17285,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Valve Steam GameOverlay; ) AppleWebKit/534.1 (KHTML, like Gecko) Chrome/6.0.444.0 Safari/534.1*/
const Device UA2034052 = {S41827, 147, { &P18476,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I577 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2201953 = {S41828, 145, { &P18476,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SAMSUNG-SGH-I577 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2454731 = {S41829, 69, { &P18476,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SAMSUNG-SGH-I577 Build/IMM76D)*/
const Device UA172582 = {S41830, 128, { &P17595,&P17296,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13*/
const Device UA172639 = {S41831, 128, { &P17595,&P17296,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13*/
const Device UA172912 = {S41832, 128, { &P17595,&P17296,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13*/
const Device UA199732 = {S41833, 128, { &P17595,&P17296,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13*/
const Device UA295360 = {S41834, 127, { &P17595,&P17296,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.15 Safari/534.13*/
const Device UA303966 = {S41835, 128, { &P17595,&P17296,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13*/
const Device UA387148 = {S41836, 126, { &P17595,&P17296,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.0 Safari/534.13*/
const Device UA672271 = {S41837, 128, { &P17595,&P17296,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13*/
const Device UA820491 = {S41838, 127, { &P17595,&P17296,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.84 Safari/534.13*/
const Device UA2435093 = {S41839, 142, { &P15767,&P17146,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9A405 Safari/7534.48.3*/
const Device UA2437463 = {S41840, 140, { &P15767,&P17146,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9A334 Safari/7534.48.3*/
const Device UA2604350 = {S41841, 70, { &P15364,&P16110,&P22171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:23.0) Gecko/X Firefox/23.0*/
const Device UA2894863 = {S41842, 88, { &P15364,&P16110,&P22171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:23.0) Gecko/X Firefox/23.0 AlexaToolbar/alxf-2.18*/
const Device UA2508166 = {S41843, 62, { &P17976,&P17979,&P23665,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:21.0) Gecko/21.0 Firefox/21.0*/
const Device UA2712736 = {S41844, 72, { &P17976,&P17979,&P23665,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:21.0) Gecko/21.0 Firefox/21.0,gzip(gfe)*/
const Device UA2793247 = {S41845, 68, { &P17976,&P17979,&P23665,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:21.0a2) Gecko/21.0a2 Firefox/21.0a2*/
const Device UA1996344 = {S41846, 139, { &P12814,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I777 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2015072 = {S41847, 139, { &P12814,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I777 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2204395 = {S41848, 139, { &P12814,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I777 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2231060 = {S41849, 139, { &P12814,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I777 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2366706 = {S41850, 132, { &P12814,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I777 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2367253 = {S41851, 139, { &P12814,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I777 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2402324 = {S41852, 61, { &P12814,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I777 Build/JZO54K)*/
const Device UA2527458 = {S41853, 156, { &P12814,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Galaxy S II Epic 4G Touch Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2722814 = {S41854, 61, { &P12814,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SGH-I777 Build/JRO03H)*/
const Device UA2746702 = {S41855, 138, { &P12814,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I777 Build/cMIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2792383 = {S41856, 147, { &P12814,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I777 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2862062 = {S41857, 69, { &P12814,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-I777 Build/JZO54K)*/
const Device UA103307 = {S41858, 40, { &P17490,&P18378,&P18403,&P18092 }};	/*Useragent - Opera/9.30 (Nintendo Wii; U; ; 3642; en)*/
const Device UA2436626 = {S41859, 146, { &P18317,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Transformer Pad TF300T Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2486260 = {S41860, 130, { &P18317,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; TF300T Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2535478 = {S41861, 129, { &P18317,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; TF300T Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2560280 = {S41862, 151, { &P18317,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300T Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2712284 = {S41863, 152, { &P18317,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300TG Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2738165 = {S41864, 152, { &P18317,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300TL Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2865483 = {S41865, 139, { &P18317,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Pad TF300T Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2703934 = {S41866, 156, { &P21857,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2748433 = {S41867, 78, { &P21857,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HUAWEI Y300-0100 Build/HuaweiY300-0100)*/
const Device UA2425610 = {S41868, 140, { &P18736,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; HTC_K2_UL Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2468516 = {S41869, 140, { &P18736,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_C525u Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469885 = {S41870, 140, { &P18736,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_OneSV Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2482989 = {S41871, 141, { &P18736,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC One SV Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2505203 = {S41872, 63, { &P18736,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC One SV Build/IMM76I)*/
const Device UA2530327 = {S41873, 151, { &P18736,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-ch; HTC_OneSV/1.17.166.7 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2535955 = {S41874, 151, { &P18736,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; HTC_OneSV/1.17.169.7 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2537468 = {S41875, 151, { &P18736,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_OneSV/1.18.161.2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2565496 = {S41876, 151, { &P18736,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HTC_OneSV/1.17.162.6 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2582284 = {S41877, 160, { &P18736,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; HTC_C525u-orange-LS/1.17.73.5 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2603703 = {S41878, 151, { &P18736,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; HTC_OneSV/1.16.164.5 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2667190 = {S41879, 152, { &P18736,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HTC_OneSV/1.17.111.13 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2698573 = {S41880, 151, { &P18736,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HTC_K2_UL/1.17.111.8 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2709148 = {S41881, 161, { &P18736,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; HTC_C525u-orange-LS/1.17.73.10 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2767167 = {S41882, 151, { &P18736,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; HTC_OneSV/1.17.118.5 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA614724 = {S41883, 102, { &P18625,&P18110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (SMART-TV; X11; Linux i686) AppleWebKit/534.7 (KHTML, like Gecko) Version/5.0 Safari/534.7*/
const Device UA2580100 = {S41884, 106, { &P18625,&P18110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (SMART-TV; X11; Linux i686) AppleWebKit/535.20+ (KHTML, like Gecko) Version/5.0 Safari/535.20+*/
const Device UA573204 = {S41885, 102, { &P16246,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; SGH-i677)*/
const Device UA716499 = {S41886, 103, { &P15364,&P18110,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.7.10) Gecko/X Firefox/1.0.4 (Debian package 1.0.4-2sarge5)*/
const Device UA344831 = {S41887, 70, { &P15364,&P5690,&P18245,&P18163 }};	/*Useragent - Mozilla/5.0/Nutch-1.1 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.13)*/
const Device UA2489019 = {S41888, 112, { &P18648,&P5043,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia306/03.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2257642 = {S41889, 140, { &P19878,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I497 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555424 = {S41890, 69, { &P19878,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SAMSUNG-SGH-I497 Build/IMM76D)*/
const Device UA2643098 = {S41891, 132, { &P19878,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I497 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2728879 = {S41892, 139, { &P22970,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T599 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2864809 = {S41893, 140, { &P22970,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T599N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878289 = {S41894, 61, { &P22970,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T599 Build/JZO54K)*/
const Device UA2878466 = {S41895, 140, { &P22970,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-T599V Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3013271 = {S41896, 62, { &P22970,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T599N Build/JZO54K)*/
const Device UA3091436 = {S41897, 62, { &P22970,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T599V Build/JZO54K)*/
const Device UA50523 = {S41898, 154, { &P17308,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROIDX Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola DROIDX*/
const Device UA56152 = {S41899, 156, { &P17308,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROIDX Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola DROIDX*/
const Device UA67850 = {S41900, 155, { &P17308,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROIDXBuild/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola DROIDX*/
const Device UA67854 = {S41901, 155, { &P17308,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.1; en-us; DROIDX Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola DROIDX*/
const Device UA67998 = {S41902, 155, { &P17308,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;DROIDX Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola DROIDX*/
const Device UA68112 = {S41903, 155, { &P17308,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROIDX Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1480X854 motorola DROIDX*/
const Device UA68150 = {S41904, 155, { &P17308,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; DROIDX Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola DROIDX*/
const Device UA68175 = {S41905, 155, { &P17308,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROIDX Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 480X854 motorola DROIDX*/
const Device UA89937 = {S41906, 133, { &P17308,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROIDX Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA159340 = {S41907, 56, { &P17308,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; DROIDX Build/VZW)*/
const Device UA475325 = {S41908, 54, { &P17308,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; DROIDX Build/VZW)*/
const Device UA3065333 = {S41909, 140, { &P22867,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5303 Build/12.0.A.1.257) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3130870 = {S41910, 134, { &P18635,&P21637,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus 4 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA2734913 = {S41911, 139, { &P18559,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-LG730 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2740109 = {S41912, 61, { &P18559,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-LG730 Build/JZO54K)*/
const Device UA609259 = {S41913, 64, { &P17968,&P17979,&P20435,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:15.0) Gecko/15.0 Firefox/15.0a1*/
const Device UA902157 = {S41914, 64, { &P17968,&P17979,&P20435,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:15.0) Gecko/15.0 Firefox/15.0a2*/
const Device UA925084 = {S41915, 62, { &P17968,&P17979,&P20435,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:15.0) Gecko/15.0 Firefox/15.0*/
const Device UA2040383 = {S41916, 64, { &P17968,&P17979,&P20435,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:15.0) Gecko/15.0 Firefox/15.0.1*/
const Device UA2042025 = {S41917, 68, { &P17968,&P17979,&P20435,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:15.0.1) Gecko/15.0.1 Firefox/15.0.1*/
const Device UA2800692 = {S41918, 142, { &P12280,&P22882,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4372q Safari/8536.25*/
const Device UA2885397 = {S41919, 142, { &P12280,&P22882,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4400f Safari/8536.25*/
const Device UA2947254 = {S41920, 142, { &P12280,&P22882,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4414e Safari/8536.25*/
const Device UA3028145 = {S41921, 142, { &P12280,&P22882,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4435d Safari/8536.25*/
const Device UA3034285 = {S41922, 142, { &P12280,&P22882,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4449a Safari/8536.25*/
const Device UA3038870 = {S41923, 142, { &P12280,&P22882,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4449d Safari/8536.25*/
const Device UA3094480 = {S41924, 140, { &P12280,&P22882,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A465 Safari/8536.25*/
const Device UA3115623 = {S41925, 142, { &P12280,&P22882,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A501 Safari/8536.25*/
const Device UA2554247 = {S41926, 146, { &P21611,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2586647 = {S41927, 68, { &P21611,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI-M931 Build/HuaweiM931)*/
const Device UA2586650 = {S41928, 209, { &P21611,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 androidAppVersion/2.5.22 AndroidWebkitWrapper BillingSupported*/
const Device UA236944 = {S41929, 138, { &P17937,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MOTWX435KT Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA378730 = {S41930, 62, { &P17937,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; MOTWX435KT Build/FROYO)*/
const Device UA891308 = {S41931, 144, { &P17937,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2 (PR5); en-us; MOTWX435KT Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023972 = {S41932, 136, { &P17937,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-; MOTWX435KT Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2409804 = {S41933, 133, { &P17937,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Triumph Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2674855 = {S41934, 129, { &P17595,&P17295,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/25.0.648.205 Safari/534.16*/
const Device UA2714964 = {S41935, 117, { &P17595,&P17295,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/25.0.782.112 Safari/535.1*/
const Device UA2715036 = {S41936, 119, { &P17595,&P17295,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/25.0.742.122 Safari/534.30*/
const Device UA2715031 = {S41937, 117, { &P17595,&P17295,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.25 (KHTML, like Gecko) Chrome/25.0.703.0 Safari/534.25*/
const Device UA2716152 = {S41938, 129, { &P17595,&P17295,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/25.0.648.205 Safari/534.16*/
const Device UA924170 = {S41939, 69, { &P22667,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Xbox)*/
const Device UA2715147 = {S41940, 88, { &P15364,&P16110,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:21.0) Gecko/X Firefox/21.0 AlexaToolbar/alxf-2.18*/
const Device UA2739408 = {S41941, 74, { &P15364,&P16110,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:16.0.1) Gecko/X Firefox/21.0.1*/
const Device UA2784867 = {S41942, 85, { &P15364,&P16110,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:21.0) Gecko/X Firefox/21.0 IceDragon/21.0.1.15*/
const Device UA24319 = {S41943, 133, { &P15679,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; ADR6325 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51879 = {S41944, 153, { &P15679,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; USCCADR6325US Carrier ID 45 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA53599 = {S41945, 135, { &P15679,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; ADR6325VW Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70447 = {S41946, 134, { &P15679,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2; en-us; ADR6325VW Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA234131 = {S41947, 57, { &P15679,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; ADR6325 Build/FRF91)*/
const Device UA2019775 = {S41948, 140, { &P15679,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; ADR6325 Build/FRF91) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2025922 = {S41949, 134, { &P15679,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; ADR6325VW Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA2603648 = {S41950, 137, { &P21648,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2709215 = {S41951, 138, { &P21648,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505X Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2711481 = {S41952, 130, { &P21648,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA79079 = {S41953, 123, { &P14404,&P17484,&P3834,&P18092 }};	/*Useragent - SAMSUNG-GT-S5260/S5260XXKG2 SHP/VPP/R5 Dolfin/2.0 NexPlayer/3.0 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B*/
const Device UA2160147 = {S41954, 132, { &P14404,&P17484,&P3834,&P18092 }};	/*Useragent - SAMSUNG-GT-S5260P-ORANGE/S5260PBVKJ1 SHP/VPP/R5 Dolfin/2.0 NexPlayer/3.0 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B*/
const Device UA91053 = {S41955, 117, { &P17595,&P17295,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA256008 = {S41956, 117, { &P17595,&P17295,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA575365 = {S41957, 117, { &P17595,&P17295,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_2) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2*/
const Device UA628220 = {S41958, 117, { &P17595,&P17295,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA629764 = {S41959, 117, { &P17595,&P17295,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_5) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA2146815 = {S41960, 117, { &P17595,&P17295,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2*/
const Device UA2146967 = {S41961, 117, { &P17595,&P17295,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2*/
const Device UA2462822 = {S41962, 117, { &P17595,&P17295,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2*/
const Device UA2674528 = {S41963, 129, { &P17595,&P17295,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/15.0.648.205 Safari/534.16*/
const Device UA2718648 = {S41964, 129, { &P17595,&P17295,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/15.0.648.205 Safari/534.16*/
const Device UA32854 = {S41965, 70, { &P15364,&P16110,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:5.0) Gecko/20100101 Firefox/5.0*/
const Device UA33252 = {S41966, 135, { &P17935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PG06100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA191814 = {S41967, 135, { &P17935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PG06100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA200839 = {S41968, 59, { &P17935,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; PG06100 Build/GRI40)*/
const Device UA249020 = {S41969, 135, { &P17935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PG06100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA309176 = {S41970, 135, { &P17935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PG06100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA329037 = {S41971, 135, { &P17935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PG06100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA382189 = {S41972, 135, { &P17935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PG06100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625398 = {S41973, 59, { &P17935,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; PG06100 Build/GRJ22)*/
const Device UA838188 = {S41974, 135, { &P17935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PG06100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2640952 = {S41975, 135, { &P17935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PG06100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA23270 = {S41976, 136, { &P12918,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LG-LS855 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA33435 = {S41977, 136, { &P12918,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LS855 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633790 = {S41978, 60, { &P12918,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-LS855 Build/GRJ22)*/
const Device UA2028606 = {S41979, 141, { &P12918,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LS855 Build/CTMod 3.68) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2424829 = {S41980, 147, { &P12918,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LS855 Build/CTMod 3.75 Final) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2734515 = {S41981, 141, { &P12918,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LS855 Build/CTMod 3.80) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030996 = {S41982, 62, { &P17976,&P17979,&P18522,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:18.0) Gecko/18.0 Firefox/18.0*/
const Device UA2384490 = {S41983, 68, { &P17976,&P17979,&P18522,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:18.0a2) Gecko/18.0a2 Firefox/18.0a2*/
const Device UA2457656 = {S41984, 72, { &P17976,&P17979,&P18522,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:18.0) Gecko/18.0 Firefox/18.0,gzip(gfe)*/
const Device UA2516373 = {S41985, 68, { &P17976,&P17979,&P18522,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:18.0.2) Gecko/18.0.2 Firefox/18.0.2*/
const Device UA2566745 = {S41986, 138, { &P16362,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566744 = {S41987, 142, { &P16362,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710VMUB Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566750 = {S41988, 141, { &P16362,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710BST Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2605937 = {S41989, 131, { &P16362,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2666810 = {S41990, 138, { &P16362,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SPH-D710 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA925212 = {S41991, 128, { &P14777,&P17484,&P4732,&P18092 }};	/*Useragent - SAMSUNG-SGH-A817/A817UCLD2; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.21648; U; en-US) Opera 9.50*/
const Device UA2671067 = {S41992, 149, { &P19894,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R830 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 USCC-R830*/
const Device UA2673224 = {S41993, 61, { &P19894,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-R830 Build/JZO54K)*/
const Device UA2781988 = {S41994, 140, { &P19894,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R830C Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022487 = {S41995, 62, { &P19894,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-R830C Build/JZO54K)*/
const Device UA2384820 = {S41996, 31, { &P14188,&P5129,&P24673,&P18092 }};	/*Useragent - Nokia5233/UCWEB8.7.0.187/50/999*/
const Device UA2387677 = {S41997, 119, { &P14188,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5233) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2405480 = {S41998, 41, { &P14188,&P5129,&P24673,&P18092 }};	/*Useragent - Nokia5233UCBrowser/8.7.1.234/50/444/UCWEB*/
const Device UA2422965 = {S41999, 120, { &P14188,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia 5233) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/352/UCWEB Mobile*/
const Device UA2429980 = {S42000, 101, { &P14188,&P5129,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5233) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA100053 = {S42001, 70, { &P15364,&P5690,&P22728,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Avant Browser)*/
const Device UA109432 = {S42002, 179, { &P15364,&P5690,&P22728,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Avant Browser; SV1; Avant Browser; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; yplus 4.1.00b)*/
const Device UA123599 = {S42003, 131, { &P15364,&P5690,&P22728,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Avant Browser; Avant Browser; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.1)*/
const Device UA260794 = {S42004, 124, { &P15364,&P5690,&P22728,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Avant Browser [avantbrowser.com]; iOpus-I-M; QXW03416; .NET CLR 1.1.4322)*/
const Device UA261829 = {S42005, 128, { &P15364,&P5690,&P22728,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Avant Browser)*/
const Device UA855825 = {S42006, 85, { &P15364,&P5690,&P22728,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Avant Browser; Avant Browser)*/
const Device UA1859856 = {S42007, 149, { &P15364,&P5690,&P22728,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Avant Browser; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 2.8; .NET CLR 2.0.50727)*/
const Device UA2362551 = {S42008, 138, { &P15364,&P5690,&P22728,&P18092 }};	/*Useragent - Sogou Web Sprider(compatible; Mozilla 4.0; MSIE 6.0; Windows NT 5.1; SV1; Avant Browser; InfoPath.1; .NET CLR 2.0.50727; .NET CLR1.1.4322)*/
const Device UA2746962 = {S42009, 183, { &P15364,&P5690,&P22728,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Avant Browser)*/
const Device UA2774428 = {S42010, 177, { &P15364,&P5690,&P22728,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Avant Browser; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; Alexa Toolbar; .NET CLR 2.0.50727)*/
const Device UA2985907 = {S42011, 186, { &P15364,&P5690,&P22728,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Avant Browser)*/
const Device UA2798185 = {S42012, 86, { &P17968,&P17979,&P17370,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/5.0.18302/30.3214; U; en) Presto/2.8.119 Version/11.10*/
const Device UA3015973 = {S42013, 88, { &P17968,&P17979,&P17370,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/5.0.23388/27.2338; U; en) Presto/2.8.119 320x480 Android*/
const Device UA2884698 = {S42014, 134, { &P15364,&P17285,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100 (Edition Next)*/
const Device UA2943977 = {S42015, 141, { &P15364,&P17285,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100 (Edition Next)*/
const Device UA3027527 = {S42016, 119, { &P15364,&P17285,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153*/
const Device UA3028002 = {S42017, 126, { &P15364,&P17285,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153*/
const Device UA3029602 = {S42018, 141, { &P15364,&P17285,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Campaign 21)*/
const Device UA3029913 = {S42019, 137, { &P15364,&P17285,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Rambler)*/
const Device UA3031401 = {S42020, 148, { &P15364,&P17285,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153 (Edition Campaign 21)*/
const Device UA578287 = {S42021, 152, { &P18494,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-R930 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 USCC-R930*/
const Device UA871245 = {S42022, 66, { &P18494,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-R930 Build/GINGERBREAD)*/
const Device UA3109 = {S42023, 146, { &P14784,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-goapk.com; zh-cn; HTC Hero Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA3114 = {S42024, 139, { &P14784,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-gb; Cronos Hero Build/ERD72) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA3175 = {S42025, 136, { &P14784,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; HTC HERO Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA3200 = {S42026, 161, { &P14784,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1 (7hero-astar9.3); ru-ru; HTC Hero Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA3235 = {S42027, 151, { &P14784,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; HTC Hero-Orange Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA30895 = {S42028, 144, { &P14784,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HTC Hero Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA64017 = {S42029, 142, { &P14784,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; HTC Hero Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA65403 = {S42030, 136, { &P14784,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-in; HTC Hero Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA66115 = {S42031, 144, { &P14784,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC_HERO Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA369924 = {S42032, 144, { &P14784,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Hero Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA782721 = {S42033, 149, { &P14784,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-v1.0; en-gb; HTC Hero Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA821570 = {S42034, 130, { &P14784,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; HTC Hero Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2023488 = {S42035, 144, { &P14784,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; HTC HERO Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17HTC HERO*/
const Device UA2973272 = {S42036, 140, { &P20705,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One 801e Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973943 = {S42037, 135, { &P20705,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975030 = {S42038, 132, { &P20705,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; One Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021643 = {S42039, 136, { &P20705,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; HTC_One Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023534 = {S42040, 144, { &P20705,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One dual sim Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023854 = {S42041, 161, { &P20705,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One with MoDaCo.SWITCH Beta 8 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024210 = {S42042, 137, { &P20705,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC_PN071 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2203244 = {S42043, 114, { &P18267,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-02/11.57; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2405797 = {S42044, 113, { &P18267,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-02/10.90; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5*/
const Device UA79904 = {S42045, 81, { &P15364,&P17285,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/8*/
const Device UA159879 = {S42046, 119, { &P15364,&P17285,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10*/
const Device UA779153 = {S42047, 156, { &P15364,&P17285,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10,gzip(gfe) (via translate.google.com)*/
const Device UA347737 = {S42048, 116, { &P17595,&P17295,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA351723 = {S42049, 116, { &P17595,&P17295,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA352591 = {S42050, 116, { &P17595,&P17295,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_5) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA353623 = {S42051, 116, { &P17595,&P17295,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA366367 = {S42052, 116, { &P17595,&P17295,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7*/
const Device UA380867 = {S42053, 116, { &P17595,&P17295,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA587490 = {S42054, 116, { &P17595,&P17295,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA693790 = {S42055, 116, { &P17595,&P17295,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA2674814 = {S42056, 129, { &P17595,&P17295,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/16.0.648.205 Safari/534.16*/
const Device UA2715586 = {S42057, 129, { &P17595,&P17295,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/16.0.648.205 Safari/534.16*/
const Device UA910750 = {S42058, 127, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; Build/JRN84D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2030158 = {S42059, 137, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; cm_tenderloin Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2034452 = {S42060, 129, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2358576 = {S42061, 137, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; cm_tenderloin Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2366978 = {S42062, 128, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; EPAD Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2384121 = {S42063, 123, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2385191 = {S42064, 139, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Transformer Pad Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2385565 = {S42065, 128, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MiSS Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2398767 = {S42066, 147, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Dark JellyBellyG1 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2401476 = {S42067, 135, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; X720D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2404428 = {S42068, 139, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; VegaBean Beta 6 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2404548 = {S42069, 136, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PG8600 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2467335 = {S42070, 124, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1;  Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2482045 = {S42071, 156, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_gq2002_5647_notv Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2483281 = {S42072, 137, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; unknown Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2483513 = {S42073, 153, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbla668_9p017 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2484482 = {S42074, 159, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbla668_9p017_fwvga Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2485852 = {S42075, 138, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; skz_tenderloin Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2485914 = {S42076, 150, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbl1_9p017 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2490522 = {S42077, 157, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Motorola Galaxy s3 Build/9.8.2O-72_VZW-8) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2504521 = {S42078, 152, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1911_v77_gq2000_9p017 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2505252 = {S42079, 157, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1911_v77_gq2000_9p017_6628 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2505562 = {S42080, 166, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1909c_v77_gq1008_9p017_sui_a41_6628 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2506261 = {S42081, 161, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbla668_9p017_nozhrtw Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2506548 = {S42082, 155, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1908_v77_jbl1_9p017_6628 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2506604 = {S42083, 152, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_gq2002_9p017 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2507348 = {S42084, 137, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.5; iPhone5 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2528446 = {S42085, 144, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbl1 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2555950 = {S42086, 150, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jdt1_9p017 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2557240 = {S42087, 147, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_cxq_asx Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2558287 = {S42088, 159, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1911_v77_gq2000_a41_6628_v20 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2598346 = {S42089, 153, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbla668_fwvga Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2599699 = {S42090, 134, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AML-MX REF Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2659426 = {S42091, 153, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1920_v77_zlh_9p017_asx Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2661468 = {S42092, 155, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1909l_v77_gq1008_a41_v20 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2696924 = {S42093, 131, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Android Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2728980 = {S42094, 155, { &P17968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_gq2002_9p017_fg Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA43016 = {S42095, 163, { &P12931,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-C729 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA368817 = {S42096, 162, { &P12931,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-C729Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA370372 = {S42097, 162, { &P12931,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; LG-C729 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA653077 = {S42098, 59, { &P12931,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-C729 Build/GRJ22)*/
const Device UA1940188 = {S42099, 162, { &P12931,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-C729 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA1966935 = {S42100, 162, { &P12931,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;LG-C729 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2022744 = {S42101, 162, { &P12931,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; LG-C729 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2030490 = {S42102, 137, { &P18468,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IdeaTabA2109A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2397792 = {S42103, 66, { &P18468,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; IdeaTabA2109A Build/IMM76D)*/
const Device UA2598146 = {S42104, 134, { &P18468,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IdeaTabA2109A Build/IMM76D) UC AppleWebKit/534.31 (KHTML like Gecko) Mobile Safari/534.31*/
const Device UA2710842 = {S42105, 138, { &P22597,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I545 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2753354 = {S42106, 60, { &P22597,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SCH-I545 Build/JDQ39)*/
const Device UA2937312 = {S42107, 136, { &P22597,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-; SCH-I545 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3006510 = {S42108, 139, { &P22597,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I545 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092213 = {S42109, 139, { &P22597,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I545L Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3093635 = {S42110, 174, { &P22597,&P18598,&P19802,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.2.2;dev=jfltevzw;devm=SCH-I545;man=samsung;brw=Android;shwd=1080x1920;shwa=1080x1845;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA582136 = {S42111, 69, { &P15364,&P5690,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0(Windows; U; Windows NT 5.2; rv:1.9.2) Gecko/X Firefox/8.0*/
const Device UA582647 = {S42112, 79, { &P15364,&P5690,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.2.3) Gecko/X Firefox/8.0*/
const Device UA590205 = {S42113, 68, { &P15364,&P5690,&P17698,&P18092 }};	/*Useragent - User-Agent: Mozilla/5.0 (Windows NT 5.2; rv:8.0) Gecko/X Firefox/8.0*/
const Device UA660855 = {S42114, 68, { &P15364,&P5690,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:8.0) Gecko/X Firefox/8.0 Firefox/8.0*/
const Device UA845804 = {S42115, 79, { &P15364,&P5690,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.3) Gecko/X Firefox/8.0*/
const Device UA1992492 = {S42116, 63, { &P15364,&P5690,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:8.0; en_us) Gecko/X Firefox/8.0*/
const Device UA612419 = {S42117, 112, { &P18221,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia302/14.53; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6*/
const Device UA2355219 = {S42118, 112, { &P18221,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia302/14.78; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.9*/
const Device UA2530172 = {S42119, 140, { &P13141,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I727R Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2536314 = {S42120, 140, { &P13141,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E110S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2603774 = {S42121, 62, { &P13141,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I727R Build/JZO54K)*/
const Device UA2790297 = {S42122, 62, { &P13141,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E110S Build/JZO54K)*/
const Device UA2997332 = {S42123, 139, { &P13141,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; GT-I9210 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3119444 = {S42124, 138, { &P13141,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SGH-I727R Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3062537 = {S42125, 134, { &P21261,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.3; I9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3063130 = {S42126, 136, { &P21261,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.9; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3064640 = {S42127, 136, { &P21261,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094437 = {S42128, 134, { &P21261,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.3; I9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109930 = {S42129, 149, { &P21261,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.9; GT-I9500 Build/JDQ39.I9500ZSUAMDF) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3113444 = {S42130, 136, { &P21261,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.9; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114789 = {S42131, 137, { &P21261,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2974028 = {S42132, 149, { &P18113,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021134 = {S42133, 142, { &P18113,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2502857 = {S42134, 140, { &P14854,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHW-M250K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2504584 = {S42135, 140, { &P14854,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHW-M250S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2531607 = {S42136, 140, { &P14854,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHW-M250L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2653876 = {S42137, 62, { &P14854,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHW-M250S Build/JZO54K)*/
const Device UA2663337 = {S42138, 62, { &P14854,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHW-M250K Build/JZO54K)*/
const Device UA2784039 = {S42139, 147, { &P14854,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHW-M250K/KKJME3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2858041 = {S42140, 62, { &P14854,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHW-M250L Build/JZO54K)*/
const Device UA2951873 = {S42141, 143, { &P14854,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SHW-M250S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2988356 = {S42142, 138, { &P14854,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; SHW-M250S Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2988740 = {S42143, 138, { &P14854,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; SHW-M250K Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3030155 = {S42144, 147, { &P14854,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHW-M250K/KKJMG1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA627220 = {S42145, 150, { &P19872,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I847 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1961517 = {S42146, 74, { &P19872,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SAMSUNG-SGH-I847 Build/GINGERBREAD)*/
const Device UA2643218 = {S42147, 148, { &P19872,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SAMSUNG-SGH-I847 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA907083 = {S42148, 155, { &P12280,&P18117,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9B206 Safari/7534.48.3*/
const Device UA907235 = {S42149, 153, { &P12280,&P18117,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9B179 Safari/7534.48.3*/
const Device UA910223 = {S42150, 153, { &P12280,&P18117,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9B176 Safari/7534.48.3*/
const Device UA910448 = {S42151, 155, { &P12280,&P18117,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9B208 Safari/7534.48.3*/
const Device UA1964833 = {S42152, 153, { &P12280,&P18117,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9A405 Safari/7534.48.3*/
const Device UA2462363 = {S42153, 222, { &P18254,&P7133,&P17190,&P19831 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MDDR; .NET4.0C; .NET4.0E; .NET CLR 1.1.4322; Tablet PC 2.0); 360Spider*/
const Device UA901298 = {S42154, 132, { &P15882,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P5110 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA905296 = {S42155, 132, { &P15882,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P5110 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1959645 = {S42156, 133, { &P15882,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  GT-P5110 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2608128 = {S42157, 133, { &P15882,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  GT-P5110 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2420370 = {S42158, 139, { &P19920,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-MS870 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2559740 = {S42159, 61, { &P19920,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-MS870 Build/IMM76L)*/
const Device UA2566454 = {S42160, 130, { &P18639,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 10 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567345 = {S42161, 131, { &P18639,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 10 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2599949 = {S42162, 129, { &P18639,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Nexus 10 Build/JVP15P) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2225294 = {S42163, 111, { &P20273,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; HTC6990LVW)*/
const Device UA2974041 = {S42164, 140, { &P22394,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6603 Build/10.1.A.1.434) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA347746 = {S42165, 185, { &P12581,&P5055,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.2 Nokia500/011.033; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.37 Mobile Safari/533.4 3gpp-gba*/
const Device UA640056 = {S42166, 8, { &P12581,&P5055,&P3804,&P18092 }};	/*Useragent - Nokia500*/
const Device UA2029805 = {S42167, 139, { &P20453,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; NABI2-NV7A Build/IMM76L)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2203695 = {S42168, 141, { &P20453,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NABI2-NV7A Build/IMM76L)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2403333 = {S42169, 144, { &P20453,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; NABI2-NV7A-UK Build/IMM76L)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2423912 = {S42170, 144, { &P20453,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ie; NABI2-NV7A-IE Build/IMM76L)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2689396 = {S42171, 141, { &P20453,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NABI2-NV7A Build/IMM76D)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3090811 = {S42172, 142, { &P20453,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NABI2-NV7A Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon*/
const Device UA2426677 = {S42173, 62, { &P18606,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; PadFone 2 Build/JRO03L)*/
const Device UA2427461 = {S42174, 140, { &P18606,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; PadFone 2 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467947 = {S42175, 133, { &P18606,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PadFone 2 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA898943 = {S42176, 139, { &P18456,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T699 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2382542 = {S42177, 61, { &P18456,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-T699 Build/IMM76D)*/
const Device UA110713 = {S42178, 128, { &P15364,&P7133,&P22444,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Iron/5.0.381.0 Chrome/5.0.381 Safari/533.4*/
const Device UA238557 = {S42179, 133, { &P15802,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_Amaze_4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA238575 = {S42180, 140, { &P15802,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_Amaze_4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA350516 = {S42181, 140, { &P15802,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; HTC Amaze 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA395913 = {S42182, 64, { &P15802,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC_Amaze_4G Build/GRJ22)*/
const Device UA623986 = {S42183, 139, { &P15802,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;HTC_Amaze_4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA630953 = {S42184, 136, { &P15802,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; HTC_Ruby Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA680415 = {S42185, 139, { &P15802,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.4; en-us; HTC_Amaze_4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA683301 = {S42186, 64, { &P15802,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Amaze 4G Build/GRJ22)*/
const Device UA698224 = {S42187, 139, { &P15802,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_Amaze_4GBuild/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA750676 = {S42188, 132, { &P15802,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;HTC_Amaze_4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA762331 = {S42189, 132, { &P15802,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_Amaze_4GBuild/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA1904162 = {S42190, 60, { &P15802,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Ruby Build/GRJ22)*/
const Device UA2030640 = {S42191, 140, { &P15802,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Amaze 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030644 = {S42192, 129, { &P15802,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Ruby Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2375420 = {S42193, 137, { &P15802,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-pk; HTC_Amaze Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2590465 = {S42194, 136, { &P15802,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-pk; HTC Ruby Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3056956 = {S42195, 155, { &P15802,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HITECH-Amaze-S500 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2937734 = {S42196, 130, { &P23956,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SM-T310 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2966315 = {S42197, 59, { &P23956,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SM-T310 Build/JDQ39)*/
const Device UA3020688 = {S42198, 131, { &P23956,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SM-T310X Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2197777 = {S42199, 153, { &P12280,&P17090,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8H7 Safari/7534.48.3*/
const Device UA2205557 = {S42200, 153, { &P12280,&P17090,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8J2 Safari/7534.48.3*/
const Device UA2205610 = {S42201, 153, { &P12280,&P17090,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8G4 Safari/7534.48.3*/
const Device UA2205875 = {S42202, 153, { &P12280,&P17090,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8F190 Safari/7534.48.3*/
const Device UA2205880 = {S42203, 153, { &P12280,&P17090,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8L1 Safari/7534.48.3*/
const Device UA2235500 = {S42204, 153, { &P12280,&P17090,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8K2 Safari/7534.48.3*/
const Device UA2659371 = {S42205, 150, { &P12280,&P17090,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 435 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/21.0.1180.77 Mobile/8L1 Safari/7534.48.3*/
const Device UA2559969 = {S42206, 67, { &P15364,&P7133,&P21791,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:19.0) Gecko/X SeaMonkey/2.16*/
const Device UA2569625 = {S42207, 82, { &P15364,&P7133,&P21791,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:19.0) Gecko/X Firefox/19.0 SeaMonkey/2.16.2*/
const Device UA2948336 = {S42208, 135, { &P18635,&P21637,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus 4 Build/JWR66V) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.111 Mobile Safari/537.36*/
const Device UA582145 = {S42209, 100, { &P15364,&P7133,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.3) Gecko/X Firefox/8.0 (.NET CLR 3.5.30729)*/
const Device UA645661 = {S42210, 75, { &P15364,&P7133,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:8.0) Gecko/X Firefox/8.0 Firefox/8.0*/
const Device UA654092 = {S42211, 68, { &P15364,&P7133,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:8.0) Gecko/X Firefox/8.0*/
const Device UA923027 = {S42212, 82, { &P15364,&P7133,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:8.0) Gecko/X Firefox/8.02011-10-05 17:28:18*/
const Device UA1934175 = {S42213, 81, { &P15364,&P7133,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.1; Win64; x64; rv:8.0.1) Gecko/X Firefox/8.0.1*/
const Device UA2259017 = {S42214, 70, { &P15364,&P7133,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:8.0.1) Gecko/X Firefox/8.0.1,gzip(gfe)*/
const Device UA2441032 = {S42215, 75, { &P15364,&P7133,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Firefox/8.0.1; Windows NT 6.1; WOW64; Trident/5.0)*/
const Device UA2561910 = {S42216, 82, { &P15364,&P7133,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.1.2) Gecko/X Firefox/8 GTB5*/
const Device UA56079 = {S42217, 147, { &P13233,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3G Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA56097 = {S42218, 148, { &P13233,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3G Build/FRG83B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67059 = {S42219, 148, { &P13233,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67086 = {S42220, 147, { &P13233,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; T-Mobile myTouch 3G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67155 = {S42221, 146, { &P13233,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-; T-Mobile myTouch 3G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67493 = {S42222, 147, { &P13233,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-MobilemyTouch 3G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67595 = {S42223, 147, { &P13233,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.1; en-us; T-Mobile myTouch 3G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67617 = {S42224, 147, { &P13233,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;T-Mobile myTouch 3G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67670 = {S42225, 147, { &P13233,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3GBuild/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67723 = {S42226, 147, { &P13233,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA67847 = {S42227, 147, { &P13233,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch3G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA68046 = {S42228, 155, { &P13233,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3G Build/FRG83D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA161686 = {S42229, 72, { &P13233,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; T-Mobile myTouch 3G Build/FRG83D)*/
const Device UA625516 = {S42230, 148, { &P13233,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; T-Mobile myTouch 3G Build/FRK76C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA631666 = {S42231, 72, { &P13233,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.3; T-Mobile myTouch 3G Build/FRK76C)*/
const Device UA844231 = {S42232, 151, { &P13233,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3G Build/FRD91_INT) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025357 = {S42233, 147, { &P13233,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; T-MobilemyTouch 3G Build/FRK76C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027190 = {S42234, 147, { &P13233,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.3; en-us; T-Mobile myTouch 3G Build/FRK76C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029312 = {S42235, 147, { &P13233,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; en-us;T-Mobile myTouch 3G Build/FRK76C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030632 = {S42236, 146, { &P13233,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; es-; T-Mobile myTouch 3G Build/FRK76C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030669 = {S42237, 147, { &P13233,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.3; en-us; T-Mobile myTouch 3G Build/FRK76C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA601064 = {S42238, 142, { &P18465,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Kindle Fire Build/GRK39F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA675601 = {S42239, 146, { &P18465,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Amazon Kindle Fire Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA723712 = {S42240, 142, { &P18465,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Kindle Fire Build/IML74k) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA758734 = {S42241, 149, { &P18465,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Amazon Kindle Fire Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA830752 = {S42242, 149, { &P18465,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Amazon Kindle Fire Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1982635 = {S42243, 68, { &P18465,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Amazon Kindle Fire Build/MR1)*/
const Device UA2026333 = {S42244, 156, { &P18465,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4-2.4.20; en-us; Amazon Kindle Fire Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028881 = {S42245, 151, { &P18465,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Amazon Kindle Fire Build/6.2.1_GX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034002 = {S42246, 71, { &P18465,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Amazon Kindle Fire Build/IML74k)*/
const Device UA2044022 = {S42247, 149, { &P18465,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Amazon Kindle Fire Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2044075 = {S42248, 132, { &P18465,&P18066,&P18062,&P18092 }};	/*Useragent - Biblia/1.1.3 (Linux; U; Android 4.0.4; es-US; Amazon Kindle Fire Build/cm_otter-userdebug 4.0.4 IMM76I eng.x0xhellx0x.X.X test-keys)*/
const Device UA2044282 = {S42249, 64, { &P18465,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Kindle Fire Build/IML74k)*/
const Device UA2179108 = {S42250, 128, { &P18465,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2352254 = {S42251, 57, { &P18465,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; KFOT Build/IML74K)*/
const Device UA2355757 = {S42252, 135, { &P18465,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; KFOT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426763 = {S42253, 136, { &P18465,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; KF012 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2983735 = {S42254, 126, { &P18465,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; KFOT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA33441 = {S42255, 142, { &P16362,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SPH-D710 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA231617 = {S42256, 66, { &P16362,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SPH-D710 Build/GINGERBREAD)*/
const Device UA352863 = {S42257, 66, { &P16362,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SPH-D710 Build/GINGERBREAD)*/
const Device UA622553 = {S42258, 142, { &P16362,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-D710 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA759824 = {S42259, 140, { &P16362,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-; SPH-D710 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021046 = {S42260, 142, { &P16362,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SPH-D710 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021264 = {S42261, 149, { &P16362,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-D710 Build/GINGERBREAD) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2025709 = {S42262, 137, { &P16362,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; MID7107SC Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029095 = {S42263, 143, { &P16362,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; DOOV_D710 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2412408 = {S42264, 154, { &P16362,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android GINGERBREAD 2.3.6; en-us; SPH-D710 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3690 = {S42265, 76, { &P14880,&P5026,&P3410,&P18092 }};	/*Useragent - BlackBerry8320/4.5.0.81 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100*/
const Device UA2974014 = {S42266, 149, { &P18317,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974477 = {S42267, 150, { &P18317,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300TL Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2975942 = {S42268, 150, { &P18317,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300TG Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022201 = {S42269, 128, { &P18317,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; TF300T Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022791 = {S42270, 137, { &P18317,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Pad TF300T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023386 = {S42271, 143, { &P18317,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Transformer Pad TF300T Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA174963 = {S42272, 26, { &P18254,&P7480,&P17470,&P18092 }};	/*Useragent - nutch-solr-integration/1.1*/
const Device UA563382 = {S42273, 40, { &P18254,&P7480,&P17470,&P18092 }};	/*Useragent - Tiny Tiny RSS/1.5.5 (http://tt-rss.org/)*/
const Device UA623120 = {S42274, 41, { &P18254,&P7480,&P17470,&P18092 }};	/*Useragent - Tiny Tiny RSS/1.5.10 (http://tt-rss.org/)*/
const Device UA1963249 = {S42275, 41, { &P18254,&P7480,&P17470,&P18092 }};	/*Useragent - Tiny Tiny RSS/1.5.11 (http://tt-rss.org/)*/
const Device UA2151603 = {S42276, 40, { &P18254,&P7480,&P17470,&P18092 }};	/*Useragent - Tiny Tiny RSS/1.5.9 (http://tt-rss.org/)*/
const Device UA2262476 = {S42277, 40, { &P18254,&P7480,&P17470,&P18092 }};	/*Useragent - Tiny Tiny RSS/1.6.0 (http://tt-rss.org/)*/
const Device UA2308256 = {S42278, 42, { &P18254,&P7480,&P17470,&P18092 }};	/*Useragent - Tiny Tiny RSS/1.5.8.1 (http://tt-rss.org/)*/
const Device UA2360624 = {S42279, 36, { &P18254,&P7480,&P17470,&P18092 }};	/*Useragent - R6_VoteReader(www.radian6.com/votes)*/
const Device UA2362486 = {S42280, 40, { &P18254,&P7480,&P17470,&P18092 }};	/*Useragent - Tiny Tiny RSS/1.6.1 (http://tt-rss.org/)*/
const Device UA2461018 = {S42281, 40, { &P18254,&P7480,&P17470,&P18092 }};	/*Useragent - Tiny Tiny RSS/1.6.2 (http://tt-rss.org/)*/
const Device UA2487941 = {S42282, 81, { &P18254,&P7480,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; LinkChecker/8.4; +http://wummel.github.com/linkchecker/)*/
const Device UA2567488 = {S42283, 40, { &P18254,&P7480,&P17470,&P18092 }};	/*Useragent - Tiny Tiny RSS/1.7.2 (http://tt-rss.org/)*/
const Device UA2584963 = {S42284, 40, { &P18254,&P7480,&P17470,&P18092 }};	/*Useragent - Tiny Tiny RSS/1.7.4 (http://tt-rss.org/)*/
const Device UA2602585 = {S42285, 40, { &P18254,&P7480,&P17470,&P18092 }};	/*Useragent - Tiny Tiny RSS/1.7.5 (http://tt-rss.org/)*/
const Device UA2671762 = {S42286, 40, { &P18254,&P7480,&P17470,&P18092 }};	/*Useragent - Tiny Tiny RSS/1.7.8 (http://tt-rss.org/)*/
const Device UA2705800 = {S42287, 40, { &P18254,&P7480,&P17470,&P18092 }};	/*Useragent - Tiny Tiny RSS/1.7.6 (http://tt-rss.org/)*/
const Device UA2752956 = {S42288, 40, { &P18254,&P7480,&P17470,&P18092 }};	/*Useragent - Tiny Tiny RSS/1.7.9 (http://tt-rss.org/)*/
const Device UA2855770 = {S42289, 93, { &P18254,&P7480,&P17470,&P18092 }};	/*Useragent - Python-urllib/2.7 AppEngine-Google; (+http://code.google.com/appengine; appid: s~servicescdt)*/
const Device UA2870122 = {S42290, 38, { &P18254,&P7480,&P17470,&P18092 }};	/*Useragent - Tiny Tiny RSS/1.8 (http://tt-rss.org/)*/
const Device UA2029253 = {S42291, 141, { &P20943,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; htc_h2000c Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2205644 = {S42292, 141, { &P20943,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; HTC_H2000C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3033642 = {S42293, 133, { &P18680,&P18110,&P24334,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Safari/535.19 Silk-Accelerated=true*/
const Device UA24703 = {S42294, 174, { &P13648,&P5055,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaE7-00/014.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.8.10 3gpp-gba*/
const Device UA33300 = {S42295, 187, { &P13648,&P5055,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaE7-00/022.014; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.37 Mobile Safari/533.4 3gpp-gba*/
const Device UA2772844 = {S42296, 138, { &P17346,&P19826,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B144 Safari/8536.25*/
const Device UA2772847 = {S42297, 140, { &P17346,&P19826,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B329 Safari/8536.25*/
const Device UA2773053 = {S42298, 140, { &P17346,&P19826,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B146 Safari/8536.25*/
const Device UA2773491 = {S42299, 138, { &P17346,&P19826,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10B141 Safari/8536.25*/
const Device UA4564 = {S42300, 46, { &P18088,&P17017,&P18122,&P18092 }};	/*Useragent - Opera/8.01 (J2ME/MIDP; Opera Mini; en; U; ssr)*/
const Device UA112294 = {S42301, 41, { &P18088,&P17017,&P18122,&P18092 }};	/*Useragent - Opera/9.50 (J2ME/MIDP; Opera Mini; U; en)*/
const Device UA243021 = {S42302, 70, { &P18088,&P17017,&P18122,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini; U; en) Presto/2.8.119 Version/10.54*/
const Device UA623095 = {S42303, 70, { &P18088,&P17017,&P18122,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini; U; en) Presto/2.8.119 Version/11.10*/
const Device UA774156 = {S42304, 61, { &P18088,&P17017,&P18122,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini; U; en) Presto/2.8.119  ? ?*/
const Device UA2431806 = {S42305, 79, { &P18088,&P17017,&P18122,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini//28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2481235 = {S42306, 72, { &P18088,&P17017,&P18122,&P18092 }};	/*Useragent - 0 Opera/9.80 (J2ME/MIDP; Opera Mini; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2714975 = {S42307, 79, { &P18088,&P17017,&P18122,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini//29.3417; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2874179 = {S42308, 79, { &P18088,&P17017,&P18122,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini//30.3214; U; en) Presto/2.8.119 Version/11.10*/
const Device UA3095423 = {S42309, 79, { &P18088,&P17017,&P18122,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini//31.1395; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2033402 = {S42310, 143, { &P20399,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PantechP8010 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422033 = {S42311, 65, { &P20399,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; PantechP8010 Build/IMM76D)*/
const Device UA1978423 = {S42312, 132, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026906 = {S42313, 61, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P6200 Build/IMM76D)*/
const Device UA2223505 = {S42314, 133, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT-P6200L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2382640 = {S42315, 130, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2608452 = {S42316, 132, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-P6200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2612554 = {S42317, 62, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Vyrso/2.0.7 (Linux; U; Android 4.0.3; en-US; GT-P6200 Build/X)*/
const Device UA2613528 = {S42318, 130, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6200 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2614460 = {S42319, 153, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6200 Build/IMM76D; 1024*552) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2614919 = {S42320, 280, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6200 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 parameters/{scr=552_1024,cm=1,ql=h,imei=X,bm=1,eng=adrmini15,mdm=1,mdl=GT-P6200,ps=1} tiantian(securitypay,securityinstalled) tiantian(tenpay)*/
const Device UA2614928 = {S42321, 132, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6200 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615193 = {S42322, 73, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.0.4; en-US; GT-P6200 Build/IMM76D.DXLPB)*/
const Device UA2615909 = {S42323, 130, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6200 Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2616437 = {S42324, 62, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P6200L Build/IMM76D)*/
const Device UA2617331 = {S42325, 145, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2618962 = {S42326, 132, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6200) antivirusHttpClient/2.0 (KHTML, like Gecko) Version/Version 2.3/Antivirus pro*/
const Device UA2619681 = {S42327, 139, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6200 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2619782 = {S42328, 139, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2620091 = {S42329, 91, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6200 Build/IMM76D) Daily Mail Online/1.8.3*/
const Device UA2620420 = {S42330, 73, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.7 (Linux; U; Android 4.0.4; en-GB; GT-P6200 Build/IMM76D.DDLP5)*/
const Device UA2622089 = {S42331, 132, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GT-P6200 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622288 = {S42332, 75, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.0.4; ru-RU; GT-P6200 Build/IMM76D.XXLQ1)*/
const Device UA2627281 = {S42333, 74, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.12 (Linux; U; Android 4.0.4; en-US; GT-P6200 Build/IMM76D.XXLQ3)*/
const Device UA2628848 = {S42334, 264, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6200L Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 parameters/{scr=552_1024,cm=1,ql=l,imei=X,bm=1,eng=adrmini15,mdm=1,mdl=GT-P6200L,ps=1} tiantian(securitypay) tiantian(tenpay)*/
const Device UA2629558 = {S42335, 134, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6200 Build/REL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2631540 = {S42336, 131, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-; GT-P6200L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2637374 = {S42337, 151, { &P15985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6200 Build/IMM76D) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.2 Version/4.0 Mobile Safari/530.17*/
const Device UA2973480 = {S42338, 138, { &P15575,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974037 = {S42339, 137, { &P15575,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974473 = {S42340, 137, { &P15575,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974612 = {S42341, 138, { &P15575,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100T Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974804 = {S42342, 138, { &P15575,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100M Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976025 = {S42343, 138, { &P15575,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976113 = {S42344, 138, { &P15575,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100T Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA48447 = {S42345, 151, { &P12709,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; SonyEricssonMK16a Build/4.0.A.2.335) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48448 = {S42346, 139, { &P12709,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; MK16a Build/4.0.A.2.335) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49979 = {S42347, 152, { &P12709,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonMK16a Build/4.0.1.A.0.24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA356332 = {S42348, 152, { &P12709,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; SonyEricssonMK16a Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA703392 = {S42349, 141, { &P12709,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; MK16a Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA758694 = {S42350, 153, { &P12709,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; SonyEricssonMK16a Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA789441 = {S42351, 140, { &P12709,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; MK16a Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA794659 = {S42352, 140, { &P12709,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MK16a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA804215 = {S42353, 152, { &P12709,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonMK16a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA829572 = {S42354, 64, { &P12709,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; MK16a Build/4.0.2.A.0.58)*/
const Device UA2019050 = {S42355, 63, { &P12709,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; MK16a Build/4.0.A.2.335)*/
const Device UA2556275 = {S42356, 138, { &P19911,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E970 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566880 = {S42357, 143, { &P19911,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E970/E970 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2667308 = {S42358, 60, { &P19911,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E970 Build/JZO54K)*/
const Device UA2676592 = {S42359, 146, { &P19911,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E970/E97020j Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA109436 = {S42360, 139, { &P17968,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; generic Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA250816 = {S42361, 143, { &P17968,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; xpndr_ihome Build/DRD35) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA782529 = {S42362, 137, { &P17968,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; eeepc Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2018061 = {S42363, 48, { &P17968,&P5062,&P3441,&P18092 }};	/*Useragent - CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 1.6)*/
const Device UA216786 = {S42364, 119, { &P17595,&P17295,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA218591 = {S42365, 119, { &P17595,&P17295,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_2) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA219227 = {S42366, 119, { &P17595,&P17295,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA326982 = {S42367, 119, { &P17595,&P17295,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA327035 = {S42368, 119, { &P17595,&P17295,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_5) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA641720 = {S42369, 119, { &P17595,&P17295,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA668581 = {S42370, 119, { &P17595,&P17295,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA715299 = {S42371, 119, { &P17595,&P17295,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30*/
const Device UA2674801 = {S42372, 129, { &P17595,&P17295,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/12.0.648.205 Safari/534.16*/
const Device UA2707706 = {S42373, 129, { &P17595,&P17295,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/12.0.648.205 Safari/534.16*/
const Device UA2607499 = {S42374, 138, { &P20991,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; ME371MG Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2607503 = {S42375, 60, { &P20991,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; ME371MG Build/JZO54K)*/
const Device UA2607506 = {S42376, 131, { &P20991,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ME371MG Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2769909 = {S42377, 106, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.246 Mobile Safari/537.35+*/
const Device UA2799179 = {S42378, 106, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.402 Mobile Safari/537.35+*/
const Device UA2799398 = {S42379, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1337 Mobile Safari/537.35+*/
const Device UA2870129 = {S42380, 106, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.228 Mobile Safari/537.35+*/
const Device UA2873993 = {S42381, 106, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.444 Mobile Safari/537.35+*/
const Device UA2876607 = {S42382, 106, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.483 Mobile Safari/537.35+*/
const Device UA2879108 = {S42383, 105, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.93 Mobile Safari/537.35+*/
const Device UA2879715 = {S42384, 106, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.341 Mobile Safari/537.35+*/
const Device UA2885275 = {S42385, 106, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.468 Mobile Safari/537.35+*/
const Device UA2887023 = {S42386, 106, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.519 Mobile Safari/537.35+*/
const Device UA2888654 = {S42387, 106, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.552 Mobile Safari/537.35+*/
const Device UA2945533 = {S42388, 106, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.663 Mobile Safari/537.35+*/
const Device UA2948014 = {S42389, 106, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.720 Mobile Safari/537.35+*/
const Device UA2948413 = {S42390, 106, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.783 Mobile Safari/537.35+*/
const Device UA2972881 = {S42391, 106, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.810 Mobile Safari/537.35+*/
const Device UA2973806 = {S42392, 106, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.852 Mobile Safari/537.35+*/
const Device UA2976829 = {S42393, 106, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.936 Mobile Safari/537.35+*/
const Device UA3024923 = {S42394, 106, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.981 Mobile Safari/537.35+*/
const Device UA3025635 = {S42395, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1023 Mobile Safari/537.35+*/
const Device UA3028257 = {S42396, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1077 Mobile Safari/537.35+*/
const Device UA3029783 = {S42397, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1047 Mobile Safari/537.35+*/
const Device UA3030867 = {S42398, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1197 Mobile Safari/537.35+*/
const Device UA3036012 = {S42399, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1113 Mobile Safari/537.35+*/
const Device UA3036054 = {S42400, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1155 Mobile Safari/537.35+*/
const Device UA3036327 = {S42401, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1425 Mobile Safari/537.35+*/
const Device UA3036422 = {S42402, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1323 Mobile Safari/537.35+*/
const Device UA3037095 = {S42403, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1371 Mobile Safari/537.35+*/
const Device UA3039379 = {S42404, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1443 Mobile Safari/537.35+*/
const Device UA3050322 = {S42405, 105, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.42 Mobile Safari/537.35+*/
const Device UA3061049 = {S42406, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1157 Mobile Safari/537.35+*/
const Device UA3066106 = {S42407, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1521 Mobile Safari/537.35+*/
const Device UA3067696 = {S42408, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1671 Mobile Safari/537.35+*/
const Device UA3068144 = {S42409, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1629 Mobile Safari/537.35+*/
const Device UA3068184 = {S42410, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1581 Mobile Safari/537.35+*/
const Device UA3080790 = {S42411, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1725 Mobile Safari/537.35+*/
const Device UA3080879 = {S42412, 106, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.1.200 Mobile Safari/537.35+*/
const Device UA3082206 = {S42413, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1719 Mobile Safari/537.35+*/
const Device UA3084015 = {S42414, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1707 Mobile Safari/537.35+*/
const Device UA3092465 = {S42415, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1065 Mobile Safari/537.35+*/
const Device UA3096093 = {S42416, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1743 Mobile Safari/537.35+*/
const Device UA3096166 = {S42417, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1737 Mobile Safari/537.35+*/
const Device UA3096908 = {S42418, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1731 Mobile Safari/537.35+*/
const Device UA3109173 = {S42419, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1445 Mobile Safari/537.35+*/
const Device UA3110073 = {S42420, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1761 Mobile Safari/537.35+*/
const Device UA3113578 = {S42421, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1767 Mobile Safari/537.35+*/
const Device UA3127913 = {S42422, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1233 Mobile Safari/537.35+*/
const Device UA3129719 = {S42423, 106, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.1.536 Mobile Safari/537.35+*/
const Device UA3131726 = {S42424, 107, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1779 Mobile Safari/537.35+*/
const Device UA3131811 = {S42425, 106, { &P19794,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Touch) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.1.641 Mobile Safari/537.35+*/
const Device UA924757 = {S42426, 62, { &P17976,&P17979,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:17.0) Gecko/17.0 Firefox/17.0*/
const Device UA2028816 = {S42427, 68, { &P17976,&P17979,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:17.0a1) Gecko/17.0a1 Firefox/17.0a1*/
const Device UA2228367 = {S42428, 68, { &P17976,&P17979,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:17.0a2) Gecko/17.0a2 Firefox/17.0a2*/
const Device UA2368647 = {S42429, 72, { &P17976,&P17979,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:17.0) Gecko/17.0 Firefox/17.0,gzip(gfe)*/
const Device UA605613 = {S42430, 154, { &P14877,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9000 Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA677947 = {S42431, 154, { &P14877,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9000 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031208 = {S42432, 158, { &P14877,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9000 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032260 = {S42433, 164, { &P14877,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9000 Build/GINGERBREAD; CyanogenMod-RemICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA624860 = {S42434, 54, { &P23144,&P18570,&P17596,&P18092 }};	/*Useragent - MobileSafari/7534.48.3 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA70195 = {S42435, 155, { &P15679,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; USCCADR6325US Carrier ID 45 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70186 = {S42436, 135, { &P15679,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6325 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA637173 = {S42437, 59, { &P15679,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; ADR6325 Build/GRJ22)*/
const Device UA2973890 = {S42438, 130, { &P18437,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8020 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974105 = {S42439, 130, { &P18437,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8005 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974158 = {S42440, 130, { &P18437,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974303 = {S42441, 130, { &P18437,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8020 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2975955 = {S42442, 130, { &P18437,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA20526 = {S42443, 125, { &P12108,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia7230-1c/5.0 (10.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA45128 = {S42444, 71, { &P12108,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia7230/5.0 (10.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7230*/
const Device UA64378 = {S42445, 122, { &P12108,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia7230/5.0 (06.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA16613 = {S42446, 139, { &P17108,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; Inspire 4G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42580 = {S42447, 141, { &P17108,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC-A9192/1.0Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42602 = {S42448, 141, { &P17108,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;HTC-A9192/1.0 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49088 = {S42449, 141, { &P17108,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC-A9192/1.0 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA54416 = {S42450, 142, { &P17108,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC-A9192/1.0 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70448 = {S42451, 141, { &P17108,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; HTC-A9192/1.0 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76603 = {S42452, 141, { &P17108,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC-A9192/1.0 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA568620 = {S42453, 139, { &P17108,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-mx; Inspire HD Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024638 = {S42454, 63, { &P17108,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; Inspire HD Build/FRG83D)*/
const Device UA10297 = {S42455, 145, { &P14837,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I5800 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA156929 = {S42456, 67, { &P14837,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I5800 Build/ECLAIR)*/
const Device UA2030714 = {S42457, 152, { &P14837,&P5477,&P4861,&P18092 }};	/*Useragent - 2.1 GT-I5800---Mozilla/5.0 (Linux; U; Android 2.1; xx-xx; GT-I5800 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2672633 = {S42458, 88, { &P17595,&P17294,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.4; zh-CN; rv:1.9.0.17) Gecko/X Firefox/3.0.17*/
const Device UA316940 = {S42459, 189, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Acoo Browser; GTB5; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Maxthon; InfoPath.1; .NET CLR 3.5.30729; .NET CLR 3.0.30618)*/
const Device UA502123 = {S42460, 146, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Maxthon; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.04506; Media Center PC 5.0; InfoPath.1)*/
const Device UA518251 = {S42461, 83, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.0; SV1; .NET CLR 1.1.4322; MAXTHON)*/
const Device UA887123 = {S42462, 216, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Maxthon; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; .NET CLR 1.1.4322; InfoPath.2)*/
const Device UA1872958 = {S42463, 146, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Maxthon; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET CLR 3.0.30618)*/
const Device UA1990450 = {S42464, 72, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; Maxthon)*/
const Device UA1992234 = {S42465, 72, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 6.0; Maxthon)*/
const Device UA2165875 = {S42466, 72, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Maxthon)*/
const Device UA2168020 = {S42467, 72, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; Maxthon)*/
const Device UA2221925 = {S42468, 72, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Maxthon)*/
const Device UA2321640 = {S42469, 83, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; SV1; .NET CLR 1.1.4322; MAXTHON)*/
const Device UA2408467 = {S42470, 146, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Maxthon; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.04506; Media Center PC 5.0; InfoPath.1)*/
const Device UA2408617 = {S42471, 146, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Maxthon; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; .NET CLR 1.1.4322)*/
const Device UA2447551 = {S42472, 160, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAXTHON)*/
const Device UA2473512 = {S42473, 72, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; MAXTHON)*/
const Device UA2574798 = {S42474, 106, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)*/
const Device UA2686047 = {S42475, 87, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; Avant Browser; Maxthon)*/
const Device UA2686102 = {S42476, 77, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; SV1; Maxthon)*/
const Device UA2742945 = {S42477, 81, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Maxthon; Maxthon)*/
const Device UA2747406 = {S42478, 106, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)*/
const Device UA2775718 = {S42479, 77, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Maxthon; SV1)*/
const Device UA2787362 = {S42480, 81, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Maxthon; Maxthon)*/
const Device UA2788687 = {S42481, 77, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; SV1; Maxthon)*/
const Device UA2816924 = {S42482, 87, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Avant Browser; Maxthon)*/
const Device UA2836444 = {S42483, 72, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; MAXTHON)*/
const Device UA2839416 = {S42484, 87, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; Maxthon; Avant Browser)*/
const Device UA2889111 = {S42485, 81, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; Maxthon; Maxthon)*/
const Device UA2949954 = {S42486, 77, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; SV1; Maxthon)*/
const Device UA2950151 = {S42487, 87, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Avant Browser; Maxthon)*/
const Device UA2980988 = {S42488, 81, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 6.0; Maxthon; Maxthon)*/
const Device UA3047153 = {S42489, 107, { &P15364,&P17285,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.0; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)*/
const Device UA2170824 = {S42490, 140, { &P19902,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-R530M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2403568 = {S42491, 62, { &P19902,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-R530M Build/IMM76D)*/
const Device UA2695483 = {S42492, 138, { &P19902,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SCH-R530M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA34957 = {S42493, 73, { &P14848,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaX3-00/5.0 (11.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax3-00*/
const Device UA54008 = {S42494, 123, { &P14848,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaX3-00/5.0 (04.11) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA359164 = {S42495, 132, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/17.0.963.12) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.12 Safari/535.11*/
const Device UA398520 = {S42496, 132, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/17.0.963.46) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.46 Safari/535.11*/
const Device UA401346 = {S42497, 132, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/17.0.963.56) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA401373 = {S42498, 88, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/17.0.963.56)*/
const Device UA401378 = {S42499, 95, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/17.0.963.56)*/
const Device UA401405 = {S42500, 125, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/17.0.963.56) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA412760 = {S42501, 132, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/17.0.963.65) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.65 Safari/535.11*/
const Device UA412948 = {S42502, 125, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/17.0.963.66) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11*/
const Device UA413008 = {S42503, 132, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/17.0.963.66) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11*/
const Device UA427909 = {S42504, 132, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/17.0.963.78) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.78 Safari/535.11*/
const Device UA427965 = {S42505, 125, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/17.0.963.78) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.78 Safari/535.11*/
const Device UA428600 = {S42506, 125, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/17.0.963.79) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11*/
const Device UA428620 = {S42507, 132, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/17.0.963.79) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11*/
const Device UA428728 = {S42508, 196, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/17.0.963.79; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA458302 = {S42509, 132, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/17.0.963.83) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.83 Safari/535.11*/
const Device UA589517 = {S42510, 230, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/17.0.963.46; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; MS-RTC LM 8; AskTbORJ/5.13.1.18107)*/
const Device UA590496 = {S42511, 193, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/17.0.963.56; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)*/
const Device UA603156 = {S42512, 125, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/17.0.963.83) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.83 Safari/535.11*/
const Device UA900862 = {S42513, 281, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/17.0.963.78; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8; Microsoft Outlook 14.0.6025; ms-office; MSOffice 14)*/
const Device UA923754 = {S42514, 125, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/17.0.963.46) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.46 Safari/535.11*/
const Device UA2155364 = {S42515, 267, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/17.0.963.46; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; Tablet PC 2.0; MS-RTC LM 8; QG Managed)*/
const Device UA2314914 = {S42516, 182, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/17.0.963.65; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2333221 = {S42517, 212, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MDDR; MS-RTC LM 8; .NET4.0C; BRI/2; chromeframe/17.0.963.46)*/
const Device UA2495428 = {S42518, 203, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/17.0.963.65; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2809293 = {S42519, 207, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/17.0.963.78; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; MS-RTC LM 8; InfoPath.3)*/
const Device UA2861762 = {S42520, 260, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/17.0.963.78; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8; Microsoft Outlook 14.0.6131; ms-office; MSOffice 14)*/
const Device UA3047813 = {S42521, 96, { &P15364,&P7133,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/17.0.963.78)*/
const Device UA2436652 = {S42522, 116, { &P17968,&P17979,&P18520,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/7.28879/27.1662; U; en-us) Presto/2.8.119 Version/11.10 UCBrowser/8.5.3.246/145/355/*/
const Device UA2558536 = {S42523, 130, { &P19783,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ME172V Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2665359 = {S42524, 59, { &P19783,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ME172V Build/JRO03H)*/
const Device UA2741736 = {S42525, 137, { &P15575,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753684 = {S42526, 138, { &P15575,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2756399 = {S42527, 138, { &P15575,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2756411 = {S42528, 138, { &P15575,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783540 = {S42529, 137, { &P15575,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9100 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA192626 = {S42530, 118, { &P17595,&P17295,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_0) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24*/
const Device UA195666 = {S42531, 118, { &P17595,&P17295,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA316292 = {S42532, 118, { &P17595,&P17295,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA316507 = {S42533, 118, { &P17595,&P17295,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_2) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24*/
const Device UA536156 = {S42534, 118, { &P17595,&P17295,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA539088 = {S42535, 118, { &P17595,&P17295,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24*/
const Device UA678590 = {S42536, 118, { &P17595,&P17295,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24*/
const Device UA685093 = {S42537, 118, { &P17595,&P17295,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_5) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.57 Safari/534.24*/
const Device UA903994 = {S42538, 127, { &P17595,&P17295,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.20 (KHTML, like Gecko) Chrome/11.0.672.2 Safari/534.20*/
const Device UA2375762 = {S42539, 118, { &P17595,&P17295,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.24 (KHTML	 like Gecko) Chrome/11.0.696.65 Safari/534.24*/
const Device UA2674606 = {S42540, 129, { &P17595,&P17295,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/11.0.648.205 Safari/534.16*/
const Device UA2718646 = {S42541, 129, { &P17595,&P17295,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/11.0.648.205 Safari/534.16*/
const Device UA180538 = {S42542, 125, { &P15767,&P17090,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 Twitter for iPad*/
const Device UA181987 = {S42543, 125, { &P15767,&P17090,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190 Twitter for iPad*/
const Device UA182911 = {S42544, 125, { &P15767,&P17090,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 Twitter for iPad*/
const Device UA191862 = {S42545, 125, { &P15767,&P17090,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 Twitter for iPad*/
const Device UA191933 = {S42546, 125, { &P15767,&P17090,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Twitter for iPad*/
const Device UA222940 = {S42547, 125, { &P15767,&P17090,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2 Twitter for iPad*/
const Device UA233963 = {S42548, 125, { &P15767,&P17090,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Twitter for iPad*/
const Device UA309638 = {S42549, 125, { &P15767,&P17090,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F191 Twitter for iPad*/
const Device UA312737 = {S42550, 125, { &P15767,&P17090,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H8 Twitter for iPad*/
const Device UA11052 = {S42551, 141, { &P14202,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; sv-se; E15i Build/1.3.A.0.62) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA15075 = {S42552, 141, { &P14202,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; E15i Build/1.3.A.0.50) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA15081 = {S42553, 153, { &P14202,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; SonyEricssonE15i Build/1.3.A.0.58) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA15085 = {S42554, 154, { &P14202,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-ie; SonyEricssonE15iv Build/1.3.A.0.50) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA15106 = {S42555, 141, { &P14202,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; E15i Build/1.3.A.0.58) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA15128 = {S42556, 155, { &P14202,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; fr-fr; SonyEricssonE15i-o Build/1.3.A.0.50) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA49082 = {S42557, 153, { &P14202,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; es-mx; SonyEricssonE15a Build/1.3.A.0.50) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA49084 = {S42558, 141, { &P14202,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; es-mx; E15a Build/1.3.A.0.50) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA72198 = {S42559, 141, { &P14202,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; E15a Build/1.3.A.0.68) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA253878 = {S42560, 74, { &P14202,&P5062,&P3441,&P18092 }};	/*Useragent - PVPlayer SonyEricssonE15i CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 1.6)*/
const Device UA2203747 = {S42561, 112, { &P15785,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia201/11.56; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA1996264 = {S42562, 155, { &P18283,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996834 = {S42563, 155, { &P18283,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1998006 = {S42564, 155, { &P18283,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1999862 = {S42565, 155, { &P18283,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2004292 = {S42566, 155, { &P18283,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2006967 = {S42567, 155, { &P18283,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2197402 = {S42568, 155, { &P18283,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2198086 = {S42569, 155, { &P18283,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2198991 = {S42570, 155, { &P18283,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2235150 = {S42571, 159, { &P18283,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I535 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2238411 = {S42572, 155, { &P18283,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258129 = {S42573, 159, { &P18283,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T999 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2504587 = {S42574, 167, { &P18283,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-t999 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985365 = {S42575, 153, { &P18283,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; SC-06D Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2974065 = {S42576, 137, { &P18645,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L710 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974297 = {S42577, 137, { &P18645,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9305 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974811 = {S42578, 145, { &P18645,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I747 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974890 = {S42579, 136, { &P18645,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I747 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975029 = {S42580, 138, { &P18645,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530U Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021501 = {S42581, 137, { &P18645,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-I747 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022805 = {S42582, 138, { &P18645,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530C Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022819 = {S42583, 145, { &P18645,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SAMSUNG-SGH-I747 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA91108 = {S42584, 96, { &P15364,&P7133,&P21901,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/15.0.874.121)*/
const Device UA252119 = {S42585, 208, { &P15364,&P7133,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/15.0.874.106; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; MS-RTC LM 8)*/
const Device UA252152 = {S42586, 132, { &P15364,&P7133,&P21901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/15.0.874.106) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.106 Safari/535.2*/
const Device UA252190 = {S42587, 125, { &P15364,&P7133,&P21901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/15.0.874.106) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.106 Safari/535.2*/
const Device UA252304 = {S42588, 89, { &P15364,&P7133,&P21901,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/15.0.874.106)*/
const Device UA257424 = {S42589, 216, { &P15364,&P7133,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/15.0.874.121; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.2)*/
const Device UA632992 = {S42590, 125, { &P15364,&P7133,&P21901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/15.0.874.121) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA646377 = {S42591, 132, { &P15364,&P7133,&P21901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/15.0.874.121) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA686989 = {S42592, 222, { &P15364,&P7133,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/15.0.874.121; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)*/
const Device UA728667 = {S42593, 132, { &P15364,&P7133,&P21901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/15.0.874.120) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.120 Safari/535.2*/
const Device UA874345 = {S42594, 125, { &P15364,&P7133,&P21901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/15.0.874.120) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.120 Safari/535.2*/
const Device UA923372 = {S42595, 175, { &P15364,&P7133,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/15.0.874.121; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2)*/
const Device UA2245920 = {S42596, 222, { &P15364,&P7133,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/15.0.874.121; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)*/
const Device UA2247364 = {S42597, 216, { &P15364,&P7133,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/15.0.874.121; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C; .NET4.0E)*/
const Device UA2294449 = {S42598, 128, { &P15364,&P7133,&P21901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/15.0.849.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/15.0.849.0 Safari/535.1*/
const Device UA2295372 = {S42599, 121, { &P15364,&P7133,&P21901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/15.0.854.0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.854.0 Safari/535.2*/
const Device UA2473325 = {S42600, 219, { &P15364,&P7133,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/15.0.874.120; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.2; Zune 4.7)*/
const Device UA2721035 = {S42601, 243, { &P15364,&P7133,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/15.0.874.121; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.2; Tablet PC 2.0)*/
const Device UA2360578 = {S42602, 124, { &P15364,&P7133,&P22463,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11*/
const Device UA2360820 = {S42603, 117, { &P15364,&P7133,&P22463,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11*/
const Device UA1876974 = {S42604, 100, { &P15364,&P16110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.60 Safari/534.24*/
const Device UA1879568 = {S42605, 107, { &P15364,&P16110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.60 Safari/534.24*/
const Device UA2301255 = {S42606, 108, { &P15364,&P16110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.2) AppleWebKit/534.23 (KHTML, like Gecko) Chrome/11.0.686.3 Safari/534.23*/
const Device UA2370816 = {S42607, 118, { &P15364,&P16110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/534.20 (KHTML, like Gecko) Chrome/11.0.672.2 Safari/534.20*/
const Device UA2363192 = {S42608, 57, { &P15364,&P5690,&P24163,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; Edition IBIS; Trident/5.0)*/
const Device UA909496 = {S42609, 139, { &P18633,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ke; U8185 Build/HuaweiU8185) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027796 = {S42610, 63, { &P18633,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; U8185 Build/HuaweiU8185)*/
const Device UA2853025 = {S42611, 147, { &P18633,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U8185 Build/r0_dr-brand_hwu8185) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA827227 = {S42612, 150, { &P19875,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I827 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA924085 = {S42613, 143, { &P19875,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; SGH-I827D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2317812 = {S42614, 74, { &P19875,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SAMSUNG-SGH-I827 Build/GINGERBREAD)*/
const Device UA2789365 = {S42615, 67, { &P19875,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-I827D Build/GINGERBREAD)*/
const Device UA925405 = {S42616, 137, { &P17595,&P18207,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA2017342 = {S42617, 137, { &P17595,&P18207,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA2202826 = {S42618, 137, { &P17595,&P18207,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA2556569 = {S42619, 137, { &P17595,&P18207,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA2781348 = {S42620, 137, { &P17595,&P18207,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA3091820 = {S42621, 137, { &P17595,&P18207,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA3064871 = {S42622, 136, { &P22594,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3064659 = {S42623, 136, { &P22585,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L720 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2945963 = {S42624, 90, { &P18518,&P5567,&P18525,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/6.0.3/27.2338; U; en) Presto/2.8.119 320X240 Samsung SCH-U380*/
const Device UA51874 = {S42625, 141, { &P15819,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; USCCADR6285US Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA576291 = {S42626, 138, { &P15819,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Hero S Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA646780 = {S42627, 62, { &P15819,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Hero S Build/GRJ22)*/
const Device UA2814686 = {S42628, 131, { &P15819,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Hero S Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2772432 = {S42629, 132, { &P15767,&P18117,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9B176 Safari/7534.48.3*/
const Device UA2772850 = {S42630, 134, { &P15767,&P18117,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9B206 Safari/7534.48.3*/
const Device UA2938822 = {S42631, 132, { &P15767,&P18117,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9A405 Safari/7534.48.3*/
const Device UA2795390 = {S42632, 129, { &P23203,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; A1-810 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2879625 = {S42633, 58, { &P23203,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; A1-810 Build/JDQ39)*/
const Device UA3124349 = {S42634, 49, { &P23203,&P18598,&P19802,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android 4.2.2) acer_A1-810*/
const Device UA7134 = {S42635, 73, { &P14669,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (04.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-00*/
const Device UA33448 = {S42636, 123, { &P14669,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (08.25) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA2984890 = {S42637, 186, { &P14669,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA2606933 = {S42638, 60, { &P15364,&P18110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux x86_64) Presto/2.12.388 Version/12.15*/
const Device UA2675063 = {S42639, 71, { &P15364,&P18110,&P22577,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.15*/
const Device UA2751259 = {S42640, 63, { &P15364,&P18110,&P22577,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; X11; Linux i686) Opera 12.15*/
const Device UA2781338 = {S42641, 68, { &P15364,&P18110,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686) Presto/2.12.388 Version/12.15,gzip(gfe)*/
const Device UA72779 = {S42642, 131, { &P12628,&P6374,&P4418,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9350; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.261 Mobile Safari/534.11+*/
const Device UA2974150 = {S42643, 137, { &P18595,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974320 = {S42644, 137, { &P18595,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2488358 = {S42645, 116, { &P17968,&P17979,&P18527,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/7.28879/27.1662; U; en-us) Presto/2.8.119 Version/11.10 UCBrowser/8.6.1.262/145/355/*/
const Device UA98853 = {S42646, 66, { &P15364,&P7127,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; MyIE2; Maxthon)*/
const Device UA2157614 = {S42647, 72, { &P15364,&P7127,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Maxthon)*/
const Device UA2163984 = {S42648, 72, { &P15364,&P7127,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Maxthon)*/
const Device UA2165286 = {S42649, 72, { &P15364,&P7127,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 5.0; Maxthon)*/
const Device UA2166006 = {S42650, 72, { &P15364,&P7127,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Maxthon)*/
const Device UA2166431 = {S42651, 72, { &P15364,&P7127,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 5.0; Maxthon)*/
const Device UA2167192 = {S42652, 72, { &P15364,&P7127,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Maxthon)*/
const Device UA2408621 = {S42653, 60, { &P15364,&P7127,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; Maxthon)*/
const Device UA2686649 = {S42654, 87, { &P15364,&P7127,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Avant Browser; Maxthon)*/
const Device UA2725624 = {S42655, 81, { &P15364,&P7127,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Maxthon; Maxthon)*/
const Device UA2743566 = {S42656, 77, { &P15364,&P7127,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Maxthon; SV1)*/
const Device UA2775837 = {S42657, 87, { &P15364,&P7127,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Avant Browser; Maxthon)*/
const Device UA2775980 = {S42658, 106, { &P15364,&P7127,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)*/
const Device UA2776834 = {S42659, 87, { &P15364,&P7127,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Avant Browser; Maxthon)*/
const Device UA2788265 = {S42660, 77, { &P15364,&P7127,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; SV1; Maxthon)*/
const Device UA2824876 = {S42661, 77, { &P15364,&P7127,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; SV1; Maxthon)*/
const Device UA2827577 = {S42662, 87, { &P15364,&P7127,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Maxthon; Avant Browser)*/
const Device UA2838272 = {S42663, 81, { &P15364,&P7127,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Maxthon; Maxthon)*/
const Device UA2980264 = {S42664, 77, { &P15364,&P7127,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Maxthon; SV1)*/
const Device UA2981965 = {S42665, 81, { &P15364,&P7127,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Maxthon; Maxthon)*/
const Device UA2982793 = {S42666, 106, { &P15364,&P7127,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)*/
const Device UA2983251 = {S42667, 81, { &P15364,&P7127,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Maxthon; Maxthon)*/
const Device UA2901022 = {S42668, 88, { &P15364,&P5690,&P24342,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; QQBrowser/7.3.11251.400)*/
const Device UA2945688 = {S42669, 93, { &P15364,&P5690,&P24342,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; QQBrowser/7.3.11251.400)*/
const Device UA2963715 = {S42670, 80, { &P15364,&P5690,&P24342,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QQBrowser/7.3.11251.400)*/
const Device UA2965112 = {S42671, 88, { &P15364,&P5690,&P24342,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; QQBrowser/7.3.11251.400)*/
const Device UA2570872 = {S42672, 140, { &P18448,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST26i Build/11.2.A.0.21) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2667427 = {S42673, 140, { &P18448,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST26a Build/11.2.A.0.21) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA29897 = {S42674, 99, { &P17346,&P17084,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko)*/
const Device UA90031 = {S42675, 102, { &P17346,&P17084,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en_us) AppleWebKit/525.18.1 (KHTML, like Gecko)*/
const Device UA133917 = {S42676, 112, { &P17346,&P17084,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B117*/
const Device UA143323 = {S42677, 112, { &P17346,&P17084,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B118*/
const Device UA624415 = {S42678, 125, { &P17346,&P17084,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B118 Safari/532.9*/
const Device UA655746 = {S42679, 246, { &P17346,&P17084,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_1 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod4,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.1;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBSF/2.0]*/
const Device UA727455 = {S42680, 140, { &P17346,&P17084,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B118 iPod4,1 BingWeb/3.03.1428.X*/
const Device UA1846804 = {S42681, 125, { &P17346,&P17084,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B117 Safari/532.9*/
const Device UA2162088 = {S42682, 140, { &P17346,&P17084,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B117 iPod3,1 BingWeb/3.03.1428.X*/
const Device UA2674428 = {S42683, 110, { &P17346,&P17084,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 41 like Mac OS X; en-US) AppleWebKit/532.9 (KHTML like Gecko) Mobile/8B118*/
const Device UA67684 = {S42684, 156, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v1.02.09-2010.11.30_English; en-us; MID Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA67713 = {S42685, 137, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; en-us; MID 1 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA69022 = {S42686, 129, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0.5; ru-ru Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA69277 = {S42687, 155, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v1.02_15.14-MC_US@20110514; en-us; MID Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA69292 = {S42688, 155, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v1.02_15.14-MC_US@20110503; en-us; MID Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA116591 = {S42689, 128, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; en- Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA142216 = {S42690, 130, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; md-us Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA153358 = {S42691, 130, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; en-us Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA167563 = {S42692, 156, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android S20_09_30_B_PD_INX7E_ENG_6410POP; en-us Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA296384 = {S42693, 156, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android S20_09_30_B_PD_INX7E_ENG_6410POP; md-us Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA301854 = {S42694, 130, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; en-ca Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA306009 = {S42695, 157, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v1.02_14.13-MC_EN-2011.01.13; en-us; MID Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA314022 = {S42696, 140, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; en-us;  Build/v1.2.1-17197fd) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA498137 = {S42697, 130, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; nl-nl Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA507557 = {S42698, 153, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android SL20_X_B_PD_INX7E_ENG_6410POP; nl-nl Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA513839 = {S42699, 130, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; es-us Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA518987 = {S42700, 130, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; es-es Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA633443 = {S42701, 138, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; en-us;  Build/RC18-e904b9c) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA747705 = {S42702, 130, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; ar-eg Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA800141 = {S42703, 130, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; fr-fr Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1858702 = {S42704, 130, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; ru-ru Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1878908 = {S42705, 130, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; en-gb Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1902033 = {S42706, 130, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; en-au Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018490 = {S42707, 51, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android Eclair)*/
const Device UA2018508 = {S42708, 132, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0.6_b1; en-us Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018841 = {S42709, 132, { &P17968,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0.6_b1; ru-ru Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA102729 = {S42710, 91, { &P18254,&P18110,&P17825,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/3.5; Linux) KHTML/3.5.5 (like Gecko) (Exabot-Thumbnails)*/
const Device UA164136 = {S42711, 227, { &P15364,&P7133,&P21926,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MS-RTC LM 8; InfoPath.3; .NET4.0C; .NET4.0E) chromeframe/8.0.552.224*/
const Device UA496204 = {S42712, 143, { &P15364,&P7133,&P21926,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10 chromeframe/8.0.552.224*/
const Device UA672157 = {S42713, 229, { &P15364,&P7133,&P21926,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; Tablet PC 2.0) chromeframe/8.0.552.224*/
const Device UA739568 = {S42714, 94, { &P15364,&P7133,&P21926,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0) chromeframe/8.0.552.224*/
const Device UA882815 = {S42715, 87, { &P15364,&P7133,&P21926,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0) chromeframe/8.0.552.224*/
const Device UA896868 = {S42716, 185, { &P15364,&P7133,&P21926,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) chromeframe/8.0.552.237*/
const Device UA2639383 = {S42717, 214, { &P15364,&P7133,&P21926,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; .NET4.0E) chromeframe/8.0.552.237*/
const Device UA2773443 = {S42718, 227, { &P15364,&P7133,&P21926,&P18092 }};	/*Useragent - 'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media enter PC 6.0; MS-RTC LM 8; InfoPath.3; .NET4.0C; .NET4.0E) chromeframe/8.0.552.224*/
const Device UA2386951 = {S42719, 35, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android 4.2)*/
const Device UA2390392 = {S42720, 37, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android 4.2.1)*/
const Device UA2407285 = {S42721, 137, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; toroplus Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2531399 = {S42722, 37, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android 4.2.2)*/
const Device UA2600563 = {S42723, 154, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Android SDK built for x86 Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2660594 = {S42724, 137, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; iphone 5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2661915 = {S42725, 138, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; en-us; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2698373 = {S42726, 138, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2704292 = {S42727, 148, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2.2.2.2.2.2.2.2; en-us; Build/OPENMASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2709550 = {S42728, 144, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; Aveo NOTE II+ Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2716928 = {S42729, 130, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2727809 = {S42730, 128, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734029 = {S42731, 136, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; PAD-2 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734530 = {S42732, 148, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; e2001_v89_gq2008s Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2735048 = {S42733, 148, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; e2001_v89_jdt1qhd Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2735352 = {S42734, 149, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; e2001_v89_jbl1a698 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2736772 = {S42735, 143, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; v89_jbl1a698 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2749436 = {S42736, 137, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; en-us; iPhone Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2750784 = {S42737, 138, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; vi-vn; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2762820 = {S42738, 135, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; en-us; iPhone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2768043 = {S42739, 150, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; e2001_v89_gq2002qhd Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2790705 = {S42740, 136, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; en-us; iPhone5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2792268 = {S42741, 147, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Android SDK built for x86 Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2796215 = {S42742, 155, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; e2001_v89_jbl1a668_fwvga Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2864646 = {S42743, 161, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; PIXELDROID HD2 Build/PIXEL_4_2_2_JELLY_BEAN) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878960 = {S42744, 149, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; e2001_v89_gq2002sc Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2942510 = {S42745, 138, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; vi-vn; VDM HD4 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956701 = {S42746, 138, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; es-es; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2961357 = {S42747, 155, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; e2001v21_v89_jbl1a698_2g Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2989037 = {S42748, 136, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; iPhone5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2989799 = {S42749, 137, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; tr-tr; iPhone Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994701 = {S42750, 65, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; v89_jbl1a698 Build/JOP40D)*/
const Device UA2995714 = {S42751, 59, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; unknown Build/JDQ39)*/
const Device UA2996054 = {S42752, 138, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; ru-ru; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999058 = {S42753, 130, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3002568 = {S42754, 130, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3013922 = {S42755, 150, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; e2001v21_v89_zlh_hd Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3046589 = {S42756, 137, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; unknown Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049485 = {S42757, 130, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049760 = {S42758, 123, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3050203 = {S42759, 130, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Android Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3057444 = {S42760, 126, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MID Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3057514 = {S42761, 142, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Pioneer E71t/V100 Linux/3.4.5 Android/4.2.1  Release/12.24.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2 System/Android 4.2.1;*/
const Device UA3072766 = {S42762, 138, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; th-th; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3072801 = {S42763, 147, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; e2001_v89_zlh_hd Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078752 = {S42764, 137, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; TWD_MID Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088073 = {S42765, 137, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; iphone Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088820 = {S42766, 138, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; en-us; TWD_MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3093164 = {S42767, 129, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; X Build/HMX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3093198 = {S42768, 138, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; Android Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3101014 = {S42769, 138, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; tr-tr; iPhone5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3101129 = {S42770, 148, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Pioneer S90w Build/PioneerS90w) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3120235 = {S42771, 137, { &P17968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.0; vi-vn; iPhone Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA10626 = {S42772, 156, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; MotoMB511 Build/RUTEM_U3_01.19.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10627 = {S42773, 157, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; MotoMB511 Build/RUTEM_U3_01.14.20) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10633 = {S42774, 153, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB511 Build/RUTEM_U3_01.14.16) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10635 = {S42775, 157, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; MotoMB511 Build/RUTEM_U3_01.17.12) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10640 = {S42776, 156, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; MotoMB511 Build/RUTEM_U3_01.17.4) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10653 = {S42777, 152, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; MB511 Build/RUTEM_U3_01.19.4) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10654 = {S42778, 152, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB511 Build/RUTEM_U3_01.10.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10655 = {S42779, 152, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB511 Build/RUTEM_U3_01.14.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10656 = {S42780, 205, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-MB511/Blur_Version.0.60.101.MB511.Brasil.en.BR Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; MB511 Build/RUTLA_U3_00.60.101) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10659 = {S42781, 156, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; MotoMB511 Build/RUTEM_U3_01.19.8) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA11849 = {S42782, 182, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB511 Build/Blur_Version.1.27.12.MB511.ATT.en.US Flex/P044) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA14482 = {S42783, 152, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB511 Build/RUTEM_U3_01.14.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA14483 = {S42784, 157, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MotoMB511 Build/RUTEM_U3_01.17.10) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA14489 = {S42785, 203, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-MB511/Blur_Version.0.53.0.MB511.Movistar.en.01 Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; MB511 Build/RUTLA_U3_00.53.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA14490 = {S42786, 152, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-za; MB511 Build/RUTEM_U3_01.08.4) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA14492 = {S42787, 215, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-MB511/Blur_Version.0.49.102.MB511.AmericaMovil.en.MX Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; MB511 Build/RUTLA_U3_00.49.101_R02) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA14494 = {S42788, 151, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; MB511 Build/RURGS_U3_1.81.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA14498 = {S42789, 152, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; MB511 Build/RUTEM_U3_01.12.4) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA14499 = {S42790, 152, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB511 Build/RUTEM_U3_01.13.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA14504 = {S42791, 145, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MotoMB511 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA14512 = {S42792, 203, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-MB511/Blur_Version.0.45.204.MB511.Vivo.en.BR Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; MB511 Build/RUTLA_U3_00.45.204) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA14513 = {S42793, 152, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-nz; MB511 Build/RUTEM_U3_01.12.8) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA14519 = {S42794, 156, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MotoMB511 Build/RUTEM_U3_01.08.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA14520 = {S42795, 202, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-MB511/Blur_Version.0.45.203.MB511.TIM.en.BR Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; MB511 Build/RUTLA_U3_00.45.203) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA14522 = {S42796, 185, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; MB511 Build/Blur_Version.3.1.810.MB511.Rogers.en.CA Flex/P014) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA30151 = {S42797, 181, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB511 Build/Blur_Version.1.28.4.MB511.ATT.en.US Flex/P044) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA30152 = {S42798, 151, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB511 Build/RUTH_U3_00.28.4) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA52841 = {S42799, 153, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; MB511 Build/RUTEM_U3_01.14.20) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA52850 = {S42800, 182, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB511 Build/Blur_Version.0.19.41.MB511.ATT.en.US Flex/P025) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA79024 = {S42801, 153, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB511 Build/RUTEM_U3_01.14.10) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA89769 = {S42802, 146, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB511 Build/ERE27) AppleWebKit/525.10 +(KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA786602 = {S42803, 204, { &P13511,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-MB511/Blur_Version.0.60.0.MB511.Latam.en.01 Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB511 Build/RUTLA_U3_00.60.0_R01) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA841373 = {S42804, 132, { &P14908,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P6210 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1968853 = {S42805, 132, { &P14908,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2024027 = {S42806, 61, { &P14908,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-P6210 Build/IML74K)*/
const Device UA2033489 = {S42807, 61, { &P14908,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P6210 Build/IMM76D)*/
const Device UA2608746 = {S42808, 139, { &P14908,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3 LeoROM; zh-tw; GT-P6210 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2609231 = {S42809, 64, { &P14908,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/1.0.3 (Linux; U; Android 4.0.3; en-US; GT-P6210 Build/X.X)*/
const Device UA2610288 = {S42810, 133, { &P14908,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; GT-P6210 Build/RKS7S01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2611565 = {S42811, 90, { &P14908,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/1.1.3 (Linux; U; Android 4.0.3; es-US; GT-P6210 Build/Novo 7 Essential v0.6 X [D80])*/
const Device UA2613682 = {S42812, 162, { &P14908,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6210 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2614521 = {S42813, 117, { &P14908,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Android 4.0.4; en-us; GT-P6210 Build/IMM76D) FreeWheelAdManager/5.0.0-r8951-X;com.gotv.crackle.handset/X*/
const Device UA2614991 = {S42814, 73, { &P14908,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.0.4; en-US; GT-P6210 Build/IMM76D.UELPC)*/
const Device UA2616578 = {S42815, 73, { &P14908,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.0.4; ru-RU; GT-P6210 Build/IMM76D.XXLPL)*/
const Device UA2616986 = {S42816, 145, { &P14908,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2618061 = {S42817, 145, { &P14908,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P6210 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2618785 = {S42818, 73, { &P14908,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.4; en-GB; GT-P6210 Build/IMM76D.XXLPN)*/
const Device UA2619897 = {S42819, 56, { &P14908,&P18066,&P18062,&P18092 }};	/*Useragent - Android/4.6.1 (Linux; U; Android 4.0.4; en_US; GT-P6210)*/
const Device UA2621986 = {S42820, 127, { &P14908,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GT-P6210 Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2624067 = {S42821, 130, { &P14908,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6210 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2631321 = {S42822, 94, { &P14908,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.apps.plus/X (Linux; U; Android 4.0.4; en_US; GT-P6210; Build/IMM76D) (gzip)*/
const Device UA2635171 = {S42823, 92, { &P14908,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.youtube/3.5.5(Linux; U; Android 4.0.4; en_US; GT-P6210 Build/IMM76D) gzip*/
const Device UA2635173 = {S42824, 130, { &P14908,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P6210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA31667 = {S42825, 140, { &P12444,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; U20i Build/1.1.A.0.8) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA31670 = {S42826, 141, { &P12444,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; U20i Build/1.1.A.0.12) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA31680 = {S42827, 155, { &P12444,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; SonyEricssonU20iv Build/1.2.A.1.174) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA31683 = {S42828, 142, { &P12444,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; U20i Build/1.2.A.1.174) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA37509 = {S42829, 141, { &P12444,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; es-es; U20i Build/1.1.A.0.13) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA54849 = {S42830, 154, { &P12444,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; SonyEricssonU20i Build/1.2.A.1.174) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA129662 = {S42831, 80, { &P15364,&P18110,&P17825,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/3.5; Linux) KHTML/3.5.5 (like Gecko) (Debian)*/
const Device UA2261508 = {S42832, 140, { &P18447,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT30p Build/7.0.A.3.195) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2560014 = {S42833, 139, { &P18488,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2578021 = {S42834, 139, { &P18488,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L300 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2668162 = {S42835, 61, { &P18488,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SPH-L300 Build/JZO54K)*/
const Device UA2435376 = {S42836, 154, { &P12280,&P17090,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8J2 Safari/6533.18.5*/
const Device UA2436264 = {S42837, 154, { &P12280,&P17090,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8G4 Safari/6533.18.5*/
const Device UA2436439 = {S42838, 154, { &P12280,&P17090,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8L1 Safari/6533.18.5*/
const Device UA2437209 = {S42839, 154, { &P12280,&P17090,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8F190 Safari/6533.18.5*/
const Device UA2437360 = {S42840, 154, { &P12280,&P17090,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8H7 Safari/6533.18.5*/
const Device UA2461211 = {S42841, 154, { &P12280,&P17090,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8K2 Safari/6533.18.5*/
const Device UA2858042 = {S42842, 150, { &P12280,&P17090,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 431 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML like Gecko) CriOS/23.0.1271.96 Mobile/8G4 Safari/6533.18.5*/
const Device UA11301 = {S42843, 224, { &P13746,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-; LG-GT540 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA14562 = {S42844, 160, { &P13746,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1 Black Edition by Quarx; ru-ru; GT540 Build/ERE27) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA20595 = {S42845, 185, { &P13746,&P5477,&P4861,&P18092 }};	/*Useragent - LG-GT540f/v20a Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; GT540f Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA39512 = {S42846, 156, { &P13746,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1 Black Edition by Quarx; en-us; GT540 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA39997 = {S42847, 146, { &P13746,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1 FY.FA668; vi-vn; GT540 Build/ERE27) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA43003 = {S42848, 227, { &P13746,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-ca; LG-GT540f Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA44125 = {S42849, 141, { &P13746,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT540 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA44130 = {S42850, 145, { &P13746,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; pl-pl; GT540 Build/ERE27) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA51686 = {S42851, 177, { &P13746,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; LG-GT540/V20a Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA52883 = {S42852, 146, { &P13746,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; et-ee; GT540 Build/ERE27) AppleWebKit/528.5+ (KHTML, nagu Gecko) Versioon/3.1.2 Mobile Safari/525.20,1*/
const Device UA64651 = {S42853, 226, { &P13746,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; LG-GT540 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA67974 = {S42854, 139, { &P13746,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; GT540 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA153867 = {S42855, 63, { &P13746,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT540 Build/ERE27)*/
const Device UA1912638 = {S42856, 127, { &P13746,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT540 Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2932933 = {S42857, 145, { &P13746,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-mx; GT540 Build/ERE27) AppleWebKit/528.5+ (KHTML, like Gecko) Versi?n/3.1.2 Mobile Safari/525.20.1*/
const Device UA2203746 = {S42858, 112, { &P18221,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia302/14.26; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2387205 = {S42859, 111, { &P18221,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia302/14.26; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5*/
const Device UA150367 = {S42860, 110, { &P15364,&P18110,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA150796 = {S42861, 108, { &P15364,&P18110,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA843077 = {S42862, 109, { &P15364,&P18110,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fr-FR) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.514.0 Safari/534.7*/
const Device UA1864092 = {S42863, 94, { &P15364,&P18110,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; en-US) AppleWebKit/534.6 (KHTML, like Gecko) Chrome/7.0.500.0 Safari/534.6*/
const Device UA2127618 = {S42864, 116, { &P15364,&P18110,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US) AppleWebKit/534.8 (KHTML, like Gecko) Chrome/7.0.521.0 Safari/534.8*/
const Device UA2131433 = {S42865, 108, { &P15364,&P18110,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; ru-RU) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA2139375 = {S42866, 180, { &P15364,&P18110,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1.15) Gecko/X Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/7.0.540.0 Safari/534.10*/
const Device UA251884 = {S42867, 136, { &P21317,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-VM701 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA646434 = {S42868, 60, { &P21317,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-VM701 Build/GRJ22)*/
const Device UA2759013 = {S42869, 142, { &P21317,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-VM701 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA600319 = {S42870, 107, { &P17595,&P17295,&P22505,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en; rv:1.9.2.28) Gecko/X Camino/2.1.2 (like Firefox/3.6.28)*/
const Device UA10101 = {S42871, 110, { &P15198,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-S3650C/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA51617 = {S42872, 118, { &P15198,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-S3650W/S3650WXEJA2 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA62617 = {S42873, 109, { &P15198,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-S3650/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA46895 = {S42874, 147, { &P15501,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.53) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA46904 = {S42875, 99, { &P15501,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.1-update1;Zh_cn;E16i;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA49274 = {S42876, 159, { &P15501,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; SonyEricssonE16i Build/2.1.1.A.0.53) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA60024 = {S42877, 147, { &P15501,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.46) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA369197 = {S42878, 152, { &P15501,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.53) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA564423 = {S42879, 69, { &P15501,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; E16i Build/2.1.1.A.0.53)*/
const Device UA575223 = {S42880, 133, { &P15501,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.53) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2021000 = {S42881, 133, { &P15501,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.46) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2144684 = {S42882, 118, { &P15364,&P22188,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux MIPS32 1074Kf CPS QuadCore; en-US; rv:1.9.2.13) Gecko/X Fedora/3.6.13-1.fc14 Firefox/3.6.13*/
const Device UA2167039 = {S42883, 143, { &P13124,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PantechP9070 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2353044 = {S42884, 65, { &P13124,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; PantechP9070 Build/IMM76I)*/
const Device UA622546 = {S42885, 76, { &P17595,&P17296,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:8.0.1) Gecko/X Firefox/8.0.1*/
const Device UA2261458 = {S42886, 140, { &P18724,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-H Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2365775 = {S42887, 135, { &P18724,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SmartTabII7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2405358 = {S42888, 132, { &P18724,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A2107A-H Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2405792 = {S42889, 140, { &P18724,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-F Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2487698 = {S42890, 83, { &P15364,&P7133,&P22368,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.4) Gecko/X Firefox/15.4.1 PaleMoon/15.4.1*/
const Device UA2487699 = {S42891, 96, { &P15364,&P7133,&P22368,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:15.4) Gecko/X Firefox/15.4.1-x64 PaleMoon/15.4.1-x64*/
const Device UA2008441 = {S42892, 139, { &P13064,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I727 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029080 = {S42893, 147, { &P13064,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I727 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2178343 = {S42894, 147, { &P13064,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I727 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2201790 = {S42895, 139, { &P13064,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I727 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2235172 = {S42896, 139, { &P13064,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I727 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258126 = {S42897, 147, { &P13064,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2402067 = {S42898, 69, { &P13064,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K)*/
const Device UA2455566 = {S42899, 139, { &P13064,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I727 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2794446 = {S42900, 145, { &P13064,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA863576 = {S42901, 136, { &P21119,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SoftwinerEvb Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1951734 = {S42902, 143, { &P21119,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SoftwinerEvb Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025035 = {S42903, 139, { &P21119,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SoftwinerEvbV13 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2034600 = {S42904, 65, { &P21119,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SoftwinerEvb Build/IML74K)*/
const Device UA2155591 = {S42905, 136, { &P21119,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SoftwinerEvb Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2203560 = {S42906, 143, { &P21119,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SoftwinerEvb Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2339859 = {S42907, 65, { &P21119,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SoftwinerEvb Build/IMM76D)*/
const Device UA2482928 = {S42908, 144, { &P21119,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Softwinerf721 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2486829 = {S42909, 144, { &P21119,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Softwinerf900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2596491 = {S42910, 144, { &P21119,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Softwinerf761 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658501 = {S42911, 142, { &P21119,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SoftwinerfX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2711584 = {S42912, 145, { &P21119,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Softwinerf761I Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2727509 = {S42913, 139, { &P21119,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SoftwinerEvbV13 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2781446 = {S42914, 145, { &P21119,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Softwinerkf026 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2790966 = {S42915, 66, { &P21119,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Softwinerf761 Build/IMM76D)*/
const Device UA2854696 = {S42916, 137, { &P21119,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; Softwinerf900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2895021 = {S42917, 145, { &P21119,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Softwinerf761K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3085779 = {S42918, 66, { &P21119,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Softwinerf900 Build/IMM76D)*/
const Device UA2431537 = {S42919, 79, { &P18088,&P5567,&P18525,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/6.0.0/28.3392; U; id) Presto/2.8.119 Version/11.10*/
const Device UA2945976 = {S42920, 84, { &P18088,&P5567,&P18525,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/6.0.0/27.2338; U; en) Presto/2.8.119 240X320 Casio C781*/
const Device UA438788 = {S42921, 137, { &P18301,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; VS840 4G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA438789 = {S42922, 61, { &P18301,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; VS840 4G Build/GRK39F)*/
const Device UA609326 = {S42923, 150, { &P15201,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  HTC Sensation Z710e Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA609488 = {S42924, 147, { &P15201,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  HTC Sensation 4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA610033 = {S42925, 164, { &P15201,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  HTC Sensation 4G with Beats Audio Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA901563 = {S42926, 149, { &P15201,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710e Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA901882 = {S42927, 146, { &P15201,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA906815 = {S42928, 163, { &P15201,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G with Beats Audio Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA909124 = {S42929, 145, { &P15201,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Sensation Z710e Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2025195 = {S42930, 166, { &P15201,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G XE with Beats Audio Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2027312 = {S42931, 142, { &P15201,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Sensation 4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2386046 = {S42932, 149, { &P15201,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Z710e Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2469394 = {S42933, 146, { &P15201,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation 4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2727264 = {S42934, 142, { &P15201,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Sensation 4G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2597379 = {S42935, 139, { &P18697,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I415 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2669867 = {S42936, 61, { &P18697,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-I415 Build/JZO54K)*/
const Device UA1969948 = {S42937, 134, { &P20941,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Chaser Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2339882 = {S42938, 58, { &P20941,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Chaser Build/GRJ90)*/
const Device UA2677338 = {S42939, 77, { &P17595,&P21691,&P18430,&P18092 }};	/*Useragent - Safari/8536.29.13 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro10%2C1)*/
const Device UA2679271 = {S42940, 71, { &P17595,&P21691,&P18430,&P18092 }};	/*Useragent - Safari/8536.29.13 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (iMac12%2C2)*/
const Device UA2709185 = {S42941, 73, { &P17595,&P21691,&P18430,&P18092 }};	/*Useragent - Safari/8536.29.13 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBook5%2C1)*/
const Device UA2716981 = {S42942, 73, { &P17595,&P21691,&P18430,&P18092 }};	/*Useragent - Safari/8536.29.13 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (Macmini3%2C1)*/
const Device UA2717767 = {S42943, 76, { &P17595,&P21691,&P18430,&P18092 }};	/*Useragent - Safari/8536.29.13 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookAir5%2C2)*/
const Device UA2719402 = {S42944, 72, { &P17595,&P21691,&P18430,&P18092 }};	/*Useragent - Safari/8536.29.13 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacPro3%2C1)*/
const Device UA1961918 = {S42945, 74, { &P15364,&P6704,&P22732,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Avant Browser)*/
const Device UA1964614 = {S42946, 74, { &P15364,&P6704,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Avant Browser)*/
const Device UA2339272 = {S42947, 79, { &P15364,&P6704,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Avant Browser; SV1)*/
const Device UA2364563 = {S42948, 89, { &P15364,&P6704,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Avant Browser; Avant Browser)*/
const Device UA2395451 = {S42949, 89, { &P15364,&P6704,&P22732,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; Avant Browser; Avant Browser)*/
const Device UA2413031 = {S42950, 79, { &P15364,&P6704,&P22732,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows 98; SV1; Avant Browser)*/
const Device UA644780 = {S42951, 178, { &P13648,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.3 NokiaE7-00; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.4.2.6 Mobile Safari/533.4 3gpp-gba*/
const Device UA2006128 = {S42952, 191, { &P13648,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.3 NokiaE7-00/111.040.1511; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.1.4 Mobile Safari/535.1 3gpp-gba*/
const Device UA2675527 = {S42953, 138, { &P15575,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676020 = {S42954, 138, { &P15575,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100T Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676024 = {S42955, 137, { &P15575,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676198 = {S42956, 137, { &P15575,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676683 = {S42957, 138, { &P15575,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100T Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676830 = {S42958, 130, { &P15575,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2677170 = {S42959, 130, { &P15575,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2677278 = {S42960, 138, { &P15575,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677284 = {S42961, 138, { &P15575,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100M Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2606862 = {S42962, 70, { &P15364,&P17285,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:22.0) Gecko/X Firefox/22.0*/
const Device UA2771506 = {S42963, 81, { &P15364,&P17285,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:22.0) Gecko/X Firefox/22.0 AlexaToolbar/alxf-2.18*/
const Device UA2946389 = {S42964, 77, { &P15364,&P17285,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:22.0) Gecko/X Firefox/22.0 IceDragon/22.0.0.1*/
const Device UA2657817 = {S42965, 140, { &P20871,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S6810P Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2717662 = {S42966, 161, { &P20871,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-S6810P/S6810PXXAMC3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768285 = {S42967, 62, { &P20871,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S6810P Build/JZO54K)*/
const Device UA2796648 = {S42968, 161, { &P20871,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-S6810P/S6810PXXAMC4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3019696 = {S42969, 161, { &P20871,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; SAMSUNG GT-S6810P/S6810PXXAMD3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3023814 = {S42970, 161, { &P20871,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; SAMSUNG GT-S6810P/S6810PXXAME3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3076919 = {S42971, 160, { &P20871,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; sk-sk; GT-S6810P-ORANGE/S6810PXXAMC5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA663652 = {S42972, 118, { &P15364,&P16110,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/534.15 (KHTML, like Gecko) Chrome/10.0.612.1 Safari/534.15*/
const Device UA2714748 = {S42973, 108, { &P15364,&P16110,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/10.0.1410.64 Safari/537.31*/
const Device UA2200320 = {S42974, 82, { &P15364,&P7133,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0; Avant TriCore) Gecko/X Firefox/15.0.1*/
const Device UA2232699 = {S42975, 80, { &P15364,&P7133,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:16.0; Avant TriCore) Gecko/X Firefox/16.0*/
const Device UA2406893 = {S42976, 80, { &P15364,&P7133,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:17.0; Avant TriCore) Gecko/X Firefox/17.0*/
const Device UA2533074 = {S42977, 80, { &P15364,&P7133,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:18.0; Avant TriCore) Gecko/X Firefox/18.0*/
const Device UA2571479 = {S42978, 80, { &P15364,&P7133,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:19.0; Avant TriCore) Gecko/X Firefox/19.0*/
const Device UA2718666 = {S42979, 80, { &P15364,&P7133,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:20.0; Avant TriCore) Gecko/X Firefox/20.0*/
const Device UA2877696 = {S42980, 80, { &P15364,&P7133,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:21.0; Avant TriCore) Gecko/X Firefox/21.0*/
const Device UA2946094 = {S42981, 80, { &P15364,&P7133,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:22.0; Avant TriCore) Gecko/X Firefox/22.0*/
const Device UA3032462 = {S42982, 80, { &P15364,&P7133,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:23.0; Avant TriCore) Gecko/X Firefox/23.0*/
const Device UA858467 = {S42983, 76, { &P17595,&P18207,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:6.0.2) Gecko/X Firefox/6.0.2*/
const Device UA2974289 = {S42984, 138, { &P18280,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974489 = {S42985, 138, { &P18280,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC VLE_U Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975730 = {S42986, 138, { &P18280,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021647 = {S42987, 154, { &P18280,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Special Edition Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3064769 = {S42988, 139, { &P18447,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.1.141) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067050 = {S42989, 138, { &P12968,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.1.96) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2436535 = {S42990, 135, { &P12280,&P18432,&P22900,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.2 Mobile/10A523 Safari/8536.25*/
const Device UA2436638 = {S42991, 133, { &P12280,&P18432,&P22900,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.2 Mobile/10A403 Safari/8536.25*/
const Device UA1955942 = {S42992, 151, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Full AOSP on Toro-VZW Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2030442 = {S42993, 146, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Dark Jelly Belly Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2229703 = {S42994, 139, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LT18 Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2350042 = {S42995, 162, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Toasted Marshmallow Build/Toasted Marshmallow) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2415685 = {S42996, 140, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC ENRC2B Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2421510 = {S42997, 138, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ThinkPadTablet Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2425521 = {S42998, 132, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; 9300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2432476 = {S42999, 135, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Tablet-PC-4 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2455431 = {S43000, 149, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Full AOSP on Maguro Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2467201 = {S43001, 132, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; tervigon Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2480624 = {S43002, 135, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; imito am802 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2480916 = {S43003, 139, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Notion Ink ADAM Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2483186 = {S43004, 132, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-P600 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2484534 = {S43005, 130, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CTP980 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2487009 = {S43006, 141, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AllviewCity Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2502756 = {S43007, 129, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; iM970 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2503266 = {S43008, 138, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; mfld_pr2 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2503865 = {S43009, 139, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AMOI N820 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2521016 = {S43010, 146, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.5; TREQ_A10_Pocket2 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2530303 = {S43011, 134, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TAB R105BK Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2558062 = {S43012, 139, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ACGM727KC Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2558262 = {S43013, 135, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.4; Note3 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2559413 = {S43014, 133, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ST80208-1 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2578749 = {S43015, 126, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; G1 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2579353 = {S43016, 132, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CRXT1075 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2580549 = {S43017, 133, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ST80216-1 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2580954 = {S43018, 137, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; W27(Dualcore) Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2595046 = {S43019, 130, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MaxiIQ Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2604087 = {S43020, 132, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; DIN09.97 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2654740 = {S43021, 141, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC DLX_WLJ Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2658691 = {S43022, 148, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Cat Tablet PHOENIX 8.1J0 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2668780 = {S43023, 127, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TBD1083B Build/X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2674299 = {S43024, 126, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TBDB763 Build/X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2694414 = {S43025, 130, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Thrive Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2694432 = {S43026, 129, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; N80HH Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2697480 = {S43027, 130, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CTP988 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2701649 = {S43028, 134, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Pad 7208HD Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2702092 = {S43029, 130, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ICOU10 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2703678 = {S43030, 133, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ACGM816KC Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2718671 = {S43031, 126, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TBDB863 Build/X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2732917 = {S43032, 138, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; YOUPU U6 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2746328 = {S43033, 128, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EPAD Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2749173 = {S43034, 133, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GEM10312G Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2761994 = {S43035, 143, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; N101 DUAL CORE2 V11 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2762933 = {S43036, 135, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; I9420 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2765558 = {S43037, 134, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nivo Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2769056 = {S43038, 130, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CMP748 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2781324 = {S43039, 141, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ACGM727KC_N Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2796590 = {S43040, 142, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Infinity POP Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2830837 = {S43041, 137, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Allwinner A10 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2837532 = {S43042, 136, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; OT-995 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2844409 = {S43043, 126, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; X7 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2844467 = {S43044, 135, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; I9277 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2844573 = {S43045, 135, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; 3dp71g Build/10.2.C.0.83) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2848902 = {S43046, 134, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B92M Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2849192 = {S43047, 134, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B94M Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2849539 = {S43048, 128, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Z930 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2850363 = {S43049, 136, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EN-TAB309 Build/IML74K..X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2853858 = {S43050, 132, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; DINO7.17 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2859927 = {S43051, 130, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAD701 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2864107 = {S43052, 135, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; S4701 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2889555 = {S43053, 128, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ARIA Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2936836 = {S43054, 153, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Bouygues Telecom Bs 351 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2942573 = {S43055, 134, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; I110 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2951853 = {S43056, 129, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT400 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2956613 = {S43057, 126, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; S9 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2957555 = {S43058, 138, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E455g Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2959572 = {S43059, 140, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Amazing A4 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2962149 = {S43060, 134, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A600 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2985293 = {S43061, 142, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Quickfone N2 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2988873 = {S43062, 132, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Z1 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2992933 = {S43063, 139, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; CyrusApel Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2995339 = {S43064, 136, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; xTAB-70dci Build/w28ref.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2995416 = {S43065, 130, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ITQ701 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2996347 = {S43066, 130, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AQ710A Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2999295 = {S43067, 130, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; KLYAN2 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2999685 = {S43068, 135, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SP401 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2999711 = {S43069, 134, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TP7-1000DC Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3001757 = {S43070, 134, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Z993 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3002944 = {S43071, 141, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ZTE Kis Pro Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3005248 = {S43072, 141, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Positivo Ypy L700 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3005456 = {S43073, 132, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TPC-840M Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3010325 = {S43074, 126, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TBD1083 Build/X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3010778 = {S43075, 138, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGP351 Build/10.1.1.A.1.307) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3015987 = {S43076, 139, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; CRXT 1125 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3019690 = {S43077, 142, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; NaviPad TM-7045 3G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3019790 = {S43078, 131, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TBDG773 Build/Zeki.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3043561 = {S43079, 152, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MBX reference board (g02ref) Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3047035 = {S43080, 162, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A850 S/K Build/VEGAVIET.COM JELLYBEAN V1.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3049593 = {S43081, 149, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; i-mobile i-STYLE 7A Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3050672 = {S43082, 133, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MID-7089D Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3050880 = {S43083, 138, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CRCB2094 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3054995 = {S43084, 129, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; UM-A13B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3056701 = {S43085, 144, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; one touch D920 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3057359 = {S43086, 137, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P712 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3057512 = {S43087, 139, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; smart_a17 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3058175 = {S43088, 146, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Casper VIA A3216 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3073877 = {S43089, 139, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; M35c Build/12.0.B.3.26) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3078540 = {S43090, 139, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HALOVALUE Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3088670 = {S43091, 136, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Perfeo7777HD Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3092798 = {S43092, 134, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ACGM8001KD Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3093532 = {S43093, 143, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Positivo Ypy L1050E Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3100640 = {S43094, 130, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1;  Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA3102503 = {S43095, 142, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Quickfone X2 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3102811 = {S43096, 136, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TP-800 Build/ICS.TP-800.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3103199 = {S43097, 138, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZTE V956 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3107053 = {S43098, 146, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; INHON PAPILIO G2 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3107716 = {S43099, 133, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TAQ-10042 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3121276 = {S43100, 138, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E975w Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3121630 = {S43101, 131, { &P15364,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; DINO762 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA354214 = {S43102, 80, { &P15492,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-C3322/C3322DDKK1 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2462382 = {S43103, 69, { &P19763,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; C6602 Build/TEST.YUGA.XSSM.93)*/
const Device UA2462397 = {S43104, 142, { &P19763,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C6602 Build/10.1.A.1.290) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2462407 = {S43105, 146, { &P19763,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SonyC6602 Build/10.1.A.1.113) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2462403 = {S43106, 142, { &P19763,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C6602 Build/10.1.A.1.182) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2515524 = {S43107, 147, { &P19763,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-02E Build/10.1.D.0.317) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2564149 = {S43108, 143, { &P19763,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-02E Build/10.1.D.0.317) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566150 = {S43109, 143, { &P19763,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-02E Build/10.1.D.0.322) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567061 = {S43110, 147, { &P19763,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-02E Build/10.1.D.0.322) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2601268 = {S43111, 147, { &P19763,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-02E Build/10.1.D.0.333) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2602471 = {S43112, 142, { &P19763,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C6602 Build/10.1.A.1.350) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2670099 = {S43113, 64, { &P19763,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; C6602 Build/10.1.A.1.434)*/
const Device UA2672673 = {S43114, 143, { &P19763,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-02E Build/10.1.D.0.333) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2674089 = {S43115, 142, { &P19763,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C6602 Build/10.1.A.1.434) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2704823 = {S43116, 145, { &P19763,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SonyL36h Build/10.1.A.1.350) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2716440 = {S43117, 147, { &P19763,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-02E Build/10.1.D.0.343) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2716459 = {S43118, 143, { &P19763,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-02E Build/10.1.D.0.343) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2717066 = {S43119, 65, { &P19763,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SO-02E Build/10.1.D.0.343)*/
const Device UA2767886 = {S43120, 144, { &P19763,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C6602 Build/10.1.1.A.1.253) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2870440 = {S43121, 147, { &P19763,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SonyL36h Build/10.1.1.A.1.253) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2870742 = {S43122, 144, { &P19763,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; C6602 Build/10.1.1.A.1.307) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2675765 = {S43123, 140, { &P23905,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100P Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2715412 = {S43124, 62, { &P23905,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9100P Build/JZO54K)*/
const Device UA2755636 = {S43125, 159, { &P23905,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100P-ORANGE/I9100PXXLSQ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2901150 = {S43126, 159, { &P23905,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100P-ORANGE/I9100PXXLSS Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2947994 = {S43127, 160, { &P23905,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; SAMSUNG GT-I9100P/I9100PXXLSQ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3065749 = {S43128, 160, { &P23905,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; SAMSUNG GT-I9100P/I9100PXXLSU Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2195950 = {S43129, 150, { &P19857,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; ALCATEL ONE TOUCH 903 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2355407 = {S43130, 150, { &P19857,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-nz; ALCATEL_ONE_TOUCH_903 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2366761 = {S43131, 151, { &P19857,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ALCATEL ONE TOUCH 903D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2385443 = {S43132, 151, { &P19857,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ALCATEL ONE TOUCH 903A Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA180492 = {S43133, 88, { &P15364,&P7133,&P21923,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0) chromeframe/10.0.648.205*/
const Device UA308538 = {S43134, 95, { &P15364,&P7133,&P21923,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0) chromeframe/10.0.648.204*/
const Device UA632199 = {S43135, 229, { &P15364,&P7133,&P21923,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/10.0.648.204; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8)*/
const Device UA672510 = {S43136, 145, { &P15364,&P7133,&P21923,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16 chromeframe/10.0.648.204*/
const Device UA1874200 = {S43137, 187, { &P15364,&P7133,&P21923,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/10.0.648.205; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)*/
const Device UA2540821 = {S43138, 178, { &P15364,&P7133,&P21923,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/10.0.648.151; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2; .NET4.0C)*/
const Device UA3040805 = {S43139, 208, { &P15364,&P7133,&P21923,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/10.0.648.204; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)*/
const Device UA2431211 = {S43140, 77, { &P18088,&P18550,&P17373,&P18092 }};	/*Useragent - Opera/9.80 (Bada; Opera Mini/6.5/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA1986865 = {S43141, 117, { &P17595,&P18207,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2*/
const Device UA2042872 = {S43142, 117, { &P17595,&P18207,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2*/
const Device UA2202814 = {S43143, 117, { &P17595,&P18207,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2*/
const Device UA2586038 = {S43144, 117, { &P17595,&P18207,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2*/
const Device UA2800027 = {S43145, 117, { &P17595,&P18207,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2*/
const Device UA3094880 = {S43146, 117, { &P17595,&P18207,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2*/
const Device UA48412 = {S43147, 162, { &P12698,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; LGC660 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA74441 = {S43148, 135, { &P12698,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LG-C660 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA77323 = {S43149, 162, { &P12698,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LGC660 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA627833 = {S43150, 164, { &P12698,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-sa; LG-C660h Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA652940 = {S43151, 157, { &P12698,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; LGC660r/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA680788 = {S43152, 163, { &P12698,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; LGC660h Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA708141 = {S43153, 136, { &P12698,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; LG-C660R Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA739558 = {S43154, 157, { &P12698,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; LGC660h/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA1946506 = {S43155, 160, { &P12698,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; nl-; LGC660 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA1950763 = {S43156, 60, { &P12698,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-C660h Build/GRJ22)*/
const Device UA2022067 = {S43157, 59, { &P12698,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-C660 Build/GRJ22)*/
const Device UA2022563 = {S43158, 60, { &P12698,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-C660R Build/GRI40)*/
const Device UA2023076 = {S43159, 133, { &P12698,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; nl-; LG-C660 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2392358 = {S43160, 135, { &P12698,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; LG-C660h Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Versao/4.0 Mobile Safari/533.1*/
const Device UA357279 = {S43161, 138, { &P16319,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; GS02 Build/HuaweiU8860) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA403352 = {S43162, 139, { &P16319,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U8860 Build/HuaweiU8860) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA663915 = {S43163, 63, { &P16319,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; U8860 Build/HuaweiU8860)*/
const Device UA706720 = {S43164, 138, { &P16319,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ja-jp; GS02 Build/HuaweiU8860) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA746446 = {S43165, 127, { &P16319,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; U8860 Build/HuaweiU8860) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1945158 = {S43166, 146, { &P16319,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; HUAWEI_U8860 Build/HuaweiU8860) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1955017 = {S43167, 142, { &P16319,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; U8860-51 Build/HuaweiU8860) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022639 = {S43168, 146, { &P16319,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; HUAWEI_U8860 Build/HuaweiU8860) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024645 = {S43169, 139, { &P16319,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; U8860 Build/HuaweiU8860) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033296 = {S43170, 132, { &P16319,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; U8860 Build/HuaweiU8860) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA8994 = {S43171, 165, { &P12142,&P5129,&P3690,&P18092 }};	/*Useragent - SonyEricssonU5a/R2EA; Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525*/
const Device UA63721 = {S43172, 166, { &P12142,&P5129,&P3690,&P18092 }};	/*Useragent - SonyEricssonU5at/R2FA; Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525*/
const Device UA63756 = {S43173, 166, { &P12142,&P5129,&P3690,&P18092 }};	/*Useragent - SonyEricssonU5iv/R2EA; Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525*/
const Device UA64616 = {S43174, 165, { &P12142,&P5129,&P3690,&P18092 }};	/*Useragent - SonyEricssonU5i/R2EA; Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525*/
const Device UA93023 = {S43175, 29, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - SuperBot/4.7.0.74 (Windows 7)*/
const Device UA191968 = {S43176, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.6.0_25; Europe/de) http://yacy.net/bot.html*/
const Device UA347152 = {S43177, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.6.0_29; Europe/en) http://yacy.net/bot.html*/
const Device UA348575 = {S43178, 96, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0; America/en) http://yacy.net/bot.html*/
const Device UA359652 = {S43179, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_02; Europe/de) http://yacy.net/bot.html*/
const Device UA569098 = {S43180, 95, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0; Europe/de) http://yacy.net/bot.html*/
const Device UA600329 = {S43181, 99, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.6.0_31; America/en) http://yacy.net/bot.html*/
const Device UA602996 = {S43182, 140, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko; WEBTRENDSPAGEINFOBOT) Version/5.0.3 Safari/533.19.4*/
const Device UA685155 = {S43183, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.6.0_26; Europe/de) http://yacy.net/bot.html*/
const Device UA703759 = {S43184, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.6.0_29; Europe/de) http://yacy.net/bot.html*/
const Device UA741860 = {S43185, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.6.0_31; Europe/de) http://yacy.net/bot.html*/
const Device UA762446 = {S43186, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.6.0_23; Europe/de) http://yacy.net/bot.html*/
const Device UA893474 = {S43187, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_04; Europe/de) http://yacy.net/bot.html*/
const Device UA1905403 = {S43188, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.6.0_29; Europe/fr) http://yacy.net/bot.html*/
const Device UA1933772 = {S43189, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_01; Europe/de) http://yacy.net/bot.html*/
const Device UA2228460 = {S43190, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_07; Europe/de) http://yacy.net/bot.html*/
const Device UA2230388 = {S43191, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_07; Europe/fr) http://yacy.net/bot.html*/
const Device UA2320916 = {S43192, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_05; Europe/de) http://yacy.net/bot.html*/
const Device UA2361844 = {S43193, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_09; Europe/de) http://yacy.net/bot.html*/
const Device UA2422886 = {S43194, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_04; Europe/en) http://yacy.net/bot.html*/
const Device UA2598709 = {S43195, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_13; Europe/de) http://yacy.net/bot.html*/
const Device UA2683958 = {S43196, 99, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_04; America/en) http://yacy.net/bot.html*/
const Device UA2692688 = {S43197, 105, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; aggregator VocusBot 0.4; +http://www.vocus.com/vnhs.html)*/
const Device UA2794350 = {S43198, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_21; Europe/de) http://yacy.net/bot.html*/
const Device UA2795066 = {S43199, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_10; Europe/fr) http://yacy.net/bot.html*/
const Device UA2952316 = {S43200, 99, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld-global; amd64 Windows 7 6.1; java 1.7.0_04; America/en) http://yacy.net/bot.html*/
const Device UA3015805 = {S43201, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_25; Europe/de) http://yacy.net/bot.html*/
const Device UA3019841 = {S43202, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (webportal-global; amd64 Windows 7 6.1; java 1.7.0_25; Europe/en) http://yacy.net/bot.html*/
const Device UA3023322 = {S43203, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.6.0_38; Europe/de) http://yacy.net/bot.html*/
const Device UA3026682 = {S43204, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (webportal-global; amd64 Windows 7 6.1; java 1.8.0-ea; Europe/en) http://yacy.net/bot.html*/
const Device UA3030545 = {S43205, 96, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (webportal-global; x86 Windows 7 6.1; java 1.7.0_25; Europe/en) http://yacy.net/bot.html*/
const Device UA3041825 = {S43206, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (webportal-global; amd64 Windows 7 6.1; java 1.7.0_04; Europe/de) http://yacy.net/bot.html*/
const Device UA3056750 = {S43207, 99, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_25; America/en) http://yacy.net/bot.html*/
const Device UA3057923 = {S43208, 97, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (amd64 Windows 7 6.1; java 1.6.0_20; Europe/de) http://yacy.net/bot.html;c2VvLWNyYXdsZXI=*/
const Device UA3057948 = {S43209, 95, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (x86 Windows 7 6.1; java 1.6.0_18; Europe/sv) http://yacy.net/bot.html;c2VvLWNyYXdsZXI=*/
const Device UA3077736 = {S43210, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_25; Europe/en) http://yacy.net/bot.html*/
const Device UA3092497 = {S43211, 98, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 7 6.1; java 1.7.0_25; Europe/fr) http://yacy.net/bot.html*/
const Device UA3120237 = {S43212, 80, { &P18254,&P7133,&P18245,&P18091 }};	/*Useragent - yacybot (amd64 Windows 7 6.1; java 1.6.0_17; Europe/de) http://yacy.net/bot.html*/
const Device UA257517 = {S43213, 138, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Nexus S Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA347043 = {S43214, 137, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Nexus S Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA352178 = {S43215, 138, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus S Build/IML69B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA353071 = {S43216, 138, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus S Build/IML70C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA353129 = {S43217, 138, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus S Build/IML74B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA358781 = {S43218, 138, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA359205 = {S43219, 60, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Nexus S Build/IML74K)*/
const Device UA360284 = {S43220, 153, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Full Android on Crespo Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA368970 = {S43221, 137, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA602902 = {S43222, 138, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Nexus S Build/IMM76B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA622382 = {S43223, 142, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30*/
const Device UA622445 = {S43224, 142, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; Nexus S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30*/
const Device UA625154 = {S43225, 138, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA647503 = {S43226, 124, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Nexus S Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA674154 = {S43227, 60, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Nexus S Build/IMM76D)*/
const Device UA762106 = {S43228, 137, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Crespo Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA766873 = {S43229, 138, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA823783 = {S43230, 138, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM30D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA905716 = {S43231, 138, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA909423 = {S43232, 138, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM76M) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1903215 = {S43233, 139, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Nexus S Build/ICS_MR0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1978275 = {S43234, 137, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Crespo Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1985157 = {S43235, 138, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2024237 = {S43236, 138, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM20C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026742 = {S43237, 135, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus S Build/v15) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028717 = {S43238, 129, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Nexus S Build/IMM76L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2029384 = {S43239, 129, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Nexus S Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2949810 = {S43240, 131, { &P14574,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; nexus s Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2524760 = {S43241, 142, { &P21606,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LGL86C/V100 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2561238 = {S43242, 137, { &P21606,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LGL86C Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665062 = {S43243, 59, { &P21606,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LGL86C Build/IMM76L)*/
const Device UA3065095 = {S43244, 138, { &P18295,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094770 = {S43245, 138, { &P18295,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EndeavorU Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094848 = {S43246, 134, { &P18295,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; One X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111555 = {S43247, 138, { &P18295,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA609501 = {S43248, 133, { &P12879,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  GT-P7510 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA609506 = {S43249, 133, { &P12879,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  GT-P7510 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA902245 = {S43250, 132, { &P12879,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P7510 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1971318 = {S43251, 132, { &P12879,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2168079 = {S43252, 132, { &P12879,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T859 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2610523 = {S43253, 138, { &P12879,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA60649 = {S43254, 137, { &P12557,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB612 Build/3.4.2-120) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61102 = {S43255, 141, { &P12557,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB612 Build/KRNS-X4-1.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61257 = {S43256, 142, { &P12557,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB612 Build/KRNS-X4-0.13.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61333 = {S43257, 141, { &P12557,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB612 Build/KRNS-X4-1.1.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61380 = {S43258, 141, { &P12557,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB612 Build/KRNS-X4-1.1.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA251136 = {S43259, 142, { &P12557,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB612 Build/KRNS-X4-1.1.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA680893 = {S43260, 66, { &P12557,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; MB612 Build/KRNS-X4-1.1.10)*/
const Device UA613244 = {S43261, 82, { &P17968,&P17979,&P18160,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:10.0.4) Gecko/10.0.4 Firefox/10.0.4 Fennec/10.0.4*/
const Device UA900545 = {S43262, 82, { &P17968,&P17979,&P18160,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:10.0.5) Gecko/10.0.5 Firefox/10.0.5 Fennec/10.0.5*/
const Device UA49737 = {S43263, 136, { &P18174,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; VS920 4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA628822 = {S43264, 60, { &P18174,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; VS920 4G Build/GRJ90)*/
const Device UA630033 = {S43265, 137, { &P18174,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; VS920 4G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA846840 = {S43266, 61, { &P18174,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; VS920 4G Build/GRK39F)*/
const Device UA2362770 = {S43267, 151, { &P18640,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT901 Build/7.7.1Q-144_SLS_S240-58) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA157921 = {S43268, 69, { &P15364,&P5690,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 5.1; .NET CLR 1.1.4322)*/
const Device UA170416 = {S43269, 50, { &P15364,&P5690,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 5.1)*/
const Device UA512047 = {S43270, 242, { &P15364,&P5690,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; InfoPath.3; Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; Paragon Labels))*/
const Device UA623206 = {S43271, 46, { &P15364,&P5690,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows XP)*/
const Device UA916106 = {S43272, 69, { &P15364,&P5690,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 5.2; .NET CLR 1.1.4322)*/
const Device UA1874656 = {S43273, 62, { &P15364,&P5690,&P4551,&P18092 }};	/*Useragent - Mozilla/3.0 (compatible;MSIE 5.0;  Windows NT 5.1; QQDownload)*/
const Device UA2274328 = {S43274, 51, { &P15364,&P5690,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.1)*/
const Device UA2286930 = {S43275, 62, { &P15364,&P5690,&P4551,&P18092 }};	/*Useragent - Mozilla/6.0 (compatible;MSIE 5.0;  Windows NT 5.1; QQDownload)*/
const Device UA2502468 = {S43276, 108, { &P15364,&P5690,&P4551,&P18092 }};	/*Useragent - ('Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 5.1; .NET CLR 1.1.4322)', 'Internet Explorer 5, Windows XP')*/
const Device UA2545003 = {S43277, 63, { &P15364,&P5690,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; Windows NT 5.1; Trident/3.0)*/
const Device UA2658785 = {S43278, 63, { &P15364,&P5690,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; Windows NT 5.2; Trident/1.0)*/
const Device UA2745007 = {S43279, 51, { &P15364,&P5690,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (Compatible; MSIE 5.0; Windows NT 5.1;)*/
const Device UA904449 = {S43280, 135, { &P18113,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT912 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA904814 = {S43281, 154, { &P18113,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; DROID RAZR Build/6.7.1-68_DHD-15_M4-7) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA907947 = {S43282, 156, { &P18113,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA911272 = {S43283, 150, { &P18113,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2026288 = {S43284, 140, { &P18113,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Droid Razr Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2043757 = {S43285, 139, { &P18559,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LG730 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2402543 = {S43286, 61, { &P18559,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-LG730 Build/IMM76L)*/
const Device UA2766043 = {S43287, 142, { &P22867,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C5303 Build/12.0.A.1.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768066 = {S43288, 64, { &P22867,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; C5303 Build/12.0.A.1.257)*/
const Device UA2780094 = {S43289, 146, { &P22867,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyC5303 Build/12.0.A.1.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2783990 = {S43290, 142, { &P22867,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C5303 Build/12.0.A.1.257) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2784783 = {S43291, 146, { &P22867,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyC5303 Build/12.0.A.1.257) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2962446 = {S43292, 146, { &P22867,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyC5303 Build/12.0.A.1.237) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3062270 = {S43293, 142, { &P22867,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C5303 Build/12.0.A.2.245) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3063528 = {S43294, 146, { &P22867,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyC5303 Build/12.0.A.2.245) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3080770 = {S43295, 146, { &P22867,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonyC5303 Build/12.0.A.2.254) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3095122 = {S43296, 142, { &P22867,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C5303 Build/12.0.A.2.254) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103206 = {S43297, 66, { &P22867,&P18111,&P18112,&P18092 }};	/*Useragent - SonyC5303 Build/12.0.A.1.257 stagefright/1.2 (Linux;Android 4.1.2)*/
const Device UA2017929 = {S43298, 189, { &P12581,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.3 Nokia500/111.021.0028; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.1.4 Mobile Safari/535.1 3gpp-gba*/
const Device UA3064874 = {S43299, 137, { &P15575,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065717 = {S43300, 137, { &P15575,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066883 = {S43301, 138, { &P15575,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067903 = {S43302, 138, { &P15575,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100M Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3082480 = {S43303, 138, { &P15575,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100T Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083023 = {S43304, 138, { &P15575,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083346 = {S43305, 138, { &P15575,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100T Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA429114 = {S43306, 129, { &P13341,&P5156,&P3519,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9105; en-GB) AppleWebKit/534.8+ (KHTML, like Gecko) Version/6.0.0.668 Mobile Safari/534.8+*/
const Device UA429139 = {S43307, 77, { &P13341,&P5156,&P3519,&P18092 }};	/*Useragent - BlackBerry9105/6.0.0.570 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/123*/
const Device UA24526 = {S43308, 136, { &P12568,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; GT-B7510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60722 = {S43309, 137, { &P12568,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; GT-B7510L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61377 = {S43310, 137, { &P12568,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; GT-B7510B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA551750 = {S43311, 60, { &P12568,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; GT-B7510 Build/FROYO)*/
const Device UA727850 = {S43312, 124, { &P12568,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; GT-B7510 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1893475 = {S43313, 61, { &P12568,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; GT-B7510L Build/FROYO)*/
const Device UA2439987 = {S43314, 134, { &P12568,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-; GT-B7510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2533512 = {S43315, 139, { &P19865,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A850K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2560027 = {S43316, 139, { &P19865,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A850S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2562853 = {S43317, 61, { &P19865,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A850K Build/JZO54K)*/
const Device UA2564269 = {S43318, 139, { &P19865,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A850L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2600856 = {S43319, 61, { &P19865,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A850S Build/JZO54K)*/
const Device UA2676467 = {S43320, 144, { &P19865,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A850K/2.12 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2705756 = {S43321, 61, { &P19865,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A850L Build/JZO54K)*/
const Device UA2741387 = {S43322, 144, { &P19865,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A850K/2.14 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2753829 = {S43323, 144, { &P19865,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A850K/2.16 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2862434 = {S43324, 142, { &P19865,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A850L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30*/
const Device UA2945962 = {S43325, 144, { &P19865,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A850K/2.22 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955347 = {S43326, 152, { &P19865,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A850K/A850K2.10 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30*/
const Device UA2988476 = {S43327, 149, { &P19865,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; IM-A850K/A850K2.10 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997429 = {S43328, 142, { &P19865,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A850S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30*/
const Device UA2999015 = {S43329, 151, { &P19865,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A850S/L/K/2.17 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30*/
const Device UA3100569 = {S43330, 137, { &P19865,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; IM-A850L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA33217 = {S43331, 142, { &P14211,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SPH-P100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48799 = {S43332, 140, { &P14211,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-; SPH-P100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78474 = {S43333, 77, { &P14211,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.1 (Linux; U; Android 2.3.4; en_US; SPH-P100 Build/GINGERBREAD.EF17)*/
const Device UA322123 = {S43334, 66, { &P14211,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SPH-P100 Build/GINGERBREAD)*/
const Device UA334833 = {S43335, 134, { &P15897,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; XT530 Build/V4.35C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA452907 = {S43336, 134, { &P15897,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; XT530 Build/V4.480) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623245 = {S43337, 134, { &P15897,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; XT530 Build/V4.530) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA639057 = {S43338, 134, { &P15897,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT531 Build/V4.410) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA681156 = {S43339, 144, { &P15897,&P5629,&P15760,&P18092 }};	/*Useragent - MOT-XT530/Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; XT530 Build/V4.460) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA688706 = {S43340, 134, { &P15897,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; XT531 Build/V4.31B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA693221 = {S43341, 134, { &P15897,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; XT531 Build/V4.54A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA693417 = {S43342, 134, { &P15897,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; XT530 Build/V4.35G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA720912 = {S43343, 133, { &P15897,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT531 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA796980 = {S43344, 134, { &P15897,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; XT531 Build/V4.510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA799835 = {S43345, 134, { &P15897,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; XT531 Build/V4.530) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA852718 = {S43346, 134, { &P15897,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; XT531 Build/V4.480) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA862941 = {S43347, 134, { &P15897,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; XT531 Build/V4.540) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA895430 = {S43348, 134, { &P15897,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; XT530 Build/V4.54C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023330 = {S43349, 122, { &P15897,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; XT531 Build/V4.480) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2024356 = {S43350, 121, { &P15897,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT531 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2028985 = {S43351, 134, { &P15897,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; XT531 Build/V4.48D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2544192 = {S43352, 126, { &P15897,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT531 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3117455 = {S43353, 58, { &P15897,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; XT530 Build/V4.530)*/
const Device UA453944 = {S43354, 87, { &P15364,&P17285,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.62*/
const Device UA453968 = {S43355, 87, { &P15364,&P17285,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.62*/
const Device UA603008 = {S43356, 86, { &P15364,&P17285,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4133); ru) Presto/2.10.229 Version/11.62*/
const Device UA603099 = {S43357, 101, { &P15364,&P17285,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.10.229 Version/11.62,gzip(gfe) (via translate.google.com)*/
const Device UA603153 = {S43358, 87, { &P15364,&P17285,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.6 (build 03278); ru) Presto/2.10.229 Version/11.62*/
const Device UA603366 = {S43359, 87, { &P15364,&P17285,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.62*/
const Device UA607651 = {S43360, 87, { &P15364,&P17285,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5310); ru) Presto/2.10.229 Version/11.62*/
const Device UA623730 = {S43361, 64, { &P15364,&P17285,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; pl) Presto/2.10.229 Version/11.62*/
const Device UA686712 = {S43362, 86, { &P15364,&P17285,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.62*/
const Device UA755415 = {S43363, 87, { &P15364,&P17285,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.62*/
const Device UA790322 = {S43364, 72, { &P15364,&P17285,&P18331,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.62*/
const Device UA822645 = {S43365, 87, { &P15364,&P17285,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.4 (build 02647); ru) Presto/2.10.229 Version/11.62*/
const Device UA2164319 = {S43366, 116, { &P15364,&P17285,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0); en) Presto/2.10.229 Version/11.62*/
const Device UA2165157 = {S43367, 86, { &P15364,&P17285,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4876); ru) Presto/2.10.229 Version/11.62*/
const Device UA2369487 = {S43368, 65, { &P15364,&P17285,&P18331,&P18092 }};	/*Useragent - Opera/11.62 (Windows NT 6.0; U; en) Presto/2.10.229 Version/11.62*/
const Device UA2974017 = {S43369, 137, { &P19861,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975065 = {S43370, 137, { &P19861,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-I605 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023028 = {S43371, 130, { &P19861,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA609405 = {S43372, 137, { &P17968,&P18149,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  cm_tenderloin Build/GWK74) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA900643 = {S43373, 136, { &P17968,&P18149,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; cm_tenderloin Build/GWK74) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA901457 = {S43374, 137, { &P17968,&P18149,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; cm_tenderloin Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA905488 = {S43375, 138, { &P17968,&P18149,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  cm_tenderloin Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA20522 = {S43376, 135, { &P13964,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-au; GT-I5510T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA23166 = {S43377, 135, { &P13964,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-us; GT-I5510L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41358 = {S43378, 134, { &P13964,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I5510 Build/Arkas) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58873 = {S43379, 133, { &P13964,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; -au; GT-I5510T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60385 = {S43380, 137, { &P13964,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; GT-I5510L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60570 = {S43381, 137, { &P13964,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; GT-I5510T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA64313 = {S43382, 134, { &P13964,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I5510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA190121 = {S43383, 58, { &P13964,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I5510 Build/FROYO)*/
const Device UA646131 = {S43384, 122, { &P13964,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I5510 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2019749 = {S43385, 61, { &P13964,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-I5510T Build/FROYO)*/
const Device UA3041402 = {S43386, 132, { &P13964,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-; GT-I5510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1953585 = {S43387, 74, { &P15364,&P6704,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Avant Browser)*/
const Device UA1954696 = {S43388, 74, { &P15364,&P6704,&P22731,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Avant Browser)*/
const Device UA2328620 = {S43389, 79, { &P15364,&P6704,&P22731,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Avant Browser; SV1)*/
const Device UA2375259 = {S43390, 89, { &P15364,&P6704,&P22731,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Avant Browser; Avant Browser)*/
const Device UA2395641 = {S43391, 89, { &P15364,&P6704,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; Avant Browser; Avant Browser)*/
const Device UA2444856 = {S43392, 79, { &P15364,&P6704,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows 98; SV1; Avant Browser)*/
const Device UA925389 = {S43393, 119, { &P15364,&P5690,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA1996345 = {S43394, 126, { &P15364,&P5690,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA2338946 = {S43395, 119, { &P15364,&P5690,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA2553322 = {S43396, 129, { &P15364,&P5690,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7,gzip(gfe)*/
const Device UA2505093 = {S43397, 73, { &P17595,&P18578,&P18430,&P18092 }};	/*Useragent - Safari/7536.26.17 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (Macmini5%2C1)*/
const Device UA2869738 = {S43398, 75, { &P17595,&P18578,&P18430,&P18092 }};	/*Useragent - Safari/7536.30.1 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookAir4%2C2)*/
const Device UA2872743 = {S43399, 75, { &P17595,&P18578,&P18430,&P18092 }};	/*Useragent - Safari/7536.30.1 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)*/
const Device UA2876097 = {S43400, 70, { &P17595,&P18578,&P18430,&P18092 }};	/*Useragent - Safari/7536.30.1 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (iMac12%2C1)*/
const Device UA2896792 = {S43401, 69, { &P17595,&P18578,&P18430,&P18092 }};	/*Useragent - Safari/7536.30.1 CFNetwork/520.5.1 Darwin/11.4.2 (i386) (MacPro1%2C1)*/
const Device UA2899429 = {S43402, 70, { &P17595,&P18578,&P18430,&P18092 }};	/*Useragent - Safari/7536.30.1 CFNetwork/520.5.1 Darwin/11.4.2 (i386) (MacBook4%2C1)*/
const Device UA72 = {S43403, 61, { &P17475,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia6300/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA130 = {S43404, 71, { &P17475,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia6300/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6300*/
const Device UA1964036 = {S43405, 128, { &P15602,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A501 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2025851 = {S43406, 57, { &P15602,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; A501 Build/IML74K)*/
const Device UA2668374 = {S43407, 139, { &P21506,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; GT-S7710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2732547 = {S43408, 61, { &P21506,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S7710 Build/JZO54K)*/
const Device UA2892280 = {S43409, 140, { &P21506,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-S7710L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3020404 = {S43410, 159, { &P21506,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-S7710/S7710XXAMF1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA458640 = {S43411, 151, { &P18318,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ASUS Transformer Pad TF700T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1995975 = {S43412, 80, { &P18318,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; ASUS Transformer Pad TF700T Build/IML74K)*/
const Device UA2361685 = {S43413, 152, { &P18318,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; da-dk; ASUS Transformer Pad TF700KL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2820947 = {S43414, 139, { &P18318,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ASUS Pad TF700T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA603638 = {S43415, 141, { &P15364,&P17817,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/10.10 Chromium/18.0.1025.151 Chrome/18.0.1025.151 Safari/535.19*/
const Device UA603807 = {S43416, 139, { &P15364,&P17817,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/10.10 Chromium/18.0.1025.151 Chrome/18.0.1025.151 Safari/535.19*/
const Device UA3118007 = {S43417, 149, { &P15364,&P17817,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/10.10 Chromium/18.0.1025.151 Chrome/18.0.1025.151 Safari/535.19,gzip(gfe)*/
const Device UA2383934 = {S43418, 104, { &P15364,&P7133,&P19813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.7.0 Safari/534.34*/
const Device UA3026368 = {S43419, 111, { &P15803,&P5843,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia200/11.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA2201622 = {S43420, 92, { &P15364,&P7133,&P22365,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:15.0) Gecko/X Firefox/15.2-x64 PaleMoon/15.2-x64*/
const Device UA2201627 = {S43421, 79, { &P15364,&P7133,&P22365,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X Firefox/15.2 PaleMoon/15.2*/
const Device UA33453 = {S43422, 139, { &P12841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC PH39100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA75693 = {S43423, 150, { &P12841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_PH39100/1.59.502.2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA75970 = {S43424, 150, { &P12841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_PH39100/1.63.502.2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA77208 = {S43425, 150, { &P12841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_PH39100/1.07.502.0 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA362879 = {S43426, 150, { &P12841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_PH39100/1.63.502.3 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA452972 = {S43427, 150, { &P12841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC_PH39100/1.63.502.4 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA576584 = {S43428, 150, { &P12841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_PH39100/1.63.502.4 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633631 = {S43429, 63, { &P12841,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC PH39100 Build/GRJ22)*/
const Device UA763042 = {S43430, 139, { &P12841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC_PH39100 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1943340 = {S43431, 139, { &P12841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC PH39100 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3064645 = {S43432, 129, { &P18556,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BNTV600 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA42781 = {S43433, 129, { &P17686,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; MZ601 Build/H.6.5-10) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA75016 = {S43434, 129, { &P17686,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; MZ601 Build/H.6.5-17) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA351549 = {S43435, 131, { &P17686,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; MZ601 Build/H.6.5-17-3) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA452503 = {S43436, 60, { &P17686,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; MZ601 Build/H.6.5-17-1)*/
const Device UA622890 = {S43437, 131, { &P17686,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-au; MZ601 Build/H.6.5-17-1) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA633688 = {S43438, 135, { &P17686,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ601 Build/H.6.1-17_CN-54) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA3064777 = {S43439, 149, { &P18113,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094712 = {S43440, 142, { &P18113,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3065473 = {S43441, 138, { &P18280,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066537 = {S43442, 138, { &P18280,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC VLE_U Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083758 = {S43443, 138, { &P18280,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111016 = {S43444, 154, { &P18280,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Special Edition Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2488520 = {S43445, 112, { &P18583,&P5043,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia110/03.04; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2386255 = {S43446, 71, { &P17595,&P19835,&P18430,&P18092 }};	/*Useragent - Safari/8536.26.17 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (iMac12%2C2)*/
const Device UA2387307 = {S43447, 77, { &P17595,&P19835,&P18430,&P18092 }};	/*Useragent - Safari/8536.26.17 CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (MacBookPro10%2C1)*/
const Device UA2404286 = {S43448, 72, { &P17595,&P19835,&P18430,&P18092 }};	/*Useragent - Safari/8536.26.17 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacPro3%2C1)*/
const Device UA2404317 = {S43449, 76, { &P17595,&P19835,&P18430,&P18092 }};	/*Useragent - Safari/8536.26.17 CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (MacBookAir5%2C2)*/
const Device UA2404336 = {S43450, 73, { &P17595,&P19835,&P18430,&P18092 }};	/*Useragent - Safari/8536.26.17 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBook5%2C1)*/
const Device UA2428952 = {S43451, 73, { &P17595,&P19835,&P18430,&P18092 }};	/*Useragent - Safari/8536.26.17 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (Macmini3%2C1)*/
const Device UA173997 = {S43452, 62, { &P15364,&P17285,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; en) Presto/2.8.99 Version/11.10*/
const Device UA198791 = {S43453, 63, { &P15364,&P17285,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; en) Presto/2.8.131 Version/11.11*/
const Device UA311742 = {S43454, 86, { &P15364,&P17285,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.8.131 Version/11.10*/
const Device UA382086 = {S43455, 34, { &P15364,&P17285,&P17365,&P18092 }};	/*Useragent - Opera/11.1 (Windows NT 6.0: U: en)*/
const Device UA393202 = {S43456, 86, { &P15364,&P17285,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.8.131 Version/11.11*/
const Device UA702564 = {S43457, 86, { &P15364,&P17285,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.8.131 Version/11.11*/
const Device UA812294 = {S43458, 86, { &P15364,&P17285,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.8.131 Version/11.10*/
const Device UA880141 = {S43459, 86, { &P15364,&P17285,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.8.131 Version/11.10*/
const Device UA887004 = {S43460, 86, { &P15364,&P17285,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.8.131 Version/11.11*/
const Device UA2369275 = {S43461, 41, { &P15364,&P17285,&P17365,&P18092 }};	/*Useragent - Opera/11.1 (Windows NT 6.0; WOW64; U; en)*/
const Device UA2235405 = {S43462, 116, { &P17968,&P17979,&P18524,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/7.28879/27.1662; U; en-us) Presto/2.8.119 Version/11.10 UCBrowser/8.4.1.204/145/444/*/
const Device UA2431035 = {S43463, 139, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I747 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467473 = {S43464, 139, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-L710 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2486058 = {S43465, 147, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SAMSUNG-SGH-I747 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2506883 = {S43466, 140, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-R530U Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2531270 = {S43467, 147, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SAMSUNG-SGH-I747 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2535641 = {S43468, 139, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-L710 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2536313 = {S43469, 138, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I747 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2537567 = {S43470, 139, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I747 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2565200 = {S43471, 138, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L710 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2568077 = {S43472, 146, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I747 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2603724 = {S43473, 139, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530U Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2662325 = {S43474, 60, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGH-I747 Build/JDQ39)*/
const Device UA2714114 = {S43475, 147, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I747 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2716463 = {S43476, 139, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L710 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2737985 = {S43477, 69, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SAMSUNG-SGH-I747 Build/JDQ39E)*/
const Device UA2768339 = {S43478, 61, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SPH-L710 Build/JDQ39E)*/
const Device UA2780126 = {S43479, 138, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9305 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2845549 = {S43480, 69, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; SAMSUNG-SGH-I747 Build/JOP40D)*/
const Device UA2860590 = {S43481, 175, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-CA; SPH-L710Android 4.2.2; fr-CA; SCH-R530U Build Build/JRO03L) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2863161 = {S43482, 139, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I747 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956237 = {S43483, 140, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530U Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956249 = {S43484, 61, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9305 Build/JDQ39E)*/
const Device UA2985219 = {S43485, 61, { &P18645,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; SGH-I747 Build/JOP40D)*/
const Device UA2752958 = {S43486, 138, { &P15575,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753456 = {S43487, 138, { &P15575,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100T Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753548 = {S43488, 138, { &P15575,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100T Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753846 = {S43489, 137, { &P15575,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753973 = {S43490, 138, { &P15575,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100M Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754725 = {S43491, 137, { &P15575,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755073 = {S43492, 138, { &P15575,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892855 = {S43493, 130, { &P15575,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA1996138 = {S43494, 147, { &P18114,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Transformer Prime TF201 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2230990 = {S43495, 147, { &P18114,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Transformer Prime TF201 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2261562 = {S43496, 148, { &P18114,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1;  Transformer Prime TF201 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2458175 = {S43497, 129, { &P18114,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TF201 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2011682 = {S43498, 116, { &P15364,&P7133,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1*/
const Device UA2011854 = {S43499, 123, { &P15364,&P7133,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1*/
const Device UA2888674 = {S43500, 129, { &P18639,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 10 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Safari/537.36*/
const Device UA3022787 = {S43501, 130, { &P18639,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 10 Build/JOP40F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023136 = {S43502, 128, { &P18639,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Nexus 10 Build/JVP15P) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA138279 = {S43503, 48, { &P15364,&P18110,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (X11; Linux i686; U; en) Presto/2.1.1*/
const Device UA171521 = {S43504, 48, { &P15364,&P18110,&P18337,&P18092 }};	/*Useragent - Opera/9.60 (X11; Linux i686; U; en) Presto/2.1.1*/
const Device UA204674 = {S43505, 48, { &P15364,&P18110,&P18337,&P18092 }};	/*Useragent - Opera/9.61 (X11; Linux i686; U; en) Presto/2.1.1*/
const Device UA236517 = {S43506, 48, { &P15364,&P18110,&P18337,&P18092 }};	/*Useragent - Opera/9.62 (X11; Linux i686; U; pl) Presto/2.1.1*/
const Device UA261423 = {S43507, 50, { &P15364,&P18110,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (X11; Linux x86_64; U; en) Presto/2.1.1*/
const Device UA444715 = {S43508, 68, { &P15364,&P18110,&P18337,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux x86_64; en) Opera 9.60*/
const Device UA871015 = {S43509, 81, { &P15364,&P18110,&P18337,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; U; en; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 9.60*/
const Device UA70446 = {S43510, 167, { &P13157,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Sensation XE with Beats Audio Z715e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70754 = {S43511, 161, { &P13157,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Sensation XE with Beats Audio Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70858 = {S43512, 155, { &P13157,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC_SensationXE_Beats_Z715e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74695 = {S43513, 153, { &P13157,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; HTC_SensationXE_Beats/1.0 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369339 = {S43514, 153, { &P13157,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sv-se; enhanced HTC Sensation XE Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA574253 = {S43515, 91, { &P13157,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Sensation XE with Beats Audio Z715e Build/GRJ22)*/
const Device UA626246 = {S43516, 155, { &P13157,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; HTC_SensationXE_Beats_Z715a Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA635115 = {S43517, 167, { &P13157,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; HTC Sensation XE with Beats Audio Z715a Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA653387 = {S43518, 155, { &P13157,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Sensation XE with Beats Audio Z715e Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA655252 = {S43519, 85, { &P13157,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Sensation XE with Beats Audio Build/GRJ22)*/
const Device UA702565 = {S43520, 149, { &P13157,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; HTC_SensationXE_Beats Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA720086 = {S43521, 149, { &P13157,&P5629,&P15760,&P18092 }};	/*Useragent - HTC_SensationXE_Beats_Z715e_Mozilla/5.0 (Linux; U; Android 2.3.4;it-it;Build/GRH78C) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA1962925 = {S43522, 160, { &P13157,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Sensation XE with Beats Audio Z715e Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2022909 = {S43523, 125, { &P13157,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Z715e Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2024333 = {S43524, 164, { &P13157,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Sensation with Beats Audio Z715e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025199 = {S43525, 137, { &P13157,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Z715e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025464 = {S43526, 130, { &P13157,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Z715e Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1971607 = {S43527, 112, { &P18219,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia203/20.36; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6*/
const Device UA1991705 = {S43528, 112, { &P18219,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia202/20.36; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6*/
const Device UA2402511 = {S43529, 113, { &P18219,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia202/20.52; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA2482528 = {S43530, 113, { &P18219,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia203/20.52; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA2554608 = {S43531, 139, { &P18636,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; VS930 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2710612 = {S43532, 61, { &P18636,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; VS930 4G Build/JZO54K)*/
const Device UA2974143 = {S43533, 136, { &P22585,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L720 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2770010 = {S43534, 139, { &P21283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-S6310 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2800235 = {S43535, 140, { &P21283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S6310N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2855753 = {S43536, 140, { &P21283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-S6310T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867745 = {S43537, 140, { &P21283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-S6310L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937261 = {S43538, 159, { &P21283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; el-gr; SAMSUNG GT-S6310/S6310XXAMC4 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2953238 = {S43539, 61, { &P21283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S6310 Build/JZO54K)*/
const Device UA2962400 = {S43540, 161, { &P21283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; SAMSUNG GT-S6310N/S6310NXXAMD1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2993614 = {S43541, 159, { &P21283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ro-ro; SAMSUNG GT-S6310/S6310XXAMB6 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3000713 = {S43542, 62, { &P21283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S6310N Build/JZO54K)*/
const Device UA3056641 = {S43543, 140, { &P21283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; GT-S6310B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3059493 = {S43544, 161, { &P21283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-pt; SAMSUNG GT-S6310N/S6310NXXAMG1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3060498 = {S43545, 160, { &P21283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-S6310N-ORANGE/S6310NXXAMG1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3074268 = {S43546, 62, { &P21283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S6310L Build/JZO54K)*/
const Device UA3108892 = {S43547, 160, { &P21283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-S6310N-ORANGE/S6310NXXAME1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121610 = {S43548, 62, { &P21283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S6310T Build/JZO54K)*/
const Device UA3127117 = {S43549, 160, { &P21283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-S6310N-ORANGE/S6310NXXAMG5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556971 = {S43550, 140, { &P21636,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_C525c Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2582012 = {S43551, 136, { &P21636,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C525c Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2710955 = {S43552, 58, { &P21636,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; C525c Build/IMM76I)*/
const Device UA2202903 = {S43553, 73, { &P15364,&P16110,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:19.0) Gecko/19.0 Firefox/19.0*/
const Device UA2202905 = {S43554, 68, { &P15364,&P16110,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:19.0) Gecko/19.0 Firefox/19.0*/
const Device UA925439 = {S43555, 128, { &P15869,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Xoom Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2030097 = {S43556, 129, { &P15869,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1;  Xoom Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2234371 = {S43557, 128, { &P15869,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xoom Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2503193 = {S43558, 129, { &P15869,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2;  Xoom Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2824681 = {S43559, 125, { &P15869,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; Xoom Build/JRN82) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2202484 = {S43560, 112, { &P12989,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia303/13.47; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2387099 = {S43561, 111, { &P12989,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia303/13.44; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5*/
const Device UA3079616 = {S43562, 52, { &P23144,&P23096,&P17596,&P18092 }};	/*Useragent - MobileSafari/9537.53 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA2703895 = {S43563, 60, { &P22594,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGH-M919 Build/JDQ39)*/
const Device UA2712908 = {S43564, 138, { &P22594,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-M919 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957044 = {S43565, 136, { &P22594,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-; SGH-M919 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3000120 = {S43566, 139, { &P22594,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-M919 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA622544 = {S43567, 76, { &P17595,&P17297,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:8.0.1) Gecko/X Firefox/8.0.1*/
const Device UA2885328 = {S43568, 70, { &P15364,&P16110,&P23479,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:25.0) Gecko/X Firefox/25.0*/
const Device UA2885470 = {S43569, 65, { &P15364,&P16110,&P23479,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:25.0) Gecko/X Firefox/25.0*/
const Device UA1969609 = {S43570, 136, { &P20108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ZTE-N910 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2416755 = {S43571, 60, { &P20108,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; ZTE-N910 Build/GRJ90)*/
const Device UA2973945 = {S43572, 129, { &P18556,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BNTV600 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA64890 = {S43573, 77, { &P12181,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry9530/5.0.0.328 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/105*/
const Device UA2701261 = {S43574, 140, { &P22602,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-S738C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781771 = {S43575, 62, { &P22602,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-S738C Build/IMM76D)*/
const Device UA2675580 = {S43576, 136, { &P18713,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707895 = {S43577, 136, { &P18713,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P768 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712896 = {S43578, 129, { &P18713,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2937406 = {S43579, 129, { &P18713,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P769 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2937996 = {S43580, 136, { &P18713,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P769 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2717019 = {S43581, 74, { &P18552,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT897 Build/9.8.2Q-122_XT897_FFW-5)*/
const Device UA2718179 = {S43582, 152, { &P18552,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2718935 = {S43583, 152, { &P18552,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT897 Build/9.8.2Q-122_XT897_FFW-7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2964017 = {S43584, 152, { &P18552,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT897 Build/9.8.2Q-122_XT897_FFW-8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999009 = {S43585, 81, { &P18552,&P18111,&P18112,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android 4.1.2;motorola XT897 Build/9.8.2Q-122_XT897_FFW-5)*/
const Device UA22259 = {S43586, 139, { &P16331,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HUAWEI-M860 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59613 = {S43587, 140, { &P16331,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HUAWEI-M860 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78145 = {S43588, 183, { &P16331,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HUAWEI-M860 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 androidAppVersion/2.1 AndroidWebkitWrapper*/
const Device UA182705 = {S43589, 132, { &P16331,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; M860 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA195727 = {S43590, 64, { &P16331,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; HUAWEI-M860 Build/FRG83G)*/
const Device UA578137 = {S43591, 144, { &P16331,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HUAWEI-M860 Build/HuaweiM860) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018647 = {S43592, 63, { &P16331,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; HUAWEI-M860 Build/FRG83)*/
const Device UA2018831 = {S43593, 139, { &P16331,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Huawei Ascend Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA786591 = {S43594, 97, { &P15364,&P17814,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux ppc; en-GB; rv:1.9.0.12) Gecko/X Ubuntu/8.10 (intrepid) Firefox/3.0.12*/
const Device UA924966 = {S43595, 137, { &P17595,&P17297,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA925054 = {S43596, 137, { &P17595,&P17297,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA925399 = {S43597, 137, { &P17595,&P17297,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA1996322 = {S43598, 137, { &P17595,&P17297,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA1996355 = {S43599, 137, { &P17595,&P17297,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA2176670 = {S43600, 137, { &P17595,&P17297,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA26494 = {S43601, 116, { &P12080,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-S3653/S365MDDJJ2 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA49543 = {S43602, 138, { &P13054,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6425LVW Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70694 = {S43603, 141, { &P13054,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6425LVW 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369048 = {S43604, 138, { &P13054,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HTC_Mecha Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623251 = {S43605, 62, { &P13054,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; ADR6425LVW Build/GRJ22)*/
const Device UA727364 = {S43606, 138, { &P13054,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HTC Mecha Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024561 = {S43607, 140, { &P13054,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us;ADR6425LVW 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024731 = {S43608, 62, { &P13054,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; HTC Mecha Build/GRK39F)*/
const Device UA2027784 = {S43609, 157, { &P13054,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Mecha Build/TEAMVICIOUS-MIUI X BETA-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030004 = {S43610, 140, { &P13054,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.4; en-us; ADR6425LVW 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030203 = {S43611, 148, { &P13054,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Mecha Build/TeamVicious-MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA777177 = {S43612, 74, { &P15364,&P16110,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:10.0.2) Gecko/X Firefox/10.0.2*/
const Device UA891489 = {S43613, 81, { &P15364,&P16110,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; rv = 10.0.1) Gecko/X Firefox/10.0.1 companydatatrees*/
const Device UA625967 = {S43614, 131, { &P18475,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-au; GT-P7320T Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA658933 = {S43615, 130, { &P18475,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; pl-pl; GT-P7320 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2033651 = {S43616, 59, { &P18475,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7320 Build/MASTER)*/
const Device UA2034103 = {S43617, 60, { &P18475,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7320T Build/MASTER)*/
const Device UA2222759 = {S43618, 149, { &P18475,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7320/P7320XXLC4 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2396378 = {S43619, 149, { &P18475,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7320/P7320BOLD1 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2397928 = {S43620, 148, { &P18475,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7320-ORANGE/P7320BVLE3 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2412180 = {S43621, 149, { &P18475,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7320/P7320BOLC1 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2504598 = {S43622, 149, { &P18475,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; it-it; SAMSUNG-GT-P7320/P7320XXLI2 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2524162 = {S43623, 129, { &P18475,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; -au; GT-P7320T Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2608374 = {S43624, 149, { &P18475,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; SAMSUNG-GT-P7320/P7320BOLB2 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2610652 = {S43625, 149, { &P18475,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7320/P7320BOKK2 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2611445 = {S43626, 138, { &P18475,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-au; GT-P7320T Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2616139 = {S43627, 149, { &P18475,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7320/P7320XXLC3 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2619156 = {S43628, 278, { &P18475,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-au; GT-P7320T Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.12;FBDM/{density=1.0,width=1280,height=752};FBLC/en_AU;FB_FW/2;FBCR/Telstra;FBPN/com.facebook.katana;FBDV/GT-P7320T;FBSV/3.2;]*/
const Device UA2619595 = {S43629, 148, { &P18475,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7320-ORANGE/P7320BVLJ1 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2622125 = {S43630, 149, { &P18475,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; it-it; SAMSUNG-GT-P7320/P7320XXLJ1 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2622174 = {S43631, 148, { &P18475,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7320-ORANGE/P7320BVLD2 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2624085 = {S43632, 149, { &P18475,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; SAMSUNG-GT-P7320/P7320XXLB1 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2633533 = {S43633, 149, { &P18475,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7320/P7320XXLG1 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2636242 = {S43634, 147, { &P18475,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; da-dk; GT-P7320 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13,gzip(gfe)*/
const Device UA625986 = {S43635, 131, { &P17623,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; VTAB1008 Build/HTK55) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA752323 = {S43636, 60, { &P17623,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; VTAB1008 Build/HTK55)*/
const Device UA2034325 = {S43637, 138, { &P17623,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; VTAB1008 Build/HTK55) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA33372 = {S43638, 187, { &P19976,&P5055,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaE6-00/021.014; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.28 Mobile Safari/533.4 3gpp-gba*/
const Device UA1966801 = {S43639, 143, { &P18290,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MZ617 4G Build/7.7.1-85_MZ617-26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1995664 = {S43640, 143, { &P18290,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MZ617 4G Build/7.7.1-85_MZ617-27) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2005521 = {S43641, 140, { &P18290,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MZ617 Build/7.7.1-85_MZ617-27) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2180930 = {S43642, 69, { &P18290,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MZ617 Build/7.7.1-85_MZ617-27)*/
const Device UA2648131 = {S43643, 143, { &P18290,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MZ617 4G Build/7.7.1-85_MZ617-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2585301 = {S43644, 146, { &P18125,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2859606 = {S43645, 140, { &P18125,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA74962 = {S43646, 190, { &P12707,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.3 Nokia701/111.020.0307; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.4.1.14 Mobile Safari/533.4 3gpp-gba*/
const Device UA604108 = {S43647, 190, { &P12707,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.4 Nokia701/112.010.1404; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.2.1.20 Mobile Safari/535.1 3gpp-gba*/
const Device UA1947831 = {S43648, 177, { &P12707,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.3 Nokia701; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.4.1.14 Mobile Safari/533.4 3gpp-gba*/
const Device UA2205904 = {S43649, 190, { &P12707,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.5 Nokia701/113.010.1506; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.2.21 Mobile Safari/535.1 3gpp-gba*/
const Device UA2948411 = {S43650, 73, { &P15364,&P7480,&P18245,&P18116 }};	/*Useragent - Mozilla/5.0 (compatible; AcoonBot/4.12.1; +http://www.acoon.de/robot.asp)*/
const Device UA2676702 = {S43651, 146, { &P15767,&P19826,&P22593,&P18092 }};	/*Useragent - Mozilla/5.0(iPad; U;CPU OS 6_1 like Mac OS X; zh-CN; iPad3,4) AppleWebKit/534.46 (KHTML, like Gecko) UCBrowser/2.0.1.280 U3/0.8.0 Safari/7543.48.3*/
const Device UA913406 = {S43652, 60, { &P15364,&P16110,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; rv:14.0) Gecko/X Firefox/14.0.1*/
const Device UA1997062 = {S43653, 72, { &P15364,&P16110,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:14.0) Gecko/X Firefox/14.0.1*/
const Device UA251612 = {S43654, 139, { &P18041,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; YP-G1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA665591 = {S43655, 63, { &P18041,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; YP-G1 Build/GINGERBREAD)*/
const Device UA693767 = {S43656, 139, { &P18041,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; YP-G1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA699621 = {S43657, 140, { &P18041,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; YP-GB1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2809578 = {S43658, 137, { &P18041,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; YP-G1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA347129 = {S43659, 97, { &P15364,&P5690,&P22374,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.3 (KHTML, like Gecko) Maxthon/3.3.1.600 Safari/535.3*/
const Device UA366235 = {S43660, 116, { &P15364,&P5690,&P22374,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.3 (KHTML, like Gecko) Maxthon/3.3.3.1000 Chrome/16.0.883.0 Safari/535.3*/
const Device UA834653 = {S43661, 125, { &P15364,&P5690,&P22374,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.3.6.2000 Chrome/18.0.966.0 Safari/535.12*/
const Device UA900233 = {S43662, 118, { &P15364,&P5690,&P22374,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.3.9.1000 Chrome/18.0.966.0 Safari/535.12*/
const Device UA1993666 = {S43663, 125, { &P15364,&P5690,&P22374,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.3.7.2000 Chrome/18.0.966.0 Safari/535.12*/
const Device UA2156960 = {S43664, 118, { &P15364,&P5690,&P22374,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.3.9.2000 Chrome/18.0.966.0 Safari/535.12*/
const Device UA2553810 = {S43665, 130, { &P21769,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ME301T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2658293 = {S43666, 59, { &P21769,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ME301T Build/JRO03C)*/
const Device UA2670582 = {S43667, 145, { &P18640,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2675948 = {S43668, 138, { &P18640,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2353216 = {S43669, 113, { &P18502,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8X by HTC)*/
const Device UA2366446 = {S43670, 97, { &P18502,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; PM23300)*/
const Device UA2503814 = {S43671, 121, { &P18502,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8X by HTC; Orange)*/
const Device UA2582438 = {S43672, 126, { &P18502,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8X by HTC; 2.00.161.04)*/
const Device UA2599797 = {S43673, 126, { &P18502,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8X by HTC; 2.00.161.01)*/
const Device UA2717115 = {S43674, 126, { &P18502,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8X by HTC; 2.01.161.00)*/
const Device UA2861276 = {S43675, 126, { &P18502,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8X by HTC; 2.01.166.00)*/
const Device UA2862429 = {S43676, 126, { &P18502,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8X by HTC; 2.01.163.00)*/
const Device UA3048740 = {S43677, 126, { &P18502,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8X by HTC; 2.01.172.00)*/
const Device UA3062755 = {S43678, 126, { &P18502,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8X by HTC; 3.41.161.02)*/
const Device UA3071625 = {S43679, 126, { &P18502,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8X by HTC; 1.00.161.02)*/
const Device UA3100306 = {S43680, 126, { &P18502,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8X by HTC; 2.00.172.04)*/
const Device UA3119038 = {S43681, 95, { &P18502,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; C620e)*/
const Device UA901261 = {S43682, 132, { &P15919,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P3100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA911507 = {S43683, 132, { &P15919,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1969242 = {S43684, 133, { &P15919,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  GT-P3100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA1994823 = {S43685, 133, { &P15919,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P3100B Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2360233 = {S43686, 133, { &P15919,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  GT-P3100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2386116 = {S43687, 133, { &P15919,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3100B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2614279 = {S43688, 134, { &P15919,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  GT-P3100B Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2622707 = {S43689, 134, { &P15919,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  GT-P3100B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA230432 = {S43690, 129, { &P16950,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; streak7 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA250318 = {S43691, 135, { &P16950,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Dell Streak 7 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA682830 = {S43692, 64, { &P16950,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; Dell Streak 7 Build/HTJ85B)*/
const Device UA2487082 = {S43693, 152, { &P18552,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT897 Build/7.7.1Q-6_SPR-125_ASA-14) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA412912 = {S43694, 137, { &P15364,&P17819,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.11 (KHTML, like Gecko) Ubuntu/11.10 Chromium/17.0.963.65 Chrome/17.0.963.65 Safari/535.11*/
const Device UA412926 = {S43695, 135, { &P15364,&P17819,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.11 (KHTML, like Gecko) Ubuntu/11.10 Chromium/17.0.963.65 Chrome/17.0.963.65 Safari/535.11*/
const Device UA124007 = {S43696, 15, { &P15364,&P17017,&P18107,&P18092 }};	/*Useragent - Mozilla/Firefox*/
const Device UA2567900 = {S43697, 30, { &P15364,&P17017,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (Gecko) Firefox/64*/
const Device UA2716362 = {S43698, 29, { &P15364,&P17017,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 NewsFox/1.0.8.4.2*/
const Device UA2676181 = {S43699, 149, { &P18113,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676342 = {S43700, 142, { &P18113,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA79892 = {S43701, 89, { &P17595,&P17295,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_5) AppleWebKit (KHTML, like Gecko) Chrome/5*/
const Device UA112808 = {S43702, 124, { &P17595,&P17295,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/532.9 (KHTML, like Gecko) Chrome/5.0.307.7 Safari/532.9*/
const Device UA121454 = {S43703, 125, { &P17595,&P17295,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_0; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.99 Safari/533.4*/
const Device UA136076 = {S43704, 126, { &P17595,&P17295,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.127 Safari/533.4*/
const Device UA146646 = {S43705, 126, { &P17595,&P17295,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.125 Safari/533.4*/
const Device UA268815 = {S43706, 125, { &P17595,&P17295,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.99 Safari/533.4*/
const Device UA470320 = {S43707, 126, { &P17595,&P17295,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.127 Safari/533.4*/
const Device UA2128569 = {S43708, 126, { &P17595,&P17295,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.125 Safari/533.4*/
const Device UA2161232 = {S43709, 124, { &P17595,&P17295,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.371.0 Safari/533.4*/
const Device UA2394955 = {S43710, 124, { &P17595,&P17295,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/532.9 (KHTML	 like Gecko) Chrome/5.0.307.9 Safari/532.9*/
const Device UA2585262 = {S43711, 89, { &P17595,&P21691,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro9%2C2)*/
const Device UA2600220 = {S43712, 80, { &P17595,&P21691,&P17470,&P18092 }};	/*Useragent - WebProcess/8536.28.10 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro9%2C2)*/
const Device UA2603257 = {S43713, 89, { &P17595,&P21691,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookAir5%2C2)*/
const Device UA2603773 = {S43714, 86, { &P17595,&P21691,&P17470,&P18092 }};	/*Useragent - ThumbnailService/39600.038 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro10%2C1)*/
const Device UA2606965 = {S43715, 84, { &P17595,&P21691,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (iMac13%2C1)*/
const Device UA2666982 = {S43716, 85, { &P17595,&P21691,&P17470,&P18092 }};	/*Useragent - ThumbnailService/39600.038 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookAir4%2C2)*/
const Device UA2667613 = {S43717, 80, { &P17595,&P21691,&P17470,&P18092 }};	/*Useragent - ThumbnailService/39600.038 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (iMac12%2C2)*/
const Device UA2669638 = {S43718, 86, { &P17595,&P21691,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (Macmini6%2C2)*/
const Device UA2701380 = {S43719, 74, { &P17595,&P21691,&P17470,&P18092 }};	/*Useragent - Socialite/7845 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro10%2C1)*/
const Device UA2705618 = {S43720, 58, { &P17595,&P21691,&P17470,&P18092 }};	/*Useragent - Miro/5.0.4 (http://www.getmiro.com/; Darwin 12.3.0 x86_64)*/
const Device UA2707559 = {S43721, 75, { &P17595,&P21691,&P17470,&P18092 }};	/*Useragent - WebProcess/8536.28.10 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (iMac13%2C1)*/
const Device UA2712748 = {S43722, 71, { &P17595,&P21691,&P17470,&P18092 }};	/*Useragent - NewsBar/162 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro10%2C1)*/
const Device UA2733182 = {S43723, 92, { &P17595,&P21691,&P17470,&P18092 }};	/*Useragent - XProtectUpdater (unknown version) CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro9%2C2)*/
const Device UA2733840 = {S43724, 89, { &P17595,&P21691,&P17470,&P18092 }};	/*Useragent - Norton%20Identity%20Safe/1.03 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro10%2C1)*/
const Device UA2737453 = {S43725, 80, { &P17595,&P21691,&P17470,&P18092 }};	/*Useragent - WebProcess/8536.29.13 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookAir5%2C2)*/
const Device UA2860381 = {S43726, 79, { &P17595,&P21691,&P17470,&P18092 }};	/*Useragent - Reader%20Notifier/4 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro10%2C1)*/
const Device UA2860483 = {S43727, 80, { &P17595,&P21691,&P17470,&P18092 }};	/*Useragent - CoverScout%203/3.5.7 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro10%2C1)*/
const Device UA2866649 = {S43728, 74, { &P17595,&P21691,&P17470,&P18092 }};	/*Useragent - CoverScout%203/3.5.7 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (iMac12%2C2)*/
const Device UA354831 = {S43729, 153, { &P18125,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID4 4G Build/6.5.1_73_DR4-224_TA-6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA359730 = {S43730, 154, { &P18125,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID4 4G Build/6.5.1_167_DR4-1_M1-207) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA397719 = {S43731, 154, { &P18125,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID4 4G Build/6.5.1_167_DR4-1_M1-215) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA590135 = {S43732, 157, { &P18125,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID4 4G Build/6.5.1_167_DR4-1_M1-215-LK) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA611150 = {S43733, 75, { &P18125,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; DROID4 Build/6.5.1_167_DR4-1_M1-219)*/
const Device UA624726 = {S43734, 154, { &P18125,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID4 4G Build/6.5.1_167_DR4-1_M1-219) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626132 = {S43735, 151, { &P18125,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID4 Build/6.5.1_167_DR4-1_M1-219) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633778 = {S43736, 151, { &P18125,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID4 Build/6.5.1_167_DR4-1_M1-215) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1903921 = {S43737, 148, { &P18125,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID4 4G Build/6.5.1_73_DR4-220) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020017 = {S43738, 148, { &P18125,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID4 4G Build/6.5.1_73_DR4-223) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027495 = {S43739, 153, { &P18125,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID4 4G Build/6.5.1_167_DR4-1_M1-219) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA2028579 = {S43740, 153, { &P18125,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.3.6; en-us; DROID4 4G Build/6.5.1_167_DR4-1_M1-219) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA910469 = {S43741, 147, { &P15767,&P18432,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A5338d Safari/7534.48.3*/
const Device UA2198542 = {S43742, 145, { &P15767,&P18432,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A403 Safari/7534.48.3*/
const Device UA2260773 = {S43743, 147, { &P15767,&P18432,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A523 Safari/7534.48.3*/
const Device UA2436647 = {S43744, 166, { &P15767,&P18432,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/19.0.1084.60 Version/6.0 Mobile/10A403 Safari/8536.25 HaoWangZhiDaQuanHD 1.4*/
const Device UA2466997 = {S43745, 148, { &P15767,&P18432,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A8426 Safari/7534.48.3*/
const Device UA2529395 = {S43746, 147, { &P15767,&P18432,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_0_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A550 Safari/7534.48.3*/
const Device UA2225892 = {S43747, 138, { &P19861,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-I605 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2432349 = {S43748, 138, { &P19861,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA249681 = {S43749, 136, { &P15794,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Runnymede; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA252560 = {S43750, 150, { &P15794,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_SensationXL_Beats_X315e; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA737926 = {S43751, 155, { &P15794,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC/SensationXL_Beats/1.05.161.4; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA765626 = {S43752, 150, { &P15794,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_SensationXL_Beats_X315b; en-th) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA25381 = {S43753, 130, { &P14682,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia6303iclassic/5.0 (07.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA25378 = {S43754, 87, { &P14682,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia6303iclassic/5.0 (07.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303iclassic*/
const Device UA575308 = {S43755, 123, { &P17595,&P18120,&P22525,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-us) AppleWebKit/535+ (KHTML, like Gecko) Version/5.0 Safari/535.4+ Midori/0.4*/
const Device UA699599 = {S43756, 124, { &P17595,&P18120,&P22525,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-us) AppleWebKit/534+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ Midori/0.4*/
const Device UA785417 = {S43757, 131, { &P17595,&P18120,&P22525,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-us) AppleWebKit/535+ (KHTML, like Gecko) Version/5.0 Sabayon Safari/535.7+ Midori/0.4*/
const Device UA850802 = {S43758, 124, { &P17595,&P18120,&P22525,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-us) AppleWebKit/535+ (KHTML, like Gecko) Version/5.0 Safari/535.20+ Midori/0.4*/
const Device UA1960188 = {S43759, 124, { &P17595,&P18120,&P22525,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-gb) AppleWebKit/535+ (KHTML, like Gecko) Version/5.0 Safari/535.22+ Midori/0.4*/
const Device UA2439140 = {S43760, 123, { &P17595,&P18120,&P22525,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; pl-pl) AppleWebKit/531+ (KHTML, like Gecko) Version/5.0 Safari/531.2+ Midori/0.4*/
const Device UA2700288 = {S43761, 123, { &P17595,&P18120,&P22525,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-us) AppleWebKit/537+ (KHTML, like Gecko) Version/5.0 Safari/537.6+ Midori/0.4*/
const Device UA2974138 = {S43762, 136, { &P22594,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA7915 = {S43763, 152, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB502 Build/BASEM_U3_01.12.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA7980 = {S43764, 151, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.1-update1; en-us; MB502 Build/BASIL_U3_02.60.5) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA8002 = {S43765, 151, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB502Build/BASIL_U3_03.90.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA8010 = {S43766, 151, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us;MB502 Build/BASIL_U3_03.90.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA8023 = {S43767, 151, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.1-update1; en-us; MB502 Build/BASIL_U3_03.90.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA8025 = {S43768, 151, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.1-update1; en-us; MB502 Build/BASIL_U3_03.90.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA7999 = {S43769, 145, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB502 Build/BASIL2174) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA8326 = {S43770, 151, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB502 Build/BASIL_U3_03.90.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 MobileSafari/530.17*/
const Device UA9497 = {S43771, 152, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; MB502 Build/BASTE_U3_02.39.5) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA9511 = {S43772, 145, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB502 Build/BASIL2177) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA9514 = {S43773, 152, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB502 Build/BASEM_U3_01.12.4) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA9525 = {S43774, 152, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; MB502 Build/BASTE_U3_02.39.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA9526 = {S43775, 151, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.1-update1; en-us; MB502 Build/BASIL_U3_02.60.5) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA11640 = {S43776, 152, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB502 Build/BASIL_U3_03.90.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA18107 = {S43777, 201, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-MB502/Blur_Version.0.40.0.MB502.Brasil.en.BR Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; MB502 Build/BASLA_U3_00.40.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA26830 = {S43778, 152, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB502 Build/BASIL_U3_02.60.5) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA38397 = {S43779, 159, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-NEW_T%5P_MB502; zh-cn; MB502 Build/BASIL_U3_03.90.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA41575 = {S43780, 215, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-MB502/Blur_Version.0.32.209.MB502.AmericaMovil.en.01 Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB502 Build/BASLA_U3_00.32.203_R04) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA52124 = {S43781, 151, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB502Build/BASIL_U3_02.60.5) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA52129 = {S43782, 151, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us;MB502 Build/BASIL_U3_02.60.5) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA52143 = {S43783, 150, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; MB502 Build/BASIL_U3_03.90.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA53729 = {S43784, 207, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-MB502/Blur_Version.0.32.202.MB502.Movistar.en.01 Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; MB502 Build/BASLA_U3_00.32.202) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA53730 = {S43785, 207, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-MB502/Blur_Version.0.32.101.MB502.Vivo.en.BR Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; MB502 Build/BASLA_U3_00.32.101_R02) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA53731 = {S43786, 211, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-MB502/Blur_Version.0.30.1.MB502.AmericaMovil.en.MX Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; MB502 Build/BASLA_U3_00.30.0_R01) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA53732 = {S43787, 205, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-MB502/Blur_Version.0.32.101.MB502.Brasil.en.BR Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; MB502 Build/BASLA_U3_00.32.101) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA57291 = {S43788, 211, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-MB502/Blur_Version.0.32.205.MB502.Movilnet.en.VE Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; MB502 Build/BASLA_U3_00.32.203_R02) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA370973 = {S43789, 152, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; he-il; MB502 Build/BASEM_U3_01.19.4) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA551657 = {S43790, 74, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; MB502 Build/BASIL_U3_03.90.7)*/
const Device UA2020653 = {S43791, 138, { &P13420,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; MB502 Build/BASEM_U3_01.12.4) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2030779 = {S43792, 151, { &P18471,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT907 Build/7.7.1Q-144_SMQ_S136-10) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261549 = {S43793, 144, { &P18471,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT905 Build/7.7.1Q_GCIRD-16) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2035260 = {S43794, 165, { &P18250,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR MAXX 4G Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2204239 = {S43795, 165, { &P18250,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Motorola RAZR MAXX Build/6.7.2-180_DHD-16_NA-27) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2353159 = {S43796, 165, { &P18250,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Motorola RAZR MAXX Build/6.7.2-180_DHD-16_NA-30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2554282 = {S43797, 87, { &P18250,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Motorola RAZR MAXX Build/6.7.2-180_DHD-16_NA-30)*/
const Device UA925291 = {S43798, 159, { &P16292,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1998799 = {S43799, 159, { &P16292,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2197252 = {S43800, 159, { &P16292,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2231805 = {S43801, 163, { &P16292,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Galaxy Nexus Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2241310 = {S43802, 161, { &P16292,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus 7 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2963970 = {S43803, 134, { &P21261,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.3; I9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36*/
const Device UA2971254 = {S43804, 136, { &P21261,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.9; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36*/
const Device UA2974141 = {S43805, 136, { &P21261,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020695 = {S43806, 134, { &P21261,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.3; I9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020787 = {S43807, 134, { &P21261,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; I9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022946 = {S43808, 137, { &P21261,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023937 = {S43809, 136, { &P21261,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.9; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3057614 = {S43810, 144, { &P21261,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy GT-I9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA61442 = {S43811, 140, { &P16467,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; PantechP8000 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61444 = {S43812, 153, { &P16467,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; PantechP8000/JPUS01242011 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA635450 = {S43813, 64, { &P16467,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; PantechP8000 Build/FRG83)*/
const Device UA440778 = {S43814, 129, { &P15364,&P7133,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/18.0.1025.142) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19*/
const Device UA440786 = {S43815, 136, { &P15364,&P7133,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/18.0.1025.142) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19*/
const Device UA453711 = {S43816, 226, { &P15364,&P7133,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/18.0.1025.142; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; MS-RTC EA 2; MS-RTC LM 8; .NET4.0C)*/
const Device UA454056 = {S43817, 190, { &P15364,&P7133,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/18.0.1025.142; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3)*/
const Device UA457616 = {S43818, 136, { &P15364,&P7133,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/18.0.1025.151) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19*/
const Device UA457730 = {S43819, 129, { &P15364,&P7133,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/18.0.1025.151) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19*/
const Device UA603580 = {S43820, 136, { &P15364,&P7133,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/18.0.1025.152) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.152 Safari/535.19*/
const Device UA603605 = {S43821, 129, { &P15364,&P7133,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/18.0.1025.152) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.152 Safari/535.19*/
const Device UA605628 = {S43822, 97, { &P15364,&P7133,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/18.0.1025.162)*/
const Device UA605648 = {S43823, 129, { &P15364,&P7133,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/18.0.1025.162) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19*/
const Device UA605659 = {S43824, 90, { &P15364,&P7133,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/18.0.1025.162)*/
const Device UA605777 = {S43825, 136, { &P15364,&P7133,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/18.0.1025.162) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19*/
const Device UA606288 = {S43826, 205, { &P15364,&P7133,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/18.0.1025.162; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA608852 = {S43827, 189, { &P15364,&P7133,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/18.0.1025.162; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.3; .NET4.0C; .NET4.0E)*/
const Device UA614916 = {S43828, 129, { &P15364,&P7133,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/18.0.1025.168) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA614974 = {S43829, 136, { &P15364,&P7133,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/18.0.1025.168) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA694267 = {S43830, 193, { &P15364,&P7133,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/18.0.1025.168; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)*/
const Device UA830982 = {S43831, 183, { &P15364,&P7133,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/18.0.1025.54; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2331969 = {S43832, 200, { &P15364,&P7133,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; GTB7.2; chromeframe/18.0.1025.11; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; eSobiSubscriber 2.0.4.16; .NET4.0C)*/
const Device UA2364719 = {S43833, 230, { &P15364,&P7133,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/18.0.1025.152; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)*/
const Device UA3071320 = {S43834, 91, { &P15364,&P7133,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/18.0.1025.162)*/
const Device UA3071512 = {S43835, 98, { &P15364,&P7133,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/18.0.1025.162)*/
const Device UA214586 = {S43836, 96, { &P14726,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG-C900)*/
const Device UA216202 = {S43837, 97, { &P14726,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG-C900k)*/
const Device UA246244 = {S43838, 97, { &P14726,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG-C900B)*/
const Device UA2030306 = {S43839, 155, { &P19891,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Samsung Galaxy Note I717 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030537 = {S43840, 139, { &P19891,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I717 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2176569 = {S43841, 147, { &P19891,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I717 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2190115 = {S43842, 147, { &P19891,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I717 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2235163 = {S43843, 147, { &P19891,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257988 = {S43844, 139, { &P19891,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I717 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257994 = {S43845, 147, { &P19891,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I717 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2352202 = {S43846, 139, { &P19891,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I717 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2366474 = {S43847, 69, { &P19891,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-I717 Build/JZO54K)*/
const Device UA2768959 = {S43848, 145, { &P19891,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778106 = {S43849, 147, { &P19891,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; Samsung-SGH-I717 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2197575 = {S43850, 122, { &P15364,&P7133,&P22462,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4*/
const Device UA2197577 = {S43851, 115, { &P15364,&P7133,&P22462,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4*/
const Device UA2879172 = {S43852, 133, { &P18680,&P18110,&P23483,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Safari/535.19 Silk-Accelerated=true*/
const Device UA863716 = {S43853, 141, { &P15972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-hk; HTC_One_XL Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA900660 = {S43854, 141, { &P15972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC One XL Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1953011 = {S43855, 63, { &P15972,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC One XL Build/IML74K)*/
const Device UA2027637 = {S43856, 152, { &P15972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC/One_XL/1.82.162.5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034185 = {S43857, 141, { &P15972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_One_XL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2170601 = {S43858, 141, { &P15972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; HTC One XL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2175597 = {S43859, 63, { &P15972,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC One XL Build/IMM76D)*/
const Device UA2180549 = {S43860, 152, { &P15972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HTC/One_XL/2.33.162.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257192 = {S43861, 152, { &P15972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC/One_XL/1.82.162.6 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2360557 = {S43862, 152, { &P15972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HTC_One_XL/2.27.111.4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2362504 = {S43863, 152, { &P15972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; HTC/One_XL/2.34.163.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2403555 = {S43864, 153, { &P15972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-ch; HTC/One_XL/2.41.166.51 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2457369 = {S43865, 153, { &P15972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; HTC/One_XL/2.41.163.53 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2500033 = {S43866, 152, { &P15972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HTC_One_XL/2.27.111.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597141 = {S43867, 138, { &P15972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTCOneXL Build/IMM76D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2652356 = {S43868, 153, { &P15972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; HTC/One_XL/2.41.163.51 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031617 = {S43869, 150, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257724 = {S43870, 141, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U8825D Build/HuaweiU8825D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2257881 = {S43871, 146, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C92B933SP05; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2365933 = {S43872, 142, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C92B936; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2379706 = {S43873, 146, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C92B931SP06; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2383483 = {S43874, 142, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C92B938; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2383696 = {S43875, 150, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8825D Build/HuaweiC8825D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2395545 = {S43876, 141, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8825D Build/HuaweiC8825D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2403653 = {S43877, 146, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C92B943SP01; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2418726 = {S43878, 72, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI U8825D Build/HuaweiU8825D)*/
const Device UA2420833 = {S43879, 136, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8825D Build/HuaweiC8825D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2422661 = {S43880, 152, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HUAWEI U8825-1 Build/HuaweiU8825-1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528557 = {S43881, 142, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C92B945; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2653391 = {S43882, 142, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C92B941; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2654971 = {S43883, 74, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI U8825-1 Build/HuaweiU8825-1)*/
const Device UA2692446 = {S43884, 72, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI C8825D Build/HuaweiC8825D)*/
const Device UA2714793 = {S43885, 144, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Bs 401 Build/HuaweiU8825-1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2764006 = {S43886, 142, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C92B947; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2777942 = {S43887, 146, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C92B939SP01; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2778728 = {S43888, 143, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C117B948; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2934000 = {S43889, 143, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C117B943; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2991174 = {S43890, 142, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8825D/C8825DV100R001C92B968; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2994698 = {S43891, 157, { &P20094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; HUAWEI Ascend G 330 Build/HuaweiU8825-1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA902868 = {S43892, 131, { &P18632,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AN10BG3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1964891 = {S43893, 134, { &P18632,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; AN10BG3-LZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257293 = {S43894, 133, { &P18632,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; AN10BG3DT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2496193 = {S43895, 60, { &P18632,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; AN10BG3 Build/IML74K)*/
const Device UA1988083 = {S43896, 141, { &P15079,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MK16i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030534 = {S43897, 153, { &P15079,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonMK16i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2226021 = {S43898, 63, { &P15079,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MK16i Build/4.1.B.0.587)*/
const Device UA2226690 = {S43899, 141, { &P15079,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MK16i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2200541 = {S43900, 151, { &P18318,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700T Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2356275 = {S43901, 151, { &P18318,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ASUS Transformer Pad TF700T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2466250 = {S43902, 152, { &P18318,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700KL Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2481322 = {S43903, 139, { &P18318,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Pad TF700T Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2670610 = {S43904, 129, { &P18283,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2670616 = {S43905, 137, { &P18283,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-T999 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2672304 = {S43906, 129, { &P18283,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I535 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676195 = {S43907, 136, { &P18283,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I535 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676234 = {S43908, 136, { &P18283,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676728 = {S43909, 136, { &P18283,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T999 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677172 = {S43910, 137, { &P18283,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9300 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709128 = {S43911, 129, { &P18283,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T999 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2713396 = {S43912, 137, { &P18283,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SCH-I535 Build/JOP40G) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2887167 = {S43913, 60, { &P15364,&P18110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux x86_64) Presto/2.12.388 Version/12.16*/
const Device UA2964203 = {S43914, 71, { &P15364,&P18110,&P23820,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.16*/
const Device UA3012616 = {S43915, 65, { &P15364,&P18110,&P23820,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; X11; Linux x86_64) Opera 12.16*/
const Device UA3077758 = {S43916, 68, { &P15364,&P18110,&P23820,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686) Presto/2.12.388 Version/12.16,gzip(gfe)*/
const Device UA2424994 = {S43917, 137, { &P18468,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IdeaTabA2109A Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2505401 = {S43918, 66, { &P18468,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; IdeaTabA2109A Build/JRO03R)*/
const Device UA2004702 = {S43919, 64, { &P15364,&P7127,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.10.289 Version/12.02*/
const Device UA2479742 = {S43920, 86, { &P15364,&P7127,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; MRA 6.0 (build 5972); ru) Presto/2.10.289 Version/12.02*/
const Device UA2740018 = {S43921, 62, { &P17968,&P17979,&P22773,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:24.0) Gecko/24.0 Firefox/24.0*/
const Device UA3018856 = {S43922, 68, { &P17968,&P17979,&P22773,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:24.0a2) Gecko/24.0a2 Firefox/24.0a2*/
const Device UA33104 = {S43923, 150, { &P12853,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_PH06130/1.23.502.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627408 = {S43924, 150, { &P12853,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_PH06130/2.14.502.2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA923988 = {S43925, 150, { &P12853,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_PH06130/1.23.502.1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA221412 = {S43926, 104, { &P14637,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG-E900; Orange)*/
const Device UA223352 = {S43927, 97, { &P14637,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG-E900h)*/
const Device UA225270 = {S43928, 96, { &P14637,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG-E900)*/
const Device UA371364 = {S43929, 97, { &P14637,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG E-900)*/
const Device UA925708 = {S43930, 103, { &P14637,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG-E900; TESCO)*/
const Device UA925871 = {S43931, 102, { &P14637,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG-E900; O2UK)*/
const Device UA203945 = {S43932, 97, { &P14781,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; MWP6985)*/
const Device UA166326 = {S43933, 107, { &P15364,&P17285,&P20804,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 6.0; en-US) AppleWebKit/527+ (KHTML, like Gecko, Safari/419.3) Qt/4.5.3*/
const Device UA2388119 = {S43934, 82, { &P15364,&P7133,&P20457,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:17.0) Gecko/X Firefox/17.0 SeaMonkey/2.14.1*/
const Device UA2459553 = {S43935, 69, { &P15364,&P7133,&P20457,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:17.0) Gecko/X SeaMonkey/2.14.1*/
const Device UA117688 = {S43936, 56, { &P15364,&P6704,&P17193,&P18092 }};	/*Useragent - Mozilla/6.0 (compatible; MSIE 7.01; AOL 9.0; Windows 98)*/
const Device UA338720 = {S43937, 120, { &P15364,&P6704,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows 98; SpamBlockerUtility 6.3.91; SpamBlockerUtility 6.2.91; .NET CLR 4.1.89;GB)*/
const Device UA792821 = {S43938, 56, { &P15364,&P6704,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows 98; SV1; SV1)*/
const Device UA1960173 = {S43939, 64, { &P15364,&P6704,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; SV1)*/
const Device UA1988417 = {S43940, 74, { &P15364,&P6704,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; MSN Optimized)*/
const Device UA1994149 = {S43941, 64, { &P15364,&P6704,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; SV1)*/
const Device UA2338007 = {S43942, 69, { &P15364,&P6704,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; SV1; SV1)*/
const Device UA2702751 = {S43943, 54, { &P15364,&P6704,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Win98; Trident/3.0)*/
const Device UA2849791 = {S43944, 60, { &P15364,&P6704,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Win 9x 4.90; Trident/2.0)*/
const Device UA2262470 = {S43945, 101, { &P15216,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.67; en-us; 8520/5.2.0.67) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2262543 = {S43946, 101, { &P15216,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.64; en-us; 8520/5.2.0.64) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2355293 = {S43947, 103, { &P15216,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.104; en-us; 8520/5.2.0.104) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2385483 = {S43948, 103, { &P15216,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.113; en-us; 8520/5.2.0.113) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2403041 = {S43949, 103, { &P15216,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.135; en-us; 8520/4.2.0.135) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2423818 = {S43950, 101, { &P15216,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.96; en-us; 8520/5.2.0.96) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2425078 = {S43951, 101, { &P15216,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.47; en-us; 8520/5.2.0.47) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2425505 = {S43952, 103, { &P15216,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.118; en-us; 8520/4.2.0.118) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2467150 = {S43953, 101, { &P15216,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.76; en-us; 8520/5.2.0.76) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2504008 = {S43954, 103, { &P15216,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.110; en-us; 8520/5.2.0.110) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2504655 = {S43955, 103, { &P15216,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.111; en-us; 8520/5.2.0.111) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2528134 = {S43956, 101, { &P15216,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.58; en-us; 8520/5.2.0.58) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2694311 = {S43957, 101, { &P15216,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.23; en-us; 8520/5.2.0.23) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2696470 = {S43958, 103, { &P15216,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.122; en-us; 8520/4.2.0.122) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2778102 = {S43959, 101, { &P15216,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.94; en-us; 8520/5.2.0.94) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2842552 = {S43960, 101, { &P15216,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.31; en-us; 8520/5.2.0.31) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2983053 = {S43961, 103, { &P15216,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.102; en-us; 8520/5.2.0.102) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2985880 = {S43962, 103, { &P15216,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.116; en-us; 8520/4.2.0.116) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3071290 = {S43963, 103, { &P15216,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.124; en-us; 8520/4.2.0.124) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2580283 = {S43964, 127, { &P14111,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Arc Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2606182 = {S43965, 125, { &P14111,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-; Arc Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2659765 = {S43966, 56, { &P14111,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Arc Build/JRO03L)*/
const Device UA2713502 = {S43967, 138, { &P22585,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L720 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719300 = {S43968, 60, { &P22585,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SPH-L720 Build/JDQ39)*/
const Device UA3048955 = {S43969, 136, { &P22585,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-; SPH-L720 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3109288 = {S43970, 174, { &P22585,&P18598,&P19802,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.2.2;dev=jfltespr;devm=SPH-L720;man=samsung;brw=Android;shwd=1080x1920;shwa=1080x1845;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA3064994 = {S43971, 137, { &P19861,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068053 = {S43972, 137, { &P19861,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-I605 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114297 = {S43973, 130, { &P19861,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2796312 = {S43974, 145, { &P20076,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT26w Build/6.2.B.0.200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2796543 = {S43975, 63, { &P20076,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LT26w Build/6.2.B.0.200)*/
const Device UA2869301 = {S43976, 141, { &P20076,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT26w Build/6.2.B.0.200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2894927 = {S43977, 145, { &P20076,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT26w Build/6.2.B.0.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958625 = {S43978, 141, { &P20076,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT26w Build/6.2.B.0.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3063156 = {S43979, 144, { &P20076,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT26w Build/6.2.B.1.96) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079787 = {S43980, 140, { &P20076,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT26w Build/6.2.B.1.96) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2676105 = {S43981, 140, { &P22394,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6603 Build/10.1.A.1.434) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676171 = {S43982, 133, { &P22394,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6603 Build/10.1.A.1.434) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2027803 = {S43983, 136, { &P20961,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ADR3010 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2481647 = {S43984, 60, { &P20961,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; ADR3010 Build/GRK39F)*/
const Device UA26812 = {S43985, 77, { &P13783,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry9100/5.0.0.921 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/102*/
const Device UA2566493 = {S43986, 138, { &P18595,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566512 = {S43987, 138, { &P18595,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567018 = {S43988, 131, { &P18595,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567111 = {S43989, 131, { &P18595,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA3074391 = {S43990, 138, { &P18595,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT_I8190 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA205696 = {S43991, 119, { &P17595,&P17297,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30*/
const Device UA210283 = {S43992, 119, { &P17595,&P17297,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA229456 = {S43993, 119, { &P17595,&P17297,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA629418 = {S43994, 119, { &P17595,&P17297,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA682852 = {S43995, 119, { &P17595,&P17297,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA2202447 = {S43996, 119, { &P17595,&P17297,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA2748335 = {S43997, 138, { &P18621,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E610 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2800123 = {S43998, 60, { &P18621,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E610 Build/JZO54K)*/
const Device UA2800351 = {S43999, 139, { &P18621,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E612f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865516 = {S44000, 138, { &P18621,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E612 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996449 = {S44001, 60, { &P18621,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E612 Build/JZO54K)*/
const Device UA2998293 = {S44002, 153, { &P18621,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-E610/E61020d-EUR-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3009613 = {S44003, 144, { &P18621,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; LG-E610v/V20d Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3019976 = {S44004, 144, { &P18621,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-E610v/V20e Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3028251 = {S44005, 150, { &P18621,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-E610/V20b-Orange Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3054862 = {S44006, 143, { &P18621,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; LG-E610/V20a Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3073806 = {S44007, 61, { &P18621,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E612f Build/JZO54K)*/
const Device UA3091959 = {S44008, 135, { &P18621,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; bs; LG-E610 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121160 = {S44009, 139, { &P18621,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E612g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA67003 = {S44010, 162, { &P14745,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; WX445 Build/VZW) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 240X320 motorola WX445*/
const Device UA160461 = {S44011, 61, { &P14745,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.1-update1; WX445 Build/VZW)*/
const Device UA234722 = {S44012, 162, { &P14745,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; WX445 Build/OPM) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 240X320 motorola WX445*/
const Device UA2019455 = {S44013, 161, { &P14745,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; WX445 Build/VZW) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17240X320 motorola WX445*/
const Device UA459318 = {S44014, 142, { &P16292,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.2; Galaxy Nexus Build/ICL53F) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA609388 = {S44015, 143, { &P16292,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.1;  Galaxy Nexus Build/ITL41F) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA609509 = {S44016, 143, { &P16292,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  Galaxy Nexus Build/ITL41F) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA616645 = {S44017, 143, { &P16292,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.2;  Galaxy Nexus Build/ICL53F) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA658060 = {S44018, 143, { &P16292,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  Galaxy Nexus Build/IMM76K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA668833 = {S44019, 137, { &P16292,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  Galaxy Nexus Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA887347 = {S44020, 142, { &P16292,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.1; Galaxy Nexus Build/ITL41F) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA898065 = {S44021, 138, { &P16292,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.2; Galaxy X Build/ICL53F) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA899773 = {S44022, 142, { &P16292,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA901713 = {S44023, 142, { &P16292,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Galaxy Nexus Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA902154 = {S44024, 136, { &P16292,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2008874 = {S44025, 138, { &P16292,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Galaxy X Build/IMM76K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2024565 = {S44026, 154, { &P16292,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Sprint Galaxy Nexus CDMA Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2034132 = {S44027, 149, { &P16292,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4 IMM76D; Galaxy Nexus Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2759724 = {S44028, 145, { &P16292,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.1; GT-I9250_EUR_XX Build/ICL23D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2705398 = {S44029, 137, { &P22585,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L720 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2719301 = {S44030, 130, { &P22585,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L720 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA50319 = {S44031, 46, { &P15364,&P6729,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows 95)*/
const Device UA2791891 = {S44032, 54, { &P15364,&P6729,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; Win95; Trident/1.0)*/
const Device UA3034287 = {S44033, 46, { &P15364,&P6729,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01 Windows 95)*/
const Device UA79888 = {S44034, 89, { &P17595,&P17295,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_5) AppleWebKit (KHTML, like Gecko) Chrome/4*/
const Device UA110168 = {S44035, 125, { &P17595,&P17295,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5*/
const Device UA260685 = {S44036, 125, { &P17595,&P17295,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5*/
const Device UA290567 = {S44037, 125, { &P17595,&P17295,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_0; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5*/
const Device UA491468 = {S44038, 125, { &P17595,&P17295,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5*/
const Device UA727947 = {S44039, 125, { &P17595,&P17295,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5*/
const Device UA826721 = {S44040, 124, { &P17595,&P17295,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.0 Safari/532.5*/
const Device UA862433 = {S44041, 124, { &P17595,&P17295,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/4.0.202.0 Safari/532.0*/
const Device UA2258343 = {S44042, 125, { &P17595,&P17295,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/532.5 (KHTML	 like Gecko) Chrome/4.0.249.30 Safari/532.5*/
const Device UA2642440 = {S44043, 125, { &P17595,&P17295,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/4.0.202.0 Safari/532.0)*/
const Device UA2191511 = {S44044, 147, { &P17989,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V8000_USA_Cricket Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2554211 = {S44045, 147, { &P17989,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N8000_USA_Cricket Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA172314 = {S44046, 143, { &P15364,&P17817,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Ubuntu/10.10 Chromium/9.0.597.94 Chrome/9.0.597.94 Safari/534.13*/
const Device UA495177 = {S44047, 143, { &P15364,&P17817,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.14 (KHTML, like Gecko) Ubuntu/10.10 Chromium/9.0.600.0 Chrome/9.0.600.0 Safari/534.14*/
const Device UA735651 = {S44048, 120, { &P15364,&P17817,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/540.0 (KHTML, like Gecko) Ubuntu/10.10 Chrome/9.1.0.0 Safari/540.0*/
const Device UA818854 = {S44049, 145, { &P15364,&P17817,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux armv7l; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Ubuntu/10.10 Chromium/9.0.597.94 Chrome/9.0.597.94 Safari/534.13*/
const Device UA2423589 = {S44050, 63, { &P15972,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC One XL Build/JRO03C)*/
const Device UA2428261 = {S44051, 141, { &P15972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; HTC One XL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2430759 = {S44052, 141, { &P15972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_One_XL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469366 = {S44053, 152, { &P15972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_XL/3.17.111.1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469501 = {S44054, 152, { &P15972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-ch; HTC/One_XL/3.17.166.2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2581153 = {S44055, 152, { &P15972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; HTC/One_XL/3.17.163.2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2679515 = {S44056, 152, { &P15972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC/One_XL/3.17.162.7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2714034 = {S44057, 152, { &P15972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC/One_XL/3.17.162.8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2739583 = {S44058, 152, { &P15972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; HTC_One_XL/3.17.111.2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937833 = {S44059, 141, { &P15972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One XL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2951632 = {S44060, 142, { &P15972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC EVA_UTL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999259 = {S44061, 141, { &P15972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One XL Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2700065 = {S44062, 134, { &P22754,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HP Slate 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2796043 = {S44063, 63, { &P22754,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HP Slate 7 Build/JRO03H)*/
const Device UA2974293 = {S44064, 137, { &P16362,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974488 = {S44065, 140, { &P16362,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710BST Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974724 = {S44066, 141, { &P16362,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710VMUB Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021955 = {S44067, 137, { &P16362,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SPH-D710 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA701628 = {S44068, 116, { &P17595,&P18207,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA2002724 = {S44069, 116, { &P17595,&P18207,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA2202489 = {S44070, 116, { &P17595,&P18207,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA2602578 = {S44071, 116, { &P17595,&P18207,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA2797560 = {S44072, 116, { &P17595,&P18207,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA3114347 = {S44073, 116, { &P17595,&P18207,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA13246 = {S44074, 173, { &P13775,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB520 Build/Blur_Version.37.4.0.MB520.ATT.en.US Flex/P015) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA13247 = {S44075, 185, { &P13775,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB520 Build/Blur_Version.1.0.939.MB520-continuous.ATT.en.US Flex/P008) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20327 = {S44076, 174, { &P13775,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB520 Build/Blur_Version.36.10.0.MB520.ATT.en.US Flex/P014) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20329 = {S44077, 147, { &P13775,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB520 Build/3.4.2-125_KOB_FFW-4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20330 = {S44078, 175, { &P13775,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB520 Build/Blur_Version.6.16.939.MB520.ATT.en.US Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40868 = {S44079, 147, { &P13775,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB520 Build/3.4.2-125_KOB_CCI-6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42862 = {S44080, 131, { &P13775,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB520 Build/STAB2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA44058 = {S44081, 174, { &P13775,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB520 Build/Blur_Version.35.9.94.MB520.ATT.en.US Flex/P012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA56509 = {S44082, 133, { &P13775,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB520 Build/3.4.2_8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA56513 = {S44083, 174, { &P13775,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB520 Build/Blur_Version.36.15.0.MB520.ATT.en.US Flex/P015) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58781 = {S44084, 177, { &P13775,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB520 Build/Blur_Version.38.6.0.MB520.Generic.en.US Flex/P002) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58782 = {S44085, 173, { &P13775,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB520 Build/Blur_Version.37.5.0.MB520.ATT.en.US Flex/P015) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA377390 = {S44086, 71, { &P13775,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; MB520 Build/3.4.2-125_KOB_FFW-4)*/
const Device UA2021592 = {S44087, 71, { &P13775,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; MB520 Build/3.4.2-125_KOB_CCI-6)*/
const Device UA129787 = {S44088, 89, { &P18182,&P5448,&P4536,&P18092 }};	/*Useragent - Opera/9.80 (Windows Mobile; WCE; Opera Mobi/WMD-50433; U; en) Presto/2.4.13 Version/10.00*/
const Device UA2547338 = {S44089, 102, { &P18182,&P5448,&P4536,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Windows Mobile; WCE; Opera Mobi/WMD-50433; U; de) Presto/2.4.13 Version/10.00*/
const Device UA1995672 = {S44090, 115, { &P18643,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia311/03.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.13.1*/
const Device UA2670758 = {S44091, 138, { &P15575,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2670905 = {S44092, 138, { &P15575,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2670902 = {S44093, 138, { &P15575,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2672517 = {S44094, 137, { &P15575,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9100 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676081 = {S44095, 137, { &P15575,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676575 = {S44096, 130, { &P15575,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2713237 = {S44097, 131, { &P15575,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2713854 = {S44098, 131, { &P15575,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2736316 = {S44099, 131, { &P15575,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA23035 = {S44100, 56, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - MAUI WAP Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23119 = {S44101, 85, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - NX-W702/V0.1.4 Release/24.8.2010 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA76471 = {S44102, 86, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - GHT PIANO/MTK Release/Oct-23-2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA90162 = {S44103, 85, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - GHT CHAT/MTK Release/Oct-23-2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA432838 = {S44104, 107, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - MTK_MAUI_01/07 Release/22.11.2007 Browser/wap2.0 Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA573612 = {S44105, 84, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - T23 WIFI Duo/MTKRelease/2011/07/01Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0*/
const Device UA2484602 = {S44106, 261, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - SAGETEL55_FTE_11B_HW (MRE\3.0.00(20480) resolution\X chipset\MT6255 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) V09-MX-KL01-T605-MP Release/2012.11.14 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2505952 = {S44107, 242, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - KP26_11B_HW (MRE\3.0.00(20480) resolution\X chipset\MT6255 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) V06_MX_KP26_T731_MP Release/2012.11.15 WAP Browser/MAUI (HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2506064 = {S44108, 250, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - SAGETEL55_FTE_11B_HW (MRE\3.0.00(20480) resolution\X chipset\MT6255 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) V07-MX-KL01-T605-MP Release/2012.11.03 WAP Browser/MAUI (HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2528565 = {S44109, 259, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - GIONEE50_12864_11B_HW (MRE\2.5.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) 1307SY_0201_V0802 Release/2012.10.17 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2556630 = {S44110, 264, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - TINNO50_6464_W1224_11B_HW (MRE\2.5.00(1536) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\1 gsensor\0 keyboard\reduced) F5100B_V14_X_2130 Release/2012.11.21 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 vi-VN*/
const Device UA2557754 = {S44111, 259, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - GIONEE50_12864_11B_HW (MRE\3.0.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) 1303QM_0201_V0933 Release/2012.11.20 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2582287 = {S44112, 261, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - SAGETEL55_FTE_11B_HW (MRE\3.0.00(20480) resolution\X chipset\MT6255 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) V13-MX-KL01-T605-MP Release/2012.12.29 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2597348 = {S44113, 254, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - TINNO50_12864_W1224_11B_HW (MRE\2.5.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\1 gsensor\0 keyboard\reduced) F7100B_V15_X_1548 Release/2012.11.22 WAP Browser/MAUI (HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 vi-VN*/
const Device UA2597527 = {S44114, 267, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - GIONEE50_12864_COSMOS_11B_HW (MRE\2.5.00(2500) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) 1302SY_0201_V1934 Release/2012.11.15 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2656092 = {S44115, 254, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - TINNO50_12864_W1224_11B_HW (MRE\2.5.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\1 gsensor\0 keyboard\reduced) F7100B_V13_X_1850 Release/2012.11.12 WAP Browser/MAUI (HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 vi-VN*/
const Device UA2656956 = {S44116, 424, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - TINNO50_6464_W1224_11B_HW (MRE/3.0.00(1536);MAUI/B_PP_F1_PK_F2_08_V9;BDATE/2012/11/06 15:53;LCD/X;CHIP/MT6250;KEY/Normal;TOUCH/0;CAMERA/1;SENSOR/0;DEV/TINNO50_6464_W1224_11B_HW;WAP Browser/MAUI (HTTP PGDL;HTTPS);GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/005;GAMELOFT/) B2040B_PP_F1_PK_F2_08_V9 Release/2012.11.06 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2657483 = {S44117, 269, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - RATECH50_12864_11B_HW (MRE\2.5.00(3072) resolution\X chipset\MT6250 touch\0 tpannel\0 camera\1 gsensor\0 keyboard\qwerty) E532_TV40_A1_11B_V4_13_1117 Release/2012.11.17 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2659041 = {S44118, 264, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - RATECH55_X_11B_HW (MRE\2.5.00(2560) resolution\X chipset\MT6255 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\qwerty) E630_T736_A1_11B_V5_8_1109 Release/2012.11.10 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2659435 = {S44119, 268, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - SYM.S103.GIONEE50_12864_11B_HW (MRE\3.0.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) 1303SY_0201_V0972 Release/2012.12.15 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2660458 = {S44120, 261, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - SAGETEL55_FTE_11B_HW (MRE\3.0.00(20480) resolution\X chipset\MT6255 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) V07-MX-KL02-T611-MP Release/2012.12.14 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2662340 = {S44121, 302, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - RATECH55_X_11B_HW (MRE\2.5.00(2560) resolution\X chipset\MT6255 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normalWAP Browser/MAUI (HTTP PGDL;HTTPS)) A638_T511_B1_MLA_11B_V2_6_1218 Release/2012.12.19 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2663753 = {S44122, 261, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - TINNO50_12864_W1224_11B_HW (MRE\2.5.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\1 gsensor\0 keyboard\reduced) F7100B_PP_F4_PK_F2_02_V9 Release/2012.12.11 WAP Browser/MAUI (HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2699330 = {S44123, 251, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - RATECH55_1G256_11B_HW (MRE\2.3.00(9560) resolution\X chipset\MT6255 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) X610_T721_A3_11B_V4_15_1213 Release/2012.12.14 WAP Browser/MAUI Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2703187 = {S44124, 252, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - RATECH55_1G256_11B_HW (MRE\2.3.00(1536) resolution\X chipset\MT6255 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) X615_6255_CY_TV70_B2_27S1116 Release/2012.11.16 WAP Browser/MAUI Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2705503 = {S44125, 261, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - SAGETEL55_FTE_11B_HW (MRE\3.0.00(20480) resolution\X chipset\MT6255 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) V11-MX-KL02-T611-MP Release/2013.01.24 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2707872 = {S44126, 259, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - GIONEE50_12864_11B_HW (MRE\3.0.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) 1303QM_0201_V0986 Release/2013.01.05 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2709413 = {S44127, 261, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - SAGETEL55_FTE_11B_HW (MRE\3.0.00(20480) resolution\X chipset\MT6255 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) V14-MX-KL01-T605-MP Release/2013.02.19 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2712457 = {S44128, 242, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - KP26_11B_HW (MRE\3.0.00(20480) resolution\X chipset\MT6255 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) V10_MX_KP26_T731_MP Release/2013.01.05 WAP Browser/MAUI (HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2726773 = {S44129, 252, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - RATECH55_1G256_11B_HW (MRE\2.3.00(1536) resolution\X chipset\MT6255 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) X615_6255_CY_TV70_B2_21S0716 Release/2012.07.16 WAP Browser/MAUI Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2728778 = {S44130, 268, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - RATECH55_X_11B_HW (MRE\2.5.00(2560) resolution\X chipset\MT6255 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\qwerty) E690_T638_A1_MLA_11B_V4_5_1108 Release/2012.11.09 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2729913 = {S44131, 268, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - RATECH55_X_11B_HW (MRE\2.5.00(2560) resolution\X chipset\MT6255 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) A638_T551_B1_MLA_11B_V5_3_1026 Release/2012.10.30 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2729970 = {S44132, 268, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - RATECH50_12864_11B_HW (MRE\2.5.00(3072) resolution\X chipset\MT6250 touch\0 tpannel\0 camera\1 gsensor\0 keyboard\qwerty) E532_T531_B2_11B_V3_6_1119 Release/2012.11.19 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2732184 = {S44133, 259, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - GIONEE50_12864_11B_HW (MRE\3.0.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) 1300QM_0201_V0525 Release/2013.01.11 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2734318 = {S44134, 425, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - TINNO50_6464_W1224_11B_HW (MRE/3.0.00(1536);MAUI/B_MP_F4_PK_F20_V8_07;BDATE/2012/12/10 21:07;LCD/X;CHIP/MT6250;KEY/Reduced;TOUCH/1;CAMERA/1;SENSOR/0;DEV/TINNO50_6464_W1224_11B_HW;WAP Browser/MAUI (HTTP PGDL;HTTPS);GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/005;GAMELOF) F5100B_MP_F4_PK_F20_V8_07 Release/2012.12.10 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2734458 = {S44135, 302, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - RATECH55_X_11B_HW (MRE\2.5.00(2560) resolution\X chipset\MT6255 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normalWAP Browser/MAUI (HTTP PGDL;HTTPS)) A638_T551_B1_MLA_11B_V5_8_0129 Release/2013.01.31 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2735303 = {S44136, 268, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - HONGYU50_COSMOS_11B_HW (MRE\3.0.00(2500) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) K60_karbonn_P108A_V1.4_X Release/2013.03.18 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2737000 = {S44137, 261, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - SAGETEL55_FTE_11B_HW (MRE\3.0.00(20480) resolution\X chipset\MT6255 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) V12-MX-KL02-T611-MP Release/2013.02.28 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2737112 = {S44138, 262, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - TINNO50_12864_W1224_11B_HW (MRE\2.5.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\1 gsensor\0 keyboard\reduced) F7100B_PP_F4_PK_F2_02_V11 Release/2013.03.07 WAP Browser/MAUI (HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2778390 = {S44139, 267, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - GIONEE50_12864_COSMOS_11B_HW (MRE\2.5.00(2500) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) 1302SY_0201_V1915 Release/2012.10.20 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2780118 = {S44140, 259, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - GIONEE50_12864_11B_HW (MRE\3.0.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) 1305QM_0201_V1958 Release/2013.01.05 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2792288 = {S44141, 259, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - GIONEE50_12864_11B_HW (MRE\2.5.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) 1305GA_0201_V1711 Release/2012.10.24 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2793860 = {S44142, 302, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - RATECH55_X_11B_HW (MRE\2.5.00(2560) resolution\X chipset\MT6255 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normalWAP Browser/MAUI (HTTP PGDL;HTTPS)) A638_T511_B1_MLA_11B_V2_9_0304 Release/2013.03.05 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2794336 = {S44143, 261, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - SAGETEL55_FTE_11B_HW (MRE\3.0.00(20480) resolution\X chipset\MT6255 touch\0 tpannel\0 camera\1 gsensor\0 keyboard\normal) V09_MX_KL16_TV30_MP Release/2013.02.07 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2844836 = {S44144, 265, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - TINNO50_6464_W1224_11B_HW (MRE\2.5.00(800) resolution\X chipset\MT6250 touch\0 tpannel\0 camera\1 gsensor\0 keyboard\normal) B2040A_B18TV_X_V1.05 Release/2012.10.01 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2853072 = {S44145, 267, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - GIONEE50_12864_COSMOS_11B_HW (MRE\2.5.00(2500) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) 1302GI_0201_V1957 Release/2012.12.25 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2854830 = {S44146, 261, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - SAGETEL55_FTE_11B_HW (MRE\3.0.00(20480) resolution\X chipset\MT6255 touch\0 tpannel\0 camera\1 gsensor\0 keyboard\normal) V02_MX_KL16_T608_MP Release/2012.12.29 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2855697 = {S44147, 302, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - RATECH55_X_11B_HW (MRE\2.5.00(2560) resolution\X chipset\MT6255 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normalWAP Browser/MAUI (HTTP PGDL;HTTPS)) A638_T551_B1_MLA_11B_V5_7_0108 Release/2013.01.09 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2858838 = {S44148, 250, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - RATECH55_1G256_11B_HW (MRE\2.3.00(9560) resolution\X chipset\MT6255 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) X601_T751_B3_11B_V5_6_1121 Release/2012.11.24 WAP Browser/MAUI Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2865671 = {S44149, 301, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - RATECH55_X_11B_HW (MRE\2.5.00(3072) resolution\X chipset\MT6255 touch\0 tpannel\0 camera\1 gsensor\0 keyboard\normalWAP Browser/MAUI (HTTP PGDL;HTTPS)) A685_T21_A1_MLA_11B_V4_5_0411 Release/2013.04.11 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2868659 = {S44150, 255, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - KP515_T756_HW (MRE\3.0.00(20480) resolution\X chipset\MT6255 touch\1 tpannel\1 camera\1 gsensor\0 keyboard\qwerty) V05-MX-KP515-T756-MP Release/2013.02.26 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2892485 = {S44151, 301, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - RATECH55_X_11B_HW (MRE\2.5.00(3072) resolution\X chipset\MT6255 touch\0 tpannel\0 camera\1 gsensor\0 keyboard\normalWAP Browser/MAUI (HTTP PGDL;HTTPS)) A685_T21_A1_MLA_11B_V4_4_0330 Release/2013.03.30 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2934609 = {S44152, 251, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - RATECH55_1G256_11B_HW (MRE\2.3.00(1536) resolution\X chipset\MT6255 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) X615_6255_CY_M50_B2_07S1031 Release/2012.10.31 WAP Browser/MAUI Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2935897 = {S44153, 259, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - GIONEE50_12864_11B_HW (MRE\3.0.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) 1303GA_0201_V0997 Release/2013.01.18 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2935967 = {S44154, 302, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - RATECH55_X_11B_HW (MRE\2.5.00(2560) resolution\X chipset\MT6255 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normalWAP Browser/MAUI (HTTP PGDL;HTTPS)) A638_T511_B1_MLA_11B_V2_7_0109 Release/2013.01.10 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2956139 = {S44155, 261, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - SAGETEL55_FTE_11B_HW (MRE\3.0.00(20480) resolution\X chipset\MT6255 touch\0 tpannel\0 camera\1 gsensor\0 keyboard\normal) V04_MX_KL16_TV65_MP Release/2013.02.20 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2957275 = {S44156, 265, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - SAGETEL55_FTE_11B_HW (MRE\3.0.00(20480) resolution\X chipset\MT6255 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) V04-MX-KL01-T605-ETH-MP Release/2013.01.23 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2986458 = {S44157, 268, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - RATECH55_X_11B_HW (MRE\2.5.00(2560) resolution\X chipset\MT6255 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\qwerty) E690_T638_A1_MLA_11B_V5_2_1221 Release/2012.12.22 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2987226 = {S44158, 259, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - GIONEE50_12864_11B_HW (MRE\3.0.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) 1305QM_0201_V1944 Release/2012.12.15 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2988030 = {S44159, 259, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - GIONEE50_12864_11B_HW (MRE\3.0.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) 1303GI_0202_V0981 Release/2012.12.26 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2989071 = {S44160, 266, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - RATECH55_X_11B_HW (MRE\3.0.00(2560) resolution\X chipset\MT6255 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\qwerty) E630_T736_A1_11B_V5_9_1123_S Release/2012.11.28 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2994417 = {S44161, 259, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - GIONEE50_12864_11B_HW (MRE\3.0.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) 1303GA_0201_V1056 Release/2013.04.02 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2994974 = {S44162, 264, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - SAGETEL55_FTE_11B_HW (MRE\3.0.00(20480) resolution\X chipset\MT6255 touch\0 tpannel\0 camera\1 gsensor\0 keyboard\normal) V02_MX_KL16_T608_ET_MP Release/2013.03.13 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2995140 = {S44163, 259, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - GIONEE50_12864_11B_HW (MRE\2.5.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) 1307SY_0201_V1019 Release/2013.01.30 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2995219 = {S44164, 267, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - GIONEE50_12864_COSMOS_11B_HW (MRE\2.5.00(2500) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) 1302SY_0201_V1960 Release/2013.01.23 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2996763 = {S44165, 250, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - RATECH55_1G256_11B_HW (MRE\2.3.00(9560) resolution\X chipset\MT6255 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) X601_T751_B3_11B_V6_0_0201 Release/2013.02.01 WAP Browser/MAUI Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2998047 = {S44166, 261, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - SAGETEL55_FTE_11B_HW (MRE\3.0.00(20480) resolution\X chipset\MT6255 touch\0 tpannel\0 camera\1 gsensor\0 keyboard\normal) V04_MX_KL16_T608_MP Release/2013.03.15 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2998333 = {S44167, 268, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - RATECH55_X_11B_HW (MRE\2.5.00(2560) resolution\X chipset\MT6255 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\qwerty) E690_T638_A1_MLA_11B_V5_4_0204 Release/2013.02.04 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2999289 = {S44168, 425, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - TINNO50_6464_W1224_11B_HW (MRE/3.0.00(1536);MAUI/D_MP_F4_PK_F20_V8_09;BDATE/2013/03/05 10:08;LCD/X;CHIP/MT6250;KEY/Reduced;TOUCH/1;CAMERA/1;SENSOR/0;DEV/TINNO50_6464_W1224_11B_HW;WAP Browser/MAUI (HTTP PGDL;HTTPS);GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/005;GAMELOF) F5100D_MP_F4_PK_F20_V8_09 Release/2013.03.05 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA3000796 = {S44169, 267, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - SYM.D67.GIONEE50_12864_11B_HW (MRE\3.0.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) 1300SY_0201_V0543 Release/2013.03.21 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA3002886 = {S44170, 268, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - SYM.S103.GIONEE50_12864_11B_HW (MRE\3.0.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) 1303SY_0201_V1060 Release/2013.04.18 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA3005895 = {S44171, 267, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - SYM.D67.GIONEE50_12864_11B_HW (MRE\3.0.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) 1300SY_0201_V0557 Release/2013.04.18 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA3028329 = {S44172, 413, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - HONGYU50_COSMOS_11B_HW (MRE/3.0.00(2500);MAUI/_GLT01_P108B_KON_X;BDATE/2013/05/13 15:41;LCD/X;CHIP/MT6250;KEY/Reduced;TOUCH/1;CAMERA/0;SENSOR/0;DEV/HONGYU50_COSMOS_11B_HW;WAP Browser/MAUI (HTTP PGDL;HTTPS);GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/005;GAMEL) C737D_GLT01_P108B_KON_X Release/2013.05.13 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA3045804 = {S44173, 259, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - GIONEE50_12864_11B_HW (MRE\3.0.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) 1305GA_0201_V1952 Release/2012.12.25 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA3048390 = {S44174, 266, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - RATECH55_X_11B_HW (MRE\3.0.00(2560) resolution\X chipset\MT6255 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\qwerty) E630_T736_A1_11B_V6_0_0222_S Release/2013.02.22 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA3049424 = {S44175, 259, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - GIONEE50_12864_11B_HW (MRE\3.0.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) 1303GI_0202_V1039 Release/2013.03.13 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA3049746 = {S44176, 427, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - TINNO50_6464_W1224_11B_HW (MRE/3.0.00(1536);MAUI/B_MP_F3F8_PK_QMB_02_10;BDATE/2013/02/06 11:36;LCD/X;CHIP/MT6250;KEY/Normal;TOUCH/0;CAMERA/1;SENSOR/0;DEV/TINNO50_6464_W1224_11B_HW;WAP Browser/MAUI (HTTP PGDL;HTTPS);GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/005;GAMELO) B2060B_MP_F3F8_PK_QMB_02_10 Release/2013.02.06 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA3056781 = {S44177, 427, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - TINNO50_6464_W1224_11B_HW (MRE/3.0.00(1536);MAUI/B_MP_F3F8_PK_QMB_05_11;BDATE/2013/03/28 15:17;LCD/X;CHIP/MT6250;KEY/Normal;TOUCH/0;CAMERA/1;SENSOR/0;DEV/TINNO50_6464_W1224_11B_HW;WAP Browser/MAUI (HTTP PGDL;HTTPS);GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/005;GAMELO) B2070B_MP_F3F8_PK_QMB_05_11 Release/2013.03.28 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA3057812 = {S44178, 430, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - SYM.FT33.HONGYU50_COSMOS_11B_HW (MRE/3.0.00(2500);MAUI/ONY_FT33_C737D1_M01_S04B_X;BDATE/2013/05/24 10:52;LCD/X;CHIP/MT6250;KEY/Reduced;TOUCH/1;CAMERA/0;SENSOR/0;DEV/HONGYU50_COSMOS_11B_HW;WAP Browser/MAUI (HTTP PGDL;HTTPS);GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/0) SYMPHONY_FT33_C737D1_M01_S04B_X Release/2013.05.24 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA3057870 = {S44179, 262, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - TINNO50_12864_W1224_11B_HW (MRE\2.5.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\1 gsensor\0 keyboard\reduced) F7101A_TW30TV_DUO_X_V16_0 Release/2013.06.09 WAP Browser/MAUI (HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA3074080 = {S44180, 259, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - GIONEE50_12864_11B_HW (MRE\3.0.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) 1303GA_0201_V1058 Release/2013.04.17 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA3079851 = {S44181, 262, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - TINNO50_12864_W1224_11B_HW (MRE\2.5.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\1 gsensor\0 keyboard\reduced) F7100B_PP_F4_PK_F2_02_V14 Release/2013.07.01 WAP Browser/MAUI (HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA3088348 = {S44182, 259, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - GIONEE50_12864_11B_HW (MRE\3.0.00(3072) resolution\X chipset\MT6250 touch\1 tpannel\0 camera\1 gsensor\0 keyboard\normal) 1305GA_0201_V1968 Release/2013.03.23 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA3089007 = {S44183, 255, { &P18088,&P17017,&P4780,&P18092 }};	/*Useragent - KP515_T756_HW (MRE\3.0.00(20480) resolution\X chipset\MT6255 touch\1 tpannel\1 camera\1 gsensor\0 keyboard\qwerty) V08-MX-KP515-T756-MP Release/2013.04.02 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA401754 = {S44184, 125, { &P15364,&P7133,&P22455,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11*/
const Device UA409919 = {S44185, 118, { &P15364,&P7133,&P22455,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11*/
const Device UA2470943 = {S44186, 133, { &P15364,&P18118,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Ubuntu Chromium/23.0.1271.97 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2470944 = {S44187, 131, { &P15364,&P18118,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.11 (KHTML, like Gecko) Ubuntu Chromium/23.0.1271.97 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2986363 = {S44188, 141, { &P15364,&P18118,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.11 (KHTML, like Gecko) Ubuntu Chromium/23.0.1271.97 Chrome/23.0.1271.97 Safari/537.11,gzip(gfe)*/
const Device UA2719571 = {S44189, 112, { &P22579,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 925)*/
const Device UA2800205 = {S44190, 122, { &P22579,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 925; Vodafone)*/
const Device UA2938110 = {S44191, 120, { &P22579,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 925; ANZ892)*/
const Device UA3003037 = {S44192, 120, { &P22579,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 925; Orange)*/
const Device UA3023569 = {S44193, 130, { &P22579,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 925; BOUYGUES TELECOM)*/
const Device UA3066654 = {S44194, 120, { &P22579,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 925; 1326-2)*/
const Device UA2570463 = {S44195, 158, { &P18283,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2772767 = {S44196, 157, { &P18283,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2773082 = {S44197, 157, { &P18283,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2773881 = {S44198, 157, { &P18283,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2774048 = {S44199, 157, { &P18283,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2774055 = {S44200, 158, { &P18283,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2774071 = {S44201, 157, { &P18283,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799253 = {S44202, 157, { &P18283,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; GT-I9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2868676 = {S44203, 158, { &P18283,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999V Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2938009 = {S44204, 157, { &P18283,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-I535 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA924970 = {S44205, 119, { &P15364,&P17285,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA924979 = {S44206, 126, { &P15364,&P17285,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA94063 = {S44207, 60, { &P18088,&P17017,&P18123,&P18092 }};	/*Useragent - Opera/8.01 (J2ME/MIDP; Opera Mini/2.0.4509/1724; en; U; ssr)*/
const Device UA2389386 = {S44208, 51, { &P18088,&P17017,&P18123,&P18092 }};	/*Useragent - Opera/9.50 (J2ME/MIDP; Opera Mini/2/28.3165; U; en)*/
const Device UA64381 = {S44209, 155, { &P14453,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE51-2/411.34.001; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA64776 = {S44210, 154, { &P14453,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE51-1/100.34.20; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA101035 = {S44211, 68, { &P18254,&P17350,&P17384,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows NT; MS Search 5.0 Robot)*/
const Device UA108786 = {S44212, 68, { &P18254,&P17350,&P17384,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows NT; MS Search 6.0 Robot)*/
const Device UA2166821 = {S44213, 153, { &P15354,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyEricssonST17i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257425 = {S44214, 153, { &P15354,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; SonyEricssonST17a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2418912 = {S44215, 63, { &P15354,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ST17i Build/4.1.B.0.587)*/
const Device UA2466137 = {S44216, 141, { &P15354,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ST17i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2787021 = {S44217, 141, { &P15354,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; ST17i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2787195 = {S44218, 141, { &P15354,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; ST17a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2043995 = {S44219, 112, { &P18591,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 810)*/
const Device UA880844 = {S44220, 139, { &P18316,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; LG-F160S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA886662 = {S44221, 139, { &P18316,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; LG-F160L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899488 = {S44222, 139, { &P18316,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; LG-F160K Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028256 = {S44223, 61, { &P18316,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-F160K Build/IML74K)*/
const Device UA2028496 = {S44224, 139, { &P18316,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F160L Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031746 = {S44225, 61, { &P18316,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-F160L Build/IML74K)*/
const Device UA2043760 = {S44226, 137, { &P18316,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; LG Optimus LTE2 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2175684 = {S44227, 140, { &P18316,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F160LV Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2396859 = {S44228, 61, { &P18316,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-F160L Build/IMM76I)*/
const Device UA2495808 = {S44229, 61, { &P18316,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-F160S Build/IML74K)*/
const Device UA2500706 = {S44230, 138, { &P18316,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; LG-F160 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2692336 = {S44231, 62, { &P18316,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-F160LV Build/IMM76I)*/
const Device UA2693057 = {S44232, 143, { &P18316,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; LG-F160L Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?i Safari/534.30*/
const Device UA2817161 = {S44233, 129, { &P18316,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; LG-F160 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA93287 = {S44234, 114, { &P15364,&P5690,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Version/3.1.2 Safari/525.21*/
const Device UA93322 = {S44235, 114, { &P15364,&P5690,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.17*/
const Device UA104972 = {S44236, 112, { &P15364,&P5690,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13*/
const Device UA122454 = {S44237, 112, { &P15364,&P5690,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Version/3.1 Safari/525.19*/
const Device UA278856 = {S44238, 111, { &P15364,&P5690,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Version/3.1 Safari/525.1*/
const Device UA898613 = {S44239, 114, { &P15364,&P5690,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; ru-RU) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13.3*/
const Device UA899220 = {S44240, 114, { &P15364,&P5690,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; ca-es) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20*/
const Device UA1861669 = {S44241, 105, { &P15364,&P5690,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13*/
const Device UA1876197 = {S44242, 107, { &P15364,&P5690,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en) AppleWebKit/525.9 (KHTML, like Gecko) Version/3.1 Safari/525.9*/
const Device UA1953429 = {S44243, 114, { &P15364,&P5690,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; pt-BR) AppleWebKit/525.19 (KHTML, like Gecko) Version/3.1.2 Safari/525.21*/
const Device UA2302447 = {S44244, 112, { &P15364,&P5690,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.26 (KHTML, like Gecko) Version/3.1 Safari/525.26*/
const Device UA2741709 = {S44245, 129, { &P18639,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 10 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2773997 = {S44246, 130, { &P18639,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 10 Build/JOP40F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2782798 = {S44247, 128, { &P18639,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Nexus 10 Build/JVP15P) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2566524 = {S44248, 138, { &P12972,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566641 = {S44249, 138, { &P12972,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567011 = {S44250, 139, { &P12972,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567239 = {S44251, 139, { &P12972,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989D Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2601592 = {S44252, 146, { &P12972,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-T989 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2862478 = {S44253, 131, { &P12972,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2989218 = {S44254, 131, { &P12972,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2428454 = {S44255, 136, { &P17595,&P18207,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/21.0.1180.75 Safari/537.1*/
const Device UA2583284 = {S44256, 136, { &P17595,&P18207,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.3.6000 Chrome/21.0.1180.75 Safari/537.1*/
const Device UA2739971 = {S44257, 138, { &P17595,&P18207,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.22 (KHTML, like Gecko) Maxthon/4.0.4.3000 Chrome/25.0.1364.99 Safari/537.22*/
const Device UA2883823 = {S44258, 138, { &P17595,&P18207,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.22 (KHTML, like Gecko) Maxthon/4.0.5.2000 Chrome/25.0.1364.99 Safari/537.22*/
const Device UA2963084 = {S44259, 138, { &P17595,&P18207,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.22 (KHTML, like Gecko) Maxthon/4.0.5.2000 Chrome/25.0.1364.99 Safari/537.22*/
const Device UA3029635 = {S44260, 138, { &P17595,&P18207,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.22 (KHTML, like Gecko) Maxthon/4.0.6.3000 Chrome/25.0.1364.99 Safari/537.22*/
const Device UA3095231 = {S44261, 138, { &P17595,&P18207,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.22 (KHTML, like Gecko) Maxthon/4.0.6.3000 Chrome/25.0.1364.99 Safari/537.22*/
const Device UA349573 = {S44262, 145, { &P16263,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; HUAWEI MediaPad Build/HuaweiMediaPad) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA395712 = {S44263, 138, { &P16263,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; MediaPad Build/HuaweiMediaPad) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA633278 = {S44264, 141, { &P16263,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; SpringBoard Build/HuaweiMediaPad) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1943076 = {S44265, 74, { &P16263,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; HUAWEI MediaPad Build/HuaweiMediaPad)*/
const Device UA2021741 = {S44266, 70, { &P16263,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; SpringBoard Build/HuaweiMediaPad)*/
const Device UA2009012 = {S44267, 74, { &P20360,&P22839,&P17394,&P18092 }};	/*Useragent - ZTE-Z431/1.4.0 NetFront/4.2 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2433590 = {S44268, 74, { &P20360,&P22839,&P17394,&P18092 }};	/*Useragent - ZTE-Z431/1.7.0 NetFront/4.2 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2773066 = {S44269, 143, { &P12280,&P18117,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9B206 Safari/7534.48.3*/
const Device UA2773496 = {S44270, 143, { &P12280,&P18117,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9B208 Safari/7534.48.3*/
const Device UA2783672 = {S44271, 141, { &P12280,&P18117,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9B179 Safari/7534.48.3*/
const Device UA2784156 = {S44272, 141, { &P12280,&P18117,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9B176 Safari/7534.48.3*/
const Device UA909526 = {S44273, 62, { &P17976,&P17979,&P24193,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:16.0) Gecko/16.0 Firefox/16.0*/
const Device UA1996242 = {S44274, 68, { &P17976,&P17979,&P24193,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:16.0a2) Gecko/16.0a2 Firefox/16.0a2*/
const Device UA2204452 = {S44275, 68, { &P17976,&P17979,&P24193,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:16.0.1) Gecko/16.0.1 Firefox/16.0.1*/
const Device UA2229555 = {S44276, 99, { &P17976,&P17979,&P24193,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:16.0) Gecko/16.0 Firefox/16.0,gzip(gfe) (via translate.google.com)*/
const Device UA2236865 = {S44277, 68, { &P17976,&P17979,&P24193,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:16.0.2) Gecko/16.0.2 Firefox/16.0.2*/
const Device UA2256941 = {S44278, 72, { &P17976,&P17979,&P24193,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:16.0) Gecko/16.0 Firefox/16.0,gzip(gfe)*/
const Device UA2386240 = {S44279, 141, { &P18223,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2502917 = {S44280, 138, { &P18223,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xperia P Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2531304 = {S44281, 141, { &P18223,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LT22i Build/6.1.1.B.1.75) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2981195 = {S44282, 141, { &P18223,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  LT22i Build/6.1.B.0.544) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA2413772 = {S44283, 128, { &P19862,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2426292 = {S44284, 128, { &P19862,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2529018 = {S44285, 57, { &P19862,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; A110 Build/JZO54K)*/
const Device UA2583922 = {S44286, 144, { &P19862,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; Micromax A110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2704729 = {S44287, 155, { &P19862,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Micromax A110 (Canvas 2) Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2727268 = {S44288, 57, { &P19862,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; A110 Build/JRO03C)*/
const Device UA2749162 = {S44289, 66, { &P19862,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Micromax A110 Build/JRO03C)*/
const Device UA2956070 = {S44290, 144, { &P19862,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Micromax A110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997705 = {S44291, 141, { &P19862,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; Raven A110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3045239 = {S44292, 144, { &P19862,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A110 Canvas 2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3047198 = {S44293, 144, { &P19862,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; Micromax A110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099658 = {S44294, 135, { &P19862,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3101582 = {S44295, 142, { &P19862,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; Micromax A110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA213186 = {S44296, 229, { &P15364,&P7133,&P21898,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/13.0.782.107; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; InfoPath.3; MS-RTC LM 8)*/
const Device UA236558 = {S44297, 96, { &P15364,&P7133,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/13.0.782.215)*/
const Device UA335329 = {S44298, 89, { &P15364,&P7133,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/13.0.782.215)*/
const Device UA628759 = {S44299, 125, { &P15364,&P7133,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/13.0.782.215) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.215 Safari/535.1*/
const Device UA634841 = {S44300, 132, { &P15364,&P7133,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/13.0.782.215) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.215 Safari/535.1*/
const Device UA660043 = {S44301, 125, { &P15364,&P7133,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/13.0.782.112) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1*/
const Device UA769035 = {S44302, 132, { &P15364,&P7133,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/13.0.782.107) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1*/
const Device UA780689 = {S44303, 132, { &P15364,&P7133,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/13.0.782.220) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1*/
const Device UA806976 = {S44304, 132, { &P15364,&P7133,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/13.0.782.112) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1*/
const Device UA878256 = {S44305, 125, { &P15364,&P7133,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/13.0.782.220) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1*/
const Device UA2218204 = {S44306, 240, { &P15364,&P7133,&P21898,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/13.0.782.215; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; InfoPath.3; .NET4.0C; .NET4.0E; BOIE9;FRFR)*/
const Device UA2259061 = {S44307, 125, { &P15364,&P7133,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/13.0.782.107) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1*/
const Device UA2297839 = {S44308, 202, { &P15364,&P7133,&P21898,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/13.0.782.220; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0)*/
const Device UA2473419 = {S44309, 195, { &P15364,&P7133,&P21898,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/13.0.782.107; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3)*/
const Device UA2641951 = {S44310, 203, { &P15364,&P7133,&P21898,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/13.0.782.107; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.2; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA2986413 = {S44311, 207, { &P15364,&P7133,&P21898,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/13.0.782.107; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3)*/
const Device UA567814 = {S44312, 63, { &P18254,&P17285,&P17190,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0 ; Claritybot)*/
const Device UA2741647 = {S44313, 138, { &P18295,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754642 = {S44314, 134, { &P18295,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; One X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755546 = {S44315, 138, { &P18295,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EndeavorU Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773350 = {S44316, 138, { &P18295,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937629 = {S44317, 138, { &P18295,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; HTC One X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA92768 = {S44318, 43, { &P15364,&P18146,&P17470,&P18092 }};	/*Useragent - Mozilla/4.02 [en] (X11; I; SunOS 5.6 sun4u)*/
const Device UA105708 = {S44319, 44, { &P15364,&P18146,&P17470,&P18092 }};	/*Useragent - Mozilla/3.01Gold (X11; I; SunOS 5.5.1 sun4m)*/
const Device UA109744 = {S44320, 43, { &P15364,&P18146,&P17470,&P18092 }};	/*Useragent - Mozilla/4.76 [ru] (X11; U; SunOS 5.7 sun4u)*/
const Device UA123527 = {S44321, 43, { &P15364,&P18146,&P17470,&P18092 }};	/*Useragent - Mozilla/4.76 [en] (X11; U; SunOS 5.7 sun4u)*/
const Device UA148324 = {S44322, 45, { &P15364,&P18146,&P17470,&P18092 }};	/*Useragent - Mozilla/4.05 [en] (X11; I; SunOS 4.1.4 sun4m)*/
const Device UA169390 = {S44323, 37, { &P15364,&P18146,&P17470,&P18092 }};	/*Useragent - Mozilla/3.0 (X11; I; SunOS 5.4 sun4m)*/
const Device UA173893 = {S44324, 39, { &P15364,&P18146,&P17470,&P18092 }};	/*Useragent - NCSA Mosaic/1.0 (X11;SunOS 4.1.4 sun4m)*/
const Device UA302383 = {S44325, 43, { &P15364,&P18146,&P17470,&P18092 }};	/*Useragent - Mozilla/4.61 [en] (X11; U; SunOS 5.6 sun4u)*/
const Device UA515290 = {S44326, 58, { &P15364,&P18146,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; SunOS sun4u; en-US; rv:1.2.1) Gecko/X*/
const Device UA152739 = {S44327, 165, { &P15364,&P7133,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; WOW64; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA523678 = {S44328, 55, { &P15364,&P7133,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; SV1)*/
const Device UA529916 = {S44329, 48, { &P15364,&P7133,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible;MSIE 6.0;Windows NT 6.1)*/
const Device UA630865 = {S44330, 63, { &P15364,&P7133,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; Trident/5.0)*/
const Device UA747022 = {S44331, 63, { &P15364,&P7133,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 6.1; Trident/5.0)*/
const Device UA785540 = {S44332, 70, { &P15364,&P7133,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 6.1; WOW64; Trident/5.0)*/
const Device UA833968 = {S44333, 158, { &P15364,&P7133,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2217151 = {S44334, 166, { &P15364,&P7133,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)*/
const Device UA2277281 = {S44335, 132, { &P15364,&P7133,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322)*/
const Device UA2642480 = {S44336, 75, { &P15364,&P7133,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 6.1; Win64; x64; Trident/5.0)*/
const Device UA2727514 = {S44337, 195, { &P15364,&P7133,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; MS-RTC LM 8)*/
const Device UA2816891 = {S44338, 94, { &P15364,&P7133,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; WOW64; SLCC2; .NET CLR 2.0.50727; .NET4.0C)*/
const Device UA2834991 = {S44339, 79, { &P15364,&P7133,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows NT 6.1; Trident/5.0)*/
const Device UA2259177 = {S44340, 82, { &P15364,&P7133,&P20761,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:16.0) Gecko/X Firefox/16.0 SeaMonkey/2.13.2*/
const Device UA2386417 = {S44341, 69, { &P15364,&P7133,&P20761,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:16.0) Gecko/X SeaMonkey/2.13.2*/
const Device UA2354618 = {S44342, 132, { &P22043,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P5113 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2362479 = {S44343, 132, { &P22043,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P5113 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA94690 = {S44344, 62, { &P15364,&P17285,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; en) Presto/2.2.15 Version/10.01*/
const Device UA109136 = {S44345, 62, { &P15364,&P17285,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; de) Presto/2.2.15 Version/10.00*/
const Device UA175123 = {S44346, 85, { &P15364,&P17285,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.3 (build 02564); ru) Presto/2.2.15 Version/10.00*/
const Device UA199773 = {S44347, 63, { &P15364,&P17285,&P4812,&P18092 }};	/*Useragent - Opera/10.00 (Windows NT 6.0; U; Edition Turbo; en) Presto/2.2.0*/
const Device UA442134 = {S44348, 66, { &P15364,&P17285,&P4812,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.0; ja) Opera 10.00*/
const Device UA2382314 = {S44349, 149, { &P19894,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-R830 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 USCC-R830*/
const Device UA2457995 = {S44350, 61, { &P19894,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-R830 Build/IMM76D)*/
const Device UA2671058 = {S44351, 105, { &P15364,&P5690,&P17202,&P18091 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.6) Gecko/X Firefox/2.0.0.6 Ara.com.tr AraBot 1.0*/
const Device UA5155 = {S44352, 132, { &P12216,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; ADR6300Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA5158 = {S44353, 132, { &P12216,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us;ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA5159 = {S44354, 132, { &P12216,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA5160 = {S44355, 132, { &P12216,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA5162 = {S44356, 132, { &P12216,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA28075 = {S44357, 133, { &P12216,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA28084 = {S44358, 135, { &P12216,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; ADR6300 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA56070 = {S44359, 135, { &P12216,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67874 = {S44360, 140, { &P12216,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; ADR6300 Build/FRF91) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA140848 = {S44361, 57, { &P12216,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; ADR6300 Build/FRF91)*/
const Device UA2020641 = {S44362, 121, { &P12216,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; ADR6300 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA623908 = {S44363, 140, { &P15390,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P925/V20p Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA690333 = {S44364, 135, { &P15390,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P925 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA874536 = {S44365, 59, { &P15390,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; LG-P925 Build/GRJ90)*/
const Device UA2554127 = {S44366, 163, { &P15390,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P925/V20p Build/PromethiusProject v1.5.1 20P) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2933268 = {S44367, 135, { &P15390,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-mx; LG-P925 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Versi?n/4.0 Mobile Safari/533.1*/
const Device UA2945958 = {S44368, 157, { &P18283,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2946986 = {S44369, 157, { &P18283,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2965602 = {S44370, 157, { &P18283,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T999 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3020512 = {S44371, 158, { &P18283,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3030579 = {S44372, 158, { &P18283,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3031103 = {S44373, 157, { &P18283,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3032893 = {S44374, 157, { &P18283,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2489857 = {S44375, 123, { &P17346,&P19826,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Twitter for iPhone*/
const Device UA2489951 = {S44376, 123, { &P17346,&P19826,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Twitter for iPhone*/
const Device UA2532610 = {S44377, 125, { &P17346,&P19826,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Twitter for iPhone*/
const Device UA2585559 = {S44378, 125, { &P17346,&P19826,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Twitter for iPhone*/
const Device UA2489342 = {S44379, 112, { &P18557,&P20706,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia308/05.85; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA166839 = {S44380, 135, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; HTC HD2 Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA169560 = {S44381, 136, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC HD2 Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA176576 = {S44382, 135, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC HD2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA179328 = {S44383, 135, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; nl-nl; HTC HD2 Build/GRI54) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA187409 = {S44384, 135, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC HD2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA190692 = {S44385, 131, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HD2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA192055 = {S44386, 135, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC HD2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA209017 = {S44387, 135, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC HD2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA209803 = {S44388, 135, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC HD2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA241287 = {S44389, 131, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HD2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250032 = {S44390, 135, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC HD2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA252233 = {S44391, 132, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HD2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA304436 = {S44392, 135, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC HD2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA306639 = {S44393, 136, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC HD2 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA315554 = {S44394, 135, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC HD2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369842 = {S44395, 147, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ie; HTC HD2 Build/1.4.1-MIUIVANILLA) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA377013 = {S44396, 144, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HD2 Gingerbread Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA387297 = {S44397, 135, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; HTC HD2 Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA624890 = {S44398, 59, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC HD2 Build/FRF91)*/
const Device UA692125 = {S44399, 59, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC HD2 Build/GRI40)*/
const Device UA693815 = {S44400, 135, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC HD2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA727261 = {S44401, 131, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; HD2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA782411 = {S44402, 132, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HD2 Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA819282 = {S44403, 135, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; HTC HD2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA891484 = {S44404, 135, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC HD2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1865574 = {S44405, 147, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC HD2 Gingerbread Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1871943 = {S44406, 59, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC HD2 Build/FRF91)*/
const Device UA1946218 = {S44407, 130, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HD2 Build/HSTR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1990293 = {S44408, 144, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC HD2 Build/JaguarMIUI-V21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018478 = {S44409, 59, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.1; HTC HD2 Build/GRH78)*/
const Device UA2018599 = {S44410, 131, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; HD2 Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019692 = {S44411, 144, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC HD2 Build/JaguarMIUI-V14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019803 = {S44412, 146, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC HD2 Build/JaguaraMIUI V-15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021045 = {S44413, 131, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; HD2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021947 = {S44414, 123, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC HD2 Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022284 = {S44415, 59, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC HD2 Build/GRJ90)*/
const Device UA2023127 = {S44416, 56, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; HD2 Build/GRK39F)*/
const Device UA2023873 = {S44417, 144, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC HD2 Build/JaguarMIUI-V20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026007 = {S44418, 133, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; T8585 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034043 = {S44419, 144, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC HD2 Build/GWK74; MokeeOS) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2404378 = {S44420, 136, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; NexusHD2 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982718 = {S44421, 135, { &P13999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC HD2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2714169 = {S44422, 70, { &P15364,&P17285,&P22171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:23.0) Gecko/X Firefox/23.0*/
const Device UA2963923 = {S44423, 68, { &P15364,&P17285,&P22171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:23.0) Gecko/X Firefox/23.0,gzip(gfe)*/
const Device UA2258140 = {S44424, 113, { &P18463,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; SAMSUNG; GT-I8750)*/
const Device UA2349262 = {S44425, 114, { &P18463,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; SAMSUNG; SGH-T899M)*/
const Device UA23036 = {S44426, 78, { &P18095,&P18094,&P4536,&P18092 }};	/*Useragent - Opera/9.80 (S60; SymbOS; Opera Mobi/498; U; en-GB) Presto/2.4.18 Version/10.00*/
const Device UA359637 = {S44427, 115, { &P15140,&P17483,&P4732,&P18092 }};	/*Useragent - LG-GT550/V11d; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.18377; U; en-US) Opera 9.50*/
const Device UA2428545 = {S44428, 229, { &P18321,&P5629,&P17373,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/6.5.27431/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2435243 = {S44429, 227, { &P18321,&P5629,&P17373,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/6.5.27431/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2706475 = {S44430, 119, { &P15364,&P18110,&P21712,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) CasperJS/1.0.2+Phantomjs/1.9.0 Safari/534.34*/
const Device UA2717122 = {S44431, 117, { &P15364,&P18110,&P21712,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) CasperJS/1.0.2+Phantomjs/1.9.0 Safari/534.34*/
const Device UA3039873 = {S44432, 141, { &P12280,&P22882,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.3 Mobile/11A4414e Safari/8536.25*/
const Device UA3039874 = {S44433, 141, { &P12280,&P22882,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.5 Mobile/11A4435d Safari/8536.25*/
const Device UA3082796 = {S44434, 140, { &P12280,&P22882,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A465 Safari/8536.25*/
const Device UA3082799 = {S44435, 142, { &P12280,&P22882,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A4449d Safari/8536.25*/
const Device UA3083432 = {S44436, 142, { &P12280,&P22882,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A4449a Safari/8536.25*/
const Device UA3110675 = {S44437, 140, { &P12280,&P22882,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A466 Safari/8536.25*/
const Device UA3115633 = {S44438, 142, { &P12280,&P22882,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A501 Safari/8536.25*/
const Device UA3116186 = {S44439, 143, { &P12280,&P22882,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A470a Safari/8536.25*/
const Device UA623228 = {S44440, 74, { &P17595,&P17295,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:11.0) Gecko/X Firefox/11.0*/
const Device UA32924 = {S44441, 146, { &P12794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PantechP9060 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48737 = {S44442, 146, { &P12794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PantechP9060 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA640711 = {S44443, 70, { &P12794,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; PantechP9060 Build/GINGERBREAD)*/
const Device UA492615 = {S44444, 75, { &P15364,&P18144,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; OpenBSD i386; en-US; rv:1.9.2.8) Gecko/X Firefox/3.6.8*/
const Device UA54481 = {S44445, 77, { &P15234,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry9330/5.0.0.913 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/104*/
const Device UA10216 = {S44446, 78, { &P14218,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-C3300/C3300DDDJG2 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64770 = {S44447, 74, { &P14218,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C3300K/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA90509 = {S44448, 89, { &P14218,&P17484,&P4223,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-GT-C3300i/BUKA1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA163362 = {S44449, 89, { &P14218,&P17484,&P4223,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-GT-C3300K/BUJG4 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA164326 = {S44450, 89, { &P14218,&P17484,&P4223,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-GT-C3300K/BUJH1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA170044 = {S44451, 89, { &P14218,&P17484,&P4223,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-GT-C3300K/BUJG6 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA173834 = {S44452, 89, { &P14218,&P17484,&P4223,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-GT-C3300K/BUJK1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA408474 = {S44453, 89, { &P14218,&P17484,&P4223,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-GT-C3300K/BUJI1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA517892 = {S44454, 89, { &P14218,&P17484,&P4223,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-GT-C3300K/PIJI1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA662116 = {S44455, 82, { &P14218,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C3300i/C3300iXXKJ2 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2639392 = {S44456, 89, { &P14218,&P17484,&P4223,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-GT-C3300K/AEJG4 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA925401 = {S44457, 137, { &P17595,&P17296,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA1996051 = {S44458, 137, { &P17595,&P17296,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA1996260 = {S44459, 137, { &P17595,&P17296,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA1996721 = {S44460, 137, { &P17595,&P17296,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA1996835 = {S44461, 137, { &P17595,&P17296,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA2167230 = {S44462, 137, { &P17595,&P17296,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_5) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA2168773 = {S44463, 137, { &P17595,&P17296,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA2226370 = {S44464, 137, { &P17595,&P17296,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_1) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA2258143 = {S44465, 137, { &P17595,&P17296,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_2) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA2597000 = {S44466, 121, { &P17595,&P17296,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 1060) AppleWebKit/535.7 (KHTML like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77*/
const Device UA2422999 = {S44467, 139, { &P19853,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7102 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2460271 = {S44468, 130, { &P19853,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7102 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2506425 = {S44469, 61, { &P19853,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-N7102 Build/JRO03C)*/
const Device UA2530134 = {S44470, 139, { &P19853,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; zh-tw; GT-N7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2579836 = {S44471, 139, { &P19853,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; GT-N7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580310 = {S44472, 139, { &P19853,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-gb; GT-N7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659410 = {S44473, 139, { &P19853,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; es-es; GT-N7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2667882 = {S44474, 139, { &P19853,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2702400 = {S44475, 139, { &P19853,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; vi-vn; GT-N7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2702917 = {S44476, 139, { &P19853,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; it-it; GT-N7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2706005 = {S44477, 139, { &P19853,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7102 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2707127 = {S44478, 139, { &P19853,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; fr-fr; GT-N7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2737506 = {S44479, 61, { &P19853,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N7102 Build/JZO54K)*/
const Device UA2749204 = {S44480, 139, { &P19853,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; de-de; GT-N7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2848775 = {S44481, 61, { &P19853,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.9; GT-N7102 Build/IMM76D)*/
const Device UA2984243 = {S44482, 139, { &P19853,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; zh-cn; GT-N7102 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996625 = {S44483, 130, { &P19853,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7102 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA108224 = {S44484, 118, { &P15364,&P7133,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.4.154.25 Safari/525.19*/
const Device UA120981 = {S44485, 118, { &P15364,&P7133,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.2.149.27 Safari/525.13*/
const Device UA142362 = {S44486, 117, { &P15364,&P7133,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.3.154.0 Safari/525.19*/
const Device UA710163 = {S44487, 96, { &P15364,&P7133,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/0.0.0.0 Safari/534.24*/
const Device UA736433 = {S44488, 103, { &P15364,&P7133,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/0.0.0.0 Safari/534.24*/
const Device UA2139524 = {S44489, 115, { &P15364,&P7133,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; de) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.4.154.29 Safari/525.19*/
const Device UA2164258 = {S44490, 113, { &P15364,&P7133,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.3 (KHTML, like Gecko) Chrome/0.0.0.0 Safari/532.3*/
const Device UA2212684 = {S44491, 115, { &P15364,&P7133,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; de) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.2.149.27 Safari/525.13*/
const Device UA2375122 = {S44492, 101, { &P15364,&P7133,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/0.1.1.7 Safari/535.7*/
const Device UA2945943 = {S44493, 134, { &P15364,&P16110,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/28.0.1500.71) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36*/
const Device UA2969148 = {S44494, 134, { &P15364,&P16110,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/28.0.1500.72) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36*/
const Device UA3025909 = {S44495, 90, { &P15364,&P16110,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; chromeframe/28.0.1500.95)*/
const Device UA3026210 = {S44496, 97, { &P15364,&P16110,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/28.0.1500.95)*/
const Device UA3027129 = {S44497, 96, { &P15364,&P16110,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/28.0.1500.95)*/
const Device UA3028763 = {S44498, 134, { &P15364,&P16110,&P22282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/28.0.1500.95) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA3030287 = {S44499, 186, { &P15364,&P16110,&P22282,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; HPNTDFJS; chromeframe/28.0.1500.95)*/
const Device UA592278 = {S44500, 142, { &P18700,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-B5512 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA802472 = {S44501, 143, { &P18700,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-B5512B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1958100 = {S44502, 130, { &P18700,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-B5512 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1989378 = {S44503, 66, { &P18700,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-B5512 Build/GINGERBREAD)*/
const Device UA2222758 = {S44504, 140, { &P18700,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-B5512 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2566507 = {S44505, 138, { &P18509,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567254 = {S44506, 131, { &P18509,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA33342 = {S44507, 144, { &P18009,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Sprint APC715CKT Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA220357 = {S44508, 135, { &P18009,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PH44100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623060 = {S44509, 59, { &P18009,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; PH44100 Build/GRJ22)*/
const Device UA2447620 = {S44510, 148, { &P18009,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-tw; HTC_EVO_Design_C715e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3070059 = {S44511, 148, { &P18009,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-tw; HTC EVO Design C715e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA895155 = {S44512, 134, { &P15364,&P7133,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/20.0.1132.11) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.11 Safari/536.11*/
const Device UA908805 = {S44513, 134, { &P15364,&P7133,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/20.0.1132.43) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.43 Safari/536.11*/
const Device UA909537 = {S44514, 127, { &P15364,&P7133,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/20.0.1132.43) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.43 Safari/536.11*/
const Device UA909874 = {S44515, 134, { &P15364,&P7133,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/20.0.1132.47) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11*/
const Device UA909946 = {S44516, 127, { &P15364,&P7133,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/20.0.1132.47) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11*/
const Device UA911089 = {S44517, 194, { &P15364,&P7133,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; chromeframe/20.0.1132.47)*/
const Device UA912391 = {S44518, 96, { &P15364,&P7133,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/20.0.1132.57)*/
const Device UA912519 = {S44519, 89, { &P15364,&P7133,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/20.0.1132.57)*/
const Device UA912600 = {S44520, 134, { &P15364,&P7133,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/20.0.1132.57) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA912627 = {S44521, 127, { &P15364,&P7133,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/20.0.1132.57) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA912785 = {S44522, 196, { &P15364,&P7133,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/20.0.1132.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3)*/
const Device UA912888 = {S44523, 197, { &P15364,&P7133,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/20.0.1132.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA924053 = {S44524, 231, { &P15364,&P7133,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/20.0.1132.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; Tablet PC 2.0)*/
const Device UA1950597 = {S44525, 208, { &P15364,&P7133,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/20.0.1132.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; MS-RTC LM 8; InfoPath.3)*/
const Device UA2217796 = {S44526, 188, { &P15364,&P7133,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/20.0.1132.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3)*/
const Device UA2338765 = {S44527, 166, { &P15364,&P7133,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; chromeframe/20.0.1132.57)*/
const Device UA2360231 = {S44528, 203, { &P15364,&P7133,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/20.0.1132.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0)*/
const Device UA2494610 = {S44529, 199, { &P15364,&P7133,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Win64; x64; Trident/4.0; chromeframe/20.0.1132.57; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)*/
const Device UA2814366 = {S44530, 199, { &P15364,&P7133,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; chromeframe/20.0.1132.57; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C)*/
const Device UA2889273 = {S44531, 97, { &P15364,&P7133,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; chromeframe/20.0.1132.57)*/
const Device UA2992072 = {S44532, 90, { &P15364,&P7133,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; chromeframe/20.0.1132.47)*/
const Device UA40054 = {S44533, 145, { &P13591,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-us; ZTE-BLADE Build/P729USV1.0.0B01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA53550 = {S44534, 146, { &P13591,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; lt-lt; ZTE-BLADE Build/OPEN-MARKET-V1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA64767 = {S44535, 135, { &P13591,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; ZTE-BLADE Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA190103 = {S44536, 59, { &P13591,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; ZTE-BLADE Build/FRF91)*/
const Device UA680434 = {S44537, 137, { &P13591,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; ZTE-BLADE Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA838624 = {S44538, 125, { &P13591,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; ZTE-BLADE Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1901242 = {S44539, 61, { &P13591,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; ZTE-BLADE Build/FRF91)*/
const Device UA199701 = {S44540, 55, { &P15364,&P6704,&P17387,&P18091 }};	/*Useragent - New-Sogou-Spider/1.0 (compatible; MSIE 5.5; Windows 98)*/
const Device UA841235 = {S44541, 56, { &P15364,&P6704,&P17387,&P18091 }};	/*Useragent - Sogou-Test-Spider/4.0 (compatible; MSIE 5.5; Windows 98)*/
const Device UA2674187 = {S44542, 62, { &P15364,&P5690,&P22225,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:20.0) Gecko/X SeaMonkey/2.17.1*/
const Device UA2674544 = {S44543, 75, { &P15364,&P5690,&P22225,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:20.0) Gecko/X Firefox/20.0 SeaMonkey/2.17.1*/
const Device UA2714278 = {S44544, 82, { &P15364,&P5690,&P22225,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:20.0) Gecko/X Firefox/20.0 SeaMonkey/2.17.1*/
const Device UA454072 = {S44545, 98, { &P15364,&P7133,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.1.6014(Windows;Microsoft Windows (6.1.7601.1);en;kml:2.2;client:Free;type:default)*/
const Device UA608512 = {S44546, 122, { &P15364,&P7133,&P18208,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.2.2.6613 Safari/532.4*/
const Device UA608487 = {S44547, 98, { &P15364,&P7133,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);en;kml:2.2;client:Free;type:default)*/
const Device UA612124 = {S44548, 98, { &P15364,&P7133,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7600.0);en;kml:2.2;client:Free;type:default)*/
const Device UA612446 = {S44549, 98, { &P15364,&P7133,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);de;kml:2.2;client:Free;type:default)*/
const Device UA725493 = {S44550, 98, { &P15364,&P7133,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);nl;kml:2.2;client:Free;type:default)*/
const Device UA768117 = {S44551, 98, { &P15364,&P7133,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);fr;kml:2.2;client:Free;type:default)*/
const Device UA894549 = {S44552, 102, { &P15364,&P7133,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);es-419;kml:2.2;client:Free;type:default)*/
const Device UA1951165 = {S44553, 98, { &P15364,&P7133,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);it;kml:2.2;client:Free;type:default)*/
const Device UA1953694 = {S44554, 98, { &P15364,&P7133,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);ru;kml:2.2;client:Free;type:default)*/
const Device UA1966066 = {S44555, 98, { &P15364,&P7133,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);pt;kml:2.2;client:Free;type:default)*/
const Device UA1978431 = {S44556, 98, { &P15364,&P7133,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);es;kml:2.2;client:Free;type:default)*/
const Device UA1994222 = {S44557, 98, { &P15364,&P7133,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);tr;kml:2.2;client:Free;type:default)*/
const Device UA2153260 = {S44558, 98, { &P15364,&P7133,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.0.5905(Windows;Microsoft Windows (6.1.7601.1);de;kml:2.2;client:Free;type:default)*/
const Device UA2153883 = {S44559, 98, { &P15364,&P7133,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);ko;kml:2.2;client:Free;type:default)*/
const Device UA2222006 = {S44560, 97, { &P15364,&P7133,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.1.7601.1);en;kml:2.2;client:Pro;type:default)*/
const Device UA67320 = {S44561, 134, { &P14813,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-US670 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA68159 = {S44562, 137, { &P14813,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-US670 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90445 = {S44563, 141, { &P14813,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-us; LG-US670 Build/FRF91) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA389906 = {S44564, 58, { &P14813,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; LG-US670 Build/FRF91)*/
const Device UA686236 = {S44565, 61, { &P14813,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-US670 Build/FRG83G)*/
const Device UA247672 = {S44566, 122, { &P15841,&P17358,&P24416,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; RIM Tablet OS 2.0.0; en-US) AppleWebKit/535.1+ (KHTML, like Gecko) Version/7.2.0.0 Safari/535.1+*/
const Device UA382056 = {S44567, 122, { &P15841,&P17358,&P24416,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; RIM Tablet OS 2.0.0; en-US) AppleWebKit/535.8+ (KHTML, like Gecko) Version/7.2.0.0 Safari/535.8+*/
const Device UA623017 = {S44568, 122, { &P15841,&P17358,&P24416,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; RIM Tablet OS 2.0.1; en-US) AppleWebKit/535.8+ (KHTML, like Gecko) Version/7.2.0.1 Safari/535.8+*/
const Device UA2404656 = {S44569, 132, { &P18465,&P18110,&P19804,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Safari/535.19 Silk-Accelerated=true*/
const Device UA2510968 = {S44570, 58, { &P15364,&P18110,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686) Presto/2.12.388 Version/12.14*/
const Device UA2531767 = {S44571, 69, { &P15364,&P18110,&P21172,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:2.0) Gecko/X Firefox/4.0 Opera 12.14*/
const Device UA2662802 = {S44572, 63, { &P15364,&P18110,&P21172,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; X11; Linux i686) Opera 12.14*/
const Device UA925514 = {S44573, 138, { &P18438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-E973 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2233959 = {S44574, 139, { &P18438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F200L Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2347451 = {S44575, 139, { &P18438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F200K Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2348626 = {S44576, 139, { &P18438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F200S Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2385667 = {S44577, 146, { &P18438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-E973/E97310f Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2433809 = {S44578, 146, { &P18438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-E973/E97310h Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2651492 = {S44579, 60, { &P18438,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-E973 Build/IMM76L)*/
const Device UA2777347 = {S44580, 61, { &P18438,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-F200S Build/IMM76L)*/
const Device UA2826975 = {S44581, 61, { &P18438,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-F200L Build/IMM76L)*/
const Device UA2983266 = {S44582, 146, { &P18438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-E973/E97310c Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA924324 = {S44583, 76, { &P17595,&P18207,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:2.0.1) Gecko/X Firefox/4.0.1*/
const Device UA66962 = {S44584, 102, { &P14802,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i917)*/
const Device UA67916 = {S44585, 103, { &P14802,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i917.)*/
const Device UA90381 = {S44586, 100, { &P14802,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0) SAMSUNG;SGH-i917*/
const Device UA848283 = {S44587, 102, { &P14802,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible: MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i917)*/
const Device UA925715 = {S44588, 99, { &P14802,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; CETUS)*/
const Device UA2765129 = {S44589, 117, { &P14802,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible: MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; WpsLondonTest; SAMSUNG; SGH-i917)*/
const Device UA907507 = {S44590, 64, { &P17976,&P17979,&P20435,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:15.0) Gecko/15.0 Firefox/15.0a2*/
const Device UA924831 = {S44591, 62, { &P17976,&P17979,&P20435,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:15.0) Gecko/15.0 Firefox/15.0*/
const Device UA2027722 = {S44592, 64, { &P17976,&P17979,&P20435,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:15.0) Gecko/15.0 Firefox/15.0a1*/
const Device UA2041282 = {S44593, 64, { &P17976,&P17979,&P20435,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:15.0) Gecko/15.0 Firefox/15.0.1*/
const Device UA2042318 = {S44594, 68, { &P17976,&P17979,&P20435,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:15.0.1) Gecko/15.0.1 Firefox/15.0.1*/
const Device UA2205385 = {S44595, 101, { &P17976,&P17979,&P20435,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:15.0) Gecko/15.0 Firefox/15.0.1,gzip(gfe) (via translate.google.com)*/
const Device UA2762217 = {S44596, 74, { &P17976,&P17979,&P20435,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:15.0) Gecko/15.0 Firefox/15.0.1,gzip(gfe)*/
const Device UA2555614 = {S44597, 132, { &P19874,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-i705 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2668781 = {S44598, 61, { &P19874,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-i705 Build/JZO54K)*/
const Device UA181106 = {S44599, 129, { &P17595,&P17295,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.127 Safari/534.16*/
const Device UA311588 = {S44600, 129, { &P17595,&P17295,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16*/
const Device UA543602 = {S44601, 129, { &P17595,&P17295,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16*/
const Device UA674482 = {S44602, 129, { &P17595,&P17295,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA767623 = {S44603, 129, { &P17595,&P17295,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA770983 = {S44604, 129, { &P17595,&P17295,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA780822 = {S44605, 129, { &P17595,&P17295,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16*/
const Device UA2281076 = {S44606, 128, { &P17595,&P17295,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.82 Safari/534.16*/
const Device UA10289 = {S44607, 134, { &P14837,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I5800 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA68304 = {S44608, 136, { &P14837,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; hr-hr; GT-I5800 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA169449 = {S44609, 58, { &P14837,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I5800 Build/FROYO)*/
const Device UA211574 = {S44610, 135, { &P14837,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-I5800D Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019479 = {S44611, 59, { &P14837,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I5800D Build/FROYO)*/
const Device UA904701 = {S44612, 137, { &P19784,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; LIFETAB_P9514 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2335904 = {S44613, 66, { &P19784,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; LIFETAB_P9514 Build/IML74K)*/
const Device UA2029663 = {S44614, 147, { &P19867,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I437 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426240 = {S44615, 69, { &P19867,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SAMSUNG-SGH-I437 Build/IMM76D)*/
const Device UA422246 = {S44616, 113, { &P15364,&P17285,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94; ru) Presto/2.10.229 Version/11.61*/
const Device UA422252 = {S44617, 101, { &P15364,&P17285,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; en) Presto/2.10.229 Version/11.61,gzip(gfe) (via translate.google.com)*/
const Device UA454947 = {S44618, 66, { &P15364,&P17285,&P3403,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; en) Opera 11.61*/
const Device UA588512 = {S44619, 87, { &P15364,&P17285,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.4 (build 02647); ru) Presto/2.10.229 Version/11.61*/
const Device UA588702 = {S44620, 86, { &P15364,&P17285,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4876); ru) Presto/2.10.229 Version/11.61*/
const Device UA590220 = {S44621, 87, { &P15364,&P17285,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5231); ru) Presto/2.10.229 Version/11.61*/
const Device UA594692 = {S44622, 87, { &P15364,&P17285,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5244); ru) Presto/2.10.229 Version/11.61*/
const Device UA622575 = {S44623, 64, { &P15364,&P17285,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; en) Presto/2.10.229 Version/11.61*/
const Device UA680130 = {S44624, 87, { &P15364,&P17285,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.61*/
const Device UA688254 = {S44625, 86, { &P15364,&P17285,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.61*/
const Device UA703639 = {S44626, 87, { &P15364,&P17285,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.61*/
const Device UA721226 = {S44627, 87, { &P15364,&P17285,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.61*/
const Device UA753426 = {S44628, 72, { &P15364,&P17285,&P3403,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.61*/
const Device UA753663 = {S44629, 86, { &P15364,&P17285,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4664); ru) Presto/2.10.229 Version/11.61*/
const Device UA1952576 = {S44630, 87, { &P15364,&P17285,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03790); ru) Presto/2.10.229 Version/11.61*/
const Device UA1993721 = {S44631, 87, { &P15364,&P17285,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.61*/
const Device UA2312880 = {S44632, 87, { &P15364,&P17285,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.10 (build 5196); ru) Presto/2.10.229 Version/11.61*/
const Device UA2324173 = {S44633, 122, { &P15364,&P17285,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; &quot;Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0); en) Presto/2.10.229 Version/11.61*/
const Device UA2339281 = {S44634, 86, { &P15364,&P17285,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4157); ru) Presto/2.10.229 Version/11.61*/
const Device UA2339816 = {S44635, 86, { &P15364,&P17285,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4598); ru) Presto/2.10.229 Version/11.61*/
const Device UA2507883 = {S44636, 49, { &P12280,&P19826,&P17470,&P18092 }};	/*Useragent - FeeddlerRSS 1.12.5 (iPhone; iPhone OS 6.1; en_US)*/
const Device UA2565714 = {S44637, 51, { &P12280,&P19826,&P17470,&P18092 }};	/*Useragent - FeeddlerRSS 1.12.5 (iPhone; iPhone OS 6.1.2; en_US)*/
const Device UA2585694 = {S44638, 61, { &P12280,&P19826,&P17470,&P18092 }};	/*Useragent - Appcelerator Titanium/1.7.2 (iPhone/6.1.3; iPhone OS; en_US;)*/
const Device UA2607315 = {S44639, 51, { &P12280,&P19826,&P17470,&P18092 }};	/*Useragent - FeeddlerRSS 1.12.5 (iPhone; iPhone OS 6.1.3; en_US)*/
const Device UA2738291 = {S44640, 51, { &P12280,&P19826,&P17470,&P18092 }};	/*Useragent - FeeddlerRSS 1.12.5 (iPhone; iPhone OS 6.1.4; en_US)*/
const Device UA2861044 = {S44641, 46, { &P12280,&P19826,&P17470,&P18092 }};	/*Useragent - Miu Ptt 1.9.2 (iPhone; iPhone OS 6.1.3; en_US)*/
const Device UA2680157 = {S44642, 117, { &P15364,&P7133,&P22466,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36*/
const Device UA2680488 = {S44643, 124, { &P15364,&P7133,&P22466,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36*/
const Device UA2673637 = {S44644, 90, { &P15364,&P7133,&P22370,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.0.1-x64*/
const Device UA2680475 = {S44645, 81, { &P15364,&P7133,&P22370,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.0.1*/
const Device UA351991 = {S44646, 87, { &P12280,&P17779,&P17373,&P18092 }};	/*Useragent - Opera/9.80 (iPhone; Opera Mini/6.5.1.23995/26.1283; U; fr) Presto/2.8.119 Version/10.54*/
const Device UA2431737 = {S44647, 81, { &P12280,&P17779,&P17373,&P18092 }};	/*Useragent - Opera/9.80 (iPhone; Opera Mini/6.5.2/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2431990 = {S44648, 89, { &P18115,&P5764,&P17780,&P18092 }};	/*Useragent - Opera/9.80 (BlackBerry; Opera Mini/6.1.25376/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2467043 = {S44649, 139, { &P20013,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F180K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469128 = {S44650, 139, { &P20013,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F180L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2471118 = {S44651, 139, { &P20013,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F180S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2697837 = {S44652, 61, { &P20013,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F180L Build/JZO54K)*/
const Device UA2729756 = {S44653, 61, { &P20013,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F180S Build/JZO54K)*/
const Device UA2753907 = {S44654, 148, { &P20013,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F180K/F180K20k Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2791480 = {S44655, 61, { &P20013,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F180K Build/JZO54K)*/
const Device UA2997543 = {S44656, 138, { &P20013,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; LG-F180 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3080488 = {S44657, 148, { &P20013,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F180K/F180K20n Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA349591 = {S44658, 107, { &P17595,&P18207,&P18363,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/535.9 (KHTML, like Gecko) Version/5.2 Safari/535.9*/
const Device UA401695 = {S44659, 113, { &P17595,&P18207,&P18363,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/535.18.5 (KHTML, like Gecko) Version/5.2 Safari/535.18.5*/
const Device UA440551 = {S44660, 111, { &P17595,&P18207,&P18363,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.2.3 (KHTML, like Gecko) Version/5.2 Safari/536.2.3*/
const Device UA454825 = {S44661, 107, { &P17595,&P18207,&P18363,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.5 (KHTML, like Gecko) Version/5.2 Safari/536.5*/
const Device UA603814 = {S44662, 111, { &P17595,&P18207,&P18363,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.6.1 (KHTML, like Gecko) Version/5.2 Safari/536.6.1*/
const Device UA614889 = {S44663, 111, { &P17595,&P18207,&P18363,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.8.5 (KHTML, like Gecko) Version/5.2 Safari/536.8.5*/
const Device UA801591 = {S44664, 107, { &P17595,&P18207,&P18363,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.9 (KHTML, like Gecko) Version/5.2 Safari/536.9*/
const Device UA814876 = {S44665, 113, { &P17595,&P18207,&P18363,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.10.3 (KHTML, like Gecko) Version/5.2 Safari/536.10.3*/
const Device UA842701 = {S44666, 109, { &P17595,&P18207,&P18363,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.11 (KHTML, like Gecko) Version/5.2 Safari/536.11*/
const Device UA2596286 = {S44667, 111, { &P17595,&P18207,&P18363,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 108) AppleWebKit/535.18.5 (KHTML like Gecko) Version/5.2 Safari/535.18.5*/
const Device UA2599867 = {S44668, 111, { &P17595,&P18207,&P18363,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 108) AppleWebKit/536.10.3 (KHTML like Gecko) Version/5.2 Safari/536.10.3*/
const Device UA604064 = {S44669, 190, { &P15791,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.4 Nokia603/112.010.1404; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.2.1.20 Mobile Safari/535.1 3gpp-gba*/
const Device UA631306 = {S44670, 190, { &P15791,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.3 Nokia603/111.020.0310; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.4.1.14 Mobile Safari/533.4 3gpp-gba*/
const Device UA2205928 = {S44671, 190, { &P15791,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.5 Nokia603/113.010.1506; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.2.21 Mobile Safari/535.1 3gpp-gba*/
const Device UA2492645 = {S44672, 132, { &P18465,&P18110,&P19837,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Safari/535.19 Silk-Accelerated=true*/
const Device UA11605 = {S44673, 144, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-au; MotoA953 Build/MILS2_U6_2.2.19) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11609 = {S44674, 144, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-sg; MotoA953 Build/MILS2_U6_2.4.18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11615 = {S44675, 144, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MotoA953 Build/MILS2_U6_2.4.16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11621 = {S44676, 144, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MotoA953 Build/MILS2_U6_2.4.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11625 = {S44677, 195, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-A953/Blur_Version.3.18.3.A953.AmericaMovil.en.MX Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; A953 Build/MILA2_U6_3.18.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20167 = {S44678, 133, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; A953Blur Build/S274) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20168 = {S44679, 185, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-A953/Blur_Version.3.9.0.A953.Brasil.en.BR Mozilla/5.0 (Linux; U; Android 2.2; en-us; A953 Build/MILA2_U6_3.9.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA26765 = {S44680, 144, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; MotoA953 Build/MILS2_U6_2.2.16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA26770 = {S44681, 143, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; MotoA953 Build/MILS2_U6_2.3.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA26773 = {S44682, 139, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; de-de; MotoA953 Build/MS2_2.2.13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA26783 = {S44683, 187, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-A953/Blur_Version.3.12.0.A953.Brasil.en.BR Mozilla/5.0 (Linux; U; Android 2.2; pt-br; A953 Build/MILA2_U6_3.12.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA26784 = {S44684, 133, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MotoA953 Build/S274) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40410 = {S44685, 188, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-A953/Blur_Version.3.25.0.A953.Latam.en.01 Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; A953 Build/MILA2_U6_3.25.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40408 = {S44686, 186, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-A953/Blur_Version.3.21.0.A953.TIM.en.BR Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A953 Build/MILA2_U6_3.21.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40425 = {S44687, 144, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; MotoA953 Build/MILS2_U6_2.4.19) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40431 = {S44688, 188, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-A953/Blur_Version.3.25.1.A953.Latam.en.01 Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; A953 Build/MILA2_U6_3.25.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52161 = {S44689, 134, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MotoA953 Build/STAB2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55614 = {S44690, 188, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-A953/Blur_Version.3.18.4.A953.Latam.en.01 Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; A953 Build/MILA2_U6_3.18.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55627 = {S44691, 192, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-A953/Blur_Version.3.18.100.A953.Latam.en.01 Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; A953 Build/MILA2_U6_3.18.100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55644 = {S44692, 146, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; MotoA953 Build/MILS2_U6_2.4.24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55647 = {S44693, 195, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-A953/Blur_Version.3.18.1.A953.AmericaMovil.en.01 Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; A953 Build/MILA2_U6_3.18.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55648 = {S44694, 145, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MotoA953 Build/MILS2_U6_2.5.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55652 = {S44695, 191, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-A953/Blur_Version.3.15.3.A953.Movistar.en.01 Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; A953 Build/MILA2_U6_3.15.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70770 = {S44696, 146, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; he-il; MotoA953 Build/MILS2_U6_2.5.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71746 = {S44697, 186, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-A953/Blur_Version.3.24.0.A953.TIM.en.BR Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; A953 Build/MILA2_U6_3.24.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74358 = {S44698, 144, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MotoA953 Build/MILS2_U6_2.2.20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90119 = {S44699, 133, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; A953Blur Build/S260) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA448629 = {S44700, 64, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; A953 Build/MILA2_U6_3.12.0)*/
const Device UA694834 = {S44701, 188, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-A953/Blur_Version.3.23.0.A953.Latam.en.01 Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; A953 Build/MILA2_U6_3.23.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018429 = {S44702, 68, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; MotoA953 Build/MILS2_U6_2.4.18)*/
const Device UA2019190 = {S44703, 70, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; MotoA953 Build/MILS2_U6_2.4.24)*/
const Device UA2019522 = {S44704, 132, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; MotoA953 Build/MILS2_U6_2.2.16) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2811449 = {S44705, 137, { &P13633,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; MotoA953 Build/MILS2_U6_2.2.16) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2795566 = {S44706, 164, { &P22972,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG-SGH-I537 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA196480 = {S44707, 143, { &P17961,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SHW-M180S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA205710 = {S44708, 143, { &P17961,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; SHW-M180L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA210692 = {S44709, 143, { &P17961,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SHW-M180S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA235438 = {S44710, 143, { &P17961,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; SHW-M180L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA332065 = {S44711, 143, { &P17961,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; SHW-M180W Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA333886 = {S44712, 143, { &P17961,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; SHW-M180K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA354724 = {S44713, 143, { &P17961,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHW-M180K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA378704 = {S44714, 143, { &P17961,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; SHW-M180K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA451972 = {S44715, 67, { &P17961,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M180S Build/GINGERBREAD)*/
const Device UA640363 = {S44716, 143, { &P17961,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M180S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA735762 = {S44717, 143, { &P17961,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHW-M180L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1911245 = {S44718, 67, { &P17961,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHW-M180S Build/GINGERBREAD)*/
const Device UA1976876 = {S44719, 143, { &P17961,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHW-M180W Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018880 = {S44720, 67, { &P17961,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SHW-M180S Build/GINGERBREAD)*/
const Device UA2019877 = {S44721, 67, { &P17961,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M180L Build/GINGERBREAD)*/
const Device UA68916 = {S44722, 100, { &P14640,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; OMNIA7)*/
const Device UA68975 = {S44723, 108, { &P14640,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; OMNIA7; Orange)*/
const Device UA925878 = {S44724, 102, { &P14640,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; GT-I8700)*/
const Device UA2721989 = {S44725, 98, { &P14640,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; OMNIA7)*/
const Device UA626229 = {S44726, 95, { &P15364,&P17285,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:12.0) Gecko/X Firefox/12.0,gzip(gfe) (via translate.google.com)*/
const Device UA743793 = {S44727, 70, { &P15364,&P17285,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:12.0) Gecko/X Firefox/12.0*/
const Device UA796480 = {S44728, 78, { &P15364,&P17285,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:12.0) Gecko/X Firefox/12.0 Firefox/12.0*/
const Device UA97236 = {S44729, 62, { &P15364,&P5690,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows XP) Opera 6.0  [en]*/
const Device UA120897 = {S44730, 43, { &P15364,&P5690,&P17466,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows XP; U) Opera 6.01 [en]*/
const Device UA130292 = {S44731, 62, { &P15364,&P5690,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows XP) Opera 6.05 [en]*/
const Device UA412813 = {S44732, 62, { &P15364,&P5690,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows XP) Opera 6.06 [fr]*/
const Device UA633551 = {S44733, 43, { &P15364,&P5690,&P17466,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows XP; U) Opera 6.06 [en]*/
const Device UA785598 = {S44734, 63, { &P15364,&P5690,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows XP) Opera 6.01  [ru]*/
const Device UA824262 = {S44735, 31, { &P15364,&P5690,&P17466,&P18092 }};	/*Useragent - Opera/6.04 (Windows XP; U) [en]*/
const Device UA838758 = {S44736, 62, { &P15364,&P5690,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows XP) Opera 6.04 [fr]*/
const Device UA857456 = {S44737, 31, { &P15364,&P5690,&P17466,&P18092 }};	/*Useragent - Opera/6.05 (Windows XP; U) [de]*/
const Device UA2472833 = {S44738, 62, { &P15364,&P5690,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows XP) Opera 6.03 [en]*/
const Device UA2638753 = {S44739, 31, { &P15364,&P5690,&P17466,&P18092 }};	/*Useragent - Opera/6.01 (Windows XP; U) [ru]*/
const Device UA2792816 = {S44740, 76, { &P15364,&P5690,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.1) Opera 6.02 [en]*/
const Device UA887368 = {S44741, 139, { &P18636,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VS930 4G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2417553 = {S44742, 61, { &P18636,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; VS930 4G Build/IMM76I)*/
const Device UA2299912 = {S44743, 116, { &P15364,&P16110,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.53 Safari/534.3*/
const Device UA2741717 = {S44744, 136, { &P21648,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774083 = {S44745, 137, { &P21648,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-i9505 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879340 = {S44746, 137, { &P21648,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505G Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2257392 = {S44747, 141, { &P20960,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; htc_h1000c Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3025083 = {S44748, 119, { &P17595,&P21721,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA251541 = {S44749, 118, { &P17595,&P17297,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA334967 = {S44750, 118, { &P17595,&P17297,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA417638 = {S44751, 117, { &P17595,&P17297,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.0 Safari/534.24*/
const Device UA656641 = {S44752, 127, { &P17595,&P17297,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_0; en-US) AppleWebKit/534.21 (KHTML, like Gecko) Chrome/11.0.678.0 Safari/534.21*/
const Device UA742779 = {S44753, 118, { &P17595,&P17297,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA755800 = {S44754, 118, { &P17595,&P17297,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA2201495 = {S44755, 118, { &P17595,&P17297,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA2543702 = {S44756, 128, { &P17595,&P17297,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7; en-us) AppleWebKit/534.20.8 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.13*/
const Device UA899201 = {S44757, 112, { &P18583,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia110/03.04; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.5.16.6*/
const Device UA2754409 = {S44758, 145, { &P18645,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I747 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754421 = {S44759, 136, { &P18645,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I747 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754574 = {S44760, 137, { &P18645,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L710 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755521 = {S44761, 137, { &P18645,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9305 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769372 = {S44762, 145, { &P18645,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SAMSUNG-SGH-I747 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773188 = {S44763, 138, { &P18645,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530U Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773962 = {S44764, 137, { &P18645,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-I747 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2870034 = {S44765, 137, { &P18645,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SPH-L710 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877904 = {S44766, 138, { &P18645,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530C Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2974267 = {S44767, 139, { &P18447,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.1.141) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA5115 = {S44768, 140, { &P13349,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-tw; HTC Aria A6380 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA6376 = {S44769, 137, { &P13349,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HTC Aria Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA7561 = {S44770, 140, { &P13349,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-au; HTC_Aria_A6380 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA16258 = {S44771, 142, { &P13349,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC_Gratia_A6380 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA22080 = {S44772, 142, { &P13349,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; sv-se; HTC Gratia A6380 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA53750 = {S44773, 133, { &P13349,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; S31HT Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA56697 = {S44774, 145, { &P13349,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HTC_Gratia_A6380 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90710 = {S44775, 144, { &P13349,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; de-at; HTC Aria Build/FRG83G) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA434220 = {S44776, 61, { &P13349,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; HTC Aria Build/FRG83G)*/
const Device UA641999 = {S44777, 64, { &P13349,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; HTC Aria A6380 Build/FRF91)*/
const Device UA655016 = {S44778, 66, { &P13349,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; HTC Gratia A6380 Build/FRF91)*/
const Device UA2021005 = {S44779, 128, { &P13349,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Aria A6380 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2026666 = {S44780, 133, { &P13349,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Aria A6380 Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2512432 = {S44781, 75, { &P17595,&P18207,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2) Presto/2.12.388 Version/12.14*/
const Device UA2658958 = {S44782, 138, { &P18653,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P870 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2675609 = {S44783, 146, { &P18653,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P870/P87020d Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2716585 = {S44784, 60, { &P18653,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P870 Build/JZO54K)*/
const Device UA2540510 = {S44785, 135, { &P20760,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Event Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2540512 = {S44786, 57, { &P20760,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Event Build/IML77)*/
const Device UA2715721 = {S44787, 98, { &P15364,&P7133,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (6.1.7601.1);en;kml:2.2;client:Free;type:default)*/
const Device UA2737584 = {S44788, 98, { &P15364,&P7133,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (6.1.7601.1);de;kml:2.2;client:Free;type:default)*/
const Device UA2738813 = {S44789, 98, { &P15364,&P7133,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (6.1.7600.0);en;kml:2.2;client:Free;type:default)*/
const Device UA2781587 = {S44790, 102, { &P15364,&P7133,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (6.1.7601.1);es-419;kml:2.2;client:Free;type:default)*/
const Device UA2795272 = {S44791, 98, { &P15364,&P7133,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (6.1.7601.1);fr;kml:2.2;client:Free;type:default)*/
const Device UA2864922 = {S44792, 98, { &P15364,&P7133,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (6.1.7601.1);es;kml:2.2;client:Free;type:default)*/
const Device UA2867729 = {S44793, 103, { &P15364,&P7133,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (6.1.7601.1);zh-Hans;kml:2.2;client:Free;type:default)*/
const Device UA2878936 = {S44794, 97, { &P15364,&P7133,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (6.1.7601.1);en;kml:2.2;client:Pro;type:default)*/
const Device UA2898051 = {S44795, 98, { &P15364,&P7133,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1871(Windows;Microsoft Windows (6.1.7601.1);en;kml:2.2;client:Free;type:default)*/
const Device UA2973153 = {S44796, 122, { &P15364,&P7133,&P22779,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/7.1.1.1888 Safari/532.4*/
const Device UA2974683 = {S44797, 98, { &P15364,&P7133,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1888(Windows;Microsoft Windows (6.1.7601.1);en;kml:2.2;client:Free;type:default)*/
const Device UA3018349 = {S44798, 98, { &P15364,&P7133,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1888(Windows;Microsoft Windows (6.1.7601.1);de;kml:2.2;client:Free;type:default)*/
const Device UA3021600 = {S44799, 98, { &P15364,&P7133,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1888(Windows;Microsoft Windows (6.1.7600.0);en;kml:2.2;client:Free;type:default)*/
const Device UA3025541 = {S44800, 102, { &P15364,&P7133,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1888(Windows;Microsoft Windows (6.1.7601.1);es-419;kml:2.2;client:Free;type:default)*/
const Device UA3078239 = {S44801, 98, { &P15364,&P7133,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1888(Windows;Microsoft Windows (6.1.7601.1);es;kml:2.2;client:Free;type:default)*/
const Device UA3092279 = {S44802, 98, { &P15364,&P7133,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1888(Windows;Microsoft Windows (6.1.7601.1);fr;kml:2.2;client:Free;type:default)*/
const Device UA21314 = {S44803, 138, { &P12519,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 1.5; en-us; MB501 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA21320 = {S44804, 138, { &P12519,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB501 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 MobileSafari/525.20.1*/
const Device UA21336 = {S44805, 138, { &P12519,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android1.5; en-us; MB501 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA21337 = {S44806, 138, { &P12519,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB501Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA21339 = {S44807, 138, { &P12519,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us;MB501 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA44397 = {S44808, 137, { &P12519,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-; MB501 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA54792 = {S44809, 139, { &P12519,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB501 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA89740 = {S44810, 140, { &P12519,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB501 Build/CUPCAKE) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA47701 = {S44811, 151, { &P12082,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; SonyEricssonX10a Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47702 = {S44812, 139, { &P12082,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; X10a Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70732 = {S44813, 151, { &P12082,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ph; SonyEricssonX10a Build/3.0.1.G.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA569122 = {S44814, 131, { &P12082,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; X10 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA616491 = {S44815, 63, { &P12082,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; X10a Build/3.0.1.G.0.75)*/
const Device UA753665 = {S44816, 152, { &P12082,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; X10 TripNMiUI-1.8.26 Build/TripNMiUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1909318 = {S44817, 140, { &P12082,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; X10 Mini Pro Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1941562 = {S44818, 132, { &P12082,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; X10 Build/COB017) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028936 = {S44819, 132, { &P12082,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; X10 Build/COB020) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2733563 = {S44820, 137, { &P12082,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.9; en-us; X10 Build/6.0.B.0.743) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2750135 = {S44821, 138, { &P12082,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SMART X10 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625234 = {S44822, 73, { &P15364,&P5690,&P22340,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko/X Firefox/11.0 CometBird/11.0*/
const Device UA659712 = {S44823, 76, { &P15364,&P5690,&P22340,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko/X Firefox/11.0 CometBrowser/11.0*/
const Device UA1955873 = {S44824, 80, { &P15364,&P5690,&P22340,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:11.0) Gecko/X Firefox/11.0 CometBird/11.0*/
const Device UA2408229 = {S44825, 139, { &P16349,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Sony Tablet S Build/TISU0143) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2846839 = {S44826, 140, { &P16349,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  Sony Tablet S Build/TISU0143) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2566519 = {S44827, 131, { &P18437,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8000 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566761 = {S44828, 131, { &P18437,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8020 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566953 = {S44829, 131, { &P18437,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8020 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566970 = {S44830, 131, { &P18437,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8000 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583374 = {S44831, 131, { &P18437,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8005 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA625222 = {S44832, 130, { &P22052,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-P7501 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA658848 = {S44833, 149, { &P22052,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7501/P7501BOKL2 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA674184 = {S44834, 149, { &P22052,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7501/P7501BOLA1 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA837245 = {S44835, 59, { &P22052,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7501 Build/HTJ85B)*/
const Device UA2337967 = {S44836, 149, { &P22052,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; SAMSUNG-GT-P7501/P7501BUKL3 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2607944 = {S44837, 128, { &P22052,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P7501 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2608078 = {S44838, 137, { &P22052,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-P7501 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2609434 = {S44839, 71, { &P22052,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 3.2; en_US; GT-P7501 Build/HTJ85B.XXKJ7)*/
const Device UA2620283 = {S44840, 71, { &P22052,&P6437,&P16179,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 3.2; en-GB; GT-P7501 Build/HTJ85B.XXLB1)*/
const Device UA2566497 = {S44841, 150, { &P18317,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300T Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583342 = {S44842, 128, { &P18317,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; TF300T Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2709212 = {S44843, 151, { &P18317,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300TG Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2750963 = {S44844, 156, { &P18317,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; es-US; ASUS Transformer Pad TF300T Build/JOP40D) AppleWebKit/537.22 (KHTML like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2781969 = {S44845, 151, { &P18317,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300TL Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2753923 = {S44846, 134, { &P15364,&P16110,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/27.0.1453.94) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36*/
const Device UA2784543 = {S44847, 136, { &P15364,&P16110,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/27.0.1453.110) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36*/
const Device UA2873997 = {S44848, 98, { &P15364,&P16110,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/27.0.1453.116)*/
const Device UA2875520 = {S44849, 136, { &P15364,&P16110,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/27.0.1453.116) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2876697 = {S44850, 91, { &P15364,&P16110,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; chromeframe/27.0.1453.116)*/
const Device UA2883815 = {S44851, 177, { &P15364,&P16110,&P21920,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; chromeframe/27.0.1453.116)*/
const Device UA777058 = {S44852, 137, { &P14017,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LG-P990 Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1991482 = {S44853, 137, { &P14017,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; LG-P990 Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2403639 = {S44854, 138, { &P14017,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-P990 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2412132 = {S44855, 60, { &P14017,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P990 Build/IMM76L)*/
const Device UA2528841 = {S44856, 146, { &P14017,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; LG-P990/P99030a Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580004 = {S44857, 140, { &P14017,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-P990hN Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597523 = {S44858, 135, { &P14017,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en; LG-P990 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2700093 = {S44859, 146, { &P14017,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; LG-P990/P99030b Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA79881 = {S44860, 81, { &P15364,&P17285,&P17237,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/2*/
const Device UA103761 = {S44861, 116, { &P15364,&P17285,&P17237,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/530.5 (KHTML, like Gecko) Chrome/2.0.172.37 Safari/530.5*/
const Device UA140114 = {S44862, 114, { &P15364,&P17285,&P17237,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/530.6 (KHTML like Gecko) Chrome/2.0.174.0 Safari/530.6*/
const Device UA899171 = {S44863, 85, { &P15364,&P17285,&P17237,&P18092 }};	/*Useragent - Mozilla/6.0 (Windows; U; Windows NT 6.0; en-US) Gecko/X Chrome/2.0.172.6 Safari/530.7*/
const Device UA2522106 = {S44864, 96, { &P15364,&P17285,&P17237,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.10 (KHTML, like Gecko) Chrome/2.0.1.3 Safari/536.10*/
const Device UA2810767 = {S44865, 117, { &P15364,&P17285,&P17237,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; AppleWebKit/530.6; KHTML?? like Gecko; Chrome/2.0.174.0 Safari/530.6)*/
const Device UA48261 = {S44866, 174, { &P12663,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LG-E730/V09a Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA640191 = {S44867, 164, { &P12663,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; LG-E730/V10b Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA754240 = {S44868, 159, { &P12663,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-E730 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA795772 = {S44869, 164, { &P12663,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; LG-E730f Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA1904992 = {S44870, 164, { &P12663,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; LG-E730/V10a Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA1944193 = {S44871, 59, { &P12663,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-E730 Build/GRJ22)*/
const Device UA2535654 = {S44872, 164, { &P12663,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; LG-E730/V10j Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA2543865 = {S44873, 164, { &P12663,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ro-ro; LG-E730/V10f Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA2559590 = {S44874, 164, { &P12663,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; LG-E730/V10i Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA2933279 = {S44875, 136, { &P12663,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; LG-E730f Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Versi?n/4.0 Mobile Safari/533.1*/
const Device UA641641 = {S44876, 117, { &P20199,&P17483,&P17394,&P18092 }};	/*Useragent - NetFront/4.2 (BMP 1.0.4; U; en-us; LG; NetFront/4.2/AMB) Sprint LN272 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA699794 = {S44877, 116, { &P20199,&P17483,&P17394,&P18092 }};	/*Useragent - NetFront/4.2 (BMP 1.0.4; U; en-us; LG; NetFront/4.2/AMB) Boost LG272 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA3022357 = {S44878, 118, { &P20199,&P17483,&P17394,&P18092 }};	/*Useragent - NetFront/4.2 (BMP 1.0.4; U; en-us; LG; NetFront/4.2/AMB) Sprint LN272S MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA136467 = {S44879, 125, { &P17595,&P17296,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA136459 = {S44880, 125, { &P17595,&P17296,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA167083 = {S44881, 125, { &P17595,&P17296,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA184296 = {S44882, 125, { &P17595,&P17296,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA202899 = {S44883, 125, { &P17595,&P17296,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA277604 = {S44884, 125, { &P17595,&P17296,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA478120 = {S44885, 124, { &P17595,&P17296,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.2 (KHTML, like Gecko) Chrome/6.0.451.0 Safari/534.2*/
const Device UA643300 = {S44886, 125, { &P17595,&P17296,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.2 (KHTML, like Geckto) Chrome/6.0.451.0 Safari/534.2*/
const Device UA689958 = {S44887, 125, { &P17595,&P17296,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA1892703 = {S44888, 124, { &P17595,&P17296,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.456.0 Safari/534.3*/
const Device UA2332262 = {S44889, 125, { &P17595,&P17296,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/534.3 (KHTML	 like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA210514 = {S44890, 126, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA243120 = {S44891, 128, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Xoom Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA244921 = {S44892, 128, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Xoom Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA250722 = {S44893, 128, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; Xoom Build/HLK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA328202 = {S44894, 55, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; Xoom Build/HTJ85B)*/
const Device UA334216 = {S44895, 128, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; Xoom Build/HLK42B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA341547 = {S44896, 128, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; Xoom Build/HLK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA342293 = {S44897, 57, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; Xoom Build/HTK75D)*/
const Device UA354847 = {S44898, 57, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.4; Xoom Build/HLK75F)*/
const Device UA402664 = {S44899, 125, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLJ76) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402694 = {S44900, 125, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLJ86) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402703 = {S44901, 125, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HTJ85) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402704 = {S44902, 125, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLK08) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402708 = {S44903, 125, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLK11) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402713 = {S44904, 125, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HTJ77) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402717 = {S44905, 126, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLK36B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402718 = {S44906, 126, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLJ77B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402719 = {S44907, 125, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLK42) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402726 = {S44908, 128, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Xoom Build/HTK55B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402727 = {S44909, 126, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLK33C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402731 = {S44910, 125, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HTJ78) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402735 = {S44911, 126, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLK01B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402742 = {S44912, 125, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLK25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402745 = {S44913, 125, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLJ83) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402750 = {S44914, 128, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; Xoom Build/HLK42C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402777 = {S44915, 57, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.2; Xoom Build/HLK42B)*/
const Device UA402782 = {S44916, 128, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; Xoom Build/HLK42E) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402787 = {S44917, 127, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; Xoom Build/HLK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402799 = {S44918, 127, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Xoom Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402839 = {S44919, 135, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Xoom Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA402884 = {S44920, 125, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HTJ74) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402946 = {S44921, 135, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.4; en-us; Xoom Build/HLK75F) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA402962 = {S44922, 128, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.4; en-us; Xoom Build/HLK75E) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402968 = {S44923, 128, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.4; en-us; Xoom Build/HLK75F) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA403013 = {S44924, 126, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Xoom Build/HLK28B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA403023 = {S44925, 128, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.6; en-us; Xoom Build/HLK75H) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA403032 = {S44926, 128, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.6; en-us; Xoom Build/HLK75G) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA403108 = {S44927, 126, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; ru-; Xoom Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA403184 = {S44928, 128, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Xoom Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA403294 = {S44929, 135, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.6; en-us; Xoom Build/HLK75H) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA403298 = {S44930, 57, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.6; Xoom Build/HLK75H)*/
const Device UA2025744 = {S44931, 133, { &P15869,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; MZ505 Build/Moto_MZ505) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA33440 = {S44932, 142, { &P12836,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; IDEOS S7 Slim Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA56974 = {S44933, 141, { &P12836,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; IDEOS S7 Slim Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA558873 = {S44934, 66, { &P12836,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; IDEOS S7 Slim Build/FRG83G)*/
const Device UA2019377 = {S44935, 130, { &P12836,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; IDEOS S7 Slim Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2028728 = {S44936, 142, { &P12836,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; IDEOS S7 Slim Build/pirate) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2005305 = {S44937, 96, { &P18254,&P5690,&P17381,&P18091 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN; rv:1.8.0.11) Gecko/X Firefox/1.5.0.11; 360spider*/
const Device UA2558737 = {S44938, 158, { &P20129,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2668468 = {S44939, 159, { &P20129,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2C Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2675088 = {S44940, 159, { &P20129,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2S Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2719259 = {S44941, 160, { &P20129,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2SC Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2749101 = {S44942, 159, { &P20129,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2A Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2855433 = {S44943, 159, { &P20129,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; MI 2 Build/miui.es) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2858606 = {S44944, 166, { &P20129,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; MI 2 Build/miui.es JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2892132 = {S44945, 167, { &P20129,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; MI 2S Build/miui.es JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2997784 = {S44946, 158, { &P20129,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; MI 2S Build/JRO03L) AppleWebKit/534.30 (KHTML, zoals Gecko) Versie/4.0 MobielSafari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA3014398 = {S44947, 167, { &P20129,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; MI 2A Build/miui.es JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA3048693 = {S44948, 157, { &P20129,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; MI 2 Build/JRO03L) AppleWebKit/534.30 (KHTML, zoals Gecko) Versie/4.0 MobielSafari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA3100527 = {S44949, 156, { &P20129,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MI 2 Build/WiUi) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2664628 = {S44950, 122, { &P20741,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 520; Vodafone)*/
const Device UA2675610 = {S44951, 120, { &P20741,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 520; Orange)*/
const Device UA2714805 = {S44952, 112, { &P20741,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia 520)*/
const Device UA2715741 = {S44953, 120, { &P20741,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 520; ANZ914)*/
const Device UA2738337 = {S44954, 120, { &P20741,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 520; Meteor)*/
const Device UA2892921 = {S44955, 119, { &P20741,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia 520T; CMCC)*/
const Device UA3095937 = {S44956, 120, { &P20741,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 520; ANZ915)*/
const Device UA3102662 = {S44957, 151, { &P20741,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 520),gzip(gfe) (via docs.google.com/viewer)*/
const Device UA800072 = {S44958, 81, { &P15364,&P17285,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.8.1b2) Gecko/X Firefox/2.0b2*/
const Device UA2288339 = {S44959, 67, { &P15364,&P17285,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; U; tr; rv:1.8.1) Gecko/X Firefox/2.0.0*/
const Device UA2566633 = {S44960, 137, { &P18283,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566692 = {S44961, 137, { &P18283,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I535 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566806 = {S44962, 138, { &P18283,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9300 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566943 = {S44963, 137, { &P18283,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T999 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567114 = {S44964, 138, { &P18283,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SCH-I535 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583417 = {S44965, 138, { &P18283,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-T999 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583790 = {S44966, 130, { &P18283,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I535 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2600029 = {S44967, 130, { &P18283,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667061 = {S44968, 130, { &P18283,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T999 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2490690 = {S44969, 64, { &P15364,&P17285,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; WOW64) Presto/2.12.388 Version/12.13*/
const Device UA2491504 = {S44970, 57, { &P15364,&P17285,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.13*/
const Device UA2511633 = {S44971, 80, { &P15364,&P17285,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.5 (build 02842)) Presto/2.12.388 Version/12.13*/
const Device UA2511832 = {S44972, 67, { &P15364,&P17285,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.13,gzip(gfe)*/
const Device UA2512363 = {S44973, 79, { &P15364,&P17285,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6001)) Presto/2.12.388 Version/12.13*/
const Device UA2597674 = {S44974, 79, { &P15364,&P17285,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 5972)) Presto/2.12.388 Version/12.13*/
const Device UA2699893 = {S44975, 80, { &P15364,&P17285,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.10 (build 5310)) Presto/2.12.388 Version/12.13*/
const Device UA2731078 = {S44976, 79, { &P15364,&P17285,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.9 (build 4953)) Presto/2.12.388 Version/12.13*/
const Device UA2850915 = {S44977, 79, { &P15364,&P17285,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.13*/
const Device UA2851468 = {S44978, 80, { &P15364,&P17285,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; MRA 5.7 (build 03797)) Presto/2.12.388 Version/12.13*/
const Device UA251389 = {S44979, 132, { &P17939,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; K080 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA638017 = {S44980, 56, { &P17939,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; K080 Build/GRI40)*/
const Device UA903494 = {S44981, 131, { &P17939,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Vox Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1971405 = {S44982, 55, { &P17939,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; Vox Build/GRJ22)*/
const Device UA603636 = {S44983, 124, { &P15364,&P7133,&P22457,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19*/
const Device UA603712 = {S44984, 117, { &P15364,&P7133,&P22457,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19*/
const Device UA2301223 = {S44985, 118, { &P15364,&P16110,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.1.249.1064 Safari/532.5*/
const Device UA2301382 = {S44986, 116, { &P15364,&P16110,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.78 Safari/532.5*/
const Device UA2559897 = {S44987, 146, { &P23910,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT30at Build/9.1.F.1.120) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566657 = {S44988, 142, { &P23910,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT30at Build/9.1.F.1.120) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2667558 = {S44989, 64, { &P23910,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LT30at Build/9.1.F.1.120)*/
const Device UA2765168 = {S44990, 146, { &P23910,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT30at Build/9.1.F.1.119) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2989781 = {S44991, 144, { &P23910,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; SonyLT30at Build/9.1.F.1.120) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3045504 = {S44992, 66, { &P23910,&P18111,&P18112,&P18092 }};	/*Useragent - SonyLT30at Build/9.1.F.1.120 stagefright/1.2 (Linux;Android 4.1.2)*/
const Device UA2671119 = {S44993, 129, { &P20705,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2672983 = {S44994, 141, { &P20705,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One 801e Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675630 = {S44995, 135, { &P20705,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTCONE Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676354 = {S44996, 136, { &P20705,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713530 = {S44997, 138, { &P20705,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC_PN071 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2716846 = {S44998, 128, { &P20705,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTCONE Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2717077 = {S44999, 131, { &P20705,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC_PN071 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2029577 = {S45000, 139, { &P12942,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I927 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2451678 = {S45001, 61, { &P12942,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I927 Build/IMM76D)*/
const Device UA32748 = {S45002, 136, { &P12755,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ZTE-Z990 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48575 = {S45003, 136, { &P12755,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ZTE-Z990 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA694942 = {S45004, 60, { &P12755,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; ZTE-Z990 Build/GRJ22)*/
const Device UA2219145 = {S45005, 142, { &P12755,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Gingerbread; en-us; ZTE-Z990 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA33287 = {S45006, 107, { &P15874,&P17322,&P22720,&P18092 }};	/*Useragent - Mozilla/5.0 (MeeGo; NokiaN9) AppleWebKit/534.13 (KHTML, like Gecko) NokiaBrowser/8.5.0 Mobile Safari/534.13*/
const Device UA8755 = {S45007, 153, { &P13503,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; de-at; HTC Desire Z  1.82.166.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA8776 = {S45008, 152, { &P13503,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; HTC Desire Z 1.85.861.4 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA8778 = {S45009, 152, { &P13503,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; de-ch; HTC Desire Z 1.82.166.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA8789 = {S45010, 152, { &P13503,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Desire Z 1.82.161.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA8803 = {S45011, 150, { &P13503,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DL DesireZ v3.4 final Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA8822 = {S45012, 152, { &P13503,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; HTC Desire Z 1.85.861.3 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11905 = {S45013, 138, { &P13503,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; HTC-A7275 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA12449 = {S45014, 143, { &P13503,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC_DesireZ_A7272 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA12508 = {S45015, 149, { &P13503,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Desire Z 1.34.161.6 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA12530 = {S45016, 152, { &P13503,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Desire Z 1.34.161.6 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA12545 = {S45017, 152, { &P13503,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Desire Z 1.72.161.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA12546 = {S45018, 152, { &P13503,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; de-ch; HTC Desire Z 1.72.166.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA12786 = {S45019, 149, { &P13503,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; de-at; HTC Desire Z 1.34.166.5 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38905 = {S45020, 145, { &P13503,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC_DesireZ_A7272 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52592 = {S45021, 150, { &P13503,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; HTC_DesireZ-orange-LS Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA62827 = {S45022, 135, { &P13503,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-ca; HTC-A7275 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA63281 = {S45023, 146, { &P13503,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC_DesireZ_A7272 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71761 = {S45024, 152, { &P13503,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; de-ch; HTC Desire Z 1.34.166.5 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2008920 = {S45025, 140, { &P18386,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I757M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2227017 = {S45026, 62, { &P18386,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I757M Build/IMM76D)*/
const Device UA2762584 = {S45027, 138, { &P18386,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SGH-I757M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2435852 = {S45028, 149, { &P18385,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 7 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2512624 = {S45029, 149, { &P18385,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 7 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2538619 = {S45030, 148, { &P18385,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2582232 = {S45031, 155, { &P18385,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2703819 = {S45032, 156, { &P18385,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 7 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708065 = {S45033, 149, { &P18385,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2728869 = {S45034, 156, { &P18385,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 7 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781628 = {S45035, 156, { &P18385,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958061 = {S45036, 147, { &P18385,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/4.60; CyanogenMod-4.60) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA776816 = {S45037, 141, { &P16319,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U8860 Build/HuaweiU8860) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025521 = {S45038, 127, { &P16319,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; U8860 Build/HuaweiU8860) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2033577 = {S45039, 63, { &P16319,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; U8860 Build/HuaweiU8860)*/
const Device UA2034441 = {S45040, 132, { &P16319,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; U8860 Build/HuaweiU8860) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2787156 = {S45041, 143, { &P16319,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; U8860 Build/HuaweiU8860) AppleWebKit/534.30 (KHTML, like Gecko) Phi?n b?n/4.0 Mobile Safari/534.30*/
const Device UA192316 = {S45042, 88, { &P15364,&P7133,&P21896,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/11.0.696.57)*/
const Device UA314727 = {S45043, 95, { &P15364,&P7133,&P21896,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/11.0.696.65)*/
const Device UA535292 = {S45044, 176, { &P15364,&P7133,&P21896,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/11.0.696.68; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA1882219 = {S45045, 132, { &P15364,&P7133,&P21896,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/11.0.696.71) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24*/
const Device UA2135355 = {S45046, 125, { &P15364,&P7133,&P21896,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/11.0.696.57) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.57 Safari/534.24*/
const Device UA2259318 = {S45047, 132, { &P15364,&P7133,&P21896,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/11.0.696.57) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.57 Safari/534.24*/
const Device UA2282658 = {S45048, 132, { &P15364,&P7133,&P21896,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/11.0.696.65) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.65 Safari/534.24*/
const Device UA2284051 = {S45049, 125, { &P15364,&P7133,&P21896,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/11.0.696.68) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA2284348 = {S45050, 132, { &P15364,&P7133,&P21896,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/11.0.696.68) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA2286186 = {S45051, 125, { &P15364,&P7133,&P21896,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/11.0.696.71) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24*/
const Device UA3065148 = {S45052, 129, { &P18554,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BNTV400 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2676261 = {S45053, 137, { &P19861,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-I605 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676493 = {S45054, 137, { &P19861,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676740 = {S45055, 130, { &P19861,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-I605 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2707730 = {S45056, 130, { &P19861,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA251959 = {S45057, 122, { &P15364,&P7133,&P18210,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.1.0.5001 Safari/532.4*/
const Device UA623390 = {S45058, 101, { &P15364,&P7133,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);en-CA;kml:2.2;client:Free;type:default)*/
const Device UA627706 = {S45059, 101, { &P15364,&P7133,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);de-DE;kml:2.2;client:Free;type:default)*/
const Device UA629198 = {S45060, 101, { &P15364,&P7133,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);fr-FR;kml:2.2;client:Free;type:default)*/
const Device UA630100 = {S45061, 101, { &P15364,&P7133,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);it-IT;kml:2.2;client:Free;type:default)*/
const Device UA642780 = {S45062, 102, { &P15364,&P7133,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7600.0);es-419;kml:2.2;client:Free;type:default)*/
const Device UA647798 = {S45063, 101, { &P15364,&P7133,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);en-US;kml:2.2;client:Free;type:default)*/
const Device UA672119 = {S45064, 98, { &P15364,&P7133,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);en;kml:2.2;client:Free;type:default)*/
const Device UA675479 = {S45065, 101, { &P15364,&P7133,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);en-GB;kml:2.2;client:Free;type:default)*/
const Device UA686583 = {S45066, 101, { &P15364,&P7133,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7600.0);en-GB;kml:2.2;client:Free;type:default)*/
const Device UA692751 = {S45067, 101, { &P15364,&P7133,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);de-CH;kml:2.2;client:Free;type:default)*/
const Device UA699290 = {S45068, 101, { &P15364,&P7133,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);en-AU;kml:2.2;client:Free;type:default)*/
const Device UA712624 = {S45069, 101, { &P15364,&P7133,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7600.0);en-US;kml:2.2;client:Free;type:default)*/
const Device UA738722 = {S45070, 101, { &P15364,&P7133,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);ru-RU;kml:2.2;client:Free;type:default)*/
const Device UA742608 = {S45071, 102, { &P15364,&P7133,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);es-419;kml:2.2;client:Free;type:default)*/
const Device UA748690 = {S45072, 101, { &P15364,&P7133,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.1.7601.1);nl-NL;kml:2.2;client:Free;type:default)*/
const Device UA2487638 = {S45073, 117, { &P15364,&P7133,&P22464,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17*/
const Device UA2487639 = {S45074, 124, { &P15364,&P7133,&P22464,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17*/
const Device UA2537072 = {S45075, 132, { &P18465,&P18110,&P21167,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Safari/535.19 Silk-Accelerated=true*/
const Device UA1985847 = {S45076, 121, { &P18643,&P5043,&P18517,&P18092 }};	/*Useragent - Nokia311/5.0 (03.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA2695913 = {S45077, 84, { &P18643,&P5043,&P18517,&P18092 }};	/*Useragent - Nokia311/5.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 Nokia311/05.92 CLDC-1.1*/
const Device UA2366512 = {S45078, 68, { &P15364,&P18110,&P19806,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:17.0) Gecko/17.0 Thunderbird/17.0*/
const Device UA292320 = {S45079, 110, { &P15364,&P5690,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.1 (KHTML, like Gecko) CEF/0.0.0.0 Safari/534.1*/
const Device UA345433 = {S45080, 104, { &P15364,&P5690,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1.1; U; en) AppleWebKit/535.2 (KHTML, like Gecko) xp/234.76 Safari/535.2 01/07*/
const Device UA433564 = {S45081, 116, { &P15364,&P5690,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/528+ (KHTML, like Gecko, Safari/528.0) Lunascape/5.0.2.0*/
const Device UA468052 = {S45082, 113, { &P15364,&P5690,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; fa-IR) AppleWebKit/532.4 (KHTML, like Gecko) Usejump/0.10.2 Safari/532.4*/
const Device UA1932632 = {S45083, 101, { &P15364,&P5690,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Lunascape/6.5.8.24780 Safari/535.1*/
const Device UA2205324 = {S45084, 101, { &P15364,&P5690,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.3 (KHTML, like Gecko) Lunascape/6.8.0.25815 Safari/535.3*/
const Device UA2208483 = {S45085, 132, { &P15364,&P5690,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/523.15 (KHTML, like Gecko, Safari/419.3) Arora/0.3 (Change: 287 c9dfb30)*/
const Device UA2355673 = {S45086, 92, { &P15364,&P5690,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.34 (KHTML, like Gecko) iLivid/1.0 Safari/534.34*/
const Device UA2368538 = {S45087, 79, { &P15364,&P5690,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0(Windows NT 5.1) AppleWebKit/534.50 (KHTML, like Gecko) Safari/535.1*/
const Device UA2370155 = {S45088, 116, { &P15364,&P5690,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 5.1; en-US) AppleWebKit/529 (KHTML, like Gecko, Safari/529.0) Lunascape/4.9.9.94*/
const Device UA2396672 = {S45089, 94, { &P15364,&P5690,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.11 (KHTML, Like Gecko) 20.0.1132.47 Safari/536.11*/
const Device UA2423673 = {S45090, 90, { &P15364,&P5690,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (N; Windows NT 5.1) AppleWebKit/534.34 (KHTML, like Gecko) Raptr Safari/534.34*/
const Device UA2505960 = {S45091, 96, { &P15364,&P5690,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) SuperBird/23.0 Safari/537.11*/
const Device UA2532390 = {S45092, 104, { &P15364,&P5690,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML, like Gecko) Superbird/24.0.1312.57 Safari/537.17*/
const Device UA2543619 = {S45093, 116, { &P15364,&P5690,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 5.2; ru-RU) AppleWebKit/529 (KHTML, like Gecko, Safari/529.0) Lunascape/4.9.9.94*/
const Device UA2588150 = {S45094, 120, { &P15364,&P5690,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; cs-CZ) AppleWebKit/533.3 (KHTML, like Gecko) Lunascape/6.1.0.20995 Safari/533.3*/
const Device UA2599958 = {S45095, 88, { &P15364,&P5690,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.21 (KHTML, like Gecko) legacy Safari/537.21*/
const Device UA2716715 = {S45096, 103, { &P15364,&P5690,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Superbird/27.0.1450.0 Safari/537.36*/
const Device UA2752922 = {S45097, 88, { &P15364,&P5690,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.21 (KHTML, like Gecko) Capter Safari/537.21*/
const Device UA2796778 = {S45098, 104, { &P15364,&P5690,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Superbird/27.0.1454.94 Safari/537.22*/
const Device UA2804993 = {S45099, 117, { &P15364,&P5690,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; pt-BR) AppleWebKit/534.12 (KHTML, like Gecko) Navscape/Pre-0.1 Safari/534.12*/
const Device UA3042860 = {S45100, 92, { &P15364,&P5690,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.14 (KHTML, like Gecko) JavaFX/2.2 Safari/535.14*/
const Device UA460993 = {S45101, 85, { &P17595,&P17294,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.4; fr; rv:1.9.1.7) Gecko/X Firefox/3.5.7*/
const Device UA2621572 = {S45102, 130, { &P15932,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; GT-P3110 Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622921 = {S45103, 61, { &P15932,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-P3110 Build/JOP40D)*/
const Device UA2623136 = {S45104, 59, { &P15932,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2; GT-P3110 Build/JOP40C)*/
const Device UA2623225 = {S45105, 113, { &P15932,&P18598,&P19802,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.2.1; en-US; GT-P3110 Build/cm_p3110-userdebug 4.2.1 JOP40D eng..X.X test-keys)*/
const Device UA2631686 = {S45106, 60, { &P15932,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P3110 Build/JDQ39)*/
const Device UA2632535 = {S45107, 139, { &P15932,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3110 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2636905 = {S45108, 138, { &P15932,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-P3110 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2794879 = {S45109, 132, { &P15932,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3110 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2997176 = {S45110, 131, { &P15932,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-P3110 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2774331 = {S45111, 151, { &P17595,&P18207,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.29 Safari/537.36 OPR/15.0.1147.24 (Edition Next)*/
const Device UA2885387 = {S45112, 152, { &P17595,&P18207,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100 (Edition Next)*/
const Device UA2969602 = {S45113, 152, { &P17595,&P18207,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100 (Edition Next)*/
const Device UA3026377 = {S45114, 137, { &P17595,&P18207,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153*/
const Device UA3027884 = {S45115, 137, { &P17595,&P18207,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153*/
const Device UA3028474 = {S45116, 137, { &P17595,&P18207,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153*/
const Device UA3030064 = {S45117, 137, { &P17595,&P18207,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153*/
const Device UA3108062 = {S45118, 137, { &P17595,&P18207,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153*/
const Device UA450850 = {S45119, 72, { &P15364,&P7133,&P17209,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows NT 6.1; Win64; x64; rv:4.0a1) Gecko/X Firefox/7.0a1*/
const Device UA598020 = {S45120, 73, { &P15364,&P7133,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:10.0a1) Gecko/X Firefox/7.0.1*/
const Device UA855178 = {S45121, 86, { &P15364,&P7133,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:7.0.1) Gecko/X Firefox/7.0.12011-10-16 20:23:00*/
const Device UA872322 = {S45122, 54, { &P15364,&P7133,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0(WindowsNT6.1;rv:7.0.1)Gecko/XFirefox/7.0.1*/
const Device UA1956764 = {S45123, 161, { &P15364,&P7133,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1;  rv:7.0.1) Chrome/12.0.702.0 Gecko/X AppleWebKit/533.3 (KHTML, like Gecko) Version/5.0.5 Safari/533.3  Awesomium/1.6.5 Firefox/7.0.1*/
const Device UA2350161 = {S45124, 82, { &P15364,&P7133,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.1.2) Gecko/X Firefox/7 GTB5*/
const Device UA2359831 = {S45125, 75, { &P15364,&P7133,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:7.0) Gecko/X Firefox/7.0 Firefox/7.0*/
const Device UA2370768 = {S45126, 100, { &P15364,&P7133,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:7.0a1) Gecko/X Firefox/7.0a1 (.NET CLR 3.5.30729)*/
const Device UA42786 = {S45127, 131, { &P17686,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; de-de; MZ601 Build/H.6.3-25-9) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA43050 = {S45128, 135, { &P17686,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; MZ601 Build/H.6.1-17_CN-40) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA58676 = {S45129, 129, { &P17686,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; MZ601 Build/H.6.3-14) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA58700 = {S45130, 129, { &P17686,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; MZ601 Build/H.6.3-16) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA58715 = {S45131, 131, { &P17686,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; MZ601 Build/H.6.3-25-5) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA58720 = {S45132, 129, { &P17686,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-au; MZ601 Build/H.6.3-25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA58728 = {S45133, 129, { &P17686,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; MZ601 Build/H.6.3-24) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA75422 = {S45134, 129, { &P17686,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; MZ601 Build/H.6.3-20) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA75812 = {S45135, 131, { &P17686,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; ko-kr; MZ601 Build/H.6.3-25.2) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA330025 = {S45136, 60, { &P17686,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; MZ601 Build/H.6.3-25-5)*/
const Device UA657589 = {S45137, 131, { &P17686,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; es-es; MZ601 Build/H.6.3-28-3) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA720042 = {S45138, 156, { &P14186,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-S5830 Build/GRWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027694 = {S45139, 162, { &P14186,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5830 Build/GRWK74; CyanogenMod7-APOCALYPSE) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2383416 = {S45140, 156, { &P14186,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5830 Build/GRWK74; JOYOS-CyanogenMod) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2476752 = {S45141, 148, { &P14186,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT-S5830 Build/GRWK74; CM7.2-RC3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2205261 = {S45142, 104, { &P18254,&P18110,&P19813,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.7.0 Safari/534.34*/
const Device UA2258602 = {S45143, 102, { &P18254,&P18110,&P19813,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.7.0 Safari/534.34*/
const Device UA2198253 = {S45144, 141, { &P17625,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST25i Build/6.1.1.B.1.10) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2205675 = {S45145, 141, { &P17625,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST25a Build/6.1.1.B.1.10) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2467182 = {S45146, 138, { &P17625,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xperia U Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2973963 = {S45147, 129, { &P18554,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BNTV400 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2741723 = {S45148, 149, { &P18113,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798991 = {S45149, 142, { &P18113,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA458641 = {S45150, 97, { &P18309,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; PI86100)*/
const Device UA782831 = {S45151, 97, { &P18309,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Radiant)*/
const Device UA2434569 = {S45152, 149, { &P17346,&P18117,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9B206 Safari/7534.48.3*/
const Device UA2435629 = {S45153, 147, { &P17346,&P18117,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9B176 Safari/7534.48.3*/
const Device UA2659113 = {S45154, 145, { &P17346,&P18117,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 511 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML like Gecko) CriOS/23.0.1271.91 Mobile/9B206 Safari/7534.48.3*/
const Device UA2483141 = {S45155, 138, { &P20146,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2530320 = {S45156, 139, { &P20146,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA222009 = {S45157, 104, { &P14651,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; 7 Trophy T8686)*/
const Device UA234957 = {S45158, 98, { &P14651,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; 7 Trophy)*/
const Device UA630711 = {S45159, 95, { &P14651,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Spark)*/
const Device UA798417 = {S45160, 97, { &P14651,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; PC40100)*/
const Device UA925707 = {S45161, 105, { &P14651,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Touch-IT Trophy)*/
const Device UA925853 = {S45162, 96, { &P14651,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Trophy)*/
const Device UA2670891 = {S45163, 138, { &P18295,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EndeavorU Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675272 = {S45164, 134, { &P18295,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; One X Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676010 = {S45165, 138, { &P18295,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676423 = {S45166, 131, { &P18295,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2677289 = {S45167, 138, { &P18295,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One X Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA29943 = {S45168, 146, { &P12280,&P17131,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5H11 Safari/525.20*/
const Device UA63029 = {S45169, 144, { &P12280,&P17131,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_2 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5G77 Safari/525.20*/
const Device UA140907 = {S45170, 143, { &P12280,&P17131,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_2 like Mac OS X; en-us) AppleWebKit/525.181 (KHTML, like Gecko) Version/3.1.1 Mobile/5H11 Safari/525.20*/
const Device UA69392 = {S45171, 151, { &P15208,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; SAMSUNG-SGH-I997R Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA616490 = {S45172, 75, { &P15208,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SAMSUNG-SGH-I997R Build/GINGERBREAD)*/
const Device UA2020121 = {S45173, 153, { &P15208,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; SAMSUNG-SGH-I997R Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2027622 = {S45174, 144, { &P15208,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; SAMSUNG-SGH-I997R Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2439857 = {S45175, 149, { &P15208,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; SAMSUNG-SGH-I997R Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2898053 = {S45176, 76, { &P15364,&P18110,&P22171,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:23.0) Gecko/X Firefox/23.0 Iceweasel/23.0*/
const Device UA2944299 = {S45177, 69, { &P15364,&P18110,&P22171,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 on x86_64; rv:23.0) Gecko/X Firefox/23.0*/
const Device UA3093285 = {S45178, 65, { &P15364,&P18110,&P22171,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:23.0) Gecko/X Iceweasel/23.0.1*/
const Device UA2245677 = {S45179, 110, { &P18435,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; ARM; Trident/6.0; Touch; .NET4.0E; .NET4.0C; Tablet PC 2.0)*/
const Device UA2363986 = {S45180, 118, { &P18435,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; ARM; Trident/6.0; Touch; .NET4.0E; .NET4.0C; Tablet PC 2.0; ARMBJS)*/
const Device UA2424724 = {S45181, 114, { &P18435,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; ARM; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; ms-office)*/
const Device UA2595102 = {S45182, 135, { &P18435,&P16110,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; ARM; Trident/6.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; ARMBJS; ms-office; MSOffice 15)*/
const Device UA2673870 = {S45183, 69, { &P15364,&P18110,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 on x86_64; rv:21.0) Gecko/X Firefox/21.0*/
const Device UA2674972 = {S45184, 76, { &P15364,&P18110,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:21.0) Gecko/X Firefox/21.0 Iceweasel/21.0*/
const Device UA2784973 = {S45185, 117, { &P15364,&P18110,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en; rv:1.9.0.19) Gecko/X Mozilla/5.0 (X11; Linux i686; rv:21.0) Gecko/X Firefox/21.0*/
const Device UA2870268 = {S45186, 63, { &P15364,&P18110,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:21.0) Gecko/X Iceweasel/21.0*/
const Device UA3014089 = {S45187, 100, { &P15364,&P18110,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Ultimate Edition; X11; Linux x64; rv:8.0) Gecko/X Ultimate Edition / 3.5 Firefox/21.0.1*/
const Device UA53164 = {S45188, 143, { &P14106,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-I9000T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA420781 = {S45189, 67, { &P14106,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-I9000T Build/GINGERBREAD)*/
const Device UA1995373 = {S45190, 143, { &P14106,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-I9000T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2709040 = {S45191, 129, { &P18556,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BNTV600 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA14514 = {S45192, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; ME511 Build/RuthGC_0.18.4) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA20191 = {S45193, 146, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-ru; Beeline M2 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA24296 = {S45194, 138, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gh; l9 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA40938 = {S45195, 141, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; TSP21 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA44063 = {S45196, 141, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; th-th; Smile Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA53707 = {S45197, 148, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-ru; Beeline_E300 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA53706 = {S45198, 143, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-by; MTC Evo Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA57269 = {S45199, 156, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-bd; Grameenphone Crystal Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA67880 = {S45200, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; de-de; ADM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA68641 = {S45201, 141, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; AIRIS Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA165754 = {S45202, 140, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A28 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA166497 = {S45203, 140, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; X6D Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA298381 = {S45204, 188, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Android for Livall TCC8902 GENBook (multi-language) Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA298575 = {S45205, 132, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-gb; jv02 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA299375 = {S45206, 144, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MID-7AS Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA299753 = {S45207, 141, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; wwe10 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA312190 = {S45208, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; es-es; MW0811 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA354459 = {S45209, 63, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; M718 Build/ECLAIR)*/
const Device UA354464 = {S45210, 141, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; M718 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA355512 = {S45211, 151, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.4; en-us; SHAGM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA370559 = {S45212, 118, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; E239 Build/ERD79) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA385275 = {S45213, 140, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A32 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA387350 = {S45214, 139, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; md-us; deb Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA390586 = {S45215, 150, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-za; NUM7UGW Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA393508 = {S45216, 153, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; ViewQuest SLATE2 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA417962 = {S45217, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.2; es-es; STM7UW Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA432922 = {S45218, 143, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-2010.11.4; de-de; XST2 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA434926 = {S45219, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; es-es; ARGM7U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA446937 = {S45220, 145, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; ZEN Touch 2 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA497322 = {S45221, 69, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - PVPlayer E10i CORE/7.507.1.1 OpenCORE/2.05 (Linux;Android2.1-update1)*/
const Device UA503730 = {S45222, 151, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; PALROID PR70HD Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA525243 = {S45223, 144, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; ZTE-LINK Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA535926 = {S45224, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.2; es-es; STM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA540897 = {S45225, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.8; es-es; STM7UW Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA543116 = {S45226, 155, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-us; MID-710YH-M4 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA553209 = {S45227, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.2; es-es; STM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA559046 = {S45228, 150, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.9; es-es; STM703U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA566770 = {S45229, 114, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; YL-Coolpad_D539/1.01.008.X.D539; 320*480; CTC/2.0) CoolpadWebkit/530.18*/
const Device UA632313 = {S45230, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-us; EBW204 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA654208 = {S45231, 141, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; V7500 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA658992 = {S45232, 148, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-uk; WR762 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA677415 = {S45233, 130, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; OMAP_SS Build/ECLAIR) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA681549 = {S45234, 147, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; WellcoM-A88 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA690213 = {S45235, 148, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-uk; WR761 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA691752 = {S45236, 142, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; M82VG Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA692418 = {S45237, 147, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-us; M702 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA704295 = {S45238, 152, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-tw; Taiwan Mobile T2 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA713198 = {S45239, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.9; en-us; NXM707 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA713358 = {S45240, 141, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; L8 Build/L8_R05_X) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA716386 = {S45241, 151, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; Maestro Build/Maestro_R26_X) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA716701 = {S45242, 150, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.9; es-es; STM703U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA721811 = {S45243, 144, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; OMAP_SS Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA726463 = {S45244, 151, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.1.0.1; es-es; ARGM7U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA750311 = {S45245, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; es-es; STM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA762678 = {S45246, 114, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android 2.1-update1;zh-tw)AppleWebKit/530.17(KHTML,like Gecko)Version/4.0 Mobile Safari/530.17*/
const Device UA768067 = {S45247, 152, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.1.21; en-us; PMP3084B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA768387 = {S45248, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.1.0; es-es; STM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA770193 = {S45249, 148, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-us; WR780 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA791948 = {S45250, 152, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.1.15; en-us; PMP3084B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA801551 = {S45251, 150, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.7; es-es; STM703U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA811329 = {S45252, 150, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; de-de; Genesis Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA880292 = {S45253, 158, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; HS-E87/E434.6.01.02.00; 240*320; CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/3.1.2 Mobile Safari/530.17*/
const Device UA1858705 = {S45254, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; sv-se; SIM710 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1859564 = {S45255, 145, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-kr; ESP_E201V Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1865233 = {S45256, 150, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Scroll Tablet Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1882183 = {S45257, 144, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; tr-tr; hkdkc100 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1886246 = {S45258, 150, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.3; es-es; STM703U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1894130 = {S45259, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.9; es-es; STM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1894806 = {S45260, 114, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; YL-Coolpad_E239/1.01.015.X.E239; 320*480; CTC/2.0) CoolpadWebkit/530.18*/
const Device UA1895544 = {S45261, 141, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; F9191 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1900441 = {S45262, 118, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; D539 Build/ERD79) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1901089 = {S45263, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.1.5; es-es; ARGM7U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1920412 = {S45264, 151, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.6; es-es; GASEM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1922235 = {S45265, 153, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.4; es-es; DAHSM710UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1938729 = {S45266, 157, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; es-es; PAPYRE_pad_7.1 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1941477 = {S45267, 152, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.1.3; es-es; GASEM710C Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1967010 = {S45268, 151, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.8; es-es; GASEM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018100 = {S45269, 132, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-gb; TOSHIBA_AC_AND_AZ) AppleWebkit/530.17(KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018116 = {S45270, 139, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; w9 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018218 = {S45271, 150, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; es-es; SIGMM7U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018223 = {S45272, 114, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; YL-Coolpad_N930/1.01.012.X.N930; 480*800; CTC/2.0) CoolpadWebkit/530.17*/
const Device UA2018306 = {S45273, 142, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; Nvsbl Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018469 = {S45274, 143, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-2010.12.7; de-de; XST2 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018540 = {S45275, 69, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - PVPlayer U20i CORE/7.507.1.1 OpenCORE/2.05 (Linux;Android2.1-update1)*/
const Device UA2018604 = {S45276, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.2; nl-nl; LUM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018646 = {S45277, 136, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; ja-jp; NTT-ME ME-DPF1.0 Build/0001) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA2018740 = {S45278, 152, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; es-es; HackDROID Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018864 = {S45279, 195, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-TC21-HCV13.0-B20; en-us; ATZ 7015/7015A HONEYCOMB V13.0-B20 ACADEV Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018954 = {S45280, 147, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; WellcoM-A86 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018974 = {S45281, 151, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.5; es-es; GASEM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019002 = {S45282, 169, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Android 2.1-update1; zh-cn; ZXY-ZTE-C X920/X920-C-TS9CD038; 240*320;CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019012 = {S45283, 148, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; W711 Build/W711_VER10.21) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019078 = {S45284, 163, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; TL-Twin_T90/E370.6.01.02.00; 320*480; CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/3.1.2 Mobile Safari/530.17*/
const Device UA2019101 = {S45285, 155, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; D000-X-C01 Build/01.10.12-01.20) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019156 = {S45286, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.1.2; es-es; STM7UW Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019180 = {S45287, 151, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Android 2.1;zh-cn;HS-E89/E329.6.02.00.00;240*320;CTC/2.0) AppleWebKit/530.17(KHTML, like Gecko) Version/3.1.2 Mobile Safari/530.17*/
const Device UA2019199 = {S45288, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.4; es-es; STM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019235 = {S45289, 152, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.1.1; es-es; GASEM710C Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019215 = {S45290, 152, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.5; es-es; GASEM710C Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019242 = {S45291, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.8; es-es; STM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019274 = {S45292, 162, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; HX-HE-N100/E331.6.01.01.00; 240*320; CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/3.1.2 Mobile Safari/530.17*/
const Device UA2019365 = {S45293, 146, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; pt-br; BAK Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019484 = {S45294, 172, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-v1.02_15.14-M_ES-2011.03.23; es-es; Pandora Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2020112 = {S45295, 150, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.6; es-es; STM703U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2020222 = {S45296, 165, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; YFZ-EPHONE_A8/E381.6.01.02.00; 240*320; CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/3.1.2 Mobile Safari/530.17*/
const Device UA2020275 = {S45297, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; es-es; STM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2020338 = {S45298, 135, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; ME511 Build/RuthGC_0.18.4) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2020552 = {S45299, 153, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.3.12; es-es; GASEIM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2020961 = {S45300, 150, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.1; es-es; ARGM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2021445 = {S45301, 148, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.2.1; de-de; BRM7U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2021503 = {S45302, 150, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.6; en-za; NUM7UGW Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2021885 = {S45303, 152, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; pt-br; VOXM710UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2021964 = {S45304, 148, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; IBAK-GPS701 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2022808 = {S45305, 158, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; HS-E89/E329.6.02.00.00; 240*320; CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/3.1.2 Mobile Safari/530.17*/
const Device UA2022898 = {S45306, 151, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; IBAK-GPS702DTV Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2023107 = {S45307, 151, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.13; pt-pt; ZIPM707 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2023149 = {S45308, 146, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; pl-pl; K30 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2024089 = {S45309, 129, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; OMS1_6 Build/ECLAIR) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2024360 = {S45310, 146, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Rstream-A1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2028299 = {S45311, 150, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.2; es-es; ARGM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2031965 = {S45312, 150, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.1; en-za; NUM7UGW Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2290580 = {S45313, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.7; es-es; STM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2311563 = {S45314, 148, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; es-es; TL-7A Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2416472 = {S45315, 100, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Android 2.1; en-us) AppleWebKit/1.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2475516 = {S45316, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; nl-nl; EB0700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2518713 = {S45317, 150, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; ru-ru; RBK-490 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2641172 = {S45318, 149, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.2.0; es-es; ARGM7U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2659144 = {S45319, 153, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; PM1152 Tabula (V9) Build/ERE27) AppleWebKit/530.17 (KHTML like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2752801 = {S45320, 128, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; ru-ru; DV Build/Donut) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2808007 = {S45321, 143, { &P15364,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; VW1 Build/VW1_R14_X) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2419975 = {S45322, 144, { &P19843,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZTE BLADE III Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2456649 = {S45323, 144, { &P19843,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hr-hr; Vip Racer III Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2466957 = {S45324, 144, { &P19843,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hr-hr; ZTE Blade III Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2482690 = {S45325, 140, { &P19843,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; BLADE III Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2529197 = {S45326, 140, { &P19843,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Blade III Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2554429 = {S45327, 143, { &P19843,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; BLADE III_IL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2762841 = {S45328, 66, { &P19843,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ZTE BLADE III Build/IMM76D)*/
const Device UA2889466 = {S45329, 62, { &P19843,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Blade III Build/IMM76D)*/
const Device UA108404 = {S45330, 35, { &P15364,&P18110,&P18405,&P18092 }};	/*Useragent - Opera/9.23 (X11; Linux i686; U; en)*/
const Device UA169572 = {S45331, 35, { &P15364,&P18110,&P18405,&P18092 }};	/*Useragent - Opera/9.20 (X11; Linux i686; U; en)*/
const Device UA196353 = {S45332, 35, { &P15364,&P18110,&P18405,&P18092 }};	/*Useragent - Opera/9.24 (X11; Linux i686; U; en)*/
const Device UA197083 = {S45333, 35, { &P15364,&P18110,&P18405,&P18092 }};	/*Useragent - Opera/9.25 (X11; Linux i686; U; en)*/
const Device UA197231 = {S45334, 35, { &P15364,&P18110,&P18405,&P18092 }};	/*Useragent - Opera/9.21 (X11; Linux i686; U; en)*/
const Device UA209847 = {S45335, 35, { &P15364,&P18110,&P18405,&P18092 }};	/*Useragent - Opera/9.27 (X11; Linux i686; U; ru)*/
const Device UA236702 = {S45336, 35, { &P15364,&P18110,&P18405,&P18092 }};	/*Useragent - Opera/9.26 (X11; Linux i686; U; ru)*/
const Device UA241680 = {S45337, 38, { &P15364,&P18110,&P18405,&P18092 }};	/*Useragent - Opera/9.22 (X11; Linux i686; U; pt-br)*/
const Device UA415679 = {S45338, 66, { &P15364,&P18110,&P18405,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux i686; en) Opera 9.27*/
const Device UA625715 = {S45339, 66, { &P15364,&P18110,&P18405,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux i686; en) Opera 9.22*/
const Device UA2757830 = {S45340, 79, { &P15364,&P18110,&P18405,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; U; en; rv:1.8.0) Gecko/X Firefox/1.5.0 Opera 9.23*/
const Device UA1956325 = {S45341, 137, { &P20945,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; M660 Build/HuaweiM660) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2402419 = {S45342, 61, { &P20945,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; M660 Build/HuaweiM660)*/
const Device UA14235 = {S45343, 53, { &P13804,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-S1310-A/1004 UP.Browser/7.2.6.1.840 (GUI) MMP/2.0*/
const Device UA44202 = {S45344, 53, { &P13804,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-S1310-A/1003 UP.Browser/7.2.6.1.840 (GUI) MMP/2.0*/
const Device UA44203 = {S45345, 53, { &P13804,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-S1310-A/1002 UP.Browser/7.2.6.1.840 (GUI) MMP/2.0*/
const Device UA56598 = {S45346, 53, { &P13804,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-S1310-A/1103 UP.Browser/7.2.6.1.840 (GUI) MMP/2.0*/
const Device UA627157 = {S45347, 53, { &P13804,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-S1310-A/1105 UP.Browser/7.2.6.1.840 (GUI) MMP/2.0*/
const Device UA2559655 = {S45348, 53, { &P13804,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-S1310-A/1110 UP.Browser/7.2.6.1.840 (GUI) MMP/2.0*/
const Device UA876531 = {S45349, 102, { &P18076,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; SGH-i667)*/
const Device UA2492500 = {S45350, 114, { &P15786,&P5043,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-05/08.69; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2531404 = {S45351, 116, { &P15786,&P5043,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-05.1/08.69; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA342177 = {S45352, 133, { &P17482,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; ARCHOS 80G9 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA643080 = {S45353, 135, { &P17482,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; ARCHOS 80G9 Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1917727 = {S45354, 64, { &P17482,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; ARCHOS 80G9 Build/HTK75D)*/
const Device UA2023342 = {S45355, 62, { &P17482,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; ARCHOS 80G9 Build/HTJ85B)*/
const Device UA800951 = {S45356, 129, { &P18704,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AT200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2376960 = {S45357, 58, { &P18704,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; AT200 Build/IML74K)*/
const Device UA2025990 = {S45358, 132, { &P12889,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2027924 = {S45359, 132, { &P12889,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2502258 = {S45360, 61, { &P12889,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7300 Build/IMM76D)*/
const Device UA2556017 = {S45361, 150, { &P12889,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P7300-ORANGE/P7300BVLP7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2608109 = {S45362, 132, { &P12889,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P7300 Build/HTJ85B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2608755 = {S45363, 132, { &P12889,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P7300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2609293 = {S45364, 118, { &P12889,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/1.1.2 (Linux; U; Android 4.0.4; en-US; GT-P7300 Build/cm_p5-userdebug 4.0.4 IMM76L eng.kalltkaffe.X.X test-keys)*/
const Device UA2616160 = {S45365, 73, { &P12889,&P18066,&P18062,&P18092 }};	/*Useragent - Vyrso/2.1.2 (Linux; U; Android 4.0.4; en-US; GT-P7300 Build/IMM76D.ZSLP7)*/
const Device UA2616242 = {S45366, 130, { &P12889,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7300 Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2616853 = {S45367, 145, { &P12889,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2616968 = {S45368, 139, { &P12889,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2618884 = {S45369, 150, { &P12889,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P7300-ORANGE/P7300BVLP4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2621971 = {S45370, 93, { &P12889,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.youtube/4.0.23(Linux; U; Android 4.0.4; en_US; GT-P7300 Build/IMM76D) gzip*/
const Device UA2622345 = {S45371, 130, { &P12889,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7300 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2622469 = {S45372, 130, { &P12889,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P7300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2625140 = {S45373, 75, { &P12889,&P18066,&P18062,&P18092 }};	/*Useragent - Vyrso/2.2.8.1 (Linux; U; Android 4.0.4; in-ID; GT-P7300 Build/IMM76D.UBLP4)*/
const Device UA2626325 = {S45374, 74, { &P12889,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.10 (Linux; U; Android 4.0.4; en-PH; GT-P7300 Build/IMM76D.XXLQ6)*/
const Device UA2628419 = {S45375, 133, { &P12889,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; GT-P7300B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2629142 = {S45376, 144, { &P12889,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; in-id; GT-P7300 Build/IMM76D) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2631575 = {S45377, 62, { &P12889,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7300B Build/IMM76D)*/
const Device UA2994537 = {S45378, 151, { &P12889,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; SAMSUNG GT-P7300/P7300XXLQD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2976687 = {S45379, 128, { &P18639,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus 10 Build/JWR66V) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA94491 = {S45380, 20, { &P18254,&P17017,&P17470,&P18092 }};	/*Useragent - Xenu Link Sleuth 1.3*/
const Device UA96312 = {S45381, 21, { &P18254,&P17017,&P17470,&P18092 }};	/*Useragent - Xenu Link Sleuth 1.2i*/
const Device UA100532 = {S45382, 22, { &P18254,&P17017,&P17470,&P18092 }};	/*Useragent - Xenu Link Sleuth 1.3.5*/
const Device UA101344 = {S45383, 21, { &P18254,&P17017,&P17470,&P18092 }};	/*Useragent - Xenu Link Sleuth 1.3c*/
const Device UA103781 = {S45384, 22, { &P18254,&P17017,&P17470,&P18092 }};	/*Useragent - Xenu Link Sleuth 1.3.4*/
const Device UA103971 = {S45385, 21, { &P18254,&P17017,&P17470,&P18092 }};	/*Useragent - Xenu Link Sleuth 1.2j*/
const Device UA123375 = {S45386, 21, { &P18254,&P17017,&P17470,&P18092 }};	/*Useragent - Xenu Link Sleuth 1.2e*/
const Device UA138054 = {S45387, 21, { &P18254,&P17017,&P17470,&P18092 }};	/*Useragent - Xenu Link Sleuth 1.2g*/
const Device UA261005 = {S45388, 23, { &P18254,&P17017,&P17470,&P18092 }};	/*Useragent - Lynx/2.6 libwww-FM/2.14*/
const Device UA265183 = {S45389, 21, { &P18254,&P17017,&P17470,&P18092 }};	/*Useragent - Xenu Link Sleuth 1.2d*/
const Device UA302021 = {S45390, 21, { &P18254,&P17017,&P17470,&P18092 }};	/*Useragent - Xenu Link Sleuth 1.2f*/
const Device UA529290 = {S45391, 55, { &P18254,&P17017,&P17470,&P18092 }};	/*Useragent - BinGet/1.00.A (http://www.bin-co.com/php/scripts/load/)*/
const Device UA901715 = {S45392, 72, { &P18254,&P17017,&P17470,&P18092 }};	/*Useragent - Mechanize/2.5.1 Ruby/1.9.2p290 (http://github.com/tenderlove/mechanize/)*/
const Device UA1966440 = {S45393, 37, { &P18254,&P17017,&P17470,&P18092 }};	/*Useragent - Facebook Update/1.2.205.0;winhttp;cup*/
const Device UA2669347 = {S45394, 75, { &P18254,&P17017,&P17470,&P18092 }};	/*Useragent - Mechanize/2.6.0 Ruby/1.9.3p392 (http://github.com/sparklemotion/mechanize/)*/
const Device UA2886399 = {S45395, 73, { &P16329,&P23592,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PLAYSTATION 3 4.46) AppleWebKit/531.22.8 (KHTML, like Gecko)*/
const Device UA3064770 = {S45396, 130, { &P23773,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8010 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3064977 = {S45397, 130, { &P23773,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8010 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2510984 = {S45398, 82, { &P15364,&P5690,&P20456,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:18.0) Gecko/X Firefox/18.0 SeaMonkey/2.15.2*/
const Device UA2515446 = {S45399, 75, { &P15364,&P5690,&P20456,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:18.0) Gecko/X Firefox/18.0 SeaMonkey/2.15.2*/
const Device UA2516400 = {S45400, 62, { &P15364,&P5690,&P20456,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:18.0) Gecko/X SeaMonkey/2.15.2*/
const Device UA2708682 = {S45401, 137, { &P22597,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2756885 = {S45402, 130, { &P22597,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2261564 = {S45403, 132, { &P22037,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3113 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2382139 = {S45404, 132, { &P22037,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3113 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA625810 = {S45405, 80, { &P16518,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-E2222/E2222JVKJ3 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2157076 = {S45406, 74, { &P16518,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-E2222L/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2489089 = {S45407, 72, { &P12280,&P19826,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10B142 (iPhone; U; CPU OS 6_1 like Mac OS X; en_us)*/
const Device UA2511934 = {S45408, 74, { &P12280,&P19826,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10B145 (iPhone; U; CPU OS 6_1_1 like Mac OS X; en_us)*/
const Device UA2532769 = {S45409, 74, { &P12280,&P19826,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10B146 (iPhone; U; CPU OS 6_1_2 like Mac OS X; en_us)*/
const Device UA2585466 = {S45410, 74, { &P12280,&P19826,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10B329 (iPhone; U; CPU OS 6_1_3 like Mac OS X; en_us)*/
const Device UA2710743 = {S45411, 74, { &P12280,&P19826,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10B350 (iPhone; U; CPU OS 6_1_4 like Mac OS X; en_us)*/
const Device UA2566449 = {S45412, 141, { &P22394,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6603 Build/10.1.A.1.350) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567501 = {S45413, 134, { &P22394,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6603 Build/10.1.A.1.350) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA527229 = {S45414, 84, { &P17595,&P18120,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; fr; rv:1.8.1.16) Gecko/X Firefox/2.0.0.16*/
const Device UA642005 = {S45415, 88, { &P17595,&P18120,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.8.1a2) Gecko/X BonEcho/2.0a2*/
const Device UA889673 = {S45416, 84, { &P17595,&P18120,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.8) Gecko/X Firefox/2.0a1*/
const Device UA852194 = {S45417, 76, { &P17595,&P18207,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:5.0.1) Gecko/X Firefox/5.0.1*/
const Device UA2202612 = {S45418, 112, { &P18219,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia202/20.28; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2230170 = {S45419, 112, { &P18219,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia203/20.36; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2407078 = {S45420, 111, { &P18219,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia202/20.28; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5*/
const Device UA5758 = {S45421, 123, { &P12287,&P5843,&P4030,&P18092 }};	/*Useragent - Nokia6700c/2.0 (13.21) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA64844 = {S45422, 125, { &P12287,&P5843,&P4030,&P18092 }};	/*Useragent - Nokia6700c-1/2.0 (10.50) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA190999 = {S45423, 88, { &P15364,&P17285,&P21896,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/11.0.696.57)*/
const Device UA316148 = {S45424, 95, { &P15364,&P17285,&P21896,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/11.0.696.68)*/
const Device UA448772 = {S45425, 125, { &P15364,&P17285,&P21896,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/11.0.696.57) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.57 Safari/534.24*/
const Device UA557767 = {S45426, 162, { &P15364,&P17285,&P21896,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/11.0.696.57; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA732465 = {S45427, 173, { &P15364,&P17285,&P21896,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB6.6; chromeframe/11.0.696.65; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)*/
const Device UA1882169 = {S45428, 125, { &P15364,&P17285,&P21896,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/11.0.696.71) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24*/
const Device UA2472948 = {S45429, 201, { &P15364,&P17285,&P21896,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; FDM; .NET4.0C; .NET4.0E; chromeframe/11.0.696.57)*/
const Device UA2517416 = {S45430, 201, { &P15364,&P17285,&P21896,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; FDM; .NET4.0C; .NET4.0E; chromeframe/11.0.696.57)*/
const Device UA1995707 = {S45431, 153, { &P15877,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2445738 = {S45432, 135, { &P15877,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; XT910 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2450101 = {S45433, 147, { &P15877,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2526808 = {S45434, 144, { &P15877,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT910K Build/6.7.4-27_SA-46) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA56057 = {S45435, 128, { &P14631,&P17484,&P4732,&P18092 }};	/*Useragent - SAMSUNG-SGH-A927/A927UCJF5; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.19039; U; en-KR) Opera 9.50*/
const Device UA623375 = {S45436, 62, { &P15228,&P5043,&P4030,&P18092 }};	/*Useragent - NokiaC2-03/2.0 (07.48) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2694700 = {S45437, 73, { &P15228,&P5043,&P4030,&P18092 }};	/*Useragent - NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-03*/
const Device UA3043737 = {S45438, 125, { &P15228,&P5043,&P4030,&P18092 }};	/*Useragent - NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA3025463 = {S45439, 111, { &P15785,&P5843,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia201/11.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA2708987 = {S45440, 129, { &P18554,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BNTV400 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA5982 = {S45441, 143, { &P12338,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; T-Mobile_Espresso Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA30781 = {S45442, 146, { &P12338,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile_Espresso Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA50579 = {S45443, 145, { &P12338,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;T-Mobile_Espresso Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA50582 = {S45444, 145, { &P12338,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile_Espresso Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA50584 = {S45445, 145, { &P12338,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; T-Mobile_Espresso Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA50587 = {S45446, 145, { &P12338,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.1; en-us; T-Mobile_Espresso Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA50589 = {S45447, 145, { &P12338,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile_EspressoBuild/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2715362 = {S45448, 130, { &P19861,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-I605 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.74 Safari/537.36*/
const Device UA2741705 = {S45449, 137, { &P19861,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2756011 = {S45450, 137, { &P19861,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-I605 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892597 = {S45451, 130, { &P19861,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3025460 = {S45452, 113, { &P18267,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-02/11.79; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA2603050 = {S45453, 82, { &P18435,&P21460,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (IE 11.0; Windows NT 6.3; ARM; Trident/7.0; Touch; rv:11.0) like Gecko*/
const Device UA2708939 = {S45454, 73, { &P18435,&P21460,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.3; ARM; Trident/7.0; Touch; rv:11.0) like Gecko*/
const Device UA2751180 = {S45455, 66, { &P18435,&P21460,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.3; ARM; Trident/7.0; rv:11.0) like Gecko*/
const Device UA2232348 = {S45456, 135, { &P12605,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XT875 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261635 = {S45457, 135, { &P12605,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT875 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2668899 = {S45458, 152, { &P12605,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2716499 = {S45459, 146, { &P12605,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3065134 = {S45460, 145, { &P18645,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I747 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065383 = {S45461, 137, { &P18645,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L710 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065721 = {S45462, 136, { &P18645,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I747 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068332 = {S45463, 137, { &P18645,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9305 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083385 = {S45464, 138, { &P18645,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530U Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083757 = {S45465, 137, { &P18645,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-I747 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3112420 = {S45466, 145, { &P18645,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SAMSUNG-SGH-I747 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA215310 = {S45467, 102, { &P16237,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; SGH-i937)*/
const Device UA925782 = {S45468, 99, { &P16237,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; Haden)*/
const Device UA92452 = {S45469, 116, { &P15364,&P5690,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.29*/
const Device UA93289 = {S45470, 118, { &P15364,&P5690,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA104777 = {S45471, 116, { &P15364,&P5690,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.28 (KHTML, like Gecko) Version/3.2.2 Safari/525.28.1*/
const Device UA106247 = {S45472, 112, { &P15364,&P5690,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/531.0 (KHTML, like Gecko) Version/3.2.1 Safari/531.0*/
const Device UA136816 = {S45473, 117, { &P15364,&P5690,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.13*/
const Device UA414739 = {S45474, 116, { &P15364,&P5690,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/525.28 (KHTML, like Gecko) Version/3.2.2 Safari/525.28.1*/
const Device UA2126957 = {S45475, 114, { &P15364,&P5690,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; de-DE) AppleWebKit/528+ (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA33469 = {S45476, 191, { &P12236,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 Nokia6700s/071.004; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.26 Mobile Safari/533.4 3gpp-gba*/
const Device UA56224 = {S45477, 194, { &P12236,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 Nokia6700s-1c/071.004; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.26 Mobile Safari/533.4 3gpp-gba*/
const Device UA63849 = {S45478, 169, { &P12236,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 Nokia6700s-1c/033.014; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6*/
const Device UA64771 = {S45479, 166, { &P12236,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 Nokia6700s/032.005; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.1.6*/
const Device UA675345 = {S45480, 40, { &P12236,&P5890,&P3439,&P18092 }};	/*Useragent - (C)Nokia6700s/SymbianOS/9.1 Series60/3.0*/
const Device UA2490169 = {S45481, 57, { &P15364,&P16110,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2) Presto/2.12.388 Version/12.13*/
const Device UA2490347 = {S45482, 64, { &P15364,&P16110,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64) Presto/2.12.388 Version/12.13*/
const Device UA2510929 = {S45483, 86, { &P15364,&P16110,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6015)) Presto/2.12.388 Version/12.13*/
const Device UA2660244 = {S45484, 86, { &P15364,&P16110,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5989)) Presto/2.12.388 Version/12.13*/
const Device UA2765306 = {S45485, 86, { &P15364,&P16110,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6037)) Presto/2.12.388 Version/12.13*/
const Device UA3086449 = {S45486, 86, { &P15364,&P16110,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 6.0 (build 6005)) Presto/2.12.388 Version/12.13*/
const Device UA912738 = {S45487, 125, { &P15364,&P7133,&P22460,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11*/
const Device UA913263 = {S45488, 118, { &P15364,&P7133,&P22460,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11*/
const Device UA11000 = {S45489, 134, { &P13526,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2; en-us; myTouch4G Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA15018 = {S45490, 136, { &P13526,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; myTouch4GBuild/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA15020 = {S45491, 138, { &P13526,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; myTouch4G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA15027 = {S45492, 136, { &P13526,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; myTouch4G Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA15030 = {S45493, 136, { &P13526,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;myTouch4G Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA15034 = {S45494, 134, { &P13526,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2; en-us; myTouch4G Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA15038 = {S45495, 136, { &P13526,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; myTouch4G Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA15047 = {S45496, 136, { &P13526,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.1; en-us; myTouch4G Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA39742 = {S45497, 134, { &P13526,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; myTouch4GBuild/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA62864 = {S45498, 135, { &P13526,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; myTouch4G Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA63167 = {S45499, 137, { &P13526,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; myTouch4G Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2368546 = {S45500, 131, { &P12779,&P7106,&P15829,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9370; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.755 Mobile Safari/534.11+*/
const Device UA23811 = {S45501, 136, { &P14467,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA23812 = {S45502, 134, { &P14467,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA62985 = {S45503, 136, { &P14467,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA310643 = {S45504, 60, { &P14467,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; SPH-M920 Build/FROYO)*/
const Device UA2018586 = {S45505, 143, { &P14467,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SPH-M920 Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2566498 = {S45506, 138, { &P16356,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566684 = {S45507, 138, { &P16356,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-N7000 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566882 = {S45508, 138, { &P16356,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T879 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567570 = {S45509, 139, { &P16356,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000B Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2584084 = {S45510, 139, { &P16356,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160L Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2600348 = {S45511, 139, { &P16356,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602400 = {S45512, 139, { &P16356,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160K Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602401 = {S45513, 131, { &P16356,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2602996 = {S45514, 136, { &P16356,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-05D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2878438 = {S45515, 132, { &P16356,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M480W Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA3045934 = {S45516, 139, { &P16356,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SHV-E160S Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA3030631 = {S45517, 111, { &P18643,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia311/05.92; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA2669019 = {S45518, 75, { &P17595,&P17297,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5) Presto/2.12.388 Version/12.15*/
const Device UA1991438 = {S45519, 135, { &P12605,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT875 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2205789 = {S45520, 155, { &P12605,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2355890 = {S45521, 149, { &P12605,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2433358 = {S45522, 112, { &P19748,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia205/03.18; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.34*/
const Device UA2462449 = {S45523, 114, { &P19748,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia205.1/03.18; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.34*/
const Device UA2715589 = {S45524, 113, { &P19748,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2055/03.20; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.34*/
const Device UA2791413 = {S45525, 114, { &P19748,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia205.3/03.19; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.34*/
const Device UA2894707 = {S45526, 113, { &P19748,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2052/03.20; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.34*/
const Device UA3103181 = {S45527, 113, { &P19748,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2051/03.20; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.34*/
const Device UA2741601 = {S45528, 138, { &P18280,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754418 = {S45529, 138, { &P18280,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC VLE_U Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755279 = {S45530, 138, { &P18280,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2871783 = {S45531, 154, { &P18280,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Special Edition Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3039888 = {S45532, 132, { &P15767,&P22882,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.5 Mobile/11A4435d Safari/8536.25*/
const Device UA3082794 = {S45533, 131, { &P15767,&P22882,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A465 Safari/8536.25*/
const Device UA3082986 = {S45534, 133, { &P15767,&P22882,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A4449d Safari/8536.25*/
const Device UA3095934 = {S45535, 133, { &P15767,&P22882,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A4449a Safari/8536.25*/
const Device UA3115378 = {S45536, 133, { &P15767,&P22882,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A501 Safari/8536.25*/
const Device UA2753008 = {S45537, 137, { &P16362,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754259 = {S45538, 141, { &P16362,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710VMUB Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754949 = {S45539, 140, { &P16362,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710BST Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867500 = {S45540, 137, { &P16362,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SPH-D710 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2044000 = {S45541, 140, { &P18564,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X+ Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2382935 = {S45542, 142, { &P18564,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC EVARE_UL Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA625258 = {S45543, 135, { &P20936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Venture Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029457 = {S45544, 59, { &P20936,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Venture Build/GRJ90)*/
const Device UA250285 = {S45545, 135, { &P15364,&P17819,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/11.10 Chromium/14.0.835.202 Chrome/14.0.835.202 Safari/535.1*/
const Device UA250438 = {S45546, 137, { &P15364,&P17819,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/11.10 Chromium/14.0.835.202 Chrome/14.0.835.202 Safari/535.1*/
const Device UA881073 = {S45547, 172, { &P15364,&P17819,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/11.10 Chromium/14.0.835.202 Chrome/14.0.835.202 Safari/535.1,gzip(gfe) (via translate.google.com)*/
const Device UA2602773 = {S45548, 152, { &P19768,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ALCATEL ONE TOUCH 4010X Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2753409 = {S45549, 152, { &P19768,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; ALCATEL ONE TOUCH 4010A Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2781926 = {S45550, 152, { &P19768,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; ALCATEL ONE TOUCH 4010D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3015655 = {S45551, 152, { &P19768,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; ALCATEL ONE TOUCH 4010E Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2236373 = {S45552, 140, { &P19848,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; SHW-M440S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423761 = {S45553, 140, { &P19848,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHW-M440S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2656471 = {S45554, 62, { &P19848,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHW-M440S Build/JZO54K)*/
const Device UA2950571 = {S45555, 62, { &P19848,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SHW-M440S Build/JRO03C)*/
const Device UA2426725 = {S45556, 132, { &P20035,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I925 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555112 = {S45557, 144, { &P20035,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I925U Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 USCC-I925U*/
const Device UA2652501 = {S45558, 132, { &P20035,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I925 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2751333 = {S45559, 61, { &P20035,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-I925 Build/JZO54K)*/
const Device UA2948496 = {S45560, 80, { &P15364,&P7133,&P24330,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:23.0) Gecko/X Firefox/23.0 SeaMonkey/2.20*/
const Device UA3057981 = {S45561, 67, { &P15364,&P7133,&P24330,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:23.0) Gecko/X SeaMonkey/2.20*/
const Device UA2973475 = {S45562, 139, { &P16292,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Galaxy Nexus Build/JWR66V) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2600653 = {S45563, 133, { &P18680,&P18110,&P21708,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Safari/535.19 Silk-Accelerated=true*/
const Device UA100058 = {S45564, 107, { &P15364,&P18110,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/532.6 (KHTML, like Gecko) Chrome/4.0.266.0 Safari/532.6*/
const Device UA719507 = {S45565, 109, { &P15364,&P18110,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; fr-FR) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/4.0.202.0 Safari/532.0*/
const Device UA829427 = {S45566, 116, { &P15364,&P18110,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US) AppleWebKit/532.2 (KHTML, like Gecko) Chrome/4.0.221.8 Safari/532.2*/
const Device UA2131415 = {S45567, 109, { &P15364,&P18110,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/4.0.202.0 Safari/532.0*/
const Device UA2540801 = {S45568, 120, { &P15364,&P18110,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Slackware Linux x86_64; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5*/
const Device UA2391601 = {S45569, 132, { &P20011,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Next7P12 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2427968 = {S45570, 135, { &P20011,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Next7P12-8G Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2453184 = {S45571, 132, { &P20011,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Next7P12 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA8602 = {S45572, 138, { &P17214,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; ViewPad 7a Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52402 = {S45573, 137, { &P17214,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; ViewPad7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55240 = {S45574, 136, { &P17214,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; ViewPad7 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA64315 = {S45575, 134, { &P17214,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; ViewPad7 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90469 = {S45576, 141, { &P17214,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ru-; ViewPad7 Build/FRG83) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA534815 = {S45577, 58, { &P17214,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; ViewPad7 Build/FRF91)*/
const Device UA561282 = {S45578, 61, { &P17214,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; ViewPad7 Build/FRG83G)*/
const Device UA2462396 = {S45579, 140, { &P19763,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C6602 Build/10.1.A.1.78) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2604380 = {S45580, 141, { &P19763,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6602 Build/10.1.A.1.434) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2716914 = {S45581, 142, { &P19763,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-02E Build/10.1.D.0.343) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA891746 = {S45582, 158, { &P13999,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NexusHD2 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025360 = {S45583, 162, { &P13999,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC HD2 Build/IMM76L; CyanogenMod-BlueOrange) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027660 = {S45584, 153, { &P13999,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC HD2 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2200012 = {S45585, 160, { &P13999,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NexusHD2 Build/IMM76L; CyanogenMod-CM9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2502203 = {S45586, 173, { &P13999,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC HD2 Build/IMM76L; CyanogenMod-huhjun iCyanogen v1.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2788890 = {S45587, 169, { &P13999,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC HD2 Build/IMM76D; CyanogenMod-Cm9ight_leo_v1.1a) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044442 = {S45588, 159, { &P13999,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; HD2 Build/IMM76I; CyanogenMod-9.1.0 Final) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2945377 = {S45589, 135, { &P18665,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; N861 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2947987 = {S45590, 57, { &P18665,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; N861 Build/JZO54K)*/
const Device UA2566501 = {S45591, 139, { &P18280,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566763 = {S45592, 132, { &P18280,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567560 = {S45593, 139, { &P18280,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667059 = {S45594, 155, { &P18280,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Special Edition Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2706286 = {S45595, 139, { &P18280,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC VLE_U Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA250612 = {S45596, 137, { &P15364,&P17816,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/10.04 Chromium/14.0.835.202 Chrome/14.0.835.202 Safari/535.1*/
const Device UA345661 = {S45597, 131, { &P15364,&P17816,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/10.04 Chromium/14.0.813.0 Chrome/14.0.813.0 Safari/535.1*/
const Device UA2559703 = {S45598, 139, { &P18507,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8160 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2732936 = {S45599, 61, { &P18507,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8160 Build/JZO54K)*/
const Device UA2768595 = {S45600, 140, { &P18507,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; GT-I8160P Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2939578 = {S45601, 140, { &P18507,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; GT-I8160L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088464 = {S45602, 62, { &P18507,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8160P Build/JZO54K)*/
const Device UA3106594 = {S45603, 62, { &P18507,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8160L Build/JZO54K)*/
const Device UA2974487 = {S45604, 144, { &P18640,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA196489 = {S45605, 140, { &P17963,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; NXM726 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA349257 = {S45606, 142, { &P17963,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; NXM726HN Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA650148 = {S45607, 151, { &P17963,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; NXM726HN_HASTINGS Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1944050 = {S45608, 66, { &P17963,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.1; NXM726HN Build/GINGERBREAD)*/
const Device UA2813211 = {S45609, 140, { &P17963,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-; NXM726HN Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA151676 = {S45610, 131, { &P17942,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Comet Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA190355 = {S45611, 55, { &P17942,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; Comet Build/FRF91)*/
const Device UA400929 = {S45612, 130, { &P17942,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2; en-us; Comet Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA434166 = {S45613, 130, { &P17942,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us;Comet Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA435084 = {S45614, 130, { &P17942,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Comet Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA448260 = {S45615, 130, { &P17942,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2; en-us; Comet Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA635996 = {S45616, 178, { &P12588,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.3 NokiaC6-01; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.4.2.6 Mobile Safari/533.4 3gpp-gba*/
const Device UA2017658 = {S45617, 191, { &P12588,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.3 NokiaC6-01/111.040.1511; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.1.4 Mobile Safari/535.1 3gpp-gba*/
const Device UA16687 = {S45618, 67, { &P14557,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia6303c_CMCC/2.0 (08.90) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63899 = {S45619, 123, { &P14557,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia6303c/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA63925 = {S45620, 129, { &P14557,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia6303classic/2.0 (09.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA69638 = {S45621, 85, { &P14557,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303classic*/
const Device UA3130738 = {S45622, 138, { &P18283,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131174 = {S45623, 137, { &P18283,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131333 = {S45624, 137, { &P18283,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131355 = {S45625, 137, { &P18283,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131404 = {S45626, 137, { &P18283,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132613 = {S45627, 138, { &P18283,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132860 = {S45628, 138, { &P18283,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA455244 = {S45629, 107, { &P17595,&P17297,&P22505,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en; rv:1.9.2.28) Gecko/X Camino/2.1.2 (like Firefox/3.6.28)*/
const Device UA2238809 = {S45630, 97, { &P17595,&P17295,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:16.0) Gecko/X Firefox/16.0 AlexaToolbar/alxf-2.17*/
const Device UA54028 = {S45631, 99, { &P17595,&P17296,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.0.19) Gecko/2010031218 Firefox/3.0.19*/
const Device UA552805 = {S45632, 84, { &P17595,&P17296,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6.1; en-US; rv:1.9) Gecko/X Firefox/3.0*/
const Device UA366626 = {S45633, 51, { &P18254,&P7480,&P18245,&P18155 }};	/*Useragent - SkimBot/1.0 (www.skimlinks.com <dev@skimlinks.com>)*/
const Device UA2451698 = {S45634, 138, { &P19920,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS870 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2607518 = {S45635, 164, { &P21793,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9205 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3113453 = {S45636, 177, { &P21793,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; fr-fr; SAMSUNG GT-I9205/I9205XXUAMG7 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3125983 = {S45637, 177, { &P21793,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; fr-fr; SAMSUNG GT-I9205/I9205XXUAMG4 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA106500 = {S45638, 118, { &P15364,&P7133,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/1.0.154.36 Safari/525.19*/
const Device UA539051 = {S45639, 116, { &P15364,&P7133,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/1, 6, 0, 6 Safari/533.3*/
const Device UA539710 = {S45640, 116, { &P15364,&P7133,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/1, 6, 0, 4 Safari/533.3*/
const Device UA661660 = {S45641, 104, { &P15364,&P7133,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.3 (KHTML, like Gecko) Chrome/1, 6, 0, 4 Safari/535.3*/
const Device UA760547 = {S45642, 116, { &P15364,&P7133,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/1, 6, 0, 3 Safari/533.3*/
const Device UA2144088 = {S45643, 116, { &P15364,&P7133,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/1, 6, 2, 0 Safari/533.3*/
const Device UA2164465 = {S45644, 97, { &P15364,&P7133,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.3 (KHTML, like Gecko) Chrome/1, 6, 0, 4 Safari/535.3*/
const Device UA2805160 = {S45645, 116, { &P15364,&P7133,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/1, 6, 1, 0 Safari/533.3*/
const Device UA2974046 = {S45646, 137, { &P12972,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974598 = {S45647, 138, { &P12972,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989D Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975062 = {S45648, 138, { &P12972,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975750 = {S45649, 137, { &P12972,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3025627 = {S45650, 145, { &P12972,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-T989 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA7035 = {S45651, 138, { &P14604,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 1.5; en-us; MB200 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA7036 = {S45652, 138, { &P14604,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB200Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA7095 = {S45653, 144, { &P14604,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB200-demo Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA7102 = {S45654, 138, { &P14604,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us;MB200 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA7108 = {S45655, 138, { &P14604,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android1.5; en-us; MB200 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA27622 = {S45656, 137, { &P14604,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-; MB200 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA54945 = {S45657, 139, { &P14604,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB200 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA88941 = {S45658, 140, { &P14604,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB200 Build/CUPCAKE) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2018673 = {S45659, 120, { &P14604,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB200 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mob*/
const Device UA2669999 = {S45660, 145, { &P18452,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT29i Build/9.1.B.0.411) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2675219 = {S45661, 141, { &P18452,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT29i Build/9.1.B.0.411) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2869764 = {S45662, 62, { &P18452,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LT29i Build/9.1.B.1.67)*/
const Device UA2871781 = {S45663, 144, { &P18452,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT29i Build/9.1.B.1.67) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2877600 = {S45664, 140, { &P18452,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT29i Build/9.1.B.1.67) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034099 = {S45665, 132, { &P14905,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2397634 = {S45666, 61, { &P14905,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7100 Build/IMM76D)*/
const Device UA2608278 = {S45667, 132, { &P14905,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P7100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2609069 = {S45668, 132, { &P14905,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P7100 Build/HTJ85B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2614655 = {S45669, 130, { &P14905,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-; GT-P7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615866 = {S45670, 145, { &P14905,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA2620919 = {S45671, 74, { &P14905,&P18066,&P18062,&P18092 }};	/*Useragent - Biblia/2.2.7 (Linux; U; Android 4.0.4; es-MX; GT-P7100 Build/IMM76D.BULG3)*/
const Device UA2621120 = {S45672, 132, { &P14905,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7100 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2626034 = {S45673, 139, { &P14905,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-P7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2629129 = {S45674, 132, { &P14905,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; GT-P7100 Build/HTJ85B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2362366 = {S45675, 101, { &P18444,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 820)*/
const Device UA2423711 = {S45676, 109, { &P18444,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 820; ANZ825)*/
const Device UA2467071 = {S45677, 111, { &P18444,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 820; Vodafone)*/
const Device UA2656203 = {S45678, 109, { &P18444,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 820; Orange)*/
const Device UA2984884 = {S45679, 101, { &P18444,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Nokia 820)*/
const Device UA3067001 = {S45680, 109, { &P18444,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 820; 1328-3)*/
const Device UA3030770 = {S45681, 113, { &P15228,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-03/07.48; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA85159 = {S45682, 46, { &P16333,&P18180,&P17652,&P18092 }};	/*Useragent - Mozilla/4.0 (PSP (PlayStation Portable); 2.00)*/
const Device UA2566625 = {S45683, 151, { &P18505,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567335 = {S45684, 144, { &P18505,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583212 = {S45685, 150, { &P18505,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MB886 Build/9.8.2Q-8_MB886_FFW-11) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA7244 = {S45686, 139, { &P11702,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; zh-cn; ME600 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA7252 = {S45687, 182, { &P11702,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-ca; MB300 Build/Blur_Version.0.13.1145.MB300.TELUS.en.CA Flex/P009) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA7254 = {S45688, 182, { &P11702,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-ca; MB300 Build/Blur_Version.0.13.1129.MB300.TELUS.en.CA Flex/P008) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA25476 = {S45689, 180, { &P11702,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB300 Build/Blur_Version.0.12.1151.MB300.ATT.en.US Flex/P012) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA25480 = {S45690, 180, { &P11702,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB300 Build/Blur_Version.0.12.1199.MB300.ATT.en.US Flex/P014) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA25483 = {S45691, 178, { &P11702,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB300 Build/Blur_Version.0.13.37.MB300.ATT.en.US Flex/P014) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA25484 = {S45692, 178, { &P11702,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB300 Build/Blur_Version.0.13.35.MB300.ATT.en.US Flex/P014) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA25486 = {S45693, 180, { &P11702,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB300 Build/Blur_Version.0.12.1214.MB300.ATT.en.US Flex/P014) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA63002 = {S45694, 139, { &P11702,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; MB300 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2485469 = {S45695, 128, { &P12280,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293*/
const Device UA2503987 = {S45696, 141, { &P12280,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2514722 = {S45697, 145, { &P12280,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA2535910 = {S45698, 130, { &P12280,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293*/
const Device UA2562022 = {S45699, 143, { &P12280,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2564672 = {S45700, 147, { &P12280,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA2570610 = {S45701, 141, { &P12280,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/6531.22.7 iLunascape/3203*/
const Device UA2581665 = {S45702, 130, { &P12280,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293*/
const Device UA2606819 = {S45703, 130, { &P12280,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293*/
const Device UA2607445 = {S45704, 143, { &P12280,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2670334 = {S45705, 147, { &P12280,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA2675606 = {S45706, 141, { &P12280,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/6531.22.7 iLunascape/3205*/
const Device UA2700772 = {S45707, 143, { &P12280,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2701549 = {S45708, 141, { &P12280,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_2 like Mac OS X) SparqEye/1.2.2 AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A293 Safari/6531.22.7*/
const Device UA2712928 = {S45709, 143, { &P12280,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_4 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2738281 = {S45710, 130, { &P12280,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_4 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293*/
const Device UA2750477 = {S45711, 141, { &P12280,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X) SparqEye/1.2.2 AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A293 Safari/6531.22.7*/
const Device UA2752520 = {S45712, 147, { &P12280,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_4 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA2767140 = {S45713, 148, { &P12280,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/10B329 Safari/6531.22.7*/
const Device UA2937173 = {S45714, 146, { &P12280,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/10B146 Safari/6531.22.7*/
const Device UA2962871 = {S45715, 141, { &P12280,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/6531.22.7 iLunascape/3212*/
const Device UA2997308 = {S45716, 141, { &P12280,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_4 like Mac OS X) SparqEye/1.2.2 AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A293 Safari/6531.22.7*/
const Device UA2257600 = {S45717, 144, { &P19862,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2480877 = {S45718, 66, { &P19862,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Micromax A110 Build/IMM76D)*/
const Device UA2790281 = {S45719, 142, { &P19862,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; Micromax A110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2843325 = {S45720, 130, { &P19862,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A110 Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2669850 = {S45721, 137, { &P21261,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2714033 = {S45722, 130, { &P21261,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA924913 = {S45723, 137, { &P14574,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus S Build/JRO03E) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1985907 = {S45724, 135, { &P14574,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; Nexus S Build/JRN83C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1994861 = {S45725, 152, { &P14574,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Full Android on Crespo Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1996303 = {S45726, 138, { &P14574,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1;  Nexus S Build/JRO03E) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA2233789 = {S45727, 137, { &P14574,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2367346 = {S45728, 138, { &P14574,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2;  Nexus S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA627373 = {S45729, 138, { &P18480,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; N850 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032693 = {S45730, 62, { &P18480,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; N850 Build/GINGERBREAD)*/
const Device UA2704394 = {S45731, 133, { &P18680,&P18110,&P22588,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Safari/535.19 Silk-Accelerated=true*/
const Device UA2257363 = {S45732, 144, { &P18008,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; one touch T10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2401281 = {S45733, 128, { &P18008,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; CT10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2457321 = {S45734, 127, { &P18008,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2594340 = {S45735, 66, { &P18008,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; one touch T10 Build/IML74K)*/
const Device UA2652703 = {S45736, 131, { &P18008,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; EVOAT10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2867509 = {S45737, 135, { &P18008,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HYUNDAI T10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2566500 = {S45738, 131, { &P15876,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P5100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566540 = {S45739, 131, { &P15876,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P5100 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567023 = {S45740, 131, { &P15876,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T779 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA668278 = {S45741, 56, { &P15364,&P16110,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; rv:2.0) Gecko/X Firefox/4.0*/
const Device UA1904934 = {S45742, 80, { &P15364,&P16110,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:2.0b12pre) Gecko/X Firefox/4.0b12pre*/
const Device UA2296664 = {S45743, 86, { &P15364,&P16110,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; WOW64; en-US; rv:2.0b1) Gecko/X Firefox/4.0b1*/
const Device UA2225028 = {S45744, 121, { &P15364,&P16110,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 AvantBrowser/Tri-Core*/
const Device UA2225165 = {S45745, 128, { &P15364,&P16110,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 AvantBrowser/Tri-Core*/
const Device UA2362129 = {S45746, 121, { &P15364,&P16110,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; Avant TriCore) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.96 Safari/537.4*/
const Device UA2367782 = {S45747, 114, { &P15364,&P16110,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Avant TriCore) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.96 Safari/537.4*/
const Device UA2433551 = {S45748, 123, { &P15364,&P16110,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; Avant TriCore) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2462074 = {S45749, 116, { &P15364,&P16110,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Avant TriCore) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2479294 = {S45750, 121, { &P15364,&P16110,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; Avant TriCore) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2530842 = {S45751, 123, { &P15364,&P16110,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; Avant TriCore) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2606859 = {S45752, 124, { &P15364,&P16110,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; Avant TriCore) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2670483 = {S45753, 117, { &P15364,&P16110,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Avant TriCore) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2712550 = {S45754, 116, { &P15364,&P16110,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Avant TriCore) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2718659 = {S45755, 123, { &P15364,&P16110,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; Avant TriCore) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2877522 = {S45756, 124, { &P15364,&P16110,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36*/
const Device UA2887636 = {S45757, 117, { &P15364,&P16110,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36*/
const Device UA2945815 = {S45758, 123, { &P15364,&P16110,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36*/
const Device UA2971266 = {S45759, 116, { &P15364,&P16110,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36*/
const Device UA3114758 = {S45760, 123, { &P15364,&P16110,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36*/
const Device UA3115275 = {S45761, 116, { &P15364,&P16110,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36*/
const Device UA3065326 = {S45762, 134, { &P20705,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; HTC One Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065589 = {S45763, 134, { &P20705,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; One Build/JWR66V.H10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3112821 = {S45764, 139, { &P20705,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; HTC One 801e Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126320 = {S45765, 165, { &P20705,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; HTC One with MoDaCo.SWITCH Beta 13 Build/JWR66V.H10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127848 = {S45766, 133, { &P20705,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; HTCONE Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2012813 = {S45767, 156, { &P12280,&P18432,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A5376e Safari/7534.48.3*/
const Device UA2040925 = {S45768, 154, { &P12280,&P18432,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A403 Safari/7534.48.3*/
const Device UA2203816 = {S45769, 154, { &P12280,&P18432,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A405 Safari/7534.48.3*/
const Device UA2261395 = {S45770, 156, { &P12280,&P18432,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A523 Safari/7534.48.3*/
const Device UA2386107 = {S45771, 156, { &P12280,&P18432,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A525 Safari/7534.48.3*/
const Device UA2486298 = {S45772, 156, { &P12280,&P18432,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A551 Safari/7534.48.3*/
const Device UA2201118 = {S45773, 82, { &P17595,&P17297,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5; U; en) Presto/2.10.289 Version/12.02*/
const Device UA84805 = {S45774, 121, { &P17346,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/3A100a Safari/419.3*/
const Device UA86999 = {S45775, 120, { &P17346,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/4A102 Safari/419.3*/
const Device UA88317 = {S45776, 118, { &P17346,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/4B1 Safari/419.3*/
const Device UA106792 = {S45777, 119, { &P17346,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/4A93 Safari/419.3*/
const Device UA111531 = {S45778, 120, { &P17346,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/3B48b Safari/419.3*/
const Device UA274525 = {S45779, 121, { &P17346,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/3A110a Safari/419.3*/
const Device UA2849645 = {S45780, 120, { &P17346,&P17779,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML like Gecko) Version/3.0 Mobile/3A101a Safari/419.3*/
const Device UA690290 = {S45781, 132, { &P18631,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PMP5080B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA893306 = {S45782, 135, { &P18631,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PMP5080CPRO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2443379 = {S45783, 64, { &P18631,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; PMP5080CPRO Build/IML74K)*/
const Device UA2741740 = {S45784, 130, { &P18437,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2753059 = {S45785, 130, { &P18437,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2753348 = {S45786, 130, { &P18437,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8005 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2753621 = {S45787, 130, { &P18437,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8020 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754491 = {S45788, 130, { &P18437,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8020 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2375870 = {S45789, 143, { &P20227,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LG-L40G/V100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2413374 = {S45790, 138, { &P20227,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LG-L40G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654290 = {S45791, 60, { &P20227,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-L40G Build/IML74K)*/
const Device UA2677408 = {S45792, 142, { &P12280,&P19826,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 UCBrowser/9.0.1.284 Mobile*/
const Device UA2679631 = {S45793, 142, { &P12280,&P19826,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 UCBrowser/9.0.1.284 Mobile*/
const Device UA2712911 = {S45794, 142, { &P12280,&P19826,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 UCBrowser/9.0.1.284 Mobile*/
const Device UA2713771 = {S45795, 140, { &P12280,&P19826,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 UCBrowser/9.0.1.284 Mobile*/
const Device UA2715414 = {S45796, 140, { &P12280,&P19826,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 UCBrowser/9.0.1.284 Mobile*/
const Device UA2715742 = {S45797, 140, { &P12280,&P19826,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 UCBrowser/9.0.1.284 Mobile*/
const Device UA2736298 = {S45798, 142, { &P12280,&P19826,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145 UCBrowser/9.0.1.284 Mobile*/
const Device UA2795768 = {S45799, 140, { &P12280,&P19826,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; en-US) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 UCBrowser/9.0.0.260 Mobile*/
const Device UA2995738 = {S45800, 172, { &P12280,&P19826,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X; en-US) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/10B329 Safari/7534.48.3 UCBrowser/9.0.0.260*/
const Device UA819730 = {S45801, 107, { &P17595,&P18207,&P22505,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en; rv:1.9.2.28) Gecko/X Camino/2.1.2 (like Firefox/3.6.28)*/
const Device UA2502233 = {S45802, 105, { &P17595,&P18207,&P22505,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en; rv:1.9.2.28) Gecko/X Camino/2.1.2 (like Firefox/13.0)*/
const Device UA723310 = {S45803, 145, { &P15975,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Light Tab 2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1930535 = {S45804, 145, { &P15975,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; MegaFon V9+ Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023937 = {S45805, 137, { &P15975,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; V9A Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032969 = {S45806, 69, { &P15975,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Light Tab 2 Build/GINGERBREAD)*/
const Device UA2335286 = {S45807, 137, { &P15975,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; th-th; V9+ Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2427992 = {S45808, 141, { &P15975,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-za; ZTE V9A Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2597362 = {S45809, 144, { &P15975,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Light Tab 2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2686274 = {S45810, 61, { &P15975,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; V9A Build/GINGERBREAD)*/
const Device UA1994528 = {S45811, 146, { &P18512,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT881 Build/6.7.3-130_YUS-11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025010 = {S45812, 150, { &P18512,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT881 Build/6.7.3-85_YTZUSC-LE-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030867 = {S45813, 139, { &P18512,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT881 Build/6.7.3-144) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2377091 = {S45814, 68, { &P18512,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; XT881 Build/6.7.3-130_YUS-11)*/
const Device UA2741712 = {S45815, 149, { &P18317,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754330 = {S45816, 150, { &P18317,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300TL Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754378 = {S45817, 150, { &P18317,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300TG Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2769280 = {S45818, 128, { &P18317,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; TF300T Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2892728 = {S45819, 137, { &P18317,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Pad TF300T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA30382 = {S45820, 64, { &P15364,&P17285,&P17215,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.0; en) Opera 9.5*/
const Device UA93011 = {S45821, 34, { &P15364,&P17285,&P17215,&P18092 }};	/*Useragent - Opera/9.52 (Windows NT 6.0; U; en)*/
const Device UA103008 = {S45822, 34, { &P15364,&P17285,&P17215,&P18092 }};	/*Useragent - Opera/9.51 (Windows NT 6.0; U; en)*/
const Device UA121091 = {S45823, 34, { &P15364,&P17285,&P17215,&P18092 }};	/*Useragent - Opera/9.50 (Windows NT 6.0; U; en)*/
const Device UA1863208 = {S45824, 65, { &P15364,&P17285,&P17215,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.0; en) Opera 9.50*/
const Device UA1885411 = {S45825, 33, { &P15364,&P17285,&P17215,&P18092 }};	/*Useragent - Opera/9.5 (Windows NT 6.0; U; en)*/
const Device UA2569626 = {S45826, 75, { &P15364,&P5690,&P21791,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:19.0) Gecko/X Firefox/19.0 SeaMonkey/2.16.2*/
const Device UA2583734 = {S45827, 82, { &P15364,&P5690,&P21791,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:19.0) Gecko/X Firefox/19.0 SeaMonkey/2.16.2*/
const Device UA2601104 = {S45828, 62, { &P15364,&P5690,&P21791,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:19.0) Gecko/X SeaMonkey/2.16.2*/
const Device UA2202766 = {S45829, 112, { &P18643,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia311/03.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2405400 = {S45830, 111, { &P18643,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia311/03.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5*/
const Device UA2041690 = {S45831, 115, { &P15767,&P18117,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Twitter for iPhone*/
const Device UA2041905 = {S45832, 117, { &P15767,&P18117,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Twitter for iPhone*/
const Device UA635689 = {S45833, 82, { &P16455,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-S5610K/S5610KDDKK2 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2262907 = {S45834, 80, { &P16455,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-S5610/S5610XXLF4 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA239631 = {S45835, 142, { &P16304,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-R730 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA683581 = {S45836, 66, { &P16304,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SCH-R730 Build/GINGERBREAD)*/
const Device UA2258449 = {S45837, 73, { &P16329,&P18667,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PLAYSTATION 3 4.31) AppleWebKit/531.22.8 (KHTML, like Gecko)*/
const Device UA2467372 = {S45838, 139, { &P23771,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2437114 = {S45839, 72, { &P12280,&P18432,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.1 (iPhone4,1; iPhone OS 6.0.1; en_US; en) AppleWebKit/420+*/
const Device UA2437200 = {S45840, 72, { &P12280,&P18432,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.1 (iPhone5,2; iPhone OS 6.0.2; en_US; en) AppleWebKit/420+*/
const Device UA2515310 = {S45841, 70, { &P12280,&P18432,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPhone4,1; iPhone OS 6.0; en_US; en) AppleWebKit/420+*/
const Device UA15657 = {S45842, 165, { &P11830,&P5129,&P3690,&P18092 }};	/*Useragent - SonyEricssonU1i/R2AA; Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525*/
const Device UA15656 = {S45843, 166, { &P11830,&P5129,&P3690,&P18092 }};	/*Useragent - SonyEricssonU1iv/R2AA; Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525*/
const Device UA63857 = {S45844, 165, { &P11830,&P5129,&P3690,&P18092 }};	/*Useragent - SonyEricssonU1a/R2AA; Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525*/
const Device UA2945952 = {S45845, 87, { &P16488,&P5567,&P17371,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/5.1/27.2338; U; en) Presto/2.8.119 240X400 Pantech CDM8992*/
const Device UA925411 = {S45846, 126, { &P15364,&P16110,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA925423 = {S45847, 119, { &P15364,&P16110,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA2566061 = {S45848, 124, { &P15364,&P7133,&P22465,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Iron/25.0.1400.0 Chrome/25.0.1400.0 Safari/537.22*/
const Device UA2566293 = {S45849, 117, { &P15364,&P7133,&P22465,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML, like Gecko) Iron/25.0.1400.0 Chrome/25.0.1400.0 Safari/537.22*/
const Device UA2679606 = {S45850, 43, { &P15364,&P18580,&P22902,&P18092 }};	/*Useragent - Mercury/7.4 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA887159 = {S45851, 139, { &P18722,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; IM-A830L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA895140 = {S45852, 139, { &P18722,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; IM-A830S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA902974 = {S45853, 139, { &P18722,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; IM-A830K Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1964920 = {S45854, 61, { &P18722,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; IM-A830L Build/IML74K)*/
const Device UA2023278 = {S45855, 61, { &P18722,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; IM-A830S Build/IML74K)*/
const Device UA2026710 = {S45856, 61, { &P18722,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; IM-A830K Build/IML74K)*/
const Device UA2375966 = {S45857, 137, { &P18722,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; IM-A830K Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2393512 = {S45858, 137, { &P18722,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; IM-A830S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2476807 = {S45859, 137, { &P18722,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; IM-A830L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665489 = {S45860, 135, { &P18722,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; SKY-A830 Build/LTE) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2700536 = {S45861, 140, { &P18722,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; SKY-A830S/L/K Build/LTE) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2700745 = {S45862, 138, { &P18722,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; IM-A830S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2750766 = {S45863, 142, { &P18722,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; IM-A830 Build/Vesion 1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2779525 = {S45864, 136, { &P18722,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SKY-A830 Build/LTE) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986926 = {S45865, 138, { &P18722,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; IM-A830K Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA3071641 = {S45866, 141, { &P18722,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SKY-A830S/L/K Build/LTE) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2741625 = {S45867, 76, { &P15364,&P18110,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:22.0) Gecko/X Firefox/22.0 Iceweasel/22.0*/
const Device UA2753386 = {S45868, 69, { &P15364,&P18110,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 on x86_64; rv:22.0) Gecko/X Firefox/22.0*/
const Device UA2962399 = {S45869, 63, { &P15364,&P18110,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:22.0) Gecko/X Iceweasel/22.0*/
const Device UA2167547 = {S45870, 138, { &P15803,&P5843,&P24668,&P18092 }};	/*Useragent - Nokia200/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia200) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA7983 = {S45871, 138, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB611 Build/BGN_1.1.23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA7991 = {S45872, 135, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/BGN_1.0.23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA7994 = {S45873, 135, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2; en-us; MB611 Build/BGN_1.0.23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA8019 = {S45874, 131, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/STAB2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA8325 = {S45875, 135, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us;MB611 Build/BGN_1.0.23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA8329 = {S45876, 137, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB611 Build/BGN_1.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA9505 = {S45877, 130, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/S246) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA9506 = {S45878, 136, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/BGN_1.0.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA9508 = {S45879, 138, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB611 Build/BGN_1.1.18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA9510 = {S45880, 132, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/1.0.11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA26829 = {S45881, 136, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/BGN_1.0.23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA26831 = {S45882, 132, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/1.0.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38378 = {S45883, 138, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB611 Build/BGN_1.1.29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38388 = {S45884, 136, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/BGN_1.0.24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52095 = {S45885, 137, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB611 Build/BGN_1.1.30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA52106 = {S45886, 131, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/1.0.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52115 = {S45887, 130, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/S266) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52120 = {S45888, 138, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB611 Build/BGN_1.1.25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52131 = {S45889, 136, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/BGN_1.0.15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52146 = {S45890, 137, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;MB611 Build/BGN_1.1.30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52155 = {S45891, 130, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/S269) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52159 = {S45892, 137, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.2; en-us; MB611 Build/BGN_1.1.30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52149 = {S45893, 137, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.2; en-us; MB611 Build/BGN_1.1.30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55658 = {S45894, 138, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB611 Build/BGN_1.1.27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55657 = {S45895, 138, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB611 Build/BGN_1.1.30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA69608 = {S45896, 136, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/BGN_1.0.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74977 = {S45897, 130, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MB611 Build/S275) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370329 = {S45898, 138, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB611 Build/BGN_1.1.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA409144 = {S45899, 62, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; MB611 Build/BGN_1.1.30)*/
const Device UA2019340 = {S45900, 60, { &P17718,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; MB611 Build/BGN_1.0.23)*/
const Device UA2029571 = {S45901, 134, { &P19904,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C5170 Build/IML77) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2482335 = {S45902, 135, { &P19904,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Hydro Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1996275 = {S45903, 155, { &P18645,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1997992 = {S45904, 163, { &P18645,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1999152 = {S45905, 163, { &P18645,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2002709 = {S45906, 155, { &P18645,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2183088 = {S45907, 156, { &P18645,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-R530U Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2194057 = {S45908, 155, { &P18645,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2198668 = {S45909, 163, { &P18645,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2236716 = {S45910, 167, { &P18645,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I747 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258199 = {S45911, 159, { &P18645,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L710 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2384457 = {S45912, 160, { &P18645,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530U Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422984 = {S45913, 132, { &P23773,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8010 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2485135 = {S45914, 132, { &P23773,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8010 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA791697 = {S45915, 67, { &P15364,&P18110,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux x86_64; U; ru) Presto/2.10.229 Version/11.64*/
const Device UA2221227 = {S45916, 73, { &P15364,&P18110,&P18347,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.64*/
const Device UA2023898 = {S45917, 143, { &P19876,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-S950C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2466220 = {S45918, 67, { &P19876,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-S950C Build/GINGERBREAD)*/
const Device UA2369645 = {S45919, 131, { &P18715,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID8048 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2889338 = {S45920, 60, { &P18715,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; MID8048 Build/IML74K)*/
const Device UA623473 = {S45921, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; it-it; SmartTab10-MSM8260-V03a-MarX-Vodafone-IT) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA631554 = {S45922, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; tr-tr; SmartTab10-MSM8260-V03a-MarX-Vodafone-TR) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA653524 = {S45923, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-ie; SmartTab10-MSM8260-V03a-MarX-Vodafone-IE) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA674033 = {S45924, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; SmartTab10-MSM8260-V02d-DecX-Vodafone-ZA) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA690084 = {S45925, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; sr-rs; SmartTab10-MSM8260-V02d-DecX-Vodafone-RS) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA758987 = {S45926, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; es-es; SmartTab10-MSM8260-V02d-DecX-Vodafone-ES) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA766534 = {S45927, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-nz; SmartTab10-MSM8260-V02d-DecX-Vodafone-NZ) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA834059 = {S45928, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; tr-tr; SmartTab10-MSM8260-V02d-DecX-Vodafone-TR) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA894822 = {S45929, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; hu-hu; SmartTab10-MSM8260-V03a-MarX-Vodafone-HU) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA895834 = {S45930, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; SmartTab10-MSM8260-V03b-AprX-Vodafone-ZA) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA902148 = {S45931, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; nl-nl; SmartTab10-MSM8260-V03b-AprX-Vodafone-NL) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA909126 = {S45932, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; it-it; SmartTab10-MSM8260-V03b-AprX-Vodafone-IT) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA910022 = {S45933, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; tr-tr; SmartTab10-MSM8260-V03b-MayX-Vodafone-TR) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1952790 = {S45934, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; hu-hu; SmartTab10-MSM8260-V02d-DecX-Vodafone-HU) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1960161 = {S45935, 158, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; es-es; SmartTab10-MSM8260-V03z-AprX-Vodafone-ES-GotaTest) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1964089 = {S45936, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; cs-cz; SmartTab10-MSM8260-V03a-MarX-Vodafone-CZ) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1971647 = {S45937, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; hu-hu; SmartTab10-MSM8260-V03b-AprX-Vodafone-HU) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1984648 = {S45938, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ro-ro; SmartTab10-MSM8260-V03a-MarX-Vodafone-RO) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1989459 = {S45939, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ro-ro; SmartTab10-MSM8260-V03b-AprX-Vodafone-RO) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2031455 = {S45940, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; nl-nl; SmartTab10-MSM8260-V03a-MarX-Vodafone-NL) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2034111 = {S45941, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; el-gr; SmartTab10-MSM8260-V03a-MarX-Vodafone-GR) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2411970 = {S45942, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; es-es; SmartTab10-MSM8260-V03b-AprX-Vodafone-ES) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2642783 = {S45943, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; SmartTab10-MSM8260-V03b-AprX-Vodafone-RS) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2652695 = {S45944, 156, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; SmartTab10-MSM8260-V03a-MarX-Vodafone-LK) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2889187 = {S45945, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; el-gr; SmartTab10-MSM8260-V03b-AprX-Vodafone-GR) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2983293 = {S45946, 154, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; it-it; SmartTab10-MSM8260-V03b-AprX-Vodafone-IT) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2988685 = {S45947, 149, { &P15903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; vi-vn; SmartTab10-MSM8260-V03a-FebX-Vodafone-DE) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA93514 = {S45948, 34, { &P15364,&P5690,&P22484,&P18092 }};	/*Useragent - Opera/9.10 (Windows NT 5.1; U; en)*/
const Device UA459455 = {S45949, 34, { &P15364,&P5690,&P22484,&P18092 }};	/*Useragent - Opera/9.10 (Windows NT 5.2; U; en)*/
const Device UA670234 = {S45950, 78, { &P15364,&P5690,&P22484,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.8.0) Gecko/X Firefox/1.5.0 Opera 9.10*/
const Device UA2438013 = {S45951, 34, { &P15364,&P5690,&P22484,&P18092 }};	/*Useragent - Opera/9.12 (Windows NT 5.1; U; ru)*/
const Device UA2746275 = {S45952, 70, { &P15364,&P5690,&P22484,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; en) Opera 9.12*/
const Device UA2547815 = {S45953, 136, { &P22746,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N9100 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2702645 = {S45954, 58, { &P22746,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; N9100 Build/IMM76I)*/
const Device UA3065376 = {S45955, 130, { &P18702,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8013 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080915 = {S45956, 130, { &P18702,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8013 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2489337 = {S45957, 114, { &P14635,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-00/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA140257 = {S45958, 25, { &P15364,&P17017,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 Firefox/3.5.6*/
const Device UA792809 = {S45959, 35, { &P15364,&P17017,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 Firefox/3.5.6 Tapatalk/*/
const Device UA2197574 = {S45960, 41, { &P15364,&P17017,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 Firefox/3.5.6 Tapatalk/1.14.2*/
const Device UA2202800 = {S45961, 38, { &P15364,&P17017,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 Firefox/3.5.6 Tapatalk/2.4*/
const Device UA2260125 = {S45962, 41, { &P15364,&P17017,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 Firefox/3.5.6 Tapatalk/1.14.3*/
const Device UA2368560 = {S45963, 40, { &P15364,&P17017,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 Firefox/3.5.6 Tapatalk/2.4.5*/
const Device UA2368585 = {S45964, 41, { &P15364,&P17017,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 Firefox/3.5.6 Tapatalk/1.15.0*/
const Device UA2407243 = {S45965, 41, { &P15364,&P17017,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 Firefox/3.5.6 Tapatalk/1.15.1*/
const Device UA2435299 = {S45966, 40, { &P15364,&P17017,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 Firefox/3.5.6 Tapatalk/2.4.7*/
const Device UA2489179 = {S45967, 41, { &P15364,&P17017,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 Firefox/3.5.6 Tapatalk/2.4.10*/
const Device UA2511357 = {S45968, 41, { &P15364,&P17017,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 Firefox/3.5.6 Tapatalk/1.16.0*/
const Device UA2511495 = {S45969, 41, { &P15364,&P17017,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 Firefox/3.5.6 Tapatalk/2.4.11*/
const Device UA2562408 = {S45970, 41, { &P15364,&P17017,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 Firefox/3.5.6 Tapatalk/2.4.13*/
const Device UA2895430 = {S45971, 41, { &P15364,&P17017,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 Firefox/3.5.6 Tapatalk/1.17.1*/
const Device UA3066822 = {S45972, 40, { &P15364,&P17017,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 Firefox/3.5.6 Tapatalk/2.0.2*/
const Device UA403341 = {S45973, 102, { &P16463,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; GT-I8350)*/
const Device UA641392 = {S45974, 109, { &P16463,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; Omnia W; Orange)*/
const Device UA757580 = {S45975, 103, { &P16463,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; GT-I8350T)*/
const Device UA2486964 = {S45976, 70, { &P15767,&P19826,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10B141 (iPad; U; CPU OS 6_1 like Mac OS X; en_us)*/
const Device UA2532513 = {S45977, 72, { &P15767,&P19826,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10B146 (iPad; U; CPU OS 6_1_2 like Mac OS X; en_us)*/
const Device UA2585373 = {S45978, 72, { &P15767,&P19826,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10B329 (iPad; U; CPU OS 6_1_3 like Mac OS X; en_us)*/
const Device UA156 = {S45979, 160, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; Milestone XT720 Build/STR_U2_02.1E.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA165 = {S45980, 167, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; Milestone XT720 Build/Itfunz_XT720_NewBegin) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA167 = {S45981, 160, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Milestone XT720 Build/STR_U2_03.20.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA169 = {S45982, 160, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Milestone XT720 Build/STR_U2_01.1E.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA177 = {S45983, 159, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; Motorola XT720 Build/STR_U2_05.1F.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA178 = {S45984, 159, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-ca; Motorola XT720 Build/STR_U2_05.1F.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA179 = {S45985, 160, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-ru; Milestone XT720 Build/STR_U2_03.20.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA180 = {S45986, 160, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Milestone XT720 Build/STR_U2_01.18.3) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA3485 = {S45987, 175, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1_Hiapk_1.1_XXKJ; zh-cn; Milestone XT720 Build/STR_U2_01.18.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA7218 = {S45988, 161, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Milestone XT720 Build/STR_U2_00.18.0D) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA26366 = {S45989, 160, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Milestone XT720 Build/STR_U2_01.18.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA37963 = {S45990, 160, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Milestone XT720 Build/STR_U2_04.1E.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA51425 = {S45991, 160, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Milestone XT720 Build/STR_U2_02.18.3) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA54078 = {S45992, 160, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Milestone XT720 Build/STR_U2_04.1E.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA54080 = {S45993, 160, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Milestone XT720 Build/STR_U2_01.1E.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA54081 = {S45994, 160, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Milestone XT720 Build/STR_U2_00.15.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA383655 = {S45995, 153, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-kr; XT720 Build/STSKT_N_79.11.39R) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA417323 = {S45996, 163, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-kr; XT720 Build/KyuHyuk ROM for XT720 1.5.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA492151 = {S45997, 154, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT720 Build/STSKT_N_79.11.36R1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA492727 = {S45998, 153, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT720 Build/STSKT_N_79.22.44R) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1864225 = {S45999, 82, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Milestone XT720 Build/STR_U2_01.1E.0)*/
const Device UA2018067 = {S46000, 153, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-kr; XT720 Build/STSKT_N_79.22.39R) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018147 = {S46001, 153, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT720 Build/STSKT_N_79.11.41R) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018167 = {S46002, 153, { &P16103,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT720 Build/STSKT_N_79.11.44R) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2700101 = {S46003, 112, { &P22713,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 521)*/
const Device UA2676076 = {S46004, 129, { &P18639,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 10 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676486 = {S46005, 128, { &P18639,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Nexus 10 Build/JVP15P) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676686 = {S46006, 130, { &P18639,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 10 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA4251 = {S46007, 92, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; SPV M3100; OpVer 14.123.1.614)*/
const Device UA8640 = {S46008, 61, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x240)*/
const Device UA24318 = {S46009, 68, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 240x320)*/
const Device UA39916 = {S46010, 91, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; SPV M700; OpVer 19.123.1.615)*/
const Device UA41765 = {S46011, 94, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 240x320; SPV C600; OpVer 11.2.1.5)*/
const Device UA45104 = {S46012, 66, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - TyTN Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)*/
const Device UA54075 = {S46013, 78, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Sprint:PPC-6700; PPC; 240x320)*/
const Device UA54093 = {S46014, 90, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; SPV M600; OpVer 13.10.1.107)*/
const Device UA57568 = {S46015, 94, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 240x320; SPV C600; OpVer 11.2.1.8)*/
const Device UA59517 = {S46016, 91, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 480x640; SPV M5000; OpVer 7.130.1.145)*/
const Device UA80314 = {S46017, 108, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; SDA/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1)*/
const Device UA80319 = {S46018, 108, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; SDA/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1)*/
const Device UA80325 = {S46019, 61, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)*/
const Device UA80317 = {S46020, 129, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 176x220; Smartphone; SDA/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1)*/
const Device UA80370 = {S46021, 93, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 176x220; SPV C500; OpVer 4.1.1.4)*/
const Device UA80371 = {S46022, 93, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 176x220; SPV C500; OpVer 4.1.2.0)*/
const Device UA80397 = {S46023, 90, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; SPV M2000; OpVer 5.31.1.124)*/
const Device UA80401 = {S46024, 90, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; SPV M2000; OpVer 5.40.2.223)*/
const Device UA80404 = {S46025, 56, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; 240x320)*/
const Device UA80405 = {S46026, 89, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; SPV M500; OpVer 6.06.3.123)*/
const Device UA80407 = {S46027, 86, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; SPV M600; Configuration/CLDC-1.1)*/
const Device UA85184 = {S46028, 87, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; SPV M5000; Configuration/CLDC-1.1)*/
const Device UA86352 = {S46029, 52, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)*/
const Device UA86568 = {S46030, 90, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; SPV M600; OpVer 13.10.2.112)*/
const Device UA87050 = {S46031, 90, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; SPV M1500; OpVer 3.11.1.167)*/
const Device UA87362 = {S46032, 94, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 240x320; SPV C550; OpVer 8.2.1.10)*/
const Device UA87363 = {S46033, 98, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 320x240; SPV E600; OpVer 20.118.1.615)*/
const Device UA87902 = {S46034, 80, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Sprint:SPH-ip830w; PPC; 240x320)*/
const Device UA91758 = {S46035, 68, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 176x220)*/
const Device UA101837 = {S46036, 58, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows NT Windows CE)*/
const Device UA118170 = {S46037, 83, { &P18088,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; MSN Companion 2.0; 800x600; Compaq)*/
const Device UA459128 = {S46038, 131, { &P20470,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID9742 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2322767 = {S46039, 60, { &P20470,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; MID9742 Build/IML74K)*/
const Device UA2668180 = {S46040, 77, { &P17595,&P17296,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:2.0.1) Gecko/X Firefox/19.0.2*/
const Device UA204357 = {S46041, 140, { &P17948,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SC-02C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA251975 = {S46042, 140, { &P17948,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SC-02C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA378767 = {S46043, 138, { &P17948,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-; SC-02C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA380540 = {S46044, 140, { &P17948,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ja-jp; SC-02C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA409304 = {S46045, 64, { &P17948,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SC-02C Build/GINGERBREAD)*/
const Device UA575300 = {S46046, 138, { &P17948,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-; SC-02C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625742 = {S46047, 138, { &P17948,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ja-; SC-02C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA713885 = {S46048, 64, { &P17948,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SC-02C Build/GINGERBREAD)*/
const Device UA1906707 = {S46049, 64, { &P17948,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SC-02C Build/GINGERBREAD)*/
const Device UA2023982 = {S46050, 166, { &P17948,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja_jp; SC-02C Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)*/
const Device UA2596693 = {S46051, 138, { &P18243,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-P880 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2667704 = {S46052, 60, { &P18243,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P880 Build/JZO54K)*/
const Device UA2750947 = {S46053, 154, { &P18243,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P880/P88020a-EUR-XXX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998167 = {S46054, 143, { &P18243,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; LG-P880/V20a Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3062122 = {S46055, 154, { &P18243,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P880/P88020b-EUR-XXX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2755034 = {S46056, 153, { &P15767,&P18117,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10 UCBrowser/2.3.2.300*/
const Device UA2362029 = {S46057, 129, { &P18680,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2426540 = {S46058, 58, { &P18680,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; KFJWI Build/IMM76D)*/
const Device UA2604677 = {S46059, 136, { &P18680,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; KFJWI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3086516 = {S46060, 127, { &P18680,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; KFJWI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1221 = {S46061, 62, { &P16074,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia3110c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA1224 = {S46062, 73, { &P16074,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia3110c/2.0 (06.02) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia3110c*/
const Device UA1289 = {S46063, 67, { &P16074,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia3110c_CMCC/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA80490 = {S46064, 9, { &P16074,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia3110*/
const Device UA7027 = {S46065, 122, { &P14286,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-S8000-Bouygues/S8000AGIG4 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA53671 = {S46066, 132, { &P14286,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-S8000/S8000XPJA1 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 SS-Widget/S8000-FM*/
const Device UA54963 = {S46067, 120, { &P14286,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-S8000-ORANGE/S8000BVIG4 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA1997828 = {S46068, 153, { &P17968,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; cm_tenderloin Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2355371 = {S46069, 153, { &P17968,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; cm_tenderloin Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2368736 = {S46070, 153, { &P17968,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PG8600 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2487383 = {S46071, 154, { &P17968,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; skz_tenderloin Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2657569 = {S46072, 157, { &P17968,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; pa_tenderloin Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2151839 = {S46073, 117, { &P15364,&P16110,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.126 Safari/533.4*/
const Device UA2533008 = {S46074, 112, { &P21407,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 928)*/
const Device UA2167718 = {S46075, 142, { &P14540,&P5843,&P24668,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-01) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2425618 = {S46076, 138, { &P14123,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-P970 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2503614 = {S46077, 60, { &P14123,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P970 Build/IMM76L)*/
const Device UA2666032 = {S46078, 153, { &P14123,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; LG-P970/P97030b-EUR-XX Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2751010 = {S46079, 139, { &P14123,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-sa; LG-P970h Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2767310 = {S46080, 145, { &P14123,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; LG-P970-Orange Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768650 = {S46081, 146, { &P14123,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-P970g-parrot Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2793457 = {S46082, 139, { &P14123,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-P970g Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2935898 = {S46083, 153, { &P14123,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; LG-P970/P97030c-208-10 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2938103 = {S46084, 146, { &P14123,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; LG-P970/P97030c Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2489758 = {S46085, 145, { &P15767,&P19826,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B141 Safari/7534.48.3*/
const Device UA2535182 = {S46086, 147, { &P15767,&P19826,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_1_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B146 Safari/7534.48.3*/
const Device UA2607026 = {S46087, 147, { &P15767,&P19826,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_1_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B329 Safari/7534.48.3*/
const Device UA2765815 = {S46088, 147, { &P15767,&P19826,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_1_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B147 Safari/7534.48.3*/
const Device UA852564 = {S46089, 76, { &P17595,&P18207,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:8.0.1) Gecko/X Firefox/8.0.1*/
const Device UA2430055 = {S46090, 131, { &P18454,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGPT13 Build/TID0142) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2436237 = {S46091, 131, { &P18454,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGPT12 Build/TID0142) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA70721 = {S46092, 124, { &P12280,&P17146,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Twitter for iPhone*/
const Device UA91001 = {S46093, 126, { &P12280,&P17146,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Twitter for iPhone*/
const Device UA622992 = {S46094, 126, { &P12280,&P17146,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Twitter for iPhone*/
const Device UA2371956 = {S46095, 126, { &P12280,&P17146,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9Z999 Twitter for iPhone*/
const Device UA64286 = {S46096, 77, { &P12054,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry9500/5.0.0.841 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/120*/
const Device UA44166 = {S46097, 79, { &P14533,&P6011,&P4093,&P18092 }};	/*Useragent - BlackBerry9630fts/4.7.1.40 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/104*/
const Device UA62068 = {S46098, 76, { &P14533,&P6011,&P4093,&P18092 }};	/*Useragent - BlackBerry9630/4.7.1.61 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105*/
const Device UA56098 = {S46099, 99, { &P13868,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; HD7 T9292)*/
const Device UA56340 = {S46100, 93, { &P13868,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; HD7)*/
const Device UA184133 = {S46101, 95, { &P13868,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; T9296)*/
const Device UA847029 = {S46102, 100, { &P13868,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/ 7.0; HTC; HD7 T9292)*/
const Device UA925729 = {S46103, 94, { &P13868,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Gold)*/
const Device UA2857969 = {S46104, 115, { &P13868,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/ 7.0; WpsLondonTest; HTC; HD7 T9292)*/
const Device UA79845 = {S46105, 34, { &P15364,&P17017,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0;*/
const Device UA141575 = {S46106, 8, { &P15364,&P17017,&P17190,&P18092 }};	/*Useragent - MSIE 8.0*/
const Device UA835204 = {S46107, 170, { &P15364,&P17017,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; ; Trident/4.0; SV1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Policy)*/
const Device UA2509628 = {S46108, 52, { &P15364,&P17017,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; PPC; Trident/4.0)*/
const Device UA2581666 = {S46109, 47, { &P15364,&P17017,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident/4.0)*/
const Device UA2792230 = {S46110, 57, { &P15364,&P17017,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; PPC; SV1; Trident/1.0)*/
const Device UA2974016 = {S46111, 130, { &P15882,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P5110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974378 = {S46112, 130, { &P15882,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P5110 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2367627 = {S46113, 156, { &P18460,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI U8950N-51 Build/HuaweiU8950N-51) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2385650 = {S46114, 150, { &P18460,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI U8950D Build/HuaweiU8950D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467190 = {S46115, 154, { &P18460,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HUAWEI U8950N-1 Build/HuaweiU8950N-1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2480510 = {S46116, 141, { &P18460,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U8950D Build/HuaweiU8950D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2506077 = {S46117, 152, { &P18460,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI U8950-1 Build/HuaweiU8950-1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2553416 = {S46118, 154, { &P18460,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-ve; HUAWEI U8950-51 Build/HuaweiU8950-51) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654779 = {S46119, 72, { &P18460,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI U8950D Build/HuaweiU8950D)*/
const Device UA2657910 = {S46120, 74, { &P18460,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI U8950-1 Build/HuaweiU8950-1)*/
const Device UA2982961 = {S46121, 78, { &P18460,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI U8950N-51 Build/HuaweiU8950N-51)*/
const Device UA1956281 = {S46122, 132, { &P17337,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P7500 Build/HTJ85B) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1994791 = {S46123, 132, { &P17337,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2030767 = {S46124, 133, { &P17337,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500R Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2184219 = {S46125, 133, { &P17337,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500M Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2346804 = {S46126, 133, { &P17337,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2444036 = {S46127, 133, { &P17337,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  GT-P7500 Build/HTJ85B) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2608728 = {S46128, 133, { &P17337,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  GT-P7500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2614416 = {S46129, 138, { &P17337,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2619346 = {S46130, 143, { &P17337,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 wI-fI & 3g Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2620175 = {S46131, 142, { &P17337,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 WIFI & 3G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA428922 = {S46132, 135, { &P15364,&P17818,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.11 (KHTML, like Gecko) Ubuntu/11.04 Chromium/17.0.963.79 Chrome/17.0.963.79 Safari/535.11*/
const Device UA841760 = {S46133, 137, { &P15364,&P17818,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.11 (KHTML, like Gecko) Ubuntu/11.04 Chromium/17.0.963.65 Chrome/17.0.963.65 Safari/535.11*/
const Device UA853465 = {S46134, 13, { &P15364,&P17017,&P17209,&P18092 }};	/*Useragent - Firefox/7.0.1*/
const Device UA899637 = {S46135, 84, { &P15364,&P17017,&P17209,&P18092 }};	/*Useragent - RebelMouse/0.1 Mozilla/5.0 (compatible; http://rebelmouse.com) Gecko/X Firefox/7.0.1*/
const Device UA2942037 = {S46136, 140, { &P21636,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_C525c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2964021 = {S46137, 136, { &P21636,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C525c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2972553 = {S46138, 58, { &P21636,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; C525c Build/JZO54K)*/
const Device UA818352 = {S46139, 139, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA875501 = {S46140, 136, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; i9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA908232 = {S46141, 136, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1960166 = {S46142, 134, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; i9220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1961419 = {S46143, 139, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1993171 = {S46144, 136, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.6; en-us; i9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026663 = {S46145, 134, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; I9220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027515 = {S46146, 130, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9220 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2027658 = {S46147, 136, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.5; en-us; HK-I9220 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029654 = {S46148, 136, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.6; en-us; i9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030429 = {S46149, 136, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; GT-I9220 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031640 = {S46150, 130, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9220 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2032275 = {S46151, 137, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; GT-I9220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257296 = {S46152, 137, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2410718 = {S46153, 58, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.6; i9220 Build/IML74K)*/
const Device UA2444995 = {S46154, 136, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9220 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2448185 = {S46155, 61, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9220 Build/IMM76D)*/
const Device UA2465604 = {S46156, 136, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2653321 = {S46157, 148, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; GT_I9220 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2721163 = {S46158, 58, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; i9220 Build/IML74K)*/
const Device UA2744360 = {S46159, 134, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I9220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2775778 = {S46160, 156, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - GT-I9220 Linux/3.0.13 Android/4.1.1 Release/07.21.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2810070 = {S46161, 58, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; i9220 Build/IML74K)*/
const Device UA2818047 = {S46162, 168, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.6; en-us; Angelobiz v3.2 for Haipai Noble I9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878130 = {S46163, 134, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.6; en-us; I9220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042851 = {S46164, 136, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; i9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3043072 = {S46165, 72, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 4.0.9; GT_I9220 Build/GINGERBREAD.XXKL3)*/
const Device UA3071462 = {S46166, 137, { &P22875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.6; it-it; i92200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA857632 = {S46167, 135, { &P19874,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-i705 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026160 = {S46168, 132, { &P19874,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I705 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2419081 = {S46169, 61, { &P19874,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-i705 Build/IMM76D)*/
const Device UA2528293 = {S46170, 132, { &P19874,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; SCH-i705 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA568267 = {S46171, 133, { &P15936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; XT316 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA641463 = {S46172, 134, { &P15936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT316 Build/V4.26F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA653318 = {S46173, 134, { &P15936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT316 Build/V4.26M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA663887 = {S46174, 134, { &P15936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT316 Build/V4.26H) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA703595 = {S46175, 134, { &P15936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT316 Build/V4.26P) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA733720 = {S46176, 143, { &P15936,&P5629,&P15760,&P18092 }};	/*Useragent - MOT-XT316/Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; XT316 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA788077 = {S46177, 134, { &P15936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT316 Build/V4.26K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA817054 = {S46178, 134, { &P15936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT316 Build/V4.26O) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA853642 = {S46179, 134, { &P15936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT316 Build/V4.26U) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA864192 = {S46180, 134, { &P15936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; XT316 Build/V4.620) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1917249 = {S46181, 134, { &P15936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT316 Build/V4.34A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1971691 = {S46182, 144, { &P15936,&P5629,&P15760,&P18092 }};	/*Useragent - MOT-XT316/Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; XT316 Build/V4.480) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1978975 = {S46183, 134, { &P15936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; XT316 Build/V4.630) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019500 = {S46184, 134, { &P15936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT316 Build/V4.340) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023646 = {S46185, 134, { &P15936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT316 Build/V4.26Q) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024540 = {S46186, 134, { &P15936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; XT316 Build/V4.26R) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026693 = {S46187, 134, { &P15936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT316 Build/V4.26T) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033701 = {S46188, 134, { &P15936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; XT316 Build/V4.500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2594512 = {S46189, 134, { &P15936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; XT316 Build/V4.650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2815584 = {S46190, 204, { &P15936,&P5629,&P15760,&P18092 }};	/*Useragent - MOT-XT316/1.0 Android/2.3.4 Release/9.15.2011 Browser/AppleWebKit533.1 Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; XT316 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2860291 = {S46191, 147, { &P15936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5 Ambras Build; es-us; XT316 Build/V4.620) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2974154 = {S46192, 137, { &P18509,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2259146 = {S46193, 71, { &P15364,&P7133,&P18691,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:16.0) Gecko/X Thunderbird/16.0.2*/
const Device UA2973965 = {S46194, 130, { &P22043,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P5113 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022139 = {S46195, 130, { &P22043,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P5113 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA43170 = {S46196, 136, { &P14046,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; HTC-Vivo Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73612 = {S46197, 136, { &P14046,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC-Vivo Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2424764 = {S46198, 141, { &P20277,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_H3000C Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA630607 = {S46199, 123, { &P15547,&P17484,&P3834,&P18092 }};	/*Useragent - SAMSUNG-GT-C6712/C6712DDKK3 SHP/VPP/R5 Dolfin/2.0 NexPlayer/3.0 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B*/
const Device UA2586653 = {S46200, 139, { &P21628,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8552 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768933 = {S46201, 61, { &P21628,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8552 Build/JZO54K)*/
const Device UA3015289 = {S46202, 140, { &P21628,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; GT-I8552B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049856 = {S46203, 137, { &P21628,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-I8552 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108551 = {S46204, 62, { &P21628,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8552B Build/JZO54K)*/
const Device UA3130871 = {S46205, 137, { &P18442,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131348 = {S46206, 145, { &P18442,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131470 = {S46207, 137, { &P18442,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131643 = {S46208, 138, { &P18442,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131710 = {S46209, 137, { &P18442,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131758 = {S46210, 138, { &P18442,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132007 = {S46211, 137, { &P18442,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132013 = {S46212, 137, { &P18442,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132708 = {S46213, 138, { &P18442,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132872 = {S46214, 137, { &P18442,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132949 = {S46215, 138, { &P18442,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E250K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3064878 = {S46216, 144, { &P18640,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA20818 = {S46217, 151, { &P15133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; ja-jp; SonyEricssonSO-01C Build/3.0.D.2.79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20819 = {S46218, 139, { &P15133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; ja-jp; SO-01C Build/3.0.D.2.79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20821 = {S46219, 152, { &P15133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; ja-jp; SonyEricssonSO-01C Build/3.0.1.D.2.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43142 = {S46220, 141, { &P15133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SO-01C Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43145 = {S46221, 153, { &P15133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonSO-01C Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43150 = {S46222, 142, { &P15133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SO-01C Build/3.0.1.F.0.131) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43152 = {S46223, 154, { &P15133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonSO-01C Build/3.0.1.F.0.131) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59094 = {S46224, 164, { &P15133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; ja_jp; SO-01C Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.2 Mobile (Dragonfruit)*/
const Device UA59103 = {S46225, 142, { &P15133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SO-01C Build/3.0.1.F.0.126) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59104 = {S46226, 154, { &P15133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonSO-01C Build/3.0.1.F.0.126) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90763 = {S46227, 149, { &P15133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.3.2; ja-jp; SonyEricssonSO-01C Build/3.0.D.2.79)AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA253241 = {S46228, 152, { &P15133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-01C Build/4.0.1.C.1.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA421776 = {S46229, 140, { &P15133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SO-01C Build/4.0.1.C.1.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA453704 = {S46230, 153, { &P15133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-01C Build/4.0.1.C.1.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625519 = {S46231, 141, { &P15133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SO-01C Build/4.0.1.C.1.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1894227 = {S46232, 66, { &P15133,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SO-01C Build/3.0.1.F.0.131)*/
const Device UA2004712 = {S46233, 153, { &P15133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-01C Build/4.0.1.C.1.24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2005515 = {S46234, 141, { &P15133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SO-01C Build/4.0.1.C.1.24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018443 = {S46235, 63, { &P15133,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.2; SO-01C Build/3.0.D.2.79)*/
const Device UA2206195 = {S46236, 153, { &P15133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-01C Build/4.0.1.C.1.31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2206554 = {S46237, 141, { &P15133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SO-01C Build/4.0.1.C.1.31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2352905 = {S46238, 65, { &P15133,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SO-01C Build/4.0.1.C.1.31)*/
const Device UA3065240 = {S46239, 137, { &P16362,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065735 = {S46240, 140, { &P16362,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710BST Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066212 = {S46241, 141, { &P16362,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710VMUB Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2384725 = {S46242, 132, { &P18702,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8013 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2423351 = {S46243, 132, { &P18702,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8013 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA925194 = {S46244, 128, { &P14028,&P17484,&P4732,&P18092 }};	/*Useragent - SAMSUNG-SGH-A667/A667UCLD2; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.20822; U; en-US) Opera 9.50*/
const Device UA20302 = {S46245, 167, { &P13858,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; de-de; LG-E720/V10b; Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA41006 = {S46246, 163, { &P13858,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; it-it; LG-E720 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA45216 = {S46247, 161, { &P13858,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; it-it; LG-E720 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA53936 = {S46248, 162, { &P13858,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-ca; LG-E720b Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA58758 = {S46249, 165, { &P13858,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; LG-E720/V10d; Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.2*/
const Device UA2018347 = {S46250, 58, { &P13858,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; LG-E720b Build/FRF91)*/
const Device UA2804328 = {S46251, 57, { &P13858,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; LG-E720 Build/FRF91)*/
const Device UA2566814 = {S46252, 139, { &P18645,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530C Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566934 = {S46253, 146, { &P18645,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567257 = {S46254, 139, { &P18645,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I747M Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583109 = {S46255, 138, { &P18645,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-L710 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583372 = {S46256, 139, { &P18645,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530U Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2601590 = {S46257, 139, { &P18645,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2601775 = {S46258, 139, { &P18645,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210L Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2603536 = {S46259, 139, { &P18645,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210K Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668281 = {S46260, 132, { &P18645,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530C Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2733494 = {S46261, 142, { &P18645,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Galaxy S III Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2794703 = {S46262, 139, { &P18645,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA6403 = {S46263, 222, { &P13746,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 1.6; en-us; LG-GT540; Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA9238 = {S46264, 137, { &P13746,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; GT540 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA10086 = {S46265, 174, { &P13746,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; LG-GT540/V10b; Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA32160 = {S46266, 170, { &P13746,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; LG-GT540g; Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA43619 = {S46267, 182, { &P13746,&P5062,&P3441,&P18092 }};	/*Useragent - LG-GT540f/v10a Mozilla/5.0 (Linux; U; Android 1.6; en-us; GT540f; Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA50935 = {S46268, 222, { &P13746,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android1.6; en-us; LG-GT540; Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA54085 = {S46269, 229, { &P13746,&P5062,&P3441,&P18092 }};	/*Useragent - LG-GT540 Mozilla/5.0 (Linux; U; Android 1.6; en-us; GT540; Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA64262 = {S46270, 224, { &P13746,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; LG-GT540f; Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA64533 = {S46271, 223, { &P13746,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; LG-GT540; Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA64723 = {S46272, 224, { &P13746,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; LG-GT540 ; Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA3065482 = {S46273, 136, { &P22596,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067899 = {S46274, 137, { &P22596,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970C Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083278 = {S46275, 137, { &P22596,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970X Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA88475 = {S46276, 110, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.0; U; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Pre/1.0*/
const Device UA88713 = {S46277, 110, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.1; U; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Pre/1.0*/
const Device UA88965 = {S46278, 110, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.2; U; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Pre/1.0*/
const Device UA89113 = {S46279, 112, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.3.1; U; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Pre/1.0*/
const Device UA89271 = {S46280, 112, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.3.5; U; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Pre/1.0*/
const Device UA89868 = {S46281, 108, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.1.1; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0*/
const Device UA90016 = {S46282, 106, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; de-DE) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0*/
const Device UA114228 = {S46283, 106, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; en-GB) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0*/
const Device UA120926 = {S46284, 106, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; en-CA) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0*/
const Device UA122238 = {S46285, 106, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0*/
const Device UA125954 = {S46286, 106, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; es-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0*/
const Device UA136040 = {S46287, 106, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; fr-CA) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0*/
const Device UA140286 = {S46288, 106, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.0; U; es-MX) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0*/
const Device UA265558 = {S46289, 106, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.0; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0*/
const Device UA269578 = {S46290, 106, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; es-ES) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0*/
const Device UA269614 = {S46291, 106, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; en-IE) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0*/
const Device UA305755 = {S46292, 106, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/2.1.0; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0*/
const Device UA459530 = {S46293, 108, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.1.1; U; es-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0*/
const Device UA459979 = {S46294, 114, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.3.5.1; U; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Pre/1.0*/
const Device UA521006 = {S46295, 106, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/2.1.0; U; en-GB) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0*/
const Device UA525553 = {S46296, 106, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; it-IT) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0*/
const Device UA548799 = {S46297, 106, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/2.1.0; U; de-DE) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0*/
const Device UA1851424 = {S46298, 106, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; zh-CN) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0*/
const Device UA1895309 = {S46299, 106, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/2.1.0; U; es-MX) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0*/
const Device UA2258631 = {S46300, 106, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; es-MX) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0*/
const Device UA2264294 = {S46301, 110, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.1; U; en-CA) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Pre/1.0*/
const Device UA2270532 = {S46302, 106, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; fr-FR) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0*/
const Device UA2339364 = {S46303, 106, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; en-US) AppleWebKit/532.2 (KHTML	 like Gecko) Version/1.0 Safari/532.2 Pre/1.0*/
const Device UA2801515 = {S46304, 106, { &P16940,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.0; U; es-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.0*/
const Device UA3064992 = {S46305, 137, { &P16356,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080212 = {S46306, 138, { &P16356,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080825 = {S46307, 138, { &P16356,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081228 = {S46308, 138, { &P16356,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081614 = {S46309, 138, { &P16356,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083178 = {S46310, 131, { &P16356,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M480W Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094214 = {S46311, 135, { &P16356,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SC-05D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109651 = {S46312, 130, { &P16356,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3111144 = {S46313, 137, { &P16356,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA456739 = {S46314, 147, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_DesireHD/1.0 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA579446 = {S46315, 137, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Desire HD Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA652270 = {S46316, 144, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Desire HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA780715 = {S46317, 149, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_DesireHD_A9191 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA828581 = {S46318, 140, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Desire HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2007232 = {S46319, 140, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Desire HD Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2023999 = {S46320, 66, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Desire HD Build/IML74K)*/
const Device UA2187340 = {S46321, 144, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Desire HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2355657 = {S46322, 144, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Desire_HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2372862 = {S46323, 62, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Desire HD Build/IMM76L)*/
const Device UA2386617 = {S46324, 145, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Desire HD Build/HuaweiU8815) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2419756 = {S46325, 149, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_DesireHD_A9191 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2421630 = {S46326, 66, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Desire HD Build/IML74K)*/
const Device UA2455669 = {S46327, 144, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Desire_HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2502665 = {S46328, 144, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_Desire_HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2581501 = {S46329, 145, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_DesireHD Build/ViperDHD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658690 = {S46330, 152, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3 (B952); en-gb; Desire HD Build/HuaweiU8815) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2704774 = {S46331, 143, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_DesireHD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2732172 = {S46332, 146, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Desire HD Build/ViperDHD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2814416 = {S46333, 144, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Desire HD Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2827931 = {S46334, 149, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; HTC_DesireHD_X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2992680 = {S46335, 142, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Desire HD Build/SVHD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088454 = {S46336, 144, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ua; HTC Desire HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3100924 = {S46337, 147, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; HTC_DesireHD_A9191 Build/ucxl) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3110596 = {S46338, 142, { &P14225,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC_Desire_HD Build/SVHD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566537 = {S46339, 131, { &P15882,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P5110 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566709 = {S46340, 131, { &P15882,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P5110 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA3111115 = {S46341, 119, { &P17595,&P21721,&P24143,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.66 Safari/537.36*/
const Device UA14269 = {S46342, 73, { &P14283,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia6230i/2.0 (03.70) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6230i*/
const Device UA63496 = {S46343, 62, { &P14283,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia6230i/2.0 (03.25) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2026150 = {S46344, 141, { &P18484,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; USCC-US730 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257900 = {S46345, 139, { &P18484,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-US730 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2416844 = {S46346, 61, { &P18484,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-US730 Build/IMM76L)*/
const Device UA2425866 = {S46347, 58, { &P15364,&P18110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686) Presto/2.12.388 Version/12.12*/
const Device UA2468047 = {S46348, 71, { &P15364,&P18110,&P20412,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.12*/
const Device UA2468517 = {S46349, 63, { &P15364,&P18110,&P20412,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; X11; Linux i686) Opera 12.12*/
const Device UA2558018 = {S46350, 112, { &P15364,&P18110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux x86_64; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)) Presto/2.12.388 Version/12.12*/
const Device UA2847192 = {S46351, 68, { &P15364,&P18110,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686) Presto/2.12.388 Version/12.12,gzip(gfe)*/
const Device UA33113 = {S46352, 139, { &P15857,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; HTC Panache Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA657735 = {S46353, 63, { &P15857,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Panache Build/GRI40)*/
const Device UA2040272 = {S46354, 152, { &P18505,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MB886 Build/7.7.1Q-144_MB886_MR1-22) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3034190 = {S46355, 105, { &P21001,&P17484,&P18328,&P18092 }};	/*Useragent - SAMSUNG-GT-S3802 Opera/9.80 (J2ME/MIDP; Opera Mini/7.1.32830/30.3697; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2741739 = {S46356, 144, { &P22582,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784119 = {S46357, 136, { &P22582,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2712085 = {S46358, 110, { &P23589,&P18378,&P22586,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo WiiU) AppleWebKit/534.52 (KHTML, like Gecko) NX/2.1.0.10.9 NintendoBrowser/1.5.0.8047.EU*/
const Device UA2712564 = {S46359, 110, { &P23589,&P18378,&P22586,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo WiiU) AppleWebKit/534.52 (KHTML, like Gecko) NX/2.1.0.10.9 NintendoBrowser/1.5.0.8047.US*/
const Device UA2718968 = {S46360, 110, { &P23589,&P18378,&P22586,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo WiiU) AppleWebKit/534.52 (KHTML, like Gecko) NX/2.1.0.10.9 NintendoBrowser/1.5.0.8047.JP*/
const Device UA3115928 = {S46361, 105, { &P21842,&P17484,&P17369,&P18092 }};	/*Useragent - SAMSUNG-GT-E2202 Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.32420/31.1475; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2240798 = {S46362, 101, { &P18634,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 822)*/
const Device UA2333081 = {S46363, 139, { &P18594,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LS860 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555385 = {S46364, 61, { &P18594,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-LS860 Build/IMM76I)*/
const Device UA2556974 = {S46365, 135, { &P18594,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mach Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2974157 = {S46366, 136, { &P22596,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974893 = {S46367, 137, { &P22596,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970C Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976031 = {S46368, 137, { &P22596,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970X Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974144 = {S46369, 130, { &P23773,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8010 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2975559 = {S46370, 130, { &P23773,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8010 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2167362 = {S46371, 139, { &P21022,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-sa; LG-E615f Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2231925 = {S46372, 138, { &P21022,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-E615 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2552353 = {S46373, 60, { &P21022,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-E615 Build/IMM76D)*/
const Device UA3042900 = {S46374, 61, { &P21022,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-E615f Build/IMM76D)*/
const Device UA2231055 = {S46375, 132, { &P15882,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P5110 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2401770 = {S46376, 132, { &P15882,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P5110 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2737900 = {S46377, 141, { &P22867,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5303 Build/12.0.A.1.257) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2974019 = {S46378, 130, { &P15876,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P5100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974363 = {S46379, 130, { &P15876,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P5100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021063 = {S46380, 130, { &P15876,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T779 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA89936 = {S46381, 145, { &P17308,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; DROIDX Build/VZW) AppleWebKit/525.10  (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA894020 = {S46382, 181, { &P17308,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; DROIDX Build/VZW) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 480X854 motorola DROIDX companydatatrees*/
const Device UA2974149 = {S46383, 139, { &P12968,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.0.200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2562723 = {S46384, 138, { &P19856,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LS970 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2408066 = {S46385, 139, { &P14877,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9000 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2457644 = {S46386, 61, { &P14877,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-I9000 Build/JOP40D)*/
const Device UA2563857 = {S46387, 139, { &P14877,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9000 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566343 = {S46388, 138, { &P14877,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9000 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695815 = {S46389, 137, { &P14877,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; GT-I9000 Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2737437 = {S46390, 61, { &P14877,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9000 Build/JDQ39E)*/
const Device UA2737692 = {S46391, 139, { &P14877,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9000 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2863592 = {S46392, 142, { &P14877,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA820914 = {S46393, 137, { &P18469,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; IdeaTabS2109A-F Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA910581 = {S46394, 137, { &P18469,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IdeaTabS2109A-F Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2394390 = {S46395, 68, { &P18469,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; IdeaTabS2109A-F Build/IMM76D)*/
const Device UA627632 = {S46396, 135, { &P18299,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; PantechP4100 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1955799 = {S46397, 64, { &P18299,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; PantechP4100 Build/HTK75)*/
const Device UA3065130 = {S46398, 130, { &P22606,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2974305 = {S46399, 130, { &P18702,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8013 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974361 = {S46400, 130, { &P18702,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8013 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3065480 = {S46401, 144, { &P18614,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT890 Build/9.8.2I-50_SMI-26) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA899133 = {S46402, 140, { &P20947,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCEVOV4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA907744 = {S46403, 62, { &P20947,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTCEVOV4G Build/IML74K)*/
const Device UA2575278 = {S46404, 146, { &P20947,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCEVOV4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2983345 = {S46405, 138, { &P20947,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; HTCEVOV4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2973904 = {S46406, 138, { &P16356,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974380 = {S46407, 137, { &P16356,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974734 = {S46408, 138, { &P16356,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021383 = {S46409, 138, { &P16356,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022051 = {S46410, 138, { &P16356,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022647 = {S46411, 131, { &P16356,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M480W Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023084 = {S46412, 130, { &P16356,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023530 = {S46413, 137, { &P16356,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3030027 = {S46414, 135, { &P16356,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SC-05D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA574314 = {S46415, 103, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA2192559 = {S46416, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; en-GB) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2192566 = {S46417, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; de-DE) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2200377 = {S46418, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; en-US) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2235302 = {S46419, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; nl-NL) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2337241 = {S46420, 100, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.804.0 Safari/535.1*/
const Device UA2349410 = {S46421, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; en-00) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2350035 = {S46422, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; es-ES) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2366407 = {S46423, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; en-NZ) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2380765 = {S46424, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; en-AU) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2382508 = {S46425, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; en-IE) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2382748 = {S46426, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; it-IT) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2383400 = {S46427, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; pl-PL) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2400870 = {S46428, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; fr-FR) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2420116 = {S46429, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; fr-CA) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2420325 = {S46430, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; da-DK) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2421678 = {S46431, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; es-BR) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2454632 = {S46432, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; cs-CZ) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2455166 = {S46433, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; tr-TR) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2465690 = {S46434, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; es-US) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2480605 = {S46435, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; bg-BG) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2502228 = {S46436, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; fi-FI) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2502461 = {S46437, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; el-GR) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2523851 = {S46438, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; sv-SE) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2524539 = {S46439, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; hr-HR) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2527368 = {S46440, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; ru-RU) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2554056 = {S46441, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; pt-BR) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2554339 = {S46442, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; no-NO) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2593714 = {S46443, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; th-00) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2692406 = {S46444, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; hu-HU) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2693718 = {S46445, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; sk-SK) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2746590 = {S46446, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; pt-PT) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2777071 = {S46447, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; ja-JP) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2840109 = {S46448, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; en-SG) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2844263 = {S46449, 121, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; ro-RO) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA3024911 = {S46450, 126, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; en-GB) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA3025302 = {S46451, 126, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; it-IT) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA3027062 = {S46452, 126, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; en-US) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1*/
const Device UA3027066 = {S46453, 126, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; en-NZ) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1*/
const Device UA3028089 = {S46454, 126, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; en-AU) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1*/
const Device UA3029802 = {S46455, 126, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; nl-NL) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA3030443 = {S46456, 126, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; fr-FR) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA3030445 = {S46457, 126, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; de-DE) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA3058562 = {S46458, 126, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; cs-CZ) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA3058830 = {S46459, 126, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; es-ES) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA3078907 = {S46460, 126, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; en-00) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1*/
const Device UA3078983 = {S46461, 126, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; pl-PL) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA3079063 = {S46462, 126, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; tr-TR) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA3092472 = {S46463, 126, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; en-IE) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA3092527 = {S46464, 126, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; pt-BR) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1*/
const Device UA3107932 = {S46465, 126, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; da-DK) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA3125202 = {S46466, 126, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; sv-SE) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.2 Chrome/14.0.835.202 Safari/535.1*/
const Device UA3125245 = {S46467, 126, { &P15364,&P18145,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; es-BR) AppleWebKit/535.1 (KHTML, like Gecko) Viera/1.5.1 Chrome/14.0.835.202 Safari/535.1*/
const Device UA3064647 = {S46468, 137, { &P18295,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One X Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083592 = {S46469, 134, { &P18295,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; One X Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA638861 = {S46470, 71, { &P15364,&P18110,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:8.0) Gecko/X Firefox/8.0 Cunaguaro/8.0*/
const Device UA678139 = {S46471, 60, { &P15364,&P18110,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0; (X11; Linux x86_64; rv:8.0) Gecko/X Firefox/8.0*/
const Device UA703989 = {S46472, 81, { &P15364,&P18110,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 on x86_64; rv:8.0) Gecko/X Firefox/8.0 Iceweasel/8.0*/
const Device UA2404324 = {S46473, 135, { &P20129,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2429474 = {S46474, 126, { &P20129,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2654416 = {S46475, 126, { &P20129,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build JRO03L) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2719157 = {S46476, 127, { &P20129,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2S Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2733510 = {S46477, 136, { &P20129,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2C Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2735710 = {S46478, 136, { &P20129,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2S Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2751428 = {S46479, 137, { &P20129,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2SC Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781789 = {S46480, 136, { &P20129,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2A Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2845665 = {S46481, 73, { &P20129,&P18111,&P18112,&P18092 }};	/*Useragent - AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; MI 2 Build/JRO03L)*/
const Device UA2864990 = {S46482, 128, { &P20129,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2SC Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2866785 = {S46483, 127, { &P20129,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2A Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3059542 = {S46484, 58, { &P20129,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; MI 2S MIUI/JLB21.0)*/
const Device UA2428512 = {S46485, 144, { &P19842,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HTC_Butterfly Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483660 = {S46486, 144, { &P19842,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HTC Butterfly Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2484902 = {S46487, 66, { &P19842,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC Butterfly Build/JRO03C)*/
const Device UA2578832 = {S46488, 140, { &P19842,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC_X920e Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2701413 = {S46489, 140, { &P19842,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC X920e Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2889569 = {S46490, 62, { &P19842,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC X920e Build/JRO03C)*/
const Device UA1861347 = {S46491, 86, { &P18095,&P18109,&P17371,&P18092 }};	/*Useragent - Opera/9.80 (Series 60; Opera Mini/5.1.22784/23.334; U; en) Presto/2.5.25 Version/10.54*/
const Device UA2431239 = {S46492, 88, { &P18095,&P18109,&P17371,&P18092 }};	/*Useragent - Opera/9.80 (Series 60; Opera Mini/5.1.22783/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA665409 = {S46493, 62, { &P18267,&P5043,&P4037,&P18092 }};	/*Useragent - NokiaX2-02/2.0 (11.57) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2554669 = {S46494, 73, { &P18267,&P5043,&P4037,&P18092 }};	/*Useragent - NokiaX2-02/2.0 (11.79) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-02*/
const Device UA2844403 = {S46495, 125, { &P18267,&P5043,&P4037,&P18092 }};	/*Useragent - NokiaX2-02/2.0 (11.79) Profile/MIDP-2.1 Configuration/CLDC-1.1 NokiaX2-02/2.0 (11.79) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2487372 = {S46496, 145, { &P18614,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT890 Build/9.8.2I-50_SMI-26) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2554835 = {S46497, 132, { &P19879,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I915 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2668258 = {S46498, 61, { &P19879,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-I915 Build/JZO54K)*/
const Device UA2754413 = {S46499, 137, { &P18595,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755531 = {S46500, 137, { &P18595,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2790748 = {S46501, 138, { &P20937,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ADR910L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2792007 = {S46502, 141, { &P20937,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ADR910L 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2870608 = {S46503, 60, { &P20937,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; ADR910L Build/JZO54K)*/
const Device UA454641 = {S46504, 88, { &P17968,&P5629,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.3; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.10.254 Version/12.00*/
const Device UA595682 = {S46505, 88, { &P17968,&P5629,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.5; Linux; Opera Mobi/ADR-X; U; de) Presto/2.10.254 Version/12.00*/
const Device UA613254 = {S46506, 91, { &P17968,&P5629,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.7; Linux; Opera Mobi/ADR-X; U; es-ES) Presto/2.10.254 Version/12.00*/
const Device UA677613 = {S46507, 88, { &P17968,&P5629,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.6; Linux; Opera Mobi/ADY-X; U; ru) Presto/2.10.254 Version/12.00*/
const Device UA840385 = {S46508, 88, { &P17968,&P5629,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.4; Linux; Opera Mobi/ADY-X; U; ru) Presto/2.10.254 Version/12.00*/
const Device UA2026434 = {S46509, 91, { &P17968,&P5629,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.2; Linux; Opera Mobi/ADR-X; U; es-ES) Presto/2.10.254 Version/12.00*/
const Device UA2030013 = {S46510, 88, { &P17968,&P5629,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.9; Linux; Opera Mobi/ADR-X; U; en) Presto/2.10.254 Version/12.00*/
const Device UA2031535 = {S46511, 88, { &P17968,&P5629,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.1; Linux; Opera Mobi/ADR-X; U; en) Presto/2.10.254 Version/12.00*/
const Device UA58528 = {S46512, 126, { &P13783,&P5156,&P3519,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9100; es) AppleWebKit/534.8+ (KHTML, like Gecko) Version/6.0.0.534 Mobile Safari/534.8+*/
const Device UA69469 = {S46513, 73, { &P13783,&P5156,&P3519,&P18092 }};	/*Useragent - BlackBerry9100/6.0.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/331*/
const Device UA3034990 = {S46514, 106, { &P21002,&P17484,&P18369,&P18092 }};	/*Useragent - SAMSUNG-GT-C3312R Opera/9.80 (J2ME/MIDP; Opera Mini/7.0.32584/30.3697; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2028938 = {S46515, 143, { &P19778,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PantechP9090 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2729512 = {S46516, 65, { &P19778,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; PantechP9090 Build/IMM76I)*/
const Device UA659625 = {S46517, 139, { &P13079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; 007HW Build/HuaweiU8850) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA684084 = {S46518, 146, { &P13079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HUAWEI-U8850 Build/HuaweiU8850) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA814658 = {S46519, 139, { &P13079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; 007HW Build/HuaweiU8850) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1988933 = {S46520, 146, { &P13079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HUAWEI-U8850 Build/HuaweiU8850) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024642 = {S46521, 70, { &P13079,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HUAWEI-U8850 Build/HuaweiU8850)*/
const Device UA2031507 = {S46522, 139, { &P13079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; 007HW Build/HuaweiU8850) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2815092 = {S46523, 70, { &P13079,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; HUAWEI-U8850 Build/HuaweiU8850)*/
const Device UA2549 = {S46524, 94, { &P13977,&P5890,&P3439,&P18092 }};	/*Useragent - SoftBank/1.0/705NK/NKJ001 Series60/3.0 NokiaN73/3.0650 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA19218 = {S46525, 82, { &P13977,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN73-2/3.0638.0.0.1_rm132 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA28070 = {S46526, 76, { &P13977,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN73-5/3.0628.0.0.6 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA56177 = {S46527, 89, { &P13977,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN73-1/4.0839.42.2.1Series60/3.0 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64398 = {S46528, 77, { &P13977,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN73-1/4.0839.42.2.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2385477 = {S46529, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.236; en-us; 9700/6.6.0.236) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2403251 = {S46530, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.124; en-us; 9700/6.6.0.124) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2422464 = {S46531, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.248; en-us; 9700/6.6.0.248) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2423057 = {S46532, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.241; en-us; 9700/6.6.0.241) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2426631 = {S46533, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.175; en-us; 9700/5.1.0.175) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2429660 = {S46534, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.207; en-us; 9700/6.6.0.207) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2456016 = {S46535, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.212; en-us; 9700/6.6.0.212) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2458019 = {S46536, 101, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.50; en-us; 9700/6.6.0.50) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2467175 = {S46537, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.195; en-us; 9700/6.6.0.195) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2480652 = {S46538, 101, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.98; en-us; 9700/5.1.0.98) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2481293 = {S46539, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.146; en-us; 9700/5.1.0.146) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2482965 = {S46540, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.246; en-us; 9700/6.6.0.246) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2505127 = {S46541, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.200; en-us; 9700/6.6.0.200) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2505179 = {S46542, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.223; en-us; 9700/6.6.0.223) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2528427 = {S46543, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.216; en-us; 9700/6.6.0.216) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2553670 = {S46544, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.177; en-us; 9700/5.1.0.177) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2554447 = {S46545, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.105; en-us; 9700/6.6.0.105) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2555372 = {S46546, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.137; en-us; 9700/6.6.0.137) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2594418 = {S46547, 101, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.86; en-us; 9700/6.6.0.86) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2595286 = {S46548, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.147; en-us; 9700/5.1.0.147) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2653388 = {S46549, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.153; en-us; 9700/5.1.0.153) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2655522 = {S46550, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.151; en-us; 9700/6.6.0.151) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2694633 = {S46551, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.121; en-us; 9700/5.1.0.121) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2695577 = {S46552, 101, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.63; en-us; 9700/6.6.0.63) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2695760 = {S46553, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.189; en-us; 9700/6.6.0.189) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2696259 = {S46554, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.109; en-us; 9700/6.6.0.109) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2696313 = {S46555, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.116; en-us; 9700/5.1.0.116) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2746840 = {S46556, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.160; en-us; 9700/5.1.0.160) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2791565 = {S46557, 101, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.90; en-us; 9700/5.1.0.90) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2841557 = {S46558, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.165; en-us; 9700/5.1.0.165) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2934947 = {S46559, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.250; en-us; 9700/6.6.0.250) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2951129 = {S46560, 103, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.112; en-us; 9700/5.1.0.112) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3119290 = {S46561, 101, { &P11730,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.91; en-us; 9700/5.1.0.91) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA49906 = {S46562, 127, { &P13105,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; G100W Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA630066 = {S46563, 129, { &P13105,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; G100W Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2022381 = {S46564, 58, { &P13105,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; G100W Build/HTK55D)*/
const Device UA2032328 = {S46565, 140, { &P17691,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I957 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2219125 = {S46566, 133, { &P17691,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I957R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257628 = {S46567, 133, { &P17691,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I957M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2341579 = {S46568, 133, { &P17691,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I957D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2481086 = {S46569, 132, { &P17691,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I957 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2839603 = {S46570, 62, { &P17691,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I957R Build/IMM76D)*/
const Device UA3099673 = {S46571, 69, { &P17691,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SAMSUNG-SGH-I957 Build/IMM76D)*/
const Device UA2436487 = {S46572, 139, { &P12280,&P17090,&P22900,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.2 Mobile/8H7 Safari/6533.18.5*/
const Device UA2468955 = {S46573, 139, { &P12280,&P17090,&P22900,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA2469933 = {S46574, 139, { &P12280,&P17090,&P22900,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.2 Mobile/8J2 Safari/6533.18.5*/
const Device UA741098 = {S46575, 140, { &P13109,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; HTC-X710a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034246 = {S46576, 62, { &P13109,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC-X710a Build/IML74K)*/
const Device UA43523 = {S46577, 142, { &P13004,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SO-02C Build/3.0.1.F.0.131) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43526 = {S46578, 154, { &P13004,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonSO-02C Build/3.0.1.F.0.131) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43529 = {S46579, 153, { &P13004,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonSO-02C Build/3.0.1.F.0.99) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59505 = {S46580, 142, { &P13004,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SO-02C Build/3.0.1.F.0.126) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59507 = {S46581, 154, { &P13004,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonSO-02C Build/3.0.1.F.0.126) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA576019 = {S46582, 152, { &P13004,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-02C Build/4.0.1.C.1.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626440 = {S46583, 153, { &P13004,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-02C Build/4.0.1.C.1.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627535 = {S46584, 141, { &P13004,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SO-02C Build/4.0.1.C.1.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA680563 = {S46585, 140, { &P13004,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SO-02C Build/4.0.1.C.1.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1896479 = {S46586, 66, { &P13004,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SO-02C Build/3.0.1.F.0.131)*/
const Device UA2002571 = {S46587, 141, { &P13004,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SO-02C Build/4.0.1.C.1.24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018018 = {S46588, 153, { &P13004,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-02C Build/4.0.1.C.1.24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019459 = {S46589, 83, { &P13004,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonSO-02C Build/3.0.1.F.0.28)*/
const Device UA2033276 = {S46590, 65, { &P13004,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SO-02C Build/4.0.1.C.1.24)*/
const Device UA23662 = {S46591, 127, { &P16598,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; A500 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA316122 = {S46592, 56, { &P16598,&P6545,&P4755,&P18092 }};	/*Useragent - Dalvik/1.5.0 (Linux; U; Android 3.0.1; A500 Build/HRI66)*/
const Device UA713661 = {S46593, 134, { &P16598,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; he-il; A500 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA368825 = {S46594, 132, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Touchpad Build/HLK75F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA673813 = {S46595, 132, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Touchpad Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1971042 = {S46596, 134, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TOUCHPAD 7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1996283 = {S46597, 138, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CnM-TOUCHPAD 7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029954 = {S46598, 145, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CnM-TOUCHPAD 9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2218679 = {S46599, 133, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Touchpad7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257335 = {S46600, 137, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; CnM-TOUCHPAD7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257758 = {S46601, 140, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; CnM Touchpad 9.7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2384678 = {S46602, 144, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; CnM-TOUCHPAD9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2404535 = {S46603, 144, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VS TOUCHPAD 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422978 = {S46604, 148, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Versus TouchPad 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426365 = {S46605, 145, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; CnM TOUCHPAD 9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2437486 = {S46606, 145, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; CnM TouchPad 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2449843 = {S46607, 67, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; CnM-TOUCHPAD 7 Build/IML74K)*/
const Device UA2463939 = {S46608, 144, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VS-TOUCHPAD 9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467114 = {S46609, 141, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; TOUCHPAD 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2504660 = {S46610, 143, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Versus Touchpad 9.7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2505616 = {S46611, 145, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; CnM-TOUCHPAD 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2535605 = {S46612, 139, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Vs TOUCHPAD 9.7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2561943 = {S46613, 148, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Versus TouchPad 9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2688226 = {S46614, 142, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; Blueron TouchPad10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2697383 = {S46615, 70, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Versus TouchPad 7 Build/IMM76D)*/
const Device UA2748346 = {S46616, 147, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; CnM Touchpad 4.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2847859 = {S46617, 67, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; CnM TouchPad 7 Build/IMM76D)*/
const Device UA3099017 = {S46618, 69, { &P16044,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; CnM Touchpad 9.7 Build/IMM76D)*/
const Device UA2670624 = {S46619, 137, { &P18645,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-I747 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675401 = {S46620, 129, { &P18645,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I747 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2675513 = {S46621, 137, { &P18645,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I747 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2675850 = {S46622, 129, { &P18645,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L710 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676275 = {S46623, 144, { &P18645,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I747 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676448 = {S46624, 136, { &P18645,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I747 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676981 = {S46625, 136, { &P18645,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L710 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676996 = {S46626, 137, { &P18645,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530U Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677180 = {S46627, 145, { &P18645,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SAMSUNG-SGH-I747 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708867 = {S46628, 137, { &P18645,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SPH-L710 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2719438 = {S46629, 137, { &P18645,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9305 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2861222 = {S46630, 137, { &P18645,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530C Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA874480 = {S46631, 21, { &P15767,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPad1C1/902.206*/
const Device UA2367077 = {S46632, 22, { &P15767,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPad2C1/1001.523*/
const Device UA2385888 = {S46633, 22, { &P15767,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPad2C2/1001.523*/
const Device UA2386201 = {S46634, 22, { &P15767,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPad3C1/1001.523*/
const Device UA2403957 = {S46635, 22, { &P15767,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPad3C4/1001.523*/
const Device UA2428404 = {S46636, 23, { &P15767,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPad3C6/1001.8426*/
const Device UA2668838 = {S46637, 22, { &P15767,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPad2C1/1002.329*/
const Device UA2668849 = {S46638, 22, { &P15767,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPad2C5/1002.329*/
const Device UA2669617 = {S46639, 22, { &P15767,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPad3C3/1002.329*/
const Device UA2669719 = {S46640, 22, { &P15767,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPad3C4/1002.329*/
const Device UA2670652 = {S46641, 22, { &P15767,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPad2C2/1002.329*/
const Device UA2675223 = {S46642, 22, { &P15767,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPad3C1/1002.329*/
const Device UA2703625 = {S46643, 22, { &P15767,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPad2C4/1002.329*/
const Device UA2704050 = {S46644, 22, { &P15767,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPad3C6/1002.329*/
const Device UA2767123 = {S46645, 22, { &P15767,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPad2C7/1002.329*/
const Device UA2858099 = {S46646, 67, { &P15767,&P17779,&P17596,&P18092 }};	/*Useragent - gamed/4.10.17.1.6.13.5.2.1 (iPad2,5; 6.1.3; 10B329; GameKit-781.18)*/
const Device UA49000 = {S46647, 150, { &P12082,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; SonyEricssonX10a Build/R2CA016) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2877019 = {S46648, 80, { &P15364,&P7133,&P24329,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:22.0) Gecko/X Firefox/22.0 SeaMonkey/2.19*/
const Device UA2962877 = {S46649, 67, { &P15364,&P7133,&P24329,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:22.0) Gecko/X SeaMonkey/2.19*/
const Device UA3018995 = {S46650, 85, { &P15364,&P7133,&P24329,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:22.0) Gecko/X Firefox/22.0 SeaMonkey/2.19*/
const Device UA2676347 = {S46651, 140, { &P16362,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710BST Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676491 = {S46652, 137, { &P16362,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676789 = {S46653, 130, { &P16362,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2677165 = {S46654, 141, { &P16362,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710VMUB Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735360 = {S46655, 137, { &P16362,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SPH-D710 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA31144 = {S46656, 81, { &P13451,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-B3410R/B3410UXJC4 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62063 = {S46657, 73, { &P13451,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-B3410/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA896218 = {S46658, 140, { &P22592,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I717R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031566 = {S46659, 62, { &P22592,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I717R Build/IMM76D)*/
const Device UA2549432 = {S46660, 138, { &P22592,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SGH-I717R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2882693 = {S46661, 140, { &P22394,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6603 Build/10.3.A.0.423) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3026687 = {S46662, 135, { &P22394,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6603 Build/10.3.1.A.0.244) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2509384 = {S46663, 59, { &P15364,&P18110,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; Linux i686; Trident/3.0)*/
const Device UA2658841 = {S46664, 74, { &P15364,&P18110,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; Linux x86_64; Win64; IA64; Trident/5.0)*/
const Device UA2791723 = {S46665, 66, { &P15364,&P18110,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; Linux i686; WOW64; Trident/5.0)*/
const Device UA2791871 = {S46666, 64, { &P15364,&P18110,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; Linux i686; SV1; Trident/5.0)*/
const Device UA2849346 = {S46667, 70, { &P15364,&P18110,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; Linux i686; Tablet PC; Trident/3.0)*/
const Device UA252394 = {S46668, 137, { &P15364,&P17819,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Ubuntu/11.10 Chromium/15.0.874.106 Chrome/15.0.874.106 Safari/535.2*/
const Device UA255438 = {S46669, 135, { &P15364,&P17819,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.2 (KHTML, like Gecko) Ubuntu/11.10 Chromium/15.0.874.120 Chrome/15.0.874.120 Safari/535.2*/
const Device UA2221446 = {S46670, 135, { &P15364,&P17819,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.2 (KHTML	 like Gecko) Ubuntu/11.10 Chromium/15.0.874.120 Chrome/15.0.874.120 Safari/535.2*/
const Device UA2755143 = {S46671, 62, { &P17976,&P17979,&P22773,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:24.0) Gecko/24.0 Firefox/24.0*/
const Device UA3017351 = {S46672, 68, { &P17976,&P17979,&P22773,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:24.0a2) Gecko/24.0a2 Firefox/24.0a2*/
const Device UA3108419 = {S46673, 72, { &P17976,&P17979,&P22773,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:24.0) Gecko/24.0 Firefox/24.0,gzip(gfe)*/
const Device UA2206719 = {S46674, 101, { &P18623,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 510)*/
const Device UA2482601 = {S46675, 101, { &P18623,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Nokia 510)*/
const Device UA2582575 = {S46676, 109, { &P18623,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 510; Orange)*/
const Device UA3027877 = {S46677, 111, { &P12989,&P5843,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia303/14.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA18077 = {S46678, 133, { &P14011,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; U8500 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40739 = {S46679, 146, { &P14011,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-tw; HUAWEI_IDEOS_U8500 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41549 = {S46680, 139, { &P14011,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; U8500 Build/HuaweiU8500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57273 = {S46681, 145, { &P14011,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-in; HuaweiU8500 Build/HuaweiU8500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57279 = {S46682, 139, { &P14011,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pl-pl; U8500 Build/HuaweiU8500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA876241 = {S46683, 137, { &P14011,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-ph; ICE Build/HuaweiU8500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018436 = {S46684, 57, { &P14011,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; U8500 Build/FRF91)*/
const Device UA2019248 = {S46685, 63, { &P14011,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; U8500 Build/HuaweiU8500)*/
const Device UA2020654 = {S46686, 141, { &P14011,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ru-by; MTC Evo Build/HuaweiU8500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022296 = {S46687, 152, { &P14011,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; HUAWEI_IDEOS_U8500 Build/HuaweiU8500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022441 = {S46688, 140, { &P14011,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-kh; MF8503 Build/HuaweiU8500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA144168 = {S46689, 136, { &P15364,&P5690,&P21929,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729) chromeframe/5.0.317.0*/
const Device UA571811 = {S46690, 169, { &P15364,&P5690,&P21929,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET4.0C; .NET4.0E; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729) chromeframe/5.0.317.0*/
const Device UA1850524 = {S46691, 141, { &P15364,&P5690,&P21929,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729) chromeframe/5.0.317.0*/
const Device UA2857283 = {S46692, 143, { &P20399,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PantechP8010 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2886371 = {S46693, 65, { &P20399,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; PantechP8010 Build/JZO54K)*/
const Device UA3093855 = {S46694, 171, { &P20399,&P18111,&P18112,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.1.2;dev=oscar;devm=PantechP8010;man=PANTECH;brw=Android;shwd=540x888;shwa=540x851;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA53266 = {S46695, 82, { &P11742,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-GT-C5212i/C5212iJPKD1 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62414 = {S46696, 77, { &P11742,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-C5212/C5212DDIK1 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2204810 = {S46697, 126, { &P18501,&P6590,&P4223,&P18092 }};	/*Useragent - NetFront/3.5.1(BREW 5.0.2.1; U; en-us; SAMSUNG; NetFront/3.5.1/WAP) Boost M390 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2509382 = {S46698, 62, { &P15364,&P18110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Linux x86_64; Trident/2.0)*/
const Device UA2658767 = {S46699, 69, { &P15364,&P18110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Linux x86_64; WOW64; Trident/2.0)*/
const Device UA2658945 = {S46700, 65, { &P15364,&P18110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Linux i686; SV1; Trident/2.0)*/
const Device UA2775285 = {S46701, 70, { &P15364,&P18110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; X11; U; Linux x86_64; Trident/6.0)*/
const Device UA2791688 = {S46702, 75, { &P15364,&P18110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Linux x86_64; Win64; IA64; Trident/2.0)*/
const Device UA2791873 = {S46703, 71, { &P15364,&P18110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Linux i686; Tablet PC; Trident/4.0)*/
const Device UA3064974 = {S46704, 129, { &P22228,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SM-T210 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3065150 = {S46705, 130, { &P22228,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SM-T210R Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA815465 = {S46706, 124, { &P15364,&P17285,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19 AvantBrowser/Tri-Core*/
const Device UA911754 = {S46707, 123, { &P15364,&P17285,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11 AvantBrowser/Tri-Core*/
const Device UA2040414 = {S46708, 121, { &P15364,&P17285,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 AvantBrowser/Tri-Core*/
const Device UA2235745 = {S46709, 114, { &P15364,&P17285,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Avant TriCore) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2359228 = {S46710, 114, { &P15364,&P17285,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Avant TriCore) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.96 Safari/537.4*/
const Device UA2433553 = {S46711, 116, { &P15364,&P17285,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Avant TriCore) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2473528 = {S46712, 121, { &P15364,&P17285,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5 AvantBrowser/Tri-Core*/
const Device UA2564493 = {S46713, 116, { &P15364,&P17285,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Avant TriCore) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2671070 = {S46714, 117, { &P15364,&P17285,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Avant TriCore) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2710021 = {S46715, 116, { &P15364,&P17285,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Avant TriCore) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2877751 = {S46716, 117, { &P15364,&P17285,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36*/
const Device UA2968054 = {S46717, 116, { &P15364,&P17285,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.71 Safari/537.36*/
const Device UA3097278 = {S46718, 116, { &P15364,&P17285,&P22736,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Avant TriCore) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36*/
const Device UA912242 = {S46719, 113, { &P18506,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia305/03.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.13*/
const Device UA2743961 = {S46720, 114, { &P18506,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia3050/03.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.13*/
const Device UA3043597 = {S46721, 174, { &P18506,&P5043,&P18273,&P18092 }};	/*Useragent - Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Series40; Nokia305/03.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.13*/
const Device UA2338833 = {S46722, 135, { &P21012,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I815 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2399266 = {S46723, 132, { &P21012,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I815 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2595282 = {S46724, 61, { &P21012,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-I815 Build/IMM76D)*/
const Device UA184734 = {S46725, 69, { &P17595,&P17296,&P18333,&P18092 }};	/*Useragent - iTunes/10.2.2 (Macintosh; Intel Mac OS X 10.6.7) AppleWebKit/533.21.1*/
const Device UA664590 = {S46726, 66, { &P17595,&P17296,&P18333,&P18092 }};	/*Useragent - iTunes/10.1.1 (Macintosh; Intel Mac OS X 10.6.1) AppleWebKit/531.9*/
const Device UA742783 = {S46727, 69, { &P17595,&P17296,&P18333,&P18092 }};	/*Useragent - iTunes/10.0.1 (Macintosh; Intel Mac OS X 10.6.4) AppleWebKit/533.18.1*/
const Device UA1873621 = {S46728, 69, { &P17595,&P17296,&P18333,&P18092 }};	/*Useragent - iTunes/10.0.1 (Macintosh; Intel Mac OS X 10.6.6) AppleWebKit/533.18.1*/
const Device UA2041866 = {S46729, 67, { &P17595,&P17296,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Macintosh; Intel Mac OS X 10.6.8) AppleWebKit/534.57.2*/
const Device UA2127744 = {S46730, 67, { &P17595,&P17296,&P18333,&P18092 }};	/*Useragent - iTunes/10.1 (Macintosh; Intel Mac OS X 10.6.5) AppleWebKit/533.18.1*/
const Device UA2370018 = {S46731, 57, { &P17595,&P17296,&P18333,&P18092 }};	/*Useragent - iTunes/9.0.3 (Macintosh; U; Intel Mac OS X 10_6_2; en-ca)*/
const Device UA2393623 = {S46732, 70, { &P17595,&P17296,&P18333,&P18092 }};	/*Useragent - iTunes/10.6.3 (Macintosh; Intel Mac OS X 10.6.3) AppleWebKit/531.21.11*/
const Device UA2428939 = {S46733, 59, { &P17595,&P17296,&P18333,&P18092 }};	/*Useragent - iTunes/11.0.1 (Macintosh; OS X 10.6.8) AppleWebKit/534.57.2*/
const Device UA370248 = {S46734, 142, { &P14186,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633525 = {S46735, 147, { &P14186,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S5830 Build/ICECREAMSANDWICH) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA690782 = {S46736, 147, { &P14186,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-gb; GT-S5830 Build/ICECREAMSANDWICH) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA812661 = {S46737, 147, { &P14186,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-S5830 Build/ICECREAMSANDWICH) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1948174 = {S46738, 71, { &P14186,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 4.0.1; GT-S5830 Build/ICECREAMSANDWICH)*/
const Device UA1969796 = {S46739, 142, { &P14186,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028554 = {S46740, 142, { &P14186,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; es-es; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028749 = {S46741, 145, { &P14186,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; es-; GT-S5830 Build/ICECREAMSANDWICH) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2169482 = {S46742, 140, { &P14186,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S5830 Build/JELLYBEAN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2522491 = {S46743, 139, { &P14186,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S5830 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2545119 = {S46744, 144, { &P14186,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.5; en-us; S5830/Galaxy Ace Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2703900 = {S46745, 143, { &P14186,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S5830 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA764775 = {S46746, 76, { &P17595,&P18207,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:7.0.1) Gecko/X Firefox/7.0.1*/
const Device UA2462459 = {S46747, 112, { &P19747,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia206/03.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.36*/
const Device UA2560884 = {S46748, 114, { &P19747,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia206.1/03.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.36*/
const Device UA3088441 = {S46749, 173, { &P19747,&P5043,&P18567,&P18092 }};	/*Useragent - Nokia206/2.0 (03.59) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Series40; Nokia206/03.59; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.36*/
const Device UA2148161 = {S46750, 42, { &P17346,&P17779,&P17470,&P18092 }};	/*Useragent - server-bag [iPhone OS,5.0.1,9A405,iPod4,1]*/
const Device UA2163702 = {S46751, 42, { &P17346,&P17779,&P17470,&P18092 }};	/*Useragent - server-bag [iPhone OS,5.1.1,9B206,iPod4,1]*/
const Device UA2219738 = {S46752, 38, { &P17346,&P17779,&P17470,&P18092 }};	/*Useragent - Apple iPod v5.1.1 YouTube v1.0.0.9B206*/
const Device UA2586884 = {S46753, 40, { &P17346,&P17779,&P17470,&P18092 }};	/*Useragent - server-bag [iPhone OS,4.3.3,8J2,iPod4,1]*/
const Device UA2669291 = {S46754, 65, { &P17346,&P17779,&P17470,&P18092 }};	/*Useragent - Appcelerator Titanium/1.7.2 (iPod touch/6.1.3; iPhone OS; en_US;)*/
const Device UA2874146 = {S46755, 102, { &P15364,&P17188,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2753088 = {S46756, 136, { &P18713,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P769 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753282 = {S46757, 136, { &P18713,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753328 = {S46758, 136, { &P18713,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P768 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2896309 = {S46759, 136, { &P18713,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LGMS769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3037410 = {S46760, 133, { &P18466,&P18110,&P24339,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Safari/535.19 Silk-Accelerated=false*/
const Device UA912253 = {S46761, 78, { &P15364,&P5690,&P22578,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; Edition Next; en) Presto/2.11.310 Version/12.50*/
const Device UA2697577 = {S46762, 140, { &P19878,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I497 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2943991 = {S46763, 69, { &P19878,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-I497 Build/JZO54K)*/
const Device UA2673372 = {S46764, 136, { &P21648,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2719297 = {S46765, 129, { &P21648,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA1362 = {S46766, 101, { &P11628,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E250/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA5300 = {S46767, 106, { &P11628,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E250-VODA/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA53973 = {S46768, 109, { &P11628,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E250i-ORANGE/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA64673 = {S46769, 115, { &P11628,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E250-ORANGE/E250DBVHE2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA81575 = {S46770, 121, { &P11628,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-SGH-E250V/BUGB2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA88774 = {S46771, 121, { &P11628,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-SGH-E250i/BAIC2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA88785 = {S46772, 122, { &P11628,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-SGH-E250V/DBUHC1/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA94082 = {S46773, 122, { &P11628,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-SGH-E250V/DBUGK1/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA442783 = {S46774, 121, { &P11628,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-SGH-E250V/BUGD1/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA679775 = {S46775, 109, { &P11628,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E250i/E250iJAKD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA2263267 = {S46776, 121, { &P11628,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-SGH-E250V/AMGC1/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA63062 = {S46777, 84, { &P20126,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK800iv/R1JC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64742 = {S46778, 83, { &P20126,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK800i/R8BF Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA3064771 = {S46779, 137, { &P20146,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065154 = {S46780, 138, { &P20146,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2509627 = {S46781, 59, { &P15364,&P18110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Linux i686; Trident/1.0)*/
const Device UA2580053 = {S46782, 66, { &P15364,&P18110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Linux x86_64; SV1; Trident/4.0)*/
const Device UA2581069 = {S46783, 68, { &P15364,&P18110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Linux x86_64; WOW64; Trident/2.0)*/
const Device UA2658783 = {S46784, 74, { &P15364,&P18110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Linux x86_64; Win64; IA64; Trident/2.0)*/
const Device UA2792686 = {S46785, 72, { &P15364,&P18110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Linux x86_64; Tablet PC; Trident/3.0)*/
const Device UA2385750 = {S46786, 146, { &P18614,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT890 Build/8.7.1I-110_IFW-39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2042225 = {S46787, 191, { &P19976,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.3 NokiaE6-00/111.140.0058; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.1.4 Mobile Safari/535.1 3gpp-gba*/
const Device UA2676274 = {S46788, 149, { &P18317,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300T Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676840 = {S46789, 127, { &P18317,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; TF300T Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2712921 = {S46790, 150, { &P18317,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300TL Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2718006 = {S46791, 150, { &P18317,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF300TG Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA199889 = {S46792, 129, { &P17687,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; ja-jp; MZ604 Build/H.6.4-15) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA236792 = {S46793, 129, { &P17687,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; MZ604 Build/H.6.4-20) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA330633 = {S46794, 58, { &P17687,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; MZ604 Build/H.6.4-20)*/
const Device UA436816 = {S46795, 131, { &P17687,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; de-de; MZ604 Build/H.6.4-20-3) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA560168 = {S46796, 129, { &P17687,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-au; MZ604 Build/H.6.4-16) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2223385 = {S46797, 137, { &P13644,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PC36100 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2238892 = {S46798, 133, { &P13644,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; EVO Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2402824 = {S46799, 137, { &P13644,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PC36100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2421408 = {S46800, 133, { &P13644,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2832900 = {S46801, 140, { &P13644,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EVO 4G LTE Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2563039 = {S46802, 147, { &P18512,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT881 Build/9.8.3O-118_USC-14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666606 = {S46803, 69, { &P18512,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT881 Build/9.8.3O-118_USC-14)*/
const Device UA3028235 = {S46804, 147, { &P18512,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT881 Build/9.8.3O-118_USC-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2404777 = {S46805, 139, { &P18593,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; GT-S6010 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556546 = {S46806, 140, { &P18593,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; GT-S6010L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2846024 = {S46807, 61, { &P18593,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-S6010 Build/IMM76D)*/
const Device UA28079 = {S46808, 140, { &P15065,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID PRO Build/V2_2.26.99) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42855 = {S46809, 189, { &P15065,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-XT610/Blur_Version.0.22.2.XT610.Personal.en.AR Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; XT610 Build/0.22.0_R02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70793 = {S46810, 189, { &P15065,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-XT610/Blur_Version.0.26.1.XT610.Movistar.en.01 Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; XT610 Build/0.26.0_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73200 = {S46811, 141, { &P15065,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; XT610 Build/V2U_3.4.2-154) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76416 = {S46812, 141, { &P15065,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; XT610 Build/V2U_3.4.2-179) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90436 = {S46813, 141, { &P15065,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID PRO Build/S275) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA151615 = {S46814, 139, { &P15065,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID PRO Build/V2_2.26.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA159711 = {S46815, 142, { &P15065,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROID PRO Build/V2_2.26.60) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA178769 = {S46816, 146, { &P15065,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROID PRO Build/3.4.2_80_V2_87) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA317922 = {S46817, 70, { &P15065,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; DROID PRO Build/3.4.2_80_V2_87)*/
const Device UA389706 = {S46818, 146, { &P15065,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROID PRO Build/3.4.2_80_V2_86) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA559884 = {S46819, 146, { &P15065,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROID PRO Build/3.4.2_80_V2_85) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA805291 = {S46820, 188, { &P15065,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-XT610/Blur_Version.0.17.62.XT610.Latam.en.01 Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; XT610 Build/0.17.0_R062) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA869800 = {S46821, 193, { &P15065,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-XT610/Blur_Version.0.17.9.XT610.AmericaMovil.en.MX Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; XT610 Build/0.17.0_R09) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA895477 = {S46822, 193, { &P15065,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-XT610/Blur_Version.0.23.1.XT610.AmericaMovil.en.01 Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; XT610 Build/0.23.0_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019428 = {S46823, 140, { &P15065,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROID PRO Build/V2.26.61) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA613243 = {S46824, 82, { &P17976,&P17979,&P18160,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:10.0.4) Gecko/10.0.4 Firefox/10.0.4 Fennec/10.0.4*/
const Device UA900544 = {S46825, 82, { &P17976,&P17979,&P18160,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:10.0.5) Gecko/10.0.5 Firefox/10.0.5 Fennec/10.0.5*/
const Device UA903088 = {S46826, 132, { &P15932,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P3110 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2007541 = {S46827, 132, { &P15932,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3110 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2024890 = {S46828, 133, { &P15932,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  GT-P3110 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2613590 = {S46829, 133, { &P15932,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  GT-P3110 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA349267 = {S46830, 154, { &P12896,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P999 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA422370 = {S46831, 166, { &P12896,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P999 Build/GRI40; CyanogenMod-Eaglesblood_build) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030532 = {S46832, 169, { &P12896,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P999 Build/GWK74; CyanogenMod-HellFire Phoenix 2.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2589728 = {S46833, 175, { &P12896,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P999 Build/GRJ22; CyanogenMod-7.2.0; LG-P990 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3026778 = {S46834, 101, { &P15364,&P17188,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA2179430 = {S46835, 143, { &P18292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MZ609 4G Build/7.7.1-85_MZ609-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2235817 = {S46836, 140, { &P18292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MZ609 Build/7.7.1-85_MZ609-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2455328 = {S46837, 69, { &P18292,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MZ609 Build/7.7.1-85_MZ609-31)*/
const Device UA2816633 = {S46838, 143, { &P18292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MZ609 4G Build/7.7.1-85_MZ609-27) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2368866 = {S46839, 132, { &P20107,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMP5580C Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2655497 = {S46840, 61, { &P20107,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; PMP5580C Build/IMM76I)*/
const Device UA43866 = {S46841, 156, { &P13941,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; S7 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 S7V100R001C13B021*/
const Device UA58931 = {S46842, 145, { &P13941,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; HUAWEI-S7 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA63777 = {S46843, 144, { &P13941,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Ideos S7 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA368632 = {S46844, 151, { &P13941,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HUAWEI Ideos S7 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA368974 = {S46845, 135, { &P13941,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; S7 Build/4749) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA396527 = {S46846, 135, { &P13941,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; S7 Build/4431) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA409583 = {S46847, 135, { &P13941,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; S7 Build/4626) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA497520 = {S46848, 66, { &P13941,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Ideos S7 Build/ERE27)*/
const Device UA643323 = {S46849, 135, { &P13941,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; S7 Build/4078) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018598 = {S46850, 60, { &P13941,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; S7 Build/ERE27)*/
const Device UA2019830 = {S46851, 135, { &P13941,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; S7 Build/3987) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2261699 = {S46852, 135, { &P13941,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; cs-cz; S7 Build/4625) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2476167 = {S46853, 135, { &P13941,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; cs-cz; S7 Build/4657) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2566483 = {S46854, 131, { &P22043,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P5113 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2633638 = {S46855, 131, { &P22043,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P5113 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA827413 = {S46856, 133, { &P22043,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  GT-P5113 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA900903 = {S46857, 132, { &P22043,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P5113 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA912476 = {S46858, 132, { &P22043,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P5113 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2391494 = {S46859, 75, { &P15364,&P5690,&P20457,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:17.0) Gecko/X Firefox/17.0 SeaMonkey/2.14.1*/
const Device UA2407250 = {S46860, 82, { &P15364,&P5690,&P20457,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:17.0) Gecko/X Firefox/17.0 SeaMonkey/2.14.1*/
const Device UA2426055 = {S46861, 62, { &P15364,&P5690,&P20457,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:17.0) Gecko/X SeaMonkey/2.14.1*/
const Device UA792183 = {S46862, 112, { &P12989,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia303/14.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6*/
const Device UA2467164 = {S46863, 113, { &P12989,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia303/14.87; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.62.10*/
const Device UA2199634 = {S46864, 139, { &P18686,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A810K Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2232268 = {S46865, 139, { &P18686,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A810S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2353320 = {S46866, 61, { &P18686,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-A810S Build/IMM76I)*/
const Device UA2383307 = {S46867, 61, { &P18686,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-A810K Build/IMM76I)*/
const Device UA2425978 = {S46868, 137, { &P18686,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A810S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2502206 = {S46869, 137, { &P18686,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A810K Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2678365 = {S46870, 149, { &P18686,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A810K/A810K2.14 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2706223 = {S46871, 157, { &P18686,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A810K Build/VEGAVIET.COM ROM ICS V1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2764471 = {S46872, 155, { &P18686,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A810S Build/VEGAVIET.COM ROM ICS V1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2779882 = {S46873, 155, { &P18686,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A810S Build/VEGAVIET.COM ROM ICS V2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2795884 = {S46874, 147, { &P18686,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A810K/A810K2.14 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2995404 = {S46875, 142, { &P18686,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A810S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30*/
const Device UA3047513 = {S46876, 155, { &P18686,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A810S Build/VEGAVIET.COM ROM ICS V4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA3107603 = {S46877, 152, { &P18686,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A810K/A810K2.14 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30*/
const Device UA2713045 = {S46878, 136, { &P22585,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L720 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2715809 = {S46879, 129, { &P22585,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L720 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA42905 = {S46880, 143, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC_Flyer_P510e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58832 = {S46881, 143, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_Flyer_P510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61085 = {S46882, 142, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_Flyer_P512 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370620 = {S46883, 67, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.4; en_US; HTC Flyer Build/GRJ22)*/
const Device UA429982 = {S46884, 143, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Flyer P510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA429998 = {S46885, 66, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Flyer P512 Build/GRI40)*/
const Device UA430001 = {S46886, 67, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Flyer P510e Build/GRI40)*/
const Device UA430029 = {S46887, 67, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Flyer P510e Build/GRJ22)*/
const Device UA430065 = {S46888, 131, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC_Flyer Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA430079 = {S46889, 145, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_Flyer_P512_NA Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA430080 = {S46890, 69, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC_Flyer_P512_NA Build/GRJ22)*/
const Device UA430083 = {S46891, 142, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Flyer P512 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA430120 = {S46892, 137, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; HTC_Flyer Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA430124 = {S46893, 66, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC Flyer P512 Build/GRJ22)*/
const Device UA430131 = {S46894, 137, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; HTC Flyer Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA647470 = {S46895, 142, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_Flyer_P512 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA747411 = {S46896, 137, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Flyer Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023497 = {S46897, 131, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Flyer P510e Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2024534 = {S46898, 137, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; HTC_Flyer Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027469 = {S46899, 144, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us-; HTC Flyer P510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2597337 = {S46900, 142, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Flyer P510e Build/GRJ22) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2597658 = {S46901, 140, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTCFlyerP510e Build/GRJ22) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2658102 = {S46902, 139, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTCFlyerP512 Build/GRI40) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2658472 = {S46903, 141, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Flyer P512 Build/GRI40) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2659280 = {S46904, 135, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTCFlyer Build/GRJ22) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2849895 = {S46905, 140, { &P12532,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTCFlyerP510e Build/GRI40) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA6529 = {S46906, 132, { &P18088,&P17295,&P18325,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7;en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17 Skyfire/2.0*/
const Device UA9700 = {S46907, 133, { &P18088,&P17295,&P18325,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17 Skyfire/2.0*/
const Device UA42301 = {S46908, 132, { &P18088,&P17295,&P18325,&P18092 }};	/*Useragent - Mozilla/5.0(Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17 Skyfire/2.0*/
const Device UA741207 = {S46909, 133, { &P18088,&P17295,&P18325,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.18 (KHTML, like Gecko) Version/4.0 Safari/528.17 Skyfire/2.0*/
const Device UA2489836 = {S46910, 112, { &P18644,&P5043,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia112/03.32; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2973947 = {S46911, 130, { &P15932,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974166 = {S46912, 130, { &P15932,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA898864 = {S46913, 115, { &P15364,&P7133,&P22458,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5*/
const Device UA899161 = {S46914, 122, { &P15364,&P7133,&P22458,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5*/
const Device UA2509668 = {S46915, 146, { &P15767,&P19826,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/7534.48.3 TeamStream HD (iPad2,1 284 1.7)*/
const Device UA2532856 = {S46916, 148, { &P15767,&P19826,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 TeamStream HD (iPad2,1 284 1.7)*/
const Device UA2604567 = {S46917, 150, { &P15767,&P19826,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 TeamStream HD (iPad2,1 336 1.8.1)*/
const Device UA2566639 = {S46918, 131, { &P22037,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3113 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2633689 = {S46919, 131, { &P22037,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3113 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA3062708 = {S46920, 139, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; v89_zlh_hd Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3080082 = {S46921, 132, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; R821 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080540 = {S46922, 127, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; M9 pro Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080904 = {S46923, 129, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; M470BSE Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080928 = {S46924, 160, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; 06_v89_jbla768_asx_s4lockscreen Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081099 = {S46925, 130, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; AT10LE-A Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3081122 = {S46926, 132, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; S510 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081479 = {S46927, 132, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; W8s Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3082229 = {S46928, 137, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; STAIRWAY Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083148 = {S46929, 144, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Lenovo S820_ROW Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083498 = {S46930, 141, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; v89_jbl1a668 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083655 = {S46931, 123, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; M6 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3093629 = {S46932, 138, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ZTE V967S Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093680 = {S46933, 128, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ME302KL Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3093662 = {S46934, 131, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; X50D Build/X50D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094271 = {S46935, 133, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; A240 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094519 = {S46936, 126, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; U55GT Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094684 = {S46937, 132, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; CW-V88-QUAD Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094727 = {S46938, 126, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; EPAD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094833 = {S46939, 136, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9197 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094849 = {S46940, 131, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; DLX Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094851 = {S46941, 133, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; W200 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094872 = {S46942, 137, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SM-T217A Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3108622 = {S46943, 136, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; IMO s89 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3109371 = {S46944, 137, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; DARKSIDE Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109616 = {S46945, 161, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nocturnal Google Edition CDMA 5.0 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109749 = {S46946, 131, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Q5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110303 = {S46947, 148, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; 1080P-N003 Build/1080P1080P-N003) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110340 = {S46948, 134, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; i6000 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110733 = {S46949, 127, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; M7 pro Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3110844 = {S46950, 149, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; MyPhone Agua Iceberg Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110842 = {S46951, 140, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; v89_gq2008s Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111014 = {S46952, 141, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; v89_jbl1a698 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111244 = {S46953, 129, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; M470BSS Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3111481 = {S46954, 144, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; AUXUS Nuclea N1 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111520 = {S46955, 127, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; B1-711 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3111525 = {S46956, 133, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; 7102 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111529 = {S46957, 137, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC_PO582 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111648 = {S46958, 136, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; IMO S89 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3112164 = {S46959, 134, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; I9389 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3112885 = {S46960, 133, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; B943 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3113024 = {S46961, 134, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ZP990 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3113179 = {S46962, 136, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; TCL S950 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3113322 = {S46963, 135, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; HY5001 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3113518 = {S46964, 130, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; AT10PE-A Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3113638 = {S46965, 150, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; 01v21_v89_jbl1a698_2g Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3113799 = {S46966, 127, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; p4wifi Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3113805 = {S46967, 129, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; U30GT C4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3114075 = {S46968, 152, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; HUAWEI G700-U00 Build/HuaweiG700-U00) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114082 = {S46969, 136, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R960 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114304 = {S46970, 135, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ThL W9 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114356 = {S46971, 137, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; POMP W89 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114795 = {S46972, 142, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GSmart Aku A1 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114922 = {S46973, 138, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy-S4 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3115044 = {S46974, 128, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SM-T215 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3115067 = {S46975, 140, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Lenovo S750 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126227 = {S46976, 135, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; VOW V5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126234 = {S46977, 136, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Glory 2 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126269 = {S46978, 133, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; S5-F Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126459 = {S46979, 145, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337Z Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126482 = {S46980, 135, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; enrc2b Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126523 = {S46981, 138, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; M-PP2G530 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126560 = {S46982, 129, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5220 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126564 = {S46983, 131, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; S4 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126669 = {S46984, 129, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Slate 21 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126735 = {S46985, 128, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ifiveX2 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126878 = {S46986, 134, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Ban-G Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126984 = {S46987, 133, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; H920 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126987 = {S46988, 126, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; M9pro Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3127012 = {S46989, 146, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; 01v21_v89_gq2008s Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127115 = {S46990, 144, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Novo10 Catpain QuadCore Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3127129 = {S46991, 151, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ALCATEL ONE TOUCH 6040D Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127133 = {S46992, 134, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; H3088 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127270 = {S46993, 132, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; OZZY Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127266 = {S46994, 136, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; HS-U970 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127398 = {S46995, 133, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; T600 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127448 = {S46996, 131, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; V973 Core4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3127655 = {S46997, 138, { &P15364,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASP-5000H Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2510259 = {S46998, 61, { &P15364,&P18110,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; Linux x86_64; Trident/2.0)*/
const Device UA2700364 = {S46999, 66, { &P15364,&P18110,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; Linux x86_64; SV1; Trident/1.0)*/
const Device UA2791864 = {S47000, 68, { &P15364,&P18110,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; Linux x86_64; WOW64; Trident/1.0)*/
const Device UA2792331 = {S47001, 73, { &P15364,&P18110,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; Linux x86_64; Win64; x64; Trident/5.0)*/
const Device UA2792545 = {S47002, 72, { &P15364,&P18110,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; Linux x86_64; Tablet PC; Trident/5.0)*/
const Device UA2974449 = {S47003, 141, { &P16362,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710VMUB Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974478 = {S47004, 140, { &P16362,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710BST Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974600 = {S47005, 137, { &P16362,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2026914 = {S47006, 139, { &P19854,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-R760 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2476162 = {S47007, 61, { &P19854,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-R760 Build/IMM76D)*/
const Device UA2431361 = {S47008, 147, { &P18465,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Amazon Kindle Fire Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2537952 = {S47009, 148, { &P18465,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amazon Kindle Fire Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2731158 = {S47010, 149, { &P18465,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amazon Kindle Fire2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2847757 = {S47011, 69, { &P18465,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2; Amazon Kindle Fire Build/JOP40C)*/
const Device UA2855316 = {S47012, 70, { &P18465,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; Amazon Kindle Fire Build/JDQ39)*/
const Device UA2507956 = {S47013, 160, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813/C8813V100R001C92B159; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA2530310 = {S47014, 139, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI C8813 Build/HuaweiC8813) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2557623 = {S47015, 160, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813/C8813V100R001C92B162; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA2558297 = {S47016, 148, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI C8813 Build/HuaweiC8813) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558571 = {S47017, 160, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813/C8813V100R001C92B167; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA2563907 = {S47018, 160, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813/C8813V100R001C92B166; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA2659494 = {S47019, 139, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI C8813 Build HuaweiC8813) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2668484 = {S47020, 162, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813D/C8813DV100R001C92B168; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA2670060 = {S47021, 160, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813/C8813V100R001C92B169; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA2700838 = {S47022, 70, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HUAWEI C8813 Build/HuaweiC8813)*/
const Device UA2704481 = {S47023, 162, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813D/C8813DV100R001C92B166; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA2736804 = {S47024, 162, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813D/C8813DV100R001C92B172; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA2766356 = {S47025, 150, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HUAWEI C8813D Build/HuaweiC8813D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2796922 = {S47026, 162, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_C8813Q/C8813QV100R001C92B186; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA2861815 = {S47027, 72, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HUAWEI C8813D Build/HuaweiC8813D)*/
const Device UA2866793 = {S47028, 162, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_C8813Q/C8813QV100R001C92B185; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA2870925 = {S47029, 163, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813D/C8813DV100R001C419B172; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA2878482 = {S47030, 141, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI C8813D Build/HuaweiC8813D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2878710 = {S47031, 150, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI C8813Q Build/HuaweiC8813Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879385 = {S47032, 162, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_C8813Q/C8813QV100R001C92B189; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA2942487 = {S47033, 160, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813/C8813V100R001C92B173; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA2957620 = {S47034, 141, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI C8813Q Build/HuaweiC8813Q) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3001462 = {S47035, 72, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HUAWEI C8813Q Build/HuaweiC8813Q)*/
const Device UA3006120 = {S47036, 162, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_C8813Q/C8813QV100R001C92B191; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA3020450 = {S47037, 162, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_C8813Q/C8813QV100R001C92B193; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA3050131 = {S47038, 141, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI C8813Q Build HuaweiC8813Q) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA3108144 = {S47039, 160, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813/C8813V100R001C92B175; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA3119769 = {S47040, 160, { &P21351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HW-HUAWEI_C8813/C8813V100R001C92B168; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA360699 = {S47041, 109, { &P11689,&P5776,&P24663,&P18092 }};	/*Useragent - NokiaN70-1/5.0737.3.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.9.1.120/27/351/UCWEB*/
const Device UA584052 = {S47042, 108, { &P11689,&P5776,&P24663,&P18092 }};	/*Useragent - NokiaN70/ 5.0741.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.9.1.120/27/351/UCWEB*/
const Device UA911255 = {S47043, 140, { &P22590,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I717M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026980 = {S47044, 62, { &P22590,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I717M Build/IMM76D)*/
const Device UA2548918 = {S47045, 138, { &P22590,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SGH-I717M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3064648 = {S47046, 138, { &P23771,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA98141 = {S47047, 45, { &P15364,&P6704,&P4812,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows 98; US) Opera 10.00 [en]*/
const Device UA433382 = {S47048, 58, { &P15364,&P6704,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows 98; U; en) Presto/2.2.15 Version/10.00*/
const Device UA767503 = {S47049, 58, { &P15364,&P6704,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows 98; U; en) Presto/2.2.15 Version/10.01*/
const Device UA2161804 = {S47050, 62, { &P15364,&P6704,&P4812,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; en) Opera 10.00*/
const Device UA2638060 = {S47051, 101, { &P15364,&P17838,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1.7) Gecko/X Ubuntu/9.10 (karmic) Firefox/3.5.7 GTB6*/
const Device UA3064778 = {S47052, 144, { &P22972,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I537 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA662931 = {S47053, 64, { &P15364,&P18146,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; SunOS i86pc; rv:10.0.3) Gecko/X Firefox/10.0.3*/
const Device UA50502 = {S47054, 154, { &P13286,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID2 Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola DROID2*/
const Device UA68136 = {S47055, 153, { &P13286,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID2 Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 480X854 motorola DROID2*/
const Device UA68274 = {S47056, 153, { &P13286,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2; en-us; DROID2 Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola DROID2*/
const Device UA68273 = {S47057, 153, { &P13286,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2; en-us; DROID2 Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola DROID2*/
const Device UA68442 = {S47058, 153, { &P13286,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID2Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola DROID2*/
const Device UA90020 = {S47059, 137, { &P13286,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID2 Build/VZW) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA157854 = {S47060, 132, { &P13286,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROID2 Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA285329 = {S47061, 54, { &P13286,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; DROID2 Build/VZW)*/
const Device UA2018150 = {S47062, 132, { &P13286,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID2WE Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019475 = {S47063, 132, { &P13286,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA609514 = {S47064, 147, { &P13064,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  SAMSUNG-SGH-I727 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA872675 = {S47065, 147, { &P13064,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA903215 = {S47066, 146, { &P13064,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA904370 = {S47067, 146, { &P13064,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I727 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA912054 = {S47068, 138, { &P13064,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I727 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA48418 = {S47069, 163, { &P12706,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; LG-P690 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA48421 = {S47070, 169, { &P12706,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LG-P690 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA48422 = {S47071, 163, { &P12706,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; LG-P690 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA48423 = {S47072, 174, { &P12706,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; LG-P690/${Version} Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA682382 = {S47073, 123, { &P12706,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; LG-P690 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA752259 = {S47074, 59, { &P12706,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P690 Build/GRJ22)*/
const Device UA2033344 = {S47075, 160, { &P12706,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ar; LG-P690 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2402612 = {S47076, 158, { &P12706,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P690 Build/GRI40; neko95@xda-developers) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2649904 = {S47077, 167, { &P12706,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; LG-P690-Net Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA16913 = {S47078, 175, { &P15118,&P5055,&P3804,&P18092 }};	/*Useragent - NokiaX7-00.1/011.519 (Symbian/3; Series60/5.2 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.7.2 3gpp-gba*/
const Device UA33472 = {S47079, 187, { &P15118,&P5055,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaX7-00/021.013; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.28 Mobile Safari/533.4 3gpp-gba*/
const Device UA41313 = {S47080, 189, { &P15118,&P5055,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.2 NokiaX7-00.1/022.008; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.31 Mobile Safari/533.4 3gpp-gba*/
const Device UA2708599 = {S47081, 73, { &P16329,&P22583,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PLAYSTATION 3 4.41) AppleWebKit/531.22.8 (KHTML, like Gecko)*/
const Device UA641457 = {S47082, 174, { &P13157,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Sensation XE with Beats Audio Z715e Build/GRJ22)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA643660 = {S47083, 176, { &P13157,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Sensation XE with Beats Audio Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Mobile Maxthon*/
const Device UA2022297 = {S47084, 191, { &P13157,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Sensation XE with Beats Audio Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Mobile Maxthon Mobile Maxthon*/
const Device UA511654 = {S47085, 87, { &P15364,&P5690,&P20917,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.1b3pre) Gecko/X Firefox/3.1b3pre*/
const Device UA756386 = {S47086, 78, { &P15364,&P5690,&P20917,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; it; rv:1.9.1b2) Gecko/X Firefox/3.1b2*/
const Device UA1845063 = {S47087, 93, { &P15364,&P5690,&P20917,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1 ; x64; en-US; rv:1.9.1b2pre) Gecko/X Firefox/3.1b2pre*/
const Device UA2166985 = {S47088, 139, { &P14959,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9103 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2377802 = {S47089, 61, { &P14959,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9103 Build/IMM76D)*/
const Device UA2813187 = {S47090, 142, { &P14959,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2951019 = {S47091, 137, { &P14959,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-I9103 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752974 = {S47092, 138, { &P12972,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754329 = {S47093, 137, { &P12972,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755078 = {S47094, 138, { &P12972,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989D Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2756001 = {S47095, 137, { &P12972,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879357 = {S47096, 145, { &P12972,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-T989 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2032387 = {S47097, 126, { &P13031,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; blaze Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2603086 = {S47098, 154, { &P18466,&P18066,&P21708,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2670392 = {S47099, 75, { &P17595,&P17296,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8) Presto/2.12.388 Version/12.15*/
const Device UA2795263 = {S47100, 86, { &P17595,&P17296,&P22577,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6.8; rv:2.0) Gecko/X Firefox/4.0 Opera 12.15*/
const Device UA2509252 = {S47101, 59, { &P15364,&P18110,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; Linux i686; Trident/1.0)*/
const Device UA2580078 = {S47102, 73, { &P15364,&P18110,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; Linux x86_64; Win64; x64; Trident/3.0)*/
const Device UA2700277 = {S47103, 66, { &P15364,&P18110,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; Linux i686; WOW64; Trident/4.0)*/
const Device UA2791906 = {S47104, 64, { &P15364,&P18110,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; Linux i686; SV1; Trident/3.0)*/
const Device UA2851095 = {S47105, 70, { &P15364,&P18110,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; Linux i686; Tablet PC; Trident/1.0)*/
const Device UA50220 = {S47106, 156, { &P17951,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Milestone Build/SHOLS_U2_02.31.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA50264 = {S47107, 156, { &P17951,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; Milestone Build/SHOLS_U2_02.34.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA50322 = {S47108, 156, { &P17951,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; Milestone Build/SHOLS_U2_03.10.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA50380 = {S47109, 156, { &P17951,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; Milestone Build/SHOLS_U2_03.11.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA50381 = {S47110, 156, { &P17951,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Milestone Build/SHOLS_U2_02.36.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA53560 = {S47111, 156, { &P17951,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Milestone Build/SHOLS_U2_02.27.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA56062 = {S47112, 156, { &P17951,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; Milestone Build/SHOLS_U2_02.36.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA56122 = {S47113, 156, { &P17951,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; Milestone Build/SHOLS_U2_02.38.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA56320 = {S47114, 148, { &P17951,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; Milestone Build/SGC03040) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA56800 = {S47115, 156, { &P17951,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; it-it; Milestone Build/SHOLS_U2_02.38.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA67253 = {S47116, 156, { &P17951,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; Milestone Build/SHOLS_U2_02.38.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA67415 = {S47117, 156, { &P17951,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; he-il; Milestone Build/SHOLS_U2_02.27.C) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA67760 = {S47118, 156, { &P17951,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; Milestone Build/SHOLS_U2_02.34.3) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA478283 = {S47119, 156, { &P17951,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-ru; Milestone Build/SHOLS_U2_02.27.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA683804 = {S47120, 78, { &P17951,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Milestone Build/SHOLS_U2_02.31.0)*/
const Device UA767721 = {S47121, 156, { &P17951,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Milestone Build/SHOLS_U2_02.27.B) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA819619 = {S47122, 156, { &P17951,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Milestone Build/SHOLS_U2_02.31.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1844413 = {S47123, 156, { &P17951,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-ru; Milestone Build/SHOLS_U2_02.27.9) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1861932 = {S47124, 148, { &P17951,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; Milestone Build/SHOLS_U2_02.36.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018339 = {S47125, 156, { &P17951,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Milestone Build/SHOLS_U2_04.08.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2020939 = {S47126, 134, { &P17951,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; Milestone Build/SHOLS_U2_02.36.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA646325 = {S47127, 111, { &P18254,&P17285,&P3409,&P18091 }};	/*Useragent - Mozilla/5.0 (Windows; Crawler; U; Windows NT 6.0; en-US; rv:1.9.0.7) Gecko/X Firefox/3.0.7 (.NET CLR 3.5.30729)*/
const Device UA351026 = {S47128, 132, { &P18601,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N762 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1939052 = {S47129, 56, { &P18601,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; N762 Build/GRJ22)*/
const Device UA68027 = {S47130, 169, { &P14911,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Milestone X Build/RETAIL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola Milestone X*/
const Device UA68207 = {S47131, 167, { &P14911,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Milestone X Build/AWCC) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola Milestone X*/
const Device UA68208 = {S47132, 169, { &P14911,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Milestone X Build/RETAIL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola Milestone X*/
const Device UA68481 = {S47133, 167, { &P14911,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Milestone X Build/NTEL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola Milestone X*/
const Device UA68835 = {S47134, 165, { &P14911,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Milestone X Build/CS) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola Milestone X*/
const Device UA181916 = {S47135, 167, { &P14911,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Milestone X Build/NTEL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola Milestone X*/
const Device UA2019131 = {S47136, 64, { &P14911,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; Milestone X Build/RETAIL)*/
const Device UA2753705 = {S47137, 136, { &P22597,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA196549 = {S47138, 95, { &P17669,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; T7575)*/
const Device UA2413148 = {S47139, 97, { &P17669,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Nokia; T7575)*/
const Device UA2026939 = {S47140, 147, { &P18626,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I547 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2343491 = {S47141, 140, { &P18626,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I547C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2481053 = {S47142, 69, { &P18626,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SAMSUNG-SGH-I547 Build/IMM76D)*/
const Device UA2695194 = {S47143, 62, { &P18626,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I547C Build/IMM76D)*/
const Device UA2507982 = {S47144, 152, { &P15803,&P5843,&P24676,&P18092 }};	/*Useragent - Nokia200/2.0 (11.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia200) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2974044 = {S47145, 130, { &P22037,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3113 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3024575 = {S47146, 130, { &P22037,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3113 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2418217 = {S47147, 131, { &P16976,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT300SE Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2437355 = {S47148, 129, { &P16976,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT300 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2974083 = {S47149, 149, { &P18505,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MB886 Build/9.8.2Q-8_MB886_FFW-11) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974273 = {S47150, 150, { &P18505,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA710893 = {S47151, 49, { &P15364,&P22310,&P17201,&P18092 }};	/*Useragent - Mozilla/1.22 (compatible; MSIE 10.0; Windows 3.1)*/
const Device UA2792796 = {S47152, 57, { &P15364,&P22310,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Win3.11; Trident/2.0)*/
const Device UA2566802 = {S47153, 139, { &P12968,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26i Build/6.1.A.2.55) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2766704 = {S47154, 132, { &P12968,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26i Build/6.1.A.2.45) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA1949978 = {S47155, 115, { &P15364,&P16110,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/534.6 (KHTML, like Gecko) Chrome/7.0.503.0 Safari/534.6*/
const Device UA1990356 = {S47156, 105, { &P15364,&P16110,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.28 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.28*/
const Device UA2708849 = {S47157, 136, { &P22597,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2937019 = {S47158, 129, { &P22597,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3121618 = {S47159, 137, { &P22597,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545L Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA234683 = {S47160, 142, { &P17595,&P17296,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Instapaper/4.0 (+http://www.instapaper.com/)*/
const Device UA458608 = {S47161, 138, { &P18320,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; Padfone Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1972223 = {S47162, 138, { &P18320,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; PadFone Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026726 = {S47163, 131, { &P18320,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PadFone Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033491 = {S47164, 60, { &P18320,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; PadFone Build/IML74K)*/
const Device UA2178418 = {S47165, 131, { &P18320,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; PadFone Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2224590 = {S47166, 138, { &P18320,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; PadFone Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2242281 = {S47167, 60, { &P18320,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; PadFone Build/IMM76I)*/
const Device UA46821 = {S47168, 215, { &P15487,&P5267,&P3834,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5750E-Orange/S5750EAFKE1; U; Bada/1.0; fr-fr) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 NexPlayer/3.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B*/
const Device UA2672372 = {S47169, 140, { &P18453,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT25i Build/9.1.A.1.140) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1963623 = {S47170, 145, { &P19855,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; XT885 Build/6.7.3-135_YIC-8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2232430 = {S47171, 145, { &P19855,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; XT885 Build/6.7.3-179_YIC-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2449931 = {S47172, 67, { &P19855,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; XT885 Build/6.7.3-135_YIC-8)*/
const Device UA2689765 = {S47173, 149, { &P19855,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT885 Build/6.7.2_GC-200-YTZU-6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2821020 = {S47174, 149, { &P19855,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT885 Build/6.7.2_GC-180-YTZU-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA909263 = {S47175, 139, { &P18002,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A760S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1982788 = {S47176, 61, { &P18002,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-A760S Build/IMM76D)*/
const Device UA2157099 = {S47177, 137, { &P18002,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A760S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2418352 = {S47178, 137, { &P18002,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A760S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2579086 = {S47179, 143, { &P18002,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A760SXPERIA Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2763432 = {S47180, 145, { &P18002,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A760S_Optimus Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2811480 = {S47181, 142, { &P18002,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; IM-A760S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2850270 = {S47182, 147, { &P18002,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A760S/A770K FIX Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2985709 = {S47183, 143, { &P18002,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A760S/A770K Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2974732 = {S47184, 130, { &P18639,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 10 Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3034484 = {S47185, 59, { &P18639,&P21637,&P23997,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.3; Nexus 10 Build/JWR66Y)*/
const Device UA3035681 = {S47186, 130, { &P18639,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 10 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3039394 = {S47187, 130, { &P18639,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 10 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3063543 = {S47188, 130, { &P18639,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 10 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA30027 = {S47189, 138, { &P12280,&P17145,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Twitter for iPhone*/
const Device UA30028 = {S47190, 139, { &P12280,&P17145,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148a Twitter for iPhone*/
const Device UA171626 = {S47191, 138, { &P12280,&P17145,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E200 Twitter for iPhone*/
const Device UA182769 = {S47192, 138, { &P12280,&P17145,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_7 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E303 Twitter for iPhone*/
const Device UA194217 = {S47193, 138, { &P12280,&P17145,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E401 Twitter for iPhone*/
const Device UA236446 = {S47194, 139, { &P12280,&P17145,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_10 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E600 Twitter for iPhone*/
const Device UA304778 = {S47195, 138, { &P12280,&P17145,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E128 Twitter for iPhone*/
const Device UA329878 = {S47196, 138, { &P12280,&P17145,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_9 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E501 Twitter for iPhone*/
const Device UA387724 = {S47197, 136, { &P12280,&P17145,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2 like Mac OS X; nl-nl) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C134 Twitter for iPhone*/
const Device UA2974796 = {S47198, 145, { &P18125,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3025450 = {S47199, 111, { &P18506,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia305/05.92; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA49526 = {S47200, 143, { &P13048,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; GT-S5660M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA452506 = {S47201, 67, { &P13048,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-S5660M Build/GINGERBREAD)*/
const Device UA2257126 = {S47202, 143, { &P21063,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ONE_TOUCH_960C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2777977 = {S47203, 67, { &P21063,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; ONE_TOUCH_960C Build/GRK39F)*/
const Device UA50281 = {S47204, 145, { &P13215,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I5700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA50518 = {S47205, 149, { &P13215,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; GT-I5700 Build/ECLAIR) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA53573 = {S47206, 146, { &P13215,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; GT-I5700R Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA61321 = {S47207, 146, { &P13215,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; GT-I5700L Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA67824 = {S47208, 137, { &P13215,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-gb; GT-I5700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA133332 = {S47209, 67, { &P13215,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I5700 Build/ECLAIR)*/
const Device UA507843 = {S47210, 150, { &P13215,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-ca; GT-I5700R Build/ECLAIR) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2018254 = {S47211, 141, { &P13215,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; fr-fr; GT-I5700 Build/ECLAIR) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2019530 = {S47212, 131, { &P13215,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; GT-I5700 Build/ECLAIR) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA441432 = {S47213, 125, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.12.12934.201*/
const Device UA457796 = {S47214, 193, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.12.12934.201*/
const Device UA643679 = {S47215, 186, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.12.12934.201*/
const Device UA686164 = {S47216, 123, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.7 (KHTML, like Gecko) Chrome/20.0.1099.0 Safari/536.7 QQBrowser/6.12.13631.201*/
const Device UA818681 = {S47217, 191, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0) QQBrowser/6.12.12934.201*/
const Device UA1967134 = {S47218, 165, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C) QQBrowser/6.12.12934.201*/
const Device UA1991685 = {S47219, 162, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.12.12934.201,gzip(gfe) (via translate.google.com)*/
const Device UA2162851 = {S47220, 79, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; QQBrowser/6.12)*/
const Device UA2163174 = {S47221, 71, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; SV1; QQBrowser/6.12)*/
const Device UA2326562 = {S47222, 211, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; eSobiSubscriber 1.0.0.40; BRI/2; MAAR; .NET4.0C) QQBrowser/6.12.12934.201*/
const Device UA2330397 = {S47223, 71, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; SV1; QQBrowser/6.12)*/
const Device UA2332762 = {S47224, 79, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; QQBrowser/6.12)*/
const Device UA2395861 = {S47225, 113, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)*/
const Device UA2413186 = {S47226, 167, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/6.12)*/
const Device UA2554320 = {S47227, 113, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)*/
const Device UA2653165 = {S47228, 79, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; QQBrowser/6.12)*/
const Device UA2695957 = {S47229, 114, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)*/
const Device UA2745903 = {S47230, 194, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; QQBrowser/6.12)*/
const Device UA2822447 = {S47231, 187, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; QQBrowser/6.12)*/
const Device UA2833229 = {S47232, 72, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.1; SV1; QQBrowser/6.12)*/
const Device UA2982481 = {S47233, 194, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; QQBrowser/6.12)*/
const Device UA2982667 = {S47234, 80, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; QQBrowser/6.12)*/
const Device UA3098865 = {S47235, 167, { &P15364,&P7133,&P24195,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/6.12)*/
const Device UA2216397 = {S47236, 68, { &P15364,&P18110,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Linux 2.6.26-1-amd64) Lobo/0.98.3*/
const Device UA2509392 = {S47237, 61, { &P15364,&P18110,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Linux x86_64; Trident/1.0)*/
const Device UA2661532 = {S47238, 68, { &P15364,&P18110,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Linux x86_64; WOW64; Trident/4.0)*/
const Device UA2792612 = {S47239, 64, { &P15364,&P18110,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Linux i686; SV1; Trident/1.0)*/
const Device UA2792645 = {S47240, 73, { &P15364,&P18110,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Linux x86_64; Win64; x64; Trident/3.0)*/
const Device UA2793093 = {S47241, 70, { &P15364,&P18110,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Linux i686; Tablet PC; Trident/1.0)*/
const Device UA2436624 = {S47242, 130, { &P15767,&P17090,&P22900,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA2436730 = {S47243, 130, { &P15767,&P17090,&P22900,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.2 Mobile/8H7 Safari/6533.18.5*/
const Device UA2437364 = {S47244, 130, { &P15767,&P17090,&P22900,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.2 Mobile/8J2 Safari/6533.18.5*/
const Device UA2455509 = {S47245, 139, { &P19854,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-R760 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2560471 = {S47246, 140, { &P19854,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R760X Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2703367 = {S47247, 62, { &P19854,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-R760X Build/JZO54K)*/
const Device UA2840226 = {S47248, 139, { &P19854,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-R760 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2670897 = {S47249, 130, { &P18595,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2673093 = {S47250, 130, { &P18595,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676176 = {S47251, 137, { &P18595,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676259 = {S47252, 137, { &P18595,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2586645 = {S47253, 145, { &P21611,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA403385 = {S47254, 128, { &P18239,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A510 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1967494 = {S47255, 57, { &P18239,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; A510 Build/IML74K)*/
const Device UA2974155 = {S47256, 137, { &P20146,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974454 = {S47257, 138, { &P20146,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA33269 = {S47258, 139, { &P12888,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; M9300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA452355 = {S47259, 63, { &P12888,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; M9300 Build/GINGERBREAD)*/
const Device UA436121 = {S47260, 51, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - SexyShelves/9.02.1 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA452497 = {S47261, 52, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - ClassifiedAds/1.7.7 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA624188 = {S47262, 47, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - GQMagazine/1.0 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA650199 = {S47263, 53, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - AtomicBrowser/6.0.1 CFNetwork/485.12.30 Darwin/10.4.0*/
const Device UA657388 = {S47264, 54, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - HOT%20HD%20WPs/9.19.1 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA670163 = {S47265, 54, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - OfflinePagesPro/1.4.1 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA676921 = {S47266, 44, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - Terra/1.3.6 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA709724 = {S47267, 58, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - Celeb%20Wallpapers/9.13.1 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA721933 = {S47268, 47, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - Clipish/9.29.1 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA759751 = {S47269, 58, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - 2%20Million%20WPs/9.16.1 CFNetwork/485.12.30 Darwin/10.4.0*/
const Device UA766781 = {S47270, 54, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - EpicWallPaper2011/1.6 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA793575 = {S47271, 52, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - InstapaperPro/4.0.1 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA843838 = {S47272, 49, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - WOW%20WPs/9.26.1 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA907764 = {S47273, 51, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - Google/2.0.1.10455 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA909203 = {S47274, 57, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - ImageSearcherFreeS/1.0.5 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA1861752 = {S47275, 50, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - 360%20Browser/2.1 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA1897261 = {S47276, 50, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - AtomicLite/5.6.0 CFNetwork/485.12.30 Darwin/10.4.0*/
const Device UA1939360 = {S47277, 53, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - ImageSearcherS/2.1.1 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA1988534 = {S47278, 54, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - Perfect%20Browser/7.2 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA1994118 = {S47279, 51, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - Mercury3Free/6.0.1 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA2128319 = {S47280, 44, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - AdSheet/1.0 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA2161144 = {S47281, 56, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - Hot%20Wallpapers/9.28.1 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA2162813 = {S47282, 54, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - Downloader%20copy/1.0 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA2164079 = {S47283, 47, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - Mercury3/6.0.1 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA2220011 = {S47284, 52, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - Love%20Pictures/3.6 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA2242348 = {S47285, 50, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - Funny%20Pics/2.92 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA2285850 = {S47286, 48, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - ImageSearch/1.4 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA2333863 = {S47287, 47, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - ysearch2/3.0.2 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA2413192 = {S47288, 43, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - Images/1.4 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA2413645 = {S47289, 46, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - browser/31759 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA2419787 = {S47290, 46, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - Twitter/4.3.2 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA2439993 = {S47291, 48, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - Holidays/9.13.1 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA2454576 = {S47292, 45, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - LinkedIn/5.1 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA2570567 = {S47293, 92, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - curl/7.21.2 (x86_64-apple-darwin10.4.0) libcurl/7.21.2 OpenSSL/1.0.1c zlib/1.2.7 libidn/1.19*/
const Device UA2641977 = {S47294, 51, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - iDownloaderPro/2.0 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA2742061 = {S47295, 47, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - Wallpapers/1.6 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA2819059 = {S47296, 54, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - Pic%20Collage/2.20.21 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA2821744 = {S47297, 48, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - 123people/3.1.1 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA2980157 = {S47298, 56, { &P15364,&P18575,&P17470,&P18092 }};	/*Useragent - ImageSearcherProS/2.1.2 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA3033374 = {S47299, 80, { &P17595,&P18207,&P19806,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:17.0) Gecko/X Thunderbird/17.0.8*/
const Device UA2566445 = {S47300, 150, { &P18317,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300T Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566521 = {S47301, 151, { &P18317,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300TG Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567388 = {S47302, 151, { &P18317,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300TL Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2860876 = {S47303, 145, { &P18317,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Transformer Pad TF300T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA1879517 = {S47304, 117, { &P15364,&P16110,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/534.12 (KHTML, like Gecko) Chrome/9.0.576.0 Safari/534.12*/
const Device UA59853 = {S47305, 139, { &P17947,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC_Lexikon Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2670823 = {S47306, 134, { &P15364,&P16110,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/26.0.1410.43) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/537.31*/
const Device UA2675122 = {S47307, 97, { &P15364,&P16110,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/26.0.1410.64)*/
const Device UA2678089 = {S47308, 134, { &P15364,&P16110,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/26.0.1410.64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2679562 = {S47309, 90, { &P15364,&P16110,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; chromeframe/26.0.1410.64)*/
const Device UA2709882 = {S47310, 96, { &P15364,&P16110,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/26.0.1410.64)*/
const Device UA2716187 = {S47311, 176, { &P15364,&P16110,&P21919,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; chromeframe/26.0.1410.64)*/
const Device UA2716700 = {S47312, 127, { &P15364,&P16110,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; chromeframe/26.0.1410.64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2948373 = {S47313, 118, { &P15364,&P17285,&P22763,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1*/
const Device UA2969415 = {S47314, 125, { &P15364,&P17285,&P22763,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.1.0.4000 Chrome/26.0.1410.43 Safari/537.1*/
const Device UA612702 = {S47315, 133, { &P22037,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  GT-P3113 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA901600 = {S47316, 132, { &P22037,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P3113 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA912473 = {S47317, 132, { &P22037,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3113 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2385140 = {S47318, 103, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.236; en-us; 9300/6.6.0.236) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2385268 = {S47319, 103, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.207; en-us; 9300/6.6.0.207) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2385544 = {S47320, 103, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.212; en-us; 9300/6.6.0.212) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2423052 = {S47321, 103, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.195; en-us; 9300/6.6.0.195) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2425635 = {S47322, 103, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.124; en-us; 9300/6.6.0.124) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2426531 = {S47323, 101, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.3.0.63; en-us; 9300/6.3.0.63) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2502747 = {S47324, 101, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.3.0.39; en-us; 9300/6.3.0.39) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2504864 = {S47325, 103, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.241; en-us; 9300/6.6.0.241) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2505295 = {S47326, 103, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.223; en-us; 9300/6.6.0.223) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2528529 = {S47327, 103, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.248; en-us; 9300/6.6.0.248) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2555371 = {S47328, 103, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.246; en-us; 9300/6.6.0.246) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2578437 = {S47329, 101, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.3.0.23; en-us; 9300/6.3.0.23) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2653940 = {S47330, 103, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.200; en-us; 9300/6.6.0.200) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2659303 = {S47331, 101, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.86; en-us; 9300/6.6.0.86) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2695020 = {S47332, 103, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.105; en-us; 9300/6.6.0.105) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2727479 = {S47333, 103, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.216; en-us; 9300/6.6.0.216) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2728173 = {S47334, 101, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.3.0.31; en-us; 9300/6.3.0.31) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2843627 = {S47335, 101, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.50; en-us; 9300/6.6.0.50) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2950950 = {S47336, 103, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.137; en-us; 9300/6.6.0.137) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2955360 = {S47337, 103, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.250; en-us; 9300/6.6.0.250) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3044047 = {S47338, 101, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.3.0.57; en-us; 9300/6.3.0.57) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3118212 = {S47339, 103, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.151; en-us; 9300/6.6.0.151) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3118348 = {S47340, 101, { &P15722,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.3.0.34; en-us; 9300/6.3.0.34) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA256050 = {S47341, 99, { &P15364,&P5690,&P17193,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727) yolinkBot*/
const Device UA430252 = {S47342, 106, { &P15364,&P5690,&P17193,&P18091 }};	/*Useragent - SNK Screenshot Bot/0.20 ( abuse at savenkeep dot_com ) (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0)*/
const Device UA2292234 = {S47343, 106, { &P15364,&P5690,&P17193,&P18091 }};	/*Useragent - Peeplo Screenshot Bot/0.20 ( abuse at peeplo dot_com ) (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0)*/
const Device UA906109 = {S47344, 198, { &P18220,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.4 Nokia808PureView/112.020.0309; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.2.1.22 Mobile Safari/535.1 3gpp-gba*/
const Device UA2205903 = {S47345, 198, { &P18220,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.5 Nokia808PureView/113.010.1507; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.2.21 Mobile Safari/535.1 3gpp-gba*/
const Device UA2653335 = {S47346, 182, { &P18220,&P24389,&P3804,&P18092 }};	/*Useragent - Nokia808/113.010.1507 (Symbian/3; Series60/5.5 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) Version/3.0 NokiaBrowser/8.3.2.21 3gpp-gba*/
const Device UA2428127 = {S47347, 139, { &P19845,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE N9120 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2756728 = {S47348, 43, { &P15364,&P18570,&P18157,&P18092 }};	/*Useragent - Puffin/9073 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2783873 = {S47349, 50, { &P15364,&P18570,&P18157,&P18092 }};	/*Useragent - Puffin%20Free/9090 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2203630 = {S47350, 112, { &P18506,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia305/03.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2387894 = {S47351, 111, { &P18506,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia305/03.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5*/
const Device UA3044221 = {S47352, 173, { &P18506,&P5043,&P18567,&P18092 }};	/*Useragent - Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Series40; Nokia305/03.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.31*/
const Device UA3064653 = {S47353, 139, { &P17172,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094186 = {S47354, 133, { &P17172,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Transformer Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3115058 = {S47355, 127, { &P17172,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TF101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2509455 = {S47356, 59, { &P15364,&P18110,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Linux i686; Trident/5.0)*/
const Device UA2580054 = {S47357, 68, { &P15364,&P18110,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Linux x86_64; WOW64; Trident/5.0)*/
const Device UA2581303 = {S47358, 72, { &P15364,&P18110,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Linux i686; Win64; IA64; Trident/2.0)*/
const Device UA2792786 = {S47359, 72, { &P15364,&P18110,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Linux x86_64; Tablet PC; Trident/1.0)*/
const Device UA2792803 = {S47360, 66, { &P15364,&P18110,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Linux x86_64; SV1; Trident/1.0)*/
const Device UA2947367 = {S47361, 165, { &P18442,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3016897 = {S47362, 157, { &P18442,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3030578 = {S47363, 157, { &P18442,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3031224 = {S47364, 157, { &P18442,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3031484 = {S47365, 157, { &P18442,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3031691 = {S47366, 157, { &P18442,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3032696 = {S47367, 158, { &P18442,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3032888 = {S47368, 157, { &P18442,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3059598 = {S47369, 158, { &P18442,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA616677 = {S47370, 131, { &P18699,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID8042 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2642927 = {S47371, 60, { &P18699,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; MID8042 Build/IML74K)*/
const Device UA33213 = {S47372, 144, { &P12885,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SAMSUNG-SGH-I857 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1939935 = {S47373, 68, { &P12885,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; SAMSUNG-SGH-I857 Build/FROYO)*/
const Device UA2606169 = {S47374, 140, { &P18719,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C1505 Build/11.3.A.0.47) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2676235 = {S47375, 138, { &P18280,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676488 = {S47376, 131, { &P18280,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676645 = {S47377, 154, { &P18280,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Special Edition Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676978 = {S47378, 138, { &P18280,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712626 = {S47379, 138, { &P18280,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC VLE_U Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2768177 = {S47380, 131, { &P18280,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC VLE_U Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA106308 = {S47381, 44, { &P15364,&P17017,&P17384,&P18092 }};	/*Useragent - Mozilla/4.0 WebTV/2.6 (compatible; MSIE 4.0)*/
const Device UA121036 = {S47382, 48, { &P15364,&P17017,&P17384,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Mac_PowerPC)*/
const Device UA253772 = {S47383, 34, { &P15364,&P17017,&P17384,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.0;*/
const Device UA430163 = {S47384, 12, { &P15364,&P17017,&P17384,&P18092 }};	/*Useragent - MSIE 4\.0b2;*/
const Device UA461796 = {S47385, 78, { &P15364,&P17285,&P20917,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; fr; rv:1.9.1b1) Gecko/X Firefox/3.1b1*/
const Device UA2129047 = {S47386, 91, { &P15364,&P17285,&P20917,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0 x64; en-US; rv:1.9.1b2pre) Gecko/X Firefox/3.1b2pre*/
const Device UA2129049 = {S47387, 93, { &P15364,&P17285,&P20917,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0 ; x64; en-US; rv:1.9.1b2pre) Gecko/X Firefox/3.1b2pre*/
const Device UA628883 = {S47388, 157, { &P16036,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus One Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034533 = {S47389, 160, { &P16036,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; Nexus One Build/GRK39F; JRsoft - CyanogenMod) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1903521 = {S47390, 119, { &P15364,&P16110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.215 Safari/534.10*/
const Device UA827325 = {S47391, 85, { &P15364,&P5690,&P19772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; de-DE; rv:1.8.1.23) Gecko/X SeaMonkey/1.1.18*/
const Device UA2412164 = {S47392, 103, { &P15364,&P5690,&P19772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.2pre) Gecko/X SeaMonkey/1.1 Mnenhy/0.7.4.10005*/
const Device UA2442123 = {S47393, 100, { &P15364,&P5690,&P19772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.6) Gecko/X Firefox/2.0.0.11 SeaMonkey/1.1.8*/
const Device UA2638233 = {S47394, 85, { &P15364,&P5690,&P19772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.8.1.19) Gecko/X SeaMonkey/1.1.14*/
const Device UA2236522 = {S47395, 132, { &P16356,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7000 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2238526 = {S47396, 138, { &P16356,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261568 = {S47397, 132, { &P16356,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2261605 = {S47398, 138, { &P16356,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7000 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2468301 = {S47399, 133, { &P16356,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M480W Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2578069 = {S47400, 136, { &P16356,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SC-05D Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2596117 = {S47401, 139, { &P16356,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2596563 = {S47402, 139, { &P16356,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2604117 = {S47403, 139, { &P16356,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2727506 = {S47404, 139, { &P16356,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000B Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2778464 = {S47405, 133, { &P16356,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M480S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2790647 = {S47406, 133, { &P16356,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M480K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA30763 = {S47407, 152, { &P12338,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.1-update1; en-us; T-Mobile_Espresso Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA30764 = {S47408, 152, { &P12338,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us;T-Mobile_Espresso Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA30769 = {S47409, 152, { &P12338,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.1-update1; en-us; T-Mobile_Espresso Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA30777 = {S47410, 152, { &P12338,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; T-Mobile_Espresso Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 MobileSafari/530.17*/
const Device UA30780 = {S47411, 152, { &P12338,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; T-Mobile_EspressoBuild/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA63406 = {S47412, 153, { &P12338,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; T-Mobile_Espresso Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA89773 = {S47413, 146, { &P12338,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; HTC Espresso Build/ERD35B) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA190970 = {S47414, 125, { &P15364,&P5690,&P21896,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/11.0.696.57) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.57 Safari/534.24*/
const Device UA207092 = {S47415, 125, { &P15364,&P5690,&P21896,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/11.0.696.71) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24*/
const Device UA378837 = {S47416, 169, { &P15364,&P5690,&P21896,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/11.0.696.71; .NET CLR 2.0.50727; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 3.0.04506.30; .NET4.0C; .NET4.0E)*/
const Device UA393651 = {S47417, 125, { &P15364,&P5690,&P21896,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/11.0.696.65) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.65 Safari/534.24*/
const Device UA560277 = {S47418, 217, { &P15364,&P5690,&P21896,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Tablet PC 1.7; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.1; chromeframe/11.0.696.65)*/
const Device UA560377 = {S47419, 132, { &P15364,&P5690,&P21896,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; chromeframe/11.0.696.77) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Safari/534.24*/
const Device UA698122 = {S47420, 255, { &P15364,&P5690,&P21896,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB6.6; BTRS31753; chromeframe/11.0.696.71; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; BRI/1)*/
const Device UA702321 = {S47421, 203, { &P15364,&P5690,&P21896,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.1; .NET4.0C; .NET4.0E; chromeframe/11.0.696.65)*/
const Device UA786140 = {S47422, 125, { &P15364,&P5690,&P21896,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/11.0.696.68) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA2137946 = {S47423, 125, { &P15364,&P5690,&P21896,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/11.0.696.77) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.77 Safari/534.24*/
const Device UA2371349 = {S47424, 125, { &P15364,&P5690,&P21896,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/11.0.696.60) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.60 Safari/534.24*/
const Device UA2817189 = {S47425, 220, { &P15364,&P5690,&P21896,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/11.0.696.68; EmbeddedWB 14,52 from: http://www.bsalsa.com/ EmbeddedWB 14.52; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; BOIE8;ENUS)*/
const Device UA609336 = {S47426, 129, { &P16598,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  A500 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA609359 = {S47427, 136, { &P16598,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  Iconia A500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA901581 = {S47428, 128, { &P16598,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A500 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA907320 = {S47429, 135, { &P16598,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Iconia A500 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1995566 = {S47430, 128, { &P16598,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2475386 = {S47431, 137, { &P16598,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Acer Iconia A500 Build/MR1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2792431 = {S47432, 134, { &P16598,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A500 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA97920 = {S47433, 53, { &P15364,&P6704,&P17384,&P18092 }};	/*Useragent - (compatible; MSIE 4.01; MSN 2.5; AOL 4.0; Windows 98)*/
const Device UA100884 = {S47434, 47, { &P15364,&P6704,&P17384,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows 98)*/
const Device UA372473 = {S47435, 64, { &P15364,&P6704,&P17384,&P18092 }};	/*Useragent - Mozilla/4.0(compatible; MSIE 4.01; MSN 2.5; AOL 4.0; Windows 98)*/
const Device UA465439 = {S47436, 46, { &P15364,&P6704,&P17384,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.0; Windows 98)*/
const Device UA729888 = {S47437, 47, { &P15364,&P6704,&P17384,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.0; Windows 98 )*/
const Device UA744493 = {S47438, 46, { &P15364,&P6704,&P17384,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 4.0; Windows 98)*/
const Device UA1939345 = {S47439, 47, { &P15364,&P6704,&P17384,&P18092 }};	/*Useragent - Mozilla/4.5 (compatible; MSIE 4.01; Windows 98)*/
const Device UA2729587 = {S47440, 60, { &P15364,&P6704,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; Win 9x 4.90; Trident/1.0)*/
const Device UA2792822 = {S47441, 54, { &P15364,&P6704,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; Win98; Trident/5.0)*/
const Device UA33381 = {S47442, 125, { &P16598,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; A500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA240542 = {S47443, 54, { &P16598,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; A500 Build/HMJ37)*/
const Device UA702657 = {S47444, 132, { &P16598,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; he-il; A500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2018918 = {S47445, 141, { &P16598,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; Acer Iconia Tab A500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA3065081 = {S47446, 139, { &P18223,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2261636 = {S47447, 132, { &P15932,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3110 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2385215 = {S47448, 132, { &P15932,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3110 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2622781 = {S47449, 133, { &P15932,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1;  GT-P3110 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2201561 = {S47450, 141, { &P20608,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq edison Build/1.1.1 IMM76D X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2203503 = {S47451, 180, { &P20608,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison Build/1.0.0 c1006-user 4.0.4 IMM76D eng.xmypf.X.X test-keys) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2204149 = {S47452, 155, { &P20608,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq edison Build/1.1.1 bq_edison_c1006_X_user) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2205666 = {S47453, 140, { &P20608,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison Build/1.1.4 X-10:36) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2237129 = {S47454, 140, { &P20608,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison Build/1.1.5 X-11:03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2359207 = {S47455, 140, { &P20608,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison Build/1.1.7 X-11:59) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2383942 = {S47456, 140, { &P20608,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison Build/1.1.2 X-14:11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2406900 = {S47457, 140, { &P20608,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison Build/1.1.8 X-16:29) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2431822 = {S47458, 145, { &P20608,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison Build/1.1.8-1015 X-10:20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2461822 = {S47459, 146, { &P20608,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison Build/1.1.10-1015 X-18:00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2470410 = {S47460, 146, { &P20608,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison Build/1.1.11-1015 X-10:30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2934271 = {S47461, 69, { &P20608,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; bq Edison Build/1.1.8 X-16:29)*/
const Device UA395759 = {S47462, 73, { &P12897,&P5843,&P4030,&P18092 }};	/*Useragent - NokiaC2-00/2.0 (03.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-00*/
const Device UA397366 = {S47463, 62, { &P12897,&P5843,&P4030,&P18092 }};	/*Useragent - NokiaC2-00/2.0 (03.82) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2205045 = {S47464, 142, { &P13124,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP9070 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2460971 = {S47465, 157, { &P18442,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-N7100 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2490685 = {S47466, 157, { &P18442,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7105 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2511237 = {S47467, 157, { &P18442,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7105 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2536297 = {S47468, 156, { &P18442,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7105 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2565026 = {S47469, 156, { &P18442,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L900 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2565476 = {S47470, 156, { &P18442,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7100 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2565910 = {S47471, 157, { &P18442,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-N7100 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2711244 = {S47472, 157, { &P18442,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L900 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2718799 = {S47473, 157, { &P18442,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7105 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2733019 = {S47474, 158, { &P18442,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T889 Build/JDQ39; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2866111 = {S47475, 166, { &P18442,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I317 Build/JDQ39; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2897172 = {S47476, 173, { &P18442,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I317 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/t0lteatt*/
const Device UA2942805 = {S47477, 162, { &P18442,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7100 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/n7100*/
const Device UA2942959 = {S47478, 165, { &P18442,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T889 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/t0ltetmo*/
const Device UA2960685 = {S47479, 159, { &P18442,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I317 Build/JDQ39E; CyanogenMod-10.1.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA428373 = {S47480, 159, { &P17835,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; md-us; Nook Tablet Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA459182 = {S47481, 173, { &P17835,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Nook Tablet Build/GINGERBREAD; CyanogenMod-7.1.0 XDA Team) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2816129 = {S47482, 151, { &P17835,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; BNTV250 Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2490274 = {S47483, 124, { &P15767,&P19826,&P21178,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.1 Mobile/10B141 Safari/8536.25*/
const Device UA2532952 = {S47484, 126, { &P15767,&P19826,&P21178,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.1 Mobile/10B146 Safari/8536.25*/
const Device UA2558899 = {S47485, 125, { &P15767,&P19826,&P21178,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/6.1 Mobile/9B176 Safari/7534.48.3*/
const Device UA2562710 = {S47486, 126, { &P15767,&P19826,&P21178,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.1 Mobile/10B147 Safari/8536.25*/
const Device UA2585532 = {S47487, 126, { &P15767,&P19826,&P21178,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.1 Mobile/10B329 Safari/8536.25*/
const Device UA2956831 = {S47488, 128, { &P15767,&P19826,&P21178,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.1.3 Mobile/10B329 Safari/8536.25*/
const Device UA2437324 = {S47489, 151, { &P18318,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF700T Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2748976 = {S47490, 150, { &P18318,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ASUS Transformer Pad TF700T Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA925195 = {S47491, 139, { &P14877,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9000 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996154 = {S47492, 139, { &P14877,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9000 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029740 = {S47493, 139, { &P14877,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; GT-I9000 Build/JRO03E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029800 = {S47494, 61, { &P14877,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-I9000 Build/JRO03L)*/
const Device UA2031348 = {S47495, 139, { &P14877,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9000 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2232119 = {S47496, 139, { &P14877,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2233475 = {S47497, 142, { &P14877,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9000 Build/JellyHeff) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2236868 = {S47498, 139, { &P14877,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9000 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2242374 = {S47499, 61, { &P14877,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9000 Build/JZO54K)*/
const Device UA2455971 = {S47500, 144, { &P14877,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9000 Build/By lucainno) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2827451 = {S47501, 139, { &P14877,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; GT-I9000 Build/JRO03U) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2933787 = {S47502, 153, { &P14877,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; GT-I9000 Build/JRO03L by muchopoli83) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 M?vil Safari/534.30*/
const Device UA3099394 = {S47503, 130, { &P14877,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9000 Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2566505 = {S47504, 140, { &P18447,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.0.489) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566816 = {S47505, 133, { &P18447,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.0.489) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2668975 = {S47506, 138, { &P18447,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xperia T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2204911 = {S47507, 52, { &P15767,&P18580,&P17470,&P18092 }};	/*Useragent - MobileRSSFree-iPad/3.1.4 CFNetwork/609 Darwin/13.0.0*/
const Device UA2230433 = {S47508, 64, { &P15767,&P18580,&P17470,&P18092 }};	/*Useragent - Dolphin%20Browser%20for%20iPad/5.4.1 CFNetwork/609 Darwin/13.0.0*/
const Device UA2535208 = {S47509, 52, { &P15767,&P18580,&P17470,&P18092 }};	/*Useragent - MobileRSS-iPad/3.1.4 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2566254 = {S47510, 58, { &P15767,&P18580,&P17470,&P18092 }};	/*Useragent - Perfect%20Browser-iPad/7.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2598932 = {S47511, 58, { &P15767,&P18580,&P17470,&P18092 }};	/*Useragent - EvernoteWebClip-iPad/2.2.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2603674 = {S47512, 53, { &P15767,&P18580,&P17470,&P18092 }};	/*Useragent - YandexSearch-iPad/683 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2604722 = {S47513, 49, { &P15767,&P18580,&P17470,&P18092 }};	/*Useragent - browseripad/36901 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2738517 = {S47514, 55, { &P15767,&P18580,&P17470,&P18092 }};	/*Useragent - CitrixReceiver-iPad/172 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2509397 = {S47515, 61, { &P15364,&P18110,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Linux x86_64; Trident/4.0)*/
const Device UA2580059 = {S47516, 74, { &P15364,&P18110,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Linux x86_64; Win64; IA64; Trident/3.0)*/
const Device UA2700259 = {S47517, 66, { &P15364,&P18110,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Linux i686; WOW64; Trident/4.0)*/
const Device UA2729546 = {S47518, 72, { &P15364,&P18110,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Linux x86_64; Tablet PC; Trident/2.0)*/
const Device UA2792548 = {S47519, 66, { &P15364,&P18110,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Linux x86_64; SV1; Trident/4.0)*/
const Device UA2034652 = {S47520, 142, { &P18462,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire X Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA93326 = {S47521, 51, { &P15364,&P18145,&P17470,&P18092 }};	/*Useragent - Mozilla/4.76 [en] (X11; U; FreeBSD 4.4-STABLE i386)*/
const Device UA130267 = {S47522, 67, { &P15364,&P18145,&P17470,&P18092 }};	/*Useragent - Mozilla/3.0 (WorldGate Gazelle 3.5.1 build 11; FreeBSD2.2.8-STABLE)*/
const Device UA253806 = {S47523, 68, { &P15364,&P18145,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/3.2; FreeBSD) (KHTML, like Gecko)*/
const Device UA477917 = {S47524, 57, { &P15364,&P18145,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; FreeBSD i386; en-US; rv:1.5) Gecko/X*/
const Device UA2665233 = {S47525, 58, { &P15364,&P18145,&P17470,&P18092 }};	/*Useragent - python-requests/1.1.0 CPython/2.7.2 FreeBSD/9.0-RELEASE-p3*/
const Device UA49444 = {S47526, 139, { &P13026,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SHARP-ADS1 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49445 = {S47527, 139, { &P13026,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SHARP-ADS1 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA762653 = {S47528, 63, { &P13026,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; SHARP-ADS1 Build/FRG83G)*/
const Device UA54007 = {S47529, 133, { &P17945,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; PB99400 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA184205 = {S47530, 57, { &P17945,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; PB99400 Build/FRF91)*/
const Device UA1996107 = {S47531, 115, { &P18648,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia306/03.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.13.1*/
const Device UA12924 = {S47532, 133, { &P17931,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2; en-us; SGH-T849 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA12928 = {S47533, 133, { &P17931,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T849 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA12933 = {S47534, 133, { &P17931,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T849Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA12934 = {S47535, 133, { &P17931,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2; en-us; SGH-T849 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38563 = {S47536, 134, { &P17931,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T849 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA56213 = {S47537, 133, { &P17931,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us;SGH-T849 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA258716 = {S47538, 148, { &P17931,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; SGH-T849 Build/TOSHIBA_FOLIO_AND_A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369517 = {S47539, 70, { &P17931,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.2; en_US; SGH-T849 Build/FROYO.UVJJB)*/
const Device UA376444 = {S47540, 58, { &P17931,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; SGH-T849 Build/FROYO)*/
const Device UA17969 = {S47541, 139, { &P13991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;ADR6400L 4G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA17970 = {S47542, 139, { &P13991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; ADR6400L 4GBuild/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA53598 = {S47543, 140, { &P13991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; ADR6400L 4G Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA53600 = {S47544, 137, { &P13991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; ADR6400L Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA310575 = {S47545, 61, { &P13991,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; ADR6400L Build/FRG83D)*/
const Device UA719371 = {S47546, 136, { &P13991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ADR6400L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1905235 = {S47547, 134, { &P13991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; ADR6400L Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2897914 = {S47548, 58, { &P15364,&P17285,&P23479,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:25.0) Gecko/X Firefox/25.0*/
const Device UA2971587 = {S47549, 70, { &P15364,&P17285,&P23479,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:25.0) Gecko/X Firefox/25.0*/
const Device UA64807 = {S47550, 151, { &P12029,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaN82/20.0.062; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA623289 = {S47551, 164, { &P16049,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; LG-P698f Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA640765 = {S47552, 163, { &P16049,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; LG-P698 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA1946739 = {S47553, 59, { &P16049,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P698 Build/GRJ22)*/
const Device UA1950833 = {S47554, 160, { &P16049,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ar; LG-P698 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2775029 = {S47555, 60, { &P16049,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P698f Build/GRJ22)*/
const Device UA2567844 = {S47556, 126, { &P15767,&P18432,&P22901,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.3 Mobile/10A523 Safari/8536.25*/
const Device UA2570002 = {S47557, 124, { &P15767,&P18432,&P22901,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.3 Mobile/10A403 Safari/8536.25*/
const Device UA908887 = {S47558, 140, { &P22591,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I717D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2023946 = {S47559, 62, { &P22591,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I717D Build/IMM76D)*/
const Device UA2686753 = {S47560, 138, { &P22591,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; SGH-I717D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2740292 = {S47561, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA2740956 = {S47562, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA2754139 = {S47563, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; it-it) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA2755112 = {S47564, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; es-es) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA2756552 = {S47565, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; de-de) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA2769992 = {S47566, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fr-fr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA2781952 = {S47567, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; zh-cn) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA2782366 = {S47568, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ru-ru) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA2782536 = {S47569, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; cs-cz) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA2798754 = {S47570, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pl-pl) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA2798875 = {S47571, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nl-nl) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA2869336 = {S47572, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-au) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA2879154 = {S47573, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; hu-hu) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA2892570 = {S47574, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; es-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA2937636 = {S47575, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-ca) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA2937908 = {S47576, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; tr-tr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA2957281 = {S47577, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; zh-tw) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA2996883 = {S47578, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; de-at) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA2998231 = {S47579, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; el-gr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA2998915 = {S47580, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ar-ae) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA2999549 = {S47581, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fr-ca) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA2999613 = {S47582, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; sv-se) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA3001254 = {S47583, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-in) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA3049464 = {S47584, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; vi-vn) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA3049946 = {S47585, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; fa-fa) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA3073682 = {S47586, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; de-ch) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA3073782 = {S47587, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nl-be) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA3073816 = {S47588, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; in-id) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA3074149 = {S47589, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; pt-br) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA3088761 = {S47590, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-ph) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA3102987 = {S47591, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; da-dk) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA3103098 = {S47592, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-ie) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA3103150 = {S47593, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; nb-no) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA3121672 = {S47594, 128, { &P17595,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ja-jp) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1 Maxthon*/
const Device UA18549 = {S47595, 146, { &P17930,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SCH-I100 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 USCC-I100*/
const Device UA57802 = {S47596, 136, { &P17930,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SCH-I100 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA692586 = {S47597, 60, { &P17930,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SCH-I100 Build/FROYO)*/
const Device UA1894582 = {S47598, 60, { &P17930,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; SCH-I100 Build/FROYO)*/
const Device UA2203817 = {S47599, 114, { &P13035,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-02/06.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2258381 = {S47600, 116, { &P13035,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-02.1/06.52; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA3064763 = {S47601, 137, { &P18509,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA196236 = {S47602, 108, { &P15364,&P6596,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 0.13.507) AppleWebKit/534.35 (KHTML, like Gecko) Chrome/13.0.763.0 Safari/534.35*/
const Device UA200324 = {S47603, 108, { &P15364,&P6596,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 0.13.509) AppleWebKit/534.35 (KHTML, like Gecko) Chrome/13.0.764.0 Safari/534.35*/
const Device UA212848 = {S47604, 110, { &P15364,&P6596,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 13.587.100) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.108 Safari/535.1*/
const Device UA241358 = {S47605, 110, { &P15364,&P6596,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 13.587.135) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1*/
const Device UA322014 = {S47606, 107, { &P15364,&P6596,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 0.13.587) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.14 Safari/535.1*/
const Device UA333348 = {S47607, 110, { &P15364,&P6596,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 13.587.113) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.213 Safari/535.1*/
const Device UA334937 = {S47608, 108, { &P15364,&P6596,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 13.587.48) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.43 Safari/535.1*/
const Device UA336892 = {S47609, 110, { &P15364,&P6596,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 13.587.129) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.218 Safari/535.1*/
const Device UA546783 = {S47610, 108, { &P15364,&P6596,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 0.13.477) AppleWebKit/534.33 (KHTML, like Gecko) Chrome/13.0.753.0 Safari/534.33*/
const Device UA631757 = {S47611, 110, { &P15364,&P6596,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 13.587.126) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.216 Safari/535.1*/
const Device UA634401 = {S47612, 109, { &P15364,&P6596,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 13.587.93) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1*/
const Device UA643176 = {S47613, 108, { &P15364,&P6596,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 13.587.65) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.86 Safari/535.1*/
const Device UA644722 = {S47614, 108, { &P15364,&P6596,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 13.587.80) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.99 Safari/535.1*/
const Device UA897458 = {S47615, 108, { &P15364,&P6596,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 13.587.43) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.37 Safari/535.1*/
const Device UA900303 = {S47616, 108, { &P15364,&P6596,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 13.587.32) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.27 Safari/535.1*/
const Device UA908709 = {S47617, 106, { &P15364,&P6596,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 0.13.558) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.775.2 Safari/535.1*/
const Device UA1887420 = {S47618, 108, { &P15364,&P6596,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 13.587.40) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.34 Safari/535.1*/
const Device UA1888699 = {S47619, 108, { &P15364,&P6596,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 13.587.44) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.38 Safari/535.1*/
const Device UA1952879 = {S47620, 139, { &P18281,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-SU540 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1979277 = {S47621, 138, { &P18281,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-P940 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996368 = {S47622, 140, { &P18281,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-KU5400 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2178423 = {S47623, 143, { &P18281,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-P940/V20d Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2338817 = {S47624, 143, { &P18281,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-P940-Orange Build/V20d) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2411231 = {S47625, 60, { &P18281,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P940 Build/IMM76D)*/
const Device UA2643749 = {S47626, 62, { &P18281,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-KU5400 Build/IMM76D)*/
const Device UA2737445 = {S47627, 144, { &P18281,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-KU5400/20i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879890 = {S47628, 133, { &P18466,&P18110,&P23482,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Safari/535.19 Silk-Accelerated=false*/
const Device UA3065100 = {S47629, 136, { &P23521,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081037 = {S47630, 137, { &P23521,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195L Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094609 = {S47631, 137, { &P23521,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195T Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA213501 = {S47632, 133, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; X8 Build/GingerDX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA568711 = {S47633, 130, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; X8 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA575688 = {S47634, 143, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; X8 Build/GingerDX; GingerDX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA714035 = {S47635, 135, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Xperia X8 Build/V10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA735665 = {S47636, 154, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15i Build/4.0.1.A.0.283; GingerXperia) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA816874 = {S47637, 154, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15i Build/3.0.1.A.0.145; GingerXperia) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1907383 = {S47638, 128, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; X8 Build/V12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1927998 = {S47639, 57, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; X8 Build/GingerDX)*/
const Device UA1938464 = {S47640, 151, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; X8 Build/GingeeXperia; GingerXperia) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1994253 = {S47641, 153, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15i Build/4.0.2.A.0.69; GingerXperia) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018839 = {S47642, 137, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Xperia_X8 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019482 = {S47643, 57, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; X8 Build/GingerDX)*/
const Device UA2020049 = {S47644, 130, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; X8 Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020233 = {S47645, 134, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; Xperia X8 Build/V8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020449 = {S47646, 128, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; pl-pl; X8 Build/V13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020897 = {S47647, 134, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; Xperia X8 Build/V5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021497 = {S47648, 64, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; E15i Build/3.0.1.A.0.145)*/
const Device UA2025734 = {S47649, 146, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15i Build/2.0.0; STORNMIX-v 2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027752 = {S47650, 142, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; E15a Build/Xperience stock) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028773 = {S47651, 156, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; E15a Build/4.0.1.A.0.283; Sony Xperia X8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032622 = {S47652, 168, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Gingerbread 2.3.7; es-es; E15a Build/3.0.1.A.0.145; Sony Xperia X8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034633 = {S47653, 160, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7-XPE; es-es; E15a Build/4.0.1.A.0.283; XPERIENCE STK2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2236452 = {S47654, 153, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; X8 Build/3.0.1.A.0.145; GingerDX-v029) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2409819 = {S47655, 139, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Xperia X8 Build/SEMC V3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2412704 = {S47656, 153, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15i Build/GingerZaraki; GingerZaraki) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2450336 = {S47657, 149, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15i Build/GingerSony; GingerSony) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2453086 = {S47658, 150, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15i Build/AltisV7.3; Xperia-Altis) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2594442 = {S47659, 140, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15i Build/10.0; PrimeHD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2729460 = {S47660, 184, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15i Build/fde +6  yousef-p and Ahmad.H; FDE +6 yousef-p and Ahmad.H) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2815881 = {S47661, 143, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ru-ru; E15 Build/GRH55; Leopard-X8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2819554 = {S47662, 158, { &P14202,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; E15i Build/4.0.2.A.0.42; Xperia-Altis.V7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2512792 = {S47663, 75, { &P17595,&P17297,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5) Presto/2.12.388 Version/12.14*/
const Device UA897052 = {S47664, 71, { &P15364,&P16110,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; U; en) Presto/2.10.289 Version/12.00*/
const Device UA2002983 = {S47665, 71, { &P15364,&P16110,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; U; en) Presto/2.10.289 Version/12.01*/
const Device UA2424225 = {S47666, 138, { &P18283,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SCH-I535 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2426184 = {S47667, 138, { &P18283,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-T999 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2426187 = {S47668, 138, { &P18283,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9300 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2532484 = {S47669, 137, { &P18283,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T999 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2534059 = {S47670, 137, { &P18283,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I535 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2537056 = {S47671, 137, { &P18283,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3011585 = {S47672, 164, { &P18283,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; it-it; SAMSUNG GT-I9300 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2490553 = {S47673, 112, { &P15241,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia300/07.57; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA190297 = {S47674, 136, { &P17933,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SCH-R915 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633824 = {S47675, 60, { &P17933,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SCH-R915 Build/FROYO)*/
const Device UA2435391 = {S47676, 168, { &P18283,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-I535 Build/JOP40D; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2507601 = {S47677, 168, { &P18283,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-T999 Build/JOP40D; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2532479 = {S47678, 167, { &P18283,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T999 Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2536149 = {S47679, 167, { &P18283,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I535 Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583094 = {S47680, 167, { &P18283,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2975802 = {S47681, 173, { &P18283,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I535 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/d2vzw*/
const Device UA3020842 = {S47682, 173, { &P18283,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T999 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/d2tmo*/
const Device UA3021965 = {S47683, 173, { &P18283,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/i9300*/
const Device UA2462474 = {S47684, 130, { &P19783,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ME172V Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA854393 = {S47685, 213, { &P18088,&P22857,&P18373,&P18092 }};	/*Useragent - ASTRO36_TD/v3 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA2154844 = {S47686, 211, { &P18088,&P22857,&P18373,&P18092 }};	/*Useragent - Spice M6688 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA2173798 = {S47687, 315, { &P18088,&P22857,&P18373,&P18092 }};	/*Useragent - Symphony FT45 (MRE\2.3.00(20480) resolution\X chipset\MT6255 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA2336994 = {S47688, 211, { &P18088,&P22857,&P18373,&P18092 }};	/*Useragent - PhilipsX623 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; ru-RU) Presto/2.5.28 Version/10.10*/
const Device UA2383418 = {S47689, 314, { &P18088,&P22857,&P18373,&P18092 }};	/*Useragent - ASTRO36_TD/v3 (MRE\2.3.00(3072) resolution\X chipset\MT6255 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA2455980 = {S47690, 221, { &P18088,&P22857,&P18373,&P18092 }};	/*Useragent - ASTRO36_TD/v3 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera (HTTPS) Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA2505358 = {S47691, 314, { &P18088,&P22857,&P18373,&P18092 }};	/*Useragent - ASTRO36_TD/v3 (MRE\2.3.00(5120) resolution\X chipset\MT6255 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA2527395 = {S47692, 314, { &P18088,&P22857,&P18373,&P18092 }};	/*Useragent - ASTRO36_TD/v3 (MRE\2.4.00(20480) resolution\X chipset\MT6255 touch\0 tpannel\0 camera\1 gsensor\0 keyboard\qwerty) MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA2547824 = {S47693, 347, { &P18088,&P22857,&P18373,&P18092 }};	/*Useragent - ASTRO36_TD/v3 (MRE&#92;2.3.00(20480) resolution&#92;X chipset&#92;MT6255 touch&#92;1 tpannel&#92;1 camera&#92;0 gsensor&#92;0 keyboard&#92;reduced) MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; id-ID) Presto/2.5.28 Version/10.10*/
const Device UA1981960 = {S47694, 140, { &P18225,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; U9200 Build/HuaweiU9200) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261502 = {S47695, 144, { &P18225,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U9202L-1 Build/HuaweiU9202L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261512 = {S47696, 142, { &P18225,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; U9200E Build/HuaweiU9200E) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261595 = {S47697, 142, { &P18225,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GS03 Build/HuaweiU9200-92) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2502663 = {S47698, 144, { &P18225,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U9202L-3 Build/HuaweiU9202L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3072011 = {S47699, 144, { &P18225,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U9202L-4 Build/HuaweiU9202L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1993831 = {S47700, 88, { &P17595,&P18207,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en-US; rv:1.9.1.5) Gecko/X Firefox/3.5.5*/
const Device UA2429480 = {S47701, 139, { &P23772,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190L Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2516494 = {S47702, 139, { &P23772,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA148820 = {S47703, 53, { &P18254,&P7480,&P18245,&P19832 }};	/*Useragent - Pingdom.com_bot_version_1.4_(http://www.pingdom.com/)*/
const Device UA798186 = {S47704, 52, { &P18254,&P7480,&P18245,&P19832 }};	/*Useragent - Pingdom.com_bot_version_1.4_(http://www.pingdom.com)*/
const Device UA7003 = {S47705, 154, { &P12484,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; HTC_Wildfire_A3333 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA62808 = {S47706, 154, { &P12484,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC_Wildfire_A3333 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA3065469 = {S47707, 135, { &P18283,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-I9300 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068207 = {S47708, 135, { &P18283,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SCH-I535 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3082811 = {S47709, 135, { &P18283,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SGH-T999 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114557 = {S47710, 143, { &P18283,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SAMSUNG-SGH-T999 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA513282 = {S47711, 72, { &P18254,&P17017,&P18245,&P18165 }};	/*Useragent - Mozilla/5.0 (compatible; heritrix/1.8.0 +http://yourwebsite.domain.com/)*/
const Device UA517579 = {S47712, 98, { &P18254,&P17017,&P18245,&P18165 }};	/*Useragent - Mozilla/5.0 (compatible; heritrix/2.0.2-SNAPSHOT-X.X +http://www.expertsearch.nl +@OPERATOR_FROM@)*/
const Device UA631472 = {S47713, 61, { &P18254,&P17017,&P18245,&P18165 }};	/*Useragent - Mozilla/5.0 (compatible; heritrix/3.0.0 +http://www.yell.com)*/
const Device UA633522 = {S47714, 72, { &P18254,&P17017,&P18245,&P18165 }};	/*Useragent - Mozilla/5.0 (compatible; heritrix/1.14.0+http://www.nationallibrary.fi/)*/
const Device UA837257 = {S47715, 68, { &P18254,&P17017,&P18245,&P18165 }};	/*Useragent - Mozilla/5.0 (compatible; heritrix/1.7.0 +http://www.greaterera.com/)*/
const Device UA845531 = {S47716, 79, { &P18254,&P17017,&P18245,&P18165 }};	/*Useragent - Mozilla/5.0 (compatible; heritrix/1.5.0-X http://pandora.nla.gov.au/crawl.html)*/
const Device UA896441 = {S47717, 67, { &P18254,&P17017,&P18245,&P18165 }};	/*Useragent - Mozilla/5.0 (compatible; heritrix/3.1.0 +http://internetmemory.org)*/
const Device UA1896729 = {S47718, 62, { &P18254,&P17017,&P18245,&P18165 }};	/*Useragent - Mozilla/5.0 (compatible; heritrix/1.6.0 http://www.worio.com/)*/
const Device UA1907045 = {S47719, 62, { &P18254,&P17017,&P18245,&P18165 }};	/*Useragent - Mozilla/5.0 (compatible; heritrix/1.4t http://www.truveo.com/)*/
const Device UA1966877 = {S47720, 66, { &P18254,&P17017,&P18245,&P18165 }};	/*Useragent - Mozilla/5.0 (compatible; heritrix/@VERSION@ +http://www.mibka.com)*/
const Device UA9897 = {S47721, 62, { &P14862,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2700c/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA15513 = {S47722, 64, { &P14862,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.97) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA15562 = {S47723, 77, { &P14862,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2700c-2*/
const Device UA2030544 = {S47724, 137, { &P18468,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabA2109A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2567836 = {S47725, 135, { &P12280,&P18432,&P22901,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.3 Mobile/10A523 Safari/8536.25*/
const Device UA2570654 = {S47726, 133, { &P12280,&P18432,&P22901,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.3 Mobile/10A403 Safari/8536.25*/
const Device UA2507839 = {S47727, 114, { &P13107,&P5043,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-06/07.48; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA141098 = {S47728, 97, { &P15364,&P18110,&P20909,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/533.3 (KHTML, like Gecko) rekonq Safari/533.3*/
const Device UA236689 = {S47729, 89, { &P15364,&P18110,&P20909,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) rekonq Safari/534.34*/
const Device UA873806 = {S47730, 103, { &P15364,&P18110,&P20909,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) rekonq Safari/534.X-10-16 20:21:01*/
const Device UA3115162 = {S47731, 105, { &P20999,&P17484,&P18369,&P18092 }};	/*Useragent - SAMSUNG-GT-S5292 Opera/9.80 (J2ME/MIDP; Opera Mini/7.0.32584/31.1475; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2566700 = {S47732, 138, { &P18283,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566799 = {S47733, 138, { &P18283,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I535 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567251 = {S47734, 136, { &P18283,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-06D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567326 = {S47735, 138, { &P18283,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583174 = {S47736, 139, { &P18283,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300T Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667826 = {S47737, 139, { &P18283,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999V Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668009 = {S47738, 135, { &P18283,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i9300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668475 = {S47739, 131, { &P18283,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2732977 = {S47740, 138, { &P18283,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-l9300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2935498 = {S47741, 138, { &P18283,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9300 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA159097 = {S47742, 78, { &P18095,&P18094,&P18373,&P18092 }};	/*Useragent - Opera/9.80 (S60; SymbOS; Opera Mobi/1178; U; en-GB) Presto/2.5.28 Version/10.1*/
const Device UA2029043 = {S47743, 146, { &P23910,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyLT30at Build/7.0.B.0.165) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032015 = {S47744, 146, { &P23910,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyLT30at Build/7.0.B.1.104) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034658 = {S47745, 154, { &P23910,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyLT30at Build/VRC-X-1604-1052-ATT) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034836 = {S47746, 145, { &P23910,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyLT30at Build/7.0.B.1.68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2230571 = {S47747, 146, { &P23910,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyLT30at Build/7.0.B.1.152) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2361895 = {S47748, 142, { &P23910,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT30at Build/7.0.B.1.152) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2480782 = {S47749, 64, { &P23910,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT30at Build/7.0.B.1.152)*/
const Device UA181123 = {S47750, 137, { &P16168,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; NookColor Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA197405 = {S47751, 137, { &P16168,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NookColor Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA236666 = {S47752, 137, { &P16168,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; NookColor Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250625 = {S47753, 138, { &P16168,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; NookColor Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA324512 = {S47754, 137, { &P16168,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NookColor Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA342484 = {S47755, 137, { &P16168,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; NookColor Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA354023 = {S47756, 67, { &P16168,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.3 (Linux; U; Android 2.3.7; en_US; NookColor Build/GWK74)*/
const Device UA622370 = {S47757, 146, { &P16168,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; NookColor Build/GRK39F) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.1 Mobile Safari/534.30*/
const Device UA630944 = {S47758, 62, { &P16168,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; NookColor Build/GRK39F)*/
const Device UA2018710 = {S47759, 61, { &P16168,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; NookColor Build/GRI40)*/
const Device UA2019193 = {S47760, 61, { &P16168,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; NookColor Build/GRI40)*/
const Device UA2019374 = {S47761, 61, { &P16168,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; NookColor Build/GRJ22)*/
const Device UA2032772 = {S47762, 140, { &P16168,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5.1; en-us; Nook Color Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2566756 = {S47763, 131, { &P18702,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8013 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2569929 = {S47764, 131, { &P18702,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8013 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA50178 = {S47765, 143, { &P13189,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile G1 Build/DMD64) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA50371 = {S47766, 142, { &P13189,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us;T-Mobile G1 Build/DMD64) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA50438 = {S47767, 142, { &P13189,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile G1 Build/DMD64) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 MobileSafari/525.20.1*/
const Device UA52793 = {S47768, 141, { &P13189,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; de-; T-Mobile G1 Build/DRC92) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA52806 = {S47769, 145, { &P13189,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile G1 Build/Super D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA53566 = {S47770, 142, { &P13189,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android1.6; en-us; T-Mobile G1 Build/DMD64) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA61781 = {S47771, 143, { &P13189,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; T-Mobile G1 Build/DRC92) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA62294 = {S47772, 143, { &P13189,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile G1 Build/DRC83) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA67146 = {S47773, 142, { &P13189,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 1.6; en-us; T-Mobile G1 Build/DMD64) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA67613 = {S47774, 143, { &P13189,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile G1 Build/DMD64) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA68375 = {S47775, 120, { &P13189,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-fr; T-Mobile G1 Build/DRC83) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2*/
const Device UA2018123 = {S47776, 143, { &P13189,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile G1 Build/DRC63) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA50021 = {S47777, 140, { &P12783,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; tr-tr; GM FOX Build/HuaweiU8350) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA641153 = {S47778, 139, { &P12783,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; U8350 Build/HuaweiU8350) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1933830 = {S47779, 143, { &P12783,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; nl-be; Barcelona Build/HuaweiU8350) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019787 = {S47780, 63, { &P12783,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; U8350 Build/HuaweiU8350)*/
const Device UA2345078 = {S47781, 148, { &P12783,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-ar; Personal U8350 Build/HuaweiU8350) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2465967 = {S47782, 142, { &P12783,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-ar; U8350-51 Build/HuaweiU8350) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1997048 = {S47783, 141, { &P18183,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XOOM 2 Build/7.7.1-85_MZ615-26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2167558 = {S47784, 142, { &P18183,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XOOM 2 Build/7.7.1-128_MZ615-12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2418513 = {S47785, 133, { &P18183,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; XOOM 2 Build/7.7.1_128) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2449179 = {S47786, 70, { &P18183,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; XOOM 2 Build/7.7.1-85_MZ615-26)*/
const Device UA2593586 = {S47787, 141, { &P18183,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XOOM 2 Build/7.7.1-128_MZ615-6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2799285 = {S47788, 59, { &P18088,&P17017,&P18412,&P18092 }};	/*Useragent - Opera/9.50 (J2ME/MIDP; Opera Mini/7.5.33466/29.3757; U; en)*/
const Device UA2408362 = {S47789, 101, { &P15364,&P17188,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA1994810 = {S47790, 72, { &P15364,&P18346,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 4.1; Maxthon)*/
const Device UA2159021 = {S47791, 72, { &P15364,&P18346,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Maxthon)*/
const Device UA2159613 = {S47792, 72, { &P15364,&P18346,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; Maxthon)*/
const Device UA2162411 = {S47793, 72, { &P15364,&P18346,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 4.1; Maxthon)*/
const Device UA2165786 = {S47794, 72, { &P15364,&P18346,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Maxthon)*/
const Device UA2166606 = {S47795, 72, { &P15364,&P18346,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; Maxthon)*/
const Device UA2723828 = {S47796, 87, { &P15364,&P18346,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; Avant Browser; Maxthon)*/
const Device UA2744621 = {S47797, 81, { &P15364,&P18346,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Maxthon; Maxthon)*/
const Device UA2787140 = {S47798, 77, { &P15364,&P18346,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; Maxthon; SV1)*/
const Device UA2787407 = {S47799, 87, { &P15364,&P18346,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; Maxthon; Avant Browser)*/
const Device UA2789764 = {S47800, 81, { &P15364,&P18346,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; Maxthon; Maxthon)*/
const Device UA2790270 = {S47801, 77, { &P15364,&P18346,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; SV1; Maxthon)*/
const Device UA2815014 = {S47802, 77, { &P15364,&P18346,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; SV1; Maxthon)*/
const Device UA2821424 = {S47803, 81, { &P15364,&P18346,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; Maxthon; Maxthon)*/
const Device UA2842508 = {S47804, 87, { &P15364,&P18346,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Avant Browser; Maxthon)*/
const Device UA2935249 = {S47805, 107, { &P15364,&P18346,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.10; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)*/
const Device UA2949932 = {S47806, 87, { &P15364,&P18346,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Maxthon; Avant Browser)*/
const Device UA2980151 = {S47807, 77, { &P15364,&P18346,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Maxthon; SV1)*/
const Device UA2980204 = {S47808, 81, { &P15364,&P18346,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Maxthon; Maxthon)*/
const Device UA3085419 = {S47809, 107, { &P15364,&P18346,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.10; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)*/
const Device UA1994436 = {S47810, 117, { &P17595,&P18207,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1*/
const Device UA2042126 = {S47811, 117, { &P17595,&P18207,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1*/
const Device UA2201612 = {S47812, 117, { &P17595,&P18207,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1*/
const Device UA2600218 = {S47813, 117, { &P17595,&P18207,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1*/
const Device UA2873720 = {S47814, 117, { &P17595,&P18207,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1*/
const Device UA3126673 = {S47815, 117, { &P17595,&P18207,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1*/
const Device UA898782 = {S47816, 114, { &P15786,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-05/08.69; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6*/
const Device UA905574 = {S47817, 115, { &P15786,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-05/08.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA2338445 = {S47818, 116, { &P15786,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-05.1/08.69; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6*/
const Device UA2443469 = {S47819, 118, { &P15786,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-05.1/08.60b; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA609776 = {S47820, 139, { &P12814,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  SGH-I777 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA900945 = {S47821, 138, { &P12814,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I777 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA902135 = {S47822, 138, { &P12814,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-I777 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA908949 = {S47823, 146, { &P12814,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I777 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2432653 = {S47824, 146, { &P12814,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I777 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2202900 = {S47825, 77, { &P17595,&P18207,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:19.0) Gecko/19.0 Firefox/19.0*/
const Device UA611 = {S47826, 64, { &P14708,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA606 = {S47827, 77, { &P14708,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5130c-2*/
const Device UA2661 = {S47828, 77, { &P14708,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia5130/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Nokia5130/07.95*/
const Device UA64825 = {S47829, 61, { &P14708,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia5130/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2231984 = {S47830, 132, { &P18466,&P18110,&P18690,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.2 Safari/535.19 Silk-Accelerated=true*/
const Device UA609209 = {S47831, 140, { &P19786,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; fj_vortex Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2441826 = {S47832, 62, { &P19786,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; fj_vortex Build/IML74K)*/
const Device UA2438883 = {S47833, 84, { &P15364,&P7127,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9a1) Gecko/X GranParadiso/3.0a1*/
const Device UA2439139 = {S47834, 82, { &P15364,&P7127,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9a1) Gecko/X GranParadiso/2.0*/
const Device UA2752252 = {S47835, 82, { &P15364,&P7127,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9.1.1) Gecko/X Firefox/3.0.10*/
const Device UA175806 = {S47836, 135, { &P17927,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; C771 Build/C771M030) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA220990 = {S47837, 135, { &P17927,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; C771 Build/C771M050) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA223933 = {S47838, 59, { &P17927,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; C771 Build/C771M050)*/
const Device UA2018390 = {S47839, 135, { &P17927,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; C771 Build/C771M020) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA628869 = {S47840, 136, { &P18319,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-AS680 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1939542 = {S47841, 60, { &P18319,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-AS680 Build/GRJ22)*/
const Device UA2481509 = {S47842, 138, { &P21607,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; VS870 4G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2586642 = {S47843, 138, { &P21607,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LG-VS870 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2974151 = {S47844, 139, { &P17172,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021005 = {S47845, 127, { &P17172,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TF101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021425 = {S47846, 133, { &P17172,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Transformer Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3064662 = {S47847, 149, { &P18318,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF700T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3081620 = {S47848, 149, { &P18318,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ASUS Transformer Pad TF700T Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA37611 = {S47849, 109, { &P12457,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2550/E2550DXXKC3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA62193 = {S47850, 102, { &P12457,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2550L/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA151245 = {S47851, 116, { &P12457,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-GT-E2550/XXJF5 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA163359 = {S47852, 116, { &P12457,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-GT-E2550/XXJI1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA302196 = {S47853, 116, { &P12457,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-GT-E2550/XXJH3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA315372 = {S47854, 116, { &P12457,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-GT-E2550/XXJL2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA20759 = {S47855, 152, { &P23922,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; SonyEricssonR800iv Build/3.0.A.2.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59009 = {S47856, 154, { &P23922,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-nz; SonyEricssonR800iv Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59063 = {S47857, 154, { &P23922,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; SonyEricssonR800iv Build/3.0.1.A.0.146) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73055 = {S47858, 152, { &P23922,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-za; SonyEricssonR800iv Build/3.0.A.2.181) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA77396 = {S47859, 152, { &P23922,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ie; SonyEricssonR800iv Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA628619 = {S47860, 153, { &P23922,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonR800iv Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2386780 = {S47861, 153, { &P23922,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SonyEricssonR800iv Build/4.0.2.A.0.79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2559356 = {S47862, 153, { &P23922,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SonyEricssonR800iv Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2996534 = {S47863, 153, { &P23922,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; SonyEricssonR800iv Build/4.0.2.A.0.79) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2488275 = {S47864, 92, { &P14188,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5233) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2505451 = {S47865, 31, { &P14188,&P5129,&P24675,&P18092 }};	/*Useragent - Nokia5233/UCWEB8.8.0.212/50/800*/
const Device UA2597049 = {S47866, 93, { &P14188,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia 5233) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA1998256 = {S47867, 160, { &P15438,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Sensation Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1998887 = {S47868, 160, { &P15438,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Sensation Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2203274 = {S47869, 160, { &P15438,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Sensation Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2238456 = {S47870, 160, { &P15438,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Sensation Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2425314 = {S47871, 156, { &P15438,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Sensation Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2503795 = {S47872, 180, { &P15438,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Sensation mEMBER's Choice Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2233812 = {S47873, 76, { &P17595,&P19835,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookPro8%2C2)*/
const Device UA2233824 = {S47874, 72, { &P17595,&P19835,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacPro3%2C1)*/
const Device UA2233827 = {S47875, 71, { &P17595,&P19835,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (iMac12%2C2)*/
const Device UA2233840 = {S47876, 76, { &P17595,&P19835,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookAir4%2C2)*/
const Device UA2234665 = {S47877, 73, { &P17595,&P19835,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBook5%2C1)*/
const Device UA2354267 = {S47878, 73, { &P17595,&P19835,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (Macmini5%2C1)*/
const Device UA2428459 = {S47879, 146, { &P18645,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SAMSUNG-SGH-I747 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2428742 = {S47880, 138, { &P18645,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SPH-L710 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2430601 = {S47881, 138, { &P18645,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-I747 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2460688 = {S47882, 139, { &P18645,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SCH-R530U Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2531998 = {S47883, 137, { &P18645,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I747 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2532471 = {S47884, 137, { &P18645,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L710 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2536320 = {S47885, 145, { &P18645,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I747 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2536614 = {S47886, 138, { &P18645,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530U Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2561458 = {S47887, 137, { &P18645,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9305 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA609378 = {S47888, 129, { &P15869,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  Xoom Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA609721 = {S47889, 128, { &P15869,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  Xoom Build/IML77) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA900630 = {S47890, 128, { &P15869,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xoom Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA907717 = {S47891, 127, { &P15869,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Xoom Build/IML77) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2357196 = {S47892, 133, { &P15869,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ505 Build/Moto_MZ505) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA147833 = {S47893, 63, { &P15364,&P18110,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; en) Presto/2.6.30 Version/10.63*/
const Device UA265490 = {S47894, 63, { &P15364,&P18110,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; en) Presto/2.6.30 Version/10.60*/
const Device UA278612 = {S47895, 63, { &P15364,&P18110,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; en) Presto/2.6.30 Version/10.62*/
const Device UA536339 = {S47896, 66, { &P15364,&P18110,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; es-ES) Presto/2.6.30 Version/10.61*/
const Device UA554181 = {S47897, 88, { &P15364,&P18110,&P18269,&P18092 }};	/*Useragent - Opera/10.60 (Linux armv6l ; U; CE-HTML/1.0 NETTV/3.0.1;; en) Presto/2.6.33 Version/10.60*/
const Device UA640945 = {S47898, 87, { &P15364,&P18110,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv6l ; U; CE-HTML/1.0 NETTV/3.0.1;; en) Presto/2.6.33 Version/10.60*/
const Device UA2262143 = {S47899, 86, { &P15364,&P18110,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l ; U; CE-HTML/1.0 NETTV/3.2.2; en) Presto/2.6.33 Version/10.60*/
const Device UA2432247 = {S47900, 81, { &P12280,&P17779,&P18525,&P18092 }};	/*Useragent - Opera/9.80 (iPhone; Opera Mini/6.0.1/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2770692 = {S47901, 198, { &P18645,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-I747M Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.2/552,2.0.2.552,2013-05-22 23:05, -0400, ca*/
const Device UA3027718 = {S47902, 198, { &P18645,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I747M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, ca*/
const Device UA3027890 = {S47903, 197, { &P18645,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us*/
const Device UA3028626 = {S47904, 205, { &P18645,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0500, us*/
const Device UA3029047 = {S47905, 197, { &P18645,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9305 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb*/
const Device UA3030126 = {S47906, 205, { &P18645,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us*/
const Device UA3036004 = {S47907, 198, { &P18645,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530U Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0500, us*/
const Device UA1976970 = {S47908, 139, { &P18488,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-L300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2421433 = {S47909, 61, { &P18488,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SPH-L300 Build/IMM76D)*/
const Device UA149278 = {S47910, 125, { &P17595,&P17296,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA167875 = {S47911, 125, { &P17595,&P17296,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA177293 = {S47912, 125, { &P17595,&P17296,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA200707 = {S47913, 125, { &P17595,&P17296,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA291275 = {S47914, 125, { &P17595,&P17296,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA291693 = {S47915, 125, { &P17595,&P17296,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA291945 = {S47916, 125, { &P17595,&P17296,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA373128 = {S47917, 125, { &P17595,&P17296,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA490052 = {S47918, 125, { &P17595,&P17296,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA3065237 = {S47919, 150, { &P18505,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081352 = {S47920, 152, { &P18505,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MB886 Build/9.8.2Q-50_MB886_NII_TA-2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA161900 = {S47921, 128, { &P17595,&P17295,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.231 Safari/534.10*/
const Device UA168222 = {S47922, 128, { &P17595,&P17295,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10*/
const Device UA300918 = {S47923, 128, { &P17595,&P17295,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_0; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10*/
const Device UA302549 = {S47924, 128, { &P17595,&P17295,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10*/
const Device UA303235 = {S47925, 128, { &P17595,&P17295,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10*/
const Device UA778781 = {S47926, 128, { &P17595,&P17295,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10*/
const Device UA872186 = {S47927, 128, { &P17595,&P17295,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.215 Safari/534.10*/
const Device UA1854969 = {S47928, 128, { &P17595,&P17295,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.231 Safari/534.10*/
const Device UA2258236 = {S47929, 128, { &P17595,&P17295,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.10 (KHTML	 like Gecko) Chrome/8.0.552.237 Safari/534.10*/
const Device UA160876 = {S47930, 139, { &P16950,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Dell Streak 7 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA178599 = {S47931, 135, { &P16950,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Dell Streak 7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA378267 = {S47932, 66, { &P16950,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; Dell Streak 7 Build/FRG83G)*/
const Device UA556879 = {S47933, 134, { &P16950,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.2; en-us; Dell Streak 7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2018772 = {S47934, 134, { &P16950,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; DellStreak 7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2018835 = {S47935, 134, { &P16950,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;Dell Streak 7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2018836 = {S47936, 134, { &P16950,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Dell Streak 7Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2019271 = {S47937, 134, { &P16950,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.2; en-us; Dell Streak 7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA351691 = {S47938, 87, { &P15364,&P17285,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.10.229 Version/11.60*/
const Device UA352193 = {S47939, 86, { &P15364,&P17285,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4876); ru) Presto/2.10.229 Version/11.60*/
const Device UA352903 = {S47940, 86, { &P15364,&P17285,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4139); ru) Presto/2.10.229 Version/11.60*/
const Device UA353298 = {S47941, 87, { &P15364,&P17285,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03686); ru) Presto/2.10.229 Version/11.60*/
const Device UA355985 = {S47942, 86, { &P15364,&P17285,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4664); ru) Presto/2.10.229 Version/11.60*/
const Device UA398306 = {S47943, 86, { &P15364,&P17285,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4953); ru) Presto/2.10.229 Version/11.60*/
const Device UA628903 = {S47944, 64, { &P15364,&P17285,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; en) Presto/2.10.229 Version/11.60*/
const Device UA653811 = {S47945, 87, { &P15364,&P17285,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.4 (build 02647); ru) Presto/2.10.229 Version/11.60*/
const Device UA692738 = {S47946, 86, { &P15364,&P17285,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4598); ru) Presto/2.10.229 Version/11.60*/
const Device UA701201 = {S47947, 87, { &P15364,&P17285,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.10.229 Version/11.60*/
const Device UA712664 = {S47948, 87, { &P15364,&P17285,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.10.229 Version/11.60*/
const Device UA1934529 = {S47949, 66, { &P15364,&P17285,&P18104,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; en) Opera 11.60*/
const Device UA1935524 = {S47950, 87, { &P15364,&P17285,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03790); ru) Presto/2.10.229 Version/11.60*/
const Device UA1938641 = {S47951, 86, { &P15364,&P17285,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4947); ru) Presto/2.10.229 Version/11.60*/
const Device UA1945868 = {S47952, 116, { &P15364,&P17285,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0); en) Presto/2.10.229 Version/11.60*/
const Device UA2306549 = {S47953, 87, { &P15364,&P17285,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.6 (build 03278); ru) Presto/2.10.229 Version/11.60*/
const Device UA2307172 = {S47954, 86, { &P15364,&P17285,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4157); ru) Presto/2.10.229 Version/11.60*/
const Device UA2307441 = {S47955, 86, { &P15364,&P17285,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4848); ru) Presto/2.10.229 Version/11.60*/
const Device UA2307654 = {S47956, 86, { &P15364,&P17285,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4133); ru) Presto/2.10.229 Version/11.60*/
const Device UA2974266 = {S47957, 149, { &P18318,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF700T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022009 = {S47958, 149, { &P18318,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ASUS Transformer Pad TF700T Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA1149 = {S47959, 76, { &P11689,&P5776,&P3756,&P18092 }};	/*Useragent - NokiaN70-1/5.0737.3.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA6835 = {S47960, 75, { &P11689,&P5776,&P3756,&P18092 }};	/*Useragent - NokiaN70/ 5.0741.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA755108 = {S47961, 35, { &P11689,&P5776,&P3756,&P18092 }};	/*Useragent - NokiaN70/SymbianOS/8.0 Series60/2.6*/
const Device UA1840407 = {S47962, 37, { &P11689,&P5776,&P3756,&P18092 }};	/*Useragent - NokiaN70-1/SymbianOS/8.0 Series60/2.6*/
const Device UA2947949 = {S47963, 118, { &P15767,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Mobile/11A4414e Twitter for iPhone*/
const Device UA3016941 = {S47964, 118, { &P15767,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Mobile/11A4400f Twitter for iPhone*/
const Device UA3026313 = {S47965, 120, { &P15767,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4435d Twitter for iPhone*/
const Device UA3033026 = {S47966, 120, { &P15767,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449a Twitter for iPhone*/
const Device UA3035718 = {S47967, 120, { &P15767,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d Twitter for iPhone*/
const Device UA3081133 = {S47968, 118, { &P15767,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Twitter for iPhone*/
const Device UA3115353 = {S47969, 120, { &P15767,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Twitter for iPhone*/
const Device UA90596 = {S47970, 139, { &P17946,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; USCC-US760 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA644410 = {S47971, 63, { &P17946,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; USCC-US760 Build/FRG83G)*/
const Device UA907574 = {S47972, 137, { &P18243,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P880 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2381243 = {S47973, 138, { &P18243,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P880g Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2225171 = {S47974, 139, { &P20013,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F180L Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2226281 = {S47975, 139, { &P20013,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F180S Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257897 = {S47976, 139, { &P20013,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F180K Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2648420 = {S47977, 61, { &P20013,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-F180L Build/IMM76L)*/
const Device UA2980997 = {S47978, 61, { &P20013,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-F180S Build/IMM76L)*/
const Device UA2603336 = {S47979, 111, { &P15767,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Flipboard/2.0*/
const Device UA2605457 = {S47980, 113, { &P15767,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Flipboard/2.0*/
const Device UA2605664 = {S47981, 113, { &P15767,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0*/
const Device UA2679612 = {S47982, 115, { &P15767,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.1*/
const Device UA2679616 = {S47983, 115, { &P15767,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Flipboard/2.0.1*/
const Device UA2739315 = {S47984, 115, { &P15767,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.2*/
const Device UA2752893 = {S47985, 113, { &P15767,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Flipboard/2.0.2*/
const Device UA2753806 = {S47986, 115, { &P15767,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Flipboard/2.0.2*/
const Device UA2796933 = {S47987, 115, { &P15767,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.3*/
const Device UA2888564 = {S47988, 115, { &P15767,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.4*/
const Device UA2896914 = {S47989, 115, { &P15767,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Flipboard/2.0.4*/
const Device UA2944535 = {S47990, 113, { &P15767,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Flipboard/2.0.4*/
const Device UA3038809 = {S47991, 115, { &P15767,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.5*/
const Device UA3067726 = {S47992, 115, { &P15767,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.6*/
const Device UA3096098 = {S47993, 115, { &P15767,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.7*/
const Device UA158605 = {S47994, 115, { &P17595,&P17295,&P18106,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-US) AppleWebKit/530.5 (KHTML, like Gecko) Chrome/ Safari/530.5*/
const Device UA686335 = {S47995, 109, { &P17595,&P17295,&P18106,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Mac OS X 10_5_7; en-US) AppleWebKit/530.5 (KHTML, like Gecko) Chrome/ Safari/530.5*/
const Device UA894816 = {S47996, 115, { &P17595,&P17295,&P18106,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-US) AppleWebKit/530.9 (KHTML, like Gecko) Chrome/ Safari/530.9*/
const Device UA2814090 = {S47997, 115, { &P17595,&P17295,&P18106,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-US) AppleWebKit/530.6 (KHTML, like Gecko) Chrome/ Safari/530.6*/
const Device UA3065155 = {S47998, 137, { &P22584,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337M Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA31210 = {S47999, 151, { &P14864,&P5520,&P3797,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8500/S8500XXJL2; U; Bada/1.2; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.2 Mobile WVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA2033115 = {S48000, 142, { &P20285,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; XOOM 2 Build/7.7.1-128_MZ616-11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2167537 = {S48001, 142, { &P20285,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XOOM 2 Build/7.7.1-128_MZ616-12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2432246 = {S48002, 147, { &P20285,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MZ616 Build/7.7.1-141-6-PAST-UMTS-LA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2465330 = {S48003, 71, { &P20285,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ616-12)*/
const Device UA800181 = {S48004, 76, { &P17595,&P18207,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:9.0.1) Gecko/X Firefox/9.0.1*/
const Device UA2383923 = {S48005, 147, { &P19866,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; ONE TOUCH 983 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2666000 = {S48006, 155, { &P19866,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; ALCATEL_one_touch_983 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2983027 = {S48007, 71, { &P19866,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; ONE TOUCH 983 Build/GINGERBREAD)*/
const Device UA3065386 = {S48008, 126, { &P15869,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xoom Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3081478 = {S48009, 126, { &P15869,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Xoom Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2897269 = {S48010, 136, { &P15575,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-i9100 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36*/
const Device UA2973469 = {S48011, 137, { &P15575,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974384 = {S48012, 137, { &P15575,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974453 = {S48013, 138, { &P15575,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023529 = {S48014, 137, { &P15575,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-i9100G Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA218999 = {S48015, 138, { &P12657,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_EVO3D_X515m; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA227919 = {S48016, 138, { &P12657,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_EVO3D_X515a; en-ca) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA575492 = {S48017, 144, { &P12657,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_X515a V1.24.841.3; en-au) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2640332 = {S48018, 132, { &P12657,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_X515m; zh-cn) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2042044 = {S48019, 92, { &P17595,&P17294,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:10.0.7) Gecko/X Firefox/10.0.7 TenFourFox/7450*/
const Device UA3065377 = {S48020, 137, { &P16362,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065611 = {S48021, 141, { &P16362,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710VMUB Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067075 = {S48022, 140, { &P16362,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710BST Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA252331 = {S48023, 150, { &P13999,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC HD2 Build/GRI40; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA380081 = {S48024, 151, { &P13999,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC HD2 Build/GRI40; HyperDroid-CM7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA699062 = {S48025, 148, { &P13999,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HD2 Build/GRK39F; HyperDroid-CM7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA736665 = {S48026, 207, { &P13999,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC HD2 Build/GRI40; cope & warrenb present: AmeriCanAndroid GB237 CyanogenMod7.2 HTC LEO X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA858950 = {S48027, 147, { &P13999,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HD2 Build/GRI40; HyperDroid-CM7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020365 = {S48028, 146, { &P13999,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HD2 Build/GRI40; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021131 = {S48029, 172, { &P13999,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; HTC HD2 Build/GRI40; Negro90 V.18 CyanogenMod 7.1 MAGLDR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3117100 = {S48030, 150, { &P13999,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; HTC HD2 Build/GWK74; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2566488 = {S48031, 140, { &P17172,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2862481 = {S48032, 134, { &P17172,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Transformer Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2990081 = {S48033, 128, { &P17172,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TF101 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA139770 = {S48034, 141, { &P15364,&P17816,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Ubuntu/10.04 Chromium/6.0.472.62 Chrome/6.0.472.62 Safari/534.3*/
const Device UA372434 = {S48035, 143, { &P15364,&P17816,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Ubuntu/10.04 Chromium/6.0.472.53 Chrome/6.0.472.53 Safari/534.3*/
const Device UA2029716 = {S48036, 165, { &P18242,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-sa; LG-P720h Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2031867 = {S48037, 136, { &P18242,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-P725 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033148 = {S48038, 164, { &P18242,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; LG-P720 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2452600 = {S48039, 169, { &P18242,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; LG-P720/V10e Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2494500 = {S48040, 60, { &P18242,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-P725 Build/GRK39F)*/
const Device UA2575582 = {S48041, 141, { &P18242,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; LG-P725/V10f Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2685775 = {S48042, 169, { &P18242,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; LG-P720/V10a Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2759042 = {S48043, 141, { &P18242,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; LG-P725/V10d Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2973957 = {S48044, 130, { &P15882,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P5110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974280 = {S48045, 130, { &P15882,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P5110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA25401 = {S48046, 44, { &P15364,&P5690,&P18407,&P18092 }};	/*Useragent - Opera/9.70 (WindowsNT 5.1; U; en) TMO-US_LEO*/
const Device UA25400 = {S48047, 44, { &P15364,&P5690,&P18407,&P18092 }};	/*Useragent - Opera/9.70 (Windows NT5.1; U; en) TMO-US_LEO*/
const Device UA129081 = {S48048, 45, { &P15364,&P5690,&P18407,&P18092 }};	/*Useragent - Opera/9.70 (Windows NT 5.1; U; en) TMO-US_LEO*/
const Device UA374145 = {S48049, 33, { &P15364,&P5690,&P18407,&P18092 }};	/*Useragent - Opera 9.7 (Windows NT 5.2; U; en)*/
const Device UA2974383 = {S48050, 144, { &P18614,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT890 Build/9.8.2I-50_SMI-26) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2202664 = {S48051, 131, { &P20196,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID7048 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2889446 = {S48052, 60, { &P20196,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; MID7048 Build/IML74K)*/
const Device UA25927 = {S48053, 149, { &P14784,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; fr-fr; HTC Hero-Orange Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA53743 = {S48054, 142, { &P14784,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-gb; HTC Hero Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2775358 = {S48055, 142, { &P14784,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; de-ch; HTC Hero Build/CUPCAKE) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2033985 = {S48056, 150, { &P19868,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; ALCATEL_one_touch_985 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2157932 = {S48057, 151, { &P19868,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; ALCATEL ONE TOUCH 985N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2350640 = {S48058, 151, { &P19868,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; ALCATEL ONE TOUCH 985D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2412574 = {S48059, 150, { &P19868,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; ALCATEL ONE TOUCH 985 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2166806 = {S48060, 139, { &P20973,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LW770 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2655730 = {S48061, 61, { &P20973,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-LW770 Build/IMM76I)*/
const Device UA2361523 = {S48062, 58, { &P15364,&P18110,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686) Presto/2.12.388 Version/12.11*/
const Device UA2405632 = {S48063, 69, { &P15364,&P18110,&P21814,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:2.0) Gecko/X Firefox/4.0 Opera 12.11*/
const Device UA2422233 = {S48064, 63, { &P15364,&P18110,&P21814,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; X11; Linux i686) Opera 12.11*/
const Device UA2766658 = {S48065, 76, { &P15364,&P18110,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; Opera TV Store/5521) Presto/2.12.362 Version/12.11*/
const Device UA3039779 = {S48066, 129, { &P15364,&P18110,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; Model/Sony-KDL-46EX650 SonyCEBrowser/1.0 (KDL-46EX650; CTV/1.0; JPN; en)) Presto/2.12.362 Version/12.11*/
const Device UA3106947 = {S48067, 123, { &P15364,&P18110,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-50R550A; CC/USA) Presto/2.12.362 Version/12.11*/
const Device UA2974268 = {S48068, 139, { &P20289,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974895 = {S48069, 139, { &P20289,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC6435LVW Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2708203 = {S48070, 140, { &P18223,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.0.400) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2974601 = {S48071, 139, { &P18564,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X+ Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022696 = {S48072, 141, { &P18564,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC EVARE_UL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2566492 = {S48073, 131, { &P23773,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8010 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566618 = {S48074, 131, { &P23773,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8010 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA53965 = {S48075, 124, { &P14082,&P6590,&P4223,&P18092 }};	/*Useragent - NetFront/3.5.1 (BREW 5.0.1.2; U; en-us; LG; NetFront/3.5.1/AMB) Sprint LN510 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA56394 = {S48076, 250, { &P14082,&P6590,&P4223,&P18092 }};	/*Useragent - NetFront/3.5.1 (BREW 5.0.1.2; U; en-us; LG; NetFront/3.5.1/AMB) Sprint LN510 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1, NetFront/3.5.1 (BREW 5.0.1.2; U; en-us; LG; NetFront/3.5.1/AMB) Sprint LN510 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2774408 = {S48077, 164, { &P22871,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9295 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3026933 = {S48078, 177, { &P22871,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; de-de; SAMSUNG GT-I9295/I9295XXUAMF5 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2416730 = {S48079, 140, { &P18474,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; GT-I9210T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2485877 = {S48080, 62, { &P18474,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9210T Build/IML74K)*/
const Device UA2847632 = {S48081, 138, { &P18474,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-I9210T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2678364 = {S48082, 92, { &P12280,&P19826,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 6_1_3; zh-CN; iPh4,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile*/
const Device UA2712553 = {S48083, 92, { &P12280,&P19826,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 6_1_4; zh-CN; iPh5,2) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile*/
const Device UA2716513 = {S48084, 92, { &P12280,&P19826,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 6_1_3; zh-CN; iPh3,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile*/
const Device UA2736606 = {S48085, 92, { &P12280,&P19826,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 6_1_3; en-US; iPh2,1) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2781448 = {S48086, 92, { &P12280,&P19826,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 6_1_3; zh-CN; iPh3,2) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile*/
const Device UA2892315 = {S48087, 92, { &P12280,&P19826,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 6_1_4; zh-CN; iPh5,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile*/
const Device UA2992850 = {S48088, 92, { &P12280,&P19826,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 6_1_3; en-US; iPh3,3) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2994621 = {S48089, 89, { &P12280,&P19826,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 6_1_3; vi; iPh3,1) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA867936 = {S48090, 131, { &P15254,&P7106,&P15829,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9981; en-GB) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.342 Mobile Safari/534.11+*/
const Device UA90608 = {S48091, 116, { &P15364,&P22740,&P22741,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux armv7l; en-GB; rv:1.9.2.3pre) Gecko/20100723 Firefox/3.5 Maemo Browser 1.7.4.8 RX-51 N900*/
const Device UA680446 = {S48092, 128, { &P20183,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE Plus Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA816538 = {S48093, 142, { &P20183,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; MI-ONE Plus Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA834210 = {S48094, 142, { &P20183,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE Plus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899031 = {S48095, 137, { &P20183,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; MI-ONE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA906279 = {S48096, 133, { &P20183,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE Plus Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1955325 = {S48097, 137, { &P20183,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1961084 = {S48098, 123, { &P20183,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; MI-ONE Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1965476 = {S48099, 128, { &P20183,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1987958 = {S48100, 142, { &P20183,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; zh-cn; MI-ONE Plus Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2024155 = {S48101, 128, { &P20183,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; MI-ONE Plus Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2024842 = {S48102, 123, { &P20183,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2033461 = {S48103, 64, { &P20183,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MI-ONE Plus Build/IMM76D)*/
const Device UA2395522 = {S48104, 128, { &P20183,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; MI-ONE Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2788864 = {S48105, 140, { &P20183,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE Plus Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2988629 = {S48106, 64, { &P20183,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MI-ONE Plus MIUI/ICS24.0)*/
const Device UA2461947 = {S48107, 100, { &P15364,&P18110,&P19819,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) konqueror/4.9.5 Safari/534.34*/
const Device UA2506473 = {S48108, 68, { &P15364,&P18110,&P19819,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) KHTML/4.9.5 (like Gecko) Konqueror/4.9*/
const Device UA2551810 = {S48109, 63, { &P15364,&P18110,&P19819,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux) KHTML/4.9.1 (like Gecko) Konqueror/4.9*/
const Device UA2746045 = {S48110, 56, { &P15364,&P18110,&P19819,&P18092 }};	/*Useragent - Mozilla/5.0 (X11) KHTML/4.9.1 (like Gecko) Konqueror/4.9*/
const Device UA249978 = {S48111, 81, { &P15364,&P5690,&P21900,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; chromeframe/14.0.835.186)*/
const Device UA250094 = {S48112, 125, { &P15364,&P5690,&P21900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/14.0.835.202) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA250167 = {S48113, 140, { &P15364,&P5690,&P21900,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; chromeframe/14.0.835.202; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA250171 = {S48114, 76, { &P15364,&P5690,&P21900,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; chromeframe/14.0.835.202)*/
const Device UA340058 = {S48115, 125, { &P15364,&P5690,&P21900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/14.0.835.163) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.163 Safari/535.1*/
const Device UA437201 = {S48116, 172, { &P15364,&P5690,&P21900,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; chromeframe/14.0.835.202; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA569879 = {S48117, 125, { &P15364,&P5690,&P21900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/14.0.835.109) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.109 Safari/535.1*/
const Device UA574595 = {S48118, 179, { &P15364,&P5690,&P21900,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; chromeframe/14.0.835.202; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA643216 = {S48119, 125, { &P15364,&P5690,&P21900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/14.0.835.186) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1*/
const Device UA838649 = {S48120, 125, { &P15364,&P5690,&P21900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/14.0.835.187) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.187 Safari/535.1*/
const Device UA1895635 = {S48121, 123, { &P15364,&P5690,&P21900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/14.0.835.94) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.94 Safari/535.1*/
const Device UA1904775 = {S48122, 148, { &P15364,&P5690,&P21900,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/14.0.835.202; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0)*/
const Device UA2258833 = {S48123, 123, { &P15364,&P5690,&P21900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/14.0.835.15) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.15 Safari/535.1*/
const Device UA2541230 = {S48124, 151, { &P15364,&P5690,&P21900,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; chromeframe/14.0.803.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2803731 = {S48125, 121, { &P15364,&P5690,&P21900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; chromeframe/14.0.803.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.803.0 Safari/535.1*/
const Device UA32149 = {S48126, 172, { &P11964,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE75-2/202.12.01; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.1.19424*/
const Device UA35916 = {S48127, 152, { &P11964,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2NokiaE75-1/100.48.78 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA63358 = {S48128, 155, { &P11964,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 NokiaE75-1 /110.48.125 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA64764 = {S48129, 168, { &P11964,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE75-1/211.12.01; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.1.1*/
const Device UA635915 = {S48130, 239, { &P15767,&P17146,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405,Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA674398 = {S48131, 141, { &P15767,&P17146,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2495023 = {S48132, 139, { &P15767,&P17146,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2669310 = {S48133, 143, { &P15767,&P17146,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_0_1 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.10*/
const Device UA2703531 = {S48134, 142, { &P15767,&P17146,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_0_1 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2173879 = {S48135, 139, { &P18682,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-M950 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455463 = {S48136, 61, { &P18682,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SPH-M950 Build/IMM76I)*/
const Device UA609433 = {S48137, 141, { &P13054,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  ADR6425LVW Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA902709 = {S48138, 140, { &P13054,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR6425LVW Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2172433 = {S48139, 140, { &P13054,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6425LVW Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2570156 = {S48140, 125, { &P15364,&P18110,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2772763 = {S48141, 124, { &P15364,&P18110,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2613446 = {S48142, 61, { &P17323,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P6800 Build/JRO03H)*/
const Device UA2614943 = {S48143, 121, { &P17323,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.1.1; en-US; GT-P6800 Build/pa_p6800-userdebug 4.1.1 JRO03L eng.androguide.X.X test-keys)*/
const Device UA2615126 = {S48144, 132, { &P17323,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; GT-P6800 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615772 = {S48145, 153, { &P17323,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P6800 Build/JRO03L; 800*1216) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2615843 = {S48146, 132, { &P17323,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; GT-P6800 Build/JR003H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616442 = {S48147, 145, { &P17323,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6800 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2616480 = {S48148, 133, { &P17323,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6800 Build/IMML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616789 = {S48149, 61, { &P17323,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P6800 Build/JZO54K)*/
const Device UA2618404 = {S48150, 116, { &P17323,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.1.2; en-SG; GT-P6800 Build/cm_p6800-userdebug 4.1.2 JZO54K eng.aorth.X.X test-keys)*/
const Device UA2619858 = {S48151, 75, { &P17323,&P18111,&P18112,&P18092 }};	/*Useragent - GoogleAnalytics/2.0 (Linux; U; Android 4.1.1; en-us; GT-P6800 Build/JRO03R)*/
const Device UA2619860 = {S48152, 130, { &P17323,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P6800 Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2620411 = {S48153, 132, { &P17323,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P6800 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2630231 = {S48154, 130, { &P17323,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P6800 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3121017 = {S48155, 139, { &P17323,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P6800 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2974360 = {S48156, 144, { &P22972,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I537 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2709202 = {S48157, 141, { &P12280,&P17146,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9A334 Safari/7534.48.3*/
const Device UA2717587 = {S48158, 143, { &P12280,&P17146,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9A406 Safari/7534.48.3*/
const Device UA2718800 = {S48159, 143, { &P12280,&P17146,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9A405 Safari/7534.48.3*/
const Device UA3065140 = {S48160, 137, { &P12972,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067201 = {S48161, 137, { &P12972,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067599 = {S48162, 138, { &P12972,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3082866 = {S48163, 138, { &P12972,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989D Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA901283 = {S48164, 135, { &P21037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; LIFETAB_S9512 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA901781 = {S48165, 135, { &P21037,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; LIFETAB_S9512 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2032117 = {S48166, 66, { &P21037,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LIFETAB_S9512 Build/IMM76D)*/
const Device UA2586658 = {S48167, 139, { &P21627,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC first Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA218657 = {S48168, 112, { &P15364,&P6596,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 12.433.257) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.130 Safari/534.30*/
const Device UA323070 = {S48169, 112, { &P15364,&P6596,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 12.433.114) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.105 Safari/534.30*/
const Device UA334166 = {S48170, 112, { &P15364,&P6596,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 12.433.231) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.113 Safari/534.30*/
const Device UA435976 = {S48171, 111, { &P15364,&P6596,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 12.433.109) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.93 Safari/534.30*/
const Device UA451499 = {S48172, 112, { &P15364,&P6596,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 12.0.742.91) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.93 Safari/534.30*/
const Device UA651799 = {S48173, 112, { &P15364,&P6596,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 12.433.216) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.105 Safari/534.30*/
const Device UA752231 = {S48174, 109, { &P15364,&P6596,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 0.12.433) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.77 Safari/534.30*/
const Device UA912988 = {S48175, 105, { &P15364,&P6596,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 0.0.0) AppleWebKit/534.28 (KHTML, like Gecko) Chrome/12.0.723.1 Safari/534.28*/
const Device UA623343 = {S48176, 138, { &P20938,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; D930 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2606562 = {S48177, 178, { &P18254,&P18110,&P18097,&P18091 }};	/*Useragent - Influencebot/0.9; (Automatic classification of websites; http://www.influencebox.com/; info@influencebox.com)User-Agent: Mozilla/5.0 (X11; Linux i686; rv:9.0) Gecko/X Firefox/9.0*/
const Device UA2717083 = {S48178, 166, { &P18254,&P18110,&P18097,&P18091 }};	/*Useragent - Influencebot/0.9; (Automatic classification of websites; http://www.influencebox.com/; info@influencebox.com)Mozilla/5.0 (X11; Linux i686; rv:9.0) Gecko/X Firefox/9.0*/
const Device UA2235767 = {S48179, 73, { &P16329,&P18646,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PLAYSTATION 3 4.30) AppleWebKit/531.22.8 (KHTML, like Gecko)*/
const Device UA646591 = {S48180, 180, { &P17595,&P17297,&P22921,&P18092 }};	/*Useragent - NewsBlur Page Fetcher (1 subscriber) - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)*/
const Device UA2151299 = {S48181, 181, { &P17595,&P17297,&P22921,&P18092 }};	/*Useragent - NewsBlur Page Fetcher (2 subscribers) - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)*/
const Device UA2215914 = {S48182, 138, { &P17595,&P17297,&P22921,&P18092 }};	/*Useragent - NewsBlur Feed Finder (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)*/
const Device UA2310468 = {S48183, 181, { &P17595,&P17297,&P22921,&P18092 }};	/*Useragent - NewsBlur Page Fetcher (3 subscribers) - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)*/
const Device UA2317712 = {S48184, 181, { &P17595,&P17297,&P22921,&P18092 }};	/*Useragent - NewsBlur Page Fetcher (4 subscribers) - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)*/
const Device UA2348727 = {S48185, 165, { &P17595,&P17297,&P22921,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/536.2.3 (KHTML, like Gecko) Version/5.2 (NewsBlur Feed Fetcher - 56 subscribers - http://www.newsblur.com)*/
const Device UA2505608 = {S48186, 163, { &P17595,&P17297,&P22921,&P18092 }};	/*Useragent - NewsBlur Feed Fetcher - 1 subscriber - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/536.2.3 (KHTML, like Gecko) Version/5.2)*/
const Device UA2556991 = {S48187, 164, { &P17595,&P17297,&P22921,&P18092 }};	/*Useragent - NewsBlur Feed Fetcher - 3 subscribers - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/536.2.3 (KHTML, like Gecko) Version/5.2)*/
const Device UA2557060 = {S48188, 164, { &P17595,&P17297,&P22921,&P18092 }};	/*Useragent - NewsBlur Feed Fetcher - 2 subscribers - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/536.2.3 (KHTML, like Gecko) Version/5.2)*/
const Device UA2573900 = {S48189, 181, { &P17595,&P17297,&P22921,&P18092 }};	/*Useragent - NewsBlur Page Fetcher (6 subscribers) - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)*/
const Device UA2573950 = {S48190, 181, { &P17595,&P17297,&P22921,&P18092 }};	/*Useragent - NewsBlur Page Fetcher (5 subscribers) - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)*/
const Device UA2574262 = {S48191, 181, { &P17595,&P17297,&P22921,&P18092 }};	/*Useragent - NewsBlur Page Fetcher (0 subscribers) - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)*/
const Device UA2589222 = {S48192, 181, { &P17595,&P17297,&P22921,&P18092 }};	/*Useragent - NewsBlur Page Fetcher (7 subscribers) - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)*/
const Device UA2651805 = {S48193, 182, { &P17595,&P17297,&P22921,&P18092 }};	/*Useragent - NewsBlur Page Fetcher (56 subscribers) - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)*/
const Device UA2658420 = {S48194, 168, { &P17595,&P17297,&P22921,&P18092 }};	/*Useragent - NewsBlur Content Fetcher - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3)*/
const Device UA2698481 = {S48195, 164, { &P17595,&P17297,&P22921,&P18092 }};	/*Useragent - NewsBlur Feed Fetcher - 4 subscribers - http://www.newsblur.com (Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/536.2.3 (KHTML, like Gecko) Version/5.2)*/
const Device UA2713956 = {S48196, 138, { &P22584,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337M Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2717898 = {S48197, 131, { &P22584,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337M Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA46823 = {S48198, 152, { &P15488,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonIS11S Build/3.0.1.C.1.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46824 = {S48199, 140, { &P15488,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11S Build/3.0.1.C.1.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46830 = {S48200, 139, { &P15488,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11S Build/3.0.1.C.2.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46834 = {S48201, 151, { &P15488,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonIS11S Build/3.0.1.C.2.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59931 = {S48202, 139, { &P15488,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11S Build/3.0.1.C.1.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59932 = {S48203, 151, { &P15488,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonIS11S Build/3.0.1.C.1.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59933 = {S48204, 140, { &P15488,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11S Build/3.0.1.C.0.80) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59934 = {S48205, 152, { &P15488,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonIS11S Build/3.0.1.C.0.80) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70593 = {S48206, 152, { &P15488,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonIS11S Build/3.0.1.C.3.31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70641 = {S48207, 140, { &P15488,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11S Build/3.0.1.C.3.31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA253559 = {S48208, 153, { &P15488,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonIS11S Build/4.0.1.B.0.112) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626134 = {S48209, 141, { &P15488,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; IS11S Build/4.0.1.B.0.112) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2256905 = {S48210, 141, { &P15488,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; IS11S Build/4.0.1.B.0.117) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2256919 = {S48211, 153, { &P15488,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonIS11S Build/4.0.1.B.0.117) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2571969 = {S48212, 153, { &P15488,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonIS11S Build/4.0.1.B.0.118) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2584108 = {S48213, 141, { &P15488,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; IS11S Build/4.0.1.B.0.118) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2935417 = {S48214, 65, { &P15488,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; IS11S Build/4.0.1.B.0.118)*/
const Device UA92638 = {S48215, 42, { &P15364,&P17350,&P4618,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.0B; Win32)*/
const Device UA94703 = {S48216, 47, { &P15364,&P17350,&P4618,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.0B; Windows NT)*/
const Device UA141171 = {S48217, 61, { &P15364,&P17350,&P4618,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.02; Update a; AK; Windows NT)*/
const Device UA141275 = {S48218, 47, { &P15364,&P17350,&P4618,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.01; Windows NT)*/
const Device UA2509357 = {S48219, 52, { &P15364,&P17350,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; 68K; Trident/1.0)*/
const Device UA2580072 = {S48220, 59, { &P15364,&P17350,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; 68K; WOW64; Trident/1.0)*/
const Device UA2658759 = {S48221, 64, { &P15364,&P17350,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; 68K; Win64; x64; Trident/2.0)*/
const Device UA2792546 = {S48222, 57, { &P15364,&P17350,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; 68K; SV1; Trident/5.0)*/
const Device UA807255 = {S48223, 98, { &P15364,&P17838,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-GB; rv:1.9.2.13) Gecko/X Ubuntu/9.10 (karmic) Firefox/3.6.13*/
const Device UA2566928 = {S48224, 139, { &P18295,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One X Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567103 = {S48225, 139, { &P18295,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One X Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2431237 = {S48226, 89, { &P18115,&P5764,&P18525,&P18092 }};	/*Useragent - Opera/9.80 (BlackBerry; Opera Mini/6.0.24209/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA454504 = {S48227, 127, { &P15767,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9B176 Safari/6533.18.5*/
const Device UA454681 = {S48228, 153, { &P15767,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X; zh-cn Model:iPad2,1) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA456750 = {S48229, 124, { &P15767,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X; 6534.46) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5*/
const Device UA615707 = {S48230, 138, { &P15767,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_1_1 like Mac OS X; en_us) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA624154 = {S48231, 141, { &P15767,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_1_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA626276 = {S48232, 136, { &P15767,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X; en_us) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA640809 = {S48233, 139, { &P15767,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA663888 = {S48234, 129, { &P15767,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9B206 Safari/6533.18.5*/
const Device UA807030 = {S48235, 126, { &P15767,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_1_1 like Mac OS X; 6534.46) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5*/
const Device UA2043387 = {S48236, 128, { &P15767,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/6533.18.5 DiigoBrowser*/
const Device UA2197906 = {S48237, 158, { &P15767,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/6533.18.5 DiigoBrowser Safari/6533.18.5 DiigoBrowser*/
const Device UA2229705 = {S48238, 126, { &P15767,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/6533.18.5 DiigoBrowser*/
const Device UA2352084 = {S48239, 188, { &P15767,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/6533.18.5 DiigoBrowser Safari/6533.18.5 DiigoBrowser Safari/6533.18.5 DiigoBrowser*/
const Device UA2936920 = {S48240, 161, { &P15767,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_1_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5*/
const Device UA2660593 = {S48241, 135, { &P23836,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; 801a Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA627600 = {S48242, 150, { &P18510,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ALCATEL ONE TOUCH 918N Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA895029 = {S48243, 151, { &P18510,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ALCATEL ONE TOUCH 918N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033399 = {S48244, 150, { &P18510,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; ALCATEL_one_touch_918N Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2670733 = {S48245, 130, { &P18437,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8005 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2671215 = {S48246, 130, { &P18437,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8020 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676173 = {S48247, 130, { &P18437,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8000 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676445 = {S48248, 130, { &P18437,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8020 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676927 = {S48249, 130, { &P18437,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8000 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2204765 = {S48250, 134, { &P18665,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N861 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1960923 = {S48251, 138, { &P20287,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MOT-XT535 Build/V1.540) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1996372 = {S48252, 138, { &P20287,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; MOT-XT535 Build/V1.60A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025437 = {S48253, 134, { &P20287,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT535 Build/V1.540) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028553 = {S48254, 127, { &P20287,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT535 Build/V1.47G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2029341 = {S48255, 134, { &P20287,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; XT535 Build/V1.50A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030813 = {S48256, 138, { &P20287,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; MOT-XT535 Build/V1.57A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031928 = {S48257, 138, { &P20287,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MOT-XT535 Build/V1.47G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033289 = {S48258, 138, { &P20287,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MOT-XT535 Build/V1.50A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2228128 = {S48259, 138, { &P20287,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MOT-XT535 Build/V1.50D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2464467 = {S48260, 134, { &P20287,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; XT535 Build/V1.57A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2484376 = {S48261, 138, { &P20287,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; MOT-XT535 Build/V1.610) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2497646 = {S48262, 134, { &P20287,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; XT535 Build/V1.60A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2520587 = {S48263, 134, { &P20287,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT535 Build/V1.47G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2787179 = {S48264, 58, { &P20287,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; XT535 Build/V1.540)*/
const Device UA2820482 = {S48265, 138, { &P20287,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MOT-XT535 Build/V1.47I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2848337 = {S48266, 134, { &P20287,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; XT535 Build/V1.610) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3117546 = {S48267, 138, { &P20287,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MOT-XT535 Build/V1.47H) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA616453 = {S48268, 46, { &P12280,&P18432,&P17470,&P18092 }};	/*Useragent - Facebook 4110.0 (iPhone; iPhone OS 6.0; en_US)*/
const Device UA2356448 = {S48269, 61, { &P12280,&P18432,&P17470,&P18092 }};	/*Useragent - Appcelerator Titanium/1.7.2 (iPhone/6.0.1; iPhone OS; en_US;)*/
const Device UA2356778 = {S48270, 51, { &P12280,&P18432,&P17470,&P18092 }};	/*Useragent - FeeddlerRSS 1.12.5 (iPhone; iPhone OS 6.0.1; en_US)*/
const Device UA2431322 = {S48271, 44, { &P12280,&P18432,&P17470,&P18092 }};	/*Useragent - Pulse 3.0.3 (iPhone; iPhone OS 6.0.2; en_US)*/
const Device UA2402711 = {S48272, 43, { &P11684,&P5129,&P24673,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UCWEB8.7.0.187/50/999*/
const Device UA2404814 = {S48273, 131, { &P11684,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5800 XpressMusic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2430801 = {S48274, 53, { &P11684,&P5129,&P24673,&P18092 }};	/*Useragent - Nokia5800 XpressMusicUCBrowser/8.7.1.234/50/444/UCWEB*/
const Device UA2458576 = {S48275, 104, { &P11684,&P5129,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5800d-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA145084 = {S48276, 174, { &P15364,&P17285,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Maxthon 2.0)*/
const Device UA237471 = {S48277, 221, { &P15364,&P17285,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; InfoPath.2; .NET CLR 3.5.20706; .NET CLR 3.0.04506; MAXTHON 2.0)*/
const Device UA291802 = {S48278, 153, { &P15364,&P17285,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Maxthon 2.0)*/
const Device UA435690 = {S48279, 164, { &P15364,&P17285,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; MAXTHON 2.0)*/
const Device UA575484 = {S48280, 189, { &P15364,&P17285,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; Win64; x64; Trident/4.0; .NET CLR 2.0.50727; SLCC1; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; Maxthon 2.0)*/
const Device UA702450 = {S48281, 88, { &P15364,&P17285,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; SLCC2; Maxthon 2.0; DigExt; Zune 4.7)*/
const Device UA1858720 = {S48282, 87, { &P15364,&P17285,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SV1; .NET CLR 1.1.4322; MAXTHON 2.0)*/
const Device UA2128209 = {S48283, 177, { &P15364,&P17285,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.04506; Media Center PC 5.0; .NET CLR 1.1.4322; .NET CLR 3.5.21022; MAXTHON 2.0)*/
const Device UA2129550 = {S48284, 157, { &P15364,&P17285,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; InfoPath.2; eMusic DLM/3; MAXTHON 2.0)*/
const Device UA2210403 = {S48285, 181, { &P15364,&P17285,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET4.0C; .NET CLR 3.0.30729; Maxthon 2.0)*/
const Device UA2671315 = {S48286, 138, { &P15364,&P17285,&P22372,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB0.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MAXTHON 2.0)*/
const Device UA2756415 = {S48287, 129, { &P18556,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BNTV600 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3096541 = {S48288, 133, { &P15767,&P18432,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A523 Safari/8536.25*/
const Device UA3096657 = {S48289, 131, { &P15767,&P18432,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A403 Safari/8536.25*/
const Device UA3096683 = {S48290, 133, { &P15767,&P18432,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A550 Safari/8536.25*/
const Device UA3096724 = {S48291, 134, { &P15767,&P18432,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A8426 Safari/8536.25*/
const Device UA3109513 = {S48292, 131, { &P15767,&P18432,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A407 Safari/8536.25*/
const Device UA3109542 = {S48293, 134, { &P15767,&P18432,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A8500 Safari/8536.25*/
const Device UA3115494 = {S48294, 131, { &P15767,&P18432,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A406 Safari/8536.25*/
const Device UA2973485 = {S48295, 140, { &P18582,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT28at Build/6.1.C.1.111) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973900 = {S48296, 137, { &P18582,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT28h Build/6.1.E.3.7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020973 = {S48297, 138, { &P18582,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT28i Build/6.1.E.2.68) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2043046 = {S48298, 72, { &P12280,&P18432,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10A403 (iPhone; U; CPU OS 6_0 like Mac OS X; en_us)*/
const Device UA2242184 = {S48299, 74, { &P12280,&P18432,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10A523 (iPhone; U; CPU OS 6_0_1 like Mac OS X; en_us)*/
const Device UA2437602 = {S48300, 74, { &P12280,&P18432,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10A551 (iPhone; U; CPU OS 6_0_2 like Mac OS X; en_us)*/
const Device UA2200369 = {S48301, 134, { &P19907,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C5155 Build/IML77) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2228727 = {S48302, 134, { &P19907,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Rise Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2566690 = {S48303, 139, { &P18280,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566883 = {S48304, 139, { &P18280,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One S Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566903 = {S48305, 139, { &P18280,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC VLE_U Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583213 = {S48306, 139, { &P18280,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC VLE_U Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA628271 = {S48307, 114, { &P15228,&P5043,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-03/07.48; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.9*/
const Device UA2195621 = {S48308, 131, { &P15767,&P18432,&P22898,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.0 Mobile/10A403 Safari/8536.25*/
const Device UA2719305 = {S48309, 139, { &P22602,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-S738C Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA20209 = {S48310, 158, { &P12249,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-mx; SonyEricssonU20a Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA49035 = {S48311, 147, { &P12249,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; U20a Build/2.0.1.A.0.47) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA49042 = {S48312, 146, { &P12249,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-mx; U20a Build/2.0.A.0.504) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA56847 = {S48313, 147, { &P12249,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-co; U20a Build/2.0.2.A.0.24) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA58648 = {S48314, 146, { &P12249,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-mx; U20a Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA722358 = {S48315, 68, { &P12249,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; U20a Build/2.1.1.A.0.6)*/
const Device UA3065131 = {S48316, 129, { &P23955,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5210 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA53554 = {S48317, 144, { &P14031,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I5503 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA53798 = {S48318, 145, { &P14031,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; GT-I5503T Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA634400 = {S48319, 66, { &P14031,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I5503 Build/ERE27)*/
const Device UA699357 = {S48320, 130, { &P14031,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I5503 Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2019095 = {S48321, 67, { &P14031,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I5503T Build/ERE27)*/
const Device UA2393013 = {S48322, 142, { &P14031,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; GT-I5503 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2202577 = {S48323, 81, { &P17968,&P5629,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.4; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2202581 = {S48324, 81, { &P17968,&P5629,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.3; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2202582 = {S48325, 81, { &P17968,&P5629,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.5; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2202587 = {S48326, 81, { &P17968,&P5629,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.6; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2237758 = {S48327, 81, { &P17968,&P5629,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.2; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2262384 = {S48328, 81, { &P17968,&P5629,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.7; Linux; Opera Mobi/46154) Presto/2.11.355 Version/12.10*/
const Device UA2598100 = {S48329, 81, { &P17968,&P5629,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.1; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2698358 = {S48330, 81, { &P17968,&P5629,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.8; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA13793 = {S48331, 77, { &P12473,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2730c-1*/
const Device UA13817 = {S48332, 64, { &P12473,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA32166 = {S48333, 62, { &P12473,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2730c/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63970 = {S48334, 65, { &P12473,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2730c-1b/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2314075 = {S48335, 139, { &P20975,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-AS730 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654181 = {S48336, 61, { &P20975,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-AS730 Build/IMM76L)*/
const Device UA2469372 = {S48337, 168, { &P18645,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-L710 Build/JOP40D; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469656 = {S48338, 168, { &P18645,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I747 Build/JOP40D; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2532160 = {S48339, 167, { &P18645,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I747 Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2532472 = {S48340, 167, { &P18645,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L710 Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2601891 = {S48341, 168, { &P18645,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530U Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2856602 = {S48342, 167, { &P18645,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; GT-I9305 Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2974252 = {S48343, 173, { &P18645,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I747 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/d2att*/
const Device UA2985482 = {S48344, 169, { &P18645,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-R530U Build/JOP40D; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3021537 = {S48345, 173, { &P18645,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L710 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/d2spr*/
const Device UA3092448 = {S48346, 173, { &P18645,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9305 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/i9305*/
const Device UA2570155 = {S48347, 158, { &P18442,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2570594 = {S48348, 158, { &P18442,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2772747 = {S48349, 157, { &P18442,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2773253 = {S48350, 157, { &P18442,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2773279 = {S48351, 165, { &P18442,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2773909 = {S48352, 157, { &P18442,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2774042 = {S48353, 157, { &P18442,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2774049 = {S48354, 157, { &P18442,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2783292 = {S48355, 157, { &P18442,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7105 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2783704 = {S48356, 158, { &P18442,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2796776 = {S48357, 158, { &P18442,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2937752 = {S48358, 157, { &P18442,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R950 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA781653 = {S48359, 140, { &P18296,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  HTC One V Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA903911 = {S48360, 139, { &P18296,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One V Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2353032 = {S48361, 135, { &P18296,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; One V Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2503362 = {S48362, 139, { &P18296,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One V Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA609404 = {S48363, 142, { &P12841,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  HTC PH39100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA900908 = {S48364, 141, { &P12841,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC PH39100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA23040 = {S48365, 89, { &P15364,&P5890,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.1; U; en-us) AppleWebKit/413 (KHTML, like Gecko) Safari/413 es65*/
const Device UA87657 = {S48366, 81, { &P15364,&P5890,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.1; U; ar) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA87716 = {S48367, 81, { &P15364,&P5890,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.1; U; id) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA52036 = {S48368, 151, { &P17941,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A854 Build/MILER_X1_01.22.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA55979 = {S48369, 151, { &P17941,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A854 Build/MILER_X1_00.26.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA55983 = {S48370, 151, { &P17941,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; A854 Build/MILER_X1_37.10.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA56796 = {S48371, 151, { &P17941,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A854 Build/MILER_X1_01.22.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA67442 = {S48372, 151, { &P17941,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A854 Build/MILER_X1_38.31.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA90308 = {S48373, 151, { &P17941,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A854 Build/MILER_X1_00.26.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019314 = {S48374, 73, { &P17941,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; A854 Build/MILER_X1_37.10.0)*/
const Device UA2020132 = {S48375, 156, { &P17941,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A854 Build/MILER_X1_00.26.1) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA69329 = {S48376, 128, { &P14973,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; LG-V909 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA566499 = {S48377, 57, { &P14973,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; LG-V909 Build/HMJ37)*/
const Device UA2017653 = {S48378, 82, { &P17595,&P17296,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en) Presto/2.10.289 Version/12.02*/
const Device UA2349587 = {S48379, 90, { &P17595,&P17296,&P19961,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6.8; en; rv:2.0) Gecko/X Firefox/4.0 Opera 12.02*/
const Device UA2947005 = {S48380, 165, { &P18645,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2947549 = {S48381, 157, { &P18645,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2965159 = {S48382, 158, { &P18645,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747M Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2966401 = {S48383, 158, { &P18645,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3014138 = {S48384, 158, { &P18645,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3031250 = {S48385, 165, { &P18645,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3031519 = {S48386, 158, { &P18645,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3032882 = {S48387, 157, { &P18645,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3038894 = {S48388, 158, { &P18645,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3060683 = {S48389, 157, { &P18645,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3092766 = {S48390, 158, { &P18645,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA43357 = {S48391, 141, { &P15297,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; MT15a Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43365 = {S48392, 153, { &P15297,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; SonyEricssonMT15a Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43370 = {S48393, 151, { &P15297,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; pt-br; SonyEricssonMT15a Build/3.0.A.2.130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59287 = {S48394, 150, { &P15297,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; sv-se; SonyEricssonMT15a Build/3.0.A.2.15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60641 = {S48395, 151, { &P15297,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; sv-se; SonyEricssonMT15a Build/4.0.A.2.286) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76927 = {S48396, 139, { &P15297,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-cr; MT15a Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78736 = {S48397, 151, { &P15297,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-cr; SonyEricssonMT15a Build/4.0.A.2.368) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA347376 = {S48398, 152, { &P15297,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; SonyEricssonMT15a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA643427 = {S48399, 140, { &P15297,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; MT15a Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA683479 = {S48400, 152, { &P15297,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SonyEricssonMT15a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA707212 = {S48401, 140, { &P15297,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MT15a Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1904620 = {S48402, 64, { &P15297,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; MT15a Build/4.0.2.A.0.42)*/
const Device UA2021494 = {S48403, 65, { &P15297,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; MT15a Build/3.0.1.A.0.145)*/
const Device UA2373860 = {S48404, 151, { &P15297,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-gb; SonyEricssonMT15a Build/3.0.A.2.182) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA371357 = {S48405, 82, { &P15364,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0*/
const Device UA391983 = {S48406, 83, { &P15364,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0)*/
const Device UA756611 = {S48407, 121, { &P15364,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; ZTE; Windows Phone - Internet 7; SFR)*/
const Device UA878608 = {S48408, 97, { &P15364,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; PC40200)*/
const Device UA925745 = {S48409, 101, { &P15364,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Nokia; Champagne)*/
const Device UA2260438 = {S48410, 106, { &P15364,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; ZTE; USCCN859; USC-US)*/
const Device UA2384416 = {S48411, 103, { &P15364,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Nokia; Radar C110e)*/
const Device UA2645784 = {S48412, 98, { &P15364,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; samsung; i917)*/
const Device UA2651657 = {S48413, 95, { &P15364,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; ZTE; V965W)*/
const Device UA2693900 = {S48414, 106, { &P15364,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Nokia; 7 Trophy T8686)*/
const Device UA2727950 = {S48415, 96, { &P15364,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; ZTE; Mimosa)*/
const Device UA2746297 = {S48416, 99, { &P15364,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Nokia; mwp6985)*/
const Device UA3000417 = {S48417, 95, { &P15364,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; 909)*/
const Device UA3021839 = {S48418, 106, { &P15364,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; HTCPO881; SPRINT)*/
const Device UA3100983 = {S48419, 103, { &P15364,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; SCH-R860U)*/
const Device UA2532480 = {S48420, 140, { &P18072,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E120S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2560202 = {S48421, 140, { &P18072,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SHV-E120K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2563536 = {S48422, 140, { &P18072,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SHV-E120L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2694321 = {S48423, 62, { &P18072,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E120L Build/JZO54K)*/
const Device UA2711363 = {S48424, 147, { &P18072,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E120K/KKJMD2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2731974 = {S48425, 62, { &P18072,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E120S Build/JZO54K)*/
const Device UA2779656 = {S48426, 62, { &P18072,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E120K Build/JZO54K)*/
const Device UA2981815 = {S48427, 140, { &P18072,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SHV-E120K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3062361 = {S48428, 147, { &P18072,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E120K/KKJMH1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA197608 = {S48429, 95, { &P14632,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; T8788)*/
const Device UA628979 = {S48430, 98, { &P14632,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Mondrian)*/
const Device UA815418 = {S48431, 98, { &P14632,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Surround)*/
const Device UA925787 = {S48432, 100, { &P14632,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; 7 Surround)*/
const Device UA925896 = {S48433, 106, { &P14632,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; 7 Mondrian T8788)*/
const Device UA1988452 = {S48434, 132, { &P18437,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2233052 = {S48435, 132, { &P18437,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8005 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2476432 = {S48436, 133, { &P18437,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  GT-N8000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA911879 = {S48437, 139, { &P18021,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A770K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026464 = {S48438, 61, { &P18021,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-A770K Build/IMM76D)*/
const Device UA2027870 = {S48439, 137, { &P18021,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A770K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2595607 = {S48440, 137, { &P18021,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A770K Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2741124 = {S48441, 144, { &P18021,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A770K/2.23 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2868603 = {S48442, 142, { &P18021,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A770K/2.23 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA229830 = {S48443, 142, { &P15201,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Sensation_Z710e; en-in) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA333407 = {S48444, 139, { &P15201,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Sensation_4G; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA720172 = {S48445, 133, { &P15201,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC Z710e; zh-cn) AndroidWebKit/533.16 (KHTML, Like Gecko) Version/5.0 Safari/533.1*/
const Device UA828945 = {S48446, 117, { &P15364,&P17285,&P18333,&P18092 }};	/*Useragent - iTunes/10.6.1 (Windows; Microsoft Windows Vista Home Basic Edition Service Pack 1 (Build 6001)) AppleWebKit/534.54.16*/
const Device UA867446 = {S48447, 102, { &P15364,&P17285,&P18333,&P18092 }};	/*Useragent - iTunes/10.6.1 (Windows; Microsoft Windows Vista Home Basic Edition (Build 6000)) AppleWebKit/534.54.16*/
const Device UA1958796 = {S48448, 114, { &P15364,&P17285,&P18333,&P18092 }};	/*Useragent - iTunes/10.6.3 (Windows; Microsoft Windows Vista Business Edition Service Pack 1 (Build 6001)) AppleWebKit/534.57.2*/
const Device UA2042637 = {S48449, 116, { &P15364,&P17285,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows Vista Home Premium Edition Service Pack 2 (Build 6002)) AppleWebKit/536.26.9*/
const Device UA2203593 = {S48450, 124, { &P15364,&P17285,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows Vista x64 x64 Home Premium Edition Service Pack 2 (Build 6002)) AppleWebKit/536.26.9*/
const Device UA2204017 = {S48451, 112, { &P15364,&P17285,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows Vista Business Edition Service Pack 2 (Build 6002)) AppleWebKit/536.26.9*/
const Device UA2204020 = {S48452, 112, { &P15364,&P17285,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows Vista Ultimate Edition Service Pack 2 (Build 6002)) AppleWebKit/536.26.9*/
const Device UA2204813 = {S48453, 114, { &P15364,&P17285,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows Vista Home Basic Edition Service Pack 2 (Build 6002)) AppleWebKit/536.26.9*/
const Device UA2204833 = {S48454, 116, { &P15364,&P17285,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows Vista Home Premium Edition Service Pack 1 (Build 6001)) AppleWebKit/536.26.9*/
const Device UA2205233 = {S48455, 101, { &P15364,&P17285,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows Vista Home Premium Edition (Build 6000)) AppleWebKit/536.26.9*/
const Device UA2232967 = {S48456, 120, { &P15364,&P17285,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows Vista x64 x64 Ultimate Edition Service Pack 2 (Build 6002)) AppleWebKit/536.26.9*/
const Device UA2242312 = {S48457, 120, { &P15364,&P17285,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows Vista x64 x64 Business Edition Service Pack 2 (Build 6002)) AppleWebKit/536.26.9*/
const Device UA2596532 = {S48458, 126, { &P15364,&P17285,&P18333,&P18092 }};	/*Useragent - iTunes/11.0.1 (Windows; Microsoft Windows Vista x64 x64 Home Premium Edition Service Pack 1 (Build 6001)) AppleWebKit/536.27.1*/
const Device UA251720 = {S48459, 63, { &P15364,&P17285,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; en) Presto/2.9.168 Version/11.52*/
const Device UA251886 = {S48460, 86, { &P15364,&P17285,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.9.168 Version/11.52*/
const Device UA251987 = {S48461, 86, { &P15364,&P17285,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.9.168 Version/11.52*/
const Device UA344850 = {S48462, 86, { &P15364,&P17285,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.9.168 Version/11.52*/
const Device UA575546 = {S48463, 85, { &P15364,&P17285,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4157); ru) Presto/2.9.168 Version/11.52*/
const Device UA576479 = {S48464, 72, { &P15364,&P17285,&P22542,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.52*/
const Device UA686590 = {S48465, 85, { &P15364,&P17285,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.9 (build 4876); ru) Presto/2.9.168 Version/11.52*/
const Device UA869525 = {S48466, 86, { &P15364,&P17285,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03790); ru) Presto/2.9.168 Version/11.52*/
const Device UA1903036 = {S48467, 85, { &P15364,&P17285,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4664); ru) Presto/2.9.168 Version/11.52*/
const Device UA2301602 = {S48468, 86, { &P15364,&P17285,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03686); ru) Presto/2.9.168 Version/11.52*/
const Device UA2302246 = {S48469, 86, { &P15364,&P17285,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.4 (build 02647); ru) Presto/2.9.168 Version/11.52*/
const Device UA2304294 = {S48470, 86, { &P15364,&P17285,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.6 (build 03278); ru) Presto/2.9.168 Version/11.52*/
const Device UA2369169 = {S48471, 64, { &P15364,&P17285,&P22542,&P18092 }};	/*Useragent - Opera/11.52 (Windows NT 6.0; U; en) Presto/2.9.168 Version/11.52*/
const Device UA901703 = {S48472, 122, { &P15364,&P7133,&P22431,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5 Comodo_Dragon/19.2.0.0*/
const Device UA901784 = {S48473, 129, { &P15364,&P7133,&P22431,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5 Comodo_Dragon/19.2.0.0*/
const Device UA2605791 = {S48474, 122, { &P12280,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0*/
const Device UA2605795 = {S48475, 122, { &P12280,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Flipboard/2.0*/
const Device UA2679611 = {S48476, 124, { &P12280,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.1*/
const Device UA2679618 = {S48477, 124, { &P12280,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Flipboard/2.0.1*/
const Device UA2710761 = {S48478, 124, { &P12280,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Flipboard/2.0.1*/
const Device UA2739311 = {S48479, 124, { &P12280,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.2*/
const Device UA2739317 = {S48480, 124, { &P12280,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Flipboard/2.0.2*/
const Device UA2796730 = {S48481, 124, { &P12280,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Flipboard/2.0.3*/
const Device UA2796732 = {S48482, 124, { &P12280,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.3*/
const Device UA2888582 = {S48483, 124, { &P12280,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Flipboard/2.0.4*/
const Device UA2888614 = {S48484, 124, { &P12280,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.4*/
const Device UA3038195 = {S48485, 124, { &P12280,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.5*/
const Device UA3038785 = {S48486, 124, { &P12280,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Flipboard/2.0.5*/
const Device UA3067719 = {S48487, 124, { &P12280,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Flipboard/2.0.6*/
const Device UA3067724 = {S48488, 124, { &P12280,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.6*/
const Device UA90840 = {S48489, 127, { &P15767,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9A334 Safari/6533.18.5*/
const Device UA359969 = {S48490, 151, { &P15767,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5 like Mac OS X; zh-cn Model:iPad2,1) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA456324 = {S48491, 138, { &P15767,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_0_1 like Mac OS X; en_us) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA589744 = {S48492, 150, { &P15767,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_0 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5 DiigoBrowser*/
const Device UA635976 = {S48493, 129, { &P15767,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9A405 Safari/6533.18.5*/
const Device UA640615 = {S48494, 139, { &P15767,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_0 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA662085 = {S48495, 141, { &P15767,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_0_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA704528 = {S48496, 126, { &P15767,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_0_1 like Mac OS X; 6534.46) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5*/
const Device UA800333 = {S48497, 136, { &P15767,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_0 like Mac OS X; en_us) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA915857 = {S48498, 129, { &P15767,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9A5259f Safari/6533.18.5*/
const Device UA2232464 = {S48499, 128, { &P15767,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/6533.18.5 DiigoBrowser*/
const Device UA2481400 = {S48500, 137, { &P15767,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_0 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J3 Safari/6533.18.5*/
const Device UA2505289 = {S48501, 138, { &P15767,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0(iPad; U; CPU OS 5_0 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F191 Safari/6533.18.5*/
const Device UA2748440 = {S48502, 127, { &P15767,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0 Mobile/9A405 Safari/7534.48.3*/
const Device UA2984208 = {S48503, 139, { &P15767,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_0 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9A5248d Safari/6533.18.5*/
const Device UA436683 = {S48504, 133, { &P15900,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; XT311 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA629594 = {S48505, 144, { &P15900,&P5629,&P15760,&P18092 }};	/*Useragent - MOT-XT311/Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; XT311 Build/V4.36M) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA731017 = {S48506, 144, { &P15900,&P5629,&P15760,&P18092 }};	/*Useragent - MOT-XT311/Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; XT311 Build/V4.33B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA736045 = {S48507, 144, { &P15900,&P5629,&P15760,&P18092 }};	/*Useragent - MOT-XT311/Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; XT311 Build/V4.36J) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA741002 = {S48508, 144, { &P15900,&P5629,&P15760,&P18092 }};	/*Useragent - MOT-XT311/Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; XT311 Build/V4.40C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA851598 = {S48509, 144, { &P15900,&P5629,&P15760,&P18092 }};	/*Useragent - MOT-XT311/Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; XT311 Build/V4.53B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA855008 = {S48510, 144, { &P15900,&P5629,&P15760,&P18092 }};	/*Useragent - MOT-XT311/Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; XT311 Build/V4.36B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1904094 = {S48511, 144, { &P15900,&P5629,&P15760,&P18092 }};	/*Useragent - MOT-XT311/Mozilla/5.0 (Linux; U; Android 2.3.4; ro-ro; XT311 Build/V4.36H) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1937843 = {S48512, 134, { &P15900,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; XT311 Build/V4.480) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1941003 = {S48513, 134, { &P15900,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; XT311 Build/V4.530) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1962836 = {S48514, 134, { &P15900,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; XT311 Build/V4.26Q) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028135 = {S48515, 144, { &P15900,&P5629,&P15760,&P18092 }};	/*Useragent - MOT-XT311/Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; XT311 Build/V4.54B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2685524 = {S48516, 144, { &P15900,&P5629,&P15760,&P18092 }};	/*Useragent - MOT-XT311/Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; XT311 Build/V4.33A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2508158 = {S48517, 154, { &P12280,&P19826,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B144 Safari/7534.48.3*/
const Device UA2510133 = {S48518, 154, { &P12280,&P19826,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B142 Safari/7534.48.3*/
const Device UA2511506 = {S48519, 154, { &P12280,&P19826,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B141 Safari/7534.48.3*/
const Device UA2531241 = {S48520, 154, { &P12280,&P19826,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B143 Safari/7534.48.3*/
const Device UA2533313 = {S48521, 156, { &P12280,&P19826,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B145 Safari/7534.48.3*/
const Device UA2535164 = {S48522, 156, { &P12280,&P19826,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B146 Safari/7534.48.3*/
const Device UA2606994 = {S48523, 156, { &P12280,&P19826,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B329 Safari/7534.48.3*/
const Device UA2768706 = {S48524, 156, { &P12280,&P19826,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_4 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B350 Safari/7534.48.3*/
const Device UA134735 = {S48525, 131, { &P16151,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-US) AppleWebKit/528.5+ (KHTML, like Gecko, Safari/528.5+) Version/4.0 Kindle/3.0 (screen 600x800; rotate)*/
const Device UA2755530 = {S48526, 139, { &P18447,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.1.140) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2749786 = {S48527, 139, { &P22872,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S5282 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2893003 = {S48528, 61, { &P22872,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S5282 Build/JZO54K)*/
const Device UA3000532 = {S48529, 137, { &P22872,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-S5282 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA6900 = {S48530, 104, { &P11698,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonW995/R1HA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.5*/
const Device UA2390427 = {S48531, 207, { &P12280,&P17134,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16 AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2*/
const Device UA2741542 = {S48532, 139, { &P18682,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-M950 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2753009 = {S48533, 61, { &P18682,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SPH-M950 Build/JZO54K)*/
const Device UA1966233 = {S48534, 142, { &P18697,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I415 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033887 = {S48535, 139, { &P18697,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I415 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2444833 = {S48536, 61, { &P18697,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-I415 Build/IMM76D)*/
const Device UA2421463 = {S48537, 139, { &P19902,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530M Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2467840 = {S48538, 139, { &P19902,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-R530M Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2434173 = {S48539, 136, { &P17595,&P17296,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/21.0.1180.75 Safari/537.1*/
const Device UA2674327 = {S48540, 143, { &P17595,&P17296,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.1; Maxthon (4.0.4.1000);*/
const Device UA3026825 = {S48541, 138, { &P17595,&P17296,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.22 (KHTML, like Gecko) Maxthon/4.0.6.3000 Chrome/25.0.1364.99 Safari/537.22*/
const Device UA667902 = {S48542, 121, { &P12989,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia303/5.0 (14.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA2167203 = {S48543, 128, { &P19862,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A110 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2403100 = {S48544, 128, { &P19862,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A110 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2665502 = {S48545, 143, { &P19862,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Micromax A110 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2564882 = {S48546, 139, { &P19870,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-L710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2668720 = {S48547, 61, { &P19870,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-L710 Build/JZO54K)*/
const Device UA3065477 = {S48548, 137, { &P21607,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; VS870 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2405010 = {S48549, 122, { &P12193,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaC5-00.2) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2405337 = {S48550, 120, { &P12193,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaC5-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2422710 = {S48551, 102, { &P12193,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac5-00) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2427193 = {S48552, 104, { &P12193,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac5-00.2) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA613623 = {S48553, 152, { &P18317,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  ASUS Transformer Pad TF300T Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA900677 = {S48554, 151, { &P18317,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300T Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA908789 = {S48555, 152, { &P18317,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300TG Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1987428 = {S48556, 146, { &P18317,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Transformer Pad TF300T Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2026070 = {S48557, 148, { &P18317,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Transformer Prime TF300T Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2027965 = {S48558, 152, { &P18317,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300TL Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2033246 = {S48559, 139, { &P18317,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ASUS Pad TF300T Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA170725 = {S48560, 145, { &P17930,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SCH-I100 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA377386 = {S48561, 67, { &P17930,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SCH-I100 Build/ECLAIR)*/
const Device UA2462479 = {S48562, 130, { &P19782,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B1-A71 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2234087 = {S48563, 153, { &P20283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; AZ210A-ORANGE Build/AZ210A_ICS_02.19I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257431 = {S48564, 153, { &P20283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; AZ210A-ORANGE Build/AZ210A_ICS_02.27I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2354383 = {S48565, 146, { &P20283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; AZ210A Build/AZ210A_ICS_02.19I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2367027 = {S48566, 153, { &P20283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; AZ210B-ORANGE Build/AZ210B_ICS_02.27I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2367688 = {S48567, 146, { &P20283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; AZ210A Build/AZ210A_ICS_02.27I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2594428 = {S48568, 70, { &P20283,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; AZ210A Build/AZ210A_ICS_02.27I)*/
const Device UA2653218 = {S48569, 146, { &P20283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; AZ210B Build/AZ210B_ICS_02.27I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2879631 = {S48570, 153, { &P20283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; AZ210A-ORANGE Build/AZ210A_ICS_02.34I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2901197 = {S48571, 146, { &P20283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; AZ210A Build/AZ210A_ICS_02.34I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2972004 = {S48572, 153, { &P20283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; AZ210B-ORANGE Build/AZ210B_ICS_02.34I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3053204 = {S48573, 146, { &P20283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; AZ210B Build/AZ210B_ICS_02.34I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA896480 = {S48574, 147, { &P19891,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA902537 = {S48575, 146, { &P19891,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA905100 = {S48576, 146, { &P19891,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I717 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2545224 = {S48577, 140, { &P19891,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA856560 = {S48578, 152, { &P13505,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB525 Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031458 = {S48579, 167, { &P13505,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; MB525 Build/WhiteRabbitEdition-1.5.2; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2454550 = {S48580, 167, { &P13505,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; MB525 Build/WhiteRabbitEdition-1.7.1; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2235402 = {S48581, 138, { &P18559,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LG730 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2405030 = {S48582, 120, { &P12449,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE5-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2423298 = {S48583, 102, { &P12449,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiae5-00) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2423901 = {S48584, 32, { &P12449,&P5890,&P24673,&P18092 }};	/*Useragent - NOKIAE5-00/UCWEB8.7.0.187/28/999*/
const Device UA2425728 = {S48585, 42, { &P12449,&P5890,&P24673,&P18092 }};	/*Useragent - NOKIAE5-00UCBrowser/8.7.1.234/28/444/UCWEB*/
const Device UA2483523 = {S48586, 122, { &P12449,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE5-00.2) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2566960 = {S48587, 145, { &P18645,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I747 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567015 = {S48588, 137, { &P18645,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L710 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567134 = {S48589, 137, { &P18645,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I747 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567208 = {S48590, 138, { &P18645,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-I747 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567948 = {S48591, 138, { &P18645,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530U Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567951 = {S48592, 137, { &P18645,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9305 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2570777 = {S48593, 146, { &P18645,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SAMSUNG-SGH-I747 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2583143 = {S48594, 138, { &P18645,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I747 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583200 = {S48595, 130, { &P18645,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L710 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2604497 = {S48596, 138, { &P18645,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SPH-L710 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2666719 = {S48597, 138, { &P18645,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530C Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667180 = {S48598, 130, { &P18645,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I747 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2974160 = {S48599, 137, { &P22584,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337M Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974483 = {S48600, 137, { &P21607,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; VS870 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2028817 = {S48601, 139, { &P19856,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LS970 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2428421 = {S48602, 61, { &P19856,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-LS970 Build/IMM76L)*/
const Device UA2709745 = {S48603, 144, { &P22582,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714022 = {S48604, 137, { &P22582,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2974282 = {S48605, 151, { &P12605,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022195 = {S48606, 144, { &P12605,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA90429 = {S48607, 145, { &P17863,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SCH-I400 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA173718 = {S48608, 67, { &P17863,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SCH-I400 Build/ECLAIR)*/
const Device UA545406 = {S48609, 144, { &P17863,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SCH-I400 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 MobileSafari/530.17*/
const Device UA2018640 = {S48610, 143, { &P17863,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-; SCH-I400 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018738 = {S48611, 144, { &P17863,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us;SCH-I400 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019191 = {S48612, 144, { &P17863,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SCH-I400Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019453 = {S48613, 144, { &P17863,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.1-update1; en-us; SCH-I400 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2872129 = {S48614, 155, { &P23505,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Vodafone 875 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30 SVN/040HEG1*/
const Device UA2667263 = {S48615, 101, { &P20741,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 520)*/
const Device UA2707834 = {S48616, 111, { &P20741,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 520; Vodafone)*/
const Device UA2751762 = {S48617, 109, { &P20741,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 520; ANZ914)*/
const Device UA2751784 = {S48618, 109, { &P20741,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 520; Orange)*/
const Device UA2956871 = {S48619, 109, { &P20741,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 520; Meteor)*/
const Device UA3088357 = {S48620, 101, { &P20741,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Nokia 520)*/
const Device UA3109769 = {S48621, 109, { &P20741,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 520; ANZ915)*/
const Device UA2540508 = {S48622, 134, { &P20760,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Event Build/IML77) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2201391 = {S48623, 140, { &P15989,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21i2 Build/11.0.A.1.8) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261516 = {S48624, 140, { &P15989,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21a2 Build/11.0.A.1.8) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261608 = {S48625, 140, { &P15989,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21a Build/11.0.A.0.16) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2386919 = {S48626, 140, { &P15989,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21i Build/11.0.A.4.22) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2010892 = {S48627, 147, { &P18385,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2198815 = {S48628, 151, { &P18385,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03R; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2238236 = {S48629, 147, { &P18385,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2241301 = {S48630, 158, { &P18385,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03L; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2241558 = {S48631, 165, { &P18385,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Nexus 7 Build/JRO03H; CyanogenMod-znsj.taobao.com jiji) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2456476 = {S48632, 154, { &P18385,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3033021 = {S48633, 80, { &P17595,&P17296,&P19806,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:17.0) Gecko/X Thunderbird/17.0.8*/
const Device UA2566626 = {S48634, 131, { &P12879,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583248 = {S48635, 131, { &P12879,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T859 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2637591 = {S48636, 131, { &P12879,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P7510 Build/HTJ85B) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA678119 = {S48637, 144, { &P19998,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HW-HUAWEI_C8812/C8812V100R001C92B913SP02; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA1962078 = {S48638, 134, { &P19998,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1993649 = {S48639, 148, { &P19998,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1995790 = {S48640, 139, { &P19998,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2026652 = {S48641, 70, { &P19998,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HUAWEI C8812 Build/HuaweiC8812)*/
const Device UA2034278 = {S48642, 139, { &P19998,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build HuaweiC8812) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2385254 = {S48643, 142, { &P19998,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8812E/C8812EV100R001C92B942; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2420549 = {S48644, 142, { &P19998,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8812E/C8812EV100R001C92B939; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2421811 = {S48645, 141, { &P19998,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8812E Build/HuaweiC8812E) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2423695 = {S48646, 150, { &P19998,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8812E Build/HuaweiC8812E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2466271 = {S48647, 142, { &P19998,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8812E/C8812EV100R001C92B937; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2503671 = {S48648, 146, { &P19998,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8812E/C8812EV100R001C92B943SP01; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2523786 = {S48649, 136, { &P19998,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8812E Build/HuaweiC8812E) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2529569 = {S48650, 142, { &P19998,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8812E/C8812EV100R001C92B949; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2556468 = {S48651, 146, { &P19998,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8812E/C8812EV100R001C92B942SP02; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2578026 = {S48652, 72, { &P19998,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI C8812E Build/HuaweiC8812E)*/
const Device UA2652737 = {S48653, 141, { &P19998,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8812E Build HuaweiC8812E) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2791012 = {S48654, 142, { &P19998,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8812E/C8812EV100R001C92B946; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2836898 = {S48655, 142, { &P19998,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8812 Build/IMM76) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950677 = {S48656, 142, { &P19998,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8812E/C8812EV100R001C92B938; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA3118568 = {S48657, 147, { &P19998,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8812E/C8812EV100R001C117B945SP01; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2586621 = {S48658, 140, { &P21604,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C2105 Build/15.0.A.1.31) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2586628 = {S48659, 140, { &P21604,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C2105 Build/15.0.A.0.20) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3033825 = {S48660, 96, { &P21640,&P17484,&P17369,&P18092 }};	/*Useragent - SAMSUNG Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.30421/30.3697; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2670741 = {S48661, 144, { &P18640,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673213 = {S48662, 137, { &P18640,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA8985 = {S48663, 125, { &P13369,&P5208,&P4223,&P18092 }};	/*Useragent - NetFront/3.5.1(BREW 3.1.5; U; en-us; SAMSUNG; NetFront/3.1.5/WAP) Sprint M350 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA9499 = {S48664, 148, { &P17951,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; Milestone Build/SHOLS_U2_05.06.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA56064 = {S48665, 138, { &P17951,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Milestone Build/FRG22D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67035 = {S48666, 142, { &P17951,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Milestone Build/R.U.R.1920) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67784 = {S48667, 138, { &P17951,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Milestone Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67866 = {S48668, 148, { &P17951,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; Milestone Build/SHOLS_U2_05.26.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67971 = {S48669, 148, { &P17951,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Milestone Build/SHOLS_U2_05.26.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA68085 = {S48670, 148, { &P17951,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; Milestone Build/SHOLS_U2_05.26.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA68162 = {S48671, 148, { &P17951,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pl-pl; Milestone Build/SHOLS_U2_05.26.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA435635 = {S48672, 148, { &P17951,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; Milestone Build/SHOLS_U2_05.27.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA439779 = {S48673, 148, { &P17951,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; Milestone Build/SHOLS_U2_05.27.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA657345 = {S48674, 148, { &P17951,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; Milestone Build/SHOLS_U2_05.27.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA869497 = {S48675, 149, { &P17951,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; fr-ca; Milestone Build/ SHOLS_U2_05.26.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1900118 = {S48676, 135, { &P17951,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Milestone Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018286 = {S48677, 136, { &P17951,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; Milestone Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018462 = {S48678, 62, { &P17951,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; Milestone Build/FRG83G)*/
const Device UA2018694 = {S48679, 148, { &P17951,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; Milestone Build/SHOLS_U2_05.20.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019150 = {S48680, 72, { &P17951,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; Milestone Build/SHOLS_U2_05.26.4)*/
const Device UA2021324 = {S48681, 130, { &P17951,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; Milestone Build/R.U.R.1920) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA32047 = {S48682, 96, { &P12455,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-SGH-T669/1.0 SHP/VPP/R5 Dolfin/1.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA33293 = {S48683, 103, { &P12455,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-SGH-T669Y/T669USKC5 SHP/VPP/R5 Dolfin/1.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA75602 = {S48684, 106, { &P12455,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-SGH-T669-MKB/T669UEKG4 SHP/VPP/R5 Dolfin/1.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA31980 = {S48685, 149, { &P12453,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HTC-A6366/1.0 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32032 = {S48686, 147, { &P12453,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HTC Liberty Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA50908 = {S48687, 148, { &P12453,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us;HTC-A6366/1.0 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA167156 = {S48688, 69, { &P12453,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; HTC Liberty Build/ERE27)*/
const Device UA2568848 = {S48689, 102, { &P15364,&P17188,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA20278 = {S48690, 131, { &P17686,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-gb; MZ601 Build/H.6.1-25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA20283 = {S48691, 133, { &P17686,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-gb; MZ601 Build/H.6.1-38-1) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA20280 = {S48692, 131, { &P17686,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-gb; MZ601 Build/H.6.1-17) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA42754 = {S48693, 134, { &P17686,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; sv-se; MZ601 Build/H.6.1-38-13) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA42768 = {S48694, 131, { &P17686,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-au; MZ601 Build/H.6.1-23) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA42778 = {S48695, 133, { &P17686,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; de-de; MZ601 Build/H.6.1-38-2) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA58692 = {S48696, 131, { &P17686,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-sg; MZ601 Build/H.6.1-40) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA58693 = {S48697, 133, { &P17686,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; de-de; MZ601 Build/H.6.1-38-5) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA58694 = {S48698, 133, { &P17686,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-au; MZ601 Build/H.6.1-38-9) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA58706 = {S48699, 131, { &P17686,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-gb; MZ601 Build/H.6.1-38) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA58710 = {S48700, 134, { &P17686,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; tr-tr; MZ601 Build/H.6.1-38-11) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA58717 = {S48701, 129, { &P17686,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0; de-de; MZ601 Build/H.6.1-11) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA58953 = {S48702, 133, { &P17686,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; MZ601 Build/H.6.1-37.1) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA864974 = {S48703, 62, { &P17686,&P6545,&P4755,&P18092 }};	/*Useragent - Dalvik/1.5.0 (Linux; U; Android 3.0.1; MZ601 Build/H.6.1-38-9)*/
const Device UA2566495 = {S48704, 140, { &P18564,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X+ Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567028 = {S48705, 142, { &P18564,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC EVARE_UL Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567516 = {S48706, 133, { &P18564,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X+ Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA3031101 = {S48707, 124, { &P15364,&P18110,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA197716 = {S48708, 135, { &P15364,&P17816,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.24 (KHTML, like Gecko) Ubuntu/10.04 Chromium/11.0.696.68 Chrome/11.0.696.68 Safari/534.24*/
const Device UA390232 = {S48709, 135, { &P15364,&P17816,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Ubuntu/10.04 Chromium/11.0.696.0 Chrome/11.0.696.0 Safari/534.24*/
const Device UA2131054 = {S48710, 143, { &P15364,&P17816,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.21 (KHTML, like Gecko) Ubuntu/10.04 Chromium/11.0.674.0 Chrome/11.0.674.0 Safari/534.21*/
const Device UA2275765 = {S48711, 145, { &P15364,&P17816,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.21 (KHTML, like Gecko) Ubuntu/10.04 Chromium/11.0.680.0 Chrome/11.0.680.0 Safari/534.21*/
const Device UA2671307 = {S48712, 138, { &P12972,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989D Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676237 = {S48713, 137, { &P12972,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676688 = {S48714, 138, { &P12972,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676741 = {S48715, 137, { &P12972,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734959 = {S48716, 145, { &P12972,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-T989 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2860874 = {S48717, 130, { &P12972,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708400 = {S48718, 151, { &P18552,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA50370 = {S48719, 128, { &P12481,&P17603,&P4732,&P18092 }};	/*Useragent - PantechP2020/JIUS05172010R; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.19039; U; en-US) Opera 9.50*/
const Device UA67333 = {S48720, 125, { &P12481,&P17603,&P4732,&P18092 }};	/*Useragent - PantechP2020/JIUS05172010R; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.19039; U; es) Opera 9.50*/
const Device UA627166 = {S48721, 120, { &P12481,&P17603,&P4732,&P18092 }};	/*Useragent - PantechP2020/JIUSX; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.19039; U; en-US) Opera 9.50*/
const Device UA2219901 = {S48722, 117, { &P12481,&P17603,&P4732,&P18092 }};	/*Useragent - PantechP2020/JIUSX; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.19039; U; es) Opera 9.50*/
const Device UA2347868 = {S48723, 153, { &P20017,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; ALCATEL ONE TOUCH 997D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2506509 = {S48724, 152, { &P20017,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sr-rs; ALCATEL ONE TOUCH 997 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2554618 = {S48725, 75, { &P20017,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ALCATEL ONE TOUCH 997D Build/IMM76D)*/
const Device UA2740105 = {S48726, 138, { &P18559,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LG730 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2754648 = {S48727, 130, { &P15876,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T779 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754941 = {S48728, 130, { &P15876,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P5100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755540 = {S48729, 130, { &P15876,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P5100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2511671 = {S48730, 75, { &P17595,&P17296,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8) Presto/2.12.388 Version/12.14*/
const Device UA33361 = {S48731, 161, { &P14984,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Nokia5250/30.0.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413 3gpp-gba*/
const Device UA2754946 = {S48732, 136, { &P22594,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2204895 = {S48733, 70, { &P15767,&P18432,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10A403 (iPad; U; CPU OS 6_0 like Mac OS X; en_us)*/
const Device UA2242197 = {S48734, 72, { &P15767,&P18432,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10A523 (iPad; U; CPU OS 6_0_1 like Mac OS X; en_us)*/
const Device UA2426931 = {S48735, 72, { &P15767,&P18432,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10A550 (iPad; U; CPU OS 6_0_2 like Mac OS X; en_us)*/
const Device UA3065083 = {S48736, 130, { &P12879,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080932 = {S48737, 130, { &P12879,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T859 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA609346 = {S48738, 170, { &P13157,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA609840 = {S48739, 164, { &P13157,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA901221 = {S48740, 169, { &P13157,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA903734 = {S48741, 163, { &P13157,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA908855 = {S48742, 169, { &P13157,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715a Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2023764 = {S48743, 165, { &P13157,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2462075 = {S48744, 169, { &P13157,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1994536 = {S48745, 78, { &P15364,&P5690,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Avant Browser)*/
const Device UA2162567 = {S48746, 78, { &P15364,&P5690,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; Avant Browser)*/
const Device UA2164658 = {S48747, 78, { &P15364,&P5690,&P22732,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Avant Browser)*/
const Device UA2166802 = {S48748, 78, { &P15364,&P5690,&P22732,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; Avant Browser)*/
const Device UA2775608 = {S48749, 93, { &P15364,&P5690,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Avant Browser; Avant Browser)*/
const Device UA2775632 = {S48750, 93, { &P15364,&P5690,&P22732,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; Avant Browser; Avant Browser)*/
const Device UA2787146 = {S48751, 83, { &P15364,&P5690,&P22732,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Avant Browser; SV1)*/
const Device UA2788548 = {S48752, 83, { &P15364,&P5690,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; SV1; Avant Browser)*/
const Device UA2950153 = {S48753, 83, { &P15364,&P5690,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Avant Browser; SV1)*/
const Device UA2950252 = {S48754, 93, { &P15364,&P5690,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; Avant Browser; Avant Browser)*/
const Device UA2950258 = {S48755, 93, { &P15364,&P5690,&P22732,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.1; Avant Browser; Avant Browser)*/
const Device UA2950486 = {S48756, 83, { &P15364,&P5690,&P22732,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.2; SV1; Avant Browser)*/
const Device UA899014 = {S48757, 137, { &P21027,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-AS695 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2543426 = {S48758, 61, { &P21027,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-AS695 Build/GRK39F)*/
const Device UA2944783 = {S48759, 97, { &P17595,&P18207,&P22171,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:23.0) Gecko/X Firefox/23.0 AlexaToolbar/alxf-2.18*/
const Device UA54931 = {S48760, 110, { &P14980,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-GT-B7722/DDJH6 SHP/VPP/R5 Dolfin/1.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA2566686 = {S48761, 137, { &P18713,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P760 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566753 = {S48762, 137, { &P18713,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P769 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567263 = {S48763, 137, { &P18713,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P768 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667522 = {S48764, 130, { &P18713,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P769 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2698241 = {S48765, 139, { &P18679,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-B5330 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2863567 = {S48766, 61, { &P18679,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-B5330 Build/JZO54K)*/
const Device UA2865333 = {S48767, 140, { &P18679,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-B5330L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3073662 = {S48768, 137, { &P18679,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-B5330 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108506 = {S48769, 140, { &P18679,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; GT-B5330B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA5978 = {S48770, 153, { &P16745,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.1; en-us; T-Mobile myTouch 3G Slide Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA5985 = {S48771, 154, { &P16745,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3G Slide Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA50572 = {S48772, 153, { &P16745,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3G SlideBuild/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA56053 = {S48773, 151, { &P16745,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; T-Mobile myTouch 3G Slide Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA56327 = {S48774, 153, { &P16745,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3G Slide Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67065 = {S48775, 151, { &P16745,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; T-Mobile myTouch 3G Slide Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA377064 = {S48776, 78, { &P16745,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; T-Mobile myTouch 3G Slide Build/FRG83D)*/
const Device UA2019055 = {S48777, 153, { &P16745,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3G Slide Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA2019067 = {S48778, 153, { &P16745,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch3G Slide Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019129 = {S48779, 153, { &P16745,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; T-Mobile myTouch 3G Slide Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019227 = {S48780, 153, { &P16745,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile myTouch 3GSlide Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019364 = {S48781, 153, { &P16745,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;T-Mobile myTouch 3G Slide Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019544 = {S48782, 153, { &P16745,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-MobilemyTouch 3G Slide Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2886518 = {S48783, 65, { &P15364,&P7133,&P23479,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:25.0) Gecko/X Firefox/25.0*/
const Device UA3092796 = {S48784, 82, { &P15364,&P7133,&P23479,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:25.0) Gecko/X Firefox/25.0 SeaMonkey/2.22a2*/
const Device UA3111800 = {S48785, 79, { &P15364,&P7133,&P23479,&P18092 }};	/*Useragent - <![CDATA[Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:25.0) Gecko/X Firefox/25.0*/
const Device UA901326 = {S48786, 62, { &P15364,&P7127,&P4551,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0) Undrip Bot*/
const Device UA2974073 = {S48787, 138, { &P18645,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974797 = {S48788, 145, { &P18645,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975800 = {S48789, 138, { &P18645,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976114 = {S48790, 138, { &P18645,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I747M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021058 = {S48791, 137, { &P18645,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-L710 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021149 = {S48792, 138, { &P18645,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021645 = {S48793, 138, { &P18645,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530U Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023773 = {S48794, 138, { &P18645,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2781960 = {S48795, 146, { &P18451,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT26ii Build/6.2.B.0.200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2869482 = {S48796, 142, { &P18451,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT26ii Build/6.2.B.0.200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2869598 = {S48797, 146, { &P18451,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT26ii Build/6.2.B.0.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2869888 = {S48798, 64, { &P18451,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LT26ii Build/6.2.B.0.200)*/
const Device UA2893385 = {S48799, 142, { &P18451,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT26ii Build/6.2.B.0.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3066829 = {S48800, 145, { &P18451,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT26ii Build/6.2.B.1.96) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079473 = {S48801, 141, { &P18451,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT26ii Build/6.2.B.1.96) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA627139 = {S48802, 108, { &P15364,&P5690,&P18333,&P18092 }};	/*Useragent - iTunes/10.5.1 (Windows; Microsoft Windows XP Professional Service Pack 2 (Build 2600)) AppleWebKit/534.51.22*/
const Device UA1994416 = {S48803, 116, { &P15364,&P5690,&P18333,&P18092 }};	/*Useragent - iTunes/10.6.1 (Windows; Microsoft Windows XP Home Edition Dodatek Service Pack 3 (Build 2600)) AppleWebKit/534.54.16*/
const Device UA2041298 = {S48804, 105, { &P15364,&P5690,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows XP Professional Service Pack 3 (Build 2600)) AppleWebKit/536.26.9*/
const Device UA2042964 = {S48805, 105, { &P15364,&P5690,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; Microsoft Windows XP Home Edition Service Pack 3 (Build 2600)) AppleWebKit/536.26.9*/
const Device UA2318988 = {S48806, 94, { &P15364,&P5690,&P18333,&P18092 }};	/*Useragent - iTunes/10.6.3 (Windows; Microsoft Windows XP Service Pack 2 (Build 2600)) AppleWebKit/534.57.2*/
const Device UA2322656 = {S48807, 108, { &P15364,&P5690,&P18333,&P18092 }};	/*Useragent - iTunes/10.6.1 (Windows; Microsoft Windows XP Home Edition Service Pack 2 (Build 2600)) AppleWebKit/534.54.16*/
const Device UA640109 = {S48808, 57, { &P15364,&P18110,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:9.0) Gecko/X Firefox/9.0*/
const Device UA683011 = {S48809, 77, { &P15364,&P18110,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:9.0.1) Gecko/X Firefox/9.0.1 Firefox/9.0.1*/
const Device UA706497 = {S48810, 71, { &P15364,&P18110,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:9.0) Gecko/X Firefox/9.0 Iceweasel/9.0*/
const Device UA2312444 = {S48811, 51, { &P15364,&P18110,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; rv:9.0.1) Gecko/X Firefox/9.0.1*/
const Device UA2257597 = {S48812, 136, { &P20730,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; P771A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2552966 = {S48813, 58, { &P20730,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; P771A Build/IMM76D)*/
const Device UA457528 = {S48814, 139, { &P17905,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SO-03D Build/6.0.A.5.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627144 = {S48815, 151, { &P17905,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonSO-03D Build/6.0.A.5.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA628251 = {S48816, 150, { &P17905,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonSO-03D Build/6.0.A.5.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA665931 = {S48817, 138, { &P17905,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SO-03D Build/6.0.A.5.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA862995 = {S48818, 151, { &P17905,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonSO-03D Build/6.0.A.5.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA869495 = {S48819, 139, { &P17905,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SO-03D Build/6.0.A.5.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2374541 = {S48820, 151, { &P17905,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonSO-03D Build/6.0.A.5.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2405771 = {S48821, 139, { &P17905,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SO-03D Build/6.0.A.5.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2655108 = {S48822, 63, { &P17905,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; SO-03D Build/6.0.A.5.14)*/
const Device UA2566696 = {S48823, 141, { &P18582,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT28at Build/6.1.C.1.105) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567093 = {S48824, 138, { &P18582,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT28h Build/6.1.E.3.7) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567523 = {S48825, 139, { &P18582,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT28i Build/6.1.E.2.68) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2888595 = {S48826, 47, { &P15364,&P18580,&P22245,&P18092 }};	/*Useragent - Flipboard/2.0.4 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2490775 = {S48827, 114, { &P15837,&P6909,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-05/08.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA249382 = {S48828, 135, { &P17954,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID1125 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458799 = {S48829, 160, { &P17954,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; MID1125 Build/GRI40) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA458883 = {S48830, 59, { &P17954,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID1125 Build/GRI40)*/
const Device UA459042 = {S48831, 69, { &P17954,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.3; en_US; MID1125 Build/MID1125 X)*/
const Device UA2030643 = {S48832, 136, { &P17954,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; MID1125A Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA895669 = {S48833, 135, { &P18717,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PMP5097CPRO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2445300 = {S48834, 64, { &P18717,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; PMP5097CPRO Build/IML74K)*/
const Device UA899641 = {S48835, 122, { &P15364,&P7133,&P22430,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5 Comodo_Dragon/19.1.0.0*/
const Device UA899655 = {S48836, 129, { &P15364,&P7133,&P22430,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5 Comodo_Dragon/19.1.0.0*/
const Device UA412584 = {S48837, 135, { &P18278,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-05D Build/V16R24C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA613202 = {S48838, 135, { &P18278,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-05D Build/V18R26A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625890 = {S48839, 133, { &P18278,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-05D Build/F0001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA632075 = {S48840, 135, { &P18278,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-05D Build/V17R25B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA910447 = {S48841, 135, { &P18278,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-05D Build/V19R28A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2236726 = {S48842, 135, { &P18278,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-05D Build/V20R29A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2425715 = {S48843, 59, { &P18278,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; F-05D Build/V20R29A)*/
const Device UA2567739 = {S48844, 135, { &P18278,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-05D Build/V21R30C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2883805 = {S48845, 135, { &P18278,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-05D Build/V22R41A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA216840 = {S48846, 139, { &P15364,&P17816,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.30 (KHTML, like Gecko) Ubuntu/10.04 Chromium/12.0.742.112 Chrome/12.0.742.112 Safari/534.30*/
const Device UA218257 = {S48847, 137, { &P15364,&P17816,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.30 (KHTML, like Gecko) Ubuntu/10.04 Chromium/12.0.742.112 Chrome/12.0.742.112 Safari/534.30*/
const Device UA642410 = {S48848, 148, { &P16263,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  HUAWEI MediaPad Build/HuaweiMediaPad) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA902542 = {S48849, 147, { &P16263,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI MediaPad Build/HuaweiMediaPad) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2196267 = {S48850, 143, { &P16263,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SpringBoard Build/HuaweiMediaPad) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2362652 = {S48851, 147, { &P16263,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MediaPad 7 Lite Build/HuaweiMediaPad) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2702236 = {S48852, 148, { &P16263,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MediaPad 10 LINK Build/HuaweiMediaPad) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2791509 = {S48853, 136, { &P16263,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT01 Build/HuaweiMediaPad) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2677231 = {S48854, 137, { &P21607,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; VS870 4G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713167 = {S48855, 130, { &P21607,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; VS870 4G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2566523 = {S48856, 149, { &P15201,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710e Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566909 = {S48857, 146, { &P15201,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567563 = {S48858, 149, { &P15201,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Z710e Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667707 = {S48859, 163, { &P15201,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G with Beats Audio Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2862523 = {S48860, 146, { &P15201,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation 4G Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2670838 = {S48861, 131, { &P19749,&P7106,&P15829,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9315; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.912 Mobile Safari/534.11+*/
const Device UA910099 = {S48862, 151, { &P17346,&P18117,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9B176 Safari/7534.48.3*/
const Device UA910459 = {S48863, 153, { &P17346,&P18117,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9B206 Safari/7534.48.3*/
const Device UA2600768 = {S48864, 150, { &P17346,&P18117,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 511 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/19.0.1084.60 Mobile/9B206 Safari/7534.48.3*/
const Device UA2658398 = {S48865, 149, { &P17346,&P18117,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 51 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/19.0.1084.60 Mobile/9B176 Safari/7534.48.3*/
const Device UA227999 = {S48866, 126, { &P12765,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; A100 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA575625 = {S48867, 128, { &P12765,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; A100 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA582220 = {S48868, 57, { &P12765,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; A100 Build/HTK55D)*/
const Device UA641797 = {S48869, 55, { &P12765,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; A100 Build/HTJ85B)*/
const Device UA2566525 = {S48870, 131, { &P15919,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3100 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566916 = {S48871, 131, { &P15919,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P3100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2634143 = {S48872, 132, { &P15919,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3100B Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2636560 = {S48873, 132, { &P15919,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P3100B Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA526030 = {S48874, 87, { &P15364,&P17017,&P4780,&P18092 }};	/*Useragent - Nexian NX-T905/MTK Release/10.01.2009 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1*/
const Device UA616233 = {S48875, 89, { &P15364,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-G915/MTK Release/27.11.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA616236 = {S48876, 88, { &P15364,&P17017,&P4780,&P18092 }};	/*Useragent - Nexian-NXG777/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA616232 = {S48877, 88, { &P15364,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-D981/MTK Release/23.3.2010 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA616270 = {S48878, 85, { &P15364,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-T901/MTK Release/19.04.2011 Browser/MAUI Profile/MIDP2.0Configuration/CLDC-1*/
const Device UA616271 = {S48879, 85, { &P15364,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-A725/MTK Release/12.07.2011 Browser/MAUI Profile/MIDP2.0Configuration/CLDC-1*/
const Device UA616294 = {S48880, 87, { &P15364,&P17017,&P4780,&P18092 }};	/*Useragent - Nexian NX-T908/MTK Release/10.01.2009 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1*/
const Device UA616307 = {S48881, 86, { &P15364,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-G269/MTK Release/01.01.2012 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1*/
const Device UA616328 = {S48882, 85, { &P15364,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-G855/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1*/
const Device UA2535636 = {S48883, 83, { &P15364,&P17017,&P4780,&P18092 }};	/*Useragent - Jigsaw/2.3.0 W3C_CSS_Validator_JFouffa/2.0 (See <http://validator.w3.org/services>)*/
const Device UA2508299 = {S48884, 162, { &P15438,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; HTC Sensation Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2509577 = {S48885, 162, { &P15438,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; HTC Sensation Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2536616 = {S48886, 157, { &P15438,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Sensation Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567192 = {S48887, 161, { &P15438,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC Sensation Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708919 = {S48888, 162, { &P15438,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC Sensation Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2973962 = {S48889, 138, { &P23771,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974033 = {S48890, 137, { &P19856,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LS970 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2043316 = {S48891, 123, { &P17346,&P18432,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Twitter for iPhone*/
const Device UA2233259 = {S48892, 123, { &P17346,&P18432,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A406 Twitter for iPhone*/
const Device UA2261385 = {S48893, 125, { &P17346,&P18432,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Twitter for iPhone*/
const Device UA2748103 = {S48894, 132, { &P22873,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N5120 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2937438 = {S48895, 61, { &P22873,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N5120 Build/JZO54K)*/
const Device UA632680 = {S48896, 140, { &P13889,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P505/V20f Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA690910 = {S48897, 135, { &P13889,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P505 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA691137 = {S48898, 136, { &P13889,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; LG-P505R Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027591 = {S48899, 59, { &P13889,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P505 Build/GRJ22)*/
const Device UA2033930 = {S48900, 137, { &P13889,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; LG-P505CH Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2933413 = {S48901, 135, { &P13889,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; LG-P505 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Versi?n/4.0 Mobile Safari/533.1*/
const Device UA2423174 = {S48902, 143, { &P20091,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMP5880D Build/PMP5880D_v1.0.1_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2425845 = {S48903, 143, { &P20091,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMP5880D Build/PMP5880D_v1.0.2_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2435430 = {S48904, 143, { &P20091,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMP5880D Build/PMP5880D_v1.0.3_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2698451 = {S48905, 72, { &P20091,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; PMP5880D Build/PMP5880D_v1.0.3_X)*/
const Device UA179778 = {S48906, 142, { &P14029,&P5267,&P3834,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5330/1.0; U; Bada/1.0; es) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA2844838 = {S48907, 139, { &P23642,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-M840 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2961622 = {S48908, 61, { &P23642,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SPH-M840 Build/JZO54K)*/
const Device UA628898 = {S48909, 60, { &P15803,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia200/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2691197 = {S48910, 69, { &P15803,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia200/2.0 (11.64) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia200*/
const Device UA2170778 = {S48911, 131, { &P18708,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; U30GT-H Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2416954 = {S48912, 60, { &P18708,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; U30GT-H Build/JRO03H)*/
const Device UA2761963 = {S48913, 144, { &P18708,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; U30GT-H Wanden.co.uk Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2763686 = {S48914, 148, { &P18708,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; U30GT-H_www.wanden.co.uk Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3048198 = {S48915, 132, { &P18708,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; U30GT-HS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA32 = {S48916, 91, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_HD2_T8585-Orange; Windows Phone 6.5)*/
const Device UA4998 = {S48917, 88, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_HD2_T8585; Windows Phone 6.5.3.5)*/
const Device UA5850 = {S48918, 110, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD2/3.14.172.3 (04666); Windows Phone 6.5)*/
const Device UA5855 = {S48919, 110, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD2/1.44.162.6 (70494); Windows Phone 6.5)*/
const Device UA8581 = {S48920, 96, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_HD2; T-Mobile_LEO; Windows Phone 6.5.3.5)*/
const Device UA10050 = {S48921, 80, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HD2_T8585; Windows Phone 6.5)*/
const Device UA10051 = {S48922, 84, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_HD2_T8585; Windows Phone 6.5)*/
const Device UA10054 = {S48923, 83, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; T-Mobile_LEO; Windows Phone 6.5)*/
const Device UA10055 = {S48924, 81, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5 HTC_HD2/1.0)*/
const Device UA30459 = {S48925, 110, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD2/1.72.172.0 (82124); Windows Phone 6.5)*/
const Device UA30464 = {S48926, 110, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD2/1.72.162.0 (82124); Windows Phone 6.5)*/
const Device UA30467 = {S48927, 110, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD2/1.72.161.3 (82124); Windows Phone 6.5)*/
const Device UA30469 = {S48928, 110, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD2/3.14.162.5 (04666); Windows Phone 6.5)*/
const Device UA30471 = {S48929, 110, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD2/3.14.164.4 (04666); Windows Phone 6.5)*/
const Device UA30472 = {S48930, 110, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD2/1.44.163.3 (70494); Windows Phone 6.5)*/
const Device UA30454 = {S48931, 110, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD2/1.49.164.2 (71528); Windows Phone 6.5)*/
const Device UA31302 = {S48932, 104, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - HTC_HD2_T9193 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; V1.72.841.0 (82124); Windows Phone 6.5)*/
const Device UA38014 = {S48933, 104, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5 HTC_HD2/1.0; Windows Phone 6.5.3.5)*/
const Device UA46760 = {S48934, 110, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD2/3.14.161.3 (04666); Windows Phone 6.5)*/
const Device UA438869 = {S48935, 110, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD2/3.14.163.3 (04666); Windows Phone 6.5)*/
const Device UA481385 = {S48936, 87, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; T-Mobile_LEO; Windows Phone 6.5.3.5)*/
const Device UA887029 = {S48937, 104, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - HTC_HD2_T9193 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; V1.49.841.1 (71528); Windows Phone 6.5)*/
const Device UA925697 = {S48938, 78, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_HD2; Windows Phone 6.5)*/
const Device UA925890 = {S48939, 88, { &P13999,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_HD2_T8585_SOD; Windows Phone 6.5)*/
const Device UA402914 = {S48940, 137, { &P20285,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 Build/1.6.0_243-MZ616) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA403029 = {S48941, 139, { &P20285,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 Build/1.6.0_268.1-MZ616) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA403037 = {S48942, 139, { &P20285,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 Build/1.6.0_258.1-MZ616) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA403074 = {S48943, 139, { &P20285,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 Build/1.6.0_268.3-MZ616) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA403097 = {S48944, 137, { &P20285,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 Build/1.6.0_236-MZ616) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA403145 = {S48945, 139, { &P20285,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-au; XOOM 2 Build/1.6.0_268.4-MZ616) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA403309 = {S48946, 139, { &P20285,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-au; XOOM 2 Build/1.6.0_281.4-MZ616) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA711957 = {S48947, 139, { &P20285,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; iw-il; XOOM 2 Build/1.6.0_281.8-MZ616) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA767039 = {S48948, 139, { &P20285,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; de-de; XOOM 2 Build/1.6.0_268.6-MZ616) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA787346 = {S48949, 140, { &P20285,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; pt-br; MZ616 Build/1.6.0M_272.18_MZ616) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA1964591 = {S48950, 141, { &P20285,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-au; XOOM 2 Build/1.6.0_268.4-MZ616) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2026102 = {S48951, 142, { &P20285,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; pt-br; MZ616 Build/1.6.0M_272.18_MZ616) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2026633 = {S48952, 141, { &P20285,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; de-de; XOOM 2 Build/1.6.0_281.4-MZ616) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2034005 = {S48953, 141, { &P20285,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; de-de; XOOM 2 Build/1.6.0_268.6-MZ616) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2519772 = {S48954, 70, { &P20285,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.2; XOOM 2 Build/1.6.0_268.4-MZ616)*/
const Device UA2034685 = {S48955, 158, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/BLUE1.0-DECOUPLED-X-1500) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034707 = {S48956, 165, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/TESTFEATURE-SUPERSTAMINA-X-0856) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034726 = {S48957, 145, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/7.0.A.0.735) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034787 = {S48958, 156, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SonyLT29i Build/ICS-BLUE-QCT-MW-X-1944) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034806 = {S48959, 158, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyLT29i Build/BLUE1.0-DECOUPLED-X-1530) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034869 = {S48960, 145, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/9.0.A.0.226) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034893 = {S48961, 165, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyLT29i Build/TESTFEATURE-SUPERSTAMINA-X-0839) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034894 = {S48962, 145, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/7.0.A.1.155) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034902 = {S48963, 158, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/BLUE1.0-DECOUPLED-X-1750) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034916 = {S48964, 145, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/7.0.A.1.107) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034929 = {S48965, 145, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/7.0.A.1.291) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034694 = {S48966, 145, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/7.0.A.1.300) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034946 = {S48967, 156, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; SonyLT29i Build/ICS-BLUE-QCT-MW-X-0654) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034948 = {S48968, 158, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyLT29i Build/BLUE1.0-DECOUPLED-X-1600) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034955 = {S48969, 149, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/VRC-X-1342-1050) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034974 = {S48970, 158, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/BLUE1.0-DECOUPLED-X-1700) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2035022 = {S48971, 145, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/7.0.D.1.119) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2035047 = {S48972, 145, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/9.0.A.0.189) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2035081 = {S48973, 143, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyLT29i Build/7.0.A.2.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2035142 = {S48974, 145, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyLT29i Build/7.0.A.1.303) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2230704 = {S48975, 145, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; SonyLT29i Build/7.0.A.1.307) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2386224 = {S48976, 145, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; SonyLT29i Build/7.0.A.3.197) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422817 = {S48977, 145, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; SonyLT29i Build/7.0.A.3.212) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426435 = {S48978, 63, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT29i Build/7.0.A.3.197)*/
const Device UA2537453 = {S48979, 141, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT29i Build/7.0.A.3.212) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556485 = {S48980, 132, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT29i Build/7.0.A.3.197) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2577726 = {S48981, 141, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; LT29i Build/7.0.A.1.307) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2578173 = {S48982, 141, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT29i Build/7.0.A.3.197) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695888 = {S48983, 136, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT29i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3042828 = {S48984, 145, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT29i Build/7.0.A.3.193) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3117964 = {S48985, 141, { &P18452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT29i Build/7.0.A.1.303) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA175720 = {S48986, 100, { &P16963,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; DELL; Venue Pro)*/
const Device UA3065102 = {S48987, 137, { &P19856,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LS970 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2793281 = {S48988, 140, { &P18564,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC One X+ Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2990260 = {S48989, 62, { &P18564,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC One X+ Build/JDQ39)*/
const Device UA3059378 = {S48990, 142, { &P18564,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC_OneXplus Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3062644 = {S48991, 153, { &P18564,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC_OneXplus/2.17.161.2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA624651 = {S48992, 143, { &P18474,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-I9210T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA887751 = {S48993, 67, { &P18474,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9210T Build/GINGERBREAD)*/
const Device UA8947 = {S48994, 153, { &P17944,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; USCCADR6275US Carrier ID 45 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA224601 = {S48995, 151, { &P18028,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Milestone PLUS Build/3.4.2_80_V2_25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA243755 = {S48996, 151, { &P18028,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Milestone PLUS Build/3.4.2_80_V2_26) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021300 = {S48997, 75, { &P18028,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; Milestone PLUS Build/3.4.2_80_V2_25)*/
const Device UA2043778 = {S48998, 151, { &P18028,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Milestone PLUS Build/3.4.2_80_V2_11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2235737 = {S48999, 138, { &P18301,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS840 4G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2741403 = {S49000, 137, { &P18301,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS840PP Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA903236 = {S49001, 70, { &P15364,&P18110,&P18565,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) KHTML/4.8.4 (like Gecko) Konqueror/4.8*/
const Device UA2189171 = {S49002, 68, { &P15364,&P18110,&P18565,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) KHTML/4.8.5 (like Gecko) Konqueror/4.8*/
const Device UA2194197 = {S49003, 100, { &P15364,&P18110,&P18565,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) konqueror/4.8.5 Safari/534.34*/
const Device UA2260165 = {S49004, 98, { &P15364,&P18110,&P18565,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/533.3 (KHTML, like Gecko) konqueror/4.8.5 Safari/533.3*/
const Device UA2349719 = {S49005, 56, { &P15364,&P18110,&P18565,&P18092 }};	/*Useragent - Mozilla/5.0 (X11) KHTML/4.8.5 (like Gecko) Konqueror/4.8*/
const Device UA2203598 = {S49006, 53, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - IUC(U;iOS 5.1.1;Zh-cn;320*480;)/UCWEB8.7.1.201/42/800*/
const Device UA2204019 = {S49007, 53, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - IUC(U;iOS 5.0.1;Zh-cn;320*480;)/UCWEB8.7.1.201/42/800*/
const Device UA2235609 = {S49008, 51, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - IUC(U;iOS 4.1;Zh-cn;320*480;)/UCWEB8.7.1.201/41/800*/
const Device UA2350044 = {S49009, 53, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB8.7.1.201/41/800*/
const Device UA2387153 = {S49010, 87, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; opera) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2387243 = {S49011, 87, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; mstar) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2389110 = {S49012, 86, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; maui) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2390859 = {S49013, 86, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; j2me) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2404098 = {S49014, 87, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; spice) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2404638 = {S49015, 89, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; generic) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2405631 = {S49016, 87, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; mstar) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2406789 = {S49017, 87, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; opera) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2415222 = {S49018, 53, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - IUC(U;iOS 4.2.1;Zh-cn;320*480;)/UCWEB8.7.1.201/41/800*/
const Device UA2424538 = {S49019, 89, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; mozilla) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2427230 = {S49020, 96, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; mstar platform) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2427651 = {S49021, 90, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sunmicro) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2428322 = {S49022, 90, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sunmicro) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2429422 = {S49023, 89, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; generic) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2430316 = {S49024, 89, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; karbonn) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2431963 = {S49025, 83, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; id; maui) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2460204 = {S49026, 86, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; bird) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2467480 = {S49027, 94, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; jbed-fastbcc) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2481687 = {S49028, 83, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; id; maui) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2527618 = {S49029, 88, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; huawei) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2555396 = {S49030, 99, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; 11a wbg3505a_0302) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2578121 = {S49031, 94, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; maui browser) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2778083 = {S49032, 86, { &P18088,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; jbed) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2201963 = {S49033, 71, { &P17595,&P19835,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (iMac12%2C2)*/
const Device UA2205274 = {S49034, 76, { &P17595,&P19835,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookAir4%2C2)*/
const Device UA2227442 = {S49035, 72, { &P17595,&P19835,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacPro3%2C1)*/
const Device UA2350491 = {S49036, 73, { &P17595,&P19835,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBook5%2C1)*/
const Device UA2367778 = {S49037, 77, { &P17595,&P19835,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.2.3 Darwin/12.2.1 (x86_64) (MacBookPro10%2C1)*/
const Device UA2454538 = {S49038, 73, { &P17595,&P19835,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (Macmini5%2C1)*/
const Device UA49251 = {S49039, 148, { &P12971,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MB632 Build/5.5.1Q-83_ELW-FIT2-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49253 = {S49040, 147, { &P12971,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB632 Build/5.5.1Q-110_ELW-TA-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73543 = {S49041, 150, { &P12971,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; MB632 Build/5.5.1Q-110_ELWLA-TA-23) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623859 = {S49042, 148, { &P12971,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MB632 Build/5.5.1Q-110_ELW-TA-36) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA654409 = {S49043, 150, { &P12971,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; MB632 Build/5.5.1Q-110_ELWLA-TA-28) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA672177 = {S49044, 150, { &P12971,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; MB632 Build/5.5.1Q-110_ELWLA-TA-39) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA694695 = {S49045, 148, { &P12971,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; MB632 Build/5.5.1Q-110_ELW-TA-21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA787529 = {S49046, 153, { &P12971,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB632 Build/5.5.1Q-110-L28_ELWUS-TA-1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1951173 = {S49047, 148, { &P12971,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; MB632 Build/5.5.1Q-110_ELW-TA-29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1957746 = {S49048, 148, { &P12971,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MB632 Build/5.5.1Q-110_ELW-TA-38) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1962395 = {S49049, 148, { &P12971,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MB632 Build/5.5.1Q-110_ELW-TA-41) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1965827 = {S49050, 150, { &P12971,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; MB632 Build/5.5.1Q-110_ELWLA-TA-38) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1973565 = {S49051, 150, { &P12971,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; MB632 Build/5.5.1Q-110_ELWLA-TA-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024865 = {S49052, 151, { &P12971,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; MB632 Build/5.5.1Q-110_ELWMEA-TA-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2338470 = {S49053, 150, { &P12971,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; MB632 Build/5.5.1Q-110_ELWLA-TA-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2643530 = {S49054, 150, { &P12971,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; MB632 Build/5.5.1Q-110_ELWLA-TA-29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA660556 = {S49055, 140, { &P18504,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-LS840 Build/ZVC.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA910778 = {S49056, 140, { &P18504,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-LS840 Build/ZVD.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1985960 = {S49057, 64, { &P18504,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; LG-LS840 Build/ZVD.GWK74)*/
const Device UA3130723 = {S49058, 137, { &P18645,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3130745 = {S49059, 138, { &P18645,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131081 = {S49060, 138, { &P18645,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131300 = {S49061, 138, { &P18645,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747M Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131353 = {S49062, 145, { &P18645,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131469 = {S49063, 137, { &P18645,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131663 = {S49064, 137, { &P18645,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131682 = {S49065, 138, { &P18645,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530C Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131718 = {S49066, 138, { &P18645,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132837 = {S49067, 145, { &P18645,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA2437516 = {S49068, 150, { &P15882,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P5110 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2558566 = {S49069, 150, { &P15882,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P5110 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2565479 = {S49070, 149, { &P15882,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P5110 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2633496 = {S49071, 157, { &P15882,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P5110 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2635238 = {S49072, 147, { &P15882,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-; GT-P5110 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2970951 = {S49073, 157, { &P15882,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P5110 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1.1/p5110*/
const Device UA2974043 = {S49074, 139, { &P18223,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3065088 = {S49075, 141, { &P18462,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080832 = {S49076, 151, { &P18462,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire XC dual sim Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081382 = {S49077, 150, { &P18462,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X dual sim Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126672 = {S49078, 141, { &P18462,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA4747 = {S49079, 117, { &P12174,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-F480I/BOIG2 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA27441 = {S49080, 108, { &P12174,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-F480I-ORANGE/AFIJ1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA27454 = {S49081, 110, { &P12174,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-F480I-Bouygues/AGII1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA64026 = {S49082, 126, { &P12174,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-F480I-Vodafone/BUIJ2 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA64437 = {S49083, 99, { &P12174,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-F480i/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA38537 = {S49084, 135, { &P14106,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-au; GT-I9000T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA166460 = {S49085, 137, { &P14106,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; GT-I9000T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA659323 = {S49086, 59, { &P14106,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I9000T Build/FROYO)*/
const Device UA2019127 = {S49087, 61, { &P14106,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-I9000T Build/FROYO)*/
const Device UA2257298 = {S49088, 131, { &P20209,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID7036 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2566711 = {S49089, 140, { &P20289,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567318 = {S49090, 133, { &P20289,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2599973 = {S49091, 140, { &P20289,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC6435LVW Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2185252 = {S49092, 31, { &P14188,&P5129,&P24672,&P18092 }};	/*Useragent - Nokia5233/UCWEB8.6.0.180/50/800*/
const Device UA2193304 = {S49093, 119, { &P14188,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5233) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile*/
const Device UA2421696 = {S49094, 120, { &P14188,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia 5233) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/352/UCWEB Mobile*/
const Device UA2974137 = {S49095, 140, { &P22867,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5303 Build/12.0.A.1.257) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2484555 = {S49096, 152, { &P15877,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MOTOROLA RAZR Build/9.8.2O-72_VZW-8) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2583708 = {S49097, 148, { &P15877,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2382138 = {S49098, 139, { &P20103,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; EK-GC100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469371 = {S49099, 139, { &P20103,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; EK-GC100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2482536 = {S49100, 147, { &P20103,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-EK-GC100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2651721 = {S49101, 61, { &P20103,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; EK-GC100 Build/JRO03C)*/
const Device UA2660070 = {S49102, 61, { &P20103,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; EK-GC100 Build/JZO54K)*/
const Device UA2892839 = {S49103, 147, { &P20103,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-EK-GC100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2741605 = {S49104, 130, { &P15882,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P5110 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755134 = {S49105, 130, { &P15882,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P5110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA86536 = {S49106, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F704i(c100;TB;W23H12)*/
const Device UA87608 = {S49107, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 L704i(c100;TB;W24H14)*/
const Device UA87611 = {S49108, 33, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 L705iX(c100;TB;W24H14)*/
const Device UA87613 = {S49109, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 L852i(c100;TB;W24H14)*/
const Device UA90390 = {S49110, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 L03C(c500;TB;W24H16)*/
const Device UA90391 = {S49111, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N03C(c500;TB;W24H16)*/
const Device UA90388 = {S49112, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P03C(c500;TB;W24H16)*/
const Device UA90387 = {S49113, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH04C(c500;TB;W24H16)*/
const Device UA373905 = {S49114, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P705i(c100;TB;W24H15)*/
const Device UA374233 = {S49115, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F04A(c100;TB;W24H17)*/
const Device UA400337 = {S49116, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 L03B(c100;TB;W24H16)*/
const Device UA415471 = {S49117, 33, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH903i(c100;TB;W24H16)*/
const Device UA432213 = {S49118, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N02B(c500;TB;W24H16)*/
const Device UA445955 = {S49119, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH04B(c500;TB;W24H14)*/
const Device UA481773 = {S49120, 33, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SO906i(c100;TB;W24H18)*/
const Device UA484290 = {S49121, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P704i(c100;TB;W24H12)*/
const Device UA488784 = {S49122, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 D705i(c100;TB;W24H16)*/
const Device UA494904 = {S49123, 103, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 D903i(c100;TB;W28H20) (compatible; Hatena-Mobile-Gateway/1.2; +http://mgw.hatena.ne.jp/help)*/
const Device UA503583 = {S49124, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F705i(c100;TB;W24H17)*/
const Device UA509977 = {S49125, 35, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P706imyu(c100;TB;W24H15)*/
const Device UA511789 = {S49126, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P06A(c100;TB;W24H15)*/
const Device UA512879 = {S49127, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH05B(c500;TB;W24H16)*/
const Device UA515927 = {S49128, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 L06A(c100;TB;W24H15)*/
const Device UA519359 = {S49129, 33, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SO706i(c100;TB;W24H16)*/
const Device UA520852 = {S49130, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N01A(c100;TB;W24H16)*/
const Device UA522113 = {S49131, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F904i(c100;TB;W24H17)*/
const Device UA523808 = {S49132, 33, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH904i(c100;TB;W24H16)*/
const Device UA554224 = {S49133, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N05A(c100;TB;W24H16)*/
const Device UA559326 = {S49134, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH11C(c500;TB;W24H16)*/
const Device UA560973 = {S49135, 33, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SO705i(c100;TB;W24H16)*/
const Device UA561202 = {S49136, 33, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH07A3(c500;TB;W24H14)*/
const Device UA561474 = {S49137, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH10C(c500;TB;W24H16)*/
const Device UA563633 = {S49138, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F10C(c500;TB;W24H16)*/
const Device UA566006 = {S49139, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F11C(c500;TB;W24H16)*/
const Device UA569238 = {S49140, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P06C(c500;TB;W24H16)*/
const Device UA572106 = {S49141, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N05C(c500;TB;W30H20)*/
const Device UA635276 = {S49142, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P904i(c100;TB;W24H15)*/
const Device UA640569 = {S49143, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH02B(c500;TB;W24H16)*/
const Device UA650218 = {S49144, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N03B(c500;TB;W24H16)*/
const Device UA671550 = {S49145, 33, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SO905i(c100;TB;W24H18)*/
const Device UA672949 = {S49146, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N09A3(c500;TB;W24H16)*/
const Device UA731751 = {S49147, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N04A(c100;TB;W30H20)*/
const Device UA736784 = {S49148, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F09C(c500;TB;W24H16)*/
const Device UA756240 = {S49149, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F906i(c100;TB;W24H17)*/
const Device UA757053 = {S49150, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P09A3(c500;TB;W24H15)*/
const Device UA762493 = {S49151, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P05C(c500;TB;W24H16)*/
const Device UA767630 = {S49152, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N904i(c100;TB;W24H16)*/
const Device UA1852317 = {S49153, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F706i(c100;TB;W24H17)*/
const Device UA1894829 = {S49154, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 CA01C(c500;TB;W30H20)*/
const Device UA1895017 = {S49155, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P04C(c500;TB;W24H16)*/
const Device UA1939258 = {S49156, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F02D(c500;TB;W24H16)*/
const Device UA1942069 = {S49157, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N03D(c500;TB;W24H16)*/
const Device UA2138221 = {S49158, 35, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P705imyu(c100;TB;W24H15)*/
const Device UA2156211 = {S49159, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F04D(c500;TB;W24H16)*/
const Device UA2207793 = {S49160, 33, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH706i(c100;TB;W24H16)*/
const Device UA2211611 = {S49161, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH05C(c500;TB;W30H15)*/
const Device UA2216597 = {S49162, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P03D(c500;TB;W24H16)*/
const Device UA2216717 = {S49163, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N02D(c500;TB;W24H16)*/
const Device UA2242247 = {S49164, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH03D(c500;TB;W24H16)*/
const Device UA2242301 = {S49165, 33, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH703i(c100;TB;W24H12)*/
const Device UA2265098 = {S49166, 33, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH705i(c100;TB;W24H16)*/
const Device UA2268190 = {S49167, 33, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N703iD(c100;TB;W24H12)*/
const Device UA2269522 = {S49168, 33, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH704i(c100;TB;W24H16)*/
const Device UA2274781 = {S49169, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 L04A(c100;TB;W24H15)*/
const Device UA2292935 = {S49170, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F07A(c100;TB;W20H08)*/
const Device UA2494656 = {S49171, 32, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH05D(c500;TB;W24H16)*/
const Device UA2519542 = {S49172, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F06D(c500;TB;W24H16)*/
const Device UA2527501 = {S49173, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P01E(c500;TB;W24H16)*/
const Device UA2694810 = {S49174, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F01E(c500;TB;W24H16)*/
const Device UA2786169 = {S49175, 31, { &P15364,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 L01B(c500;TB;W24H16)*/
const Device UA118847 = {S49176, 48, { &P15364,&P21442,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows 2000)*/
const Device UA122828 = {S49177, 48, { &P15364,&P21442,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows 2000)*/
const Device UA2755862 = {S49178, 129, { &P18554,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BNTV400 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2756414 = {S49179, 136, { &P22585,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L720 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA924267 = {S49180, 139, { &P19910,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; LG-E617G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2475881 = {S49181, 61, { &P19910,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-E617G Build/IML74K)*/
const Device UA600178 = {S49182, 70, { &P15364,&P16110,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:11.0) Gecko/X Firefox/11.0*/
const Device UA624881 = {S49183, 65, { &P15364,&P16110,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:11.0) Gecko/X Firefox/11.0*/
const Device UA641789 = {S49184, 111, { &P15364,&P6596,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2268.0.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.1 Safari/536.11*/
const Device UA668374 = {S49185, 112, { &P15364,&P6596,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2237.0.2012) AppleWebKit/536.10 (KHTML, like Gecko) Chrome/20.0.1125.0 Safari/536.10*/
const Device UA895252 = {S49186, 111, { &P15364,&P6596,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2268.16.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.11 Safari/536.11*/
const Device UA898477 = {S49187, 111, { &P15364,&P6596,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2268.23.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.15 Safari/536.11*/
const Device UA900407 = {S49188, 111, { &P15364,&P6596,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2268.42.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.22 Safari/536.11*/
const Device UA902002 = {S49189, 113, { &P15364,&P6596,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2268.42.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.22 Safari/536.11*/
const Device UA903438 = {S49190, 111, { &P15364,&P6596,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2268.70.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.34 Safari/536.11*/
const Device UA904678 = {S49191, 111, { &P15364,&P6596,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2268.85.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.37 Safari/536.11*/
const Device UA905433 = {S49192, 113, { &P15364,&P6596,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2268.85.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.37 Safari/536.11*/
const Device UA909493 = {S49193, 112, { &P15364,&P6596,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2268.105.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.54 Safari/536.11*/
const Device UA910788 = {S49194, 114, { &P15364,&P6596,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2268.105.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.54 Safari/536.11*/
const Device UA912504 = {S49195, 112, { &P15364,&P6596,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2268.111.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA913269 = {S49196, 114, { &P15364,&P6596,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2268.111.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA925315 = {S49197, 112, { &P15364,&P6596,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2268.124.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.59 Safari/536.11*/
const Device UA925510 = {S49198, 114, { &P15364,&P6596,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2268.124.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.59 Safari/536.11*/
const Device UA2222817 = {S49199, 110, { &P15364,&P6596,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2141.0.2012) AppleWebKit/536.8 (KHTML, like Gecko) Chrome/20.0.1102.0 Safari/536.8*/
const Device UA194725 = {S49200, 134, { &P13999,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC HD2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA209779 = {S49201, 134, { &P13999,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC HD2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA252084 = {S49202, 134, { &P13999,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC HD2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA337612 = {S49203, 130, { &P13999,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HD2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA575349 = {S49204, 130, { &P13999,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HD2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA641162 = {S49205, 134, { &P13999,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC HD2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA657822 = {S49206, 122, { &P13999,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC HD2 Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1895103 = {S49207, 58, { &P13999,&P24694,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC HD2 Build/MIUI)*/
const Device UA1938719 = {S49208, 54, { &P13999,&P24694,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; HD2 Build/MIUI)*/
const Device UA1981405 = {S49209, 127, { &P13999,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC HD2 Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2019198 = {S49210, 58, { &P13999,&P24694,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC HD2 Build/MIUI)*/
const Device UA2019327 = {S49211, 148, { &P13999,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC HD2 Build/MIUIAndroid 2.3.4a) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021448 = {S49212, 58, { &P13999,&P24694,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC HD2 Build/MIUI)*/
const Device UA2021894 = {S49213, 137, { &P13999,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; HTC HD2 Build/MIUIclk) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023323 = {S49214, 122, { &P13999,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC HD2 Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2033136 = {S49215, 127, { &P13999,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC HD2 Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1972923 = {S49216, 131, { &P20864,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A13-MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2174520 = {S49217, 138, { &P20864,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A13-MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257752 = {S49218, 131, { &P20864,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A13-MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2335088 = {S49219, 60, { &P20864,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; A13-MID Build/IML74K)*/
const Device UA2352735 = {S49220, 60, { &P20864,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; A13-MID Build/IMM76D)*/
const Device UA2420902 = {S49221, 137, { &P20864,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A13MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2457105 = {S49222, 138, { &P20864,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; mid-A13 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2502762 = {S49223, 130, { &P20864,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A13MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2526023 = {S49224, 59, { &P20864,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; A13MID Build/IMM76D)*/
const Device UA2778063 = {S49225, 60, { &P20864,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; mid-A13 Build/IMM76D)*/
const Device UA2848572 = {S49226, 138, { &P20864,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A13_MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3043041 = {S49227, 131, { &P20864,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; A13_MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3044637 = {S49228, 131, { &P20864,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID-A13 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2681004 = {S49229, 139, { &P22382,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8262 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2959591 = {S49230, 61, { &P22382,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8262 Build/JZO54K)*/
const Device UA3124490 = {S49231, 137, { &P22382,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-I8262 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2679138 = {S49232, 131, { &P18454,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGPT12 Build/TJS0166) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2709605 = {S49233, 131, { &P18454,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGPT13 Build/TJS0166) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2973949 = {S49234, 130, { &P12879,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2976689 = {S49235, 130, { &P12879,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T859 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2566798 = {S49236, 133, { &P13644,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583214 = {S49237, 126, { &P13644,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2584090 = {S49238, 133, { &P13644,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; EVO Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA3082995 = {S49239, 133, { &P15767,&P18432,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A523 Safari/8536.25*/
const Device UA3082996 = {S49240, 131, { &P15767,&P18432,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A403 Safari/8536.25*/
const Device UA3083010 = {S49241, 133, { &P15767,&P18432,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A550 Safari/8536.25*/
const Device UA3083002 = {S49242, 131, { &P15767,&P18432,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A407 Safari/8536.25*/
const Device UA3083406 = {S49243, 134, { &P15767,&P18432,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A8500 Safari/8536.25*/
const Device UA3083410 = {S49244, 134, { &P15767,&P18432,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A8426 Safari/8536.25*/
const Device UA3095975 = {S49245, 131, { &P15767,&P18432,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A406 Safari/8536.25*/
const Device UA2873711 = {S49246, 120, { &P17595,&P21721,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA3051327 = {S49247, 135, { &P17595,&P21721,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.36 (KHTML, like Gecko) Iron/27.0.1500.0 Chrome/27.0.1500.0 Safari/537.36*/
const Device UA243904 = {S49248, 150, { &P13157,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_SensationXE_Beats_Z715e; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA708369 = {S49249, 150, { &P13157,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_SensationXE_Beats_Z715a; en-au) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA604102 = {S49250, 73, { &P15364,&P17285,&P22340,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:11.0) Gecko/X Firefox/11.0 CometBird/11.0*/
const Device UA1953934 = {S49251, 76, { &P15364,&P17285,&P22340,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:11.0) Gecko/X Firefox/11.0 CometBrowser/11.0*/
const Device UA2205380 = {S49252, 147, { &P15880,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2226434 = {S49253, 147, { &P15880,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ME865 Build/6.7.2_GC-220-EDS-3) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2836999 = {S49254, 141, { &P15880,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2364808 = {S49255, 123, { &P15364,&P18110,&P18519,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) CasperJS/1.0.0-RC1+Phantomjs/1.6.0 Safari/534.34*/
const Device UA2387608 = {S49256, 121, { &P15364,&P18110,&P18519,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) CasperJS/1.0.0-RC1+Phantomjs/1.6.0 Safari/534.34*/
const Device UA76627 = {S49257, 80, { &P12835,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-S3770/S3770AHKG3 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA636609 = {S49258, 82, { &P12835,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-S3770K/S3770KDDKK2 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA657443 = {S49259, 74, { &P12835,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-S3770L/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA1953305 = {S49260, 74, { &P12835,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-S3770M/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2339113 = {S49261, 82, { &P12835,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-S3770Y/S3770YXXLC1 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA744128 = {S49262, 143, { &P15594,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; SGH-T589W Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2813116 = {S49263, 67, { &P15594,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-T589W Build/GINGERBREAD)*/
const Device UA2408202 = {S49264, 110, { &P23589,&P18378,&P19820,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo WiiU) AppleWebKit/534.52 (KHTML, like Gecko) NX/2.1.0.8.23 NintendoBrowser/1.1.0.7579.EU*/
const Device UA2408197 = {S49265, 110, { &P23589,&P18378,&P19820,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo WiiU) AppleWebKit/534.52 (KHTML, like Gecko) NX/2.1.0.8.23 NintendoBrowser/1.1.0.7579.US*/
const Device UA2425855 = {S49266, 110, { &P23589,&P18378,&P19820,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo WiiU) AppleWebKit/534.52 (KHTML, like Gecko) NX/2.1.0.8.23 NintendoBrowser/1.1.0.7579.JP*/
const Device UA19948 = {S49267, 132, { &P13685,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-; LS670 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24652 = {S49268, 134, { &P13685,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LS670 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55115 = {S49269, 133, { &P13685,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LS670 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55116 = {S49270, 131, { &P13685,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LS670 Build/ZVC) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA63275 = {S49271, 131, { &P13685,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; LS670 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90293 = {S49272, 140, { &P13685,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LS670 Build/FRG83) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA186229 = {S49273, 58, { &P13685,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LS670 Build/FRG83G)*/
const Device UA500611 = {S49274, 57, { &P13685,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; LS670 Build/FRG83)*/
const Device UA534014 = {S49275, 141, { &P13685,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LS670 Build/FRG83G) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA631042 = {S49276, 60, { &P18219,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia202/2.0 (20.28) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2167052 = {S49277, 60, { &P18219,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia203/2.0 (20.36) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2642259 = {S49278, 69, { &P18219,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia202/2.0 (20.28) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia202*/
const Device UA41573 = {S49279, 137, { &P14014,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Liquid MT Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59618 = {S49280, 137, { &P14014,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Liquid MT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73301 = {S49281, 137, { &P14014,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Liquid MT Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA647481 = {S49282, 138, { &P14014,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Liquid MT Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1937754 = {S49283, 62, { &P14014,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; Liquid MT Build/GRK39F)*/
const Device UA2021029 = {S49284, 132, { &P14014,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Liquid MT Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0  Safari/533.1*/
const Device UA2772410 = {S49285, 140, { &P12968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.0.200) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2539251 = {S49286, 145, { &P18614,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT890 Build/9.8.2I-50_SMI-26) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2606520 = {S49287, 138, { &P18614,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT890 Build/9.8.2I-50_SMI-29) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22*/
const Device UA127846 = {S49288, 73, { &P17595,&P18120,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X; U; en) Presto/2.6.30 Version/10.61*/
const Device UA135552 = {S49289, 73, { &P17595,&P18120,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X; U; en) Presto/2.6.30 Version/10.62*/
const Device UA149245 = {S49290, 71, { &P17595,&P18120,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; PPC Mac OS X; U; en) Presto/2.6.30 Version/10.63*/
const Device UA643144 = {S49291, 73, { &P17595,&P18120,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X; U; en) Presto/2.6.30 Version/10.60*/
const Device UA94715 = {S49292, 46, { &P17595,&P17296,&P18215,&P18092 }};	/*Useragent - Apple Mac OS X v10.6.2 CoreMedia v1.0.0.10C540*/
const Device UA109634 = {S49293, 46, { &P17595,&P17296,&P18215,&P18092 }};	/*Useragent - Apple Mac OS X v10.6.4 CoreMedia v1.0.0.10F569*/
const Device UA150766 = {S49294, 46, { &P17595,&P17296,&P18215,&P18092 }};	/*Useragent - Apple Mac OS X v10.6.5 CoreMedia v1.0.0.10H574*/
const Device UA160226 = {S49295, 47, { &P17595,&P17296,&P18215,&P18092 }};	/*Useragent - Apple Mac OS X v10.6.4 CoreMedia v1.0.0.10F2108*/
const Device UA168294 = {S49296, 46, { &P17595,&P17296,&P18215,&P18092 }};	/*Useragent - Apple Mac OS X v10.6.6 CoreMedia v1.0.0.10J567*/
const Device UA176876 = {S49297, 46, { &P17595,&P17296,&P18215,&P18092 }};	/*Useragent - Apple Mac OS X v10.6.7 CoreMedia v1.0.0.10J869*/
const Device UA177248 = {S49298, 47, { &P17595,&P17296,&P18215,&P18092 }};	/*Useragent - Apple Mac OS X v10.6.7 CoreMedia v1.0.0.10J3250*/
const Device UA181544 = {S49299, 47, { &P17595,&P17296,&P18215,&P18092 }};	/*Useragent - Apple Mac OS X v10.6.6 CoreMedia v1.0.0.10J3210*/
const Device UA189427 = {S49300, 47, { &P17595,&P17296,&P18215,&P18092 }};	/*Useragent - Apple Mac OS X v10.6.3 CoreMedia v1.0.0.10D2125*/
const Device UA196029 = {S49301, 47, { &P17595,&P17296,&P18215,&P18092 }};	/*Useragent - Apple Mac OS X v10.6.7 CoreMedia v1.0.0.10J4139*/
const Device UA197798 = {S49302, 46, { &P17595,&P17296,&P18215,&P18092 }};	/*Useragent - Apple Mac OS X v10.6.8 CoreMedia v1.0.0.10K540*/
const Device UA235394 = {S49303, 46, { &P17595,&P17296,&P18215,&P18092 }};	/*Useragent - Apple Mac OS X v10.6.8 CoreMedia v1.0.0.10K549*/
const Device UA391451 = {S49304, 44, { &P17595,&P17296,&P18215,&P18092 }};	/*Useragent - Apple Mac OS X v10.6 CoreMedia v1.0.0.10A432*/
const Device UA448132 = {S49305, 46, { &P17595,&P17296,&P18215,&P18092 }};	/*Useragent - Apple Mac OS X v10.6.3 CoreMedia v1.0.0.10D575*/
const Device UA812363 = {S49306, 47, { &P17595,&P17296,&P18215,&P18092 }};	/*Useragent - Apple Mac OS X v10.6.4 CoreMedia v1.0.0.10F3061*/
const Device UA2585769 = {S49307, 73, { &P16329,&P21715,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PLAYSTATION 3 4.40) AppleWebKit/531.22.8 (KHTML, like Gecko)*/
const Device UA1962750 = {S49308, 127, { &P20518,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2028285 = {S49309, 128, { &P20518,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; A10 Build/unknown) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2426722 = {S49310, 127, { &P20518,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; A10 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2428794 = {S49311, 134, { &P20518,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2727440 = {S49312, 56, { &P20518,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; A10 Build/IMM76D)*/
const Device UA2729738 = {S49313, 127, { &P20518,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-eg; A10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2986750 = {S49314, 146, { &P20518,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; Treq A10 Pocket Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3070412 = {S49315, 130, { &P20518,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; A10 Build/R13.4-rc1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2368399 = {S49316, 75, { &P17595,&P17297,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5) Presto/2.12.388 Version/12.11*/
const Device UA23176 = {S49317, 134, { &P14031,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I5503 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60656 = {S49318, 135, { &P14031,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-nz; GT-I5503T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60658 = {S49319, 137, { &P14031,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; GT-I5503T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019643 = {S49320, 59, { &P14031,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I5503T Build/FROYO)*/
const Device UA2019753 = {S49321, 58, { &P14031,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I5503 Build/FROYO)*/
const Device UA2020295 = {S49322, 61, { &P14031,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-I5503T Build/FROYO)*/
const Device UA2974140 = {S49323, 130, { &P22606,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2710257 = {S49324, 140, { &P23227,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I467 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2937287 = {S49325, 133, { &P23227,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I467M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2962779 = {S49326, 69, { &P23227,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-I467 Build/JZO54K)*/
const Device UA2349624 = {S49327, 182, { &P14540,&P5843,&P24672,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA175530 = {S49328, 134, { &P17938,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; AS740 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA184198 = {S49329, 134, { &P17938,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; US740 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA735239 = {S49330, 58, { &P17938,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; US740 Build/FRG83G)*/
const Device UA2021493 = {S49331, 58, { &P17938,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; AS740 Build/FRG83G)*/
const Device UA2566698 = {S49332, 146, { &P18114,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Transformer Prime TF201 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667202 = {S49333, 146, { &P18114,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Transformer Prime TF201 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2863701 = {S49334, 128, { &P18114,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TF201 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA576163 = {S49335, 156, { &P15364,&P18110,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1 solfo-linkchecker/1.0 (http://solfo.com/linkbot.html)*/
const Device UA2385762 = {S49336, 98, { &P15364,&P5690,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.1.8244(Windows;Microsoft Windows (5.1.2600.3);en;kml:2.2;client:Free;type:default)*/
const Device UA2427592 = {S49337, 98, { &P15364,&P5690,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.2.8415(Windows;Microsoft Windows (5.1.2600.3);en;kml:2.2;client:Free;type:default)*/
const Device UA2529636 = {S49338, 98, { &P15364,&P5690,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.2.8415(Windows;Microsoft Windows (5.1.2600.3);de;kml:2.2;client:Free;type:default)*/
const Device UA2568080 = {S49339, 98, { &P15364,&P5690,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.3);en;kml:2.2;client:Free;type:default)*/
const Device UA2568081 = {S49340, 122, { &P15364,&P5690,&P20429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 5.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/7.0.3.8542 Safari/532.4*/
const Device UA2602520 = {S49341, 98, { &P15364,&P5690,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.2);en;kml:2.2;client:Free;type:default)*/
const Device UA2603569 = {S49342, 98, { &P15364,&P5690,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.3);de;kml:2.2;client:Free;type:default)*/
const Device UA2666080 = {S49343, 98, { &P15364,&P5690,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.3);it;kml:2.2;client:Free;type:default)*/
const Device UA2666737 = {S49344, 98, { &P15364,&P5690,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.3);ru;kml:2.2;client:Free;type:default)*/
const Device UA2666972 = {S49345, 98, { &P15364,&P5690,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.3);fr;kml:2.2;client:Free;type:default)*/
const Device UA2700357 = {S49346, 103, { &P15364,&P5690,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.3);zh-Hans;kml:2.2;client:Free;type:default)*/
const Device UA2704955 = {S49347, 98, { &P15364,&P5690,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.3);nl;kml:2.2;client:Free;type:default)*/
const Device UA2705751 = {S49348, 98, { &P15364,&P5690,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.3);es;kml:2.2;client:Free;type:default)*/
const Device UA2708165 = {S49349, 102, { &P15364,&P5690,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.3);es-419;kml:2.2;client:Free;type:default)*/
const Device UA2780181 = {S49350, 98, { &P15364,&P5690,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.3);pt;kml:2.2;client:Free;type:default)*/
const Device UA2795159 = {S49351, 98, { &P15364,&P5690,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (5.1.2600.2);de;kml:2.2;client:Free;type:default)*/
const Device UA14842 = {S49352, 178, { &P13517,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; Garmin-Asus A50 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17; A50-V5.0.73-user-20101105*/
const Device UA1871543 = {S49353, 73, { &P13517,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Garmin-Asus A50 Build/ERE27)*/
const Device UA2018667 = {S49354, 169, { &P13517,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; Garmin-Asus A50 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17; A50-V5.0.73-user-X*/
const Device UA2566546 = {S49355, 131, { &P15882,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P5110 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566796 = {S49356, 131, { &P15882,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P5110 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2675797 = {S49357, 76, { &P15364,&P18110,&P22225,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:20.0) Gecko/X Firefox/20.0 SeaMonkey/2.17.1*/
const Device UA2862308 = {S49358, 63, { &P15364,&P18110,&P22225,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:20.0) Gecko/X SeaMonkey/2.17.1*/
const Device UA2956378 = {S49359, 65, { &P15364,&P18110,&P22225,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:20.0) Gecko/X SeaMonkey/2.17.1*/
const Device UA352682 = {S49360, 156, { &P16542,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ALCATEL_one_touch_909S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA882276 = {S49361, 156, { &P16542,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ALCATEL_one_touch_909B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1941664 = {S49362, 156, { &P16542,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; ALCATEL_one_touch_909A Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1993533 = {S49363, 161, { &P16542,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; USCC_ALCATEL_one_touch_909B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA9140 = {S49364, 142, { &P16584,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-ca; SAMSUNG-SGH-I896 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38284 = {S49365, 140, { &P16584,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ru-; SAMSUNG-SGH-I896 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA204869 = {S49366, 66, { &P16584,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; SAMSUNG-SGH-I896 Build/FROYO)*/
const Device UA2702558 = {S49367, 133, { &P18466,&P18110,&P22587,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Safari/535.19 Silk-Accelerated=false*/
const Device UA2707886 = {S49368, 97, { &P17595,&P18207,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:20.0) Gecko/X Firefox/20.0 AlexaToolbar/alxf-2.18*/
const Device UA68907 = {S49369, 128, { &P13866,&P17484,&P4732,&P18092 }};	/*Useragent - SAMSUNG-SGH-A847/A847UCJK2; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.17574; U; en-US) Opera 9.50*/
const Device UA2387384 = {S49370, 151, { &P15803,&P5843,&P24673,&P18092 }};	/*Useragent - Nokia200/2.0 (11.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia200) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2566441 = {S49371, 131, { &P15932,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3110 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566968 = {S49372, 131, { &P15932,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3110 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA3065085 = {S49373, 128, { &P21769,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ME301T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA362289 = {S49374, 120, { &P15364,&P7133,&P22454,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7*/
const Device UA362650 = {S49375, 113, { &P15364,&P7133,&P22454,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7*/
const Device UA909835 = {S49376, 144, { &P15767,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8L1 Safari/7534.48.3*/
const Device UA910378 = {S49377, 144, { &P15767,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8F191 Safari/7534.48.3*/
const Device UA910807 = {S49378, 144, { &P15767,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8J2 Safari/7534.48.3*/
const Device UA910821 = {S49379, 144, { &P15767,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8G4 Safari/7534.48.3*/
const Device UA911069 = {S49380, 144, { &P15767,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8F190 Safari/7534.48.3*/
const Device UA911190 = {S49381, 144, { &P15767,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8J3 Safari/7534.48.3*/
const Device UA911230 = {S49382, 144, { &P15767,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8H7 Safari/7534.48.3*/
const Device UA911474 = {S49383, 144, { &P15767,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8K2 Safari/7534.48.3*/
const Device UA2260715 = {S49384, 144, { &P15767,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8H8 Safari/7534.48.3*/
const Device UA54314 = {S49385, 73, { &P13392,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C3510/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2732651 = {S49386, 139, { &P22905,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-VS410PP Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2957929 = {S49387, 63, { &P22905,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-VS410PP Build/GRK39F)*/
const Device UA3014534 = {S49388, 138, { &P20146,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9082 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057626 = {S49389, 60, { &P20146,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9082 Build/JDQ39)*/
const Device UA2468382 = {S49390, 132, { &P18465,&P18110,&P19805,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.6 Safari/535.19 Silk-Accelerated=true*/
const Device UA2755333 = {S49391, 137, { &P18509,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA192255 = {S49392, 136, { &P17687,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; MZ604 Build/H.6.2-21-demo) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA313670 = {S49393, 131, { &P17687,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; ja-jp; MZ604 Build/H.6.2-20) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA314388 = {S49394, 131, { &P17687,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-ca; MZ604 Build/H.6.2-21) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA314690 = {S49395, 131, { &P17687,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; MZ604 Build/H.6.2-24) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA318670 = {S49396, 131, { &P17687,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; MZ604 Build/H.6.2-26) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA371355 = {S49397, 131, { &P17687,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; MZ604 Build/H.6.2-28) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA559093 = {S49398, 133, { &P17687,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; de-de; MZ604 Build/H.6.2-35.1) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA717825 = {S49399, 131, { &P17687,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-gb; MZ604 Build/H.6.2-34) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1885102 = {S49400, 60, { &P17687,&P6545,&P4755,&P18092 }};	/*Useragent - Dalvik/1.5.0 (Linux; U; Android 3.0.1; MZ604 Build/H.6.2-24)*/
const Device UA2755560 = {S49401, 197, { &P18283,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.2/552,2.0.2.552,2013-05-22 23:05, +0100, gb*/
const Device UA3027199 = {S49402, 197, { &P18283,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T999 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us*/
const Device UA3027747 = {S49403, 197, { &P18283,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I535 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us*/
const Device UA2974807 = {S49404, 132, { &P13644,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022204 = {S49405, 132, { &P13644,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; EVO Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA236553 = {S49406, 69, { &P17595,&P17297,&P18333,&P18092 }};	/*Useragent - iTunes/10.4.1 (Macintosh; Intel Mac OS X 10.7.1) AppleWebKit/534.48.3*/
const Device UA420290 = {S49407, 67, { &P17595,&P17297,&P18333,&P18092 }};	/*Useragent - iTunes/10.3.1 (Macintosh; Intel Mac OS X 10.7) AppleWebKit/534.48.3*/
const Device UA440968 = {S49408, 70, { &P17595,&P17297,&P18333,&P18092 }};	/*Useragent - iTunes/10.6.1 (Macintosh; Intel Mac OS X 10.7.3) AppleWebKit/534.53.11*/
const Device UA692974 = {S49409, 70, { &P17595,&P17297,&P18333,&P18092 }};	/*Useragent - iTunes/10.4.1 (Macintosh; Intel Mac OS X 10.7.2) AppleWebKit/534.51.22*/
const Device UA901952 = {S49410, 69, { &P17595,&P17297,&P18333,&P18092 }};	/*Useragent - iTunes/10.6.3 (Macintosh; Intel Mac OS X 10.7.4) AppleWebKit/534.56.5*/
const Device UA2199847 = {S49411, 67, { &P17595,&P17297,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Macintosh; Intel Mac OS X 10.7.5) AppleWebKit/534.57.7*/
const Device UA2405939 = {S49412, 57, { &P17595,&P17297,&P18333,&P18092 }};	/*Useragent - iTunes/11.0 (Macintosh; OS X 10.7.4) AppleWebKit/534.56.5*/
const Device UA2429048 = {S49413, 59, { &P17595,&P17297,&P18333,&P18092 }};	/*Useragent - iTunes/11.0.1 (Macintosh; OS X 10.7.5) AppleWebKit/534.57.7*/
const Device UA646851 = {S49414, 91, { &P15364,&P7133,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.8.1.20) Gecko/X BRI/1 Firefox/2.0.0.20*/
const Device UA881488 = {S49415, 67, { &P15364,&P7133,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; U; en; rv:1.8.1) Gecko/X Firefox/2.0.0*/
const Device UA1872664 = {S49416, 83, { &P15364,&P7133,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.8.1.3) Gecko/X BonEcho/2.0.0.3*/
const Device UA2460333 = {S49417, 61, { &P15364,&P7133,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:17.0) Gecko/X Firefox/20.6.14*/
const Device UA2511475 = {S49418, 130, { &P15767,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/6533.18.5 iLunascape/3201*/
const Device UA2514228 = {S49419, 139, { &P15767,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA2534357 = {S49420, 141, { &P15767,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_1_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA2537316 = {S49421, 136, { &P15767,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_1 like Mac OS X; ko_kr) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA2564184 = {S49422, 132, { &P15767,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/6533.18.5 iLunascape/3203*/
const Device UA2571306 = {S49423, 138, { &P15767,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_1_2 like Mac OS X; ko_kr) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA2582863 = {S49424, 173, { &P15767,&P19826,&P17138,&P18092 }};	/*Useragent - User-Agent: Mozilla/5.0 (iPad; U; CPU OS 6_1_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5*/
const Device UA2607468 = {S49425, 141, { &P15767,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_1_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA2669797 = {S49426, 138, { &P15767,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_1_3 like Mac OS X; ko_kr) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA2713012 = {S49427, 161, { &P15767,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_1_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5*/
const Device UA2764807 = {S49428, 153, { &P15767,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_1 like Mac OS X; zh-cn Model:iPad2,1) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA2898645 = {S49429, 132, { &P15767,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/6533.18.5 iLunascape/3212*/
const Device UA94794 = {S49430, 82, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 1.1.4322; MSIECrawler)*/
const Device UA98531 = {S49431, 88, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; MSIECrawler)*/
const Device UA101820 = {S49432, 87, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; MSIECrawler)*/
const Device UA104844 = {S49433, 158, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; NN4.1.0.0; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; MSIECrawler)*/
const Device UA112166 = {S49434, 68, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; MSIECrawler)*/
const Device UA120401 = {S49435, 119, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; InfoPath.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; MSIECrawler)*/
const Device UA143399 = {S49436, 132, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSIECrawler)*/
const Device UA143402 = {S49437, 63, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; MSIECrawler)*/
const Device UA144562 = {S49438, 80, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; InfoPath.2; MSIECrawler)*/
const Device UA144821 = {S49439, 78, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; PeoplePal 3.0; MSIECrawler)*/
const Device UA151893 = {S49440, 107, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; MSIECrawler)*/
const Device UA233517 = {S49441, 76, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; GTB7.1; MSIECrawler)*/
const Device UA277560 = {S49442, 151, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSIECrawler)*/
const Device UA283647 = {S49443, 144, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; MSIECrawler)*/
const Device UA298703 = {S49444, 144, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; InfoPath.2; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSIECrawler)*/
const Device UA374974 = {S49445, 99, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; InfoPath.2; MSIECrawler)*/
const Device UA382525 = {S49446, 80, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; InfoPath.1; MSIECrawler)*/
const Device UA384830 = {S49447, 100, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; InfoPath.2; .NET CLR 2.0.50727; MSIECrawler)*/
const Device UA501317 = {S49448, 82, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 1.0.3705; MSIECrawler)*/
const Device UA518295 = {S49449, 87, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.0.3705; MSIECrawler)*/
const Device UA541125 = {S49450, 186, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; {73D5DF9F-E500-BB8D-C9C1-7F04A4EB0838}; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSIECrawler)*/
const Device UA551542 = {S49451, 163, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; MSIECrawler)*/
const Device UA691852 = {S49452, 127, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; MSIECrawler)*/
const Device UA699441 = {S49453, 76, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; GTB7.2; MSIECrawler)*/
const Device UA712381 = {S49454, 83, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 2.0.50727; MSIECrawler)*/
const Device UA730931 = {S49455, 176, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSIECrawler)*/
const Device UA745472 = {S49456, 100, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; InfoPath.2; MSIECrawler)*/
const Device UA780453 = {S49457, 95, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; GTB6; Ant.com Toolbar 1.6; MSIECrawler)*/
const Device UA873993 = {S49458, 79, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; FunWebProducts; MSIECrawler)*/
const Device UA893765 = {S49459, 111, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; MSIECrawler)*/
const Device UA1841053 = {S49460, 76, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; GTB6.6; MSIECrawler)*/
const Device UA1852665 = {S49461, 84, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; FunWebProducts; MSIECrawler)*/
const Device UA1894643 = {S49462, 115, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Embedded Web Browser from: http://bsalsa.com/; MSIECrawler)*/
const Device UA1896381 = {S49463, 87, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Crayon Crawler; snprtz|TX; (R1 1.5))*/
const Device UA1980539 = {S49464, 76, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; GTB7.3; MSIECrawler)*/
const Device UA2127235 = {S49465, 158, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; FunWebProducts; ezPeer+ v1.0 Beta (0.4.1.98); ezPeer+ v1.0 (0.5.0.00); .NET CLR 1.1.4322; MSIECrawler)*/
const Device UA2127807 = {S49466, 107, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; MSIECrawler)*/
const Device UA2217515 = {S49467, 95, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; BTRSX; .NET CLR 2.0.50727; MSIECrawler)*/
const Device UA2266754 = {S49468, 151, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322; MSIECrawler)*/
const Device UA2267584 = {S49469, 151, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSIECrawler)*/
const Device UA2279425 = {S49470, 119, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2; MSIECrawler)*/
const Device UA2313953 = {S49471, 84, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QQDownload 708; MSIECrawler)*/
const Device UA2333380 = {S49472, 88, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; qdesk 2.3.1186.202; MSIECrawler)*/
const Device UA2487775 = {S49473, 235, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; BTRSX; SIMBAR={4CDE8DFA-3984-11E2-8CE6-XDA9311}; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; MSIECrawler)*/
const Device UA2566202 = {S49474, 245, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; BTRSX; SIMBAR={4CDE8DFA-3984-11E2-8CE6-XDA9311}; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; MSIECrawler)*/
const Device UA2569438 = {S49475, 245, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; SIMBAR={4CDE8DFA-3984-11E2-8CE6-XDA9311}; BTRSX; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; MSIECrawler)*/
const Device UA2673414 = {S49476, 249, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; SIMBAR={4CDE8DFA-3984-11E2-8CE6-XDA9311}; BTRS99959; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; MSIECrawler)*/
const Device UA2752757 = {S49477, 297, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; SIMBAR={4CDE8DFA-3984-11E2-8CE6-XDA9311}; AlexaToolbar/amzni-3.0; BTRS99956; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E; AlexaToolbar/amzni-3.0; MSIECrawler)*/
const Device UA2776502 = {S49478, 76, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; GTB7.4; MSIECrawler)*/
const Device UA3057920 = {S49479, 85, { &P18254,&P5690,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; MSIECrawler);c2VvLWNyYXdsZXI=*/
const Device UA2678888 = {S49480, 101, { &P15364,&P17188,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA2040720 = {S49481, 80, { &P14824,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-E2252/E2252DDLG2 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2257772 = {S49482, 139, { &P19865,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A850L Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2338336 = {S49483, 139, { &P19865,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A850S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2349729 = {S49484, 139, { &P19865,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A850K Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2454767 = {S49485, 61, { &P19865,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-A850K Build/IMM76I)*/
const Device UA2464522 = {S49486, 61, { &P19865,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-A850L Build/IMM76I)*/
const Device UA2464860 = {S49487, 61, { &P19865,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-A850S Build/IMM76I)*/
const Device UA2577775 = {S49488, 137, { &P19865,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A850L Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2696464 = {S49489, 138, { &P19865,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A850L Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2698125 = {S49490, 137, { &P19865,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A850K Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2846065 = {S49491, 137, { &P19865,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A850S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2395175 = {S49492, 136, { &P20213,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N9500 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2764167 = {S49493, 58, { &P20213,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; N9500 Build/IMM76I)*/
const Device UA31014 = {S49494, 173, { &P14873,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Garminfone Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17; A50-V5.0.70-user-20101025*/
const Device UA447404 = {S49495, 165, { &P14873,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Garminfone Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 MobileSafari/530.17; A50-V5.0.70-user-X*/
const Device UA544120 = {S49496, 165, { &P14873,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; GarminfoneBuild/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17; A50-V5.0.70-user-X*/
const Device UA652193 = {S49497, 68, { &P14873,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Garminfone Build/ERE27)*/
const Device UA810638 = {S49498, 165, { &P14873,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.1-update1; en-us; Garminfone Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17; A50-V5.0.70-user-X*/
const Device UA816846 = {S49499, 165, { &P14873,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.1-update1; en-us; Garminfone Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17; A50-V5.0.70-user-X*/
const Device UA1864663 = {S49500, 165, { &P14873,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us;Garminfone Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17; A50-V5.0.70-user-X*/
const Device UA2953444 = {S49501, 140, { &P24044,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-R740C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022833 = {S49502, 62, { &P24044,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-R740C Build/IMM76D)*/
const Device UA1977437 = {S49503, 136, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.5; en-us; I9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2360372 = {S49504, 139, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-i9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2397312 = {S49505, 136, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.5; en-gb; I9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2432108 = {S49506, 136, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; I9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2456702 = {S49507, 136, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; i9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2466950 = {S49508, 136, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-gb; I9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2482986 = {S49509, 137, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; i92200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2504840 = {S49510, 136, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; i9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2546050 = {S49511, 136, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.5; es-es; I9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555215 = {S49512, 139, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9220 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2574565 = {S49513, 136, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.5; es-us; I9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2574843 = {S49514, 136, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.5; it-it; I9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580213 = {S49515, 137, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652955 = {S49516, 139, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.6; en-us; GT-i9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2693232 = {S49517, 137, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.5; it-it; I92200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2711702 = {S49518, 137, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; i92200 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2726201 = {S49519, 139, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2727987 = {S49520, 130, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9220 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2778459 = {S49521, 61, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9220 Build/JZO54K)*/
const Device UA2792120 = {S49522, 136, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; fr-fr; I9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2841482 = {S49523, 136, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; es-es; I9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2842737 = {S49524, 137, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.5; en-us; I92200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878255 = {S49525, 136, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; nl-nl; I9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3042652 = {S49526, 136, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HK-I9220 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3071021 = {S49527, 137, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.5; en-gb; I92200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3120605 = {S49528, 139, { &P22875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-i9220 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2709260 = {S49529, 130, { &P22596,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2714045 = {S49530, 137, { &P22596,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA3065157 = {S49531, 145, { &P18114,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Transformer Prime TF201 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA250071 = {S49532, 135, { &P17997,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID7012 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458771 = {S49533, 69, { &P17997,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.2 (Linux; U; Android 2.3.3; en_US; MID7012 Build/MID7012 X)*/
const Device UA458823 = {S49534, 59, { &P17997,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID7012 Build/GRI40)*/
const Device UA2697919 = {S49535, 61, { &P18722,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A830L Build/JZO54K)*/
const Device UA2703115 = {S49536, 139, { &P18722,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A830S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2713482 = {S49537, 139, { &P18722,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A830L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2714622 = {S49538, 144, { &P18722,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A830K/2.09 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2729529 = {S49539, 61, { &P18722,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A830K Build/JZO54K)*/
const Device UA2781850 = {S49540, 142, { &P18722,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30*/
const Device UA2861000 = {S49541, 138, { &P18722,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830L Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2986807 = {S49542, 61, { &P18722,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A830S Build/JZO54K)*/
const Device UA2992684 = {S49543, 138, { &P18722,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2994016 = {S49544, 138, { &P18722,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830K Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2997320 = {S49545, 142, { &P18722,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30*/
const Device UA2998315 = {S49546, 171, { &P18722,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830S Build/Phi?n b?n Galaxy S4 - By Hanzoo1504) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30*/
const Device UA2999268 = {S49547, 151, { &P18722,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830K/2.12 Build/Hanzoo1504) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30*/
const Device UA3049049 = {S49548, 147, { &P18722,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830K/2.09 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30*/
const Device UA3049583 = {S49549, 146, { &P18722,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830L Build/Hanzoo1504) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30*/
const Device UA3062379 = {S49550, 172, { &P18722,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A830K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 NAVER(inapp; search; 230; 4.7.7)*/
const Device UA3074340 = {S49551, 147, { &P18722,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830K/2.11 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30*/
const Device UA3078247 = {S49552, 145, { &P18722,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830KE/1.06 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102784 = {S49553, 148, { &P18722,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; IM-A830K/2.12 Build/Hanzoo1504) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3109365 = {S49554, 145, { &P18722,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; IM-A830KE/1.07 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3114663 = {S49555, 144, { &P18722,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A830K/2.13 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469540 = {S49556, 136, { &P19850,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; X325a Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2856411 = {S49557, 143, { &P19850,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; X325a-parrot Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3082989 = {S49558, 134, { &P15767,&P18117,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9B206 Safari/7534.48.3*/
const Device UA3096071 = {S49559, 132, { &P15767,&P18117,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9B176 Safari/7534.48.3*/
const Device UA2739938 = {S49560, 148, { &P22850,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI H881C Build/HuaweiH881C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937676 = {S49561, 70, { &P22850,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI H881C Build/HuaweiH881C)*/
const Device UA2755539 = {S49562, 144, { &P18640,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2028896 = {S49563, 138, { &P18234,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-P895 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2500381 = {S49564, 60, { &P18234,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P895 Build/IMM76D)*/
const Device UA2522605 = {S49565, 140, { &P18234,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-sa; LG-P895qb Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566514 = {S49566, 156, { &P18113,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583204 = {S49567, 149, { &P18113,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA3561 = {S49568, 126, { &P18088,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) 480x800; XV6975; Window Mobile 6.5 Professional;*/
const Device UA5802 = {S49569, 86, { &P18088,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) RhodiumW*/
const Device UA12707 = {S49570, 87, { &P18088,&P5448,&P4808,&P18092 }};	/*Useragent - 8900b/MR2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA13452 = {S49571, 46, { &P18088,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE)*/
const Device UA13468 = {S49572, 126, { &P18088,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) 480x800; XV6875; Window Mobile 6.1 Professional;*/
const Device UA21132 = {S49573, 89, { &P18088,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Sprint S511*/
const Device UA21194 = {S49574, 93, { &P18088,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Sprint-SPH-i350*/
const Device UA38514 = {S49575, 87, { &P18088,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) T-01A_6.5*/
const Device UA42969 = {S49576, 92, { &P18088,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Sprint:PPC6800*/
const Device UA46511 = {S49577, 83, { &P18088,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) T-01A*/
const Device UA69555 = {S49578, 67, { &P18088,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 320x240)*/
const Device UA93302 = {S49579, 77, { &P18088,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA94208 = {S49580, 109, { &P18088,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 480x640; DX900/2.0 Profile*/
const Device UA99607 = {S49581, 51, { &P18088,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC)*/
const Device UA101697 = {S49582, 128, { &P18088,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) 480x800; XV6875.1; Window Mobile 6.5 Professional;*/
const Device UA134444 = {S49583, 58, { &P18088,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone)*/
const Device UA273788 = {S49584, 79, { &P18088,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 320x240; Smartphone)*/
const Device UA277849 = {S49585, 59, { &P18088,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE, BrailleNote)*/
const Device UA60287 = {S49586, 103, { &P12132,&P17485,&P4223,&P18092 }};	/*Useragent - SonyEricssonJ10/R7CA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2*/
const Device UA63507 = {S49587, 105, { &P12132,&P17485,&P4223,&P18092 }};	/*Useragent - SonyEricssonJ10i2/R7CA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2*/
const Device UA64550 = {S49588, 104, { &P12132,&P17485,&P4223,&P18092 }};	/*Useragent - SonyEricssonJ10i/R7CA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2*/
const Device UA610060 = {S49589, 140, { &P13644,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  HTC Evo 4G Build/GRI40) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA866242 = {S49590, 134, { &P13644,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  EVO Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA900697 = {S49591, 133, { &P13644,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; EVO Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA908972 = {S49592, 137, { &P13644,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PC36100 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2017926 = {S49593, 133, { &P13644,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; EVO Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2026185 = {S49594, 139, { &P13644,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Evo 4G Build/GRI40) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA599710 = {S49595, 88, { &P17968,&P18066,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.3; Linux; Opera Mobi/ADR-X; U; de) Presto/2.10.254 Version/12.00*/
const Device UA613215 = {S49596, 88, { &P17968,&P18066,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.4; Linux; Opera Mobi/ADR-X; U; en) Presto/2.10.254 Version/12.00*/
const Device UA834036 = {S49597, 88, { &P17968,&P18066,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.2; Linux; Opera Mobi/ADR-X; U; de) Presto/2.10.254 Version/12.00*/
const Device UA2028523 = {S49598, 91, { &P17968,&P18066,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.1; Linux; Opera Mobi/ADR-X; U; en-GB) Presto/2.10.254 Version/12.00*/
const Device UA2413503 = {S49599, 88, { &P17968,&P18066,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.9; Linux; Opera Mobi/ADR-X; U; en) Presto/2.10.254 Version/12.00*/
const Device UA3065481 = {S49600, 137, { &P15575,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083329 = {S49601, 138, { &P15575,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083437 = {S49602, 137, { &P15575,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094215 = {S49603, 137, { &P15575,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-i9100G Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2508163 = {S49604, 164, { &P12280,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B142 Version/6.1 Safari/8536.25 Safari/6533.18.5*/
const Device UA2510762 = {S49605, 164, { &P12280,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B143 Version/6.1 Safari/8536.25 Safari/6533.18.5*/
const Device UA2538240 = {S49606, 166, { &P12280,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B146 Version/6.1 Safari/8536.25 Safari/6533.18.5*/
const Device UA2561168 = {S49607, 148, { &P12280,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; ko_kr) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5*/
const Device UA2569609 = {S49608, 182, { &P12280,&P19826,&P17138,&P18092 }};	/*Useragent - User-Agent: Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5*/
const Device UA2582070 = {S49609, 150, { &P12280,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_2 like Mac OS X; ko_kr) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5*/
const Device UA2597580 = {S49610, 164, { &P12280,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B141 Version/6.1 Safari/8536.25 Safari/6533.18.5*/
const Device UA2607309 = {S49611, 166, { &P12280,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B329 Version/6.1 Safari/8536.25 Safari/6533.18.5*/
const Device UA2660235 = {S49612, 164, { &P12280,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B144 Version/6.1 Safari/8536.25 Safari/6533.18.5*/
const Device UA2671537 = {S49613, 150, { &P12280,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; ko_kr) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5*/
const Device UA2714036 = {S49614, 170, { &P12280,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5*/
const Device UA2738249 = {S49615, 170, { &P12280,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5*/
const Device UA2739507 = {S49616, 166, { &P12280,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B350 Version/6.1 Safari/8536.25 Safari/6533.18.5*/
const Device UA2854290 = {S49617, 166, { &P12280,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B145 Version/6.1 Safari/8536.25 Safari/6533.18.5*/
const Device UA2995766 = {S49618, 150, { &P12280,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_4 like Mac OS X; ko_kr) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5*/
const Device UA3072475 = {S49619, 139, { &P12280,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B329 Safari/6533.18.5*/
const Device UA3073086 = {S49620, 135, { &P12280,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; 6536.26) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5*/
const Device UA369753 = {S49621, 142, { &P15805,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC_Holiday Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026968 = {S49622, 147, { &P15805,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-kr; HTC_Raider_X710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032442 = {S49623, 142, { &P15805,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Holiday Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2496250 = {S49624, 140, { &P15805,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; -au; HTC_Holiday Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2518423 = {S49625, 69, { &P15805,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Raider X710e Build/IML74K)*/
const Device UA2574649 = {S49626, 147, { &P15805,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-kr; HTC Raider X710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3067300 = {S49627, 135, { &P21648,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-I9505 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081279 = {S49628, 141, { &P21648,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-I9505G Build/JWR66Y.S003) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA78012 = {S49629, 140, { &P17281,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; SCH-I905 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 USCC-I905*/
const Device UA686826 = {S49630, 133, { &P17281,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; SCH-I905 4G Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA694681 = {S49631, 59, { &P17281,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; SCH-I905 Build/HTJ85B)*/
const Device UA2024469 = {S49632, 137, { &P17281,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; SCH-I905 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA168414 = {S49633, 144, { &P18254,&P18110,&P17470,&P18092 }};	/*Useragent - curl/7.18.2 (x86_64-pc-linux-gnu) libcurl/7.18.2 OpenSSL/0.9.8g zlib/1.2.3.3 libidn/1.8 (FlipboardProxy/1.1; +http://flipboard.com/browserproxy)*/
const Device UA2945956 = {S49634, 88, { &P17098,&P5567,&P17370,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/5.0/27.2338; U; en) Presto/2.8.119 240X320 Samsung SCH-U660*/
const Device UA2197817 = {S49635, 132, { &P15919,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2355053 = {S49636, 132, { &P15919,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2510412 = {S49637, 133, { &P15919,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100B Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2622735 = {S49638, 138, { &P15919,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2625025 = {S49639, 133, { &P15919,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1;  GT-P3100 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2626256 = {S49640, 132, { &P15919,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.0; GT-P3100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2627315 = {S49641, 133, { &P15919,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2;  GT-P3100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2628506 = {S49642, 134, { &P15919,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1;  GT-P3100B Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2632155 = {S49643, 160, { &P15919,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2 Jelly Bean; Galaxy Tab 2 7.0 GT-P3100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2492385 = {S49644, 101, { &P15364,&P17188,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2973967 = {S49645, 137, { &P18283,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974447 = {S49646, 137, { &P18283,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I535 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020864 = {S49647, 137, { &P18283,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-l9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022551 = {S49648, 137, { &P18283,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023206 = {S49649, 135, { &P18283,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-06D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023800 = {S49650, 138, { &P18283,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300T Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3057979 = {S49651, 138, { &P18283,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999V Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3092236 = {S49652, 134, { &P18283,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2423979 = {S49653, 140, { &P19844,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ARCHOS 80 COBALT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2461751 = {S49654, 69, { &P19844,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ARCHOS 80 COBALT Build/IMM76D)*/
const Device UA1906254 = {S49655, 117, { &P17595,&P18207,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA2002800 = {S49656, 117, { &P17595,&P18207,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA2202284 = {S49657, 117, { &P17595,&P18207,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA2599135 = {S49658, 117, { &P17595,&P18207,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA2869900 = {S49659, 117, { &P17595,&P18207,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA3113742 = {S49660, 117, { &P17595,&P18207,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA251965 = {S49661, 122, { &P15364,&P5690,&P18210,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 5.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.1.0.5001 Safari/532.4*/
const Device UA421906 = {S49662, 101, { &P15364,&P5690,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);es-ES;kml:2.2;client:Free;type:default)*/
const Device UA622988 = {S49663, 101, { &P15364,&P5690,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);en-US;kml:2.2;client:Free;type:default)*/
const Device UA674006 = {S49664, 101, { &P15364,&P5690,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);pt-BR;kml:2.2;client:Free;type:default)*/
const Device UA689535 = {S49665, 101, { &P15364,&P5690,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);en-GB;kml:2.2;client:Free;type:default)*/
const Device UA694982 = {S49666, 101, { &P15364,&P5690,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);it-IT;kml:2.2;client:Free;type:default)*/
const Device UA705486 = {S49667, 101, { &P15364,&P5690,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);en-AU;kml:2.2;client:Free;type:default)*/
const Device UA720069 = {S49668, 98, { &P15364,&P5690,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);en;kml:2.2;client:Free;type:default)*/
const Device UA726294 = {S49669, 101, { &P15364,&P5690,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.2);en-US;kml:2.2;client:Free;type:default)*/
const Device UA753899 = {S49670, 102, { &P15364,&P5690,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);es-419;kml:2.2;client:Free;type:default)*/
const Device UA758332 = {S49671, 101, { &P15364,&P5690,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);fr-FR;kml:2.2;client:Free;type:default)*/
const Device UA768247 = {S49672, 98, { &P15364,&P5690,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);de;kml:2.2;client:Free;type:default)*/
const Device UA772416 = {S49673, 101, { &P15364,&P5690,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);de-DE;kml:2.2;client:Free;type:default)*/
const Device UA772918 = {S49674, 101, { &P15364,&P5690,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);ru-RU;kml:2.2;client:Free;type:default)*/
const Device UA775605 = {S49675, 101, { &P15364,&P5690,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);nl-NL;kml:2.2;client:Free;type:default)*/
const Device UA1914992 = {S49676, 101, { &P15364,&P5690,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (5.1.2600.3);en-CA;kml:2.2;client:Free;type:default)*/
const Device UA359560 = {S49677, 137, { &P15841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/345) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA576407 = {S49678, 137, { &P15841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/262) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA609217 = {S49679, 142, { &P15841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/2.0.1.30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA629645 = {S49680, 142, { &P15841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/2.0.0.56) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2004009 = {S49681, 142, { &P15841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/2.0.1.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023722 = {S49682, 141, { &P15841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PlayBook Build/GINGERBREAD) AppleWebKit/535.1+ (KHTML, like Gecko) Version/7.2.0.0 Safari/535.1+*/
const Device UA2033848 = {S49683, 142, { &P15841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/2.1.0.48) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2169097 = {S49684, 143, { &P15841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/2.1.0.123) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2193495 = {S49685, 143, { &P15841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/2.1.0.131) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2532593 = {S49686, 144, { &P15841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-gb; PlayBook Build/10.0.0.279) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2582454 = {S49687, 144, { &P15841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/10.0.0.336) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2782698 = {S49688, 144, { &P15841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/10.1.0.103) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2860897 = {S49689, 144, { &P15841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/10.0.0.347) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2958530 = {S49690, 144, { &P15841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-gb; PlayBook Build/10.1.0.123) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2995345 = {S49691, 143, { &P15841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/10.1.0.83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3015660 = {S49692, 144, { &P15841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/10.1.0.147) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3083091 = {S49693, 143, { &P15841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/2.1.0.147) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2741719 = {S49694, 137, { &P15575,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754383 = {S49695, 137, { &P15575,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769987 = {S49696, 138, { &P15575,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2873520 = {S49697, 130, { &P15575,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2893316 = {S49698, 138, { &P15575,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100G Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2974913 = {S49699, 126, { &P15869,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xoom Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022812 = {S49700, 126, { &P15869,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Xoom Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA69856 = {S49701, 73, { &P13035,&P5043,&P4030,&P18092 }};	/*Useragent - NokiaC2-02/2.0 (06.51) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-02*/
const Device UA74467 = {S49702, 62, { &P13035,&P5043,&P4030,&P18092 }};	/*Useragent - NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA77552 = {S49703, 64, { &P13035,&P5043,&P4030,&P18092 }};	/*Useragent - NokiaC2-02.1/2.0 (06.52) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2470549 = {S49704, 158, { &P14225,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Desire HD Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2532353 = {S49705, 158, { &P14225,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Desire HD Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2562414 = {S49706, 157, { &P14225,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Desire HD Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2737793 = {S49707, 158, { &P14225,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Desire HD Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2385738 = {S49708, 112, { &P18644,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia112/03.32; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.5.16.9*/
const Device UA2693607 = {S49709, 140, { &P23705,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C811 4G Build/C811M010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878975 = {S49710, 140, { &P23705,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C811 4G Build/C811M020) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3012802 = {S49711, 62, { &P23705,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; C811 4G Build/C811M020)*/
const Device UA3015104 = {S49712, 143, { &P23705,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C811 4G Build/C811M021_DU) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2227330 = {S49713, 146, { &P14862,&P5204,&P24668,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia2700c-2) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2753032 = {S49714, 140, { &P18582,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT28at Build/6.1.C.1.111) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755072 = {S49715, 138, { &P18582,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT28i Build/6.1.E.2.68) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755537 = {S49716, 137, { &P18582,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT28h Build/6.1.E.3.7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA1995644 = {S49717, 132, { &P18075,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; KIS Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2554857 = {S49718, 137, { &P18075,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; KIS PLUS Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2689643 = {S49719, 56, { &P18075,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; KIS Build/GRK39F)*/
const Device UA2512227 = {S49720, 156, { &P14635,&P5843,&P24676,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac3-00) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2204914 = {S49721, 142, { &P15228,&P5043,&P24668,&P18092 }};	/*Useragent - NokiaC2-03/2.0 (07.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-03) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA925418 = {S49722, 118, { &P15364,&P5690,&P22375,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.4.2.3000 Chrome/18.0.966.0 Safari/535.12*/
const Device UA1996404 = {S49723, 125, { &P15364,&P5690,&P22375,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.4.2.3000 Chrome/18.0.966.0 Safari/535.12*/
const Device UA1996839 = {S49724, 118, { &P15364,&P5690,&P22375,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.4.2.3000 Chrome/18.0.966.0 Safari/535.12*/
const Device UA1996843 = {S49725, 125, { &P15364,&P5690,&P22375,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.4.2.3000 Chrome/18.0.966.0 Safari/535.12*/
const Device UA2605289 = {S49726, 133, { &P18466,&P18110,&P21707,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Safari/535.19 Silk-Accelerated=false*/
const Device UA2355320 = {S49727, 132, { &P18466,&P18110,&P19803,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Safari/535.19 Silk-Accelerated=true*/
const Device UA2974362 = {S49728, 137, { &P16356,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974475 = {S49729, 137, { &P16356,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-N7000 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974656 = {S49730, 137, { &P16356,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T879 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020479 = {S49731, 138, { &P16356,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020939 = {S49732, 138, { &P16356,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021166 = {S49733, 131, { &P16356,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M480W Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022179 = {S49734, 135, { &P16356,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-05D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022218 = {S49735, 138, { &P16356,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022336 = {S49736, 138, { &P16356,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000B Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA197090 = {S49737, 85, { &P15364,&P7133,&P21929,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0) chromeframe/5.0.317.0*/
const Device UA401024 = {S49738, 225, { &P15364,&P7133,&P21929,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3) chromeframe/5.0.375.62*/
const Device UA570746 = {S49739, 92, { &P15364,&P7133,&P21929,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0) chromeframe/5.0.317.0*/
const Device UA726477 = {S49740, 240, { &P15364,&P7133,&P21929,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3; Tablet PC 2.0) chromeframe/5.0.375.62*/
const Device UA2272038 = {S49741, 244, { &P15364,&P7133,&P21929,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MDDR; InfoPath.2; OfficeLiveConnector.1.4; OfficeLivePatch.1.3) chromeframe/5.0.317.0*/
const Device UA2479556 = {S49742, 173, { &P15364,&P7133,&P21929,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E) chromeframe/5.0.375.62*/
const Device UA2668462 = {S49743, 86, { &P15364,&P7133,&P21929,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0) chromeframe/5.0.317.0*/
const Device UA2989387 = {S49744, 93, { &P15364,&P7133,&P21929,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0) chromeframe/5.0.317.0*/
const Device UA2967889 = {S49745, 125, { &P15767,&P22882,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA3024395 = {S49746, 127, { &P15767,&P22882,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA3094873 = {S49747, 118, { &P15767,&P22882,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari Version/5.1*/
const Device UA3109483 = {S49748, 128, { &P15767,&P22882,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/5.1 Mobile/11A465 Safari/7537.51.1*/
const Device UA3111613 = {S49749, 161, { &P15767,&P22882,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Version/5.1 Safari/7534.48.3 NAVER(inapp; search; 103; 4.7.6)*/
const Device UA3115460 = {S49750, 129, { &P15767,&P22882,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA3115673 = {S49751, 120, { &P15767,&P22882,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Safari Version/5.1*/
const Device UA3127822 = {S49752, 116, { &P15767,&P22882,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari/7534.48.3*/
const Device UA3128974 = {S49753, 163, { &P15767,&P22882,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Version/5.1 Safari/7534.48.3 NAVER(inapp; search; 103; 4.7.6)*/
const Device UA3129911 = {S49754, 135, { &P15767,&P22882,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Version/5.1 Safari/7534.48.3 Kikin/1.9.5*/
const Device UA192111 = {S49755, 137, { &P18001,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; GT-I5510M Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA566943 = {S49756, 61, { &P18001,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; GT-I5510M Build/FROYO)*/
const Device UA259495 = {S49757, 11, { &P18254,&P7480,&P18418,&P18091 }};	/*Useragent - Voyager/1.0*/
const Device UA2356518 = {S49758, 16, { &P18254,&P7480,&P18418,&P18091 }};	/*Useragent - parsijoo-crawler*/
const Device UA2717763 = {S49759, 89, { &P18254,&P7480,&P18418,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; parsijoo; +http://www.parsijoo.ir/; ehsan.mousakazemi@gmail.com)*/
const Device UA3057922 = {S49760, 28, { &P18254,&P7480,&P18418,&P18091 }};	/*Useragent - Voyager/1.0;c2VvLWNyYXdsZXI=*/
const Device UA2974996 = {S49761, 138, { &P12972,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975038 = {S49762, 145, { &P12972,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T989 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975086 = {S49763, 137, { &P12972,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2234950 = {S49764, 128, { &P19775,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A210 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2436485 = {S49765, 127, { &P15767,&P18117,&P22900,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.2 Mobile/9B206 Safari/7534.48.3*/
const Device UA2437116 = {S49766, 125, { &P15767,&P18117,&P22900,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.2 Mobile/9B176 Safari/7534.48.3*/
const Device UA2676325 = {S49767, 137, { &P18509,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676822 = {S49768, 130, { &P18509,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA628757 = {S49769, 178, { &P15118,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.3 NokiaX7-00; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.4.2.6 Mobile Safari/533.4 3gpp-gba*/
const Device UA2042344 = {S49770, 191, { &P15118,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.3 NokiaX7-00/111.040.1511; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.1.4 Mobile Safari/535.1 3gpp-gba*/
const Device UA2422571 = {S49771, 102, { &P13324,&P5055,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokian8-00) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2422745 = {S49772, 32, { &P13324,&P5055,&P24673,&P18092 }};	/*Useragent - NokiaN8-00/UCWEB8.7.0.187/51/999*/
const Device UA2426537 = {S49773, 123, { &P13324,&P5055,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; NokiaN8-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile*/
const Device UA2461145 = {S49774, 42, { &P13324,&P5055,&P24673,&P18092 }};	/*Useragent - NokiaN8-00UCBrowser/8.7.1.234/51/355/UCWEB*/
const Device UA246169 = {S49775, 164, { &P17711,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P504 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA1907351 = {S49776, 60, { &P17711,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P504 Build/FRG83G)*/
const Device UA2020150 = {S49777, 163, { &P17711,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;LG-P504 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2020155 = {S49778, 163, { &P17711,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P504 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2023668 = {S49779, 163, { &P17711,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.2; en-us; LG-P504 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA92804 = {S49780, 62, { &P15364,&P17285,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.5.22 Version/10.51*/
const Device UA96094 = {S49781, 62, { &P15364,&P17285,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.5.24 Version/10.53*/
const Device UA135711 = {S49782, 62, { &P15364,&P17285,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; en) Presto/2.5.24 Version/10.52*/
const Device UA264618 = {S49783, 62, { &P15364,&P17285,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; en) Presto/2.5.24 Version/10.54*/
const Device UA271978 = {S49784, 62, { &P15364,&P17285,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; en) Presto/2.5.22 Version/10.50*/
const Device UA50072 = {S49785, 151, { &P13233,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile myTouch 3G Build/DRC92) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA50179 = {S49786, 151, { &P13233,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile myTouch 3G Build/DMD64) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA50233 = {S49787, 151, { &P13233,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile myTouch 3G Build/DRD35) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA50241 = {S49788, 151, { &P13233,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile myTouch 3G Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA50249 = {S49789, 154, { &P13233,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; WOWMobile myTouch 3G Build/unknown) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA56081 = {S49790, 151, { &P13233,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile myTouch 3G Build/DRC83) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA443337 = {S49791, 158, { &P13233,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; T-Mobile myTouch 3G Build/Super D beta) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2763894 = {S49792, 138, { &P22970,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2891875 = {S49793, 139, { &P22970,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599N Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA899397 = {S49794, 60, { &P15785,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia201/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2759161 = {S49795, 69, { &P15785,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia201/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia201*/
const Device UA653236 = {S49796, 136, { &P17211,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; ARCHOS 101G9 Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA668481 = {S49797, 134, { &P17211,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; ARCHOS 101G9 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2022624 = {S49798, 65, { &P17211,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; ARCHOS 101G9 Build/HTK75D)*/
const Device UA2708688 = {S49799, 134, { &P22754,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HP Slate 7 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2885400 = {S49800, 133, { &P15767,&P22882,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4400f Safari/8536.25*/
const Device UA2947258 = {S49801, 133, { &P15767,&P22882,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4414e Safari/8536.25*/
const Device UA3080761 = {S49802, 133, { &P15767,&P22882,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4449d Safari/8536.25*/
const Device UA3096207 = {S49803, 131, { &P15767,&P22882,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A465 Safari/8536.25*/
const Device UA3115675 = {S49804, 133, { &P15767,&P22882,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A501 Safari/8536.25*/
const Device UA87001 = {S49805, 145, { &P12280,&P17129,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_0 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5A345 Safari/525.20*/
const Device UA87085 = {S49806, 145, { &P12280,&P17129,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_0 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5A347 Safari/525.20*/
const Device UA372316 = {S49807, 146, { &P12280,&P17129,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_0 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5A347 Safari/525.200*/
const Device UA466000 = {S49808, 145, { &P12280,&P17129,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_0_2 like Mac OS X; es-es) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5C1 Safari/525.20*/
const Device UA734733 = {S49809, 147, { &P12280,&P17129,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_0_1 like Mac OS X; es-es) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5B108 Safari/525.20*/
const Device UA2149676 = {S49810, 144, { &P12280,&P17129,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_0 like Mac OS X; en-us) AppleWebKit/528.4.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5A347 Safari/525.20*/
const Device UA2712421 = {S49811, 160, { &P12280,&P17129,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_0_1 like Mac OS X; haw-US) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5G77 Safari/525.20*/
const Device UA2982653 = {S49812, 141, { &P12280,&P17129,&P4571,&P18092 }};	/*Useragent - 'Mozilla/5.0 (iPhone; U; CPU iOS 2_0 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/XXXXX Safari/525.20'*/
const Device UA81115 = {S49813, 84, { &P18088,&P5890,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.1; U; en-us) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA2975058 = {S49814, 145, { &P18114,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Transformer Prime TF201 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2672692 = {S49815, 135, { &P16356,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-05D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2672984 = {S49816, 138, { &P16356,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000B Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673963 = {S49817, 138, { &P16356,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160L Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675070 = {S49818, 137, { &P16356,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T879 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676093 = {S49819, 137, { &P16356,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676278 = {S49820, 137, { &P16356,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-N7000 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2710837 = {S49821, 130, { &P16356,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2713166 = {S49822, 138, { &P16356,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713320 = {S49823, 138, { &P16356,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160K Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735438 = {S49824, 138, { &P16356,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SHV-E160S Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735814 = {S49825, 131, { &P16356,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M480W Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA816712 = {S49826, 147, { &P13452,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA870406 = {S49827, 158, { &P13452,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1984908 = {S49828, 151, { &P13452,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2027293 = {S49829, 147, { &P13452,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2609081 = {S49830, 154, { &P13452,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P1000 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2611468 = {S49831, 148, { &P13452,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; GT-P1000T Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2611694 = {S49832, 148, { &P13452,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT-P1000L Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2611753 = {S49833, 148, { &P13452,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; GT-P1000T Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2612972 = {S49834, 152, { &P13452,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P1000N Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2614673 = {S49835, 148, { &P13452,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT-P1000L Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2618799 = {S49836, 148, { &P13452,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; GT-P1000N Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2632731 = {S49837, 158, { &P13452,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P1000 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2634270 = {S49838, 148, { &P13452,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2741697 = {S49839, 130, { &P22043,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P5113 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2892771 = {S49840, 130, { &P22043,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P5113 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3063762 = {S49841, 138, { &P18283,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999V Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3065996 = {S49842, 137, { &P18283,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066540 = {S49843, 137, { &P18283,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I535 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080899 = {S49844, 137, { &P18283,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094500 = {S49845, 135, { &P18283,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-06D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094633 = {S49846, 137, { &P18283,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-l9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111023 = {S49847, 138, { &P18283,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300T Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2191004 = {S49848, 91, { &P15364,&P17285,&P22775,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0) SlimBrowser/6.01.069*/
const Device UA2566429 = {S49849, 85, { &P15364,&P17285,&P22775,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; SlimBrowser/6.01.100)*/
const Device UA2749950 = {S49850, 100, { &P15364,&P17285,&P22775,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Avant Browser; SlimBrowser/6.01.101)*/
const Device UA2754587 = {S49851, 136, { &P21261,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2866886 = {S49852, 137, { &P21261,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868194 = {S49853, 134, { &P21261,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.3; I9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892996 = {S49854, 134, { &P21261,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; I9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3003184 = {S49855, 136, { &P21261,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.9; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA197020 = {S49856, 137, { &P11747,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Desire Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA209819 = {S49857, 137, { &P11747,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Desire Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA229488 = {S49858, 151, { &P11747,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Desire Build/MIUIAndroid 2.3.4a) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA252033 = {S49859, 137, { &P11747,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Desire Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA338044 = {S49860, 140, { &P11747,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Desire Build/MIUI-XJ) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA421787 = {S49861, 140, { &P11747,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire Build/MIUI-XJ) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA534058 = {S49862, 137, { &P11747,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA544950 = {S49863, 166, { &P11747,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-au; HTC Desire Build/MIUI HTC_Desire_A8183 V2.26.841.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA562747 = {S49864, 61, { &P11747,&P24694,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Desire Build/MIUI)*/
const Device UA567468 = {S49865, 142, { &P11747,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Desire CDMA Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA695460 = {S49866, 61, { &P11747,&P24694,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Desire Build/MIUI)*/
const Device UA711296 = {S49867, 125, { &P11747,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Desire Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1882966 = {S49868, 142, { &P11747,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Desire CDMA Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1883392 = {S49869, 166, { &P11747,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; HTC Desire Build/MIUI HTC_Desire_A8183 V2.26.841.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1930610 = {S49870, 142, { &P11747,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Desire CDMA Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1971792 = {S49871, 130, { &P11747,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Desire Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2020161 = {S49872, 125, { &P11747,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2020868 = {S49873, 139, { &P11747,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; HTC Desire Build/MIUIes) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024208 = {S49874, 140, { &P11747,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; HTC Desire Build/MIUIExt) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2741741 = {S49875, 130, { &P23773,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8010 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755337 = {S49876, 130, { &P23773,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8010 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA21383 = {S49877, 151, { &P12375,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN81 SymbianOS/9.2 Series60/3.1 Release/10.1.028 Mozilla/5.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA63500 = {S49878, 152, { &P12375,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaN81-3/11.0.045 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA63800 = {S49879, 152, { &P12375,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN81/ SymbianOS/9.2 Series60/3.1 Release/20.1.009 Mozilla/5.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA64875 = {S49880, 152, { &P12375,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaN81-1/11.0.045 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA69520 = {S49881, 8, { &P12375,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN81*/
const Device UA508920 = {S49882, 42, { &P12375,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN81(14.01)/SymbianOS/9.1 Series60/3.0*/
const Device UA2717590 = {S49883, 140, { &P17346,&P18432,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A523 Safari/8536.25*/
const Device UA2717890 = {S49884, 138, { &P17346,&P18432,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A403 Safari/8536.25*/
const Device UA2718544 = {S49885, 138, { &P17346,&P18432,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/10A406 Safari/8536.25*/
const Device UA3063679 = {S49886, 139, { &P20289,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC6435LVW Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3065144 = {S49887, 139, { &P20289,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA9036 = {S49888, 105, { &P16003,&P17484,&P3636,&P18092 }};	/*Useragent - SAMSUNG-GT-S5233A/1.0 SHP/VPP/R5 Jasmine/0.8 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA419232 = {S49889, 93, { &P15364,&P17350,&P20804,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; ; en-US) AppleWebKit/527+ (KHTML, like Gecko, Safari/419.3) Qt/4.5.3*/
const Device UA2754263 = {S49890, 130, { &P18702,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8013 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755082 = {S49891, 130, { &P18702,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8013 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA625770 = {S49892, 178, { &P13128,&P5629,&P15760,&P18092 }};	/*Useragent - LG-P690f/V10c Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; LG-P690f Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2023188 = {S49893, 161, { &P13128,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru; LG-P690f Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2024146 = {S49894, 60, { &P13128,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P690f Build/GRJ22)*/
const Device UA94729 = {S49895, 116, { &P17595,&P18146,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (MSIE 7.0; Macintosh; U; SunOS; X11; gu; SV1; InfoPath.2; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648)*/
const Device UA2978841 = {S49896, 129, { &P17595,&P18146,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (MSIE 7.0; Macintosh; U; Trident/4.0; SunOS; X11; gu; SV1; InfoPath.2; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648)*/
const Device UA79877 = {S49897, 81, { &P15364,&P17285,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/1*/
const Device UA131975 = {S49898, 118, { &P15364,&P17285,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/1.0.154.48 Safari/525.19*/
const Device UA599493 = {S49899, 138, { &P15364,&P17285,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/1.0.154.42 Safari/525.19	Chrome 1, Win Vista*/
const Device UA639951 = {S49900, 116, { &P15364,&P17285,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/1, 6, 0, 6 Safari/533.3*/
const Device UA2411371 = {S49901, 104, { &P15364,&P17285,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.3 (KHTML, like Gecko) Chrome/1, 6, 0, 4 Safari/535.3*/
const Device UA2741735 = {S49902, 137, { &P16356,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769349 = {S49903, 138, { &P16356,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769351 = {S49904, 138, { &P16356,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769385 = {S49905, 131, { &P16356,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M480W Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2782531 = {S49906, 137, { &P16356,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783454 = {S49907, 138, { &P16356,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798394 = {S49908, 138, { &P16356,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892470 = {S49909, 130, { &P16356,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA899871 = {S49910, 140, { &P15850,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-LU6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026329 = {S49911, 131, { &P15850,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LG-LU6200 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2027895 = {S49912, 62, { &P15850,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-LU6200 Build/IMM76D)*/
const Device UA2483520 = {S49913, 145, { &P15850,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; LG-LU6200/V20c Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2592545 = {S49914, 145, { &P15850,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LU6200/V20e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2696128 = {S49915, 145, { &P15850,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; LG-LU6200/V-V2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2696284 = {S49916, 145, { &P15850,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; LG-LU6200/VNov Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2701868 = {S49917, 145, { &P15850,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; LG-LU6200/V20F Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2724029 = {S49918, 140, { &P15850,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LG-LU6200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2763303 = {S49919, 145, { &P15850,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LU6200/V20f Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2988035 = {S49920, 148, { &P15850,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LU6200 Build/Sovandyphone X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3086512 = {S49921, 166, { &P15850,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; LG-LU6200/BELL-P930-V20F-FOR-LU6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA180249 = {S49922, 43, { &P17595,&P18120,&P18405,&P18092 }};	/*Useragent - Opera/9.27 (Macintosh; PPC Mac OS X; U; en)*/
const Device UA198345 = {S49923, 45, { &P17595,&P18120,&P18405,&P18092 }};	/*Useragent - Opera/9.23 (Macintosh; Intel Mac OS X; U; en)*/
const Device UA223159 = {S49924, 45, { &P17595,&P18120,&P18405,&P18092 }};	/*Useragent - Opera/9.21 (Macintosh; Intel Mac OS X; U; fr)*/
const Device UA269416 = {S49925, 43, { &P17595,&P18120,&P18405,&P18092 }};	/*Useragent - Opera/9.22 (Macintosh; PPC Mac OS X; U; en)*/
const Device UA289628 = {S49926, 45, { &P17595,&P18120,&P18405,&P18092 }};	/*Useragent - Opera/9.20 (Macintosh; Intel Mac OS X; U; en)*/
const Device UA476449 = {S49927, 45, { &P17595,&P18120,&P18405,&P18092 }};	/*Useragent - Opera/9.24 (Macintosh; Intel Mac OS X; U; en)*/
const Device UA544918 = {S49928, 43, { &P17595,&P18120,&P18405,&P18092 }};	/*Useragent - Opera/9.26 (Macintosh; PPC Mac OS X; U; en)*/
const Device UA546794 = {S49929, 45, { &P17595,&P18120,&P18405,&P18092 }};	/*Useragent - Opera/9.25 (Macintosh; Intel Mac OS X; U; en)*/
const Device UA624867 = {S49930, 25, { &P17595,&P18120,&P18405,&P18092 }};	/*Useragent - Opera/9.23 (Mac OS X; ru)*/
const Device UA812756 = {S49931, 89, { &P17595,&P18120,&P18405,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X; U; en; rv:1.8.0) Gecko/X Firefox/1.5.0 Opera 9.27*/
const Device UA2679143 = {S49932, 145, { &P22613,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-E430-Orange Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2713271 = {S49933, 143, { &P22613,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; hr-hr; LG-E430/V10c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2716744 = {S49934, 146, { &P22613,&P18111,&P18112,&P18092 }};	/*Useragent - LG-E430 Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E430 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2795529 = {S49935, 60, { &P22613,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E430 Build/JZO54K)*/
const Device UA2862708 = {S49936, 153, { &P22613,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-E430/E43010a-EUR-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865458 = {S49937, 143, { &P22613,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; LG-E430/V10b Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3016212 = {S49938, 143, { &P22613,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; LG-E430/V10d Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049725 = {S49939, 153, { &P22613,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E430/E43010a-TSC-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA238201 = {S49940, 101, { &P14886,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; 7 Pro T7576)*/
const Device UA248552 = {S49941, 113, { &P14886,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; USCCHTC-PC93100; USC-US)*/
const Device UA809148 = {S49942, 97, { &P14886,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; PC93100)*/
const Device UA2233969 = {S49943, 140, { &P19924,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ARCHOS 97 CARBON Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2550745 = {S49944, 69, { &P19924,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; ARCHOS 97 CARBON Build/IML74K)*/
const Device UA2428391 = {S49945, 113, { &P18503,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8S by HTC)*/
const Device UA2506235 = {S49946, 121, { &P18503,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8S by HTC; Orange)*/
const Device UA2507102 = {S49947, 95, { &P18503,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; A620e)*/
const Device UA2530106 = {S49948, 95, { &P18503,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; A620b)*/
const Device UA2557590 = {S49949, 95, { &P18503,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; A620m)*/
const Device UA2558370 = {S49950, 126, { &P18503,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8S by HTC; 1.04.161.01)*/
const Device UA2597152 = {S49951, 126, { &P18503,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8S by HTC; 1.04.163.03)*/
const Device UA2670643 = {S49952, 126, { &P18503,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8S by HTC; 2.00.161.01)*/
const Device UA2734535 = {S49953, 126, { &P18503,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8S by HTC; 2.00.163.02)*/
const Device UA3058940 = {S49954, 126, { &P18503,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8S by HTC; 3.29.161.01)*/
const Device UA3072534 = {S49955, 126, { &P18503,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8S by HTC; 2.00.166.01)*/
const Device UA3079664 = {S49956, 126, { &P18503,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Windows Phone 8S by HTC; 3.29.163.01)*/
const Device UA2523750 = {S49957, 136, { &P21606,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LGL86C Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA675752 = {S49958, 154, { &P16373,&P17342,&P17454,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S7250D/S7250DXXKK1; U; Bada/2.0; it-it) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile HVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA686917 = {S49959, 161, { &P16373,&P17342,&P17454,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S7250-BOUYGUES/S7250AGLA3; U; Bada/2.0; fr-fr) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile HVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA1995778 = {S49960, 127, { &P20347,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Kurio7 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2414481 = {S49961, 125, { &P20347,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-; Kurio7 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3032615 = {S49962, 80, { &P17595,&P17297,&P19806,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:17.0) Gecko/X Thunderbird/17.0.8*/
const Device UA609700 = {S49963, 129, { &P18282,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  A200 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA901688 = {S49964, 128, { &P18282,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A200 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2815600 = {S49965, 128, { &P18282,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A200 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2701808 = {S49966, 139, { &P22610,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S6810 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2767968 = {S49967, 140, { &P22610,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-S6810L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956007 = {S49968, 61, { &P22610,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S6810 Build/JZO54K)*/
const Device UA2958996 = {S49969, 140, { &P22610,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; GT-S6810B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998093 = {S49970, 62, { &P22610,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S6810L Build/JZO54K)*/
const Device UA3001588 = {S49971, 140, { &P22610,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-S6810M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3107020 = {S49972, 62, { &P22610,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S6810M Build/JZO54K)*/
const Device UA2366131 = {S49973, 113, { &P19765,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; SAMSUNG; SCH-I930)*/
const Device UA484638 = {S49974, 234, { &P15364,&P17285,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Avant Browser; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)*/
const Device UA639074 = {S49975, 176, { &P15364,&P17285,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Avant Browser)*/
const Device UA711800 = {S49976, 155, { &P15364,&P17285,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Avant Browser)*/
const Device UA833930 = {S49977, 168, { &P15364,&P17285,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Avant Browser; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; Avant Browser)*/
const Device UA1952156 = {S49978, 78, { &P15364,&P17285,&P22730,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 6.0; Avant Browser)*/
const Device UA2221235 = {S49979, 183, { &P15364,&P17285,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Avant Browser)*/
const Device UA2393372 = {S49980, 147, { &P15364,&P17285,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Avant Browser; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; Avant Browser)*/
const Device UA2409456 = {S49981, 163, { &P15364,&P17285,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Avant Browser; Avant Browser; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; Tablet PC 2.0)*/
const Device UA625162 = {S49982, 115, { &P13035,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-02/06.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.62.10*/
const Device UA800367 = {S49983, 117, { &P13035,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-02.1/06.52; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.62.10*/
const Device UA2353240 = {S49984, 114, { &P13035,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-02/07.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6*/
const Device UA2553532 = {S49985, 116, { &P13035,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-02.1/07.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6*/
const Device UA911772 = {S49986, 144, { &P13941,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; S7 Build/Smartfren.androtab.v25.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1995016 = {S49987, 151, { &P13941,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; S7 Build/Smartfren.andromaxtab7.0.v.27.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026003 = {S49988, 144, { &P13941,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; S7 Build/Smartfren.androtab.v19.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029103 = {S49989, 138, { &P13941,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2204030 = {S49990, 151, { &P13941,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; S7 Build/Smartfren.andromaxtab7.0.v.37.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2364575 = {S49991, 129, { &P13941,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S7 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2427258 = {S49992, 151, { &P13941,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S7 Build/Smartfren.andromaxtab7.0.v.38.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2450586 = {S49993, 151, { &P13941,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S7 Build/Smartfren.andromaxtab7.0.v.32.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2521593 = {S49994, 133, { &P13941,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; ENERGY s7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2530074 = {S49995, 126, { &P13941,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; S7 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2848598 = {S49996, 137, { &P13941,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; YTONE S7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2950938 = {S49997, 139, { &P13941,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; ELIYA_S7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3042340 = {S49998, 60, { &P13941,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; vivo S7 Build/IMM76D)*/
const Device UA3047028 = {S49999, 133, { &P13941,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; S7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2753856 = {S50000, 145, { &P18645,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753914 = {S50001, 138, { &P18645,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769381 = {S50002, 137, { &P18645,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-L710 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769618 = {S50003, 138, { &P18645,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773449 = {S50004, 138, { &P18645,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773610 = {S50005, 138, { &P18645,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530U Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773819 = {S50006, 138, { &P18645,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I747M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783005 = {S50007, 138, { &P18645,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2974272 = {S50008, 130, { &P15919,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974812 = {S50009, 130, { &P15919,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022651 = {S50010, 131, { &P15919,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100B Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA380073 = {S50011, 130, { &P22041,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-au; GT-P6810 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2024005 = {S50012, 59, { &P22041,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P6810 Build/HTJ85B)*/
const Device UA2608127 = {S50013, 137, { &P22041,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-au; GT-P6810 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2615017 = {S50014, 128, { &P22041,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6810 Build/HTJ85B) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2616862 = {S50015, 128, { &P22041,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P6810 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2617789 = {S50016, 71, { &P22041,&P6437,&P16179,&P18092 }};	/*Useragent - Vyrso/2.2.3 (Linux; U; Android 3.2; en-US; GT-P6810 Build/HTJ85B.ZSKL2)*/
const Device UA2618941 = {S50017, 269, { &P22041,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-au; GT-P6810 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.11;FBDM/{density=1.0,width=800,height=1232};FBLC/en_AU;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P6810;FBSV/3.2;]*/
const Device UA2621744 = {S50018, 65, { &P22041,&P6437,&P16179,&P18092 }};	/*Useragent - Vyrso/2.2.7 (Linux; U; Android 3.2; en-AU; GT-P6810 Build/HTJ85B)*/
const Device UA13242 = {S50019, 156, { &P12144,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-; A6277 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 Sprint APA6277KT*/
const Device UA25966 = {S50020, 168, { &P12144,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update2 (freshhh); en-us; A6277 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 Sprint APA6277KT*/
const Device UA30342 = {S50021, 156, { &P12144,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-fresh; en-us; A6277 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 Sprint APA6277KT*/
const Device UA44022 = {S50022, 158, { &P12144,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A6277 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 Sprint APA6277KT*/
const Device UA2431808 = {S50023, 89, { &P18088,&P22637,&P17373,&P18092 }};	/*Useragent - Opera/9.80 (SpreadTrum; Opera Mini/6.5.28991/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2364882 = {S50024, 140, { &P20289,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA55988 = {S50025, 97, { &P14726,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; LG; LG-C900B)*/
const Device UA56803 = {S50026, 96, { &P14726,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; LG; LG-C900)*/
const Device UA67650 = {S50027, 97, { &P14726,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; LG; LG-C900k)*/
const Device UA2539961 = {S50028, 134, { &P15364,&P16110,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/25.0.1364.97) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22*/
const Device UA2567989 = {S50029, 98, { &P15364,&P16110,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/25.0.1364.172)*/
const Device UA2568840 = {S50030, 91, { &P15364,&P16110,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; chromeframe/25.0.1364.172)*/
const Device UA2570552 = {S50031, 136, { &P15364,&P16110,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/25.0.1364.172) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2571084 = {S50032, 136, { &P15364,&P16110,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/25.0.1364.152) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22*/
const Device UA2600299 = {S50033, 177, { &P15364,&P16110,&P21918,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; chromeframe/25.0.1364.172)*/
const Device UA2606270 = {S50034, 97, { &P15364,&P16110,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/25.0.1364.172)*/
const Device UA2857136 = {S50035, 129, { &P15364,&P16110,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; chromeframe/25.0.1364.172) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA346833 = {S50036, 113, { &P15364,&P7133,&P22453,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.2 Chrome/15.0.900.2 Safari/535.2*/
const Device UA346892 = {S50037, 120, { &P15364,&P7133,&P22453,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.2 Chrome/15.0.900.2 Safari/535.2*/
const Device UA1995008 = {S50038, 132, { &P22041,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6810 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2450958 = {S50039, 61, { &P22041,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P6810 Build/IMM76D)*/
const Device UA2615198 = {S50040, 139, { &P22041,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; lt-lt; GT-P6810 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2615736 = {S50041, 145, { &P22041,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6810 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA2620217 = {S50042, 94, { &P22041,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.apps.plus/X (Linux; U; Android 4.0.4; en_GB; GT-P6810; Build/IMM76D) (gzip)*/
const Device UA2621650 = {S50043, 89, { &P22041,&P18066,&P18062,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.2.8.1 (Linux; U; Android 4.0.4; en-AU; GT-P6810 Build/IMM76D.XXLPK)*/
const Device UA2625649 = {S50044, 88, { &P22041,&P18066,&P18062,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.2.10 (Linux; U; Android 4.0.4; zh-HK; GT-P6810 Build/IMM76D.ZSLP8)*/
const Device UA2631133 = {S50045, 130, { &P22041,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P6810 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2632318 = {S50046, 130, { &P22041,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6810 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2636213 = {S50047, 261, { &P22041,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GT-P6810 Build/IMM76D) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.4 Version/4.0 Mobile Safari/530.17 baidubrowser/061_1.8.4.2_diordna_257_0821/gnusmas_51_4.0.4_0186P-TG/Xc/42F751AF98F8EAD052B3E00D9F3F4F35%7C0/1*/
const Device UA2637545 = {S50048, 175, { &P22041,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6810 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 GoogleGoggles-Android/1.9; gzip*/
const Device UA1974530 = {S50049, 137, { &P18621,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E610 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1996918 = {S50050, 138, { &P18621,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612f Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2028912 = {S50051, 137, { &P18621,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2178021 = {S50052, 138, { &P18621,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E610v Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2235573 = {S50053, 138, { &P18621,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612g Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2981693 = {S50054, 138, { &P18621,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  LG-E612 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA232946 = {S50055, 139, { &P16349,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Sony Tablet S Build/THMAS10000) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA713608 = {S50056, 140, { &P16349,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; de-de; Sony Tablet S Build/THMASD10000) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1906542 = {S50057, 68, { &P16349,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; Sony Tablet S Build/THMAS10000)*/
const Device UA2754345 = {S50058, 149, { &P18505,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MB886 Build/9.8.2Q-8_MB886_FFW-11) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754588 = {S50059, 150, { &P18505,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3066885 = {S50060, 137, { &P16356,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068270 = {S50061, 137, { &P16356,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-N7000 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081243 = {S50062, 138, { &P16356,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3082494 = {S50063, 138, { &P16356,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3082602 = {S50064, 138, { &P16356,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083653 = {S50065, 137, { &P16356,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T879 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126782 = {S50066, 135, { &P16356,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-05D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126794 = {S50067, 138, { &P16356,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000B Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA33066 = {S50068, 41, { &P12843,&P17484,&P3625,&P18092 }};	/*Useragent - sam-r380 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA90491 = {S50069, 137, { &P15869,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0; en-us; Xoom Build/HRI39) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA181087 = {S50070, 127, { &P15869,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Xoom Build/HWI69) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA183412 = {S50071, 127, { &P15869,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Xoom Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA304049 = {S50072, 125, { &P15869,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0; en-us; Xoom Build/HRI39) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402607 = {S50073, 127, { &P15869,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Xoom Build/HRI62) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402605 = {S50074, 126, { &P15869,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0; en-us; Xoom Build/HRI35F) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402622 = {S50075, 128, { &P15869,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Xoom Build/HWI63B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402643 = {S50076, 128, { &P15869,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Xoom Build/HWI61B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402646 = {S50077, 56, { &P15869,&P6545,&P4755,&P18092 }};	/*Useragent - Dalvik/1.5.0 (Linux; U; Android 3.0.1; Xoom Build/HWI69)*/
const Device UA402668 = {S50078, 128, { &P15869,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Xoom Build/HRI60B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402678 = {S50079, 125, { &P15869,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0; en-us; Xoom Build/HWI59) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402724 = {S50080, 127, { &P15869,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Xoom Build/HWI73) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402845 = {S50081, 131, { &P15869,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; fr-ca; Xoom Build/HONEYCOMB) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402766 = {S50082, 128, { &P15869,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Xoom Build/HRI82B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402943 = {S50083, 125, { &P15869,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; ru-; Xoom Build/HWI69) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA403040 = {S50084, 125, { &P15869,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0; en-us; Xoom Build/HRI53) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA403068 = {S50085, 126, { &P15869,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0; en-us; Xoom Build/HRI35C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2257219 = {S50086, 137, { &P15869,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0; en-us; Xoom Build/HRI39) AppleWebKit/525.10  (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2957087 = {S50087, 131, { &P24365,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SM-T211 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3021202 = {S50088, 60, { &P24365,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SM-T211 Build/JZO54K)*/
const Device UA202101 = {S50089, 63, { &P15364,&P17285,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; en) Presto/2.9.168 Version/11.50*/
const Device UA332950 = {S50090, 86, { &P15364,&P17285,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.9.168 Version/11.50*/
const Device UA333509 = {S50091, 86, { &P15364,&P17285,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.9.168 Version/11.50*/
const Device UA556453 = {S50092, 112, { &P15364,&P17285,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.6 (build 03278); MRSPUTNIK OW 2, 2, 0, 94; ru) Presto/2.9.168 Version/11.50*/
const Device UA666471 = {S50093, 86, { &P15364,&P17285,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.9.168 Version/11.50*/
const Device UA806991 = {S50094, 85, { &P15364,&P17285,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4157); ru) Presto/2.9.168 Version/11.50*/
const Device UA814760 = {S50095, 100, { &P15364,&P17285,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; en) Presto/2.9.168 Version/11.50,gzip(gfe) (via translate.google.com)*/
const Device UA1888245 = {S50096, 86, { &P15364,&P17285,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.4 (build 02647); ru) Presto/2.9.168 Version/11.50*/
const Device UA1890197 = {S50097, 66, { &P15364,&P17285,&P17366,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; en) Opera 11.50*/
const Device UA2041368 = {S50098, 82, { &P17595,&P18207,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; en) Presto/2.10.289 Version/12.02*/
const Device UA238802 = {S50099, 110, { &P15364,&P5690,&P18106,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/A.B (KHTML, like Gecko) Chrome/X.Y.Z.W Safari/A.B.*/
const Device UA2794720 = {S50100, 164, { &P21794,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9152 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3026058 = {S50101, 111, { &P18648,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia306/03.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA2361872 = {S50102, 75, { &P17595,&P18207,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2) Presto/2.12.388 Version/12.11*/
const Device UA2508419 = {S50103, 114, { &P14540,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-01/08.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2764705 = {S50104, 116, { &P14540,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-01.1/07.22; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2664479 = {S50105, 132, { &P22606,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5110 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA911654 = {S50106, 151, { &P18318,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF700T Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2376342 = {S50107, 152, { &P18318,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ASUS Transformer Pad TF700KL Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2476001 = {S50108, 139, { &P18318,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ASUS Pad TF700T Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3080137 = {S50109, 135, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MD7305 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080252 = {S50110, 133, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A600 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080303 = {S50111, 133, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; I110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081224 = {S50112, 137, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E455g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081386 = {S50113, 126, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MiSS Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3081783 = {S50114, 145, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; G526-L22 Build/HuaweiG526-L22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083270 = {S50115, 137, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E451g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093616 = {S50116, 143, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Camelio Family tablet Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3093824 = {S50117, 133, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; H100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094095 = {S50118, 133, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B943 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094185 = {S50119, 133, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; EG98 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094442 = {S50120, 128, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EGS102 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094725 = {S50121, 131, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EGP008 Build/EGP108FW1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094798 = {S50122, 132, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A20 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094980 = {S50123, 133, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; N810 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3095258 = {S50124, 130, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PLT8223G Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3108394 = {S50125, 133, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nivo Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3108805 = {S50126, 133, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; 9920 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3109249 = {S50127, 130, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PMP3970B Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3109375 = {S50128, 138, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Boston 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109376 = {S50129, 152, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Bouygues Telecom Bs 351 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109480 = {S50130, 127, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; y7053 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3109626 = {S50131, 128, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; WPT005 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3109928 = {S50132, 145, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; G526-L11 Build/HuaweiG526-L11) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110394 = {S50133, 124, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; G1 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3110788 = {S50134, 136, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P712 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110863 = {S50135, 139, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Positivo Ypy L700 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3110933 = {S50136, 128, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Thrive Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3111211 = {S50137, 137, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MTC 970H Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111410 = {S50138, 130, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SN97T41W Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3112544 = {S50139, 138, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XOLO Play Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3112705 = {S50140, 140, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XO Learning tablet Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3113030 = {S50141, 129, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TBDG773 Build/Zeki.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3113520 = {S50142, 133, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3113806 = {S50143, 127, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PRO7D Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3114300 = {S50144, 133, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ICON Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114302 = {S50145, 133, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; F600 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114357 = {S50146, 126, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; V72A Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3114563 = {S50147, 137, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E415f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114670 = {S50148, 132, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AML-MX REF Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3114853 = {S50149, 130, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; iQTab 3G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3114927 = {S50150, 134, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A9600 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126222 = {S50151, 127, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; iQTab Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126235 = {S50152, 138, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; smart_a17 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126261 = {S50153, 136, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGP351 Build/10.1.1.A.1.307) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126346 = {S50154, 131, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ACGM816KC Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126394 = {S50155, 128, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MD1005 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126471 = {S50156, 137, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S7392 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126506 = {S50157, 128, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MaxiIQ Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126507 = {S50158, 141, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Positivo Ypy L1050E Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126521 = {S50159, 133, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Z998 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126525 = {S50160, 136, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MTC 970 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126602 = {S50161, 133, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SZENIO 2500 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126605 = {S50162, 133, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Tablet-PC-4 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126622 = {S50163, 143, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Infinity_GN708 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126744 = {S50164, 131, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HS_7DTB14 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126785 = {S50165, 137, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E410B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126786 = {S50166, 150, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI P2-6070 Build/HuaweiP2-6070) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126813 = {S50167, 138, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XTAB-781+ Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126916 = {S50168, 133, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Joyz Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126979 = {S50169, 134, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Boost Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126980 = {S50170, 136, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ThinkPadTablet Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3127018 = {S50171, 145, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Casper_VIA_A6108 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127023 = {S50172, 126, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MT7A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3127317 = {S50173, 145, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MX Enjoy TV BOX Build/V1.03.04MX01_X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3127353 = {S50174, 133, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Z995 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127399 = {S50175, 145, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Forward_Infinity Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127904 = {S50176, 133, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; N800 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3128442 = {S50177, 148, { &P15364,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Huawei Y301A2 Build/HuaweiY301A2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3869 = {S50178, 77, { &P12125,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia6500s-1/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6500s-1*/
const Device UA3887 = {S50179, 64, { &P12125,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia6500s-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA25462 = {S50180, 62, { &P12125,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia6500s/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA894997 = {S50181, 47, { &P18671,&P17484,&P5001,&P18092 }};	/*Useragent - SCH-U365/1.0 NetFront/3.0.22.2.23 (GUI) MMP/2.0*/
const Device UA2408236 = {S50182, 76, { &P15364,&P5690,&P22367,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:15.3) Gecko/X Firefox/15.3.2 PaleMoon/15.3.2*/
const Device UA2430013 = {S50183, 83, { &P15364,&P5690,&P22367,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:15.3) Gecko/X Firefox/15.3.2 PaleMoon/15.3.2*/
const Device UA2384067 = {S50184, 138, { &P19894,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R830 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2230724 = {S50185, 178, { &P15803,&P5843,&P24672,&P18092 }};	/*Useragent - Nokia200/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia200) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile*/
const Device UA2753200 = {S50186, 138, { &P16356,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000B Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754161 = {S50187, 137, { &P16356,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T879 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754573 = {S50188, 137, { &P16356,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754684 = {S50189, 137, { &P16356,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-N7000 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769579 = {S50190, 138, { &P16356,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782605 = {S50191, 138, { &P16356,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798923 = {S50192, 138, { &P16356,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E160K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2800000 = {S50193, 135, { &P16356,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-05D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867662 = {S50194, 138, { &P16356,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SHV-E160S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879413 = {S50195, 131, { &P16356,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M480W Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2223029 = {S50196, 132, { &P15876,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P5100 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2261656 = {S50197, 132, { &P15876,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P5100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2626815 = {S50198, 133, { &P15876,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1;  GT-P5100 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2712356 = {S50199, 132, { &P15876,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T779 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3074001 = {S50200, 133, { &P15876,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2;  GT-P5100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA19716 = {S50201, 146, { &P13278,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-mx; E10a Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA44248 = {S50202, 158, { &P13278,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-mx; SonyEricssonE10a Build/2.0.A.0.504) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA44256 = {S50203, 147, { &P13278,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; E10a Build/2.0.1.A.0.47) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA44261 = {S50204, 147, { &P13278,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-co; E10a Build/2.0.2.A.0.24) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA169743 = {S50205, 146, { &P13278,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-mx; E10a Build/2.0.A.0.504) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019793 = {S50206, 68, { &P13278,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; E10a Build/2.1.1.A.0.6)*/
const Device UA142710 = {S50207, 47, { &P15364,&P17017,&P18068,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 2.1; Mac_PowerPC)*/
const Device UA2509424 = {S50208, 52, { &P15364,&P17017,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; PPC; Trident/3.0)*/
const Device UA2509913 = {S50209, 52, { &P15364,&P17017,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; PPC; Trident/4.0)*/
const Device UA2658926 = {S50210, 57, { &P15364,&P17017,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; PPC; SV1; Trident/1.0)*/
const Device UA2851073 = {S50211, 57, { &P15364,&P17017,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; PPC; SV1; Trident/5.0)*/
const Device UA23081 = {S50212, 155, { &P13706,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Nokia5235-1d/12.8.092; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA40984 = {S50213, 194, { &P13706,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5235-1b/50.8.001; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.25 Mobile Safari/533.4 3gpp-gba*/
const Device UA60292 = {S50214, 194, { &P13706,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5235-1d/50.8.001; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.25 Mobile Safari/533.4 3gpp-gba*/
const Device UA62992 = {S50215, 169, { &P13706,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5235-1b/20.8.007; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.3*/
const Device UA64817 = {S50216, 152, { &P13706,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Nokia5235/12.6.092; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA354943 = {S50217, 191, { &P13706,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5235/51.6.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 Mobile Safari/533.4 3gpp-gba*/
const Device UA2874067 = {S50218, 102, { &P15364,&P7127,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA24287 = {S50219, 170, { &P12329,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaN86-3/30.009;; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.1.19424*/
const Device UA30649 = {S50220, 169, { &P12329,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN86/30.101 (SymbianOS/9.3; U; Series60/3.2 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/425 (KHTML, like Gecko) Vesion/3.0 Browser NG/7.1.13036*/
const Device UA64665 = {S50221, 170, { &P12329,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaN86-1/30.009;; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.1.19424*/
const Device UA67345 = {S50222, 38, { &P12329,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN868MP/SymbianOS/9.1 Series60/3.0*/
const Device UA2203590 = {S50223, 114, { &P14635,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-00/08.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2457732 = {S50224, 113, { &P14635,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-00/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5*/
const Device UA2387659 = {S50225, 118, { &P14358,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE63) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile*/
const Device UA2389103 = {S50226, 40, { &P14358,&P5890,&P24673,&P18092 }};	/*Useragent - NokiaE63UCBrowser/8.7.1.234/28/352/UCWEB*/
const Device UA2433276 = {S50227, 92, { &P14358,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiae63-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2455840 = {S50228, 30, { &P14358,&P5890,&P24673,&P18092 }};	/*Useragent - NokiaE63/UCWEB8.7.0.187/28/999*/
const Device UA70654 = {S50229, 112, { &P13094,&P5629,&P15760,&P18092 }};	/*Useragent - Lenovo-A60/S100 Linux/2.6.35.7 Android/2.3.3 Release/04.19.2011 Browser/AppleWebKit533.1 Profile/ Configuration/*/
const Device UA652623 = {S50230, 138, { &P13094,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Lenovo A60 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA702862 = {S50231, 138, { &P13094,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Lenovo A60 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA703893 = {S50232, 126, { &P13094,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Lenovo A60 Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2019686 = {S50233, 62, { &P13094,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; Lenovo A60 Build/GRI40)*/
const Device UA2022828 = {S50234, 62, { &P13094,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Lenovo A60 Build/GRJ90)*/
const Device UA2024526 = {S50235, 126, { &P13094,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A60 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2024668 = {S50236, 131, { &P13094,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A60 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2033721 = {S50237, 131, { &P13094,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Lenovo A60 Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2168680 = {S50238, 147, { &P13094,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Lenovo A60+ Build/LenovoLePhone) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2347817 = {S50239, 142, { &P13094,&P5629,&P15760,&P18092 }};	/*Useragent - Lenovo A60+ Linux/2.6.35 Android/2.3.6 Release/05.04.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1*/
const Device UA2497263 = {S50240, 71, { &P13094,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; Lenovo A60+ Build/LenovoLePhone)*/
const Device UA2549942 = {S50241, 140, { &P13094,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A60+ Build/LenovoLePhone) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2772750 = {S50242, 165, { &P18645,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2773114 = {S50243, 157, { &P18645,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2773276 = {S50244, 157, { &P18645,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2773888 = {S50245, 158, { &P18645,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2773883 = {S50246, 158, { &P18645,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I747M Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2784888 = {S50247, 158, { &P18645,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799834 = {S50248, 157, { &P18645,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2887662 = {S50249, 158, { &P18645,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2897702 = {S50250, 158, { &P18645,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530C Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3121466 = {S50251, 158, { &P18645,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E210S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA380989 = {S50252, 109, { &P15364,&P6596,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1625.0.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1017.3 Safari/535.19*/
const Device UA399090 = {S50253, 111, { &P15364,&P6596,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1660.20.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.29 Safari/535.19*/
const Device UA401410 = {S50254, 111, { &P15364,&P6596,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1660.34.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.32 Safari/535.19*/
const Device UA402486 = {S50255, 111, { &P15364,&P6596,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1660.44.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.39 Safari/535.19*/
const Device UA412713 = {S50256, 111, { &P15364,&P6596,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1660.57.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.46 Safari/535.19*/
const Device UA427689 = {S50257, 111, { &P15364,&P6596,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1660.71.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.52 Safari/535.19*/
const Device UA457620 = {S50258, 113, { &P15364,&P6596,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1660.112.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19*/
const Device UA582365 = {S50259, 111, { &P15364,&P6596,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1494.0.2012) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.983.0 Safari/535.15*/
const Device UA601311 = {S50260, 111, { &P15364,&P6596,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1660.84.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.98 Safari/535.19*/
const Device UA602270 = {S50261, 112, { &P15364,&P6596,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1660.94.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.120 Safari/535.19*/
const Device UA608464 = {S50262, 113, { &P15364,&P6596,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1660.132.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.165 Safari/535.19*/
const Device UA614155 = {S50263, 113, { &P15364,&P6596,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1660.143.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA660091 = {S50264, 111, { &P15364,&P6596,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1424.0.2011) AppleWebKit/535.12 (KHTML, like Gecko) Chrome/18.0.966.0 Safari/535.12*/
const Device UA663122 = {S50265, 111, { &P15364,&P6596,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1533.0.2012) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.995.0 Safari/535.15*/
const Device UA664544 = {S50266, 112, { &P15364,&P6596,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1602.0.2012) AppleWebKit/535.18 (KHTML, like Gecko) Chrome/18.0.1012.1 Safari/535.18*/
const Device UA690551 = {S50267, 111, { &P15364,&P6596,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1494.0.2011) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.983.0 Safari/535.15*/
const Device UA747322 = {S50268, 112, { &P15364,&P6596,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1563.0.2012) AppleWebKit/535.16 (KHTML, like Gecko) Chrome/18.0.1003.0 Safari/535.16*/
const Device UA754407 = {S50269, 112, { &P15364,&P6596,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1660.98.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.140 Safari/535.19*/
const Device UA1943219 = {S50270, 111, { &P15364,&P6596,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1465.0.2011) AppleWebKit/535.14 (KHTML, like Gecko) Chrome/18.0.973.0 Safari/535.14*/
const Device UA2160073 = {S50271, 115, { &P15364,&P6596,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1660.99.2012) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.145 Safari/535.19*/
const Device UA2028753 = {S50272, 137, { &P19911,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E970 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA903036 = {S50273, 131, { &P20709,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID1042 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2827482 = {S50274, 60, { &P20709,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; MID1042 Build/IML74K)*/
const Device UA2566746 = {S50275, 138, { &P16356,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567125 = {S50276, 131, { &P16356,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583965 = {S50277, 138, { &P16356,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7000 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2584862 = {S50278, 139, { &P16356,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2600972 = {S50279, 132, { &P16356,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M480W Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2603859 = {S50280, 139, { &P16356,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667172 = {S50281, 139, { &P16356,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2937054 = {S50282, 139, { &P16356,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000B Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA55198 = {S50283, 73, { &P14297,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S3350/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2511376 = {S50284, 145, { &P18640,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT901 Build/9.8.2Q-50_SLS-7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2670560 = {S50285, 146, { &P18640,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2676771 = {S50286, 68, { &P18640,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13)*/
const Device UA2942870 = {S50287, 146, { &P18640,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT901 Build/9.8.2Q-50_SLS-14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955263 = {S50288, 75, { &P18640,&P18111,&P18112,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android 4.1.2;motorola XT901 Build/9.8.2Q-50_SLS-13)*/
const Device UA2755109 = {S50289, 145, { &P18125,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3067658 = {S50290, 137, { &P18582,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT28h Build/6.1.E.3.7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067659 = {S50291, 140, { &P18582,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT28at Build/6.1.C.1.111) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083659 = {S50292, 137, { &P18582,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT28i Build/6.1.E.3.7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3064661 = {S50293, 139, { &P18736,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA79875 = {S50294, 81, { &P15364,&P17285,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0) AppleWebKit (KHTML, like Gecko) Chrome/0*/
const Device UA149375 = {S50295, 117, { &P15364,&P17285,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.3.154.9 Safari/525.19*/
const Device UA408129 = {S50296, 118, { &P15364,&P17285,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.4.154.29 Safari/525.19*/
const Device UA408934 = {S50297, 117, { &P15364,&P17285,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.2.152.0 Safari/525.19*/
const Device UA1889335 = {S50298, 115, { &P15364,&P17285,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; de) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.2.149.27 Safari/525.13*/
const Device UA1910699 = {S50299, 96, { &P15364,&P17285,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/0.0.0.0 Safari/534.24*/
const Device UA2139886 = {S50300, 115, { &P15364,&P17285,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; de) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.4.154.29 Safari/525.19*/
const Device UA2259824 = {S50301, 113, { &P15364,&P17285,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/0.0.0.0 Safari/533.3*/
const Device UA3096536 = {S50302, 142, { &P12280,&P18432,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A551 Safari/8536.25*/
const Device UA3096543 = {S50303, 140, { &P12280,&P18432,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A403 Safari/8536.25*/
const Device UA3096548 = {S50304, 142, { &P12280,&P18432,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A525 Safari/8536.25*/
const Device UA3096576 = {S50305, 142, { &P12280,&P18432,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A523 Safari/8536.25*/
const Device UA3096833 = {S50306, 140, { &P12280,&P18432,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A405 Safari/8536.25*/
const Device UA1996445 = {S50307, 129, { &P17926,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2510070 = {S50308, 156, { &P15228,&P5043,&P24676,&P18092 }};	/*Useragent - NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-03) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA98 = {S50309, 151, { &P13554,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 NokiaN79-3/11.049; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA20489 = {S50310, 149, { &P13554,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN79/SymbianOS/9.3 Series60/3.2 Release/32.001; Mozilla/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML,like Gecko) Safari/413*/
const Device UA64192 = {S50311, 151, { &P13554,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 NokiaN79-1/32.001; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA2754241 = {S50312, 141, { &P16362,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710VMUB Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755060 = {S50313, 140, { &P16362,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710BST Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755113 = {S50314, 137, { &P16362,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA12329 = {S50315, 149, { &P14753,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 NokiaN96-3/3.00; Profile/MIDP-2.1 Configuration/CLDC-1.1;) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA53885 = {S50316, 149, { &P14753,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN96/SymbianOS/9.3 Series60/3.2 Release/30.033; Mozilla/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML,like Gecko) Safari/413*/
const Device UA64734 = {S50317, 149, { &P14753,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 NokiaN96-1/3.00; Profile/MIDP-2.1 Configuration/CLDC-1.1;) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA67054 = {S50318, 9, { &P14753,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia N96*/
const Device UA33200 = {S50319, 144, { &P17968,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 BilgiSisPhone (Linux; U; Android ; en-us; Generic Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90026 = {S50320, 92, { &P17968,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 0.5; en-us) AppleWebKit/522+ (KHTML, like Gecko) Safari/419.3*/
const Device UA2028419 = {S50321, 138, { &P17968,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android unknown; zh-tw; VITA V8 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031846 = {S50322, 92, { &P17968,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 0.5; en-us) AppleWebKit/522  (KHTML, like Gecko) Safari/419.3*/
const Device UA2230753 = {S50323, 57, { &P17968,&P17979,&P20428,&P18092 }};	/*Useragent - Player/LG Player 1.0 for Android(stagefright alternative)*/
const Device UA2327908 = {S50324, 22, { &P17968,&P17979,&P20428,&P18092 }};	/*Useragent - LG Player 1.0; Android*/
const Device UA2517923 = {S50325, 20, { &P17968,&P17979,&P20428,&P18092 }};	/*Useragent - Android Application:*/
const Device UA2528625 = {S50326, 35, { &P17968,&P17979,&P20428,&P18092 }};	/*Useragent - Android ClemBrowser/1.0 Clempad_CPT*/
const Device UA2660841 = {S50327, 153, { &P17968,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 5.3.4(Android 5.1.1); en-us; iPhone 4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2664405 = {S50328, 153, { &P17968,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 5.3.3(Android 5.1.1); en-us; iPhone 4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2666123 = {S50329, 153, { &P17968,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 5.3.9(Android 5.1.1); en-us; iPhone_4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2709396 = {S50330, 153, { &P17968,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 5.4.4(Android 5.1.1); en-us; iPhone_4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2727114 = {S50331, 16, { &P17968,&P17979,&P20428,&P18092 }};	/*Useragent - Android-Wifi/0.1*/
const Device UA2733153 = {S50332, 153, { &P17968,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 5.3.8(Android 5.1.1); en-us; iPhone_4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2736552 = {S50333, 153, { &P17968,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 5.4.8(Android 5.1.1); en-us; iPhone_4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2795290 = {S50334, 153, { &P17968,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 5.4.7(Android 5.1.1); en-us; iPhone_4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2864740 = {S50335, 153, { &P17968,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 5.5.0(Android 5.1.1); en-us; iPhone_4S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA716652 = {S50336, 156, { &P14225,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Desire HD Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2713562 = {S50337, 146, { &P22685,&P18111,&P18112,&P18092 }};	/*Useragent - LG-E460 Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E460 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865720 = {S50338, 143, { &P22685,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; LG-E460/V10c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867266 = {S50339, 60, { &P22685,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E460 Build/JZO54K)*/
const Device UA2872497 = {S50340, 143, { &P22685,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-E460/V10d Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2943397 = {S50341, 143, { &P22685,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-E460/V10f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2963442 = {S50342, 153, { &P22685,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; LG-E460/E46010a-SFR-FR Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996911 = {S50343, 153, { &P22685,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-E460/E46010f-EUR-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996955 = {S50344, 160, { &P22685,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; LG-E460-Orange/E46010b-ORG-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997151 = {S50345, 135, { &P22685,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; et; LG-E460 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998421 = {S50346, 153, { &P22685,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-E460/E46010c-EUR-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998513 = {S50347, 139, { &P22685,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E460f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3010733 = {S50348, 153, { &P22685,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; LG-E460/E46010b-SFR-FR Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3048800 = {S50349, 152, { &P22685,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E460/E46010b-O2-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088770 = {S50350, 143, { &P22685,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; hu-hu; LG-E460/V10a Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3091501 = {S50351, 160, { &P22685,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ro-ro; LG-E460-Orange/E46010e-ORG-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258398 = {S50352, 115, { &P18557,&P20706,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia308/05.80; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.13.8*/
const Device UA3043852 = {S50353, 176, { &P18557,&P20706,&P18273,&P18092 }};	/*Useragent - Nokia308/2.0 (05.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Series40; Nokia308/05.80; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.13.8*/
const Device UA2964016 = {S50354, 72, { &P15364,&P18110,&P21710,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) KHTML/4.10.5 (like Gecko) Konqueror/4.10*/
const Device UA2965085 = {S50355, 101, { &P15364,&P18110,&P21710,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.21 (KHTML, like Gecko) konqueror/4.10.5 Safari/537.21*/
const Device UA2966821 = {S50356, 101, { &P15364,&P18110,&P21710,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) konqueror/4.10.5 Safari/534.34*/
const Device UA3015545 = {S50357, 58, { &P15364,&P18110,&P21710,&P18092 }};	/*Useragent - Mozilla/5.0 (X11) KHTML/4.10.5 (like Gecko) Konqueror/4.10*/
const Device UA3017910 = {S50358, 70, { &P15364,&P18110,&P21710,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) KHTML/4.10.5 (like Gecko) Konqueror/4.10*/
const Device UA3026783 = {S50359, 143, { &P12280,&P17146,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9A406 Safari/7534.48.3*/
const Device UA3028971 = {S50360, 143, { &P12280,&P17146,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9A405 Safari/7534.48.3*/
const Device UA3030405 = {S50361, 141, { &P12280,&P17146,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9A334 Safari/7534.48.3*/
const Device UA232893 = {S50362, 95, { &P18073,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; T9295)*/
const Device UA2195623 = {S50363, 133, { &P12280,&P18432,&P22898,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mercury/7.0 Mobile/10A403 Safari/8536.25*/
const Device UA1980043 = {S50364, 128, { &P18733,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M532 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3042530 = {S50365, 57, { &P18733,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; M532 Build/IML74K)*/
const Device UA2671102 = {S50366, 143, { &P18505,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2672528 = {S50367, 149, { &P18505,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MB886 Build/9.8.2Q-8_MB886_FFW-11) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676577 = {S50368, 150, { &P18505,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2507980 = {S50369, 156, { &P14826,&P5843,&P24676,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac1-01) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA899646 = {S50370, 82, { &P17595,&P17297,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.7.4; U; ru) Presto/2.10.289 Version/12.00*/
const Device UA2201252 = {S50371, 82, { &P17595,&P17297,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5; U; en) Presto/2.10.289 Version/12.01*/
const Device UA2674418 = {S50372, 130, { &P15876,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T779 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676189 = {S50373, 130, { &P15876,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P5100 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676229 = {S50374, 130, { &P15876,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P5100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA126653 = {S50375, 34, { &P15364,&P7127,&P18405,&P18092 }};	/*Useragent - Opera/9.25 (Windows NT 5.0; U; en)*/
const Device UA168765 = {S50376, 34, { &P15364,&P7127,&P18405,&P18092 }};	/*Useragent - Opera/9.23 (Windows NT 5.0; U; en)*/
const Device UA174926 = {S50377, 34, { &P15364,&P7127,&P18405,&P18092 }};	/*Useragent - Opera/9.22 (Windows NT 5.0; U; en)*/
const Device UA301599 = {S50378, 34, { &P15364,&P7127,&P18405,&P18092 }};	/*Useragent - Opera/9.21 (Windows NT 5.0; U; de)*/
const Device UA332983 = {S50379, 34, { &P15364,&P7127,&P18405,&P18092 }};	/*Useragent - Opera/9.26 (Windows NT 5.0; U; fr)*/
const Device UA683795 = {S50380, 34, { &P15364,&P7127,&P18405,&P18092 }};	/*Useragent - Opera/9.27 (Windows NT 5.0; U; en)*/
const Device UA2269746 = {S50381, 34, { &P15364,&P7127,&P18405,&P18092 }};	/*Useragent - Opera/9.20 (Windows NT 5.0; U; en)*/
const Device UA2275922 = {S50382, 34, { &P15364,&P7127,&P18405,&P18092 }};	/*Useragent - Opera/9.24 (Windows NT 5.0; U; ru)*/
const Device UA2536944 = {S50383, 144, { &P21020,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; Micromax A116 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2701815 = {S50384, 66, { &P21020,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Micromax A116 Build/JZO54K)*/
const Device UA3102232 = {S50385, 142, { &P21020,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; Micromax A116 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2675322 = {S50386, 137, { &P16292,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Galaxy X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676345 = {S50387, 141, { &P16292,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Galaxy Nexus Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676479 = {S50388, 141, { &P16292,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676641 = {S50389, 134, { &P16292,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2677294 = {S50390, 139, { &P16292,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; Galaxy Nexus Build/JRN84D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709746 = {S50391, 134, { &P16292,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Galaxy Nexus Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA30720 = {S50392, 127, { &P17595,&P17295,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0*/
const Device UA167590 = {S50393, 146, { &P17595,&P17295,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PowerPC Mac OS X 10_5_8; en-US) AppleWebKit/531.21.8+(KHTML, like Gecko, Safari/528.16) Version/5.10.3 OmniWeb/622.14.0*/
const Device UA328925 = {S50394, 144, { &P17595,&P17295,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0*/
const Device UA486268 = {S50395, 127, { &P17595,&P17295,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-US) AppleWebKit/528.16 (KHTML, like Gecko, Safari/528.16) OmniWeb/v622.8.0*/
const Device UA650564 = {S50396, 148, { &P17595,&P17295,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PowerPC Mac OS X 10_5_8; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0*/
const Device UA2166872 = {S50397, 148, { &P17595,&P17295,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0*/
const Device UA634546 = {S50398, 105, { &P17595,&P17296,&P22505,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en; rv:1.9.2.24) Gecko/X Camino/2.1 (like Firefox/3.6.24)*/
const Device UA2347275 = {S50399, 105, { &P17595,&P17296,&P22505,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en; rv:1.9.2.28) Gecko/X Camino/2.1.2 (like Firefox/13.0)*/
const Device UA2168359 = {S50400, 142, { &P18267,&P5043,&P24668,&P18092 }};	/*Useragent - NokiaX2-02/2.0 (11.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-02) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2402581 = {S50401, 31, { &P11929,&P5129,&P24673,&P18092 }};	/*Useragent - Nokia5230/UCWEB8.7.0.187/50/999*/
const Device UA2406344 = {S50402, 119, { &P11929,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5230) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2425884 = {S50403, 41, { &P11929,&P5129,&P24673,&P18092 }};	/*Useragent - Nokia5230UCBrowser/8.7.1.234/50/444/UCWEB*/
const Device UA2505417 = {S50404, 101, { &P11929,&P5129,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5230) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA635992 = {S50405, 136, { &P15973,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-AS855 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1945532 = {S50406, 60, { &P15973,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-AS855 Build/GRJ22)*/
const Device UA372489 = {S50407, 30, { &P18254,&P5690,&P18245,&P18091 }};	/*Useragent - SuperBot/4.7.0.74 (Windows XP)*/
const Device UA613368 = {S50408, 65, { &P18254,&P5690,&P18245,&P18091 }};	/*Useragent - Mozilla/4.0 (Add Catalog; Windows NT 5.1; infobot.4942@yahoo.com)*/
const Device UA915812 = {S50409, 30, { &P18254,&P5690,&P18245,&P18091 }};	/*Useragent - SuperBot/4.4.0.60 (Windows XP)*/
const Device UA2682289 = {S50410, 91, { &P18254,&P5690,&P18245,&P18091 }};	/*Useragent - WebAlta Crawler/1.3.12 (http://www.webalta.ru/bot.html) (Windows; U; Windows NT 5.1; ru-RU)*/
const Device UA2783642 = {S50411, 130, { &P18254,&P5690,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.6) Gecko/X Firefox/2.0.0.6  - WebCrawler http://cognitiveseo.com/bot.html*/
const Device UA3057938 = {S50412, 96, { &P18254,&P5690,&P18245,&P18091 }};	/*Useragent - yacybot (x86 Windows XP 5.1; java 1.6.0_17; Europe/de) http://yacy.net/bot.html;c2VvLWNyYXdsZXI=*/
const Device UA3069364 = {S50413, 91, { &P18254,&P5690,&P18245,&P18091 }};	/*Useragent - WebAlta Crawler/1.3.11 (http://www.webalta.ru/bot.html) (Windows; U; Windows NT 5.1; ru-RU)*/
const Device UA3078222 = {S50414, 97, { &P18254,&P5690,&P18245,&P18091 }};	/*Useragent - yacybot (x86 Windows XP 5.1; java 1.6.0_19; America/en) http://yacy.net/bot.html;c2VvLWNyYXdsZXI=*/
const Device UA3091929 = {S50415, 98, { &P18254,&P5690,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; x86 Windows XP 5.1; java 1.7.0_25; America/en) http://yacy.net/bot.html*/
const Device UA702738 = {S50416, 80, { &P15364,&P16110,&P22340,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:11.0) Gecko/X Firefox/11.0 CometBird/11.0*/
const Device UA2810393 = {S50417, 83, { &P15364,&P16110,&P22340,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:11.0) Gecko/X Firefox/11.0 CometBrowser/11.0*/
const Device UA2527533 = {S50418, 137, { &P19911,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E970 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2974492 = {S50419, 141, { &P18462,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021236 = {S50420, 141, { &P18462,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022835 = {S50421, 150, { &P18462,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X dual sim Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023917 = {S50422, 151, { &P18462,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire XC dual sim Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA8914 = {S50423, 178, { &P13363,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; it-it; Garmin-Asus A10 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17; A10-V5.0.67-user-20101018*/
const Device UA538008 = {S50424, 169, { &P13363,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; Garmin-Asus A10 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17; A10-V5.0.71-user-X*/
const Device UA705173 = {S50425, 73, { &P13363,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Garmin-Asus A10 Build/ERE27)*/
const Device UA3065249 = {S50426, 151, { &P12605,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094926 = {S50427, 144, { &P12605,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA101287 = {S50428, 69, { &P18088,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5)*/
const Device UA131712 = {S50429, 73, { &P18088,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5.3.5)*/
const Device UA2227792 = {S50430, 142, { &P14635,&P5843,&P24668,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (08.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac3-00) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2471719 = {S50431, 140, { &P16044,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; CnM TouchPad 9DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2489173 = {S50432, 143, { &P16044,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; CnM TouchPad 10.1DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2764509 = {S50433, 72, { &P16044,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; CnM TouchPad 10.1DC Build/JRO03H)*/
const Device UA2792307 = {S50434, 69, { &P16044,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; CnM TouchPad 9DC Build/JRO03H)*/
const Device UA2851307 = {S50435, 69, { &P16044,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; CnM TouchPad 7DC Build/JRO03H)*/
const Device UA2988361 = {S50436, 140, { &P16044,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; CnM TouchPad 7DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2489756 = {S50437, 125, { &P12280,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Safari Version/5.1*/
const Device UA2490137 = {S50438, 125, { &P12280,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 Safari Version/5.1*/
const Device UA2511146 = {S50439, 125, { &P12280,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Safari Version/5.1*/
const Device UA2512589 = {S50440, 125, { &P12280,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari Version/5.1*/
const Device UA2512860 = {S50441, 127, { &P12280,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145 Safari Version/5.1*/
const Device UA2533036 = {S50442, 127, { &P12280,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari Version/5.1*/
const Device UA2566191 = {S50443, 128, { &P12280,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B146 Safari/7536.26*/
const Device UA2585576 = {S50444, 127, { &P12280,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari Version/5.1*/
const Device UA2710578 = {S50445, 128, { &P12280,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B329 Safari/7536.26*/
const Device UA2713962 = {S50446, 127, { &P12280,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari Version/5.1*/
const Device UA2990732 = {S50447, 137, { &P12280,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/10B329 Safari/7534.48.3*/
const Device UA2997062 = {S50448, 128, { &P12280,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B350 Safari/7536.26*/
const Device UA3044714 = {S50449, 126, { &P12280,&P19826,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B144 Safari/7536.26*/
const Device UA192836 = {S50450, 138, { &P17172,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Transformer TF101 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA640489 = {S50451, 145, { &P17172,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA656699 = {S50452, 67, { &P17172,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; Transformer TF101 Build/HMJ37)*/
const Device UA2031186 = {S50453, 139, { &P17172,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; de-de; Transformer TF101 Build/HMJ37) AppleWebKit/534.13+ (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA33103 = {S50454, 138, { &P15865,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Status Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA630806 = {S50455, 62, { &P15865,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Status Build/GRI40)*/
const Device UA786005 = {S50456, 138, { &P15865,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Status Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1956042 = {S50457, 62, { &P15865,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HTC Status Build/GRJ90)*/
const Device UA132926 = {S50458, 7, { &P17968,&P17979,&P18589,&P18092 }};	/*Useragent - Android*/
const Device UA2753859 = {S50459, 137, { &P18283,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754939 = {S50460, 135, { &P18283,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-06D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755061 = {S50461, 137, { &P18283,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I535 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2756745 = {S50462, 137, { &P18283,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782624 = {S50463, 138, { &P18283,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300T Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2800284 = {S50464, 137, { &P18283,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-l9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867776 = {S50465, 138, { &P18283,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999V Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868700 = {S50466, 134, { &P18283,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2741734 = {S50467, 139, { &P17172,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2769281 = {S50468, 133, { &P17172,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Transformer Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2873530 = {S50469, 127, { &P17172,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TF101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2974275 = {S50470, 130, { &P15876,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P5100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2975073 = {S50471, 130, { &P15876,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T779 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2975736 = {S50472, 130, { &P15876,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P5100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA894309 = {S50473, 132, { &P17687,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  MZ604 Build/I.7.1-34) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA903813 = {S50474, 131, { &P17687,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MZ604 Build/I.7.1-34) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1960532 = {S50475, 131, { &P17687,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ604 Build/I.7.1-45) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2784861 = {S50476, 136, { &P20872,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGP311 Build/10.1.C.0.370) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA181389 = {S50477, 69, { &P15767,&P17090,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8H7 (iPad; U; CPU OS 4_3_2 like Mac OS X; en_us)*/
const Device UA182276 = {S50478, 69, { &P15767,&P17090,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8G4 (iPad; U; CPU OS 4_3_1 like Mac OS X; en_us)*/
const Device UA191956 = {S50479, 69, { &P15767,&P17090,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8J2 (iPad; U; CPU OS 4_3_3 like Mac OS X; en_us)*/
const Device UA223082 = {S50480, 69, { &P15767,&P17090,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8K2 (iPad; U; CPU OS 4_3_4 like Mac OS X; en_us)*/
const Device UA236015 = {S50481, 69, { &P15767,&P17090,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8L1 (iPad; U; CPU OS 4_3_5 like Mac OS X; en_us)*/
const Device UA311647 = {S50482, 69, { &P15767,&P17090,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8F191 (iPad; U; CPU OS 4_3 like Mac OS X; en_us)*/
const Device UA2160547 = {S50483, 164, { &P15767,&P17090,&P18215,&P18092 }};	/*Useragent - Mozilla/5.0 AppleCoreMedia/1.0.0.8J2 (iPad; U; CPU OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F190 Safari/6533.18.5*/
const Device UA2934356 = {S50484, 163, { &P15767,&P17090,&P18215,&P18092 }};	/*Useragent - Mozilla/5.0 AppleCoreMedia/1.0.0.8J2 (iPad; U; CPU OS 4_3like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/6533.18.5*/
const Device UA2434644 = {S50485, 75, { &P17595,&P17297,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5) Presto/2.12.388 Version/12.12*/
const Device UA361739 = {S50486, 142, { &P14908,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-P1010 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1982408 = {S50487, 66, { &P14908,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-P1010 Build/GINGERBREAD)*/
const Device UA2614285 = {S50488, 150, { &P14908,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-P1010 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.7 Mobile Safari/534.30*/
const Device UA2615054 = {S50489, 78, { &P14908,&P5629,&P15760,&P18092 }};	/*Useragent - Vyrso/2.1.2 (Linux; U; Android 2.3.6; en-US; GT-P1010 Build/GINGERBREAD.XXKPI)*/
const Device UA2615788 = {S50490, 78, { &P14908,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 2.3.6; en-US; GT-P1010 Build/GINGERBREAD.XXKPO)*/
const Device UA2616445 = {S50491, 78, { &P14908,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 2.3.6; cs-CZ; GT-P1010 Build/GINGERBREAD.XXKPG)*/
const Device UA2618515 = {S50492, 142, { &P14908,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT-P1010 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2619278 = {S50493, 136, { &P14908,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; GT-P6210 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2623329 = {S50494, 135, { &P14908,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-P1010 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2626135 = {S50495, 140, { &P14908,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-; GT-P1010 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2635557 = {S50496, 180, { &P14908,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-P1010 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 GoogleGoggles-Android/1.9; gzip*/
const Device UA204618 = {S50497, 135, { &P18620,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID7022 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458732 = {S50498, 59, { &P18620,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID7022 Build/GRI40)*/
const Device UA458988 = {S50499, 69, { &P18620,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.3 (Linux; U; Android 2.3.3; en_US; MID7022 Build/MID7022 X)*/
const Device UA2566539 = {S50500, 150, { &P18318,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700T Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2603299 = {S50501, 151, { &P18318,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700KL Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2435907 = {S50502, 157, { &P16356,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7000 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2509694 = {S50503, 157, { &P16356,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7000 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2534909 = {S50504, 156, { &P16356,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7000 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2699375 = {S50505, 164, { &P16356,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-T879 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2707661 = {S50506, 157, { &P16356,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7000 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258472 = {S50507, 137, { &P17595,&P18207,&P18629,&P18093 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2; facebookexternalhit)AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1309.0Safari/537.17*/
const Device UA2658655 = {S50508, 139, { &P22609,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S6312 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781151 = {S50509, 61, { &P22609,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S6312 Build/JZO54K)*/
const Device UA3102281 = {S50510, 137, { &P22609,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-S6312 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3035292 = {S50511, 139, { &P18564,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X+ Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3108618 = {S50512, 141, { &P18564,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC EVARE_UL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA2261306 = {S50513, 182, { &P14826,&P5843,&P24672,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (04.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac1-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile*/
const Device UA2492472 = {S50514, 139, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7105 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2531274 = {S50515, 139, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7105 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2537629 = {S50516, 138, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7105 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580510 = {S50517, 139, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-L900 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2599060 = {S50518, 138, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L900 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2602473 = {S50519, 138, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I317 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2701120 = {S50520, 138, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7100 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734232 = {S50521, 144, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Galaxy Note II Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2765812 = {S50522, 138, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T889 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768260 = {S50523, 139, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7105 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778786 = {S50524, 139, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7100 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2862319 = {S50525, 61, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-N7100 Build/JDQ39E)*/
const Device UA2865804 = {S50526, 139, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-N7100 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2866622 = {S50527, 139, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878928 = {S50528, 137, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; N7100+ Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2889424 = {S50529, 139, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892042 = {S50530, 146, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I317 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994033 = {S50531, 136, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; N7100 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994664 = {S50532, 61, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-N7105 Build/JDQ39E)*/
const Device UA2996139 = {S50533, 69, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SAMSUNG-SGH-I317 Build/JDQ39E)*/
const Device UA2998652 = {S50534, 139, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998860 = {S50535, 139, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T889 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3000723 = {S50536, 147, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I317 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3124425 = {S50537, 139, { &P18442,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L900 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA250911 = {S50538, 142, { &P22207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; NXM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA573590 = {S50539, 144, { &P22207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; it-it; DATAM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020096 = {S50540, 66, { &P22207,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.1; NXM803HC Build/GINGERBREAD)*/
const Device UA904513 = {S50541, 53, { &P18088,&P17017,&P24671,&P18092 }};	/*Useragent - IUC(U;iOS 5.0.1;Zh-cn;320*480;)/UCWEB8.5.0.163/42/800*/
const Device UA911490 = {S50542, 53, { &P18088,&P17017,&P24671,&P18092 }};	/*Useragent - IUC(U;iOS 5.1.1;Zh-cn;320*480;)/UCWEB8.5.1.178/42/997*/
const Device UA2230797 = {S50543, 114, { &P18088,&P17017,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; mstar) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2314468 = {S50544, 53, { &P18088,&P17017,&P24671,&P18092 }};	/*Useragent - IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB8.5.0.163/41/800*/
const Device UA2321841 = {S50545, 51, { &P18088,&P17017,&P24671,&P18092 }};	/*Useragent - IUC(U;iOS 4.1;Zh-cn;320*480;)/UCWEB8.5.0.163/41/800*/
const Device UA2382971 = {S50546, 113, { &P18088,&P17017,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; maui) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2383873 = {S50547, 114, { &P18088,&P17017,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; opera) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2400716 = {S50548, 113, { &P18088,&P17017,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; j2me) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2478051 = {S50549, 114, { &P18088,&P17017,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; spice) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2507243 = {S50550, 51, { &P18088,&P17017,&P24671,&P18092 }};	/*Useragent - IUC(U;iOS 4_1;en-us;320*480;)/UCWEB8.5.1.228/41/355*/
const Device UA2591602 = {S50551, 116, { &P18088,&P17017,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; generic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2594466 = {S50552, 110, { &P18088,&P17017,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; id; maui) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2693768 = {S50553, 116, { &P18088,&P17017,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; mozilla) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/405/UCWEB Mobile*/
const Device UA2746418 = {S50554, 121, { &P18088,&P17017,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; jbed-fastbcc) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2822004 = {S50555, 123, { &P18088,&P17017,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; mstar platform) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2889220 = {S50556, 121, { &P18088,&P17017,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; maui browser) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2973653 = {S50557, 137, { &P18442,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7105 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973911 = {S50558, 137, { &P18442,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974874 = {S50559, 145, { &P18442,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I317 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976097 = {S50560, 137, { &P18442,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T889 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976695 = {S50561, 137, { &P18442,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L900 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021994 = {S50562, 137, { &P18442,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I317 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022398 = {S50563, 135, { &P18442,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Note II Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022886 = {S50564, 142, { &P18442,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Note II Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023111 = {S50565, 137, { &P18442,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R950 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024353 = {S50566, 130, { &P18442,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3795 = {S50567, 151, { &P16072,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 NokiaN78-5/11.043; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA19900 = {S50568, 149, { &P16072,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN78/SymbianOS/9.3 Series60/3.2 Release/30.014; Mozilla/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML,like Gecko) Safari/413*/
const Device UA64592 = {S50569, 151, { &P16072,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 NokiaN78-1/12.046; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA502882 = {S50570, 38, { &P16072,&P5890,&P3439,&P18092 }};	/*Useragent - (C)NokiaN78/SymbianOS/9.1 Series60/3.0*/
const Device UA2566522 = {S50571, 141, { &P18223,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667402 = {S50572, 134, { &P18223,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2668478 = {S50573, 138, { &P18223,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xperia P Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2974603 = {S50574, 138, { &P19902,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA642892 = {S50575, 151, { &P19908,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; SAMSUNG-SGH-I927R Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020581 = {S50576, 75, { &P19908,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SAMSUNG-SGH-I927R Build/GINGERBREAD)*/
const Device UA2427590 = {S50577, 143, { &P21119,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SoftwinerEvb Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2707231 = {S50578, 141, { &P21119,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; SoftwinerEvb Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2728860 = {S50579, 65, { &P21119,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SoftwinerEvb Build/JRO03C)*/
const Device UA2851944 = {S50580, 136, { &P21119,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SoftwinerEvb Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3025919 = {S50581, 101, { &P15364,&P7127,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA220273 = {S50582, 122, { &P18017,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; K1 Build/HMJ37) AppleWebKit/534.13(KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA640941 = {S50583, 136, { &P18017,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; K1 Build/K1_A301_03_03_X_US) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA651042 = {S50584, 136, { &P18017,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; K1 Build/K1_A301_04_04_X_US) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA756073 = {S50585, 136, { &P18017,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; K1 Build/K1_A301_02_02_X_CA) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA794743 = {S50586, 136, { &P18017,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; K1 Build/K1_A301_02_02_X_US) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1894445 = {S50587, 136, { &P18017,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; K1 Build/K1_A301_01_01_X_US) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1906316 = {S50588, 136, { &P18017,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; K1 Build/K1_A301_03_02_X_CN) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2021764 = {S50589, 136, { &P18017,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; es-es; K1 Build/K1_A301_01_01_X_SG) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2022791 = {S50590, 65, { &P18017,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; K1 Build/K1_A301_02_02_X_CA)*/
const Device UA2025708 = {S50591, 136, { &P18017,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; K1 Build/K1_A301_03_03_X_UK) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2566486 = {S50592, 137, { &P15575,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566901 = {S50593, 138, { &P15575,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100G Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567401 = {S50594, 138, { &P15575,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2584092 = {S50595, 130, { &P15575,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667701 = {S50596, 139, { &P15575,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100G Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA251253 = {S50597, 71, { &P17595,&P17297,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.11C74 (Macintosh; U; Intel Mac OS X 10_7_2; en_us)*/
const Device UA398524 = {S50598, 71, { &P17595,&P17297,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.11D50 (Macintosh; U; Intel Mac OS X 10_7_3; en_us)*/
const Device UA398585 = {S50599, 72, { &P17595,&P17297,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.11D50b (Macintosh; U; Intel Mac OS X 10_7_3; en_us)*/
const Device UA630479 = {S50600, 72, { &P17595,&P17297,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.11D50d (Macintosh; U; Intel Mac OS X 10_7_3; en_us)*/
const Device UA634493 = {S50601, 71, { &P17595,&P17297,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.11E53 (Macintosh; U; Intel Mac OS X 10_7_4; en_us)*/
const Device UA640986 = {S50602, 73, { &P17595,&P17297,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.11C2002 (Macintosh; U; Intel Mac OS X 10_7_2; en_us)*/
const Device UA668209 = {S50603, 73, { &P17595,&P17297,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.11D2001 (Macintosh; U; Intel Mac OS X 10_7_3; en_us)*/
const Device UA1989389 = {S50604, 73, { &P17595,&P17297,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.11E2620 (Macintosh; U; Intel Mac OS X 10_7_4; en_us)*/
const Device UA2161420 = {S50605, 73, { &P17595,&P17297,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.11E2705 (Macintosh; U; Intel Mac OS X 10_7_4; en_us)*/
const Device UA2164629 = {S50606, 73, { &P17595,&P17297,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.11E2617 (Macintosh; U; Intel Mac OS X 10_7_4; en_us)*/
const Device UA2195613 = {S50607, 71, { &P17595,&P17297,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.11G63 (Macintosh; U; Intel Mac OS X 10_7_5; en_us)*/
const Device UA2205972 = {S50608, 71, { &P17595,&P17297,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.11G56 (Macintosh; U; Intel Mac OS X 10_7_5; en_us)*/
const Device UA2237419 = {S50609, 72, { &P17595,&P17297,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.11G63b (Macintosh; U; Intel Mac OS X 10_7_5; en_us)*/
const Device UA2421069 = {S50610, 71, { &P17595,&P17297,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.11G63 (Macintosh; U; Intel Mac OS X 10_7_5; de_de)*/
const Device UA2455381 = {S50611, 71, { &P17595,&P17297,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.11G63 (Macintosh; U; Intel Mac OS X 10_7_5; fr_fr)*/
const Device UA912426 = {S50612, 65, { &P17595,&P18207,&P18333,&P18092 }};	/*Useragent - iTunes/10.6.3 (Macintosh; Intel Mac OS X 10.8) AppleWebKit/536.25*/
const Device UA2042324 = {S50613, 65, { &P17595,&P18207,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Macintosh; Intel Mac OS X 10.8.1) AppleWebKit/536.25*/
const Device UA2203505 = {S50614, 68, { &P17595,&P18207,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Macintosh; Intel Mac OS X 10.8.2) AppleWebKit/536.26.14*/
const Device UA2420882 = {S50615, 53, { &P17595,&P18207,&P18333,&P18092 }};	/*Useragent - iTunes/11.0 (Macintosh; OS X 10.8) AppleWebKit/536.25*/
const Device UA2428905 = {S50616, 60, { &P17595,&P18207,&P18333,&P18092 }};	/*Useragent - iTunes/11.0.1 (Macintosh; OS X 10.8.2) AppleWebKit/536.26.14*/
const Device UA2566147 = {S50617, 60, { &P17595,&P18207,&P18333,&P18092 }};	/*Useragent - iTunes/11.0.2 (Macintosh; OS X 10.8.3) AppleWebKit/536.28.10*/
const Device UA2784016 = {S50618, 59, { &P17595,&P18207,&P18333,&P18092 }};	/*Useragent - iTunes/11.0.4 (Macintosh; OS X 10.8.4) AppleWebKit/536.30.1*/
const Device UA3096182 = {S50619, 57, { &P17595,&P18207,&P18333,&P18092 }};	/*Useragent - iTunes/11.1 (Macintosh; OS X 10.8.5) AppleWebKit/536.30.1*/
const Device UA2457383 = {S50620, 136, { &P18442,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580216 = {S50621, 139, { &P18442,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2581892 = {S50622, 156, { &P18442,&P18066,&P18062,&P18092 }};	/*Useragent - GT-N7100 Linux/3.0.13 Android/4.1.1 Release/02.01.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2659123 = {S50623, 146, { &P18442,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N9550/N7100 PAD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2762385 = {S50624, 139, { &P18442,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; GT-N7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957582 = {S50625, 143, { &P18442,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.5; en-us; N7100 Build/MocorDroid4.0.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA16091 = {S50626, 77, { &P11729,&P5026,&P3410,&P18092 }};	/*Useragent - BlackBerry8100/4.5.0.182 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102*/
const Device UA2567964 = {S50627, 74, { &P15364,&P5690,&P22369,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:19.0) Gecko/X Firefox/19.0 PaleMoon/19.0.2*/
const Device UA2604176 = {S50628, 81, { &P15364,&P5690,&P22369,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:19.0) Gecko/X Firefox/19.0 PaleMoon/19.0.2*/
const Device UA2680895 = {S50629, 97, { &P17595,&P17296,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:20.0) Gecko/X Firefox/20.0 AlexaToolbar/alxf-2.18*/
const Device UA250380 = {S50630, 135, { &P15364,&P17817,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/10.10 Chromium/14.0.835.202 Chrome/14.0.835.202 Safari/535.1*/
const Device UA557993 = {S50631, 133, { &P15364,&P17817,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/10.10 Chromium/14.0.808.0 Chrome/14.0.808.0 Safari/535.1*/
const Device UA686667 = {S50632, 143, { &P21033,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U1 Build/HCL ME Tablet U1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2396341 = {S50633, 141, { &P21033,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; U1 Build/HCL ME Tablet U1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2817407 = {S50634, 65, { &P21033,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; U1 Build/HCL ME Tablet U1)*/
const Device UA2754146 = {S50635, 97, { &P17595,&P18207,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:22.0) Gecko/X Firefox/22.0 AlexaToolbar/alxf-2.18*/
const Device UA2899126 = {S50636, 72, { &P17595,&P18207,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:22.0) Gecko/X Firefox/22*/
const Device UA3026055 = {S50637, 111, { &P19748,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia205/03.18; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3028253 = {S50638, 113, { &P19748,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia205.1/03.19; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3028260 = {S50639, 113, { &P19748,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia205.3/03.19; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3029726 = {S50640, 112, { &P19748,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2055/03.20; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3030555 = {S50641, 112, { &P19748,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2052/03.20; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3078873 = {S50642, 112, { &P19748,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2051/03.20; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3096866 = {S50643, 140, { &P17346,&P22882,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A465 Safari/8536.25*/
const Device UA3109471 = {S50644, 142, { &P17346,&P22882,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A4449d Safari/8536.25*/
const Device UA3115614 = {S50645, 142, { &P17346,&P22882,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/11A501 Safari/8536.25*/
const Device UA60915 = {S50646, 108, { &P11987,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-T749-MKB/T749UEJJ7 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA63845 = {S50647, 104, { &P11987,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-T749/T749UVIK1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA252108 = {S50648, 153, { &P17283,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB860 Build/4.5.91; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA782594 = {S50649, 150, { &P17283,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB860 Build/4.5.91; CM7.2 RC1 Base) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA872805 = {S50650, 154, { &P17283,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB860 Build/4.5.141; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2006258 = {S50651, 153, { &P17283,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MB860 Build/4.5.141; CM7 X [yyyymmdd]) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2710785 = {S50652, 132, { &P15767,&P17146,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9A334 Safari/7534.48.3*/
const Device UA2717593 = {S50653, 134, { &P15767,&P17146,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9A405 Safari/7534.48.3*/
const Device UA2754711 = {S50654, 130, { &P15932,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755519 = {S50655, 130, { &P15932,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2531197 = {S50656, 146, { &P21698,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Prestige 7L_4.1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2670336 = {S50657, 148, { &P21698,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Prestige 7L_4.1 Build/Prestige) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3045402 = {S50658, 68, { &P21698,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Prestige 7L_4.1 Build/JRO03C)*/
const Device UA2973467 = {S50659, 141, { &P16292,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Galaxy Nexus Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973488 = {S50660, 141, { &P16292,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021200 = {S50661, 137, { &P16292,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Galaxy X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021559 = {S50662, 139, { &P16292,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; Galaxy Nexus Build/JRN84D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA277588 = {S50663, 76, { &P18088,&P17017,&P17470,&P18092 }};	/*Useragent - jBrowser/J2ME Profile/MIDP-1.0 Configuration/CLDC-1.0 (Google WAP Proxy/1.0)*/
const Device UA2471636 = {S50664, 74, { &P18088,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; CRI/1.3; +http://www.creativeresearchindexer.com)*/
const Device UA347057 = {S50665, 140, { &P18600,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ja-jp; SC-03D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA660168 = {S50666, 138, { &P18600,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ja-; SC-03D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA663801 = {S50667, 64, { &P18600,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SC-03D Build/gingerbread)*/
const Device UA2423592 = {S50668, 186, { &P18600,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ja-jp; SC-03D Build/gingerbread) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 YJApp-ANDROID jp.co.yahoo.android.yjtop/1.7.2*/
const Device UA3065335 = {S50669, 138, { &P21627,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC first Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2187560 = {S50670, 148, { &P13452,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P1000 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2258000 = {S50671, 148, { &P13452,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P1000 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2455277 = {S50672, 151, { &P13452,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P1000 Build/JRO03R; CyanogenMod-Sapir) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2611364 = {S50673, 148, { &P13452,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P1000 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2614705 = {S50674, 149, { &P13452,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; GT-P1000L Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615528 = {S50675, 158, { &P13452,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P1000 Build/JRO03R; CyanogenMod-Sapir) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2615989 = {S50676, 149, { &P13452,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P1000L Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616462 = {S50677, 149, { &P13452,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; GT-P1000L Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616948 = {S50678, 149, { &P13452,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; GT-P1000N Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617160 = {S50679, 158, { &P13452,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P1000 Build/JZO54K; CyanogenMod-Sapir) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2619906 = {S50680, 149, { &P13452,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-P1000N Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2630583 = {S50681, 147, { &P13452,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-; GT-P1000N Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2630977 = {S50682, 149, { &P13452,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; GT-P1000N Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2030350 = {S50683, 146, { &P20007,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; XT886 Build/6.7.3-135_YIC-17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2362997 = {S50684, 146, { &P20007,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT886 Build/6.7.3-179_YIC-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2428115 = {S50685, 146, { &P20007,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-ca; XT886 Build/6.7.3-179_YIC-11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2838925 = {S50686, 139, { &P20007,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT886 Build/6.7.3-169) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2933812 = {S50687, 68, { &P20007,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; XT886 Build/6.7.3-135_YIC-17)*/
const Device UA2511080 = {S50688, 156, { &P18267,&P5043,&P24676,&P18092 }};	/*Useragent - NokiaX2-02/2.0 (11.79) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-02) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA90307 = {S50689, 145, { &P12562,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID2 GLOBAL Build/S273) AppleWebKit/525.10  (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA151890 = {S50690, 138, { &P12562,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID2 GLOBAL Build/S273) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA154993 = {S50691, 62, { &P12562,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; DROID2 GLOBAL Build/S273)*/
const Device UA181504 = {S50692, 137, { &P12562,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2; en-us; DROID2 GLOBAL Build/S273) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA644144 = {S50693, 142, { &P12562,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROID2 GLOBAL Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA816155 = {S50694, 137, { &P12562,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID2 GLOBALBuild/S273) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA880058 = {S50695, 137, { &P12562,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us;DROID2 GLOBAL Build/S273) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1875248 = {S50696, 137, { &P12562,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID2GLOBAL Build/S273) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1878147 = {S50697, 137, { &P12562,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; DROID2 GLOBAL Build/S273) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA2018446 = {S50698, 151, { &P12562,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROID2 GLOBAL Build/3.4.2_80_D2G_16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2367477 = {S50699, 140, { &P18716,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-S730M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747337 = {S50700, 62, { &P18716,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-S730M Build/IMM76D)*/
const Device UA2963628 = {S50701, 147, { &P18716,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-S730M-parrot Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA64624 = {S50702, 158, { &P11932,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia6210Navigator/05.16; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA67082 = {S50703, 19, { &P11932,&P5890,&P3439,&P18092 }};	/*Useragent - NOKIA6210 Navigator*/
const Device UA74984 = {S50704, 150, { &P11932,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia6210s/05.16 (SymbianOS/9.3; U; Series60/3.2 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA2432161 = {S50705, 88, { &P18095,&P18109,&P18526,&P18092 }};	/*Useragent - Opera/9.80 (Series 60; Opera Mini/6.7.29868/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2481186 = {S50706, 55, { &P15364,&P17350,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Win64; Trident/6.0)*/
const Device UA2502450 = {S50707, 55, { &P15364,&P17350,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Win32; Trident/6.0)*/
const Device UA2509366 = {S50708, 53, { &P15364,&P17350,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; 68K; Trident/5.0)*/
const Device UA2658792 = {S50709, 66, { &P15364,&P17350,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; PPC; Win64; IA64; Trident/5.0)*/
const Device UA2660231 = {S50710, 60, { &P15364,&P17350,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; PPC; WOW64; Trident/2.0)*/
const Device UA2791648 = {S50711, 58, { &P15364,&P17350,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; 68K; SV1; Trident/3.0)*/
const Device UA2791794 = {S50712, 63, { &P15364,&P17350,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; PPC; .NET CLR; Trident/1.0)*/
const Device UA2849345 = {S50713, 60, { &P15364,&P17350,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; 68K; WOW64; Trident/1.0)*/
const Device UA61527 = {S50714, 137, { &P17710,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; LG-P925G Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70061 = {S50715, 137, { &P17710,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; fr-ca; LG-P925g Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA674792 = {S50716, 61, { &P17710,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P925g Build/FRG83G)*/
const Device UA2958773 = {S50717, 130, { &P23975,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; B1-710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Safari/534.30*/
const Device UA3018849 = {S50718, 130, { &P23975,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; B1-710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3020643 = {S50719, 59, { &P23975,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; B1-710 Build/JZO54K)*/
const Device UA2204367 = {S50720, 114, { &P15786,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-05/08.69; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2382662 = {S50721, 116, { &P15786,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-05.1/08.69; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2555149 = {S50722, 113, { &P15786,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-05/08.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5*/
const Device UA2878428 = {S50723, 144, { &P23683,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Prism II Build/HuaweiU8686) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3013336 = {S50724, 66, { &P23683,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Prism II Build/HuaweiU8686)*/
const Device UA2460872 = {S50725, 139, { &P12814,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I777 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2536139 = {S50726, 139, { &P12814,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I777 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2562765 = {S50727, 138, { &P12814,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I777 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2704218 = {S50728, 60, { &P12814,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGH-I777 Build/JDQ39)*/
const Device UA2712302 = {S50729, 138, { &P12814,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-US; SGH-i777 Build/JOP40D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752158 = {S50730, 139, { &P12814,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I777 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2192580 = {S50731, 80, { &P18657,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-C3262/C3262DDLHA NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA757605 = {S50732, 131, { &P18027,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID7042 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2544172 = {S50733, 60, { &P18027,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; MID7042 Build/IML74K)*/
const Device UA2755110 = {S50734, 139, { &P18564,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X+ Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2771272 = {S50735, 141, { &P18564,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC EVARE_UL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2973654 = {S50736, 148, { &P15201,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975791 = {S50737, 145, { &P15201,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022825 = {S50738, 162, { &P15201,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G with Beats Audio Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023393 = {S50739, 148, { &P15201,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Z710e Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA165608 = {S50740, 35, { &P15364,&P18110,&P18105,&P18092 }};	/*Useragent - Opera/9.52 (X11; Linux i686; U; fr)*/
const Device UA2470970 = {S50741, 134, { &P15364,&P16110,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/24.0.1312.52) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17*/
const Device UA2489240 = {S50742, 90, { &P15364,&P16110,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; chromeframe/24.0.1312.57)*/
const Device UA2492004 = {S50743, 134, { &P15364,&P16110,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/24.0.1312.56) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17*/
const Device UA2492451 = {S50744, 97, { &P15364,&P16110,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/24.0.1312.57)*/
const Device UA2508468 = {S50745, 96, { &P15364,&P16110,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/24.0.1312.57)*/
const Device UA2512050 = {S50746, 134, { &P15364,&P16110,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/24.0.1312.57) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2512323 = {S50747, 176, { &P15364,&P16110,&P21915,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; chromeframe/24.0.1312.57)*/
const Device UA632069 = {S50748, 157, { &P16168,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; NookColor Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2408185 = {S50749, 154, { &P18466,&P18066,&P19804,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2510528 = {S50750, 112, { &P18558,&P20706,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia309/05.85; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2754582 = {S50751, 151, { &P12605,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769451 = {S50752, 144, { &P12605,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA62485 = {S50753, 113, { &P11945,&P17484,&P3636,&P18092 }};	/*Useragent - SAMSUNG-GT-S5233S/S5233SDDJJ1 SHP/VPP/R5 Jasmine/0.8 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA2754714 = {S50754, 130, { &P22037,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3113 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2783532 = {S50755, 130, { &P22037,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3113 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA869 = {S50756, 47, { &P13999,&P5222,&P4626,&P18092 }};	/*Useragent - HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; fr)*/
const Device UA4999 = {S50757, 47, { &P13999,&P5222,&P4626,&P18092 }};	/*Useragent - HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; de)*/
const Device UA16210 = {S50758, 47, { &P13999,&P5222,&P4626,&P18092 }};	/*Useragent - HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; ru)*/
const Device UA16211 = {S50759, 50, { &P13999,&P5222,&P4626,&P18092 }};	/*Useragent - HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; zh-tw)*/
const Device UA16218 = {S50760, 47, { &P13999,&P5222,&P4626,&P18092 }};	/*Useragent - HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; nl)*/
const Device UA16216 = {S50761, 50, { &P13999,&P5222,&P4626,&P18092 }};	/*Useragent - HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; es-ES)*/
const Device UA16219 = {S50762, 47, { &P13999,&P5222,&P4626,&P18092 }};	/*Useragent - HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; da)*/
const Device UA16220 = {S50763, 47, { &P13999,&P5222,&P4626,&P18092 }};	/*Useragent - HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; el)*/
const Device UA16222 = {S50764, 48, { &P13999,&P5222,&P4626,&P18092 }};	/*Useragent - HTC_HD2_T8585 Opera/9.70 (Windows NT 5.1; U; de)*/
const Device UA16224 = {S50765, 47, { &P13999,&P5222,&P4626,&P18092 }};	/*Useragent - HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; pt)*/
const Device UA62029 = {S50766, 47, { &P13999,&P5222,&P4626,&P18092 }};	/*Useragent - HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; sv)*/
const Device UA62996 = {S50767, 47, { &P13999,&P5222,&P4626,&P18092 }};	/*Useragent - HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; nb)*/
const Device UA63299 = {S50768, 47, { &P13999,&P5222,&P4626,&P18092 }};	/*Useragent - HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; it)*/
const Device UA63847 = {S50769, 47, { &P13999,&P5222,&P4626,&P18092 }};	/*Useragent - HTC_HD2_T8585 Opera/9.7 (Windows NT 5.1; U; en)*/
const Device UA30135 = {S50770, 118, { &P12051,&P17484,&P3636,&P18092 }};	/*Useragent - SAMSUNG-GT-S5600-ORANGE/S5600BVIH3 SHP/VPP/R5 Jasmine/0.8 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA2710019 = {S50771, 129, { &P22596,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2714667 = {S50772, 136, { &P22596,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2796120 = {S50773, 137, { &P22596,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970C Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2865796 = {S50774, 130, { &P22596,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970C Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2947874 = {S50775, 137, { &P22596,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970X Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3091775 = {S50776, 130, { &P22596,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970X Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2408423 = {S50777, 101, { &P15364,&P7127,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA49464 = {S50778, 133, { &P13034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SH80F Build/A8024) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA230101 = {S50779, 134, { &P13034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SH-12C Build/S5068) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA318489 = {S50780, 134, { &P13034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SH-12C Build/S5050) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA397168 = {S50781, 134, { &P13034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SH-12C Build/S7140) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA421825 = {S50782, 134, { &P13034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SH-12C Build/S9300) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623601 = {S50783, 134, { &P13034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SH-12C Build/S4040) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA797527 = {S50784, 133, { &P13034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; SH80F Build/S9104) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA817871 = {S50785, 133, { &P13034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; de-de; SH80F Build/SA060) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA851909 = {S50786, 58, { &P13034,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SH-12C Build/S7140)*/
const Device UA2020154 = {S50787, 133, { &P13034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SH80F Build/RA060) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020744 = {S50788, 133, { &P13034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; SH80F Build/R9102) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031883 = {S50789, 134, { &P13034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SH-12C Build/S7170) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2812616 = {S50790, 64, { &P13034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SH-12C Build/S5050)*/
const Device UA2502002 = {S50791, 133, { &P18475,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; GT-P7320T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2503674 = {S50792, 132, { &P18475,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nb-no; GT-P7320 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622710 = {S50793, 61, { &P18475,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7320 Build/IMM76D)*/
const Device UA2631010 = {S50794, 62, { &P18475,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7320T Build/IMM76D)*/
const Device UA2637890 = {S50795, 213, { &P18475,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; GT-P7320 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.4.1-100 (Android 4.0.4; samsung GT-P7320 Build/IMM76D.XXLPI)*/
const Device UA2944602 = {S50796, 97, { &P17595,&P17296,&P22171,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:23.0) Gecko/X Firefox/23.0 AlexaToolbar/alxf-2.18*/
const Device UA3083004 = {S50797, 142, { &P12280,&P18432,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A551 Safari/8536.25*/
const Device UA3083005 = {S50798, 142, { &P12280,&P18432,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A523 Safari/8536.25*/
const Device UA3083008 = {S50799, 140, { &P12280,&P18432,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A403 Safari/8536.25*/
const Device UA3083013 = {S50800, 140, { &P12280,&P18432,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A405 Safari/8536.25*/
const Device UA3083411 = {S50801, 142, { &P12280,&P18432,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A525 Safari/8536.25*/
const Device UA2676355 = {S50802, 138, { &P12968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26i Build/6.1.A.2.55) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2706709 = {S50803, 131, { &P12968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26i Build/6.1.A.2.45) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2669948 = {S50804, 142, { &P18462,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA6570 = {S50805, 72, { &P12139,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA6579 = {S50806, 93, { &P12139,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5310xpressmusic*/
const Device UA25770 = {S50807, 61, { &P12139,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia5310/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA54713 = {S50808, 66, { &P12139,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia5310_CMCC/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA54828 = {S50809, 77, { &P12139,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia5310XpressMusic_CMCC/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA54727 = {S50810, 80, { &P14971,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-B5722/B5722DDJA1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA613223 = {S50811, 90, { &P17976,&P18066,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.4; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00*/
const Device UA613252 = {S50812, 90, { &P17976,&P18066,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.3; Linux; Opera Tablet/ADR-X; U; de) Presto/2.10.254 Version/12.00*/
const Device UA2034129 = {S50813, 109, { &P17976,&P18066,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.3 + Den Eds Update 2; Linux; Opera Tablet/ADR-X; U; nl) Presto/2.10.254 Version/12.00*/
const Device UA93710 = {S50814, 62, { &P15364,&P17285,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.2.15 Version/10.10*/
const Device UA865966 = {S50815, 79, { &P15364,&P17285,&P18272,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; U; tr; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 10.10*/
const Device UA1852986 = {S50816, 66, { &P15364,&P17285,&P18272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.0; tr) Opera 10.10*/
const Device UA2369108 = {S50817, 34, { &P15364,&P17285,&P18272,&P18092 }};	/*Useragent - Opera/10.1 (Windows NT 6.0; U; en)*/
const Device UA845536 = {S50818, 138, { &P18662,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ISW13HT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2174463 = {S50819, 142, { &P18662,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; HTC_J_Z321e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2236467 = {S50820, 138, { &P18662,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; ISW13HT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2345644 = {S50821, 142, { &P18662,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; HTC J Z321e Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2348294 = {S50822, 64, { &P18662,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC J Z321e Build/IMM76D)*/
const Device UA2502897 = {S50823, 60, { &P18662,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ISW13HT Build/IMM76D)*/
const Device UA2684247 = {S50824, 60, { &P18662,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; ISW13HT Build/IML74K)*/
const Device UA2801261 = {S50825, 150, { &P22968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HUAWEI P6-U06 Build/HuaweiP6-U06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2801265 = {S50826, 143, { &P22968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; P6-U06 Build/HuaweiP6-U06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3017457 = {S50827, 72, { &P22968,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; HUAWEI P6-U06 Build/HuaweiP6-U06)*/
const Device UA3106339 = {S50828, 141, { &P22968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HUAWEI P6-U06 Build/HuaweiP6-U06) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3108230 = {S50829, 157, { &P22968,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; HUAWEI P6-U06-orange Build/HuaweiP6-U06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA43706 = {S50830, 155, { &P15206,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE-90-1/07.24.0.3; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA64873 = {S50831, 153, { &P15206,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE90-1/400.34.93 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA2957788 = {S50832, 164, { &P24039,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9190 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA29942 = {S50833, 145, { &P12280,&P6274,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5F136 Safari/525.20*/
const Device UA32026 = {S50834, 139, { &P12280,&P6274,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone 2_1 like Mac OS X; en) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile/5F136 Safari/525.20.1*/
const Device UA87380 = {S50835, 155, { &P12280,&P6274,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone Simulator; U; CPU iPhone OS 2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5F135 Safari/525.20*/
const Device UA120984 = {S50836, 131, { &P12280,&P6274,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5F136*/
const Device UA2597960 = {S50837, 154, { &P12280,&P6274,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 21 like Max OS X; en-us) AppleWebKit/525.18.1 (KHTML like Geko) BB2C/1.3.33 Version/3.1.1 Mobile/5F136 Safari/525.20*/
const Device UA2735695 = {S50838, 144, { &P12280,&P6274,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_1 like Mac OS X; es-US) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5F90 Safari/525.20*/
const Device UA56113 = {S50839, 128, { &P13654,&P17484,&P4732,&P18092 }};	/*Useragent - SAMSUNG-SGH-A687/A687UCJH1; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.16479; U; en-US) Opera 9.50*/
const Device UA251925 = {S50840, 145, { &P12841,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_PH39100/1.63.502.4; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA194361 = {S50841, 139, { &P17158,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; VS910 4G Build/VS910ZV4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA214316 = {S50842, 139, { &P17158,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; VS910 4G Build/VS910ZV6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA220042 = {S50843, 63, { &P17158,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; VS910 4G Build/VS910ZV6)*/
const Device UA601686 = {S50844, 139, { &P17158,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; VS910 4G Build/VS910V6A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018338 = {S50845, 139, { &P17158,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; VS910 4G Build/VS910ZV1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA50214 = {S50846, 139, { &P17943,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; calgary Build/DRC92) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA67683 = {S50847, 139, { &P17943,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; calgary Build/DRC92) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2605819 = {S50848, 154, { &P18465,&P18066,&P21708,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA31582 = {S50849, 104, { &P12440,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-T359/T359UVJI2 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA132729 = {S50850, 74, { &P15364,&P17350,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; PRTG Network Monitor (www.paessler.com); Windows)*/
const Device UA251244 = {S50851, 137, { &P15850,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; LG-LU6200 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA576244 = {S50852, 61, { &P15850,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; LG-LU6200 Build/GRJ90)*/
const Device UA2022691 = {S50853, 125, { &P15850,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LG-LU6200 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2034488 = {S50854, 130, { &P15850,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LG-LU6200 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA43679 = {S50855, 80, { &P13723,&P22283,&P4223,&P18092 }};	/*Useragent - Mozilla/4.0 (Brew MP 1.0.2; U; en-us; Sanyo; NetFront/3.5.1/AMB) Sprint SCP-6780*/
const Device UA61074 = {S50856, 78, { &P13723,&P22283,&P4223,&P18092 }};	/*Useragent - Mozilla/4.0 (Brew MP 1.0.2; U; en-us; Sanyo; NetFront/3.5.1/AMB) Boost SCP6780*/
const Device UA2200616 = {S50857, 75, { &P15364,&P5690,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:15.0; Avant TriCore) Gecko/X Firefox/15.0.1*/
const Device UA2235162 = {S50858, 73, { &P15364,&P5690,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:16.0; Avant TriCore) Gecko/X Firefox/16.0*/
const Device UA2406895 = {S50859, 73, { &P15364,&P5690,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:17.0; Avant TriCore) Gecko/X Firefox/17.0*/
const Device UA2535325 = {S50860, 73, { &P15364,&P5690,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:18.0; Avant TriCore) Gecko/X Firefox/18.0*/
const Device UA2567490 = {S50861, 73, { &P15364,&P5690,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:19.0; Avant TriCore) Gecko/X Firefox/19.0*/
const Device UA2709461 = {S50862, 73, { &P15364,&P5690,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:20.0; Avant TriCore) Gecko/X Firefox/20.0*/
const Device UA2886656 = {S50863, 73, { &P15364,&P5690,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:21.0; Avant TriCore) Gecko/X Firefox/21.0*/
const Device UA2968685 = {S50864, 73, { &P15364,&P5690,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:22.0; Avant TriCore) Gecko/X Firefox/22.0*/
const Device UA3035526 = {S50865, 73, { &P15364,&P5690,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:23.0; Avant TriCore) Gecko/X Firefox/23.0*/
const Device UA3064978 = {S50866, 127, { &P23203,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; A1-810 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA347546 = {S50867, 140, { &P12681,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Nexus S 4G Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA359467 = {S50868, 155, { &P12681,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Full Android on Crespo4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA359470 = {S50869, 141, { &P12681,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus S 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA359857 = {S50870, 140, { &P12681,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus S 4G Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA369447 = {S50871, 140, { &P12681,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S 4G Build/IMM26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA371234 = {S50872, 140, { &P12681,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus S 4G Build/IMM26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA454608 = {S50873, 141, { &P12681,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA624537 = {S50874, 63, { &P12681,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Nexus S 4G Build/IMM76D)*/
const Device UA663817 = {S50875, 145, { &P12681,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus S 4G Build/GWK74) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA723668 = {S50876, 141, { &P12681,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA725718 = {S50877, 141, { &P12681,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S 4G Build/IMM76B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA858214 = {S50878, 141, { &P12681,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S 4G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA869509 = {S50879, 140, { &P12681,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Crespo4g Build/IMLM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA903757 = {S50880, 141, { &P12681,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S 4G Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2022204 = {S50881, 63, { &P12681,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Nexus S 4G Build/IML74K)*/
const Device UA2030300 = {S50882, 139, { &P12681,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; Nexus S 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA650828 = {S50883, 164, { &P21102,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; LG-P690b Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2588362 = {S50884, 60, { &P21102,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-P690b Build/GRJ22)*/
const Device UA62767 = {S50885, 77, { &P12210,&P5026,&P3410,&P18092 }};	/*Useragent - BlackBerry8120/4.5.0.124 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100*/
const Device UA910458 = {S50886, 146, { &P15767,&P17146,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9A405 Safari/7534.48.3*/
const Device UA910613 = {S50887, 144, { &P15767,&P17146,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9A334 Safari/7534.48.3*/
const Device UA2508710 = {S50888, 140, { &P20529,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-us; Micromax A89 Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2561027 = {S50889, 134, { &P20529,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A89 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2764618 = {S50890, 56, { &P20529,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; A89 Build/IMM76D)*/
const Device UA633757 = {S50891, 192, { &P15845,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaC5-06/20.6.039; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.27 Mobile Safari/533.4 3gpp-gba*/
const Device UA2325980 = {S50892, 162, { &P15845,&P5129,&P3690,&P18092 }};	/*Useragent - NokiaC5-06/22.6.007 (SymbianOS/9.4; U; Series60/5.0 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Safari/525 3gpp-gba*/
const Device UA925922 = {S50893, 89, { &P22667,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Xbox)*/
const Device UA2239381 = {S50894, 154, { &P12280,&P18432,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3 TeamStream (iPhone4,1 276 2.2)*/
const Device UA2362780 = {S50895, 158, { &P12280,&P18432,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3 TeamStream (iPhone5,1 279 2.2.1)*/
const Device UA2405711 = {S50896, 158, { &P12280,&P18432,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 TeamStream (iPhone4,1 293 2.2.2)*/
const Device UA2482996 = {S50897, 156, { &P12280,&P18432,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Safari/7534.48.3 TeamStream (iPhone5,1 293 2.2.2)*/
const Device UA2488063 = {S50898, 158, { &P12280,&P18432,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari/7534.48.3 TeamStream (iPhone5,1 306 2.3.1)*/
const Device UA777355 = {S50899, 132, { &P18731,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ONDA MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1958344 = {S50900, 152, { &P18731,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ONDA_MID Build/IML74K; 768*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA1959082 = {S50901, 61, { &P18731,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; ONDA MID Build/IML74K)*/
const Device UA2025086 = {S50902, 130, { &P18731,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ONDA MID Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2337909 = {S50903, 132, { &P18731,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ONDA MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2385381 = {S50904, 143, { &P18731,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ONDA MID Build/ICS.g12refM1006.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2401057 = {S50905, 144, { &P18731,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ONDA MID Build/ICS.g12refM708A2.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2466531 = {S50906, 144, { &P18731,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ONDA MID Build/ICS.g12refM704A1.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2467354 = {S50907, 144, { &P18731,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ONDA MID Build/ICS.g12refM703A1.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2522960 = {S50908, 143, { &P18731,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ONDA MID Build/ICS.g12refM1005.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2526948 = {S50909, 147, { &P18731,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ONDA MID Build/ICS.g12refM806A1SPI.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2554599 = {S50910, 144, { &P18731,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ONDA MID Build/ICS.g12refM806A2.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2649385 = {S50911, 132, { &P18731,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ONDA_MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2812379 = {S50912, 132, { &P18731,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ONDA_MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2835776 = {S50913, 142, { &P18731,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; ONDA MID Build/ICS.g12refM805.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2982585 = {S50914, 144, { &P18731,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; ONDA MID Build/ICS.g12refM708A1.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2366703 = {S50915, 100, { &P20273,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; HTC6990LVW)*/
const Device UA3065132 = {S50916, 138, { &P23772,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083190 = {S50917, 138, { &P23772,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190L Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2568844 = {S50918, 144, { &P12280,&P17146,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9A405 Safari/7534.48.3*/
const Device UA2569821 = {S50919, 142, { &P12280,&P17146,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9A334 Safari/7534.48.3*/
const Device UA2570189 = {S50920, 144, { &P12280,&P17146,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9A406 Safari/7534.48.3*/
const Device UA2585293 = {S50921, 146, { &P18125,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2606843 = {S50922, 139, { &P18125,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2876602 = {S50923, 73, { &P15364,&P5690,&P24329,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:22.0) Gecko/X Firefox/22.0 SeaMonkey/2.19*/
const Device UA2944882 = {S50924, 80, { &P15364,&P5690,&P24329,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:22.0) Gecko/X Firefox/22.0 SeaMonkey/2.19*/
const Device UA2962876 = {S50925, 60, { &P15364,&P5690,&P24329,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:22.0) Gecko/X SeaMonkey/2.19*/
const Device UA1996408 = {S50926, 142, { &P20124,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ZTE Grand X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031567 = {S50927, 130, { &P20124,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ZTE U970 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2223098 = {S50928, 138, { &P20124,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4;zh-cn; ZTE V970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2421143 = {S50929, 130, { &P20124,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE V970 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2468460 = {S50930, 139, { &P20124,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4;es-es; ZTE V970M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2477386 = {S50931, 139, { &P20124,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; ZTE V970 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2726744 = {S50932, 61, { &P20124,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ZTE V970 Build/IMM76D)*/
const Device UA2779554 = {S50933, 141, { &P20124,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Grand X(M) Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2839162 = {S50934, 125, { &P20124,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE V970 Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA3006206 = {S50935, 142, { &P20124,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; bg-bg; Grand X Pro Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3053279 = {S50936, 138, { &P20124,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; Grand X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA63852 = {S50937, 104, { &P14498,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-A887/A887UCIJ1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA2708922 = {S50938, 97, { &P17595,&P17297,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:20.0) Gecko/X Firefox/20.0 AlexaToolbar/alxf-2.18*/
const Device UA2796198 = {S50939, 56, { &P15767,&P19826,&P22883,&P18092 }};	/*Useragent - Incredimail 1.2.1 rv:4897 (iPad; iPhone OS 6.1.3; en_US)*/
const Device UA2796242 = {S50940, 122, { &P15767,&P19826,&P22883,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Incredimail/1.2.1.4897*/
const Device UA2867272 = {S50941, 120, { &P15767,&P19826,&P22883,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Incredimail/1.2.1.4897*/
const Device UA3024247 = {S50942, 122, { &P15767,&P19826,&P22883,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Incredimail/1.2.4.5649*/
const Device UA63442 = {S50943, 177, { &P14359,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5228/21.1.004; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.5.2 3gpp-gba*/
const Device UA671246 = {S50944, 191, { &P14359,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Nokia5228/51.1.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.33 Mobile Safari/533.4 3gpp-gba*/
const Device UA220508 = {S50945, 143, { &P18044,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; INQ Cloud Touch Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA226746 = {S50946, 151, { &P18044,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; INQ Cloud Touch -parrot Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA706142 = {S50947, 144, { &P18044,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; INQ Cloud Touch Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA708032 = {S50948, 67, { &P18044,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; INQ Cloud Touch Build/FRG83)*/
const Device UA2027452 = {S50949, 68, { &P18044,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; INQ Cloud Touch Build/FRG83G)*/
const Device UA1994998 = {S50950, 132, { &P23773,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8010 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2516583 = {S50951, 165, { &P12972,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SAMSUNG-SGH-T989 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2537065 = {S50952, 164, { &P12972,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-T989 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2969579 = {S50953, 175, { &P12972,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-T989 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/hercules*/
const Device UA2431644 = {S50954, 162, { &P18233,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; ALCATEL_one_touch_995 Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2518471 = {S50955, 84, { &P18233,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ALCATEL_one_touch_995 Build/IceCreamSandwich)*/
const Device UA2695270 = {S50956, 163, { &P18233,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ALCATEL_one_touch_995S Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3117606 = {S50957, 163, { &P18233,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ALCATEL_one_touch_995A Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1997272 = {S50958, 141, { &P18222,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS12S Build/6.1.D.0.170) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257748 = {S50959, 140, { &P18222,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS12S Build/6.1.D.1.74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2511236 = {S50960, 152, { &P18222,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonyEricssonIS12S Build/6.1.D.1.91) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2511245 = {S50961, 140, { &P18222,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS12S Build/6.1.D.1.91) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3033398 = {S50962, 141, { &P18222,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS12S Build/6.1.D.1.103) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108471 = {S50963, 63, { &P18222,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; IS12S Build/6.1.D.1.103)*/
const Device UA2597910 = {S50964, 60, { &P18320,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; PadFone Build/JRO03L)*/
const Device UA2599372 = {S50965, 138, { &P18320,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PadFone Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659815 = {S50966, 131, { &P18320,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PadFone Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA427495 = {S50967, 56, { &P15767,&P18570,&P17470,&P18092 }};	/*Useragent - MobileRSSFree-iPad/3.1.4 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA440549 = {S50968, 52, { &P15767,&P18570,&P17470,&P18092 }};	/*Useragent - MobileRSS-iPad/3.1.4 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2206108 = {S50969, 49, { &P15767,&P18570,&P17470,&P18092 }};	/*Useragent - browseripad/32992 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2342931 = {S50970, 55, { &P15767,&P18570,&P17470,&P18092 }};	/*Useragent - CitrixReceiver-iPad/166 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2355874 = {S50971, 60, { &P15767,&P18570,&P17470,&P18092 }};	/*Useragent - Perfect%20Browser-iPad/6.0.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2383200 = {S50972, 68, { &P15767,&P18570,&P17470,&P18092 }};	/*Useragent - Dolphin%20Browser%20for%20iPad/5.4.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2441597 = {S50973, 54, { &P15767,&P18570,&P17470,&P18092 }};	/*Useragent - MSN%20for%20iPad/1.0.2 CFNetwork/548.0.4 Darwin/11.0.0*/
const Device UA2654116 = {S50974, 53, { &P15767,&P18570,&P17470,&P18092 }};	/*Useragent - YandexSearch-iPad/474 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2685131 = {S50975, 62, { &P15767,&P18570,&P17470,&P18092 }};	/*Useragent - WordsWithFriendsFreeiPad/4.12 CFNetwork/485.13.9 Darwin/11.0.0*/
const Device UA2797246 = {S50976, 93, { &P15364,&P16110,&P22774,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; SlimBrowser/7.00.035)*/
const Device UA2870119 = {S50977, 86, { &P15364,&P16110,&P22774,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; SlimBrowser/7.00.035)*/
const Device UA3088928 = {S50978, 108, { &P15364,&P16110,&P22774,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Avant Browser; SlimBrowser/7.00.035)*/
const Device UA2041418 = {S50979, 84, { &P15364,&P7133,&P20755,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X Firefox/15.0.1 SeaMonkey/2.12.1*/
const Device UA2341802 = {S50980, 67, { &P15364,&P7133,&P20755,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X SeaMonkey/2.12*/
const Device UA2434730 = {S50981, 137, { &P18468,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; IdeaTabA2109A Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2554936 = {S50982, 137, { &P22126,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Torque Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2766644 = {S50983, 59, { &P22126,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Torque Build/IMM76I)*/
const Device UA2671105 = {S50984, 137, { &P15575,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100G Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2672110 = {S50985, 129, { &P15575,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676245 = {S50986, 137, { &P15575,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676251 = {S50987, 136, { &P15575,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734800 = {S50988, 138, { &P15575,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100G Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673743 = {S50989, 139, { &P18447,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.1.140) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675469 = {S50990, 132, { &P18447,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.1.140) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2866145 = {S50991, 137, { &P18447,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xperia T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2617090 = {S50992, 132, { &P15985,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; GT-P6200 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617682 = {S50993, 61, { &P15985,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P6200 Build/JZO54K)*/
const Device UA2618257 = {S50994, 132, { &P15985,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6200 Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622375 = {S50995, 61, { &P15985,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P6200 Build/JRO03R)*/
const Device UA2622838 = {S50996, 118, { &P15985,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.1.2; en-GB; GT-P6200 Build/cm_p6200-userdebug 4.1.2 JZO54K eng.aorth.X.X test-keys)*/
const Device UA2628354 = {S50997, 132, { &P15985,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; GT-P6200 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2675311 = {S50998, 132, { &P15985,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P6200 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2783184 = {S50999, 133, { &P15985,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; GT-P6200L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3121168 = {S51000, 62, { &P15985,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P6200L Build/JZO54K)*/
const Device UA2550642 = {S51001, 135, { &P22746,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N9100 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2676629 = {S51002, 151, { &P12605,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2678565 = {S51003, 144, { &P12605,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2974023 = {S51004, 130, { &P15919,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974991 = {S51005, 130, { &P15919,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P3100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3006006 = {S51006, 159, { &P24481,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SAMSUNG GT-S7560/S7560XXAME9 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3009887 = {S51007, 139, { &P24481,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7560 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057563 = {S51008, 61, { &P24481,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-S7560 Build/IMM76I)*/
const Device UA3060023 = {S51009, 158, { &P24481,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-S7560-ORANGE/S7560XXAMG7 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3126407 = {S51010, 159, { &P24481,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; SAMSUNG GT-S7560/S7560XXAMGF Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA241708 = {S51011, 63, { &P15364,&P17285,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; en) Presto/2.9.168 Version/11.51*/
const Device UA243823 = {S51012, 86, { &P15364,&P17285,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03797); ru) Presto/2.9.168 Version/11.51*/
const Device UA244158 = {S51013, 100, { &P15364,&P17285,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.9.168 Version/11.51,gzip(gfe) (via translate.google.com)*/
const Device UA244738 = {S51014, 86, { &P15364,&P17285,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03796); ru) Presto/2.9.168 Version/11.51*/
const Device UA336700 = {S51015, 86, { &P15364,&P17285,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.5 (build 02842); ru) Presto/2.9.168 Version/11.51*/
const Device UA337969 = {S51016, 85, { &P15364,&P17285,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4598); ru) Presto/2.9.168 Version/11.51*/
const Device UA569649 = {S51017, 86, { &P15364,&P17285,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.6 (build 03278); ru) Presto/2.9.168 Version/11.51*/
const Device UA570767 = {S51018, 85, { &P15364,&P17285,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.8 (build 4664); ru) Presto/2.9.168 Version/11.51*/
const Device UA804392 = {S51019, 86, { &P15364,&P17285,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.4 (build 02647); ru) Presto/2.9.168 Version/11.51*/
const Device UA2297351 = {S51020, 86, { &P15364,&P17285,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; MRA 5.7 (build 03686); ru) Presto/2.9.168 Version/11.51*/
const Device UA2669264 = {S51021, 80, { &P15364,&P18140,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux x86_64; Edition Linux Mint) Presto/2.12.388 Version/12.15*/
const Device UA2488270 = {S51022, 95, { &P12193,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaC5-00.2) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2507006 = {S51023, 102, { &P12193,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac5-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2510521 = {S51024, 93, { &P12193,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaC5-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2697695 = {S51025, 104, { &P12193,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac5-00.2) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2536405 = {S51026, 153, { &P20192,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Orange Daytona Build/HuaweiG510-0200) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2582707 = {S51027, 155, { &P20192,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0200 Build/HuaweiG510-0200) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2659389 = {S51028, 155, { &P20192,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0100 Build/HuaweiG510-0100) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2778698 = {S51029, 155, { &P20192,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2863483 = {S51030, 155, { &P20192,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0251 Build/HuaweiG510-0251) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2462420 = {S51031, 139, { &P19777,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105P Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2599561 = {S51032, 138, { &P19777,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA313010 = {S51033, 143, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.7)*/
const Device UA514107 = {S51034, 149, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.6)*/
const Device UA539939 = {S51035, 149, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.7)*/
const Device UA561758 = {S51036, 154, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.7)*/
const Device UA628646 = {S51037, 143, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.9)*/
const Device UA658182 = {S51038, 144, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.10)*/
const Device UA680473 = {S51039, 154, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.9)*/
const Device UA723692 = {S51040, 149, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.9)*/
const Device UA1900545 = {S51041, 149, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.8)*/
const Device UA1996857 = {S51042, 144, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.11)*/
const Device UA2190117 = {S51043, 171, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.4; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.12)*/
const Device UA2205108 = {S51044, 163, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.14)*/
const Device UA2264635 = {S51045, 162, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.3)*/
const Device UA2297255 = {S51046, 172, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; Sleipnir/2.9.8)*/
const Device UA2409081 = {S51047, 184, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.2; Sleipnir/2.9.3)*/
const Device UA2409168 = {S51048, 149, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 2.0.50727; Sleipnir/2.9.2)*/
const Device UA2436017 = {S51049, 163, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.15)*/
const Device UA2495991 = {S51050, 91, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; InfoPath.1; Sleipnir/2.9.4)*/
const Device UA2541907 = {S51051, 174, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; InfoPath.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322; Sleipnir/2.9.2)*/
const Device UA2640495 = {S51052, 239, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB0.0; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 1.1.4322; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; OfficeLiveConnector.1.3; OfficeLivePatch.0.0; Sleipnir/2.9.6)*/
const Device UA2716047 = {S51053, 163, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.16)*/
const Device UA2939377 = {S51054, 144, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.17)*/
const Device UA3024133 = {S51055, 144, { &P15364,&P5690,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.9.18)*/
const Device UA2741721 = {S51056, 149, { &P18318,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF700T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2773347 = {S51057, 149, { &P18318,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ASUS Transformer Pad TF700T Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA38532 = {S51058, 146, { &P14106,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; GT-I9000T Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA500676 = {S51059, 68, { &P14106,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I9000T Build/ECLAIR)*/
const Device UA2425707 = {S51060, 75, { &P17595,&P18207,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2) Presto/2.12.388 Version/12.12*/
const Device UA2233821 = {S51061, 190, { &P18283,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, gb*/
const Device UA2424628 = {S51062, 190, { &P18283,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:38, us*/
const Device UA2676374 = {S51063, 199, { &P18283,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I535 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, -0400, us*/
const Device UA2676492 = {S51064, 199, { &P18283,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, +0100, gb*/
const Device UA3027025 = {S51065, 160, { &P18283,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-T999 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2tmo*/
const Device UA3028081 = {S51066, 160, { &P18283,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9300 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i9300*/
const Device UA3028324 = {S51067, 160, { &P18283,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I535 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2vzw*/
const Device UA3061503 = {S51068, 160, { &P18283,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-gb; GT-I9300 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i9300*/
const Device UA3062214 = {S51069, 160, { &P18283,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-T999 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2tmo*/
const Device UA3063526 = {S51070, 160, { &P18283,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I535 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2vzw*/
const Device UA3096096 = {S51071, 160, { &P18283,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I535 Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2vzw*/
const Device UA3110835 = {S51072, 160, { &P18283,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I535 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2vzw*/
const Device UA3111758 = {S51073, 160, { &P18283,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-T999 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2tmo*/
const Device UA3113747 = {S51074, 160, { &P18283,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9300 Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i9300*/
const Device UA3113750 = {S51075, 160, { &P18283,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-T999 Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2tmo*/
const Device UA3115531 = {S51076, 160, { &P18283,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-gb; GT-I9300 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i9300*/
const Device UA1993955 = {S51077, 135, { &P20753,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN7DG3ST-CP Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1996222 = {S51078, 133, { &P20753,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN7DG3-CP Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026108 = {S51079, 133, { &P20753,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; AN7DG3-LZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2413463 = {S51080, 130, { &P20753,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN7DG3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2559533 = {S51081, 131, { &P20753,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN7DG3B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2741714 = {S51082, 139, { &P20289,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755115 = {S51083, 139, { &P20289,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC6435LVW Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2703930 = {S51084, 139, { &P22584,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-ca; SGH-I337M Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2713720 = {S51085, 61, { &P22584,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGH-I337M Build/JDQ39)*/
const Device UA2956721 = {S51086, 137, { &P22584,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-; SGH-I337M Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA7987 = {S51087, 152, { &P13998,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT300 Build/SESGC_U3_00.35.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA18060 = {S51088, 163, { &P13998,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; XT300 Build/SESLA_U3_01.71.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA18058 = {S51089, 163, { &P13998,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; XT300 Build/SESLA_U3_01.71.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA33225 = {S51090, 163, { &P13998,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; XT300 Build/SESLA_U3_01.71.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA38361 = {S51091, 155, { &P13998,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT300 Build/SESGC_U3_10.10.0_RT) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA41525 = {S51092, 163, { &P13998,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; XT300 Build/SESLA_U3_01.72.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA41526 = {S51093, 163, { &P13998,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT300 Build/SESLA_U3_01.49.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA53645 = {S51094, 163, { &P13998,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; XT300 Build/SESLA_U3_01.44.5) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA53644 = {S51095, 163, { &P13998,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; XT300 Build/SESLA_U3_01.44.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA53646 = {S51096, 163, { &P13998,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; XT300 Build/SESLA_U3_01.72.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA53643 = {S51097, 163, { &P13998,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT300 Build/SESLA_U3_01.60.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA53642 = {S51098, 163, { &P13998,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-br; XT300 Build/SESLA_U3_01.44.4) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA55268 = {S51099, 167, { &P13998,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; XT300 Build/SESLA_U3_01.53.1_R01) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA78721 = {S51100, 163, { &P13998,&P5477,&P4861,&P18092 }};	/*Useragent - MOT-XT300/ Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; XT300 Build/SESLA_U3_01.71.3) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA90094 = {S51101, 152, { &P13998,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; XT300 Build/SESLA_U3_01.24.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1892780 = {S51102, 74, { &P13998,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.1-update1; XT300 Build/SESLA_U3_01.71.0)*/
const Device UA2022711 = {S51103, 155, { &P13998,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT300 Build/SESGC_U3_10.10.0_CU) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2029744 = {S51104, 138, { &P13998,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT300 Build/SESGC_U3_00.35.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA24407 = {S51105, 137, { &P13941,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; IDEOS S7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40798 = {S51106, 149, { &P13941,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-au; S7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 S7V100R001C13B119*/
const Device UA56385 = {S51107, 148, { &P13941,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; S7 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 S7V100R001C13B115*/
const Device UA70839 = {S51108, 136, { &P13941,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; IDEOS S7 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA641269 = {S51109, 61, { &P13941,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; IDEOS S7 Build/FRG83G)*/
const Device UA1935114 = {S51110, 146, { &P13941,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ru-ru; Oysters SmaKit S7 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA51924 = {S51111, 142, { &P16584,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; SGH-I896 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA421788 = {S51112, 66, { &P16584,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SGH-I896 Build/GINGERBREAD)*/
const Device UA2676018 = {S51113, 130, { &P15882,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P5110 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676243 = {S51114, 130, { &P15882,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P5110 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2353220 = {S51115, 190, { &P18645,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9305 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/135,1.9.6.135,2012-08-29 19:33, gb*/
const Device UA2367389 = {S51116, 190, { &P18645,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:38, us*/
const Device UA2505931 = {S51117, 198, { &P18645,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:38, us*/
const Device UA2674441 = {S51118, 200, { &P18645,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-I747M Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, -0400, ca*/
const Device UA2676994 = {S51119, 199, { &P18645,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, -0400, us*/
const Device UA2678424 = {S51120, 199, { &P18645,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9305 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, +0100, gb*/
const Device UA2860573 = {S51121, 200, { &P18645,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530U Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, -0500, us*/
const Device UA9881 = {S51122, 149, { &P13445,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN85/SymbianOS/9.3 Series60/3.2 Release/31.002; Mozilla/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML,like Gecko) Safari/413*/
const Device UA54171 = {S51123, 151, { &P13445,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 NokiaN85-3/11.047; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA64652 = {S51124, 151, { &P13445,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 NokiaN85-1/31.002; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA872551 = {S51125, 140, { &P18707,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; MD_LIFETAB_P9516 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2473580 = {S51126, 69, { &P18707,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; MD_LIFETAB_P9516 Build/IML74K)*/
const Device UA2716996 = {S51127, 138, { &P23776,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGP321 Build/10.1.1.A.1.253) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA8558 = {S51128, 111, { &P14972,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-GT-S3370C/1.0 SHP/VPP/R5 Dolfin/1.5 Qtv/5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-N*/
const Device UA24669 = {S51129, 111, { &P14972,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-GT-S3370L/1.0 SHP/VPP/R5 Dolfin/1.5 Qtv/5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-N*/
const Device UA53846 = {S51130, 115, { &P14972,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-GT-S3370B/S3370DO SHP/VPP/R5 Dolfin/1.5 Qtv/5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-N*/
const Device UA63112 = {S51131, 117, { &P14972,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-GT-S3370/S3370DDJE1 SHP/VPP/R5 Dolfin/1.5 Qtv/5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-N*/
const Device UA2705240 = {S51132, 138, { &P22596,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R970 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2709257 = {S51133, 60, { &P22596,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SCH-R970 Build/JDQ39)*/
const Device UA2794310 = {S51134, 139, { &P22596,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R970C Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2864250 = {S51135, 61, { &P22596,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SCH-R970C Build/JDQ39)*/
const Device UA2937659 = {S51136, 139, { &P22596,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R970X Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3020185 = {S51137, 61, { &P22596,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SCH-R970X Build/JDQ39)*/
const Device UA3115852 = {S51138, 89, { &P18115,&P5764,&P18549,&P18092 }};	/*Useragent - Opera/9.80 (BlackBerry; Opera Mini/4.5.33868/31.1475; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2974136 = {S51139, 128, { &P21769,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ME301T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2567448 = {S51140, 141, { &P20708,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XOLO A1000 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2961253 = {S51141, 138, { &P20708,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; IdeaTabA1000-F Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2997750 = {S51142, 138, { &P20708,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; IdeaTabA1000-T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3012787 = {S51143, 67, { &P20708,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; IdeaTabA1000-F Build/JZO54K)*/
const Device UA3022245 = {S51144, 138, { &P20708,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; IdeaTabA1000-G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3092521 = {S51145, 67, { &P20708,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; IdeaTabA1000-G Build/JZO54K)*/
const Device UA2670898 = {S51146, 138, { &P18125,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676433 = {S51147, 145, { &P18125,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3065379 = {S51148, 128, { &P23702,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ME173X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3127377 = {S51149, 127, { &P23702,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ME173X Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2738346 = {S51150, 149, { &P22749,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-04E Build/10.1.1.D.0.179) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2769751 = {S51151, 145, { &P22749,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-04E Build/10.1.1.D.0.179) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2895813 = {S51152, 148, { &P22749,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-04E Build/10.1.1.D.2.26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2965446 = {S51153, 148, { &P22749,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-04E Build/10.1.1.D.2.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3015653 = {S51154, 66, { &P22749,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SO-04E Build/10.1.1.D.2.31)*/
const Device UA3019728 = {S51155, 144, { &P22749,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-04E Build/10.1.1.D.2.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3124254 = {S51156, 144, { &P22749,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-04E Build/10.1.1.D.2.26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA105524 = {S51157, 47, { &P15364,&P6729,&P17384,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows 95)*/
const Device UA260735 = {S51158, 46, { &P15364,&P6729,&P17384,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.0; Windows 95)*/
const Device UA2791855 = {S51159, 54, { &P15364,&P6729,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; Win95; Trident/1.0)*/
const Device UA2004618 = {S51160, 139, { &P18630,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A800S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2023703 = {S51161, 61, { &P18630,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-A800S Build/IMM76I)*/
const Device UA2355530 = {S51162, 137, { &P18630,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A800S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2524968 = {S51163, 163, { &P18630,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A800S Build/A800S 216 V6.5 VH by atulaphan) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2526723 = {S51164, 160, { &P18630,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IM-A800S Build/A800S 221 v6.7 by atulaphan) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597518 = {S51165, 167, { &P18630,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A800S Build/PHIEN BAN GALAXY S A800S BY HANZOO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2650890 = {S51166, 160, { &P18630,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IM-A800S Build/A800S 216 V6.4 by atulaphan) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695812 = {S51167, 160, { &P18630,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IM-A800S Build/A800S 222 v6.8 by atulaphan) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2733276 = {S51168, 157, { &P18630,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A800S Build/VEGAVIET.COM ROM ICS V2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2793422 = {S51169, 157, { &P18630,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A800S Build/VEGAVIET.COM ROM ICS V1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2849317 = {S51170, 136, { &P18630,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A800S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3072499 = {S51171, 150, { &P18630,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A800S Build/www.tkvietnam.com) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3098741 = {S51172, 161, { &P18630,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IM-A800S Build/A800S 219 V6.6 by atulaphanI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103419 = {S51173, 144, { &P18630,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A800S Build/wwww.avio.vn) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2719026 = {S51174, 83, { &P18141,&P22711,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 2.12) AppleWebKit/536.26 (KHTML, like Gecko) Silk/3.2*/
const Device UA22268 = {S51175, 73, { &P14673,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-E2652/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64463 = {S51176, 74, { &P14673,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-E2652W/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA79130 = {S51177, 148, { &P14673,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-E2652/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1, SAMSUNG-GT-E2652/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2425943 = {S51178, 139, { &P18622,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A820L Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2485496 = {S51179, 138, { &P18622,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2528734 = {S51180, 61, { &P18622,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-A820L Build/IMM76I)*/
const Device UA2530211 = {S51181, 137, { &P18622,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A820L Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555935 = {S51182, 144, { &P18622,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/HOANGMINHHUU) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2557789 = {S51183, 144, { &P18622,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/Vegaviet.com) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2656143 = {S51184, 136, { &P18622,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/di?n) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2702273 = {S51185, 145, { &P18622,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IM-A820L Build/HOANGMINHHUU) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2733049 = {S51186, 179, { &P18622,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/Versions Rom Galaxy S.Production by Hanzoo1504) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734582 = {S51187, 183, { &P18622,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/Super Rom Optimus HD Pro. Production By Hanzoo1054) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747314 = {S51188, 154, { &P18622,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/MINHHAI-A820L-VEGAVIET) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2764316 = {S51189, 146, { &P18622,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/ChoiMobile.vn) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044594 = {S51190, 160, { &P18622,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/tungon_ blackberry@yahoo.com) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA3073000 = {S51191, 142, { &P18622,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/Tkvietnam) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867131 = {S51192, 127, { &P22394,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6603 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2880695 = {S51193, 140, { &P22394,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6603 Build/10.3.A.0.423) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2035010 = {S51194, 145, { &P18453,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT25i Build/9.0.D.0.235) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2035088 = {S51195, 145, { &P18453,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT25i Build/9.0.D.0.219) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2404020 = {S51196, 145, { &P18453,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT25i Build/9.0.D.0.484) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2430311 = {S51197, 146, { &P18453,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT25i Build/9.0.1.D.0.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2482319 = {S51198, 63, { &P18453,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT25i Build/9.0.D.0.484)*/
const Device UA2485485 = {S51199, 142, { &P18453,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; LT25i Build/9.0.1.D.0.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2845341 = {S51200, 145, { &P18453,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SonyLT25i Build/9.0.D.0.457) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2848511 = {S51201, 141, { &P18453,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; LT25i Build/9.0.D.0.484) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA63115 = {S51202, 79, { &P14478,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry8350i/5.0.0.1051 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/103*/
const Device UA458816 = {S51203, 69, { &P21077,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.3 (Linux; U; Android 2.3.3; en_US; MID8127 Build/MID8127 X)*/
const Device UA458830 = {S51204, 135, { &P21077,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID8127 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458934 = {S51205, 59, { &P21077,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID8127 Build/GRI40)*/
const Device UA50508 = {S51206, 145, { &P14090,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I5801 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA142975 = {S51207, 67, { &P14090,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I5801 Build/ECLAIR)*/
const Device UA655950 = {S51208, 131, { &P14090,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I5801 Build/ECLAIR) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2496875 = {S51209, 143, { &P14090,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; GT-I5801 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA156661 = {S51210, 136, { &P16954,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Dell Venue Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA180342 = {S51211, 139, { &P16954,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Dell Venue Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA716452 = {S51212, 60, { &P16954,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; Dell Venue Build/FRF91)*/
const Device UA2019330 = {S51213, 63, { &P16954,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; Dell Venue Build/FRG83G)*/
const Device UA2029759 = {S51214, 135, { &P19879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I915 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2427018 = {S51215, 132, { &P19879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I915 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2529892 = {S51216, 61, { &P19879,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-I915 Build/IMM76D)*/
const Device UA75720 = {S51217, 33, { &P15255,&P17017,&P3534,&P18092 }};	/*Useragent - HUAWEI-M635/001.00 WAP/OBIGO/Q05A*/
const Device UA2166866 = {S51218, 128, { &P19775,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2454916 = {S51219, 57, { &P19775,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; A210 Build/IMM76D)*/
const Device UA3117792 = {S51220, 143, { &P19775,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo_A2105 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2753781 = {S51221, 145, { &P15201,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754406 = {S51222, 148, { &P15201,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769434 = {S51223, 148, { &P15201,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Z710e Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799088 = {S51224, 162, { &P15201,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G with Beats Audio Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879252 = {S51225, 145, { &P15201,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754153 = {S51226, 141, { &P16292,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755256 = {S51227, 137, { &P16292,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Galaxy X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755343 = {S51228, 141, { &P16292,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Galaxy Nexus Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769766 = {S51229, 139, { &P16292,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; Galaxy Nexus Build/JRN84D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA626422 = {S51230, 130, { &P20948,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; LePanII Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1950616 = {S51231, 57, { &P20948,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/151 (Linux; U; Android 3.2.1; LePanII Build/HTK75)*/
const Device UA1996996 = {S51232, 163, { &P13064,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I727 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2014303 = {S51233, 163, { &P13064,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I727 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2232264 = {S51234, 163, { &P13064,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I727 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2237959 = {S51235, 163, { &P13064,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I727 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3065149 = {S51236, 132, { &P13644,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093979 = {S51237, 132, { &P13644,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; EVO Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2566939 = {S51238, 149, { &P18472,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT925 Build/7.7.1Q-144_VQL_S3-58) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567376 = {S51239, 159, { &P18472,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR HD Build/7.7.1Q-144_VQW_S334-25) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583128 = {S51240, 142, { &P18472,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT925 Build/7.7.1Q-144_VQL_S3-58) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583968 = {S51241, 152, { &P18472,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR HD Build/7.7.1Q-144_VQW_S334-25) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2986568 = {S51242, 75, { &P15364,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (Googlebot compatible; Dayviewbot/2.0; http://www.dayview.com/)*/
const Device UA3000041 = {S51243, 75, { &P15364,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1; http://www.semetrical.com/bot.html)*/
const Device UA3018428 = {S51244, 66, { &P15364,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot/2.1 (http://www.googlebot.com/bot.html);c2VvLWNyYXdsZXI=*/
const Device UA3026060 = {S51245, 79, { &P15364,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; GoogleApps script; +http://script.google.com/bot.html)*/
const Device UA3029738 = {S51246, 93, { &P15364,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html) (support@google.com)*/
const Device UA3029737 = {S51247, 91, { &P15364,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html) support@google.com*/
const Device UA3047673 = {S51248, 52, { &P15364,&P7480,&P18245,&P18089 }};	/*Useragent - * Googlebot/2.1 (+http://www.googlebot.com/bot.html)*/
const Device UA3097407 = {S51249, 38, { &P15364,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1*/
const Device UA3097420 = {S51250, 46, { &P15364,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot/2.1; +http://www.google.com/bot.html*/
const Device UA3102623 = {S51251, 58, { &P15364,&P7480,&P18245,&P18089 }};	/*Useragent - Mozilla/5.0 (compatible; Googlebot/2.1; +http://import.io)*/
const Device UA3130824 = {S51252, 47, { &P15364,&P7480,&P18245,&P18089 }};	/*Useragent - Googlebot-News (2.3.3, ruby 1.9.3 (2013-02-22))*/
const Device UA99633 = {S51253, 34, { &P15364,&P7133,&P18405,&P18092 }};	/*Useragent - Opera/9.24 (Windows NT 6.1; U; en)*/
const Device UA99981 = {S51254, 34, { &P15364,&P7133,&P18405,&P18092 }};	/*Useragent - Opera/9.26 (Windows NT 6.1; U; en)*/
const Device UA112187 = {S51255, 34, { &P15364,&P7133,&P18405,&P18092 }};	/*Useragent - Opera/9.27 (Windows NT 6.1; U; en)*/
const Device UA113030 = {S51256, 34, { &P15364,&P7133,&P18405,&P18092 }};	/*Useragent - Opera/9.25 (Windows NT 6.1; U; en)*/
const Device UA166696 = {S51257, 34, { &P15364,&P7133,&P18405,&P18092 }};	/*Useragent - Opera/9.20 (Windows NT 6.1; U; en)*/
const Device UA168268 = {S51258, 34, { &P15364,&P7133,&P18405,&P18092 }};	/*Useragent - Opera/9.23 (Windows NT 6.1; U; en)*/
const Device UA265957 = {S51259, 34, { &P15364,&P7133,&P18405,&P18092 }};	/*Useragent - Opera/9.21 (Windows NT 6.1; U; en)*/
const Device UA431186 = {S51260, 34, { &P15364,&P7133,&P18405,&P18092 }};	/*Useragent - Opera/9.22 (Windows NT 6.1; U; en)*/
const Device UA2360780 = {S51261, 117, { &P15364,&P5690,&P22463,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11*/
const Device UA2458075 = {S51262, 124, { &P15364,&P5690,&P22463,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11*/
const Device UA2695419 = {S51263, 117, { &P15364,&P5690,&P22463,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11*/
const Device UA1997067 = {S51264, 163, { &P12972,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-T989 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2016297 = {S51265, 163, { &P12972,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-T989 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2197348 = {S51266, 163, { &P12972,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-T989 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2233838 = {S51267, 167, { &P12972,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-T989 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA403386 = {S51268, 128, { &P18248,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA908718 = {S51269, 128, { &P18248,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2336885 = {S51270, 57, { &P18248,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; A700 Build/IMM76D)*/
const Device UA3043317 = {S51271, 129, { &P18248,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MA700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2199095 = {S51272, 146, { &P12972,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-T989 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2205806 = {S51273, 138, { &P12972,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T989 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2232116 = {S51274, 138, { &P12972,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2238345 = {S51275, 146, { &P12972,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T989 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2738647 = {S51276, 139, { &P12972,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989D Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2422061 = {S51277, 132, { &P22052,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P7501 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622194 = {S51278, 61, { &P22052,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7501 Build/IMM76D)*/
const Device UA2629362 = {S51279, 139, { &P22052,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P7501 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2630276 = {S51280, 130, { &P22052,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-; GT-P7501 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2714043 = {S51281, 151, { &P22052,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SAMSUNG GT-P7501/P7501XXLPD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2974605 = {S51282, 138, { &P21627,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC first Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA909772 = {S51283, 155, { &P12280,&P17146,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9A405 Safari/7534.48.3*/
const Device UA909781 = {S51284, 155, { &P12280,&P17146,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9A406 Safari/7534.48.3*/
const Device UA909971 = {S51285, 153, { &P12280,&P17146,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9A334 Safari/7534.48.3*/
const Device UA2175673 = {S51286, 145, { &P15854,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XOOM 2 ME Build/7.7.1-128_MZ607-14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257668 = {S51287, 144, { &P15854,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XOOM 2 ME Build/7.7.1-85_MZ607-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2933895 = {S51288, 74, { &P15854,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ607-14)*/
const Device UA2741603 = {S51289, 151, { &P15767,&P19826,&P22783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.2.1 (like Version/10B329 Mobile/6.1.3 Safari/8536.25)*/
const Device UA2782879 = {S51290, 147, { &P15767,&P19826,&P22783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.2.1 (like Version/10B141 Mobile/6.1 Safari/8536.25)*/
const Device UA2799507 = {S51291, 151, { &P15767,&P19826,&P22783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.2.1 (like Version/10B146 Mobile/6.1.2 Safari/8536.25)*/
const Device UA609349 = {S51292, 144, { &P15438,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  HTC Sensation Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA901689 = {S51293, 139, { &P15438,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC-Z710a Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA901844 = {S51294, 139, { &P15438,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Sensation Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA903048 = {S51295, 149, { &P15438,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710a Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA904014 = {S51296, 143, { &P15438,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA904351 = {S51297, 143, { &P15438,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA907294 = {S51298, 139, { &P15438,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Sensation Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2026128 = {S51299, 148, { &P15438,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2879219 = {S51300, 113, { &P23522,&P20706,&P24335,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia501/10.0.2; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.0.0.0.73*/
const Device UA3015567 = {S51301, 115, { &P23522,&P20706,&P24335,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia501s/10.0.12; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.0.0.0.75*/
const Device UA2039867 = {S51302, 73, { &P16329,&P18572,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PLAYSTATION 3 4.25) AppleWebKit/531.22.8 (KHTML, like Gecko)*/
const Device UA2402442 = {S51303, 32, { &P14033,&P5129,&P24673,&P18092 }};	/*Useragent - NokiaC5-03/UCWEB8.7.0.187/50/999*/
const Device UA2405923 = {S51304, 120, { &P14033,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaC5-03) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2428095 = {S51305, 42, { &P14033,&P5129,&P24673,&P18092 }};	/*Useragent - NokiaC5-03UCBrowser/8.7.1.234/50/444/UCWEB*/
const Device UA2507793 = {S51306, 159, { &P17835,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; NookTablet Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2600373 = {S51307, 175, { &P17835,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Barnes & Noble Nook Tablet Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2662986 = {S51308, 159, { &P17835,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; NookTablet Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666109 = {S51309, 158, { &P17835,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; NookTablet Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2668169 = {S51310, 174, { &P17835,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Barnes & Noble Nook Tablet Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2782341 = {S51311, 177, { &P17835,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Barnes & Noble Nook Tablet Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878731 = {S51312, 174, { &P17835,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-CA; Barnes & Noble Nook Tablet Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937191 = {S51313, 184, { &P17835,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; NookTablet Build/JDQ39E; CyanogenMod-acclaim_10.1_21APR2013_HD_SDC) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA201700 = {S51314, 187, { &P18088,&P17017,&P18373,&P18092 }};	/*Useragent - ASTRO36_TD/v3 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA2257159 = {S51315, 137, { &P19939,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; MOT-XT303 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2466294 = {S51316, 139, { &P19939,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT303 Build/07.17.03R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2553531 = {S51317, 139, { &P19939,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT303 Build/07.1E.00R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2777780 = {S51318, 139, { &P19939,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT303 Build/07.22.00R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2793098 = {S51319, 139, { &P19939,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT303 Build/07.17.04R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2837128 = {S51320, 63, { &P19939,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; XT303 Build/07.17.03R_S)*/
const Device UA2934081 = {S51321, 139, { &P19939,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT303 Build/07.1B.05R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2984721 = {S51322, 139, { &P19939,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT303 Build/07.1F.01R_S) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3065591 = {S51323, 137, { &P12972,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068195 = {S51324, 138, { &P12972,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093579 = {S51325, 145, { &P12972,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T989 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2607521 = {S51326, 167, { &P21792,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; en-us; SAMSUNG GT-I9200 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0.X Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2863171 = {S51327, 164, { &P21792,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG GT-I9200 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA50048 = {S51328, 73, { &P13153,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-E2220/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA924227 = {S51329, 149, { &P18465,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Amazon Kindle Fire Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033239 = {S51330, 71, { &P18465,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Amazon Kindle Fire Build/JRO03C)*/
const Device UA2044086 = {S51331, 131, { &P18465,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.0.2 (Linux; U; Android 4.1.1; en-US; Amazon Kindle Fire Build/full_otter-userdebug 4.1.1 JRO03C eng.hashcode.X.X test-keys)*/
const Device UA2044091 = {S51332, 128, { &P18465,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/1.1.3 (Linux; U; Android 4.1.1; en-US; Amazon Kindle Fire Build/cm_otter-userdebug 4.1.1 JRO03C eng.twagner.X.X test-keys)*/
const Device UA2044093 = {S51333, 129, { &P18465,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/1.1.3 (Linux; U; Android 4.1.1; en-US; Amazon Kindle Fire Build/full_otter-userdebug 4.1.1 JRO03C eng.0xD34D.X.X test-keys)*/
const Device UA2044101 = {S51334, 128, { &P18465,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/1.1.3 (Linux; U; Android 4.1.1; en-US; Amazon Kindle Fire Build/cm_otter-userdebug 4.1.1 JRO03H eng.twagner.X.X test-keys)*/
const Device UA2044104 = {S51335, 128, { &P18465,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.0.8 (Linux; U; Android 4.1.1; en-US; Amazon Kindle Fire Build/cm_otter-userdebug 4.1.1 JRO03L eng.twagner.X.X test-keys)*/
const Device UA2044109 = {S51336, 130, { &P18465,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.0.11 (Linux; U; Android 4.1.1; en-US; Amazon Kindle Fire Build/cm_otter-userdebug 4.1.1 JRO03L eng.hashcode.X.X test-keys)*/
const Device UA2044113 = {S51337, 126, { &P18465,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.1.1; en-US; Amazon Kindle Fire Build/cm_otter-userdebug 4.1.1 JRO03R eng.build.X.X test-keys)*/
const Device UA2044177 = {S51338, 140, { &P18465,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Amazon Kindle Fire Build/JRO03L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2044185 = {S51339, 149, { &P18465,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Amazon Kindle Fire Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2044213 = {S51340, 155, { &P18465,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Amazon Kindle Fire Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA2044217 = {S51341, 149, { &P18465,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Amazon Kindle Fire Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2044227 = {S51342, 149, { &P18465,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Amazon Kindle Fire Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2044229 = {S51343, 155, { &P18465,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Amazon Kindle Fire Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA2044290 = {S51344, 141, { &P18465,&P18111,&P18112,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.0.11 (Linux; U; Android 4.1.1; en-US; Amazon Kindle Fire Build/cm_otter-userdebug 4.1.1 JRO03L eng.build.X.X test-keys)*/
const Device UA2402358 = {S51345, 71, { &P18465,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Amazon Kindle Fire Build/JZO54K)*/
const Device UA2568842 = {S51346, 141, { &P17346,&P18432,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A523 Safari/8536.25*/
const Device UA2569374 = {S51347, 139, { &P17346,&P18432,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A403 Safari/8536.25*/
const Device UA2583549 = {S51348, 139, { &P17346,&P18432,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/10A406 Safari/8536.25*/
const Device UA2974030 = {S51349, 137, { &P19920,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS870 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA167736 = {S51350, 107, { &P15364,&P7133,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.0 Safari/533.3*/
const Device UA896189 = {S51351, 140, { &P19848,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHW-M440S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034271 = {S51352, 62, { &P19848,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHW-M440S Build/IMM76D)*/
const Device UA7787 = {S51353, 104, { &P12026,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonC510/R1HA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.5*/
const Device UA21951 = {S51354, 105, { &P12026,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonC510a/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3*/
const Device UA55164 = {S51355, 117, { &P12026,&P17485,&P3976,&P18092 }};	/*Useragent - Mozilla/4.0 SonyEricssonC510v/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3*/
const Device UA61982 = {S51356, 118, { &P12026,&P17485,&P3976,&P18092 }};	/*Useragent - Mozilla/4.0 SonyEricssonC510av/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3*/
const Device UA372249 = {S51357, 196, { &P15364,&P7133,&P18147,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; MS-RTC LM 8)*/
const Device UA399180 = {S51358, 164, { &P15364,&P7133,&P18147,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA545112 = {S51359, 193, { &P15364,&P7133,&P18147,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; Zune 3.0)*/
const Device UA649194 = {S51360, 179, { &P15364,&P7133,&P18147,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)*/
const Device UA2439942 = {S51361, 203, { &P15364,&P7133,&P18147,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3; .NET4.0E)*/
const Device UA2974619 = {S51362, 139, { &P18736,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA352014 = {S51363, 147, { &P18290,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ617 4G Build/1.6.0M-218.14.Demo2-MZ617) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA361965 = {S51364, 140, { &P18290,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ617 Build/1.6.0M-218.14-MZ617) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA575683 = {S51365, 140, { &P18290,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ617 4G Build/1.6.0_190.10-MZ617) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA627148 = {S51366, 140, { &P18290,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; MZ617 4G Build/1.6.0M_279_MZ617) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA632748 = {S51367, 141, { &P18290,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ617 4G Build/1.6.0M-218.14-MZ617) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA683099 = {S51368, 139, { &P18290,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; MZ617 Build/1.6.0M_279_MZ617) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA753814 = {S51369, 68, { &P18290,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.2; MZ617 Build/1.6.0M_279_MZ617)*/
const Device UA2023235 = {S51370, 140, { &P18290,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; MZ617 4G Build/1.6.0M_280_MZ617) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA6361 = {S51371, 142, { &P12453,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HTC-A6366/1.0 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA8834 = {S51372, 137, { &P12453,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-au; HTC Liberty Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32034 = {S51373, 139, { &P12453,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; HTC-A6366/1.0 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA37551 = {S51374, 141, { &P12453,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.2; en-us; HTC-A6366/1.0 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA53748 = {S51375, 139, { &P12453,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; HTC Liberty Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA278311 = {S51376, 133, { &P12453,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Liberty Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA287347 = {S51377, 135, { &P12453,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Liberty Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018340 = {S51378, 61, { &P12453,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; HTC Liberty Build/FRF91)*/
const Device UA902187 = {S51379, 101, { &P15364,&P5690,&P22343,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.32) Gecko/X Firefox/3.6.32 (Palemoon/3.6.32)*/
const Device UA2681088 = {S51380, 82, { &P15364,&P5690,&P22343,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2) Gecko/X Palemoon/3.6.0.5*/
const Device UA2816399 = {S51381, 101, { &P15364,&P5690,&P22343,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.2.32) Gecko/X Firefox/3.6.32 (Palemoon/3.6.32)*/
const Device UA2361795 = {S51382, 134, { &P15364,&P16110,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/23.0.1271.64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2387049 = {S51383, 127, { &P15364,&P16110,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; chromeframe/23.0.1271.64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2407032 = {S51384, 134, { &P15364,&P16110,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/23.0.1271.95) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11*/
const Device UA2408386 = {S51385, 97, { &P15364,&P16110,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/23.0.1271.97)*/
const Device UA2432752 = {S51386, 90, { &P15364,&P16110,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; chromeframe/23.0.1271.97)*/
const Device UA2433795 = {S51387, 176, { &P15364,&P16110,&P21914,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; chromeframe/23.0.1271.97)*/
const Device UA2435567 = {S51388, 134, { &P15364,&P16110,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/23.0.1271.97) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2437253 = {S51389, 96, { &P15364,&P16110,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/23.0.1271.97)*/
const Device UA2458336 = {S51390, 127, { &P15364,&P16110,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; chromeframe/23.0.1271.97) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2435543 = {S51391, 105, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2437659 = {S51392, 104, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2661166 = {S51393, 123, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; en-GB) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.0.4 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2766496 = {S51394, 123, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; pl-PL) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.0.4 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2779268 = {S51395, 123, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; de-DE) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.0.4 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2890464 = {S51396, 123, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; nl-NL) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.0.4 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2891956 = {S51397, 123, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; en-AU) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.0.4 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2935888 = {S51398, 123, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; it-IT) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.0.4 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2954687 = {S51399, 123, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; fr-FR) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.0.4 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2955854 = {S51400, 123, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; es-ES) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.0.4 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA3030098 = {S51401, 123, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (FreeBSD; U; Viera; en-US) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA3031793 = {S51402, 128, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; en-AU) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA3032059 = {S51403, 128, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; en-US) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA3034862 = {S51404, 128, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; en-GB) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA3036344 = {S51405, 128, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; de-DE) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA3058915 = {S51406, 128, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; fr-FR) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA3058957 = {S51407, 128, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; it-IT) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA3059897 = {S51408, 128, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; nl-NL) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA3060070 = {S51409, 128, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; es-ES) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA3079517 = {S51410, 128, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; en-NZ) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA3107996 = {S51411, 128, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; pl-PL) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA3108100 = {S51412, 128, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; sv-SE) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA3125238 = {S51413, 128, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; da-DK) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA3125256 = {S51414, 128, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; cs-CZ) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA3125461 = {S51415, 128, { &P15364,&P18145,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD; U; Viera; en-IE) AppleWebKit/537.11 (KHTML, like Gecko) Viera/3.1.8 Chrome/23.0.1271.97 Safari/537.11*/
const Device UA7806 = {S51416, 149, { &P13413,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; nl-be; San Francisco Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA9416 = {S51417, 157, { &P13413,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-pt; Optimus San Francisco Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA26397 = {S51418, 146, { &P13413,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; BASE lutea Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA61976 = {S51419, 156, { &P13413,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Orange San Francisco Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA659773 = {S51420, 78, { &P13413,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Orange San Francisco Build/ERE27)*/
const Device UA2606980 = {S51421, 112, { &P19747,&P5043,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia206/03.59; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2668873 = {S51422, 114, { &P19747,&P5043,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia206.1/03.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA4875 = {S51423, 72, { &P11795,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-C3053/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49108 = {S51424, 72, { &P11795,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-C3050/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61613 = {S51425, 127, { &P11795,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-C3050-ORANGE/C3050BVIH2 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA61919 = {S51426, 113, { &P11795,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-C3050/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA62892 = {S51427, 142, { &P11795,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-C3050C/1.0 Release/1.19.2009 Browser/Openwave6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA64639 = {S51428, 113, { &P11795,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-C3053/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA103328 = {S51429, 140, { &P11795,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungC3050/C3050BUIC6 Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA131532 = {S51430, 140, { &P11795,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungC3050/C3050BUII1 Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA167158 = {S51431, 140, { &P11795,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungC3050/C3050BUJI1 Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA275795 = {S51432, 140, { &P11795,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungC3050/C3050BUIJ1 Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA436491 = {S51433, 140, { &P11795,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungC3050/C3050BUKC1 Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA494971 = {S51434, 140, { &P11795,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungC3050/C3050BUIG1 Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA652247 = {S51435, 140, { &P11795,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungC3050/C3050BUJK1 Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA1845120 = {S51436, 140, { &P11795,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungC3050/C3050BUIF3 Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA2275937 = {S51437, 140, { &P11795,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungC3050/C3050BUID2 Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA2973958 = {S51438, 144, { &P21611,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2489642 = {S51439, 101, { &P18718,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 620)*/
const Device UA2558007 = {S51440, 111, { &P18718,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 620; Vodafone)*/
const Device UA2664712 = {S51441, 109, { &P18718,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 620; ANZ846)*/
const Device UA2995052 = {S51442, 109, { &P18718,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 620; Meteor)*/
const Device UA2672391 = {S51443, 145, { &P19762,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI MT1-U06 Build/HuaweiMT1-U06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2860012 = {S51444, 74, { &P19762,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HUAWEI MT1-U06 Build/HuaweiMT1-U06)*/
const Device UA2868559 = {S51445, 152, { &P19762,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HUAWEI MT1-U06 Build/HuaweiMT1-U06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3002883 = {S51446, 159, { &P19762,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HUAWEI MT1-U06 Build/HuaweiMT1-U06C02B105) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583362 = {S51447, 138, { &P20240,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LG-E971 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2703286 = {S51448, 60, { &P20240,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E971 Build/JZO54K)*/
const Device UA2671474 = {S51449, 138, { &P19894,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2853212 = {S51450, 139, { &P19894,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830C Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2709228 = {S51451, 86, { &P15364,&P7133,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/4.1.0)*/
const Device UA2710687 = {S51452, 87, { &P15364,&P7133,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/4.1.0)*/
const Device UA2753384 = {S51453, 86, { &P15364,&P7133,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/4.1.2)*/
const Device UA2753392 = {S51454, 87, { &P15364,&P7133,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/4.1.2)*/
const Device UA2774072 = {S51455, 116, { &P15364,&P7133,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 Sleipnir/4.1.3*/
const Device UA2774136 = {S51456, 123, { &P15364,&P7133,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 Sleipnir/4.1.3*/
const Device UA2784076 = {S51457, 87, { &P15364,&P7133,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/4.1.3)*/
const Device UA2784890 = {S51458, 86, { &P15364,&P7133,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/4.1.3)*/
const Device UA2877450 = {S51459, 124, { &P15364,&P7133,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36 Sleipnir/4.1.4*/
const Device UA2877512 = {S51460, 117, { &P15364,&P7133,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36 Sleipnir/4.1.4*/
const Device UA2877563 = {S51461, 87, { &P15364,&P7133,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/4.1.4)*/
const Device UA2880358 = {S51462, 86, { &P15364,&P7133,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/4.1.4)*/
const Device UA633595 = {S51463, 97, { &P15364,&P17813,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.16) Gecko/X Ubuntu/8.04 (hardy) Firefox/3.6.16*/
const Device UA900539 = {S51464, 131, { &P20004,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2487271 = {S51465, 139, { &P20004,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Karbonn A2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2685344 = {S51466, 55, { &P20004,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; A2 Build/GRK39F)*/
const Device UA2986356 = {S51467, 63, { &P20004,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; Karbonn A2 Build/GRK39F)*/
const Device UA2986902 = {S51468, 148, { &P20004,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Callbar A2 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2800688 = {S51469, 139, { &P17346,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A4372q Safari/8536.25*/
const Device UA2885396 = {S51470, 139, { &P17346,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.0 Mobile/11A4400f Safari/8536.25*/
const Device UA2947247 = {S51471, 139, { &P17346,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11A4414e Safari/8536.25*/
const Device UA3026311 = {S51472, 141, { &P17346,&P22882,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/6.0 Mobile/11A4435d Safari/8536.25*/
const Device UA49350 = {S51473, 170, { &P17709,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; LG-C555-parrot Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA49352 = {S51474, 163, { &P17709,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; LG-C555 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2019608 = {S51475, 59, { &P17709,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-C555 Build/FRG83)*/
const Device UA2203164 = {S51476, 43, { &P23144,&P18580,&P17470,&P18092 }};	/*Useragent - Facebook/4110.0 CFNetwork/609 Darwin/13.0.0*/
const Device UA2740419 = {S51477, 42, { &P23144,&P18580,&P17470,&P18092 }};	/*Useragent - Tumblr/117 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA610894 = {S51478, 141, { &P12798,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  LT18i Build/4.1.A.0.562) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA901039 = {S51479, 140, { &P12798,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LT18i Build/4.1.A.0.562) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1995374 = {S51480, 140, { &P12798,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT18i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2032235 = {S51481, 140, { &P12798,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LT18a Build/4.1.A.0.562) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2229004 = {S51482, 140, { &P12798,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT18a Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2380920 = {S51483, 141, { &P12798,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  LT18i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA3064991 = {S51484, 132, { &P22754,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HP Slate 7 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3096509 = {S51485, 140, { &P17346,&P19826,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B329 Safari/8536.25*/
const Device UA3109536 = {S51486, 138, { &P17346,&P19826,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B144 Safari/8536.25*/
const Device UA3109894 = {S51487, 140, { &P17346,&P19826,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B146 Safari/8536.25*/
const Device UA3110177 = {S51488, 138, { &P17346,&P19826,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10B141 Safari/8536.25*/
const Device UA63265 = {S51489, 145, { &P12311,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SPH-M900 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA89992 = {S51490, 150, { &P12311,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SPH-M900 Build/ECLAIR) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA431264 = {S51491, 67, { &P12311,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SPH-M900 Build/ECLAIR)*/
const Device UA1992961 = {S51492, 146, { &P17595,&P18207,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_0; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0*/
const Device UA1996618 = {S51493, 148, { &P17595,&P18207,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_0; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0*/
const Device UA2040830 = {S51494, 148, { &P17595,&P18207,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_1; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0*/
const Device UA2204649 = {S51495, 148, { &P17595,&P18207,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_2; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0*/
const Device UA2229327 = {S51496, 146, { &P17595,&P18207,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_2; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0*/
const Device UA2236274 = {S51497, 144, { &P17595,&P18207,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_2; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0*/
const Device UA2340560 = {S51498, 146, { &P17595,&P18207,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_1; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0*/
const Device UA2382299 = {S51499, 127, { &P17595,&P18207,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_2; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0*/
const Device UA2555179 = {S51500, 148, { &P17595,&P18207,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_3; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0*/
const Device UA2695557 = {S51501, 146, { &P17595,&P18207,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_3; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0*/
const Device UA2705968 = {S51502, 144, { &P17595,&P18207,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_3; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0*/
const Device UA2763460 = {S51503, 127, { &P17595,&P18207,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_3; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0*/
const Device UA2780787 = {S51504, 148, { &P17595,&P18207,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_4; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0*/
const Device UA2835519 = {S51505, 144, { &P17595,&P18207,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_2; en-US) AppleWebKit/531.21.8+(KHTML, like Gecko, Safari/528.16) Version/5.10.3 OmniWeb/622.14.0*/
const Device UA2939422 = {S51506, 144, { &P17595,&P18207,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_4; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0*/
const Device UA3002732 = {S51507, 127, { &P17595,&P18207,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_4; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0*/
const Device UA3008436 = {S51508, 146, { &P17595,&P18207,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_4; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0*/
const Device UA3092860 = {S51509, 148, { &P17595,&P18207,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_5; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0*/
const Device UA2203818 = {S51510, 132, { &P20968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Le Pan S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2567971 = {S51511, 81, { &P15364,&P16110,&P22369,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:19.0) Gecko/X Firefox/19.0 PaleMoon/19.0.2*/
const Device UA2567962 = {S51512, 90, { &P15364,&P16110,&P22369,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:19.0) Gecko/X Firefox/19.0 PaleMoon/19.0.2-x64*/
const Device UA2191473 = {S51513, 135, { &P19788,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; E350 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2500172 = {S51514, 57, { &P19788,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; E350 Build/IML74K)*/
const Device UA2657151 = {S51515, 135, { &P19788,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; E350 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2746837 = {S51516, 133, { &P19788,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-; E350 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3086246 = {S51517, 57, { &P19788,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; E350 Build/IMM76D)*/
const Device UA2782337 = {S51518, 106, { &P18095,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; 909)*/
const Device UA2204002 = {S51519, 99, { &P15364,&P17188,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA3063715 = {S51520, 138, { &P18645,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3067065 = {S51521, 145, { &P18645,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081050 = {S51522, 137, { &P18645,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-L710 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081101 = {S51523, 138, { &P18645,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I747M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3082058 = {S51524, 138, { &P18645,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083941 = {S51525, 138, { &P18645,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094746 = {S51526, 138, { &P18645,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530U Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114358 = {S51527, 138, { &P18645,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2003034 = {S51528, 86, { &P18254,&P7480,&P18245,&P18164 }};	/*Useragent - Pixray-Seeker/2.0 (Pixray-Seeker; http://www.pixray.com/pixraybot; crawler@pixray.com)*/
const Device UA2261979 = {S51529, 88, { &P18254,&P7480,&P18245,&P18164 }};	/*Useragent - Pixray-Seeker/2.0 (Pixray-Seeker; +http://www.pixray.com/pixraybot; +crawler@pixray.com)*/
const Device UA2262059 = {S51530, 71, { &P18254,&P7480,&P18245,&P18164 }};	/*Useragent - Pixray-Seeker/2.0 (http://www.pixray.com/pixraybot; crawler@pixray.com)*/
const Device UA2357567 = {S51531, 72, { &P18254,&P7480,&P18245,&P18164 }};	/*Useragent - Pixray-Seeker/2.0 (http://www.pixray.com/pixraybot; +crawler@pixray.com)*/
const Device UA2405202 = {S51532, 155, { &P15228,&P5043,&P24673,&P18092 }};	/*Useragent - NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac2-03) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA794873 = {S51533, 63, { &P15984,&P17483,&P3459,&P18092 }};	/*Useragent - LG-C395/V10e Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2431224 = {S51534, 226, { &P18283,&P18066,&P18369,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2797432 = {S51535, 77, { &P17595,&P22878,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro10%2C1)*/
const Device UA2869950 = {S51536, 76, { &P17595,&P22878,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookAir5%2C2)*/
const Device UA2879461 = {S51537, 71, { &P17595,&P22878,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (iMac12%2C2)*/
const Device UA2938159 = {S51538, 73, { &P17595,&P22878,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (Macmini6%2C2)*/
const Device UA2957393 = {S51539, 72, { &P17595,&P22878,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacPro5%2C1)*/
const Device UA3011663 = {S51540, 73, { &P17595,&P22878,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBook5%2C1)*/
const Device UA3065385 = {S51541, 138, { &P19902,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA63147 = {S51542, 77, { &P12054,&P5826,&P4093,&P18092 }};	/*Useragent - BlackBerry9500/4.7.0.181 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/120*/
const Device UA2404102 = {S51543, 84, { &P22965,&P7472,&P3534,&P18092 }};	/*Useragent - ALCATEL-OT871A/980 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA2768842 = {S51544, 84, { &P22965,&P7472,&P3534,&P18092 }};	/*Useragent - ALCATEL-OT871A/984 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA1991938 = {S51545, 139, { &P18747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A840S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027633 = {S51546, 61, { &P18747,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-A840S Build/IMM76I)*/
const Device UA2399235 = {S51547, 137, { &P18747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-A840S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2505184 = {S51548, 138, { &P18747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A840S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2663869 = {S51549, 155, { &P18747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A840S Build/VEGAVIET.COM ROM ICS V4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2732548 = {S51550, 155, { &P18747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A840S Build/VEGAVIET.COM ROM ICS V5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2764228 = {S51551, 156, { &P18747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IM-A840S Build/VEGAVIET.COM ROM ICS V4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985683 = {S51552, 155, { &P18747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A840S Build/VEGAVIET.COM ROM ICS V2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA3100818 = {S51553, 162, { &P18747,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A840S Build/VEGAVIET.COM ROM ICS V6.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?i Safari/534.30*/
const Device UA3025464 = {S51554, 111, { &P18583,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia110/03.33; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA2672310 = {S51555, 138, { &P18582,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT28i Build/6.1.E.2.68) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676691 = {S51556, 137, { &P18582,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT28h Build/6.1.E.3.7) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708584 = {S51557, 140, { &P18582,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT28at Build/6.1.C.1.111) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3065340 = {S51558, 130, { &P17337,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3068338 = {S51559, 131, { &P17337,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500R Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094157 = {S51560, 131, { &P17337,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2755123 = {S51561, 137, { &P19856,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LS970 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2204098 = {S51562, 99, { &P15364,&P7127,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2507447 = {S51563, 118, { &P15364,&P7127,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 8.0; Windows NT 5.0) Chrome/22.0.1229.79*/
const Device UA2529239 = {S51564, 118, { &P15364,&P7127,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 7.0; Windows NT 5.0) Chrome/22.0.1229.79*/
const Device UA2737760 = {S51565, 123, { &P15364,&P7127,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 9.0; Windows NT 5.0WOW64) Chrome/22.0.1229.79*/
const Device UA2678441 = {S51566, 85, { &P15364,&P7133,&P22560,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:20.0) Gecko/X Firefox/20.0 IceDragon/20.0.1.14*/
const Device UA690125 = {S51567, 93, { &P17595,&P17295,&P20917,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.1b3) Gecko/X Firefox/3.1b3 GTB5*/
const Device UA2568328 = {S51568, 102, { &P15364,&P7127,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA1965978 = {S51569, 147, { &P18224,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8815 Build/HuaweiU8815) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2027995 = {S51570, 147, { &P18224,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8818 Build/HuaweiU8818) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2364860 = {S51571, 141, { &P18224,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Ascend G300 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2456142 = {S51572, 154, { &P18224,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI Ascend G 300 Build/HuaweiU8815) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2510694 = {S51573, 156, { &P18224,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8815N Build/HuaweiU8815NC02B942) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2996423 = {S51574, 143, { &P18224,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; U8815-51 Build/HuaweiU8815) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2595672 = {S51575, 139, { &P18747,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A840S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695654 = {S51576, 61, { &P18747,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A840S Build/JZO54K)*/
const Device UA2705034 = {S51577, 142, { &P18747,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A840S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30*/
const Device UA2707389 = {S51578, 140, { &P18747,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A840SP Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2779407 = {S51579, 138, { &P18747,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A840S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2935993 = {S51580, 137, { &P18747,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; IM-A840S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997798 = {S51581, 143, { &P18747,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A840SP Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30*/
const Device UA3121281 = {S51582, 149, { &P18747,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A840S Build/GALAXY_S5_PRO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30*/
const Device UA2887909 = {S51583, 154, { &P18466,&P18066,&P23483,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA3006273 = {S51584, 59, { &P23702,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; ME173X Build/JOP40D)*/
const Device UA3014341 = {S51585, 130, { &P23702,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ME173X Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422327 = {S51586, 213, { &P20081,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; SAMSUNG-GT-N7108_TD/1.0 Android/4.1.1 Release/08.30.2012 Browser/AppleWebKit534.30 Build/JRO03C) ApplelWebkit/534.30 (KHTML,like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2454885 = {S51587, 130, { &P20081,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7108 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2458021 = {S51588, 139, { &P20081,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7108 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2554966 = {S51589, 61, { &P20081,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-N7108 Build/JRO03C)*/
const Device UA2747531 = {S51590, 139, { &P20081,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7108 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2795584 = {S51591, 61, { &P20081,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-N7108 Build/JZO54K)*/
const Device UA2957703 = {S51592, 130, { &P20081,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7108 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2975083 = {S51593, 138, { &P18295,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One X Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975952 = {S51594, 138, { &P18295,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA653539 = {S51595, 62, { &P15364,&P18110,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:6.0.2) Gecko/X Abrowser/6.0.2*/
const Device UA774879 = {S51596, 77, { &P15364,&P18110,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:6.0a2) Gecko/X Firefox/6.0a2 Iceweasel/6.0a2*/
const Device UA906465 = {S51597, 99, { &P15364,&P18110,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux armv7l; rv:6.0.2) Gecko/X Firefox/6.0.2 Version.5.9.905.XT875.Verizon.en.US*/
const Device UA2041419 = {S51598, 77, { &P15364,&P5690,&P20755,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:15.0) Gecko/X Firefox/15.0.1 SeaMonkey/2.12.1*/
const Device UA2200233 = {S51599, 62, { &P15364,&P5690,&P20755,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:15.0) Gecko/X SeaMonkey/2.12.1*/
const Device UA2230327 = {S51600, 77, { &P15364,&P5690,&P20755,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:15.0) Gecko/X Firefox/15.0.1 SeaMonkey/2.12.1*/
const Device UA54793 = {S51601, 108, { &P11996,&P17484,&P4086,&P18092 }};	/*Useragent - SAMSUNG-GT-S5560/1.0 SHP/VPP/R5 Dolfin/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA2782792 = {S51602, 97, { &P17595,&P17296,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:22.0) Gecko/X Firefox/22.0 AlexaToolbar/alxf-2.18*/
const Device UA774272 = {S51603, 110, { &P15364,&P17832,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.13) Gecko/X Ubuntu/7.10 (gutsy) Firefox/2.0.0.13 (Linux Mint)*/
const Device UA782200 = {S51604, 110, { &P15364,&P17832,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.16) Gecko/X Ubuntu/7.10 (gutsy) Firefox/2.0.0.16 (Linux Mint)*/
const Device UA2288737 = {S51605, 97, { &P15364,&P17832,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-GB; rv:1.8.1.10) Gecko/X Ubuntu/7.10 (gutsy) Firefox/2.0.0.10*/
const Device UA2233410 = {S51606, 69, { &P13123,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GT-S5360; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2482052 = {S51607, 74, { &P13123,&P18110,&P24663,&P18092 }};	/*Useragent - Openwave (Linux; U; 2.3.6; zh-cn; GT-S5360; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2553342 = {S51608, 71, { &P13123,&P18110,&P24663,&P18092 }};	/*Useragent - Opera (Linux; U; 2.3.6; zh-cn; GT-S5360; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2696996 = {S51609, 70, { &P13123,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GT-S5360L; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2843622 = {S51610, 70, { &P13123,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GT-S5360B; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2754269 = {S51611, 132, { &P13644,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773525 = {S51612, 132, { &P13644,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; EVO Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2487833 = {S51613, 53, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - IUC(U;iOS 5.1.1;Zh-cn;320*480;)/UCWEB8.9.0.238/42/997*/
const Device UA2507921 = {S51614, 87, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2507945 = {S51615, 88, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mstar) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2533114 = {S51616, 90, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; generic) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2559156 = {S51617, 95, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui browser) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2560095 = {S51618, 97, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mstar platform) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2560139 = {S51619, 95, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; jbed-fastbcc) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2562007 = {S51620, 53, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - IUC(U;iOS 5.0.1;Zh-cn;320*480;)/UCWEB8.9.1.271/42/800*/
const Device UA2565379 = {S51621, 53, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - IUC(U;iOS 6_1_2;en-us;320*480;)/UCWEB8.9.0.238/44/355*/
const Device UA2580086 = {S51622, 84, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; vi; maui) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2581032 = {S51623, 91, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sunmicro) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2581066 = {S51624, 84, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; id; maui) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2582066 = {S51625, 53, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB8.9.1.271/41/800*/
const Device UA2659485 = {S51626, 85, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; id; mstar) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2659713 = {S51627, 97, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile /AJSC/1.5*/
const Device UA2661324 = {S51628, 90, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; karbonn) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2662290 = {S51629, 99, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile TRUSTED/1.0*/
const Device UA2662737 = {S51630, 100, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; opera) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile TRUSTED/1.0*/
const Device UA2699741 = {S51631, 100, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; spice) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile TRUSTED/1.0*/
const Device UA2732325 = {S51632, 88, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile,*/
const Device UA2779083 = {S51633, 80, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0*/
const Device UA2852276 = {S51634, 90, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; Pt-BR; mozilla) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2853313 = {S51635, 87, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; jbed) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA3071496 = {S51636, 99, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui wap browser) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA3118955 = {S51637, 85, { &P18088,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; vi; mstar) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA649161 = {S51638, 80, { &P14188,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5233)/UC Browser8.2.0.132/50/355/UCWEB Mobile*/
const Device UA674939 = {S51639, 31, { &P14188,&P5129,&P24667,&P18092 }};	/*Useragent - Nokia5233/UCWEB8.2.0.116/50/800*/
const Device UA2160378 = {S51640, 88, { &P14188,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokia5233) UCBrowser8.2.1.144/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA2674215 = {S51641, 135, { &P18283,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-06D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676016 = {S51642, 137, { &P18283,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676483 = {S51643, 137, { &P18283,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I535 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676980 = {S51644, 137, { &P18283,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708385 = {S51645, 138, { &P18283,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300T Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2711695 = {S51646, 130, { &P18283,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2734808 = {S51647, 137, { &P18283,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9300 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735314 = {S51648, 138, { &P18283,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T999V Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2736225 = {S51649, 134, { &P18283,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; I9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2736610 = {S51650, 137, { &P18283,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-l9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2767260 = {S51651, 137, { &P18283,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I939 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA79310 = {S51652, 131, { &P12779,&P6374,&P4418,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9370; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.374 Mobile Safari/534.11+*/
const Device UA173283 = {S51653, 128, { &P17595,&P17295,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13*/
const Device UA301347 = {S51654, 127, { &P17595,&P17295,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.84 Safari/534.13*/
const Device UA305317 = {S51655, 128, { &P17595,&P17295,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_0; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.107 Safari/534.13*/
const Device UA307253 = {S51656, 128, { &P17595,&P17295,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.107 Safari/534.13*/
const Device UA528162 = {S51657, 128, { &P17595,&P17295,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.107 Safari/534.13*/
const Device UA667475 = {S51658, 128, { &P17595,&P17295,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13*/
const Device UA787548 = {S51659, 128, { &P17595,&P17295,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.107 Safari/534.13*/
const Device UA1863809 = {S51660, 128, { &P17595,&P17295,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13*/
const Device UA2566887 = {S51661, 139, { &P18450,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST23i Build/11.0.A.5.5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583123 = {S51662, 140, { &P18450,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST23a Build/11.0.A.2.16) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA3066873 = {S51663, 141, { &P16292,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068334 = {S51664, 141, { &P16292,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Galaxy Nexus Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081107 = {S51665, 137, { &P16292,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Galaxy X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081781 = {S51666, 139, { &P16292,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; Galaxy Nexus Build/JRN84D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA9013 = {S51667, 137, { &P16036,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; Nexus One Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA50157 = {S51668, 145, { &P16036,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Nexus One Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA50213 = {S51669, 146, { &P16036,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Nexus One Build/EPE54B) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA67923 = {S51670, 123, { &P16036,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; Nexus One Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile*/
const Device UA89187 = {S51671, 142, { &P16036,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; Nexus One Build/ERD78) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA368636 = {S51672, 142, { &P16036,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; Nexus One Build/ERD72) AppleWebKit/525.10  (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA368707 = {S51673, 137, { &P16036,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; Nexus One Build/ERE19) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1951973 = {S51674, 143, { &P16036,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en_us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018068 = {S51675, 145, { &P16036,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Nexus One Build/EPF30) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2658394 = {S51676, 136, { &P16036,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; Nexus One Build/ERD79) AppleWebKit/530.17 (KHTML like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2658595 = {S51677, 145, { &P16036,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Nexus One Build/ERE36B) AppleWebKit/530.17 (KHTML like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA3058545 = {S51678, 130, { &P16036,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; Nexus One Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2389856 = {S51679, 154, { &P17905,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonyEricssonSO-03D Build/6.1.F.0.106) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2390429 = {S51680, 142, { &P17905,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-03D Build/6.1.F.0.106) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467167 = {S51681, 64, { &P17905,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SO-03D Build/6.1.F.0.106)*/
const Device UA3031378 = {S51682, 142, { &P17905,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-03D Build/6.1.F.0.128) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423872 = {S51683, 140, { &P18736,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One SV Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2585753 = {S51684, 143, { &P15390,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P925/P92530b Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2606215 = {S51685, 135, { &P15390,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P925 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2766146 = {S51686, 57, { &P15390,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P925 Build/ICS)*/
const Device UA780734 = {S51687, 114, { &P14635,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-00/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6*/
const Device UA907255 = {S51688, 115, { &P14635,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-00/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA615554 = {S51689, 162, { &P18386,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; SAMSUNG SGH-I757M/I757MUGLCA Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA660196 = {S51690, 143, { &P18386,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; SGH-I757M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022428 = {S51691, 150, { &P18386,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I757 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024809 = {S51692, 67, { &P18386,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-I757M Build/GINGERBREAD)*/
const Device UA90201 = {S51693, 141, { &P17938,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; AS740 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA90451 = {S51694, 172, { &P17938,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; US740 Build/ERE27) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (innerActive-ANDROID-206)*/
const Device UA140365 = {S51695, 141, { &P17938,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; US740 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA553356 = {S51696, 63, { &P17938,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; US740 Build/ERE27)*/
const Device UA2019204 = {S51697, 63, { &P17938,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; AS740 Build/ERE27)*/
const Device UA9952 = {S51698, 7, { &P15364,&P18588,&P18107,&P18092 }};	/*Useragent - Firefox*/
const Device UA624932 = {S51699, 137, { &P16571,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; Acer E320 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA646782 = {S51700, 144, { &P16571,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; Acer E320-orange Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032959 = {S51701, 61, { &P16571,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; Acer E320 Build/GRJ22)*/
const Device UA2425558 = {S51702, 75, { &P17595,&P17296,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8) Presto/2.12.388 Version/12.12*/
const Device UA2461257 = {S51703, 138, { &P20260,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; onetouch EVO7 Build/WIND_01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2554839 = {S51704, 138, { &P20260,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; onetouch EVO7 Build/MAIN_01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2666627 = {S51705, 141, { &P20260,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; onetouch EVO7 Build/TMOBILE_PL) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2168626 = {S51706, 139, { &P15767,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_0 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA2197113 = {S51707, 127, { &P15767,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/6533.18.5 DiigoBrowser*/
const Device UA2201514 = {S51708, 157, { &P15767,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/6533.18.5 DiigoBrowser Safari/6533.18.5 DiigoBrowser*/
const Device UA2202088 = {S51709, 187, { &P15767,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/6533.18.5 DiigoBrowser Safari/6533.18.5 DiigoBrowser Safari/6533.18.5 DiigoBrowser*/
const Device UA2203944 = {S51710, 136, { &P15767,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_0 like Mac OS X; en_us) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA2237622 = {S51711, 128, { &P15767,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10A403 Safari/6533.18.5*/
const Device UA2349550 = {S51712, 217, { &P15767,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/6533.18.5 DiigoBrowser Safari/6533.18.5 DiigoBrowser Safari/6533.18.5 DiigoBrowser Safari/6533.18.5 DiigoBrowser*/
const Device UA2356949 = {S51713, 141, { &P15767,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_0_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA2381963 = {S51714, 151, { &P15767,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6 like Mac OS X; zh-cn Model:iPad2,1) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA2386837 = {S51715, 138, { &P15767,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_0_1 like Mac OS X; en_us) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5*/
const Device UA2433995 = {S51716, 132, { &P15767,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/6533.18.5 iLunascape/3200*/
const Device UA2481129 = {S51717, 130, { &P15767,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10A523 Safari/6533.18.5*/
const Device UA2486413 = {S51718, 141, { &P15767,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_0_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA2843072 = {S51719, 126, { &P15767,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_0_1 like Mac OS X; 6536.26) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5*/
const Device UA2996365 = {S51720, 161, { &P15767,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_0_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5*/
const Device UA2755357 = {S51721, 138, { &P12968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26i Build/6.1.A.2.55) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2938240 = {S51722, 134, { &P12968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LT26i Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA22618 = {S51723, 13, { &P15498,&P17483,&P17660,&P18092 }};	/*Useragent - LG/U8120/v1.0*/
const Device UA2540006 = {S51724, 154, { &P18466,&P18066,&P21167,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA1997332 = {S51725, 156, { &P18295,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC One X Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2014131 = {S51726, 152, { &P18295,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; One X Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2201512 = {S51727, 152, { &P18295,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; One X Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2205660 = {S51728, 156, { &P18295,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC One X Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2234669 = {S51729, 156, { &P18295,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC One X Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258039 = {S51730, 156, { &P18295,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; One X Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2660062 = {S51731, 155, { &P18295,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One X Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2844958 = {S51732, 156, { &P18295,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; EndeavorU Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2012426 = {S51733, 116, { &P15364,&P5690,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1*/
const Device UA2170811 = {S51734, 123, { &P15364,&P5690,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1*/
const Device UA2195034 = {S51735, 116, { &P15364,&P5690,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1*/
const Device UA2437329 = {S51736, 150, { &P15932,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3110 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2532986 = {S51737, 150, { &P15932,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3110 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2535476 = {S51738, 149, { &P15932,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3110 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2624507 = {S51739, 157, { &P15932,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3110 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2630065 = {S51740, 148, { &P15932,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-; GT-P3110 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2632244 = {S51741, 147, { &P15932,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; es-; GT-P3110 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2968947 = {S51742, 157, { &P15932,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3110 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1.2/p3110*/
const Device UA2636477 = {S51743, 129, { &P12879,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/537.35 (KHTML, like Gecko) Chrome/27.0.1445.2 Safari/537.35*/
const Device UA2754726 = {S51744, 130, { &P12879,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755089 = {S51745, 130, { &P12879,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T859 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2739980 = {S51746, 129, { &P15767,&P18117,&P22902,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.4.2 Mobile/9B206 Safari/7534.48.3*/
const Device UA2799912 = {S51747, 127, { &P15767,&P18117,&P22902,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.4.2 Mobile/9B176 Safari/7534.48.3*/
const Device UA59163 = {S51748, 132, { &P15364,&P18110,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone-W1/V02c_19Jan2011_ZA; U; Linux; en-ZA) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA250354 = {S51749, 114, { &P15364,&P18110,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; EBRD1101; EXT) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2155903 = {S51750, 121, { &P15364,&P18110,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; 2.2; zh-cn; sdk Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2192467 = {S51751, 114, { &P15364,&P18110,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; EBRD1201; EXT) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2302605 = {S51752, 118, { &P15364,&P18110,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; EBRD1101(P); INT1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2309095 = {S51753, 94, { &P15364,&P18110,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;zh-tw)AppleWebKit/530.17(KHTML,like Gecko)Version/4.0 Mobile Safari/530.17*/
const Device UA2462368 = {S51754, 84, { &P15364,&P7133,&P22556,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:17.0) Gecko/X Firefox/17.0 IceDragon/18.0.1.0*/
const Device UA2489793 = {S51755, 84, { &P15364,&P7133,&P22556,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:18.0) Gecko/X Firefox/18.0 IceDragon/18.0.3.1*/
const Device UA2489674 = {S51756, 154, { &P18466,&P18066,&P19837,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA3035268 = {S51757, 106, { &P21000,&P17484,&P18369,&P18092 }};	/*Useragent - SAMSUNG-GT-S5222R Opera/9.80 (J2ME/MIDP; Opera Mini/7.0.32584/30.3697; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2554152 = {S51758, 139, { &P21384,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F240L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2700472 = {S51759, 61, { &P21384,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F240L Build/JZO54K)*/
const Device UA2028805 = {S51760, 134, { &P23855,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; WM8850-mid Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2340015 = {S51761, 63, { &P23855,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; WM8850-mid Build/IML74K)*/
const Device UA2985329 = {S51762, 136, { &P23855,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; WM8850_X6-7V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3042908 = {S51763, 134, { &P23855,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; WM8850_10V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA479357 = {S51764, 86, { &P15364,&P6704,&P17282,&P18092 }};	/*Useragent - Firefox/1.0 (Windows; U; Win98; en-US; Localization; rv:1.4) Gecko/X Netscape/7.1 (ax)*/
const Device UA2502914 = {S51765, 70, { &P15364,&P6704,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win98; es-AR; rv:1.7.6) Gecko/X Firefox/1.0.2*/
const Device UA2506948 = {S51766, 76, { &P15364,&P6704,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win 9x 4.90; es-AR; rv:1.7.6) Gecko/X Firefox/1.0.2*/
const Device UA3037416 = {S51767, 133, { &P18465,&P18110,&P24339,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Safari/535.19 Silk-Accelerated=false*/
const Device UA623027 = {S51768, 154, { &P13644,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PC36100 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA14386 = {S51769, 152, { &P14182,&P5267,&P3834,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5333/S5333DDJI5; U; Bada/1.0; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA2512431 = {S51770, 78, { &P15364,&P18140,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; Edition Linux Mint) Presto/2.12.388 Version/12.14*/
const Device UA2962219 = {S51771, 97, { &P17595,&P17297,&P22171,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:23.0) Gecko/X Firefox/23.0 AlexaToolbar/alxf-2.18*/
const Device UA4134 = {S51772, 140, { &P14997,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Acer Liquid Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20161 = {S51773, 177, { &P14997,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; Liquid Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Acer_S100/Acer_LiquidE_4.008.08_EMEA_VFIT*/
const Device UA20542 = {S51774, 138, { &P14997,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Acer Liquid Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20543 = {S51775, 133, { &P14997,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Liquid Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA23764 = {S51776, 131, { &P14997,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; cs-; Liquid Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018424 = {S51777, 57, { &P14997,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; Liquid Build/MASTER)*/
const Device UA2023023 = {S51778, 64, { &P14997,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; Acer Liquid Build/FRG83G)*/
const Device UA157261 = {S51779, 135, { &P17952,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SHW-M110S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA167883 = {S51780, 137, { &P17952,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SHW-M110S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA177565 = {S51781, 61, { &P17952,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SHW-M110S Build/FROYO)*/
const Device UA495028 = {S51782, 59, { &P17952,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; SHW-M110S Build/FROYO)*/
const Device UA2018387 = {S51783, 137, { &P17952,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; SHW-M110S Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2743554 = {S51784, 128, { &P17952,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; SHW-M110S Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA64274 = {S51785, 105, { &P13337,&P17485,&P4223,&P18092 }};	/*Useragent - SonyEricssonJ108i/R7EA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2*/
const Device UA427333 = {S51786, 113, { &P15767,&P18117,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Twitter for iPad*/
const Device UA615678 = {S51787, 115, { &P15767,&P18117,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Twitter for iPad*/
const Device UA1954721 = {S51788, 113, { &P15767,&P18117,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Twitter for iPad*/
const Device UA2769848 = {S51789, 140, { &P17595,&P17296,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3) AppleWebKit/534.31 (KHTML, like Gecko) Chrome/17.0.558.0 Safari/534.31 UCBrowser/2.3.2.300*/
const Device UA459196 = {S51790, 73, { &P18323,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-S5222/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2675811 = {S51791, 75, { &P15364,&P17285,&P22225,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:20.0) Gecko/X Firefox/20.0 SeaMonkey/2.17.1*/
const Device UA924833 = {S51792, 137, { &P21005,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-10D Build/V16R45C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031045 = {S51793, 137, { &P21005,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-10D Build/V18R46F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033075 = {S51794, 137, { &P21005,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-10D Build/V15R43D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2368742 = {S51795, 137, { &P21005,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-10D Build/V20R47F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2603604 = {S51796, 137, { &P21005,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-10D Build/V21R48A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2990807 = {S51797, 59, { &P21005,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; F-10D Build/V21R48A)*/
const Device UA3021810 = {S51798, 137, { &P21005,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-10D Build/V22R49C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2433032 = {S51799, 156, { &P14574,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus S Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567437 = {S51800, 155, { &P14574,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus S Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2662468 = {S51801, 156, { &P14574,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus S Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2968199 = {S51802, 164, { &P14574,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus S Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/crespo*/
const Device UA2566502 = {S51803, 131, { &P17337,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566910 = {S51804, 132, { &P17337,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500R Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2633704 = {S51805, 139, { &P17337,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 WIFI_3G Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22*/
const Device UA2633885 = {S51806, 132, { &P17337,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2634689 = {S51807, 132, { &P17337,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500M Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2634755 = {S51808, 138, { &P17337,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2635192 = {S51809, 131, { &P17337,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P7500 Build/HTJ85B) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA598788 = {S51810, 113, { &P18221,&P5843,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia302/14.26; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.11*/
const Device UA2676094 = {S51811, 130, { &P23773,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8010 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676103 = {S51812, 130, { &P23773,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8010 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA627995 = {S51813, 127, { &P21222,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; AT1S0 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA633182 = {S51814, 129, { &P21222,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; AT1S0 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2019350 = {S51815, 56, { &P21222,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; AT1S0 Build/HTJ85B)*/
const Device UA2022902 = {S51816, 58, { &P21222,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; AT1S0 Build/HTK55D)*/
const Device UA3017280 = {S51817, 158, { &P15575,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3031218 = {S51818, 157, { &P15575,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3032700 = {S51819, 158, { &P15575,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3033325 = {S51820, 158, { &P15575,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA90205 = {S51821, 134, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; it-it; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML,like Gecko) Version/4.0 Safari/530.17*/
const Device UA142938 = {S51822, 124, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux 2.6;  en-gb; TOSHIBA_AC_AND_AZ) AppleWebKit/530.17(KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA258100 = {S51823, 134, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; en-gb; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML,like Gecko) Version/4.0 Safari/530.17*/
const Device UA258110 = {S51824, 132, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; fr-fr; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1*/
const Device UA258139 = {S51825, 132, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; nl-nl; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1*/
const Device UA258141 = {S51826, 132, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; en-gb; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1*/
const Device UA258162 = {S51827, 134, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; nl-nl; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML,like Gecko) Version/4.0 Safari/530.17*/
const Device UA258166 = {S51828, 134, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; en-us; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML,like Gecko) Version/4.0 Safari/530.17*/
const Device UA258171 = {S51829, 132, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; en-us; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1*/
const Device UA258222 = {S51830, 130, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; fr-; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1*/
const Device UA258242 = {S51831, 132, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; es-es; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1*/
const Device UA258247 = {S51832, 132, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; it-it; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1*/
const Device UA258249 = {S51833, 132, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; fr-; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML,like Gecko) Version/4.0 Safari/530.17*/
const Device UA258260 = {S51834, 130, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; nl-; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1*/
const Device UA258348 = {S51835, 134, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; fr-fr; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML,like Gecko) Version/4.0 Safari/530.17*/
const Device UA258240 = {S51836, 132, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; de-de; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1*/
const Device UA258360 = {S51837, 130, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; es-; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1*/
const Device UA258437 = {S51838, 132, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; cs-cz; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1*/
const Device UA258470 = {S51839, 132, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; sv-se; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Safari/533.1*/
const Device UA258495 = {S51840, 134, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; zh-cn; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML,like Gecko) Version/4.0 Safari/530.17*/
const Device UA258582 = {S51841, 134, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; es-es; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML,like Gecko) Version/4.0 Safari/530.17*/
const Device UA258807 = {S51842, 134, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; de-de; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML,like Gecko) Version/4.0 Safari/530.17*/
const Device UA258879 = {S51843, 132, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; en-; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML,like Gecko) Version/4.0 Safari/530.17*/
const Device UA309071 = {S51844, 94, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA505996 = {S51845, 121, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux 2.6;  de; TOSHIBA_AC_AND_AZ) AppleWebKit/530.17(KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA551976 = {S51846, 88, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA671826 = {S51847, 121, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux 2.6;  it; TOSHIBA_AC_AND_AZ) AppleWebKit/530.17(KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA730215 = {S51848, 156, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Bigdroid_2.0; zh-cn; I600/FRF85B; 1024*600) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17 ThunderSoft_BigDroid/2.0*/
const Device UA835346 = {S51849, 124, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux 2.6;  en-us; TOSHIBA_AC_AND_AZ) AppleWebKit/530.17(KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA1845492 = {S51850, 121, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux 2.6;  fr; TOSHIBA_AC_AND_AZ) AppleWebKit/530.17(KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA2128822 = {S51851, 121, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux 2.6;  nl; TOSHIBA_AC_AND_AZ) AppleWebKit/530.17(KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA2142587 = {S51852, 121, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux 2.6;  es; TOSHIBA_AC_AND_AZ) AppleWebKit/530.17(KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA2142615 = {S51853, 164, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Bigdroid_2.0; zh-cn; SMB-1002-WIFI/FRF91; 1024*600) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17 ThunderSoft_BigDroid/2.0*/
const Device UA2153960 = {S51854, 127, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux ; en-US) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.4 en-US;L6200 Version/4.0 Safari/531.2+*/
const Device UA2284599 = {S51855, 162, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Bigdroid_2.0; zh-cn; SMB-1002-3G/FRF91; 1024*600) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17 ThunderSoft_BigDroid/2.0*/
const Device UA2374946 = {S51856, 127, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux ; fr-CA) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.4 fr-CA;L6200 Version/4.0 Safari/531.2+*/
const Device UA2375681 = {S51857, 127, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux ; en-CA) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.4 en-CA;L6200 Version/4.0 Safari/531.2+*/
const Device UA2582728 = {S51858, 155, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Cosmote_My_mini_Tab 4.0.4; ro-ro; Cosmote_My_mini_Tab Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2802117 = {S51859, 121, { &P15364,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux 2.6;  ru; TOSHIBA_AC_AND_AZ) AppleWebKit/530.17(KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA2579787 = {S51860, 139, { &P18456,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T699 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867401 = {S51861, 61, { &P18456,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T699 Build/JZO54K)*/
const Device UA401953 = {S51862, 118, { &P15364,&P5690,&P22455,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11*/
const Device UA833255 = {S51863, 125, { &P15364,&P5690,&P22455,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11*/
const Device UA2708253 = {S51864, 139, { &P17346,&P18117,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9B176 Safari/7534.48.3*/
const Device UA2717094 = {S51865, 141, { &P17346,&P18117,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9B206 Safari/7534.48.3*/
const Device UA2025029 = {S51866, 140, { &P15809,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC_S510b Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2232880 = {S51867, 146, { &P15809,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-hk; HTC_Rhyme_S510b Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2395645 = {S51868, 131, { &P15809,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC S510b Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2444777 = {S51869, 126, { &P15809,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC S510b Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2502481 = {S51870, 146, { &P15809,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-hk; HTC Rhyme S510b Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2545934 = {S51871, 140, { &P15809,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC S510b Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2693085 = {S51872, 68, { &P15809,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Rhyme S510b Build/IML74K)*/
const Device UA2836998 = {S51873, 137, { &P15809,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Rhyme S510b Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA252482 = {S51874, 129, { &P15364,&P7133,&P22413,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Comodo_Dragon/15.0.3.0 Chrome/15.0.874.102 Safari/535.2*/
const Device UA252487 = {S51875, 122, { &P15364,&P7133,&P22413,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.2 (KHTML, like Gecko) Comodo_Dragon/15.0.3.0 Chrome/15.0.874.102 Safari/535.2*/
const Device UA32992 = {S51876, 74, { &P14970,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-E3210L/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA698567 = {S51877, 81, { &P14970,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-E3210/E3210ODFKK1 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA777675 = {S51878, 82, { &P14970,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-E3210B/E3210BDOKJ1 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA616377 = {S51879, 141, { &P12280,&P18432,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2162964 = {S51880, 128, { &P12280,&P18432,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293*/
const Device UA2200463 = {S51881, 145, { &P12280,&P18432,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA2236236 = {S51882, 139, { &P12280,&P18432,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/6531.22.7 iLunascape/3165*/
const Device UA2261337 = {S51883, 130, { &P12280,&P18432,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293*/
const Device UA2356797 = {S51884, 143, { &P12280,&P18432,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2359199 = {S51885, 147, { &P12280,&P18432,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA2403726 = {S51886, 141, { &P12280,&P18432,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/6531.22.7 iLunascape/3195*/
const Device UA2434681 = {S51887, 141, { &P12280,&P18432,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/6531.22.7 iLunascape/3200*/
const Device UA2437106 = {S51888, 130, { &P12280,&P18432,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293*/
const Device UA2460900 = {S51889, 143, { &P12280,&P18432,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2486700 = {S51890, 141, { &P12280,&P18432,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari/6531.22.7 iLunascape/3200*/
const Device UA2487317 = {S51891, 147, { &P12280,&P18432,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA2557214 = {S51892, 141, { &P12280,&P18432,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X) SparqEye/1.2.2 AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A293 Safari/6531.22.7*/
const Device UA428926 = {S51893, 135, { &P15364,&P17817,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.11 (KHTML, like Gecko) Ubuntu/10.10 Chromium/17.0.963.79 Chrome/17.0.963.79 Safari/535.11*/
const Device UA795102 = {S51894, 137, { &P15364,&P17817,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.11 (KHTML, like Gecko) Ubuntu/10.10 Chromium/17.0.963.65 Chrome/17.0.963.65 Safari/535.11*/
const Device UA2508249 = {S51895, 156, { &P14540,&P5843,&P24676,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-01) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2794840 = {S51896, 160, { &P14540,&P5843,&P24676,&P18092 }};	/*Useragent - NokiaX2-01.1/5.0 (07.22) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-01.1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA23577 = {S51897, 117, { &P12165,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-S5230W/S523WXIJK1 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA50081 = {S51898, 133, { &P13257,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0; en-us; Droid Build/ERD15) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA62232 = {S51899, 135, { &P13257,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0.1; en-us; Droid Build/ESD56) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA62929 = {S51900, 133, { &P13257,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0; en-us; Droid Build/ESD20) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA68183 = {S51901, 138, { &P13257,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0; en-us; Droid Build/ESD20) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA89000 = {S51902, 130, { &P13257,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; en-us; sholes) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA540100 = {S51903, 134, { &P13257,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0; en-us; Droid Build/ESD20) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/ 530.17*/
const Device UA3065597 = {S51904, 145, { &P15201,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067486 = {S51905, 148, { &P15201,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094789 = {S51906, 148, { &P15201,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Z710e Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110913 = {S51907, 162, { &P15201,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G with Beats Audio Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA686199 = {S51908, 82, { &P18095,&P18094,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (S60; SymbOS; Opera Mobi/SYB-X; U; es-LA) Presto/2.10.254 Version/12.00*/
const Device UA1949525 = {S51909, 142, { &P19880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-R940 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2547531 = {S51910, 66, { &P19880,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-R940 Build/GINGERBREAD)*/
const Device UA427426 = {S51911, 122, { &P17346,&P18117,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Twitter for iPhone*/
const Device UA615688 = {S51912, 124, { &P17346,&P18117,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Twitter for iPhone*/
const Device UA30745 = {S51913, 65, { &P15364,&P7127,&P17213,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; en) Opera 8.01*/
const Device UA107814 = {S51914, 64, { &P15364,&P7127,&P17213,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; en) Opera 8.0*/
const Device UA159828 = {S51915, 34, { &P15364,&P7127,&P17213,&P18092 }};	/*Useragent - Opera/8.54 (Windows NT 5.0; U; en)*/
const Device UA692778 = {S51916, 34, { &P15364,&P7127,&P17213,&P18092 }};	/*Useragent - Opera/8.50 (Windows NT 5.0; U; en)*/
const Device UA723331 = {S51917, 34, { &P15364,&P7127,&P17213,&P18092 }};	/*Useragent - Opera/8.52 (Windows NT 5.0; U; en)*/
const Device UA757890 = {S51918, 65, { &P15364,&P7127,&P17213,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; en) Opera 8.02*/
const Device UA782310 = {S51919, 34, { &P15364,&P7127,&P17213,&P18092 }};	/*Useragent - Opera/8.01 (Windows NT 5.0; U; de)*/
const Device UA1884308 = {S51920, 34, { &P15364,&P7127,&P17213,&P18092 }};	/*Useragent - Opera/8.51 (Windows NT 5.0; U; en)*/
const Device UA1893295 = {S51921, 34, { &P15364,&P7127,&P17213,&P18092 }};	/*Useragent - Opera/8.53 (Windows NT 5.0; U; en)*/
const Device UA2472894 = {S51922, 33, { &P15364,&P7127,&P17213,&P18092 }};	/*Useragent - Opera/8.0 (Windows NT 5.0; U; en)*/
const Device UA2496554 = {S51923, 34, { &P15364,&P7127,&P17213,&P18092 }};	/*Useragent - Opera/8.00 (Windows NT 5.0; U; en)*/
const Device UA2504710 = {S51924, 86, { &P15364,&P7127,&P17213,&P18092 }};	/*Useragent - Opera 8.0 (Win 2000): Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; en) Opera 8.0*/
const Device UA2201316 = {S51925, 96, { &P15364,&P7133,&P22364,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:15.0) Gecko/X Firefox/15.1.1-x64 PaleMoon/15.1.1-x64*/
const Device UA2201704 = {S51926, 83, { &P15364,&P7133,&P22364,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X Firefox/15.1.1 PaleMoon/15.1.1*/
const Device UA2973463 = {S51927, 136, { &P18301,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS840PP Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975023 = {S51928, 137, { &P18301,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS840 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA1997231 = {S51929, 156, { &P14225,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Desire HD Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257940 = {S51930, 184, { &P14225,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Desire HD Build/JRO03R; CyanogenMod-JellyTime_R13_Unlucky_Edition2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2433997 = {S51931, 167, { &P14225,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Desire HD Build/JZO54K; CyanogenMod-JellyTime_R30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2692189 = {S51932, 169, { &P14225,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Desire HD Build/JRO03L; CyanogenMod-JellyTime_R10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2256949 = {S51933, 160, { &P21024,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; USCC_ALCATEL_one_touch_988 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2705608 = {S51934, 155, { &P21024,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ALCATEL_one_touch_988 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3085858 = {S51935, 84, { &P21024,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; USCC_ALCATEL_one_touch_988 Build/GINGERBREAD)*/
const Device UA617858 = {S51936, 79, { &P12280,&P17779,&P17370,&P18092 }};	/*Useragent - Opera/9.80 (iPhone; Opera Mini/5.0.X/22.414; U; en) Presto/2.5.25 Version/10.54*/
const Device UA618731 = {S51937, 65, { &P12280,&P17779,&P17370,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/iPhone;Opera Mini/5.0.X/886; U; ja)Presto/2.4.15*/
const Device UA619149 = {S51938, 69, { &P12280,&P17779,&P17370,&P18092 }};	/*Useragent - Opera/9.80 (iPhone; Opera Mini/5.0.0176/764; U; en) Presto/2.4.154.15*/
const Device UA2432170 = {S51939, 79, { &P12280,&P17779,&P17370,&P18092 }};	/*Useragent - Opera/9.80 (iPhone; Opera Mini/5.0/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA3064872 = {S51940, 128, { &P19783,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ME172V Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2406142 = {S51941, 155, { &P14635,&P5843,&P24673,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac3-00) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA1994162 = {S51942, 85, { &P18141,&P18597,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 1.80) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2*/
const Device UA2043053 = {S51943, 85, { &P18141,&P18597,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 1.81) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2*/
const Device UA243761 = {S51944, 121, { &P15364,&P7133,&P22451,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.1 Chrome/13.0.800.1 Safari/535.1*/
const Device UA243980 = {S51945, 114, { &P15364,&P7133,&P22451,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.1 Chrome/13.0.800.1 Safari/535.1*/
const Device UA641062 = {S51946, 178, { &P14017,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Optimus 2X Build/GRJ22; CyanogenMod-7.2.0; LG-P990 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024694 = {S51947, 153, { &P14017,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; Optimus 2X Build/GRJ22; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2437464 = {S51948, 150, { &P12879,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7510 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2516380 = {S51949, 150, { &P12879,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7510 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2534905 = {S51950, 149, { &P12879,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7510 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2635453 = {S51951, 147, { &P12879,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-; GT-P7510 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2712391 = {S51952, 150, { &P12879,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7510 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2670583 = {S51953, 148, { &P15201,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710e Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676678 = {S51954, 145, { &P15201,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677291 = {S51955, 148, { &P15201,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Z710e Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708557 = {S51956, 162, { &P15201,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G with Beats Audio Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735871 = {S51957, 145, { &P15201,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation 4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3028613 = {S51958, 113, { &P13035,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-02/06.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3030747 = {S51959, 115, { &P13035,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-02.1/06.52; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA2391134 = {S51960, 155, { &P14826,&P5843,&P24673,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac1-01) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA202839 = {S51961, 142, { &P18002,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; IM-A760S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA242668 = {S51962, 142, { &P18002,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; IM-A760S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA657353 = {S51963, 66, { &P18002,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; IM-A760S Build/GINGERBREAD)*/
const Device UA788574 = {S51964, 140, { &P18002,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; IM-A760S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019790 = {S51965, 66, { &P18002,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; IM-A760S Build/GINGERBREAD)*/
const Device UA2020857 = {S51966, 140, { &P18002,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; IM-A760S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2328254 = {S51967, 147, { &P18002,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; IM-A760S/770K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2197591 = {S51968, 115, { &P15364,&P5690,&P22462,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4*/
const Device UA2366415 = {S51969, 115, { &P15364,&P5690,&P22462,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4*/
const Device UA2385036 = {S51970, 122, { &P15364,&P5690,&P22462,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4*/
const Device UA2202410 = {S51971, 128, { &P20350,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M975 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2485864 = {S51972, 128, { &P20350,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M975 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2778671 = {S51973, 57, { &P20350,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; M975 Build/JRO03H)*/
const Device UA2385730 = {S51974, 103, { &P14523,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.4.0.105; en-us; 9800/6.4.0.105) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2428657 = {S51975, 103, { &P14523,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.223; en-us; 9800/6.6.0.223) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2466544 = {S51976, 103, { &P14523,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.207; en-us; 9800/6.6.0.207) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2467019 = {S51977, 103, { &P14523,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.4.0.120; en-us; 9800/6.4.0.120) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2482220 = {S51978, 103, { &P14523,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.236; en-us; 9800/6.6.0.236) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2504296 = {S51979, 103, { &P14523,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.212; en-us; 9800/6.6.0.212) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2553449 = {S51980, 103, { &P14523,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.249; en-us; 9800/6.6.0.249) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2555150 = {S51981, 103, { &P14523,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.246; en-us; 9800/6.6.0.246) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2556335 = {S51982, 103, { &P14523,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.195; en-us; 9800/6.6.0.195) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2594672 = {S51983, 103, { &P14523,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.4.0.210; en-us; 9800/6.4.0.210) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2597892 = {S51984, 103, { &P14523,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.241; en-us; 9800/6.6.0.241) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2653952 = {S51985, 103, { &P14523,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.200; en-us; 9800/6.6.0.200) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2654111 = {S51986, 103, { &P14523,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.248; en-us; 9800/6.6.0.248) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2654637 = {S51987, 101, { &P14523,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.4.0.64; en-us; 9800/6.4.0.64) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2696234 = {S51988, 103, { &P14523,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.4.0.142; en-us; 9800/6.4.0.142) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2762940 = {S51989, 103, { &P14523,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.216; en-us; 9800/6.6.0.216) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2985759 = {S51990, 103, { &P14523,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.4.0.211; en-us; 9800/6.4.0.211) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3086016 = {S51991, 101, { &P14523,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.4.0.99; en-us; 9800/6.4.0.99) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2753046 = {S51992, 137, { &P20146,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754331 = {S51993, 138, { &P20146,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2488408 = {S51994, 132, { &P20160,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2557994 = {S51995, 140, { &P18736,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2975018 = {S51996, 138, { &P22602,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-S738C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA1997078 = {S51997, 165, { &P18465,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Amazon Kindle Fire Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2044145 = {S51998, 158, { &P18465,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; Amazon Kindle Fire Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2044197 = {S51999, 165, { &P18465,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Amazon Kindle Fire Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2234194 = {S52000, 165, { &P18465,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Amazon Kindle Fire Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2710487 = {S52001, 164, { &P18465,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-CA; Amazon Kindle Fire Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA54799 = {S52002, 142, { &P15995,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Sprint APA7373KT Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2367224 = {S52003, 75, { &P17595,&P17296,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8) Presto/2.12.388 Version/12.11*/
const Device UA3024806 = {S52004, 111, { &P19747,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia206/03.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3028269 = {S52005, 113, { &P19747,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia206.1/03.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA2569595 = {S52006, 143, { &P21373,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F240K/10g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2570858 = {S52007, 143, { &P21373,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F240K/10f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583958 = {S52008, 143, { &P21373,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F240K/10h Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2596788 = {S52009, 139, { &P21373,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F240K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2672748 = {S52010, 143, { &P21373,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F240K/10l Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2678020 = {S52011, 143, { &P21373,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F240K/10n Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2713621 = {S52012, 143, { &P21373,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F240K/10o Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2793317 = {S52013, 61, { &P21373,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F240K Build/JZO54K)*/
const Device UA2877703 = {S52014, 143, { &P21373,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F240K/10r Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2741710 = {S52015, 145, { &P18114,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Transformer Prime TF201 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2870624 = {S52016, 145, { &P18114,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Transformer Prime TF201 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA56073 = {S52017, 106, { &P14738,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; 7 Mozart; Orange)*/
const Device UA66948 = {S52018, 95, { &P14738,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; T8697)*/
const Device UA66952 = {S52019, 104, { &P14738,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; 7 Mozart T8698)*/
const Device UA66953 = {S52020, 98, { &P14738,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; 7 Mozart)*/
const Device UA925756 = {S52021, 96, { &P14738,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Mozart)*/
const Device UA925875 = {S52022, 97, { &P14738,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; PD67100)*/
const Device UA2739818 = {S52023, 86, { &P17968,&P17979,&P18413,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/7.7.33426/29.3551; U; zh) Presto/2.8.119 Version/11.10*/
const Device UA2673799 = {S52024, 101, { &P15364,&P7127,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA236548 = {S52025, 67, { &P18254,&P7480,&P18245,&P19828 }};	/*Useragent - Mozilla/5.0 (compatible; TweetmemeBot/2.11; +http://tweetmeme.com/)*/
const Device UA900116 = {S52026, 41, { &P22796,&P17484,&P3625,&P18092 }};	/*Useragent - sam-r390 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA2388207 = {S52027, 136, { &P19985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-02E Build/SB140) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2437663 = {S52028, 136, { &P19985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-02E Build/SC110) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2571348 = {S52029, 136, { &P19985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-02E Build/S2130) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2698693 = {S52030, 58, { &P19985,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SH-02E Build/SC110)*/
const Device UA2739946 = {S52031, 136, { &P19985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-02E Build/S4100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2672371 = {S52032, 130, { &P16356,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2674209 = {S52033, 138, { &P16356,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675842 = {S52034, 137, { &P16356,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7000 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676442 = {S52035, 137, { &P16356,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676633 = {S52036, 138, { &P16356,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2706842 = {S52037, 138, { &P16356,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E160K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707912 = {S52038, 131, { &P16356,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M480W Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2719405 = {S52039, 138, { &P16356,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000B Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2226847 = {S52040, 138, { &P19856,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LS970 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2559240 = {S52041, 141, { &P23910,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30at Build/9.1.F.1.120) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2676333 = {S52042, 130, { &P22043,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P5113 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA903081 = {S52043, 142, { &P14969,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA909025 = {S52044, 138, { &P14969,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Desire S Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1954415 = {S52045, 143, { &P14969,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  HTC Desire S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA1990663 = {S52046, 139, { &P14969,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  Desire S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA1993399 = {S52047, 142, { &P14969,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2390481 = {S52048, 143, { &P14969,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire SV Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2230322 = {S52049, 126, { &P22818,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; M1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2525007 = {S52050, 55, { &P22818,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; M1 Build/JRO03H)*/
const Device UA2763571 = {S52051, 133, { &P22818,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; M1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1991584 = {S52052, 142, { &P19997,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Lenovo P700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025227 = {S52053, 133, { &P19997,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo P700 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2027665 = {S52054, 64, { &P19997,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Lenovo P700 Build/IML74K)*/
const Device UA2030512 = {S52055, 128, { &P19997,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo P700 Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2257817 = {S52056, 141, { &P19997,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; Lenovo P700i; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2418203 = {S52057, 143, { &P19997,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lenovo P700i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2454813 = {S52058, 65, { &P19997,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo P700i Build/IMM76D)*/
const Device UA2464555 = {S52059, 134, { &P19997,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo P700i Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2956771 = {S52060, 136, { &P19997,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; P700i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA31286 = {S52061, 38, { &P12413,&P17483,&P4223,&P18092 }};	/*Useragent - LGE-VM510 NetFront/3.5.1 (GUI) MMP/2.0*/
const Device UA4300 = {S52062, 138, { &P12110,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; nl-nl; HTC Legend Build/ERD72) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA4304 = {S52063, 157, { &P12110,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; de-at; HTC Legend 2.05.177.2 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA24905 = {S52064, 149, { &P12110,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; fr-fr; HTC Legend 1.32.163.1 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA24922 = {S52065, 149, { &P12110,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; fr-fr; HTC Legend 1.23.163.1 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA24995 = {S52066, 149, { &P12110,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; fr-fr; HTC Legend 1.32.163.2 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA24998 = {S52067, 149, { &P12110,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-gb; HTC Legend 1.32.161.4 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA25223 = {S52068, 149, { &P12110,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-gb; HTC Legend 1.23.161.1 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA25235 = {S52069, 157, { &P12110,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; de-at; HTC Legend 1.31.177.2 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA25237 = {S52070, 157, { &P12110,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Legend 1.32.161.4 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA25238 = {S52071, 157, { &P12110,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Legend 2.05.161.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA25244 = {S52072, 149, { &P12110,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; de-at; HTC Legend 1.31.177.2 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA25246 = {S52073, 146, { &P12110,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; HTC Legend Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA25253 = {S52074, 157, { &P12110,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Legend 1.23.161.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA25316 = {S52075, 145, { &P12110,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1 Legend; zh-cn; HTC Legend Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA25320 = {S52076, 157, { &P12110,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; HTC Legend 2.05.163.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA25324 = {S52077, 157, { &P12110,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; HTC Legend 1.32.163.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA26795 = {S52078, 138, { &P12110,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-gb; HTC Legend Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA36594 = {S52079, 157, { &P12110,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; HTC Legend 1.23.163.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA388185 = {S52080, 140, { &P12110,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; HTC Legend Build/cupcake) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA659820 = {S52081, 68, { &P12110,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; HTC Legend Build/ERE27)*/
const Device UA679827 = {S52082, 60, { &P12110,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1; HTC Legend Build/ERD79)*/
const Device UA2021452 = {S52083, 124, { &P12110,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; HTC Legend Build/ERD79) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA899125 = {S52084, 139, { &P18216,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Sony Tablet P Build/TISU0085) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA911627 = {S52085, 139, { &P18216,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Sony Tablet P Build/TISU0093) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2403115 = {S52086, 139, { &P18216,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; Sony Tablet P Build/TISU0124) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2436231 = {S52087, 139, { &P18216,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Sony Tablet P Build/TISU0144) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951470 = {S52088, 68, { &P18216,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Sony Tablet P Build/TISU0144)*/
const Device UA2876904 = {S52089, 140, { &P18449,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST27i Build/6.2.A.1.100) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2899033 = {S52090, 140, { &P18449,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST27a Build/6.2.A.1.100) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2673031 = {S52091, 138, { &P18645,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530U Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673189 = {S52092, 138, { &P18645,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673222 = {S52093, 137, { &P18645,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-L710 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676505 = {S52094, 138, { &P18645,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I747M Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676581 = {S52095, 138, { &P18645,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530C Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676943 = {S52096, 138, { &P18645,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210K Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676999 = {S52097, 145, { &P18645,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2706677 = {S52098, 138, { &P18645,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E210L Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2560716 = {S52099, 142, { &P21150,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ARCHOS 80 TITANIUM Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2749407 = {S52100, 71, { &P21150,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ARCHOS 80 TITANIUM Build/JRO03H)*/
const Device UA2566630 = {S52101, 139, { &P19902,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530M Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567130 = {S52102, 132, { &P19902,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530M Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2974067 = {S52103, 138, { &P18280,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC VLE_U Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974919 = {S52104, 138, { &P18280,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975815 = {S52105, 138, { &P18280,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023140 = {S52106, 138, { &P18280,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC VLE_U Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2507636 = {S52107, 152, { &P18506,&P5043,&P24676,&P18092 }};	/*Useragent - Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia305) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2753272 = {S52108, 145, { &P12972,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T989 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754709 = {S52109, 137, { &P12972,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755387 = {S52110, 138, { &P12972,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2676073 = {S52111, 139, { &P17172,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2712463 = {S52112, 133, { &P17172,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Transformer Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735316 = {S52113, 127, { &P17172,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TF101 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2672770 = {S52114, 145, { &P22641,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; SonyC1504 Build/11.3.A.0.47) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2751518 = {S52115, 141, { &P22641,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; C1504 Build/11.3.A.0.47) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768978 = {S52116, 145, { &P22641,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; SonyC1504 Build/11.3.A.0.58) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937440 = {S52117, 145, { &P22641,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; SonyC1504 Build/11.3.A.0.56) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956418 = {S52118, 63, { &P22641,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; C1504 Build/11.3.A.0.47)*/
const Device UA2972388 = {S52119, 145, { &P22641,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; SonyC1504 Build/11.3.A.2.13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3015367 = {S52120, 145, { &P22641,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; SonyC1504 Build/11.3.A.0.59) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3019513 = {S52121, 141, { &P22641,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; C1504 Build/11.3.A.2.13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057072 = {S52122, 141, { &P22641,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; C1504 Build/11.3.A.0.59) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057496 = {S52123, 141, { &P22641,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-mx; C1504 Build/11.3.A.0.58) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2533235 = {S52124, 165, { &P13064,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SAMSUNG-SGH-I727 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2535477 = {S52125, 164, { &P13064,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I727 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2969944 = {S52126, 176, { &P13064,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I727 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/skyrocket*/
const Device UA33065 = {S52127, 152, { &P12321,&P5208,&P4223,&P18092 }};	/*Useragent - Mozilla/4.0 (BREW 3.1.5; U; en-us; Sanyo; NetFront/3.5.1/AMB) Boost SCP6760, Mozilla/4.0 (BREW 3.1.5; U; en-us; Sanyo; NetFront/3.5.1/AMB) Boost SCP6760*/
const Device UA54557 = {S52128, 75, { &P12321,&P5208,&P4223,&P18092 }};	/*Useragent - Mozilla/4.0 (BREW 3.1.5; U; en-us; Sanyo; NetFront/3.5.1/AMB) Boost SCP6760*/
const Device UA55211 = {S52129, 77, { &P12321,&P5208,&P4223,&P18092 }};	/*Useragent - Mozilla/4.0 (BREW 3.1.5; U; en-us; Sanyo; NetFront/3.5.1/AMB) Sprint SCP-6760*/
const Device UA2215291 = {S52130, 77, { &P12321,&P5208,&P4223,&P18092 }};	/*Useragent - Mozilla/4.0 (BREW 3.1.5; U; en-us; Sanyo; NetFront/3.5.1/AMB) Sanyo  SCP-6760*/
const Device UA55302 = {S52131, 133, { &P12955,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; Blade Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55310 = {S52132, 137, { &P12955,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; hu-hu; ZTE-Blade Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA181982 = {S52133, 133, { &P12955,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; Blade Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA235501 = {S52134, 133, { &P12955,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Blade Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA249704 = {S52135, 133, { &P12955,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Blade Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA571472 = {S52136, 137, { &P12955,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; ZTE-Blade Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA573891 = {S52137, 57, { &P12955,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; Blade Build/GRJ22)*/
const Device UA705771 = {S52138, 121, { &P12955,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Blade Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA743341 = {S52139, 133, { &P12955,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; Blade Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018592 = {S52140, 57, { &P12955,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; Blade Build/FRG83)*/
const Device UA2026640 = {S52141, 126, { &P12955,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Blade Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2566691 = {S52142, 131, { &P15919,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566755 = {S52143, 131, { &P15919,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2634379 = {S52144, 132, { &P15919,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100B Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2752946 = {S52145, 136, { &P18713,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P769 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754326 = {S52146, 136, { &P18713,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P768 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755268 = {S52147, 136, { &P18713,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P760 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA635181 = {S52148, 131, { &P20967,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID7033 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2671098 = {S52149, 130, { &P19856,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LS970 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676035 = {S52150, 137, { &P19856,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LS970 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA437256 = {S52151, 42, { &P14188,&P5129,&P24665,&P18092 }};	/*Useragent - Nokia5233/UC Browser8.0.4.121/50/444/UCWEB*/
const Device UA440219 = {S52152, 30, { &P14188,&P5129,&P24665,&P18092 }};	/*Useragent - Nokia5233/UCWEB8.0.3.99/50/800*/
const Device UA2808264 = {S52153, 43, { &P14188,&P5129,&P24665,&P18092 }};	/*Useragent - Nokia 5233/UC Browser8.0.4.121/50/351/UCWEB*/
const Device UA63355 = {S52154, 77, { &P11631,&P5026,&P3410,&P18092 }};	/*Useragent - BlackBerry8110/4.5.0.182 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102*/
const Device UA48710 = {S52155, 150, { &P12787,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; el-gr; ALCATEL_one_touch_908 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA75023 = {S52156, 151, { &P12787,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ALCATEL_one_touch_908S Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1946895 = {S52157, 151, { &P12787,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ALCATEL_one_touch_908A Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019899 = {S52158, 74, { &P12787,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; ALCATEL_one_touch_908 Build/FRG83G)*/
const Device UA1972502 = {S52159, 131, { &P18708,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; U30GT-H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2024312 = {S52160, 60, { &P18708,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; U30GT-H Build/IMM76D)*/
const Device UA2411372 = {S52161, 152, { &P18708,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U30GT-H Build/IMM76D; 1280*752) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2029203 = {S52162, 137, { &P18283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-I9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2386040 = {S52163, 137, { &P18283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; BP-I9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2554025 = {S52164, 134, { &P18283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; I9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2844860 = {S52165, 61, { &P18283,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; BP-I9300 Build/GRK39F)*/
const Device UA2043759 = {S52166, 141, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Grand X In Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2206818 = {S52167, 151, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Grand X In Build/GXI_TATV1.0.0B03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2206817 = {S52168, 151, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; Grand X In Build/GXI_TTRV1.0.0B03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2206819 = {S52169, 151, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Grand X In Build/GXI_TATV1.0.0B02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2387510 = {S52170, 151, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; Grand X In Build/GXI_TTRV1.0.0B04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423781 = {S52171, 151, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Grand X In Build/GXI_TPLV1.0.0B04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467138 = {S52172, 151, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; Grand X In Build/GXI_THUV1.0.0B04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2530290 = {S52173, 151, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ro-ro; Grand X In Build/GXI_TROV1.0.0B01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555364 = {S52174, 151, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sk-sk; Grand X In Build/GXI_OSKV1.0.0B02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597862 = {S52175, 151, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Grand X In Build/GXI_TMKV1.0.0B04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658019 = {S52176, 148, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Grand X In Build/GXI_V1.0.0B02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659376 = {S52177, 148, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Grand X In Build/GXI_V1.0.0B01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2731673 = {S52178, 152, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sk-sk; Grand X In Build/GXI_TMSKV1.0.0B01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2737746 = {S52179, 151, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; Grand X In Build/GXI_TPLV1.0.0B06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2737871 = {S52180, 151, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; Grand X In Build/GXI_TTRV1.0.0B06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781672 = {S52181, 151, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; Grand X In Build/GXI_THUV1.0.0B06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2792412 = {S52182, 151, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Grand X In Build/GXI_GERV1.0.0B01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2794075 = {S52183, 151, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Grand X In Build/GXI_GERV1.0.0B02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892023 = {S52184, 151, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Grand X In Build/GXI_TMKV1.0.0B06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2935144 = {S52185, 148, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; Grand X In Build/GXI_V1.0.0B04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984491 = {S52186, 151, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; Grand X In Build/GXI_TSKV1.0.0B02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996892 = {S52187, 152, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sk-sk; Grand X In Build/GXI_TMSKV1.0.0B02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3071263 = {S52188, 151, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; Grand X In Build/GXI_TSKV1.0.0B03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3087108 = {S52189, 152, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Grand X In Build/GXI_ATBPV1.0.0B01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3106000 = {S52190, 151, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Grand X In Build/GXI_ESPV1.0.0B01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121132 = {S52191, 148, { &P18461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; Grand X In Build/GXI_V1.0.0B05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3026768 = {S52192, 134, { &P15767,&P17146,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9A405 Safari/7534.48.3*/
const Device UA3030177 = {S52193, 132, { &P15767,&P17146,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9A334 Safari/7534.48.3*/
const Device UA633403 = {S52194, 82, { &P17595,&P17295,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.10.229 Version/11.61*/
const Device UA2469446 = {S52195, 77, { &P15364,&P16110,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (IE 11.0; Windows NT 6.2; WOW64; Trident/7.0; rv:11.0) like Gecko*/
const Device UA2472038 = {S52196, 70, { &P15364,&P16110,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (IE 11.0; Windows NT 6.2; Trident/7.0; rv:11.0) like Gecko*/
const Device UA2472060 = {S52197, 82, { &P15364,&P16110,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (IE 11.0; Windows NT 6.2; Win64; x64; Trident/7.0; rv:11.0) like Gecko*/
const Device UA2897410 = {S52198, 50, { &P15364,&P16110,&P21461,&P18092 }};	/*Useragent - Windows-RSS-Platform/2.0 (IE 11.0; Windows NT 6.2)*/
const Device UA2937144 = {S52199, 68, { &P15364,&P16110,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; Trident/7.0; rv:11.0) like Gecko*/
const Device UA2937640 = {S52200, 86, { &P15364,&P16110,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; Trident/7.0; WebView/1.0; rv:11.0) like Gecko*/
const Device UA2944550 = {S52201, 139, { &P15364,&P16110,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; Trident/7.0; rv:11.0; MSN 9.0;MSN 9.1;MSN 9.6;MSN 10.0;MSN 10.2;MSN 10.5; MSNbMSNI; MSNmen-us; MSNcOTH)*/
const Device UA3011512 = {S52202, 97, { &P15364,&P16110,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (AOL 9.7; AOLBuild 4343.1028; Windows NT 6.2; WOW64; Trident/7.0; rv:11.0) like Gecko*/
const Device UA2407100 = {S52203, 133, { &P18680,&P18110,&P19804,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Safari/535.19 Silk-Accelerated=true*/
const Device UA3065728 = {S52204, 137, { &P22970,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065984 = {S52205, 138, { &P22970,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083238 = {S52206, 138, { &P22970,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2886777 = {S52207, 112, { &P18506,&P5043,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia305/05.92; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA2556703 = {S52208, 136, { &P17595,&P17297,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.0.2000 Chrome/21.0.1180.75 Safari/537.1*/
const Device UA2567067 = {S52209, 136, { &P17595,&P17297,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/4.0.3.6000 Chrome/21.0.1180.75 Safari/537.1*/
const Device UA2768934 = {S52210, 138, { &P17595,&P17297,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.22 (KHTML, like Gecko) Maxthon/4.0.4.3000 Chrome/25.0.1364.99 Safari/537.22*/
const Device UA3027729 = {S52211, 138, { &P17595,&P17297,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.22 (KHTML, like Gecko) Maxthon/4.0.6.3000 Chrome/25.0.1364.99 Safari/537.22*/
const Device UA99491 = {S52212, 56, { &P17595,&P18120,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Macintosh; PPC Mac OS X; U; en) Presto/2.1.1*/
const Device UA104189 = {S52213, 58, { &P17595,&P18120,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Macintosh; Intel Mac OS X; U; en) Presto/2.1.1*/
const Device UA111487 = {S52214, 56, { &P17595,&P18120,&P18337,&P18092 }};	/*Useragent - Opera/9.62 (Macintosh; PPC Mac OS X; U; en) Presto/2.1.1*/
const Device UA242006 = {S52215, 58, { &P17595,&P18120,&P18337,&P18092 }};	/*Useragent - Opera/9.60 (Macintosh; Intel Mac OS X; U; de) Presto/2.1.1*/
const Device UA2208747 = {S52216, 58, { &P17595,&P18120,&P18337,&P18092 }};	/*Useragent - Opera/9.61 (Macintosh; Intel Mac OS X; U; de) Presto/2.1.1*/
const Device UA152514 = {S52217, 125, { &P17595,&P17296,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5*/
const Device UA166066 = {S52218, 125, { &P17595,&P17296,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5*/
const Device UA176010 = {S52219, 125, { &P17595,&P17296,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5*/
const Device UA224424 = {S52220, 125, { &P17595,&P17296,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5*/
const Device UA260050 = {S52221, 125, { &P17595,&P17296,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5*/
const Device UA265363 = {S52222, 125, { &P17595,&P17296,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5*/
const Device UA459868 = {S52223, 125, { &P17595,&P17296,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.30 Safari/532.5*/
const Device UA831208 = {S52224, 124, { &P17595,&P17296,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/4.0.202.0 Safari/532.0*/
const Device UA859557 = {S52225, 124, { &P17595,&P17296,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/4.0.211.2 Safari/532.0*/
const Device UA2796461 = {S52226, 143, { &P22395,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6606 Build/10.1.1.B.0.166) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2753816 = {S52227, 162, { &P15575,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; GT-I9100 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2753823 = {S52228, 162, { &P15575,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.3; GT-I9100 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2753928 = {S52229, 162, { &P15575,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.4; GT-I9100 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2772388 = {S52230, 162, { &P15575,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-I9100 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2796199 = {S52231, 163, { &P15575,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; SGH-S959G Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2867353 = {S52232, 163, { &P15575,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; GT-I9100T Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2957443 = {S52233, 163, { &P15575,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-I9100G Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2958650 = {S52234, 163, { &P15575,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.3; GT-I9100M Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA3088452 = {S52235, 163, { &P15575,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.3; GT-I9100T Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453*/
const Device UA925308 = {S52236, 125, { &P15364,&P17285,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/21.0.1180.60) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.60 Safari/537.1*/
const Device UA1997182 = {S52237, 132, { &P15364,&P17285,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/21.0.1180.60) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.60 Safari/537.1*/
const Device UA2001933 = {S52238, 125, { &P15364,&P17285,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/21.0.1180.83) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1*/
const Device UA2005456 = {S52239, 125, { &P15364,&P17285,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/21.0.1180.79) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.79 Safari/537.1*/
const Device UA2006443 = {S52240, 132, { &P15364,&P17285,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/21.0.1180.79) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.79 Safari/537.1*/
const Device UA2011091 = {S52241, 125, { &P15364,&P17285,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/21.0.1180.77) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.77 Safari/537.1*/
const Device UA2015627 = {S52242, 89, { &P15364,&P17285,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/21.0.1180.89)*/
const Device UA2015635 = {S52243, 96, { &P15364,&P17285,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/21.0.1180.89)*/
const Device UA2040804 = {S52244, 132, { &P15364,&P17285,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/21.0.1180.89) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2041363 = {S52245, 209, { &P15364,&P17285,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; GTB7.4; chromeframe/21.0.1180.89; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA2042005 = {S52246, 172, { &P15364,&P17285,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/21.0.1180.89; GTB7.4; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)*/
const Device UA2042073 = {S52247, 125, { &P15364,&P17285,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/21.0.1180.89) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2178145 = {S52248, 188, { &P15364,&P17285,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; chromeframe/21.0.1180.79; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MS-RTC LM 8; InfoPath.3)*/
const Device UA2200772 = {S52249, 194, { &P15364,&P17285,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; chromeframe/21.0.1180.89; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)*/
const Device UA2204922 = {S52250, 194, { &P15364,&P17285,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/21.0.1180.89; SLCC1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.5.21022; InfoPath.2; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA2651022 = {S52251, 185, { &P15364,&P17285,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/21.0.1180.89; GTB7.4; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729)*/
const Device UA2689832 = {S52252, 231, { &P15364,&P17285,&P21912,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/21.0.1180.89; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 1.1.4322; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.1; .NET4.0C; .NET4.0E)*/
const Device UA2737194 = {S52253, 97, { &P17595,&P18207,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:21.0) Gecko/X Firefox/21.0 AlexaToolbar/alxf-2.18*/
const Device UA738273 = {S52254, 114, { &P15364,&P18145,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; FreeBSD i386; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA760715 = {S52255, 116, { &P15364,&P18145,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; FreeBSD x86_64; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA2492641 = {S52256, 133, { &P18680,&P18110,&P19837,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Safari/535.19 Silk-Accelerated=true*/
const Device UA2944320 = {S52257, 142, { &P19763,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; C6602 Build/10.3.A.0.423) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3028135 = {S52258, 66, { &P19763,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; C6602 Build/10.3.1.A.0.244)*/
const Device UA3028234 = {S52259, 144, { &P19763,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; C6602 Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079729 = {S52260, 147, { &P19763,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; SonyL36h Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2462489 = {S52261, 145, { &P19770,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; ALCATEL ONE TOUCH 5020D Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2737625 = {S52262, 154, { &P19770,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ALCATEL ONE TOUCH 5020E Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2795987 = {S52263, 154, { &P19770,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ALCATEL ONE TOUCH 5020D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2797670 = {S52264, 154, { &P19770,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; ALCATEL ONE TOUCH 5020A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2866116 = {S52265, 154, { &P19770,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ALCATEL ONE TOUCH 5020X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2935725 = {S52266, 154, { &P19770,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ALCATEL ONE TOUCH 5020W Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA183007 = {S52267, 139, { &P12681,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Nexus S 4G Build/GRJ06D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA188783 = {S52268, 138, { &P12681,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Nexus S 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA233415 = {S52269, 138, { &P12681,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Nexus S 4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA245164 = {S52270, 62, { &P12681,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; Nexus S 4G Build/GWK74)*/
const Device UA249366 = {S52271, 138, { &P12681,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus S 4G Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250053 = {S52272, 138, { &P12681,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus S 4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA449511 = {S52273, 62, { &P12681,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; Nexus S 4G Build/GRJ22)*/
const Device UA562055 = {S52274, 62, { &P12681,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Nexus S 4G Build/GRJ90)*/
const Device UA861092 = {S52275, 138, { &P12681,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Nexus S 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1996933 = {S52276, 137, { &P12681,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; xx-xx; Nexus S 4G Build/GRJ90) AppleWebKit/533.1  (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA496746 = {S52277, 102, { &P15364,&P7127,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9.1.9) Gecko/X Firefox/3.5.9 WinNT-EVI 23.04.2010*/
const Device UA2125897 = {S52278, 75, { &P15364,&P7127,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; de-DE; rv:1.8) Gecko/X Firefox/3.5*/
const Device UA9222 = {S52279, 108, { &P15197,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-GT-B5310/1.0 SHP/VPP/R5 Dolfin/1.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA19952 = {S52280, 109, { &P15197,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-GT-B5310U/1.0 SHP/VPP/R5 Dolfin/1.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA27293 = {S52281, 116, { &P15197,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-GT-B5310R/B5310UXJB5 SHP/VPP/R5 Dolfin/1.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA49355 = {S52282, 109, { &P15197,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-GT-B5310L/1.0 SHP/VPP/R5 Dolfin/1.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA361021 = {S52283, 144, { &P12482,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T959 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA366432 = {S52284, 139, { &P12482,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T959 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA656525 = {S52285, 139, { &P12482,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA667599 = {S52286, 61, { &P12482,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SGH-T959 Build/IML74K)*/
const Device UA683277 = {S52287, 144, { &P12482,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; SGH-T959 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA780527 = {S52288, 139, { &P12482,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA807704 = {S52289, 139, { &P12482,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA854517 = {S52290, 139, { &P12482,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959 Build/IMM30B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1991667 = {S52291, 139, { &P12482,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027290 = {S52292, 138, { &P12482,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959 Build/IMM26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA399553 = {S52293, 107, { &P15364,&P5690,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.0 Safari/533.3*/
const Device UA2540815 = {S52294, 107, { &P15364,&P5690,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.1 Safari/533.3*/
const Device UA2734423 = {S52295, 165, { &P22595,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-ca; SAMSUNG SGH-M919V Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2741731 = {S52296, 130, { &P15882,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P5110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755347 = {S52297, 130, { &P15882,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P5110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2663704 = {S52298, 139, { &P22607,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S5303 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2740080 = {S52299, 140, { &P22607,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT-S5303B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955874 = {S52300, 61, { &P22607,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-S5303 Build/IMM76D)*/
const Device UA26703 = {S52301, 108, { &P12580,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2121/E2121JBJF1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA51371 = {S52302, 110, { &P12580,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2121B/E2121BXXKC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA60816 = {S52303, 110, { &P12580,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2121N/E2121NJCJJ1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA62476 = {S52304, 102, { &P12580,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2121L/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA460854 = {S52305, 93, { &P15364,&P22187,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.1.9) Gecko/X Fedora/3.5.9-1.fc11 Firefox/3.5.9*/
const Device UA924435 = {S52306, 134, { &P20274,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IdeaTab_A1107 Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2448963 = {S52307, 63, { &P20274,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; IdeaTab_A1107 Build/MR1)*/
const Device UA2563861 = {S52308, 145, { &P21507,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SonyC1605 Build/11.1.A.0.64) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2668988 = {S52309, 141, { &P21507,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C1605 Build/11.1.A.0.68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2698703 = {S52310, 141, { &P21507,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C1605 Build/11.1.A.0.64) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2780474 = {S52311, 63, { &P21507,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; C1605 Build/11.1.A.0.68)*/
const Device UA2570009 = {S52312, 158, { &P15575,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2772905 = {S52313, 157, { &P15575,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2774284 = {S52314, 158, { &P15575,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2782830 = {S52315, 158, { &P15575,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2937875 = {S52316, 158, { &P15575,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2356415 = {S52317, 131, { &P20293,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID7034 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA24726 = {S52318, 74, { &P13540,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-E2152L/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62730 = {S52319, 80, { &P13540,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-E2152/E2152DDJH1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA202351 = {S52320, 143, { &P16744,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; SHW-M130L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA211469 = {S52321, 143, { &P16744,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; SHW-M130L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA676788 = {S52322, 143, { &P16744,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M130K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA682784 = {S52323, 143, { &P16744,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SHW-M130K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA695605 = {S52324, 143, { &P16744,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SHW-M130K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA702254 = {S52325, 143, { &P16744,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHW-M130L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018761 = {S52326, 67, { &P16744,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SHW-M130L Build/GINGERBREAD)*/
const Device UA2019600 = {S52327, 67, { &P16744,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M130K Build/GINGERBREAD)*/
const Device UA2019772 = {S52328, 67, { &P16744,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M130L Build/GINGERBREAD)*/
const Device UA2021279 = {S52329, 67, { &P16744,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHW-M130K Build/GINGERBREAD)*/
const Device UA427408 = {S52330, 69, { &P15767,&P18117,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.9B176 (iPad; U; CPU OS 5_1 like Mac OS X; en_us)*/
const Device UA615715 = {S52331, 71, { &P15767,&P18117,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.9B206 (iPad; U; CPU OS 5_1_1 like Mac OS X; en_us)*/
const Device UA97119 = {S52332, 139, { &P17346,&P17134,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2658305 = {S52333, 137, { &P17346,&P17134,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 30 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2879636 = {S52334, 133, { &P18465,&P18110,&P23482,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Safari/535.19 Silk-Accelerated=false*/
const Device UA73970 = {S52335, 142, { &P13964,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I5510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA678561 = {S52336, 142, { &P13964,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I5510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA691862 = {S52337, 143, { &P13964,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-I5510T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022318 = {S52338, 66, { &P13964,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I5510 Build/GINGERBREAD)*/
const Device UA2026483 = {S52339, 143, { &P13964,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-I5510L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2775809 = {S52340, 143, { &P13964,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; GT-I5510L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2515962 = {S52341, 76, { &P15364,&P18110,&P20456,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:18.0) Gecko/X Firefox/18.0 SeaMonkey/2.15.2*/
const Device UA2985502 = {S52342, 63, { &P15364,&P18110,&P20456,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:18.0) Gecko/X SeaMonkey/2.15.1*/
const Device UA2566707 = {S52343, 136, { &P17968,&P18149,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; cm_tenderloin Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567574 = {S52344, 135, { &P17968,&P18149,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; cm_tenderloin Build/GWK74) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2257929 = {S52345, 173, { &P17835,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Barnes & Noble Nook Tablet Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257967 = {S52346, 173, { &P17835,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Barnes & Noble Nook Tablet Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258056 = {S52347, 173, { &P17835,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Barnes & Noble Nook Tablet Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2487259 = {S52348, 157, { &P17835,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; NookTablet Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3115785 = {S52349, 86, { &P17968,&P17979,&P24498,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/16.0.1212/31.1475; U; en) Presto/2.8.119 Version/11.10*/
const Device UA360475 = {S52350, 139, { &P13999,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; NexusHD2 Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA366141 = {S52351, 139, { &P13999,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; NexusHD2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA369180 = {S52352, 138, { &P13999,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC HD2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA669124 = {S52353, 138, { &P13999,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC HD2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA689285 = {S52354, 61, { &P13999,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; NexusHD2 Build/IML74K)*/
const Device UA856438 = {S52355, 138, { &P13999,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC HD2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1950283 = {S52356, 138, { &P13999,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; HTC HD2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2023444 = {S52357, 138, { &P13999,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC HD2 Build/GRK39F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030709 = {S52358, 60, { &P13999,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC HD2 Build/IMM76I)*/
const Device UA2033346 = {S52359, 61, { &P13999,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; NexusHD2 Build/IMM76L)*/
const Device UA2355637 = {S52360, 145, { &P13999,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC HD2 EvoHd2 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2364269 = {S52361, 138, { &P13999,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; NexusHD2 Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2379200 = {S52362, 134, { &P13999,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; zh-tw; Hd2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2473422 = {S52363, 139, { &P13999,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; NexusHD2 Build/GRK39F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2475678 = {S52364, 140, { &P13999,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4RC; en-us; Evo Hd2 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2565553 = {S52365, 145, { &P20006,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyLT30a Build/9.1.A.0.489) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583263 = {S52366, 141, { &P20006,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LT30a Build/9.1.A.0.489) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2704221 = {S52367, 63, { &P20006,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LT30a Build/9.1.A.0.489)*/
const Device UA2710304 = {S52368, 145, { &P20006,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyLT30a Build/9.1.A.1.140) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752303 = {S52369, 141, { &P20006,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LT30a Build/9.1.A.1.140) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2861804 = {S52370, 145, { &P20006,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyLT30a Build/9.1.A.0.492) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3017444 = {S52371, 145, { &P20006,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyLT30a Build/9.1.A.1.141) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2773065 = {S52372, 143, { &P12280,&P17146,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9A405 Safari/7534.48.3*/
const Device UA2783580 = {S52373, 143, { &P12280,&P17146,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9A406 Safari/7534.48.3*/
const Device UA2783674 = {S52374, 141, { &P12280,&P17146,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9A334 Safari/7534.48.3*/
const Device UA397793 = {S52375, 157, { &P15759,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Vision Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA638333 = {S52376, 148, { &P15759,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Vision Build/GRI40; ILWT-CM7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA169800 = {S52377, 131, { &P18061,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; sdk Build/GSI11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA174775 = {S52378, 131, { &P18061,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA395055 = {S52379, 144, { &P18061,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; google_sdk Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA523047 = {S52380, 55, { &P18061,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; sdk Build/GRI34)*/
const Device UA658285 = {S52381, 141, { &P18061,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; rk29sdk Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA871416 = {S52382, 124, { &P18061,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA882716 = {S52383, 129, { &P18061,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; sdk Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1901337 = {S52384, 68, { &P18061,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; google_sdk Build/GINGERBREAD)*/
const Device UA2018337 = {S52385, 55, { &P18061,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.1; sdk Build/GSI11)*/
const Device UA2018412 = {S52386, 138, { &P18061,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; google_sdk Build/GSI11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021873 = {S52387, 65, { &P18061,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.1; rk29sdk Build/GINGERBREAD)*/
const Device UA2027101 = {S52388, 134, { &P18061,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; rk29sdk Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2354253 = {S52389, 161, { &P18061,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 BaDoinkDownloader_Android_2.0*/
const Device UA2384624 = {S52390, 161, { &P18061,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 BaDoinkDownloader_Android_2.0*/
const Device UA2402265 = {S52391, 161, { &P18061,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 BaDoinkDownloader_Android_2.0*/
const Device UA2423087 = {S52392, 161, { &P18061,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 BaDoinkDownloader_Android_2.0*/
const Device UA2457240 = {S52393, 139, { &P19863,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U9508 Build/HuaweiU9508) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2467405 = {S52394, 148, { &P19863,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2595154 = {S52395, 70, { &P19863,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI U9508 Build/HuaweiU9508)*/
const Device UA2974810 = {S52396, 150, { &P18552,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2361209 = {S52397, 124, { &P15364,&P16110,&P22463,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11*/
const Device UA2369706 = {S52398, 117, { &P15364,&P16110,&P22463,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11*/
const Device UA2974718 = {S52399, 131, { &P17337,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500R Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974801 = {S52400, 130, { &P17337,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023243 = {S52401, 131, { &P17337,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA1904 = {S52402, 77, { &P11644,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2600c-2/2.0 (05.61) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2600c-2*/
const Device UA1907 = {S52403, 64, { &P11644,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2600c-2/2.0 (05.61) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA13872 = {S52404, 62, { &P11644,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2600c/2.0 (05.21) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA13875 = {S52405, 73, { &P11644,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2600c/2.0 (05.22) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2600c*/
const Device UA20355 = {S52406, 65, { &P11644,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2600c-2b/2.0 (06.45) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA25213 = {S52407, 69, { &P11644,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2600c-2_CMCC/2.0 (05.61) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA29318 = {S52408, 67, { &P11644,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2600c_CMCC/2.0 (05.22) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2672680 = {S52409, 132, { &P13644,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; EVO Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675578 = {S52410, 125, { &P13644,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676182 = {S52411, 132, { &P13644,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2671834 = {S52412, 132, { &P18564,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X+ Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676194 = {S52413, 139, { &P18564,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X+ Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676841 = {S52414, 141, { &P18564,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC EVARE_UL Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2531585 = {S52415, 133, { &P18680,&P18110,&P21167,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Safari/535.19 Silk-Accelerated=true*/
const Device UA2353074 = {S52416, 129, { &P20730,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; P771A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2426596 = {S52417, 135, { &P20730,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; P771A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2875171 = {S52418, 146, { &P22972,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I537 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2885356 = {S52419, 68, { &P22972,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SAMSUNG-SGH-I537 Build/JDQ39)*/
const Device UA3079479 = {S52420, 187, { &P22972,&P18598,&P19802,&P18092 }};	/*Useragent - label=ypmobapp;anv=YPmobile4.0.1;os=Android 4.2.2;dev=jactivelteatt;devm=SAMSUNG-SGH-I537;man=samsung;brw=Android;shwd=1080x1920;shwa=1080x1845;cs=yes;is=yes;as=yes;vs=yes;jss=yes;ls=DAM;*/
const Device UA575254 = {S52421, 131, { &P15003,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHW-M380W Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA576268 = {S52422, 131, { &P15003,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHW-M380K Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA576508 = {S52423, 131, { &P15003,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHW-M380S Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA578468 = {S52424, 131, { &P15003,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHW-M380S Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA630542 = {S52425, 131, { &P15003,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHW-M380W Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA672864 = {S52426, 131, { &P15003,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHW-M380K Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2024600 = {S52427, 60, { &P15003,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; SHW-M380W Build/HTK75D)*/
const Device UA3065577 = {S52428, 136, { &P21935,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E980 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3064983 = {S52429, 127, { &P16976,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT300 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3067596 = {S52430, 129, { &P16976,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT300SE Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2040601 = {S52431, 71, { &P15364,&P7133,&P18523,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X Thunderbird/15.0.1*/
const Device UA2201976 = {S52432, 113, { &P15767,&P18432,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Flipboard/1.9.7*/
const Device UA2205977 = {S52433, 113, { &P15767,&P18432,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Flipboard/1.9.6*/
const Device UA2260608 = {S52434, 113, { &P15767,&P18432,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Flipboard/1.9.8*/
const Device UA2260649 = {S52435, 115, { &P15767,&P18432,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/1.9.8*/
const Device UA2356776 = {S52436, 115, { &P15767,&P18432,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/1.9.7*/
const Device UA2386970 = {S52437, 116, { &P15767,&P18432,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8426 Flipboard/1.9.8*/
const Device UA2404402 = {S52438, 113, { &P15767,&P18432,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A407 Flipboard/1.9.8*/
const Device UA2425568 = {S52439, 113, { &P15767,&P18432,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A406 Flipboard/1.9.8*/
const Device UA2428531 = {S52440, 115, { &P15767,&P18432,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550 Flipboard/1.9.8*/
const Device UA2433189 = {S52441, 116, { &P15767,&P18432,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8500 Flipboard/1.9.8*/
const Device UA2578456 = {S52442, 115, { &P15767,&P18432,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/1.9.6*/
const Device UA3066879 = {S52443, 138, { &P22602,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-S738C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065991 = {S52444, 143, { &P18645,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SAMSUNG-SGH-I747 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068219 = {S52445, 135, { &P18645,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SPH-L710 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080992 = {S52446, 135, { &P18645,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-I9305 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094221 = {S52447, 136, { &P18645,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SCH-R530U Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094446 = {S52448, 135, { &P18645,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SGH-I747 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA562102 = {S52449, 89, { &P15364,&P5690,&P19773,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1b3pre) Gecko/X SeaMonkey/2.0a3pre*/
const Device UA895407 = {S52450, 97, { &P15364,&P5690,&P19773,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.13) Gecko/X Mnenhy/0.8.3 SeaMonkey/2.0.8*/
const Device UA2212654 = {S52451, 90, { &P15364,&P5690,&P19773,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-CA; rv:1.9.1.11pre) Gecko/X SeaMonkey/2.0.6pre*/
const Device UA9172 = {S52452, 147, { &P13384,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ie; Vodafone 845 Build/B220) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA9178 = {S52453, 147, { &P13384,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Vodafone 845 Build/B219) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA9179 = {S52454, 147, { &P13384,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Vodafone 845 Build/B226) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA9183 = {S52455, 147, { &P13384,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-nz; Vodafone 845 Build/B233) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA9185 = {S52456, 148, { &P13384,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-nz; Vodafone 845 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA9187 = {S52457, 147, { &P13384,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; Vodafone 845 Build/B221) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA9199 = {S52458, 147, { &P13384,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; tr-tr; Vodafone 845 Build/B232) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA9200 = {S52459, 147, { &P13384,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Vodafone 845 Build/B223) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA13080 = {S52460, 141, { &P13384,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; U8120 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA38267 = {S52461, 140, { &P13384,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; pt-pt; Vodafone 845 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA51925 = {S52462, 147, { &P13384,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; el-gr; Vodafone 845 Build/B264) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA52756 = {S52463, 141, { &P13384,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update2; tr-tr; U8120 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA62888 = {S52464, 147, { &P13384,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; lt-lt; Vodafone 845 Build/B236) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018464 = {S52465, 69, { &P13384,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Vodafone 845 Build/B233)*/
const Device UA180564 = {S52466, 134, { &P13257,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Droid Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA192033 = {S52467, 134, { &P13257,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Droid Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA207041 = {S52468, 134, { &P13257,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Droid Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA236780 = {S52469, 134, { &P13257,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Droid Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA249752 = {S52470, 134, { &P13257,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Droid Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250715 = {S52471, 133, { &P13257,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA302128 = {S52472, 134, { &P13257,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; Droid Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA304740 = {S52473, 139, { &P13257,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; Droid Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA311440 = {S52474, 134, { &P13257,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Droid Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA314812 = {S52475, 133, { &P13257,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Droid Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA544057 = {S52476, 58, { &P13257,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; Droid Build/FRG83D)*/
const Device UA647248 = {S52477, 133, { &P13257,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Droid Build/GRI54) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA663476 = {S52478, 153, { &P13257,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID Build/FRG83G; liquid.sholes.X.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA677593 = {S52479, 58, { &P13257,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; Droid Build/FRG83G)*/
const Device UA1869592 = {S52480, 58, { &P13257,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; Droid Build/FRG83D)*/
const Device UA1996631 = {S52481, 148, { &P13257,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; Momodesign MD Droid Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019307 = {S52482, 58, { &P13257,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Droid Build/FRG83G)*/
const Device UA2555855 = {S52483, 141, { &P13257,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Droid Build/GRH78C; 11000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA77886 = {S52484, 145, { &P21058,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; MEDION LIFE P4310 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA361889 = {S52485, 145, { &P21058,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; MEDION LIFE P4310 Build/GRJ26) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA662795 = {S52486, 145, { &P21058,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; da-dk; MEDION LIFE P4310 Build/GRJ25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA694619 = {S52487, 145, { &P21058,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; MEDION LIFE P4310 Build/GRJ29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA746442 = {S52488, 145, { &P21058,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; nl-be; MEDION LIFE P4310 Build/GRJ27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA829287 = {S52489, 145, { &P21058,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-at; MEDION LIFE P4310 Build/GRJ30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1983579 = {S52490, 145, { &P21058,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; MEDION LIFE P4310 Build/GRJ32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1997030 = {S52491, 145, { &P21058,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; MEDION LIFE P4310 Build/GRJ34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024152 = {S52492, 145, { &P21058,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-ch; MEDION LIFE P4310 Build/GRJ31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2222430 = {S52493, 145, { &P21058,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; MEDION LIFE P4310 Build/GRJ28) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67355 = {S52494, 134, { &P12973,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A101IT Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA302125 = {S52495, 58, { &P12973,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; A101IT Build/FROYO)*/
const Device UA549272 = {S52496, 134, { &P12973,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A101B Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA781258 = {S52497, 134, { &P12973,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A101C Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA252404 = {S52498, 135, { &P15364,&P17816,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.2 (KHTML, like Gecko) Ubuntu/10.04 Chromium/15.0.874.106 Chrome/15.0.874.106 Safari/535.2*/
const Device UA252417 = {S52499, 137, { &P15364,&P17816,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Ubuntu/10.04 Chromium/15.0.874.106 Chrome/15.0.874.106 Safari/535.2*/
const Device UA2205975 = {S52500, 134, { &P18466,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2486894 = {S52501, 157, { &P16362,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-D710 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2515528 = {S52502, 157, { &P16362,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-D710 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2537623 = {S52503, 156, { &P16362,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-D710 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2714666 = {S52504, 157, { &P16362,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-D710 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2676027 = {S52505, 130, { &P22037,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3113 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2734928 = {S52506, 130, { &P22037,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3113 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA33421 = {S52507, 127, { &P15602,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; A501 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA861733 = {S52508, 56, { &P15602,&P6545,&P4755,&P18092 }};	/*Useragent - Dalvik/1.5.0 (Linux; U; Android 3.0.1; A501 Build/HRI66)*/
const Device UA251924 = {S52509, 150, { &P12216,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6300 Build/FRF91; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA680202 = {S52510, 154, { &P12216,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6300 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1963019 = {S52511, 154, { &P12216,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6300 Build/GRJ22; Condemned CM7 V15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2403638 = {S52512, 137, { &P20015,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; SC-03E Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2579014 = {S52513, 59, { &P20015,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SC-03E Build/JRO03C)*/
const Device UA700311 = {S52514, 141, { &P18040,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; YP-GB70 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA851717 = {S52515, 142, { &P18040,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; YP-GB70D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2587903 = {S52516, 65, { &P18040,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; YP-GB70 Build/GINGERBREAD)*/
const Device UA2488874 = {S52517, 104, { &P11684,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5800 XpressMusic) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2528719 = {S52518, 43, { &P11684,&P5129,&P24675,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UCWEB8.8.0.212/50/999*/
const Device UA2656535 = {S52519, 104, { &P11684,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; nokia5800d-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2729969 = {S52520, 105, { &P11684,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5800w XpressMusic) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2843999 = {S52521, 44, { &P11684,&P5129,&P24675,&P18092 }};	/*Useragent - Nokia5800w XpressMusic/UCWEB8.8.0.212/50/800*/
const Device UA2974382 = {S52522, 138, { &P23772,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975025 = {S52523, 138, { &P23772,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190L Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3111798 = {S52524, 83, { &P17595,&P18207,&P22772,&P18092 }};	/*Useragent - <![CDATA[Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:24.0) Gecko/X Firefox/24.0*/
const Device UA2753003 = {S52525, 138, { &P18295,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One X Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753286 = {S52526, 138, { &P18295,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2173376 = {S52527, 130, { &P18299,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; PantechP4100 Build/IMM76I) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA3049816 = {S52528, 49, { &P15364,&P18578,&P17470,&P18092 }};	/*Useragent - python-requests/1.2.3 CPython/2.7.1 Darwin/11.4.2*/
const Device UA2435484 = {S52529, 167, { &P18465,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Amazon Kindle Fire Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2514388 = {S52530, 167, { &P18465,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Amazon Kindle Fire Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2565037 = {S52531, 166, { &P18465,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amazon Kindle Fire Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2599426 = {S52532, 168, { &P18465,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Amazon Kindle Fire2 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2664399 = {S52533, 167, { &P18465,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amazon Kindle Fire2 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2713994 = {S52534, 167, { &P18465,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amazon Kindle Fire Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2765849 = {S52535, 159, { &P18465,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amazon Kindle Fire Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2892142 = {S52536, 168, { &P18465,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amazon Kindle Fire2 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3033960 = {S52537, 154, { &P18466,&P18066,&P24334,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2381819 = {S52538, 150, { &P18227,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI U9510E Build/HuaweiU9510E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2503401 = {S52539, 148, { &P18227,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI U9510 Build/HuaweiU9510) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2547357 = {S52540, 141, { &P18227,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U9510E Build/HuaweiU9510E) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2726309 = {S52541, 72, { &P18227,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI U9510E Build/HuaweiU9510E)*/
const Device UA3071047 = {S52542, 152, { &P18227,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; HUAWEI U9510E Build/HuaweiU9510E) AppleWebKit/534.30 (KHTML, like Gecko) Phi?n b?n/4.0 Mobile Safari/534.30*/
const Device UA2489387 = {S52543, 93, { &P12449,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAE5-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2529396 = {S52544, 102, { &P12449,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiae5-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2598491 = {S52545, 95, { &P12449,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAE5-00.2) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2952485 = {S52546, 94, { &P12449,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAE5-00u) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2986068 = {S52547, 32, { &P12449,&P5890,&P24675,&P18092 }};	/*Useragent - NOKIAE5-00/UCWEB8.8.0.212/28/999*/
const Device UA2668539 = {S52548, 31, { &P14188,&P5129,&P24676,&P18092 }};	/*Useragent - Nokia5233/UCWEB8.9.0.253/50/999*/
const Device UA2716115 = {S52549, 93, { &P14188,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5233) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2781780 = {S52550, 94, { &P14188,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia 5233) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2754405 = {S52551, 141, { &P19763,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-02E Build/10.1.D.0.343) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755599 = {S52552, 142, { &P19763,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6602 Build/10.1.1.A.1.253) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3065370 = {S52553, 126, { &P16598,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A500 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3111189 = {S52554, 133, { &P16598,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Iconia A500 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3114929 = {S52555, 126, { &P16598,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2882692 = {S52556, 112, { &P15803,&P5843,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia200/11.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA2566965 = {S52557, 155, { &P12605,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583785 = {S52558, 148, { &P12605,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA62897 = {S52559, 200, { &P14831,&P5267,&P3834,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S7233E/1.0; U; Bada/1.0; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 NexPlayer/3.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B*/
const Device UA3065242 = {S52560, 136, { &P20226,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081629 = {S52561, 137, { &P20226,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E975K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2232459 = {S52562, 141, { &P23910,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT30at Build/7.0.B.1.152) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2583606 = {S52563, 93, { &P15364,&P16110,&P22775,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; SlimBrowser/6.01.101)*/
const Device UA2602262 = {S52564, 86, { &P15364,&P16110,&P22775,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; SlimBrowser/6.01.101)*/
const Device UA2669041 = {S52565, 108, { &P15364,&P16110,&P22775,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Avant Browser; SlimBrowser/6.01.101)*/
const Device UA2676246 = {S52566, 130, { &P18702,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8013 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676933 = {S52567, 130, { &P18702,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8013 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA632406 = {S52568, 155, { &P14161,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM670 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA658387 = {S52569, 169, { &P14161,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM670 Build/Harmonia 2.03 GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA680479 = {S52570, 155, { &P14161,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM670 Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021876 = {S52571, 163, { &P14161,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; VM670 Build/Harmonia 2 GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2658888 = {S52572, 152, { &P14161,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; VM670 Build/GRI40; CyanogenMod-7.2.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA227583 = {S52573, 153, { &P18016,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; ThinkPad Tablet Build/ThinkPadTablet_A310_02) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA567603 = {S52574, 82, { &P18016,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; ThinkPad Tablet Build/ThinkPadTablet_A310_02)*/
const Device UA2023361 = {S52575, 147, { &P18016,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; ThinkPad Tablet Build/univofaz_A310_02) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2023740 = {S52576, 147, { &P18016,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; ThinkPad Tablet Build/OCSAdmin_A310_02) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2031926 = {S52577, 147, { &P18016,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; ThinkPad Tablet Build/bladenct_A310_02) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2519182 = {S52578, 147, { &P18016,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; ThinkPad Tablet Build/bladencs_A310_02) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2367530 = {S52579, 101, { &P18591,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 810)*/
const Device UA1997822 = {S52580, 152, { &P18125,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID4 Build/6.7.2-180_DR4-16_M2-37) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2366029 = {S52581, 146, { &P18125,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID4 Build/6.7.2-180_DR4-16_M2-37) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2750507 = {S52582, 131, { &P22922,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; M470BSA Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2869930 = {S52583, 60, { &P22922,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; M470BSA Build/JOP40D)*/
const Device UA2679 = {S52584, 124, { &P16177,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_Touch_Pro2/1.86.161.1 (67102)*/
const Device UA13469 = {S52585, 98, { &P16177,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) HTC_Touch_Pro2_T7379*/
const Device UA13544 = {S52586, 108, { &P16177,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_S200*/
const Device UA21962 = {S52587, 124, { &P16177,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_Touch_Pro2/1.14.161.5 (50207)*/
const Device UA21963 = {S52588, 124, { &P16177,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_Touch_Pro2/1.14.161.6 (50207)*/
const Device UA21967 = {S52589, 124, { &P16177,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_Touch_Pro2/1.86.161.0 (67102)*/
const Device UA23496 = {S52590, 90, { &P16177,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Sprint T7380*/
const Device UA23808 = {S52591, 98, { &P16177,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA24127 = {S52592, 118, { &P16177,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7381/V1.19.841.3 (51489) Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA27301 = {S52593, 90, { &P16177,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01; Windows CE; IEMobile 8.12; MSIEMobile6.0) Sprint T7380*/
const Device UA29232 = {S52594, 94, { &P16177,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373/480x800 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA32511 = {S52595, 140, { &P16177,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 480x800; HTC_Touch_Pro2_T7385-Orange; OpVer 122.190.1.610*/
const Device UA34864 = {S52596, 98, { &P16177,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373/Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA45057 = {S52597, 98, { &P16177,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7381 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA45058 = {S52598, 118, { &P16177,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7381 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) V1.93.841.0 (81210)*/
const Device UA48468 = {S52599, 124, { &P16177,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Swisscom/1.0/HTC Touch Pro2/1.86.166.1 (67102)*/
const Device UA51201 = {S52600, 140, { &P16177,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 480x800; HTC_Touch_Pro2_T7385-Orange; OpVer 122.190.2.731*/
const Device UA62172 = {S52601, 89, { &P16177,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile6.0) Sprint T7380*/
const Device UA77587 = {S52602, 124, { &P16177,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Swisscom/1.0/HTC Touch Pro2/1.86.167.2 (67102)*/
const Device UA580624 = {S52603, 74, { &P15364,&P17285,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:9.0.1) Gecko/X Firefox/9.0.1 Firefox/9.0.1*/
const Device UA629620 = {S52604, 63, { &P15364,&P17285,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:9.0) Gecko/X Firefox/9.0*/
const Device UA2670893 = {S52605, 139, { &P20289,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC6435LVW Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676272 = {S52606, 139, { &P20289,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677167 = {S52607, 132, { &P20289,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA1967249 = {S52608, 132, { &P18702,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8013 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2368526 = {S52609, 159, { &P20177,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Android 4.0.3;en-us;Digitac Ultratab;C725 Build/ICS) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2406369 = {S52610, 128, { &P20177,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C725 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2420456 = {S52611, 159, { &P20177,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Android 4.0.4;en-us;Digitac Ultratab;C725 Build/ICS) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2430880 = {S52612, 142, { &P20177,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Digitac Ultratab;C725 Build/ICS) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA3051171 = {S52613, 142, { &P20177,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Digitac Ultratab;C725 Build/ICS) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA925495 = {S52614, 131, { &P20948,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LePanII Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2753267 = {S52615, 145, { &P18442,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I317 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754265 = {S52616, 137, { &P18442,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7105 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754581 = {S52617, 137, { &P18442,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754627 = {S52618, 137, { &P18442,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T889 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754933 = {S52619, 142, { &P18442,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Note II Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769283 = {S52620, 137, { &P18442,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L900 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868104 = {S52621, 129, { &P18442,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7105 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2870780 = {S52622, 137, { &P18442,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R950 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877346 = {S52623, 137, { &P18442,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I317 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892536 = {S52624, 130, { &P18442,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754718 = {S52625, 130, { &P15919,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P3100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755108 = {S52626, 130, { &P15919,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2463956 = {S52627, 142, { &P19778,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP9090 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2754163 = {S52628, 144, { &P18614,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT890 Build/9.8.2I-50_SMI-26) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA1949 = {S52629, 61, { &P11645,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2630/2.0 (06.82) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA1939 = {S52630, 71, { &P11645,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2630/2.0 (06.82) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2630*/
const Device UA17254 = {S52631, 66, { &P11645,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2630_CMCC/2.0 (05.62) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA908427 = {S52632, 131, { &P20185,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID7035 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2481911 = {S52633, 132, { &P20185,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; MID7035C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2974906 = {S52634, 137, { &P18559,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LG730 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA696320 = {S52635, 76, { &P15364,&P5690,&P22765,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Maxthon/3.0)*/
const Device UA2218631 = {S52636, 76, { &P15364,&P5690,&P22765,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; Maxthon/3.0)*/
const Device UA2333893 = {S52637, 134, { &P15364,&P5690,&P22765,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Maxthon/3.0)*/
const Device UA1997007 = {S52638, 90, { &P17595,&P18207,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19*/
const Device UA2429903 = {S52639, 128, { &P19991,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; A511 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2657203 = {S52640, 57, { &P19991,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; A511 Build/JZO54K)*/
const Device UA90414 = {S52641, 120, { &P15841,&P17352,&P17360,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; RIM Tablet OS 1.0.0; en-US) AppleWebKit/534.8+ (KHTML, like Gecko) Version/0.0.1 Safari/534.8+*/
const Device UA181743 = {S52642, 122, { &P15841,&P17352,&P17360,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; RIM Tablet OS 1.0.0; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/0.0.1 Safari/534.11+*/
const Device UA213157 = {S52643, 124, { &P15841,&P17352,&P17360,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; RIM Tablet OS 1.0.0; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.7 Safari/534.11+*/
const Device UA2671960 = {S52644, 136, { &P18713,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P768 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676496 = {S52645, 136, { &P18713,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P769 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676510 = {S52646, 136, { &P18713,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P760 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708388 = {S52647, 129, { &P18713,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P769 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2487605 = {S52648, 101, { &P15364,&P7127,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA776132 = {S52649, 141, { &P20944,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; T-Hub2 Build/TVA301TELBG3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1936903 = {S52650, 148, { &P20944,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; T-Hub2 Build/TC-301-L27.G.3-EVT-) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030202 = {S52651, 148, { &P20944,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; T-Hub2 Build/TC-301-L27.G.7-DVT-) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2647786 = {S52652, 65, { &P20944,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; T-Hub2 Build/TVA301TELBG3)*/
const Device UA2974993 = {S52653, 126, { &P16598,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A500 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021591 = {S52654, 126, { &P16598,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023583 = {S52655, 133, { &P16598,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Iconia A500 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3057980 = {S52656, 133, { &P16598,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A500 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA910919 = {S52657, 140, { &P18496,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR6410LVW Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2224478 = {S52658, 140, { &P18496,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LVW Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2481655 = {S52659, 140, { &P18496,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR6410LRA Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA12307 = {S52660, 146, { &P15365,&P6757,&P3525,&P18092 }};	/*Useragent - HTC_Magic Mozilla/5.0 (Linux; U; Android 1.5; en-ca; Build/2.17.631.2) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA23554 = {S52661, 146, { &P15365,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-hk; HTC Magic_MR Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA23557 = {S52662, 146, { &P15365,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-tw; HTC Magic MR Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA23807 = {S52663, 143, { &P15365,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-ie; HTC Magic Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA24638 = {S52664, 143, { &P15365,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-in; HTC_Magic Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA28973 = {S52665, 143, { &P15365,&P6757,&P3525,&P18092 }};	/*Useragent - HTC_Magic Mozilla/5.0 (Linux; U; Android 1.5; en-ca; Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA50068 = {S52666, 140, { &P15365,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-gb; HTC Magic Build/CRC1) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA66418 = {S52667, 146, { &P15365,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; pl-pl; HTC Magic Build/2.17.401.2) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA66422 = {S52668, 141, { &P15365,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; it-; HTC Magic Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA88465 = {S52669, 139, { &P15365,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; sapphire; HTC_Magic) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA524316 = {S52670, 141, { &P15365,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; fr-fr; HTC Magic Build/CRB17) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA735573 = {S52671, 141, { &P15365,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-au; HTC Magic Build/CRB43) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA791949 = {S52672, 141, { &P15365,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; de-de; HTC Magic Build/CRC24) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2018885 = {S52673, 157, { &P15365,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; de-de; HTC Magic Build/PLAT-RC33) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 FirePHP/0.3*/
const Device UA2028590 = {S52674, 275, { &P15365,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; fr-fr; Mozilla/5.0 (Linux; U; Android 1.5; fr-fr; HTC Magic Build/CRB43) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA3117781 = {S52675, 157, { &P15365,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; de-de; HTC Magic Build/PLAT-RC33) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 FirePHP/0.3*/
const Device UA3030029 = {S52676, 140, { &P20705,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC One Build/JWR66V.H10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3059622 = {S52677, 58, { &P20705,&P21637,&P23997,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.3; HTC One Build/JSS15J)*/
const Device UA3059651 = {S52678, 136, { &P20705,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC_One Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3059650 = {S52679, 136, { &P20705,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC One Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078762 = {S52680, 132, { &P20705,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079380 = {S52681, 155, { &P20705,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC One Build/BlackJelly 1.1 JWR66V.H10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3093348 = {S52682, 139, { &P20705,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC One Build/JWR66Y.H1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3093588 = {S52683, 138, { &P20705,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC_PN071 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108651 = {S52684, 140, { &P20705,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC_PN07120 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3112336 = {S52685, 141, { &P20705,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; zh-tw; HTC One 801e Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125962 = {S52686, 135, { &P20705,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTCONE Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3127277 = {S52687, 63, { &P20705,&P21637,&P23997,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.3; HTC One 801e Build/JSS15J)*/
const Device UA3132614 = {S52688, 57, { &P20705,&P21637,&P23997,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.3; HTCONE Build/JSS15J)*/
const Device UA3065258 = {S52689, 137, { &P16349,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Sony Tablet S Build/TISU0143) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA5030 = {S52690, 140, { &P15364,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; MDA_Compact_V/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA7065 = {S52691, 94, { &P15364,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) T-Mobile_Rhodium*/
const Device UA21129 = {S52692, 92, { &P15364,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) T-Mobile_Maple*/
const Device UA25397 = {S52693, 90, { &P15364,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) T-Mobile_LEO*/
const Device UA43654 = {S52694, 89, { &P15364,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) USCCHTC6875*/
const Device UA512519 = {S52695, 135, { &P15364,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) GSmart-S1205;Mozilla/4.0(compatible;MSIE6.0;PPC; 240x400)*/
const Device UA2792848 = {S52696, 63, { &P15364,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; PPC; Tablet PC; Trident/4.0)*/
const Device UA2369661 = {S52697, 146, { &P20089,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySO-01E Build/9.0.G.0.238) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2391205 = {S52698, 146, { &P20089,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySO-01E Build/9.0.G.0.247) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2428627 = {S52699, 142, { &P20089,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-01E Build/9.0.G.0.247) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2487844 = {S52700, 145, { &P20089,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySO-01E Build/9.0.G.1.76) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2504313 = {S52701, 142, { &P20089,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-01E Build/9.0.G.0.238) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2508312 = {S52702, 141, { &P20089,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-01E Build/9.0.G.1.76) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2585956 = {S52703, 146, { &P20089,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySO-01E Build/9.0.G.1.108) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2601943 = {S52704, 142, { &P20089,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-01E Build/9.0.G.1.108) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2766711 = {S52705, 64, { &P20089,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SO-01E Build/9.0.G.1.108)*/
const Device UA22983 = {S52706, 143, { &P13215,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-ca; GT-I5700R Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA50206 = {S52707, 142, { &P13215,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-gb; GT-I5700 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA52790 = {S52708, 161, { &P13215,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; fr-fr; GT-I5700-ORANGE/I5700OFRIK1 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA56029 = {S52709, 143, { &P13215,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; es-us; GT-I5700L Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2018332 = {S52710, 140, { &P13215,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; ar-; GT-I5700 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2019441 = {S52711, 141, { &P13215,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; iw-; GT-I5700L Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA3117693 = {S52712, 142, { &P13215,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; fr-fr; GT-I5700 Build/CUPCAKE) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2498188 = {S52713, 140, { &P20737,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-S730G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986982 = {S52714, 62, { &P20737,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-S730G Build/IMM76D)*/
const Device UA616376 = {S52715, 134, { &P12280,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3*/
const Device UA908543 = {S52716, 137, { &P12280,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.13 (KHTML, like Gecko) Version/5.1 Mobile/10A5338d Safari/7534.48.3*/
const Device UA2197908 = {S52717, 125, { &P12280,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari Version/5.1*/
const Device UA2203411 = {S52718, 125, { &P12280,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Safari Version/5.1*/
const Device UA2261180 = {S52719, 127, { &P12280,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari Version/5.1*/
const Device UA2261184 = {S52720, 126, { &P12280,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10A403 Safari/7536.26*/
const Device UA2352194 = {S52721, 135, { &P12280,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Version/5.1 Safari/7534.48.3*/
const Device UA2356975 = {S52722, 127, { &P12280,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari Version/5.1*/
const Device UA2367595 = {S52723, 128, { &P12280,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10A523 Safari/7536.26*/
const Device UA2415300 = {S52724, 135, { &P12280,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/10A403 Safari/7534.48.3*/
const Device UA2425921 = {S52725, 137, { &P12280,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Version/5.1 Safari/7534.48.3*/
const Device UA2437636 = {S52726, 127, { &P12280,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari Version/5.1*/
const Device UA2457493 = {S52727, 137, { &P12280,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/10A523 Safari/7534.48.3*/
const Device UA2658896 = {S52728, 135, { &P12280,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 60 like Mac OS X) AppleWebKit/536.13 (KHTML like Gecko) Version/5.1 Mobile/10A5338d Safari/7534.48.3*/
const Device UA2727609 = {S52729, 134, { &P12280,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B179 Safari/7534.48.3*/
const Device UA2850782 = {S52730, 124, { &P12280,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 601 like Mac OS X) AppleWebKit/536.26 (KHTML like Gecko) Mobile/10A525 Safari Version/5.1*/
const Device UA2983538 = {S52731, 136, { &P12280,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA2203851 = {S52732, 117, { &P15364,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; Juggernaut; Semaphore)*/
const Device UA2425604 = {S52733, 108, { &P15364,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; id104)*/
const Device UA2760427 = {S52734, 111, { &P15364,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; Qualcomm; Fluid)*/
const Device UA2938137 = {S52735, 108, { &P15364,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; id302)*/
const Device UA2950689 = {S52736, 123, { &P15364,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; RM-825_developer_249)*/
const Device UA2957767 = {S52737, 114, { &P15364,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; SAMSUNG; SCH-R860U)*/
const Device UA2958292 = {S52738, 117, { &P15364,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; HTCPO881; SPRINT)*/
const Device UA2959526 = {S52739, 112, { &P15364,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia 925)*/
const Device UA2982234 = {S52740, 106, { &P15364,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; RIO_U)*/
const Device UA3012551 = {S52741, 108, { &P15364,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; id308)*/
const Device UA3047337 = {S52742, 113, { &P15364,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; SAMSUNG; SGH-I187)*/
const Device UA3050520 = {S52743, 113, { &P15364,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia 925T)*/
const Device UA3057551 = {S52744, 114, { &P15364,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; 909; ANZ875)*/
const Device UA3057639 = {S52745, 108, { &P15364,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; id307)*/
const Device UA3093031 = {S52746, 113, { &P15364,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 1520)*/
const Device UA3106675 = {S52747, 116, { &P15364,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; 909; Vodafone)*/
const Device UA3108343 = {S52748, 113, { &P15364,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia 625H)*/
const Device UA3114859 = {S52749, 112, { &P15364,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 929)*/
const Device UA3114969 = {S52750, 114, { &P15364,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; 909; ANZ892)*/
const Device UA3125619 = {S52751, 112, { &P15364,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HTC; HTCPO881; X)*/
const Device UA2673250 = {S52752, 131, { &P19902,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676603 = {S52753, 138, { &P19902,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA622908 = {S52754, 134, { &P19756,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Velocitymicro/T410) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA99700 = {S52755, 116, { &P15364,&P17285,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.29*/
const Device UA101718 = {S52756, 118, { &P15364,&P17285,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA123729 = {S52757, 116, { &P15364,&P17285,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.28 (KHTML, like Gecko) Version/3.2.2 Safari/525.28.1*/
const Device UA2216562 = {S52758, 117, { &P15364,&P17285,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; hu-HU) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.13*/
const Device UA2710975 = {S52759, 49, { &P15364,&P21691,&P17470,&P18092 }};	/*Useragent - python-requests/1.1.0 CPython/2.7.2 Darwin/12.3.0*/
const Device UA3049648 = {S52760, 49, { &P15364,&P21691,&P17470,&P18092 }};	/*Useragent - python-requests/1.2.3 CPython/2.7.2 Darwin/12.3.0*/
const Device UA2029733 = {S52761, 139, { &P19774,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PMP3370B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2933699 = {S52762, 61, { &P19774,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; PMP3370B Build/IML74K)*/
const Device UA402853 = {S52763, 132, { &P18183,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; XOOM 2 Build/1.6.0_211.04) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA402944 = {S52764, 137, { &P18183,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; XOOM 2 Build/1.6.0_218.3-MZ615) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA402959 = {S52765, 139, { &P18183,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; XOOM 2 Build/1.6.0_218.3-MZ615) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA403001 = {S52766, 68, { &P18183,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; XOOM 2 Build/1.6.0_218.3-MZ615)*/
const Device UA403048 = {S52767, 138, { &P18183,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; XOOM 2 Build/1.6.0M_268-MZ615) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA403049 = {S52768, 129, { &P18183,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; XOOM 2 Build/1.6.0_222) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA403054 = {S52769, 139, { &P18183,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 Build/1.6.0_268.4-MZ615) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA403057 = {S52770, 139, { &P18183,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; de-de; XOOM 2 Build/1.6.0_249.1-MZ615) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA403019 = {S52771, 138, { &P18183,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; XOOM 2 Build/1.6.0M_279_MZ615) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA403085 = {S52772, 138, { &P18183,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; XOOM 2 Build/1.6.0M_278-MZ615) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA403204 = {S52773, 138, { &P18183,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; XOOM 2 Build/1.6.0M_280_MZ615) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA403210 = {S52774, 140, { &P18183,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; XOOM 2 Build/1.6.0M_278-MZ615) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA403224 = {S52775, 141, { &P18183,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 Build/1.6.0_268.4-MZ615) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2025645 = {S52776, 70, { &P18183,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.2; XOOM 2 Build/1.6.0_268.4-MZ615)*/
const Device UA3132002 = {S52777, 136, { &P21648,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA2026346 = {S52778, 138, { &P12681,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; Nexus S 4G Build/JRN84D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2041421 = {S52779, 140, { &P12681,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus S 4G Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2234205 = {S52780, 140, { &P12681,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus S 4G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2497791 = {S52781, 163, { &P12681,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Full Android on Crespo4G Build/PHONELAB_JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA94717 = {S52782, 63, { &P15364,&P18110,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; en) Presto/2.2.15 Version/10.00*/
const Device UA259510 = {S52783, 63, { &P15364,&P18110,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; en) Presto/2.2.15 Version/10.01*/
const Device UA499311 = {S52784, 50, { &P15364,&P18110,&P4812,&P18092 }};	/*Useragent - Opera/10.00 (X11; Linux i686 ; U; en) Presto/2.2.0*/
const Device UA2164031 = {S52785, 68, { &P15364,&P18110,&P4812,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux i686 ; en) Opera 10.00*/
const Device UA2682286 = {S52786, 31, { &P15364,&P18110,&P4812,&P18092 }};	/*Useragent - Opera/10.0 (X11; Linux i686; U)*/
const Device UA3065002 = {S52787, 138, { &P18564,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One X+ Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA625707 = {S52788, 113, { &P15803,&P5843,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia200/10.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.11*/
const Device UA2411331 = {S52789, 139, { &P19847,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S6012 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2699399 = {S52790, 140, { &P19847,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT-S6012B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2790745 = {S52791, 137, { &P19847,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-S6012 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2848962 = {S52792, 61, { &P19847,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-S6012 Build/IMM76D)*/
const Device UA2728720 = {S52793, 140, { &P23159,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-S960L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879037 = {S52794, 62, { &P23159,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-S960L Build/JZO54K)*/
const Device UA2974992 = {S52795, 135, { &P17968,&P18149,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; cm_tenderloin Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2975011 = {S52796, 134, { &P17968,&P18149,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; cm_tenderloin Build/GWK74) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2203663 = {S52797, 112, { &P18648,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia306/03.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2390868 = {S52798, 111, { &P18648,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia306/03.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5*/
const Device UA2230079 = {S52799, 152, { &P20012,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT605 Build/5.5.1Q-JORIAN-108-TA-261) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2423442 = {S52800, 152, { &P20012,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT605 Build/5.5.1Q-JORIAN-108-TA-262) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2534887 = {S52801, 152, { &P20012,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT605 Build/5.5.1Q-JORIAN-108-TA-270) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA154926 = {S52802, 62, { &P15364,&P5690,&P18374,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.6.31 Version/10.70*/
const Device UA2152379 = {S52803, 84, { &P15364,&P5690,&P18374,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; MRA 5.9 (build 4848); ru) Presto/2.6.31 Version/10.70*/
const Device UA6384 = {S52804, 174, { &P12456,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2NokiaE73/044.008; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.9 3gpp-gba*/
const Device UA64066 = {S52805, 175, { &P12456,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE73/044.008; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.9 3gpp-gba*/
const Device UA2755351 = {S52806, 139, { &P18223,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.0.400) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA176044 = {S52807, 132, { &P17993,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS03 Build/S3251) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250342 = {S52808, 132, { &P17993,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS03 Build/S9090) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA333630 = {S52809, 132, { &P17993,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS03 Build/S5160) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626574 = {S52810, 132, { &P17993,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS03 Build/S3080) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA631820 = {S52811, 132, { &P17993,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS03 Build/S4110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2205790 = {S52812, 132, { &P17993,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS03 Build/S8301) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042969 = {S52813, 56, { &P17993,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; IS03 Build/S8301)*/
const Device UA1973769 = {S52814, 119, { &P17595,&P18207,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA2040402 = {S52815, 119, { &P17595,&P18207,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA2204144 = {S52816, 119, { &P17595,&P18207,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA2602528 = {S52817, 119, { &P17595,&P18207,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA2797750 = {S52818, 119, { &P17595,&P18207,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA3111042 = {S52819, 119, { &P17595,&P18207,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA2607004 = {S52820, 36, { &P12280,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.1.3 (4; 16GB; dt:27)*/
const Device UA2607261 = {S52821, 36, { &P12280,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.1.3 (6; 16GB; dt:73)*/
const Device UA2668890 = {S52822, 35, { &P12280,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.1.3 (4; 8GB; dt:27)*/
const Device UA2669785 = {S52823, 36, { &P12280,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.1.3 (6; 32GB; dt:73)*/
const Device UA2670301 = {S52824, 36, { &P12280,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.1.3 (4; 32GB; dt:27)*/
const Device UA2702687 = {S52825, 36, { &P12280,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.1.3 (3; 16GB; dt:24)*/
const Device UA2712146 = {S52826, 36, { &P12280,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.1.3 (6; 64GB; dt:73)*/
const Device UA2713730 = {S52827, 35, { &P12280,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.1.3 (5; 8GB; dt:70)*/
const Device UA2714842 = {S52828, 35, { &P12280,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.1.3 (3; 8GB; dt:24)*/
const Device UA2738283 = {S52829, 36, { &P12280,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.1.4 (6; 32GB; dt:82)*/
const Device UA2739831 = {S52830, 36, { &P12280,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.1.4 (6; 16GB; dt:82)*/
const Device UA2750327 = {S52831, 32, { &P12280,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.1.3 (8GB; dt:27)*/
const Device UA2752701 = {S52832, 36, { &P12280,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.1.4 (6; 16GB; dt:79)*/
const Device UA2795131 = {S52833, 36, { &P12280,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.1.3 (5; 16GB; dt:70)*/
const Device UA2799943 = {S52834, 36, { &P12280,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.1.4 (6; 64GB; dt:82)*/
const Device UA2878996 = {S52835, 36, { &P12280,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.1.4 (6; 32GB; dt:79)*/
const Device UA2936119 = {S52836, 36, { &P12280,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.1.3 (3; 32GB; dt:24)*/
const Device UA2956928 = {S52837, 36, { &P12280,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.1.4 (6; 64GB; dt:79)*/
const Device UA2989943 = {S52838, 36, { &P12280,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.1.3 (5; 32GB; dt:70)*/
const Device UA2488274 = {S52839, 91, { &P14358,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaE63) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2558306 = {S52840, 30, { &P14358,&P5890,&P24675,&P18092 }};	/*Useragent - NokiaE63/UCWEB8.8.0.212/28/999*/
const Device UA2791533 = {S52841, 92, { &P14358,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiae63-1) U2/1.0.0 UCBrowser/8.8.0.227 U2/1.0.0 Mobile*/
const Device UA48690 = {S52842, 139, { &P13591,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; ZTE-BLADE Build/OUK_B05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59608 = {S52843, 137, { &P13591,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; ZTE-BLADE Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79601 = {S52844, 137, { &P13591,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; sv-se; ZTE-BLADE Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA699864 = {S52845, 61, { &P13591,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; ZTE-BLADE Build/GRJ22)*/
const Device UA1908802 = {S52846, 137, { &P13591,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; ZTE-BLADE Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019008 = {S52847, 61, { &P13591,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; ZTE-BLADE Build/GRJ22)*/
const Device UA2021652 = {S52848, 125, { &P13591,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE-BLADE Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA119160 = {S52849, 25, { &P15364,&P17017,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 Firefox/3.0.5*/
const Device UA387391 = {S52850, 13, { &P15364,&P17017,&P3409,&P18092 }};	/*Useragent - Firefox/3.0.5*/
const Device UA401300 = {S52851, 25, { &P15364,&P17017,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 Firefox/3.0.4*/
const Device UA414420 = {S52852, 13, { &P15364,&P17017,&P3409,&P18092 }};	/*Useragent - Firefox/3.0.7*/
const Device UA446140 = {S52853, 54, { &P15364,&P17017,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 ( en-US; rv:1.9.0.3) Gecko/X Firefox/3.0.3*/
const Device UA496648 = {S52854, 64, { &P15364,&P17017,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (www.vouchers.im; en-US; rv:1.9) Gecko/X Firefox/3.0*/
const Device UA505462 = {S52855, 12, { &P15364,&P17017,&P3409,&P18092 }};	/*Useragent - Firefox/3.05*/
const Device UA585954 = {S52856, 13, { &P15364,&P17017,&P3409,&P18092 }};	/*Useragent - Firefox/3.0.3*/
const Device UA631871 = {S52857, 50, { &P15364,&P17017,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (SaferSurf rv:1.9) Gecko/X Firefox/3.0*/
const Device UA696569 = {S52858, 11, { &P15364,&P17017,&P3409,&P18092 }};	/*Useragent - Firefox/3.0*/
const Device UA890355 = {S52859, 14, { &P15364,&P17017,&P3409,&P18092 }};	/*Useragent - Firefox/3.0.15*/
const Device UA2568377 = {S52860, 135, { &P15767,&P17146,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9A405 Safari/7534.48.3*/
const Device UA2569994 = {S52861, 133, { &P15767,&P17146,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9A334 Safari/7534.48.3*/
const Device UA2676332 = {S52862, 138, { &P18280,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676436 = {S52863, 138, { &P18280,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC VLE_U Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676635 = {S52864, 138, { &P18280,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC VLE_U Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676754 = {S52865, 138, { &P18280,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One S Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2710208 = {S52866, 138, { &P18240,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; LG-P700 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2767166 = {S52867, 143, { &P18240,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P700/P700 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2861322 = {S52868, 60, { &P18240,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P700 Build/JZO54K)*/
const Device UA2868515 = {S52869, 145, { &P18240,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-P700-Orange Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2877622 = {S52870, 143, { &P18240,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; LG-P700/V20c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA211501 = {S52871, 55, { &P15364,&P18110,&P22525,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux) AppleWebKit/534.26+ Midori/0.4*/
const Device UA1986860 = {S52872, 109, { &P15364,&P18110,&P22525,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux) AppleWebKit/535.22 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/535.22 Midori/0.4*/
const Device UA2424578 = {S52873, 116, { &P15364,&P18110,&P22525,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.22 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/535.22 Midori/0.4*/
const Device UA2564588 = {S52874, 113, { &P15364,&P18110,&P22525,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux) AppleWebKit/535.22 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/535.22 Midori/"0"."4"*/
const Device UA2652641 = {S52875, 114, { &P15364,&P18110,&P22525,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.22 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/535.22 Midori/0.4*/
const Device UA2667705 = {S52876, 71, { &P15364,&P18110,&P22525,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux; rv:2.0.1) Gecko/X Firefox/4.0.1 Midori/"0"."4"*/
const Device UA563407 = {S52877, 26, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/4.3.5 (64GB)*/
const Device UA564149 = {S52878, 29, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/4.3.5 (2; 32GB)*/
const Device UA643817 = {S52879, 29, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/5.0.1 (2; 64GB)*/
const Device UA805836 = {S52880, 33, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/5.1.1 (64GB; dt:28)*/
const Device UA877115 = {S52881, 36, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/5.1.1 (2; 16GB; dt:75)*/
const Device UA1993978 = {S52882, 33, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/5.1.1 (16GB; dt:28)*/
const Device UA2141247 = {S52883, 29, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/4.3.5 (2; 16GB)*/
const Device UA2269048 = {S52884, 26, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/4.2.1 (16GB)*/
const Device UA2269102 = {S52885, 26, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/4.2.1 (32GB)*/
const Device UA2334742 = {S52886, 33, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/5.1.1 (32GB; dt:28)*/
const Device UA2654167 = {S52887, 36, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/6.0.1 (5; 64GB; dt:85)*/
const Device UA2669622 = {S52888, 36, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/6.1.3 (5; 32GB; dt:78)*/
const Device UA2670104 = {S52889, 36, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/6.1.3 (2; 32GB; dt:75)*/
const Device UA2683147 = {S52890, 29, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/5.0.1 (3; 32GB)*/
const Device UA2713662 = {S52891, 36, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/6.1.3 (2; 64GB; dt:75)*/
const Device UA2733483 = {S52892, 36, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/6.1.3 (5; 64GB; dt:78)*/
const Device UA2767147 = {S52893, 36, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/6.1.3 (5; 16GB; dt:78)*/
const Device UA2861988 = {S52894, 36, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/6.1.3 (5; 32GB; dt:85)*/
const Device UA2935657 = {S52895, 36, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/6.1.3 (5; 16GB; dt:88)*/
const Device UA2935734 = {S52896, 36, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/6.1.3 (5; 16GB; dt:85)*/
const Device UA2954675 = {S52897, 36, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/6.1.3 (3; 64GB; dt:76)*/
const Device UA2989642 = {S52898, 36, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/6.1.3 (5; 32GB; dt:87)*/
const Device UA2989769 = {S52899, 36, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/6.1.3 (4; 64GB; dt:78)*/
const Device UA2990128 = {S52900, 36, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/6.1.3 (5; 32GB; dt:84)*/
const Device UA2990899 = {S52901, 36, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/6.1.3 (5; 32GB; dt:88)*/
const Device UA2991495 = {S52902, 36, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/6.1.3 (5; 64GB; dt:88)*/
const Device UA2992254 = {S52903, 36, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/6.1.3 (5; 16GB; dt:87)*/
const Device UA3047084 = {S52904, 36, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/6.1.3 (4; 32GB; dt:78)*/
const Device UA3072555 = {S52905, 36, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/6.1.3 (3; 32GB; dt:76)*/
const Device UA3087245 = {S52906, 36, { &P15767,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPad-M/6.1.3 (3; 16GB; dt:76)*/
const Device UA2198464 = {S52907, 151, { &P17346,&P17090,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8J2 Safari/7534.48.3*/
const Device UA2205489 = {S52908, 151, { &P17346,&P17090,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8L1 Safari/7534.48.3*/
const Device UA2234085 = {S52909, 151, { &P17346,&P17090,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8F190 Safari/7534.48.3*/
const Device UA2236041 = {S52910, 151, { &P17346,&P17090,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8G4 Safari/7534.48.3*/
const Device UA2236286 = {S52911, 151, { &P17346,&P17090,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8K2 Safari/7534.48.3*/
const Device UA2236428 = {S52912, 151, { &P17346,&P17090,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/8H7 Safari/7534.48.3*/
const Device UA3066520 = {S52913, 129, { &P18639,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 10 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3082765 = {S52914, 128, { &P18639,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Nexus 10 Build/JVP15P) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3083762 = {S52915, 130, { &P18639,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 10 Build/JOP40F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA67661 = {S52916, 137, { &P14849,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; GALAXY_Tab Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA153848 = {S52917, 134, { &P14849,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SCH-I800 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA162585 = {S52918, 58, { &P14849,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; SCH-I800 Build/FROYO)*/
const Device UA582349 = {S52919, 133, { &P16336,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; U8800Pro Build/HuaweiU8800Pro) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA640376 = {S52920, 145, { &P16336,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; U8800Pro Build/HuaweiU8800Pro) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023549 = {S52921, 69, { &P16336,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; U8800Pro Build/HuaweiU8800Pro)*/
const Device UA2027052 = {S52922, 138, { &P16336,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; U8800Pro Build/HuaweiU8800Pro) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA627348 = {S52923, 140, { &P23905,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  GT-I9100P Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA901662 = {S52924, 139, { &P23905,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100P Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2201656 = {S52925, 139, { &P23905,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100P Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2770106 = {S52926, 139, { &P12968,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.0.200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773317 = {S52927, 132, { &P20705,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; One Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773557 = {S52928, 135, { &P20705,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784804 = {S52929, 140, { &P20705,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One 801e Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879300 = {S52930, 136, { &P20705,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; HTC_One Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3003077 = {S52931, 137, { &P20705,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC_PN071 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2996367 = {S52932, 131, { &P24367,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-P5200 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3049055 = {S52933, 60, { &P24367,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P5200 Build/JDQ39)*/
const Device UA3059984 = {S52934, 152, { &P24367,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; cs-cz; SAMSUNG GT-P5200/P5200XXUAMFC Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2716979 = {S52935, 154, { &P18466,&P18066,&P22588,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA370233 = {S52936, 137, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Internet TV Box Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA370340 = {S52937, 133, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Internet TV Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA396912 = {S52938, 127, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; EPAD Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA579181 = {S52939, 131, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; MUSN-T800 Build/HTK55D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA630821 = {S52940, 129, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Thrive Build/v_1.2) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA655983 = {S52941, 127, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; es-es; T1001 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA727955 = {S52942, 138, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; he-il; Liberty Tab G100 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA825247 = {S52943, 143, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Speed_MB-Z4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA845408 = {S52944, 136, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; tr-tr; Nirvana_Tablet Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1900947 = {S52945, 128, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; THRiVE Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1935732 = {S52946, 130, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; ja-jp; EB-A71GJ Build/HTK55D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA1990747 = {S52947, 128, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; tr-tr; P10MAN Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2021603 = {S52948, 147, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Huawei S7-312u Build/HuaweiS7-312uB021) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2022739 = {S52949, 132, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; pt-br; TA1013 Build/SEMP_X_001) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2023517 = {S52950, 131, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; zh-cn; M1101AS Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2026587 = {S52951, 129, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ru-ru; ts9708b Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2027048 = {S52952, 132, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; pt-br; YPY10FTA Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2028298 = {S52953, 126, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; T91 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2030956 = {S52954, 129, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Revue Build/KA2X-X.X) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2031590 = {S52955, 137, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; Camangi-Mangrove7 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2032837 = {S52956, 141, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; ZY Z6330S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032862 = {S52957, 133, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; pt-br; YPY_10FTA Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2033896 = {S52958, 131, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHV-E140K Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2413620 = {S52959, 129, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ru-ru; ts9705b Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2463887 = {S52960, 134, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; NSZ-GS7/GX70 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2468908 = {S52961, 127, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; M97B Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2544812 = {S52962, 129, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; V66 4G Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2556409 = {S52963, 133, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-tw; IGS-1000 Build/IGS_X_001) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2642538 = {S52964, 130, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; zh-cn; M170AT Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2644027 = {S52965, 129, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ru-ru; ts1009b Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2650850 = {S52966, 134, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us;USCCADT71US Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2746480 = {S52967, 125, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; th-th; FE6 Build/HTK55D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2758481 = {S52968, 136, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; zh-cn; IdeaTabV2010A Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2759059 = {S52969, 131, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; zh-cn; M1101AT Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2980783 = {S52970, 128, { &P15364,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; VAP430 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2437676 = {S52971, 141, { &P17172,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Transformer TF101 Build/JB_MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2535261 = {S52972, 143, { &P17172,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Transformer TF101 Build/JB_MR1.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2565478 = {S52973, 140, { &P17172,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Transformer TF101 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2666135 = {S52974, 69, { &P17172,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; Transformer TF101 Build/JDQ39)*/
const Device UA2998608 = {S52975, 141, { &P17172,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Transformer Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3009803 = {S52976, 142, { &P17172,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Transformer Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2406340 = {S52977, 133, { &P18466,&P18110,&P19816,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Safari/535.19 Silk-Accelerated=false*/
const Device UA2882688 = {S52978, 114, { &P15228,&P5043,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-03/07.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA310280 = {S52979, 149, { &P15364,&P17816,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Ubuntu/10.04 Chromium/10.0.648.133 Chrome/10.0.648.133 Safari/534.16*/
const Device UA557600 = {S52980, 149, { &P15364,&P17816,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux armv7l; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Ubuntu/10.04 Chromium/10.0.648.133 Chrome/10.0.648.133 Safari/534.16*/
const Device UA795521 = {S52981, 143, { &P15364,&P17816,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.15 (KHTML, like Gecko) Ubuntu/10.04 Chromium/10.0.612.3 Chrome/10.0.612.3 Safari/534.15*/
const Device UA2487695 = {S52982, 76, { &P15364,&P5690,&P22368,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:15.4) Gecko/X Firefox/15.4.1 PaleMoon/15.4.1*/
const Device UA2531698 = {S52983, 83, { &P15364,&P5690,&P22368,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:15.4) Gecko/X Firefox/15.4.1 PaleMoon/15.4.1*/
const Device UA2203633 = {S52984, 114, { &P15837,&P6909,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-05/08.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2704750 = {S52985, 138, { &P21283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2768382 = {S52986, 139, { &P21283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310N Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2863318 = {S52987, 139, { &P21283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2867137 = {S52988, 139, { &P21283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2566689 = {S52989, 127, { &P15869,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xoom Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583961 = {S52990, 127, { &P15869,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Xoom Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2737636 = {S52991, 88, { &P15364,&P16110,&P24342,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; QQBrowser/7.3.8581.400)*/
const Device UA2948462 = {S52992, 96, { &P15364,&P16110,&P24342,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; QQBrowser/7.3.11251.400)*/
const Device UA2956717 = {S52993, 114, { &P15364,&P16110,&P24342,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; qdesk 2.4.1264.203; Windows NT 6.2; WOW64; Trident/6.0; QQBrowser/7.3.8581.400)*/
const Device UA2469294 = {S52994, 139, { &P16356,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-N7000 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2507090 = {S52995, 61, { &P16356,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-N7000 Build/JOP40D)*/
const Device UA2536176 = {S52996, 139, { &P16356,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7000 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567445 = {S52997, 138, { &P16356,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7000 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2737446 = {S52998, 139, { &P16356,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7000 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768643 = {S52999, 61, { &P16356,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-N7000 Build/JDQ39E)*/
const Device UA421354 = {S53000, 74, { &P15238,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-S5270L/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA641180 = {S53001, 82, { &P15238,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-S5270K/S5270KDDKK1 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA1986130 = {S53002, 74, { &P15238,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-S5270Y/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA118723 = {S53003, 62, { &P15364,&P21445,&P4809,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows ME) Opera 5.11 [en]*/
const Device UA152323 = {S53004, 31, { &P15364,&P21445,&P4809,&P18092 }};	/*Useragent - Opera/5.11 (Windows ME; U) [ru]*/
const Device UA271508 = {S53005, 63, { &P15364,&P21445,&P4809,&P18092 }};	/*Useragent - Mozilla/6.0 (compatible; MSIE 7.03; Windows ME) Opera 5.19 [jp]*/
const Device UA600145 = {S53006, 62, { &P15364,&P21445,&P4809,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows ME) Opera 5.12 [de]*/
const Device UA92805 = {S53007, 63, { &P18254,&P7127,&P4551,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; fr_crawler)*/
const Device UA372587 = {S53008, 66, { &P18254,&P7127,&P4551,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; fr_crawler)/V1*/
const Device UA376055 = {S53009, 93, { &P18254,&P7127,&P4551,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; FeedFinder-2.0; http://bloggz.se/crawler)*/
const Device UA538783 = {S53010, 97, { &P18254,&P7127,&P4551,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; BlogCrawler-Fetchy; http://bloggz.se/crawler)*/
const Device UA626941 = {S53011, 64, { &P18254,&P7127,&P4551,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; MSIECrawler)*/
const Device UA751299 = {S53012, 79, { &P18254,&P7127,&P4551,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; YComp 5.0.2.6; MSIECrawler)*/
const Device UA1874090 = {S53013, 94, { &P18254,&P7127,&P4551,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; FeedFetcher-2.0; http://bloggz.se/crawler)*/
const Device UA2354901 = {S53014, 186, { &P14862,&P5204,&P24672,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia2700c-2) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2675975 = {S53015, 130, { &P15919,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P3100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676504 = {S53016, 130, { &P15919,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3100 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2406359 = {S53017, 141, { &P18303,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT27i Build/6.1.1.B.1.54) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2483198 = {S53018, 141, { &P18303,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xperia Sola Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2870011 = {S53019, 104, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.402 Mobile Safari/537.35+*/
const Device UA2895613 = {S53020, 104, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.483 Mobile Safari/537.35+*/
const Device UA2898722 = {S53021, 104, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.552 Mobile Safari/537.35+*/
const Device UA2943722 = {S53022, 104, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.519 Mobile Safari/537.35+*/
const Device UA2973130 = {S53023, 104, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.810 Mobile Safari/537.35+*/
const Device UA2973807 = {S53024, 104, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.852 Mobile Safari/537.35+*/
const Device UA2997652 = {S53025, 105, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1337 Mobile Safari/537.35+*/
const Device UA3005151 = {S53026, 104, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.341 Mobile Safari/537.35+*/
const Device UA3024338 = {S53027, 104, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.936 Mobile Safari/537.35+*/
const Device UA3024383 = {S53028, 105, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1023 Mobile Safari/537.35+*/
const Device UA3026337 = {S53029, 105, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1047 Mobile Safari/537.35+*/
const Device UA3032329 = {S53030, 105, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1323 Mobile Safari/537.35+*/
const Device UA3036318 = {S53031, 105, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1425 Mobile Safari/537.35+*/
const Device UA3037101 = {S53032, 105, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1371 Mobile Safari/537.35+*/
const Device UA3037270 = {S53033, 105, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1155 Mobile Safari/537.35+*/
const Device UA3039440 = {S53034, 105, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1443 Mobile Safari/537.35+*/
const Device UA3058772 = {S53035, 105, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1113 Mobile Safari/537.35+*/
const Device UA3068025 = {S53036, 105, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1629 Mobile Safari/537.35+*/
const Device UA3068185 = {S53037, 105, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1581 Mobile Safari/537.35+*/
const Device UA3082216 = {S53038, 105, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1719 Mobile Safari/537.35+*/
const Device UA3083838 = {S53039, 105, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1671 Mobile Safari/537.35+*/
const Device UA3094868 = {S53040, 105, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1725 Mobile Safari/537.35+*/
const Device UA3096168 = {S53041, 105, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1737 Mobile Safari/537.35+*/
const Device UA3096699 = {S53042, 105, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1731 Mobile Safari/537.35+*/
const Device UA3110819 = {S53043, 105, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1761 Mobile Safari/537.35+*/
const Device UA3114345 = {S53044, 105, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1743 Mobile Safari/537.35+*/
const Device UA3129008 = {S53045, 105, { &P21285,&P22880,&P22893,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.2.0.1767 Mobile Safari/537.35+*/
const Device UA2422198 = {S53046, 121, { &P12581,&P5055,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; Nokia500) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile*/
const Device UA2423044 = {S53047, 40, { &P12581,&P5055,&P24673,&P18092 }};	/*Useragent - Nokia500UCBrowser/8.7.1.234/51/355/UCWEB*/
const Device UA2427232 = {S53048, 100, { &P12581,&P5055,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia500) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2371724 = {S53049, 115, { &P15364,&P16110,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/531.0 (KHTML, like Gecko) Chrome/3.0.190.1 Safari/531.0*/
const Device UA2030092 = {S53050, 129, { &P20352,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M-270 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2368714 = {S53051, 129, { &P20352,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M-270 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2845206 = {S53052, 58, { &P20352,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; M-270 Build/IML74K)*/
const Device UA606026 = {S53053, 136, { &P19850,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; X325a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2516127 = {S53054, 75, { &P15364,&P17285,&P20456,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:18.0) Gecko/X Firefox/18.0 SeaMonkey/2.15.2*/
const Device UA2567840 = {S53055, 43, { &P15364,&P18580,&P22901,&P18092 }};	/*Useragent - Mercury/7.3 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2728299 = {S53056, 122, { &P23283,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; Huawei; H883G; HuaweiH883G)*/
const Device UA352114 = {S53057, 147, { &P18292,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ609 4G Build/1.6.0M-218.14.Demo2-MZ609) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA355576 = {S53058, 141, { &P18292,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ609 4G Build/1.6.0M-218.14-MZ609) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA358718 = {S53059, 140, { &P18292,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ609 Build/1.6.0M-218.14-MZ609) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA626332 = {S53060, 140, { &P18292,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; MZ609 4G Build/1.6.0M_279_MZ609) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA727100 = {S53061, 139, { &P18292,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; MZ609 Build/1.6.0M_279_MZ609) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2034318 = {S53062, 68, { &P18292,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.2; MZ609 Build/1.6.0M_279_MZ609)*/
const Device UA2808434 = {S53063, 69, { &P18292,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; MZ609 Build/1.6.0M-218.14-MZ609)*/
const Device UA2257853 = {S53064, 158, { &P13999,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC HD2 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2427716 = {S53065, 159, { &P13999,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; NexusHD2 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2502264 = {S53066, 163, { &P13999,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC HD2 Build/JRO03R; CyanogenMod-cmtight_leo) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2662320 = {S53067, 169, { &P13999,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZeJstersHD2 Build/JZO54K; CyanogenMod-JellyTime_R32) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA68842 = {S53068, 103, { &P14630,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; SGH-i917R)*/
const Device UA2973662 = {S53069, 129, { &P22228,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SM-T210 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974139 = {S53070, 130, { &P22228,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SM-T210R Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA161973 = {S53071, 123, { &P17595,&P17294,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/3.0.4 Safari/523.12*/
const Device UA899130 = {S53072, 152, { &P20283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; AZ210A-ORANGE Build/AZ210A_GB_01.01I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA900832 = {S53073, 145, { &P20283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; AZ210A Build/AZ210A_GB_01.01I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA908293 = {S53074, 152, { &P20283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; AZ210B-ORANGE Build/AZ210B_GB_01.01I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033831 = {S53075, 145, { &P20283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; AZ210B Build/AZ210B_GB_01.01I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2337433 = {S53076, 152, { &P20283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; AZ210A-ORANGE Build/AZ210A_GB_01.06I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2366550 = {S53077, 152, { &P20283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; AZ210B-ORANGE Build/AZ210B_GB_01.06I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2527101 = {S53078, 145, { &P20283,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; AZ210A Build/AZ210A_GB_01.06I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2808759 = {S53079, 69, { &P20283,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; AZ210A Build/AZ210A_GB_01.01I)*/
const Device UA46622 = {S53080, 147, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA330810 = {S53081, 147, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA377041 = {S53082, 147, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA436619 = {S53083, 147, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA524665 = {S53084, 145, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA571313 = {S53085, 145, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8L1 Safari/6531.22.7*/
const Device UA639577 = {S53086, 148, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8J2 Safari/6531.22.7*/
const Device UA783719 = {S53087, 145, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8K2 Safari/6531.22.7*/
const Device UA815289 = {S53088, 130, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293*/
const Device UA835551 = {S53089, 130, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293*/
const Device UA849097 = {S53090, 145, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8H7 Safari/6531.22.7*/
const Device UA2139407 = {S53091, 143, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2142421 = {S53092, 143, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2369894 = {S53093, 143, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2374503 = {S53094, 130, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293*/
const Device UA2437859 = {S53095, 143, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2437982 = {S53096, 141, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2439236 = {S53097, 143, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8J2 Safari/6531.22.7*/
const Device UA2445444 = {S53098, 128, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293*/
const Device UA2472843 = {S53099, 145, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; vi-vn) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8J2 Safari/6531.22.7*/
const Device UA2472974 = {S53100, 145, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8K2 Safari/6531.22.7*/
const Device UA2477998 = {S53101, 145, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; vi-vn) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8F190 Safari/6531.22.7*/
const Device UA2520458 = {S53102, 130, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293*/
const Device UA2544560 = {S53103, 145, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-gb) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8k2 Safari/6531.22.7*/
const Device UA2639470 = {S53104, 147, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; es-es) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA2987156 = {S53105, 147, { &P12280,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; vi-vn) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8F190 Safari/6531.22.7*/
const Device UA2738180 = {S53106, 97, { &P17595,&P17296,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:21.0) Gecko/X Firefox/21.0 AlexaToolbar/alxf-2.18*/
const Device UA2740288 = {S53107, 147, { &P22850,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI H881C Build/HuaweiH881C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2201628 = {S53108, 72, { &P15364,&P5690,&P22365,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:15.0) Gecko/X Firefox/15.2 PaleMoon/15.2*/
const Device UA2358434 = {S53109, 96, { &P15364,&P5690,&P22365,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:15.2) Gecko/X Firefox/15.2.1-x64 PaleMoon/15.2.1-x64*/
const Device UA2456128 = {S53110, 83, { &P15364,&P5690,&P22365,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:15.2) Gecko/X Firefox/15.2.1 PaleMoon/15.2.1*/
const Device UA2577846 = {S53111, 98, { &P15364,&P17285,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.0; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727)*/
const Device UA2646904 = {S53112, 64, { &P15364,&P17285,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.0; Trident/6.0)*/
const Device UA2653366 = {S53113, 56, { &P15364,&P17285,&P17201,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.0; SV1)*/
const Device UA2694375 = {S53114, 152, { &P15364,&P17285,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.0; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA2950848 = {S53115, 179, { &P15364,&P17285,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.0; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2423297 = {S53116, 139, { &P15575,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100G Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2426173 = {S53117, 138, { &P15575,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2535458 = {S53118, 137, { &P15575,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-i9100 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2537062 = {S53119, 138, { &P15575,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100G Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2708531 = {S53120, 138, { &P15575,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39E) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2607496 = {S53121, 131, { &P20991,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ME371MG Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2469946 = {S53122, 64, { &P15364,&P7127,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:10.0.12) Gecko/X Firefox/10.0.12*/
const Device UA2354701 = {S53123, 133, { &P21334,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2366707 = {S53124, 141, { &P21334,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; QMobile A8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2466227 = {S53125, 55, { &P21334,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; A8 Build/IMM76D)*/
const Device UA6073 = {S53126, 168, { &P14873,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; Garminfone Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 MobileSafari/525.20.1; A50-V4.0.12-user-20100414*/
const Device UA31031 = {S53127, 169, { &P14873,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; Garminfone Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1; A50-V4.0.21-user-20100604*/
const Device UA446859 = {S53128, 161, { &P14873,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us;Garminfone Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1; A50-V4.0.12-user-X*/
const Device UA498790 = {S53129, 161, { &P14873,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 1.6; en-us; Garminfone Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1; A50-V4.0.21-user-X*/
const Device UA2018297 = {S53130, 161, { &P14873,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android1.6; en-us; Garminfone Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1; A50-V4.0.12-user-X*/
const Device UA2018687 = {S53131, 161, { &P14873,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; GarminfoneBuild/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1; A50-V4.0.12-user-X*/
const Device UA2932918 = {S53132, 164, { &P14873,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; fr-ca; n?vifone A50 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1; A50-V4.0.29-user-X*/
const Device UA433690 = {S53133, 36, { &P15364,&P17017,&P23048,&P18092 }};	/*Useragent - NSPlayer/12.0.7600.16385 WMFSDK/12.0*/
const Device UA515200 = {S53134, 24, { &P15364,&P17017,&P23048,&P18092 }};	/*Useragent - NSPlayer/12.0.7600.16385*/
const Device UA536789 = {S53135, 24, { &P15364,&P17017,&P23048,&P18092 }};	/*Useragent - NSPlayer/12.0.7601.17514*/
const Device UA2225003 = {S53136, 49, { &P15364,&P17017,&P23048,&P18092 }};	/*Useragent - NSPlayer/12.00.9200.16384 WMFSDK/12.00.9200.16384*/
const Device UA2261999 = {S53137, 49, { &P15364,&P17017,&P23048,&P18092 }};	/*Useragent - NSPlayer/12.00.9200.16420 WMFSDK/12.00.9200.16420*/
const Device UA2367272 = {S53138, 49, { &P15364,&P17017,&P23048,&P18092 }};	/*Useragent - NSPlayer/12.00.9200.16437 WMFSDK/12.00.9200.16437*/
const Device UA2401280 = {S53139, 49, { &P15364,&P17017,&P23048,&P18092 }};	/*Useragent - NSPlayer/12.00.9200.16409 WMFSDK/12.00.9200.16409*/
const Device UA2476120 = {S53140, 36, { &P15364,&P17017,&P23048,&P18092 }};	/*Useragent - NSPlayer/12.0.9200.16384 WMFSDK/12.0*/
const Device UA2692673 = {S53141, 24, { &P15364,&P17017,&P23048,&P18092 }};	/*Useragent - NSPlayer/12.0.9200.16384*/
const Device UA2709248 = {S53142, 67, { &P15364,&P17017,&P23048,&P18092 }};	/*Useragent - NSPlayer/12.00.7601.17514 WMFSDK/12.00.7601.17514 SE 2.X MetaSr 1.0*/
const Device UA2734575 = {S53143, 54, { &P15364,&P17017,&P23048,&P18092 }};	/*Useragent - NSPlayer/12.0.7601.17514 WMFSDK/12.0 SE 2.X MetaSr 1.0*/
const Device UA2766911 = {S53144, 67, { &P15364,&P17017,&P23048,&P18092 }};	/*Useragent - NSPlayer/12.00.7600.16597 WMFSDK/12.00.7600.16597 SE 2.X MetaSr 1.0*/
const Device UA2863871 = {S53145, 54, { &P15364,&P17017,&P23048,&P18092 }};	/*Useragent - NSPlayer/12.0.7600.16385 WMFSDK/12.0 SE 2.X MetaSr 1.0*/
const Device UA2891778 = {S53146, 67, { &P15364,&P17017,&P23048,&P18092 }};	/*Useragent - NSPlayer/12.00.7600.16385 WMFSDK/12.00.7600.16385 SE 2.X MetaSr 1.0*/
const Device UA2996870 = {S53147, 49, { &P15364,&P17017,&P23048,&P18092 }};	/*Useragent - NSPlayer/12.00.9200.16550 WMFSDK/12.00.9200.16550*/
const Device UA2353297 = {S53148, 182, { &P18267,&P5043,&P24672,&P18092 }};	/*Useragent - NokiaX2-02/2.0 (11.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-02) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA3065341 = {S53149, 150, { &P18552,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2204691 = {S53150, 114, { &P14714,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02/07.16; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.34*/
const Device UA2236091 = {S53151, 116, { &P14714,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2387382 = {S53152, 155, { &P14540,&P5843,&P24673,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiax2-01) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2491913 = {S53153, 149, { &P17346,&P19826,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B144 Safari/7534.48.3*/
const Device UA2509844 = {S53154, 149, { &P17346,&P19826,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B141 Safari/7534.48.3*/
const Device UA2532766 = {S53155, 151, { &P17346,&P19826,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B146 Safari/7534.48.3*/
const Device UA2607412 = {S53156, 151, { &P17346,&P19826,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/10B329 Safari/7534.48.3*/
const Device UA2362444 = {S53157, 182, { &P14635,&P5843,&P24672,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (08.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac3-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile*/
const Device UA2513078 = {S53158, 112, { &P19748,&P5043,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia205/03.18; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2706696 = {S53159, 114, { &P19748,&P5043,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia205.1/03.18; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA3065364 = {S53160, 136, { &P23776,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGP321 Build/10.3.1.A.0.244) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2741633 = {S53161, 147, { &P18283,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2753385 = {S53162, 147, { &P18283,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I535 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2772862 = {S53163, 147, { &P18283,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T999 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2938051 = {S53164, 147, { &P18283,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2998470 = {S53165, 148, { &P18283,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-I9300T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3057290 = {S53166, 148, { &P18283,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-T999V Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA636267 = {S53167, 73, { &P18481,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-C3520/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2974286 = {S53168, 136, { &P19911,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E970 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2390866 = {S53169, 155, { &P18267,&P5043,&P24673,&P18092 }};	/*Useragent - NokiaX2-02/2.0 (11.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiax2-02) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2752935 = {S53170, 138, { &P18280,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC VLE_U Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753070 = {S53171, 138, { &P18280,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755999 = {S53172, 138, { &P18280,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783279 = {S53173, 138, { &P18280,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC VLE_U Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2515423 = {S53174, 165, { &P19891,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SAMSUNG-SGH-I717 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2534894 = {S53175, 164, { &P19891,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I717 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2710947 = {S53176, 167, { &P19891,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I717 Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA75382 = {S53177, 121, { &P13035,&P5043,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-02/06.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.2.26.8*/
const Device UA2162506 = {S53178, 116, { &P13035,&P5043,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-02.1/07.48; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.9*/
const Device UA2740576 = {S53179, 91, { &P15364,&P7133,&P22767,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Maxthon/4.0.6.2000)*/
const Device UA2752839 = {S53180, 90, { &P15364,&P7133,&P22767,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Maxthon/4.0.6.2000)*/
const Device UA2769424 = {S53181, 83, { &P15364,&P7133,&P22767,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Maxthon/4.0.6.2000)*/
const Device UA2769663 = {S53182, 84, { &P15364,&P7133,&P22767,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Maxthon/4.0.6.2000)*/
const Device UA2796788 = {S53183, 198, { &P15364,&P7133,&P22767,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; Maxthon/4.0.6.2000)*/
const Device UA2937381 = {S53184, 181, { &P15364,&P7133,&P22767,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Maxthon/4.0.6.2000)*/
const Device UA2986202 = {S53185, 75, { &P15364,&P18110,&P20905,&P18092 }};	/*Useragent - Mozilla/3.0 (X11; U; Linux 2.4.2-2 i586; en-US; m18) Gecko/X Netscape6/6.01*/
const Device UA634218 = {S53186, 145, { &P20183,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA636192 = {S53187, 133, { &P20183,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA901801 = {S53188, 138, { &P20183,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1979422 = {S53189, 69, { &P20183,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; MI-ONE Plus Build/GINGERBREAD)*/
const Device UA2027848 = {S53190, 138, { &P20183,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2480637 = {S53191, 145, { &P20183,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2725644 = {S53192, 138, { &P20183,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MI-ONE Plus Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2889018 = {S53193, 140, { &P20183,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA50300 = {S53194, 12, { &P13979,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia6220c-1*/
const Device UA53456 = {S53195, 150, { &P13979,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia6220c/04.13 (SymbianOS/9.3; U; Series60/3.2 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA64772 = {S53196, 152, { &P13979,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia6220c-1/05.15; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA56150 = {S53197, 98, { &P14651,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; 7 Trophy)*/
const Device UA56175 = {S53198, 104, { &P14651,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; 7 Trophy T8686)*/
const Device UA925805 = {S53199, 98, { &P14651,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible: MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; 7 Trophy)*/
const Device UA925825 = {S53200, 97, { &P14651,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; PC40100)*/
const Device UA925848 = {S53201, 95, { &P14651,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Spark)*/
const Device UA2405802 = {S53202, 120, { &P13552,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaC6-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2422874 = {S53203, 42, { &P13552,&P5129,&P24673,&P18092 }};	/*Useragent - NokiaC6-00UCBrowser/8.7.1.234/50/444/UCWEB*/
const Device UA2458023 = {S53204, 102, { &P13552,&P5129,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac6-00) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2201568 = {S53205, 122, { &P12280,&P18432,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Flipboard/1.9.6*/
const Device UA2202147 = {S53206, 122, { &P12280,&P18432,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Flipboard/1.9.7*/
const Device UA2203197 = {S53207, 122, { &P12280,&P18432,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Flipboard/1.9.7*/
const Device UA2260974 = {S53208, 122, { &P12280,&P18432,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Flipboard/1.9.8*/
const Device UA2260977 = {S53209, 122, { &P12280,&P18432,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Flipboard/1.9.8*/
const Device UA2261073 = {S53210, 124, { &P12280,&P18432,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/1.9.8*/
const Device UA2261147 = {S53211, 124, { &P12280,&P18432,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Flipboard/1.9.8*/
const Device UA2356715 = {S53212, 124, { &P12280,&P18432,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/1.9.7*/
const Device UA2437568 = {S53213, 124, { &P12280,&P18432,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Flipboard/1.9.8*/
const Device UA2009386 = {S53214, 141, { &P11747,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC Desire Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257950 = {S53215, 141, { &P11747,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC Desire Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2353007 = {S53216, 141, { &P11747,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC Desire Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2476367 = {S53217, 141, { &P11747,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Desire Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2595712 = {S53218, 61, { &P11747,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC Desire Build/CM10)*/
const Device UA2835538 = {S53219, 142, { &P11747,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC Desire Build/unknown) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892389 = {S53220, 146, { &P11747,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; Desire_L_by_HTC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996409 = {S53221, 154, { &P11747,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_Desire_600_dual_sim Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3000016 = {S53222, 145, { &P11747,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; HTC_Desire_600 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078429 = {S53223, 145, { &P11747,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; HTC_Desire_500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2676199 = {S53224, 149, { &P18318,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF700T Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2737457 = {S53225, 149, { &P18318,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ASUS Transformer Pad TF700T Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2437697 = {S53226, 113, { &P19761,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HUAWEI; Ascend W1)*/
const Device UA2437696 = {S53227, 157, { &P19761,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HUAWEI; HUAWEI Ascend W1 A4BA5B9A-2CD4-430D-B216-EA57BAF03A15)*/
const Device UA2555410 = {S53228, 110, { &P19761,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HUAWEI; W1-U00)*/
const Device UA2876975 = {S53229, 125, { &P19761,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; Huawei; W1-U34; Huawei W1-U34)*/
const Device UA2986540 = {S53230, 110, { &P19761,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HUAWEI; W1-C00)*/
const Device UA3065317 = {S53231, 144, { &P21611,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065467 = {S53232, 136, { &P19911,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E970 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA178198 = {S53233, 127, { &P17595,&P17296,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0*/
const Device UA196400 = {S53234, 144, { &P17595,&P17296,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0*/
const Device UA200827 = {S53235, 144, { &P17595,&P17296,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0*/
const Device UA211834 = {S53236, 144, { &P17595,&P17296,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/531.21.8+(KHTML, like Gecko, Safari/528.16) Version/5.10.3 OmniWeb/622.14.0*/
const Device UA306834 = {S53237, 144, { &P17595,&P17296,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/531.21.8+(KHTML, like Gecko, Safari/528.16) Version/5.10.3 OmniWeb/622.14.0*/
const Device UA326372 = {S53238, 127, { &P17595,&P17296,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0*/
const Device UA359649 = {S53239, 146, { &P17595,&P17296,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0*/
const Device UA408016 = {S53240, 127, { &P17595,&P17296,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0*/
const Device UA491138 = {S53241, 127, { &P17595,&P17296,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0*/
const Device UA684273 = {S53242, 144, { &P17595,&P17296,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/531.21.8+(KHTML, like Gecko, Safari/528.16) Version/5.10.3 OmniWeb/622.14.0*/
const Device UA697468 = {S53243, 146, { &P17595,&P17296,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.17.0*/
const Device UA864455 = {S53244, 127, { &P17595,&P17296,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0*/
const Device UA876667 = {S53245, 144, { &P17595,&P17296,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/531.21.8+(KHTML, like Gecko, Safari/528.16) Version/5.10.3 OmniWeb/622.14.0*/
const Device UA1859295 = {S53246, 127, { &P17595,&P17296,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0*/
const Device UA1997050 = {S53247, 148, { &P17595,&P17296,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0*/
const Device UA2296054 = {S53248, 144, { &P17595,&P17296,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0*/
const Device UA2981113 = {S53249, 148, { &P17595,&P17296,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0*/
const Device UA668874 = {S53250, 81, { &P15364,&P7127,&P20917,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9.0.2) Gecko/X Firefox/3.1.6*/
const Device UA2421653 = {S53251, 147, { &P20524,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT626 Build/5.5.1Q-391_IR_TA-26) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2552437 = {S53252, 147, { &P20524,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; XT626 Build/5.5.1Q-391_IR_TA-30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2735504 = {S53253, 147, { &P20524,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; XT626 Build/5.5.1Q-391_IR_TA-72) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2987064 = {S53254, 147, { &P20524,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT626 Build/5.5.1Q-391_IR_TA-32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3008431 = {S53255, 147, { &P20524,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; XT626 Build/5.5.1Q-391_IR_TA-82) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA863909 = {S53256, 138, { &P21055,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CMP741d Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2724450 = {S53257, 136, { &P21055,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; CMP741d Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2587546 = {S53258, 76, { &P15364,&P18145,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; FreeBSD amd64; en-US; rv:1.9.0.7) Gecko/X Firefox/3.0.7*/
const Device UA605488 = {S53259, 162, { &P16292,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA788168 = {S53260, 162, { &P16292,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA909169 = {S53261, 162, { &P16292,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1981966 = {S53262, 165, { &P16292,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/GLAZE6-16; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2024518 = {S53263, 162, { &P16292,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76L; CyanogenMod-Glazed ICS 7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA10040 = {S53264, 149, { &P11880,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia5320/04.13 (SymbianOS/9.3; U; Series60/3.2 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA16397 = {S53265, 152, { &P11880,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia5320d-1/04.13; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA17260 = {S53266, 123, { &P11880,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia5320di/05.302 SymbianOS/9.3 Series60/3.2 Release/03.19.2009 Browser/Mozilla5.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA35694 = {S53267, 182, { &P11880,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 NokianOS/9.3; U; Series60/3.2 Nokia5320d-1/04.13; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA54444 = {S53268, 153, { &P11880,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia5320d-1b/03.08; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA62920 = {S53269, 165, { &P11880,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia 5320di XpressMusic/05.302; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA62921 = {S53270, 164, { &P11880,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia5320di XpressMusic/06.203; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA67849 = {S53271, 56, { &P11880,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia5320diXpressMusic(19.01)/SymbianOS/9.1 Series60/3.0*/
const Device UA77295 = {S53272, 155, { &P11880,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia5320_CMCC/06.103 (SymbianOS/9.3; U; Series60/3.2 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA88754 = {S53273, 43, { &P11880,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia5320(19.01)/SymbianOS/9.1 Series60/3.0*/
const Device UA26623 = {S53274, 41, { &P13611,&P17484,&P3625,&P18092 }};	/*Useragent - sam-r360 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA624826 = {S53275, 129, { &P18704,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-gb; AT200 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA739263 = {S53276, 127, { &P18704,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; AT200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2030625 = {S53277, 58, { &P18704,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; AT200 Build/HTK55D)*/
const Device UA2167466 = {S53278, 138, { &P18484,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-US730 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA38571 = {S53279, 152, { &P13691,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB508 Build/SAGE_U3_10.23.20) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA38572 = {S53280, 182, { &P13691,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB508 Build/Blur_Version.1.23.20.MB508.ATT.en.US Flex/P033) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA555556 = {S53281, 74, { &P13691,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; MB508 Build/SAGE_U3_10.23.20)*/
const Device UA30843 = {S53282, 40, { &P11751,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA64445 = {S53283, 52, { &P11751,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP500-Orange/V10i Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA64495 = {S53284, 43, { &P11751,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP500-CH Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA88841 = {S53285, 58, { &P11751,&P17483,&P17602,&P18092 }};	/*Useragent - Vodafone/1.0/LG-KP500/V10b/Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA2202618 = {S53286, 114, { &P13107,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-06/07.48; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2457760 = {S53287, 113, { &P13107,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-06/06.97; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5*/
const Device UA2754922 = {S53288, 130, { &P15919,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754942 = {S53289, 130, { &P15919,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2769530 = {S53290, 131, { &P15919,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100B Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA603306 = {S53291, 75, { &P15364,&P18110,&P18331,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.62*/
const Device UA630627 = {S53292, 65, { &P15364,&P18110,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; en) Presto/2.10.229 Version/11.62*/
const Device UA2196739 = {S53293, 138, { &P18666,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; Micromax A25 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2234913 = {S53294, 132, { &P18666,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A25 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2524285 = {S53295, 56, { &P18666,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; A25 Build/GRK39F)*/
const Device UA2982183 = {S53296, 120, { &P18666,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A25 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA749077 = {S53297, 81, { &P15364,&P16110,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.1.3) Gecko/X Firefox/3.5.3*/
const Device UA2671110 = {S53298, 130, { &P20146,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676178 = {S53299, 137, { &P20146,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676827 = {S53300, 138, { &P20146,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3087624 = {S53301, 131, { &P20146,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2886766 = {S53302, 112, { &P18643,&P5043,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia311/05.92; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA371590 = {S53303, 63, { &P15364,&P18110,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; de) Presto/2.2.15 Version/10.11*/
const Device UA810061 = {S53304, 108, { &P15364,&P18110,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; ADR-X; U; TSB_CLOUD_COMPANION;TOSHIBA_AC_AND_AZ; ru) Presto/2.5.28 Version/10.1*/
const Device UA1841859 = {S53305, 67, { &P15364,&P18110,&P18272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux i686; de) Opera 10.10*/
const Device UA2147449 = {S53306, 78, { &P15364,&P18110,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (X11; U; Linux i686; en-US; rv:1.9.2.3) Presto/2.2.15 Version/10.10*/
const Device UA604001 = {S53307, 117, { &P15364,&P5690,&P22457,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19*/
const Device UA2158807 = {S53308, 124, { &P15364,&P5690,&P22457,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19*/
const Device UA2335211 = {S53309, 117, { &P15364,&P5690,&P22457,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19*/
const Device UA2262158 = {S53310, 75, { &P17595,&P17297,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5) Presto/2.12.388 Version/12.10*/
const Device UA3066222 = {S53311, 137, { &P19920,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS870 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA58328 = {S53312, 105, { &P11660,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonC902c/R3EG Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.3*/
const Device UA63963 = {S53313, 104, { &P11660,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonC902/R3EA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.3*/
const Device UA2232991 = {S53314, 86, { &P15364,&P7133,&P19987,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.8.0)*/
const Device UA2356479 = {S53315, 79, { &P15364,&P7133,&P19987,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.8.1)*/
const Device UA2356480 = {S53316, 86, { &P15364,&P7133,&P19987,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.8.1)*/
const Device UA2362695 = {S53317, 86, { &P15364,&P7133,&P19987,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.8.2)*/
const Device UA2387598 = {S53318, 79, { &P15364,&P7133,&P19987,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.8.2)*/
const Device UA2406933 = {S53319, 79, { &P15364,&P7133,&P19987,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.8.3)*/
const Device UA2407326 = {S53320, 86, { &P15364,&P7133,&P19987,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.8.3)*/
const Device UA2429729 = {S53321, 86, { &P15364,&P7133,&P19987,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.8.4)*/
const Device UA2430925 = {S53322, 121, { &P15364,&P7133,&P19987,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 Sleipnir/3.8.4*/
const Device UA2437346 = {S53323, 114, { &P15364,&P7133,&P19987,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 Sleipnir/3.8.4*/
const Device UA2459983 = {S53324, 79, { &P15364,&P7133,&P19987,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.8.4)*/
const Device UA2839009 = {S53325, 79, { &P15364,&P7133,&P19987,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.8.0)*/
const Device UA2946994 = {S53326, 82, { &P18088,&P5567,&P17371,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/5.1/27.2338; U; en) Presto/2.8.119 240X320 Casio C781*/
const Device UA2261189 = {S53327, 182, { &P15228,&P5043,&P24672,&P18092 }};	/*Useragent - NokiaC2-03/2.0 (07.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-03) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile*/
const Device UA2873618 = {S53328, 141, { &P20277,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_H3000C Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483328 = {S53329, 137, { &P20226,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2557321 = {S53330, 138, { &P20226,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E975K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3073512 = {S53331, 138, { &P20226,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E975T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA442723 = {S53332, 89, { &P15364,&P5690,&P18199,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.21) Gecko/X Thunderbird/2.0.0.21*/
const Device UA472710 = {S53333, 89, { &P15364,&P5690,&P18199,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.8.1.23) Gecko/X Thunderbird/2.0.0.23*/
const Device UA135408 = {S53334, 84, { &P18254,&P5690,&P17193,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; Crawler; MSIE 7.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727)*/
const Device UA492025 = {S53335, 140, { &P18254,&P5690,&P17193,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSIECrawler)*/
const Device UA685753 = {S53336, 150, { &P18254,&P5690,&P17193,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.1) (TARGUSinfo Web Crawler www.targusinfo.com)*/
const Device UA804804 = {S53337, 76, { &P18254,&P5690,&P17193,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; MSIECrawler)*/
const Device UA2265533 = {S53338, 63, { &P18254,&P5690,&P17193,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; MSIECrawler)*/
const Device UA2273330 = {S53339, 159, { &P18254,&P5690,&P17193,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; MSIECrawler)*/
const Device UA3018313 = {S53340, 102, { &P18254,&P5690,&P17193,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Crayon Crawler; .NET CLR 1.1.4322);c2VvLWNyYXdsZXI=*/
const Device UA3057928 = {S53341, 162, { &P18254,&P5690,&P17193,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; iOpus-I-M; Infocrawler; SU 3.23; SIMBAR={29BC6696-626E-44F1-91BE-215A3CX}; Mozill;c2VvLWNyYXdsZXI=*/
const Device UA622984 = {S53342, 134, { &P19757,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Velocitymicro/T408) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2531753 = {S53343, 139, { &P12972,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-T989 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2536319 = {S53344, 138, { &P12972,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T989 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566228 = {S53345, 146, { &P12972,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-T989 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2663265 = {S53346, 60, { &P12972,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGH-T989 Build/JDQ39)*/
const Device UA2784909 = {S53347, 147, { &P12972,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-T989 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936777 = {S53348, 69, { &P12972,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SAMSUNG-SGH-T989 Build/JDQ39E)*/
const Device UA2557722 = {S53349, 77, { &P17595,&P21691,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro10%2C1)*/
const Device UA2580041 = {S53350, 71, { &P17595,&P21691,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (iMac12%2C2)*/
const Device UA2581337 = {S53351, 76, { &P17595,&P21691,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookAir5%2C2)*/
const Device UA2604431 = {S53352, 73, { &P17595,&P21691,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBook5%2C1)*/
const Device UA2698881 = {S53353, 72, { &P17595,&P21691,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacPro5%2C1)*/
const Device UA2712241 = {S53354, 73, { &P17595,&P21691,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (Macmini3%2C1)*/
const Device UA3066220 = {S53355, 137, { &P18301,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS840 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068315 = {S53356, 136, { &P18301,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS840PP Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2040800 = {S53357, 99, { &P15364,&P7127,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2529671 = {S53358, 118, { &P15364,&P7127,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 8.0; Windows NT 5.0) Chrome/21.0.1229.79*/
const Device UA2555747 = {S53359, 118, { &P15364,&P7127,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 7.0; Windows NT 5.0) Chrome/21.0.1229.79*/
const Device UA2737759 = {S53360, 123, { &P15364,&P7127,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 9.0; Windows NT 5.0WOW64) Chrome/21.0.1229.79*/
const Device UA2167799 = {S53361, 82, { &P17595,&P17297,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Macintosh; Intel Mac OS X 10_7_3; Trident/6.0)*/
const Device UA2753699 = {S53362, 121, { &P17968,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754420 = {S53363, 135, { &P17968,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; cm_tenderloin Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2769794 = {S53364, 152, { &P17968,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbla668_fwvga Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773344 = {S53365, 136, { &P17968,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; skz_tenderloin Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2782626 = {S53366, 128, { &P17968,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783171 = {S53367, 127, { &P17968,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; V1013 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2783354 = {S53368, 158, { &P17968,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1911_v77_gq2000_a41_6628_v20 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783835 = {S53369, 143, { &P17968,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbl1 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784533 = {S53370, 154, { &P17968,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1909l_v77_gq1008_a41_v20 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2800086 = {S53371, 154, { &P17968,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1908_v77_jbl1_9p017_6628 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2800676 = {S53372, 152, { &P17968,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1920_v77_zlh_9p017_asx Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867408 = {S53373, 147, { &P17968,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_gq2002tc Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877592 = {S53374, 136, { &P17968,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.5; iPhone5 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877839 = {S53375, 156, { &P17968,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Motorola Galaxy s3 Build/9.8.2O-72_VZW-8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879482 = {S53376, 158, { &P17968,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbla668_9p017_fwvga Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893293 = {S53377, 145, { &P17968,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_gq2008 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2938521 = {S53378, 148, { &P17968,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1920_v77_zlh_fwvga Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2999671 = {S53379, 146, { &P17968,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; e2001_v89_gq2008s Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3001125 = {S53380, 147, { &P17968,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; e2001_v89_jbl1a698 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA1959843 = {S53381, 80, { &P14188,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5233)/UC Browser8.5.0.183/50/355/UCWEB Mobile*/
const Device UA1986689 = {S53382, 31, { &P14188,&P5129,&P24671,&P18092 }};	/*Useragent - Nokia5233/UCWEB8.5.0.163/50/800*/
const Device UA2652431 = {S53383, 128, { &P14188,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokia5233) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile  3gpp-gba*/
const Device UA2784167 = {S53384, 139, { &P23169,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT914 Build/2_32A_2012) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2997918 = {S53385, 130, { &P24366,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SM-T311 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3007705 = {S53386, 59, { &P24366,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SM-T311 Build/JDQ39)*/
const Device UA2380771 = {S53387, 138, { &P20975,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-AS730 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2041239 = {S53388, 85, { &P15364,&P18140,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; Edition Linux Mint; en) Presto/2.10.289 Version/12.02*/
const Device UA899114 = {S53389, 160, { &P18472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR HD 4G Build/7.7.1Q-78_VQW_LE-27) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2206777 = {S53390, 157, { &P18472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR HD Build/7.7.1Q-78_VQW_LE-27) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2206774 = {S53391, 160, { &P18472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR HD 4G Build/7.7.1Q-126_VQW_RL-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2206793 = {S53392, 147, { &P18472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR HD Build/7.7.1Q_83) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2206800 = {S53393, 160, { &P18472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR HD 4G Build/7.7.1Q-78_VQW_LE-25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2206801 = {S53394, 158, { &P18472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR HD Build/7.7.1Q-144_VQW_S3-34) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2206806 = {S53395, 160, { &P18472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR HD Build/7.7.1Q-144_VQW_S334-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2206811 = {S53396, 160, { &P18472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR HD Build/7.7.1Q-144_VQW_S334-25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2206813 = {S53397, 151, { &P18472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR HD 4G Build/7.7.1Q_140) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257589 = {S53398, 153, { &P18472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; XT925 Build/7.7.1Q-144_VQU_S3-35-19) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257674 = {S53399, 161, { &P18472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR HD Build/7.7.1Q-144_VQW_S33425-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2356322 = {S53400, 83, { &P18472,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; DROID RAZR HD Build/7.7.1Q-144_VQW_S33425-2)*/
const Device UA2404144 = {S53401, 152, { &P18472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; XT925 Build/7.7.1Q-144_VQL_S3-49.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423468 = {S53402, 150, { &P18472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT925 Build/7.7.1Q-144_VQL_S3-58) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2456266 = {S53403, 156, { &P18472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; XT925 Build/7.7.1Q_144_VQU_DE_S3-35-26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2527325 = {S53404, 150, { &P18472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; XT925 Build/7.7.1Q-144_VQL_S3-49) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2552306 = {S53405, 75, { &P18472,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; XT925 Build/7.7.1Q-144_VQU_S3-35-19)*/
const Device UA2693754 = {S53406, 155, { &P18472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; XT925 Build/7.7.1Q-144_VQU_S3-35-1101) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2701462 = {S53407, 150, { &P18472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; XT925 Build/7.7.1Q-144_VQL_S3-61) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2746927 = {S53408, 90, { &P18472,&P18066,&P18062,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android 4.0.4;motorola DROID RAZR HD Build/7.7.1Q-144_VQW_S33425-2)*/
const Device UA2043550 = {S53409, 138, { &P18506,&P5043,&P24668,&P18092 }};	/*Useragent - Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia305) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA3044898 = {S53410, 199, { &P18506,&P5043,&P24668,&P18092 }};	/*Useragent - Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia305) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2033116 = {S53411, 142, { &P20452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Lenovo S880 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2167972 = {S53412, 133, { &P20452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S880 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2420519 = {S53413, 64, { &P20452,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Lenovo S880 Build/IML74K)*/
const Device UA2950810 = {S53414, 142, { &P20452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-mx; M4TEL SS880 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2532103 = {S53415, 130, { &P21769,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ME301T Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2569279 = {S53416, 66, { &P17968,&P17979,&P21777,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:19.0) Gecko/19.0 K9Browser/19.0.0*/
const Device UA2676291 = {S53417, 138, { &P18295,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One X Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676838 = {S53418, 138, { &P18295,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One X Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA624897 = {S53419, 136, { &P18622,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; IM-A820L Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1938460 = {S53420, 60, { &P18622,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; IM-A820L Build/GRJ90)*/
const Device UA1969947 = {S53421, 134, { &P18622,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; IM-A820L Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024812 = {S53422, 129, { &P18622,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; IM-A820L Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2555628 = {S53423, 149, { &P18622,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; IM-A820L Build/viethoa boi galeri) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2933926 = {S53424, 171, { &P18622,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; IM-A820L Build/GRJ90-1.53-B2 Vi?t h?a, Mod By Ho?ng H?i) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2934090 = {S53425, 161, { &P18622,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-cn; IM-A820L Build/Ver 1.53 B3 Final by Ho?ng H?i) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA639064 = {S53426, 113, { &P12989,&P5843,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia303/13.47; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.10*/
const Device UA2167090 = {S53427, 139, { &P18674,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7005 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2550496 = {S53428, 61, { &P18674,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-N7005 Build/IMM76D)*/
const Device UA2566775 = {S53429, 138, { &P19920,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS870 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667643 = {S53430, 131, { &P19920,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS870 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566705 = {S53431, 146, { &P13064,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2666880 = {S53432, 146, { &P13064,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I727 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2676037 = {S53433, 130, { &P12879,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676044 = {S53434, 130, { &P12879,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T859 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3039893 = {S53435, 83, { &P18141,&P24347,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 2.61) AppleWebKit/536.26 (KHTML, like Gecko) Silk/3.2*/
const Device UA2204241 = {S53436, 112, { &P15241,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia300/07.03; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2426610 = {S53437, 111, { &P15241,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia300/07.03; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5*/
const Device UA2491831 = {S53438, 83, { &P18141,&P20531,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 2.05) AppleWebKit/536.26 (KHTML, like Gecko) Silk/3.2*/
const Device UA2974490 = {S53439, 137, { &P16349,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Sony Tablet S Build/TISU0143) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA9714 = {S53440, 48, { &P13438,&P17484,&P5001,&P18092 }};	/*Useragent - SCH-U460L/1.0 NetFront/3.0.22.2.21 (GUI) MMP/2.0*/
const Device UA43563 = {S53441, 48, { &P13438,&P17484,&P5001,&P18092 }};	/*Useragent - SCH-U460P/1.0 NetFront/3.0.22.2.21 (GUI) MMP/2.0*/
const Device UA635322 = {S53442, 137, { &P19935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A818 Duo Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2423735 = {S53443, 150, { &P19935,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MyPhone A818 Slim Duo Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2809123 = {S53444, 61, { &P19935,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; A818 Duo Build/GRK39F)*/
const Device UA2387669 = {S53445, 118, { &P13977,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaN73) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile*/
const Device UA2422660 = {S53446, 40, { &P13977,&P5890,&P24673,&P18092 }};	/*Useragent - NokiaN73UCBrowser/8.7.1.234/28/444/UCWEB*/
const Device UA2362365 = {S53447, 138, { &P20898,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Acer-Z110 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2502931 = {S53448, 133, { &P20898,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; Z110 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2655076 = {S53449, 57, { &P20898,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; Z110 Build/GRK39F)*/
const Device UA2676085 = {S53450, 130, { &P15932,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3110 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676338 = {S53451, 130, { &P15932,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3110 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2368958 = {S53452, 139, { &P18716,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730M Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2487061 = {S53453, 161, { &P20407,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; ZTE V791 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/ZTE-Android-MMS-V2.0*/
const Device UA2557052 = {S53454, 132, { &P20407,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3;vi-vn;ZTE V791 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2580379 = {S53455, 137, { &P20407,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; ZTE V791 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2749803 = {S53456, 61, { &P20407,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; ZTE V791 Build/GRK39F)*/
const Device UA2751936 = {S53457, 136, { &P20407,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; ZTE V791 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2766053 = {S53458, 133, { &P20407,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3;en_us; ZTE V791 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3065457 = {S53459, 139, { &P18719,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C1505 Build/11.3.A.2.13) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA9405 = {S53460, 104, { &P11953,&P17485,&P4223,&P18092 }};	/*Useragent - SonyEricssonU10a/R7BA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2*/
const Device UA64503 = {S53461, 104, { &P11953,&P17485,&P4223,&P18092 }};	/*Useragent - SonyEricssonU10i/R7BA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2*/
const Device UA1996989 = {S53462, 152, { &P17283,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB860 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2012507 = {S53463, 152, { &P17283,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB860 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2205041 = {S53464, 152, { &P17283,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB860 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2233262 = {S53465, 152, { &P17283,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MB860 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2360880 = {S53466, 142, { &P19860,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Karbonn A9+ Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426710 = {S53467, 134, { &P19860,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A9+ Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2476361 = {S53468, 141, { &P19860,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; KPT A9PLUS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2560319 = {S53469, 146, { &P19860,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A9+(Flare S100) Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2744051 = {S53470, 127, { &P19860,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; KPT A9PLUS Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2854308 = {S53471, 139, { &P19860,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S100/A9+ Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2874511 = {S53472, 148, { &P19860,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CELKON A9+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2889407 = {S53473, 56, { &P19860,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; A9+ Build/IMM76I)*/
const Device UA1941348 = {S53474, 81, { &P15364,&P16110,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.0.3) Gecko/X Firefox/3.0.3*/
const Device UA2032120 = {S53475, 146, { &P13064,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I727 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2033263 = {S53476, 138, { &P13064,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I727 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2231074 = {S53477, 146, { &P13064,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261634 = {S53478, 138, { &P13064,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I727 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2974476 = {S53479, 138, { &P19845,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE N9120 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA14538 = {S53480, 48, { &P13513,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-Buckle/ UP.Browser/7.2.7.2.541 (GUI) MMP/2.0*/
const Device UA14537 = {S53481, 48, { &P13513,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-Buckle/ UP.Browser/7.2.7.2.549 (GUI) MMP/2.0*/
const Device UA33089 = {S53482, 55, { &P13513,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-Buckle/ABN1004 UP.Browser/7.2.7.2.551 (GUI) MMP/2.0*/
const Device UA137329 = {S53483, 55, { &P13513,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-Buckle/ABO1008 UP.Browser/7.2.7.2.551 (GUI) MMP/2.0*/
const Device UA207141 = {S53484, 55, { &P13513,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-Buckle/ABO1102 UP.Browser/7.2.7.2.551 (GUI) MMP/2.0*/
const Device UA229389 = {S53485, 55, { &P13513,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-Buckle/ABO1107 UP.Browser/7.2.7.2.551 (GUI) MMP/2.0*/
const Device UA2542585 = {S53486, 55, { &P13513,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-Buckle/ABN1104 UP.Browser/7.2.7.2.551 (GUI) MMP/2.0*/
const Device UA607780 = {S53487, 163, { &P15438,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Sensation Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA806775 = {S53488, 159, { &P15438,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Sensation Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031128 = {S53489, 163, { &P15438,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Sensation Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2377850 = {S53490, 155, { &P15438,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Sensation Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2260133 = {S53491, 54, { &P15364,&P18118,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:15.0) Gecko/X*/
const Device UA2487134 = {S53492, 48, { &P15364,&P18118,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:13.0)*/
const Device UA2719299 = {S53493, 173, { &P22600,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E300K/KKUAMDK Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2741260 = {S53494, 173, { &P22600,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E300K/KKUAME6 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2783710 = {S53495, 173, { &P22600,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E300K/KKUAME7 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2883128 = {S53496, 173, { &P22600,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E300K/KKUAMF3 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2969514 = {S53497, 173, { &P22600,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E300K/KKUAMG2 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3083584 = {S53498, 173, { &P22600,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E300K/KKUAMH2 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2974147 = {S53499, 134, { &P20872,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGP311 Build/10.1.C.0.370) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA117822 = {S53500, 129, { &P18061,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; sdk Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA139498 = {S53501, 136, { &P18061,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; google_sdk Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA158171 = {S53502, 60, { &P18061,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; google_sdk Build/FRF91)*/
const Device UA187598 = {S53503, 138, { &P18061,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; rk2818sdk Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA302898 = {S53504, 53, { &P18061,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; sdk Build/FRF91)*/
const Device UA2018095 = {S53505, 129, { &P18061,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; sdk Build/FRF42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018295 = {S53506, 137, { &P18061,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; rk2818sdk Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019217 = {S53507, 131, { &P18061,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; tr-tr; sdk Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2536822 = {S53508, 138, { &P18061,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U; Android 2.2; en-us; sdk Build/FRF91) AppleWebKit/533.1 BOLT/3.02 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3068055 = {S53509, 138, { &P18295,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068064 = {S53510, 138, { &P18295,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One X Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA625005 = {S53511, 136, { &P16345,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; el-gr; RACERII Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA643165 = {S53512, 136, { &P16345,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; RacerII Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA771790 = {S53513, 136, { &P16345,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; RACERII Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2474651 = {S53514, 136, { &P16345,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; de-at; RacerII Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2508300 = {S53515, 152, { &P18643,&P5043,&P24676,&P18092 }};	/*Useragent - Nokia311/5.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia311) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA623834 = {S53516, 132, { &P15072,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; E310 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1946265 = {S53517, 56, { &P15072,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; E310 Build/GRJ90)*/
const Device UA2022719 = {S53518, 132, { &P15072,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; E310 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2555074 = {S53519, 139, { &P21385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F240S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2729693 = {S53520, 61, { &P21385,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F240S Build/JZO54K)*/
const Device UA2480079 = {S53521, 148, { &P20552,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT621 Build/5.5.1Q_PRIMUS_TA-207) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2531008 = {S53522, 148, { &P20552,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT621 Build/5.5.1Q_PRIMUS_TA-215) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2659294 = {S53523, 148, { &P20552,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT621 Build/5.5.1Q_PRIMUS_TA-214) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3002362 = {S53524, 148, { &P20552,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT621 Build/5.5.1Q_PRIMUS_TA-229) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3024938 = {S53525, 148, { &P20552,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; XT621 Build/5.5.1Q_PRIMUS_MR-217) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA50976 = {S53526, 105, { &P11646,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonW595c/R3EG Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.3*/
const Device UA53978 = {S53527, 105, { &P11646,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonW595a/R3DA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.2*/
const Device UA64671 = {S53528, 104, { &P11646,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonW595/R3EG Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.3*/
const Device UA1994971 = {S53529, 60, { &P18506,&P5043,&P18517,&P18092 }};	/*Useragent - Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2744455 = {S53530, 61, { &P18506,&P5043,&P18517,&P18092 }};	/*Useragent - Nokia3050/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2952038 = {S53531, 121, { &P18506,&P5043,&P18517,&P18092 }};	/*Useragent - Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2487319 = {S53532, 154, { &P18295,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; One X Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2538606 = {S53533, 153, { &P18295,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One X Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556866 = {S53534, 158, { &P18295,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; HTC One X Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2565383 = {S53535, 157, { &P18295,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC One X Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2737836 = {S53536, 154, { &P18295,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One X Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2971371 = {S53537, 169, { &P18295,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC One X Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/endeavoru*/
const Device UA3076279 = {S53538, 156, { &P18295,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC One X Build/4.75; CyanogenMod-4.75) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA99374 = {S53539, 44, { &P15364,&P6704,&P4809,&P18092 }};	/*Useragent - Mozilla/4.76 (Windows 98; U) Opera 5.12 [en]*/
const Device UA261004 = {S53540, 31, { &P15364,&P6704,&P4809,&P18092 }};	/*Useragent - Opera/5.12 (Windows 98; U) [en]*/
const Device UA375295 = {S53541, 32, { &P15364,&P6704,&P4809,&P18092 }};	/*Useragent - Opera/5.02 (Windows 98; U)  [en]*/
const Device UA600499 = {S53542, 62, { &P15364,&P6704,&P4809,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows 98) Opera 5.02 [en]*/
const Device UA693102 = {S53543, 31, { &P15364,&P6704,&P4809,&P18092 }};	/*Useragent - Opera/5.11 (Windows 98; U) [en]*/
const Device UA731828 = {S53544, 62, { &P15364,&P6704,&P4809,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows 98) Opera 5.12 [en]*/
const Device UA2405462 = {S53545, 134, { &P21508,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GEM10313BK Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2654951 = {S53546, 63, { &P21508,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GEM10313BK Build/JRO03H)*/
const Device UA2970047 = {S53547, 129, { &P18639,&P21637,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus 10 Build/JWR66V) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.111 Safari/537.36*/
const Device UA2945130 = {S53548, 136, { &P18713,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P769 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36*/
const Device UA2974615 = {S53549, 136, { &P18713,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P768 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975806 = {S53550, 136, { &P18713,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P760 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA1894402 = {S53551, 155, { &P18254,&P7127,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.7.7) NimbleCrawler 1.11 obeys UserAgent NimbleCrawler For problems contact: crawler_at_dataalchemy.com*/
const Device UA2784969 = {S53552, 157, { &P18254,&P7127,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Mozilla/5.0 (Windows NT 5.0; rv:21.0) Gecko/X Firefox/21.0; +http://wiki.github.com/bixo/bixo/bixocrawler; bixo-dev@yahoogroups.com)*/
const Device UA2566515 = {S53553, 131, { &P15932,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P3110 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566643 = {S53554, 131, { &P15932,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3110 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2675684 = {S53555, 145, { &P18114,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Transformer Prime TF201 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676017 = {S53556, 145, { &P18114,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Transformer Prime TF201 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2583786 = {S53557, 149, { &P18318,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ASUS Transformer Pad TF700T Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2675428 = {S53558, 150, { &P18318,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF700T Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566509 = {S53559, 139, { &P23771,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567696 = {S53560, 132, { &P23771,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2753468 = {S53561, 136, { &P22596,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798630 = {S53562, 137, { &P22596,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970C Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA255859 = {S53563, 155, { &P16044,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Touchpad Build/GRJ22; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA364267 = {S53564, 171, { &P16044,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Touchpad Build/GRJ22; XRON/CM7 vB2.8 Odexed and bloated) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA911396 = {S53565, 139, { &P21949,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F100L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2442969 = {S53566, 61, { &P21949,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-F100L Build/IMM76D)*/
const Device UA662546 = {S53567, 125, { &P15364,&P17285,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/19.0.1084.46) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5*/
const Device UA897343 = {S53568, 89, { &P15364,&P17285,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/19.0.1084.52)*/
const Device UA897388 = {S53569, 96, { &P15364,&P17285,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/19.0.1084.52)*/
const Device UA897393 = {S53570, 125, { &P15364,&P17285,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/19.0.1084.52) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5*/
const Device UA897522 = {S53571, 132, { &P15364,&P17285,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/19.0.1084.52) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5*/
const Device UA897759 = {S53572, 144, { &P15364,&P17285,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/19.0.1084.52; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506)*/
const Device UA898780 = {S53573, 181, { &P15364,&P17285,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; chromeframe/19.0.1084.52; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA901924 = {S53574, 125, { &P15364,&P17285,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/19.0.1084.56) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA903930 = {S53575, 174, { &P15364,&P17285,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.3; chromeframe/19.0.1084.56; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)*/
const Device UA2163713 = {S53576, 163, { &P15364,&P17285,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/19.0.1084.52; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA2497211 = {S53577, 150, { &P15364,&P17285,&P21908,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; chromeframe/19.0.1084.52; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.21022; .NET CLR 3.0.30729)*/
const Device UA2259038 = {S53578, 64, { &P15364,&P5690,&P18691,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:16.0) Gecko/X Thunderbird/16.0.2*/
const Device UA2403310 = {S53579, 71, { &P15364,&P5690,&P18691,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:16.0) Gecko/X Thunderbird/16.0.2*/
const Device UA89805 = {S53580, 99, { &P17346,&P17144,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko)*/
const Device UA108095 = {S53581, 112, { &P17346,&P17144,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A293*/
const Device UA124988 = {S53582, 114, { &P17346,&P17144,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A400*/
const Device UA125866 = {S53583, 101, { &P17346,&P17144,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko)*/
const Device UA763813 = {S53584, 246, { &P17346,&P17144,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_0 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod2,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.0;FBSS/1; FBCR/;FBID/phone;FBLC/en_US;FBSF/1.0]*/
const Device UA783337 = {S53585, 250, { &P17346,&P17144,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_0_2 like Mac OS X; en_US) AppleWebKit (KHTML, like Gecko) Mobile [FBAN/FBForIPhone;FBAV/4.1.1;FBBV/4110.0;FBDV/iPod3,1;FBMD/iPod touch;FBSN/iPhone OS;FBSV/4.0.2;FBSS/1; FBCR/;FBID/phone;FBLC/en_US;FBSF/1.0]*/
const Device UA2261405 = {S53586, 140, { &P17346,&P17144,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A293 iPod2,1 BingWeb/3.03.1428.X*/
const Device UA899865 = {S53587, 137, { &P17948,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; SC-02C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033665 = {S53588, 135, { &P17948,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-; SC-02C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034490 = {S53589, 59, { &P17948,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SC-02C Build/IML74K)*/
const Device UA2511333 = {S53590, 160, { &P14862,&P5204,&P24676,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2700c-2) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2749051 = {S53591, 156, { &P14862,&P5204,&P24676,&P18092 }};	/*Useragent - Nokia2700c/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2700c) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA63151 = {S53592, 100, { &P11798,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-L700/DDID4 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA609296 = {S53593, 134, { &P17211,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  ARCHOS 101G9 Build/MR1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA901280 = {S53594, 133, { &P17211,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 101G9 Build/MR1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2261513 = {S53595, 136, { &P17211,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 101G9 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA70866 = {S53596, 87, { &P15875,&P17485,&P3499,&P18092 }};	/*Useragent - SonyEricssonCK15i/R3AE017 TelecaBrowser/Q07C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA396509 = {S53597, 87, { &P15875,&P17485,&P3499,&P18092 }};	/*Useragent - SonyEricssonCK15a/R3AC020 TelecaBrowser/Q07C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2026135 = {S53598, 133, { &P21103,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Next10P12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA194604 = {S53599, 141, { &P19789,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; KM-S200 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA569915 = {S53600, 141, { &P19789,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; KM-S220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA697574 = {S53601, 141, { &P19789,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; KM-S200 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1990351 = {S53602, 142, { &P19789,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; KM-S220H Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020176 = {S53603, 65, { &P19789,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; KM-S220 Build/GINGERBREAD)*/
const Device UA2022164 = {S53604, 65, { &P19789,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; KM-S200 Build/GINGERBREAD)*/
const Device UA2172968 = {S53605, 146, { &P18652,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Micromax A87 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2507912 = {S53606, 141, { &P18652,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A87 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2551951 = {S53607, 70, { &P18652,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Micromax A87 Build/GINGERBREAD)*/
const Device UA2827813 = {S53608, 134, { &P18652,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Micromax A87 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2882687 = {S53609, 114, { &P18267,&P5043,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-02/11.79; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA2387654 = {S53610, 118, { &P11854,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE71) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile*/
const Device UA2390368 = {S53611, 40, { &P11854,&P5890,&P24673,&P18092 }};	/*Useragent - NokiaE71UCBrowser/8.7.1.234/28/352/UCWEB*/
const Device UA2420678 = {S53612, 30, { &P11854,&P5890,&P24673,&P18092 }};	/*Useragent - NokiaE71/UCWEB8.7.0.187/28/999*/
const Device UA2429526 = {S53613, 92, { &P11854,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiae71-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2654381 = {S53614, 119, { &P11854,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE71x) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile*/
const Device UA3070888 = {S53615, 92, { &P11854,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiae71-2) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA1989 = {S53616, 89, { &P12056,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia7210Supernova/2.0 (04.86) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7210supernova*/
const Device UA2012 = {S53617, 70, { &P12056,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA36251 = {S53618, 67, { &P12056,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia7210c_CMCC/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA62548 = {S53619, 62, { &P12056,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia7210c/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2896425 = {S53620, 164, { &P23698,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG-SGH-I527 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3057301 = {S53621, 157, { &P23698,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-ca; SGH-I527M Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3078353 = {S53622, 164, { &P23698,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-ca; SGH-I527M-parrot Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA476615 = {S53623, 94, { &P15364,&P18110,&P22516,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.7pre) Gecko/X Firefox/2.0.0.6 Navigator/9.0b3*/
const Device UA902986 = {S53624, 107, { &P15364,&P22857,&P18373,&P18092 }};	/*Useragent - Browser/Opera Spice M6868n Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA2149501 = {S53625, 190, { &P15364,&P22857,&P18373,&P18092 }};	/*Useragent - Lenovo-TD39t_TD/S100 LMP/LML Release/2011.04.15 Browser/Opera Profile/MIDP2.0 Configuration/CLDC1.1 MBBMS/2.0 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; zh-CN) Presto/2.5.28 Version/10.10*/
const Device UA2157621 = {S53626, 168, { &P15364,&P22857,&P18373,&P18092 }};	/*Useragent - Huawei/1.0/HUAWEI-G6609   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA2165053 = {S53627, 168, { &P15364,&P22857,&P18373,&P18092 }};	/*Useragent - Huawei/1.0/HUAWEI-G6609   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; ro-RO) Presto/2.5.28 Version/10.10*/
const Device UA2219462 = {S53628, 168, { &P15364,&P22857,&P18373,&P18092 }};	/*Useragent - Huawei/1.0/HUAWEI-G6609   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; es-SA) Presto/2.5.28 Version/10.10*/
const Device UA2314379 = {S53629, 206, { &P15364,&P22857,&P18373,&P18092 }};	/*Useragent - Mot-WX308/v3 BTSTLE_G_0A.22.09RPS Release/10.11.2011 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA2331690 = {S53630, 168, { &P15364,&P22857,&P18373,&P18092 }};	/*Useragent - Huawei/1.0/HUAWEI-G6609   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; sr-SR) Presto/2.5.28 Version/10.10*/
const Device UA2346037 = {S53631, 324, { &P15364,&P22857,&P18373,&P18092 }};	/*Useragent - Fly_E154 ASTRO36_TD/v3 (MRE\2.3.00(20480) resolution\X chipset\MT6255 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; ru-RU) Presto/2.5.28 Version/10.10*/
const Device UA2379106 = {S53632, 194, { &P15364,&P22857,&P18373,&P18092 }};	/*Useragent - Huawei/1.0/HUAWEI-G6609/G6609V100R001C221B201SP01   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; pt-PT) Presto/2.5.28 Version/10.10*/
const Device UA2421440 = {S53633, 222, { &P15364,&P22857,&P18373,&P18092 }};	/*Useragent - Fly_E154 ASTRO36_TD/v3 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; ru-RU) Presto/2.5.28 Version/10.10*/
const Device UA2758357 = {S53634, 107, { &P15364,&P22857,&P18373,&P18092 }};	/*Useragent - Browser/Opera Spice M6868n Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; hi-IN) Presto/2.5.28 Version/10.10*/
const Device UA2777760 = {S53635, 168, { &P15364,&P22857,&P18373,&P18092 }};	/*Useragent - Huawei/1.0/HUAWEI-G6609   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; fr-FR) Presto/2.5.28 Version/10.10*/
const Device UA2789630 = {S53636, 168, { &P15364,&P22857,&P18373,&P18092 }};	/*Useragent - Huawei/1.0/HUAWEI-G6609   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; de-DE) Presto/2.5.28 Version/10.10*/
const Device UA2815567 = {S53637, 168, { &P15364,&P22857,&P18373,&P18092 }};	/*Useragent - Huawei/1.0/HUAWEI-G6609   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; bg-BG) Presto/2.5.28 Version/10.10*/
const Device UA3042236 = {S53638, 168, { &P15364,&P22857,&P18373,&P18092 }};	/*Useragent - Huawei/1.0/HUAWEI-G6609   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; it-IT) Presto/2.5.28 Version/10.10*/
const Device UA3066533 = {S53639, 147, { &P15877,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2938653 = {S53640, 137, { &P22126,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Torque Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2968736 = {S53641, 59, { &P22126,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Torque Build/JZO54K)*/
const Device UA633535 = {S53642, 87, { &P15364,&P7133,&P18198,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.28) Gecko/X Thunderbird/3.1.20*/
const Device UA2367501 = {S53643, 133, { &P20477,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ODYS-NOON Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2511351 = {S53644, 128, { &P20477,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; NOON Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2844015 = {S53645, 78, { &P20477,&P18111,&P18112,&P18092 }};	/*Useragent - AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; ODYS-NOON Build/JRO03H)*/
const Device UA3006758 = {S53646, 131, { &P20477,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; NOON970 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2566491 = {S53647, 138, { &P16349,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Sony Tablet S Build/TISU0143) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2428229 = {S53648, 132, { &P20154,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMP5770D Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA194830 = {S53649, 140, { &P17949,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SC-02B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625714 = {S53650, 140, { &P17949,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ja-jp; SC-02B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA754075 = {S53651, 64, { &P17949,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SC-02B Build/GINGERBREAD)*/
const Device UA852710 = {S53652, 64, { &P17949,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SC-02B Build/GINGERBREAD)*/
const Device UA2869811 = {S53653, 144, { &P22972,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I537 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2885079 = {S53654, 137, { &P22972,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I537 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3065325 = {S53655, 137, { &P19894,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066893 = {S53656, 138, { &P19894,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830C Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA729866 = {S53657, 116, { &P23331,&P24094,&P24304,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips ; U; HbbTV/1.1.1 (; Philips; ; ; ; ) CE-HTML/1.0 NETTV/3.2.1; en) Presto/2.6.33 Version/10.70*/
const Device UA2351627 = {S53658, 117, { &P23331,&P24094,&P24304,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; HbbTV/1.1.1 (; Philips; ; ; ; ) CE-HTML/1.0 NETTV/4.0.2; en) Presto/2.10.250 Version/11.60*/
const Device UA2205624 = {S53659, 150, { &P17346,&P17146,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9A405 Safari/7534.48.3*/
const Device UA2234492 = {S53660, 148, { &P17346,&P17146,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/9A334 Safari/7534.48.3*/
const Device UA2659443 = {S53661, 147, { &P17346,&P17146,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 501 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/21.0.1180.80 Mobile/9A405 Safari/7534.48.3*/
const Device UA2384945 = {S53662, 155, { &P20534,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PAP4040_DUO Build/PrestigioPAP4040DUO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2844901 = {S53663, 77, { &P20534,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; PAP4040_DUO Build/PrestigioPAP4040DUO)*/
const Device UA55019 = {S53664, 77, { &P13241,&P5026,&P3410,&P18092 }};	/*Useragent - BlackBerry8830/4.5.0.186 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105*/
const Device UA689286 = {S53665, 42, { &P11880,&P5890,&P24665,&P18092 }};	/*Useragent - NOKIA5320/UC Browser8.0.4.121/28/444/UCWEB*/
const Device UA2261406 = {S53666, 32, { &P11880,&P5890,&P24665,&P18092 }};	/*Useragent - Nokia5320di/UCWEB8.0.3.99/28/999*/
const Device UA160231 = {S53667, 147, { &P17595,&P17294,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PowerPC Mac OS X 10_4_11; en-US) AppleWebKit/531.21.8+(KHTML, like Gecko, Safari/528.16) Version/5.10.3 OmniWeb/622.14.0*/
const Device UA225389 = {S53668, 147, { &P17595,&P17294,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PowerPC Mac OS X 10_4_11; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0*/
const Device UA361072 = {S53669, 149, { &P17595,&P17294,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PowerPC Mac OS X 10_4_11; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0*/
const Device UA470814 = {S53670, 130, { &P17595,&P17294,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PowerPC Mac OS X 10_4_11; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0*/
const Device UA1996904 = {S53671, 151, { &P17595,&P17294,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PowerPC Mac OS X 10_4_11; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0*/
const Device UA2981061 = {S53672, 148, { &P17595,&P17294,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_8; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0*/
const Device UA2713489 = {S53673, 42, { &P17595,&P18580,&P18535,&P18092 }};	/*Useragent - Reeder/3.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2566506 = {S53674, 127, { &P16598,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A500 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2602043 = {S53675, 127, { &P16598,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A500 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2669365 = {S53676, 134, { &P16598,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A500 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2708144 = {S53677, 134, { &P16598,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Iconia A500 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2773487 = {S53678, 140, { &P17346,&P18432,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A523 Safari/8536.25*/
const Device UA2783340 = {S53679, 138, { &P17346,&P18432,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A406 Safari/8536.25*/
const Device UA2783574 = {S53680, 138, { &P17346,&P18432,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/10A403 Safari/8536.25*/
const Device UA14076 = {S53681, 68, { &P11787,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA14091 = {S53682, 85, { &P11787,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia3120classic/2.0 (07.16) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia3120classic*/
const Device UA2570041 = {S53683, 158, { &P15575,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2570264 = {S53684, 158, { &P15575,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2772904 = {S53685, 157, { &P15575,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2773089 = {S53686, 157, { &P15575,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2774067 = {S53687, 158, { &P15575,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100T Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2784826 = {S53688, 158, { &P15575,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2869274 = {S53689, 158, { &P15575,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100M Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2879296 = {S53690, 158, { &P15575,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA5465 = {S53691, 127, { &P12262,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-S8300-Vodafone/S8300BUIC6 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA8075 = {S53692, 127, { &P12262,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-S8300-Bouygues/S8300AGIC1 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA24167 = {S53693, 112, { &P12262,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-S8300T/1.0 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA29503 = {S53694, 125, { &P12262,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-S8300-ORANGE/S8300BVIE1 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA62379 = {S53695, 112, { &P12262,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-S8300H/1.0 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA63287 = {S53696, 111, { &P12262,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-S8300/1.0 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA2974787 = {S53697, 146, { &P17989,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N8000_USA_Cricket Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974997 = {S53698, 146, { &P17989,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V8000_USA_Cricket Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3063563 = {S53699, 178, { &P24456,&P18066,&P18062,&P18163 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-us; Tuna Build/IFK77E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.67 commoncrawl.org/research//Nutch-1.7-SNAPSHOT*/
const Device UA839488 = {S53700, 129, { &P20588,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN9G3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2753275 = {S53701, 137, { &P22584,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337M Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2673946 = {S53702, 140, { &P16362,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710BST Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676348 = {S53703, 137, { &P16362,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676604 = {S53704, 141, { &P16362,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710VMUB Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676850 = {S53705, 130, { &P16362,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2709240 = {S53706, 134, { &P16362,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710VMUB Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2862518 = {S53707, 137, { &P16362,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SPH-D710 Build/JRO03R) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA100382 = {S53708, 60, { &P18088,&P17017,&P18124,&P18092 }};	/*Useragent - Opera/8.01 (J2ME/MIDP; Opera Mini/1.2.3214/1724; en; U; ssr)*/
const Device UA607041 = {S53709, 137, { &P21115,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; M650 Build/HuaweiM650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA630530 = {S53710, 137, { &P21115,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; M650 Build/HuaweiM650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2484305 = {S53711, 157, { &P14186,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-S5830 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2767454 = {S53712, 155, { &P14186,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-CA; GT-S5830 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2851934 = {S53713, 157, { &P14186,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-S5830 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3012644 = {S53714, 163, { &P14186,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-S5830 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/cooper*/
const Device UA2974794 = {S53715, 142, { &P19763,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6602 Build/10.1.1.A.1.253) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974905 = {S53716, 141, { &P19763,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-02E Build/10.1.D.0.343) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA905104 = {S53717, 142, { &P15802,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC_Amaze_4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA909212 = {S53718, 138, { &P15802,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Ruby Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA909403 = {S53719, 142, { &P15802,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Amaze 4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2424598 = {S53720, 135, { &P15802,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Amaze Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2424786 = {S53721, 161, { &P15802,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Ruby Build/Speed Rom 8.0 By EclipzeRemix) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2431188 = {S53722, 83, { &P18088,&P5567,&P17369,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/4.4.29939/28.3392; U; id) Presto/2.8.119 Version/11.10*/
const Device UA2432411 = {S53723, 157, { &P12814,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I777 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2514408 = {S53724, 157, { &P12814,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I777 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2536601 = {S53725, 156, { &P12814,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I777 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719450 = {S53726, 157, { &P12814,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I777 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2436610 = {S53727, 136, { &P12280,&P18117,&P22900,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.2 Mobile/9B206 Safari/7534.48.3*/
const Device UA2437193 = {S53728, 134, { &P12280,&P18117,&P22900,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.2 Mobile/9B176 Safari/7534.48.3*/
const Device UA2708245 = {S53729, 119, { &P18254,&P18110,&P22919,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.10.0 (development) Safari/534.34*/
const Device UA3018290 = {S53730, 113, { &P18254,&P18110,&P22919,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.10.0 (development) Safari/534.34*/
const Device UA3029153 = {S53731, 138, { &P17346,&P18432,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A406 Safari/8536.25*/
const Device UA3029363 = {S53732, 140, { &P17346,&P18432,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A523 Safari/8536.25*/
const Device UA3029374 = {S53733, 138, { &P17346,&P18432,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/10A403 Safari/8536.25*/
const Device UA3024808 = {S53734, 111, { &P18644,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia112/03.32; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA2233605 = {S53735, 142, { &P20231,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Karbonn A21 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2404503 = {S53736, 134, { &P20231,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A21 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2698113 = {S53737, 175, { &P20231,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Karbonn A21 Themed and Tweaked by Santo_Rock Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778498 = {S53738, 56, { &P20231,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; A21 Build/IMM76I)*/
const Device UA2798810 = {S53739, 105, { &P19958,&P17484,&P17373,&P18092 }};	/*Useragent - SAMSUNG-GT-S5220 Opera/9.80 (J2ME/MIDP; Opera Mini/6.5.28754/30.3214; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2531003 = {S53740, 74, { &P15364,&P18110,&P21791,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:19.0) Gecko/X Firefox/19.0 SeaMonkey/2.16*/
const Device UA2508328 = {S53741, 114, { &P14714,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02/07.16; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2536782 = {S53742, 116, { &P14714,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA19832 = {S53743, 136, { &P14207,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; ADR6350 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA19869 = {S53744, 139, { &P14207,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; pcdadr6350 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58449 = {S53745, 138, { &P14207,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; pcdadr6350 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA58448 = {S53746, 138, { &P14207,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; pcdadr6350Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58451 = {S53747, 137, { &P14207,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ko-; pcdadr6350 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58465 = {S53748, 138, { &P14207,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.1; en-us; pcdadr6350 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58473 = {S53749, 138, { &P14207,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.1; en-us; pcdadr6350 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58474 = {S53750, 138, { &P14207,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;pcdadr6350 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA317891 = {S53751, 60, { &P14207,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; ADR6350 Build/FRG83D)*/
const Device UA2233695 = {S53752, 132, { &P20132,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U; Android 4.0.4; engb; Build/IMM76D) AppleWebKit/534.30 (KHTML,like Gecko) Version/4.0 Safari/534.30;SurfTab_7.0*/
const Device UA2367284 = {S53753, 140, { &P20132,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; ST70104-1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2428478 = {S53754, 132, { &P20132,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U; Android 4.0.4; engb; SurfTab_7.0 Build/IMM76D) AppleWebKit/534.30 (KHTML,like Gecko) Version/4.0 Safari/534.30*/
const Device UA2431271 = {S53755, 229, { &P13122,&P5629,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-B5510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2431428 = {S53756, 229, { &P13122,&P5629,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-B5510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2567403 = {S53757, 84, { &P15364,&P7133,&P22559,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:19.0) Gecko/X Firefox/19.0 IceDragon/19.0.1.0*/
const Device UA205702 = {S53758, 122, { &P15364,&P7133,&P22450,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Iron/12.0.750.0 Chrome/12.0.750.0 Safari/534.30*/
const Device UA207224 = {S53759, 115, { &P15364,&P7133,&P22450,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.30 (KHTML, like Gecko) Iron/12.0.750.0 Chrome/12.0.750.0 Safari/534.30*/
const Device UA624196 = {S53760, 65, { &P15364,&P18110,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; en) Presto/2.10.229 Version/11.61*/
const Device UA838518 = {S53761, 73, { &P15364,&P18110,&P3403,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.61*/
const Device UA2668066 = {S53762, 83, { &P18141,&P22274,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 2.10) AppleWebKit/536.26 (KHTML, like Gecko) Silk/3.2*/
const Device UA1873892 = {S53763, 178, { &P15364,&P17285,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; Avant Browser; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; .NET CLR 3.5.21022; InfoPath.2)*/
const Device UA1994535 = {S53764, 78, { &P15364,&P17285,&P22731,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Avant Browser)*/
const Device UA2165922 = {S53765, 78, { &P15364,&P17285,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Avant Browser)*/
const Device UA2788373 = {S53766, 83, { &P15364,&P17285,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Avant Browser; SV1)*/
const Device UA2789196 = {S53767, 93, { &P15364,&P17285,&P22731,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Avant Browser; Avant Browser)*/
const Device UA2950123 = {S53768, 83, { &P15364,&P17285,&P22731,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; SV1; Avant Browser)*/
const Device UA2950233 = {S53769, 93, { &P15364,&P17285,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 6.0; Avant Browser; Avant Browser)*/
const Device UA2681028 = {S53770, 139, { &P22390,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S5310 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2961813 = {S53771, 159, { &P22390,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SAMSUNG GT-S5310/S5310XXAME2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3015945 = {S53772, 61, { &P22390,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S5310 Build/JZO54K)*/
const Device UA3017043 = {S53773, 140, { &P22390,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-S5310L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057775 = {S53774, 140, { &P22390,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; GT-S5310B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058236 = {S53775, 159, { &P22390,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ie; SAMSUNG GT-S5310/S5310XXAMF1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3059797 = {S53776, 140, { &P22390,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S5310G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125444 = {S53777, 159, { &P22390,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; el-gr; SAMSUNG GT-S5310/S5310XXAME5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA156452 = {S53778, 132, { &P17949,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; SC-02B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA175699 = {S53779, 134, { &P17949,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SC-02B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625182 = {S53780, 58, { &P17949,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SC-02B Build/FROYO)*/
const Device UA765987 = {S53781, 56, { &P17949,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; SC-02B Build/FROYO)*/
const Device UA2018195 = {S53782, 119, { &P17949,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; SC-02B Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile*/
const Device UA2755354 = {S53783, 137, { &P21607,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; VS870 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA64334 = {S53784, 41, { &P14589,&P17484,&P3625,&P18092 }};	/*Useragent - sam-r350 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA105060 = {S53785, 114, { &P15364,&P17285,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Version/3.1.2 Safari/525.21*/
const Device UA263490 = {S53786, 114, { &P15364,&P17285,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13.3*/
const Device UA263793 = {S53787, 114, { &P15364,&P17285,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.17*/
const Device UA141393 = {S53788, 62, { &P15364,&P7127,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.6.30 Version/10.62*/
const Device UA152156 = {S53789, 62, { &P15364,&P7127,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.6.30 Version/10.63*/
const Device UA268428 = {S53790, 62, { &P15364,&P7127,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.6.30 Version/10.60*/
const Device UA272607 = {S53791, 62, { &P15364,&P7127,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.6.30 Version/10.61*/
const Device UA2974325 = {S53792, 147, { &P15877,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2753268 = {S53793, 137, { &P19920,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS870 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753451 = {S53794, 158, { &P18472,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR HD Build/7.7.1Q-144_VQW_S334-25) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754634 = {S53795, 148, { &P18472,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT925 Build/7.7.1Q-144_VQL_S3-58) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA49396 = {S53796, 134, { &P14872,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SPH-D700 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA64468 = {S53797, 136, { &P14872,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SPH-D700 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA160367 = {S53798, 60, { &P14872,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SPH-D700 Build/FROYO)*/
const Device UA902685 = {S53799, 118, { &P15364,&P17285,&P22375,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.4.1.1000 Chrome/18.0.966.0 Safari/535.12*/
const Device UA906588 = {S53800, 125, { &P15364,&P17285,&P22375,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.4.1.1000 Chrome/18.0.966.0 Safari/535.12*/
const Device UA112082 = {S53801, 118, { &P15364,&P7133,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1*/
const Device UA145026 = {S53802, 116, { &P15364,&P7133,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.2.3 Safari/525.29*/
const Device UA259535 = {S53803, 116, { &P15364,&P7133,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/525.28 (KHTML, like Gecko) Version/3.2.2 Safari/525.28.1*/
const Device UA387194 = {S53804, 117, { &P15364,&P7133,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.13*/
const Device UA724853 = {S53805, 112, { &P15364,&P7133,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/531.0 (KHTML, like Gecko) Version/3.2.1 Safari/531.0*/
const Device UA2304293 = {S53806, 128, { &P15364,&P7133,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/531.3 (KHTML, like Gecko) Version/3.2.1 Safari/531.3 Awesomium/1.5.1*/
const Device UA2758212 = {S53807, 112, { &P15364,&P7133,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Version/3.2.1 Safari/532.0*/
const Device UA901294 = {S53808, 139, { &P20947,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTCEVOV4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2487641 = {S53809, 117, { &P15364,&P5690,&P22464,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17*/
const Device UA2598101 = {S53810, 117, { &P15364,&P5690,&P22464,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17*/
const Device UA2660052 = {S53811, 124, { &P15364,&P5690,&P22464,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17*/
const Device UA925791 = {S53812, 102, { &P18560,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; GT-S7530)*/
const Device UA925869 = {S53813, 103, { &P18560,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; GT-S7530E)*/
const Device UA2415238 = {S53814, 103, { &P18560,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; GT-S7530L)*/
const Device UA2798753 = {S53815, 139, { &P18452,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT29i Build/9.1.B.1.67) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2892349 = {S53816, 138, { &P23521,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9195 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937156 = {S53817, 60, { &P23521,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9195 Build/JDQ39)*/
const Device UA3108760 = {S53818, 139, { &P23521,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; es-us; GT-I9195L Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121799 = {S53819, 139, { &P23521,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; GT-I9195T Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2205189 = {S53820, 146, { &P14708,&P5204,&P24668,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5130c-2) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2721954 = {S53821, 140, { &P14708,&P5204,&P24668,&P18092 }};	/*Useragent - Nokia5130/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia5130) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2972284 = {S53822, 140, { &P16292,&P21637,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Galaxy Nexus Build/JWR66V) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA43723 = {S53823, 163, { &P17944,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; USCCADR6275US Carrier ID 45 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2741604 = {S53824, 160, { &P12280,&P19826,&P22783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.2.1 (like Version/10B350 Mobile/6.1.4 Safari/8536.25)*/
const Device UA2755138 = {S53825, 160, { &P12280,&P19826,&P22783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.2.1 (like Version/10B329 Mobile/6.1.3 Safari/8536.25)*/
const Device UA2769105 = {S53826, 160, { &P12280,&P19826,&P22783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.2.1 (like Version/10B146 Mobile/6.1.2 Safari/8536.25)*/
const Device UA2581503 = {S53827, 77, { &P17595,&P21691,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro10%2C1)*/
const Device UA2606345 = {S53828, 71, { &P17595,&P21691,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (iMac12%2C2)*/
const Device UA2669037 = {S53829, 76, { &P17595,&P21691,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookAir5%2C2)*/
const Device UA2703277 = {S53830, 72, { &P17595,&P21691,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacPro3%2C1)*/
const Device UA2705232 = {S53831, 73, { &P17595,&P21691,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (Macmini6%2C2)*/
const Device UA2767257 = {S53832, 73, { &P17595,&P21691,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBook5%2C1)*/
const Device UA3027630 = {S53833, 139, { &P17346,&P18117,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9B176 Safari/7534.48.3*/
const Device UA3028695 = {S53834, 141, { &P17346,&P18117,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9B206 Safari/7534.48.3*/
const Device UA2599456 = {S53835, 133, { &P18465,&P18110,&P21707,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Safari/535.19 Silk-Accelerated=false*/
const Device UA351158 = {S53836, 146, { &P13633,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; MotoA953 Build/MILS2_U6_4.1-22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA717489 = {S53837, 142, { &P13633,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; A953 Build/4.5.2-51_M2L-27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA759368 = {S53838, 146, { &P13633,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; A953 Build/4.5.2-51_M2L-27_R01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019833 = {S53839, 136, { &P13633,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; pt-br; MotoA953 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022377 = {S53840, 70, { &P13633,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; MotoA953 Build/MILS2_U6_4.1-22)*/
const Device UA2022393 = {S53841, 134, { &P13633,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; A953 Build/WajkIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034585 = {S53842, 146, { &P13633,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; A953 Build/4.5.2-51_M2L-27_R02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2372319 = {S53843, 146, { &P13633,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; A953 Build/4.5.2-51_M2L-27_R03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3065471 = {S53844, 146, { &P17989,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V8000_USA_Cricket Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066211 = {S53845, 146, { &P17989,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N8000_USA_Cricket Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA641048 = {S53846, 130, { &P19852,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-P7511 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2021463 = {S53847, 59, { &P19852,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P7511 Build/HTJ85B)*/
const Device UA2610246 = {S53848, 137, { &P19852,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-P7511 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2617704 = {S53849, 141, { &P19852,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-P7511 Build/HTJ85B.XXLA4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2892568 = {S53850, 164, { &P23697,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9192 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA6804 = {S53851, 64, { &P14664,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2330c-2/2.0 (08.20) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA27503 = {S53852, 77, { &P14664,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2330c-2/2.0 (09.85) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2330c-2*/
const Device UA43725 = {S53853, 62, { &P14664,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2330c/2.0 (08.20) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63395 = {S53854, 65, { &P14664,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2330c-2b/2.0 (08.13) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2707894 = {S53855, 133, { &P18465,&P18110,&P22587,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Safari/535.19 Silk-Accelerated=false*/
const Device UA3065248 = {S53856, 152, { &P20192,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Orange Daytona Build/HuaweiG510-0200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067920 = {S53857, 154, { &P20192,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0200 Build/HuaweiG510-0200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068062 = {S53858, 154, { &P20192,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0100 Build/HuaweiG510-0100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081110 = {S53859, 154, { &P20192,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0251 Build/HuaweiG510-0251) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126472 = {S53860, 154, { &P20192,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA428900 = {S53861, 137, { &P15364,&P17816,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.11 (KHTML, like Gecko) Ubuntu/10.04 Chromium/17.0.963.79 Chrome/17.0.963.79 Safari/535.11*/
const Device UA428923 = {S53862, 135, { &P15364,&P17816,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.11 (KHTML, like Gecko) Ubuntu/10.04 Chromium/17.0.963.79 Chrome/17.0.963.79 Safari/535.11*/
const Device UA3064100 = {S53863, 107, { &P17968,&P17979,&P23665,&P18163 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:21.0) Gecko/21.0 Firefox/21.0 commoncrawl.org/research//Nutch-1.7-SNAPSHOT*/
const Device UA53974 = {S53864, 29, { &P17906,&P17017,&P4626,&P18092 }};	/*Useragent - HUAWEI-M735/001.00 Opera/9.70*/
const Device UA2425831 = {S53865, 140, { &P15972,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One XL Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2780623 = {S53866, 141, { &P15972,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC EVA_UTL Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2973959 = {S53867, 129, { &P16976,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT300SE Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974470 = {S53868, 127, { &P16976,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT300 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA31247 = {S53869, 79, { &P12410,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-T139/T139UVJC1 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2433645 = {S53870, 157, { &P12496,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I897 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2533293 = {S53871, 157, { &P12496,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I897 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2564106 = {S53872, 156, { &P12496,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I897 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2970423 = {S53873, 171, { &P12496,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I897 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/captivatemtd*/
const Device UA2568103 = {S53874, 130, { &P23253,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID704 Build/MID704) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2858515 = {S53875, 75, { &P23253,&P18111,&P18112,&P18092 }};	/*Useragent - AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; MID704 Build/MID704)*/
const Device UA3108451 = {S53876, 130, { &P23253,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; MID704 Build/Tab746) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3125964 = {S53877, 130, { &P23253,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; Tab746 Build/MID704) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA78040 = {S53878, 141, { &P18084,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Transformer TF101G Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA332254 = {S53879, 140, { &P18084,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Transformer TF101G Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA729583 = {S53880, 70, { &P18084,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; Transformer TF101G Build/HTK75)*/
const Device UA2021338 = {S53881, 158, { &P18084,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1 KRAKD by jcarrz1; en-us; Transformer TF101G Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2214232 = {S53882, 87, { &P15364,&P5690,&P21955,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2 x64; en-US; rv:1.9.0.7) Gecko/X Minefield/3.0.7*/
const Device UA2682440 = {S53883, 87, { &P15364,&P5690,&P21955,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9a8pre) Gecko/X Minefield/3.0a8pre*/
const Device UA2720439 = {S53884, 81, { &P15364,&P5690,&P21955,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9a1) Gecko/X Minefield/3.0a1*/
const Device UA3064105 = {S53885, 190, { &P12280,&P17144,&P4201,&P18163 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293 Safari/6531.22.7 commoncrawl.org/research//Nutch-1.7-SNAPSHOT*/
const Device UA912773 = {S53886, 118, { &P15364,&P5690,&P22460,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11*/
const Device UA2160826 = {S53887, 125, { &P15364,&P5690,&P22460,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11*/
const Device UA2223182 = {S53888, 118, { &P15364,&P5690,&P22460,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11*/
const Device UA30792 = {S53889, 61, { &P15209,&P5768,&P4030,&P18092 }};	/*Useragent - Nokia6233/2.0 (05.60) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2406809 = {S53890, 60, { &P15364,&P18146,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; SunOS i86pc; rv:17.0) Gecko/X Firefox/17.0*/
const Device UA183709 = {S53891, 134, { &P17346,&P17090,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190 Twitter for iPhone*/
const Device UA183838 = {S53892, 134, { &P17346,&P17090,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 Twitter for iPhone*/
const Device UA193875 = {S53893, 134, { &P17346,&P17090,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Twitter for iPhone*/
const Device UA225453 = {S53894, 134, { &P17346,&P17090,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8K2 Twitter for iPhone*/
const Device UA233388 = {S53895, 134, { &P17346,&P17090,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Twitter for iPhone*/
const Device UA313518 = {S53896, 134, { &P17346,&P17090,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 Twitter for iPhone*/
const Device UA395455 = {S53897, 76, { &P17595,&P18120,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X; U; en-Us) Presto/2.5.24 Version/10.53*/
const Device UA624292 = {S53898, 71, { &P17595,&P18120,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; PPC Mac OS X; U; de) Presto/2.5.24 Version/10.54*/
const Device UA801988 = {S53899, 73, { &P17595,&P18120,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X; U; en) Presto/2.5.24 Version/10.52*/
const Device UA887217 = {S53900, 130, { &P20726,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN9G2I Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033283 = {S53901, 129, { &P20726,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; AN9G2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2589480 = {S53902, 59, { &P20726,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; AN9G2I Build/IML74K)*/
const Device UA3064095 = {S53903, 115, { &P15364,&P7133,&P17200,&P18163 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0) commoncrawl.org/research//Nutch-1.7-SNAPSHOT*/
const Device UA245176 = {S53904, 120, { &P15364,&P7133,&P22452,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1*/
const Device UA245196 = {S53905, 113, { &P15364,&P7133,&P22452,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1*/
const Device UA2488285 = {S53906, 92, { &P11929,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5230) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2505732 = {S53907, 31, { &P11929,&P5129,&P24675,&P18092 }};	/*Useragent - Nokia5230/UCWEB8.8.0.212/50/800*/
const Device UA2700715 = {S53908, 94, { &P11929,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5230-c) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2749754 = {S53909, 95, { &P11929,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5230-1c) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2851053 = {S53910, 93, { &P11929,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia 5230) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA608441 = {S53911, 122, { &P15364,&P5690,&P18208,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 5.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.2.2.6613 Safari/532.4*/
const Device UA612009 = {S53912, 98, { &P15364,&P5690,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);en;kml:2.2;client:Free;type:default)*/
const Device UA833531 = {S53913, 98, { &P15364,&P5690,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.1.6014(Windows;Microsoft Windows (5.1.2600.3);en;kml:2.2;client:Free;type:default)*/
const Device UA856980 = {S53914, 103, { &P15364,&P5690,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);zh-Hans;kml:2.2;client:Free;type:default)*/
const Device UA887577 = {S53915, 98, { &P15364,&P5690,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.2);en;kml:2.2;client:Free;type:default)*/
const Device UA1956516 = {S53916, 98, { &P15364,&P5690,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);fr;kml:2.2;client:Free;type:default)*/
const Device UA1964182 = {S53917, 98, { &P15364,&P5690,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);de;kml:2.2;client:Free;type:default)*/
const Device UA1992670 = {S53918, 98, { &P15364,&P5690,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);ru;kml:2.2;client:Free;type:default)*/
const Device UA2161831 = {S53919, 98, { &P15364,&P5690,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);ko;kml:2.2;client:Free;type:default)*/
const Device UA2163146 = {S53920, 98, { &P15364,&P5690,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);es;kml:2.2;client:Free;type:default)*/
const Device UA2221710 = {S53921, 98, { &P15364,&P5690,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);it;kml:2.2;client:Free;type:default)*/
const Device UA2334191 = {S53922, 102, { &P15364,&P5690,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);es-419;kml:2.2;client:Free;type:default)*/
const Device UA2339418 = {S53923, 98, { &P15364,&P5690,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);nl;kml:2.2;client:Free;type:default)*/
const Device UA2412237 = {S53924, 98, { &P15364,&P5690,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);pt;kml:2.2;client:Free;type:default)*/
const Device UA2641991 = {S53925, 97, { &P15364,&P5690,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);en;kml:2.2;client:Pro;type:default)*/
const Device UA2685887 = {S53926, 98, { &P15364,&P5690,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (5.1.2600.3);cs;kml:2.2;client:Free;type:default)*/
const Device UA2386730 = {S53927, 131, { &P23515,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TD-1010 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2595428 = {S53928, 60, { &P23515,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; TD-1010 Build/IMM76D)*/
const Device UA2741557 = {S53929, 138, { &P18682,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-M950 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA176864 = {S53930, 126, { &P18625,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (SmartHub; SMART-TV; U; Linux/SmartTV) AppleWebKit/531.2+ (KHTML, like Gecko) WebBrowser/1.0 SmartTV Safari/531.2+*/
const Device UA624515 = {S53931, 135, { &P19909,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; LG-P935 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1959007 = {S53932, 59, { &P19909,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; LG-P935 Build/GRJ90)*/
const Device UA2422823 = {S53933, 123, { &P14983,&P5055,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; NokiaC7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile*/
const Device UA2423647 = {S53934, 102, { &P14983,&P5055,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac7-00) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2484236 = {S53935, 42, { &P14983,&P5055,&P24673,&P18092 }};	/*Useragent - NokiaC7-00UCBrowser/8.7.1.234/51/355/UCWEB*/
const Device UA3066889 = {S53936, 139, { &P18449,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST27i Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080836 = {S53937, 139, { &P18449,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST27a Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067061 = {S53938, 138, { &P18606,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067289 = {S53939, 131, { &P18606,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3082991 = {S53940, 140, { &P17346,&P19826,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B329 Safari/8536.25*/
const Device UA3083419 = {S53941, 140, { &P17346,&P19826,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B146 Safari/8536.25*/
const Device UA3095885 = {S53942, 138, { &P17346,&P19826,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B141 Safari/8536.25*/
const Device UA3096178 = {S53943, 138, { &P17346,&P19826,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10B144 Safari/8536.25*/
const Device UA708512 = {S53944, 136, { &P22051,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; ARCHOS 70it2 Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2008897 = {S53945, 138, { &P16362,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SPH-D710 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261535 = {S53946, 138, { &P16362,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2712819 = {S53947, 142, { &P16362,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710VMUB Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2861513 = {S53948, 141, { &P16362,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710BST Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2985038 = {S53949, 138, { &P24478,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P659 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3007610 = {S53950, 60, { &P24478,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P659 Build/JZO54K)*/
const Device UA902151 = {S53951, 146, { &P13042,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Incredible S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA910599 = {S53952, 146, { &P13042,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Incredible S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2023827 = {S53953, 142, { &P13042,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Incredible S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2403454 = {S53954, 136, { &P23523,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; I9377 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2535410 = {S53955, 139, { &P23523,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; GT-i9377 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752576 = {S53956, 136, { &P23523,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; I9377 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2855389 = {S53957, 61, { &P23523,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-i9377 Build/JRO03C)*/
const Device UA3086811 = {S53958, 58, { &P23523,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; I9377 Build/JRO03C)*/
const Device UA2569971 = {S53959, 75, { &P15364,&P17285,&P21791,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:19.0) Gecko/X Firefox/19.0 SeaMonkey/2.16.2*/
const Device UA2513350 = {S53960, 75, { &P17595,&P17295,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8) Presto/2.12.388 Version/12.14*/
const Device UA3065478 = {S53961, 137, { &P18559,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LG730 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA6827 = {S53962, 77, { &P12380,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia1680c-2/2.0 (06.82) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia1680c-2*/
const Device UA6829 = {S53963, 64, { &P12380,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia1680c-2/2.0 (07.60) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA30976 = {S53964, 65, { &P12380,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia1680c-2b/2.0 (05.61) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA30979 = {S53965, 70, { &P12380,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia1680c-2b_CMCC/2.0 (06.82) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA171875 = {S53966, 134, { &P15364,&P7133,&P22492,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.3.4641 Chrome/7.0.517.450 Safari/534.7*/
const Device UA2515246 = {S53967, 144, { &P20729,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Micromax A27 Build/IML74K) AppleWebKit/534.30(KHTML,like Gecko) 			Version/4.0 Mobile Safari/534.30*/
const Device UA2853770 = {S53968, 74, { &P20729,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Micromax A27 Build/MocorDroid2.3.5)*/
const Device UA102264 = {S53969, 73, { &P17595,&P18120,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X; U; en) Presto/2.2.15 Version/10.00*/
const Device UA112395 = {S53970, 71, { &P17595,&P18120,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; PPC Mac OS X; U; en) Presto/2.2.15 Version/10.01*/
const Device UA2158392 = {S53971, 88, { &P17595,&P18120,&P4812,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X; U; en; rv:1.8.1) Gecko/X Firefox/2.0.0 Opera 10.00*/
const Device UA2209918 = {S53972, 59, { &P17595,&P18120,&P4812,&P18092 }};	/*Useragent - Opera/10.00 (Macintosh; Intel Mac OS X; U; en) Presto/2.2.0*/
const Device UA1182 = {S53973, 154, { &P15366,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-ie; HTC Tattoo 1.67.174.27 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA17077 = {S53974, 154, { &P15366,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; es-es; HTC Tattoo 1.67.164.18 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA17086 = {S53975, 154, { &P15366,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; HTC Tattoo 1.67.161.18 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA17092 = {S53976, 153, { &P15366,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; HTC Tattoo 1.67.161.5 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA17104 = {S53977, 154, { &P15366,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; tr-tr; HTC Tattoo 1.67.184.60 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA17109 = {S53978, 154, { &P15366,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; pt-pt; HTC Tattoo 1.67.173.37 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA35618 = {S53979, 153, { &P15366,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; es-es; HTC Tattoo 1.65.161.2 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA35619 = {S53980, 153, { &P15366,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; HTC Tattoo 1.64.161.1 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA62547 = {S53981, 148, { &P15366,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; HTC_TATTOO_A3288 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA72597 = {S53982, 149, { &P15366,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; fr-ch; HTC Tattoo-Orange Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA281398 = {S53983, 142, { &P15366,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; HTC Tattoo Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA3098421 = {S53984, 148, { &P15366,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; HTC_TATTOO_A3288 Build/DRC79) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA63049 = {S53985, 77, { &P12181,&P5826,&P4093,&P18092 }};	/*Useragent - BlackBerry9530/4.7.0.148 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105*/
const Device UA95827 = {S53986, 71, { &P17595,&P18120,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; PPC Mac OS X; U; en) Presto/2.2.15 Version/10.10*/
const Device UA92389 = {S53987, 50, { &P15364,&P5690,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.1)*/
const Device UA432799 = {S53988, 138, { &P15364,&P5690,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA462400 = {S53989, 52, { &P15364,&P5690,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows XP Prof.)*/
const Device UA633431 = {S53990, 150, { &P15364,&P5690,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.2; .NET CLR 1.1.4322; InfoPath.2; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022; FDM)*/
const Device UA857803 = {S53991, 169, { &P15364,&P5690,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)*/
const Device UA2208049 = {S53992, 55, { &P15364,&P5690,&P17387,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.5; Windows NT 5.1; SV1)*/
const Device UA2222497 = {S53993, 224, { &P15364,&P5690,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.2; .NET CLR 1.1.4322) (Compatible; ; ; Trident/4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.0.3705; .NET CLR 1.1.4322)*/
const Device UA2595684 = {S53994, 282, { &P15364,&P5690,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; MSIE 5.5; Windows NT 5.0; GTB7.4; EasyBits GO v1.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; OfficeLiveConnector.1.3; OfficeLivePatch.0.0; InfoPath.2; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; AskTbAVR-4/5.15.11.30498)*/
const Device UA2431004 = {S53995, 135, { &P20097,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; CT1010 Build/IMM76I.v006) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2436170 = {S53996, 135, { &P20097,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; CT1010 Build/IMM76I.v007) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2457689 = {S53997, 136, { &P20097,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; CT1010 Build/IMM76I.v007S) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2484520 = {S53998, 136, { &P20097,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-be; CT1010 Build/IMM76I.v007B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2704862 = {S53999, 136, { &P20097,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-be; CT1010 Build/IMM76I.v008B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2716051 = {S54000, 135, { &P20097,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; CT1010 Build/IMM76I.v008) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2557835 = {S54001, 105, { &P12193,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac5-00.2) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2597645 = {S54002, 103, { &P12193,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac5-00) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2716726 = {S54003, 94, { &P12193,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaC5-00) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2717873 = {S54004, 96, { &P12193,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaC5-00.2) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2674010 = {S54005, 117, { &P15364,&P5690,&P22466,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36*/
const Device UA2862208 = {S54006, 117, { &P15364,&P5690,&P22466,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36*/
const Device UA2936657 = {S54007, 124, { &P15364,&P5690,&P22466,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36*/
const Device UA2390871 = {S54008, 143, { &P20091,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMP5880D Build/PMP5880D_v1.0.0_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2566547 = {S54009, 139, { &P20947,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTCEVOV4G Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2675150 = {S54010, 82, { &P15364,&P16110,&P22225,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:20.0) Gecko/X Firefox/20.0 SeaMonkey/2.17.1*/
const Device UA2731018 = {S54011, 139, { &P22999,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A870S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2765523 = {S54012, 139, { &P22999,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A870L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2871708 = {S54013, 144, { &P22999,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A870K/1.27 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956986 = {S54014, 144, { &P22999,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A870K/1.20 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2988657 = {S54015, 61, { &P22999,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A870L Build/JZO54K)*/
const Device UA3005039 = {S54016, 144, { &P22999,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A870K/1.32 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049752 = {S54017, 142, { &P22999,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A870L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30*/
const Device UA3059872 = {S54018, 172, { &P22999,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A870K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 NAVER(inapp; search; 230; 4.7.7)*/
const Device UA3075992 = {S54019, 142, { &P22999,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A870S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30*/
const Device UA3102199 = {S54020, 61, { &P22999,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A870S Build/JZO54K)*/
const Device UA3120374 = {S54021, 61, { &P22999,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A870K Build/JZO54K)*/
const Device UA3124969 = {S54022, 147, { &P22999,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A870K/1.44 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30*/
const Device UA204249 = {S54023, 142, { &P18021,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; IM-A770K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA554371 = {S54024, 66, { &P18021,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; IM-A770K Build/GINGERBREAD)*/
const Device UA571247 = {S54025, 140, { &P18021,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; IM-A770K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3083949 = {S54026, 112, { &P18643,&P5043,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia311/05.92; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA2385110 = {S54027, 101, { &P14929,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.5.0.54; en-us; 9780/6.5.0.54) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2421112 = {S54028, 103, { &P14929,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.207; en-us; 9780/6.6.0.207) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2502634 = {S54029, 103, { &P14929,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.248; en-us; 9780/6.6.0.248) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2503072 = {S54030, 103, { &P14929,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.236; en-us; 9780/6.6.0.236) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2505140 = {S54031, 103, { &P14929,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.246; en-us; 9780/6.6.0.246) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2553879 = {S54032, 103, { &P14929,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.223; en-us; 9780/6.6.0.223) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2594452 = {S54033, 103, { &P14929,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.212; en-us; 9780/6.6.0.212) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2594764 = {S54034, 103, { &P14929,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.5.0.136; en-us; 9780/6.5.0.136) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2653870 = {S54035, 103, { &P14929,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.5.0.153; en-us; 9780/6.5.0.153) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2654867 = {S54036, 103, { &P14929,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.241; en-us; 9780/6.6.0.241) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2695057 = {S54037, 101, { &P14929,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.5.0.93; en-us; 9780/6.5.0.93) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2728125 = {S54038, 103, { &P14929,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.216; en-us; 9780/6.6.0.216) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2762715 = {S54039, 103, { &P14929,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.200; en-us; 9780/6.6.0.200) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2985164 = {S54040, 103, { &P14929,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.5.0.147; en-us; 9780/6.5.0.147) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3045000 = {S54041, 103, { &P14929,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.6.0.250; en-us; 9780/6.6.0.250) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2488067 = {S54042, 133, { &P18466,&P18110,&P19838,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Safari/535.19 Silk-Accelerated=false*/
const Device UA2755311 = {S54043, 138, { &P23771,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2665589 = {S54044, 140, { &P22641,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C1504 Build/11.3.A.0.47) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA900117 = {S54045, 125, { &P15364,&P16110,&P22374,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.3.9.1000 Chrome/18.0.966.0 Safari/535.12*/
const Device UA901423 = {S54046, 118, { &P15364,&P16110,&P22374,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.3.9.1000 Chrome/18.0.966.0 Safari/535.12*/
const Device UA2440400 = {S54047, 116, { &P15364,&P16110,&P22374,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.3 (KHTML, like Gecko) Maxthon/3.3.3.1000 Chrome/16.0.883.0 Safari/535.3*/
const Device UA2758198 = {S54048, 123, { &P15364,&P16110,&P22374,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.3 (KHTML, like Gecko) Maxthon/3.3.3.1000 Chrome/16.0.883.0 Safari/535.3*/
const Device UA2043685 = {S54049, 114, { &P15767,&P18432,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Twitter for iPad*/
const Device UA2260603 = {S54050, 116, { &P15767,&P18432,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Twitter for iPad*/
const Device UA2657562 = {S54051, 116, { &P15767,&P18432,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550 Twitter for iPad*/
const Device UA2696997 = {S54052, 117, { &P15767,&P18432,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8426 Twitter for iPad*/
const Device UA2841749 = {S54053, 114, { &P15767,&P18432,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A407 Twitter for iPad*/
const Device UA2945969 = {S54054, 87, { &P16519,&P5567,&P17371,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/5.1/27.2338; U; en) Presto/2.8.119 320X240 Pantech TXT8045*/
const Device UA2389838 = {S54055, 76, { &P15364,&P18110,&P20457,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:17.0) Gecko/X Firefox/17.0 SeaMonkey/2.14.1*/
const Device UA2459291 = {S54056, 63, { &P15364,&P18110,&P20457,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:17.0) Gecko/X SeaMonkey/2.14.1*/
const Device UA2568382 = {S54057, 142, { &P17346,&P18117,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9B206 Safari/7534.48.3*/
const Device UA2584268 = {S54058, 140, { &P17346,&P18117,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9B176 Safari/7534.48.3*/
const Device UA109908 = {S54059, 31, { &P12280,&P17779,&P17470,&P18092 }};	/*Useragent - Yahoo! (iPhone Inquisitor; 1.0)*/
const Device UA790156 = {S54060, 10, { &P12280,&P17779,&P17470,&P18092 }};	/*Useragent - iPhone Vox*/
const Device UA804631 = {S54061, 62, { &P12280,&P17779,&P17470,&P18092 }};	/*Useragent - WhatsEye (iPhone) AppleWebKit (KHTML, like Gecko) Mobile/8F192*/
const Device UA1964820 = {S54062, 31, { &P12280,&P17779,&P17470,&P18092 }};	/*Useragent - Apple iPhone v9B206 Maps v5.1.1*/
const Device UA2242228 = {S54063, 67, { &P12280,&P17779,&P17470,&P18092 }};	/*Useragent - com.apple.invitation-registration [iPhone OS,5.1.1,9B206,iPhone3,3]*/
const Device UA2268710 = {S54064, 31, { &P12280,&P17779,&P17470,&P18092 }};	/*Useragent - Apple iPhone v8C148 Maps v4.2.1*/
const Device UA2302898 = {S54065, 31, { &P12280,&P17779,&P17470,&P18092 }};	/*Useragent - Apple iPhone v9A405 Maps v5.0.1*/
const Device UA2308683 = {S54066, 44, { &P12280,&P17779,&P17470,&P18092 }};	/*Useragent - server-bag [iPhone OS,5.0.1,9A406,iPhone4,1]*/
const Device UA2319267 = {S54067, 40, { &P12280,&P17779,&P17470,&P18092 }};	/*Useragent - Apple iPhone v5.1.1 YouTube v1.0.0.9B206*/
const Device UA2337567 = {S54068, 37, { &P12280,&P17779,&P17470,&P18092 }};	/*Useragent - Apple iPhone v5.1.1 Stocks v3.0.9B206*/
const Device UA2364275 = {S54069, 44, { &P12280,&P17779,&P17470,&P18092 }};	/*Useragent - server-bag [iPhone OS,5.1.1,9B208,iPhone3,1]*/
const Device UA2392430 = {S54070, 40, { &P12280,&P17779,&P17470,&P18092 }};	/*Useragent - Apple iPhone v5.0.1 YouTube v1.0.0.9A405*/
const Device UA2410605 = {S54071, 29, { &P12280,&P17779,&P17470,&P18092 }};	/*Useragent - Apple iPhone v9B176 Maps v5.1*/
const Device UA2437204 = {S54072, 23, { &P12280,&P17779,&P17470,&P18092 }};	/*Useragent - Tribune App iPhone v1.0*/
const Device UA2441263 = {S54073, 29, { &P12280,&P17779,&P17470,&P18092 }};	/*Useragent - Apple iPhone v9B179 Maps v5.1*/
const Device UA2472951 = {S54074, 38, { &P12280,&P17779,&P17470,&P18092 }};	/*Useragent - Apple iPhone v5.0 YouTube v1.0.0.9A334*/
const Device UA2588298 = {S54075, 67, { &P12280,&P17779,&P17470,&P18092 }};	/*Useragent - com.apple.invitation-registration [iPhone OS,5.1.1,9B206,iPhone4,1]*/
const Device UA2596666 = {S54076, 16, { &P12280,&P17779,&P17470,&P18092 }};	/*Useragent - eBayiPhone/2.8.0*/
const Device UA2659349 = {S54077, 16, { &P12280,&P17779,&P17470,&P18092 }};	/*Useragent - eBayiPhone/2.7.2*/
const Device UA2973274 = {S54078, 137, { &P18243,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P880g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974034 = {S54079, 136, { &P18243,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P880 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA90269 = {S54080, 135, { &P17961,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SHW-M180S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90425 = {S54081, 135, { &P17961,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SHW-M180L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA309205 = {S54082, 137, { &P17961,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; SHW-M180K Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA312055 = {S54083, 137, { &P17961,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; SHW-M180W Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA314625 = {S54084, 137, { &P17961,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SHW-M180S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA315912 = {S54085, 137, { &P17961,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SHW-M180L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA510768 = {S54086, 59, { &P17961,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; SHW-M180S Build/FROYO)*/
const Device UA2018698 = {S54087, 59, { &P17961,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; SHW-M180L Build/FROYO)*/
const Device UA2018873 = {S54088, 61, { &P17961,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SHW-M180S Build/FROYO)*/
const Device UA2193490 = {S54089, 33, { &P22798,&P5208,&P3534,&P18092 }};	/*Useragent - HUAWEI-M636/001.00 WAP/OBIGO/Q05A*/
const Device UA2934537 = {S54090, 101, { &P22798,&P5208,&P3534,&P18092 }};	/*Useragent - HUAWEI-M636/001.00 Browser/Obigo-Browser/Q05A MMS/UNIBOX/V1.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2434234 = {S54091, 157, { &P14183,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I997 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2534826 = {S54092, 157, { &P14183,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I997 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2537069 = {S54093, 156, { &P14183,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I997 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2753093 = {S54094, 157, { &P14183,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I997 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2235505 = {S54095, 76, { &P17595,&P18578,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C2)*/
const Device UA2352652 = {S54096, 71, { &P17595,&P18578,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (iMac12%2C2)*/
const Device UA2382650 = {S54097, 76, { &P17595,&P18578,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookAir4%2C2)*/
const Device UA2417730 = {S54098, 70, { &P17595,&P18578,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/520.5.1 Darwin/11.4.2 (i386) (MacPro1%2C1)*/
const Device UA2419096 = {S54099, 73, { &P17595,&P18578,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBook5%2C1)*/
const Device UA1996047 = {S54100, 131, { &P17686,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ601 Build/I.7.1-45) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2411657 = {S54101, 131, { &P17686,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MZ601 Build/I.7.1-34) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA908829 = {S54102, 57, { &P15364,&P17017,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (masking-agent; rv:14.0) Gecko/X Firefox/14.0*/
const Device UA924950 = {S54103, 59, { &P15364,&P17017,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (masking-agent; rv:14.0) Gecko/X Firefox/14.0.1*/
const Device UA1982150 = {S54104, 52, { &P15364,&P17017,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (SaferSurf rv:14.0) Gecko/X Firefox/14.0*/
const Device UA1995264 = {S54105, 53, { &P15364,&P17017,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; rv:14.0) Gecko/X Firefox/14.0.1*/
const Device UA2391606 = {S54106, 182, { &P18385,&P18598,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; Nexus 7 Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.10/221,1.9.10.221,2012-11-19 13:37, us*/
const Device UA2429001 = {S54107, 191, { &P18385,&P18598,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Nexus 7 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0000, gb*/
const Device UA2676923 = {S54108, 190, { &P18385,&P18598,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, -0400, us*/
const Device UA3096131 = {S54109, 112, { &P15803,&P5843,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia200/11.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA2753870 = {S54110, 138, { &P19902,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA8842 = {S54111, 104, { &P13351,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-T479/T479UVJH1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA626531 = {S54112, 87, { &P15302,&P17485,&P3499,&P18092 }};	/*Useragent - SonyEricssonWT13i/R3AF010 TelecaBrowser/Q07C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2431929 = {S54113, 89, { &P18115,&P5764,&P17371,&P18092 }};	/*Useragent - Opera/9.80 (BlackBerry; Opera Mini/5.1.22303/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA694678 = {S54114, 152, { &P12955,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Blade Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2843109 = {S54115, 142, { &P12955,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Blade Build/GRJ22; Eco-CM7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3064869 = {S54116, 127, { &P17926,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA1957483 = {S54117, 138, { &P18687,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P708g Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1966438 = {S54118, 137, { &P18687,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P705 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1976528 = {S54119, 138, { &P18687,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P705g Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2160597 = {S54120, 138, { &P18687,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P705f Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2471651 = {S54121, 86, { &P17968,&P17979,&P18414,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/7.6.32764/28.3234; U; zh) Presto/2.8.119 Version/11.10*/
const Device UA1981384 = {S54122, 130, { &P18723,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AN7FG3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2824331 = {S54123, 59, { &P18723,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; AN7FG3 Build/IML74K)*/
const Device UA2367747 = {S54124, 130, { &P23039,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; PC1038 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2603507 = {S54125, 130, { &P23039,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; PC1038 Build/PC1038) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2655498 = {S54126, 75, { &P23039,&P18111,&P18112,&P18092 }};	/*Useragent - AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; PC1038 Build/JRO03H)*/
const Device UA2872126 = {S54127, 49, { &P15364,&P22878,&P17470,&P18092 }};	/*Useragent - python-requests/1.2.3 CPython/2.7.2 Darwin/12.4.0*/
const Device UA3074501 = {S54128, 49, { &P15364,&P22878,&P17470,&P18092 }};	/*Useragent - python-requests/1.2.3 CPython/2.7.5 Darwin/12.4.0*/
const Device UA214226 = {S54129, 139, { &P18023,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; INFOBAR A01 Build/S7142) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA225756 = {S54130, 139, { &P18023,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; INFOBAR A01 Build/S6160) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA245605 = {S54131, 139, { &P18023,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; INFOBAR A01 Build/S9081) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA397542 = {S54132, 139, { &P18023,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; INFOBAR A01 Build/SB240) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625033 = {S54133, 139, { &P18023,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; INFOBAR A01 Build/S3081) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626261 = {S54134, 139, { &P18023,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; INFOBAR A01 Build/SA271) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA754593 = {S54135, 139, { &P18023,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; INFOBAR A01 Build/S8270) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031037 = {S54136, 139, { &P18023,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; INFOBAR A01 Build/S8020) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2520814 = {S54137, 139, { &P18023,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; INFOBAR A01 Build/S7050) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2649616 = {S54138, 63, { &P18023,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; INFOBAR A01 Build/S8020)*/
const Device UA2754271 = {S54139, 126, { &P15869,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xoom Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2773345 = {S54140, 126, { &P15869,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Xoom Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA20334 = {S54141, 133, { &P15072,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; E310 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1889165 = {S54142, 57, { &P15072,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; E310 Build/FRG83G)*/
const Device UA2235420 = {S54143, 75, { &P17595,&P18207,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2) Presto/2.12.388 Version/12.10*/
const Device UA2429974 = {S54144, 193, { &P15364,&P7133,&P24673,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)UCBrowser/8.7.1.234/50/444/UCWEB*/
const Device UA2676191 = {S54145, 140, { &P18223,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677038 = {S54146, 137, { &P18223,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xperia P Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2706790 = {S54147, 133, { &P18223,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2586574 = {S54148, 137, { &P20119,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; LIFETAB_S9714 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2766110 = {S54149, 66, { &P20119,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; LIFETAB_S9714 Build/JRO03R)*/
const Device UA24194 = {S54150, 73, { &P11724,&P5033,&P3417,&P18092 }};	/*Useragent - BlackBerry8310/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102*/
const Device UA36508 = {S54151, 135, { &P11724,&P5033,&P3417,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320) BlackBerry8310/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/107*/
const Device UA45073 = {S54152, 135, { &P11724,&P5033,&P3417,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320) BlackBerry8310/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102*/
const Device UA75288 = {S54153, 135, { &P11724,&P5033,&P3417,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320) BlackBerry8310/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/154*/
const Device UA2505393 = {S54154, 138, { &P21350,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8730 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2796801 = {S54155, 139, { &P21350,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8730T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2673705 = {S54156, 90, { &P15364,&P16110,&P22370,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.0.1-x64*/
const Device UA2680230 = {S54157, 81, { &P15364,&P16110,&P22370,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.0.1*/
const Device UA3067604 = {S54158, 138, { &P18280,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067912 = {S54159, 138, { &P18280,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC VLE_U Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080594 = {S54160, 138, { &P18280,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094535 = {S54161, 138, { &P18280,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC VLE_U Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2404641 = {S54162, 127, { &P20827,&P5629,&P15760,&P18092 }};	/*Useragent - MT6515M-A1+/1.0 Linux/2.6.35.7 Android/2.3.6 Release/10.30.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2468014 = {S54163, 126, { &P20827,&P5629,&P15760,&P18092 }};	/*Useragent - MT6515M-P5/1.0 Linux/2.6.35.7 Android 2.3.6 Release/11.29.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2507382 = {S54164, 127, { &P20827,&P5629,&P15760,&P18092 }};	/*Useragent - MT6515M-A1+/1.0 Linux/2.6.35.7 Android/2.3.6 Release/10.29.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2507521 = {S54165, 127, { &P20827,&P5629,&P15760,&P18092 }};	/*Useragent - MT6515M-A1+/1.0 Linux/2.6.35.7 Android 2.3.6 Release/12.09.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2654676 = {S54166, 127, { &P20827,&P5629,&P15760,&P18092 }};	/*Useragent - MT6515M-A1+/1.0 Linux/2.6.35.7 Android/2.3.6 Release/gf.if.hfgh Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2696871 = {S54167, 127, { &P20827,&P5629,&P15760,&P18092 }};	/*Useragent - MT6515M-GS2/1.0 Linux/2.6.35.7 Android 2.3.6 Release/11.13.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2696911 = {S54168, 139, { &P20827,&P5629,&P15760,&P18092 }};	/*Useragent - MT6515M-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android 2.3.6 Release/12.03.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2728994 = {S54169, 129, { &P20827,&P5629,&P15760,&P18092 }};	/*Useragent - MT6515M-PH350/1.0 Linux/2.6.35.7 Android 2.3.6 Release/11.22.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2763084 = {S54170, 127, { &P20827,&P5629,&P15760,&P18092 }};	/*Useragent - MT6515M-A1+/1.0 Linux/2.6.35.7 Android 2.3.6 Release/11.01.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2764526 = {S54171, 126, { &P20827,&P5629,&P15760,&P18092 }};	/*Useragent - MT6515M-P5/1.0 Linux/2.6.35.7 Android 2.3.6 Release/12.08.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2764955 = {S54172, 127, { &P20827,&P5629,&P15760,&P18092 }};	/*Useragent - MT6515M-A1+/1.0 Linux/2.6.35.7 Android 2.3.6 Release/gh.fo.hfgh Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2937699 = {S54173, 213, { &P20827,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.9; zh-cn; MT6515M-HAPPLE/1.0 Android/2.3.9 Release/04.18.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2984239 = {S54174, 141, { &P20827,&P5629,&P15760,&P18092 }};	/*Useragent - MT6515-I-T-3002/1.0 Linux/Apps_4.0.32.9-perf Android/2.3.5 Release/11.07.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2985968 = {S54175, 130, { &P20827,&P5629,&P15760,&P18092 }};	/*Useragent - MT6515M-HTH668/1.0 Linux/2.6.35.7 Android/2.3.6 Release/09.22.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3043707 = {S54176, 139, { &P20827,&P5629,&P15760,&P18092 }};	/*Useragent - MT6515M-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/2.3.6 Release/10.27.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2566900 = {S54177, 142, { &P18462,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire X Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2866753 = {S54178, 135, { &P18462,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire X Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA3083632 = {S54179, 140, { &P21602,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5302 Build/12.0.A.2.254) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA22710 = {S54180, 71, { &P12059,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2690*/
const Device UA22715 = {S54181, 61, { &P12059,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA50385 = {S54182, 128, { &P13248,&P17484,&P4732,&P18092 }};	/*Useragent - SAMSUNG-SGH-A797/A797UCJC1; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.17793; U; en-US) Opera 9.50*/
const Device UA2974872 = {S54183, 139, { &P13054,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR6425LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA237668 = {S54184, 133, { &P15364,&P17818,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Ubuntu/11.04 Chromium/15.0.871.0 Chrome/15.0.871.0 Safari/535.2*/
const Device UA252420 = {S54185, 135, { &P15364,&P17818,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.2 (KHTML, like Gecko) Ubuntu/11.04 Chromium/15.0.874.106 Chrome/15.0.874.106 Safari/535.2*/
const Device UA609976 = {S54186, 6, { &P18254,&P18588,&P18245,&P18091 }};	/*Useragent - coccoc*/
const Device UA646195 = {S54187, 128, { &P18478,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; SC-02D Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA724077 = {S54188, 130, { &P18478,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-P6201 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2030699 = {S54189, 57, { &P18478,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; SC-02D Build/HTJ85B)*/
const Device UA2495929 = {S54190, 59, { &P18478,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P6201 Build/HTJ85B)*/
const Device UA2610961 = {S54191, 128, { &P18478,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6201 Build/HTJ85B) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2620202 = {S54192, 137, { &P18478,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; id-id; GT-P6201 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2622527 = {S54193, 265, { &P18478,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6201 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [FBAN/FB4A;FBAV/1.9.12;FBDM/{density=1.0,width=600,height=976};FBLC/en_US;FBCR/Smart;FBPN/com.facebook.katana;FBDV/GT-P6201;FBSV/3.2;]*/
const Device UA2630235 = {S54194, 128, { &P18478,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P6201 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2635251 = {S54195, 90, { &P18478,&P6437,&P16179,&P18092 }};	/*Useragent - com.google.android.youtube/4.3.9(Linux; U; Android 3.2; en_US; GT-P6201 Build/HTJ85B) gzip*/
const Device UA2235374 = {S54196, 112, { &P18583,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia110/03.04; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2457899 = {S54197, 111, { &P18583,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia110/03.26; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5*/
const Device UA427331 = {S54198, 71, { &P12280,&P18117,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.9B176 (iPhone; U; CPU OS 5_1 like Mac OS X; en_us)*/
const Device UA615686 = {S54199, 73, { &P12280,&P18117,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.9B206 (iPhone; U; CPU OS 5_1_1 like Mac OS X; en_us)*/
const Device UA2469314 = {S54200, 101, { &P19975,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 505)*/
const Device UA2435 = {S54201, 128, { &P18088,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) PPC; 240x320; CHT 9100/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA3948 = {S54202, 81, { &P18088,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12; en-US; KIN.Two 1.0)*/
const Device UA3950 = {S54203, 81, { &P18088,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12; es-US; KIN.Two 1.0)*/
const Device UA3949 = {S54204, 81, { &P18088,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12; en-US; KIN.One 1.0)*/
const Device UA7320 = {S54205, 81, { &P18088,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12; es-US; KIN.One 1.0)*/
const Device UA54003 = {S54206, 111, { &P18088,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Smartphone; 240x320; SPV E650; OpVer 22.122.1.613*/
const Device UA73156 = {S54207, 111, { &P18088,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Smartphone; 240x320; SPV E650; OpVer 22.114.1.612*/
const Device UA86326 = {S54208, 76, { &P18088,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Sprint:PPC6800*/
const Device UA87477 = {S54209, 67, { &P18088,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) S01SH*/
const Device UA87912 = {S54210, 69, { &P18088,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) WS011SH*/
const Device UA88612 = {S54211, 103, { &P18088,&P5448,&P17173,&P18092 }};	/*Useragent - Lenovo_ET600/Lenovo WindowsMobile/6 Release/4.22.2007 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA88633 = {S54212, 78, { &P18088,&P5448,&P17173,&P18092 }};	/*Useragent - Techfaith Moffett/2.0 WAP2.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA161513 = {S54213, 61, { &P18088,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA64388 = {S54214, 77, { &P11984,&P5035,&P3418,&P18092 }};	/*Useragent - BlackBerry8220/4.6.0.180 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100*/
const Device UA575228 = {S54215, 71, { &P15364,&P18118,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:10.0a1) Gecko/X Firefox/10.0a1*/
const Device UA590270 = {S54216, 88, { &P15364,&P18118,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:10.0.2) Gecko/X Firefox/10.0.2 Firefox/10.0.2*/
const Device UA1950786 = {S54217, 68, { &P15364,&P18118,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:10.0) Gecko/X Abrowser/10.0*/
const Device UA2384289 = {S54218, 71, { &P15364,&P18118,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:10.0.5) Gecko/X Firefox/10.0.5*/
const Device UA2406376 = {S54219, 96, { &P15364,&P16110,&P22367,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:15.3) Gecko/X Firefox/15.3.2-x64 PaleMoon/15.3.2-x64*/
const Device UA2406649 = {S54220, 83, { &P15364,&P16110,&P22367,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:15.3) Gecko/X Firefox/15.3.2 PaleMoon/15.3.2*/
const Device UA2975085 = {S54221, 127, { &P17926,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA253775 = {S54222, 8, { &P15364,&P17017,&P17232,&P18092 }};	/*Useragent - Chrome/1*/
const Device UA263971 = {S54223, 31, { &P15364,&P17017,&P17232,&P18092 }};	/*Useragent - Chrome/1.0.154.53 Safari/525.19*/
const Device UA547990 = {S54224, 11, { &P15364,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M5100*/
const Device UA627240 = {S54225, 10, { &P15364,&P22835,&P17470,&P18092 }};	/*Useragent - Spice QT53*/
const Device UA632129 = {S54226, 12, { &P15364,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M-5180*/
const Device UA687721 = {S54227, 11, { &P15364,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M5460*/
const Device UA1940506 = {S54228, 11, { &P15364,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M6450*/
const Device UA1952906 = {S54229, 11, { &P15364,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M5456*/
const Device UA2151854 = {S54230, 12, { &P15364,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M-5455*/
const Device UA2161677 = {S54231, 11, { &P15364,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M5225*/
const Device UA2166399 = {S54232, 11, { &P15364,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M5665*/
const Device UA2366001 = {S54233, 30, { &P15364,&P22835,&P17470,&P18092 }};	/*Useragent - Spice_M-5400/WapBrowserProfile*/
const Device UA2396424 = {S54234, 11, { &P15364,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M5670*/
const Device UA2416918 = {S54235, 11, { &P15364,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M5800*/
const Device UA2496411 = {S54236, 10, { &P15364,&P22835,&P17470,&P18092 }};	/*Useragent - Spice QT59*/
const Device UA3058027 = {S54237, 113, { &P15364,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M5915 /Spreadtrum/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0Config/CLD-1.1*/
const Device UA3094133 = {S54238, 12, { &P15364,&P22835,&P17470,&P18092 }};	/*Useragent - Spice Mi-354*/
const Device UA103515 = {S54239, 34, { &P15364,&P7133,&P17215,&P18092 }};	/*Useragent - Opera/9.51 (Windows NT 6.1; U; en)*/
const Device UA128596 = {S54240, 34, { &P15364,&P7133,&P17215,&P18092 }};	/*Useragent - Opera/9.52 (Windows NT 6.1; U; en)*/
const Device UA132616 = {S54241, 34, { &P15364,&P7133,&P17215,&P18092 }};	/*Useragent - Opera/9.50 (Windows NT 6.1; U; en)*/
const Device UA626887 = {S54242, 146, { &P20345,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; San Francisco II Build/OUK_B03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA737694 = {S54243, 146, { &P20345,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; San Francisco II Build/OUK_B04) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1941195 = {S54244, 70, { &P20345,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; San Francisco II Build/OUK_B03)*/
const Device UA2026238 = {S54245, 146, { &P20345,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; sk-sk; San Francisco II Build/OSK_B04) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2416067 = {S54246, 146, { &P20345,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ro-ro; San Francisco II Build/OMD_B01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3080716 = {S54247, 143, { &P18442,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SAMSUNG-SGH-I317 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080744 = {S54248, 135, { &P18442,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-N7100 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080912 = {S54249, 135, { &P18442,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SPH-L900 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081192 = {S54250, 135, { &P18442,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SGH-T889 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081229 = {S54251, 135, { &P18442,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-N7105 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083218 = {S54252, 141, { &P18442,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Galaxy Note II Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110734 = {S54253, 135, { &P18442,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SGH-I317 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2194426 = {S54254, 153, { &P12709,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SonyEricssonMK16a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2379294 = {S54255, 141, { &P12709,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MK16a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2721856 = {S54256, 141, { &P12709,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MK16a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2828862 = {S54257, 63, { &P12709,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MK16a Build/4.1.B.0.587)*/
const Device UA2755542 = {S54258, 130, { &P22606,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2769694 = {S54259, 129, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAD1001 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2782384 = {S54260, 133, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 7102 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782637 = {S54261, 133, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782782 = {S54262, 136, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AMI-TT2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783376 = {S54263, 138, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Explosion Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798844 = {S54264, 131, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; CTA Build/ICS.casper.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2799034 = {S54265, 134, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GLORY Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799324 = {S54266, 141, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BASE_Lutea_3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799392 = {S54267, 136, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A11Note Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799416 = {S54268, 133, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; I110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867775 = {S54269, 128, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S7916E Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2867809 = {S54270, 131, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868221 = {S54271, 134, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Spark Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868303 = {S54272, 142, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tmn smart a18 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869084 = {S54273, 139, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MOBHIG0080 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2876522 = {S54274, 133, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N900 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877415 = {S54275, 131, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; P6 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877578 = {S54276, 124, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2879311 = {S54277, 135, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CJ ThL Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879649 = {S54278, 134, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BT210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892363 = {S54279, 137, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Alpha GT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892471 = {S54280, 137, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Infinity Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892526 = {S54281, 128, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; 1040RS Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2892562 = {S54282, 134, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JY139 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892590 = {S54283, 136, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A13-DLR Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892804 = {S54284, 129, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MD-0697 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2892906 = {S54285, 138, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB9008GB Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892959 = {S54286, 134, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; 740RS Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892989 = {S54287, 128, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MIRA-2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2893001 = {S54288, 133, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; S5-N Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893123 = {S54289, 138, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; StarPAD 7 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893156 = {S54290, 138, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID_Robin Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893375 = {S54291, 144, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMURAI10(QuadCore) Build/SAMURAI10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2893559 = {S54292, 138, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SMP53-210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937435 = {S54293, 130, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; POM727MC Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2937642 = {S54294, 146, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Avvio_790 Build/AvvioAvvio_790) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937695 = {S54295, 127, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M1005 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2937725 = {S54296, 133, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; STXS Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2938010 = {S54297, 132, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; P4i Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2938155 = {S54298, 134, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HD II Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2938241 = {S54299, 138, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SMP45-210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2938299 = {S54300, 132, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A13 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2957433 = {S54301, 131, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 5I Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2957852 = {S54302, 146, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GO Live Dual Core Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2958318 = {S54303, 127, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Clear Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2958631 = {S54304, 128, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M1005D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2997487 = {S54305, 142, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; hexing77_ics2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2997760 = {S54306, 133, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; K-BOOK PLUS Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2998536 = {S54307, 124, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; G1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2998588 = {S54308, 127, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CT710 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2999277 = {S54309, 143, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Freelander I20 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2999452 = {S54310, 138, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Netbook PC 12700 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2999894 = {S54311, 127, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M1001 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3000418 = {S54312, 134, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V8502 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3004774 = {S54313, 146, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; magcomm77_cu_ics2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3049623 = {S54314, 133, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; X714 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3049831 = {S54315, 128, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VL-109 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3050264 = {S54316, 128, { &P15364,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BC1077 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3066872 = {S54317, 138, { &P20076,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26w Build/6.2.B.1.96) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA609678 = {S54318, 133, { &P17482,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  ARCHOS 80G9 Build/MR1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA902488 = {S54319, 132, { &P17482,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 80G9 Build/MR1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2231131 = {S54320, 135, { &P17482,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80G9 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2033044 = {S54321, 152, { &P12848,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB526 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257888 = {S54322, 152, { &P12848,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MB526 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566808 = {S54323, 139, { &P18296,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One V Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2600333 = {S54324, 139, { &P18296,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One V Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2753776 = {S54325, 138, { &P18450,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST23i Build/11.0.A.5.5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774081 = {S54326, 139, { &P18450,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST23a Build/11.0.A.2.16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA20252 = {S54327, 54, { &P15089,&P5214,&P3487,&P18092 }};	/*Useragent - KWC-S2300/ABR1003 UP.Browser/7.2.7.2.551 (GUI) MMP/2.0*/
const Device UA20427 = {S54328, 56, { &P15089,&P5214,&P3487,&P18092 }};	/*Useragent - KWC-S2300-C/ACF1002 UP.Browser/7.2.7.2.549 (GUI) MMP/2.0*/
const Device UA31570 = {S54329, 54, { &P15089,&P5214,&P3487,&P18092 }};	/*Useragent - KWC-S2300/ABQ1000 UP.Browser/7.2.7.2.549 (GUI) MMP/2.0*/
const Device UA31571 = {S54330, 54, { &P15089,&P5214,&P3487,&P18092 }};	/*Useragent - KWC-S2300/ABQ1103 UP.Browser/7.2.7.2.549 (GUI) MMP/2.0*/
const Device UA45019 = {S54331, 54, { &P15089,&P5214,&P3487,&P18092 }};	/*Useragent - KWC-S2300/ABU1001 UP.Browser/7.2.7.2.552 (GUI) MMP/2.0*/
const Device UA54559 = {S54332, 54, { &P15089,&P5214,&P3487,&P18092 }};	/*Useragent - KWC-S2300/ABR1001 UP.Browser/7.2.7.2.551 (GUI) MMP/2.0*/
const Device UA750724 = {S54333, 54, { &P15089,&P5214,&P3487,&P18092 }};	/*Useragent - KWC-S2300/ABR1004 UP.Browser/7.2.7.2.551 (GUI) MMP/2.0*/
const Device UA924942 = {S54334, 92, { &P15364,&P7133,&P22361,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:12.3) Gecko/X Firefox/12.3-x64 PaleMoon/12.3-x64*/
const Device UA1999886 = {S54335, 83, { &P15364,&P7133,&P22361,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.3) Gecko/X Firefox/12.3r2 PaleMoon/12.3r2*/
const Device UA2566627 = {S54336, 141, { &P12841,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC PH39100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2974358 = {S54337, 136, { &P23521,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA369219 = {S54338, 141, { &P15854,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; XOOM 2 ME Build/1.6.0M_278-MZ607) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA402929 = {S54339, 136, { &P15854,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; XOOM 2 ME Build/1.6.0MW-211.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA402941 = {S54340, 140, { &P15854,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; XOOM 2 ME Build/1.6.0_218.3-MZ607) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA403064 = {S54341, 142, { &P15854,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; XOOM 2 ME Build/1.6.0_218.3-MZ607) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA403075 = {S54342, 142, { &P15854,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 ME Build/1.6.0_268.3-MZ607) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA403083 = {S54343, 71, { &P15854,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; XOOM 2 ME Build/1.6.0_218.3-MZ607)*/
const Device UA403122 = {S54344, 138, { &P15854,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; XOOM 2 ME Build/1.6.0MW-211.7) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA403141 = {S54345, 149, { &P15854,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; XOOM 2 ME Build/1.6.0_218.3-MZ607) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA403143 = {S54346, 142, { &P15854,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 ME Build/1.6.0_268.4-MZ607) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA403168 = {S54347, 142, { &P15854,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 ME Build/1.6.0_249.1-MZ607) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA403195 = {S54348, 143, { &P15854,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; pt-br; XOOM 2 ME Build/1.6.0_218.11-MZ607) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA403229 = {S54349, 143, { &P15854,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; XOOM 2 ME Build/1.6.0M_278-MZ607) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA403319 = {S54350, 141, { &P15854,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; XOOM 2 ME Build/1.6.0M_268-MZ607) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA874709 = {S54351, 144, { &P15854,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 ME Build/1.6.0_268.4-MZ607) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2025486 = {S54352, 141, { &P15854,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 ME Build/1.6.0M_280_MZ607) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2513535 = {S54353, 158, { &P18283,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2514085 = {S54354, 158, { &P18283,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2604689 = {S54355, 158, { &P18283,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2672239 = {S54356, 158, { &P18283,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I535 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2738830 = {S54357, 158, { &P18283,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T999 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA3101461 = {S54358, 159, { &P18283,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-I9300T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA3102254 = {S54359, 162, { &P18283,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA2566543 = {S54360, 146, { &P19891,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2732975 = {S54361, 139, { &P19891,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2234360 = {S54362, 97, { &P17595,&P18207,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:16.0) Gecko/X Firefox/16.0 AlexaToolbar/alxf-2.17*/
const Device UA2192285 = {S54363, 93, { &P15364,&P18110,&P20911,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) rekonq/1.1 Safari/534.34*/
const Device UA33259 = {S54364, 123, { &P14010,&P17484,&P3834,&P18092 }};	/*Useragent - SAMSUNG-GT-S5263/S5263DDKE1 SHP/VPP/R5 Dolfin/2.0 NexPlayer/3.0 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B*/
const Device UA2405763 = {S54365, 120, { &P14748,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE72-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2427242 = {S54366, 32, { &P14748,&P5890,&P24673,&P18092 }};	/*Useragent - NokiaE72-1/UCWEB8.7.0.187/28/999*/
const Device UA2458213 = {S54367, 102, { &P14748,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiae72-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2458384 = {S54368, 42, { &P14748,&P5890,&P24673,&P18092 }};	/*Useragent - NOKIAE72-1UCBrowser/8.7.1.234/28/444/UCWEB*/
const Device UA196548 = {S54369, 133, { &P15364,&P17817,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.35 (KHTML, like Gecko) Ubuntu/10.10 Chromium/13.0.764.0 Chrome/13.0.764.0 Safari/534.35*/
const Device UA2293090 = {S54370, 135, { &P15364,&P17817,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/10.10 Chromium/13.0.782.99 Chrome/13.0.782.99 Safari/535.1*/
const Device UA198043 = {S54371, 145, { &P17976,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; MID_Serials Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA213463 = {S54372, 143, { &P17976,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; MID_Serials Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1940728 = {S54373, 67, { &P17976,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3; MID_Serials Build/GINGERBREAD)*/
const Device UA2018959 = {S54374, 69, { &P17976,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.1; MID_Serials Build/GINGERBREAD)*/
const Device UA2027158 = {S54375, 134, { &P17976,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Parrot Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027395 = {S54376, 145, { &P17976,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; MID_Serials Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2221213 = {S54377, 165, { &P15364,&P16110,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Avant Browser)*/
const Device UA2360140 = {S54378, 158, { &P15364,&P16110,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Avant Browser)*/
const Device UA2425870 = {S54379, 186, { &P15364,&P16110,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Media Center PC 6.0; Avant Browser)*/
const Device UA3041847 = {S54380, 179, { &P15364,&P16110,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Avant Browser)*/
const Device UA3118266 = {S54381, 180, { &P15364,&P16110,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Avant Browser)*/
const Device UA2974162 = {S54382, 149, { &P18317,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974448 = {S54383, 150, { &P18317,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300TG Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021161 = {S54384, 150, { &P18317,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300TL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3024025 = {S54385, 144, { &P18317,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Transformer Pad TF300T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA809644 = {S54386, 141, { &P15972,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  HTC One XL Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA901271 = {S54387, 140, { &P15972,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One XL Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1995857 = {S54388, 140, { &P15972,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One XL Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA910159 = {S54389, 67, { &P17968,&P17979,&P21457,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:14.0) Gecko/14.0 K9Browser/14.0.b5*/
const Device UA2031619 = {S54390, 66, { &P17968,&P17979,&P21457,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:14.0) Gecko/14.0 K9Browser/14.0.1*/
const Device UA1996199 = {S54391, 146, { &P18372,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySO-04D Build/7.0.D.1.117) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2010458 = {S54392, 142, { &P18372,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-04D Build/7.0.D.1.117) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2413764 = {S54393, 64, { &P18372,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SO-04D Build/7.0.D.1.117)*/
const Device UA2437379 = {S54394, 146, { &P18372,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySO-04D Build/7.0.D.1.130) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2437409 = {S54395, 142, { &P18372,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-04D Build/7.0.D.1.130) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2602762 = {S54396, 146, { &P18372,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySO-04D Build/7.0.D.1.137) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2673595 = {S54397, 142, { &P18372,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-04D Build/7.0.D.1.137) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA15714 = {S54398, 61, { &P11868,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia6280/2.0 (03.60) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2973666 = {S54399, 136, { &P23776,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGP321 Build/10.1.1.A.1.307) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2508871 = {S54400, 156, { &P14669,&P5843,&P24676,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-00) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2257715 = {S54401, 145, { &P20816,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; WIKO-CINK SLIM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2657860 = {S54402, 67, { &P20816,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; WIKO-CINK SLIM Build/IMM76D)*/
const Device UA3067589 = {S54403, 140, { &P17625,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST25i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081175 = {S54404, 140, { &P17625,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST25a Build/6.1.1.B.1.10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2974799 = {S54405, 145, { &P13064,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA122457 = {S54406, 117, { &P17595,&P17297,&P17335,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7; en-us) AppleWebKit/533.4 (KHTML, like Gecko) Version/4.1 Safari/533.4*/
const Device UA2430894 = {S54407, 78, { &P15364,&P18140,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; Edition Linux Mint) Presto/2.12.388 Version/12.12*/
const Device UA2662936 = {S54408, 138, { &P16263,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; dtab01 Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2799414 = {S54409, 148, { &P16263,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MediaPad 10 LINK Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2969234 = {S54410, 148, { &P16263,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fi-fi; MediaPad 7 Vogue Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3018056 = {S54411, 148, { &P16263,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MediaPad 7 Youth Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3024716 = {S54412, 145, { &P16263,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Telpad Dual S Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3101414 = {S54413, 67, { &P16263,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; dtab01 Build/HuaweiMediaPad)*/
const Device UA3102973 = {S54414, 77, { &P16263,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; MediaPad 10 LINK Build/HuaweiMediaPad)*/
const Device UA2737627 = {S54415, 156, { &P18283,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; SGH-T999 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2769390 = {S54416, 156, { &P18283,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9300 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2796541 = {S54417, 156, { &P18283,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SCH-I535 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2867971 = {S54418, 156, { &P18283,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; GT-I9300 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2869561 = {S54419, 156, { &P18283,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SGH-T999 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA64142 = {S54420, 77, { &P11728,&P5026,&P3410,&P18092 }};	/*Useragent - BlackBerry8820/4.5.0.110 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102*/
const Device UA2566892 = {S54421, 131, { &P15876,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P5100 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583167 = {S54422, 131, { &P15876,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P5100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2670035 = {S54423, 131, { &P15876,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T779 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA645424 = {S54424, 83, { &P15364,&P7127,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; es-ES; rv:1.8.0.3) Gecko/X Firefox/1.5.0.3*/
const Device UA615798 = {S54425, 125, { &P15364,&P5690,&P24199,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.13.13719.201*/
const Device UA625726 = {S54426, 88, { &P15364,&P5690,&P24199,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0) QQBrowser/6.13.13719.201*/
const Device UA634050 = {S54427, 80, { &P15364,&P5690,&P24199,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) QQBrowser/6.13.13719.201*/
const Device UA49721 = {S54428, 138, { &P13077,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HW-HUAWEI_C8650/C8650V100R001C92B825; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA655447 = {S54429, 127, { &P13077,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; C8650 Build/HuaweiC8650) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA753579 = {S54430, 151, { &P13077,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8650+ Build/V100R001C92B861) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA757106 = {S54431, 139, { &P13077,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8650+ Build/V100R001C92B861) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1943062 = {S54432, 139, { &P13077,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-id; C8650 Build/HuaweiC8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1953106 = {S54433, 132, { &P13077,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; C8650 Build/HuaweiC8650) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2019846 = {S54434, 63, { &P13077,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; C8650 Build/HuaweiC8650)*/
const Device UA2021900 = {S54435, 121, { &P13077,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; c8650 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2023449 = {S54436, 133, { &P13077,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; C8650 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024071 = {S54437, 139, { &P13077,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; C8650 Build/HuaweiC8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025964 = {S54438, 144, { &P13077,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8650+ Build/V100R001C92B861) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2027152 = {S54439, 132, { &P13077,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.3; zh-cn; C8650 Build HuaweiC8650) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2440535 = {S54440, 167, { &P13077,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en_id; C8650 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.2.0 Mobile (Dragonfruit)*/
const Device UA2547281 = {S54441, 139, { &P13077,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; C8650 Build/HuaweiC8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2642858 = {S54442, 75, { &P13077,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; HUAWEI C8650+ Build/V100R001C92B861)*/
const Device UA2810889 = {S54443, 144, { &P13077,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8650+ Build V100R001C92B861) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2812593 = {S54444, 133, { &P13077,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; C8650/C8650+ Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2975071 = {S54445, 140, { &P12841,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC PH39100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA906244 = {S54446, 142, { &P18382,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire C Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2427652 = {S54447, 142, { &P18382,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire C Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3067070 = {S54448, 136, { &P21793,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9205 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA363325 = {S54449, 107, { &P14763,&P5776,&P24663,&P18092 }};	/*Useragent - NokiaN72/5.0819.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.9.1.120/27/351/UCWEB*/
const Device UA1934919 = {S54450, 96, { &P14763,&P5776,&P24663,&P18092 }};	/*Useragent - NokiaN72/ 5.0741.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UCWEB7.9.0.94/27/999*/
const Device UA2203824 = {S54451, 114, { &P14540,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-01/08.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2421566 = {S54452, 116, { &P14540,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-01.1/07.22; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2457619 = {S54453, 113, { &P14540,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-01/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5*/
const Device UA2362613 = {S54454, 131, { &P20715,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID9740 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2732587 = {S54455, 131, { &P20715,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID9740 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA350971 = {S54456, 139, { &P18659,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; ISW11K Build/134.3.0005) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA351377 = {S54457, 139, { &P18659,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; ISW11K Build/136.1.0000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA363054 = {S54458, 139, { &P18659,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; ISW11K Build/138.2.0001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627653 = {S54459, 139, { &P18659,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; ISW11K Build/143.2.0001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA632317 = {S54460, 139, { &P18659,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; ISW11K Build/142.2.0000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1993659 = {S54461, 139, { &P18659,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; ISW11K Build/144.2.0000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2206643 = {S54462, 139, { &P18659,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; ISW11K Build/145.0.0002) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652719 = {S54463, 63, { &P18659,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; ISW11K Build/145.0.0002)*/
const Device UA2436483 = {S54464, 152, { &P17346,&P17090,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8L1 Safari/6533.18.5*/
const Device UA2437362 = {S54465, 152, { &P17346,&P17090,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8J2 Safari/6533.18.5*/
const Device UA2468868 = {S54466, 152, { &P17346,&P17090,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8G4 Safari/6533.18.5*/
const Device UA2469123 = {S54467, 152, { &P17346,&P17090,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8F190 Safari/6533.18.5*/
const Device UA2469441 = {S54468, 152, { &P17346,&P17090,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8H7 Safari/6533.18.5*/
const Device UA2556743 = {S54469, 152, { &P17346,&P17090,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/8K2 Safari/6533.18.5*/
const Device UA353811 = {S54470, 84, { &P17968,&P17979,&P4860,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/6.24212/26.1305; U; en) Presto/2.8.119 Version/10.54*/
const Device UA890675 = {S54471, 84, { &P17968,&P17979,&P4860,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/6.25979/26.1670; U; es) Presto/2.8.119 Version/10.54*/
const Device UA2289850 = {S54472, 82, { &P17968,&P17979,&P4860,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/6.25174/24.902; U; de) Presto/2.5.25 Version/10.54*/
const Device UA2236722 = {S54473, 132, { &P18465,&P18110,&P18690,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.2 Safari/535.19 Silk-Accelerated=true*/
const Device UA2679629 = {S54474, 83, { &P18141,&P22275,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 2.11) AppleWebKit/536.26 (KHTML, like Gecko) Silk/3.2*/
const Device UA2567009 = {S54475, 138, { &P19856,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LS970 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583697 = {S54476, 131, { &P19856,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LS970 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2389093 = {S54477, 159, { &P14862,&P5204,&P24673,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia2700c-2) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2982975 = {S54478, 155, { &P14862,&P5204,&P24673,&P18092 }};	/*Useragent - Nokia2700c/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia2700c) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2944326 = {S54479, 145, { &P18582,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonyLT28h Build/6.2.B.0.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2964603 = {S54480, 141, { &P18582,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT28h Build/6.2.B.0.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2971340 = {S54481, 145, { &P18582,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyLT28i Build/6.2.B.0.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3006572 = {S54482, 63, { &P18582,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LT28h Build/6.2.B.0.211)*/
const Device UA3020746 = {S54483, 141, { &P18582,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LT28i Build/6.2.B.0.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058324 = {S54484, 63, { &P18582,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LT28i Build/6.2.B.0.211)*/
const Device UA2567224 = {S54485, 140, { &P13054,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR6425LVW Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2666839 = {S54486, 140, { &P13054,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6425LVW Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2783270 = {S54487, 136, { &P22727,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGP312 Build/10.1.C.0.370) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3016940 = {S54488, 83, { &P18141,&P24346,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 2.60) AppleWebKit/536.26 (KHTML, like Gecko) Silk/3.2*/
const Device UA2753458 = {S54489, 131, { &P17337,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500R Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755526 = {S54490, 130, { &P17337,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2798878 = {S54491, 131, { &P17337,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2938157 = {S54492, 131, { &P17337,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2029589 = {S54493, 127, { &P20175,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1S Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2224284 = {S54494, 136, { &P20175,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2228414 = {S54495, 128, { &P20175,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1SC Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2350217 = {S54496, 137, { &P20175,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2416747 = {S54497, 127, { &P20175,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1S Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2525874 = {S54498, 128, { &P20175,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1SC Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2689157 = {S54499, 122, { &P20175,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1S Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2725283 = {S54500, 58, { &P20175,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MI 1S Build/IMM76D)*/
const Device UA2750032 = {S54501, 58, { &P20175,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MI 1S MIUI/ICS24.0)*/
const Device UA2677117 = {S54502, 146, { &P15767,&P18117,&P22593,&P18092 }};	/*Useragent - Mozilla/5.0(iPad; U;CPU OS 5_1 like Mac OS X; zh-CN; iPad3,1) AppleWebKit/534.46 (KHTML, like Gecko) UCBrowser/2.0.1.280 U3/0.8.0 Safari/7543.48.3*/
const Device UA2948287 = {S54503, 93, { &P14188,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5233) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3018592 = {S54504, 94, { &P14188,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia 5233) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA244112 = {S54505, 135, { &P17957,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID8125 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458772 = {S54506, 76, { &P17957,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.2 beta 2 (Linux; U; Android 2.3.3; es_US; MID8125 Build/MID8125 X)*/
const Device UA459007 = {S54507, 59, { &P17957,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID8125 Build/GRI40)*/
const Device UA2261290 = {S54508, 142, { &P14826,&P5843,&P24668,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (04.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac1-01) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2748484 = {S54509, 138, { &P22748,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E425 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2784800 = {S54510, 160, { &P22748,&P18111,&P18112,&P18092 }};	/*Useragent - LG-E425f-AB/E425f10d Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; LG-E425f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879141 = {S54511, 139, { &P22748,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E425g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2995096 = {S54512, 60, { &P22748,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E425 Build/JZO54K)*/
const Device UA2997299 = {S54513, 61, { &P22748,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E425f Build/JZO54K)*/
const Device UA3103229 = {S54514, 61, { &P22748,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E425g Build/JZO54K)*/
const Device UA2974379 = {S54515, 131, { &P18606,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974918 = {S54516, 138, { &P18606,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2737224 = {S54517, 97, { &P17595,&P17297,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:21.0) Gecko/X Firefox/21.0 AlexaToolbar/alxf-2.18*/
const Device UA2753338 = {S54518, 134, { &P17968,&P18149,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; cm_tenderloin Build/GWK74) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754266 = {S54519, 135, { &P17968,&P18149,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; cm_tenderloin Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2974145 = {S54520, 137, { &P19894,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976139 = {S54521, 138, { &P19894,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830C Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2206014 = {S54522, 139, { &P19870,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-L710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455079 = {S54523, 61, { &P19870,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-L710 Build/IMM76D)*/
const Device UA3131343 = {S54524, 139, { &P16292,&P21637,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Galaxy Nexus Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3065384 = {S54525, 138, { &P18724,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-H Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3065524 = {S54526, 133, { &P18724,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SmartTabII7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Safari/537.36*/
const Device UA3068321 = {S54527, 138, { &P18724,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-F Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3081276 = {S54528, 130, { &P18724,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A2107A-H Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA64208 = {S54529, 104, { &P12502,&P17485,&P4223,&P18092 }};	/*Useragent - SonyEricssonW20i/R7DA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2*/
const Device UA2567690 = {S54530, 146, { &P18512,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT881 Build/9.8.3O-118_USC-14) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA766842 = {S54531, 127, { &P21212,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; N90 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1960436 = {S54532, 138, { &P21212,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N90 DUAL CORE2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1994693 = {S54533, 142, { &P21212,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N90 DUAL CORE2 V12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2365340 = {S54534, 139, { &P21212,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N90 DUAL CORE 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2412198 = {S54535, 56, { &P21212,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; N90 Build/IML74K)*/
const Device UA2545086 = {S54536, 147, { &P21212,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; N90 Build/IML74K; 768*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2199853 = {S54537, 47, { &P18254,&P18580,&P17470,&P18092 }};	/*Useragent - FreeRSSReader/1.9.5 CFNetwork/609 Darwin/13.0.0*/
const Device UA2205990 = {S54538, 45, { &P18254,&P18580,&P17470,&P18092 }};	/*Useragent - MobileRSSFree/3.4 CFNetwork/609 Darwin/13.0.0*/
const Device UA2238605 = {S54539, 41, { &P18254,&P18580,&P17470,&P18092 }};	/*Useragent - RSSRadio/2.07 CFNetwork/609 Darwin/13.0.0*/
const Device UA2369035 = {S54540, 43, { &P18254,&P18580,&P17470,&P18092 }};	/*Useragent - MobileRSS/3.4.1 CFNetwork/609 Darwin/13.0.0*/
const Device UA2509848 = {S54541, 50, { &P18254,&P18580,&P17470,&P18092 }};	/*Useragent - FeeddlerRSS/1.12.5 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2659832 = {S54542, 56, { &P18254,&P18580,&P17470,&P18092 }};	/*Useragent - MusicDownloaderSSA/1.0.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2664050 = {S54543, 52, { &P18254,&P18580,&P17470,&P18092 }};	/*Useragent - Pics4Facebook/9.15.1 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2540015 = {S54544, 133, { &P18466,&P18110,&P21418,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Safari/535.19 Silk-Accelerated=false*/
const Device UA2565796 = {S54545, 149, { &P15919,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3100 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2624912 = {S54546, 155, { &P15919,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3100 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2629191 = {S54547, 150, { &P15919,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3100 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2752025 = {S54548, 149, { &P15919,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; fr-CA; GT-P3100 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Safari/534.30*/
const Device UA2968699 = {S54549, 157, { &P15919,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3100 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1.2/p3100*/
const Device UA2566813 = {S54550, 136, { &P17968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; cm_tenderloin Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583293 = {S54551, 137, { &P17968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; skz_tenderloin Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583340 = {S54552, 122, { &P17968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2584416 = {S54553, 128, { &P17968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; V1013 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2602012 = {S54554, 144, { &P17968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbl1 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2603302 = {S54555, 157, { &P17968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Motorola Galaxy s3 Build/9.8.2O-72_VZW-8) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2603478 = {S54556, 159, { &P17968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1911_v77_gq2000_a41_6628_v20 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2603940 = {S54557, 153, { &P17968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbla668_fwvga Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667430 = {S54558, 159, { &P17968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbla668_9p017_fwvga Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667834 = {S54559, 155, { &P17968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1909l_v77_gq1008_a41_v20 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2702482 = {S54560, 155, { &P17968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1908_v77_jbl1_9p017_6628 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2702969 = {S54561, 129, { &P17968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2733055 = {S54562, 153, { &P17968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1920_v77_zlh_9p017_asx Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2766309 = {S54563, 146, { &P17968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_gq2002 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2860029 = {S54564, 137, { &P17968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.5; iPhone5 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2865276 = {S54565, 153, { &P17968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_gq2008_ov8825 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2891038 = {S54566, 150, { &P17968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jdt1_9p017 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2988292 = {S54567, 147, { &P17968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; e2001_v89_gq2008s Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2989053 = {S54568, 149, { &P17968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1920_v77_zlh_fwvga Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2989660 = {S54569, 144, { &P17968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1921_v77_cxq1 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2990234 = {S54570, 153, { &P17968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbla668_9p017 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA3064879 = {S54571, 137, { &P18673,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080537 = {S54572, 138, { &P18673,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562L Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2974495 = {S54573, 140, { &P17625,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST25i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021667 = {S54574, 140, { &P17625,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST25a Build/6.1.1.B.1.10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2770085 = {S54575, 139, { &P18448,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST26a Build/11.2.A.0.21) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2772140 = {S54576, 139, { &P18448,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST26i Build/11.2.A.0.31) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3031565 = {S54577, 69, { &P15364,&P18110,&P22772,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 on x86_64; rv:24.0) Gecko/X Firefox/24.0*/
const Device UA3031564 = {S54578, 76, { &P15364,&P18110,&P22772,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:24.0) Gecko/X Firefox/24.0 Iceweasel/24.0*/
const Device UA2566533 = {S54579, 140, { &P18447,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT30p Build/7.0.A.3.195) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668492 = {S54580, 133, { &P18447,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT30p Build/7.0.A.3.195) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2242365 = {S54581, 53, { &P18088,&P17017,&P24675,&P18092 }};	/*Useragent - IUC(U;iOS 5.1.1;Zh-cn;320*480;)/UCWEB8.8.0.212/42/997*/
const Device UA2356276 = {S54582, 53, { &P18088,&P17017,&P24675,&P18092 }};	/*Useragent - IUC(U;iOS 5.0.1;Zh-cn;320*480;)/UCWEB8.8.0.212/42/800*/
const Device UA2429600 = {S54583, 86, { &P18088,&P17017,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2436625 = {S54584, 53, { &P18088,&P17017,&P24675,&P18092 }};	/*Useragent - IUC(U;iOS 6_0_1;en-us;320*480;)/UCWEB8.8.4.247/44/355*/
const Device UA2461759 = {S54585, 87, { &P18088,&P17017,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; mstar) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2461811 = {S54586, 86, { &P18088,&P17017,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; maui) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2469292 = {S54587, 87, { &P18088,&P17017,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; spice) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2484170 = {S54588, 87, { &P18088,&P17017,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; opera) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2486751 = {S54589, 86, { &P18088,&P17017,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; maui) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2487018 = {S54590, 89, { &P18088,&P17017,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; mozilla) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2487068 = {S54591, 89, { &P18088,&P17017,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; generic) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2556007 = {S54592, 94, { &P18088,&P17017,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; jbed-fastbcc) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2557594 = {S54593, 79, { &P18088,&P17017,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0*/
const Device UA2557779 = {S54594, 96, { &P18088,&P17017,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; mstar platform) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2596772 = {S54595, 90, { &P18088,&P17017,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; sunmicro) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2657254 = {S54596, 94, { &P18088,&P17017,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; maui browser) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2838301 = {S54597, 53, { &P18088,&P17017,&P24675,&P18092 }};	/*Useragent - IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB8.8.0.212/41/800*/
const Device UA2847745 = {S54598, 87, { &P18088,&P17017,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; mstar) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2383044 = {S54599, 138, { &P20973,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LW770 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA251829 = {S54600, 126, { &P18051,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; F-01D Build/F0001) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA624691 = {S54601, 128, { &P18051,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; F-01D Build/V27R41C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA912980 = {S54602, 128, { &P18051,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; F-01D Build/V28R43A) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2033072 = {S54603, 57, { &P18051,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; F-01D Build/V28R43A)*/
const Device UA635710 = {S54604, 63, { &P15364,&P16110,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:9.0) Gecko/X Firefox/9.0*/
const Device UA645747 = {S54605, 68, { &P15364,&P16110,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:9.0) Gecko/X Firefox/9.0*/
const Device UA2589043 = {S54606, 72, { &P15364,&P16110,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; rv:9.0.1) Gecko/X Firefox/9.0.1 Firefox 9.0*/
const Device UA2233788 = {S54607, 136, { &P18299,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP4100 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2040899 = {S54608, 60, { &P15364,&P17188,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0; rv:15.0) Gecko/X Firefox/15.0.1*/
const Device UA23802 = {S54609, 131, { &P15160,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Light Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43231 = {S54610, 133, { &P15160,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Light Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA630377 = {S54611, 55, { &P15160,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; Light Build/FRF91)*/
const Device UA37364 = {S54612, 123, { &P14635,&P5843,&P17431,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-00/08.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.0.9.17.2*/
const Device UA2235201 = {S54613, 131, { &P11684,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5800 XpressMusic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile*/
const Device UA2342053 = {S54614, 43, { &P11684,&P5129,&P24672,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UCWEB8.6.0.180/50/800*/
const Device UA2727748 = {S54615, 131, { &P11684,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokia5800d-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/84/352/UCWEB Mobile  3gpp-gba*/
const Device UA2761221 = {S54616, 44, { &P11684,&P5129,&P24672,&P18092 }};	/*Useragent - Nokia5800w XpressMusic/UCWEB8.6.0.180/50/800*/
const Device UA2205873 = {S54617, 120, { &P12193,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaC5-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile*/
const Device UA2258413 = {S54618, 122, { &P12193,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaC5-00.2) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/355/UCWEB Mobile*/
const Device UA2385839 = {S54619, 131, { &P12193,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiac5-00.2) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA2403856 = {S54620, 129, { &P12193,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiac5-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA2674217 = {S54621, 146, { &P22598,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyC6506 Build/10.1.A.1.395) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2674219 = {S54622, 64, { &P22598,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; C6506 Build/10.1.A.1.395)*/
const Device UA2714073 = {S54623, 142, { &P22598,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; C6506 Build/10.1.A.1.395) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734422 = {S54624, 142, { &P22598,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C6506 Build/10.1.A.1.434) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2795823 = {S54625, 146, { &P22598,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyC6506 Build/10.1.A.1.350) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2796766 = {S54626, 144, { &P22598,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C6506 Build/10.1.1.A.1.253) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879021 = {S54627, 142, { &P22598,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-mx; C6506 Build/10.1.A.1.350) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2991242 = {S54628, 138, { &P24373,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LGMS659 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3029069 = {S54629, 60, { &P24373,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LGMS659 Build/JZO54K)*/
const Device UA191330 = {S54630, 116, { &P15767,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; Intel Mac OS X 10_6_3; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA658946 = {S54631, 111, { &P15767,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; Intel Mac OS X 10_6_8) AppleWebKit/534.30 (KHTML, like Gecko) Version/5.0.5 Safari/534.30*/
const Device UA2205678 = {S54632, 111, { &P15767,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; Intel Mac OS X 10_6_8) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.5 Safari/536.26*/
const Device UA630978 = {S54633, 136, { &P17710,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-ca; LG-P925g Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025202 = {S54634, 60, { &P17710,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; LG-P925g Build/GRJ90)*/
const Device UA2973960 = {S54635, 127, { &P23203,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; A1-810 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3082990 = {S54636, 143, { &P12280,&P18117,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9B206 Safari/7534.48.3*/
const Device UA3083951 = {S54637, 143, { &P12280,&P18117,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9B208 Safari/7534.48.3*/
const Device UA3095884 = {S54638, 141, { &P12280,&P18117,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9B176 Safari/7534.48.3*/
const Device UA3096039 = {S54639, 141, { &P12280,&P18117,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9B179 Safari/7534.48.3*/
const Device UA50273 = {S54640, 128, { &P13217,&P17603,&P4732,&P18092 }};	/*Useragent - PANTECH-P7000/JDUS10052009; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.15645; U; en-US) Opera 9.50*/
const Device UA50284 = {S54641, 125, { &P13217,&P17603,&P4732,&P18092 }};	/*Useragent - PANTECH-P7000/JDUS10052009; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.15645; U; es) Opera 9.50*/
const Device UA2977423 = {S54642, 118, { &P13217,&P17603,&P4732,&P18092 }};	/*Useragent - PANTECH-P7000/JDUSX; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.15645; U; fr) Opera 9.50*/
const Device UA3096160 = {S54643, 131, { &P15767,&P22882,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/11A465 Safari/8536.25*/
const Device UA3115636 = {S54644, 133, { &P15767,&P22882,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/11A501 Safari/8536.25*/
const Device UA2237175 = {S54645, 132, { &P18466,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2566573 = {S54646, 117, { &P15364,&P5690,&P22465,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Iron/25.0.1400.0 Chrome/25.0.1400.0 Safari/537.22*/
const Device UA2704289 = {S54647, 124, { &P15364,&P5690,&P22465,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Iron/25.0.1400.0 Chrome/25.0.1400.0 Safari/537.22*/
const Device UA2935354 = {S54648, 117, { &P15364,&P5690,&P22465,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/537.22 (KHTML, like Gecko) Iron/25.0.1400.0 Chrome/25.0.1400.0 Safari/537.22*/
const Device UA2537054 = {S54649, 130, { &P21772,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D2-721 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2855436 = {S54650, 75, { &P21772,&P18111,&P18112,&P18092 }};	/*Useragent - AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; D2-721 Build/JRO03H)*/
const Device UA3020079 = {S54651, 131, { &P21772,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D2-721G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3107931 = {S54652, 60, { &P21772,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; D2-721G Build/JRO03H)*/
const Device UA2754851 = {S54653, 150, { &P18552,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2074 = {S54654, 60, { &P11649,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia6030/2.0 (5.11) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21250 = {S54655, 61, { &P11649,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia6030b/2.0 (5.25) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA3098628 = {S54656, 285, { &P11649,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia6030/2.0 (5.40) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2), Nokia6030/2.0 (5.40) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA79897 = {S54657, 89, { &P17595,&P17295,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_5) AppleWebKit (KHTML, like Gecko) Chrome/6*/
const Device UA383932 = {S54658, 125, { &P17595,&P17295,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.53 Safari/534.3*/
const Device UA471417 = {S54659, 125, { &P17595,&P17295,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.55 Safari/534.3*/
const Device UA652975 = {S54660, 124, { &P17595,&P17295,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.2 (KHTML, like Gecko) Chrome/6.0.453.1 Safari/534.2*/
const Device UA705354 = {S54661, 125, { &P17595,&P17295,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA1839810 = {S54662, 125, { &P17595,&P17295,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA1840524 = {S54663, 125, { &P17595,&P17295,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA3130875 = {S54664, 128, { &P18639,&P21637,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus 10 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA895878 = {S54665, 64, { &P18711,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-SGH-A157/A157UCLD1 SHP/VPP/R5 NetFront/4.2 SMM-MMS/1.2.0*/
const Device UA2529757 = {S54666, 132, { &P21202,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-P500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2935313 = {S54667, 61, { &P21202,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SPH-P500 Build/JZO54K)*/
const Device UA2753327 = {S54668, 144, { &P21611,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA70983 = {S54669, 145, { &P16336,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; U8800Pro Build/HuaweiU8800Pro) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79571 = {S54670, 146, { &P16336,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; el-gr; U8800 Pro Build/HuaweiU8800Pro) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA653358 = {S54671, 149, { &P16336,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; U8800Pro Build/HuaweiU8800ProB032) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA671251 = {S54672, 145, { &P16336,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; IDEOS X5 Build/HuaweiU8800Pro) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021521 = {S54673, 143, { &P16336,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-cn; U8800+ Build/HuaweiU8800Pro) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023388 = {S54674, 143, { &P16336,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-za; RBM_HD Build/HuaweiU8800Pro) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2686445 = {S54675, 144, { &P16336,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ru-by; MTC Neo Build/HuaweiU8800Pro) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA671415 = {S54676, 143, { &P18730,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M340S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023634 = {S54677, 67, { &P18730,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHW-M340S Build/GINGERBREAD)*/
const Device UA2175379 = {S54678, 142, { &P14669,&P5843,&P24668,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-00) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2566530 = {S54679, 130, { &P18385,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567247 = {S54680, 130, { &P18385,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus 7 Build/JRO03S) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2602924 = {S54681, 128, { &P18385,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; Nexus 7 Build/JRN84D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2190362 = {S54682, 182, { &P14540,&P5843,&P24671,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2403401 = {S54683, 139, { &P18606,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2406019 = {S54684, 133, { &P18606,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2974094 = {S54685, 134, { &P19904,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Hydro Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975530 = {S54686, 133, { &P19904,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C5170 Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973902 = {S54687, 135, { &P21606,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LGL86C Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2205695 = {S54688, 40, { &P17595,&P18580,&P18534,&P18092 }};	/*Useragent - Reeder/1.5.7 CFNetwork/609 Darwin/13.0.0*/
const Device UA603517 = {S54689, 137, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.162.11; es-es) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA682422 = {S54690, 137, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.162.11; de-de) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA828007 = {S54691, 137, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.162.11; nl-nl) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA835928 = {S54692, 137, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.161.11; en-gb) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA864402 = {S54693, 137, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.161.11; en-ie) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA897228 = {S54694, 131, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710a; en-au) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA899398 = {S54695, 121, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC-Z710a; en-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA1953784 = {S54696, 121, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC-Z710a; fr-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA1956783 = {S54697, 137, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.163.12; fr-fr) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA1983788 = {S54698, 137, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.166.11; de-ch) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2152530 = {S54699, 137, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.165.11; it-it) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2157091 = {S54700, 137, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.162.11; en-de) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2334847 = {S54701, 137, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.166.11; fr-ch) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2388521 = {S54702, 137, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.162.52; de-de) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2404780 = {S54703, 137, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.161.52; en-gb) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2405116 = {S54704, 137, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.162.52; nl-nl) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2423675 = {S54705, 137, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.161.52; en-ie) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2428136 = {S54706, 137, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.162.52; es-es) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2467242 = {S54707, 137, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.166.52; de-ch) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2483297 = {S54708, 137, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.163.53; fr-fr) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2554930 = {S54709, 137, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.165.52; it-it) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2984359 = {S54710, 137, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.166.52; fr-ch) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA3085046 = {S54711, 137, { &P15438,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/Sensation/3.32.163.12; de-at) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA846766 = {S54712, 49, { &P15364,&P17017,&P22528,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MidEx 0.01; Midori 0.01)*/
const Device UA2483664 = {S54713, 50, { &P15364,&P17017,&P22528,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MidEx 0.01; Midori 0.01;)*/
const Device UA2189930 = {S54714, 146, { &P18649,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Micromax A57 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2500864 = {S54715, 70, { &P18649,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Micromax A57 Build/GINGERBREAD)*/
const Device UA2753764 = {S54716, 156, { &P18442,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7100 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2769481 = {S54717, 156, { &P18442,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N7100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2770709 = {S54718, 156, { &P18442,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-N719 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2879523 = {S54719, 156, { &P18442,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SGH-T889 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2997388 = {S54720, 156, { &P18442,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7105 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2997494 = {S54721, 156, { &P18442,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; SCH-N719 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2405233 = {S54722, 140, { &P18452,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT29i Build/7.0.A.3.212) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA878789 = {S54723, 142, { &P19752,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; BLU Studio 5.3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2775660 = {S54724, 66, { &P19752,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; BLU Studio 5.3 Build/GRJ90)*/
const Device UA2566637 = {S54725, 139, { &P18448,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST26i Build/11.0.A.7.5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2601153 = {S54726, 139, { &P18448,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST26a Build/11.0.A.7.5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2974386 = {S54727, 133, { &P18665,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; N861 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA458717 = {S54728, 135, { &P17996,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID1126 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458995 = {S54729, 69, { &P17996,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.3 (Linux; U; Android 2.3.3; en_US; MID1126 Build/MID1126 X)*/
const Device UA459040 = {S54730, 59, { &P17996,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID1126 Build/GRI40)*/
const Device UA23997 = {S54731, 117, { &P13784,&P17484,&P3834,&P18092 }};	/*Useragent - SAMSUNG-GT-B2710/B2710XXJK1 SHP/VPP/R5 Dolfin/2.0 QTV/5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B*/
const Device UA24709 = {S54732, 119, { &P13784,&P17484,&P3834,&P18092 }};	/*Useragent - SAMSUNG-GT-B2710D/B2710DDDJJ2 SHP/VPP/R5 Dolfin/2.0 QTV/5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B*/
const Device UA94196 = {S54733, 104, { &P16971,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-A877/A877UCID3 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA2011374 = {S54734, 132, { &P12879,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P7510 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2235647 = {S54735, 132, { &P12879,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7510 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2765493 = {S54736, 141, { &P23240,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; USCC-C6721 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2894488 = {S54737, 63, { &P23240,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; USCC-C6721 Build/JZO54K)*/
const Device UA2773493 = {S54738, 134, { &P15767,&P17146,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9A405 Safari/7534.48.3*/
const Device UA2784632 = {S54739, 132, { &P15767,&P17146,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9A334 Safari/7534.48.3*/
const Device UA3066230 = {S54740, 136, { &P22871,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9295 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3064993 = {S54741, 138, { &P23227,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I467 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3083476 = {S54742, 131, { &P23227,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I467M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2415874 = {S54743, 138, { &P18594,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LS860 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2556835 = {S54744, 134, { &P18594,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Mach Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA379322 = {S54745, 54, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC46 UP.Browser/6.2_7.2.7.1.K.7.108 (GUI) MMP/2.0*/
const Device UA415622 = {S54746, 58, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SA3B UP.Browser/6.2_7.2.7.1.K.1.5.1.120 (GUI) MMP/2.0*/
const Device UA470846 = {S54747, 54, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SH35 UP.Browser/6.2_7.2.7.1.K.2.207 (GUI) MMP/2.0*/
const Device UA487744 = {S54748, 58, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SN3C UP.Browser/6.2_7.2.7.1.K.1.5.1.120 (GUI) MMP/2.0*/
const Device UA492114 = {S54749, 51, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-HI38 UP.Browser/6.2.0.11.1.3.110 (GUI) MMP/2.0*/
const Device UA525993 = {S54750, 54, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3K UP.Browser/6.2_7.2.7.1.K.2.234 (GUI) MMP/2.0*/
const Device UA529900 = {S54751, 48, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3A UP.Browser/6.2.0.11.2.1e (GUI) MMP/2.0*/
const Device UA551804 = {S54752, 54, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SH3C UP.Browser/6.2_7.2.7.1.K.3.352 (GUI) MMP/2.0*/
const Device UA560612 = {S54753, 54, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-CA3B UP.Browser/6.2_7.2.7.1.K.2.234 (GUI) MMP/2.0*/
const Device UA563608 = {S54754, 54, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3Y UP.Browser/6.2_7.2.7.1.K.7.112 (GUI) MMP/2.0*/
const Device UA563822 = {S54755, 54, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3Z UP.Browser/6.2_7.2.7.1.K.7.112 (GUI) MMP/2.0*/
const Device UA569236 = {S54756, 54, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-HI3F UP.Browser/6.2_7.2.7.1.K.3.354 (GUI) MMP/2.0*/
const Device UA624103 = {S54757, 54, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SN3U UP.Browser/6.2_7.2.7.1.K.7.112 (GUI) MMP/2.0*/
const Device UA662023 = {S54758, 54, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SH37 UP.Browser/6.2_7.2.7.1.K.2.232 (GUI) MMP/2.0*/
const Device UA669785 = {S54759, 54, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-HI3G UP.Browser/6.2_7.2.7.1.K.3.354 (GUI) MMP/2.0*/
const Device UA696824 = {S54760, 54, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-CA3K UP.Browser/6.2_7.2.7.1.K.7.112 (GUI) MMP/2.0*/
const Device UA698176 = {S54761, 54, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3M UP.Browser/6.2_7.2.7.1.K.2.234 (GUI) MMP/2.0*/
const Device UA758527 = {S54762, 58, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SA3D UP.Browser/6.2_7.2.7.1.K.1.5.1.120 (GUI) MMP/2.0*/
const Device UA786011 = {S54763, 54, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SA3E UP.Browser/6.2_7.2.7.1.K.2.232 (GUI) MMP/2.0*/
const Device UA1891716 = {S54764, 54, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-CA3D UP.Browser/6.2_7.2.7.1.K.3.354 (GUI) MMP/2.0*/
const Device UA1933103 = {S54765, 54, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-FJ31 UP.Browser/6.2_7.2.7.1.K.8.160 (GUI) MMP/2.0*/
const Device UA1981294 = {S54766, 54, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SN3V UP.Browser/6.2_7.2.7.1.K.7.113 (GUI) MMP/2.0*/
const Device UA2267356 = {S54767, 54, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3L UP.Browser/6.2_7.2.7.1.K.2.234 (GUI) MMP/2.0*/
const Device UA2270461 = {S54768, 50, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC38 UP.Browser/6.2.0.11.1.2.2e (GUI) MMP/2.0*/
const Device UA2339879 = {S54769, 56, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC4A UP.Browser/6.2_7.2.7.1.K.7.1.104 (GUI) MMP/2.0*/
const Device UA2697863 = {S54770, 51, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-PT37 UP.Browser/6.2.0.17.2.4.1.2 (GUI) MMP/2.0*/
const Device UA2786166 = {S54771, 45, { &P15364,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC45 UP.Browser/6.2.0.17.2 (GUI) MMP/2.0*/
const Device UA2406448 = {S54772, 75, { &P15364,&P17285,&P20457,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:17.0) Gecko/X Firefox/17.0 SeaMonkey/2.14.1*/
const Device UA2515657 = {S54773, 47, { &P15767,&P19826,&P17470,&P18092 }};	/*Useragent - FeeddlerRSS 1.12.5 (iPad; iPhone OS 6.1; en_US)*/
const Device UA2563361 = {S54774, 49, { &P15767,&P19826,&P17470,&P18092 }};	/*Useragent - FeeddlerRSS 1.12.5 (iPad; iPhone OS 6.1.2; en_US)*/
const Device UA2604681 = {S54775, 59, { &P15767,&P19826,&P17470,&P18092 }};	/*Useragent - Appcelerator Titanium/1.7.2 (iPad/6.1.3; iPhone OS; en_US;)*/
const Device UA2607372 = {S54776, 49, { &P15767,&P19826,&P17470,&P18092 }};	/*Useragent - FeeddlerRSS 1.12.5 (iPad; iPhone OS 6.1.3; en_US)*/
const Device UA2507724 = {S54777, 116, { &P14482,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-01.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2560376 = {S54778, 114, { &P14482,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-01/07.15; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA3083956 = {S54779, 112, { &P18506,&P5043,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia305/05.92; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA2161092 = {S54780, 78, { &P15364,&P18346,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Avant Browser)*/
const Device UA2167310 = {S54781, 78, { &P15364,&P18346,&P22731,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Avant Browser)*/
const Device UA2775597 = {S54782, 83, { &P15364,&P18346,&P22731,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Avant Browser; SV1)*/
const Device UA2787370 = {S54783, 83, { &P15364,&P18346,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Avant Browser; SV1)*/
const Device UA2950214 = {S54784, 93, { &P15364,&P18346,&P22731,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Avant Browser; Avant Browser)*/
const Device UA2950353 = {S54785, 93, { &P15364,&P18346,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 4.1; Avant Browser; Avant Browser)*/
const Device UA2030496 = {S54786, 139, { &P20076,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26w Build/6.1.A.2.45) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3061598 = {S54787, 142, { &P18442,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Note II Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3065999 = {S54788, 137, { &P18442,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080020 = {S54789, 137, { &P18442,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7105 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080713 = {S54790, 145, { &P18442,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I317 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081393 = {S54791, 137, { &P18442,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T889 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094832 = {S54792, 137, { &P18442,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L900 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127069 = {S54793, 137, { &P18442,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R950 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2956254 = {S54794, 136, { &P24368,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C5215 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3027893 = {S54795, 58, { &P24368,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; C5215 Build/JZO54K)*/
const Device UA574996 = {S54796, 90, { &P15216,&P7149,&P4779,&P18092 }};	/*Useragent - BlackBerry8520/5.2.0.64 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/159/444*/
const Device UA2735243 = {S54797, 138, { &P22916,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E450 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2780705 = {S54798, 139, { &P22916,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E450f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2897087 = {S54799, 139, { &P22916,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E450g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937497 = {S54800, 61, { &P22916,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E450f Build/JZO54K)*/
const Device UA2998389 = {S54801, 60, { &P22916,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E450 Build/JZO54K)*/
const Device UA3019674 = {S54802, 155, { &P22916,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LG-E450B/E450B11e-BLM-CA Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3021280 = {S54803, 155, { &P22916,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LG-E450B/E450B11c-BLM-CA Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3107562 = {S54804, 61, { &P22916,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E450g Build/JZO54K)*/
const Device UA3108085 = {S54805, 139, { &P22916,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LG-E450B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA30326 = {S54806, 73, { &P12297,&P5208,&P4628,&P18092 }};	/*Useragent - Mozilla/4.0 (BREW 3.1.5; U; en-us; Sanyo; Polaris/6.0/AMB) Boost SCP-2700*/
const Device UA54633 = {S54807, 74, { &P12297,&P5208,&P4628,&P18092 }};	/*Useragent - Mozilla/4.0 (BREW 3.1.5; U; en-us; Sanyo; Polaris/6.0/AMB) Sprint SCP-2700*/
const Device UA2205325 = {S54808, 171, { &P13123,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2235822 = {S54809, 171, { &P13123,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2554454 = {S54810, 171, { &P13123,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SCH-i509 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2203472 = {S54811, 74, { &P18254,&P5690,&P17200,&P18092 }};	/*Useragent - CybEye.com/2.0 (compatible; MSIE 9.0; Windows NT 5.1; Trident/4.0; GTB6.4)*/
const Device UA2361889 = {S54812, 128, { &P20268,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T711 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2764566 = {S54813, 128, { &P20268,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; T711 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1335 = {S54814, 61, { &P11625,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia5300/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA529482 = {S54815, 71, { &P11625,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia5300/2.0 (05.00) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia5300*/
const Device UA79835 = {S54816, 34, { &P15364,&P17017,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0;*/
const Device UA464937 = {S54817, 34, { &P15364,&P17017,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0)*/
const Device UA2509912 = {S54818, 52, { &P15364,&P17017,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; PPC; Trident/1.0)*/
const Device UA2792676 = {S54819, 57, { &P15364,&P17017,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; PPC; SV1; Trident/3.0)*/
const Device UA3064990 = {S54820, 129, { &P22922,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; M470BSA Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA437923 = {S54821, 42, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.5 (Downcast v2.5.5 (Build 1001))*/
const Device UA452914 = {S54822, 42, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.5 (Downcast v2.5.2 (Build 1006))*/
const Device UA457596 = {S54823, 42, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.6 (Downcast v2.7.0 (Build 1001))*/
const Device UA576564 = {S54824, 42, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.0 (Downcast v2.1.3 (Build 1004))*/
const Device UA611155 = {S54825, 42, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.6 (Downcast v2.7.1 (Build 1029))*/
const Device UA636012 = {S54826, 42, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.5 (Downcast v2.5.6 (Build 1015))*/
const Device UA901099 = {S54827, 42, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.6 (Downcast v2.7.2 (Build 1012))*/
const Device UA905289 = {S54828, 42, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.6 (Downcast v2.7.3 (Build 1004))*/
const Device UA912982 = {S54829, 42, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.6 (Downcast v2.7.4 (Build 1008))*/
const Device UA1979141 = {S54830, 33, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.5.2 (PodCruncher 1.9.2)*/
const Device UA1997035 = {S54831, 42, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.6 (Downcast v2.7.5 (Build 1013))*/
const Device UA2001886 = {S54832, 42, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.6 (Downcast v2.7.6 (Build 1035))*/
const Device UA2007595 = {S54833, 42, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.6 (Downcast v2.7.7 (Build 1000))*/
const Device UA2203586 = {S54834, 42, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.6 (Downcast v2.7.8 (Build 1053))*/
const Device UA2205043 = {S54835, 42, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.6 (Downcast v2.7.9 (Build 1004))*/
const Device UA2232332 = {S54836, 43, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.6 (Downcast v2.7.10 (Build 1015))*/
const Device UA2242324 = {S54837, 43, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.6 (Downcast v2.7.11 (Build 1032))*/
const Device UA2242355 = {S54838, 31, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.5.2 (PodCruncher 2.0)*/
const Device UA2363160 = {S54839, 42, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Downcast/2.7.12 (Build 1005))*/
const Device UA2487582 = {S54840, 32, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 Downcast/2.7.15.1007*/
const Device UA2533321 = {S54841, 31, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 Downcast/2.8.0.1025*/
const Device UA2582776 = {S54842, 31, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 Downcast/2.8.1.1034*/
const Device UA2599987 = {S54843, 31, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 Downcast/2.8.2.2001*/
const Device UA2740587 = {S54844, 31, { &P15364,&P7480,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 Downcast/2.8.3.2097*/
const Device UA3035431 = {S54845, 151, { &P17595,&P17297,&P24176,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36 OPR/16.0.1196.55 (Edition Next)*/
const Device UA3068054 = {S54846, 136, { &P17595,&P17297,&P24176,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36 OPR/16.0.1196.73*/
const Device UA3084200 = {S54847, 136, { &P17595,&P17297,&P24176,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.11 Safari/537.36  OPR/16.0.1146.1*/
const Device UA3084230 = {S54848, 161, { &P17595,&P17297,&P24176,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.49 Safari/537.36 OPR/16.0.1196.41 (Edition Next),gzip(gfe)*/
const Device UA3084232 = {S54849, 151, { &P17595,&P17297,&P24176,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.49 Safari/537.36 OPR/16.0.1196.41 (Edition Next)*/
const Device UA3084244 = {S54850, 151, { &P17595,&P17297,&P24176,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36 OPR/16.0.1196.55 (Edition Next)*/
const Device UA3084269 = {S54851, 146, { &P17595,&P17297,&P24176,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36 OPR/16.0.1196.62,gzip(gfe)*/
const Device UA3084270 = {S54852, 136, { &P17595,&P17297,&P24176,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36 OPR/16.0.1196.73*/
const Device UA3084271 = {S54853, 155, { &P17595,&P17297,&P24176,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36 OPR/16.0.1196.62 (Edition Freesoft)*/
const Device UA3084274 = {S54854, 136, { &P17595,&P17297,&P24176,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36 OPR/16.0.1196.73*/
const Device UA3084280 = {S54855, 154, { &P17595,&P17297,&P24176,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36 OPR/16.0.1196.73 (Edition Rambler)*/
const Device UA2012160 = {S54856, 132, { &P21117,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; HS-9DTB4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2426219 = {S54857, 136, { &P21117,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; HS-9DTB4-4GB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2981994 = {S54858, 61, { &P21117,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HS-9DTB4 Build/IMM76D)*/
const Device UA2315026 = {S54859, 150, { &P20019,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; ALCATEL_one_touch_916 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2366902 = {S54860, 151, { &P20019,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; ALCATEL ONE TOUCH 916D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2476476 = {S54861, 150, { &P20019,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; ALCATEL ONE TOUCH 916 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2593338 = {S54862, 159, { &P20019,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-pt; Alcatel_one_touch_916_Helsinki Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652844 = {S54863, 151, { &P20019,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; ALCATEL ONE TOUCH 916A Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA114816 = {S54864, 45, { &P17595,&P18120,&P17215,&P18092 }};	/*Useragent - Opera/9.50 (Macintosh; Intel Mac OS X; U; en)*/
const Device UA152059 = {S54865, 45, { &P17595,&P18120,&P17215,&P18092 }};	/*Useragent - Opera/9.51 (Macintosh; Intel Mac OS X; U; en)*/
const Device UA174676 = {S54866, 45, { &P17595,&P18120,&P17215,&P18092 }};	/*Useragent - Opera/9.52 (Macintosh; Intel Mac OS X; U; en)*/
const Device UA1993043 = {S54867, 140, { &P19979,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E170K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027488 = {S54868, 140, { &P19979,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E170S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030054 = {S54869, 140, { &P19979,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E170L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2775129 = {S54870, 62, { &P19979,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHV-E170K Build/IMM76D)*/
const Device UA2437646 = {S54871, 83, { &P18141,&P19825,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 2.02) AppleWebKit/536.26 (KHTML, like Gecko) Silk/3.2*/
const Device UA2677083 = {S54872, 133, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; H100 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2706896 = {S54873, 126, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MiSS Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2707756 = {S54874, 136, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ThinkPadTablet Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708558 = {S54875, 127, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; y7053 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2713173 = {S54876, 133, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B943 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734894 = {S54877, 134, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A9600 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735924 = {S54878, 137, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; YOUPU U6 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2736022 = {S54879, 131, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; M7 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2750934 = {S54880, 134, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.4; Note3 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2780776 = {S54881, 134, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; X720D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2780942 = {S54882, 133, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; 9930 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2860928 = {S54883, 137, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CrystalQuadCore Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2863874 = {S54884, 124, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; G1 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2958810 = {S54885, 135, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MD7305 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2964347 = {S54886, 124, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; S9 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3003822 = {S54887, 125, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M71 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3006564 = {S54888, 127, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Wb7-L Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3007537 = {S54889, 126, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MT7A Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3015399 = {S54890, 131, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ST70216-1 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3016820 = {S54891, 144, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Starmobile JUMP Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3018363 = {S54892, 136, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MTC 970 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3021932 = {S54893, 130, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TBQC1063 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3028126 = {S54894, 139, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Digital2 Platinum Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3056929 = {S54895, 130, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PLT8223G Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3059487 = {S54896, 137, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MTC 970H Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3072887 = {S54897, 138, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Boston 4G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3072916 = {S54898, 145, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; G526-L22 Build/HuaweiG526-L22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3078334 = {S54899, 135, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Mi-491 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3092197 = {S54900, 139, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Positivo Ypy L700 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3092469 = {S54901, 128, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EGS102 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3107993 = {S54902, 137, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E415f Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3108084 = {S54903, 137, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S7392 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3108501 = {S54904, 143, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Camelio Family tablet Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3109272 = {S54905, 133, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; N810 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3121581 = {S54906, 134, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; N9510 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3124954 = {S54907, 138, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; WHAM WT72 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3125151 = {S54908, 140, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XO Learning tablet Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3126064 = {S54909, 148, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Huawei Y301A2 Build/HuaweiY301A2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3127524 = {S54910, 133, { &P15364,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; N800 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2752950 = {S54911, 141, { &P18462,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2938584 = {S54912, 150, { &P18462,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X dual sim Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2341416 = {S54913, 149, { &P20088,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666N Build/HuaweiU8666N) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2355482 = {S54914, 149, { &P20088,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E Build/HuaweiU8666E) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2386286 = {S54915, 155, { &P20088,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E-51 Build/HuaweiU8666E-51) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2974081 = {S54916, 138, { &P18724,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-H Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974907 = {S54917, 138, { &P18724,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-F Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022142 = {S54918, 133, { &P18724,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SmartTabII7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022344 = {S54919, 130, { &P18724,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A2107A-H Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2891018 = {S54920, 139, { &P23760,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-LG870 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3018389 = {S54921, 61, { &P23760,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-LG870 Build/JZO54K)*/
const Device UA56012 = {S54922, 100, { &P14640,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; OMNIA7)*/
const Device UA56099 = {S54923, 108, { &P14640,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; OMNIA7; Orange)*/
const Device UA90317 = {S54924, 102, { &P14640,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; GT-i8700)*/
const Device UA2566757 = {S54925, 141, { &P17625,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST25i Build/6.1.1.B.1.10) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2569932 = {S54926, 141, { &P17625,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST25a Build/6.1.1.B.1.10) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2461240 = {S54927, 131, { &P20420,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; I10A-LE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2008881 = {S54928, 155, { &P16362,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-D710 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2200321 = {S54929, 159, { &P16362,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-D710 Build/JRO03R; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258198 = {S54930, 155, { &P16362,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-D710 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA171618 = {S54931, 134, { &P18041,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; YP-GB1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA192217 = {S54932, 134, { &P18041,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; YP-GB1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA563889 = {S54933, 133, { &P18041,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; YP-G1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1877645 = {S54934, 58, { &P18041,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; YP-GB1 Build/FROYO)*/
const Device UA2019974 = {S54935, 133, { &P18041,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; YP-G1 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032694 = {S54936, 57, { &P18041,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; YP-G1 Build/FROYO)*/
const Device UA12894 = {S54937, 171, { &P11702,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB300 Build/GINtech_Systems_DeBlurred Flex/P007) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA12898 = {S54938, 141, { &P11702,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; ME600 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA34378 = {S54939, 183, { &P11702,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB300 Build/Blur_Version.3.0.1390.MB300.ATT.en.US Flex/P007) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA34379 = {S54940, 141, { &P11702,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB300 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA40644 = {S54941, 183, { &P11702,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; ME600 Build/Blur_Version.3.0.1390.MB300.ATT.en.US Flex/P020) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA55967 = {S54942, 142, { &P11702,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB300 Build/iaio72) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1865299 = {S54943, 63, { &P11702,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; MB300 Build/ERD79)*/
const Device UA2754225 = {S54944, 137, { &P18559,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LG730 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA350741 = {S54945, 144, { &P12496,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; SGH-I897 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA360091 = {S54946, 144, { &P12496,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-I897 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA398477 = {S54947, 139, { &P12496,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-I897 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA454822 = {S54948, 139, { &P12496,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I897 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA647704 = {S54949, 139, { &P12496,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I897 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA887243 = {S54950, 139, { &P12496,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I897 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025356 = {S54951, 61, { &P12496,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SGH-I897 Build/IML74K)*/
const Device UA2027930 = {S54952, 139, { &P12496,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I897 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031184 = {S54953, 139, { &P12496,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I897 Build/IML76K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033199 = {S54954, 61, { &P12496,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I897 Build/IMM76L)*/
const Device UA2897662 = {S54955, 157, { &P15364,&P7133,&P22777,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.4.2.34167 (Firestorm-Release; firestorm skin) Safari/533.3*/
const Device UA2973903 = {S54956, 138, { &P19777,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975039 = {S54957, 137, { &P19777,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2030427 = {S54958, 155, { &P16356,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7000 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2041597 = {S54959, 155, { &P16356,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7000 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2237864 = {S54960, 159, { &P16356,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2352437 = {S54961, 159, { &P16356,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7000 Build/JRO03R; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3071046 = {S54962, 153, { &P16356,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; SC-05D Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2156452 = {S54963, 34, { &P12280,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.0 (6; 32GB; dt:73)*/
const Device UA2202936 = {S54964, 34, { &P12280,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.0 (6; 64GB; dt:82)*/
const Device UA2356590 = {S54965, 36, { &P12280,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.0.1 (6; 16GB; dt:82)*/
const Device UA2356684 = {S54966, 36, { &P12280,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.0.1 (6; 16GB; dt:73)*/
const Device UA2356979 = {S54967, 36, { &P12280,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.0.1 (4; 16GB; dt:27)*/
const Device UA2362634 = {S54968, 36, { &P12280,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.0.1 (4; 32GB; dt:27)*/
const Device UA2362727 = {S54969, 36, { &P12280,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.0.1 (6; 32GB; dt:82)*/
const Device UA2367887 = {S54970, 35, { &P12280,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.0.1 (4; 8GB; dt:27)*/
const Device UA2385899 = {S54971, 36, { &P12280,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.0.1 (6; 64GB; dt:73)*/
const Device UA2386202 = {S54972, 35, { &P12280,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.0.1 (3; 8GB; dt:24)*/
const Device UA2425091 = {S54973, 36, { &P12280,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.0.1 (3; 16GB; dt:24)*/
const Device UA2456527 = {S54974, 36, { &P12280,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.0.1 (5; 16GB; dt:70)*/
const Device UA2481003 = {S54975, 35, { &P12280,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.0.1 (5; 8GB; dt:70)*/
const Device UA2486104 = {S54976, 36, { &P12280,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.0.2 (6; 16GB; dt:79)*/
const Device UA2593348 = {S54977, 34, { &P12280,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.0 (6; 32GB; dt:79)*/
const Device UA2727321 = {S54978, 36, { &P12280,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPhone/6.0.1 (3; 32GB; dt:24)*/
const Device UA69795 = {S54979, 123, { &P15228,&P5043,&P17433,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-03/06.51; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.1.21.8.2*/
const Device UA2351524 = {S54980, 69, { &P15649,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.1; zh-cn; GT-S5570; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2402000 = {S54981, 69, { &P15649,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GT-S5570; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2403026 = {S54982, 70, { &P15649,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GT-S5570I; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2360613 = {S54983, 137, { &P23546,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AD683G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2653852 = {S54984, 59, { &P23546,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; AD683G Build/IMM76I)*/
const Device UA3064987 = {S54985, 129, { &P20991,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ME371MG Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2234220 = {S54986, 76, { &P15364,&P18110,&P20761,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:16.0) Gecko/X Firefox/16.0 SeaMonkey/2.13.1*/
const Device UA2567476 = {S54987, 83, { &P18141,&P21717,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 2.06) AppleWebKit/536.26 (KHTML, like Gecko) Silk/3.2*/
const Device UA3030891 = {S54988, 152, { &P15803,&P5843,&P24679,&P18092 }};	/*Useragent - Nokia200/2.0 (11.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia200) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA22607 = {S54989, 127, { &P15495,&P6590,&P4223,&P18092 }};	/*Useragent - NetFront/3.5.1(BREW 5.0.2.1; U; en-us; Samsung ; NetFront/3.5.1/WAP) Boost M260 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA900910 = {S54990, 129, { &P16976,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT300 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3020790 = {S54991, 138, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XTAB-781+ Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020796 = {S54992, 128, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AN7IG3 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3020800 = {S54993, 133, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; F600 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021067 = {S54994, 133, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SZENIO 2500 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021100 = {S54995, 135, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MD7305 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021270 = {S54996, 133, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B943 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021492 = {S54997, 133, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A600 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021493 = {S54998, 126, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MiSS Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021495 = {S54999, 134, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Boost Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021549 = {S55000, 136, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MTC 970 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021665 = {S55001, 137, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E451g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021782 = {S55002, 133, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; 9920 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021809 = {S55003, 138, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Boston 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021819 = {S55004, 128, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MaxiIQ Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022000 = {S55005, 134, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A9600 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022188 = {S55006, 130, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PMP3970B Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022362 = {S55007, 130, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SN97T41W Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022342 = {S55008, 137, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; YOUPU U6 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022413 = {S55009, 127, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; it900 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022549 = {S55010, 139, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Amazing A4 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022552 = {S55011, 128, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; WPT005 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022626 = {S55012, 132, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A20 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022630 = {S55013, 152, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Bouygues Telecom Bs 351 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022637 = {S55014, 128, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MD7305 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022681 = {S55015, 130, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; iQTab 3G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022703 = {S55016, 137, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CrystalQuadCore Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022855 = {S55017, 133, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Joyz Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022905 = {S55018, 130, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PLT8223G Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023125 = {S55019, 124, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; V8 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023410 = {S55020, 136, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ThinkPadTablet Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023445 = {S55021, 137, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E455g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023691 = {S55022, 133, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; H100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023771 = {S55023, 138, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XOLO Play Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023915 = {S55024, 131, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; M7 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023922 = {S55025, 128, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; U35GT2 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3024040 = {S55026, 133, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024145 = {S55027, 127, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; y7053 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3027010 = {S55028, 126, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; V72A Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3057717 = {S55029, 133, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; W700 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3058060 = {S55030, 134, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MD700 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3058119 = {S55031, 145, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Forward_Infinity Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3059558 = {S55032, 145, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; G526-L22 Build/HuaweiG526-L22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3077930 = {S55033, 135, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; H9500+ Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3078499 = {S55034, 133, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; 9930 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3078733 = {S55035, 133, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; I110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3092399 = {S55036, 143, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; one touch D920 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3107723 = {S55037, 131, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ACGM816KC Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3107742 = {S55038, 140, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ACGM727KC_N Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3125054 = {S55039, 134, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.4; Note3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3126199 = {S55040, 137, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S7392 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3127818 = {S55041, 136, { &P15364,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MID-X15 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2665521 = {S55042, 141, { &P22598,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6506 Build/10.1.A.1.395) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA634074 = {S55043, 87, { &P15364,&P5690,&P18198,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.28) Gecko/X Thunderbird/3.1.20*/
const Device UA2391298 = {S55044, 139, { &P20508,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Cynus T2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2846899 = {S55045, 61, { &P20508,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Cynus T2 Build/IMM76D)*/
const Device UA2566824 = {S55046, 153, { &P15877,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2737345 = {S55047, 134, { &P23182,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; PMP7280C3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2999084 = {S55048, 63, { &P23182,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; PMP7280C3G Build/JZO54K)*/
const Device UA3062782 = {S55049, 138, { &P23182,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; PMP7280C3G_DUO Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA18727 = {S55050, 124, { &P11635,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia6230/2.0 (03.15) Profile/MIDP-2.0 Configuration/CLDC-1.1, Nokia6230/2.0 (03.15) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64584 = {S55051, 61, { &P11635,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia6230/2.0 (03.15) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2754408 = {S55052, 126, { &P16598,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A500 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2769382 = {S55053, 126, { &P16598,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2798505 = {S55054, 133, { &P16598,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Iconia A500 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2868348 = {S55055, 133, { &P16598,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A500 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA92009 = {S55056, 119, { &P17595,&P17295,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3;) AppleWebKit/523.10.3 (KHTML, like Gecko) Version/3.0.4 Safari/523.10*/
const Device UA129398 = {S55057, 125, { &P17595,&P17295,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.0.4 Safari/523.10*/
const Device UA262322 = {S55058, 125, { &P17595,&P17295,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.0.4 Safari/523.15*/
const Device UA274328 = {S55059, 121, { &P17595,&P17295,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_2; en-us) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.0.4 Safari/523.15*/
const Device UA373866 = {S55060, 123, { &P17595,&P17295,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.0.4 Safari/523.15*/
const Device UA386386 = {S55061, 125, { &P17595,&P17295,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.0.4 Safari/523.10*/
const Device UA387108 = {S55062, 123, { &P17595,&P17295,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.0.4 Safari/523.15*/
const Device UA500255 = {S55063, 123, { &P17595,&P17295,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.0.4 Safari/523.10*/
const Device UA704633 = {S55064, 125, { &P17595,&P17295,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.0.4 Safari/523.15*/
const Device UA742747 = {S55065, 125, { &P17595,&P17295,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.0.4 Safari/523.15*/
const Device UA840570 = {S55066, 123, { &P17595,&P17295,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.0.4 Safari/523.10*/
const Device UA1861038 = {S55067, 125, { &P17595,&P17295,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/525.28.3 (KHTML, like Gecko) Version/3.0.4 Safari/523.10*/
const Device UA1863001 = {S55068, 123, { &P17595,&P17295,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.0.4 Safari/523.10*/
const Device UA1920653 = {S55069, 127, { &P17595,&P17295,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.0.4 Safari/523.13.2*/
const Device UA2159239 = {S55070, 121, { &P17595,&P17295,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_3; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.0.4 Safari/523.15*/
const Device UA2159742 = {S55071, 121, { &P17595,&P17295,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5; en-us) AppleWebKit/528.7+ (KHTML, like Gecko) Version/3.0.4 Safari/523.10*/
const Device UA2258281 = {S55072, 125, { &P17595,&P17295,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-us) AppleWebKit/525.8+ (KHTML, like Gecko) Version/3.0.4 Safari/523.10.6*/
const Device UA2269484 = {S55073, 125, { &P17595,&P17295,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; es-es) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/3.0.4 Safari/523.15*/
const Device UA2274823 = {S55074, 123, { &P17595,&P17295,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; en-us) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.0.4 Safari/523.10*/
const Device UA2257343 = {S55075, 150, { &P20088,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-nz; HUAWEI U8666N Build/HuaweiU8666N) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2367051 = {S55076, 150, { &P20088,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HUAWEI U8666E Build/HuaweiU8666E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2527981 = {S55077, 156, { &P20088,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HUAWEI U8666E-51 Build/HuaweiU8666E-51) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2696340 = {S55078, 72, { &P20088,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HUAWEI U8666E Build/HuaweiU8666E)*/
const Device UA2985170 = {S55079, 78, { &P20088,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HUAWEI U8666E-51 Build/HuaweiU8666E-51)*/
const Device UA2257528 = {S55080, 152, { &P20988,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PLT7035-PL Build/Curtis International Ltd) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2403661 = {S55081, 149, { &P20988,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PLT7035 Build/Curtis International Ltd) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2427173 = {S55082, 133, { &P20988,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PLT7035-B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA20885 = {S55083, 149, { &P16092,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; DROID X2 Build/4.4.1_274_DTN-14.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20887 = {S55084, 149, { &P16092,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; DROID X2 Build/4.4.1_274_DTN-14.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20889 = {S55085, 147, { &P16092,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; DROID X2 Build/4.4.1_274_DTN-14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59123 = {S55086, 148, { &P16092,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2.2; en-us; DROID X2 Build/4.4.1_274_DTN-14.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59124 = {S55087, 148, { &P16092,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; DROIDX2 Build/4.4.1_274_DTN-14.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59125 = {S55088, 148, { &P16092,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us;DROID X2 Build/4.4.1_274_DTN-14.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59133 = {S55089, 148, { &P16092,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.2; en-us; DROID X2 Build/4.4.1_274_DTN-14.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59141 = {S55090, 147, { &P16092,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; DROID X2 Build/4.4.1_274_DTN-35) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59139 = {S55091, 148, { &P16092,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; DROID X2Build/4.4.1_274_DTN-14.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA551869 = {S55092, 73, { &P16092,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; DROID X2 Build/4.4.1_274_DTN-14.8)*/
const Device UA2566503 = {S55093, 128, { &P17926,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT100 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2558280 = {S55094, 103, { &P12449,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae5-00) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2716540 = {S55095, 94, { &P12449,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE5-00) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2751924 = {S55096, 96, { &P12449,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE5-00.2) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2953041 = {S55097, 105, { &P12449,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae5-00.2) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3102333 = {S55098, 95, { &P12449,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE5-00u) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA138665 = {S55099, 207, { &P15364,&P5690,&P17193,&P19831 }};	/*Useragent - HuaweiSymantecSpider/1.0+DSE-support@huaweisymantec.com+(compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR ; http://www.huaweisymantec.com/en/IRL/spider)*/
const Device UA574251 = {S55100, 69, { &P15364,&P5690,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0(Windows; U; Windows NT 5.2; rv:1.9.2) Gecko/X Firefox/6.0*/
const Device UA648177 = {S55101, 68, { &P15364,&P5690,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:6.0) Gecko/X Firefox/6.0 Firefox/6.0*/
const Device UA2152966 = {S55102, 80, { &P15364,&P5690,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; ru-RU; rv:5.0) Gecko/false  Firefox/6.0*/
const Device UA2233977 = {S55103, 70, { &P15364,&P5690,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:6.0.2) Gecko/X Firefox/6.0.2,gzip(gfe)*/
const Device UA906293 = {S55104, 113, { &P15241,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia300/07.03; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA2403948 = {S55105, 112, { &P15241,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia300/07.57; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6*/
const Device UA3029407 = {S55106, 102, { &P15364,&P18346,&P21934,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36*/
const Device UA2042275 = {S55107, 99, { &P15364,&P17188,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2356770 = {S55108, 139, { &P19798,&P17979,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (;;; en-us; Huawei-U8651S Build/U8651SV100R001USAC85B843) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3065569 = {S55109, 137, { &P23642,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-M840 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2337815 = {S55110, 56, { &P18254,&P17820,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) Ubuntu/12.04 Codebot/1.0*/
const Device UA2973482 = {S55111, 139, { &P18449,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST27i Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975718 = {S55112, 139, { &P18449,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST27a Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA925397 = {S55113, 137, { &P17595,&P17295,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA2759427 = {S55114, 137, { &P17595,&P17295,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_4) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.385 Chrome/16.0.912.77 Safari/535.7*/
const Device UA2933760 = {S55115, 137, { &P17595,&P17295,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_6) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA755792 = {S55116, 134, { &P21479,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LGL85C Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033498 = {S55117, 58, { &P21479,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LGL85C Build/GRJ22)*/
const Device UA361786 = {S55118, 130, { &P21541,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; TPA60W Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA637873 = {S55119, 128, { &P21541,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; TPA60W Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2022506 = {S55120, 59, { &P21541,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; TPA60W Build/HTK55D)*/
const Device UA2893206 = {S55121, 152, { &P15803,&P5843,&P24678,&P18092 }};	/*Useragent - Nokia200/2.0 (11.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia200) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA63706 = {S55122, 90, { &P11679,&P5776,&P3756,&P18092 }};	/*Useragent - Nokia6630/1.0 (5.03.08) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA59308 = {S55123, 103, { &P14743,&P17485,&P4223,&P18092 }};	/*Useragent - SonyEricssonJ20/R7CA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2*/
const Device UA63992 = {S55124, 104, { &P14743,&P17485,&P4223,&P18092 }};	/*Useragent - SonyEricssonJ20i/R7CA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2*/
const Device UA2566622 = {S55125, 137, { &P18243,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P880 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567267 = {S55126, 138, { &P18243,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P880g Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2706876 = {S55127, 130, { &P18243,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P880 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2797164 = {S55128, 139, { &P22851,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT919 Build/2_290_2012) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2200872 = {S55129, 128, { &P18248,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A700 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2838839 = {S55130, 128, { &P18248,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A700 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2570137 = {S55131, 150, { &P15575,&P17979,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android; 4.1.2; GT-I9100 Build/X) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1234.12 Mobile Safari/537.22 OPR/14.0.123.123*/
const Device UA368821 = {S55132, 145, { &P17108,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Inspire 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA369251 = {S55133, 67, { &P17108,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Inspire 4G Build/IML74K)*/
const Device UA898062 = {S55134, 140, { &P17108,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC-A9192 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1974966 = {S55135, 141, { &P17108,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Inspire 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2009065 = {S55136, 141, { &P17108,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Inspire 4G Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027365 = {S55137, 141, { &P17108,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Inspire 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033250 = {S55138, 144, { &P17108,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC-A9192/1.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2362531 = {S55139, 145, { &P17108,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_Inspire_4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2366841 = {S55140, 145, { &P17108,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_Inspire_4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2466635 = {S55141, 145, { &P17108,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Inspire 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2762326 = {S55142, 67, { &P17108,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Inspire 4G Build/IML74K)*/
const Device UA536222 = {S55143, 104, { &P15364,&P7133,&P19773,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-GB; rv:1.9.1.17) Gecko/X (like Firefox/3.x) SeaMonkey/2.0.12*/
const Device UA2260270 = {S55144, 89, { &P15364,&P7133,&P19773,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.1b3pre) Gecko/X SeaMonkey/2.0a3pre*/
const Device UA606400 = {S55145, 142, { &P19881,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; GT-S5367 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2495351 = {S55146, 66, { &P19881,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S5367 Build/GINGERBREAD)*/
const Device UA244027 = {S55147, 79, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.8)*/
const Device UA313842 = {S55148, 79, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.7)*/
const Device UA415130 = {S55149, 79, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.6)*/
const Device UA484429 = {S55150, 79, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.4)*/
const Device UA486114 = {S55151, 177, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Sleipnir/2.9.6)*/
const Device UA614335 = {S55152, 80, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.10)*/
const Device UA635716 = {S55153, 79, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.9)*/
const Device UA685081 = {S55154, 86, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Sleipnir/2.9.9)*/
const Device UA720927 = {S55155, 177, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Sleipnir/2.9.9)*/
const Device UA735127 = {S55156, 87, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Sleipnir/2.9.10)*/
const Device UA835577 = {S55157, 86, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Sleipnir/2.9.8)*/
const Device UA858778 = {S55158, 86, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Sleipnir/2.9.7)*/
const Device UA924880 = {S55159, 80, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.11)*/
const Device UA2008975 = {S55160, 80, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.12)*/
const Device UA2144710 = {S55161, 79, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.3)*/
const Device UA2168189 = {S55162, 87, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Sleipnir/2.9.11)*/
const Device UA2205250 = {S55163, 80, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.14)*/
const Device UA2234295 = {S55164, 87, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Sleipnir/2.9.14)*/
const Device UA2344760 = {S55165, 87, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Sleipnir/2.9.12)*/
const Device UA2382501 = {S55166, 157, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Sleipnir/2.9.14)*/
const Device UA2429052 = {S55167, 87, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Sleipnir/2.9.15)*/
const Device UA2435943 = {S55168, 80, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.15)*/
const Device UA2516940 = {S55169, 153, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Sleipnir/2.9.4)*/
const Device UA2676711 = {S55170, 80, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.16)*/
const Device UA2780941 = {S55171, 87, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Sleipnir/2.9.16)*/
const Device UA2799309 = {S55172, 80, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.17)*/
const Device UA2837959 = {S55173, 178, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Sleipnir/2.9.14)*/
const Device UA2959784 = {S55174, 87, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Sleipnir/2.9.17)*/
const Device UA3028278 = {S55175, 80, { &P15364,&P17285,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/2.9.18)*/
const Device UA1996416 = {S55176, 154, { &P14574,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2002681 = {S55177, 154, { &P14574,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2201778 = {S55178, 154, { &P14574,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2367096 = {S55179, 158, { &P14574,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus S Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA72767 = {S55180, 46, { &P15244,&P6476,&P3625,&P18092 }};	/*Useragent - LGE-MN270/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA2507008 = {S55181, 132, { &P21141,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; Lenovo-P770/S100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2597536 = {S55182, 142, { &P21141,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Lenovo P770 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2657774 = {S55183, 133, { &P21141,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Lenovo P770 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2698324 = {S55184, 64, { &P21141,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Lenovo P770 Build/JRO03C)*/
const Device UA2764809 = {S55185, 136, { &P21141,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; vi-vn; Lenovo-P770_ROW/S100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA3087253 = {S55186, 142, { &P21141,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Lenovo P770 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2876038 = {S55187, 103, { &P15364,&P18346,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA1991073 = {S55188, 132, { &P17168,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7310 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2611281 = {S55189, 132, { &P17168,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P7310 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA907381 = {S55190, 153, { &P18706,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_9p017 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2028025 = {S55191, 148, { &P18706,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_jbl_253 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2030289 = {S55192, 154, { &P18706,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_ov5647 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261464 = {S55193, 149, { &P18706,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_5647 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261532 = {S55194, 152, { &P18706,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; e1809c_v77_gq1008_5647 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2338316 = {S55195, 152, { &P18706,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_5640 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2339767 = {S55196, 149, { &P18706,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_jbl_5647 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2353361 = {S55197, 148, { &P18706,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_253 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2466953 = {S55198, 150, { &P18706,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_9p017 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2467735 = {S55199, 153, { &P18706,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_253_6628 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2529577 = {S55200, 153, { &P18706,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_gq1008_9p017 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2532494 = {S55201, 158, { &P18706,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_gq1008_9p017_6628 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2554076 = {S55202, 149, { &P18706,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_5640 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2648340 = {S55203, 158, { &P18706,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_ov5647_uta Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2659602 = {S55204, 157, { &P18706,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_gq1008_5647_6628 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2746324 = {S55205, 145, { &P18706,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_jdt1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2748163 = {S55206, 155, { &P18706,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_9p017_6628 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2239315 = {S55207, 84, { &P15364,&P7133,&P22554,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:16.1) Gecko/X Firefox/16.1 IceDragon/16.1.0.1*/
const Device UA23578 = {S55208, 133, { &P12888,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; M9300 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA555181 = {S55209, 57, { &P12888,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; M9300 Build/FRG83)*/
const Device UA901504 = {S55210, 101, { &P15364,&P7133,&P22343,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.32) Gecko/X Firefox/3.6.32 (Palemoon/3.6.32)*/
const Device UA2355226 = {S55211, 135, { &P21088,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Stealth 9.7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2594277 = {S55212, 134, { &P21088,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; Stealth9.7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2695303 = {S55213, 64, { &P21088,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Stealth 9.7 Build/IMM76D)*/
const Device UA2390398 = {S55214, 155, { &P14669,&P5843,&P24673,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiax2-00) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA62792 = {S55215, 82, { &P12077,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S3600i/S3600iJPIL1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA7029 = {S55216, 83, { &P11700,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW810i/R4EA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53990 = {S55217, 84, { &P11700,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW810iv/R4EA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62401 = {S55218, 83, { &P11700,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW810c/R4EA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA180939 = {S55219, 116, { &P18026,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; ElocityA7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA360255 = {S55220, 123, { &P18026,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; ElocityA7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA902269 = {S55221, 115, { &P13107,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-06/07.48; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA2353944 = {S55222, 114, { &P13107,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-06/07.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6*/
const Device UA2571398 = {S55223, 61, { &P15767,&P17779,&P21178,&P18092 }};	/*Useragent - YelpWebView/1.0 ipad/6_1_2 YelpApp/6.5.0 (x-screen-size 1.0;)*/
const Device UA2671207 = {S55224, 61, { &P15767,&P17779,&P21178,&P18092 }};	/*Useragent - YelpWebView/1.0 ipad/6_1_3 YelpApp/6.5.1 (x-screen-size 1.0;)*/
const Device UA2719291 = {S55225, 61, { &P15767,&P17779,&P21178,&P18092 }};	/*Useragent - YelpWebView/1.0 ipad/6_1_3 YelpApp/6.6.1 (x-screen-size 1.0;)*/
const Device UA2733894 = {S55226, 22, { &P15767,&P17779,&P21178,&P18092 }};	/*Useragent - iPad2,1/6.1.3 (10B329)*/
const Device UA2734425 = {S55227, 43, { &P15767,&P17779,&P21178,&P18092 }};	/*Useragent - server-bag [iPhone OS,6.1.3,10B329,iPad3,1]*/
const Device UA2754150 = {S55228, 61, { &P15767,&P17779,&P21178,&P18092 }};	/*Useragent - YelpWebView/1.0 ipad/6_1_3 YelpApp/6.7.0 (x-screen-size 2.0;)*/
const Device UA2754268 = {S55229, 61, { &P15767,&P17779,&P21178,&P18092 }};	/*Useragent - YelpWebView/1.0 ipad/6_1_3 YelpApp/6.7.0 (x-screen-size 1.0;)*/
const Device UA2859151 = {S55230, 22, { &P15767,&P17779,&P21178,&P18092 }};	/*Useragent - iPad2,5/6.1.3 (10B329)*/
const Device UA2869364 = {S55231, 61, { &P15767,&P17779,&P21178,&P18092 }};	/*Useragent - YelpWebView/1.0 ipad/6_1_3 YelpApp/6.8.0 (x-screen-size 2.0;)*/
const Device UA2872326 = {S55232, 61, { &P15767,&P17779,&P21178,&P18092 }};	/*Useragent - YelpWebView/1.0 ipad/6_1_3 YelpApp/6.8.0 (x-screen-size 1.0;)*/
const Device UA2966721 = {S55233, 61, { &P15767,&P17779,&P21178,&P18092 }};	/*Useragent - YelpWebView/1.0 ipad/6_1_3 YelpApp/6.9.1 (x-screen-size 1.0;)*/
const Device UA2967891 = {S55234, 61, { &P15767,&P17779,&P21178,&P18092 }};	/*Useragent - YelpWebView/1.0 ipad/6_1_3 YelpApp/6.9.1 (x-screen-size 2.0;)*/
const Device UA3028151 = {S55235, 61, { &P15767,&P17779,&P21178,&P18092 }};	/*Useragent - YelpWebView/1.0 ipad/6_1_3 YelpApp/6.9.2 (x-screen-size 1.0;)*/
const Device UA3034374 = {S55236, 61, { &P15767,&P17779,&P21178,&P18092 }};	/*Useragent - YelpWebView/1.0 ipad/6_1_3 YelpApp/7.0.1 (x-screen-size 1.0;)*/
const Device UA3037093 = {S55237, 61, { &P15767,&P17779,&P21178,&P18092 }};	/*Useragent - YelpWebView/1.0 ipad/6_1_3 YelpApp/7.0.1 (x-screen-size 2.0;)*/
const Device UA3080856 = {S55238, 61, { &P15767,&P17779,&P21178,&P18092 }};	/*Useragent - YelpWebView/1.0 ipad/6_1_3 YelpApp/7.1.0 (x-screen-size 2.0;)*/
const Device UA2676329 = {S55239, 138, { &P23771,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676739 = {S55240, 131, { &P23771,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2975709 = {S55241, 134, { &P18295,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; One X Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021486 = {S55242, 138, { &P18295,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; HTC One X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022499 = {S55243, 137, { &P18295,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One X Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA79901 = {S55244, 89, { &P17595,&P17295,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_5) AppleWebKit (KHTML, like Gecko) Chrome/7*/
const Device UA150011 = {S55245, 125, { &P17595,&P17295,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA292084 = {S55246, 125, { &P17595,&P17295,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA385562 = {S55247, 125, { &P17595,&P17295,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA431686 = {S55248, 125, { &P17595,&P17295,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA480464 = {S55249, 125, { &P17595,&P17295,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA483966 = {S55250, 125, { &P17595,&P17295,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_0; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA484279 = {S55251, 125, { &P17595,&P17295,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA2375732 = {S55252, 125, { &P17595,&P17295,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.7 (KHTML	 like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA2355666 = {S55253, 137, { &P21356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; TECNO N3 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2600783 = {S55254, 138, { &P21356,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; TECNO N3S Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2695193 = {S55255, 61, { &P21356,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; TECNO N3 Build/master)*/
const Device UA2488370 = {S55256, 171, { &P13123,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2558399 = {S55257, 171, { &P13123,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2699811 = {S55258, 171, { &P13123,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SCH-i509 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2703859 = {S55259, 172, { &P13123,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5360L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2853487 = {S55260, 172, { &P13123,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5360B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2974746 = {S55261, 175, { &P13123,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5360 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA180408 = {S55262, 138, { &P21214,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; enspert_e201 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018406 = {S55263, 62, { &P21214,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; enspert_e201 Build/FRF91)*/
const Device UA257244 = {S55264, 50, { &P15364,&P17188,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 ( Windows; U; Windows NT4.0; FireFox )*/
const Device UA745032 = {S55265, 59, { &P15364,&P17188,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; WinNT4.0; en-US; rv:0.9.5) Gecko/X*/
const Device UA2362758 = {S55266, 132, { &P18465,&P18110,&P19803,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Safari/535.19 Silk-Accelerated=true*/
const Device UA2567524 = {S55267, 140, { &P18448,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST26i Build/11.2.A.0.21) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2601242 = {S55268, 133, { &P18448,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST26i Build/11.2.A.0.21) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566538 = {S55269, 169, { &P13157,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566977 = {S55270, 163, { &P13157,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2584083 = {S55271, 169, { &P13157,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715a Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2584864 = {S55272, 169, { &P13157,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2975055 = {S55273, 138, { &P18296,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One V Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023852 = {S55274, 138, { &P18296,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One V Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA116004 = {S55275, 135, { &P15365,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; HTC Magic Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA147377 = {S55276, 137, { &P15365,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Magic Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA161905 = {S55277, 138, { &P15365,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Magic Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1874256 = {S55278, 62, { &P15365,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; HTC Magic Build/FRG83D)*/
const Device UA2022244 = {S55279, 125, { &P15365,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; HTC Magic Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA626400 = {S55280, 113, { &P15785,&P5843,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia201/11.21; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.11*/
const Device UA62366 = {S55281, 82, { &P11903,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C3212i/C3212IDDJF2 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62630 = {S55282, 80, { &P11903,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C3212/C3212DDIL1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2228686 = {S55283, 136, { &P23981,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; 9300+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2650419 = {S55284, 58, { &P23981,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; 9300+ Build/IMM76D)*/
const Device UA2788417 = {S55285, 136, { &P23981,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; 9300+ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3070613 = {S55286, 136, { &P23981,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; zh-tw; 9300+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2670620 = {S55287, 136, { &P18442,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L900 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676281 = {S55288, 136, { &P18442,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7100 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676386 = {S55289, 129, { &P18442,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7105 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676654 = {S55290, 142, { &P18442,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Note II Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676670 = {S55291, 136, { &P18442,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7105 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708545 = {S55292, 145, { &P18442,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I317 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2711816 = {S55293, 129, { &P18442,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7100 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2737827 = {S55294, 137, { &P18442,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T889 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA1972610 = {S55295, 60, { &P18648,&P5043,&P18517,&P18092 }};	/*Useragent - Nokia306/2.0 (03.63) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2672473 = {S55296, 137, { &P12972,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2674245 = {S55297, 130, { &P12972,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676679 = {S55298, 145, { &P12972,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T989 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2715722 = {S55299, 138, { &P12972,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989D Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734560 = {S55300, 138, { &P12972,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T989 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2996451 = {S55301, 131, { &P12972,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989D Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2383793 = {S55302, 147, { &P17174,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MediaPad 10 FHD Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2727269 = {S55303, 76, { &P17174,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MediaPad 10 FHD Build/HuaweiMediaPad)*/
const Device UA2676175 = {S55304, 130, { &P15882,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P5110 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676588 = {S55305, 130, { &P15882,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P5110 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2752959 = {S55306, 130, { &P15876,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P5100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754585 = {S55307, 130, { &P15876,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P5100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754801 = {S55308, 130, { &P15876,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T779 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA51105 = {S55309, 118, { &P12351,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-S3653W/S3653WJPJC4 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA2509665 = {S55310, 91, { &P11854,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaE71) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2600330 = {S55311, 92, { &P11854,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAE71x) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2696731 = {S55312, 92, { &P11854,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiae71-1) U2/1.0.0 UCBrowser/8.8.0.227 U2/1.0.0 Mobile*/
const Device UA2748666 = {S55313, 30, { &P11854,&P5890,&P24675,&P18092 }};	/*Useragent - NokiaE71/UCWEB8.8.0.212/28/800*/
const Device UA2888608 = {S55314, 74, { &P17595,&P21721,&P22171,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:23.0) Gecko/X Firefox/23.0*/
const Device UA2973269 = {S55315, 136, { &P21935,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E980 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2539371 = {S55316, 146, { &P18614,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT890 Build/8.7.1I-110_IFW-39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2855746 = {S55317, 139, { &P18614,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT890 Build/8.7.1I-110_IFW-39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22*/
const Device UA3067198 = {S55318, 138, { &P21283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067186 = {S55319, 137, { &P21283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3082891 = {S55320, 138, { &P21283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083205 = {S55321, 138, { &P21283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109307 = {S55322, 138, { &P21283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA6945 = {S55323, 61, { &P11699,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia5200/2.0 (05.53) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA6965 = {S55324, 71, { &P11699,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia5200/2.0 (03.93) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia5200*/
const Device UA2750982 = {S55325, 135, { &P20705,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2944861 = {S55326, 140, { &P20705,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One 801e Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2998876 = {S55327, 137, { &P20705,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC_PN071 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3007127 = {S55328, 128, { &P20705,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3020236 = {S55329, 133, { &P20705,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One 801e Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3058843 = {S55330, 130, { &P20705,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC_PN071 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3108016 = {S55331, 144, { &P20705,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One dual sim Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2892043 = {S55332, 135, { &P23187,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; V370 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2959235 = {S55333, 57, { &P23187,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; V370 Build/JOP40D)*/
const Device UA3030939 = {S55334, 134, { &P23187,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; nl-nl; V370 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3059265 = {S55335, 56, { &P23187,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; V370 Build/JDQ39)*/
const Device UA3065622 = {S55336, 126, { &P18282,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A200 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA783756 = {S55337, 135, { &P17976,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID_Serials Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1956824 = {S55338, 142, { &P17976,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID_Serials Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026614 = {S55339, 133, { &P17976,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Tablet PC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2027670 = {S55340, 130, { &P17976,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; tablet Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2223191 = {S55341, 133, { &P17976,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Tablet PC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2480662 = {S55342, 127, { &P17976,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; M71 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2507799 = {S55343, 148, { &P17976,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 1688-CJ-C30Q8-31S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556016 = {S55344, 137, { &P17976,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Tablet Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2588372 = {S55345, 59, { &P17976,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; MID970 Build/IML74K)*/
const Device UA2779897 = {S55346, 134, { &P17976,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Tablet 7.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2853262 = {S55347, 139, { &P17976,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MID900SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3046244 = {S55348, 141, { &P17976,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Tablet 7'' Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049063 = {S55349, 142, { &P17976,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Android Tablet 7'' Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2242291 = {S55350, 73, { &P15575,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; GT-I9100G; 480*800) UCWEB7.9.3.103/139/32092*/
const Device UA2303321 = {S55351, 70, { &P15575,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; GT-I9108; 480*800) UCWEB7.9.3.103/139/800*/
const Device UA2403507 = {S55352, 69, { &P15575,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; GT-I9100; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2423017 = {S55353, 69, { &P15575,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; GT-I9100; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2466978 = {S55354, 70, { &P15575,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; GT-I9100G; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2845661 = {S55355, 70, { &P15575,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; GT-I9100T; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2538616 = {S55356, 135, { &P12280,&P18117,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2562157 = {S55357, 133, { &P12280,&P18117,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2843427 = {S55358, 135, { &P12280,&P18117,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2952969 = {S55359, 135, { &P12280,&P18117,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA2974035 = {S55360, 128, { &P19783,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ME172V Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3065092 = {S55361, 136, { &P20146,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9082 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2387505 = {S55362, 135, { &P14997,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S100 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654175 = {S55363, 57, { &P14997,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; S100 Build/IMM76I)*/
const Device UA1814 = {S55364, 74, { &P14763,&P5776,&P3756,&P18092 }};	/*Useragent - NokiaN72/2.0617.1.0.3 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53477 = {S55365, 150, { &P14763,&P5776,&P3756,&P18092 }};	/*Useragent - NokiaN72/2.0617.1.0.3 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1, NokiaN72/2.0617.1.0.3 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA69523 = {S55366, 35, { &P14763,&P5776,&P3756,&P18092 }};	/*Useragent - NokiaN72/SymbianOS/8.0 Series60/2.6*/
const Device UA16705 = {S55367, 135, { &P13806,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; ADR6225 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2234356 = {S55368, 148, { &P20182,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-uk; Xolo_X900 Build/AZ510_ICS_02.19I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2365654 = {S55369, 128, { &P20182,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422230 = {S55370, 148, { &P20182,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-uk; Xolo_X900 Build/AZ510_ICS_02.24I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2738298 = {S55371, 148, { &P20182,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-uk; Xolo_X900 Build/AZ510_ICS_02.31I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2872447 = {S55372, 148, { &P20182,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-uk; Xolo_X900 Build/AZ510_ICS_02.36I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA64421 = {S55373, 73, { &P14663,&P5208,&P3499,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Teleca Q7; Brew 3.1.5; U; en) 320X240 LGE VX9200*/
const Device UA2754798 = {S55374, 133, { &P19904,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C5170 Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755738 = {S55375, 134, { &P19904,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Hydro Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754236 = {S55376, 146, { &P17989,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V8000_USA_Cricket Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774077 = {S55377, 146, { &P17989,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N8000_USA_Cricket Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783291 = {S55378, 151, { &P17595,&P17296,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.29 Safari/537.36 OPR/15.0.1147.24 (Edition Next)*/
const Device UA3027547 = {S55379, 137, { &P17595,&P17296,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153*/
const Device UA3058777 = {S55380, 137, { &P17595,&P17296,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153*/
const Device UA2755136 = {S55381, 140, { &P18223,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868910 = {S55382, 139, { &P18223,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LT22i Build/6.2.A.0.400) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937879 = {S55383, 137, { &P18223,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xperia P Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA20202 = {S55384, 59, { &P15364,&P5214,&P3487,&P18092 }};	/*Useragent - KCI-S2100-VMUB/ACP1000 UP.Browser/7.2.7.2.564 (GUI) MMP/2.0*/
const Device UA49277 = {S55385, 54, { &P15364,&P5214,&P3487,&P18092 }};	/*Useragent - KCI-S2100/ACM1000 UP.Browser/7.2.7.2.565 (GUI) MMP/2.0*/
const Device UA228246 = {S55386, 54, { &P15364,&P5214,&P3487,&P18092 }};	/*Useragent - KCI-S2100/ACQ1007 UP.Browser/7.2.7.2.565 (GUI) MMP/2.0*/
const Device UA786783 = {S55387, 54, { &P15364,&P5214,&P3487,&P18092 }};	/*Useragent - KCI-S2100/ACM1002 UP.Browser/7.2.7.2.565 (GUI) MMP/2.0*/
const Device UA362900 = {S55388, 152, { &P13251,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Dell Streak Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/streak.407.0*/
const Device UA369590 = {S55389, 127, { &P13251,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Dell Streak Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA705684 = {S55390, 63, { &P13251,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; Dell Streak Build/GRI40)*/
const Device UA1884147 = {S55391, 145, { &P13251,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Dell Streak Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1961620 = {S55392, 145, { &P13251,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Dell Streak Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2508085 = {S55393, 129, { &P20139,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-GT-A7100/1.0 Linux/2.6.35.7 Android/4.0.4 Release/01.14.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA563169 = {S55394, 90, { &P17595,&P17297,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.1.17) Gecko/X Firefox/3.5.17*/
const Device UA2784727 = {S55395, 141, { &P12280,&P19826,&P22879,&P18092 }};	/*Useragent - MQQBrowser/43 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3*/
const Device UA2784744 = {S55396, 142, { &P12280,&P19826,&P22879,&P18092 }};	/*Useragent - MQQBrowser/43 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3*/
const Device UA2797500 = {S55397, 142, { &P12280,&P19826,&P22879,&P18092 }};	/*Useragent - MQQBrowser/43 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3*/
const Device UA2797639 = {S55398, 139, { &P12280,&P19826,&P22879,&P18092 }};	/*Useragent - MQQBrowser/43 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Safari/7534.48.3*/
const Device UA2868843 = {S55399, 139, { &P12280,&P19826,&P22879,&P18092 }};	/*Useragent - MQQBrowser/43 Mozilla/5.0 (iPhone 4; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Safari/7534.48.3*/
const Device UA2876108 = {S55400, 140, { &P12280,&P19826,&P22879,&P18092 }};	/*Useragent - MQQBrowser/43 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 Safari/7534.48.3*/
const Device UA2958220 = {S55401, 142, { &P12280,&P19826,&P22879,&P18092 }};	/*Useragent - MQQBrowser/43 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145 Safari/7534.48.3*/
const Device UA3065250 = {S55402, 137, { &P21628,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080313 = {S55403, 138, { &P21628,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3096082 = {S55404, 114, { &P15228,&P5043,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-03/07.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA924885 = {S55405, 138, { &P21086,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CLP281A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257248 = {S55406, 131, { &P21086,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CLP281X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2384847 = {S55407, 132, { &P23952,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GOCLEVER Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2654598 = {S55408, 61, { &P23952,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GOCLEVER Build/JRO03H)*/
const Device UA2669934 = {S55409, 104, { &P15364,&P7133,&P21712,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.9.0 Safari/534.34*/
const Device UA2955890 = {S55410, 119, { &P15364,&P7133,&P21712,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) CasperJS/1.0.2+Phantomjs/1.9.0 Safari/534.34*/
const Device UA2960782 = {S55411, 97, { &P15364,&P7133,&P21712,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.9.1 Safari/534.34*/
const Device UA3065610 = {S55412, 138, { &P19845,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE N9120 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA1956262 = {S55413, 78, { &P15364,&P7127,&P22732,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Avant Browser)*/
const Device UA2154763 = {S55414, 78, { &P15364,&P7127,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Avant Browser)*/
const Device UA2687063 = {S55415, 83, { &P15364,&P7127,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; SV1; Avant Browser)*/
const Device UA2722636 = {S55416, 83, { &P15364,&P7127,&P22732,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; SV1; Avant Browser)*/
const Device UA2822269 = {S55417, 93, { &P15364,&P7127,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Avant Browser; Avant Browser)*/
const Device UA2950004 = {S55418, 93, { &P15364,&P7127,&P22732,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 5.0; Avant Browser; Avant Browser)*/
const Device UA2667535 = {S55419, 141, { &P12280,&P19826,&P22589,&P18092 }};	/*Useragent - MQQBrowser/42 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3*/
const Device UA2676529 = {S55420, 142, { &P12280,&P19826,&P22589,&P18092 }};	/*Useragent - MQQBrowser/42 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3*/
const Device UA2703912 = {S55421, 139, { &P12280,&P19826,&P22589,&P18092 }};	/*Useragent - MQQBrowser/42 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Safari/7534.48.3*/
const Device UA2708700 = {S55422, 140, { &P12280,&P19826,&P22589,&P18092 }};	/*Useragent - MQQBrowser/42 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 Safari/7534.48.3*/
const Device UA2710039 = {S55423, 142, { &P12280,&P19826,&P22589,&P18092 }};	/*Useragent - MQQBrowser/421 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3*/
const Device UA2710361 = {S55424, 142, { &P12280,&P19826,&P22589,&P18092 }};	/*Useragent - MQQBrowser/42 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145 Safari/7534.48.3*/
const Device UA2713477 = {S55425, 143, { &P12280,&P19826,&P22589,&P18092 }};	/*Useragent - MQQBrowser/421 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3*/
const Device UA2733665 = {S55426, 139, { &P12280,&P19826,&P22589,&P18092 }};	/*Useragent - MQQBrowser/42 Mozilla/5.0 (iPhone 4; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Safari/7534.48.3*/
const Device UA2737629 = {S55427, 143, { &P12280,&P19826,&P22589,&P18092 }};	/*Useragent - MQQBrowser/421 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3*/
const Device UA2737794 = {S55428, 141, { &P12280,&P19826,&P22589,&P18092 }};	/*Useragent - MQQBrowser/421 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 Safari/7534.48.3*/
const Device UA2752489 = {S55429, 140, { &P12280,&P19826,&P22589,&P18092 }};	/*Useragent - MQQBrowser/421 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Safari/7534.48.3*/
const Device UA2752596 = {S55430, 140, { &P12280,&P19826,&P22589,&P18092 }};	/*Useragent - MQQBrowser/421 Mozilla/5.0 (iPhone 4; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Safari/7534.48.3*/
const Device UA2768854 = {S55431, 143, { &P12280,&P19826,&P22589,&P18092 }};	/*Useragent - MQQBrowser/421 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145 Safari/7534.48.3*/
const Device UA2995925 = {S55432, 141, { &P12280,&P19826,&P22589,&P18092 }};	/*Useragent - MQQBrowser/42 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3*/
const Device UA741739 = {S55433, 96, { &P15364,&P22191,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; fr; rv:1.8.1.12) Gecko/X Fedora/2.0.0.12-1.fc8 Firefox/2.0.0.12*/
const Device UA18094 = {S55434, 145, { &P14014,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Acer Liquid Metal Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18103 = {S55435, 141, { &P14014,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Liquid Metal Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41570 = {S55436, 146, { &P14014,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Acer Liquid Metal Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA54353 = {S55437, 185, { &P14014,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; it-it; Liquid Metal Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Acer_S120/Acer_LiquidMetal_1.000.14_EMEA_VFIT*/
const Device UA368678 = {S55438, 69, { &P14014,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; Acer Liquid Metal Build/FRG83)*/
const Device UA1906818 = {S55439, 140, { &P14014,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; Liquid Metal Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018573 = {S55440, 63, { &P14014,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; Liquid Metal Build/MASTER)*/
const Device UA3031353 = {S55441, 203, { &P12280,&P18432,&P18431,&P18089 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A5376e Safari/8536.25 (compatible; Googlebot-Mobile/2.1; +http://www.google.com/bot.html)*/
const Device UA3114368 = {S55442, 176, { &P12280,&P18432,&P18431,&P18089 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/537.4 (KHTML, like Gecko; Google Page Speed Insights) Version/6.0 Mobile/10A525 Safari/8536.25 GoogleBot/2.1*/
const Device UA2008064 = {S55443, 163, { &P19891,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I717 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2199795 = {S55444, 163, { &P19891,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I717 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2237682 = {S55445, 167, { &P19891,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I717 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566952 = {S55446, 140, { &P15989,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21i Build/11.0.A.4.22) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583249 = {S55447, 140, { &P15989,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21a Build/11.0.A.0.16) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2584860 = {S55448, 140, { &P15989,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21i2 Build/11.0.A.6.5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2702620 = {S55449, 140, { &P15989,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21a2 Build/11.0.A.1.8) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA3083429 = {S55450, 114, { &P18267,&P5043,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-02/11.79; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA2024301 = {S55451, 128, { &P12973,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2400970 = {S55452, 159, { &P12973,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734128 = {S55453, 142, { &P12973,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Celkon_A101 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2832157 = {S55454, 57, { &P12973,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; A101 Build/IML74K)*/
const Device UA2489373 = {S55455, 154, { &P18465,&P18066,&P19837,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA8315 = {S55456, 135, { &P13485,&P5027,&P3412,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320) BlackBerry8330/4.3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/104*/
const Device UA8320 = {S55457, 73, { &P13485,&P5027,&P3412,&P18092 }};	/*Useragent - BlackBerry8330/4.3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105*/
const Device UA34546 = {S55458, 135, { &P13485,&P5027,&P3412,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320) BlackBerry8330/4.3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/126*/
const Device UA2567020 = {S55459, 147, { &P15880,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583199 = {S55460, 147, { &P15880,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ME865 Build/6.7.2_GC-220-EDS-3) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2953568 = {S55461, 140, { &P15880,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2755331 = {S55462, 139, { &P13054,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR6425LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867014 = {S55463, 139, { &P13054,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6425LVW Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA63310 = {S55464, 118, { &P11656,&P17483,&P4848,&P18092 }};	/*Useragent - LG/KU990-Orange/v10ci Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63930 = {S55465, 110, { &P11656,&P17483,&P4848,&P18092 }};	/*Useragent - LG/KU990/v10a Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64221 = {S55466, 110, { &P11656,&P17483,&P4848,&P18092 }};	/*Useragent - LG-KU990/v10a Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA86263 = {S55467, 123, { &P11656,&P17483,&P4848,&P18092 }};	/*Useragent - Vodafone/1.0/LG-KU990/V10a Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88092 = {S55468, 123, { &P11656,&P17483,&P4848,&P18092 }};	/*Useragent - Vodafone/1.0/LG-KU990/V10i Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88233 = {S55469, 123, { &P11656,&P17483,&P4848,&P18092 }};	/*Useragent - Vodafone/1.0/LG-KU990/V10h Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88629 = {S55470, 123, { &P11656,&P17483,&P4848,&P18092 }};	/*Useragent - Vodafone/1.0/LG-KU990/V10c Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA101596 = {S55471, 123, { &P11656,&P17483,&P4848,&P18092 }};	/*Useragent - Vodafone/1.0/LG-KU990/v10b Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA123887 = {S55472, 123, { &P11656,&P17483,&P4848,&P18092 }};	/*Useragent - Vodafone/1.0/LG-KU990/V10j Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA124872 = {S55473, 123, { &P11656,&P17483,&P4848,&P18092 }};	/*Useragent - Vodafone/1.0/LG-KU990/V10f Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA287581 = {S55474, 123, { &P11656,&P17483,&P4848,&P18092 }};	/*Useragent - Vodafone/1.0/LG-KU990/V10g Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2155400 = {S55475, 78, { &P15364,&P7127,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Avant Browser)*/
const Device UA2163576 = {S55476, 78, { &P15364,&P7127,&P22731,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Avant Browser)*/
const Device UA2823203 = {S55477, 83, { &P15364,&P7127,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Avant Browser; SV1)*/
const Device UA2839262 = {S55478, 83, { &P15364,&P7127,&P22731,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Avant Browser; SV1)*/
const Device UA2841811 = {S55479, 93, { &P15364,&P7127,&P22731,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Avant Browser; Avant Browser)*/
const Device UA2950089 = {S55480, 93, { &P15364,&P7127,&P22731,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Trident 4.0; Windows NT 5.0; Avant Browser; Avant Browser)*/
const Device UA2016141 = {S55481, 155, { &P12496,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I897 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029567 = {S55482, 169, { &P12496,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I897 Build/JRO03H; CyanogenMod-helly_bean_cappy) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2200621 = {S55483, 155, { &P12496,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I897 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2361546 = {S55484, 159, { &P12496,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I897 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA607431 = {S55485, 128, { &P12280,&P18117,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293*/
const Device UA625432 = {S55486, 145, { &P12280,&P18117,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA667364 = {S55487, 143, { &P12280,&P18117,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA671337 = {S55488, 130, { &P12280,&P18117,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293*/
const Device UA686477 = {S55489, 141, { &P12280,&P18117,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA799137 = {S55490, 147, { &P12280,&P18117,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA1986356 = {S55491, 135, { &P12280,&P18117,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/6531.22.7 iLunascape/4*/
const Device UA2163202 = {S55492, 147, { &P12280,&P18117,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/9B206 Safari/6531.22.7*/
const Device UA2260854 = {S55493, 140, { &P12280,&P18117,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/6531.22.7 iLunascape/3165*/
const Device UA2328639 = {S55494, 135, { &P12280,&P18117,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/6531.22.7 iLunascape/4*/
const Device UA2573879 = {S55495, 145, { &P12280,&P18117,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; vi-vn) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/9B176 Safari/6531.22.7*/
const Device UA2042094 = {S55496, 60, { &P15364,&P7127,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:15.0) Gecko/X Firefox/15.0.1*/
const Device UA2229872 = {S55497, 141, { &P20088,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; U8666-1 Build/HuaweiU8666) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2594239 = {S55498, 141, { &P20088,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-by; MTC Fit Build/HuaweiU8666) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2688681 = {S55499, 145, { &P20088,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-mx; U8666-51 Build/HuaweiU8666-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2943476 = {S55500, 142, { &P20088,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; U8666-51 Build/HuaweiU8666) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA8883 = {S55501, 46, { &P13999,&P5448,&P4626,&P18092 }};	/*Useragent - Opera/9.7 (Windows NT 5.1; HTC_HD2/1.0; U; en)*/
const Device UA19018 = {S55502, 46, { &P13999,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_HD_T8585 Opera/9.7 (Windows NT 5.1; U; en)*/
const Device UA31298 = {S55503, 67, { &P13999,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_HD2_T9193 Opera/9.7 (Windows NT 5.1; U; en) V1.49.841.1 (71528)*/
const Device UA31299 = {S55504, 67, { &P13999,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_HD2_T9193 Opera/9.7 (Windows NT 5.1; U; en) V1.72.841.0 (82124)*/
const Device UA35409 = {S55505, 88, { &P13999,&P5448,&P4626,&P18092 }};	/*Useragent - PPC; 480x800; HTC_HD2_T8585-Orange; OpVer 114.143.2.731 Opera/9.7 (Windows NT 5.1; U;fr)*/
const Device UA54763 = {S55506, 54, { &P13999,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_HD2_T8585 Opera/9.70 (Windows NT 5.1; U; en)*/
const Device UA89141 = {S55507, 76, { &P13999,&P5448,&P4626,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_HD2/1.49.164.2 (71528) Opera/9.7 (Windows NT 5.1; U; es-ES)*/
const Device UA100984 = {S55508, 73, { &P13999,&P5448,&P4626,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_HD2/1.72.162.0 (82124) Opera/9.7 (Windows NT 5.1; U; de)*/
const Device UA116898 = {S55509, 73, { &P13999,&P5448,&P4626,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_HD2/1.72.172.0 (82124) Opera/9.7 (Windows NT 5.1; U; nl)*/
const Device UA128468 = {S55510, 73, { &P13999,&P5448,&P4626,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_HD2/1.72.161.3 (82124) Opera/9.7 (Windows NT 5.1; U; en)*/
const Device UA135029 = {S55511, 73, { &P13999,&P5448,&P4626,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_HD2/1.44.163.3 (70494) Opera/9.7 (Windows NT 5.1; U; fr)*/
const Device UA171711 = {S55512, 73, { &P13999,&P5448,&P4626,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_HD2/3.14.162.5 (04666) Opera/9.7 (Windows NT 5.1; U; de)*/
const Device UA171963 = {S55513, 73, { &P13999,&P5448,&P4626,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_HD2/3.14.161.3 (04666) Opera/9.7 (Windows NT 5.1; U; en)*/
const Device UA376983 = {S55514, 73, { &P13999,&P5448,&P4626,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_HD2/3.14.163.3 (04666) Opera/9.7 (Windows NT 5.1; U; fr)*/
const Device UA433048 = {S55515, 73, { &P13999,&P5448,&P4626,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_HD2/3.14.172.3 (04666) Opera/9.7 (Windows NT 5.1; U; nl)*/
const Device UA443071 = {S55516, 76, { &P13999,&P5448,&P4626,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_HD2/1.72.164.4 (82124) Opera/9.7 (Windows NT 5.1; U; es-ES)*/
const Device UA720494 = {S55517, 73, { &P13999,&P5448,&P4626,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_HD2/1.44.172.7 (70494) Opera/9.7 (Windows NT 5.1; U; nl)*/
const Device UA809289 = {S55518, 73, { &P13999,&P5448,&P4626,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_HD2/1.44.162.6 (70494) Opera/9.7 (Windows NT 5.1; U; de)*/
const Device UA1861002 = {S55519, 76, { &P13999,&P5448,&P4626,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_HD2/3.14.164.4 (04666) Opera/9.7 (Windows NT 5.1; U; es-ES)*/
const Device UA3065390 = {S55520, 137, { &P21350,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8730 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083194 = {S55521, 138, { &P21350,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8730T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA30 = {S55522, 133, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 480x800; HTC_HD2_T8585-Orange; OpVer 114.143.2.731*/
const Device UA73 = {S55523, 133, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 480x800; HTC_HD2_T8585-Orange; OpVer 156.314.2.732*/
const Device UA31 = {S55524, 133, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 480x800; HTC_HD2_T8585-Orange; OpVer 114.172.1.612*/
const Device UA3935 = {S55525, 134, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; HTC_HD2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA5001 = {S55526, 85, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Leo Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA5848 = {S55527, 117, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/1.49.164.2 (71528)*/
const Device UA5854 = {S55528, 117, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/3.14.161.3 (04666)*/
const Device UA5856 = {S55529, 117, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/3.14.163.3 (04666)*/
const Device UA7203 = {S55530, 133, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 480x800; HTC_HD2_T8585-Orange; OpVer 114.172.2.733*/
const Device UA10049 = {S55531, 87, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - HD2_T8585 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA16208 = {S55532, 91, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_HD2_T8585/480x800 4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA16217 = {S55533, 99, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) HTC_HD2_T8585/480x800*/
const Device UA30452 = {S55534, 117, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/1.72.172.0 (82124)*/
const Device UA30453 = {S55535, 117, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/1.44.163.3 (70494)*/
const Device UA30457 = {S55536, 117, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/1.44.172.7 (70494)*/
const Device UA30463 = {S55537, 117, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/1.72.161.3 (82124)*/
const Device UA30466 = {S55538, 117, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/1.72.162.0 (82124)*/
const Device UA30474 = {S55539, 117, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/1.72.164.4 (82124)*/
const Device UA30476 = {S55540, 117, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/3.14.172.3 (04666)*/
const Device UA31300 = {S55541, 111, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_HD2_T9193 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) V1.49.841.1 (71528)*/
const Device UA31301 = {S55542, 111, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_HD2_T9193 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) V1.72.841.0 (82124)*/
const Device UA35406 = {S55543, 85, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_HD2_Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA35413 = {S55544, 91, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_HD2_T8585 Mozilla\4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA38636 = {S55545, 91, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_HD2_T9193 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA46761 = {S55546, 117, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/3.14.162.5 (04666)*/
const Device UA46762 = {S55547, 117, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_HD2/3.14.164.4 (04666)*/
const Device UA51410 = {S55548, 133, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 480x800; HTC_HD2_T8585-Orange; OpVer 156.314.1.612*/
const Device UA57109 = {S55549, 150, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_HD2_T8585/1.64.621.6 (77873) Mozilla/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA61917 = {S55550, 91, { &P13999,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_HD2_T8585 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA456218 = {S55551, 102, { &P18254,&P18110,&P19840,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.5.0 Safari/534.34*/
const Device UA602157 = {S55552, 104, { &P18254,&P18110,&P19840,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.5.0 Safari/534.34*/
const Device UA850168 = {S55553, 114, { &P18254,&P18110,&P19840,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.5.0 (development) Safari/534.34*/
const Device UA108996 = {S55554, 140, { &P18061,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; sdk Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA174720 = {S55555, 152, { &P18061,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-us; rk2818sdk Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA268575 = {S55556, 131, { &P18061,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; sdk Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA275589 = {S55557, 138, { &P18061,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; google_sdk Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA445559 = {S55558, 62, { &P18061,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; sdk Build/ECLAIR)*/
const Device UA2018146 = {S55559, 60, { &P18061,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1; google_sdk Build/ERD79)*/
const Device UA608875 = {S55560, 160, { &P15759,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Vision Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899816 = {S55561, 170, { &P15759,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Vision Build/IMM76I; CyanogenMod-Audacity Beta 4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA906877 = {S55562, 162, { &P15759,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Vision Build/IMM76L; CyanogenMod-mimicry) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1972661 = {S55563, 168, { &P15759,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-us; HTC Vision Build/IMM76I; CyanogenMod-QuadraSense 1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2885322 = {S55564, 152, { &P17595,&P17297,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100 (Edition Next)*/
const Device UA3027769 = {S55565, 137, { &P17595,&P17297,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153*/
const Device UA3058740 = {S55566, 137, { &P17595,&P17297,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153*/
const Device UA24684 = {S55567, 116, { &P15736,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2232/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA561470 = {S55568, 75, { &P15736,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2232/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA635923 = {S55569, 154, { &P17937,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Triumph Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2475135 = {S55570, 157, { &P17937,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Triumph Build/GRJ22; CM7 Illuminatus v0.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2236136 = {S55571, 120, { &P12449,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE5-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/355/UCWEB Mobile*/
const Device UA2479942 = {S55572, 129, { &P12449,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiae5-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/84/352/UCWEB Mobile  3gpp-gba*/
const Device UA2503287 = {S55573, 122, { &P12449,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE5-00.2) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile*/
const Device UA4155 = {S55574, 38, { &P12274,&P17484,&P3430,&P18092 }};	/*Useragent - SAMSUNG-SGH-i900Orange/BVIB2 Opera 9.5*/
const Device UA7916 = {S55575, 33, { &P12274,&P17484,&P3430,&P18092 }};	/*Useragent - SAMSUNG-SGH-i900V/JBHH1 Opera 9.5*/
const Device UA63854 = {S55576, 30, { &P12274,&P17484,&P3430,&P18092 }};	/*Useragent - SAMSUNG-SGH-i900/1.0 Opera 9.5*/
const Device UA79284 = {S55577, 104, { &P12274,&P17484,&P3430,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG-SGH-i900/1.0 Opera 9.5; U; (null); rv:1.8.1) Gecko/20061208 Firefox/2.0.0 Opera 9.5*/
const Device UA899336 = {S55578, 57, { &P15364,&P17017,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (masking-agent; rv:13.0) Gecko/X Firefox/13.0*/
const Device UA903205 = {S55579, 59, { &P15364,&P17017,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (masking-agent; rv:13.0) Gecko/X Firefox/13.0.1*/
const Device UA905860 = {S55580, 52, { &P15364,&P17017,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (SaferSurf rv:13.0) Gecko/X Firefox/13.0*/
const Device UA903892 = {S55581, 115, { &P14540,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-01/08.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA1975208 = {S55582, 114, { &P14540,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-01/07.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6*/
const Device UA1991057 = {S55583, 117, { &P14540,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-01.1/07.22; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.62.10*/
const Device UA24254 = {S55584, 105, { &P13308,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonJ105i/R1HA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.5*/
const Device UA62919 = {S55585, 105, { &P13308,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonJ105a/R1HA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.5*/
const Device UA2975803 = {S55586, 138, { &P12968,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26i Build/6.1.A.2.55) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA899335 = {S55587, 115, { &P15364,&P5690,&P22458,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5*/
const Device UA1952214 = {S55588, 115, { &P15364,&P5690,&P22458,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5*/
const Device UA1957810 = {S55589, 122, { &P15364,&P5690,&P22458,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5*/
const Device UA2705175 = {S55590, 156, { &P22856,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; HUAWEI Y300-0151 Build/HuaweiY300-0151) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878881 = {S55591, 78, { &P22856,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HUAWEI Y300-0151 Build/HuaweiY300-0151)*/
const Device UA175587 = {S55592, 95, { &P17669,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; T7575)*/
const Device UA188749 = {S55593, 136, { &P17999,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1-rc8; en-us; T301 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA193145 = {S55594, 132, { &P17999,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T301 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA193542 = {S55595, 136, { &P17999,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1-rc7; en-us; T301 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1927225 = {S55596, 56, { &P17999,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; T301 Build/FRG83)*/
const Device UA2566708 = {S55597, 131, { &P18437,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8000 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583371 = {S55598, 131, { &P18437,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8005 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2975004 = {S55599, 140, { &P18223,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022596 = {S55600, 139, { &P18223,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LT22i Build/6.2.A.0.400) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2537568 = {S55601, 134, { &P21603,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; AMID-9743G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2702101 = {S55602, 63, { &P21603,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; AMID-9743G Build/JZO54K)*/
const Device UA2491512 = {S55603, 58, { &P15364,&P18110,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686) Presto/2.12.388 Version/12.13*/
const Device UA2850670 = {S55604, 71, { &P15364,&P18110,&P22540,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:2.0) Gecko/X Firefox/4.0 Opera 12.13*/
const Device UA2191295 = {S55605, 178, { &P15803,&P5843,&P24671,&P18092 }};	/*Useragent - Nokia200/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia200) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA630023 = {S55606, 130, { &P19877,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; SGH-T869 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2024917 = {S55607, 59, { &P19877,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; SGH-T869 Build/HTJ85B)*/
const Device UA2470186 = {S55608, 32, { &P14033,&P5129,&P24675,&P18092 }};	/*Useragent - NokiaC5-03/UCWEB8.8.0.212/50/999*/
const Device UA2509843 = {S55609, 93, { &P14033,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaC5-03) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA43586 = {S55610, 148, { &P12371,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; SO-01B Build/2.0.B.0.138) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA43593 = {S55611, 149, { &P12371,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; SO-01B Build/2.0.1.B.0.19) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA627045 = {S55612, 149, { &P12371,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; SO-01B Build/2.0.2.B.0.34) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA667243 = {S55613, 149, { &P12371,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; SO-01B Build/2.0.2.B.0.29) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2020445 = {S55614, 159, { &P12371,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; ja_jp; SO-01B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.3 Mobile (Dragonfruit)*/
const Device UA2261943 = {S55615, 161, { &P12371,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; SonyEricssonSO-01B Build/2.0.2.B.0.37) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2385729 = {S55616, 149, { &P12371,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; SO-01B Build/2.0.2.B.0.37) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA3070543 = {S55617, 71, { &P12371,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SO-01B Build/2.0.2.B.0.37)*/
const Device UA623432 = {S55618, 136, { &P18630,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; IM-A800S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA731527 = {S55619, 134, { &P18630,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; IM-A800S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1901859 = {S55620, 60, { &P18630,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; IM-A800S Build/GRJ90)*/
const Device UA2552347 = {S55621, 153, { &P18630,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; IM-A800S Build/V5.4 (TV) by atulaphan) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3086998 = {S55622, 155, { &P18630,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; IM-A800S Build/VEGAVIET.COM ROM GB V2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2487643 = {S55623, 117, { &P15364,&P16110,&P22464,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17*/
const Device UA2487936 = {S55624, 124, { &P15364,&P16110,&P22464,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17*/
const Device UA2937090 = {S55625, 165, { &P24097,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SGH-M919N Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2355582 = {S55626, 145, { &P20841,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 10 Build/1.1.5 X-15:30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2367494 = {S55627, 145, { &P20841,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 10 Build/1.1.7 X-14:18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2403756 = {S55628, 145, { &P20841,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 10 Build/1.1.4 X-13:45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2426072 = {S55629, 145, { &P20841,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 10 Build/1.1.8 X-15:22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2427663 = {S55630, 150, { &P20841,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 10 Build/1.1.8-1015 X-16:30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2462073 = {S55631, 151, { &P20841,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 10 Build/1.1.10-1015 X-18:00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2470419 = {S55632, 151, { &P20841,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 10 Build/1.1.11-1015 X-10:30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2486491 = {S55633, 153, { &P20841,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 10 3G Build/1.1.9-1015 X-15:48) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2506876 = {S55634, 154, { &P20841,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 10 3G Build/1.1.10-1015 X-18:00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2530673 = {S55635, 154, { &P20841,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 10 3G Build/1.1.11-1015 X-16:17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2756013 = {S55636, 137, { &P18301,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS840 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784206 = {S55637, 136, { &P18301,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS840PP Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA69468 = {S55638, 125, { &P12973,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; A101 Build/HTJ85) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA77905 = {S55639, 128, { &P12973,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; A101 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2032923 = {S55640, 57, { &P12973,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; A101 Build/HTK55D)*/
const Device UA791152 = {S55641, 141, { &P16467,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; PantechP8000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027278 = {S55642, 65, { &P16467,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; PantechP8000 Build/GRK39F)*/
const Device UA2440001 = {S55643, 140, { &P16467,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PantechP8000 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2487700 = {S55644, 83, { &P15364,&P16110,&P22368,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:15.4) Gecko/X Firefox/15.4.1 PaleMoon/15.4.1*/
const Device UA2488271 = {S55645, 96, { &P15364,&P16110,&P22368,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:15.4) Gecko/X Firefox/15.4.1-x64 PaleMoon/15.4.1-x64*/
const Device UA2674210 = {S55646, 152, { &P17968,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbla668_fwvga Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676082 = {S55647, 135, { &P17968,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; cm_tenderloin Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676668 = {S55648, 136, { &P17968,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; skz_tenderloin Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676737 = {S55649, 121, { &P17968,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2677005 = {S55650, 143, { &P17968,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbl1 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2706647 = {S55651, 127, { &P17968,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; V1013 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2706975 = {S55652, 156, { &P17968,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Motorola Galaxy s3 Build/9.8.2O-72_VZW-8) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707486 = {S55653, 158, { &P17968,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1911_v77_gq2000_a41_6628_v20 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713946 = {S55654, 154, { &P17968,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1909l_v77_gq1008_a41_v20 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734747 = {S55655, 154, { &P17968,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1908_v77_jbl1_9p017_6628 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734896 = {S55656, 128, { &P17968,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735785 = {S55657, 158, { &P17968,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jbla668_9p017_fwvga Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2736087 = {S55658, 147, { &P17968,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_gq2002tc Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2736643 = {S55659, 146, { &P17968,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; e2001_v89_gq2008s Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2751767 = {S55660, 152, { &P17968,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1920_v77_zlh_9p017_asx Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2793857 = {S55661, 136, { &P17968,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.5; iPhone5 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2860997 = {S55662, 143, { &P17968,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1921_v77_cxq1 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2865643 = {S55663, 148, { &P17968,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1920_v77_zlh_fwvga Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2993358 = {S55664, 155, { &P17968,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; e1901_v77_jdt1_9p017_fwvga Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA622921 = {S55665, 67, { &P15364,&P16110,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:8.0.1) Gecko/X Firefox/8.0.1*/
const Device UA2459257 = {S55666, 135, { &P20868,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; M040 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2484919 = {S55667, 126, { &P20868,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; M040 Build/JRO03H) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2598096 = {S55668, 57, { &P20868,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; M040 Build/JRO03H)*/
const Device UA2719601 = {S55669, 251, { &P20868,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; M040 Build/JRO03H) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.4 Version/4.0 Mobile Safari/530.17 baidubrowser/023_1.8.4.2_diordna_0821_008/uzieM_61_1.1.4_040M/Xb/35BC577D8B1D44338D9CE3C5EE1466B2%7CX/1*/
const Device UA2557083 = {S55670, 136, { &P22126,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Torque Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1933486 = {S55671, 155, { &P17320,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ALCATEL_one_touch_990 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1963744 = {S55672, 155, { &P17320,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; ALCATEL ONE TOUCH 990 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1965526 = {S55673, 156, { &P17320,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-cl; ALCATEL_one_touch_990A Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2411960 = {S55674, 155, { &P17320,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; ALCATEL one touch 990 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2475826 = {S55675, 156, { &P17320,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ALCATEL_one_touch_990S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2518643 = {S55676, 161, { &P17320,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; TheMOONSTONE ALCATEL OT-990 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2518661 = {S55677, 156, { &P17320,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-ec; ALCATEL ONE TOUCH 990A Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2725032 = {S55678, 156, { &P17320,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-mn; ALCATEL one touch 990C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2797713 = {S55679, 85, { &P15364,&P17285,&P22774,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; SlimBrowser/7.00.035)*/
const Device UA2877266 = {S55680, 92, { &P15364,&P17285,&P22774,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; SlimBrowser/7.00.035)*/
const Device UA655461 = {S55681, 79, { &P15364,&P18144,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; OpenBSD i386; en-US; rv:1.8.1.14) Gecko/X Firefox/2.0.0.14*/
const Device UA655632 = {S55682, 76, { &P15364,&P18144,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; OpenBSD ppc; en-US; rv:1.8.1.9) Gecko/X BonEcho/2.0.0.9*/
const Device UA2878316 = {S55683, 136, { &P23700,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; N9101 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998325 = {S55684, 58, { &P23700,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; N9101 Build/JZO54K)*/
const Device UA2042129 = {S55685, 138, { &P18499,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; ISW11SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2594245 = {S55686, 60, { &P18499,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ISW11SC Build/IMM76D)*/
const Device UA2651654 = {S55687, 136, { &P18499,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-; ISW11SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1953779 = {S55688, 78, { &P15364,&P18346,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; Avant Browser)*/
const Device UA2167918 = {S55689, 78, { &P15364,&P18346,&P22732,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; Avant Browser)*/
const Device UA2777167 = {S55690, 93, { &P15364,&P18346,&P22732,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; Avant Browser; Avant Browser)*/
const Device UA2789073 = {S55691, 83, { &P15364,&P18346,&P22732,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; SV1; Avant Browser)*/
const Device UA2933661 = {S55692, 83, { &P15364,&P18346,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; SV1; Avant Browser)*/
const Device UA2981452 = {S55693, 93, { &P15364,&P18346,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Trident 5.0; Windows NT 4.1; Avant Browser; Avant Browser)*/
const Device UA2011710 = {S55694, 155, { &P12482,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T959 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028628 = {S55695, 155, { &P12482,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T959 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2203953 = {S55696, 155, { &P12482,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T959 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2354899 = {S55697, 159, { &P12482,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T959 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2554738 = {S55698, 152, { &P12482,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T959 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2672657 = {S55699, 75, { &P17595,&P17295,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8) Presto/2.12.388 Version/12.15*/
const Device UA2390862 = {S55700, 151, { &P18506,&P5043,&P24673,&P18092 }};	/*Useragent - Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia305) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA3029643 = {S55701, 113, { &P15786,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-05/08.69; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3058925 = {S55702, 115, { &P15786,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-05.1/08.69; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA2431856 = {S55703, 219, { &P15876,&P18066,&P18369,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2461344 = {S55704, 219, { &P15876,&P18066,&P18369,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2625823 = {S55705, 217, { &P15876,&P18066,&P18369,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3492; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2626454 = {S55706, 217, { &P15876,&P18066,&P18369,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3492; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2468892 = {S55707, 136, { &P23383,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; H7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778692 = {S55708, 58, { &P23383,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; H7100 Build/JRO03C)*/
const Device UA1958937 = {S55709, 153, { &P13025,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; SonyEricssonWT19a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028211 = {S55710, 141, { &P13025,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; WT19a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2648210 = {S55711, 141, { &P13025,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-ar; WT19a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2983392 = {S55712, 63, { &P13025,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; WT19a Build/4.1.B.0.587)*/
const Device UA61515 = {S55713, 52, { &P12739,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-WX430/00.72 UP.Browser/7.2.7.5.708 (GUI) MMP/2.0*/
const Device UA12705 = {S55714, 71, { &P18088,&P5448,&P22270,&P18092 }};	/*Useragent - 8900b/MR2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA13454 = {S55715, 76, { &P18088,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Touch 3G T3232*/
const Device UA30383 = {S55716, 110, { &P18088,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) 480x640; XV6950; Window Mobile 6.1 Professional;*/
const Device UA34837 = {S55717, 67, { &P18088,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) S21HT*/
const Device UA36223 = {S55718, 55, { &P18088,&P5448,&P22270,&P18092 }};	/*Useragent - A3100 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA40635 = {S55719, 67, { &P18088,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) S22HT*/
const Device UA40704 = {S55720, 147, { &P18088,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) e811_CU/1.0 CEOS/5.2.20269 Release/11.24.2009 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA40781 = {S55721, 65, { &P18088,&P5448,&P22270,&P18092 }};	/*Useragent - default (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) default*/
const Device UA53198 = {S55722, 86, { &P18088,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Traveler 117;VGA;480x640*/
const Device UA60992 = {S55723, 78, { &P18088,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) AnyDATA ASP-210C*/
const Device UA62524 = {S55724, 76, { &P18088,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Sprint:PPC6800*/
const Device UA87844 = {S55725, 78, { &P18088,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Sprint PPC6850SP*/
const Device UA88515 = {S55726, 69, { &P18088,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) WS020SH*/
const Device UA89821 = {S55727, 144, { &P18088,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) MOVISTAR/1.0/e810/VEN_TM_SE_P180A2V1.0.1 IE Mobile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA111701 = {S55728, 85, { &P18088,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Sprint:SPH-i325 320x240*/
const Device UA114359 = {S55729, 61, { &P18088,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA132143 = {S55730, 110, { &P18088,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) 320x320; XV6900; Window Mobile 6.1 Professional;*/
const Device UA629182 = {S55731, 158, { &P15539,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Glacier Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024430 = {S55732, 158, { &P15539,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Glacier Build/GRI40; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2166395 = {S55733, 154, { &P15539,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Glacier Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2459924 = {S55734, 97, { &P15364,&P7133,&P20806,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.4 Safari/534.34*/
const Device UA2683835 = {S55735, 90, { &P15364,&P7133,&P20806,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.2 Safari/534.34*/
const Device UA2407068 = {S55736, 120, { &P12088,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaX6-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2423146 = {S55737, 32, { &P12088,&P5129,&P24673,&P18092 }};	/*Useragent - NokiaX6-00/UCWEB8.7.0.187/50/999*/
const Device UA2425800 = {S55738, 42, { &P12088,&P5129,&P24673,&P18092 }};	/*Useragent - NokiaX6-00UCBrowser/8.7.1.234/50/444/UCWEB*/
const Device UA2943983 = {S55739, 64, { &P22598,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; C6506 Build/10.3.A.0.423)*/
const Device UA2945966 = {S55740, 146, { &P22598,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-ca; SonyC6506 Build/10.3.A.0.423) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2973284 = {S55741, 142, { &P22598,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-ca; C6506 Build/10.3.A.0.423) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3033718 = {S55742, 144, { &P22598,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; C6506 Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3126337 = {S55743, 143, { &P22598,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; C6506 Build/10.3.1.A.2.67) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA29930 = {S55744, 117, { &P12280,&P17134,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7A400*/
const Device UA29932 = {S55745, 102, { &P12280,&P17134,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko)*/
const Device UA29935 = {S55746, 104, { &P12280,&P17134,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko)*/
const Device UA29939 = {S55747, 115, { &P12280,&P17134,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7A341*/
const Device UA48697 = {S55748, 161, { &P18604,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; Android edition by sfr STARADDICT Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA79638 = {S55749, 161, { &P18604,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; Android edition by sfr STARADDICT Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2349847 = {S55750, 173, { &P18604,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; Smartphone Android by SFR STARADDICT II Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA925804 = {S55751, 114, { &P18088,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; Microsoft; Virtual)*/
const Device UA2258913 = {S55752, 75, { &P15364,&P17285,&P20761,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:16.0) Gecko/X Firefox/16.0 SeaMonkey/2.13.2*/
const Device UA2586640 = {S55753, 139, { &P21607,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; VS870 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2718750 = {S55754, 61, { &P21607,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; VS870 4G Build/JZO54K)*/
const Device UA3065592 = {S55755, 135, { &P21606,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LGL86C Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA63905 = {S55756, 122, { &P14555,&P5208,&P4223,&P18092 }};	/*Useragent - NetFront/3.5.1 (BREW 3.1.5; U; en-us; LG; NetFront/3.5.1/WAP) Sprint LN240 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA179244 = {S55757, 133, { &P17958,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MID8024 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458676 = {S55758, 57, { &P17958,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; MID8024 Build/FRF91)*/
const Device UA458810 = {S55759, 83, { &P17958,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/0.9.2 (Linux; U; Android 2.2; en_US; MID8024 Build/MID Serials 2.2 FRF91 X.X)*/
const Device UA459067 = {S55760, 83, { &P17958,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.2; en_US; MID8024 Build/MID Serilas 2.2 FRF91 X.X)*/
const Device UA9138 = {S55761, 153, { &P16584,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; SAMSUNG-SGH-I896 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA38283 = {S55762, 151, { &P16584,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-; SAMSUNG-SGH-I896 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA541899 = {S55763, 75, { &P16584,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SAMSUNG-SGH-I896 Build/ECLAIR)*/
const Device UA255925 = {S55764, 108, { &P15364,&P6596,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1193.65.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.44 Safari/535.7*/
const Device UA345479 = {S55765, 108, { &P15364,&P6596,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1193.15.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.10 Safari/535.7*/
const Device UA347499 = {S55766, 109, { &P15364,&P6596,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1193.105.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA353647 = {S55767, 109, { &P15364,&P6596,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1193.117.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.66 Safari/535.7*/
const Device UA366333 = {S55768, 109, { &P15364,&P6596,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1193.158.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7*/
const Device UA368206 = {S55769, 109, { &P15364,&P6596,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1193.194.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA919950 = {S55770, 108, { &P15364,&P6596,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1193.45.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.32 Safari/535.7*/
const Device UA2757915 = {S55771, 109, { &P15364,&P6596,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1144.0.2011) AppleWebKit/535.6 (KHTML, like Gecko) Chrome/16.0.903.0 Safari/535.6*/
const Device UA2676526 = {S55772, 117, { &P15364,&P16110,&P22466,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36*/
const Device UA2679537 = {S55773, 124, { &P15364,&P16110,&P22466,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36*/
const Device UA44236 = {S55774, 154, { &P14851,&P5267,&P3834,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5753E/S5753EDDJK6; U; Bada/1.0; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA369193 = {S55775, 134, { &P18686,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; IM-A810S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA630642 = {S55776, 136, { &P18686,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; IM-A810K Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA659724 = {S55777, 136, { &P18686,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; IM-A810S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020559 = {S55778, 60, { &P18686,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; IM-A810S Build/GRJ90)*/
const Device UA2034394 = {S55779, 60, { &P18686,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; IM-A810K Build/GRJ90)*/
const Device UA2256921 = {S55780, 134, { &P18686,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-; IM-A810K Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2987898 = {S55781, 159, { &P18686,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; IM-A810K Build/PHIEN BAN GALAXY S SKY A810K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA913368 = {S55782, 142, { &P18465,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Amazon Kindle Fire Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2044111 = {S55783, 148, { &P18465,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Amazon Kindle Fire Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2044115 = {S55784, 142, { &P18465,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Amazon Kindle Fire Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2756578 = {S55785, 138, { &P21627,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC first Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3030990 = {S55786, 133, { &P18464,&P18110,&P24334,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Safari/535.19 Silk-Accelerated=true*/
const Device UA697071 = {S55787, 133, { &P15389,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; u8800 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA710976 = {S55788, 139, { &P15389,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; U8800 Build/HuaweiU8800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA847784 = {S55789, 140, { &P15389,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Huawei u8800 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023745 = {S55790, 63, { &P15389,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; U8800 Build/HuaweiU8800)*/
const Device UA2025339 = {S55791, 127, { &P15389,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; U8800 Build/HuaweiU8800) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2027860 = {S55792, 145, { &P15389,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; U8800pro Build/HuaweiU8800pro) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2511127 = {S55793, 68, { &P15767,&P19826,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPad2,1; iPhone OS 6.1; en_US; en) AppleWebKit/420+*/
const Device UA2564173 = {S55794, 70, { &P15767,&P19826,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPad2,1; iPhone OS 6.1.2; en_US; en) AppleWebKit/420+*/
const Device UA2607003 = {S55795, 70, { &P15767,&P19826,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPad2,1; iPhone OS 6.1.3; en_US; en) AppleWebKit/420+*/
const Device UA2975006 = {S55796, 129, { &P23955,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5210 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA190447 = {S55797, 157, { &P18029,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Full Android on S5PV210 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA750333 = {S55798, 155, { &P18029,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-gb; Full Android on S5PV210 Build/HONEYCOMB) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1882478 = {S55799, 152, { &P18029,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; Full Android on S5PV210 Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019102 = {S55800, 81, { &P18029,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.1; Full Android on S5PV210 Build/GINGERBREAD)*/
const Device UA347403 = {S55801, 128, { &P15364,&P7133,&P22415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.1.1.0 Chrome/16.0.912.63 Safari/535.7*/
const Device UA347460 = {S55802, 121, { &P15364,&P7133,&P22415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.1.1.0 Chrome/16.0.912.63 Safari/535.7*/
const Device UA2974455 = {S55803, 148, { &P18472,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT925 Build/7.7.1Q-144_VQL_S3-58) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975074 = {S55804, 158, { &P18472,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR HD Build/7.7.1Q-144_VQW_S334-25) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3031128 = {S55805, 156, { &P18442,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7100 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3032941 = {S55806, 156, { &P18442,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-N719 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3039175 = {S55807, 156, { &P18442,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N7100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3061649 = {S55808, 156, { &P18442,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7105 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA1995104 = {S55809, 53, { &P18088,&P17017,&P24672,&P18092 }};	/*Useragent - IUC(U;iOS 5.0.1;Zh-cn;320*480;)/UCWEB8.6.0.180/42/800*/
const Device UA1995418 = {S55810, 53, { &P18088,&P17017,&P24672,&P18092 }};	/*Useragent - IUC(U;iOS 5.1.1;Zh-cn;320*480;)/UCWEB8.6.0.180/42/800*/
const Device UA1996003 = {S55811, 53, { &P18088,&P17017,&P24672,&P18092 }};	/*Useragent - IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB8.6.0.180/41/800*/
const Device UA2166964 = {S55812, 51, { &P18088,&P17017,&P24672,&P18092 }};	/*Useragent - IUC(U;iOS 4.1;Zh-cn;320*480;)/UCWEB8.6.0.180/41/800*/
const Device UA2230440 = {S55813, 113, { &P18088,&P17017,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; j2me) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/84/352/UCWEB Mobile*/
const Device UA2348889 = {S55814, 113, { &P18088,&P17017,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; maui) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2350927 = {S55815, 114, { &P18088,&P17017,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; mstar) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2366822 = {S55816, 114, { &P18088,&P17017,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; spice) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2382672 = {S55817, 116, { &P18088,&P17017,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; generic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile*/
const Device UA2383039 = {S55818, 123, { &P18088,&P17017,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; mstar platform) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2383246 = {S55819, 114, { &P18088,&P17017,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; opera) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2401863 = {S55820, 117, { &P18088,&P17017,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; sunmicro) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2403165 = {S55821, 121, { &P18088,&P17017,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; maui browser) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2420300 = {S55822, 110, { &P18088,&P17017,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; vi; maui) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2503779 = {S55823, 116, { &P18088,&P17017,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; karbonn) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2778175 = {S55824, 86, { &P18088,&P17017,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; zh-cn; maui) U2/1.0.0 UCBrowser/8.6.1.237 U2/1.0.0 Mobile*/
const Device UA2790423 = {S55825, 116, { &P18088,&P17017,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; mozilla) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/405/UCWEB Mobile*/
const Device UA2182244 = {S55826, 84, { &P18254,&P18110,&P18245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-gb) AppleWebKit/418.9.1 (KHTML, like Gecko) SiteCon/8.10.9*/
const Device UA2672575 = {S55827, 33, { &P18254,&P18110,&P18245,&P18092 }};	/*Useragent - Feedly/1.0 (Linux x86_64; rv:1.0)*/
const Device UA2859169 = {S55828, 41, { &P18254,&P18110,&P18245,&P18092 }};	/*Useragent - Needle/0.4.8 (Node.js v0.8.22; linux x64)*/
const Device UA2361766 = {S55829, 164, { &P20414,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; Vodafone SmartTab II 10 Build/SmartTabII10_A403_068_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2468251 = {S55830, 164, { &P20414,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; Vodafone SmartTab II 10 Build/SmartTabII10_A403_V72_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2508334 = {S55831, 164, { &P20414,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; Vodafone SmartTab II 10 Build/SmartTabII10_A403_K85_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2593774 = {S55832, 136, { &P20414,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; SmartTabII10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2600308 = {S55833, 164, { &P20414,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; Vodafone SmartTab II 10 Build/SmartTabII10_A403_K83_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2421861 = {S55834, 101, { &P15250,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.6.0.51; en-us; 9360/9.6.0.51) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2423463 = {S55835, 103, { &P15250,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 8.0.0.388; en-us; 9360/8.0.0.388) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2480803 = {S55836, 103, { &P15250,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 8.0.0.499; en-us; 9360/8.0.0.499) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2555202 = {S55837, 103, { &P15250,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.6.0.111; en-us; 9360/9.6.0.111) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2594730 = {S55838, 103, { &P15250,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 8.0.0.510; en-us; 9360/8.0.0.510) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2654093 = {S55839, 101, { &P15250,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.6.0.80; en-us; 9360/9.6.0.80) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2697024 = {S55840, 103, { &P15250,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 8.0.0.359; en-us; 9360/8.0.0.359) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2764087 = {S55841, 103, { &P15250,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 8.0.0.520; en-us; 9360/8.0.0.520) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2795724 = {S55842, 101, { &P15250,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.6.0.39; en-us; 9360/9.6.0.39) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2840452 = {S55843, 101, { &P15250,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.6.0.86; en-us; 9360/9.6.0.86) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2848818 = {S55844, 101, { &P15250,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.6.0.26; en-us; 9360/9.6.0.26) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2849670 = {S55845, 103, { &P15250,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.6.0.112; en-us; 9360/9.6.0.112) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2849814 = {S55846, 103, { &P15250,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 8.0.0.464; en-us; 9360/8.0.0.464) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2951076 = {S55847, 103, { &P15250,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 8.0.0.391; en-us; 9360/8.0.0.391) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2951294 = {S55848, 103, { &P15250,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 8.0.0.419; en-us; 9360/8.0.0.419) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2951972 = {S55849, 101, { &P15250,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.6.0.24; en-us; 9360/9.6.0.24) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3047034 = {S55850, 101, { &P15250,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.6.0.93; en-us; 9360/9.6.0.93) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2895186 = {S55851, 62, { &P17968,&P17979,&P23660,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:25.0) Gecko/25.0 Firefox/25.0*/
const Device UA3077849 = {S55852, 68, { &P17968,&P17979,&P23660,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:25.0a1) Gecko/25.0a1 Firefox/25.0a1*/
const Device UA3108097 = {S55853, 68, { &P17968,&P17979,&P23660,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:25.0a2) Gecko/25.0a2 Firefox/25.0a2*/
const Device UA2405266 = {S55854, 58, { &P15364,&P7127,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:17.0) Gecko/X Firefox/17.0*/
const Device UA1991165 = {S55855, 151, { &P20964,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; Turkcell Maxi Plus 5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028858 = {S55856, 149, { &P20964,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-; Turkcell Maxi Plus 5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2813151 = {S55857, 73, { &P20964,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Turkcell Maxi Plus 5 Build/IMM76D)*/
const Device UA50343 = {S55858, 139, { &P13234,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; Behold2 Build/DONUT) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA67185 = {S55859, 141, { &P13234,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; Behold II Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA609425 = {S55860, 129, { &P12765,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  A100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA901029 = {S55861, 128, { &P12765,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2225283 = {S55862, 134, { &P12765,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2199567 = {S55863, 58, { &P15364,&P7127,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:16.0) Gecko/X Firefox/16.0*/
const Device UA2404369 = {S55864, 154, { &P18465,&P18066,&P19804,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA658971 = {S55865, 113, { &P18219,&P5843,&P17438,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia202/20.28; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15*/
const Device UA1959293 = {S55866, 113, { &P18219,&P5843,&P17438,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia203/20.26; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15*/
const Device UA2754577 = {S55867, 139, { &P18453,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT25i Build/9.1.A.1.140) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2435185 = {S55868, 72, { &P15364,&P7133,&P22562,&P18092 }};	/*Useragent - Mozilla/8.0(Windows; U; Windows 7; en-Us; rv : 1.7) Gecko/X Firefox/15.1*/
const Device UA2741794 = {S55869, 101, { &P22713,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 521)*/
const Device UA2585479 = {S55870, 155, { &P21857,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA346815 = {S55871, 113, { &P15364,&P5690,&P22453,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.2 Chrome/15.0.900.2 Safari/535.2*/
const Device UA2216248 = {S55872, 113, { &P15364,&P5690,&P22453,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.1 Chrome/15.0.900.1 Safari/535.2*/
const Device UA2259221 = {S55873, 120, { &P15364,&P5690,&P22453,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.2 Chrome/15.0.900.2 Safari/535.2*/
const Device UA912272 = {S55874, 78, { &P15364,&P17285,&P22578,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; Edition Next; en) Presto/2.11.310 Version/12.50*/
const Device UA545389 = {S55875, 98, { &P15364,&P6704,&P22516,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows 98; en-US; rv:1.8.1.5pre) Gecko/X Firefox/2.0.0.4 Navigator/9.0b2*/
const Device UA739994 = {S55876, 94, { &P15364,&P6704,&P22516,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.8.1.8pre) Gecko/X Firefox/2.0.0.7 Navigator/9.0RC1*/
const Device UA830534 = {S55877, 97, { &P15364,&P6704,&P22516,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win 9x 4.90; en-US; rv:1.8.1.8pre) Gecko/X Firefox/2.0.0.7 Navigator/9.0*/
const Device UA904417 = {S55878, 85, { &P15364,&P7133,&P22360,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.2.1) Gecko/X Firefox/12.2.1 PaleMoon/12.2.1*/
const Device UA904860 = {S55879, 98, { &P15364,&P7133,&P22360,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:12.2.1) Gecko/X Firefox/12.2.1-x64 PaleMoon/12.2.1-x64*/
const Device UA2974877 = {S55880, 138, { &P20947,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTCEVOV4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975099 = {S55881, 137, { &P20226,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E975K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975932 = {S55882, 136, { &P20226,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2422926 = {S55883, 115, { &P18558,&P20706,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia309/05.85; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.13.8*/
const Device UA62573 = {S55884, 80, { &P13770,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C6112/C6112DDJB2 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2567945 = {S55885, 133, { &P17968,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583075 = {S55886, 137, { &P17968,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TWD_MID Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583418 = {S55887, 133, { &P17968,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583564 = {S55888, 126, { &P17968,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2584854 = {S55889, 117, { &P17968,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Build/X) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2602927 = {S55890, 144, { &P17968,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbl1 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602990 = {S55891, 129, { &P17968,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2605923 = {S55892, 149, { &P17968,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1808_v75_jbl1_5640 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2666881 = {S55893, 149, { &P17968,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1808_v75_jbl2_6628 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2666892 = {S55894, 158, { &P17968,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; e1909c_v77_gq1008_9p017_6228 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667035 = {S55895, 136, { &P17968,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Tender Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667340 = {S55896, 122, { &P17968,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667523 = {S55897, 144, { &P17968,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1808_v75_jbl2 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667635 = {S55898, 128, { &P17968,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; V1013 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2732896 = {S55899, 131, { &P17968,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; EBM727KC Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2750106 = {S55900, 126, { &P17968,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2779940 = {S55901, 133, { &P17968,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Pad Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2793746 = {S55902, 144, { &P17968,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; INFINITY_POWER Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2953758 = {S55903, 158, { &P17968,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1809l_v75_gq1008_ov5647_v20 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2974292 = {S55904, 126, { &P18282,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A200 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA903234 = {S55905, 64, { &P15364,&P7127,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.10.289 Version/12.00*/
const Device UA1996994 = {S55906, 64, { &P15364,&P7127,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.10.289 Version/12.01*/
const Device UA3020299 = {S55907, 130, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; AT10LE-A Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021062 = {S55908, 123, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; M6 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021424 = {S55909, 124, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; S1S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021533 = {S55910, 133, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; T600 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021640 = {S55911, 131, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; C3 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021733 = {S55912, 147, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; 06_v89_jbla768_asx Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021783 = {S55913, 132, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; CW-V88-QUAD Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021828 = {S55914, 141, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; v89_jbl1a668 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022001 = {S55915, 131, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Z5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022106 = {S55916, 133, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; 7102 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022317 = {S55917, 140, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; v89_gq2008s Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022387 = {S55918, 144, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Lenovo S820_ROW Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022395 = {S55919, 130, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; AT10PE-A Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022613 = {S55920, 136, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; IMO S89 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022635 = {S55921, 133, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; B943 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022644 = {S55922, 138, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Transformer Prime Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022704 = {S55923, 129, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; V818mini Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022720 = {S55924, 138, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; redhookbay Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022799 = {S55925, 139, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; v89_zlh_hd Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023149 = {S55926, 150, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; 06v20_v89_jbla768_asx Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023155 = {S55927, 133, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; X508 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023176 = {S55928, 132, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; S510 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023196 = {S55929, 134, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; H3088 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023229 = {S55930, 135, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; VOW V5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023277 = {S55931, 134, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; I9389 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023625 = {S55932, 126, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; EPAD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023636 = {S55933, 136, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; TCL S950 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023828 = {S55934, 140, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; v89_gq3008s Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023833 = {S55935, 138, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ZTE V967S Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023877 = {S55936, 129, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; M470BSE Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3024091 = {S55937, 131, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Q5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024107 = {S55938, 135, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; HY5001 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024211 = {S55939, 131, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; V811 Core4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3027003 = {S55940, 160, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; 06_v89_jbla768_asx_s4lockscreen Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3030874 = {S55941, 144, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; AUXUS Nuclea N1 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3057724 = {S55942, 129, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-p6810 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3058076 = {S55943, 132, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; R821 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3058081 = {S55944, 131, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Z2 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3058191 = {S55945, 144, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; v89_jbla768_asx Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3078267 = {S55946, 132, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; x909 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3092204 = {S55947, 123, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; U8 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3092391 = {S55948, 131, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HYUNDAI T7 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3107922 = {S55949, 137, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; HX-9299A Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3124991 = {S55950, 152, { &P15364,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; HUAWEI G610-U00 Build/HuaweiG610-U00) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2353295 = {S55951, 75, { &P17595,&P17296,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8) Presto/2.12.388 Version/12.10*/
const Device UA662592 = {S55952, 87, { &P12832,&P17485,&P3499,&P18092 }};	/*Useragent - SonyEricssonCK13i/R5AA032 TelecaBrowser/Q07C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA636829 = {S55953, 142, { &P18631,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; PMP5080B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA814871 = {S55954, 144, { &P18631,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; ru-ru; PMP5080BRU Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA909256 = {S55955, 144, { &P18009,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTCEVODesign4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1953980 = {S55956, 137, { &P18009,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PH44100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2425848 = {S55957, 146, { &P20436,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Cyclone Voyager Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556618 = {S55958, 68, { &P20436,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Cyclone Voyager Build/JRO03H)*/
const Device UA2796171 = {S55959, 141, { &P20436,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Cyclone Voyager 2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2566496 = {S55960, 140, { &P18453,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT25i Build/9.1.A.0.490) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583227 = {S55961, 133, { &P18453,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT25i Build/9.1.A.0.490) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2025678 = {S55962, 135, { &P21383,&P5629,&P15760,&P18092 }};	/*Useragent - LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/06.14.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2223404 = {S55963, 135, { &P21383,&P5629,&P15760,&P18092 }};	/*Useragent - LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/07.06.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2256940 = {S55964, 143, { &P21383,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A288t Build/MocorDroid2.3.5) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2375814 = {S55965, 150, { &P21383,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A288t Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2375957 = {S55966, 143, { &P21383,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A288t Build MocorDroid2.3.5) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2380551 = {S55967, 135, { &P21383,&P5629,&P15760,&P18092 }};	/*Useragent - LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/08.01.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2382225 = {S55968, 135, { &P21383,&P5629,&P15760,&P18092 }};	/*Useragent - LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/08.16.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2426528 = {S55969, 135, { &P21383,&P5629,&P15760,&P18092 }};	/*Useragent - LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/10.22.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2520509 = {S55970, 135, { &P21383,&P5629,&P15760,&P18092 }};	/*Useragent - LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/06.20.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2524613 = {S55971, 111, { &P21383,&P5629,&P15760,&P18092 }};	/*Useragent - Lenovo-A288t_TD/S100 Linux/2.6.35 Android/2.3.5 Release/02.29.2012 Browser/AppleWebkit533.1 Mobile Safari/533.1*/
const Device UA2528631 = {S55972, 135, { &P21383,&P5629,&P15760,&P18092 }};	/*Useragent - LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/11.23.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2654061 = {S55973, 135, { &P21383,&P5629,&P15760,&P18092 }};	/*Useragent - LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/09.28.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2695469 = {S55974, 135, { &P21383,&P5629,&P15760,&P18092 }};	/*Useragent - LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/10.09.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2701654 = {S55975, 135, { &P21383,&P5629,&P15760,&P18092 }};	/*Useragent - LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/12.28.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2776013 = {S55976, 74, { &P21383,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Lenovo A288t Build/MocorDroid2.3.5)*/
const Device UA2838100 = {S55977, 135, { &P21383,&P5629,&P15760,&P18092 }};	/*Useragent - LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/09.20.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2998837 = {S55978, 135, { &P21383,&P5629,&P15760,&P18092 }};	/*Useragent - LENOVO-Lenovo-A288t/1.0 Linux/2.6.35.7 Android/2.3.5 Release/03.27.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2204344 = {S55979, 139, { &P20235,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Cynus T1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2421971 = {S55980, 61, { &P20235,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Cynus T1 Build/IMM76D)*/
const Device UA2403647 = {S55981, 128, { &P18239,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A510 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2404173 = {S55982, 128, { &P18239,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A510 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2566946 = {S55983, 132, { &P15869,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ505 Build/Moto_MZ505) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567137 = {S55984, 127, { &P15869,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xoom Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2603534 = {S55985, 126, { &P15869,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Xoom Build/IML77) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA631948 = {S55986, 143, { &P22592,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; SGH-I717R Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA671980 = {S55987, 67, { &P22592,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-I717R Build/GINGERBREAD)*/
const Device UA854243 = {S55988, 141, { &P22592,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SGH-I717R Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2460906 = {S55989, 150, { &P17337,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7500 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2567553 = {S55990, 149, { &P17337,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7500 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2629040 = {S55991, 150, { &P17337,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7500 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2713885 = {S55992, 150, { &P17337,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7500 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2424480 = {S55993, 128, { &P15869,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Xoom Build/JB_MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2459552 = {S55994, 128, { &P15869,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Xoom Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2537628 = {S55995, 127, { &P15869,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Xoom Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2559183 = {S55996, 130, { &P15869,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Xoom Build/JB_MR1.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2764664 = {S55997, 56, { &P15869,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; Xoom Build/JDQ39)*/
const Device UA655063 = {S55998, 82, { &P17595,&P17297,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.7.3; U; en) Presto/2.10.229 Version/11.61*/
const Device UA2199242 = {S55999, 118, { &P14358,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE63) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile*/
const Device UA2226061 = {S56000, 30, { &P14358,&P5890,&P24672,&P18092 }};	/*Useragent - NokiaE63/UCWEB8.6.0.180/28/999*/
const Device UA2401581 = {S56001, 119, { &P14358,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiae63-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2753128 = {S56002, 147, { &P15877,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA67702 = {S56003, 137, { &P14979,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.0; en-us; A70BHT Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA67813 = {S56004, 137, { &P14979,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.1; md-us; A70BHT Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA363864 = {S56005, 137, { &P14979,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.1.1; en-us; A70BHT Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2020809 = {S56006, 140, { &P14979,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.0; es-es; A70BHT-LZ Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2021358 = {S56007, 140, { &P14979,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.4; fr-fr; A70BHT-SC Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2022623 = {S56008, 140, { &P14979,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.0; us-us; A70BHT-PR Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2489117 = {S56009, 91, { &P13977,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaN73) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2406402 = {S56010, 76, { &P15364,&P17285,&P22367,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:15.3) Gecko/X Firefox/15.3.2 PaleMoon/15.3.2*/
const Device UA2406631 = {S56011, 96, { &P15364,&P17285,&P22367,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:15.3) Gecko/X Firefox/15.3.2-x64 PaleMoon/15.3.2-x64*/
const Device UA2510023 = {S56012, 226, { &P12280,&P19826,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA2511758 = {S56013, 226, { &P12280,&P19826,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA2532296 = {S56014, 228, { &P12280,&P19826,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA2534978 = {S56015, 228, { &P12280,&P19826,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA2537376 = {S56016, 226, { &P12280,&P19826,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA2562477 = {S56017, 226, { &P12280,&P19826,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA2606925 = {S56018, 228, { &P12280,&P19826,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA2739709 = {S56019, 228, { &P12280,&P19826,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA3033508 = {S56020, 156, { &P18283,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-I9300 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3039402 = {S56021, 156, { &P18283,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9300 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3058466 = {S56022, 156, { &P18283,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-I939 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3065074 = {S56023, 156, { &P18283,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SGH-T999 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3065548 = {S56024, 156, { &P18283,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SCH-I535 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3108747 = {S56025, 157, { &P18283,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-I939D Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA184241 = {S56026, 134, { &P15364,&P7133,&P22448,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Iron/10.0.650.1 Chrome/10.0.650.1 Safari/534.16*/
const Device UA2540022 = {S56027, 154, { &P18465,&P18066,&P21167,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA105835 = {S56028, 47, { &P15364,&P7127,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 5.0; U; en) Presto/2.1.1*/
const Device UA167798 = {S56029, 47, { &P15364,&P7127,&P18337,&P18092 }};	/*Useragent - Opera/9.62 (Windows NT 5.0; U; en) Presto/2.1.1*/
const Device UA171548 = {S56030, 47, { &P15364,&P7127,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 5.0; U; en) Presto/2.1.1*/
const Device UA241696 = {S56031, 47, { &P15364,&P7127,&P18337,&P18092 }};	/*Useragent - Opera/9.60 (Windows NT 5.0; U; de) Presto/2.1.1*/
const Device UA535344 = {S56032, 65, { &P15364,&P7127,&P18337,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; de) Opera 9.64*/
const Device UA2138214 = {S56033, 47, { &P15364,&P7127,&P18337,&P18092 }};	/*Useragent - Opera/9.61 (Windows NT 5.0; U; en) Presto/2.1.1*/
const Device UA6610 = {S56034, 150, { &P13815,&P5477,&P4861,&P18092 }};	/*Useragent - HTC_Wildfire_Mozilla/5.0 (Linux; U; Android 2.1; it-it; HTC Buzz Build/ERD79) AppleWebKit/ 530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32249 = {S56035, 159, { &P13815,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Wildfire 1.14.161.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32251 = {S56036, 159, { &P13815,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; nl-nl; HTC Wildfire 1.14.172.2 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32265 = {S56037, 159, { &P13815,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Wildfire 1.26.161.3 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32270 = {S56038, 158, { &P13815,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; HTC_Wildfire-orange-LS Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32271 = {S56039, 159, { &P13815,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; HTC Wildfire 1.13.163.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32272 = {S56040, 159, { &P13815,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; HTC Wildfire 1.14.162.3 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32296 = {S56041, 159, { &P13815,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; HTC Wildfire 1.25.162.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32313 = {S56042, 159, { &P13815,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; HTC Wildfire 1.36.164.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32317 = {S56043, 159, { &P13815,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; HTC Wildfire 1.29.163.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32391 = {S56044, 159, { &P13815,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; nl-nl; HTC Wildfire 1.25.172.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA37780 = {S56045, 159, { &P13815,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; de-ch; HTC Wildfire 1.29.163.2 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA51173 = {S56046, 159, { &P13815,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; HTC Wildfire 1.17.164.3 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA89840 = {S56047, 155, { &P13815,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; HTC Buzz 1.02.161.2 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA132858 = {S56048, 148, { &P13815,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Wildfire Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA157732 = {S56049, 70, { &P13815,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; HTC Wildfire Build/ERE27)*/
const Device UA2021637 = {S56050, 134, { &P13815,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; HTC Wildfire Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1990783 = {S56051, 78, { &P15364,&P18579,&P17470,&P18092 }};	/*Useragent - curl/7.24.0 (x86_64-apple-darwin12.0) libcurl/7.24.0 OpenSSL/0.9.8r zlib/1.2.5*/
const Device UA2799879 = {S56052, 78, { &P15364,&P18579,&P17470,&P18092 }};	/*Useragent - curl/7.24.0 (x86_64-apple-darwin12.0) libcurl/7.24.0 OpenSSL/0.9.8x zlib/1.2.5*/
const Device UA3109377 = {S56053, 78, { &P15364,&P18579,&P17470,&P18092 }};	/*Useragent - curl/7.24.0 (x86_64-apple-darwin12.0) libcurl/7.24.0 OpenSSL/0.9.8y zlib/1.2.5*/
const Device UA2670901 = {S56054, 148, { &P18472,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT925 Build/7.7.1Q-144_VQL_S3-58) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2671049 = {S56055, 151, { &P18472,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR HD Build/7.7.1Q-144_VQW_S334-25) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2671104 = {S56056, 141, { &P18472,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT925 Build/7.7.1Q-144_VQL_S3-58) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2677186 = {S56057, 158, { &P18472,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR HD Build/7.7.1Q-144_VQW_S334-25) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3066524 = {S56058, 140, { &P22598,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6506 Build/10.3.A.0.423) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2970466 = {S56059, 91, { &P15364,&P7133,&P23490,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Maxthon/4.1.0.4000)*/
const Device UA2971324 = {S56060, 84, { &P15364,&P7133,&P23490,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0; Maxthon/4.1.0.4000)*/
const Device UA3015871 = {S56061, 83, { &P15364,&P7133,&P23490,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Maxthon/4.1.0.4000)*/
const Device UA3021440 = {S56062, 90, { &P15364,&P7133,&P23490,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Maxthon/4.1.0.4000)*/
const Device UA2567110 = {S56063, 135, { &P19904,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Hydro Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567256 = {S56064, 134, { &P19904,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C5170 Build/IML77) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA68310 = {S56065, 138, { &P14922,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; LG-LU6800 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA643275 = {S56066, 62, { &P14922,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-LU6800 Build/FRG83G)*/
const Device UA2160621 = {S56067, 104, { &P15364,&P18145,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.65 Safari/535.11*/
const Device UA2411131 = {S56068, 103, { &P15364,&P18145,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.83 Safari/535.11*/
const Device UA672313 = {S56069, 138, { &P12788,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  PG86100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA902065 = {S56070, 137, { &P12788,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PG86100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1994991 = {S56071, 137, { &P12788,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PG86100 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2029662 = {S56072, 140, { &P12788,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Evo 3D GSM Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2199093 = {S56073, 137, { &P12788,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ISW12HT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2422105 = {S56074, 118, { &P12004,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN95) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2427354 = {S56075, 122, { &P12004,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN95 8GB) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2457677 = {S56076, 87, { &P12004,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; fa; nokian95) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2459563 = {S56077, 40, { &P12004,&P5890,&P24673,&P18092 }};	/*Useragent - NOKIAN95UCBrowser/8.7.1.234/28/444/UCWEB*/
const Device UA2505452 = {S56078, 44, { &P12004,&P5890,&P24673,&P18092 }};	/*Useragent - NOKIAN95 8GBUCBrowser/8.7.1.234/28/444/UCWEB*/
const Device UA2983483 = {S56079, 94, { &P12004,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokian95_8gb) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2510527 = {S56080, 114, { &P14669,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-00/08.25; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2634037 = {S56081, 130, { &P17337,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P7500 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.26 Safari/537.31*/
const Device UA2676022 = {S56082, 130, { &P17337,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2677293 = {S56083, 131, { &P17337,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500R Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2707996 = {S56084, 131, { &P17337,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500M Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2387661 = {S56085, 151, { &P18643,&P5043,&P24673,&P18092 }};	/*Useragent - Nokia311/5.0 (03.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia311) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2753056 = {S56086, 149, { &P18317,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2753147 = {S56087, 150, { &P18317,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300TG Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754066 = {S56088, 150, { &P18317,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300TL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2875247 = {S56089, 144, { &P18317,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Transformer Pad TF300T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2975082 = {S56090, 130, { &P20160,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA30924 = {S56091, 75, { &P12370,&P5208,&P4223,&P18092 }};	/*Useragent - Mozilla/4.0 (BREW 3.1.5; U; en-us; Sanyo; NetFront/3.5.1/AMB) Boost SCP3810*/
const Device UA55051 = {S56092, 77, { &P12370,&P5208,&P4223,&P18092 }};	/*Useragent - Mozilla/4.0 (BREW 3.1.5; U; en-us; Sanyo; NetFront/3.5.1/AMB) Sprint SCP-3810*/
const Device UA246979 = {S56093, 140, { &P20722,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; AN7CG2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019460 = {S56094, 64, { &P20722,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; AN7CG2 Build/GINGERBREAD)*/
const Device UA2222939 = {S56095, 139, { &P23805,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-nz; U8186 Build/HuaweiU8186) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982143 = {S56096, 63, { &P23805,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; U8186 Build/HuaweiU8186)*/
const Device UA2257380 = {S56097, 137, { &P23130,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; H2000+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2577933 = {S56098, 59, { &P23130,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; H2000+ Build/IMM76D)*/
const Device UA2671108 = {S56099, 150, { &P18317,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300TL Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676270 = {S56100, 149, { &P18317,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300T Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676331 = {S56101, 150, { &P18317,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF300TG Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2734721 = {S56102, 144, { &P18317,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Transformer Pad TF300T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3065090 = {S56103, 138, { &P22851,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT919 Build/2_290_2012) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2566451 = {S56104, 130, { &P16976,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT300SE Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566966 = {S56105, 128, { &P16976,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT300 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2974018 = {S56106, 132, { &P22754,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HP Slate 7 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA606019 = {S56107, 154, { &P12496,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I897 Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA615654 = {S56108, 154, { &P12496,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I897 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031380 = {S56109, 158, { &P12496,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I897 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2754382 = {S56110, 140, { &P12841,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC PH39100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877806 = {S56111, 140, { &P12841,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC PH39100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3027019 = {S56112, 141, { &P21648,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-I9505G Build/JWR66V.S014) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3027678 = {S56113, 135, { &P21648,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-I9505 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3125267 = {S56114, 162, { &P21648,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; en-gb; SAMSUNG GT-I9505 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Version/1.5 Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3039218 = {S56115, 105, { &P20093,&P17484,&P17369,&P18092 }};	/*Useragent - SAMSUNG-GT-E2250 Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.29465/30.3745; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2194864 = {S56116, 182, { &P14826,&P5843,&P24671,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac1-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2368018 = {S56117, 117, { &P15364,&P17285,&P22463,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11*/
const Device UA2391584 = {S56118, 124, { &P15364,&P17285,&P22463,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11*/
const Device UA2676253 = {S56119, 135, { &P17968,&P18149,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; cm_tenderloin Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676490 = {S56120, 134, { &P17968,&P18149,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; cm_tenderloin Build/GWK74) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA238593 = {S56121, 64, { &P15364,&P18110,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; en) Presto/2.9.168 Version/11.51*/
const Device UA839065 = {S56122, 90, { &P15364,&P18110,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; ru) Presto/2.9.168 Version/11.51 (ru-RU,ru;q=0.9,en;q=0.8)*/
const Device UA1995866 = {S56123, 135, { &P18238,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; E330 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2709184 = {S56124, 161, { &P13123,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; GT-S5360 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2714260 = {S56125, 161, { &P13123,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5360 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2998295 = {S56126, 162, { &P13123,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-LA; GT-S5360L Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3000088 = {S56127, 161, { &P13123,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; SCH-i509 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2352470 = {S56128, 182, { &P14669,&P5843,&P24672,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (04.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile*/
const Device UA2673349 = {S56129, 131, { &P15919,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100B Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676276 = {S56130, 130, { &P15919,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676591 = {S56131, 130, { &P15919,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3096282 = {S56132, 157, { &P18283,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3096296 = {S56133, 157, { &P18283,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T999 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3096365 = {S56134, 157, { &P18283,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3112644 = {S56135, 157, { &P18283,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9300 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3129224 = {S56136, 158, { &P18283,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3068324 = {S56137, 140, { &P18223,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126345 = {S56138, 139, { &P18223,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LT22i Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA163806 = {S56139, 129, { &P17969,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0; en-us; pandigital9hr) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA219621 = {S56140, 130, { &P17969,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0; en-us; pandigital9hr2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA882439 = {S56141, 53, { &P18088,&P17017,&P24670,&P18092 }};	/*Useragent - IUC(U;iOS 5.0.1;Zh-cn;320*480;)/UCWEB8.4.1.169/42/997*/
const Device UA886270 = {S56142, 53, { &P18088,&P17017,&P24670,&P18092 }};	/*Useragent - IUC(U;iOS 5.1.1;Zh-cn;320*480;)/UCWEB8.4.1.169/42/997*/
const Device UA894228 = {S56143, 53, { &P18088,&P17017,&P24670,&P18092 }};	/*Useragent - IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB8.4.1.169/41/997*/
const Device UA2175308 = {S56144, 53, { &P18088,&P17017,&P24670,&P18092 }};	/*Useragent - IUC(U;iOS 5_1_1;en-us;320*480;)/UCWEB8.4.0.159/42/355*/
const Device UA2219079 = {S56145, 53, { &P18088,&P17017,&P24670,&P18092 }};	/*Useragent - IUC(U;iOS 4.2.1;Zh-cn;320*480;)/UCWEB8.4.1.169/41/997*/
const Device UA2219838 = {S56146, 74, { &P18088,&P17017,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; spice) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2222869 = {S56147, 74, { &P18088,&P17017,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; opera) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2223143 = {S56148, 73, { &P18088,&P17017,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; maui) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2224168 = {S56149, 73, { &P18088,&P17017,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; j2me) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2229405 = {S56150, 74, { &P18088,&P17017,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; mstar) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2382295 = {S56151, 83, { &P18088,&P17017,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; mstar platform) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2476467 = {S56152, 76, { &P18088,&P17017,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; generic) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2588038 = {S56153, 51, { &P18088,&P17017,&P24670,&P18092 }};	/*Useragent - IUC(U;iOS 4.1;Zh-cn;320*480;)/UCWEB8.4.1.169/41/997*/
const Device UA2816560 = {S56154, 76, { &P18088,&P17017,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; mozilla) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2832769 = {S56155, 71, { &P18088,&P17017,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; kau; maui) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2891258 = {S56156, 141, { &P24101,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ADR930L 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2965580 = {S56157, 138, { &P24101,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ADR930L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3017527 = {S56158, 60, { &P24101,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; ADR930L Build/JZO54K)*/
const Device UA2367233 = {S56159, 128, { &P20269,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; A211 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2727984 = {S56160, 57, { &P20269,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; A211 Build/JRO03H)*/
const Device UA624173 = {S56161, 58, { &P15364,&P17188,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0; rv:12.0) Gecko/X Firefox/12.0*/
const Device UA2227038 = {S56162, 112, { &P19974,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia111/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.5.16.9*/
const Device UA2383523 = {S56163, 114, { &P19974,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia111.1/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.5.16.9*/
const Device UA446789 = {S56164, 96, { &P15364,&P18133,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.5) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.5*/
const Device UA472523 = {S56165, 98, { &P15364,&P18133,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.11) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.11*/
const Device UA499457 = {S56166, 97, { &P15364,&P18133,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.3) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.17*/
const Device UA534141 = {S56167, 98, { &P15364,&P18133,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.13) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.13*/
const Device UA648764 = {S56168, 98, { &P15364,&P18133,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.18) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.17*/
const Device UA673097 = {S56169, 96, { &P15364,&P18133,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.7) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.7*/
const Device UA691574 = {S56170, 96, { &P15364,&P18133,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.8) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.8*/
const Device UA899412 = {S56171, 98, { &P15364,&P18133,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; fi-FI; rv:1.9.0.13) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.13*/
const Device UA1863952 = {S56172, 98, { &P15364,&P18133,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.17) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.17*/
const Device UA2128101 = {S56173, 97, { &P15364,&P18133,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.6) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.17*/
const Device UA2138093 = {S56174, 96, { &P15364,&P18133,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.5) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.4*/
const Device UA2758960 = {S56175, 117, { &P15364,&P18133,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.8) Gecko/X Linux Mint/6 (Felicia) Firefox/3.0.8 (.NET CLR 3.5.30729)*/
const Device UA2753622 = {S56176, 138, { &P19845,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE N9120 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA246634 = {S56177, 135, { &P17160,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Slider SL101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA249373 = {S56178, 134, { &P17160,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Slider SL101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA712461 = {S56179, 64, { &P17160,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; Slider SL101 Build/HTK75)*/
const Device UA2998305 = {S56180, 143, { &P19778,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PantechP9090 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3026054 = {S56181, 65, { &P19778,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; PantechP9090 Build/JZO54K)*/
const Device UA51971 = {S56182, 138, { &P13960,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; V9 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1872253 = {S56183, 60, { &P13960,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; V9 Build/ERE27)*/
const Device UA223986 = {S56184, 137, { &P18074,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11CA Build/01.08.00) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA238616 = {S56185, 137, { &P18074,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11CA Build/01.10.20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA251099 = {S56186, 137, { &P18074,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11CA Build/01.12.00) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA331546 = {S56187, 137, { &P18074,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11CA Build/01.09.00) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA348577 = {S56188, 137, { &P18074,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11CA Build/01.12.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA348844 = {S56189, 137, { &P18074,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11CA Build/01.13.60) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1955253 = {S56190, 137, { &P18074,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11CA Build/01.14.50) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019560 = {S56191, 137, { &P18074,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11CA Build/01.11.30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2531117 = {S56192, 137, { &P18074,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11CA Build/01.15.20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2751919 = {S56193, 137, { &P18074,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11CA Build/01.16.00) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2431068 = {S56194, 171, { &P13123,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2469199 = {S56195, 171, { &P13123,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA17069 = {S56196, 66, { &P11897,&P17483,&P3499,&P18092 }};	/*Useragent - LG-GR500R/V10a Teleca/Q7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA64725 = {S56197, 66, { &P11897,&P17483,&P3499,&P18092 }};	/*Useragent - LG-GR500F/V10a Teleca/Q7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA64871 = {S56198, 65, { &P11897,&P17483,&P3499,&P18092 }};	/*Useragent - LG-GR500/V10j Teleca/Q7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2508179 = {S56199, 140, { &P21019,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E270K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2712759 = {S56200, 147, { &P21019,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E270K/KKAMC3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2713469 = {S56201, 147, { &P21019,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E270K/KKAMD1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2857098 = {S56202, 62, { &P21019,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E270K Build/JZO54K)*/
const Device UA2870015 = {S56203, 147, { &P21019,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E270K/KKAME2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3023226 = {S56204, 147, { &P21019,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E270K/KKAMF2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA353 = {S56205, 61, { &P11808,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia5610/2.0 (08.20) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA13405 = {S56206, 64, { &P11808,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia5610d-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA15336 = {S56207, 73, { &P11808,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia5610 XpressMusic/2.0 (08.23) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA62906 = {S56208, 66, { &P11808,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia5610_CMCC/2.0 (09.70) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA160497 = {S56209, 132, { &P15364,&P7133,&P22446,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Iron/8.0.555.0 Chrome/8.0.555.0 Safari/534.10*/
const Device UA2672612 = {S56210, 137, { &P19894,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2674847 = {S56211, 130, { &P19894,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2957259 = {S56212, 138, { &P19894,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830C Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3022954 = {S56213, 131, { &P19894,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830C Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2753878 = {S56214, 138, { &P22602,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-S738C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3131669 = {S56215, 136, { &P22597,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA189140 = {S56216, 49, { &P15364,&P17017,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; ; Trident/5.0)*/
const Device UA253770 = {S56217, 34, { &P15364,&P17017,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0;*/
const Device UA2509427 = {S56218, 52, { &P15364,&P17017,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; PPC; Trident/3.0)*/
const Device UA2791861 = {S56219, 57, { &P15364,&P17017,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; PPC; SV1; Trident/4.0)*/
const Device UA2437307 = {S56220, 132, { &P20473,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAB10-201 Build/Xenta) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2597311 = {S56221, 148, { &P20473,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; Xenta TAB10-201 Build/Xenta TAB10-201) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2986177 = {S56222, 93, { &P20473,&P18111,&P18112,&P18092 }};	/*Useragent - AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; Xenta TAB10-201 Build/Xenta TAB10-201)*/
const Device UA2878923 = {S56223, 133, { &P18464,&P18110,&P23483,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Safari/535.19 Silk-Accelerated=true*/
const Device UA33425 = {S56224, 41, { &P12915,&P17484,&P3625,&P18092 }};	/*Useragent - sam-r261 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA2406141 = {S56225, 155, { &P17324,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; GT-I9001 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483087 = {S56226, 170, { &P17324,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; GT-i9001 Build/JZO54K; CyanogenMod-EHNDROIX II i9001) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA239633 = {S56227, 135, { &P15364,&P17818,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/11.04 Chromium/13.0.782.215 Chrome/13.0.782.215 Safari/535.1*/
const Device UA339436 = {S56228, 135, { &P15364,&P17818,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/11.04 Chromium/13.0.782.41 Chrome/13.0.782.41 Safari/535.1*/
const Device UA2430623 = {S56229, 137, { &P18470,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ARCHOS 101G10 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2697239 = {S56230, 66, { &P18470,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ARCHOS 101G10 Build/JRO03L)*/
const Device UA2424319 = {S56231, 99, { &P15364,&P16110,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) iLivid/1.0 Safari/534.34*/
const Device UA2528334 = {S56232, 92, { &P15364,&P16110,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.34 (KHTML, like Gecko) iLivid/1.0 Safari/534.34*/
const Device UA2564196 = {S56233, 111, { &P15364,&P16110,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Superbird/24.0.1312.57 Safari/537.17*/
const Device UA2654927 = {S56234, 97, { &P15364,&P16110,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (N; Windows NT 6.2; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) Raptr Safari/534.34*/
const Device UA2665022 = {S56235, 104, { &P15364,&P16110,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.17 (KHTML, like Gecko) Superbird/24.0.1312.57 Safari/537.17*/
const Device UA2869890 = {S56236, 111, { &P15364,&P16110,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Superbird/27.0.1454.94 Safari/537.22*/
const Device UA2988711 = {S56237, 94, { &P15364,&P16110,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.15 (KHTML, like Gecko) Steam Safari/535.15*/
const Device UA3007712 = {S56238, 104, { &P15364,&P16110,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.22 (KHTML, like Gecko) Superbird/27.0.1454.94 Safari/537.22*/
const Device UA3024253 = {S56239, 111, { &P15364,&P16110,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Superbird/28.0.1500.71 Safari/537.36*/
const Device UA3044104 = {S56240, 103, { &P15364,&P16110,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) SuperBird/23.0 Safari/537.11*/
const Device UA3057364 = {S56241, 94, { &P15364,&P16110,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Steam Safari/535.19*/
const Device UA3078732 = {S56242, 104, { &P15364,&P16110,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Superbird/28.0.1500.71 Safari/537.36*/
const Device UA3065382 = {S56243, 140, { &P12841,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC PH39100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110839 = {S56244, 140, { &P12841,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC PH39100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA14231 = {S56245, 158, { &P13800,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia5630_CMCC/011.023 (SymbianOS/9.3; U; Series60/3.2.3 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA39773 = {S56246, 153, { &P13800,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2Nokia5630d-1/013.009; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA64611 = {S56247, 154, { &P13800,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia5630d-1/013.009; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA88449 = {S56248, 156, { &P13800,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2.3 Nokia5630d-1/011.001; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA24591 = {S56249, 160, { &P15478,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia6790s-1b/ATT.03.22 Mozilla/5.0 SymbianOS/9.3; U; [en]; Series60/3.2; Profile/MIDP-2.1 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA44088 = {S56250, 154, { &P15478,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia6790s-1c/20.007; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA7088 = {S56251, 148, { &P14604,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB200 Build/brwnMOD v3.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA7092 = {S56252, 139, { &P14604,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-; MB200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA27632 = {S56253, 140, { &P14604,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us;MB200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA27638 = {S56254, 140, { &P14604,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.1-update1; en-us; MB200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA54947 = {S56255, 141, { &P14604,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA65915 = {S56256, 140, { &P14604,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB200Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA65916 = {S56257, 140, { &P14604,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 MobileSafari/530.17*/
const Device UA65917 = {S56258, 133, { &P14604,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; MB200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA65921 = {S56259, 140, { &P14604,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.1-update1; en-us; MB200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA71211 = {S56260, 135, { &P14604,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1.5; es-es; MB200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA423653 = {S56261, 145, { &P14604,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB200 Build/ERD91_INT) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA550928 = {S56262, 63, { &P14604,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.1-update1; MB200 Build/ERD79)*/
const Device UA2531070 = {S56263, 156, { &P14711,&P5204,&P24676,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-01) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2975744 = {S56264, 134, { &P20213,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N9500 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA1991758 = {S56265, 138, { &P13991,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR6400L Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2040901 = {S56266, 141, { &P13991,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ThunderBolt Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2513341 = {S56267, 138, { &P13991,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6400L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA192937 = {S56268, 115, { &P15364,&P7133,&P22449,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML, like Gecko) Iron/11.0.700.2 Chrome/11.0.700.2 Safari/534.24*/
const Device UA193884 = {S56269, 122, { &P15364,&P7133,&P22449,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Iron/11.0.700.2 Chrome/11.0.700.2 Safari/534.24*/
const Device UA2946991 = {S56270, 157, { &P15575,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2947380 = {S56271, 157, { &P15575,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3015445 = {S56272, 158, { &P15575,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3016288 = {S56273, 158, { &P15575,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100T Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3031210 = {S56274, 158, { &P15575,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3059195 = {S56275, 158, { &P15575,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100M Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA1993858 = {S56276, 139, { &P20597,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F100S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2645613 = {S56277, 61, { &P20597,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-F100S Build/IMM76D)*/
const Device UA2822275 = {S56278, 130, { &P20597,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LG-F100S Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2680602 = {S56279, 74, { &P15364,&P5690,&P22370,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.0.1*/
const Device UA2718624 = {S56280, 81, { &P15364,&P5690,&P22370,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.0.1*/
const Device UA2454874 = {S56281, 103, { &P18463,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; SGH-T899M)*/
const Device UA2455523 = {S56282, 102, { &P18463,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; GT-I8750)*/
const Device UA2035303 = {S56283, 101, { &P18445,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 920)*/
const Device UA2427237 = {S56284, 109, { &P18445,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 920; ANZ821)*/
const Device UA2503839 = {S56285, 111, { &P18445,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 920; Vodafone)*/
const Device UA2597433 = {S56286, 109, { &P18445,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 920; Orange)*/
const Device UA2707941 = {S56287, 102, { &P18445,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Nokia 920T)*/
const Device UA2848906 = {S56288, 101, { &P18445,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Nokia 920)*/
const Device UA2671103 = {S56289, 133, { &P19763,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6602 Build/10.1.A.1.434) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676279 = {S56290, 140, { &P19763,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6602 Build/10.1.A.1.434) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2718640 = {S56291, 141, { &P19763,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-02E Build/10.1.D.0.343) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2622897 = {S56292, 61, { &P15876,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-P5100 Build/JOP40D)*/
const Device UA2735638 = {S56293, 131, { &P15876,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; GT-P5100 Build/JOP40D) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Safari/534.30*/
const Device UA2996276 = {S56294, 61, { &P15876,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P5100 Build/JDQ39E)*/
const Device UA3068414 = {S56295, 131, { &P15876,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; GT-P5100 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA64367 = {S56296, 96, { &P19968,&P17485,&P3976,&P18092 }};	/*Useragent - Mozilla/4.0 SonyEricssonW910iv/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA64798 = {S56297, 83, { &P19968,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonW910i/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2197870 = {S56298, 138, { &P15803,&P5843,&P24670,&P18092 }};	/*Useragent - Nokia200/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia200) UCBrowser8.4.0.159/70/444/UCWEB Mobile*/
const Device UA2738002 = {S56299, 98, { &P15364,&P5690,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (5.1.2600.3);en;kml:2.2;client:Free;type:default)*/
const Device UA2768516 = {S56300, 103, { &P15364,&P5690,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (5.1.2600.3);zh-Hans;kml:2.2;client:Free;type:default)*/
const Device UA2795270 = {S56301, 98, { &P15364,&P5690,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (5.1.2600.2);en;kml:2.2;client:Free;type:default)*/
const Device UA2973132 = {S56302, 122, { &P15364,&P5690,&P22779,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 5.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/7.1.1.1888 Safari/532.4*/
const Device UA3018901 = {S56303, 98, { &P15364,&P5690,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1888(Windows;Microsoft Windows (5.1.2600.3);en;kml:2.2;client:Free;type:default)*/
const Device UA3020707 = {S56304, 98, { &P15364,&P5690,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1888(Windows;Microsoft Windows (5.1.2600.2);en;kml:2.2;client:Free;type:default)*/
const Device UA3125475 = {S56305, 98, { &P15364,&P5690,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1888(Windows;Microsoft Windows (5.1.2600.3);de;kml:2.2;client:Free;type:default)*/
const Device UA6436 = {S56306, 134, { &P12460,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-; SCH-R880 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32094 = {S56307, 136, { &P12460,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SCH-R880 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA637292 = {S56308, 60, { &P12460,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SCH-R880 Build/FROYO)*/
const Device UA2027979 = {S56309, 141, { &P20124,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ZTE Grand X Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2346366 = {S56310, 138, { &P20124,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2423637 = {S56311, 139, { &P20124,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970M Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2454964 = {S56312, 138, { &P20124,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ZTE V970 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2535020 = {S56313, 140, { &P20124,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Grand X(M) Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2578100 = {S56314, 141, { &P20124,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE-U V970M Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2778708 = {S56315, 141, { &P20124,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Grand X Pro Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2879908 = {S56316, 137, { &P20124,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Grand X Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA195095 = {S56317, 146, { &P16352,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HUAWEI-U9000 Build/HuaweiU9000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA556464 = {S56318, 70, { &P16352,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; HUAWEI-U9000 Build/HuaweiU9000)*/
const Device UA2773501 = {S56319, 141, { &P17346,&P18117,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9B206 Safari/7534.48.3*/
const Device UA2800588 = {S56320, 139, { &P17346,&P18117,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9B176 Safari/7534.48.3*/
const Device UA2424687 = {S56321, 141, { &P18453,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT25i Build/9.0.1.D.0.10) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2754535 = {S56322, 128, { &P21769,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ME301T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3039906 = {S56323, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (yLr7EYRY+BPE6cyYr8SmFfIYdh8JYq1AqYls1MG3Xsk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039907 = {S56324, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (eXOS9+r/txvf7GWAR83bE831NJ1A0skcz/3ifI/hSRg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039908 = {S56325, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (WwP6TFR8dkb3nE2b3iz6/+6FvKRTq9DcAiq91iNmRRA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039909 = {S56326, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (eQgPy3Q2MPSoxbaOkDQ9rgyEnIm4AQzLO2w5u0yhK0w=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039910 = {S56327, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (rAarqQZ39qF9yogSsDjqVrcJDs77SqxPLdKD3G3RTVg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039912 = {S56328, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (e+JzrrzKBmWDkr+EEqA1qSoiT104X513i5C6nqA7rzE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039913 = {S56329, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (E6in5QOY71GVa82exQB8887za0VXlMNIevf4LRK0CBs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039914 = {S56330, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (UuCyn/MDeKIqEOTue1DYHMGflhX4w4+4cfNzdo9/ADA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039916 = {S56331, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (VyUcpeAR8Ai8spfwg6tb72hAeolwd7YD7IzHnQBYBN0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039917 = {S56332, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (vu02ioweEmbE1k05PbdvwLx/SYSGaWQhXdg3hlnQiKI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039919 = {S56333, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (wwzFDV3lHrr3kWautEAS4WWFNBGNIdsQ5Sr/xIy1o0Y=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039924 = {S56334, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (9nsdhJ/+S6cos8/qpISdjN34FLUX2gAPzMSYb+0S+p4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039925 = {S56335, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (9McWXSTeEmMMfA6OsMV9kep3k6cuVZuHY6NWIDd8I2c=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039926 = {S56336, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (g/GwcJbJeh+6nQ/8SRBrYyh23xhe/8NPibCGJRr9E1g=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039927 = {S56337, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (fC9aZpPd6Hn/OYBx2h81JRUl3rqmVvjwW9K9UqUEf78=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039928 = {S56338, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (8ENVDUFtaYrI+znmdpzFWFaKl5iYRPcxfQCK5iQ+Exg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039929 = {S56339, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ro7B4K6y6K5Rejh0OZPGHXzt2Vb8ilglPyaDOLXzgqA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039930 = {S56340, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (QkfdtMJ96CzPwvqRrtR+41uuFIWZeOaVmq+ds5tJ//w=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039931 = {S56341, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (QDsFFh/GywLoYHpNZElVJa1IXkbKb5uqLgqYi6JsZ0w=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039933 = {S56342, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (2VvZcHNQ0uUtnyVfS5XXpWpTW0uePqs8OMeQocRC54c=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039934 = {S56343, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (UMmnbVoi0nLSMEdfHgMFETIY4p7zr8Ss5bF+T3tJr2s=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039935 = {S56344, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (1j9Klv6ZovFXBK/nyrgG/iH+jY/bvSpyuNsuzCAL2kc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039936 = {S56345, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (oVaX3xu8lS6+dxqSTxJyCyJ+2By0urQwPAPZDhxZX8Q=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039938 = {S56346, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (659XPlOKKBiubejHMHV73+zxqMzVDxAZmvCKmHMwVgY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039939 = {S56347, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (RxDps3R7fTXYeIKbpiKNzqii48KMn3QVW+Id41TNiRY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039941 = {S56348, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Nszvs8ocD3cLViHON78UentvzX/lZBQFQeGypzznosI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039943 = {S56349, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (gzguGHikfSlBmx1Uvv5uki6ydx11N9G1XtqTJLgyJbk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039944 = {S56350, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Dc9XSTm++s9+r7kiONBrIAeCrm1I8yINUtZWOg4WxNY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039945 = {S56351, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (BiJcJ/NnWWNRCJbpE1IDAUARApQI13bBpXjUKIJ3NX0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039946 = {S56352, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (/fePx6eQbNOJc18VKF+tUCcbSw7dqUiyUeAWMr1Oe2E=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039947 = {S56353, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (NZhZ48QdASSwhLUCfDdPk788vO53KRHFxUX0yUzs8WY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039948 = {S56354, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (B4+CrD9j8S7HbuyOvQLVn3xRku3nUUZiU8mwvow+fF4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039949 = {S56355, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (HRfVdPGb2DXDoQv7j0jiZ7kumjupqYEZuuv+uUjYaXY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039950 = {S56356, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (EnLPfK3gBP87LW9Hd6JIkOZwJI246mpl+xkiH+y8vqA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039951 = {S56357, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (MAMz0qVOAVH1zmRrKL6LTwU7KrhI+1chexANAMUmFM4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039954 = {S56358, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (wwV7b+ehRKit4Z/Dcpf5Iyt+XWId08eJmLj0BeAM1ik=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039955 = {S56359, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (FgZ9ZDQJMtbyd+5UMDlLOlswH+0rh9Wzij5CW0gcD5g=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039956 = {S56360, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (27R4y1cxKja/+yOuL1iUKmJfspVxIpm4qgV/wcGnnng=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039957 = {S56361, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (emYYDLcpXdnGC4F7/lsxEyicjrvVS3AKJI6j/hO5EiE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039958 = {S56362, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (mEHSren5PNQEk2sVQQOzhnX0SPrYAQ6x+xwMaR4JcZg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039959 = {S56363, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (7zlzhARynJN0A3DR1n2rAVz6ogiv+TqieWs82IN6w7c=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039960 = {S56364, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (rB52Gi4rXs2Hd+lWYo6EBLZ6nlUb+W5M115D/RKUNpU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039961 = {S56365, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (/pvbdYxWRjdoWSNGyIEiz24A/TxU5NIpGCrI1c+6BO8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039963 = {S56366, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (0I21w8U5AvqqNw4mgaFumiiimI3WBMq9In3/oI8wSVo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039964 = {S56367, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (arfzlppEmoiA53tb+e6+XK5ub99P4csLo3yOjLfiryE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039965 = {S56368, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (5iKulOK5fSdkCAflu2pPnoc+mVMN1AGKenxJLFPYung=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039967 = {S56369, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (6PNbJqYPVfeNgfBReG2mSHUyWmtwktcmfHhI0M3t0O4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039969 = {S56370, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (EBwxSqWg5Wyih9g4o0X6Fnn3KtSCv25+Z+q3jvSY/4Y=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039970 = {S56371, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (MJlfzfmnC1aXm16FnVsEDvR7IPIhBO2LA7vphtTU+/Q=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039972 = {S56372, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (tn6Q5f9Ot0lhlMN1eraayuqNMoK2OuAKELMhxZQMzr0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039973 = {S56373, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (jYgJA+CQFZBTIh8fIZ8yjQSipb2lRoSHvR9EkMfGUKc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039971 = {S56374, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (9ueqGW6b1Fz57VroPB3Hd/4XUdOCNbcny4nMxcaCeE0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039974 = {S56375, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (04khbePl+otyC+t7Z1Pu2Rbap5uJ/LllvSrEAqSrJyo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039975 = {S56376, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (JfnV4ayfSahhDq3h8ZW6haERMx/9Vr37XmTYkOwRFC4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039976 = {S56377, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Sd3AqNoBhGZf18vsXJ0/8nH6XhxAVPOOtWHj+vYtE0E=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039977 = {S56378, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (7datnnrdtSacUg2wppZ/l12m0AoNxiaS++MDoWK56ts=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039978 = {S56379, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (/jRFM94SIl+0rwrScZlAfRpeVd1zX9LVAc31pppjmOQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039979 = {S56380, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (5JnbqYmhynI9cNOdpWUQdRtcgOmRf2qSRPfS2jgTqMU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039980 = {S56381, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (SY7Up89ngF6J8ZipjX/NoWXRdPkjrE9YRa4jUcMAnI0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039981 = {S56382, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (LmSzCLvkVtv8go56pkt0CddiEHAq3YDx2qKsCpola4o=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039982 = {S56383, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (eLxYEzaXKSGyVCtDG8d1ouh030x3nVxlEbl8rNaF8D8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039983 = {S56384, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (NYhSzv2V4fErCf970dbkWk1gTpEr/VQoLOcOKvbOAIw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039984 = {S56385, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (GBaT+dAKdb5ag0K1t73SVwzni3araYhmuJUEvFd+aLQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039985 = {S56386, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (102RA7gBLDVd58NEHncNUo8mdSgXueovHUHvzElkp7I=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039986 = {S56387, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (H9vuO55J/W/eNCjf7LhP6UHNesJkTSkpq/3U3wuSATg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039993 = {S56388, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (3OcK1N4hdsHIuNk6EOP87XJ4nHTQGDAiLkIKY2Bb3UQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040002 = {S56389, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (DfTxNfIc1SJWFW3YLg5NgpQppGO+uX/9ERxv4qLmHTM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040003 = {S56390, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (2087ant8VnOSsF8CFzumB4dSFXO1oFyqgTXTVG1EBz8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040005 = {S56391, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (KUyUZWf8ZTgrrfMVs9t1/AnFjVeflpi9+6XWpGMjI+A=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040007 = {S56392, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (1UuigJfJyxyQtlTlacrUKuXHRwy1OwqkhfAafGxOnoE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040008 = {S56393, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (F87GmG3U5UzGPls4nzXh8E0YdQDCbyzs7f3KnzyYCWY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040009 = {S56394, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (K+6qnOvOy9mHa8gypoOGCxZ0+9Q/XdToNVvpen9WhxA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040006 = {S56395, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (4blVetg+KABCZSS+P+rTlbEQ/omF/lPtkIZXO0oioAo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040010 = {S56396, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (MIMnM26aJxYcC7D6UcR/UbhgWJ7ZkH2LmIsIMSx2ZJM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040011 = {S56397, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (6Yw3OOeWeoGqtg/kx7EAagivd9LN2yOQ+V34bgG2p8E=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040012 = {S56398, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (+g+tKZzOq3m0Eugbze7XxqEERXmoJFBrjSDY0bcAZKc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040013 = {S56399, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (QOrVbdS+eL/r0Slz60PYt6HyAZGbaOIDfchqmBBpV6o=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040014 = {S56400, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (OIAQUKTlXuMbp8WFJ/ocZV18fzht3HjfhkuAmcorvLc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040015 = {S56401, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (MLXScUUnhU/hofqC9vs5OHYQ4TkagfNIkyUfE2GAmFo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040016 = {S56402, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (GnkUPP2lBJm/9lZYoP7uGAYRR0ms99IG/Q5qufwgfOQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040017 = {S56403, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (O/uMX6WZOdiKtdhCcmJ2uSeNg3Q0+oIwjDpAIRP3kNM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040018 = {S56404, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (PXapA5Z/MGQaDC7vjDXgexwbBdCzIkaopbjJAOsjbbM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040019 = {S56405, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Ojv/+Fixi9wu+PKtowPVhCpxwKZ8Wnx2KXh218/Uk0k=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040020 = {S56406, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Cz5aCNaTFtXXsKT2oFHQd3qPc+QLU0v4dHo1orRVM64=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040021 = {S56407, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (K4d71TAHukdLN+JPiJ11cONTHk+8LRVjt1q41xl4vy4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040022 = {S56408, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (F8mmnbiyVe0G9nvtIG8BSOM6dNSgqO9W4tJ7i9N3++s=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040023 = {S56409, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (I1LB8FMPASyXKcBjEBcL240oWtFG6JXhL+umWvuUcfc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040024 = {S56410, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Gb5K8kVWawT4LUxyaZLuen3KaKj8jtDJazcUmbrOWOE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040025 = {S56411, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (2rQeXPsC3ePKld67gekywWvCSzi71y7a/gNvje6OBok=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040026 = {S56412, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (EJifqhr2KN4J9xAlKJJdhadmpGTcwX8wR16iE7hNJbs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040027 = {S56413, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (60rM7YJMgxHqoCw68TlAJr5wr2bgrFIsvtEoy8jVELY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040028 = {S56414, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (QJe6dPVgINLoKdCHOq+uFSCo1D53BFcliDisLQrHUh4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040029 = {S56415, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (LhplWfaTuWxKYSG0sR7ZNNc5WYl1Aek4yz1jsXOPEMQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040030 = {S56416, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (6y+XVtkcmUcScnuMY4hWaj5HPWngNx88XntxfGm+3rs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040031 = {S56417, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (CXIaASHC25QDsRh4jhP3qQhlDSjFCtbcgeyZaw2BRQw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040032 = {S56418, 137, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 () Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040033 = {S56419, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (KNsE1EASYyeCh9D38j3CdBbjRSgp1qgR3XU0PzTLYCk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040035 = {S56420, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ESKbJrIxO/OM+TIVZX/LEyPAwKTZ4XlxhvYo5y6FP1k=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3039932 = {S56421, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (q2LVBpKdjbTyqvLqnu3cXX5gjjEx1Jq+0EvSmPd3Vpw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040036 = {S56422, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (GZTg5JsTFXdJASbcqxtJ4kc227JaAEjA9qCSMp3LpEM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040037 = {S56423, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (9hRsNiBmaMF8LcGaO6q7poecL5ltV06i1ehDP02K7xY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040038 = {S56424, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (HGAlINyHJ63WZpMfRSkYNnKwuad2cIqEK7jujdf8hn0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040039 = {S56425, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (MhomaEPz8o9QtTg/tWkYogXfAhagASQ8MA5iQzKAXks=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040040 = {S56426, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Dzon2AOH6VZ/H8TzCrtS+PkGAVMhxyeTIhm/Q11rGR0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040041 = {S56427, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (6Ax7j/13R6Z+KPeeg3aRWmnpPSgOnfx/HYFNi9rU+2M=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040042 = {S56428, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (P/JRMeIn4QwFCty0H+lik+8+BIbrthQEdtY89HAjZD0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040043 = {S56429, 203, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (PSjeKGWdCylYhWGfjYvF0dKDtfIL4BxWi9vri2hRbT0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2 AlexaToolbar/alxg-3.1*/
const Device UA3040044 = {S56430, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (HCp7eVfyKxXZNDQvdaBJRfFA7oFojwRgXl2rHa4JnCk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040045 = {S56431, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (OSN90Wsfxm9WWBpVchbma031XNL8JAPoCLlVDDzOeNE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040046 = {S56432, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (M1IESb2ieydVOooaIxiujqJ0nx0cCRZISR5PbPbwffY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040047 = {S56433, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (0VlR2JAtqxZ6fkq0Y7BbxKf/TDooTZhhpdMuzAEu3yo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040048 = {S56434, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (1XiV8WgCLfifNzZmhfphbeGZxNPI8JR8bm8H24JHXRg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040049 = {S56435, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (A2DQFIrA3vbr+RZxhUCaOI0Jh6xBymHa1zqg2pShAM0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040050 = {S56436, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (/NX7InNn6d3CRfo3caRNV4mqCQJtRcZm1LIa9kNCw3s=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040051 = {S56437, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (FEDdaLoNUMVKNv9SOf3b8WwTXYqAtJCYcZNmS5gj7ME=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040052 = {S56438, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (4m6sVCI4nP2PkRhoYwuo1oUTC8xkiI8OMfdMKaooMog=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040053 = {S56439, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (CZ2zxrlGsSGJMJ0K9VCK0WmASFXw539bhAd3I7sgcFM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040054 = {S56440, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (NYUqLfyWlXYPLOEAxB+0NB9op+as7RylykItSXUTzdU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040055 = {S56441, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (2kD9OO3QeDnxgxb1LsKxCtrVVY6KXG3NIe99Nala+nY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040056 = {S56442, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (1eGZWrh/GNRuPXlJ6gdzFytm+ofPtfIEje3w3et5jIM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040057 = {S56443, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (PP6xmQBlu5tkR2DPw4D/PGYsePZAXxHD8IMHOEI50lY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040058 = {S56444, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (7zeHZopFpbXZh+fsx0qv92Y120A/VprIybj2ogv4BEc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040059 = {S56445, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (GrB9prK+u1lhwLXPUHW9+h6ktavdko1Cmpq24Fb0dUM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040060 = {S56446, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (24IOq23G4tDKCeD+daqEDDbrkF86sWfSwi+pDx8iKnE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040062 = {S56447, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (IpQ478U5sV2cpUIQluc4yCEzMpy0TUG6UBzIvvsrCXs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040063 = {S56448, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (PFtdHkTvl+DlzVDiw97aq8a9hi+E53Y61rHRkoKMJtw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040065 = {S56449, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (0xC+aZXjvyX3z7vZ6WQchghPLa0zEU855ZHU9K75bPg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040066 = {S56450, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (6tk33y5hhueCSio09/ceEZwcEtv31GHeKbrx8GSD8sM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040067 = {S56451, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (6y4KDAUZ4Yeu3B0wudIC/OxcUC2/x6g5MQ01IVviMK0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040034 = {S56452, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (KJvmEcjv1LthE68S1zu1N/UIUYsE/ZDYLGZ5Gw4eRiY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040071 = {S56453, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (J/fOmXi1yuShbr0lH505R5fBBaJd7oFZ++SvxADvX8g=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040073 = {S56454, 203, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (h7ZC/WHQ9ZKuisFdHtvkNG9m5Fq4tBP+BVPku54TnqY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2 AlexaToolbar/alxg-3.1*/
const Device UA3040074 = {S56455, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (/Tt3hBeM8qsUV2uLIAg9pIXy7QvL8IBvhoxZVsTRuJE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040075 = {S56456, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (1HRiFjd9JskKRYL4nyP2KXm139CqELAxiDBxxGEWpEc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040076 = {S56457, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (E2v+q782bYW0z2LhHmbj5nbRRpt5n+NIiUNai9sV+os=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040077 = {S56458, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (JC+r/fEBSdQnnr/4V3OO2w971TESd/38xS1sOuRD9yU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040079 = {S56459, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Abow04eTqUYMazUU2A9MlFSENbcHAe9l2owj1V+oa3I=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040080 = {S56460, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (0o5+iTBLyJBsBjjbQUwsVcmGcuTqU1iX4fERm5FJV0Q=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040081 = {S56461, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (HSuiFJLlNVl1mLogg0+c1vXyQwvXI8WFvdYPp4OpdKE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040070 = {S56462, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (FlFrGmq8hPjvjnvAwYbvDUW1X4sEmLSAWUNGmLsZ6PU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040083 = {S56463, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (AkOL++mrzrNymKlj+LjvrDXGA20mHjXpK2CaEl/GE/8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040084 = {S56464, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (OD/YP4Evvmz5Q3QPJlFM5b7mb8mDdI+b/yED5cWl9KU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040086 = {S56465, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (NMPAH+H2Deh9P/6B3fPHBlUUE8Rb0A/q7exPmi3Ur4Y=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040085 = {S56466, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (IV9Hs5mQaN/R0K+zLm63tnr3ZJNQbeB4Dk1GQQNc2GQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040088 = {S56467, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (GqeZ/RtN8BLyxeYOHOxkaY4KRe8VeSSCcYwSFXqJ5ws=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040087 = {S56468, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (OGxyVI07puJEIltprVOx+nKUtFBW0X1nDLz6GIOU7TM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040089 = {S56469, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (08NM6fWO0spYbbrISyT640jdADTaDrcwt7LJUmRJA84=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040090 = {S56470, 164, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (--USERAGENTOMITTEDOMGVESS--) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040092 = {S56471, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (J3Nh6dtKZAw7Cxyd8WJ4+OrIdcooZ8AJlO4IDvCnkts=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040093 = {S56472, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (M6a+B/nOIHuCUB2BQqmNvoEviZIg+t2ZHrFWHXzK+7g=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040094 = {S56473, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (9D6Sa+VnLKjIy6fis+4kAk0XRz4VCJcB4HW9rPam/mU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040095 = {S56474, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (2fRebjeePe+IThJAi9oQVdnQFS9Kq724/uCmKHxzufY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040096 = {S56475, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (1hU8GbFimod1nJypd07LoGVb4Rfp0XSPzKRPjsmCI24=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040068 = {S56476, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Q3g5Tl3qaAFbveuPIfDyEUoQt4Nq2243JCb/v1x9ZWk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040110 = {S56477, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (JAMkPgy34jtg5uC/PTfQye1NRFZmn+kq6LJvRXI+l8A=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040111 = {S56478, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (1YWAElGXJ0rYPS9OR/1Y8e4X/ZVQbpRu/WZgXxoC+Kc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040112 = {S56479, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (HE/1yDoqWxsCUFZOqUsWhH9FzepmDyjWvn2MfkcgD4Q=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040113 = {S56480, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (6e5YUf22go55jLeYCrxeLl4VBU4hHaD1vd6RkSY0CUE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040114 = {S56481, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (xfMcA+UNukSoKxiQo4oK63DE5u5v+y6FWE3ME12jJoM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040115 = {S56482, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (PAsw5rddaZSz7ApwnjzKfYerpKku+1vEeA33lIeYJrU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040116 = {S56483, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Nnzjz7RTQiSfsQM2njoNSsyxiiWpADCyA+qu2iFqKSo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040117 = {S56484, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (H3Md1XIJnTbE+bu2rA68p1zm1RLiEeeTztPpy0c371E=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040118 = {S56485, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (g3kuz6HvwD1E+pHG1Nwxvc1pwnRrGwY8opYv4mY7fRs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040120 = {S56486, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (fhKsIQ7I0F/5bX2vyefYcXT+g29812oL5z5q7M4WQT8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040119 = {S56487, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (5JwzvCez8rvUtaD9kCenNCNNepcLraRJjgWelsjuYy4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040122 = {S56488, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (g64W3wbVizIE1HlD2HNCBMVgzXLIdbrQexhlzTdIzLw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040123 = {S56489, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (syHhPbBQAJ9JLQbRWYzB34hFtYYmA6BvGnWJ5TZ/ZAg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040124 = {S56490, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (XPokjOYxFHsUdRJ3O2EuYjHwfjohEnE9z01PffnbHEI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040126 = {S56491, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (o8DUVnIvuRUEqRNFk3Vi22zpr6r87h5RilbXlOl5JmI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040129 = {S56492, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (XkKA8LspQ7HAPCh3iWpgQ8bGpHzXRROGeMnxx60F5Mc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040132 = {S56493, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (YjVMepHWYwA48Iw9s1Wt71D3KHixEXQZSSaLao3egQk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040133 = {S56494, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (RO9oj+u2gYTRUfZifbLi9HK8Bk0GiQbZWK6TsLivua8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040134 = {S56495, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (xokYZOXYGKSLMXnZxQkcjAxr4E3a8+0X3xBe/nw/XKA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040135 = {S56496, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (jBSQpmdOVOtqd7mo8YyM6kWhFZK5tDpS4ZMcSkcmGLY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040136 = {S56497, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ZmEufAdvOJ6+jpzS29ugr8sbtxJwbNEwTvj+UjrNdn8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040137 = {S56498, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (rt4R8kudpZt4cdRTDJpP4Dml+DiDYOmChp3nTiVQ9us=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040138 = {S56499, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (vFojr9LdxbndzTN7qN3F42k6KXC7MpQtw5IRLvnZMwU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040139 = {S56500, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (nuS3iPfF+4oBZXsWfuw2wYSgxrH/3AxDEPP29Ubp/qM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040140 = {S56501, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (uehqSy5ycXDRBKP+jbwUI1YPVd1yTBKlCXkDiKHDrj4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040141 = {S56502, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (SOoQfsPbm1lAUwyU4IeKc7/J6f1R/CRljpFkYKaSSUI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040142 = {S56503, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (jggpP0hQtttQ7LSCMvnVZXoDT+E1L1TbRG9xP1kRKfI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040144 = {S56504, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (t7uBc2IWaOPWwcmDKIxzdAhY3o7s1/OSI9TrQHtDr8I=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040145 = {S56505, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (R/8lZYo/R4I9wlH+sCKID7do032Kv5VfatEvYt02tgA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040106 = {S56506, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (402ZbSXLcKvG/yLyMRNi4dNkuf9vKU7noCMU6FN1e2U=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040147 = {S56507, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (XT28UluKEur58RliQm9ffzA9Ed0PI9ZHkAaWZudRZQc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040148 = {S56508, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Yo0INpJdWDNv/vcPYCxyRy1H7gyy40OQW+o58kOLROA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040149 = {S56509, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (seO21gpOtp8uTe8EjZClilM5rgzyrlWqV92C7FPS+dA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040150 = {S56510, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (n0erFnR5T93tQbcRdc/KEZPIAMVtMqew1kqMHu6BIV4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040151 = {S56511, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (cnSggKKdlaO+LI2nd8S44ATEvyTvlMEScWLzZqKu508=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040152 = {S56512, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (qTjnaiLnccJjRdPHGnzmGJn9JEUIy6xlHH8iU6mzucs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040153 = {S56513, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (bTReBGAmyU0p9EZguECS3zebxG/O7ntBasc9XCGRZt4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040156 = {S56514, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (fA7uvGuHN3kN7vCweg7Zs3esLnIVDCJzy7vZPZDzq4w=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040160 = {S56515, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (t5mBoP0o3e1bIQMnWqz/r5moBZD074zuu0ecMlND88c=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040161 = {S56516, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (XbePOaXVC76N7gVPYtb6cDOKV9UmpBN/1T0lsb1CZTc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040143 = {S56517, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (VAGxVr3aE18uG9UGNhxfYQKo8MfRq62rziav6QZT9EQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040181 = {S56518, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (de25zPN9VnTBgaGnFSBrZfvLPthsE6zX+tHYaeeeyFc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040183 = {S56519, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (gzU1iOuRlXSBqvWxux/rXl70e13x6Y73FslOAlL67gg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040185 = {S56520, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (r60KQyMXvoJrV1J2MHgd/PaRSVOMjfujl16LDOj4fWo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040186 = {S56521, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (2STuB04lUCSCz+ci9Sh9LJc6f08JB8Rnvw6U8nyrOYY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040188 = {S56522, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (t8KTdxr3CCQJs0qCrxcYV8EfKDYQAn9KVgD6PdMFVoo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040189 = {S56523, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (jFStrNCKwLCtSNpH0kaAT8gmOTLStyWawS580arwYhU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040190 = {S56524, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (hTcsqC3sfa1UOOX2GKI3MWYCYllaTVDaI1A/hICTkOk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040191 = {S56525, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (z+fvGPOyIDB7AlyW1GAPcCji/Jny6v/TzN9NQqOAIiI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040192 = {S56526, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (zusPZowyBv5A5OSo+4DJD8FSIhUCdGoPm6C4OJ8yMRk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040193 = {S56527, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (xR36Tq9pb/2KNCkXC3Nu1HBrhQMzwBt+vlts4pxjmLs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040194 = {S56528, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (hQvjPP1taKyzkeKiRUPZ3NTlquLiYxP+qELySAzKmOM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040196 = {S56529, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (O7daEPPR2YhzQal6PXK9cr2EkanWTzKcqJRvh58/wgI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040197 = {S56530, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (WglbkfnoMaByhxZ1oJq8mu6cScKsBTVgnaJsO2yb+DY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040198 = {S56531, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (gJLwyb+WNaIFI2tk6C5dBKHyqjk2e2ena4NbpsbRcYI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040195 = {S56532, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (1Hh3oX+JxLNpJl6v5FCKXwE+1kcRk2Xl8l+7ZGPFiSU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040199 = {S56533, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (HtfMH4ztVEkcy+JAR7uq5dxwuGNyDc+BklZPJOWo44o=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040200 = {S56534, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (nnZZUZ+GX+YEpGIGlOJa5427vfm1tUJWFTEPhReqhZs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040201 = {S56535, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (HaeTNjDSX4UdHMWq9aS3jL9drc626A8mXIkMMny+Mso=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040202 = {S56536, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (a0limEdCixWcisKTaGJRl+qgelJUYrCO3lpRT0UZyjI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040203 = {S56537, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (8r8TI2g8Oosihh+RPpg4HTUQ3JsSWh1I6GqrKsFVUvg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040204 = {S56538, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (4zEpNsdsk56kxPCiX/fn/exXDBjxEaJutLKRld+tnsg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040206 = {S56539, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (k2d90rHgrieCxWZ7j7LEwgJtAttkMnZE2P+0KY+tAnI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040174 = {S56540, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (yNfKQihQ3vhQpjkr4G538EERM2gOcht/OpHxMyECjjY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040207 = {S56541, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Vn9aWhTXlKjYxWa1AWKrDjQV4GolAIAA337m5tXcGPo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040208 = {S56542, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (QdGHGwV6DDLxhFQ5VF1VBo6UHURoMyWCYOYFqlg8icA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040209 = {S56543, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (kUTIS1zqXnVReNICAdbwrjEMW5ZfYPu1ee+TBmvCDuk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040210 = {S56544, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (uIrp5lLCuxa/l8XOMZgYRvQmAFNWx7Ra4iNpD1RndyE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040211 = {S56545, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (jtol5VG/jdIaur/ZLzPGeTwP0rwaQ1HlN3YGEguVM0E=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040212 = {S56546, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (CJZKkKysm/LkU+fZlIfebm7a9VNMuopZW8bqizzu1lI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040213 = {S56547, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (iffEFlsDsdqaBuxLflOhK0ZAKZdV93wIQ++3QmooXvY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040214 = {S56548, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (dkywy/UnKmkuNFgW2Aaoh+pQkL2+hgY0KnwsezA9PYg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040215 = {S56549, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (w7A7KBPuA1dSEatlDUAw2vfxuwXIuBUClMa13UCb5OU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040216 = {S56550, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (o+FY3XzStuX79VZqANaMtweaN+NmEO15VbhkMaMkAVg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040217 = {S56551, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (0+Zhtkn1WHTLRgjrx5iFmgjETLJ4PrLntWQjqzfxU00=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040218 = {S56552, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (XhiVp9ukLvJhUkb7XXwouDGXbNA+JHdu5Lm3x79INNY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040219 = {S56553, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (eAzY+kYN0idgz3M1TZsokaJ1cYrypz8oiwQaYqFHcR4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040220 = {S56554, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (JbfcMZkRZLHrEVQewXKvYc7wFwc7MUNRwhQqL1BXcys=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040221 = {S56555, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (x3958fvVfFCU21B657tQahv9dsJWT9nbCCztNA43fKI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040222 = {S56556, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (M3hkW9nUg6Dp8U7nLeE08moI2QB7qI/hkerVZ364Pw0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040223 = {S56557, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (VMipwgxElVUUmp0W1xF5HMIRPfgiOpvnuUFIWGElMiA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040225 = {S56558, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ekLz1M/BXVWgI0h9U0OQLG8D1dQihLpRf5wxUVxHV/g=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040226 = {S56559, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (YJhmyxXWiDJcbjw5ehHYTrZ24rSkO/qPAWVgBYeBRls=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040227 = {S56560, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Yw7CXMh97LNm2m2Avt/kFEczoO2FO5Ti04gpAhzIqSk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040228 = {S56561, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (VrodkL6YAZXTdclpYFNRzqNAzvnqKUGpnrtNEcHQiLY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040229 = {S56562, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (X4CqHbr60EuXxPPS5gLxgDnOSuuM8kkrHItBDazl9qY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040231 = {S56563, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (rxKhWmzaHoxTw/h67C+jnzGPXhvThAo8hQl/0UDc3Fw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040230 = {S56564, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (kQ2Jmk0yidFt8RYmQK5Owzr+Vs2qrPq6WpcUhJZMahc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040232 = {S56565, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (5zWCDjxZBKthbAcjYKkCbjb4p13zjVOwN7wRNmt2tKk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040233 = {S56566, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (fqpxMAp4qobzduuLJD1YDLgCYhpjzwBtd6CAKRgP0SU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040234 = {S56567, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (aQkIWuVoJXWrBe68DMKAc1iw/iekakAOvLpsk0zlkck=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040205 = {S56568, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Z1LEme1Oit3lvLHyrOwekP1UyjSj4OzR7xoPAuHnmfo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040235 = {S56569, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (IOyPaUr1QAPbOxHyJsvuGAzfwFgMiroZPdN7bEvJpas=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040236 = {S56570, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (RbtcTs/JkhUvpTymuH3B9PRX2yQLJLm0x5kl13fSinI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040238 = {S56571, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (QRvfESr+Z96Hj/zLUrdHRgvv+b7YAKJoHlt4Ib2Nhz0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040240 = {S56572, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (TNk5OrHC1cdBwbsxSf9GDYba1ebTTeB0Lv91smUY0BU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040241 = {S56573, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ws5oNBzDH1lFhC4gYNEtj68K0ZiorScjR7mBwJdGHco=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040243 = {S56574, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (YJxLL7kX0A4Or/0URhJklkH+e4kL6zwjoChsP/AYteg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040242 = {S56575, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (tmcE6pNlnWk/bsOM7/ghgQFOf6UC9tGzNg3xgrBPB0k=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040244 = {S56576, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ZzcZ9AYfEOcJ9lhSYF9ZdA3wxO7FaGsplEU1IZWRH/Q=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040245 = {S56577, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (hBr6ukyHIg1eYJTvD8htVXhrUvsfOu89RV5J7bE6yK0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040249 = {S56578, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (rwOnC8kMMVUR72abFcdENgIXAxvJ4PSjR6536Ap0pWg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040250 = {S56579, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (aQz97QYZtsMgm5KZZqvKzxyXvG4yuecbUC+HENETmaM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040251 = {S56580, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (TC0JkyfLsQ88z/VhLsmfJsVs8ox+9y/8X9lg+iiM47E=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040253 = {S56581, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (k1lo7MMfGRzz+H9kX0m1ErWwKP6WX9rAl1+BEAT0HxM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040252 = {S56582, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (SHVSI35bhyK4PTHi6tmODwin+msZjjYMJ0VZJYGDF1U=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040254 = {S56583, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (o8u5P/w47yiT9EAgMQTYbosdzSOTsLwW+Jodpmz0mIg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040255 = {S56584, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (UigEF+mmriU2Mz/7g2pS545JqUmoJHzl1DTo+i/EArY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040256 = {S56585, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (yRb20x6RvBeMpZgTV+JJ41tbBp51y2q/uvIlx+OsQXI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040257 = {S56586, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (c/mSum0BxDohjtRBryehWKdiFpjkT7uOUL5ndY3KN2M=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040258 = {S56587, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ZBySmGBP70JM5ZAqpAdVILhGsw1KxaTJuifqSR7FxyU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040239 = {S56588, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (GMFsA4xDuwpFwtioxE5wjlqKVF18n1xHlxKo0Ys03Jk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040260 = {S56589, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (WKKNEiGPExM+NZzLS3xwOi2UAiCvCuH+EZWhrH36F8w=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040261 = {S56590, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (unmw3fwrmzUnYwK6osi874tmNS4FQEgXz+BM20YXnWQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040262 = {S56591, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (umzQg4lvRq0uV3OeMw9GXajP/1qhhS1umsC93VoFHOc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040263 = {S56592, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (aSsd9DXCDv1IuI9SaZ5OALZTXvXyV+pmJoD2fZ/QEEo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040264 = {S56593, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Z9ayOgYdpkziBpFWAHurb2+NdhUEMkal0Coi48WhYSg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040265 = {S56594, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (5sCy/OHleEkDvQA7JlBWNnGm5LTU5eOLa5zM3ojgyHk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040266 = {S56595, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (qhZpk3sOWiUQtaqqf/xeZbClL8ldmNsFneCI8zp87QY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040267 = {S56596, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (+LXkPnc/5BJU4J4T9RUDaEgikXxIlIJg8343auYBxHM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040268 = {S56597, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (jgyvZY491AZPD+uUM2Pd0i4hLNZWXHX9buI5w8Mi8co=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040269 = {S56598, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (6kbLVUoR17n+3ZtBm/D/QiBKuOGhC7co4TuOtdJOqjs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040270 = {S56599, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (iCKPpRbdNJgnRPe/onFHWYRI+JFtOSS02ggdX5embdg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040259 = {S56600, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (yF8vrzhY/PIrceenqGk9VxD9j4CrunW/Xo4a7cr6gF8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040272 = {S56601, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (QdJ5XIU6jKRGUoJffaSuKszcaZvKV86NGLZyebgBWhs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040273 = {S56602, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (V1ZZpCcb9C9BO5DutYZMhcgpgKXaOyqs7wZF01Vpg+4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040275 = {S56603, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (nYIctuRB/beFyVtOo9EwVcX1/eSHY36Pcc9ZK/F6mzI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040274 = {S56604, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (oSFMNOWJ7mee1o3PNlu10IDyy3flDA9esJ/3OXAKdeg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040276 = {S56605, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (UFaPbkUuaQOsgoDza0t7vrvV7hcA3g9Hj4tSHymtEJw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040277 = {S56606, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (xZ27BdtV8deE4szZoEIQjShfFfQIpNVCawxEKsfokQ4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040278 = {S56607, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (1g6f596zPKJ5ypdiLPyAiRnz+JdQJLbtqC+5Mgi+wFk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040279 = {S56608, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (rqZhywrLlRmI6qan4QYva0VIv9pNu29JPwgQ3VhVnlU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040280 = {S56609, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (z6t3OSr4heg/B3fj1rYFX4jH3ShUYvzxcuq0pNSPAq8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040281 = {S56610, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (7lvLBTtP67zeagboNKTYLkipG7bSruoFQr2Wz3nsLWc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040282 = {S56611, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (cVXDIxup5VPssh1LjEq2w4bQxzKLIVWsBSNmPTkYZyA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040284 = {S56612, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (knZfiemMM6RyO9NlESomR2atx1NZTWArdUMoQO2D2wA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040285 = {S56613, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ku9IcPmDzDD7Jto9RmYJJ9+wxVwgQdfYf3pUtTJK1F8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040286 = {S56614, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Wo5/LgfN39aUMbvU8NGie8UFL7Zf9lAv6UpvjEHn6t8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040287 = {S56615, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (meNfYiy6I3ppmw9xO1YdLt/65DtxIg6HxfjVaeSNgTo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040288 = {S56616, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (u2Z4xCgkXKKJgGda3H6R1VbXZMkH+zXH5YdYURonh/0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040289 = {S56617, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (yMcNLr07ESmuK0cFsyM3Zd7Gf2T+bweeGAlc6jXz/Zc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040290 = {S56618, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (lWfaUqAQOJMuqvzUqT9VuXt9zmTPlhfNotkj/HLjPrs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040292 = {S56619, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (XT2dowopqZqR8BylfK4xdBjMa4skVqLx431RySaPQbQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040293 = {S56620, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (aNh9Fm6zp7LXk9aXx1RHU2fQrtgv+YzFi6fW1JfM7H4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040294 = {S56621, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (hxaQY8SX8wbwE43zLRqXWDvqJXsJM89gNxRqWy/TxKI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040295 = {S56622, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (VKixsQPwnrpBozTTnkCTM3GrVtsL7a+mT9Gyk8jkqcY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040296 = {S56623, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (RxBH3wYjb6Sly1R+1xH14F0wxWgTxcLI3INNl2shFOg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040297 = {S56624, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (pFriUtpOaEm9C4moKeOBbGau7kSnh412mKWf7QG8E1s=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040299 = {S56625, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (edrtrjeUeJ/HBNh0MpeJ6QRATQPQ5zSVkFZ8BtDcquI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040300 = {S56626, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (mzGgN/HzNhhgcvWYTn7WA7LyCbALi8EHImgUE1eIqNw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040301 = {S56627, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (UnvLVuRInG8yeNZq/pppEmui6kHjzlGTb/58O/trQQ0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040302 = {S56628, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (iV5YOqAYc6rvNVSKpp0NBNV91yqFt1Mx6Fm4wk6SnnE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040303 = {S56629, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (hTEc1eA2BcEa3yh/c2Qxi8jOWzcdaD5lI+mvSto3zsc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040271 = {S56630, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (321hm9c0TRNP7JOZEHRzLLxKdfRoD3NHAZ+x3kmS3YE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040308 = {S56631, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ef5l5ZXucdFpSpM5gtvZHnL6gwLIdKahXdXzPyufCrI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040309 = {S56632, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (zfVFJuVlanQy+j7b5igmxdrQBuM7tLLoyB/15AnGSbI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040310 = {S56633, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (dvOQjkz7JIYiNMyng/VswHoLQitlbOzxfgkwaqISpFE=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040311 = {S56634, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (sQYjERz1ubvf65h3weMYRRjvhbcmcr6ag/WtrJyeXrs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040312 = {S56635, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (u9QzCRtRKC6sc2ZwyEgxahuVpobPf1KT8Tv30x2xIaw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040313 = {S56636, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (0ThAtahPUpxeDvmtAfpu7xqmR3Wwga7KCQ53WWxOJ6M=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040314 = {S56637, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (W/ZfIKAvgoX0iKDoanNFSQ4ACAdqbGggBE6bPCMbXZw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040315 = {S56638, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (guASzxOkmpjI9kKu7RwJpgxQiJ3TYboX8iK1bzYJd8o=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040316 = {S56639, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (WycBmyeI9G4eXQcHsrILjwZzEi3rHMPr4R2UQIpjw6o=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040317 = {S56640, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (bym/xt2RJ8DKOe4+SZX6g1RU7uZ2HTr+ZBZvREFub6Y=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040318 = {S56641, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (qssOqQLCTdajZgLd7qBK74liefWIj4LdnbH4gdLZwlk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040319 = {S56642, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (h2Fzqc2mOkW+AbtBovdqFFsqJpddHF3tTZH4n1TpMv8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040320 = {S56643, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (feJvHBLFG/ji20JUMOg/s+nAZZGPVC0N55W+RVEOHGY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040321 = {S56644, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (wOEMDNSLp57AsSjYXxyOpqo9t30vj8mQCFTAiZ/0lrs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040322 = {S56645, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (UK6V7Py2kcVw/dnKbuymGiNThXGxPoNf0MfUd5E0K5Y=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040323 = {S56646, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (XslUDt22kn7C4dJjfK0MegbfsuCzRALbq2DN4Ak8n7Q=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040324 = {S56647, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (mmq2AxmpN6mTMn6dp5NSJ7jB+K+UrXVCF1weHJXwOKA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040325 = {S56648, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ylmEa52W207SG1GnYzMlpW/W6ng1RFzD+I7wxcuPKuQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040326 = {S56649, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ydp6PmWhcHtHxSicI9MrDBi3hJ4bWjQvsJnmTiFeosM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040328 = {S56650, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (JOTkjMjEbQDYvULeoLEl5PRAXJ0sWDZI1gpvWEufgiY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040329 = {S56651, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (jwe5xXmWM+FABSn/IX52P5t787xuw1HOrpnGICks5Ig=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040330 = {S56652, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (jwD2y6YFkctosT5FELDx07yy1LfQUWIvcaRaHNV882U=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040331 = {S56653, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (n16iuKG0oiIZnNtFgMy/mOWoqu1fpgvKVGcJqPYizhk=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040332 = {S56654, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (wtOEhI1rrZiiNRw5yqCC7E77WKPb191krsNJXz4Ewns=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040334 = {S56655, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (nk+7B0vEnuz9VvsyAkLTldkqNT8gyT9s2jnHz5TJZ5Q=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040333 = {S56656, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (VeMx9Kg9TSc6N+NLIDDJQk+yAs2Jc88hDOAWC581Q9s=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040335 = {S56657, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (YfGJjXRFhbCAoZyY/ZjJxzMXEnDXZ8i/UufR5exOpDI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040336 = {S56658, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (i4DjSJEBwdLJDDtV37i4IpGJXfG7l1GcnLuPNHSOyBQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040337 = {S56659, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (kSamzH/nGjnKspyMoq+MPnFVFkSdI+FUde17GDT6ReQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040338 = {S56660, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (vHypc+b66opo5xuvH61R/KYC1CgrxNFSSnAMWuJR+Dc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040339 = {S56661, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Qs9BpIh4b7R5/ukfRa6di+OpQmwuF9qP8XEjo8ajS9E=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040305 = {S56662, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (g6nvFfchIynZi8ANz6XPf+CbzWUMVLJqJcEp+hN8JyU=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040342 = {S56663, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (p5usOvJvGacb2+FJcN3rghOYHJYwhX+I17yMH4Yx6Vw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040343 = {S56664, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (sEs/QgeTfo2EUcNTXfMGw96mktdIjjns6eMcXPzoLLA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040344 = {S56665, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (f3ps+xVnjEglJhFnHdOHBktKl4Qxxl+w8rUrkP4Rzbs=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040345 = {S56666, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (VO3Cbt53bR4mF1f/KXCifMsVISUWSMiP5HnS+7W7Y7I=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040346 = {S56667, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (TvD5+NJC4hh3xHHnjJnpt8Qe1olxdCk8mJQ9D+lsjOI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040347 = {S56668, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Uqh8UtattBH59jvLUNGh4NMNm8n3kbZ4BJkSZdeOWg4=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040348 = {S56669, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Wrc3hJH1W1SeI011OGGbYem0vXXKezLwAYmzDIcEEsA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040349 = {S56670, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ryJBKeu+uiLV4QGqbHOToZWHPvVlbJJXgguWpuINYr0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040350 = {S56671, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (iX6d3X3xDayNQLOZwAQiEIMEp4QZYD9P6BQ9jzQ4ZeM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040351 = {S56672, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (a5awpctAWy8CBe3R9TQKINjJZ7kwi/vK+qc8jRZTTiY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040352 = {S56673, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (MsPsoLwb2Ir3YkLDG0+RDOlxNCEeQUSUQHcS7+G7Tlo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040355 = {S56674, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (iEd5/mrBMp3jquOCBoKF4chHs0LdCG9clqZ+PMIDmu8=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040356 = {S56675, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (FoXF9MbUulGnnbL+qVXRpDt9NfLHG5jgsokcwJDkvqc=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040357 = {S56676, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Y6J0A4AypvhLrYtEDuplyc/ynkWaSb7/uFFTeDesFfw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040359 = {S56677, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (+vFlTvo3odzKW4iJxF+oWfftiUgjiSVun4xBZ+hcJ4g=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040360 = {S56678, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (kZs/CGP3zxi2UauiBWnApUogcOo8hj2GUJ9Y4VZYek0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040361 = {S56679, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (sPHxgDKpO8H7xz7UJx+863KHtKvj0XxHSH5oNWkOt+Y=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040362 = {S56680, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (TfYrHRyrSD4Kn2M0+GHEmoJTgX6OLxy4kW1ZuuQLIDM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040363 = {S56681, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (xgdGEK/fdvz2DGSE6Cf120dXoeFmqXKs/HQR+fraLIg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040365 = {S56682, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (agP4pt2yF6VEAl/KmMDs3f1CXhclqRUfJNuiR+MRv7g=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040366 = {S56683, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ihqEMjN167zr2GmPJWGvZCFgJH/tSQBYSmR17H/gkyA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040368 = {S56684, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Vy2KMyWEwGGvp3cl/rtuqCv3GP+HgkVPsX958DCddZQ=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040369 = {S56685, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (eo6cGIfmNqYAHrPM09mzURU8F/OcgdJuC+LA96Z73uY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040370 = {S56686, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (SZeodRuzUSRiq9cM5cnj8UCpB1uBuIiqyXA6fqD+ieI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040340 = {S56687, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (SPU+RE2VQzssJ5t5NWWLsxt4x06naEd0rksRNlcG7jI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040374 = {S56688, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (dUMw/7yTgSAJ/xiEUvTtzrkGWfo5Xctzi8sRdllzPpw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040375 = {S56689, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (3Qfs5yafqd3YgrMNJbg2WHLWmXoeGjurTwEsdjqo1mo=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040376 = {S56690, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (hILhIG4TGDutA2R4AllG0WfhAxA9J0OdjZGDgjTAvWA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040377 = {S56691, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (UPtiktRoWcoIMGlKY3cW/HdQR+kiog1z4OVVSu03eaw=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040378 = {S56692, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (SCUcFEIJSvh5LwRRoCzbzZDWYDIFqr3QsfHM7TATKlM=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040379 = {S56693, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (sml52qM/QbK84BElcdbkmKv9EsJ3w8gorvqi+VXblNA=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040380 = {S56694, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (LtGWyc+Jvdn7blfLvV1xn3l/Ak60aKawjq/pNVDPrZ0=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040381 = {S56695, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (5b63Fxx1SW/mYX4ICILj5mcuri4RsZCX0NwYVs4/p2Y=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040383 = {S56696, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (sKBlqb3ROanUFyNonRs/B7i7iApx2iN1aOE8r0n9ULY=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040385 = {S56697, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ztiL7JhKyXAD2V+3UOUlRJoNKMoCLO9o3ualW0A3AyI=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040388 = {S56698, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (qsClFhjoEOQSmG/K9NJNCM0QWiAyyK4u3mc1rIinWEg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040371 = {S56699, 181, { &P15364,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (31JHh2fQ2msj2U0nKQqQqjMuJr1f6XDxBpzeELAc++o=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA616663 = {S56700, 148, { &P18114,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  Transformer Prime TF201 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA890755 = {S56701, 147, { &P18114,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Transformer Prime TF201 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA910040 = {S56702, 147, { &P18114,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Transformer Prime TF201 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1991867 = {S56703, 129, { &P18114,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TF201 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2355446 = {S56704, 131, { &P21106,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID1045 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2511262 = {S56705, 131, { &P21106,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; MID1045 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA56092 = {S56706, 96, { &P14637,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; LG; LG-E900)*/
const Device UA56769 = {S56707, 104, { &P14637,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; LG; LG-E900; Orange)*/
const Device UA67242 = {S56708, 97, { &P14637,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; LG; LG-E900h)*/
const Device UA90514 = {S56709, 94, { &P14637,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0) LG;LG-E900*/
const Device UA2676056 = {S56710, 126, { &P15869,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Xoom Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676487 = {S56711, 126, { &P15869,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xoom Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2887912 = {S56712, 154, { &P18465,&P18066,&P23483,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2388457 = {S56713, 83, { &P18141,&P19824,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 2.01) AppleWebKit/536.26 (KHTML, like Gecko) Silk/3.2*/
const Device UA2406007 = {S56714, 140, { &P14225,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Desire HD Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2536879 = {S56715, 140, { &P14225,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Desire HD Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555629 = {S56716, 62, { &P14225,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; Desire HD Build/JOP40D)*/
const Device UA2560450 = {S56717, 139, { &P14225,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Desire HD Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2704701 = {S56718, 61, { &P14225,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; Desire HD Build/JDQ39)*/
const Device UA3073447 = {S56719, 140, { &P14225,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Desire HD Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567006 = {S56720, 138, { &P16362,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602806 = {S56721, 131, { &P16362,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667358 = {S56722, 138, { &P16362,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SPH-D710 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2669917 = {S56723, 142, { &P16362,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710VMUB Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2672337 = {S56724, 141, { &P16362,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710BST Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2711814 = {S56725, 109, { &P15364,&P18110,&P22527,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux) AppleWebKit/535.22 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/535.22 Midori/0.5*/
const Device UA2862011 = {S56726, 54, { &P15364,&P18110,&P22527,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux) AppleWebKit/537.6+ Midori/0.5*/
const Device UA2891922 = {S56727, 67, { &P15364,&P18110,&P22527,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux; rv:2.0.1) Gecko/X Firefox/4.0.1 Midori/0.5*/
const Device UA2235000 = {S56728, 76, { &P15364,&P22725,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; Opera Mobi/1; MeeGo) Presto/2.11.355 Version/12.10*/
const Device UA2367409 = {S56729, 78, { &P15364,&P22725,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Linux i686; Opera Mobi/46154; MeeGo) Presto/2.11.355 Version/12.10*/
const Device UA158270 = {S56730, 144, { &P17950,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MID7015 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA172571 = {S56731, 145, { &P17950,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MID7015A Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA458645 = {S56732, 66, { &P17950,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; MID7015 Build/ECLAIR)*/
const Device UA458646 = {S56733, 67, { &P17950,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; MID7015A Build/ECLAIR)*/
const Device UA458681 = {S56734, 144, { &P17950,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; MID7015A Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA458873 = {S56735, 119, { &P17950,&P5477,&P4861,&P18092 }};	/*Useragent - Vyrso/0.9.3 (Linux; U; Android 2.1-update1; en_US; MID7015 Build/MID7015-eng 2.1-update1 ECLAIR eng.enzo.X.X test-keys)*/
const Device UA2204701 = {S56736, 95, { &P17595,&P17297,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5; U; Edition MAS; en) Presto/2.10.229 Version/11.64*/
const Device UA725826 = {S56737, 80, { &P15364,&P5690,&P22304,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT   5.1; en-US; rv:1.8) Gecko/X SeaMonkey/1.0b*/
const Device UA1842952 = {S56738, 83, { &P15364,&P5690,&P22304,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.0.4) Gecko/X SeaMonkey/1.0.2*/
const Device UA633210 = {S56739, 143, { &P17974,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; pandigitalopp12/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA624169 = {S56740, 72, { &P17595,&P17297,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:9.0) Gecko/X Firefox/9.0*/
const Device UA1938929 = {S56741, 90, { &P17595,&P17297,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:9.0.1) Gecko/X Firefox/9.0.1 Firefox/9.0.1*/
const Device UA680043 = {S56742, 159, { &P13815,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Wildfire Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1974502 = {S56743, 142, { &P14540,&P5843,&P24670,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiax2-01) UCBrowser8.4.0.159/70/352/UCWEB Mobile*/
const Device UA2367029 = {S56744, 186, { &P14708,&P5204,&P24672,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5130c-2) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2424819 = {S56745, 180, { &P14708,&P5204,&P24672,&P18092 }};	/*Useragent - Nokia5130/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5130) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2510757 = {S56746, 140, { &P12280,&P19826,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 Safari/7534.48.3*/
const Device UA2510766 = {S56747, 139, { &P12280,&P19826,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Safari/7534.48.3*/
const Device UA2510773 = {S56748, 139, { &P12280,&P19826,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 Mozilla/5.0 (iPhone 4; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Safari/7534.48.3*/
const Device UA2533039 = {S56749, 142, { &P12280,&P19826,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3*/
const Device UA2536834 = {S56750, 142, { &P12280,&P19826,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145 Safari/7534.48.3*/
const Device UA2606936 = {S56751, 142, { &P12280,&P19826,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3*/
const Device UA2796249 = {S56752, 141, { &P12280,&P19826,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3*/
const Device UA88559 = {S56753, 112, { &P15364,&P17017,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (en-us) AppleWebKit/525.13 (KHTML, like Gecko; Google Wireless Transcoder) Version/3.1 Safari/525.13*/
const Device UA1841244 = {S56754, 90, { &P15364,&P17017,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (SaferSurf) AppleWebKit/525.19 (KHTML, like Gecko) Version/3.1.2 Safari/525.21*/
const Device UA2234845 = {S56755, 128, { &P20556,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; A701 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2552695 = {S56756, 57, { &P20556,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; A701 Build/JRO03H)*/
const Device UA2388268 = {S56757, 132, { &P18061,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; sdk Build/JB_MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426756 = {S56758, 125, { &P18061,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; sdk Build/JB_MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2482795 = {S56759, 139, { &P18061,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; google_sdk Build/JB_MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566236 = {S56760, 136, { &P18061,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; sdk Build/JB_MR1.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2601078 = {S56761, 129, { &P18061,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; sdk Build/JB_MR1.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2732857 = {S56762, 143, { &P18061,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; google_sdk Build/JB_MR1.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2765611 = {S56763, 58, { &P18061,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; sdk Build/JB_MR1.1)*/
const Device UA2962070 = {S56764, 130, { &P18061,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; rk31sdk Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2988627 = {S56765, 161, { &P18061,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 BaDoinkDownloader_Android_2.0*/
const Device UA2990497 = {S56766, 130, { &P18061,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; rk30sdk Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2566894 = {S56767, 138, { &P18559,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LG730 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667052 = {S56768, 131, { &P18559,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LG730 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2586622 = {S56769, 132, { &P21604,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; C2105 Build/15.0.A.0.33) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2864624 = {S56770, 141, { &P21604,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C2105 Build/15.0.A.1.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892414 = {S56771, 145, { &P21604,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; SonyC2105 Build/15.0.A.1.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2895202 = {S56772, 141, { &P21604,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C2105 Build/15.0.A.1.36) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058097 = {S56773, 63, { &P21604,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; C2105 Build/15.0.A.2.17)*/
const Device UA3059532 = {S56774, 141, { &P21604,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C2105 Build/15.0.A.2.17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3080343 = {S56775, 145, { &P21604,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; SonyC2105 Build/15.0.A.2.17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2706874 = {S56776, 165, { &P22599,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E300S Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA869373 = {S56777, 129, { &P20127,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AT270 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2788448 = {S56778, 58, { &P20127,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; AT270 Build/IML74K)*/
const Device UA1995940 = {S56779, 141, { &P17172,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Transformer TF101 Build/MASTER) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2030832 = {S56780, 135, { &P17172,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Transformer Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2232485 = {S56781, 141, { &P17172,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Transformer TF101 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2237778 = {S56782, 135, { &P17172,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Transformer Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2566823 = {S56783, 138, { &P18301,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS840 4G Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2867397 = {S56784, 138, { &P22679,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955943 = {S56785, 60, { &P22679,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P710 Build/JZO54K)*/
const Device UA2970587 = {S56786, 153, { &P22679,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-P710/P71010a-VDF-ES Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996569 = {S56787, 143, { &P22679,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P710/V10a Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3000155 = {S56788, 153, { &P22679,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P710/P71010d-EUR-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3016593 = {S56789, 143, { &P22679,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P710/V10c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3023893 = {S56790, 153, { &P22679,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-P710/P71010e-EUR-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3061199 = {S56791, 153, { &P22679,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-P710/P71010b-VDF-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2755137 = {S56792, 137, { &P16349,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Sony Tablet S Build/TISU0143) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3030625 = {S56793, 111, { &P15241,&P5843,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia300/07.57; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA909830 = {S56794, 153, { &P12280,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8F190 Safari/7534.48.3*/
const Device UA910372 = {S56795, 153, { &P12280,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8J2 Safari/7534.48.3*/
const Device UA910359 = {S56796, 153, { &P12280,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8L1 Safari/7534.48.3*/
const Device UA910436 = {S56797, 153, { &P12280,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8H7 Safari/7534.48.3*/
const Device UA910892 = {S56798, 153, { &P12280,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8G4 Safari/7534.48.3*/
const Device UA912304 = {S56799, 153, { &P12280,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8K2 Safari/7534.48.3*/
const Device UA64863 = {S56800, 165, { &P12399,&P5129,&P3690,&P18092 }};	/*Useragent - SonyEricssonU8i/R2EA; Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525*/
const Device UA2734711 = {S56801, 144, { &P22863,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; en-US; SONY XperiaTablet Build/JOP22F) AppleWebKit/535.19 (KHTML like Gecko) Chrome/18.0.1025.464 Safari/535.19*/
const Device UA3034336 = {S56802, 134, { &P18680,&P18110,&P24339,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Safari/535.19 Silk-Accelerated=false*/
const Device UA2179402 = {S56803, 138, { &P19891,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I717 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2198248 = {S56804, 146, { &P19891,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I717 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2238302 = {S56805, 146, { &P19891,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261570 = {S56806, 138, { &P19891,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2367026 = {S56807, 140, { &P19891,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2558182 = {S56808, 144, { &P21475,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Micromax A35 Build/IML74K) AppleWebKit/534.30(KHTML,like Gecko) 			Version/4.0 Mobile Safari/534.30*/
const Device UA2956086 = {S56809, 74, { &P21475,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Micromax A35 Build/MocorDroid2.3.5)*/
const Device UA2586578 = {S56810, 141, { &P21602,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C5302 Build/12.0.A.0.224) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2713389 = {S56811, 141, { &P21602,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5302 Build/12.0.A.1.211) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2754687 = {S56812, 136, { &P19911,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E970 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA69307 = {S56813, 127, { &P15211,&P17603,&P4732,&P18092 }};	/*Useragent - PantechP2030/JSUS06132011; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.22250; U; en-US) Opera 9.50*/
const Device UA2148041 = {S56814, 117, { &P15211,&P17603,&P4732,&P18092 }};	/*Useragent - PantechP2030/JSUSX; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.22250; U; es) Opera 9.50*/
const Device UA99044 = {S56815, 113, { &P15364,&P17285,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en) AppleWebKit/522.11.3 (KHTML, like Gecko) Version/3.0 Safari/522.11.3*/
const Device UA275094 = {S56816, 112, { &P15364,&P17285,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/523.15 (KHTML, like Gecko) Version/3.0 Safari/523.15*/
const Device UA431085 = {S56817, 115, { &P15364,&P17285,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en) AppleWebKit/522.15.5 (KHTML, like Gecko) Version/3.0.3 Safari/522.15.5*/
const Device UA771077 = {S56818, 115, { &P15364,&P17285,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en) AppleWebKit/522.12.1 (KHTML, like Gecko) Version/3.0.1 Safari/522.12.2*/
const Device UA809665 = {S56819, 115, { &P15364,&P17285,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; nl) AppleWebKit/522.13.1 (KHTML, like Gecko) Version/3.0.2 Safari/522.13.1*/
const Device UA816885 = {S56820, 109, { &P15364,&P17285,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en) AppleWebKit/525+ (KHTML, like Gecko) Version/3.0.4 Safari/523.11*/
const Device UA837163 = {S56821, 116, { &P15364,&P17285,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; da-DK) AppleWebKit/523.12.9 (KHTML, like Gecko) Version/3.0 Safari/523.12.9*/
const Device UA897217 = {S56822, 112, { &P15364,&P17285,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; sv-SE) AppleWebKit/523.13 (KHTML, like Gecko) Version/3.0 Safari/523.13*/
const Device UA2957859 = {S56823, 139, { &P24559,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ZTE-Z992 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092384 = {S56824, 57, { &P24559,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Z992 Build/JRO03C)*/
const Device UA763351 = {S56825, 170, { &P15794,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA902212 = {S56826, 169, { &P15794,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA903370 = {S56827, 169, { &P15794,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315b Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2337516 = {S56828, 159, { &P15794,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Sensation XL with Beats Audio Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2738227 = {S56829, 139, { &P18687,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-P708g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768540 = {S56830, 138, { &P18687,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-P705 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879361 = {S56831, 139, { &P18687,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LG-P705g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2954957 = {S56832, 60, { &P18687,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P705 Build/JZO54K)*/
const Device UA2995975 = {S56833, 61, { &P18687,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P708g Build/JZO54K)*/
const Device UA2975941 = {S56834, 146, { &P15880,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975995 = {S56835, 146, { &P15880,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ME865 Build/6.7.2_GC-220-EDS-3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974900 = {S56836, 145, { &P13064,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021253 = {S56837, 145, { &P13064,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I727 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA1869751 = {S56838, 56, { &P15364,&P18346,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.10; SV1)*/
const Device UA1874455 = {S56839, 64, { &P15364,&P18346,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.10; Trident/4.0)*/
const Device UA2373578 = {S56840, 152, { &P15364,&P18346,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.10; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA2443297 = {S56841, 98, { &P15364,&P18346,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.10; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)*/
const Device UA2815374 = {S56842, 179, { &P15364,&P18346,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.10; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA64785 = {S56843, 152, { &P11834,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia6760s-1/03.45; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA20847 = {S56844, 145, { &P13042,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; HTC Incredible S Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA23722 = {S56845, 150, { &P13042,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC_IncredibleS_S710e Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41772 = {S56846, 148, { &P13042,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-tw; HTC_IncredibleS_S710e Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA667400 = {S56847, 69, { &P13042,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; HTC Incredible S Build/FRG83D)*/
const Device UA725762 = {S56848, 133, { &P13042,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; HTC Incredible S Build/FRG83D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2364811 = {S56849, 138, { &P13042,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; HTC Incredible S Build/FRG83D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3065461 = {S56850, 154, { &P21857,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA672450 = {S56851, 154, { &P14967,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P500 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032040 = {S56852, 177, { &P14967,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; LG-P500 Build/GRI40; NightlyFourE Based CyanogenMod 7 Nightly) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2754531 = {S56853, 145, { &P13064,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769239 = {S56854, 115, { &P20744,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia301.1/02.03b; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.48*/
const Device UA2879725 = {S56855, 112, { &P20744,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia301/02.33; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.54*/
const Device UA3108647 = {S56856, 114, { &P20744,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia301.2/02.51; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.55*/
const Device UA2348113 = {S56857, 136, { &P20343,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZP900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2526193 = {S56858, 137, { &P20343,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZP900S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2834978 = {S56859, 58, { &P20343,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ZP900 Build/IMM76D)*/
const Device UA2567090 = {S56860, 152, { &P18552,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT897 Build/7.7.1Q-6_SPR-125_ASA-14) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2509859 = {S56861, 58, { &P15364,&P7127,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:19.0) Gecko/X Firefox/19.0*/
const Device UA3130797 = {S56862, 135, { &P20705,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132877 = {S56863, 137, { &P20705,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC_PN071 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA2222403 = {S56864, 138, { &P18673,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2385376 = {S56865, 139, { &P18673,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562L Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2566794 = {S56866, 127, { &P18282,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A200 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2508082 = {S56867, 156, { &P12897,&P5843,&P24676,&P18092 }};	/*Useragent - NokiaC2-00/2.0 (03.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-00) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA3065727 = {S56868, 129, { &P18385,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3067908 = {S56869, 129, { &P18385,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus 7 Build/JRO03S) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3111880 = {S56870, 127, { &P18385,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; Nexus 7 Build/JRN84D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2171630 = {S56871, 112, { &P19972,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia113/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.5.16.9*/
const Device UA2754264 = {S56872, 138, { &P20947,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTCEVOV4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2203978 = {S56873, 136, { &P21123,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EM543 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2432755 = {S56874, 102, { &P15364,&P18346,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA3065571 = {S56875, 138, { &P24044,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R740C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA227576 = {S56876, 132, { &P13505,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB525 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA575334 = {S56877, 132, { &P13505,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MB525 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA649563 = {S56878, 120, { &P13505,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MB525 Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA659746 = {S56879, 135, { &P13505,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; MB525 Build/MIUI Es) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA669600 = {S56880, 135, { &P13505,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; MB525 Build/MIUI Es) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1919923 = {S56881, 56, { &P13505,&P24694,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; MB525 Build/MIUI)*/
const Device UA1965613 = {S56882, 125, { &P13505,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MB525 Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2018719 = {S56883, 132, { &P13505,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MB525 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018997 = {S56884, 135, { &P13505,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; MB525 Build/MIUI Es) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027769 = {S56885, 132, { &P13505,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ME525 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2813230 = {S56886, 133, { &P13505,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ME525+ Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2816658 = {S56887, 125, { &P13505,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ME525 Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2404825 = {S56888, 42, { &P11905,&P5890,&P24673,&P18092 }};	/*Useragent - NOKIA6120cUCBrowser/8.7.1.234/28/352/UCWEB*/
const Device UA2404967 = {S56889, 120, { &P11905,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA6120c) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2433441 = {S56890, 92, { &P11905,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6120c) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2567007 = {S56891, 146, { &P12814,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I777 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567944 = {S56892, 146, { &P12814,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I777 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583779 = {S56893, 138, { &P12814,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I777 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602925 = {S56894, 138, { &P12814,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-I777 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA20663 = {S56895, 105, { &P13712,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-T469V/T469VTJH1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA43626 = {S56896, 105, { &P13712,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-T469W/T469UQIK5 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA54895 = {S56897, 104, { &P13712,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-T469/T469UVIG4 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA420886 = {S56898, 143, { &P16587,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SHW-M190S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1900926 = {S56899, 67, { &P16587,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M190S Build/GINGERBREAD)*/
const Device UA1965793 = {S56900, 143, { &P16587,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M190S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026124 = {S56901, 67, { &P16587,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SHW-M190S Build/GINGERBREAD)*/
const Device UA648756 = {S56902, 136, { &P21066,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PMID701C Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA660967 = {S56903, 137, { &P21949,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; LG-F100L Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025614 = {S56904, 61, { &P21949,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-F100L Build/GRK39F)*/
const Device UA924467 = {S56905, 82, { &P15364,&P7133,&P22316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:14.0) Gecko/X Firefox/14.0.1 SeaMonkey/2.11*/
const Device UA1995416 = {S56906, 67, { &P15364,&P7133,&P22316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:14.0) Gecko/X SeaMonkey/2.11*/
const Device UA2508659 = {S56907, 160, { &P14708,&P5204,&P24676,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5130c-2) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2597387 = {S56908, 154, { &P14708,&P5204,&P24676,&P18092 }};	/*Useragent - Nokia5130/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5130) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2975742 = {S56909, 139, { &P15989,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21i Build/11.0.A.4.22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020969 = {S56910, 139, { &P15989,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21i2 Build/11.0.A.6.5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022504 = {S56911, 139, { &P15989,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21a2 Build/11.0.A.1.8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023385 = {S56912, 139, { &P15989,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21a Build/11.0.A.0.16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA67205 = {S56913, 97, { &P14781,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; MWP6985)*/
const Device UA666625 = {S56914, 81, { &P15364,&P17843,&P17825,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/3.5; Linux) KHTML/3.5.5 (like Gecko) (Kubuntu)*/
const Device UA485121 = {S56915, 87, { &P17595,&P18120,&P22500,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.8.0.4) Gecko/X Camino/1.0.2*/
const Device UA2134464 = {S56916, 76, { &P17595,&P18120,&P22500,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-US; rv:1.8) Gecko/X Camino/1.0b1*/
const Device UA610619 = {S56917, 144, { &P14225,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  HTC Desire HD Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA1959177 = {S56918, 139, { &P14225,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Desire HD Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1960541 = {S56919, 143, { &P14225,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire HD Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261499 = {S56920, 143, { &P14225,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire HD Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2416911 = {S56921, 144, { &P14225,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Desire HD Build/HuaweiU8815) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA11709 = {S56922, 76, { &P11726,&P5026,&P3410,&P18092 }};	/*Useragent - BlackBerry8800/4.5.0.69 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/123*/
const Device UA547184 = {S56923, 27, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/4.3.3 (3; 64GB)*/
const Device UA576829 = {S56924, 27, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/5.0.1 (4; 32GB)*/
const Device UA627869 = {S56925, 26, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/4.2.1 (2; 8GB)*/
const Device UA628940 = {S56926, 27, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/4.3.5 (4; 32GB)*/
const Device UA631586 = {S56927, 21, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/3.1.3 (2)*/
const Device UA635229 = {S56928, 26, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/4.2.1 (4; 8GB)*/
const Device UA641108 = {S56929, 25, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/4.1 (4; 64GB)*/
const Device UA642336 = {S56930, 27, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/4.3.5 (3; 32GB)*/
const Device UA643327 = {S56931, 27, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/4.2.1 (2; 16GB)*/
const Device UA645675 = {S56932, 26, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/5.0.1 (4; 8GB)*/
const Device UA645927 = {S56933, 26, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/4.3.5 (4; 8GB)*/
const Device UA650540 = {S56934, 27, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/4.2.1 (2; 32GB)*/
const Device UA658335 = {S56935, 24, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/4.1 (4; 8GB)*/
const Device UA672255 = {S56936, 17, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/3.1.3*/
const Device UA678428 = {S56937, 27, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/5.0.1 (3; 32GB)*/
const Device UA691491 = {S56938, 34, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/5.1.1 (4; 32GB; dt:71)*/
const Device UA706525 = {S56939, 34, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/5.1.1 (3; 32GB; dt:25)*/
const Device UA795795 = {S56940, 33, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/5.1.1 (4; 8GB; dt:71)*/
const Device UA811586 = {S56941, 34, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/5.1.1 (4; 64GB; dt:71)*/
const Device UA895323 = {S56942, 34, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/5.1.1 (3; 64GB; dt:25)*/
const Device UA2126296 = {S56943, 24, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/4.0 (2; 8GB)*/
const Device UA2242342 = {S56944, 24, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/4.1 (2; 8GB)*/
const Device UA2242343 = {S56945, 32, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/6.0 (4; 32GB; dt:71)*/
const Device UA2267737 = {S56946, 25, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/4.1 (2; 32GB)*/
const Device UA2267942 = {S56947, 25, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/4.1 (4; 32GB)*/
const Device UA2268873 = {S56948, 27, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/4.2.1 (4; 32GB)*/
const Device UA2362660 = {S56949, 33, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/6.0.1 (4; 8GB; dt:71)*/
const Device UA2392404 = {S56950, 27, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/5.0.1 (4; 64GB)*/
const Device UA2426961 = {S56951, 34, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/6.0.1 (4; 32GB; dt:81)*/
const Device UA2504049 = {S56952, 34, { &P17346,&P17779,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/6.0.1 (4; 16GB; dt:71)*/
const Device UA625056 = {S56953, 145, { &P20957,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; T-Mobile Vivacity Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024009 = {S56954, 69, { &P20957,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; T-Mobile Vivacity Build/GRJ22)*/
const Device UA2524957 = {S56955, 136, { &P20957,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Vivacity Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA107004 = {S56956, 4, { &P18254,&P18588,&P18589,&P18092 }};	/*Useragent - Lynx*/
const Device UA123547 = {S56957, 6, { &P18254,&P18588,&P18589,&P18092 }};	/*Useragent - FavOrg*/
const Device UA377693 = {S56958, 3, { &P18254,&P18588,&P18589,&P18092 }};	/*Useragent - RSS*/
const Device UA2742841 = {S56959, 7, { &P18254,&P18588,&P18589,&P18092 }};	/*Useragent - VisuRss*/
const Device UA2945954 = {S56960, 7, { &P18254,&P18588,&P18589,&P18092 }};	/*Useragent - Feedbin*/
const Device UA2258895 = {S56961, 64, { &P15364,&P17285,&P18691,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:16.0) Gecko/X Thunderbird/16.0.2*/
const Device UA2975053 = {S56962, 139, { &P18496,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LVW Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022947 = {S56963, 139, { &P18496,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR6410LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024117 = {S56964, 139, { &P18496,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LRA Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3065596 = {S56965, 134, { &P19904,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Hydro Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067053 = {S56966, 133, { &P19904,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C5170 Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2483182 = {S56967, 136, { &P21261,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3004363 = {S56968, 143, { &P21261,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.0; en-us; I9500 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3030020 = {S56969, 156, { &P21261,&P18066,&P18062,&P18092 }};	/*Useragent - GT-I9500 Linux/3.0.13 Android/4.2.2 Release/07.22.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA175512 = {S56970, 133, { &P18619,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MID7024 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458690 = {S56971, 137, { &P18619,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-es; CFW-MID7024 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458702 = {S56972, 57, { &P18619,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; MID7024 Build/FRF91)*/
const Device UA458812 = {S56973, 83, { &P18619,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/0.9.3 (Linux; U; Android 2.2; en_US; MID7024 Build/MID Serials 2.2 FRF91 X.X)*/
const Device UA692567 = {S56974, 137, { &P21040,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Next7S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2707586 = {S56975, 137, { &P22584,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337M Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2739795 = {S56976, 130, { &P22584,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337M Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2367192 = {S56977, 138, { &P16292,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Galaxy Nexus Build/IMM96K) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/30.0.1207.1 Mobile Safari/8536.25*/
const Device UA3131463 = {S56978, 140, { &P16292,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132876 = {S56979, 141, { &P16292,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA2944509 = {S56980, 112, { &P15364,&P20706,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia210/04.12; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.53*/
const Device UA3011454 = {S56981, 114, { &P15364,&P20706,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia210.2/04.12; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.53*/
const Device UA3015658 = {S56982, 114, { &P15364,&P20706,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia210.4/04.11; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.53*/
const Device UA3021169 = {S56983, 114, { &P15364,&P20706,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia210.5/04.11; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.53*/
const Device UA3079593 = {S56984, 114, { &P15364,&P20706,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia210.3/04.12; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.53*/
const Device UA2421107 = {S56985, 141, { &P23501,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; BLU DASH 3.5 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2507276 = {S56986, 144, { &P23501,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; BMOBILE DASH 3.5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2581946 = {S56987, 140, { &P23501,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; BLU DASH 3.5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2747533 = {S56988, 145, { &P23501,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; BLU DASH 3.5 VIVA Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2996957 = {S56989, 136, { &P23501,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; DASH 3.5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2974925 = {S56990, 137, { &P18673,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023225 = {S56991, 138, { &P18673,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562L Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973892 = {S56992, 139, { &P18719,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C1505 Build/11.3.A.2.13) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA683473 = {S56993, 85, { &P15364,&P18110,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux armv7l; rv:7.0.1; HERCULES; EC-SLIM-EX) Gecko/X Firefox/7.0.1*/
const Device UA717570 = {S56994, 70, { &P15364,&P18110,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux ppc; rv:7.0) Gecko/X Firefox/7.0 Iceweasel/7.0*/
const Device UA780849 = {S56995, 75, { &P15364,&P18110,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:7.0.1) Gecko/X Firefox/7.0.1 Firefox/7.0.1*/
const Device UA896613 = {S56996, 50, { &P15364,&P18110,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0X11Linuxi686rv7.0.1Gecko/XFirefox/7.0.1*/
const Device UA2147788 = {S56997, 62, { &P15364,&P18110,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:7.0.1) Gecko/X Abrowser/7.0.1*/
const Device UA2203355 = {S56998, 58, { &P15364,&P17188,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0; rv:16.0) Gecko/X Firefox/16.0*/
const Device UA2974623 = {S56999, 140, { &P23910,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30at Build/9.1.F.1.120) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA87605 = {S57000, 35, { &P15341,&P17017,&P17470,&P18092 }};	/*Useragent - LCT_L300B_01/L3PM510/WAP2.0 Profile*/
const Device UA138488 = {S57001, 8, { &P15341,&P17017,&P17470,&P18092 }};	/*Useragent - PM/3.3.5*/
const Device UA164937 = {S57002, 61, { &P15341,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (MobilePhone PM-8200/US/1.0) NetFront/3.x MMP/2.0*/
const Device UA2259994 = {S57003, 82, { &P15364,&P16110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.2.3) Gecko/X Namoroka/3.6.3*/
const Device UA2336915 = {S57004, 76, { &P15364,&P16110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko)*/
const Device UA2352347 = {S57005, 57, { &P15364,&P16110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:12.0) Gecko/ /12.0*/
const Device UA2502207 = {S57006, 84, { &P15364,&P16110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/533+ (KHTML, like Gecko)*/
const Device UA2574368 = {S57007, 69, { &P15364,&P16110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.57.2 (KHTML, like Gecko)*/
const Device UA3012284 = {S57008, 41, { &P15364,&P16110,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows 8 6.2) Java/1.7.0_25*/
const Device UA2756016 = {S57009, 127, { &P17926,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA172168 = {S57010, 143, { &P15364,&P17816,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Ubuntu/10.04 Chromium/9.0.597.94 Chrome/9.0.597.94 Safari/534.13*/
const Device UA408329 = {S57011, 143, { &P15364,&P17816,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Ubuntu/10.04 Chromium/9.0.595.0 Chrome/9.0.595.0 Safari/534.13*/
const Device UA717241 = {S57012, 145, { &P15364,&P17816,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux armv7l; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Ubuntu/10.04 Chromium/9.0.597.94 Chrome/9.0.597.94 Safari/534.13*/
const Device UA2323151 = {S57013, 33, { &P22801,&P17017,&P3534,&P18092 }};	/*Useragent - HUAWEI-M570/001.00 WAP/OBIGO/Q05A*/
const Device UA2900020 = {S57014, 156, { &P18267,&P5043,&P24678,&P18092 }};	/*Useragent - NokiaX2-02/2.0 (11.79) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-02) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2461032 = {S57015, 157, { &P12482,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-T959 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2562887 = {S57016, 157, { &P12482,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-T959 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567433 = {S57017, 156, { &P12482,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T959 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708328 = {S57018, 157, { &P12482,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T959 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3031402 = {S57019, 156, { &P18267,&P5043,&P24679,&P18092 }};	/*Useragent - NokiaX2-02/2.0 (11.79) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-02) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA49703 = {S57020, 98, { &P13073,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-SGH-T379/1.0 SHP/VPP/R5 NetFront/4.2 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA61568 = {S57021, 125, { &P12746,&P5208,&P4223,&P18092 }};	/*Useragent - NetFront/3.5.1(BREW 3.1.5; U; en-us; SAMSUNG; NetFront/3.1.5/WAP) Sprint M380 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2431284 = {S57022, 82, { &P18088,&P17017,&P18274,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/9.5/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2406515 = {S57023, 120, { &P18364,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaC5-05) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2423532 = {S57024, 42, { &P18364,&P5129,&P24673,&P18092 }};	/*Useragent - NokiaC5-05UCBrowser/8.7.1.234/50/444/UCWEB*/
const Device UA2422981 = {S57025, 75, { &P17595,&P17295,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8) Presto/2.12.388 Version/12.12*/
const Device UA599052 = {S57026, 89, { &P17968,&P5096,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2; Linux; Opera Mobi/ADR-X; U; es-ES) Presto/2.10.254 Version/12.00*/
const Device UA599899 = {S57027, 88, { &P17968,&P5096,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2.3; Linux; Opera Mobi/ADR-X; U; en) Presto/2.10.254 Version/12.00*/
const Device UA630564 = {S57028, 88, { &P17968,&P5096,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2.1; Linux; Opera Mobi/ADR-X; U; pl) Presto/2.10.254 Version/12.00*/
const Device UA799139 = {S57029, 88, { &P17968,&P5096,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2.2; Linux; Opera Mobi/ADR-X; U; pl) Presto/2.10.254 Version/12.00*/
const Device UA30903 = {S57030, 46, { &P12366,&P6476,&P3625,&P18092 }};	/*Useragent - LGE-MN240/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA2390387 = {S57031, 137, { &P20416,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-04E Build/V10R27E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2436691 = {S57032, 137, { &P20416,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-04E Build/V11R29D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2510944 = {S57033, 137, { &P20416,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-04E Build/V13R30C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2601706 = {S57034, 137, { &P20416,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-04E Build/V14R31C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936170 = {S57035, 59, { &P20416,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; F-04E Build/V14R31C)*/
const Device UA3063081 = {S57036, 137, { &P20416,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-04E Build/V15R32D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2887915 = {S57037, 104, { &P15364,&P21460,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.3; Windows NT 6.3; Win64; x64; Trident/7.0; rv:11.0) like Gecko/X Firefox/12.0*/
const Device UA2897727 = {S57038, 92, { &P15364,&P21460,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.3; Windows NT 6.3; Trident/7.0; rv:11.0) like Gecko/X Firefox/12.0*/
const Device UA2976849 = {S57039, 88, { &P15364,&P21460,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.3; Win64; x64; Trident/7.0; rv:11.0) like Gecko/X Firefox/12.0*/
const Device UA3023253 = {S57040, 76, { &P15364,&P21460,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.3; Trident/7.0; rv:11.0) like Gecko/X Firefox/12.0*/
const Device UA2423566 = {S57041, 58, { &P15364,&P7127,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:18.0) Gecko/X Firefox/18.0*/
const Device UA2672745 = {S57042, 128, { &P17968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Build/JWQ87D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.49 Mobile Safari/537.31*/
const Device UA2673258 = {S57043, 136, { &P17968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TWD_MID Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675076 = {S57044, 125, { &P17968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2677709 = {S57045, 132, { &P17968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2706873 = {S57046, 143, { &P17968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; INFINITY_POWER Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707137 = {S57047, 132, { &P17968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Pad Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707325 = {S57048, 135, { &P17968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Tender Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707599 = {S57049, 128, { &P17968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707728 = {S57050, 143, { &P17968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbl1 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707759 = {S57051, 116, { &P17968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Build/X) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708045 = {S57052, 133, { &P17968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID Serials Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708381 = {S57053, 143, { &P17968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1808_v75_jbl2 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708720 = {S57054, 157, { &P17968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1809l_v75_gq1008_ov5647_v20 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2710099 = {S57055, 148, { &P17968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1808_v75_jbl1_5640 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712528 = {S57056, 132, { &P17968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713325 = {S57057, 121, { &P17968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735529 = {S57058, 157, { &P17968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; e1909c_v77_gq1008_9p017_6228 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2736925 = {S57059, 148, { &P17968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1808_v75_jbl2_6628 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2751669 = {S57060, 127, { &P17968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; V1013 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2767867 = {S57061, 130, { &P17968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; EBM727KC Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2780935 = {S57062, 125, { &P17968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2936387 = {S57063, 145, { &P17968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbaol1 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2566699 = {S57064, 137, { &P14574,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567587 = {S57065, 137, { &P14574,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus S Build/JRO03E) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2666967 = {S57066, 130, { &P14574,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2676287 = {S57067, 137, { &P19920,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS870 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707588 = {S57068, 130, { &P19920,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS870 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2403676 = {S57069, 137, { &P14123,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P970 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2734880 = {S57070, 138, { &P14123,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P970h Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2867055 = {S57071, 138, { &P14123,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P970g Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2427863 = {S57072, 140, { &P20479,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lenco CoolTab-70 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2755557 = {S57073, 140, { &P22867,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5303 Build/12.0.A.1.257) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2260694 = {S57074, 178, { &P18506,&P5043,&P24672,&P18092 }};	/*Useragent - Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia305) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2230709 = {S57075, 141, { &P19779,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A1+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2578230 = {S57076, 65, { &P19779,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; A1+ Build/MocorDroid2.3.5)*/
const Device UA2741792 = {S57077, 101, { &P21407,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 928)*/
const Device UA2752965 = {S57078, 129, { &P18454,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGPT13 Build/TJS0166) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755517 = {S57079, 129, { &P18454,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGPT12 Build/TJS0166) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2753547 = {S57080, 138, { &P18296,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One V Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2957601 = {S57081, 138, { &P18296,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One V Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA54463 = {S57082, 10, { &P11658,&P5776,&P3756,&P18092 }};	/*Useragent - Nokia 6680*/
const Device UA56662 = {S57083, 180, { &P11658,&P5776,&P3756,&P18092 }};	/*Useragent - Nokia6680/1.0 (4.04.07) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0Configuration/CLDC-1.1, Nokia6680/1.0 (4.04.07) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA63745 = {S57084, 90, { &P11658,&P5776,&P3756,&P18092 }};	/*Useragent - Nokia6680/1.0 (4.04.07) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA80863 = {S57085, 9, { &P11658,&P5776,&P3756,&P18092 }};	/*Useragent - Nokia6680*/
const Device UA1960411 = {S57086, 135, { &P24021,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HW-T18D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1960896 = {S57087, 134, { &P24021,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HW-T18 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2680916 = {S57088, 128, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Build/JWR11G) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.49 Mobile Safari/537.36*/
const Device UA2754815 = {S57089, 125, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755552 = {S57090, 136, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TWD_MID Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769979 = {S57091, 132, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783944 = {S57092, 132, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2796248 = {S57093, 143, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1808_v75_jbl2 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799861 = {S57094, 143, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbl1 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867072 = {S57095, 132, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Pad Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867210 = {S57096, 148, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1808_v75_jbl1_5640 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867744 = {S57097, 130, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; EBM727KC Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2867846 = {S57098, 157, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1809l_v75_gq1008_ov5647_v20 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868934 = {S57099, 127, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Build/ICS.g04ref.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2876353 = {S57100, 128, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877621 = {S57101, 121, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2879434 = {S57102, 143, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; INFINITY_POWER Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892791 = {S57103, 146, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_hjy1a6x Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892885 = {S57104, 135, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Tender Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937436 = {S57105, 157, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; e1909c_v77_gq1008_9p017_6228 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937816 = {S57106, 133, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID Serials Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2958246 = {S57107, 147, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbl1a535 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2958889 = {S57108, 125, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2997465 = {S57109, 151, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; e1909c_v77_gq1008_5647 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3088566 = {S57110, 145, { &P17968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1808_v75_jbaol1 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937428 = {S57111, 163, { &P24102,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG SM-C101 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2797654 = {S57112, 145, { &P23402,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; BLU STUDIO 5.0 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3019372 = {S57113, 142, { &P23402,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Studio 5.0s Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3090052 = {S57114, 67, { &P23402,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; BLU STUDIO 5.0 Build/JZO54K)*/
const Device UA2167425 = {S57115, 115, { &P14826,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC1-01/05.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.5.16.11*/
const Device UA2376528 = {S57116, 118, { &P14826,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC1-01.1/04.40a; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.5.16.11*/
const Device UA2356481 = {S57117, 127, { &P18554,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; NOOK BNTV400 BUILD/ICS) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2382901 = {S57118, 140, { &P15767,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_0 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2385605 = {S57119, 141, { &P15767,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPad OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293 Safari/6531.22.7*/
const Device UA2526105 = {S57120, 140, { &P15767,&P17144,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_0 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B500 Safari/531.21.10*/
const Device UA2753018 = {S57121, 140, { &P17625,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST25i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755070 = {S57122, 140, { &P17625,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST25a Build/6.1.1.B.1.10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868439 = {S57123, 137, { &P17625,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xperia U Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3130801 = {S57124, 135, { &P20705,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTCONE Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132012 = {S57125, 136, { &P20705,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA62120 = {S57126, 67, { &P14372,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GM360i/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63534 = {S57127, 66, { &P14372,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GM360/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2233055 = {S57128, 115, { &P15364,&P16110,&P22462,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4*/
const Device UA2233056 = {S57129, 122, { &P15364,&P16110,&P22462,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4*/
const Device UA2708158 = {S57130, 160, { &P13042,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Incredible S Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2793780 = {S57131, 160, { &P13042,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Incredible S Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957513 = {S57132, 161, { &P13042,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Incredible S Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3015683 = {S57133, 171, { &P13042,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Incredible S S710e Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/vivo*/
const Device UA2737879 = {S57134, 136, { &P20213,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; N9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2866732 = {S57135, 58, { &P20213,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; N9500 Build/JOP40D)*/
const Device UA623729 = {S57136, 135, { &P20962,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-03D Build/V21R28E) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625728 = {S57137, 133, { &P20962,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-03D Build/F0001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627195 = {S57138, 135, { &P20962,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-03D Build/V22R30A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA635305 = {S57139, 136, { &P20962,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-03D Build/V23R31Ac) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA714859 = {S57140, 136, { &P20962,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-03D Build/V22R29Bc) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1942815 = {S57141, 57, { &P20962,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; F-03D Build/F0001)*/
const Device UA2562105 = {S57142, 135, { &P20962,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-03D Build/V23R32C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2562381 = {S57143, 135, { &P20962,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-03D Build/V24R33C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3067068 = {S57144, 128, { &P23975,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B1-710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA7232 = {S57145, 12, { &P17968,&P6757,&P3525,&P18092 }};	/*Useragent - Android 1.5;*/
const Device UA50203 = {S57146, 118, { &P17968,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA56143 = {S57147, 132, { &P17968,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; vi-vn Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA67438 = {S57148, 132, { &P17968,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA140308 = {S57149, 139, { &P17968,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; Pole1 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2026270 = {S57150, 127, { &P24510,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; N10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2385061 = {S57151, 127, { &P24510,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N10 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2594173 = {S57152, 130, { &P24510,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N10 Build/R13.4-rc1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2695731 = {S57153, 127, { &P24510,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; N10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA458745 = {S57154, 139, { &P17958,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; MID8024 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458766 = {S57155, 143, { &P17958,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; CFW-MID8024 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458837 = {S57156, 63, { &P17958,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3; MID8024 Build/GINGERBREAD)*/
const Device UA459016 = {S57157, 67, { &P17958,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3; CFW-MID8024 Build/GINGERBREAD)*/
const Device UA143709 = {S57158, 119, { &P15364,&P17285,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/527+ (KHTML, like Gecko, Safari/419.3) Arora/0.6 (Change: )*/
const Device UA347089 = {S57159, 119, { &P15364,&P17285,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Kobo Desktop Edition Safari/532.4*/
const Device UA555977 = {S57160, 112, { &P15364,&P17285,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) VIDEODOWNLOAD Safari/533.3*/
const Device UA704570 = {S57161, 110, { &P15364,&P17285,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.1 (KHTML, like Gecko) CEF/0.0.0.0 Safari/534.1*/
const Device UA2383129 = {S57162, 101, { &P15364,&P17285,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.3 (KHTML, like Gecko) Lunascape/6.8.0.25815 Safari/535.3*/
const Device UA2385193 = {S57163, 92, { &P15364,&P17285,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.34 (KHTML, like Gecko) iLivid/1.0 Safari/534.34*/
const Device UA2394163 = {S57164, 116, { &P15364,&P17285,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/528+ (KHTML, like Gecko, Safari/528.0) Lunascape/5.1.2.0*/
const Device UA2409329 = {S57165, 117, { &P15364,&P17285,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/529 (KHTML, like Gecko, Safari/529.0) Lunascape/4.9.9.100*/
const Device UA2456630 = {S57166, 99, { &P15364,&P17285,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) iLivid/1.0 Safari/534.34*/
const Device UA2595673 = {S57167, 90, { &P15364,&P17285,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (N; Windows NT 6.0) AppleWebKit/534.34 (KHTML, like Gecko) Raptr Safari/534.34*/
const Device UA2768736 = {S57168, 103, { &P15364,&P17285,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Superbird/27.0.1450.0 Safari/537.36*/
const Device UA2793738 = {S57169, 104, { &P15364,&P17285,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.17 (KHTML, like Gecko) Superbird/24.0.1312.57 Safari/537.17*/
const Device UA2868770 = {S57170, 104, { &P15364,&P17285,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.22 (KHTML, like Gecko) Superbird/27.0.1453.93 Safari/537.22*/
const Device UA2880209 = {S57171, 137, { &P23642,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-M840 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2944354 = {S57172, 130, { &P23642,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-M840 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2879130 = {S57173, 136, { &P23521,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2897916 = {S57174, 129, { &P23521,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3092337 = {S57175, 137, { &P23521,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195T Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3108927 = {S57176, 137, { &P23521,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195L Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA366271 = {S57177, 116, { &P15364,&P17285,&P22374,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.3 (KHTML, like Gecko) Maxthon/3.3.3.1000 Chrome/16.0.883.0 Safari/535.3*/
const Device UA648783 = {S57178, 123, { &P15364,&P17285,&P22374,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.3 (KHTML, like Gecko) Maxthon/3.3.3.1000 Chrome/16.0.883.0 Safari/535.3*/
const Device UA901433 = {S57179, 118, { &P15364,&P17285,&P22374,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.3.9.1000 Chrome/18.0.966.0 Safari/535.12*/
const Device UA902434 = {S57180, 125, { &P15364,&P17285,&P22374,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.3.9.2000 Chrome/18.0.966.0 Safari/535.12*/
const Device UA688026 = {S57181, 142, { &P12568,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-B7510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA840679 = {S57182, 143, { &P12568,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; iw-il; GT-B7510B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024954 = {S57183, 66, { &P12568,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-B7510 Build/GINGERBREAD)*/
const Device UA2389885 = {S57184, 87, { &P15364,&P7133,&P22555,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:17.0) Gecko/17.0 Firefox/17.0 IceDragon/17.0.1.0*/
const Device UA2676177 = {S57185, 126, { &P16598,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A500 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2707919 = {S57186, 133, { &P16598,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Iconia A500 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708355 = {S57187, 126, { &P16598,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A500 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2768140 = {S57188, 133, { &P16598,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A500 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA28068 = {S57189, 98, { &P15364,&P17844,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.3) Gecko/20061201 Firefox/2.0.0.3 (Ubuntu-feisty)*/
const Device UA455323 = {S57190, 146, { &P18348,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC Velocity 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA767869 = {S57191, 68, { &P18348,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Velocity 4G Build/IML74K)*/
const Device UA896718 = {S57192, 152, { &P18348,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-hk; HTC_Velocity_4G_X710s Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027268 = {S57193, 152, { &P18348,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-hk; HTC Velocity 4G X710s Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3085053 = {S57194, 74, { &P18348,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC Velocity 4G X710s Build/IML74K)*/
const Device UA187755 = {S57195, 141, { &P16331,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Huawei Ascend Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA196259 = {S57196, 141, { &P16331,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Huawei Ascend Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA212784 = {S57197, 132, { &P16331,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; M860 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA225390 = {S57198, 132, { &P16331,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; M860 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA248876 = {S57199, 132, { &P16331,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; M860 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2603083 = {S57200, 144, { &P16331,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HUAWEI-M860 Build/HuaweiM860) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA512075 = {S57201, 97, { &P15364,&P18132,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.10) Gecko/X Linux Mint/5 (Elyssa) Firefox/3.0.10*/
const Device UA858529 = {S57202, 97, { &P15364,&P18132,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.11) Gecko/X Linux Mint/5 (Elyssa) Firefox/3.0.11*/
const Device UA897124 = {S57203, 97, { &P15364,&P18132,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.10) Gecko/X Linux Mint/5 (Elyssa) Firefox/3.0.10*/
const Device UA1899116 = {S57204, 97, { &P15364,&P18132,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.12) Gecko/X Linux Mint/5 (Elyssa) Firefox/3.0.12*/
const Device UA1939696 = {S57205, 95, { &P15364,&P18132,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; es-AR; rv:1.9.0.4) Gecko/X Linux Mint/5 (Elyssa) Firefox/3.0.4*/
const Device UA2126149 = {S57206, 95, { &P15364,&P18132,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.5) Gecko/X Linux Mint/5 (Elyssa) Firefox/3.0.5*/
const Device UA2128177 = {S57207, 95, { &P15364,&P18132,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.7) Gecko/X Linux Mint/5 (Elyssa) Firefox/3.0.7*/
const Device UA2130659 = {S57208, 95, { &P15364,&P18132,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.8) Gecko/X Linux Mint/5 (Elyssa) Firefox/3.0.8*/
const Device UA2431114 = {S57209, 229, { &P13123,&P5629,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2432255 = {S57210, 229, { &P13123,&P5629,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2974123 = {S57211, 128, { &P17968,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974250 = {S57212, 136, { &P17968,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; skz_tenderloin Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021031 = {S57213, 147, { &P17968,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; e2001_v89_jbl1a698 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021068 = {S57214, 121, { &P17968,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021080 = {S57215, 134, { &P17968,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; cm_tenderloin Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021379 = {S57216, 146, { &P17968,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; e2001_v89_gq2008s Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021538 = {S57217, 148, { &P17968,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; e2001v21_v89_zlh_hd Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022267 = {S57218, 141, { &P17968,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; v89_jbl1a698 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023419 = {S57219, 134, { &P17968,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; iPhone5 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023919 = {S57220, 153, { &P17968,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; e2001v21_v89_jbl1a698_2g Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023929 = {S57221, 153, { &P17968,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; e2001_v89_jbl1a668_fwvga Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3065734 = {S57222, 135, { &P15575,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-I9100 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094028 = {S57223, 136, { &P15575,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-I9100G Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA40912 = {S57224, 76, { &P13787,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/1.0 Beta 11 (Linux; U; Android 2.2; en_US; SGH-I987 Build/FROYO.UCJK1)*/
const Device UA64021 = {S57225, 134, { &P13787,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-I987 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA532094 = {S57226, 58, { &P13787,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; SGH-I987 Build/FROYO)*/
const Device UA67403 = {S57227, 134, { &P14854,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; de-de; GT-I9010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA173366 = {S57228, 137, { &P14854,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SHW-M240S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018581 = {S57229, 61, { &P14854,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SHW-M240S Build/FROYO)*/
const Device UA911758 = {S57230, 132, { &P17323,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6800 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2620539 = {S57231, 132, { &P17323,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P6800 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2625412 = {S57232, 133, { &P17323,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  GT-P6800 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2974501 = {S57233, 137, { &P18636,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; VS930 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA61911 = {S57234, 73, { &P14348,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C3200/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2975037 = {S57235, 138, { &P23227,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I467 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022484 = {S57236, 131, { &P23227,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I467M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA23698 = {S57237, 144, { &P14997,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-tw; LiquidE Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA23699 = {S57238, 148, { &P14997,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Acer Liquid Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA62549 = {S57239, 143, { &P14997,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Liquid Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA482188 = {S57240, 65, { &P14997,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Liquid Build/ECLAIR)*/
const Device UA2202578 = {S57241, 81, { &P17968,&P18066,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.3; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2202579 = {S57242, 81, { &P17968,&P18066,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.4; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2402151 = {S57243, 81, { &P17968,&P18066,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.1; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2424895 = {S57244, 81, { &P17968,&P18066,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.9; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2556327 = {S57245, 79, { &P17968,&P18066,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2729343 = {S57246, 81, { &P17968,&P18066,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.2; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2536073 = {S57247, 132, { &P21854,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PLT7044K Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2853831 = {S57248, 77, { &P21854,&P18111,&P18112,&P18092 }};	/*Useragent - AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; PLT7044K Build/JRO03H)*/
const Device UA5216 = {S57249, 131, { &P13999,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; vi-; HTC HD2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA8717 = {S57250, 148, { &P13999,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Android Powered HD2 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18426 = {S57251, 136, { &P13999,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; vi-vn; HTC_HD2 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20553 = {S57252, 135, { &P13999,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC HD2 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20605 = {S57253, 136, { &P13999,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC HD2 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA28260 = {S57254, 137, { &P13999,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HD2_T8585 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA28309 = {S57255, 137, { &P13999,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; HD2_T8585 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA28338 = {S57256, 135, { &P13999,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HD2_T8585 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38795 = {S57257, 142, { &P13999,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-tw; HTC_HD2_T8585 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA53649 = {S57258, 139, { &P13999,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC_HD2_T8585 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA53657 = {S57259, 133, { &P13999,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC HD2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA166816 = {S57260, 135, { &P13999,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC HD2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA290324 = {S57261, 57, { &P13999,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; HTC HD2 Build/FRF91)*/
const Device UA296704 = {S57262, 134, { &P13999,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC HD2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1856948 = {S57263, 60, { &P13999,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; HTC HD2 Build/FRG83D)*/
const Device UA2024188 = {S57264, 121, { &P13999,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC HD2 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2168264 = {S57265, 142, { &P14711,&P5204,&P24668,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-01) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA912056 = {S57266, 73, { &P16329,&P18533,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PLAYSTATION 3 4.21) AppleWebKit/531.22.8 (KHTML, like Gecko)*/
const Device UA2676439 = {S57267, 137, { &P18295,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One X Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676658 = {S57268, 138, { &P18295,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; HTC One X Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676824 = {S57269, 133, { &P18295,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; One X Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2860985 = {S57270, 150, { &P18295,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Full AOSP on EndeavorU Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3047472 = {S57271, 130, { &P18295,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One X Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA630598 = {S57272, 133, { &P15364,&P17819,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.7 (KHTML, like Gecko) Ubuntu/11.10 Chromium/16.0.912.77 Chrome/16.0.912.77 Safari/535.7*/
const Device UA634929 = {S57273, 135, { &P15364,&P17819,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.7 (KHTML, like Gecko) Ubuntu/11.10 Chromium/16.0.912.77 Chrome/16.0.912.77 Safari/535.7*/
const Device UA1999471 = {S57274, 133, { &P15364,&P17819,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.7 (KHTML, like Gecko) Ubuntu/11.10 Chromium/16.0.912.77 Chrome/16 0.912.77 Safari/535.7*/
const Device UA13998 = {S57275, 61, { &P14460,&P5890,&P4030,&P18092 }};	/*Useragent - Nokia2760/2.0 (05.20) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA14531 = {S57276, 66, { &P14460,&P5890,&P4030,&P18092 }};	/*Useragent - Nokia2760_CMCC/2.0 (05.20) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA1998748 = {S57277, 140, { &P18290,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ617 Build/7.7.1-85_MZ617-27) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2425163 = {S57278, 136, { &P18605,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; HTL21 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2656695 = {S57279, 58, { &P18605,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTL21 Build/JRO03C)*/
const Device UA2406463 = {S57280, 58, { &P15364,&P17188,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0; rv:17.0) Gecko/X Firefox/17.0*/
const Device UA2404169 = {S57281, 155, { &P18460,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-51 Build/HuaweiU8950N-51) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2424207 = {S57282, 151, { &P18460,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-1 Build/HuaweiU8950-1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2482036 = {S57283, 149, { &P18460,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950D Build/HuaweiU8950D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2505036 = {S57284, 153, { &P18460,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-1 Build/HuaweiU8950N-1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2526856 = {S57285, 153, { &P18460,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-51 Build/HuaweiU8950-51) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3024148 = {S57286, 163, { &P16292,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/toro*/
const Device UA3062476 = {S57287, 163, { &P16292,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/toro*/
const Device UA3096103 = {S57288, 163, { &P16292,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/toro*/
const Device UA3116189 = {S57289, 163, { &P16292,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Galaxy Nexus Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/toro*/
const Device UA63996 = {S57290, 78, { &P12933,&P5208,&P4521,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Polaris 6.2; Brew 3.1.5; en)/240X320 Samsung sam-r631*/
const Device UA798907 = {S57291, 85, { &P15364,&P7127,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-GB; rv:1.8.1.12) Gecko/X Firefox/2.0.0.12*/
const Device UA2406289 = {S57292, 135, { &P21346,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NovoCrystal Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2509145 = {S57293, 130, { &P21346,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NovoCrystal Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2983844 = {S57294, 64, { &P21346,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; NovoCrystal Build/JRO03C)*/
const Device UA2975939 = {S57295, 140, { &P22598,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6506 Build/10.3.A.0.423) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973477 = {S57296, 133, { &P20760,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Event Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2197031 = {S57297, 71, { &P15364,&P18110,&P23479,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; de-DE; rv:99.0) Gecko/X Firefox/25.0.0*/
const Device UA3031573 = {S57298, 78, { &P15364,&P18110,&P23479,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:25.0) Gecko/X Firefox/25.0 Iceweasel/25.0a2*/
const Device UA2028973 = {S57299, 154, { &P17324,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-I9001 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2653803 = {S57300, 167, { &P17324,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-i9001 Build/IMM76L; CyanogenMod-EHNDROIX i9001) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2813694 = {S57301, 154, { &P17324,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-I9001 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2230819 = {S57302, 146, { &P20168,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Maxwell Plus Build/1.0.0 X-10:39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2367288 = {S57303, 146, { &P20168,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Maxwell Plus Build/1.0.2 X-14:58) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2427639 = {S57304, 146, { &P20168,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Maxwell Plus Build/1.0.3 X-14:07) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2754239 = {S57305, 146, { &P15880,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ME865 Build/6.7.2_GC-220-EDS-3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2756010 = {S57306, 146, { &P15880,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA233175 = {S57307, 141, { &P14225,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_DesireHD_A9191; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA552522 = {S57308, 145, { &P14225,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_DesireHD_A9191/1.0; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2324192 = {S57309, 136, { &P14225,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Desire_HD; en-hk) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA379422 = {S57310, 167, { &P15364,&P17486,&P18373,&P18092 }};	/*Useragent - MOT-EX119 MIDP-2.0/CLDC-1.1 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U) Presto/2.5.28 Version/10.10*/
const Device UA656904 = {S57311, 167, { &P15364,&P17486,&P18373,&P18092 }};	/*Useragent - MOT-EX139 MIDP-2.0/CLDC-1.1 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U) Presto/2.5.28 Version/10.10*/
const Device UA1905554 = {S57312, 174, { &P15364,&P17486,&P18373,&P18092 }};	/*Useragent - MOT-EX226 MIDP-2.0/CLDC-1.1 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA2401371 = {S57313, 174, { &P15364,&P17486,&P18373,&P18092 }};	/*Useragent - MOT-EX505 MIDP-2.0/CLDC-1.1 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA2196574 = {S57314, 182, { &P18267,&P5043,&P24671,&P18092 }};	/*Useragent - NokiaX2-02/2.0 (11.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-02) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA3095795 = {S57315, 142, { &P15767,&P22882,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/11A465 Safari/7534.48.3*/
const Device UA3130390 = {S57316, 144, { &P15767,&P22882,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/11A501 Safari/7534.48.3*/
const Device UA24004 = {S57317, 136, { &P15650,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SPH-M580 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA206984 = {S57318, 60, { &P15650,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; SPH-M580 Build/FROYO)*/
const Device UA790317 = {S57319, 131, { &P20158,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029644 = {S57320, 139, { &P20158,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; tr-tr; Trident A7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2227282 = {S57321, 132, { &P20158,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; A7* Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2809533 = {S57322, 55, { &P20158,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; A7 Build/GRK39F)*/
const Device UA63481 = {S57323, 172, { &P12126,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 Nokia6730c-1/021.009; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.1.13126*/
const Device UA64407 = {S57324, 168, { &P12126,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 Nokia6730c/031.022; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.3.1*/
const Device UA67952 = {S57325, 12, { &P12126,&P5890,&P3439,&P18092 }};	/*Useragent - NOKIA6730c-1*/
const Device UA702651 = {S57326, 152, { &P14799,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-I5500 Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA712411 = {S57327, 161, { &P14799,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-I5500 Build/GINGERBREAD; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2569607 = {S57328, 122, { &P15364,&P17285,&P20429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 6.0; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/7.0.3.8542 Safari/532.4*/
const Device UA2767714 = {S57329, 98, { &P15364,&P17285,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.0.6000.0);en;kml:2.2;client:Free;type:default)*/
const Device UA2428630 = {S57330, 138, { &P14574,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus S Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2504912 = {S57331, 60, { &P14574,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; Nexus S Build/JOP40D)*/
const Device UA2531164 = {S57332, 138, { &P14574,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus S Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567191 = {S57333, 137, { &P14574,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2600289 = {S57334, 137, { &P14574,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; crespo Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708993 = {S57335, 138, { &P14574,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus S Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865594 = {S57336, 60, { &P14574,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; Nexus S Build/JDQ39E)*/
const Device UA2752973 = {S57337, 129, { &P16976,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT300SE Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754586 = {S57338, 127, { &P16976,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT300 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2674595 = {S57339, 102, { &P15364,&P18346,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31*/
const Device UA56058 = {S57340, 95, { &P14632,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; T8788)*/
const Device UA925785 = {S57341, 98, { &P14632,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Surround)*/
const Device UA925847 = {S57342, 98, { &P14632,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Mondrian)*/
const Device UA362492 = {S57343, 113, { &P15364,&P5690,&P22454,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7*/
const Device UA398249 = {S57344, 120, { &P15364,&P5690,&P22454,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7*/
const Device UA368764 = {S57345, 39, { &P17968,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (daytona 4.5.1A-DTN-150-30)*/
const Device UA369662 = {S57346, 25, { &P17968,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (VS700 GRJ22)*/
const Device UA2018106 = {S57347, 128, { &P17968,&P17979,&P21556,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0; en-us; generic) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2026365 = {S57348, 28, { &P17968,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (pyramid IML74K)*/
const Device UA2033309 = {S57349, 28, { &P17968,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (holiday IML74K)*/
const Device UA2192452 = {S57350, 26, { &P17968,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (aloha FRG83D)*/
const Device UA2217629 = {S57351, 27, { &P17968,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (kingdom GRJ22)*/
const Device UA2242185 = {S57352, 40, { &P17968,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (cdma_targa 5.5.1_84_DBN-74)*/
const Device UA2374723 = {S57353, 41, { &P17968,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (cdma_droid2 4.5.1_57_DR4-51)*/
const Device UA2474456 = {S57354, 50, { &P17968,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (cdma_maserati 6.5.1_167_DR4-1_M1-219)*/
const Device UA2520686 = {S57355, 28, { &P17968,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (shooter IML74K)*/
const Device UA2545597 = {S57356, 28, { &P17968,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (kingdom IML74K)*/
const Device UA2700156 = {S57357, 26, { &P17968,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (mecha IMM76D)*/
const Device UA2808938 = {S57358, 26, { &P17968,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (d2att IMM76D)*/
const Device UA3041788 = {S57359, 29, { &P17968,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (stingray IMM76L)*/
const Device UA3085563 = {S57360, 23, { &P17968,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (e0 GRK39F)*/
const Device UA3111727 = {S57361, 153, { &P15364,&P7133,&P23486,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.6.6.X (Second Life Release; default skin) Safari/533.3*/
const Device UA895243 = {S57362, 112, { &P15767,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.9.1*/
const Device UA895259 = {S57363, 114, { &P15767,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.1*/
const Device UA901046 = {S57364, 112, { &P15767,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.9.2*/
const Device UA902201 = {S57365, 114, { &P15767,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.2*/
const Device UA907462 = {S57366, 114, { &P15767,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.3*/
const Device UA909958 = {S57367, 114, { &P15767,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.4*/
const Device UA910647 = {S57368, 112, { &P15767,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.9.4*/
const Device UA924791 = {S57369, 114, { &P15767,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.5*/
const Device UA924826 = {S57370, 112, { &P15767,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.9.5*/
const Device UA1997808 = {S57371, 114, { &P15767,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.6*/
const Device UA2000847 = {S57372, 112, { &P15767,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.9.6*/
const Device UA2202083 = {S57373, 114, { &P15767,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.7*/
const Device UA2202917 = {S57374, 112, { &P15767,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.9.7*/
const Device UA2260562 = {S57375, 114, { &P15767,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.8*/
const Device UA2367655 = {S57376, 112, { &P15767,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.9.8*/
const Device UA2974646 = {S57377, 162, { &P13157,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975570 = {S57378, 168, { &P13157,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021527 = {S57379, 148, { &P13157,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Z715e Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023029 = {S57380, 168, { &P13157,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2402969 = {S57381, 136, { &P20480,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JY-G3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695640 = {S57382, 58, { &P20480,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; JY-G3 Build/IMM76D)*/
const Device UA2700426 = {S57383, 135, { &P20584,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Z120 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2461946 = {S57384, 68, { &P15364,&P7133,&P22564,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:16.0) Gecko/X Firefox/16.0 epic/16.0*/
const Device UA2893808 = {S57385, 74, { &P15364,&P18110,&P24329,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:22.0) Gecko/X Firefox/22.0 SeaMonkey/2.19*/
const Device UA3028158 = {S57386, 158, { &P18635,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/mako*/
const Device UA3062465 = {S57387, 158, { &P18635,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/mako*/
const Device UA3095679 = {S57388, 158, { &P18635,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/mako*/
const Device UA3115771 = {S57389, 158, { &P18635,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/mako*/
const Device UA2367430 = {S57390, 141, { &P20931,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; iBall Slide i9702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2367506 = {S57391, 143, { &P20931,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iBall Slide 3G 7334 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2551788 = {S57392, 150, { &P20931,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; iBall Slide 3G 7307 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2600291 = {S57393, 149, { &P20931,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iBall Slide 3G7271 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2653695 = {S57394, 143, { &P20931,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; iBall Slide i5715 Build/tablet-c) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2668064 = {S57395, 152, { &P20931,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; iBall Slide i6030 Build/iBall-Slide-i6030) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2673051 = {S57396, 134, { &P20213,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N9500 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2956329 = {S57397, 127, { &P20213,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N9500 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA925515 = {S57398, 137, { &P18438,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E973 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2366388 = {S57399, 138, { &P18438,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F200L Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2384846 = {S57400, 138, { &P18438,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F200K Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2400260 = {S57401, 138, { &P18438,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F200S Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2365490 = {S57402, 138, { &P20103,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EK-GC100 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2466863 = {S57403, 146, { &P20103,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-EK-GC100 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2470072 = {S57404, 138, { &P20103,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; EK-GC100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2998975 = {S57405, 146, { &P20103,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-EK-GC100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA596 = {S57406, 83, { &P13220,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK770i/R8BC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63283 = {S57407, 84, { &P13220,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK770iv/R8BC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA723925 = {S57408, 115, { &P15837,&P6909,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-05/08.35; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.11*/
const Device UA2490748 = {S57409, 153, { &P15767,&P18117,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10 UCBrowser/2.2.2.259*/
const Device UA2784 = {S57410, 49, { &P12040,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GW300-Bouygues Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA4530 = {S57411, 68, { &P12040,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GW300FD/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA44585 = {S57412, 74, { &P12040,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GW300-orange/V09a Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA54612 = {S57413, 74, { &P12040,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GW300-Orange/V10c Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA64625 = {S57414, 66, { &P12040,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GW300/V100 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2566768 = {S57415, 137, { &P19911,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E970 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602009 = {S57416, 130, { &P19911,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E970 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA241938 = {S57417, 144, { &P17595,&P17297,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_2; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0*/
const Device UA323730 = {S57418, 144, { &P17595,&P17297,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_0; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0*/
const Device UA359122 = {S57419, 146, { &P17595,&P17297,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_2; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0*/
const Device UA583824 = {S57420, 146, { &P17595,&P17297,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_3; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0*/
const Device UA659359 = {S57421, 146, { &P17595,&P17297,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0*/
const Device UA681861 = {S57422, 144, { &P17595,&P17297,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_3; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0*/
const Device UA740239 = {S57423, 144, { &P17595,&P17297,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0*/
const Device UA1984842 = {S57424, 127, { &P17595,&P17297,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0*/
const Device UA1996217 = {S57425, 148, { &P17595,&P17297,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0*/
const Device UA2158877 = {S57426, 144, { &P17595,&P17297,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/531.21.8+(KHTML, like Gecko, Safari/528.16) Version/5.10.3 OmniWeb/622.14.0*/
const Device UA2199417 = {S57427, 146, { &P17595,&P17297,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.1 OmniWeb/622.18.0*/
const Device UA2201856 = {S57428, 148, { &P17595,&P17297,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11.2 OmniWeb/622.19.3.0*/
const Device UA2352571 = {S57429, 144, { &P17595,&P17297,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-US) AppleWebKit/533.21.1+(KHTML, like Gecko, Safari/533.19.4) Version/5.11 OmniWeb/622.16.0*/
const Device UA2838144 = {S57430, 127, { &P17595,&P17297,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-US) AppleWebKit/531.9+(KHTML, like Gecko, Safari/528.16) OmniWeb/v622.10.0*/
const Device UA2838625 = {S57431, 144, { &P17595,&P17297,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-US) AppleWebKit/531.21.8+(KHTML, like Gecko, Safari/528.16) Version/5.10.3 OmniWeb/622.14.0*/
const Device UA610135 = {S57432, 143, { &P18465,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  Amazon Kindle Fire Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA901587 = {S57433, 142, { &P18465,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Amazon Kindle Fire Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2044069 = {S57434, 135, { &P18465,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Kindle Fire Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2044083 = {S57435, 139, { &P18465,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Amazon Kindle Fire Build/MR1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2044087 = {S57436, 149, { &P18465,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4-2.4.20; Amazon Kindle Fire Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2044092 = {S57437, 148, { &P18465,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Amazon Kindle Fire Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2401396 = {S57438, 128, { &P18465,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2455879 = {S57439, 135, { &P18465,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KF012 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2537328 = {S57440, 137, { &P18438,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E973 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2600129 = {S57441, 138, { &P18438,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F200S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2603708 = {S57442, 138, { &P18438,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F200K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2670088 = {S57443, 138, { &P18438,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F200L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1996811 = {S57444, 88, { &P17968,&P18111,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1.1; Linux; Opera Mobi/ADR-X; U; fr) Presto/2.10.254 Version/12.00*/
const Device UA2031433 = {S57445, 86, { &P17968,&P18111,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1; Linux; Opera Mobi/ADR-X; U; en) Presto/2.10.254 Version/12.00*/
const Device UA2503071 = {S57446, 88, { &P17968,&P18111,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1.2; Linux; Opera Mobi/ADY-X; U; ru) Presto/2.10.254 Version/12.00*/
const Device UA63974 = {S57447, 75, { &P11669,&P5776,&P3756,&P18092 }};	/*Useragent - Nokia7610/2008.05.06 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64376 = {S57448, 92, { &P11669,&P5776,&P3756,&P18092 }};	/*Useragent - Nokia7610/2.0 (7.0642.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA2126678 = {S57449, 36, { &P11669,&P5776,&P3756,&P18092 }};	/*Useragent - Nokia7610/SymbianOS/8.0 Series60/2.6*/
const Device UA2031422 = {S57450, 133, { &P15003,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHW-M380W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033050 = {S57451, 133, { &P15003,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHW-M380S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2240473 = {S57452, 133, { &P15003,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHW-M380K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2762390 = {S57453, 62, { &P15003,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHW-M380W Build/IMM76D)*/
const Device UA2173910 = {S57454, 138, { &P18300,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T769 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2437258 = {S57455, 131, { &P20955,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U; Android 4.1; en-gb; ST10216-1Build/ST10216-1) AppleWebKit/534.30 (KHTML,like Gecko) Version/4.0 Safari/534.30*/
const Device UA2846758 = {S57456, 65, { &P20955,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ST10216-1 Build/ST10216-1)*/
const Device UA2942530 = {S57457, 146, { &P20955,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ST10216-2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30;SurfTab_10.1*/
const Device UA686572 = {S57458, 143, { &P12280,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.10*/
const Device UA700848 = {S57459, 148, { &P12280,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0(iPhone; U; CPU iPhone OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B314 Safari/531.21.10*/
const Device UA777629 = {S57460, 158, { &P12280,&P17141,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone Simulator; U; CPU iPhone OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7D11 Safari/531.21.10*/
const Device UA2353415 = {S57461, 141, { &P21212,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; N90 DUAL CORE2 CZ Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2465745 = {S57462, 138, { &P21212,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; N90 DUAL CORE2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2528698 = {S57463, 131, { &P21212,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N90DC_S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2778574 = {S57464, 60, { &P21212,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; N90DC_S Build/JRO03H)*/
const Device UA3044154 = {S57465, 152, { &P21212,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; N90 DUAL CORE FHD Build/N90 DUAL CORE FHD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3070256 = {S57466, 142, { &P21212,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; N90 DUAL CORE2 V12 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3021581 = {S57467, 156, { &P14826,&P5843,&P24679,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac1-01) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA14178 = {S57468, 62, { &P11793,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia3500c/2.0 (05.51) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA67902 = {S57469, 73, { &P11793,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia3500c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia3500c*/
const Device UA2200862 = {S57470, 95, { &P17595,&P17297,&P19836,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5; U; Edition MAS; en) Presto/2.10.229 Version/11.67*/
const Device UA2365234 = {S57471, 65, { &P15364,&P18145,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64; rv:17.0) Gecko/17.0 Firefox/17.0*/
const Device UA2567119 = {S57472, 140, { &P18496,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LVW Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2669082 = {S57473, 140, { &P18496,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR6410LRA Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2859806 = {S57474, 140, { &P18496,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR6410LVW Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2891001 = {S57475, 133, { &P18496,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LVW Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2974816 = {S57476, 129, { &P18385,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2976107 = {S57477, 129, { &P18385,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus 7 Build/JRO03S) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021814 = {S57478, 127, { &P18385,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; Nexus 7 Build/JRN84D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA693553 = {S57479, 128, { &P14903,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; LG-V900 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA719014 = {S57480, 133, { &P14903,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; de-de; LG-V900/v10e Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2022790 = {S57481, 57, { &P14903,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; LG-V900 Build/HMJ37)*/
const Device UA2011111 = {S57482, 138, { &P14877,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9000 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2231063 = {S57483, 138, { &P14877,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9000 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2233627 = {S57484, 190, { &P18283,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, gb*/
const Device UA2234187 = {S57485, 190, { &P18283,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T999 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, us*/
const Device UA2235619 = {S57486, 190, { &P18283,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I535 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, us*/
const Device UA2429709 = {S57487, 58, { &P15364,&P17188,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0; rv:18.0) Gecko/X Firefox/18.0*/
const Device UA2554915 = {S57488, 138, { &P19870,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-L710 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA62426 = {S57489, 139, { &P12144,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; A6277 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA67725 = {S57490, 139, { &P12144,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; A6277 Build/CUPCAKE) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA668705 = {S57491, 141, { &P18216,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Sony Tablet P Build/THMAAS0045) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA678111 = {S57492, 140, { &P18216,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-gb; Sony Tablet P Build/THMAS0042) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA754658 = {S57493, 139, { &P18216,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; Sony Tablet P Build/THMAS11001) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA769802 = {S57494, 141, { &P18216,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-gb; Sony Tablet P Build/THMASU0035) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA869720 = {S57495, 138, { &P18216,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; Sony Tablet P Build/THMD01900) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1905494 = {S57496, 139, { &P18216,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; Sony Tablet P Build/THMAS11002) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA436960 = {S57497, 142, { &P22875,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA715404 = {S57498, 142, { &P22875,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA804205 = {S57499, 130, { &P22875,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1965014 = {S57500, 135, { &P22875,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2023172 = {S57501, 66, { &P22875,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9220 Build/GINGERBREAD)*/
const Device UA2483062 = {S57502, 134, { &P22875,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; I9220 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2722539 = {S57503, 133, { &P22875,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; I9220 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA611324 = {S57504, 142, { &P21713,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NWZ-Z1000Series Build/C121) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA854836 = {S57505, 142, { &P21713,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NWZ-Z1000Series Build/B100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033964 = {S57506, 66, { &P21713,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; NWZ-Z1000Series Build/C121)*/
const Device UA609643 = {S57507, 142, { &P14540,&P5843,&P24667,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiax2-01) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA526347 = {S57508, 84, { &P17595,&P18120,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US; rv:1.8.1.16) Gecko/X Firefox/3.0.1*/
const Device UA599482 = {S57509, 98, { &P17595,&P18120,&P3409,&P18092 }};	/*Useragent - Mozilla/6.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:2.0.0.0) Gecko/X Firefox/3.0	Firefox 3.0*/
const Device UA1868789 = {S57510, 77, { &P17595,&P18120,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; rv:1.7.2) Gecko/X Firefox/3.0*/
const Device UA2203281 = {S57511, 164, { &P18250,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Motorola RAZR MAXX Build/6.7.2-180_DHD-16_NA-27) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2946978 = {S57512, 90, { &P17098,&P5567,&P18525,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/6.0.0/27.2338; U; en) Presto/2.8.119 240X320 Samsung SCH-U660*/
const Device UA3065466 = {S57513, 129, { &P17687,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ604 Build/I.7.1-45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3081115 = {S57514, 129, { &P17687,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MZ604 Build/I.7.1-34) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA76203 = {S57515, 150, { &P18660,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; ISW11M Build/4.5.1A-1_KDI-92_LE-24) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA360224 = {S57516, 151, { &P18660,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; ISW11M Build/4.5.1A-1_KDI-95_MR1-47) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA614354 = {S57517, 151, { &P18660,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; ISW11M Build/4.5.1A-1_KDI-95_MR3-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA741866 = {S57518, 151, { &P18660,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; ISW11M Build/4.5.1A-1_KDI-95_MR2-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2191017 = {S57519, 150, { &P18660,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; ISW11M Build/4.5.1A-1_KDI-95_MR4-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982035 = {S57520, 74, { &P18660,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; ISW11M Build/4.5.1A-1_KDI-95_MR4-3)*/
const Device UA783344 = {S57521, 90, { &P15364,&P17815,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2) Gecko/X Ubuntu/9.04 (jaunty) Firefox/3.6*/
const Device UA2228272 = {S57522, 136, { &P23894,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V1277 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2838519 = {S57523, 58, { &P23894,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; V1277 Build/IMM76D)*/
const Device UA2507824 = {S57524, 58, { &P15364,&P17188,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0; rv:19.0) Gecko/X Firefox/19.0*/
const Device UA156845 = {S57525, 136, { &P17346,&P17145,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Twitter for iPhone*/
const Device UA2384991 = {S57526, 142, { &P20540,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PAP4300 DUO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3067643 = {S57527, 139, { &P13054,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR6425LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA16202 = {S57528, 47, { &P13546,&P17484,&P5001,&P18092 }};	/*Useragent - SCH-U360/1.0 NetFront/3.0.22.2.22 (GUI) MMP/2.0*/
const Device UA62357 = {S57529, 33, { &P17899,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/A930/Screen-240X320*/
const Device UA2008453 = {S57530, 156, { &P18280,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One S Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2199799 = {S57531, 156, { &P18280,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One S Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2355268 = {S57532, 160, { &P18280,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One S Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2368947 = {S57533, 130, { &P22265,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT7033 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3065595 = {S57534, 136, { &P18621,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E610 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081120 = {S57535, 137, { &P18621,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E610v Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081394 = {S57536, 136, { &P18621,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083080 = {S57537, 137, { &P18621,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083332 = {S57538, 137, { &P18621,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612f Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA1852 = {S57539, 77, { &P12138,&P5026,&P3410,&P18092 }};	/*Useragent - BlackBerry8130/4.5.0.175 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105*/
const Device UA2223451 = {S57540, 142, { &P14826,&P5843,&P24670,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (04.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac1-01) UCBrowser8.4.0.159/70/444/UCWEB Mobile*/
const Device UA2367959 = {S57541, 138, { &P20240,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-E971 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2983532 = {S57542, 60, { &P20240,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-E971 Build/IMM76L)*/
const Device UA2719032 = {S57543, 140, { &P18223,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.0.400) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2738195 = {S57544, 133, { &P18223,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.0.400) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA923968 = {S57545, 127, { &P12657,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_EVO3D_X515a; en-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA1964671 = {S57546, 127, { &P12657,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_EVO3D_X515m; zh-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA1986233 = {S57547, 127, { &P12657,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_EVO3D_X515m; en-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA1995968 = {S57548, 127, { &P12657,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_EVO3D_X515m; en-hk) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2156420 = {S57549, 127, { &P12657,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_EVO3D_X515m; en-gb) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2227552 = {S57550, 127, { &P12657,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_EVO3D_X515a; en-au) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2235178 = {S57551, 133, { &P12657,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_X515a V3.29.841.1; en-au) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2339798 = {S57552, 127, { &P12657,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_EVO3D_X515m; zh-cn) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2374132 = {S57553, 127, { &P12657,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_EVO3D_X515m; de-de) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2410394 = {S57554, 127, { &P12657,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_EVO3D_X515a; fr-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2722175 = {S57555, 127, { &P12657,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_EVO3D_X515m; vi-vn) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2889282 = {S57556, 133, { &P12657,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_X515a V1.24.841.3; en-au) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2676328 = {S57557, 139, { &P18448,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST26i Build/11.2.A.0.21) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708698 = {S57558, 132, { &P18448,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST26i Build/11.2.A.0.21) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3094336 = {S57559, 139, { &P18448,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST26a Build/11.2.A.0.33) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA355743 = {S57560, 134, { &P18651,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SH-01D Build/SB082) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA371271 = {S57561, 134, { &P18651,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SH-01D Build/S1110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625343 = {S57562, 134, { &P18651,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-01D Build/S2040) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA899634 = {S57563, 134, { &P18651,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-01D Build/S5100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA923831 = {S57564, 134, { &P18651,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-01D Build/S6270) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1979425 = {S57565, 58, { &P18651,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SH-01D Build/S6270)*/
const Device UA2011090 = {S57566, 130, { &P20586,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN8CG3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2487940 = {S57567, 139, { &P20819,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; VS TOUCHTAB 7DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2738247 = {S57568, 141, { &P20819,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; VS TOUCHTAB 7DC V Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2988438 = {S57569, 68, { &P20819,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; VS TOUCHTAB 7DC Build/JRO03H)*/
const Device UA2583140 = {S57570, 135, { &P21636,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C525c Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2749634 = {S57571, 134, { &P23162,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; B15 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955280 = {S57572, 56, { &P23162,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; B15 Build/JZO54K)*/
const Device UA401110 = {S57573, 162, { &P21478,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; WX442 Build/REV) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 240X320 motorola WX442*/
const Device UA2753165 = {S57574, 138, { &P23772,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753883 = {S57575, 138, { &P23772,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190L Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2566621 = {S57576, 137, { &P18621,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E610 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566825 = {S57577, 138, { &P18621,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612g Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566920 = {S57578, 138, { &P18621,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612f Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567346 = {S57579, 137, { &P18621,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583561 = {S57580, 138, { &P18621,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E610v Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2974915 = {S57581, 134, { &P21636,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C525c Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA50458 = {S57582, 141, { &P13234,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; Behold2 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA126330 = {S57583, 50, { &P15364,&P6729,&P4618,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.0; AK; Windows 95)*/
const Device UA127737 = {S57584, 66, { &P15364,&P6729,&P4618,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.02; Update a; AOL 3.0; Windows 95)*/
const Device UA481364 = {S57585, 47, { &P15364,&P6729,&P4618,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.01; Windows 95)*/
const Device UA2851227 = {S57586, 54, { &P15364,&P6729,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; Win95; Trident/1.0)*/
const Device UA632278 = {S57587, 47, { &P23144,&P18570,&P17470,&P18092 }};	/*Useragent - Facebook/4110.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2754371 = {S57588, 42, { &P23144,&P18570,&P17470,&P18092 }};	/*Useragent - Tumblr/117 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2973670 = {S57589, 138, { &P22970,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975717 = {S57590, 137, { &P22970,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975726 = {S57591, 138, { &P22970,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA243038 = {S57592, 79, { &P14377,&P22283,&P4223,&P18092 }};	/*Useragent - Mozilla/4.0 (Brew MP 1.0.2; U; en-us; Kyocera; NetFront/3.5.1/AMB) Sprint S3015*/
const Device UA2560818 = {S57593, 82, { &P14377,&P22283,&P4223,&P18092 }};	/*Useragent - Mozilla/4.0 (Brew MP 1.0.2; U; en-us; Kyocera; NetFront/3.5.1/AMB) payLo S3015-PLB*/
const Device UA3034176 = {S57594, 154, { &P18465,&P18066,&P24334,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2566513 = {S57595, 130, { &P18454,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGPT12 Build/TID0142) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567520 = {S57596, 130, { &P18454,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGPT13 Build/TID0142) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2728291 = {S57597, 141, { &P22958,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; USCC-US780 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2801275 = {S57598, 139, { &P22958,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-US780 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2801278 = {S57599, 61, { &P22958,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-US780 Build/JZO54K)*/
const Device UA2226355 = {S57600, 31, { &P11929,&P5129,&P24672,&P18092 }};	/*Useragent - Nokia5230/UCWEB8.6.0.180/50/800*/
const Device UA2237326 = {S57601, 119, { &P11929,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5230) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile*/
const Device UA2462379 = {S57602, 141, { &P19764,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C6502 Build/10.1.A.0.233) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2606408 = {S57603, 141, { &P19764,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6502 Build/10.1.A.1.434) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3065605 = {S57604, 145, { &P13064,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA903567 = {S57605, 102, { &P23051,&P17603,&P4392,&P18092 }};	/*Useragent - PantechP6020/JXUSX BMP/1.0.4 DeviceId/X NetFront/4.1 OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2569881 = {S57606, 149, { &P18385,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2772756 = {S57607, 148, { &P18385,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2868898 = {S57608, 149, { &P18385,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 7 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3026675 = {S57609, 113, { &P14635,&P5843,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-00/08.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3026310 = {S57610, 156, { &P14635,&P5843,&P24679,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (08.71) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac3-00) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA18515 = {S57611, 134, { &P14090,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I5801 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA715612 = {S57612, 58, { &P14090,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I5801 Build/FROYO)*/
const Device UA768044 = {S57613, 122, { &P14090,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I5801 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA681535 = {S57614, 95, { &P15364,&P17350,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT) AppleWebKit/534.20 (KHTML, like Gecko) Chrome/11.0.672.2 Safari/534.20*/
const Device UA3042313 = {S57615, 94, { &P15364,&P17350,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT) AppleWebKit/534.20 (KHTML like Gecko) Chrome/11.0.672.2 Safari/534.20*/
const Device UA2712083 = {S57616, 140, { &P22935,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-I9080L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956209 = {S57617, 62, { &P22935,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9080L Build/JZO54K)*/
const Device UA2965424 = {S57618, 139, { &P24063,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; IdeaTab S6000-F Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3016421 = {S57619, 68, { &P24063,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; IdeaTab S6000-F Build/JOP40D)*/
const Device UA3018593 = {S57620, 138, { &P24063,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; pl-pl; IdeaTab S6000-H Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3108254 = {S57621, 138, { &P24063,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; IdeaTab S6000-F Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3067054 = {S57622, 168, { &P13157,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081784 = {S57623, 162, { &P13157,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094184 = {S57624, 168, { &P13157,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2403459 = {S57625, 171, { &P22098,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; VodafoneSmartChat-MSM7225A-V02c-SepX-Vodafone-IT/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2455581 = {S57626, 171, { &P22098,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ro-ro; VodafoneSmartChat-MSM7225A-V02c-SepX-Vodafone-RO/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2486015 = {S57627, 171, { &P22098,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; VodafoneSmartChat-MSM7225A-V02c-SepX-Vodafone-IE/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2504526 = {S57628, 171, { &P22098,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; VodafoneSmartChat-MSM7225A-V02c-SepX-Vodafone-ES/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2506514 = {S57629, 171, { &P22098,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; VodafoneSmartChat-MSM7225A-V02c-SepX-Vodafone-UK/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2603587 = {S57630, 171, { &P22098,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; VodafoneSmartChat-MSM7225A-V03a-DecX-Vodafone-ES/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2604174 = {S57631, 171, { &P22098,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; VodafoneSmartChat-MSM7225A-V03a-DecX-Vodafone-IT/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2665535 = {S57632, 171, { &P22098,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ro-ro; VodafoneSmartChat-MSM7225A-V03a-DecX-Vodafone-RO/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2702949 = {S57633, 171, { &P22098,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-pt; VodafoneSmartChat-MSM7225A-V03a-DecX-Vodafone-PT/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2747761 = {S57634, 172, { &P22098,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; VodafoneSmartChat-MSM7225A- V02a-SepX-Vodafone-NZ/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2752720 = {S57635, 171, { &P22098,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; VodafoneSmartChat-MSM7225A-V03a-DecX-Vodafone-UK/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2852782 = {S57636, 171, { &P22098,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; el-gr; VodafoneSmartChat-MSM7225A-V03a-DecX-Vodafone-GR/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2999251 = {S57637, 171, { &P22098,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-pt; VodafoneSmartChat-MSM7225A-V03a-MarX-Vodafone-PT/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2672161 = {S57638, 120, { &P20743,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 720; ANZ885)*/
const Device UA2700796 = {S57639, 122, { &P20743,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 720; Vodafone)*/
const Device UA2779843 = {S57640, 112, { &P20743,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia 720)*/
const Device UA2953317 = {S57641, 119, { &P20743,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia 720T; CMCC)*/
const Device UA3103431 = {S57642, 120, { &P20743,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 720; Orange)*/
const Device UA2961218 = {S57643, 138, { &P24044,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R740C Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3057700 = {S57644, 131, { &P24044,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R740C Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA9078 = {S57645, 73, { &P11814,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-B3210/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2683068 = {S57646, 99, { &P11814,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-B3210/1.0 Release/10.19.2009 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2156475 = {S57647, 117, { &P14862,&P5204,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2700c-2/09.97; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA2042443 = {S57648, 132, { &P18466,&P18110,&P18617,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.1 Safari/535.19 Silk-Accelerated=true*/
const Device UA595218 = {S57649, 90, { &P17976,&P5629,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.4; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00*/
const Device UA595668 = {S57650, 90, { &P17976,&P5629,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.3; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00*/
const Device UA599545 = {S57651, 90, { &P17976,&P5629,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00*/
const Device UA599681 = {S57652, 90, { &P17976,&P5629,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.5; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00*/
const Device UA896805 = {S57653, 90, { &P17976,&P5629,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.6; Linux; Opera Tablet/ADR-X; U; de) Presto/2.10.254 Version/12.00*/
const Device UA2025399 = {S57654, 91, { &P17976,&P5629,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3; Linux; Opera Tablet/ADR-X; U; en-GB) Presto/2.10.254 Version/12.00*/
const Device UA2032936 = {S57655, 90, { &P17976,&P5629,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.7; Linux; Opera Tablet/35779; U; en) Presto/2.10.254 Version/12.00*/
const Device UA2753450 = {S57656, 129, { &P18385,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755118 = {S57657, 129, { &P18385,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus 7 Build/JRO03S) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2769541 = {S57658, 127, { &P18385,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; Nexus 7 Build/JRN84D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2892579 = {S57659, 136, { &P18385,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA57010 = {S57660, 123, { &P14540,&P5843,&P17431,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-01/08.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.0.9.17.2*/
const Device UA2490686 = {S57661, 113, { &P17595,&P21721,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.26.2 (KHTML, like Gecko) Version/6.1 Safari/537.26.2*/
const Device UA2509818 = {S57662, 113, { &P17595,&P21721,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.28.2 (KHTML, like Gecko) Version/6.1 Safari/537.28.2*/
const Device UA2515422 = {S57663, 113, { &P17595,&P21721,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.30.1 (KHTML, like Gecko) Version/6.1 Safari/537.30.1*/
const Device UA2563613 = {S57664, 109, { &P17595,&P21721,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.32 (KHTML, like Gecko) Version/6.1 Safari/537.32*/
const Device UA2569928 = {S57665, 115, { &P17595,&P21721,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.31.15 (KHTML, like Gecko) Version/6.1 Safari/537.31.15*/
const Device UA2583150 = {S57666, 109, { &P17595,&P21721,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.33 (KHTML, like Gecko) Version/6.1 Safari/537.33*/
const Device UA2606674 = {S57667, 109, { &P17595,&P21721,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.34 (KHTML, like Gecko) Version/6.1 Safari/537.34*/
const Device UA2670914 = {S57668, 109, { &P17595,&P21721,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.37 (KHTML, like Gecko) Version/6.1 Safari/537.37*/
const Device UA2671897 = {S57669, 109, { &P17595,&P21721,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.36 (KHTML, like Gecko) Version/6.1 Safari/537.36*/
const Device UA2675244 = {S57670, 115, { &P17595,&P21721,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.35.11 (KHTML, like Gecko) Version/6.1 Safari/537.35.11*/
const Device UA2713781 = {S57671, 109, { &P17595,&P21721,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.38 (KHTML, like Gecko) Version/6.1 Safari/537.38*/
const Device UA2713967 = {S57672, 113, { &P17595,&P21721,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.39.2 (KHTML, like Gecko) Version/6.1 Safari/537.39.1*/
const Device UA2715810 = {S57673, 109, { &P17595,&P21721,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.40 (KHTML, like Gecko) Version/6.1 Safari/537.40*/
const Device UA2740167 = {S57674, 109, { &P17595,&P21721,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.41 (KHTML, like Gecko) Version/6.1 Safari/537.41*/
const Device UA2755382 = {S57675, 113, { &P17595,&P21721,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.42.1 (KHTML, like Gecko) Version/6.1 Safari/537.42.1*/
const Device UA2756697 = {S57676, 109, { &P17595,&P21721,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9) AppleWebKit/537.43 (KHTML, like Gecko) Version/6.1 Safari/537.43*/
const Device UA823634 = {S57677, 89, { &P15364,&P22418,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; pt-BR; rv:1.8.0.3) Gecko/X Ubuntu/dapper Firefox/1.5.0.3*/
const Device UA658539 = {S57678, 73, { &P19958,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-S5220/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2791997 = {S57679, 74, { &P19958,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-S5220R/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64321 = {S57680, 168, { &P14367,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 Nokia6710s/031.022; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.3.1*/
const Device UA68548 = {S57681, 10, { &P14367,&P5890,&P3439,&P18092 }};	/*Useragent - NOKIA6710s*/
const Device UA626158 = {S57682, 148, { &P22064,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; SmartTab7-MSM8260-V02d-DecX-Vodafone-ZA) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA707231 = {S57683, 148, { &P22064,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; es-es; SmartTab7-MSM8260-V02d-DecX-Vodafone-ES) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA713197 = {S57684, 148, { &P22064,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ro-ro; SmartTab7-MSM8260-V02d-DecX-Vodafone-RO) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA805216 = {S57685, 148, { &P22064,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; es-es; SmartTab7-MSM8260-V03a-FebX-Vodafone-ES) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA864860 = {S57686, 148, { &P22064,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-nz; SmartTab7-MSM8260-V02d-DecX-Vodafone-NZ) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA900495 = {S57687, 148, { &P22064,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; hu-hu; SmartTab7-MSM8260-V03b-AprX-Vodafone-HU) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA902525 = {S57688, 148, { &P22064,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; es-es; SmartTab7-MSM8260-V03b-AprX-Vodafone-ES) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1972140 = {S57689, 148, { &P22064,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ro-ro; SmartTab7-MSM8260-V03b-AprX-Vodafone-RO) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1972315 = {S57690, 148, { &P22064,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; SmartTab7-MSM8260-V03b-AprX-Vodafone-ZA) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2025520 = {S57691, 148, { &P22064,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; SmartTab7-MSM8260-V03a-JanX-Vodafone-DE) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2026315 = {S57692, 148, { &P22064,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; hu-hu; SmartTab7-MSM8260-V03a-JanX-Vodafone-HU) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2031990 = {S57693, 148, { &P22064,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; SmartTab7-MSM8260-V03b-AprX-Vodafone-DE) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2032457 = {S57694, 148, { &P22064,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; cs-cz; SmartTab7-MSM8260-V03b-AprX-Vodafone-CZ) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2444440 = {S57695, 132, { &P22064,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; es-es; SmartTab7 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA3085263 = {S57696, 148, { &P22064,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; pt-pt; SmartTab7-MSM8260-V03b-AprX-Vodafone-PT) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2260473 = {S57697, 87, { &P15364,&P17017,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (en-US) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229 Safari/537.4*/
const Device UA213198 = {S57698, 130, { &P15003,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; ko-kr; SHW-M380K Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA219000 = {S57699, 130, { &P15003,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; ko-kr; SHW-M380W Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA331397 = {S57700, 130, { &P15003,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; ko-kr; SHW-M380S Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2019243 = {S57701, 59, { &P15003,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; SHW-M380W Build/HMJ37)*/
const Device UA2020169 = {S57702, 59, { &P15003,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; SHW-M380S Build/HMJ37)*/
const Device UA2512187 = {S57703, 146, { &P20515,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Micromax A54 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2660074 = {S57704, 137, { &P20515,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; A54 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2952393 = {S57705, 61, { &P20515,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; A54 Build/GINGERBREAD)*/
const Device UA6794 = {S57706, 104, { &P11683,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonC905/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3*/
const Device UA2203476 = {S57707, 83, { &P17976,&P5629,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.3; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2236475 = {S57708, 83, { &P17976,&P5629,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.4; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2237115 = {S57709, 83, { &P17976,&P5629,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.1; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2237117 = {S57710, 83, { &P17976,&P5629,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.6; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2237762 = {S57711, 83, { &P17976,&P5629,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.5; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2354480 = {S57712, 81, { &P17976,&P5629,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2356618 = {S57713, 83, { &P17976,&P5629,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.7; Linux; Opera Tablet/46154) Presto/2.11.355 Version/12.10*/
const Device UA2424021 = {S57714, 123, { &P12588,&P5055,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; NokiaC6-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile*/
const Device UA2483288 = {S57715, 102, { &P12588,&P5055,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac6-01) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2556601 = {S57716, 42, { &P12588,&P5055,&P24673,&P18092 }};	/*Useragent - NokiaC6-01UCBrowser/8.7.1.234/51/355/UCWEB*/
const Device UA2773712 = {S57717, 131, { &P22943,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; E270BSA Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA7931 = {S57718, 71, { &P11712,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia2610/2.0 (04.50) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia2610*/
const Device UA7943 = {S57719, 61, { &P11712,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia2610/2.0 (04.90) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA1987422 = {S57720, 70, { &P14186,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; GT-S5830; 320*480) UCWEB7.9.4.145/139/800*/
const Device UA2424250 = {S57721, 69, { &P14186,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.1; zh-cn; GT-S5830; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2555361 = {S57722, 70, { &P14186,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GT-S5830M; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2746952 = {S57723, 70, { &P14186,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; GT-S5830L; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2384313 = {S57724, 182, { &P15228,&P5043,&P24671,&P18092 }};	/*Useragent - NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-03) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2599847 = {S57725, 133, { &P18464,&P18110,&P21708,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Safari/535.19 Silk-Accelerated=true*/
const Device UA2853925 = {S57726, 139, { &P23293,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-LS720 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058186 = {S57727, 61, { &P23293,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-LS720 Build/JZO54K)*/
const Device UA2237042 = {S57728, 47, { &P15767,&P18432,&P17470,&P18092 }};	/*Useragent - FeeddlerRSS 1.12.4 (iPad; iPhone OS 6.0; en_US)*/
const Device UA2362604 = {S57729, 59, { &P15767,&P18432,&P17470,&P18092 }};	/*Useragent - Appcelerator Titanium/1.7.2 (iPad/6.0.1; iPhone OS; en_US;)*/
const Device UA2362794 = {S57730, 49, { &P15767,&P18432,&P17470,&P18092 }};	/*Useragent - FeeddlerRSS 1.12.5 (iPad; iPhone OS 6.0.1; en_US)*/
const Device UA2425394 = {S57731, 137, { &P21085,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; J-Q88B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2511261 = {S57732, 138, { &P21085,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; J-Q88B? Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2510316 = {S57733, 131, { &P15767,&P19826,&P20779,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/2.8.1 Mobile/10B141 Safari/7534.48.3*/
const Device UA2533035 = {S57734, 133, { &P15767,&P19826,&P20779,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/2.8.1 Mobile/10B146 Safari/7534.48.3*/
const Device UA2564673 = {S57735, 133, { &P15767,&P19826,&P20779,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/2.8.1 Mobile/10B147 Safari/7534.48.3*/
const Device UA2585834 = {S57736, 133, { &P15767,&P19826,&P20779,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/2.8.1 Mobile/10B329 Safari/7534.48.3*/
const Device UA2511337 = {S57737, 112, { &P19972,&P5043,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia113/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2361653 = {S57738, 103, { &P18669,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.49.0.31; en-us; 9320/9.49.0.31) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2466511 = {S57739, 103, { &P18669,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.49.0.76; en-us; 9320/9.49.0.76) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2698519 = {S57740, 103, { &P18669,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.49.0.77; en-us; 9320/9.49.0.77) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2728508 = {S57741, 103, { &P18669,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.49.0.47; en-us; 9320/9.49.0.47) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3049099 = {S57742, 103, { &P18669,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.49.0.98; en-us; 9320/9.49.0.98) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3120516 = {S57743, 103, { &P18669,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.49.0.53; en-us; 9320/9.49.0.53) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3067298 = {S57744, 144, { &P23698,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I527 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080977 = {S57745, 137, { &P23698,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I527M Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2028609 = {S57746, 129, { &P17686,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MZ601 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029811 = {S57747, 129, { &P17686,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MZ601 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029960 = {S57748, 129, { &P17686,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MZ601 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2354623 = {S57749, 129, { &P17686,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MZ601 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2527556 = {S57750, 58, { &P17686,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; MZ601 Build/JZO54K)*/
const Device UA2833770 = {S57751, 129, { &P17686,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; MZ601 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2719477 = {S57752, 154, { &P18465,&P18066,&P22588,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2867821 = {S57753, 105, { &P12193,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac5-00.2) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2968797 = {S57754, 96, { &P12193,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaC5-00.2) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA2970459 = {S57755, 94, { &P12193,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaC5-00) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA2994403 = {S57756, 103, { &P12193,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac5-00) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3031098 = {S57757, 156, { &P21648,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA609458 = {S57758, 139, { &P14877,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  GT-I9000 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA609630 = {S57759, 139, { &P14877,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  GT-I9000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA902817 = {S57760, 138, { &P14877,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9000 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA907295 = {S57761, 138, { &P14877,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9000 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2974114 = {S57762, 141, { &P15802,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC_Amaze_4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974648 = {S57763, 137, { &P15802,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Ruby Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976089 = {S57764, 141, { &P15802,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Amaze 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA438888 = {S57765, 135, { &P18305,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-hk; LG-P936 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2413837 = {S57766, 140, { &P18305,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; LG-P936/V10c Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2415122 = {S57767, 140, { &P18305,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; LG-P936/V10e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2544665 = {S57768, 132, { &P18305,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; sv; LG-P936 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2889069 = {S57769, 59, { &P18305,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-P936 Build/GRJ90)*/
const Device UA2980887 = {S57770, 140, { &P18305,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-pt; LG-P936/V10f Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1964541 = {S57771, 115, { &P12897,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-00/03.82; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.5.16.11*/
const Device UA2566919 = {S57772, 139, { &P19845,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE N9120 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2754075 = {S57773, 162, { &P13157,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755533 = {S57774, 168, { &P13157,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783657 = {S57775, 168, { &P13157,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783686 = {S57776, 168, { &P13157,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799204 = {S57777, 168, { &P13157,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation XE with Beats Audio z715e Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868008 = {S57778, 148, { &P13157,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Z715e Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2468523 = {S57779, 132, { &P18466,&P18110,&P19805,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.6 Safari/535.19 Silk-Accelerated=true*/
const Device UA2801180 = {S57780, 74, { &P17595,&P21721,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:22.0) Gecko/X Firefox/22.0*/
const Device UA2427281 = {S57781, 87, { &P15364,&P7133,&P24354,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; QQBrowser/7.0.4350.400)*/
const Device UA2427883 = {S57782, 94, { &P15364,&P7133,&P24354,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; QQBrowser/7.0.4350.400)*/
const Device UA2432390 = {S57783, 175, { &P15364,&P7133,&P24354,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/7.0.4350.400)*/
const Device UA2460262 = {S57784, 95, { &P15364,&P7133,&P24354,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; QQBrowser/7.0.4350.400)*/
const Device UA2460913 = {S57785, 182, { &P15364,&P7133,&P24354,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/7.0.4350.400)*/
const Device UA2486354 = {S57786, 175, { &P15364,&P7133,&P24354,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; QQBrowser/7.0.4350.400)*/
const Device UA2676667 = {S57787, 139, { &P13054,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR6425LVW Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2859661 = {S57788, 139, { &P13054,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6425LVW Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2878816 = {S57789, 150, { &P15364,&P21460,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.3; WOW64; Trident/7.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729)*/
const Device UA2891427 = {S57790, 83, { &P15364,&P21460,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.3; Trident/7.0; .NET4.0E; .NET4.0C)*/
const Device UA2944111 = {S57791, 171, { &P15364,&P21460,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.3; WOW64; Trident/7.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729)*/
const Device UA2999142 = {S57792, 95, { &P15364,&P21460,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.3; Win64; x64; Trident/7.0; .NET4.0E; .NET4.0C)*/
const Device UA3105840 = {S57793, 177, { &P15364,&P21460,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.3; WOW64; Trident/7.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3)*/
const Device UA2366882 = {S57794, 145, { &P20007,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT886 Build/6.7.3-179_YIC-10) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2670995 = {S57795, 101, { &P20743,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 720)*/
const Device UA2781352 = {S57796, 109, { &P20743,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 720; ANZ885)*/
const Device UA2936373 = {S57797, 111, { &P20743,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 720; Vodafone)*/
const Device UA4149 = {S57798, 91, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-SGH-i900/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900/1.0*/
const Device UA4150 = {S57799, 82, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900/1.0*/
const Device UA4159 = {S57800, 78, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-SGH-i900Orange/AFHH1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA7876 = {S57801, 85, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/JBHG5*/
const Device UA7878 = {S57802, 85, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/BUHJ2*/
const Device UA7880 = {S57803, 85, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/ARHG5*/
const Device UA7877 = {S57804, 85, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/ARHJ2*/
const Device UA7918 = {S57805, 85, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/ACHH1*/
const Device UA7924 = {S57806, 85, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/AEIB2*/
const Device UA7925 = {S57807, 85, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/AEHJ2*/
const Device UA7927 = {S57808, 85, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/ACHH3*/
const Device UA7928 = {S57809, 85, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/AMHK1*/
const Device UA7929 = {S57810, 85, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/AMHJ1*/
const Device UA27719 = {S57811, 85, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/AUHJ2*/
const Device UA27720 = {S57812, 85, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/AEHG4*/
const Device UA27724 = {S57813, 85, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/ACHJ2*/
const Device UA34241 = {S57814, 85, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/AMHG5*/
const Device UA34244 = {S57815, 85, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/MJHJ2*/
const Device UA34245 = {S57816, 85, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/NVHH2*/
const Device UA34247 = {S57817, 85, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/MIHH3*/
const Device UA63995 = {S57818, 70, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-SGH-i900/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA65979 = {S57819, 85, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/BAHK2*/
const Device UA65982 = {S57820, 85, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/JBHH1*/
const Device UA75133 = {S57821, 85, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V/AEHH2*/
const Device UA87098 = {S57822, 79, { &P12274,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i900V*/
const Device UA92942 = {S57823, 71, { &P15364,&P18110,&P17812,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/3.4; Linux) KHTML/3.4.0 (like Gecko)*/
const Device UA167782 = {S57824, 131, { &P18010,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A43 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018488 = {S57825, 55, { &P18010,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; A43 Build/FROYO)*/
const Device UA632625 = {S57826, 65, { &P15364,&P18110,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; en) Presto/2.10.229 Version/11.60*/
const Device UA2230179 = {S57827, 118, { &P17595,&P18207,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2*/
const Device UA2235386 = {S57828, 119, { &P17595,&P18207,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Version/5.1.1 Safari/536.26.14*/
const Device UA2502218 = {S57829, 125, { &P17595,&P18207,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_2; en-us) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3*/
const Device UA2554231 = {S57830, 118, { &P17595,&P18207,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7*/
const Device UA2652638 = {S57831, 119, { &P17595,&P18207,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22*/
const Device UA2668696 = {S57832, 119, { &P17595,&P18207,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) Version/5.1.1 Safari/536.28.10*/
const Device UA2872141 = {S57833, 117, { &P17595,&P18207,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) Version/5.1.1 Safari/536.30.1*/
const Device UA2997351 = {S57834, 125, { &P17595,&P18207,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_4; en-us) AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3*/
const Device UA3008717 = {S57835, 117, { &P17595,&P18207,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) Version/5.1.9 Safari/534.59.8*/
const Device UA2436547 = {S57836, 39, { &P15364,&P18580,&P22900,&P18092 }};	/*Useragent - Mercury/7.2 CFNetwork/609 Darwin/13.0.0*/
const Device UA2534835 = {S57837, 111, { &P21820,&P18110,&P17250,&P18092 }};	/*Useragent - TBD753B Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA3064646 = {S57838, 137, { &P22382,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2882683 = {S57839, 112, { &P18557,&P20706,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia308/05.85; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA3069029 = {S57840, 134, { &P15767,&P19826,&P24411,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/10B329 Safari/7534.48.3*/
const Device UA3080765 = {S57841, 134, { &P15767,&P19826,&P24411,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/10B146 Safari/7534.48.3*/
const Device UA3080772 = {S57842, 134, { &P15767,&P19826,&P24411,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/10B147 Safari/7534.48.3*/
const Device UA3084014 = {S57843, 132, { &P15767,&P19826,&P24411,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/10B141 Safari/7534.48.3*/
const Device UA50440 = {S57844, 10, { &P14839,&P5890,&P3439,&P18092 }};	/*Useragent - NOKIA6720c*/
const Device UA53968 = {S57845, 154, { &P14839,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia6720c-1b/012.008 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA64549 = {S57846, 151, { &P14839,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia6720c/012.008 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA870455 = {S57847, 104, { &P15364,&P17285,&P18198,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; de; rv:1.9.2.17) Gecko/X Thunderbird/3.1.10 ThunderBrowse/3.3.5*/
const Device UA2754813 = {S57848, 135, { &P21606,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LGL86C Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA27613 = {S57849, 98, { &P14603,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-B2100/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA43711 = {S57850, 112, { &P14603,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-B2100-ORANGE/B2100AFIF3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA103683 = {S57851, 113, { &P14603,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-B2100/BUIC6 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA371651 = {S57852, 113, { &P14603,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-B2100/BUJA1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA2263927 = {S57853, 113, { &P14603,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-B2100/BUIF2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA2423749 = {S57854, 136, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; toroplus Build/JOP40C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2531011 = {S57855, 134, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; i777 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2750706 = {S57856, 148, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; UTOOPadM80E Build/E8HD-ELITE-1280x768) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2953947 = {S57857, 138, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; YOUPU U6 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2988009 = {S57858, 140, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Full AOSP on Manta Build/JVP15P) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2997405 = {S57859, 130, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; X2-Lite Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2997546 = {S57860, 151, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; MBX reference board (g18ref) Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3010764 = {S57861, 130, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; X2-16GB Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3015324 = {S57862, 164, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9506 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3018972 = {S57863, 164, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SGH-T399 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3019134 = {S57864, 137, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; X130_HD Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3048514 = {S57865, 164, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SPH-L520 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3049380 = {S57866, 165, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E310L Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3050618 = {S57867, 164, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-au; SAMSUNG GT-I9197 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3057838 = {S57868, 130, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; STB9097 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3057999 = {S57869, 137, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Glory 2 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3058242 = {S57870, 164, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG-SM-C105A Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3073546 = {S57871, 164, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG-SGH-I257 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3073927 = {S57872, 165, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E310S Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3079746 = {S57873, 132, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; CRXT 1131 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3090463 = {S57874, 139, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ZTE V880H Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3091492 = {S57875, 169, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SCH-R960 USCC Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3099973 = {S57876, 150, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Full JellyBean on Mako Build/LGEP19) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3101208 = {S57877, 144, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; signal77_cu_jb Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3101468 = {S57878, 164, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG SPH-L600 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3103360 = {S57879, 126, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; S1A Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3109033 = {S57880, 173, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E310K/KKUAMH3 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3110338 = {S57881, 165, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; SAMSUNG-SGH-I337Z Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3123123 = {S57882, 164, { &P15364,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-au; SAMSUNG GT-I9507 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3068065 = {S57883, 136, { &P23776,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGP321 Build/10.1.1.A.1.307) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2158534 = {S57884, 62, { &P15786,&P5043,&P4030,&P18092 }};	/*Useragent - NokiaC2-05/2.0 (08.69) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2590032 = {S57885, 64, { &P15786,&P5043,&P4030,&P18092 }};	/*Useragent - NokiaC2-05.1/2.0 (08.69) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA3033373 = {S57886, 147, { &P18385,&P21637,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus 7 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA50386 = {S57887, 128, { &P13249,&P17484,&P4732,&P18092 }};	/*Useragent - SAMSUNG-SGH-A897/A897UCJC1; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.15304; U; en-US) Opera 9.50*/
const Device UA154129 = {S57888, 130, { &P15364,&P7133,&P22445,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Iron/7.0.520.0 Chrome/7.0.520.0 Safari/534.7*/
const Device UA2030912 = {S57889, 137, { &P20937,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR910L Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2561767 = {S57890, 137, { &P20937,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR910L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2510217 = {S57891, 93, { &P13552,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaC6-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2596443 = {S57892, 32, { &P13552,&P5129,&P24675,&P18092 }};	/*Useragent - NokiaC6-00/UCWEB8.8.0.212/50/800*/
const Device UA2763888 = {S57893, 102, { &P13552,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac6-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3067066 = {S57894, 132, { &P23187,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; V370 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068210 = {S57895, 133, { &P23187,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; V370 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA629616 = {S57896, 71, { &P15364,&P16110,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; U; en) Presto/2.10.229 Version/11.64*/
const Device UA3065339 = {S57897, 148, { &P22968,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HUAWEI P6-U06 Build/HuaweiP6-U06) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111088 = {S57898, 155, { &P22968,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HUAWEI P6-U06-orange Build/HuaweiP6-U06) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA650854 = {S57899, 119, { &P17595,&P18120,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/534.53.11 (KHTML, like Gecko) Version 4.0.5 Safari/534.53.11*/
const Device UA660583 = {S57900, 117, { &P17595,&P18120,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/534.52.7 (KHTML, like Gecko) Version 4.0.5 Safari/534.52.7*/
const Device UA734798 = {S57901, 144, { &P17595,&P18120,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en_US) AppleWebKit/530.19.2 (KHTML, like Gecko) Dreamweaver/11.0.4.4993 Version/4.0.2 Safari/530.19.2*/
const Device UA746273 = {S57902, 117, { &P17595,&P18120,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/534.55.3 (KHTML, like Gecko) Version 4.0.5 Safari/534.55.3*/
const Device UA758561 = {S57903, 117, { &P17595,&P18120,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/534.56.5 (KHTML, like Gecko) Version 4.0.5 Safari/534.56.5*/
const Device UA794714 = {S57904, 117, { &P17595,&P18120,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/533.21.1 (KHTML, like Gecko) Version 4.0.5 Safari/533.21.1*/
const Device UA1958131 = {S57905, 113, { &P17595,&P18120,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/536.25 (KHTML, like Gecko) Version 4.0.5 Safari/536.25*/
const Device UA2203573 = {S57906, 119, { &P17595,&P18120,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/536.26.14 (KHTML, like Gecko) Version 4.0.5 Safari/536.26.14*/
const Device UA2233544 = {S57907, 117, { &P17595,&P18120,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/534.57.7 (KHTML, like Gecko) Version 4.0.5 Safari/534.57.7*/
const Device UA2585272 = {S57908, 119, { &P17595,&P18120,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/536.28.10 (KHTML, like Gecko) Version 4.0.5 Safari/536.28.10*/
const Device UA2781675 = {S57909, 117, { &P17595,&P18120,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/536.30.1 (KHTML, like Gecko) Version 4.0.5 Safari/536.30.1*/
const Device UA2866706 = {S57910, 117, { &P17595,&P18120,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/534.59.8 (KHTML, like Gecko) Version 4.0.5 Safari/534.59.8*/
const Device UA633285 = {S57911, 142, { &P17977,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; IM-A710K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1931757 = {S57912, 66, { &P17977,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; IM-A710K Build/GINGERBREAD)*/
const Device UA1937057 = {S57913, 140, { &P17977,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; IM-A710K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2745288 = {S57914, 135, { &P17977,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; IM-A710K Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2843691 = {S57915, 154, { &P17977,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; IM-A710K Build/GINGERBREAD--XiaoXingGe) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA63788 = {S57916, 126, { &P16012,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia3710fold/5.0 (04.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA455972 = {S57917, 151, { &P18222,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonIS12S Build/6.0.D.0.272) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA599917 = {S57918, 151, { &P18222,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonIS12S Build/6.0.D.0.270) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA599975 = {S57919, 151, { &P18222,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonIS12S Build/6.0.D.0.271) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA603593 = {S57920, 139, { &P18222,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; IS12S Build/6.0.D.0.272) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA679145 = {S57921, 139, { &P18222,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; IS12S Build/6.0.D.0.271) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA862471 = {S57922, 151, { &P18222,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonIS12S Build/6.0.D.0.274) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA885516 = {S57923, 139, { &P18222,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; IS12S Build/6.0.D.0.274) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1997677 = {S57924, 151, { &P18222,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonIS12S Build/6.0.D.0.275) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029937 = {S57925, 139, { &P18222,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; IS12S Build/6.0.D.0.275) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2811251 = {S57926, 63, { &P18222,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; IS12S Build/6.0.D.0.274)*/
const Device UA2009033 = {S57927, 92, { &P15364,&P7133,&P22363,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:15.0) Gecko/X Firefox/15.0-x64 PaleMoon/15.0-x64*/
const Device UA2017656 = {S57928, 79, { &P15364,&P7133,&P22363,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X Firefox/15.0 PaleMoon/15.0*/
const Device UA2535968 = {S57929, 93, { &P14358,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae63-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2716555 = {S57930, 92, { &P14358,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE63) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2781080 = {S57931, 30, { &P14358,&P5890,&P24676,&P18092 }};	/*Useragent - NokiaE63/UCWEB8.9.0.253/28/999*/
const Device UA2886394 = {S57932, 134, { &P18680,&P18110,&P23482,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Safari/535.19 Silk-Accelerated=false*/
const Device UA3068194 = {S57933, 138, { &P18296,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One V Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2566281 = {S57934, 124, { &P15364,&P16110,&P22465,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Iron/25.0.1400.0 Chrome/25.0.1400.0 Safari/537.22*/
const Device UA2569450 = {S57935, 117, { &P15364,&P16110,&P22465,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.22 (KHTML, like Gecko) Iron/25.0.1400.0 Chrome/25.0.1400.0 Safari/537.22*/
const Device UA905828 = {S57936, 128, { &P23887,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T15A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2204925 = {S57937, 128, { &P23887,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T15A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2754384 = {S57938, 136, { &P18243,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P880 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2757188 = {S57939, 137, { &P18243,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P880g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3028293 = {S57940, 65, { &P15364,&P5690,&P23479,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:25.0) Gecko/X Firefox/25.0*/
const Device UA3108345 = {S57941, 83, { &P15364,&P5690,&P23479,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:25.0) Gecko/X Firefox/25.0 AlexaToolbar/amznf-3.0.X*/
const Device UA2974316 = {S57942, 130, { &P15932,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974502 = {S57943, 130, { &P15932,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P3110 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974117 = {S57944, 137, { &P23642,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-M840 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA850065 = {S57945, 82, { &P17595,&P17296,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en) Presto/2.10.289 Version/12.00*/
const Device UA2003681 = {S57946, 82, { &P17595,&P17296,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en) Presto/2.10.289 Version/12.01*/
const Device UA2391318 = {S57947, 130, { &P20831,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EGP007 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2469865 = {S57948, 162, { &P11747,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; HTC Desire Build/VJ CM10.1; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2563960 = {S57949, 162, { &P11747,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC Desire Build/VJ CM10.1; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752694 = {S57950, 159, { &P11747,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC Desire Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2705389 = {S57951, 133, { &P18464,&P18110,&P22588,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Safari/535.19 Silk-Accelerated=true*/
const Device UA8837 = {S57952, 78, { &P14478,&P6217,&P3418,&P18092 }};	/*Useragent - BlackBerry8350i/4.6.1.313 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/103*/
const Device UA1996039 = {S57953, 148, { &P12879,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7510 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2014323 = {S57954, 148, { &P12879,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7510 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2237715 = {S57955, 148, { &P12879,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7510 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2258137 = {S57956, 148, { &P12879,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7510 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2626464 = {S57957, 169, { &P12879,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-P7510 Build/JRO03L; CyanogenMod-AlexandrOid 4.1.2 Win 8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2627442 = {S57958, 155, { &P12879,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-P7510 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2242307 = {S57959, 43, { &P15364,&P18580,&P22243,&P18092 }};	/*Useragent - Flipboard/1.9.8 CFNetwork/609 Darwin/13.0.0*/
const Device UA2973487 = {S57960, 133, { &P19907,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C5155 Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974889 = {S57961, 133, { &P19907,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Rise Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3065369 = {S57962, 139, { &P15989,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21i Build/11.0.A.4.22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081695 = {S57963, 139, { &P15989,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21i2 Build/11.0.A.6.5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094718 = {S57964, 139, { &P15989,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21a Build/11.0.A.0.16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110308 = {S57965, 139, { &P15989,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21a2 Build/11.0.A.1.8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA766084 = {S57966, 147, { &P12657,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  HTC EVO 3D X515m Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA906653 = {S57967, 146, { &P12657,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515m Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA912271 = {S57968, 146, { &P12657,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515a Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2024546 = {S57969, 139, { &P12657,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC_X515E Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1960633 = {S57970, 130, { &P20983,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9308 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2029489 = {S57971, 139, { &P20983,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9308 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2376120 = {S57972, 61, { &P20983,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9308 Build/IMM76D)*/
const Device UA2368398 = {S57973, 80, { &P15364,&P18140,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux x86_64; Edition Linux Mint) Presto/2.12.388 Version/12.11*/
const Device UA2734630 = {S57974, 154, { &P22906,&P18066,&P18062,&P18092 }};	/*Useragent - Lenovo A390_ROW:Mozilla/5.0 (Linux; U;  Android 4.0.4; zh-cn; Lenovo A390_ROW/S030) AppleWebKit534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2868777 = {S57975, 146, { &P22906,&P18066,&P18062,&P18092 }};	/*Useragent - Lenovo-A390t_TD/S100 Release/11.2012 Mozilla/5.0 (Linux; U; Android 4.0.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956886 = {S57976, 143, { &P22906,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo A390t Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987715 = {S57977, 149, { &P22906,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A390 Build/LenovoLePhone) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2988957 = {S57978, 153, { &P22906,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lenovo A390_ROW Build/LenovoLePhone) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994044 = {S57979, 68, { &P22906,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo A390_ROW Build/IMM76I)*/
const Device UA2996239 = {S57980, 134, { &P22906,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo A390t Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2997775 = {S57981, 159, { &P22906,&P18066,&P18062,&P18092 }};	/*Useragent - Lenovo A390 Linux/3.0.13 Android/4.0.4 Release/04.03.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2998186 = {S57982, 146, { &P22906,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lenovo A390_ROW Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3048612 = {S57983, 134, { &P22906,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo A390t Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA3058152 = {S57984, 139, { &P22906,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;  Android 4.0.4; zh-cn; Lenovo A390_ROW/S030) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3101815 = {S57985, 140, { &P22906,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A390 Build/LenovoLePhone) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3121476 = {S57986, 153, { &P22906,&P18066,&P18062,&P18092 }};	/*Useragent - Lenovo A390_ROW:Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A390_ROW/S030) AppleWebKit534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA105112 = {S57987, 70, { &P15364,&P18110,&P17389,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/2.2.2; Linux 2.4.14-xfs; X11; i686)*/
const Device UA356849 = {S57988, 132, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; GT-P1000 Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA763544 = {S57989, 137, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA822126 = {S57990, 137, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2030379 = {S57991, 144, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033605 = {S57992, 61, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P1000 Build/IMM76L)*/
const Device UA2034253 = {S57993, 132, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P1000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555623 = {S57994, 132, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P1000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2609800 = {S57995, 142, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2610234 = {S57996, 61, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-P1000 Build/IML74K)*/
const Device UA2610760 = {S57997, 114, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/1.1.3.2 (Linux; U; Android 4.0.4; ko-KR; GT-P1000N Build/cm_p1n-eng 4.0.4 IMM76I eng.humberos.X.X test-keys)*/
const Device UA2611904 = {S57998, 129, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; GT-P1000 Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2612675 = {S57999, 141, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; GT-P1000 Build/MjhieuS3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2614672 = {S58000, 62, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P1000L Build/IMM76L)*/
const Device UA2615895 = {S58001, 157, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; GT-P1000 Build/Smartfren.andromaxtab7.0.v.27.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616001 = {S58002, 150, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA2616090 = {S58003, 139, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; vi-vn; GT-P1000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2616303 = {S58004, 132, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616809 = {S58005, 130, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P1000 Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2617192 = {S58006, 121, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Bu AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617193 = {S58007, 145, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA2618241 = {S58008, 67, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 4.0.4; en-US; GT-P1000 Build/IMM76I)*/
const Device UA2618519 = {S58009, 136, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P1000 Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2618748 = {S58010, 130, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P1000 Build/IMM76L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2619300 = {S58011, 130, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P1000 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2621360 = {S58012, 91, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IMM76L) Daily Mail Online/1.8.3*/
const Device UA2622092 = {S58013, 117, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.0.4; en-US; GT-P1000 Build/cm_p1-userdebug 4.0.4 IMM76L eng.jenkins.X.X test-keys)*/
const Device UA2622100 = {S58014, 150, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; GT-P1000 Build/Smartfren.androtab.v25.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622329 = {S58015, 139, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; GT-P1000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2622408 = {S58016, 162, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P1000 Build/IMM76L) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-X)*/
const Device UA2625139 = {S58017, 157, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/Smartfren.andromaxtab7.0.v.37.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2628305 = {S58018, 132, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2629402 = {S58019, 139, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P1000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2630289 = {S58020, 149, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; GT-P1000 Build/MjhieuGalaxyNote) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2631040 = {S58021, 127, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Groupon/2.1.2483 (Android 4.0.4; Samsung GT-P1000 / Samsung GT-P1000; )[preload=false;locale=en_US;clientidbase=android-google]*/
const Device UA2637636 = {S58022, 135, { &P13452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P1000 Build/R13.4-rc2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2242306 = {S58023, 47, { &P15364,&P18570,&P22243,&P18092 }};	/*Useragent - Flipboard/1.9.8 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2174370 = {S58024, 82, { &P22693,&P6590,&P4223,&P18092 }};	/*Useragent - NetFront/3.5.1(BREW 5.0.2.1; U; en-us; SAMSUNG; NetFront/3.5.1/WAP) PayLo M270-PLB*/
const Device UA2566766 = {S58025, 97, { &P15364,&P18110,&P21782,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) rekonq/2.2.1 Safari/534.34*/
const Device UA2571288 = {S58026, 97, { &P15364,&P18110,&P21782,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.21 (KHTML, like Gecko) rekonq/2.2.1 Safari/537.21*/
const Device UA2733518 = {S58027, 148, { &P23230,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Huawei-H867G Build/HuaweiH867G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2770770 = {S58028, 147, { &P18442,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2771507 = {S58029, 147, { &P18442,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L900 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2782336 = {S58030, 155, { &P18442,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2868314 = {S58031, 147, { &P18442,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T889 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2937950 = {S58032, 147, { &P18442,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7105 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2957403 = {S58033, 148, { &P18442,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I317M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2739354 = {S58034, 148, { &P23198,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI H868C Build/HuaweiH868C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103962 = {S58035, 70, { &P23198,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI H868C Build/HuaweiH868C)*/
const Device UA2975021 = {S58036, 141, { &P20399,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PantechP8010 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA30987 = {S58037, 102, { &P13226,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2120L/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA55178 = {S58038, 110, { &P13226,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2120B/E2120BDDIL1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA61639 = {S58039, 110, { &P13226,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2120i/E2120iDDJE1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA64577 = {S58040, 101, { &P13226,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2120/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA925480 = {S58041, 60, { &P15364,&P17188,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0; rv:14.0) Gecko/X Firefox/14.0.1*/
const Device UA23593 = {S58042, 140, { &P14913,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SPH-M820-BST Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24393 = {S58043, 138, { &P14913,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SPH-M820 Build/Build_Ver) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA197127 = {S58044, 64, { &P14913,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; SPH-M820-BST Build/FROYO)*/
const Device UA907435 = {S58045, 60, { &P15364,&P17188,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0; rv:13.0) Gecko/X Firefox/13.0.1*/
const Device UA3132857 = {S58046, 137, { &P18471,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-78) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA2873103 = {S58047, 142, { &P12280,&P22882,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/11A4372q Safari/8536.25*/
const Device UA3096889 = {S58048, 140, { &P12280,&P22882,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/11A465 Safari/8536.25*/
const Device UA3131033 = {S58049, 142, { &P12280,&P22882,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/11A501 Safari/8536.25*/
const Device UA241258 = {S58050, 135, { &P20971,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID7125 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458880 = {S58051, 59, { &P20971,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID7125 Build/GRI40)*/
const Device UA458950 = {S58052, 69, { &P20971,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.3; en_US; MID7125 Build/MID7125 X)*/
const Device UA657001 = {S58053, 154, { &P14872,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D700 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030999 = {S58054, 158, { &P14872,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D700 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580663 = {S58055, 138, { &P21506,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2868445 = {S58056, 139, { &P21506,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2507608 = {S58057, 114, { &P14711,&P5204,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-01/11.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA3024972 = {S58058, 162, { &P15575,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9100G Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i9100g*/
const Device UA3026276 = {S58059, 160, { &P15575,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9100 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i9100*/
const Device UA3062775 = {S58060, 160, { &P15575,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9100 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i9100*/
const Device UA3093361 = {S58061, 162, { &P15575,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9100G Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i9100g*/
const Device UA3110081 = {S58062, 160, { &P15575,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9100 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i9100*/
const Device UA3113435 = {S58063, 160, { &P15575,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9100 Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i9100*/
const Device UA3126461 = {S58064, 162, { &P15575,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-gb; GT-I9100 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2.0/i9100*/
const Device UA62966 = {S58065, 84, { &P11633,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-G600-VODA/G600ACHF1 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64049 = {S58066, 79, { &P11633,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-G600/G600MSGH2 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2430326 = {S58067, 132, { &P19852,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P7511 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2625041 = {S58068, 61, { &P19852,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P7511 Build/IMM76D)*/
const Device UA2896178 = {S58069, 156, { &P14635,&P5843,&P24678,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (08.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac3-00) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2386693 = {S58070, 108, { &P15364,&P16110,&P24152,&P18092 }};	/*Useragent - Mozilla/16.0 (Windows NT 6.2; Win64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/32.0.1297.0 Safari/8536.25*/
const Device UA3131440 = {S58071, 107, { &P15364,&P16110,&P24152,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/32.0.1659.2 Safari/537.36*/
const Device UA3131885 = {S58072, 100, { &P15364,&P16110,&P24152,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/32.0.1659.2 Safari/537.36*/
const Device UA3026648 = {S58073, 140, { &P22867,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5303 Build/12.0.A.2.245) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3059631 = {S58074, 133, { &P22867,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5303 Build/12.0.A.2.245) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2973887 = {S58075, 154, { &P20192,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0100 Build/HuaweiG510-0100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974374 = {S58076, 152, { &P20192,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Orange Daytona Build/HuaweiG510-0200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974460 = {S58077, 154, { &P20192,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0200 Build/HuaweiG510-0200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975698 = {S58078, 154, { &P20192,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0251 Build/HuaweiG510-0251) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023113 = {S58079, 154, { &P20192,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2032475 = {S58080, 151, { &P17172,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2234197 = {S58081, 151, { &P17172,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Transformer Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2234846 = {S58082, 151, { &P17172,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2513215 = {S58083, 93, { &P14748,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaE72-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2657769 = {S58084, 102, { &P14748,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiae72-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2748974 = {S58085, 93, { &P14748,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaE72-2) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2791987 = {S58086, 32, { &P14748,&P5890,&P24675,&P18092 }};	/*Useragent - NOKIAE72-1/UCWEB8.8.0.212/28/999*/
const Device UA2954406 = {S58087, 91, { &P14748,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaE72) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA140669 = {S58088, 25, { &P15364,&P17017,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 Firefox/3.6.8*/
const Device UA442714 = {S58089, 13, { &P15364,&P17017,&P17558,&P18092 }};	/*Useragent - Firefox/3.6.3*/
const Device UA530861 = {S58090, 26, { &P15364,&P17017,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 Firefox/3.6.12*/
const Device UA549045 = {S58091, 54, { &P15364,&P17017,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; rv:1.9.2) Gecko/X Firefox/3.6*/
const Device UA643848 = {S58092, 34, { &P15364,&P17017,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 Gecko/X Firefox/3.6.10*/
const Device UA1858931 = {S58093, 74, { &P15364,&P17017,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; U; Warp 4.5; en-US; rv:1.9.2.13) Gecko/X Firefox/3.6.13*/
const Device UA2737127 = {S58094, 14, { &P15364,&P17017,&P17558,&P18092 }};	/*Useragent - Firefox/3.6.27*/
const Device UA5643 = {S58095, 71, { &P12280,&P17090,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8G4 (iPhone; U; CPU OS 4_3_1 like Mac OS X; en_us)*/
const Device UA5667 = {S58096, 71, { &P12280,&P17090,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8F190 (iPhone; U; CPU OS 4_3 like Mac OS X; en_us)*/
const Device UA5674 = {S58097, 71, { &P12280,&P17090,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8H7 (iPhone; U; CPU OS 4_3_2 like Mac OS X; en_us)*/
const Device UA5714 = {S58098, 71, { &P12280,&P17090,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8J2 (iPhone; U; CPU OS 4_3_3 like Mac OS X; en_us)*/
const Device UA37144 = {S58099, 71, { &P12280,&P17090,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8L1 (iPhone; U; CPU OS 4_3_5 like Mac OS X; en_us)*/
const Device UA46560 = {S58100, 71, { &P12280,&P17090,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8K2 (iPhone; U; CPU OS 4_3_4 like Mac OS X; en_us)*/
const Device UA369860 = {S58101, 183, { &P15649,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-S5570 Build/GRI41; Cyanogen-MUSIC2.1ro.setupwizard.mode=OPTIONAL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627036 = {S58102, 155, { &P15649,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5570 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA696614 = {S58103, 155, { &P15649,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5570 Build/GRI41; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3065986 = {S58104, 133, { &P18665,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; N861 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068316 = {S58105, 138, { &P12968,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26i Build/6.1.A.2.45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA232607 = {S58106, 138, { &P22550,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; Titanium Build/ECLAIR) AppleWebKit/530.17+ (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA690353 = {S58107, 144, { &P22550,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; Titanium Build/ESE81) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1898812 = {S58108, 66, { &P22550,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.1-update1; Titanium Build/ESE81)*/
const Device UA23106 = {S58109, 36, { &P15565,&P17017,&P4223,&P18092 }};	/*Useragent - SPH-M575-VMUB NetFront/3.5.1 MMP/2.0*/
const Device UA206019 = {S58110, 133, { &P17955,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MID1024 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458701 = {S58111, 57, { &P17955,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; MID1024 Build/FRF91)*/
const Device UA458786 = {S58112, 83, { &P17955,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/0.9.1 (Linux; U; Android 2.2; en_US; MID1024 Build/MID Serials 2.2 FRF91 X.X)*/
const Device UA369230 = {S58113, 143, { &P20582,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; AN10G2-LN Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA573522 = {S58114, 140, { &P20582,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; AN10G2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA737339 = {S58115, 64, { &P20582,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.1; AN10G2 Build/GINGERBREAD)*/
const Device UA1932246 = {S58116, 143, { &P20582,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; fr-fr; AN10G2-LZ Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023393 = {S58117, 143, { &P20582,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; AN10G2-KN Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2204783 = {S58118, 138, { &P12814,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I777 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2238294 = {S58119, 138, { &P12814,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I777 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2877602 = {S58120, 146, { &P12814,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I777 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3065091 = {S58121, 126, { &P19775,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A210 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA576303 = {S58122, 70, { &P13123,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.5;Zh_cn;GT-S5360;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA686697 = {S58123, 70, { &P13123,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;GT-S5360;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA1946800 = {S58124, 119, { &P13123,&P5629,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.3.6;Zh_cn;GT-S5360;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/444*/
const Device UA668822 = {S58125, 70, { &P15364,&P17285,&P17260,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Windows NT 6.0) Gecko/X Firefox/3.5 NjuiceBot*/
const Device UA740868 = {S58126, 129, { &P15364,&P17285,&P17260,&P18091 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-GB; rv:1.9.1.2) Firefox/3.5.5 spellr.usBot/4.0 (http://checkdog.com/faqs.php) Gecko/X*/
const Device UA2383089 = {S58127, 129, { &P15364,&P17285,&P17260,&P18091 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-GB; rv:1.9.1.2) Firefox/3.5.5 spellr.usBot/4.4 (http://checkdog.com/faqs.php) Gecko/X*/
const Device UA2601662 = {S58128, 129, { &P15364,&P17285,&P17260,&P18091 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-GB; rv:1.9.1.2) Firefox/3.5.5 spellr.usBot/4.6 (http://checkdog.com/faqs.php) Gecko/X*/
const Device UA2863025 = {S58129, 129, { &P15364,&P17285,&P17260,&P18091 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-GB; rv:1.9.1.2) Firefox/3.5.5 spellr.usBot/4.9 (http://checkdog.com/faqs.php) Gecko/X*/
const Device UA2994151 = {S58130, 128, { &P15364,&P17285,&P17260,&P18091 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-GB; rv:1.9.1.2) Firefox/3.5.5 CheckDogBot/4.9 (http://checkdog.com/faqs.php) Gecko/X*/
const Device UA899798 = {S58131, 60, { &P18583,&P5043,&P3605,&P18092 }};	/*Useragent - Nokia110/2.0 (03.04) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2746170 = {S58132, 69, { &P18583,&P5043,&P3605,&P18092 }};	/*Useragent - Nokia110/2.0 (03.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia110*/
const Device UA2973461 = {S58133, 145, { &P19891,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974036 = {S58134, 126, { &P15869,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xoom Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2975546 = {S58135, 131, { &P15869,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ505 Build/Moto_MZ505) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022632 = {S58136, 125, { &P15869,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Xoom Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA612774 = {S58137, 168, { &P18465,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Kindle Fire Build/IMM76D; CyanogenMod-CM9 Reloaded) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA699823 = {S58138, 186, { &P18465,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Amazon Kindle Fire Build/IMM76D; CyanogenMod-CM9 Reloaded v1.3 OTTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA802477 = {S58139, 168, { &P18465,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Amazon Kindle Fire Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA900122 = {S58140, 173, { &P18465,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Amazon Kindle Fire Build/IML74k; CyanogenMod-Energy CM9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA905650 = {S58141, 164, { &P18465,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Amazon Kindle Fire Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1990651 = {S58142, 157, { &P18465,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Amazon Kindle Fire Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA879216 = {S58143, 137, { &P17160,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  Slider SL101 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA902696 = {S58144, 136, { &P17160,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Slider SL101 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2753958 = {S58145, 138, { &P18724,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-H Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754479 = {S58146, 138, { &P18724,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-F Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2769477 = {S58147, 130, { &P18724,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A2107A-H Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2770686 = {S58148, 133, { &P18724,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SmartTabII7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2973276 = {S58149, 139, { &P20076,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26w Build/6.2.B.0.200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2947255 = {S58150, 133, { &P18665,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; N861 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2971383 = {S58151, 126, { &P18665,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; N861 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3065243 = {S58152, 138, { &P23169,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT914 Build/2_32A_2012) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA85824 = {S58153, 99, { &P15364,&P18110,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Linux 2.4.18-rmk7-pxa3-embedix armv5tel; 480x640) Opera 6.0 [en]*/
const Device UA152322 = {S58154, 42, { &P15364,&P18110,&P17466,&P18092 }};	/*Useragent - Opera/6.x (Linux 2.4.8-26mdk i686; U) [en]*/
const Device UA600491 = {S58155, 57, { &P15364,&P18110,&P17466,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux 2.4.18-18.7.x i686; U) Opera 6.03 [en]*/
const Device UA898223 = {S58156, 73, { &P15364,&P18110,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Linux 2.4.20-4GB i686) Opera 6.03 [en]*/
const Device UA364205 = {S58157, 128, { &P18282,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; A200 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2024350 = {S58158, 57, { &P18282,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; A200 Build/HTK55D)*/
const Device UA699473 = {S58159, 64, { &P15364,&P16110,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; U; en) Presto/2.10.229 Version/11.61*/
const Device UA686949 = {S58160, 159, { &P15881,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Incredible 2 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1960684 = {S58161, 160, { &P15881,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Incredible 2 Build/GRI40; Condemned CM7 V1.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2670840 = {S58162, 103, { &P15364,&P17814,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.1pre) Gecko/X Ubuntu/8.10 (intrepid) Namoroka/3.6.1pre*/
const Device UA2567965 = {S58163, 74, { &P15364,&P17285,&P22369,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:19.0) Gecko/X Firefox/19.0 PaleMoon/19.0.2*/
const Device UA2584859 = {S58164, 90, { &P15364,&P17285,&P22369,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:19.0) Gecko/X Firefox/19.0 PaleMoon/19.0.2-x64*/
const Device UA2751091 = {S58165, 154, { &P22836,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ja-jp; SC-04E Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3502 = {S58166, 68, { &P15364,&P5448,&P22270,&P18092 }};	/*Useragent - Xda_Mantle/240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA5798 = {S58167, 73, { &P15364,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) USCCHTC6850*/
const Device UA7542 = {S58168, 75, { &P15364,&P5448,&P22270,&P18092 }};	/*Useragent - Treo850/v0100 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA13456 = {S58169, 69, { &P15364,&P5448,&P22270,&P18092 }};	/*Useragent - Xda_diamond/480x640 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA20426 = {S58170, 82, { &P15364,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) i-mate 9502; 480x640*/
const Device UA21377 = {S58171, 80, { &P15364,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) T-Mobile_Cleopatra*/
const Device UA50778 = {S58172, 116, { &P15364,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) ASUS-P552w/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA59266 = {S58173, 99, { &P15364,&P5448,&P22270,&P18092 }};	/*Useragent - HPiPAQVoiceMessenger/1.0/Mozilla/4.0  (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) C11G25L/1.0*/
const Device UA69544 = {S58174, 82, { &P15364,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) i-mate 8150/ 480x640*/
const Device UA87102 = {S58175, 76, { &P15364,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Sprint:MotoQ9c*/
const Device UA87407 = {S58176, 122, { &P15364,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) 320x320; VZW; UTStar-XV6800; Window Mobile 6.1 Professional;*/
const Device UA87628 = {S58177, 74, { &P15364,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) ASUSP750/1.0*/
const Device UA87891 = {S58178, 106, { &P15364,&P5448,&P22270,&P18092 }};	/*Useragent - HPiPAQVoiceMessenger/1.0/Mozilla/4.0  (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Smartphone;240x320*/
const Device UA88420 = {S58179, 76, { &P15364,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Alltel:MotoQ9c*/
const Device UA265670 = {S58180, 115, { &P15364,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) ASUS-P565/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA508746 = {S58181, 92, { &P15364,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; X610/2.0 Profile*/
const Device UA2131981 = {S58182, 74, { &P15364,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_Touch_3G Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA3020534 = {S58183, 74, { &P15364,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE, BrailleNote; IEMobile 7.11)*/
const Device UA625621 = {S58184, 134, { &P21528,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PMI920 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022425 = {S58185, 58, { &P21528,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; PMI920 Build/GRJ22)*/
const Device UA2601192 = {S58186, 128, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Clear Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2601193 = {S58187, 134, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N900 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2601594 = {S58188, 140, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MOBHIG0080 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602439 = {S58189, 134, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 9300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2606245 = {S58190, 130, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAD1001 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667633 = {S58191, 129, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M1005D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667634 = {S58192, 137, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A13-DLR Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668199 = {S58193, 132, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; mfld_dv10 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22*/
const Device UA2668619 = {S58194, 128, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M1005 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2702481 = {S58195, 143, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tmn smart a18 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2703288 = {S58196, 132, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N1 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2706412 = {S58197, 125, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T7 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2707254 = {S58198, 139, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Explosion Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2708394 = {S58199, 135, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; 740RS Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2708614 = {S58200, 139, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB9008GB Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2710634 = {S58201, 142, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BASE_Lutea_3 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2732442 = {S58202, 135, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BT210 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2732946 = {S58203, 138, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Infinity Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2733057 = {S58204, 132, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; CTA Build/ICS.casper.X) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2750272 = {S58205, 129, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AN13FP Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2766759 = {S58206, 133, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; QW TB-1217 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2766963 = {S58207, 137, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A11Note Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2794474 = {S58208, 139, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; StarPAD 7 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2794869 = {S58209, 128, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M1001 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2856353 = {S58210, 146, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; INFINITY_PASSION Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2861424 = {S58211, 133, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Q75 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2862256 = {S58212, 134, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SmartPadv10 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2938217 = {S58213, 135, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TM450 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2954102 = {S58214, 135, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JY139 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2958431 = {S58215, 127, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; R10S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22*/
const Device UA2988715 = {S58216, 128, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CT710 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2989982 = {S58217, 133, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; QW TB-1207 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2990195 = {S58218, 130, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GEM9212 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA3050541 = {S58219, 139, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Genius G1 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA3050630 = {S58220, 136, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB292 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA3056644 = {S58221, 138, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; K-BOOKMW Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA3086957 = {S58222, 134, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A721 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA3086970 = {S58223, 139, { &P15364,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MTC_968 Build/ICECREAM) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2768443 = {S58224, 136, { &P23179,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; ZP980 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998182 = {S58225, 58, { &P23179,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; ZP980 Build/JOP40D)*/
const Device UA3107879 = {S58226, 137, { &P23179,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; ZP980H Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2404601 = {S58227, 159, { &P14708,&P5204,&P24673,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5130c-2) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2457924 = {S58228, 153, { &P14708,&P5204,&P24673,&P18092 }};	/*Useragent - Nokia5130/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5130) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2566590 = {S58229, 58, { &P15364,&P17188,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0; rv:20.0) Gecko/X Firefox/20.0*/
const Device UA2755111 = {S58230, 133, { &P18295,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; One X Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755523 = {S58231, 138, { &P18295,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One X Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2796158 = {S58232, 138, { &P18295,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; HTC One X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA49501 = {S58233, 131, { &P18083,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; V9C Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020352 = {S58234, 55, { &P18083,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; V9C Build/GRJ22)*/
const Device UA2658977 = {S58235, 136, { &P18083,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Light Tab Build/GRJ22) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2719154 = {S58236, 154, { &P21968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ALCATEL ONE TOUCH 4030D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2935902 = {S58237, 76, { &P21968,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ALCATEL ONE TOUCH 4030D Build/JRO03C)*/
const Device UA2029762 = {S58238, 144, { &P16598,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A500 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032485 = {S58239, 186, { &P16598,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Iconia A500 Build/JRO03H; CyanogenMod-Tegraowners JB ROM v8 (thor & digetx)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033035 = {S58240, 144, { &P16598,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A500 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2235600 = {S58241, 187, { &P16598,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Iconia A500 Build/JRO03L; CyanogenMod-Tegraowners JB ROM v12 (thor & digetx)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2362866 = {S58242, 166, { &P16598,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A500 Build/JZO54K; CyanogenMod-Android 4.1.2 CM10 build) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2458281 = {S58243, 136, { &P23090,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9400 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984317 = {S58244, 58, { &P23090,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; i9400 Build/IMM76D)*/
const Device UA906404 = {S58245, 131, { &P21062,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; PP4MT-7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2981098 = {S58246, 60, { &P21062,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; PP4MT-7 Build/IML74K)*/
const Device UA2200193 = {S58247, 115, { &P15364,&P17285,&P22462,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4*/
const Device UA2236471 = {S58248, 122, { &P15364,&P17285,&P22462,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4*/
const Device UA2676921 = {S58249, 138, { &P18296,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One V Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2750808 = {S58250, 138, { &P18296,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One V Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA24246 = {S58251, 137, { &P17947,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; HTC_Lexikon Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA28091 = {S58252, 122, { &P12092,&P5208,&P18244,&P18092 }};	/*Useragent - Mozilla/5.0 (BREW; U; BREW 3.1.5; en) AppleWebKit/525.26 (KHTML, like Gecko) Polaris/7.0 Safari/525.26 480X800 LGE VX11000*/
const Device UA253773 = {S58253, 35, { &P15364,&P17017,&P17201,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 10.0;*/
const Device UA2509625 = {S58254, 53, { &P15364,&P17017,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; PPC; Trident/3.0)*/
const Device UA2654753 = {S58255, 50, { &P15364,&P17017,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; ; Trident/6.0)*/
const Device UA2791812 = {S58256, 58, { &P15364,&P17017,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; PPC; SV1; Trident/4.0)*/
const Device UA687567 = {S58257, 81, { &P12449,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE5-00)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA1946716 = {S58258, 32, { &P12449,&P5890,&P24667,&P18092 }};	/*Useragent - NOKIAE5-00/UCWEB8.2.0.116/28/800*/
const Device UA2164215 = {S58259, 83, { &P12449,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE5-00.2)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA2319956 = {S58260, 89, { &P12449,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokiae5-00) UCBrowser8.2.1.144/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA2945033 = {S58261, 155, { &P23440,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Vodafone 975 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30 SVN/130HVG2*/
const Device UA3027309 = {S58262, 160, { &P18645,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L710 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2spr*/
const Device UA3032721 = {S58263, 168, { &P18645,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I747 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2att*/
const Device UA3062471 = {S58264, 168, { &P18645,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I747 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2att*/
const Device UA3063532 = {S58265, 160, { &P18645,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L710 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2spr*/
const Device UA3083913 = {S58266, 160, { &P18645,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-au; GT-I9305 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i9305*/
const Device UA3096094 = {S58267, 168, { &P18645,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I747 Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2att*/
const Device UA3109618 = {S58268, 170, { &P18645,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I747 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2.0/d2att*/
const Device UA3110831 = {S58269, 160, { &P18645,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L710 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2spr*/
const Device UA3114331 = {S58270, 160, { &P18645,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L710 Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2spr*/
const Device UA3116184 = {S58271, 168, { &P18645,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I747 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/d2att*/
const Device UA2361448 = {S58272, 137, { &P18278,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-05D Build/V08R31C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583234 = {S58273, 137, { &P18278,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-05D Build/V09R32B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2844882 = {S58274, 59, { &P18278,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; F-05D Build/V08R31C)*/
const Device UA2882591 = {S58275, 137, { &P18278,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-05D Build/V11R40A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3061213 = {S58276, 137, { &P18278,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-05D Build/V12R41A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567576 = {S58277, 134, { &P18466,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2582683 = {S58278, 137, { &P20240,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E971 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3029352 = {S58279, 142, { &P19763,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6602 Build/10.3.1.A.0.244) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3029362 = {S58280, 135, { &P19763,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6602 Build/10.3.1.A.0.244) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2974238 = {S58281, 129, { &P17687,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MZ604 Build/I.7.1-34) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974359 = {S58282, 129, { &P17687,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ604 Build/I.7.1-45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA16384 = {S58283, 64, { &P14977,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia5000d-2/1.0 (04.89) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA53776 = {S58284, 65, { &P14977,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia5000d-2b/1.0 (03.54) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA61703 = {S58285, 61, { &P14977,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia5000/1.0 (06.32) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA250470 = {S58286, 136, { &P18045,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM003SH Build/S4030) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA348736 = {S58287, 136, { &P18045,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM003SH Build/S4040) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA609468 = {S58288, 136, { &P18045,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM003SH Build/S4080) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622475 = {S58289, 142, { &P18045,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM003SH Build/S4080) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30*/
const Device UA648738 = {S58290, 136, { &P18045,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM003SH Build/S4070) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA658693 = {S58291, 60, { &P18045,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SBM003SH Build/S4080)*/
const Device UA2679624 = {S58292, 136, { &P18045,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM003SH Build/S4090) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2774304 = {S58293, 136, { &P18045,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM003SH Build/S4110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2769313 = {S58294, 165, { &P18556,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Barnes & Noble Nook HD+ Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2866452 = {S58295, 172, { &P18556,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Barnes & Noble Nook HD+ Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2896176 = {S58296, 159, { &P18556,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; BN NookHD+ Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1/ovation*/
const Device UA2784136 = {S58297, 131, { &P15767,&P19826,&P20783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/10B329 Safari/7534.48.3*/
const Device UA2796269 = {S58298, 129, { &P15767,&P19826,&P20783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/10B141 Safari/7534.48.3*/
const Device UA2800097 = {S58299, 131, { &P15767,&P19826,&P20783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/10B146 Safari/7534.48.3*/
const Device UA2964453 = {S58300, 131, { &P15767,&P19826,&P20783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/10B147 Safari/7534.48.3*/
const Device UA2884321 = {S58301, 156, { &P14826,&P5843,&P24678,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac1-01) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2567092 = {S58302, 130, { &P17687,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ604 Build/I.7.1-45) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567222 = {S58303, 130, { &P17687,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MZ604 Build/I.7.1-34) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2753277 = {S58304, 138, { &P18606,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754579 = {S58305, 131, { &P18606,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2782538 = {S58306, 154, { &P22963,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ALCATEL ONE TOUCH 4030A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2892941 = {S58307, 76, { &P22963,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ALCATEL ONE TOUCH 4030A Build/JRO03C)*/
const Device UA627493 = {S58308, 86, { &P15364,&P18119,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; nl; rv:1.8.1.3) Gecko/X Firefox/2.0.0.3 (Ubuntu-edgy)*/
const Device UA1884374 = {S58309, 89, { &P15364,&P18119,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-GB; rv:1.8.1b1) Gecko/X BonEcho/2.0b1 (Ubuntu-edgy)*/
const Device UA1958091 = {S58310, 139, { &P21112,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; OPPOX907 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027669 = {S58311, 140, { &P21112,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; th-TH; OPPOX9017 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257611 = {S58312, 126, { &P21112,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; X907 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2574685 = {S58313, 57, { &P21112,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; X907 Build/IML74K)*/
const Device UA2788755 = {S58314, 58, { &P21112,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; X9017 Build/IML74K)*/
const Device UA2791412 = {S58315, 136, { &P21112,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru; OPPOX907 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA251355 = {S58316, 140, { &P13815,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Wildfire Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA308909 = {S58317, 140, { &P13815,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Wildfire Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA315930 = {S58318, 140, { &P13815,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Wildfire Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA731461 = {S58319, 64, { &P13815,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Wildfire Build/GRI40)*/
const Device UA2018434 = {S58320, 64, { &P13815,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; HTC Wildfire Build/GRI40)*/
const Device UA2019429 = {S58321, 140, { &P13815,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Wildfire Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021769 = {S58322, 128, { &P13815,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Wildfire Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022116 = {S58323, 140, { &P13815,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Wildfire Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2381293 = {S58324, 137, { &P18470,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ARCHOS 101G10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2754270 = {S58325, 126, { &P15869,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xoom Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754637 = {S58326, 131, { &P15869,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ505 Build/Moto_MZ505) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2783420 = {S58327, 125, { &P15869,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Xoom Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2974791 = {S58328, 129, { &P20991,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ME371MG Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3065087 = {S58329, 135, { &P18468,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; IdeaTabA2109A Build/JRO03R) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2566635 = {S58330, 140, { &P18225,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; U9200 Build/HuaweiU9200) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583345 = {S58331, 144, { &P18225,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U9202L-1 Build/HuaweiU9202L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2769619 = {S58332, 133, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; H100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784002 = {S58333, 137, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CrystalQuadCore Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2797695 = {S58334, 136, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ThinkPadTablet Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2798449 = {S58335, 134, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A9600 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798861 = {S58336, 126, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MiSS Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2798897 = {S58337, 133, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SZENIO 2500 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2799092 = {S58338, 127, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; y7053 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2799390 = {S58339, 133, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B943 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2866911 = {S58340, 124, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; G1 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2867281 = {S58341, 135, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; H9500+ Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867440 = {S58342, 135, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MD7305 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869118 = {S58343, 129, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PMID720 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2872917 = {S58344, 133, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SZENIO 2000 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2877856 = {S58345, 137, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; YOUPU U6 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877862 = {S58346, 128, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AN7IG3 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2877880 = {S58347, 134, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Boost Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879202 = {S58348, 131, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; M7 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879209 = {S58349, 134, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.4; Note3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892770 = {S58350, 128, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Thrive Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2893066 = {S58351, 138, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XTAB-781+ Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893353 = {S58352, 133, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; W125 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893439 = {S58353, 127, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; it900 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2893562 = {S58354, 134, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; eZee Tab1004 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2937726 = {S58355, 127, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; DPS Dual 7 Build/X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2937790 = {S58356, 124, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; V2 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2937822 = {S58357, 133, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; F600 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937895 = {S58358, 126, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; V72A Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2938130 = {S58359, 133, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Tablet-PC-4 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2960182 = {S58360, 133, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; I110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2999353 = {S58361, 128, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MaxiIQ Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2999790 = {S58362, 138, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AMOI N820 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3019947 = {S58363, 130, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; iQTab 3G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3058281 = {S58364, 132, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Engage 8 Build/Engage 8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3093458 = {S58365, 133, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; EG98 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3103391 = {S58366, 150, { &P15364,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI D2-0082 Build/HuaweiD2-0082) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3016433 = {S58367, 74, { &P15364,&P18110,&P24330,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:23.0) Gecko/X Firefox/23.0 SeaMonkey/2.20*/
const Device UA3107852 = {S58368, 63, { &P15364,&P18110,&P24330,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:23.0) Gecko/X SeaMonkey/2.20*/
const Device UA3065738 = {S58369, 139, { &P18447,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT30p Build/7.0.A.1.303) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067898 = {S58370, 133, { &P20760,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Event Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA252026 = {S58371, 122, { &P15364,&P17285,&P18210,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 6.0; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.1.0.5001 Safari/532.4*/
const Device UA788745 = {S58372, 101, { &P15364,&P17285,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6000.0);en-US;kml:2.2;client:Free;type:default)*/
const Device UA2306474 = {S58373, 101, { &P15364,&P17285,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6000.0);de-DE;kml:2.2;client:Free;type:default)*/
const Device UA2807880 = {S58374, 98, { &P15364,&P17285,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6000.0);en;kml:2.2;client:Free;type:default)*/
const Device UA909465 = {S58375, 74, { &P15364,&P7133,&P22535,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:10.0.3) Gecko/X Firefox/10.0.3 Epic/10.0.3*/
const Device UA2224755 = {S58376, 142, { &P12897,&P5843,&P24668,&P18092 }};	/*Useragent - NokiaC2-00/2.0 (03.82) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac2-00) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2177019 = {S58377, 142, { &P13123,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2445962 = {S58378, 66, { &P13123,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 4.1.1; GT-S5360 Build/GINGERBREAD)*/
const Device UA2426085 = {S58379, 147, { &P20090,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; MyPhone A848 Duo Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2459478 = {S58380, 148, { &P20090,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; MyPhone A848i Duo Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567300 = {S58381, 148, { &P21646,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; BLU STUDIO 5.3 II Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2890330 = {S58382, 70, { &P21646,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; BLU STUDIO 5.3 II Build/JZO54K)*/
const Device UA738539 = {S58383, 97, { &P15364,&P5690,&P22331,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.2) Gecko/X Firefox/3.6.2 CometBird/3.6.2*/
const Device UA756933 = {S58384, 103, { &P15364,&P5690,&P22331,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN; rv:1.9.2.13) Gecko/X Firefox/3.6.13 CometBrowser/3.6.13*/
const Device UA2209315 = {S58385, 100, { &P15364,&P5690,&P22331,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.2.16) Gecko/X Firefox/3.6.16 CometBird/3.6.16*/
const Device UA2559222 = {S58386, 139, { &P21488,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A860K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2582763 = {S58387, 149, { &P21488,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A860K/A860K1.20 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2671472 = {S58388, 144, { &P21488,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A860K/1.29 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2754152 = {S58389, 144, { &P21488,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A860K/1.38 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2792958 = {S58390, 61, { &P21488,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A860K Build/JZO54K)*/
const Device UA2973723 = {S58391, 144, { &P21488,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A860K/1.43 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597397 = {S58392, 139, { &P20871,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810P Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA628260 = {S58393, 115, { &P18267,&P5043,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-02/10.91; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.11*/
const Device UA924847 = {S58394, 60, { &P15364,&P7127,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:14.0) Gecko/X Firefox/14.0.1*/
const Device UA2041517 = {S58395, 82, { &P17595,&P17295,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.10.289 Version/12.02*/
const Device UA2258045 = {S58396, 149, { &P16168,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; NookColor Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA605699 = {S58397, 135, { &P20966,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; T-01D Build/V41R39C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627688 = {S58398, 133, { &P20966,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; T-01D Build/F0001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA640343 = {S58399, 135, { &P20966,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; T-01D Build/V39R38A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA905085 = {S58400, 135, { &P20966,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; T-01D Build/V43R41C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1955536 = {S58401, 135, { &P20966,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; T-01D Build/V40R39B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031996 = {S58402, 59, { &P20966,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; T-01D Build/V43R41C)*/
const Device UA907 = {S58403, 65, { &P15210,&P5448,&P3430,&P18092 }};	/*Useragent - SAMSUNG-GT-i8000T Opera 9.5 (Windows CE; Opera Mobi; U; en) DVJG1*/
const Device UA16320 = {S58404, 63, { &P15210,&P5448,&P3430,&P18092 }};	/*Useragent - SAMSUNG-GT-i8000T/1.0 (Windows CE; Opera Mobi; U; en) Opera 9.5*/
const Device UA16322 = {S58405, 96, { &P15210,&P5448,&P3430,&P18092 }};	/*Useragent - SAMSUNG-GT-i8000/1.0 Opera 9.5 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA16364 = {S58406, 66, { &P15210,&P5448,&P3430,&P18092 }};	/*Useragent - SAMSUNG-GT-i8000V/NXXJD6 (Windows CE; Opera Mobi; U; en) Opera 9.5*/
const Device UA24063 = {S58407, 63, { &P15210,&P5448,&P3430,&P18092 }};	/*Useragent - SAMSUNG-GT-i8000H/1.0 (Windows CE; Opera Mobi; U; en) Opera 9.5*/
const Device UA30228 = {S58408, 62, { &P15210,&P5448,&P3430,&P18092 }};	/*Useragent - SAMSUNG-GT-i8000/1.0 (Windows CE; Opera Mobi; U; en) Opera 9.5*/
const Device UA34473 = {S58409, 70, { &P15210,&P5448,&P3430,&P18092 }};	/*Useragent - SAMSUNG-GT-i8000Orange/BVIJ1 (Windows CE; Opera Mobi; U; en) Opera 9.5*/
const Device UA2569098 = {S58410, 103, { &P15364,&P18346,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2936471 = {S58411, 144, { &P19842,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; HTC Butterfly Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937232 = {S58412, 143, { &P19842,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-tw; HTC_Butterfly Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957936 = {S58413, 65, { &P19842,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC Butterfly Build/JDQ39)*/
const Device UA3017808 = {S58414, 143, { &P19842,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-tw; HTC Butterfly Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058485 = {S58415, 139, { &P19842,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HTC_X920e Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058567 = {S58416, 139, { &P19842,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HTC X920e Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1373 = {S58417, 64, { &P12309,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2680s-2/1.0 (05.28) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA1381 = {S58418, 77, { &P12309,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2680s-2/1.0 (06.17) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2680s-2*/
const Device UA23386 = {S58419, 65, { &P12309,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2680s-2b/1.0 (05.28) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA30359 = {S58420, 62, { &P12309,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2680s/1.0 (06.17) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA55143 = {S58421, 67, { &P12309,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2680s_CMCC/1.0 (06.17) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2753617 = {S58422, 139, { &P15989,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21i Build/11.0.A.4.22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769538 = {S58423, 139, { &P15989,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21i2 Build/11.0.A.6.5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769872 = {S58424, 139, { &P15989,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21a Build/11.0.A.0.16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869189 = {S58425, 139, { &P15989,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21a2 Build/11.0.A.1.8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2205701 = {S58426, 44, { &P17595,&P18570,&P18534,&P18092 }};	/*Useragent - Reeder/1.5.7 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA676854 = {S58427, 97, { &P15364,&P18145,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; FreeBSD amd64; en-US; rv:1.8.1.20) Gecko/X Firefox/2.0.0.20 Kazehakase/0.5.4*/
const Device UA2389090 = {S58428, 155, { &P12897,&P5843,&P24673,&P18092 }};	/*Useragent - NokiaC2-00/2.0 (03.82) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac2-00) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA30056 = {S58429, 133, { &P12280,&P17084,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B117 Twitter for iPhone*/
const Device UA77773 = {S58430, 182, { &P12695,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Moto_ME860 Build/Blur_Version.45.0.25.ME860.Retail.en.05 Flex/P011) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA353977 = {S58431, 144, { &P12695,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ME860 Build/4.5.3-118_OLY-14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA716802 = {S58432, 68, { &P12695,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; ME860 Build/4.5.2A-74_OLH-25)*/
const Device UA719030 = {S58433, 144, { &P12695,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ME860 Build/4.5.2A-74_OLH-25) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA768234 = {S58434, 132, { &P12695,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ME860 Build/4.5.3-118_OLY-14) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1944829 = {S58435, 182, { &P12695,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; Moto_ME860 Build/Blur_Version.45.29.0.ME860.Retail.en.05 Flex/P011) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021427 = {S58436, 132, { &P12695,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ME860 Build/4.5.2A-74_OLH-25) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022759 = {S58437, 143, { &P12695,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME860 Build/4.5.3-118_OLY-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029406 = {S58438, 137, { &P12695,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME860 Build/4.5.2A-74_OLH-25) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2031985 = {S58439, 137, { &P12695,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ME860 Build/4.5.3-118_OLY-14) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2812102 = {S58440, 144, { &P12695,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; ME860 Build/4.5.2A-74_OLH-29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2570121 = {S58441, 161, { &P16292,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2772910 = {S58442, 160, { &P16292,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2772922 = {S58443, 161, { &P16292,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA905949 = {S58444, 60, { &P15364,&P7127,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:13.0) Gecko/X Firefox/13.0.1*/
const Device UA2365867 = {S58445, 133, { &P20145,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; NS-13T001 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2708780 = {S58446, 129, { &P22594,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708847 = {S58447, 136, { &P22594,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA26641 = {S58448, 48, { &P13616,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-Torino/ UP.Browser/7.2.7.2.561 (GUI) MMP/2.0*/
const Device UA1970112 = {S58449, 70, { &P17595,&P18207,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.12A269 (Macintosh; U; Intel Mac OS X 10_8; en_us)*/
const Device UA2186295 = {S58450, 71, { &P17595,&P18207,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.12B19 (Macintosh; U; Intel Mac OS X 10_8_1; en_us)*/
const Device UA2200542 = {S58451, 71, { &P17595,&P18207,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.12C54 (Macintosh; U; Intel Mac OS X 10_8_2; en_us)*/
const Device UA2231047 = {S58452, 71, { &P17595,&P18207,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.12C60 (Macintosh; U; Intel Mac OS X 10_8_2; en_us)*/
const Device UA2404344 = {S58453, 73, { &P17595,&P18207,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.12C3006 (Macintosh; U; Intel Mac OS X 10_8_2; en_us)*/
const Device UA2458314 = {S58454, 73, { &P17595,&P18207,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.12C3012 (Macintosh; U; Intel Mac OS X 10_8_2; en_us)*/
const Device UA2466673 = {S58455, 73, { &P17595,&P18207,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.12C2034 (Macintosh; U; Intel Mac OS X 10_8_2; en_us)*/
const Device UA2502202 = {S58456, 71, { &P17595,&P18207,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.12C60 (Macintosh; U; Intel Mac OS X 10_8_2; de_de)*/
const Device UA2557998 = {S58457, 73, { &P17595,&P18207,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.12C3103 (Macintosh; U; Intel Mac OS X 10_8_2; en_us)*/
const Device UA2584146 = {S58458, 71, { &P17595,&P18207,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.12D78 (Macintosh; U; Intel Mac OS X 10_8_3; en_us)*/
const Device UA2799901 = {S58459, 71, { &P17595,&P18207,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.12E55 (Macintosh; U; Intel Mac OS X 10_8_4; en_us)*/
const Device UA3001780 = {S58460, 71, { &P17595,&P18207,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.12E55 (Macintosh; U; Intel Mac OS X 10_8_4; de_de)*/
const Device UA3002282 = {S58461, 73, { &P17595,&P18207,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.12E3067 (Macintosh; U; Intel Mac OS X 10_8_4; en_us)*/
const Device UA3088849 = {S58462, 71, { &P17595,&P18207,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.12E55 (Macintosh; U; Intel Mac OS X 10_8_4; fr_fr)*/
const Device UA3096134 = {S58463, 71, { &P17595,&P18207,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.12F37 (Macintosh; U; Intel Mac OS X 10_8_5; en_us)*/
const Device UA2030987 = {S58464, 139, { &P21010,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMID4311 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2602529 = {S58465, 155, { &P18680,&P18066,&P21708,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA909361 = {S58466, 137, { &P19784,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LIFETAB_P9514 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2553486 = {S58467, 139, { &P18637,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9260 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2709404 = {S58468, 139, { &P18637,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9260 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778997 = {S58469, 61, { &P18637,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9260 Build/JZO54K)*/
const Device UA2867191 = {S58470, 139, { &P18637,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-i9260 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950818 = {S58471, 61, { &P18637,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-I9260 Build/JRO03C)*/
const Device UA3086335 = {S58472, 130, { &P18637,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9260 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2571694 = {S58473, 69, { &P18254,&P7480,&P18418,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; TourlentaScanner/0.6; +http://tourlenta.com)*/
const Device UA2677092 = {S58474, 73, { &P18254,&P7480,&P18418,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; parsijoo-search-engine +http://www.parsijoo.ir/)*/
const Device UA2702722 = {S58475, 38, { &P18254,&P7480,&P18418,&P18092 }};	/*Useragent - AskQuickly v2 (http://askquickly.org/)*/
const Device UA2796345 = {S58476, 19, { &P18254,&P7480,&P18418,&P18092 }};	/*Useragent - mb-searchengine 1.0*/
const Device UA2975015 = {S58477, 145, { &P12814,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I777 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975811 = {S58478, 137, { &P12814,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I777 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2405710 = {S58479, 135, { &P17595,&P18207,&P22463,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11*/
const Device UA2580010 = {S58480, 135, { &P17595,&P18207,&P22463,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11*/
const Device UA2799108 = {S58481, 135, { &P17595,&P18207,&P22463,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11*/
const Device UA3115273 = {S58482, 135, { &P17595,&P18207,&P22463,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11*/
const Device UA781050 = {S58483, 140, { &P19859,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1985887 = {S58484, 131, { &P19859,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2027438 = {S58485, 126, { &P19859,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2360227 = {S58486, 62, { &P19859,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC T328w Build/IML74K)*/
const Device UA2474664 = {S58487, 131, { &P19859,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2460776 = {S58488, 172, { &P16292,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JOP40D; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2675095 = {S58489, 171, { &P16292,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39; LiquidSmooth-Liquid-JB-v2.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2975712 = {S58490, 176, { &P16292,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/toro*/
const Device UA16009 = {S58491, 61, { &P11850,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia2626/2.0 (06.80) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2947723 = {S58492, 81, { &P20377,&P5567,&P17371,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/5.1/27.2338; U; en) Presto/2.8.119 320X240 ZTE F-450*/
const Device UA2261537 = {S58493, 138, { &P18174,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS920 4G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2566881 = {S58494, 151, { &P18640,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT901 Build/7.7.1Q-144_SLS_S240-58) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583108 = {S58495, 144, { &P18640,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT901 Build/7.7.1Q-144_SLS_S240-58) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2676426 = {S58496, 138, { &P18450,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST23i Build/11.0.A.5.5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676997 = {S58497, 139, { &P18450,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST23a Build/11.0.A.2.16) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2878934 = {S58498, 143, { &P23683,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Prism II Build/HuaweiU8686) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA28454 = {S58499, 142, { &P12249,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; es-mx; U20a Build/1.2.A.1.181) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA28460 = {S58500, 142, { &P12249,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; pt-br; U20a Build/1.2.A.1.174) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA28465 = {S58501, 152, { &P12249,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-au; SonyEricssonU20a Build/1.1.A.0.8) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA28474 = {S58502, 140, { &P12249,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; es-mx; U20a Build/1.1.A.0.8) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA28478 = {S58503, 141, { &P12249,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-au; U20a Build/1.1.A.0.12) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA45943 = {S58504, 140, { &P12249,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; es-mx; U20a Build/1.1.A.0.9) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA45946 = {S58505, 142, { &P12249,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; U20a Build/1.2.A.1.183) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA1993192 = {S58506, 138, { &P21355,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; KM-E100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2551121 = {S58507, 60, { &P21355,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; KM-E100 Build/IMM76D)*/
const Device UA5795 = {S58508, 166, { &P12290,&P17483,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (LG-GD880/V10b Browser/AppleWebKit/531 Widget/LGMW/3.0 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA30261 = {S58509, 158, { &P12290,&P17483,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 LG-GD880 Browser/AppleWebKit/531 Widget/LGMW/3.0 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1Configuration/CLDC-1.1*/
const Device UA43550 = {S58510, 166, { &P12290,&P17483,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (LG/GD880/V10a Browser/AppleWebKit/531 Widget/LGMW/3.0 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA53605 = {S58511, 173, { &P12290,&P17483,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (LG-GD880-Orange/V10d Browser/AppleWebKit/531 Widget/LGMW/3.0 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA53606 = {S58512, 173, { &P12290,&P17483,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (LG-GD880-Orange/V10b Browser/AppleWebKit/531 Widget/LGMW/3.0 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA54175 = {S58513, 179, { &P12290,&P17483,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-GD880/V10e Browser/AppleWebKit/531 Widget/LGMW/3.0 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA56228 = {S58514, 166, { &P12290,&P17483,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (LG/GD880/V10e Browser/AppleWebKit/531 Widget/LGMW/3.0 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA56923 = {S58515, 179, { &P12290,&P17483,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-GD880/V10a Browser/AppleWebKit/531 Widget/LGMW/3.0 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA63050 = {S58516, 179, { &P12290,&P17483,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-GD880/V10b Browser/AppleWebKit/531 Widget/LGMW/3.0 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA64882 = {S58517, 161, { &P12290,&P17483,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (LG/GD880 Browser/AppleWebKit/531 Widget/LGMW/3.0 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA753909 = {S58518, 146, { &P17972,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; pandigitalsprnova1/sourceidDLX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2486395 = {S58519, 150, { &P15876,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P5100 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2603204 = {S58520, 149, { &P15876,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P5100 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2629266 = {S58521, 150, { &P15876,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P5100 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3015920 = {S58522, 157, { &P15876,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P5100 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1.2/p5100*/
const Device UA2974803 = {S58523, 136, { &P18621,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E610 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975700 = {S58524, 137, { &P18621,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975796 = {S58525, 136, { &P18621,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975813 = {S58526, 137, { &P18621,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E610v Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976013 = {S58527, 137, { &P18621,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612f Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2003757 = {S58528, 142, { &P19846,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-05D Build/7.0.D.1.117) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2010541 = {S58529, 146, { &P19846,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySO-05D Build/7.0.D.1.117) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2437397 = {S58530, 146, { &P19846,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySO-05D Build/7.0.D.1.130) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2437556 = {S58531, 142, { &P19846,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-05D Build/7.0.D.1.130) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2604043 = {S58532, 146, { &P19846,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySO-05D Build/7.0.D.1.137) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2670917 = {S58533, 142, { &P19846,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-05D Build/7.0.D.1.137) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2404198 = {S58534, 147, { &P20105,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; MyPhone A919 Duo Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2951275 = {S58535, 69, { &P20105,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MyPhone A919 Duo Build/IMM76D)*/
const Device UA2753687 = {S58536, 139, { &P18736,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2756012 = {S58537, 145, { &P13064,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879313 = {S58538, 145, { &P13064,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I727 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA51100 = {S58539, 77, { &P13485,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry8330/5.0.0.438 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/389*/
const Device UA1986394 = {S58540, 131, { &P20948,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LePanII Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2447883 = {S58541, 60, { &P20948,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LePanII Build/IMM76D)*/
const Device UA3083201 = {S58542, 148, { &P18472,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT925 Build/7.7.1Q-144_VQL_S3-58) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083552 = {S58543, 158, { &P18472,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR HD Build/7.7.1Q-144_VQW_S334-25) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA49215 = {S58544, 47, { &P12958,&P6476,&P3625,&P18092 }};	/*Useragent - LGE-VN251P/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA49216 = {S58545, 47, { &P12958,&P6476,&P3625,&P18092 }};	/*Useragent - LGE-VN251L/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA2191009 = {S58546, 132, { &P17281,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I905 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2228506 = {S58547, 130, { &P20358,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MW0711 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2676101 = {S58548, 137, { &P16349,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Sony Tablet S Build/TISU0143) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2705489 = {S58549, 144, { &P15767,&P18432,&P22593,&P18092 }};	/*Useragent - Mozilla/5.0(iPad; U;CPU OS 6 like Mac OS X; zh-CN; iPad3,4) AppleWebKit/534.46 (KHTML, like Gecko) UCBrowser/2.0.1.280 U3/0.8.0 Safari/7543.48.3*/
const Device UA2353552 = {S58550, 182, { &P12897,&P5843,&P24672,&P18092 }};	/*Useragent - NokiaC2-00/2.0 (03.82) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile*/
const Device UA2509896 = {S58551, 52, { &P15364,&P17017,&P22273,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; PPC; Trident/5.0)*/
const Device UA2659634 = {S58552, 57, { &P15364,&P17017,&P22273,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; PPC; SV1; Trident/2.0)*/
const Device UA2563812 = {S58553, 136, { &P22229,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NABIXD-NV10A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2598341 = {S58554, 112, { &P21003,&P20706,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia310/07.35; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.36*/
const Device UA16727 = {S58555, 77, { &P11886,&P5026,&P3410,&P18092 }};	/*Useragent - BlackBerry8700/4.5.0.182 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102*/
const Device UA2200011 = {S58556, 130, { &P20844,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAB462 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2599170 = {S58557, 24, { &P12280,&P17779,&P21178,&P18092 }};	/*Useragent - iPhone3,1/6.1.2 (10B146)*/
const Device UA2607095 = {S58558, 45, { &P12280,&P17779,&P21178,&P18092 }};	/*Useragent - server-bag [iPhone OS,6.1.3,10B329,iPhone4,1]*/
const Device UA2664231 = {S58559, 24, { &P12280,&P17779,&P21178,&P18092 }};	/*Useragent - iPhone4,1/6.1.2 (10B146)*/
const Device UA2699053 = {S58560, 24, { &P12280,&P17779,&P21178,&P18092 }};	/*Useragent - iPhone4,1/6.1.1 (10B145)*/
const Device UA2700565 = {S58561, 24, { &P12280,&P17779,&P21178,&P18092 }};	/*Useragent - iPhone5,2/6.1.2 (10B146)*/
const Device UA2700710 = {S58562, 22, { &P12280,&P17779,&P21178,&P18092 }};	/*Useragent - iPhone5,2/6.1 (10B143)*/
const Device UA2712487 = {S58563, 24, { &P12280,&P17779,&P21178,&P18092 }};	/*Useragent - iPhone4,1/6.1.3 (10B329)*/
const Device UA2729924 = {S58564, 22, { &P12280,&P17779,&P21178,&P18092 }};	/*Useragent - iPhone4,1/6.1 (10B142)*/
const Device UA2764272 = {S58565, 22, { &P12280,&P17779,&P21178,&P18092 }};	/*Useragent - iPhone3,1/6.1 (10B144)*/
const Device UA2793724 = {S58566, 24, { &P12280,&P17779,&P21178,&P18092 }};	/*Useragent - iPhone3,1/6.1.3 (10B329)*/
const Device UA2853594 = {S58567, 24, { &P12280,&P17779,&P21178,&P18092 }};	/*Useragent - iPhone2,1/6.1.2 (10B146)*/
const Device UA2860281 = {S58568, 24, { &P12280,&P17779,&P21178,&P18092 }};	/*Useragent - iPhone5,2/6.1.3 (10B329)*/
const Device UA2866368 = {S58569, 24, { &P12280,&P17779,&P21178,&P18092 }};	/*Useragent - iPhone5,2/6.1.4 (10B350)*/
const Device UA2954232 = {S58570, 24, { &P12280,&P17779,&P21178,&P18092 }};	/*Useragent - iPhone2,1/6.1.3 (10B329)*/
const Device UA2990806 = {S58571, 24, { &P12280,&P17779,&P21178,&P18092 }};	/*Useragent - iPhone3,3/6.1.3 (10B329)*/
const Device UA2996732 = {S58572, 24, { &P12280,&P17779,&P21178,&P18092 }};	/*Useragent - iPhone5,1/6.1.4 (10B350)*/
const Device UA2768663 = {S58573, 103, { &P12449,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae5-00) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2946182 = {S58574, 94, { &P12449,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE5-00) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3019676 = {S58575, 96, { &P12449,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE5-00.2) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA2898727 = {S58576, 156, { &P18442,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N7100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2901100 = {S58577, 156, { &P18442,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7100 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2943898 = {S58578, 156, { &P18442,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-N719 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2961965 = {S58579, 156, { &P18442,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7105 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2963319 = {S58580, 156, { &P18442,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SPH-L900 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2964598 = {S58581, 156, { &P18442,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7105 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3001656 = {S58582, 156, { &P18442,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; SCH-N719 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2234946 = {S58583, 143, { &P19746,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BLU VIVO 4.3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2480354 = {S58584, 139, { &P19746,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VIVO 4.3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3070687 = {S58585, 61, { &P19746,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; VIVO 4.3 Build/IMM76D)*/
const Device UA7858 = {S58586, 61, { &P11707,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia6070/2.0 (03.20) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87059 = {S58587, 39, { &P16329,&P18177,&P17652,&P18092 }};	/*Useragent - Mozilla/4.0 (PS3 (PlayStation 3); 1.00)*/
const Device UA87851 = {S58588, 33, { &P16329,&P18177,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PLAYSTATION 3; 1.00)*/
const Device UA2566811 = {S58589, 138, { &P20146,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567002 = {S58590, 139, { &P20146,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567400 = {S58591, 131, { &P20146,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2859567 = {S58592, 132, { &P20146,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2603165 = {S58593, 58, { &P15364,&P7127,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:20.0) Gecko/X Firefox/20.0*/
const Device UA2257499 = {S58594, 135, { &P21642,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NXM805ND_EN Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2511140 = {S58595, 51, { &P18088,&P19826,&P24676,&P18092 }};	/*Useragent - IUC(U;iOS 6.1;Zh-cn;320*568;)/UCWEB8.9.0.238/44/997*/
const Device UA2533254 = {S58596, 53, { &P18088,&P19826,&P24676,&P18092 }};	/*Useragent - IUC(U;iOS 6.1.2;Zh-cn;320*480;)/UCWEB8.9.0.238/44/997*/
const Device UA50201 = {S58597, 140, { &P16332,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; Galaxy Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA88910 = {S58598, 141, { &P16332,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; fr-fr; Galaxy Build/CUPCAKE) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2021588 = {S58599, 138, { &P16332,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; tr-; Galaxy Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2031153 = {S58600, 140, { &P16332,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; de-de; Galaxy Build/CUPCAKE) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA605108 = {S58601, 154, { &P12814,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I777 Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA666773 = {S58602, 154, { &P12814,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I777 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030990 = {S58603, 158, { &P12814,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I777 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2227776 = {S58604, 142, { &P15228,&P5043,&P24670,&P18092 }};	/*Useragent - NokiaC2-03/2.0 (07.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac2-03) UCBrowser8.4.0.159/70/444/UCWEB Mobile*/
const Device UA2351770 = {S58605, 89, { &P17595,&P17295,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:17.0) Gecko/17.0 Firefox/17.0 TenFourFox/G5*/
const Device UA2798572 = {S58606, 77, { &P18088,&P5567,&P17373,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/6.5/30.3214; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2462720 = {S58607, 136, { &P15767,&P17084,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA26479 = {S58608, 105, { &P13602,&P17484,&P3636,&P18092 }};	/*Useragent - SAMSUNG-GT-S5233W/1.0 SHP/VPP/R5 Jasmine/0.8 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA2754632 = {S58609, 143, { &P18225,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U9202L-1 Build/HuaweiU9202L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2756587 = {S58610, 139, { &P18225,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; U9200 Build/HuaweiU9200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA435358 = {S58611, 132, { &P15364,&P17838,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.33 (KHTML, like Gecko) Ubuntu/9.10 Chromium/13.0.752.0 Chrome/13.0.752.0 Safari/534.33*/
const Device UA2137080 = {S58612, 134, { &P15364,&P17838,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.36 (KHTML, like Gecko) Ubuntu/9.10 Chromium/13.0.768.0 Chrome/13.0.768.0 Safari/534.36*/
const Device UA2567194 = {S58613, 147, { &P17989,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V8000_USA_Cricket Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2795754 = {S58614, 147, { &P17989,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N8000_USA_Cricket Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA62182 = {S58615, 88, { &P11833,&P16000,&P3772,&P18092 }};	/*Useragent - SonyEricssonP1i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 768) Opera 8.65 [en]*/
const Device UA614896 = {S58616, 71, { &P15364,&P7133,&P18367,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.0) Gecko/X Thunderbird/12.0.1*/
const Device UA2673774 = {S58617, 138, { &P13064,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676257 = {S58618, 145, { &P13064,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3047486 = {S58619, 137, { &P13064,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I727 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2508539 = {S58620, 68, { &P17346,&P19826,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPod4,1; iPhone OS 6.1; en_US; en) AppleWebKit/420+*/
const Device UA2534989 = {S58621, 70, { &P17346,&P19826,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPod4,1; iPhone OS 6.1.2; en_US; en) AppleWebKit/420+*/
const Device UA2606890 = {S58622, 70, { &P17346,&P19826,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPod4,1; iPhone OS 6.1.3; en_US; en) AppleWebKit/420+*/
const Device UA2973663 = {S58623, 139, { &P15972,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One XL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020569 = {S58624, 140, { &P15972,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC EVA_UTL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023869 = {S58625, 139, { &P15972,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One XL Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA53283 = {S58626, 181, { &P13536,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia6110Navigator/4.24 (SymbianOS/9.2; U; Series60/3.1 Nokia6110Navigator/4.24; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) Mozilla/5.0 AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA63542 = {S58627, 157, { &P13536,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Nokia6110Navigator/4.22; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA87023 = {S58628, 148, { &P13536,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Nokia6110/3.01; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA2230868 = {S58629, 75, { &P15364,&P17285,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:15.0; Avant TriCore) Gecko/X Firefox/15.0.1*/
const Device UA2258899 = {S58630, 73, { &P15364,&P17285,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:16.0; Avant TriCore) Gecko/X Firefox/16.0*/
const Device UA2429519 = {S58631, 73, { &P15364,&P17285,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:17.0; Avant TriCore) Gecko/X Firefox/17.0*/
const Device UA2601538 = {S58632, 73, { &P15364,&P17285,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:19.0; Avant TriCore) Gecko/X Firefox/19.0*/
const Device UA2738241 = {S58633, 73, { &P15364,&P17285,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:20.0; Avant TriCore) Gecko/X Firefox/20.0*/
const Device UA2854992 = {S58634, 73, { &P15364,&P17285,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:18.0; Avant TriCore) Gecko/X Firefox/18.0*/
const Device UA2899030 = {S58635, 73, { &P15364,&P17285,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:21.0; Avant TriCore) Gecko/X Firefox/21.0*/
const Device UA2967920 = {S58636, 73, { &P15364,&P17285,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:22.0; Avant TriCore) Gecko/X Firefox/22.0*/
const Device UA3062563 = {S58637, 73, { &P15364,&P17285,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:23.0; Avant TriCore) Gecko/X Firefox/23.0*/
const Device UA184771 = {S58638, 97, { &P15364,&P22190,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.13) Gecko Fedora/1.9.2.13-12.fc14 Firefox/3.6.13*/
const Device UA2405316 = {S58639, 78, { &P18254,&P17017,&P18158,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; FlipboardRSS/1.1; +http://flipboard.com/browserproxy)*/
const Device UA2974596 = {S58640, 145, { &P19891,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3065138 = {S58641, 136, { &P24039,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9190 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA771666 = {S58642, 63, { &P15364,&P16110,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; U; en) Presto/2.8.131 Version/11.10*/
const Device UA1881334 = {S58643, 63, { &P15364,&P16110,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; U; en) Presto/2.8.131 Version/11.11*/
const Device UA2369499 = {S58644, 34, { &P15364,&P16110,&P17365,&P18092 }};	/*Useragent - Opera/11.1 (Windows NT 6.2: U; en)*/
const Device UA2204912 = {S58645, 138, { &P18643,&P5043,&P24668,&P18092 }};	/*Useragent - Nokia311/5.0 (03.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia311) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2567013 = {S58646, 134, { &P18665,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N861 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2425237 = {S58647, 138, { &P19917,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-P765 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2844452 = {S58648, 60, { &P19917,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P765 Build/IMM76L)*/
const Device UA174016 = {S58649, 132, { &P15364,&P7133,&P22447,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Iron/9.0.600.2 Chrome/9.0.600.2 Safari/534.13*/
const Device UA843695 = {S58650, 57, { &P18088,&P17017,&P17372,&P18092 }};	/*Useragent - Opera/9.50 (J2ME/MIDP; Opera Mini/5.21463/27.1659; U; en)*/
const Device UA890462 = {S58651, 86, { &P18088,&P17017,&P17372,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/5.22371/26.1670; U; es) Presto/2.8.119 Version/10.54*/
const Device UA1995862 = {S58652, 132, { &P14908,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6210 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2023750 = {S58653, 132, { &P14908,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P6210 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2609598 = {S58654, 139, { &P14908,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3 LeoROM; GT-P6210 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA195340 = {S58655, 140, { &P14225,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Desire HD Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA210694 = {S58656, 140, { &P14225,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Desire HD Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA251939 = {S58657, 140, { &P14225,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Desire HD Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA396394 = {S58658, 136, { &P14225,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Desire HD Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA537945 = {S58659, 140, { &P14225,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Desire HD Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA859857 = {S58660, 128, { &P14225,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Desire HD Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1906127 = {S58661, 64, { &P14225,&P24694,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Desire HD Build/MIUI)*/
const Device UA2020387 = {S58662, 136, { &P14225,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Desire HD Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034299 = {S58663, 133, { &P14225,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Desire HD Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2717896 = {S58664, 67, { &P15364,&P7133,&P20794,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:7.0.1) Gecko/X Postbox/3.0.8*/
const Device UA2607048 = {S58665, 34, { &P15767,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.1.3 (2; 16GB; dt:74)*/
const Device UA2668545 = {S58666, 34, { &P15767,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.1.3 (4; 32GB; dt:77)*/
const Device UA2669003 = {S58667, 34, { &P15767,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.1.3 (5; 16GB; dt:86)*/
const Device UA2670857 = {S58668, 34, { &P15767,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.1.3 (4; 16GB; dt:77)*/
const Device UA2670961 = {S58669, 34, { &P15767,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.1.3 (5; 32GB; dt:86)*/
const Device UA2672150 = {S58670, 34, { &P15767,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.1.3 (2; 32GB; dt:74)*/
const Device UA2702641 = {S58671, 34, { &P15767,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.1.3 (5; 64GB; dt:83)*/
const Device UA2704329 = {S58672, 34, { &P15767,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.1.3 (4; 64GB; dt:77)*/
const Device UA2711383 = {S58673, 34, { &P15767,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.1.3 (5; 16GB; dt:83)*/
const Device UA2733565 = {S58674, 34, { &P15767,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.1.3 (2; 64GB; dt:74)*/
const Device UA2750603 = {S58675, 34, { &P15767,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.1.3 (5; 32GB; dt:83)*/
const Device UA2795827 = {S58676, 34, { &P15767,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.1.3 (5; 64GB; dt:86)*/
const Device UA2531760 = {S58677, 135, { &P17595,&P18207,&P22464,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17*/
const Device UA2585273 = {S58678, 135, { &P17595,&P18207,&P22464,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17*/
const Device UA2765551 = {S58679, 135, { &P17595,&P18207,&P22464,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17*/
const Device UA2780915 = {S58680, 135, { &P17595,&P18207,&P22464,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17*/
const Device UA3114968 = {S58681, 135, { &P17595,&P18207,&P22464,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17*/
const Device UA3066215 = {S58682, 140, { &P23910,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30at Build/9.1.F.1.120) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA30846 = {S58683, 135, { &P13644,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; PC36100 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA31521 = {S58684, 144, { &P13644,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; Sprint APA9292KT Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA31522 = {S58685, 143, { &P13644,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; PC36100 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA62411 = {S58686, 152, { &P13644,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Sprint APA9292KT Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA285325 = {S58687, 119, { &P13644,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Sprint APA9292KT Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko)*/
const Device UA561899 = {S58688, 140, { &P13644,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; xx-xx; PC36100 Build/ERE27) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2018565 = {S58689, 135, { &P13644,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; PC36100 Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA666662 = {S58690, 63, { &P15364,&P16110,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:6.0) Gecko/X Firefox/6.0*/
const Device UA2596252 = {S58691, 135, { &P18737,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; X909 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2660155 = {S58692, 57, { &P18737,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; X909 Build/JRO03L)*/
const Device UA2581564 = {S58693, 140, { &P23969,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-nz; GT-I8190T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2765766 = {S58694, 62, { &P23969,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8190T Build/JZO54K)*/
const Device UA2671581 = {S58695, 130, { &P21769,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ME301T Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2752928 = {S58696, 145, { &P12814,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I777 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754647 = {S58697, 145, { &P12814,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I777 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783084 = {S58698, 137, { &P12814,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I777 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799212 = {S58699, 137, { &P12814,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-I777 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA903658 = {S58700, 115, { &P15837,&P6909,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-05/08.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA2738957 = {S58701, 138, { &P12280,&P18117,&P22902,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.4.2 Mobile/9B206 Safari/7534.48.3*/
const Device UA2796380 = {S58702, 136, { &P12280,&P18117,&P22902,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.4.2 Mobile/9B176 Safari/7534.48.3*/
const Device UA2895187 = {S58703, 156, { &P15228,&P5043,&P24678,&P18092 }};	/*Useragent - NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-03) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2671843 = {S58704, 137, { &P18673,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675388 = {S58705, 138, { &P18673,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562L Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3088274 = {S58706, 130, { &P18673,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2367698 = {S58707, 75, { &P17595,&P17295,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8) Presto/2.12.388 Version/12.11*/
const Device UA2408206 = {S58708, 131, { &P15767,&P18432,&P20779,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/2.8 Mobile/10A523 Safari/7534.48.3*/
const Device UA2426552 = {S58709, 129, { &P15767,&P18432,&P20779,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/2.8 Mobile/10A403 Safari/7534.48.3*/
const Device UA2458382 = {S58710, 129, { &P15767,&P18432,&P20779,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/2.8 Mobile/10A407 Safari/7534.48.3*/
const Device UA2467413 = {S58711, 132, { &P15767,&P18432,&P20779,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/2.8 Mobile/10A8426 Safari/7534.48.3*/
const Device UA2531112 = {S58712, 133, { &P15767,&P18432,&P20779,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/2.8.1 Mobile/10A550 Safari/7534.48.3*/
const Device UA2661860 = {S58713, 134, { &P15767,&P18432,&P20779,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/2.8.1 Mobile/10A8500 Safari/7534.48.3*/
const Device UA2672089 = {S58714, 129, { &P19911,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E970 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2677176 = {S58715, 136, { &P19911,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E970 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA750985 = {S58716, 142, { &P19938,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; SCH-I589 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2576839 = {S58717, 140, { &P19938,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-; SCH-I589 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2722444 = {S58718, 142, { &P19938,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i589 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995190 = {S58719, 129, { &P21222,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AT1S0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2689745 = {S58720, 58, { &P21222,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; AT1S0 Build/IMM76D)*/
const Device UA2621217 = {S58721, 132, { &P15882,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P5110 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622851 = {S58722, 61, { &P15882,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-P5110 Build/JOP40D)*/
const Device UA2631649 = {S58723, 60, { &P15882,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P5110 Build/JDQ39)*/
const Device UA2636020 = {S58724, 93, { &P15882,&P18598,&P19802,&P18092 }};	/*Useragent - com.google.android.youtube/4.2.16(Linux; U; Android 4.2.1; en_GB; GT-P5110 Build/JOP40G) gzip*/
const Device UA3096917 = {S58725, 131, { &P15882,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; GT-P5110 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3028633 = {S58726, 156, { &P14540,&P5843,&P24679,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-01) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3108223 = {S58727, 160, { &P14540,&P5843,&P24679,&P18092 }};	/*Useragent - NokiaX2-01.1/5.0 (07.22) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-01.1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3112833 = {S58728, 122, { &P14540,&P5843,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.1; en-US; NokiaX2-01) U2/1.0.0 UCBrowser/9.2.1.331 U2/1.0.0 Mobile Opera Mini/7.1.32052/30.3697*/
const Device UA64885 = {S58729, 61, { &P11792,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia6020/2.0 (04.90) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2272247 = {S58730, 71, { &P11792,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia6020/2.0 (04.90) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6020*/
const Device UA635573 = {S58731, 99, { &P15364,&P22197,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; zh-TW; rv:1.8.0.10) Gecko/X Fedora/1.5.0.10-1.fc5 Firefox/1.5.0.10*/
const Device UA2975956 = {S58732, 133, { &P18466,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023425 = {S58733, 126, { &P18466,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2708745 = {S58734, 138, { &P22610,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2750137 = {S58735, 139, { &P22610,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2964929 = {S58736, 139, { &P22610,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810M Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3007885 = {S58737, 139, { &P22610,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810B Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3092481 = {S58738, 139, { &P22610,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810E Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA354761 = {S58739, 142, { &P21180,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; NXM908HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA669638 = {S58740, 128, { &P20376,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T10A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2175408 = {S58741, 128, { &P20376,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T10A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2974391 = {S58742, 140, { &P17346,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.12 Mobile/11A4414e Safari/8536.25*/
const Device UA3030229 = {S58743, 140, { &P17346,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A4435d Safari/8536.25*/
const Device UA3033399 = {S58744, 140, { &P17346,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A4449a Safari/8536.25*/
const Device UA3036125 = {S58745, 140, { &P17346,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A4449d Safari/8536.25*/
const Device UA3094949 = {S58746, 138, { &P17346,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A465 Safari/8536.25*/
const Device UA3128252 = {S58747, 140, { &P17346,&P22882,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/11A501 Safari/8536.25*/
const Device UA3065614 = {S58748, 134, { &P21636,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C525c Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA571709 = {S58749, 136, { &P15820,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; LG-SU640 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020702 = {S58750, 60, { &P15820,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; LG-SU640 Build/GRJ90)*/
const Device UA2534728 = {S58751, 114, { &P19974,&P5043,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia111.1/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2535580 = {S58752, 112, { &P19974,&P5043,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia111/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA10158 = {S58753, 49, { &P13472,&P17017,&P3402,&P18092 }};	/*Useragent - Cricket-A310/1.0 UP.Browser/6.3.0.7 (GUI) MMP/2.0*/
const Device UA2673880 = {S58754, 43, { &P15364,&P18570,&P22902,&P18092 }};	/*Useragent - Mercury/7.4 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA20990 = {S58755, 73, { &P11967,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2220s/2.0 (09.55) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2220s*/
const Device UA20991 = {S58756, 62, { &P11967,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2220s/2.0 (09.55) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA53773 = {S58757, 64, { &P11967,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2220s-b/2.0 (10.20) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA347412 = {S58758, 185, { &P15364,&P7133,&P24197,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1;	Trident/4.0; SLCC2;	.NET CLR 2.0.50727; .NET CLR 3.5.30729;	.NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E) QQBrowser/6.0*/
const Device UA627626 = {S58759, 114, { &P15364,&P7133,&P24197,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.0*/
const Device UA846491 = {S58760, 150, { &P15364,&P7133,&P24197,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.16; QQBrowser/6.0.8082.201 (webkit)*/
const Device UA898435 = {S58761, 64, { &P15364,&P7133,&P24197,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1) QQBrowser/6.0*/
const Device UA2297134 = {S58762, 123, { &P15364,&P7133,&P24197,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.0.8405.201*/
const Device UA2206200 = {S58763, 136, { &P18675,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01E Build/S9130) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2389492 = {S58764, 136, { &P18675,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01E Build/SA310) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2404485 = {S58765, 136, { &P18675,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01E Build/S9132) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2459117 = {S58766, 138, { &P18675,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01EVW Build/SB020) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2561095 = {S58767, 138, { &P18675,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01EVW Build/S1100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2672866 = {S58768, 136, { &P18675,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01E Build/S3110) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2673138 = {S58769, 138, { &P18675,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01EVW Build/S3120) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2796629 = {S58770, 136, { &P18675,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01E Build/S5270) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2872336 = {S58771, 138, { &P18675,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01EVW Build/S5270) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA648910 = {S58772, 143, { &P17976,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0; en-us; MID_Serials Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA695656 = {S58773, 145, { &P17976,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0UI; en-us; MID_Serials Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022382 = {S58774, 67, { &P17976,&P6545,&P4755,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 3.0; MID_Serials Build/GINGERBREAD)*/
const Device UA616034 = {S58775, 64, { &P15364,&P7127,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.10.229 Version/11.64*/
const Device UA645674 = {S58776, 101, { &P15364,&P17350,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 7.1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30*/
const Device UA2334997 = {S58777, 108, { &P15364,&P17350,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 7.1; en-us) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30*/
const Device UA2374295 = {S58778, 108, { &P15364,&P17350,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 7.1; es-mx) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30*/
const Device UA2027188 = {S58779, 136, { &P21109,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; BLADEII Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2404414 = {S58780, 143, { &P20143,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison 3G Build/1.1.7 X-17:05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2458800 = {S58781, 148, { &P20143,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison 3G Build/1.1.8-1015 X-14:40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2461123 = {S58782, 148, { &P20143,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison 3G Build/1.1.9-1015 X-17:37) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2461142 = {S58783, 143, { &P20143,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison 3G Build/1.1.9 X-10:23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2469382 = {S58784, 149, { &P20143,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison 3G Build/1.1.10-1015 X-18:00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2508030 = {S58785, 149, { &P20143,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison 3G Build/1.1.11-1015 X-15:24) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2537043 = {S58786, 144, { &P20143,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison 3G Build/1.1.11 X-10:07) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555242 = {S58787, 143, { &P20143,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Edison 3G Build/1.1.8 X-17:02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA56909 = {S58788, 122, { &P13914,&P5843,&P4030,&P18092 }};	/*Useragent - Nokia5330/5.0 (09.85) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA64233 = {S58789, 125, { &P13914,&P5843,&P4030,&P18092 }};	/*Useragent - Nokia5330-1d/5.0 (06.88) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA2756642 = {S58790, 139, { &P18496,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LVW Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867895 = {S58791, 139, { &P18496,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LRA Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2958558 = {S58792, 139, { &P18496,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR6410LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2386778 = {S58793, 112, { &P18643,&P5043,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia311/05.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA2556460 = {S58794, 140, { &P21495,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E220S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952691 = {S58795, 62, { &P21495,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E220S Build/JZO54K)*/
const Device UA2665599 = {S58796, 31, { &P11929,&P5129,&P24676,&P18092 }};	/*Useragent - Nokia5230/UCWEB8.9.0.253/50/999*/
const Device UA2716548 = {S58797, 93, { &P11929,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5230) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2866898 = {S58798, 95, { &P11929,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5230-c) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2995801 = {S58799, 96, { &P11929,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5230-1c) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2892116 = {S58800, 138, { &P23231,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E988 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998971 = {S58801, 60, { &P23231,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E988 Build/JZO54K)*/
const Device UA2892152 = {S58802, 129, { &P22228,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SM-T210 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2945362 = {S58803, 130, { &P22228,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SM-T210R Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA723281 = {S58804, 140, { &P16036,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus One Build/GRK39F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA753747 = {S58805, 140, { &P16036,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus One Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA796477 = {S58806, 137, { &P16036,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en_us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1975622 = {S58807, 140, { &P16036,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus One Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025537 = {S58808, 140, { &P16036,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus One Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2366733 = {S58809, 138, { &P16036,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus One Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2446473 = {S58810, 138, { &P16036,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus One Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2659348 = {S58811, 136, { &P16036,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2985118 = {S58812, 137, { &P16036,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus One Build/FRF91) AppleWebKit/534.3 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.3*/
const Device UA2566528 = {S58813, 139, { &P23905,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100P Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566810 = {S58814, 139, { &P23905,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100P Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2204851 = {S58815, 138, { &P18219,&P5843,&P24668,&P18092 }};	/*Useragent - Nokia202/2.0 (20.28) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia202) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2732680 = {S58816, 137, { &P22916,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E450 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2752240 = {S58817, 138, { &P22916,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E450f Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2879750 = {S58818, 138, { &P22916,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E450g Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2991509 = {S58819, 138, { &P22916,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E450B Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2897841 = {S58820, 156, { &P14540,&P5843,&P24678,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.71) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-01) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3055550 = {S58821, 160, { &P14540,&P5843,&P24678,&P18092 }};	/*Useragent - NokiaX2-01.1/5.0 (07.22) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-01.1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA56046 = {S58822, 112, { &P13662,&P17483,&P4732,&P18092 }};	/*Useragent - LG-GW370/V10c; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.17086; U; en) Opera 9.50*/
const Device UA2572340 = {S58823, 85, { &P15364,&P7127,&P19772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.8.1.17) Gecko/X SeaMonkey/1.1.12*/
const Device UA2682735 = {S58824, 100, { &P15364,&P7127,&P19772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; de-AT; rv:1.8.1.21) Gecko/X SeaMonkey/1.1.15 Mnenhy/0.7.6.0*/
const Device UA2652615 = {S58825, 142, { &P21587,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Q-Smart S18 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2677040 = {S58826, 148, { &P21587,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U; Android 4.0;en-US;Q-Smart S18 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2425042 = {S58827, 140, { &P19844,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80 COBALT Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2567402 = {S58828, 142, { &P18382,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire C Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567694 = {S58829, 142, { &P18382,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire C Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA10986 = {S58830, 138, { &P15539,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us;HTC Glacier Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10989 = {S58831, 138, { &P15539,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Glacier Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 MobileSafari/533.1*/
const Device UA11019 = {S58832, 138, { &P15539,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android2.2.1; en-us; HTC Glacier Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA15042 = {S58833, 139, { &P15539,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Glacier Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA15044 = {S58834, 137, { &P15539,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; HTC Glacier Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA69622 = {S58835, 140, { &P15539,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Glacier Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90292 = {S58836, 146, { &P15539,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Glacier Build/FRG83) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA159556 = {S58837, 63, { &P15539,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; HTC Glacier Build/FRG83)*/
const Device UA555183 = {S58838, 61, { &P15539,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; HTC Glacier Build/FRF91)*/
const Device UA2509400 = {S58839, 52, { &P15364,&P17017,&P4808,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; PPC; Trident/1.0)*/
const Device UA2658786 = {S58840, 57, { &P15364,&P17017,&P4808,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; PPC; SV1; Trident/4.0)*/
const Device UA2568340 = {S58841, 127, { &P15767,&P18117,&P22901,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.3 Mobile/9B206 Safari/7534.48.3*/
const Device UA2601524 = {S58842, 125, { &P15767,&P18117,&P22901,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.3 Mobile/9B176 Safari/7534.48.3*/
const Device UA2405878 = {S58843, 155, { &P14711,&P5204,&P24673,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac2-01) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2483331 = {S58844, 125, { &P23968,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/11.05.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2484471 = {S58845, 125, { &P23968,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/11.07.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2504668 = {S58846, 125, { &P23968,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/10.20.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2530086 = {S58847, 125, { &P23968,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/12.06.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2535653 = {S58848, 125, { &P23968,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/12.05.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2577940 = {S58849, 125, { &P23968,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/09.26.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2699941 = {S58850, 125, { &P23968,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/ab.`e.b`ab Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2707547 = {S58851, 125, { &P23968,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/02.26.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2767519 = {S58852, 125, { &P23968,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/03.13.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2849567 = {S58853, 125, { &P23968,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/12.11.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2864672 = {S58854, 125, { &P23968,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/03.08.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2935351 = {S58855, 125, { &P23968,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/`b.bf.b`ac Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2951712 = {S58856, 125, { &P23968,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/12.10.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2984967 = {S58857, 125, { &P23968,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-S930/1.0 Linux/2.6.35.7 Android/4.0.3 Release/ab.`f.b`ab Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2986194 = {S58858, 126, { &P23968,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-S9300/1.0 Linux/2.6.35.7 Android/4.0.3 Release/12.06.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA316300 = {S58859, 131, { &P15364,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; VegaComb Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA338065 = {S58860, 136, { &P15364,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Flashback 8.0 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA344987 = {S58861, 131, { &P15364,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; GtabComb Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA413007 = {S58862, 137, { &P15364,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Flashback 10.0 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA574701 = {S58863, 136, { &P15364,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Flashback 9.0 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA640686 = {S58864, 130, { &P15364,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; gTablet Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA769449 = {S58865, 141, { &P15364,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Android Illuminate Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1893700 = {S58866, 136, { &P15364,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Flashback 7.1 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1908525 = {S58867, 136, { &P15364,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-gb; Flashback 8.1 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1964283 = {S58868, 146, { &P15364,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0UI; pt-br; Tabra QAV801 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018948 = {S58869, 132, { &P15364,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; GtabComb Build/NIHCPX) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2483380 = {S58870, 143, { &P15364,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0UI; en-us; REL-7305B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2573574 = {S58871, 137, { &P15364,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0UI; tr-tr; F4S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2849789 = {S58872, 142, { &P15364,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0UI; en-us; mTAB NEO2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2981466 = {S58873, 160, { &P15364,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0; en-us; SPH-M828C Build/HONEYCOMB; HONEYCOMB PRECEDENT) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2586652 = {S58874, 138, { &P21628,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3013248 = {S58875, 139, { &P21628,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552B Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2002586 = {S58876, 123, { &P15364,&P17285,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1*/
const Device UA2014789 = {S58877, 116, { &P15364,&P17285,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1*/
const Device UA2671191 = {S58878, 139, { &P15364,&P17820,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu/12.04 Chromium/25.0.1364.172 Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2672588 = {S58879, 141, { &P15364,&P17820,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu/12.04 Chromium/25.0.1364.172 Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2457666 = {S58880, 134, { &P20498,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMP7100D3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3065255 = {S58881, 137, { &P18636,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; VS930 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2354420 = {S58882, 152, { &P20017,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2413782 = {S58883, 151, { &P20017,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA33465 = {S58884, 105, { &P15921,&P17485,&P4223,&P18092 }};	/*Useragent - SonyEricssonJ108a/R7EA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2*/
const Device UA674012 = {S58885, 81, { &P12193,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaC5-00)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA754141 = {S58886, 83, { &P12193,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaC5-00.2)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA2543021 = {S58887, 91, { &P12193,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokiac5-00.2) UCBrowser8.2.1.144/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA2040947 = {S58888, 134, { &P17595,&P18207,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1*/
const Device UA2201585 = {S58889, 134, { &P17595,&P18207,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1*/
const Device UA2454775 = {S58890, 134, { &P17595,&P18207,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1*/
const Device UA2601731 = {S58891, 134, { &P17595,&P18207,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1*/
const Device UA2797053 = {S58892, 134, { &P17595,&P18207,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1*/
const Device UA3115547 = {S58893, 134, { &P17595,&P18207,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1*/
const Device UA3031209 = {S58894, 156, { &P15228,&P5043,&P24679,&P18092 }};	/*Useragent - NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-03) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA53579 = {S58895, 152, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0.1; en-us; Droid (AdamZ Enhanced) Build/ESD56) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA67451 = {S58896, 137, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.0; es-es; STM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA68569 = {S58897, 137, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.0; sv-se; 7CK100 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA68668 = {S58898, 138, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.0; nl-nl; TP_10D8 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA69070 = {S58899, 113, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0; en-us;) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA178476 = {S58900, 159, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v1.02.11-2010.12.17_English; en-us; TR718D Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA300110 = {S58901, 159, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v1.02.09-2010.11.22_English; en-us; TR718D Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA330111 = {S58902, 159, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v1.02.15_14-M_EN-2011.03.29; en-us; M8002R Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA389059 = {S58903, 156, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.0; ld-us; Android-for-Rockchip-2818 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA389783 = {S58904, 157, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v1.02_14.13-EN-2011.01.11; en-en; TR718D Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA440003 = {S58905, 159, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v1.02.15_14-M_EN-2011.03.18; en-us; M8002R Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA449093 = {S58906, 159, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v1.02_14.13-M_EN-2011.03.18; en-en; TR718D Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA449912 = {S58907, 156, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v1.02_15.13-ES-2011.04.19; es-es; N6701 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA458728 = {S58908, 158, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v1.02_14.13_CH-2011.03.31; es-es; MID1002 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA508827 = {S58909, 137, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.0; es-es; STM7UW Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA539853 = {S58910, 142, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; fr-fr; Tablet-P27 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA655581 = {S58911, 139, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.0; es-es; ARCHM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA691061 = {S58912, 149, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v1.02_15.14-M_GB@X; en-gb; A93KQ Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA695004 = {S58913, 135, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.1; en-us; A7EC Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA715465 = {S58914, 136, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.0; fr-fr; CT701 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA765930 = {S58915, 137, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.0; es-es; STM7UC Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA767703 = {S58916, 159, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v1.02_14.13-M_EN-2011.01.13; es-es; TR718D Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1897882 = {S58917, 159, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v1.02_14.13-M_EN-2011.01.28; en-en; TR718D Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1941695 = {S58918, 140, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.4; fr-fr; A7EB-FLFR Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018487 = {S58919, 52, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android Eclair Build/ECLAIR)*/
const Device UA2018664 = {S58920, 147, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Android_2.1-1.1.0; es-es; T300 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018829 = {S58921, 157, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v1.02_15.14-EN-2011.03.23; en-za; TR718D Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018935 = {S58922, 139, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; tr-tr; FENOMEN Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019005 = {S58923, 163, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v1.02_15.13-ES-2011.03.29; es-es; AIRIS-OnePAD Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019171 = {S58924, 155, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v1.02_14.13-ES-2011.01.15; es-es; M721 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2020037 = {S58925, 163, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v1.02_15.14-FR--2011.02.21; fr-fr; MFC150FR_01 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2020113 = {S58926, 155, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v1.02_14.13-ES-2011.02.21; es-es; M721 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2022565 = {S58927, 138, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; es-es; Taicho Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2023129 = {S58928, 138, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.5; cs-cz; A7EB-WK Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2026679 = {S58929, 136, { &P15364,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.0; da-dk; NTM7U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2402904 = {S58930, 137, { &P23190,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Marquis_MP977 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2566955 = {S58931, 134, { &P19907,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Rise Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567379 = {S58932, 134, { &P19907,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C5155 Build/IML77) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2943781 = {S58933, 62, { &P17976,&P17979,&P23660,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:25.0) Gecko/25.0 Firefox/25.0*/
const Device UA2702691 = {S58934, 140, { &P20708,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XOLO A1000 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2963318 = {S58935, 138, { &P20708,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IdeaTabA1000-F Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3021736 = {S58936, 138, { &P20708,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IdeaTabA1000-G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2974087 = {S58937, 141, { &P18382,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire C Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976119 = {S58938, 141, { &P18382,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA840823 = {S58939, 133, { &P14612,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; A777 Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025190 = {S58940, 134, { &P14612,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; WT7.0 Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033405 = {S58941, 135, { &P14612,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A7 ART Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2348183 = {S58942, 131, { &P14612,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; GUNIUS713 Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2519437 = {S58943, 145, { &P14612,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; GOCLEVER TAB T75 Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2525088 = {S58944, 134, { &P14612,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; TAKARA MID72 Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2526920 = {S58945, 139, { &P14612,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A777-C-MID-general Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2652179 = {S58946, 126, { &P14612,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A777 Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2652700 = {S58947, 143, { &P14612,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Prology T-710T Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2692482 = {S58948, 144, { &P14612,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Bliss Pad T7012 Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2763846 = {S58949, 150, { &P14612,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Prology iMap-7000 Tab Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2837611 = {S58950, 137, { &P14612,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Note-700 Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2981614 = {S58951, 141, { &P14612,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; TAKARA MID72 Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA34592 = {S58952, 135, { &P11729,&P5810,&P3417,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320) BlackBerry8100/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100*/
const Device UA63749 = {S58953, 73, { &P11729,&P5810,&P3417,&P18092 }};	/*Useragent - BlackBerry8100/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100*/
const Device UA60980 = {S58954, 171, { &P12607,&P5096,&P3508,&P18092 }};	/*Useragent - LG-C550 Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; LG-C550 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA60982 = {S58955, 164, { &P12607,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; LG-C550 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA77405 = {S58956, 160, { &P12607,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en; LG-C550 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA90558 = {S58957, 130, { &P12607,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; L-04C Build/FRF91) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA651621 = {S58958, 134, { &P12607,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; L-04C Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023372 = {S58959, 58, { &P12607,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; L-04C Build/FRG83G)*/
const Device UA2216460 = {S58960, 163, { &P15364,&P16110,&P22765,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Maxthon/3.0)*/
const Device UA2337837 = {S58961, 84, { &P15364,&P16110,&P22765,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Maxthon/3.0)*/
const Device UA2376003 = {S58962, 156, { &P15364,&P16110,&P22765,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Maxthon/3.0)*/
const Device UA2412046 = {S58963, 184, { &P15364,&P16110,&P22765,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Maxthon/3.0)*/
const Device UA2415909 = {S58964, 77, { &P15364,&P16110,&P22765,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; Maxthon/3.0)*/
const Device UA2442616 = {S58965, 177, { &P15364,&P16110,&P22765,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; Trident/6.0; .NET4.0E; .NET4.0C; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Maxthon/3.0)*/
const Device UA2777705 = {S58966, 178, { &P15364,&P16110,&P22765,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; Maxthon/3.0)*/
const Device UA2566617 = {S58967, 138, { &P19856,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LS970 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583570 = {S58968, 131, { &P19856,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LS970 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2662701 = {S58969, 139, { &P22651,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; LG-E435f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2714214 = {S58970, 138, { &P22651,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E435 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2935206 = {S58971, 60, { &P22651,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E435 Build/JZO54K)*/
const Device UA2936570 = {S58972, 139, { &P22651,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E435g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2993317 = {S58973, 61, { &P22651,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E435f Build/JZO54K)*/
const Device UA56027 = {S58974, 128, { &P14619,&P17484,&P4732,&P18092 }};	/*Useragent - SAMSUNG-SGH-A597/A597UCJG1; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.19903; U; en-US) Opera 9.50*/
const Device UA2423787 = {S58975, 103, { &P14533,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.424; en-us; 9630/4.2.0.424) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2428209 = {S58976, 103, { &P14533,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.451; en-us; 9630/4.2.0.451) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2504367 = {S58977, 103, { &P14533,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.456; en-us; 9630/4.2.0.456) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2553993 = {S58978, 103, { &P14533,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.284; en-us; 9630/4.2.0.284) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2554465 = {S58979, 103, { &P14533,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.433; en-us; 9630/4.2.0.433) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2554614 = {S58980, 103, { &P14533,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.334; en-us; 9630/4.2.0.334) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2594768 = {S58981, 103, { &P14533,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.179; en-us; 9630/4.2.0.179) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2594792 = {S58982, 103, { &P14533,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.285; en-us; 9630/4.2.0.285) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2596911 = {S58983, 103, { &P14533,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.494; en-us; 9630/4.2.0.494) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2654806 = {S58984, 103, { &P14533,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.214; en-us; 9630/4.2.0.214) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2842237 = {S58985, 103, { &P14533,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.477; en-us; 9630/4.2.0.477) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2951718 = {S58986, 101, { &P14533,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.1.0.40; en-us; 9630/4.1.0.40) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3065103 = {S58987, 138, { &P20947,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTCEVOV4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2511 = {S58988, 89, { &P15364,&P17779,&P17470,&P18092 }};	/*Useragent - Tiphone T78/1.0 Browser/wap2.0 Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA35487 = {S58989, 89, { &P15364,&P17779,&P17470,&P18092 }};	/*Useragent - Tiphone MY1/1.0 Browser/wap2.0 Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63842 = {S58990, 89, { &P15364,&P17779,&P17470,&P18092 }};	/*Useragent - Tiphone T33/1.0 Browser/wap2.0 Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA89775 = {S58991, 57, { &P15364,&P17779,&P17470,&P18092 }};	/*Useragent - tiphoneE88/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA1945117 = {S58992, 78, { &P15364,&P17779,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (iOS; U; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.1*/
const Device UA2374786 = {S58993, 78, { &P15364,&P17779,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (iOS; U; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.2*/
const Device UA2526946 = {S58994, 78, { &P15364,&P17779,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (iOS; U; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.5*/
const Device UA2552258 = {S58995, 78, { &P15364,&P17779,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (iOS; U; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.4*/
const Device UA2656927 = {S58996, 78, { &P15364,&P17779,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (iOS; U; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.6*/
const Device UA2738516 = {S58997, 42, { &P15364,&P17779,&P17470,&P18092 }};	/*Useragent - /media/sandisk/af_mobile/user_agent/iphone*/
const Device UA2810980 = {S58998, 78, { &P15364,&P17779,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (iOS; U; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.3*/
const Device UA2936807 = {S58999, 78, { &P15364,&P17779,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (iOS; U; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.7*/
const Device UA3103348 = {S59000, 78, { &P15364,&P17779,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (iOS; U; en) AppleWebKit/533.19.4 (KHTML, like Gecko) AdobeAIR/3.8*/
const Device UA216043 = {S59001, 142, { &P12945,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; IM-A780L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA566528 = {S59002, 66, { &P12945,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; IM-A780L Build/GINGERBREAD)*/
const Device UA2027419 = {S59003, 139, { &P12945,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A780 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028207 = {S59004, 83, { &P12945,&P5629,&P15760,&P18092 }};	/*Useragent - Lenovo A780/1.0 Linux/2.6.35.7 Android/2.3.5 Browser/AppleWebKit533.1 Configuration*/
const Device UA2032674 = {S59005, 132, { &P12945,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A780 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2744095 = {S59006, 63, { &P12945,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Lenovo A780 Build/GRJ90)*/
const Device UA1568 = {S59007, 104, { &P11636,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonW705/R1GA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.4*/
const Device UA22097 = {S59008, 118, { &P11636,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonW705u/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 model-orange JavaPlatform/JP-8.4.3*/
const Device UA2797502 = {S59009, 144, { &P22972,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I537 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA163726 = {S59010, 149, { &P17959,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-us; NXM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA228258 = {S59011, 149, { &P17959,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.6; en-us; NXM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA242211 = {S59012, 149, { &P17959,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.6; en-us; NXM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA637319 = {S59013, 150, { &P17959,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.14; en-us; NXM901 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2643 = {S59014, 120, { &P11941,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW580c_CMCC/R8BE SEMC-SW/R8BE Release/01.15.2008 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA18831 = {S59015, 83, { &P11941,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW580i/R8BE Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61879 = {S59016, 83, { &P11941,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW580c/R8BE Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA1977460 = {S59017, 154, { &P18283,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T999 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028535 = {S59018, 154, { &P18283,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9300 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031053 = {S59019, 158, { &P18283,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2586657 = {S59020, 140, { &P21627,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC first Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2586659 = {S59021, 62, { &P21627,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC first Build/JZO54K)*/
const Device UA3065023 = {S59022, 136, { &P18713,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P769 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3068070 = {S59023, 136, { &P18713,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P768 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068254 = {S59024, 136, { &P18713,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P760 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA675620 = {S59025, 118, { &P15364,&P17285,&P22455,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.1 Chrome/17.0.1000.1 Safari/535.11*/
const Device UA830482 = {S59026, 125, { &P15364,&P17285,&P22455,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.2 Chrome/17.0.1000.2 Safari/535.11*/
const Device UA576656 = {S59027, 81, { &P18095,&P18094,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (S60; SymbOS; Opera Mobi/SYB-X; U; en-US) Presto/2.9.201 Version/11.50*/
const Device UA2511455 = {S59028, 93, { &P11905,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA6120c) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA27074 = {S59029, 112, { &P14709,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-M8800/M8800ADIF2 SHP/VPP/R5 NetFront/3.5 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA49283 = {S59030, 151, { &P12974,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; it-it; ALCATEL one touch 890D Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90616 = {S59031, 151, { &P12974,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ALCATEL_one_touch_890D Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370854 = {S59032, 139, { &P12974,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; ALCATEL one touch 890D Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1902369 = {S59033, 75, { &P12974,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; ALCATEL one touch 890D Build/FRG83G)*/
const Device UA615721 = {S59034, 48, { &P12280,&P18117,&P17470,&P18092 }};	/*Useragent - Facebook 4110.0 (iPhone; iPhone OS 5.1.1; en_US)*/
const Device UA658580 = {S59035, 46, { &P12280,&P18117,&P17470,&P18092 }};	/*Useragent - Facebook 4110.0 (iPhone; iPhone OS 5.1; en_US)*/
const Device UA1996894 = {S59036, 65, { &P12280,&P18117,&P17470,&P18092 }};	/*Useragent - Instagram 2.5.1 (iPhone; iPhone OS 5.1.1; en_US) AppleWebKit/420+*/
const Device UA2463271 = {S59037, 67, { &P12280,&P18117,&P17470,&P18092 }};	/*Useragent - com.cnn.iphone/150.85 (unknown, iPhone OS 5.1.1, iPhone, Scale/2.X)*/
const Device UA682544 = {S59038, 139, { &P15794,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXL_Beats_X315e; zh-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA899345 = {S59039, 139, { &P15794,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXL_Beats_X315b; en-au) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA1993080 = {S59040, 139, { &P15794,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXL_Beats_X315e; en-gb) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2155006 = {S59041, 139, { &P15794,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXL_Beats_X315e; en-hk) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2164775 = {S59042, 139, { &P15794,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXL_Beats_X315e; en-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2168157 = {S59043, 139, { &P15794,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXL_Beats_X315e; en-in) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2222984 = {S59044, 144, { &P15794,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/SensationXL_Beats/2.32.161.2; de-ch) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2336897 = {S59045, 139, { &P15794,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXL_Beats_X315b; en-nz) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2589517 = {S59046, 144, { &P15794,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/SensationXL_Beats/2.32.161.2; de-at) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2722642 = {S59047, 144, { &P15794,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/SensationXL_Beats/2.32.161.2; fr-ch) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2815256 = {S59048, 139, { &P15794,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXL_Beats_X315e; ru-ru) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2819510 = {S59049, 139, { &P15794,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXL_Beats_X315e; zh-cn) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2830914 = {S59050, 144, { &P15794,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/SensationXL_Beats/2.32.161.2; en-gb) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA3043161 = {S59051, 139, { &P15794,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXL_Beats_X315e; vi-vn) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA911293 = {S59052, 133, { &P15102,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MZ605 Build/I.7.1-39.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1995356 = {S59053, 133, { &P15102,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; MZ605 Build/I.7.1-39.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2558133 = {S59054, 156, { &P13035,&P5043,&P24676,&P18092 }};	/*Useragent - NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-02) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2571566 = {S59055, 152, { &P15785,&P5843,&P24676,&P18092 }};	/*Useragent - Nokia201/2.0 (11.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia201) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA5074 = {S59056, 121, { &P15526,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Vodafone/1.0/HTC_HD_mini/1.11.172.2 (87652)*/
const Device UA5080 = {S59057, 121, { &P15526,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Vodafone/1.0/HTC_HD_mini/1.10.161.1 (86611)*/
const Device UA7548 = {S59058, 121, { &P15526,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Vodafone/1.0/HTC_HD_mini/1.41.161.1 (10904)*/
const Device UA7549 = {S59059, 121, { &P15526,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Vodafone/1.0/HTC_HD_mini/1.36.184.2 (94538)*/
const Device UA7551 = {S59060, 121, { &P15526,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Vodafone/1.0/HTC_HD_mini/1.11.164.1 (87733)*/
const Device UA22918 = {S59061, 144, { &P15526,&P5448,&P17183,&P18092 }};	/*Useragent - HD_mini/1.36.621.4 (94538) Mozilla/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA28149 = {S59062, 95, { &P15526,&P5448,&P17183,&P18092 }};	/*Useragent - HTC_HD_mini_T5555 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA28158 = {S59063, 99, { &P15526,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) HD_mini_T5555/320x480*/
const Device UA38204 = {S59064, 121, { &P15526,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Vodafone/1.0/HTC_HD_mini/1.41.162.1 (10904)*/
const Device UA38205 = {S59065, 121, { &P15526,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Swisscom/1.0/HTC_HD_mini/1.11.166.1 (87652)*/
const Device UA51839 = {S59066, 121, { &P15526,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Vodafone/1.0/HTC_HD_mini/1.41.164.1 (10904)*/
const Device UA62695 = {S59067, 91, { &P15526,&P5448,&P17183,&P18092 }};	/*Useragent - HD_mini_T5555 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA73997 = {S59068, 121, { &P15526,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Vodafone/1.0/HTC_HD_mini/1.11.162.1 (87652)*/
const Device UA74835 = {S59069, 121, { &P15526,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Swisscom/1.0/HTC_HD_mini/1.41.166.1 (10904)*/
const Device UA79581 = {S59070, 121, { &P15526,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Swisscom/1.0/HTC_HD_mini/1.11.167.1 (87652)*/
const Device UA89737 = {S59071, 121, { &P15526,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Vodafone/1.0/HTC_HD_mini/1.00.164.1 (80896)*/
const Device UA1973235 = {S59072, 117, { &P14714,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA1987939 = {S59073, 115, { &P14714,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02/05.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA2777832 = {S59074, 116, { &P14714,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02.5/07.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6*/
const Device UA2949819 = {S59075, 114, { &P14714,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02/07.16; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6*/
const Device UA2423989 = {S59076, 119, { &P15373,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5530) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2428446 = {S59077, 41, { &P15373,&P5129,&P24673,&P18092 }};	/*Useragent - Nokia5530UCBrowser/8.7.1.234/50/444/UCWEB*/
const Device UA2727167 = {S59078, 31, { &P15373,&P5129,&P24673,&P18092 }};	/*Useragent - Nokia5530/UCWEB8.7.0.187/50/999*/
const Device UA207154 = {S59079, 135, { &P15364,&P17818,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.24 (KHTML, like Gecko) Ubuntu/11.04 Chromium/11.0.696.71 Chrome/11.0.696.71 Safari/534.24*/
const Device UA316380 = {S59080, 137, { &P15364,&P17818,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Ubuntu/11.04 Chromium/11.0.696.71 Chrome/11.0.696.71 Safari/534.24*/
const Device UA2338489 = {S59081, 143, { &P15364,&P17818,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.21 (KHTML, like Gecko) Ubuntu/11.04 Chromium/11.0.676.0 Chrome/11.0.676.0 Safari/534.21*/
const Device UA2752639 = {S59082, 93, { &P14358,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae63-1) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2947849 = {S59083, 92, { &P14358,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE63) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA1996043 = {S59084, 155, { &P14872,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-D700 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2042167 = {S59085, 155, { &P14872,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-D700 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2201508 = {S59086, 155, { &P14872,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-D700 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2362249 = {S59087, 159, { &P14872,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-D700 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2384738 = {S59088, 155, { &P14872,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-D700 Build/JZ054K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2557474 = {S59089, 138, { &P18458,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I200 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2567219 = {S59090, 138, { &P18458,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I200 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668786 = {S59091, 131, { &P18458,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I200 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2712207 = {S59092, 58, { &P15364,&P7127,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:21.0) Gecko/X Firefox/21.0*/
const Device UA2963928 = {S59093, 197, { &P18442,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7105 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.5/580,2.0.5.580,2013-06-26 22:40, +0100, gb*/
const Device UA3027730 = {S59094, 197, { &P18442,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L900 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0500, us*/
const Device UA3027733 = {S59095, 205, { &P18442,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us*/
const Device UA3028228 = {S59096, 198, { &P18442,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I317M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, ca*/
const Device UA3028271 = {S59097, 197, { &P18442,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T889 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us*/
const Device UA3029028 = {S59098, 197, { &P18442,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7105 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb*/
const Device UA3029632 = {S59099, 197, { &P18442,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb*/
const Device UA872373 = {S59100, 114, { &P19946,&P17483,&P4732,&P18092 }};	/*Useragent - LG-A340/V10d; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.22250; U; en-US) Opera 9.50*/
const Device UA2752116 = {S59101, 131, { &P22922,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; M470BSA Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2974031 = {S59102, 126, { &P19775,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A210 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA30370 = {S59103, 142, { &P12311,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; SPH-M900 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA67672 = {S59104, 142, { &P12311,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; SPH-M900 Build/CUPCAKE) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA89017 = {S59105, 129, { &P12311,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; SPH-M900) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA129707 = {S59106, 136, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (iDEN; U; CPU iDEN Android Opus One; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5G77 Safari/525.20*/
const Device UA641282 = {S59107, 153, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v2.2; en-us; WM8650 Build/FRG83D_v2.2 Android Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1899129 = {S59108, 154, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Cyberus 3.5; en-us; Cyberus Tablet Build/DRD20) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA1948719 = {S59109, 137, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android DG2.30; md-us; SYTAB7MX Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020203 = {S59110, 139, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android BoosT3.5; es-us; HTC D/S Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021126 = {S59111, 77, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android v2.2; WM8650 Build/FRG83D_v2.2 Android Froyo)*/
const Device UA2463861 = {S59112, 139, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android unknown; zh-tw; GETEK-A8 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2528487 = {S59113, 160, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 5.2.6(Android 5.1.1); en-us; mtk6515_6626_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2560823 = {S59114, 160, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 5.2.9(Android 5.1.1); en-us; mtk6515_6626_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2723154 = {S59115, 134, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android unknown; zh-cn; W99 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2732841 = {S59116, 142, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android GFIVE-4.0; en-us; GFIVE A77 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2743632 = {S59117, 141, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android unknown; zh-cn; NOAIN A900 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2744068 = {S59118, 140, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android unknown; zh-cn; TETC-W318 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2745358 = {S59119, 139, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android unknown; zh-cn; SOP-W168 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2764098 = {S59120, 137, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android unknown; vi-vn; Sen S1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2791753 = {S59121, 160, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 5.2.8(Android 5.1.1); en-us; mtk6515_6626_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2845607 = {S59122, 160, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 5.2.1(Android 5.1.1); en-us; mtk6515_6626_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2980239 = {S59123, 140, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android unknown; zh-cn; HOSIN U98 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2984220 = {S59124, 142, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android unknown; en-us; MAXX MA-808 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3012964 = {S59125, 148, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android MASSTEL 4.0; vi-vn; MASSTEL (M18) Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3043763 = {S59126, 145, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Freakenstien 1.1; en-us; Hotrod Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3057159 = {S59127, 157, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android GFIVE 4.0; es-es; GFIVE Spark(A1) Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3070092 = {S59128, 18, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - mSpot Android Aria*/
const Device UA3074285 = {S59129, 146, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Android OS,v2.3.6; en-; DTC GT6  Build/GT830) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3099623 = {S59130, 160, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 5.2.3(Android 5.1.1); en-us; mtk6515_6626_gb2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3100011 = {S59131, 167, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android GFIVE-4.0; en-us; GFIVE Blade X (F600) Build/F600-V1.0-2012.11.13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3108664 = {S59132, 151, { &P15364,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android QuadCore-4.0.0; vi-vn; M180 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2569630 = {S59133, 82, { &P15364,&P16110,&P21791,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:19.0) Gecko/X Firefox/19.0 SeaMonkey/2.16.2*/
const Device UA244464 = {S59134, 113, { &P15364,&P5690,&P22452,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1*/
const Device UA573784 = {S59135, 120, { &P15364,&P5690,&P22452,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1*/
const Device UA907440 = {S59136, 139, { &P13141,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I727R Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2223445 = {S59137, 139, { &P13141,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SHV-E110S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261604 = {S59138, 139, { &P13141,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E110S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2381179 = {S59139, 138, { &P13141,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9210 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2672897 = {S59140, 130, { &P15876,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T779 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676681 = {S59141, 130, { &P15876,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P5100 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676825 = {S59142, 130, { &P15876,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P5100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA69828 = {S59143, 123, { &P13035,&P5043,&P17433,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-02/06.51; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.1.21.8.2*/
const Device UA766322 = {S59144, 118, { &P13035,&P5043,&P17433,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-02.1/06.52; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.1.21.8.2*/
const Device UA908443 = {S59145, 70, { &P15364,&P7133,&P17209,&P18163 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:2.0.1) Gecko/X Firefox/7.0.1/Nutch-1.4*/
const Device UA2380618 = {S59146, 148, { &P23102,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HUAWEI U8681 Build/HuaweiU8681) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3043440 = {S59147, 70, { &P23102,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HUAWEI U8681 Build/HuaweiU8681)*/
const Device UA2535079 = {S59148, 152, { &P18219,&P5843,&P24676,&P18092 }};	/*Useragent - Nokia202/2.0 (20.52) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia202) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2934694 = {S59149, 152, { &P18219,&P5843,&P24676,&P18092 }};	/*Useragent - Nokia203/2.0 (20.52) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia203) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2566914 = {S59150, 139, { &P18724,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-F Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567688 = {S59151, 139, { &P18724,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-H Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583322 = {S59152, 134, { &P18724,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SmartTabII7 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667968 = {S59153, 131, { &P18724,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A2107A-H Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2974020 = {S59154, 129, { &P22922,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; M470BSA Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA173269 = {S59155, 107, { &P15364,&P5690,&P20804,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 5.1; en-US) AppleWebKit/527+ (KHTML, like Gecko, Safari/419.3) Qt/4.5.3*/
const Device UA814105 = {S59156, 107, { &P15364,&P5690,&P20804,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 5.2; en-US) AppleWebKit/527+ (KHTML, like Gecko, Safari/419.3) Qt/4.5.3*/
const Device UA2207080 = {S59157, 109, { &P15364,&P5690,&P20804,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 5.2; en-US) AppleWebKit/523.15 (KHTML, like Gecko, Safari/419.3) Qt/4.4.0*/
const Device UA2802069 = {S59158, 107, { &P15364,&P5690,&P20804,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Qt/4.6.3 Safari/532.4*/
const Device UA2200751 = {S59159, 134, { &P19788,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; E350 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2527975 = {S59160, 134, { &P19788,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; E350 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA43745 = {S59161, 104, { &P13734,&P17484,&P3636,&P18092 }};	/*Useragent - SAMSUNG-GT-S5603/1.0 SHP/VPP/R5 Jasmine/0.8 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA62311 = {S59162, 105, { &P13734,&P17484,&P3636,&P18092 }};	/*Useragent - SAMSUNG-GT-S5603T/1.0 SHP/VPP/R5 Jasmine/0.8 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA2974817 = {S59163, 139, { &P23240,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; USCC-C6721 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA629848 = {S59164, 79, { &P14358,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE63)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA683186 = {S59165, 30, { &P14358,&P5890,&P24667,&P18092 }};	/*Useragent - NokiaE63/UCWEB8.2.0.116/28/800*/
const Device UA2360093 = {S59166, 79, { &P14358,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokiae63-1) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2712104 = {S59167, 58, { &P15364,&P17188,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0; rv:21.0) Gecko/X Firefox/21.0*/
const Device UA2780285 = {S59168, 61, { &P22599,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SHV-E300S Build/JDQ39)*/
const Device UA3036772 = {S59169, 172, { &P22599,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ko-kr; SHV-E300S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 NAVER(inapp; search; 230; 4.7.7)*/
const Device UA2974617 = {S59170, 138, { &P19878,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I497 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA54094 = {S59171, 44, { &P15364,&P17017,&P3625,&P18092 }};	/*Useragent - pcdc751/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA54503 = {S59172, 79, { &P15364,&P17017,&P3625,&P18092 }};	/*Useragent - Amoi-F90/Plat-F/WAP2.0/MIDP1.0/CLDC1.0 UP.Browser/6.2.2.6.f.1.100 (GUI) MMP/1.0*/
const Device UA62958 = {S59173, 45, { &P15364,&P17017,&P3625,&P18092 }};	/*Useragent - WP8990/T05_0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA85992 = {S59174, 48, { &P15364,&P17017,&P3625,&P18092 }};	/*Useragent - OPWV-SDK/62 UP.Browser/6.2.2.1.208 (GUI) MMP/2.0*/
const Device UA86913 = {S59175, 47, { &P15364,&P17017,&P3625,&P18092 }};	/*Useragent - KWC-KX18/10073 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA86983 = {S59176, 57, { &P15364,&P17017,&P3625,&P18092 }};	/*Useragent - M1000-2X0/1.0.11 UP.Browser/6.2.3.9.g.1.111 (GUI) MMP/2.0*/
const Device UA115804 = {S59177, 44, { &P15364,&P17017,&P3625,&P18092 }};	/*Useragent - pcdc731/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA537098 = {S59178, 32, { &P15364,&P17017,&P3625,&P18092 }};	/*Useragent - UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA905771 = {S59179, 39, { &P15364,&P17017,&P3625,&P18092 }};	/*Useragent - r270v1 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA166716 = {S59180, 137, { &P16744,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SHW-M130L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA174607 = {S59181, 137, { &P16744,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SHW-M130K Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA433143 = {S59182, 135, { &P16744,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SHW-M130K Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA541096 = {S59183, 61, { &P16744,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SHW-M130L Build/FROYO)*/
const Device UA728592 = {S59184, 135, { &P16744,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SHW-M130L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019251 = {S59185, 61, { &P16744,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SHW-M130K Build/FROYO)*/
const Device UA51081 = {S59186, 143, { &P13806,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; ADR6225 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA53806 = {S59187, 122, { &P14035,&P5208,&P4223,&P18092 }};	/*Useragent - NetFront/3.5.1 (BREW 3.1.5; U; en-us; LG; NetFront/3.5.1/AMB) Sprint LX610 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2346395 = {S59188, 93, { &P18436,&P16110,&P22733,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; Avant Browser)*/
const Device UA2427350 = {S59189, 98, { &P18436,&P16110,&P22733,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Win64; x64; Trident/6.0; Touch; Avant Browser)*/
const Device UA2496947 = {S59190, 86, { &P18436,&P16110,&P22733,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; Touch; Avant Browser)*/
const Device UA2752680 = {S59191, 140, { &P21605,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C2104 Build/15.0.A.1.31) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA38750 = {S59192, 80, { &P12490,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK700c/R2AE SEMC-Browser/4.0.3Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49189 = {S59193, 87, { &P12490,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK700c/R2E SEMC-Browser/4.0 Profile/MIDP-1.0 MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53858 = {S59194, 80, { &P12490,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK700c/R2L SEMC-Browser/4.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53859 = {S59195, 81, { &P12490,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK700c/R2AA SEMC-Browser/4.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63526 = {S59196, 81, { &P12490,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK700c/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA221039 = {S59197, 133, { &P18039,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; F-12C Build/F0001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA729797 = {S59198, 57, { &P18039,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; F-12C Build/F0001)*/
const Device UA1993289 = {S59199, 61, { &P18254,&P7480,&P18418,&P18165 }};	/*Useragent - Mozilla/5.0 (compatible; heritrix/1.14.4 +http://parsijoo.ir)*/
const Device UA2676236 = {S59200, 154, { &P12605,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734725 = {S59201, 147, { &P12605,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676034 = {S59202, 140, { &P17625,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST25i Build/6.1.1.B.1.54) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676340 = {S59203, 140, { &P17625,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST25a Build/6.1.1.B.1.10) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2736091 = {S59204, 137, { &P17625,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xperia U Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA21300 = {S59205, 97, { &P17968,&P5477,&P4861,&P18092 }};	/*Useragent - Android (Linux; U; Android 2.1-update1; zh-cn; mooncake) AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA26214 = {S59206, 147, { &P17968,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-by; MTC_Android Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA40855 = {S59207, 94, { &P17968,&P5477,&P4861,&P18092 }};	/*Useragent - Android (Linux; U; Android 2.1-update1; zh-cn; C8500) AppleWebKit/528.5+ (KHTML) Version/3.1.2*/
const Device UA42511 = {S59208, 134, { &P17968,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fa_ir) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky/1.5.0 Safari/533.1*/
const Device UA44231 = {S59209, 150, { &P17968,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-ua; life:) Android Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA56045 = {S59210, 188, { &P17968,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Android for Telechips TCC8900 Evaluation Board (US) Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA56779 = {S59211, 170, { &P17968,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Android for Telechips TCC8902 MID Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA67274 = {S59212, 185, { &P17968,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Android for Telechips TCC8900 Release Board (US) Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA67428 = {S59213, 200, { &P17968,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Android for Telechips TCC8900 Evaluation Board (multi-language) Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA67556 = {S59214, 181, { &P17968,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; Android for Telechips TCC8902 Tablet-PC (DE) Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA124204 = {S59215, 56, { &P17968,&P5477,&P4861,&P18092 }};	/*Useragent - CORE/6.506.4.1 OpenCORE/2.02 (Linux;Android 2.1-update1)*/
const Device UA290439 = {S59216, 140, { &P17968,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MID Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA301441 = {S59217, 148, { &P17968,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Android MID Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA751344 = {S59218, 129, { &P17968,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1.2; ru-ru Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA770126 = {S59219, 129, { &P17968,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1.1; ru-ru Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1987520 = {S59220, 135, { &P17968,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; OMAP_SS Build/ECLAIR) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2018210 = {S59221, 146, { &P17968,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-us; MID Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018850 = {S59222, 144, { &P17968,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; generic Build/EPE54B) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019990 = {S59223, 82, { &P17968,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2021074 = {S59224, 134, { &P17968,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-kr Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2021550 = {S59225, 149, { &P17968,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; sr-rs; Vip_MiniDroid Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2392644 = {S59226, 151, { &P17968,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; it-it; TT-7000H Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2737637 = {S59227, 137, { &P22760,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ThL W8 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2960060 = {S59228, 137, { &P22760,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; it-it; ThL W8 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999211 = {S59229, 59, { &P22760,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; ThL W8 Build/JOP40D)*/
const Device UA3006052 = {S59230, 138, { &P22760,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; ThL W8+ Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA49931 = {S59231, 137, { &P13109,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-ca; HTC-X710a Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA681170 = {S59232, 61, { &P13109,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; HTC-X710a Build/GRJ22)*/
const Device UA2360153 = {S59233, 137, { &P23190,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Marquis_MP977 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2507714 = {S59234, 142, { &P20824,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; VS TOUCHTAB 10.1DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2764610 = {S59235, 71, { &P20824,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; VS TOUCHTAB 10.1DC Build/JRO03H)*/
const Device UA2555861 = {S59236, 139, { &P21481,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A860S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778551 = {S59237, 61, { &P21481,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A860S Build/JZO54K)*/
const Device UA2732949 = {S59238, 137, { &P18595,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-I8190 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2797575 = {S59239, 137, { &P18595,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-I8190 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3088754 = {S59240, 61, { &P18595,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; GT-I8190 Build/GRK39F)*/
const Device UA3131466 = {S59241, 145, { &P18472,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-79) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132955 = {S59242, 151, { &P18472,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT925 Build/9.8.2Q-8-XT925_VQL-12.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3065581 = {S59243, 136, { &P22727,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGP312 Build/10.3.1.C.0.136) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA20388 = {S59244, 175, { &P13691,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB508 Build/Blur_Version.2.112.12.MB508.ATT.en.US Flex/P007) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42871 = {S59245, 175, { &P13691,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB508 Build/Blur_Version.2.159.17.MB508.ATT.en.US Flex/P011) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42869 = {S59246, 151, { &P13691,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MB508 Build/3.4.2-159_SAG-17_SecATT) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026986 = {S59247, 130, { &P21839,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; WS802C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2353532 = {S59248, 136, { &P21839,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Penta WS802C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2458121 = {S59249, 136, { &P21839,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Penta WS802C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2205123 = {S59250, 130, { &P20187,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Novo7Fire Build/Xqm) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257545 = {S59251, 130, { &P20187,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Novo7Fire Build/Xtm) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2456387 = {S59252, 131, { &P20187,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Novo7Fire Build/X.qm) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2838845 = {S59253, 131, { &P20187,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Novo7Fire Build/X.tm) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2678419 = {S59254, 137, { &P18301,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS840 4G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2864848 = {S59255, 130, { &P18301,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS840 4G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2996921 = {S59256, 136, { &P18301,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS840PP Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA24290 = {S59257, 52, { &P15675,&P5214,&P3487,&P18092 }};	/*Useragent - KWC-Rio/ACI1005 UP.Browser/7.2.7.2.563 (GUI) MMP/2.0*/
const Device UA49419 = {S59258, 52, { &P15675,&P5214,&P3487,&P18092 }};	/*Useragent - KWC-Rio/ACI1003 UP.Browser/7.2.7.2.563 (GUI) MMP/2.0*/
const Device UA63312 = {S59259, 52, { &P15675,&P5214,&P3487,&P18092 }};	/*Useragent - KWC-Rio/ACJ1005 UP.Browser/7.2.7.2.562 (GUI) MMP/2.0*/
const Device UA156947 = {S59260, 52, { &P15675,&P5214,&P3487,&P18092 }};	/*Useragent - KWC-Rio/ABY1009 UP.Browser/7.2.7.2.562 (GUI) MMP/2.0*/
const Device UA629240 = {S59261, 53, { &P15675,&P5214,&P3487,&P18092 }};	/*Useragent - KWC-Rio/ACK10006 UP.Browser/7.2.7.2.563 (GUI) MMP/2.0*/
const Device UA673120 = {S59262, 73, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.1-update1; zh-cn; C8500; 240*320) UCWEB7.9.3.103/139/999*/
const Device UA707286 = {S59263, 72, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; TCL_A919; 320*480) UCWEB7.9.3.103/139/32463*/
const Device UA750402 = {S59264, 69, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; Nexus_S; 480*800) UCWEB7.9.3.103/139/800*/
const Device UA787792 = {S59265, 68, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; 7260; 480*800) UCWEB7.9.3.103/139/32781*/
const Device UA1906416 = {S59266, 64, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; M9; 640*960) UCWEB7.9.3.103/139/800*/
const Device UA1943506 = {S59267, 69, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; GN380; 480*800) UCWEB7.9.3.103/139/32702*/
const Device UA1965990 = {S59268, 69, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; GN106; 320*480) UCWEB7.9.3.103/139/32702*/
const Device UA1978474 = {S59269, 69, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; GN205; 480*800) UCWEB7.9.3.103/139/32702*/
const Device UA2158694 = {S59270, 73, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; MI-ONE_Plus; 480*854) UCWEB7.9.4.145/139/800*/
const Device UA2301108 = {S59271, 75, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.1-update1; zh-cn; OMAP_SS; 480*800) UCWEB7.9.3.103/139/800*/
const Device UA2302234 = {S59272, 64, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.1; zh-cn; D539; 320*480) UCWEB7.9.3.103/139/800*/
const Device UA2304305 = {S59273, 75, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.1-update1; zh-cn; XT702; 480*854) UCWEB7.9.3.103/139/32160*/
const Device UA2308469 = {S59274, 65, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 1.5; zh-cn; ME600; 320*480) UCWEB7.9.3.103/139/800*/
const Device UA2324705 = {S59275, 72, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HOSIN_U3; 320*480) UCWEB7.9.3.103/139/32684*/
const Device UA2337121 = {S59276, 72, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; TCL_W939; 320*480) UCWEB7.9.3.103/139/32870*/
const Device UA2414711 = {S59277, 68, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; Z3Z6Z7; 320*480) UCWEB7.9.4.145/139/800*/
const Device UA2426227 = {S59278, 63, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; G1; 480*800) UCWEB7.9.0.94/139/352*/
const Device UA2428038 = {S59279, 67, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GLX_G1; 320*480) UCWEB7.9.0.94/139/352*/
const Device UA2482562 = {S59280, 63, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.7; zh-cn; X8; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2543406 = {S59281, 75, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; YUSUN_-_W_800; 320*480) UCWEB7.9.3.103/139/800*/
const Device UA2545339 = {S59282, 72, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; ZTE_N960; 480*800) UCWEB7.9.3.103/139/32602*/
const Device UA2649167 = {S59283, 73, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; signal75_gb2; 320*480) UCWEB7.9.0.94/139/800*/
const Device UA2693194 = {S59284, 66, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; C208; 320*480) UCWEB7.9.4.145/139/800*/
const Device UA2775377 = {S59285, 76, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; Coolpad_7005; 320*480) UCWEB7.9.3.103/139/32765*/
const Device UA2787126 = {S59286, 77, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.2; zh-cn; Coolpad_W706+; 320*480) UCWEB7.9.4.145/139/32411*/
const Device UA2811628 = {S59287, 66, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; A811; 320*480) UCWEB7.9.4.145/139/800*/
const Device UA2820948 = {S59288, 67, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; W58SX; 320*480) UCWEB7.9.4.145/139/800*/
const Device UA2830413 = {S59289, 67, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; MI_1S; 480*854) UCWEB7.9.4.145/139/800*/
const Device UA2839820 = {S59290, 63, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; A7; 320*480) UCWEB7.9.0.94/139/355*/
const Device UA2952806 = {S59291, 65, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; Z10; 768*1280) UCWEB7.9.0.94/139/352*/
const Device UA2980253 = {S59292, 67, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; TSC_Z; 320*480) UCWEB7.9.4.145/139/800*/
const Device UA2981281 = {S59293, 70, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; EBEST_F5; 320*480) UCWEB7.9.4.145/139/800*/
const Device UA2983321 = {S59294, 72, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; T-Mobile_G2; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA3043297 = {S59295, 69, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; Epade_S3; 320*480) UCWEB7.9.0.94/139/800*/
const Device UA3098398 = {S59296, 77, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; SPHS_on_Hsdroid; 320*480) UCWEB7.9.4.145/139/800*/
const Device UA3099445 = {S59297, 64, { &P15364,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; A21; 540*960) UCWEB7.9.0.94/139/444*/
const Device UA2456876 = {S59298, 126, { &P20894,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2762059 = {S59299, 55, { &P20894,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; M3 Build/JRO03H)*/
const Device UA2404245 = {S59300, 43, { &P20322,&P22784,&P17470,&P18092 }};	/*Useragent - http://www.msai.in/uaprof/micromax/X455.xml*/
const Device UA455129 = {S59301, 134, { &P16001,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-06D Build/S3130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626183 = {S59302, 134, { &P16001,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-06D Build/S2281) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA897826 = {S59303, 134, { &P16001,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-06D Build/S5081) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1993513 = {S59304, 138, { &P16001,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-06DNERV Build/S6050) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1996913 = {S59305, 138, { &P16001,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-06DNERV Build/S7231) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1997126 = {S59306, 134, { &P16001,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-06D Build/S7231) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030795 = {S59307, 58, { &P16001,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SH-06D Build/S7231)*/
const Device UA2230692 = {S59308, 138, { &P16001,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-06DNERV Build/S8310) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2570034 = {S59309, 156, { &P18635,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2772906 = {S59310, 155, { &P18635,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2676080 = {S59311, 140, { &P12841,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC PH39100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734781 = {S59312, 140, { &P12841,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC PH39100 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA33389 = {S59313, 146, { &P24569,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; tr-tr; Turkcell T20 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA575794 = {S59314, 146, { &P24569,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; tr-tr; Turkcell T20 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA655729 = {S59315, 146, { &P24569,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; tr-tr; Turkcell T20 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026743 = {S59316, 146, { &P24569,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; tr-tr; Turkcell T20 Build/HuaweiU8650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1997678 = {S59317, 139, { &P12945,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A780L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2644322 = {S59318, 61, { &P12945,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-A780L Build/IMM76D)*/
const Device UA2039613 = {S59319, 117, { &P15767,&P17146,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Twitter for iPhone*/
const Device UA2197902 = {S59320, 115, { &P15767,&P17146,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Twitter for iPhone*/
const Device UA899803 = {S59321, 153, { &P15297,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; SonyEricssonMT15a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031775 = {S59322, 141, { &P15297,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MT15a Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2550368 = {S59323, 141, { &P15297,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MT15a Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1995177 = {S59324, 138, { &P21083,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM107SH Build/S0006) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1995794 = {S59325, 138, { &P21083,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM107SH Build/S0010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028701 = {S59326, 60, { &P21083,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SBM107SH Build/S0010)*/
const Device UA2820889 = {S59327, 139, { &P21083,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM107SHB Build/S0006) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2876921 = {S59328, 138, { &P21083,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM107SH Build/S0024) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3061355 = {S59329, 138, { &P21083,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM107SH Build/S0026) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3065744 = {S59330, 146, { &P15880,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083223 = {S59331, 146, { &P15880,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ME865 Build/6.7.2_GC-220-EDS-3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067179 = {S59332, 136, { &P21794,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9152 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2367721 = {S59333, 147, { &P20263,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Digitac Ultratab;C925 Build/ICS) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2467763 = {S59334, 151, { &P20263,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U; Android 4.0.3; en-us;Digitac Ultratab;C925 Build/ICS) AppleWebKit/534.30 (KHTML,like Gecko) Version/4.0 Safari/534.30;SurfTab_7.0*/
const Device UA88703 = {S59335, 69, { &P18182,&P5448,&P4626,&P18092 }};	/*Useragent - Opera/9.7 (Windows Mobile; PPC; Opera Mobi/35166; U; en) Presto/2.2.1*/
const Device UA103909 = {S59336, 69, { &P18182,&P5448,&P4626,&P18092 }};	/*Useragent - Opera/9.7 (WindowsMobile; PPC; Opera Mobi/35267; U; en; Presto/2.1.1)*/
const Device UA2676277 = {S59337, 146, { &P15880,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676915 = {S59338, 146, { &P15880,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ME865 Build/6.7.2_GC-220-EDS-3) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2753102 = {S59339, 139, { &P18447,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT30p Build/7.0.A.3.195) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2435237 = {S59340, 77, { &P18088,&P5567,&P17370,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/5.0/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2560572 = {S59341, 140, { &P20006,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30a Build/9.1.A.0.489) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2192190 = {S59342, 134, { &P20140,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GEM10312BK Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2231519 = {S59343, 134, { &P20140,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GEM10312BK Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2709225 = {S59344, 64, { &P21602,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; C5302 Build/12.0.A.1.211)*/
const Device UA2752368 = {S59345, 142, { &P21602,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; C5302 Build/12.0.A.1.211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2960186 = {S59346, 142, { &P21602,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C5302 Build/12.0.A.1.257) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3059411 = {S59347, 142, { &P21602,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; C5302 Build/12.0.A.2.245) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3107096 = {S59348, 142, { &P21602,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C5302 Build/12.0.A.1.284) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3111705 = {S59349, 142, { &P21602,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; C5302 Build/12.0.A.2.254) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2673554 = {S59350, 140, { &P15877,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676336 = {S59351, 147, { &P15877,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA662305 = {S59352, 114, { &P13107,&P5043,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-06/07.48; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.9*/
const Device UA251868 = {S59353, 132, { &P18011,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; IS05 Build/S9290) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458457 = {S59354, 132, { &P18011,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; IS05 Build/S3080) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020434 = {S59355, 130, { &P18011,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-; IS05 Build/S9290) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2197243 = {S59356, 132, { &P18011,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; IS05 Build/S8221) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1988077 = {S59357, 130, { &P20583,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN7BG3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257487 = {S59358, 145, { &P20558,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; XOOM 2 ME Build/7.7.1-128_MZ608-14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2386542 = {S59359, 147, { &P20558,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MZ608 Build/7.7.1-141-7-FLEM-UMTS-LA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2708404 = {S59360, 151, { &P18552,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2752160 = {S59361, 144, { &P18552,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA3131351 = {S59362, 144, { &P22582,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA2237537 = {S59363, 154, { &P19779,&P5629,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A1+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2509358 = {S59364, 52, { &P15364,&P17017,&P4578,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; PPC; Trident/4.0)*/
const Device UA2852704 = {S59365, 57, { &P15364,&P17017,&P4578,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; PPC; SV1; Trident/1.0)*/
const Device UA2707405 = {S59366, 137, { &P22685,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E460 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2866521 = {S59367, 138, { &P22685,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E460f Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2674164 = {S59368, 141, { &P15364,&P19822,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu/12.10 Chromium/25.0.1364.172 Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2674177 = {S59369, 139, { &P15364,&P19822,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu/12.10 Chromium/25.0.1364.172 Chrome/25.0.1364.172 Safari/537.22*/
const Device UA615806 = {S59370, 125, { &P15364,&P7133,&P24199,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.13.13719.201*/
const Device UA636852 = {S59371, 186, { &P15364,&P7133,&P24199,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.13.13719.201*/
const Device UA775809 = {S59372, 193, { &P15364,&P7133,&P24199,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.13.13719.201*/
const Device UA1970062 = {S59373, 165, { &P15364,&P7133,&P24199,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C) QQBrowser/6.13.13719.201*/
const Device UA2394874 = {S59374, 178, { &P15364,&P7133,&P24199,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MALN; .NET4.0C) QQBrowser/6.13.13719.201*/
const Device UA2442510 = {S59375, 206, { &P15364,&P7133,&P24199,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; qdesk 2.3.1186.202; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.13.13719.201*/
const Device UA2982724 = {S59376, 197, { &P15364,&P7133,&P24199,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; MALN) QQBrowser/6.13.13719.201*/
const Device UA3118053 = {S59377, 253, { &P15364,&P7133,&P24199,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; QQDownload 716; Mozilla/4.0(Compatible-EmbeddedWB 14.59 http://bsalsa.com/ ; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0) QQBrowser/6.13.13719.201*/
const Device UA2554377 = {S59378, 153, { &P20002,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; ALCATEL ONE TOUCH 992D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950763 = {S59379, 75, { &P20002,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ALCATEL ONE TOUCH 992D Build/IMM76D)*/
const Device UA2540566 = {S59380, 60, { &P20587,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P715 Build/JZO54K)*/
const Device UA2540581 = {S59381, 138, { &P20587,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-P715 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2753278 = {S59382, 138, { &P19878,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I497 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2753961 = {S59383, 140, { &P23910,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30at Build/9.1.F.1.120) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3084416 = {S59384, 41, { &P17968,&P17979,&P24495,&P18092 }};	/*Useragent - Dolphin http client/10.1.5(105) (Android)*/
const Device UA3084417 = {S59385, 41, { &P17968,&P17979,&P24495,&P18092 }};	/*Useragent - Dolphin http client/10.1.5(104) (Android)*/
const Device UA3084415 = {S59386, 41, { &P17968,&P17979,&P24495,&P18092 }};	/*Useragent - Dolphin http client/10.1.4(103) (Android)*/
const Device UA3111789 = {S59387, 48, { &P17968,&P17979,&P24495,&P18092 }};	/*Useragent - Dolphin http client/10.1.0 Beta 3(247) (Android)*/
const Device UA3114993 = {S59388, 41, { &P17968,&P17979,&P24495,&P18092 }};	/*Useragent - Dolphin http client/10.1.0(248) (Android)*/
const Device UA3115133 = {S59389, 48, { &P17968,&P17979,&P24495,&P18092 }};	/*Useragent - Dolphin http client/10.1.0 Beta 2(246) (Android)*/
const Device UA910674 = {S59390, 127, { &P15767,&P17090,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari Version/5.1*/
const Device UA1950949 = {S59391, 127, { &P15767,&P17090,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari Version/5.1*/
const Device UA2404500 = {S59392, 152, { &P15767,&P17090,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; zh-cn Model:iPad2,1) UC AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7543.48.3*/
const Device UA2566095 = {S59393, 129, { &P15767,&P17090,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 4_3_3 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA2567683 = {S59394, 129, { &P15767,&P17090,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 4_3_5 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA2640635 = {S59395, 127, { &P15767,&P17090,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 Safari Version/5.1*/
const Device UA2758389 = {S59396, 127, { &P15767,&P17090,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 Safari Version/5.1*/
const Device UA2792715 = {S59397, 129, { &P15767,&P17090,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 4_3_2 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA2857914 = {S59398, 127, { &P15767,&P17090,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 4_3 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA2890963 = {S59399, 129, { &P15767,&P17090,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 4_3_1 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA3071751 = {S59400, 129, { &P15767,&P17090,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 4_3_4 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA2406956 = {S59401, 133, { &P18465,&P18110,&P19816,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Safari/535.19 Silk-Accelerated=false*/
const Device UA2752413 = {S59402, 139, { &P23167,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-H9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996667 = {S59403, 61, { &P23167,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-H9500 Build/JOP40D)*/
const Device UA50196 = {S59404, 128, { &P14612,&P17484,&P4732,&P18092 }};	/*Useragent - SAMSUNG-SGH-A777/A777UCIF2; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.14315; U; en-US) Opera 9.50*/
const Device UA2429194 = {S59405, 130, { &P21071,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Novo10 Hero Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2659432 = {S59406, 74, { &P21071,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Novo 10 Hero QuadCore Build/JRO03C)*/
const Device UA2696214 = {S59407, 59, { &P21071,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Novo10 Hero Build/X)*/
const Device UA2699445 = {S59408, 145, { &P21071,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Novo 10 Hero QuadCore Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2540542 = {S59409, 74, { &P21002,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-C3312R/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2877970 = {S59410, 165, { &P23215,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E330S Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3020701 = {S59411, 134, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BT210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021146 = {S59412, 146, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Avvio_790 Build/AvvioAvvio_790) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021223 = {S59413, 131, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 5I Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021260 = {S59414, 131, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; CTA Build/ICS.casper.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021573 = {S59415, 136, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A11Note Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021737 = {S59416, 143, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CA-201L Build/CA201LU410_00) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021741 = {S59417, 146, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; magcomm77_cu_ics2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021961 = {S59418, 134, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JY139 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022005 = {S59419, 134, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; 740RS Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022047 = {S59420, 138, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; EverShine Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022133 = {S59421, 138, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SMP53-210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022404 = {S59422, 133, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022503 = {S59423, 133, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N900 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022624 = {S59424, 129, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAD1001 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023192 = {S59425, 142, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tmn smart a18 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023236 = {S59426, 127, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M1001 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023244 = {S59427, 134, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TM450 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023241 = {S59428, 141, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N8000_USA_RS Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023294 = {S59429, 131, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; P6 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023434 = {S59430, 133, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 7102 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023439 = {S59431, 133, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; I110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023591 = {S59432, 141, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BASE_Lutea_3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023593 = {S59433, 138, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SMP45-210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023585 = {S59434, 139, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ViewPad 7D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023685 = {S59435, 138, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST70104-2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023835 = {S59436, 127, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T9002 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3024814 = {S59437, 126, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M980 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3026074 = {S59438, 132, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A13 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3057754 = {S59439, 134, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GLORY Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3058110 = {S59440, 128, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S7916E Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3059118 = {S59441, 139, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MOBHIG0080 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3078259 = {S59442, 134, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M9100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3092152 = {S59443, 137, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PC7058ME Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3092311 = {S59444, 128, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HX-M02 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3107821 = {S59445, 137, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Infinity Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3108002 = {S59446, 135, { &P15364,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 3G7271 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2674961 = {S59447, 139, { &P18453,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT25i Build/9.1.A.1.140) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714088 = {S59448, 132, { &P18453,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT25i Build/9.1.A.1.140) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA22437 = {S59449, 179, { &P13474,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaX5-01.2/043.016; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.7.4 3gpp-gba*/
const Device UA64309 = {S59450, 177, { &P13474,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaX5-01/043.005; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.3 3gpp-gba*/
const Device UA44140 = {S59451, 146, { &P13790,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; GT-I5800L Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019579 = {S59452, 68, { &P13790,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; GT-I5800L Build/ECLAIR)*/
const Device UA54051 = {S59453, 93, { &P12711,&P5222,&P17183,&P18092 }};	/*Useragent - Motorola_ES405B/191 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5.3.5)*/
const Device UA314000 = {S59454, 96, { &P12711,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Motorola_ES405B_19103; Windows Phone 6.5.3.5)*/
const Device UA375423 = {S59455, 92, { &P12711,&P5222,&P17183,&P18092 }};	/*Useragent - Motorola_ES405B/20 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5.3.5)*/
const Device UA673974 = {S59456, 113, { &P12711,&P5222,&P17183,&P18092 }};	/*Useragent - Motorola_ES405B/27 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Motorola_ES405B_256; Windows Phone 6.5.3.5)*/
const Device UA740791 = {S59457, 94, { &P12711,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Motorola_ES405B_312; Windows Phone 6.5.3.5)*/
const Device UA823050 = {S59458, 92, { &P12711,&P5222,&P17183,&P18092 }};	/*Useragent - Motorola_ES405B/22 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5.3.5)*/
const Device UA2787245 = {S59459, 94, { &P12711,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Motorola_ES405B_256; Windows Phone 6.5.3.5)*/
const Device UA2878387 = {S59460, 113, { &P12711,&P5222,&P17183,&P18092 }};	/*Useragent - Motorola_ES405B/28 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Motorola_ES405B_256; Windows Phone 6.5.3.5)*/
const Device UA2989244 = {S59461, 97, { &P12711,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Motorola_ES405B_44.000; Windows Phone 6.5.3.5)*/
const Device UA2230829 = {S59462, 32, { &P14033,&P5129,&P24672,&P18092 }};	/*Useragent - NokiaC5-03/UCWEB8.6.0.180/50/800*/
const Device UA2239221 = {S59463, 120, { &P14033,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaC5-03) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile*/
const Device UA3037793 = {S59464, 139, { &P15972,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One XL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3083261 = {S59465, 140, { &P15972,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC EVA_UTL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA49342 = {S59466, 73, { &P12995,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-E2600/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA672928 = {S59467, 87, { &P12995,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-E2600_ORANGE/E2600BVKL5 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2602798 = {S59468, 140, { &P21507,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C1605 Build/11.1.A.0.68) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA11498 = {S59469, 150, { &P13613,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; HTC_A3335 V2.38.841.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA26630 = {S59470, 150, { &P13613,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; HTC_A3335 V1.21.841.3 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55514 = {S59471, 150, { &P13613,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; HTC_A3335 V1.30.841.1 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA909272 = {S59472, 138, { &P20265,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; Micromax A52 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1959045 = {S59473, 132, { &P20265,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A52 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2595580 = {S59474, 144, { &P20265,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; Micromax-Xzoom A52 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2758692 = {S59475, 56, { &P20265,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; A52 Build/GRK39F)*/
const Device UA3100227 = {S59476, 151, { &P20265,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; Micromax-Xtreme Droid A52 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1996483 = {S59477, 133, { &P23896,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; S5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2507962 = {S59478, 145, { &P23896,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S5 Build/S5-V1.0-2012.11.30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2561203 = {S59479, 145, { &P23896,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; S5 Build/S5-V1.0-2012.12.01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2780853 = {S59480, 133, { &P23896,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2796385 = {S59481, 145, { &P23896,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S5 Build/S5-V1.0-2013.04.12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2676249 = {S59482, 138, { &P20947,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTCEVOV4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2862326 = {S59483, 131, { &P20947,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTCEVOV4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA5471 = {S59484, 180, { &P12379,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; pt-br; LG Eve) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA5481 = {S59485, 152, { &P12379,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; LG Eve-Orange) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 (LG_UA; AD_LOGON=LGE.NET)*/
const Device UA7180 = {S59486, 215, { &P12379,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; LG Eve-Orange) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA29538 = {S59487, 143, { &P12379,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-ca; LG GW620R Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA64693 = {S59488, 173, { &P12379,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; LG Eve) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2385864 = {S59489, 46, { &P23056,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-UN272/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA2528738 = {S59490, 46, { &P23056,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-AN272/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA2672904 = {S59491, 139, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MOBHIG0080 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2706646 = {S59492, 142, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tmn smart a18 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708232 = {S59493, 133, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 7102 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708376 = {S59494, 129, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAD1001 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708911 = {S59495, 135, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CJ ThL Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712065 = {S59496, 131, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N1 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712267 = {S59497, 141, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BASE_Lutea_3 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712277 = {S59498, 124, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T7 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2712461 = {S59499, 131, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; CTA Build/ICS.casper.X) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2712524 = {S59500, 133, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N900 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712532 = {S59501, 136, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A11Note Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713239 = {S59502, 137, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Infinity Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734820 = {S59503, 138, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Explosion Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734862 = {S59504, 137, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Alpha GT Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735229 = {S59505, 131, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; P6 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735452 = {S59506, 146, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 1688-CJ-C30Q8-31S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735569 = {S59507, 125, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A76 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2736275 = {S59508, 133, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2750859 = {S59509, 127, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Clear Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2750883 = {S59510, 134, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GLORY Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2750966 = {S59511, 145, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; INFINITY_PASSION Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2751729 = {S59512, 140, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A26 Build/CROSSCROSS_A26) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2768511 = {S59513, 128, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M1005D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2780798 = {S59514, 134, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; s4502 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2860081 = {S59515, 135, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; cyclone Titan Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2860733 = {S59516, 128, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S7916E Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2861225 = {S59517, 136, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AMI-TT2 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2861924 = {S59518, 142, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; hexing77_ics2 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2862058 = {S59519, 135, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Strike Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2862815 = {S59520, 132, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Q75 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2864160 = {S59521, 127, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M1005 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2864510 = {S59522, 134, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; X9015 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2993454 = {S59523, 134, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HD II Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3048374 = {S59524, 134, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JY139 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3057734 = {S59525, 133, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GHIA 27154P Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3114560 = {S59526, 138, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V8200plus Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3120070 = {S59527, 140, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PICOpad GEW Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3126181 = {S59528, 136, { &P15364,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; QT-70BW Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2974032 = {S59529, 134, { &P22727,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGP312 Build/10.1.C.0.370) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA1993837 = {S59530, 140, { &P12059,&P5204,&P24668,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia2690) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2355478 = {S59531, 163, { &P18300,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-T769 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2356798 = {S59532, 135, { &P20427,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SHL21 Build/SA132) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2368094 = {S59533, 135, { &P20427,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SHL21 Build/SB070) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2433723 = {S59534, 135, { &P20427,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SHL21 Build/SB300) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2471642 = {S59535, 135, { &P20427,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SHL21 Build/SC260) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2538048 = {S59536, 135, { &P20427,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SHL21 Build/S2140) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3100541 = {S59537, 57, { &P20427,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SHL21 Build/S2140)*/
const Device UA2674665 = {S59538, 129, { &P17595,&P17295,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/24.0.648.205 Safari/534.16*/
const Device UA2715048 = {S59539, 117, { &P17595,&P17295,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/24.0.697.0 Safari/534.24*/
const Device UA2736681 = {S59540, 129, { &P17595,&P17295,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/24.0.648.205 Safari/534.16*/
const Device UA64508 = {S59541, 87, { &P13344,&P17485,&P3499,&P18092 }};	/*Useragent - SonyEricssonW100i/R1CA009 TelecaBrowser/Q07C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA368715 = {S59542, 161, { &P12871,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Wildfire S Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627448 = {S59543, 157, { &P12871,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Wildfire S Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1958149 = {S59544, 154, { &P12871,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Wildfire S Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029917 = {S59545, 160, { &P12871,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Wildfire S A510e Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2389073 = {S59546, 153, { &P15364,&P7133,&P19966,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.4.2.X (Second Life Release; default skin) Safari/533.3*/
const Device UA2754502 = {S59547, 133, { &P19907,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C5155 Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755264 = {S59548, 133, { &P19907,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Rise Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA127471 = {S59549, 60, { &P18088,&P17017,&P17367,&P18092 }};	/*Useragent - Opera/8.01 (J2ME/MIDP; Opera Mini/3.0.6306/1724; en; U; ssr)*/
const Device UA468058 = {S59550, 79, { &P15364,&P6704,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win 9x 4.90; nl; rv:1.8.1.20) Gecko/X Firefox/2.0.0.20*/
const Device UA719775 = {S59551, 74, { &P15364,&P6704,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.8.1.3) Gecko/X Firefox/2.0.0.3*/
const Device UA924782 = {S59552, 75, { &P15364,&P5690,&P22316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:14.0) Gecko/X Firefox/14.0.1 SeaMonkey/2.11*/
const Device UA1995198 = {S59553, 60, { &P15364,&P5690,&P22316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:14.0) Gecko/X SeaMonkey/2.11*/
const Device UA1995366 = {S59554, 82, { &P15364,&P5690,&P22316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:14.0) Gecko/X Firefox/14.0.1 SeaMonkey/2.11*/
const Device UA217091 = {S59555, 134, { &P18079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SH-13C Build/S7160) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA242585 = {S59556, 134, { &P18079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SH-13C Build/S9090) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA381698 = {S59557, 134, { &P18079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SH-13C Build/SC190) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA901173 = {S59558, 134, { &P18079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SH-13C Build/S4090) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030927 = {S59559, 134, { &P18079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SH-13C Build/S8100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032267 = {S59560, 58, { &P18079,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SH-13C Build/S4090)*/
const Device UA1995020 = {S59561, 135, { &P14111,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Arc S Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2381690 = {S59562, 127, { &P14111,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Arc Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2402445 = {S59563, 140, { &P14111,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT15a Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2510416 = {S59564, 93, { &P12088,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaX6-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2656451 = {S59565, 32, { &P12088,&P5129,&P24675,&P18092 }};	/*Useragent - NokiaX6-00/UCWEB8.8.0.212/50/800*/
const Device UA2529304 = {S59566, 150, { &P13452,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P1000 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622220 = {S59567, 151, { &P13452,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-us; GT-P1000N Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2624052 = {S59568, 157, { &P13452,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P1000 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2630795 = {S59569, 150, { &P13452,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P1000 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2633414 = {S59570, 156, { &P13452,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; GT-P1000 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2633520 = {S59571, 149, { &P13452,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P1000 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2740083 = {S59572, 150, { &P13452,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P1000 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA912800 = {S59573, 101, { &P15364,&P7127,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA2528550 = {S59574, 118, { &P15364,&P7127,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 8.0; Windows NT 5.0) Chrome/20.0.1229.79*/
const Device UA2528880 = {S59575, 118, { &P15364,&P7127,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 7.0; Windows NT 5.0) Chrome/20.0.1229.79*/
const Device UA2737758 = {S59576, 123, { &P15364,&P7127,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 9.0; Windows NT 5.0WOW64) Chrome/20.0.1229.79*/
const Device UA2431426 = {S59577, 229, { &P15991,&P5629,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2354998 = {S59578, 145, { &P20439,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySOL21 Build/9.0.F.0.195) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2391344 = {S59579, 145, { &P20439,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySOL21 Build/9.0.F.0.226) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2404012 = {S59580, 141, { &P20439,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SOL21 Build/9.0.F.0.195) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2430474 = {S59581, 140, { &P20439,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SOL21 Build/9.0.F.2.71) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2436146 = {S59582, 144, { &P20439,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySOL21 Build/9.0.F.2.71) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2490328 = {S59583, 145, { &P20439,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySOL21 Build/9.0.F.2.103) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2536062 = {S59584, 141, { &P20439,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SOL21 Build/9.0.F.2.103) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2564035 = {S59585, 145, { &P20439,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonySOL21 Build/9.0.F.2.128) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2582753 = {S59586, 141, { &P20439,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SOL21 Build/9.0.F.2.128) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654507 = {S59587, 141, { &P20439,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SOL21 Build/9.0.F.0.226) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2676346 = {S59588, 145, { &P13064,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734937 = {S59589, 145, { &P13064,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I727 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA114105 = {S59590, 47, { &P15364,&P6704,&P4618,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.01; Windows 98)*/
const Device UA2658813 = {S59591, 60, { &P15364,&P6704,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; Win 9x 4.90; Trident/1.0)*/
const Device UA2792648 = {S59592, 54, { &P15364,&P6704,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; Win98; Trident/2.0)*/
const Device UA459024 = {S59593, 135, { &P18014,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID7014 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA459046 = {S59594, 69, { &P18014,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.4 (Linux; U; Android 2.3.3; en_US; MID7014 Build/MID7014 X)*/
const Device UA626013 = {S59595, 133, { &P20946,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; P-02D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA757895 = {S59596, 57, { &P20946,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; P-02D Build/GRJ90)*/
const Device UA2486435 = {S59597, 75, { &P17595,&P18207,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2) Presto/2.12.388 Version/12.13*/
const Device UA2799175 = {S59598, 137, { &P20937,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ADR910L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA20577 = {S59599, 45, { &P14455,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-606A/1.0 ObigoInternetBrowser/Q03C*/
const Device UA20590 = {S59600, 85, { &P14455,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-606A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA52441 = {S59601, 91, { &P14455,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-606/010_05 1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA62847 = {S59602, 84, { &P14455,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-606/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA64337 = {S59603, 97, { &P14455,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-606/1.0 model-orange Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA175465 = {S59604, 139, { &P17149,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SCH-I510 4G Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA182655 = {S59605, 136, { &P17149,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SCH-I510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA197640 = {S59606, 60, { &P17149,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SCH-I510 Build/FROYO)*/
const Device UA252484 = {S59607, 122, { &P15364,&P5690,&P22413,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.2 (KHTML, like Gecko) Comodo_Dragon/15.0.3.0 Chrome/15.0.874.102 Safari/535.2*/
const Device UA1905986 = {S59608, 129, { &P15364,&P5690,&P22413,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Comodo_Dragon/15.0.3.0 Chrome/15.0.874.102 Safari/535.2*/
const Device UA895681 = {S59609, 136, { &P23752,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; CT705 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA896034 = {S59610, 138, { &P23752,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; CT705FR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2419637 = {S59611, 137, { &P20732,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; YP-GP1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3070796 = {S59612, 59, { &P20732,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; YP-GP1 Build/IMM76D)*/
const Device UA3067601 = {S59613, 141, { &P18382,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire C Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081351 = {S59614, 141, { &P18382,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2505406 = {S59615, 155, { &P20018,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; ALCATEL ONE TOUCH 993D Build/ICECREAM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2527853 = {S59616, 154, { &P20018,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ALCATEL ONE TOUCH 993 Build/ICECREAM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2979786 = {S59617, 77, { &P20018,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ALCATEL ONE TOUCH 993D Build/ICECREAM)*/
const Device UA2674521 = {S59618, 129, { &P17595,&P17295,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/26.0.648.205 Safari/534.16*/
const Device UA2715044 = {S59619, 117, { &P17595,&P17295,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.27 (KHTML, like Gecko) Chrome/26.0.712.0 Safari/534.27*/
const Device UA2716002 = {S59620, 129, { &P17595,&P17295,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/26.0.648.205 Safari/534.16*/
const Device UA2231508 = {S59621, 169, { &P15364,&P19822,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/537.6+ Ubuntu/12.10 (3.6.0-0ubuntu1) Epiphany/3.6.0*/
const Device UA2260059 = {S59622, 167, { &P15364,&P19822,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/537.6+ Ubuntu/12.10 (3.6.0-0ubuntu1) Epiphany/3.6.0*/
const Device UA2386611 = {S59623, 169, { &P15364,&P19822,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/537.6+ Ubuntu/12.10 (3.6.1-0ubuntu1) Epiphany/3.6.1*/
const Device UA2386896 = {S59624, 167, { &P15364,&P19822,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/537.6+ Ubuntu/12.10 (3.6.1-0ubuntu1) Epiphany/3.6.1*/
const Device UA49802 = {S59625, 46, { &P13085,&P6476,&P3625,&P18092 }};	/*Useragent - LGE-VN150/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA90808 = {S59626, 113, { &P15767,&P17146,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Twitter for iPad*/
const Device UA90875 = {S59627, 115, { &P15767,&P17146,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Twitter for iPad*/
const Device UA171378 = {S59628, 134, { &P15364,&P5690,&P22492,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.3.4641 Chrome/7.0.517.450 Safari/534.7*/
const Device UA1866600 = {S59629, 134, { &P15364,&P5690,&P22492,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.3.4641 Chrome/7.0.517.450 Safari/534.7*/
const Device UA608220 = {S59630, 154, { &P16356,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-N7000 Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA866368 = {S59631, 154, { &P16356,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-N7000 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2200754 = {S59632, 158, { &P16356,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-N7000 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2333683 = {S59633, 158, { &P16356,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-N7000 Build/IML74K; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2982736 = {S59634, 159, { &P16356,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SHV-E160S Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1970687 = {S59635, 132, { &P16044,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Touchpad Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1995256 = {S59636, 140, { &P16044,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TOUCHPAD 7 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2029649 = {S59637, 144, { &P16044,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; CnM-TOUCHPAD 7 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2029681 = {S59638, 144, { &P16044,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CnM-TOUCHPAD 9 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2032564 = {S59639, 139, { &P16044,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Touchpad7 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261457 = {S59640, 143, { &P16044,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; CnM-TOUCHPAD7 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2413519 = {S59641, 143, { &P16044,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS-TOUCHPAD 9 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2426605 = {S59642, 144, { &P16044,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CnM TOUCHPAD 9 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2452353 = {S59643, 143, { &P16044,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CnM-TOUCHPAD9 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2481358 = {S59644, 140, { &P16044,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CnM Touchpad 9.7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2483170 = {S59645, 140, { &P16044,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TOUCHPAD 7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2486495 = {S59646, 147, { &P16044,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Versus TouchPad 7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2556008 = {S59647, 144, { &P16044,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CnM TouchPad 7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2653516 = {S59648, 134, { &P16044,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Touchpad10 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3043871 = {S59649, 144, { &P16044,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CnM-TOUCHPAD 7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2257538 = {S59650, 137, { &P18600,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SC-03D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2481207 = {S59651, 135, { &P18600,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-; SC-03D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2842040 = {S59652, 59, { &P18600,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SC-03D Build/IMM76D)*/
const Device UA2974452 = {S59653, 140, { &P21602,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5302 Build/12.0.A.1.257) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2492413 = {S59654, 102, { &P15364,&P18346,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2676012 = {S59655, 141, { &P18462,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire X Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA72080 = {S59656, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; en-us; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA72185 = {S59657, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; zh-tw; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA72216 = {S59658, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; en-gb; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA72423 = {S59659, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; de-ch; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA72585 = {S59660, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; en-ca; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA72668 = {S59661, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; zh-cn; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA72774 = {S59662, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; sv-se; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA72896 = {S59663, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; nl-nl; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA73061 = {S59664, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; nb-no; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA73064 = {S59665, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; fr-fr; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA73088 = {S59666, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; de-de; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA73577 = {S59667, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; fr-ch; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA73656 = {S59668, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; fr-ca; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA74201 = {S59669, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; pt-pt; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA74334 = {S59670, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; cs-cz; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA74841 = {S59671, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; da-dk; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA77607 = {S59672, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; fi-fi; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA196743 = {S59673, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; zh-tw; Transformer TF101 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA236444 = {S59674, 140, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; en-us; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA247891 = {S59675, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; es-es; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA250307 = {S59676, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; it-it; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA316457 = {S59677, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; en-us; Transformer TF101 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA318568 = {S59678, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; en-us; Transformer TF101 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA329621 = {S59679, 140, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; en-gb; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA331541 = {S59680, 140, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; nl-nl; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA331897 = {S59681, 140, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; fr-fr; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA395718 = {S59682, 140, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; zh-tw; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA550174 = {S59683, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; nl-nl; Transformer TF101 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA563342 = {S59684, 140, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; en-ca; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA574336 = {S59685, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; pl-pl; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA574834 = {S59686, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; pt-br; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA629983 = {S59687, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; en-gb; Transformer TF101 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA737777 = {S59688, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; ru-ru; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA766054 = {S59689, 140, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; it-it; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA821211 = {S59690, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; en-ca; Transformer TF101 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA829144 = {S59691, 140, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; de-de; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA858549 = {S59692, 140, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; sv-se; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA875867 = {S59693, 140, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; es-es; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA1887485 = {S59694, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; de-de; Transformer TF101 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA1905464 = {S59695, 139, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; nl-be; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA2141908 = {S59696, 140, { &P17172,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; zh-cn; Transformer TF101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA448971 = {S59697, 150, { &P17974,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; es-us Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA542963 = {S59698, 148, { &P17974,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; en- Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA549775 = {S59699, 150, { &P17974,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; es-es Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA706856 = {S59700, 150, { &P17974,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; pt-br Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA716247 = {S59701, 150, { &P17974,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; nl-nl Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA747008 = {S59702, 150, { &P17974,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; en-in Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA765587 = {S59703, 150, { &P17974,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; en-gb Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018415 = {S59704, 150, { &P17974,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; vi-vn Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018725 = {S59705, 150, { &P17974,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; en-sg Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019136 = {S59706, 150, { &P17974,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; en-nz Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019847 = {S59707, 150, { &P17974,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; ca-es Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019904 = {S59708, 150, { &P17974,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; bg-bg Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2023512 = {S59709, 72, { &P17974,&P7305,&P3553,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android Eclair Build/pandigitalopc1/sourceidDLX)*/
const Device UA2493691 = {S59710, 150, { &P17974,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; de-de Build/pandigitalopc1/sourceidDLX) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2026049 = {S59711, 127, { &P20478,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; JY-G2 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2364526 = {S59712, 136, { &P20478,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; JY-G2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2421066 = {S59713, 136, { &P20478,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; JY-G2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2464188 = {S59714, 127, { &P20478,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; JY-G2 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2879924 = {S59715, 152, { &P18506,&P5043,&P24678,&P18092 }};	/*Useragent - Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia305) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3028188 = {S59716, 113, { &P13107,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-06/07.48; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA195135 = {S59717, 133, { &P14176,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; IS04 Build/FFK300) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623359 = {S59718, 133, { &P14176,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; IS04 Build/FFK301) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA702184 = {S59719, 57, { &P14176,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; IS04 Build/FFK300)*/
const Device UA2769750 = {S59720, 58, { &P15364,&P17188,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0; rv:22.0) Gecko/X Firefox/22.0*/
const Device UA55264 = {S59721, 101, { &P13378,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-J700/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA63780 = {S59722, 114, { &P13378,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-J700-ORANGE/J700ABHB2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA183879 = {S59723, 63, { &P15364,&P22857,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (MTK; Nucleus; U; en-US) Presto/2.4.18 Version/10.00*/
const Device UA430112 = {S59724, 81, { &P15364,&P22857,&P4812,&P18092 }};	/*Useragent - Fly_E190 Wi-Fi    Opera/9.80 (MTK; Nucleus; U; ru-RU) Presto/2.4.18 Version/10.00*/
const Device UA2474417 = {S59725, 73, { &P15364,&P22857,&P4812,&P18092 }};	/*Useragent - Fly E185  Opera/9.80 (MTK; Nucleus; U; ru-RU) Presto/2.4.18 Version/10.00*/
const Device UA2573045 = {S59726, 72, { &P15364,&P22857,&P4812,&P18092 }};	/*Useragent - MVL G99  Opera/9.80 (MTK; Nucleus; U; en-US) Presto/2.4.18 Version/10.00*/
const Device UA213669 = {S59727, 135, { &P17956,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID7016 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458688 = {S59728, 59, { &P17956,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID7016 Build/GRI40)*/
const Device UA458884 = {S59729, 69, { &P17956,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.3 (Linux; U; Android 2.3.3; en_US; MID7016 Build/MID7016 X)*/
const Device UA461983 = {S59730, 92, { &P15364,&P17815,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; it-IT; rv:1.9.0.2) Gecko/X Ubuntu/9.04 (jaunty) Firefox/3.5*/
const Device UA862160 = {S59731, 89, { &P15364,&P17815,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11;U; Linux i686; en-GB; rv:1.9.1) Gecko/X Ubuntu/9.04 (jaunty) Firefox/3.5*/
const Device UA62022 = {S59732, 73, { &P12301,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S3100/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA1996836 = {S59733, 102, { &P14777,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-SGH-A817/A817UCLD2 SHP/VPP/R5 Dolfin/1.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA680524 = {S59734, 81, { &P15492,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-C3322/C3322DDKK1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA2730437 = {S59735, 139, { &P23159,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-S960L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2773594 = {S59736, 139, { &P23240,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; USCC-C6721 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2796239 = {S59737, 132, { &P23240,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; USCC-C6721 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2348865 = {S59738, 182, { &P14711,&P5204,&P24672,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile*/
const Device UA2537751 = {S59739, 142, { &P18504,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LS840 Build/ZVK.IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2385276 = {S59740, 159, { &P12798,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Arc S Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768900 = {S59741, 133, { &P22920,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; WL-101GQC Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2958595 = {S59742, 62, { &P22920,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; WL-101GQC Build/JZO54K)*/
const Device UA2188213 = {S59743, 139, { &P19902,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530M Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2666004 = {S59744, 137, { &P22613,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E430 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA13448 = {S59745, 104, { &P14607,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-A867/A867UCHJ3 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA31250 = {S59746, 104, { &P12411,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-A697/A697UCJB2 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA3067060 = {S59747, 138, { &P20871,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2566976 = {S59748, 138, { &P13991,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6400L Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583085 = {S59749, 141, { &P13991,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ThunderBolt Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2658692 = {S59750, 138, { &P22609,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6312 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2737046 = {S59751, 153, { &P19770,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020A Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2737579 = {S59752, 153, { &P19770,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020D Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2752549 = {S59753, 153, { &P19770,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020E Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2752814 = {S59754, 153, { &P19770,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020X Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2964012 = {S59755, 153, { &P19770,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020W Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2753871 = {S59756, 133, { &P18665,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N861 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2975936 = {S59757, 139, { &P18447,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT30p Build/7.0.A.1.303) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA402956 = {S59758, 142, { &P20558,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; XOOM 2 ME Build/1.6.0_268.1-MZ608) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA403044 = {S59759, 142, { &P20558,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 ME Build/1.6.0_268.4-MZ608) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA403091 = {S59760, 142, { &P20558,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 ME Build/1.6.0_258.1-MZ608) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA403281 = {S59761, 142, { &P20558,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; sv-se; XOOM 2 ME Build/1.6.0_281.4-MZ608) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA721137 = {S59762, 139, { &P20558,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; pt-br; MZ608 Build/1.6.0M_272.4_MZ608) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA797748 = {S59763, 140, { &P20558,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; pt-br; MZ608 Build/1.6.0M_272.12_MZ608) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA870799 = {S59764, 140, { &P20558,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; pt-br; MZ608 Build/1.6.0M_272.18_MZ608) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2027215 = {S59765, 142, { &P20558,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; pt-br; MZ608 Build/1.6.0M_272.12_MZ608) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2032840 = {S59766, 142, { &P20558,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; de-de; XOOM 2 ME Build/1.6.0_268.6-MZ608) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA3101684 = {S59767, 140, { &P20558,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; pt-br; MZ608 Build/1.6.0M_272.24_MZ608) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA614148 = {S59768, 81, { &P15364,&P7133,&P21171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.0) Gecko/X Firefox/12.0 SeaMonkey/2.9.1*/
const Device UA676574 = {S59769, 68, { &P15364,&P7133,&P21171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.0) Gecko/X SeaMonkey/2.9.1*/
const Device UA786790 = {S59770, 139, { &P19930,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; U8600 Build/HuaweiU8600) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA823115 = {S59771, 74, { &P15364,&P7133,&P18191,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:10.0) Gecko/X Thunderbird/10.0*/
const Device UA1991034 = {S59772, 140, { &P22817,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DEM752HCF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2489066 = {S59773, 171, { &P15991,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3020106 = {S59774, 175, { &P15991,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6102 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA95751 = {S59775, 130, { &P15364,&P7133,&P22443,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.9 (KHTML, like Gecko) Iron/4.0.280.0 Chrome/4.0.280.0 Safari/532.9*/
const Device UA2042759 = {S59776, 134, { &P17595,&P17296,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1*/
const Device UA2840195 = {S59777, 134, { &P17595,&P17296,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1*/
const Device UA2666492 = {S59778, 145, { &P15364,&P17979,&P18112,&P18092 }};	/*Useragent - MT6517-MT6517-H369X/1.0 Linux/Apps_4.0.32.9-perf Android/4.1.1 Release/01.16.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2708103 = {S59779, 145, { &P15364,&P17979,&P18112,&P18092 }};	/*Useragent - MT6517-MT6517-H369X/1.0 Linux/Apps_4.0.32.9-perf Android/4.1.1 Release/03.20.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2765635 = {S59780, 95, { &P15364,&P17979,&P18112,&P18092 }};	/*Useragent - CoolPad8730_CMCC_TD/1.0 Linux/3.1.10 Android/4.1.1 Release/12.28.2012 Browser/AppleWebkit534.31*/
const Device UA2767328 = {S59781, 137, { &P15364,&P17979,&P18112,&P18092 }};	/*Useragent - MT-GT-T7100/1.0 Linux/Apps_2.6.32.9-perf Android/4.1.1 Release/03.21.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2879185 = {S59782, 124, { &P15364,&P17979,&P18112,&P18092 }};	/*Useragent - sprd-NOTE2/1.0 Linux/2.6.35.7 Android/4.1 Release/03.26.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2892776 = {S59783, 136, { &P15364,&P17979,&P18112,&P18092 }};	/*Useragent - E9000 Linux/2.6.35 Android/4.1.1 Release/03.19.2013 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1*/
const Device UA2937338 = {S59784, 134, { &P15364,&P17979,&P18112,&P18092 }};	/*Useragent - sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/4.1 Release/05.03.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2937593 = {S59785, 134, { &P15364,&P17979,&P18112,&P18092 }};	/*Useragent - sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/4.1 Release/03.28.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2951973 = {S59786, 126, { &P15364,&P17979,&P18112,&P18092 }};	/*Useragent - sprd-HT-G2/1.0 Linux/2.6.35.7 Android/4.1.3 Release/11.28.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2955050 = {S59787, 82, { &P15364,&P17979,&P18112,&P18092 }};	/*Useragent - ZTEU956_TD/1.0 Linux/3.4.0 Android/4.1 Release/1.18.2013 Browser/AppleWebKit534.30*/
const Device UA2965682 = {S59788, 134, { &P15364,&P17979,&P18112,&P18092 }};	/*Useragent - sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/4.1 Release/05.09.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2995903 = {S59789, 114, { &P15364,&P17979,&P18112,&P18092 }};	/*Useragent - HS-T958_TD/1.0 Android/4.1.2 Release/15.03.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1;*/
const Device UA2997923 = {S59790, 124, { &P15364,&P17979,&P18112,&P18092 }};	/*Useragent - sprd-NOTE2/1.0 Linux/2.6.35.7 Android/4.1 Release/03.21.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2999140 = {S59791, 134, { &P15364,&P17979,&P18112,&P18092 }};	/*Useragent - sprd-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android/4.1 Release/04.10.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2999464 = {S59792, 125, { &P15364,&P17979,&P18112,&P18092 }};	/*Useragent - sprd-SKK-A7/1.0 Linux/2.6.35.7 Android/4.1 Release/05.06.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3002480 = {S59793, 126, { &P15364,&P17979,&P18112,&P18092 }};	/*Useragent - sprd-NAIDE/1.0 Linux/2.6.35.7 Android/4.1.0 Release/05.06.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3004365 = {S59794, 126, { &P15364,&P17979,&P18112,&P18092 }};	/*Useragent - sprd-NOTE2/1.0 Linux/2.6.35.7 Android/4.1.1 Release/05.14.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3048900 = {S59795, 128, { &P15364,&P17979,&P18112,&P18092 }};	/*Useragent - samsung-SKK-A3/1.0 Linux/2.6.35.7 Android/4.1 Release/04.16.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3050621 = {S59796, 81, { &P15364,&P17979,&P18112,&P18092 }};	/*Useragent - ZTEU819_TD/1.0 Linux/3.4.0 Android/4.1 Release/2.8.2013 Browser/AppleWebKit534.30*/
const Device UA3089444 = {S59797, 92, { &P15364,&P17979,&P18112,&P18092 }};	/*Useragent - Coolpad8185_CMCC_TD/1.0 Linux/3.4.5 Android/4.1.2 Release/3.15.2013 Browser/AppleWebkit533.1*/
const Device UA3120433 = {S59798, 127, { &P15364,&P17979,&P18112,&P18092 }};	/*Useragent - sprd-haowanda/1.0 Linux/2.6.35.7 Android/4.1 Release/01.15.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA8479 = {S59799, 175, { &P13443,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Samsung/I8910-ORANGE/BVORAIE4; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525*/
const Device UA8478 = {S59800, 175, { &P13443,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Samsung/I8910-ORANGE/BVIDEIE2; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525*/
const Device UA9820 = {S59801, 157, { &P13443,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 Samsung/I8910;; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/*/
const Device UA9822 = {S59802, 175, { &P13443,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Samsung/I8910-ORANGE/BVORAIF3; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525*/
const Device UA9823 = {S59803, 175, { &P13443,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Samsung/I8910-ORANGE/BVORAII1; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525*/
const Device UA9824 = {S59804, 175, { &P13443,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Samsung/I8910-ORANGE/BVORAIE2; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525*/
const Device UA64705 = {S59805, 159, { &P13443,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Samsung/I8910; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525*/
const Device UA2885369 = {S59806, 160, { &P16044,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; TouchPad Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1/tenderloin*/
const Device UA14362 = {S59807, 182, { &P13775,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB520 Build/Blur_Version.3.5.470.MB520.ATT.en.US Flex/P047) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA14363 = {S59808, 182, { &P13775,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB520 Build/Blur_Version.3.4.120.MB520.ATT.en.US Flex/P031) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA40788 = {S59809, 182, { &P13775,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB520 Build/Blur_Version.3.2.810.MB520.ATT.en.US Flex/P032) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA44344 = {S59810, 150, { &P13775,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB520 Build/KOBE_U3_5.35.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA44345 = {S59811, 182, { &P13775,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB520 Build/Blur_Version.3.5.350.MB520.ATT.en.US Flex/P044) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA56649 = {S59812, 182, { &P13775,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB520 Build/Blur_Version.3.5.340.MB520.ATT.en.US Flex/P044) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA90423 = {S59813, 156, { &P13775,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB520 Build/KOBE_U3_2.144.0) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA370263 = {S59814, 150, { &P13775,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB520 Build/KOBE_U3_5.47.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA442278 = {S59815, 67, { &P15364,&P17285,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:2.0b5) Gecko/X Firefox/4.0b5*/
const Device UA531076 = {S59816, 74, { &P15364,&P17285,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:2.0.1) Gecko/X Firefox/4.0.1 Firefox/4.0.1*/
const Device UA558278 = {S59817, 80, { &P15364,&P17285,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:2.0b12pre) Gecko/X Firefox/4.0b12pre*/
const Device UA889981 = {S59818, 90, { &P15364,&P17285,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 6.0; rv:2.0b3) Gecko/X Firefox/4.0b3 (.NET CLR 3.5.30729)*/
const Device UA1940687 = {S59819, 133, { &P15160,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; Light Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1992354 = {S59820, 133, { &P15160,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Light Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2567245 = {S59821, 142, { &P15802,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC_Amaze_4G Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567261 = {S59822, 142, { &P15802,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Amaze 4G Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2585176 = {S59823, 138, { &P15802,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Ruby Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2673944 = {S59824, 162, { &P13157,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675567 = {S59825, 168, { &P13157,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676008 = {S59826, 168, { &P13157,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707087 = {S59827, 168, { &P13157,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715a Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2737298 = {S59828, 148, { &P13157,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Z715e Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2893805 = {S59829, 73, { &P15364,&P17285,&P24329,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:22.0) Gecko/X Firefox/22.0 SeaMonkey/2.19*/
const Device UA2489180 = {S59830, 152, { &P13644,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; EVO Build/IMM76D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2513076 = {S59831, 152, { &P13644,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; EVO Build/JRO03C; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2536301 = {S59832, 152, { &P13644,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; EVO Build/JRO03C; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2868462 = {S59833, 156, { &P13644,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; PC36100 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA50392 = {S59834, 146, { &P13250,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; Opus One Build/RBE.00.00) AppleWebKit/528.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile Safari/525.20.1*/
const Device UA1960570 = {S59835, 168, { &P21659,&P22857,&P18373,&P18092 }};	/*Useragent - Huawei/1.0/HUAWEI-G7300   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA1983989 = {S59836, 168, { &P21659,&P22857,&P18373,&P18092 }};	/*Useragent - Huawei/1.0/HUAWEI-G7300   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; es-SA) Presto/2.5.28 Version/10.10*/
const Device UA2161860 = {S59837, 165, { &P21659,&P22857,&P18373,&P18092 }};	/*Useragent - Huawei/1.0/HUAWEI-G7300   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; es) Presto/2.5.28 Version/10.10*/
const Device UA2371964 = {S59838, 168, { &P21659,&P22857,&P18373,&P18092 }};	/*Useragent - Huawei/1.0/HUAWEI-G7300   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; vi-VN) Presto/2.5.28 Version/10.10*/
const Device UA2377516 = {S59839, 168, { &P21659,&P22857,&P18373,&P18092 }};	/*Useragent - Huawei/1.0/HUAWEI-G7300   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; it-IT) Presto/2.5.28 Version/10.10*/
const Device UA2414103 = {S59840, 168, { &P21659,&P22857,&P18373,&P18092 }};	/*Useragent - Huawei/1.0/HUAWEI-G7300   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; pt-PT) Presto/2.5.28 Version/10.10*/
const Device UA2463447 = {S59841, 168, { &P21659,&P22857,&P18373,&P18092 }};	/*Useragent - Huawei/1.0/HUAWEI-G7300   Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; lt-LT) Presto/2.5.28 Version/10.10*/
const Device UA2950176 = {S59842, 166, { &P21659,&P22857,&P18373,&P18092 }};	/*Useragent - Huawei/1.0/HUAWEI-G7300 Browser/Opera MMS/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; es-SA) Presto/2.5.28 Version/10.10*/
const Device UA79463 = {S59843, 144, { &P15805,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-kr; HTC_Raider_X710e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369703 = {S59844, 144, { &P15805,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Raider X710e Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623275 = {S59845, 139, { &P15805,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; HTC_Holiday Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA757859 = {S59846, 144, { &P15805,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-kr; HTC_Raider_X710e Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021735 = {S59847, 132, { &P15805,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Raider X710e Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2025212 = {S59848, 141, { &P15805,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Raider 4G Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033586 = {S59849, 68, { &P15805,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Raider X710e Build/GWK74)*/
const Device UA2806739 = {S59850, 144, { &P15805,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-kr; HTC Raider X710e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2390417 = {S59851, 139, { &P18701,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-T100K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695567 = {S59852, 137, { &P18701,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; IM-T100K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2729864 = {S59853, 137, { &P18701,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-T100K Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2951232 = {S59854, 61, { &P18701,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; IM-T100K Build/IMM76D)*/
const Device UA3062321 = {S59855, 144, { &P18701,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-T100K/2.21 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1993220 = {S59856, 28, { &P11860,&P5776,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0,Nokia6681/2.0*/
const Device UA2676765 = {S59857, 130, { &P22606,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5110 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA60948 = {S59858, 136, { &P12600,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SCH-i559 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA187907 = {S59859, 136, { &P12600,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; SCH-I559 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA542398 = {S59860, 60, { &P12600,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SCH-i559 Build/FROYO)*/
const Device UA694095 = {S59861, 136, { &P12600,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; SCH-I559 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1933443 = {S59862, 124, { &P12600,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SCH-i559 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2030354 = {S59863, 129, { &P12600,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SCH-i559 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2423025 = {S59864, 103, { &P14888,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.4.0.560; en-us; 9650/4.4.0.560) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2528170 = {S59865, 103, { &P14888,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.4.0.438; en-us; 9650/4.4.0.438) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2555798 = {S59866, 103, { &P14888,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.4.0.315; en-us; 9650/4.4.0.315) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2597065 = {S59867, 103, { &P14888,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.4.0.314; en-us; 9650/4.4.0.314) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2652837 = {S59868, 103, { &P14888,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.4.0.486; en-us; 9650/4.4.0.486) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2653117 = {S59869, 103, { &P14888,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.318; en-us; 9650/4.2.0.318) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2695350 = {S59870, 103, { &P14888,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.4.0.147; en-us; 9650/4.4.0.147) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2790594 = {S59871, 103, { &P14888,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.437; en-us; 9650/4.2.0.437) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2847234 = {S59872, 103, { &P14888,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.4.0.348; en-us; 9650/4.4.0.348) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2849488 = {S59873, 103, { &P14888,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.4.0.234; en-us; 9650/4.4.0.234) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2890536 = {S59874, 103, { &P14888,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.4.0.185; en-us; 9650/4.4.0.185) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2934177 = {S59875, 103, { &P14888,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.424; en-us; 9650/4.2.0.424) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2984047 = {S59876, 103, { &P14888,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.4.0.562; en-us; 9650/4.4.0.562) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3043453 = {S59877, 103, { &P14888,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.334; en-us; 9650/4.2.0.334) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3099670 = {S59878, 103, { &P14888,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.4.0.376; en-us; 9650/4.4.0.376) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2430152 = {S59879, 170, { &P18556,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Barnes & Noble Nook HD+ Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580630 = {S59880, 163, { &P18556,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Barnes & Noble Nook HD+ Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA908786 = {S59881, 137, { &P18320,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PadFone Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA911457 = {S59882, 131, { &P18320,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PadFone Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1966003 = {S59883, 131, { &P18320,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PadFone Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1995655 = {S59884, 137, { &P18320,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PadFone Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2436564 = {S59885, 143, { &P19842,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Butterfly Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2561896 = {S59886, 133, { &P18465,&P18110,&P21418,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Safari/535.19 Silk-Accelerated=false*/
const Device UA902541 = {S59887, 155, { &P18016,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ThinkPad Tablet Build/ThinkPadTablet_A400_03) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2564108 = {S59888, 142, { &P23813,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NABIJR-NV5A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2676595 = {S59889, 145, { &P19891,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735873 = {S59890, 145, { &P19891,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I717 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2750989 = {S59891, 138, { &P19891,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2489080 = {S59892, 113, { &P15767,&P19826,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Flipboard/1.9.8*/
const Device UA2532651 = {S59893, 115, { &P15767,&P19826,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Flipboard/1.9.8*/
const Device UA2533477 = {S59894, 115, { &P15767,&P19826,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B147 Flipboard/1.9.8*/
const Device UA2585464 = {S59895, 115, { &P15767,&P19826,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/1.9.8*/
const Device UA2490908 = {S59896, 114, { &P15767,&P19826,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Twitter for iPad*/
const Device UA2533141 = {S59897, 116, { &P15767,&P19826,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Twitter for iPad*/
const Device UA2607011 = {S59898, 116, { &P15767,&P19826,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Twitter for iPad*/
const Device UA2755335 = {S59899, 126, { &P18282,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A200 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA29447 = {S59900, 66, { &P12258,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2560825 = {S59901, 132, { &P21402,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; Lenovo-S890/S100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2701909 = {S59902, 142, { &P21402,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Lenovo S890 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2729477 = {S59903, 64, { &P21402,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Lenovo S890 Build/JRO03C)*/
const Device UA3074101 = {S59904, 142, { &P21402,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Lenovo S890 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1974184 = {S59905, 137, { &P18739,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOR801 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1990741 = {S59906, 134, { &P18739,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-TH; R8015 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025237 = {S59907, 121, { &P18739,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; R801 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2257139 = {S59908, 126, { &P18739,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; R801 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2372288 = {S59909, 138, { &P18739,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; OPPOR8015 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2376063 = {S59910, 57, { &P18739,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; R801 Build/GRK39F)*/
const Device UA2786659 = {S59911, 58, { &P18739,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; R8015 Build/GRK39F)*/
const Device UA2676172 = {S59912, 149, { &P18318,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700T Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676353 = {S59913, 150, { &P18318,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700KL Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2864599 = {S59914, 149, { &P18318,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ASUS Transformer Pad TF700T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3065584 = {S59915, 141, { &P20399,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PantechP8010 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2947153 = {S59916, 105, { &P11684,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5800 XpressMusic) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3570 = {S59917, 142, { &P13278,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; pt-br; E10a Build/1.2.A.1.174) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA3571 = {S59918, 154, { &P13278,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; es-mx; SonyEricssonE10a Build/1.2.A.1.181) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA3573 = {S59919, 140, { &P13278,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; es-mx; E10a Build/1.1.A.0.8) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA3581 = {S59920, 142, { &P13278,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; es-mx; E10a Build/1.2.A.1.181) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA3587 = {S59921, 140, { &P13278,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; pt-br; E10a Build/1.1.A.0.9) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA37970 = {S59922, 141, { &P13278,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; es-co; E10a Build/1.0.A.1.38) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA37973 = {S59923, 141, { &P13278,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; E10a Build/1.1.A.0.12) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2236469 = {S59924, 171, { &P15991,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2603868 = {S59925, 143, { &P21374,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ARCHOS 101 Titanium Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2994062 = {S59926, 72, { &P21374,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ARCHOS 101 Titanium Build/JRO03H)*/
const Device UA2674745 = {S59927, 137, { &P12814,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I777 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676508 = {S59928, 145, { &P12814,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I777 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677173 = {S59929, 145, { &P12814,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I777 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707351 = {S59930, 137, { &P12814,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-I777 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA683773 = {S59931, 140, { &P23884,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ZiiLABS ZiiO7  Build/v2.02.04) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile Safari/528.16*/
const Device UA69531 = {S59932, 135, { &P15223,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; Zio Build/DRC92) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA69545 = {S59933, 59, { &P15223,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; es-us; Zio Build/DRC92)*/
const Device UA1997248 = {S59934, 150, { &P13644,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EVO Build/IML74K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028612 = {S59935, 154, { &P13644,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PC36100 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2233256 = {S59936, 150, { &P13644,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; EVO Build/IML74K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2360504 = {S59937, 154, { &P13644,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; EVO Build/IMM76D; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455521 = {S59938, 154, { &P13644,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PC36100 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2552697 = {S59939, 171, { &P13644,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PC36100 Build/JRO03R; CyanogenMod-Team D.I.R.T RemICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2006081 = {S59940, 139, { &P17595,&P18207,&P22376,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.19 (KHTML, like Gecko) Maxthon/1.0.3.1000 Chrome/18.0.1025.142 Safari/535.19*/
const Device UA2015101 = {S59941, 139, { &P17595,&P18207,&P22376,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/535.19 (KHTML, like Gecko) Maxthon/1.0.3.1000 Chrome/18.0.1025.142 Safari/535.19*/
const Device UA2197905 = {S59942, 136, { &P17595,&P18207,&P22376,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/1.0.4.1000 Chrome/21.0.1180.75 Safari/537.1*/
const Device UA2203949 = {S59943, 136, { &P17595,&P18207,&P22376,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/1.0.4.2000 Chrome/21.0.1180.75 Safari/537.1*/
const Device UA2361556 = {S59944, 136, { &P17595,&P18207,&P22376,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/1.0.4.2000 Chrome/21.0.1180.75 Safari/537.1*/
const Device UA2451140 = {S59945, 139, { &P17595,&P18207,&P22376,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.19 (KHTML, like Gecko) Maxthon/1.0.3.1000 Chrome/18.0.1025.142 Safari/535.19*/
const Device UA2595492 = {S59946, 136, { &P17595,&P18207,&P22376,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/1.0.4.2000 Chrome/21.0.1180.75 Safari/537.1*/
const Device UA2260244 = {S59947, 67, { &P15364,&P18110,&P18691,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:16.0) Gecko/X Thunderbird/16.0.2*/
const Device UA2234378 = {S59948, 134, { &P20859,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PMP3170B Build/PMP3170B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2384594 = {S59949, 138, { &P20859,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PMP3170BEN Build/PMP3170BEN) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2489425 = {S59950, 133, { &P18465,&P18110,&P19838,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Safari/535.19 Silk-Accelerated=false*/
const Device UA614853 = {S59951, 92, { &P15364,&P7133,&P22357,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:12.0) Gecko/X Firefox/12.0-x64 PaleMoon/12.0-x64*/
const Device UA614868 = {S59952, 79, { &P15364,&P7133,&P22357,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.0) Gecko/X Firefox/12.0 PaleMoon/12.0*/
const Device UA2385903 = {S59953, 151, { &P20094,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825-1 Build/HuaweiU8825-1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2401524 = {S59954, 149, { &P20094,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2502155 = {S59955, 149, { &P20094,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI C8825D Build/HuaweiC8825D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2731534 = {S59956, 143, { &P20094,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Bs 401 Build/HuaweiU8825-1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2605816 = {S59957, 155, { &P18466,&P18066,&P21707,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2753613 = {S59958, 141, { &P15802,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Amaze 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754578 = {S59959, 141, { &P15802,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC_Amaze_4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2770278 = {S59960, 137, { &P15802,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Ruby Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA736527 = {S59961, 137, { &P21438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; LG-F120K Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2810159 = {S59962, 61, { &P21438,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-F120K Build/GRK39F)*/
const Device UA2369184 = {S59963, 71, { &P15364,&P16110,&P20404,&P18092 }};	/*Useragent - Opera/12.80 (Windows NT 6.2; WOW64; U; en)Presto/22.9.168 Version/12.80*/
const Device UA2558992 = {S59964, 93, { &P11854,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae71-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2716447 = {S59965, 92, { &P11854,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE71) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2752323 = {S59966, 93, { &P11854,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE71x) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2794315 = {S59967, 30, { &P11854,&P5890,&P24676,&P18092 }};	/*Useragent - NokiaE71/UCWEB8.9.0.253/28/999*/
const Device UA16968 = {S59968, 128, { &P13960,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-au; V9 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA761132 = {S59969, 130, { &P13960,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; V9 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018901 = {S59970, 52, { &P13960,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; V9 Build/FRF91)*/
const Device UA607744 = {S59971, 147, { &P12972,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T989 Build/IML74K; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2028961 = {S59972, 147, { &P12972,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T989 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2031402 = {S59973, 166, { &P12972,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-T989 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2974729 = {S59974, 137, { &P18458,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3068399 = {S59975, 145, { &P19867,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083664 = {S59976, 146, { &P19867,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110904 = {S59977, 146, { &P19867,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437Z Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA609590 = {S59978, 154, { &P16362,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA811918 = {S59979, 154, { &P16362,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA904423 = {S59980, 154, { &P16362,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA895608 = {S59981, 95, { &P17595,&P17296,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; Edition MAS; en) Presto/2.10.229 Version/11.64*/
const Device UA895202 = {S59982, 151, { &P18511,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; URBANO PROGRESSO Build/008.2.2400) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA905781 = {S59983, 151, { &P18511,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; URBANO PROGRESSO Build/008.4.2480) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996697 = {S59984, 151, { &P18511,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; URBANO PROGRESSO Build/010.0.3000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257274 = {S59985, 151, { &P18511,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; URBANO PROGRESSO Build/011.0.3100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2676506 = {S59986, 129, { &P18385,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676985 = {S59987, 129, { &P18385,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus 7 Build/JRO03S) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2707594 = {S59988, 127, { &P18385,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; Nexus 7 Build/JRN84D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2860575 = {S59989, 136, { &P18385,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA38364 = {S59990, 138, { &P17718,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB611 Build/BGN_2.0.83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38370 = {S59991, 138, { &P17718,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB611 Build/BGN_2.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38394 = {S59992, 138, { &P17718,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MB611 Build/BGN_2.0.41) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA858175 = {S59993, 148, { &P17718,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB611 Build/4.5.1-149-BGN-2.0-87) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020469 = {S59994, 148, { &P17718,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB611 Build/4.5.1-149-BGN-2.0-84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2436056 = {S59995, 149, { &P17346,&P17146,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9A405 Safari/7534.48.3*/
const Device UA2469442 = {S59996, 147, { &P17346,&P17146,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/9A334 Safari/7534.48.3*/
const Device UA2870146 = {S59997, 101, { &P22579,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 925)*/
const Device UA2900423 = {S59998, 111, { &P22579,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 925; Vodafone)*/
const Device UA3019222 = {S59999, 109, { &P22579,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 925; ANZ892)*/
const Device UA3080876 = {S60000, 109, { &P22579,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 925; 1326-2)*/
const Device UA3065321 = {S60001, 135, { &P22126,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Torque Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2715817 = {S60002, 76, { &P17595,&P18207,&P20794,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:7.0.1) Gecko/X Postbox/3.0.8*/
const Device UA2356859 = {S60003, 148, { &P15767,&P18432,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 TeamStream HD (iPad2,1 284 1.7)*/
const Device UA2424710 = {S60004, 146, { &P15767,&P18432,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3 TeamStream HD (iPad2,1 284 1.7)*/
const Device UA2461380 = {S60005, 148, { &P15767,&P18432,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550 Safari/7534.48.3 TeamStream HD (iPad2,5 284 1.7)*/
const Device UA2528236 = {S60006, 149, { &P15767,&P18432,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A8426 Safari/7534.48.3 TeamStream HD (iPad3,6 284 1.7)*/
const Device UA2353070 = {S60007, 178, { &P18643,&P5043,&P24672,&P18092 }};	/*Useragent - Nokia311/5.0 (03.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia311) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA464796 = {S60008, 64, { &P18182,&P5448,&P3430,&P18092 }};	/*Useragent - Opera/9.51 Beta (Microsoft Windows; PPC; Opera Mobi/1718; U; en)*/
const Device UA2362430 = {S60009, 147, { &P23102,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8681 Build/HuaweiU8681) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA742813 = {S60010, 135, { &P21080,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fr-fr; Blade S Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2261523 = {S60011, 137, { &P21022,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E615 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2361461 = {S60012, 138, { &P21022,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E615f Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA625916 = {S60013, 143, { &P22590,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; SGH-I717M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA813460 = {S60014, 67, { &P22590,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-I717M Build/GINGERBREAD)*/
const Device UA2423829 = {S60015, 60, { &P18557,&P20706,&P18517,&P18092 }};	/*Useragent - Nokia308/2.0 (05.85) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2790572 = {S60016, 84, { &P18557,&P20706,&P18517,&P18092 }};	/*Useragent - Nokia308/2.0 (05.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 Nokia308/05.80 CLDC-1.1*/
const Device UA1996516 = {S60017, 125, { &P15364,&P16110,&P22375,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.4.2.3000 Chrome/18.0.966.0 Safari/535.12*/
const Device UA1997034 = {S60018, 118, { &P15364,&P16110,&P22375,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.4.2.3000 Chrome/18.0.966.0 Safari/535.12*/
const Device UA2975946 = {S60019, 137, { &P18488,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974156 = {S60020, 135, { &P18468,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; IdeaTabA2109A Build/JRO03R) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3064873 = {S60021, 139, { &P18582,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT28h Build/6.2.B.0.211) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066519 = {S60022, 139, { &P18582,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT28i Build/6.2.B.0.211) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068320 = {S60023, 134, { &P20213,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N9500 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA234678 = {S60024, 142, { &P18701,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; IM-T100K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1902932 = {S60025, 66, { &P18701,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; IM-T100K Build/GINGERBREAD)*/
const Device UA1940457 = {S60026, 140, { &P18701,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; IM-T100K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2787796 = {S60027, 142, { &P18701,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; IM-T100K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA379001 = {S60028, 82, { &P17595,&P22475,&P17317,&P18092 }};	/*Useragent - Safari5533.22.3 CFNetwork/438.16 Darwin/9.8.0 (Power%20Macintosh) (PowerBook5%2C6)*/
const Device UA473992 = {S60029, 67, { &P17595,&P22475,&P17317,&P18092 }};	/*Useragent - Safari5533.18.5 CFNetwork/438.14 Darwin/9.8.0 (i386) (MacBook4%2C1)*/
const Device UA499157 = {S60030, 64, { &P17595,&P22475,&P17317,&P18092 }};	/*Useragent - Safari5533.19.4 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac7%2C1)*/
const Device UA546801 = {S60031, 67, { &P17595,&P22475,&P17317,&P18092 }};	/*Useragent - Safari5533.21.1 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook5%2C2)*/
const Device UA564782 = {S60032, 66, { &P17595,&P22475,&P17317,&P18092 }};	/*Useragent - Safari5533.22.3 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacPro1%2C1)*/
const Device UA643786 = {S60033, 70, { &P17595,&P22475,&P17317,&P18092 }};	/*Useragent - Safari5533.22.3 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBookPro5%2C4)*/
const Device UA650117 = {S60034, 67, { &P17595,&P22475,&P17317,&P18092 }};	/*Useragent - Safari5533.22.3 CFNetwork/438.16 Darwin/9.8.0 (i386) (Macmini2%2C1)*/
const Device UA680048 = {S60035, 70, { &P17595,&P22475,&P17317,&P18092 }};	/*Useragent - Safari5533.21.1 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBookPro5%2C1)*/
const Device UA702630 = {S60036, 82, { &P17595,&P22475,&P17317,&P18092 }};	/*Useragent - Safari5533.22.3 CFNetwork/438.16 Darwin/9.8.0 (Power%20Macintosh) (PowerMac12%2C1)*/
const Device UA712193 = {S60037, 82, { &P17595,&P22475,&P17317,&P18092 }};	/*Useragent - Safari5533.21.1 CFNetwork/438.16 Darwin/9.8.0 (Power%20Macintosh) (PowerMac11%2C2)*/
const Device UA713361 = {S60038, 70, { &P17595,&P22475,&P17317,&P18092 }};	/*Useragent - Safari5533.22.3 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBookAir1%2C1)*/
const Device UA742666 = {S60039, 64, { &P17595,&P22475,&P17317,&P18092 }};	/*Useragent - Safari5533.21.1 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac8%2C1)*/
const Device UA762047 = {S60040, 67, { &P17595,&P22475,&P17317,&P18092 }};	/*Useragent - Safari5533.19.4 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook5%2C1)*/
const Device UA792421 = {S60041, 64, { &P17595,&P22475,&P17317,&P18092 }};	/*Useragent - Safari5533.22.3 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac5%2C1)*/
const Device UA838321 = {S60042, 70, { &P17595,&P22475,&P17317,&P18092 }};	/*Useragent - Safari5533.19.4 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBookPro4%2C1)*/
const Device UA896882 = {S60043, 71, { &P17595,&P22475,&P17317,&P18092 }};	/*Useragent - Safari5533.20.27 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBookPro4%2C1)*/
const Device UA1868188 = {S60044, 68, { &P17595,&P22475,&P17317,&P18092 }};	/*Useragent - Safari5533.20.27 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook5%2C2)*/
const Device UA1875166 = {S60045, 67, { &P17595,&P22475,&P17317,&P18092 }};	/*Useragent - Safari5533.21.1 CFNetwork/438.16 Darwin/9.8.0 (i386) (Macmini2%2C1)*/
const Device UA2276622 = {S60046, 65, { &P17595,&P22475,&P17317,&P18092 }};	/*Useragent - Safari5533.20.27 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac9%2C1)*/
const Device UA3098367 = {S60047, 67, { &P17595,&P22475,&P17317,&P18092 }};	/*Useragent - Safari5533.22.3 CFNetwork/438.14 Darwin/9.8.0 (i386) (MacBook5%2C1)*/
const Device UA645926 = {S60048, 68, { &P15364,&P5690,&P22766,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Maxthon/3.0)*/
const Device UA729831 = {S60049, 63, { &P15364,&P5690,&P22766,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Maxthon/3.0)*/
const Device UA2147565 = {S60050, 126, { &P15364,&P5690,&P22766,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; Maxthon/3.0)*/
const Device UA3096231 = {S60051, 112, { &P12989,&P5843,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia303/13.47; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA68109 = {S60052, 131, { &P14905,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; GT-P7100 Build/HRI83) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA319725 = {S60053, 60, { &P14905,&P6545,&P4755,&P18092 }};	/*Useragent - Dalvik/1.5.0 (Linux; U; Android 3.0.1; GT-P7100 Build/HRI83)*/
const Device UA2607582 = {S60054, 129, { &P14905,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; ru-; GT-P7100 Build/HRI83) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2607586 = {S60055, 129, { &P14905,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0; xx-xx; GT-P7100 Build/HRI83) AppleWebkit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2607656 = {S60056, 129, { &P14905,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0; en-us; GT-P7100 Build/HRI53) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA224200 = {S60057, 143, { &P18085,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SHW-M290K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA651253 = {S60058, 143, { &P18085,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M290K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020357 = {S60059, 67, { &P18085,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M290K Build/GINGERBREAD)*/
const Device UA666201 = {S60060, 62, { &P13107,&P5043,&P4030,&P18092 }};	/*Useragent - NokiaC2-06/2.0 (07.48) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2147126 = {S60061, 73, { &P13107,&P5043,&P4030,&P18092 }};	/*Useragent - NokiaC2-06/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-06*/
const Device UA68324 = {S60062, 136, { &P14161,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LG-VM670 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA69091 = {S60063, 142, { &P14161,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-VM670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73496 = {S60064, 136, { &P14161,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-VM670 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA173623 = {S60065, 140, { &P14161,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LG Optimus V Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA213510 = {S60066, 142, { &P14161,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-VM670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA318362 = {S60067, 136, { &P14161,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-VM670 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA342450 = {S60068, 136, { &P14161,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM670 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA376451 = {S60069, 133, { &P14161,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; VM670 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA421732 = {S60070, 136, { &P14161,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM670 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA659818 = {S60071, 169, { &P14161,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM670 Build/ICS19; ICSandwich2.3.7 - Final Version) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA678150 = {S60072, 132, { &P14161,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; M670 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA751613 = {S60073, 159, { &P14161,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM670 Build/ICS40; ICSandwich2.3.7-FINAL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA829253 = {S60074, 151, { &P14161,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM670 Build/ICS43; ICSandwich v3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1973252 = {S60075, 151, { &P14161,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM670 Build/ICS23; ICSandwich v2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021700 = {S60076, 60, { &P14161,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; LG-VM670 Build/GRI40)*/
const Device UA913169 = {S60077, 118, { &P15364,&P17285,&P22460,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11*/
const Device UA1992533 = {S60078, 125, { &P15364,&P17285,&P22460,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11*/
const Device UA2508572 = {S60079, 91, { &P12004,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAN95) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2537636 = {S60080, 95, { &P12004,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAN95 8GB) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2895004 = {S60081, 80, { &P15364,&P16110,&P24329,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:22.0) Gecko/X Firefox/22.0 SeaMonkey/2.19*/
const Device UA2973674 = {S60082, 136, { &P18438,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E973 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973912 = {S60083, 137, { &P18438,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F200S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974930 = {S60084, 137, { &P18438,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F200K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975805 = {S60085, 137, { &P18438,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F200L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA924829 = {S60086, 132, { &P12879,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7510 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1996423 = {S60087, 132, { &P12879,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7510 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1999203 = {S60088, 132, { &P12879,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7510 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2361484 = {S60089, 132, { &P12879,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7510 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2455836 = {S60090, 61, { &P12879,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P7510 Build/JZO54K)*/
const Device UA2579371 = {S60091, 132, { &P12879,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7510 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2611030 = {S60092, 116, { &P12879,&P18111,&P18112,&P18092 }};	/*Useragent - Vyrso/1.1.3 (Linux; U; Android 4.1.1; en-US; GT-P7510 Build/cm_p4wifi-userdebug 4.1.1 JRO03H eng.root.X.X test-keys)*/
const Device UA2612835 = {S60093, 124, { &P12879,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.0.9 (Linux; U; Android 4.1.1; en-US; GT-P7510 Build/full_GT-P7510-userdebug 4.1.1 JRO03L eng.maplesci.X.X test-keys)*/
const Device UA2614136 = {S60094, 117, { &P12879,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.0.11 (Linux; U; Android 4.1.1; en-GB; GT-P7510 Build/cm_p4wifi-userdebug 4.1.1 JRO03L eng.matt.X.X test-keys)*/
const Device UA2614254 = {S60095, 61, { &P12879,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P7510 Build/JRO03H)*/
const Device UA2616694 = {S60096, 120, { &P12879,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.1.2 (Linux; U; Android 4.1.1; en-US; GT-P7510 Build/cm_p4wifi-userdebug 4.1.1 JRO03H eng.maplesci.X.X test-keys)*/
const Device UA2616776 = {S60097, 125, { &P12879,&P18111,&P18112,&P18092 }};	/*Useragent - Groupon/2.0.2414 (Android 4.1.1; Samsung P4wifi / Samsung GT-P7510; )[preload=false;locale=en_US;clientidbase=android-google]*/
const Device UA2616778 = {S60098, 273, { &P12879,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7510 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [FBAN/FB4A;FBAV/1.9.11;FBDM/{density=1.0,width=1280,height=752};FBLC/en_US;FB_FW/2;FBCR/;FBPN/com.facebook.katana;FBDV/GT-P7510;FBSV/4.1.1;]*/
const Device UA2616837 = {S60099, 93, { &P12879,&P18111,&P18112,&P18092 }};	/*Useragent - com.google.android.youtube/4.1.23(Linux; U; Android 4.1.1; en_US; GT-P7510 Build/JRO03H) gzip*/
const Device UA2618353 = {S60100, 145, { &P12879,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7510 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2619209 = {S60101, 130, { &P12879,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-P7510 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2619465 = {S60102, 130, { &P12879,&P18111,&P18112,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.2.3 (Linux; U; Android 4.1.2; en-US; GT-P7510 Build/cm_p4wifi-userdebug 4.1.2 JZO54K eng.root.X.X test-keys)*/
const Device UA2621653 = {S60103, 94, { &P12879,&P18111,&P18112,&P18092 }};	/*Useragent - com.google.android.apps.plus/X (Linux; U; Android 4.1.2; en_US; GT-P7510; Build/JZO54K) (gzip)*/
const Device UA2622103 = {S60104, 87, { &P12879,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.2.7 (Linux; U; Android 4.1.2; en-CA; GT-P7510 Build/LIQUID PURE CM10 31.10 RC1)*/
const Device UA2622409 = {S60105, 135, { &P12879,&P18111,&P18112,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.2.8.1 (Linux; U; Android 4.1.1; en-US; GT-P7510 Build/cm_p4wifi-userdebug 4.1.1 JRO03L eng.matt.X.X release-keys)*/
const Device UA2627789 = {S60106, 132, { &P12879,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P7510 Build/JZO05K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2637487 = {S60107, 82, { &P12879,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7510 Build/JZO54K) Mint.com/2.6.1*/
const Device UA1993938 = {S60108, 142, { &P14635,&P5843,&P24670,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (07.20) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac3-00) UCBrowser8.4.0.159/70/352/UCWEB Mobile*/
const Device UA2366064 = {S60109, 144, { &P18721,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Micromax A90S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2404252 = {S60110, 135, { &P18721,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; A90S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2843402 = {S60111, 57, { &P18721,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; A90S Build/IMM76D)*/
const Device UA2510092 = {S60112, 140, { &P20517,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; CINK PEAX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2675895 = {S60113, 142, { &P22209,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Titanium S5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2795594 = {S60114, 64, { &P22209,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Titanium S5 Build/JZO54K)*/
const Device UA3028238 = {S60115, 138, { &P20705,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; HTC One Build/JWR66V.H10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3034283 = {S60116, 134, { &P20705,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; One Build/JWR66V.H10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2515844 = {S60117, 82, { &P15364,&P16110,&P20456,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:18.0) Gecko/X Firefox/18.0 SeaMonkey/2.15.2*/
const Device UA2482971 = {S60118, 138, { &P15767,&P19826,&P21787,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA2533343 = {S60119, 140, { &P15767,&P19826,&P21787,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA2607392 = {S60120, 140, { &P15767,&P19826,&P21787,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA3065109 = {S60121, 151, { &P15972,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC_One_XL/5.08.111.2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3067832 = {S60122, 151, { &P15972,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; de-de; HTC/One_XL/5.06.162.1 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079481 = {S60123, 151, { &P15972,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; de-ch; HTC/One_XL/5.06.166.1 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3080634 = {S60124, 62, { &P15972,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC One XL Build/JDQ39)*/
const Device UA3080636 = {S60125, 140, { &P15972,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; HTC One XL Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3080806 = {S60126, 140, { &P15972,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-sg; HTC_One_XL Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3094001 = {S60127, 151, { &P15972,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; HTC/One_XL/5.08.163.2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2499839 = {S60128, 81, { &P15364,&P18110,&P18345,&P18163 }};	/*Useragent - Mozilla/Nutch-1.5.1 (Mozilla/5.0 (X11; Linux i686; rv:15.0) Gecko/X Firefox/15.0)*/
const Device UA2679284 = {S60129, 152, { &P12605,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2712121 = {S60130, 145, { &P12605,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2862387 = {S60131, 135, { &P12605,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT875 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2712565 = {S60132, 152, { &P15803,&P5843,&P24677,&P18092 }};	/*Useragent - Nokia200/2.0 (11.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia200) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2754332 = {S60133, 130, { &P15932,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2756006 = {S60134, 130, { &P15932,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P3110 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA243743 = {S60135, 114, { &P15364,&P5690,&P22451,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.1 Chrome/13.0.800.1 Safari/535.1*/
const Device UA813909 = {S60136, 121, { &P15364,&P5690,&P22451,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.1 Chrome/13.0.800.1 Safari/535.1*/
const Device UA2753556 = {S60137, 145, { &P19891,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755562 = {S60138, 139, { &P12280,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4m*/
const Device UA2770456 = {S60139, 141, { &P12280,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3 Sleipnir/2.4.2m*/
const Device UA2773301 = {S60140, 141, { &P12280,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.2m*/
const Device UA2773925 = {S60141, 141, { &P12280,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3 Sleipnir/2.4.1m*/
const Device UA2773937 = {S60142, 141, { &P12280,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.1m*/
const Device UA2796775 = {S60143, 141, { &P12280,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.4.2m*/
const Device UA2877297 = {S60144, 139, { &P12280,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3 Sleipnir/2.4m*/
const Device UA2884019 = {S60145, 141, { &P12280,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3 Sleipnir/2.4.3m*/
const Device UA2885448 = {S60146, 141, { &P12280,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.3m*/
const Device UA2900484 = {S60147, 141, { &P12280,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.4m*/
const Device UA2945012 = {S60148, 141, { &P12280,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3 Sleipnir/2.4.4m*/
const Device UA2964306 = {S60149, 141, { &P12280,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.4.4m*/
const Device UA3037439 = {S60150, 141, { &P12280,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3 Sleipnir/2.4.5m*/
const Device UA3037443 = {S60151, 141, { &P12280,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.5m*/
const Device UA3093027 = {S60152, 141, { &P12280,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.4.5m*/
const Device UA2975541 = {S60153, 136, { &P12788,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PG86100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021829 = {S60154, 136, { &P12788,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ISW12HT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA67640 = {S60155, 164, { &P14856,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; LG-SU660 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA67801 = {S60156, 165, { &P14856,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; LG-SU660 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA434939 = {S60157, 61, { &P14856,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-SU660 Build/FRG83G)*/
const Device UA2405058 = {S60158, 129, { &P18680,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA342036 = {S60159, 126, { &P17934,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0; en-us;) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 (Kobo Touch)*/
const Device UA1996944 = {S60160, 137, { &P18647,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; T-02D Build/V15R47B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2204918 = {S60161, 137, { &P18647,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; T-02D Build/V17R48D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2234344 = {S60162, 137, { &P18647,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; T-02D Build/V18R49C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426006 = {S60163, 137, { &P18647,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; T-02D Build/V19R50B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934396 = {S60164, 59, { &P18647,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; T-02D Build/V19R50B)*/
const Device UA3032622 = {S60165, 137, { &P18647,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; T-02D Build/V20R52A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2235606 = {S60166, 155, { &P18465,&P18066,&P18616,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.1 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2566703 = {S60167, 137, { &P12788,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PG86100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602011 = {S60168, 137, { &P12788,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ISW12HT Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2676582 = {S60169, 138, { &P23772,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190L Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677154 = {S60170, 138, { &P23772,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714384 = {S60171, 131, { &P23772,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2767225 = {S60172, 131, { &P23772,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190L Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2755269 = {S60173, 133, { &P18466,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782613 = {S60174, 126, { &P18466,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3065253 = {S60175, 137, { &P21506,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094443 = {S60176, 138, { &P21506,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA251360 = {S60177, 129, { &P17595,&P17297,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA409596 = {S60178, 129, { &P17595,&P17297,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_3; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA668461 = {S60179, 129, { &P17595,&P17297,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA817773 = {S60180, 129, { &P17595,&P17297,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA2258782 = {S60181, 129, { &P17595,&P17297,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA614149 = {S60182, 102, { &P15364,&P7127,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA2528551 = {S60183, 118, { &P15364,&P7127,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 8.0; Windows NT 5.0) Chrome/18.0.1229.79*/
const Device UA2529845 = {S60184, 118, { &P15364,&P7127,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 7.0; Windows NT 5.0) Chrome/18.0.1229.79*/
const Device UA2737762 = {S60185, 123, { &P15364,&P7127,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 9.0; Windows NT 5.0WOW64) Chrome/18.0.1229.79*/
const Device UA2715174 = {S60186, 142, { &P12280,&P18432,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 UCBrowser/9.0.1.284 Mobile*/
const Device UA2715882 = {S60187, 142, { &P12280,&P18432,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 UCBrowser/9.0.1.284 Mobile*/
const Device UA2716588 = {S60188, 142, { &P12280,&P18432,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 UCBrowser/9.0.1.284 Mobile*/
const Device UA2751375 = {S60189, 140, { &P12280,&P18432,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 UCBrowser/9.0.1.284 Mobile*/
const Device UA2955750 = {S60190, 140, { &P12280,&P18432,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 UCBrowser/9.0.1.284 Mobile*/
const Device UA2716733 = {S60191, 94, { &P14033,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC5-03) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2732938 = {S60192, 32, { &P14033,&P5129,&P24676,&P18092 }};	/*Useragent - NokiaC5-03/UCWEB8.9.0.253/50/999*/
const Device UA901760 = {S60193, 69, { &P17595,&P17295,&P18333,&P18092 }};	/*Useragent - iTunes/10.6.3 (Macintosh; Intel Mac OS X 10.5.8) AppleWebKit/534.50.2*/
const Device UA903698 = {S60194, 67, { &P17595,&P17295,&P18333,&P18092 }};	/*Useragent - iTunes/10.6.3 (Macintosh; PPC Mac OS X 10.5.8) AppleWebKit/534.50.2*/
const Device UA2973481 = {S60195, 145, { &P12972,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-T989 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974597 = {S60196, 136, { &P12972,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T989 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3132633 = {S60197, 63, { &P15364,&P18615,&P24713,&P18092 }};	/*Useragent - Mozilla/5.0 (Mobile; Nexus 10; rv:18.1) Gecko/18.1 Firefox/18.1*/
const Device UA12596 = {S60198, 77, { &P11744,&P5026,&P3410,&P18092 }};	/*Useragent - BlackBerry8300/4.5.0.182 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102*/
const Device UA2181911 = {S60199, 141, { &P12110,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; NGM Legend Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2459501 = {S60200, 142, { &P12110,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NGM Legend2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483475 = {S60201, 135, { &P12110,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; novo7Legend Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2505427 = {S60202, 143, { &P12110,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NGM LegendXL Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2550829 = {S60203, 137, { &P12110,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; Legend Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2728899 = {S60204, 142, { &P12110,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; NGM_Legend2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878297 = {S60205, 143, { &P12110,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; NGM_LegendXL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2982898 = {S60206, 64, { &P12110,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; novo7Legend Build/IMM76D)*/
const Device UA2670756 = {S60207, 134, { &P19904,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Hydro Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2678421 = {S60208, 133, { &P19904,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C5170 Build/IML77) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3073088 = {S60209, 126, { &P19904,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C5170 Build/IML77) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2974792 = {S60210, 141, { &P13124,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP9070 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3067190 = {S60211, 142, { &P21542,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6503 Build/10.3.1.A.0.244) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2753869 = {S60212, 145, { &P19891,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937889 = {S60213, 145, { &P19891,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I717 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2369690 = {S60214, 133, { &P22254,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; M-MP101S2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2093 = {S60215, 83, { &P13206,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK530i/R8BB Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24773 = {S60216, 83, { &P13206,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK530c/R8BA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2676179 = {S60217, 127, { &P17926,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT100 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3065981 = {S60218, 136, { &P23697,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9192 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067597 = {S60219, 145, { &P13064,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126219 = {S60220, 145, { &P13064,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I727 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2947542 = {S60221, 89, { &P16488,&P5567,&P18525,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/6.0.3/27.2338; U; en) Presto/2.8.119 240X400 Pantech CDM8992*/
const Device UA2754473 = {S60222, 133, { &P20760,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Event Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA33248 = {S60223, 87, { &P14060,&P17485,&P3499,&P18092 }};	/*Useragent - SonyEricssonW150i/R2BA006 TelecaBrowser/Q07C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA538566 = {S60224, 97, { &P15364,&P5690,&P22479,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.14) Gecko/X Firefox/2.0.0.14 Flock/1.2.2*/
const Device UA813967 = {S60225, 97, { &P15364,&P5690,&P22479,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.8.1.14) Gecko/X Firefox/2.0.0.14 Flock/1.2.1*/
const Device UA2784815 = {S60226, 134, { &P20872,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGP311 Build/10.1.C.0.370) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA227719 = {S60227, 128, { &P17926,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; AT100 Build/HRI83) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2230023 = {S60228, 146, { &P14862,&P5204,&P24670,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia2700c-2) UCBrowser8.4.0.159/70/444/UCWEB Mobile*/
const Device UA2676015 = {S60229, 136, { &P18243,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P880 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676752 = {S60230, 137, { &P18243,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P880g Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2751344 = {S60231, 129, { &P18243,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P880 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676193 = {S60232, 131, { &P18606,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676755 = {S60233, 138, { &P18606,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA13511 = {S60234, 101, { &P11754,&P16000,&P3772,&P18092 }};	/*Useragent - Opera 8.54 SonyEricssonP990i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 513) Opera 8.65 [en]*/
const Device UA54642 = {S60235, 100, { &P11754,&P16000,&P3772,&P18092 }};	/*Useragent - Opera 7.0 SonyEricssonP990i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 513) Opera 8.65 [en]*/
const Device UA54816 = {S60236, 90, { &P11754,&P16000,&P3772,&P18092 }};	/*Useragent - SonyEricssonP990i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 598) Opera 8.65 [en]*/
const Device UA64143 = {S60237, 90, { &P11754,&P16000,&P3772,&P18092 }};	/*Useragent - SonyEricssonP990i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 306) Opera 8.60 [en]*/
const Device UA23251 = {S60238, 132, { &P13257,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; Droid Build/FRG22D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA50208 = {S60239, 141, { &P13257,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Droid Build/ESE81) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA50272 = {S60240, 133, { &P13257,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; Droid Build/ESE81) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA50396 = {S60241, 133, { &P13257,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; Droid Build/ESE53) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA56102 = {S60242, 142, { &P13257,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1 (ESE 53); en-us; Droid Build/ESE53) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA56333 = {S60243, 141, { &P13257,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Droid Build/ESE53) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA68424 = {S60244, 133, { &P13257,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; Droid Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA167680 = {S60245, 145, { &P13257,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Vip Droid Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2567097 = {S60246, 122, { &P15767,&P19826,&P21569,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Incredimail/1.0.4.3811*/
const Device UA2583197 = {S60247, 120, { &P15767,&P19826,&P21569,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Incredimail/1.0.4.3811*/
const Device UA2583413 = {S60248, 122, { &P15767,&P19826,&P21569,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B147 Incredimail/1.0.4.3811*/
const Device UA2601585 = {S60249, 56, { &P15767,&P19826,&P21569,&P18092 }};	/*Useragent - Incredimail 1.0.4 rv:3811 (iPad; iPhone OS 6.1.2; en_US)*/
const Device UA2672776 = {S60250, 122, { &P15767,&P19826,&P21569,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Incredimail/1.0.5.4111*/
const Device UA2675589 = {S60251, 56, { &P15767,&P19826,&P21569,&P18092 }};	/*Useragent - Incredimail 1.0.5 rv:4111 (iPad; iPhone OS 6.1.3; en_US)*/
const Device UA402006 = {S60252, 137, { &P15364,&P17820,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.11 (KHTML, like Gecko) Ubuntu/12.04 Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.11*/
const Device UA428865 = {S60253, 135, { &P15364,&P17820,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.11 (KHTML, like Gecko) Ubuntu/12.04 Chromium/17.0.963.79 Chrome/17.0.963.79 Safari/535.11*/
const Device UA704870 = {S60254, 169, { &P15364,&P17820,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Ubuntu/12.04 (3.4.1-0ubuntu1) Epiphany/3.4.1*/
const Device UA819507 = {S60255, 171, { &P15364,&P17820,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Ubuntu/12.04 (3.4.1-0ubuntu1) Epiphany/3.4.1*/
const Device UA2333383 = {S60256, 175, { &P15364,&P17820,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Ubuntu/12.04 (3.4.0.1-2ubuntu1) Epiphany/3.4.0.1*/
const Device UA2444652 = {S60257, 173, { &P15364,&P17820,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+ Ubuntu/12.04 (3.4.0.1-2ubuntu1) Epiphany/3.4.0.1*/
const Device UA2431354 = {S60258, 152, { &P20438,&P18066,&P18062,&P18092 }};	/*Useragent - N400 Linux/3.0.13 Android/4.0.4 Release/11.23.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2555984 = {S60259, 135, { &P20438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N400 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2975945 = {S60260, 145, { &P19867,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020717 = {S60261, 146, { &P19867,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA362852 = {S60262, 140, { &P12821,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us;Sprint ATP515CKIT Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2753079 = {S60263, 141, { &P18462,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA587364 = {S60264, 141, { &P18499,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ja-jp; ISW11SC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA656586 = {S60265, 139, { &P18499,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ja-; ISW11SC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024434 = {S60266, 65, { &P18499,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; ISW11SC Build/GINGERBREAD)*/
const Device UA2489986 = {S60267, 147, { &P15767,&P19826,&P21789,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1.2 (like Version/10B141 Mobile/6.1 Safari/8536.25)*/
const Device UA2532749 = {S60268, 151, { &P15767,&P19826,&P21789,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1.2 (like Version/10B146 Mobile/6.1.2 Safari/8536.25)*/
const Device UA2585697 = {S60269, 151, { &P15767,&P19826,&P21789,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1.2 (like Version/10B329 Mobile/6.1.3 Safari/8536.25)*/
const Device UA3081034 = {S60270, 138, { &P22848,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT920 Build/2_32D_2009) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA903742 = {S60271, 84, { &P15364,&P7133,&P21170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:13.0) Gecko/X Firefox/13.0.1 SeaMonkey/2.10.1*/
const Device UA908701 = {S60272, 69, { &P15364,&P7133,&P21170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:13.0) Gecko/X SeaMonkey/2.10.1*/
const Device UA2706353 = {S60273, 137, { &P22748,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E425 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2750629 = {S60274, 138, { &P22748,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E425f Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2869453 = {S60275, 138, { &P22748,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E425g Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3065365 = {S60276, 130, { &P17168,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7310 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA379607 = {S60277, 109, { &P15364,&P17843,&P17826,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/4.4; Linux 2.6.32-22-generic; X11; en_US) KHTML/4.4.3 (like Gecko) Kubuntu*/
const Device UA329681 = {S60278, 145, { &P12280,&P17146,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA632028 = {S60279, 143, { &P12280,&P17146,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA696685 = {S60280, 130, { &P12280,&P17146,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293*/
const Device UA700758 = {S60281, 147, { &P12280,&P17146,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA1949761 = {S60282, 137, { &P12280,&P17146,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/6531.22.7 iLunascape/4*/
const Device UA2129991 = {S60283, 141, { &P12280,&P17146,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2337263 = {S60284, 128, { &P12280,&P17146,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293*/
const Device UA2492628 = {S60285, 142, { &P12280,&P17146,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone U CPU iPhone OS 5_0 like Mac OS X en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293 Safari/6531.22.7*/
const Device UA2573296 = {S60286, 147, { &P12280,&P17146,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/9A405 Safari/6531.22.7*/
const Device UA2745484 = {S60287, 136, { &P12280,&P17146,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU OS 5_0 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0 Mobile/8C148 Safari/528.16*/
const Device UA3069037 = {S60288, 136, { &P15767,&P22882,&P24462,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Coast/1.1.0.00000 Mobile/11A4449b Safari/7534.48.3*/
const Device UA3111824 = {S60289, 136, { &P15767,&P22882,&P24462,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Coast/1.1.1.64324 Mobile/11A501 Safari/7534.48.3*/
const Device UA3116450 = {S60290, 134, { &P15767,&P22882,&P24462,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Coast/1.1.0.64021 Mobile/11A465 Safari/7534.48.3*/
const Device UA3116589 = {S60291, 136, { &P15767,&P22882,&P24462,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Coast/1.1.0.64021 Mobile/11A4449d Safari/7534.48.3*/
const Device UA2366479 = {S60292, 70, { &P20134,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GT-S5830i; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2655700 = {S60293, 135, { &P20156,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; S500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2779855 = {S60294, 57, { &P20156,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; S500 Build/JZO54K)*/
const Device UA22919 = {S60295, 126, { &P16589,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; S300 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0  Safari/533.1*/
const Device UA22920 = {S60296, 132, { &P16589,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; S300 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60286 = {S60297, 132, { &P16589,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; S300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA685652 = {S60298, 141, { &P16589,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; KM-S300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021425 = {S60299, 56, { &P16589,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; S300 Build/GRI40)*/
const Device UA2721260 = {S60300, 65, { &P16589,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; KM-S300 Build/GINGERBREAD)*/
const Device UA2885302 = {S60301, 112, { &P18219,&P5843,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia202/20.28; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA2963492 = {S60302, 112, { &P18219,&P5843,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia203/20.52; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA2974284 = {S60303, 138, { &P13141,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I727R Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021237 = {S60304, 138, { &P13141,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E110S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3034636 = {S60305, 137, { &P13141,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9210 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA900968 = {S60306, 132, { &P20120,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A45 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2573452 = {S60307, 56, { &P20120,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; A45 Build/GRK39F)*/
const Device UA2582932 = {S60308, 135, { &P23183,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T1-D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA54662 = {S60309, 166, { &P11768,&P17485,&P3487,&P18092 }};	/*Useragent - SonyEricssonW395/R1EA Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1, SonyEricssonW395/R1EA Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA54663 = {S60310, 250, { &P11768,&P17485,&P3487,&P18092 }};	/*Useragent - SonyEricssonW395/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1, SonyEricssonW395/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1, SonyEricssonW395/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63702 = {S60311, 82, { &P11768,&P17485,&P3487,&P18092 }};	/*Useragent - SonyEricssonW395/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA220049 = {S60312, 134, { &P18048,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS12SH Build/S7141) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA235751 = {S60313, 134, { &P18048,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS12SH Build/S6160) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA247091 = {S60314, 134, { &P18048,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS12SH Build/S9081) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA379980 = {S60315, 134, { &P18048,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS12SH Build/SB220) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458429 = {S60316, 134, { &P18048,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS12SH Build/S3081) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622425 = {S60317, 142, { &P18048,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS12SH Build/S3081) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.1 Mobile Safari/534.30*/
const Device UA2031056 = {S60318, 134, { &P18048,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS12SH Build/S8030) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3070443 = {S60319, 58, { &P18048,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; IS12SH Build/S8030)*/
const Device UA898208 = {S60320, 99, { &P15364,&P7127,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5*/
const Device UA2505499 = {S60321, 118, { &P15364,&P7127,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 8.0; Windows NT 5.0) Chrome/19.0.1229.79*/
const Device UA2555306 = {S60322, 118, { &P15364,&P7127,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 7.0; Windows NT 5.0) Chrome/19.0.1229.79*/
const Device UA2737757 = {S60323, 123, { &P15364,&P7127,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Randomized by FreeSafeIP.com/upgrade-to-remove; compatible; MSIE 9.0; Windows NT 5.0WOW64) Chrome/19.0.1229.79*/
const Device UA2752934 = {S60324, 138, { &P18448,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST26i Build/11.0.A.7.5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753792 = {S60325, 138, { &P18448,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST26a Build/11.0.A.7.5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754258 = {S60326, 139, { &P18449,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST27i Build/6.2.A.0.400) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2897962 = {S60327, 139, { &P18449,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST27a Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2405199 = {S60328, 155, { &P13035,&P5043,&P24673,&P18092 }};	/*Useragent - NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac2-02) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2430562 = {S60329, 157, { &P14872,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-D700 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2562199 = {S60330, 156, { &P14872,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-D700 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2968479 = {S60331, 166, { &P14872,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-D700 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/epicmtd*/
const Device UA2737449 = {S60332, 134, { &P18283,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3.0; en-us; I9300 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3032709 = {S60333, 137, { &P18283,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-gb; GT-I9300 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3036414 = {S60334, 137, { &P18283,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I535 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3062137 = {S60335, 137, { &P18283,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I535 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3062645 = {S60336, 59, { &P18283,&P21637,&P23997,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.3; GT-I9300 Build/JSS15Q)*/
const Device UA3079639 = {S60337, 137, { &P18283,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-T999 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3082172 = {S60338, 145, { &P18283,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-T999 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3093333 = {S60339, 137, { &P18283,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-gb; GT-I9300 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108426 = {S60340, 59, { &P18283,&P21637,&P23997,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.3; SCH-I535 Build/JSS15Q)*/
const Device UA3021254 = {S60341, 137, { &P15932,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P3110 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021502 = {S60342, 129, { &P15932,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P3110 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2900946 = {S60343, 112, { &P18221,&P5843,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia302/14.26; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA3067915 = {S60344, 139, { &P18496,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LVW Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109366 = {S60345, 139, { &P18496,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LRA Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126958 = {S60346, 139, { &P18496,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR6410LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA924959 = {S60347, 107, { &P15364,&P6596,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2650.0.0) AppleWebKit/537.2 (KHTML, like Gecko) Chrome/22.0.1215.0 Safari/537.2*/
const Device UA925081 = {S60348, 107, { &P15364,&P6596,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2628.0.0) AppleWebKit/537.2 (KHTML, like Gecko) Chrome/22.0.1210.0 Safari/537.2*/
const Device UA1996213 = {S60349, 109, { &P15364,&P6596,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2628.0.0) AppleWebKit/537.2 (KHTML, like Gecko) Chrome/22.0.1210.0 Safari/537.2*/
const Device UA1997125 = {S60350, 107, { &P15364,&P6596,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2694.0.0) AppleWebKit/537.3 (KHTML, like Gecko) Chrome/22.0.1222.0 Safari/537.3*/
const Device UA2001935 = {S60351, 110, { &P15364,&P6596,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2723.18.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.8 Safari/537.4*/
const Device UA2003607 = {S60352, 109, { &P15364,&P6596,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2723.25.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.12 Safari/537.4*/
const Device UA2003723 = {S60353, 108, { &P15364,&P6596,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2723.18.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.8 Safari/537.4*/
const Device UA2005587 = {S60354, 109, { &P15364,&P6596,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2723.0.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.0 Safari/537.4*/
const Device UA2010802 = {S60355, 107, { &P15364,&P6596,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2723.0.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.0 Safari/537.4*/
const Device UA2014090 = {S60356, 111, { &P15364,&P6596,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2723.37.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.21 Safari/537.4*/
const Device UA2014085 = {S60357, 109, { &P15364,&P6596,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2723.37.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.21 Safari/537.4*/
const Device UA2016058 = {S60358, 109, { &P15364,&P6596,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2723.50.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.26 Safari/537.4*/
const Device UA2040842 = {S60359, 109, { &P15364,&P6596,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2723.50.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.26 Safari/537.4*/
const Device UA2040843 = {S60360, 111, { &P15364,&P6596,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2723.50.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.26 Safari/537.4*/
const Device UA2041406 = {S60361, 111, { &P15364,&P6596,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2723.50.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.26 Safari/537.4*/
const Device UA2042076 = {S60362, 109, { &P15364,&P6596,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2723.52.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.26 Safari/537.4*/
const Device UA2043391 = {S60363, 111, { &P15364,&P6596,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2723.63.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.33 Safari/537.4*/
const Device UA2043591 = {S60364, 109, { &P15364,&P6596,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2723.63.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.33 Safari/537.4*/
const Device UA2810428 = {S60365, 110, { &P15364,&P6596,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2556.0.2012) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/22.0.1198.0 Safari/537.1*/
const Device UA2570956 = {S60366, 124, { &P17595,&P18120,&P20429,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Google Earth/7.0.3.8542 Safari/533.3*/
const Device UA2753199 = {S60367, 138, { &P19777,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754249 = {S60368, 137, { &P19777,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA637227 = {S60369, 112, { &P15241,&P5843,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia300/07.03; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.9*/
const Device UA2468133 = {S60370, 146, { &P15869,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Xoom Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2562369 = {S60371, 145, { &P15869,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Xoom Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2661702 = {S60372, 146, { &P15869,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Xoom Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2668939 = {S60373, 152, { &P15869,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Xoom Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2715471 = {S60374, 146, { &P15869,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Xoom Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA23983 = {S60375, 116, { &P15605,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2230/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA2144826 = {S60376, 75, { &P15605,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2230/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2154913 = {S60377, 117, { &P15605,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2230L/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA2342611 = {S60378, 135, { &P21306,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MOMO11Speed Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2695764 = {S60379, 64, { &P21306,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; MOMO11Speed Build/JRO03H)*/
const Device UA588758 = {S60380, 135, { &P20593,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CUBE U9GT 2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2024047 = {S60381, 129, { &P20593,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; U9GT2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029272 = {S60382, 64, { &P20593,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; CUBE U9GT 2 Build/IML74K)*/
const Device UA2031776 = {S60383, 155, { &P20593,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; CUBE_U9GT_2 Build/IML74K; 768*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA3067587 = {S60384, 145, { &P19891,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA624713 = {S60385, 101, { &P15364,&P22192,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);nl-NL;kml:2.2;client:Free;type:default)*/
const Device UA632799 = {S60386, 98, { &P15364,&P22192,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);de;kml:2.2;client:Free;type:default)*/
const Device UA636832 = {S60387, 101, { &P15364,&P22192,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);en-US;kml:2.2;client:Free;type:default)*/
const Device UA651656 = {S60388, 98, { &P15364,&P22192,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);en;kml:2.2;client:Free;type:default)*/
const Device UA666808 = {S60389, 101, { &P15364,&P22192,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6001.1);en-US;kml:2.2;client:Free;type:default)*/
const Device UA683895 = {S60390, 101, { &P15364,&P22192,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);en-GB;kml:2.2;client:Free;type:default)*/
const Device UA695766 = {S60391, 101, { &P15364,&P22192,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);de-DE;kml:2.2;client:Free;type:default)*/
const Device UA708802 = {S60392, 101, { &P15364,&P22192,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);it-IT;kml:2.2;client:Free;type:default)*/
const Device UA724685 = {S60393, 98, { &P15364,&P22192,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6001.1);en;kml:2.2;client:Free;type:default)*/
const Device UA854854 = {S60394, 101, { &P15364,&P22192,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);fr-FR;kml:2.2;client:Free;type:default)*/
const Device UA1915881 = {S60395, 101, { &P15364,&P22192,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);en-CA;kml:2.2;client:Free;type:default)*/
const Device UA1935063 = {S60396, 98, { &P15364,&P22192,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);nl;kml:2.2;client:Free;type:default)*/
const Device UA2148136 = {S60397, 98, { &P15364,&P22192,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);fr;kml:2.2;client:Free;type:default)*/
const Device UA2148571 = {S60398, 101, { &P15364,&P22192,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);en-AU;kml:2.2;client:Free;type:default)*/
const Device UA2215656 = {S60399, 101, { &P15364,&P22192,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Windows;Microsoft Windows (6.0.6002.2);pt-BR;kml:2.2;client:Free;type:default)*/
const Device UA3083425 = {S60400, 112, { &P18648,&P5043,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia306/03.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA2203512 = {S60401, 56, { &P15364,&P16110,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; (Build 9200)) AppleWebKit/536.26.9*/
const Device UA2204320 = {S60402, 56, { &P15364,&P16110,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; (Build 8400)) AppleWebKit/536.26.9*/
const Device UA2234078 = {S60403, 56, { &P15364,&P16110,&P18333,&P18092 }};	/*Useragent - iTunes/10.7 (Windows; (Build 8250)) AppleWebKit/536.26.9*/
const Device UA2562712 = {S60404, 99, { &P15364,&P16110,&P18333,&P18092 }};	/*Useragent - iTunes/11.0.2 (Windows; Microsoft Windows 8 x64 Business Edition (Build 9200)) AppleWebKit/536.27.1*/
const Device UA2562773 = {S60405, 103, { &P15364,&P16110,&P18333,&P18092 }};	/*Useragent - iTunes/11.0.2 (Windows; Microsoft Windows 8 x64 Home Premium Edition (Build 9200)) AppleWebKit/536.27.1*/
const Device UA2703013 = {S60406, 95, { &P15364,&P16110,&P18333,&P18092 }};	/*Useragent - iTunes/11.0.2 (Windows; Microsoft Windows 8 Business Edition (Build 9200)) AppleWebKit/536.27.1*/
const Device UA681490 = {S60407, 174, { &P15778,&P17486,&P18373,&P18092 }};	/*Useragent - MOT-EX225 MIDP-2.0/CLDC-1.1 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA2229913 = {S60408, 156, { &P14017,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LG-P990 Build/GRJ22; CyanogenMod-9.9.9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2356632 = {S60409, 153, { &P14017,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P990 Build/GRJ22; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558222 = {S60410, 158, { &P14017,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P990 Build/IMM76L; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3068328 = {S60411, 141, { &P15802,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC_Amaze_4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081038 = {S60412, 141, { &P15802,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Amaze 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081389 = {S60413, 137, { &P15802,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Ruby Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068252 = {S60414, 136, { &P22685,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E460 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094932 = {S60415, 137, { &P22685,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E460f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2527089 = {S60416, 140, { &P21155,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; Lenovo A800; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2656216 = {S60417, 142, { &P21155,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lenovo A800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2656281 = {S60418, 64, { &P21155,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo A800 Build/IMM76D)*/
const Device UA2956132 = {S60419, 142, { &P21155,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lenovo A800 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA31162 = {S60420, 78, { &P15437,&P17484,&P4628,&P18092 }};	/*Useragent - Samsung-SPHM540BST Polaris/6.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63077 = {S60421, 75, { &P15437,&P17484,&P4628,&P18092 }};	/*Useragent - Samsung-SPHM540 Polaris/6.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88996 = {S60422, 141, { &P15365,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; HTC Magic Build/DRC92) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA460274 = {S60423, 141, { &P15365,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; zh-cn; HTC Magic Build/DRC83) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA477974 = {S60424, 141, { &P15365,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; HTC Magic Build/DRD35) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2502861 = {S60425, 128, { &P18733,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; M532 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2985832 = {S60426, 57, { &P18733,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; M532 Build/JRO03C)*/
const Device UA2567215 = {S60427, 146, { &P12972,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T989 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583781 = {S60428, 138, { &P12972,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2600975 = {S60429, 139, { &P12972,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T989 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2707678 = {S60430, 131, { &P12972,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2738316 = {S60431, 139, { &P12972,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989D Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA454935 = {S60432, 161, { &P13991,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6400L Build/FRG83D; ThunderShed-v1.5_CM7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA628934 = {S60433, 156, { &P13991,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6400L Build/FRG83D; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA674641 = {S60434, 155, { &P13991,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6400L Build/GRJ22; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA901453 = {S60435, 160, { &P13991,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6400L Build/GRJ22; ThunderShed-v1.6_CM7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368808 = {S60436, 142, { &P18019,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; IM-A730S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1951463 = {S60437, 140, { &P18019,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; IM-A730S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021716 = {S60438, 66, { &P18019,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; IM-A730S Build/GINGERBREAD)*/
const Device UA2765102 = {S60439, 153, { &P18019,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7 (link_ke); vi-vn; IM-A730S Build/Vegaviet.com) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2952420 = {S60440, 151, { &P18019,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7 (link_ke); en-; IM-A730S Build/Vegaviet.com) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2042070 = {S60441, 152, { &P18471,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT907 Build/7.7.1Q-144_SMQ_S136-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2192096 = {S60442, 74, { &P18471,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; XT907 Build/7.7.1Q-144_SMQ_S136-10)*/
const Device UA2352377 = {S60443, 81, { &P18471,&P18066,&P18062,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android 4.0.4;motorola XT907 Build/7.7.1Q-144_SMQ_S136-10)*/
const Device UA2404142 = {S60444, 145, { &P18471,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; XT905 Build/7.7.1Q_GCIRD-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3043545 = {S60445, 67, { &P18471,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; XT905 Build/7.7.1Q_GCIRD-16)*/
const Device UA3039427 = {S60446, 156, { &P18673,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S7562 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3107987 = {S60447, 157, { &P18673,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-S7562i Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3068218 = {S60448, 136, { &P18438,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E973 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081186 = {S60449, 137, { &P18438,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F200K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081687 = {S60450, 137, { &P18438,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F200L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083297 = {S60451, 137, { &P18438,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F200S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2572106 = {S60452, 138, { &P21525,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SBM203SH Build/S0012) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2606185 = {S60453, 138, { &P21525,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SBM203SH Build/S0018) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2672157 = {S60454, 60, { &P21525,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SBM203SH Build/S0018)*/
const Device UA2970155 = {S60455, 138, { &P21525,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SBM203SH Build/S0022) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3128716 = {S60456, 138, { &P21525,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SBM203SH Build/S0024) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3096245 = {S60457, 112, { &P15785,&P5843,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia201/11.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA2434135 = {S60458, 138, { &P22678,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MAXWEST Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2702589 = {S60459, 160, { &P22678,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MAXWEST Build/MAXWEST-V03.00.00_2013.01.05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752332 = {S60460, 138, { &P22678,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MAXWEST Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957220 = {S60461, 160, { &P22678,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MAXWEST Build/MAXWEST-V03.00.01_2013.03.15) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455726 = {S60462, 133, { &P20004,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; A2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555734 = {S60463, 148, { &P20004,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; Cross A2 Build/ICS_IMM76D_B156) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2716520 = {S60464, 142, { &P20004,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Karbonn A2+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2762789 = {S60465, 55, { &P20004,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; A2 Build/IMM76I)*/
const Device UA68043 = {S60466, 130, { &P14903,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-gb; LG-V900 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA558250 = {S60467, 135, { &P14903,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; de-de; LG-V900/v10d Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1904594 = {S60468, 130, { &P14903,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; tr-tr; LG-V901 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2019387 = {S60469, 59, { &P14903,&P6545,&P4755,&P18092 }};	/*Useragent - Dalvik/1.5.0 (Linux; U; Android 3.0.1; LG-V900 Build/HRI66)*/
const Device UA152309 = {S60470, 58, { &P15364,&P6704,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows 98; U; en) Presto/2.6.30 Version/10.63*/
const Device UA284300 = {S60471, 58, { &P15364,&P6704,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows 98; U; en) Presto/2.6.30 Version/10.62*/
const Device UA543815 = {S60472, 58, { &P15364,&P6704,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows 98; U; de) Presto/2.6.30 Version/10.61*/
const Device UA2431067 = {S60473, 171, { &P15991,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2671268 = {S60474, 112, { &P21003,&P20706,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia310/07.35; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA452095 = {S60475, 132, { &P20249,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; HTC PG09410 Build/HMJ15) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA571886 = {S60476, 142, { &P20249,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us;HTC_PG09410/1.17.502.3 Build/HMJ15) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA672903 = {S60477, 142, { &P20249,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us;HTC_PG09410/1.30.502.1 Build/HMJ15) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA711235 = {S60478, 130, { &P20249,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-ca; HTC-P715a Build/HMJ15) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA765684 = {S60479, 129, { &P20249,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-ca;HTC-P715a Build/HMJ15) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2540556 = {S60480, 59, { &P20249,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; HTC-P715a Build/HMJ15)*/
const Device UA2540562 = {S60481, 55, { &P20249,&P6546,&P4459,&P18092 }};	/*Useragent - Android/4.6.1 (Linux; U; Android 3.1; en_CA; HTC-P715a)*/
const Device UA2806680 = {S60482, 61, { &P20249,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; HTC PG09410 Build/HMJ15)*/
const Device UA2567016 = {S60483, 140, { &P15972,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One XL Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583270 = {S60484, 133, { &P15972,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One XL Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2675439 = {S60485, 91, { &P17595,&P17296,&P22225,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:20.0) Gecko/X Firefox/20.0 SeaMonkey/2.17.1*/
const Device UA2936590 = {S60486, 78, { &P17595,&P17296,&P22225,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:20.0) Gecko/X SeaMonkey/2.17.1*/
const Device UA2753208 = {S60487, 136, { &P18621,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E610 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753174 = {S60488, 136, { &P18621,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753771 = {S60489, 137, { &P18621,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E610v Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755087 = {S60490, 137, { &P18621,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755265 = {S60491, 137, { &P18621,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612f Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2940150 = {S60492, 136, { &P22126,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Torque Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA43862 = {S60493, 148, { &P13758,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Alcatel OT-980 Build/DR4) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA43863 = {S60494, 150, { &P13758,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Alcatel OT-980 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA62625 = {S60495, 151, { &P13758,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Alcatel OT-980A Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA691277 = {S60496, 97, { &P17595,&P17297,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:11.0) Gecko/X Firefox/11.0 AlexaToolbar/alxf-2.15*/
const Device UA867247 = {S60497, 87, { &P17595,&P17297,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:11.0) Gecko/X Firefox/11.0 Firefox/11.0*/
const Device UA3027198 = {S60498, 111, { &P21003,&P20706,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia310/07.35; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3065143 = {S60499, 130, { &P18437,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA624891 = {S60500, 77, { &P15364,&P18144,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; OpenBSD i386; en-US; rv:1.8.0.5) Gecko/X Firefox/1.5.0.5*/
const Device UA2487701 = {S60501, 76, { &P15364,&P17285,&P22368,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:15.4) Gecko/X Firefox/15.4.1 PaleMoon/15.4.1*/
const Device UA2508298 = {S60502, 96, { &P15364,&P17285,&P22368,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:15.4) Gecko/X Firefox/15.4.1-x64 PaleMoon/15.4.1-x64*/
const Device UA22881 = {S60503, 135, { &P13241,&P5033,&P3417,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320) BlackBerry8830/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105*/
const Device UA22885 = {S60504, 73, { &P13241,&P5033,&P3417,&P18092 }};	/*Useragent - BlackBerry8830/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105*/
const Device UA36225 = {S60505, 135, { &P13241,&P5033,&P3417,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320) BlackBerry8830/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/104*/
const Device UA49222 = {S60506, 115, { &P12963,&P17603,&P4392,&P18092 }};	/*Useragent - PantechP6010/JNUS06062011D BMP/1.0.2 DeviceId/141020 NetFront/4.1 OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA679177 = {S60507, 102, { &P12963,&P17603,&P4392,&P18092 }};	/*Useragent - PantechP6010/JNUSX BMP/1.0.2 DeviceId/X NetFront/4.1 OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA50314 = {S60508, 127, { &P13626,&P17603,&P4732,&P18092 }};	/*Useragent - PantechP9020/JEUS04262010; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.18216; U; en-US) Opera 9.50*/
const Device UA508886 = {S60509, 117, { &P13626,&P17603,&P4732,&P18092 }};	/*Useragent - PantechP9020/JEUSX; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.18216; U; es) Opera 9.50*/
const Device UA2197968 = {S60510, 95, { &P17595,&P18207,&P19836,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; Edition MAS; en) Presto/2.10.229 Version/11.67*/
const Device UA64189 = {S60511, 76, { &P12298,&P5208,&P4521,&P18092 }};	/*Useragent - Mozilla/4.0(compatible;Polaris 6.2;Brew 3.1.5;U;en)/400x240 Samsung SCH-U820*/
const Device UA2359564 = {S60512, 139, { &P15364,&P17820,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Ubuntu/12.04 Chromium/23.0.1271.64 Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2362339 = {S60513, 137, { &P15364,&P17820,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.11 (KHTML, like Gecko) Ubuntu/12.04 Chromium/23.0.1271.64 Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2365607 = {S60514, 133, { &P20882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; OP0118-12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2420411 = {S60515, 134, { &P20882,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en_us-; OP0118-12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2488334 = {S60516, 75, { &P17595,&P17297,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5) Presto/2.12.388 Version/12.13*/
const Device UA2749777 = {S60517, 154, { &P19771,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ALCATEL ONE TOUCH 5035A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781731 = {S60518, 154, { &P19771,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; ALCATEL ONE TOUCH 5035D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2992418 = {S60519, 154, { &P19771,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; ALCATEL ONE TOUCH 5035X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879666 = {S60520, 133, { &P23691,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; ND-8014RK Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2963421 = {S60521, 139, { &P23691,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S5280 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3056272 = {S60522, 61, { &P23691,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S5280 Build/JZO54K)*/
const Device UA2385446 = {S60523, 157, { &P21415,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Android 4.0.4; en-us; Nuqleo; Quantum 7 Build/ICS) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2427763 = {S60524, 146, { &P21415,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nuqleo; Quantum 7 Build/ICS) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA2655041 = {S60525, 133, { &P21415,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Quantum 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA5295 = {S60526, 67, { &P11643,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia6500c_CMCC/2.0 (08.20) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA21932 = {S60527, 62, { &P11643,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia6500c/2.0 (06.61) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA784750 = {S60528, 146, { &P21347,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HUAWEI U8661 Build/HuaweiU8661) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2445612 = {S60529, 139, { &P21347,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI U8661 Build/HuaweiU8661) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2787642 = {S60530, 70, { &P21347,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; HUAWEI U8661 Build/HuaweiU8661)*/
const Device UA2670752 = {S60531, 127, { &P16976,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT300 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676252 = {S60532, 129, { &P16976,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT300SE Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2699804 = {S60533, 146, { &P23255,&P18111,&P18112,&P18092 }};	/*Useragent - HUAWEI_G520-5000_TD/1.0 Android/4.1.2 (Linux; U; Android 4.1.2; zh-cn) Release/01.31.2013 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30*/
const Device UA2703074 = {S60534, 156, { &P23255,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HUAWEI G520-0000 Build/HuaweiG520-0000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2780021 = {S60535, 156, { &P23255,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI G520-5000 Build/HuaweiG520-5000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936824 = {S60536, 78, { &P23255,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HUAWEI G520-0000 Build/HuaweiG520-0000)*/
const Device UA2957017 = {S60537, 78, { &P23255,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HUAWEI G520-5000 Build/HuaweiG520-5000)*/
const Device UA3102251 = {S60538, 147, { &P23255,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI G520-0000 Build/HuaweiG520-0000) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3120530 = {S60539, 147, { &P23255,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI G520-5000 Build/HuaweiG520-5000) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2365571 = {S60540, 182, { &P14635,&P5843,&P24671,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (08.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac3-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2487665 = {S60541, 156, { &P14017,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; LG-P990 Build/IMM76L; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567442 = {S60542, 156, { &P14017,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-P990 Build/IMM76L; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA396306 = {S60543, 97, { &P18036,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; MWP6885)*/
const Device UA910814 = {S60544, 136, { &P21006,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; L-05D Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031145 = {S60545, 58, { &P21006,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; L-05D Build/IMM76I)*/
const Device UA87387 = {S60546, 145, { &P17346,&P6274,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 2_1_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5F138 Safari/525.20*/
const Device UA165485 = {S60547, 143, { &P17346,&P6274,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5F137 Safari/525.20*/
const Device UA318135 = {S60548, 149, { &P15364,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Htcclay's SuperCM7 G2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA859190 = {S60549, 157, { &P15364,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Inspire 4g Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1948306 = {S60550, 156, { &P15364,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Liquid Mt Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1960652 = {S60551, 164, { &P15364,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Precedent Build/GINGERBREAD; CM7.1.0-PrecedentR1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1978571 = {S60552, 177, { &P15364,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7 Beta 5 modded by cas_xp; en-us; TC970 Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019142 = {S60553, 150, { &P15364,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Htcclay's SuperCM7 NS2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022125 = {S60554, 156, { &P15364,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; fr-fr; Folio 100 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025355 = {S60555, 151, { &P15364,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Crescent Build/GRJ22; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027244 = {S60556, 146, { &P15364,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; u8510 Build/ICS CM7 - teammat3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027878 = {S60557, 150, { &P15364,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; M865 Build/GINGERBREAD; CM7.2-M865) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033986 = {S60558, 155, { &P15364,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; Swift Me Build/GRJ90; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2443362 = {S60559, 177, { &P15364,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7 Beta 4 modded by cas_xp; en-us; TC970 Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2477060 = {S60560, 149, { &P15364,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; TC970 Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2549692 = {S60561, 153, { &P15364,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; a107 Build/GINGERBREAD; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2641122 = {S60562, 161, { &P15364,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; tr-tr; Perfect Rom v4 Build/FRG83; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2788249 = {S60563, 176, { &P15364,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7 Xperia Edition v.2; tr-tr; Xperia 945 Build/FRG83; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2790366 = {S60564, 154, { &P15364,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; IdeaPad A1 Build/GRK393; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2809472 = {S60565, 168, { &P15364,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; Geeksphone ZERO Build/FRG83; ZerO_CR 3.0 CyanogenMod) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2857448 = {S60566, 157, { &P15364,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7+; es-es; Nexus One+ Build/GRK39F; CyanogenMod-7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3097923 = {S60567, 172, { &P15364,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; LG-SU660 Build/GRJ22; CyanogenMod-7; LG-P990 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3099338 = {S60568, 156, { &P15364,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; m865 Build/Ultimate CM7 V2; Ultiamte CM7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA600190 = {S60569, 138, { &P18677,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; SBM104SH Build/S0024) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA603596 = {S60570, 138, { &P18677,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; SBM104SH Build/S0026) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA661597 = {S60571, 138, { &P18677,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; SBM104SH Build/S0018) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA905439 = {S60572, 138, { &P18677,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; SBM104SH Build/S0028) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2199523 = {S60573, 138, { &P18677,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; SBM104SH Build/S0030) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2366387 = {S60574, 60, { &P18677,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SBM104SH Build/S0030)*/
const Device UA2532294 = {S60575, 138, { &P18677,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; SBM104SH Build/S0032) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2881988 = {S60576, 138, { &P18677,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; SBM104SH Build/S0034) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3080145 = {S60577, 138, { &P18677,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; SBM104SH Build/S0038) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2315524 = {S60578, 233, { &P12371,&P5062,&P3441,&P18168 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; SonyEricssonSO-01B Build/R1EA018) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 (compatible; ichiro/mobile goo;+http://search.goo.ne.jp/option/use/sub4/sub4-1/)*/
const Device UA2391587 = {S60579, 129, { &P18464,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2482766 = {S60580, 58, { &P18464,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; KFJWA Build/IMM76D)*/
const Device UA2781847 = {S60581, 71, { &P17595,&P22878,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (iMac12%2C2)*/
const Device UA2782138 = {S60582, 77, { &P17595,&P22878,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro10%2C1)*/
const Device UA2784473 = {S60583, 76, { &P17595,&P22878,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookAir5%2C2)*/
const Device UA2800129 = {S60584, 73, { &P17595,&P22878,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBook5%2C1)*/
const Device UA2866270 = {S60585, 72, { &P17595,&P22878,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacPro3%2C1)*/
const Device UA2938351 = {S60586, 145, { &P24048,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-tw; HTC_Butterfly_S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2965726 = {S60587, 67, { &P24048,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC Butterfly s Build/JDQ39)*/
const Device UA2977072 = {S60588, 145, { &P24048,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-tw; HTC Butterfly s Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022851 = {S60589, 145, { &P24048,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-sg; HTC_Butterfly_s Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA209083 = {S60590, 135, { &P14877,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; GT-I9000 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA251950 = {S60591, 135, { &P14877,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-I9000 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA318336 = {S60592, 135, { &P14877,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT-I9000 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369612 = {S60593, 137, { &P14877,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-I9000 Build/MIUIAR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA637919 = {S60594, 59, { &P14877,&P24694,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; GT-I9000 Build/MIUI)*/
const Device UA2019433 = {S60595, 59, { &P14877,&P24694,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; GT-I9000 Build/MIUI)*/
const Device UA2023298 = {S60596, 123, { &P14877,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-I9000 Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2026043 = {S60597, 135, { &P14877,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.9; en-us; GT-I9000 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026081 = {S60598, 128, { &P14877,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-I9000 Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2667859 = {S60599, 135, { &P15184,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-P920 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936153 = {S60600, 57, { &P15184,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P920 Build/ICS)*/
const Device UA2676435 = {S60601, 133, { &P18466,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734741 = {S60602, 126, { &P18466,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA50353 = {S60603, 139, { &P17192,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; Archos5 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2755348 = {S60604, 155, { &P18645,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2756701 = {S60605, 147, { &P18645,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2867679 = {S60606, 148, { &P18645,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-I747M Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2877927 = {S60607, 148, { &P18645,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530U Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2892379 = {S60608, 148, { &P18645,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-I9305T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2892892 = {S60609, 147, { &P18645,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9305 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2943380 = {S60610, 148, { &P18645,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I747M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3000344 = {S60611, 155, { &P18645,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2974357 = {S60612, 145, { &P18626,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I547 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975972 = {S60613, 138, { &P18626,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I547C Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021781 = {S60614, 145, { &P18626,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I547 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA909100 = {S60615, 129, { &P18725,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AN8G3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2715180 = {S60616, 150, { &P18552,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2738191 = {S60617, 143, { &P18552,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA64350 = {S60618, 123, { &P14592,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia3720c/2.0 (10.16) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA2400900 = {S60619, 142, { &P19759,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lenovo K860 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2418599 = {S60620, 133, { &P19759,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo K860 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2593949 = {S60621, 64, { &P19759,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo K860 Build/IMM76D)*/
const Device UA2594796 = {S60622, 143, { &P19759,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo K860i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2745324 = {S60623, 128, { &P19759,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo K860 Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2598639 = {S60624, 103, { &P14748,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae72-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2717070 = {S60625, 94, { &P14748,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE72-1) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2868607 = {S60626, 94, { &P14748,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE72-2) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2993754 = {S60627, 32, { &P14748,&P5890,&P24676,&P18092 }};	/*Useragent - NokiaE72-1/UCWEB8.9.0.253/28/999*/
const Device UA156749 = {S60628, 127, { &P15767,&P17145,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Twitter for iPad*/
const Device UA488837 = {S60629, 126, { &P15767,&P17145,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C134b Twitter for iPad*/
const Device UA1850482 = {S60630, 125, { &P15767,&P17145,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C134 Twitter for iPad*/
const Device UA2800146 = {S60631, 58, { &P15364,&P7127,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:22.0) Gecko/X Firefox/22.0*/
const Device UA13721 = {S60632, 66, { &P11769,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia3600slide/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA54626 = {S60633, 62, { &P11769,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia3600s/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA910213 = {S60634, 136, { &P18655,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-09D Build/S6130) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996485 = {S60635, 136, { &P18655,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-09D Build/S7090) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2200310 = {S60636, 136, { &P18655,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-09D Build/S9070) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2437283 = {S60637, 136, { &P18655,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-09D Build/SB220) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2716819 = {S60638, 136, { &P18655,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-09D Build/S4060) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2837147 = {S60639, 58, { &P18655,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SH-09D Build/S9070)*/
const Device UA2508010 = {S60640, 93, { &P13123,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5360) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2764684 = {S60641, 93, { &P13123,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; SCH-i509) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3119717 = {S60642, 94, { &P13123,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5360B) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2755356 = {S60643, 136, { &P12788,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PG86100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782904 = {S60644, 136, { &P12788,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ISW12HT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893002 = {S60645, 136, { &P12788,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PG86100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2772605 = {S60646, 112, { &P23005,&P18110,&P17250,&P18092 }};	/*Useragent - TBDC1093 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2975034 = {S60647, 136, { &P14574,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus S Build/JRO03E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975547 = {S60648, 136, { &P14574,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3065476 = {S60649, 168, { &P15794,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081473 = {S60650, 168, { &P15794,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315b Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2182343 = {S60651, 131, { &P21825,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; fusion5 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2469214 = {S60652, 132, { &P21825,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; fusion 5 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2506807 = {S60653, 131, { &P21825,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; fusion5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2656575 = {S60654, 132, { &P21825,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; fusion5 Build/FR-MX68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2839712 = {S60655, 131, { &P21825,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; FUSION5 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2889584 = {S60656, 154, { &P23239,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ALCATEL ONE TOUCH 6030X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2938225 = {S60657, 161, { &P23239,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; ALCATEL_ONE_TOUCH_6030X_Orange Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2973490 = {S60658, 168, { &P15794,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974064 = {S60659, 168, { &P15794,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315b Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA251052 = {S60660, 139, { &P16044,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HP Touchpad Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA363805 = {S60661, 154, { &P16044,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Touchpad Build/GRJ22; XRONified-Odexed) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA677621 = {S60662, 60, { &P16044,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; Touchpad Build/GRJ22)*/
const Device UA1903943 = {S60663, 63, { &P16044,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; HP Touchpad Build/GRJ22)*/
const Device UA1986830 = {S60664, 117, { &P14708,&P5204,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia5130c-2/07.95; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA2445643 = {S60665, 114, { &P14708,&P5204,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia5130/07.97; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA2031051 = {S60666, 136, { &P18663,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-10D Build/S8130) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2206198 = {S60667, 136, { &P18663,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-10D Build/S9130) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2390390 = {S60668, 136, { &P18663,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-10D Build/SB010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2601667 = {S60669, 136, { &P18663,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-10D Build/S3070) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2673233 = {S60670, 136, { &P18663,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-10D Build/S4040) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2714636 = {S60671, 136, { &P18663,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-10D Build/S4180) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2508777 = {S60672, 93, { &P13324,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaN8-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2425416 = {S60673, 101, { &P12141,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.67; en-us; 8900/5.2.0.67) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2466392 = {S60674, 103, { &P12141,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.104; en-us; 8900/5.2.0.104) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2482541 = {S60675, 101, { &P12141,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.31; en-us; 8900/5.2.0.31) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2503081 = {S60676, 101, { &P12141,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.96; en-us; 8900/5.2.0.96) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2578785 = {S60677, 103, { &P12141,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.102; en-us; 8900/4.2.0.102) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2594904 = {S60678, 101, { &P12141,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.76; en-us; 8900/5.2.0.76) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2595395 = {S60679, 103, { &P12141,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.127; en-us; 8900/4.2.0.127) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2694929 = {S60680, 101, { &P12141,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.58; en-us; 8900/5.2.0.58) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2695414 = {S60681, 101, { &P12141,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.94; en-us; 8900/4.2.0.94) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2763172 = {S60682, 101, { &P12141,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.47; en-us; 8900/5.2.0.47) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2844825 = {S60683, 103, { &P12141,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.113; en-us; 8900/4.2.0.113) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2848025 = {S60684, 103, { &P12141,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.111; en-us; 8900/5.2.0.111) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2983591 = {S60685, 103, { &P12141,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.108; en-us; 8900/4.2.0.108) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3099541 = {S60686, 103, { &P12141,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.102; en-us; 8900/5.2.0.102) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3066528 = {S60687, 137, { &P24097,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919N Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA1994663 = {S60688, 139, { &P17595,&P18207,&P24518,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11*/
const Device UA2179812 = {S60689, 139, { &P17595,&P18207,&P24518,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11*/
const Device UA2202648 = {S60690, 139, { &P17595,&P18207,&P24518,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11*/
const Device UA2585258 = {S60691, 139, { &P17595,&P18207,&P24518,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11*/
const Device UA2795532 = {S60692, 139, { &P17595,&P18207,&P24518,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11*/
const Device UA3093127 = {S60693, 139, { &P17595,&P18207,&P24518,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11*/
const Device UA2506418 = {S60694, 142, { &P18662,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HTC_J_Z321e Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558217 = {S60695, 142, { &P18662,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HTC J Z321e Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2559257 = {S60696, 64, { &P18662,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC J Z321e Build/JRO03C)*/
const Device UA429278 = {S60697, 131, { &P15254,&P6374,&P4418,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9981; en-GB) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.579 Mobile Safari/534.11+*/
const Device UA640798 = {S60698, 137, { &P20597,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; LG-F100S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2642352 = {S60699, 61, { &P20597,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-F100S Build/GRK39F)*/
const Device UA2753976 = {S60700, 137, { &P19894,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA401598 = {S60701, 73, { &P16329,&P18203,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PLAYSTATION 3 4.11) AppleWebKit/531.22.8 (KHTML, like Gecko)*/
const Device UA2676098 = {S60702, 130, { &P15932,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3110 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676942 = {S60703, 130, { &P15932,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P3110 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3083962 = {S60704, 86, { &P17968,&P17979,&P18415,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/7.8.33965/31.1386; U; zh) Presto/2.8.119 Version/11.10*/
const Device UA609329 = {S60705, 138, { &P14574,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  Nexus S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA609907 = {S60706, 138, { &P14574,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  Nexus S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA748228 = {S60707, 137, { &P14574,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Nexus S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA901499 = {S60708, 137, { &P14574,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Nexus S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA910417 = {S60709, 136, { &P14574,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Crespo Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2444625 = {S60710, 136, { &P14574,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.1; Nexus S Build/GWK74) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2444905 = {S60711, 152, { &P14574,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Full Android on Crespo Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA630179 = {S60712, 58, { &P15364,&P17188,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0; rv:11.0) Gecko/X Firefox/11.0*/
const Device UA2194217 = {S60713, 62, { &P15364,&P18146,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; SunOS i86pc; rv:15.0) Gecko/X Firefox/15.0.1*/
const Device UA2566687 = {S60714, 127, { &P19775,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A210 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2676926 = {S60715, 152, { &P15877,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3096278 = {S60716, 157, { &P18442,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3096401 = {S60717, 157, { &P18442,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3109472 = {S60718, 157, { &P18442,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3110451 = {S60719, 165, { &P18442,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3114507 = {S60720, 158, { &P18442,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I317M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3115339 = {S60721, 157, { &P18442,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T889 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3115428 = {S60722, 157, { &P18442,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L900 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA9491 = {S60723, 122, { &P14916,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-C3303/C3303DDJG4 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA33434 = {S60724, 116, { &P14916,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-C3303i/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA565505 = {S60725, 124, { &P14916,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-C3303K/C3303KJPKA1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA52754 = {S60726, 141, { &P12528,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update2; tr-tr; U8100 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA55200 = {S60727, 149, { &P12528,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; Huawei_8100-9 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA61735 = {S60728, 141, { &P12528,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; U8100 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA731309 = {S60729, 63, { &P12528,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; U8100 Build/ERE27)*/
const Device UA728733 = {S60730, 161, { &P14877,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-I9000 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3021137 = {S60731, 152, { &P18506,&P5043,&P24679,&P18092 }};	/*Useragent - Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia305) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA24266 = {S60732, 74, { &P12092,&P5208,&P3499,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Teleca Q7; Brew 3.1.5; U; en) 480X800 LGE VX11000*/
const Device UA28090 = {S60733, 73, { &P12092,&P5208,&P3499,&P18092 }};	/*Useragent - Mozilla/5.0(compatible; Teleca Q7; Brew 3.1.5; U; en) 480X800 LGE VX11000*/
const Device UA300701 = {S60734, 62, { &P15364,&P7133,&P20411,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.2.15 Version/10.20*/
const Device UA2369270 = {S60735, 41, { &P15364,&P7133,&P20411,&P18092 }};	/*Useragent - Opera/10.2 (Windows NT 6.1; WOW64; U; en)*/
const Device UA60918 = {S60736, 83, { &P13183,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK810a/R1KC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64797 = {S60737, 83, { &P13183,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK810i/R8BA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA253829 = {S60738, 83, { &P13183,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK810c/R1KC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA90751 = {S60739, 136, { &P18071,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM006SH Build/S0012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA221987 = {S60740, 136, { &P18071,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM006SH Build/S0018) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA236746 = {S60741, 136, { &P18071,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM006SH Build/S0026) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA251385 = {S60742, 136, { &P18071,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM006SH Build/S0038) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA901189 = {S60743, 136, { &P18071,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM006SH Build/S0042) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029246 = {S60744, 60, { &P18071,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SBM006SH Build/S0042)*/
const Device UA2488180 = {S60745, 136, { &P18071,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM006SH Build/S0044) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2879707 = {S60746, 136, { &P18071,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM006SH Build/S0064) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA851284 = {S60747, 64, { &P17976,&P17979,&P18628,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:14.0) Gecko/14.0 Firefox/14.0a2*/
const Device UA893053 = {S60748, 62, { &P17976,&P17979,&P18628,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:14.0) Gecko/14.0 Firefox/14.0*/
const Device UA901736 = {S60749, 74, { &P17976,&P17979,&P18628,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:14.0) Gecko/14.0 Firefox/14.0 Fennec/14.0*/
const Device UA924846 = {S60750, 64, { &P17976,&P17979,&P18628,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:14.0) Gecko/14.0 Firefox/14.0.1*/
const Device UA1956050 = {S60751, 61, { &P17976,&P17979,&P18628,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:14.0) Gecko/14.0 Fennec/14.0*/
const Device UA2028885 = {S60752, 64, { &P17976,&P17979,&P18628,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:14.0) Gecko/14.0 Firefox/14.0a1*/
const Device UA2363132 = {S60753, 113, { &P18502,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Windows Phone 8X by HTC)*/
const Device UA2382122 = {S60754, 97, { &P18502,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; PM23300)*/
const Device UA2666082 = {S60755, 126, { &P18502,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Windows Phone 8X by HTC; 2.00.161.04)*/
const Device UA2698422 = {S60756, 121, { &P18502,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Windows Phone 8X by HTC; Orange)*/
const Device UA2794043 = {S60757, 126, { &P18502,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Windows Phone 8X by HTC; 2.01.161.00)*/
const Device UA3126000 = {S60758, 126, { &P18502,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Windows Phone 8X by HTC; 3.41.161.02)*/
const Device UA2457229 = {S60759, 132, { &P20241,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; PMP5570C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2427049 = {S60760, 140, { &P19858,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; KYL21 Build/015.3.3510) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2427455 = {S60761, 140, { &P19858,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; KYL21 Build/017.0.3a30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469528 = {S60762, 140, { &P19858,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; KYL21 Build/018.0.3a90) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2677873 = {S60763, 140, { &P19858,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; KYL21 Build/019.0.3b00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3072953 = {S60764, 62, { &P19858,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; KYL21 Build/019.0.3b00)*/
const Device UA2708141 = {S60765, 141, { &P22622,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Micromax A72 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3102717 = {S60766, 65, { &P22622,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; Micromax A72 Build/GRK39F)*/
const Device UA2027739 = {S60767, 137, { &P18234,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P895 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2452360 = {S60768, 139, { &P18234,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P895qb Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2753546 = {S60769, 137, { &P18636,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; VS930 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2973897 = {S60770, 130, { &P18437,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023544 = {S60771, 130, { &P18437,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8005 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA50277 = {S60772, 127, { &P13213,&P17603,&P4732,&P18092 }};	/*Useragent - PantechP2000/JGUS03292010; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.16479; U; en-US) Opera 9.50*/
const Device UA547731 = {S60773, 117, { &P13213,&P17603,&P4732,&P18092 }};	/*Useragent - PantechP2000/JGUSX; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.16479; U; es) Opera 9.50*/
const Device UA1958000 = {S60774, 146, { &P14862,&P5204,&P24667,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia2700c-2) UCBrowser8.2.0.132/69/452/UCWEB Mobile*/
const Device UA2566779 = {S60775, 153, { &P18280,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One S Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768889 = {S60776, 156, { &P18280,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One S Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3764 = {S60777, 71, { &P11667,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia6085/2.0 (05.03) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6085*/
const Device UA3783 = {S60778, 61, { &P11667,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia6085/2.0 (06.00) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA69361 = {S60779, 30, { &P14188,&P5129,&P24663,&P18092 }};	/*Useragent - Nokia5233/UCWEB7.9.0.94/50/999*/
const Device UA248910 = {S60780, 42, { &P14188,&P5129,&P24663,&P18092 }};	/*Useragent - Nokia5233/UC Browser7.9.0.102/50/355/UCWEB*/
const Device UA3026118 = {S60781, 105, { &P20109,&P17484,&P17373,&P18092 }};	/*Useragent - SAMSUNG-GT-S3572 Opera/9.80 (J2ME/MIDP; Opera Mini/6.5.30242/30.3593; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2754477 = {S60782, 137, { &P18458,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2028437 = {S60783, 150, { &P21240,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-style Q2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029061 = {S60784, 150, { &P21240,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i-mobile i-STYLE Q2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2231310 = {S60785, 154, { &P21240,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-STYLE Q2 DUO Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2777321 = {S60786, 76, { &P21240,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; i-mobile i-STYLE Q2 DUO Build/IMM76D)*/
const Device UA2973895 = {S60787, 145, { &P12657,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515m Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974066 = {S60788, 145, { &P12657,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021720 = {S60789, 138, { &P12657,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC_X515E Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3065431 = {S60790, 138, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SMP45-210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3083477 = {S60791, 141, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N8000_USA_RS Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093546 = {S60792, 136, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAP5430 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093982 = {S60793, 133, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094155 = {S60794, 142, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tmn smart a18 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094714 = {S60795, 141, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BASE_Lutea_3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3095259 = {S60796, 136, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A11Note Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109121 = {S60797, 132, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Pad Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109304 = {S60798, 129, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAD1001 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3109931 = {S60799, 138, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V8200plus Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109942 = {S60800, 138, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SMP53-210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111034 = {S60801, 143, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; INFINITY_POWER Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111145 = {S60802, 134, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BT210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111853 = {S60803, 126, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M980 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3114926 = {S60804, 133, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 7102 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126304 = {S60805, 133, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N900 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126329 = {S60806, 135, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; cyclone Titan Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126360 = {S60807, 127, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Clear Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126403 = {S60808, 127, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TR101 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126427 = {S60809, 126, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TR71 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126574 = {S60810, 132, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A13 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126654 = {S60811, 130, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Next7D12 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126821 = {S60812, 134, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GLORY Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126824 = {S60813, 131, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 5I Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126872 = {S60814, 129, { &P15364,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TPC7102 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3131660 = {S60815, 73, { &P16329,&P24705,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PLAYSTATION 3 4.50) AppleWebKit/531.22.8 (KHTML, like Gecko)*/
const Device UA62848 = {S60816, 104, { &P11689,&P16000,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Opera 8.65; Nokia N70/5.0638.3.0.1; 9414) Opera 8.65 [en]*/
const Device UA413687 = {S60817, 92, { &P11689,&P16000,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Nokia N70/5.0741.4.0.1; 9404) Opera 8.65 [en]*/
const Device UA688426 = {S60818, 92, { &P11689,&P16000,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Nokia N70/5.0737.3.0.1; 1657) Opera 8.60 [en]*/
const Device UA2894626 = {S60819, 156, { &P18283,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9300 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2944159 = {S60820, 156, { &P18283,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-I9300 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2962568 = {S60821, 156, { &P18283,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SCH-I535 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2966348 = {S60822, 156, { &P18283,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SGH-T999 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2972111 = {S60823, 156, { &P18283,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-I939 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3005045 = {S60824, 157, { &P18283,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-I939D Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2365794 = {S60825, 149, { &P18227,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U9510E Build/HuaweiU9510E) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2430892 = {S60826, 147, { &P18227,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U9510 Build/HuaweiU9510) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2383425 = {S60827, 140, { &P19878,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I497 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA659105 = {S60828, 137, { &P12896,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LG-P999 Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA666409 = {S60829, 137, { &P12896,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA680816 = {S60830, 137, { &P12896,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA761219 = {S60831, 138, { &P12896,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA826462 = {S60832, 138, { &P12896,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LG-P999 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1995831 = {S60833, 138, { &P12896,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028875 = {S60834, 59, { &P12896,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P999 Build/GRI40)*/
const Device UA2033918 = {S60835, 59, { &P12896,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; LG-P999 Build/GRI40)*/
const Device UA2356919 = {S60836, 53, { &P18088,&P18432,&P24675,&P18092 }};	/*Useragent - IUC(U;iOS 6.0.1;Zh-cn;320*480;)/UCWEB8.8.0.212/44/997*/
const Device UA2367183 = {S60837, 53, { &P18088,&P18432,&P24675,&P18092 }};	/*Useragent - IUC(U;iOS 6.0.1;Zh-cn;320*568;)/UCWEB8.8.0.212/44/997*/
const Device UA118705 = {S60838, 56, { &P15364,&P18146,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; SunOS 5.9 sun4u; X11)*/
const Device UA2030811 = {S60839, 161, { &P11747,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC Desire Build/JRO03C; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2355162 = {S60840, 161, { &P11747,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC Desire Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2428524 = {S60841, 159, { &P11747,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC Desire Build/CM10; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580050 = {S60842, 63, { &P15364,&P5448,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; 68K; Tablet PC; Trident/2.0)*/
const Device UA2658796 = {S60843, 63, { &P15364,&P5448,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; PPC; Tablet PC; Trident/5.0)*/
const Device UA2659000 = {S60844, 58, { &P15364,&P5448,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; WindowsCE; Trident/5.0)*/
const Device UA2792340 = {S60845, 63, { &P15364,&P5448,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; 68K; Tablet PC; Trident/3.0)*/
const Device UA2792840 = {S60846, 58, { &P15364,&P5448,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; WindowsCE; Trident/4.0)*/
const Device UA2850987 = {S60847, 63, { &P15364,&P5448,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; PPC; Tablet PC; Trident/4.0)*/
const Device UA2672511 = {S60848, 139, { &P15989,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21a Build/11.0.A.0.16) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2674444 = {S60849, 139, { &P15989,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21i2 Build/11.0.A.6.5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676419 = {S60850, 139, { &P15989,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21i Build/11.0.A.4.22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735643 = {S60851, 139, { &P15989,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21a2 Build/11.0.A.1.8) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2571475 = {S60852, 141, { &P23910,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30at Build/9.1.F.1.120) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2607160 = {S60853, 134, { &P23910,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30at Build/9.1.F.1.120) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA3026168 = {S60854, 141, { &P15364,&P18110,&P18427,&P18089 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko; Google Page Speed Insights) Chrome/22.0.1229 Safari/537.4 GoogleBot/2.1*/
const Device UA3065343 = {S60855, 139, { &P23240,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; USCC-C6721 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA643771 = {S60856, 138, { &P18482,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; Micromax A75 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA844347 = {S60857, 131, { &P18482,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; A75 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029545 = {S60858, 119, { &P18482,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; A75 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2721020 = {S60859, 55, { &P18482,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; A75 Build/GRJ22)*/
const Device UA2040628 = {S60860, 83, { &P15364,&P16110,&P22578,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; Win64; x64; Edition Next) Presto/2.12.378 Version/12.50*/
const Device UA2707034 = {S60861, 156, { &P14540,&P5843,&P24677,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.71) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-01) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2032026 = {S60862, 139, { &P21098,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-ve; CM980 Build/HuaweiCM980) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2691527 = {S60863, 63, { &P21098,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; CM980 Build/HuaweiCM980)*/
const Device UA3100095 = {S60864, 144, { &P21098,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5 Plus; es-ve; CM980 Build/HuaweiCM980) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2035282 = {S60865, 140, { &P18451,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26ii Build/6.1.A.2.45) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2423225 = {S60866, 142, { &P21657,&P18066,&P18062,&P18092 }};	/*Useragent - HUAWEI_T8950_TD/1.0 Android/4.0.4 (Linux; U; Android 4.0.4; zh-cn) Release/05.31.2012 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30*/
const Device UA2475927 = {S60867, 141, { &P21657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8950D Build/HuaweiC8950D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2481744 = {S60868, 150, { &P21657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8950D Build/HuaweiC8950D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483745 = {S60869, 146, { &P21657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8950D/C8950DV100R001C92B944SP01; 540*960; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2497213 = {S60870, 139, { &P21657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8950 Build/HuaweiT8950) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2503361 = {S60871, 146, { &P21657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8950D/C8950DV100R001C92B941SP02; 540*960; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2528596 = {S60872, 72, { &P21657,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI C8950D Build/HuaweiC8950D)*/
const Device UA2528961 = {S60873, 142, { &P21657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8950D/C8950DV100R001C92B948; 540*960; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2653241 = {S60874, 148, { &P21657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8950 Build/HuaweiT8950) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2766062 = {S60875, 142, { &P21657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8950D/C8950DV100R001C92B955; 540*960; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2841415 = {S60876, 142, { &P21657,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HW-HUAWEI_C8950D/C8950DV100R001C92B941; 540*960; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA3043781 = {S60877, 70, { &P21657,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI T8950 Build/HuaweiT8950)*/
const Device UA2566504 = {S60878, 131, { &P23773,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8010 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2601291 = {S60879, 112, { &P19973,&P5043,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia114/03.33; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2676185 = {S60880, 141, { &P18462,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714701 = {S60881, 134, { &P18462,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2879895 = {S60882, 150, { &P18462,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X dual sim Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3014252 = {S60883, 151, { &P18462,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire XC dual sim Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2041281 = {S60884, 64, { &P15364,&P5690,&P18523,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:15.0) Gecko/X Thunderbird/15.0.1*/
const Device UA2205762 = {S60885, 64, { &P15364,&P5690,&P18523,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:15.0) Gecko/X Thunderbird/15.0.1*/
const Device UA2752721 = {S60886, 156, { &P23003,&P18066,&P18062,&P18092 }};	/*Useragent - TECNO Q1 Linux/3.0.13 Android/4.0.4 Release/04.13.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2767397 = {S60887, 139, { &P23003,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TECNO Q1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3047837 = {S60888, 61, { &P23003,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; TECNO Q1 Build/IMM76D)*/
const Device UA2894141 = {S60889, 152, { &P18643,&P5043,&P24678,&P18092 }};	/*Useragent - Nokia311/5.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia311) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2973677 = {S60890, 138, { &P18452,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT29i Build/9.1.B.1.67) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3068046 = {S60891, 126, { &P19862,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080707 = {S60892, 142, { &P19862,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Micromax A110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094747 = {S60893, 126, { &P19862,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA89076 = {S60894, 75, { &P17220,&P5208,&P4521,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Polaris 6.2; BREW 3.1.5; U; en)/240X400 LGE VX8575*/
const Device UA643167 = {S60895, 64, { &P15364,&P16110,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; U; en) Presto/2.10.229 Version/11.60*/
const Device UA673358 = {S60896, 115, { &P15786,&P5043,&P17438,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-05/08.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15*/
const Device UA2154765 = {S60897, 118, { &P15786,&P5043,&P17438,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-05.1/08.60b; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15*/
const Device UA3065988 = {S60898, 130, { &P20035,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094233 = {S60899, 131, { &P20035,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2407232 = {S60900, 133, { &P18464,&P18110,&P19804,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Safari/535.19 Silk-Accelerated=true*/
const Device UA2754415 = {S60901, 129, { &P17687,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MZ604 Build/I.7.1-34) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754708 = {S60902, 129, { &P17687,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ604 Build/I.7.1-45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA12154 = {S60903, 124, { &P11732,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-SGH-I637/UCID4 Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA2261609 = {S60904, 141, { &P18449,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST27i Build/6.1.1.B.1.10) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2467676 = {S60905, 141, { &P18449,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST27a Build/6.1.1.B.1.54) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2387948 = {S60906, 120, { &P15364,&P18110,&P22463,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11*/
const Device UA2388478 = {S60907, 118, { &P15364,&P18110,&P22463,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11*/
const Device UA2528126 = {S60908, 136, { &P18283,&P17979,&P18062,&P18092 }};	/*Useragent - I9300 Linux/2.6.35 Android/4.0.4 Release/10.12.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1*/
const Device UA2751594 = {S60909, 132, { &P18283,&P17979,&P18062,&P18092 }};	/*Useragent - samsung-GT-I9300/1.0 Linux/2.6.35.7 Android/4.0.4 Release/03.25.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2878308 = {S60910, 129, { &P18283,&P17979,&P18062,&P18092 }};	/*Useragent - samsung-I9300/1.0 Linux/2.6.35.7 Android/4.0.4 Release/12.25.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2892091 = {S60911, 127, { &P18283,&P17979,&P18062,&P18092 }};	/*Useragent - I9300-I9300/1.0 Linux/2.6.35.7 Android/4.0.3 Release/02.28.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2988335 = {S60912, 127, { &P18283,&P17979,&P18062,&P18092 }};	/*Useragent - I9300-I9300/1.0 Linux/2.6.35.7 Android/4.0.3 Release/01.21.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2997751 = {S60913, 124, { &P18283,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-I9300/1.0 Linux/2.6.35.7 Android/4.0 Release/04.11.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2755352 = {S60914, 152, { &P15877,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2974371 = {S60915, 136, { &P21793,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9205 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3031340 = {S60916, 158, { &P18295,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2228142 = {S60917, 142, { &P13035,&P5043,&P24668,&P18092 }};	/*Useragent - NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-02) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA349304 = {S60918, 185, { &P15364,&P5690,&P24197,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1;	Trident/4.0; SLCC2;	.NET CLR 2.0.50727; .NET CLR 3.5.30729;	.NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E) QQBrowser/6.0*/
const Device UA571505 = {S60919, 78, { &P15364,&P5690,&P24197,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) QQBrowser/6.0.8405.201*/
const Device UA627023 = {S60920, 114, { &P15364,&P5690,&P24197,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.0*/
const Device UA668812 = {S60921, 123, { &P15364,&P5690,&P24197,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.0.8405.201*/
const Device UA788987 = {S60922, 150, { &P15364,&P5690,&P24197,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/7.0.517.0 Safari/534.16; QQBrowser/6.0.8082.201 (webkit)*/
const Device UA898326 = {S60923, 64, { &P15364,&P5690,&P24197,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1) QQBrowser/6.0*/
const Device UA2588297 = {S60924, 64, { &P15364,&P5690,&P24197,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2) QQBrowser/6.0*/
const Device UA629817 = {S60925, 135, { &P19784,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; LIFETAB_P9514 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1935834 = {S60926, 64, { &P19784,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; LIFETAB_P9514 Build/HTJ85B)*/
const Device UA2462488 = {S60927, 154, { &P20749,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ALCATEL ONE TOUCH 6033X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA1956732 = {S60928, 141, { &P20251,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-T328d/1.23.1401.1) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1995173 = {S60929, 126, { &P20251,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328d Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2026831 = {S60930, 131, { &P20251,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328d Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2028505 = {S60931, 141, { &P20251,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-T328d/1.48.1401.5) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028565 = {S60932, 141, { &P20251,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-T328d/1.40.1401.2) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028636 = {S60933, 141, { &P20251,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-T328d/1.55.1401.2) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028782 = {S60934, 129, { &P20251,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328d) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029278 = {S60935, 131, { &P20251,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328d Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2230836 = {S60936, 141, { &P20251,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-T328d/1.55.1401.4) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2458077 = {S60937, 141, { &P20251,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-T328d/1.67.1401.3) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2641110 = {S60938, 62, { &P20251,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC T328d Build/IML74K)*/
const Device UA2850295 = {S60939, 141, { &P20251,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-T328d/1.67.1401.6) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA900386 = {S60940, 136, { &P23864,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; DATAM819HD_A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA904606 = {S60941, 136, { &P23864,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; DATAM819HD_B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1969848 = {S60942, 136, { &P23864,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; DATAM819HD_C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1967642 = {S60943, 130, { &P23873,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BC1003 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1968603 = {S60944, 130, { &P23873,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; BC1003 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2686691 = {S60945, 59, { &P23873,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; BC1003 Build/IMM76D)*/
const Device UA1995471 = {S60946, 139, { &P12496,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I897 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030079 = {S60947, 139, { &P12496,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I897 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032021 = {S60948, 139, { &P12496,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I897 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2236063 = {S60949, 139, { &P12496,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I897 Build/JRO03U) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2237160 = {S60950, 139, { &P12496,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I897 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257977 = {S60951, 139, { &P12496,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I897 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2420590 = {S60952, 61, { &P12496,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I897 Build/JZO54K)*/
const Device UA2457159 = {S60953, 139, { &P12496,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I897 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2201625 = {S60954, 72, { &P15364,&P17285,&P22365,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:15.0) Gecko/X Firefox/15.2 PaleMoon/15.2*/
const Device UA2235410 = {S60955, 92, { &P15364,&P17285,&P22365,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:15.0) Gecko/X Firefox/15.2-x64 PaleMoon/15.2-x64*/
const Device UA12628 = {S60956, 67, { &P11745,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GT365/V11c Teleca/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA728062 = {S60957, 135, { &P21184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; ZTE T28 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2362398 = {S60958, 143, { &P21184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; ZTE T28 Prepaid Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1991469 = {S60959, 136, { &P21007,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; P-06D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2441003 = {S60960, 58, { &P21007,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; P-06D Build/IMM76D)*/
const Device UA1995593 = {S60961, 138, { &P18683,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ISW13F Build/V51R37G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2012023 = {S60962, 138, { &P18683,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ISW13F Build/V53R39C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031374 = {S60963, 138, { &P18683,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ISW13F Build/V55R41F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2239160 = {S60964, 138, { &P18683,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ISW13F Build/V61R47A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2368093 = {S60965, 138, { &P18683,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ISW13F Build/V69R51I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2387396 = {S60966, 138, { &P18683,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ISW13F Build/V67R49E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2795699 = {S60967, 138, { &P18683,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ISW13F Build/V71R54A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2899258 = {S60968, 138, { &P18683,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ISW13F Build/V75R58A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA897 = {S60969, 101, { &P15210,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG-GT-i8000V/BUIJ1*/
const Device UA898 = {S60970, 102, { &P15210,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG-GT-i8000V/NXXJD6*/
const Device UA905 = {S60971, 87, { &P15210,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-GT-i8000H/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA16326 = {S60972, 101, { &P15210,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG-GT-i8000V/BUII1*/
const Device UA16363 = {S60973, 91, { &P15210,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-GT-i8000T Mozilla (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA34475 = {S60974, 101, { &P15210,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG-GT-i8000V/BUJA3*/
const Device UA34467 = {S60975, 102, { &P15210,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG-GT-i8000V/NXXIL5*/
const Device UA35447 = {S60976, 94, { &P15210,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-GT-i8000Orange/BVJB3 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA61801 = {S60977, 86, { &P15210,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-GT-i8000/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA76868 = {S60978, 102, { &P15210,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG-GT-i8000V/NXXJE2*/
const Device UA171380 = {S60979, 134, { &P15364,&P17285,&P22492,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.3.4641 Chrome/7.0.517.450 Safari/534.7*/
const Device UA2258506 = {S60980, 120, { &P13552,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaC6-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile*/
const Device UA2261211 = {S60981, 32, { &P13552,&P5129,&P24672,&P18092 }};	/*Useragent - NokiaC6-00/UCWEB8.6.0.180/50/800*/
const Device UA2654666 = {S60982, 129, { &P13552,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiac6-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/84/352/UCWEB Mobile  3gpp-gba*/
const Device UA2974274 = {S60983, 137, { &P21350,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8730 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976122 = {S60984, 138, { &P21350,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8730T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA624875 = {S60985, 91, { &P15364,&P22200,&P17826,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/4.4; Linux) KHTML/4.4.1 (like Gecko) Fedora/4.4.1-1.fc12*/
const Device UA903684 = {S60986, 77, { &P15364,&P5690,&P21170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:13.0) Gecko/X Firefox/13.0.1 SeaMonkey/2.10.1*/
const Device UA906443 = {S60987, 62, { &P15364,&P5690,&P21170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:13.0) Gecko/X SeaMonkey/2.10.1*/
const Device UA907377 = {S60988, 84, { &P15364,&P5690,&P21170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:13.0) Gecko/X Firefox/13.0.1 SeaMonkey/2.10.1*/
const Device UA2362529 = {S60989, 139, { &P20834,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Luna TAB474 Build/LunaTAB474) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2531903 = {S60990, 152, { &P18583,&P5043,&P24676,&P18092 }};	/*Useragent - Nokia110/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia110) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA3030599 = {S60991, 150, { &P18437,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8020 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3030840 = {S60992, 150, { &P18437,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3031214 = {S60993, 150, { &P18437,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA44168 = {S60994, 184, { &P11766,&P5776,&P3756,&P18092 }};	/*Useragent - Nokia3230/2.0 (5.0614.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0Configuration/CLDC-1.0, Nokia3230/2.0 (5.0614.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0Configuration/CLDC-1.0*/
const Device UA64458 = {S60995, 91, { &P11766,&P5776,&P3756,&P18092 }};	/*Useragent - Nokia3230/2.0 (5.0614.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0Configuration/CLDC-1.0*/
const Device UA80510 = {S60996, 9, { &P11766,&P5776,&P3756,&P18092 }};	/*Useragent - Nokia3230*/
const Device UA90086 = {S60997, 118, { &P18088,&P6476,&P4732,&P18092 }};	/*Useragent - SHARP-STX2/MC.40; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.19039; U; en-US) Opera 9.50*/
const Device UA752857 = {S60998, 176, { &P15364,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Full Android on Supersonic Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1981009 = {S60999, 154, { &P15364,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Crescent Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1982006 = {S61000, 158, { &P15364,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Crescent Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2024154 = {S61001, 154, { &P15364,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Wildfire Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2419567 = {S61002, 153, { &P15364,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Ray Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2419690 = {S61003, 154, { &P15364,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Play Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2445355 = {S61004, 149, { &P15364,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Pro Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2525683 = {S61005, 155, { &P15364,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Neo V Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2553220 = {S61006, 187, { &P15364,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; EHNDROIX-i9003 Build/IMM76L; CyanogenMod-EHNDROIX 12.9.3 Based on 9.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2649765 = {S61007, 144, { &P15364,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; novo7 Build/IML74K; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2692686 = {S61008, 154, { &P15364,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Mini Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2692918 = {S61009, 153, { &P15364,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Neo Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2776280 = {S61010, 145, { &P15364,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-be; MP907C Build/IML74K; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2195936 = {S61011, 138, { &P18458,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I200 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2711054 = {S61012, 132, { &P18223,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.0.400) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2718809 = {S61013, 139, { &P18223,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.0.400) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA30912 = {S61014, 122, { &P12369,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-S7070/S7070ADJD1 SHP/VPP/R5 Jasmine/1.0 Nextreaming/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA395693 = {S61015, 75, { &P17595,&P18577,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro6%2C2)*/
const Device UA450552 = {S61016, 70, { &P17595,&P18577,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (iMac10%2C1)*/
const Device UA558022 = {S61017, 75, { &P17595,&P18577,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookAir3%2C2)*/
const Device UA1888494 = {S61018, 72, { &P17595,&P18577,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBook4%2C1)*/
const Device UA2139123 = {S61019, 72, { &P17595,&P18577,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (Macmini3%2C1)*/
const Device UA2139794 = {S61020, 71, { &P17595,&P18577,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacPro1%2C1)*/
const Device UA899048 = {S61021, 137, { &P21329,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; IS703C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029579 = {S61022, 143, { &P21329,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Penta IS703C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2468007 = {S61023, 130, { &P21329,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IS703C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2766732 = {S61024, 136, { &P21329,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Penta IS703C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2539859 = {S61025, 139, { &P13141,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I727R Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2656200 = {S61026, 139, { &P13141,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E110S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2667493 = {S61027, 156, { &P17324,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9001 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2961887 = {S61028, 164, { &P17324,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9001 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/ariesve*/
const Device UA3106200 = {S61029, 178, { &P17324,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; it-it; GT-i9001 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/EHNDROIX III i9001/ariesve*/
const Device UA1978755 = {S61030, 138, { &P18676,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM106SH Build/S0010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2005525 = {S61031, 138, { &P18676,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM106SH Build/S0020) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2171847 = {S61032, 60, { &P18676,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SBM106SH Build/S0020)*/
const Device UA2897735 = {S61033, 138, { &P18676,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM106SH Build/S0026) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1997509 = {S61034, 135, { &P22021,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Kogan-Agora Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2599833 = {S61035, 142, { &P22021,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KHPHN05ANDA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2740485 = {S61036, 159, { &P22021,&P18066,&P18062,&P18092 }};	/*Useragent - KHPHN05ANDA Linux/3.0.13 Android/4.0.4 Release/04.16.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2600223 = {S61037, 148, { &P23122,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Karbonn A6 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3026046 = {S61038, 111, { &P19973,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia114/03.33; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3067048 = {S61039, 129, { &P17686,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ601 Build/I.7.1-45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA95065 = {S61040, 105, { &P15364,&P17285,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; aggregator VocusBot 0.4; +http://www.vocus.com/vnhs.html)*/
const Device UA297822 = {S61041, 62, { &P15364,&P17285,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) Jayde NicheBot*/
const Device UA629266 = {S61042, 142, { &P22213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; PTAB7200 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2352584 = {S61043, 132, { &P21087,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A690/S001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2552425 = {S61044, 133, { &P21087,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A690 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2652787 = {S61045, 140, { &P21087,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Lenovo A690 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2746446 = {S61046, 64, { &P21087,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; Lenovo A690 Build/GRK39F)*/
const Device UA23346 = {S61047, 94, { &P13687,&P17603,&P3534,&P18092 }};	/*Useragent - PantechP7040/JLUS05242010 Browser/Obigo/Q05A OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA38521 = {S61048, 95, { &P13687,&P17603,&P3534,&P18092 }};	/*Useragent - PantechP7040/JFUS03152010F Browser/Obigo/Q05A OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA60524 = {S61049, 94, { &P13687,&P17603,&P3534,&P18092 }};	/*Useragent - PantechP7040/JLUS04042011 Browser/Obigo/Q05A OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA691070 = {S61050, 31, { &P14188,&P5129,&P24670,&P18092 }};	/*Useragent - Nokia5233/UCWEB8.4.0.150/50/800*/
const Device UA893821 = {S61051, 80, { &P14188,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-in; Nokia5233)/UC Browser8.4.0.159/50/352/UCWEB Mobile*/
const Device UA2373893 = {S61052, 81, { &P14188,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-in; Nokia 5233)/UC Browser8.4.0.159/50/352/UCWEB Mobile*/
const Device UA2381350 = {S61053, 88, { &P14188,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokia5233) UCBrowser8.4.0.159/70/352/UCWEB Mobile  3gpp-gba*/
const Device UA2489281 = {S61054, 122, { &P12280,&P19826,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 Flipboard/1.9.8*/
const Device UA2489628 = {S61055, 122, { &P12280,&P19826,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Flipboard/1.9.8*/
const Device UA2511756 = {S61056, 122, { &P12280,&P19826,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Flipboard/1.9.8*/
const Device UA2512872 = {S61057, 124, { &P12280,&P19826,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145 Flipboard/1.9.8*/
const Device UA2515670 = {S61058, 122, { &P12280,&P19826,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Flipboard/1.9.8*/
const Device UA2533381 = {S61059, 124, { &P12280,&P19826,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Flipboard/1.9.8*/
const Device UA2585568 = {S61060, 124, { &P12280,&P19826,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/1.9.8*/
const Device UA2892201 = {S61061, 124, { &P12280,&P19826,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Flipboard/1.9.8*/
const Device UA1564 = {S61062, 86, { &P13216,&P17485,&P3600,&P18092 }};	/*Useragent - SonyEricssonK750i/R1L Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA26848 = {S61063, 75, { &P13646,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-C3010/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA34404 = {S61064, 116, { &P13646,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-C3010/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA2218315 = {S61065, 135, { &P20156,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S500 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556308 = {S61066, 139, { &P20156,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; TCL S500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3043937 = {S61067, 146, { &P20156,&P18066,&P18062,&P18092 }};	/*Useragent - TCL S500 Linux/3.0.13 Android/4.0.4 Release/ Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.4;*/
const Device UA2756754 = {S61068, 188, { &P18385,&P18598,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Nexus 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.2/552,2.0.2.552,2013-05-22 23:05, +0100, gb*/
const Device UA2753276 = {S61069, 137, { &P18488,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753690 = {S61070, 137, { &P20226,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E975K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754938 = {S61071, 136, { &P20226,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754797 = {S61072, 140, { &P22598,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6506 Build/10.1.A.1.395) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA16542 = {S61073, 135, { &P15345,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-P506 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA56862 = {S61074, 136, { &P15345,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P506 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA557607 = {S61075, 59, { &P15345,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; LG-P506 Build/FRG83)*/
const Device UA742928 = {S61076, 60, { &P15345,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-P506 Build/FRG83G)*/
const Device UA850316 = {S61077, 135, { &P15345,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-P506 Build/FRG85) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019901 = {S61078, 133, { &P15345,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-P506 Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA704073 = {S61079, 103, { &P15364,&P7133,&P22331,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; zh-TW; rv:1.9.2.16) Gecko/X Firefox/3.6.16 CometBrowser/3.6.16*/
const Device UA1870312 = {S61080, 97, { &P15364,&P7133,&P22331,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; bg; rv:1.9.2.16) Gecko/X Firefox/3.6.16 CometBird/3.6.16*/
const Device UA1992924 = {S61081, 128, { &P19991,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; A511 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2567118 = {S61082, 139, { &P13141,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I727R Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2584849 = {S61083, 132, { &P13141,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I727R Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667165 = {S61084, 139, { &P13141,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E110S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2030918 = {S61085, 140, { &P20128,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; AT-AS43D2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2428735 = {S61086, 140, { &P20128,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; AT-AS43D2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2428303 = {S61087, 134, { &P19996,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A15 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2954753 = {S61088, 139, { &P19996,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MITO A15 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3118542 = {S61089, 56, { &P19996,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; A15 Build/IMM76I)*/
const Device UA2754950 = {S61090, 139, { &P15972,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One XL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2796586 = {S61091, 139, { &P15972,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One XL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937351 = {S61092, 140, { &P15972,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC EVA_UTL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2974818 = {S61093, 137, { &P22595,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919V Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA16600 = {S61094, 82, { &P12643,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C5010E/C5010EDWJI1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA38482 = {S61095, 82, { &P12643,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C5010D/C5010DBGJG3 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49892 = {S61096, 74, { &P12643,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C5010L/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61174 = {S61097, 74, { &P12643,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C5010B/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62044 = {S61098, 80, { &P12643,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C5010/C5010DDJF3 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2028582 = {S61099, 138, { &P19909,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-P935 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2596585 = {S61100, 148, { &P19909,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; LG OPTIMUS LTE HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2649754 = {S61101, 60, { &P19909,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P935 Build/IMM76D)*/
const Device UA2732598 = {S61102, 134, { &P18680,&P18110,&P22587,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Safari/535.19 Silk-Accelerated=false*/
const Device UA2975043 = {S61103, 137, { &P18300,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T769 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3061490 = {S61104, 138, { &P12657,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC_X515E Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3064980 = {S61105, 145, { &P12657,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515m Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080319 = {S61106, 145, { &P12657,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066881 = {S61107, 134, { &P24368,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5215 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA359363 = {S61108, 136, { &P18678,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM102SH Build/S0008) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369059 = {S61109, 136, { &P18678,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM102SH Build/S0018) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA379997 = {S61110, 136, { &P18678,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM102SH Build/S0012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA424761 = {S61111, 136, { &P18678,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM102SH Build/S0020) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA900462 = {S61112, 136, { &P18678,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM102SH Build/S0022) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA900705 = {S61113, 60, { &P18678,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SBM102SH Build/S0022)*/
const Device UA1990838 = {S61114, 137, { &P18678,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM102SH2 Build/S0002) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033591 = {S61115, 61, { &P18678,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SBM102SH2 Build/S0002)*/
const Device UA2673083 = {S61116, 143, { &P18225,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U9202L-1 Build/HuaweiU9202L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676334 = {S61117, 139, { &P18225,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; U9200 Build/HuaweiU9200) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676289 = {S61118, 139, { &P18496,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LVW Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2767782 = {S61119, 139, { &P18496,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR6410LVW Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3101702 = {S61120, 139, { &P18496,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LRA Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3026373 = {S61121, 113, { &P15837,&P6909,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-05/08.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA27981 = {S61122, 105, { &P12205,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonC903a/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3*/
const Device UA64347 = {S61123, 104, { &P12205,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonC903/R1GA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.4*/
const Device UA29945 = {S61124, 118, { &P12280,&P17141,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B367*/
const Device UA29961 = {S61125, 120, { &P12280,&P17141,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B500*/
const Device UA29965 = {S61126, 120, { &P12280,&P17141,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_2_1 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B405*/
const Device UA2801307 = {S61127, 136, { &P12280,&P17141,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7W367a Safari/531.21.10*/
const Device UA2978637 = {S61128, 119, { &P12280,&P17141,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.20 (KHTML, like Gecko) Mobile/7B298g*/
const Device UA197883 = {S61129, 136, { &P21043,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SH Build/S0020) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA248870 = {S61130, 136, { &P21043,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SH Build/S0038) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA420918 = {S61131, 136, { &P21043,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SH Build/S0026) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA451998 = {S61132, 137, { &P21043,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SHJ Build/S0012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA562258 = {S61133, 137, { &P21043,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SHJ Build/S0006) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA616185 = {S61134, 136, { &P21043,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SH Build/S0048) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626770 = {S61135, 136, { &P21043,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SH Build/S0040) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA629112 = {S61136, 137, { &P21043,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SHJ Build/S0024) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA718227 = {S61137, 137, { &P21043,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SHJ Build/S0016) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1936590 = {S61138, 137, { &P21043,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SHK Build/S0008) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019601 = {S61139, 137, { &P21043,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SHK Build/S0006) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024092 = {S61140, 137, { &P21043,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SHK Build/S0016) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026463 = {S61141, 60, { &P21043,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SBM007SH Build/S0048)*/
const Device UA3091718 = {S61142, 136, { &P21043,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SBM007SH Build/S0058) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2361992 = {S61143, 133, { &P18680,&P18110,&P19803,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Safari/535.19 Silk-Accelerated=true*/
const Device UA587647 = {S61144, 103, { &P17595,&P17296,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; Edition MacAppStore; en) Presto/2.10.229 Version/11.61*/
const Device UA588596 = {S61145, 90, { &P17595,&P17296,&P3403,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6.8; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.61*/
const Device UA634958 = {S61146, 72, { &P17595,&P17295,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:9.0) Gecko/X Firefox/9.0*/
const Device UA2752968 = {S61147, 141, { &P18382,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire C Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755261 = {S61148, 141, { &P18382,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2975077 = {S61149, 135, { &P22126,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Torque Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA899521 = {S61150, 139, { &P15820,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-SU640 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026645 = {S61151, 61, { &P15820,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-SU640 Build/IMM76D)*/
const Device UA2413953 = {S61152, 73, { &P19955,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-S3570/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2658234 = {S61153, 138, { &P22958,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-US780 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2753273 = {S61154, 136, { &P12972,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T989 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755124 = {S61155, 145, { &P12972,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-T989 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA624125 = {S61156, 86, { &P18088,&P22740,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; Maemo; Opera Mobi/14; U; en-GB) Presto/2.9.201 Version/11.50*/
const Device UA1934712 = {S61157, 86, { &P18088,&P22740,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv6l; Maemo; Opera Mobi/14; U; en-GB) Presto/2.9.201 Version/11.50*/
const Device UA2431179 = {S61158, 230, { &P20134,&P5629,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2408262 = {S61159, 68, { &P12280,&P18432,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 (iPhone4,1; iPhone OS 6.0.1; en_US) AppleWebKit/420+*/
const Device UA2430818 = {S61160, 66, { &P12280,&P18432,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 (iPhone4,1; iPhone OS 6.0; en_US) AppleWebKit/420+*/
const Device UA2459844 = {S61161, 68, { &P12280,&P18432,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 (iPhone5,1; iPhone OS 6.0.2; en_US) AppleWebKit/420+*/
const Device UA2667736 = {S61162, 135, { &P20690,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl_NL; V360 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2856376 = {S61163, 57, { &P20690,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; V360 Build/JRO03C)*/
const Device UA3021162 = {S61164, 139, { &P21604,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C2105 Build/15.0.A.2.17) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3059577 = {S61165, 132, { &P21604,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C2105 Build/15.0.A.2.17) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2368892 = {S61166, 133, { &P20527,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950014 = {S61167, 135, { &P20527,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; A6 Build/HuaqinA6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566747 = {S61168, 139, { &P18606,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567116 = {S61169, 132, { &P18606,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2570661 = {S61170, 159, { &P18295,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2773261 = {S61171, 158, { &P18295,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2675405 = {S61172, 136, { &P14574,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus S Build/JRO03E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676344 = {S61173, 136, { &P14574,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734763 = {S61174, 129, { &P14574,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2680138 = {S61175, 128, { &P21769,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ME301T Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3031710 = {S61176, 152, { &P20129,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3033505 = {S61177, 153, { &P20129,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2S Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3035727 = {S61178, 153, { &P20129,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2A Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3037092 = {S61179, 154, { &P20129,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2SC Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3059458 = {S61180, 153, { &P20129,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2C Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2717581 = {S61181, 156, { &P18267,&P5043,&P24677,&P18092 }};	/*Useragent - NokiaX2-02/2.0 (11.79) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-02) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2973270 = {S61182, 130, { &P17168,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7310 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA184138 = {S61183, 134, { &P15364,&P5690,&P22448,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Iron/10.0.650.1 Chrome/10.0.650.1 Safari/534.16*/
const Device UA670792 = {S61184, 92, { &P11684,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5800 XpressMusic)/UC Browser8.2.0.132/50/355/UCWEB Mobile*/
const Device UA678197 = {S61185, 43, { &P11684,&P5129,&P24667,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UCWEB8.2.0.116/50/800*/
const Device UA1943462 = {S61186, 44, { &P11684,&P5129,&P24667,&P18092 }};	/*Useragent - Nokia5800w XpressMusic/UCWEB8.2.0.116/50/800*/
const Device UA2339014 = {S61187, 91, { &P11684,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokia5800d-1) UCBrowser8.2.1.144/69/352/UCWEB Mobile  3gpp-gba*/
const Device UA907781 = {S61188, 116, { &P17595,&P17295,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/22.0.1183.0 Safari/537.1*/
const Device UA2674480 = {S61189, 129, { &P17595,&P17295,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/22.0.648.205 Safari/534.16*/
const Device UA2715170 = {S61190, 129, { &P17595,&P17295,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/22.0.648.205 Safari/534.16*/
const Device UA1996216 = {S61191, 154, { &P16436,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I8150 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2424626 = {S61192, 119, { &P13706,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5235) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2458250 = {S61193, 41, { &P13706,&P5129,&P24673,&P18092 }};	/*Useragent - Nokia5235UCBrowser/8.7.1.234/50/444/UCWEB*/
const Device UA3066517 = {S61194, 136, { &P21792,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9200 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA25575 = {S61195, 67, { &P12135,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA26450 = {S61196, 76, { &P12135,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GD510-Bouygues/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63685 = {S61197, 74, { &P12135,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GD510-Orange/V10d Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA8041 = {S61198, 83, { &P11717,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12; Microsoft ZuneHD 4.3)*/
const Device UA8043 = {S61199, 83, { &P11717,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12; Microsoft ZuneHD 4.5)*/
const Device UA8042 = {S61200, 83, { &P11717,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12; Microsoft ZuneHD 4.1)*/
const Device UA63128 = {S61201, 118, { &P12115,&P17483,&P4848,&P18092 }};	/*Useragent - LG/KU990i-Orange/v10a Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63763 = {S61202, 111, { &P12115,&P17483,&P4848,&P18092 }};	/*Useragent - LG/KU990i/v10a Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64064 = {S61203, 120, { &P12115,&P17483,&P4848,&P18092 }};	/*Useragent - LG/KU990i-Bouygues/v10a Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88463 = {S61204, 124, { &P12115,&P17483,&P4848,&P18092 }};	/*Useragent - Vodafone/1.0/LG-KU990i/V10a Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA93301 = {S61205, 124, { &P12115,&P17483,&P4848,&P18092 }};	/*Useragent - Vodafone/1.0/LG-KU990i/V10c Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA125192 = {S61206, 124, { &P12115,&P17483,&P4848,&P18092 }};	/*Useragent - Vodafone/1.0/LG-KU990i/V10d Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA131540 = {S61207, 124, { &P12115,&P17483,&P4848,&P18092 }};	/*Useragent - Vodafone/1.0/LG-KU990i/V10b Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2490683 = {S61208, 75, { &P17595,&P17296,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8) Presto/2.12.388 Version/12.13*/
const Device UA2504841 = {S61209, 103, { &P15259,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.276; en-us; 9900/5.1.0.276) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2527731 = {S61210, 103, { &P15259,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.532; en-us; 9900/5.1.0.532) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2595468 = {S61211, 103, { &P15259,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.0.0.734; en-us; 9900/5.0.0.734) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2653515 = {S61212, 103, { &P15259,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.230; en-us; 9900/5.1.0.230) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2695836 = {S61213, 103, { &P15259,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.393; en-us; 9900/5.1.0.393) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2696004 = {S61214, 103, { &P15259,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.546; en-us; 9900/5.1.0.546) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2746797 = {S61215, 103, { &P15259,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.0.0.496; en-us; 9900/5.0.0.496) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2762752 = {S61216, 103, { &P15259,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.0.0.464; en-us; 9900/5.0.0.464) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2778660 = {S61217, 103, { &P15259,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.0.0.735; en-us; 9900/5.0.0.735) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2889194 = {S61218, 103, { &P15259,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.415; en-us; 9900/5.1.0.415) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2952020 = {S61219, 103, { &P15259,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.181; en-us; 9900/5.1.0.181) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2952111 = {S61220, 103, { &P15259,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.0.0.694; en-us; 9900/5.0.0.694) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2985906 = {S61221, 103, { &P15259,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.0.0.733; en-us; 9900/5.0.0.733) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3044575 = {S61222, 103, { &P15259,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.338; en-us; 9900/5.1.0.338) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3071353 = {S61223, 101, { &P15259,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.75; en-us; 9900/5.1.0.75) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3087539 = {S61224, 103, { &P15259,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.641; en-us; 9900/5.1.0.641) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2973909 = {S61225, 154, { &P21857,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA463382 = {S61226, 83, { &P15364,&P17285,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.8.0.4) Gecko/X Firefox/1.5.0.4*/
const Device UA2740103 = {S61227, 138, { &P18559,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LG730 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2796412 = {S61228, 131, { &P18559,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LG730 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2974079 = {S61229, 141, { &P18462,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA1997032 = {S61230, 155, { &P14183,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I997 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2012754 = {S61231, 155, { &P14183,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I997 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2200854 = {S61232, 155, { &P14183,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I997 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2232006 = {S61233, 155, { &P14183,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I997 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2228695 = {S61234, 131, { &P21990,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; chagall Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2489575 = {S61235, 131, { &P20985,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MIDC410 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2753403 = {S61236, 133, { &P20985,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MIDC410PR Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2669589 = {S61237, 138, { &P19917,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; LG-P765 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2713397 = {S61238, 138, { &P19917,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-P765 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2863504 = {S61239, 60, { &P19917,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P765 Build/JZO54K)*/
const Device UA18552 = {S61240, 88, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; i-mate JAMin PPC; 240x320; PPC; 240x320)*/
const Device UA22091 = {S61241, 80, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - Dopod900 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 480x640; Dopod900)*/
const Device UA57957 = {S61242, 120, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; i-mate JASJAM V2 PPC; 240x320; i-mate JASJAM PPC; 240x320; PPC; 240x320)*/
const Device UA59516 = {S61243, 89, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; i-mate JASJAR PPC; 640x480; PPC; 240x320)*/
const Device UA63904 = {S61244, 89, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; i-mate JASJAM PPC; 240x320; PPC; 240x320)*/
const Device UA64007 = {S61245, 105, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; MDA Pro/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1)*/
const Device UA80328 = {S61246, 76, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; HP iPAQ h6300)*/
const Device UA80360 = {S61247, 81, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 Sprint:MotoQ (compatible; MSIE 4.01; Windows CE; Smartphone; 176x220)*/
const Device UA86970 = {S61248, 96, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - GIGABYTE-g-Smart Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; GIGABYTE-g-Smart)*/
const Device UA87396 = {S61249, 78, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC i-mate PDAL; 240x320; PPC)*/
const Device UA87900 = {S61250, 86, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; i-mate JAQ PPC; 240x320; PPC; 240x320)*/
const Device UA481600 = {S61251, 47, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE)*/
const Device UA490759 = {S61252, 61, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 800x480)*/
const Device UA587372 = {S61253, 63, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC;   240x320)*/
const Device UA709290 = {S61254, 61, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPS; 240x320)*/
const Device UA772414 = {S61255, 59, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone)*/
const Device UA878478 = {S61256, 96, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 PPC (compatible; MSIE 4.01; Windows CE; PPC; 240x320; Sprint:PPC-6700; PPC; 240x320)*/
const Device UA886027 = {S61257, 86, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Sprint:SPH-ip320; Smartphone; 176x220)*/
const Device UA897127 = {S61258, 66, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; PPC)*/
const Device UA2211721 = {S61259, 60, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0(compatible; MSIE 4.01; Windows CE; PPC; 240x320)*/
const Device UA2297889 = {S61260, 78, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Sprint;PPC-i830; PPC; 240x320)*/
const Device UA2791752 = {S61261, 63, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; PPC; Tablet PC; Trident/1.0)*/
const Device UA2807643 = {S61262, 97, { &P15364,&P5448,&P4578,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_prophet/2.15.3.113/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)*/
const Device UA371964 = {S61263, 23, { &P15364,&P17017,&P23046,&P18092 }};	/*Useragent - NSPlayer/11.0.5721.5262*/
const Device UA372979 = {S61264, 35, { &P15364,&P17017,&P23046,&P18092 }};	/*Useragent - NSPlayer/11.0.5721.5262 WMFSDK/11.0*/
const Device UA418221 = {S61265, 24, { &P15364,&P17017,&P23046,&P18092 }};	/*Useragent - NSPlayer/11.0.6002.18005*/
const Device UA430165 = {S61266, 35, { &P15364,&P17017,&P23046,&P18092 }};	/*Useragent - NSPlayer/11.0.5721.5245 WMFSDK/11.0*/
const Device UA459853 = {S61267, 35, { &P15364,&P17017,&P23046,&P18092 }};	/*Useragent - NSPlayer/11.0.6001.7000 WMFSDK/11.0*/
const Device UA527152 = {S61268, 23, { &P15364,&P17017,&P23046,&P18092 }};	/*Useragent - NSPlayer/11.0.5705.5043*/
const Device UA542282 = {S61269, 23, { &P15364,&P17017,&P23046,&P18092 }};	/*Useragent - NSPlayer/11.0.6000.6346*/
const Device UA623063 = {S61270, 35, { &P15364,&P17017,&P23046,&P18092 }};	/*Useragent - NSPlayer/11.0.6001.7006 WMFSDK/11.0*/
const Device UA745817 = {S61271, 35, { &P15364,&P17017,&P23046,&P18092 }};	/*Useragent - NSPlayer/11.0.6000.6351 WMFSDK/11.0*/
const Device UA1882023 = {S61272, 23, { &P15364,&P17017,&P23046,&P18092 }};	/*Useragent - NSPlayer/11.0.5721.5269*/
const Device UA2133070 = {S61273, 47, { &P15364,&P17017,&P23046,&P18092 }};	/*Useragent - NSPlayer/11.00.6001.7000 WMFSDK/11.00.6001.7000*/
const Device UA2672765 = {S61274, 53, { &P15364,&P17017,&P23046,&P18092 }};	/*Useragent - NSPlayer/11.0.5721.5275 WMFSDK/11.0 SE 2.X MetaSr 1.0*/
const Device UA2709548 = {S61275, 53, { &P15364,&P17017,&P23046,&P18092 }};	/*Useragent - NSPlayer/11.0.5721.5262 WMFSDK/11.0 SE 2.X MetaSr 1.0*/
const Device UA2981261 = {S61276, 45, { &P15364,&P17017,&P23046,&P18092 }};	/*Useragent - NSPlayer/11.0.5721.5275 WMFSDK/11.0 LBBROWSER*/
const Device UA2994598 = {S61277, 53, { &P15364,&P17017,&P23046,&P18092 }};	/*Useragent - NSPlayer/11.0.5721.5145 WMFSDK/11.0 SE 2.X MetaSr 1.0*/
const Device UA2713896 = {S61278, 158, { &P15989,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.1.8) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2715432 = {S61279, 158, { &P15989,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.4.27) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752188 = {S61280, 158, { &P15989,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.6.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2784711 = {S61281, 158, { &P15989,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.0.16) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2796886 = {S61282, 158, { &P15989,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.6.8) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2798857 = {S61283, 158, { &P15989,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.4.22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2974635 = {S61284, 142, { &P19862,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Micromax A110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975532 = {S61285, 126, { &P19862,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021095 = {S61286, 126, { &P19862,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3125037 = {S61287, 153, { &P19862,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Micromax A110 (Canvas 2) Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA33261 = {S61288, 105, { &P13015,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-C414Y/C414USKA4 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA49407 = {S61289, 105, { &P13015,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-C414R/C414UXKD1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA49436 = {S61290, 105, { &P13015,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-C414M/C414BMKD3 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA774954 = {S61291, 104, { &P13015,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-C414/C414TLJK9 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA2144148 = {S61292, 105, { &P13015,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-C414W/C414UQKD1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA410632 = {S61293, 133, { &P18276,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-08D Build/F0001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633966 = {S61294, 135, { &P18276,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-08D Build/V10R21B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA904805 = {S61295, 135, { &P18276,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-08D Build/V12R23A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032940 = {S61296, 59, { &P18276,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; F-08D Build/V12R23A)*/
const Device UA460853 = {S61297, 86, { &P15364,&P17285,&P18199,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; de; rv:1.8.1.24) Gecko/X Thunderbird/2.0.0.24*/
const Device UA830996 = {S61298, 137, { &P20592,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-SU870 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2588308 = {S61299, 61, { &P20592,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LG-SU870 Build/GRK39F)*/
const Device UA2365556 = {S61300, 137, { &P20240,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E971 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2673365 = {S61301, 138, { &P18724,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-F Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2673872 = {S61302, 133, { &P18724,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SmartTabII7 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676441 = {S61303, 138, { &P18724,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-H Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2709417 = {S61304, 130, { &P18724,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A2107A-H Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2232575 = {S61305, 161, { &P18233,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL_one_touch_995 Build/IceCreamSandwich) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2763598 = {S61306, 162, { &P18233,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL_one_touch_995S Build/IceCreamSandwich) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2848131 = {S61307, 162, { &P18233,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL_one_touch_995A Build/IceCreamSandwich) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2257417 = {S61308, 153, { &P18241,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-E400 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA162474 = {S61309, 132, { &P17487,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A70S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA719519 = {S61310, 56, { &P17487,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; A70S Build/FROYO)*/
const Device UA676219 = {S61311, 53, { &P18088,&P17017,&P24668,&P18092 }};	/*Useragent - IUC(U;iOS 5.0.1;Zh-cn;320*480;)/UCWEB8.3.1.151/42/997*/
const Device UA803358 = {S61312, 53, { &P18088,&P17017,&P24668,&P18092 }};	/*Useragent - IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB8.3.1.151/41/997*/
const Device UA822529 = {S61313, 51, { &P18088,&P17017,&P24668,&P18092 }};	/*Useragent - IUC(U;iOS 4.1;Zh-cn;320*480;)/UCWEB8.3.1.151/41/800*/
const Device UA862716 = {S61314, 73, { &P18088,&P17017,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; j2me) UCBrowser8.3.0.154/69/352/UCWEB Mobile*/
const Device UA1956244 = {S61315, 74, { &P18088,&P17017,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; mstar) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA1972356 = {S61316, 53, { &P18088,&P17017,&P24668,&P18092 }};	/*Useragent - IUC(U;iOS 5.1.1;Zh-cn;320*480;)/UCWEB8.3.1.151/42/997*/
const Device UA1985669 = {S61317, 83, { &P18088,&P17017,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; mstar platform) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA1994430 = {S61318, 74, { &P18088,&P17017,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; spice) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2156423 = {S61319, 53, { &P18088,&P17017,&P24668,&P18092 }};	/*Useragent - IUC(U;iOS 4.2.1;Zh-cn;320*480;)/UCWEB8.3.1.151/41/997*/
const Device UA2156464 = {S61320, 74, { &P18088,&P17017,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; opera) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2166334 = {S61321, 73, { &P18088,&P17017,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; maui) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2393276 = {S61322, 53, { &P18088,&P17017,&P24668,&P18092 }};	/*Useragent - IUC(U;iOS 5_1_1;en-us;320*480;)/UCWEB8.3.1.173/42/355*/
const Device UA2394892 = {S61323, 76, { &P18088,&P17017,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; generic) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2950157 = {S61324, 74, { &P18088,&P17017,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; kau; mozilla) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA1970023 = {S61325, 131, { &P20513,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032533 = {S61326, 130, { &P20513,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; A5 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2759226 = {S61327, 136, { &P20513,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HOOW A5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2960832 = {S61328, 146, { &P20513,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; id-id; CROSS A5 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1977303 = {S61329, 133, { &P21237,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TOUCHMATE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2203960 = {S61330, 133, { &P21237,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TOUCHMATE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2508326 = {S61331, 136, { &P21449,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SH-04E Build/S1090) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566586 = {S61332, 136, { &P21449,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SH-04E Build/S2130) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2783874 = {S61333, 136, { &P21449,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SH-04E Build/S5170) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2897348 = {S61334, 136, { &P21449,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SH-04E Build/S6181) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3077430 = {S61335, 58, { &P21449,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SH-04E Build/S6181)*/
const Device UA2540496 = {S61336, 147, { &P21264,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; PadFone Infinity Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2540504 = {S61337, 140, { &P21264,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; PadFone Infinity Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2795625 = {S61338, 69, { &P21264,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; PadFone Infinity Build/JZO54K)*/
const Device UA148134 = {S61339, 34, { &P15364,&P17188,&P17466,&P18092 }};	/*Useragent - Opera/6.x (Windows NT 4.0; U) [de]*/
const Device UA427449 = {S61340, 35, { &P15364,&P17188,&P17466,&P18092 }};	/*Useragent - Opera/6.02 (Windows NT 4.0; U) [de]*/
const Device UA603231 = {S61341, 35, { &P15364,&P17188,&P17466,&P18092 }};	/*Useragent - Opera/6.04 (Windows NT 4.0; U) [de]*/
const Device UA685654 = {S61342, 66, { &P15364,&P17188,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 4.0) Opera 6.01 [en]*/
const Device UA807691 = {S61343, 66, { &P15364,&P17188,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 4.0) Opera 6.04 [en]*/
const Device UA820463 = {S61344, 35, { &P15364,&P17188,&P17466,&P18092 }};	/*Useragent - Opera/6.05 (Windows NT 4.0; U) [de]*/
const Device UA821315 = {S61345, 47, { &P15364,&P17188,&P17466,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0; U) Opera 6.05 [en]*/
const Device UA822872 = {S61346, 65, { &P15364,&P17188,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 4.0) Opera 6.0 [de]*/
const Device UA871463 = {S61347, 66, { &P15364,&P17188,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 4.0) Opera 6.02 [en]*/
const Device UA2959788 = {S61348, 146, { &P24095,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyC5306 Build/12.0.A.1.284) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3020566 = {S61349, 142, { &P24095,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; C5306 Build/12.0.A.1.284) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022054 = {S61350, 64, { &P24095,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; C5306 Build/12.0.A.1.284)*/
const Device UA2193787 = {S61351, 118, { &P11854,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE71) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile*/
const Device UA2228988 = {S61352, 30, { &P11854,&P5890,&P24672,&P18092 }};	/*Useragent - NokiaE71/UCWEB8.6.0.180/28/999*/
const Device UA2552846 = {S61353, 119, { &P11854,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiae71-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/84/352/UCWEB Mobile*/
const Device UA2426815 = {S61354, 130, { &P20446,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-05E Build/V12R34G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2566279 = {S61355, 130, { &P20446,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-05E Build/V13R36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2784723 = {S61356, 130, { &P20446,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-05E Build/V16R39A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3014178 = {S61357, 130, { &P20446,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-05E Build/V17R40A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2717573 = {S61358, 156, { &P15575,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-I9100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2770349 = {S61359, 156, { &P15575,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-I9100 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2781646 = {S61360, 157, { &P15575,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-I9100G Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2800685 = {S61361, 157, { &P15575,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9100G Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA226840 = {S61362, 18, { &P15364,&P18110,&P4810,&P18092 }};	/*Useragent - Opera/7.54 (Linux)*/
const Device UA625912 = {S61363, 58, { &P15364,&P18110,&P4810,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux 2.4.21-0.13mdk i686; U) Opera 7.11 [en]*/
const Device UA820379 = {S61364, 23, { &P15364,&P18110,&P4810,&P18092 }};	/*Useragent - Opera/7.51 (Linux) [en]*/
const Device UA838844 = {S61365, 67, { &P15364,&P18110,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux i686) Opera 7.54 [en]*/
const Device UA889371 = {S61366, 48, { &P15364,&P18110,&P4810,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; U) Opera 7.54 [en]*/
const Device UA2161153 = {S61367, 52, { &P15364,&P18110,&P4810,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i386; U) Opera 7.60  [en-GB]*/
const Device UA2282856 = {S61368, 44, { &P15364,&P18110,&P4810,&P18092 }};	/*Useragent - MSIE (MSIE 6.0; X11; Linux; i686) Opera 7.23*/
const Device UA2283901 = {S61369, 37, { &P15364,&P18110,&P4810,&P18092 }};	/*Useragent - Opera/7.50 (X11; Linux i686; U)  [en]*/
const Device UA2331354 = {S61370, 62, { &P15364,&P18110,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux i686) Opera 7.23*/
const Device UA2506853 = {S61371, 37, { &P15364,&P18110,&P4810,&P18092 }};	/*Useragent - Opera/7.23 (X11; Linux i686; U)  [en]*/
const Device UA2742036 = {S61372, 37, { &P15364,&P18110,&P4810,&P18092 }};	/*Useragent - Opera/7.54 (X11; Linux i686; U)  [en]*/
const Device UA2234293 = {S61373, 71, { &P14202,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.1-update1; zh-cn; E15i; 320*480) UCWEB7.9.0.94/139/352*/
const Device UA2482669 = {S61374, 71, { &P14202,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.1-update1; zh-cn; E15a; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2503787 = {S61375, 65, { &P14202,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.7; zh-cn; E15i; 320*480) UCWEB7.9.0.94/139/352*/
const Device UA2834695 = {S61376, 73, { &P14202,&P18110,&P24663,&P18092 }};	/*Useragent - Opera (Linux; U; 2.1-update1; zh-cn; E15i; 320*480) UCWEB7.9.0.94/139/352*/
const Device UA710046 = {S61377, 141, { &P13018,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  MT11i Build/4.1.A.0.562) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA1952765 = {S61378, 140, { &P13018,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MT11i Build/4.1.A.0.562) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2029618 = {S61379, 141, { &P13018,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  MT11i Build/4.1.B.0.431) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA2205958 = {S61380, 140, { &P13018,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT11i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2420328 = {S61381, 140, { &P13018,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT11a Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2567347 = {S61382, 139, { &P20076,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26w Build/6.1.A.2.45) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA3064774 = {S61383, 145, { &P18626,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I547 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081105 = {S61384, 138, { &P18626,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I547C Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094277 = {S61385, 145, { &P18626,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I547 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA48381 = {S61386, 73, { &P16260,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-C3330/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA453500 = {S61387, 96, { &P15364,&P7133,&P22356,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:11.0) Gecko/X Firefox/11.0.1-x64 PaleMoon/11.0.1-x64*/
const Device UA453502 = {S61388, 83, { &P15364,&P7133,&P22356,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:11.0) Gecko/X Firefox/11.0.1 PaleMoon/11.0.1*/
const Device UA2557605 = {S61389, 139, { &P21490,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; IM-A860L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2764198 = {S61390, 61, { &P21490,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; IM-A860L Build/JZO54K)*/
const Device UA3102386 = {S61391, 142, { &P21490,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A860L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Di?n tho?iSafari/534.30*/
const Device UA2712770 = {S61392, 154, { &P22960,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; ALCATEL ONE TOUCH 6030D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2994631 = {S61393, 76, { &P22960,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ALCATEL ONE TOUCH 6030D Build/JRO03C)*/
const Device UA2753774 = {S61394, 134, { &P17968,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; cm_tenderloin Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755538 = {S61395, 128, { &P17968,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783626 = {S61396, 146, { &P17968,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; e2001_v89_gq2008s Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2800210 = {S61397, 147, { &P17968,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; e2001_v89_jbl1a698 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2800384 = {S61398, 141, { &P17968,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; v89_jbl1a698 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2800585 = {S61399, 136, { &P17968,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; skz_tenderloin Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2879122 = {S61400, 121, { &P17968,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2893004 = {S61401, 146, { &P17968,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; e2001_v89_jdt1qhd Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937624 = {S61402, 148, { &P17968,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; e2001v21_v89_zlh_hd Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2958030 = {S61403, 134, { &P17968,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; iPhone5 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2998010 = {S61404, 145, { &P17968,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; e2001_v89_zlh_hd Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA616385 = {S61405, 82, { &P17465,&P7133,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; Opera Tablet/15165; U; en) Presto/2.8.149 Version/11.1*/
const Device UA2722870 = {S61406, 148, { &P12280,&P17779,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 1_2_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5*/
const Device UA2740665 = {S61407, 145, { &P20439,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySOL21 Build/9.1.D.0.395) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752903 = {S61408, 141, { &P20439,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SOL21 Build/9.1.D.0.395) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2900959 = {S61409, 145, { &P20439,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySOL21 Build/9.1.D.0.401) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2944114 = {S61410, 141, { &P20439,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SOL21 Build/9.1.D.0.401) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2963499 = {S61411, 63, { &P20439,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SOL21 Build/9.1.D.0.401)*/
const Device UA367436 = {S61412, 128, { &P15364,&P7133,&P22417,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.2.1.0 Chrome/16.0.912.75 Safari/535.7*/
const Device UA367681 = {S61413, 121, { &P15364,&P7133,&P22417,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.2.1.0 Chrome/16.0.912.75 Safari/535.7*/
const Device UA684998 = {S61414, 135, { &P21328,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3;es_MX; ZTE-U V856 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA844069 = {S61415, 136, { &P21328,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; ZTE-V856 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2809473 = {S61416, 60, { &P21328,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; ZTE-V856 Build/GRJ22)*/
const Device UA2367628 = {S61417, 180, { &P12059,&P5204,&P24672,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia2690) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2466591 = {S61418, 143, { &P20076,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xperia Acro S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2784164 = {S61419, 140, { &P20076,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26w Build/6.2.B.0.200) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2754503 = {S61420, 134, { &P22746,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N9100 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2242332 = {S61421, 69, { &P15991,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GT-S6102; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA68114 = {S61422, 136, { &P12973,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.1.7; md-us; A101B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA241676 = {S61423, 137, { &P12973,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.1.20; md-us; A101C Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA699154 = {S61424, 136, { &P12973,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 6.1.8; md-us; A101B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1876932 = {S61425, 136, { &P12973,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.1.5; fr-fr; A101B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2021966 = {S61426, 136, { &P12973,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 6.1.8; nl-nl; A101B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2022378 = {S61427, 136, { &P12973,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 6.1.8; fr-fr; A101B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2022457 = {S61428, 136, { &P12973,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 6.1.8; es-es; A101B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2674880 = {S61429, 91, { &P17595,&P18207,&P22225,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:20.0) Gecko/X Firefox/20.0 SeaMonkey/2.17.1*/
const Device UA70859 = {S61430, 135, { &P18043,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; ISW11HT Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA191888 = {S61431, 142, { &P18033,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; IM-A740S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA196562 = {S61432, 142, { &P18033,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; IM-A750K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1889966 = {S61433, 140, { &P18033,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; IM-A750K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019517 = {S61434, 66, { &P18033,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; IM-A750K Build/GINGERBREAD)*/
const Device UA2542822 = {S61435, 140, { &P18033,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; IM-A740S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA338370 = {S61436, 136, { &P21034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SH Build/S0008) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA345831 = {S61437, 136, { &P21034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SH Build/S0012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA632730 = {S61438, 136, { &P21034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SH Build/S0018) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA685875 = {S61439, 136, { &P21034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SH Build/S0016) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA895049 = {S61440, 136, { &P21034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SH Build/S0022) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1972209 = {S61441, 137, { &P21034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SHY Build/S0024) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020007 = {S61442, 137, { &P21034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SHY Build/S0014) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023777 = {S61443, 137, { &P21034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SHY Build/S0020) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025114 = {S61444, 137, { &P21034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SHY Build/S0016) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034529 = {S61445, 60, { &P21034,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SBM009SH Build/S0022)*/
const Device UA2583853 = {S61446, 136, { &P21034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SH Build/S0026) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2796623 = {S61447, 136, { &P21034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SH Build/S0032) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2862161 = {S61448, 137, { &P21034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SHY Build/S0028) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2999727 = {S61449, 137, { &P21034,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM009SHY Build/S0030) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1942319 = {S61450, 111, { &P15364,&P6590,&P4626,&P18092 }};	/*Useragent - Mozilla/5.0 (HW-HUAWEI_U2801;U;REX/1.0;BREW/5.0.4.1;Profile/MIDP-2.0 Configuration/CLDC-1.1;128*160) Opera/9.70*/
const Device UA2392347 = {S61451, 111, { &P15364,&P6590,&P4626,&P18092 }};	/*Useragent - Mozilla/5.0 (HW-HUAWEI_U3220;U;REX/1.0;BREW/5.0.4.1;Profile/MIDP-2.0 Configuration/CLDC-1.1;240*320) Opera/9.70*/
const Device UA2410277 = {S61452, 112, { &P15364,&P6590,&P4626,&P18092 }};	/*Useragent - Mozilla/5.0 (HW-HUAWEI_U5200/;U;REX/1.0;BREW/5.0.4.1;Profile/MIDP-2.0 Configuration/CLDC-1.1;240*320) Opera/9.70*/
const Device UA2410441 = {S61453, 112, { &P15364,&P6590,&P4626,&P18092 }};	/*Useragent - Mozilla/5.0 (HW-HUAWEI_U5300/;U;REX/1.0;BREW/5.0.4.1;Profile/MIDP-2.0 Configuration/CLDC-1.1;240*320) Opera/9.70*/
const Device UA2202544 = {S61454, 152, { &P17346,&P18432,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A403 Safari/7534.48.3*/
const Device UA2356966 = {S61455, 154, { &P17346,&P18432,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10A523 Safari/7534.48.3*/
const Device UA2597846 = {S61456, 150, { &P17346,&P18432,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 60 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/19.0.1084.60 Mobile/10A403 Safari/7534.48.3*/
const Device UA904227 = {S61457, 69, { &P15364,&P7133,&P18515,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:14.0) Gecko/X Thunderbird/14.0*/
const Device UA2534933 = {S61458, 140, { &P20608,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Edison Build/2.0.0 X-15:47) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2666979 = {S61459, 140, { &P20608,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Edison Build/2.1.0 X-12:38) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2734115 = {S61460, 69, { &P20608,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; bq Edison Build/2.1.0 X-12:38)*/
const Device UA2879604 = {S61461, 142, { &P20608,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Edison 2 Build/1.0.0_X-18:36) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3030380 = {S61462, 142, { &P20608,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Edison 2 Build/1.0.1_X-14:02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3108042 = {S61463, 71, { &P20608,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; bq Edison 2 Build/1.0.1_X-14:02)*/
const Device UA2567237 = {S61464, 131, { &P17168,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7310 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2637871 = {S61465, 131, { &P17168,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P7310 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566898 = {S61466, 137, { &P18009,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PH44100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567014 = {S61467, 144, { &P18009,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTCEVODesign4G Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2878840 = {S61468, 138, { &P22804,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-P713 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2944508 = {S61469, 140, { &P22804,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; LG-P713GO Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2989607 = {S61470, 60, { &P22804,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P713 Build/JZO54K)*/
const Device UA2870576 = {S61471, 122, { &P17595,&P18207,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/5.0.354.0 Safari/533.3*/
const Device UA902391 = {S61472, 140, { &P15397,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ST18i Build/4.1.A.0.562) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1977979 = {S61473, 140, { &P15397,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST18a Build/4.1.B.0.431) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2016159 = {S61474, 140, { &P15397,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST18i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2566974 = {S61475, 139, { &P23772,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190L Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567203 = {S61476, 139, { &P23772,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2606462 = {S61477, 132, { &P23772,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2668437 = {S61478, 132, { &P23772,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190L Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2868528 = {S61479, 136, { &P20872,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SGP311 Build/10.1.C.0.344) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2876966 = {S61480, 136, { &P20872,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGP311 Build/10.1.C.0.370) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2879580 = {S61481, 65, { &P20872,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGP311 Build/10.1.C.0.370)*/
const Device UA3019449 = {S61482, 67, { &P20872,&P18111,&P18112,&P18092 }};	/*Useragent - SonySGP311 Build/10.1.C.0.370 stagefright/1.2 (Linux;Android 4.1.2)*/
const Device UA910643 = {S61483, 153, { &P17346,&P17146,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9A405 Safari/7534.48.3*/
const Device UA911124 = {S61484, 151, { &P17346,&P17146,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9A334 Safari/7534.48.3*/
const Device UA2597520 = {S61485, 150, { &P17346,&P17146,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 501 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/19.0.1084.60 Mobile/9A405 Safari/7534.48.3*/
const Device UA2598032 = {S61486, 149, { &P17346,&P17146,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 50 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/19.0.1084.60 Mobile/9A334 Safari/7534.48.3*/
const Device UA2034702 = {S61487, 145, { &P20006,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; SonyLT30a Build/7.0.A.1.300) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034738 = {S61488, 145, { &P20006,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SonyLT30a Build/7.0.A.1.303) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2388445 = {S61489, 145, { &P20006,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SonyLT30a Build/7.0.A.3.195) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2456926 = {S61490, 141, { &P20006,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LT30a Build/7.0.A.3.195) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2843866 = {S61491, 63, { &P20006,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LT30a Build/7.0.A.3.195)*/
const Device UA904877 = {S61492, 138, { &P18654,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; WX04K Build/143.3.0000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2010808 = {S61493, 138, { &P18654,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; WX04K Build/145.2.0000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029556 = {S61494, 138, { &P18654,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; WX04K Build/145.0.0000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2202832 = {S61495, 138, { &P18654,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; WX04K Build/146.2.0000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2606280 = {S61496, 138, { &P18654,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; WX04K Build/147.2.0000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2863593 = {S61497, 62, { &P18654,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; WX04K Build/147.2.0000)*/
const Device UA2385830 = {S61498, 134, { &P20807,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; B92 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2595122 = {S61499, 56, { &P20807,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; B92 Build/IMM76D)*/
const Device UA17399 = {S61500, 64, { &P15661,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2720a-2/2.0 (09.55) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA23918 = {S61501, 65, { &P15661,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2720a-2b/2.0 (08.64) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA24070 = {S61502, 62, { &P15661,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2720a/2.0 (09.55) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA43660 = {S61503, 41, { &P13720,&P17484,&P3625,&P18092 }};	/*Useragent - sam-r570 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA190403 = {S61504, 131, { &P17712,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-ca; LG-V905R Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2019564 = {S61505, 60, { &P17712,&P6545,&P4755,&P18092 }};	/*Useragent - Dalvik/1.5.0 (Linux; U; Android 3.0.1; LG-V905R Build/HRI66)*/
const Device UA178974 = {S61506, 144, { &P16103,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; XT720 Build/SHOLS_U2_05.26.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA302195 = {S61507, 145, { &P16103,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; XT720 Build/STSKT_N_79.33.50R) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA701238 = {S61508, 148, { &P16103,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Milestone XT720 Build/R.U.R.1920) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018639 = {S61509, 69, { &P16103,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; XT720 Build/STSKT_N_79.33.50R)*/
const Device UA2025039 = {S61510, 148, { &P16103,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; Milestone XT720 Build/R.U.R.1920) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA9964 = {S61511, 133, { &P13453,&P6590,&P4223,&P18092 }};	/*Useragent - NetFront/3.5.1 (BREW 5.0.2.1; U; en-us; Samsung ; NetFront/3.5.1/AMB) Sprint SPH-M570 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA198216 = {S61512, 140, { &P17964,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SC-01C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019257 = {S61513, 64, { &P17964,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; SC-01C Build/GINGERBREAD)*/
const Device UA2235598 = {S61514, 140, { &P17964,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ja-jp; SC-01C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA903432 = {S61515, 137, { &P18240,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P700 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2532720 = {S61516, 144, { &P21623,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; i-mobile IQ 6 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2604139 = {S61517, 145, { &P21623,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; i-mobile IQ 6A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2858719 = {S61518, 66, { &P21623,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; i-mobile IQ 6 Build/JRO03C)*/
const Device UA2754370 = {S61519, 136, { &P14574,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773725 = {S61520, 136, { &P14574,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus S Build/JRO03E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3026681 = {S61521, 129, { &P15767,&P19826,&P20784,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/10B141 Safari/7534.48.3*/
const Device UA3030801 = {S61522, 131, { &P15767,&P19826,&P20784,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/10B329 Safari/7534.48.3*/
const Device UA3033330 = {S61523, 131, { &P15767,&P19826,&P20784,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/10B146 Safari/7534.48.3*/
const Device UA3079557 = {S61524, 131, { &P15767,&P19826,&P20784,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/10B147 Safari/7534.48.3*/
const Device UA55866 = {S61525, 84, { &P11948,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK550i//R6BC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62725 = {S61526, 103, { &P11948,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK550c/R1JD Release/Jan-23-2007 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA92418 = {S61527, 130, { &P15364,&P5690,&P22443,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.9 (KHTML, like Gecko) Iron/4.0.280.0 Chrome/4.0.280.0 Safari/532.9*/
const Device UA2364026 = {S61528, 130, { &P15364,&P5690,&P22443,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/532.8 (KHTML, like Gecko) Iron/4.0.275.2 Chrome/4.0.275.2 Safari/532.8*/
const Device UA191879 = {S61529, 150, { &P17960,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.11; en-us; NXM736 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA233757 = {S61530, 150, { &P17960,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.19; en-us; NXM736 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA395837 = {S61531, 150, { &P17960,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.14; en-us; NXM736 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2513768 = {S61532, 158, { &P18645,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2670938 = {S61533, 158, { &P18645,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2672737 = {S61534, 166, { &P18645,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2707789 = {S61535, 158, { &P18645,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9305 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2712901 = {S61536, 159, { &P18645,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-I747M Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2734573 = {S61537, 159, { &P18645,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530U Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2870352 = {S61538, 135, { &P20976,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; W100 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050025 = {S61539, 57, { &P20976,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; W100 Build/JOP40D)*/
const Device UA5443 = {S61540, 144, { &P15134,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; LG-C710; Build/Eclair) AppleWebKit/530.17(KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10194 = {S61541, 144, { &P15134,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; LG-C710h Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10195 = {S61542, 146, { &P15134,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; LG-C710h; Build/Eclair) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018770 = {S61543, 66, { &P15134,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; LG-C710h Build/ERE27)*/
const Device UA86927 = {S61544, 83, { &P18088,&P5448,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows CE 5.2; rv:1.8.1.4pre) Gecko/20070327 Minimo/0.020*/
const Device UA121524 = {S61545, 58, { &P18088,&P5448,&P17470,&P18092 }};	/*Useragent - Mozilla/6.0 (compatible; MSIE 7.01; Windows NT Windows CE)*/
const Device UA275592 = {S61546, 47, { &P18088,&P5448,&P17470,&P18092 }};	/*Useragent - Mozilla/1.1 (compatible; MSPIE 2.0; Windows CE)*/
const Device UA2676422 = {S61547, 139, { &P18447,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT30p Build/7.0.A.3.195) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734851 = {S61548, 132, { &P18447,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT30p Build/7.0.A.3.195) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2711125 = {S61549, 157, { &P18680,&P18066,&P18616,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; haw-US; KFJWI Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.1 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA156280 = {S61550, 26, { &P18088,&P17017,&P24643,&P18092 }};	/*Useragent - J2ME/UCWEB7.2.2.54/139/352*/
const Device UA1881917 = {S61551, 26, { &P18088,&P17017,&P24643,&P18092 }};	/*Useragent - J2ME/UCWEB7.2.2.54/139/351*/
const Device UA403355 = {S61552, 136, { &P18226,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U9500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2221007 = {S61553, 141, { &P18226,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U9500 Build/HuaweiU9500) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2339400 = {S61554, 132, { &P18226,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; U9500 Build/HuaweiU9500) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2787454 = {S61555, 63, { &P18226,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; U9500 Build/HuaweiU9500)*/
const Device UA2950725 = {S61556, 143, { &P18226,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; U9500E Build/HuaweiU9500E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA641248 = {S61557, 143, { &P22591,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-ca; SGH-I717D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA710579 = {S61558, 67, { &P22591,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SGH-I717D Build/GINGERBREAD)*/
const Device UA2203009 = {S61559, 136, { &P18651,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01D Build/S8230) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2538613 = {S61560, 136, { &P18651,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01D Build/S2050) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2552744 = {S61561, 58, { &P18651,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SH-01D Build/S8230)*/
const Device UA3066672 = {S61562, 136, { &P18651,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-01D Build/S8060) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2508651 = {S61563, 139, { &P20820,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; VS TOUCHTAB 8DC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2857022 = {S61564, 68, { &P20820,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; VS TOUCHTAB 8DC Build/JRO03H)*/
const Device UA3064981 = {S61565, 125, { &P14111,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Arc Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA389694 = {S61566, 75, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader949 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac7%2C1)*/
const Device UA511142 = {S61567, 81, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader949 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBookPro4%2C1)*/
const Device UA515146 = {S61568, 69, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - Spector%20Pro3731 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook5%2C1)*/
const Device UA516381 = {S61569, 79, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader1116 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook5%2C1)*/
const Device UA518450 = {S61570, 66, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - Spector%20Pro3731 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac9%2C1)*/
const Device UA521309 = {S61571, 82, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader1569 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBookPro5%2C5)*/
const Device UA530867 = {S61572, 66, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - Spector%20Pro2657 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac8%2C1)*/
const Device UA539369 = {S61573, 69, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - Spector%20Pro3950 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook4%2C1)*/
const Device UA571961 = {S61574, 66, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - Spector%20Pro3987 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac9%2C1)*/
const Device UA650813 = {S61575, 76, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader1116 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac9%2C1)*/
const Device UA692895 = {S61576, 78, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader884 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook4%2C1)*/
const Device UA700077 = {S61577, 78, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader949 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook3%2C1)*/
const Device UA718401 = {S61578, 79, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader1569 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook4%2C1)*/
const Device UA739329 = {S61579, 66, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - Spector%20Pro3950 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac9%2C1)*/
const Device UA780622 = {S61580, 75, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader884 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac8%2C1)*/
const Device UA1858786 = {S61581, 79, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader1444 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook4%2C1)*/
const Device UA1865155 = {S61582, 81, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader884 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBookPro4%2C1)*/
const Device UA1897836 = {S61583, 77, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - GrandReporterUpdater1.1.0 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook5%2C2)*/
const Device UA1904919 = {S61584, 66, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - Spector%20Pro4195 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac8%2C1)*/
const Device UA1949077 = {S61585, 79, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader1725 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook4%2C1)*/
const Device UA1974076 = {S61586, 66, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - Spector%20Pro4413 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac8%2C1)*/
const Device UA2279360 = {S61587, 82, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader1116 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBookPro4%2C1)*/
const Device UA2283550 = {S61588, 66, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - Spector%20Pro2743 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac9%2C1)*/
const Device UA2287668 = {S61589, 76, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader1569 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac7%2C1)*/
const Device UA2297358 = {S61590, 69, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - Spector%20Pro3987 CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook4%2C1)*/
const Device UA2574683 = {S61591, 82, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBook4%2C1)*/
const Device UA2594249 = {S61592, 79, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac8%2C1)*/
const Device UA2812777 = {S61593, 76, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader1725 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac8%2C1)*/
const Device UA2933927 = {S61594, 85, { &P17595,&P22475,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/438.16 Darwin/9.8.0 (i386) (MacBookPro5%2C5)*/
const Device UA443306 = {S61595, 114, { &P15364,&P5690,&P22481,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.7) Gecko/X Firefox/3.5.7 Flock/2.5.6 (.NET CLR 3.5.30729)*/
const Device UA463233 = {S61596, 103, { &P15364,&P5690,&P22481,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.16) Gecko/X Firefox/3.0.16 Flock/2.5.6 FBSMTWB*/
const Device UA2705799 = {S61597, 165, { &P22601,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ko-kr; SAMSUNG SHV-E300L Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2893204 = {S61598, 67, { &P23776,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGP321 Build/10.1.1.A.1.253)*/
const Device UA2898413 = {S61599, 142, { &P23776,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonySGP321 Build/10.1.1.A.1.307) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2936945 = {S61600, 138, { &P23776,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SGP321 Build/10.1.1.A.1.253) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2938161 = {S61601, 142, { &P23776,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SonySGP321 Build/10.1.1.A.1.253) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2943405 = {S61602, 138, { &P23776,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; SGP321 Build/10.1.1.A.1.307) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA266930 = {S61603, 54, { &P15364,&P18110,&P22524,&P18092 }};	/*Useragent - Midori/0.2.2 (X11; Linux i686; U; en-us) WebKit/531.2+*/
const Device UA289214 = {S61604, 47, { &P15364,&P18110,&P22524,&P18092 }};	/*Useragent - Midori/0.2 (X11; Linux; U; en-us) WebKit/531.2+*/
const Device UA514198 = {S61605, 43, { &P15364,&P18110,&P22524,&P18092 }};	/*Useragent - Midori/0.2 (X11; Linux; U; c) WebKit/531.2+*/
const Device UA703097 = {S61606, 96, { &P15364,&P18110,&P22524,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux; en-us) AppleWebKit/534+ (KHTML, like Gecko) Safari/534.7+ Midori/0.2*/
const Device UA752687 = {S61607, 47, { &P15364,&P18110,&P22524,&P18092 }};	/*Useragent - Midori/0.2 (X11; Linux; U; de-de) WebKit/531.2+*/
const Device UA794014 = {S61608, 47, { &P15364,&P18110,&P22524,&P18092 }};	/*Useragent - Midori/0.2 (X11; Linux; U; en-gb) WebKit/531.2+*/
const Device UA874777 = {S61609, 54, { &P15364,&P18110,&P22524,&P18092 }};	/*Useragent - Midori/0.2.2 (X11; Linux i686; U; en-gb) WebKit/531.2+*/
const Device UA1878448 = {S61610, 54, { &P15364,&P18110,&P22524,&P18092 }};	/*Useragent - Midori/0.2.2 (X11; Linux i686; U; de-de) WebKit/531.2+*/
const Device UA2210651 = {S61611, 56, { &P15364,&P18110,&P22524,&P18092 }};	/*Useragent - Midori/0.2.2 (X11; Linux x86_64; U; en-us) WebKit/531.2+*/
const Device UA2242566 = {S61612, 47, { &P15364,&P18110,&P22524,&P18092 }};	/*Useragent - Midori/0.2 (X11; Linux; U; es-ve) WebKit/531.2+*/
const Device UA2280554 = {S61613, 47, { &P15364,&P18110,&P22524,&P18092 }};	/*Useragent - Midori/0.2 (X11; Linux; U; fr-fr) WebKit/531.2+*/
const Device UA2492832 = {S61614, 47, { &P15364,&P18110,&P22524,&P18092 }};	/*Useragent - Midori/0.2 (X11; Linux; U; es-es) WebKit/531.2+*/
const Device UA2641447 = {S61615, 54, { &P15364,&P18110,&P22524,&P18092 }};	/*Useragent - Midori/0.2.0 (X11; Linux i686; U; en-us) WebKit/531.2+*/
const Device UA2681361 = {S61616, 54, { &P15364,&P18110,&P22524,&P18092 }};	/*Useragent - Midori/0.2.2 (X11; Linux i686; U; ru-ru) WebKit/531.2+*/
const Device UA2815728 = {S61617, 77, { &P15364,&P18110,&P22524,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux; es-ve; rv:1.9.0.2) Gecko/X Firefox/3.8 Midori/0.2*/
const Device UA2978767 = {S61618, 54, { &P15364,&P18110,&P22524,&P18092 }};	/*Useragent - Midori/0.2.2 (X11; Linux i686; U; ja-jp) WebKit/531.2+*/
const Device UA3040723 = {S61619, 47, { &P15364,&P18110,&P22524,&P18092 }};	/*Useragent - Midori/0.2 (X11; Linux; U; it-it) WebKit/531.2+*/
const Device UA2511749 = {S61620, 166, { &P18283,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9300 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2538000 = {S61621, 166, { &P18283,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-I9300 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2558490 = {S61622, 166, { &P18283,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; SGH-T999 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2603585 = {S61623, 166, { &P18283,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; SCH-I535 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2730731 = {S61624, 166, { &P18283,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; SCH-I535 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2953335 = {S61625, 166, { &P18283,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; SGH-T999 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3028936 = {S61626, 170, { &P18283,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9300 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3100063 = {S61627, 166, { &P18283,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-us ; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2670764 = {S61628, 86, { &P15364,&P7133,&P19995,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/4.0.0)*/
const Device UA2670770 = {S61629, 87, { &P15364,&P7133,&P19995,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/4.0.0)*/
const Device UA2675748 = {S61630, 117, { &P15364,&P7133,&P19995,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22 Sleipnir/4.0.1*/
const Device UA2675885 = {S61631, 86, { &P15364,&P7133,&P19995,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/4.0.1)*/
const Device UA2675985 = {S61632, 87, { &P15364,&P7133,&P19995,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/4.0.1)*/
const Device UA2676000 = {S61633, 124, { &P15364,&P7133,&P19995,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22 Sleipnir/4.0.1*/
const Device UA2862948 = {S61634, 79, { &P15364,&P7133,&P19995,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/4.0.0)*/
const Device UA33302 = {S61635, 69, { &P14256,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2370/E1055GJVKB3 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA40757 = {S61636, 136, { &P13724,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-i909 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA54878 = {S61637, 136, { &P13724,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SCH-i909 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA704401 = {S61638, 124, { &P13724,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SCH-i909 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA799295 = {S61639, 124, { &P13724,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-i909 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1875315 = {S61640, 136, { &P13724,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-I9088 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1959776 = {S61641, 136, { &P13724,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; zh-tw; SCH-i909 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028421 = {S61642, 129, { &P13724,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SCH-i909 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2029850 = {S61643, 129, { &P13724,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; zh-cn; SCH-i909 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2030882 = {S61644, 129, { &P13724,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-i909 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2543196 = {S61645, 129, { &P13724,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-I9088 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2813339 = {S61646, 60, { &P13724,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; SCH-i909 Build/FROYO)*/
const Device UA2566681 = {S61647, 150, { &P18317,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300T Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583959 = {S61648, 151, { &P18317,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300TG Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2603903 = {S61649, 151, { &P18317,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300TL Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2714934 = {S61650, 156, { &P18673,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S7562 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2938012 = {S61651, 157, { &P18673,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-S7562i Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3067200 = {S61652, 128, { &P24366,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SM-T311 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA64679 = {S61653, 170, { &P14834,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 Nokia5730s-1/200.12.87; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.1.1*/
const Device UA253769 = {S61654, 34, { &P15364,&P17017,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5;*/
const Device UA2029362 = {S61655, 131, { &P21311,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CMP741E Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2031359 = {S61656, 124, { &P21311,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; CMP741E Build/ITL41D) AppleWebKit/434.50 (KHTML, like Gecko) Version/4.0 Mobile*/
const Device UA2676431 = {S61657, 126, { &P15869,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xoom Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676669 = {S61658, 131, { &P15869,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ505 Build/Moto_MZ505) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708562 = {S61659, 125, { &P15869,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Xoom Build/IML77) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708383 = {S61660, 198, { &P15991,&P18598,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2 jellybean # ariel-tbc /UCHIHA; en-CA ; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2871477 = {S61661, 141, { &P23505,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Vodafone 875 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2939523 = {S61662, 134, { &P23505,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Vodafone 875 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2193199 = {S61663, 178, { &P18506,&P5043,&P24671,&P18092 }};	/*Useragent - Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia305) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2430861 = {S61664, 89, { &P18115,&P5764,&P4719,&P18092 }};	/*Useragent - Opera/9.80 (BlackBerry; Opera Mini/4.3.25172/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2568071 = {S61665, 138, { &P13064,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I727 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2603860 = {S61666, 146, { &P13064,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I727 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2717073 = {S61667, 147, { &P13064,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I727 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936508 = {S61668, 69, { &P13064,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SAMSUNG-SGH-I727 Build/JDQ39E)*/
const Device UA54136 = {S61669, 112, { &P14633,&P17484,&P18336,&P18092 }};	/*Useragent - SAMSUNG-GT-I6410-Vodafone/I6410BUJC1 Linux/X2/R1 Opera/9.6 SMS-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA63255 = {S61670, 74, { &P14633,&P17484,&P18336,&P18092 }};	/*Useragent - Opera/9.62 (X11;Linux i686; U; en) Presto/2.2.0/SAMSUNG-GT-I6410-Vodafone/*/
const Device UA2535743 = {S61671, 70, { &P15767,&P18432,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPad2,1; iPhone OS 6.0.1; en_US; en) AppleWebKit/420+*/
const Device UA2581062 = {S61672, 70, { &P15767,&P18432,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPad2,5; iPhone OS 6.0.2; en_US; en) AppleWebKit/420+*/
const Device UA2665130 = {S61673, 68, { &P15767,&P18432,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPad2,1; iPhone OS 6.0; en_US; en) AppleWebKit/420+*/
const Device UA2601408 = {S61674, 142, { &P18462,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2766567 = {S61675, 135, { &P18462,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2675159 = {S61676, 135, { &P17595,&P18207,&P22466,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36*/
const Device UA2781690 = {S61677, 135, { &P17595,&P18207,&P22466,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36*/
const Device UA2866400 = {S61678, 135, { &P17595,&P18207,&P22466,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36*/
const Device UA3096216 = {S61679, 135, { &P17595,&P18207,&P22466,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_5) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36*/
const Device UA3068330 = {S61680, 145, { &P19891,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2404745 = {S61681, 151, { &P18219,&P5843,&P24673,&P18092 }};	/*Useragent - Nokia202/2.0 (20.28) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia202) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2736989 = {S61682, 90, { &P18219,&P5843,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; Nokia203/20.37) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2985406 = {S61683, 151, { &P18219,&P5843,&P24673,&P18092 }};	/*Useragent - Nokia203/2.0 (20.36) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia203) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2567003 = {S61684, 140, { &P18736,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One SV Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2892849 = {S61685, 137, { &P22382,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2944518 = {S61686, 130, { &P22382,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2974716 = {S61687, 130, { &P20035,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022838 = {S61688, 131, { &P20035,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2975959 = {S61689, 137, { &P21628,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022180 = {S61690, 138, { &P21628,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3065133 = {S61691, 137, { &P24063,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; IdeaTab S6000-F Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080218 = {S61692, 136, { &P24063,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; IdeaTab S6000-H Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3093665 = {S61693, 136, { &P24063,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; IdeaTab S6000-F Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2262004 = {S61694, 27, { &P15364,&P18148,&P17470,&P18092 }};	/*Useragent - ZDM/4.0; Windows Mobile 8.0*/
const Device UA2558097 = {S61695, 137, { &P18243,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P880 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2679615 = {S61696, 47, { &P15364,&P18570,&P22245,&P18092 }};	/*Useragent - Flipboard/2.0.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA40011 = {S61697, 46, { &P13692,&P6476,&P3625,&P18092 }};	/*Useragent - LGE-MN180/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA2404100 = {S61698, 98, { &P15364,&P22192,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.1.8244(Windows;Microsoft Windows (6.0.6002.2);en;kml:2.2;client:Free;type:default)*/
const Device UA2460721 = {S61699, 98, { &P15364,&P22192,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.2.8415(Windows;Microsoft Windows (6.0.6002.2);en;kml:2.2;client:Free;type:default)*/
const Device UA2570299 = {S61700, 98, { &P15364,&P22192,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.0.6002.2);en;kml:2.2;client:Free;type:default)*/
const Device UA2601663 = {S61701, 98, { &P15364,&P22192,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.0.6001.1);en;kml:2.2;client:Free;type:default)*/
const Device UA2667187 = {S61702, 98, { &P15364,&P22192,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.0.6002.2);de;kml:2.2;client:Free;type:default)*/
const Device UA2667642 = {S61703, 98, { &P15364,&P22192,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.0.6002.2);nl;kml:2.2;client:Free;type:default)*/
const Device UA2697056 = {S61704, 98, { &P15364,&P22192,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.2.8415(Windows;Microsoft Windows (6.0.6002.2);de;kml:2.2;client:Free;type:default)*/
const Device UA2700311 = {S61705, 98, { &P15364,&P22192,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.0.6002.2);fr;kml:2.2;client:Free;type:default)*/
const Device UA2732837 = {S61706, 98, { &P15364,&P22192,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.0.6002.2);it;kml:2.2;client:Free;type:default)*/
const Device UA2749707 = {S61707, 98, { &P15364,&P22192,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.0.6002.2);es;kml:2.2;client:Free;type:default)*/
const Device UA458774 = {S61708, 59, { &P20978,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID7120 Build/GRI40)*/
const Device UA458795 = {S61709, 135, { &P20978,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID7120 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA459045 = {S61710, 136, { &P20978,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID7120A Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2507732 = {S61711, 123, { &P21853,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Y3 Build/HCL) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2706153 = {S61712, 131, { &P21853,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Connect-3G Build/HCL) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2854159 = {S61713, 121, { &P21853,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; Y3 Build/HCL) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2676239 = {S61714, 138, { &P19845,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE N9120 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735309 = {S61715, 131, { &P19845,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE N9120 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2029057 = {S61716, 144, { &P15438,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Sensation Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2236212 = {S61717, 144, { &P15438,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Sensation Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258001 = {S61718, 144, { &P15438,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Sensation Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2353938 = {S61719, 144, { &P15438,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Sensation Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2354213 = {S61720, 66, { &P15438,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC Sensation Build/JZO54K)*/
const Device UA2557686 = {S61721, 62, { &P15438,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Sensation Build/JZO54K)*/
const Device UA2042095 = {S61722, 61, { &P15364,&P18346,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10; rv:15.0) Gecko/X Firefox/15.0.1*/
const Device UA2753343 = {S61723, 130, { &P18437,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2872991 = {S61724, 130, { &P18437,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8005 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2969247 = {S61725, 139, { &P16292,&P21637,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Galaxy Nexus Build/JWR66V) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA177760 = {S61726, 82, { &P17595,&P17294,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.4.11; U; en) Presto/2.8.131 Version/11.10*/
const Device UA203431 = {S61727, 82, { &P17595,&P17294,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.4.11; U; en) Presto/2.8.131 Version/11.11*/
const Device UA2595591 = {S61728, 142, { &P20543,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lenovo A660 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695055 = {S61729, 135, { &P20543,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; a660 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2701573 = {S61730, 140, { &P20543,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; Lenovo A660; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2762585 = {S61731, 64, { &P20543,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo A660 Build/IMM76D)*/
const Device UA2671112 = {S61732, 139, { &P18736,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676872 = {S61733, 132, { &P18736,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3065579 = {S61734, 141, { &P14969,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066539 = {S61735, 142, { &P14969,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire SV Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3035591 = {S61736, 134, { &P22727,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGP312 Build/10.1.C.0.370) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Safari/537.36*/
const Device UA14392 = {S61737, 47, { &P13856,&P6476,&P3625,&P18092 }};	/*Useragent - LGE-VN270L/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA14393 = {S61738, 47, { &P13856,&P6476,&P3625,&P18092 }};	/*Useragent - LGE-VN270P/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA45020 = {S61739, 46, { &P13856,&P6476,&P3625,&P18092 }};	/*Useragent - LGE-VN270/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA2367023 = {S61740, 133, { &P12280,&P17146,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A403 Safari/8536.25*/
const Device UA2387493 = {S61741, 133, { &P12280,&P17146,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2484174 = {S61742, 135, { &P12280,&P17146,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2532927 = {S61743, 135, { &P12280,&P17146,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2661077 = {S61744, 133, { &P12280,&P17146,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B143 Safari/8536.25*/
const Device UA2663448 = {S61745, 133, { &P12280,&P17146,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2986801 = {S61746, 135, { &P12280,&P17146,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA2436427 = {S61747, 132, { &P15932,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P3110 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2534061 = {S61748, 131, { &P15932,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P3110 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2621899 = {S61749, 130, { &P15932,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; GT-P3110 Build/JOP40C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2673536 = {S61750, 146, { &P17989,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N8000_USA_Cricket Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676271 = {S61751, 146, { &P17989,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V8000_USA_Cricket Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3034017 = {S61752, 154, { &P18635,&P21637,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus 4 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA205694 = {S61753, 115, { &P15364,&P5690,&P22450,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML, like Gecko) Iron/12.0.750.0 Chrome/12.0.750.0 Safari/534.30*/
const Device UA211920 = {S61754, 122, { &P15364,&P5690,&P22450,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Iron/12.0.750.0 Chrome/12.0.750.0 Safari/534.30*/
const Device UA553439 = {S61755, 115, { &P15364,&P5690,&P22450,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.30 (KHTML, like Gecko) Iron/12.0.750.0 Chrome/12.0.750.0 Safari/534.30*/
const Device UA2204111 = {S61756, 100, { &P15364,&P18346,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2202584 = {S61757, 83, { &P17976,&P18066,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.4; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2202588 = {S61758, 83, { &P17976,&P18066,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.3; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2422898 = {S61759, 93, { &P17976,&P18066,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.3; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10,gzip(gfe)*/
const Device UA2484017 = {S61760, 93, { &P17976,&P18066,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.4; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10,gzip(gfe)*/
const Device UA2564446 = {S61761, 162, { &P18283,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2734788 = {S61762, 177, { &P18283,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9300 Build/_miui.polimorph_ v5.c) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2996760 = {S61763, 170, { &P18283,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; GT-I9300 Build/miui.es JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA3086835 = {S61764, 164, { &P18283,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9300 Build/By Rohit) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA3088684 = {S61765, 161, { &P18283,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, zoals Gecko) Versie/4.0 MobielSafari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2707835 = {S61766, 156, { &P14826,&P5843,&P24677,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac1-01) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2892151 = {S61767, 61, { &P22600,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SHV-E300K Build/JDQ39)*/
const Device UA3036766 = {S61768, 172, { &P22600,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ko-kr; SHV-E300K Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 NAVER(inapp; search; 230; 4.7.7)*/
const Device UA3066207 = {S61769, 141, { &P18462,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA63408 = {S61770, 73, { &P16601,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C3530/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA1996373 = {S61771, 133, { &P21054,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3;en-au; ZTE T760 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032710 = {S61772, 136, { &P21054,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; ZTE T760 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2202156 = {S61773, 95, { &P17595,&P17297,&P20792,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5; U; Edition MAS; en) Presto/2.10.229 Version/11.66*/
const Device UA2229505 = {S61774, 186, { &P14862,&P5204,&P24671,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia2700c-2) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2401474 = {S61775, 136, { &P23193,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCL21 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2951236 = {S61776, 58, { &P23193,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCL21 Build/IMM76D)*/
const Device UA2567126 = {S61777, 142, { &P19763,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-02E Build/10.1.D.0.322) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2585168 = {S61778, 134, { &P19763,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6602 Build/10.1.A.1.350) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2601375 = {S61779, 141, { &P19763,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6602 Build/10.1.A.1.434) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2885350 = {S61780, 78, { &P15364,&P21460,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (MSIE 9.0; Windows NT 6.3; WOW64; Trident/7.0; rv:11.0) like Gecko*/
const Device UA2899409 = {S61781, 83, { &P15364,&P21460,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (MSIE 9.0; Windows NT 6.3; Win64; x64; Trident/7.0; rv:11.0) like Gecko*/
const Device UA2944090 = {S61782, 71, { &P15364,&P21460,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (MSIE 9.0; Windows NT 6.3; Trident/7.0; rv:11.0) like Gecko*/
const Device UA2991262 = {S61783, 70, { &P15364,&P21460,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.3; WOW64; Trident/7.0)*/
const Device UA3022897 = {S61784, 156, { &P14669,&P5843,&P24679,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-00) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA630029 = {S61785, 83, { &P12835,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-S3770K/S3770KDDKK2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA2678919 = {S61786, 129, { &P18454,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGPT13 Build/TJS0166) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2679778 = {S61787, 129, { &P18454,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGPT12 Build/TJS0166) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3132008 = {S61788, 137, { &P19861,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA2368733 = {S61789, 130, { &P21907,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT8088 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2675320 = {S61790, 138, { &P19894,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2714649 = {S61791, 131, { &P19894,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2753266 = {S61792, 138, { &P13141,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I727R Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782639 = {S61793, 138, { &P13141,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E110S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2975050 = {S61794, 141, { &P19778,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP9090 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3065077 = {S61795, 133, { &P20129,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MI 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094071 = {S61796, 134, { &P20129,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MI 2A Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA64420 = {S61797, 52, { &P12209,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KS360-Orange/V10g Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA64454 = {S61798, 40, { &P12209,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KS360 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA773312 = {S61799, 102, { &P15364,&P5690,&P22516,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.8.1.7pre) Gecko/X Firefox/2.0.0.6 Navigator/9.0b3*/
const Device UA2131030 = {S61800, 111, { &P15364,&P5690,&P22516,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.11pre) Gecko/X Firefox/2.0.0.11 Navigator/9.0.0.5 GTB5*/
const Device UA2168140 = {S61801, 138, { &P18459,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS950 4G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2361891 = {S61802, 83, { &P18141,&P19823,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 2.00) AppleWebKit/536.26 (KHTML, like Gecko) Silk/3.2*/
const Device UA2716168 = {S61803, 81, { &P23129,&P22283,&P4392,&P18092 }};	/*Useragent - Mozilla/4.0 (Brew MP 1.0.4; U; en-us; Kyocera; NetFront/4.1/AMB) Sprint S2151-BST*/
const Device UA2782726 = {S61804, 81, { &P23129,&P22283,&P4392,&P18092 }};	/*Useragent - Mozilla/4.0 (Brew MP 1.0.4; U; en-us; Kyocera; NetFront/4.1/AMB) Sprint S2151-PLB*/
const Device UA67124 = {S61805, 148, { &P14642,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Motorola-XT5 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA67164 = {S61806, 150, { &P14642,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Motorola-XT502 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1869583 = {S61807, 150, { &P14642,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; Motorola-XT500 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018161 = {S61808, 72, { &P14642,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Motorola-XT502 Build/ERE27)*/
const Device UA2022683 = {S61809, 136, { &P14642,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; Motorola-XT502 Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA801414 = {S61810, 157, { &P14967,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P500 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026187 = {S61811, 157, { &P14967,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P500 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422382 = {S61812, 159, { &P14967,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P500 Build/IMM76L; CyanogenMod-ICySnap) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2764581 = {S61813, 163, { &P14967,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P500 Build/NightlyFourE; CyanogenMod-9.1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558659 = {S61814, 152, { &P18557,&P20706,&P24676,&P18092 }};	/*Useragent - Nokia308/2.0 (05.85) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia308) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2599214 = {S61815, 136, { &P21869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZP950 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2713181 = {S61816, 137, { &P21869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZP950+ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2991258 = {S61817, 137, { &P21869,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZP950H Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3065731 = {S61818, 145, { &P12814,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I777 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081864 = {S61819, 137, { &P12814,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I777 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA830441 = {S61820, 162, { &P13064,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I727 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031141 = {S61821, 166, { &P13064,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I727 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1951966 = {S61822, 155, { &P16553,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959V Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1993537 = {S61823, 158, { &P16553,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959V Build/RemICS-UX; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA55140 = {S61824, 145, { &P14872,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018081 = {S61825, 67, { &P14872,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SPH-D700 Build/ECLAIR)*/
const Device UA87703 = {S61826, 92, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyPhoneS21/MTK Release/V0.6 Mar-06-2008 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA87893 = {S61827, 96, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyPhoneB22Duo/MTK Release/V0.44 Apr-01-2008 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA88661 = {S61828, 94, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyPhoneTV21Duo/MTK Release/V6 Jan-12-2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA88876 = {S61829, 97, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyPhoneB23DuoAM/MTK Release/V0.1 Aug-25-2008 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA88934 = {S61830, 94, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyPhoneMS28Duo/MTK Release/V7 May-04-2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA88877 = {S61831, 94, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyPhoneB21AM/MTK Release/V0.3 Aug-23-2008 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA89139 = {S61832, 93, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyPhoneQ21Duo/MTK Release/V7 Apl-01-2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA90778 = {S61833, 93, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyPhoneQ19iDuo/MTK Release/V3 AUG-3-2010 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA179493 = {S61834, 85, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyphoneTW1DUO/MTKRelease/2011/02/25Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0*/
const Device UA420139 = {S61835, 85, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyphoneTW8DUO/MTKRelease/2011/03/03Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0*/
const Device UA440192 = {S61836, 88, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyphoneQW29 DUO/MTKRelease/MAY-27-2011Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0*/
const Device UA506391 = {S61837, 87, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyphoneQP29Duo/MTKRelease/NOV-11-2010Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0*/
const Device UA554708 = {S61838, 99, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyPhoneQTV20iDuo/MTK Release/V1.8 JULY-22-2010 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA578606 = {S61839, 84, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyphoneT18TV/MTKRelease/2011/09/19Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0*/
const Device UA639132 = {S61840, 88, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyphoneQW29 DUO/MTKRelease/DEC-18-2010Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0*/
const Device UA900824 = {S61841, 88, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyphoneT23 Duo/MTKRelease/JULY-08-2010Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0*/
const Device UA1979525 = {S61842, 88, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyphoneTW28Duo/MTK Release/Feb-14-2012Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0*/
const Device UA2134210 = {S61843, 92, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyPhoneB12 Duo/MTKRelease/V2.0 MAY-14-2010Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0*/
const Device UA2149082 = {S61844, 87, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyPhoneT28TVDuo/MTKRelease/2011/09/29Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0*/
const Device UA2158381 = {S61845, 85, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyphoneTW1Duo/MTKRelease/2011/11/24Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0*/
const Device UA2260447 = {S61846, 90, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyphoneTW29TVDuo/MTK Release/Feb-20-2012Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0*/
const Device UA2273446 = {S61847, 98, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyPhoneQTV20Duo/MTK Release/V1.8 JULY-22-2010 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA2299877 = {S61848, 96, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyPhoneQTV20iDuo/MTK Release/V7 AUG-25-2011 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA2303942 = {S61849, 94, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyPhoneB16TV Duo/MTKRelease/V0.2 AUG-22-2011Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0*/
const Device UA2310667 = {S61850, 92, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyPhoneB16TV Duo/MTKRelease/V4 AUG-26-2011Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0*/
const Device UA2370185 = {S61851, 87, { &P15364,&P22840,&P4780,&P18092 }};	/*Useragent - MyphoneQW30Duo/MTKRelease/FEB-22-2011Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0*/
const Device UA2415967 = {S61852, 136, { &P23160,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KB901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2478024 = {S61853, 58, { &P23160,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; KB901 Build/IMM76D)*/
const Device UA3086459 = {S61854, 129, { &P23160,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KB901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2752962 = {S61855, 137, { &P18673,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773717 = {S61856, 138, { &P18673,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562L Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2756427 = {S61857, 107, { &P18435,&P16110,&P22827,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; ARM; Touch; WPDesktop) UCBrowser/3.0.0.285*/
const Device UA2948284 = {S61858, 107, { &P18435,&P16110,&P22827,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; ARM; Touch; WPDesktop) UCBrowser/3.0.1.302*/
const Device UA2973271 = {S61859, 129, { &P17686,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ601 Build/I.7.1-45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2975044 = {S61860, 145, { &P12814,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I777 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976110 = {S61861, 145, { &P12814,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I777 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022555 = {S61862, 137, { &P12814,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I777 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022738 = {S61863, 137, { &P12814,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-I777 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3068203 = {S61864, 137, { &P23293,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LS720 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA50207 = {S61865, 60, { &P11886,&P7063,&P3606,&P18092 }};	/*Useragent - BlackBerry8700/4.1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2530743 = {S61866, 162, { &P18283,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-I535 Build/JOP40G; Carbon-Carbon-JB-v1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567828 = {S61867, 161, { &P18283,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I535 Build/JDQ39; Carbon-Carbon-JB-v1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2674165 = {S61868, 163, { &P18283,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I535 Build/JDQ39; Carbon-CARBON-JB-v1.6.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2710514 = {S61869, 161, { &P18283,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/JDQ39; Carbon-CARBON-JB-v1.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2710836 = {S61870, 163, { &P18283,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T999 Build/JDQ39; Carbon-CARBON-JB-v1.6.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2738713 = {S61871, 162, { &P18283,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I535 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752810 = {S61872, 162, { &P18283,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T999 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2800258 = {S61873, 162, { &P18283,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2667364 = {S61874, 156, { &P22980,&P18066,&P18062,&P18092 }};	/*Useragent - TECNO D5 Linux/3.0.13 Android/4.0.4 Release/01.28.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2780796 = {S61875, 139, { &P22980,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TECNO D5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2954795 = {S61876, 61, { &P22980,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; TECNO D5 Build/IMM76D)*/
const Device UA2753543 = {S61877, 139, { &P21604,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C2105 Build/15.0.A.1.31) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA468800 = {S61878, 79, { &P15364,&P7127,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-GB; rv:1.7.6) Gecko/X Firefox/1.0.1*/
const Device UA6437 = {S61879, 143, { &P12460,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; SCH-R880 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA13640 = {S61880, 145, { &P12460,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SCH-R880 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA444091 = {S61881, 67, { &P12460,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SCH-R880 Build/ECLAIR)*/
const Device UA3067047 = {S61882, 137, { &P18488,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA21655 = {S61883, 141, { &P15315,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; T3020 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA76144 = {S61884, 139, { &P15315,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; T3020 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2020202 = {S61885, 63, { &P15315,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; T3020 Build/ERE27)*/
const Device UA2205788 = {S61886, 118, { &P13977,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaN73) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile*/
const Device UA2031569 = {S61887, 131, { &P21060,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID9042 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2794871 = {S61888, 152, { &P20993,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HUAWEI P2-6011 Build/HuaweiP2-6011) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049651 = {S61889, 74, { &P20993,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HUAWEI P2-6011 Build/HuaweiP2-6011)*/
const Device UA3088556 = {S61890, 163, { &P20993,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; HUAWEI P2-6011-orange Build/hwP2-6011C109B007) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3066877 = {S61891, 137, { &P18458,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2974291 = {S61892, 125, { &P14111,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Arc Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA684860 = {S61893, 141, { &P18720,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; Micromax A50 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2775084 = {S61894, 65, { &P18720,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; Micromax A50 Build/GRK39F)*/
const Device UA2676779 = {S61895, 138, { &P20714,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID8065 Build/ICS.MID8065.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA61479 = {S61896, 110, { &P12728,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S5511T/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 S5511AJKD4*/
const Device UA2472995 = {S61897, 109, { &P12728,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S5511/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 S5511JJKI1*/
const Device UA642232 = {S61898, 41, { &P14358,&P5890,&P24665,&P18092 }};	/*Useragent - NokiaE63/UC Browser8.0.4.121/28/444/UCWEB*/
const Device UA734631 = {S61899, 29, { &P14358,&P5890,&P24665,&P18092 }};	/*Useragent - NokiaE63/UCWEB8.0.3.99/28/999*/
const Device UA3030594 = {S61900, 157, { &P18595,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3032673 = {S61901, 157, { &P18595,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3065484 = {S61902, 137, { &P22595,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919V Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA67751 = {S61903, 95, { &P18073,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; T9295)*/
const Device UA1994546 = {S61904, 171, { &P20215,&P5629,&P15760,&P18092 }};	/*Useragent - LG-P692/Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; LG-P692 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2462369 = {S61905, 77, { &P15364,&P5690,&P22556,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:17.0) Gecko/X Firefox/17.0 IceDragon/18.0.1.0*/
const Device UA2489238 = {S61906, 77, { &P15364,&P5690,&P22556,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:18.0) Gecko/X Firefox/18.0 IceDragon/18.0.3.1*/
const Device UA2487653 = {S61907, 117, { &P15364,&P17285,&P22464,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17*/
const Device UA2487939 = {S61908, 124, { &P15364,&P17285,&P22464,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17*/
const Device UA904336 = {S61909, 71, { &P15364,&P7133,&P18514,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:13.0) Gecko/X Thunderbird/13.0.1*/
const Device UA1996400 = {S61910, 136, { &P21992,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.6; en-us; A9550 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2685840 = {S61911, 136, { &P21992,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; A9550 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2817707 = {S61912, 58, { &P21992,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.6; A9550 Build/IML74K)*/
const Device UA2177490 = {S61913, 130, { &P20720,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN7G2I Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2707549 = {S61914, 154, { &P22961,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; ALCATEL ONE TOUCH 6010D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2995269 = {S61915, 76, { &P22961,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ALCATEL ONE TOUCH 6010D Build/JRO03C)*/
const Device UA2960974 = {S61916, 143, { &P21648,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9505G Build/JWR66N.S005) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2972446 = {S61917, 143, { &P21648,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9505G Build/JWR66V.S001) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3030141 = {S61918, 143, { &P21648,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9505G Build/JWR66V.S014) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3031355 = {S61919, 65, { &P21648,&P21637,&P23997,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.3; GT-I9505G Build/JWR66V.S014)*/
const Device UA3093549 = {S61920, 143, { &P21648,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9505G Build/JWR66Y.S003) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA164248 = {S61921, 79, { &P15364,&P5690,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6545; Pro)*/
const Device UA164249 = {S61922, 79, { &P15364,&P5690,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.4518; Pro)*/
const Device UA167186 = {S61923, 79, { &P15364,&P5690,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6425; Pro)*/
const Device UA178030 = {S61924, 79, { &P15364,&P5690,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6554; Pro)*/
const Device UA245344 = {S61925, 79, { &P15364,&P5690,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6562; Pro)*/
const Device UA270143 = {S61926, 79, { &P15364,&P5690,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6535; Pro)*/
const Device UA465111 = {S61927, 79, { &P15364,&P5690,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6529; Pro)*/
const Device UA516599 = {S61928, 79, { &P15364,&P5690,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6213; Pro)*/
const Device UA575482 = {S61929, 79, { &P15364,&P5690,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6607; Pro)*/
const Device UA628899 = {S61930, 79, { &P15364,&P5690,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6320; Pro)*/
const Device UA2010485 = {S61931, 79, { &P15364,&P5690,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6662; Pro)*/
const Device UA2464123 = {S61932, 79, { &P15364,&P5690,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 5.2; Microsoft Office Outlook 12.0.6662; Pro)*/
const Device UA2472202 = {S61933, 79, { &P15364,&P5690,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6020; Pro)*/
const Device UA2806301 = {S61934, 79, { &P15364,&P5690,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 5.2; Microsoft Office Outlook 12.0.6607; Pro)*/
const Device UA3083621 = {S61935, 79, { &P15364,&P5690,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 5.1; Microsoft Office Outlook 12.0.6683; Pro)*/
const Device UA408511 = {S61936, 107, { &P15364,&P18110,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.1 Safari/532.0*/
const Device UA775638 = {S61937, 116, { &P15364,&P18110,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.198.0 Safari/532.0*/
const Device UA2144818 = {S61938, 142, { &P15364,&P18110,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; LeOS1.0; zh-cn; lepad_001b Build/PQXU100.4.0091.X) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.33 Safari/532.0*/
const Device UA2308582 = {S61939, 142, { &P15364,&P18110,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; LeOS1.0; zh-cn; lepad_001b Build/PQXU100.4.0097.X) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.33 Safari/532.0*/
const Device UA2542028 = {S61940, 142, { &P15364,&P18110,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; LeOS1.0; zh-cn; lepad_001n Build/PQXU100.4.0073.X) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.33 Safari/532.0*/
const Device UA2222935 = {S61941, 136, { &P21119,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SoftwinerEvb Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2230843 = {S61942, 142, { &P21119,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SoftwinerEvb Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2398569 = {S61943, 136, { &P21119,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SoftwinerEvb Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2413227 = {S61944, 142, { &P21119,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SoftwinerEvb Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2418087 = {S61945, 143, { &P21119,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Softwinerf900 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2504336 = {S61946, 143, { &P21119,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Softwinerf721 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2579420 = {S61947, 143, { &P21119,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Softwinerf761 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2686461 = {S61948, 137, { &P21119,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Softwinerq702 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2975543 = {S61949, 137, { &P18682,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-M950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA49219 = {S61950, 139, { &P12961,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; u8820 Build/HuaweiU8820) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA758362 = {S61951, 139, { &P12961,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; U8820 Build/HuaweiU8820) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2812051 = {S61952, 63, { &P12961,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; U8820 Build/HuaweiU8820)*/
const Device UA705948 = {S61953, 140, { &P19785,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; CT1002 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA924178 = {S61954, 79, { &P14358,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE63)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA1987739 = {S61955, 30, { &P14358,&P5890,&P24671,&P18092 }};	/*Useragent - NokiaE63/UCWEB8.5.0.163/28/999*/
const Device UA2594333 = {S61956, 119, { &P14358,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiae63-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile*/
const Device UA2753790 = {S61957, 138, { &P23905,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100P Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755359 = {S61958, 138, { &P23905,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100P Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3066522 = {S61959, 141, { &P13124,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP9070 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA715817 = {S61960, 72, { &P18088,&P5208,&P17470,&P18092 }};	/*Useragent - BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 90005; Lang: zhcn) ucweb-squid*/
const Device UA1886993 = {S61961, 71, { &P18088,&P5208,&P17470,&P18092 }};	/*Useragent - BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 2497; Lang: zhcn) ucweb-squid*/
const Device UA2409430 = {S61962, 72, { &P18088,&P5208,&P17470,&P18092 }};	/*Useragent - BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 39054; Lang: zhcn) ucweb-squid*/
const Device UA2455606 = {S61963, 111, { &P18088,&P5208,&P17470,&P18092 }};	/*Useragent - BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 600; Lang: zhcn; Profile/MIDP-2.0_Configuration/CLDC-1.1) ucweb-squid*/
const Device UA3023686 = {S61964, 160, { &P14862,&P5204,&P24679,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2700c-2) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3060348 = {S61965, 156, { &P14862,&P5204,&P24679,&P18092 }};	/*Useragent - Nokia2700c/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2700c) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA2425798 = {S61966, 133, { &P17615,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; SHV-E230S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2483094 = {S61967, 133, { &P17615,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; SHV-E230K Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2483309 = {S61968, 133, { &P17615,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ko-kr; SHV-E230L Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2508107 = {S61969, 133, { &P17615,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E230K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2508698 = {S61970, 133, { &P17615,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E230S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2581735 = {S61971, 133, { &P17615,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E230L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2893930 = {S61972, 140, { &P17615,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E230K/KKVME2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3031037 = {S61973, 140, { &P17615,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E230K/KKVMF2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3065609 = {S61974, 138, { &P21264,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PadFone Infinity Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3068051 = {S61975, 145, { &P21264,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PadFone Infinity Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067921 = {S61976, 146, { &P16263,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 10 LINK Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080660 = {S61977, 146, { &P16263,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 7 Vogue Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080823 = {S61978, 143, { &P16263,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Telpad Dual S Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3081779 = {S61979, 136, { &P16263,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; dtab01 Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3083362 = {S61980, 146, { &P16263,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 7 Youth Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA910885 = {S61981, 151, { &P17346,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8J2 Safari/7534.48.3*/
const Device UA911348 = {S61982, 151, { &P17346,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8G4 Safari/7534.48.3*/
const Device UA911591 = {S61983, 151, { &P17346,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8F190 Safari/7534.48.3*/
const Device UA911560 = {S61984, 151, { &P17346,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8L1 Safari/7534.48.3*/
const Device UA911893 = {S61985, 151, { &P17346,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8H7 Safari/7534.48.3*/
const Device UA912129 = {S61986, 151, { &P17346,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_4 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/8K2 Safari/7534.48.3*/
const Device UA2600201 = {S61987, 148, { &P17346,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 435 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/19.0.1084.60 Mobile/8L1 Safari/7534.48.3*/
const Device UA2659355 = {S61988, 148, { &P17346,&P17090,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 433 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML like Gecko) CriOS/19.0.1084.60 Mobile/8J2 Safari/7534.48.3*/
const Device UA1953349 = {S61989, 82, { &P20983,&P17979,&P18062,&P18092 }};	/*Useragent - SAMSUNG-GT-I9308_TD/1.0 Android/4.0.3 Release/02.15.2012 Browser/AppleWebKit534.30*/
const Device UA2567124 = {S61990, 142, { &P13124,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP9070 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2898940 = {S61991, 131, { &P21793,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9205 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2937761 = {S61992, 60, { &P21793,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9205 Build/JDQ39)*/
const Device UA2945877 = {S61993, 161, { &P16292,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2947553 = {S61994, 160, { &P16292,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2570280 = {S61995, 108, { &P15364,&P17350,&P20429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; ; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/7.0.3.8542 Safari/532.4*/
const Device UA2563065 = {S61996, 152, { &P18221,&P5843,&P24676,&P18092 }};	/*Useragent - Nokia302/5.0 (14.78) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia302) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2676676 = {S61997, 126, { &P18282,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A200 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA430161 = {S61998, 95, { &P18293,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; ZTE; Tania)*/
const Device UA9437 = {S61999, 106, { &P12516,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-S8003/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA62392 = {S62000, 109, { &P12516,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-S8003/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA90929 = {S62001, 71, { &P12280,&P17146,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.9A334 (iPhone; U; CPU OS 5_0 like Mac OS X; en_us)*/
const Device UA91002 = {S62002, 73, { &P12280,&P17146,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.9A405 (iPhone; U; CPU OS 5_0_1 like Mac OS X; en_us)*/
const Device UA162934 = {S62003, 132, { &P15364,&P5690,&P22446,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Iron/8.0.555.0 Chrome/8.0.555.0 Safari/534.10*/
const Device UA498711 = {S62004, 132, { &P15364,&P5690,&P22446,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Iron/8.0.555.0 Chrome/8.0.555.0 Safari/534.10*/
const Device UA199603 = {S62005, 111, { &P18086,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Microsoft; XDeviceEmulator)*/
const Device UA310522 = {S62006, 100, { &P18086,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Microsoft; CEPC)*/
const Device UA2509671 = {S62007, 93, { &P18364,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaC5-05) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA659722 = {S62008, 143, { &P21068,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE C1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA756638 = {S62009, 131, { &P21068,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE C1 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA905096 = {S62010, 136, { &P21068,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE C1 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2032063 = {S62011, 67, { &P21068,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; MI-ONE C1 Build/GINGERBREAD)*/
const Device UA2032811 = {S62012, 136, { &P21068,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE C1 Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2040846 = {S62013, 132, { &P18465,&P18110,&P18617,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.1 Safari/535.19 Silk-Accelerated=true*/
const Device UA2236375 = {S62014, 132, { &P20349,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Novo7Flame Build/X.qm) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2350817 = {S62015, 132, { &P20349,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Novo7Flame Build/X.tm) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA15889 = {S62016, 72, { &P11841,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia5220XpressMusic/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA61740 = {S62017, 61, { &P11841,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia5220/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA1870671 = {S62018, 62, { &P15364,&P16110,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; U; en) Presto/2.7.62 Version/11.01*/
const Device UA2148377 = {S62019, 62, { &P15364,&P16110,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; U; en) Presto/2.7.62 Version/11.00*/
const Device UA2466147 = {S62020, 135, { &P20255,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R811 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467950 = {S62021, 140, { &P20255,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-Th; OPPOR8111 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2653082 = {S62022, 126, { &P20255,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R811 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2748222 = {S62023, 136, { &P20255,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; R8111 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778502 = {S62024, 57, { &P20255,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; R811 Build/IMM76D)*/
const Device UA2844244 = {S62025, 58, { &P20255,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; R8111 Build/IMM76D)*/
const Device UA3100626 = {S62026, 136, { &P20255,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R811W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3068052 = {S62027, 145, { &P12972,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-T989 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3082985 = {S62028, 136, { &P12972,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T989 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA631781 = {S62029, 21, { &P17968,&P17979,&P18062,&P18092 }};	/*Useragent - Android/4.0.4-EAS-1.3*/
const Device UA766844 = {S62030, 21, { &P17968,&P17979,&P18062,&P18092 }};	/*Useragent - Android/4.0.3-EAS-1.3*/
const Device UA2385547 = {S62031, 39, { &P17968,&P17979,&P18062,&P18092 }};	/*Useragent - Windowshop/1.50.03 (Android/4.0.3/KFTT)*/
const Device UA2430912 = {S62032, 39, { &P17968,&P17979,&P18062,&P18092 }};	/*Useragent - Windowshop/1.51.09 (Android/4.0.3/KFTT)*/
const Device UA2459481 = {S62033, 39, { &P17968,&P17979,&P18062,&P18092 }};	/*Useragent - Windowshop/1.51.09 (Android/4.0.3/KFOT)*/
const Device UA2528290 = {S62034, 40, { &P17968,&P17979,&P18062,&P18092 }};	/*Useragent - Windowshop/1.51.06 (Android/4.0.4/KFJWI)*/
const Device UA2671728 = {S62035, 39, { &P17968,&P17979,&P18062,&P18092 }};	/*Useragent - Windowshop/1.51.42 (Android/4.0.3/KFTT)*/
const Device UA2712482 = {S62036, 39, { &P17968,&P17979,&P18062,&P18092 }};	/*Useragent - Windowshop/1.51.43 (Android/4.0.3/KFTT)*/
const Device UA2737243 = {S62037, 39, { &P17968,&P17979,&P18062,&P18092 }};	/*Useragent - Windowshop/1.51.43 (Android/4.0.3/KFOT)*/
const Device UA2759764 = {S62038, 66, { &P17968,&P17979,&P18062,&P18092 }};	/*Useragent - Xtreme Labs Android HTTP Framework on device with Android OS 4.0.4*/
const Device UA2799832 = {S62039, 39, { &P17968,&P17979,&P18062,&P18092 }};	/*Useragent - Windowshop/1.51.46 (Android/4.0.3/KFTT)*/
const Device UA2937656 = {S62040, 39, { &P17968,&P17979,&P18062,&P18092 }};	/*Useragent - Windowshop/1.51.46 (Android/4.0.3/KFOT)*/
const Device UA3015543 = {S62041, 39, { &P17968,&P17979,&P18062,&P18092 }};	/*Useragent - Windowshop/1.51.61 (Android/4.0.3/KFTT)*/
const Device UA3018526 = {S62042, 39, { &P17968,&P17979,&P18062,&P18092 }};	/*Useragent - Windowshop/1.51.61 (Android/4.0.3/KFOT)*/
const Device UA3081572 = {S62043, 39, { &P17968,&P17979,&P18062,&P18092 }};	/*Useragent - Windowshop/1.51.62 (Android/4.0.3/KFTT)*/
const Device UA609005 = {S62044, 157, { &P14574,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA673560 = {S62045, 153, { &P14574,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2198884 = {S62046, 157, { &P14574,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2386760 = {S62047, 80, { &P15364,&P16110,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:16.0; Avant TriCore) Gecko/X Firefox/16.0*/
const Device UA2426749 = {S62048, 80, { &P15364,&P16110,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:17.0; Avant TriCore) Gecko/X Firefox/17.0*/
const Device UA2562198 = {S62049, 80, { &P15364,&P16110,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:18.0; Avant TriCore) Gecko/X Firefox/18.0*/
const Device UA2603522 = {S62050, 80, { &P15364,&P16110,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:19.0; Avant TriCore) Gecko/X Firefox/19.0*/
const Device UA2738252 = {S62051, 80, { &P15364,&P16110,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:20.0; Avant TriCore) Gecko/X Firefox/20.0*/
const Device UA2886357 = {S62052, 80, { &P15364,&P16110,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:21.0; Avant TriCore) Gecko/X Firefox/21.0*/
const Device UA2970834 = {S62053, 80, { &P15364,&P16110,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:22.0; Avant TriCore) Gecko/X Firefox/22.0*/
const Device UA3062273 = {S62054, 80, { &P15364,&P16110,&P22734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:23.0; Avant TriCore) Gecko/X Firefox/23.0*/
const Device UA421611 = {S62055, 134, { &P21324,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; i940 Build/ECLAIR) AppleWebKit/530.17+ (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA844783 = {S62056, 140, { &P21324,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; i940 Build/ESE81) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1966382 = {S62057, 62, { &P21324,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.1-update1; i940 Build/ESE81)*/
const Device UA2567129 = {S62058, 169, { &P15794,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567378 = {S62059, 169, { &P15794,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315b Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2974921 = {S62060, 145, { &P13064,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I727 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022795 = {S62061, 136, { &P13064,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I727 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA647462 = {S62062, 73, { &P15364,&P17188,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; WinNT4.0; en-US; rv:1.7.9) Gecko/X Firefox/1.0.5*/
const Device UA2354278 = {S62063, 146, { &P21649,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; POV_TAB-PROTAB30-IPS10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3043446 = {S62064, 75, { &P21649,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; POV_TAB-PROTAB30-IPS10 Build/JRO03H)*/
const Device UA2560583 = {S62065, 143, { &P23499,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BLU DASH 4.0 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879879 = {S62066, 139, { &P23499,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Dash 4.0 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2224148 = {S62067, 149, { &P13123,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-S5360 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2423068 = {S62068, 112, { &P19973,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia114/03.33; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.5.16.9*/
const Device UA148342 = {S62069, 130, { &P15364,&P5690,&P22445,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Iron/7.0.520.1 Chrome/7.0.520.1 Safari/534.7*/
const Device UA495816 = {S62070, 130, { &P15364,&P5690,&P22445,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Iron/7.0.520.1 Chrome/7.0.520.1 Safari/534.7*/
const Device UA2412202 = {S62071, 113, { &P15364,&P5690,&P22445,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Iron/7.0.520.1 Safari/534.7*/
const Device UA2261778 = {S62072, 142, { &P14669,&P5843,&P24670,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiax2-00) UCBrowser8.4.0.159/70/444/UCWEB Mobile*/
const Device UA2529307 = {S62073, 135, { &P21104,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; X507 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2669763 = {S62074, 136, { &P21104,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; X507S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA13898 = {S62075, 104, { &P19971,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonT700/R3EA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.3*/
const Device UA573929 = {S62076, 90, { &P17968,&P5629,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.7; Linux; Opera Mobi/ADR-X; U; en-GB) Presto/2.8.149 Version/11.10*/
const Device UA716712 = {S62077, 87, { &P17968,&P5629,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.5; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.8.149 Version/11.10*/
const Device UA880313 = {S62078, 87, { &P17968,&P5629,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.3; Linux; Opera Mobi/ADR-X; U; it) Presto/2.8.149 Version/11.10*/
const Device UA1891280 = {S62079, 90, { &P17968,&P5629,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.2; Linux; Opera Mobi/ADR-X; U; en-GB) Presto/2.8.149 Version/11.10*/
const Device UA1906278 = {S62080, 92, { &P17968,&P5629,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.4; Linux; Opera Mobi/build-X; U; en-GB) Presto/2.8.149 Version/11.10*/
const Device UA2688473 = {S62081, 87, { &P17968,&P5629,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.6; Linux; Opera Mobi/ADY-X; U; en) Presto/2.8.149 Version/11.10*/
const Device UA603319 = {S62082, 133, { &P12280,&P18117,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 TeamStream (iPhone 199 1.6)*/
const Device UA603374 = {S62083, 133, { &P12280,&P18117,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 TeamStream (iPhone 199 1.6)*/
const Device UA654129 = {S62084, 155, { &P12280,&P18117,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3 TeamStream (iPhone4,1 215 1.6.1)*/
const Device UA2154853 = {S62085, 155, { &P12280,&P18117,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3 TeamStream (iPhone3,1 215 1.6.1)*/
const Device UA2197917 = {S62086, 155, { &P12280,&P18117,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 TeamStream (iPhone4,1 267 2.1)*/
const Device UA2441614 = {S62087, 135, { &P12280,&P18117,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 TeamStream (iPhone 199 1.6)*/
const Device UA2844137 = {S62088, 157, { &P12280,&P18117,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3 TeamStream (iPhone3,1 293 2.2.2)*/
const Device UA604397 = {S62089, 117, { &P15364,&P17285,&P22457,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19*/
const Device UA607692 = {S62090, 124, { &P15364,&P17285,&P22457,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19*/
const Device UA2512002 = {S62091, 166, { &P18673,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3021775 = {S62092, 170, { &P18673,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S7562 Build/IMM76I) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2798864 = {S62093, 144, { &P21020,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-in; Micromax A116 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3056867 = {S62094, 66, { &P21020,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; Micromax A116 Build/JOP40D)*/
const Device UA3057722 = {S62095, 144, { &P21020,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Micromax A116 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1959317 = {S62096, 43, { &P11684,&P5129,&P24671,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UCWEB8.5.0.163/50/999*/
const Device UA1995581 = {S62097, 92, { &P11684,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5800 XpressMusic)/UC Browser8.5.0.183/50/355/UCWEB Mobile*/
const Device UA2788568 = {S62098, 131, { &P11684,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokia5800d-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile  3gpp-gba*/
const Device UA2752783 = {S62099, 97, { &P15364,&P18110,&P21781,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.4.3 Safari/534.34*/
const Device UA2754059 = {S62100, 99, { &P15364,&P18110,&P21781,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.21 (KHTML, like Gecko) QupZilla/1.4.3 Safari/537.21*/
const Device UA3131443 = {S62101, 138, { &P15575,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132580 = {S62102, 137, { &P15575,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132938 = {S62103, 138, { &P15575,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3067183 = {S62104, 129, { &P24365,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SM-T211 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2351618 = {S62105, 130, { &P20014,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SC-01E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3070703 = {S62106, 59, { &P20014,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SC-01E Build/IMM76D)*/
const Device UA2514463 = {S62107, 172, { &P21489,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Infectious Rezound Build/IML74K; CyanogenMod-INFECTION) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2569597 = {S62108, 201, { &P21489,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Rezound on TRON-2.0 Build/IML74K; CyanogenMod-A product of the Umbrella Corporation) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2716801 = {S62109, 178, { &P21489,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Infected HTC Rezound Build/IML74K; CyanogenMod-INFECTION 2.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2737919 = {S62110, 187, { &P21489,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Rezound on TRON 2.1 Build/IML74K; CyanogenMod-By IAmTheOneTheyCallNeo) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2938736 = {S62111, 191, { &P21489,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2 Jelly Bean; en-us; Rezound on TRON Build/IML74K; CyanogenMod-IAmTheOneTheyCallNeo) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3021718 = {S62112, 181, { &P21489,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Infected Rezound Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/INFECTION 2.9.1/vigor*/
const Device UA2974923 = {S62113, 137, { &P13991,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6400L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022984 = {S62114, 140, { &P13991,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Thunderbolt Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974998 = {S62115, 138, { &P22851,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT919 Build/2_290_2012) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA470735 = {S62116, 84, { &P17595,&P18120,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; fr; rv:1.8.0.12) Gecko/X Firefox/1.5.0.12*/
const Device UA915770 = {S62117, 90, { &P17595,&P18120,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; es-ES; rv:1.8.0.3) Gecko/X Firefox/1.5.0.3*/
const Device UA2420557 = {S62118, 138, { &P20284,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xenta TAB9-200 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2460967 = {S62119, 131, { &P20284,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAB9-200 Build/Xenta) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422017 = {S62120, 137, { &P20179,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GOCLEVER TAB T76 Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2730327 = {S62121, 137, { &P20179,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; GOCLEVER TAB T76 Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2738720 = {S62122, 152, { &P20129,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2739993 = {S62123, 153, { &P20129,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2S Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2755003 = {S62124, 154, { &P20129,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2SC Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2782013 = {S62125, 153, { &P20129,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2C Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2937407 = {S62126, 153, { &P20129,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2A Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3096035 = {S62127, 112, { &P19747,&P5043,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia206/03.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA3111173 = {S62128, 114, { &P19747,&P5043,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia206.1/03.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA25391 = {S62129, 105, { &P11944,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonT715a/R1GA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.4*/
const Device UA62589 = {S62130, 104, { &P11944,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonT715/R1HA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.5*/
const Device UA2466 = {S62131, 54, { &P11978,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; es-ES)*/
const Device UA2469 = {S62132, 51, { &P11978,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; pt)*/
const Device UA2467 = {S62133, 51, { &P11978,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; el)*/
const Device UA21177 = {S62134, 51, { &P11978,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA21178 = {S62135, 54, { &P11978,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; zh-tw)*/
const Device UA21180 = {S62136, 51, { &P11978,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; fr)*/
const Device UA21181 = {S62137, 51, { &P11978,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; nl)*/
const Device UA21182 = {S62138, 51, { &P11978,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; ru)*/
const Device UA21184 = {S62139, 51, { &P11978,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; de)*/
const Device UA21185 = {S62140, 51, { &P11978,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; it)*/
const Device UA21895 = {S62141, 51, { &P11978,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; nb)*/
const Device UA22172 = {S62142, 51, { &P11978,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; sv)*/
const Device UA35930 = {S62143, 51, { &P11978,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch2_T3333 Opera/9.50 (Windows NT 5.1; U; da)*/
const Device UA54119 = {S62144, 52, { &P11978,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch3G_T3333 Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA89029 = {S62145, 80, { &P11978,&P5448,&P3430,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_Touch2/1.29.176.2 (67024) Opera/9.50 (Windows NT 5.1; U; es-ES)*/
const Device UA143981 = {S62146, 50, { &P11978,&P5448,&P3430,&P18092 }};	/*Useragent - Opera/9.50 (Windows NT 5.1; HTC_Touch2/1.0; U; en)*/
const Device UA90890 = {S62147, 124, { &P17346,&P17146,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Twitter for iPhone*/
const Device UA90893 = {S62148, 122, { &P17346,&P17146,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Twitter for iPhone*/
const Device UA2233620 = {S62149, 190, { &P18645,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-L710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, us*/
const Device UA2233833 = {S62150, 198, { &P18645,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, us*/
const Device UA2234690 = {S62151, 191, { &P18645,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I747M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, ca*/
const Device UA2462436 = {S62152, 60, { &P19748,&P5043,&P18517,&P18092 }};	/*Useragent - Nokia205/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2706157 = {S62153, 62, { &P19748,&P5043,&P18517,&P18092 }};	/*Useragent - Nokia205.1/2.0 (03.19) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2955659 = {S62154, 61, { &P19748,&P5043,&P18517,&P18092 }};	/*Useragent - Nokia2055/2.0 (03.20) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA3103686 = {S62155, 62, { &P19748,&P5043,&P18517,&P18092 }};	/*Useragent - Nokia205.3/2.0 (03.19) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA26340 = {S62156, 85, { &P13179,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW350iv/R10BA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64009 = {S62157, 84, { &P13179,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW350i/R11CA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20320 = {S62158, 147, { &P16967,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update-1; en-us; Dell_Aero Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari.530.17*/
const Device UA42860 = {S62159, 146, { &P16967,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; Dell Aero Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2020262 = {S62160, 68, { &P16967,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Dell Aero Build/ECLAIR)*/
const Device UA33016 = {S62161, 135, { &P12830,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; ZTE T60 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA639287 = {S62162, 135, { &P12830,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; ZTE T60 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3117105 = {S62163, 59, { &P12830,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; ZTE T60 Build/GRJ22)*/
const Device UA2232158 = {S62164, 133, { &P21030,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAC-97032 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2966033 = {S62165, 89, { &P16519,&P5567,&P18525,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/6.0.3/27.2338; U; en) Presto/2.8.119 320X240 Pantech TXT8045*/
const Device UA2658231 = {S62166, 146, { &P20139,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-A7100 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2738011 = {S62167, 212, { &P20139,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; sprd-GT-A7100/1.0 Android/4.0.4 Release/04.07.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2778691 = {S62168, 70, { &P20139,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 4.0.4; GT-A7100 Build/MocorDroid2.3.5)*/
const Device UA2782532 = {S62169, 212, { &P20139,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; sprd-GT-A7100/1.0 Android/4.0.4 Release/04.25.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3012895 = {S62170, 212, { &P20139,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; sprd-GT-A7100/1.0 Android/4.0.4 Release/06.15.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3022318 = {S62171, 212, { &P20139,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; sprd-GT-A7100/1.0 Android/4.0.4 Release/06.01.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3088709 = {S62172, 212, { &P20139,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; sprd-GT-A7100/1.0 Android/4.0.4 Release/`d.be.b`ac Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3125201 = {S62173, 212, { &P20139,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; sprd-GT-A7100/1.0 Android/4.0.4 Release/06.28.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3125212 = {S62174, 212, { &P20139,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; sprd-GT-A7100/1.0 Android/4.0.4 Release/05.31.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3125394 = {S62175, 212, { &P20139,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; sprd-GT-A7100/1.0 Android/4.0.4 Release/07.24.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA796278 = {S62176, 117, { &P19850,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; X325a; en-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2167693 = {S62177, 117, { &P19850,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; X325a; fr-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2748920 = {S62178, 117, { &P19850,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; X325a; en-us) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2736264 = {S62179, 139, { &P22848,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT920 Build/2_310_2009) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2672534 = {S62180, 144, { &P17595,&P17295,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17; Maxthon (4.0.4.1000);*/
const Device UA2674730 = {S62181, 147, { &P17595,&P17295,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.18 (KHTML,like Gecko) Version/3.1.2 Safari/525.20.1; Maxthon (4.0.4.1000);*/
const Device UA2676351 = {S62182, 137, { &P18559,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LG730 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2736197 = {S62183, 130, { &P18559,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LG730 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2403942 = {S62184, 142, { &P19871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3099379 = {S62185, 66, { &P19871,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-S6800 Build/GINGERBREAD)*/
const Device UA2663176 = {S62186, 134, { &P23217,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; PMP7170B3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2941023 = {S62187, 136, { &P23217,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMP7170B3GCE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2784824 = {S62188, 161, { &P15575,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2795191 = {S62189, 161, { &P15575,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9108 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2797284 = {S62190, 161, { &P15575,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2799511 = {S62191, 162, { &P15575,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9100G Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2800671 = {S62192, 161, { &P15575,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2994499 = {S62193, 161, { &P15575,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3048181 = {S62194, 162, { &P15575,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-I9100G Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA40114 = {S62195, 134, { &P15404,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; MB861 Build/4.5.91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA74188 = {S62196, 140, { &P15404,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB861 Build/4.5.2A-KT-75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA629622 = {S62197, 141, { &P15404,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; MB861 Build/4.5.2A-KT-167) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020042 = {S62198, 64, { &P15404,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; MB861 Build/4.5.2A-KT-75)*/
const Device UA2023245 = {S62199, 65, { &P15404,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; MB861 Build/4.5.2A-KT-167)*/
const Device UA570319 = {S62200, 73, { &P15364,&P18145,&P17828,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/4.5; FreeBSD) KHTML/4.5.4 (like Gecko)*/
const Device UA2567740 = {S62201, 140, { &P17968,&P5629,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile*/
const Device UA2667546 = {S62202, 140, { &P17968,&P5629,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile*/
const Device UA2793078 = {S62203, 140, { &P17968,&P5629,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile*/
const Device UA2793266 = {S62204, 140, { &P17968,&P5629,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile*/
const Device UA2965031 = {S62205, 140, { &P17968,&P5629,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fa-fa) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.8181 Mobile*/
const Device UA2560408 = {S62206, 127, { &P21216,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; A78 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2935134 = {S62207, 56, { &P21216,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; A78 Build/JRO03H)*/
const Device UA2938259 = {S62208, 127, { &P21216,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; A78 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA569013 = {S62209, 118, { &P15364,&P18110,&P24304,&P18092 }};	/*Useragent - Opera/10.60 (Linux i686 ; U; HbbTV/1.1.1 (;;;;;); CE-HTML; TechniSat MultyVision ISIO; de) Presto/2.6.33 Version/10.60*/
const Device UA716356 = {S62210, 111, { &P15364,&P18110,&P24304,&P18092 }};	/*Useragent - Opera/9.80 (Linux sh4; U; HbbTV/1.1.1(;;;;;); CE-HTML; TechniSat DIGIT ISIO S; en) Presto/2.8.115 Version/11.10*/
const Device UA718160 = {S62211, 113, { &P15364,&P18110,&P24304,&P18092 }};	/*Useragent - Opera/10.60 (Linux sh4 ; U; HbbTV/1.1.1 (;;;;;); CE-HTML; TechniSat DIGIT ISIO S; de) Presto/2.6.33 Version/10.60*/
const Device UA2168602 = {S62212, 117, { &P15364,&P18110,&P24304,&P18092 }};	/*Useragent - Opera/9.80 (Linux i686; U; HbbTV/1.1.1 (;;;;;); CE-HTML; TechniSat MultyVision ISIO; de) Presto/2.9.167 Version/11.50*/
const Device UA2262250 = {S62213, 138, { &P15364,&P18110,&P24304,&P18092 }};	/*Useragent - Opera/10.60 (Linux sh4 ; U; HbbTV/1.1.1 (+PVR; Loewe; SL150; LOH/3.10;;) CE-HTML/1.0 Config(L:deu,CC:DEU); en) Presto/2.6.33 Version/10.60*/
const Device UA2364595 = {S62214, 138, { &P15364,&P18110,&P24304,&P18092 }};	/*Useragent - Opera/9.80 (Linux sh4; U; HbbTV/1.1.1 (+PVR; Loewe; SL150; LOH/3.10;;) CE-HTML/1.0 Config(L:deu,CC:DEU); en) Presto/2.10.250 Version/11.60*/
const Device UA2366549 = {S62215, 112, { &P15364,&P18110,&P24304,&P18092 }};	/*Useragent - Opera/9.80 (Linux sh4; U; HbbTV/1.1.1 (;;;;;); CE-HTML; TechniSat Digit ISIO S; de) Presto/2.9.167 Version/11.50*/
const Device UA2473028 = {S62216, 102, { &P15364,&P18110,&P24304,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; U; HbbTV/1.1.1 (;tv2n;videoweb;1.0.0;1.0;); en) Presto/2.8.115 Version/11.10*/
const Device UA2720496 = {S62217, 126, { &P15364,&P18110,&P24304,&P18092 }};	/*Useragent - Opera/10.60 (Linux sh4 ; U; HbbTV/1.1.1 (; LOH/2.01; -----;;;) CE-HTML/1.0 Config(L:de,CC:DE); en) Presto/2.6.33 Version/10.60*/
const Device UA2743136 = {S62218, 138, { &P15364,&P18110,&P24304,&P18092 }};	/*Useragent - Opera/9.80 (Linux sh4; U; HbbTV/1.1.1 (+PVR; Loewe; SL150; LOH/3.10;;) CE-HTML/1.0 Config(L:nld,CC:NLD); en) Presto/2.10.250 Version/11.60*/
const Device UA2810878 = {S62219, 98, { &P15364,&P18110,&P24304,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv6l; U; NETRANGEMMH;HbbTV/1.1.1;CE-HTML/1.0; en) Presto/2.8.115 Version/11.10*/
const Device UA2992399 = {S62220, 115, { &P15364,&P18110,&P24304,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv6l; U; NETRANGEMMH;HbbTV/1.1.1;CE-HTML/1.0;THOMSON LF1V394; en) Presto/2.10.250 Version/11.60*/
const Device UA30031 = {S62221, 135, { &P12280,&P17144,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A306 Twitter for iPhone*/
const Device UA30057 = {S62222, 133, { &P12280,&P17144,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A293 Twitter for iPhone*/
const Device UA30058 = {S62223, 135, { &P12280,&P17144,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A400 Twitter for iPhone*/
const Device UA68826 = {S62224, 42, { &P11684,&P5129,&P24665,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UCWEB8.0.0.89/50/351*/
const Device UA437272 = {S62225, 54, { &P11684,&P5129,&P24665,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UC Browser8.0.4.121/50/444/UCWEB*/
const Device UA1901009 = {S62226, 43, { &P11684,&P5129,&P24665,&P18092 }};	/*Useragent - Nokia5800w XpressMusic/UCWEB8.0.3.99/50/800*/
const Device UA2567348 = {S62227, 117, { &P15364,&P17285,&P22465,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.22 (KHTML, like Gecko) Iron/25.0.1400.0 Chrome/25.0.1400.0 Safari/537.22*/
const Device UA2582988 = {S62228, 124, { &P15364,&P17285,&P22465,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Iron/25.0.1400.0 Chrome/25.0.1400.0 Safari/537.22*/
const Device UA2736362 = {S62229, 59, { &P15364,&P18346,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10; rv:21.0) Gecko/X Firefox/21.0*/
const Device UA2748580 = {S62230, 139, { &P22941,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S6812 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2942035 = {S62231, 140, { &P22941,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; GT-S6812B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049227 = {S62232, 61, { &P22941,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S6812 Build/JZO54K)*/
const Device UA3054944 = {S62233, 140, { &P22941,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-S6812i Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2886498 = {S62234, 112, { &P12989,&P5843,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia303/14.38; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA67276 = {S62235, 75, { &P14916,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-C3303K/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA67441 = {S62236, 81, { &P14916,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-C3303/C3303DDJG4 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA68297 = {S62237, 75, { &P14916,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-C3303i/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA16316 = {S62238, 93, { &P16225,&P5448,&P3430,&P18092 }};	/*Useragent - Opera/9.5 (Microsoft Windows; Windows CE; Opera Mobi/9.5; U; en) 480x800 SAMSUNG SCH-i920 PPC*/
const Device UA71975 = {S62239, 92, { &P16225,&P5448,&P3430,&P18092 }};	/*Useragent - Opera/9.5 (Microsoft Windows; Windows CE; OperaMobi/9.5; U; en) 480x800 SAMSUNG SCH-i920 PPC*/
const Device UA199997 = {S62240, 134, { &P18078,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11SH Build/S6160) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA220457 = {S62241, 134, { &P18078,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11SH Build/S7150) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA248993 = {S62242, 134, { &P18078,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11SH Build/S9081) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA628574 = {S62243, 134, { &P18078,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11SH Build/SB280) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA631722 = {S62244, 134, { &P18078,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11SH Build/S3085) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1900202 = {S62245, 134, { &P18078,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11SH Build/S8270) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031041 = {S62246, 134, { &P18078,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; IS11SH Build/S8030) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2204383 = {S62247, 116, { &P14482,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-01.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2229238 = {S62248, 114, { &P14482,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-01/07.15; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2361275 = {S62249, 101, { &P18444,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 820)*/
const Device UA2527743 = {S62250, 111, { &P18444,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 820; Vodafone)*/
const Device UA2528549 = {S62251, 109, { &P18444,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 820; ANZ825)*/
const Device UA3108795 = {S62252, 109, { &P18444,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 820; 1328-3)*/
const Device UA3119498 = {S62253, 101, { &P18444,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Nokia 820)*/
const Device UA2973675 = {S62254, 133, { &P23187,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; V370 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023728 = {S62255, 132, { &P23187,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; V370 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3065613 = {S62256, 137, { &P18594,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LS860 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110803 = {S62257, 133, { &P18594,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Mach Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2754823 = {S62258, 134, { &P20213,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N9500 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA9922 = {S62259, 87, { &P16627,&P17483,&P3499,&P18092 }};	/*Useragent - LG-GS170 Browser/Obigo-Q7.0 MMS/LG-MMS-V1.0/1.2 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA601381 = {S62260, 73, { &P15364,&P18110,&P22314,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:11.0) Gecko/X Firefox/11.0 SeaMonkey/2.8*/
const Device UA2361879 = {S62261, 143, { &P21269,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Intex Aqua 3.2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2716826 = {S62262, 162, { &P20134,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5830i Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3131340 = {S62263, 136, { &P22594,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA94678 = {S62264, 92, { &P18254,&P7480,&P17387,&P18091 }};	/*Useragent - YahooFeedSeeker/2.0 (compatible; Mozilla 4.0; MSIE 5.5; http://publisher.yahoo.com/rssguide)*/
const Device UA165924 = {S62265, 145, { &P18254,&P7480,&P17387,&P18091 }};	/*Useragent - Y!J-BRO/YFSJ crawler (compatible; Mozilla 4.0; MSIE 5.5; http://help.yahoo.co.jp/help/jp/search/indexing/indexing-15.html; YahooFeedSeekerJp/2.0)*/
const Device UA531005 = {S62266, 164, { &P18254,&P7480,&P17387,&P18091 }};	/*Useragent - Y!J-BRO/YFSJ crawler (compatible; Mozilla 4.0; MSIE 5.5; http://help.yahoo.co.jp/help/jp/search/indexing/indexing-15.html; YahooFeedSeekerJp/2.0; users 0; views 28)*/
const Device UA697454 = {S62267, 96, { &P15364,&P17350,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; ; en-NZ) AppleWebKit/527+ (KHTML, like Gecko, Safari/419.3) Arora/0.8.0*/
const Device UA2411087 = {S62268, 102, { &P15364,&P17350,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; ; cs-CZ) AppleWebKit/528+ (KHTML, like Gecko, Safari/528.0) Lunascape/5.1.2.0*/
const Device UA2495914 = {S62269, 102, { &P15364,&P17350,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; ; cs-CZ) AppleWebKit/532+ (KHTML, like Gecko, Safari/532.0) Lunascape/5.1.2.3*/
const Device UA2815754 = {S62270, 98, { &P15364,&P17350,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; ; en-US) AppleWebKit/533.3 (KHTML, like Gecko) VIDEODOWNLOAD Safari/533.3*/
const Device UA2567336 = {S62271, 146, { &P13064,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602707 = {S62272, 139, { &P13064,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA46888 = {S62273, 113, { &P14028,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-A667TL-parrot/A667TLKA2 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA53789 = {S62274, 104, { &P14028,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-A667/A667UCJI2 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA57921 = {S62275, 106, { &P14028,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-A667TL/A667TLKA2 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA55982 = {S62276, 182, { &P13535,&P5843,&P4732,&P18092 }};	/*Useragent - Nokia6350/2.0(ATT.12.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 Nokia6350/12.40; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.16803; U; en-US) Opera 9.50*/
const Device UA361824 = {S62277, 134, { &P18383,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS13SH Build/SC100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA426955 = {S62278, 134, { &P18383,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS13SH Build/S2150) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA613283 = {S62279, 134, { &P18383,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS13SH Build/S3300) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623054 = {S62280, 134, { &P18383,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS13SH Build/SA310) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA924909 = {S62281, 134, { &P18383,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS13SH Build/S6291) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2206534 = {S62282, 134, { &P18383,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS13SH Build/S9100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2607058 = {S62283, 134, { &P18383,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS13SH Build/S3060) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2969147 = {S62284, 162, { &P18113,&P22211,&P19802,&P18091 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT912 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/spyder*/
const Device UA728082 = {S62285, 141, { &P15767,&P18117,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA1975771 = {S62286, 139, { &P15767,&P18117,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2367208 = {S62287, 140, { &P15767,&P18117,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2429934 = {S62288, 151, { &P15803,&P5843,&P24675,&P18092 }};	/*Useragent - Nokia200/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia200) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2469369 = {S62289, 171, { &P18283,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SCH-I535, 39, 1.8.2)*/
const Device UA2486555 = {S62290, 171, { &P18283,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SGH-T999, 39, 1.8.2)*/
const Device UA2670288 = {S62291, 171, { &P18283,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I535 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SCH-I535, 42, 1.9.1)*/
const Device UA2734254 = {S62292, 171, { &P18283,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (GT-I9300, 42, 1.9.1)*/
const Device UA2877937 = {S62293, 171, { &P18283,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T999 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SGH-T999, 47, 1.9.7)*/
const Device UA2670849 = {S62294, 137, { &P22760,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ThL W8 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2855879 = {S62295, 59, { &P22760,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; ThL W8 Build/JZO54K)*/
const Device UA3021806 = {S62296, 152, { &P18643,&P5043,&P24679,&P18092 }};	/*Useragent - Nokia311/5.0 (07.36) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia311) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3080409 = {S62297, 137, { &P22916,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E450g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081334 = {S62298, 137, { &P22916,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E450B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083150 = {S62299, 137, { &P22916,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E450f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083452 = {S62300, 136, { &P22916,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E450 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA54037 = {S62301, 73, { &P14100,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S3353/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2468433 = {S62302, 143, { &P20471,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V1 Build/HCL ME Tablet V1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2671907 = {S62303, 159, { &P20471,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CONNECT 2G Build/HCL ME Tablet CONNECT 2G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA27535 = {S62304, 74, { &P14643,&P17484,&P18336,&P18092 }};	/*Useragent - Opera/9.62 (X11;Linux i686; U; en) Presto/2.2.0/SAMSUNG-GT-I8320-Vodafone/*/
const Device UA61978 = {S62305, 112, { &P14643,&P17484,&P18336,&P18092 }};	/*Useragent - SAMSUNG-GT-I8320-Vodafone/I8320BUJC1 Linux/X2/R1 Opera/9.6 SMS-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA2432152 = {S62306, 226, { &P15575,&P18066,&P17373,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Opera/9.80 (Android; Opera Mini/6.5.28192/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2772908 = {S62307, 158, { &P12968,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26i Build/6.1.A.2.55) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA568057 = {S62308, 136, { &P14856,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-SU660 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA572189 = {S62309, 60, { &P14856,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-SU660 Build/GRJ22)*/
const Device UA2043655 = {S62310, 69, { &P15364,&P22740,&P20435,&P18092 }};	/*Useragent - Mozilla/5.0 (Maemo; Mobile; rv:15.0) Gecko/X Firefox/15.0 Fennec/15.0*/
const Device UA2367267 = {S62311, 132, { &P20035,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2596556 = {S62312, 133, { &P20035,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925U Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2842549 = {S62313, 132, { &P20035,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-I925 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2566915 = {S62314, 152, { &P18125,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID4 Build/6.7.2-180_DR4-16_M2-37) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2754186 = {S62315, 137, { &P18300,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T769 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA50425 = {S62316, 115, { &P13255,&P17483,&P4732,&P18092 }};	/*Useragent - LG-GR700/V10g; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.15814; U; en-US) Opera 9.50*/
const Device UA2026713 = {S62317, 136, { &P19971,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SimTab T700A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2353421 = {S62318, 135, { &P19971,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ET7006C-P12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2411225 = {S62319, 141, { &P19971,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; WEXLER.BOOK_T7008 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2554513 = {S62320, 137, { &P19971,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; MT7003 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2655643 = {S62321, 129, { &P19971,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-be; CT700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2659242 = {S62322, 131, { &P19971,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT7001E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2659261 = {S62323, 136, { &P19971,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; T7004 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2726872 = {S62324, 133, { &P19971,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; EvoPAD T7000 Build/MID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2791244 = {S62325, 137, { &P19971,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; MT7005 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950726 = {S62326, 129, { &P19971,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; T7003 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2986704 = {S62327, 135, { &P19971,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ET7008C-H12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2230950 = {S62328, 133, { &P18466,&P18110,&P19814,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.2 Safari/535.19 Silk-Accelerated=false*/
const Device UA2674149 = {S62329, 117, { &P15364,&P17285,&P22466,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36*/
const Device UA2719144 = {S62330, 124, { &P15364,&P17285,&P22466,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36*/
const Device UA2886782 = {S62331, 112, { &P18583,&P5043,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia110/03.04; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA2407103 = {S62332, 157, { &P15364,&P7133,&P19965,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.3.1.31155 (Firestorm-Release; firestorm skin) Safari/533.3*/
const Device UA2432007 = {S62333, 229, { &P15649,&P5629,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2432667 = {S62334, 230, { &P15649,&P5629,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2483285 = {S62335, 31, { &P15373,&P5129,&P24675,&P18092 }};	/*Useragent - Nokia5530/UCWEB8.8.0.212/50/999*/
const Device UA2513024 = {S62336, 92, { &P15373,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5530) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2508560 = {S62337, 172, { &P20134,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2792179 = {S62338, 172, { &P20134,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5839i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3020789 = {S62339, 176, { &P20134,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5830i Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2753858 = {S62340, 130, { &P20160,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3013092 = {S62341, 142, { &P12280,&P19826,&P24341,&P18092 }};	/*Useragent - MQQBrowser/44 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3*/
const Device UA3032773 = {S62342, 142, { &P12280,&P19826,&P24341,&P18092 }};	/*Useragent - MQQBrowser/44 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3*/
const Device UA3032786 = {S62343, 141, { &P12280,&P19826,&P24341,&P18092 }};	/*Useragent - MQQBrowser/44 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3*/
const Device UA3056932 = {S62344, 140, { &P12280,&P19826,&P24341,&P18092 }};	/*Useragent - MQQBrowser/44 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 Safari/7534.48.3*/
const Device UA3057255 = {S62345, 139, { &P12280,&P19826,&P24341,&P18092 }};	/*Useragent - MQQBrowser/44 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Safari/7534.48.3*/
const Device UA3078176 = {S62346, 139, { &P12280,&P19826,&P24341,&P18092 }};	/*Useragent - MQQBrowser/44 Mozilla/5.0 (iPhone 4; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Safari/7534.48.3*/
const Device UA161793 = {S62347, 71, { &P15364,&P18110,&P17826,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/4.4; Linux) KHTML/4.4.5 (like Gecko)*/
const Device UA2383421 = {S62348, 138, { &P21440,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOMO9 plus III Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2423836 = {S62349, 131, { &P21440,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOMO9-3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2598714 = {S62350, 132, { &P21440,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MOMO9-3S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2816121 = {S62351, 131, { &P21440,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MOMO9-3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2429443 = {S62352, 135, { &P20097,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CT1010 Build/IMM76I.v007) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2461173 = {S62353, 136, { &P19864,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; ARCHOS 80XSK Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2765379 = {S62354, 154, { &P22962,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; ALCATEL ONE TOUCH 6030A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2882676 = {S62355, 112, { &P18648,&P5043,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia306/05.93; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA2509854 = {S62356, 57, { &P15364,&P17017,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (masking-agent; rv:19.0) Gecko/X Firefox/19.0*/
const Device UA3096915 = {S62357, 139, { &P15767,&P22882,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/11A465 Safari/8536.25*/
const Device UA3129484 = {S62358, 141, { &P15767,&P22882,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/11A501 Safari/8536.25*/
const Device UA33365 = {S62359, 152, { &P16602,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; E210 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 ACER_E210/1.006.00*/
const Device UA2018414 = {S62360, 143, { &P16602,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; E210 Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1ACER_E210/1.0*/
const Device UA30653 = {S62361, 137, { &P14222,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) 320x240; VZW; UTStar-XV6175.1; Windows Mobile 6.5 Standard;*/
const Device UA54469 = {S62362, 134, { &P14222,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) 320x240; VZW; UTStar-XV6175; Window Mobile 6.1 Standard;*/
const Device UA609631 = {S62363, 143, { &P18084,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  Transformer TF101G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA901585 = {S62364, 142, { &P18084,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2235739 = {S62365, 41, { &P15364,&P18580,&P22898,&P18092 }};	/*Useragent - Mercury/7.0.1 CFNetwork/609 Darwin/13.0.0*/
const Device UA3061060 = {S62366, 138, { &P12968,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.1.96) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3093719 = {S62367, 131, { &P12968,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.1.96) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA165280 = {S62368, 81, { &P17595,&P17294,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.4.11; U; en) Presto/2.7.62 Version/11.00*/
const Device UA303947 = {S62369, 81, { &P17595,&P17294,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.4.11; U; en) Presto/2.7.62 Version/11.01*/
const Device UA454197 = {S62370, 90, { &P17976,&P6437,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.2.1; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.10.254 Version/12.00*/
const Device UA848937 = {S62371, 90, { &P17976,&P6437,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.2.6; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00*/
const Device UA1969792 = {S62372, 88, { &P17976,&P6437,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.2; Linux; Opera Tablet/ADY-X; U; ru) Presto/2.10.254 Version/12.00*/
const Device UA2028943 = {S62373, 93, { &P17976,&P6437,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.2.2; Linux; Opera Tablet/ADR-X; U; en-GB) Presto/2.10.254 Version/12.00*/
const Device UA2781616 = {S62374, 145, { &P19862,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-in; Micromax A110Q Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957794 = {S62375, 67, { &P19862,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; Micromax A110Q Build/JOP40D)*/
const Device UA3067605 = {S62376, 150, { &P19762,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI MT1-U06 Build/HuaweiMT1-U06) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA669133 = {S62377, 164, { &P16092,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB870 Build/4.5.1A-DTN-150-30; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2367095 = {S62378, 91, { &P17595,&P17294,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:17.0) Gecko/17.0 Firefox/17.0 TenFourFox/7450*/
const Device UA2513673 = {S62379, 72, { &P12280,&P18117,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPhone4,1; iPhone OS 5.1.1; en_US; en) AppleWebKit/420+*/
const Device UA2703883 = {S62380, 70, { &P12280,&P18117,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPhone4,1; iPhone OS 5.1; en_US; en) AppleWebKit/420+*/
const Device UA2454886 = {S62381, 142, { &P20001,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo S720 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455293 = {S62382, 133, { &P20001,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo S720 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2502039 = {S62383, 64, { &P20001,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo S720 Build/IMM76D)*/
const Device UA2530658 = {S62384, 140, { &P20001,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; Lenovo S720; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558821 = {S62385, 137, { &P21935,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E980 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3083421 = {S62386, 112, { &P18583,&P5043,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia110/03.33; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA627848 = {S62387, 87, { &P17595,&P17297,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; fr; rv:1.9.0.19) Gecko/X Firefox/3.0.19*/
const Device UA2198365 = {S62388, 182, { &P12897,&P5843,&P24671,&P18092 }};	/*Useragent - NokiaC2-00/2.0 (03.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2406465 = {S62389, 57, { &P15364,&P17017,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (masking-agent; rv:17.0) Gecko/X Firefox/17.0*/
const Device UA2407364 = {S62390, 137, { &P20119,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; LIFETAB_S9714 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2754175 = {S62391, 141, { &P13124,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP9070 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798457 = {S62392, 83, { &P17968,&P17979,&P18122,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/444.66/30.3214; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2876262 = {S62393, 77, { &P17968,&P17979,&P18122,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini//30.3214; U; en) Presto/2.8.119 Version/11.10*/
const Device UA3116359 = {S62394, 84, { &P17968,&P17979,&P18122,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/1963.55/31.1475; U; en) Presto/2.8.119 Version/11.10*/
const Device UA346966 = {S62395, 98, { &P18254,&P18110,&P18205,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.3.0 Safari/534.34*/
const Device UA663016 = {S62396, 104, { &P18254,&P18110,&P18205,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; C) AppleWebKit/533.3 (KHTML, like Gecko) PhantomJS/1.3.0 Safari/533.3*/
const Device UA724388 = {S62397, 99, { &P18254,&P18110,&P18205,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux; C -) AppleWebKit/532.4 (KHTML, like Gecko) PhantomJS/1.3.0 Safari/532.4*/
const Device UA1965203 = {S62398, 137, { &P13074,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P930 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1995059 = {S62399, 137, { &P13074,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P930 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2753050 = {S62400, 128, { &P19782,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B1-A71 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2753470 = {S62401, 128, { &P19783,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ME172V Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA399148 = {S62402, 70, { &P15795,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 710;7.10;7740)*/
const Device UA720040 = {S62403, 70, { &P15795,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 710;7.10;8107)*/
const Device UA925862 = {S62404, 70, { &P15795,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 710;7.10;8773)*/
const Device UA2413765 = {S62405, 70, { &P15795,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 710;7.10;8779)*/
const Device UA2496927 = {S62406, 70, { &P15795,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 710;7.10;8112)*/
const Device UA2864297 = {S62407, 93, { &P11854,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae71-1) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2967805 = {S62408, 92, { &P11854,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE71) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3019481 = {S62409, 93, { &P11854,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE71x) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA2203658 = {S62410, 226, { &P12280,&P18432,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA2348888 = {S62411, 226, { &P12280,&P18432,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA2357274 = {S62412, 228, { &P12280,&P18432,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA2386010 = {S62413, 228, { &P12280,&P18432,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA2461013 = {S62414, 228, { &P12280,&P18432,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA2234656 = {S62415, 120, { &P11905,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA6120c) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile*/
const Device UA2433636 = {S62416, 140, { &P18465,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Amazon Kindle Fire Build/JOP40C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2434395 = {S62417, 142, { &P18465,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Amazon Kindle Fire Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2507931 = {S62418, 143, { &P18465,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Amazon Kindle Fire2 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2532170 = {S62419, 141, { &P18465,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2566318 = {S62420, 146, { &P19891,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I717 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2607208 = {S62421, 138, { &P19891,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I717 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2751836 = {S62422, 147, { &P19891,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I717 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2794851 = {S62423, 69, { &P19891,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SAMSUNG-SGH-I717 Build/JDQ39E)*/
const Device UA2600969 = {S62424, 134, { &P18680,&P18110,&P21707,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Safari/535.19 Silk-Accelerated=false*/
const Device UA2975701 = {S62425, 152, { &P15877,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA92820 = {S62426, 113, { &P15364,&P5690,&P22441,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Iron/3.0.197.0 Safari/532.0*/
const Device UA2207517 = {S62427, 113, { &P15364,&P5690,&P22441,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Iron/3.0.197.0 Safari/532.0*/
const Device UA2267892 = {S62428, 113, { &P15364,&P5690,&P22441,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/531.0 (KHTML, like Gecko) Iron/3.0.189.0 Safari/531.0*/
const Device UA2364170 = {S62429, 126, { &P15364,&P5690,&P22441,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/0.0.0 Safari/532.0 Iron/3.0.197.0*/
const Device UA2410175 = {S62430, 106, { &P15364,&P5690,&P22441,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1) AppleWebKit/532.0 (KHTML, like Gecko) Iron/3.0.197.0 Safari/532.0*/
const Device UA180091 = {S62431, 82, { &P15364,&P22190,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.13) Gecko Fedora/1.9.2.13-3.el5*/
const Device UA234765 = {S62432, 90, { &P15364,&P22190,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US; rv:1.9.2.18) Gecko Fedora/1.9.2.18-1.fc14*/
const Device UA2566912 = {S62433, 134, { &P18295,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; One X Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567342 = {S62434, 138, { &P18295,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One X Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583566 = {S62435, 139, { &P18295,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; HTC One X Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA1988627 = {S62436, 137, { &P20977,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; MOT-XT390 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027213 = {S62437, 127, { &P20977,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT390 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2029990 = {S62438, 139, { &P20977,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; XT390 Build/07.15.04R_D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2686435 = {S62439, 134, { &P20977,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT390 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2562101 = {S62440, 138, { &P22227,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-uk; @TAB-NINE-X2-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2863764 = {S62441, 67, { &P22227,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; @TAB-NINE-X2-B Build/JRO03H)*/
const Device UA2672189 = {S62442, 133, { &P19907,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Rise Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677302 = {S62443, 133, { &P19907,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C5155 Build/IML77) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2674430 = {S62444, 137, { &P18621,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612g Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675268 = {S62445, 137, { &P18621,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E610v Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675521 = {S62446, 137, { &P18621,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612f Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675846 = {S62447, 136, { &P18621,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676437 = {S62448, 136, { &P18621,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E610 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA8683 = {S62449, 135, { &P13437,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us;T-Mobile G2 Build/FRF91) AppleWebKit/533.1(KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA8685 = {S62450, 135, { &P13437,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; T-MobileG2 Build/FRF91) AppleWebKit/533.1(KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10329 = {S62451, 135, { &P13437,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;Android 2.2; en-us; T-Mobile G2 Build/FRF91) AppleWebKit/533.1(KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA63929 = {S62452, 136, { &P13437,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; T-Mobile G2 Build/FRF91) AppleWebKit/533.1(KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA399667 = {S62453, 61, { &P13437,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; T-Mobile G2 Build/FRF91)*/
const Device UA2018252 = {S62454, 139, { &P13437,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile G2 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2567085 = {S62455, 137, { &P18442,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7105 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567562 = {S62456, 137, { &P18442,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7100 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583193 = {S62457, 137, { &P18442,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L900 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602041 = {S62458, 130, { &P18442,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7105 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2605540 = {S62459, 138, { &P18442,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-N7100 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2732929 = {S62460, 143, { &P18442,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Note II Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2737342 = {S62461, 138, { &P22595,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919V Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2752707 = {S62462, 131, { &P22595,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919V Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2974894 = {S62463, 137, { &P18174,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS920 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3033734 = {S62464, 136, { &P18635,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.17 Mobile Safari/537.36*/
const Device UA613311 = {S62465, 74, { &P15364,&P5690,&P21171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:12.0) Gecko/X Firefox/12.0 SeaMonkey/2.9.1*/
const Device UA740297 = {S62466, 61, { &P15364,&P5690,&P21171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:12.0) Gecko/X SeaMonkey/2.9.1*/
const Device UA833660 = {S62467, 74, { &P15364,&P5690,&P21171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:12.0) Gecko/X Firefox/12.0 SeaMonkey/2.9.1*/
const Device UA1993998 = {S62468, 81, { &P12193,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaC5-00)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA1994280 = {S62469, 83, { &P12193,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaC5-00.2)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA3070348 = {S62470, 129, { &P12193,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiac5-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile  3gpp-gba*/
const Device UA1987267 = {S62471, 135, { &P21226,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; E860 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2588263 = {S62472, 57, { &P21226,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; E860 Build/IML74K)*/
const Device UA2353529 = {S62473, 133, { &P21634,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; M-MP875S2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2516389 = {S62474, 91, { &P17595,&P17296,&P20456,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:18.0) Gecko/X Firefox/18.0 SeaMonkey/2.15.2*/
const Device UA4211 = {S62475, 55, { &P16177,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7381 Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA7063 = {S62476, 120, { &P16177,&P5448,&P3430,&P18092 }};	/*Useragent - PPC; 480x800; HTC_Touch_Pro2_T7385-Orange; OpVer 94.119.2.733 Opera/9.5 (Microsoft Windows 6.1; Opera Mobi/16071; U; fr)*/
const Device UA13467 = {S62477, 79, { &P16177,&P5448,&P3430,&P18092 }};	/*Useragent - Opera/9.5(Microsoft Windows; PPC; Opera Mobi/16643; U; en) HTC_Touch_Pro2_T7379*/
const Device UA13535 = {S62478, 55, { &P16177,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Opera/9.50 (Windows NT 5.1; U; de)*/
const Device UA24125 = {S62479, 75, { &P16177,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7381 Opera/9.50 (Windows NT 5.1; U; en) V1.19.841.3 (51489)*/
const Device UA24126 = {S62480, 75, { &P16177,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7381 Opera/9.50 (Windows NT 5.1; U; en) V1.93.841.0 (81210)*/
const Device UA32512 = {S62481, 121, { &P16177,&P5448,&P3430,&P18092 }};	/*Useragent - PPC; 480x800; HTC_Touch_Pro2_T7385-Orange; OpVer 122.190.2.731 Opera/9.5 (Microsoft Windows 6.5; Opera Mobi/17518; U; fr)*/
const Device UA48469 = {S62482, 63, { &P16177,&P5448,&P3430,&P18092 }};	/*Useragent - Swisscom/1.0/HTC_Touch_Pro2/ Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA61470 = {S62483, 63, { &P16177,&P5448,&P3430,&P18092 }};	/*Useragent - Swisscom/1.0/HTC_Touch_Pro2/ Opera/9.50 (Windows NT 5.1; U; de)*/
const Device UA66688 = {S62484, 55, { &P16177,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA114333 = {S62485, 81, { &P16177,&P5448,&P3430,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_Touch_Pro2/1.86.161.1 (67102) Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA270961 = {S62486, 55, { &P16177,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Opera/9.50 (Windows NT 5.1; U; nl)*/
const Device UA273317 = {S62487, 81, { &P16177,&P5448,&P3430,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_Touch_Pro2/1.86.161.0 (67102) Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA295261 = {S62488, 81, { &P16177,&P5448,&P3430,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_Touch_Pro2/1.14.161.6 (50207) Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA385884 = {S62489, 55, { &P16177,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Opera/9.50 (Windows NT 5.1; U; nb)*/
const Device UA863563 = {S62490, 55, { &P16177,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Opera/9.50 (Windows NT 5.1; U; it)*/
const Device UA1839935 = {S62491, 81, { &P16177,&P5448,&P3430,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_Touch_Pro2/1.14.161.5 (50207) Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA2126872 = {S62492, 55, { &P16177,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Opera/9.50 (Windows NT 5.1; U; fr)*/
const Device UA62869 = {S62493, 65, { &P12324,&P17483,&P3534,&P18092 }};	/*Useragent - LG-CU920/V1.0s Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA102575 = {S62494, 41, { &P15364,&P17017,&P17389,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/2.2.2)*/
const Device UA727501 = {S62495, 142, { &P21147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; PocketBook A10 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2658105 = {S62496, 141, { &P21147,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PocketBook A10 Build/GRJ90) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2205674 = {S62497, 136, { &P15767,&P18432,&P21787,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA2362711 = {S62498, 140, { &P15767,&P18432,&P21787,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA2460905 = {S62499, 140, { &P15767,&P18432,&P21787,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA2676286 = {S62500, 133, { &P18665,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N861 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2974113 = {S62501, 136, { &P20240,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E971 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2937336 = {S62502, 144, { &P22395,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C6606 Build/10.1.1.B.0.166) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3015265 = {S62503, 66, { &P22395,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; C6606 Build/10.1.1.B.0.166)*/
const Device UA3125092 = {S62504, 68, { &P22395,&P18111,&P18112,&P18092 }};	/*Useragent - SonyC6606 Build/10.1.1.B.0.166 stagefright/1.2 (Linux;Android 4.1.2)*/
const Device UA13427 = {S62505, 89, { &P13296,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/X1i/1.03.175.7*/
const Device UA13426 = {S62506, 90, { &P13296,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/X1i/1.05.175.13*/
const Device UA13433 = {S62507, 171, { &P13296,&P5448,&P22270,&P18092 }};	/*Useragent - SonyEricssonX1i/R3AA Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SonyEricssonX1i/R3AA Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA13431 = {S62508, 83, { &P13296,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Swisscom/1.0/X1i/R3AA*/
const Device UA13690 = {S62509, 89, { &P13296,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/X1i/1.02.175.3*/
const Device UA22305 = {S62510, 65, { &P13296,&P5448,&P22270,&P18092 }};	/*Useragent - X1i Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA34829 = {S62511, 89, { &P13296,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Swisscom/1.0/X1i/1.02.166.7*/
const Device UA54976 = {S62512, 122, { &P13296,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SonyEricssonX1i/R3AA Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA89099 = {S62513, 144, { &P13296,&P5448,&P22270,&P18092 }};	/*Useragent - Vodafone/1.0/SEX1i/R3AA Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) UP.Link/6.3.1.20.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA466813 = {S62514, 126, { &P13296,&P5448,&P22270,&P18092 }};	/*Useragent - Vodafone/1.0/SEX1i/R1AA Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)  Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA1960816 = {S62515, 140, { &P16319,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; U8860 Build/HuaweiU8860) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2709884 = {S62516, 140, { &P20517,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; CINK PEAX Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2862857 = {S62517, 140, { &P20517,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; Cink Peax Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892005 = {S62518, 62, { &P20517,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; CINK PEAX Build/JRO03C)*/
const Device UA3078514 = {S62519, 142, { &P20517,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; CINK PEAX 2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1886770 = {S62520, 129, { &P15364,&P5690,&P18362,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729) Sleipnir/2.8.3*/
const Device UA2144495 = {S62521, 65, { &P15364,&P5690,&P18362,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Sleipnir/2.8.1*/
const Device UA2297116 = {S62522, 116, { &P15364,&P5690,&P18362,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR 2.0.50727) Sleipnir/2.8.4*/
const Device UA2408592 = {S62523, 185, { &P15364,&P5690,&P18362,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Sleipnir/2.8.5)*/
const Device UA2409115 = {S62524, 136, { &P15364,&P5690,&P18362,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.2; WOW64; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022) Sleipnir/2.8.4*/
const Device UA2473172 = {S62525, 151, { &P15364,&P5690,&P18362,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648) Sleipnir/2.8.0*/
const Device UA2586722 = {S62526, 153, { &P15364,&P5690,&P18362,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729) Sleipnir/2.8.4*/
const Device UA2040854 = {S62527, 135, { &P15364,&P17820,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.1 (KHTML, like Gecko) Ubuntu/12.04 Chromium/21.0.1180.79 Chrome/21.0.1180.79 Safari/537.1*/
const Device UA2042680 = {S62528, 137, { &P15364,&P17820,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.1 (KHTML, like Gecko) Ubuntu/12.04 Chromium/21.0.1180.79 Chrome/21.0.1180.79 Safari/537.1*/
const Device UA2947170 = {S62529, 93, { &P11929,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5230) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3107668 = {S62530, 95, { &P11929,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5230-c) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3067067 = {S62531, 136, { &P22679,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068253 = {S62532, 138, { &P15972,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One XL Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA1960316 = {S62533, 143, { &P20135,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Enjoy 7 Plus Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558919 = {S62534, 246, { &P21405,&P5629,&P15760,&P18092 }};	/*Useragent - Lenovo-A278t_TD/S100 Linux/2.6.35 Android/2.3.5 Release/09.03.2012 Browser/AppleWebkit533.1 Mobile Safari/533.1LENOVO-Lenovo-A278t/1.0 Linux/2.6.35.7 Android 2.3.5 Release/12.27.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2733879 = {S62535, 111, { &P21405,&P5629,&P15760,&P18092 }};	/*Useragent - Lenovo-A278t_TD/S100 Linux/2.6.35 Android/2.3.5 Release/09.03.2012 Browser/AppleWebkit533.1 Mobile Safari/533.1*/
const Device UA2764044 = {S62536, 150, { &P21405,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A278t Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2952303 = {S62537, 143, { &P21405,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A278t Build/MocorDroid2.3.5) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3044674 = {S62538, 74, { &P21405,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Lenovo A278t Build/MocorDroid2.3.5)*/
const Device UA2976093 = {S62539, 143, { &P18009,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTCEVODesign4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023767 = {S62540, 136, { &P18009,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PH44100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA451670 = {S62541, 124, { &P17595,&P18120,&P18210,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.1.0.5001 Safari/532.4*/
const Device UA1986525 = {S62542, 131, { &P23915,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995554 = {S62543, 141, { &P19997,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Lenovo P700 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2353037 = {S62544, 142, { &P19997,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo P700i Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2206706 = {S62545, 114, { &P18606,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; PadFone 2) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA3096045 = {S62546, 132, { &P15767,&P22882,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/11A465 Safari/8536.25*/
const Device UA3130005 = {S62547, 134, { &P15767,&P22882,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/11A501 Safari/8536.25*/
const Device UA307 = {S62548, 143, { &P12544,&P17486,&P20449,&P18092 }};	/*Useragent - MOT-V8/R601_G_80.44.2BR Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola V8; 3288) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.50 [zh-tw]*/
const Device UA305 = {S62549, 137, { &P12544,&P17486,&P20449,&P18092 }};	/*Useragent - MOT-V8/R601_G_80.56.1BRP Mozilla/4.0 (compatible; MSIE 6.0 Linux; Motorola V8;nnn) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.50[yy]*/
const Device UA311 = {S62550, 143, { &P12544,&P17486,&P20449,&P18092 }};	/*Useragent - MOT-V8/R601_G_80.54.0CR Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola V8; 3109) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.50 [zh-tw]*/
const Device UA312 = {S62551, 141, { &P12544,&P17486,&P20449,&P18092 }};	/*Useragent - MOT-MOTORAZRV8_CMCC/1.0 LinuxOS/2.6.10 Release/06.30.2007 Browser/Opera8.50 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R601_G_80.44.2DR*/
const Device UA14431 = {S62552, 139, { &P12544,&P17486,&P20449,&P18092 }};	/*Useragent - MOT-MOTORAZRV82GB/1.0 LinuxOS/2.6.10 Release/08.30.2007 Browser/Opera8.50 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R601_G_80.54.60R*/
const Device UA14446 = {S62553, 143, { &P12544,&P17486,&P20449,&P18092 }};	/*Useragent - MOT-V8/R601_G_80.41.1BR Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola V8; 2961) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.50 [zh-tw]*/
const Device UA31410 = {S62554, 86, { &P12544,&P17486,&P20449,&P18092 }};	/*Useragent - motorazrV8/R601_G_80.56.1BR Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.50 [es-LA]*/
const Device UA35106 = {S62555, 146, { &P12544,&P17486,&P20449,&P18092 }};	/*Useragent - MOT-MOTORAZRV82GB_CMCC/1.0 LinuxOS/2.6.10 Release/08.30.2007 Browser/Opera8.50 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R601_G_80.56.15R_B*/
const Device UA64689 = {S62556, 141, { &P12544,&P17486,&P20449,&P18092 }};	/*Useragent - motorazrV8/R601_G_80.44.2BRP Mozilla/4.0 (compatible; MSIE 6.0 Linux; Motorola V8;nnn) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.50[yy]*/
const Device UA368856 = {S62557, 152, { &P12784,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Skate Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA905525 = {S62558, 151, { &P13505,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB525 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027240 = {S62559, 151, { &P13505,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB525 Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028550 = {S62560, 151, { &P13505,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MB525 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2759785 = {S62561, 173, { &P13505,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MB525+ Build/IMM76L; CyanogenMod-Fast Edition Black 2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA905094 = {S62562, 132, { &P13452,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2609352 = {S62563, 137, { &P13452,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P1000 Build/GINGERBREAD) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2610133 = {S62564, 138, { &P13452,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  GT-P1000 Build/GINGERBREAD) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2611471 = {S62565, 133, { &P13452,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000L Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2612402 = {S62566, 140, { &P13452,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P1000 Build/MjhieuS3) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2612983 = {S62567, 133, { &P13452,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  GT-P1000 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2613294 = {S62568, 138, { &P13452,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2617384 = {S62569, 133, { &P13452,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000N Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2626082 = {S62570, 138, { &P13452,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; GT-P1000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2566697 = {S62571, 130, { &P17686,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ601 Build/I.7.1-45) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA344494 = {S62572, 128, { &P17965,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; SC-01D Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2030023 = {S62573, 57, { &P17965,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; SC-01D Build/MASTER)*/
const Device UA2257297 = {S62574, 146, { &P21333,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; INO_ONE Build/INO_ONEINO_ONE) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2673460 = {S62575, 143, { &P21333,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; INO_ONE_PLUS Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3046667 = {S62576, 138, { &P21333,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; INO_ONE Build/IMM74D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567207 = {S62577, 138, { &P18174,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS920 4G Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2676675 = {S62578, 129, { &P17687,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ604 Build/I.7.1-45) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676726 = {S62579, 129, { &P17687,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MZ604 Build/I.7.1-34) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA163828 = {S62580, 132, { &P17964,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; SC-01C Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1864615 = {S62581, 56, { &P17964,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; SC-01C Build/FROYO)*/
const Device UA2204374 = {S62582, 138, { &P18456,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T699 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2423743 = {S62583, 118, { &P13163,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaN97) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2483851 = {S62584, 40, { &P13163,&P5129,&P24673,&P18092 }};	/*Useragent - NokiaN97UCBrowser/8.7.1.234/50/444/UCWEB*/
const Device UA2656716 = {S62585, 92, { &P13163,&P5129,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokian97-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2566922 = {S62586, 138, { &P18673,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583424 = {S62587, 139, { &P18673,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562L Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA3027206 = {S62588, 113, { &P14540,&P5843,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-01/08.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3108467 = {S62589, 115, { &P14540,&P5843,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-01.1/07.22; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA428799 = {S62590, 100, { &P15364,&P17188,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11*/
const Device UA1997947 = {S62591, 155, { &P16757,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I500 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2001056 = {S62592, 155, { &P16757,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I500 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2233618 = {S62593, 155, { &P16757,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I500 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2239152 = {S62594, 155, { &P16757,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I500 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2485940 = {S62595, 171, { &P18645,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SPH-L710, 39, 1.8.2)*/
const Device UA2670289 = {S62596, 171, { &P18645,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SPH-L710, 42, 1.9.1)*/
const Device UA2670293 = {S62597, 187, { &P18645,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SAMSUNG-SGH-I747, 42, 1.9.1)*/
const Device UA2750623 = {S62598, 173, { &P18645,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-I747M Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SGH-I747M, 42, 1.9.1)*/
const Device UA2944177 = {S62599, 173, { &P18645,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I747M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SGH-I747M, 47, 1.9.7)*/
const Device UA2954106 = {S62600, 173, { &P18645,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530U Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SCH-R530U, 42, 1.9.1)*/
const Device UA3063789 = {S62601, 187, { &P18645,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SAMSUNG-SGH-I747, 54, 2.0.1)*/
const Device UA2532958 = {S62602, 131, { &P21820,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TBD753B Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2663181 = {S62603, 162, { &P15575,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA3002734 = {S62604, 170, { &P15575,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-I9100 Build/miui.es JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA3017797 = {S62605, 168, { &P15575,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9100 Build/JZO54K.ASNET) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA3047063 = {S62606, 163, { &P15575,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9100G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA94343 = {S62607, 7, { &P18254,&P7480,&P18589,&P18190 }};	/*Useragent - BDFetch*/
const Device UA398953 = {S62608, 78, { &P18254,&P7480,&P18245,&P18170 }};	/*Useragent - Moreoverbot/5.1 (+http://www.moreover.com; webmaster@moreover.com) Mozilla/5.0*/
const Device UA1996695 = {S62609, 78, { &P18254,&P7480,&P18245,&P18170 }};	/*Useragent - Mozilla/5.0 Moreoverbot/5.1 (+http://www.moreover.com; webmaster@moreover.com)*/
const Device UA824853 = {S62610, 50, { &P15364,&P17188,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.0)*/
const Device UA2165462 = {S62611, 63, { &P15364,&P17188,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.0; Trident/4.0)*/
const Device UA2217603 = {S62612, 55, { &P15364,&P17188,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.0; SV1)*/
const Device UA2443480 = {S62613, 151, { &P15364,&P17188,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.0; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA2495866 = {S62614, 97, { &P15364,&P17188,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)*/
const Device UA2851601 = {S62615, 57, { &P15364,&P17188,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; WinNT4.0; Trident/4.0)*/
const Device UA3042899 = {S62616, 178, { &P15364,&P17188,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA2010505 = {S62617, 160, { &P19936,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A100/Micromax_A100_V1.0; 480*854;  CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528552 = {S62618, 160, { &P19936,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A100/Micromax_A100_V1.1; 480*854;  CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469431 = {S62619, 157, { &P16757,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-I500 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2562920 = {S62620, 156, { &P16757,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I500 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2563977 = {S62621, 157, { &P16757,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-I500 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708769 = {S62622, 157, { &P16757,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I500 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2581081 = {S62623, 116, { &P14862,&P5204,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2700c-2/09.97; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2566701 = {S62624, 146, { &P12657,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515m Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583369 = {S62625, 146, { &P12657,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515a Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2989791 = {S62626, 139, { &P12657,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC_X515E Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2570230 = {S62627, 98, { &P18254,&P16110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 8 6.2; java 1.6.0_41; Europe/de) http://yacy.net/bot.html*/
const Device UA3008066 = {S62628, 98, { &P18254,&P16110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 8 6.2; java 1.7.0_25; Europe/de) http://yacy.net/bot.html*/
const Device UA3058492 = {S62629, 99, { &P18254,&P16110,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows 8 6.2; java 1.7.0_25; America/en) http://yacy.net/bot.html*/
const Device UA64441 = {S62630, 107, { &P11827,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-D880/D880XAHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA2460051 = {S62631, 128, { &P18115,&P7106,&P15829,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9620; es) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.807 Mobile Safari/534.11+*/
const Device UA3022843 = {S62632, 93, { &P13123,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5360) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3063436 = {S62633, 90, { &P13123,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; vi; GT-S5360) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA371531 = {S62634, 4, { &P15767,&P17779,&P18589,&P18092 }};	/*Useragent - iPad*/
const Device UA2738761 = {S62635, 40, { &P15767,&P17779,&P18589,&P18092 }};	/*Useragent - /media/sandisk/af_mobile/user_agent/ipad*/
const Device UA2468808 = {S62636, 129, { &P23885,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N70-S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2762885 = {S62637, 58, { &P23885,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; N70-S Build/JRO03H)*/
const Device UA2671928 = {S62638, 146, { &P22626,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Iris 349 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2965749 = {S62639, 147, { &P22626,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Iris 349+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3028262 = {S62640, 204, { &P22582,&P18598,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us*/
const Device UA3065459 = {S62641, 149, { &P18318,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094093 = {S62642, 150, { &P18318,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700KL Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA22686 = {S62643, 74, { &P13667,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-T369R/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63697 = {S62644, 73, { &P13667,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-T369/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA707814 = {S62645, 111, { &P15364,&P7480,&P18099,&P18091 }};	/*Useragent - lb-spider/Mozilla/5.0 Gecko/X Firefox/10.0.2 (lb-spider; http://www.linkbutler.de/spider; spider@linkbutler.de)*/
const Device UA2679684 = {S62646, 130, { &P18454,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGPT12 Build/TJS0166) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2708923 = {S62647, 130, { &P18454,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGPT13 Build/TJS0166) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2699112 = {S62648, 155, { &P22856,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0151 Build/HuaweiY300-0151) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2754060 = {S62649, 141, { &P19778,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP9090 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA24753 = {S62650, 46, { &P15700,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-UN200/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA31365 = {S62651, 80, { &P12417,&P17483,&P4628,&P18092 }};	/*Useragent - User-Agent:LG-LX600 Polaris/6.0 MMP/2.0 Profilie/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63144 = {S62652, 68, { &P12417,&P17483,&P4628,&P18092 }};	/*Useragent - LG-LX600 Polaris/6.0 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA165349 = {S62653, 79, { &P15364,&P7133,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 6.1; Microsoft Office Outlook 12.0.6545; Pro)*/
const Device UA176624 = {S62654, 79, { &P15364,&P7133,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 6.1; Microsoft Office Outlook 12.0.4518; Pro)*/
const Device UA189125 = {S62655, 79, { &P15364,&P7133,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 6.1; Microsoft Office Outlook 12.0.6554; Pro)*/
const Device UA244752 = {S62656, 79, { &P15364,&P7133,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 6.1; Microsoft Office Outlook 12.0.6562; Pro)*/
const Device UA379899 = {S62657, 79, { &P15364,&P7133,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 6.1; Microsoft Office Outlook 12.0.6607; Pro)*/
const Device UA516462 = {S62658, 79, { &P15364,&P7133,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 6.1; Microsoft Office Outlook 12.0.6320; Pro)*/
const Device UA623029 = {S62659, 79, { &P15364,&P7133,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 6.1; Microsoft Office Outlook 12.0.6425; Pro)*/
const Device UA633693 = {S62660, 79, { &P15364,&P7133,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 6.1; Microsoft Office Outlook 12.0.6535; Pro)*/
const Device UA2010223 = {S62661, 79, { &P15364,&P7133,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 6.1; Microsoft Office Outlook 12.0.6662; Pro)*/
const Device UA2802060 = {S62662, 79, { &P15364,&P7133,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 6.1; Microsoft Office Outlook 12.0.6213; Pro)*/
const Device UA3083299 = {S62663, 79, { &P15364,&P7133,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 6.1; Microsoft Office Outlook 12.0.6683; Pro)*/
const Device UA350373 = {S62664, 140, { &P20858,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Scroll Excel Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2487104 = {S62665, 150, { &P20005,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Micromax A65 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2672611 = {S62666, 130, { &P18437,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8005 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676021 = {S62667, 130, { &P18437,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8000 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2973656 = {S62668, 141, { &P14969,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975005 = {S62669, 142, { &P14969,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire SV Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021557 = {S62670, 141, { &P14969,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2258390 = {S62671, 120, { &P14748,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE72-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/355/UCWEB Mobile*/
const Device UA2342120 = {S62672, 32, { &P14748,&P5890,&P24672,&P18092 }};	/*Useragent - NokiaE72-1/UCWEB8.6.0.180/28/800*/
const Device UA2578966 = {S62673, 129, { &P14748,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiae72-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/84/352/UCWEB Mobile  3gpp-gba*/
const Device UA2566544 = {S62674, 142, { &P14969,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567507 = {S62675, 143, { &P14969,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire SV Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2861683 = {S62676, 142, { &P14969,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire S Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2755119 = {S62677, 137, { &P18174,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS920 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA15477 = {S62678, 70, { &P14569,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia7610Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA62978 = {S62679, 62, { &P14569,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia7610s/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2007126 = {S62680, 119, { &P15364,&P18110,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1*/
const Device UA2040311 = {S62681, 117, { &P15364,&P18110,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1*/
const Device UA3085813 = {S62682, 82, { &P15364,&P18110,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux amd64) Iron/21.0.1200.0 Chrome/21.0.1200.0 Safari/537.1*/
const Device UA2484657 = {S62683, 134, { &P20158,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A7* Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2549239 = {S62684, 139, { &P20158,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; epade A7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934565 = {S62685, 56, { &P20158,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; A7* Build/IMM76D)*/
const Device UA2997370 = {S62686, 136, { &P20158,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A7 3G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2511133 = {S62687, 154, { &P12059,&P5204,&P24676,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2690) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2678442 = {S62688, 78, { &P15364,&P5690,&P22560,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:20.0) Gecko/X Firefox/20.0 IceDragon/20.0.1.14*/
const Device UA2707618 = {S62689, 161, { &P15991,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6102 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA27375 = {S62690, 96, { &P16087,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)  480x800 LGE VS750*/
const Device UA2201279 = {S62691, 193, { &P15364,&P7133,&P24672,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)UCBrowser/8.6.0.199/50/352/UCWEB*/
const Device UA2944805 = {S62692, 97, { &P15364,&P18110,&P24685,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.21 (KHTML, like Gecko) rekonq/2.3.2 Safari/537.21*/
const Device UA2961997 = {S62693, 97, { &P15364,&P18110,&P24685,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) rekonq/2.3.2 Safari/534.34*/
const Device UA2756009 = {S62694, 137, { &P13991,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6400L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2796193 = {S62695, 140, { &P13991,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ThunderBolt Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3096099 = {S62696, 47, { &P15364,&P23096,&P22245,&P18092 }};	/*Useragent - Flipboard/2.0.7 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA2974581 = {S62697, 133, { &P17172,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Transformer Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2975810 = {S62698, 138, { &P17172,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Transformer TF101 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022114 = {S62699, 133, { &P17172,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Transformer Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3065617 = {S62700, 133, { &P18466,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110342 = {S62701, 126, { &P18466,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080986 = {S62702, 139, { &P22641,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C1504 Build/11.3.A.0.47) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2258479 = {S62703, 123, { &P13324,&P5055,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; NokiaN8-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/51/355/UCWEB Mobile*/
const Device UA2380307 = {S62704, 32, { &P13324,&P5055,&P24672,&P18092 }};	/*Useragent - NokiaN8-00/UCWEB8.6.0.180/51/800*/
const Device UA2403969 = {S62705, 129, { &P13324,&P5055,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian8-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA2752949 = {S62706, 136, { &P18009,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PH44100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754170 = {S62707, 143, { &P18009,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTCEVODesign4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA200671 = {S62708, 134, { &P17995,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; SH-03C Build/S3240) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250334 = {S62709, 134, { &P17995,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; SH-03C Build/S9130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA453499 = {S62710, 134, { &P17995,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; SH-03C Build/S3010) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1905761 = {S62711, 58, { &P17995,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; SH-03C Build/S9130)*/
const Device UA2042574 = {S62712, 134, { &P17995,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; SH-03C Build/S8100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626559 = {S62713, 120, { &P21321,&P5629,&P15760,&P18092 }};	/*Useragent - SAMSUNG-GT-I9228_TD/1.0 Android/2.3.5 Release/9.15.2011 Browser/AppleWebKit533.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA731605 = {S62714, 142, { &P21321,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9228 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA737447 = {S62715, 130, { &P21321,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9228 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1992628 = {S62716, 135, { &P21321,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9228 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2029217 = {S62717, 66, { &P21321,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9228 Build/GINGERBREAD)*/
const Device UA2675379 = {S62718, 141, { &P18382,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire C Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676501 = {S62719, 141, { &P18382,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire C Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2784806 = {S62720, 138, { &P18452,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT29i Build/9.1.B.1.67) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2800707 = {S62721, 142, { &P20089,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-01E Build/9.1.C.0.473) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2801050 = {S62722, 146, { &P20089,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-01E Build/9.1.C.0.473) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2885446 = {S62723, 146, { &P20089,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-01E Build/9.1.C.0.475) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2897688 = {S62724, 142, { &P20089,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-01E Build/9.1.C.0.475) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2943782 = {S62725, 64, { &P20089,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SO-01E Build/9.1.C.0.475)*/
const Device UA3112495 = {S62726, 146, { &P20089,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-01E Build/9.1.C.1.103) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3128021 = {S62727, 142, { &P20089,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-01E Build/9.1.C.1.103) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422196 = {S62728, 139, { &P12496,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I897 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2568300 = {S62729, 138, { &P12496,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I897 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583364 = {S62730, 139, { &P12496,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I897 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2738026 = {S62731, 139, { &P12496,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I897 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957103 = {S62732, 61, { &P12496,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGH-I897 Build/JDQ39E)*/
const Device UA2433192 = {S62733, 132, { &P12879,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P7510 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2534063 = {S62734, 131, { &P12879,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7510 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA64418 = {S62735, 65, { &P12117,&P17483,&P3499,&P18092 }};	/*Useragent - LG-CF360/V10h Teleca/Q7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA408615 = {S62736, 72, { &P15364,&P6704,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows 98) Opera 7.02 [en]*/
const Device UA818335 = {S62737, 31, { &P15364,&P6704,&P4810,&P18092 }};	/*Useragent - Opera/7.02 (Windows 98; U) [en]*/
const Device UA830678 = {S62738, 57, { &P15364,&P6704,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98) Opera 7.21*/
const Device UA838403 = {S62739, 57, { &P15364,&P6704,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98) Opera 7.54*/
const Device UA858898 = {S62740, 31, { &P15364,&P6704,&P4810,&P18092 }};	/*Useragent - Opera/7.03 (Windows 98; U) [en]*/
const Device UA859177 = {S62741, 31, { &P15364,&P6704,&P4810,&P18092 }};	/*Useragent - Opera/7.23 (Windows 98; U) [en]*/
const Device UA863708 = {S62742, 72, { &P15364,&P6704,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows 98) Opera 7.03 [de]*/
const Device UA893751 = {S62743, 30, { &P15364,&P6704,&P4810,&P18092 }};	/*Useragent - Opera/7.0 (Windows 98; U) [en]*/
const Device UA1894414 = {S62744, 57, { &P15364,&P6704,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98) Opera 7.55*/
const Device UA2462586 = {S62745, 32, { &P15364,&P6704,&P4810,&P18092 }};	/*Useragent - Opera/7.54 (Windows 98; U)  [en]*/
const Device UA2484239 = {S62746, 32, { &P15364,&P6704,&P4810,&P18092 }};	/*Useragent - Opera/7.11 (Windows 98; U)  [nl]*/
const Device UA2638726 = {S62747, 62, { &P15364,&P6704,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98) Opera 7.20 [ru]*/
const Device UA2681132 = {S62748, 31, { &P15364,&P6704,&P4810,&P18092 }};	/*Useragent - Opera/7.21 (Windows 98; U) [en]*/
const Device UA2681824 = {S62749, 62, { &P15364,&P6704,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98) Opera 7.10 [en]*/
const Device UA2682206 = {S62750, 62, { &P15364,&P6704,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98) Opera 7.23 [ru]*/
const Device UA2683146 = {S62751, 62, { &P15364,&P6704,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98) Opera 7.53 [en]*/
const Device UA2684526 = {S62752, 72, { &P15364,&P6704,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows 98) Opera 7.01 [ru]*/
const Device UA2690942 = {S62753, 71, { &P15364,&P6704,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows 98) Opera 7.0 [ru]*/
const Device UA2801657 = {S62754, 31, { &P15364,&P6704,&P4810,&P18092 }};	/*Useragent - Opera/7.01 (Windows 98; U) [fr]*/
const Device UA447268 = {S62755, 104, { &P15364,&P18137,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.10) Gecko/X Linux Mint/10 (Julia) Firefox/3.6.10 GTB7.1*/
const Device UA487152 = {S62756, 99, { &P15364,&P18137,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.12) Gecko/X Linux Mint/10 (Julia) Firefox/3.6.12*/
const Device UA622901 = {S62757, 97, { &P15364,&P18137,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.24) Gecko/X Linux Mint/10 (Julia) Firefox/3.6.24*/
const Device UA640954 = {S62758, 99, { &P15364,&P18137,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.24) Gecko/X Linux Mint/10 (Julia) Firefox/3.6.24*/
const Device UA775523 = {S62759, 104, { &P15364,&P18137,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.24) Gecko/X Linux Mint/10 (Julia) Firefox/3.6.24 GTB7.1*/
const Device UA1880836 = {S62760, 104, { &P15364,&P18137,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.17) Gecko/X Linux Mint/10 (Julia) Firefox/3.6.17 GTB7.1*/
const Device UA1889876 = {S62761, 104, { &P15364,&P18137,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.18) Gecko/X Linux Mint/10 (Julia) Firefox/3.6.18 GTB7.1*/
const Device UA1900621 = {S62762, 104, { &P15364,&P18137,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.23) Gecko/X Linux Mint/10 (Julia) Firefox/3.6.23 GTB7.1*/
const Device UA1956046 = {S62763, 146, { &P14708,&P5204,&P24670,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5130c-2) UCBrowser8.4.0.159/70/352/UCWEB Mobile*/
const Device UA2840783 = {S62764, 140, { &P14708,&P5204,&P24670,&P18092 }};	/*Useragent - Nokia5130/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia5130) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2604564 = {S62765, 140, { &P21963,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; XOLO_Q800 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2858953 = {S62766, 57, { &P21963,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Q800 Build/JZO54K)*/
const Device UA2855218 = {S62767, 138, { &P22956,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; LG-P875 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103414 = {S62768, 60, { &P22956,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P875 Build/JZO54K)*/
const Device UA49238 = {S62769, 73, { &P12967,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-C3350/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA1953352 = {S62770, 87, { &P12967,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-C3350_ORANGE/C3350BVKL5 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2569825 = {S62771, 91, { &P17595,&P17296,&P21791,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:19.0) Gecko/X Firefox/19.0 SeaMonkey/2.16.2*/
const Device UA2566542 = {S62772, 136, { &P18468,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; IdeaTabA2109A Build/JRO03R) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA54421 = {S62773, 109, { &P11681,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-J700i/J700IXAIE1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA113579 = {S62774, 121, { &P11681,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-SGH-J700i/BUIE2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA141434 = {S62775, 121, { &P11681,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-SGH-J700i/JBID1/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA459924 = {S62776, 121, { &P11681,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-SGH-J700i/BUIC2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA197422 = {S62777, 133, { &P18006,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; P-07C Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1922570 = {S62778, 57, { &P18006,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; P-07C Build/GRI40)*/
const Device UA379335 = {S62779, 30, { &P11929,&P5129,&P24665,&P18092 }};	/*Useragent - Nokia5230/UCWEB8.0.3.99/50/999*/
const Device UA437292 = {S62780, 42, { &P11929,&P5129,&P24665,&P18092 }};	/*Useragent - Nokia5230/UC Browser8.0.4.121/50/444/UCWEB*/
const Device UA415748 = {S62781, 116, { &P15364,&P7133,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/523.12.9 (KHTML, like Gecko) Version/3.0 Safari/523.12.9*/
const Device UA725289 = {S62782, 115, { &P15364,&P7133,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; da) AppleWebKit/522.15.5 (KHTML, like Gecko) Version/3.0.3 Safari/522.15.5*/
const Device UA2139772 = {S62783, 115, { &P15364,&P7133,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en) AppleWebKit/522.13.1 (KHTML, like Gecko) Version/3.0.2 Safari/522.13.1*/
const Device UA2295069 = {S62784, 112, { &P15364,&P7133,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/523.15 (KHTML, like Gecko) Version/3.0 Safari/523.15*/
const Device UA2362014 = {S62785, 152, { &P18640,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT901 Build/7.7.1Q-144_SLS_S240-55) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2367902 = {S62786, 152, { &P18640,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT901 Build/7.7.1Q-144_SLS_S240-58) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2430272 = {S62787, 74, { &P18640,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; XT901 Build/7.7.1Q-144_SLS_S240-58)*/
const Device UA2673617 = {S62788, 154, { &P17968,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; cm_tenderloin Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2709756 = {S62789, 155, { &P17968,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; skz_tenderloin Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2870142 = {S62790, 163, { &P17968,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; skz_tenderloin Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2893342 = {S62791, 156, { &P17968,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; skz_tenderloin Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA67169 = {S62792, 116, { &P14768,&P22839,&P4732,&P18092 }};	/*Useragent - ZTE-F160/1.3.0; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.20822; U; en-US) Opera 9.50*/
const Device UA697045 = {S62793, 136, { &P17717,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-SU760 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022877 = {S62794, 60, { &P17717,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; LG-SU760 Build/GRJ90)*/
const Device UA923876 = {S62795, 81, { &P12449,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE5-00)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2645535 = {S62796, 83, { &P12449,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE5-00.2)/UC Browser8.5.0.183/28/444/UCWEB Mobile*/
const Device UA2654449 = {S62797, 129, { &P12449,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiae5-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile  3gpp-gba*/
const Device UA2811991 = {S62798, 32, { &P12449,&P5890,&P24671,&P18092 }};	/*Useragent - NOKIAE5-00/UCWEB8.5.0.163/28/800*/
const Device UA2227005 = {S62799, 142, { &P12897,&P5843,&P24670,&P18092 }};	/*Useragent - NokiaC2-00/2.0 (03.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac2-00) UCBrowser8.4.0.159/70/444/UCWEB Mobile*/
const Device UA2973996 = {S62800, 108, { &P15364,&P17350,&P22779,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; ; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/7.1.1.1888 Safari/532.4*/
const Device UA716521 = {S62801, 147, { &P23995,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ZiiLABS ZiiO10  Build/v2.00.14-FROYO) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile Safari/528.16*/
const Device UA656180 = {S62802, 131, { &P15201,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710e; zh-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA675301 = {S62803, 131, { &P15201,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710e; sv-se) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA734388 = {S62804, 128, { &P15201,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_4G; en-us) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA761517 = {S62805, 131, { &P15201,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710e; en-us) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA768062 = {S62806, 131, { &P15201,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710e; en-gb) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA777707 = {S62807, 131, { &P15201,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710e; nl-nl) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA923941 = {S62808, 131, { &P15201,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710e; en-in) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA1994540 = {S62809, 131, { &P15201,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710e; en-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2160064 = {S62810, 131, { &P15201,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710e; ru-ru) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2221549 = {S62811, 131, { &P15201,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710e; en-hk) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2327151 = {S62812, 131, { &P15201,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710e; fr-fr) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2545876 = {S62813, 131, { &P15201,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_Sensation_Z710e; zh-cn) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2257904 = {S62814, 136, { &P24735,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; B3000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2427386 = {S62815, 140, { &P20174,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; CINK KING Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987117 = {S62816, 62, { &P20174,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; CINK KING Build/IMM76D)*/
const Device UA2488331 = {S62817, 133, { &P18464,&P18110,&P19837,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Safari/535.19 Silk-Accelerated=true*/
const Device UA2753004 = {S62818, 137, { &P12814,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I777 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2800476 = {S62819, 145, { &P12814,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I777 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3068388 = {S62820, 145, { &P18614,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT890 Build/8.7.1I-110_SML-20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA383220 = {S62821, 49, { &P17595,&P17017,&P18108,&P18092 }};	/*Useragent - Mozilla/6.0 (Macintosh; U; Amiga-AWeb) Safari 3.1*/
const Device UA400768 = {S62822, 52, { &P17595,&P17017,&P18108,&P18092 }};	/*Useragent - QuickTime/7.6.2 (qtver=7.6.2;cpu=IA32;os=Mac 10.5.8)*/
const Device UA432395 = {S62823, 48, { &P17595,&P17017,&P18108,&P18092 }};	/*Useragent - QuickTime/7.6 (qtver=7.6;cpu=IA32;os=Mac 10.5.8)*/
const Device UA447246 = {S62824, 26, { &P17595,&P17017,&P18108,&P18092 }};	/*Useragent - MacNetwork/1.0 (Macintosh)*/
const Device UA447799 = {S62825, 50, { &P17595,&P17017,&P18108,&P18092 }};	/*Useragent - QuickTime/7.6.3 (qtver=7.6.3;cpu=IA32;os=Mac 10.6)*/
const Device UA450697 = {S62826, 53, { &P17595,&P17017,&P18108,&P18092 }};	/*Useragent - QuickTime.7.6.6 (qtver=7.6.6;cpu=IA32;os=Mac 10.6.8).*/
const Device UA459852 = {S62827, 52, { &P17595,&P17017,&P18108,&P18092 }};	/*Useragent - QuickTime/7.6.3 (qtver=7.6.3;cpu=IA32;os=Mac 10.6.1)*/
const Device UA459988 = {S62828, 52, { &P17595,&P17017,&P18108,&P18092 }};	/*Useragent - QuickTime/7.6.3 (qtver=7.6.3;cpu=IA32;os=Mac 10.6.2)*/
const Device UA493935 = {S62829, 51, { &P17595,&P17017,&P18108,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; WebCapture 3.0; Macintosh)*/
const Device UA539929 = {S62830, 48, { &P17595,&P17017,&P18108,&P18092 }};	/*Useragent - QuickTime/7.6 (qtver=7.6;cpu=IA32;os=Mac 10.5.6)*/
const Device UA573124 = {S62831, 52, { &P17595,&P17017,&P18108,&P18092 }};	/*Useragent - QuickTime/7.5.5 (qtver=7.5.5;cpu=IA32;os=Mac 10.5.8)*/
const Device UA594113 = {S62832, 31, { &P17595,&P17017,&P18108,&P18092 }};	/*Useragent - TuneUpApp/2.4.2 (Mac; OSX 10.7)*/
const Device UA644659 = {S62833, 52, { &P17595,&P17017,&P18108,&P18092 }};	/*Useragent - QuickTime/7.6.1 (qtver=7.6.1;cpu=IA32;os=Mac 10.5.8)*/
const Device UA708899 = {S62834, 53, { &P17595,&P17017,&P18108,&P18092 }};	/*Useragent - QuickTime.7.6.9 (qtver=7.6.9;cpu=IA32;os=Mac 10.5.8).*/
const Device UA872175 = {S62835, 31, { &P17595,&P17017,&P18108,&P18092 }};	/*Useragent - TuneUpApp/2.4.2 (Mac; OSX 10.6)*/
const Device UA1895322 = {S62836, 49, { &P17595,&P17017,&P18108,&P18092 }};	/*Useragent - QuickTime.7.7 (qtver=7.7;cpu=IA32;os=Mac 10.5.8).*/
const Device UA2624184 = {S62837, 61, { &P15919,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-P3100 Build/JOP40D)*/
const Device UA2633247 = {S62838, 60, { &P15919,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P3100 Build/JDQ39)*/
const Device UA2636058 = {S62839, 174, { &P15919,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ru-ru; GT-P3100 Build/JDQ39) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 GoogleGoggles-Android/1.9; gzip*/
const Device UA2937242 = {S62840, 132, { &P15919,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3100 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2995854 = {S62841, 131, { &P15919,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3100 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA64085 = {S62842, 47, { &P12112,&P17484,&P5001,&P18092 }};	/*Useragent - SCH-U450/1.0 NetFront/3.0.22.2.20 (GUI) MMP/2.0*/
const Device UA2042736 = {S62843, 77, { &P15364,&P17285,&P20755,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:15.0) Gecko/X Firefox/15.0.1 SeaMonkey/2.12.1*/
const Device UA2362750 = {S62844, 154, { &P18466,&P18066,&P19803,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2406659 = {S62845, 138, { &P14877,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9000 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2422724 = {S62846, 136, { &P14877,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; GT-I9000 Build/JOP40C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2536597 = {S62847, 137, { &P14877,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9000 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2885351 = {S62848, 155, { &P18680,&P18066,&P23483,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA925189 = {S62849, 90, { &P17976,&P18111,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1.1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00*/
const Device UA2746598 = {S62850, 90, { &P17976,&P18111,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1.2; Linux; Opera Tablet/ADY-X; U; ru) Presto/2.10.254 Version/12.00*/
const Device UA49485 = {S62851, 157, { &P12974,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; ALCATEL_one_touch_890 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA49484 = {S62852, 157, { &P12974,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; ALCATEL one touch 890 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2022453 = {S62853, 79, { &P12974,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; ALCATEL_one_touch_890 Build/ERE27)*/
const Device UA50301 = {S62854, 140, { &P13222,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Ally Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA89885 = {S62855, 145, { &P13222,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Ally Build/ERE27) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA381335 = {S62856, 125, { &P15364,&P17285,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/17.0.963.46) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.46 Safari/535.11*/
const Device UA401292 = {S62857, 132, { &P15364,&P17285,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/17.0.963.56) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11*/
const Device UA401296 = {S62858, 88, { &P15364,&P17285,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/17.0.963.56)*/
const Device UA428588 = {S62859, 125, { &P15364,&P17285,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/17.0.963.79) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11*/
const Device UA428595 = {S62860, 132, { &P15364,&P17285,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/17.0.963.79) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11*/
const Device UA437942 = {S62861, 95, { &P15364,&P17285,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/17.0.963.56)*/
const Device UA458308 = {S62862, 125, { &P15364,&P17285,&P21904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/17.0.963.83) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.83 Safari/535.11*/
const Device UA590590 = {S62863, 165, { &P15364,&P17285,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/17.0.963.56; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)*/
const Device UA603075 = {S62864, 181, { &P15364,&P17285,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/17.0.963.83; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E)*/
const Device UA2221748 = {S62865, 176, { &P15364,&P17285,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SV1; GTB7.2; chromeframe/17.0.963.46; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.30618; .NET CLR 3.5.30729)*/
const Device UA2334911 = {S62866, 184, { &P15364,&P17285,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/4.0; GTB7.2; chromeframe/17.0.963.46; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.30618; .NET CLR 3.5.30729)*/
const Device UA3069695 = {S62867, 206, { &P15364,&P17285,&P21904,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.2; chromeframe/17.0.963.46; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; InfoPath.2; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA655560 = {S62868, 80, { &P11929,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5230)/UC Browser8.2.0.132/50/355/UCWEB Mobile*/
const Device UA685818 = {S62869, 31, { &P11929,&P5129,&P24667,&P18092 }};	/*Useragent - Nokia5230/UCWEB8.2.0.116/50/800*/
const Device UA2445294 = {S62870, 82, { &P11929,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5230-c)/UC Browser8.2.0.132/50/355/UCWEB Mobile*/
const Device UA2566890 = {S62871, 138, { &P18300,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T769 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2678366 = {S62872, 141, { &P12280,&P18117,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 UCBrowser/9.0.1.284 Mobile*/
const Device UA2767998 = {S62873, 141, { &P12280,&P18117,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 UCBrowser/9.0.1.284 Mobile*/
const Device UA2864383 = {S62874, 139, { &P12280,&P18117,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 UCBrowser/9.0.1.284 Mobile*/
const Device UA2995338 = {S62875, 139, { &P12280,&P18117,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 UCBrowser/9.0.1.284 Mobile*/
const Device UA3132628 = {S62876, 136, { &P22585,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L720 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA6367 = {S62877, 94, { &P12450,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG-SCH-i220*/
const Device UA32024 = {S62878, 95, { &P12450,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG-SCH-i220a*/
const Device UA2348949 = {S62879, 68, { &P17595,&P18578,&P21570,&P18092 }};	/*Useragent - Mail/1283 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)*/
const Device UA2384500 = {S62880, 63, { &P17595,&P18578,&P21570,&P18092 }};	/*Useragent - Mail/1283 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (iMac12%2C1)*/
const Device UA2502380 = {S62881, 68, { &P17595,&P18578,&P21570,&P18092 }};	/*Useragent - Mail/1283 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookAir4%2C2)*/
const Device UA3085475 = {S62882, 65, { &P17595,&P18578,&P21570,&P18092 }};	/*Useragent - Mail/1283 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBook5%2C1)*/
const Device UA661348 = {S62883, 134, { &P15910,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; pt-br; XT317 Build/V4.33B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA833127 = {S62884, 134, { &P15910,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; pt-br; XT317 Build/V4.36C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2463886 = {S62885, 134, { &P15910,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT317 Build/V4.36A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2167285 = {S62886, 141, { &P20452,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Lenovo S880 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2355338 = {S62887, 41, { &P23054,&P17484,&P3625,&P18092 }};	/*Useragent - sam-r270 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA2779062 = {S62888, 43, { &P23054,&P17484,&P3625,&P18092 }};	/*Useragent - sam-r270v1 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA630272 = {S62889, 135, { &P18173,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; ISW11F Build/FGK500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1940174 = {S62890, 59, { &P18173,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; ISW11F Build/FGK500)*/
const Device UA2567315 = {S62891, 143, { &P15438,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567513 = {S62892, 149, { &P15438,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710a Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567568 = {S62893, 139, { &P15438,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC-Z710a Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2584087 = {S62894, 143, { &P15438,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA122964 = {S62895, 62, { &P18254,&P7127,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; fr_crawler)*/
const Device UA160098 = {S62896, 63, { &P18254,&P7127,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; MSIECrawler)*/
const Device UA529939 = {S62897, 82, { &P18254,&P7127,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; .NET CLR 1.1.4322; MSIECrawler)*/
const Device UA651397 = {S62898, 102, { &P18254,&P7127,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; MSIECrawler)*/
const Device UA721683 = {S62899, 83, { &P18254,&P7127,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; .NET CLR 2.0.50727; MSIECrawler)*/
const Device UA3078094 = {S62900, 79, { &P18254,&P7127,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; fr_crawler);c2VvLWNyYXdsZXI=*/
const Device UA3078118 = {S62901, 114, { &P18254,&P7127,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; FunWebProducts; Seekmo 11.0.96.0; MSIECrawler);c2VvLWNyYXdsZXI=*/
const Device UA477698 = {S62902, 74, { &P15364,&P18146,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; SunOS i86pc; en-US; rv:1.9.2.2) Gecko/X Firefox/3.6.2*/
const Device UA771009 = {S62903, 139, { &P20720,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; AN7G2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023305 = {S62904, 63, { &P20720,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.1; AN7G2 Build/GINGERBREAD)*/
const Device UA789322 = {S62905, 142, { &P21702,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; DATAM752HCX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1957189 = {S62906, 144, { &P21702,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; DATAM752HCX_A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026822 = {S62907, 159, { &P21702,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3 mod by Anninaone; it-it; DATAM752HCX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2199132 = {S62908, 132, { &P21702,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; M-MP715I Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2672809 = {S62909, 80, { &P15364,&P7133,&P22317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:21.0) Gecko/X Firefox/21.0 SeaMonkey/2.18*/
const Device UA2975067 = {S62910, 145, { &P18512,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT881 Build/9.8.3O-118_USC-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974446 = {S62911, 138, { &P24044,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R740C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA63162 = {S62912, 129, { &P12856,&P17483,&P3459,&P18092 }};	/*Useragent - LG-T300-Orange/V10c Obigo/Q7.3 MMS/LG-MMS-V1.1/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA64653 = {S62913, 122, { &P12856,&P17483,&P3459,&P18092 }};	/*Useragent - LG-T300/V100 Obigo/Q7.3 MMS/LG-MMS-V1.1/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA64792 = {S62914, 79, { &P12474,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-T459/T459UVIA1 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA90288 = {S62915, 140, { &P17993,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; IS03 Build/SB060) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA90543 = {S62916, 140, { &P17993,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; IS03 Build/S2080) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA169442 = {S62917, 140, { &P17993,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; IS03 Build/SC240) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA200785 = {S62918, 140, { &P17993,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; IS03 Build/S5180) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA250468 = {S62919, 140, { &P17993,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; IS03 Build/S9211) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA632073 = {S62920, 140, { &P17993,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; IS03 Build/S4120) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018531 = {S62921, 62, { &P17993,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; IS03 Build/S2080)*/
const Device UA172567 = {S62922, 134, { &P15364,&P5690,&P23729,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko)  QtWeb Internet Browser/3.7 http://www.QtWeb.net*/
const Device UA196399 = {S62923, 136, { &P15364,&P5690,&P23729,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko)  QtWeb Internet Browser/3.7.3 http://www.QtWeb.net*/
const Device UA672151 = {S62924, 136, { &P15364,&P5690,&P23729,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.3 (KHTML, like Gecko)  QtWeb Internet Browser/3.7.3 http://www.QtWeb.net*/
const Device UA821604 = {S62925, 136, { &P15364,&P5690,&P23729,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko)  QtWeb Internet Browser/3.7.5 http://www.QtWeb.net*/
const Device UA859537 = {S62926, 136, { &P15364,&P5690,&P23729,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko)  QtWeb Internet Browser/3.7.2 http://www.QtWeb.net*/
const Device UA2315227 = {S62927, 136, { &P15364,&P5690,&P23729,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/533.3 (KHTML, like Gecko)  QtWeb Internet Browser/3.7.5 http://www.QtWeb.net*/
const Device UA2355097 = {S62928, 136, { &P14186,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S5830 Build/CCGH1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2505226 = {S62929, 141, { &P14186,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-S5830 Build/REVOLUTiON) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2599126 = {S62930, 143, { &P14186,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; GT-S5830C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2790219 = {S62931, 139, { &P14186,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-S5830 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3118256 = {S62932, 60, { &P14186,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 4.1.2; GT-S5830 Build/CCGH1)*/
const Device UA3033886 = {S62933, 155, { &P18680,&P18066,&P24334,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA360307 = {S62934, 143, { &P21426,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; STM722HCZ Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA574540 = {S62935, 142, { &P21426,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; STM722HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA650344 = {S62936, 142, { &P21426,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; STM722HZ Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA662903 = {S62937, 136, { &P17307,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HS-E910 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027741 = {S62938, 133, { &P17307,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; E910 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2743471 = {S62939, 129, { &P17307,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-E910 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA760202 = {S62940, 62, { &P15364,&P23114,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.4) Gecko/X Firefox/10.0.4*/
const Device UA775139 = {S62941, 62, { &P15364,&P23114,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.2) Gecko/X Firefox/10.0.2*/
const Device UA1965235 = {S62942, 62, { &P15364,&P23114,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.5) Gecko/X Firefox/10.0.5*/
const Device UA1988351 = {S62943, 58, { &P15364,&P23114,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0) Gecko/X Firefox/10.0*/
const Device UA2175687 = {S62944, 62, { &P15364,&P23114,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.6) Gecko/X Firefox/10.0.6*/
const Device UA2197304 = {S62945, 62, { &P15364,&P23114,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.7) Gecko/X Firefox/10.0.7*/
const Device UA2354988 = {S62946, 62, { &P15364,&P23114,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.9) Gecko/X Firefox/10.0.9*/
const Device UA2403447 = {S62947, 64, { &P15364,&P23114,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.10) Gecko/X Firefox/10.0.10*/
const Device UA2406279 = {S62948, 64, { &P15364,&P23114,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.11) Gecko/X Firefox/10.0.11*/
const Device UA2489704 = {S62949, 64, { &P15364,&P23114,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.12) Gecko/X Firefox/10.0.12*/
const Device UA2413756 = {S62950, 135, { &P20213,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N9500 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2526982 = {S62951, 102, { &P19765,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; SCH-I930)*/
const Device UA2974712 = {S62952, 137, { &P19870,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-L710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA90790 = {S62953, 70, { &P15775,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 800;7.10;7720)*/
const Device UA371358 = {S62954, 70, { &P15775,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 800;7.10;8107)*/
const Device UA399154 = {S62955, 70, { &P15775,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 800;7.10;7740)*/
const Device UA925766 = {S62956, 70, { &P15775,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 800;7.10;8773)*/
const Device UA925923 = {S62957, 70, { &P15775,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 800;7.10;8112)*/
const Device UA2652419 = {S62958, 70, { &P15775,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 800;7.10;8779)*/
const Device UA2952669 = {S62959, 70, { &P15775,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 800;7.10;8783)*/
const Device UA2353321 = {S62960, 136, { &P23376,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M9000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2652904 = {S62961, 58, { &P23376,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; M9000 Build/IMM76D)*/
const Device UA2508779 = {S62962, 91, { &P12581,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia500) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2983750 = {S62963, 141, { &P18592,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One VX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3004643 = {S62964, 149, { &P18592,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PM36100/2.21.502.1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3071741 = {S62965, 63, { &P18592,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC One VX Build/JZO54K)*/
const Device UA3080462 = {S62966, 133, { &P19907,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C5155 Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081080 = {S62967, 133, { &P19907,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Rise Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2483430 = {S62968, 135, { &P20884,&P5629,&P15760,&P18092 }};	/*Useragent - LENOVO-Lenovo-A298t/1.0 Linux/2.6.35.7 Android 2.3.5 Release/10.30.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2527712 = {S62969, 143, { &P20884,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A298t Build/MocorDroid2.3.5) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2553995 = {S62970, 143, { &P20884,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A298t Build MocorDroid2.3.5) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2578713 = {S62971, 150, { &P20884,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A298t Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2595526 = {S62972, 135, { &P20884,&P5629,&P15760,&P18092 }};	/*Useragent - LENOVO-Lenovo-A298t/1.0 Linux/2.6.35.7 Android 2.3.5 Release/10.22.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2599056 = {S62973, 135, { &P20884,&P5629,&P15760,&P18092 }};	/*Useragent - LENOVO-Lenovo-A298t/1.0 Linux/2.6.35.7 Android 2.3.5 Release/12.13.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2658118 = {S62974, 135, { &P20884,&P5629,&P15760,&P18092 }};	/*Useragent - LENOVO-Lenovo-A298t/1.0 Linux/2.6.35.7 Android 2.3.5 Release/11.22.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2934364 = {S62975, 74, { &P20884,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Lenovo A298t Build/MocorDroid2.3.5)*/
const Device UA2983189 = {S62976, 135, { &P20884,&P5629,&P15760,&P18092 }};	/*Useragent - LENOVO-Lenovo-A298t/1.0 Linux/2.6.35.7 Android 2.3.5 Release/10.11.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2999409 = {S62977, 131, { &P20884,&P5629,&P15760,&P18092 }};	/*Useragent - Lenovo A298t_TD/1.0 Linux/2.6.35.7 Android 2.3.5 Release/04.07.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2676294 = {S62978, 138, { &P23905,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100P Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676434 = {S62979, 138, { &P23905,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100P Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2783165 = {S62980, 136, { &P19985,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SH-02E Build/S4260) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2938062 = {S62981, 58, { &P19985,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SH-02E Build/S4260)*/
const Device UA2944646 = {S62982, 136, { &P19985,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SH-02E Build/S6210) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA90810 = {S62983, 69, { &P15767,&P17146,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.9A334 (iPad; U; CPU OS 5_0 like Mac OS X; en_us)*/
const Device UA622912 = {S62984, 71, { &P15767,&P17146,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.9A405 (iPad; U; CPU OS 5_0_1 like Mac OS X; en_us)*/
const Device UA358437 = {S62985, 159, { &P17262,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; md-us; Kindle Fire Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1950158 = {S62986, 173, { &P17262,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; md-us; Kindle Fire Build/GRK39F; CyanogenMod-HellFireKindle_V1.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982227 = {S62987, 162, { &P17262,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Amazon Kindle Fire Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2201736 = {S62988, 76, { &P15364,&P5690,&P22364,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:15.0) Gecko/X Firefox/15.1.1 PaleMoon/15.1.1*/
const Device UA2235193 = {S62989, 96, { &P15364,&P5690,&P22364,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:15.0) Gecko/X Firefox/15.1.1-x64 PaleMoon/15.1.1-x64*/
const Device UA2505696 = {S62990, 136, { &P21815,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; P-02E Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2748377 = {S62991, 58, { &P21815,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; P-02E Build/JZO54K)*/
const Device UA2567082 = {S62992, 138, { &P19894,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R830 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2670892 = {S62993, 138, { &P18448,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST26i Build/11.0.A.7.5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676632 = {S62994, 138, { &P18448,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST26a Build/11.0.A.7.5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673062 = {S62995, 141, { &P15802,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Amaze 4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2674240 = {S62996, 137, { &P15802,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Ruby Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676190 = {S62997, 141, { &P15802,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC_Amaze_4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2702011 = {S62998, 103, { &P13552,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac6-00) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2716273 = {S62999, 94, { &P13552,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC6-00) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2988790 = {S63000, 32, { &P13552,&P5129,&P24676,&P18092 }};	/*Useragent - NokiaC6-00/UCWEB8.9.0.253/50/999*/
const Device UA2716278 = {S63001, 92, { &P13977,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaN73) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2748190 = {S63002, 147, { &P23230,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Huawei-H867G Build/HuaweiH867G) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2876818 = {S63003, 105, { &P18436,&P16110,&P21920,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; chromeframe/27.0.1453.116)*/
const Device UA48711 = {S63004, 158, { &P18229,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; Alcatel one touch 908F Orange Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA669893 = {S63005, 158, { &P18229,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; Alcatel_one_touch_908F_Orange Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA792742 = {S63006, 158, { &P18229,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; ALCATEL_one_touch_908F_Orange Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031146 = {S63007, 151, { &P18229,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; pt-pt; ALCATEL_one_touch_908F Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2642196 = {S63008, 151, { &P18229,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; Alcatel_one_touch_908F Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA174673 = {S63009, 132, { &P15364,&P5690,&P22447,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Iron/9.0.600.2 Chrome/9.0.600.2 Safari/534.13*/
const Device UA909402 = {S63010, 140, { &P11747,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1967796 = {S63011, 142, { &P11747,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire V Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2235124 = {S63012, 143, { &P11747,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire VC Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2830505 = {S63013, 149, { &P11747,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire VC T328d Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2864678 = {S63014, 142, { &P11747,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire P Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2991586 = {S63015, 142, { &P11747,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire Q Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3091843 = {S63016, 144, { &P11747,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC_Desire_200 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2203260 = {S63017, 118, { &P11945,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; gt-s5233s) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/84/352/UCWEB Mobile*/
const Device UA2419912 = {S63018, 115, { &P11945,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; vi; gt-s5233s) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2431956 = {S63019, 150, { &P18639,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 10 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2560764 = {S63020, 149, { &P18639,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 10 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2599071 = {S63021, 150, { &P18639,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 10 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2967299 = {S63022, 157, { &P18639,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 10 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1.1/manta*/
const Device UA2975013 = {S63023, 133, { &P20129,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MI 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022367 = {S63024, 134, { &P20129,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MI 2A Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3067595 = {S63025, 136, { &P12788,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PG86100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111241 = {S63026, 136, { &P12788,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ISW12HT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA190755 = {S63027, 137, { &P15364,&P17817,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Ubuntu/10.10 Chromium/11.0.696.71 Chrome/11.0.696.71 Safari/534.24*/
const Device UA317823 = {S63028, 135, { &P15364,&P17817,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.24 (KHTML, like Gecko) Ubuntu/10.10 Chromium/11.0.696.57 Chrome/11.0.696.57 Safari/534.24*/
const Device UA2274567 = {S63029, 145, { &P15364,&P17817,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.18 (KHTML, like Gecko) Ubuntu/10.10 Chromium/11.0.661.0 Chrome/11.0.661.0 Safari/534.18*/
const Device UA2275899 = {S63030, 143, { &P15364,&P17817,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.21 (KHTML, like Gecko) Ubuntu/10.10 Chromium/11.0.681.0 Chrome/11.0.681.0 Safari/534.21*/
const Device UA2276638 = {S63031, 136, { &P15364,&P17817,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686) AppleWebKit/534.21 (KHTML, like Gecko) Ubuntu/10.10 Chromium/11.0.682.0 Chrome/11.0.682.0 Safari/534.21*/
const Device UA659812 = {S63032, 123, { &P11679,&P5776,&P24663,&P18092 }};	/*Useragent - Nokia6630/1.0 (5.03.08) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.9.1.120/27/351/UCWEB*/
const Device UA1982621 = {S63033, 142, { &P18267,&P5043,&P24670,&P18092 }};	/*Useragent - NokiaX2-02/2.0 (11.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiax2-02) UCBrowser8.4.0.159/70/352/UCWEB Mobile*/
const Device UA2362675 = {S63034, 34, { &P15767,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.0.1 (2; 16GB; dt:74)*/
const Device UA2385885 = {S63035, 34, { &P15767,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.0.1 (2; 32GB; dt:74)*/
const Device UA2385884 = {S63036, 34, { &P15767,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.0.1 (4; 32GB; dt:77)*/
const Device UA2385948 = {S63037, 34, { &P15767,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.0.1 (4; 16GB; dt:77)*/
const Device UA2456507 = {S63038, 34, { &P15767,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.0.1 (5; 16GB; dt:83)*/
const Device UA2456528 = {S63039, 34, { &P15767,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.0.1 (4; 64GB; dt:77)*/
const Device UA2481354 = {S63040, 34, { &P15767,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.0.1 (2; 64GB; dt:74)*/
const Device UA2485171 = {S63041, 34, { &P15767,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.0.2 (5; 16GB; dt:86)*/
const Device UA2527379 = {S63042, 34, { &P15767,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.0.1 (5; 32GB; dt:83)*/
const Device UA2595805 = {S63043, 34, { &P15767,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.0.2 (5; 32GB; dt:86)*/
const Device UA2654171 = {S63044, 34, { &P15767,&P18432,&P18333,&P18092 }};	/*Useragent - iTunes-iPad/6.0.1 (5; 64GB; dt:83)*/
const Device UA2535307 = {S63045, 128, { &P21638,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; K701 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2779621 = {S63046, 57, { &P21638,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; K701 Build/JRO03H)*/
const Device UA2994665 = {S63047, 138, { &P21638,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Teraware K701H Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2970826 = {S63048, 131, { &P17346,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Mobile/11A4414e Twitter for iPhone*/
const Device UA3030101 = {S63049, 133, { &P17346,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4435d Twitter for iPhone*/
const Device UA3033711 = {S63050, 133, { &P17346,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449a Twitter for iPhone*/
const Device UA3036128 = {S63051, 133, { &P17346,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d Twitter for iPhone*/
const Device UA3094661 = {S63052, 131, { &P17346,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Twitter for iPhone*/
const Device UA3115389 = {S63053, 133, { &P17346,&P22882,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Twitter for iPhone*/
const Device UA32474 = {S63054, 163, { &P12487,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia5700/6.01 (SymbianOS/9.2; U; Series60/3.1 Nokia5700/6.01; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) Mozilla/5.0 AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA51191 = {S63055, 148, { &P12487,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.2 Nokia5700/1.00; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA64371 = {S63056, 150, { &P12487,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Nokia5700/3.83.1; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA2603800 = {S63057, 141, { &P21542,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6503 Build/10.1.A.1.434) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2604177 = {S63058, 140, { &P21909,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-03E Build/10.1.E.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2716917 = {S63059, 140, { &P21909,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-03E Build/10.1.E.0.250) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2755007 = {S63060, 140, { &P21909,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-03E Build/10.1.E.0.265) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2886139 = {S63061, 140, { &P21909,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-03E Build/10.1.E.0.269) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3002467 = {S63062, 136, { &P21909,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-03E Build/10.1.E.0.269) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3077641 = {S63063, 65, { &P21909,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SO-03E Build/10.1.E.0.269)*/
const Device UA3095024 = {S63064, 140, { &P21909,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-03E Build/10.1.E.0.305) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2457220 = {S63065, 130, { &P20835,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 754TPC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2844782 = {S63066, 59, { &P20835,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; 754TPC Build/IMM76D)*/
const Device UA2676989 = {S63067, 140, { &P23910,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30at Build/9.1.F.1.120) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708109 = {S63068, 133, { &P23910,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30at Build/9.1.F.1.120) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2885339 = {S63069, 138, { &P23496,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D2-927G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3126185 = {S63070, 131, { &P23496,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D2-927G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2434151 = {S63071, 138, { &P20963,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM200SH Build/S0004) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2510306 = {S63072, 138, { &P20963,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM200SH Build/S0016) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2674139 = {S63073, 138, { &P20963,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM200SH Build/S0028) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2717027 = {S63074, 60, { &P20963,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SBM200SH Build/S0028)*/
const Device UA2772909 = {S63075, 156, { &P21648,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799155 = {S63076, 139, { &P22969,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT915 Build/2_32A_2008) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2893550 = {S63077, 160, { &P14862,&P5204,&P24678,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2700c-2) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3020334 = {S63078, 156, { &P14862,&P5204,&P24678,&P18092 }};	/*Useragent - Nokia2700c/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2700c) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA440176 = {S63079, 130, { &P13960,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; V9 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA450883 = {S63080, 130, { &P13960,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; V9 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA567284 = {S63081, 130, { &P13960,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; V9 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA722273 = {S63082, 130, { &P13960,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; V9 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021837 = {S63083, 54, { &P13960,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; V9 Build/GRJ22)*/
const Device UA2032905 = {S63084, 135, { &P13960,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ThL V9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2683629 = {S63085, 134, { &P13960,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; RTK V9 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2722459 = {S63086, 136, { &P13960,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ko-kr; V9 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2810077 = {S63087, 130, { &P13960,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; V9 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2259999 = {S63088, 87, { &P15364,&P17285,&P21955,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9a5pre) Gecko/X Minefield/3.0a5pre*/
const Device UA2290035 = {S63089, 93, { &P15364,&P17285,&P21955,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0 x64; en-US; rv:1.9.0.3pre) Gecko/X Minefield/3.0.5pre*/
const Device UA1999116 = {S63090, 123, { &P15364,&P16110,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1*/
const Device UA2016858 = {S63091, 116, { &P15364,&P16110,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1*/
const Device UA2257295 = {S63092, 131, { &P21593,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CMP745E Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2581987 = {S63093, 153, { &P18113,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT912 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2738732 = {S63094, 156, { &P18113,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT912 Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2711365 = {S63095, 156, { &P14635,&P5843,&P24677,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac3-00) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA217093 = {S63096, 133, { &P17308,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROIDX Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA332898 = {S63097, 133, { &P17308,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA379932 = {S63098, 133, { &P17308,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROIDX Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023582 = {S63099, 57, { &P17308,&P24694,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; DROIDX Build/MIUI)*/
const Device UA609203 = {S63100, 157, { &P12896,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899333 = {S63101, 176, { &P12896,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/IMM76I; CyanogenMod-9.0.0; Owains Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1953763 = {S63102, 157, { &P12896,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1962515 = {S63103, 156, { &P12896,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/GRJ22; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1966422 = {S63104, 176, { &P12896,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/IMM76L; CyanogenMod-9.0.0; Owains Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2360555 = {S63105, 156, { &P12896,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/GRI40; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2548188 = {S63106, 149, { &P12896,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/GRI40; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA741970 = {S63107, 62, { &P15837,&P6909,&P4037,&P18092 }};	/*Useragent - NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2589576 = {S63108, 73, { &P15837,&P6909,&P4037,&P18092 }};	/*Useragent - NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-05*/
const Device UA2181300 = {S63109, 144, { &P15869,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xoom Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2258022 = {S63110, 144, { &P15869,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xoom Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2387399 = {S63111, 148, { &P15869,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xoom Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2565378 = {S63112, 137, { &P21829,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; F-02E Build/V16R46A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2713017 = {S63113, 137, { &P21829,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; F-02E Build/V17R48A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997090 = {S63114, 59, { &P21829,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; F-02E Build/V17R48A)*/
const Device UA2885319 = {S63115, 114, { &P13035,&P5043,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-02/06.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA2975528 = {S63116, 138, { &P21283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976023 = {S63117, 137, { &P21283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021507 = {S63118, 138, { &P21283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023133 = {S63119, 138, { &P21283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2039621 = {S63120, 99, { &P17595,&P18120,&P20806,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.2 Safari/534.34*/
const Device UA2589933 = {S63121, 107, { &P17595,&P18120,&P20806,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.8.0 Safari/533.3*/
const Device UA2422446 = {S63122, 130, { &P20153,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN7HG3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2844909 = {S63123, 59, { &P20153,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; AN7HG3 Build/IML74K)*/
const Device UA2560997 = {S63124, 158, { &P18442,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T889 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2660448 = {S63125, 158, { &P18442,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2671442 = {S63126, 158, { &P18442,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2672861 = {S63127, 158, { &P18442,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L900 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2705495 = {S63128, 158, { &P18442,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7105 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2708680 = {S63129, 166, { &P18442,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2714147 = {S63130, 158, { &P18442,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T889 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2850842 = {S63131, 158, { &P18442,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L900 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA3073448 = {S63132, 162, { &P18442,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA20833 = {S63133, 105, { &P13866,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-A847M/A847BMJK4 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA32952 = {S63134, 105, { &P13866,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-A847R/A847UXJK4 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA33019 = {S63135, 105, { &P13866,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-A847D/A847TLJK4 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA41019 = {S63136, 104, { &P13866,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-A847/A847UCJK2 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA481270 = {S63137, 109, { &P17595,&P18120,&P22503,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en; rv:1.8.1.14) Gecko/X Camino/1.6.1 (like Firefox/2.0.0.14)*/
const Device UA518264 = {S63138, 102, { &P17595,&P18120,&P22503,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en; rv:1.8.1.14) Gecko/X Camino/1.6 (like Firefox/2.0.0.14)*/
const Device UA665316 = {S63139, 88, { &P17595,&P18120,&P22503,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en; rv:1.8.1.4pre) Gecko/X Camino/1.6pre*/
const Device UA877486 = {S63140, 155, { &P21033,&P18066,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U1 Build/HCL ME Tablet U1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2588654 = {S63141, 153, { &P21033,&P18066,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 4.0.3; en-; U1 Build/HCL ME Tablet U1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2242269 = {S63142, 142, { &P21809,&P18066,&P18062,&P18092 }};	/*Useragent - HUAWEI_T8620_TD/1.0 Android/4.0.3 (Linux; U; Android 4.0.3; zh-cn) Release/05.31.2012 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30*/
const Device UA2376070 = {S63143, 132, { &P21809,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; T8620 Build/HuaweiT8620) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2455546 = {S63144, 141, { &P21809,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T8620 Build/HuaweiT8620) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2835316 = {S63145, 63, { &P21809,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; T8620 Build/HuaweiT8620)*/
const Device UA2674001 = {S63146, 74, { &P15364,&P17285,&P22370,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.0.1*/
const Device UA2680593 = {S63147, 90, { &P15364,&P17285,&P22370,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:20.0) Gecko/X Firefox/20.0 PaleMoon/20.0.1-x64*/
const Device UA2754065 = {S63148, 168, { &P15794,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315b Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755527 = {S63149, 168, { &P15794,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755133 = {S63150, 126, { &P19775,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A210 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2975001 = {S63151, 130, { &P15882,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5110 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA1629 = {S63152, 73, { &P14880,&P5033,&P3417,&P18092 }};	/*Useragent - BlackBerry8320/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100*/
const Device UA72242 = {S63153, 135, { &P14880,&P5033,&P3417,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320) BlackBerry8320/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100*/
const Device UA2530749 = {S63154, 166, { &P16292,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JOP40G; Carbon-Carbon-JB-v1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2570362 = {S63155, 165, { &P16292,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39; Carbon-Carbon-JB-v1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2672759 = {S63156, 165, { &P16292,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39; Carbon-CARBON-JB-v1.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2713605 = {S63157, 166, { &P16292,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA12695 = {S63158, 141, { &P11747,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; ADR6200 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA29101 = {S63159, 140, { &P11747,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; ADR6200Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA77849 = {S63160, 130, { &P22485,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; T3 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1962601 = {S63161, 133, { &P22485,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Android 2.3.5;en-us;TECNO T3 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021415 = {S63162, 140, { &P22485,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; Fantastic T3 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026419 = {S63163, 130, { &P22485,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; in-id; T3 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2575309 = {S63164, 135, { &P22485,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SATC T3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2587865 = {S63165, 54, { &P22485,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; T3 Build/GRJ22)*/
const Device UA3042431 = {S63166, 134, { &P22485,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Hosin T3 Build/MocorDroid2.3.5) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA191496 = {S63167, 101, { &P15364,&P7133,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.1.7600.0);en-US;kml:2.2;client:Free;type:default)*/
const Device UA203968 = {S63168, 122, { &P15364,&P7133,&P18209,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.0.3.2197 Safari/532.4*/
const Device UA204290 = {S63169, 101, { &P15364,&P7133,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.1.7601.1);en-US;kml:2.2;client:Free;type:default)*/
const Device UA299011 = {S63170, 101, { &P15364,&P7133,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.1.2032(Windows;Microsoft Windows (6.1.7600.0);en-US;kml:2.2;client:Free;type:default)*/
const Device UA321588 = {S63171, 101, { &P15364,&P7133,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.1.7600.0);de-DE;kml:2.2;client:Free;type:default)*/
const Device UA322128 = {S63172, 101, { &P15364,&P7133,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.1.7601.1);en-GB;kml:2.2;client:Free;type:default)*/
const Device UA322150 = {S63173, 101, { &P15364,&P7133,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.1.7601.1);de-DE;kml:2.2;client:Free;type:default)*/
const Device UA446526 = {S63174, 101, { &P15364,&P7133,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.1.2032(Windows;Microsoft Windows (6.1.7601.1);en-US;kml:2.2;client:Free;type:default)*/
const Device UA865160 = {S63175, 101, { &P15364,&P7133,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.2.2074(Windows;Microsoft Windows (6.1.7601.1);en-US;kml:2.2;client:Free;type:default)*/
const Device UA880776 = {S63176, 101, { &P15364,&P7133,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.2.2074(Windows;Microsoft Windows (6.1.7600.0);en-US;kml:2.2;client:Free;type:default)*/
const Device UA1886756 = {S63177, 101, { &P15364,&P7133,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.1.7601.1);en-CA;kml:2.2;client:Free;type:default)*/
const Device UA2211204 = {S63178, 101, { &P15364,&P7133,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.1.7601.1);en-AU;kml:2.2;client:Free;type:default)*/
const Device UA2439353 = {S63179, 100, { &P15364,&P7133,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.1.7601.1);en-US;kml:2.2;client:Pro;type:default)*/
const Device UA2355808 = {S63180, 133, { &P17691,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957R Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2366117 = {S63181, 133, { &P17691,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957M Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2420789 = {S63182, 133, { &P17691,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2430836 = {S63183, 140, { &P17691,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I957 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2577699 = {S63184, 132, { &P17691,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2405960 = {S63185, 134, { &P20987,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; M-MP102S2B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422546 = {S63186, 134, { &P20987,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; M-MP102S2W Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2486883 = {S63187, 140, { &P17595,&P17296,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3) AppleWebKit/534.31 (KHTML, like Gecko) Chrome/17.0.558.0 Safari/534.31 UCBrowser/2.2.2.259*/
const Device UA2755543 = {S63188, 145, { &P12657,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515m Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2770319 = {S63189, 145, { &P12657,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877104 = {S63190, 138, { &P12657,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC_X515E Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA12460 = {S63191, 136, { &P15759,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Vision Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA15021 = {S63192, 139, { &P15759,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Vision Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA33281 = {S63193, 136, { &P15759,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; HTC Vision Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55666 = {S63194, 138, { &P15759,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Vision Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57929 = {S63195, 138, { &P15759,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Vision Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58215 = {S63196, 139, { &P15759,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; HTC Vision Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA293381 = {S63197, 138, { &P15759,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Vision Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA294938 = {S63198, 60, { &P15759,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; HTC Vision Build/FRF91)*/
const Device UA304726 = {S63199, 63, { &P15759,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; HTC Vision Build/FRG83D)*/
const Device UA2018329 = {S63200, 137, { &P15759,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Vision Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA556063 = {S63201, 49, { &P18254,&P18110,&P18245,&P19831 }};	/*Useragent - WIRE/0.22 (Linux; i686; Bot,Robot,Spider,Crawler)*/
const Device UA688521 = {S63202, 51, { &P18254,&P18110,&P18245,&P19831 }};	/*Useragent - WIRE/0.22 (Linux; x86_64; Bot,Robot,Spider,Crawler)*/
const Device UA2009051 = {S63203, 139, { &P17595,&P17297,&P22376,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.19 (KHTML, like Gecko) Maxthon/1.0.3.1000 Chrome/18.0.1025.142 Safari/535.19*/
const Device UA2197912 = {S63204, 136, { &P17595,&P17297,&P22376,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/1.0.4.1000 Chrome/21.0.1180.75 Safari/537.1*/
const Device UA2234844 = {S63205, 136, { &P17595,&P17297,&P22376,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/1.0.4.2000 Chrome/21.0.1180.75 Safari/537.1*/
const Device UA2455147 = {S63206, 136, { &P17595,&P17297,&P22376,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/1.0.4.2000 Chrome/21.0.1180.75 Safari/537.1*/
const Device UA2261548 = {S63207, 144, { &P19855,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT885 Build/6.7.3-179_YIC-2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2202576 = {S63208, 81, { &P17968,&P18111,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1.1; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2507078 = {S63209, 81, { &P17968,&P18111,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1.2; Linux; Opera Mobi/ADY-X) Presto/2.11.355 Version/12.10*/
const Device UA2531156 = {S63210, 81, { &P17968,&P18111,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1.9; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2557429 = {S63211, 81, { &P17968,&P18111,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1.5; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2580419 = {S63212, 79, { &P17968,&P18111,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2425384 = {S63213, 103, { &P18285,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.50.0.28; en-us; 9220/9.50.0.28) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2456743 = {S63214, 103, { &P18285,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.50.0.50; en-us; 9220/9.50.0.50) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2481206 = {S63215, 103, { &P18285,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.50.0.36; en-us; 9220/9.50.0.36) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2578356 = {S63216, 103, { &P18285,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.50.0.80; en-us; 9220/9.50.0.80) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2694726 = {S63217, 103, { &P18285,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.50.0.79; en-us; 9220/9.50.0.79) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3049217 = {S63218, 103, { &P18285,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.50.0.99; en-us; 9220/9.50.0.99) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA675148 = {S63219, 81, { &P14912,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-C3222/C3222DDLD4 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA899439 = {S63220, 115, { &P15364,&P17285,&P22458,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5*/
const Device UA899533 = {S63221, 122, { &P15364,&P17285,&P22458,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5*/
const Device UA2195895 = {S63222, 142, { &P18183,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ615-12) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2232432 = {S63223, 138, { &P18641,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-M830 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2566646 = {S63224, 137, { &P18687,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P705 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567706 = {S63225, 138, { &P18687,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P705g Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583230 = {S63226, 138, { &P18687,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P708g Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2601159 = {S63227, 138, { &P18687,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P705f Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA61897 = {S63228, 61, { &P14050,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia3108/1.0 (03.01) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA64487 = {S63229, 98, { &P12679,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-GT-S3310/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA2258206 = {S63230, 91, { &P17595,&P17295,&P20761,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:16.0) Gecko/X Firefox/16.0 SeaMonkey/2.13.2*/
const Device UA2676756 = {S63231, 139, { &P15972,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One XL Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677007 = {S63232, 132, { &P15972,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One XL Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2754243 = {S63233, 134, { &P21636,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C525c Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3068416 = {S63234, 151, { &P20017,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083824 = {S63235, 150, { &P20017,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA715547 = {S63236, 140, { &P14854,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  SHW-M250S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA906026 = {S63237, 139, { &P14854,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SHW-M250S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1960314 = {S63238, 139, { &P14854,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SHW-M250K Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1992255 = {S63239, 139, { &P14854,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SHW-M250L Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2028240 = {S63240, 139, { &P14854,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2229451 = {S63241, 139, { &P14854,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250K Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2231086 = {S63242, 139, { &P14854,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2471632 = {S63243, 154, { &P18465,&P18066,&P19805,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.6 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2566917 = {S63244, 131, { &P17323,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6800 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2934752 = {S63245, 138, { &P23293,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LS720 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA64280 = {S63246, 83, { &P14698,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW880i/R8BA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2234296 = {S63247, 136, { &P16001,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-06D Build/S9030) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2385542 = {S63248, 140, { &P16001,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-06DNERV Build/S9061) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2563777 = {S63249, 136, { &P16001,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-06D Build/S2060) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2601085 = {S63250, 140, { &P16001,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-06DNERV Build/S2060) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2777539 = {S63251, 58, { &P16001,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SH-06D Build/S9030)*/
const Device UA3128378 = {S63252, 136, { &P16001,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-06D Build/S9090) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2403530 = {S63253, 130, { &P15767,&P18117,&P20779,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/2.8 Mobile/9B206 Safari/7534.48.3*/
const Device UA2427254 = {S63254, 128, { &P15767,&P18117,&P20779,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/2.8 Mobile/9B176 Safari/7534.48.3*/
const Device UA26393 = {S63255, 145, { &P13591,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; ZTE-BLADE Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA40048 = {S63256, 135, { &P13591,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-gb; ZTE-BLADE Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018587 = {S63257, 67, { &P13591,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; ZTE-BLADE Build/ERE27)*/
const Device UA12984 = {S63258, 146, { &P14011,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; U8500 HiQQ Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA53703 = {S63259, 141, { &P14011,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; U8500 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA53705 = {S63260, 154, { &P14011,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-tw; HUAWEI IDEOS U8500 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018408 = {S63261, 63, { &P14011,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; U8500 Build/ERE27)*/
const Device UA2019567 = {S63262, 127, { &P14011,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; U8500 Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1955691 = {S63263, 132, { &P15985,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2608297 = {S63264, 132, { &P15985,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P6200 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2621577 = {S63265, 132, { &P15985,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Safari/535.19*/
const Device UA2679925 = {S63266, 85, { &P15364,&P16110,&P22560,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:20.0) Gecko/X Firefox/20.0 IceDragon/20.0.1.14*/
const Device UA2967701 = {S63267, 92, { &P13977,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaN73) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA2973648 = {S63268, 128, { &P23702,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ME173X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA909645 = {S63269, 137, { &P18658,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; ISW16SH Build/S6151) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1995411 = {S63270, 137, { &P18658,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; ISW16SH Build/S7060) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2004627 = {S63271, 137, { &P18658,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; ISW16SH Build/S7260) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2017652 = {S63272, 137, { &P18658,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; ISW16SH Build/S8100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2200626 = {S63273, 137, { &P18658,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; ISW16SH Build/S9181) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2511046 = {S63274, 137, { &P18658,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; ISW16SH Build/S1150) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2585335 = {S63275, 137, { &P18658,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; ISW16SH Build/S2271) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2756647 = {S63276, 137, { &P18658,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; ISW16SH Build/S5082) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121287 = {S63277, 59, { &P18658,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ISW16SH Build/S5082)*/
const Device UA1939721 = {S63278, 62, { &P15364,&P16110,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; U; en) Presto/2.6.30 Version/10.63*/
const Device UA2147026 = {S63279, 62, { &P15364,&P16110,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; U; en) Presto/2.6.30 Version/10.60*/
const Device UA2409097 = {S63280, 62, { &P15364,&P16110,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; U; en) Presto/2.6.30 Version/10.61*/
const Device UA2412561 = {S63281, 62, { &P15364,&P16110,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; U; en) Presto/2.6.30 Version/10.62*/
const Device UA2362653 = {S63282, 178, { &P18219,&P5843,&P24672,&P18092 }};	/*Useragent - Nokia202/2.0 (20.28) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia202) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA141167 = {S63283, 60, { &P18254,&P6729,&P17384,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; MSIECrawler; Windows 95)*/
const Device UA435557 = {S63284, 59, { &P18254,&P6729,&P17384,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.0; MSIECrawler; Windows 95)*/
const Device UA2204877 = {S63285, 114, { &P14669,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-00/08.25; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2426097 = {S63286, 136, { &P20098,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; GN800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2652916 = {S63287, 58, { &P20098,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GN800 Build/IMM76D)*/
const Device UA2695807 = {S63288, 143, { &P20098,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; GIONEE_GN800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2745452 = {S63289, 127, { &P20098,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN800 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2934601 = {S63290, 142, { &P20098,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GPLUS_GN800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2962518 = {S63291, 58, { &P15364,&P17188,&P22171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0; rv:23.0) Gecko/X Firefox/23.0*/
const Device UA2387556 = {S63292, 137, { &P20357,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; SmartPad7503G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2485545 = {S63293, 133, { &P22707,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Flytouch9 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2703135 = {S63294, 134, { &P22707,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Flytouch 9 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2769319 = {S63295, 96, { &P15364,&P7133,&P21781,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.21 (KHTML, like Gecko) QupZilla/1.4.3 Safari/537.21*/
const Device UA2782525 = {S63296, 103, { &P15364,&P7133,&P21781,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.21 (KHTML, like Gecko) QupZilla/1.4.3 Safari/537.21*/
const Device UA3065251 = {S63297, 130, { &P22873,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5120 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3066534 = {S63298, 134, { &P23700,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; N9101 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA90556 = {S63299, 132, { &P18011,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS05 Build/S2251) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA221041 = {S63300, 132, { &P18011,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS05 Build/S7210) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA246126 = {S63301, 132, { &P18011,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS05 Build/S9140) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA319909 = {S63302, 132, { &P18011,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS05 Build/S5160) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1902405 = {S63303, 130, { &P18011,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-; IS05 Build/S9140) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023588 = {S63304, 56, { &P18011,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; IS05 Build/S9140)*/
const Device UA142711 = {S63305, 47, { &P15364,&P22310,&P4618,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.0; Windows 3.1)*/
const Device UA433201 = {S63306, 48, { &P15364,&P22310,&P4618,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.02; Windows 3.1)*/
const Device UA1842639 = {S63307, 48, { &P15364,&P22310,&P4618,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.03; Windows 3.1)*/
const Device UA2658946 = {S63308, 56, { &P15364,&P22310,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; Win3.11; Trident/4.0)*/
const Device UA2232040 = {S63309, 129, { &P21179,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MOMO8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3043313 = {S63310, 58, { &P21179,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; MOMO8 Build/JRO03H)*/
const Device UA2490755 = {S63311, 145, { &P20605,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Luna TAB07-100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2850683 = {S63312, 67, { &P20605,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Luna TAB07-100 Build/JRO03C)*/
const Device UA2676828 = {S63313, 136, { &P12788,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ISW12HT Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676979 = {S63314, 136, { &P12788,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PG86100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2715021 = {S63315, 132, { &P15364,&P18110,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; en-us; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/533.1 (KHTML,like Gecko) Version/6.0 Safari/533.1*/
const Device UA2715028 = {S63316, 134, { &P15364,&P18110,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Linux2.6; en-us; TSB_CLOUD_COMPANION;FOLIO_AND_A) AppleWebKit/530.17 (KHTML,like Gecko) Version/6.0 Safari/530.17*/
const Device UA2715038 = {S63317, 94, { &P15364,&P18110,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/6.0 Safari/530.17*/
const Device UA2257276 = {S63318, 129, { &P18491,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TABEO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2407122 = {S63319, 128, { &P15869,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Xoom Build/JB_MR1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2535468 = {S63320, 127, { &P15869,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Xoom Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2602807 = {S63321, 131, { &P21824,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID7046 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2890972 = {S63322, 138, { &P23760,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LG870 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA62860 = {S63323, 71, { &P13977,&P5890,&P4030,&P18092 }};	/*Useragent - NokiaN73-1/2.0628.0.0.1 S60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA69513 = {S63324, 65, { &P13977,&P5890,&P4030,&P18092 }};	/*Useragent - NokiaN73-2/2.0626 S60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA81107 = {S63325, 8, { &P13977,&P5890,&P4030,&P18092 }};	/*Useragent - NokiaN73*/
const Device UA2030110 = {S63326, 140, { &P23855,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2382589 = {S63327, 134, { &P23855,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2668211 = {S63328, 139, { &P22952,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F260S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2954597 = {S63329, 61, { &P22952,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F260S Build/JZO54K)*/
const Device UA2974733 = {S63330, 149, { &P18318,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021435 = {S63331, 150, { &P18318,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700KL Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA210660 = {S63332, 137, { &P17717,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-SU760 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019936 = {S63333, 61, { &P17717,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-SU760 Build/FRG83G)*/
const Device UA2020547 = {S63334, 135, { &P17717,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ru-; LG-SU760 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2877969 = {S63335, 61, { &P23215,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SHV-E330S Build/JDQ39)*/
const Device UA3036919 = {S63336, 172, { &P23215,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ko-kr; SHV-E330S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 NAVER(inapp; search; 230; 4.7.7)*/
const Device UA3067069 = {S63337, 137, { &P20013,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F180L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3082806 = {S63338, 137, { &P20013,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F180K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083193 = {S63339, 136, { &P20013,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F180 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083448 = {S63340, 137, { &P20013,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F180S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA44284 = {S63341, 85, { &P13817,&P5448,&P4578,&P18092 }};	/*Useragent - MOT-MPx220/1.430 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 176x220)*/
const Device UA67349 = {S63342, 76, { &P13817,&P5448,&P4578,&P18092 }};	/*Useragent - MOT-MPx220/1.400 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone;*/
const Device UA251932 = {S63343, 136, { &P16036,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus One Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA329851 = {S63344, 136, { &P16036,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Nexus One Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA774999 = {S63345, 136, { &P16036,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Nexus One Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021297 = {S63346, 60, { &P16036,&P24694,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; Nexus One Build/MIUI)*/
const Device UA2032258 = {S63347, 124, { &P16036,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Nexus One Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2193406 = {S63348, 142, { &P20285,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ616-12) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2428662 = {S63349, 147, { &P20285,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ616 Build/7.7.1-141-6-PAST-UMTS-LA) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2229006 = {S63350, 137, { &P20332,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax P275 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2982837 = {S63351, 66, { &P20332,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Micromax P275 Build/IMM76D)*/
const Device UA2490562 = {S63352, 156, { &P12280,&P19826,&P21789,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1.2 (like Version/10B143 Mobile/6.1 Safari/8536.25)*/
const Device UA2533479 = {S63353, 160, { &P12280,&P19826,&P21789,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1.2 (like Version/10B146 Mobile/6.1.2 Safari/8536.25)*/
const Device UA2537154 = {S63354, 160, { &P12280,&P19826,&P21789,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1.2 (like Version/10B145 Mobile/6.1.1 Safari/8536.25)*/
const Device UA2602465 = {S63355, 160, { &P12280,&P19826,&P21789,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1.2 (like Version/10B329 Mobile/6.1.3 Safari/8536.25)*/
const Device UA2710767 = {S63356, 160, { &P12280,&P19826,&P21789,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1.2 (like Version/10B350 Mobile/6.1.4 Safari/8536.25)*/
const Device UA2975881 = {S63357, 93, { &P13123,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5360) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3057589 = {S63358, 90, { &P13123,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; vi; GT-S5360) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA2005128 = {S63359, 155, { &P12814,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I777 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2012037 = {S63360, 155, { &P12814,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I777 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2201260 = {S63361, 155, { &P12814,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I777 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2238225 = {S63362, 155, { &P12814,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I777 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2564441 = {S63363, 139, { &P23073,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-uk; @TAB-TEN.1-X2-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3068192 = {S63364, 140, { &P23073,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; @TAB-TEN.1W-X2-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2669594 = {S63365, 137, { &P22936,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D2-912 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2670619 = {S63366, 137, { &P18458,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I200 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713669 = {S63367, 130, { &P18458,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I200 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3068044 = {S63368, 137, { &P18682,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-M950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2676605 = {S63369, 151, { &P18552,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT897 Build/7.7.1Q-6_SPR-125_ASA-14) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709408 = {S63370, 144, { &P18552,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT897 Build/7.7.1Q-6_SPR-125_ASA-14) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA53454 = {S63371, 111, { &P12260,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S5350/S5350DDJA3 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-N*/
const Device UA2407155 = {S63372, 86, { &P17968,&P17979,&P18526,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/6.7.29878/28.3234; U; zh) Presto/2.8.119 Version/11.10*/
const Device UA2362227 = {S63373, 125, { &P15655,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s3850) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2673371 = {S63374, 141, { &P13124,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP9070 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2994955 = {S63375, 134, { &P13124,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP9070 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA910925 = {S63376, 131, { &P21049,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GEM7012 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2180558 = {S63377, 137, { &P21330,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PMP5197DULTRA Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2726429 = {S63378, 66, { &P21330,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; PMP5197DULTRA Build/IML74K)*/
const Device UA3065600 = {S63379, 154, { &P22856,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0151 Build/HuaweiY300-0151) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA633306 = {S63380, 71, { &P15364,&P16110,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; U; en) Presto/2.10.229 Version/11.62*/
const Device UA3118245 = {S63381, 94, { &P15364,&P16110,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; U; MRA 5.10 (build 5300); ru) Presto/2.10.229 Version/11.62*/
const Device UA2734828 = {S63382, 136, { &P21065,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; CINK+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867138 = {S63383, 140, { &P21065,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; CINK FIVE Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3009609 = {S63384, 62, { &P21065,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; CINK FIVE Build/JZO54K)*/
const Device UA3088172 = {S63385, 58, { &P21065,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; CINK+ Build/JRO03C)*/
const Device UA807535 = {S63386, 138, { &P18709,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PMP7074B3G Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1980869 = {S63387, 140, { &P18709,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ru-ru; PMP7074B3GRU Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3080910 = {S63388, 137, { &P22610,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083081 = {S63389, 138, { &P22610,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083247 = {S63390, 138, { &P22610,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094614 = {S63391, 138, { &P22610,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067175 = {S63392, 129, { &P18320,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3067656 = {S63393, 136, { &P18320,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA587047 = {S63394, 83, { &P15364,&P18118,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:9.0.1) Gecko/X Firefox/9.0.1 Firefox/9.0.1*/
const Device UA639979 = {S63395, 67, { &P15364,&P18118,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Ubuntu; X11; Linux x86_64; rv:9.0) Gecko/X Firefox/9.0*/
const Device UA643631 = {S63396, 65, { &P15364,&P18118,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:9.0) Gecko/X Firefox/9.0*/
const Device UA645787 = {S63397, 108, { &P15364,&P18118,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Ubuntu; X11; Linux x86_64; rv:9.0.1) Gecko/X Firefox/9.0.1,gzip(gfe) (via translate.google.com)*/
const Device UA674723 = {S63398, 106, { &P15364,&P18118,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:9.0.1) Gecko/X Firefox/9.0.1,gzip(gfe) (via translate.google.com)*/
const Device UA700305 = {S63399, 79, { &P15364,&P18118,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Ubuntu; X11; Linux i686 on x86_64; rv:9.0.1) Gecko/X Firefox/9.0.1*/
const Device UA717124 = {S63400, 70, { &P15364,&P18118,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:9.0.1) Gecko/X Abrowser/9.0.1*/
const Device UA1941244 = {S63401, 81, { &P15364,&P18118,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:9.0.1) Gecko/X Firefox/9.0.1 FirePHP/0.6*/
const Device UA2302900 = {S63402, 69, { &P15364,&P18118,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:9.0a2) Gecko/X Firefox/9.0a2*/
const Device UA2594339 = {S63403, 79, { &P15364,&P18118,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:9.0.1) Gecko/X Firefox/9.0.1,gzip(gfe)*/
const Device UA1966394 = {S63404, 131, { &P19785,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; CT1002PLUS Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2496357 = {S63405, 136, { &P19785,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; CT1002 Build/ICS.b02ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2981138 = {S63406, 134, { &P19785,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; CT1002 Build/ICS.b02ref) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2941528 = {S63407, 129, { &P23955,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5210 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA62527 = {S63408, 134, { &P14211,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SPH-P100 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA63257 = {S63409, 136, { &P14211,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SPH-P100 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90440 = {S63410, 143, { &P14211,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SPH-P100 Build/FROYO) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA392309 = {S63411, 60, { &P14211,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SPH-P100 Build/FROYO)*/
const Device UA2018137 = {S63412, 58, { &P14211,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; SPH-P100 Build/FROYO)*/
const Device UA26212 = {S63413, 76, { &P13574,&P5298,&P4808,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; BREW 4.0.3; en )/800x480 Samsung sam-r900*/
const Device UA64895 = {S63414, 41, { &P14715,&P17484,&P3625,&P18092 }};	/*Useragent - sam-r250 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA2567145 = {S63415, 139, { &P19878,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I497 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2755416 = {S63416, 136, { &P23776,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGP321 Build/10.1.1.A.1.253) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2900008 = {S63417, 156, { &P14669,&P5843,&P24678,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-00) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA363786 = {S63418, 136, { &P18479,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS11N Build/01.02.00) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627070 = {S63419, 136, { &P18479,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS11N Build/01.04.20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA629559 = {S63420, 136, { &P18479,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS11N Build/01.03.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1967566 = {S63421, 136, { &P18479,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS11N Build/01.06.30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031436 = {S63422, 136, { &P18479,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS11N Build/01.05.00) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2532462 = {S63423, 136, { &P18479,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS11N Build/01.07.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2892977 = {S63424, 136, { &P18479,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS11N Build/01.08.00) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA634092 = {S63425, 71, { &P18670,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X335/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2598972 = {S63426, 72, { &P18670,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X335C/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2659337 = {S63427, 126, { &P21052,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A9 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2844272 = {S63428, 133, { &P21052,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; A9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2896307 = {S63429, 133, { &P21052,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A9 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985984 = {S63430, 55, { &P21052,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; A9 Build/JRO03C)*/
const Device UA2197297 = {S63431, 131, { &P20389,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3;en-au; ZTE T12 Build/FRF91)AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2477651 = {S63432, 135, { &P20389,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-au; ZTE T12 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2563106 = {S63433, 152, { &P19747,&P5043,&P24676,&P18092 }};	/*Useragent - Nokia206/2.0 (03.59) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia206) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2859989 = {S63434, 156, { &P19747,&P5043,&P24676,&P18092 }};	/*Useragent - Nokia206.1/2.0 (03.58) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia206.1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2753162 = {S63435, 138, { &P20076,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26w Build/6.1.A.2.45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754267 = {S63436, 130, { &P17168,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7310 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2974621 = {S63437, 137, { &P18459,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS950 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA577 = {S63438, 135, { &P11840,&P5338,&P3772,&P18092 }};	/*Useragent - MOT-MOTORAZRV9/A0.01.10R10 BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003266) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [en]*/
const Device UA623 = {S63439, 133, { &P11840,&P5338,&P3772,&P18092 }};	/*Useragent - MOT-MOTORAZRV9/A0.03.18R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003291) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [en]*/
const Device UA10343 = {S63440, 133, { &P11840,&P5338,&P3772,&P18092 }};	/*Useragent - MOT-MOTORAZRV9x/9E.03.3AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003352) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.60 [en]*/
const Device UA10347 = {S63441, 134, { &P11840,&P5338,&P3772,&P18092 }};	/*Useragent - MOT-MOTORAZRV9x/9E.03.28R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003348) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [en]*/
const Device UA15765 = {S63442, 134, { &P11840,&P5338,&P3772,&P18092 }};	/*Useragent - MOT-MOTORAZRV9/A0.05.20R1 BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003348) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [en]*/
const Device UA15815 = {S63443, 133, { &P11840,&P5338,&P3772,&P18092 }};	/*Useragent - MOT-MOTORAZRV9/A0.03.2ER BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003321) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [en]*/
const Device UA15827 = {S63444, 133, { &P11840,&P5338,&P3772,&P18092 }};	/*Useragent - MOT-MOTORAZRV9/A0.02.0ER BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13223296) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [en]*/
const Device UA54184 = {S63445, 134, { &P11840,&P5338,&P3772,&P18092 }};	/*Useragent - MOT-MOTORAZRV9/A0.02.07R2 BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003275) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [en]*/
const Device UA442768 = {S63446, 127, { &P11840,&P5338,&P3772,&P18092 }};	/*Useragent - MOT-MOTORAZRV9x/9E.03.39R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; X) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [en]*/
const Device UA783107 = {S63447, 127, { &P11840,&P5338,&P3772,&P18092 }};	/*Useragent - MOT-MOTORAZRV9/A0.05.20R1 BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; X) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [en]*/
const Device UA924479 = {S63448, 72, { &P15364,&P5690,&P22361,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:12.3) Gecko/X Firefox/12.3 PaleMoon/12.3*/
const Device UA1995092 = {S63449, 92, { &P15364,&P5690,&P22361,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:12.3) Gecko/X Firefox/12.3-x64 PaleMoon/12.3-x64*/
const Device UA2167019 = {S63450, 79, { &P15364,&P5690,&P22361,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:12.3) Gecko/X Firefox/12.3 PaleMoon/12.3*/
const Device UA2028103 = {S63451, 141, { &P15366,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Tattoo Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2230003 = {S63452, 154, { &P18466,&P18066,&P18690,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.2 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2734774 = {S63453, 132, { &P22918,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DA220HQL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3102865 = {S63454, 61, { &P22918,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; DA220HQL Build/IMM76D)*/
const Device UA362692 = {S63455, 113, { &P15364,&P17285,&P22454,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7*/
const Device UA397474 = {S63456, 120, { &P15364,&P17285,&P22454,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7*/
const Device UA2367340 = {S63457, 69, { &P13974,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; GT-S5670; 240*320) UCWEB7.9.0.94/139/352*/
const Device UA2383572 = {S63458, 69, { &P13974,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.1; zh-cn; GT-S5670; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2428301 = {S63459, 155, { &P14540,&P5843,&P24675,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiax2-01) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2735604 = {S63460, 136, { &P21935,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E980 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2999717 = {S63461, 129, { &P21935,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E980 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3031422 = {S63462, 152, { &P19747,&P5043,&P24679,&P18092 }};	/*Useragent - Nokia206/2.0 (03.59) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia206) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3059122 = {S63463, 156, { &P19747,&P5043,&P24679,&P18092 }};	/*Useragent - Nokia206.1/2.0 (03.58) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia206.1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3067646 = {S63464, 137, { &P18300,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T769 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA108962 = {S63465, 117, { &P17595,&P17294,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; fr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA826202 = {S63466, 71, { &P15364,&P18145,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; FreeBSD i386; en-US; rv:1.9.1) Gecko/X Firefox/3.5*/
const Device UA775908 = {S63467, 100, { &P15364,&P16110,&P22331,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.2.16) Gecko/X Firefox/3.6.16 CometBird/3.6.16*/
const Device UA2737858 = {S63468, 157, { &P15575,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9100G Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2773969 = {S63469, 156, { &P15575,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2972871 = {S63470, 124, { &P17595,&P18120,&P22779,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Google Earth/7.1.1.1888 Safari/533.3*/
const Device UA3067609 = {S63471, 137, { &P19870,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-L710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2353974 = {S63472, 78, { &P15364,&P18140,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; Edition Linux Mint) Presto/2.12.388 Version/12.10*/
const Device UA2461081 = {S63473, 129, { &P20216,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M1062 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2772903 = {S63474, 157, { &P18595,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2783699 = {S63475, 157, { &P18595,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2974660 = {S63476, 145, { &P18614,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT890 Build/8.7.1I-110_SML-20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA251934 = {S63477, 151, { &P17308,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROIDX Build/VZW; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2975703 = {S63478, 136, { &P19911,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E970 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA12119 = {S63479, 47, { &P11731,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-S1300/ UP.Browser/7.2.6.1.839 (GUI) MMP/2.0*/
const Device UA30867 = {S63480, 51, { &P11731,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-S1300/1007 UP.Browser/7.2.6.1.840 (GUI) MMP/2.0*/
const Device UA44032 = {S63481, 51, { &P11731,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-S1300/1002 UP.Browser/7.2.6.1.840 (GUI) MMP/2.0*/
const Device UA64763 = {S63482, 51, { &P11731,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-S1300/1001 UP.Browser/7.2.6.1.841 (GUI) MMP/2.0*/
const Device UA66299 = {S63483, 47, { &P11731,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-S1300/ UP.Browser/7.2.6.1.838 (GUI) MMP/2.0*/
const Device UA102201 = {S63484, 51, { &P11731,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-S1300/1009 UP.Browser/7.2.6.1.839 (GUI) MMP/2.0*/
const Device UA272320 = {S63485, 52, { &P11731,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-S1300/10081 UP.Browser/7.2.6.1.839 (GUI) MMP/2.0*/
const Device UA286123 = {S63486, 51, { &P11731,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-S1300/1007 UP.Browser/7.2.6.1.839 (GUI) MMP/2.0*/
const Device UA56761 = {S63487, 134, { &P13880,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; T-01C Build/FFR002) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA781930 = {S63488, 58, { &P13880,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; T-01C Build/FFR002)*/
const Device UA640729 = {S63489, 118, { &P17595,&P17295,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) WebClip/5534.49 Safari/5534.49*/
const Device UA658793 = {S63490, 111, { &P17595,&P17295,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_7) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50*/
const Device UA2258175 = {S63491, 112, { &P17595,&P17295,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Cruz/0.4 Safari/534.50.2*/
const Device UA2332578 = {S63492, 124, { &P17595,&P17295,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/534.50.2 (KHTML, like Gecko) Fake/1.7.2 Safari/534.50.2*/
const Device UA2643747 = {S63493, 126, { &P17595,&P17295,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en) AppleWebKit/534.54.16 (KHTML, like Gecko) Version/5.1.4 Safari/534.54.16*/
const Device UA2889795 = {S63494, 114, { &P17595,&P17295,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5) AppleWebKit/537.13  (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2*/
const Device UA781197 = {S63495, 142, { &P21129,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i929 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023774 = {S63496, 130, { &P21129,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i929 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2033926 = {S63497, 135, { &P21129,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i929 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2223387 = {S63498, 135, { &P21129,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i929 Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2643585 = {S63499, 66, { &P21129,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-i929 Build/GINGERBREAD)*/
const Device UA1987204 = {S63500, 135, { &P18314,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M032 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026922 = {S63501, 126, { &P18314,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; M032 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2374797 = {S63502, 57, { &P18314,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; M032 Build/IML74K)*/
const Device UA2015814 = {S63503, 132, { &P17337,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P7500 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2236727 = {S63504, 132, { &P17337,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7500 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2627149 = {S63505, 139, { &P17337,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7500M Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2028009 = {S63506, 133, { &P21094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A789 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2376075 = {S63507, 142, { &P21094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A789 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2415210 = {S63508, 64, { &P21094,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo A789 Build/IMM76D)*/
const Device UA2420645 = {S63509, 142, { &P21094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LENOVO A789 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2526776 = {S63510, 140, { &P21094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; Lenovo A789; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2592074 = {S63511, 149, { &P21094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; LENOVO A789-orange Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2777462 = {S63512, 149, { &P21094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lenovo A789-orange Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2983631 = {S63513, 142, { &P21094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Lenovo A789 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029015 = {S63514, 148, { &P17337,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7500 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2031364 = {S63515, 148, { &P17337,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7500 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2362228 = {S63516, 148, { &P17337,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7500 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2632920 = {S63517, 146, { &P17337,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-; GT-P7500 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2556800 = {S63518, 79, { &P15364,&P21460,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.3; Trident/7.0; MSAppHost/1.0)*/
const Device UA2935881 = {S63519, 71, { &P15364,&P21460,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.3; WOW64; Trident/6.0)*/
const Device UA2944086 = {S63520, 79, { &P15364,&P21460,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (MSIE 10.0; Windows NT 6.3; WOW64; Trident/7.0; rv:11.0) like Gecko*/
const Device UA3077038 = {S63521, 84, { &P15364,&P21460,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (MSIE 10.0; Windows NT 6.3; Win64; x64; Trident/7.0; rv:11.0) like Gecko*/
const Device UA3105994 = {S63522, 76, { &P15364,&P21460,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.3; Win64; x64; Trident/6.0)*/
const Device UA2568681 = {S63523, 43, { &P15364,&P18570,&P22901,&P18092 }};	/*Useragent - Mercury/7.3 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA27278 = {S63524, 111, { &P12273,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-M5650/M5650XIJA1 SHP/VPP/R5 Jasmine/1.0 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA29861 = {S63525, 105, { &P12273,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-M5650U/1.0 SHP/VPP/R5 Jasmine/1.0 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA1996421 = {S63526, 138, { &P19789,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; KM-S200 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257814 = {S63527, 138, { &P19789,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; KM-S220 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2686704 = {S63528, 60, { &P19789,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; KM-S200 Build/IMM76I)*/
const Device UA2596490 = {S63529, 147, { &P23137,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I407 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3086214 = {S63530, 69, { &P23137,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-I407 Build/JZO54K)*/
const Device UA2665806 = {S63531, 57, { &P15364,&P17017,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (masking-agent; rv:20.0) Gecko/X Firefox/20.0*/
const Device UA2769912 = {S63532, 141, { &P18451,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26ii Build/6.2.B.0.200) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA24470 = {S63533, 104, { &P15690,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-T559/T559UVIFA SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA183420 = {S63534, 76, { &P15364,&P7480,&P3409,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Thumbshots.ru; +http://thumbshots.ru/bot) Firefox/3*/
const Device UA566808 = {S63535, 77, { &P15364,&P22283,&P4392,&P18092 }};	/*Useragent - Mozilla/4.0 (Brew MP 1.0.2; U; en-us; Kyocera; NetFront/4.1/AMB) Sprint E4255*/
const Device UA829883 = {S63536, 77, { &P15364,&P22283,&P4392,&P18092 }};	/*Useragent - Mozilla/4.0 (Brew MP 1.0.2; U; en-us; Kyocera; NetFront/4.1/AMB) Sprint E4210*/
const Device UA2967277 = {S63537, 94, { &P13324,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaN8-00) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3066542 = {S63538, 135, { &P18468,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabA2109A Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA1981937 = {S63539, 80, { &P11929,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5230)/UC Browser8.5.0.183/50/355/UCWEB Mobile*/
const Device UA1993880 = {S63540, 31, { &P11929,&P5129,&P24671,&P18092 }};	/*Useragent - Nokia5230/UCWEB8.5.0.163/50/999*/
const Device UA2540029 = {S63541, 133, { &P18464,&P18110,&P21167,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Safari/535.19 Silk-Accelerated=true*/
const Device UA2672704 = {S63542, 136, { &P12972,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T989 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677169 = {S63543, 144, { &P12972,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-T989 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2862143 = {S63544, 137, { &P12972,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-T989 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3064139 = {S63545, 130, { &P20011,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Next7P12 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Safari/537.36*/
const Device UA3068322 = {S63546, 133, { &P20011,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Next7P12-8G Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA458694 = {S63547, 135, { &P18035,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID7127 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458885 = {S63548, 69, { &P18035,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/0.9.3 (Linux; U; Android 2.3.3; en_US; MID7127 Build/MID7127 X)*/
const Device UA807542 = {S63549, 136, { &P20590,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; novo7_Aurora Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA899097 = {S63550, 136, { &P20590,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Novo7 Aurora Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2034195 = {S63551, 65, { &P20590,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; novo7_Aurora Build/IML74K)*/
const Device UA2480040 = {S63552, 135, { &P20590,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Novo7Aurora Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2573441 = {S63553, 157, { &P20590,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; novo7_Aurora Build/IML74K; 1024*552) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2348337 = {S63554, 142, { &P19898,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I699 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2420618 = {S63555, 135, { &P19898,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I699 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2694207 = {S63556, 66, { &P19898,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-I699 Build/GINGERBREAD)*/
const Device UA2839221 = {S63557, 135, { &P19898,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I699 Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2437368 = {S63558, 131, { &P22477,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; VOYAGER Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2512588 = {S63559, 134, { &P22477,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-uk; VOYAGER-BT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2935084 = {S63560, 63, { &P22477,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; VOYAGER-BT Build/JRO03H)*/
const Device UA2782817 = {S63561, 136, { &P22836,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SC-04E Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2798631 = {S63562, 58, { &P22836,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SC-04E Build/JDQ39)*/
const Device UA79885 = {S63563, 89, { &P17595,&P17295,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_5) AppleWebKit (KHTML, like Gecko) Chrome/3*/
const Device UA509520 = {S63564, 122, { &P17595,&P17295,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-US) AppleWebKit/531.0 (KHTML, like Gecko) Chrome/3.0.183 Safari/531.0*/
const Device UA898097 = {S63565, 122, { &P17595,&P17295,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.198 Safari/532.0*/
const Device UA2166264 = {S63566, 122, { &P17595,&P17295,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.198 Safari/532.0*/
const Device UA899580 = {S63567, 85, { &P15655,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s3850) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA1985160 = {S63568, 85, { &P15655,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s3850) UCBrowser8.3.1.161/70/465/UCWEB Mobile*/
const Device UA2230465 = {S63569, 142, { &P20399,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP8010 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2235849 = {S63570, 112, { &P18644,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia112/03.26; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2257939 = {S63571, 130, { &P21352,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAB469 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2569883 = {S63572, 66, { &P17976,&P17979,&P21777,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:19.0) Gecko/19.0 K9Browser/19.0.0*/
const Device UA2772768 = {S63573, 150, { &P18437,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8000 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2784791 = {S63574, 150, { &P18437,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8000 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799524 = {S63575, 150, { &P18437,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8020 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2753069 = {S63576, 135, { &P18468,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; IdeaTabA2109A Build/JRO03R) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2753183 = {S63577, 147, { &P15575,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2957784 = {S63578, 148, { &P15575,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-I9100T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2999520 = {S63579, 148, { &P15575,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; GT-I9100G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2945978 = {S63580, 148, { &P18385,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3059974 = {S63581, 149, { &P18385,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 7 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3084114 = {S63582, 157, { &P18645,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9305 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3096273 = {S63583, 157, { &P18645,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3096281 = {S63584, 157, { &P18645,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3110350 = {S63585, 158, { &P18645,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I747M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3111438 = {S63586, 158, { &P18645,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9305T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3115505 = {S63587, 165, { &P18645,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3115507 = {S63588, 158, { &P18645,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3115558 = {S63589, 165, { &P18645,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3651 = {S63590, 125, { &P13283,&P5208,&P4223,&P18092 }};	/*Useragent - NetFront/3.5.1(BREW 3.1.5; U; en-us; SAMSUNG; NetFront/3.1.5/AMB) Sprint M560 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA1995936 = {S63591, 57, { &P15364,&P17017,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (masking-agent; rv:15.0) Gecko/X Firefox/15.0*/
const Device UA2043291 = {S63592, 59, { &P15364,&P17017,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (masking-agent; rv:15.0) Gecko/X Firefox/15.0.1*/
const Device UA2561585 = {S63593, 146, { &P20694,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; QMobile A11Note Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2663291 = {S63594, 146, { &P20694,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; QMobile_A11Note Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2769436 = {S63595, 138, { &P15364,&P5690,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.31 (KHTML, like Gecko) Chrome/17.0.558.0 Safari/534.31 UCBrowser/9.0.2.299*/
const Device UA3019972 = {S63596, 122, { &P15364,&P17285,&P22779,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 6.0; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/7.1.1.1888 Safari/532.4*/
const Device UA3012171 = {S63597, 58, { &P15364,&P7127,&P22171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:23.0) Gecko/X Firefox/23.0*/
const Device UA569723 = {S63598, 31, { &P12193,&P5129,&P24665,&P18092 }};	/*Useragent - NokiaC5-00/UCWEB8.0.3.99/28/999*/
const Device UA663106 = {S63599, 43, { &P12193,&P5129,&P24665,&P18092 }};	/*Useragent - NokiaC5-00/UC Browser8.0.4.121/28/444/UCWEB*/
const Device UA724070 = {S63600, 45, { &P12193,&P5129,&P24665,&P18092 }};	/*Useragent - NokiaC5-00.2/UC Browser8.0.4.121/28/444/UCWEB*/
const Device UA892394 = {S63601, 131, { &P21052,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2809385 = {S63602, 55, { &P21052,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; A9 Build/GRK39F)*/
const Device UA2716457 = {S63603, 94, { &P11905,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6120c) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2676837 = {S63604, 138, { &P20076,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26w Build/6.1.A.2.45) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2973466 = {S63605, 148, { &P15438,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974249 = {S63606, 142, { &P15438,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023032 = {S63607, 142, { &P15438,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023624 = {S63608, 138, { &P15438,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC-Z710a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA53716 = {S63609, 133, { &P17935,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; PG06100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90519 = {S63610, 140, { &P17935,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; PG06100 Build/FRF91) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA188370 = {S63611, 57, { &P17935,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; PG06100 Build/FRF91)*/
const Device UA1984688 = {S63612, 128, { &P15602,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A501 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2410683 = {S63613, 135, { &P15602,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Iconia A501 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2676671 = {S63614, 143, { &P18009,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTCEVODesign4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676674 = {S63615, 136, { &P18009,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PH44100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2729306 = {S63616, 141, { &P18696,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC6435LRA Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3087210 = {S63617, 63, { &P18696,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC6435LRA Build/JRO03C)*/
const Device UA3100047 = {S63618, 144, { &P18696,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_Droid_DNA Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2755640 = {S63619, 136, { &P18438,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E973 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769709 = {S63620, 137, { &P18438,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F200K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769974 = {S63621, 137, { &P18438,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F200S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783931 = {S63622, 137, { &P18438,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F200L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3024073 = {S63623, 158, { &P15989,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.4.22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3028921 = {S63624, 158, { &P15989,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.6.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3061602 = {S63625, 158, { &P15989,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.0.16) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3061607 = {S63626, 158, { &P15989,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.6.8) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3062722 = {S63627, 158, { &P15989,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.1.8) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3064903 = {S63628, 158, { &P15989,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.4.27) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3026688 = {S63629, 187, { &P18385,&P21637,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-gb; Nexus 7 Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb*/
const Device UA3027952 = {S63630, 187, { &P18385,&P21637,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us*/
const Device UA3065011 = {S63631, 187, { &P18385,&P21637,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us*/
const Device UA3080850 = {S63632, 187, { &P18385,&P21637,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-gb; Nexus 7 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.8/620,2.0.8.620,2013-09-06 15:06, +0100, gb*/
const Device UA3096220 = {S63633, 134, { &P20705,&P21637,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; One Build/JWR66V.H10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.38 Mobile Safari/537.36*/
const Device UA3131473 = {S63634, 134, { &P20705,&P21637,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; HTC One Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132952 = {S63635, 133, { &P20705,&P21637,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; HTCONE Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3067187 = {S63636, 129, { &P24367,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5200 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA24475 = {S63637, 117, { &P13540,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2152i/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA38547 = {S63638, 122, { &P13540,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2152/E2152DDJH1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA1923855 = {S63639, 76, { &P13540,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2152i/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA30837 = {S63640, 47, { &P12355,&P6476,&P3625,&P18092 }};	/*Useragent - LGE-VN250P/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA30838 = {S63641, 47, { &P12355,&P6476,&P3625,&P18092 }};	/*Useragent - LGE-VN250L/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA32883 = {S63642, 133, { &P22838,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; nl-nl; MOVE Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019997 = {S63643, 57, { &P22838,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; MOVE Build/FRG83G)*/
const Device UA105109 = {S63644, 46, { &P15364,&P17017,&P17388,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/2.1.1; X11)*/
const Device UA174800 = {S63645, 34, { &P15364,&P7127,&P4811,&P18092 }};	/*Useragent - Opera/9.00 (Windows NT 5.0; U; en)*/
const Device UA273913 = {S63646, 34, { &P15364,&P7127,&P4811,&P18092 }};	/*Useragent - Opera/9.02 (Windows NT 5.0; U; en)*/
const Device UA522144 = {S63647, 34, { &P15364,&P7127,&P4811,&P18092 }};	/*Useragent - Opera/9.01 (Windows NT 5.0; U; en)*/
const Device UA2210009 = {S63648, 33, { &P15364,&P7127,&P4811,&P18092 }};	/*Useragent - Opera/9.0 (Windows NT 5.0; U; en)*/
const Device UA1994699 = {S63649, 141, { &P17172,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer TF101 Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1997040 = {S63650, 141, { &P17172,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer TF101 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2012718 = {S63651, 141, { &P17172,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer TF101 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2028041 = {S63652, 135, { &P17172,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2231859 = {S63653, 141, { &P17172,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Transformer TF101 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2553518 = {S63654, 64, { &P17172,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Transformer Build/JZO54K)*/
const Device UA2566933 = {S63655, 127, { &P19862,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A110 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2584205 = {S63656, 143, { &P19862,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Micromax A110 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2707192 = {S63657, 127, { &P19862,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A110 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2753612 = {S63658, 137, { &P18682,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-M950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA740022 = {S63659, 59, { &P15364,&P18346,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10; rv:12.0) Gecko/X Firefox/12.0*/
const Device UA867457 = {S63660, 152, { &P12935,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; U8650 Build/benru89; CM7.1.0-benru89) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028600 = {S63661, 160, { &P12935,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Huawei u8650 Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033028 = {S63662, 163, { &P12935,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; U8650 Build/cm7-benru89; CyanogenMod7.2-benru89) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2399957 = {S63663, 160, { &P12935,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; U8650 Build/CyanogenMod 7.2; CyanogenMod 7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2983146 = {S63664, 162, { &P12935,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; U8650 Build/CyanogenMod 2.3.7; CyanogenMod 7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1979520 = {S63665, 142, { &P20170,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A95 Build/CelkonCelkon_A95) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2428491 = {S63666, 153, { &P20170,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Celkon*A95*Pro Build/CelkonCelkon_A95) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2204072 = {S63667, 238, { &P15767,&P18432,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403,Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2221634 = {S63668, 139, { &P15767,&P18432,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2356749 = {S63669, 141, { &P15767,&P18432,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2357345 = {S63670, 240, { &P15767,&P18432,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523,Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2460119 = {S63671, 141, { &P15767,&P18432,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 6_0_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2657719 = {S63672, 140, { &P15767,&P18432,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_0 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2707498 = {S63673, 131, { &P23236,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AMI-TT4 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3102497 = {S63674, 60, { &P23236,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; AMI-TT4 Build/JRO03H)*/
const Device UA61248 = {S63675, 42, { &P20210,&P17483,&P4223,&P18092 }};	/*Useragent - LGE-VP200-PLB NetFront/3.5.1 (GUI) MMP/2.0*/
const Device UA2334757 = {S63676, 147, { &P15767,&P18117,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3 TeamStream HD (iPad2,1 232 1.6.1)*/
const Device UA2404211 = {S63677, 147, { &P15767,&P18117,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 TeamStream HD (iPad1,1 284 1.7)*/
const Device UA1993033 = {S63678, 136, { &P21101,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ZP100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2842885 = {S63679, 136, { &P21101,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZP100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1995685 = {S63680, 130, { &P21805,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SN70T3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2466325 = {S63681, 131, { &P21805,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SN70T31 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA220047 = {S63682, 78, { &P13135,&P5208,&P4521,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Polaris 6.2; Brew 3.1.5; en)/240X320 Samsung sam-r640*/
const Device UA2028752 = {S63683, 139, { &P16757,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I500 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029964 = {S63684, 139, { &P16757,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I500 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2199628 = {S63685, 139, { &P16757,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I500 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2206133 = {S63686, 139, { &P16757,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I500 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258197 = {S63687, 139, { &P16757,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422478 = {S63688, 139, { &P16757,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA56043 = {S63689, 115, { &P13630,&P17483,&P4732,&P18092 }};	/*Useragent - LG-GU295/V10i; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.15957; U; en-US) Opera 9.50*/
const Device UA193492 = {S63690, 115, { &P15364,&P5690,&P22449,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko) Iron/11.0.700.2 Chrome/11.0.700.2 Safari/534.24*/
const Device UA2531150 = {S63691, 144, { &P19781,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; ARCHOS 97 TITANIUMHD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2566927 = {S63692, 151, { &P18471,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT907 Build/7.7.1Q-144_SMQ_S136-10) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567105 = {S63693, 144, { &P18471,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT905 Build/7.7.1Q_GCIRD-16) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2600977 = {S63694, 144, { &P18471,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT907 Build/7.7.1Q-144_SMQ_S136-10) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2666761 = {S63695, 137, { &P18471,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT905 Build/7.7.1Q_GCIRD-16) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2668825 = {S63696, 141, { &P22756,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; C1604 Build/11.1.A.0.68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2703604 = {S63697, 145, { &P22756,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; SonyC1604 Build/11.1.A.0.64) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2795104 = {S63698, 141, { &P22756,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; C1604 Build/11.1.A.0.64) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2863420 = {S63699, 145, { &P22756,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; SonyC1604 Build/11.1.A.0.53) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2754492 = {S63700, 145, { &P18626,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I547 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2770679 = {S63701, 138, { &P18626,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I547C Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2899419 = {S63702, 145, { &P18626,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I547 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2956647 = {S63703, 134, { &P24559,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Z992 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2258564 = {S63704, 120, { &P12088,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaX6-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile*/
const Device UA2383776 = {S63705, 32, { &P12088,&P5129,&P24672,&P18092 }};	/*Useragent - NokiaX6-00/UCWEB8.6.0.180/50/800*/
const Device UA2257240 = {S63706, 135, { &P20365,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ViewPad 10e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1962912 = {S63707, 134, { &P24017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3;it-it; ZTE-U V857 Build/FRF91)AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1994574 = {S63708, 136, { &P24017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3;sr-rs;  ZTE-U V857 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3031326 = {S63709, 157, { &P16356,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2260477 = {S63710, 140, { &P12059,&P5204,&P24670,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia2690) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA1998122 = {S63711, 143, { &P15438,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Sensation Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2237979 = {S63712, 143, { &P15438,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Sensation Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2426986 = {S63713, 139, { &P15438,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Sensation Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2752696 = {S63714, 231, { &P22854,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; zh-cn; SAMSUNG-GT-I9508_TD/1.0 Android/4.2.2 Release/03.15.2013 Browser/AppleWebKit535.19 Build/JDQ39) ApplelWebkit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2768199 = {S63715, 164, { &P22854,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; zh-cn; SAMSUNG GT-I9508 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2755372 = {S63716, 139, { &P18736,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One SV Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA1958222 = {S63717, 128, { &P20556,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; A701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2423517 = {S63718, 135, { &P20556,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2759084 = {S63719, 57, { &P20556,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; A701 Build/IMM76D)*/
const Device UA2033522 = {S63720, 138, { &P21089,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; ZTE T22 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2366753 = {S63721, 135, { &P20184,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; X506 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2457801 = {S63722, 135, { &P20184,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; X506 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2527358 = {S63723, 126, { &P21052,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2560843 = {S63724, 133, { &P21052,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050406 = {S63725, 136, { &P21052,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A9 Build/ICS.g08refema9.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2659206 = {S63726, 169, { &P23199,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad7295 Build/4.1.015.X.7295; 540*960; CUCC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2706662 = {S63727, 133, { &P23199,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad7295 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2795492 = {S63728, 64, { &P23199,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Coolpad7295 Build/JZO54K)*/
const Device UA2868984 = {S63729, 172, { &P23199,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 7295+ Build/4.1.003.X.7295+; 540*960; CUCC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958589 = {S63730, 135, { &P23199,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 7295+ Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3003496 = {S63731, 66, { &P23199,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Coolpad 7295+ Build/JZO54K)*/
const Device UA3058587 = {S63732, 172, { &P23199,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 7295A Build/4.1.003.X.7295A; 480*854; CUCC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3063995 = {S63733, 137, { &P18687,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P705f Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3081181 = {S63734, 136, { &P18687,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P705 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081340 = {S63735, 137, { &P18687,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P708g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083611 = {S63736, 137, { &P18687,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P705g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA437101 = {S63737, 136, { &P15860,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; LG-SU880 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020063 = {S63738, 60, { &P15860,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-SU880 Build/GRJ22)*/
const Device UA2042098 = {S63739, 64, { &P15364,&P17285,&P18523,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:15.0) Gecko/X Thunderbird/15.0.1*/
const Device UA2229034 = {S63740, 138, { &P12989,&P5843,&P24668,&P18092 }};	/*Useragent - Nokia303/5.0 (14.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia303) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2407112 = {S63741, 82, { &P15364,&P16110,&P20457,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:17.0) Gecko/X Firefox/17.0 SeaMonkey/2.14.1*/
const Device UA2585036 = {S63742, 106, { &P15364,&P18145,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2667506 = {S63743, 105, { &P15364,&P18145,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2570303 = {S63744, 158, { &P16356,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2773897 = {S63745, 157, { &P16356,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-N7000 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2774062 = {S63746, 157, { &P16356,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2659497 = {S63747, 136, { &P18480,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N850L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044607 = {S63748, 135, { &P18480,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N850 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA573497 = {S63749, 145, { &P18421,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; vi-vn; FPT Tablet Build/1.0.3900.0211)AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA580135 = {S63750, 148, { &P18421,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; vi-vn; FPT Tablet II Build/1.0.7044.0298)AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622517 = {S63751, 148, { &P18421,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; vi-vn; FPT Tablet II Build/1.0.5259.0261)AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622529 = {S63752, 70, { &P18421,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; FPT Tablet Build/1.0.4846.0245)*/
const Device UA622499 = {S63753, 73, { &P18421,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; FPT Tablet II Build/1.0.5779.0279)*/
const Device UA2597217 = {S63754, 144, { &P18421,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; FPT Tablet Build/1.0.4846.0245)AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2603992 = {S63755, 147, { &P18421,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; FPT Tablet II Build/1.0.5779.0279)AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995948 = {S63756, 136, { &P21009,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS17SH Build/S7070) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1997227 = {S63757, 136, { &P21009,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS17SH Build/S7251) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566302 = {S63758, 136, { &P21009,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS17SH Build/S2200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2970051 = {S63759, 94, { &P14033,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC5-03) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3039240 = {S63760, 155, { &P21261,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9500 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA51226 = {S63761, 109, { &P13293,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH902i (compatible; Y!J-SRD/1.0; http://help.yahoo.co.jp/help/jp/search/indexing/indexing-27.html)*/
const Device UA102229 = {S63762, 37, { &P15364,&P18110,&P4809,&P18092 }};	/*Useragent - Opera/5.0 (Linux 2.0.38 i386; U) [en]*/
const Device UA2297347 = {S63763, 56, { &P15364,&P18110,&P4809,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Linux) Opera 5.0 [en]*/
const Device UA241028 = {S63764, 102, { &P15364,&P17017,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Litl OS i686) AppleWebKit/534.23 (KHTML, like Gecko) Chrome/11.0.689.0 Safari/534.23*/
const Device UA253785 = {S63765, 9, { &P15364,&P17017,&P17250,&P18092 }};	/*Useragent - Chrome/11*/
const Device UA717030 = {S63766, 95, { &P15364,&P17017,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible) AppleWebKit/534.21 (KHTML, like Gecko) Chrome/11.0.682.0 Safari/534.21*/
const Device UA2437295 = {S63767, 142, { &P12280,&P18432,&P20790,&P18092 }};	/*Useragent - MQQBrowser/39 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3*/
const Device UA2437369 = {S63768, 141, { &P12280,&P18432,&P20790,&P18092 }};	/*Useragent - MQQBrowser/39 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari/7534.48.3*/
const Device UA2460565 = {S63769, 140, { &P12280,&P18432,&P20790,&P18092 }};	/*Useragent - MQQBrowser/39 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3*/
const Device UA2462224 = {S63770, 143, { &P12280,&P18432,&P20790,&P18092 }};	/*Useragent - MQQBrowser/391 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3*/
const Device UA2470332 = {S63771, 142, { &P12280,&P18432,&P20790,&P18092 }};	/*Useragent - MQQBrowser/391 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari/7534.48.3*/
const Device UA2484247 = {S63772, 141, { &P12280,&P18432,&P20790,&P18092 }};	/*Useragent - MQQBrowser/39 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3*/
const Device UA2487195 = {S63773, 141, { &P12280,&P18432,&P20790,&P18092 }};	/*Useragent - MQQBrowser/391 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3*/
const Device UA2507599 = {S63774, 142, { &P12280,&P18432,&P20790,&P18092 }};	/*Useragent - MQQBrowser/391 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3*/
const Device UA925921 = {S63775, 70, { &P18218,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 610;7.10;8773)*/
const Device UA2034651 = {S63776, 70, { &P18218,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 610;7.10;8779)*/
const Device UA2597885 = {S63777, 70, { &P18218,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 610;7.10;8858)*/
const Device UA2607480 = {S63778, 70, { &P18218,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 610;7.10;8862)*/
const Device UA2889196 = {S63779, 74, { &P18218,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 610 NFC;7.10;8779)*/
const Device UA2988466 = {S63780, 70, { &P18218,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 610;7.10;8783)*/
const Device UA3001123 = {S63781, 130, { &P24400,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SM-T315 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3058345 = {S63782, 150, { &P24400,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SAMSUNG SM-T315/T315XXUAMG3 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3107847 = {S63783, 59, { &P24400,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SM-T315 Build/JDQ39)*/
const Device UA3068407 = {S63784, 136, { &P14574,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080506 = {S63785, 136, { &P14574,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus S Build/JRO03E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2676443 = {S63786, 138, { &P13141,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I727R Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708419 = {S63787, 138, { &P13141,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E110S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2711141 = {S63788, 131, { &P13141,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I727R Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2882684 = {S63789, 112, { &P18644,&P5043,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia112/03.32; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA2974649 = {S63790, 145, { &P18556,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook HD+ Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2975949 = {S63791, 132, { &P18556,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; BN NookHD+ Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023387 = {S63792, 130, { &P18556,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nook HD+ Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA640830 = {S63793, 116, { &P14708,&P5204,&P22721,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia5130c-2/07.95; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.4.34.8*/
const Device UA2469260 = {S63794, 146, { &P18626,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I547 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2671189 = {S63795, 139, { &P18626,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I547C Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2948547 = {S63796, 146, { &P18626,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I547 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2676689 = {S63797, 137, { &P13991,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6400L Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708970 = {S63798, 140, { &P13991,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ThunderBolt Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2892511 = {S63799, 138, { &P21794,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9152 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2894258 = {S63800, 60, { &P21794,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9152 Build/JDQ39)*/
const Device UA3082855 = {S63801, 152, { &P15877,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3096251 = {S63802, 112, { &P19748,&P5043,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia205/03.18; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA3110700 = {S63803, 114, { &P19748,&P5043,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia205.1/03.18; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA67765 = {S63804, 135, { &P18026,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.0; nl-nl; A7EB Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA68416 = {S63805, 135, { &P18026,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.1; nl-nl; A7EB Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA687577 = {S63806, 135, { &P18026,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.4; en-us; A7EB Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2021825 = {S63807, 148, { &P18026,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.4; fr-fr; A7EB Build/ECLAIR.eng.root.X.X) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA862960 = {S63808, 143, { &P20010,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M340K Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1964922 = {S63809, 139, { &P18469,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabS2109A-F Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2261485 = {S63810, 139, { &P18469,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTabS2109A-F Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2362250 = {S63811, 53, { &P21032,&P18615,&P18522,&P18092 }};	/*Useragent - Mozilla/5.0 (Mobile; rv:18.0) Gecko/18.0 Firefox/18.0*/
const Device UA2754260 = {S63812, 138, { &P17172,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Transformer TF101 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2770684 = {S63813, 133, { &P17172,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Transformer Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2772678 = {S63814, 133, { &P17172,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Transformer Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2229007 = {S63815, 182, { &P14669,&P5843,&P24671,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2556724 = {S63816, 28, { &P11860,&P5776,&P21724,&P18092 }};	/*Useragent - OneBrowser/3.0,Nokia6681/2.0*/
const Device UA2558017 = {S63817, 29, { &P11860,&P5776,&P21724,&P18092 }};	/*Useragent - One Browser/3.0,Nokia6681/2.0*/
const Device UA2582851 = {S63818, 104, { &P15364,&P5690,&P19818,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.8.1 Safari/534.34*/
const Device UA3065338 = {S63819, 127, { &P16976,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT300 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA7695 = {S63820, 139, { &P16383,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; Ivy Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA40937 = {S63821, 144, { &P16383,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-kw; EQ U8110 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA56616 = {S63822, 141, { &P16383,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; sr-rs; U8110 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018409 = {S63823, 63, { &P16383,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; U8110 Build/ERE27)*/
const Device UA61643 = {S63824, 104, { &P11824,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-T919/T919UVHL3 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA1854663 = {S63825, 76, { &P15364,&P18146,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; SunOS sun4u; en-US; rv:1.8.1.1) Gecko/X Firefox/2.0.0.1*/
const Device UA380904 = {S63826, 133, { &P15364,&P17816,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.7 (KHTML, like Gecko) Ubuntu/10.04 Chromium/16.0.912.77 Chrome/16.0.912.77 Safari/535.7*/
const Device UA648100 = {S63827, 135, { &P15364,&P17816,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.7 (KHTML, like Gecko) Ubuntu/10.04 Chromium/16.0.912.77 Chrome/16.0.912.77 Safari/535.7*/
const Device UA2235199 = {S63828, 118, { &P12004,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN95) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile*/
const Device UA2366274 = {S63829, 122, { &P12004,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN95 8GB) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile*/
const Device UA2505154 = {S63830, 117, { &P12004,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian95) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2578087 = {S63831, 121, { &P12004,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian95_8gb) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2407297 = {S63832, 140, { &P20608,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Edison Build/1.1.8 X-16:29) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2421752 = {S63833, 155, { &P20608,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq edison Build/1.1.1 bq_edison_c1006_X_user) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2511906 = {S63834, 166, { &P15575,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-I9100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2512491 = {S63835, 166, { &P15575,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-I9100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2598649 = {S63836, 167, { &P15575,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-I9100G Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2602959 = {S63837, 167, { &P15575,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-I9100G Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2661606 = {S63838, 167, { &P15575,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-I9100T Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2566765 = {S63839, 146, { &P13042,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Incredible S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA3065231 = {S63840, 145, { &P18512,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT881 Build/9.8.3O-118_USC-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2339 = {S63841, 53, { &P14845,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_HD_T8282 Opera/9.50 (Windows NT 5.1; U; de)*/
const Device UA19013 = {S63842, 53, { &P14845,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_HD_T8282 Opera/9.50 (Windows NT 5.1; U; nl)*/
const Device UA19011 = {S63843, 53, { &P14845,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_HD_T8282 Opera/9.50 (Windows NT 5.1; U; pt)*/
const Device UA19015 = {S63844, 56, { &P14845,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_HD_T8282 Opera/9.50 (Windows NT 5.1; U; zh-tw)*/
const Device UA32510 = {S63845, 53, { &P14845,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_HD_T8282 Opera/9.50 (Windows NT 5.1; U; fr)*/
const Device UA44263 = {S63846, 56, { &P14845,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_HD_T8282 Opera/9.50 (Windows NT 5.1; U; es-ES)*/
const Device UA61970 = {S63847, 53, { &P14845,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_HD_T8282 Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA388375 = {S63848, 53, { &P14845,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_HD_T8282 Opera/9.50 (Windows NT 5.1; U; it)*/
const Device UA483605 = {S63849, 53, { &P14845,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_HD_T8282 Opera/9.50 (Windows NT 5.1; U; ru)*/
const Device UA488660 = {S63850, 53, { &P14845,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_HD_T8282 Opera/9.50 (Windows NT 5.1; U; nb)*/
const Device UA67740 = {S63851, 101, { &P14886,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; 7 Pro T7576)*/
const Device UA198208 = {S63852, 113, { &P14886,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; USCCHTC-PC93100; USC-US)*/
const Device UA661962 = {S63853, 30, { &P11854,&P5890,&P24667,&P18092 }};	/*Useragent - NokiaE71/UCWEB8.2.0.116/28/800*/
const Device UA683493 = {S63854, 79, { &P11854,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE71)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA2372218 = {S63855, 79, { &P11854,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokiae71-1) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA820769 = {S63856, 110, { &P21396,&P5629,&P15760,&P18092 }};	/*Useragent - Lenovo-A65/S204 Linux/2.6.35.7 Android/2.3 Release/12.07.2011 Browser/AppleWebKit533.1 Profile/ Configuration/*/
const Device UA1959352 = {S63857, 138, { &P21396,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Lenovo A65 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032109 = {S63858, 126, { &P21396,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A65 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2033837 = {S63859, 131, { &P21396,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A65 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2543045 = {S63860, 62, { &P21396,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Lenovo A65 Build/GRJ90)*/
const Device UA909975 = {S63861, 140, { &P16553,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959V Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025350 = {S63862, 140, { &P16553,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959V Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030100 = {S63863, 140, { &P16553,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959V Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2686221 = {S63864, 139, { &P16553,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959V Build/cMIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2816220 = {S63865, 62, { &P16553,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-T959V Build/IMM76L)*/
const Device UA1988679 = {S63866, 129, { &P18704,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT200 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2563744 = {S63867, 153, { &P12605,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT875 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2655780 = {S63868, 154, { &P12605,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; XT875 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2971611 = {S63869, 161, { &P12605,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT875 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/targa*/
const Device UA2672354 = {S63870, 62, { &P15364,&P18145,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64; rv:20.0) Gecko/X Firefox/20.0*/
const Device UA651833 = {S63871, 139, { &P13157,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXE_Beats_Z715e; zh-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA791647 = {S63872, 139, { &P13157,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXE_Beats_Z715e; en-hk) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA829549 = {S63873, 139, { &P13157,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXE_Beats_Z715e; en-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA848968 = {S63874, 139, { &P13157,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXE_Beats_Z715e; en-gb) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA896722 = {S63875, 139, { &P13157,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXE_Beats_Z715a; en-au) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA1954831 = {S63876, 139, { &P13157,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXE_Beats_Z715e; nl-nl) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA1962000 = {S63877, 139, { &P13157,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_SensationXE_Beats_Z715e; en-in) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2754274 = {S63878, 142, { &P19862,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Micromax A110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754480 = {S63879, 126, { &P19862,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2877786 = {S63880, 126, { &P19862,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2205640 = {S63881, 138, { &P18678,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM102SH Build/S1010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2228905 = {S63882, 139, { &P18678,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM102SH2 Build/S1010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2344552 = {S63883, 60, { &P18678,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SBM102SH Build/S1010)*/
const Device UA2499621 = {S63884, 61, { &P18678,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SBM102SH2 Build/S1010)*/
const Device UA2880817 = {S63885, 138, { &P18678,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM102SH Build/S1020) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2973317 = {S63886, 139, { &P18678,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM102SH2 Build/S1020) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3062684 = {S63887, 138, { &P18678,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM102SH Build/S1040) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3111683 = {S63888, 139, { &P18678,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SBM102SH2 Build/S1040) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3027868 = {S63889, 157, { &P15575,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9100G Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3027967 = {S63890, 156, { &P15575,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3107930 = {S63891, 157, { &P15575,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9100T Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2668138 = {S63892, 141, { &P22636,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Aquaris Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA18342 = {S63893, 64, { &P14036,&P17483,&P3459,&P18092 }};	/*Useragent - LG-GS390/V10p Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA19008 = {S63894, 76, { &P14845,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_Touch_HD_T8282/480x800 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA24232 = {S63895, 114, { &P14845,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 480x800; HTC_Touch_HD_T8282; OpVer 34.119.1.611*/
const Device UA24755 = {S63896, 80, { &P14845,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_Touch_HD_T8282 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA32503 = {S63897, 114, { &P14845,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 480x800; HTC_Touch_HD_T8282; OpVer 34.159.1.612*/
const Device UA32504 = {S63898, 107, { &P14845,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_Touch_HD_T8282-orange/PPC; 480x800 OpVer 34.119.2.731 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA44226 = {S63899, 112, { &P14845,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Touch_HD_T8282/1.14.163.3 (32434)*/
const Device UA44262 = {S63900, 139, { &P14845,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_Touch_HD_T8282/1.21.621.2 (38850) Mozilla/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA56611 = {S63901, 112, { &P14845,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Touch_HD_T8282/1.14.163.2 (28253)*/
const Device UA124998 = {S63902, 91, { &P15364,&P17017,&P17390,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/2.0.1; X11); Supports MD5-Digest; Supports gzip encoding*/
const Device UA616189 = {S63903, 75, { &P15364,&P5448,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Windows; Opera Mobi/35779; U; en) Presto/2.10.254 Version/12.00*/
const Device UA1995970 = {S63904, 140, { &P14993,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT15i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2026709 = {S63905, 141, { &P14993,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  LT15i Build/4.1.B.0.431) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA2503896 = {S63906, 60, { &P19972,&P5043,&P18517,&P18092 }};	/*Useragent - Nokia113/2.0 (03.32) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2257328 = {S63907, 130, { &P21180,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Next9P Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2565827 = {S63908, 140, { &P22657,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ASP-4500Z Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2935219 = {S63909, 62, { &P22657,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ASP-4500Z Build/JRO03C)*/
const Device UA2994733 = {S63910, 136, { &P24373,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LGMS659 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3030753 = {S63911, 129, { &P24373,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LGMS659 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3080937 = {S63912, 150, { &P18460,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-1 Build/HuaweiU8950-1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083146 = {S63913, 152, { &P18460,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-51 Build/HuaweiU8950-51) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083202 = {S63914, 154, { &P18460,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-51 Build/HuaweiU8950N-51) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110305 = {S63915, 152, { &P18460,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-1 Build/HuaweiU8950N-1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA166744 = {S63916, 33, { &P15364,&P7133,&P18404,&P18092 }};	/*Useragent - Opera 9.4 (Windows NT 6.1; U; en)*/
const Device UA599738 = {S63917, 43, { &P15364,&P7133,&P18404,&P18092 }};	/*Useragent - Opera 9.4 (Windows NT 6.1; U; en)	Opera 9.4*/
const Device UA496539 = {S63918, 105, { &P15364,&P18136,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.13) Gecko/X Linux Mint/9 (Isadora) Firefox/3.6.13 GTB7.1*/
const Device UA527199 = {S63919, 100, { &P15364,&P18136,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.14) Gecko/X Linux Mint/9 (Isadora) Firefox/3.6.14*/
const Device UA622898 = {S63920, 98, { &P15364,&P18136,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.24) Gecko/X Linux Mint/9 (Isadora) Firefox/3.6.24*/
const Device UA643202 = {S63921, 100, { &P15364,&P18136,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.24) Gecko/X Linux Mint/9 (Isadora) Firefox/3.6.24*/
const Device UA771999 = {S63922, 100, { &P15364,&P18136,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.15) Gecko/X Linux Mint/9 (Isadora) Firefox/3.6.15*/
const Device UA804435 = {S63923, 100, { &P15364,&P18136,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.21) Gecko/X Linux Mint/9 (Isadora) Firefox/3.6.21*/
const Device UA867053 = {S63924, 98, { &P15364,&P18136,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.6) Gecko/X Linux Mint/9 (Isadora) Firefox/3.6.6*/
const Device UA1909348 = {S63925, 105, { &P15364,&P18136,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.24) Gecko/X Linux Mint/9 (Isadora) Firefox/3.6.24 GTB7.1*/
const Device UA2264855 = {S63926, 98, { &P15364,&P18136,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.9) Gecko/X Linux Mint/9 (Isadora) Firefox/3.6.9*/
const Device UA2283574 = {S63927, 105, { &P15364,&P18136,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.17) Gecko/X Linux Mint/9 (Isadora) Firefox/3.6.17 GTB7.1*/
const Device UA1996973 = {S63928, 136, { &P20095,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; 101F Build/V43R36H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2000844 = {S63929, 136, { &P20095,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; 101F Build/V45R37B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2362303 = {S63930, 136, { &P20095,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; 101F Build/V49R39H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2403477 = {S63931, 136, { &P20095,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; 101F Build/V47R38J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566241 = {S63932, 136, { &P20095,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; 101F Build/V51R40E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2706381 = {S63933, 58, { &P20095,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; 101F Build/V51R40E)*/
const Device UA2237489 = {S63934, 154, { &P20018,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993D Build/ICECREAM) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2457854 = {S63935, 153, { &P20018,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993 Build/ICECREAM) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2754162 = {S63936, 150, { &P18640,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT901 Build/7.7.1Q-144_SLS_S240-58) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2974110 = {S63937, 138, { &P23169,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT914 Build/2_32A_2012) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA49630 = {S63938, 140, { &P18037,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; HTC PLS7373ADR Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2230513 = {S63939, 138, { &P18221,&P5843,&P24668,&P18092 }};	/*Useragent - Nokia302/5.0 (14.53) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia302) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2975930 = {S63940, 137, { &P16356,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023135 = {S63941, 129, { &P16356,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA901970 = {S63942, 101, { &P15364,&P17285,&P22343,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.32) Gecko/X Firefox/3.6.32 (Palemoon/3.6.32)*/
const Device UA2405862 = {S63943, 151, { &P18583,&P5043,&P24673,&P18092 }};	/*Useragent - Nokia110/2.0 (03.04) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia110) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2532978 = {S63944, 160, { &P12473,&P5204,&P24676,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2730c-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2666131 = {S63945, 156, { &P20541,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PAP4500TDUO Build/PrestigioPAP4500TDUO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937098 = {S63946, 154, { &P20541,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PAP4500DUO Build/PrestigioPAP4500DUO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2710303 = {S63947, 131, { &P15767,&P19826,&P20782,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/10B146 Safari/7534.48.3*/
const Device UA2713608 = {S63948, 131, { &P15767,&P19826,&P20782,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/10B329 Safari/7534.48.3*/
const Device UA2719401 = {S63949, 129, { &P15767,&P19826,&P20782,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/10B141 Safari/7534.48.3*/
const Device UA3048196 = {S63950, 131, { &P15767,&P19826,&P20782,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/10B147 Safari/7534.48.3*/
const Device UA2774290 = {S63951, 134, { &P22727,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGP312 Build/10.1.C.0.370) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA64648 = {S63952, 76, { &P16217,&P5208,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; BREW 3.1.5; en )/800x480 Samsung SCH-U960*/
const Device UA455366 = {S63953, 133, { &P12280,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; 6534.46) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5*/
const Device UA599759 = {S63954, 136, { &P12280,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9B176 Safari/6533.18.5*/
const Device UA599780 = {S63955, 136, { &P12280,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9B179 Safari/6533.18.5*/
const Device UA615773 = {S63956, 150, { &P12280,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en_us) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5*/
const Device UA616089 = {S63957, 148, { &P12280,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; en_us) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5*/
const Device UA646475 = {S63958, 138, { &P12280,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9B206 Safari/6533.18.5*/
const Device UA657919 = {S63959, 135, { &P12280,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; 6534.46) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5*/
const Device UA2234251 = {S63960, 137, { &P12280,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/6533.18.5 DiigoBrowser*/
const Device UA2575987 = {S63961, 150, { &P12280,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9B206 Safari/6533.18.5*/
const Device UA2775694 = {S63962, 150, { &P12280,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; es-es) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA2889109 = {S63963, 148, { &P12280,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA2261488 = {S63964, 136, { &P21839,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Penta WS802C Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2426559 = {S63965, 136, { &P21839,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Penta WS802C Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2466066 = {S63966, 130, { &P21839,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; WS802C Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2540532 = {S63967, 73, { &P21001,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-S3802/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2606127 = {S63968, 118, { &P15364,&P18110,&P22465,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.22 (KHTML, like Gecko) Iron/25.0.1400.0 Chrome/25.0.1400.0 Safari/537.22*/
const Device UA2606148 = {S63969, 120, { &P15364,&P18110,&P22465,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Iron/25.0.1400.0 Chrome/25.0.1400.0 Safari/537.22*/
const Device UA3093022 = {S63970, 72, { &P12280,&P22882,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.11A465 (iPhone; U; CPU OS 7_0 like Mac OS X; en_us)*/
const Device UA3115294 = {S63971, 75, { &P12280,&P22882,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.11A470a (iPhone; U; CPU OS 7_0_1 like Mac OS X; en_us)*/
const Device UA3115454 = {S63972, 74, { &P12280,&P22882,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.11A501 (iPhone; U; CPU OS 7_0_2 like Mac OS X; en_us)*/
const Device UA17267 = {S63973, 97, { &P14782,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE61i Orange/1.0 Series60/3.0 Nokia E61i/2.0633.65.01 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64459 = {S63974, 97, { &P14782,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE61i-1/3.0 (1.0633.62.05) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64806 = {S63975, 96, { &P14782,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE61-1/3.0 (3.0633.09.04) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA67293 = {S63976, 122, { &P14782,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.1; U; [en-us]; Series60/3.0 NokiaE61/3.0633.09.04) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA49089 = {S63977, 102, { &P14631,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-SGH-A927/A927UCJF5 SHP/VPP/R5 Dolfin/1.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA63176 = {S63978, 125, { &P14485,&P5208,&P4223,&P18092 }};	/*Useragent - NetFront/3.5.1(BREW 3.1.5; U; en-us; SAMSUNG; NetFront/3.1.5/AMB) Sprint M550 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA255858 = {S63979, 106, { &P15364,&P6596,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1324.0.0) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/17.0.942.0 Safari/535.8*/
const Device UA350361 = {S63980, 111, { &P15364,&P6596,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1384.0.2011) AppleWebKit/535.10 (KHTML, like Gecko) Chrome/17.0.955.0 Safari/535.10*/
const Device UA353558 = {S63981, 111, { &P15364,&P6596,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1408.0.2011) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.962.0 Safari/535.11*/
const Device UA366257 = {S63982, 110, { &P15364,&P6596,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1412.64.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.27 Safari/535.11*/
const Device UA367228 = {S63983, 110, { &P15364,&P6596,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1412.87.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.33 Safari/535.11*/
const Device UA368599 = {S63984, 111, { &P15364,&P6596,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1412.121.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.38 Safari/535.11*/
const Device UA380823 = {S63985, 111, { &P15364,&P6596,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1412.142.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.43 Safari/535.11*/
const Device UA381224 = {S63986, 111, { &P15364,&P6596,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1412.150.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.46 Safari/535.11*/
const Device UA411839 = {S63987, 111, { &P15364,&P6596,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1412.205.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.60 Safari/535.11*/
const Device UA427343 = {S63988, 111, { &P15364,&P6596,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1412.223.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11*/
const Device UA428344 = {S63989, 111, { &P15364,&P6596,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1412.230.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.78 Safari/535.11*/
const Device UA428353 = {S63990, 111, { &P15364,&P6596,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1412.231.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.78 Safari/535.11*/
const Device UA428909 = {S63991, 111, { &P15364,&P6596,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1412.234.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.80 Safari/535.11*/
const Device UA590250 = {S63992, 111, { &P15364,&P6596,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1412.186.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.54 Safari/535.11*/
const Device UA823285 = {S63993, 111, { &P15364,&P6596,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1412.174.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.51 Safari/535.11*/
const Device UA1926662 = {S63994, 109, { &P15364,&P6596,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1341.0.2011) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/17.0.942.0 Safari/535.8*/
const Device UA1935330 = {S63995, 111, { &P15364,&P6596,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1403.0.2011) AppleWebKit/535.10 (KHTML, like Gecko) Chrome/17.0.955.0 Safari/535.10*/
const Device UA2305429 = {S63996, 109, { &P15364,&P6596,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1319.0.2011) AppleWebKit/535.8 (KHTML, like Gecko) Chrome/17.0.940.0 Safari/535.8*/
const Device UA2441057 = {S63997, 111, { &P15364,&P6596,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 1412.245.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.81 Safari/535.11*/
const Device UA637376 = {S63998, 125, { &P11669,&P5776,&P24663,&P18092 }};	/*Useragent - Nokia7610/2.0 (7.0642.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.9.1.120/27/351/UCWEB*/
const Device UA827967 = {S63999, 75, { &P15364,&P18110,&P20917,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1b3) Gecko/X Firefox/3.1b3*/
const Device UA2753408 = {S64000, 147, { &P23198,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI H868C Build/HuaweiH868C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2754088 = {S64001, 136, { &P19911,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E970 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA631521 = {S64002, 79, { &P15364,&P18110,&P18198,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.28) Gecko/X Thunderbird/3.1.20*/
const Device UA2431076 = {S64003, 172, { &P20134,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2755128 = {S64004, 149, { &P18318,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2773915 = {S64005, 150, { &P18318,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Transformer Pad TF700KL Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2893239 = {S64006, 149, { &P18318,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ASUS Transformer Pad TF700T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2974307 = {S64007, 136, { &P18320,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974870 = {S64008, 129, { &P18320,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA49014 = {S64009, 141, { &P12493,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; U8230 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA745566 = {S64010, 63, { &P12493,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; U8230 Build/ERE27)*/
const Device UA2019758 = {S64011, 147, { &P12493,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-pt; U8230 Build/HuaweiU8230) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2502831 = {S64012, 74, { &P23092,&P22839,&P17394,&P18092 }};	/*Useragent - ZTE-F555/1.0.0 NetFront/4.2 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA3021397 = {S64013, 134, { &P24368,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5215 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3025435 = {S64014, 127, { &P24368,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5215 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA316945 = {S64015, 87, { &P15364,&P18110,&P4811,&P18092 }};	/*Useragent - Opera/9.02 (Linux armv7l; U; ARCHOS; GOGI; G6H; Version 1.7.22    (WMDRMPD: 10.1) ; en)*/
const Device UA391797 = {S64016, 35, { &P15364,&P18110,&P4811,&P18092 }};	/*Useragent - Opera/9.02 (X11; Linux i686; U; en)*/
const Device UA433911 = {S64017, 55, { &P15364,&P18110,&P4811,&P18092 }};	/*Useragent - Opera/9.02 (Linux armv5tejl; U; ARCHOS; GOGI; a605; en)*/
const Device UA469581 = {S64018, 35, { &P15364,&P18110,&P4811,&P18092 }};	/*Useragent - Opera/9.00 (X11; Linux i686; U; en)*/
const Device UA719134 = {S64019, 35, { &P15364,&P18110,&P4811,&P18092 }};	/*Useragent - Opera/9.01 (X11; Linux i686; U; en)*/
const Device UA232937 = {S64020, 128, { &P17595,&P17297,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.231 Safari/534.10*/
const Device UA344207 = {S64021, 128, { &P17595,&P17297,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_2; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10*/
const Device UA810226 = {S64022, 128, { &P17595,&P17297,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_3; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10*/
const Device UA883640 = {S64023, 128, { &P17595,&P17297,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_0; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10*/
const Device UA899117 = {S64024, 128, { &P17595,&P17297,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10*/
const Device UA2382544 = {S64025, 128, { &P17595,&P17297,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10*/
const Device UA240443 = {S64026, 109, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.811.89) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.157 Safari/535.1*/
const Device UA247406 = {S64027, 110, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.811.132) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1*/
const Device UA249803 = {S64028, 110, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.811.117) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1*/
const Device UA250771 = {S64029, 110, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.811.154) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.204 Safari/535.1*/
const Device UA329804 = {S64030, 109, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.805.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.832.0 Safari/535.1*/
const Device UA334849 = {S64031, 108, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.811.47) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.98 Safari/535.1*/
const Device UA336562 = {S64032, 109, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.811.61) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.110 Safari/535.1*/
const Device UA340248 = {S64033, 110, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.811.105) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.163 Safari/535.1*/
const Device UA622678 = {S64034, 106, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.774.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.825.0 Safari/535.1*/
const Device UA660340 = {S64035, 109, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.811.77) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.125 Safari/535.1*/
const Device UA689584 = {S64036, 109, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.642.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.789.0 Safari/535.1*/
const Device UA760044 = {S64037, 109, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.811.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.834.0 Safari/535.1*/
const Device UA878609 = {S64038, 108, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.811.18) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.19 Safari/535.1*/
const Device UA880086 = {S64039, 108, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.811.34) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.87 Safari/535.1*/
const Device UA885607 = {S64040, 108, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.811.26) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.25 Safari/535.1*/
const Device UA891497 = {S64041, 106, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.751.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.818.0 Safari/535.1*/
const Device UA1892840 = {S64042, 108, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.811.15) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.16 Safari/535.1*/
const Device UA2139358 = {S64043, 109, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.667.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.800.0 Safari/535.1*/
const Device UA2140276 = {S64044, 109, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.751.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.818.0 Safari/535.1*/
const Device UA2140802 = {S64045, 109, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.775.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.825.0 Safari/535.1*/
const Device UA2144894 = {S64046, 110, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.787.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.827.10 Safari/535.1*/
const Device UA2259959 = {S64047, 109, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.811.62) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.110 Safari/535.1*/
const Device UA2290864 = {S64048, 109, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.688.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.804.0 Safari/535.1*/
const Device UA2291142 = {S64049, 109, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.700.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.807.0 Safari/535.1*/
const Device UA2292124 = {S64050, 109, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.712.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.807.0 Safari/535.1*/
const Device UA2293991 = {S64051, 109, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.771.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.825.0 Safari/535.1*/
const Device UA2296601 = {S64052, 109, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.811.64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.112 Safari/535.1*/
const Device UA2316284 = {S64053, 109, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 14.759.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.822.0 Safari/535.1*/
const Device UA2807604 = {S64054, 109, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 15.813.2011) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.4 Safari/535.1*/
const Device UA3042303 = {S64055, 110, { &P15364,&P6596,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; CrOS i686 0.13.507) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.94 Safari/535.1*/
const Device UA2772776 = {S64056, 156, { &P20146,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9082 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2753339 = {S64057, 145, { &P18614,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT890 Build/8.7.1I-110_SML-20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755081 = {S64058, 142, { &P14969,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire SV Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755520 = {S64059, 141, { &P14969,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2957370 = {S64060, 141, { &P14969,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3023266 = {S64061, 156, { &P12897,&P5843,&P24679,&P18092 }};	/*Useragent - NokiaC2-00/2.0 (03.99) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-00) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA169533 = {S64062, 74, { &P15364,&P18145,&P17825,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/3.5; FreeBSD) KHTML/3.5.10 (like Gecko)*/
const Device UA2428305 = {S64063, 142, { &P21254,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NGM Polaris Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2535431 = {S64064, 142, { &P21254,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NGM P0laris Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2794544 = {S64065, 142, { &P21254,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; NGM_P0laris Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2845662 = {S64066, 142, { &P21254,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; NGM_Polaris Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567252 = {S64067, 140, { &P18451,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26ii Build/6.1.A.2.55) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA3068204 = {S64068, 145, { &P12814,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I777 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3082170 = {S64069, 145, { &P12814,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SAMSUNG-SGH-I777 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094918 = {S64070, 137, { &P12814,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I777 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA599867 = {S64071, 90, { &P17976,&P5096,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2.2; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00*/
const Device UA863157 = {S64072, 88, { &P17976,&P5096,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.10.254 Version/12.00*/
const Device UA875277 = {S64073, 93, { &P17976,&P5096,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2.1; Linux; Opera Tablet/ADR-X; U; en-GB) Presto/2.10.254 Version/12.00*/
const Device UA2795005 = {S64074, 145, { &P22906,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A390_ROW Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA13258 = {S64075, 129, { &P13782,&P17483,&P3459,&P18092 }};	/*Useragent - LG-T310-Orange/V10b Obigo/Q7.3 MMS/LG-MMS-V1.1/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA64872 = {S64076, 122, { &P13782,&P17483,&P3459,&P18092 }};	/*Useragent - LG-T310/V100 Obigo/Q7.3 MMS/LG-MMS-V1.1/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2676186 = {S64077, 137, { &P18300,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T769 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2753274 = {S64078, 134, { &P23807,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A50 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957863 = {S64079, 56, { &P23807,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; A50 Build/JRO03C)*/
const Device UA3096048 = {S64080, 112, { &P18644,&P5043,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia112/03.32; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA2645 = {S64081, 75, { &P11680,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KM900-Bouygues/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA21837 = {S64082, 73, { &P11680,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KM900-orange/V10v Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA54455 = {S64083, 67, { &P11680,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KM900F/V10a Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63856 = {S64084, 66, { &P11680,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KM900/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA183923 = {S64085, 140, { &P17172,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Transformer TF101 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2383915 = {S64086, 69, { &P15060,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GT-S5660; 320*480) UCWEB7.9.0.94/139/352*/
const Device UA2455608 = {S64087, 69, { &P15060,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.1; zh-cn; GT-S5660; 320*480) UCWEB7.9.0.94/139/352*/
const Device UA2753963 = {S64088, 129, { &P17686,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ601 Build/I.7.1-45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA834848 = {S64089, 115, { &P15364,&P7133,&P20917,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.1) Gecko/X Firefox/3.1b3;MEGAUPLOAD 1.0 (.NET CLR 3.5.30729)*/
const Device UA614898 = {S64090, 64, { &P15364,&P5690,&P18367,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:12.0) Gecko/X Thunderbird/12.0.1*/
const Device UA785046 = {S64091, 71, { &P15364,&P5690,&P18367,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:12.0) Gecko/X Thunderbird/12.0.1*/
const Device UA911718 = {S64092, 139, { &P13109,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC-X710a Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2433381 = {S64093, 226, { &P18283,&P18111,&P18369,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.31782/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2516343 = {S64094, 226, { &P18283,&P18111,&P18369,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.31782/28.3782; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2429935 = {S64095, 155, { &P14826,&P5843,&P24675,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac1-01) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2507771 = {S64096, 64, { &P15364,&P7133,&P20404,&P18092 }};	/*Useragent - Opera/12.80 (Windows NT 6.1; U; en) Presto/2.9.168 Version/12.80*/
const Device UA2507993 = {S64097, 65, { &P15364,&P7133,&P20404,&P18092 }};	/*Useragent - Opera/12.80 (Windows NT 6.1; U; en) Presto/2.10.289 Version/12.08*/
const Device UA2878886 = {S64098, 103, { &P14748,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae72-1) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2967922 = {S64099, 94, { &P14748,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE72-1) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3078579 = {S64100, 94, { &P14748,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE72-2) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA2202909 = {S64101, 83, { &P17976,&P18111,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1.1; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2238153 = {S64102, 83, { &P17976,&P18111,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1.2; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2658607 = {S64103, 93, { &P17976,&P18111,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1.1; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10,gzip(gfe)*/
const Device UA2845195 = {S64104, 81, { &P17976,&P18111,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2987288 = {S64105, 93, { &P17976,&P18111,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1.2; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10,gzip(gfe)*/
const Device UA3066871 = {S64106, 136, { &P23231,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E988 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA685917 = {S64107, 63, { &P15364,&P16110,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; U; en) Presto/2.9.168 Version/11.50*/
const Device UA2237413 = {S64108, 118, { &P15364,&P18110,&P22462,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4*/
const Device UA2237720 = {S64109, 116, { &P15364,&P18110,&P22462,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4*/
const Device UA2368164 = {S64110, 140, { &P21181,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; Lenovo S560; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2504473 = {S64111, 142, { &P21181,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lenovo S560 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2653022 = {S64112, 64, { &P21181,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo S560 Build/IMM76D)*/
const Device UA2426160 = {S64113, 143, { &P20134,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2747301 = {S64114, 67, { &P20134,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 4.1.1; GT-S5830i Build/GINGERBREAD)*/
const Device UA2467892 = {S64115, 151, { &P20137,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; Xtreamer_Mobile_AiKi Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2662837 = {S64116, 152, { &P20137,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; Xtreamer Mobile Aiki4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2847278 = {S64117, 73, { &P20137,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Xtreamer_Mobile_AiKi Build/IML74K)*/
const Device UA3051080 = {S64118, 137, { &P20137,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; he-il; Aiki-5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2753545 = {S64119, 142, { &P15438,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754483 = {S64120, 148, { &P15438,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754640 = {S64121, 138, { &P15438,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC-Z710a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773854 = {S64122, 142, { &P15438,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2865908 = {S64123, 142, { &P23015,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; lenovo A820 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2891662 = {S64124, 143, { &P23015,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Lenovo A820t Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936796 = {S64125, 134, { &P23015,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Lenovo A820t Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2956271 = {S64126, 64, { &P23015,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Lenovo A820 Build/JZO54K)*/
const Device UA2957603 = {S64127, 161, { &P23015,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; LNV-Lenovo_A820e/S-2-03; 960*540; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1.2 Mobile Safari/534.30*/
const Device UA2998841 = {S64128, 65, { &P23015,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Lenovo A820t Build/JZO54K)*/
const Device UA3057867 = {S64129, 142, { &P23015,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2;en-us; Lenovo A820 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30;*/
const Device UA3072682 = {S64130, 143, { &P23015,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Lenovo A820e Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2975059 = {S64131, 137, { &P18687,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P708g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975715 = {S64132, 136, { &P18687,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P705 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975861 = {S64133, 137, { &P18687,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P705g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021964 = {S64134, 137, { &P18687,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P705f Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2342885 = {S64135, 75, { &P18435,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; ARM; Trident/6.0; Touch)*/
const Device UA2425714 = {S64136, 83, { &P18435,&P16110,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; ARM; Trident/6.0; Touch; ARMBJS)*/
const Device UA2566804 = {S64137, 129, { &P19782,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B1-A71 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2712203 = {S64138, 121, { &P15364,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; MB865) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734716 = {S64139, 138, { &P15364,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Transformer Prime Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2859929 = {S64140, 136, { &P15364,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Transformer Pad Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2865440 = {S64141, 139, { &P15364,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; v89_zlh_hd Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2996190 = {S64142, 130, { &P15364,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; AT10LE-A Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3006418 = {S64143, 123, { &P15364,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; M6 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3006955 = {S64144, 132, { &P15364,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; R821 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3022832 = {S64145, 144, { &P15364,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Lenovo S820_ROW Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3023352 = {S64146, 143, { &P15364,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Primo_NX Build/WALTONWALTON) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3092638 = {S64147, 128, { &P15364,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ME302KL Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3101871 = {S64148, 142, { &P15364,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Aveo NOTE II+ Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3107215 = {S64149, 137, { &P15364,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SM-T217A Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3111694 = {S64150, 133, { &P15364,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; A240 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3121544 = {S64151, 137, { &P15364,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; STAIRWAY Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3121559 = {S64152, 136, { &P15364,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9197 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3121562 = {S64153, 132, { &P15364,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; S510 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3124837 = {S64154, 137, { &P15364,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC_PO582 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3125169 = {S64155, 129, { &P15364,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; M470BSE Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3125564 = {S64156, 127, { &P15364,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; B1-711 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3126008 = {S64157, 131, { &P15364,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; X50D Build/X50D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3127662 = {S64158, 136, { &P15364,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R960 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2347 = {S64159, 110, { &P14732,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-GT-B7330/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG-GT-B7330V/XXIJA*/
const Device UA5096 = {S64160, 87, { &P14732,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-GT-B7330R/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA9291 = {S64161, 87, { &P14732,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-GT-B7330B/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA39778 = {S64162, 94, { &P14732,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-GT-B7330Orange/BVJE2 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA63169 = {S64163, 86, { &P14732,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-GT-B7330/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA437262 = {S64164, 41, { &P13977,&P5890,&P24665,&P18092 }};	/*Useragent - NokiaN73/UC Browser8.0.4.121/28/444/UCWEB*/
const Device UA349833 = {S64165, 142, { &P18018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; IM-A690S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA694423 = {S64166, 142, { &P18018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; IM-A690L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025482 = {S64167, 140, { &P18018,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-; IM-A690S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2640152 = {S64168, 66, { &P18018,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; IM-A690S Build/GINGERBREAD)*/
const Device UA907904 = {S64169, 74, { &P15364,&P5690,&P22535,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:10.0.3) Gecko/X Firefox/10.0.3 Epic/10.0.3*/
const Device UA2364515 = {S64170, 74, { &P15364,&P5690,&P22535,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:10.0.3) Gecko/X Firefox/10.0.3 Epic/10.0.3*/
const Device UA2354074 = {S64171, 75, { &P17595,&P17295,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8) Presto/2.12.388 Version/12.10*/
const Device UA2384491 = {S64172, 155, { &P16273,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T679 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2424056 = {S64173, 31, { &P15803,&P5843,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia200/11.81)*/
const Device UA2508019 = {S64174, 93, { &P14983,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaC7-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2676692 = {S64175, 155, { &P18113,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2756015 = {S64176, 145, { &P19867,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2944279 = {S64177, 146, { &P19867,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3031262 = {S64178, 158, { &P18280,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC VLE_U Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3031336 = {S64179, 158, { &P18280,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA924237 = {S64180, 139, { &P22592,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717R Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2205327 = {S64181, 172, { &P20134,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2355891 = {S64182, 142, { &P21526,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cherry w500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2425310 = {S64183, 129, { &P22100,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M1070 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2567173 = {S64184, 77, { &P15364,&P5690,&P22559,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:19.0) Gecko/X Firefox/19.0 IceDragon/19.0.1.0*/
const Device UA14230 = {S64185, 73, { &P18088,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Verzio Envii*/
const Device UA20506 = {S64186, 75, { &P18088,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Sprint:PPC6800*/
const Device UA49500 = {S64187, 130, { &P18088,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Smartphone; 320x240; Mail/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA56680 = {S64188, 89, { &P18088,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Amoi E78/R7.0 NF-Browser/3.2*/
const Device UA57743 = {S64189, 110, { &P18088,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Smartphone; 240x320; SPV E650; OpVer 22.135.1.611*/
const Device UA66558 = {S64190, 70, { &P18088,&P5448,&P22271,&P18092 }};	/*Useragent - 8900a/1.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA87049 = {S64191, 84, { &P18088,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Sprint:SPH-i325 320x240*/
const Device UA87863 = {S64192, 115, { &P18088,&P5448,&P22271,&P18092 }};	/*Useragent - PPC; 240x320; CHT 9100/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA167619 = {S64193, 60, { &P18088,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA29998 = {S64194, 73, { &P12280,&P17145,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8C148 (iPhone; U; CPU OS 4_2_1 like Mac OS X; en_us)*/
const Device UA178849 = {S64195, 73, { &P12280,&P17145,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8E200 (iPhone; U; CPU OS 4_2_6 like Mac OS X; en_us)*/
const Device UA191861 = {S64196, 73, { &P12280,&P17145,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8E401 (iPhone; U; CPU OS 4_2_8 like Mac OS X; en_us)*/
const Device UA234073 = {S64197, 74, { &P12280,&P17145,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8E600 (iPhone; U; CPU OS 4_2_10 like Mac OS X; en_us)*/
const Device UA418347 = {S64198, 73, { &P12280,&P17145,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8E303 (iPhone; U; CPU OS 4_2_7 like Mac OS X; en_us)*/
const Device UA707098 = {S64199, 73, { &P12280,&P17145,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8E128 (iPhone; U; CPU OS 4_2_5 like Mac OS X; en_us)*/
const Device UA711408 = {S64200, 73, { &P12280,&P17145,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8E501 (iPhone; U; CPU OS 4_2_9 like Mac OS X; en_us)*/
const Device UA2409739 = {S64201, 80, { &P15364,&P17285,&P19773,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; de; rv:1.9.1.8) Gecko/X SeaMonkey/2.0.3*/
const Device UA2701238 = {S64202, 150, { &P16168,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; NookColor Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2973889 = {S64203, 140, { &P21542,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6503 Build/10.3.A.0.423) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973906 = {S64204, 137, { &P21506,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023519 = {S64205, 138, { &P21506,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975985 = {S64206, 150, { &P18640,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT901 Build/7.7.1Q-144_SLS_S240-58) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA179238 = {S64207, 118, { &P17595,&P17295,&P22889,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_8; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) iCab/4.8 Safari/533.16*/
const Device UA451782 = {S64208, 108, { &P17595,&P17295,&P22889,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) iCab/4.8 Safari/533.16*/
const Device UA765350 = {S64209, 143, { &P24005,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ZT ICS Build/ICS.f04ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2412242 = {S64210, 65, { &P24005,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; ZT ICS Build/ICS.f04ref.X)*/
const Device UA2558742 = {S64211, 139, { &P23969,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2753015 = {S64212, 145, { &P13064,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I727 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783541 = {S64213, 136, { &P13064,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I727 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753440 = {S64214, 132, { &P22754,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HP Slate 7 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2774409 = {S64215, 138, { &P22871,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9295 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3016284 = {S64216, 60, { &P22871,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9295 Build/JDQ39)*/
const Device UA2975072 = {S64217, 127, { &P16976,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT300 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA22006 = {S64218, 83, { &P13186,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW850i/R1ED Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64745 = {S64219, 84, { &P13186,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW850iv/R1ED Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA32997 = {S64220, 145, { &P12821,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Sprint ATP515CKIT Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA33451 = {S64221, 145, { &P12821,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Sprint ATP515CKIT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA62526 = {S64222, 41, { &P14476,&P17484,&P3625,&P18092 }};	/*Useragent - sam-r100 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA121182 = {S64223, 70, { &P15364,&P6704,&P20901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.4) Gecko Netscape/7.1 (ax)*/
const Device UA1857580 = {S64224, 78, { &P15364,&P6704,&P20901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win 9x 4.90; en-US; rv:1.4) Gecko/X Netscape/7.1 (ax)*/
const Device UA682491 = {S64225, 81, { &P14033,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaC5-03)/UC Browser8.2.0.132/50/355/UCWEB Mobile*/
const Device UA700632 = {S64226, 32, { &P14033,&P5129,&P24667,&P18092 }};	/*Useragent - NokiaC5-03/UCWEB8.2.0.116/50/800*/
const Device UA2975525 = {S64227, 139, { &P20006,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30a Build/9.1.A.0.489) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA64619 = {S64228, 83, { &P13240,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonW890i/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA624717 = {S64229, 132, { &P14905,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; GT-P7100 Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2608197 = {S64230, 130, { &P14905,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; ru-; GT-P7100 Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2627278 = {S64231, 139, { &P14905,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; it-it; GT-P7100 Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2236472 = {S64232, 79, { &P15364,&P16110,&P22365,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:15.0) Gecko/X Firefox/15.2 PaleMoon/15.2*/
const Device UA2259287 = {S64233, 96, { &P15364,&P16110,&P22365,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:15.2) Gecko/X Firefox/15.2.1-x64 PaleMoon/15.2.1-x64*/
const Device UA2397241 = {S64234, 138, { &P23550,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; ZTE T81 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567128 = {S64235, 131, { &P22043,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P5113 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583341 = {S64236, 131, { &P22043,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P5113 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567140 = {S64237, 140, { &P12681,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus S 4G Build/JRO03R) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2666807 = {S64238, 140, { &P12681,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus S 4G Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2858562 = {S64239, 133, { &P12681,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus S 4G Build/JRO03R) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2569947 = {S64240, 136, { &P12280,&P18117,&P22901,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.3 Mobile/9B206 Safari/7534.48.3*/
const Device UA2583189 = {S64241, 134, { &P12280,&P18117,&P22901,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.3 Mobile/9B176 Safari/7534.48.3*/
const Device UA2602022 = {S64242, 129, { &P15767,&P19826,&P20781,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/10B141 Safari/7534.48.3*/
const Device UA2602652 = {S64243, 131, { &P15767,&P19826,&P20781,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/10B329 Safari/7534.48.3*/
const Device UA2604671 = {S64244, 131, { &P15767,&P19826,&P20781,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/10B146 Safari/7534.48.3*/
const Device UA2974498 = {S64245, 145, { &P13042,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Incredible S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA6279 = {S64246, 107, { &P12436,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-A107/A107UCJJ1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA2258211 = {S64247, 80, { &P17595,&P17296,&P18691,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:16.0) Gecko/X Thunderbird/16.0.2*/
const Device UA2460884 = {S64248, 155, { &P14635,&P5843,&P24675,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (08.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac3-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2714927 = {S64249, 115, { &P15364,&P17285,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.0.0.260*/
const Device UA2784041 = {S64250, 115, { &P15364,&P17285,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.0.1.303*/
const Device UA2901189 = {S64251, 115, { &P15364,&P17285,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.0.0.290*/
const Device UA2969587 = {S64252, 115, { &P15364,&P17285,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.0.1.317*/
const Device UA63073 = {S64253, 83, { &P11627,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonK850i/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2111 = {S64254, 133, { &P17425,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV3xxv/99.41.04ER BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163189) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA13958 = {S64255, 132, { &P17425,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV3xx/98.30.72AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12003079) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA13962 = {S64256, 132, { &P17425,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV3xx/96.66.72AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11033063) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA13974 = {S64257, 130, { &P17425,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV3xx/96.B0.0AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11073156) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [en]*/
const Device UA13978 = {S64258, 131, { &P17425,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV3xx/99.41.08R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163225) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA13979 = {S64259, 131, { &P17425,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV3xx/99.31.08R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12153195) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA13980 = {S64260, 130, { &P17425,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV3xx/96.71.95R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11063081) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [en]*/
const Device UA23397 = {S64261, 130, { &P17425,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV3xx/96.B0.0FR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11073256) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [en]*/
const Device UA29469 = {S64262, 131, { &P17425,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV3xx/96.A0.0ER BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11073121) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA29472 = {S64263, 131, { &P17425,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV3xx/99.41.0CR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163234) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA29475 = {S64264, 133, { &P17425,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV3xx/99.41.04ER2 BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163189) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [it]*/
const Device UA51680 = {S64265, 132, { &P17425,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV3xxv/98.50.11R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12093118) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA66842 = {S64266, 132, { &P17425,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV3xx/96.80.55AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11073083) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA66841 = {S64267, 131, { &P17425,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV3xx/96.80.53R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11073071) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA66853 = {S64268, 133, { &P17425,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV3xxv/98.20.33BR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12043064) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA637511 = {S64269, 125, { &P17425,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV3xxv/98.50.11R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; X) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA674644 = {S64270, 124, { &P17425,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV3xx/96.B0.0FR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; X) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [vi]*/
const Device UA5008 = {S64271, 77, { &P12360,&P6217,&P3418,&P18092 }};	/*Useragent - BlackBerry8230/4.6.1.234 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105*/
const Device UA42553 = {S64272, 140, { &P15881,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Incredible 2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59637 = {S64273, 140, { &P15881,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Incredible 2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA249305 = {S64274, 140, { &P15881,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Incredible 2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA331414 = {S64275, 140, { &P15881,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Incredible 2 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA421497 = {S64276, 143, { &P15881,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Incredible 2 3D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA569733 = {S64277, 140, { &P15881,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Incredible 2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA574665 = {S64278, 140, { &P15881,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Incredible 2 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA721939 = {S64279, 64, { &P15881,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; Incredible 2 Build/GRI40)*/
const Device UA2021512 = {S64280, 140, { &P15881,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Incredible 2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA701420 = {S64281, 139, { &P20726,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; AN9G2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031444 = {S64282, 63, { &P20726,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.1; AN9G2 Build/GINGERBREAD)*/
const Device UA2351620 = {S64283, 66, { &P12798,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; LT18i; 480*854) UCWEB7.9.0.94/139/352*/
const Device UA2351759 = {S64284, 66, { &P12798,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; LT18i; 480*854) UCWEB7.9.0.94/139/352*/
const Device UA2192507 = {S64285, 132, { &P19874,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-i705 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2235419 = {S64286, 137, { &P20330,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Micromax P350 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2568826 = {S64287, 32, { &P18254,&P17350,&P18245,&P18092 }};	/*Useragent - Zeus 6003 Webster Pro V2.9 Win32*/
const Device UA2794219 = {S64288, 33, { &P18254,&P17350,&P18245,&P18092 }};	/*Useragent - Zeus 74490 Webster Pro V2.9 Win32*/
const Device UA2737957 = {S64289, 139, { &P22729,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT918 Build/2_26D_2009) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2974163 = {S64290, 126, { &P18239,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A510 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3065604 = {S64291, 136, { &P20240,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E971 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA48977 = {S64292, 145, { &P14334,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SPH-M910 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA64850 = {S64293, 103, { &P13249,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-SGH-A897/A897UCJC1 SHP/VPP/R5 Jasmine/1.0 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA2236938 = {S64294, 150, { &P21616,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-style Q3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2350761 = {S64295, 151, { &P21616,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-style Q3i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2789659 = {S64296, 72, { &P21616,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; i-mobile i-style Q3 Build/IMM76D)*/
const Device UA2259692 = {S64297, 78, { &P15364,&P5690,&P20904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; it-IT; rv:1.7.5) Gecko/X Netscape/8.1*/
const Device UA2362522 = {S64298, 79, { &P17968,&P18598,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.2; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2388549 = {S64299, 81, { &P17968,&P18598,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.2.1; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2514297 = {S64300, 81, { &P17968,&P18598,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.2.2; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2751169 = {S64301, 164, { &P22874,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; zh-cn; SAMSUNG GT-I9502 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA9170 = {S64302, 98, { &P12176,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-U900/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA24389 = {S64303, 120, { &P12176,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-U900-Vodafone/U900BUHD6 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA88757 = {S64304, 99, { &P12176,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-U900T/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA2125741 = {S64305, 102, { &P15364,&P18110,&P22304,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.8.0.14eol) Gecko/X Red Hat/1.0.9-40.el4 SeaMonkey/1.0.9*/
const Device UA2131041 = {S64306, 75, { &P15364,&P18110,&P22304,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.0.7) Gecko/X SeaMonkey/1.0.5*/
const Device UA2567012 = {S64307, 136, { &P18468,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabA2109A Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA14212 = {S64308, 159, { &P11811,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-GT505/V10b Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA14213 = {S64309, 159, { &P11811,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-GT505/V10c Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA27267 = {S64310, 132, { &P11811,&P17483,&P3728,&P18092 }};	/*Useragent - LG/GT505/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA32602 = {S64311, 139, { &P11811,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GT505-Orange/V10f Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA44136 = {S64312, 159, { &P11811,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-GT505/V10d Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA44137 = {S64313, 159, { &P11811,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-GT505/V10e Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA59192 = {S64314, 133, { &P11811,&P17483,&P3728,&P18092 }};	/*Useragent - LG/GT505e/v10c Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA64718 = {S64315, 132, { &P11811,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GT505/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2842520 = {S64316, 251, { &P11811,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GT505/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-gt505) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2843216 = {S64317, 221, { &P11811,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GT505/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; vi; lg-gt505) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2952273 = {S64318, 225, { &P11811,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GT505/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gt505) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2362301 = {S64319, 134, { &P22024,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; dL1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2509342 = {S64320, 118, { &P15364,&P18110,&P22464,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17*/
const Device UA2513109 = {S64321, 120, { &P15364,&P18110,&P22464,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17*/
const Device UA3033032 = {S64322, 156, { &P14711,&P5204,&P24679,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-01) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA2456011 = {S64323, 60, { &P18644,&P5043,&P18517,&P18092 }};	/*Useragent - Nokia112/2.0 (03.32) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2458579 = {S64324, 139, { &P20264,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Curie Build/1.0.1 X-19:40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2458783 = {S64325, 139, { &P20264,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Curie Build/1.0.2 X-19:30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2459447 = {S64326, 139, { &P20264,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Curie Build/1.0.1 X-14:56) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2537992 = {S64327, 139, { &P20264,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Curie Build/1.0.4 X-16:15) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2670065 = {S64328, 139, { &P20264,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Curie Build/1.1.0 X-14:50) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2766362 = {S64329, 68, { &P20264,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; bq Curie Build/1.1.0 X-14:50)*/
const Device UA2847605 = {S64330, 139, { &P20264,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Curie Build/1.0.0 X-17:25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2850069 = {S64331, 139, { &P20264,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Curie Build/1.0.3 X-13:48) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2961132 = {S64332, 141, { &P20264,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Curie 2 Build/1.0.0_X-18:13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3092586 = {S64333, 144, { &P20264,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Curie 2 3G Build/1.0.1_X-16:30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3125232 = {S64334, 144, { &P20264,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Curie 2 3G Build/1.0.0_X-14:13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2705356 = {S64335, 133, { &P23314,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; G3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3047047 = {S64336, 55, { &P23314,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; G3 Build/IMM76D)*/
const Device UA3099818 = {S64337, 138, { &P23314,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; AOLE G3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2673221 = {S64338, 137, { &P20226,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E975K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676753 = {S64339, 136, { &P20226,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713405 = {S64340, 129, { &P20226,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2734923 = {S64341, 130, { &P20226,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E975K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3131339 = {S64342, 136, { &P21261,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA9408 = {S64343, 66, { &P13310,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GT350/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA49208 = {S64344, 67, { &P13310,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GT350i/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA60734 = {S64345, 67, { &P13310,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GT350F/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63101 = {S64346, 73, { &P13310,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GT350-Orange/v10b Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2195620 = {S64347, 137, { &P15364,&P17820,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/12.04 Chromium/22.0.1229.79 Chrome/22.0.1229.79 Safari/537.4*/
const Device UA2200198 = {S64348, 135, { &P15364,&P17820,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/12.04 Chromium/22.0.1229.79 Chrome/22.0.1229.79 Safari/537.4*/
const Device UA2599837 = {S64349, 152, { &P18648,&P5043,&P24676,&P18092 }};	/*Useragent - Nokia306/2.0 (05.93) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia306) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2566925 = {S64350, 138, { &P18636,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS930 4G Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2676851 = {S64351, 144, { &P21611,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2781894 = {S64352, 137, { &P21611,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2885443 = {S64353, 152, { &P20129,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2896173 = {S64354, 153, { &P20129,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2S Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2897689 = {S64355, 154, { &P20129,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2SC Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2943976 = {S64356, 153, { &P20129,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2A Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2961248 = {S64357, 153, { &P20129,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2C Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA183294 = {S64358, 147, { &P15364,&P17818,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Ubuntu/11.04 Chromium/10.0.648.205 Chrome/10.0.648.205 Safari/534.16*/
const Device UA312434 = {S64359, 149, { &P15364,&P17818,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Ubuntu/11.04 Chromium/10.0.648.205 Chrome/10.0.648.205 Safari/534.16*/
const Device UA458411 = {S64360, 135, { &P18380,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-07D Build/V16R23B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA609119 = {S64361, 135, { &P18380,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-07D Build/V17R24B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626751 = {S64362, 133, { &P18380,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-07D Build/F0001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032495 = {S64363, 59, { &P18380,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; F-07D Build/V17R24B)*/
const Device UA2563353 = {S64364, 135, { &P18380,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-07D Build/V18R25A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2484309 = {S64365, 141, { &P21726,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Vandroid T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2506828 = {S64366, 142, { &P22612,&P18066,&P18062,&P18092 }};	/*Useragent - HUAWEI_T8951_TD/1.0 Android/4.0.4 (Linux; U; Android 4.0.4; zh-cn) Release/05.31.2012 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30*/
const Device UA2558496 = {S64367, 139, { &P22612,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8951 Build/HuaweiT8951) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2658897 = {S64368, 148, { &P22612,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8951 Build/HuaweiT8951) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2764629 = {S64369, 70, { &P22612,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI T8951 Build/HuaweiT8951)*/
const Device UA2719733 = {S64370, 145, { &P22656,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySOL22 Build/10.2.F.3.27) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719735 = {S64371, 144, { &P22656,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SonySOL22 Build/10.2.F.2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2769428 = {S64372, 145, { &P22656,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySOL22 Build/10.2.F.3.43) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2868521 = {S64373, 141, { &P22656,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SOL22 Build/10.2.F.3.43) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879440 = {S64374, 63, { &P22656,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SOL22 Build/10.2.F.3.43)*/
const Device UA3127373 = {S64375, 145, { &P22656,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySOL22 Build/10.2.F.3.81) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3065094 = {S64376, 137, { &P18459,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS950 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA32200 = {S64377, 89, { &P12481,&P17603,&P4223,&P18092 }};	/*Useragent - PantechP2020/JIUS05172010R NetFront/3.5 OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA814793 = {S64378, 81, { &P12481,&P17603,&P4223,&P18092 }};	/*Useragent - PantechP2020/JIUSX NetFront/3.5 OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA224141 = {S64379, 158, { &P17947,&P17979,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Donut; en-us; HTC_Lexikon_C Build/Donut) AppleWebKit/528.2+(KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 CSOUTH-6325*/
const Device UA452739 = {S64380, 150, { &P16954,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; Dell Venue Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/venue.408.0*/
const Device UA2023085 = {S64381, 62, { &P16954,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; Dell Venue Build/GRI40)*/
const Device UA2395923 = {S64382, 138, { &P16954,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.4.27; en-us; Dell Venue Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042419 = {S64383, 146, { &P16954,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; Dell Venue Build/GWK74; CM-7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2004939 = {S64384, 129, { &P17686,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MZ601 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2234204 = {S64385, 129, { &P17686,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MZ601 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2570750 = {S64386, 129, { &P23072,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; T-17B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2992248 = {S64387, 58, { &P23072,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; T-17B Build/JRO03H)*/
const Device UA2869023 = {S64388, 143, { &P23173,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMP3670B Build/PMP3670B_X_v1.0.9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3107575 = {S64389, 149, { &P23173,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMP3670B-WH Build/PMP3670B-WH_X_v2.0.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2974278 = {S64390, 138, { &P20871,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA25967 = {S64391, 132, { &P13201,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; Eris Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA34796 = {S64392, 144, { &P13201,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; Eris xtrROM5.0.1 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA66552 = {S64393, 130, { &P13201,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-; Eris Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA89991 = {S64394, 137, { &P13201,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; Eris Build/ERD79) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA113629 = {S64395, 54, { &P13201,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1; Eris Build/ERD79)*/
const Device UA354154 = {S64396, 56, { &P15364,&P17017,&P18246,&P18092 }};	/*Useragent - NexianNX-G857/MTK Release/10.1.2009 Browser/MAUI Profile*/
const Device UA616245 = {S64397, 57, { &P15364,&P17017,&P18246,&P18092 }};	/*Useragent - NexianNX-G252/MTK Release/01.01.2010 Browser/MAUI Profile*/
const Device UA616241 = {S64398, 57, { &P15364,&P17017,&P18246,&P18092 }};	/*Useragent - NexianNX-G265/MTK Release/15.02.2011 Browser/MAUI Profile*/
const Device UA616249 = {S64399, 57, { &P15364,&P17017,&P18246,&P18092 }};	/*Useragent - NexianNX-G256/MTK Release/01.01.2010 Browser/MAUI Profile*/
const Device UA616248 = {S64400, 56, { &P15364,&P17017,&P18246,&P18092 }};	/*Useragent - NexianNX-G315/MTK Release/10.1.2009 Browser/MAUI Profile*/
const Device UA616257 = {S64401, 57, { &P15364,&P17017,&P18246,&P18092 }};	/*Useragent - NexianNX-G236/MTK Release/01.01.2010 Browser/MAUI Profile*/
const Device UA616261 = {S64402, 57, { &P15364,&P17017,&P18246,&P18092 }};	/*Useragent - NexianNX-G268/MTK Release/15.02.2011 Browser/MAUI Profile*/
const Device UA616266 = {S64403, 57, { &P15364,&P17017,&P18246,&P18092 }};	/*Useragent - NexianNX-G262/MTK Release/19.04.2011 Browser/MAUI Profile*/
const Device UA616285 = {S64404, 57, { &P15364,&P17017,&P18246,&P18092 }};	/*Useragent - NexianNX-G311/MTK Release/30.05.2011 Browser/MAUI Profile*/
const Device UA616295 = {S64405, 55, { &P15364,&P17017,&P18246,&P18092 }};	/*Useragent - Nexian G263/MTK Release/01.01.2012 Browser/MAUI Profile*/
const Device UA616320 = {S64406, 57, { &P15364,&P17017,&P18246,&P18092 }};	/*Useragent - NexianNX-G332/MTK Release/22.11.2007 Browser/MAUI Profile*/
const Device UA616322 = {S64407, 57, { &P15364,&P17017,&P18246,&P18092 }};	/*Useragent - NexianNXG311T/MTK Release/20.09.2011 Browser/MAUI Profile*/
const Device UA616330 = {S64408, 55, { &P15364,&P17017,&P18246,&P18092 }};	/*Useragent - Nexian G257/MTK Release/01.01.2012 Browser/MAUI Profile*/
const Device UA616244 = {S64409, 57, { &P15364,&P17017,&P18246,&P18092 }};	/*Useragent - NexianNX-G253/MTK Release/01.01.2010 Browser/MAUI Profile*/
const Device UA616242 = {S64410, 57, { &P15364,&P17017,&P18246,&P18092 }};	/*Useragent - NexianNX-G506/MTK Release/01.01.2011 Browser/MAUI Profile*/
const Device UA2696729 = {S64411, 232, { &P15364,&P17017,&P18246,&P18092 }};	/*Useragent - CKT50_COSMOS_11B_HW (MRE\2.5.00(2500) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) RECTANGLE-S03A_FLY_L2EN_206_X Release/2012.10.26 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US*/
const Device UA2727491 = {S64412, 228, { &P15364,&P17017,&P18246,&P18092 }};	/*Useragent - CKT50_COSMOS_11B_HW (MRE\2.5.00(2500) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) COCOA-S04A_FLY_L2EN_205_X Release/2012.10.15 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US*/
const Device UA2983132 = {S64413, 209, { &P15364,&P17017,&P18246,&P18092 }};	/*Useragent - J2TECH50_FTE_6464_11B_HW (MRE\3.0.00(3096) resolution\X chipset\MT6250 touch\0 tpannel\0 camera\1 gsensor\0 keyboard\normal) Fly_MV264_V12 Release/2012.10.16 WAP Browser/MAUI (HTTPS) Profile/  Q03C1-2.40 en-US*/
const Device UA3086852 = {S64414, 209, { &P15364,&P17017,&P18246,&P18092 }};	/*Useragent - J2TECH50_FTE_6464_11B_HW (MRE\3.0.00(3096) resolution\X chipset\MT6250 touch\0 tpannel\0 camera\1 gsensor\0 keyboard\normal) Fly_MV264_V12 Release/2012.10.29 WAP Browser/MAUI (HTTPS) Profile/  Q03C1-2.40 en-US*/
const Device UA794566 = {S64415, 79, { &P17968,&P17979,&P18373,&P18092 }};	/*Useragent - Opera/9.80 (Android; Linux; Opera Mobi/ADR-X; U; de) Presto/2.5.28 Version/10.1*/
const Device UA2018759 = {S64416, 118, { &P17968,&P17979,&P18373,&P18092 }};	/*Useragent - Opera/9.80 (Android; Linux; Opera Mobi/ADR-X; U; TSB_CLOUD_COMPANION;TOSHIBA_AC_AND_AZ; en) Presto/2.5.28 Version/10.1*/
const Device UA896863 = {S64417, 136, { &P19750,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; BLU DASH Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3012466 = {S64418, 143, { &P19750,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DASH MUSIC J101 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2431853 = {S64419, 219, { &P15919,&P18066,&P18369,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2622942 = {S64420, 219, { &P15919,&P18066,&P18369,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fa-fa; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3392; U; fa) Presto/2.8.119 Version/11.10*/
const Device UA2626013 = {S64421, 217, { &P15919,&P18066,&P18369,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3445; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2559536 = {S64422, 154, { &P17475,&P5768,&P24676,&P18092 }};	/*Useragent - Nokia6300/2.0 (07.00) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6300) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2779086 = {S64423, 92, { &P17475,&P5768,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6300) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2772898 = {S64424, 162, { &P22394,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6603 Build/10.1.1.A.1.253) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2739592 = {S64425, 137, { &P21628,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2957152 = {S64426, 130, { &P21628,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA675919 = {S64427, 108, { &P18249,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Acer; Allegro; bouygues)*/
const Device UA687796 = {S64428, 98, { &P18249,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Acer; Allegro)*/
const Device UA2043761 = {S64429, 122, { &P18249,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Acer; Allegro) [UsableNet Lift Mobile]*/
const Device UA2428175 = {S64430, 142, { &P19999,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Karbonn A30 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099693 = {S64431, 64, { &P19999,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Karbonn A30 Build/IMM76D)*/
const Device UA2774070 = {S64432, 116, { &P15364,&P5690,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 Sleipnir/4.1.3*/
const Device UA2877511 = {S64433, 117, { &P15364,&P5690,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36 Sleipnir/4.1.4*/
const Device UA2879232 = {S64434, 59, { &P15364,&P18346,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10; rv:22.0) Gecko/X Firefox/22.0*/
const Device UA158246 = {S64435, 134, { &P17973,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; pandigitalp1hr) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA336574 = {S64436, 167, { &P16172,&P17486,&P18373,&P18092 }};	/*Useragent - MOT-EX118 MIDP-2.0/CLDC-1.1 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U) Presto/2.5.28 Version/10.10*/
const Device UA2580759 = {S64437, 152, { &P15241,&P5843,&P24676,&P18092 }};	/*Useragent - Nokia300/5.0 (07.03) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia300) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2974440 = {S64438, 137, { &P18697,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I415 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2369670 = {S64439, 39, { &P15364,&P18580,&P22899,&P18092 }};	/*Useragent - Mercury/7.1 CFNetwork/609 Darwin/13.0.0*/
const Device UA3061620 = {S64440, 139, { &P20708,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XOLO A1000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3083206 = {S64441, 136, { &P20708,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IdeaTabA1000-F Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3083379 = {S64442, 136, { &P20708,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IdeaTabA1000-G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA900114 = {S64443, 136, { &P15803,&P5843,&P24667,&P18092 }};	/*Useragent - Nokia200/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; kau; nokia200) UCBrowser8.2.1.144/70/355/UCWEB Mobile*/
const Device UA1952763 = {S64444, 81, { &P14033,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaC5-03)/UC Browser8.5.0.183/50/355/UCWEB Mobile*/
const Device UA1955642 = {S64445, 32, { &P14033,&P5129,&P24671,&P18092 }};	/*Useragent - NokiaC5-03/UCWEB8.5.0.163/50/800*/
const Device UA2383513 = {S64446, 144, { &P20816,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WIKO-CINK SLIM Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2432780 = {S64447, 140, { &P23156,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XOLO A700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3118551 = {S64448, 62, { &P23156,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; XOLO A700 Build/IMM76D)*/
const Device UA2754349 = {S64449, 129, { &P20991,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ME371MG Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2974868 = {S64450, 138, { &P21264,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PadFone Infinity Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2975027 = {S64451, 145, { &P21264,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PadFone Infinity Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3093548 = {S64452, 156, { &P18442,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N7100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3093738 = {S64453, 156, { &P18442,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7100 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109251 = {S64454, 156, { &P18442,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-N719 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3111521 = {S64455, 156, { &P18442,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7105 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA54187 = {S64456, 146, { &P14146,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-pt; soft stone Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA56002 = {S64457, 145, { &P14146,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; ZTE-RACER Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018708 = {S64458, 67, { &P14146,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; ZTE-RACER Build/ERE27)*/
const Device UA2798399 = {S64459, 154, { &P19771,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ALCATEL ONE TOUCH 5035X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2872219 = {S64460, 154, { &P19771,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; ALCATEL ONE TOUCH 5035D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2879050 = {S64461, 154, { &P19771,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ALCATEL ONE TOUCH 5035E Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2799372 = {S64462, 139, { &P23186,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ZTE-Z993 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3066891 = {S64463, 141, { &P19778,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PantechP9090 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067598 = {S64464, 130, { &P23773,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8010 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2171381 = {S64465, 134, { &P21234,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT560 Build/V1.67A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2446201 = {S64466, 134, { &P21234,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; pt-br; XT560 Build/V1.66A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2592862 = {S64467, 134, { &P21234,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; pt-br; XT560 Build/V1.59B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2759769 = {S64468, 134, { &P21234,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; pt-br; XT560 Build/V1.59D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2232094 = {S64469, 90, { &P15364,&P16110,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; chromeframe/22.0.1229.94)*/
const Device UA2238188 = {S64470, 97, { &P15364,&P16110,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/22.0.1229.94)*/
const Device UA2259342 = {S64471, 132, { &P15364,&P16110,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; chromeframe/22.0.1229.94) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2354631 = {S64472, 176, { &P15364,&P16110,&P21913,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; chromeframe/22.0.1229.94)*/
const Device UA2426025 = {S64473, 166, { &P15575,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-I9100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2461327 = {S64474, 166, { &P15575,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-I9100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2486121 = {S64475, 167, { &P15575,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-I9100G Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2557102 = {S64476, 167, { &P15575,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-I9100G Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2467766 = {S64477, 140, { &P19869,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Micromax A30 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2659639 = {S64478, 145, { &P19869,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Micromax A30 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2490045 = {S64479, 160, { &P12788,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Evo 3D CDMA Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2537071 = {S64480, 159, { &P12788,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Evo 3D CDMA Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2663117 = {S64481, 158, { &P12788,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Evo 3D GSM Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2704271 = {S64482, 160, { &P12788,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Evo 3D CDMA Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2738260 = {S64483, 160, { &P12788,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Evo 3D CDMA Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2767653 = {S64484, 159, { &P12788,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Evo 3D GSM Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3094458 = {S64485, 156, { &P18283,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9300 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109345 = {S64486, 156, { &P18283,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-I9300 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3112966 = {S64487, 156, { &P18283,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SCH-I535 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3114912 = {S64488, 156, { &P18283,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SGH-T999 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126611 = {S64489, 157, { &P18283,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-I939D Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA7824 = {S64490, 159, { &P11765,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-KC910/V10d Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA9901 = {S64491, 159, { &P11765,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-KC910/V10b Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA9902 = {S64492, 159, { &P11765,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-KC910/V10c Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA9903 = {S64493, 159, { &P11765,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-KC910/V10g Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA9904 = {S64494, 133, { &P11765,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/4.0 (Vodafone/1.0/LG-KC910/V08e Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA9905 = {S64495, 159, { &P11765,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-KC910/V10h Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA20510 = {S64496, 132, { &P11765,&P17483,&P3728,&P18092 }};	/*Useragent - LG-KC910-v10e Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA20511 = {S64497, 132, { &P11765,&P17483,&P3728,&P18092 }};	/*Useragent - LG-KC910-v10b Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA20512 = {S64498, 132, { &P11765,&P17483,&P3728,&P18092 }};	/*Useragent - LG-KC910-v10h Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA21018 = {S64499, 128, { &P11765,&P17483,&P3728,&P18092 }};	/*Useragent - LG-KC910Q Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA21526 = {S64500, 129, { &P11765,&P17483,&P3728,&P18092 }};	/*Useragent - LG-KC910Qa Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA29445 = {S64501, 139, { &P11765,&P17483,&P3728,&P18092 }};	/*Useragent - LG-KC910-Orange/V10e Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA54829 = {S64502, 132, { &P11765,&P17483,&P3728,&P18092 }};	/*Useragent - LG-KC910-v10d Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA62410 = {S64503, 131, { &P11765,&P17483,&P3728,&P18092 }};	/*Useragent - LG-KC910/V10a Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA62722 = {S64504, 131, { &P11765,&P17483,&P3728,&P18092 }};	/*Useragent - LG/KC910/V10a Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA89098 = {S64505, 132, { &P11765,&P17483,&P3728,&P18092 }};	/*Useragent - LG/KC910e/V10c Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA479742 = {S64506, 76, { &P15364,&P6704,&P19772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.8.1.24) Gecko/X SeaMonkey/1.1.19*/
const Device UA1884306 = {S64507, 80, { &P15364,&P6704,&P19772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win 9x 4.90; en-GB; rv:1.8.1.6) Gecko/X SeaMonkey/1.1.4*/
const Device UA2566999 = {S64508, 137, { &P12972,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T989 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567709 = {S64509, 145, { &P12972,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-T989 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2733454 = {S64510, 138, { &P12972,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-T989 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567210 = {S64511, 140, { &P18736,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668945 = {S64512, 133, { &P18736,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2784774 = {S64513, 140, { &P12968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.0.200) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2958007 = {S64514, 133, { &P12968,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.0.200) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2897275 = {S64515, 146, { &P23799,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ONE TOUCH 4011X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2965504 = {S64516, 68, { &P23799,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ONE TOUCH 4011X Build/JRO03C)*/
const Device UA2901230 = {S64517, 152, { &P19747,&P5043,&P24678,&P18092 }};	/*Useragent - Nokia206/2.0 (03.59) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia206) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3016038 = {S64518, 156, { &P19747,&P5043,&P24678,&P18092 }};	/*Useragent - Nokia206.1/2.0 (03.58) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia206.1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2973651 = {S64519, 137, { &P18641,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-M830 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2754478 = {S64520, 145, { &P18512,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT881 Build/9.8.3O-118_USC-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2865282 = {S64521, 140, { &P23184,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; iOCEAN X7 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997583 = {S64522, 62, { &P23184,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; iOCEAN X7 Build/JOP40D)*/
const Device UA2875895 = {S64523, 73, { &P16329,&P23209,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PLAYSTATION 3 4.45) AppleWebKit/531.22.8 (KHTML, like Gecko)*/
const Device UA2945880 = {S64524, 159, { &P12968,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.0.200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA647842 = {S64525, 111, { &P17595,&P17296,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:11.0) Gecko/X Firefox/11.0,gzip(gfe) (via translate.google.com)*/
const Device UA756614 = {S64526, 87, { &P17595,&P17296,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:11.0) Gecko/X Firefox/11.0 Firefox/11.0*/
const Device UA2506282 = {S64527, 139, { &P15767,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2510317 = {S64528, 238, { &P15767,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141,Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2561417 = {S64529, 240, { &P15767,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146,Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2564212 = {S64530, 141, { &P15767,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2607397 = {S64531, 240, { &P15767,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329,Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2665809 = {S64532, 141, { &P15767,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2562526 = {S64533, 132, { &P19874,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-i705 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2897961 = {S64534, 140, { &P23687,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; LIFE PLAY Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3106763 = {S64535, 62, { &P23687,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; LIFE PLAY Build/JOP40D)*/
const Device UA2999771 = {S64536, 140, { &P24513,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530X Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3109325 = {S64537, 62, { &P24513,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-R530X Build/JZO54K)*/
const Device UA3067188 = {S64538, 145, { &P13042,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Incredible S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA1961379 = {S64539, 146, { &P20067,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Micromax A56 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031695 = {S64540, 137, { &P20067,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; A56 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2259962 = {S64541, 111, { &P15364,&P6596,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3083.2.0) AppleWebKit/537.16 (KHTML, like Gecko) Chrome/24.0.1305.4 Safari/537.16*/
const Device UA2259998 = {S64542, 109, { &P15364,&P6596,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3083.1.0) AppleWebKit/537.16 (KHTML, like Gecko) Chrome/24.0.1305.3 Safari/537.16*/
const Device UA2260002 = {S64543, 112, { &P15364,&P6596,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3020.0.2012) AppleWebKit/537.13 (KHTML, like Gecko) Chrome/24.0.1290.1 Safari/537.13*/
const Device UA2260003 = {S64544, 109, { &P15364,&P6596,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 3083.2.0) AppleWebKit/537.16 (KHTML, like Gecko) Chrome/24.0.1305.4 Safari/537.16*/
const Device UA2260049 = {S64545, 111, { &P15364,&P6596,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 3083.1.0) AppleWebKit/537.16 (KHTML, like Gecko) Chrome/24.0.1305.3 Safari/537.16*/
const Device UA2675467 = {S64546, 91, { &P17595,&P17297,&P22225,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:20.0) Gecko/X Firefox/20.0 SeaMonkey/2.17.1*/
const Device UA2896968 = {S64547, 136, { &P23667,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; JY-G4 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103404 = {S64548, 58, { &P23667,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; JY-G4 Build/JOP40D)*/
const Device UA2421663 = {S64549, 132, { &P18731,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ONDA MID Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2841166 = {S64550, 61, { &P18731,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ONDA MID Build/JRO03C)*/
const Device UA2985101 = {S64551, 139, { &P18731,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ONDA MID Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422702 = {S64552, 132, { &P15882,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P5110 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2534058 = {S64553, 131, { &P15882,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5110 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2508580 = {S64554, 43, { &P17595,&P18207,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.0.X (Intel Mac OS X 10.8.2)*/
const Device UA2530393 = {S64555, 43, { &P17595,&P18207,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.1.X (Intel Mac OS X 10.8.2)*/
const Device UA2583476 = {S64556, 43, { &P17595,&P18207,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.2.X (Intel Mac OS X 10.8.2)*/
const Device UA2584133 = {S64557, 43, { &P17595,&P18207,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.2.X (Intel Mac OS X 10.8.3)*/
const Device UA2604433 = {S64558, 43, { &P17595,&P18207,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.1.X (Intel Mac OS X 10.8.3)*/
const Device UA2739549 = {S64559, 43, { &P17595,&P18207,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.4.X (Intel Mac OS X 10.8.3)*/
const Device UA2783010 = {S64560, 43, { &P17595,&P18207,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.4.X (Intel Mac OS X 10.8.4)*/
const Device UA2796930 = {S64561, 43, { &P17595,&P18207,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.5.X (Intel Mac OS X 10.8.4)*/
const Device UA2869366 = {S64562, 43, { &P17595,&P18207,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.5.X (Intel Mac OS X 10.8.3)*/
const Device UA2892998 = {S64563, 43, { &P17595,&P18207,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.2.X (Intel Mac OS X 10.8.4)*/
const Device UA2972565 = {S64564, 43, { &P17595,&P18207,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.6.X (Intel Mac OS X 10.8.4)*/
const Device UA3083088 = {S64565, 43, { &P17595,&P18207,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.7.X (Intel Mac OS X 10.8.4)*/
const Device UA3093586 = {S64566, 43, { &P17595,&P18207,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.6.X (Intel Mac OS X 10.8.5)*/
const Device UA3095966 = {S64567, 43, { &P17595,&P18207,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.7.X (Intel Mac OS X 10.8.5)*/
const Device UA2602521 = {S64568, 136, { &P14186,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-S5830 Build/CCGH1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2852868 = {S64569, 142, { &P14186,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2961528 = {S64570, 136, { &P14186,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-S5830 Build/CCGH2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2988928 = {S64571, 138, { &P14186,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-S5830 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078576 = {S64572, 141, { &P14186,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; es-ve; GT-S5830M Build/JELLYBEAN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2974608 = {S64573, 137, { &P20975,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-AS730 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA749688 = {S64574, 146, { &P14708,&P5204,&P24667,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia5130c-2) UCBrowser8.2.0.132/69/452/UCWEB Mobile*/
const Device UA2540516 = {S64575, 73, { &P20999,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-S5292/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2681029 = {S64576, 139, { &P22396,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S5312 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3077646 = {S64577, 61, { &P22396,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S5312 Build/JZO54K)*/
const Device UA3079721 = {S64578, 140, { &P22396,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; GT-S5312B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3107051 = {S64579, 140, { &P22396,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-S5312L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3067295 = {S64580, 136, { &P22613,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E430 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA64319 = {S64581, 47, { &P15526,&P5690,&P4626,&P18092 }};	/*Useragent - HD_mini_T5555 Opera/9.7 (Windows NT 5.1; U; en)*/
const Device UA16315 = {S64582, 106, { &P16225,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) 480x800 SAMSUNG SCH-i920 PPC*/
const Device UA2753915 = {S64583, 162, { &P18304,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-I9070 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2799417 = {S64584, 163, { &P18304,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-I9070P Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2975535 = {S64585, 130, { &P12879,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7510 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023776 = {S64586, 130, { &P12879,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P7510 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2975747 = {S64587, 138, { &P22848,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT920 Build/2_310_2009) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA49275 = {S64588, 79, { &P14768,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-F160-Orange/4.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA49705 = {S64589, 80, { &P14768,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-F160(Uganda)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49907 = {S64590, 85, { &P14768,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-F160(Brazil_VIVO)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54425 = {S64591, 74, { &P14768,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-F160/1.3.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2161717 = {S64592, 92, { &P14768,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-F160(orange-20-branches)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2392300 = {S64593, 92, { &P14768,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-F160(Venezuela_Movilnet)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2641240 = {S64594, 77, { &P14768,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-F160-Free/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2806429 = {S64595, 84, { &P14768,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-F160(Rwanda_MTN)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA88391 = {S64596, 79, { &P16150,&P18110,&P3976,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Linux 2.6.22) NetFront/3.4 Kindle/2.0 (screen 600x800)*/
const Device UA89219 = {S64597, 87, { &P16150,&P18110,&P3976,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Linux 2.6.22) NetFront/3.4 Kindle/2.3 (screen 600x800; rotate)*/
const Device UA90178 = {S64598, 87, { &P16150,&P18110,&P3976,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Linux 2.6.22) NetFront/3.4 Kindle/2.5 (screen 600x800; rotate)*/
const Device UA2361100 = {S64599, 147, { &P17174,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MediaPad 10 FHD Build/HuaweiMediaPad) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2366357 = {S64600, 131, { &P23354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2582143 = {S64601, 140, { &P23354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A3 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3015610 = {S64602, 127, { &P23354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3;zh-tw; A3 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2752710 = {S64603, 159, { &P22913,&P18066,&P18062,&P18092 }};	/*Useragent - Karbonn A25 Linux/3.0.13 Android/4.0.4 Release/04.15.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2996805 = {S64604, 142, { &P22913,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; Karbonn A25 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2754337 = {S64605, 135, { &P22126,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Torque Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2125663 = {S64606, 76, { &P15364,&P18110,&P19772,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.2pre) Gecko/X SeaMonkey/1.1*/
const Device UA2785935 = {S64607, 106, { &P15364,&P18110,&P19772,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; OpenVMS COMPAQ_Professional_Workstation; en-US; rv:1.8.1.17) Gecko/X SeaMonkey/1.1.12*/
const Device UA675726 = {S64608, 128, { &P21363,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T01A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1995454 = {S64609, 128, { &P21363,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T01A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032069 = {S64610, 57, { &P21363,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; T01A Build/IML74K)*/
const Device UA2722715 = {S64611, 57, { &P21363,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; T01A Build/IMM76D)*/
const Device UA2204825 = {S64612, 81, { &P15364,&P18110,&P18523,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:15.0) Gecko/X Thunderbird/15.0.1 Lightning/1.7*/
const Device UA2580572 = {S64613, 156, { &P14714,&P5843,&P24676,&P18092 }};	/*Useragent - NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax3-02) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2599877 = {S64614, 160, { &P14714,&P5843,&P24676,&P18092 }};	/*Useragent - NokiaX3-02.5/5.0 (06.75) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax3-02.5) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2975997 = {S64615, 137, { &P20973,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LW770 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA33176 = {S64616, 46, { &P16610,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-UN270/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA99644 = {S64617, 6, { &P12280,&P17779,&P18589,&P18092 }};	/*Useragent - iPhone*/
const Device UA289127 = {S64618, 140, { &P15364,&P17816,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-us) AppleWebKit/531.2+ (KHTML, like Gecko) Version/5.0 Safari/531.2+ Ubuntu/10.04 LTS () Epiphany/2.30.2*/
const Device UA2228102 = {S64619, 59, { &P15364,&P18346,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10; rv:16.0) Gecko/X Firefox/16.0*/
const Device UA2600582 = {S64620, 134, { &P23846,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A12 Build/RheaTK) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2669029 = {S64621, 134, { &P23846,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Karbonn A12 Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2728485 = {S64622, 136, { &P20944,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T-Hub2 Build/TVA301TELBG3) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2895183 = {S64623, 156, { &P14711,&P5204,&P24678,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-01) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2946984 = {S64624, 156, { &P20705,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2965603 = {S64625, 158, { &P20705,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC_PN071 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2969642 = {S64626, 155, { &P20705,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTCONE Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3083152 = {S64627, 139, { &P20006,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30a Build/9.1.A.0.489) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3069030 = {S64628, 134, { &P15767,&P22882,&P24411,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Coast/1.0.0.00000 Mobile/11A4414e Safari/7534.48.3*/
const Device UA3083308 = {S64629, 134, { &P15767,&P22882,&P24411,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/11A465 Safari/7534.48.3*/
const Device UA3084012 = {S64630, 136, { &P15767,&P22882,&P24411,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/11A4449d Safari/7534.48.3*/
const Device UA3116566 = {S64631, 136, { &P15767,&P22882,&P24411,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/11A4449a Safari/7534.48.3*/
const Device UA3116572 = {S64632, 136, { &P15767,&P22882,&P24411,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/11A4435d Safari/7534.48.3*/
const Device UA3116597 = {S64633, 136, { &P15767,&P22882,&P24411,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/11A501 Safari/7534.48.3*/
const Device UA88334 = {S64634, 116, { &P17346,&P17131,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 2_2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Mobile/5H11*/
const Device UA94075 = {S64635, 104, { &P17346,&P17131,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 2_2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko)*/
const Device UA100876 = {S64636, 102, { &P17346,&P17131,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 2_2 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko)*/
const Device UA107145 = {S64637, 117, { &P17346,&P17131,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 2_2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Mobile/5H11a*/
const Device UA261859 = {S64638, 115, { &P17346,&P17131,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 2_2 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Mobile/5G77a*/
const Device UA2009136 = {S64639, 132, { &P21064,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A44 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2718767 = {S64640, 93, { &P13123,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5360) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2737254 = {S64641, 90, { &P13123,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; vi; GT-S5360) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2772427 = {S64642, 137, { &P17595,&P18207,&P22778,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) Version/6.0.4 Safari/8536.29.13 Sleipnir/4.3.0.1*/
const Device UA2885326 = {S64643, 133, { &P17595,&P18207,&P22778,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) Version/6.0.5 Safari/8536.30.1 Sleipnir/4.3.1*/
const Device UA3105818 = {S64644, 135, { &P17595,&P18207,&P22778,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) Version/6.0.3 Safari/8536.28.10 Sleipnir/4.3.1*/
const Device UA20126 = {S64645, 114, { &P13418,&P17483,&P3459,&P18092 }};	/*Useragent - LG-GD570AW Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA55066 = {S64646, 112, { &P13418,&P17483,&P3459,&P18092 }};	/*Useragent - LG-GD570 Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA75788 = {S64647, 111, { &P13418,&P17483,&P3459,&P18092 }};	/*Useragent - LG-GD570Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA384612 = {S64648, 44, { &P15364,&P17350,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT;en-us) Firefox/3.5.9*/
const Device UA2722135 = {S64649, 63, { &P15364,&P17350,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; de) Gecko/X Firefox/3.5 (.NET CLR 3.5)*/
const Device UA624748 = {S64650, 86, { &P15364,&P7127,&P3409,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0b; Windows NT 5.0) Gecko/X Firefox/3.0.6 TweetmemeBot*/
const Device UA633530 = {S64651, 58, { &P15364,&P18146,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; SunOS i86pc; rv:8.0) Gecko/X Firefox/8.0*/
const Device UA2566924 = {S64652, 128, { &P16976,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT300 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA3068199 = {S64653, 129, { &P22943,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; E270BSA Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA64090 = {S64654, 107, { &P12076,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-A177/A177UCIC3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA221372 = {S64655, 126, { &P20581,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; ja-jp; L-06C Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA458864 = {S64656, 139, { &P17955,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; MID1024 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA459055 = {S64657, 63, { &P17955,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3; MID1024 Build/GINGERBREAD)*/
const Device UA623138 = {S64658, 133, { &P22762,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; L-01D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA797045 = {S64659, 57, { &P22762,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; L-01D Build/GRJ90)*/
const Device UA628532 = {S64660, 58, { &P18254,&P18110,&P22938,&P18092 }};	/*Useragent - Liferea/1.8.3 (Linux; en_US.UTF-8; http://liferea.sf.net/)*/
const Device UA2203853 = {S64661, 57, { &P18254,&P18110,&P22938,&P18092 }};	/*Useragent - Liferea/1.8.8 (Linux; en_GB.utf8; http://liferea.sf.net/)*/
const Device UA2219320 = {S64662, 58, { &P18254,&P18110,&P22938,&P18092 }};	/*Useragent - Liferea/1.8.3 (Linux; en_GB.UTF-8; http://liferea.sf.net/)*/
const Device UA2232612 = {S64663, 57, { &P18254,&P18110,&P22938,&P18092 }};	/*Useragent - Liferea/1.8.9 (Linux; en_GB.utf8; http://liferea.sf.net/)*/
const Device UA2242471 = {S64664, 58, { &P18254,&P18110,&P22938,&P18092 }};	/*Useragent - Liferea/1.8.5 (Linux; en_US.UTF-8; http://liferea.sf.net/)*/
const Device UA2242483 = {S64665, 58, { &P18254,&P18110,&P22938,&P18092 }};	/*Useragent - Liferea/1.8.3 (Linux; fr_FR.UTF-8; http://liferea.sf.net/)*/
const Device UA2327237 = {S64666, 58, { &P18254,&P18110,&P22938,&P18092 }};	/*Useragent - Liferea/1.8.6 (Linux; en_US.UTF-8; http://liferea.sf.net/)*/
const Device UA2362466 = {S64667, 58, { &P18254,&P18110,&P22938,&P18092 }};	/*Useragent - Liferea/1.8.10 (Linux; en_GB.utf8; http://liferea.sf.net/)*/
const Device UA2443169 = {S64668, 58, { &P18254,&P18110,&P22938,&P18092 }};	/*Useragent - Liferea/1.8.6 (Linux; fr_FR.UTF-8; http://liferea.sf.net/)*/
const Device UA2556138 = {S64669, 59, { &P18254,&P18110,&P22938,&P18092 }};	/*Useragent - Liferea/1.8.12 (Linux; en_US.UTF-8; http://liferea.sf.net/)*/
const Device UA2588809 = {S64670, 58, { &P18254,&P18110,&P22938,&P18092 }};	/*Useragent - Liferea/1.8.3 (Linux; de_DE.UTF-8; http://liferea.sf.net/)*/
const Device UA2644016 = {S64671, 58, { &P18254,&P18110,&P22938,&P18092 }};	/*Useragent - Liferea/1.8.7 (Linux; fr_FR.UTF-8; http://liferea.sf.net/)*/
const Device UA2658362 = {S64672, 59, { &P18254,&P18110,&P22938,&P18092 }};	/*Useragent - Liferea/1.8.12 (Linux; fr_FR.UTF-8; http://liferea.sf.net/)*/
const Device UA2699890 = {S64673, 58, { &P18254,&P18110,&P22938,&P18092 }};	/*Useragent - Liferea/1.8.12 (Linux; fr_FR.utf8; http://liferea.sf.net/)*/
const Device UA1963439 = {S64674, 138, { &P18740,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-TH; OPPOU7011 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026919 = {S64675, 137, { &P18740,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOU701 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2395828 = {S64676, 126, { &P18740,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; U701 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3070057 = {S64677, 57, { &P18740,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; U701 Build/GRK39F)*/
const Device UA2228885 = {S64678, 60, { &P15364,&P18146,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; SunOS i86pc; rv:16.0) Gecko/X Firefox/16.0*/
const Device UA2404761 = {S64679, 137, { &P20437,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; FJL21 Build/V23R39X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2433956 = {S64680, 137, { &P20437,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; FJL21 Build/V27R40J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2537646 = {S64681, 137, { &P20437,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; FJL21 Build/V31R42A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2677124 = {S64682, 137, { &P20437,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; FJL21 Build/V35R45B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2773907 = {S64683, 137, { &P20437,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; FJL21 Build/V37R47A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3023579 = {S64684, 137, { &P20437,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; FJL21 Build/V39R48C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2437537 = {S64685, 159, { &P12681,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus S 4G Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2531193 = {S64686, 159, { &P12681,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus S 4G Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2561457 = {S64687, 158, { &P12681,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus S 4G Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3018745 = {S64688, 169, { &P12681,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus S 4G Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/crespo4g*/
const Device UA2455609 = {S64689, 101, { &P11639,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.89; en-us; 9000/5.2.0.89) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2455865 = {S64690, 101, { &P11639,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.93; en-us; 9000/5.2.0.93) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2481422 = {S64691, 101, { &P11639,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.64; en-us; 9000/5.2.0.64) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2553063 = {S64692, 101, { &P11639,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.95; en-us; 9000/5.2.0.95) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2595125 = {S64693, 101, { &P11639,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.33; en-us; 9000/5.2.0.33) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2653686 = {S64694, 101, { &P11639,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.57; en-us; 9000/5.2.0.57) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2695100 = {S64695, 103, { &P11639,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.0.0.248; en-us; 9000/4.0.0.248) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2763062 = {S64696, 103, { &P11639,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.0.0.155; en-us; 9000/4.0.0.155) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2777860 = {S64697, 101, { &P11639,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.76; en-us; 9000/5.2.0.76) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2778364 = {S64698, 103, { &P11639,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.0.0.247; en-us; 9000/4.0.0.247) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2791155 = {S64699, 103, { &P11639,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.0.0.206; en-us; 9000/4.0.0.206) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3118289 = {S64700, 101, { &P11639,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.2.0.41; en-us; 9000/5.2.0.41) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2800203 = {S64701, 132, { &P23218,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMP5780D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3121459 = {S64702, 61, { &P23218,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; PMP5780D Build/JRO03H)*/
const Device UA28073 = {S64703, 50, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player verizon_wwe / 1.0 / inc / 2.2*/
const Device UA57884 = {S64704, 56, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_europe / 1.0 / htc_saga / 2.3.3*/
const Device UA57907 = {S64705, 51, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player o2_uk / 1.0 / htc_saga / 2.3.3*/
const Device UA438810 = {S64706, 57, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player vodafone_uk / 1.0 / htc_saga / 2.3.5*/
const Device UA2146885 = {S64707, 56, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_europe / 1.0 / htc_saga / 2.3.5*/
const Device UA2165390 = {S64708, 48, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player tmous / 1.0 / ville / 4.0.3*/
const Device UA2308764 = {S64709, 59, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player verizon_wwe / 1.0 / htc_blissc / 2.3.4*/
const Device UA2398546 = {S64710, 54, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player cingular_us / 1.0 / evita / 4.0.4*/
const Device UA2399239 = {S64711, 57, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_europe / 1.0 / endeavoru / 4.0.4*/
const Device UA2444558 = {S64712, 54, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player cingular_us / 1.0 / evita / 4.0.3*/
const Device UA2468036 = {S64713, 57, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_europe / 1.0 / endeavoru / 4.1.1*/
const Device UA2685291 = {S64714, 58, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_europe / 1.0 / htc_primou / 4.0.3*/
const Device UA2693737 = {S64715, 48, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player tmous / 1.0 / ville / 4.0.4*/
const Device UA2696176 = {S64716, 52, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player verizon_wwe / 1.0 / dlx / 4.1.1*/
const Device UA2746670 = {S64717, 57, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player verizon_wwe / 1.0 / fireball / 4.0.4*/
const Device UA2791009 = {S64718, 59, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_asia_wwe / 1.0 / endeavoru / 4.1.1*/
const Device UA2809243 = {S64719, 60, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_asia_wwe / 1.0 / htc_primou / 4.0.3*/
const Device UA2819066 = {S64720, 53, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_europe / 1.0 / ville / 4.0.4*/
const Device UA2843447 = {S64721, 54, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_europe / 1.0 / enrc2b / 4.1.1*/
const Device UA2956134 = {S64722, 46, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc / 1.0 / m7wls / 4.1.2*/
const Device UA2956190 = {S64723, 51, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player cingular_us / 1.0 / m7 / 4.1.2*/
const Device UA2985305 = {S64724, 54, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_europe / 1.0 / protou / 4.0.4*/
const Device UA3046311 = {S64725, 54, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player cingular_us / 1.0 / evita / 4.1.1*/
const Device UA3048373 = {S64726, 45, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player tmous / 1.0 / m7 / 4.1.2*/
const Device UA3086283 = {S64727, 53, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_europe / 1.0 / ville / 4.1.1*/
const Device UA3087656 = {S64728, 50, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_europe / 1.0 / m7 / 4.1.2*/
const Device UA3117031 = {S64729, 64, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player us_cellular_wwe / 1.0 / htc_lexikon / 2.3.4*/
const Device UA3118646 = {S64730, 58, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player cingular_us / 1.0 / evitareul / 4.1.1*/
const Device UA3125838 = {S64731, 51, { &P15364,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc / 1.0 / htc_europe / 4.2.2*/
const Device UA745005 = {S64732, 186, { &P15364,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.5; en-US; rv:1.9.1b3pre) Gecko/X Mozilla/5.0 (Windows; U; Windows NT 5.1; en) AppleWebKit/526.9 (KHTML, like Gecko) Version/4.0dp1 Safari/526.8*/
const Device UA899754 = {S64733, 171, { &P19891,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I717 Build/IMM76I; CyanogenMod-h0tw1r3.r5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2186922 = {S64734, 166, { &P19891,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I717 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2325429 = {S64735, 138, { &P18636,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS930 4G Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2427801 = {S64736, 132, { &P20463,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; PMP7100D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555067 = {S64737, 131, { &P20710,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID1048 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA18529 = {S64738, 156, { &P14107,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN76/CMCC SymbianOS/9.2 Series60/3.1 Release/21.1.004 Mozilla/5.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA64893 = {S64739, 152, { &P14107,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaN76-1/31.0.014 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA87814 = {S64740, 8, { &P14107,&P5890,&P3439,&P18092 }};	/*Useragent - NOKIAN76*/
const Device UA196020 = {S64741, 133, { &P18602,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; L-07C Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1942720 = {S64742, 57, { &P18602,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; L-07C Build/GRI40)*/
const Device UA907783 = {S64743, 140, { &P15047,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SK17i Build/4.1.B.0.431) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2023533 = {S64744, 140, { &P15047,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SK17a Build/4.1.B.0.431) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2229196 = {S64745, 140, { &P15047,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Sk17i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2257158 = {S64746, 134, { &P21035,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; V8403 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2835558 = {S64747, 58, { &P21035,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; V8403 Build/GRK39F)*/
const Device UA2428346 = {S64748, 139, { &P18295,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; HTC One X Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2469828 = {S64749, 135, { &P18295,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; One X Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2533175 = {S64750, 138, { &P18295,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One X Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2535480 = {S64751, 134, { &P18295,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; One X Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2558145 = {S64752, 152, { &P18644,&P5043,&P24676,&P18092 }};	/*Useragent - Nokia112/2.0 (03.32) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia112) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA14729 = {S64753, 144, { &P14225,&P17295,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7 HTC_DesireHD-orange-LS; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA782949 = {S64754, 148, { &P16322,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-mx; ALCATEL_one_touch_906Y Build/FSR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019265 = {S64755, 147, { &P16322,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-ph; ALCATEL_one_touch_906 Build/FSR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA895249 = {S64756, 123, { &P12280,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.1*/
const Device UA902477 = {S64757, 123, { &P12280,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.2*/
const Device UA909719 = {S64758, 123, { &P12280,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.4*/
const Device UA924945 = {S64759, 123, { &P12280,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.5*/
const Device UA1969876 = {S64760, 121, { &P12280,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Flipboard/1.9.4*/
const Device UA1997214 = {S64761, 123, { &P12280,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.6*/
const Device UA2170602 = {S64762, 121, { &P12280,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.9.6*/
const Device UA2175409 = {S64763, 121, { &P12280,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Flipboard/1.9.6*/
const Device UA2202228 = {S64764, 123, { &P12280,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.7*/
const Device UA2236398 = {S64765, 121, { &P12280,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Flipboard/1.9.7*/
const Device UA2260853 = {S64766, 123, { &P12280,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.8*/
const Device UA2403993 = {S64767, 123, { &P12280,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Flipboard/1.9.8*/
const Device UA2404619 = {S64768, 121, { &P12280,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Flipboard/1.9.8*/
const Device UA2454710 = {S64769, 121, { &P12280,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.9.7*/
const Device UA2841145 = {S64770, 121, { &P12280,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.9.8*/
const Device UA1995110 = {S64771, 61, { &P15364,&P18346,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10; rv:14.0) Gecko/X Firefox/14.0.1*/
const Device UA2039625 = {S64772, 132, { &P18466,&P18110,&P19989,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.0 Safari/535.19 Silk-Accelerated=true*/
const Device UA2527540 = {S64773, 139, { &P20536,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PAP3500 DUO Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32687 = {S64774, 112, { &P12514,&P5208,&P4628,&P18092 }};	/*Useragent - POLARIS/6.01 (BREW 3.1.5; U; en-us; LG; LX265; POLARIS/6.01/WAP) MMP/2.0 profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2265975 = {S64775, 107, { &P12514,&P5208,&P4628,&P18092 }};	/*Useragent - POLARIS/6.01(BREW 3.1.5;U;en-us;LG;LX265;POLARIS/6.01/WAP;)MMP/2.0 profile/MIDP-201 Configuration /CLDC-1.1*/
const Device UA624534 = {S64776, 137, { &P22184,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; CatNova8 Build/GRI40) AppleWebKit/533.16 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA903262 = {S64777, 139, { &P21227,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Polaroid Tablet Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2646228 = {S64778, 68, { &P21227,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Polaroid Tablet Build/IML74K)*/
const Device UA2030560 = {S64779, 126, { &P21233,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2985698 = {S64780, 126, { &P21233,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; T1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2567381 = {S64781, 141, { &P18449,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST27i Build/6.1.1.B.1.54) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2605945 = {S64782, 141, { &P18449,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST27a Build/6.1.1.B.1.54) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA18466 = {S64783, 138, { &P15364,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; es-; FBW1_4 Build/MASTER) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA56008 = {S64784, 141, { &P15364,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; sdkDemo Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA67348 = {S64785, 140, { &P15364,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; SLATE8 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA67552 = {S64786, 140, { &P15364,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; KESAM7 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA88850 = {S64787, 125, { &P15364,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; hero) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA445242 = {S64788, 140, { &P15364,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; SLATE5 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA512892 = {S64789, 143, { &P15364,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; htc_bahamas Build/CRB17) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA561730 = {S64790, 144, { &P15364,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; tr-tr; SiRF Dream Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA660674 = {S64791, 153, { &P15364,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; Galaxy with MCR 1.2 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA729754 = {S64792, 142, { &P15364,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; es-us; GT-i5700 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA1868494 = {S64793, 139, { &P15364,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; 750ET Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA1898418 = {S64794, 142, { &P15364,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; AEBK08FB Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2018585 = {S64795, 146, { &P15364,&P6757,&P3525,&P18092 }};	/*Useragent - Lenovo_O1plus_TD/1.0 Ophone/1.5 (Linux; Android 1.5) Release/9.1.2009 Browser/WAP 2.0 (AppleWebKit/528.5+) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2018833 = {S64796, 140, { &P15364,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-; AEBK08FB Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2807380 = {S64797, 143, { &P15364,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5.350; de-de; m7005 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA3042182 = {S64798, 151, { &P15364,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5.1.16-RT-X.X; zh-cn; K-Touch E619 Build/AliyunOs-2012) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3042284 = {S64799, 143, { &P15364,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; htc_bahamas Build/CRB17) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA347404 = {S64800, 121, { &P15364,&P5690,&P22415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.1.1.0 Chrome/16.0.912.63 Safari/535.7*/
const Device UA370023 = {S64801, 144, { &P21213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; ru-ru; PMP3384BRU Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA682610 = {S64802, 142, { &P21213,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; PMP3384B Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2383422 = {S64803, 190, { &P18442,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/148,1.9.6.148,2012-09-07 16:07, gb*/
const Device UA2400562 = {S64804, 198, { &P18442,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I317 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/148,1.9.6.148,2012-09-07 16:07, us*/
const Device UA2457860 = {S64805, 191, { &P18442,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; SGH-I317M Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/148,1.9.6.148,2012-09-07 16:07, ca*/
const Device UA2466446 = {S64806, 190, { &P18442,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7105 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/148,1.9.6.148,2012-09-07 16:07, gb*/
const Device UA2466697 = {S64807, 190, { &P18442,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L900 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/148,1.9.6.148,2012-09-07 16:07, us*/
const Device UA2676598 = {S64808, 199, { &P18442,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, +0100, gb*/
const Device UA2709067 = {S64809, 199, { &P18442,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L900 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, -0400, us*/
const Device UA2709070 = {S64810, 200, { &P18442,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I317M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, -0400, ca*/
const Device UA2711694 = {S64811, 207, { &P18442,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, -0400, us*/
const Device UA2860572 = {S64812, 199, { &P18442,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7105 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, +0800, sg*/
const Device UA2738734 = {S64813, 140, { &P22592,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I717R Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3074115 = {S64814, 62, { &P22592,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I717R Build/JZO54K)*/
const Device UA3039198 = {S64815, 156, { &P20146,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9082 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3068206 = {S64816, 137, { &P18697,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I415 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA64717 = {S64817, 103, { &P12513,&P17485,&P4223,&P18092 }};	/*Useragent - SonyEricssonW20/R7DA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2*/
const Device UA241982 = {S64818, 23, { &P15364,&P17017,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 Firefox/2.0*/
const Device UA502171 = {S64819, 11, { &P15364,&P17017,&P17202,&P18092 }};	/*Useragent - Firefox/2.0*/
const Device UA760267 = {S64820, 36, { &P15364,&P17017,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (AS) Gecko/X Firefox/2.0*/
const Device UA803334 = {S64821, 73, { &P15364,&P17017,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (BeOS; U; BeOS BePC; en-US; rv:1.8.1b2) Gecko/X Firefox/2.0b2*/
const Device UA1839838 = {S64822, 69, { &P15364,&P17017,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X; 0; 000 000 00 0 X; 00000; X) Gecko/X Firefox/2.0.0.20*/
const Device UA1857454 = {S64823, 40, { &P15364,&P17017,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (en) Gecko/X Firefox/2.0.0.4*/
const Device UA1877074 = {S64824, 74, { &P15364,&P17017,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; U; Warp 4.5; en-US; rv:1.8.1.9) Gecko/X Firefox/2.0.0.9*/
const Device UA2348219 = {S64825, 44, { &P19754,&P7472,&P3534,&P18092 }};	/*Useragent - ALCATEL-OT510A/382 ObigoInternetBrowser/Q05A*/
const Device UA414753 = {S64826, 158, { &P15364,&P7133,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 6.1; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)*/
const Device UA3070002 = {S64827, 87, { &P15364,&P7133,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 6.1; SLCC2; .NET CLR 2.0.50727; .NET4.0C)*/
const Device UA799481 = {S64828, 73, { &P19952,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-C3310/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2956862 = {S64829, 74, { &P19952,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-C3310R/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA923837 = {S64830, 139, { &P15364,&P19822,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.11 (KHTML, like Gecko) Ubuntu/12.10 Chromium/20.0.1132.47 Chrome/20.0.1132.47 Safari/536.11*/
const Device UA1992230 = {S64831, 137, { &P15364,&P19822,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.11 (KHTML, like Gecko) Ubuntu/12.10 Chromium/20.0.1132.47 Chrome/20.0.1132.47 Safari/536.11*/
const Device UA2203972 = {S64832, 160, { &P18283,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03R; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2233603 = {S64833, 160, { &P18283,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I535 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2234191 = {S64834, 160, { &P18283,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T999 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2234836 = {S64835, 160, { &P18283,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03R; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2403929 = {S64836, 160, { &P18283,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2403121 = {S64837, 142, { &P20886,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Android T3i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2487227 = {S64838, 143, { &P20886,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Vandroid T3i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2203722 = {S64839, 75, { &P15364,&P18118,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux x86_64; U; Ubuntu; fr) Presto/2.10.289 Version/12.01*/
const Device UA2407228 = {S64840, 155, { &P18466,&P18066,&P19816,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2561201 = {S64841, 134, { &P21546,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; V972 Core4 Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2580812 = {S64842, 141, { &P21546,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; V972 Core4 Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2792153 = {S64843, 63, { &P21546,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; V972 Core4 Build/MASTER)*/
const Device UA2566615 = {S64844, 127, { &P18248,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A700 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2606767 = {S64845, 140, { &P22756,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C1604 Build/11.1.A.0.68) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2661686 = {S64846, 91, { &P12004,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian95) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2699129 = {S64847, 95, { &P12004,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian95_8gb) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2708740 = {S64848, 92, { &P12004,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN95) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2709224 = {S64849, 96, { &P12004,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN95 8GB) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2754352 = {S64850, 155, { &P18113,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA825961 = {S64851, 118, { &P15364,&P16110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA878171 = {S64852, 118, { &P15364,&P16110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA2268114 = {S64853, 118, { &P15364,&P16110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5*/
const Device UA2299623 = {S64854, 120, { &P15364,&P16110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27*/
const Device UA2409006 = {S64855, 112, { &P15364,&P16110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2723615 = {S64856, 118, { &P15364,&P16110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/533.17.8 (KHTML, like Gecko) Version/5.0.1 Safari/533.17.8*/
const Device UA2791235 = {S64857, 106, { &P15364,&P16110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA2983310 = {S64858, 106, { &P15364,&P16110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4*/
const Device UA2985204 = {S64859, 102, { &P15364,&P16110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.27.1 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2224084 = {S64860, 138, { &P22097,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTabS2110AF Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2475760 = {S64861, 138, { &P22097,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTabS2110AH Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2480887 = {S64862, 60, { &P18558,&P20706,&P18517,&P18092 }};	/*Useragent - Nokia309/2.0 (05.85) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2403468 = {S64863, 137, { &P19917,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P765 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2457755 = {S64864, 151, { &P15785,&P5843,&P24673,&P18092 }};	/*Useragent - Nokia201/2.0 (11.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia201) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2425130 = {S64865, 135, { &P17172,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Transformer Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2436127 = {S64866, 141, { &P17172,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Transformer TF101 Build/JB_MR1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2536321 = {S64867, 140, { &P17172,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Transformer TF101 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2857824 = {S64868, 134, { &P17172,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Transformer Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2483321 = {S64869, 136, { &P23946,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; I7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2749956 = {S64870, 136, { &P23946,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; I7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2793743 = {S64871, 136, { &P23946,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; es-es; I7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2845864 = {S64872, 58, { &P23946,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; I7100 Build/JRO03C)*/
const Device UA2878487 = {S64873, 136, { &P23946,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-gb; I7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2988334 = {S64874, 136, { &P23946,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; de-de; I7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3043980 = {S64875, 144, { &P23946,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; HKPHONE-I7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2581003 = {S64876, 65, { &P15364,&P17350,&P4808,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; PPC; Win64; IA64; Trident/5.0)*/
const Device UA2792618 = {S64877, 59, { &P15364,&P17350,&P4808,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; PPC; WOW64; Trident/4.0)*/
const Device UA2852075 = {S64878, 62, { &P15364,&P17350,&P4808,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; PPC; .NET CLR; Trident/1.0)*/
const Device UA2797404 = {S64879, 74, { &P17595,&P21721,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:21.0) Gecko/X Firefox/21.0*/
const Device UA3027365 = {S64880, 135, { &P18283,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-I9300 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3027956 = {S64881, 135, { &P18283,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SCH-I535 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3029077 = {S64882, 135, { &P18283,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SGH-T999 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3080461 = {S64883, 145, { &P16263,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI MediaPad Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080934 = {S64884, 145, { &P16263,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MediaPad 7 Lite Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094159 = {S64885, 146, { &P16263,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MediaPad 10 LINK Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094971 = {S64886, 141, { &P16263,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SpringBoard Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA90689 = {S64887, 128, { &P13105,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; G100W Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2028131 = {S64888, 141, { &P24025,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U8812D Build/HuaweiU8812D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2468855 = {S64889, 160, { &P14969,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Desire S Build/JOP40D; CyanogenMod-CM-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2581232 = {S64890, 156, { &P14969,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Desire S Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2748699 = {S64891, 157, { &P14969,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Desire S Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892269 = {S64892, 159, { &P14969,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Desire S Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2974639 = {S64893, 136, { &P21794,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9152 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3066227 = {S64894, 141, { &P19842,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA150891 = {S64895, 143, { &P15364,&P17817,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Ubuntu/10.10 Chromium/7.0.517.44 Chrome/7.0.517.44 Safari/534.7*/
const Device UA151074 = {S64896, 141, { &P15364,&P17817,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Ubuntu/10.10 Chromium/7.0.517.44 Chrome/7.0.517.44 Safari/534.7*/
const Device UA2032712 = {S64897, 152, { &P13505,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB525 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2339497 = {S64898, 152, { &P13505,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB525 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658196 = {S64899, 137, { &P22651,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E435 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2704484 = {S64900, 138, { &P22651,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E435f Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2878919 = {S64901, 138, { &P22651,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E435g Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2753891 = {S64902, 137, { &P19870,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-L710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879683 = {S64903, 117, { &P17595,&P17297,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.43.58 (KHTML, like Gecko) Version/6.1 Safari/537.43.58*/
const Device UA2972546 = {S64904, 115, { &P17595,&P17297,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.48.2 (KHTML, like Gecko) Version/6.1 Safari/537.48.2*/
const Device UA3001414 = {S64905, 115, { &P17595,&P17297,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.46.3 (KHTML, like Gecko) Version/6.1 Safari/537.46.3*/
const Device UA3028198 = {S64906, 115, { &P17595,&P17297,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.51.3 (KHTML, like Gecko) Version/6.1 Safari/537.51.3*/
const Device UA3030447 = {S64907, 115, { &P17595,&P17297,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.54.1 (KHTML, like Gecko) Version/6.1 Safari/537.54.1*/
const Device UA3039372 = {S64908, 111, { &P17595,&P17297,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.59 (KHTML, like Gecko) Version/6.1 Safari/537.59*/
const Device UA3080547 = {S64909, 111, { &P17595,&P17297,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.66 (KHTML, like Gecko) Version/6.1 Safari/537.66*/
const Device UA3095662 = {S64910, 111, { &P17595,&P17297,&P22266,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.71 (KHTML, like Gecko) Version/6.1 Safari/537.71*/
const Device UA3080981 = {S64911, 143, { &P18009,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTCEVODesign4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3082801 = {S64912, 136, { &P18009,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PH44100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067918 = {S64913, 135, { &P24102,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SM-C101 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA195622 = {S64914, 140, { &P14202,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.9; en-us; X8 Build/HACKDROID Floyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA330448 = {S64915, 135, { &P14202,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.9; en-us; X8 Build/FroyoBread) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370812 = {S64916, 140, { &P14202,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.9; hr-hr; XPERIA X8 Build/FroyoPro) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA558348 = {S64917, 132, { &P14202,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.9; en-us; X8 Build/unknown) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA562539 = {S64918, 139, { &P14202,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.5; en-us; X8 Build/Floyo by Racht) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA653621 = {S64919, 130, { &P14202,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; X8 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA879115 = {S64920, 131, { &P14202,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; X8 Build/v.0.20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1867620 = {S64921, 128, { &P14202,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; X8 Build/RC1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1875604 = {S64922, 134, { &P14202,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; X8 Build/1.0.0-RC1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018610 = {S64923, 130, { &P14202,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; X8 Build/1.0.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020895 = {S64924, 130, { &P14202,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.5; es-es; X8 Build/Floyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028694 = {S64925, 137, { &P14202,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.9; en-us; XPERIA X8 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2412499 = {S64926, 127, { &P14202,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; X8 Build/v5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA681349 = {S64927, 200, { &P20027,&P22784,&P18373,&P18092 }};	/*Useragent - MicromaxX650 ASTRO36_TD/v3 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA2490447 = {S64928, 130, { &P23899,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MT8000 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2512854 = {S64929, 171, { &P18672,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3020457 = {S64930, 175, { &P18672,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5302 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA43562 = {S64931, 52, { &P13865,&P6476,&P3487,&P18092 }};	/*Useragent - LGE-VM101-C/1.0 UP.Browser/7.2.7.2.543 (GUI) MMP/2.0*/
const Device UA45238 = {S64932, 50, { &P13865,&P6476,&P3487,&P18092 }};	/*Useragent - LGE-VM101/1.0 UP.Browser/7.2.7.2.543 (GUI) MMP/2.0*/
const Device UA490897 = {S64933, 96, { &P15364,&P17815,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.7) Gecko/X Ubuntu/9.04 (jaunty) Shiretoko/3.5.7*/
const Device UA2126914 = {S64934, 98, { &P15364,&P17815,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1.1) Gecko/X Ubuntu/9.04 (jaunty) Shiretoko/3.5.1*/
const Device UA2431285 = {S64935, 223, { &P15649,&P5096,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2557538 = {S64936, 168, { &P18442,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7105 Build/JOP40D; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2562372 = {S64937, 167, { &P18442,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7105 Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936700 = {S64938, 167, { &P18442,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L900 Build/JDQ39; LiquidSmooth-Liquid-JB-v2.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2944803 = {S64939, 173, { &P18442,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7105 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.8/t0lte*/
const Device UA3078815 = {S64940, 173, { &P18442,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-N7100 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/n7100*/
const Device UA3107752 = {S64941, 172, { &P18442,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L900 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/l900*/
const Device UA2756734 = {S64942, 132, { &P15767,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.1m*/
const Device UA2770857 = {S64943, 132, { &P15767,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.2m*/
const Device UA2869727 = {S64944, 132, { &P15767,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.4.2m*/
const Device UA2884018 = {S64945, 132, { &P15767,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.3m*/
const Device UA2899375 = {S64946, 132, { &P15767,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.4m*/
const Device UA3013135 = {S64947, 130, { &P15767,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/7534.48.3 Sleipnir/2.4.4m*/
const Device UA3016165 = {S64948, 132, { &P15767,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.4.4m*/
const Device UA3037436 = {S64949, 132, { &P15767,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.5m*/
const Device UA3067375 = {S64950, 137, { &P18316,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081626 = {S64951, 138, { &P18316,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160LV Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083082 = {S64952, 137, { &P18316,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083257 = {S64953, 137, { &P18316,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110341 = {S64954, 136, { &P18316,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA24391 = {S64955, 73, { &P15685,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-C3752/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA131128 = {S64956, 58, { &P15364,&P6704,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows 98; U; en) Presto/2.2.15 Version/10.10*/
const Device UA2139969 = {S64957, 77, { &P18095,&P18094,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (S60; SymbOS; Opera Mobi/SYB-X; U; da) Presto/2.7.81 Version/11.00*/
const Device UA633391 = {S64958, 64, { &P15364,&P7127,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.10.229 Version/11.62*/
const Device UA2176807 = {S64959, 82, { &P17595,&P18207,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; en) Presto/2.10.289 Version/12.01*/
const Device UA2236205 = {S64960, 82, { &P17595,&P18207,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; en) Presto/2.10.289 Version/12.00*/
const Device UA1993631 = {S64961, 133, { &P22485,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Newsmy T3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2403358 = {S64962, 133, { &P22485,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Newsmy T3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2464593 = {S64963, 126, { &P22485,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2423782 = {S64964, 42, { &P12193,&P5129,&P24673,&P18092 }};	/*Useragent - NokiaC5-00UCBrowser/8.7.1.234/28/444/UCWEB*/
const Device UA2426989 = {S64965, 44, { &P12193,&P5129,&P24673,&P18092 }};	/*Useragent - NokiaC5-00.2UCBrowser/8.7.1.234/28/444/UCWEB*/
const Device UA2403644 = {S64966, 98, { &P15364,&P16110,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.1.8244(Windows;Microsoft Windows (6.2.9200.0);en;kml:2.2;client:Free;type:default)*/
const Device UA2437327 = {S64967, 98, { &P15364,&P16110,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.2.8415(Windows;Microsoft Windows (6.2.9200.0);en;kml:2.2;client:Free;type:default)*/
const Device UA2506656 = {S64968, 98, { &P15364,&P16110,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.2.8415(Windows;Microsoft Windows (6.2.9200.0);de;kml:2.2;client:Free;type:default)*/
const Device UA2570283 = {S64969, 98, { &P15364,&P16110,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.2.9200.0);en;kml:2.2;client:Free;type:default)*/
const Device UA2595357 = {S64970, 98, { &P15364,&P16110,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.1.8244(Windows;Microsoft Windows (6.2.9200.0);de;kml:2.2;client:Free;type:default)*/
const Device UA2603680 = {S64971, 98, { &P15364,&P16110,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.2.9200.0);de;kml:2.2;client:Free;type:default)*/
const Device UA2705077 = {S64972, 98, { &P15364,&P16110,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.2.9200.0);fr;kml:2.2;client:Free;type:default)*/
const Device UA2750019 = {S64973, 98, { &P15364,&P16110,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Windows;Microsoft Windows (6.2.9200.0);nl;kml:2.2;client:Free;type:default)*/
const Device UA2670757 = {S64974, 138, { &P19878,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I497 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3066223 = {S64975, 137, { &P22958,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-US780 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA50290 = {S64976, 127, { &P13425,&P17603,&P4732,&P18092 }};	/*Useragent - PANTECH-C790/JAUS08312009; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.14866; U; en-US) Opera 9.50*/
const Device UA445569 = {S64977, 117, { &P13425,&P17603,&P4732,&P18092 }};	/*Useragent - PANTECH-C790/JAUSX; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.14866; U; es) Opera 9.50*/
const Device UA311925 = {S64978, 201, { &P15364,&P5690,&P21928,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322; InfoPath.2; .NET4.0C; .NET4.0E; MS-RTC LM 8) chromeframe/6.0.472.62*/
const Device UA635808 = {S64979, 169, { &P15364,&P5690,&P21928,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729) chromeframe/6.0.472.63*/
const Device UA734900 = {S64980, 129, { &P15364,&P5690,&P21928,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.1) chromeframe/6.0.472.63*/
const Device UA470782 = {S64981, 102, { &P15364,&P17350,&P3409,&P18092 }};	/*Useragent - Mozilla/6.0 (Windows; U; Windows NT 7.0; en-US; rv:1.9.0.8) Gecko/X Firefox/3.0.9 (.NET CLR 3.5.30729)*/
const Device UA783519 = {S64982, 72, { &P15364,&P17350,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows;U;Windows NT;pl;rv:1.9.0.12) Gecko/X Firefox/3.0.12*/
const Device UA2144294 = {S64983, 83, { &P15364,&P17350,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT based; en-US; rv:1.9.0.5) Gecko/X Firefox/3.0.5*/
const Device UA2146009 = {S64984, 81, { &P15364,&P17350,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 7.0; en-US; rv:1.9.0.6) Gecko/X Firefox/3.0.6*/
const Device UA2683693 = {S64985, 92, { &P15364,&P17350,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en) AppleWebKit/526.9+ (KHTML, like Gecko) FoozKids/1.0 Firefox/3.0*/
const Device UA908797 = {S64986, 78, { &P11945,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; gt-s5233s) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2373911 = {S64987, 75, { &P11945,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; vi; gt-s5233s) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2513305 = {S64988, 114, { &P12897,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-00/03.82; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2566748 = {S64989, 130, { &P12879,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7510 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2584094 = {S64990, 131, { &P12879,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P7510 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2708288 = {S64991, 152, { &P18506,&P5043,&P24677,&P18092 }};	/*Useragent - Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia305) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA61787 = {S64992, 35, { &P14329,&P17017,&P4924,&P18092 }};	/*Useragent - HUAWEI-M750/001.00 ACS-NetFront/3.2*/
const Device UA89115 = {S64993, 37, { &P15364,&P17350,&P3692,&P18092 }};	/*Useragent - WinWAP/4.2 (Win32) WinWAP-X/4.2.0.290*/
const Device UA259502 = {S64994, 31, { &P15364,&P17350,&P3692,&P18092 }};	/*Useragent - WinWAP-SPBE/1.3 (1.3.0.2;Win32)*/
const Device UA262862 = {S64995, 51, { &P15364,&P17350,&P3692,&P18092 }};	/*Useragent - WinWAP/3.x (3.x.x.xx; Win32) (Google WAP Proxy/1.0)*/
const Device UA698908 = {S64996, 140, { &P21195,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; MediaPad Build/orange_Tahiti-LS) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2843047 = {S64997, 147, { &P21195,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; HUAWEI MediaPad Build/orange_Tahiti-LS) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2601355 = {S64998, 92, { &P15364,&P7133,&P21808,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.19pre) Gecko/X Prism zdesktop/7.2.2*/
const Device UA2422351 = {S64999, 121, { &P12707,&P5055,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; Nokia701) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile*/
const Device UA2483291 = {S65000, 100, { &P12707,&P5055,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia701) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2655543 = {S65001, 40, { &P12707,&P5055,&P24673,&P18092 }};	/*Useragent - Nokia701UCBrowser/8.7.1.234/51/355/UCWEB*/
const Device UA2754721 = {S65002, 137, { &P18459,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS950 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869463 = {S65003, 137, { &P19776,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1;en-us; Lenovo_K900/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2.1 Mobile Safari/534.30*/
const Device UA2943251 = {S65004, 141, { &P19776,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1;en-us; Lenovo_K900_ROW/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2.1 Mobile Safari/534.30*/
const Device UA2955352 = {S65005, 142, { &P19776,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Lenovo K900 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958618 = {S65006, 146, { &P19776,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Lenovo K900_ROW Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3077864 = {S65007, 68, { &P19776,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; Lenovo K900_ROW Build/JOP40D)*/
const Device UA3126429 = {S65008, 136, { &P19776,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2;zh-cn; Lenovo_K900/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2.2 Mobile Safari/534.30*/
const Device UA3067185 = {S65009, 141, { &P23505,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Vodafone 875 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA160798 = {S65010, 145, { &P15364,&P17816,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Ubuntu/10.04 Chromium/8.0.552.224 Chrome/8.0.552.224 Safari/534.10*/
const Device UA168310 = {S65011, 147, { &P15364,&P17816,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Ubuntu/10.04 Chromium/8.0.552.237 Chrome/8.0.552.237 Safari/534.10*/
const Device UA2389086 = {S65012, 153, { &P12059,&P5204,&P24673,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia2690) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2406380 = {S65013, 155, { &P18680,&P18066,&P19804,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2486440 = {S65014, 147, { &P19863,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2666818 = {S65015, 140, { &P20926,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; AT-AS40SE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2739975 = {S65016, 157, { &P20926,&P18066,&P18062,&P18092 }};	/*Useragent - AT-AS40SE Linux/3.0.13 Android/4.0.4 Release/04.10.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2709496 = {S65017, 137, { &P22679,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P710 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2756003 = {S65018, 136, { &P18243,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P880 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA6429 = {S65019, 111, { &P12458,&P17483,&P3459,&P18092 }};	/*Useragent - LG-GS505Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA44339 = {S65020, 112, { &P12458,&P17483,&P3459,&P18092 }};	/*Useragent - LG-GS505 Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63252 = {S65021, 47, { &P13896,&P17484,&P5001,&P18092 }};	/*Useragent - SCH-U350/1.0 NetFront/3.0.22.2.18 (GUI) MMP/2.0*/
const Device UA2016145 = {S65022, 139, { &P17595,&P17296,&P22376,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.19 (KHTML, like Gecko) Maxthon/1.0.3.1000 Chrome/18.0.1025.142 Safari/535.19*/
const Device UA2232359 = {S65023, 136, { &P17595,&P17296,&P22376,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/1.0.4.2000 Chrome/21.0.1180.75 Safari/537.1*/
const Device UA2427359 = {S65024, 137, { &P20178,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; SmartPad970s2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2433296 = {S65025, 139, { &P20178,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; SmartPad970s23G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3025323 = {S65026, 160, { &P14708,&P5204,&P24679,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5130c-2) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3059315 = {S65027, 154, { &P14708,&P5204,&P24679,&P18092 }};	/*Useragent - Nokia5130/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5130) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA62613 = {S65028, 118, { &P14427,&P5208,&P3534,&P18092 }};	/*Useragent - TELECA-/2.0 (BREW 3.1.5; U; EN-US; SAMSUNG; SPH-M800; Teleca/Q05A/INT) MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA632713 = {S65029, 44, { &P15767,&P18117,&P17470,&P18092 }};	/*Useragent - Facebook 4110.0 (iPad; iPhone OS 5.1; en_US)*/
const Device UA2403430 = {S65030, 49, { &P15767,&P18117,&P17470,&P18092 }};	/*Useragent - FeeddlerRSS 1.12.5 (iPad; iPhone OS 5.1.1; en_US)*/
const Device UA709790 = {S65031, 139, { &P21197,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; TQ150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2815911 = {S65032, 139, { &P21197,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; TQ150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, como Gecko) versi?n/4.0 Mobile Safari/533.1*/
const Device UA2737334 = {S65033, 132, { &P21640,&P17979,&P18062,&P18092 }};	/*Useragent - samsung-mini7100/1.0 Linux/2.6.35.7 Android/4.0.4 Release/03.25.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2753881 = {S65034, 139, { &P18719,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C1505 Build/11.3.A.0.47) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3028112 = {S65035, 161, { &P15575,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3058627 = {S65036, 161, { &P15575,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3065065 = {S65037, 161, { &P15575,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3080229 = {S65038, 162, { &P15575,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9100G Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA119183 = {S65039, 141, { &P15364,&P5690,&P22398,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Comodo_Dragon/4.1.1.11 Chrome/4.1.249.1042 Safari/532.5*/
const Device UA2267385 = {S65040, 141, { &P15364,&P5690,&P22398,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Comodo_Dragon/4.1.1.12 Chrome/4.1.249.1042 Safari/532.5*/
const Device UA138425 = {S65041, 43, { &P15364,&P6704,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows 98; U; en) Presto/2.1.1*/
const Device UA417903 = {S65042, 43, { &P15364,&P6704,&P18337,&P18092 }};	/*Useragent - Opera/9.60 (Windows 98; U; en) Presto/2.1.1*/
const Device UA797713 = {S65043, 43, { &P15364,&P6704,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows 98; U; en) Presto/2.1.1*/
const Device UA797802 = {S65044, 43, { &P15364,&P6704,&P18337,&P18092 }};	/*Useragent - Opera/9.62 (Windows 98; U; en) Presto/2.1.1*/
const Device UA1969889 = {S65045, 128, { &P21235,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T708 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2183559 = {S65046, 132, { &P21202,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-P500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2738754 = {S65047, 140, { &P22591,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I717D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049544 = {S65048, 62, { &P22591,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I717D Build/JZO54K)*/
const Device UA14326 = {S65049, 61, { &P11949,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia6131/2.0 (06.10) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA14329 = {S65050, 64, { &P11949,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia6131NFC/2.0 (05.12) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20351 = {S65051, 98, { &P15077,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-SGH-T259/1.0 SHP/VPP/R5 NetFront/4.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA341074 = {S65052, 195, { &P15364,&P7133,&P24663,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/50/354/UCWEB*/
const Device UA573585 = {S65053, 195, { &P15364,&P7133,&P24663,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/50/351/UCWEB*/
const Device UA574179 = {S65054, 195, { &P15364,&P7133,&P24663,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/50/355/UCWEB*/
const Device UA574951 = {S65055, 195, { &P15364,&P7133,&P24663,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/28/355/UCWEB*/
const Device UA580032 = {S65056, 195, { &P15364,&P7133,&P24663,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.1.120/27/352/UCWEB*/
const Device UA681042 = {S65057, 195, { &P15364,&P7133,&P24663,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/28/354/UCWEB*/
const Device UA684336 = {S65058, 195, { &P15364,&P7133,&P24663,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/27/351/UCWEB*/
const Device UA694751 = {S65059, 195, { &P15364,&P7133,&P24663,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.1.120/27/351/UCWEB*/
const Device UA694893 = {S65060, 195, { &P15364,&P7133,&P24663,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/27/354/UCWEB*/
const Device UA701834 = {S65061, 195, { &P15364,&P7133,&P24663,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/27/352/UCWEB*/
const Device UA727406 = {S65062, 195, { &P15364,&P7133,&P24663,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.1.120/27/354/UCWEB*/
const Device UA2302150 = {S65063, 195, { &P15364,&P7133,&P24663,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/28/352/UCWEB*/
const Device UA2371068 = {S65064, 195, { &P15364,&P7133,&P24663,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.1.120/27/401/UCWEB*/
const Device UA2758782 = {S65065, 195, { &P15364,&P7133,&P24663,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.1.120/27/402/UCWEB*/
const Device UA2810820 = {S65066, 195, { &P15364,&P7133,&P24663,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.1.120/26/351/UCWEB*/
const Device UA584383 = {S65067, 134, { &P18684,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS14SH Build/SC100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA614309 = {S65068, 134, { &P18684,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS14SH Build/S3300) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627474 = {S65069, 134, { &P18684,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS14SH Build/S2200) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995420 = {S65070, 134, { &P18684,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS14SH Build/S6290) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2231051 = {S65071, 134, { &P18684,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS14SH Build/S9100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2669100 = {S65072, 134, { &P18684,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS14SH Build/S3060) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2422152 = {S65073, 119, { &P14984,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5250) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2505168 = {S65074, 41, { &P14984,&P5129,&P24673,&P18092 }};	/*Useragent - Nokia5250UCBrowser/8.7.1.234/50/444/UCWEB*/
const Device UA2842777 = {S65075, 101, { &P14984,&P5129,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5250) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2974496 = {S65076, 136, { &P22871,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9295 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3096284 = {S65077, 157, { &P15575,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3115508 = {S65078, 158, { &P15575,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3130695 = {S65079, 158, { &P15575,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA237874 = {S65080, 34, { &P15364,&P7127,&P22484,&P18092 }};	/*Useragent - Opera/9.10 (Windows NT 5.0; U; ru)*/
const Device UA877560 = {S65081, 34, { &P15364,&P7127,&P22484,&P18092 }};	/*Useragent - Opera/9.12 (Windows NT 5.0; U; ru)*/
const Device UA908183 = {S65082, 61, { &P15364,&P18346,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10; rv:13.0) Gecko/X Firefox/13.0.1*/
const Device UA913458 = {S65083, 124, { &P15364,&P7133,&P22433,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11 Comodo_Dragon/20.1.1.0*/
const Device UA913460 = {S65084, 131, { &P15364,&P7133,&P22433,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11 Comodo_Dragon/20.1.1.0*/
const Device UA2257550 = {S65085, 129, { &P20940,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; T-10B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2557643 = {S65086, 146, { &P22998,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TECNO_N7 Build/TECNOTECNO_N7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3046323 = {S65087, 68, { &P22998,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; TECNO_N7 Build/TECNOTECNO_N7)*/
const Device UA2706893 = {S65088, 156, { &P14669,&P5843,&P24677,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-00) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2893403 = {S65089, 156, { &P12897,&P5843,&P24678,&P18092 }};	/*Useragent - NokiaC2-00/2.0 (03.82) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-00) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2975550 = {S65090, 130, { &P23773,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8010 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3066531 = {S65091, 136, { &P19911,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E970 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA354574 = {S65092, 37, { &P17968,&P6437,&P16179,&P18092 }};	/*Useragent - stagefright/1.1 (Linux;Android 3.2.4)*/
const Device UA379106 = {S65093, 35, { &P17968,&P6437,&P16179,&P18092 }};	/*Useragent - stagefright/1.1 (Linux;Android 3.2)*/
const Device UA573975 = {S65094, 37, { &P17968,&P6437,&P16179,&P18092 }};	/*Useragent - stagefright/1.1 (Linux;Android 3.2.1)*/
const Device UA634155 = {S65095, 125, { &P17968,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; FAR7 Build/F0001) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1932128 = {S65096, 129, { &P17968,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; eeepc Build/HLK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2020648 = {S65097, 37, { &P17968,&P6437,&P16179,&P18092 }};	/*Useragent - stagefright/1.1 (Linux;Android 3.2.2)*/
const Device UA2026449 = {S65098, 145, { &P17968,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-nz; CORE_Speed_Z4 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030331 = {S65099, 37, { &P17968,&P6437,&P16179,&P18092 }};	/*Useragent - stagefright/1.1 (Linux;Android 3.2.6)*/
const Device UA2033959 = {S65100, 127, { &P17968,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; FAR7 Build/V04R24A) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2034508 = {S65101, 127, { &P17968,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; FAR7 Build/V03R22B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2400872 = {S65102, 100, { &P17968,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Android 3.2; en-us) AppleWebKit/1.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.1*/
const Device UA2819325 = {S65103, 84, { &P17968,&P6437,&P16179,&P18092 }};	/*Useragent - stagefright/1.1 (Linux;Android 3.2.1) Model Number,V55 Software Version,V55V1.1.0B12*/
const Device UA2042123 = {S65104, 134, { &P17595,&P17297,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1*/
const Device UA2202962 = {S65105, 134, { &P17595,&P17297,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1*/
const Device UA2485542 = {S65106, 140, { &P23381,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A5Classic Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2490738 = {S65107, 155, { &P18680,&P18066,&P19837,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2712934 = {S65108, 87, { &P18088,&P17017,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2714644 = {S65109, 87, { &P18088,&P17017,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2738006 = {S65110, 88, { &P18088,&P17017,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mstar) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2751464 = {S65111, 90, { &P18088,&P17017,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; generic) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2751638 = {S65112, 88, { &P18088,&P17017,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; opera) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2768423 = {S65113, 88, { &P18088,&P17017,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; spice) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2768502 = {S65114, 90, { &P18088,&P17017,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mozilla) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2773431 = {S65115, 87, { &P18088,&P17017,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2868224 = {S65116, 87, { &P18088,&P17017,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2869314 = {S65117, 88, { &P18088,&P17017,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mstar) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2870360 = {S65118, 87, { &P18088,&P17017,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/9.0.0.295 U2/1.0.0 Mobile*/
const Device UA2872181 = {S65119, 90, { &P18088,&P17017,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; generic) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2878690 = {S65120, 84, { &P18088,&P17017,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; vi; maui) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2879247 = {S65121, 95, { &P18088,&P17017,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; jbed-fastbcc) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA3049219 = {S65122, 95, { &P18088,&P17017,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui browser) U2/1.0.0 UCBrowser/9.0.0.295 U2/1.0.0 Mobile*/
const Device UA3087752 = {S65123, 84, { &P18088,&P17017,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; id; maui) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2977090 = {S65124, 164, { &P24061,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-S7270 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3064005 = {S65125, 165, { &P24061,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; th-th; SAMSUNG GT-S7270L Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA367426 = {S65126, 99, { &P18254,&P18110,&P18202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux; C -) AppleWebKit/532.4 (KHTML, like Gecko) PhantomJS/1.4.1 Safari/532.4*/
const Device UA848198 = {S65127, 104, { &P18254,&P18110,&P18202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; C) AppleWebKit/533.3 (KHTML, like Gecko) PhantomJS/1.4.1 Safari/533.3*/
const Device UA2974393 = {S65128, 98, { &P18254,&P18110,&P18202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.4.0 Safari/534.34*/
const Device UA2236232 = {S65129, 45, { &P15364,&P18570,&P22898,&P18092 }};	/*Useragent - Mercury/7.0.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2360738 = {S65130, 182, { &P16074,&P5768,&P24672,&P18092 }};	/*Useragent - Nokia3110c/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia3110c) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2569820 = {S65131, 91, { &P17595,&P18207,&P21791,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:19.0) Gecko/X Firefox/19.0 SeaMonkey/2.16.2*/
const Device UA2772759 = {S65132, 157, { &P16356,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2863830 = {S65133, 158, { &P16356,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.53463*/
const Device UA2749919 = {S65134, 143, { &P23220,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; PMP7280C Build/PMP7280C_X_v1.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3001037 = {S65135, 143, { &P23220,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; PMP7280C Build/PMP7280C_X_v1.0.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3072905 = {S65136, 72, { &P23220,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; PMP7280C Build/PMP7280C_X_v1.0.0)*/
const Device UA2901222 = {S65137, 152, { &P15785,&P5843,&P24678,&P18092 }};	/*Useragent - Nokia201/2.0 (11.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia201) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3030822 = {S65138, 137, { &P18645,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I747 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3031323 = {S65139, 145, { &P18645,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I747 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3036638 = {S65140, 137, { &P18645,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L710 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3062135 = {S65141, 137, { &P18645,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L710 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3066941 = {S65142, 145, { &P18645,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I747 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3094137 = {S65143, 137, { &P18645,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I747 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125933 = {S65144, 67, { &P18645,&P21637,&P23997,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.3; SAMSUNG-SGH-I747 Build/JSS15Q)*/
const Device UA3130822 = {S65145, 137, { &P18645,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I747 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA190772 = {S65146, 123, { &P15364,&P7133,&P22403,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML, like Gecko) Comodo_Dragon/11.4.0.0 Chrome/11.0.696.71 Safari/534.24*/
const Device UA316640 = {S65147, 130, { &P15364,&P7133,&P22403,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Comodo_Dragon/11.4.0.0 Chrome/11.0.696.71 Safari/534.24*/
const Device UA2257390 = {S65148, 136, { &P21133,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; HS-7DTB6-4GB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2506112 = {S65149, 133, { &P13941,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ENERGY s7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2746450 = {S65150, 151, { &P13941,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S7 Build/Smartfren.andromaxtab7.0.v.41.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2799841 = {S65151, 126, { &P13941,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; S7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2934721 = {S65152, 139, { &P13941,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; ELIYA_S7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566891 = {S65153, 130, { &P20948,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LePanII Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2631829 = {S65154, 132, { &P22037,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P3113 Build/JOP40G) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3064023 = {S65155, 131, { &P22037,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P3113 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2754493 = {S65156, 152, { &P20192,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Orange Daytona Build/HuaweiG510-0200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755409 = {S65157, 154, { &P20192,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0100 Build/HuaweiG510-0100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755419 = {S65158, 154, { &P20192,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0200 Build/HuaweiG510-0200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2939423 = {S65159, 154, { &P20192,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0251 Build/HuaweiG510-0251) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2997767 = {S65160, 154, { &P20192,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3026689 = {S65161, 113, { &P14714,&P5843,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02/07.51; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3030762 = {S65162, 115, { &P14714,&P5843,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3063037 = {S65163, 137, { &P18283,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I535 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.17 Mobile Safari/537.36*/
const Device UA3131079 = {S65164, 136, { &P18283,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131350 = {S65165, 137, { &P18283,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T999 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA2568397 = {S65166, 140, { &P17968,&P18066,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile*/
const Device UA2666776 = {S65167, 140, { &P17968,&P18066,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile*/
const Device UA2404297 = {S65168, 155, { &P16436,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I8150 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422458 = {S65169, 144, { &P20253,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile IQ 5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778140 = {S65170, 66, { &P20253,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; i-mobile IQ 5 Build/IMM76D)*/
const Device UA2975070 = {S65171, 125, { &P15869,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Xoom Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021079 = {S65172, 126, { &P15869,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Xoom Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA48874 = {S65173, 55, { &P15364,&P17017,&P3807,&P18092 }};	/*Useragent - N285/P130D10/Obigo Q03C/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49625 = {S65174, 77, { &P15364,&P17017,&P3807,&P18092 }};	/*Useragent - TCL602D/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA77430 = {S65175, 90, { &P15364,&P17017,&P3807,&P18092 }};	/*Useragent - T-mobile Atlas/EFS-TMO-EE-P107A20V1.0.5 Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA90035 = {S65176, 38, { &P15364,&P17017,&P3807,&P18092 }};	/*Useragent - ZMTFTV20/R01 Browser/Obigo/Q03C/WAP2.0*/
const Device UA90729 = {S65177, 94, { &P15364,&P17017,&P3807,&P18092 }};	/*Useragent - OPPO A201/4.02/12 Release/2010.6.02 Browser/Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA90730 = {S65178, 94, { &P15364,&P17017,&P3807,&P18092 }};	/*Useragent - OPPO A520/4.02/12 Release/2010.6.02 Browser/Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA435614 = {S65179, 91, { &P15364,&P17017,&P3807,&P18092 }};	/*Useragent - Grameenphone Q100/X990_V2_Z22_EN_D18F104 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C*/
const Device UA511222 = {S65180, 93, { &P15364,&P17017,&P3807,&P18092 }};	/*Useragent - OPPOA125/4.02/12 Release/2010.8.10 Browser/Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA511490 = {S65181, 107, { &P15364,&P17017,&P3807,&P18092 }};	/*Useragent - OPPO A115/4.02/12 Release/2010.6.02 Platform/MTK Browser/Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA517183 = {S65182, 92, { &P15364,&P17017,&P3807,&P18092 }};	/*Useragent - OPPOF15/4.02/12 Release/2010.8.10 Browser/Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA522248 = {S65183, 92, { &P15364,&P17017,&P3807,&P18092 }};	/*Useragent - OPPOT15/4.02/12 Release/2010.8.10 Browser/Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA692346 = {S65184, 47, { &P15364,&P17017,&P3807,&P18092 }};	/*Useragent - R228/Obigo Q03C/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA711611 = {S65185, 55, { &P15364,&P17017,&P3807,&P18092 }};	/*Useragent - Vega/P130D10/Obigo Q03C/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA1891361 = {S65186, 93, { &P15364,&P17017,&P3807,&P18092 }};	/*Useragent - OPPOA209/4.02/12 Release/2010.8.10 Browser/Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA1896309 = {S65187, 94, { &P15364,&P17017,&P3807,&P18092 }};	/*Useragent - OPPOA115K/4.02/12 Release/2010.8.10 Browser/Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA50352 = {S65188, 142, { &P18000,&P6757,&P21556,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.15; en-us; A70HB Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA61507 = {S65189, 82, { &P13014,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-Jack 3G-Orange/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2217678 = {S65190, 73, { &P13014,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-E821S/1.0 NetFront/3.5 QTV2.4 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2301622 = {S65191, 84, { &P13014,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-Jack 3G(Telenor)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2577559 = {S65192, 93, { &P13014,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-E821-South-Africa-Vodacom/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2640782 = {S65193, 89, { &P13014,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-E821(Mexico_Iusacell)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA738867 = {S65194, 130, { &P21012,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; SCH-I815 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA792480 = {S65195, 133, { &P21012,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; SCH-I815 4G Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1983479 = {S65196, 59, { &P21012,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; SCH-I815 Build/HTJ85B)*/
const Device UA3022763 = {S65197, 96, { &P21640,&P17484,&P17371,&P18092 }};	/*Useragent - SAMSUNG Opera/9.80 (J2ME/MIDP; Opera Mini/5.1.25449/30.3558; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2424535 = {S65198, 134, { &P23088,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; C8660 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2565334 = {S65199, 155, { &P18680,&P18066,&P21167,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA3068335 = {S65200, 130, { &P19874,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-i705 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA18069 = {S65201, 73, { &P14693,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac1-02*/
const Device UA41534 = {S65202, 67, { &P14693,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaC1-02_CMCC/2.0 (05.40) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA64692 = {S65203, 62, { &P14693,&P5843,&P4037,&P18092 }};	/*Useragent - NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA89949 = {S65204, 143, { &P12216,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; ADR6300 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA3061520 = {S65205, 93, { &P14188,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5233) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA20038 = {S65206, 67, { &P13653,&P17483,&P17602,&P18092 }};	/*Useragent - LG-LGC300/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA64599 = {S65207, 72, { &P13653,&P17483,&P17602,&P18092 }};	/*Useragent - LG-C300-Orange/V10a Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA64818 = {S65208, 65, { &P13653,&P17483,&P17602,&P18092 }};	/*Useragent - LG-C300/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63772 = {S65209, 104, { &P13214,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-A767/A767UCHI4 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA53539 = {S65210, 131, { &P16957,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-in; XCD35 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA421774 = {S65211, 119, { &P16957,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-in; XCD35 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2019339 = {S65212, 55, { &P16957,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; XCD35 Build/FRF91)*/
const Device UA2033062 = {S65213, 133, { &P16957,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; XCD35 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2261539 = {S65214, 150, { &P17835,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Barnes & Noble Nook Tablet Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2261668 = {S65215, 150, { &P17835,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook Tablet Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2507781 = {S65216, 134, { &P17835,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; NookTablet Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2235623 = {S65217, 136, { &P21011,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; L-01E Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2833533 = {S65218, 58, { &P21011,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; L-01E Build/IMM76L)*/
const Device UA2406885 = {S65219, 59, { &P15364,&P18346,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10; rv:17.0) Gecko/X Firefox/17.0*/
const Device UA2567238 = {S65220, 131, { &P22037,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3113 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2633756 = {S65221, 131, { &P22037,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P3113 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2706351 = {S65222, 156, { &P15228,&P5043,&P24677,&P18092 }};	/*Useragent - NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-03) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA349415 = {S65223, 144, { &P16757,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA363967 = {S65224, 144, { &P16757,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA606528 = {S65225, 139, { &P16757,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA640245 = {S65226, 139, { &P16757,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA668468 = {S65227, 139, { &P16757,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SCH-I500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032887 = {S65228, 61, { &P16757,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SCH-I500 Build/IML74K)*/
const Device UA2237116 = {S65229, 157, { &P13123,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2401787 = {S65230, 157, { &P13123,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2420556 = {S65231, 155, { &P13123,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2389836 = {S65232, 91, { &P17595,&P17296,&P20457,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:17.0) Gecko/X Firefox/17.0 SeaMonkey/2.14.1*/
const Device UA2457934 = {S65233, 144, { &P20883,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile IQ 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2529566 = {S65234, 145, { &P20883,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile IQ 2A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2791249 = {S65235, 66, { &P20883,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; i-mobile IQ 2 Build/IMM76D)*/
const Device UA2567404 = {S65236, 84, { &P15364,&P16110,&P22559,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:19.0) Gecko/X Firefox/19.0 IceDragon/19.0.1.0*/
const Device UA2741239 = {S65237, 62, { &P15364,&P18145,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64; rv:21.0) Gecko/X Firefox/21.0*/
const Device UA2754715 = {S65238, 137, { &P16356,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782261 = {S65239, 129, { &P16356,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2877941 = {S65240, 137, { &P16356,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-N7000 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3068406 = {S65241, 133, { &P18665,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N861 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA253790 = {S65242, 71, { &P15364,&P18110,&P24472,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/4.0; Linux) KHTML/4.0.5 (like Gecko)*/
const Device UA1947368 = {S65243, 81, { &P17595,&P17297,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10_7_2; U; en) Presto/2.9.168 Version/11.60*/
const Device UA2262155 = {S65244, 103, { &P17595,&P17297,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5; U; Edition MacAppStore; en) Presto/2.10.229 Version/11.60*/
const Device UA2261541 = {S65245, 138, { &P20013,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F180L Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2354120 = {S65246, 138, { &P20013,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F180K Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2367204 = {S65247, 138, { &P20013,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F180S Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2582291 = {S65248, 164, { &P17346,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B146 Version/6.1 Safari/8536.25 Safari/6533.18.5*/
const Device UA2596751 = {S65249, 162, { &P17346,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B144 Version/6.1 Safari/8536.25 Safari/6533.18.5*/
const Device UA2671388 = {S65250, 164, { &P17346,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10B329 Version/6.1 Safari/8536.25 Safari/6533.18.5*/
const Device UA2796548 = {S65251, 168, { &P17346,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5*/
const Device UA3048824 = {S65252, 149, { &P17346,&P19826,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/10b329 Safari/6533.18.5*/
const Device UA3058059 = {S65253, 131, { &P22606,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-N5110 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3108802 = {S65254, 60, { &P22606,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-N5110 Build/JDQ39)*/
const Device UA3068331 = {S65255, 132, { &P18556,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; BN NookHD+ Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3082722 = {S65256, 145, { &P18556,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook HD+ Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3113640 = {S65257, 130, { &P18556,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nook HD+ Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA7116 = {S65258, 104, { &P12497,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-A837/A837UCHH1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA625301 = {S65259, 100, { &P14540,&P5843,&P17347,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-01/UC Browser8.0.3.107/69/444*/
const Device UA1990784 = {S65260, 139, { &P22590,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717M Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2030988 = {S65261, 139, { &P18386,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I757M Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2235101 = {S65262, 171, { &P15649,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2257028 = {S65263, 172, { &P15649,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2385633 = {S65264, 171, { &P15649,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2404071 = {S65265, 45, { &P12280,&P17779,&P18431,&P18092 }};	/*Useragent - server-bag [iPhone OS,6.0.1,10A523,iPhone4,1]*/
const Device UA2527176 = {S65266, 24, { &P12280,&P17779,&P18431,&P18092 }};	/*Useragent - iPhone4,1/6.0.1 (10A523)*/
const Device UA2578002 = {S65267, 24, { &P12280,&P17779,&P18431,&P18092 }};	/*Useragent - iPhone3,1/6.0.1 (10A523)*/
const Device UA2762071 = {S65268, 122, { &P12280,&P17779,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/6.0 Mobile/1A543a Safari/419.3*/
const Device UA2762528 = {S65269, 24, { &P12280,&P17779,&P18431,&P18092 }};	/*Useragent - iPhone5,2/6.0.1 (10A525)*/
const Device UA2818875 = {S65270, 22, { &P12280,&P17779,&P18431,&P18092 }};	/*Useragent - iPhone4,1/6.0 (10A403)*/
const Device UA2825768 = {S65271, 22, { &P12280,&P17779,&P18431,&P18092 }};	/*Useragent - iPhone3,1/6.0 (10A403)*/
const Device UA3085903 = {S65272, 122, { &P12280,&P17779,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/6.0 Mobile/1A522a Safari/419.3*/
const Device UA2238597 = {S65273, 62, { &P15364,&P18145,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64; rv:16.0) Gecko/X Firefox/16.0*/
const Device UA2355830 = {S65274, 139, { &P21768,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; GO Live S1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2790380 = {S65275, 63, { &P21768,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GO Live S1 Build/GRK39F)*/
const Device UA2485698 = {S65276, 141, { &P11747,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; HTC Desire Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566785 = {S65277, 140, { &P11747,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC Desire Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2793483 = {S65278, 142, { &P11747,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC Desire Build/VJ AOKP) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2866216 = {S65279, 63, { &P11747,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC Desire Build/JDQ39E)*/
const Device UA2976006 = {S65280, 141, { &P18504,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LS840 Build/ZVK.IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA89708 = {S65281, 152, { &P14868,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; Android Dev Phone 1 Build/DRC83) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA94074 = {S65282, 151, { &P14868,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; Android Dev Phone 1 Build/DRC83) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA351338 = {S65283, 136, { &P18052,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM005SH Build/S4050) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA422316 = {S65284, 136, { &P18052,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM005SH Build/S4080) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA437099 = {S65285, 136, { &P18052,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM005SH Build/S4040) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA610157 = {S65286, 136, { &P18052,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM005SH Build/S4090) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA676850 = {S65287, 60, { &P18052,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SBM005SH Build/S4090)*/
const Device UA2714594 = {S65288, 136, { &P18052,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM005SH Build/S4100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2784813 = {S65289, 136, { &P18052,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SBM005SH Build/S4120) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA591056 = {S65290, 167, { &P17951,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Milestone Build/SHOLS_U2_05.26.3; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026259 = {S65291, 167, { &P17951,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Milestone Build/SHOLS_U2_05.26.3; HO!NO!CM7.2.0-RC0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA396491 = {S65292, 138, { &P21201,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; ZTE_U N720 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA712348 = {S65293, 132, { &P21201,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; N720 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029034 = {S65294, 138, { &P21201,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; ZTE_U N720 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2337145 = {S65295, 138, { &P21201,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ZTE-U N720 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA553535 = {S65296, 87, { &P17968,&P5096,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2.2; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.8.149 Version/11.10*/
const Device UA558055 = {S65297, 88, { &P17968,&P5096,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2; Linux; Opera Mobi/ADR-X; U; pt-BR) Presto/2.8.149 Version/11.10*/
const Device UA645494 = {S65298, 87, { &P17968,&P5096,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2.1; Linux; Opera Mobi/ADR-X; U; sv) Presto/2.8.149 Version/11.10*/
const Device UA2023046 = {S65299, 87, { &P17968,&P5096,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2.3; Linux; Opera Mobi/ADR-X; U; en) Presto/2.8.149 Version/11.10*/
const Device UA2675336 = {S65300, 129, { &P16356,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676255 = {S65301, 136, { &P16356,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709132 = {S65302, 137, { &P16356,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-N7000 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2975934 = {S65303, 126, { &P12765,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2201772 = {S65304, 95, { &P17595,&P17296,&P19836,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; Edition MAS; en) Presto/2.10.229 Version/11.67*/
const Device UA2433078 = {S65305, 129, { &P20510,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CT920 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA7605 = {S65306, 172, { &P13363,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-; Garmin-Asus A10 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1; A10-V3.11.3-user-20100620*/
const Device UA8932 = {S65307, 173, { &P13363,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-au; Garmin-Asus A10 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1; A10-V3.7.5-user-20100525*/
const Device UA85160 = {S65308, 57, { &P15364,&P17017,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Nitro) Opera 8.50 [en]*/
const Device UA139275 = {S65309, 53, { &P15364,&P17017,&P18098,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OpenWeb 5.7.2-10) Opera 8.54*/
const Device UA173379 = {S65310, 137, { &P17976,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MID_Serails Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA394752 = {S65311, 134, { &P17976,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; Slidepad Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA673718 = {S65312, 149, { &P17976,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; lepad_001b Build/PQXU100.4.0097.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA703566 = {S65313, 149, { &P17976,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; lepad_001b Build/PQXU100.4.0091.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA813467 = {S65314, 149, { &P17976,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; lepad_001b Build/PQXU100.4.0073.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2516415 = {S65315, 70, { &P17346,&P18432,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPod4,1; iPhone OS 6.0.1; en_US; en) AppleWebKit/420+*/
const Device UA2536218 = {S65316, 68, { &P17346,&P18432,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPod4,1; iPhone OS 6.0; en_US; en) AppleWebKit/420+*/
const Device UA2466242 = {S65317, 101, { &P17595,&P18207,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; x86_64 Mac OS X 10.8.2; java 1.6.0_37; Europe/de) http://yacy.net/bot.html*/
const Device UA2671085 = {S65318, 101, { &P17595,&P18207,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; x86_64 Mac OS X 10.8.3; java 1.6.0_43; Europe/en) http://yacy.net/bot.html*/
const Device UA2718739 = {S65319, 101, { &P17595,&P18207,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; x86_64 Mac OS X 10.8.3; java 1.6.0_45; Europe/en) http://yacy.net/bot.html*/
const Device UA2793756 = {S65320, 51, { &P17595,&P18207,&P18245,&P18091 }};	/*Useragent - RSS Bot/1.0 (Mac OS X Version 10.8.3 (Build 12D78))*/
const Device UA2800301 = {S65321, 51, { &P17595,&P18207,&P18245,&P18091 }};	/*Useragent - RSS Bot/1.1 (Mac OS X Version 10.8.4 (Build 12E55))*/
const Device UA2532042 = {S65322, 59, { &P15364,&P18346,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10; rv:19.0) Gecko/X Firefox/19.0*/
const Device UA2753497 = {S65323, 167, { &P14225,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2772282 = {S65324, 157, { &P14225,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.3; Desire HD Build/GRI40) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453*/
const Device UA2783653 = {S65325, 157, { &P14225,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; Desire HD Build/GRJ90) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA3121389 = {S65326, 157, { &P14225,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.7; Desire HD Build/GRI40) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453*/
const Device UA2974366 = {S65327, 136, { &P22685,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E460 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021386 = {S65328, 137, { &P22685,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E460f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3065151 = {S65329, 137, { &P22390,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5310 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094532 = {S65330, 138, { &P22390,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5310B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094799 = {S65331, 138, { &P22390,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5310L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3112879 = {S65332, 138, { &P22390,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5310G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3095591 = {S65333, 151, { &P12280,&P22882,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/11A465 Safari/7534.48.3*/
const Device UA3129993 = {S65334, 153, { &P12280,&P22882,&P18531,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/21.0.1180.82 Mobile/11A501 Safari/7534.48.3*/
const Device UA64802 = {S65335, 61, { &P11819,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia3220/2.0 (05.50) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA15645 = {S65336, 65, { &P11828,&P5776,&P4030,&P18092 }};	/*Useragent - Nokia6600i-1c/2.0 (36.16) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA16584 = {S65337, 84, { &P14806,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-SGH-I617/UCHJ1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA2467363 = {S65338, 139, { &P12482,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-T959 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2568292 = {S65339, 138, { &P12482,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T959 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2699442 = {S65340, 139, { &P12482,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-T959 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2737443 = {S65341, 139, { &P12482,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T959 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2160979 = {S65342, 79, { &P15364,&P18110,&P18514,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:13.0) Gecko/X Thunderbird/13.0 Lightning/1.5.1*/
const Device UA2428539 = {S65343, 131, { &P20750,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; AN10DG3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2838638 = {S65344, 60, { &P20750,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; AN10DG3 Build/IML74K)*/
const Device UA2422034 = {S65345, 118, { &P14454,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE66) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/355/UCWEB Mobile*/
const Device UA2432575 = {S65346, 40, { &P14454,&P5890,&P24673,&P18092 }};	/*Useragent - NokiaE66UCBrowser/8.7.1.234/28/352/UCWEB*/
const Device UA2484031 = {S65347, 92, { &P14454,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiae66-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2840233 = {S65348, 30, { &P14454,&P5890,&P24673,&P18092 }};	/*Useragent - NokiaE66/UCWEB8.7.0.187/28/800*/
const Device UA2605434 = {S65349, 146, { &P19867,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2991322 = {S65350, 139, { &P19867,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2740102 = {S65351, 137, { &P18559,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LG730 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2755000 = {S65352, 130, { &P18559,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LG730 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2772412 = {S65353, 139, { &P23240,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; USCC-C6721 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA30167 = {S65354, 80, { &P13663,&P22283,&P4223,&P18092 }};	/*Useragent - Mozilla/4.0 (Brew MP 1.0.2; U; en-us; Sanyo; NetFront/3.5.1/AMB) Sprint SCP-3820*/
const Device UA2413802 = {S65355, 80, { &P13663,&P22283,&P4223,&P18092 }};	/*Useragent - Mozilla/4.0 (Brew MP 1.0.2; U; en-us; Sanyo; NetFront/3.5.1/AMB) Sanyo  SCP-3820*/
const Device UA68340 = {S65356, 138, { &P14917,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; LG-KU5900 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018818 = {S65357, 62, { &P14917,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-KU5900 Build/FRG83G)*/
const Device UA2034336 = {S65358, 142, { &P21090,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I619 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2496275 = {S65359, 135, { &P21090,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I619 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2760108 = {S65360, 66, { &P21090,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-I619 Build/GINGERBREAD)*/
const Device UA2204066 = {S65361, 140, { &P12280,&P18432,&P20788,&P18092 }};	/*Useragent - MQQBrowser/37 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3*/
const Device UA2232789 = {S65362, 141, { &P12280,&P18432,&P20788,&P18092 }};	/*Useragent - MQQBrowser/371 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3*/
const Device UA2260445 = {S65363, 140, { &P12280,&P18432,&P20788,&P18092 }};	/*Useragent - MQQBrowser/371 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Safari/7534.48.3*/
const Device UA2356918 = {S65364, 143, { &P12280,&P18432,&P20788,&P18092 }};	/*Useragent - MQQBrowser/371 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3*/
const Device UA2357080 = {S65365, 142, { &P12280,&P18432,&P20788,&P18092 }};	/*Useragent - MQQBrowser/371 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3*/
const Device UA2424583 = {S65366, 142, { &P12280,&P18432,&P20788,&P18092 }};	/*Useragent - MQQBrowser/37 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3*/
const Device UA2726603 = {S65367, 132, { &P12280,&P18432,&P20788,&P18092 }};	/*Useragent - MQQBrowser/37 Mozilla/5.0 ( ; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Safari/7534.48.3*/
const Device UA2484289 = {S65368, 146, { &P21274,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Intex Aqua Marvel Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2752562 = {S65369, 147, { &P21274,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Intex Aqua Marvel+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2529616 = {S65370, 138, { &P19865,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A850K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2581827 = {S65371, 138, { &P19865,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A850L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2582829 = {S65372, 138, { &P19865,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A850S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3000307 = {S65373, 142, { &P19865,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A850S/L/K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2566440 = {S65374, 127, { &P18239,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A510 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2660313 = {S65375, 62, { &P15364,&P17350,&P4578,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; PPC; .NET CLR; Trident/5.0)*/
const Device UA2792682 = {S65376, 64, { &P15364,&P17350,&P4578,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; PPC; Win64; x64; Trident/1.0)*/
const Device UA2849892 = {S65377, 59, { &P15364,&P17350,&P4578,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; PPC; WOW64; Trident/4.0)*/
const Device UA2673957 = {S65378, 137, { &P18174,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS920 4G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2780662 = {S65379, 138, { &P22950,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E977 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999448 = {S65380, 60, { &P22950,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E977 Build/JZO54K)*/
const Device UA3068047 = {S65381, 138, { &P17172,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Transformer TF101 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080976 = {S65382, 133, { &P17172,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Transformer Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3110735 = {S65383, 133, { &P17172,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Transformer Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA54089 = {S65384, 61, { &P12000,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia6288/2.0 (06.43) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA630270 = {S65385, 124, { &P21239,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; zh-cn; M9 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA691417 = {S65386, 118, { &P21239,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; M9 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA708197 = {S65387, 130, { &P21239,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; M9 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1969763 = {S65388, 123, { &P21239,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; M9 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2023226 = {S65389, 54, { &P21239,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; M9 Build/GRJ90)*/
const Device UA2034281 = {S65390, 129, { &P21239,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; zh-cn; M9 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA609486 = {S65391, 146, { &P18348,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  HTC Velocity 4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA900916 = {S65392, 145, { &P18348,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2024743 = {S65393, 151, { &P18348,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G X710s Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA769278 = {S65394, 111, { &P21228,&P5629,&P15760,&P18092 }};	/*Useragent - Lenovo-A750/S102 Linux/2.6.35.7 Android/2.3 Release/12.12.2011 Browser/AppleWebKit533.1 Profile/ Configuration/*/
const Device UA827306 = {S65395, 128, { &P21228,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A750 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1958921 = {S65396, 140, { &P21228,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A750 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1962293 = {S65397, 133, { &P21228,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A750 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2027418 = {S65398, 111, { &P21228,&P5629,&P15760,&P18092 }};	/*Useragent - Lenovo A750/S102 Linux/2.6.35.7 Android/2.3 Release/12.12.2011 Browser/AppleWebKit533.1 Profile/ Configuration/*/
const Device UA2032304 = {S65399, 64, { &P21228,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; Lenovo A750 Build/GRK39F)*/
const Device UA2809341 = {S65400, 133, { &P21228,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A750 Build GRK39F) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2676449 = {S65401, 126, { &P19775,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A210 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA381822 = {S65402, 53, { &P18088,&P17017,&P24667,&P18092 }};	/*Useragent - IUC(U;iOS 5.0.1;Zh-cn;320*480;)/UCWEB8.2.1.132/42/997*/
const Device UA706644 = {S65403, 53, { &P18088,&P17017,&P24667,&P18092 }};	/*Useragent - IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB8.2.1.132/41/997*/
const Device UA755526 = {S65404, 51, { &P18088,&P17017,&P24667,&P18092 }};	/*Useragent - IUC(U;iOS 4.1;Zh-cn;320*480;)/UCWEB8.2.2.137/41/800*/
const Device UA765602 = {S65405, 74, { &P18088,&P17017,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; opera) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA791141 = {S65406, 74, { &P18088,&P17017,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; mstar) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA801863 = {S65407, 53, { &P18088,&P17017,&P24667,&P18092 }};	/*Useragent - IUC(U;iOS 4.2.1;Zh-cn;320*480;)/UCWEB8.2.1.132/41/997*/
const Device UA853548 = {S65408, 83, { &P18088,&P17017,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; mstar platform) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA869575 = {S65409, 73, { &P18088,&P17017,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; maui) UCBrowser8.2.0.132/69/352/UCWEB Mobile*/
const Device UA1986372 = {S65410, 73, { &P18088,&P17017,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; j2me) UCBrowser8.2.0.132/69/452/UCWEB Mobile*/
const Device UA2157558 = {S65411, 53, { &P18088,&P17017,&P24667,&P18092 }};	/*Useragent - IUC(U;iOS 5.1.1;Zh-cn;320*480;)/UCWEB8.2.1.132/42/997*/
const Device UA2332900 = {S65412, 71, { &P18088,&P17017,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; kau; maui) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2217706 = {S65413, 142, { &P14635,&P5843,&P24667,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac3-00) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA631786 = {S65414, 113, { &P12989,&P5843,&P17438,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia303/14.38; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15*/
const Device UA830045 = {S65415, 85, { &P15364,&P18140,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; Edition Linux Mint; en) Presto/2.10.289 Version/12.00*/
const Device UA1997217 = {S65416, 85, { &P15364,&P18140,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; Edition Linux Mint; en) Presto/2.10.289 Version/12.01*/
const Device UA2567089 = {S65417, 140, { &P12798,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT18i Build/4.1.B.0.587) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583205 = {S65418, 140, { &P12798,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LT18i Build/4.1.A.0.562) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2973273 = {S65419, 133, { &P18665,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N861 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA743701 = {S65420, 213, { &P15584,&P7020,&P3834,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5780-ORANGE/S5780BVKJ1; U; Bada/1.1; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 NexPlayer/3.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B*/
const Device UA3098129 = {S65421, 208, { &P15584,&P7020,&P3834,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5780D/S5780DXEKG2; U; Bada/1.1; ru-ru) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 NexPlayer/3.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B*/
const Device UA632270 = {S65422, 126, { &P17586,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; it-it; OP110 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA899410 = {S65423, 121, { &P13109,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC-X710a; en-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA1989734 = {S65424, 121, { &P13109,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC-X710a; fr-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2761905 = {S65425, 121, { &P13109,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC-X710a; es-ca) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA1965986 = {S65426, 139, { &P17595,&P17297,&P24518,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11*/
const Device UA2202871 = {S65427, 139, { &P17595,&P17297,&P24518,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11*/
const Device UA2518987 = {S65428, 139, { &P17595,&P17297,&P24518,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11*/
const Device UA2589754 = {S65429, 139, { &P17595,&P17297,&P24518,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11*/
const Device UA2018166 = {S65430, 87, { &P17968,&P17979,&P20445,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Linux armv7l; rv:2.0b5pre) Gecko/X Namoroka/4.0b4pre Fennec/2.0a1*/
const Device UA2018724 = {S65431, 80, { &P17968,&P17979,&P20445,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Linux armv7l; rv:2.0.1) Gecko/X Firefox/4.0.1 Fennec/2.0.1*/
const Device UA2030866 = {S65432, 65, { &P17968,&P17979,&P20445,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:14.0) Gecko/14.0 Firefox/2.0.1.0*/
const Device UA2043689 = {S65433, 65, { &P17968,&P17979,&P20445,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:14.0) Gecko/14.0 Firefox/2.0.1.1*/
const Device UA2458852 = {S65434, 71, { &P17968,&P17979,&P20445,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:2.0.2.1) Gecko/2.0.2.1 Firefox/2.0.2.1*/
const Device UA2560190 = {S65435, 71, { &P17968,&P17979,&P20445,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:2.0.2.2) Gecko/2.0.2.2 Firefox/2.0.2.2*/
const Device UA2601194 = {S65436, 71, { &P17968,&P17979,&P20445,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:2.0.2.4) Gecko/2.0.2.4 Firefox/2.0.2.4*/
const Device UA2356500 = {S65437, 143, { &P19862,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax A110 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2537298 = {S65438, 123, { &P15364,&P7133,&P19963,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17 Sleipnir/3.9.1*/
const Device UA2537303 = {S65439, 116, { &P15364,&P7133,&P19963,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17 Sleipnir/3.9.1*/
const Device UA2566596 = {S65440, 86, { &P15364,&P7133,&P19963,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.9.1)*/
const Device UA2567064 = {S65441, 79, { &P15364,&P7133,&P19963,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.9.1)*/
const Device UA2603869 = {S65442, 87, { &P15364,&P7133,&P19963,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0; Sleipnir/3.9.1)*/
const Device UA24661 = {S65443, 84, { &P14572,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-800/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA30897 = {S65444, 85, { &P14572,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-800A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA31249 = {S65445, 172, { &P14572,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-800A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A, Alcatel-OT-800A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA46659 = {S65446, 170, { &P14572,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-800/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A, Alcatel-OT-800/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA64082 = {S65447, 97, { &P14572,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-800/1.0 model-orange Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA165853 = {S65448, 134, { &P18018,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ko-kr; IM-A690L Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA170768 = {S65449, 136, { &P18018,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; IM-A690L Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA305105 = {S65450, 136, { &P18018,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; IM-A690S Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA508386 = {S65451, 134, { &P18018,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; IM-A690S Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA526276 = {S65452, 60, { &P18018,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; IM-A690L Build/FRG83)*/
const Device UA2018787 = {S65453, 134, { &P18018,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-; IM-A690S Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019133 = {S65454, 58, { &P18018,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; IM-A690L Build/FRF91)*/
const Device UA2019181 = {S65455, 60, { &P18018,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; IM-A690S Build/FRG83)*/
const Device UA2948651 = {S65456, 132, { &P18018,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-; IM-A690S Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA567790 = {S65457, 132, { &P20249,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC-P715a; en-ca) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA761430 = {S65458, 131, { &P21219,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0; en-us; pandigital92ghr) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2544163 = {S65459, 138, { &P24014,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LGL46C Build/CV2.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2716554 = {S65460, 94, { &P12088,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaX6-00) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2955320 = {S65461, 32, { &P12088,&P5129,&P24676,&P18092 }};	/*Useragent - NokiaX6-00/UCWEB8.9.0.253/50/800*/
const Device UA2882675 = {S65462, 112, { &P18558,&P20706,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia309/05.85; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA9035 = {S65463, 110, { &P16003,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-S5233A/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA66196 = {S65464, 101, { &P11727,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-D780/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA766023 = {S65465, 103, { &P15364,&P16110,&P22516,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.8.1.12) Gecko/X Firefox/2.0.0.12 Navigator/9.0.0.6*/
const Device UA1961454 = {S65466, 132, { &P22041,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6810 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2404689 = {S65467, 151, { &P18221,&P5843,&P24673,&P18092 }};	/*Useragent - Nokia302/5.0 (14.53) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia302) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2421086 = {S65468, 136, { &P23949,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; H9500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985108 = {S65469, 58, { &P23949,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; H9500 Build/IMM76D)*/
const Device UA2530266 = {S65470, 126, { &P23313,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3071406 = {S65471, 55, { &P23313,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; M8 Build/JRO03H)*/
const Device UA2969552 = {S65472, 94, { &P11905,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6120c) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA2205330 = {S65473, 166, { &P15575,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-I9100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2235091 = {S65474, 166, { &P15575,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-I9100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2257400 = {S65475, 167, { &P15575,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-I9100G Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2366625 = {S65476, 152, { &P17323,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P6800 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2420871 = {S65477, 148, { &P17323,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6800 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2480298 = {S65478, 162, { &P17323,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6800 Build/JRO03H; CyanogenMod-Nexness PRO cm10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615976 = {S65479, 148, { &P17323,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6800 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2581282 = {S65480, 60, { &P15364,&P6704,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; Win 9x 4.90; Trident/4.0)*/
const Device UA2661193 = {S65481, 60, { &P15364,&P6704,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; Win 9x 4.90; Trident/3.0)*/
const Device UA2791832 = {S65482, 54, { &P15364,&P6704,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; Win98; Trident/2.0)*/
const Device UA2849820 = {S65483, 54, { &P15364,&P6704,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; Win98; Trident/2.0)*/
const Device UA2975544 = {S65484, 163, { &P18250,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Motorola RAZR MAXX Build/6.7.2-180_DHD-16_NA-27) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA904843 = {S65485, 78, { &P15364,&P5690,&P22360,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:12.2.1) Gecko/X Firefox/12.2.1 PaleMoon/12.2.1*/
const Device UA909205 = {S65486, 98, { &P15364,&P5690,&P22360,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:12.2.1) Gecko/X Firefox/12.2.1-x64 PaleMoon/12.2.1-x64*/
const Device UA2566839 = {S65487, 161, { &P18645,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L710 Build/JDQ39; Carbon-Carbon-JB-v1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567834 = {S65488, 161, { &P18645,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I747 Build/JDQ39; Carbon-Carbon-JB-v1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2674167 = {S65489, 163, { &P18645,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L710 Build/JDQ39; Carbon-CARBON-JB-v1.6.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2712700 = {S65490, 163, { &P18645,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I747 Build/JDQ39; Carbon-CARBON-JB-v1.6.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2739708 = {S65491, 162, { &P18645,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L710 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2739716 = {S65492, 162, { &P18645,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I747 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987939 = {S65493, 162, { &P18645,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-L710 Build/JOP40G; Carbon-Carbon-JB-v1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567001 = {S65494, 130, { &P15932,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P3110 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2633365 = {S65495, 137, { &P15932,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P3110 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2633692 = {S65496, 131, { &P15932,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P3110 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2637993 = {S65497, 129, { &P15932,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; GT-P3110 Build/JOP40C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2670268 = {S65498, 137, { &P18713,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2714472 = {S65499, 137, { &P18713,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P769 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2781119 = {S65500, 130, { &P18713,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2956315 = {S65501, 137, { &P18713,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P768 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2605889 = {S65502, 112, { &P15767,&P18117,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0*/
const Device UA2677409 = {S65503, 114, { &P15767,&P18117,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0.1*/
const Device UA2740100 = {S65504, 114, { &P15767,&P18117,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0.2*/
const Device UA2869368 = {S65505, 114, { &P15767,&P18117,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0.3*/
const Device UA2897112 = {S65506, 114, { &P15767,&P18117,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0.4*/
const Device UA3033499 = {S65507, 114, { &P15767,&P18117,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0.5*/
const Device UA3066120 = {S65508, 114, { &P15767,&P18117,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0.6*/
const Device UA2716920 = {S65509, 161, { &P14186,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5830 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2796274 = {S65510, 161, { &P14186,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; GT-S5830 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2797698 = {S65511, 161, { &P14186,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-S5830 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2799252 = {S65512, 161, { &P14186,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-S5830 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2870157 = {S65513, 162, { &P14186,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-LA; GT-S5830M Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2754351 = {S65514, 141, { &P20399,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP8010 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2865819 = {S65515, 136, { &P21794,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9152 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2938529 = {S65516, 129, { &P21794,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9152 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA169584 = {S65517, 150, { &P17963,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-us; NXM703U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA704148 = {S65518, 150, { &P17963,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.1.0; en-us; NXM703U Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA751218 = {S65519, 90, { &P17968,&P18066,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.1; Linux; Opera Mobi/ADR-X; U; en-GB) Presto/2.9.201 Version/11.50*/
const Device UA2347835 = {S65520, 87, { &P17968,&P18066,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.3; Linux; Opera Mobi/ADY-X; U; ru) Presto/2.9.201 Version/11.50*/
const Device UA2442402 = {S65521, 87, { &P17968,&P18066,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.4; Linux; Opera Mobi/ADY-X; U; ru) Presto/2.9.201 Version/11.50*/
const Device UA2422960 = {S65522, 153, { &P17475,&P5768,&P24673,&P18092 }};	/*Useragent - Nokia6300/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6300) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2504834 = {S65523, 91, { &P17475,&P5768,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6300) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2507788 = {S65524, 104, { &P15364,&P7133,&P19818,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.8.1 Safari/534.34*/
const Device UA2533387 = {S65525, 62, { &P15364,&P18145,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64; rv:19.0) Gecko/X Firefox/19.0*/
const Device UA2567505 = {S65526, 147, { &P18224,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8815 Build/HuaweiU8815) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2861249 = {S65527, 156, { &P18224,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8815N Build/HuaweiU8815NC02B942) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567087 = {S65528, 133, { &P13644,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; EVO Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567500 = {S65529, 133, { &P13644,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; EVO Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA242862 = {S65530, 126, { &P15364,&P7133,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.64.347 Chrome/13.0.782.218 Safari/535.1*/
const Device UA248414 = {S65531, 119, { &P15364,&P7133,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.64.361 Chrome/13.0.782.218 Safari/535.1*/
const Device UA255980 = {S65532, 126, { &P15364,&P7133,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.71.128 Chrome/14.0.835.202 Safari/535.1*/
const Device UA316966 = {S65533, 120, { &P15364,&P7133,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML, like Gecko) RockMelt/0.9.56.357 Chrome/11.0.696.71 Safari/534.24*/
const Device UA362624 = {S65534, 119, { &P15364,&P7133,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.72.615 Chrome/14.0.835.202 Safari/535.1*/
const Device UA647167 = {S65535, 140, { &P15364,&P7133,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) RockMelt/0.9.50.549 Chrome/10.0.648.205 Safari/534.16*/
const Device UA794514 = {S65536, 127, { &P15364,&P7133,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) RockMelt/0.9.58.494 Chrome/11.0.696.71 Safari/534.24*/
const Device UA795200 = {S65537, 138, { &P15364,&P7133,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) RockMelt/0.9.48.51 Chrome/9.0.597.107 Safari/534.13*/
const Device UA2259107 = {S65538, 121, { &P15364,&P7133,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.30 (KHTML, like Gecko) RockMelt/0.9.62.313 Chrome/12.0.742.122 Safari/534.30*/
const Device UA2395204 = {S65539, 126, { &P15364,&P7133,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML	 like Gecko) RockMelt/0.9.72.698 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2365802 = {S65540, 145, { &P17519,&P18066,&P18062,&P18092 }};	/*Useragent - HUAWEI_T8830Pro_TD/1.0 Android/4.0.4 (Linux; U; Android 4.0.4; zh-cn) Release/08.31.2012 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30*/
const Device UA2466048 = {S65541, 138, { &P17519,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; T8830Pro Build/HuaweiT8830Pro) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2577979 = {S65542, 147, { &P17519,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; T8830Pro Build/HuaweiT8830Pro) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654733 = {S65543, 69, { &P17519,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; T8830Pro Build/HuaweiT8830Pro)*/
const Device UA2433177 = {S65544, 128, { &P23881,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; X210 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3095429 = {S65545, 97, { &P17595,&P17296,&P22772,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:24.0) Gecko/X Firefox/24.0 AlexaToolbar/alxf-2.19*/
const Device UA3126245 = {S65546, 110, { &P23589,&P18378,&P24711,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo WiiU) AppleWebKit/536.28 (KHTML, like Gecko) NX/3.0.3.12.6 NintendoBrowser/2.0.0.9362.US*/
const Device UA3126787 = {S65547, 110, { &P23589,&P18378,&P24711,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo WiiU) AppleWebKit/536.28 (KHTML, like Gecko) NX/3.0.3.12.6 NintendoBrowser/2.0.0.9362.JP*/
const Device UA3130828 = {S65548, 110, { &P23589,&P18378,&P24711,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo WiiU) AppleWebKit/536.28 (KHTML, like Gecko) NX/3.0.3.12.6 NintendoBrowser/2.0.0.9362.EU*/
const Device UA28001 = {S65549, 73, { &P12210,&P5027,&P3412,&P18092 }};	/*Useragent - BlackBerry8120/4.3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100*/
const Device UA36825 = {S65550, 135, { &P12210,&P5027,&P3412,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320) BlackBerry8120/4.3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100*/
const Device UA2360608 = {S65551, 69, { &P18321,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GT-S5300; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2674519 = {S65552, 129, { &P17595,&P17295,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/23.0.648.205 Safari/534.16*/
const Device UA2719151 = {S65553, 129, { &P17595,&P17295,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/23.0.648.205 Safari/534.16*/
const Device UA13777 = {S65554, 61, { &P11772,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia6270/2.0 (03.90) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA182021 = {S65555, 143, { &P16607,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; E140 Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1ACER_E140/1.0*/
const Device UA395495 = {S65556, 146, { &P16607,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; E140 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1ACER_E140/1.0*/
const Device UA680566 = {S65557, 146, { &P21095,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; Excellent8 Android 4.0 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA791437 = {S65558, 146, { &P21095,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; Excellent8 Android 4.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2223245 = {S65559, 142, { &P14711,&P5204,&P24670,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac2-01) UCBrowser8.4.0.159/70/444/UCWEB Mobile*/
const Device UA2226732 = {S65560, 89, { &P17595,&P18579,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacBookPro9%2C2)*/
const Device UA2226961 = {S65561, 77, { &P17595,&P18579,&P17470,&P18092 }};	/*Useragent - WebProcess/8536.25 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacBookPro9%2C2)*/
const Device UA2502165 = {S65562, 89, { &P17595,&P18579,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacBookAir5%2C2)*/
const Device UA2759287 = {S65563, 77, { &P17595,&P18579,&P17470,&P18092 }};	/*Useragent - WebProcess/8536.25 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacBookAir5%2C2)*/
const Device UA1997525 = {S65564, 140, { &P21082,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A618 TV Duo Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2383912 = {S65565, 137, { &P20239,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Enjoy 10 Plus Build/1.14ST) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2398029 = {S65566, 137, { &P20239,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Enjoy 10 Plus Build/1.11ST) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2419722 = {S65567, 137, { &P20239,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Enjoy 10 Plus Build/1.11PP) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2478914 = {S65568, 137, { &P20239,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Enjoy 10 Plus Build/1.14PP) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2686986 = {S65569, 137, { &P20239,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Enjoy 10 Plus Build/1.13ST) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2707421 = {S65570, 137, { &P20239,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Enjoy 10 Plus Build/1.18ST) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2482366 = {S65571, 148, { &P19863,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2595997 = {S65572, 139, { &P19863,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI U9508 Build/HuaweiU9508) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3043751 = {S65573, 70, { &P19863,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HUAWEI U9508 Build/HuaweiU9508)*/
const Device UA2531009 = {S65574, 154, { &P21833,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ALCATEL ONE TOUCH 4030E Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2952406 = {S65575, 76, { &P21833,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ALCATEL ONE TOUCH 4030E Build/JRO03C)*/
const Device UA2679350 = {S65576, 133, { &P20244,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; bq Elcano Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2936764 = {S65577, 62, { &P20244,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; bq Elcano Build/JZO54K)*/
const Device UA2773255 = {S65578, 157, { &P16362,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2782826 = {S65579, 161, { &P16362,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710VMUB Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2784825 = {S65580, 160, { &P16362,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710BST Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2258167 = {S65581, 133, { &P23906,&P18066,&P22237,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Android 4.0.4;en-us;I10-LE Build/IML74K) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2753766 = {S65582, 137, { &P18697,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I415 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3068333 = {S65583, 137, { &P20973,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LW770 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA359546 = {S65584, 136, { &P17308,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; DROIDX Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA717337 = {S65585, 137, { &P17308,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA824235 = {S65586, 137, { &P17308,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROIDX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA828925 = {S65587, 137, { &P17308,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDX Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA829209 = {S65588, 135, { &P17308,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Droid X Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA909547 = {S65589, 137, { &P17308,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDX Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1968459 = {S65590, 138, { &P17308,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Droid X Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2412965 = {S65591, 138, { &P17308,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Droid X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026143 = {S65592, 132, { &P14905,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2611529 = {S65593, 132, { &P14905,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P7100 Build/HTJ85B) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2239277 = {S65594, 130, { &P18051,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-01D Build/V08R31A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2673788 = {S65595, 130, { &P18051,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-01D Build/V10R32D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2554445 = {S65596, 156, { &P21858,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI Y300-0000 Build/HuaweiY300-0000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2890779 = {S65597, 78, { &P21858,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HUAWEI Y300-0000 Build/HuaweiY300-0000)*/
const Device UA2783670 = {S65598, 127, { &P15364,&P18118,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:19.0) Gecko/X Firefox/19.0 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19*/
const Device UA2973473 = {S65599, 137, { &P18594,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LS860 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022646 = {S65600, 133, { &P18594,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Mach Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA45232 = {S65601, 109, { &P13861,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-M3710/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA704095 = {S65602, 114, { &P14711,&P5204,&P17433,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-01/11.20; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.1.21.8*/
const Device UA899643 = {S65603, 171, { &P13123,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352*/
const Device UA900394 = {S65604, 171, { &P13123,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352*/
const Device UA2567262 = {S65605, 142, { &P19778,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP9090 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2876121 = {S65606, 141, { &P20399,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PantechP8010 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2900834 = {S65607, 134, { &P20399,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PantechP8010 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3063297 = {S65608, 136, { &P13064,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I727 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3068318 = {S65609, 145, { &P13064,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I727 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA30765 = {S65610, 161, { &P16745,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; T-Mobile myTouch 3G Slide Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA89842 = {S65611, 166, { &P16745,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; T-Mobile myTouch 3G Slide Build/ERE27) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA274553 = {S65612, 83, { &P16745,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; T-Mobile myTouch 3G Slide Build/ERE27)*/
const Device UA2885304 = {S65613, 112, { &P15785,&P5843,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia201/11.95; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA846888 = {S65614, 145, { &P13526,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MyTouch 4g Build/AwesomeICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025290 = {S65615, 138, { &P13526,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MyTouch 4G Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027854 = {S65616, 140, { &P13526,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; myTouch4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029520 = {S65617, 151, { &P13526,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC myTouch 4G slide Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2183965 = {S65618, 141, { &P13526,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MyTouch 4G Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2399100 = {S65619, 140, { &P13526,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; myTouch4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695891 = {S65620, 145, { &P13526,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC myTouch 4g Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2727436 = {S65621, 145, { &P13526,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC myTouch 4g Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA853052 = {S65622, 158, { &P21223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_C8810 Build/C8810V100R001C92B862) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025019 = {S65623, 143, { &P21223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8810 Build/C8810V100R001C92B862) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2441841 = {S65624, 148, { &P21223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8810 Build/C8810V100R001C92B862) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2596228 = {S65625, 99, { &P21223,&P5629,&P15760,&P18092 }};	/*Useragent - HUAWEI_Y220T_TD/1.0 Android/2.3.5 SC8810_CMCC/W12.06.21 Release/06.22.2012 Browser/AppleWebkit533.1*/
const Device UA2817131 = {S65626, 148, { &P21223,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8810 Build C8810V100R001C92B862) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2850044 = {S65627, 105, { &P21223,&P5629,&P15760,&P18092 }};	/*Useragent - Bird_T9108_TD/T9108_V1.00 Android/2.3.5 SC8810_CMCC/W12.06.10 Release/04.09.2012 Browser/AppleWebKit533.1*/
const Device UA3057122 = {S65628, 102, { &P21223,&P5629,&P15760,&P18092 }};	/*Useragent - HUAWEI_Y220-T10_TD/1.0 Android/2.3.5 SC8810_CMCC/W12.06.21 Release/02.01.2013 Browser/AppleWebkit533.1*/
const Device UA2529335 = {S65629, 141, { &P21141,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Lenovo P770 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2599821 = {S65630, 145, { &P21141,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Lenovo P770_ROW Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2753418 = {S65631, 137, { &P23000,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; TECNO P3 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2773113 = {S65632, 169, { &P18113,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2343028 = {S65633, 186, { &P14708,&P5204,&P24671,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5130c-2) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2430976 = {S65634, 123, { &P13648,&P5055,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; NokiaE7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile*/
const Device UA2467293 = {S65635, 102, { &P13648,&P5055,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiae7-00) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2579465 = {S65636, 42, { &P13648,&P5055,&P24673,&P18092 }};	/*Useragent - NokiaE7-00UCBrowser/8.7.1.234/51/355/UCWEB*/
const Device UA2466852 = {S65637, 139, { &P21417,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IdeaTabA2207A-H Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2697315 = {S65638, 68, { &P21417,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; IdeaTabA2207A-H Build/IMM76D)*/
const Device UA2844270 = {S65639, 137, { &P21417,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; IdeaTabA2207A-H Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2727949 = {S65640, 137, { &P21949,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; LG-F100L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2731079 = {S65641, 139, { &P21949,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-F100L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996333 = {S65642, 61, { &P21949,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F100L Build/JZO54K)*/
const Device UA3068395 = {S65643, 134, { &P17160,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Slider SL101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3068205 = {S65644, 137, { &P18174,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS920 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3096276 = {S65645, 147, { &P18385,&P21637,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus 7 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA12155 = {S65646, 140, { &P11732,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-SGH-I637/UCIJ2 Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA253782 = {S65647, 8, { &P15364,&P17017,&P17247,&P18092 }};	/*Useragent - Chrome/8*/
const Device UA395937 = {S65648, 63, { &P15364,&P17017,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 AppleWebKit/534.10 Chrome/8.0.552.215 Safari/534.10*/
const Device UA395958 = {S65649, 72, { &P15364,&P5448,&P4779,&P18092 }};	/*Useragent - T-Mobile_LEO (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA571196 = {S65650, 80, { &P15364,&P5448,&P4779,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)/UC Browser7.8.0.95*/
const Device UA773524 = {S65651, 108, { &P15364,&P5448,&P4779,&P18092 }};	/*Useragent - 480x800; XV6975; Window Mobile 6.5 Professional; (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA800109 = {S65652, 110, { &P15364,&P5448,&P4779,&P18092 }};	/*Useragent - 480x800; XV6875.1; Window Mobile 6.5 Professional; (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA1891686 = {S65653, 71, { &P15364,&P5448,&P4779,&P18092 }};	/*Useragent - USCCHTC6875 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA1896156 = {S65654, 76, { &P15364,&P5448,&P4779,&P18092 }};	/*Useragent - T-Mobile_Rhodium (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA2295281 = {S65655, 85, { &P15364,&P5448,&P4779,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Sprint:SPH-ip830w)/UC Browser7.8.0.95*/
const Device UA2318306 = {S65656, 75, { &P15364,&P5448,&P4779,&P18092 }};	/*Useragent - Sprint-SPH-i350 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA2370330 = {S65657, 78, { &P15364,&P5448,&P4779,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95/31/352*/
const Device UA2720407 = {S65658, 74, { &P15364,&P5448,&P4779,&P18092 }};	/*Useragent - T-Mobile_Maple (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA569752 = {S65659, 27, { &P18254,&P7480,&P22897,&P18091 }};	/*Useragent - RSSOwl/2.1.2.X (X11; U; en)*/
const Device UA1894672 = {S65660, 31, { &P18254,&P7480,&P22897,&P18091 }};	/*Useragent - RSSOwl/2.1.1.X (Windows; U; en)*/
const Device UA1997037 = {S65661, 31, { &P18254,&P7480,&P22897,&P18091 }};	/*Useragent - RSSOwl/2.1.4.X (Windows; U; en)*/
const Device UA2224479 = {S65662, 27, { &P18254,&P7480,&P22897,&P18091 }};	/*Useragent - RSSOwl/2.1.4.X (X11; U; en)*/
const Device UA2408163 = {S65663, 31, { &P18254,&P7480,&P22897,&P18091 }};	/*Useragent - RSSOwl/2.1.6.X (Windows; U; en)*/
const Device UA2483800 = {S65664, 27, { &P18254,&P7480,&P22897,&P18091 }};	/*Useragent - RSSOwl/2.1.6.X (X11; U; en)*/
const Device UA16486 = {S65665, 250, { &P11939,&P17485,&P3487,&P18092 }};	/*Useragent - SonyEricssonS312/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1, SonyEricssonS312/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1, SonyEricssonS312/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA48988 = {S65666, 166, { &P11939,&P17485,&P3487,&P18092 }};	/*Useragent - SonyEricssonS312/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1, SonyEricssonS312/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA64501 = {S65667, 82, { &P11939,&P17485,&P3487,&P18092 }};	/*Useragent - SonyEricssonS312/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA50034 = {S65668, 149, { &P13148,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-nz; Huawei IDEOS X3 Build/HuaweiU8510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369061 = {S65669, 73, { &P13148,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; Huawei IDEOS X3 Build/HuaweiU8510)*/
const Device UA2816015 = {S65670, 158, { &P13148,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; pt-pt; Huawei Ideos X3 Build/joaquimorg; CM-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA657621 = {S65671, 145, { &P21070,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; Ultimate10-Android4.0 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2158571 = {S65672, 134, { &P21070,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; ULTIMATE10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA888068 = {S65673, 79, { &P15364,&P7133,&P22359,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.1) Gecko/X Firefox/12.1 PaleMoon/12.1*/
const Device UA896633 = {S65674, 92, { &P15364,&P7133,&P22359,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:12.1) Gecko/X Firefox/12.1-x64 PaleMoon/12.1-x64*/
const Device UA1980341 = {S65675, 136, { &P15389,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U8800 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1988767 = {S65676, 136, { &P15389,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; U8800 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025279 = {S65677, 136, { &P15389,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; U8800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2545726 = {S65678, 58, { &P15389,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; U8800 Build/IMM76I)*/
const Device UA2672309 = {S65679, 168, { &P15794,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315b Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676665 = {S65680, 168, { &P15794,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3081078 = {S65681, 138, { &P22969,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT915 Build/2_32A_2008) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA363889 = {S65682, 131, { &P12801,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; PG41200 Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA650731 = {S65683, 60, { &P12801,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; PG41200 Build/HTK75C)*/
const Device UA399173 = {S65684, 70, { &P18176,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 900;7.10;8107)*/
const Device UA688691 = {S65685, 70, { &P18176,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 900;7.10;8112)*/
const Device UA925748 = {S65686, 70, { &P18176,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 900;7.10;8773)*/
const Device UA925899 = {S65687, 70, { &P18176,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 900;7.10;8779)*/
const Device UA673251 = {S65688, 87, { &P17968,&P5629,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.3; Linux; Opera Mobi/ADR-X; U; fr) Presto/2.9.201 Version/11.50*/
const Device UA696440 = {S65689, 90, { &P17968,&P5629,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.7; Linux; Opera Mobi/ADR-X; U; es-ES) Presto/2.9.201 Version/11.50*/
const Device UA696833 = {S65690, 87, { &P17968,&P5629,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.4; Linux; Opera Mobi/ADR-X; U; ko) Presto/2.9.201 Version/11.50*/
const Device UA698480 = {S65691, 87, { &P17968,&P5629,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.5; Linux; Opera Mobi/ADY-X; U; ru) Presto/2.9.201 Version/11.50*/
const Device UA1912516 = {S65692, 85, { &P17968,&P5629,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3; Linux; Opera Mobi/ADR-X; U; en) Presto/2.9.201 Version/11.50*/
const Device UA1933554 = {S65693, 90, { &P17968,&P5629,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.2; Linux; Opera Mobi/ADR-X; U; en-GB) Presto/2.9.201 Version/11.50*/
const Device UA2022194 = {S65694, 87, { &P17968,&P5629,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.1; Linux; Opera Mobi/ADR-X; U; en) Presto/2.9.201 Version/11.50*/
const Device UA2653052 = {S65695, 87, { &P17968,&P5629,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.6; Linux; Opera Mobi/ADR-X; U; uk) Presto/2.9.201 Version/11.50*/
const Device UA2042682 = {S65696, 74, { &P15364,&P18110,&P22578,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux x86_64; Edition Next) Presto/2.12.378 Version/12.50*/
const Device UA2258490 = {S65697, 120, { &P18364,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaC5-05) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile*/
const Device UA2483385 = {S65698, 147, { &P20353,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BLU Touch Book 7.0 LITE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2505315 = {S65699, 139, { &P16362,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-D710 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2536763 = {S65700, 139, { &P16362,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SPH-D710 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2562911 = {S65701, 138, { &P16362,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-D710 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892584 = {S65702, 139, { &P16362,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-D710 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956987 = {S65703, 61, { &P16362,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SPH-D710 Build/JDQ39E)*/
const Device UA3080933 = {S65704, 146, { &P22850,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI H881C Build/HuaweiH881C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA308745 = {S65705, 129, { &P22546,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; HP eStation/1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2469840 = {S65706, 84, { &P15364,&P16110,&P22556,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:17.0) Gecko/X Firefox/17.0 IceDragon/18.0.1.0*/
const Device UA2489797 = {S65707, 84, { &P15364,&P16110,&P22556,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:18.0) Gecko/X Firefox/18.0 IceDragon/18.0.3.1*/
const Device UA2975851 = {S65708, 146, { &P22850,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI H881C Build/HuaweiH881C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3068409 = {S65709, 130, { &P17281,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I905 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2437144 = {S65710, 129, { &P14628,&P19941,&P3625,&P18168 }};	/*Useragent - KDDI-CA31 UP.Browser/6.2.0.7.3.129 (GUI) MMP/2.0 (compatible; ichiro/mobile goo;+http://search.goo.ne.jp/option/use/sub4/sub4-1/)*/
const Device UA2468038 = {S65711, 59, { &P15364,&P18346,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10; rv:18.0) Gecko/X Firefox/18.0*/
const Device UA2508763 = {S65712, 92, { &P13706,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5235) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2702404 = {S65713, 150, { &P22903,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Micromax A62 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2715437 = {S65714, 92, { &P12280,&P18432,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 6_0_1; zh-CN; iPh3,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile*/
const Device UA2716983 = {S65715, 92, { &P12280,&P18432,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 6_0_1; zh-CN; iPh4,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile*/
const Device UA2736759 = {S65716, 92, { &P12280,&P18432,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 6_0_2; zh-CN; iPh5,2) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile*/
const Device UA2765953 = {S65717, 138, { &P22955,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-P714 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958532 = {S65718, 60, { &P22955,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P714 Build/JZO54K)*/
const Device UA458778 = {S65719, 135, { &P17950,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID7015 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA459119 = {S65720, 59, { &P17950,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID7015 Build/GRI40)*/
const Device UA2258505 = {S65721, 119, { &P15373,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5530) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile*/
const Device UA2344261 = {S65722, 31, { &P15373,&P5129,&P24672,&P18092 }};	/*Useragent - Nokia5530/UCWEB8.6.0.180/50/800*/
const Device UA2375668 = {S65723, 146, { &P19867,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I437 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2385263 = {S65724, 147, { &P20104,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; 201M Build/7.7.1Q-164_SMJ-133) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2385714 = {S65725, 147, { &P20104,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; 201M Build/7.7.1Q-164_SMJ-143) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2428462 = {S65726, 147, { &P20104,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; 201M Build/7.7.1Q-164_SMJ-165) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528370 = {S65727, 69, { &P20104,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; 201M Build/7.7.1Q-164_SMJ-165)*/
const Device UA2532582 = {S65728, 126, { &P23071,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; T2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2779100 = {S65729, 133, { &P23071,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; T2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987311 = {S65730, 55, { &P23071,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; T2 Build/JRO03H)*/
const Device UA2674733 = {S65731, 57, { &P15364,&P7127,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0) Presto/2.12.388 Version/12.15*/
const Device UA2709491 = {S65732, 69, { &P15364,&P7127,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; Edition Yx) Presto/2.12.388 Version/12.15*/
const Device UA2753000 = {S65733, 136, { &P21935,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E980 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2973896 = {S65734, 137, { &P20013,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F180L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975950 = {S65735, 137, { &P20013,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F180K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975954 = {S65736, 137, { &P20013,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F180S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020852 = {S65737, 136, { &P20013,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F180 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974026 = {S65738, 140, { &P22598,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6506 Build/10.1.A.1.350) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA60603 = {S65739, 125, { &P11996,&P17484,&P3834,&P18092 }};	/*Useragent - SAMSUNG-GT-S5560i/S5560IXXKA5 SHP/VPP/R5 Dolfin/2.0 NexPlayer/3.0 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B*/
const Device UA56056 = {S65740, 103, { &P14630,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i917R)*/
const Device UA765252 = {S65741, 82, { &P17595,&P17295,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.10.229 Version/11.64*/
const Device UA2001271 = {S65742, 129, { &P15364,&P7133,&P22436,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 Comodo_Dragon/21.1.1.0*/
const Device UA2001288 = {S65743, 122, { &P15364,&P7133,&P22436,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 Comodo_Dragon/21.1.1.0*/
const Device UA3010013 = {S65744, 133, { &P24103,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; C2 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3091748 = {S65745, 55, { &P24103,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; C2 Build/JOP40D)*/
const Device UA18441 = {S65746, 103, { &P12061,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE65-1 Orange/1.0 (1.0633.18.02) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA55599 = {S65747, 94, { &P12061,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE65/3.0 (1.0633.58.02) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64129 = {S65748, 96, { &P12061,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE65-1/3.0 (2.0633.65.01) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA34405 = {S65749, 73, { &P13683,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-GT-B3310/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2675024 = {S65750, 141, { &P22209,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Titanium S5 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3131341 = {S65751, 149, { &P18113,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA13941 = {S65752, 62, { &P11781,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia6080b/2.0 (04.22) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA13944 = {S65753, 61, { &P11781,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia6080/2.0 (04.20) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA833995 = {S65754, 63, { &P15364,&P6704,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows 98; U; en; rv:1.8.0) Gecko/X Firefox/1.5.0*/
const Device UA881538 = {S65755, 80, { &P15364,&P6704,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win 9x 4.90; en-US; rv:1.8.0.3) Gecko/X Firefox/1.5.0.3*/
const Device UA2463433 = {S65756, 71, { &P15364,&P6704,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win98; ru; rv:1.8.0.4) Gecko/X Firefox/1.5.0.4*/
const Device UA1953075 = {S65757, 133, { &P21065,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; CINK Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030771 = {S65758, 130, { &P17965,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SC-01D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2486886 = {S65759, 133, { &P18465,&P18110,&P19817,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.6 Safari/535.19 Silk-Accelerated=false*/
const Device UA2483363 = {S65760, 46, { &P12280,&P17779,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.7 iPhone_OS/6.0.1 Device/iPhone_4*/
const Device UA2528783 = {S65761, 46, { &P12280,&P17779,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.7 iPhone_OS/5.1.1 Device/iPhone_4*/
const Device UA2530894 = {S65762, 44, { &P12280,&P17779,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.7 iPhone_OS/6.1 Device/iPhone_4*/
const Device UA2565354 = {S65763, 44, { &P12280,&P17779,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.7 iPhone_OS/6.1 Device/iPhone_5*/
const Device UA2600651 = {S65764, 46, { &P12280,&P17779,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.7 iPhone_OS/6.1.2 Device/iPhone_5*/
const Device UA2600654 = {S65765, 46, { &P12280,&P17779,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.7 iPhone_OS/6.1.2 Device/iPhone_4*/
const Device UA2664464 = {S65766, 47, { &P12280,&P17779,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.7 iPhone_OS/6.1.2 Device/iPhone_4S*/
const Device UA2698282 = {S65767, 47, { &P12280,&P17779,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.7 iPhone_OS/5.1.1 Device/iPhone_4S*/
const Device UA2705368 = {S65768, 47, { &P12280,&P17779,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.7 iPhone_OS/6.1.3 Device/iPhone_4S*/
const Device UA2706729 = {S65769, 46, { &P12280,&P17779,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.7 iPhone_OS/6.1.3 Device/iPhone_4*/
const Device UA2728277 = {S65770, 47, { &P12280,&P17779,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.7 iPhone_OS/6.0.1 Device/iPhone_4S*/
const Device UA2730434 = {S65771, 45, { &P12280,&P17779,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.7 iPhone_OS/6.1 Device/iPhone_4S*/
const Device UA2734326 = {S65772, 46, { &P12280,&P17779,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.7 iPhone_OS/6.1.3 Device/iPhone_5*/
const Device UA2781951 = {S65773, 46, { &P12280,&P17779,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.7 iPhone_OS/6.1.4 Device/iPhone_5*/
const Device UA2858027 = {S65774, 48, { &P12280,&P17779,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.7 iPhone_OS/6.1.3 Device/iPhone_3GS*/
const Device UA67446 = {S65775, 136, { &P13215,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; GT-I5700 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67674 = {S65776, 136, { &P13215,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; GT-I5700 Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA68376 = {S65777, 137, { &P13215,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; GT-I5700 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA534871 = {S65778, 60, { &P13215,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-I5700 Build/FRG83)*/
const Device UA2018537 = {S65779, 61, { &P13215,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; GT-I5700 Build/FRG83G)*/
const Device UA2021465 = {S65780, 137, { &P13215,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; GT-I5700 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2330114 = {S65781, 142, { &P14826,&P5843,&P24667,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac1-01) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA629204 = {S65782, 143, { &P18348,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; HTC Velocity 4G Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA764517 = {S65783, 149, { &P18348,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-hk; HTC_Velocity_4G_X710s Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1952534 = {S65784, 154, { &P18348,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; HTC/Velocity 4G/2.38.162.3 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024181 = {S65785, 67, { &P18348,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Velocity 4G Build/GWK74)*/
const Device UA2029880 = {S65786, 149, { &P18348,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; HTC Velocity 4G X710s Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2334698 = {S65787, 154, { &P18348,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-at; HTC/Velocity 4G/2.39.169.1 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2506642 = {S65788, 154, { &P18348,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; HTC/Velocity 4G/2.38.162.5 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2775449 = {S65789, 154, { &P18348,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; HTC/Velocity 4G/2.38.162.1 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2261501 = {S65790, 138, { &P18682,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-M950 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2347519 = {S65791, 131, { &P23930,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982074 = {S65792, 55, { &P23930,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; U1 Build/GRK39F)*/
const Device UA2557670 = {S65793, 133, { &P23821,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; 3305-1030 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2973649 = {S65794, 137, { &P18316,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974891 = {S65795, 137, { &P18316,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020520 = {S65796, 138, { &P18316,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160LV Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021670 = {S65797, 137, { &P18316,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023039 = {S65798, 136, { &P18316,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3107857 = {S65799, 139, { &P18316,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160S/K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3066213 = {S65800, 135, { &P18283,&P21637,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-I9300 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.30 Mobile Safari/537.36*/
const Device UA3131160 = {S65801, 135, { &P18283,&P21637,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SCH-I535 Build/JLS36G) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA2261237 = {S65802, 182, { &P14711,&P5204,&P24671,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2361555 = {S65803, 69, { &P18672,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GT-S5302; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA3043513 = {S65804, 71, { &P18672,&P18110,&P24663,&P18092 }};	/*Useragent - Opera (Linux; U; 2.3.6; zh-cn; GT-S5302; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2461163 = {S65805, 128, { &P23824,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M870 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2462458 = {S65806, 60, { &P19747,&P5043,&P18517,&P18092 }};	/*Useragent - Nokia206/2.0 (03.58) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2462463 = {S65807, 58, { &P19747,&P5043,&P18517,&P18092 }};	/*Useragent - Nokia206.1/2.0 (p) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2528585 = {S65808, 126, { &P23814,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; U2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2988684 = {S65809, 55, { &P23814,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; U2 Build/IMM76I)*/
const Device UA2566508 = {S65810, 141, { &P23910,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT30at Build/7.0.B.1.152) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668261 = {S65811, 138, { &P23233,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID7038 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2710123 = {S65812, 116, { &P15364,&P16110,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 Sleipnir/4.1.0*/
const Device UA2774163 = {S65813, 123, { &P15364,&P16110,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 Sleipnir/4.1.3*/
const Device UA2877572 = {S65814, 124, { &P15364,&P16110,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36 Sleipnir/4.1.4*/
const Device UA2900033 = {S65815, 117, { &P15364,&P16110,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36 Sleipnir/4.1.4*/
const Device UA2958463 = {S65816, 136, { &P20480,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; JY-G3 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3106714 = {S65817, 58, { &P20480,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; JY-G3 Build/JOP40D)*/
const Device UA2975748 = {S65818, 155, { &P18113,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA117403 = {S65819, 128, { &P15364,&P5690,&P22444,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Iron/5.0.381.0 Chrome/5.0.381 Safari/533.4*/
const Device UA2405446 = {S65820, 159, { &P12473,&P5204,&P24673,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia2730c-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2485044 = {S65821, 138, { &P20013,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F180S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2485937 = {S65822, 138, { &P20013,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F180L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2489105 = {S65823, 138, { &P20013,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F180K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2567382 = {S65824, 146, { &P18626,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I547 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2670314 = {S65825, 139, { &P18626,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I547C Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2966977 = {S65826, 146, { &P18626,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I547 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA8290 = {S65827, 153, { &P13437,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; T-Mobile_G2_Touch Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA63471 = {S65828, 104, { &P12050,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-T929/T929UVIA2 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA396523 = {S65829, 142, { &P21425,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; STM803HC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023411 = {S65830, 66, { &P21425,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.1; STM803HC Build/GINGERBREAD)*/
const Device UA1999916 = {S65831, 157, { &P12681,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S 4G Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2008837 = {S65832, 157, { &P12681,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S 4G Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2201492 = {S65833, 157, { &P12681,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S 4G Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2366958 = {S65834, 161, { &P12681,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus S 4G Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423099 = {S65835, 155, { &P20075,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia S Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2560012 = {S65836, 156, { &P21271,&P18066,&P18062,&P18092 }};	/*Useragent - AQUA 5.0 Linux/3.0.13 Android/4.0.4 Release/12.26.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2594980 = {S65837, 139, { &P21271,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AQUA 5.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2738448 = {S65838, 133, { &P23177,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAB-840_G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3103754 = {S65839, 62, { &P23177,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; TAB-840_G Build/JRO03H)*/
const Device UA102183 = {S65840, 72, { &P18254,&P6704,&P17387,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; Win 9x 4.90; MSIECrawler)*/
const Device UA691673 = {S65841, 63, { &P15364,&P17350,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 7.0; U; en) Presto/2.9.211 Version/12.00*/
const Device UA2042735 = {S65842, 88, { &P15364,&P7133,&P22553,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X Firefox/15.0.2.0 IceDragon/15.0.2.0*/
const Device UA2483202 = {S65843, 138, { &P20280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; vivo X1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2595968 = {S65844, 129, { &P20280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; vivo X1 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2728584 = {S65845, 60, { &P20280,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; vivo X1 Build/JRO03C)*/
const Device UA2890075 = {S65846, 139, { &P20280,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; vivo X1w Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2797775 = {S65847, 137, { &P23094,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; F-06E Build/V17R45B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2895072 = {S65848, 137, { &P23094,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; F-06E Build/V19R47F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2961929 = {S65849, 59, { &P23094,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; F-06E Build/V17R45B)*/
const Device UA3023826 = {S65850, 137, { &P23094,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; F-06E Build/V21R48D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3080591 = {S65851, 137, { &P13991,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6400L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA458862 = {S65852, 135, { &P20979,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID8120 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458882 = {S65853, 59, { &P20979,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID8120 Build/GRI40)*/
const Device UA2895334 = {S65854, 134, { &P23700,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; N9101 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2960971 = {S65855, 127, { &P23700,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; N9101 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA13455 = {S65856, 44, { &P13374,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_P3700 Opera/9.50 (Windows NT 5.1; U; ru)*/
const Device UA29206 = {S65857, 52, { &P13374,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Diamond Opera/9.50 (Windows NT 5.1; U; en) P3720*/
const Device UA46715 = {S65858, 82, { &P13374,&P5448,&P3430,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Opera/9.5) Alltel HTC Touch Diamond*/
const Device UA66650 = {S65859, 64, { &P13374,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Diamond Opera/9.50 (Windows NT 5.1; U; en) P3720/V2.07.841.2*/
const Device UA87008 = {S65860, 44, { &P13374,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_P3700 Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA87403 = {S65861, 59, { &P13374,&P5448,&P3430,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_Diamond Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA88470 = {S65862, 44, { &P13374,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_P3700 Opera/9.50 (Windows NT 5.1; U; nb)*/
const Device UA88471 = {S65863, 44, { &P13374,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_P3700 Opera/9.50 (Windows NT 5.1; U; da)*/
const Device UA88469 = {S65864, 44, { &P13374,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_P3700 Opera/9.50 (Windows NT 5.1; U; sv)*/
const Device UA92235 = {S65865, 47, { &P13374,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_P3700 Opera/9.50 (Windows NT 5.1; U; en-GB)*/
const Device UA553766 = {S65866, 47, { &P13374,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_P3700 Opera/9.50 (Windows NT 5.1; U; es-ES)*/
const Device UA681651 = {S65867, 44, { &P13374,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_P3700 Opera/9.50 (Windows NT 5.1; U; nl)*/
const Device UA723090 = {S65868, 44, { &P13374,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_P3700 Opera/9.50 (Windows NT 5.1; U; pt)*/
const Device UA729456 = {S65869, 44, { &P13374,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_P3700 Opera/9.50 (Windows NT 5.1; U; fr)*/
const Device UA790857 = {S65870, 44, { &P13374,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_P3700 Opera/9.50 (Windows NT 5.1; U; it)*/
const Device UA1851289 = {S65871, 44, { &P13374,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_P3700 Opera/9.50 (Windows NT 5.1; U; de)*/
const Device UA2125572 = {S65872, 47, { &P13374,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_P3700 Opera/9.50 (Windows NT 5.1; U; zh-tw)*/
const Device UA2977342 = {S65873, 59, { &P13374,&P5448,&P3430,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_Diamond Opera/9.50 (Windows NT 5.1; U; fr)*/
const Device UA230022 = {S65874, 137, { &P17623,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; VTAB1008 Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA631893 = {S65875, 61, { &P17623,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.2; VTAB1008 Build/GRH78C)*/
const Device UA2222431 = {S65876, 153, { &P21756,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; iw-il; Orange_infinity_996 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2558881 = {S65877, 134, { &P20807,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; B92 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2718550 = {S65878, 93, { &P15373,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5530) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2954926 = {S65879, 31, { &P15373,&P5129,&P24676,&P18092 }};	/*Useragent - Nokia5530/UCWEB8.9.0.253/50/800*/
const Device UA3096092 = {S65880, 97, { &P17595,&P17297,&P22772,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:24.0) Gecko/X Firefox/24.0 AlexaToolbar/alxf-2.19*/
const Device UA2886769 = {S65881, 112, { &P19973,&P5043,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia114/03.33; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA2257806 = {S65882, 128, { &P20252,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; V96A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Build/IMM76I*/
const Device UA2563719 = {S65883, 143, { &P21374,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 101 Titanium Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2675059 = {S65884, 145, { &P12657,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515a Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676350 = {S65885, 145, { &P12657,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515m Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2750907 = {S65886, 138, { &P12657,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC_X515E Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2879694 = {S65887, 105, { &P15364,&P18145,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36*/
const Device UA2886225 = {S65888, 106, { &P15364,&P18145,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36*/
const Device UA2973652 = {S65889, 130, { &P17281,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I905 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974638 = {S65890, 130, { &P17323,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6800 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA688211 = {S65891, 67, { &P15851,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;Radar 4G;7.10;7720)*/
const Device UA690874 = {S65892, 67, { &P15851,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;Radar 4G;7.10;7740)*/
const Device UA707886 = {S65893, 67, { &P15851,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;Radar 4G;7.10;8107)*/
const Device UA925706 = {S65894, 70, { &P15851,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;Radar C110e;7.10;7720)*/
const Device UA925734 = {S65895, 64, { &P15851,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;Radar;7.10;7720)*/
const Device UA925735 = {S65896, 70, { &P15851,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;Radar C110e;7.10;7740)*/
const Device UA925786 = {S65897, 67, { &P15851,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;Radar 4G;7.10;8773)*/
const Device UA925833 = {S65898, 64, { &P15851,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;Radar;7.10;7740)*/
const Device UA925790 = {S65899, 70, { &P15851,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;Radar C110e;7.10;8107)*/
const Device UA925919 = {S65900, 64, { &P15851,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;Radar;7.10;8107)*/
const Device UA2360253 = {S65901, 70, { &P15851,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;Radar C110e;7.10;8773)*/
const Device UA901868 = {S65902, 99, { &P15364,&P17188,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA2843045 = {S65903, 125, { &P15364,&P17188,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0) AppleWebKit/536.5 (KHTML, like Gecko) YaBrowser/1.1.1084.5409 Chrome/19.1.1084.5409 Safari/536.5*/
const Device UA2257137 = {S65904, 150, { &P21543,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; sr-rs; Telenor_Smart_HD Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2362340 = {S65905, 90, { &P23545,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; mauie800) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2405801 = {S65906, 90, { &P23545,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; mauie800) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2457642 = {S65907, 87, { &P23545,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; id; mauie800) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2483060 = {S65908, 87, { &P23545,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; id; mauie800) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2696056 = {S65909, 87, { &P23545,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; vi; mauie800) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA3085982 = {S65910, 90, { &P23545,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; ar-sa; mauie800) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2698960 = {S65911, 105, { &P11684,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5800d-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2732960 = {S65912, 44, { &P11684,&P5129,&P24676,&P18092 }};	/*Useragent - Nokia5800w XpressMusic/UCWEB8.9.0.253/50/999*/
const Device UA2749813 = {S65913, 43, { &P11684,&P5129,&P24676,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UCWEB8.9.0.253/50/999*/
const Device UA2768748 = {S65914, 102, { &P11684,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; id; Nokia5800 XpressMusic) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2865465 = {S65915, 106, { &P11684,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5800w XpressMusic) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2774040 = {S65916, 158, { &P18280,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2859510 = {S65917, 159, { &P18280,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2869113 = {S65918, 158, { &P18280,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC VLE_U Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2774063 = {S65919, 156, { &P20705,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2784082 = {S65920, 155, { &P20705,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTCONE Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2869559 = {S65921, 158, { &P20705,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC_PN071 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2944002 = {S65922, 128, { &P23702,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ME173X Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2362832 = {S65923, 133, { &P18466,&P18110,&P19815,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Safari/535.19 Silk-Accelerated=false*/
const Device UA2770099 = {S65924, 144, { &P19862,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Micromax A110Q Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2869481 = {S65925, 137, { &P19862,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Micromax A110Q Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22*/
const Device UA2973462 = {S65926, 139, { &P18582,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT28h Build/6.2.B.0.211) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020846 = {S65927, 139, { &P18582,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT28i Build/6.2.B.0.211) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3033832 = {S65928, 139, { &P24391,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; GT-I8260 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092380 = {S65929, 61, { &P24391,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8260 Build/JZO54K)*/
const Device UA458609 = {S65930, 112, { &P18320,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; PadFone) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA609372 = {S65931, 158, { &P14183,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I997 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA896770 = {S65932, 158, { &P14183,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I997 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA910968 = {S65933, 154, { &P14183,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I997 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA634192 = {S65934, 144, { &P21067,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; OnePAD 970 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2387271 = {S65935, 129, { &P21258,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MOMO7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2485623 = {S65936, 136, { &P21258,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MOMO7 Talent Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2426391 = {S65937, 154, { &P18283,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483428 = {S65938, 87, { &P18283,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; GT-I9300 Build/JRO03C.I9300ZNDLK4;720*1280)*/
const Device UA2507051 = {S65939, 87, { &P18283,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; GT-I9300 Build/JRO03C.I9300ZNDLL1;720*1280)*/
const Device UA2563113 = {S65940, 154, { &P18283,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2706804 = {S65941, 87, { &P18283,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-I9300 Build/JZO54K.I9300ZNEMB2;720*1280)*/
const Device UA2796074 = {S65942, 87, { &P18283,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-I9300 Build/JZO54K.I9300ZNEMD2;720*1280)*/
const Device UA2857081 = {S65943, 87, { &P18283,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-I9300 Build/JZO54K.I9300ZNEMB1;720*1280)*/
const Device UA2858766 = {S65944, 87, { &P18283,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-I9300 Build/JZO54K.I9300ZNEMA2;720*1280)*/
const Device UA3044324 = {S65945, 87, { &P18283,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; GT-I9300 Build/JRO03C.I9300ZCDLL1;720*1280)*/
const Device UA3088214 = {S65946, 87, { &P18283,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-I9300 Build/JZO54K.I9300ZCEMD2;720*1280)*/
const Device UA3118322 = {S65947, 87, { &P18283,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; GT-I9300 Build/JRO03C.I9300ZCDLK1;720*1280)*/
const Device UA2660618 = {S65948, 152, { &P12989,&P5843,&P24676,&P18092 }};	/*Useragent - Nokia303/5.0 (14.87) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia303) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2676500 = {S65949, 130, { &P17168,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7310 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708916 = {S65950, 168, { &P12280,&P17090,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5 UCBrowser/9.0.1.275*/
const Device UA2794220 = {S65951, 141, { &P12280,&P17090,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 4_3_3 like Mac OS X; zh-CN) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 UCBrowser/9.0.1.284 Mobile*/
const Device UA2995750 = {S65952, 141, { &P12280,&P17090,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 4_3_5 like Mac OS X; zh-CN) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 UCBrowser/9.0.1.284 Mobile*/
const Device UA2715891 = {S65953, 162, { &P15649,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5570I Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2783837 = {S65954, 161, { &P15649,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5570 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2868379 = {S65955, 161, { &P15649,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-S5570 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2958002 = {S65956, 161, { &P15649,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; GT-S5570 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2768633 = {S65957, 133, { &P21546,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; V972 Core4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3049052 = {S65958, 62, { &P21546,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; V972 Core4 Build/JDQ39)*/
const Device UA164294 = {S65959, 148, { &P16595,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; Micromax A60 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA695611 = {S65960, 134, { &P16595,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; Micromax A60 Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1891851 = {S65961, 70, { &P16595,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Micromax A60 Build/ERE27)*/
const Device UA510928 = {S65962, 99, { &P17595,&P18120,&P22479,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US; rv:1.8.1.14) Gecko/X Firefox/2.0.0.14 Flock/1.2.1*/
const Device UA1860598 = {S65963, 104, { &P17595,&P18120,&P22479,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.8.1.15) Gecko/X Firefox/2.0.0.15 Flock/1.2.3*/
const Device UA2405785 = {S65964, 137, { &P20921,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D2-711 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2844890 = {S65965, 59, { &P20921,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; D2-711 Build/IMM76D)*/
const Device UA2460848 = {S65966, 159, { &P14862,&P5204,&P24675,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia2700c-2) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2974240 = {S65967, 139, { &P18592,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One VX Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA389636 = {S65968, 140, { &P18020,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SKY IM-A650S Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA548784 = {S65969, 138, { &P18020,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-; SKY IM-A650S Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA550008 = {S65970, 64, { &P18020,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SKY IM-A650S Build/FRG83)*/
const Device UA2606685 = {S65971, 65, { &P15364,&P5690,&P18101,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Firefox/12.0; Windows NT 5.1;) Vbot/1303*/
const Device UA2258253 = {S65972, 80, { &P17595,&P18207,&P18691,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:16.0) Gecko/X Thunderbird/16.0.2*/
const Device UA2258091 = {S65973, 91, { &P17595,&P17296,&P20761,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:16.0) Gecko/X Firefox/16.0 SeaMonkey/2.13.2*/
const Device UA2350094 = {S65974, 178, { &P18219,&P5843,&P24671,&P18092 }};	/*Useragent - Nokia202/2.0 (20.28) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia202) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2496641 = {S65975, 63, { &P15364,&P7133,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; Windows NT 6.1; Trident/4.0)*/
const Device UA2744219 = {S65976, 63, { &P15364,&P7133,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 6.1; Trident/3.0)*/
const Device UA2752775 = {S65977, 139, { &P22853,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HTC_802w Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2797142 = {S65978, 139, { &P22853,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HTC 802w Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2938048 = {S65979, 61, { &P22853,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC 802w Build/JZO54K)*/
const Device UA2966146 = {S65980, 147, { &P22853,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; HTC_One_dual_sim Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937272 = {S65981, 130, { &P15364,&P17979,&P19802,&P18092 }};	/*Useragent - Lenovo-A830/S100 Linux/3.4.0 Android/4.2 Release/01.23.2013 Browser/AppleWebKit534.30 Profile/ Configuration/ Mobile Safari/534.30*/
const Device UA3049774 = {S65982, 82, { &P15364,&P17979,&P19802,&P18092 }};	/*Useragent - SAMSUNG-GT-I9158_TD/1.0 Android/4.2.2 Release/04.03.2013 Browser/AppleWebKit535.19*/
const Device UA3061514 = {S65983, 129, { &P15364,&P17979,&P19802,&P18092 }};	/*Useragent - sprd-GT-T9500/1.0 Linux/2.6.35.7 Android/4.2.0 Release/07.18.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3108023 = {S65984, 93, { &P15364,&P17979,&P19802,&P18092 }};	/*Useragent - Coolpad7269_CUCC_WCDMA/1.0 Linux/3.4.5 Android/4.2 Release/6.28.2013 Browser/AppleWebkit533.1*/
const Device UA3094025 = {S65985, 70, { &P15767,&P22882,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.11A465 (iPad; U; CPU OS 7_0 like Mac OS X; en_us)*/
const Device UA3115390 = {S65986, 72, { &P15767,&P22882,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.11A501 (iPad; U; CPU OS 7_0_2 like Mac OS X; en_us)*/
const Device UA780341 = {S65987, 32, { &P14748,&P5890,&P24667,&P18092 }};	/*Useragent - NokiaE72-1/UCWEB8.2.0.116/28/800*/
const Device UA788177 = {S65988, 81, { &P14748,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE72-1)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA2643024 = {S65989, 89, { &P14748,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokiae72-1) UCBrowser8.2.1.144/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA2257259 = {S65990, 151, { &P20887,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1811_v75_jbaol1_g88 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2378926 = {S65991, 151, { &P20887,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1811_v75_jbaol1_g58 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2382101 = {S65992, 145, { &P20887,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1811_v75_hjy1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2383432 = {S65993, 148, { &P20887,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; e1811_v75_jyuany1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3085659 = {S65994, 150, { &P20887,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; e1811_v75_zlh1_5647 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2376933 = {S65995, 133, { &P21137,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; HS-10DTB2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2461229 = {S65996, 68, { &P15364,&P5690,&P22564,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:16.0) Gecko/X Firefox/16.0 epic/16.0*/
const Device UA3044616 = {S65997, 68, { &P15364,&P5690,&P22564,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:16.0) Gecko/X Firefox/16.0 Epic/16.0*/
const Device UA2470515 = {S65998, 62, { &P15364,&P18145,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64; rv:18.0) Gecko/X Firefox/18.0*/
const Device UA2558620 = {S65999, 156, { &P13107,&P5043,&P24676,&P18092 }};	/*Useragent - NokiaC2-06/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-06) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2770762 = {S66000, 143, { &P22862,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; Micromax A88 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049470 = {S66001, 65, { &P22862,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Micromax A88 Build/JRO03C)*/
const Device UA2866923 = {S66002, 136, { &P21793,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9205 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2962223 = {S66003, 129, { &P21793,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9205 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA43041 = {S66004, 129, { &P15102,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ605 Build/H.6.5-14) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA73194 = {S66005, 129, { &P15102,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ605 Build/H.6.5-12) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA75816 = {S66006, 128, { &P15102,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; MZ605 Build/H.6.5-8) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA630510 = {S66007, 131, { &P15102,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; es-us; MZ605 Build/H.6.5-18.5) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA649382 = {S66008, 131, { &P15102,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; pt-br; MZ605 Build/H.6.5-18.4) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA172879 = {S66009, 137, { &P16587,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SHW-M190S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA656499 = {S66010, 61, { &P16587,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SHW-M190S Build/FROYO)*/
const Device UA379548 = {S66011, 140, { &P20965,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; IS11PT Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA427423 = {S66012, 69, { &P17346,&P18117,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.9B176 (iPod; U; CPU OS 5_1 like Mac OS X; en_us)*/
const Device UA656846 = {S66013, 71, { &P17346,&P18117,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.9B206 (iPod; U; CPU OS 5_1_1 like Mac OS X; en_us)*/
const Device UA2176688 = {S66014, 137, { &P23336,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; I_7520 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2662404 = {S66015, 137, { &P23336,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I_7520 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2421659 = {S66016, 33, { &P18267,&P5043,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (NokiaX2-02/11.79)*/
const Device UA2876916 = {S66017, 131, { &P20420,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; I10A-LE Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3106865 = {S66018, 60, { &P20420,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; I10A-LE Build/JRO03H)*/
const Device UA3080081 = {S66019, 148, { &P15438,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081685 = {S66020, 142, { &P15438,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083594 = {S66021, 138, { &P15438,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC-Z710a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083836 = {S66022, 142, { &P15438,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083006 = {S66023, 143, { &P12280,&P17146,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9A405 Safari/7534.48.3*/
const Device UA3096335 = {S66024, 143, { &P12280,&P17146,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9A406 Safari/7534.48.3*/
const Device UA3096905 = {S66025, 141, { &P12280,&P17146,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9A334 Safari/7534.48.3*/
const Device UA55286 = {S66026, 83, { &P11877,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonZ610i/R1KH Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA30114 = {S66027, 100, { &P12265,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-L760/JAGH2 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA45004 = {S66028, 116, { &P12265,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-L760-Vodafone/BUHC1 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA44118 = {S66029, 85, { &P13785,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-802A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA55188 = {S66030, 85, { &P13785,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-802Y/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA63160 = {S66031, 84, { &P13785,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-802/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA563622 = {S66032, 81, { &P15364,&P18110,&P18199,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.16) Gecko/X Thunderbird/2.0.0.16*/
const Device UA700757 = {S66033, 97, { &P15364,&P17285,&P22331,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.8) Gecko/X Firefox/3.6.8 CometBird/3.6.8*/
const Device UA1872478 = {S66034, 103, { &P15364,&P17285,&P22331,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; zh-CN; rv:1.9.2.16) Gecko/X Firefox/3.6.16 CometBrowser/3.6.16*/
const Device UA2362412 = {S66035, 182, { &P13035,&P5043,&P24672,&P18092 }};	/*Useragent - NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-02) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile*/
const Device UA2467412 = {S66036, 141, { &P21845,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-STYLE Q4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2579337 = {S66037, 142, { &P21845,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-STYLE Q4A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2843925 = {S66038, 63, { &P21845,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; i-STYLE Q4 Build/IMM76D)*/
const Device UA2738278 = {S66039, 150, { &P22864,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Micromax A26 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2754580 = {S66040, 125, { &P14111,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Arc Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2781854 = {S66041, 127, { &P23203,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; A1-810 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA55155 = {S66042, 70, { &P11782,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-S3600/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24522 = {S66043, 136, { &P13078,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; U8800-51 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67272 = {S66044, 136, { &P13078,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; ZTE-U V880 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA230097 = {S66045, 138, { &P13078,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-U V880 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA728482 = {S66046, 126, { &P13078,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-U V880 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA790206 = {S66047, 124, { &P13078,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; ZTE-U V880 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1980099 = {S66048, 131, { &P13078,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-U V880 Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2019494 = {S66049, 62, { &P13078,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; ZTE-U V880 Build/FRF91)*/
const Device UA30973 = {S66050, 58, { &P12378,&P17017,&P4924,&P18092 }};	/*Useragent - HUAWEI-M228/001.00 ACCESS-NetFront/3.2(Mobile) Mozilla/5.0*/
const Device UA54483 = {S66051, 35, { &P12378,&P17017,&P4924,&P18092 }};	/*Useragent - HUAWEI-M228/001.00 ACS-NetFront/3.2*/
const Device UA2674592 = {S66052, 104, { &P18436,&P16110,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; chromeframe/26.0.1410.64)*/
const Device UA2735899 = {S66053, 97, { &P18436,&P16110,&P21919,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; Touch; chromeframe/26.0.1410.64)*/
const Device UA2735908 = {S66054, 160, { &P14862,&P5204,&P24677,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2700c-2) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2755344 = {S66055, 139, { &P18465,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2782670 = {S66056, 141, { &P18465,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire2 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2799206 = {S66057, 138, { &P18465,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Amazon Kindle Fire Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2948004 = {S66058, 157, { &P16362,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2965855 = {S66059, 160, { &P16362,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710BST Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3059057 = {S66060, 161, { &P16362,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710VMUB Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3068279 = {S66061, 130, { &P12879,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7510 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3110552 = {S66062, 130, { &P12879,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P7510 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA14404 = {S66063, 62, { &P11801,&P5776,&P3756,&P18092 }};	/*Useragent - Nokia6600s/2.0 (05.60) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA637589 = {S66064, 43, { &P12449,&P5890,&P24665,&P18092 }};	/*Useragent - NOKIAE5-00/UC Browser8.0.4.121/28/444/UCWEB*/
const Device UA537081 = {S66065, 81, { &P15364,&P17350,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (U; Win32 i686; ru; rv:1.9.2.13) Gecko/X 3.6.13-1.fc14 Firefox/3.6.13*/
const Device UA2166047 = {S66066, 83, { &P15364,&P17350,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 7.1; en-US; rv:1.9.2.13) Gecko/X Firefox/3.6.13*/
const Device UA2329604 = {S66067, 144, { &P15364,&P17350,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows; en-US) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16 Gecko/X Firefox/3.6.4 EvoHtmlToPdf/3.5*/
const Device UA2774537 = {S66068, 45, { &P15364,&P17350,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT) Firefox/3.6*/
const Device UA2033105 = {S66069, 136, { &P20008,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; XT687 Build/V2.27D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2419512 = {S66070, 136, { &P20008,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; XT687 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2346954 = {S66071, 138, { &P15785,&P5843,&P24668,&P18092 }};	/*Useragent - Nokia201/2.0 (11.64) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia201) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2387440 = {S66072, 131, { &P23507,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GEM7008 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422628 = {S66073, 151, { &P12989,&P5843,&P24673,&P18092 }};	/*Useragent - Nokia303/5.0 (14.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia303) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2974899 = {S66074, 150, { &P18460,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-1 Build/HuaweiU8950-1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975092 = {S66075, 154, { &P18460,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-51 Build/HuaweiU8950N-51) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020851 = {S66076, 152, { &P18460,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-51 Build/HuaweiU8950-51) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021958 = {S66077, 148, { &P18460,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950D Build/HuaweiU8950D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023099 = {S66078, 152, { &P18460,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-1 Build/HuaweiU8950N-1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA378515 = {S66079, 108, { &P15364,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/2.1.1; U; de-DE) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 P160U/1.0*/
const Device UA408936 = {S66080, 106, { &P15364,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/2.1.0; U; en-GB) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.1*/
const Device UA459726 = {S66081, 114, { &P15364,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.3; U; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Desktop/1.0*/
const Device UA469912 = {S66082, 106, { &P15364,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; en-IE) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.1*/
const Device UA659143 = {S66083, 106, { &P15364,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/2.1.0; U; fr-FR) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.2*/
const Device UA717794 = {S66084, 106, { &P15364,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.5; U; zh-CN) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.1*/
const Device UA720758 = {S66085, 106, { &P15364,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/2.1.0; U; de-DE) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.1*/
const Device UA742493 = {S66086, 108, { &P15364,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/2.1.1; U; en-GB) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 P160U/1.0*/
const Device UA770332 = {S66087, 106, { &P15364,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/2.1.0; U; es-ES) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.2*/
const Device UA793683 = {S66088, 110, { &P15364,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/2.1.2; U; es-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 P160UNA/1.0*/
const Device UA812146 = {S66089, 110, { &P15364,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/2.1.2; U; zh-CN) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 P160UNA/1.0*/
const Device UA1839978 = {S66090, 106, { &P15364,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/1.4.0; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.1*/
const Device UA1866480 = {S66091, 106, { &P15364,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/2.1.0; U; fr-FR) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.1*/
const Device UA1899190 = {S66092, 108, { &P15364,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/2.1.1; U; en-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 P160U/1.0*/
const Device UA2439433 = {S66093, 106, { &P15364,&P18187,&P18175,&P18092 }};	/*Useragent - Mozilla/5.0 (webOS/2.1.0; U; es-US) AppleWebKit/532.2 (KHTML, like Gecko) Version/1.0 Safari/532.2 Pre/1.2*/
const Device UA902553 = {S66094, 132, { &P21135,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; HS-7DTB4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2171666 = {S66095, 132, { &P21135,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; HS-7DTB4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2236059 = {S66096, 135, { &P22815,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hr-hr; move 2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2480775 = {S66097, 103, { &P15232,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.276; en-us; 9810/5.1.0.276) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2595087 = {S66098, 103, { &P15232,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.0.0.734; en-us; 9810/5.0.0.734) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2655371 = {S66099, 103, { &P15232,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.532; en-us; 9810/5.1.0.532) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2728137 = {S66100, 103, { &P15232,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.0.0.469; en-us; 9810/5.0.0.469) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2728220 = {S66101, 103, { &P15232,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.230; en-us; 9810/5.1.0.230) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2848714 = {S66102, 103, { &P15232,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.0.0.736; en-us; 9810/5.0.0.736) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2951260 = {S66103, 103, { &P15232,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.507; en-us; 9810/5.1.0.507) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2951284 = {S66104, 103, { &P15232,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.546; en-us; 9810/5.1.0.546) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2984011 = {S66105, 103, { &P15232,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.0.0.496; en-us; 9810/5.0.0.496) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2986371 = {S66106, 103, { &P15232,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.0.0.451; en-us; 9810/5.0.0.451) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3071465 = {S66107, 103, { &P15232,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.181; en-us; 9810/5.1.0.181) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3099337 = {S66108, 103, { &P15232,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.401; en-us; 9810/5.1.0.401) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2504048 = {S66109, 136, { &P23758,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9430 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2753203 = {S66110, 137, { &P18641,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-M830 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753676 = {S66111, 145, { &P13042,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Incredible S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2771049 = {S66112, 139, { &P20076,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26w Build/6.2.B.0.200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879065 = {S66113, 142, { &P20076,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xperia Acro S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2865823 = {S66114, 137, { &P21283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2964350 = {S66115, 130, { &P21283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3015119 = {S66116, 138, { &P21283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3019809 = {S66117, 138, { &P21283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310N Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3108572 = {S66118, 138, { &P21283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3124520 = {S66119, 138, { &P21283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310B Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3030116 = {S66120, 159, { &P18442,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L900 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/l900*/
const Device UA3062283 = {S66121, 163, { &P18442,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-T889 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/t0ltetmo*/
const Device UA3064030 = {S66122, 159, { &P18442,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L900 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/l900*/
const Device UA3078718 = {S66123, 160, { &P18442,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-N7105 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/t0lte*/
const Device UA3079815 = {S66124, 171, { &P18442,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I317 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/t0lteatt*/
const Device UA3079816 = {S66125, 163, { &P18442,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-T889 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/t0ltetmo*/
const Device UA3080259 = {S66126, 160, { &P18442,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-N7100 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/n7100*/
const Device UA3093403 = {S66127, 160, { &P18442,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-N7105 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/t0lte*/
const Device UA3125793 = {S66128, 162, { &P18442,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I317 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2.0/t0lte*/
const Device UA3127168 = {S66129, 162, { &P18442,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I317 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2.0/t0lte*/
const Device UA3127578 = {S66130, 163, { &P18442,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-T889 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/t0ltetmo*/
const Device UA3127924 = {S66131, 159, { &P18442,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L900 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/l900*/
const Device UA3128011 = {S66132, 173, { &P18442,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I317 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2.0/t0lteatt*/
const Device UA33432 = {S66133, 123, { &P14669,&P5843,&P17431,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-00/08.25; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.0.9.17.2*/
const Device UA252414 = {S66134, 122, { &P15364,&P17285,&P22413,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.2 (KHTML, like Gecko) Comodo_Dragon/15.0.3.0 Chrome/15.0.874.102 Safari/535.2*/
const Device UA252490 = {S66135, 129, { &P15364,&P17285,&P22413,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Comodo_Dragon/15.0.3.0 Chrome/15.0.874.102 Safari/535.2*/
const Device UA632862 = {S66136, 64, { &P15364,&P7127,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.10.229 Version/11.61*/
const Device UA669632 = {S66137, 130, { &P20149,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-P6211 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2608734 = {S66138, 59, { &P20149,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2; GT-P6211 Build/HTJ85B)*/
const Device UA2620911 = {S66139, 128, { &P20149,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P6211 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2432663 = {S66140, 229, { &P18700,&P5629,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-B5512 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2362291 = {S66141, 131, { &P21865,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TAB-730 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2389088 = {S66142, 131, { &P21865,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAB-730 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2843567 = {S66143, 60, { &P21865,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; TAB-730 Build/IMM76D)*/
const Device UA2585261 = {S66144, 156, { &P18442,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2585274 = {S66145, 156, { &P18442,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7100 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2668980 = {S66146, 156, { &P18442,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-N719 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2668997 = {S66147, 156, { &P18442,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; SCH-N719 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2990623 = {S66148, 156, { &P18442,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7105 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2676188 = {S66149, 128, { &P19782,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B1-A71 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3068405 = {S66150, 141, { &P19778,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP9090 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA202058 = {S66151, 81, { &P17595,&P17296,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en) Presto/2.9.168 Version/11.50*/
const Device UA347332 = {S66152, 121, { &P15364,&P7133,&P22414,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.0.2.0 Chrome/16.0.912.59 Safari/535.7*/
const Device UA347732 = {S66153, 128, { &P15364,&P7133,&P22414,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.0.2.0 Chrome/16.0.912.59 Safari/535.7*/
const Device UA725453 = {S66154, 127, { &P22694,&P6590,&P4223,&P18092 }};	/*Useragent - NetFront/3.5.1(BREW 5.0.2.1; U; en-us; SAMSUNG; NetFront/3.5.1/WAP) Sprint M370 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2365880 = {S66155, 137, { &P20343,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; ZP900S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2383377 = {S66156, 136, { &P20343,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; ZP900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781347 = {S66157, 137, { &P20343,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZP900H Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3056826 = {S66158, 136, { &P20343,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; ZP900 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2429593 = {S66159, 155, { &P18267,&P5043,&P24675,&P18092 }};	/*Useragent - NokiaX2-02/2.0 (11.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiax2-02) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2673346 = {S66160, 150, { &P18640,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT901 Build/7.7.1Q-144_SLS_S240-58) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708262 = {S66161, 143, { &P18640,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT901 Build/7.7.1Q-144_SLS_S240-58) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2975523 = {S66162, 130, { &P19874,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-i705 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3028191 = {S66163, 111, { &P19974,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia111/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3030015 = {S66164, 113, { &P19974,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia111.1/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3068266 = {S66165, 128, { &P21769,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ME301T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA2012939 = {S66166, 129, { &P21029,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KUNO3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2203958 = {S66167, 164, { &P16292,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03R; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2238603 = {S66168, 164, { &P16292,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Galaxy Nexus Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423562 = {S66169, 118, { &P13554,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN79) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2423867 = {S66170, 40, { &P13554,&P5890,&P24673,&P18092 }};	/*Useragent - NOKIAN79UCBrowser/8.7.1.234/28/352/UCWEB*/
const Device UA2458190 = {S66171, 92, { &P13554,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokian79-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2754541 = {S66172, 104, { &P19892,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-SGH-A997/A997UCMD5 SHP/VPP/R5 NetFront/4.2 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA3088303 = {S66173, 106, { &P19892,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-SGH-A997M/A997MUGMC2 SHP/VPP/R5 NetFront/4.2 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA2566904 = {S66174, 131, { &P14908,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6210 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2634773 = {S66175, 131, { &P14908,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P6210 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2676843 = {S66176, 129, { &P17686,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ601 Build/I.7.1-45) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2716347 = {S66177, 139, { &P22935,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9080L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2754377 = {S66178, 141, { &P18504,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LS840 Build/ZVK.IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2974318 = {S66179, 134, { &P18113,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT912 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3057822 = {S66180, 140, { &P18113,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Droid Razr M Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2194699 = {S66181, 133, { &P21639,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; M-MP860S2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2274072 = {S66182, 89, { &P15364,&P5690,&P21956,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1b3pre) Gecko/X Minefield/3.1b3pre*/
const Device UA2439113 = {S66183, 103, { &P15364,&P5690,&P21956,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1b3pre) Gecko/X Minefield/3.1b2pre Firefox/3.0.3*/
const Device UA2639199 = {S66184, 89, { &P15364,&P5690,&P21956,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.1b2pre) Gecko/X Minefield/3.1b2pre*/
const Device UA2642056 = {S66185, 97, { &P15364,&P5690,&P21956,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1a1pre) Gecko/X Firefox Minefield/3.1a1pre*/
const Device UA2510300 = {S66186, 171, { &P14186,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2565566 = {S66187, 171, { &P14186,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2597795 = {S66188, 171, { &P14186,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2599640 = {S66189, 171, { &P14186,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2659919 = {S66190, 172, { &P14186,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2665448 = {S66191, 172, { &P14186,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5830L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2934825 = {S66192, 166, { &P14186,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; GT-S5830 Build/GRWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3020459 = {S66193, 175, { &P14186,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5830 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2567370 = {S66194, 135, { &P17211,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 101G9 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2604077 = {S66195, 132, { &P17211,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 101G9 Build/MR1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2658925 = {S66196, 64, { &P15364,&P17350,&P22273,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; PPC; Win64; x64; Trident/5.0)*/
const Device UA2700544 = {S66197, 62, { &P15364,&P17350,&P22273,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; PPC; .NET CLR; Trident/1.0)*/
const Device UA2792882 = {S66198, 59, { &P15364,&P17350,&P22273,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; PPC; WOW64; Trident/3.0)*/
const Device UA2676019 = {S66199, 144, { &P13064,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I727 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676776 = {S66200, 136, { &P13064,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I727 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA64004 = {S66201, 108, { &P14565,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonW518a/R1EB009 Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.2*/
const Device UA195442 = {S66202, 124, { &P17595,&P18120,&P18209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.0.3.2197 Safari/532.4*/
const Device UA250208 = {S66203, 129, { &P15364,&P7133,&P22410,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.0.1.0 Chrome/14.0.835.163 Safari/535.1*/
const Device UA250249 = {S66204, 122, { &P15364,&P7133,&P22410,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.0.1.0 Chrome/14.0.835.163 Safari/535.1*/
const Device UA455914 = {S66205, 84, { &P17968,&P17979,&P18255,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/7.29530/27.1407; U; en) Presto/2.8.119 Version/11.10*/
const Device UA614004 = {S66206, 84, { &P17968,&P17979,&P18255,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/7.29703/27.1573; U; ru) Presto/2.8.119 Version/11.10*/
const Device UA896482 = {S66207, 84, { &P17968,&P17979,&P18255,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/7.29952/27.1741; U; es) Presto/2.8.119 Version/11.10*/
const Device UA2877035 = {S66208, 87, { &P17968,&P17979,&P18255,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/7.28879/27.1662; U; en-us) Presto/2.8.119 Version/11.10*/
const Device UA2239316 = {S66209, 77, { &P15364,&P5690,&P22554,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:16.1) Gecko/X Firefox/16.1 IceDragon/16.1.0.1*/
const Device UA2486797 = {S66210, 143, { &P23317,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Rikomagic MK802IIIS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2491917 = {S66211, 75, { &P17595,&P17295,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8) Presto/2.12.388 Version/12.13*/
const Device UA2508317 = {S66212, 168, { &P15989,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST21i2 Build/11.0.A.1.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2508655 = {S66213, 168, { &P15989,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST21i Build/11.0.A.0.16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3021582 = {S66214, 172, { &P15989,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.1.8) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3092219 = {S66215, 172, { &P15989,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.4.27) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2540528 = {S66216, 74, { &P21000,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-S5222R/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2738367 = {S66217, 139, { &P22738,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT916 Build/2_26D_2014) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2676820 = {S66218, 139, { &P18736,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One SV Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2939579 = {S66219, 156, { &P23363,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Vodafone 975N Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30 SVN/070HFN2*/
const Device UA2961570 = {S66220, 138, { &P24655,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; IdeaTab A3000-F Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3092663 = {S66221, 138, { &P24655,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; IdeaTab A3000-H Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3125631 = {S66222, 67, { &P24655,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; IdeaTab A3000-H Build/JDQ39)*/
const Device UA3132768 = {S66223, 137, { &P18295,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One X Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA705451 = {S66224, 134, { &P12110,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Legend Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1939711 = {S66225, 138, { &P12110,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_Legend Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018404 = {S66226, 134, { &P12110,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; Legend Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019178 = {S66227, 138, { &P12110,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Legend Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023423 = {S66228, 58, { &P12110,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; Legend Build/GRH55)*/
const Device UA67567 = {S66229, 134, { &P14852,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-SU370 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA563477 = {S66230, 136, { &P14852,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-SU370 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA567854 = {S66231, 137, { &P14852,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-SU370 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018186 = {S66232, 58, { &P14852,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; LG-SU370 Build/FRF91)*/
const Device UA416902 = {S66233, 37, { &P15364,&P18110,&P17213,&P18092 }};	/*Useragent - Opera/8.52 (X11; Linux x86_64; U; en)*/
const Device UA556589 = {S66234, 41, { &P15364,&P18110,&P17213,&P18092 }};	/*Useragent - Opera/8.5 (GOGI; Linux sh4; U) [en] Amino*/
const Device UA778375 = {S66235, 35, { &P15364,&P18110,&P17213,&P18092 }};	/*Useragent - Opera/8.54 (X11; Linux i686; U; pl)*/
const Device UA893419 = {S66236, 35, { &P15364,&P18110,&P17213,&P18092 }};	/*Useragent - Opera/8.51 (X11; Linux i686; U; en)*/
const Device UA1874591 = {S66237, 34, { &P15364,&P18110,&P17213,&P18092 }};	/*Useragent - Opera/8.0 (X11; Linux i686; U; cs)*/
const Device UA2141925 = {S66238, 41, { &P15364,&P18110,&P17213,&P18092 }};	/*Useragent - Opera/8.51 (GOGI; Linux armv5EJl; U) [en]*/
const Device UA2271611 = {S66239, 66, { &P15364,&P18110,&P17213,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux i686; en) Opera 8.02*/
const Device UA2228221 = {S66240, 137, { &P18653,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P870 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2467980 = {S66241, 132, { &P23089,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAB07-200 Build/Xenta) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2562545 = {S66242, 116, { &P14708,&P5204,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia5130c-2/07.95; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2889632 = {S66243, 113, { &P14708,&P5204,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia5130/07.97; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2750068 = {S66244, 145, { &P22911,&P5629,&P15760,&P18092 }};	/*Useragent - Mastone_G9_TD/V2.00 Release/3.19.2012 Mozilla/5.0 (Linux; U; Android 2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2936724 = {S66245, 61, { &P22601,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SHV-E300L Build/JDQ39)*/
const Device UA3036941 = {S66246, 172, { &P22601,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ko-kr; SHV-E300L Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 NAVER(inapp; search; 230; 4.7.7)*/
const Device UA2781741 = {S66247, 152, { &P19770,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020A Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2878949 = {S66248, 152, { &P19770,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020D Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2956690 = {S66249, 152, { &P19770,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2995531 = {S66250, 145, { &P19770,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020A Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3048935 = {S66251, 145, { &P19770,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3104452 = {S66252, 145, { &P19770,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020D Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3029217 = {S66253, 157, { &P18385,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/flo*/
const Device UA3064433 = {S66254, 157, { &P18385,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/flo*/
const Device UA3127819 = {S66255, 154, { &P18385,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.2/grouper*/
const Device UA3130444 = {S66256, 157, { &P18385,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/flo*/
const Device UA3080589 = {S66257, 137, { &P22748,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E425f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081029 = {S66258, 137, { &P22748,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E425g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081177 = {S66259, 136, { &P22748,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E425 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA49129 = {S66260, 70, { &P13905,&P17483,&P3499,&P18092 }};	/*Useragent - Mozilla/4.1 (compatible; Teleca Q7; BMP 1.0.1; U; en) 240X400 LG-MN510*/
const Device UA242542 = {S66261, 159, { &P18016,&P5690,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1 ; en-us; ThinkPad Tablet Build/ThinkPadTablet_A310_02) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA631373 = {S66262, 141, { &P16156,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; NOOK BNRV300 Build/ERD79) Apple WebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA695078 = {S66263, 140, { &P16156,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; NOOK BNRV350 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2404816 = {S66264, 140, { &P16156,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; NOOK BNRV300 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA64015 = {S66265, 83, { &P11924,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonS500i/R8BE Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA12692 = {S66266, 142, { &P17622,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; DAMAGEDHERO200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA44023 = {S66267, 143, { &P17622,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HERO200 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA68044 = {S66268, 148, { &P17622,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HERO200 Build/ERE27) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA287608 = {S66269, 65, { &P17622,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; HERO200 Build/ERE27)*/
const Device UA2022171 = {S66270, 129, { &P17622,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; HERO200 Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA67607 = {S66271, 153, { &P16745,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; T-Mobile myTouch 3G Slide Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67899 = {S66272, 153, { &P16745,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; T-Mobile myTouch 3G Slide Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA68396 = {S66273, 153, { &P16745,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; T-Mobile myTouch 3G Slide Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250124 = {S66274, 153, { &P16745,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; T-Mobile myTouch 3G Slide Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA772303 = {S66275, 153, { &P16745,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; T-Mobile myTouch 3G Slide Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021747 = {S66276, 141, { &P16745,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; T-Mobile myTouch 3G Slide Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA833404 = {S66277, 128, { &P20219,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T05A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2030820 = {S66278, 57, { &P20219,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; T05A Build/IML74K)*/
const Device UA2420747 = {S66279, 128, { &P20219,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T05A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1987471 = {S66280, 77, { &P23083,&P22283,&P4392,&P18092 }};	/*Useragent - Mozilla/4.0 (Brew MP 1.0.2; U; en-us; Kyocera; NetFront/4.1/AMB) Sprint E4277*/
const Device UA2027478 = {S66281, 132, { &P12889,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2619963 = {S66282, 132, { &P12889,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P7300 Build/HTJ85B) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2625666 = {S66283, 133, { &P12889,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7300B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2191310 = {S66284, 137, { &P18664,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A858 Duo Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2223088 = {S66285, 139, { &P14856,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-SU660 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2577148 = {S66286, 61, { &P14856,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-SU660 Build/IMM76L)*/
const Device UA2536723 = {S66287, 166, { &P20146,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9082 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3022846 = {S66288, 170, { &P20146,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9082 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2566886 = {S66289, 127, { &P12765,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2739550 = {S66290, 74, { &P18141,&P22711,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 2.12) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA3081111 = {S66291, 142, { &P23683,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Prism II Build/HuaweiU8686) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA461422 = {S66292, 81, { &P18254,&P17017,&P18245,&P18343 }};	/*Useragent - SimplePie/1.0 b3.2 (Feed Parser; http://simplepie.org/; Allow like Gecko) Build/X*/
const Device UA500977 = {S66293, 75, { &P18254,&P17017,&P18245,&P18343 }};	/*Useragent - SimplePie/1.1 (Feed Parser; http://simplepie.org; Allow like Gecko) Build/X*/
const Device UA555034 = {S66294, 76, { &P18254,&P17017,&P18245,&P18343 }};	/*Useragent - SimplePie/1.0 (Feed Parser; http://simplepie.org/; Allow like Gecko) Build/X*/
const Device UA834555 = {S66295, 85, { &P18254,&P17017,&P18245,&P18343 }};	/*Useragent - SimplePie/1.1.3 (Feed Parser; http://simplepie.org; Allow like Gecko) Build/X (+curl)*/
const Device UA1963824 = {S66296, 77, { &P18254,&P17017,&P18245,&P18343 }};	/*Useragent - SimplePie/1.2.1 (Feed Parser; http://simplepie.org; Allow like Gecko) Build/X*/
const Device UA2578789 = {S66297, 77, { &P18254,&P17017,&P18245,&P18343 }};	/*Useragent - SimplePie/1.3.1 (Feed Parser; http://simplepie.org; Allow like Gecko) Build/X*/
const Device UA2969707 = {S66298, 79, { &P18254,&P17017,&P18245,&P18343 }};	/*Useragent - SimplePie/1.4-dev (Feed Parser; http://simplepie.org; Allow like Gecko) Build/X*/
const Device UA616685 = {S66299, 134, { &P22830,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; VIEWPIA-DR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA893633 = {S66300, 134, { &P22830,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; VIEWPIA-DR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2530447 = {S66301, 172, { &P15649,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2537942 = {S66302, 171, { &P15649,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2537989 = {S66303, 171, { &P15649,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2705678 = {S66304, 171, { &P15649,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3024432 = {S66305, 175, { &P15649,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5570 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2879324 = {S66306, 62, { &P15364,&P18145,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64; rv:22.0) Gecko/X Firefox/22.0*/
const Device UA2966614 = {S66307, 94, { &P13552,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC6-00) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA16314 = {S66308, 27, { &P15210,&P5049,&P3430,&P18092 }};	/*Useragent - SAMSUNG-GT-i8000T Opera 9.5*/
const Device UA63038 = {S66309, 30, { &P15210,&P5049,&P3430,&P18092 }};	/*Useragent - SAMSUNG-GT-i8000/1.0 Opera 9.5*/
const Device UA90775 = {S66310, 50, { &P15210,&P5049,&P3430,&P18092 }};	/*Useragent - Opera/9.5 (Windows NT 5.1; U; en) SAMSUNG-GT-i8000*/
const Device UA788493 = {S66311, 79, { &P13977,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-in; NokiaN73)/UC Browser8.2.0.132/28/444/UCWEB Mobile*/
const Device UA2313223 = {S66312, 30, { &P13977,&P5890,&P24667,&P18092 }};	/*Useragent - NokiaN73/UCWEB8.2.0.116/28/800*/
const Device UA2198517 = {S66313, 140, { &P18292,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ609 Build/7.7.1-85_MZ609-31) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2206122 = {S66314, 128, { &P20745,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ANGB Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2459162 = {S66315, 149, { &P15364,&P5690,&P22775,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; SlimBrowser/6.01.091)*/
const Device UA2566936 = {S66316, 140, { &P23855,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583317 = {S66317, 133, { &P23855,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2633549 = {S66318, 152, { &P15919,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100B Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2634256 = {S66319, 151, { &P15919,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2635414 = {S66320, 151, { &P15919,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.53005*/
const Device UA2773090 = {S66321, 150, { &P15919,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2783706 = {S66322, 150, { &P15919,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3100 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3064985 = {S66323, 130, { &P17323,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6800 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA32543 = {S66324, 128, { &P12500,&P17486,&P17470,&P18092 }};	/*Useragent - MOT-VA76r/R63715_U_71.01.82R BER/2.0 Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA366440 = {S66325, 98, { &P15364,&P17188,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7*/
const Device UA2437221 = {S66326, 127, { &P15767,&P17146,&P22900,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.2 Mobile/9A405 Safari/7534.48.3*/
const Device UA2529881 = {S66327, 125, { &P15767,&P17146,&P22900,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.2 Mobile/9A334 Safari/7534.48.3*/
const Device UA2666308 = {S66328, 140, { &P23008,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-us; Micromax A91 Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2750228 = {S66329, 143, { &P23008,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A91 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2701586 = {S66330, 132, { &P22873,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5120 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2427916 = {S66331, 150, { &P20544,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; Maxwell Lite Build/v1.0.0.ICS.maxwell.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2483870 = {S66332, 144, { &P20544,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Maxwell Lite Build/v1.0.2 X-08:55) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2505731 = {S66333, 143, { &P20544,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Maxwell Lite Build/1.0.2.X-17:05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2711478 = {S66334, 143, { &P20544,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Maxwell_Lite Build/1.0.2-X-19:27) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2510401 = {S66335, 91, { &P17595,&P18207,&P20456,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:18.0) Gecko/X Firefox/18.0 SeaMonkey/2.15.2*/
const Device UA2533274 = {S66336, 91, { &P13163,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaN97) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2752961 = {S66337, 136, { &P20240,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E971 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753867 = {S66338, 149, { &P18317,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300T Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2769945 = {S66339, 150, { &P18317,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300TG Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2893125 = {S66340, 150, { &P18317,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300TL Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2975958 = {S66341, 142, { &P15438,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Sensation Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973966 = {S66342, 151, { &P20017,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021247 = {S66343, 150, { &P20017,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974090 = {S66344, 135, { &P18468,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabA2109A Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3068263 = {S66345, 126, { &P12765,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA17219 = {S66346, 174, { &P13197,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN93-1/20.0.041 SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1, NokiaN93-1/20.0.041 SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA41201 = {S66347, 84, { &P13197,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN93/11.0.034 SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA50216 = {S66348, 19, { &P13197,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN93-1/20.0.041*/
const Device UA64795 = {S66349, 86, { &P13197,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN93-1/20.0.041 SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA609373 = {S66350, 189, { &P16598,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Iconia A500 Build/IMM76D; CyanogenMod-Tegraowners ICS ROM v158 (thor & digetx)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA899822 = {S66351, 189, { &P16598,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Iconia A500 Build/IMM76I; CyanogenMod-Tegraowners ICS ROM v169 (thor & digetx)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA908579 = {S66352, 189, { &P16598,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Iconia A500 Build/IMM76L; CyanogenMod-Tegraowners ICS ROM v170 (thor & digetx)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029259 = {S66353, 143, { &P16598,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A500 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2641318 = {S66354, 155, { &P16598,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; A500 Build/IML77; CyanogenMod-TeamDRH) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA758889 = {S66355, 84, { &P15364,&P18110,&P19905,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; U; de; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 10.62*/
const Device UA797932 = {S66356, 69, { &P15364,&P18110,&P19905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; X11; Linux x86_64; de) Opera 10.62*/
const Device UA904495 = {S66357, 137, { &P21037,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LIFETAB_S9512 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2025345 = {S66358, 137, { &P21037,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LIFETAB_S9512 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2385108 = {S66359, 155, { &P14433,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9003 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2760290 = {S66360, 168, { &P14433,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; GT-I9003 Build/JRO03L; CyanogenMod-remicsjb v2.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2456362 = {S66361, 105, { &P13062,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.16.0.158; en-us; 9790/9.16.0.158) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2526603 = {S66362, 103, { &P13062,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.16.0.57; en-us; 9790/9.16.0.57) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2653479 = {S66363, 103, { &P13062,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.16.0.67; en-us; 9790/9.16.0.67) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2762436 = {S66364, 105, { &P13062,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.16.0.128; en-us; 9790/9.16.0.128) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2889477 = {S66365, 105, { &P13062,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.16.0.129; en-us; 9790/9.16.0.129) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2951877 = {S66366, 103, { &P13062,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.16.0.33; en-us; 9790/9.16.0.33) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2984916 = {S66367, 105, { &P13062,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.16.0.102; en-us; 9790/9.16.0.102) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3044452 = {S66368, 105, { &P13062,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.16.0.159; en-us; 9790/9.16.0.159) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2482049 = {S66369, 153, { &P24547,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SPHS on Hsdroid Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2555272 = {S66370, 151, { &P24547,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-us; SPHS on Hsdroid Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2594550 = {S66371, 153, { &P24547,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; SPHS on Hsdroid Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2855376 = {S66372, 151, { &P24547,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-us; SPHS on Hsdroid Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2987485 = {S66373, 153, { &P24547,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; SPHS on Hsdroid Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3073435 = {S66374, 150, { &P24547,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; SPHS on Hsdroid Build/Android4.0.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2506354 = {S66375, 169, { &P18554,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Barnes & Noble Nook HD Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2679627 = {S66376, 92, { &P12280,&P18117,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 5_1_1; zh-CN; iPh4,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile*/
const Device UA2714940 = {S66377, 92, { &P12280,&P18117,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 5_1_1; zh-CN; iPh3,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile*/
const Device UA2891593 = {S66378, 92, { &P12280,&P18117,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 5_1_1; en-US; iPh2,1) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2678411 = {S66379, 145, { &P19891,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712646 = {S66380, 137, { &P19891,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2863566 = {S66381, 138, { &P19891,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2974295 = {S66382, 137, { &P22382,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975028 = {S66383, 136, { &P20937,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ADR910L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973790 = {S66384, 141, { &P24095,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5306 Build/12.0.A.1.284) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3065373 = {S66385, 130, { &P16044,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; TouchPad Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA13537 = {S66386, 54, { &P16177,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Opera/9.7 (Windows NT 5.1; U; nl)*/
const Device UA13538 = {S66387, 54, { &P16177,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Opera/9.7 (Windows NT 5.1; U; da)*/
const Device UA13539 = {S66388, 54, { &P16177,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Opera/9.7 (Windows NT 5.1; U; de)*/
const Device UA13540 = {S66389, 54, { &P16177,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Opera/9.7 (Windows NT 5.1; U; ru)*/
const Device UA13542 = {S66390, 54, { &P16177,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Opera/9.7 (Windows NT 5.1; U; nb)*/
const Device UA29233 = {S66391, 54, { &P16177,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Opera/9.7 (Windows NT 5.1; U; it)*/
const Device UA34863 = {S66392, 54, { &P16177,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Opera/9.7 (Windows NT 5.1; U; pt)*/
const Device UA49295 = {S66393, 57, { &P16177,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Opera/9.7 (Windows NT 5.1; U; es-ES)*/
const Device UA54798 = {S66394, 54, { &P16177,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Opera/9.7 (Windows NT 5.1; U; en)*/
const Device UA62486 = {S66395, 54, { &P16177,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Opera/9.7 (Windows NT 5.1; U; sv)*/
const Device UA66689 = {S66396, 54, { &P16177,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Opera/9.7 (Windows NT 5.1; U; fr)*/
const Device UA20940 = {S66397, 61, { &P11963,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia6111/2.0 (03.82) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA155229 = {S66398, 67, { &P17595,&P17296,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; PPC Mac OS X 10.6.5; Tasman 1.0)*/
const Device UA177426 = {S66399, 67, { &P17595,&P17296,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; PPC Mac OS X 10.6.7; Tasman 1.0)*/
const Device UA197861 = {S66400, 67, { &P17595,&P17296,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; PPC Mac OS X 10.6.8; Tasman 1.0)*/
const Device UA276907 = {S66401, 67, { &P17595,&P17296,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; PPC Mac OS X 10.6.4; Tasman 1.0)*/
const Device UA300268 = {S66402, 67, { &P17595,&P17296,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; PPC Mac OS X 10.6.6; Tasman 1.0)*/
const Device UA469086 = {S66403, 67, { &P17595,&P17296,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; PPC Mac OS X 10.6.3; Tasman 1.0)*/
const Device UA523814 = {S66404, 67, { &P17595,&P17296,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; PPC Mac OS X 10.6.2; Tasman 1.0)*/
const Device UA304321 = {S66405, 143, { &P15767,&P17145,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.10*/
const Device UA310683 = {S66406, 134, { &P15767,&P17145,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA666785 = {S66407, 141, { &P15767,&P17145,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8C148 Safari/6531.22.7*/
const Device UA827805 = {S66408, 251, { &P15767,&P17145,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148,Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2372904 = {S66409, 141, { &P15767,&P17145,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2480502 = {S66410, 140, { &P15767,&P17145,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2566702 = {S66411, 140, { &P18465,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583215 = {S66412, 141, { &P18465,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire2 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2600042 = {S66413, 139, { &P18465,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Amazon Kindle Fire Build/JOP40C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2707903 = {S66414, 141, { &P18465,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Amazon Kindle Fire Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2676993 = {S66415, 135, { &P18468,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; IdeaTabA2109A Build/JRO03R) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2756671 = {S66416, 137, { &P20975,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-AS730 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA26349 = {S66417, 107, { &P13586,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-A187/A187UCJH2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA923961 = {S66418, 18, { &P18254,&P17017,&P22949,&P18092 }};	/*Useragent - Apple-PubSub/65.32*/
const Device UA2167562 = {S66419, 78, { &P11945,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; gt-s5233s) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2412124 = {S66420, 75, { &P11945,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; vi; gt-s5233s) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2388653 = {S66421, 140, { &P20006,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT30a Build/7.0.A.3.195) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2437211 = {S66422, 43, { &P15364,&P18570,&P22900,&P18092 }};	/*Useragent - Mercury/7.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2466473 = {S66423, 133, { &P17976,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Tablet PC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2469878 = {S66424, 138, { &P17976,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Tablet PC 3000 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2701415 = {S66425, 127, { &P17976,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PAD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2782882 = {S66426, 139, { &P15364,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; v89_zlh_hd Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2797400 = {S66427, 127, { &P15364,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; p4wifi Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2797409 = {S66428, 140, { &P15364,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; v89_gq2008s Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867090 = {S66429, 138, { &P15364,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Transformer Prime Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2870779 = {S66430, 136, { &P15364,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-l900 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879280 = {S66431, 133, { &P15364,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Q508 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892417 = {S66432, 133, { &P15364,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; X508 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892575 = {S66433, 121, { &P15364,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; MB865) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892606 = {S66434, 140, { &P15364,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; v89_gq3008s Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892642 = {S66435, 131, { &P15364,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Z2 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892783 = {S66436, 133, { &P15364,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; 7102 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893014 = {S66437, 134, { &P15364,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; H3088 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2894241 = {S66438, 133, { &P15364,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; B943 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937703 = {S66439, 133, { &P15364,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; H920 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937809 = {S66440, 141, { &P15364,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; v89_jbl1a668 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937981 = {S66441, 132, { &P15364,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; TS4 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2957235 = {S66442, 133, { &P15364,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; A19S Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2997000 = {S66443, 142, { &P15364,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Aveo NOTE II+ Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3049852 = {S66444, 137, { &P15364,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; One S C2 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3124907 = {S66445, 129, { &P15364,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5220 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2885457 = {S66446, 93, { &P13123,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5360) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2943352 = {S66447, 90, { &P13123,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; vi; GT-S5360) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA626619 = {S66448, 136, { &P21056,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM103SH Build/S0006) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA899116 = {S66449, 136, { &P21056,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM103SH Build/S0012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1989706 = {S66450, 60, { &P21056,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SBM103SH Build/S0012)*/
const Device UA2943815 = {S66451, 136, { &P21056,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM103SH Build/S0014) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA876537 = {S66452, 166, { &P15555,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; myTouch_4G_Slide Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA911442 = {S66453, 162, { &P15555,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; myTouch_4G_Slide Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029736 = {S66454, 136, { &P21031,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ZP500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528178 = {S66455, 137, { &P21031,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZP500+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528357 = {S66456, 154, { &P20541,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PAP4500DUO Build/PrestigioPAP4500DUO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2756889 = {S66457, 136, { &P12814,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I777 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2958297 = {S66458, 137, { &P12814,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-I777 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754943 = {S66459, 130, { &P15932,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P3110 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2877843 = {S66460, 130, { &P15932,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P3110 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2879144 = {S66461, 137, { &P15932,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P3110 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2876112 = {S66462, 141, { &P20399,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PantechP8010 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA908809 = {S66463, 73, { &P16329,&P18443,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PLAYSTATION 3 4.20) AppleWebKit/531.22.8 (KHTML, like Gecko)*/
const Device UA1993859 = {S66464, 132, { &P19877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T869 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422570 = {S66465, 151, { &P18557,&P20706,&P24673,&P18092 }};	/*Useragent - Nokia308/2.0 (05.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia308) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2555502 = {S66466, 137, { &P21448,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SH530U Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2765046 = {S66467, 59, { &P21448,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SH530U Build/JZO54K)*/
const Device UA2699061 = {S66468, 134, { &P23855,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; WM8850-mid Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2952916 = {S66469, 63, { &P23855,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; WM8850-mid Build/JRO03H)*/
const Device UA2999325 = {S66470, 137, { &P23855,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; WM8850:X6-10V Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2973660 = {S66471, 140, { &P16292,&P21637,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Galaxy Nexus Build/JWR66V) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2893908 = {S66472, 140, { &P18582,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT28h Build/6.2.B.0.211) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3020881 = {S66473, 140, { &P18582,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT28i Build/6.2.B.0.211) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3023747 = {S66474, 87, { &P15364,&P17017,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3025936 = {S66475, 95, { &P15364,&P17017,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; jbed-fastbcc) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3030019 = {S66476, 90, { &P15364,&P17017,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mozilla) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3032616 = {S66477, 88, { &P15364,&P17017,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; spice) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3034284 = {S66478, 90, { &P15364,&P17017,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; generic) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3058391 = {S66479, 91, { &P15364,&P17017,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sunmicro) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3059020 = {S66480, 90, { &P15364,&P17017,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; karbonn) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3080028 = {S66481, 100, { &P15364,&P17017,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; opera) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile TRUSTED/1.0*/
const Device UA3092392 = {S66482, 90, { &P15364,&P17017,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; Pt-BR; mozilla) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3092797 = {S66483, 88, { &P15364,&P17017,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; s8000) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3092975 = {S66484, 100, { &P15364,&P17017,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; spice) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile TRUSTED/1.0*/
const Device UA3093160 = {S66485, 99, { &P15364,&P17017,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile TRUSTED/1.0*/
const Device UA3108448 = {S66486, 97, { &P15364,&P17017,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; spreadtrum6530) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3108705 = {S66487, 96, { &P15364,&P17017,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; 11a h700tecno) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA904430 = {S66488, 115, { &P14669,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-00/08.25; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA2422176 = {S66489, 70, { &P16356,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GT-N7000; 800*1280) UCWEB7.9.0.94/139/444*/
const Device UA2457039 = {S66490, 70, { &P16356,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; GT-N7000; 800*1280) UCWEB7.9.0.94/139/444*/
const Device UA2435616 = {S66491, 131, { &P20984,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LT7033E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2740416 = {S66492, 156, { &P20705,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC One Build/JZO54K; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2894888 = {S66493, 157, { &P20705,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/m7att*/
const Device UA3018633 = {S66494, 140, { &P23211,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6802 Build/14.1.B.0.461) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3027673 = {S66495, 133, { &P23211,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6802 Build/14.1.B.0.461) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2975020 = {S66496, 134, { &P23700,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; N9101 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3096830 = {S66497, 141, { &P12280,&P22882,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/11A465 Safari/8536.25*/
const Device UA3130349 = {S66498, 143, { &P12280,&P22882,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/11A501 Safari/8536.25*/
const Device UA2463057 = {S66499, 85, { &P15364,&P16110,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.8.1.20) Gecko/X Firefox/2.0.0.20*/
const Device UA2580312 = {S66500, 135, { &P18314,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; M032 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2855165 = {S66501, 57, { &P18314,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; M032 Build/JRO03H)*/
const Device UA2676238 = {S66502, 136, { &P19911,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E970 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2778923 = {S66503, 136, { &P20255,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; R8113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865619 = {S66504, 58, { &P20255,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; R8113 Build/JRO03C)*/
const Device UA2171189 = {S66505, 138, { &P12942,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I927 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2259215 = {S66506, 71, { &P15364,&P16110,&P18691,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:16.0) Gecko/X Thunderbird/16.0.2*/
const Device UA2396430 = {S66507, 138, { &P20366,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; ZTE T82 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2562397 = {S66508, 129, { &P22231,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; T-17P Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3071926 = {S66509, 58, { &P22231,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; T-17P Build/JRO03H)*/
const Device UA2577964 = {S66510, 137, { &P20119,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_S9714 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2729897 = {S66511, 140, { &P22590,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I717M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2754251 = {S66512, 142, { &P19764,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6502 Build/10.1.1.A.1.253) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2866256 = {S66513, 125, { &P21239,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; M9 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2998998 = {S66514, 54, { &P21239,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; M9 Build/JDQ39)*/
const Device UA9900 = {S66515, 104, { &P11709,&P5795,&P3978,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Series80/2.0 Nokia9300/05.22 Profile/MIDP-2.0 Configuration/CLDC-1.1)*/
const Device UA61674 = {S66516, 104, { &P11709,&P5795,&P3978,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Series80/2.0 Nokia9300i/5.50 Profile/MIDP-2.0 Configuration/CLDC-1.1)*/
const Device UA80998 = {S66517, 99, { &P11709,&P5795,&P3978,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Series80/2.0 Nokia9300i Profile/MIDP-2.0 Configuration/CLDC-1.1)*/
const Device UA568284 = {S66518, 144, { &P17595,&P17296,&P24745,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Stainless/0.8 like Version/5.1 Safari/534.48.3*/
const Device UA690676 = {S66519, 134, { &P17595,&P17296,&P24745,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Stainless/0.8 like Version/5.1 Safari/534.48.3*/
const Device UA2127579 = {S66520, 47, { &P15364,&P5448,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01; Windows CE)*/
const Device UA2658809 = {S66521, 58, { &P15364,&P5448,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; WindowsCE; Trident/5.0)*/
const Device UA2791808 = {S66522, 63, { &P15364,&P5448,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; 68K; Tablet PC; Trident/2.0)*/
const Device UA2851532 = {S66523, 63, { &P15364,&P5448,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; PPC; Tablet PC; Trident/2.0)*/
const Device UA2424489 = {S66524, 135, { &P17595,&P17297,&P22463,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11*/
const Device UA2595540 = {S66525, 135, { &P17595,&P17297,&P22463,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1300.0 Iron/23.0.1300.0 Safari/537.11*/
const Device UA2507666 = {S66526, 94, { &P20134,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2530097 = {S66527, 136, { &P20480,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; JY-G3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2852639 = {S66528, 58, { &P20480,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; JY-G3 Build/JZO54K)*/
const Device UA3049041 = {S66529, 137, { &P20480,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; JY-G3N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2565404 = {S66530, 139, { &P21886,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; SmartPad 85S23G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2707348 = {S66531, 134, { &P21886,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; M-MP85S23G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2754583 = {S66532, 130, { &P12879,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7510 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2782706 = {S66533, 130, { &P12879,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P7510 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2897822 = {S66534, 156, { &P18673,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S7562 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056939 = {S66535, 157, { &P18673,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-S7562i Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3025151 = {S66536, 157, { &P15575,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9100G Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3028940 = {S66537, 156, { &P15575,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-I9100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3062556 = {S66538, 156, { &P15575,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-I9100 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA4246 = {S66539, 96, { &P11734,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Kaiser/3.28.161.1*/
const Device UA12325 = {S66540, 71, { &P11734,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_P4550 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA23737 = {S66541, 73, { &P11734,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_TyTN_II Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA24270 = {S66542, 96, { &P11734,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Kaiser/3.28.162.2*/
const Device UA24267 = {S66543, 96, { &P11734,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Kaiser/3.08.161.0*/
const Device UA28981 = {S66544, 79, { &P11734,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_P4550/TYTN_II Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA28982 = {S66545, 97, { &P11734,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_TyTN_II Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) HTC_TyTN_II Mozilla/4.0*/
const Device UA66428 = {S66546, 105, { &P11734,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC P4550; OpVer 24.328.1.610*/
const Device UA87842 = {S66547, 105, { &P11734,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC P4550; OpVer 24.181.1.612*/
const Device UA15882 = {S66548, 159, { &P16098,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-GT400/V10d Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA15883 = {S66549, 159, { &P16098,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-GT400/V10e Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA33403 = {S66550, 141, { &P16098,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GT400-Bouygues/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA53280 = {S66551, 139, { &P16098,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GT400-Orange/V10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63316 = {S66552, 132, { &P16098,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GT400/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63663 = {S66553, 159, { &P16098,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-GT400/V10g Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA63662 = {S66554, 159, { &P16098,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-GT400/V10c Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA97465 = {S66555, 114, { &P15364,&P7133,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Version/3.1.2 Safari/525.21*/
const Device UA272913 = {S66556, 114, { &P15364,&P7133,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.17*/
const Device UA486980 = {S66557, 112, { &P15364,&P7133,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13*/
const Device UA642631 = {S66558, 131, { &P22024,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; dL1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2755428 = {S66559, 137, { &P18687,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P705g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2756753 = {S66560, 136, { &P18687,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P705 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2770057 = {S66561, 137, { &P18687,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P708g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2800285 = {S66562, 137, { &P18687,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P705f Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2779675 = {S66563, 134, { &P19971,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; GT7000DC Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2988112 = {S66564, 137, { &P19971,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; MT7008 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2991373 = {S66565, 137, { &P19971,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MT7007 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3024936 = {S66566, 130, { &P19971,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MT7001 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3058341 = {S66567, 59, { &P19971,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; MT7001 Build/JRO03C)*/
const Device UA3076926 = {S66568, 129, { &P19971,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; T7005 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2943929 = {S66569, 152, { &P18219,&P5843,&P24678,&P18092 }};	/*Useragent - Nokia202/2.0 (20.52) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia202) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3057221 = {S66570, 152, { &P18219,&P5843,&P24678,&P18092 }};	/*Useragent - Nokia203/2.0 (20.52) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia203) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2975069 = {S66571, 138, { &P19849,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7560M Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA744856 = {S66572, 166, { &P14913,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/Chicken Tits by Bloodawn)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA908989 = {S66573, 166, { &P14913,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/Chicken-Tits by Bloodawn)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1985222 = {S66574, 153, { &P14913,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2998638 = {S66575, 154, { &P14913,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2295284 = {S66576, 64, { &P15364,&P18145,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD x86_64; rv:2.0) Gecko/X Firefox/3.6.12*/
const Device UA2258136 = {S66577, 80, { &P17595,&P17297,&P18691,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:16.0) Gecko/X Thunderbird/16.0.2*/
const Device UA2235275 = {S66578, 123, { &P14983,&P5055,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; NokiaC7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/51/352/UCWEB Mobile*/
const Device UA2341971 = {S66579, 32, { &P14983,&P5055,&P24672,&P18092 }};	/*Useragent - NokiaC7-00/UCWEB8.6.0.180/51/800*/
const Device UA2403874 = {S66580, 129, { &P14983,&P5055,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiac7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA2257334 = {S66581, 135, { &P21048,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; W732 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2746458 = {S66582, 57, { &P21048,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; W732 Build/IML74K)*/
const Device UA2601053 = {S66583, 140, { &P22757,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MITO_A300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2782265 = {S66584, 140, { &P22757,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MITO A300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2988535 = {S66585, 62, { &P22757,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MITO_A300 Build/IMM76D)*/
const Device UA2996764 = {S66586, 62, { &P22757,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MITO A300 Build/IMM76D)*/
const Device UA3031481 = {S66587, 156, { &P21261,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2975548 = {S66588, 133, { &P20011,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Next7P12-8G Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3020716 = {S66589, 130, { &P20011,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Next7P12 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2996340 = {S66590, 138, { &P24039,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9190 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3015322 = {S66591, 60, { &P24039,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9190 Build/JDQ39)*/
const Device UA2468883 = {S66592, 138, { &P13644,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; PC36100 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2603663 = {S66593, 137, { &P13644,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; PC36100 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659614 = {S66594, 134, { &P13644,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; EVO Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665826 = {S66595, 134, { &P13644,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; EVO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2700859 = {S66596, 56, { &P13644,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; EVO Build/JRO03C)*/
const Device UA2935295 = {S66597, 141, { &P13644,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Evo 4G LTE Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936939 = {S66598, 138, { &P13644,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; PC36100 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2677286 = {S66599, 137, { &P18488,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L300 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2863508 = {S66600, 130, { &P18488,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L300 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2960414 = {S66601, 103, { &P12193,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac5-00) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3015804 = {S66602, 105, { &P12193,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac5-00.2) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3059761 = {S66603, 96, { &P12193,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaC5-00.2) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3061663 = {S66604, 94, { &P12193,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaC5-00) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA87812 = {S66605, 49, { &P15364,&P17017,&P3976,&P18092 }};	/*Useragent - Mozilla/5.0 R800 like Gecko/20060426 Netfront/3.4*/
const Device UA88896 = {S66606, 73, { &P15364,&P17017,&P3976,&P18092 }};	/*Useragent - Yoigo F233/1.0 ACS-NF/3.4 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA89078 = {S66607, 52, { &P15364,&P17017,&P3976,&P18092 }};	/*Useragent - samr810 Netfront/3.4 Mozilla/5.0 like Gecko/20060426*/
const Device UA493404 = {S66608, 98, { &P15364,&P17017,&P3976,&P18092 }};	/*Useragent - Mozilla/5.0 (DigitalMediaPlayer; Walkman/NWZ-X1001 Series/1.00; like Gecko; wireless) NetFront/3.4*/
const Device UA563658 = {S66609, 68, { &P15364,&P17017,&P3976,&P18092 }};	/*Useragent - Mozilla/3.0(WILLCOM;KYOCERA/WX340K/2;3.0.3.11.X/1/C256) NetFront/3.4*/
const Device UA682795 = {S66610, 98, { &P15364,&P17017,&P3976,&P18092 }};	/*Useragent - Mozilla/5.0 (DigitalMediaPlayer; Walkman/NWZ-X1000 Series/1.00; like Gecko; wireless) NetFront/3.4*/
const Device UA2229609 = {S66611, 125, { &P15547,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-c6712) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile*/
const Device UA2425992 = {S66612, 144, { &P20507,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 7 Build/1.1.3 X-11:40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2427638 = {S66613, 144, { &P20507,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 7 Build/1.0.3 X-12:51) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2427925 = {S66614, 144, { &P20507,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 7 Build/1.0.2 X-15:15) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2577961 = {S66615, 144, { &P20507,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; tablet Fnac 7 Build/1.0.0 X-15:30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3099299 = {S66616, 107, { &P15364,&P7133,&P20804,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Qt/4.6.2 Safari/532.4*/
const Device UA688637 = {S66617, 129, { &P22814,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; PC906 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA912468 = {S66618, 129, { &P22814,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; PC906 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032857 = {S66619, 130, { &P23889,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; PAD703 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2405535 = {S66620, 137, { &P23889,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; PAD703 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2840441 = {S66621, 59, { &P23889,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; PAD703 Build/IML74K)*/
const Device UA2794887 = {S66622, 139, { &P23286,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TECNO F7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3068278 = {S66623, 140, { &P22598,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6506 Build/10.1.A.1.350) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094869 = {S66624, 148, { &P12280,&P22882,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/11A465 Safari/8536.25*/
const Device UA3128976 = {S66625, 150, { &P12280,&P22882,&P19801,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/23.0.1271.100 Mobile/11A501 Safari/8536.25*/
const Device UA230579 = {S66626, 134, { &P21099,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; IS11T Build/FGK400) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA660753 = {S66627, 134, { &P21099,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; IS11T Build/FGK401) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369473 = {S66628, 133, { &P18603,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; L-02D Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034337 = {S66629, 57, { &P18603,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; L-02D Build/GWK74)*/
const Device UA740278 = {S66630, 84, { &P15364,&P7133,&P18197,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; de; rv:1.9.1.16) Gecko/X Thunderbird/3.0.11*/
const Device UA880217 = {S66631, 52, { &P15364,&P17017,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (SaferSurf rv:12.0) Gecko/X Firefox/12.0*/
const Device UA891802 = {S66632, 57, { &P15364,&P17017,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (masking-agent; rv:12.0) Gecko/X Firefox/12.0*/
const Device UA895844 = {S66633, 21, { &P15364,&P17017,&P18101,&P18092 }};	/*Useragent - Firefox/12.0 YB/5.2.2*/
const Device UA1885181 = {S66634, 62, { &P15364,&P17285,&P4551,&P18092 }};	/*Useragent - Mozilla/6.0 (compatible;MSIE 5.0;  Windows NT 6.0; QQDownload)*/
const Device UA2242381 = {S66635, 62, { &P15364,&P17285,&P4551,&P18092 }};	/*Useragent - Mozilla/3.0 (compatible;MSIE 5.0;  Windows NT 6.0; QQDownload)*/
const Device UA2502125 = {S66636, 63, { &P15364,&P17285,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; Windows NT 6.0; Trident/4.0)*/
const Device UA2720814 = {S66637, 63, { &P15364,&P17285,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 6.0; Trident/4.1)*/
const Device UA2029709 = {S66638, 131, { &P20346,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC407 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2460287 = {S66639, 137, { &P20346,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MIDC407 Build/eng.glwx.X.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2562118 = {S66640, 138, { &P20346,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC407 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659800 = {S66641, 132, { &P20346,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MIDC407 Build/MIDC407) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3099240 = {S66642, 131, { &P20346,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MIDC407 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2237745 = {S66643, 77, { &P18088,&P22740,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; Maemo; Opera Mobi/16) Presto/2.11.355 Version/12.10*/
const Device UA2566752 = {S66644, 137, { &P14877,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9000 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567394 = {S66645, 138, { &P14877,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9000 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566905 = {S66646, 164, { &P18250,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Motorola RAZR MAXX Build/6.7.2-180_DHD-16_NA-27) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567961 = {S66647, 138, { &P18459,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS950 4G Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2670736 = {S66648, 138, { &P15438,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC-Z710a Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2671821 = {S66649, 142, { &P15438,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Sensation Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2672542 = {S66650, 142, { &P15438,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2678423 = {S66651, 148, { &P15438,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710a Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA367451 = {S66652, 109, { &P15364,&P18110,&P22581,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.12 (KHTML, like Gecko) Puffin/1.3.4232M Safari/534.12*/
const Device UA427398 = {S66653, 51, { &P18254,&P18570,&P17470,&P18092 }};	/*Useragent - FreeRSSReader/1.9.5 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA633476 = {S66654, 49, { &P18254,&P18570,&P17470,&P18092 }};	/*Useragent - MobileRSSFree/3.4 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA657666 = {S66655, 45, { &P18254,&P18570,&P17470,&P18092 }};	/*Useragent - MobileRSS/3.4 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA891509 = {S66656, 52, { &P18254,&P18570,&P17470,&P18092 }};	/*Useragent - Pics4Facebook/9.15.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2318644 = {S66657, 46, { &P18254,&P18570,&P17470,&P18092 }};	/*Useragent - RSSReader/1.04 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2354989 = {S66658, 45, { &P18254,&P18570,&P17470,&P18092 }};	/*Useragent - RSSRadio/2.07 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2356693 = {S66659, 50, { &P18254,&P18570,&P17470,&P18092 }};	/*Useragent - FeeddlerRSS/1.12.5 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA611826 = {S66660, 122, { &P15364,&P17285,&P18208,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 6.0; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.2.2.6613 Safari/532.4*/
const Device UA2374913 = {S66661, 98, { &P15364,&P17285,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.0.6000.0);en;kml:2.2;client:Free;type:default)*/
const Device UA1963451 = {S66662, 134, { &P21335,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; K-Touch W619 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2383567 = {S66663, 141, { &P21335,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; K-Touch_W619 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2452103 = {S66664, 160, { &P21335,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3 YunOs 1.0.0.3; zh-cn; K-Touch W619 Build/AliyunOs-2012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2503816 = {S66665, 133, { &P21335,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; W619 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2817191 = {S66666, 65, { &P21335,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; K-Touch W619 Build/GRK39F)*/
const Device UA2817346 = {S66667, 141, { &P21335,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; K-Touch W619 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2356690 = {S66668, 139, { &P21028,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Smartfren Tab 7 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2693309 = {S66669, 129, { &P21028,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Tab 7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2515532 = {S66670, 133, { &P21522,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GEM10313S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2879 = {S66671, 68, { &P13921,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V3re/0E.43.0AR MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22810 = {S66672, 67, { &P13921,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V3r/08.BD.43R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA50996 = {S66673, 67, { &P13921,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V3e/08.03.05R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA55871 = {S66674, 66, { &P13921,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V3/0E.42.10R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA62317 = {S66675, 67, { &P13921,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V3v/08.BD.B3R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA86295 = {S66676, 67, { &P13921,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V3t/0E.C8.0DR MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA651939 = {S66677, 161, { &P12496,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-I897 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA910820 = {S66678, 101, { &P15364,&P17188,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11*/
const Device UA2262896 = {S66679, 76, { &P17595,&P18578,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookAir4%2C2)*/
const Device UA2384737 = {S66680, 71, { &P17595,&P18578,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (iMac10%2C1)*/
const Device UA2421201 = {S66681, 71, { &P17595,&P18578,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/520.5.1 Darwin/11.4.2 (i386) (MacBook2%2C1)*/
const Device UA2552990 = {S66682, 76, { &P17595,&P18578,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)*/
const Device UA2652811 = {S66683, 70, { &P17595,&P18578,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/520.5.1 Darwin/11.4.2 (i386) (MacPro1%2C1)*/
const Device UA2712951 = {S66684, 146, { &P20253,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; i-mobile IQ 5.1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2737686 = {S66685, 147, { &P20253,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; i-mobile IQ 5.1A Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102818 = {S66686, 69, { &P20253,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; i-mobile IQ 5.1A Build/JZO54K)*/
const Device UA3023699 = {S66687, 140, { &P17968,&P18111,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9737AP Mobile*/
const Device UA3034113 = {S66688, 140, { &P17968,&P18111,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP Mobile*/
const Device UA2894573 = {S66689, 160, { &P14708,&P5204,&P24678,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5130c-2) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2959789 = {S66690, 154, { &P14708,&P5204,&P24678,&P18092 }};	/*Useragent - Nokia5130/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5130) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2974711 = {S66691, 137, { &P12814,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I777 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975008 = {S66692, 136, { &P12814,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-i777 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974622 = {S66693, 142, { &P19842,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Butterfly Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3030880 = {S66694, 152, { &P15785,&P5843,&P24679,&P18092 }};	/*Useragent - Nokia201/2.0 (11.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia201) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA152290 = {S66695, 39, { &P15364,&P23112,&P18419,&P18092 }};	/*Useragent - AmigaVoyager/3.4.4 (MorphOS/PPC native)*/
const Device UA245457 = {S66696, 132, { &P23859,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T100 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA535877 = {S66697, 27, { &P18088,&P17017,&P24642,&P18092 }};	/*Useragent - J2ME/UCWEB7.0.3.45/139/7687*/
const Device UA628614 = {S66698, 27, { &P18088,&P17017,&P24642,&P18092 }};	/*Useragent - J2ME/UCWEB7.0.3.45/140/7704*/
const Device UA632155 = {S66699, 27, { &P18088,&P17017,&P24642,&P18092 }};	/*Useragent - J2ME/UCWEB7.0.3.45/140/7665*/
const Device UA638370 = {S66700, 27, { &P18088,&P17017,&P24642,&P18092 }};	/*Useragent - J2ME/UCWEB7.0.3.45/140/7689*/
const Device UA1862920 = {S66701, 27, { &P18088,&P17017,&P24642,&P18092 }};	/*Useragent - J2ME/UCWEB7.0.3.45/139/7602*/
const Device UA1886879 = {S66702, 27, { &P18088,&P17017,&P24642,&P18092 }};	/*Useragent - J2ME/UCWEB7.0.3.45/139/7685*/
const Device UA1889540 = {S66703, 27, { &P18088,&P17017,&P24642,&P18092 }};	/*Useragent - J2ME/UCWEB7.0.3.45/139/7672*/
const Device UA1893173 = {S66704, 27, { &P18088,&P17017,&P24642,&P18092 }};	/*Useragent - J2ME/UCWEB7.0.3.45/139/7694*/
const Device UA1894363 = {S66705, 27, { &P18088,&P17017,&P24642,&P18092 }};	/*Useragent - J2ME/UCWEB7.0.3.45/139/7705*/
const Device UA1905822 = {S66706, 27, { &P18088,&P17017,&P24642,&P18092 }};	/*Useragent - J2ME/UCWEB7.0.3.45/139/7695*/
const Device UA2261515 = {S66707, 146, { &P18476,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I577 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2387364 = {S66708, 131, { &P20344,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; DIT9720 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2419216 = {S66709, 154, { &P15364,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; kingdom Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422728 = {S66710, 155, { &P15364,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; tervigon Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2451570 = {S66711, 155, { &P15364,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Crescent Build/JRO03C; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2502729 = {S66712, 155, { &P15364,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Flyer Build/JZO54K; CyanogenMod-Flyer) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528777 = {S66713, 154, { &P15364,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ThinkPadTablet Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2554964 = {S66714, 155, { &P15364,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Atrix 4G Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555850 = {S66715, 171, { &P15364,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Live with Walkman Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2982888 = {S66716, 162, { &P15364,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Notion Ink ADAM Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987801 = {S66717, 170, { &P15364,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; GT-i9003 Build/JZO54K; CyanogenMod-EHNDROIX II i9003) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2989758 = {S66718, 152, { &P15364,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; warp2 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2421584 = {S66719, 134, { &P20469,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N51 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2671093 = {S66720, 142, { &P14969,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire SV Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677178 = {S66721, 141, { &P14969,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714583 = {S66722, 141, { &P14969,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire S Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2716117 = {S66723, 94, { &P13324,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaN8-00) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2768580 = {S66724, 133, { &P23896,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; S5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3073231 = {S66725, 55, { &P23896,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; S5 Build/JOP40D)*/
const Device UA2947546 = {S66726, 157, { &P16356,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3033285 = {S66727, 157, { &P16356,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-N7000 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA6535 = {S66728, 120, { &P17595,&P17017,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OSX 10_5_7; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA625429 = {S66729, 68, { &P17595,&P17017,&P17316,&P18092 }};	/*Useragent - Safari/6531.21.10 CFNetwork/454.5 Darwin/10.2.0 (i386) (MacBookPro5)*/
const Device UA775913 = {S66730, 65, { &P17595,&P17017,&P17316,&P18092 }};	/*Useragent - Safari/6531.9 CFNetwork/454.4 Darwin/10.1.1 (i386) (MacBook6%2C1)*/
const Device UA2767142 = {S66731, 135, { &P21472,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; A111 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088468 = {S66732, 57, { &P21472,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; A111 Build/JZO54K)*/
const Device UA3115088 = {S66733, 139, { &P15364,&P17350,&P23486,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; ; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.6.6.X (Second Life Release; default skin) Safari/533.3*/
const Device UA622902 = {S66734, 133, { &P15364,&P17818,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.7 (KHTML, like Gecko) Ubuntu/11.04 Chromium/16.0.912.77 Chrome/16.0.912.77 Safari/535.7*/
const Device UA638504 = {S66735, 135, { &P15364,&P17818,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.7 (KHTML, like Gecko) Ubuntu/11.04 Chromium/16.0.912.77 Chrome/16.0.912.77 Safari/535.7*/
const Device UA2030167 = {S66736, 138, { &P20396,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; ZTE R22 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2420679 = {S66737, 138, { &P23909,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID7032 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2431548 = {S66738, 217, { &P15985,&P6437,&P17373,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Opera/9.80 (Android; Opera Mini/6.5.26969/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2620280 = {S66739, 215, { &P15985,&P6437,&P17373,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Opera/9.80 (Android; Opera Mini/6.5.26969/28.3126; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2353589 = {S66740, 141, { &P21302,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Fly IQ441; Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2420940 = {S66741, 136, { &P21302,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; IQ441 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950692 = {S66742, 58, { &P21302,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; IQ441 Build/IMM76D)*/
const Device UA2739198 = {S66743, 69, { &P15364,&P18118,&P22772,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:24.0) Gecko/X Firefox/24.0*/
const Device UA249279 = {S66744, 137, { &P21529,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SYTABEX7-2 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023231 = {S66745, 61, { &P21529,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; SYTABEX7-2 Build/FRF85B)*/
const Device UA810041 = {S66746, 157, { &P13644,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PC36100 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA908416 = {S66747, 153, { &P13644,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PC36100 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2425504 = {S66748, 171, { &P13644,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4 SM 2.0.3; en-us; PC36100 Build/IMM76L; CyanogenMod-SmurfMod 9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2229020 = {S66749, 114, { &P14711,&P5204,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-01/11.20; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2258168 = {S66750, 80, { &P17595,&P17295,&P18691,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:16.0) Gecko/X Thunderbird/16.0.2*/
const Device UA2632900 = {S66751, 151, { &P15919,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P3100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2632918 = {S66752, 151, { &P15919,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3100 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2633897 = {S66753, 152, { &P15919,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3100B Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2773894 = {S66754, 150, { &P15919,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3100 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2774298 = {S66755, 150, { &P15919,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P3100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3111813 = {S66756, 153, { &P15767,&P18117,&P24623,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10 UCBrowser/2.4.2.348*/
const Device UA3649 = {S66757, 98, { &P14074,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Palm-Treo850e BM BMC*/
const Device UA5049 = {S66758, 92, { &P14074,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Telus Treo850e*/
const Device UA7541 = {S66759, 93, { &P14074,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Alltel_Treo850e*/
const Device UA18470 = {S66760, 93, { &P14074,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Sprint Treo850e*/
const Device UA18471 = {S66761, 96, { &P14074,&P5448,&P4808,&P18092 }};	/*Useragent - Treo850e/v0100 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Sprint Treo850e*/
const Device UA30906 = {S66762, 99, { &P14074,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) 320X320 Palm Treo850e*/
const Device UA51822 = {S66763, 80, { &P14074,&P5448,&P4808,&P18092 }};	/*Useragent - Treo850e/v0100 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA29931 = {S66764, 106, { &P12280,&P17131,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko)*/
const Device UA29947 = {S66765, 104, { &P12280,&P17131,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_2 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko)*/
const Device UA88110 = {S66766, 116, { &P12280,&P17131,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_2 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Mobile/5G77*/
const Device UA174236 = {S66767, 92, { &P12280,&P17131,&P17596,&P18092 }};	/*Useragent - Palm680/RC1 (iPhone; U; CPU iPhone OS 2_2_1 like Mac OS X; zh-cn)/UC Browser 7.1.1.53/39/352*/
const Device UA263789 = {S66768, 118, { &P12280,&P17131,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Mobile/5H11*/
const Device UA702504 = {S66769, 138, { &P12444,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; U20i Build/4.0.A.2.335) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019244 = {S66770, 132, { &P12444,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; U20i Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019823 = {S66771, 142, { &P12444,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; U20i Build/2.3.5.SB.V7.SP2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020072 = {S66772, 133, { &P12444,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U20i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020085 = {S66773, 139, { &P12444,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; U20i Build/GingerDX Pro) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020171 = {S66774, 140, { &P12444,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; U20i Build/3.0.1.A.0.145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022948 = {S66775, 140, { &P12444,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; U20i Build/4.0.1.A.0.283) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031785 = {S66776, 64, { &P12444,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; U20i Build/3.0.1.A.0.145)*/
const Device UA2221154 = {S66777, 134, { &P20156,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S500 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2529254 = {S66778, 138, { &P20156,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TCL S500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2436518 = {S66779, 114, { &P15364,&P5690,&P19987,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 Sleipnir/3.8.4*/
const Device UA2559258 = {S66780, 134, { &P21216,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A78 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2699573 = {S66781, 127, { &P21216,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A78 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2937223 = {S66782, 127, { &P21216,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A78 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2604216 = {S66783, 155, { &P18465,&P18066,&P21707,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2633368 = {S66784, 151, { &P15932,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3110 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2633571 = {S66785, 151, { &P15932,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3110 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2773097 = {S66786, 150, { &P15932,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3110 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2773254 = {S66787, 150, { &P15932,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3110 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2755086 = {S66788, 163, { &P18250,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Motorola RAZR MAXX Build/6.7.2-180_DHD-16_NA-27) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA227022 = {S66789, 123, { &P15364,&P7133,&P22405,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.1.0.0 Chrome/12.0.742.91 Safari/534.30*/
const Device UA227465 = {S66790, 130, { &P15364,&P7133,&P22405,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.1.0.0 Chrome/12.0.742.91 Safari/534.30*/
const Device UA831057 = {S66791, 154, { &P15060,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5660 Build/GRWK74; CyanogenMod-7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024923 = {S66792, 155, { &P15060,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5660 Build/GWK74; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652160 = {S66793, 157, { &P15060,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5660 Build/GINGERBREAD; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3098151 = {S66794, 155, { &P15060,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; GT-S5660 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1984789 = {S66795, 134, { &P21154,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT611 Build/V4.55B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024025 = {S66796, 134, { &P21154,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT611 Build/V4.53A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025103 = {S66797, 134, { &P21154,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT611 Build/V4.580) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032440 = {S66798, 134, { &P21154,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT611 Build/V4.650) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032978 = {S66799, 134, { &P21154,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT611 Build/V4.54A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2574141 = {S66800, 134, { &P21154,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-us; XT611 Build/V4.570) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2009119 = {S66801, 129, { &P15364,&P7133,&P22434,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Comodo_Dragon/21.0.2.0*/
const Device UA2009313 = {S66802, 122, { &P15364,&P7133,&P22434,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Comodo_Dragon/21.0.2.0*/
const Device UA2180804 = {S66803, 140, { &P17108,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Inspire 4G Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2221753 = {S66804, 144, { &P17108,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Inspire 4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2362616 = {S66805, 144, { &P17108,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Inspire 4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2226705 = {S66806, 178, { &P18643,&P5043,&P24671,&P18092 }};	/*Useragent - Nokia311/5.0 (03.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia311) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2374154 = {S66807, 139, { &P21376,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-F120L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2489665 = {S66808, 155, { &P18466,&P18066,&P19838,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2540590 = {S66809, 137, { &P20587,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P715 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2868452 = {S66810, 141, { &P21605,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; C2104 Build/15.0.A.1.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892721 = {S66811, 63, { &P21605,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; C2104 Build/15.0.A.1.31)*/
const Device UA2997419 = {S66812, 145, { &P21605,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyC2104 Build/15.0.A.1.31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057571 = {S66813, 145, { &P21605,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-ca; SonyC2104 Build/15.0.A.1.36) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3059264 = {S66814, 141, { &P21605,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; C2104 Build/15.0.A.2.17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2973646 = {S66815, 142, { &P21020,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Micromax A116 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975577 = {S66816, 145, { &P16263,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI MediaPad Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2976091 = {S66817, 145, { &P16263,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MediaPad 7 Lite Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022718 = {S66818, 146, { &P16263,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MediaPad 10 LINK Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023514 = {S66819, 141, { &P16263,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SpringBoard Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023004 = {S66820, 161, { &P13123,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5360 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA69394 = {S66821, 29, { &P14358,&P5890,&P24663,&P18092 }};	/*Useragent - NokiaE63/UCWEB7.9.0.94/28/800*/
const Device UA248937 = {S66822, 41, { &P14358,&P5890,&P24663,&P18092 }};	/*Useragent - NokiaE63/UC Browser7.9.0.102/28/355/UCWEB*/
const Device UA924154 = {S66823, 139, { &P22591,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2423323 = {S66824, 123, { &P14519,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaN97 mini) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2468338 = {S66825, 45, { &P14519,&P5129,&P24673,&P18092 }};	/*Useragent - NokiaN97 miniUCBrowser/8.7.1.234/50/444/UCWEB*/
const Device UA2556249 = {S66826, 134, { &P20690,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; V360 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2737464 = {S66827, 135, { &P21065,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CINK+ Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2795661 = {S66828, 139, { &P21065,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; CINK FIVE Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3033156 = {S66829, 150, { &P23773,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8010 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3079381 = {S66830, 150, { &P23773,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8010 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2389052 = {S66831, 147, { &P20259,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Starmobile ASTRA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2565913 = {S66832, 155, { &P18466,&P18066,&P21418,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2752936 = {S66833, 127, { &P16976,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT300 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA261156 = {S66834, 47, { &P15364,&P18146,&P4809,&P18092 }};	/*Useragent - Mozilla/5.0 (SunOS 5.8 sun4u; U) Opera 5.0 [en]*/
const Device UA2205515 = {S66835, 171, { &P15575,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2235142 = {S66836, 171, { &P15575,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2350291 = {S66837, 172, { &P15575,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2383373 = {S66838, 171, { &P15575,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2481950 = {S66839, 172, { &P15575,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2593888 = {S66840, 171, { &P15575,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2200378 = {S66841, 156, { &P12896,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LG-P999 Build/GRI40; CyanogenMod-9.9.9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2355238 = {S66842, 156, { &P12896,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P999 Build/GRI40; CyanogenMod-9.9.9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2437453 = {S66843, 136, { &P12280,&P17146,&P22900,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.2 Mobile/9A405 Safari/7534.48.3*/
const Device UA2484536 = {S66844, 134, { &P12280,&P17146,&P22900,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.2 Mobile/9A334 Safari/7534.48.3*/
const Device UA2566950 = {S66845, 141, { &P18303,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT27i Build/6.1.1.B.1.54) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2886768 = {S66846, 112, { &P19747,&P5043,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia206/03.59; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA3031486 = {S66847, 157, { &P20146,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3108495 = {S66848, 158, { &P20146,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3065585 = {S66849, 130, { &P20968,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Le Pan S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3065576 = {S66850, 143, { &P19862,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Micromax A110Q Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA62452 = {S66851, 100, { &P11900,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-A737/UCGI3 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA42545 = {S66852, 121, { &P14202,&P5477,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.1-update1;Zh_cn;E15i;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA627750 = {S66853, 72, { &P14202,&P5477,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.1-update1;Zh_cn;E15i;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA64042 = {S66854, 65, { &P14568,&P17483,&P17602,&P18092 }};	/*Useragent - LG-C105/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA250078 = {S66855, 89, { &P15364,&P7133,&P21900,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/14.0.835.202)*/
const Device UA250075 = {S66856, 132, { &P15364,&P7133,&P21900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/14.0.835.202) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA341219 = {S66857, 125, { &P15364,&P7133,&P21900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/14.0.835.186) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1*/
const Device UA342987 = {S66858, 125, { &P15364,&P7133,&P21900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/14.0.835.202) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA343288 = {S66859, 183, { &P15364,&P7133,&P21900,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/14.0.835.202; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E)*/
const Device UA437004 = {S66860, 96, { &P15364,&P7133,&P21900,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/14.0.835.186)*/
const Device UA573438 = {S66861, 166, { &P15364,&P7133,&P21900,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/14.0.835.202; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)*/
const Device UA573789 = {S66862, 125, { &P15364,&P7133,&P21900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/14.0.835.187) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.187 Safari/535.1*/
const Device UA645780 = {S66863, 132, { &P15364,&P7133,&P21900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/14.0.835.186) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1*/
const Device UA854195 = {S66864, 216, { &P15364,&P7133,&P21900,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; chromeframe/14.0.835.202; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET4.0C; .NET4.0E)*/
const Device UA872947 = {S66865, 132, { &P15364,&P7133,&P21900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; chromeframe/14.0.835.163) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.163 Safari/535.1*/
const Device UA1901829 = {S66866, 199, { &P15364,&P7133,&P21900,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/14.0.835.202; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3)*/
const Device UA2294898 = {S66867, 123, { &P15364,&P7133,&P21900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; chromeframe/14.0.835.94) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.94 Safari/535.1*/
const Device UA2300216 = {S66868, 192, { &P15364,&P7133,&P21900,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; chromeframe/14.0.835.202; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0)*/
const Device UA442009 = {S66869, 95, { &P15364,&P18135,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-GB; rv:1.9.2.8) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.8*/
const Device UA449740 = {S66870, 94, { &P15364,&P18135,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; nl; rv:1.9.2.17) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.17*/
const Device UA480462 = {S66871, 99, { &P15364,&P18135,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.12) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.12*/
const Device UA481048 = {S66872, 97, { &P15364,&P18135,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.11) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.11*/
const Device UA539678 = {S66873, 99, { &P15364,&P18135,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-GB; rv:1.9.2.17) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.17*/
const Device UA541109 = {S66874, 104, { &P15364,&P18135,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.17) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.17 GTB7.1*/
const Device UA544989 = {S66875, 97, { &P15364,&P18135,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; nb-NO; rv:1.9.2.17) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.17*/
const Device UA656475 = {S66876, 97, { &P15364,&P18135,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.2.17) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.17*/
const Device UA749480 = {S66877, 97, { &P15364,&P18135,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-GB; rv:1.9.2.15) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.15*/
const Device UA791290 = {S66878, 94, { &P15364,&P18135,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; it; rv:1.9.2.17) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.17*/
const Device UA875089 = {S66879, 95, { &P15364,&P18135,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.9) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.9*/
const Device UA2210891 = {S66880, 97, { &P15364,&P18135,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; fi-FI; rv:1.9.2.17) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.17*/
const Device UA2260217 = {S66881, 96, { &P15364,&P18135,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; de; rv:1.9.2.17) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.17*/
const Device UA2801549 = {S66882, 94, { &P15364,&P18135,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; fr; rv:1.9.2.13) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.13*/
const Device UA2803164 = {S66883, 97, { &P15364,&P18135,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; pl-PL; rv:1.9.2.17) Gecko/X Linux Mint/8 (Helena) Firefox/3.6.17*/
const Device UA576173 = {S66884, 115, { &P15228,&P5043,&P17438,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-03/06.51; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15*/
const Device UA2708986 = {S66885, 152, { &P18643,&P5043,&P24677,&P18092 }};	/*Useragent - Nokia311/5.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia311) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2880096 = {S66886, 140, { &P19763,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6602 Build/10.3.A.0.423) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA64860 = {S66887, 65, { &P12336,&P17483,&P3534,&P18092 }};	/*Useragent - LG-CU720/V1.0l Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA44858 = {S66888, 90, { &P13838,&P5776,&P3756,&P18092 }};	/*Useragent - Nokia6600/1.0 (5.53.0) SymbianOS/7.0s Series60/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA141958 = {S66889, 109, { &P18254,&P17350,&P4551,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; Girafabot; girafabot at girafa dot com; http://www.girafa.com)*/
const Device UA227577 = {S66890, 121, { &P15364,&P17285,&P22451,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.0 Chrome/13.0.800.0 Safari/535.1*/
const Device UA237438 = {S66891, 114, { &P15364,&P17285,&P22451,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.1 Chrome/13.0.800.1 Safari/535.1*/
const Device UA901996 = {S66892, 85, { &P18141,&P18571,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 1.69) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2*/
const Device UA2507803 = {S66893, 159, { &P13054,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ADR6425LVW Build/IML74K; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2561413 = {S66894, 159, { &P13054,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ADR6425LVW Build/IML74K; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2515313 = {S66895, 150, { &P18385,&P18598,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 7 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2671056 = {S66896, 149, { &P18385,&P18598,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2714539 = {S66897, 153, { &P18385,&P18598,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA2658995 = {S66898, 58, { &P15364,&P5448,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; WindowsCE; Trident/4.0)*/
const Device UA2700402 = {S66899, 63, { &P15364,&P5448,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; PPC; Tablet PC; Trident/3.0)*/
const Device UA2851659 = {S66900, 63, { &P15364,&P5448,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; 68K; Tablet PC; Trident/2.0)*/
const Device UA3026042 = {S66901, 196, { &P21648,&P18598,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9505 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb*/
const Device UA2753269 = {S66902, 133, { &P18280,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; One S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783240 = {S66903, 137, { &P18280,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754073 = {S66904, 133, { &P20011,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Next7P12-8G Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2771264 = {S66905, 130, { &P20011,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Next7P12 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3130945 = {S66906, 130, { &P18437,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8020 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3132770 = {S66907, 130, { &P18437,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA63960 = {S66908, 166, { &P12452,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 NokiaE55-1/034.001; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.1.5*/
const Device UA633228 = {S66909, 104, { &P14708,&P5204,&P17347,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5130c-2/UC Browser8.0.3.107/69/444*/
const Device UA1933776 = {S66910, 98, { &P14708,&P5204,&P17347,&P18092 }};	/*Useragent - Nokia5130/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5130/UC Browser8.0.3.107/69/444*/
const Device UA2721365 = {S66911, 254, { &P14708,&P5204,&P17347,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/69/444*/
const Device UA614936 = {S66912, 102, { &P15364,&P17188,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA30479 = {S66913, 49, { &P12318,&P17017,&P3487,&P18092 }};	/*Useragent - SPH-M340/1.0 UP.Browser/7.2.7.2.510 (GUI) MMP/2.0*/
const Device UA33198 = {S66914, 51, { &P12318,&P17017,&P3487,&P18092 }};	/*Useragent - SPH-M340-C/1.0 UP.Browser/7.2.7.2.510 (GUI) MMP/2.0*/
const Device UA54552 = {S66915, 41, { &P14238,&P17484,&P3625,&P18092 }};	/*Useragent - sam-r560 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA169521 = {S66916, 53, { &P13668,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/3.0 (Planetweb/2.100 JS SSL US; Dreamcast US)*/
const Device UA379318 = {S66917, 81, { &P16518,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-E2222/E2222DDKE5 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA2488292 = {S66918, 32, { &P23144,&P19826,&P17470,&P18092 }};	/*Useragent - iOS/6.1 (10B143) dataaccessd/1.0*/
const Device UA2492637 = {S66919, 32, { &P23144,&P19826,&P17470,&P18092 }};	/*Useragent - iOS/6.1 (10B142) dataaccessd/1.0*/
const Device UA2492642 = {S66920, 32, { &P23144,&P19826,&P17470,&P18092 }};	/*Useragent - iOS/6.1 (10B141) dataaccessd/1.0*/
const Device UA2492648 = {S66921, 32, { &P23144,&P19826,&P17470,&P18092 }};	/*Useragent - iOS/6.1 (10B144) dataaccessd/1.0*/
const Device UA2516520 = {S66922, 34, { &P23144,&P19826,&P17470,&P18092 }};	/*Useragent - iOS/6.1.1 (10B145) dataaccessd/1.0*/
const Device UA2532508 = {S66923, 34, { &P23144,&P19826,&P17470,&P18092 }};	/*Useragent - iOS/6.1.2 (10B146) dataaccessd/1.0*/
const Device UA2585296 = {S66924, 34, { &P23144,&P19826,&P17470,&P18092 }};	/*Useragent - iOS/6.1.3 (10B329) dataaccessd/1.0*/
const Device UA2709270 = {S66925, 34, { &P23144,&P19826,&P17470,&P18092 }};	/*Useragent - iOS/6.1.4 (10B350) dataaccessd/1.0*/
const Device UA2671393 = {S66926, 142, { &P19764,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C6502 Build/10.1.A.1.434) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2707567 = {S66927, 64, { &P19764,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; C6502 Build/10.1.A.1.434)*/
const Device UA2870782 = {S66928, 144, { &P19764,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C6502 Build/10.1.1.A.1.253) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3073785 = {S66929, 142, { &P19764,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-co; C6502 Build/10.1.A.1.350) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3074141 = {S66930, 146, { &P19764,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SonyC6502 Build/10.1.A.1.350) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2970462 = {S66931, 134, { &P21636,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C525c Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3082731 = {S66932, 126, { &P15869,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Xoom Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA1946260 = {S66933, 123, { &P11658,&P5776,&P24663,&P18092 }};	/*Useragent - Nokia6680/1.0 (5.04.40) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.9.1.120/27/352/UCWEB*/
const Device UA627433 = {S66934, 133, { &P21183,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; P-01D Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023831 = {S66935, 57, { &P21183,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; P-01D Build/GRH78)*/
const Device UA1987168 = {S66936, 135, { &P22875,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i9220 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2025293 = {S66937, 135, { &P22875,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; i9220 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2025792 = {S66938, 138, { &P22875,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9220 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2026281 = {S66939, 135, { &P22875,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.6; i9220 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2160157 = {S66940, 138, { &P22875,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9220 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2479646 = {S66941, 135, { &P22875,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; I9220 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2950749 = {S66942, 135, { &P22875,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; i9220 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2430722 = {S66943, 152, { &P20002,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 992D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2670742 = {S66944, 130, { &P12879,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P7510 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676446 = {S66945, 129, { &P12879,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7510 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2667839 = {S66946, 59, { &P15364,&P18346,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10; rv:20.0) Gecko/X Firefox/20.0*/
const Device UA2968517 = {S66947, 92, { &P12581,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia500) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3066541 = {S66948, 130, { &P14908,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA13749 = {S66949, 60, { &P13557,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia6060/2.0 (4.10) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20535 = {S66950, 61, { &P13557,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia6060v/2.0 (3.03) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA545615 = {S66951, 149, { &P14800,&P17979,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (MOT-XT800/TITA_M2_20.08.0;U;Android/2.1-update1;480*854;CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA197811 = {S66952, 33, { &P15364,&P17017,&P4777,&P18092 }};	/*Useragent - S8500 UCWEB6.0/UC Browser7.7.0.81*/
const Device UA436559 = {S66953, 41, { &P15364,&P17017,&P4777,&P18092 }};	/*Useragent - S8500 UCWEB6.0/UC Browser7.7.0.81/179/352*/
const Device UA306729 = {S66954, 62, { &P15364,&P7127,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; ru) Presto/2.2.15 Version/10.00*/
const Device UA447095 = {S66955, 62, { &P15364,&P7127,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.2.15 Version/10.01*/
const Device UA2029351 = {S66956, 131, { &P21608,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC409 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2232467 = {S66957, 138, { &P21608,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC409 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2363119 = {S66958, 125, { &P12080,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s3653) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/405/UCWEB Mobile*/
const Device UA2258478 = {S66959, 121, { &P12581,&P5055,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; Nokia500) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/51/355/UCWEB Mobile*/
const Device UA2426990 = {S66960, 133, { &P21887,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; TAD-97052 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2437631 = {S66961, 133, { &P21887,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; TAC-97052 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2502874 = {S66962, 133, { &P23047,&P5629,&P15760,&P18092 }};	/*Useragent - K-Touch_T619/X_8510_V0101 Mozilla/5.0 (Linux; U; Android 2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2552852 = {S66963, 143, { &P23047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; K-Touch T619 Build/MocorDroid2.3.5) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2578105 = {S66964, 143, { &P23047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; K-Touch T619 Build MocorDroid2.3.5) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2728196 = {S66965, 74, { &P23047,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; K-Touch T619 Build/MocorDroid2.3.5)*/
const Device UA2974277 = {S66966, 130, { &P14908,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA261659 = {S66967, 71, { &P15364,&P18110,&P24474,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/4.2; Linux) KHTML/4.2.2 (like Gecko)*/
const Device UA2484443 = {S66968, 147, { &P12280,&P19826,&P21787,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA2533034 = {S66969, 149, { &P12280,&P19826,&P21787,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA2566607 = {S66970, 149, { &P12280,&P19826,&P21787,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA2606954 = {S66971, 149, { &P12280,&P19826,&P21787,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA2709295 = {S66972, 149, { &P12280,&P19826,&P21787,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA2600431 = {S66973, 135, { &P21882,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-in; Micromax P362 Build) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2900574 = {S66974, 64, { &P19764,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; C6502 Build/10.3.A.0.423)*/
const Device UA2943765 = {S66975, 142, { &P19764,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; C6502 Build/10.3.A.0.423) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058749 = {S66976, 144, { &P19764,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; C6502 Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1958432 = {S66977, 78, { &P15364,&P18346,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 4.1; Alexa Toolbar)*/
const Device UA1976514 = {S66978, 78, { &P15364,&P18346,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 4.1; Alexa Toolbar)*/
const Device UA1994220 = {S66979, 68, { &P15364,&P18346,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 4.1; SV1)*/
const Device UA2166986 = {S66980, 68, { &P15364,&P18346,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 4.1; SV1)*/
const Device UA2217091 = {S66981, 64, { &P15364,&P18346,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 4.10; Trident/4.0)*/
const Device UA2758225 = {S66982, 98, { &P15364,&P18346,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 4.10; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)*/
const Device UA3069795 = {S66983, 103, { &P15364,&P18346,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 4.10; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; SE 2.x)*/
const Device UA413302 = {S66984, 32, { &P15364,&P17017,&P23045,&P18092 }};	/*Useragent - NSPlayer/10.0.0.3705 WMFSDK/10.0*/
const Device UA459851 = {S66985, 32, { &P15364,&P17017,&P23045,&P18092 }};	/*Useragent - NSPlayer/10.0.0.4005 WMFSDK/10.0*/
const Device UA480697 = {S66986, 32, { &P15364,&P17017,&P23045,&P18092 }};	/*Useragent - NSPlayer/10.0.0.4007 WMFSDK/10.0*/
const Device UA502842 = {S66987, 20, { &P15364,&P17017,&P23045,&P18092 }};	/*Useragent - NSPlayer/10.0.0.4362*/
const Device UA637456 = {S66988, 32, { &P15364,&P17017,&P23045,&P18092 }};	/*Useragent - NSPlayer/10.0.0.4054 WMFSDK/10.0*/
const Device UA656815 = {S66989, 32, { &P15364,&P17017,&P23045,&P18092 }};	/*Useragent - NSPlayer/10.0.0.4374 WMFSDK/10.0*/
const Device UA1836511 = {S66990, 32, { &P15364,&P17017,&P23045,&P18092 }};	/*Useragent - NSPlayer/10.0.0.3703 WMFSDK/10.0*/
const Device UA2262179 = {S66991, 32, { &P15364,&P17017,&P23045,&P18092 }};	/*Useragent - NSPlayer/10.0.0.3702 WMFSDK/10.0*/
const Device UA2677156 = {S66992, 50, { &P15364,&P17017,&P23045,&P18092 }};	/*Useragent - NSPlayer/10.0.0.4078 WMFSDK/10.0 SE 2.X MetaSr 1.0*/
const Device UA2681777 = {S66993, 20, { &P15364,&P17017,&P23045,&P18092 }};	/*Useragent - NSPlayer/10.0.0.3650*/
const Device UA2719800 = {S66994, 32, { &P15364,&P17017,&P23045,&P18092 }};	/*Useragent - NSPlayer/10.0.0.3700 WMFSDK/10.0*/
const Device UA2742033 = {S66995, 20, { &P15364,&P17017,&P23045,&P18092 }};	/*Useragent - NSPlayer/10.0.0.3700*/
const Device UA2751145 = {S66996, 50, { &P15364,&P17017,&P23045,&P18092 }};	/*Useragent - NSPlayer/10.0.0.3802 WMFSDK/10.0 SE 2.X MetaSr 1.0*/
const Device UA2802595 = {S66997, 20, { &P15364,&P17017,&P23045,&P18092 }};	/*Useragent - NSPlayer/10.0.0.4001*/
const Device UA896821 = {S66998, 130, { &P21669,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; EXLP2B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1952343 = {S66999, 140, { &P21669,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; eGlide XL PRO II Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2657531 = {S67000, 140, { &P21669,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; eGlide XL PRO II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2420520 = {S67001, 140, { &P18740,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-TH; OPPOU7011 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528261 = {S67002, 136, { &P18740,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U701T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2655761 = {S67003, 139, { &P18740,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; OPPOU701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2664518 = {S67004, 133, { &P18740,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th; U7015 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2846480 = {S67005, 127, { &P18740,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U701T Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2955740 = {S67006, 140, { &P18740,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; OPPOU7015 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3080699 = {S67007, 153, { &P18016,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ThinkPad Tablet Build/ThinkPadTablet_A400_03) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA909644 = {S67008, 131, { &P21072,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; DIT1010 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2010149 = {S67009, 124, { &P19754,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T510a Build/M) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2360276 = {S67010, 129, { &P19754,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T510a Build/M41882) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2375851 = {S67011, 128, { &P19754,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T510 Build/M41467) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2760367 = {S67012, 128, { &P19754,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T510 Build/M41703) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2981478 = {S67013, 129, { &P19754,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T510a Build/M41930) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2165270 = {S67014, 200, { &P23018,&P22857,&P18373,&P18092 }};	/*Useragent - MOT-EX430 MIDP-2.0/CLDC-1.1 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA2206705 = {S67015, 140, { &P18606,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; PadFone 2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2402095 = {S67016, 62, { &P18606,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; PadFone 2 Build/IMM76I)*/
const Device UA2422116 = {S67017, 133, { &P18606,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-sg; PadFone 2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3082808 = {S67018, 137, { &P20975,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-AS730 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067291 = {S67019, 146, { &P19863,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA38 = {S67020, 71, { &P16503,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia7310/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7310*/
const Device UA44 = {S67021, 61, { &P16503,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia7310/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA33473 = {S67022, 66, { &P16503,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia7310_CMCC/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA679896 = {S67023, 82, { &P17595,&P17295,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.10.229 Version/11.62*/
const Device UA2194866 = {S67024, 157, { &P15364,&P7133,&P19964,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.2.2.29837 (Firestorm-Release; firestorm skin) Safari/533.3*/
const Device UA2566958 = {S67025, 130, { &P15882,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5110 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2633353 = {S67026, 131, { &P15882,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P5110 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2634451 = {S67027, 137, { &P15882,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5110 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2712454 = {S67028, 161, { &P18672,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5302 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2975792 = {S67029, 138, { &P18716,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974323 = {S67030, 139, { &P18736,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One SV Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3082716 = {S67031, 137, { &P16356,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3064998 = {S67032, 142, { &P21020,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Micromax A116 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA67304 = {S67033, 158, { &P14798,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-us; MotoroiX Build/IUS) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola MotoroiX*/
const Device UA2018803 = {S67034, 56, { &P14798,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; MotoroiX Build/IUS)*/
const Device UA2363881 = {S67035, 146, { &P12473,&P5204,&P24668,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia2730c-1) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2509061 = {S67036, 93, { &P18673,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S7562) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2704458 = {S67037, 134, { &P22926,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; M-MP75S23G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3081617 = {S67038, 143, { &P18471,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT905 Build/7.7.1Q_GCIRD-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083654 = {S67039, 150, { &P18471,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT907 Build/7.7.1Q-144_SMQ_S136-10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3132853 = {S67040, 129, { &P18556,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BNTV600 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA566578 = {S67041, 138, { &P21388,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; Karbonn_A1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020774 = {S67042, 62, { &P21388,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; Karbonn_A1 Build/FRF91)*/
const Device UA2021973 = {S67043, 126, { &P21388,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; Karbonn_A1 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1958147 = {S67044, 154, { &P14186,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S5830 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2227817 = {S67045, 186, { &P14186,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S5830 Build/IMM76L; CyanogenMod-CyanogenMod9 RC1 by MaclawStudios) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2507275 = {S67046, 129, { &P17686,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-au; MZ601 Build/JB_MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2665563 = {S67047, 128, { &P17686,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MZ601 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2882690 = {S67048, 112, { &P19748,&P5043,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia205/03.18; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA2966936 = {S67049, 114, { &P19748,&P5043,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia205.1/03.18; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA63219 = {S67050, 146, { &P13488,&P5448,&P17183,&P18092 }};	/*Useragent - SonyEricssonM1i/R1BA Browser/Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2205663 = {S67051, 145, { &P12280,&P18432,&P21787,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA2362719 = {S67052, 149, { &P12280,&P18432,&P21787,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA2430862 = {S67053, 149, { &P12280,&P18432,&P21787,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA2257167 = {S67054, 137, { &P23075,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; MOT-XT305 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652849 = {S67055, 139, { &P23075,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; XT305 Build/07.19.03R_D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2934948 = {S67056, 139, { &P23075,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-br; XT305 Build/07.18.06R_D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2351476 = {S67057, 163, { &P15798,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Explorer A310 Build/IMLK74; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2353461 = {S67058, 164, { &P15798,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Explorer A310e Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2650108 = {S67059, 163, { &P15798,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Explorer A310 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2356390 = {S67060, 133, { &P17595,&P18207,&P22462,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4*/
const Device UA2602711 = {S67061, 133, { &P17595,&P18207,&P22462,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4*/
const Device UA2879626 = {S67062, 133, { &P17595,&P18207,&P22462,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4*/
const Device UA2361744 = {S67063, 110, { &P23589,&P18378,&P19810,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo WiiU) AppleWebKit/534.52 (KHTML, like Gecko) NX/2.1.0.8.21 NintendoBrowser/1.0.0.7494.US*/
const Device UA2387428 = {S67064, 110, { &P23589,&P18378,&P19810,&P18092 }};	/*Useragent - Mozilla/5.0 (Nintendo WiiU) AppleWebKit/534.52 (KHTML, like Gecko) NX/2.1.0.8.21 NintendoBrowser/1.0.0.7494.EU*/
const Device UA2508035 = {S67065, 131, { &P18320,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2511263 = {S67066, 137, { &P18320,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2566800 = {S67067, 150, { &P18318,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF700T Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567317 = {S67068, 142, { &P18504,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LS840 Build/ZVK.IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2676819 = {S67069, 136, { &P12814,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I777 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713800 = {S67070, 137, { &P12814,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-i777 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714414 = {S67071, 129, { &P12814,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I777 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2754223 = {S67072, 137, { &P20973,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LW770 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2898293 = {S67073, 146, { &P18372,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-04D Build/9.1.C.0.475) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2899273 = {S67074, 142, { &P18372,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-04D Build/9.1.C.0.475) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3016633 = {S67075, 64, { &P18372,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SO-04D Build/9.1.C.0.475)*/
const Device UA3096285 = {S67076, 146, { &P18372,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-04D Build/9.1.C.1.103) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3111492 = {S67077, 142, { &P18372,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-04D Build/9.1.C.1.103) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3662 = {S67078, 97, { &P13374,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Diamond/1.37.172.3*/
const Device UA3666 = {S67079, 97, { &P13374,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Diamond/1.97.164.3*/
const Device UA5833 = {S67080, 86, { &P13374,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Alltel HTC Touch Diamond*/
const Device UA7300 = {S67081, 95, { &P13374,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_P3700/2.03.168.3*/
const Device UA7301 = {S67082, 97, { &P13374,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Diamond/1.37.163.4*/
const Device UA13449 = {S67083, 93, { &P13374,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_P3700 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) HTC_P3700 Mozilla/4.0*/
const Device UA13463 = {S67084, 99, { &P13374,&P5448,&P22270,&P18092 }};	/*Useragent - HTC-P3700-orange/PPC; 480x640; OpVer 30.204.2.731 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA29204 = {S67085, 85, { &P13374,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_Diamond/V2.07.841.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA29208 = {S67086, 97, { &P13374,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Diamond/1.97.161.4*/
const Device UA29205 = {S67087, 73, { &P13374,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_Diamond Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA34834 = {S67088, 99, { &P13374,&P5448,&P22270,&P18092 }};	/*Useragent - HTC-P3700-orange/PPC; 480x640; OpVer 30.137.2.732 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA34835 = {S67089, 97, { &P13374,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Diamond/1.37.161.2*/
const Device UA51451 = {S67090, 97, { &P13374,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Diamond/1.97.163.4*/
const Device UA63921 = {S67091, 71, { &P13374,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_P3700 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA66649 = {S67092, 99, { &P13374,&P5448,&P22270,&P18092 }};	/*Useragent - HTC-P3700-orange/PPC; 480x640; OpVer 30.193.2.734 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA66651 = {S67093, 99, { &P13374,&P5448,&P22270,&P18092 }};	/*Useragent - HTC-P3700-orange/PPC; 480x640; OpVer 30.126.2.732 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA74814 = {S67094, 100, { &P13374,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Diamond/%ROM_VERSION%*/
const Device UA78229 = {S67095, 97, { &P13374,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Diamond/1.37.164.2*/
const Device UA822368 = {S67096, 97, { &P15364,&P18118,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.7.12) Gecko/X Firefox/1.0.7 (Ubuntu package 1.0.7)*/
const Device UA2566893 = {S67097, 131, { &P17281,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I905 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2428575 = {S67098, 143, { &P19843,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE Blade III Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2467512 = {S67099, 139, { &P19843,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BLADE III Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2529205 = {S67100, 143, { &P19843,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Vip Racer III Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2565260 = {S67101, 142, { &P19843,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BLADE III_IL Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2458132 = {S67102, 132, { &P21995,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; SPNG9708 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2460826 = {S67103, 132, { &P21995,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; SPNG9708 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2729228 = {S67104, 138, { &P22805,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E455 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2954328 = {S67105, 60, { &P22805,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E455 Build/JZO54K)*/
const Device UA3067306 = {S67106, 139, { &P20738,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Andromax U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3106794 = {S67107, 144, { &P15767,&P22882,&P22593,&P18092 }};	/*Useragent - Mozilla/5.0(iPad; U;CPU OS 7 like Mac OS X; zh-CN; iPad3,4) AppleWebKit/534.46 (KHTML, like Gecko) UCBrowser/2.0.1.280 U3/0.8.0 Safari/7543.48.3*/
const Device UA870786 = {S67108, 137, { &P14925,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.3; de-de; A80KSC Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA144795 = {S67109, 146, { &P17952,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SHW-M110S Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA492173 = {S67110, 68, { &P17952,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SHW-M110S Build/ECLAIR)*/
const Device UA251797 = {S67111, 63, { &P15364,&P16110,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; U; en) Presto/2.9.168 Version/11.52*/
const Device UA2242178 = {S67112, 140, { &P18592,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One VX Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2363137 = {S67113, 133, { &P18680,&P18110,&P18690,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.2 Safari/535.19 Silk-Accelerated=true*/
const Device UA2389812 = {S67114, 80, { &P15364,&P5690,&P22555,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:17.0) Gecko/17.0 Firefox/17.0 IceDragon/17.0.1.0*/
const Device UA2422166 = {S67115, 128, { &P19991,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A511 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2422587 = {S67116, 137, { &P18470,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 101G10 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2666335 = {S67117, 140, { &P19979,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E170K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2751449 = {S67118, 147, { &P19979,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E170K/KKJMC1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2793330 = {S67119, 140, { &P19979,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E170L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2858413 = {S67120, 140, { &P19979,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E170S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708690 = {S67121, 138, { &P18636,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; VS930 4G Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2865630 = {S67122, 131, { &P18636,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; VS930 4G Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA54833 = {S67123, 67, { &P12391,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-C270/C270JPIL1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA44059 = {S67124, 141, { &P13776,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; U8300 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA78358 = {S67125, 147, { &P13776,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; HuaweiU8300 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018122 = {S67126, 63, { &P13776,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; U8300 Build/ERE27)*/
const Device UA122861 = {S67127, 61, { &P15364,&P6729,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 95; Alexa Toolbar)*/
const Device UA123541 = {S67128, 62, { &P15364,&P6729,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.01; Windows 95; Alexa Toolbar)*/
const Device UA2580073 = {S67129, 54, { &P15364,&P6729,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Win95; Trident/2.0)*/
const Device UA171299 = {S67130, 130, { &P17968,&P17979,&P18335,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android AOSP; en-us Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA303645 = {S67131, 130, { &P17968,&P17979,&P18335,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android AOSP; en-gb Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA416045 = {S67132, 130, { &P17968,&P17979,&P18335,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android AOSP; it-it Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA524521 = {S67133, 130, { &P17968,&P17979,&P18335,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android AOSP; pt-br Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA524761 = {S67134, 130, { &P17968,&P17979,&P18335,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android AOSP; nl-nl Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA530032 = {S67135, 130, { &P17968,&P17979,&P18335,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android AOSP; ru-ru Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA657235 = {S67136, 130, { &P17968,&P17979,&P18335,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android AOSP; es-es Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA787023 = {S67137, 130, { &P17968,&P17979,&P18335,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android AOSP; fr-fr Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1870021 = {S67138, 130, { &P17968,&P17979,&P18335,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android AOSP; de-de Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018643 = {S67139, 130, { &P17968,&P17979,&P18335,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android AOSP; es-us Build/OPENMASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2581549 = {S67140, 159, { &P15575,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; GT-I9100T Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2672029 = {S67141, 158, { &P15575,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2672675 = {S67142, 158, { &P15575,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2510775 = {S67143, 171, { &P18513,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3021055 = {S67144, 175, { &P18513,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6802 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2974085 = {S67145, 134, { &P17160,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Slider SL101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA399135 = {S67146, 115, { &P15775,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800; 800 BOUYGUES)*/
const Device UA399139 = {S67147, 111, { &P15775,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800; Vodafone)*/
const Device UA399140 = {S67148, 111, { &P15775,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800; T-Mobile)*/
const Device UA399167 = {S67149, 109, { &P15775,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800; Orange)*/
const Device UA399134 = {S67150, 106, { &P15775,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800; SFR)*/
const Device UA399172 = {S67151, 146, { &P15775,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800) Microsoft.Consulting.Phone.SkySportsNews.App*/
const Device UA399164 = {S67152, 101, { &P15775,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800)*/
const Device UA2492478 = {S67153, 76, { &P17595,&P17297,&P20794,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:7.0.1) Gecko/X Postbox/3.0.7*/
const Device UA911651 = {S67154, 131, { &P15364,&P7133,&P22432,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11 Comodo_Dragon/20.0.1.0*/
const Device UA911655 = {S67155, 124, { &P15364,&P7133,&P22432,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11 Comodo_Dragon/20.0.1.0*/
const Device UA2354663 = {S67156, 171, { &P18672,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2738446 = {S67157, 132, { &P23413,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PLT7223G Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2943998 = {S67158, 156, { &P13035,&P5043,&P24678,&P18092 }};	/*Useragent - NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-02) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA24477 = {S67159, 39, { &P14545,&P17483,&P17602,&P18092 }};	/*Useragent - LG-X330T Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA63821 = {S67160, 38, { &P14545,&P17483,&P17602,&P18092 }};	/*Useragent - LG-X330 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA47597 = {S67161, 159, { &P15585,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; MOT-XT882/SWDFS_M7_4.60.4; 540*960; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47599 = {S67162, 143, { &P15585,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; XT882 Build/SWDFS_M7_4.60.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47598 = {S67163, 159, { &P15585,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MOT-XT882/SWDFS_M7_4.80.0; 540*960; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA47600 = {S67164, 143, { &P15585,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT882 Build/SWDFS_M7_4.80.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA583867 = {S67165, 131, { &P15585,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT882 Build/SWDFS_M7_4.97.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1940393 = {S67166, 159, { &P15585,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-XT882/SWDFS_M7_4.97.0; 540*960; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020315 = {S67167, 131, { &P15585,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; XT882 Build/SWDFS_M7_4.60.4) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022398 = {S67168, 131, { &P15585,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT882 Build/SWDFS_M7_4.80.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2024361 = {S67169, 143, { &P15585,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT882 Build/SWDFS_M7_4.97.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032571 = {S67170, 136, { &P15585,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT882 Build/SWDFS_M7_4.97.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2034518 = {S67171, 136, { &P15585,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT882 Build/SWDFS_M7_4.80.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA478585 = {S67172, 97, { &P15364,&P18134,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.14) Gecko/X Linux Mint/7 (Gloria) Firefox/3.0.17*/
const Device UA511261 = {S67173, 97, { &P15364,&P18134,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.16) Gecko/X Linux Mint/7 (Gloria) Firefox/3.0.16*/
const Device UA704188 = {S67174, 97, { &P15364,&P18134,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.16) Gecko/X Linux Mint/7 (Gloria) Firefox/3.0.15*/
const Device UA708418 = {S67175, 97, { &P15364,&P18134,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.17) Gecko/X Linux Mint/7 (Gloria) Firefox/3.0.17*/
const Device UA730751 = {S67176, 97, { &P15364,&P18134,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.12) Gecko/X Linux Mint/7 (Gloria) Firefox/3.0.12*/
const Device UA843117 = {S67177, 97, { &P15364,&P18134,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.15) Gecko/X Linux Mint/7 (Gloria) Firefox/3.0.17*/
const Device UA1836528 = {S67178, 99, { &P15364,&P18134,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.0.10) Gecko/X Linux Mint/7 (Gloria) Firefox/3.0.10*/
const Device UA1840552 = {S67179, 104, { &P15364,&P18134,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.19) Gecko/X Linux Mint/7 (Gloria) Firefox/3.0.17 GTB7.1*/
const Device UA1885843 = {S67180, 99, { &P15364,&P18134,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.0.11) Gecko/X Linux Mint/7 (Gloria) Firefox/3.0.11*/
const Device UA2140246 = {S67181, 97, { &P15364,&P18134,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.13) Gecko/X Linux Mint/7 (Gloria) Firefox/3.0.17*/
const Device UA2472323 = {S67182, 97, { &P15364,&P18134,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.11) Gecko/X Linux Mint/7 (Gloria) Firefox/3.0.17*/
const Device UA664026 = {S67183, 141, { &P21821,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; TAB-PROTAB2XXL(4) Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1979865 = {S67184, 138, { &P21821,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; TAB-PROTAB2XXL Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA906117 = {S67185, 85, { &P15547,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-c6712) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA1988300 = {S67186, 85, { &P15547,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-c6712) UCBrowser8.3.1.161/69/405/UCWEB Mobile*/
const Device UA1974292 = {S67187, 134, { &P20466,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; B79 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2442995 = {S67188, 56, { &P20466,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; B79 Build/IML74K)*/
const Device UA2042668 = {S67189, 168, { &P19891,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I717 Build/JRO03L; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2235152 = {S67190, 168, { &P19891,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I717 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2236210 = {S67191, 168, { &P19891,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I717 Build/JRO03R; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2363430 = {S67192, 160, { &P19891,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I717 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2231579 = {S67193, 103, { &P15364,&P18145,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2259969 = {S67194, 102, { &P15364,&P18145,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2424406 = {S67195, 102, { &P15364,&P18145,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/537.4 (KHTML like Gecko) Chrome/22.0.1229.79 Safari/537.4*/
const Device UA2563351 = {S67196, 152, { &P17346,&P19826,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 TeamStream (iPod4,1 313 2.4)*/
const Device UA2854661 = {S67197, 150, { &P17346,&P19826,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Safari/7534.48.3 TeamStream (iPod4,1 313 2.4)*/
const Device UA3027350 = {S67198, 154, { &P17346,&P19826,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 TeamStream (iPod4,1 379 2.7.2)*/
const Device UA2566817 = {S67199, 139, { &P18290,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ617 Build/7.7.1-85_MZ617-27) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2675249 = {S67200, 137, { &P18687,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P708g Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676042 = {S67201, 136, { &P18687,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P705 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677303 = {S67202, 137, { &P18687,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P705g Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2706638 = {S67203, 137, { &P18687,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P705f Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2974599 = {S67204, 128, { &P21769,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ME301T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA26627 = {S67205, 157, { &P13613,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; HTC_A3335 V1.21.841.3 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA26629 = {S67206, 157, { &P13613,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; HTC_A3335 V1.30.841.1 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA796355 = {S67207, 95, { &P15364,&P17285,&P21923,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0) chromeframe/10.0.648.204*/
const Device UA888923 = {S67208, 88, { &P15364,&P17285,&P21923,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0) chromeframe/10.0.648.205*/
const Device UA1953100 = {S67209, 207, { &P15364,&P17285,&P21923,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/10.0.648.204; SLCC1; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 1.1.4322; .NET CLR 3.0.30729; .NET CLR 3.5.30729; MS-RTC LM 8; .NET4.0C; .NET4.0E)*/
const Device UA2385298 = {S67210, 148, { &P15882,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P5110 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2611869 = {S67211, 155, { &P15882,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2612754 = {S67212, 155, { &P15882,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GT-P5110 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2612806 = {S67213, 148, { &P15882,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; GT-P5110 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615533 = {S67214, 148, { &P15882,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; GT-P5110 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2635676 = {S67215, 146, { &P15882,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-; GT-P5110 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2429587 = {S67216, 155, { &P14669,&P5843,&P24675,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (04.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiax2-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2974463 = {S67217, 140, { &P18303,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT27i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3068048 = {S67218, 130, { &P17323,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6800 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA31593 = {S67219, 84, { &P12441,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK310iv/R4EA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63018 = {S67220, 83, { &P12441,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK310i/R4EA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA241692 = {S67221, 60, { &P15364,&P18140,&P18337,&P18092 }};	/*Useragent - Opera/9.60 (X11; Linux i686; U; Linux Mint; en) Presto/2.1.1*/
const Device UA390550 = {S67222, 60, { &P15364,&P18140,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (X11; Linux i686; U; Linux Mint; nb) Presto/2.1.1*/
const Device UA2438852 = {S67223, 60, { &P15364,&P18140,&P18337,&P18092 }};	/*Useragent - Opera/9.62 (X11; Linux i686; U; Linux Mint; en) Presto/2.1.1*/
const Device UA473269 = {S67224, 93, { &P15364,&P7133,&P21959,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1 x64; en-US; rv:1.9.3a1pre) Gecko/X Minefield/3.7a1pre*/
const Device UA2464287 = {S67225, 86, { &P15364,&P7133,&P21959,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; ru; rv:1.9.3a4pre) Gecko/X Minefield/3.7a4pre*/
const Device UA1990538 = {S67226, 137, { &P21376,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-F120L Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2566484 = {S67227, 138, { &P20973,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LW770 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2676036 = {S67228, 129, { &P18454,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGPT12 Build/TID0142) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676039 = {S67229, 129, { &P18454,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGPT13 Build/TID0142) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2753549 = {S67230, 130, { &P15882,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5110 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2892928 = {S67231, 130, { &P15882,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P5110 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2975003 = {S67232, 134, { &P18280,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; One S Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023037 = {S67233, 138, { &P18280,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One S Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974111 = {S67234, 137, { &P16362,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-D710 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3058009 = {S67235, 137, { &P16362,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SPH-D710 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974092 = {S67236, 139, { &P22641,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C1504 Build/11.3.A.0.47) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3068200 = {S67237, 137, { &P18641,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-M830 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA102228 = {S67238, 36, { &P13668,&P17017,&P18589,&P18092 }};	/*Useragent - (DreamPassport/3.0; isao/MyDiGiRabi)*/
const Device UA1953086 = {S67239, 79, { &P11854,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE71)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2221991 = {S67240, 30, { &P11854,&P5890,&P24671,&P18092 }};	/*Useragent - NokiaE71/UCWEB8.5.0.163/28/800*/
const Device UA2445590 = {S67241, 80, { &P11854,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE71x)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2844450 = {S67242, 119, { &P11854,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiae71-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile*/
const Device UA2205031 = {S67243, 132, { &P13452,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P1000 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2232798 = {S67244, 132, { &P13452,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2615471 = {S67245, 138, { &P13452,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P1000 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2615648 = {S67246, 133, { &P13452,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P1000N Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2615803 = {S67247, 138, { &P13452,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2616144 = {S67248, 133, { &P13452,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P1000L Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2616975 = {S67249, 133, { &P13452,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2618028 = {S67250, 133, { &P13452,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000N Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2629942 = {S67251, 139, { &P13452,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2598059 = {S67252, 138, { &P20200,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ARCHOS GAMEPAD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2672966 = {S67253, 139, { &P12798,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LT18i Build/4.1.A.0.562) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676928 = {S67254, 139, { &P12798,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT18i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2863809 = {S67255, 139, { &P12798,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT18a Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2737854 = {S67256, 133, { &P23833,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; i5350 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3131005 = {S67257, 130, { &P23773,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8010 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA62030 = {S67258, 141, { &P14361,&P5298,&P4626,&P18092 }};	/*Useragent - Opera/9.70 (BREW 4.0.3; U; en; Samsung; Opera Mobi; Presto/2.2.1/INT) Sprint M850 SprintTitan/OSGi-4.1 Profile/MIDP-2.1 Configuration/CDC-1.1*/
const Device UA632624 = {S67259, 117, { &P14345,&P16000,&P24663,&P18092 }};	/*Useragent - NokiaN-GageQD/1.0 SymbianOS/6.1 Series60/1.2 Profile/MIDP-1.0 Configuration/CLDC-1.0/UC Browser7.9.1.120/26/351/UCWEB*/
const Device UA2427897 = {S67260, 87, { &P15364,&P5690,&P24354,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; QQBrowser/7.0.4350.400)*/
const Device UA2468245 = {S67261, 87, { &P15364,&P5690,&P24354,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; QQBrowser/7.0.4350.400)*/
const Device UA2468938 = {S67262, 92, { &P15364,&P5690,&P24354,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SV1; QQBrowser/7.0.4350.400)*/
const Device UA2748156 = {S67263, 79, { &P15364,&P5690,&P24354,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QQBrowser/7.0.5173.400)*/
const Device UA2507827 = {S67264, 144, { &P20837,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IRIS_501 Build/LAVAIRIS501) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567320 = {S67265, 131, { &P20968,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Le Pan S Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2659223 = {S67266, 148, { &P18673,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; GT-S7562 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2792924 = {S67267, 148, { &P18673,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-gb; GT-S7562 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2935435 = {S67268, 148, { &P18673,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; th-th; GT-S7562 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2953494 = {S67269, 148, { &P18673,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; pt-br; GT-S7562 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2953815 = {S67270, 148, { &P18673,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; fr-fr; GT-S7562 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2986375 = {S67271, 148, { &P18673,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; vi-vn; GT-S7562 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2989606 = {S67272, 148, { &P18673,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-au; GT-S7562 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2996846 = {S67273, 148, { &P18673,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; ar-eg; GT-S7562 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2997195 = {S67274, 148, { &P18673,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; es-es; GT-S7562 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3045439 = {S67275, 148, { &P18673,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; tr-tr; GT-S7562 Build/GINGERBREAD.XXKL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2946992 = {S67276, 155, { &P18635,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3065000 = {S67277, 138, { &P20517,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CINK PEAX Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081612 = {S67278, 140, { &P20517,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; CINK PEAX 2 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA63944 = {S67279, 107, { &P13508,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-C6625/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA89047 = {S67280, 121, { &P13508,&P5448,&P22270,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungC6625/BUIF2 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA281454 = {S67281, 121, { &P13508,&P5448,&P22270,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungC6625/BUID1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA477154 = {S67282, 121, { &P13508,&P5448,&P22270,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungC6625/BUIE1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA499971 = {S67283, 121, { &P13508,&P5448,&P22270,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungC6625/BUIH4 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2205352 = {S67284, 171, { &P18513,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2017655 = {S67285, 72, { &P15364,&P5690,&P22363,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:15.0) Gecko/X Firefox/15.0 PaleMoon/15.0*/
const Device UA2188973 = {S67286, 92, { &P15364,&P5690,&P22363,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:15.0) Gecko/X Firefox/15.0-x64 PaleMoon/15.0-x64*/
const Device UA2257271 = {S67287, 132, { &P20968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Le Pan S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2551585 = {S67288, 61, { &P20968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Le Pan S Build/IML74K)*/
const Device UA2537081 = {S67289, 96, { &P14519,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaN97 mini) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2555854 = {S67290, 35, { &P14519,&P5129,&P24675,&P18092 }};	/*Useragent - NokiaN97 mini/UCWEB8.8.0.212/50/999*/
const Device UA2507423 = {S67291, 130, { &P21907,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LT8088 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2663075 = {S67292, 131, { &P21907,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PLT8088 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2672847 = {S67293, 101, { &P20741,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 520)*/
const Device UA2737052 = {S67294, 111, { &P20741,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 520; Vodafone)*/
const Device UA2865027 = {S67295, 109, { &P20741,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 520; ANZ914)*/
const Device UA3087884 = {S67296, 109, { &P20741,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 520; Orange)*/
const Device UA50106 = {S67297, 49, { &P13668,&P17979,&P21556,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0; en-us; dream)*/
const Device UA68058 = {S67298, 126, { &P13668,&P17979,&P21556,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0; en-us; dream) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA68174 = {S67299, 130, { &P13668,&P17979,&P21556,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0; en-us; HTC_Dream) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA1997017 = {S67300, 126, { &P18017,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; K1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1997107 = {S67301, 134, { &P18017,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Ideapad K1 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2031880 = {S67302, 125, { &P18017,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; K1 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2427127 = {S67303, 133, { &P18680,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2670611 = {S67304, 138, { &P19777,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105P Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2706792 = {S67305, 137, { &P19777,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734753 = {S67306, 131, { &P19777,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105P Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2670617 = {S67307, 140, { &P22598,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6506 Build/10.1.A.1.395) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2736508 = {S67308, 133, { &P22598,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6506 Build/10.1.A.1.395) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2975051 = {S67309, 137, { &P24097,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919N Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA679584 = {S67310, 81, { &P13552,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaC6-00)/UC Browser8.2.0.132/50/355/UCWEB Mobile*/
const Device UA785734 = {S67311, 32, { &P13552,&P5129,&P24667,&P18092 }};	/*Useragent - NokiaC6-00/UCWEB8.2.0.116/50/800*/
const Device UA1951945 = {S67312, 81, { &P13552,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaC6-00)/UC Browser8.5.0.183/50/355/UCWEB Mobile*/
const Device UA1955356 = {S67313, 32, { &P13552,&P5129,&P24671,&P18092 }};	/*Useragent - NokiaC6-00/UCWEB8.5.0.163/50/999*/
const Device UA2981508 = {S67314, 129, { &P13552,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiac6-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile  3gpp-gba*/
const Device UA2428354 = {S67315, 158, { &P14123,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P970 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2419071 = {S67316, 137, { &P22677,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ThL W3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2677283 = {S67317, 130, { &P17323,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6800 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2975081 = {S67318, 141, { &P19842,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023840 = {S67319, 142, { &P19842,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; HTC Butterfly Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA253809 = {S67320, 140, { &P17299,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SCH-M828C[X] Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA379642 = {S67321, 61, { &P17299,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; SCH-M828C Build/FROYO)*/
const Device UA744005 = {S67322, 137, { &P17299,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; SCH-M828C Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA638537 = {S67323, 74, { &P15364,&P7127,&P21171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:12.0) Gecko/X Firefox/12.0 SeaMonkey/2.9.1*/
const Device UA653090 = {S67324, 134, { &P22236,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SA3CNT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA843950 = {S67325, 153, { &P12110,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Legend Build/GRH55; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA901931 = {S67326, 141, { &P14207,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; pcdadr6350 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1995320 = {S67327, 150, { &P14207,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Incredible2_ADR6350 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028738 = {S67328, 138, { &P14207,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6350 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2179923 = {S67329, 141, { &P14207,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; pcdadr6350 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2546496 = {S67330, 138, { &P14207,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6350 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026900 = {S67331, 139, { &P18072,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SHV-E120L Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2028527 = {S67332, 139, { &P18072,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2223252 = {S67333, 139, { &P18072,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120K Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2229398 = {S67334, 139, { &P18072,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2410329 = {S67335, 139, { &P18072,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SHV-E120S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2456566 = {S67336, 135, { &P18072,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; E120L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2230489 = {S67337, 145, { &P15854,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ607-14) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2355430 = {S67338, 144, { &P23978,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; WALKMAN Build/icx1227_D101) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2427255 = {S67339, 144, { &P23978,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; WALKMAN Build/icx1227_E110) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752829 = {S67340, 144, { &P23978,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; WALKMAN Build/icx1227_E111) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2510508 = {S67341, 166, { &P18442,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-N7100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2511997 = {S67342, 166, { &P18442,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-N7100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3022880 = {S67343, 170, { &P18442,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N7100 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3046482 = {S67344, 166, { &P18442,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-N7105 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3035691 = {S67345, 170, { &P15364,&P7133,&P17252,&P18091 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; compatible; BDCbot/1.0; +http://ecommerce.bigdatacorp.com.br/faq.aspx) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1*/
const Device UA3038805 = {S67346, 174, { &P15364,&P7133,&P17252,&P18091 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; compatible; BDCbot/1.0; +http://www.bigdatacorp.com.br/bigweb/bdcbot.html) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1*/
const Device UA3068050 = {S67347, 130, { &P12889,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3067904 = {S67348, 140, { &P24095,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5306 Build/12.0.A.1.284) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA804971 = {S67349, 137, { &P21315,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; CRESTA.CTP888 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1984434 = {S67350, 137, { &P21315,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; CRESTA.CTP888 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1993596 = {S67351, 136, { &P21105,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS15SH Build/S6230) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996062 = {S67352, 136, { &P21105,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS15SH Build/S7120) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2007445 = {S67353, 136, { &P21105,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS15SH Build/S7260) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2582990 = {S67354, 136, { &P21105,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS15SH Build/S2201) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469385 = {S67355, 159, { &P17108,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Inspire 4G Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2531255 = {S67356, 159, { &P17108,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Inspire 4G Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2664408 = {S67357, 158, { &P17108,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Inspire 4G Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2751779 = {S67358, 159, { &P17108,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Inspire 4G Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2669598 = {S67359, 134, { &P20584,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Z120 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2670726 = {S67360, 135, { &P21606,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LGL86C Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734883 = {S67361, 138, { &P23127,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID7047(3G)-4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2738335 = {S67362, 131, { &P23127,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MID7047 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2893763 = {S67363, 89, { &P17595,&P17296,&P24329,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:22.0) Gecko/X Firefox/22.0 SeaMonkey/2.19*/
const Device UA62430 = {S67364, 39, { &P13863,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GX200 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA90400 = {S67365, 134, { &P18045,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; SBM003SH Build/S0800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA171958 = {S67366, 136, { &P18045,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM003SH Build/S1400) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA234761 = {S67367, 136, { &P18045,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM003SH Build/S1810) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA248355 = {S67368, 136, { &P18045,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM003SH Build/S1900) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA311109 = {S67369, 136, { &P18045,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM003SH Build/S1510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA328559 = {S67370, 136, { &P18045,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM003SH Build/S1700) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA557868 = {S67371, 136, { &P18045,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM003SH Build/S1600) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA677278 = {S67372, 134, { &P18045,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; SBM003SH Build/S1100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA687379 = {S67373, 136, { &P18045,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM003SH Build/S1201) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1912911 = {S67374, 60, { &P18045,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SBM003SH Build/S1900)*/
const Device UA90798 = {S67375, 77, { &P15364,&P7133,&P4536,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; Opera Mobi/49; U; en) Presto/2.4.18 Version/10.00*/
const Device UA625100 = {S67376, 104, { &P14862,&P5204,&P17347,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2700c-2/UC Browser8.0.3.107/69/444*/
const Device UA687192 = {S67377, 95, { &P15364,&P16110,&P22482,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6.1*/
const Device UA2230873 = {S67378, 137, { &P21465,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Candy TV Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2403212 = {S67379, 138, { &P21465,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Candy TV  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2756233 = {S67380, 142, { &P15438,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Sensation Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2871485 = {S67381, 158, { &P12280,&P22882,&P22783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.40 (KHTML, like Gecko) 1Password/4.2.1 (like Version/11A4372q Mobile/7.0 Safari/8536.25)*/
const Device UA2969274 = {S67382, 158, { &P12280,&P22882,&P22783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) 1Password/4.2.2 (like Version/11A4414e Mobile/7.0 Safari/8536.25)*/
const Device UA3094870 = {S67383, 158, { &P12280,&P22882,&P22783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) 1Password/4.2.7 (like Version/11A465 Mobile/7.0 Safari/8536.25)*/
const Device UA3115393 = {S67384, 162, { &P12280,&P22882,&P22783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) 1Password/4.2.7 (like Version/11A501 Mobile/7.0.2 Safari/8536.25)*/
const Device UA3128082 = {S67385, 163, { &P12280,&P22882,&P22783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) 1Password/4.2.7 (like Version/11A470a Mobile/7.0.1 Safari/8536.25)*/
const Device UA2943640 = {S67386, 152, { &P18557,&P20706,&P24678,&P18092 }};	/*Useragent - Nokia308/2.0 (05.85) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia308) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3068196 = {S67387, 136, { &P20937,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ADR910L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA26297 = {S67388, 73, { &P13581,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-T249/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61022 = {S67389, 71, { &P13581,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-T249R/ NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA1994363 = {S67390, 127, { &P21130,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; D50 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029834 = {S67391, 127, { &P21130,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D50 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2400403 = {S67392, 140, { &P20294,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC_T528w Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2466160 = {S67393, 131, { &P20294,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T528w Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2528009 = {S67394, 140, { &P20294,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T528w Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747420 = {S67395, 62, { &P20294,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC T528w Build/IMM76D)*/
const Device UA2424438 = {S67396, 151, { &P15241,&P5843,&P24673,&P18092 }};	/*Useragent - Nokia300/5.0 (07.03) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia300) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2739604 = {S67397, 140, { &P18386,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I757M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986537 = {S67398, 62, { &P18386,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I757M Build/JZO54K)*/
const Device UA2748507 = {S67399, 161, { &P22964,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; ALCATEL_ONE_TOUCH_6010X_Orange Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2937080 = {S67400, 154, { &P22964,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ALCATEL ONE TOUCH 6010X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2947376 = {S67401, 169, { &P18113,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2974876 = {S67402, 136, { &P21792,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9200 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2961748 = {S67403, 144, { &P19775,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-in; Micromax A210 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078235 = {S67404, 57, { &P19775,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; A210 Build/JOP40D)*/
const Device UA2963718 = {S67405, 137, { &P24097,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919N Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3019362 = {S67406, 130, { &P24097,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919N Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3025164 = {S67407, 129, { &P24419,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; de-de; ME302C Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3104614 = {S67408, 58, { &P24419,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; ME302C Build/JDQ39)*/
const Device UA31979 = {S67409, 104, { &P12446,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-T746/T746BMJB3 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA20574 = {S67410, 45, { &P14135,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-355A/1.0 ObigoInternetBrowser/Q03C*/
const Device UA24521 = {S67411, 45, { &P14135,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-355D/1.0 ObigoInternetBrowser/Q03C*/
const Device UA41390 = {S67412, 59, { &P14135,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-355/1.0 model-bouygues ObigoInternetBrowser/Q03C*/
const Device UA54157 = {S67413, 57, { &P14135,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-355/1.0 model-orange ObigoInternetBrowser/Q03C*/
const Device UA63177 = {S67414, 44, { &P14135,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-355/1.0 ObigoInternetBrowser/Q03C*/
const Device UA33219 = {S67415, 128, { &P15102,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; MZ605 Build/H.6.3-7) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA43045 = {S67416, 131, { &P15102,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; pt-br; MZ605 Build/H.6.3-28.6) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA43046 = {S67417, 129, { &P15102,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; MZ605 Build/H.6.3-28) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA43049 = {S67418, 129, { &P15102,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; MZ605 Build/H.6.3-25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA43051 = {S67419, 131, { &P15102,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; es-us; MZ605 Build/H.6.3-28.4) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA43052 = {S67420, 129, { &P15102,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; MZ605 Build/H.6.3-23) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA58950 = {S67421, 129, { &P15102,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-ca; MZ605 Build/H.6.3-16) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA58957 = {S67422, 129, { &P15102,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; MZ605 Build/H.6.3-18) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA58956 = {S67423, 129, { &P15102,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; MZ605 Build/H.6.3-21) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA58961 = {S67424, 129, { &P15102,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; MZ605 Build/H.6.1-45) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA58969 = {S67425, 129, { &P15102,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; MZ605 Build/H.6.3-29) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA73691 = {S67426, 129, { &P15102,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; MZ605 Build/H.6.3-27) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA672808 = {S67427, 41, { &P11854,&P5890,&P24665,&P18092 }};	/*Useragent - NokiaE71/UC Browser8.0.4.121/28/444/UCWEB*/
const Device UA771908 = {S67428, 29, { &P11854,&P5890,&P24665,&P18092 }};	/*Useragent - NokiaE71/UCWEB8.0.3.99/28/800*/
const Device UA705194 = {S67429, 100, { &P14826,&P5843,&P17347,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac1-01/UC Browser8.0.3.107/69/444*/
const Device UA2485257 = {S67430, 131, { &P23804,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID1060 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3081026 = {S67431, 150, { &P18640,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT901 Build/7.7.1Q-144_SLS_S240-58) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA178098 = {S67432, 64, { &P15364,&P17017,&P17825,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/3.5) KHTML/3.5.4 (like Gecko)*/
const Device UA820838 = {S67433, 93, { &P17595,&P17295,&P18198,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.2.11) Gecko/X Thunderbird/3.1.5*/
const Device UA2156004 = {S67434, 88, { &P20247,&P22835,&P17470,&P18092 }};	/*Useragent - SpiceM5600/MTK Release/01.01.2012 Browser/wap2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2158435 = {S67435, 11, { &P20247,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M5600*/
const Device UA2556558 = {S67436, 12, { &P20247,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M5600n*/
const Device UA2428179 = {S67437, 219, { &P15919,&P18066,&P17373,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/6.5.28192/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2621593 = {S67438, 217, { &P15919,&P18066,&P17373,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/6.5.28192/28.3234; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2624640 = {S67439, 219, { &P15919,&P18066,&P17373,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/6.5.28192/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2556942 = {S67440, 150, { &P15364,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666824 = {S67441, 157, { &P15364,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; A26 Build/CROSSCROSS_A26) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2699515 = {S67442, 150, { &P15364,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T753 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2699663 = {S67443, 156, { &P15364,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; Maxtron T1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2856462 = {S67444, 155, { &P15364,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; CROSS A28 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3015165 = {S67445, 149, { &P15364,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; A7# Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050307 = {S67446, 149, { &P15364,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; A22 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3071439 = {S67447, 160, { &P15364,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MAXTRON VENTUS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102100 = {S67448, 149, { &P15364,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MOVIMAX P9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2584857 = {S67449, 142, { &P16292,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.2; Galaxy Nexus Build/ICL53F) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602922 = {S67450, 142, { &P16292,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2604720 = {S67451, 142, { &P16292,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.1; Galaxy Nexus Build/ITL41F) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2864975 = {S67452, 135, { &P16292,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2718564 = {S67453, 141, { &P17346,&P17146,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9A405 Safari/7534.48.3*/
const Device UA2737805 = {S67454, 139, { &P17346,&P17146,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/9A334 Safari/7534.48.3*/
const Device UA2967842 = {S67455, 96, { &P12004,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN95 8GB) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA2970469 = {S67456, 92, { &P12004,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN95) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3030888 = {S67457, 152, { &P18583,&P5043,&P24679,&P18092 }};	/*Useragent - Nokia110/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia110) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA20225 = {S67458, 157, { &P11985,&P17483,&P3459,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-BL40/V10c Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA24451 = {S67459, 137, { &P11985,&P17483,&P3459,&P18092 }};	/*Useragent - LG-BL40-Orange/V10b Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA44273 = {S67460, 131, { &P11985,&P17483,&P3459,&P18092 }};	/*Useragent - LG/BL40g/V10a Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA51627 = {S67461, 131, { &P11985,&P17483,&P3459,&P18092 }};	/*Useragent - LG-BL40f/v1.0 Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA54556 = {S67462, 157, { &P11985,&P17483,&P3459,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-BL40/V10b Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA55146 = {S67463, 130, { &P11985,&P17483,&P3459,&P18092 }};	/*Useragent - LG-BL40/V10a Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA60585 = {S67464, 130, { &P11985,&P17483,&P3459,&P18092 }};	/*Useragent - LG-BL40-V10d Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA64077 = {S67465, 130, { &P11985,&P17483,&P3459,&P18092 }};	/*Useragent - LG/BL40/V10a Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA774855 = {S67466, 153, { &P13257,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Droid Build/FRG83G; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1992470 = {S67467, 117, { &P14482,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-01.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA2335747 = {S67468, 115, { &P14482,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-01/06.00; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA2728489 = {S67469, 116, { &P14482,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-01.5/07.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6*/
const Device UA680965 = {S67470, 81, { &P11905,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA6120c)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA2589348 = {S67471, 79, { &P11905,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokia6120c) UCBrowser8.2.0.132/69/452/UCWEB Mobile*/
const Device UA1956614 = {S67472, 133, { &P21818,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IT-M809RC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2557241 = {S67473, 92, { &P15364,&P18110,&P21970,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:10.0.12) Gecko/X conkeror/1.0pre (Debian-1.0~~pre+gitX-1)*/
const Device UA2701758 = {S67474, 90, { &P15364,&P18110,&P21970,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:10.0.12) Gecko/X conkeror/1.0pre (Debian-1.0~~pre+gitX-1)*/
const Device UA3091470 = {S67475, 77, { &P15364,&P18110,&P21970,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:22.0) Gecko/X Firefox/22.0 conkeror/1.0pre*/
const Device UA2362305 = {S67476, 136, { &P20414,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SmartTabII10 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2975865 = {S67477, 139, { &P18465,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021871 = {S67478, 141, { &P18465,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire2 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022517 = {S67479, 138, { &P18465,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Amazon Kindle Fire Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3068391 = {S67480, 136, { &P14123,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P970 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094787 = {S67481, 137, { &P14123,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P970h Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126216 = {S67482, 137, { &P14123,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P970g Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA433661 = {S67483, 62, { &P15364,&P18145,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (X11; FreeBSD 7.1-RELEASE i386; U; en) Presto/2.1.1*/
const Device UA2854622 = {S67484, 66, { &P15364,&P18145,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (X11; FreeBSD 7.4-RELEASE-p9 amd64; U; en) Presto/2.1.1*/
const Device UA2030240 = {S67485, 135, { &P16598,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Iconia A500 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2032717 = {S67486, 128, { &P16598,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A500 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2233811 = {S67487, 128, { &P16598,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A500 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2346881 = {S67488, 152, { &P18296,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; One V Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2384941 = {S67489, 152, { &P18296,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; One V Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2531700 = {S67490, 156, { &P18296,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One V Build/JZO54K; CyanogenMod-CM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2382920 = {S67491, 128, { &P20269,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A211 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2416683 = {S67492, 137, { &P23064,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ThL W1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467640 = {S67493, 137, { &P23064,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ThL W1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566693 = {S67494, 131, { &P15985,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2634390 = {S67495, 132, { &P15985,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200L Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2637793 = {S67496, 131, { &P15985,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P6200 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2605512 = {S67497, 146, { &P19867,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2753204 = {S67498, 142, { &P19842,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Butterfly Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2797329 = {S67499, 142, { &P19842,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Butterfly Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937628 = {S67500, 138, { &P19842,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC X920e Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2781010 = {S67501, 133, { &P23958,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3;en-us; ZTE V793 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3023847 = {S67502, 161, { &P23958,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; ZTE V793 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/ZTE-Android-MMS-V2.0*/
const Device UA3074155 = {S67503, 137, { &P23958,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ZTE V793 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2975710 = {S67504, 138, { &P20076,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26w Build/6.1.A.2.45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA30322 = {S67505, 73, { &P11729,&P6173,&P3417,&P18092 }};	/*Useragent - BlackBerry8100/4.2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100*/
const Device UA190796 = {S67506, 30, { &P15364,&P21445,&P18405,&P18092 }};	/*Useragent - Opera/9.25 (Windows ME; U; en)*/
const Device UA233409 = {S67507, 30, { &P15364,&P21445,&P18405,&P18092 }};	/*Useragent - Opera/9.20 (Windows ME; U; en)*/
const Device UA2232871 = {S67508, 128, { &P20556,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A701 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2383874 = {S67509, 186, { &P12473,&P5204,&P24672,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia2730c-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile*/
const Device UA2567205 = {S67510, 138, { &P18456,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T699 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2659640 = {S67511, 134, { &P23171,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; G2S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2714343 = {S67512, 105, { &P18182,&P18148,&P22782,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows Phone 8.0; Trident/7.0; rv:11.0; IEMobile/11.0; ARM; Touch; NOKIA; Nokia) like Gecko*/
const Device UA4815 = {S67513, 59, { &P12192,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA22622 = {S67514, 124, { &P12192,&P5448,&P3430,&P18092 }};	/*Useragent - PPC; 480x800; HTC_Touch_Diamond2_T5353-Orange; OpVer 93.172.2.732 Opera/9.5 (Microsoft Windows 6.1; Opera Mobi/16071; U; fr)*/
const Device UA27943 = {S67515, 62, { &P12192,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; zh-cn)*/
const Device UA27949 = {S67516, 62, { &P12192,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; zh-tw)*/
const Device UA27954 = {S67517, 59, { &P12192,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; pt)*/
const Device UA88483 = {S67518, 66, { &P12192,&P5448,&P3430,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_Touch_Diamond2 Opera/9.50 (Windows NT 5.1; U; de)*/
const Device UA89272 = {S67519, 59, { &P12192,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; sv)*/
const Device UA266939 = {S67520, 59, { &P12192,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; nl)*/
const Device UA372496 = {S67521, 59, { &P12192,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; ru)*/
const Device UA550822 = {S67522, 62, { &P12192,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; es-ES)*/
const Device UA637878 = {S67523, 66, { &P12192,&P5448,&P3430,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_Touch_Diamond2 Opera/9.50 (Windows NT 5.1; U; nl)*/
const Device UA762649 = {S67524, 59, { &P12192,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; nb)*/
const Device UA821115 = {S67525, 59, { &P12192,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; de)*/
const Device UA854336 = {S67526, 59, { &P12192,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; fr)*/
const Device UA2134767 = {S67527, 88, { &P12192,&P5448,&P3430,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_Touch_Diamond2/2.16.164.1 (67041) Opera/9.50 (Windows NT 5.1; U; es-ES)*/
const Device UA2242352 = {S67528, 59, { &P12192,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.50 (Windows NT 5.1; U; el)*/
const Device UA9430 = {S67529, 66, { &P13535,&P5843,&P4030,&P18092 }};	/*Useragent - Nokia6350-1d/2.0(ATT.5.93) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA15869 = {S67530, 124, { &P13535,&P5843,&P4030,&P18092 }};	/*Useragent - Nokia6350/2.0(ATT.4.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA152331 = {S67531, 58, { &P18254,&P18110,&P22939,&P18092 }};	/*Useragent - Liferea/0.x.x (Linux; en_US.UTF-8; http://liferea.sf.net/)*/
const Device UA566609 = {S67532, 76, { &P17595,&P17295,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:2.0pre) Gecko/X Firefox/4.0pre*/
const Device UA2353592 = {S67533, 135, { &P20361,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X401 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659120 = {S67534, 135, { &P20361,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; X401 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2362410 = {S67535, 90, { &P20025,&P22849,&P3534,&P18092 }};	/*Useragent - ALCATEL_ONE_TOUCH_838/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1ObigoInternetBrowser/Q05A*/
const Device UA2422811 = {S67536, 121, { &P15791,&P5055,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; Nokia603) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile*/
const Device UA2422871 = {S67537, 120, { &P11907,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE52-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2504803 = {S67538, 42, { &P11907,&P5890,&P24673,&P18092 }};	/*Useragent - NokiaE52-1UCBrowser/8.7.1.234/28/444/UCWEB*/
const Device UA2696108 = {S67539, 102, { &P11907,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiae52-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2598159 = {S67540, 137, { &P21081,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ZTE V790 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2676916 = {S67541, 136, { &P18438,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E973 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707758 = {S67542, 137, { &P18438,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F200K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708425 = {S67543, 137, { &P18438,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F200S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2711140 = {S67544, 129, { &P18438,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E973 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2736696 = {S67545, 137, { &P18438,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F200L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2795974 = {S67546, 144, { &P21507,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SonyC1605 Build/11.3.A.2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999660 = {S67547, 140, { &P21507,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; C1605 Build/11.3.A.2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050347 = {S67548, 62, { &P21507,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; C1605 Build/11.3.A.2.1)*/
const Device UA2882673 = {S67549, 145, { &P23546,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; New Andromax-i Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050773 = {S67550, 67, { &P23546,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; New Andromax-i Build/JZO54K)*/
const Device UA1955242 = {S67551, 154, { &P12980,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; LG-E510 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2512894 = {S67552, 146, { &P12972,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SAMSUNG-SGH-T989 Build/JOP40G) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2535474 = {S67553, 137, { &P12972,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T989 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2536593 = {S67554, 145, { &P12972,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-T989 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2537216 = {S67555, 138, { &P12972,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-T989 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2769557 = {S67556, 96, { &P15364,&P5690,&P21781,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.21 (KHTML, like Gecko) QupZilla/1.4.3 Safari/537.21*/
const Device UA2935855 = {S67557, 184, { &P18436,&P21460,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.3; WOW64; Trident/7.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3)*/
const Device UA2957454 = {S67558, 189, { &P18436,&P21460,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.3; Win64; x64; Trident/7.0; Touch; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Tablet PC 2.0; InfoPath.3)*/
const Device UA62741 = {S67559, 77, { &P14824,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-S5200/S5200DDIK3 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA251931 = {S67560, 138, { &P15539,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Glacier Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA315565 = {S67561, 138, { &P15539,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Glacier Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA330606 = {S67562, 138, { &P15539,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Glacier Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA337994 = {S67563, 150, { &P15539,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Glacier Build/MIUI demonspeed!) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA777794 = {S67564, 156, { &P15539,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; demonSPEED! Glacier Build/MIUI dS 1.7.15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018419 = {S67565, 140, { &P15539,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Glacier Build/MIUIr1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018801 = {S67566, 161, { &P15539,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; demonSPEED! Glacier Build/MIUI 1.6.17 dS v1.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019277 = {S67567, 161, { &P15539,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; demonSPEED! Glacier Build/MIUI 1.6.24 dS v1.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2213216 = {S67568, 145, { &P15539,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Glacier Build/MIUI dS_MOD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA923952 = {S67569, 79, { &P13977,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaN73)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2257664 = {S67570, 138, { &P23158,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; IMO S88 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426892 = {S67571, 155, { &P15802,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Amaze 4G Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2670729 = {S67572, 142, { &P19862,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Micromax A110 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676758 = {S67573, 126, { &P19862,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A110 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3015362 = {S67574, 59, { &P15364,&P18346,&P22171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10; rv:23.0) Gecko/X Firefox/23.0*/
const Device UA566080 = {S67575, 161, { &P23984,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; id-id; T1C Build/I700T_P4_11A_AC73_FTN_R_0_03_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA569512 = {S67576, 160, { &P23984,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; id-id; T1C Build/I700T_P6_20_AC73_FTN_R_0_01_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627414 = {S67577, 159, { &P23984,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; id-id; T1C Build/I700T_P6_T02_AC73_FT_H44_R_0_01_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA685128 = {S67578, 158, { &P23984,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; id-id; T1C Build/I700T_P6_20_AC73_FT_S42_R_0_01_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA899044 = {S67579, 132, { &P21050,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-uk; GEM10212 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032250 = {S67580, 129, { &P20812,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AT330 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2423327 = {S67581, 142, { &P20397,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; ZXY-ZTE_V6700 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2536892 = {S67582, 93, { &P18672,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5302) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2671832 = {S67583, 139, { &P18451,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26ii Build/6.1.A.2.55) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2716454 = {S67584, 94, { &P18364,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC5-05) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2719041 = {S67585, 137, { &P18636,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; VS930 4G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2738020 = {S67586, 130, { &P18636,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; VS930 4G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2973478 = {S67587, 138, { &P18290,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ617 Build/7.7.1-85_MZ617-27) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023586 = {S67588, 93, { &P15991,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6102) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3131415 = {S67589, 127, { &P18385,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Nexus 7 Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3131717 = {S67590, 128, { &P18385,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3132010 = {S67591, 129, { &P18385,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 7 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA18407 = {S67592, 137, { &P14046,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Vivo Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20846 = {S67593, 137, { &P14046,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-ca; HTC-Vivo Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA57613 = {S67594, 137, { &P14046,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; HTC_Vivo Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA362088 = {S67595, 145, { &P15911,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; XT862 Build/5.5.1_84_D3G-55) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA630926 = {S67596, 136, { &P15911,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; XT862 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA715654 = {S67597, 136, { &P15911,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT862 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA845974 = {S67598, 136, { &P15911,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT862 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA863523 = {S67599, 137, { &P15911,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026768 = {S67600, 137, { &P15911,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2341839 = {S67601, 136, { &P21016,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; LGL21 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2431073 = {S67602, 171, { &P18672,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2469299 = {S67603, 138, { &P20878,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734830 = {S67604, 71, { &P15364,&P7133,&P22169,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:21.0) Gecko/X Firefox/21.0 slurp*/
const Device UA2767668 = {S67605, 64, { &P15364,&P7133,&P22169,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:21.0) Gecko/X Firefox/21.0 slurp*/
const Device UA2767961 = {S67606, 77, { &P15364,&P7133,&P22169,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:21.0) Gecko/X Firefox/21.0 slurp slurp*/
const Device UA2736582 = {S67607, 133, { &P23704,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHW-M500W Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2756004 = {S67608, 130, { &P23773,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8010 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2891614 = {S67609, 154, { &P19767,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ALCATEL ONE TOUCH 8008D Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2 Mobile Safari/534.30*/
const Device UA26462 = {S67610, 47, { &P13599,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-VX5600/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA246372 = {S67611, 102, { &P17595,&P17296,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; Edition MacAppStore; en) Presto/2.9.168 Version/11.51*/
const Device UA2564028 = {S67612, 140, { &P20439,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SOL21 Build/9.0.F.2.128) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3000416 = {S67613, 149, { &P23323,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Archos 50 Platinum Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3074553 = {S67614, 71, { &P23323,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Archos 50 Platinum Build/JZO54K)*/
const Device UA43633 = {S67615, 30, { &P13715,&P17017,&P4902,&P18092 }};	/*Useragent - Cricket-A410/1.0 Polaris/v6.17*/
const Device UA805383 = {S67616, 139, { &P21255,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; novo7_Advanced2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2543399 = {S67617, 68, { &P21255,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; novo7_Advanced2 Build/IML74K)*/
const Device UA874423 = {S67618, 158, { &P16273,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T679 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA908787 = {S67619, 154, { &P16273,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T679 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026242 = {S67620, 158, { &P16273,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T679 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558428 = {S67621, 154, { &P12108,&P5843,&P24676,&P18092 }};	/*Useragent - Nokia7230/5.0 (10.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7230) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2755525 = {S67622, 126, { &P18239,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A510 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA121775 = {S67623, 61, { &P15364,&P17017,&P18419,&P18092 }};	/*Useragent - Mozilla/3.01 (compatible; AmigaVoyager/2.95; AmigaOS/MC680x0)*/
const Device UA1982872 = {S67624, 84, { &P13324,&P5055,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; NokiaN8-00)/UC Browser8.5.0.183/51/355/UCWEB Mobile*/
const Device UA2157267 = {S67625, 32, { &P13324,&P5055,&P24671,&P18092 }};	/*Useragent - NokiaN8-00/UCWEB8.5.0.163/51/800*/
const Device UA2399588 = {S67626, 126, { &P13324,&P5055,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; fa; nokian8-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile  3gpp-gba*/
const Device UA2710309 = {S67627, 131, { &P15364,&P18118,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.31 (KHTML, like Gecko) Ubuntu Chromium/26.0.1410.63 Chrome/26.0.1410.63 Safari/537.31*/
const Device UA2717248 = {S67628, 133, { &P15364,&P18118,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.31 (KHTML, like Gecko) Ubuntu Chromium/26.0.1410.63 Chrome/26.0.1410.63 Safari/537.31*/
const Device UA2737769 = {S67629, 163, { &P18466,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amazon Kindle Fire HD Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2858884 = {S67630, 162, { &P18466,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amazon Kindle Fire HD Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA93455 = {S67631, 130, { &P15364,&P17285,&P22443,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/532.9 (KHTML, like Gecko) Iron/4.0.280.0 Chrome/4.0.280.0 Safari/532.9*/
const Device UA2362332 = {S67632, 137, { &P15364,&P19822,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.11 (KHTML, like Gecko) Ubuntu/12.10 Chromium/23.0.1271.64 Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2362348 = {S67633, 139, { &P15364,&P19822,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Ubuntu/12.10 Chromium/23.0.1271.64 Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2579440 = {S67634, 171, { &P15364,&P19822,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/23.0.1271.95 Chrome/23.0.1271.95 Safari/537.6+ Ubuntu/12.10 (3.6.1-0ubuntu1) Epiphany/3.6.1*/
const Device UA2428320 = {S67635, 140, { &P20169,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; N320 Build/LAVALAVA_N320) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2531145 = {S67636, 93, { &P14186,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2708411 = {S67637, 94, { &P14186,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830M) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2854314 = {S67638, 94, { &P14186,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; GT-S5830L) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2663626 = {S67639, 141, { &P23476,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; sr-rs; ZTE Kis Lite Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3033940 = {S67640, 67, { &P23776,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGP321 Build/10.3.1.A.0.244)*/
const Device UA3058958 = {S67641, 138, { &P23776,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SGP321 Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3059012 = {S67642, 142, { &P23776,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SonySGP321 Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3095886 = {S67643, 134, { &P15767,&P17146,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9A405 Safari/7534.48.3*/
const Device UA3113536 = {S67644, 132, { &P15767,&P17146,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9A334 Safari/7534.48.3*/
const Device UA62797 = {S67645, 60, { &P15364,&P5208,&P3534,&P18092 }};	/*Useragent - Mozilla/4.1 (U; BREW 3.1.5; en-US; Teleca/Q05A/INT/SAM-M800)*/
const Device UA87016 = {S67646, 51, { &P15364,&P5208,&P3534,&P18092 }};	/*Useragent - Mozilla/4.1 (U; BREW 3.1.5; en-US; Teleca/Q05A/INT)*/
const Device UA165765 = {S67647, 141, { &P15364,&P7133,&P22400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Comodo_Dragon/8.0.0.4 Chrome/8.0.552.215 Safari/534.10*/
const Device UA691895 = {S67648, 43, { &P12088,&P5129,&P24665,&P18092 }};	/*Useragent - NokiaX6-00/UC Browser8.0.4.121/50/444/UCWEB*/
const Device UA1901294 = {S67649, 31, { &P12088,&P5129,&P24665,&P18092 }};	/*Useragent - NokiaX6-00/UCWEB8.0.3.99/50/800*/
const Device UA728039 = {S67650, 160, { &P11747,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Desire Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA904297 = {S67651, 160, { &P11747,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Desire Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025155 = {S67652, 165, { &P11747,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Desire CDMA Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2491517 = {S67653, 53, { &P18088,&P18432,&P24676,&P18092 }};	/*Useragent - IUC(U;iOS 6.0.1;Zh-cn;320*480;)/UCWEB8.9.0.238/44/997*/
const Device UA2491523 = {S67654, 53, { &P18088,&P18432,&P24676,&P18092 }};	/*Useragent - IUC(U;iOS 6.0.2;Zh-cn;320*568;)/UCWEB8.9.0.238/44/997*/
const Device UA2754372 = {S67655, 134, { &P18113,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT912 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893617 = {S67656, 131, { &P24144,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMID901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3068045 = {S67657, 137, { &P18507,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8160 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094969 = {S67658, 138, { &P18507,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8160P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126283 = {S67659, 138, { &P18507,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8160L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066523 = {S67660, 138, { &P22729,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT918 Build/2_26D_2009) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065337 = {S67661, 138, { &P21065,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; CINK FIVE Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094199 = {S67662, 134, { &P21065,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CINK+ Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA31229 = {S67663, 73, { &P11726,&P5810,&P3417,&P18092 }};	/*Useragent - BlackBerry8800/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102*/
const Device UA30022 = {S67664, 135, { &P12280,&P17137,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7D11 Twitter for iPhone*/
const Device UA30059 = {S67665, 135, { &P12280,&P17137,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Twitter for iPhone*/
const Device UA295282 = {S67666, 134, { &P12280,&P17137,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7C144 Twitter for iPhone*/
const Device UA2510402 = {S67667, 91, { &P17595,&P17297,&P20456,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:18.0) Gecko/X Firefox/18.0 SeaMonkey/2.15.2*/
const Device UA2460050 = {S67668, 152, { &P20176,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Woxter Tablet PC 101IPS DUAL Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2655924 = {S67669, 81, { &P20176,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Woxter Tablet PC 101IPS DUAL Build/JRO03H)*/
const Device UA3099981 = {S67670, 153, { &P20176,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Woxter Tablet PC 101 IPS DUAL Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2773099 = {S67671, 168, { &P15201,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710e Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2784798 = {S67672, 165, { &P15201,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA190700 = {S67673, 143, { &P15364,&P7133,&P22402,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Comodo_Dragon/10.0.0.2 Chrome/10.0.648.204 Safari/534.16*/
const Device UA1970670 = {S67674, 127, { &P23992,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; N12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2480029 = {S67675, 154, { &P23992,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N12 Build/N12_Android 4.0.4_X fwd.test-keys) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2684990 = {S67676, 56, { &P23992,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; N12 Build/IML74K)*/
const Device UA2819244 = {S67677, 160, { &P23992,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N12 Build/rk29sdk_Android 4.0.4_X.X fwd.test-keys) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1996830 = {S67678, 157, { &P17108,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Inspire 4G Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2383541 = {S67679, 168, { &P17108,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Inspire 4G Build/JRO03R; CyanogenMod-JellyTime_R15) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2436053 = {S67680, 168, { &P17108,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Inspire 4G Build/JZO54K; CyanogenMod-JellyTime_R30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2692682 = {S67681, 170, { &P17108,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Inspire 4G Build/JRO03L; CyanogenMod-JellyTime_R10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2406816 = {S67682, 134, { &P20813,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GoTab GBT9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2537940 = {S67683, 171, { &P15575,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2538063 = {S67684, 171, { &P15575,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2581108 = {S67685, 172, { &P15575,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2598800 = {S67686, 172, { &P15575,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2599516 = {S67687, 171, { &P15575,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2663513 = {S67688, 171, { &P15575,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2672457 = {S67689, 140, { &P18465,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire2 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676919 = {S67690, 139, { &P18465,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2709091 = {S67691, 138, { &P18465,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Amazon Kindle Fire Build/JOP40C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735514 = {S67692, 140, { &P18465,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Amazon Kindle Fire Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2753111 = {S67693, 139, { &P18451,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26ii Build/6.1.A.2.55) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2968629 = {S67694, 94, { &P14983,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC7-00) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3022472 = {S67695, 130, { &P19971,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MT7001 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA9432 = {S67696, 84, { &P11978,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5 HTC_Touch2/1.0)*/
const Device UA21179 = {S67697, 87, { &P11978,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch2_T3333; Windows Phone 6.5)*/
const Device UA39946 = {S67698, 79, { &P11978,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Mega; Windows Phone 6.5)*/
const Device UA438804 = {S67699, 88, { &P11978,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch3G_T3333; Windows Phone 6.5)*/
const Device UA438862 = {S67700, 85, { &P11978,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch2/1.0; Windows Phone 6.5)*/
const Device UA925755 = {S67701, 91, { &P11978,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch2_T3333; Windows Phone 6.5.3.5)*/
const Device UA2257700 = {S67702, 137, { &P21448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SH530U Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2760725 = {S67703, 59, { &P21448,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SH530U Build/IMM76D)*/
const Device UA2028036 = {S67704, 133, { &P21331,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; LOOX Plus Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2466706 = {S67705, 128, { &P21331,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; LOOX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2231518 = {S67706, 132, { &P21057,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A7+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2363174 = {S67707, 130, { &P24536,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VL-110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2385699 = {S67708, 33, { &P14540,&P5843,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (NokiaX2-01/08.63)*/
const Device UA2937954 = {S67709, 128, { &P23956,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SM-T310 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA67410 = {S67710, 135, { &P14861,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-KU3700 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67707 = {S67711, 137, { &P14861,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; LG-KU3700 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA436479 = {S67712, 138, { &P14861,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-KU3700 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA537456 = {S67713, 59, { &P14861,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; LG-KU3700 Build/FRF91)*/
const Device UA196304 = {S67714, 122, { &P15364,&P17285,&P22450,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Iron/12.0.750.0 Chrome/12.0.750.0 Safari/534.30*/
const Device UA205693 = {S67715, 115, { &P15364,&P17285,&P22450,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.30 (KHTML, like Gecko) Iron/12.0.750.0 Chrome/12.0.750.0 Safari/534.30*/
const Device UA623805 = {S67716, 69, { &P17595,&P18578,&P17415,&P18092 }};	/*Useragent - Safari/7534.57.2 CFNetwork/520.4.3 Darwin/11.4.0 (i386) (MacPro1%2C1)*/
const Device UA800069 = {S67717, 72, { &P17595,&P18578,&P17415,&P18092 }};	/*Useragent - Safari/7534.57.2 CFNetwork/520.4.3 Darwin/11.4.0 (x86_64) (Macmini5%2C1)*/
const Device UA841219 = {S67718, 72, { &P17595,&P18578,&P17415,&P18092 }};	/*Useragent - Safari/7534.57.2 CFNetwork/520.4.3 Darwin/11.4.0 (x86_64) (MacBook6%2C1)*/
const Device UA2362274 = {S67719, 75, { &P17595,&P18578,&P17415,&P18092 }};	/*Useragent - Safari/7534.57.7 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookPro8%2C1)*/
const Device UA2763243 = {S67720, 75, { &P17595,&P18578,&P17415,&P18092 }};	/*Useragent - Safari/7534.57.7 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (MacBookAir4%2C1)*/
const Device UA2889179 = {S67721, 70, { &P17595,&P18578,&P17415,&P18092 }};	/*Useragent - Safari/7534.57.7 CFNetwork/520.5.1 Darwin/11.4.2 (x86_64) (iMac12%2C1)*/
const Device UA756101 = {S67722, 149, { &P13960,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; V9 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1996023 = {S67723, 139, { &P12482,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T959 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029007 = {S67724, 139, { &P12482,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T959 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033049 = {S67725, 61, { &P12482,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; SGH-T959 Build/JRO03L)*/
const Device UA2199026 = {S67726, 139, { &P12482,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T959 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258128 = {S67727, 139, { &P12482,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T959 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2524885 = {S67728, 139, { &P12482,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T959 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2727223 = {S67729, 61, { &P12482,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T959 Build/JZO54K)*/
const Device UA2202074 = {S67730, 132, { &P21723,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ETAB 801 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2483763 = {S67731, 151, { &P21723,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SANSUI ETAB 801VP Build/ICS.f08refem86.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2353045 = {S67732, 66, { &P15147,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; ST15i; 320*480) UCWEB7.9.0.94/139/352*/
const Device UA2384051 = {S67733, 66, { &P15147,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; ST15i; 320*480) UCWEB7.9.0.94/139/352*/
const Device UA2361017 = {S67734, 81, { &P18657,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-C3262/C3262DDLHA Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA2510698 = {S67735, 93, { &P15991,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6102) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2667589 = {S67736, 139, { &P22611,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-S7572 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2953738 = {S67737, 61, { &P22611,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S7572 Build/JZO54K)*/
const Device UA2991637 = {S67738, 130, { &P22611,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-S7572 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2676826 = {S67739, 141, { &P20399,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP8010 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714742 = {S67740, 161, { &P19862,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax A110 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA6863 = {S67741, 154, { &P11692,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; [en]; Series60/3.2 Nokia3250/1.00; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA27554 = {S67742, 186, { &P11692,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; Series60/3.2 Nokia3250/1.00; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.1 Mobile Safari/533.4 3gpp-gba*/
const Device UA34129 = {S67743, 160, { &P11692,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Nokia3250/1.00; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525*/
const Device UA64522 = {S67744, 88, { &P11692,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia3250/2.0 (04.60) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA80512 = {S67745, 9, { &P11692,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia3250*/
const Device UA571261 = {S67746, 82, { &P12871,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_Wildfire_S_A510e; 320*480) UCWEB7.9.3.103/139/800*/
const Device UA752627 = {S67747, 82, { &P12871,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_Wildfire_S_A510e; 320*480) UCWEB7.9.1.100/139/800*/
const Device UA1903679 = {S67748, 71, { &P12871,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_A510e; 320*480) UCWEB7.9.3.103/139/800*/
const Device UA2144175 = {S67749, 82, { &P12871,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_Wildfire_S_A510e; 320*480) UCWEB7.9.3.103/139/999*/
const Device UA2353534 = {S67750, 81, { &P12871,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_Wildfire_S_A510e; 320*480) UCWEB7.9.0.94/139/352*/
const Device UA2402575 = {S67751, 81, { &P12871,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_Wildfire_S_A510e; 320*480) UCWEB7.9.0.94/139/352*/
const Device UA2422423 = {S67752, 81, { &P12871,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_Wildfire_S_A510e; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2423851 = {S67753, 81, { &P12871,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_Wildfire_S_A510e; 320*480) UCWEB7.9.0.94/139/355*/
const Device UA2442898 = {S67754, 82, { &P12871,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_Wildfire_S_A510e; 320*480) UCWEB7.9.4.145/139/800*/
const Device UA2455838 = {S67755, 81, { &P12871,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_Wildfire_S_A510e; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2721938 = {S67756, 71, { &P12871,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_A510e; 320*480) UCWEB7.9.4.145/139/800*/
const Device UA2833197 = {S67757, 81, { &P12871,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_Wildfire_S_A510e; 320*480) UCWEB7.9.0.94/139/351*/
const Device UA2843601 = {S67758, 81, { &P12871,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_Wildfire_S_A510e; 320*480) UCWEB7.9.0.94/140/444*/
const Device UA2366497 = {S67759, 171, { &P18321,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2754644 = {S67760, 130, { &P20035,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2783616 = {S67761, 131, { &P20035,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA32767 = {S67762, 130, { &P12765,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; A100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025296 = {S67763, 136, { &P12765,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Mi-A100 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2360261 = {S67764, 133, { &P12765,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A100 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2888950 = {S67765, 143, { &P12765,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; RoverPad Air A100 Build/MID) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA69027 = {S67766, 102, { &P17968,&P5477,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.1-update1; Linux; Opera Mobi/ADR-1107051709; U; en) Presto/2.8.149 Version/11.10*/
const Device UA444130 = {S67767, 72, { &P15364,&P5208,&P17470,&P18092 }};	/*Useragent - BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 32054; Lang: zhcn) ucweb-squid*/
const Device UA489939 = {S67768, 70, { &P15364,&P5208,&P17470,&P18092 }};	/*Useragent - BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 600; Lang: zhcn) ucweb-squid*/
const Device UA518271 = {S67769, 72, { &P15364,&P5208,&P17470,&P18092 }};	/*Useragent - BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 90197; Lang: zhcn) ucweb-squid*/
const Device UA539623 = {S67770, 72, { &P15364,&P5208,&P17470,&P18092 }};	/*Useragent - BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 40105; Lang: zhcn) ucweb-squid*/
const Device UA673795 = {S67771, 72, { &P15364,&P5208,&P17470,&P18092 }};	/*Useragent - BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 32060; Lang: zhcn) ucweb-squid*/
const Device UA683452 = {S67772, 72, { &P15364,&P5208,&P17470,&P18092 }};	/*Useragent - BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 40118; Lang: zhcn) ucweb-squid*/
const Device UA685544 = {S67773, 72, { &P15364,&P5208,&P17470,&P18092 }};	/*Useragent - BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 40065; Lang: zhcn) ucweb-squid*/
const Device UA706459 = {S67774, 72, { &P15364,&P5208,&P17470,&P18092 }};	/*Useragent - BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 90242; Lang: zhcn) ucweb-squid*/
const Device UA732650 = {S67775, 72, { &P15364,&P5208,&P17470,&P18092 }};	/*Useragent - BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 32065; Lang: zhcn) ucweb-squid*/
const Device UA1869756 = {S67776, 72, { &P15364,&P5208,&P17470,&P18092 }};	/*Useragent - BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 90199; Lang: zhcn) ucweb-squid*/
const Device UA1880288 = {S67777, 71, { &P15364,&P5208,&P17470,&P18092 }};	/*Useragent - BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 1199; Lang: zhcn) ucweb-squid*/
const Device UA1895759 = {S67778, 72, { &P15364,&P5208,&P17470,&P18092 }};	/*Useragent - BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 90243; Lang: zhcn) ucweb-squid*/
const Device UA2145025 = {S67779, 72, { &P15364,&P5208,&P17470,&P18092 }};	/*Useragent - BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 40155; Lang: zhcn) ucweb-squid*/
const Device UA2750263 = {S67780, 108, { &P15364,&P5208,&P17470,&P18092 }};	/*Useragent - BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: Xe; Lang: hi; Profile/MIDP-2.0_Configuration/CLDC-1.1) ucweb-squid*/
const Device UA2812622 = {S67781, 71, { &P15364,&P5208,&P17470,&P18092 }};	/*Useragent - BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 2520; Lang: zhcn) ucweb-squid*/
const Device UA2889074 = {S67782, 72, { &P15364,&P5208,&P17470,&P18092 }};	/*Useragent - BREW-Applet/0xX (BREW/3.1.5.20; DeviceId: 40175; Lang: zhcn) ucweb-squid*/
const Device UA484446 = {S67783, 86, { &P17595,&P18120,&P19772,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US; rv:1.8.1.13) Gecko/X SeaMonkey/1.1.9*/
const Device UA513156 = {S67784, 92, { &P17595,&P18120,&P19772,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.8.1.21) Gecko/X SeaMonkey/1.1.15*/
const Device UA723563 = {S67785, 134, { &P21113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; XT532 Build/V4.37B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023282 = {S67786, 134, { &P21113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT532 Build/V4.36A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029868 = {S67787, 134, { &P21113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; XT532 Build/V4.36C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2317174 = {S67788, 134, { &P21113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; XT532 Build/V4.37C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2528387 = {S67789, 134, { &P21113,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT532 Build/V4.36D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2404690 = {S67790, 139, { &P19849,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7560M Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2424490 = {S67791, 131, { &P20171,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MN-1000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2538344 = {S67792, 132, { &P20171,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; MNB1000S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2566949 = {S67793, 139, { &P17172,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Transformer TF101 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2600923 = {S67794, 134, { &P17172,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Transformer Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2601582 = {S67795, 133, { &P17172,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Transformer Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2861939 = {S67796, 142, { &P17172,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Transformer TF101 Build/JB_MR1.2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2581584 = {S67797, 132, { &P22662,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Enjoy 71 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2657625 = {S67798, 136, { &P24568,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; N9810 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3119396 = {S67799, 58, { &P24568,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; N9810 Build/JZO54K)*/
const Device UA2674423 = {S67800, 150, { &P18317,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300TG Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676576 = {S67801, 149, { &P18317,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300T Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2707911 = {S67802, 150, { &P18317,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF300TL Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2754806 = {S67803, 139, { &P18592,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One VX Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3083098 = {S67804, 42, { &P17595,&P23096,&P18535,&P18092 }};	/*Useragent - Reeder/3.2 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA2947558 = {S67805, 168, { &P15201,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2966604 = {S67806, 165, { &P15201,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2959658 = {S67807, 131, { &P24133,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; TECNO_N9 Build/TECNO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3124822 = {S67808, 60, { &P24133,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; TECNO_N9 Build/TECNO)*/
const Device UA3067592 = {S67809, 127, { &P24419,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ME302C Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA186889 = {S67810, 134, { &P15364,&P17285,&P22448,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Iron/10.0.650.0 Chrome/10.0.650.0 Safari/534.16*/
const Device UA459145 = {S67811, 136, { &P18314,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MEIZU MX Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA459148 = {S67812, 124, { &P18314,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MEIZU MX Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA459150 = {S67813, 134, { &P18314,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-; MEIZU MX Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA459157 = {S67814, 60, { &P18314,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; MEIZU MX Build/GRJ90)*/
const Device UA1955940 = {S67815, 129, { &P18314,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MEIZU MX Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2235996 = {S67816, 111, { &P17595,&P18207,&P22890,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) iCab/5.0 Safari/533.16*/
const Device UA2195633 = {S67817, 71, { &P17595,&P18207,&P21575,&P18092 }};	/*Useragent - Mac OS X/10.8.2 (12C60); ExchangeWebServices/3.0 (157); Mail/6.2 (1499)*/
const Device UA2367343 = {S67818, 73, { &P17595,&P18207,&P21575,&P18092 }};	/*Useragent - Mac OS X/10.8.2 (12C3006); ExchangeWebServices/3.0 (157); Mail/6.2 (1499)*/
const Device UA2421411 = {S67819, 73, { &P17595,&P18207,&P21575,&P18092 }};	/*Useragent - Mac OS X/10.8.2 (12C2034); ExchangeWebServices/3.0 (157); Mail/6.2 (1499)*/
const Device UA2425540 = {S67820, 73, { &P17595,&P18207,&P21575,&P18092 }};	/*Useragent - Mac OS X/10.8.2 (12C3012); ExchangeWebServices/3.0 (157); Mail/6.2 (1499)*/
const Device UA2485394 = {S67821, 73, { &P17595,&P18207,&P21575,&P18092 }};	/*Useragent - Mac OS X/10.8.2 (12C3103); ExchangeWebServices/3.0 (157); Mail/6.2 (1499)*/
const Device UA2567377 = {S67822, 137, { &P20705,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602332 = {S67823, 130, { &P20705,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2795269 = {S67824, 142, { &P20705,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One 801e Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2673616 = {S67825, 132, { &P17172,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Transformer Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676694 = {S67826, 138, { &P17172,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Transformer TF101 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2709236 = {S67827, 133, { &P17172,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Transformer Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2975531 = {S67828, 130, { &P20968,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Le Pan S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3081280 = {S67829, 141, { &P18504,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LS840 Build/ZVK.IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA1872 = {S67830, 105, { &P11642,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonW705a/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3*/
const Device UA50519 = {S67831, 150, { &P13242,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; A853 Build/SHLA_U2_03.05.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA67328 = {S67832, 150, { &P13242,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A853 Build/SHLA_U2_03.06.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA67427 = {S67833, 151, { &P13242,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A853 Build/SUSKT_U1_00.10.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA67536 = {S67834, 150, { &P13242,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; A853 Build/SHLA_U2_03.06.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA504802 = {S67835, 72, { &P13242,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; A853 Build/SHLA_U2_03.05.0)*/
const Device UA1862884 = {S67836, 151, { &P13242,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-kr; A853 Build/SUSKT_U1_00.14.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA701205 = {S67837, 145, { &P21221,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; sr-rs; Telenor_OneTouch Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA754214 = {S67838, 46, { &P20204,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-AN270/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA2385684 = {S67839, 149, { &P21474,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.9; en-us; Celkon A 85 Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2435166 = {S67840, 157, { &P20520,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; MEDION Smartphone LIFE E3501 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2460782 = {S67841, 156, { &P20520,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; MEDION Smartphone LIFE E3501 Build/GRJ34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2531699 = {S67842, 91, { &P14454,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaE66) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2558293 = {S67843, 147, { &P17174,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; MediaPad 10 FHD Build/HuaweiMediaPad) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2896604 = {S67844, 158, { &P15989,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.1.8) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2899128 = {S67845, 158, { &P15989,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.0.16) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2901208 = {S67846, 158, { &P15989,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.4.27) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2944039 = {S67847, 158, { &P15989,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.6.8) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2944994 = {S67848, 158, { &P15989,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.4.22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2963260 = {S67849, 158, { &P15989,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.6.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3130734 = {S67850, 137, { &P18595,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131530 = {S67851, 137, { &P18595,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I8190 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA2424004 = {S67852, 69, { &P14433,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GT-I9003; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2456262 = {S67853, 69, { &P14433,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.1; zh-cn; GT-I9003; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA1998273 = {S67854, 143, { &P21700,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; Penta IS701C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033335 = {S67855, 148, { &P23982,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; Etisalat Smartphone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2705239 = {S67856, 156, { &P16436,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I8150 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752329 = {S67857, 157, { &P16436,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I8150 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2868904 = {S67858, 165, { &P16436,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I8150 Build/Touchwi5; CyanogenMod-Touchwi5v2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3081221 = {S67859, 135, { &P22597,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SCH-I545 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA16212 = {S67860, 46, { &P13999,&P5690,&P4626,&P18092 }};	/*Useragent - HD2_T8585 Opera/9.7 (Windows NT 5.1; U; zh-tw)*/
const Device UA62066 = {S67861, 43, { &P13999,&P5690,&P4626,&P18092 }};	/*Useragent - HD2_T8585 Opera/9.7 (Windows NT 5.1; U; en)*/
const Device UA251913 = {S67862, 135, { &P15575,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-I9100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA571306 = {S67863, 141, { &P15575,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/MIUI 1.9.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1901650 = {S67864, 141, { &P15575,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/MIUI 1.9.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1939895 = {S67865, 59, { &P15575,&P24694,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; GT-I9100 Build/MIUI)*/
const Device UA2019170 = {S67866, 142, { &P15575,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/MIUI 1.7.29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020123 = {S67867, 142, { &P15575,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/MIUI 1.9.16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021157 = {S67868, 142, { &P15575,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/MIUI 1.8.26) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021558 = {S67869, 123, { &P15575,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-I9100 Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA795469 = {S67870, 85, { &P15364,&P17285,&P19772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.8.1.21) Gecko/X SeaMonkey/1.1.15*/
const Device UA2032025 = {S67871, 139, { &P14225,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Desire HD Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2237767 = {S67872, 139, { &P14225,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Desire HD Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2422340 = {S67873, 141, { &P12681,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus S 4G Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2564372 = {S67874, 140, { &P12681,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus S 4G Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2599293 = {S67875, 141, { &P12681,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus S 4G Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2738168 = {S67876, 141, { &P12681,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus S 4G Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2672574 = {S67877, 146, { &P23152,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; MegaFon SP-AI Build/1.2.10.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2712777 = {S67878, 146, { &P23152,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; MegaFon_SP-AI Build/1.2.10.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA30992 = {S67879, 86, { &P12390,&P5208,&P4223,&P18092 }};	/*Useragent - Mozilla/5.0 (rv:1.3; BREW 3.1.5; en)/400x240 sam-r860 like Gecko/20080531 NetFront/3.5*/
const Device UA63605 = {S67880, 129, { &P17618,&P6590,&P4223,&P18092 }};	/*Useragent - NetFront/3.5.1 (BREW 5.0.2.1; U; en-us; Samsung ; NetFront/3.5.1/WAP) Sprint M360 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA85717 = {S67881, 27, { &P17301,&P17017,&P17470,&P18092 }};	/*Useragent - portalmmm/2.0 S500i(c20;TB)*/
const Device UA605572 = {S67882, 194, { &P15364,&P7133,&P24667,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/28/351/UCWEB*/
const Device UA606294 = {S67883, 194, { &P15364,&P7133,&P24667,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/50/351/UCWEB*/
const Device UA606963 = {S67884, 194, { &P15364,&P7133,&P24667,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/50/352/UCWEB*/
const Device UA607572 = {S67885, 194, { &P15364,&P7133,&P24667,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/28/352/UCWEB*/
const Device UA671673 = {S67886, 194, { &P15364,&P7133,&P24667,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/28/355/UCWEB*/
const Device UA816778 = {S67887, 194, { &P15364,&P7133,&P24667,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/50/355/UCWEB*/
const Device UA817558 = {S67888, 194, { &P15364,&P7133,&P24667,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/50/444/UCWEB*/
const Device UA1966536 = {S67889, 194, { &P15364,&P7133,&P24667,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/50/405/UCWEB*/
const Device UA1976757 = {S67890, 194, { &P15364,&P7133,&P24667,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/28/444/UCWEB*/
const Device UA2160326 = {S67891, 194, { &P15364,&P7133,&P24667,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/28/354/UCWEB*/
const Device UA2165664 = {S67892, 194, { &P15364,&P7133,&P24667,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/50/354/UCWEB*/
const Device UA2332603 = {S67893, 194, { &P15364,&P7133,&P24667,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.2.0.132/28/405/UCWEB*/
const Device UA2404600 = {S67894, 163, { &P13337,&P17485,&P24673,&P18092 }};	/*Useragent - SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonj108i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2509667 = {S67895, 70, { &P17346,&P19826,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10B144 (iPod; U; CPU OS 6_1 like Mac OS X; en_us)*/
const Device UA2532864 = {S67896, 72, { &P17346,&P19826,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10B146 (iPod; U; CPU OS 6_1_2 like Mac OS X; en_us)*/
const Device UA2585647 = {S67897, 72, { &P17346,&P19826,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10B329 (iPod; U; CPU OS 6_1_3 like Mac OS X; en_us)*/
const Device UA2774287 = {S67898, 127, { &P23005,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TBDC1093 Build/X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2879030 = {S67899, 71, { &P15364,&P7133,&P22170,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:22.0) Gecko/X Firefox/22.0 slurp*/
const Device UA2942335 = {S67900, 64, { &P15364,&P7133,&P22170,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:22.0) Gecko/X Firefox/22.0 slurp*/
const Device UA2943724 = {S67901, 77, { &P15364,&P7133,&P22170,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:22.0) Gecko/X Firefox/22.0 slurp slurp*/
const Device UA2976117 = {S67902, 127, { &P18680,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KFJWI Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3068389 = {S67903, 133, { &P19775,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; A210 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA1688 = {S67904, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XXHHa; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA1689 = {S67905, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/DJHHb; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA2235 = {S67906, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XXHH7; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA17277 = {S67907, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/DJHJ4; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA17279 = {S67908, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XEHI5; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA17278 = {S67909, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/OXXHG3 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA17280 = {S67910, 147, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA18750 = {S67911, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/DJHI4; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA18751 = {S67912, 152, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XXHH6 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA18752 = {S67913, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XXHJ4; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA18753 = {S67914, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XEHJ2; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA18754 = {S67915, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XXHH9; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA18755 = {S67916, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/DJHJ1; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA18756 = {S67917, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XXHK1; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA18757 = {S67918, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/CBHK1; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA18759 = {S67919, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XXHG5; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA18760 = {S67920, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/DJHK2; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA18761 = {S67921, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XEHJ4; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA18762 = {S67922, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XXHL2; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA18763 = {S67923, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XEHL2; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA18764 = {S67924, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XXHJ5; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA18758 = {S67925, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/DJHI3; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA37421 = {S67926, 155, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 SAMSUNG-GT-I8510C/1.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA39195 = {S67927, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XEHI2; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA44848 = {S67928, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/DJHI1; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA50735 = {S67929, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510L/UBHL3 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA54990 = {S67930, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/DJHI2; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA62397 = {S67931, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/DJHL1; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA62815 = {S67932, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/XXHJ3; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA62881 = {S67933, 153, { &P15200,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510/DJHK4; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA263546 = {S67934, 62, { &P15364,&P5690,&P20411,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.2.15 Version/10.20*/
const Device UA2369194 = {S67935, 41, { &P15364,&P5690,&P20411,&P18092 }};	/*Useragent - Opera/10.2 (Windows NT 5.1; WOW64; U; en)*/
const Device UA2507025 = {S67936, 167, { &P18635,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 4 Build/JOP40D; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583045 = {S67937, 166, { &P18635,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3021327 = {S67938, 171, { &P18635,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/mako*/
const Device UA2567122 = {S67939, 140, { &P18452,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT29i Build/7.0.A.3.212) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2889313 = {S67940, 140, { &P24732,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-S735C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA578329 = {S67941, 139, { &P20724,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; ld-us; AN8G2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028292 = {S67942, 141, { &P20724,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; fr-fr; AN8G2DT Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA904926 = {S67943, 82, { &P17595,&P17295,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.10.289 Version/12.00*/
const Device UA2008900 = {S67944, 82, { &P17595,&P17295,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.10.289 Version/12.01*/
const Device UA2531751 = {S67945, 135, { &P21472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A111 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566959 = {S67946, 135, { &P17160,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Slider SL101 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2735977 = {S67947, 137, { &P22095,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ThL W7 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088334 = {S67948, 59, { &P22095,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; ThL W7 Build/JOP40D)*/
const Device UA2738256 = {S67949, 143, { &P23017,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Spice Mi-502 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2974045 = {S67950, 134, { &P17211,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 101G9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3020944 = {S67951, 131, { &P17211,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 101G9 Build/MR1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2976012 = {S67952, 137, { &P14877,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021056 = {S67953, 137, { &P14877,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9000 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA10241 = {S67954, 62, { &P13484,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2320c/2.0 (08.20) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA20350 = {S67955, 64, { &P13484,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2320c-2/2.0 (08.20) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA52437 = {S67956, 67, { &P13484,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2320c_CMCC/2.0 (09.73) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA62684 = {S67957, 68, { &P13484,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2320c-2b/2.0(ATT.11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA30490 = {S67958, 98, { &P13329,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-U800/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA32905 = {S67959, 74, { &P13329,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-U800E/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53446 = {S67960, 73, { &P13976,&P5208,&P3499,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Teleca Q7; Brew 3.1.5; U; en) 240X400 LGE VX9700*/
const Device UA342299 = {S67961, 70, { &P15649,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2.1;Zh_cn;GT-S5570;240*320;)UCWEB7.8.0.95/140/444*/
const Device UA2021123 = {S67962, 119, { &P15649,&P5096,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.2.1;Zh_cn;GT-S5570;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/140/444*/
const Device UA397331 = {S67963, 164, { &P15134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; LG-C710h Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA715136 = {S67964, 138, { &P15134,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; LG-C710h Build/GRI40) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA399150 = {S67965, 111, { &P15795,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 710; T-Mobile)*/
const Device UA399174 = {S67966, 101, { &P15795,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 710)*/
const Device UA925808 = {S67967, 111, { &P15795,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 710; Vodafone)*/
const Device UA925812 = {S67968, 109, { &P15795,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 710; Orange)*/
const Device UA1967995 = {S67969, 127, { &P23823,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033228 = {S67970, 127, { &P23823,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2487257 = {S67971, 141, { &P18662,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC J Z321e Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2754796 = {S67972, 137, { &P22595,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919V Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA347147 = {S67973, 113, { &P15364,&P17285,&P22453,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.2 Chrome/15.0.900.2 Safari/535.2*/
const Device UA682719 = {S67974, 120, { &P15364,&P17285,&P22453,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.2 Chrome/15.0.900.2 Safari/535.2*/
const Device UA674205 = {S67975, 131, { &P19937,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; A73 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA679800 = {S67976, 140, { &P19937,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Micromax A73 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2041821 = {S67977, 100, { &P15364,&P18346,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2427584 = {S67978, 138, { &P19854,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R760 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2571511 = {S67979, 91, { &P17595,&P17297,&P21791,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:19.0) Gecko/X Firefox/19.0 SeaMonkey/2.16.2*/
const Device UA2584770 = {S67980, 156, { &P18283,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9300 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2604437 = {S67981, 156, { &P18283,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-I9300 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2861199 = {S67982, 156, { &P18283,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-I939 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2943539 = {S67983, 152, { &P18583,&P5043,&P24678,&P18092 }};	/*Useragent - Nokia110/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia110) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA578601 = {S67984, 79, { &P15364,&P18118,&P17698,&P18092 }};	/*Useragent - User-Agent: Mozilla/5.0 (Ubuntu; X11; Linux x86_64; rv:8.0) Gecko/X Firefox/8.0*/
const Device UA671137 = {S67985, 77, { &P15364,&P18118,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:8.0) Gecko/X Firefox/8.0 Firefox/8.0*/
const Device UA715085 = {S67986, 102, { &P15364,&P18118,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:8.0) Gecko/X Firefox/8.0,gzip(gfe) (via translate.google.com)*/
const Device UA741481 = {S67987, 104, { &P15364,&P18118,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Ubuntu; X11; Linux x86_64; rv:8.0) Gecko/X Firefox/8.0,gzip(gfe) (via translate.google.com)*/
const Device UA1904892 = {S67988, 77, { &P15364,&P18118,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:8.0) Gecko/X Firefox/8.0 FirePHP/0.6*/
const Device UA1907480 = {S67989, 79, { &P15364,&P18118,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Ubuntu; X11; Linux x86_64; rv:8.0) Gecko/X Firefox/8.0 FirePHP/0.6*/
const Device UA1924618 = {S67990, 79, { &P15364,&P18118,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Ubuntu; X11; Linux x86_64; rv:8.0) Gecko/X Firefox/8.0 Firefox/8.0*/
const Device UA2307235 = {S67991, 67, { &P15364,&P18118,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:8.0) Gecko/X Firefox/8.0*/
const Device UA2402795 = {S67992, 75, { &P15364,&P18118,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Ubuntu; X11; Linux i686; rv:8.0) Gecko/X Firefox/8.0,gzip(gfe)*/
const Device UA616617 = {S67993, 96, { &P18182,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; Microsoft; 0)*/
const Device UA616623 = {S67994, 97, { &P18182,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; Juggernaut; 0)*/
const Device UA925830 = {S67995, 91, { &P18182,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; NOKIA; )*/
const Device UA2034645 = {S67996, 82, { &P18182,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0)*/
const Device UA2034647 = {S67997, 108, { &P18182,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia)*/
const Device UA909301 = {S67998, 62, { &P15364,&P5690,&P18515,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:14.0) Gecko/X Thunderbird/14.0*/
const Device UA1996192 = {S67999, 69, { &P15364,&P5690,&P18515,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:14.0) Gecko/X Thunderbird/14.0*/
const Device UA2557471 = {S68000, 65, { &P15364,&P18110,&P21943,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:10.0.12) Gecko/X Icedove/10.0.12*/
const Device UA2558117 = {S68001, 83, { &P15364,&P18110,&P21943,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:10.0.12) Gecko/X Icedove/10.0.12 Lightning/1.2b1*/
const Device UA2257627 = {S68002, 129, { &P20939,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; T-07B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2368606 = {S68003, 142, { &P12280,&P18432,&P20789,&P18092 }};	/*Useragent - MQQBrowser/38 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3*/
const Device UA2389490 = {S68004, 141, { &P12280,&P18432,&P20789,&P18092 }};	/*Useragent - MQQBrowser/38 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3*/
const Device UA2407369 = {S68005, 143, { &P12280,&P18432,&P20789,&P18092 }};	/*Useragent - MQQBrowser/381 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3*/
const Device UA2424223 = {S68006, 141, { &P12280,&P18432,&P20789,&P18092 }};	/*Useragent - MQQBrowser/381 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3*/
const Device UA2425159 = {S68007, 142, { &P12280,&P18432,&P20789,&P18092 }};	/*Useragent - MQQBrowser/381 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3*/
const Device UA2482867 = {S68008, 139, { &P12280,&P18432,&P20789,&P18092 }};	/*Useragent - MQQBrowser/38 Mozilla/5.0 (iPhone 4; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3*/
const Device UA2700433 = {S68009, 64, { &P15364,&P5448,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; PPC; Tablet PC; Trident/4.0)*/
const Device UA2792636 = {S68010, 59, { &P15364,&P5448,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; WindowsCE; Trident/5.0)*/
const Device UA2850082 = {S68011, 64, { &P15364,&P5448,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; 68K; Tablet PC; Trident/4.0)*/
const Device UA2753446 = {S68012, 137, { &P18304,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9070 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2797325 = {S68013, 138, { &P18304,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9070P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3083418 = {S68014, 141, { &P17346,&P18117,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9B206 Safari/7534.48.3*/
const Device UA3126967 = {S68015, 139, { &P17346,&P18117,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9B176 Safari/7534.48.3*/
const Device UA3096288 = {S68016, 156, { &P21648,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA33331 = {S68017, 116, { &P12899,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-C3011/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA1906301 = {S68018, 75, { &P12899,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-C3011/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA1960048 = {S68019, 71, { &P15649,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;GT-S5570I;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA1970692 = {S68020, 70, { &P15649,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;GT-S5570;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA2021088 = {S68021, 70, { &P15649,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.4;Zh_cn;GT-S5570;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA2021253 = {S68022, 70, { &P15649,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.5;Zh_cn;GT-S5570;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA2195944 = {S68023, 80, { &P15364,&P18110,&P20755,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:15.0) Gecko/X Firefox/15.0.1 SeaMonkey/2.12.1*/
const Device UA2199846 = {S68024, 73, { &P17595,&P19835,&P18537,&P18092 }};	/*Useragent - Reeder/1010.79.00 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBook5%2C1)*/
const Device UA2199849 = {S68025, 71, { &P17595,&P19835,&P18537,&P18092 }};	/*Useragent - Reeder/1010.79.00 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (iMac12%2C2)*/
const Device UA2201364 = {S68026, 76, { &P17595,&P19835,&P18537,&P18092 }};	/*Useragent - Reeder/1010.79.00 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookPro8%2C2)*/
const Device UA2202475 = {S68027, 76, { &P17595,&P19835,&P18537,&P18092 }};	/*Useragent - Reeder/1010.79.00 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookAir5%2C2)*/
const Device UA2203391 = {S68028, 136, { &P12280,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/6533.18.5 DiigoBrowser*/
const Device UA2222936 = {S68029, 148, { &P12280,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; en_us) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5*/
const Device UA2261063 = {S68030, 166, { &P12280,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/6533.18.5 DiigoBrowser Safari/6533.18.5 DiigoBrowser*/
const Device UA2352128 = {S68031, 137, { &P12280,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10A403 Safari/6533.18.5*/
const Device UA2403886 = {S68032, 150, { &P12280,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; ko_kr) AppleWebKit/6536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5*/
const Device UA2454991 = {S68033, 133, { &P12280,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; 6536.26) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5*/
const Device UA2466897 = {S68034, 139, { &P12280,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.0.2 Mobile/10A523 Safari/6533.18.5*/
const Device UA2841647 = {S68035, 135, { &P12280,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; 6536.26) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5*/
const Device UA2866688 = {S68036, 170, { &P12280,&P18432,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5*/
const Device UA2459465 = {S68037, 140, { &P20180,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XOLO_A800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2702057 = {S68038, 134, { &P20498,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; PMP7100D3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2985150 = {S68039, 63, { &P20498,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; PMP7100D3G Build/JRO03H)*/
const Device UA2670908 = {S68040, 139, { &P18452,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT29i Build/9.1.B.0.411) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2736635 = {S68041, 132, { &P18452,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT29i Build/9.1.B.0.411) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2698004 = {S68042, 138, { &P18636,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; VS930 4G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2956008 = {S68043, 60, { &P21792,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9200 Build/JDQ39)*/
const Device UA2998121 = {S68044, 131, { &P21792,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9200 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3068417 = {S68045, 140, { &P18303,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT27i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA14049 = {S68046, 64, { &P11785,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia7100s-2/1.0 (06.31) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63309 = {S68047, 62, { &P11785,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia7100s/1.0 (06.32) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA451556 = {S68048, 132, { &P17595,&P17295,&P24745,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Stainless/0.8 like Version/5.1 Safari/534.48.3*/
const Device UA585067 = {S68049, 143, { &P21200,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; my Pad P3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1994891 = {S68050, 142, { &P21200,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; myPad P3 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA589274 = {S68051, 68, { &P17968,&P17979,&P18566,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:13.0a1) Gecko/13.0a1 Firefox/13.0a1*/
const Device UA614808 = {S68052, 74, { &P17968,&P17979,&P18566,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:13.0) Gecko/13.0 Firefox/13.0 Fennec/13.0*/
const Device UA744849 = {S68053, 64, { &P17968,&P17979,&P18566,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:13.0) Gecko/13.0 Firefox/13.0a2*/
const Device UA821316 = {S68054, 64, { &P17968,&P17979,&P18566,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:13.0) Gecko/13.0 Firefox/13.0a1*/
const Device UA898025 = {S68055, 62, { &P17968,&P17979,&P18566,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:13.0) Gecko/13.0 Firefox/13.0*/
const Device UA653872 = {S68056, 112, { &P19757,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; Velocitymicro/T408) AppleWebKit/530.17(KHTML, like Gecko) Version/4.0Safari/530.17*/
const Device UA2459264 = {S68057, 140, { &P21650,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; POV_TAB-PROTAB26 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2699032 = {S68058, 142, { &P21650,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; POV_TAB-PROTAB26XL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2984869 = {S68059, 146, { &P21650,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; POV_TAB-PROTAB26-IPS10 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2461748 = {S68060, 132, { &P20257,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMP5597D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2570638 = {S68061, 131, { &P21820,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TBD753B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2987971 = {S68062, 60, { &P21820,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; TBD753B Build/JRO03C)*/
const Device UA2753012 = {S68063, 126, { &P12765,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2784887 = {S68064, 151, { &P18442,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2871099 = {S68065, 151, { &P18442,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2873145 = {S68066, 151, { &P18442,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-N719 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2999601 = {S68067, 151, { &P18442,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; SCH-N719 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3000424 = {S68068, 151, { &P18442,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7105 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2975542 = {S68069, 136, { &P22613,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E430 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3067588 = {S68070, 137, { &P22609,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6312 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065572 = {S68071, 138, { &P19849,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7560M Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2603917 = {S68072, 127, { &P15364,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MiSS Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2705789 = {S68073, 135, { &P15364,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A9600 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2707101 = {S68074, 128, { &P15364,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; y7053 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2855405 = {S68075, 137, { &P15364,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ThinkPadTablet Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2863389 = {S68076, 132, { &P15364,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1; 9300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2866275 = {S68077, 137, { &P15364,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Tablet PC 3000 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2935327 = {S68078, 131, { &P15364,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MID-709S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2990102 = {S68079, 134, { &P15364,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; eZee Tab973 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2994120 = {S68080, 130, { &P15364,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; JPAD730 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2998762 = {S68081, 121, { &P15364,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M6 Build/M6) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA3029901 = {S68082, 133, { &P15364,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PMP7380D3G Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA3077269 = {S68083, 146, { &P15364,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Casper_VIA_A6108 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA3108616 = {S68084, 162, { &P15364,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MTC 970H Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.20.1364.172 Mobile Safari/537.22*/
const Device UA3121102 = {S68085, 145, { &P15364,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Walton Primo H1 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA158360 = {S68086, 71, { &P15767,&P17145,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8C148 (iPad; U; CPU OS 4_2_1 like Mac OS X; en_us)*/
const Device UA519776 = {S68087, 69, { &P15767,&P17145,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8C134 (iPad; U; CPU OS 4_2 like Mac OS X; en_us)*/
const Device UA2847224 = {S68088, 165, { &P15767,&P17145,&P18215,&P18092 }};	/*Useragent - Mozilla/5.0 AppleCoreMedia/1.0.0.8J2 (iPad; U; CPU OS 4_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA1961158 = {S68089, 138, { &P18316,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-F160L Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1963990 = {S68090, 138, { &P18316,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-F160K Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2185590 = {S68091, 138, { &P18316,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F160L Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261473 = {S68092, 138, { &P18316,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-F160S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2380553 = {S68093, 139, { &P18316,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F160LV Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2521780 = {S68094, 137, { &P18316,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-F160 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2458153 = {S68095, 153, { &P23731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; O+_8.75_Android Build/OplusOplus_A100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2570784 = {S68096, 153, { &P23731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; O+ 8.75 Android Build/OplusOplus_A100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2507038 = {S68097, 142, { &P20289,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; HTC6435LVW 4G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2507710 = {S68098, 139, { &P20289,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; HTC6435LVW Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2891463 = {S68099, 140, { &P20289,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC6435LVW Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2995676 = {S68100, 143, { &P20289,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC6435LVW 4G Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2531402 = {S68101, 164, { &P13337,&P17485,&P24676,&P18092 }};	/*Useragent - SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj108i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2715880 = {S68102, 137, { &P15364,&P17820,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu/12.04 Chromium/27.0.1453.6 Chrome/27.0.1453.6 Safari/537.36*/
const Device UA2715993 = {S68103, 135, { &P15364,&P17820,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu/12.04 Chromium/27.0.1453.6 Chrome/27.0.1453.6 Safari/537.36*/
const Device UA2734861 = {S68104, 175, { &P18645,&P18111,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; SPH-L710Android 4.2.2; en-US; SCH-R530U Build Build/JRO03L) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3132862 = {S68105, 130, { &P22606,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3067488 = {S68106, 136, { &P24655,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; IdeaTab A3000-H Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3083490 = {S68107, 136, { &P24655,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; IdeaTab A3000-F Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3083254 = {S68108, 129, { &P22606,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N5110 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA64691 = {S68109, 105, { &P11761,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonW995a/R1HA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.5*/
const Device UA1964997 = {S68110, 124, { &P17595,&P18120,&P18208,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.2.2.7373 Safari/532.4*/
const Device UA654777 = {S68111, 98, { &P20276,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Ultimate)*/
const Device UA743698 = {S68112, 126, { &P21138,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; V55 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA815597 = {S68113, 125, { &P21138,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; V55 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2031183 = {S68114, 55, { &P21138,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; V55 Build/HTK75)*/
const Device UA1966994 = {S68115, 137, { &P21773,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; SP704C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2485507 = {S68116, 138, { &P21773,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; SP704CE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2559971 = {S68117, 136, { &P21823,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; HTX21 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2765716 = {S68118, 58, { &P21823,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTX21 Build/JRO03C)*/
const Device UA2567217 = {S68119, 145, { &P13064,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I727 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2584846 = {S68120, 137, { &P13064,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I727 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2794982 = {S68121, 138, { &P23163,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; SCH-I959 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867139 = {S68122, 129, { &P23163,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; SCH-I959 Build/JDQ39) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2958338 = {S68123, 60, { &P23163,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SCH-I959 Build/JDQ39)*/
const Device UA2947710 = {S68124, 82, { &P16082,&P5567,&P18525,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/6.0.3/27.2338; U; en) Presto/2.8.119 320X240 LG VN530*/
const Device UA62678 = {S68125, 73, { &P11631,&P5027,&P3412,&P18092 }};	/*Useragent - BlackBerry8110/4.3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102*/
const Device UA16585 = {S68126, 81, { &P14806,&P5448,&P22271,&P18092 }};	/*Useragent - SAMSUNG-SGH-I617/1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA210704 = {S68127, 139, { &P13503,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Desire Z Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA318523 = {S68128, 139, { &P13503,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Desire Z Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA437138 = {S68129, 139, { &P13503,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Desire Z Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1940029 = {S68130, 63, { &P13503,&P24694,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; HTC Desire Z Build/MIUI)*/
const Device UA2021178 = {S68131, 127, { &P13503,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Desire Z Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2519099 = {S68132, 132, { &P13503,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Desire Z Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA603199 = {S68133, 110, { &P15364,&P6596,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2046.13.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.9 Safari/536.5*/
const Device UA605908 = {S68134, 111, { &P15364,&P6596,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2046.30.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.23 Safari/536.5*/
const Device UA615465 = {S68135, 111, { &P15364,&P6596,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2046.68.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.41 Safari/536.5*/
const Device UA818770 = {S68136, 111, { &P15364,&P6596,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2046.20.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.17 Safari/536.5*/
const Device UA897920 = {S68137, 110, { &P15364,&P6596,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2046.106.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5*/
const Device UA899654 = {S68138, 110, { &P15364,&P6596,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2046.107.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5*/
const Device UA901756 = {S68139, 110, { &P15364,&P6596,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 2046.137.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.57 Safari/536.5*/
const Device UA903910 = {S68140, 112, { &P15364,&P6596,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS x86_64 2046.137.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.57 Safari/536.5*/
const Device UA807170 = {S68141, 118, { &P15364,&P7133,&P19840,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.5.1 (development) Safari/534.34*/
const Device UA2974493 = {S68142, 130, { &P17323,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6800 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA9104 = {S68143, 64, { &P15158,&P5843,&P4030,&P18092 }};	/*Useragent - Nokia2710c-2/2.0 (06.13) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA21124 = {S68144, 62, { &P15158,&P5843,&P4030,&P18092 }};	/*Useragent - Nokia2710c/2.0 (06.21) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA69777 = {S68145, 77, { &P15158,&P5843,&P4030,&P18092 }};	/*Useragent - Nokia2710c-2/2.0 (06.21) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2710c-2*/
const Device UA26956 = {S68146, 104, { &P13654,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-A687/A687UCJH1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA458536 = {S68147, 91, { &P17595,&P18120,&P18199,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-GB; rv:1.8.1.18) Gecko/X Thunderbird/2.0.0.18*/
const Device UA903166 = {S68148, 141, { &P21036,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Spice_Mi-425 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2387530 = {S68149, 149, { &P20182,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xolo_X900 Build/AZ510_ICS_02.24I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2426339 = {S68150, 138, { &P18316,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2428423 = {S68151, 138, { &P18316,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2458127 = {S68152, 138, { &P18316,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2485358 = {S68153, 139, { &P18316,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160LV Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2567121 = {S68154, 137, { &P18240,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P700 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2944548 = {S68155, 161, { &P15575,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2945606 = {S68156, 161, { &P15575,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2962914 = {S68157, 161, { &P15575,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9108 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3013233 = {S68158, 161, { &P15575,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3013892 = {S68159, 162, { &P15575,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9100G Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2464 = {S68160, 49, { &P11977,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GW520-Bouygues Teleca/WAP2.0 MIDP-2.1/CLDC-1.1*/
const Device UA63270 = {S68161, 49, { &P11977,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GW520-H3G/V10f Teleca/WAP2.0 MIDP-2.1/CLDC-1.1*/
const Device UA63467 = {S68162, 40, { &P11977,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GW520 Teleca/WAP2.0 MIDP-2.1/CLDC-1.1*/
const Device UA64341 = {S68163, 52, { &P11977,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GW520-orange/V10d Teleca/WAP2.0 MIDP-2.1/CLDC-1.1*/
const Device UA63493 = {S68164, 104, { &P14305,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonC901/R1GA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.4*/
const Device UA23984 = {S68165, 13, { &P15631,&P17483,&P17660,&P18092 }};	/*Useragent - LG/U8138/v1.0*/
const Device UA49462 = {S68166, 93, { &P13033,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-Sydney-Orange/EFS-EE-ORG-P107A20V1.0.3 Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA70974 = {S68167, 93, { &P13033,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-Sydney-Orange/EFS-EE-ORG-P107A20V1.0.5 Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA712999 = {S68168, 58, { &P15364,&P17188,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0; rv:10.0) Gecko/X Firefox/10.0*/
const Device UA2430089 = {S68169, 155, { &P14714,&P5843,&P24673,&P18092 }};	/*Useragent - NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiax3-02) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2578775 = {S68170, 159, { &P14714,&P5843,&P24673,&P18092 }};	/*Useragent - NokiaX3-02.5/5.0 (06.75) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiax3-02.5) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2507934 = {S68171, 144, { &P22174,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; SlidePad704CE Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2869235 = {S68172, 136, { &P22174,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; 704CE Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2560806 = {S68173, 155, { &P20541,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAP4500TDUO Build/PrestigioPAP4500TDUO) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2800309 = {S68174, 153, { &P20541,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAP4500DUO Build/PrestigioPAP4500DUO) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2751113 = {S68175, 158, { &P21982,&P18066,&P18062,&P18092 }};	/*Useragent - Celkon A98 Linux/3.0.13 Android/4.0.4 Release/04.08.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2854276 = {S68176, 141, { &P21982,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Celkon A98 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2582862 = {S68177, 144, { &P21645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; BLU Tank4.5 Build/ JB_001_00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2855902 = {S68178, 139, { &P21645,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TANK 4.5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2675250 = {S68179, 130, { &P19870,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-L710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2677275 = {S68180, 137, { &P19870,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-L710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2732262 = {S68181, 111, { &P23409,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; HUAWEI; 4Afrika)*/
const Device UA2974302 = {S68182, 150, { &P19762,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI MT1-U06 Build/HuaweiMT1-U06) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973964 = {S68183, 139, { &P23855,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021167 = {S68184, 132, { &P23855,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA368944 = {S68185, 134, { &P18265,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-02D Build/S1110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA454465 = {S68186, 134, { &P18265,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-02D Build/S2201) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA603563 = {S68187, 134, { &P18265,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-02D Build/S3151) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA631024 = {S68188, 134, { &P18265,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-02D Build/SB180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1964694 = {S68189, 134, { &P18265,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-02D Build/S6130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033725 = {S68190, 58, { &P18265,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SH-02D Build/S6130)*/
const Device UA2605745 = {S68191, 134, { &P18265,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SH-02D Build/S1220) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2008904 = {S68192, 172, { &P15767,&P17141,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us; android4.0; SYTABA73C 11.2.7.8) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2029027 = {S68193, 170, { &P15767,&P17141,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us;Android4.0;SYMETA10 11.10.1.1) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2351381 = {S68194, 143, { &P22858,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; U8836D Build/HuaweiU8836D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2829947 = {S68195, 134, { &P22858,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U8836D Build/HuaweiU8836D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3042982 = {S68196, 137, { &P22858,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; U8836D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423265 = {S68197, 141, { &P20117,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Fly IQ440; Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2553522 = {S68198, 136, { &P20117,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; IQ440 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099291 = {S68199, 58, { &P20117,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; IQ440 Build/IMM76D)*/
const Device UA2672047 = {S68200, 158, { &P15575,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2718546 = {S68201, 156, { &P14711,&P5204,&P24677,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (11.20) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-01) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2767571 = {S68202, 144, { &P18283,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.1;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/T-Mobile;FBPN/com.facebook.katana;FBDV/SGH-T999;FBSV/4.1.1;]*/
const Device UA2780260 = {S68203, 144, { &P18283,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/2.3;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/T-Mobile;FBPN/com.facebook.katana;FBDV/SGH-T999;FBSV/4.1.1;]*/
const Device UA2854352 = {S68204, 146, { &P18283,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/2.2.1;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/T-Mobile;FBPN/com.facebook.katana;FBDV/SGH-T999;FBSV/4.1.1;]*/
const Device UA2869536 = {S68205, 146, { &P18283,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.2.1;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/T-Mobile;FBPN/com.facebook.katana;FBDV/SGH-T999;FBSV/4.1.2;]*/
const Device UA2940152 = {S68206, 144, { &P18283,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.3;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/T-Mobile;FBPN/com.facebook.katana;FBDV/SGH-T999;FBSV/4.1.2;]*/
const Device UA3027655 = {S68207, 144, { &P18283,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.5;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/T-Mobile;FBPN/com.facebook.katana;FBDV/SGH-T999;FBSV/4.1.2;]*/
const Device UA3057167 = {S68208, 144, { &P18283,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.4;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/T-Mobile;FBPN/com.facebook.katana;FBDV/SGH-T999;FBSV/4.1.2;]*/
const Device UA3080297 = {S68209, 179, { &P18283,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.6.1;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/SCH-I535;FBSV/4.1.2;FBCA/armeabi-v7a:armeabi;]*/
const Device UA3102691 = {S68210, 144, { &P18283,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/2.3;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/T-Mobile;FBPN/com.facebook.katana;FBDV/SGH-T999;FBSV/4.1.2;]*/
const Device UA3109945 = {S68211, 171, { &P18283,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.6.1;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/T-Mobile;FBPN/com.facebook.katana;FBDV/SGH-T999;FBSV/4.1.2;FBCA/armeabi-v7a:armeabi;]*/
const Device UA2755271 = {S68212, 129, { &P22922,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; M470BSA Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2770079 = {S68213, 143, { &P22953,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F220K/10c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2864845 = {S68214, 139, { &P22953,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F220K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2870118 = {S68215, 143, { &P22953,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F220K/10d Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102156 = {S68216, 61, { &P22953,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-F220K Build/JZO54K)*/
const Device UA2974663 = {S68217, 130, { &P22873,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5120 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2961579 = {S68218, 103, { &P12449,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae5-00) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3059919 = {S68219, 94, { &P12449,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE5-00) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3126383 = {S68220, 96, { &P12449,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE5-00.2) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA55132 = {S68221, 100, { &P11694,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-G800/JBGL1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA167132 = {S68222, 132, { &P18000,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A70H Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA657611 = {S68223, 56, { &P18000,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; A70H Build/FROYO)*/
const Device UA758765 = {S68224, 133, { &P17103,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PTM1005HN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1993478 = {S68225, 133, { &P17103,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; STM1007HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2553450 = {S68226, 133, { &P17103,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-pt; MTM1005HN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2952737 = {S68227, 133, { &P17103,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PTM1005NT Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2041278 = {S68228, 64, { &P15364,&P18145,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64; rv:15.0) Gecko/X Firefox/15.0.1*/
const Device UA2431983 = {S68229, 80, { &P18088,&P17017,&P24514,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/9/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2874172 = {S68230, 80, { &P18088,&P17017,&P24514,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/9/30.3214; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2408056 = {S68231, 91, { &P17595,&P18207,&P20457,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:17.0) Gecko/X Firefox/17.0 SeaMonkey/2.14.1*/
const Device UA2675763 = {S68232, 138, { &P18512,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT881 Build/9.8.3O-118_USC-14) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676097 = {S68233, 145, { &P18512,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT881 Build/9.8.3O-118_USC-14) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2753850 = {S68234, 126, { &P18248,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A700 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2976094 = {S68235, 137, { &P19861,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I605 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2956883 = {S68236, 129, { &P24365,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SM-T211 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3068221 = {S68237, 163, { &P18250,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Motorola RAZR MAXX Build/6.7.2-180_DHD-16_NA-27) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3131095 = {S68238, 138, { &P18295,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131080 = {S68239, 138, { &P18280,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132923 = {S68240, 138, { &P18280,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC VLE_U Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA692024 = {S68241, 116, { &P14714,&P5843,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.8*/
const Device UA2398472 = {S68242, 115, { &P14714,&P5843,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02/07.51; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.11*/
const Device UA40082 = {S68243, 136, { &P13413,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; de-de; BASE lutea Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA53548 = {S68244, 138, { &P13413,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; BASE lutea Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76458 = {S68245, 149, { &P13413,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; pt-pt; Optimus San Francisco Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA373413 = {S68246, 146, { &P13413,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; Orange San Francisco Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1937011 = {S68247, 148, { &P13413,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ro-ro; Orange San Francisco Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA44179 = {S68248, 56, { &P15364,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-24.1_/00.62 UP.Browser/6.2.3.4.c.1.120 (GUI) MMP/2.0*/
const Device UA55267 = {S68249, 56, { &P15364,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-COOL0/00.62 UP.Browser/6.2.3.4.c.1.128 (GUI) MMP/2.0*/
const Device UA61977 = {S68250, 56, { &P15364,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-NCR_0/00.62 UP.Browser/6.2.3.4.c.1.115 (GUI) MMP/2.0*/
const Device UA64030 = {S68251, 56, { &P15364,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-24.0_/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0*/
const Device UA67827 = {S68252, 55, { &P15364,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-V9mm/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0*/
const Device UA87328 = {S68253, 56, { &P15364,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-MILLA/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0*/
const Device UA1845902 = {S68254, 56, { &P15364,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-E815_/00.62 UP.Browser/6.2.3.4.c.1.106 (GUI) MMP/2.0*/
const Device UA2264193 = {S68255, 56, { &P15364,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-GATW_/00.62 UP.Browser/6.2.3.4.c.1.115 (GUI) MMP/2.0*/
const Device UA86926 = {S68256, 109, { &P18088,&P5448,&P22272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) 320x320; XV6900; Window Mobile 6.0 Professional;*/
const Device UA87458 = {S68257, 66, { &P18088,&P5448,&P22272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) S11HT*/
const Device UA88519 = {S68258, 102, { &P18088,&P5448,&P22272,&P18092 }};	/*Useragent - Lenovo_ET860/Lenovo WindowsMobile/6 Release/4.22.2007 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA165002 = {S68259, 60, { &P18088,&P5448,&P22272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA442492 = {S68260, 80, { &P15364,&P18145,&P19772,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; FreeBSD amd64; en-US; rv:1.8.1.21) Gecko/X SeaMonkey/1.1.16*/
const Device UA848699 = {S68261, 70, { &P15991,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;GT-S6102;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA2496227 = {S68262, 119, { &P15991,&P5629,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.3.6;Zh_cn;GT-S6102;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/444*/
const Device UA837622 = {S68263, 94, { &P17968,&P5477,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.1-update1; Linux; Opera Mobi/ADR-X; U; pl) Presto/2.10.254 Version/12.00*/
const Device UA2483585 = {S68264, 126, { &P20827,&P18066,&P18062,&P18092 }};	/*Useragent - MT6515M-6515/1.0 Linux/2.6.35.7 Android 4.0 Release/11.13.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2523555 = {S68265, 143, { &P20827,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; zh-cn; MT6515M Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2579928 = {S68266, 137, { &P20827,&P18066,&P18062,&P18092 }};	/*Useragent - MT6515M-Connspeed-AS9/1.0 Linux/2.6.35.7 Android 4.0.4 Release/12.11.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2597010 = {S68267, 126, { &P20827,&P18066,&P18062,&P18092 }};	/*Useragent - MT6515M-226J/1.0 Linux/2.6.35.7 Android 4.0 Release/11.13.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2656459 = {S68268, 126, { &P20827,&P18066,&P18062,&P18092 }};	/*Useragent - MT6515M-6515/1.0 Linux/2.6.35.7 Android 4.0 Release/11.15.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2845555 = {S68269, 126, { &P20827,&P18066,&P18062,&P18092 }};	/*Useragent - MT6515M-6515/1.0 Linux/2.6.35.7 Android 4.0 Release/11.14.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2865758 = {S68270, 217, { &P20827,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; MT6515-MT6515-H003/1.0 Android/4.0.3 Release/04.02.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3000722 = {S68271, 217, { &P20827,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; MT6515-MT6515-H003/1.0 Android/4.0.3 Release/05.09.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3092702 = {S68272, 220, { &P20827,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; MT6515-sphs-on-hsdroi/1.0 Android/4.0.3 Release/07.18.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3118634 = {S68273, 137, { &P20827,&P18066,&P18062,&P18092 }};	/*Useragent - MT6515M-SPHS-on-Hsdroid/1.0 Linux/2.6.35.7 Android 4.0 Release/11.12.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2791836 = {S68274, 58, { &P15364,&P5448,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; WindowsCE; Trident/1.0)*/
const Device UA2792602 = {S68275, 63, { &P15364,&P5448,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; PPC; Tablet PC; Trident/5.0)*/
const Device UA2849866 = {S68276, 63, { &P15364,&P5448,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; 68K; Tablet PC; Trident/3.0)*/
const Device UA2752953 = {S68277, 130, { &P17323,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6800 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2896401 = {S68278, 89, { &P17595,&P18207,&P24329,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:22.0) Gecko/X Firefox/22.0 SeaMonkey/2.19*/
const Device UA3066526 = {S68279, 138, { &P18716,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA18523 = {S68280, 97, { &P12711,&P5448,&P17183,&P18092 }};	/*Useragent - Motorola_ES405B/191 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA18524 = {S68281, 99, { &P12711,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Motorola_ES405B_19103*/
const Device UA61402 = {S68282, 96, { &P12711,&P5448,&P17183,&P18092 }};	/*Useragent - Motorola_ES405B/20 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA869473 = {S68283, 97, { &P12711,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Motorola_ES405B_312*/
const Device UA2164821 = {S68284, 116, { &P12711,&P5448,&P17183,&P18092 }};	/*Useragent - Motorola_ES405B/27 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Motorola_ES405B_256*/
const Device UA2989891 = {S68285, 97, { &P12711,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Motorola_ES405B_256*/
const Device UA3073017 = {S68286, 100, { &P12711,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Motorola_ES405B_44.000*/
const Device UA2203280 = {S68287, 182, { &P18385,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:34, us*/
const Device UA2234662 = {S68288, 182, { &P18385,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Nexus 7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, gb*/
const Device UA2241300 = {S68289, 182, { &P18385,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:34, us*/
const Device UA2241304 = {S68290, 182, { &P18385,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03S) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:34, sg*/
const Device UA2166644 = {S68291, 143, { &P21059,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Micromax A90 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2230894 = {S68292, 134, { &P21059,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; A90 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3098585 = {S68293, 56, { &P21059,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; A90 Build/IML74K)*/
const Device UA2426564 = {S68294, 129, { &P20278,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M7088 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2963733 = {S68295, 130, { &P20278,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M7088_ Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2714755 = {S68296, 111, { &P18088,&P18148,&P22782,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows Phone 8.0; Trident/7.0; rv:11.0; IEMobile/11.0; ARM; Touch; Microsoft; Virtual) like Gecko*/
const Device UA2753006 = {S68297, 133, { &P20129,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MI 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879497 = {S68298, 134, { &P20129,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MI 2A Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754346 = {S68299, 136, { &P18320,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755129 = {S68300, 129, { &P18320,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2975080 = {S68301, 145, { &P18476,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I577 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA28393 = {S68302, 91, { &P12235,&P6138,&P4180,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; PalmSource/Palm-D050; Blazer/4.3) 16;448x320*/
const Device UA28394 = {S68303, 91, { &P12235,&P6138,&P4180,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; PalmSource/Palm-D050; Blazer/4.3) 16;320x448*/
const Device UA28395 = {S68304, 91, { &P12235,&P6138,&P4180,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; PalmSource/Palm-D050; Blazer/4.3) 16;320x320*/
const Device UA62132 = {S68305, 110, { &P13296,&P17485,&P3430,&P18092 }};	/*Useragent - Opera/9.5 (Microsoft Windows; PPC; Opera Mobi; U) SonyEricssonX1i/R3AA Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2223526 = {S68306, 85, { &P15655,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s3850) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2512987 = {S68307, 171, { &P18321,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3021198 = {S68308, 175, { &P18321,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5300 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2715019 = {S68309, 140, { &P15869,&P17296,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; en-us; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/6.0 Safari/534.13*/
const Device UA2716481 = {S68310, 139, { &P15869,&P17296,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.1; en-us; Xoom Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/6.0 Safari/534.13*/
const Device UA2975056 = {S68311, 150, { &P18471,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT907 Build/7.7.1Q-144_SMQ_S136-10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975097 = {S68312, 143, { &P18471,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT905 Build/7.7.1Q_GCIRD-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3030033 = {S68313, 168, { &P12280,&P17090,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5 UCBrowser/9.2.3.324*/
const Device UA3092641 = {S68314, 141, { &P12280,&P17090,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 4_3_3 like Mac OS X; zh-CN) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 UCBrowser/9.2.0.312 Mobile*/
const Device UA3113249 = {S68315, 114, { &P13035,&P5043,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-02/06.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA63770 = {S68316, 73, { &P11728,&P5033,&P3417,&P18092 }};	/*Useragent - BlackBerry8820/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102*/
const Device UA23934 = {S68317, 73, { &P11744,&P5033,&P3417,&P18092 }};	/*Useragent - BlackBerry8300/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102*/
const Device UA9102 = {S68318, 70, { &P13388,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHM620/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64713 = {S68319, 111, { &P13388,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHM620/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA836351 = {S68320, 66, { &P15364,&P5690,&P18191,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:10.0.2) Gecko/X Thunderbird/10.0.2*/
const Device UA2406468 = {S68321, 68, { &P15364,&P5690,&P18191,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:10.0.11) Gecko/X Thunderbird/10.0.11*/
const Device UA753949 = {S68322, 138, { &P20206,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE-N880E Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026712 = {S68323, 131, { &P20206,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE N880E Build GRK39F) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2033784 = {S68324, 131, { &P20206,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE N880E Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2352904 = {S68325, 138, { &P20206,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; ZTE N880E Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2520575 = {S68326, 62, { &P20206,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; ZTE N880E Build/GRK39F)*/
const Device UA2006091 = {S68327, 138, { &P12496,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I897 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261629 = {S68328, 138, { &P12496,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I897 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2431074 = {S68329, 171, { &P18321,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2583291 = {S68330, 128, { &P18680,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KFJWI Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2673245 = {S68331, 140, { &P18449,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST27i Build/6.1.1.B.1.54) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709092 = {S68332, 140, { &P18449,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST27a Build/6.1.1.B.1.54) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2861860 = {S68333, 138, { &P18449,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xperia Go Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3067063 = {S68334, 134, { &P22836,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SC-04E Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA23887 = {S68335, 77, { &P15634,&P22283,&P4223,&P18092 }};	/*Useragent - Mozilla/4.0 (Brew MP 1.0.2; U; en-us; Sanyo; NetFront/3.5.1/AMB) Sprint E4100*/
const Device UA640841 = {S68336, 65, { &P15364,&P7127,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; en) Opera 8.53*/
const Device UA718030 = {S68337, 65, { &P15364,&P7127,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; en) Opera 8.54*/
const Device UA815400 = {S68338, 65, { &P15364,&P7127,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; en) Opera 8.52*/
const Device UA824389 = {S68339, 46, { &P15364,&P7127,&P18098,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; U; de) Opera 8.50*/
const Device UA834467 = {S68340, 65, { &P15364,&P7127,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; ru) Opera 8.50*/
const Device UA2684179 = {S68341, 65, { &P15364,&P7127,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; ru) Opera 8.51*/
const Device UA151902 = {S68342, 25, { &P15364,&P22310,&P17470,&P18092 }};	/*Useragent - IBrowse/2.2 (Windows 3.1)*/
const Device UA153048 = {S68343, 131, { &P17346,&P17084,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B117 Twitter for iPhone*/
const Device UA159248 = {S68344, 131, { &P17346,&P17084,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B118 Twitter for iPhone*/
const Device UA725430 = {S68345, 159, { &P14969,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire S Build/GRJ90; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2564440 = {S68346, 156, { &P19861,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I605 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708320 = {S68347, 157, { &P19861,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I605 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3035649 = {S68348, 156, { &P13035,&P5043,&P24679,&P18092 }};	/*Useragent - NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-02) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA63722 = {S68349, 83, { &P13298,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK510i/R4EA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA871974 = {S68350, 110, { &P17595,&P17296,&P22890,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) iCab/5.0 Safari/533.16*/
const Device UA2757743 = {S68351, 120, { &P17595,&P17296,&P22890,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) iCab/5.0 Safari/533.16*/
const Device UA2033391 = {S68352, 140, { &P12616,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WT19i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1964598 = {S68353, 194, { &P15364,&P7133,&P24671,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.5.0.183/50/352/UCWEB*/
const Device UA1988074 = {S68354, 194, { &P15364,&P7133,&P24671,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.5.0.183/50/351/UCWEB*/
const Device UA1991975 = {S68355, 194, { &P15364,&P7133,&P24671,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.5.0.183/28/352/UCWEB*/
const Device UA1996075 = {S68356, 194, { &P15364,&P7133,&P24671,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.5.0.183/28/351/UCWEB*/
const Device UA2155949 = {S68357, 194, { &P15364,&P7133,&P24671,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.5.0.183/51/352/UCWEB*/
const Device UA2157415 = {S68358, 194, { &P15364,&P7133,&P24671,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.5.0.183/50/444/UCWEB*/
const Device UA2167006 = {S68359, 194, { &P15364,&P7133,&P24671,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.5.0.183/50/354/UCWEB*/
const Device UA2167589 = {S68360, 194, { &P15364,&P7133,&P24671,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.5.0.183/50/355/UCWEB*/
const Device UA2254820 = {S68361, 194, { &P15364,&P7133,&P24671,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.5.0.183/28/444/UCWEB*/
const Device UA2375389 = {S68362, 194, { &P15364,&P7133,&P24671,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.5.0.183/28/355/UCWEB*/
const Device UA2375916 = {S68363, 194, { &P15364,&P7133,&P24671,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.5.0.183/51/355/UCWEB*/
const Device UA2366976 = {S68364, 147, { &P20283,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AZ210A Build/AZ210A_ICS_02.27I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2455964 = {S68365, 147, { &P20283,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AZ210B Build/AZ210B_ICS_02.27I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2486638 = {S68366, 154, { &P17283,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; MB860 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2565452 = {S68367, 154, { &P17283,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; MB860 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2782982 = {S68368, 154, { &P17283,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MB860 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865374 = {S68369, 163, { &P17283,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MB860 Build/JDQ39; CyanogenMod-BeanStalk1.179) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654028 = {S68370, 140, { &P24503,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E270S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3120037 = {S68371, 62, { &P24503,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHV-E270S Build/JZO54K)*/
const Device UA2754948 = {S68372, 137, { &P14877,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782847 = {S68373, 137, { &P14877,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9000 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2865050 = {S68374, 141, { &P21334,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; QMobile_A8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3028337 = {S68375, 55, { &P21334,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; A8 Build/JRO03C)*/
const Device UA2879742 = {S68376, 137, { &P22872,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5282 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3018185 = {S68377, 130, { &P22872,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5282 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3010288 = {S68378, 132, { &P21261,&P17979,&P19802,&P18092 }};	/*Useragent - samsung-GT-I9500/1.0 Linux/2.6.35.7 Android/4.2.2 Release/06.06.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3021830 = {S68379, 126, { &P21261,&P17979,&P19802,&P18092 }};	/*Useragent - sprd-I9500/1.0 Linux/2.6.35.7 Android/4.2.2 Release/07.10.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3092516 = {S68380, 137, { &P21261,&P17979,&P19802,&P18092 }};	/*Useragent - samsung-GT-mini-I9500/1.0 Linux/2.6.35.7 Android/4.2.2 Release/07.17.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3108090 = {S68381, 126, { &P21261,&P17979,&P19802,&P18092 }};	/*Useragent - sprd-I9500/1.0 Linux/2.6.35.7 Android/4.2.2 Release/07.18.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3125466 = {S68382, 126, { &P21261,&P17979,&P19802,&P18092 }};	/*Useragent - sprd-I9500/1.0 Linux/2.6.35.7 Android/4.2.2 Release/07.29.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3065583 = {S68383, 152, { &P22962,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA44197 = {S68384, 65, { &P13802,&P17483,&P17602,&P18092 }};	/*Useragent - LG-C305/V10a Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA706831 = {S68385, 153, { &P13089,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; iw-il; U8160 Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2236208 = {S68386, 123, { &P12588,&P5055,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; NokiaC6-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/51/352/UCWEB Mobile*/
const Device UA2502968 = {S68387, 129, { &P12588,&P5055,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiac6-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA2818702 = {S68388, 32, { &P12588,&P5055,&P24672,&P18092 }};	/*Useragent - NokiaC6-01/UCWEB8.6.0.180/51/800*/
const Device UA2408125 = {S68389, 86, { &P17968,&P17979,&P18328,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/7.1.32102/28.3234; U; zh) Presto/2.8.119 Version/11.10*/
const Device UA2425730 = {S68390, 140, { &P21256,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NGM Wilco Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654903 = {S68391, 140, { &P21256,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; NGM_Wilco Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2531143 = {S68392, 165, { &P20492,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cruize W280 Build/Cherry_Cruize_W280_2012.12.30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781720 = {S68393, 140, { &P20492,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cruize W280 Build/V1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2863514 = {S68394, 140, { &P20492,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cruize W280 Build/V1.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2536303 = {S68395, 141, { &P20927,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; AT-AS45IPS Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952643 = {S68396, 63, { &P20927,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; AT-AS45IPS Build/JRO03C)*/
const Device UA2754354 = {S68397, 139, { &P20006,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30a Build/9.1.A.0.489) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2901173 = {S68398, 156, { &P15575,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2944933 = {S68399, 157, { &P15575,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9100G Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3027727 = {S68400, 111, { &P19972,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia113/03.32; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3064757 = {S68401, 138, { &P18290,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ617 Build/7.7.1-85_MZ617-27) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080382 = {S68402, 137, { &P19865,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A850L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080700 = {S68403, 137, { &P19865,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A850K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3082977 = {S68404, 137, { &P19865,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A850S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110571 = {S68405, 141, { &P19865,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A850S/L/K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA43982 = {S68406, 151, { &P13765,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Alcatel OT-981A Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA50433 = {S68407, 182, { &P17951,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0; en-us; Milestone Build/SHOLS_U2_01.03.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 NewsRob (http://newsrob.com) gzip*/
const Device UA89771 = {S68408, 150, { &P17951,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0.1; de-de; Milestone Build/SHOLS_U2_01.14.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA797088 = {S68409, 150, { &P17951,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0.1; en-ca; Milestone Build/SHOLS_U2_03.06.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA3069052 = {S68410, 149, { &P17951,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0; en-us; Milestone Build/ SHOLS_U2_01.03.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA167852 = {S68411, 133, { &P18013,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; EV-S100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA436577 = {S68412, 136, { &P18013,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; EV-S100 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019074 = {S68413, 57, { &P18013,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; EV-S100 Build/FRF91)*/
const Device UA2019968 = {S68414, 60, { &P18013,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; EV-S100 Build/FRG83G)*/
const Device UA705221 = {S68415, 78, { &P17595,&P18120,&P20918,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; rv:1.7.3) Gecko/X Firefox/0.10*/
const Device UA2394834 = {S68416, 79, { &P17595,&P18120,&P20918,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; fr; rv:1.7) Gecko/X Firefox/0.9*/
const Device UA896063 = {S68417, 151, { &P12280,&P17131,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_2 like Mac OS X; en-us)Maxthon AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5G77 Safari/525.20*/
const Device UA2258466 = {S68418, 135, { &P17595,&P18207,&P24745,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Stainless/0.8 like Version/5.1 Safari/534.48.3*/
const Device UA2428032 = {S68419, 113, { &P18503,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Windows Phone 8S by HTC)*/
const Device UA2696601 = {S68420, 95, { &P18503,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; A620m)*/
const Device UA2729797 = {S68421, 121, { &P18503,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Windows Phone 8S by HTC; Orange)*/
const Device UA2748689 = {S68422, 95, { &P18503,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; A620b)*/
const Device UA2751070 = {S68423, 126, { &P18503,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Windows Phone 8S by HTC; 2.00.161.01)*/
const Device UA2764763 = {S68424, 126, { &P18503,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Windows Phone 8S by HTC; 1.04.161.01)*/
const Device UA2985146 = {S68425, 95, { &P18503,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; A620e)*/
const Device UA2738364 = {S68426, 133, { &P23177,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TAB-840_G Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2753957 = {S68427, 140, { &P18303,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT27i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769219 = {S68428, 134, { &P21472,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A111 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2974881 = {S68429, 145, { &P19891,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I717 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024033 = {S68430, 136, { &P19891,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I717 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3080913 = {S68431, 152, { &P19771,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035A Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083746 = {S68432, 152, { &P19771,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3095523 = {S68433, 152, { &P19771,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA659575 = {S68434, 96, { &P20203,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG-E906)*/
const Device UA1984746 = {S68435, 81, { &P11905,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA6120c)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2809009 = {S68436, 32, { &P11905,&P5890,&P24671,&P18092 }};	/*Useragent - NOKIA6120c/UCWEB8.5.0.163/28/800*/
const Device UA2829938 = {S68437, 119, { &P11905,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokia6120c) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile*/
const Device UA1994004 = {S68438, 134, { &P21061,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; pt-br; XT682 Build/V1.51D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023902 = {S68439, 149, { &P12848,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; MB526 Build/GWK74; CyanogenMod-72) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2835615 = {S68440, 176, { &P12848,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; MB526 Build/WhiteRabbitEdition-1.7.1-Plus; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2566642 = {S68441, 152, { &P18505,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MB886 Build/7.7.1Q-144_MB886_MR1-22) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2676254 = {S68442, 129, { &P18320,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676757 = {S68443, 136, { &P18320,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2774330 = {S68444, 140, { &P18451,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26ii Build/6.2.B.0.200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3028610 = {S68445, 141, { &P19778,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PantechP9090 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3065979 = {S68446, 138, { &P22935,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9080L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA290906 = {S68447, 63, { &P15364,&P6704,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows 98) Opera 6.01  [en]*/
const Device UA426939 = {S68448, 31, { &P15364,&P6704,&P17466,&P18092 }};	/*Useragent - Opera/6.03 (Windows 98; U) [en]*/
const Device UA601390 = {S68449, 31, { &P15364,&P6704,&P17466,&P18092 }};	/*Useragent - Opera/6.05 (Windows 98; U) [de]*/
const Device UA915172 = {S68450, 62, { &P15364,&P6704,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows 98) Opera 6.03 [en]*/
const Device UA1967563 = {S68451, 61, { &P15364,&P6704,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows 98) Opera 6.0 [de]*/
const Device UA2329830 = {S68452, 57, { &P15364,&P6704,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows 98) Opera 6.04*/
const Device UA2495603 = {S68453, 32, { &P15364,&P6704,&P17466,&P18092 }};	/*Useragent - Opera/6.06 (Windows 98; U)  [en]*/
const Device UA2505948 = {S68454, 32, { &P15364,&P6704,&P17466,&P18092 }};	/*Useragent - Opera/6.01 (Windows 98; U)  [en]*/
const Device UA699491 = {S68455, 93, { &P15364,&P17285,&P22480,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.2) Gecko/X Firefox/3.0.2 Flock/2.0b3*/
const Device UA2257903 = {S68456, 141, { &P21859,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ALLVIEW P5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2510754 = {S68457, 142, { &P12280,&P18432,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3*/
const Device UA2531841 = {S68458, 141, { &P12280,&P18432,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari/7534.48.3*/
const Device UA2535207 = {S68459, 141, { &P12280,&P18432,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3*/
const Device UA2558339 = {S68460, 140, { &P12280,&P18432,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3*/
const Device UA2850965 = {S68461, 139, { &P12280,&P18432,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Safari/7534.48.3*/
const Device UA2734449 = {S68462, 131, { &P23135,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; U30GT 2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2754237 = {S68463, 138, { &P19849,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7560M Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773199 = {S68464, 136, { &P22847,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SH-06E Build/S5110) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2799872 = {S68465, 136, { &P22847,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SH-06E Build/S5310) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2885388 = {S68466, 136, { &P22847,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SH-06E Build/S6170) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3027874 = {S68467, 136, { &P22847,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SH-06E Build/S7190) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108279 = {S68468, 58, { &P22847,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SH-06E Build/S7190)*/
const Device UA3031638 = {S68469, 155, { &P20705,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3093196 = {S68470, 157, { &P20705,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC_PN071 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3031416 = {S68471, 152, { &P18644,&P5043,&P24679,&P18092 }};	/*Useragent - Nokia112/2.0 (03.32) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia112) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA166620 = {S68472, 147, { &P12280,&P17145,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA303005 = {S68473, 147, { &P12280,&P17145,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8C148 Safari/6531.22.7*/
const Device UA418727 = {S68474, 147, { &P12280,&P17145,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA444478 = {S68475, 150, { &P12280,&P17145,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8C148 Safari/6531.22.7*/
const Device UA539224 = {S68476, 150, { &P12280,&P17145,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8E401 Safari/6531.22.7*/
const Device UA635678 = {S68477, 143, { &P12280,&P17145,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA742159 = {S68478, 130, { &P12280,&P17145,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293*/
const Device UA2131276 = {S68479, 150, { &P12280,&P17145,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8E200 Safari/6531.22.7*/
const Device UA2392116 = {S68480, 148, { &P12280,&P17145,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8C148a Safari/6531.22.7*/
const Device UA2392883 = {S68481, 138, { &P12280,&P17145,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0 Mobile/8C148 Safari/528.16*/
const Device UA2437943 = {S68482, 143, { &P12280,&P17145,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2438983 = {S68483, 144, { &P12280,&P17145,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_10 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2742013 = {S68484, 143, { &P12280,&P17145,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA480488 = {S68485, 90, { &P17595,&P17295,&P22504,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en; rv:1.9.0.8pre) Gecko/X Camino/2.0b3pre*/
const Device UA491803 = {S68486, 117, { &P17595,&P17295,&P22504,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.5; en; rv:1.9.0.19) Gecko/X Camino/2.0.6 (MultiLang) (like Firefox/3.0.19)*/
const Device UA539515 = {S68487, 71, { &P15364,&P5448,&P4777,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA1965518 = {S68488, 110, { &P15364,&P5448,&P4777,&P18092 }};	/*Useragent - 480x800; XV6875.1; Window Mobile 6.5 Professional; (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2293418 = {S68489, 108, { &P15364,&P5448,&P4777,&P18092 }};	/*Useragent - 480x800; XV6975; Window Mobile 6.5 Professional; (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2408809 = {S68490, 71, { &P15364,&P5448,&P4777,&P18092 }};	/*Useragent - USCCHTC6875 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2462657 = {S68491, 80, { &P15364,&P5448,&P4777,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)/UC Browser7.7.1.88*/
const Device UA2541451 = {S68492, 72, { &P15364,&P5448,&P4777,&P18092 }};	/*Useragent - T-Mobile_LEO (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2757575 = {S68493, 76, { &P15364,&P5448,&P4777,&P18092 }};	/*Useragent - T-Mobile_Rhodium (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2759982 = {S68494, 75, { &P15364,&P5448,&P4777,&P18092 }};	/*Useragent - Sprint-SPH-i350 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2033940 = {S68495, 160, { &P24007,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_5210/5210_V1.10; 320*480; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2480447 = {S68496, 160, { &P24007,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; YL-Coolpad_5210/5210_V1.10; 320*480; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2543185 = {S68497, 140, { &P24007,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 5210 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2658559 = {S68498, 150, { &P24007,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_5210S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2815279 = {S68499, 133, { &P24007,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 5210 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2850046 = {S68500, 150, { &P24007,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad 5210S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2993649 = {S68501, 141, { &P24007,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 5210D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2466650 = {S68502, 73, { &P20109,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-S3572/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2508645 = {S68503, 171, { &P19862,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax A110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3023558 = {S68504, 175, { &P19862,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax_A110 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2537202 = {S68505, 96, { &P15364,&P18110,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/588.88 (KHTML, like Gecko) Version/5.1.3 Safari/588.88*/
const Device UA2560314 = {S68506, 96, { &P15364,&P18110,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/599.99 (KHTML, like Gecko) Version/5.1.3 Safari/599.99*/
const Device UA2566791 = {S68507, 156, { &P20947,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Evo V 4G Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2738444 = {S68508, 157, { &P20947,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Evo V 4G Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934832 = {S68509, 157, { &P20947,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Evo V 4G Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2582804 = {S68510, 139, { &P23077,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-uk; @TAB-SEVEN-X2-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2671143 = {S68511, 133, { &P20760,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Event Build/IML77) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2754723 = {S68512, 125, { &P15869,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Xoom Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2876805 = {S68513, 126, { &P15869,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Xoom Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755278 = {S68514, 139, { &P12798,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT18i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769803 = {S68515, 139, { &P12798,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LT18i Build/4.1.A.0.562) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2958134 = {S68516, 139, { &P12798,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT18a Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2880687 = {S68517, 133, { &P18283,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2942357 = {S68518, 133, { &P18283,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; SGH-T999 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2968894 = {S68519, 133, { &P18283,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; SCH-I535 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA575233 = {S68520, 136, { &P16290,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2;en-us;Huawei-U8520 build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1943042 = {S68521, 137, { &P16290,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2;pt-pt;U8520 build/HuaweiU8520) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019857 = {S68522, 139, { &P16290,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-au; U8520 Build/HuaweiU8520) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2574029 = {S68523, 136, { &P16290,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2;de-de;Dual build/HuaweiU8520) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2385870 = {S68524, 180, { &P17475,&P5768,&P24672,&P18092 }};	/*Useragent - Nokia6300/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia6300) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2567371 = {S68525, 142, { &P20399,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP8010 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2966375 = {S68526, 149, { &P15767,&P22882,&P22783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) 1Password/4.2.2 (like Version/11A4414e Mobile/7.0 Safari/8536.25)*/
const Device UA3094417 = {S68527, 149, { &P15767,&P22882,&P22783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) 1Password/4.2.7 (like Version/11A465 Mobile/7.0 Safari/8536.25)*/
const Device UA3131068 = {S68528, 153, { &P15767,&P22882,&P22783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) 1Password/4.2.7 (like Version/11A501 Mobile/7.0.2 Safari/8536.25)*/
const Device UA3080327 = {S68529, 131, { &P15869,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ505 Build/Moto_MZ505) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3081231 = {S68530, 126, { &P15869,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xoom Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3081242 = {S68531, 125, { &P15869,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Xoom Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA9985 = {S68532, 139, { &P16948,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-GT-B7320L/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA43795 = {S68533, 167, { &P16948,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-GT-B7320/1.0 MIDP/2.1 CLDC/1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG-GT-B7320/1.0 MIDP/2.1 CLDC/1.1 Mozilla/4.0*/
const Device UA63759 = {S68534, 116, { &P16948,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-GT-B7320/1.0 MIDP/2.1 CLDC/1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA640310 = {S68535, 113, { &P17595,&P17297,&P18363,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.2.3 (KHTML, like Gecko) Version/5.2 Safari/536.2.3*/
const Device UA877408 = {S68536, 115, { &P17595,&P17297,&P18363,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.10.3 (KHTML, like Gecko) Version/5.2 Safari/536.10.3*/
const Device UA2334726 = {S68537, 113, { &P17595,&P17297,&P18363,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/536.6.1 (KHTML, like Gecko) Version/5.2 Safari/536.6.1*/
const Device UA2373026 = {S68538, 115, { &P17595,&P17297,&P18363,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.18.5 (KHTML, like Gecko) Version/5.2 Safari/535.18.5*/
const Device UA2437103 = {S68539, 140, { &P18302,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT25i Build/4.1.B.0.631) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2433382 = {S68540, 219, { &P18437,&P18066,&P18369,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N8000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2738157 = {S68541, 156, { &P18283,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9300 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2676230 = {S68542, 145, { &P13042,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Incredible S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2772695 = {S68543, 127, { &P23203,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; A1-810 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3080980 = {S68544, 138, { &P20076,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26w Build/6.1.A.2.45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA650 = {S68545, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/XXGK4 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA651 = {S68546, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/XBGK3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA693 = {S68547, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DZHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA709 = {S68548, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DZHF1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA711 = {S68549, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/XBHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA712 = {S68550, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DFHB5 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA713 = {S68551, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DTHB3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA714 = {S68552, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DZGL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA15969 = {S68553, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/AIGI4 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA15970 = {S68554, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/JAGL4 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA15971 = {S68555, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/JCGL4 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA15972 = {S68556, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DCGL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA15973 = {S68557, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/AIHA2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA15975 = {S68558, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/XXHC2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA15976 = {S68559, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/XXHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA15978 = {S68560, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/XXHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA15979 = {S68561, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/XXGK6 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA15980 = {S68562, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/JVGL1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA15981 = {S68563, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DZHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA15974 = {S68564, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/XEGK5 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA18433 = {S68565, 156, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450V/BUHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA18434 = {S68566, 156, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450V/BUHB3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA31847 = {S68567, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DTHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA31848 = {S68568, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/JDGL4 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA31849 = {S68569, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/JPHB1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA31850 = {S68570, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/XXHB1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA31860 = {S68571, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/JPHC2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA31861 = {S68572, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/MBHC3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA35301 = {S68573, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/BGGK4 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA53848 = {S68574, 156, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450V/BUHC2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA53849 = {S68575, 156, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450V/JBJG5 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA53982 = {S68576, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DBHF1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA54190 = {S68577, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DBGL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA54586 = {S68578, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/JPGL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA62171 = {S68579, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/AIHD3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA62246 = {S68580, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/JVHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA62443 = {S68581, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DDHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA62447 = {S68582, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/XEGL1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA63646 = {S68583, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DBHD2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA63896 = {S68584, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DDHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA86682 = {S68585, 156, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450V/BUGJ6 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA86977 = {S68586, 155, { &P11843,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i450/DDHB4 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA6674 = {S68587, 96, { &P13485,&P5026,&P4948,&P18092 }};	/*Useragent - BlackBerry8330/4.5.0.186 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/126 UP.Browser/5.0.3.3*/
const Device UA17143 = {S68588, 97, { &P13485,&P5026,&P4948,&P18092 }};	/*Useragent - BlackBerry8330m/4.5.0.186 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/104 UP.Browser/5.0.3.3*/
const Device UA10056 = {S68589, 79, { &P16177,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; XV6875.1; Windows Phone 6.5)*/
const Device UA10057 = {S68590, 91, { &P16177,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch_Pro2_T7373; Windows Phone 6.5)*/
const Device UA13541 = {S68591, 95, { &P16177,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch_Pro2_T7373; Windows Phone 6.5.3.5)*/
const Device UA45056 = {S68592, 111, { &P16177,&P5222,&P17183,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7381 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; V1.93.841.0 (81210); Windows Phone 6.5)*/
const Device UA500581 = {S68593, 102, { &P16177,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch_Pro2_T7373; HD2_T8585; Windows Phone 6.5)*/
const Device UA925694 = {S68594, 77, { &P16177,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; XV6875; Windows Phone 6.5)*/
const Device UA626371 = {S68595, 184, { &P13746,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT540 Build/GRI40; SwiftDroid v2.0 ASIS (Based on CyanogenMod-7.1.0)) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1942615 = {S68596, 152, { &P13746,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; pl-pl; GT540 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022368 = {S68597, 161, { &P13746,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; pl-pl; LG GT540 Swift Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2495319 = {S68598, 152, { &P13746,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT540 Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042098 = {S68599, 161, { &P13746,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; pl-pl; LG GT540 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA607854 = {S68600, 118, { &P15364,&P18110,&P22457,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19*/
const Device UA608851 = {S68601, 120, { &P15364,&P18110,&P22457,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19*/
const Device UA2126453 = {S68602, 79, { &P15364,&P7127,&P20906,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:0.9.2) Gecko/X Netscape6/6.1*/
const Device UA2362124 = {S68603, 125, { &P14010,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5263) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2566767 = {S68604, 134, { &P17482,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80G9 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2705117 = {S68605, 131, { &P17482,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 80G9 Build/MR1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2671072 = {S68606, 146, { &P19855,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; XT885 Build/9.8.3O-148_IRD-3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952252 = {S68607, 142, { &P19855,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; XT885 Build/6.7.2_GC-385) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3101386 = {S68608, 68, { &P19855,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT885 Build/9.8.3O-148_IRD-3)*/
const Device UA2738010 = {S68609, 156, { &P15919,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; GT-P3100 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2769386 = {S68610, 156, { &P15919,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-P3100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3030553 = {S68611, 113, { &P14482,&P5843,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-01/07.51; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3030632 = {S68612, 115, { &P14482,&P5843,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-01.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3067640 = {S68613, 142, { &P19842,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Butterfly Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA31532 = {S68614, 80, { &P12437,&P5208,&P4223,&P18092 }};	/*Useragent - Mozilla/5.0(compatible; Access NetFront/3.5; BREW 3.1.5; en,es) 400x240 PCD C741*/
const Device UA587051 = {S68615, 90, { &P17595,&P17296,&P18104,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6.8; en; rv:2.0) Gecko/X Firefox/4.0 Opera 11.60*/
const Device UA629018 = {S68616, 82, { &P17595,&P17296,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en) Presto/2.10.229 Version/11.60*/
const Device UA2456966 = {S68617, 129, { &P20476,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X708S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2557451 = {S68618, 146, { &P16598,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; A500 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2714540 = {S68619, 183, { &P16598,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; A500 Build/JDQ39; CyanogenMod-Tegraowners JB-MR1 ROM v10 (thor & digetx)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2729952 = {S68620, 146, { &P16598,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; A500 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2768943 = {S68621, 184, { &P16598,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; A500 Build/JDQ39E; CyanogenMod-Tegraowners JB-MR1 ROM v11 (thor & digetx)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2564317 = {S68622, 142, { &P21358,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; iBall Andi3.5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2961809 = {S68623, 137, { &P21358,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Andi3.5r Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2962224 = {S68624, 139, { &P21358,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.8; en-us; Andi3.5KKe Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2665748 = {S68625, 60, { &P15364,&P18146,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; SunOS i86pc; rv:20.0) Gecko/X Firefox/20.0*/
const Device UA2676976 = {S68626, 136, { &P18243,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P880 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709034 = {S68627, 129, { &P18243,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P880 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2755122 = {S68628, 137, { &P18456,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T699 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3065601 = {S68629, 134, { &P17211,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 101G9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3093496 = {S68630, 131, { &P17211,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 101G9 Build/MR1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3141 = {S68631, 145, { &P15365,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-tw; HTC Magic Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA50266 = {S68632, 146, { &P15365,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HTC Magic Build/EPE54B) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA90076 = {S68633, 145, { &P15365,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; HTC Magic Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018064 = {S68634, 147, { &P15365,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HTC Magic Build/Super E) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018185 = {S68635, 67, { &P15365,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; HTC Magic Build/ERE27)*/
const Device UA2020880 = {S68636, 132, { &P15365,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; HTC Magic Build/EPE54B) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA5445 = {S68637, 158, { &P12258,&P17483,&P3459,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-GS290/V10f Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA29446 = {S68638, 158, { &P12258,&P17483,&P3459,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-GS290/V10b Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA37023 = {S68639, 158, { &P12258,&P17483,&P3459,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-GS290/V10g Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA62882 = {S68640, 158, { &P12258,&P17483,&P3459,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-GS290/V10c Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA63264 = {S68641, 158, { &P12258,&P17483,&P3459,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-GS290/V10d Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA29382 = {S68642, 61, { &P13407,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia6234/2.0 (05.10) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63609 = {S68643, 104, { &P13248,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-A797/A797UCJC1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA153304 = {S68644, 100, { &P16963,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; DELL; Venue Pro)*/
const Device UA904131 = {S68645, 154, { &P14161,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-VM670 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027427 = {S68646, 158, { &P14161,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-VM670 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2234839 = {S68647, 160, { &P16092,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB870 Build/4.5.1A-DTN-130; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2509245 = {S68648, 160, { &P16092,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MB870 Build/4.5.1A-DTN-130; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2403319 = {S68649, 71, { &P19753,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X456/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2675562 = {S68650, 73, { &P15364,&P5690,&P22317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:21.0) Gecko/X Firefox/21.0 SeaMonkey/2.18*/
const Device UA3080132 = {S68651, 134, { &P18280,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; One S Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111716 = {S68652, 137, { &P18280,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA777282 = {S68653, 89, { &P15364,&P5690,&P18201,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-GB; rv:1.8.0.14) Gecko/X Thunderbird/1.5.0.14*/
const Device UA1953538 = {S68654, 96, { &P22103,&P5629,&P15760,&P18092 }};	/*Useragent - HS-U8/1.0 Linux/2.6 Android/2.3 Browser/AppleWebKit533.1 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA1977999 = {S68655, 134, { &P22103,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-U8 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024725 = {S68656, 122, { &P22103,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-U8 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2025496 = {S68657, 91, { &P22103,&P5629,&P15760,&P18092 }};	/*Useragent - HS-U8 Linux Android/2.3.5 Browser/AppleWebKit/533.1 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA2031694 = {S68658, 127, { &P22103,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-U8 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2686610 = {S68659, 98, { &P22103,&P5629,&P15760,&P18092 }};	/*Useragent - HS-U860/1.0 Linux/2.6 Android/2.3 Browser/AppleWebKit533.1 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA2758423 = {S68660, 127, { &P22103,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; HS-U8 Build GRK39F) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA3002609 = {S68661, 136, { &P22103,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; HS-U820 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2422336 = {S68662, 150, { &P20134,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-S5830i Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2467644 = {S68663, 144, { &P21617,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile IQ 1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3118638 = {S68664, 66, { &P21617,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; i-mobile IQ 1 Build/IMM76D)*/
const Device UA2557666 = {S68665, 139, { &P20508,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Cynus T2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986539 = {S68666, 61, { &P20508,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Cynus T2 Build/JRO03C)*/
const Device UA2734419 = {S68667, 151, { &P19762,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI MT1-U06 Build/HuaweiMT1-U06) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3028146 = {S68668, 160, { &P12280,&P22882,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4435d Safari/7534.48.3 TeamStream (iPhone5,2 379 2.7.2)*/
const Device UA3034750 = {S68669, 158, { &P12280,&P22882,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d Safari/7534.48.3 TeamStream (iPhone5,2 389 2.8)*/
const Device UA3110720 = {S68670, 158, { &P12280,&P22882,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari/7534.48.3 TeamStream (iPhone5,2 408 2.9.1)*/
const Device UA3129053 = {S68671, 160, { &P12280,&P22882,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Safari/7534.48.3 TeamStream (iPhone4,1 408 2.9.1)*/
const Device UA3095881 = {S68672, 122, { &P15767,&P22882,&P22883,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Incredimail/1.2.6.6361*/
const Device UA3130334 = {S68673, 124, { &P15767,&P22882,&P22883,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Incredimail/1.2.6.6361*/
const Device UA63404 = {S68674, 105, { &P11815,&P17485,&P4223,&P18092 }};	/*Useragent - SonyEricssonU100i/R7BA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2*/
const Device UA223071 = {S68675, 145, { &P15438,&P17295,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7 HTC_Sensation-orange-LS; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA2595542 = {S68676, 140, { &P22385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2680958 = {S68677, 233, { &P22385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 baiduboxapp/042_1.4_diordna_008_084/gnusmas_61_2.1.4_D2628I-TG/Xe/A074C84EXDDB007D93FX%7CX/1*/
const Device UA2680961 = {S68678, 62, { &P22385,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8262D Build/JZO54K)*/
const Device UA2680981 = {S68679, 131, { &P22385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2681009 = {S68680, 140, { &P22385,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I8262D Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2676984 = {S68681, 129, { &P15932,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P3110 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735036 = {S68682, 136, { &P15932,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P3110 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2861772 = {S68683, 130, { &P15932,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P3110 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2974100 = {S68684, 130, { &P22043,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P5113 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022548 = {S68685, 130, { &P22043,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P5113 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2976000 = {S68686, 136, { &P18653,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P870 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3066874 = {S68687, 136, { &P18234,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P895 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110804 = {S68688, 138, { &P18234,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P895qb Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080249 = {S68689, 135, { &P16356,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-N7000 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA44305 = {S68690, 61, { &P11864,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia6151/2.0 (04.10) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2587302 = {S68691, 84, { &P15364,&P7127,&P19773,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; fr; rv:1.9b4pre) Gecko/X SeaMonkey/2.0a1pre*/
const Device UA1971281 = {S68692, 129, { &P20500,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; Cosmo Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2007356 = {S68693, 146, { &P20333,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; Micromax P500(Funbook) Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2536550 = {S68694, 60, { &P15364,&P18146,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; SunOS i86pc; rv:19.0) Gecko/X Firefox/19.0*/
const Device UA2580307 = {S68695, 138, { &P21185,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MPDC110 BT IPS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2602612 = {S68696, 99, { &P19761,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HUAWEI; W1-U00)*/
const Device UA3050340 = {S68697, 114, { &P19761,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Huawei; W1-U34; Huawei W1-U34)*/
const Device UA2751961 = {S68698, 156, { &P23163,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; zh-cn; SCH-I959 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2752895 = {S68699, 137, { &P22860,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Q700 Build/XOLOQ700) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3073822 = {S68700, 59, { &P22860,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; Q700 Build/XOLOQ700)*/
const Device UA2769419 = {S68701, 134, { &P23149,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Z788G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA191213 = {S68702, 122, { &P15364,&P5690,&P18209,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.0.3.2197 Safari/532.4*/
const Device UA192613 = {S68703, 101, { &P15364,&P5690,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (5.1.2600.3);en-US;kml:2.2;client:Free;type:default)*/
const Device UA194596 = {S68704, 98, { &P15364,&P5690,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (5.1.2600.3);en;kml:2.2;client:Free;type:default)*/
const Device UA194944 = {S68705, 101, { &P15364,&P5690,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (5.1.2600.2);en-US;kml:2.2;client:Free;type:default)*/
const Device UA320353 = {S68706, 101, { &P15364,&P5690,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (5.1.2600.3);en-GB;kml:2.2;client:Free;type:default)*/
const Device UA321605 = {S68707, 101, { &P15364,&P5690,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (5.1.2600.3);de-DE;kml:2.2;client:Free;type:default)*/
const Device UA391771 = {S68708, 101, { &P15364,&P5690,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.2.2074(Windows;Microsoft Windows (5.1.2600.3);en-US;kml:2.2;client:Free;type:default)*/
const Device UA809453 = {S68709, 101, { &P15364,&P5690,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.1.2032(Windows;Microsoft Windows (5.1.2600.3);en-US;kml:2.2;client:Free;type:default)*/
const Device UA2140545 = {S68710, 106, { &P15364,&P5690,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (5.1.2600.3);zh-Hans-CN;kml:2.2;client:Free;type:default)*/
const Device UA2288524 = {S68711, 102, { &P15364,&P5690,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (5.1.2600.3);es-419;kml:2.2;client:Free;type:default)*/
const Device UA199241 = {S68712, 136, { &P18080,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; IM-A725L Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020045 = {S68713, 60, { &P18080,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; IM-A725L Build/FRG83)*/
const Device UA758435 = {S68714, 100, { &P15364,&P17285,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:5.0) Gecko/X Firefox/5.0,gzip(gfe) (via translate.google.com)*/
const Device UA2259226 = {S68715, 80, { &P15364,&P17285,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, lik0) Gecko/X Firefox/5.0*/
const Device UA2257109 = {S68716, 137, { &P21437,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-F120S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028939 = {S68717, 139, { &P21438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F120K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257855 = {S68718, 138, { &P20290,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A11 Build/KarbonnA11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2676941 = {S68719, 141, { &P19778,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP9090 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709060 = {S68720, 131, { &P22966,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID7051 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2717001 = {S68721, 161, { &P18321,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5300 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2798996 = {S68722, 146, { &P21648,&P18598,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9505 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3080463 = {S68723, 136, { &P18621,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E610 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081047 = {S68724, 136, { &P18621,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E612 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094558 = {S68725, 137, { &P18621,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E612f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA137805 = {S68726, 106, { &P15364,&P7480,&P17202,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; MSIE 6.0; ScanAlert; +http://www.scanalert.com/bot.jsp) Firefox/2.0.0.3*/
const Device UA548187 = {S68727, 113, { &P15364,&P7480,&P17202,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; MSIE 6.0; Site Scanner Bot; +http://www.websiteprotection.com) Firefox/2.0.0.3*/
const Device UA2349786 = {S68728, 132, { &P22063,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; KidzStar Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2933277 = {S68729, 61, { &P22063,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; KidzStar Build/IML74K)*/
const Device UA2362159 = {S68730, 136, { &P17717,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-SU760 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2745838 = {S68731, 139, { &P17717,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-SU760 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2838979 = {S68732, 58, { &P17717,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-SU760 Build/ICS)*/
const Device UA2371613 = {S68733, 131, { &P15364,&P16110,&P22512,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.2.4) Gecko/X FireDownload/2.0.1 Firefox/3.6.4 Wyzo/3.6.4.1 FireTorrent/2.0.1*/
const Device UA2557786 = {S68734, 134, { &P18737,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; X909 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2772021 = {S68735, 141, { &P23245,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A3+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2886764 = {S68736, 87, { &P18088,&P17017,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2900451 = {S68737, 88, { &P18088,&P17017,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mstar) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2901171 = {S68738, 87, { &P18088,&P17017,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2938279 = {S68739, 88, { &P18088,&P17017,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; opera) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2941530 = {S68740, 95, { &P18088,&P17017,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui browser) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2962038 = {S68741, 97, { &P18088,&P17017,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mstar platform) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3005191 = {S68742, 84, { &P18088,&P17017,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; vi; maui) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3019706 = {S68743, 84, { &P18088,&P17017,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; id; maui) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2961823 = {S68744, 67, { &P15364,&P18118,&P23479,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:25.0) Gecko/X Firefox/25.0*/
const Device UA3068251 = {S68745, 135, { &P19784,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LIFETAB_P9514 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA83 = {S68746, 92, { &P13265,&P5448,&P3430,&P18092 }};	/*Useragent - Opera/9.5 (Microsoft Windows; Windows CE; Opera Mobi/9.5; U; en) Samsung-SCHI910 PPC 240x400*/
const Device UA32025 = {S68747, 96, { &P13265,&P5448,&P3430,&P18092 }};	/*Useragent - Opera/9.5 (Microsoft Windows; Windows CE; Opera Mobi/9.5; U; en) Samsung-SCHI910 BMC PPC 240x400*/
const Device UA96285 = {S68748, 89, { &P13265,&P5448,&P3430,&P18092 }};	/*Useragent - Opera/9.5 (Microsoft Windows; Windows CE; Opera Mobi/9.5; U; en) VZW:SCH-i910 PPC 240x400*/
const Device UA194233 = {S68749, 143, { &P14969,&P17295,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7 HTC_DesireS-orange-LS; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA2565788 = {S68750, 126, { &P15364,&P18110,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24 XiaoMi/MiuiBrowser/1.0*/
const Device UA2567941 = {S68751, 138, { &P19870,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-L710 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2605734 = {S68752, 131, { &P19870,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-L710 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2663174 = {S68753, 135, { &P24512,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; novo9-Spark Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2856719 = {S68754, 64, { &P24512,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; novo9-Spark Build/MASTER)*/
const Device UA3047355 = {S68755, 142, { &P24512,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; novo9-Spark Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2755287 = {S68756, 150, { &P18460,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-1 Build/HuaweiU8950-1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769575 = {S68757, 154, { &P18460,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-51 Build/HuaweiU8950N-51) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2770423 = {S68758, 152, { &P18460,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-51 Build/HuaweiU8950-51) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2866979 = {S68759, 152, { &P18460,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-1 Build/HuaweiU8950N-1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893428 = {S68760, 148, { &P18460,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950D Build/HuaweiU8950D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2969449 = {S68761, 111, { &P15364,&P18148,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-914_im_india_269) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile*/
const Device UA2969937 = {S68762, 111, { &P15364,&P18148,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-885_im_india_249) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile*/
const Device UA3016064 = {S68763, 111, { &P15364,&P18148,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; zh-CN; NOKIA; RM-822_apac_prc_204) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile*/
const Device UA3057415 = {S68764, 111, { &P15364,&P18148,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; zh-CN; NOKIA; RM-867_apac_prc_277) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile*/
const Device UA3057623 = {S68765, 112, { &P15364,&P18148,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; vi; NOKIA; RM-914_apac_vietnam_228) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile*/
const Device UA3078611 = {S68766, 118, { &P15364,&P18148,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-914_im_south_africa_243) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile*/
const Device UA3107171 = {S68767, 111, { &P15364,&P18148,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; zh-CN; NOKIA; RM-914_apac_prc_227) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile*/
const Device UA2934909 = {S68768, 91, { &P15364,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Mi-315) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2936049 = {S68769, 96, { &P15364,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Celkon_A_59) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2952568 = {S68770, 91, { &P15364,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; POCKET) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2952787 = {S68771, 88, { &P15364,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; A80) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2953845 = {S68772, 94, { &P15364,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Idea_Zeal) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2955989 = {S68773, 89, { &P15364,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; B51+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2956560 = {S68774, 87, { &P15364,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; P8) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2987289 = {S68775, 93, { &P15364,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; TECNO_T3) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2987538 = {S68776, 88, { &P15364,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A87) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2989439 = {S68777, 93, { &P15364,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Lemon_P3) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2989625 = {S68778, 88, { &P15364,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; A56) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2993762 = {S68779, 96, { &P15364,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; Spice_Mi280) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2997101 = {S68780, 108, { &P15364,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_4010X) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2997739 = {S68781, 97, { &P15364,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Symphony_W15) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3009905 = {S68782, 108, { &P15364,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_4010D) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3048872 = {S68783, 92, { &P15364,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Fly_F40) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3073550 = {S68784, 93, { &P15364,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5369) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3086812 = {S68785, 88, { &P15364,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; A19) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3101218 = {S68786, 93, { &P15364,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; AQUA_4.0) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3081088 = {S68787, 155, { &P18113,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA446019 = {S68788, 71, { &P15364,&P7127,&P20918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; rv:1.7.3) Gecko/X Firefox/0.10*/
const Device UA463305 = {S68789, 79, { &P15364,&P7127,&P20918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.8a) Gecko/X Firefox/0.8.0+*/
const Device UA859595 = {S68790, 77, { &P15364,&P7127,&P20918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.7) Gecko/X Firefox/0.9.2*/
const Device UA599533 = {S68791, 31, { &P23144,&P17017,&P17470,&P18092 }};	/*Useragent - iOS/5.1 (9B176) dataaccessd/1.0*/
const Device UA601551 = {S68792, 31, { &P23144,&P17017,&P17470,&P18092 }};	/*Useragent - iOS/5.1 (9B179) dataaccessd/1.0*/
const Device UA623871 = {S68793, 33, { &P23144,&P17017,&P17470,&P18092 }};	/*Useragent - iOS/5.0.1 (9A405) dataaccessd/1.0*/
const Device UA630370 = {S68794, 33, { &P23144,&P17017,&P17470,&P18092 }};	/*Useragent - iOS/5.1.1 (9B206) dataaccessd/1.0*/
const Device UA2043782 = {S68795, 32, { &P23144,&P17017,&P17470,&P18092 }};	/*Useragent - iOS/6.0 (10A403) dataaccessd/1.0*/
const Device UA2043784 = {S68796, 34, { &P23144,&P17017,&P17470,&P18092 }};	/*Useragent - iOS/6.0 (10A5376e) dataaccessd/1.0*/
const Device UA2242335 = {S68797, 34, { &P23144,&P17017,&P17470,&P18092 }};	/*Useragent - iOS/6.0.1 (10A523) dataaccessd/1.0*/
const Device UA2351805 = {S68798, 32, { &P23144,&P17017,&P17470,&P18092 }};	/*Useragent - iOS/6.0 (10A405) dataaccessd/1.0*/
const Device UA2386014 = {S68799, 34, { &P23144,&P17017,&P17470,&P18092 }};	/*Useragent - iOS/6.0.1 (10A525) dataaccessd/1.0*/
const Device UA2437562 = {S68800, 34, { &P23144,&P17017,&P17470,&P18092 }};	/*Useragent - iOS/6.0.2 (10A551) dataaccessd/1.0*/
const Device UA2596573 = {S68801, 34, { &P23144,&P17017,&P17470,&P18092 }};	/*Useragent - iOS/6.0.2 (10A550) dataaccessd/1.0*/
const Device UA2390935 = {S68802, 154, { &P12521,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SonyEricssonSO-02D Build/6.1.F.0.106) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2404958 = {S68803, 142, { &P12521,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-02D Build/6.1.F.0.106) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2942455 = {S68804, 142, { &P12521,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SO-02D Build/6.1.F.0.117) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2581193 = {S68805, 132, { &P19879,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I915 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2596273 = {S68806, 138, { &P20713,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID7065 Build/ICS.MID7065.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3101788 = {S68807, 67, { &P20713,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MID7065 Build/ICS.MID7065.X)*/
const Device UA2677175 = {S68808, 139, { &P12681,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus S 4G Build/JRO03R) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708464 = {S68809, 139, { &P12681,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus S 4G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2715502 = {S68810, 144, { &P23359,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; QMobile A2_Classic Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2769377 = {S68811, 141, { &P16292,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.2; Galaxy Nexus Build/ICL53F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2770926 = {S68812, 141, { &P16292,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.1; Galaxy Nexus Build/ITL41F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782623 = {S68813, 141, { &P16292,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867075 = {S68814, 134, { &P16292,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2974916 = {S68815, 128, { &P23975,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B1-710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3018140 = {S68816, 71, { &P15364,&P7133,&P22171,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:23.0) Gecko/X Firefox/23.0 slurp*/
const Device UA3032620 = {S68817, 64, { &P15364,&P7133,&P22171,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:23.0) Gecko/X Firefox/23.0 slurp*/
const Device UA3057652 = {S68818, 77, { &P15364,&P7133,&P22171,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:23.0) Gecko/X Firefox/23.0 slurp slurp*/
const Device UA2190856 = {S68819, 86, { &P15364,&P7133,&P19986,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.7.0)*/
const Device UA2202165 = {S68820, 121, { &P15364,&P7133,&P19986,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Sleipnir/3.7.3*/
const Device UA2202172 = {S68821, 114, { &P15364,&P7133,&P19986,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Sleipnir/3.7.3*/
const Device UA2202171 = {S68822, 86, { &P15364,&P7133,&P19986,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.7.3)*/
const Device UA2202656 = {S68823, 86, { &P15364,&P7133,&P19986,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.7.1)*/
const Device UA2238570 = {S68824, 121, { &P15364,&P7133,&P19986,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4 Sleipnir/3.7.4*/
const Device UA2259034 = {S68825, 114, { &P15364,&P7133,&P19986,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4 Sleipnir/3.7.4*/
const Device UA2354721 = {S68826, 86, { &P15364,&P7133,&P19986,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.7.4)*/
const Device UA2755336 = {S68827, 137, { &P16362,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-D710 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783690 = {S68828, 137, { &P16362,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SPH-D710 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA70840 = {S68829, 46, { &P15235,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-AN200/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA628441 = {S68830, 141, { &P21203,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SKYPAD Alpha 2 Build/X.01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA837144 = {S68831, 137, { &P21203,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SX-SP715A Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2540561 = {S68832, 80, { &P21203,&P5629,&P15760,&P18092 }};	/*Useragent - Logos/2.2.3 (Linux; U; Android 2.3.4; en-US; SX-SP715A Build/Skypad Alpha2.X.A1)*/
const Device UA2540577 = {S68833, 61, { &P21203,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SX-SP715A Build/GRJ22)*/
const Device UA2540621 = {S68834, 151, { &P21203,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SX-SP715A Build/GRJ22) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/1.0 Version/4.0 Mobile Safari/530.17*/
const Device UA2240799 = {S68835, 101, { &P18634,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 822)*/
const Device UA2366667 = {S68836, 136, { &P18299,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PantechP4100 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2594957 = {S68837, 65, { &P18299,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; PantechP4100 Build/IMM76I)*/
const Device UA2490520 = {S68838, 195, { &P18466,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; KFTT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-17 12:06, +0000, gb*/
const Device UA2508003 = {S68839, 188, { &P18466,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-17 12:06, -0500, us*/
const Device UA2506027 = {S68840, 101, { &P18718,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 620)*/
const Device UA2597471 = {S68841, 111, { &P18718,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 620; Vodafone)*/
const Device UA2792642 = {S68842, 109, { &P18718,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 620; ANZ846)*/
const Device UA2755549 = {S68843, 143, { &P18471,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT905 Build/7.7.1Q_GCIRD-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769361 = {S68844, 150, { &P18471,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT907 Build/7.7.1Q-144_SMQ_S136-10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2756940 = {S68845, 130, { &P17281,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I905 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA50452 = {S68846, 138, { &P14997,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; fr-fr; Liquid Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA89509 = {S68847, 138, { &P14997,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; Liquid Build/Donut) AppleWebkit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2406595 = {S68848, 138, { &P19865,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A850K Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2425120 = {S68849, 138, { &P19865,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A850S Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2426960 = {S68850, 138, { &P19865,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A850L Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2430811 = {S68851, 138, { &P18442,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-N7100 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2487841 = {S68852, 138, { &P18442,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-N7105 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2535479 = {S68853, 137, { &P18442,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7100 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2535788 = {S68854, 137, { &P18442,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7105 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2662475 = {S68855, 137, { &P18442,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L900 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2596441 = {S68856, 213, { &P22020,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; SAMSUNG-GT-I9268_TD/1.0 Android/4.1.1 Release/08.30.2012 Browser/AppleWebKit534.30 Build/JRO03C) ApplelWebkit/534.30 (KHTML,like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2764370 = {S68857, 130, { &P22020,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9268 Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2851953 = {S68858, 139, { &P22020,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9268 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878741 = {S68859, 139, { &P22020,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9268 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996208 = {S68860, 61, { &P22020,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9268 Build/JZO54K)*/
const Device UA3121622 = {S68861, 130, { &P22020,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9268 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3019622 = {S68862, 131, { &P22873,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-N5120 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3125846 = {S68863, 60, { &P22873,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-N5120 Build/JDQ39)*/
const Device UA32187 = {S68864, 149, { &P12478,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-es; Orange_Boston Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32718 = {S68865, 142, { &P12478,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-pt; Boston Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019747 = {S68866, 71, { &P12478,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Orange_Boston Build/ERE27)*/
const Device UA51109 = {S68867, 41, { &P13938,&P17484,&P3625,&P18092 }};	/*Useragent - sam-r450 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA367643 = {S68868, 121, { &P15364,&P5690,&P22417,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.2.1.0 Chrome/16.0.912.75 Safari/535.7*/
const Device UA2512783 = {S68869, 57, { &P15364,&P7127,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0) Presto/2.12.388 Version/12.14*/
const Device UA2581073 = {S68870, 69, { &P15364,&P7127,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; Edition Yx) Presto/2.12.388 Version/12.14*/
const Device UA2527963 = {S68871, 138, { &P18722,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A830L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2727878 = {S68872, 138, { &P18722,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A830K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2763422 = {S68873, 138, { &P18722,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A830S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3014847 = {S68874, 139, { &P18722,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A830KE Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2600339 = {S68875, 140, { &P21881,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Karbonn A4  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2709080 = {S68876, 123, { &P23015,&P17979,&P18112,&P18092 }};	/*Useragent - Lenovo-A820/S100 Linux/3.4.0 Android/4.1 Release/11.14.2012 Browser/AppleWebKit534.30 Profile/ Configuration/ Safari/534.30*/
const Device UA2794029 = {S68877, 108, { &P23015,&P17979,&P18112,&P18092 }};	/*Useragent - Lenovo-A820t_TD/S100 Linux/3.4.0 Android/4.1 Release/1.4.2013 Browser/AppleWebKit534.30 Mobile Safari/534.30*/
const Device UA2737776 = {S68878, 156, { &P12897,&P5843,&P24677,&P18092 }};	/*Useragent - NokiaC2-00/2.0 (03.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-00) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2749090 = {S68879, 137, { &P22955,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P714 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2970866 = {S68880, 94, { &P18364,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC5-05) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA427420 = {S68881, 48, { &P18254,&P18570,&P18245,&P18092 }};	/*Useragent - StumbleUpon/2.60 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA638073 = {S68882, 41, { &P18254,&P18570,&P18245,&P18092 }};	/*Useragent - Bing/3.03 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA899382 = {S68883, 135, { &P21118,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; KIRA N10040 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1972214 = {S68884, 146, { &P20516,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Woxter Tablet PC 97IPS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033516 = {S68885, 151, { &P20516,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Woxter Tablet PC 97IPS DUAL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2475506 = {S68886, 146, { &P20516,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Woxter Tablet PC 97IPS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2193302 = {S68887, 171, { &P14186,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2234486 = {S68888, 171, { &P14186,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2257458 = {S68889, 171, { &P14186,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2455113 = {S68890, 172, { &P14186,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5830L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2728062 = {S68891, 172, { &P14186,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2833731 = {S68892, 171, { &P14186,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2708759 = {S68893, 95, { &P17968,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Android2.x) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2753172 = {S68894, 142, { &P21020,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Micromax A116 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA64191 = {S68895, 107, { &P12396,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-A257/A257UCIC6 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA40003 = {S68896, 140, { &P15017,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; hu-hu; WayteQ Libra Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA53544 = {S68897, 135, { &P15017,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ru-ru; ZTE Libra Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58524 = {S68898, 128, { &P15017,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; pt-pt; a5 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90534 = {S68899, 147, { &P15017,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; Android Edition LIBRA Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA699785 = {S68900, 130, { &P15017,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; pt-pt; a5 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1937436 = {S68901, 137, { &P15017,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; ZTE-Libra Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029367 = {S68902, 140, { &P14225,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Desire HD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2204724 = {S68903, 140, { &P14225,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Desire HD Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2355082 = {S68904, 140, { &P14225,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Desire HD Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2384743 = {S68905, 62, { &P14225,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Desire HD Build/JZO54K)*/
const Device UA2482622 = {S68906, 143, { &P14225,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Desire HD Build/JellyMIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258455 = {S68907, 119, { &P13706,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5235) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile*/
const Device UA2404220 = {S68908, 134, { &P18680,&P18110,&P19816,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Safari/535.19 Silk-Accelerated=false*/
const Device UA2877490 = {S68909, 133, { &P23798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Mi363 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3108197 = {S68910, 135, { &P23798,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Mi363/S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2974299 = {S68911, 146, { &P16263,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 10 LINK Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021272 = {S68912, 136, { &P16263,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; dtab01 Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021578 = {S68913, 146, { &P16263,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 7 Vogue Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023754 = {S68914, 143, { &P16263,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Telpad Dual S Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3025174 = {S68915, 146, { &P16263,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 7 Youth Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA64414 = {S68916, 73, { &P11886,&P5810,&P3417,&P18092 }};	/*Useragent - BlackBerry8700/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100*/
const Device UA138530 = {S68917, 141, { &P15364,&P17817,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Ubuntu/10.10 Chromium/6.0.472.63 Chrome/6.0.472.63 Safari/534.3*/
const Device UA810057 = {S68918, 143, { &P15364,&P17817,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Ubuntu/10.10 Chromium/6.0.472.63 Chrome/6.0.472.63 Safari/534.3*/
const Device UA527956 = {S68919, 66, { &P15364,&P17017,&P18418,&P23780 }};	/*Useragent - Mozilla/5.0 (compatible; Jobrapido/1.1; +http://www.jobrapido.com)*/
const Device UA1964207 = {S68920, 131, { &P20864,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A13-MID Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2231885 = {S68921, 137, { &P20864,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261530 = {S68922, 131, { &P20864,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2455710 = {S68923, 137, { &P20864,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; mid-A13 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2465778 = {S68924, 136, { &P20864,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A13MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2339274 = {S68925, 129, { &P21051,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GEM8112 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2399246 = {S68926, 131, { &P21051,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GEM8112 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2510725 = {S68927, 93, { &P12588,&P5055,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaC6-01) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2698626 = {S68928, 32, { &P12588,&P5055,&P24675,&P18092 }};	/*Useragent - NokiaC6-01/UCWEB8.8.0.212/51/800*/
const Device UA2670744 = {S68929, 136, { &P14877,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9000 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675858 = {S68930, 137, { &P14877,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9000 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2705609 = {S68931, 139, { &P22603,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I739 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2861697 = {S68932, 130, { &P22603,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I739 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2994584 = {S68933, 61, { &P22603,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-I739 Build/JZO54K)*/
const Device UA3101885 = {S68934, 130, { &P22603,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I739 Build JZO54K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2716825 = {S68935, 140, { &P22925,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; M-MP5303G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050023 = {S68936, 62, { &P22925,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; M-MP5303G Build/JZO54K)*/
const Device UA2730314 = {S68937, 82, { &P18435,&P16110,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (IE 11.0; Windows NT 6.2; ARM; Trident/7.0; Touch; rv:11.0) like Gecko*/
const Device UA2937701 = {S68938, 86, { &P18435,&P16110,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; ARM; Trident/7.0; Touch; WebView/1.0; rv:11.0) like Gecko*/
const Device UA2975740 = {S68939, 139, { &P20738,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Andromax U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA226935 = {S68940, 123, { &P15364,&P5690,&P22405,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.1.0.0 Chrome/12.0.742.91 Safari/534.30*/
const Device UA2370621 = {S68941, 123, { &P15364,&P5690,&P22405,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.1.0.0 Chrome/12.0.742.91 Safari/534.30*/
const Device UA693671 = {S68942, 140, { &P12059,&P5204,&P24667,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia2690) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2452698 = {S68943, 143, { &P20233,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VSD220 Build/IMM76D.UI22E332_VSC) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2466881 = {S68944, 143, { &P20233,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VSD220 Build/IMM76D.UI23E802_VSC) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2664386 = {S68945, 143, { &P20233,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VSD220 Build/IMM76D.UI23EC62_VSC) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2937666 = {S68946, 143, { &P20233,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VSD220 Build/IMM76D.UI23ED12_VSC) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3088254 = {S68947, 143, { &P20233,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VSD220 Build/IMM76D.UI23ED02_VSC) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2433697 = {S68948, 132, { &P17337,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P7500 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2535461 = {S68949, 131, { &P17337,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7500 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2567316 = {S68950, 143, { &P19862,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax A110 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566529 = {S68951, 154, { &P18016,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ThinkPad Tablet Build/ThinkPadTablet_A400_03) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583947 = {S68952, 146, { &P18512,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT881 Build/9.8.3O-118_USC-14) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2603685 = {S68953, 139, { &P18512,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT881 Build/9.8.3O-118_USC-14) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2975040 = {S68954, 148, { &P22968,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HUAWEI P6-U06 Build/HuaweiP6-U06) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA4242 = {S68955, 95, { &P11734,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Vodafone/1.0/HTC_Kaiser/1.56.162.5*/
const Device UA12324 = {S68956, 104, { &P11734,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC P4550; OpVer 24.181.1.612*/
const Device UA23735 = {S68957, 72, { &P11734,&P5448,&P22271,&P18092 }};	/*Useragent - HTC_TyTN_II Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA28976 = {S68958, 104, { &P11734,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC P4550; OpVer 24.181.9.693*/
const Device UA28984 = {S68959, 104, { &P11734,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC P4550; OpVer 24.181.8.683*/
const Device UA45090 = {S68960, 95, { &P11734,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Vodafone/1.0/HTC_Kaiser/1.56.161.4*/
const Device UA45089 = {S68961, 95, { &P11734,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Vodafone/1.0/HTC_Kaiser/1.56.172.3*/
const Device UA49358 = {S68962, 121, { &P11734,&P5448,&P22271,&P18092 }};	/*Useragent - HTC P4550/1.83.621.0 Mozilla/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA66424 = {S68963, 98, { &P11734,&P5448,&P22271,&P18092 }};	/*Useragent - HTC-P4550-orange/PPC; 240x320; OpVer 24.181.2.731 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA66425 = {S68964, 70, { &P11734,&P5448,&P22271,&P18092 }};	/*Useragent - HTC_P4550 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA74386 = {S68965, 95, { &P11734,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Vodafone/1.0/HTC_Kaiser/1.56.165.5*/
const Device UA69507 = {S68966, 93, { &P13999,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; HD2)*/
const Device UA925700 = {S68967, 99, { &P13999,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; HD2 T8585)*/
const Device UA925730 = {S68968, 93, { &P13999,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.2; Trident/5.0; IEMobile/9.0;  ; LEO70)*/
const Device UA925768 = {S68969, 93, { &P13999,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; LEO)*/
const Device UA925807 = {S68970, 97, { &P13999,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; HD2 LEO)*/
const Device UA2204940 = {S68971, 125, { &P12080,&P17484,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s3653) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile*/
const Device UA2392685 = {S68972, 94, { &P15364,&P17817,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.10) Gecko/X Ubuntu/10.10 (maverick) Firefox/3*/
const Device UA2407668 = {S68973, 139, { &P14183,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I997 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2561914 = {S68974, 138, { &P14183,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I997 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2868453 = {S68975, 139, { &P14183,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I997 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2983800 = {S68976, 61, { &P14183,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; SGH-I997 Build/JOP40D)*/
const Device UA2995589 = {S68977, 61, { &P14183,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGH-I997 Build/JDQ39E)*/
const Device UA2435210 = {S68978, 138, { &P16356,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-N7000 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2484554 = {S68979, 132, { &P16356,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-N7000 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2533190 = {S68980, 137, { &P16356,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2858864 = {S68981, 131, { &P16356,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2560008 = {S68982, 152, { &P19748,&P5043,&P24676,&P18092 }};	/*Useragent - Nokia205/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia205) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2780471 = {S68983, 156, { &P19748,&P5043,&P24676,&P18092 }};	/*Useragent - Nokia205.1/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia205.1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2584167 = {S68984, 148, { &P15877,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2862736 = {S68985, 141, { &P15877,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2676187 = {S68986, 126, { &P18248,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A700 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2999683 = {S68987, 133, { &P18248,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A700 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2696597 = {S68988, 155, { &P18622,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; IM-A820L Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2698227 = {S68989, 154, { &P18622,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A820L Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2990742 = {S68990, 168, { &P18622,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A820L Build/JZO54K; CyanogenMod-CyanogenMod 10 ) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2992559 = {S68991, 174, { &P18622,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; IM-A820L Build/Hoang Nguyen; CyanogenMod-EF84L_JellyBean) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3047728 = {S68992, 188, { &P18622,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A820L Build/Ho?ng Minh H?u _ VegavietTeam; CyanogenMod-JB_LR_VGV_V5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2755684 = {S68993, 139, { &P12681,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus S 4G Build/JRO03R) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2974388 = {S68994, 154, { &P22856,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0151 Build/HuaweiY300-0151) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2422 = {S68995, 73, { &P17424,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-RAZRV3xv/85.97.CDP MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2426 = {S68996, 73, { &P17424,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-RAZRV3xM/85.9B.E5P MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61482 = {S68997, 74, { &P17424,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-RAZRV3xvM/85.97.CDP MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA82967 = {S68998, 71, { &P17424,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-RAZRV3x/01.2A.01 MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA615028 = {S68999, 64, { &P15364,&P17285,&P18367,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:12.0) Gecko/X Thunderbird/12.0.1*/
const Device UA651115 = {S69000, 85, { &P15364,&P17979,&P3508,&P18092 }};	/*Useragent - ZTE-TU812_TD/1.0 Linux/2.6.32 Android/2.2 Release/07.25.2011 Browser/AppleWebKit533.1*/
const Device UA692983 = {S69001, 91, { &P15364,&P17979,&P3508,&P18092 }};	/*Useragent - CoolPad8810_CMCC_TD/1.0 Linux/2.6.32 Android/2.2 Release/6.30.2011 Browser/AppleWebkit533.1*/
const Device UA1900451 = {S69002, 122, { &P15364,&P17979,&P3508,&P18092 }};	/*Useragent - HuaweiT8100_TD/1.0 Android/2.2 Release/12.25.2010 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 AppleWebKit/533.1*/
const Device UA2019515 = {S69003, 91, { &P15364,&P17979,&P3508,&P18092 }};	/*Useragent - CoolPad8811_CMCC_TD/1.0 Android/2.2 Release/4.10.2011 Browser/WAP 2.0 (Apple Webkit/530.18)*/
const Device UA2022715 = {S69004, 124, { &P15364,&P17979,&P3508,&P18092 }};	/*Useragent - ZTE-TU802_TD/1.0 Linux/2.6.32 Android/2.2 Release/1.1.2011 Browser/AppleWebKit530.17 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2031777 = {S69005, 75, { &P15364,&P17979,&P3508,&P18092 }};	/*Useragent - BLUA73/1.0 Linux/2.6.35 Android/2.2.2 Release/5.26.2011 Browser/WebKit533.1*/
const Device UA2752730 = {S69006, 19, { &P15364,&P17979,&P3508,&P18092 }};	/*Useragent - eBayAndroid/2.2.1.0*/
const Device UA2866225 = {S69007, 20, { &P15364,&P17979,&P3508,&P18092 }};	/*Useragent - eBayAndroid/2.2.0.25*/
const Device UA2981450 = {S69008, 121, { &P15364,&P17979,&P3508,&P18092 }};	/*Useragent - 4G_W4_TD/1.0 Android/2.2 MocorDroid/W11.32 Release/2011 Browser/AppleWebKit5333.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA3070292 = {S69009, 122, { &P15364,&P17979,&P3508,&P18092 }};	/*Useragent - TD818_TD/1.0 Android_OS/2.2 Mocor_Droid/W11 Release/2011 Browser/AppleWebKit5333.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA923865 = {S69010, 130, { &P21816,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LT7052 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2495599 = {S69011, 80, { &P15364,&P5690,&P22315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; pl-PL; rv:1.9a1) Gecko/X SeaMonkey/1.5a*/
const Device UA2676264 = {S69012, 130, { &P23773,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8010 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676585 = {S69013, 145, { &P19867,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2862286 = {S69014, 138, { &P19867,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I437 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2879801 = {S69015, 143, { &P22654,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5502 Build/10.1.1.A.1.310) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2974462 = {S69016, 136, { &P24039,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9190 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3067304 = {S69017, 138, { &P23969,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA65425 = {S69018, 134, { &P14784,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Hero Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA145370 = {S69019, 136, { &P14784,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; HTC Hero Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625783 = {S69020, 60, { &P14784,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; HTC Hero Build/FRG83)*/
const Device UA2021470 = {S69021, 137, { &P14784,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-cn; HTC Hero Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA236705 = {S69022, 128, { &P17595,&P17297,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13*/
const Device UA344659 = {S69023, 128, { &P17595,&P17297,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_2; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13*/
const Device UA719727 = {S69024, 128, { &P17595,&P17297,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_3; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.107 Safari/534.13*/
const Device UA808477 = {S69025, 128, { &P17595,&P17297,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13*/
const Device UA878697 = {S69026, 128, { &P17595,&P17297,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_0; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.107 Safari/534.13*/
const Device UA2348654 = {S69027, 128, { &P17595,&P17297,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.102 Safari/534.13*/
const Device UA2181710 = {S69028, 146, { &P20981,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; Micromax P250(Funbook) Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3033250 = {S69029, 157, { &P18673,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2959782 = {S69030, 141, { &P24066,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2;en-us; Lenovo_A706_ROW/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1.2 Mobile Safari/534.30*/
const Device UA3052007 = {S69031, 146, { &P24066,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Lenovo A706_ROW Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3055370 = {S69032, 68, { &P24066,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Lenovo A706_ROW Build/JZO54K)*/
const Device UA3061283 = {S69033, 136, { &P20124,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Grand X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3062541 = {S69034, 137, { &P20124,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3080415 = {S69035, 138, { &P20124,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081087 = {S69036, 139, { &P20124,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Grand X(M) Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083283 = {S69037, 140, { &P20124,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Grand X Pro Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA96093 = {S69038, 82, { &P17346,&P17779,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko)*/
const Device UA506322 = {S69039, 21, { &P17346,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPod2C1/803.148*/
const Device UA1934343 = {S69040, 21, { &P17346,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPod4C1/901.405*/
const Device UA2125810 = {S69041, 17, { &P17346,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPod/705.18*/
const Device UA2156059 = {S69042, 21, { &P17346,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPod4C1/902.206*/
const Device UA2503481 = {S69043, 22, { &P17346,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPod4C1/1001.523*/
const Device UA2733951 = {S69044, 67, { &P17346,&P17779,&P17596,&P18092 }};	/*Useragent - gamed/4.10.17.1.6.13.5.2.1 (iPod4,1; 6.1.3; 10B329; GameKit-781.18)*/
const Device UA2734531 = {S69045, 22, { &P17346,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPod4C1/1002.329*/
const Device UA2830731 = {S69046, 22, { &P17346,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPod4C1/1001.403*/
const Device UA2850524 = {S69047, 22, { &P17346,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPod4C1/1002.144*/
const Device UA2861785 = {S69048, 22, { &P17346,&P17779,&P17596,&P18092 }};	/*Useragent - Apple-iPod5C1/1002.329*/
const Device UA651090 = {S69049, 143, { &P18030,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SHW-M100S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019464 = {S69050, 67, { &P18030,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SHW-M100S Build/GINGERBREAD)*/
const Device UA630668 = {S69051, 141, { &P23940,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; PMID700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA851775 = {S69052, 149, { &P15767,&P17141,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) Maxthon AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.10*/
const Device UA2431071 = {S69053, 166, { &P18283,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-I9300 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2461078 = {S69054, 166, { &P18283,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9300 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2486116 = {S69055, 166, { &P18283,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; SCH-I535 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2422365 = {S69056, 155, { &P13107,&P5043,&P24673,&P18092 }};	/*Useragent - NokiaC2-06/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac2-06) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2560370 = {S69057, 170, { &P14682,&P5843,&P24676,&P18092 }};	/*Useragent - Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6303iclassic) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2738175 = {S69058, 160, { &P14708,&P5204,&P24677,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5130c-2) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA3039096 = {S69059, 155, { &P18466,&P18066,&P24339,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA3039830 = {S69060, 139, { &P17346,&P22882,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.7 Mobile/11A4435d Safari/8536.25*/
const Device UA3083404 = {S69061, 140, { &P17346,&P22882,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A4449d Safari/8536.25*/
const Device UA3095962 = {S69062, 138, { &P17346,&P22882,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A465 Safari/8536.25*/
const Device UA3130634 = {S69063, 140, { &P17346,&P22882,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/11A501 Safari/8536.25*/
const Device UA63208 = {S69064, 105, { &P14008,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonTM506/R3DA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.2*/
const Device UA576552 = {S69065, 31, { &P14033,&P5129,&P24665,&P18092 }};	/*Useragent - NokiaC5-03/UCWEB8.0.3.99/50/800*/
const Device UA679259 = {S69066, 43, { &P14033,&P5129,&P24665,&P18092 }};	/*Useragent - NokiaC5-03/UC Browser8.0.4.121/50/444/UCWEB*/
const Device UA633600 = {S69067, 71, { &P15364,&P17285,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:6.0) Gecko/X Firefox/6.0 BLNGBAR*/
const Device UA1904682 = {S69068, 100, { &P15364,&P17285,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.3) Gecko/X Firefox/6.0 (.NET CLR 3.5.30729)*/
const Device UA695280 = {S69069, 84, { &P17595,&P17295,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:8.0) Gecko/X Firefox/8.0 TenFourFox/G5*/
const Device UA1996984 = {S69070, 132, { &P17168,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P7310 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2261551 = {S69071, 132, { &P17168,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7310 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2599834 = {S69072, 142, { &P21991,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; Aqua Wonder Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2796023 = {S69073, 152, { &P21991,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; Aqua Wonder Quad Core Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665836 = {S69074, 136, { &P22615,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ICE 350e Build/FSV01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2879027 = {S69075, 136, { &P22615,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ICE 350e Build/FSV06) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3005425 = {S69076, 136, { &P22615,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ICE 350e Build/SKV01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3008145 = {S69077, 136, { &P22615,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ICE 350e Build/FSV07) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3068057 = {S69078, 152, { &P20749,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA32141 = {S69079, 66, { &P12467,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GU230/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA51284 = {S69080, 165, { &P13944,&P5129,&P3690,&P18092 }};	/*Useragent - SonyEricssonU8a/R2EA; Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525*/
const Device UA67353 = {S69081, 135, { &P17716,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-LU3700 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67541 = {S69082, 137, { &P17716,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; LG-LU3700 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA68248 = {S69083, 138, { &P17716,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-LU3700 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA439969 = {S69084, 59, { &P17716,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; LG-LU3700 Build/FRF91)*/
const Device UA1888227 = {S69085, 62, { &P17716,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-LU3700 Build/FRG83G)*/
const Device UA93968 = {S69086, 141, { &P17622,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; HERO200 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2195930 = {S69087, 180, { &P12059,&P5204,&P24671,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia2690) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2506766 = {S69088, 131, { &P23093,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC801 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2558687 = {S69089, 94, { &P15649,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570I) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2580414 = {S69090, 93, { &P15649,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2564664 = {S69091, 153, { &P18253,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT894 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658617 = {S69092, 154, { &P18253,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; XT894 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3016938 = {S69093, 164, { &P18253,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT894 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/maserati*/
const Device UA2752169 = {S69094, 139, { &P22595,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-ca; SGH-M919V Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781928 = {S69095, 61, { &P22595,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGH-M919V Build/JDQ39)*/
const Device UA2868313 = {S69096, 138, { &P22874,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9502 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957461 = {S69097, 60, { &P22874,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9502 Build/JDQ39)*/
const Device UA3092450 = {S69098, 138, { &P22874,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.9; en-us; GT-I9502 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA633680 = {S69099, 142, { &P14799,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; pt-br; GT-I5500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA738391 = {S69100, 142, { &P14799,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; GT-I5500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021888 = {S69101, 135, { &P14799,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; Galaxy5 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2474330 = {S69102, 61, { &P14799,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; GT-I5500 Build/GRK39F)*/
const Device UA2686528 = {S69103, 144, { &P14799,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT-I5500 Build/GRK39F; Stock) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA782346 = {S69104, 94, { &P17595,&P17296,&P18198,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.2.28) Gecko/X Thunderbird/3.1.20*/
const Device UA720724 = {S69105, 76, { &P15364,&P17285,&P22765,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Maxthon/3.0)*/
const Device UA1931551 = {S69106, 174, { &P15364,&P17285,&P22765,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Maxthon/3.0)*/
const Device UA2148551 = {S69107, 83, { &P15364,&P17285,&P22765,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Maxthon/3.0)*/
const Device UA2151964 = {S69108, 153, { &P15364,&P17285,&P22765,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Maxthon/3.0)*/
const Device UA2400996 = {S69109, 155, { &P14111,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Arc S Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2200553 = {S69110, 70, { &P17346,&P18432,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10A403 (iPod; U; CPU OS 6_0 like Mac OS X; en_us)*/
const Device UA2242205 = {S69111, 72, { &P17346,&P18432,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10A523 (iPod; U; CPU OS 6_0_1 like Mac OS X; en_us)*/
const Device UA2462049 = {S69112, 171, { &P15649,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2469148 = {S69113, 172, { &P15649,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2469875 = {S69114, 171, { &P15649,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2423129 = {S69115, 140, { &P20262,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; A18 Build/CrossCross_A18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2951440 = {S69116, 64, { &P20262,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; A18 Build/CrossCross_A18)*/
const Device UA2676429 = {S69117, 126, { &P18239,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A510 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3011518 = {S69118, 138, { &P22853,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HTC_802w Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3019551 = {S69119, 138, { &P22853,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HTC 802w Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3020501 = {S69120, 146, { &P22853,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ru-ru; HTC_One_dual_sim Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050783 = {S69121, 60, { &P22853,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC 802w Build/JDQ39)*/
const Device UA3083648 = {S69122, 135, { &P22582,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SGH-I337 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA24735 = {S69123, 82, { &P11710,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW300c/R9A Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64698 = {S69124, 83, { &P11710,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW300i/R4EA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA68370 = {S69125, 135, { &P14923,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; md-us; GTablet Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA195612 = {S69126, 135, { &P14923,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GTablet Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA310659 = {S69127, 135, { &P14923,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; md-us; GTablet Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA575018 = {S69128, 136, { &P14923,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GTablet Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA642881 = {S69129, 135, { &P14923,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GTablet Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA742611 = {S69130, 135, { &P14923,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; GTablet Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018887 = {S69131, 135, { &P14923,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; md-us; GTablet Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA166281 = {S69132, 62, { &P15364,&P7127,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.7.62 Version/11.00*/
const Device UA302374 = {S69133, 62, { &P15364,&P7127,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.7.62 Version/11.01*/
const Device UA861740 = {S69134, 117, { &P17595,&P17294,&P22504,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.4; fr; rv:1.9.0.19) Gecko/X Camino/2.0.7 (MultiLang) (like Firefox/3.0.19)*/
const Device UA1958983 = {S69135, 87, { &P17968,&P18066,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.4; Linux; Opera Mobi/ADR-X; U; en) Presto/2.8.149 Version/11.10*/
const Device UA2032785 = {S69136, 87, { &P17968,&P18066,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.3; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.8.149 Version/11.10*/
const Device UA2547729 = {S69137, 85, { &P17968,&P18066,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0; Linux; Opera Mobi/ADY-X; U; ru) Presto/2.8.149 Version/11.10*/
const Device UA2205738 = {S69138, 123, { &P15364,&P17285,&P24194,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.7 (KHTML, like Gecko) Chrome/20.0.1099.0 Safari/536.7 QQBrowser/6.14.15493.201*/
const Device UA2812594 = {S69139, 186, { &P15364,&P17285,&P24194,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C) QQBrowser/6.14.14517.201*/
const Device UA2182785 = {S69140, 120, { &P17595,&P18207,&P24727,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/536.25 (KHTML, like Gecko) FluidApp Version/1.6 Safari/536.25*/
const Device UA2202378 = {S69141, 128, { &P17595,&P18207,&P24727,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) FluidApp Version/1.6.1 Safari/536.26.14*/
const Device UA2583995 = {S69142, 128, { &P17595,&P18207,&P24727,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) FluidApp Version/1.6.1 Safari/536.28.10*/
const Device UA2869114 = {S69143, 126, { &P17595,&P18207,&P24727,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) FluidApp Version/1.6.1 Safari/536.30.1*/
const Device UA2456560 = {S69144, 141, { &P18455,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC-T528d/1.05.1401.1) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2503161 = {S69145, 131, { &P18455,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T528d Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2580157 = {S69146, 141, { &P18455,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC-T528d/1.19.1401.7) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2690792 = {S69147, 129, { &P18455,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T528d) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768701 = {S69148, 141, { &P18455,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC-T528d/2.14.1401.1) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2791285 = {S69149, 141, { &P18455,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC-T528d/1.19.1401.2) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2845890 = {S69150, 141, { &P18455,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC-T528d/1.05.1401.2) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2964517 = {S69151, 141, { &P18455,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC-T528d/2.18.1401.1) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099464 = {S69152, 62, { &P18455,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC T528d Build/IMM76D)*/
const Device UA2582299 = {S69153, 133, { &P21883,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; da-dk; TAD-10023 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2600340 = {S69154, 156, { &P20542,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PAP5000TDUO Build/PrestigioPAP5000TDUO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2989161 = {S69155, 78, { &P20542,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; PAP5000TDUO Build/PrestigioPAP5000TDUO)*/
const Device UA2679617 = {S69156, 137, { &P15364,&P19822,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu/12.10 Chromium/27.0.1453.6 Chrome/27.0.1453.6 Safari/537.36*/
const Device UA2719396 = {S69157, 135, { &P15364,&P19822,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu/12.10 Chromium/27.0.1453.6 Chrome/27.0.1453.6 Safari/537.36*/
const Device UA2750492 = {S69158, 111, { &P23283,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Huawei; H883G; HuaweiH883G)*/
const Device UA2879892 = {S69159, 155, { &P18466,&P18066,&P23482,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2976016 = {S69160, 135, { &P22126,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Torque Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3081348 = {S69161, 136, { &P22804,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P713 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081378 = {S69162, 138, { &P22804,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P713GO Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA63897 = {S69163, 98, { &P13894,&P5448,&P17183,&P18092 }};	/*Useragent - SAMSUNG-GT-B6520/1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA578632 = {S69164, 114, { &P14540,&P5843,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-01/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA1946923 = {S69165, 116, { &P14540,&P5843,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-01.1/07.22; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA764509 = {S69166, 129, { &P18239,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  A510 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA902500 = {S69167, 128, { &P18239,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A510 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA903662 = {S69168, 64, { &P15364,&P5690,&P18514,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:13.0) Gecko/X Thunderbird/13.0.1*/
const Device UA1961540 = {S69169, 136, { &P21653,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; TAB-PROTAB25 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033038 = {S69170, 145, { &P18512,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT881 Build/6.7.3-130_YUS-11) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2736733 = {S69171, 137, { &P22804,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P713 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2970829 = {S69172, 139, { &P22804,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P713GO Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3067180 = {S69173, 140, { &P18084,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA676258 = {S69174, 137, { &P15798,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Pico_A310b; en-au) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA756876 = {S69175, 146, { &P15798,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC/Explorer/1.17.161.3; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2167032 = {S69176, 146, { &P15798,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC/Explorer/1.29.161.3; it-it) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2341092 = {S69177, 146, { &P15798,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC/Explorer/1.41.161.3; es-es) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2426317 = {S69178, 137, { &P14574,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus S Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2536318 = {S69179, 136, { &P14574,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus S Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2429763 = {S69180, 192, { &P15882,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0000, gb*/
const Device UA2635778 = {S69181, 192, { &P15882,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-P5110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.27/381,1.9.27.381,2013-03-21 11:39, +0100, fr*/
const Device UA2457374 = {S69182, 150, { &P20020,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-nz; ALCATEL ONE TOUCH 922 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2670626 = {S69183, 129, { &P15882,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5110 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3030379 = {S69184, 150, { &P12879,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3065976 = {S69185, 143, { &P24048,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly s Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA23643 = {S69186, 163, { &P14085,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; tr-tr; LG-P503 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA53980 = {S69187, 161, { &P14085,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; LG-P503 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA57730 = {S69188, 163, { &P14085,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; tr-tr; LG-P503 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA1934515 = {S69189, 121, { &P14085,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; LG-P503 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2496390 = {S69190, 126, { &P14085,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; LG-P503 Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA69326 = {S69191, 43, { &P11684,&P5129,&P24663,&P18092 }};	/*Useragent - Nokia5800w XpressMusic/UCWEB7.9.0.94/50/999*/
const Device UA69399 = {S69192, 42, { &P11684,&P5129,&P24663,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UCWEB7.9.0.94/50/999*/
const Device UA249005 = {S69193, 54, { &P11684,&P5129,&P24663,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UC Browser7.9.0.102/50/355/UCWEB*/
const Device UA913435 = {S69194, 124, { &P15364,&P5690,&P22433,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11 Comodo_Dragon/20.1.1.0*/
const Device UA1955497 = {S69195, 81, { &P18364,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaC5-05)/UC Browser8.5.0.183/50/355/UCWEB Mobile*/
const Device UA2206712 = {S69196, 139, { &P18606,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PadFone 2 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2206716 = {S69197, 133, { &P18606,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PadFone 2 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2566619 = {S69198, 130, { &P17337,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7500 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2633355 = {S69199, 131, { &P17337,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P7500 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2676447 = {S69200, 134, { &P21636,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C525c Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2878853 = {S69201, 129, { &P23801,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; P774A Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3124599 = {S69202, 58, { &P23801,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; P774A Build/JRO03H)*/
const Device UA3080217 = {S69203, 132, { &P18295,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; One X Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3082800 = {S69204, 136, { &P18295,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; HTC One X Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA396894 = {S69205, 137, { &P20230,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; LG-LU6500 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020010 = {S69206, 61, { &P20230,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-LU6500 Build/GRJ22)*/
const Device UA1964485 = {S69207, 32, { &P12088,&P5129,&P24671,&P18092 }};	/*Useragent - NokiaX6-00/UCWEB8.5.0.163/50/800*/
const Device UA2166996 = {S69208, 81, { &P12088,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaX6-00)/UC Browser8.5.0.183/50/355/UCWEB Mobile*/
const Device UA2230659 = {S69209, 140, { &P17954,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; MID1125 Build/GreenDemon_v1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2231091 = {S69210, 128, { &P22136,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T1Ci Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2459016 = {S69211, 225, { &P22787,&P22791,&P18246,&P18092 }};	/*Useragent - TECHAIN50_11B_HW (MRE\3.0.00(2500) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) Cherry_W7_T712V_M01_S10_X Release/2012.10.25 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US*/
const Device UA2708703 = {S69212, 373, { &P22787,&P22791,&P18246,&P18092 }};	/*Useragent - TECHAIN50_11B_HW (MRE/3.0.00(2500);MAUI/y_W7_T712V_M01_S13_X;BDATE/2013/01/30 15:29;LCD/X;CHIP/MT6250;KEY/Reduced;TOUCH/1;CAMERA/0;SENSOR/0;DEV/TECHAIN50_11B_HW;WAP Browser/MAUI (HTTP PGDL;HTTPS);GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/005;GAMELOFT/00) Cherry_W7_T712V_M01_S13_X Release/2013.01.30 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US*/
const Device UA2762617 = {S69213, 225, { &P22787,&P22791,&P18246,&P18092 }};	/*Useragent - TECHAIN50_11B_HW (MRE\3.0.00(2500) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) Cherry_W7_T712V_M01_S06_X Release/2012.09.28 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US*/
const Device UA3100185 = {S69214, 226, { &P22787,&P22791,&P18246,&P18092 }};	/*Useragent - TECHAIN50_11B_HW (MRE\3.0.00(2500) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) Cherry_W7_T712V_M01_S11S_X Release/2012.11.23 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US*/
const Device UA2465154 = {S69215, 137, { &P20359,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZP300+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2530300 = {S69216, 142, { &P19759,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Lenovo K860 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2579681 = {S69217, 143, { &P19759,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Lenovo K860i Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2763065 = {S69218, 134, { &P19759,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Lenovo K860i Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2847985 = {S69219, 133, { &P19759,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Lenovo K860 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2672496 = {S69220, 33, { &P17346,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/6.1.3 (4; 8GB; dt:71)*/
const Device UA2705007 = {S69221, 34, { &P17346,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/6.1.3 (4; 32GB; dt:81)*/
const Device UA2712665 = {S69222, 34, { &P17346,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/6.1.3 (4; 32GB; dt:71)*/
const Device UA2780423 = {S69223, 34, { &P17346,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/6.1.3 (4; 16GB; dt:71)*/
const Device UA2794569 = {S69224, 34, { &P17346,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/6.1.3 (4; 64GB; dt:71)*/
const Device UA2989886 = {S69225, 34, { &P17346,&P19826,&P18333,&P18092 }};	/*Useragent - iTunes-iPod/6.1.3 (4; 64GB; dt:81)*/
const Device UA2566801 = {S69226, 137, { &P20226,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2601075 = {S69227, 138, { &P20226,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E975K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667470 = {S69228, 130, { &P20226,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2754058 = {S69229, 159, { &P13999,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; NexusHD2 Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2861551 = {S69230, 156, { &P13999,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; NexusHD2 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865477 = {S69231, 169, { &P13999,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; TouchWizMOD Build/TouchWizMOD_HD2; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2772414 = {S69232, 100, { &P15364,&P7133,&P22827,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; XBLWP7; ZuneWP7) UCBrowser/3.0.0.285*/
const Device UA2946174 = {S69233, 100, { &P15364,&P7133,&P22827,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; XBLWP7; ZuneWP7) UCBrowser/3.0.1.302*/
const Device UA22937 = {S69234, 61, { &P11857,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia6021/2.0 (04.90) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20730 = {S69235, 136, { &P15102,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Xoom MZ605 Build/H.6.1-29) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA20732 = {S69236, 131, { &P15102,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; MZ605 Build/H.6.1-32) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA20733 = {S69237, 133, { &P15102,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; MZ605 Build/H.6.1-32-1) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA20735 = {S69238, 131, { &P15102,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; MZ605 Build/H.6.1-17) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA43043 = {S69239, 134, { &P15102,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; pt-br; MZ605 Build/H.6.1-38.82) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA43048 = {S69240, 131, { &P15102,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; MZ605 Build/H.6.1-25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA58960 = {S69241, 133, { &P15102,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; pt-br; MZ605 Build/H.6.1-38.4) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA58963 = {S69242, 131, { &P15102,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; MZ605 Build/H.6.1-40) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA773687 = {S69243, 133, { &P15102,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; es-us; MZ605 Build/H.6.1-38.8) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA864759 = {S69244, 130, { &P22157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; VL-107 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257478 = {S69245, 130, { &P22157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VL-107 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2003679 = {S69246, 35, { &P15364,&P5690,&P21811,&P18092 }};	/*Useragent - Opera/10.53 (Windows NT 5.1; U; en)*/
const Device UA2239210 = {S69247, 132, { &P21831,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; M-MP720I Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2566754 = {S69248, 143, { &P19842,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Butterfly Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602285 = {S69249, 136, { &P19842,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Butterfly Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA3027653 = {S69250, 196, { &P22585,&P18598,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L720 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us*/
const Device UA3068393 = {S69251, 126, { &P15602,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A501 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA63339 = {S69252, 77, { &P13531,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-B5702/B5702JVIK1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2026275 = {S69253, 89, { &P17976,&P18066,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.3; Linux; Opera Tablet/ADY-X; U; ru) Presto/2.9.201 Version/11.50*/
const Device UA2476412 = {S69254, 92, { &P17976,&P18066,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.4; Linux; Opera Tablet/ADR-X; U; en-GB) Presto/2.9.201 Version/11.50*/
const Device UA2032478 = {S69255, 142, { &P18017,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; K1 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2384962 = {S69256, 142, { &P18017,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; K1 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2648679 = {S69257, 142, { &P18017,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; K1 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2535133 = {S69258, 135, { &P22940,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Xelio 7 pro Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2537213 = {S69259, 72, { &P17595,&P17295,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:20.0) Gecko/X Firefox/20.0*/
const Device UA2719030 = {S69260, 158, { &P17625,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST25i Build/6.0.B.3.188) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2784564 = {S69261, 158, { &P17625,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST25i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2956610 = {S69262, 158, { &P17625,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST25i Build/6.0.B.1.564) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2768402 = {S69263, 149, { &P23116,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Aqua.Active Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2974464 = {S69264, 137, { &P22958,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-US780 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975734 = {S69265, 153, { &P18016,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ThinkPad Tablet Build/ThinkPadTablet_A400_03) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3001672 = {S69266, 139, { &P23698,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I527 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3065803 = {S69267, 68, { &P23698,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SAMSUNG-SGH-I527 Build/JDQ39)*/
const Device UA3125287 = {S69268, 132, { &P23698,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-ca; SGH-I527M Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3029634 = {S69269, 113, { &P12897,&P5843,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-00/03.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3067194 = {S69270, 136, { &P22956,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P875 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA1994341 = {S69271, 138, { &P20974,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; W900 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2358441 = {S69272, 160, { &P12972,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T989 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567206 = {S69273, 138, { &P18641,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-M830 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566762 = {S69274, 137, { &P13074,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P930 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2624711 = {S69275, 132, { &P22043,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P5113 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3093522 = {S69276, 131, { &P22043,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5113 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2703359 = {S69277, 141, { &P23013,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AX2_Frenzy Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2740174 = {S69278, 141, { &P23226,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Videocon A15 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2939395 = {S69279, 140, { &P22598,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6506 Build/10.3.A.0.423) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3009499 = {S69280, 133, { &P22598,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6506 Build/10.3.A.0.423) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2974068 = {S69281, 140, { &P18084,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974469 = {S69282, 137, { &P22916,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E450f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975316 = {S69283, 136, { &P22916,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E450 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021355 = {S69284, 137, { &P22916,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E450g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024235 = {S69285, 137, { &P22916,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E450B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA50069 = {S69286, 143, { &P13189,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-gb; T-Mobile G1 Build/CRC37) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA67698 = {S69287, 143, { &P13189,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; T-Mobile G1 Build/CRB43) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA68197 = {S69288, 142, { &P13189,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; T-Mobile G1 Build/CRC1) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA88558 = {S69289, 143, { &P13189,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-gb; T-Mobile G1 Build/CRB17) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA1884548 = {S69290, 141, { &P13189,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; es-; T-Mobile G1 Build/CRB43) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA3118595 = {S69291, 143, { &P13189,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; T-Mobile G1 Build/CRB43) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari 525.20.1*/
const Device UA607771 = {S69292, 155, { &P17283,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB860 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA754886 = {S69293, 155, { &P17283,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB860 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA904971 = {S69294, 151, { &P17283,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB860 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3043432 = {S69295, 149, { &P17283,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB860 Build/MROM; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA687063 = {S69296, 59, { &P15364,&P18346,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10; rv:11.0) Gecko/X Firefox/11.0*/
const Device UA1993946 = {S69297, 135, { &P21752,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; M030 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034072 = {S69298, 126, { &P21752,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; M030 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2165168 = {S69299, 26, { &P18645,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-I747/100.40004*/
const Device UA2463658 = {S69300, 27, { &P18645,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-I747M/100.40004*/
const Device UA2700210 = {S69301, 26, { &P18645,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SPH-L710/100.40102*/
const Device UA2714653 = {S69302, 152, { &P18645,&P17484,&P17606,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/2.3;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/AT&amp-T;FBPN/com.facebook.katana;FBDV/SAMSUNG-SGH-I747;FBSV/4.1.1;]*/
const Device UA2486988 = {S69303, 147, { &P17926,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; AT100 Build/JZO54K; CyanogenMod-CM10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2508192 = {S69304, 91, { &P23545,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mauie800) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2701738 = {S69305, 88, { &P23545,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; id; mauie800) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2730160 = {S69306, 88, { &P23545,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; vi; mauie800) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2748972 = {S69307, 84, { &P23545,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mauie800) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0*/
const Device UA2987725 = {S69308, 91, { &P23545,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; ar-SA; mauie800) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2892202 = {S69309, 130, { &P23135,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; U30GT 2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA63935 = {S69310, 146, { &P12377,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Pulse Mini Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018076 = {S69311, 68, { &P12377,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Pulse Mini Build/ERE27)*/
const Device UA249262 = {S69312, 99, { &P15364,&P17188,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.187 Safari/535.1*/
const Device UA843060 = {S69313, 71, { &P20134,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;GT-S5830i;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA2685763 = {S69314, 120, { &P20134,&P5629,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.3.6;Zh_cn;GT-S5830i;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA732813 = {S69315, 115, { &P15786,&P5043,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-05/08.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.11*/
const Device UA2221401 = {S69316, 117, { &P15786,&P5043,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-05.1/08.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.11*/
const Device UA904601 = {S69317, 133, { &P17595,&P17296,&P22458,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5*/
const Device UA2260677 = {S69318, 138, { &P15241,&P5843,&P24668,&P18092 }};	/*Useragent - Nokia300/5.0 (07.44) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia300) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2717067 = {S69319, 92, { &P12581,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia500) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2753324 = {S69320, 151, { &P20017,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783603 = {S69321, 150, { &P20017,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3025163 = {S69322, 136, { &P23776,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGP321 Build/10.3.1.A.0.244) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3081607 = {S69323, 139, { &P18736,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One SV Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA64324 = {S69324, 105, { &P13317,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonTM717/R1FC Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3*/
const Device UA429053 = {S69325, 63, { &P15216,&P7149,&P15830,&P18092 }};	/*Useragent - BlackBerry8520/5.2.0.67 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA639194 = {S69326, 79, { &P17595,&P18120,&P20900,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; fr-FR; rv:1.0.2) Gecko/X Netscape/7.02*/
const Device UA683954 = {S69327, 65, { &P15364,&P18145,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD i386; rv:10.0.3) Gecko/X Firefox/10.0.3*/
const Device UA925742 = {S69328, 94, { &P11689,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; Nokia;N70)*/
const Device UA2184494 = {S69329, 85, { &P13677,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s5253) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2338148 = {S69330, 139, { &P20592,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-SU870 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2431069 = {S69331, 171, { &P18513,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2567244 = {S69332, 140, { &P18592,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One VX Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2580119 = {S69333, 138, { &P20124,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1;en-us; ZTE V970M Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30;*/
const Device UA2998232 = {S69334, 139, { &P20124,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZTE V970 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3045431 = {S69335, 61, { &P20124,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ZTE V970 Build/JRO03C)*/
const Device UA3046568 = {S69336, 139, { &P20124,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ZTE V970 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2714347 = {S69337, 153, { &P20175,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI 1S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2782193 = {S69338, 154, { &P20175,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI 1SC Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2868516 = {S69339, 141, { &P24168,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PAP4055DUO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878937 = {S69340, 138, { &P23227,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I467 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2899096 = {S69341, 156, { &P20146,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9082 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2974902 = {S69342, 130, { &P16044,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; TouchPad Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA369902 = {S69343, 135, { &P14967,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LG-P500 Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1990295 = {S69344, 137, { &P14967,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P500 Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026629 = {S69345, 138, { &P14967,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LG-P500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257652 = {S69346, 138, { &P14967,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-P500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2335974 = {S69347, 60, { &P14967,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-P500 Build/IMM76L)*/
const Device UA741947 = {S69348, 78, { &P15364,&P5690,&P20900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.0.1) Gecko/X Netscape/7.0*/
const Device UA2494628 = {S69349, 78, { &P15364,&P5690,&P20900,&P18092 }};	/*Useragent - Mozilla/7.0 (Windows; U; Windows NT 5.1; en-US; rv:0.9.2) Gecko/X Netscape/7.0*/
const Device UA2529880 = {S69350, 129, { &P21286,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U; Android 4.0.4; en-us; MP1010 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2580204 = {S69351, 142, { &P13123,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2664268 = {S69352, 142, { &P13123,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3099842 = {S69353, 139, { &P13123,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-gb; GT-S5360 Build/JELLY BEAN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2800666 = {S69354, 140, { &P15850,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-LU6200 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3043390 = {S69355, 62, { &P15850,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-lu6200 Build/JZO54K)*/
const Device UA2999662 = {S69356, 61, { &P24097,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGH-M919N Build/JDQ39)*/
const Device UA3008510 = {S69357, 139, { &P24097,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-M919N Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA8588 = {S69358, 91, { &P12420,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-808/010_02 1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA26942 = {S69359, 170, { &P12420,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-808/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A, Alcatel-OT-808/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA31371 = {S69360, 85, { &P12420,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-808A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA64413 = {S69361, 84, { &P12420,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-808/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA15011 = {S69362, 118, { &P13332,&P5208,&P3534,&P18092 }};	/*Useragent - TELECA-/2.0 (BREW 3.1.5; U; EN-US; SAMSUNG; SPH-M810; Teleca/Q05A/INT) MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA175882 = {S69363, 134, { &P13644,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PC36100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA194373 = {S69364, 134, { &P13644,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PC36100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA218947 = {S69365, 134, { &P13644,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA252555 = {S69366, 134, { &P13644,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PC36100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369020 = {S69367, 133, { &P13644,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; EVO 4G Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022153 = {S69368, 58, { &P13644,&P24694,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; PC36100 Build/MIUI)*/
const Device UA2588980 = {S69369, 126, { &P13644,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; EVO 4G Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA702941 = {S69370, 108, { &P18088,&P5690,&P17370,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/5.0 (Windows; U; Windows NT 5.1; en) AppleWebKit/886; U; en) Presto/2.4.15*/
const Device UA2473131 = {S69371, 134, { &P18088,&P5690,&P17370,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; Opera/9.80 (J2ME/MIDP; Opera Mini/5.0.18635/1030; U; en) Presto/2.4.15; ru) Presto/2.8.99 Version/11.10*/
const Device UA2494451 = {S69372, 115, { &P18088,&P5690,&P17370,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/5.0(Windows; U; Windows NT 5.1; en-US)/23.390; U; en) Presto/2.5.25 Version/10.54*/
const Device UA1994973 = {S69373, 142, { &P16074,&P5768,&P24668,&P18092 }};	/*Useragent - Nokia3110c/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia3110c) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA1997508 = {S69374, 152, { &P12731,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB855 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2193407 = {S69375, 152, { &P12731,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB855 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257976 = {S69376, 152, { &P12731,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MB855 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2362411 = {S69377, 152, { &P12731,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB855 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030561 = {S69378, 132, { &P19879,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I915 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2215478 = {S69379, 42, { &P15364,&P17486,&P3807,&P18092 }};	/*Useragent - Mot-WX292/18.01 Browser/Obigo/Q03C MMP/2.0*/
const Device UA2242665 = {S69380, 37, { &P15364,&P17486,&P3807,&P18092 }};	/*Useragent - MOT-WX416/03.03.0F Obigo/Q03C MMP/2.0*/
const Device UA2517728 = {S69381, 42, { &P15364,&P17486,&P3807,&P18092 }};	/*Useragent - Mot-WX294/18.01 Browser/Obigo/Q03C MMP/2.0*/
const Device UA2520076 = {S69382, 81, { &P15364,&P17486,&P3807,&P18092 }};	/*Useragent - Mot-WX306/1B.3 Browser/Obigo/Q03C MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2355159 = {S69383, 143, { &P21766,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; Fly IQ245 Plus Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2505837 = {S69384, 142, { &P21766,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; Fly IQ245Plus Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2400096 = {S69385, 137, { &P20119,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LIFETAB_S9714 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2505836 = {S69386, 70, { &P15775,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 800;7.10;8858)*/
const Device UA2607483 = {S69387, 70, { &P15775,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Nokia 800;7.10;8862)*/
const Device UA2607494 = {S69388, 70, { &P15775,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 800;7.10;8862)*/
const Device UA2780267 = {S69389, 70, { &P15775,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 800;7.10;8860)*/
const Device UA2672605 = {S69390, 139, { &P22865,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; AT-AS53N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3047855 = {S69391, 61, { &P22865,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; AT-AS53N Build/JZO54K)*/
const Device UA2706894 = {S69392, 141, { &P16292,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.1; Galaxy Nexus Build/ITL41F) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2711339 = {S69393, 141, { &P16292,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.2; Galaxy Nexus Build/ICL53F) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712198 = {S69394, 141, { &P16292,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2751508 = {S69395, 134, { &P16292,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA63242 = {S69396, 104, { &P14612,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-A777/A777UCIF2 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA68372 = {S69397, 137, { &P14924,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.1; ld-us; A70CHT Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA191575 = {S69398, 137, { &P14924,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.3; ld-us; A70CHT Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2369655 = {S69399, 133, { &P18464,&P18110,&P19803,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Safari/535.19 Silk-Accelerated=true*/
const Device UA2431077 = {S69400, 166, { &P18673,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2421478 = {S69401, 80, { &P20102,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-E1500/E1500JVLH6 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2509031 = {S69402, 135, { &P20865,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Scroll Plus Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2673343 = {S69403, 136, { &P16362,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-D710 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707239 = {S69404, 137, { &P16362,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SPH-D710 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3032818 = {S69405, 136, { &P23776,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGP321 Build/10.3.1.A.0.244) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2753053 = {S69406, 137, { &P21350,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8730 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879664 = {S69407, 138, { &P21350,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8730T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753791 = {S69408, 130, { &P19874,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-i705 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA49220 = {S69409, 160, { &P12962,&P6590,&P4626,&P18092 }};	/*Useragent - Mozilla/5.0 (HW-HUAWEI_U6150;U;REX/1.0;BREW/5.0.4.1;Profile/MIDP-2.0 Configuration/CLDC-1.1;320*240) Opera/9.70 (KHTML, like Gecko) Version/7.0 Mobile Opera/7.0*/
const Device UA575621 = {S69410, 118, { &P13035,&P5043,&P17438,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-02.1/so7.22; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.12*/
const Device UA623393 = {S69411, 115, { &P13035,&P5043,&P17438,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-02/06.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15*/
const Device UA754502 = {S69412, 155, { &P12482,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-T959 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA830922 = {S69413, 150, { &P21151,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LNV-Lenovo A790e Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027614 = {S69414, 143, { &P21151,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LNV-Lenovo A790e Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2028307 = {S69415, 138, { &P21151,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LNV-Lenovo A790e Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2520672 = {S69416, 74, { &P21151,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; LNV-Lenovo A790e Build/GINGERBREAD)*/
const Device UA2544244 = {S69417, 143, { &P21151,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; LNV-Lenovo A790e Build GINGERBREAD) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2432555 = {S69418, 127, { &P21282,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U; Android 4.0.4; en-us; MP700 Build/IMM76D) AppleWebKit/534.30 (KHTML,like Gecko) Version/4.0 Safari/534.30*/
const Device UA2456460 = {S69419, 157, { &P15397,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Ray Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2472076 = {S69420, 153, { &P19771,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035E Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2702564 = {S69421, 153, { &P19771,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035X Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2708224 = {S69422, 153, { &P19771,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2754961 = {S69423, 153, { &P19771,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2660310 = {S69424, 63, { &P15364,&P5448,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; 68K; Tablet PC; Trident/3.0)*/
const Device UA2660328 = {S69425, 58, { &P15364,&P5448,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; WindowsCE; Trident/2.0)*/
const Device UA2849947 = {S69426, 63, { &P15364,&P5448,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; PPC; Tablet PC; Trident/2.0)*/
const Device UA2773908 = {S69427, 157, { &P20146,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2798943 = {S69428, 130, { &P22918,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DA220HQL Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3034719 = {S69429, 167, { &P15438,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC Sensation Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/pyramid*/
const Device UA3062140 = {S69430, 167, { &P15438,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC Sensation Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/pyramid*/
const Device UA3115575 = {S69431, 167, { &P15438,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC Sensation Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/pyramid*/
const Device UA3128043 = {S69432, 167, { &P15438,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC Sensation Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/pyramid*/
const Device UA54648 = {S69433, 90, { &P11836,&P16000,&P3772,&P18092 }};	/*Useragent - SonyEricssonW960i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 768) Opera 8.65 [en]*/
const Device UA152386 = {S69434, 128, { &P15767,&P17141,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2_1 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B405 Twitter for iPad*/
const Device UA152872 = {S69435, 128, { &P15767,&P17141,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B500 Twitter for iPad*/
const Device UA292536 = {S69436, 126, { &P15767,&P17141,&P22702,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B367 Twitter for iPad*/
const Device UA692705 = {S69437, 80, { &P15364,&P18140,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9) Gecko/X Firefox/3.0 (Linux Mint)*/
const Device UA2269638 = {S69438, 86, { &P15364,&P18140,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; it-IT; rv:1.9.0.1) Gecko/X Firefox/3.0.1 (Linux Mint)*/
const Device UA2759007 = {S69439, 80, { &P15364,&P18140,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-GB; rv:1.9) Gecko/X Firefox/3.0 (Linux Mint)*/
const Device UA794921 = {S69440, 139, { &P12496,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  SGH-I897 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA905719 = {S69441, 138, { &P12496,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I897 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1982321 = {S69442, 138, { &P12496,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-I897 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA889908 = {S69443, 85, { &P13677,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s5253) UCBrowser8.3.0.154/69/352/UCWEB Mobile*/
const Device UA905382 = {S69444, 85, { &P13677,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5253) UCBrowser8.3.1.161/69/405/UCWEB Mobile*/
const Device UA900497 = {S69445, 132, { &P21425,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; STM803HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2030273 = {S69446, 133, { &P21425,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; STM803HCZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2204135 = {S69447, 129, { &P23929,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; M712 Build/HRA701S) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2362132 = {S69448, 128, { &P23929,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; M712 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2367236 = {S69449, 126, { &P23929,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; M712 Build/M712) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2658208 = {S69450, 135, { &P23929,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M712 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567331 = {S69451, 131, { &P18702,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8013 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2666989 = {S69452, 142, { &P20693,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; QMobile A12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2794699 = {S69453, 142, { &P20693,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; QMobile_A12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2755255 = {S69454, 130, { &P22037,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3113 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2783082 = {S69455, 130, { &P22037,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P3113 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3083192 = {S69456, 149, { &P18318,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF700T Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3068209 = {S69457, 137, { &P24098,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AS870 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065378 = {S69458, 136, { &P22955,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P714 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA623392 = {S69459, 99, { &P15364,&P17188,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA680129 = {S69460, 142, { &P17459,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i919 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1940880 = {S69461, 130, { &P17459,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i919 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2034486 = {S69462, 135, { &P17459,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i919 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2375758 = {S69463, 143, { &P17459,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I919U Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042582 = {S69464, 67, { &P17459,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-I919U Build/GINGERBREAD)*/
const Device UA3070259 = {S69465, 136, { &P17459,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I919U Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2027252 = {S69466, 142, { &P19897,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I779 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2413941 = {S69467, 135, { &P19897,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I779 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2789410 = {S69468, 66, { &P19897,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; SCH-I779 Build/GINGERBREAD)*/
const Device UA2029066 = {S69469, 158, { &P15932,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2389843 = {S69470, 91, { &P17595,&P17297,&P20457,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:17.0) Gecko/X Firefox/17.0 SeaMonkey/2.14.1*/
const Device UA2436190 = {S69471, 132, { &P23883,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; X10.Dual Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2460935 = {S69472, 133, { &P23883,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; X10.Dual+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2671478 = {S69473, 143, { &P18471,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT907 Build/7.7.1Q-144_SMQ_S136-10) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2674205 = {S69474, 150, { &P18471,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT907 Build/7.7.1Q-144_SMQ_S136-10) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676507 = {S69475, 143, { &P18471,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT905 Build/7.7.1Q_GCIRD-16) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735315 = {S69476, 136, { &P18471,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT905 Build/7.7.1Q_GCIRD-16) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2740519 = {S69477, 138, { &P22929,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_E10310 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2751983 = {S69478, 138, { &P20871,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810P Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3015763 = {S69479, 131, { &P20871,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810P Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2772926 = {S69480, 150, { &P23773,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8010 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2869249 = {S69481, 150, { &P23773,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N8010 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3064889 = {S69482, 137, { &P24391,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8260 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065730 = {S69483, 138, { &P22738,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT916 Build/2_26D_2014) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA191445 = {S69484, 143, { &P15364,&P5690,&P22402,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Comodo_Dragon/10.0.0.2 Chrome/10.0.648.204 Safari/534.16*/
const Device UA201718 = {S69485, 188, { &P15364,&P7133,&P4777,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/69/444*/
const Device UA324176 = {S69486, 188, { &P15364,&P7133,&P4777,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/28/444*/
const Device UA394231 = {S69487, 188, { &P15364,&P7133,&P4777,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/27/444*/
const Device UA394880 = {S69488, 188, { &P15364,&P7133,&P4777,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/50/444*/
const Device UA539099 = {S69489, 188, { &P15364,&P7133,&P4777,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/50/352*/
const Device UA550937 = {S69490, 188, { &P15364,&P7133,&P4777,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/50/351*/
const Device UA552377 = {S69491, 188, { &P15364,&P7133,&P4777,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.2.91/50/355*/
const Device UA650775 = {S69492, 188, { &P15364,&P7133,&P4777,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/28/351*/
const Device UA747228 = {S69493, 188, { &P15364,&P7133,&P4777,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/28/352*/
const Device UA827151 = {S69494, 188, { &P15364,&P7133,&P4777,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/27/351*/
const Device UA1882015 = {S69495, 188, { &P15364,&P7133,&P4777,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/27/352*/
const Device UA2949277 = {S69496, 188, { &P15364,&P7133,&P4777,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/50/354*/
const Device UA2949464 = {S69497, 188, { &P15364,&P7133,&P4777,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.7.1.88/50/350*/
const Device UA675538 = {S69498, 55, { &P15767,&P18570,&P18245,&P18092 }};	/*Useragent - Bing%20for%20iPad/1.1.3 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA567322 = {S69499, 143, { &P16541,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SHW-M220L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625759 = {S69500, 143, { &P16541,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHW-M220L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA546849 = {S69501, 97, { &P15364,&P17285,&P22479,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.8.1.14) Gecko/X Firefox/2.0.0.14 Flock/1.2.1*/
const Device UA646372 = {S69502, 143, { &P23849,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; MTS-SP101 Build/HuaweiC8511) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1937279 = {S69503, 137, { &P14917,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-KU5900 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021854 = {S69504, 61, { &P14917,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-KU5900 Build/GRJ22)*/
const Device UA703023 = {S69505, 113, { &P15364,&P17285,&P22467,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/528.5 (KHTML, like Gecko) Iron/0.4.155.0 Safari/528.5*/
const Device UA2294473 = {S69506, 113, { &P15364,&P17285,&P22467,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/528.4 (KHTML, like Gecko) Iron/0.3.155.0 Safari/X.528*/
const Device UA2309370 = {S69507, 114, { &P15364,&P17285,&P22467,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Iron/0.2.152.0 Safari/X.525*/
const Device UA854790 = {S69508, 79, { &P12004,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN95)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA886137 = {S69509, 83, { &P12004,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN95 8GB)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA2410881 = {S69510, 77, { &P12004,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian95) UCBrowser8.2.1.144/69/352/UCWEB Mobile*/
const Device UA2813647 = {S69511, 81, { &P12004,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian95_8gb) UCBrowser8.2.0.132/69/452/UCWEB Mobile*/
const Device UA2033104 = {S69512, 167, { &P18307,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Malata SMBA1002 Build/JRO03L; CyanogenMod-TEAMDRH) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556542 = {S69513, 162, { &P18307,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Malata SMBA1002 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2222531 = {S69514, 140, { &P14917,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-KU5900 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3070290 = {S69515, 62, { &P14917,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-KU5900 Build/IMM76L)*/
const Device UA2223646 = {S69516, 131, { &P21725,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; ATP7683 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2228410 = {S69517, 154, { &P12788,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PG86100 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2386700 = {S69518, 158, { &P12788,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Evo 3D CDMA Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2481426 = {S69519, 157, { &P12788,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Evo 3D GSM Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566942 = {S69520, 146, { &P19867,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I437 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2676324 = {S69521, 144, { &P19891,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I717 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2861026 = {S69522, 136, { &P19891,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I717 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2738205 = {S69523, 123, { &P15364,&P17285,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 Sleipnir/4.1.0*/
const Device UA2774139 = {S69524, 116, { &P15364,&P17285,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 Sleipnir/4.1.3*/
const Device UA2877567 = {S69525, 117, { &P15364,&P17285,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36 Sleipnir/4.1.4*/
const Device UA3019354 = {S69526, 124, { &P15364,&P17285,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36 Sleipnir/4.1.4*/
const Device UA2753011 = {S69527, 154, { &P12605,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754220 = {S69528, 130, { &P20968,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Le Pan S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2756738 = {S69529, 137, { &P18594,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LS860 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2872916 = {S69530, 133, { &P18594,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Mach Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2967025 = {S69531, 93, { &P15373,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5530) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA2958170 = {S69532, 136, { &P24039,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9190 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3019805 = {S69533, 129, { &P24039,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9190 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3067195 = {S69534, 130, { &P15985,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094741 = {S69535, 131, { &P15985,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3079819 = {S69536, 156, { &P18673,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S7562 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA64113 = {S69537, 117, { &P12423,&P17485,&P3976,&P18092 }};	/*Useragent - Mozilla/4.0 SonyEricssonW715v/R1GA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.4*/
const Device UA399157 = {S69538, 101, { &P18176,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 900)*/
const Device UA1959910 = {S69539, 131, { &P20894,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; M3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2810990 = {S69540, 142, { &P20894,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; M3 media box board Build/X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2815387 = {S69541, 130, { &P20894,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; m3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2937769 = {S69542, 145, { &P20894,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; M3 Atom Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2404092 = {S69543, 136, { &P23414,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Aqua Sx Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2460882 = {S69544, 159, { &P14708,&P5204,&P24675,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia5130c-2) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2848342 = {S69545, 153, { &P14708,&P5204,&P24675,&P18092 }};	/*Useragent - Nokia5130/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia5130) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2470259 = {S69546, 133, { &P20244,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Elcano Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2530055 = {S69547, 64, { &P15364,&P7133,&P18689,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:19.0) Gecko/X Firefox/19.0 slurp*/
const Device UA2535616 = {S69548, 71, { &P15364,&P7133,&P18689,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:19.0) Gecko/X Firefox/19.0 slurp*/
const Device UA2582758 = {S69549, 77, { &P15364,&P7133,&P18689,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:19.0) Gecko/X Firefox/19.0 slurp slurp*/
const Device UA2755432 = {S69550, 140, { &P18449,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST27i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769844 = {S69551, 140, { &P18449,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST27a Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3132024 = {S69552, 130, { &P18702,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8013 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA233536 = {S69553, 143, { &P17595,&P17296,&P22492,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.3.4641 Chrome/7.0.517.450 Safari/534.7*/
const Device UA299938 = {S69554, 143, { &P17595,&P17296,&P22492,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.3.4628 Chrome/7.0.517.450 Safari/534.7*/
const Device UA312510 = {S69555, 143, { &P17595,&P17296,&P22492,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.3.4628 Chrome/7.0.517.450 Safari/534.7*/
const Device UA914496 = {S69556, 143, { &P17595,&P17296,&P22492,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.0.4568 Chrome/7.0.517.440 Safari/534.7*/
const Device UA658466 = {S69557, 43, { &P13552,&P5129,&P24665,&P18092 }};	/*Useragent - NokiaC6-00/UC Browser8.0.4.121/50/444/UCWEB*/
const Device UA704320 = {S69558, 31, { &P13552,&P5129,&P24665,&P18092 }};	/*Useragent - NokiaC6-00/UCWEB8.0.3.99/50/800*/
const Device UA2383641 = {S69559, 118, { &P13163,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaN97) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/444/UCWEB Mobile*/
const Device UA2843085 = {S69560, 119, { &P13163,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian97-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2401382 = {S69561, 153, { &P23448,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; CROSS A10 Build/VENUS_00.03.109.H1059) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2579794 = {S69562, 153, { &P23448,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; CROSS A10 Build/VENUS_00.03.133.H1059) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2697655 = {S69563, 153, { &P23448,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; CROSS A10 Build/VENUS_00.03.161.H1059) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2704717 = {S69564, 159, { &P23448,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; CROSS A10 Build/VENUS_00.03.161.25.02.H1059) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2799015 = {S69565, 137, { &P22970,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2901595 = {S69566, 138, { &P22970,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2961810 = {S69567, 138, { &P22970,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2975741 = {S69568, 136, { &P14123,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P970 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021375 = {S69569, 137, { &P14123,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P970h Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3067661 = {S69570, 152, { &P22785,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire 600 dual sim Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093687 = {S69571, 143, { &P22785,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire 600 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA10137 = {S69572, 139, { &P18046,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; Commtiva-N700 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA21509 = {S69573, 141, { &P18046,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Commtiva-N700 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52401 = {S69574, 142, { &P18046,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Commtiva-N700 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018524 = {S69575, 63, { &P18046,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; Commtiva-N700 Build/FRF91)*/
const Device UA54468 = {S69576, 50, { &P14221,&P6476,&P3487,&P18092 }};	/*Useragent - LGE-VM265/1.0 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0*/
const Device UA69377 = {S69577, 30, { &P11929,&P5129,&P24663,&P18092 }};	/*Useragent - Nokia5230/UCWEB7.9.0.94/50/999*/
const Device UA249004 = {S69578, 42, { &P11929,&P5129,&P24663,&P18092 }};	/*Useragent - Nokia5230/UC Browser7.9.0.102/50/355/UCWEB*/
const Device UA778009 = {S69579, 44, { &P11929,&P5129,&P24663,&P18092 }};	/*Useragent - Nokia5230-c/UC Browser7.9.0.102/50/355/UCWEB*/
const Device UA765007 = {S69580, 119, { &P17595,&P17296,&P22504,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en; rv:1.9.0.19) Gecko/X Camino/2.0.8 (MultiLang) (like Firefox/3.0.19)*/
const Device UA691830 = {S69581, 32, { &P12088,&P5129,&P24667,&P18092 }};	/*Useragent - NokiaX6-00/UCWEB8.2.0.116/50/800*/
const Device UA773193 = {S69582, 81, { &P12088,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaX6-00)/UC Browser8.2.0.132/50/355/UCWEB Mobile*/
const Device UA631920 = {S69583, 139, { &P21013,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; INFOBAR C01 Build/S4030) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA677917 = {S69584, 139, { &P21013,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; INFOBAR C01 Build/S2200) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1952892 = {S69585, 139, { &P21013,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; INFOBAR C01 Build/S1190) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995597 = {S69586, 139, { &P21013,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; INFOBAR C01 Build/S6290) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2605842 = {S69587, 139, { &P21013,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; INFOBAR C01 Build/S2270) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA685500 = {S69588, 140, { &P22210,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-pt; tmn smart a7 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2367629 = {S69589, 120, { &P13324,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaN8-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile*/
const Device UA2436244 = {S69590, 57, { &P15364,&P7127,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0) Presto/2.12.388 Version/12.12*/
const Device UA2557858 = {S69591, 69, { &P15364,&P7127,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; Edition Yx) Presto/2.12.388 Version/12.12*/
const Device UA2734175 = {S69592, 171, { &P18442,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L900 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SPH-L900, 42, 1.9.1)*/
const Device UA2734239 = {S69593, 187, { &P18442,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SAMSUNG-SGH-I317, 42, 1.9.1)*/
const Device UA2767315 = {S69594, 171, { &P18442,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T889 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SGH-T889, 42, 1.9.1)*/
const Device UA2990076 = {S69595, 173, { &P18442,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I317M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SGH-I317M, 42, 1.9.1)*/
const Device UA2716334 = {S69596, 144, { &P15767,&P17146,&P22593,&P18092 }};	/*Useragent - Mozilla/5.0(iPad; U;CPU OS 5 like Mac OS X; zh-CN; iPad2,1) AppleWebKit/534.46 (KHTML, like Gecko) UCBrowser/2.0.1.280 U3/0.8.0 Safari/7543.48.3*/
const Device UA2774156 = {S69597, 157, { &P18283,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3050252 = {S69598, 157, { &P18283,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I535 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3019688 = {S69599, 138, { &P22948,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; LG-P716 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049216 = {S69600, 60, { &P22948,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P716 Build/JZO54K)*/
const Device UA2901179 = {S69601, 152, { &P18644,&P5043,&P24678,&P18092 }};	/*Useragent - Nokia112/2.0 (03.32) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia112) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3131472 = {S69602, 137, { &P22584,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337M Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA54022 = {S69603, 90, { &P11860,&P5776,&P3756,&P18092 }};	/*Useragent - Nokia6681/2.0 (6.09.00) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA337647 = {S69604, 63, { &P15364,&P16110,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; U; en) Presto/2.9.168 Version/11.51*/
const Device UA594698 = {S69605, 137, { &P14922,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; LG-LU6800 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031074 = {S69606, 61, { &P14922,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-LU6800 Build/GRJ22)*/
const Device UA672690 = {S69607, 83, { &P16455,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-S5610K/S5610KDDKK2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA2152821 = {S69608, 57, { &P16455,&P17484,&P17606,&P18092 }};	/*Useragent - Samsung5610/3.83; Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA3098828 = {S69609, 81, { &P16455,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-S5610/S5610XXLF4 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA2343375 = {S69610, 142, { &P14714,&P5843,&P24668,&P18092 }};	/*Useragent - NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax3-02) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2652409 = {S69611, 146, { &P14714,&P5843,&P24668,&P18092 }};	/*Useragent - NokiaX3-02.5/5.0 (06.75) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax3-02.5) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2367223 = {S69612, 43, { &P15364,&P18570,&P22899,&P18092 }};	/*Useragent - Mercury/7.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2388550 = {S69613, 141, { &P20089,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SO-01E Build/9.0.G.0.247) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2566620 = {S69614, 137, { &P16356,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2606247 = {S69615, 130, { &P16356,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667034 = {S69616, 138, { &P16356,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-N7000 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2558008 = {S69617, 127, { &P23961,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-TSM_T3/1.0 Linux/2.6.35.7 Android/2.3.7 Release/01.07.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2667864 = {S69618, 127, { &P23961,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-TSM_T3/1.0 Linux/2.6.35.7 Android/2.3.7 Release/02.28.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2749822 = {S69619, 127, { &P23961,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-TSM_T3/1.0 Linux/2.6.35.7 Android/2.3.7 Release/01.23.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2781884 = {S69620, 127, { &P23961,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-TSM_T3/1.0 Linux/2.6.35.7 Android/2.3.7 Release/04.22.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2849749 = {S69621, 144, { &P23961,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-au; TSM_T3 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2867419 = {S69622, 127, { &P23961,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-TSM_T3/1.0 Linux/2.6.35.7 Android/2.3.7 Release/05.03.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2995506 = {S69623, 127, { &P23961,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-TSM_T3/1.0 Linux/2.6.35.7 Android/2.3.7 Release/04.07.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3054667 = {S69624, 127, { &P23961,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-TSM_T3/1.0 Linux/2.6.35.7 Android/2.3.7 Release/06.09.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2562875 = {S69625, 132, { &P22206,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; M-MP855i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3028268 = {S69626, 196, { &P22594,&P18598,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-M919 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us*/
const Device UA2968004 = {S69627, 94, { &P12088,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaX6-00) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA2974038 = {S69628, 133, { &P17482,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80G9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022701 = {S69629, 130, { &P17482,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 80G9 Build/MR1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974730 = {S69630, 130, { &P12889,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3083442 = {S69631, 137, { &P17172,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Transformer TF101 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA523332 = {S69632, 133, { &P18032,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SK-S100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA539347 = {S69633, 136, { &P18032,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SK-S100 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019172 = {S69634, 60, { &P18032,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; SK-S100 Build/FRG83G)*/
const Device UA610105 = {S69635, 99, { &P17595,&P17296,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:6.0.2) Gecko/X Firefox/6.0.2 AlexaToolbar/alxf-2.15*/
const Device UA727238 = {S69636, 90, { &P17595,&P17296,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:6.0.2) Gecko/X Firefox/6.0.2 Firefox/6.0.2*/
const Device UA2198885 = {S69637, 140, { &P15147,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST15i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2338459 = {S69638, 140, { &P15147,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST15a Build/4.1.B.0.431) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1993086 = {S69639, 131, { &P21704,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; PP4MT-9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2403740 = {S69640, 133, { &P22156,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GEM10312S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2429923 = {S69641, 133, { &P22156,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GEM10312S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2427025 = {S69642, 35, { &P14708,&P5204,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia5130c-2/07.95)*/
const Device UA2430986 = {S69643, 132, { &P20425,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; N-03E Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2992202 = {S69644, 54, { &P20425,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; N-03E Build/AX)*/
const Device UA2471322 = {S69645, 77, { &P15364,&P17285,&P22556,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:17.0) Gecko/X Firefox/17.0 IceDragon/18.0.1.0*/
const Device UA2490856 = {S69646, 77, { &P15364,&P17285,&P22556,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:18.0) Gecko/X Firefox/18.0 IceDragon/18.0.3.1*/
const Device UA2510707 = {S69647, 135, { &P21879,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; PAD-FMD700P Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2714037 = {S69648, 136, { &P21879,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; PAD-FMD700HD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2529868 = {S69649, 142, { &P23257,&P18066,&P18062,&P18092 }};	/*Useragent - HUAWEI_T8833_TD/1.0 Android/4.0.4 (Linux; U; Android 4.0.4; zh-cn) Release/05.31.2012 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30*/
const Device UA2746524 = {S69650, 148, { &P23257,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8833 Build/HuaweiT8833) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778910 = {S69651, 139, { &P23257,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8833 Build/HuaweiT8833) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2988037 = {S69652, 70, { &P23257,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HUAWEI T8833 Build/HuaweiT8833)*/
const Device UA2716790 = {S69653, 139, { &P20517,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CINK PEAX Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2754643 = {S69654, 139, { &P23855,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754650 = {S69655, 132, { &P23855,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2974883 = {S69656, 130, { &P15985,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021429 = {S69657, 131, { &P15985,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3068063 = {S69658, 136, { &P20587,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P715 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3096306 = {S69659, 112, { &P15241,&P5843,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia300/07.57; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA31803 = {S69660, 100, { &P11696,&P5448,&P3772,&P18092 }};	/*Useragent - MOT-Q9 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 320x240/01.06.08R) Opera 8.65 [en]*/
const Device UA61851 = {S69661, 95, { &P11696,&P5448,&P3772,&P18092 }};	/*Useragent - MOT-Q9/01.09.28R Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 320x240) Opera 8.65*/
const Device UA62405 = {S69662, 95, { &P11696,&P5448,&P3772,&P18092 }};	/*Useragent - MOT-Q9/01.07.37R Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 240x320) Opera 8.65*/
const Device UA380051 = {S69663, 195, { &P15364,&P7133,&P24665,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.4.121/28/351/UCWEB*/
const Device UA397476 = {S69664, 195, { &P15364,&P7133,&P24665,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.4.121/50/351/UCWEB*/
const Device UA453495 = {S69665, 195, { &P15364,&P7133,&P24665,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.4.121/28/352/UCWEB*/
const Device UA633080 = {S69666, 195, { &P15364,&P7133,&P24665,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/50/444/UCWEB*/
const Device UA655014 = {S69667, 195, { &P15364,&P7133,&P24665,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/50/355/UCWEB*/
const Device UA659717 = {S69668, 195, { &P15364,&P7133,&P24665,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.4.121/50/444/UCWEB*/
const Device UA686531 = {S69669, 195, { &P15364,&P7133,&P24665,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/28/444/UCWEB*/
const Device UA690192 = {S69670, 195, { &P15364,&P7133,&P24665,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/50/351/UCWEB*/
const Device UA695387 = {S69671, 195, { &P15364,&P7133,&P24665,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/28/352/UCWEB*/
const Device UA699830 = {S69672, 195, { &P15364,&P7133,&P24665,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/50/354/UCWEB*/
const Device UA709577 = {S69673, 195, { &P15364,&P7133,&P24665,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/28/354/UCWEB*/
const Device UA721620 = {S69674, 195, { &P15364,&P7133,&P24665,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.4.121/50/352/UCWEB*/
const Device UA743971 = {S69675, 195, { &P15364,&P7133,&P24665,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/50/352/UCWEB*/
const Device UA787081 = {S69676, 195, { &P15364,&P7133,&P24665,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.4.121/28/444/UCWEB*/
const Device UA848830 = {S69677, 195, { &P15364,&P7133,&P24665,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.4.121/50/355/UCWEB*/
const Device UA1966075 = {S69678, 139, { &P21827,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; ADM712HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2385681 = {S69679, 139, { &P20596,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Luna TAB274 Build/LunaTAB274) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2427623 = {S69680, 103, { &P12825,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.32.0.88; en-us; 9380/9.32.0.88) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2595356 = {S69681, 103, { &P12825,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.32.0.51; en-us; 9380/9.32.0.51) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2653350 = {S69682, 103, { &P12825,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.32.0.59; en-us; 9380/9.32.0.59) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2746573 = {S69683, 103, { &P12825,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.32.0.40; en-us; 9380/9.32.0.40) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2843961 = {S69684, 105, { &P12825,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.32.0.143; en-us; 9380/9.32.0.143) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2951086 = {S69685, 105, { &P12825,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.32.0.114; en-us; 9380/9.32.0.114) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3071948 = {S69686, 105, { &P12825,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 9.32.0.144; en-us; 9380/9.32.0.144) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2603721 = {S69687, 143, { &P21864,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; POV_TAB-P701 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2678440 = {S69688, 78, { &P15364,&P17285,&P22560,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:20.0) Gecko/X Firefox/20.0 IceDragon/20.0.1.14*/
const Device UA2975953 = {S69689, 136, { &P18234,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P895 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022836 = {S69690, 138, { &P18234,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P895qb Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA180625 = {S69691, 135, { &P18040,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; YP-GB70 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA564977 = {S69692, 59, { &P18040,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; YP-GB70 Build/FROYO)*/
const Device UA1870253 = {S69693, 135, { &P18040,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; YP-GB70 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA142536 = {S69694, 87, { &P18254,&P7480,&P17191,&P18091 }};	/*Useragent - NewzCrawler/1.9 (compatible; MSIE 6.00; Newz Crawler 1.9; http://www.newzcrawler.com/ )*/
const Device UA394690 = {S69695, 63, { &P18254,&P7480,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Sitekit.net Linkchecker Bot;*/
const Device UA517788 = {S69696, 66, { &P18254,&P7480,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0) WormlyBot; (+http://wormly.com)*/
const Device UA1894933 = {S69697, 100, { &P18254,&P7480,&P17191,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; BecomeBot/2.x; MSIE 6.0 compatible; http://www.become.com/site_owners.html)*/
const Device UA1895352 = {S69698, 78, { &P18254,&P7480,&P17191,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Podtech Network; crawler_admin@podtech.net)*/
const Device UA1899197 = {S69699, 105, { &P18254,&P7480,&P17191,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; BecomeJPBot/2.3; MSIE 6.0 compatible; +http://www.become.co.jp/site_owners.html)*/
const Device UA1905606 = {S69700, 100, { &P18254,&P7480,&P17191,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; BecomeBot/1.xx; MSIE 6.0 compatible; http://www.become.com/webmasters.html)*/
const Device UA1946886 = {S69701, 50, { &P18254,&P7480,&P17191,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; iaskspider/1.0; MSIE 6.0)*/
const Device UA3042165 = {S69702, 65, { &P18254,&P7480,&P17191,&P18091 }};	/*Useragent - IRLbot/2.0 (compatible; MSIE 6.0; http://irl.cs.tamu.edu/crawler)*/
const Device UA243039 = {S69703, 34, { &P15364,&P7127,&P17215,&P18092 }};	/*Useragent - Opera/9.52 (Windows NT 5.0; U; sv)*/
const Device UA302841 = {S69704, 34, { &P15364,&P7127,&P17215,&P18092 }};	/*Useragent - Opera/9.51 (Windows NT 5.0; U; en)*/
const Device UA522453 = {S69705, 34, { &P15364,&P7127,&P17215,&P18092 }};	/*Useragent - Opera/9.50 (Windows NT 5.0; U; en)*/
const Device UA652795 = {S69706, 106, { &P18095,&P18094,&P24515,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/9.80 (S60; SymbOS; Opera Mobi/23.348; U; en) Presto/2.5.25 Version/10.54*/
const Device UA895460 = {S69707, 130, { &P21018,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MID74C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2734846 = {S69708, 138, { &P20711,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID1065 Build/ICS.MID1065.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3031632 = {S69709, 159, { &P18223,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3067058 = {S69710, 152, { &P22963,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA571324 = {S69711, 114, { &P15364,&P18145,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ Epiphany/2.30.6*/
const Device UA632517 = {S69712, 137, { &P20465,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; 101K Build/155.0.0228) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA734060 = {S69713, 137, { &P20465,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; 101K Build/158.2.0237) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA748102 = {S69714, 137, { &P20465,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; 101K Build/159.3.0247) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1996927 = {S69715, 137, { &P20465,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; 101K Build/160.4.0252) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2476081 = {S69716, 61, { &P20465,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; 101K Build/160.4.0252)*/
const Device UA2707565 = {S69717, 137, { &P20465,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; 101K Build/161.5.0254) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3059681 = {S69718, 137, { &P20465,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; 101K Build/162.6.0256) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA696104 = {S69719, 47, { &P15364,&P5690,&P4809,&P18092 }};	/*Useragent - Opera/5.40 (Windows NT 5.1; U; pl) Version/10.5*/
const Device UA1873836 = {S69720, 47, { &P15364,&P5690,&P4809,&P18092 }};	/*Useragent - Opera/5.10 (Windows NT 5.1; U; pl) Version/10.5*/
const Device UA1900198 = {S69721, 62, { &P15364,&P5690,&P4809,&P18092 }};	/*Useragent - Opera/5.30 (Windows NT 5.1; U; pl) Presto/2.3.15 Version/10.10*/
const Device UA2133012 = {S69722, 47, { &P15364,&P5690,&P4809,&P18092 }};	/*Useragent - Opera/5.30 (Windows NT 5.1; U; pl) Version/10.5*/
const Device UA2148620 = {S69723, 47, { &P15364,&P5690,&P4809,&P18092 }};	/*Useragent - Opera/5.20 (Windows NT 5.1; U; pl) Version/10.5*/
const Device UA2281471 = {S69724, 62, { &P15364,&P5690,&P4809,&P18092 }};	/*Useragent - Opera/5.10 (Windows NT 5.1; U; pl) Presto/2.1.15 Version/10.10*/
const Device UA2289507 = {S69725, 62, { &P15364,&P5690,&P4809,&P18092 }};	/*Useragent - Opera/5.40 (Windows NT 5.1; U; pl) Presto/2.4.15 Version/10.10*/
const Device UA2462901 = {S69726, 62, { &P15364,&P5690,&P4809,&P18092 }};	/*Useragent - Opera/5.20 (Windows NT 5.1; U; pl) Presto/2.2.15 Version/10.10*/
const Device UA2383732 = {S69727, 66, { &P12616,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; WT19i; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2456573 = {S69728, 66, { &P12616,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; WT19i; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2486881 = {S69729, 134, { &P18680,&P18110,&P19838,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Safari/535.19 Silk-Accelerated=false*/
const Device UA2566930 = {S69730, 138, { &P18484,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-US730 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2620703 = {S69731, 77, { &P14908,&P18111,&P18112,&P18092 }};	/*Useragent - AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; GT-P6210 Build/JRO03H)*/
const Device UA2620704 = {S69732, 132, { &P14908,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-ca; GT-P6210 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2623754 = {S69733, 139, { &P14908,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P6210 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2628351 = {S69734, 61, { &P14908,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P6210 Build/JZO54K)*/
const Device UA2637508 = {S69735, 132, { &P14908,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; GT-P6210 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2755009 = {S69736, 132, { &P14908,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P6210 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2754716 = {S69737, 138, { &P18290,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ617 Build/7.7.1-85_MZ617-27) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755702 = {S69738, 145, { &P19891,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I717 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3036658 = {S69739, 93, { &P13123,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5360) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3080515 = {S69740, 90, { &P13123,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; vi; GT-S5360) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3080544 = {S69741, 137, { &P22872,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5282 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA241877 = {S69742, 136, { &P12973,&P7305,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.6.0; md-us; A101C Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA420175 = {S69743, 89, { &P15364,&P22725,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (MeeGo; U; en-AU) AppleWebKit/534.3 (KHTML, like Gecko) Qt/4.7.4 Safari/534.3*/
const Device UA557545 = {S69744, 89, { &P15364,&P22725,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (MeeGo; U; fi-FI) AppleWebKit/534.3 (KHTML, like Gecko) Qt/4.7.4 Safari/534.3*/
const Device UA574679 = {S69745, 89, { &P15364,&P22725,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (MeeGo; U; en-GB) AppleWebKit/534.3 (KHTML, like Gecko) Qt/4.7.4 Safari/534.3*/
const Device UA625731 = {S69746, 88, { &P15364,&P22725,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (MeeGo) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.7.4 Mobile Safari/534.34*/
const Device UA709214 = {S69747, 89, { &P15364,&P22725,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (MeeGo; U; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Qt/4.7.4 Safari/534.3*/
const Device UA2804572 = {S69748, 89, { &P15364,&P22725,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (MeeGo; U; en-ZA) AppleWebKit/534.3 (KHTML, like Gecko) Qt/4.7.4 Safari/534.3*/
const Device UA2029164 = {S69749, 109, { &P21094,&P17979,&P18062,&P18092 }};	/*Useragent - Lenovo A789/S210 Linux/3.0.13 Android/4.0 Release/05.11.2012 Browser/AppleWebKit533.1 Profile/ Configuration/*/
const Device UA2352935 = {S69750, 138, { &P20173,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; vi-vn; Limo X2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469079 = {S69751, 150, { &P20173,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1_jelly_bean; vi-vn; Limo X2+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2835685 = {S69752, 138, { &P20173,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; en-us; Limo X2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3011241 = {S69753, 139, { &P20173,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; Limo X2+ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2365869 = {S69754, 132, { &P21012,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I815 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2700405 = {S69755, 63, { &P15364,&P5690,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; Windows NT 5.2; Trident/5.0)*/
const Device UA2792541 = {S69756, 63, { &P15364,&P5690,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; Windows NT 5.2; Trident/1.0)*/
const Device UA2792679 = {S69757, 63, { &P15364,&P5690,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; Windows NT 5.1; Trident/3.0)*/
const Device UA2851533 = {S69758, 63, { &P15364,&P5690,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; Windows NT 5.1; Trident/3.0)*/
const Device UA2754350 = {S69759, 137, { &P18484,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-US730 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774400 = {S69760, 136, { &P22871,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9295 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3018944 = {S69761, 129, { &P22871,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9295 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3033862 = {S69762, 159, { &P16292,&P21637,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Galaxy Nexus Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA201065 = {S69763, 105, { &P15364,&P24094,&P17335,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux mips; en) AppleWebKit/531.2+ (KHTML, like Gecko) Version/4.1.4 Safari/531.2+*/
const Device UA591957 = {S69764, 89, { &P15364,&P7133,&P22354,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:9.2) Gecko/X Firefox/9.2-x64 PaleMoon/9.2-x64*/
const Device UA656163 = {S69765, 69, { &P15364,&P7133,&P22354,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:9.2) Gecko/X Firefox/9.2 PaleMoon/9.2*/
const Device UA908132 = {S69766, 115, { &P14711,&P5204,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-01/11.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA892619 = {S69767, 81, { &P12193,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-in; NokiaC5-00)/UC Browser8.4.0.159/28/352/UCWEB Mobile*/
const Device UA896282 = {S69768, 83, { &P12193,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-in; NokiaC5-00.2)/UC Browser8.4.0.159/28/352/UCWEB Mobile*/
const Device UA2420877 = {S69769, 91, { &P12193,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokiac5-00.2) UCBrowser8.4.0.159/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA2652829 = {S69770, 89, { &P12193,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokiac5-00) UCBrowser8.4.0.159/70/352/UCWEB Mobile  3gpp-gba*/
const Device UA1994715 = {S69771, 28, { &P11860,&P5776,&P21946,&P18092 }};	/*Useragent - MQQBrowser/1.3,Nokia6681/2.0*/
const Device UA2479944 = {S69772, 129, { &P20593,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; U9GT2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2504554 = {S69773, 129, { &P20593,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; U9GT4 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2659345 = {S69774, 130, { &P20593,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; U9GT V Build/U9GT V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2728178 = {S69775, 134, { &P20593,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Cube U9GT2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2995696 = {S69776, 137, { &P20593,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; U9GT V_Core 4 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2666923 = {S69777, 71, { &P15364,&P7133,&P22167,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:20.0) Gecko/X Firefox/20.0 slurp*/
const Device UA2668894 = {S69778, 64, { &P15364,&P7133,&P22167,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:20.0) Gecko/X Firefox/20.0 slurp*/
const Device UA2675778 = {S69779, 77, { &P15364,&P7133,&P22167,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:20.0) Gecko/X Firefox/20.0 slurp slurp*/
const Device UA2732133 = {S69780, 153, { &P23239,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030X Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2795275 = {S69781, 160, { &P23239,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL_ONE_TOUCH_6030X_Orange Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2976696 = {S69782, 137, { &P18484,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-US730 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3083292 = {S69783, 138, { &P23705,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C811 4G Build/C811M020) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA214126 = {S69784, 144, { &P13042,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_IncredibleS_S710e; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA251103 = {S69785, 64, { &P15364,&P18110,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/10.54*/
const Device UA416728 = {S69786, 66, { &P15364,&P18110,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; en-GB) Presto/2.5.24 Version/10.53*/
const Device UA1888028 = {S69787, 58, { &P15364,&P18110,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Linux i686; U; en) Presto/2.5.22 Version/10.51*/
const Device UA409569 = {S69788, 135, { &P18271,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM009SH Build/S4070) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA574450 = {S69789, 135, { &P18271,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM009SH Build/S4030) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA613533 = {S69790, 135, { &P18271,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM009SH Build/S4080) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA631231 = {S69791, 135, { &P18271,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM009SH Build/S4040) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024662 = {S69792, 59, { &P18271,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; DM009SH Build/S4080)*/
const Device UA2716982 = {S69793, 135, { &P18271,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM009SH Build/S4090) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2870159 = {S69794, 135, { &P18271,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM009SH Build/S4110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA660666 = {S69795, 100, { &P14669,&P5843,&P17347,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (08.25) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-00/UC Browser8.0.3.107/69/444*/
const Device UA584321 = {S69796, 135, { &P16534,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Advent Vega Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA675762 = {S69797, 128, { &P16534,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Vega Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA673875 = {S69798, 63, { &P15364,&P18346,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10; rv:10.0.2) Gecko/X Firefox/10.0.2*/
const Device UA766699 = {S69799, 139, { &P17968,&P24691,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; aokp_tenderloin Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA814074 = {S69800, 139, { &P17968,&P24691,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; aokp_tenderloin Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2024233 = {S69801, 138, { &P17968,&P24691,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; aokp_tenderloin Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA244926 = {S69802, 113, { &P15364,&P17285,&P22452,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1*/
const Device UA342113 = {S69803, 120, { &P15364,&P17285,&P22452,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1*/
const Device UA684225 = {S69804, 147, { &P13286,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID2 Build/VZW; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA751806 = {S69805, 161, { &P16757,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SCH-I500 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA743887 = {S69806, 142, { &P17933,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-R915 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024328 = {S69807, 142, { &P17933,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-R915 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1961175 = {S69808, 119, { &P21242,&P5629,&P15760,&P18092 }};	/*Useragent - Lenovo-A366t_TD/S100 Linux/2.6.35 Android/2.3 Release/11.01.2011 Browser/AppleWebkit533.1 Mobile Safari/533.1 MBBMS/2.0*/
const Device UA2027610 = {S69809, 134, { &P21242,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A366t Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2031564 = {S69810, 129, { &P21242,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A366t Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2032082 = {S69811, 141, { &P21242,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A366t Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2412270 = {S69812, 134, { &P21242,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A366t Build GRK39F) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2743656 = {S69813, 65, { &P21242,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; Lenovo A366t Build/GRK39F)*/
const Device UA1976851 = {S69814, 67, { &P17976,&P17979,&P21457,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:14.0) Gecko/14.0 K9Browser/14.0.b5*/
const Device UA2204795 = {S69815, 66, { &P17976,&P17979,&P21457,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:14.0) Gecko/14.0 K9Browser/14.0.1*/
const Device UA2403425 = {S69816, 122, { &P22695,&P22283,&P17394,&P18092 }};	/*Useragent - NetFront/4.2 (BREW 1.0.4; U; en-us; Samsung; NetFront/4.2/WAP) Sprint M400 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2423289 = {S69817, 123, { &P19976,&P5055,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; NokiaE6-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile*/
const Device UA2458865 = {S69818, 129, { &P20127,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AT270 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2513235 = {S69819, 105, { &P15364,&P18145,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2532348 = {S69820, 104, { &P15364,&P18145,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA2567474 = {S69821, 131, { &P19874,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-i705 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2669468 = {S69822, 133, { &P23162,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B15 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3022269 = {S69823, 93, { &P18673,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S7562) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA2962406 = {S69824, 152, { &P18221,&P5843,&P24678,&P18092 }};	/*Useragent - Nokia302/5.0 (14.78) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia302) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA63938 = {S69825, 113, { &P12100,&P17484,&P3636,&P18092 }};	/*Useragent - SAMSUNG-GT-S5233T/1.0 SHP/VPP/R5 Jasmine/0.8 NexPlayer/2.12 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA31045 = {S69826, 49, { &P12395,&P17017,&P3402,&P18092 }};	/*Useragent - Cricket-A300/1.0 UP.Browser/6.3.0.7 (GUI) MMP/2.0*/
const Device UA854720 = {S69827, 66, { &P15364,&P6704,&P20918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.6) Gecko/X Firefox/0.8*/
const Device UA2270942 = {S69828, 68, { &P15364,&P6704,&P20918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.7) Gecko/X Firefox/0.9.3*/
const Device UA2289948 = {S69829, 67, { &P15364,&P6704,&P20918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win 9x 4.90; rv:1.7) Gecko/X Firefox/0.9.3*/
const Device UA2472148 = {S69830, 70, { &P15364,&P6704,&P20918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win 9x 4.90; rv:1.7.3) Gecko/X Firefox/0.10.1*/
const Device UA2516983 = {S69831, 72, { &P15364,&P6704,&P20918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win 9x 4.90; en-US; rv:1.6) Gecko/X Firefox/0.8*/
const Device UA2846010 = {S69832, 83, { &P15364,&P6704,&P20918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win98; rv:1.7.3) Gecko/X Firefox/0.10.1 StumbleUpon/1.9993*/
const Device UA2257396 = {S69833, 138, { &P23911,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID7030 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2404051 = {S69834, 148, { &P15932,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3110 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2612524 = {S69835, 155, { &P15932,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3110 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2614019 = {S69836, 148, { &P15932,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3110 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615200 = {S69837, 155, { &P15932,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; GT-P3110 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2615970 = {S69838, 148, { &P15932,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GT-P3110 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2626175 = {S69839, 150, { &P15932,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2630485 = {S69840, 146, { &P15932,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ja-; GT-P3110 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2660967 = {S69841, 143, { &P23241,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; GPLUS_GN708W Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2765018 = {S69842, 137, { &P23241,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GN708W Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2856704 = {S69843, 59, { &P23241,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GN708W Build/JZO54K)*/
const Device UA2676026 = {S69844, 128, { &P21769,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ME301T Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2718786 = {S69845, 157, { &P12968,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT26i Build/6.1.A.2.55) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2738301 = {S69846, 157, { &P12968,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LT26i Build/6.1.A.2.45) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2996940 = {S69847, 158, { &P12968,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LT26i Build/6.1.A.0.452) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA18861 = {S69848, 80, { &P11947,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK700i/R2N SEMC-Browser/4.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA18862 = {S69849, 182, { &P11947,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK700i/R2AG SEMC-Browser/4.0.3 Profile/MIDP-1.0 MIDP-2.0 Configuration/CLDC-1.1, SonyEricssonK700i/R2AG SEMC-Browser/4.0.3 Profile/MIDP-1.0 MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62240 = {S69850, 81, { &P11947,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK700i/R2AA SEMC-Browser/4.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62585 = {S69851, 81, { &P11947,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK700i/R2AY SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64515 = {S69852, 87, { &P11947,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK700i/R2A SEMC-Browser/4.0 Profile/MIDP-1.0 MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA170558 = {S69853, 136, { &P17962,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; IM-A720L Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA447297 = {S69854, 60, { &P17962,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; IM-A720L Build/FRG83)*/
const Device UA2403729 = {S69855, 35, { &P14862,&P5204,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia2700c-2/09.95)*/
const Device UA2537960 = {S69856, 93, { &P11907,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaE52-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2532255 = {S69857, 140, { &P21851,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Spice MI352 Build/GRJ90) AppleWebKit/5.33.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2850754 = {S69858, 140, { &P21851,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Spice MI-352 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2580198 = {S69859, 151, { &P23090,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9400 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2753288 = {S69860, 134, { &P17160,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Slider SL101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2753892 = {S69861, 140, { &P21602,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5302 Build/12.0.A.1.211) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3030823 = {S69862, 165, { &P18114,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Transformer Prime TF201 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3096148 = {S69863, 127, { &P12280,&P22882,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari Version/5.1*/
const Device UA3110077 = {S69864, 130, { &P12280,&P22882,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A470a Safari Version/5.1*/
const Device UA3115630 = {S69865, 129, { &P12280,&P22882,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Safari Version/5.1*/
const Device UA3125854 = {S69866, 129, { &P12280,&P22882,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d Safari Version/5.1*/
const Device UA3128124 = {S69867, 127, { &P12280,&P22882,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A466 Safari Version/5.1*/
const Device UA9389 = {S69868, 111, { &P12044,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-F490/F490XEHB2 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA15744 = {S69869, 118, { &P12044,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-F490-ORANGE/F490AFHD1 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA22835 = {S69870, 120, { &P12044,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-F490-Vodafone/F490BUHB2 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA24743 = {S69871, 157, { &P12435,&P17483,&P3459,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-BL20/V10b Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA31525 = {S69872, 130, { &P12435,&P17483,&P3459,&P18092 }};	/*Useragent - LG/BL20/v10a Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA54371 = {S69873, 137, { &P12435,&P17483,&P3459,&P18092 }};	/*Useragent - LG-BL20-Orange/v10e Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA59881 = {S69874, 157, { &P12435,&P17483,&P3459,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-BL20/V10d Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA62626 = {S69875, 130, { &P12435,&P17483,&P3459,&P18092 }};	/*Useragent - LG-BL20/v10a Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA21033 = {S69876, 83, { &P13820,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK320c/R4GC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61645 = {S69877, 83, { &P13820,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK320i/R4GC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA182801 = {S69878, 136, { &P17977,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; IM-A710K Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1863532 = {S69879, 60, { &P17977,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; IM-A710K Build/FRG83)*/
const Device UA2019218 = {S69880, 134, { &P17977,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-; IM-A710K Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA706684 = {S69881, 81, { &P18364,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaC5-05)/UC Browser8.2.0.132/50/355/UCWEB Mobile*/
const Device UA902366 = {S69882, 137, { &P18662,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ISW13HT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2227320 = {S69883, 141, { &P18662,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC J Z321e Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2237775 = {S69884, 137, { &P18662,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ISW13HT Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2201432 = {S69885, 136, { &P21421,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KATBL10A16DA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2230512 = {S69886, 57, { &P20037,&P22784,&P21795,&P18092 }};	/*Useragent - MicromaxX292/MstarBrowser1.0/MIDP2.0Configuration/CLDC1.1*/
const Device UA2879367 = {S69887, 58, { &P20037,&P22784,&P21795,&P18092 }};	/*Useragent - MicromaxX292i/MstarBrowser1.0/MIDP2.0Configuration/CLDC1.1*/
const Device UA2468057 = {S69888, 129, { &P21892,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X6-7A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2469420 = {S69889, 133, { &P21892,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X6-7A-Q88 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2485629 = {S69890, 140, { &P21892,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X6-7A MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2603741 = {S69891, 140, { &P21892,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X6-7A-Q88 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583235 = {S69892, 145, { &P21644,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us;BLU Quattro 4.5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950864 = {S69893, 149, { &P21644,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BLU Quattro 4.5 HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2671195 = {S69894, 137, { &P19917,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P765 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3027894 = {S69895, 113, { &P14711,&P5204,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-01/11.40; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA68392 = {S69896, 102, { &P11689,&P5776,&P4777,&P18092 }};	/*Useragent - NokiaN70-1/5.0737.3.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.7.1.88/27/444*/
const Device UA195852 = {S69897, 101, { &P11689,&P5776,&P4777,&P18092 }};	/*Useragent - NokiaN70/ 5.0741.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.7.1.88/27/444*/
const Device UA893441 = {S69898, 81, { &P12449,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-in; NOKIAE5-00)/UC Browser8.4.0.159/28/352/UCWEB Mobile*/
const Device UA2503109 = {S69899, 89, { &P12449,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokiae5-00) UCBrowser8.4.0.159/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA1982895 = {S69900, 146, { &P21337,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; sr-rs; Telenor Touch Plus Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995395 = {S69901, 151, { &P21337,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; sr-rs; Telenor Touch Pad Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2202601 = {S69902, 141, { &P12280,&P18117,&P20788,&P18092 }};	/*Useragent - MQQBrowser/37 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA2232801 = {S69903, 142, { &P12280,&P18117,&P20788,&P18092 }};	/*Useragent - MQQBrowser/371 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA2260237 = {S69904, 140, { &P12280,&P18117,&P20788,&P18092 }};	/*Useragent - MQQBrowser/37 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3*/
const Device UA2366587 = {S69905, 141, { &P12280,&P18117,&P20788,&P18092 }};	/*Useragent - MQQBrowser/371 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3*/
const Device UA2761981 = {S69906, 140, { &P12280,&P18117,&P20788,&P18092 }};	/*Useragent - MQQBrowser/371 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3*/
const Device UA3117941 = {S69907, 139, { &P12280,&P18117,&P20788,&P18092 }};	/*Useragent - MQQBrowser/371 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3*/
const Device UA2226928 = {S69908, 131, { &P21698,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VL-107L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2360406 = {S69909, 178, { &P18583,&P5043,&P24672,&P18092 }};	/*Useragent - Nokia110/2.0 (03.04) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia110) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile*/
const Device UA2456774 = {S69910, 33, { &P14635,&P5843,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (NokiaC3-00/08.65)*/
const Device UA2703537 = {S69911, 142, { &P19792,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; HTC_DesireU Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2869749 = {S69912, 152, { &P19792,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC_Desire_U_dual_sim Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3005447 = {S69913, 152, { &P19792,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC Desire U dual sim Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3087251 = {S69914, 143, { &P19792,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; HTC Desire U Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3020037 = {S69915, 162, { &P18243,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; de-de; LG-P880 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/p880*/
const Device UA2974727 = {S69916, 136, { &P13074,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P930 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA10202 = {S69917, 62, { &P13475,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2323c/2.0 (08.20) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA14467 = {S69918, 64, { &P13475,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2323c-2/2.0 (08.20) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA23725 = {S69919, 79, { &P13750,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SCH-W259/W259DDDA2 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA753447 = {S69920, 64, { &P15140,&P17483,&P3459,&P18092 }};	/*Useragent - LG-GT550/V11d Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA49463 = {S69921, 66, { &P13518,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-N295-Sydney-Optimus/Obigo Q03C/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA77543 = {S69922, 59, { &P13518,&P22839,&P3807,&P18092 }};	/*Useragent - zte-N295/P107A20/Obigo Q03C/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2398256 = {S69923, 65, { &P13518,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE/ZTE-G N295 Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2758731 = {S69924, 59, { &P13518,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE/N295 Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA600515 = {S69925, 85, { &P13677,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s5253) UCBrowser8.2.0.132/69/452/UCWEB Mobile*/
const Device UA609245 = {S69926, 85, { &P13677,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s5253) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2813987 = {S69927, 38, { &P13677,&P17484,&P24667,&P18092 }};	/*Useragent - samsung-gt-s5253/UCWEB8.2.0.116/69/352*/
const Device UA605534 = {S69928, 160, { &P17172,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Transformer TF101 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA889084 = {S69929, 160, { &P17172,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Transformer TF101 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2344892 = {S69930, 154, { &P17172,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Transformer Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1977705 = {S69931, 133, { &P20688,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; hr-hr; Tele2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2466974 = {S69932, 134, { &P20688,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hr-hr; Tele2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029763 = {S69933, 132, { &P17337,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7500 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2504139 = {S69934, 132, { &P17337,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2611173 = {S69935, 132, { &P17337,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; GT-P7500 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2611416 = {S69936, 116, { &P17337,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/1.1.3 (Linux; U; Android 4.1.1; sk-SK; GT-P7500 Build/cm_p4-userdebug 4.1.1 JRO03C eng.maplesci.X.X test-keys)*/
const Device UA2612539 = {S69937, 61, { &P17337,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P7500 Build/JRO03L)*/
const Device UA2616233 = {S69938, 145, { &P17337,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; da-dk; GT-P7500 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2616769 = {S69939, 137, { &P17337,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; vi-vn; GT-P7500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2616916 = {S69940, 61, { &P17337,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P7500 Build/JZO54K)*/
const Device UA2618667 = {S69941, 116, { &P17337,&P18111,&P18112,&P18092 }};	/*Useragent - Vyrso/2.2.3 (Linux; U; Android 4.1.1; en-GB; GT-P7500 Build/cm_p4-userdebug 4.1.1 JRO03H eng.maplesci.X.X test-keys)*/
const Device UA2618818 = {S69942, 145, { &P17337,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; lt-lt; GT-P7500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2620069 = {S69943, 151, { &P17337,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 GSA/2.0.0.X*/
const Device UA2622482 = {S69944, 132, { &P17337,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2625397 = {S69945, 145, { &P17337,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fi-fi; GT-P7500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2627156 = {S69946, 62, { &P17337,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P7500M Build/JZO54K)*/
const Device UA2628429 = {S69947, 122, { &P17337,&P18111,&P18112,&P18092 }};	/*Useragent - Groupon/2.2.2572 (Android 4.1.2; Samsung P4 / Samsung GT-P7500; )[preload=false;locale=en_CA;clientidbase=android-samsung]*/
const Device UA2261565 = {S69948, 132, { &P17323,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6800 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2341863 = {S69949, 132, { &P17323,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P6800 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2614252 = {S69950, 138, { &P17323,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P6800 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2616966 = {S69951, 138, { &P17323,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6800 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2239222 = {S69952, 139, { &P20869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; A878 Duo Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423064 = {S69953, 126, { &P24551,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; M2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2673183 = {S69954, 146, { &P18224,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8815 Build/HuaweiU8815) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2936160 = {S69955, 155, { &P18224,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8815N Build/HuaweiU8815NC02B942) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2739405 = {S69956, 144, { &P22737,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; Micromax A115 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088415 = {S69957, 66, { &P22737,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Micromax A115 Build/JZO54K)*/
const Device UA2754063 = {S69958, 135, { &P18468,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabA2109A Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754338 = {S69959, 128, { &P21769,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ME301T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2782562 = {S69960, 135, { &P24077,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; H120 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3002024 = {S69961, 57, { &P24077,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; H120 Build/JOP40D)*/
const Device UA2796945 = {S69962, 140, { &P17346,&P22882,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4372q Safari/8536.25*/
const Device UA2885401 = {S69963, 140, { &P17346,&P22882,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4400f Safari/8536.25*/
const Device UA2947932 = {S69964, 140, { &P17346,&P22882,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A4414e Safari/8536.25*/
const Device UA3111960 = {S69965, 138, { &P17346,&P22882,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/11A465 Safari/8536.25*/
const Device UA251682 = {S69966, 81, { &P17595,&P17295,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.9.168 Version/11.52*/
const Device UA626195 = {S69967, 76, { &P15364,&P17285,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:7.0.1; BrowserMob RBU) Gecko/X Firefox/7.0.1*/
const Device UA2949323 = {S69968, 79, { &P15364,&P17285,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:7.0a1) Gecko/X Firefox/7.0a1*/
const Device UA877238 = {S69969, 83, { &P15364,&P22835,&P18272,&P18092 }};	/*Useragent - Spice M5885   Browser/Opera  Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA2445922 = {S69970, 83, { &P15364,&P22835,&P18272,&P18092 }};	/*Useragent - Spice M5885   Browser/Opera  Opera/9.80 (MTK; U; hi-IN) Presto/2.5.28 Version/10.10*/
const Device UA2814652 = {S69971, 80, { &P15364,&P22835,&P18272,&P18092 }};	/*Useragent - Spice M5885 Browser/Opera Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA900428 = {S69972, 136, { &P23381,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; QMobile A5 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1959022 = {S69973, 146, { &P20454,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 7019 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026769 = {S69974, 139, { &P20454,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 7019 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2415816 = {S69975, 142, { &P20454,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 7019A Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1994030 = {S69976, 132, { &P21381,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A520/S101) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026313 = {S69977, 132, { &P21381,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A520 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2028189 = {S69978, 139, { &P21381,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A520 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032271 = {S69979, 127, { &P21381,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A520 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2817313 = {S69980, 63, { &P21381,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; Lenovo A520 Build/GRJ90)*/
const Device UA3117679 = {S69981, 143, { &P21381,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A520GRAY Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3017608 = {S69982, 81, { &P14663,&P5567,&P17370,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/5.0/27.2338; U; en) Presto/2.8.119 320X240 LG VX9200*/
const Device UA2385240 = {S69983, 70, { &P18283,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; GT-I9300; 720*1280) UCWEB7.9.0.94/139/444*/
const Device UA2424451 = {S69984, 169, { &P14682,&P5843,&P24673,&P18092 }};	/*Useragent - Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6303iclassic) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2566487 = {S69985, 139, { &P19844,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80 COBALT Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2656255 = {S69986, 22, { &P15364,&P17017,&P22842,&P18092 }};	/*Useragent - VLC/2.0.6 LibVLC/2.0.6*/
const Device UA2712213 = {S69987, 93, { &P18673,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S7562) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2754937 = {S69988, 129, { &P20948,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LePanII Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2888299 = {S69989, 165, { &P22582,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I337 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/jflteatt*/
const Device UA2900843 = {S69990, 114, { &P13107,&P5043,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-06/07.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA3081482 = {S69991, 139, { &P18592,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One VX Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA44923 = {S69992, 47, { &P12527,&P17484,&P5001,&P18092 }};	/*Useragent - SCH-U640/1.0 NetFront/3.0.22.1.19 (GUI) MMP/2.0*/
const Device UA76632 = {S69993, 157, { &P15931,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MOT-XT883/SLNCT-57.1.40; 540*960; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369788 = {S69994, 158, { &P15931,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-XT883/SLNCT-60.3.250; 540*960; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA689634 = {S69995, 141, { &P15931,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; XT883 Build/SLNCT-57.1.60) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021421 = {S69996, 157, { &P15931,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MOT-XT883/SLNCT-57.1.60; 540*960; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024844 = {S69997, 129, { &P15931,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT883 Build/SLNCT-57.1.40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2027413 = {S69998, 134, { &P15931,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT883 Build/SLNCT-57.1.40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2034536 = {S69999, 135, { &P15931,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT883 Build/SLNCT-60.3.250) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3117129 = {S70000, 142, { &P15931,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT883 Build/SLNCT-60.3.250) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA104481 = {S70001, 13, { &P16305,&P22839,&P17470,&P18092 }};	/*Useragent - ZTE-Me/Mobile*/
const Device UA624012 = {S70002, 146, { &P18114,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; Transformer Prime TF201 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2024388 = {S70003, 128, { &P18114,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; ja-jp; TF201 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA608795 = {S70004, 151, { &P12879,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA629719 = {S70005, 147, { &P12879,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA907922 = {S70006, 147, { &P12879,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2610249 = {S70007, 154, { &P12879,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2616424 = {S70008, 270, { &P12879,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-P7510 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.2.0-57 (Android 4.0.4; samsung GT-P7510 Build/cm_p4wifi-userdebug 4.0.4 IMM76L eng.root.X.X test-keys)*/
const Device UA2523843 = {S70009, 138, { &P21300,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-be; Lenco TAB-1014 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2559131 = {S70010, 114, { &P14826,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC1-01/06.15; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2755332 = {S70011, 130, { &P14908,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2868718 = {S70012, 144, { &P22756,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; SonyC1604 Build/11.3.A.2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957388 = {S70013, 140, { &P22756,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; C1604 Build/11.3.A.2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3074339 = {S70014, 62, { &P22756,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; C1604 Build/11.3.A.2.1)*/
const Device UA524211 = {S70015, 79, { &P15364,&P17843,&P17828,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/4.5; Linux) KHTML/4.5.3 (like Gecko) Kubuntu*/
const Device UA2155909 = {S70016, 117, { &P12473,&P5204,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2730c-1/10.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA1944961 = {S70017, 137, { &P18742,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOR807 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1981343 = {S70018, 121, { &P18742,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; R807 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2025083 = {S70019, 126, { &P18742,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; R807 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2686943 = {S70020, 57, { &P18742,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; R807 Build/GRK39F)*/
const Device UA2162005 = {S70021, 85, { &P12080,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s3653) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA1990573 = {S70022, 130, { &P23873,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BC1003 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2031964 = {S70023, 130, { &P23873,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; BC1003 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2655269 = {S70024, 134, { &P23873,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BC1003 EUS Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2511509 = {S70025, 166, { &P15919,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P3100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2631675 = {S70026, 166, { &P15919,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-P3100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3009493 = {S70027, 170, { &P15919,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-P3100 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.276 Mobile*/
const Device UA2749569 = {S70028, 131, { &P21974,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; RC9731C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2566494 = {S70029, 136, { &P19784,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LIFETAB_P9514 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2676693 = {S70030, 125, { &P15869,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Xoom Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2713329 = {S70031, 126, { &P15869,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Xoom Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2736628 = {S70032, 151, { &P22806,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; i-mobile i-style 7.1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957387 = {S70033, 73, { &P22806,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; i-mobile i-style 7.1 Build/JRO03C)*/
const Device UA2937106 = {S70034, 154, { &P23709,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HUAWEI G525-U00 Build/HuaweiG525-U00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996305 = {S70035, 76, { &P23709,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HUAWEI G525-U00 Build/HuaweiG525-U00)*/
const Device UA208933 = {S70036, 131, { &P12482,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; T959 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA251927 = {S70037, 131, { &P12482,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; T959 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA535944 = {S70038, 131, { &P12482,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; T959 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025557 = {S70039, 55, { &P12482,&P24694,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; T959 Build/MIUI)*/
const Device UA2026020 = {S70040, 124, { &P12482,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; T959 Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2285885 = {S70041, 135, { &P12482,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SGH-T959 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA465688 = {S70042, 50, { &P15364,&P17188,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 4.0)*/
const Device UA720684 = {S70043, 60, { &P15364,&P17188,&P17193,&P18092 }};	/*Useragent - Mozilla/6.0 (compatible;MSIE 7.0;  Windows NT 4.0; QQLOADOW)*/
const Device UA770502 = {S70044, 58, { &P15364,&P17188,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible;MSIE 7.0;  Windows NT 4.0;  MyIE2)*/
const Device UA773598 = {S70045, 58, { &P15364,&P17188,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible;MSIE 7.0;  Windows NT 4.0;  MyIE2)*/
const Device UA873827 = {S70046, 62, { &P15364,&P17188,&P17193,&P18092 }};	/*Useragent - Mozilla/3.0 (compatible;MSIE 7.0;  Windows NT 4.0; QQDownload)*/
const Device UA2320949 = {S70047, 55, { &P15364,&P17188,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 4.0; SV1)*/
const Device UA2730136 = {S70048, 57, { &P15364,&P17188,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; WinNT4.0; Trident/2.0)*/
const Device UA2888915 = {S70049, 97, { &P15364,&P17188,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727)*/
const Device UA600196 = {S70050, 92, { &P17595,&P17294,&P18198,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.4; en-US; rv:1.9.2.28) Gecko/X Thunderbird/3.1.20*/
const Device UA2028255 = {S70051, 129, { &P22558,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; RK702 Build/RK2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2166961 = {S70052, 127, { &P22558,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; RK702 Build/2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2574721 = {S70053, 129, { &P22558,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; RK702 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2833771 = {S70054, 129, { &P22558,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-ca; RK702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2187906 = {S70055, 141, { &P22030,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; Brava_DM-990 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2483292 = {S70056, 141, { &P22030,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; Brava DM-990 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2569829 = {S70057, 142, { &P17346,&P17146,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9A405 Safari/7534.48.3*/
const Device UA2667566 = {S70058, 140, { &P17346,&P17146,&P21416,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/25.0.1364.124 Mobile/9A334 Safari/7534.48.3*/
const Device UA2670622 = {S70059, 140, { &P18303,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT27i Build/6.1.1.B.1.54) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2719271 = {S70060, 161, { &P18513,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6802 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2975732 = {S70061, 137, { &P24063,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; IdeaTab S6000-F Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021722 = {S70062, 136, { &P24063,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; IdeaTab S6000-H Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA7367 = {S70063, 116, { &P11797,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-D900i-ORANGE/D900iABGF2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA63699 = {S70064, 102, { &P11797,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-D900i/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA301932 = {S70065, 135, { &P18030,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SHW-M100S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA514847 = {S70066, 137, { &P18030,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SHW-M100S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018764 = {S70067, 61, { &P18030,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SHW-M100S Build/FROYO)*/
const Device UA361186 = {S70068, 168, { &P22214,&P5629,&P15760,&P18092 }};	/*Useragent - Spice Mi350n Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; huaqin73_gb Build/ALPS.GB.FDD2.MP.V2.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2980609 = {S70069, 170, { &P22214,&P5629,&P15760,&P18092 }};	/*Useragent - Spice Mi350n Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Spice Mi-350n Build/ALPS.GB.FDD2.MP.V2.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA637208 = {S70070, 133, { &P15364,&P17817,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.7 (KHTML, like Gecko) Ubuntu/10.10 Chromium/16.0.912.77 Chrome/16.0.912.77 Safari/535.7*/
const Device UA694464 = {S70071, 135, { &P15364,&P17817,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.7 (KHTML, like Gecko) Ubuntu/10.10 Chromium/16.0.912.77 Chrome/16.0.912.77 Safari/535.7*/
const Device UA514323 = {S70072, 131, { &P15364,&P17285,&P22512,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.4) Gecko/X FireDownload/2.0.1 Firefox/3.6.4 Wyzo/3.6.4.1 FireTorrent/2.0.1*/
const Device UA1900410 = {S70073, 81, { &P14225,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_Desire_HD_A9191; 480*800) UCWEB7.9.3.103/139/800*/
const Device UA2148499 = {S70074, 81, { &P14225,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_Desire_HD_A9191; 480*800) UCWEB7.9.3.103/139/999*/
const Device UA2318182 = {S70075, 81, { &P14225,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_Desire_HD_A9191; 480*800) UCWEB7.9.4.145/139/800*/
const Device UA2366402 = {S70076, 80, { &P14225,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_Desire_HD_A9191; 480*800) UCWEB7.9.0.94/139/352*/
const Device UA2423475 = {S70077, 80, { &P14225,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_Desire_HD_A9191; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2480159 = {S70078, 80, { &P14225,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_Desire_HD_A9191; 480*800) UCWEB7.9.0.94/139/351*/
const Device UA2481500 = {S70079, 68, { &P14225,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2; zh-cn; Desire_HD; 480*800) UCWEB7.9.0.94/139/352*/
const Device UA2503218 = {S70080, 70, { &P14225,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; Desire_HD; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2840504 = {S70081, 80, { &P14225,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_Desire_HD_A9191; 480*800) UCWEB7.9.0.94/139/355*/
const Device UA2134654 = {S70082, 64, { &P17595,&P18120,&P22892,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; iCab 3.0.3; Macintosh; U; PPC Mac OS X)*/
const Device UA2242308 = {S70083, 39, { &P17595,&P18120,&P22892,&P18092 }};	/*Useragent - iCab/3.0.2 (Macintosh; U; PPC Mac OS X)*/
const Device UA2978881 = {S70084, 39, { &P17595,&P18120,&P22892,&P18092 }};	/*Useragent - iCab/3.0.5 (Macintosh; U; PPC Mac OS X)*/
const Device UA2234819 = {S70085, 138, { &P21247,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; NGM Orion Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2467642 = {S70086, 123, { &P15118,&P5055,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; NokiaX7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile*/
const Device UA2505491 = {S70087, 102, { &P15118,&P5055,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiax7-00) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2556872 = {S70088, 42, { &P15118,&P5055,&P24673,&P18092 }};	/*Useragent - NokiaX7-00UCBrowser/8.7.1.234/51/355/UCWEB*/
const Device UA2983801 = {S70089, 125, { &P15118,&P5055,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; NokiaX7-00.1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile*/
const Device UA2487075 = {S70090, 147, { &P20509,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; MyPhone A888 Duo Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2756017 = {S70091, 130, { &P22043,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P5113 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2769038 = {S70092, 130, { &P22043,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P5113 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2869897 = {S70093, 135, { &P14997,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; S100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3074174 = {S70094, 57, { &P14997,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; S100 Build/JZO54K)*/
const Device UA336257 = {S70095, 137, { &P15364,&P17816,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/10.04 Chromium/13.0.782.215 Chrome/13.0.782.215 Safari/535.1*/
const Device UA377583 = {S70096, 133, { &P15364,&P17816,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.33 (KHTML, like Gecko) Ubuntu/10.04 Chromium/13.0.752.0 Chrome/13.0.752.0 Safari/534.33*/
const Device UA347731 = {S70097, 64, { &P15364,&P7127,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.10.229 Version/11.60*/
const Device UA458715 = {S70098, 139, { &P18619,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; MID7024 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458755 = {S70099, 63, { &P18619,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3; MID7024 Build/GINGERBREAD)*/
const Device UA458784 = {S70100, 141, { &P18619,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; MID7024 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2183971 = {S70101, 134, { &P18017,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Ideapad K1 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2258054 = {S70102, 168, { &P13064,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I727 Build/JRO03R; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2363434 = {S70103, 160, { &P13064,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I727 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2384546 = {S70104, 168, { &P13064,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I727 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2576925 = {S70105, 168, { &P13064,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I727 Build/JRO03L; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2461174 = {S70106, 155, { &P15228,&P5043,&P24675,&P18092 }};	/*Useragent - NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac2-03) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2512819 = {S70107, 141, { &P12280,&P18117,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA2557637 = {S70108, 140, { &P12280,&P18117,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3*/
const Device UA2600183 = {S70109, 139, { &P12280,&P18117,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3*/
const Device UA2659865 = {S70110, 138, { &P12280,&P18117,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3*/
const Device UA2558939 = {S70111, 131, { &P23515,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TD-1010 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2567664 = {S70112, 77, { &P15364,&P17285,&P22559,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:19.0) Gecko/X Firefox/19.0 IceDragon/19.0.1.0*/
const Device UA2567957 = {S70113, 136, { &P21606,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LGL86C Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2676821 = {S70114, 137, { &P18459,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS950 4G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2901213 = {S70115, 165, { &P22597,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I545 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/jfltevzw*/
const Device UA55802 = {S70116, 142, { &P17995,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; SH-03C Build/SB110) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA90544 = {S70117, 142, { &P17995,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; SH-03C Build/S1130) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA200575 = {S70118, 142, { &P17995,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; SH-03C Build/S2140) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA878103 = {S70119, 64, { &P17995,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SH-03C Build/S2140)*/
const Device UA160343 = {S70120, 132, { &P15364,&P17285,&P22446,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Iron/8.0.555.0 Chrome/8.0.555.0 Safari/534.10*/
const Device UA627128 = {S70121, 134, { &P21182,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; IS12F Build/FGK600) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2202852 = {S70122, 51, { &P15767,&P18580,&P18245,&P18092 }};	/*Useragent - Bing%20for%20iPad/1.1.3 CFNetwork/609 Darwin/13.0.0*/
const Device UA2258191 = {S70123, 91, { &P17595,&P17297,&P20761,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:16.0) Gecko/X Firefox/16.0 SeaMonkey/2.13.2*/
const Device UA2257665 = {S70124, 138, { &P20481,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DII PAD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2376944 = {S70125, 148, { &P15876,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5100 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2455506 = {S70126, 148, { &P15876,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P5100 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2612227 = {S70127, 155, { &P15876,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5100 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2615961 = {S70128, 148, { &P15876,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5100 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2632863 = {S70129, 155, { &P15876,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; GT-P5100 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567337 = {S70130, 145, { &P19891,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I717 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667831 = {S70131, 138, { &P19891,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I717 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2672197 = {S70132, 141, { &P18504,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LS840 Build/ZVK.IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2702364 = {S70133, 145, { &P17346,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2703092 = {S70134, 147, { &P17346,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2713864 = {S70135, 147, { &P17346,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA3056284 = {S70136, 139, { &P17346,&P19826,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/6531.22.7 iLunascape/3212*/
const Device UA2756087 = {S70137, 146, { &P18224,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8815 Build/HuaweiU8815) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2899410 = {S70138, 95, { &P17968,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Android2.x) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3055011 = {S70139, 92, { &P17968,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; vi; Android2.x) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2974914 = {S70140, 129, { &P20420,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; I10A-LE Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974595 = {S70141, 138, { &P19854,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R760X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973465 = {S70142, 137, { &P19854,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R760 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2956379 = {S70143, 129, { &P22922,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; M470BSA Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3081625 = {S70144, 154, { &P20088,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E-51 Build/HuaweiU8666E-51) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083606 = {S70145, 148, { &P20088,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E Build/HuaweiU8666E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126318 = {S70146, 148, { &P20088,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666N Build/HuaweiU8666N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA236474 = {S70147, 131, { &P15364,&P7133,&P22406,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.2.0.0 Chrome/12.0.742.112 Safari/534.30*/
const Device UA236671 = {S70148, 124, { &P15364,&P7133,&P22406,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.2.0.0 Chrome/12.0.742.112 Safari/534.30*/
const Device UA2352512 = {S70149, 134, { &P17595,&P17297,&P24745,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Stainless/0.8 like Version/5.1 Safari/534.48.3*/
const Device UA395841 = {S70150, 61, { &P20064,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX263/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA703377 = {S70151, 31, { &P20064,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX263/Q03C MAUI-browser/*/
const Device UA1993314 = {S70152, 131, { &P21310,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NM7RK-C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2703810 = {S70153, 137, { &P22805,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E455 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2704294 = {S70154, 137, { &P23165,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; H7500+ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2989787 = {S70155, 59, { &P23165,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; H7500+ Build/JZO54K)*/
const Device UA3046647 = {S70156, 136, { &P23165,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; H7500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3032518 = {S70157, 165, { &P18472,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-79) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3130722 = {S70158, 136, { &P18713,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3130944 = {S70159, 136, { &P18713,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P768 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131302 = {S70160, 136, { &P18713,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131323 = {S70161, 136, { &P18713,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LGMS769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA2427066 = {S70162, 65, { &P11950,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; X10i; 480*854) UCWEB7.9.0.94/139/444*/
const Device UA2466664 = {S70163, 71, { &P11950,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.1-update1; zh-cn; X10i; 480*854) UCWEB7.9.0.94/139/444*/
const Device UA2010271 = {S70164, 141, { &P21039,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Spice_Mi-355 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029420 = {S70165, 134, { &P21354,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; AX515 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2220632 = {S70166, 98, { &P19885,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-SGH-T159/1.0 SHP/VPP/R5 NetFront/4.2 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA2461248 = {S70167, 168, { &P15989,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST21i2 Build/11.0.A.1.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2462048 = {S70168, 168, { &P15989,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST21i Build/11.0.A.0.16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2427299 = {S70169, 135, { &P19985,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SH-02E Build/SC110) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2432456 = {S70170, 139, { &P20174,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CINK KING Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2602366 = {S70171, 139, { &P12280,&P19826,&P21783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.3m*/
const Device UA2670514 = {S70172, 139, { &P12280,&P19826,&P21783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.3m*/
const Device UA2679625 = {S70173, 141, { &P12280,&P19826,&P21783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.3.1m*/
const Device UA2711506 = {S70174, 141, { &P12280,&P19826,&P21783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 Safari/7534.48.3 Sleipnir/2.3.2m*/
const Device UA2716100 = {S70175, 141, { &P12280,&P19826,&P21783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.3.2m*/
const Device UA2716103 = {S70176, 141, { &P12280,&P19826,&P21783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.3.2m*/
const Device UA2862979 = {S70177, 141, { &P12280,&P19826,&P21783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.3.1m*/
const Device UA2975721 = {S70178, 130, { &P15985,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021879 = {S70179, 131, { &P15985,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3020296 = {S70180, 139, { &P20124,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Grand X(M) Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021096 = {S70181, 137, { &P20124,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021574 = {S70182, 140, { &P20124,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ZTE Grand X Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022016 = {S70183, 140, { &P20124,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Grand X Pro Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022260 = {S70184, 136, { &P20124,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Grand X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023380 = {S70185, 138, { &P20124,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2997690 = {S70186, 129, { &P24419,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ME302C Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1965032 = {S70187, 129, { &P22128,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; OT107 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026440 = {S70188, 129, { &P22128,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; OT107 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2011110 = {S70189, 95, { &P17595,&P17296,&P20792,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; Edition MAS; en) Presto/2.10.229 Version/11.66*/
const Device UA2261543 = {S70190, 145, { &P20558,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ608-14) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2383353 = {S70191, 147, { &P20558,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ608 Build/7.7.1-141-7-FLEM-UMTS-LA) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2461335 = {S70192, 171, { &P14186,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2469777 = {S70193, 171, { &P14186,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2484448 = {S70194, 171, { &P14186,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2506560 = {S70195, 171, { &P14186,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2422095 = {S70196, 145, { &P23925,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; POLYTRON W2430 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2763006 = {S70197, 136, { &P23925,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; W2430 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2559753 = {S70198, 156, { &P14848,&P5843,&P24676,&P18092 }};	/*Useragent - NokiaX3-00/5.0 (11.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax3-00) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2755410 = {S70199, 127, { &P18680,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KFJWI Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3023451 = {S70200, 140, { &P17968,&P5629,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9737AP Mobile*/
const Device UA3059595 = {S70201, 140, { &P17968,&P5629,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AT Mobile*/
const Device UA3061095 = {S70202, 140, { &P17968,&P5629,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP Mobile*/
const Device UA2974239 = {S70203, 136, { &P22679,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA1995436 = {S70204, 136, { &P21125,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; L-06D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031385 = {S70205, 140, { &P21125,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; L-06DJOJO Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2226709 = {S70206, 71, { &P15364,&P7133,&P18428,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:16.0) Gecko/X Firefox/16.0 slurp*/
const Device UA2258928 = {S70207, 64, { &P15364,&P7133,&P18428,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:16.0) Gecko/X Firefox/16.0 slurp*/
const Device UA2259176 = {S70208, 77, { &P15364,&P7133,&P18428,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:16.0) Gecko/X Firefox/16.0 slurp slurp*/
const Device UA2401943 = {S70209, 70, { &P15364,&P7133,&P18428,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:16.0) Gecko/X Firefox/16.0 slurp slurp*/
const Device UA2459150 = {S70210, 162, { &P15759,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; HTC Vision Build/JOP40D; CyanogenMod-CM-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2603888 = {S70211, 158, { &P15759,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC Vision Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3056400 = {S70212, 159, { &P15759,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC Vision Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2673366 = {S70213, 130, { &P20035,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2714156 = {S70214, 131, { &P20035,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925U Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2715883 = {S70215, 161, { &P18304,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9070 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2753045 = {S70216, 134, { &P17211,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 101G9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2783535 = {S70217, 131, { &P17211,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 101G9 Build/MR1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2942036 = {S70218, 137, { &P24481,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7560 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2975566 = {S70219, 130, { &P17337,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7500 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023719 = {S70220, 130, { &P17337,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P7500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2964999 = {S70221, 159, { &P15989,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21i Build/11.0.A.4.22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3059701 = {S70222, 159, { &P15989,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21i2 Build/11.0.A.1.8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA627167 = {S70223, 160, { &P14417,&P5129,&P3690,&P18092 }};	/*Useragent - Nokia5236/51.9.002 (SymbianOS/9.4; U; Series60/5.0; Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/413 (KHTML,like Gecko) Safari/413 3gpp-gba*/
const Device UA2423145 = {S70224, 69, { &P14877,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; GT-I9000; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2467236 = {S70225, 67, { &P14877,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2; zh-cn; GT-I9000; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2360169 = {S70226, 146, { &P18626,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I547 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2382085 = {S70227, 139, { &P18626,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I547C Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2410939 = {S70228, 47, { &P15364,&P16110,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 6.2; U; en) Presto/2.1.1*/
const Device UA2494145 = {S70229, 47, { &P15364,&P16110,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows NT 6.2; U; en) Presto/2.1.1*/
const Device UA2518730 = {S70230, 47, { &P15364,&P16110,&P18337,&P18092 }};	/*Useragent - Opera/9.62 (Windows NT 6.2; U; en) Presto/2.1.1*/
const Device UA2774901 = {S70231, 47, { &P15364,&P16110,&P18337,&P18092 }};	/*Useragent - Opera/9.60 (Windows NT 6.2; U; en) Presto/2.1.1*/
const Device UA2669514 = {S70232, 142, { &P22632,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Titanium S1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2676593 = {S70233, 125, { &P14111,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Arc Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2753786 = {S70234, 137, { &P21506,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937705 = {S70235, 138, { &P21506,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2767449 = {S70236, 154, { &P22856,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0151 Build/HuaweiY300-0151) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2992251 = {S70237, 147, { &P22856,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0151 Build/HuaweiY300-0151) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2769867 = {S70238, 93, { &P23000,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; TECNO_P3) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3004046 = {S70239, 35, { &P17968,&P21637,&P23997,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android 4.3)*/
const Device UA3031371 = {S70240, 128, { &P17968,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3063983 = {S70241, 128, { &P17968,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3128367 = {S70242, 128, { &P17968,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Build/JLS36C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3081281 = {S70243, 136, { &P12814,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-i777 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081284 = {S70244, 137, { &P12814,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I777 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2261459 = {S70245, 134, { &P21048,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; W732 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2505881 = {S70246, 139, { &P21081,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; ZTE V790 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2529525 = {S70247, 135, { &P21081,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V790 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2703969 = {S70248, 147, { &P23118,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; Aqua Star Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2974457 = {S70249, 146, { &P19863,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974479 = {S70250, 140, { &P18183,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ615-12) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3056065 = {S70251, 141, { &P24378,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1;en-us; Lenovo_P780_ROW/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2.1 Mobile Safari/534.30*/
const Device UA3076227 = {S70252, 137, { &P24378,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1;en-us; Lenovo_P780/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2.1 Mobile Safari/534.30*/
const Device UA3078638 = {S70253, 146, { &P24378,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Lenovo P780_ROW Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3106985 = {S70254, 68, { &P24378,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; Lenovo P780_ROW Build/JOP40D)*/
const Device UA54017 = {S70255, 115, { &P12303,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-GT-B3410W/1.0 SHP/VPP/R5 Dolfin/1.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-N*/
const Device UA43574 = {S70256, 70, { &P14124,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-S3500/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54234 = {S70257, 100, { &P14124,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-GT-S3500C/1.0 Release/10.23.2008 Browser/NetFront3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA262616 = {S70258, 97, { &P14124,&P17484,&P3976,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungS3500/S3500BUIB3 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA540041 = {S70259, 136, { &P18019,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; IM-A730S Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1896861 = {S70260, 137, { &P18019,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; IM-A730S Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018943 = {S70261, 60, { &P18019,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; IM-A730S Build/FRG83)*/
const Device UA2020554 = {S70262, 135, { &P18019,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-; IM-A730S Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2472362 = {S70263, 134, { &P18019,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-; IM-A730S Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA351581 = {S70264, 135, { &P21225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID7010 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458906 = {S70265, 59, { &P21225,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; MID7010 Build/GRI40)*/
const Device UA1967193 = {S70266, 136, { &P21225,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID7010A Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA673408 = {S70267, 136, { &P15364,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Revue Build/KA2X-X.X) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA743986 = {S70268, 129, { &P15364,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Tostab03 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2020781 = {S70269, 127, { &P15364,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; THRiVE Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2168122 = {S70270, 128, { &P15364,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom 4G Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2444897 = {S70271, 147, { &P15364,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Haipai_Noble_I9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2940157 = {S70272, 219, { &P15364,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-cn;HTCP715 /1.0 Android/3.1 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/HMJ15) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA3047229 = {S70273, 132, { &P15364,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-in; VOX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057468 = {S70274, 133, { &P15364,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Videocon A26 Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA3103171 = {S70275, 140, { &P15364,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Swipe Xylus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1980350 = {S70276, 154, { &P14433,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9003 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2431450 = {S70277, 230, { &P15649,&P5629,&P17373,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/6.5.27431/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2367136 = {S70278, 139, { &P20525,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; A898 Duo Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2530205 = {S70279, 145, { &P23437,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; iBuddy Connect Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3118965 = {S70280, 67, { &P23437,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; iBuddy Connect Build/IMM76D)*/
const Device UA2567260 = {S70281, 129, { &P21769,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ME301T Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2791791 = {S70282, 63, { &P15364,&P17285,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; Windows NT 6.0; Trident/3.0)*/
const Device UA2850762 = {S70283, 63, { &P15364,&P17285,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; Windows NT 6.0; Trident/3.0)*/
const Device UA2676727 = {S70284, 136, { &P20240,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E971 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714424 = {S70285, 129, { &P20240,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E971 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2734492 = {S70286, 135, { &P15364,&P17017,&P18106,&P18092 }};	/*Useragent - Mozilla/5.0 (en-US) AppleWebKit/[WEBKIT_VERSION] (KHTML, like Gecko) Chrome/[CHROME_VERSION] Safari/[WEBKIT_VERSION] pss-webkit-request*/
const Device UA2754224 = {S70287, 130, { &P19879,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I915 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2880227 = {S70288, 138, { &P19878,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I497 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2942088 = {S70289, 137, { &P22390,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5310 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3057095 = {S70290, 138, { &P22390,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5310B Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3058748 = {S70291, 138, { &P22390,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5310L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3077711 = {S70292, 130, { &P22390,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5310 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2975048 = {S70293, 137, { &P12942,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I927 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA55103 = {S70294, 80, { &P15364,&P5890,&P3439,&P18092 }};	/*Useragent - H1-1/20.0.041 SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64093 = {S70295, 150, { &P15364,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 E71-3/410.21.010; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA81124 = {S70296, 111, { &P15364,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.1; U; [en]; SymbianOS/9.1 Series60/3.0) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA313944 = {S70297, 30, { &P15364,&P5890,&P3439,&P18092 }};	/*Useragent - E63/SymbianOS/9.1 Series60/3.0*/
const Device UA504632 = {S70298, 32, { &P15364,&P5890,&P3439,&P18092 }};	/*Useragent - E63-1/SymbianOS/9.1 Series60/3.0*/
const Device UA519127 = {S70299, 47, { &P15364,&P5890,&P3439,&P18092 }};	/*Useragent - (C)NokiaNXX/SymbianOS/9.1 Series60/3.0 3gpp-gba*/
const Device UA542017 = {S70300, 30, { &P15364,&P5890,&P3439,&P18092 }};	/*Useragent - E71/SymbianOS/9.1 Series60/3.0*/
const Device UA717585 = {S70301, 30, { &P15364,&P5890,&P3439,&P18092 }};	/*Useragent - E66/SymbianOS/9.1 Series60/3.0*/
const Device UA1886376 = {S70302, 32, { &P15364,&P5890,&P3439,&P18092 }};	/*Useragent - E71-1/SymbianOS/9.1 Series60/3.0*/
const Device UA370501 = {S70303, 143, { &P20537,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_7260 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA833072 = {S70304, 114, { &P20537,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_7260/2.3.002.X.7260; 480*800; CTC/2.0) CoolpadWebkit/533.1*/
const Device UA2413277 = {S70305, 116, { &P20537,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn;YL-Coolpad_7260A/2.3.002.X.7260+; 480*800; CUCC/3.0) CoolpadWebkit/533.1*/
const Device UA2644480 = {S70306, 144, { &P20537,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_7260A Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622327 = {S70307, 140, { &P20972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-07D Build/S5230) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30*/
const Device UA903248 = {S70308, 136, { &P20972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-07D Build/S5230) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1992648 = {S70309, 136, { &P20972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-07D Build/S6151) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2004097 = {S70310, 136, { &P20972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-07D Build/S7190) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3030656 = {S70311, 136, { &P20972,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-07D Build/S7230) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA895067 = {S70312, 92, { &P11684,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-in; Nokia5800 XpressMusic)/UC Browser8.4.0.159/50/352/UCWEB Mobile*/
const Device UA2162331 = {S70313, 43, { &P11684,&P5129,&P24670,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UCWEB8.4.0.150/50/999*/
const Device UA2167400 = {S70314, 91, { &P11684,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokia5800d-1) UCBrowser8.4.0.159/69/352/UCWEB Mobile  3gpp-gba*/
const Device UA2257744 = {S70315, 129, { &P21028,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Tab 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2483373 = {S70316, 139, { &P21028,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Smartfren Tab 7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2696231 = {S70317, 131, { &P21028,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.11; en-us; Tab 7G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2852950 = {S70318, 146, { &P21028,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; PIRANHA Pro II Tab 7.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2986624 = {S70319, 135, { &P21028,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; StarTab 715 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2665527 = {S70320, 134, { &P23848,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TAB10DUALC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3018934 = {S70321, 138, { &P23848,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TAB10DUALC 8GB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2718811 = {S70322, 95, { &P17968,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Android2.x) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2974924 = {S70323, 132, { &P13644,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; EVO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3057984 = {S70324, 139, { &P13644,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Evo 4G LTE Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2965573 = {S70325, 93, { &P14358,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae63-1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3060048 = {S70326, 92, { &P14358,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE63) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3029227 = {S70327, 155, { &P20705,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/m7spr*/
const Device UA3032597 = {S70328, 158, { &P20705,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; One Build/JWR66V.H10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/m7ul*/
const Device UA55256 = {S70329, 117, { &P13294,&P17485,&P3976,&P18092 }};	/*Useragent - Mozilla/4.0 SonyEricssonW595v/R3EE Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.3*/
const Device UA98033 = {S70330, 113, { &P15364,&P7133,&P22441,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Iron/3.0.197.0 Safari/532.0*/
const Device UA2364093 = {S70331, 113, { &P15364,&P7133,&P22441,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/531.0 (KHTML, like Gecko) Iron/3.0.189.0 Safari/531.0*/
const Device UA188706 = {S70332, 32, { &P15364,&P18144,&P17470,&P18092 }};	/*Useragent - Links (2.2; OpenBSD 4.8 i386; x)*/
const Device UA353516 = {S70333, 141, { &P21084,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; NW-Z1000Series Build/A100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA610553 = {S70334, 141, { &P21084,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; NW-Z1000Series Build/C121) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA711433 = {S70335, 141, { &P21084,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; NW-Z1000Series Build/B111) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1947366 = {S70336, 141, { &P21084,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; NW-Z1000Series Build/B110) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625560 = {S70337, 134, { &P21025,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; IS11LG Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625897 = {S70338, 70, { &P15364,&P6704,&P20900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.0.2) Gecko/X Netscape/7.02*/
const Device UA2484463 = {S70339, 75, { &P15364,&P6704,&P20900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win 9x 4.90; ja-JP; rv:1.0.1) Gecko/X Netscape/7.0*/
const Device UA886715 = {S70340, 30, { &P14358,&P5890,&P24670,&P18092 }};	/*Useragent - NokiaE63/UCWEB8.4.0.150/28/800*/
const Device UA895959 = {S70341, 79, { &P14358,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-in; NokiaE63)/UC Browser8.4.0.159/28/352/UCWEB Mobile*/
const Device UA2425054 = {S70342, 79, { &P14358,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokiae63-1) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA803600 = {S70343, 146, { &P23886,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; BASE Tab 7.1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034283 = {S70344, 196, { &P12871,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC Wildfire S A510e Build/ASN01; CyanogenMod-CM9_last_modified_by_BIT_liumiao) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2456956 = {S70345, 164, { &P12871,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Wildfire S A510e Build/ASNX; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2653324 = {S70346, 155, { &P12871,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Marvelc Build/ASNX; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029165 = {S70347, 148, { &P17168,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7310 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2385302 = {S70348, 148, { &P17168,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7310 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2194320 = {S70349, 136, { &P21229,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; iball Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2396064 = {S70350, 73, { &P20083,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-E3300/1.0 NETFRONT/4.2 PROFILE/MIDP-2.0 CONFIGURATION/CLDC-1.1*/
const Device UA2496795 = {S70351, 74, { &P20083,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-E3300L/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2671147 = {S70352, 145, { &P18626,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I547 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675695 = {S70353, 138, { &P18626,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I547C Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2767753 = {S70354, 138, { &P18626,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SAMSUNG-SGH-I547 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2701024 = {S70355, 138, { &P20712,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID4331 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Tablet Safari/534.30*/
const Device UA2708348 = {S70356, 136, { &P16263,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; dtab01 Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2879327 = {S70357, 146, { &P16263,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 10 LINK Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2969162 = {S70358, 146, { &P16263,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 7 Vogue Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3026361 = {S70359, 146, { &P16263,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 7 Youth Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3056431 = {S70360, 148, { &P16263,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 7 Lite II Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3058267 = {S70361, 143, { &P16263,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Telpad Dual S Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA20244 = {S70362, 122, { &P14912,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-C3222/C3222DDJL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA401274 = {S70363, 130, { &P15364,&P7133,&P22422,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.3.0.0 Chrome/17.0.963.46 Safari/535.11*/
const Device UA590210 = {S70364, 123, { &P15364,&P7133,&P22422,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.3.0.0 Chrome/17.0.963.46 Safari/535.11*/
const Device UA858677 = {S70365, 97, { &P15364,&P18146,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; SunOS i86pc; en-US; rv:1.7.12) Gecko/X Firefox/1.0.7 (Nexenta package 1.0.7)*/
const Device UA833063 = {S70366, 138, { &P18714,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V889D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026718 = {S70367, 126, { &P18714,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V889D Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2028727 = {S70368, 138, { &P18714,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE V889D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033833 = {S70369, 131, { &P18714,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V889D Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3042240 = {S70370, 62, { &P18714,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; ZTE V889D Build/GRK39F)*/
const Device UA2233257 = {S70371, 40, { &P17968,&P17979,&P23028,&P18092 }};	/*Useragent - Dolphin http client/9.0.0(184) (Android)*/
const Device UA2236183 = {S70372, 40, { &P17968,&P17979,&P23028,&P18092 }};	/*Useragent - Dolphin http client/9.0.1(185) (Android)*/
const Device UA2360773 = {S70373, 40, { &P17968,&P17979,&P23028,&P18092 }};	/*Useragent - Dolphin http client/9.0.2(190) (Android)*/
const Device UA2367760 = {S70374, 40, { &P17968,&P17979,&P23028,&P18092 }};	/*Useragent - Dolphin http client/9.0.3(191) (Android)*/
const Device UA2422257 = {S70375, 124, { &P12375,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIANokia N81) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile*/
const Device UA2457917 = {S70376, 46, { &P12375,&P5890,&P24673,&P18092 }};	/*Useragent - NOKIANokia N81UCBrowser/8.7.1.234/28/352/UCWEB*/
const Device UA2727699 = {S70377, 92, { &P12375,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokian81-3) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2846402 = {S70378, 128, { &P12375,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIANokia N81 8GB) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2530148 = {S70379, 142, { &P21980,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; UBISLATE7C+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3006640 = {S70380, 143, { &P21980,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; UBISLATE 7C+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050287 = {S70381, 140, { &P21980,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; UBISLATE7C+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3118957 = {S70382, 64, { &P21980,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; UBISLATE7C+ Build/IMM76D)*/
const Device UA2598827 = {S70383, 171, { &P18554,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Barnes & Noble Nook HD Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2599228 = {S70384, 170, { &P18554,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Barnes & Noble Nook HD Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768445 = {S70385, 171, { &P18554,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Barnes & Noble Nook HD Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2971203 = {S70386, 163, { &P18554,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; BN Nook HD Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1/hummingbird*/
const Device UA2670728 = {S70387, 139, { &P23855,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2672515 = {S70388, 132, { &P23855,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2895333 = {S70389, 161, { &P13123,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5360 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3003047 = {S70390, 161, { &P13123,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; GT-S5360 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2974814 = {S70391, 136, { &P23231,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E988 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3065782 = {S70392, 150, { &P18348,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G X710s Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3083134 = {S70393, 144, { &P18348,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA33329 = {S70394, 70, { &P12898,&P17483,&P3499,&P18092 }};	/*Useragent - Mozilla/4.1 (compatible; Teleca Q7; BMP 1.0.1; U; en) 240X400 LG-AN510*/
const Device UA502022 = {S70395, 101, { &P15364,&P5690,&P21959,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; Win64; x64; en-US; rv:1.9.3a5pre) Gecko/X Minefield/3.7a5pre*/
const Device UA2476388 = {S70396, 89, { &P15364,&P5690,&P21959,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.3a5pre) Gecko/X Minefield/3.7a5pre*/
const Device UA1894147 = {S70397, 132, { &P15364,&P5690,&P23735,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/527+ (KHTML, like Gecko) QtWeb Internet Browser/1.2 http://www.QtWeb.net*/
const Device UA1992587 = {S70398, 81, { &P14748,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE72-1)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2162858 = {S70399, 32, { &P14748,&P5890,&P24671,&P18092 }};	/*Useragent - NOKIAE72-1/UCWEB8.5.0.163/28/800*/
const Device UA2983491 = {S70400, 129, { &P14748,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiae72-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile  3gpp-gba*/
const Device UA2170990 = {S70401, 134, { &P18017,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Ideapad K1 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2362361 = {S70402, 126, { &P18017,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; K1 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2421173 = {S70403, 126, { &P18017,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; K1 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2380306 = {S70404, 139, { &P18738,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; OPPOR817 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2480443 = {S70405, 126, { &P18738,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R817 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2748490 = {S70406, 136, { &P18738,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R817T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2839867 = {S70407, 57, { &P18738,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; R817 Build/IMM76D)*/
const Device UA2987670 = {S70408, 58, { &P18738,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; R817T Build/IMM76D)*/
const Device UA2483298 = {S70409, 138, { &P21081,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V790 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2528951 = {S70410, 134, { &P21081,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V790 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2507979 = {S70411, 151, { &P21349,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; VIETTEL V8404 Build/HuaweiY210-0200) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2857134 = {S70412, 158, { &P21349,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; VIETTEL V8404 Unlock Build/HuaweiY210-0200) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3087003 = {S70413, 158, { &P21349,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; VIETTEL V8404 - Y210 Build/HuaweiY210-0010) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2565416 = {S70414, 134, { &P18680,&P18110,&P21418,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Safari/535.19 Silk-Accelerated=false*/
const Device UA2667073 = {S70415, 131, { &P22476,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZTE V70 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2676596 = {S70416, 137, { &P18456,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T699 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2892338 = {S70417, 154, { &P24715,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PAP4322DUO Build/PrestigioPAP4322DUO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2974686 = {S70418, 138, { &P13109,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC-X710a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA63655 = {S70419, 73, { &P11943,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-D980/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA907677 = {S70420, 51, { &P15767,&P17779,&P17470,&P18092 }};	/*Useragent - Pulp/1.5.2 (iPad; http://www.acrylicapps.com/pulp/)*/
const Device UA2335406 = {S70421, 38, { &P15767,&P17779,&P17470,&P18092 }};	/*Useragent - Apple iPad v5.1.1 YouTube v1.0.0.9B206*/
const Device UA2410290 = {S70422, 42, { &P15767,&P17779,&P17470,&P18092 }};	/*Useragent - server-bag [iPhone OS,5.1.1,9B206,iPad1,1]*/
const Device UA2596438 = {S70423, 14, { &P15767,&P17779,&P17470,&P18092 }};	/*Useragent - eBayiPad/2.2.1*/
const Device UA423489 = {S70424, 150, { &P12521,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonSO-02D Build/6.0.A.4.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA604844 = {S70425, 150, { &P12521,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonSO-02D Build/6.0.A.5.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA605522 = {S70426, 138, { &P12521,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SO-02D Build/6.0.A.5.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA659512 = {S70427, 138, { &P12521,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SO-02D Build/6.0.A.4.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA864571 = {S70428, 151, { &P12521,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonSO-02D Build/6.0.A.5.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA886131 = {S70429, 139, { &P12521,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SO-02D Build/6.0.A.5.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1966365 = {S70430, 63, { &P12521,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; SO-02D Build/6.0.A.5.12)*/
const Device UA2390952 = {S70431, 151, { &P12521,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SonyEricssonSO-02D Build/6.0.A.5.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2727833 = {S70432, 139, { &P12521,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ja-jp; SO-02D Build/6.0.A.5.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1997065 = {S70433, 136, { &P17595,&P17296,&P22460,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11*/
const Device UA2547845 = {S70434, 136, { &P17595,&P17296,&P22460,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11*/
const Device UA2507823 = {S70435, 62, { &P15364,&P18144,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64; rv:18.0) Gecko/X Firefox/18.0*/
const Device UA2508083 = {S70436, 160, { &P18224,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Ascend G300 Build/JOP40C; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567671 = {S70437, 160, { &P18224,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Ascend G300 Build/JOP40C; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2670209 = {S70438, 149, { &P12280,&P18117,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B179 Maxthon/4.0.4.1000 Safari/537.4*/
const Device UA2749186 = {S70439, 138, { &P22941,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6812 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2965516 = {S70440, 139, { &P22941,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6812B Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2890008 = {S70441, 88, { &P15364,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A11) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2890083 = {S70442, 99, { &P15364,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; iBuddy_Connect) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2934660 = {S70443, 97, { &P15364,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Videocon_A30) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2934704 = {S70444, 87, { &P15364,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; V1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2936443 = {S70445, 98, { &P15364,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax_A101) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2953558 = {S70446, 88, { &P15364,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MID) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2955529 = {S70447, 97, { &P15364,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; Penta_IS701C) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2987770 = {S70448, 87, { &P15364,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Y3) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2988271 = {S70449, 87, { &P15364,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; U1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2990304 = {S70450, 89, { &P15364,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; VP10) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2996516 = {S70451, 95, { &P15364,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Connect-3G) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3045420 = {S70452, 88, { &P15364,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; W60) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3049476 = {S70453, 93, { &P15364,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Cloud_Z5) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3087229 = {S70454, 89, { &P15364,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; CT-1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3120957 = {S70455, 88, { &P15364,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; W70) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3121000 = {S70456, 91, { &P15364,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; Lafone) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3068418 = {S70457, 133, { &P20690,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; V360 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA56054 = {S70458, 32, { &P14617,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH01B(c500;TB;W24H16)*/
const Device UA90285 = {S70459, 32, { &P14617,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH01C(c500;TB;W24H16)*/
const Device UA284800 = {S70460, 32, { &P14617,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH01A(c100;TB;W24H16)*/
const Device UA61855 = {S70461, 116, { &P16616,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2530/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA1968691 = {S70462, 137, { &P18743,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOR805 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029125 = {S70463, 126, { &P18743,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; R805 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2031526 = {S70464, 121, { &P18743,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; R805 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2820855 = {S70465, 57, { &P18743,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; R805 Build/GRK39F)*/
const Device UA1995099 = {S70466, 62, { &P15364,&P18146,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; SunOS i86pc; rv:14.0) Gecko/X Firefox/14.0.1*/
const Device UA2772486 = {S70467, 129, { &P15767,&P17146,&P22902,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.4.2 Mobile/9A405 Safari/7534.48.3*/
const Device UA3121506 = {S70468, 127, { &P15767,&P17146,&P22902,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.4.2 Mobile/9A334 Safari/7534.48.3*/
const Device UA2956396 = {S70469, 128, { &P23975,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B1-710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3081240 = {S70470, 145, { &P18476,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I577 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083207 = {S70471, 140, { &P21141,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Lenovo P770 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109250 = {S70472, 140, { &P21141,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Lenovo P770 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA481855 = {S70473, 79, { &P17595,&P18120,&P22502,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en; rv:1.8.1.8) Gecko/X Camino/1.5.2*/
const Device UA2439921 = {S70474, 87, { &P17595,&P18120,&P22502,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X Mach-O; en; rv:1.8.1.12) Gecko/X Camino/1.5.5*/
const Device UA606261 = {S70475, 125, { &P15364,&P7133,&P22427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.152 Safari/535.19 Comodo_Dragon/18.1.2.0*/
const Device UA606560 = {S70476, 132, { &P15364,&P7133,&P22427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.152 Safari/535.19 Comodo_Dragon/18.1.2.0*/
const Device UA771622 = {S70477, 92, { &P21527,&P5629,&P15760,&P18092 }};	/*Useragent - CoolPad8150_CMCC_TD/1.0 Linux/2.6.35 Android/2.3 Release/12.25.2011 Browser/AppleWebkit533.1*/
const Device UA866977 = {S70478, 120, { &P21527,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8150 Build/GWK74) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2026790 = {S70479, 132, { &P21527,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8150 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028448 = {S70480, 125, { &P21527,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8150 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2374254 = {S70481, 125, { &P21527,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.7; zh-cn; 8150 Build GWK74) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA830148 = {S70482, 138, { &P15364,&P17350,&P23729,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT based; en-US) AppleWebKit/533.3 (KHTML, like Gecko)  QtWeb Internet Browser/3.7.5 http://www.QtWeb.net*/
const Device UA2145160 = {S70483, 138, { &P15364,&P17350,&P23729,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT based; en-US) AppleWebKit/533.3 (KHTML, like Gecko)  QtWeb Internet Browser/3.7.3 http://www.QtWeb.net*/
const Device UA2228559 = {S70484, 131, { &P20832,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MIDC497 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2425990 = {S70485, 131, { &P20832,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MIDC497 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2566918 = {S70486, 134, { &P19788,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; E350 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2624546 = {S70487, 192, { &P15882,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +1300, nz*/
const Device UA2628347 = {S70488, 192, { &P15882,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.20/286,1.9.20.286,2013-01-25 13:38, +0100, de*/
const Device UA2632354 = {S70489, 192, { &P15882,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0100, es*/
const Device UA2754944 = {S70490, 132, { &P13644,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; EVO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754246 = {S70491, 154, { &P21857,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2781341 = {S70492, 131, { &P22943,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; E270BSA Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2937598 = {S70493, 60, { &P22943,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; E270BSA Build/JRO03H)*/
const Device UA2974720 = {S70494, 154, { &P20088,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E-51 Build/HuaweiU8666E-51) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021717 = {S70495, 148, { &P20088,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E Build/HuaweiU8666E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3028229 = {S70496, 148, { &P20088,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666N Build/HuaweiU8666N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2987134 = {S70497, 130, { &P24597,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; AT10-A Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3107978 = {S70498, 59, { &P24597,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; AT10-A Build/JOP40D)*/
const Device UA3082864 = {S70499, 130, { &P21012,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I815 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA22409 = {S70500, 51, { &P15458,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-W845/00.72 UP.Browser/7.2.7.5.663 (GUI) MMP/2.0*/
const Device UA804323 = {S70501, 153, { &P13070,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-E739 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025033 = {S70502, 157, { &P13070,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-E739 Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2381754 = {S70503, 150, { &P15028,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; LG-P350 Build/GRJ90; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2453701 = {S70504, 151, { &P15028,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; LG-P350 Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2742986 = {S70505, 154, { &P15028,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; LG-P350 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2812353 = {S70506, 157, { &P15028,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; Optimus Me Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030247 = {S70507, 138, { &P14959,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9103 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2403090 = {S70508, 151, { &P21520,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cloudfone Thrill 430g Build/SKV05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2424961 = {S70509, 151, { &P21520,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cloudfone Thrill 430g Build/FSV04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2457375 = {S70510, 132, { &P13452,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P1000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2466270 = {S70511, 61, { &P13452,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P1000 Build/JZO54K)*/
const Device UA2610953 = {S70512, 132, { &P13452,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P1000 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615487 = {S70513, 61, { &P13452,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P1000 Build/JRO03R)*/
const Device UA2615535 = {S70514, 132, { &P13452,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; GT-P1000 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616731 = {S70515, 130, { &P13452,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P1000 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2617768 = {S70516, 132, { &P13452,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P1000 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2618722 = {S70517, 75, { &P13452,&P18111,&P18112,&P18092 }};	/*Useragent - GoogleAnalytics/2.0 (Linux; U; Android 4.1.2; ko-kr; GT-P1000 Build/JZO54K)*/
const Device UA2619531 = {S70518, 145, { &P13452,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P1000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.6 Mobile Safari/534.30*/
const Device UA2620934 = {S70519, 139, { &P13452,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P1000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2621513 = {S70520, 62, { &P13452,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P1000N Build/JZO54K)*/
const Device UA2623141 = {S70521, 110, { &P13452,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.1.2; en-GB; GT-P1000 Build/cm_p1-userdebug 4.1.2 JZO54K eng..X.X test-keys)*/
const Device UA2625697 = {S70522, 139, { &P13452,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P1000 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2630827 = {S70523, 124, { &P13452,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P1000) antivirusAM/2.4.5 (KHTML, like Gecko) Version/Version 2.4.5/antivirus*/
const Device UA2632539 = {S70524, 142, { &P13452,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.0; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2633682 = {S70525, 127, { &P13452,&P18111,&P18112,&P18092 }};	/*Useragent - Groupon/2.3.2787 (Android 4.1.2; Samsung GT-P1000 / Samsung GT-P1000; )[preload=false;locale=en_CA;clientidbase=android-google]*/
const Device UA2634250 = {S70526, 157, { &P13452,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P1000 Build/Smartfren.andromaxtab7.0.v.41.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2635294 = {S70527, 133, { &P13452,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-P1000N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2676729 = {S70528, 130, { &P14908,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6210 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2664131 = {S70529, 147, { &P18114,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Transformer Prime TF201 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2667309 = {S70530, 140, { &P20738,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Andromax U Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2794919 = {S70531, 133, { &P20738,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Andromax U Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2670800 = {S70532, 113, { &P15767,&P18432,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/2.0*/
const Device UA2674302 = {S70533, 111, { &P15767,&P18432,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Flipboard/2.0*/
const Device UA2714936 = {S70534, 115, { &P15767,&P18432,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/2.0.1*/
const Device UA2769184 = {S70535, 115, { &P15767,&P18432,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/2.0.2*/
const Device UA2769779 = {S70536, 113, { &P15767,&P18432,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Flipboard/2.0.2*/
const Device UA2862221 = {S70537, 113, { &P15767,&P18432,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Flipboard/2.0.1*/
const Device UA2879733 = {S70538, 115, { &P15767,&P18432,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/2.0.3*/
const Device UA2961649 = {S70539, 115, { &P15767,&P18432,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/2.0.4*/
const Device UA3013466 = {S70540, 113, { &P15767,&P18432,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Flipboard/2.0.4*/
const Device UA3109174 = {S70541, 115, { &P15767,&P18432,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/2.0.6*/
const Device UA2710912 = {S70542, 138, { &P20963,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SBM200SH Build/S1002) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768396 = {S70543, 60, { &P20963,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SBM200SH Build/S1002)*/
const Device UA3030657 = {S70544, 138, { &P20963,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SBM200SH Build/S1004) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2772526 = {S70545, 161, { &P21020,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Micromax A116 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2753127 = {S70546, 140, { &P21542,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6503 Build/10.1.A.1.434) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774335 = {S70547, 157, { &P12972,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2796391 = {S70548, 157, { &P12972,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2800096 = {S70549, 158, { &P12972,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2893202 = {S70550, 158, { &P12972,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989D Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2974714 = {S70551, 142, { &P19862,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax A110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3030372 = {S70552, 157, { &P19861,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3021485 = {S70553, 136, { &P20146,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9082 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA31102 = {S70554, 98, { &P12398,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-A707/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA33294 = {S70555, 41, { &P12702,&P17484,&P3625,&P18092 }};	/*Useragent - sam-r260 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA251354 = {S70556, 129, { &P15364,&P7133,&P22412,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.2.2.0 Chrome/14.0.835.202 Safari/535.1*/
const Device UA251358 = {S70557, 122, { &P15364,&P7133,&P22412,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.2.2.0 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2024292 = {S70558, 137, { &P18281,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P940 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2338759 = {S70559, 139, { &P18281,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-KU5400 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2446933 = {S70560, 138, { &P18281,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-SU540 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2341776 = {S70561, 81, { &P16292,&P5629,&P15760,&P18092 }};	/*Useragent - SAMSUNG-GT-I9050_TD/1.0 Android/2.3.6 Release/01.15.2012 Browser/AppleWebKit533.1*/
const Device UA2375882 = {S70562, 135, { &P16292,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9050 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2522469 = {S70563, 142, { &P16292,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9050 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2981529 = {S70564, 66, { &P16292,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GT-I9050 Build/GINGERBREAD)*/
const Device UA2984285 = {S70565, 134, { &P16292,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; I9250 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2424261 = {S70566, 133, { &P18731,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1;  ONDA MID Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2429417 = {S70567, 132, { &P18731,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ONDA MID Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2505694 = {S70568, 138, { &P18731,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ONDA MID Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2984341 = {S70569, 139, { &P18731,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1;  ONDA MID Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA2507694 = {S70570, 95, { &P14202,&P5477,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E15i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2705991 = {S70571, 95, { &P14202,&P5477,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E15a) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2557673 = {S70572, 131, { &P23511,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SC-91JB Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2659672 = {S70573, 74, { &P18141,&P22274,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 2.10) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2676932 = {S70574, 134, { &P17160,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Slider SL101 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2753877 = {S70575, 136, { &P14123,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P970 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877407 = {S70576, 137, { &P14123,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P970h Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2753885 = {S70577, 139, { &P20738,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Andromax U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2974723 = {S70578, 130, { &P21012,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I815 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974636 = {S70579, 140, { &P18449,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST27i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022547 = {S70580, 140, { &P18449,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST27a Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3082742 = {S70581, 130, { &P17337,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7500 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3132865 = {S70582, 129, { &P18554,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BNTV400 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA26788 = {S70583, 73, { &P13634,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GS500-parrot/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63233 = {S70584, 66, { &P13634,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2262086 = {S70585, 110, { &P20403,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE NOVA/G-R3100-Hollywood-Orange/R3100_V4_Z0_FR_D18GOUK104 Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2419335 = {S70586, 111, { &P20403,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE NOVA/G-R3100-Hollywood-Tmobile/R3100_V4_Z1_FR_D18GTMO101 Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2790285 = {S70587, 110, { &P20403,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE NOVA/G-R3100-Hollywood-Orange/R3100_V7_Z3_ES_D18GOSP101 Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2406470 = {S70588, 139, { &P11751,&P17483,&P24673,&P18092 }};	/*Useragent - LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-kp500 teleca) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2581805 = {S70589, 97, { &P20729,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A27) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2715161 = {S70590, 68, { &P20104,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; 201M Build/9.8.2Q-34_SMJ-102)*/
const Device UA2715757 = {S70591, 146, { &P20104,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; 201M Build/9.8.2Q-34_SMJ-102) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2754273 = {S70592, 137, { &P19861,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I605 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2899123 = {S70593, 155, { &P21261,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9500 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2799702 = {S70594, 135, { &P23395,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T5-A Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2943699 = {S70595, 152, { &P23395,&P18066,&P18062,&P18092 }};	/*Useragent - T5-A Linux/3.0.13 Android/4.0.4 Release/05.16.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA3080013 = {S70596, 137, { &P18722,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A830L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080211 = {S70597, 137, { &P18722,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A830K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3082815 = {S70598, 137, { &P18722,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A830S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109122 = {S70599, 138, { &P18722,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A830KE Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA6250 = {S70600, 143, { &P13668,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; es-es; htc_dream Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA25467 = {S70601, 146, { &P13668,&P6757,&P3525,&P18092 }};	/*Useragent - HTC_Dream Mozilla/5.0 (Linux; U; Android 1.5; en-ca; Build/1.86.631.1) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA25468 = {S70602, 143, { &P13668,&P6757,&P3525,&P18092 }};	/*Useragent - HTC_Dream Mozilla/5.0 (Linux; U; Android 1.5; en-ca; Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA31297 = {S70603, 143, { &P13668,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-za; HTC_Dream Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA50082 = {S70604, 141, { &P13668,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-sg; HTC Dream Build/CRC28) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA50296 = {S70605, 143, { &P13668,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-za; HTC Dream Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2520518 = {S70606, 143, { &P13668,&P6757,&P3525,&P18092 }};	/*Useragent - HTC_Dream Mozilla/5.0 (Linux; U; Android 1.5; en-ca; Build/CUPCAKE) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA27298 = {S70607, 105, { &P12168,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonW508a/R1GA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.4*/
const Device UA226990 = {S70608, 132, { &P13526,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; myTouch4G; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA1902753 = {S70609, 136, { &P13526,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_myTouch4G; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA241414 = {S70610, 122, { &P15364,&P7133,&P22409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.4.0.0 Chrome/13.0.782.220 Safari/535.1*/
const Device UA241779 = {S70611, 129, { &P15364,&P7133,&P22409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.4.0.0 Chrome/13.0.782.220 Safari/535.1*/
const Device UA768212 = {S70612, 139, { &P14183,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I997 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1991039 = {S70613, 139, { &P14183,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I997 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025235 = {S70614, 134, { &P14183,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I997 Build/4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026888 = {S70615, 139, { &P14183,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I997 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA910575 = {S70616, 138, { &P13999,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NexusHD2 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1968549 = {S70617, 137, { &P13999,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC HD2 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2465097 = {S70618, 133, { &P13999,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; Hd2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2653638 = {S70619, 144, { &P13999,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC HD2 EvoHd2 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2387021 = {S70620, 118, { &P12581,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia500) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile*/
const Device UA2384196 = {S70621, 66, { &P15397,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; ST18i; 480*854) UCWEB7.9.0.94/139/352*/
const Device UA2385100 = {S70622, 66, { &P15397,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; ST18i; 480*854) UCWEB7.9.0.94/139/444*/
const Device UA2354579 = {S70623, 132, { &P18478,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P6201 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2597296 = {S70624, 130, { &P18478,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SC-02D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616096 = {S70625, 61, { &P18478,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P6201 Build/IMM76D)*/
const Device UA2628038 = {S70626, 93, { &P18478,&P18066,&P18062,&P18092 }};	/*Useragent - com.google.android.youtube/4.2.16(Linux; U; Android 4.0.4; de_DE; GT-P6201 Build/IMM76D) gzip*/
const Device UA2631578 = {S70627, 130, { &P18478,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P6201 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2484308 = {S70628, 149, { &P20221,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Starmobile CRYSTAL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2600903 = {S70629, 127, { &P14111,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Arc Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2885303 = {S70630, 112, { &P21003,&P20706,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia310/07.35; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA2974719 = {S70631, 136, { &P18621,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E610 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021967 = {S70632, 137, { &P18621,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E612f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022610 = {S70633, 136, { &P18621,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E612 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023916 = {S70634, 137, { &P18621,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E610v Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA8856 = {S70635, 74, { &P13354,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-F102 H3G/1.0 ACS-NF/3.4 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA21877 = {S70636, 73, { &P13354,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-F102i/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA24721 = {S70637, 72, { &P13354,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-F102/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63581 = {S70638, 74, { &P13354,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-F102 H3G/2.0 ACS-NF/3.4 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA50280 = {S70639, 128, { &P13214,&P17484,&P4732,&P18092 }};	/*Useragent - SAMSUNG-SGH-A767/A767UCIF2; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.14315; U; en-US) Opera 9.50*/
const Device UA507789 = {S70640, 45, { &P15364,&P17486,&P17602,&P18092 }};	/*Useragent - MOT-KINGFISHER Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA1994146 = {S70641, 40, { &P15364,&P17486,&P17602,&P18092 }};	/*Useragent - MOT-EX116 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA632662 = {S70642, 78, { &P11945,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; gt-s5233s) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA739718 = {S70643, 143, { &P18734,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; AN10BG2DT Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1979251 = {S70644, 141, { &P18734,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; fr-fr; AN10BG2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2230863 = {S70645, 144, { &P18282,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A200 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422203 = {S70646, 144, { &P18282,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; a200 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2425192 = {S70647, 155, { &P14848,&P5843,&P24673,&P18092 }};	/*Useragent - NokiaX3-00/5.0 (11.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiax3-00) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2535267 = {S70648, 135, { &P17595,&P17297,&P22464,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1350.0 Iron/24.0.1350.0 Safari/537.17*/
const Device UA2675515 = {S70649, 137, { &P18697,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I415 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676263 = {S70650, 128, { &P19783,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ME172V Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676917 = {S70651, 126, { &P12765,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3031263 = {S70652, 157, { &P18283,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3092927 = {S70653, 157, { &P18283,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I535 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA14144 = {S70654, 61, { &P11791,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia7500/2.0 (04.62) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA90394 = {S70655, 132, { &P18025,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS06 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018251 = {S70656, 130, { &P18025,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-; IS06 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018497 = {S70657, 56, { &P18025,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; IS06 Build/FRG83)*/
const Device UA778930 = {S70658, 151, { &P12955,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Blade Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA909516 = {S70659, 151, { &P12955,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Blade Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1959686 = {S70660, 151, { &P12955,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Blade Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025578 = {S70661, 155, { &P12955,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ZTE Blade Build/ITL41D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984573 = {S70662, 157, { &P12955,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZTE Blade Build/IMM76D; CyanogenMod-Eco) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528029 = {S70663, 26, { &P18283,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-T999/100.40101*/
const Device UA2578740 = {S70664, 26, { &P18283,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-I9300/100.40102*/
const Device UA2667741 = {S70665, 26, { &P18283,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SCH-I535/100.40102*/
const Device UA1977916 = {S70666, 148, { &P21097,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Mobiistar_Touch_S01 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2810890 = {S70667, 72, { &P21097,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; Mobiistar Touch S01 Build/GRK39F)*/
const Device UA2424886 = {S70668, 118, { &P12581,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia500) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2424416 = {S70669, 134, { &P20129,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MI 2 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2752231 = {S70670, 135, { &P20129,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MI 2S Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3030525 = {S70671, 136, { &P20129,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MI 2SC Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2715110 = {S70672, 142, { &P23106,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TA-FONE A34 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2962442 = {S70673, 145, { &P23106,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TA-FONE A34 HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2793354 = {S70674, 136, { &P18740,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; U7015 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997617 = {S70675, 58, { &P18740,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; U7015 Build/JRO03C)*/
const Device UA191934 = {S70676, 143, { &P15364,&P17817,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ Ubuntu/10.10 (2.30.6-1ubuntu5) Epiphany/2.30.6*/
const Device UA285276 = {S70677, 136, { &P15364,&P17817,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-us) AppleWebKit/531.2+ (KHTML, like Gecko) Version/5.0 Safari/531.2+ Ubuntu/10.10 () Epiphany/2.30.2*/
const Device UA1982375 = {S70678, 130, { &P21275,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M1050S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2030693 = {S70679, 130, { &P21275,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M1050S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1984421 = {S70680, 149, { &P19997,&P17979,&P18062,&P18092 }};	/*Useragent - Lenovo P700/S100 Linux/3.0.13 Android/4.0.3 Release/12.12.2011 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30*/
const Device UA2382231 = {S70681, 122, { &P19997,&P17979,&P18062,&P18092 }};	/*Useragent - Lenovo P700iLenovo-P700i/S101 Linux/3.0.13 Android/4.0 Release/05.28.2012 Browser/AppleWebKit533.1 Profile/ Configuration/*/
const Device UA2428482 = {S70682, 133, { &P20138,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099631 = {S70683, 138, { &P20138,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Vsun i1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2508707 = {S70684, 91, { &P14453,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaE51) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2580811 = {S70685, 136, { &P21878,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; 201F Build/V31R11K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2670689 = {S70686, 136, { &P21878,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; 201F Build/V33R12A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734078 = {S70687, 58, { &P21878,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; 201F Build/V33R12A)*/
const Device UA3029382 = {S70688, 136, { &P21878,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; 201F Build/V35R13E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2754629 = {S70689, 136, { &P13074,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P930 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA5616 = {S70690, 70, { &P15425,&P5448,&P3430,&P18092 }};	/*Useragent - SAMSUNG-GT-B7300Orange/BVIJ1 (Windows CE; Opera Mobi; U; en) Opera 9.5*/
const Device UA64655 = {S70691, 62, { &P15425,&P5448,&P3430,&P18092 }};	/*Useragent - SAMSUNG-GT-B7300/1.0 (Windows CE; Opera Mobi; U; en) Opera 9.5*/
const Device UA62525 = {S70692, 83, { &P23010,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK790i/R8BF Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA163338 = {S70693, 71, { &P18054,&P17483,&P3499,&P18092 }};	/*Useragent - Mozilla/4.1 (compatible; Teleca Q7; BMP 1.0.1; U; en) 240X400 LGE UN510*/
const Device UA428930 = {S70694, 123, { &P15364,&P7133,&P22424,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11 Comodo_Dragon/17.5.2.0*/
const Device UA428931 = {S70695, 130, { &P15364,&P7133,&P22424,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11 Comodo_Dragon/17.5.2.0*/
const Device UA607624 = {S70696, 154, { &P12482,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959 Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA736162 = {S70697, 154, { &P12482,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA902278 = {S70698, 154, { &P12482,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA614893 = {S70699, 72, { &P15364,&P5690,&P22357,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:12.0) Gecko/X Firefox/12.0 PaleMoon/12.0*/
const Device UA688143 = {S70700, 92, { &P15364,&P5690,&P22357,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:12.0) Gecko/X Firefox/12.0-x64 PaleMoon/12.0-x64*/
const Device UA2514707 = {S70701, 152, { &P17346,&P19826,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B144 Safari/7534.48.3*/
const Device UA2564878 = {S70702, 154, { &P17346,&P19826,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B146 Safari/7534.48.3*/
const Device UA2670934 = {S70703, 154, { &P17346,&P19826,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/10B329 Safari/7534.48.3*/
const Device UA2599405 = {S70704, 135, { &P18113,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT912 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936735 = {S70705, 58, { &P18113,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; XT912 Build/JDQ39E)*/
const Device UA3012460 = {S70706, 142, { &P18113,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Droid Razr M Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2753284 = {S70707, 130, { &P17323,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6800 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2764113 = {S70708, 137, { &P22956,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P875 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3097273 = {S70709, 112, { &P19973,&P5043,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia114/03.33; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA22395 = {S70710, 118, { &P11713,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonG705u/R1BA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 model-orange JavaPlatform/JP-8.4.0*/
const Device UA64838 = {S70711, 104, { &P11713,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonG705/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3*/
const Device UA8860 = {S70712, 73, { &P11804,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GW525-parrot/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA27585 = {S70713, 40, { &P11804,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GW525 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA35105 = {S70714, 67, { &P11804,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GW525g/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA399181 = {S70715, 33, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/D772/Screen-176X220*/
const Device UA430906 = {S70716, 50, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/DT200/MIDP2.0/CLDC1.1/Screen-240x320*/
const Device UA443976 = {S70717, 33, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/A610/Screen-176X220*/
const Device UA459948 = {S70718, 33, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/D159/Screen-128X128*/
const Device UA459950 = {S70719, 33, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/D175/Screen-128X160*/
const Device UA461225 = {S70720, 48, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/X90/MIDP2.0/CLDC1.1/Screen-320X240*/
const Device UA492887 = {S70721, 49, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/A927/MIDP2.0/CLDC1.1/Screen-240x320*/
const Device UA561700 = {S70722, 49, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/FT05/MIDP2.0/CLDC1.1/Screen-240X320*/
const Device UA629386 = {S70723, 33, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/B858/Screen-240X320*/
const Device UA629606 = {S70724, 49, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/V320/MIDP2.0/CLDC1.1/Screen-240X320*/
const Device UA631739 = {S70725, 49, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/B922/MIDP2.0/CLDC1.1/Screen-240X320*/
const Device UA633081 = {S70726, 49, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/C350/MIDP2.0/CLDC1.1/Screen-240X320*/
const Device UA635832 = {S70727, 34, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/B2012/Screen-128X128*/
const Device UA640522 = {S70728, 34, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/B2016/Screen-128X128*/
const Device UA641484 = {S70729, 33, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/D172/Screen-128x160*/
const Device UA642771 = {S70730, 49, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/A660/MIDP2.0/CLDC1.1/Screen-240x320*/
const Device UA644363 = {S70731, 49, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/A662/MIDP2.0/CLDC1.1/Screen-240X320*/
const Device UA648399 = {S70732, 33, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/H711/Screen-176X220*/
const Device UA651406 = {S70733, 49, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/A662/MIDP2.0/CLDC1.1/Screen-240X400*/
const Device UA654819 = {S70734, 49, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/A168/MIDP2.0/CLDC1.1/Screen-320X430*/
const Device UA1859990 = {S70735, 49, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/V760/MIDP2.0/CLDC1.1/Screen-240X400*/
const Device UA2125528 = {S70736, 49, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/DT28/MIDP2.0/CLDC1.1/Screen-240x320*/
const Device UA2263820 = {S70737, 49, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/W990/MIDP2.0/CLDC1.1/Screen-240x320*/
const Device UA2264019 = {S70738, 49, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/A995/MIDP2.0/CLDC1.1/Screen-320x430*/
const Device UA2264151 = {S70739, 33, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/D173/Screen-128x160*/
const Device UA2265264 = {S70740, 49, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/A903/MIDP2.0/CLDC1.1/Screen-240X320*/
const Device UA2267030 = {S70741, 49, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/C800/MIDP2.0/CLDC1.1/Screen-240x320*/
const Device UA2267646 = {S70742, 49, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/H800/MIDP2.0/CLDC1.1/Screen-176X220*/
const Device UA2439250 = {S70743, 33, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/Q981/Screen-240X320*/
const Device UA2493062 = {S70744, 49, { &P15364,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/M600/MIDP2.0/CLDC1.1/Screen-240X320*/
const Device UA608695 = {S70745, 137, { &P21308,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; CTP810 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1984823 = {S70746, 137, { &P21308,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; CTP810 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1972879 = {S70747, 86, { &P16384,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s5380k) UCBrowser8.3.0.154/69/405/UCWEB Mobile*/
const Device UA1989951 = {S70748, 86, { &P16384,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5380k) UCBrowser8.3.1.161/69/405/UCWEB Mobile*/
const Device UA2029181 = {S70749, 136, { &P12731,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB855 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031905 = {S70750, 136, { &P12731,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB855 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2239099 = {S70751, 136, { &P12731,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB855 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2354263 = {S70752, 136, { &P12731,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MB855 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2232521 = {S70753, 166, { &P15919,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P3100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2383352 = {S70754, 166, { &P15919,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2619989 = {S70755, 167, { &P15919,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3100B Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2429931 = {S70756, 151, { &P18506,&P5043,&P24675,&P18092 }};	/*Useragent - Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia305) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2555832 = {S70757, 139, { &P17324,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9001 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2594641 = {S70758, 61, { &P17324,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-i9001 Build/JZO54K)*/
const Device UA2773123 = {S70759, 142, { &P22862,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Micromax A88 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2879570 = {S70760, 135, { &P22862,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Micromax A88 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22*/
const Device UA2882667 = {S70761, 133, { &P18442,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3001344 = {S70762, 133, { &P18442,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; SGH-T889 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3001508 = {S70763, 141, { &P18442,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I317 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3011853 = {S70764, 133, { &P18442,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; SPH-L900 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3051554 = {S70765, 133, { &P18442,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.1; en-gb; GT-N7100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2974592 = {S70766, 138, { &P21065,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; CINK FIVE Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021090 = {S70767, 134, { &P21065,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CINK+ Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021755 = {S70768, 138, { &P24114,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; LG-E440 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3077801 = {S70769, 60, { &P24114,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E440 Build/JZO54K)*/
const Device UA3108294 = {S70770, 153, { &P24114,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; LG-E440/E44010a-EUR-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3026039 = {S70771, 113, { &P14669,&P5843,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-00/08.25; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3067902 = {S70772, 152, { &P22960,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA10053 = {S70773, 99, { &P12191,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5; garmin-asus-Nuvifone-M10/1.0)*/
const Device UA27940 = {S70774, 107, { &P12191,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5) garmin-asus-Nuvifone-M10/1.0 Opera9.5*/
const Device UA27941 = {S70775, 99, { &P12191,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5; garmin-asus-Nuvifone-M10/2.0)*/
const Device UA47607 = {S70776, 129, { &P17281,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; SCH-I905 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA47616 = {S70777, 132, { &P17281,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; SCH-I905 4G Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA395337 = {S70778, 58, { &P17281,&P6546,&P4459,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.1; SCH-I905 Build/HMJ37)*/
const Device UA48971 = {S70779, 142, { &P13880,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; T-01C Build/TER018) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA657518 = {S70780, 64, { &P13880,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; T-01C Build/TER018)*/
const Device UA70529 = {S70781, 136, { &P21143,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-i809 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA668510 = {S70782, 136, { &P21143,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SCH-i809 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA715273 = {S70783, 124, { &P21143,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-i809 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1954676 = {S70784, 136, { &P21143,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; zh-cn; SCH-i809 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020310 = {S70785, 124, { &P21143,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SCH-i809 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2033408 = {S70786, 129, { &P21143,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-i809 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2814925 = {S70787, 129, { &P21143,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; zh-cn; SCH-i809 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA153490 = {S70788, 145, { &P18004,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; CBW Blaze Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA463482 = {S70789, 85, { &P17595,&P18120,&P22498,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.7.2) Gecko/X Camino/0.8.1*/
const Device UA465143 = {S70790, 51, { &P15364,&P7127,&P17384,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows NT 5.0)*/
const Device UA495327 = {S70791, 50, { &P15364,&P7127,&P17384,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.0; Windows NT 5.0)*/
const Device UA2376048 = {S70792, 49, { &P15364,&P7127,&P17384,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.0; WindowsNT 5.0)*/
const Device UA2792798 = {S70793, 63, { &P15364,&P7127,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; Windows NT 5.0; Trident/1.0)*/
const Device UA573556 = {S70794, 70, { &P14186,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.4;Zh_cn;GT-S5830;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA753712 = {S70795, 70, { &P14186,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.3;Zh_cn;GT-S5830;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA1986552 = {S70796, 70, { &P14186,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;GT-S5830;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA2371681 = {S70797, 70, { &P14186,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.5;Zh_cn;GT-S5830;320*480;)UCWEB7.8.0.95/139/352*/
const Device UA2350847 = {S70798, 41, { &P15767,&P17779,&P18431,&P18092 }};	/*Useragent - server-bag [iPhone OS,6.0,10A403,iPad3,1]*/
const Device UA2694602 = {S70799, 22, { &P15767,&P17779,&P18431,&P18092 }};	/*Useragent - iPad2,1/6.0.1 (10A523)*/
const Device UA2694663 = {S70800, 22, { &P15767,&P17779,&P18431,&P18092 }};	/*Useragent - iPad3,1/6.0.1 (10A523)*/
const Device UA2827925 = {S70801, 20, { &P15767,&P17779,&P18431,&P18092 }};	/*Useragent - iPad3,1/6.0 (10A403)*/
const Device UA2839656 = {S70802, 22, { &P15767,&P17779,&P18431,&P18092 }};	/*Useragent - iPad3,3/6.0.1 (10A523)*/
const Device UA2383478 = {S70803, 57, { &P20051,&P22784,&P21795,&P18092 }};	/*Useragent - MicromaxX279/MstarBrowser1.0/MIDP2.0Configuration/CLDC1.1*/
const Device UA2756014 = {S70804, 136, { &P18653,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P870 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869473 = {S70805, 130, { &P24536,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; VL-110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA654552 = {S70806, 66, { &P21988,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.1; PD_Novel Build/GINGERBREAD)*/
const Device UA667277 = {S70807, 148, { &P21988,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PD_Novel Build/PR71_A_BN_W_A_C_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA691020 = {S70808, 148, { &P21988,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PD_Novel Build/PR72_A_BN_W_A_C_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1944367 = {S70809, 148, { &P21988,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PD_Novel Build/PR77_A_KZ_W_A_C_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024670 = {S70810, 148, { &P21988,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PD_Novel Build/PR73_A_BN_W_A_C_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025768 = {S70811, 148, { &P21988,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PD_Novel Build/PR74_A_AM_W_A_C_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2807825 = {S70812, 148, { &P21988,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PD_Novel Build/PR76_A_KZ_W_A_C_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA842084 = {S70813, 151, { &P12848,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB526 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1982567 = {S70814, 151, { &P12848,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB526 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2170361 = {S70815, 132, { &P18465,&P18110,&P19989,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.0 Safari/535.19 Silk-Accelerated=true*/
const Device UA2483181 = {S70816, 70, { &P18283,&P18111,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.1.2; zh-cn; GT-I9300; 720*1280) UCWEB7.9.0.94/139/444*/
const Device UA2467227 = {S70817, 150, { &P18461,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Grand X In Build/GXI_TPLV1.0.0B04) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2530969 = {S70818, 128, { &P23841,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-TSM_T9i/1.0 Linux/2.6.35.7 Android/2.3.7 Release/01.05.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2556695 = {S70819, 128, { &P23841,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-TSM_T9i/1.0 Linux/2.6.35.7 Android/2.3.7 Release/12.18.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2708898 = {S70820, 128, { &P23841,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-TSM_T9i/1.0 Linux/2.6.35.7 Android/2.3.7 Release/01.28.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3045379 = {S70821, 128, { &P23841,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-TSM_T9i/1.0 Linux/2.6.35.7 Android/2.3.7 Release/01.07.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3099625 = {S70822, 145, { &P23841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; TSM_T9i Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3107529 = {S70823, 211, { &P23841,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; sprd-TSM_T9i/1.0 Android/2.3.7 Release/05.31.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2617799 = {S70824, 183, { &P15919,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.7/210,1.9.7.210,2012-10-25 19:53, in*/
const Device UA2625735 = {S70825, 193, { &P15919,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +0700, kh*/
const Device UA2630384 = {S70826, 192, { &P15919,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.22/288,1.9.22.288,2013-01-31 13:09, +0700, id*/
const Device UA2632838 = {S70827, 192, { &P15919,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0000, gb*/
const Device UA2895902 = {S70828, 136, { &P22727,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGP312 Build/10.1.C.0.370) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2938175 = {S70829, 65, { &P22727,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGP312 Build/10.1.C.0.370)*/
const Device UA2999256 = {S70830, 136, { &P22727,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGP312 Build/10.1.C.0.344) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2899408 = {S70831, 156, { &P15575,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-I9100 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2961462 = {S70832, 157, { &P15575,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9100G Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2962956 = {S70833, 156, { &P15575,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-I9100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3131314 = {S70834, 140, { &P22867,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5303 Build/12.0.A.2.254) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA5475 = {S70835, 202, { &P12263,&P6757,&P3525,&P18092 }};	/*Useragent - LG-GW620 Mozilla/5.0 (Linux; U; Android 1.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA29535 = {S70836, 140, { &P12263,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-; LG GW620 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA46194 = {S70837, 142, { &P12263,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; fr-fr; LG GW620 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA8941 = {S70838, 50, { &P13155,&P17486,&P3487,&P18092 }};	/*Useragent - Motorola-QA30 UP.Browser/7.2.7.5.605 (GUI) MMP/2.0*/
const Device UA15238 = {S70839, 51, { &P13155,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-QA30/00.72 UP.Browser/7.2.7.5.598 (GUI) MMP/2.0*/
const Device UA29859 = {S70840, 51, { &P13155,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-QA30/00.72 UP.Browser/7.2.7.5.610 (GUI) MMP/2.0*/
const Device UA31518 = {S70841, 51, { &P13155,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-QA30/00.72 UP.Browser/7.2.7.5.565 (GUI) MMP/2.0*/
const Device UA31519 = {S70842, 51, { &P13155,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-QA30/00.72 UP.Browser/7.2.7.5.619 (GUI) MMP/2.0*/
const Device UA37476 = {S70843, 51, { &P13155,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-QA30/00.72 UP.Browser/7.2.7.5.546 (GUI) MMP/2.0*/
const Device UA50054 = {S70844, 51, { &P13155,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-QA30/00.72 UP.Browser/7.2.7.5.622 (GUI) MMP/2.0*/
const Device UA19941 = {S70845, 99, { &P13560,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S5510T/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA23826 = {S70846, 98, { &P13560,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S5510/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA271065 = {S70847, 47, { &P15364,&P6729,&P17193,&P18092 }};	/*Useragent - Mozilla/6.0 (compatible; MSIE 7.01; Windows 95)*/
const Device UA529283 = {S70848, 46, { &P15364,&P6729,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows 95)*/
const Device UA2793138 = {S70849, 54, { &P15364,&P6729,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Win95; Trident/1.0)*/
const Device UA764620 = {S70850, 149, { &P13123,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2782415 = {S70851, 150, { &P13123,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA889924 = {S70852, 141, { &P21469,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ru-ru; PocketBook A7 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1996172 = {S70853, 136, { &P17595,&P17297,&P22460,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11*/
const Device UA2204769 = {S70854, 136, { &P17595,&P17297,&P22460,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11*/
const Device UA2651006 = {S70855, 136, { &P17595,&P17297,&P22460,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11*/
const Device UA2234420 = {S70856, 126, { &P16384,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5380k) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile*/
const Device UA2587171 = {S70857, 83, { &P15364,&P7127,&P22304,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.8.0.6) Gecko/X SeaMonkey/1.0.4*/
const Device UA2506180 = {S70858, 141, { &P21155,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A800 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2567112 = {S70859, 139, { &P19902,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530M Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2636399 = {S70860, 151, { &P12879,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.53463*/
const Device UA2772915 = {S70861, 150, { &P12879,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2674204 = {S70862, 163, { &P18250,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Motorola RAZR MAXX Build/6.7.2-180_DHD-16_NA-27) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2773892 = {S70863, 157, { &P18673,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2953930 = {S70864, 158, { &P18673,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.53463*/
const Device UA2756535 = {S70865, 138, { &P19854,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R760X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2795751 = {S70866, 133, { &P23187,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; V370 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3029659 = {S70867, 132, { &P23187,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; V370 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3050576 = {S70868, 126, { &P23187,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; V370 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2974643 = {S70869, 154, { &P12605,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2965043 = {S70870, 130, { &P24106,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CLP285 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA62557 = {S70871, 65, { &P12590,&P5208,&P4628,&P18092 }};	/*Useragent - Mozilla/4.0 (BREW 3.1.5; U; en-us; Sanyo; Polaris/6.0/AMB) PL2700*/
const Device UA1994053 = {S70872, 171, { &P15991,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA904279 = {S70873, 147, { &P17337,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2608095 = {S70874, 147, { &P17337,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; GT-P7500 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2618435 = {S70875, 150, { &P17337,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; GT-P7500 Build/GWK74; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422261 = {S70876, 69, { &P18673,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; GT-S7562; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2455972 = {S70877, 117, { &P24029,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; YL-Coolpad_5860S/4.0.011.X.5860S; 480*800; CTC/2.0) CoolpadWebkit/534.30*/
const Device UA2464659 = {S70878, 136, { &P24029,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; 5860S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654193 = {S70879, 58, { &P24029,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; 5860S Build/IMM76I)*/
const Device UA2838723 = {S70880, 127, { &P24029,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; 5860S Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2485534 = {S70881, 135, { &P23876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099714 = {S70882, 57, { &P23876,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; T970 Build/IMM76D)*/
const Device UA2697470 = {S70883, 144, { &P20538,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PAP4020DUO Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2582690 = {S70884, 141, { &P22628,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; NGM Miracle Build/IMM76D) AppleWebKit/534.30 KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2863526 = {S70885, 142, { &P22628,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; NGM_Miracle Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2738717 = {S70886, 136, { &P23124,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ph; A919i Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102676 = {S70887, 58, { &P23124,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; A919i Build/JZO54K)*/
const Device UA2752937 = {S70888, 137, { &P20013,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F180L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774122 = {S70889, 137, { &P20013,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F180K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2797412 = {S70890, 137, { &P20013,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F180S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2938069 = {S70891, 136, { &P20013,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F180 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2944302 = {S70892, 152, { &P18648,&P5043,&P24678,&P18092 }};	/*Useragent - Nokia306/2.0 (05.93) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia306) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2974074 = {S70893, 134, { &P22836,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SC-04E Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974093 = {S70894, 141, { &P23505,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Vodafone 875 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA369623 = {S70895, 65, { &P18151,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; NOVO7PALADIN Build/IML74K)*/
const Device UA380714 = {S70896, 137, { &P18151,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; NOVO7PALADIN Build/ICS_MR0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA760699 = {S70897, 136, { &P18151,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; NOVO7PALADIN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA449082 = {S70898, 79, { &P15364,&P18110,&P22306,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:2.0pre) Gecko/X Firefox/4.0pre SeaMonkey/2.1b3*/
const Device UA1894744 = {S70899, 81, { &P15364,&P18110,&P22306,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.3a4pre) Gecko/X SeaMonkey/2.1a1pre*/
const Device UA578672 = {S70900, 116, { &P14708,&P5204,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia5130c-2/07.97; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA727867 = {S70901, 133, { &P21841,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ASTROPLUS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2426502 = {S70902, 133, { &P21841,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ASTROPLUS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2432242 = {S70903, 146, { &P20104,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 201M Build/7.7.1Q-164_SMJ-165) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2676982 = {S70904, 142, { &P19842,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Butterfly Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707241 = {S70905, 135, { &P19842,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Butterfly Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2707470 = {S70906, 142, { &P19842,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Butterfly Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712670 = {S70907, 93, { &P15991,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6102) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2736490 = {S70908, 126, { &P21239,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M9 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2995080 = {S70909, 55, { &P21239,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; M9 Build/JRO03H)*/
const Device UA2793312 = {S70910, 138, { &P24582,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE T790 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2892098 = {S70911, 136, { &P21792,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9200 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3008268 = {S70912, 129, { &P21792,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9200 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA26419 = {S70913, 79, { &P13593,&P17017,&P4223,&P18092 }};	/*Useragent - INQ/Q1-200H/1.0.65 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA26420 = {S70914, 79, { &P13593,&P17017,&P4223,&P18092 }};	/*Useragent - INQ/Q1-200H/1.0.70 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA55322 = {S70915, 79, { &P13593,&P17017,&P4223,&P18092 }};	/*Useragent - INQ/Q1-200H/1.0.56 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61452 = {S70916, 79, { &P13593,&P17017,&P4223,&P18092 }};	/*Useragent - INQ/Q1-200H/1.0.48 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62449 = {S70917, 79, { &P13593,&P17017,&P4223,&P18092 }};	/*Useragent - INQ/Q1-200H/1.0.63 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63276 = {S70918, 79, { &P13593,&P17017,&P4223,&P18092 }};	/*Useragent - INQ/Q1-200H/1.0.45 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA249288 = {S70919, 43, { &P12449,&P5890,&P24663,&P18092 }};	/*Useragent - NOKIAE5-00/UC Browser7.9.0.102/28/355/UCWEB*/
const Device UA805700 = {S70920, 45, { &P12449,&P5890,&P24663,&P18092 }};	/*Useragent - NOKIAE5-00.2/UC Browser7.9.0.102/28/355/UCWEB*/
const Device UA2429697 = {S70921, 67, { &P15364,&P7133,&P24686,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:10.0.2) Gecko/X Yoono/1.8.32*/
const Device UA624485 = {S70922, 130, { &P20464,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ja-jp; N-06D Build/AX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034092 = {S70923, 54, { &P20464,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; N-06D Build/AX)*/
const Device UA734010 = {S70924, 114, { &P21588,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_5860/2.3.002.X.5860; 480*800; CTC/2.0) CoolpadWebkit/533.1*/
const Device UA751741 = {S70925, 120, { &P21588,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; 5860 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1957944 = {S70926, 132, { &P21588,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; 5860 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1964768 = {S70927, 116, { &P21588,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_5860+/2.3.009.X.5860+; 480*800; CTC/2.0) CoolpadWebkit/533.1*/
const Device UA2031766 = {S70928, 125, { &P21588,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; 5860 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2441738 = {S70929, 125, { &P21588,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; 5860 Build GRJ90) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA3088518 = {S70930, 116, { &P21588,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_5860 /2.3.009.X.5860 ; 480*800; CTC/2.0) CoolpadWebkit/533.1*/
const Device UA2028622 = {S70931, 138, { &P13999,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC HD2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257993 = {S70932, 138, { &P13999,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC HD2 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423092 = {S70933, 61, { &P13999,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; NexusHD2 Build/JZO54K)*/
const Device UA2654728 = {S70934, 138, { &P13999,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC HD2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2655068 = {S70935, 134, { &P13999,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HD2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2731637 = {S70936, 143, { &P13999,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; Infinity HD2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2760130 = {S70937, 139, { &P13999,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NexusHD2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2794916 = {S70938, 144, { &P13999,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; Infinity HD2+ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952944 = {S70939, 134, { &P13999,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HD2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986700 = {S70940, 134, { &P13999,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; HTC HD2 Build/DI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257327 = {S70941, 131, { &P20860,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GEM7112 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2458356 = {S70942, 139, { &P16757,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-I500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665759 = {S70943, 138, { &P16757,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734499 = {S70944, 139, { &P16757,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2850638 = {S70945, 139, { &P16757,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-I500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2935348 = {S70946, 144, { &P16757,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049151 = {S70947, 139, { &P16757,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I500 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2509303 = {S70948, 153, { &P21833,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030E Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2567123 = {S70949, 131, { &P12889,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2635957 = {S70950, 132, { &P12889,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7300B Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2735476 = {S70951, 72, { &P15364,&P18145,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (X11; FreeBSD 9.1-STABLE amd64) Presto/2.12.388 Version/12.15*/
const Device UA2733531 = {S70952, 140, { &P22959,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAC-70051 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2737256 = {S70953, 155, { &P18466,&P18066,&P22587,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2737397 = {S70954, 144, { &P22752,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax P560 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2753857 = {S70955, 137, { &P18316,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2772791 = {S70956, 137, { &P18316,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782629 = {S70957, 137, { &P18316,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877358 = {S70958, 138, { &P18316,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160LV Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3088419 = {S70959, 139, { &P18316,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160S/K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769728 = {S70960, 146, { &P22850,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI H881C Build/HuaweiH881C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2794418 = {S70961, 138, { &P22854,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9508 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937535 = {S70962, 60, { &P22854,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9508 Build/JDQ39)*/
const Device UA49904 = {S70963, 84, { &P13104,&P6476,&P4223,&P18092 }};	/*Useragent - ZTE-T95 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1 firmware/T95 T03*/
const Device UA898320 = {S70964, 167, { &P13999,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC HD2 Build/IMM76I; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA908636 = {S70965, 167, { &P13999,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC HD2 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2758697 = {S70966, 171, { &P13999,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC HD2 Leo Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1950985 = {S70967, 125, { &P15364,&P16110,&P22460,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11*/
const Device UA1963938 = {S70968, 118, { &P15364,&P16110,&P22460,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11*/
const Device UA1994025 = {S70969, 138, { &P20151,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; TOOKY A19 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032778 = {S70970, 136, { &P20151,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; TOOKY A9 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2413076 = {S70971, 140, { &P20151,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TOOKY T1982 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2413489 = {S70972, 128, { &P20151,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TOOKY T1982 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2449909 = {S70973, 138, { &P20151,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; TOOKY A81 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2475623 = {S70974, 133, { &P20151,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TOOKY T1982 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2758775 = {S70975, 140, { &P20151,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TOOKY T1981 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2834583 = {S70976, 138, { &P20151,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TOOKY T85 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2432276 = {S70977, 139, { &P15767,&P18432,&P21788,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.0.3/X (like Mobile/10A525 Safari/8536.25)*/
const Device UA2436421 = {S70978, 139, { &P15767,&P18432,&P21788,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.0.3/X (like Mobile/10A525 Safari/8536.25)*/
const Device UA2491940 = {S70979, 74, { &P18141,&P20531,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 2.05) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2566998 = {S70980, 141, { &P18084,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101G Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2602078 = {S70981, 134, { &P20729,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A27 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3015439 = {S70982, 140, { &P20729,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; CROSS A27 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2901216 = {S70983, 140, { &P19764,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6502 Build/10.3.A.0.423) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3017877 = {S70984, 133, { &P19764,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6502 Build/10.3.A.0.423) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2974321 = {S70985, 135, { &P19784,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LIFETAB_P9514 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA359365 = {S70986, 136, { &P23844,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; SPX-5_3G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2366414 = {S70987, 100, { &P20273,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; HTC6990LVW)*/
const Device UA2420762 = {S70988, 140, { &P20511,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Fly_IQ442 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3085809 = {S70989, 62, { &P20511,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Fly_IQ442 Build/IMM76D)*/
const Device UA2567266 = {S70990, 139, { &P19849,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7560M Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2676290 = {S70991, 133, { &P20129,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MI 2 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2993634 = {S70992, 126, { &P20129,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MI 2 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2784918 = {S70993, 155, { &P21261,&P18598,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9500 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2783145 = {S70994, 141, { &P17346,&P17146,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9A405 Safari/7534.48.3*/
const Device UA2879602 = {S70995, 139, { &P17346,&P17146,&P22820,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/27.0.1453.10 Mobile/9A334 Safari/7534.48.3*/
const Device UA2878601 = {S70996, 147, { &P20518,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; QMobile Noir A10 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952766 = {S70997, 147, { &P20518,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; QMobile A10 Noir Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3020934 = {S70998, 142, { &P20518,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Karbonn A10 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3074271 = {S70999, 134, { &P20518,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A10 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2897670 = {S71000, 146, { &P19846,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-05D Build/9.1.C.0.475) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2944930 = {S71001, 142, { &P19846,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-05D Build/9.1.C.0.475) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3056575 = {S71002, 64, { &P19846,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SO-05D Build/9.1.C.0.475)*/
const Device UA3115119 = {S71003, 146, { &P19846,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SonySO-05D Build/9.1.C.1.103) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3131385 = {S71004, 142, { &P19846,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SO-05D Build/9.1.C.1.103) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3035655 = {S71005, 151, { &P18442,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3039502 = {S71006, 151, { &P18442,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3059109 = {S71007, 151, { &P18442,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-N719 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3125286 = {S71008, 151, { &P18442,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7105 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3081183 = {S71009, 137, { &P16362,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-D710 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3082920 = {S71010, 136, { &P18653,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P870 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA43685 = {S71011, 107, { &P13725,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-A197/A197UCJH2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA98491 = {S71012, 67, { &P17595,&P17294,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; PPC Mac OS X 10.4.9; Tasman 1.0)*/
const Device UA1951985 = {S71013, 76, { &P15364,&P7133,&P22538,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.0) Gecko/X Firefox/12.0 Kylo/1.1.1*/
const Device UA2200640 = {S71014, 99, { &P15364,&P18110,&P21778,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.3.5 Safari/534.34*/
const Device UA2558452 = {S71015, 99, { &P15364,&P18110,&P21778,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.21 (KHTML, like Gecko) QupZilla/1.3.5 Safari/537.21*/
const Device UA2167969 = {S71016, 138, { &P18506,&P5043,&P24670,&P18092 }};	/*Useragent - Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia305) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2468898 = {S71017, 132, { &P24011,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DEM752NC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2671146 = {S71018, 152, { &P20192,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Orange Daytona Build/HuaweiG510-0200) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2671479 = {S71019, 154, { &P20192,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0200 Build/HuaweiG510-0200) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709487 = {S71020, 154, { &P20192,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0100 Build/HuaweiG510-0100) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2992784 = {S71021, 154, { &P20192,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2900429 = {S71022, 143, { &P22655,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5503 Build/10.1.1.A.1.310) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2754652 = {S71023, 137, { &P12942,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I927 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892538 = {S71024, 152, { &P22963,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030A Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3013529 = {S71025, 145, { &P22963,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030A Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2974927 = {S71026, 126, { &P15602,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A501 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3080857 = {S71027, 155, { &P21261,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; GT-I9500 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA421993 = {S71028, 43, { &P11905,&P5890,&P24665,&P18092 }};	/*Useragent - NOKIA6120c/UC Browser8.0.4.121/28/444/UCWEB*/
const Device UA368769 = {S71029, 142, { &P20167,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-S5820 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA644439 = {S71030, 121, { &P20167,&P5629,&P15760,&P18092 }};	/*Useragent - SAMSUNG-GT-S5820_TD/1.0 Android/2.3.4 Release/12.15.2010 Browser/AppleWebKit533.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA697599 = {S71031, 130, { &P20167,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5820 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2021813 = {S71032, 130, { &P20167,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-S5820 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2024101 = {S71033, 142, { &P20167,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5820 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031607 = {S71034, 135, { &P20167,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5820 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2518872 = {S71035, 135, { &P20167,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-S5820 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA607309 = {S71036, 143, { &P15869,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA902336 = {S71037, 143, { &P15869,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2474123 = {S71038, 143, { &P15869,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Xoom Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA663378 = {S71039, 155, { &P12453,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Liberty Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028660 = {S71040, 155, { &P22037,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031408 = {S71041, 148, { &P22037,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2354266 = {S71042, 148, { &P22037,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3113 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2612371 = {S71043, 155, { &P22037,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2614756 = {S71044, 148, { &P22037,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2040403 = {S71045, 154, { &P18466,&P18066,&P18617,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.1 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2672775 = {S71046, 134, { &P20813,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GoTab GBT9 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2717888 = {S71047, 93, { &P13706,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5235) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2752967 = {S71048, 153, { &P18016,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ThinkPad Tablet Build/ThinkPadTablet_A400_03) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754816 = {S71049, 137, { &P19854,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R760 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2795937 = {S71050, 151, { &P20993,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI P2-6011 Build/HuaweiP2-6011) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2855731 = {S71051, 121, { &P23362,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; SAMSUNG; SPH-I800; SPRINT)*/
const Device UA2944179 = {S71052, 88, { &P15364,&P17017,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; spice) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2958531 = {S71053, 90, { &P15364,&P17017,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; generic) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2972710 = {S71054, 90, { &P15364,&P17017,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mozilla) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2972718 = {S71055, 91, { &P15364,&P17017,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sunmicro) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2974676 = {S71056, 95, { &P15364,&P17017,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; jbed-fastbcc) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3004428 = {S71057, 90, { &P15364,&P17017,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; Pt-BR; mozilla) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3009458 = {S71058, 90, { &P15364,&P17017,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; karbonn) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3012722 = {S71059, 99, { &P15364,&P17017,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; maui) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile TRUSTED/1.0*/
const Device UA3056521 = {S71060, 96, { &P15364,&P17017,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; 11a h700tecno) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3057974 = {S71061, 87, { &P15364,&P17017,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; bird) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2997853 = {S71062, 144, { &P23698,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I527 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3073720 = {S71063, 137, { &P23698,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I527 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3078281 = {S71064, 137, { &P23698,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I527M Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA14106 = {S71065, 61, { &P11788,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia6101/2.0 (04.90) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63961 = {S71066, 149, { &P12031,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Nokia6124c/4.34; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA67206 = {S71067, 10, { &P12031,&P5890,&P3439,&P18092 }};	/*Useragent - NOKIA6124c*/
const Device UA238596 = {S71068, 64, { &P15364,&P17017,&P17812,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/3.4) KHTML/3.4.2 (like Gecko)*/
const Device UA637684 = {S71069, 140, { &P21198,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Dell Streak 10 Pro Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2033608 = {S71070, 161, { &P21198,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; Dell_Streak_10_Pro Build/HTJ85B; 1280*752) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2361358 = {S71071, 137, { &P15364,&P17820,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.17 (KHTML, like Gecko) Ubuntu/12.04 Chromium/24.0.1312.2 Chrome/24.0.1312.2 Safari/537.17*/
const Device UA2368524 = {S71072, 135, { &P15364,&P17820,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.17 (KHTML, like Gecko) Ubuntu/12.04 Chromium/24.0.1312.2 Chrome/24.0.1312.2 Safari/537.17*/
const Device UA2678417 = {S71073, 137, { &P12814,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I777 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2751566 = {S71074, 130, { &P12814,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I777 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2768098 = {S71075, 137, { &P12814,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I777 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2753283 = {S71076, 136, { &P18240,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P700 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2797439 = {S71077, 131, { &P15767,&P18432,&P20783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/10A523 Safari/7534.48.3*/
const Device UA2864918 = {S71078, 129, { &P15767,&P18432,&P20783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/10A403 Safari/7534.48.3*/
const Device UA2892922 = {S71079, 131, { &P15767,&P18432,&P20783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/10A550 Safari/7534.48.3*/
const Device UA3005196 = {S71080, 129, { &P15767,&P18432,&P20783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/10A407 Safari/7534.48.3*/
const Device UA3006959 = {S71081, 132, { &P15767,&P18432,&P20783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/10A8426 Safari/7534.48.3*/
const Device UA3050476 = {S71082, 132, { &P15767,&P18432,&P20783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/10A8500 Safari/7534.48.3*/
const Device UA2879558 = {S71083, 138, { &P15364,&P5690,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/534.31 (KHTML, like Gecko) Chrome/17.0.558.0 Safari/534.31 UCBrowser/9.1.0.297*/
const Device UA3031247 = {S71084, 157, { &P18471,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-78) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3023822 = {S71085, 152, { &P19973,&P5043,&P24679,&P18092 }};	/*Useragent - Nokia114/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia114) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3082912 = {S71086, 132, { &P23855,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3083544 = {S71087, 139, { &P23855,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; WM8850-mid Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080216 = {S71088, 133, { &P19788,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; E350 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA64554 = {S71089, 92, { &P11755,&P5776,&P3756,&P18092 }};	/*Useragent - Nokia6670/2.0 (6.0522.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA50407 = {S71090, 143, { &P13251,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; Dell Streak Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2018601 = {S71091, 143, { &P13251,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; Dell Streak Build/Donut AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/ 525.20.1*/
const Device UA396897 = {S71092, 132, { &P13109,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC-X710a; en-ca) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA453201 = {S71093, 131, { &P17595,&P17297,&P22454,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7*/
const Device UA586881 = {S71094, 131, { &P17595,&P17297,&P22454,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7*/
const Device UA704787 = {S71095, 131, { &P17595,&P17297,&P22454,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7*/
const Device UA2230576 = {S71096, 131, { &P17595,&P17297,&P22454,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7*/
const Device UA852416 = {S71097, 96, { &P15364,&P17813,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86; sv-SE; rv:1.8.1.12) Gecko/X Ubuntu/8.04 (hardy) Firefox/2.0.0.12*/
const Device UA636864 = {S71098, 133, { &P21069,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; P-05D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029267 = {S71099, 57, { &P21069,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; P-05D Build/GRJ90)*/
const Device UA673883 = {S71100, 158, { &P16757,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I500 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA678949 = {S71101, 158, { &P16757,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I500 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA910596 = {S71102, 154, { &P16757,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I500 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA878143 = {S71103, 135, { &P22061,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; ZMCK900 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA902690 = {S71104, 133, { &P17595,&P17297,&P22458,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5*/
const Device UA1970488 = {S71105, 133, { &P17595,&P17297,&P22458,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5*/
const Device UA2167456 = {S71106, 133, { &P17595,&P17297,&P22458,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5*/
const Device UA2349024 = {S71107, 117, { &P23545,&P17017,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; mauie800) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2653327 = {S71108, 114, { &P23545,&P17017,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; id; mauie800) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2385988 = {S71109, 198, { &P12056,&P5204,&P24672,&P18092 }};	/*Useragent - Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia7210supernova) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2422732 = {S71110, 118, { &P14453,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE51) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/355/UCWEB Mobile*/
const Device UA2481112 = {S71111, 81, { &P20394,&P5629,&P15760,&P18092 }};	/*Useragent - ZTEU790_TD/1.0 Linux/2.6.35 Android/2.3 Release/7.5.2012 Browser/AppleWebKit533.1*/
const Device UA2655686 = {S71112, 137, { &P20394,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE U790 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2655745 = {S71113, 130, { &P20394,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE U790 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3044339 = {S71114, 61, { &P20394,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; ZTE U790 Build/GRK39F)*/
const Device UA2487017 = {S71115, 134, { &P22121,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; SPNG9716DC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2528991 = {S71116, 114, { &P15364,&P22379,&P17217,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Opera 11.0; Windows Server 2003; SV1) Admin search system: nikolaydovydov@km.ru*/
const Device UA2604838 = {S71117, 138, { &P18697,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I415 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2676273 = {S71118, 127, { &P16976,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT300 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2754501 = {S71119, 149, { &P18318,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF700T Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2938078 = {S71120, 137, { &P23504,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; UMI X2 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3077457 = {S71121, 59, { &P23504,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; UMI X2 Build/JOP40D)*/
const Device UA3067652 = {S71122, 136, { &P24114,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E440 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3060546 = {S71123, 144, { &P12280,&P22882,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X; zh-CN) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d UCBrowser/9.0.1.284 Mobile*/
const Device UA3097269 = {S71124, 142, { &P12280,&P22882,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X; zh-CN) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 UCBrowser/9.0.1.284 Mobile*/
const Device UA3129820 = {S71125, 144, { &P12280,&P22882,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X; zh-CN) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 UCBrowser/9.0.1.284 Mobile*/
const Device UA31043 = {S71126, 47, { &P12394,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-VX9100/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA2022952 = {S71127, 132, { &P21589,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; GS01 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2368509 = {S71128, 140, { &P15767,&P18117,&P21787,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA2466637 = {S71129, 155, { &P23714,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; O+_8.9_Android Build/OplusOplus_A1000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566944 = {S71130, 140, { &P20006,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30a Build/9.1.A.0.489) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2864971 = {S71131, 133, { &P20006,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30a Build/9.1.A.0.489) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2717081 = {S71132, 94, { &P14983,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC7-00) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA3020943 = {S71133, 134, { &P16292,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021649 = {S71134, 141, { &P16292,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023531 = {S71135, 141, { &P16292,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.2; Galaxy Nexus Build/ICL53F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023832 = {S71136, 141, { &P16292,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.1; Galaxy Nexus Build/ITL41F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3034008 = {S71137, 156, { &P22872,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5282 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3080708 = {S71138, 144, { &P22906,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A390_ROW Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA194755 = {S71139, 134, { &P12216,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6300 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA216285 = {S71140, 134, { &P12216,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ADR6300 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA692020 = {S71141, 134, { &P12216,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6300 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA598332 = {S71142, 88, { &P17976,&P6546,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.1; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.10.254 Version/12.00*/
const Device UA622437 = {S71143, 162, { &P12814,&P24691,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I777 Build/Task650 & Ktoonsez AOKP) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.1 Mobile Safari/534.30*/
const Device UA903400 = {S71144, 156, { &P12814,&P24691,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I777 Build/Task650 & Ktoonsez AOKP) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026289 = {S71145, 78, { &P12814,&P24691,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SGH-I777 Build/Task650 & Ktoonsez AOKP)*/
const Device UA668270 = {S71146, 138, { &P20959,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Celkon A88 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995067 = {S71147, 60, { &P15364,&P17017,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (masking-agent; rv:16.0) Gecko/16.0 Firefox/16.0*/
const Device UA2470662 = {S71148, 69, { &P15364,&P17017,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Gecko/X Firefox/16.0; Trident/5.0)*/
const Device UA2658347 = {S71149, 69, { &P15364,&P17017,&P18428,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Gecko/X Firefox/16.0; Trident/4.0)*/
const Device UA2700186 = {S71150, 70, { &P15364,&P17017,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Gecko/X Firefox/16.0; Trident/6.0)*/
const Device UA2567477 = {S71151, 131, { &P19879,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I915 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2600696 = {S71152, 152, { &P18558,&P20706,&P24676,&P18092 }};	/*Useragent - Nokia309/2.0 (05.85) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia309) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2601909 = {S71153, 153, { &P21968,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030D Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2729640 = {S71154, 153, { &P22962,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030A Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2754631 = {S71155, 138, { &P18716,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3018101 = {S71156, 142, { &P21542,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; pt-br; C6503 Build/10.3.A.0.423) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3027300 = {S71157, 66, { &P21542,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; C6503 Build/10.3.1.A.0.244)*/
const Device UA3061785 = {S71158, 144, { &P21542,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; C6503 Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2901107 = {S71159, 154, { &P12059,&P5204,&P24678,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2690) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3030826 = {S71160, 158, { &P23771,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2974901 = {S71161, 139, { &P12681,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus S 4G Build/JRO03R) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974593 = {S71162, 129, { &P22943,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; E270BSA Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3082727 = {S71163, 134, { &P18113,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT912 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA16675 = {S71164, 151, { &P13907,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; tr-tr; ZTE-JOE Build/VF945_V02c_PT) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA24223 = {S71165, 156, { &P13907,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; tr-tr; Vodafone 945 Build/VF945_V02e_TR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA41196 = {S71166, 156, { &P13907,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Vodafone 945 Build/VF945_V01a_QA) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA49131 = {S71167, 156, { &P13907,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; cs-cz; Vodafone 945 Build/VF945_V02d_CZ) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA49134 = {S71168, 156, { &P13907,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-za; Vodafone 945 Build/VF945_V02b_SA) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA49133 = {S71169, 156, { &P13907,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-pt; Vodafone 945 Build/VF945_V02c_PT) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA90550 = {S71170, 156, { &P13907,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; it-it; Vodafone 945 Build/VF945_V02d_DE) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA26713 = {S71171, 75, { &P13340,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GC900-Bouygues/V10a Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA26714 = {S71172, 66, { &P13340,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GC900/V10a Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63976 = {S71173, 73, { &P13340,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GC900-orange/V10n Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA31598 = {S71174, 48, { &P12443,&P17484,&P5001,&P18092 }};	/*Useragent - SCH-U750L/1.0 NetFront/3.0.22.2.17 (GUI) MMP/2.0*/
const Device UA631656 = {S71175, 100, { &P15364,&P7127,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.83 Safari/535.11*/
const Device UA858586 = {S71176, 128, { &P20197,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; T02A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2421302 = {S71177, 128, { &P20197,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T02A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA869101 = {S71178, 79, { &P15364,&P5690,&P23696,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.8b5) Gecko/X Firefox/1.4.1*/
const Device UA2211626 = {S71179, 76, { &P15364,&P5690,&P23696,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; it; rv:1.8b5) Gecko/X Firefox/1.4.1*/
const Device UA2583394 = {S71180, 151, { &P18460,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-1 Build/HuaweiU8950-1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2603541 = {S71181, 155, { &P18460,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-51 Build/HuaweiU8950N-51) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2666763 = {S71182, 149, { &P18460,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950D Build/HuaweiU8950D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2704297 = {S71183, 153, { &P18460,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-51 Build/HuaweiU8950-51) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2989189 = {S71184, 153, { &P18460,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-1 Build/HuaweiU8950N-1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2623327 = {S71185, 192, { &P15876,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, -0200, br*/
const Device UA2625986 = {S71186, 192, { &P15876,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +0000, gb*/
const Device UA2669222 = {S71187, 148, { &P23793,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PNDPP4MT9G3 Build/ICS.g08refmid1061.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2676028 = {S71188, 133, { &P18280,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; One S Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709085 = {S71189, 137, { &P18280,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One S Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2974613 = {S71190, 138, { &P18386,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I757M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3067585 = {S71191, 135, { &P20119,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_S9714 Build/JRO03R) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA54625 = {S71192, 86, { &P12924,&P17485,&P3600,&P18092 }};	/*Useragent - SonyEricssonV600i/R2K Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA136042 = {S71193, 122, { &P15364,&P7133,&P18211,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/5.2.1.1588 Safari/532.4*/
const Device UA475456 = {S71194, 102, { &P15364,&P7133,&P18211,&P18092 }};	/*Useragent - GoogleEarth/5.2.1.1588(Windows;Microsoft Windows (6.1.7600.0);es-419;kml:2.2;client:Free;type:default)*/
const Device UA869071 = {S71195, 101, { &P15364,&P7133,&P18211,&P18092 }};	/*Useragent - GoogleEarth/5.2.1.1588(Windows;Microsoft Windows (6.1.7600.0);en-US;kml:2.2;client:Free;type:default)*/
const Device UA879938 = {S71196, 101, { &P15364,&P7133,&P18211,&P18092 }};	/*Useragent - GoogleEarth/5.2.1.1588(Windows;Microsoft Windows (6.1.7601.1);en-US;kml:2.2;client:Free;type:default)*/
const Device UA1884829 = {S71197, 101, { &P15364,&P7133,&P18211,&P18092 }};	/*Useragent - GoogleEarth/5.2.1.1588(Windows;Microsoft Windows (6.1.7601.1);de-DE;kml:2.2;client:Free;type:default)*/
const Device UA309538 = {S71198, 135, { &P15575,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; SMT-i9100 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA378661 = {S71199, 137, { &P15575,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; SMT-i9100 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2230614 = {S71200, 158, { &P12056,&P5204,&P24668,&P18092 }};	/*Useragent - Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia7210supernova) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2566967 = {S71201, 146, { &P19891,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2601197 = {S71202, 138, { &P19891,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2866556 = {S71203, 141, { &P23829,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; G2 Build/ICS.Trio_Stealth_G2.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3091781 = {S71204, 133, { &P23829,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; G2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121018 = {S71205, 70, { &P23829,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; G2 Build/ICS.Trio_Stealth_G2.X)*/
const Device UA2752655 = {S71206, 139, { &P24583,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; EK-GC110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3039278 = {S71207, 161, { &P18513,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6802 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3083356 = {S71208, 151, { &P18505,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MB886 Build/7.7.1Q-144_MB886_MR1-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA63748 = {S71209, 101, { &P17078,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-M200/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA130861 = {S71210, 120, { &P17078,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-SGH-M200/AMHI4/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA45014 = {S71211, 98, { &P13854,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C5130/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA63060 = {S71212, 99, { &P13854,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C5130S/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA64799 = {S71213, 99, { &P13854,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C5130U/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA4372 = {S71214, 81, { &P12638,&P5448,&P22272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) SAMSUNG-SGH-i780/1.0*/
const Device UA25404 = {S71215, 90, { &P12638,&P5448,&P22272,&P18092 }};	/*Useragent - SAMSUNG-SGH-i780/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) SAMSUNG-SGH-i780/1.0*/
const Device UA26656 = {S71216, 77, { &P12638,&P5448,&P22272,&P18092 }};	/*Useragent - SAMSUNG-SGH-i780ORANGE/BVGK3 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA32864 = {S71217, 74, { &P12638,&P5448,&P22272,&P18092 }};	/*Useragent - SAMSUNG-SGH-i780TIM/AIHA3 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA33162 = {S71218, 78, { &P12638,&P5448,&P22272,&P18092 }};	/*Useragent - SAMSUNG-SGH-i780SFR/AEHC6_HD2 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA61123 = {S71219, 72, { &P12638,&P5448,&P22272,&P18092 }};	/*Useragent - SAMSUNG-SGH-i780V/MJHA3 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA64243 = {S71220, 69, { &P12638,&P5448,&P22272,&P18092 }};	/*Useragent - SAMSUNG-SGH-i780/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA24620 = {S71221, 65, { &P13808,&P17483,&P3499,&P18092 }};	/*Useragent - LG-GU297/V10d Teleca/Q7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA44217 = {S71222, 66, { &P13808,&P17483,&P3499,&P18092 }};	/*Useragent - LG-GU290f/V10f Teleca/Q7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA135297 = {S71223, 67, { &P17595,&P17295,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; PPC Mac OS X 10.5.8; Tasman 1.0)*/
const Device UA1842368 = {S71224, 67, { &P17595,&P17295,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; PPC Mac OS X 10.5.6; Tasman 1.0)*/
const Device UA184008 = {S71225, 69, { &P17346,&P17090,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8G4 (iPod; U; CPU OS 4_3_1 like Mac OS X; en_us)*/
const Device UA191881 = {S71226, 69, { &P17346,&P17090,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8J2 (iPod; U; CPU OS 4_3_3 like Mac OS X; en_us)*/
const Device UA237779 = {S71227, 69, { &P17346,&P17090,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8L1 (iPod; U; CPU OS 4_3_5 like Mac OS X; en_us)*/
const Device UA420567 = {S71228, 69, { &P17346,&P17090,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8K2 (iPod; U; CPU OS 4_3_4 like Mac OS X; en_us)*/
const Device UA435134 = {S71229, 69, { &P17346,&P17090,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8H7 (iPod; U; CPU OS 4_3_2 like Mac OS X; en_us)*/
const Device UA691564 = {S71230, 69, { &P17346,&P17090,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8F190 (iPod; U; CPU OS 4_3 like Mac OS X; en_us)*/
const Device UA193427 = {S71231, 115, { &P15364,&P17285,&P22449,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.24 (KHTML, like Gecko) Iron/11.0.700.2 Chrome/11.0.700.2 Safari/534.24*/
const Device UA320285 = {S71232, 122, { &P15364,&P17285,&P22449,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Iron/11.0.700.3 Chrome/11.0.700.3 Safari/534.24*/
const Device UA735242 = {S71233, 139, { &P23808,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; it-it; TT102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1952325 = {S71234, 154, { &P12849,&P17342,&P17454,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S7230E/S723EXXLB3; U; Bada/2.0; en-us) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile WQVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA2355599 = {S71235, 112, { &P19972,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia113/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2934241 = {S71236, 111, { &P19972,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia113/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.1.0.5*/
const Device UA2356282 = {S71237, 133, { &P17595,&P17297,&P22462,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4*/
const Device UA2424054 = {S71238, 133, { &P17595,&P17297,&P22462,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1250.0 Iron/22.0.2150.0 Safari/537.4*/
const Device UA2670911 = {S71239, 134, { &P22746,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N9100 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676250 = {S71240, 134, { &P17211,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 101G9 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2711298 = {S71241, 131, { &P17211,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 101G9 Build/MR1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2974304 = {S71242, 137, { &P18507,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8160 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021927 = {S71243, 138, { &P18507,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8160L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021996 = {S71244, 138, { &P18507,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8160P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2969231 = {S71245, 135, { &P24053,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SHL22 Build/S6290) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3020950 = {S71246, 57, { &P24053,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SHL22 Build/S6290)*/
const Device UA3081023 = {S71247, 138, { &P19854,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R760X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA556521 = {S71248, 93, { &P15364,&P5690,&P22480,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.0.5) Gecko/X Firefox/3.0.5 Flock/2.0.3*/
const Device UA785191 = {S71249, 93, { &P15364,&P5690,&P22480,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.1) Gecko/X Firefox/3.0.1 Flock/2.0b2*/
const Device UA687934 = {S71250, 89, { &P17595,&P17296,&P22506,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:2.0.1) Gecko/X Firefox/4.0.1 Camino/2.2.1*/
const Device UA2204787 = {S71251, 51, { &P17595,&P18120,&P20788,&P18092 }};	/*Useragent - MQQBrowser/37 (iOS 4S; U; CPU like Mac OS X; zh-cn)*/
const Device UA2204927 = {S71252, 50, { &P17595,&P18120,&P20788,&P18092 }};	/*Useragent - MQQBrowser/37 (iOS 4; U; CPU like Mac OS X; zh-cn)*/
const Device UA2366582 = {S71253, 50, { &P17595,&P18120,&P20788,&P18092 }};	/*Useragent - MQQBrowser/37 (iOS 5; U; CPU like Mac OS X; zh-cn)*/
const Device UA2226868 = {S71254, 146, { &P20980,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A97i Build/CelkonA97i_Celkon) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2457337 = {S71255, 134, { &P24553,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; X21 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3016345 = {S71256, 105, { &P11684,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5800d-1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3061838 = {S71257, 105, { &P11684,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5800 XpressMusic) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3082868 = {S71258, 130, { &P15985,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126473 = {S71259, 131, { &P15985,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA416155 = {S71260, 30, { &P15364,&P17658,&P17656,&P18092 }};	/*Useragent - DoCoMo/1.0/N505i/c20/TB/W24H12*/
const Device UA90302 = {S71261, 96, { &P12970,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0) Asus;Galaxy6*/
const Device UA850779 = {S71262, 98, { &P12970,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/ 7.0; Asus;Galaxy6)*/
const Device UA2765094 = {S71263, 113, { &P12970,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/ 7.0; WpsLondonTest; Asus;Galaxy6)*/
const Device UA347614 = {S71264, 121, { &P15364,&P17285,&P22415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.1.1.0 Chrome/16.0.912.63 Safari/535.7*/
const Device UA352177 = {S71265, 128, { &P15364,&P17285,&P22415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.1.1.0 Chrome/16.0.912.63 Safari/535.7*/
const Device UA444303 = {S71266, 144, { &P12280,&P17134,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/3.1.1 Mobile/7A280f Safari/525.20*/
const Device UA2720155 = {S71267, 143, { &P12280,&P17134,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/525.181 (KHTML, like Gecko) Version/3.1.1 Mobile/5H11 Safari/525.20*/
const Device UA627504 = {S71268, 30, { &P15364,&P6704,&P17213,&P18092 }};	/*Useragent - Opera/8.50 (Windows 98; U; en)*/
const Device UA656574 = {S71269, 30, { &P15364,&P6704,&P17213,&P18092 }};	/*Useragent - Opera/8.53 (Windows 98; U; en)*/
const Device UA742717 = {S71270, 30, { &P15364,&P6704,&P17213,&P18092 }};	/*Useragent - Opera/8.54 (Windows 98; U; en)*/
const Device UA1837319 = {S71271, 30, { &P15364,&P6704,&P17213,&P18092 }};	/*Useragent - Opera/8.51 (Windows 98; U; en)*/
const Device UA2271650 = {S71272, 60, { &P15364,&P6704,&P17213,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; en) Opera 8.0*/
const Device UA2681669 = {S71273, 61, { &P15364,&P6704,&P17213,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; ru) Opera 8.02*/
const Device UA3069236 = {S71274, 61, { &P15364,&P6704,&P17213,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; ru) Opera 8.01*/
const Device UA904004 = {S71275, 103, { &P15364,&P18145,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA2156226 = {S71276, 102, { &P15364,&P18145,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA2457494 = {S71277, 102, { &P15364,&P18145,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/536.5 (KHTML like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA2355199 = {S71278, 141, { &P21181,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S560 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2367047 = {S71279, 74, { &P15364,&P7133,&P18429,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:17.0) Gecko/17.0 Firefox/17.0 slurp*/
const Device UA2405739 = {S71280, 71, { &P15364,&P7133,&P18429,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:17.0) Gecko/X Firefox/17.0 slurp*/
const Device UA2423587 = {S71281, 77, { &P15364,&P7133,&P18429,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:17.0) Gecko/X Firefox/17.0 slurp slurp*/
const Device UA2426753 = {S71282, 64, { &P15364,&P7133,&P18429,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:17.0) Gecko/X Firefox/17.0 slurp*/
const Device UA2437423 = {S71283, 128, { &P18008,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ST10 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2508193 = {S71284, 159, { &P13526,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; myTouch 4g Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666699 = {S71285, 158, { &P13526,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; myTouch 4g Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2769400 = {S71286, 159, { &P13526,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; myTouch 4G Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566913 = {S71287, 152, { &P20017,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2603539 = {S71288, 151, { &P20017,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567198 = {S71289, 137, { &P16362,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-D710 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583416 = {S71290, 138, { &P16362,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SPH-D710 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2862525 = {S71291, 130, { &P16362,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-D710 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2658728 = {S71292, 138, { &P22497,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; w30hd(QuadCore) Build/W30hd) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2603405 = {S71293, 140, { &P18452,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT29i Build/9.1.B.0.411) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2675848 = {S71294, 130, { &P20011,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Next7P12 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2709164 = {S71295, 133, { &P20011,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Next7P12-8G Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2769662 = {S71296, 156, { &P16356,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N7000 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2734265 = {S71297, 112, { &P18557,&P20706,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia308/07.55; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.36*/
const Device UA2974767 = {S71298, 93, { &P15991,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6102) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA2333135 = {S71299, 96, { &P15803,&P5843,&P17347,&P18092 }};	/*Useragent - Nokia200/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia200/UC Browser8.0.3.107/69/444*/
const Device UA630051 = {S71300, 75, { &P15364,&P18110,&P20458,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:8.0.1) Gecko/X Firefox/8.0.1 SeaMonkey/2.5*/
const Device UA1955997 = {S71301, 136, { &P20858,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Scroll Excel Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1971651 = {S71302, 78, { &P15364,&P23114,&P20460,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.5) Gecko/X Firefox/10.0.5 SeaMonkey/2.7.2*/
const Device UA1988001 = {S71303, 76, { &P15364,&P23114,&P20460,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.2) Gecko/X Firefox/10.0.2 SeaMonkey/2.7*/
const Device UA2163257 = {S71304, 78, { &P15364,&P23114,&P20460,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.4) Gecko/X Firefox/10.0.4 SeaMonkey/2.7.2*/
const Device UA2362475 = {S71305, 78, { &P15364,&P23114,&P20460,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.7) Gecko/X Firefox/10.0.7 SeaMonkey/2.7.2*/
const Device UA2482191 = {S71306, 80, { &P15364,&P23114,&P20460,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.11) Gecko/X Firefox/10.0.11 SeaMonkey/2.7.2*/
const Device UA2536664 = {S71307, 80, { &P15364,&P23114,&P20460,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.12) Gecko/X Firefox/10.0.12 SeaMonkey/2.7.2*/
const Device UA2653265 = {S71308, 80, { &P15364,&P23114,&P20460,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.10) Gecko/X Firefox/10.0.10 SeaMonkey/2.7.2*/
const Device UA2746464 = {S71309, 78, { &P15364,&P23114,&P20460,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:10.0.9) Gecko/X Firefox/10.0.9 SeaMonkey/2.7.2*/
const Device UA2180443 = {S71310, 146, { &P20262,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Karbonn_A18 Build/KarbonnA18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2353791 = {S71311, 125, { &P13677,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5253) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/405/UCWEB Mobile*/
const Device UA2206703 = {S71312, 135, { &P18605,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTL21 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2532134 = {S71313, 88, { &P19779,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A1+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2557799 = {S71314, 134, { &P23802,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; W30(QuadCore) Build/W30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2558114 = {S71315, 131, { &P23512,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SC-74JB Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2567396 = {S71316, 137, { &P14123,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P970 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2669368 = {S71317, 97, { &P21475,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A35) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2974871 = {S71318, 133, { &P19788,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; E350 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021153 = {S71319, 133, { &P19788,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; E350 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974642 = {S71320, 144, { &P18348,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023805 = {S71321, 150, { &P18348,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G X710s Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3067911 = {S71322, 140, { &P22209,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Titanium S5 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3096248 = {S71323, 114, { &P14635,&P5843,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-00/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA2031651 = {S71324, 131, { &P21822,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; T2V Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2384213 = {S71325, 128, { &P21613,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Aurora-II Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2479819 = {S71326, 128, { &P21613,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; Aurora-II Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2552524 = {S71327, 142, { &P21613,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Aurora-II Build/ICS.Aurora-II.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2423316 = {S71328, 132, { &P15876,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P5100 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2561582 = {S71329, 131, { &P15876,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5100 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2508657 = {S71330, 91, { &P13554,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAN79) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2483539 = {S71331, 60, { &P15364,&P18146,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; SunOS i86pc; rv:18.0) Gecko/X Firefox/18.0*/
const Device UA2679639 = {S71332, 74, { &P18141,&P22275,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 2.11) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA3031068 = {S71333, 156, { &P22597,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3029498 = {S71334, 129, { &P24624,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; A1-811 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3039431 = {S71335, 156, { &P16356,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N7000 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA86816 = {S71336, 124, { &P12164,&P5033,&P3417,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) BlackBerry8707/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/150*/
const Device UA68278 = {S71337, 46, { &P11684,&P5129,&P4030,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UCBrowser7.7.0.81/50/999*/
const Device UA118861 = {S71338, 21, { &P11684,&P5129,&P4030,&P18092 }};	/*Useragent - Nokia5800 XpressMusic*/
const Device UA466007 = {S71339, 109, { &P17595,&P18120,&P22516,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.8.1.5pre) Gecko/X Firefox/2.0.0.4 Navigator/9.0b2*/
const Device UA647343 = {S71340, 106, { &P17595,&P18120,&P22516,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US; rv:1.8.1.8pre) Gecko/X Firefox/2.0.0.8 Navigator/9.0.0.1*/
const Device UA696016 = {S71341, 137, { &P14821,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; LG-LU3000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027563 = {S71342, 137, { &P14821,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; LG-LU3000 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2223744 = {S71343, 150, { &P21221,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sr-rs; Telenor_OneTouch Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2808917 = {S71344, 184, { &P21221,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; Telenor_OneTouch Build/qltsar Telenor One Touch Minimal Edition v1.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA893126 = {S71345, 157, { &P14017,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P990 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2384293 = {S71346, 156, { &P14017,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P990 Build/GRJ22; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2519381 = {S71347, 176, { &P14017,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; LG-P990 Build/IMM76L; CyanogenMod-9.0.0; Owains Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029185 = {S71348, 136, { &P22016,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; X710d Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2486390 = {S71349, 143, { &P21651,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; POV_TAB-PROTAB25XXL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2749497 = {S71350, 144, { &P21651,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; POV_TAB-PROTAB25XXL8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2674222 = {S71351, 127, { &P18680,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KFJWI Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2714613 = {S71352, 139, { &P22739,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Cynus F3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3120228 = {S71353, 61, { &P22739,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Cynus F3 Build/JRO03C)*/
const Device UA2755080 = {S71354, 137, { &P19894,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R830 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2944703 = {S71355, 142, { &P12280,&P19826,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 UCBrowser/9.1.0.287 Mobile*/
const Device UA2945091 = {S71356, 140, { &P12280,&P19826,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 UCBrowser/9.1.0.287 Mobile*/
const Device UA3017370 = {S71357, 140, { &P12280,&P19826,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 UCBrowser/9.1.0.287 Mobile*/
const Device UA3017578 = {S71358, 140, { &P12280,&P19826,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 UCBrowser/9.1.0.287 Mobile*/
const Device UA3057037 = {S71359, 142, { &P12280,&P19826,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 UCBrowser/9.1.0.287 Mobile*/
const Device UA3057118 = {S71360, 142, { &P12280,&P19826,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145 UCBrowser/9.1.0.287 Mobile*/
const Device UA3057369 = {S71361, 142, { &P12280,&P19826,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 UCBrowser/9.1.0.287 Mobile*/
const Device UA2976098 = {S71362, 149, { &P18318,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF700T Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3068211 = {S71363, 142, { &P19862,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax A110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094575 = {S71364, 156, { &P15575,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109668 = {S71365, 157, { &P15575,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9100G Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA287878 = {S71366, 58, { &P15364,&P21445,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows ME; U; en) Presto/2.6.30 Version/10.63*/
const Device UA414144 = {S71367, 58, { &P15364,&P21445,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows ME; U; en) Presto/2.6.30 Version/10.62*/
const Device UA470510 = {S71368, 58, { &P15364,&P21445,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows ME; U; en) Presto/2.6.30 Version/10.61*/
const Device UA2400190 = {S71369, 142, { &P21870,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; BLU_Elite_3.8 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2458994 = {S71370, 71, { &P15364,&P7133,&P18521,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:18.0) Gecko/X Firefox/18.0 slurp*/
const Device UA2484547 = {S71371, 64, { &P15364,&P7133,&P18521,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:18.0) Gecko/X Firefox/18.0 slurp*/
const Device UA2487483 = {S71372, 77, { &P15364,&P7133,&P18521,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:18.0) Gecko/X Firefox/18.0 slurp slurp*/
const Device UA2563483 = {S71373, 145, { &P21987,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; GIONEE_Gpad_G1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567142 = {S71374, 137, { &P12814,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I777 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2601158 = {S71375, 138, { &P12814,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-i777 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2869478 = {S71376, 138, { &P23227,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I467 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2947436 = {S71377, 134, { &P21636,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C525c Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36*/
const Device UA3083071 = {S71378, 140, { &P24378,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Lenovo P780 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083294 = {S71379, 144, { &P24378,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Lenovo P780_ROW Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA120599 = {S71380, 47, { &P15364,&P17017,&P22267,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.5; Mac_PowerPC)*/
const Device UA612653 = {S71381, 135, { &P17595,&P17297,&P22457,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19*/
const Device UA664568 = {S71382, 135, { &P17595,&P17297,&P22457,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19*/
const Device UA2231988 = {S71383, 135, { &P17595,&P17297,&P22457,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19*/
const Device UA2437393 = {S71384, 114, { &P15364,&P17285,&P19987,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 Sleipnir/3.8.4*/
const Device UA2505953 = {S71385, 121, { &P15364,&P17285,&P19987,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 Sleipnir/3.8.4*/
const Device UA2529194 = {S71386, 79, { &P15364,&P17285,&P19987,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/3.8.4)*/
const Device UA2239012 = {S71387, 138, { &P18679,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-B5330 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2482202 = {S71388, 139, { &P18679,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-B5330L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2366673 = {S71389, 155, { &P14969,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Desire S Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2402720 = {S71390, 155, { &P14969,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC Saga Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2424436 = {S71391, 153, { &P12108,&P5843,&P24673,&P18092 }};	/*Useragent - Nokia7230/5.0 (10.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia7230) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2433511 = {S71392, 129, { &P20510,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CT920 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2469173 = {S71393, 192, { &P15932,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0000, gb*/
const Device UA2616831 = {S71394, 190, { &P15932,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3110 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, us*/
const Device UA2624534 = {S71395, 199, { &P15932,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, -0500, us*/
const Device UA2633487 = {S71396, 192, { &P15932,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-P3110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0100, fr*/
const Device UA2675932 = {S71397, 138, { &P21627,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC first Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735400 = {S71398, 131, { &P21627,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC first Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2701489 = {S71399, 93, { &P13163,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian97-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2716725 = {S71400, 92, { &P13163,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaN97) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2869199 = {S71401, 136, { &P20937,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ADR910L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2997297 = {S71402, 154, { &P23672,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ALCATEL ONE TOUCH 4030X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA3073798 = {S71403, 76, { &P23672,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ALCATEL ONE TOUCH 4030X Build/JRO03C)*/
const Device UA28437 = {S71404, 91, { &P12243,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Snap_S521 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA63011 = {S71405, 65, { &P14471,&P17483,&P17602,&P18092 }};	/*Useragent - LG-S310/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA249832 = {S71406, 45, { &P12193,&P5129,&P24663,&P18092 }};	/*Useragent - NokiaC5-00.2/UC Browser7.9.0.102/28/355/UCWEB*/
const Device UA342760 = {S71407, 43, { &P12193,&P5129,&P24663,&P18092 }};	/*Useragent - NokiaC5-00/UC Browser7.9.0.102/28/355/UCWEB*/
const Device UA587287 = {S71408, 75, { &P15364,&P18145,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0   (X11; U; FreeBSD i386; en-US; rv:1.7.8) Gecko/X Firefox/1.0.4*/
const Device UA2846008 = {S71409, 73, { &P15364,&P18145,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; FreeBSD i386; en-US; rv:1.7.6) Gecko/X Firefox/1.0.1*/
const Device UA680938 = {S71410, 139, { &P21146,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; Amazing A1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2980363 = {S71411, 63, { &P21146,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; Amazing A1 Build/GRK39F)*/
const Device UA698251 = {S71412, 162, { &P15881,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Incredible 2 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA785108 = {S71413, 162, { &P15881,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Incredible 2 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA906570 = {S71414, 158, { &P15881,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Incredible 2 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA911562 = {S71415, 140, { &P18707,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MD_LIFETAB_P9516 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2041840 = {S71416, 80, { &P17595,&P17296,&P18523,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:15.0) Gecko/X Thunderbird/15.0.1*/
const Device UA2716741 = {S71417, 139, { &P19858,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KYL21 Build/019.0.3b00) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2583783 = {S71418, 100, { &P15364,&P16110,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.2; Software Development; QQ:X; http://www.zk1t.com/)*/
const Device UA2594721 = {S71419, 254, { &P15364,&P16110,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.2; Win64; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; IDCRL 8.0.1132.0; IDCRL-cfg 13.500.20639.0; App svchost.exe, 6.2.9200.16420, {DF60E2DF-88AD-4526-AE21-83D130EF0F68})*/
const Device UA2648459 = {S71420, 137, { &P15364,&P16110,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.2; WOW64; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729)*/
const Device UA2792527 = {S71421, 63, { &P15364,&P16110,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 6.2; Trident/3.0)*/
const Device UA2717659 = {S71422, 139, { &P20597,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F100S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3020524 = {S71423, 94, { &P20134,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3066205 = {S71424, 137, { &P12942,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I927 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA21290 = {S71425, 75, { &P12308,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHU600-ORANGE/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA30357 = {S71426, 76, { &P12308,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHU600B-ORANGE/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA48986 = {S71427, 68, { &P12308,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHU600/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA46466 = {S71428, 138, { &P12280,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9A5248d Safari/6533.18.5*/
const Device UA256754 = {S71429, 138, { &P12280,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9A405 Safari/6533.18.5*/
const Device UA626856 = {S71430, 150, { &P12280,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en_us) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5*/
const Device UA631324 = {S71431, 135, { &P12280,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; 6534.46) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5*/
const Device UA661704 = {S71432, 159, { &P12280,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5 DiigoBrowser*/
const Device UA751927 = {S71433, 148, { &P12280,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X; en_us) AppleWebKit/6534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148a Safari/6533.18.5*/
const Device UA801101 = {S71434, 136, { &P12280,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9A334 Safari/6533.18.5*/
const Device UA2308591 = {S71435, 138, { &P12280,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9A406 Safari/6533.18.5*/
const Device UA2527191 = {S71436, 134, { &P12280,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0 Mobile/9A334 Safari/7534.48.3*/
const Device UA2786302 = {S71437, 148, { &P12280,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5*/
const Device UA3043421 = {S71438, 148, { &P12280,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA369309 = {S71439, 144, { &P14872,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA412397 = {S71440, 139, { &P14872,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SPH-D700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA660358 = {S71441, 139, { &P14872,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA803340 = {S71442, 139, { &P14872,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D700 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899437 = {S71443, 139, { &P14872,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033967 = {S71444, 61, { &P14872,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SPH-D700 Build/IML74K)*/
const Device UA2545650 = {S71445, 61, { &P14872,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; SPH-D700 Build/IMM76L)*/
const Device UA708755 = {S71446, 156, { &P17308,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDX Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA888150 = {S71447, 156, { &P17308,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDX Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA907386 = {S71448, 152, { &P17308,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDX Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2238739 = {S71449, 134, { &P22816,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; S22 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3117948 = {S71450, 56, { &P22816,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; S22 Build/IMM76D)*/
const Device UA2353907 = {S71451, 142, { &P21046,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A89 Build/CelkonCelkon_A89) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2403089 = {S71452, 71, { &P20056,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X446/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2562610 = {S71453, 143, { &P21362,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SYMPHONY_W90 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2562930 = {S71454, 143, { &P21362,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SYMPHONY W90 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580270 = {S71455, 135, { &P21752,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; M030 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2602013 = {S71456, 139, { &P23582,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; fnac 4.5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2707843 = {S71457, 135, { &P17968,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; skz_tenderloin Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2711758 = {S71458, 134, { &P17968,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; cm_tenderloin Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2713875 = {S71459, 147, { &P17968,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; e2001_v89_jbl1a698 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714469 = {S71460, 128, { &P17968,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2715167 = {S71461, 146, { &P17968,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; e2001_v89_gq2008s Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2718872 = {S71462, 141, { &P17968,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; v89_jbl1a698 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2737580 = {S71463, 146, { &P22908,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Spark TV Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2753965 = {S71464, 132, { &P13644,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; EVO Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769770 = {S71465, 132, { &P13644,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; EVO Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3081189 = {S71466, 136, { &P19917,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P765 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA435978 = {S71467, 47, { &P15364,&P18585,&P17470,&P18092 }};	/*Useragent - Mercury3/4.3.3 CFNetwork/485.10.2 Darwin/10.3.1*/
const Device UA489154 = {S71468, 44, { &P15364,&P18585,&P17470,&P18092 }};	/*Useragent - SpringBoard/50 CFNetwork/485.2 Darwin/10.3.1*/
const Device UA516188 = {S71469, 58, { &P15364,&P18585,&P17470,&P18092 }};	/*Useragent - Million%20HD%20WPs/8.80.1 CFNetwork/485.10.2 Darwin/10.3.1*/
const Device UA517842 = {S71470, 38, { &P15364,&P18585,&P17470,&P18092 }};	/*Useragent - Web/1.0 CFNetwork/467.12 Darwin/10.3.1*/
const Device UA568244 = {S71471, 42, { &P15364,&P18585,&P17470,&P18092 }};	/*Useragent - Terra/1.3.5 CFNetwork/467.12 Darwin/10.3.1*/
const Device UA629349 = {S71472, 47, { &P15364,&P18585,&P17470,&P18092 }};	/*Useragent - GQMagazine/1.0 CFNetwork/485.10.2 Darwin/10.3.1*/
const Device UA659952 = {S71473, 50, { &P15364,&P18585,&P17470,&P18092 }};	/*Useragent - AtomicBrowser/6.0.1 CFNetwork/467.12 Darwin/10.3.1*/
const Device UA739304 = {S71474, 56, { &P15364,&P18585,&P17470,&P18092 }};	/*Useragent - Hot%20Wallpapers/9.12.1 CFNetwork/485.10.2 Darwin/10.3.1*/
const Device UA804409 = {S71475, 49, { &P15364,&P18585,&P17470,&P18092 }};	/*Useragent - AtomicLite/5.6.0 CFNetwork/485.10.2 Darwin/10.3.1*/
const Device UA819471 = {S71476, 47, { &P15364,&P18585,&P17470,&P18092 }};	/*Useragent - Clipish/9.29.1 CFNetwork/485.10.2 Darwin/10.3.1*/
const Device UA1892925 = {S71477, 52, { &P15364,&P18585,&P17470,&P18092 }};	/*Useragent - Perfect%20Browser/5.96 CFNetwork/485.2 Darwin/10.3.1*/
const Device UA1927431 = {S71478, 58, { &P15364,&P18585,&P17470,&P18092 }};	/*Useragent - Celeb%20Wallpapers/9.13.1 CFNetwork/485.10.2 Darwin/10.3.1*/
const Device UA1945794 = {S71479, 54, { &P15364,&P18585,&P17470,&P18092 }};	/*Useragent - HOT%20HD%20WPs/9.19.1 CFNetwork/485.10.2 Darwin/10.3.1*/
const Device UA2135480 = {S71480, 52, { &P15364,&P18585,&P17470,&P18092 }};	/*Useragent - 360%20Browser/3.1.1 CFNetwork/485.10.2 Darwin/10.3.1*/
const Device UA2150384 = {S71481, 54, { &P15364,&P18585,&P17470,&P18092 }};	/*Useragent - 2%20Million%20WPs/9.16.1 CFNetwork/485.2 Darwin/10.3.1*/
const Device UA2375888 = {S71482, 46, { &P15364,&P18585,&P17470,&P18092 }};	/*Useragent - browser/31759 CFNetwork/485.10.2 Darwin/10.3.1*/
const Device UA2441294 = {S71483, 47, { &P15364,&P18585,&P17470,&P18092 }};	/*Useragent - LinkedIn/5.0.1 CFNetwork/485.10.2 Darwin/10.3.1*/
const Device UA2445541 = {S71484, 51, { &P15364,&P18585,&P17470,&P18092 }};	/*Useragent - Mercury3Free/6.0.1 CFNetwork/485.10.2 Darwin/10.3.1*/
const Device UA2643203 = {S71485, 43, { &P15364,&P18585,&P17470,&P18092 }};	/*Useragent - Images/1.4 CFNetwork/485.10.2 Darwin/10.3.1*/
const Device UA2801320 = {S71486, 47, { &P15364,&P18585,&P17470,&P18092 }};	/*Useragent - Wallpapers/1.6 CFNetwork/485.10.2 Darwin/10.3.1*/
const Device UA622682 = {S71487, 132, { &P21215,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; 101P Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA634876 = {S71488, 56, { &P21215,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; 101P Build/GRJ90)*/
const Device UA900286 = {S71489, 137, { &P20267,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; P300(Funbook) Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2743018 = {S71490, 66, { &P20267,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; P300(Funbook) Build/IML74K)*/
const Device UA1997816 = {S71491, 79, { &P15364,&P22725,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Linux; Opera Mobi/8; MeeGo; U; en-GB) Presto/2.10.254 Version/12.00*/
const Device UA2234999 = {S71492, 163, { &P21230,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Mobiistar Touch S03 Build/VENUS_00.03.142.H1091) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2504665 = {S71493, 164, { &P21230,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Mobiistar Touch S03 Build/VENUS_00.03.155.H1091A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2557568 = {S71494, 164, { &P21230,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Mobiistar Touch S03 Build/VENUS_00.03.166.H1091A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2425245 = {S71495, 140, { &P15767,&P17090,&P20779,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) MQQBrowser/2.8 Mobile/8J2 Safari/7534.48.3*/
const Device UA2430914 = {S71496, 140, { &P15767,&P17090,&P20779,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) MQQBrowser/2.8 Mobile/8L1 Safari/7534.48.3*/
const Device UA2537074 = {S71497, 34, { &P12280,&P17134,&P17470,&P18092 }};	/*Useragent - Tweetcaster for iPhone 3.0.0.13133*/
const Device UA2713471 = {S71498, 140, { &P22866,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; AT-AS43D3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2753734 = {S71499, 160, { &P14969,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; HTC Desire S Build/GRJ90) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2879719 = {S71500, 146, { &P20253,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; th-th; i-mobile IQ 5.3 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103116 = {S71501, 68, { &P20253,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; i-mobile IQ 5.3 Build/JOP40D)*/
const Device UA2943972 = {S71502, 133, { &P18665,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; N861 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2975042 = {S71503, 138, { &P23705,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C811 4G Build/C811M020) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA403 = {S71504, 76, { &P11705,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-L7e/08.01.0AR MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/1.0*/
const Device UA2102 = {S71505, 74, { &P11705,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-L7_MinhThuc/00.00.00 MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA7794 = {S71506, 66, { &P11705,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-L7/08.B7.DCR MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA7845 = {S71507, 71, { &P11705,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-L7_V2.1/08.B7.ACR MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA15423 = {S71508, 98, { &P11705,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-L7i/AAUG2103AA 08.02.06R/10.21.2005 MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/1.0*/
const Device UA30848 = {S71509, 71, { &P11705,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-L7 .T.H/08.B7.DCR MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61650 = {S71510, 72, { &P11705,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-L7_Itunes/00.00.00 MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61958 = {S71511, 67, { &P11705,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-L7v/08.B7.ABR MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA50367 = {S71512, 145, { &P13244,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; Motorola_i1 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2233247 = {S71513, 160, { &P18645,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I747 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258021 = {S71514, 168, { &P18645,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03R; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2404146 = {S71515, 160, { &P18645,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L710 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2242215 = {S71516, 157, { &P20925,&P18066,&P18062,&P18092 }};	/*Useragent - AT-AS45SE Linux/3.0.13 Android/4.0.4 Release/09.28.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2480845 = {S71517, 140, { &P20925,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; AT-AS45SE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2348526 = {S71518, 171, { &P21598,&P18066,&P18062,&P18092 }};	/*Useragent - Mobiistar Touch Kem 452 Linux/3.0.13 Android/4.0.4 Release/09.21.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2701838 = {S71519, 159, { &P21598,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar_Touch_KEM_452_HDMI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2833588 = {S71520, 154, { &P21598,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar Touch Kem 452 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2935863 = {S71521, 157, { &P21598,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-; Mobiistar_Touch_KEM_452_HDMI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567585 = {S71522, 138, { &P12814,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I777 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2938301 = {S71523, 146, { &P12814,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I777 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2601881 = {S71524, 138, { &P18697,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I415 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2738190 = {S71525, 152, { &P15785,&P5843,&P24677,&P18092 }};	/*Useragent - Nokia201/2.0 (11.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia201) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2752113 = {S71526, 155, { &P23722,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; O+ 8.36 Android Build/Oplus8.36_Oplus) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049231 = {S71527, 77, { &P23722,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; O+ 8.36 Android Build/Oplus8.36_Oplus)*/
const Device UA2755998 = {S71528, 133, { &P17482,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80G9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2782597 = {S71529, 130, { &P17482,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 80G9 Build/MR1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2814340 = {S71530, 89, { &P15364,&P7127,&P21959,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9.3a4pre) Gecko/X Minefield/3.7a4pre*/
const Device UA2933855 = {S71531, 143, { &P15364,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: wow-proxy)*/
const Device UA2978619 = {S71532, 142, { &P15364,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: xxdaili4)*/
const Device UA3000831 = {S71533, 151, { &P15364,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: s~scroll-down-hrd)*/
const Device UA3010761 = {S71534, 148, { &P15364,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: s~surf6009-hrd)*/
const Device UA3019452 = {S71535, 149, { &P15364,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: s~proxy6002-hrd)*/
const Device UA3020575 = {S71536, 148, { &P15364,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: s~appcodes-hrd)*/
const Device UA3020640 = {S71537, 150, { &P15364,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: s~proxydisk9-hrd)*/
const Device UA3020920 = {S71538, 150, { &P15364,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: s~surfproxy4-hrd)*/
const Device UA3057943 = {S71539, 87, { &P15364,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0 +http://www.bing.com/bingbot.htm);c2VvLWNyYXdsZXI=*/
const Device UA3074327 = {S71540, 147, { &P15364,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: s~surf702-hrd)*/
const Device UA3078609 = {S71541, 153, { &P15364,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: s~swedishdanish-hrd)*/
const Device UA3092178 = {S71542, 150, { &P15364,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: s~surfproxy6-hrd)*/
const Device UA3105087 = {S71543, 147, { &P15364,&P7480,&P18245,&P18121 }};	/*Useragent - Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm) AppEngine-Google; (+http://code.google.com/appengine; appid: s~surf603-hrd)*/
const Device UA2975738 = {S71544, 138, { &P22610,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021668 = {S71545, 138, { &P22610,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023220 = {S71546, 137, { &P22610,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022938 = {S71547, 134, { &P24368,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5215 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023597 = {S71548, 161, { &P21020,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Micromax A116 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3094082 = {S71549, 140, { &P18449,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST27i Build/6.1.1.B.1.10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114928 = {S71550, 140, { &P18449,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST27a Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA25755 = {S71551, 73, { &P12138,&P5027,&P3412,&P18092 }};	/*Useragent - BlackBerry8130/4.3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105*/
const Device UA2659309 = {S71552, 133, { &P12866,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; BASE Tab Build/FRF91) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA703694 = {S71553, 100, { &P14826,&P5843,&P17338,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (05.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac1-01/UC Browser7.9.0.102/69/352*/
const Device UA305146 = {S71554, 143, { &P21361,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; YP-G50 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018455 = {S71555, 65, { &P21361,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; YP-G50 Build/ECLAIR)*/
const Device UA396135 = {S71556, 133, { &P24556,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; CT704 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA898917 = {S71557, 85, { &P12137,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s5620) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA1954917 = {S71558, 85, { &P12137,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5620) UCBrowser8.3.1.161/69/405/UCWEB Mobile*/
const Device UA2360915 = {S71559, 158, { &P22037,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2608504 = {S71560, 158, { &P22037,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2507403 = {S71561, 141, { &P20001,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S720 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2676924 = {S71562, 137, { &P18636,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS930 4G Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676940 = {S71563, 138, { &P18290,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ617 Build/7.7.1-85_MZ617-27) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2970574 = {S71564, 136, { &P23697,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9192 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3092362 = {S71565, 129, { &P23697,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9192 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3063905 = {S71566, 130, { &P17482,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 80G9 Build/MR1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Safari/537.36*/
const Device UA3065147 = {S71567, 133, { &P17482,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80G9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3068067 = {S71568, 138, { &P19844,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80 COBALT Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA49812 = {S71569, 139, { &P13090,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; C8800 Build/HuaweiC8800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49813 = {S71570, 138, { &P13090,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HW-HUAWEI_C8800/C8800V100R001C92B818; 480*800; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2019723 = {S71571, 127, { &P13090,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; C8800 Build/HuaweiC8800) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2642489 = {S71572, 139, { &P13090,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; C8800 Build/HuaweiC8800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA205035 = {S71573, 134, { &P18049,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; DISCO10 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA851755 = {S71574, 58, { &P18049,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; DISCO10 Build/FRF85B)*/
const Device UA651336 = {S71575, 127, { &P17586,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; it-it; OP110 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA800002 = {S71576, 142, { &P21371,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; ViewSonic-V350 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA859284 = {S71577, 144, { &P21047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; ODYS Space Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3117653 = {S71578, 157, { &P21047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; ODYS Space_TKI_BRR233v9 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1990192 = {S71579, 134, { &P21813,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fa-ir; GLX G1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2496187 = {S71580, 134, { &P21813,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; fa-ir; GLX G1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3119288 = {S71581, 138, { &P21813,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GLX G111+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1996888 = {S71582, 136, { &P17595,&P18207,&P22460,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11*/
const Device UA2001939 = {S71583, 136, { &P17595,&P18207,&P22460,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11*/
const Device UA2200547 = {S71584, 136, { &P17595,&P18207,&P22460,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11*/
const Device UA2673132 = {S71585, 136, { &P17595,&P18207,&P22460,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11*/
const Device UA3012748 = {S71586, 136, { &P17595,&P18207,&P22460,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11*/
const Device UA2567021 = {S71587, 138, { &P18682,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-M950 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602486 = {S71588, 140, { &P17968,&P18111,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile*/
const Device UA2603409 = {S71589, 140, { &P17968,&P18111,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile*/
const Device UA2735004 = {S71590, 133, { &P24571,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; JXD-P1000 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA44164 = {S71591, 102, { &P13795,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-C5220/1.0 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA2467258 = {S71592, 120, { &P13324,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaN8-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2507907 = {S71593, 108, { &P17595,&P18120,&P19818,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.8.1 Safari/534.34*/
const Device UA2533205 = {S71594, 172, { &P12056,&P5204,&P24676,&P18092 }};	/*Useragent - Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7210supernova) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2560809 = {S71595, 141, { &P22624,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Burst S280 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666392 = {S71596, 139, { &P22747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; PIONEER_P1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2711877 = {S71597, 139, { &P22747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Pioneer_P1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2708936 = {S71598, 139, { &P22604,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I879 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3073920 = {S71599, 61, { &P22604,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SCH-I879 Build/JZO54K)*/
const Device UA2773256 = {S71600, 157, { &P18509,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2797425 = {S71601, 127, { &P23001,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M97 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2945972 = {S71602, 158, { &P12968,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26i Build/6.1.A.2.55) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2974654 = {S71603, 137, { &P22609,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6312 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3019117 = {S71604, 99, { &P15364,&P21460,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.3) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA3020748 = {S71605, 106, { &P15364,&P21460,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.3; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA3081051 = {S71606, 137, { &P18484,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-US730 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA126705 = {S71607, 66, { &P15364,&P18110,&P20418,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; ; Linux x86_64) Opera 7.50 [en]*/
const Device UA2207844 = {S71608, 49, { &P15364,&P18110,&P20418,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; U) Opera 7.50  [en]*/
const Device UA448068 = {S71609, 74, { &P15364,&P17017,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; U; Warp 4.5; en-US; rv:1.8.0.7) Gecko/X Firefox/1.5.0.7*/
const Device UA559227 = {S71610, 47, { &P15364,&P17017,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.5;) Firefox/1.5*/
const Device UA2583086 = {S71611, 131, { &P12879,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7510 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2634810 = {S71612, 131, { &P12879,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P7510 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2738244 = {S71613, 137, { &P22595,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919V Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2997741 = {S71614, 130, { &P22595,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919V Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2964282 = {S71615, 136, { &P24042,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; ARNOVA 101 G4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2478 = {S71616, 103, { &P18088,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320; SPV M650; OpVer 21.138.9.693*/
const Device UA24789 = {S71617, 103, { &P18088,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320;SPV M3100; OpVer 14.211.1.613*/
const Device UA35932 = {S71618, 103, { &P18088,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320; SPV M650; OpVer 21.184.2.732*/
const Device UA39917 = {S71619, 127, { &P18088,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320; CHT 9100/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62876 = {S71620, 60, { &P18088,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8)*/
const Device UA64907 = {S71621, 103, { &P18088,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320;SPV M3100; OpVer 14.211.2.732*/
const Device UA71993 = {S71622, 103, { &P18088,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320;SPV M3100; OpVer 14.211.1.611*/
const Device UA76405 = {S71623, 103, { &P18088,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320;SPV M3100; OpVer 14.211.1.612*/
const Device UA86528 = {S71624, 130, { &P18088,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) Smartphone; 320x240; Mail/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA86986 = {S71625, 65, { &P18088,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) G500*/
const Device UA87367 = {S71626, 87, { &P18088,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) SPV E610 /SW2.1.1 MSIE/6.0*/
const Device UA331243 = {S71627, 124, { &P16968,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; FujitsuToshibaMobileCommun; IS12T; KDDI)*/
const Device UA2144953 = {S71628, 76, { &P15364,&P18144,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; OpenBSD amd64; en-US; rv:1.9.0.1) Gecko/X Firefox/3.0.1*/
const Device UA2458143 = {S71629, 144, { &P20867,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; miTab NEO Build/ICS.g08refem618.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2468127 = {S71630, 142, { &P20867,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; miTab NEO Build/ICS.miTab NEO.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2538345 = {S71631, 92, { &P14984,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5250) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2600528 = {S71632, 138, { &P20713,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID7065 Build/ICS.MID7065.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2713454 = {S71633, 161, { &P16356,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; GT-N7000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2737822 = {S71634, 161, { &P16356,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-N7000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA5075 = {S71635, 77, { &P15526,&P5448,&P4626,&P18092 }};	/*Useragent - Swisscom/1.0/HTC_HD_mini/Opera 9.7/1.11.170.1 (87652) (Windows NT 5.1; U; en)*/
const Device UA28148 = {S71636, 54, { &P15526,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_HD_mini_T5555 Opera/9.7 (Windows NT 5.1; U; zh-tw)*/
const Device UA28151 = {S71637, 51, { &P15526,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_HD_mini_T5555 Opera/9.7 (Windows NT 5.1; U; en)*/
const Device UA90153 = {S71638, 80, { &P15526,&P5448,&P4626,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_HD_mini/Opera 9.7/1.11.164.1 (87733)/(Windows NT 5.1; U; es-ES)*/
const Device UA146093 = {S71639, 77, { &P15526,&P5448,&P4626,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_HD_mini/Opera 9.7/1.10.161.1 (86611)/(Windows NT 5.1; U; en)*/
const Device UA178549 = {S71640, 77, { &P15526,&P5448,&P4626,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_HD_mini/Opera 9.7/1.41.161.1 (10904)/(Windows NT 5.1; U; en)*/
const Device UA494705 = {S71641, 77, { &P15526,&P5448,&P4626,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_HD_mini/Opera 9.7/1.11.162.1 (87652)/(Windows NT 5.1; U; de)*/
const Device UA511281 = {S71642, 77, { &P15526,&P5448,&P4626,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_HD_mini/Opera 9.7/1.41.162.1 (10904)/(Windows NT 5.1; U; de)*/
const Device UA2262600 = {S71643, 77, { &P15526,&P5448,&P4626,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_HD_mini/Opera 9.7/1.36.184.2 (94538)/(Windows NT 5.1; U; en)*/
const Device UA371002 = {S71644, 141, { &P18013,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; EV-S100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA630408 = {S71645, 161, { &P14183,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-I997 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA826749 = {S71646, 155, { &P14183,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-I997 Build/GINGERBREAD; SuperCM7-v1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA600594 = {S71647, 85, { &P15655,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s3850) UCBrowser8.2.0.132/69/452/UCWEB Mobile*/
const Device UA616087 = {S71648, 85, { &P15655,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s3850) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2810643 = {S71649, 38, { &P15655,&P17484,&P24667,&P18092 }};	/*Useragent - samsung-gt-s3850/UCWEB8.2.0.116/69/352*/
const Device UA603874 = {S71650, 85, { &P18141,&P22220,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 1.67) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2*/
const Device UA1995881 = {S71651, 132, { &P15767,&P18117,&P20776,&P18092 }};	/*Useragent - MQQBrowser/2.5.1 Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA2007376 = {S71652, 130, { &P15767,&P18117,&P20776,&P18092 }};	/*Useragent - MQQBrowser/2.5.1 Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3*/
const Device UA2210090 = {S71653, 113, { &P15364,&P5690,&P22467,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/528.4 (KHTML, like Gecko) Iron/0.3.155.0 Safari/X.528*/
const Device UA2213031 = {S71654, 114, { &P15364,&P5690,&P22467,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Iron/0.2.152.0 Safari/X.525*/
const Device UA2741936 = {S71655, 113, { &P15364,&P5690,&P22467,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/528.5 (KHTML, like Gecko) Iron/0.4.155.0 Safari/528.5*/
const Device UA2424668 = {S71656, 124, { &P12142,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU5) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2426010 = {S71657, 171, { &P15575,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2469389 = {S71658, 171, { &P15575,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2596876 = {S71659, 171, { &P15575,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2845885 = {S71660, 172, { &P15575,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2556064 = {S71661, 129, { &P20270,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; U20GT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2862977 = {S71662, 132, { &P20270,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; U20GT_WS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2667116 = {S71663, 137, { &P23455,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D2-712 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2782043 = {S71664, 138, { &P12280,&P17146,&P22902,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.4.2 Mobile/9A405 Safari/7534.48.3*/
const Device UA3054017 = {S71665, 136, { &P12280,&P17146,&P22902,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.4.2 Mobile/9A334 Safari/7534.48.3*/
const Device UA3081387 = {S71666, 137, { &P14877,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126442 = {S71667, 137, { &P14877,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9000 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA6006 = {S71668, 79, { &P13265,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x400) Opera VZW:SCH-i910*/
const Device UA105415 = {S71669, 102, { &P13265,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) VZW:SCH-i910 PPC 240x400*/
const Device UA572279 = {S71670, 100, { &P14540,&P5843,&P17338,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-01/UC Browser7.9.0.102/69/352*/
const Device UA818535 = {S71671, 108, { &P19799,&P17017,&P19800,&P18092 }};	/*Useragent - Mozilla/5.0 (DTV) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.4 AQUOSBrowser/1.0 (US00DTV;V;0001;0001)*/
const Device UA905664 = {S71672, 136, { &P21566,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; MIDC010PR001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2001279 = {S71673, 122, { &P15364,&P5690,&P22436,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 Comodo_Dragon/21.1.1.0*/
const Device UA2724412 = {S71674, 129, { &P15364,&P5690,&P22436,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 Comodo_Dragon/21.1.1.0*/
const Device UA2030843 = {S71675, 152, { &P12955,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Blade Build/JRO03C; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2355267 = {S71676, 152, { &P12955,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Blade Build/JRO03C; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2040955 = {S71677, 71, { &P15364,&P7133,&P18345,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X Firefox/15.0 slurp*/
const Device UA2043221 = {S71678, 73, { &P15364,&P7133,&P18345,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X Firefox/15.0.1 slurp*/
const Device UA2192194 = {S71679, 66, { &P15364,&P7133,&P18345,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:15.0) Gecko/X Firefox/15.0.1 slurp*/
const Device UA2205384 = {S71680, 79, { &P15364,&P7133,&P18345,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/X Firefox/15.0.1 slurp slurp*/
const Device UA2337879 = {S71681, 64, { &P15364,&P7133,&P18345,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:15.0) Gecko/X Firefox/15.0 slurp*/
const Device UA2158210 = {S71682, 140, { &P17475,&P5768,&P24668,&P18092 }};	/*Useragent - Nokia6300/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia6300) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2422321 = {S71683, 166, { &P18673,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2258081 = {S71684, 133, { &P18465,&P18110,&P19814,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.2 Safari/535.19 Silk-Accelerated=false*/
const Device UA2423541 = {S71685, 118, { &P15364,&P16110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; zh-CN) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19.1*/
const Device UA2475287 = {S71686, 114, { &P15364,&P16110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/531.9 (KHTML, like Gecko) Version/4.0.3 Safari/531.9.1*/
const Device UA2517929 = {S71687, 118, { &P15364,&P16110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA2518212 = {S71688, 119, { &P15364,&P16110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10*/
const Device UA2534722 = {S71689, 153, { &P22638,&P5629,&P21729,&P18092 }};	/*Useragent - OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; W20 Build/Symphony_W20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2537321 = {S71690, 155, { &P21152,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ALCATEL ONE TOUCH 930D Build/1.2.10.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2712803 = {S71691, 153, { &P22961,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010D Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2752947 = {S71692, 140, { &P18084,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754940 = {S71693, 130, { &P15985,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2799042 = {S71694, 131, { &P15985,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2800088 = {S71695, 139, { &P22756,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C1604 Build/11.3.A.2.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2961577 = {S71696, 136, { &P20146,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9082 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3092289 = {S71697, 129, { &P20146,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9082 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2976100 = {S71698, 137, { &P22748,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E425f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021323 = {S71699, 137, { &P22748,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E425g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022378 = {S71700, 136, { &P22748,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E425 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA63193 = {S71701, 97, { &P13301,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-B2700/XXIB2 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA56347 = {S71702, 151, { &P13437,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-gb; T-Mobile G2 Touch Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA64279 = {S71703, 151, { &P13437,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-gb; T-Mobile_G2_Touch Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA88822 = {S71704, 152, { &P13437,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-gb; T-Mobile G2 Touch Build/CUPCAKE) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA3117828 = {S71705, 151, { &P13437,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-gb; T-Mobile_G2_Touch Build/CUPCAKE) AppleWebKit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA67327 = {S71706, 78, { &P11742,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-C5212/C5212DDIK1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA2370593 = {S71707, 83, { &P11742,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-C5212i/C5212iJPKD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA427490 = {S71708, 136, { &P17595,&P17297,&P22455,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11*/
const Device UA632729 = {S71709, 136, { &P17595,&P17297,&P22455,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11*/
const Device UA1986603 = {S71710, 136, { &P17595,&P17297,&P22455,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11*/
const Device UA2204372 = {S71711, 136, { &P17595,&P17297,&P22455,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11*/
const Device UA924766 = {S71712, 76, { &P15364,&P18110,&P22316,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:14.0) Gecko/X Firefox/14.0.1 SeaMonkey/2.11*/
const Device UA2422695 = {S71713, 119, { &P11880,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA5320) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2555519 = {S71714, 41, { &P11880,&P5890,&P24673,&P18092 }};	/*Useragent - NOKIA5320UCBrowser/8.7.1.234/28/444/UCWEB*/
const Device UA2843436 = {S71715, 94, { &P11880,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5320d-1) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2567106 = {S71716, 142, { &P21119,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SoftwinerEvb Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2666765 = {S71717, 135, { &P21119,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SoftwinerEvb Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2702447 = {S71718, 143, { &P21119,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Softwinerf900 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602723 = {S71719, 157, { &P15575,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.53463*/
const Device UA2774065 = {S71720, 157, { &P15575,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799227 = {S71721, 157, { &P15575,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2658325 = {S71722, 136, { &P21554,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A5000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2711516 = {S71723, 31, { &P18254,&P7480,&P22895,&P18091 }};	/*Useragent - RSSOwl/2.2.0.X (Windows; U; en)*/
const Device UA2770675 = {S71724, 27, { &P18254,&P7480,&P22895,&P18091 }};	/*Useragent - RSSOwl/2.2.0.X (X11; U; en)*/
const Device UA2754624 = {S71725, 145, { &P16263,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI MediaPad Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2782728 = {S71726, 141, { &P16263,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SpringBoard Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2867362 = {S71727, 145, { &P16263,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MediaPad 7 Lite Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2877824 = {S71728, 146, { &P16263,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MediaPad 10 LINK Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3094191 = {S71729, 155, { &P17595,&P18120,&P23486,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.6.5.X (Second Life Release; default skin) Safari/533.3*/
const Device UA2947353 = {S71730, 157, { &P18509,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2975963 = {S71731, 141, { &P20399,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP8010 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974119 = {S71732, 137, { &P19894,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R830 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA4810 = {S71733, 58, { &P12192,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; nb)*/
const Device UA4812 = {S71734, 58, { &P12192,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; fr)*/
const Device UA23974 = {S71735, 58, { &P12192,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; sv)*/
const Device UA27945 = {S71736, 58, { &P12192,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; nl)*/
const Device UA27947 = {S71737, 61, { &P12192,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; zh-tw)*/
const Device UA27951 = {S71738, 58, { &P12192,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; ru)*/
const Device UA27952 = {S71739, 58, { &P12192,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; de)*/
const Device UA27953 = {S71740, 58, { &P12192,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; da)*/
const Device UA27955 = {S71741, 58, { &P12192,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; it)*/
const Device UA36627 = {S71742, 59, { &P12192,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.70 (Windows NT 5.1; U; en)*/
const Device UA36806 = {S71743, 58, { &P12192,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; pt)*/
const Device UA36807 = {S71744, 61, { &P12192,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; es-ES)*/
const Device UA45610 = {S71745, 58, { &P12192,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; el)*/
const Device UA51260 = {S71746, 58, { &P12192,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Opera/9.7 (Windows NT 5.1; U; en)*/
const Device UA675030 = {S71747, 140, { &P21199,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; tr-tr; Turkcell T11 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2531411 = {S71748, 131, { &P23510,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SC-90JB Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2567967 = {S71749, 138, { &P12942,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I927 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2849983 = {S71750, 58, { &P15364,&P5448,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; WindowsCE; Trident/4.0)*/
const Device UA2849984 = {S71751, 63, { &P15364,&P5448,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; 68K; Tablet PC; Trident/5.0)*/
const Device UA2898939 = {S71752, 108, { &P17595,&P18120,&P21712,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.9.1 Safari/534.34*/
const Device UA2734385 = {S71753, 132, { &P20241,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; PMP5570C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2796299 = {S71754, 134, { &P23207,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A4+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA22381 = {S71755, 140, { &P15447,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; in-id; PICOpad-QGN Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA166171 = {S71756, 139, { &P15447,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; PICOpad-QGN Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA451798 = {S71757, 133, { &P20490,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; L102 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030692 = {S71758, 130, { &P22025,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VL-879 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2204934 = {S71759, 118, { &P11945,&P17484,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; gt-s5233s) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile*/
const Device UA2457244 = {S71760, 160, { &P12132,&P17485,&P24673,&P18092 }};	/*Useragent - SonyEricssonJ10i2/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; fa; sonyericssonj10i2) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2482566 = {S71761, 158, { &P12132,&P17485,&P24673,&P18092 }};	/*Useragent - SonyEricssonJ10i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; fa; sonyericssonj10i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2507603 = {S71762, 84, { &P18436,&P16110,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (IE 11.0; Windows NT 6.2; WOW64; Trident/7.0; Touch; rv:11.0) like Gecko*/
const Device UA2558807 = {S71763, 89, { &P18436,&P16110,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (IE 11.0; Windows NT 6.2; Win64; x64; Trident/7.0; Touch; rv:11.0) like Gecko*/
const Device UA2938821 = {S71764, 88, { &P18436,&P16110,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; Trident/7.0; Touch; WebView/1.0; rv:11.0) like Gecko*/
const Device UA2958872 = {S71765, 93, { &P18436,&P16110,&P21461,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; Trident/7.0; Touch; WebView/1.0; rv:11.0) like Gecko*/
const Device UA2556253 = {S71766, 157, { &P15649,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-S5570 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2664428 = {S71767, 156, { &P15649,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-S5570 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958278 = {S71768, 159, { &P15649,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-S5570 Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2557629 = {S71769, 147, { &P15364,&P18066,&P21729,&P18092 }};	/*Useragent - OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Symphony Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2602706 = {S71770, 149, { &P15364,&P18066,&P21729,&P18092 }};	/*Useragent - OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; W70 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2855042 = {S71771, 149, { &P15364,&P18066,&P21729,&P18092 }};	/*Useragent - OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; W80 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2961923 = {S71772, 150, { &P15364,&P18066,&P21729,&P18092 }};	/*Useragent - OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xplorer T8i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3051363 = {S71773, 150, { &P15364,&P18066,&P21729,&P18092 }};	/*Useragent - OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xplorer T7i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2675075 = {S71774, 132, { &P13644,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; EVO Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676337 = {S71775, 132, { &P13644,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; EVO Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708332 = {S71776, 165, { &P15798,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Explorer A310e Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2710623 = {S71777, 141, { &P12280,&P17146,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 UCBrowser/9.0.1.284 Mobile*/
const Device UA2715426 = {S71778, 141, { &P12280,&P17146,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 UCBrowser/9.0.1.284 Mobile*/
const Device UA2974472 = {S71779, 141, { &P11747,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire V Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022967 = {S71780, 141, { &P11747,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire P Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023613 = {S71781, 142, { &P11747,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire VC Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3081187 = {S71782, 135, { &P19861,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SCH-I605 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA361934 = {S71783, 46, { &P12280,&P17146,&P17470,&P18092 }};	/*Useragent - Facebook 4100.0 (iPhone; iPhone OS 5.0; en_US)*/
const Device UA2463382 = {S71784, 48, { &P12280,&P17146,&P17470,&P18092 }};	/*Useragent - Instagram 2.4.0 (iPhone; iPhone OS 5.0.1; en_US)*/
const Device UA2476068 = {S71785, 63, { &P12280,&P17146,&P17470,&P18092 }};	/*Useragent - Instagram 2.5.1 (iPhone; iPhone OS 5.0; en_US) AppleWebKit/420+*/
const Device UA160544 = {S71786, 60, { &P16591,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxQ50/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA183968 = {S71787, 142, { &P13242,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; A853 Build/SHLA_U2_05.14.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA190690 = {S71788, 142, { &P13242,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; A853 Build/SHLA_U2_05.12.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA392533 = {S71789, 142, { &P13242,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; A853 Build/SHLA_U2_05.15.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA448409 = {S71790, 143, { &P13242,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A853 Build/SUSKT_U1_01.19.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA721970 = {S71791, 133, { &P13242,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; A853 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1852929 = {S71792, 132, { &P13242,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; A853 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1882895 = {S71793, 142, { &P13242,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; A853 Build/SHLA_U2_05.15.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019091 = {S71794, 67, { &P13242,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; A853 Build/SUSKT_U1_01.19.0)*/
const Device UA625974 = {S71795, 81, { &P13324,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaN8-00)/UC Browser8.2.0.132/50/355/UCWEB Mobile*/
const Device UA2345534 = {S71796, 140, { &P12108,&P5843,&P24668,&P18092 }};	/*Useragent - Nokia7230/5.0 (10.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia7230) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2028873 = {S71797, 139, { &P14183,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I997 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029127 = {S71798, 139, { &P14183,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I997 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2187904 = {S71799, 139, { &P14183,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I997 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2342372 = {S71800, 141, { &P14183,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-i997 Build/SGH-i997) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2355372 = {S71801, 139, { &P14183,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I997 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2502605 = {S71802, 61, { &P14183,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-I997 Build/JZO54K)*/
const Device UA2594327 = {S71803, 141, { &P14183,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I997 Build/SGH-I977) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528927 = {S71804, 138, { &P21153,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-03E Build/V16R24Ac) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558077 = {S71805, 138, { &P21153,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-03E Build/V18R26Dc) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2579791 = {S71806, 137, { &P21153,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-03E Build/V16R24A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2581030 = {S71807, 137, { &P21153,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-03E Build/V18R26D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2897926 = {S71808, 138, { &P21153,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-03E Build/V20R29Bc) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2898913 = {S71809, 137, { &P21153,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-03E Build/V20R29B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3093340 = {S71810, 137, { &P21153,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-03E Build/V21R31A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3093405 = {S71811, 138, { &P21153,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; F-03E Build/V21R31Ac) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2782599 = {S71812, 145, { &P12280,&P18117,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B179 Safari/537.4 Maxthon*/
const Device UA2677381 = {S71813, 119, { &P17595,&P21721,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.65 Safari/537.31*/
const Device UA2735101 = {S71814, 138, { &P20714,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID8065 Build/ICS.MID8065.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2974750 = {S71815, 97, { &P20729,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A27) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3033073 = {S71816, 139, { &P21605,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C2104 Build/15.0.A.2.17) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3060066 = {S71817, 132, { &P21605,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C2104 Build/15.0.A.2.17) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3006416 = {S71818, 138, { &P23697,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9192 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3011545 = {S71819, 60, { &P23697,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9192 Build/JDQ39)*/
const Device UA3080367 = {S71820, 136, { &P24061,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-S7270 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3095060 = {S71821, 137, { &P24061,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-S7270L Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081232 = {S71822, 138, { &P24513,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080296 = {S71823, 93, { &P13123,&P5629,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5360) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3115128 = {S71824, 90, { &P13123,&P5629,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; vi; GT-S5360) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA17269 = {S71825, 62, { &P14340,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia3109c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA39056 = {S71826, 67, { &P14340,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia3109c_CMCC/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA67656 = {S71827, 141, { &P18061,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0; en-us; sdk Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17</string>*/
const Device UA67678 = {S71828, 140, { &P18061,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.0; en-us; rk2818sdk Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018958 = {S71829, 152, { &P18061,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Android_2.1-1.1.3; es-es; rk2818sdk Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA379134 = {S71830, 134, { &P19785,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; CT1002 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030520 = {S71831, 129, { &P18088,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; OP111 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2716269 = {S71832, 97, { &P14519,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaN97 mini) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA3100600 = {S71833, 35, { &P14519,&P5129,&P24676,&P18092 }};	/*Useragent - NokiaN97 mini/UCWEB8.9.0.253/50/999*/
const Device UA3116165 = {S71834, 106, { &P21002,&P17484,&P18328,&P18092 }};	/*Useragent - SAMSUNG-GT-C3312R Opera/9.80 (J2ME/MIDP; Opera Mini/7.1.33680/31.1475; U; en) Presto/2.8.119 Version/11.10*/
const Device UA3081780 = {S71835, 152, { &P19771,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3082589 = {S71836, 152, { &P19771,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083756 = {S71837, 152, { &P19771,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA1599 = {S71838, 30, { &P15364,&P17017,&P3601,&P18092 }};	/*Useragent - Amoi 8512/R23.0 NF-Browser/3.3*/
const Device UA17201 = {S71839, 30, { &P15364,&P17017,&P3601,&P18092 }};	/*Useragent - Amoi 8512/R21.0 NF-Browser/3.3*/
const Device UA24235 = {S71840, 30, { &P15364,&P17017,&P3601,&P18092 }};	/*Useragent - Amoi 8512/R18.0 NF-Browser/3.3*/
const Device UA24236 = {S71841, 30, { &P15364,&P17017,&P3601,&P18092 }};	/*Useragent - Amoi 8512/R22.0 NF-Browser/3.3*/
const Device UA39021 = {S71842, 30, { &P15364,&P17017,&P3601,&P18092 }};	/*Useragent - Amoi 8512/R18.5 NF-Browser/3.3*/
const Device UA53595 = {S71843, 58, { &P15364,&P17017,&P3601,&P18092 }};	/*Useragent - ACS-NF/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 Qtv/4.3*/
const Device UA54362 = {S71844, 78, { &P15364,&P17017,&P3601,&P18092 }};	/*Useragent - Red Mobile F188/1.0 ACS-NF/3.3 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63693 = {S71845, 30, { &P15364,&P17017,&P3601,&P18092 }};	/*Useragent - Amoi 8512/R18.7 NF-Browser/3.3*/
const Device UA63694 = {S71846, 30, { &P15364,&P17017,&P3601,&P18092 }};	/*Useragent - Amoi 8512/R28.0 NF-Browser/3.3*/
const Device UA86495 = {S71847, 61, { &P15364,&P17017,&P3601,&P18092 }};	/*Useragent - Mozilla/4.0 (MobilePhone PL6650/US/1.0) NetFront/3.3L MMP/2.0*/
const Device UA633273 = {S71848, 68, { &P15364,&P17017,&P3601,&P18092 }};	/*Useragent - W186/Qualcom6246/X/NetFront/3.3Profile/MIDP-2.0Configuration/CLDC1.1*/
const Device UA5050 = {S71849, 87, { &P15545,&P5448,&P22270,&P18092 }};	/*Useragent - Palm850/V0100 Mozilla/4.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA22997 = {S71850, 75, { &P15545,&P5448,&P22270,&P18092 }};	/*Useragent - palm850/V0100 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA11327 = {S71851, 116, { &P11962,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-U700-Vodafone/BUGE7 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA54631 = {S71852, 98, { &P11962,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-U700/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA49424 = {S71853, 144, { &P13020,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HW-HUAWEI_C8500S/C8500SV100R001C92B627SP01; 240*320; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA589768 = {S71854, 140, { &P13020,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HW-HUAWEI_C8500S/C8500SV100R001C92B655; 240*320; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA84080 = {S71855, 56, { &P15364,&P17483,&P17191,&P18092 }};	/*Useragent - Mozilla/4.1 (compatible; MSIE 6.0; ) 400x240 LGE VX10000*/
const Device UA89764 = {S71856, 63, { &P15364,&P17483,&P17191,&P18092 }};	/*Useragent - LGE-MX9700_VEZ/1.0 Mozilla/4.1 (compatible; MSIE 6.0; ) 240X400*/
const Device UA210686 = {S71857, 132, { &P17108,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC-A9192; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA344364 = {S71858, 136, { &P17108,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC-A9192/1.0; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2201713 = {S71859, 96, { &P15364,&P17285,&P22364,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:15.0) Gecko/X Firefox/15.1.1-x64 PaleMoon/15.1.1-x64*/
const Device UA2202230 = {S71860, 76, { &P15364,&P17285,&P22364,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:15.0) Gecko/X Firefox/15.1.1 PaleMoon/15.1.1*/
const Device UA2461151 = {S71861, 134, { &P20300,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; W60 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528367 = {S71862, 145, { &P21134,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; iBall Andi4.5q Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555250 = {S71863, 127, { &P24554,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N78 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2599248 = {S71864, 134, { &P24554,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N78 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955805 = {S71865, 127, { &P24554,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N78 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2601515 = {S71866, 138, { &P18488,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L300 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2732204 = {S71867, 137, { &P21028,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; eZee' Tab 706 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2795767 = {S71868, 139, { &P21028,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Smartfren Tab 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2849850 = {S71869, 145, { &P21028,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PIRANHA Ultra Tab 7.0 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2892319 = {S71870, 135, { &P21028,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; StarTab 769 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3057618 = {S71871, 138, { &P21028,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Tab 705M Build/PD200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3101206 = {S71872, 68, { &P21028,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Smartfren Tab 7 Build/JRO03H)*/
const Device UA2673074 = {S71873, 139, { &P18592,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One VX Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676586 = {S71874, 130, { &P22043,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P5113 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708912 = {S71875, 130, { &P22043,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P5113 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2713701 = {S71876, 142, { &P23107,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; TA-FONE A37 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2965534 = {S71877, 145, { &P23107,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; TA-FONE A37 HD Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2772914 = {S71878, 158, { &P18296,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One V Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2867403 = {S71879, 139, { &P19853,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7102 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957793 = {S71880, 139, { &P19853,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-N7102 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3100811 = {S71881, 149, { &P19853,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1Jelly Bean; vi-vn; GT-N7102 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2944539 = {S71882, 150, { &P15767,&P18117,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/6.0 Mobile/9B206 Safari/7534.48.3 YaBrowser/2.1.1364.172*/
const Device UA2987891 = {S71883, 126, { &P15767,&P18117,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA2988672 = {S71884, 125, { &P15767,&P18117,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Version/6.0 Safari/8536.25*/
const Device UA3059835 = {S71885, 126, { &P15767,&P18117,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A406 Safari/8536.25*/
const Device UA3086506 = {S71886, 124, { &P15767,&P18117,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA3123945 = {S71887, 148, { &P15767,&P18117,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/6.0 Mobile/9B176 Safari/7534.48.3 YaBrowser/2.1.1364.172*/
const Device UA2974879 = {S71888, 140, { &P20285,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ616-12) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021144 = {S71889, 145, { &P20285,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ616 Build/7.7.1-141-6-PAST-UMTS-LA) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA17907 = {S71890, 127, { &P12175,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-S7350-Bouygues/S7350AGIC3 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA27465 = {S71891, 111, { &P12175,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-S7350/1.0 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA45243 = {S71892, 103, { &P12175,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-S7350i/1.0 SHP/VPP/R5 NetFront/3.5 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA8551 = {S71893, 123, { &P16948,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-GT-B7320L/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA24247 = {S71894, 100, { &P16948,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-GT-B7320/1.0 MIDP/2.1 CLDC/1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA64173 = {S71895, 65, { &P17332,&P17483,&P17602,&P18092 }};	/*Useragent - LG-C310/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA64174 = {S71896, 75, { &P23539,&P17017,&P4628,&P18092 }};	/*Useragent - UTSTARCOM-GTX75/UC1.88 POLARIS/6.00 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA417952 = {S71897, 47, { &P15364,&P17285,&P24163,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; ru) Presto/2.1.1*/
const Device UA886416 = {S71898, 129, { &P15364,&P7133,&P22429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 Comodo_Dragon/19.0.3.0*/
const Device UA886419 = {S71899, 122, { &P15364,&P7133,&P22429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 Comodo_Dragon/19.0.3.0*/
const Device UA2594729 = {S71900, 138, { &P18747,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A840S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2864767 = {S71901, 139, { &P18747,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A840SP Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2670614 = {S71902, 129, { &P20948,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LePanII Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2768462 = {S71903, 139, { &P21507,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C1605 Build/11.3.A.2.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2797443 = {S71904, 133, { &P23002,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IRULU-MID Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2974153 = {S71905, 140, { &P21141,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Lenovo P770 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022990 = {S71906, 140, { &P21141,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Lenovo P770 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976017 = {S71907, 138, { &P23969,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021511 = {S71908, 138, { &P22969,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT915 Build/2_32A_2008) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA64317 = {S71909, 89, { &P11869,&P16000,&P3772,&P18092 }};	/*Useragent - SonyEricssonG700/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 958) Opera 8.65 [en]*/
const Device UA95592 = {S71910, 62, { &P15767,&P17141,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.7B367 (iPad; U; CPU OS 3_2 like Mac OS X)*/
const Device UA114717 = {S71911, 64, { &P15767,&P17141,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.7B405 (iPad; U; CPU OS 3_2_1 like Mac OS X)*/
const Device UA135124 = {S71912, 64, { &P15767,&P17141,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.7B500 (iPad; U; CPU OS 3_2_2 like Mac OS X)*/
const Device UA3098754 = {S71913, 165, { &P15767,&P17141,&P18215,&P18092 }};	/*Useragent - Mozilla/5.0 AppleCoreMedia/1.0.0.8J2 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA380133 = {S71914, 131, { &P17595,&P17296,&P22454,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7*/
const Device UA2394742 = {S71915, 131, { &P17595,&P17296,&P22454,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.7 (KHTML	 like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7*/
const Device UA673895 = {S71916, 141, { &P22821,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; KPN Smart 200 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030734 = {S71917, 142, { &P21248,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; NGM_WINN Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2260203 = {S71918, 130, { &P15767,&P18117,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA2368900 = {S71919, 128, { &P15767,&P18117,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3*/
const Device UA2315327 = {S71920, 138, { &P18488,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-L300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2487461 = {S71921, 133, { &P20605,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAB07-100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2566956 = {S71922, 138, { &P20975,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-AS730 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2673362 = {S71923, 150, { &P18460,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-1 Build/HuaweiU8950-1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709494 = {S71924, 152, { &P18460,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-51 Build/HuaweiU8950-51) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709744 = {S71925, 154, { &P18460,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-51 Build/HuaweiU8950N-51) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734838 = {S71926, 148, { &P18460,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950D Build/HuaweiU8950D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735114 = {S71927, 152, { &P18460,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950N-1 Build/HuaweiU8950N-1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2944656 = {S71928, 138, { &P23986,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SBM206SH Build/S0004) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2969670 = {S71929, 138, { &P23986,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SBM206SH Build/S0008) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3015292 = {S71930, 60, { &P23986,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SBM206SH Build/S0008)*/
const Device UA7718 = {S71931, 102, { &P13399,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-F250L/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA63653 = {S71932, 101, { &P13399,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-F250/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA67129 = {S71933, 66, { &P12505,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GX500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA908320 = {S71934, 127, { &P15364,&P17285,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/20.0.1132.43) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.43 Safari/536.11*/
const Device UA910587 = {S71935, 127, { &P15364,&P17285,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/20.0.1132.47) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11*/
const Device UA910713 = {S71936, 134, { &P15364,&P17285,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/20.0.1132.47) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11*/
const Device UA912444 = {S71937, 89, { &P15364,&P17285,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/20.0.1132.57)*/
const Device UA912581 = {S71938, 96, { &P15364,&P17285,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/20.0.1132.57)*/
const Device UA912587 = {S71939, 127, { &P15364,&P17285,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/20.0.1132.57) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA912595 = {S71940, 174, { &P15364,&P17285,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/20.0.1132.57; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA913001 = {S71941, 134, { &P15364,&P17285,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/20.0.1132.57) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA924057 = {S71942, 177, { &P15364,&P17285,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/20.0.1132.57; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)*/
const Device UA924082 = {S71943, 156, { &P15364,&P17285,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/20.0.1132.57; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618)*/
const Device UA1954969 = {S71944, 202, { &P15364,&P17285,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; GTB7.3; chromeframe/20.0.1132.57; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)*/
const Device UA2338226 = {S71945, 194, { &P15364,&P17285,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/20.0.1132.57; SLCC1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.5.21022; InfoPath.2; .NET CLR 3.5.30729; .NET CLR 3.0.30729)*/
const Device UA2425411 = {S71946, 203, { &P15364,&P17285,&P21910,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; chromeframe/20.0.1132.47; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E)*/
const Device UA683148 = {S71947, 167, { &P21047,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; ODYS Space Build/I700T_P7_T04_TSCL_FT_R_0_03_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2201672 = {S71948, 136, { &P21249,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; NGM Action Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2413679 = {S71949, 135, { &P22875,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.5; I9220 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2415815 = {S71950, 138, { &P22875,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9220 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2481913 = {S71951, 135, { &P22875,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; i9220 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2554373 = {S71952, 136, { &P22875,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; i92200 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2579057 = {S71953, 138, { &P22875,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9220 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2838458 = {S71954, 136, { &P22875,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.5; I92200 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2951599 = {S71955, 135, { &P22875,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; I9220 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2437331 = {S71956, 139, { &P18474,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9210T Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2512904 = {S71957, 146, { &P13064,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SAMSUNG-SGH-I727 Build/JOP40G) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2534064 = {S71958, 145, { &P13064,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I727 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2565564 = {S71959, 137, { &P13064,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I727 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2532230 = {S71960, 142, { &P23803,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Orbit_5.1DC Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580356 = {S71961, 133, { &P21893,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MSD7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2866155 = {S71962, 140, { &P21893,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MSD7Android Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2997360 = {S71963, 139, { &P21893,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MSD7Smarty Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3010247 = {S71964, 141, { &P21893,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MSD7 Android Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2675256 = {S71965, 137, { &P18682,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-M950 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675419 = {S71966, 136, { &P23235,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; VT75C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3068339 = {S71967, 136, { &P22950,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E977 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080936 = {S71968, 137, { &P20103,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; EK-GC100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109167 = {S71969, 137, { &P20103,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EK-GC100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065468 = {S71970, 144, { &P24066,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Lenovo A706_ROW Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA14259 = {S71971, 61, { &P11796,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia7360/2.0 (04.90) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA66955 = {S71972, 143, { &P15364,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; S3C6410ZYOF Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA67362 = {S71973, 153, { &P15364,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Edge DINGO-1.11; Android 1.6; en-us; edge Build/DRD20) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA122766 = {S71974, 148, { &P15364,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; fr-fr; BYT-P726G; Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1, gzip*/
const Device UA485022 = {S71975, 143, { &P15364,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; Dell Mini 5 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA501442 = {S71976, 156, { &P15364,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; zh-cn; Lenovo 3GC101/S-5-02;320*533;CTC/2.0) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2018083 = {S71977, 145, { &P15364,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; th-th; COSMOS_IM8500 Build/DRD35) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA152413 = {S71978, 141, { &P15364,&P17816,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Ubuntu/10.04 Chromium/7.0.517.44 Chrome/7.0.517.44 Safari/534.7*/
const Device UA2266553 = {S71979, 141, { &P15364,&P17816,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.9 (KHTML, like Gecko) Ubuntu/10.04 Chromium/7.0.530.0 Chrome/7.0.530.0 Safari/534.9*/
const Device UA2426522 = {S71980, 130, { &P15767,&P17146,&P20779,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/2.8 Mobile/9A405 Safari/7534.48.3*/
const Device UA2852011 = {S71981, 130, { &P15767,&P17146,&P20779,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/2.8.1 Mobile/9A334 Safari/7534.48.3*/
const Device UA2503890 = {S71982, 139, { &P18300,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T769 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555159 = {S71983, 150, { &P18300,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T769 Build/Baked-T769-JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2578746 = {S71984, 147, { &P18300,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-T769 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778168 = {S71985, 69, { &P18300,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SAMSUNG-SGH-T769 Build/JZO54K)*/
const Device UA2530486 = {S71986, 70, { &P18176,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 900;7.10;8858)*/
const Device UA2607492 = {S71987, 70, { &P18176,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 900;7.10;8862)*/
const Device UA3100856 = {S71988, 70, { &P18176,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 900;7.10;8860)*/
const Device UA2566907 = {S71989, 127, { &P15602,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A501 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2755069 = {S71990, 141, { &P21119,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SoftwinerEvb Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799866 = {S71991, 143, { &P21119,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Softwinerkf026 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867897 = {S71992, 134, { &P21119,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SoftwinerEvb Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2893098 = {S71993, 142, { &P21119,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Softwinerf900 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893500 = {S71994, 142, { &P21119,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Softwinerf761 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2974116 = {S71995, 151, { &P18505,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MB886 Build/7.7.1Q-144_MB886_MR1-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3081272 = {S71996, 140, { &P18183,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ615-12) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA693062 = {S71997, 140, { &P21828,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; i867 Build/ESE81) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2024795 = {S71998, 62, { &P21828,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.1-update1; i867 Build/ESE81)*/
const Device UA1978620 = {S71999, 131, { &P21021,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328t Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2025704 = {S72000, 140, { &P21021,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328t Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030355 = {S72001, 126, { &P21021,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328t Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2375836 = {S72002, 62, { &P21021,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; HTC T328t Build/IML74K)*/
const Device UA2520488 = {S72003, 131, { &P21021,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328t Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2226809 = {S72004, 35, { &P14519,&P5129,&P24672,&P18092 }};	/*Useragent - NokiaN97 mini/UCWEB8.6.0.180/50/999*/
const Device UA2239274 = {S72005, 123, { &P14519,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaN97 mini) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/352/UCWEB Mobile*/
const Device UA2354088 = {S72006, 142, { &P23979,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; sr-rs; Telenor_Smart Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2530501 = {S72007, 166, { &P15575,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2662593 = {S72008, 167, { &P15575,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9100G Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3057884 = {S72009, 170, { &P15575,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9100 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2536579 = {S72010, 93, { &P15575,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; GT-I9100) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2764298 = {S72011, 94, { &P15575,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; GT-I9100G) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2561772 = {S72012, 147, { &P21625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-mobile i-STYLE 6 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2598033 = {S72013, 148, { &P21625,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-mobile i-STYLE 6A Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2597573 = {S72014, 138, { &P21953,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Ctrl_V1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2599851 = {S72015, 145, { &P21953,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GIONEE_Ctrl_V1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2707096 = {S72016, 153, { &P22964,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010X Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2792212 = {S72017, 160, { &P22964,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL_ONE_TOUCH_6010X_Orange Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2947697 = {S72018, 157, { &P15575,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3060741 = {S72019, 157, { &P15575,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9100 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2976697 = {S72020, 142, { &P23683,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Prism II Build/HuaweiU8686) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3034050 = {S72021, 154, { &P17475,&P5768,&P24679,&P18092 }};	/*Useragent - Nokia6300/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6300) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3125480 = {S72022, 92, { &P17475,&P5768,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6300) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA62169 = {S72023, 139, { &P15223,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Zio Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA63834 = {S72024, 139, { &P15223,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Zio Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA23279 = {S72025, 131, { &P12879,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; GT-P7510 Build/HRI83) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA47609 = {S72026, 135, { &P12879,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; GT-P7510 Build/HONEYCOMB) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA369558 = {S72027, 120, { &P12879,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android 3.0.1; zh-tw; GT-P7510 Build/HRI83) AppleWebKit/534.13(KHTML,?? Gecko)?/4.0 ???Safari/534.13*/
const Device UA2607570 = {S72028, 60, { &P12879,&P6545,&P4755,&P18092 }};	/*Useragent - Dalvik/1.5.0 (Linux; U; Android 3.0.1; GT-P7510 Build/HRI83)*/
const Device UA185274 = {S72029, 140, { &P15364,&P7133,&P22401,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Comodo_Dragon/9.0.0.3 Chrome/9.0.597.98 Safari/534.13*/
const Device UA829368 = {S72030, 155, { &P15575,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-I9100 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029721 = {S72031, 163, { &P15575,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; GT-I9100 Build/XB-CM7-DEV-24; CyanogenMod-7.2.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA702449 = {S72032, 70, { &P13974,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.4;Zh_cn;GT-S5670;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA2029998 = {S72033, 70, { &P13974,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;GT-S5670;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA2012820 = {S72034, 147, { &P15699,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Droid Incredible Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2174592 = {S72035, 147, { &P15699,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Droid Incredible Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258074 = {S72036, 147, { &P15699,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Droid Incredible Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2760107 = {S72037, 147, { &P15699,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Droid Incredible Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2676747 = {S72038, 130, { &P15985,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735574 = {S72039, 131, { &P15985,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200L Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676233 = {S72040, 136, { &P18240,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P700 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2753872 = {S72041, 145, { &P18476,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I577 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2974080 = {S72042, 135, { &P20119,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_S9714 Build/JRO03R) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3033649 = {S72043, 156, { &P21628,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8552 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3083343 = {S72044, 136, { &P13074,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P930 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA53832 = {S72045, 146, { &P14042,&P5448,&P17183,&P18092 }};	/*Useragent - SonyEricssonX2i/R3AA Browser/Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA64362 = {S72046, 145, { &P14042,&P5448,&P17183,&P18092 }};	/*Useragent - SonyEricssonX2/R3AA Browser/Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA49805 = {S72047, 183, { &P13088,&P17486,&P18373,&P18092 }};	/*Useragent - MOT-EX223 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA395167 = {S72048, 167, { &P13088,&P17486,&P18373,&P18092 }};	/*Useragent - MOT-EX223 MIDP-2.0/CLDC-1.1 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U) Presto/2.5.28 Version/10.10*/
const Device UA2650643 = {S72049, 183, { &P13088,&P17486,&P18373,&P18092 }};	/*Useragent - MOT-EX223 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U; es-SA) Presto/2.5.28 Version/10.10*/
const Device UA3117782 = {S72050, 183, { &P13088,&P17486,&P18373,&P18092 }};	/*Useragent - MOT-EX223 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U; pt-BZ) Presto/2.5.28 Version/10.10*/
const Device UA63687 = {S72051, 123, { &P16024,&P17483,&P3459,&P18092 }};	/*Useragent - LG-T310i/V100 Obigo/Q7.3 MMS/LG-MMS-V1.1/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA830118 = {S72052, 94, { &P18563,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Motorola_MC659B_312; Windows Phone 6.5.3.5)*/
const Device UA925710 = {S72053, 94, { &P18563,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Motorola_MC659B_301; Windows Phone 6.5.3.5)*/
const Device UA925759 = {S72054, 96, { &P18563,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Motorola_MC659B_21103; Windows Phone 6.5.3.5)*/
const Device UA925789 = {S72055, 96, { &P18563,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Motorola_MC659B_40211; Windows Phone 6.5.3.5)*/
const Device UA925842 = {S72056, 93, { &P18563,&P5222,&P17183,&P18092 }};	/*Useragent - Motorola_MC659B/200 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5.3.5)*/
const Device UA1959503 = {S72057, 138, { &P21293,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; Fly IQ245 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1992538 = {S72058, 159, { &P15877,&P18066,&P18062,&P18091 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID SPYDER Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028127 = {S72059, 143, { &P15877,&P18066,&P18062,&P18091 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID SPYDER Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029621 = {S72060, 143, { &P15877,&P18066,&P18062,&P18091 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID SPYDER Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2832067 = {S72061, 162, { &P15877,&P18066,&P18062,&P18091 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID SPYDER 4G Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1993816 = {S72062, 151, { &P18298,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT928 Build/6.7.2_GC-158-DNRCG-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032971 = {S72063, 142, { &P18298,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT928 Build/6.7.2_GC-158-DNRCG-10) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2227312 = {S72064, 150, { &P18298,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT928 Build/6.7.2_GC-203-DNRCG-8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2230180 = {S72065, 150, { &P18298,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT928 Build/6.7.2_GC-203-DNRCG-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2420046 = {S72066, 141, { &P18298,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; XT928 Build/6.7.2_GC-203-DNRCG-9) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2415675 = {S72067, 127, { &P20518,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A10 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2424595 = {S72068, 133, { &P20518,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A10 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2444013 = {S72069, 127, { &P20518,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A10 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2497770 = {S72070, 137, { &P20518,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A10 MID Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2459054 = {S72071, 139, { &P21132,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Andi4.5h Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2534666 = {S72072, 126, { &P23814,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U2 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2675855 = {S72073, 137, { &P22930,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D2-727 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2991886 = {S72074, 59, { &P22930,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; D2-727 Build/JRO03H)*/
const Device UA2974657 = {S72075, 137, { &P19865,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A850K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974815 = {S72076, 137, { &P19865,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A850L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020300 = {S72077, 137, { &P19865,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A850S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3061678 = {S72078, 161, { &P15991,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6102 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3094681 = {S72079, 151, { &P18552,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT897 Build/7.7.1Q-6_SPR-125_CSP-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3131305 = {S72080, 138, { &P23771,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA90527 = {S72081, 136, { &P18052,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM005SH Build/S0500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250152 = {S72082, 136, { &P18052,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM005SH Build/S0900) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA334695 = {S72083, 136, { &P18052,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM005SH Build/S0810) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA344881 = {S72084, 136, { &P18052,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM005SH Build/S0610) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA561794 = {S72085, 136, { &P18052,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; SBM005SH Build/S0700) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024265 = {S72086, 60, { &P18052,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SBM005SH Build/S0900)*/
const Device UA438360 = {S72087, 145, { &P17172,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Transformer TF101 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2366386 = {S72088, 88, { &P15364,&P16110,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Sleipnir/2.9.14)*/
const Device UA2384958 = {S72089, 81, { &P15364,&P16110,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; Sleipnir/2.9.14)*/
const Device UA2436311 = {S72090, 88, { &P15364,&P16110,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Sleipnir/2.9.15)*/
const Device UA2529022 = {S72091, 81, { &P15364,&P16110,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; Sleipnir/2.9.15)*/
const Device UA2715777 = {S72092, 88, { &P15364,&P16110,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Sleipnir/2.9.16)*/
const Device UA2896668 = {S72093, 88, { &P15364,&P16110,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Sleipnir/2.9.17)*/
const Device UA3027961 = {S72094, 88, { &P15364,&P16110,&P18350,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Sleipnir/2.9.18)*/
const Device UA2495470 = {S72095, 93, { &P15364,&P17285,&P21956,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0 x64; en-US; rv:1.9.1b2pre) Gecko/X Minefield/3.1b2pre*/
const Device UA2516880 = {S72096, 89, { &P15364,&P17285,&P21956,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.1b2pre) Gecko/X Minefield/3.1b2pre*/
const Device UA2420615 = {S72097, 138, { &P18637,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9260 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2503335 = {S72098, 138, { &P18637,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9260 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2423754 = {S72099, 153, { &P23725,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; O+_8.5_Android Build/OplusOplus_A51) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2434306 = {S72100, 138, { &P12814,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-I777 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2537052 = {S72101, 137, { &P12814,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I777 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2462487 = {S72102, 153, { &P20749,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033X Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2485363 = {S72103, 155, { &P12841,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Vivid 4G Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567249 = {S72104, 139, { &P13109,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC-X710a Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567738 = {S72105, 34, { &P12280,&P17137,&P17470,&P18092 }};	/*Useragent - Tweetcaster for iPhone 3.1.0.13306*/
const Device UA2570336 = {S72106, 143, { &P23850,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Q7A Build/ICS_IMM76D_B206) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768452 = {S72107, 143, { &P19746,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; BLU_VIVO 4.3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2898059 = {S72108, 93, { &P15991,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6102) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA9431 = {S72109, 137, { &P11978,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; HTC_Touch2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20638 = {S72110, 145, { &P11978,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Mega/1.28.621.4 (66092) Mozilla/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA20849 = {S72111, 135, { &P11978,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 240x320; HTC_Mega-T3333-Orange; OpVer 113.128.10.702*/
const Device UA32947 = {S72112, 120, { &P11978,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_Touch2/1.29.176.2 (67024)*/
const Device UA53433 = {S72113, 134, { &P11978,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 240x320; HTC_Mega-T3333-Orange; OpVer 113.128.2.733*/
const Device UA54118 = {S72114, 95, { &P11978,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Touch3G_T3333 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA64859 = {S72115, 94, { &P11978,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Touch2_T3333 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA696639 = {S72116, 100, { &P14635,&P5843,&P17347,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (04.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac3-00/UC Browser8.0.3.107/69/444*/
const Device UA358983 = {S72117, 138, { &P21409,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SYNET7LP-DS Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA394673 = {S72118, 133, { &P21409,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; p7901a Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA531884 = {S72119, 134, { &P21409,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; p7901ui Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022337 = {S72120, 62, { &P21409,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; SYNET7LP-DS Build/FRF85B)*/
const Device UA436040 = {S72121, 118, { &P17595,&P18120,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.86 Safari/533.4*/
const Device UA1995271 = {S72122, 92, { &P15364,&P17285,&P22361,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:12.3) Gecko/X Firefox/12.3-x64 PaleMoon/12.3-x64*/
const Device UA2003831 = {S72123, 76, { &P15364,&P17285,&P22361,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:12.3) Gecko/X Firefox/12.3r2 PaleMoon/12.3r2*/
const Device UA1989073 = {S72124, 89, { &P20084,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-S5229-Vodafone/S5229BULC3 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2200086 = {S72125, 138, { &P14872,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SPH-D700 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2238298 = {S72126, 138, { &P14872,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D700 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2351551 = {S72127, 125, { &P12137,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5620) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/405/UCWEB Mobile*/
const Device UA2355161 = {S72128, 146, { &P21207,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Andi3n Build/iBalliBall_Andi3n) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2421044 = {S72129, 133, { &P20145,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; NS-13T001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2654355 = {S72130, 62, { &P20145,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; NS-13T001 Build/IML74K)*/
const Device UA2423375 = {S72131, 143, { &P23891,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TP10.1-1500DC-metal Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2563213 = {S72132, 70, { &P15767,&P18117,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPad1,1; iPhone OS 5.1.1; en_US; en) AppleWebKit/420+*/
const Device UA2483084 = {S72133, 130, { &P21443,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MOMO12 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2536355 = {S72134, 131, { &P23509,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SC-72JB Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2676330 = {S72135, 130, { &P19874,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-i705 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2701228 = {S72136, 93, { &P14454,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae66-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2718555 = {S72137, 92, { &P14454,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE66) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA3032168 = {S72138, 140, { &P21602,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5302 Build/12.0.A.2.245) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3093274 = {S72139, 133, { &P21602,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5302 Build/12.0.A.2.245) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2753837 = {S72140, 162, { &P16356,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-N7000 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2957618 = {S72141, 162, { &P16356,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; GT-N7000 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453*/
const Device UA3031047 = {S72142, 164, { &P22582,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2975697 = {S72143, 138, { &P14225,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Desire HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2970320 = {S72144, 157, { &P12972,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3059374 = {S72145, 157, { &P12972,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3060065 = {S72146, 158, { &P12972,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA32945 = {S72147, 135, { &P12801,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PG41200 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32998 = {S72148, 135, { &P12801,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PG41200 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA561304 = {S72149, 59, { &P12801,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; PG41200 Build/GRJ22)*/
const Device UA190761 = {S72150, 123, { &P15364,&P5690,&P22403,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko) Comodo_Dragon/11.4.0.0 Chrome/11.0.696.71 Safari/534.24*/
const Device UA2138097 = {S72151, 130, { &P15364,&P5690,&P22403,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Comodo_Dragon/11.4.0.0 Chrome/11.0.696.71 Safari/534.24*/
const Device UA505642 = {S72152, 87, { &P15364,&P5690,&P18197,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.1.16) Gecko/X Thunderbird/3.0.11*/
const Device UA2758691 = {S72153, 107, { &P15364,&P5690,&P18197,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; es-ES; rv:1.9.1.7) Gecko/X Thunderbird/3.0.1 ThunderBrowse/3.2.8.1*/
const Device UA1886793 = {S72154, 97, { &P15364,&P5690,&P22488,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.11) Gecko/X Firefox/2.0.0.11 Flock/1.0.5*/
const Device UA720336 = {S72155, 71, { &P15364,&P18118,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Ubuntu 9.10; ru-RU; rv:7.0.1) Gecko/X Firefox/7.0.1*/
const Device UA765138 = {S72156, 142, { &P20182,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-uk; X900 Build/AZ510_GB_01.12I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995922 = {S72157, 142, { &P20182,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-uk; X900 Build/AZ510_GB_01.23I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033345 = {S72158, 142, { &P20182,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-uk; X900 Build/AZ510_GB_01.09I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2659092 = {S72159, 166, { &P18296,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC One V Build/IML74K) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2428397 = {S72160, 33, { &P14669,&P5843,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (NokiaX2-00/08.25)*/
const Device UA2773103 = {S72161, 160, { &P18223,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2719737 = {S72162, 136, { &P22653,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; HTL22 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3006372 = {S72163, 58, { &P22653,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTL22 Build/JZO54K)*/
const Device UA2769174 = {S72164, 138, { &P20124,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782677 = {S72165, 137, { &P20124,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782712 = {S72166, 139, { &P20124,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Grand X(M) Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799086 = {S72167, 140, { &P20124,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ZTE Grand X Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868578 = {S72168, 140, { &P20124,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Grand X Pro Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2942531 = {S72169, 136, { &P20124,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Grand X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2975046 = {S72170, 144, { &P20007,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT886 Build/6.7.3-179_YIC-10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3082936 = {S72171, 134, { &P23179,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ZP980 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068397 = {S72172, 137, { &P22600,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300K Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA813285 = {S72173, 137, { &P14861,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-KU3700 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019583 = {S72174, 61, { &P14861,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; LG-KU3700 Build/GRJ22)*/
const Device UA64076 = {S72175, 105, { &P12340,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonF100i/R1HA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.5*/
const Device UA898962 = {S72176, 85, { &P14010,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s5263) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2373068 = {S72177, 85, { &P14010,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5263) UCBrowser8.3.1.161/69/405/UCWEB Mobile*/
const Device UA1956057 = {S72178, 132, { &P20734,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; W626 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026025 = {S72179, 128, { &P20734,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Philips W626 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA3117375 = {S72180, 140, { &P20734,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Philips W626 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033097 = {S72181, 138, { &P12482,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-T959 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261679 = {S72182, 138, { &P12482,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T959 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261518 = {S72183, 137, { &P18470,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 101G10 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2383196 = {S72184, 178, { &P18221,&P5843,&P24672,&P18092 }};	/*Useragent - Nokia302/5.0 (14.53) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia302) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2507994 = {S72185, 151, { &P20841,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 10 Build/1.1.11-1015 X-10:30) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2662883 = {S72186, 154, { &P20841,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 10 3G Build/1.1.11-1015 X-16:17) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2469088 = {S72187, 128, { &P18008,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ST10 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3044622 = {S72188, 57, { &P18008,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ST10 Build/JRO03C)*/
const Device UA3105409 = {S72189, 139, { &P18008,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; DTC GT10 Build/Bullet) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2562020 = {S72190, 138, { &P19861,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I605 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3011205 = {S72191, 139, { &P19861,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I605 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049769 = {S72192, 61, { &P19861,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SCH-I605 Build/JDQ39E)*/
const Device UA2675033 = {S72193, 154, { &P21857,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3120568 = {S72194, 147, { &P21857,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2772751 = {S72195, 165, { &P18114,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Transformer Prime TF201 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2975000 = {S72196, 138, { &P17691,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I957 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022151 = {S72197, 131, { &P17691,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957R Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022823 = {S72198, 131, { &P17691,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023116 = {S72199, 131, { &P17691,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974310 = {S72200, 152, { &P22962,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2964834 = {S72201, 140, { &P24108,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; W8 beyond Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3023312 = {S72202, 93, { &P18672,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5302) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3080548 = {S72203, 135, { &P23855,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; WM8850:X6-10V Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080903 = {S72204, 139, { &P23855,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; WM8850-mid Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094687 = {S72205, 132, { &P23855,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; WM8850-mid Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA10256 = {S72206, 156, { &P13488,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5; SonyEricssonM1i/R1BA; Profile/MIDP-2.1; Configuration/CLDC-1.1; Windows Phone 6.5.3.5)*/
const Device UA63580 = {S72207, 61, { &P11770,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia7260/2.0 (05.90) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA42064 = {S72208, 141, { &P14007,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; Racer Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA57266 = {S72209, 146, { &P14007,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; ZTE-U X850 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA119661 = {S72210, 135, { &P13668,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; HTC Dream Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA142447 = {S72211, 137, { &P13668,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Dream Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018164 = {S72212, 61, { &P13668,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; HTC Dream Build/FRG83)*/
const Device UA299078 = {S72213, 141, { &P15364,&P5690,&P22400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Comodo_Dragon/8.0.0.4 Chrome/8.0.552.215 Safari/534.10*/
const Device UA330648 = {S72214, 12, { &P12193,&P5129,&P17470,&P18092 }};	/*Useragent - NokiaC5-00.2*/
const Device UA2751871 = {S72215, 106, { &P12193,&P5129,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaC5-00*/
const Device UA2683811 = {S72216, 142, { &P14669,&P5843,&P24667,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiax2-00) UCBrowser8.2.0.132/69/452/UCWEB Mobile*/
const Device UA612060 = {S72217, 135, { &P17595,&P17296,&P22457,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19*/
const Device UA2226837 = {S72218, 125, { &P15547,&P17484,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-c6712) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2466284 = {S72219, 67, { &P15989,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; ST21i2; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2483006 = {S72220, 66, { &P15989,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; ST21i; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2424591 = {S72221, 57, { &P20057,&P22784,&P21795,&P18092 }};	/*Useragent - MicromaxX324/MstarBrowser1.0/MIDP2.0Congiguration/CLDC1.1*/
const Device UA2958453 = {S72222, 61, { &P20057,&P22784,&P21795,&P18092 }};	/*Useragent - Micromax X324i/MstarBrowser1.0/MIDP-2.0 Configuration/CLDC1.1*/
const Device UA2506205 = {S72223, 26, { &P11860,&P5776,&P21720,&P18092 }};	/*Useragent - iBrowser/2.0,Nokia6681/2.0*/
const Device UA2511286 = {S72224, 93, { &P18513,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6802) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2555436 = {S72225, 101, { &P11724,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.68; en-us; 8310/2.7.0.68) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2654408 = {S72226, 103, { &P11724,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.106; en-us; 8310/2.7.0.106) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2695881 = {S72227, 103, { &P11724,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.105; en-us; 8310/2.7.0.105) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2777707 = {S72228, 101, { &P11724,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.90; en-us; 8310/2.7.0.90) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2982883 = {S72229, 103, { &P11724,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.109; en-us; 8310/2.7.0.109) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3070391 = {S72230, 101, { &P11724,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.92; en-us; 8310/2.7.0.92) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2556770 = {S72231, 71, { &P21745,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X295/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2959642 = {S72232, 72, { &P21745,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X295i/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2566448 = {S72233, 134, { &P17172,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Transformer Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2601772 = {S72234, 140, { &P17172,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Transformer TF101 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567139 = {S72235, 126, { &P15869,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Xoom Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667356 = {S72236, 127, { &P15869,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Xoom Build/JB_MR1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2664512 = {S72237, 162, { &P18442,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2753271 = {S72238, 138, { &P17691,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I957 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2783306 = {S72239, 131, { &P17691,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957R Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2876517 = {S72240, 131, { &P17691,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2893377 = {S72241, 131, { &P17691,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2974075 = {S72242, 138, { &P19844,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80 COBALT Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2961656 = {S72243, 152, { &P12989,&P5843,&P24678,&P18092 }};	/*Useragent - Nokia303/5.0 (14.87) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia303) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA54784 = {S72244, 108, { &P13439,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2130/E2130DDIL1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA90083 = {S72245, 52, { &P13110,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-WX400/00.72 UP.Browser/7.2.7.5.708 (GUI) MMP/2.0*/
const Device UA924024 = {S72246, 133, { &P17595,&P18207,&P22458,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5*/
const Device UA1999491 = {S72247, 133, { &P17595,&P18207,&P22458,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5*/
const Device UA2195935 = {S72248, 133, { &P17595,&P18207,&P22458,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5*/
const Device UA2669343 = {S72249, 133, { &P17595,&P18207,&P22458,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5*/
const Device UA3003071 = {S72250, 133, { &P17595,&P18207,&P22458,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5*/
const Device UA1959354 = {S72251, 85, { &P15547,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-c6712) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA1960728 = {S72252, 79, { &P12004,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN95)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA1994039 = {S72253, 83, { &P12004,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN95 8GB)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2828825 = {S72254, 118, { &P12004,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; fa; nokian95_8gb) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA1993347 = {S72255, 139, { &P17595,&P17296,&P24518,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11*/
const Device UA2652980 = {S72256, 139, { &P17595,&P17296,&P24518,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.5.0.2311 Safari/535.11*/
const Device UA2030274 = {S72257, 133, { &P22362,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; it-it; SmartTab10 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2372670 = {S72258, 132, { &P22362,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; es-es; SmartTab10 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2979982 = {S72259, 62, { &P22362,&P6437,&P16179,&P18092 }};	/*Useragent - Dalvik/1.5.1 (Linux; U; Android 3.2.1; SmartTab10 Build/HTK75)*/
const Device UA2384582 = {S72260, 69, { &P18513,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GT-S6802; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2385173 = {S72261, 135, { &P23193,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCL21 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2677279 = {S72262, 136, { &P19861,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I605 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735570 = {S72263, 129, { &P19861,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I605 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2714609 = {S72264, 152, { &P18219,&P5843,&P24677,&P18092 }};	/*Useragent - Nokia202/2.0 (20.52) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia202) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2969895 = {S72265, 93, { &P13706,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5235) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA2938302 = {S72266, 140, { &P24488,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; tr-tr; Discovery Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3106077 = {S72267, 62, { &P24488,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; Discovery Build/JOP40D)*/
const Device UA180373 = {S72268, 76, { &P15364,&P18110,&P17829,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/4.6; Linux) KHTML/4.6.0 (like Gecko) SUSE*/
const Device UA396930 = {S72269, 125, { &P17595,&P17297,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_2; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA667384 = {S72270, 125, { &P17595,&P17297,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_3; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.41 Safari/534.7*/
const Device UA895981 = {S72271, 125, { &P17595,&P17297,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA2349843 = {S72272, 125, { &P17595,&P17297,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA2819902 = {S72273, 124, { &P17595,&P17297,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_0; en-US) AppleWebKit/534.6 (KHTML, like Gecko) Chrome/7.0.500.0 Safari/534.6*/
const Device UA2141323 = {S72274, 93, { &P17595,&P17295,&P19773,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.1b3pre) Gecko/X SeaMonkey/2.0a2*/
const Device UA670230 = {S72275, 137, { &P23898,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-gb; ELONEX765ET Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA783312 = {S72276, 60, { &P15364,&P18146,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; SunOS i86pc; rv:11.0) Gecko/X Firefox/11.0*/
const Device UA2199025 = {S72277, 139, { &P21075,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Q-Smart S1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2258057 = {S72278, 148, { &P12889,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7300 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2420916 = {S72279, 148, { &P12889,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7300 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2486323 = {S72280, 87, { &P17595,&P18207,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.2.8415(Macintosh;Mac OS X (10.8.2);en;kml:2.2;client:Free;type:default)*/
const Device UA2601969 = {S72281, 87, { &P17595,&P18207,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Macintosh;Mac OS X (10.8.2);en;kml:2.2;client:Free;type:default)*/
const Device UA2669126 = {S72282, 87, { &P17595,&P18207,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Macintosh;Mac OS X (10.8.3);en;kml:2.2;client:Free;type:default)*/
const Device UA2780680 = {S72283, 87, { &P17595,&P18207,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Macintosh;Mac OS X (10.8.3);es;kml:2.2;client:Free;type:default)*/
const Device UA2859488 = {S72284, 87, { &P17595,&P18207,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Macintosh;Mac OS X (10.8.3);de;kml:2.2;client:Free;type:default)*/
const Device UA2876967 = {S72285, 87, { &P17595,&P18207,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Macintosh;Mac OS X (10.8.4);en;kml:2.2;client:Free;type:default)*/
const Device UA2505654 = {S72286, 135, { &P20735,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X403 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2508012 = {S72287, 93, { &P13974,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; GT-S5670) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2567100 = {S72288, 140, { &P16319,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; U8860 Build/HuaweiU8860) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2580570 = {S72289, 132, { &P21415,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; QUANTUM7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2676748 = {S72290, 135, { &P18468,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabA2109A Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3028396 = {S72291, 141, { &P17346,&P17146,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9A405 Safari/7534.48.3*/
const Device UA3078799 = {S72292, 139, { &P17346,&P17146,&P24183,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/28.0.1500.16 Mobile/9A334 Safari/7534.48.3*/
const Device UA3000715 = {S72293, 128, { &P24366,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SM-T311 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3083198 = {S72294, 136, { &P22948,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P716 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA573886 = {S72295, 70, { &P15575,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.3;Zh_cn;GT-I9100;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA1916747 = {S72296, 70, { &P15575,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.5;Zh_cn;GT-I9100;480*800;)UCWEB7.8.0.95/139/351*/
const Device UA1939635 = {S72297, 71, { &P15575,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.4;Zh_cn;GT-I9100G;480*800;)UCWEB7.8.0.95/139/351*/
const Device UA2019280 = {S72298, 119, { &P15575,&P5629,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.3.3;Zh_cn;GT-I9100;480*800;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA2027153 = {S72299, 71, { &P15575,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;GT-I9100G;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA2030653 = {S72300, 70, { &P15575,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;GT-I9100;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA1952319 = {S72301, 155, { &P14202,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15i Build/3.0.1.A.0.145; MiniCM7-2.2.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2816269 = {S72302, 147, { &P14202,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E15i Build/GRH55; CyanogenMod 7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2718730 = {S72303, 67, { &P15364,&P16110,&P20794,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:7.0.1) Gecko/X Postbox/3.0.8*/
const Device UA2505177 = {S72304, 147, { &P23445,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; Ilium_S200 Build/ALPS.GB2.MP.V1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2730093 = {S72305, 149, { &P23445,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; Ilium_S200 Build/HuaqinIlium_S200) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2558382 = {S72306, 141, { &P21973,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; Aqua flash Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583966 = {S72307, 127, { &P18465,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KFOT Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667435 = {S72308, 135, { &P18465,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KF012 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2862479 = {S72309, 141, { &P18465,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Amazon Kindle Fire Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2708920 = {S72310, 130, { &P17337,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7500 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2751025 = {S72311, 130, { &P17337,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P7500 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2879409 = {S72312, 135, { &P24170,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T1-E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3080328 = {S72313, 136, { &P21022,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E615 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094524 = {S72314, 137, { &P21022,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E615f Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083191 = {S72315, 127, { &P18704,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT200 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA427394 = {S72316, 136, { &P17595,&P17296,&P22455,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11*/
const Device UA850932 = {S72317, 74, { &P17595,&P17295,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:6.0.2) Gecko/X Firefox/6.0.2*/
const Device UA661413 = {S72318, 135, { &P21108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; DM012SH Build/S0004) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA899145 = {S72319, 135, { &P21108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; DM012SH Build/S0008) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033247 = {S72320, 59, { &P21108,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; DM012SH Build/S0008)*/
const Device UA2961340 = {S72321, 135, { &P21108,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; DM012SH Build/S0010) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA761839 = {S72322, 126, { &P15364,&P17350,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en_US) AppleWebKit/533.19.4 (KHTML, like Gecko) Dreamweaver/12.0.0.5808 Version/5.0.3 Safari/533.19.4*/
const Device UA2197803 = {S72323, 140, { &P15517,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT15i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2234947 = {S72324, 143, { &P21341,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Spice Mi-500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2237122 = {S72325, 157, { &P15991,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2480224 = {S72326, 155, { &P15991,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2421306 = {S72327, 143, { &P23978,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WALKMAN Build/icx1227_E110) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2427265 = {S72328, 136, { &P20015,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SC-03E Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2582841 = {S72329, 150, { &P18461,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Grand X In Build/GXI_TPLV1.0.0B04) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2600710 = {S72330, 144, { &P21733,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; i-mobile IQ 3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2853666 = {S72331, 66, { &P21733,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; i-mobile IQ 3 Build/JRO03C)*/
const Device UA2676601 = {S72332, 135, { &P22126,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Torque Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2975948 = {S72333, 135, { &P20015,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SC-03E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3030816 = {S72334, 159, { &P18447,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.1.141) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2975724 = {S72335, 139, { &P12798,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT18i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023242 = {S72336, 139, { &P12798,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LT18i Build/4.1.A.0.562) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3028656 = {S72337, 161, { &P19862,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax A110 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3083289 = {S72338, 142, { &P21020,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Micromax A116 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3113800 = {S72339, 133, { &P21020,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; A116 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA8626 = {S72340, 157, { &P12247,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3 U; [en]; Series60/3.2 Nokia6650d-1bH/1.00 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA63682 = {S72341, 160, { &P12247,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia6650d-1bh/ATT.3.26 Mozilla/5.0 SymbianOS/9.3; U; [en]; Series60/3.2; Profile/MIDP-2.1 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA64005 = {S72342, 153, { &P12247,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia6650d-1c/03.09; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA13244 = {S72343, 49, { &P14784,&P18110,&P17470,&P18092 }};	/*Useragent - QQBrowser (Linux; U; zh-cn; HTC Hero Build/FRF91)*/
const Device UA744486 = {S72344, 142, { &P17962,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; IM-A720L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA799095 = {S72345, 81, { &P15364,&P17296,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Intel Mac OS X 10.6; rv:7.0.1) Gecko/X Firefox/7.0.1*/
const Device UA2259277 = {S72346, 82, { &P15364,&P16110,&P20761,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:16.0) Gecko/X Firefox/16.0 SeaMonkey/2.13.2*/
const Device UA2402194 = {S72347, 64, { &P18666,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; A25; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2428536 = {S72348, 128, { &P21933,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M729 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2847536 = {S72349, 57, { &P21933,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; M729 Build/IML74K)*/
const Device UA2489424 = {S72350, 155, { &P18465,&P18066,&P19838,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2533406 = {S72351, 91, { &P12029,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAN82) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2580813 = {S72352, 143, { &P20491,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Thunder S220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2675597 = {S72353, 46, { &P15364,&P22277,&P17470,&P18092 }};	/*Useragent - Guzzle/3.3.1 curl/7.22.0 PHP/5.3.10-1ubuntu3.6*/
const Device UA2676089 = {S72354, 133, { &P17482,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80G9 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708972 = {S72355, 130, { &P17482,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 80G9 Build/MR1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2679927 = {S72356, 142, { &P15438,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Sensation Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2782668 = {S72357, 160, { &P18382,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire C Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2712382 = {S72358, 161, { &P19862,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax A110 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2753452 = {S72359, 130, { &P12889,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2892364 = {S72360, 142, { &P23321,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ARCHOS FAMILYPAD 2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2782603 = {S72361, 154, { &P23175,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ALCATEL ONE TOUCH 5021E Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA3036133 = {S72362, 140, { &P17968,&P18066,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP Mobile*/
const Device UA2869723 = {S72363, 137, { &P21261,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9500 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3049654 = {S72364, 139, { &P21261,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-i9500 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3074027 = {S72365, 139, { &P21261,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA5015 = {S72366, 128, { &P12530,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC_P3450/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA11508 = {S72367, 128, { &P12530,&P5448,&P22271,&P18092 }};	/*Useragent - PPC; 240x320; HTC_P3450/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA19949 = {S72368, 121, { &P12530,&P5448,&P22271,&P18092 }};	/*Useragent - HTC P3450/2.20.621.1 Mozilla/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA40311 = {S72369, 95, { &P12530,&P5448,&P22271,&P18092 }};	/*Useragent - HTC-P3450-Ten/PPC; 240x320; OpVer 23.227.2.792 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA55528 = {S72370, 104, { &P12530,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC P3450; OpVer 23.221.1.611*/
const Device UA86653 = {S72371, 98, { &P12530,&P5448,&P22271,&P18092 }};	/*Useragent - HTC-P3450-orange/PPC; 240x320; OpVer 23.221.2.741 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA86988 = {S72372, 70, { &P12530,&P5448,&P22271,&P18092 }};	/*Useragent - HTC-Touch/Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA87373 = {S72373, 110, { &P12530,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC P3450 White; OpVer 23.227.1.801*/
const Device UA87845 = {S72374, 105, { &P12530,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC P3450; OpVer 23.221.15.751*/
const Device UA1864686 = {S72375, 78, { &P15364,&P5690,&P21925,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) chromeframe/9.0.597.98*/
const Device UA2130985 = {S72376, 141, { &P15364,&P5690,&P21925,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13 chromeframe/9.0.597.98*/
const Device UA3119161 = {S72377, 201, { &P15364,&P5690,&P21925,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; InfoPath.3) chromeframe/9.0.597.98*/
const Device UA379719 = {S72378, 84, { &P20368,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-T50 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1 firmware/T50 T03*/
const Device UA791178 = {S72379, 200, { &P15364,&P17285,&P21928,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E) chromeframe/6.0.472.63*/
const Device UA2014857 = {S72380, 138, { &P14183,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I997 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2234193 = {S72381, 138, { &P14183,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I997 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2342857 = {S72382, 142, { &P13107,&P5043,&P24668,&P18092 }};	/*Useragent - NokiaC2-06/2.0 (07.29) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-06) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2562064 = {S72383, 133, { &P21887,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; TAD-97052 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2567386 = {S72384, 138, { &P14877,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9000 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2706198 = {S72385, 138, { &P14877,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9000 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567960 = {S72386, 146, { &P16263,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI MediaPad Build/HuaweiMediaPad) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2584157 = {S72387, 142, { &P16263,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SpringBoard Build/HuaweiMediaPad) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2732907 = {S72388, 146, { &P16263,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MediaPad 7 Lite Build/HuaweiMediaPad) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2670732 = {S72389, 149, { &P18318,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ASUS Transformer Pad TF700T Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2897685 = {S72390, 114, { &P14635,&P5843,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-00/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA2998908 = {S72391, 138, { &P21395,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Lenovo A3000-H Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3060875 = {S72392, 137, { &P21395,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Lenovo A3000-H Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3125348 = {S72393, 66, { &P21395,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; Lenovo A3000-H Build/JDQ39)*/
const Device UA3083599 = {S72394, 133, { &P24559,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Z992 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093819 = {S72395, 152, { &P20129,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3094234 = {S72396, 153, { &P20129,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2S Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3094447 = {S72397, 153, { &P20129,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2A Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3095062 = {S72398, 154, { &P20129,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2SC Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3114925 = {S72399, 153, { &P20129,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2C Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3131467 = {S72400, 136, { &P23521,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA31288 = {S72401, 61, { &P12414,&P5843,&P4030,&P18092 }};	/*Useragent - Nokia7020/2.0 (08.10) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA64773 = {S72402, 41, { &P14703,&P17483,&P3625,&P18092 }};	/*Useragent - LG-AN430 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA250142 = {S72403, 122, { &P15364,&P5690,&P22410,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.0.1.0 Chrome/14.0.835.163 Safari/535.1*/
const Device UA792930 = {S72404, 76, { &P15364,&P18146,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; SunOS sun4u; en-US; rv:1.8.0.1) Gecko/X Firefox/1.5.0.1*/
const Device UA647619 = {S72405, 119, { &P17595,&P17297,&P22504,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en; rv:1.9.0.19) Gecko/X Camino/2.0.7 (MultiLang) (like Firefox/3.0.19)*/
const Device UA624543 = {S72406, 144, { &P20861,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; Scroll Essential Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034250 = {S72407, 144, { &P20861,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Scroll Essential Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257321 = {S72408, 147, { &P16336,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U8800Pro Build/HuaweiU8800Pro) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2537449 = {S72409, 93, { &P18283,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9300) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3118642 = {S72410, 70, { &P18283,&P18111,&P24667,&P18092 }};	/*Useragent - JUC (Linux; U; 4.1.9; zh-cn; GT-I9300; 480*800) UCWEB8.2.2.135/145/800*/
const Device UA2675064 = {S72411, 142, { &P21020,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Micromax A116 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2751335 = {S72412, 135, { &P21020,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Micromax A116 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676680 = {S72413, 130, { &P17281,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I905 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2864885 = {S72414, 143, { &P23408,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ARCHOS 97B TITANIUM Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2945998 = {S72415, 125, { &P17595,&P18207,&P24729,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) FluidApp Version/1842 Safari/536.30.1*/
const Device UA3020855 = {S72416, 135, { &P23855,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; WM8850:X6-10V Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021779 = {S72417, 139, { &P23855,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; WM8850-mid Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022995 = {S72418, 132, { &P23855,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; WM8850-mid Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2597728 = {S72419, 71, { &P15364,&P18110,&P18191,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:10.0.12) Gecko/X Thunderbird/10.0.12*/
const Device UA1972674 = {S72420, 80, { &P15373,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5530)/UC Browser8.5.0.183/50/352/UCWEB Mobile*/
const Device UA1992283 = {S72421, 31, { &P15373,&P5129,&P24671,&P18092 }};	/*Useragent - Nokia5530/UCWEB8.5.0.163/50/999*/
const Device UA2231974 = {S72422, 136, { &P22762,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; L-01D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2466156 = {S72423, 135, { &P22522,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; NetTAB THOR Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2658968 = {S72424, 140, { &P22522,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; NetTAB THOR mini Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2510700 = {S72425, 138, { &P21355,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; KM-E100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2599612 = {S72426, 114, { &P14848,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-00/11.00; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2533209 = {S72427, 141, { &P21736,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LAVA iris 351 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2607510 = {S72428, 65, { &P21736,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; LAVA iris 351 Build/GRJ90)*/
const Device UA2667170 = {S72429, 165, { &P20183,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI-ONE Plus Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2991241 = {S72430, 160, { &P20183,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI-ONE Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2714892 = {S72431, 156, { &P16356,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-N7000 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2737774 = {S72432, 156, { &P16356,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-N7000 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2754412 = {S72433, 138, { &P18641,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-M830 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2892630 = {S72434, 140, { &P24407,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; KYY21 Build/100.0.2480) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3059790 = {S72435, 140, { &P24407,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; KYY21 Build/205.0.0d00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3105061 = {S72436, 62, { &P24407,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; KYY21 Build/100.0.2480)*/
const Device UA2941633 = {S72437, 160, { &P12473,&P5204,&P24678,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2730c-1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2975696 = {S72438, 139, { &P20289,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC6435LVW Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3077909 = {S72439, 137, { &P20289,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; HTC6435LVW Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3031580 = {S72440, 156, { &P18713,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P768 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3033321 = {S72441, 156, { &P18713,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3059196 = {S72442, 156, { &P18713,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3022148 = {S72443, 137, { &P23293,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LS720 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA39872 = {S72444, 137, { &P14467,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; SPH-M920 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA55138 = {S72445, 145, { &P14467,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SPH-M920 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA24500 = {S72446, 41, { &P15697,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-R430 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA67481 = {S72447, 135, { &P12955,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; ZTE Blade Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1880484 = {S72448, 131, { &P12955,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Blade Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018426 = {S72449, 59, { &P12955,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; ZTE Blade Build/FRF91)*/
const Device UA38520 = {S72450, 88, { &P13686,&P17603,&P4223,&P18092 }};	/*Useragent - PantechP9050/JJUS09132010 NetFront/3.5 OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA497379 = {S72451, 100, { &P13686,&P17603,&P4223,&P18092 }};	/*Useragent - PantechP9050/JJUSX NetFront/3.5 OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1 Novarra-Vision/8.0*/
const Device UA91852 = {S72452, 113, { &P15364,&P5690,&P22440,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/530.1 (KHTML, like Gecko) Iron/2.0.168.0 Safari/530.1*/
const Device UA97673 = {S72453, 113, { &P15364,&P5690,&P22440,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/530.9 (KHTML, like Gecko) Iron/2.0.178.0 Safari/530.9*/
const Device UA624573 = {S72454, 82, { &P17595,&P17296,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en) Presto/2.10.229 Version/11.62*/
const Device UA1885755 = {S72455, 86, { &P17968,&P5629,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.4; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.7.81 Version/11.00*/
const Device UA1907180 = {S72456, 89, { &P17968,&P5629,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.6; Linux; Opera Mobi/ADR-X; U; en-GB) Presto/2.7.81 Version/11.00*/
const Device UA1935893 = {S72457, 86, { &P17968,&P5629,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.5; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.7.81 Version/11.00*/
const Device UA2018752 = {S72458, 86, { &P17968,&P5629,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.3; Linux; Opera Mobi/ADR-X; U; pl) Presto/2.7.81 Version/11.00*/
const Device UA2020768 = {S72459, 86, { &P17968,&P5629,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.7; Linux; Opera Mobi/ADR-X; U; en) Presto/2.7.81 Version/11.00*/
const Device UA766829 = {S72460, 87, { &P15364,&P18140,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux x86_64; U; Edition Linux Mint; en) Presto/2.10.229 Version/11.64*/
const Device UA2205255 = {S72461, 166, { &P16356,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-N7000 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2257352 = {S72462, 166, { &P16356,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-N7000 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2555327 = {S72463, 107, { &P15364,&P16110,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)*/
const Device UA2727144 = {S72464, 65, { &P15364,&P16110,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.2; SV1; MAXTHON)*/
const Device UA2746642 = {S72465, 73, { &P15364,&P16110,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; MAXTHON)*/
const Device UA2982547 = {S72466, 161, { &P15364,&P16110,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAXTHON)*/
const Device UA2600095 = {S72467, 154, { &P12955,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Blade Build/JRO03C; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2697065 = {S72468, 154, { &P12955,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Blade Build/JRO03C; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2600588 = {S72469, 141, { &P21402,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Lenovo S890 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2754724 = {S72470, 130, { &P17337,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7500 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2869606 = {S72471, 130, { &P17337,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P7500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755330 = {S72472, 133, { &P18737,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; X909 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA53564 = {S72473, 61, { &P14865,&P5776,&P3756,&P18092 }};	/*Useragent - NokiaN90-1/2.0527.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA67735 = {S72474, 8, { &P14865,&P5776,&P3756,&P18092 }};	/*Useragent - NokiaN90*/
const Device UA2580152 = {S72475, 115, { &P14865,&P5776,&P3756,&P18092 }};	/*Useragent - NokiaN90-1/3.0545.5.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1 (en-US; rv:9.3.2) Clecko/X Classilla/CFM*/
const Device UA452500 = {S72476, 114, { &P15228,&P5043,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-03/06.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA2384468 = {S72477, 165, { &P15649,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2230106 = {S72478, 136, { &P21246,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; NGM Quasar Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2725321 = {S72479, 134, { &P21246,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; it-it; Quasar Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2257379 = {S72480, 130, { &P20724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; AN8G2I Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2430214 = {S72481, 88, { &P15364,&P22189,&P19819,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) KHTML/4.9.3 (like Gecko) Konqueror/4.9 Fedora/4.9.3-2.fc17*/
const Device UA2486037 = {S72482, 90, { &P15364,&P22189,&P19819,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) KHTML/4.9.5 (like Gecko) Konqueror/4.9 Fedora/4.9.5-2.fc17*/
const Device UA2465748 = {S72483, 138, { &P21325,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ThL V11 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2833291 = {S72484, 141, { &P21325,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; CJ-ThL V11 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2512725 = {S72485, 175, { &P15798,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3021199 = {S72486, 179, { &P15798,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC_Explorer_A310e Build/GRJ90) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2560451 = {S72487, 151, { &P22659,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cloudfone Thrill 430d Build/SKV02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2701781 = {S72488, 151, { &P22659,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cloudfone Thrill 430d Build/FSV02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2560507 = {S72489, 154, { &P20542,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PAP5000DUO Build/PrestigioPAP5000DUO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2582480 = {S72490, 160, { &P18635,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39; Carbon-Carbon-JB-v1.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2670020 = {S72491, 160, { &P18635,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39; Carbon-CARBON-JB-v1.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752851 = {S72492, 161, { &P18635,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2677158 = {S72493, 138, { &P19849,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7560M Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2754272 = {S72494, 159, { &P18113,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2975943 = {S72495, 152, { &P19771,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035A Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022409 = {S72496, 152, { &P19771,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024144 = {S72497, 152, { &P19771,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3033946 = {S72498, 160, { &P12473,&P5204,&P24679,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2730c-1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA243203 = {S72499, 81, { &P17595,&P17295,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.9.168 Version/11.51*/
const Device UA656229 = {S72500, 66, { &P15364,&P6704,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows 98; SV1; Avant Browser)*/
const Device UA847437 = {S72501, 76, { &P15364,&P6704,&P22730,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Windows 98; Avant Browser; Avant Browser)*/
const Device UA1954295 = {S72502, 74, { &P15364,&P6704,&P22730,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; Avant Browser)*/
const Device UA1961128 = {S72503, 74, { &P15364,&P6704,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; Avant Browser)*/
const Device UA2257875 = {S72504, 79, { &P15364,&P6704,&P22730,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; Avant Browser; SV1)*/
const Device UA2337587 = {S72505, 89, { &P15364,&P6704,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows 98; Avant Browser; Avant Browser)*/
const Device UA615302 = {S72506, 83, { &P15364,&P17350,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 7.0; en-US; rv:1.9.2.11) Gecko/X Firefox/5.0.10*/
const Device UA2260662 = {S72507, 142, { &P16074,&P5768,&P24670,&P18092 }};	/*Useragent - Nokia3110c/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia3110c) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2232110 = {S72508, 36, { &P23144,&P17779,&P22802,&P18092 }};	/*Useragent - Instacast/2.2.4 (like iTunes/10.1.2)*/
const Device UA2163176 = {S72509, 85, { &P15364,&P16110,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/6.0; Avant Browser)*/
const Device UA2260412 = {S72510, 78, { &P15364,&P16110,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; Trident/6.0; Avant Browser)*/
const Device UA2831860 = {S72511, 90, { &P15364,&P16110,&P22732,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; Win64; x64; Trident/6.0; Avant Browser)*/
const Device UA2258147 = {S72512, 91, { &P17595,&P18207,&P20761,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:16.0) Gecko/X Firefox/16.0 SeaMonkey/2.13.2*/
const Device UA2563451 = {S72513, 113, { &P12108,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia7230/10.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA3100203 = {S72514, 116, { &P12108,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia7230-1c/10.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2567341 = {S72515, 138, { &P18316,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583275 = {S72516, 138, { &P18316,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583565 = {S72517, 138, { &P18316,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2666886 = {S72518, 139, { &P18316,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160LV Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567454 = {S72519, 155, { &P20542,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAP5000TDUO Build/PrestigioPAP5000TDUO) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2770059 = {S72520, 161, { &P18214,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S7500 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2737390 = {S72521, 142, { &P23011,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; iris405 Build/LAVAIRIS405) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2781660 = {S72522, 130, { &P15767,&P18117,&P20783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/9B206 Safari/7534.48.3*/
const Device UA2958986 = {S72523, 128, { &P15767,&P18117,&P20783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/9B176 Safari/7534.48.3*/
const Device UA2867211 = {S72524, 142, { &P24648,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; G928 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2974658 = {S72525, 143, { &P19862,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Micromax A110Q Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975539 = {S72526, 136, { &P23697,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9192 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3082981 = {S72527, 137, { &P23215,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SHV-E330S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA1994062 = {S72528, 161, { &P14294,&P5129,&P3690,&P18092 }};	/*Useragent - Nokia5238/51.9.002 (SymbianOS/9.4; U; Series60/5.0 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Safari/525 3gpp-gba*/
const Device UA49531 = {S72529, 84, { &P13051,&P17979,&P3508,&P18092 }};	/*Useragent - ZTE-TU880_TD/1.0 Linux/2.6.32 Android/2.2 Release/5.25.2011 Browser/AppleWebKit533.1*/
const Device UA54595 = {S72530, 73, { &P14242,&P5208,&P3499,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Teleca Q7; Brew 3.1.5; U; en) 240X480 LGE VX9600*/
const Device UA63877 = {S72531, 66, { &P14551,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GW305/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA624778 = {S72532, 43, { &P14748,&P5890,&P24665,&P18092 }};	/*Useragent - NokiaE72-1/UC Browser8.0.4.121/28/444/UCWEB*/
const Device UA2482574 = {S72533, 62, { &P17595,&P18577,&P21570,&P18092 }};	/*Useragent - Mail/1085 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (iMac10%2C1)*/
const Device UA2551799 = {S72534, 67, { &P17595,&P18577,&P21570,&P18092 }};	/*Useragent - Mail/1085 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro7%2C1)*/
const Device UA2834128 = {S72535, 64, { &P17595,&P18577,&P21570,&P18092 }};	/*Useragent - Mail/1085 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBook7%2C1)*/
const Device UA626688 = {S72536, 79, { &P12581,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia500)/UC Browser8.2.0.132/50/355/UCWEB Mobile*/
const Device UA2358167 = {S72537, 44, { &P17595,&P18570,&P18535,&P18092 }};	/*Useragent - Reeder/3.0.5 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2384197 = {S72538, 173, { &P14202,&P5477,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E15i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2842039 = {S72539, 173, { &P14202,&P5477,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E15a Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2502119 = {S72540, 89, { &P12449,&P5890,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiae5-00) UCBrowser8.3.1.161/70/352/UCWEB Mobile  3gpp-gba*/
const Device UA2431080 = {S72541, 166, { &P16356,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-N7000 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2485213 = {S72542, 166, { &P16356,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-N7000 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2426630 = {S72543, 151, { &P20516,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Woxter Tablet PC 97IPS DUAL Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2713846 = {S72544, 145, { &P20516,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Woxter 97 IPS DUAL 3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2843463 = {S72545, 80, { &P20516,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Woxter Tablet PC 97IPS DUAL Build/JRO03H)*/
const Device UA2509945 = {S72546, 140, { &P11751,&P17483,&P24676,&P18092 }};	/*Useragent - LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-kp500 teleca) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2676937 = {S72547, 153, { &P18016,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ThinkPad Tablet Build/ThinkPadTablet_A400_03) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2731416 = {S72548, 132, { &P22918,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DA220HQL Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3029075 = {S72549, 196, { &P22597,&P18598,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I545 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us*/
const Device UA2798686 = {S72550, 136, { &P23150,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Q1000 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2973898 = {S72551, 138, { &P20517,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CINK PEAX Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3068401 = {S72552, 135, { &P20015,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SC-03E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3082161 = {S72553, 140, { &P20285,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ616-12) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3093947 = {S72554, 145, { &P20285,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ616 Build/7.7.1-141-6-PAST-UMTS-LA) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA6156 = {S72555, 127, { &P12206,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-M7600-Vodafone/M7600BUIF2 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA53614 = {S72556, 134, { &P12206,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-M7600H/1.0 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1/*MzU1MTcxMDMwMDUwODY4*/
const Device UA63887 = {S72557, 118, { &P12206,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-M7600/M7600XXIDA SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA63888 = {S72558, 125, { &P12206,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-M7600-ORANGE/M7600BVIF1 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA62866 = {S72559, 61, { &P14457,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia7373/2.0 (05.50) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63652 = {S72560, 122, { &P14532,&P17483,&P3459,&P18092 }};	/*Useragent - LG-T325/V100 Obigo/Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA179830 = {S72561, 135, { &P12453,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Liberty Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250867 = {S72562, 136, { &P12453,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Liberty Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA302882 = {S72563, 135, { &P12453,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; Liberty Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA319792 = {S72564, 135, { &P12453,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Liberty Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA333683 = {S72565, 135, { &P12453,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Liberty Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025608 = {S72566, 60, { &P12453,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; Liberty Build/GRK39F)*/
const Device UA663105 = {S72567, 87, { &P17968,&P5096,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2.1; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.9.201 Version/11.50*/
const Device UA723013 = {S72568, 85, { &P17968,&P5096,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.9.201 Version/11.50*/
const Device UA792241 = {S72569, 87, { &P17968,&P5096,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2.2; Linux; Opera Mobi/ADR-X; U; fr) Presto/2.9.201 Version/11.50*/
const Device UA478851 = {S72570, 99, { &P15364,&P18110,&P20918,&P18092 }};	/*Useragent - Firefox 0.8 (Linux)" useragent="Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.6) Gecko/X Firefox/0.8*/
const Device UA773540 = {S72571, 70, { &P15364,&P18110,&P20918,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; NetBSD i386; en-GB; rv:1.7) Gecko/X Firefox/0.9.1*/
const Device UA2726658 = {S72572, 81, { &P15364,&P18110,&P20918,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; rv:1.7.3) Gecko/X Firefox/0.10.1libwww-perl/6.04*/
const Device UA695708 = {S72573, 123, { &P11860,&P5776,&P24663,&P18092 }};	/*Useragent - Nokia6681/2.0 (6.09.00) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.9.1.120/27/351/UCWEB*/
const Device UA865474 = {S72574, 151, { &P14849,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I800 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2034045 = {S72575, 147, { &P14849,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I800 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2383991 = {S72576, 151, { &P14849,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I800 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2019735 = {S72577, 137, { &P20295,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC S610d Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020426 = {S72578, 125, { &P20295,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC S610d Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2024713 = {S72579, 137, { &P20295,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-cn; HTC_S610d Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2543193 = {S72580, 130, { &P20295,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC S610d Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2376611 = {S72581, 146, { &P12473,&P5204,&P24670,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia2730c-1) UCBrowser8.4.0.159/70/444/UCWEB Mobile*/
const Device UA2593272 = {S72582, 68, { &P12280,&P17779,&P19984,&P18092 }};	/*Useragent - YahooMobile/1.0 (Football '12; 4.0) (Apple; iPhone; iPhone OS/5.1.1)*/
const Device UA2684561 = {S72583, 68, { &P12280,&P17779,&P19984,&P18092 }};	/*Useragent - YahooMobile/1.0 (Baseball '12; 2.2) (Apple; iPhone; iPhone OS/5.1.1)*/
const Device UA2687271 = {S72584, 94, { &P12280,&P17779,&P19984,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (iPhone Messenger; 2.2.7.5443) (6693; Apple; iPhone; iPhone OS/5.1.1)*/
const Device UA2979165 = {S72585, 73, { &P12280,&P17779,&P19984,&P18092 }};	/*Useragent - YahooMobile/1.0 (iPhone Sportacular; 95) (Apple; iPhone; iPhone OS/5.1.1)*/
const Device UA2369015 = {S72586, 57, { &P15364,&P7127,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0) Presto/2.12.388 Version/12.11*/
const Device UA2655512 = {S72587, 69, { &P15364,&P7127,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; Edition Yx) Presto/2.12.388 Version/12.11*/
const Device UA2423631 = {S72588, 158, { &P15575,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2457292 = {S72589, 157, { &P15575,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2504131 = {S72590, 158, { &P15575,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2527916 = {S72591, 157, { &P15575,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2727783 = {S72592, 157, { &P15575,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2727934 = {S72593, 85, { &P15575,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Adr (Linux; U; 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD.ZSKI3;480*800)*/
const Device UA2842763 = {S72594, 157, { &P15575,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9108 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2842982 = {S72595, 86, { &P15575,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Adr (Linux; U; 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD.ZCLC2;480*800)*/
const Device UA2844190 = {S72596, 157, { &P15575,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2983964 = {S72597, 158, { &P15575,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2466233 = {S72598, 132, { &P18465,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2508846 = {S72599, 171, { &P18304,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3020547 = {S72600, 175, { &P18304,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9070 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2511713 = {S72601, 168, { &P17625,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2706725 = {S72602, 115, { &P23200,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; YL-Coolpad_5890/4.1.007.X.5890; 540*960; CTC/2.0) CoolpadWebkit/534.30*/
const Device UA2732046 = {S72603, 143, { &P23200,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 5890 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2891199 = {S72604, 134, { &P23200,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 5890 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2997537 = {S72605, 65, { &P23200,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Coolpad 5890 Build/JZO54K)*/
const Device UA2751194 = {S72606, 141, { &P20738,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Andromax U Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2861659 = {S72607, 63, { &P20738,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Andromax U Build/JZO54K)*/
const Device UA2768898 = {S72608, 135, { &P24509,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; ARNOVA 97G4 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3082744 = {S72609, 152, { &P21968,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080080 = {S72610, 148, { &P18227,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U9510E Build/HuaweiU9510E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA237625 = {S72611, 99, { &P15364,&P17188,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1*/
const Device UA220495 = {S72612, 136, { &P16168,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; NookColor Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA556478 = {S72613, 136, { &P16168,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NookColor Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA811654 = {S72614, 136, { &P16168,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; NookColor Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA782604 = {S72615, 133, { &P20386,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; novo7_ELF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2034480 = {S72616, 62, { &P20386,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; novo7_ELF Build/IML74K)*/
const Device UA2840621 = {S72617, 133, { &P20386,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Novo7 ELF Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA896417 = {S72618, 72, { &P15364,&P5690,&P22359,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:12.1) Gecko/X Firefox/12.1 PaleMoon/12.1*/
const Device UA2426484 = {S72619, 141, { &P12280,&P18117,&P20790,&P18092 }};	/*Useragent - MQQBrowser/39 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA2458594 = {S72620, 140, { &P12280,&P18117,&P20790,&P18092 }};	/*Useragent - MQQBrowser/39 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3*/
const Device UA2469475 = {S72621, 142, { &P12280,&P18117,&P20790,&P18092 }};	/*Useragent - MQQBrowser/391 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA2848826 = {S72622, 141, { &P12280,&P18117,&P20790,&P18092 }};	/*Useragent - MQQBrowser/391 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3*/
const Device UA2506386 = {S72623, 154, { &P19853,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7102 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666113 = {S72624, 87, { &P19853,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; GT-N7102 Build/JRO03C.N7102ZCAMB3;720*1280)*/
const Device UA2731642 = {S72625, 87, { &P19853,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; GT-N7102 Build/JRO03C.N7102ZNAMB1;720*1280)*/
const Device UA2768765 = {S72626, 154, { &P19853,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7102 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986938 = {S72627, 87, { &P19853,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; GT-N7102 Build/JRO03C.N7102ZNAMA6;720*1280)*/
const Device UA2567329 = {S72628, 137, { &P18320,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PadFone Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567567 = {S72629, 130, { &P18320,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PadFone Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2709254 = {S72630, 130, { &P18320,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PadFone Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567399 = {S72631, 139, { &P22592,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717R Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2598026 = {S72632, 148, { &P20729,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4(6577); en-us; Celkon A27  Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2702921 = {S72633, 140, { &P20729,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; CROSS A27 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2854605 = {S72634, 165, { &P20729,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A27 Build/BCM21654AndroidTK_SystemRel_1.3.2P.05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2660410 = {S72635, 156, { &P14482,&P5843,&P24676,&P18092 }};	/*Useragent - NokiaC3-01/5.0 (06.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac3-01) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2854840 = {S72636, 160, { &P14482,&P5843,&P24676,&P18092 }};	/*Useragent - NokiaC3-01.5/5.0 (06.75) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac3-01.5) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2714102 = {S72637, 166, { &P14225,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2799370 = {S72638, 156, { &P14225,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; Desire HD Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3030879 = {S72639, 152, { &P15241,&P5843,&P24679,&P18092 }};	/*Useragent - Nokia300/5.0 (07.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia300) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3037311 = {S72640, 127, { &P18680,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KFJWI Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Safari/537.36*/
const Device UA3074357 = {S72641, 90, { &P12280,&P22882,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 7_0; zh-CN; iPh5,2) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile*/
const Device UA3103356 = {S72642, 90, { &P12280,&P22882,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 7_0; zh-CN; iPh4,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile*/
const Device UA3125234 = {S72643, 90, { &P12280,&P22882,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 7_0; zh-CN; iPh3,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile*/
const Device UA3125551 = {S72644, 90, { &P12280,&P22882,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 7_0; zh-CN; iPh5,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile*/
const Device UA38511 = {S72645, 143, { &P17945,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; PB99400 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA536925 = {S72646, 65, { &P17945,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; PB99400 Build/ERE27)*/
const Device UA571939 = {S72647, 149, { &P20931,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-in; iBall Slide i7011 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA696801 = {S72648, 151, { &P20931,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-in; iBall Slide i7011 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024684 = {S72649, 149, { &P20931,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-in; iBall Slide i7119 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA608443 = {S72650, 98, { &P15364,&P22192,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.0.6002.2);en;kml:2.2;client:Free;type:default)*/
const Device UA869513 = {S72651, 98, { &P15364,&P22192,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.1.6014(Windows;Microsoft Windows (6.0.6002.2);en;kml:2.2;client:Free;type:default)*/
const Device UA1956369 = {S72652, 98, { &P15364,&P22192,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.0.6001.1);en;kml:2.2;client:Free;type:default)*/
const Device UA2162623 = {S72653, 98, { &P15364,&P22192,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.0.6002.2);de;kml:2.2;client:Free;type:default)*/
const Device UA2335367 = {S72654, 98, { &P15364,&P22192,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.0.6002.2);nl;kml:2.2;client:Free;type:default)*/
const Device UA2445125 = {S72655, 98, { &P15364,&P22192,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.0.6002.2);fr;kml:2.2;client:Free;type:default)*/
const Device UA900168 = {S72656, 116, { &P15364,&P18110,&P22458,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5*/
const Device UA901397 = {S72657, 118, { &P15364,&P18110,&P22458,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5*/
const Device UA907246 = {S72658, 64, { &P15364,&P18144,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64; rv:13.0) Gecko/X Firefox/13.0.1*/
const Device UA2197193 = {S72659, 171, { &P18214,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2229000 = {S72660, 146, { &P20284,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; Xenta TAB9-200 Build/Xenta TAB9-200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2236517 = {S72661, 138, { &P19870,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-L710 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2256966 = {S72662, 132, { &P20976,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; W100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2817892 = {S72663, 139, { &P20976,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GHONG W100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2405447 = {S72664, 171, { &P12056,&P5204,&P24673,&P18092 }};	/*Useragent - Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia7210supernova) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2507163 = {S72665, 156, { &P12216,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ADR6300 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2599082 = {S72666, 155, { &P12216,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ADR6300 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768963 = {S72667, 156, { &P12216,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ADR6300 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2531000 = {S72668, 151, { &P23970,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; PROG95B Build/G95-V1.0-2012.12.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2557176 = {S72669, 151, { &P23970,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; PROG95B Build/G95-V1.0-2012.12.14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2737731 = {S72670, 151, { &P23970,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; PROG95B Build/G95-V1.0-2013.03.22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3047739 = {S72671, 151, { &P23970,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; PROG95B Build/G95-V1.0-2012.12.07) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2559732 = {S72672, 152, { &P19973,&P5043,&P24676,&P18092 }};	/*Useragent - Nokia114/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia114) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2566771 = {S72673, 139, { &P19777,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105P Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA3072610 = {S72674, 138, { &P19777,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2581754 = {S72675, 103, { &P13324,&P5055,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian8-00) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2705872 = {S72676, 32, { &P13324,&P5055,&P24676,&P18092 }};	/*Useragent - NokiaN8-00/UCWEB8.9.0.253/51/999*/
const Device UA2707923 = {S72677, 138, { &P22753,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A818i Duo Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2757299 = {S72678, 153, { &P22785,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire 600 dual sim Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2945610 = {S72679, 143, { &P15364,&P17350,&P22777,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; ; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.4.2.34167 (Firestorm-Release; firestorm skin) Safari/533.3*/
const Device UA3081688 = {S72680, 132, { &P23162,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B15 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA8728 = {S72681, 106, { &P13498,&P5448,&P17183,&P18092 }};	/*Useragent - SAMSUNG-GT-B7350Orange/BVJI1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA10411 = {S72682, 98, { &P13498,&P5448,&P17183,&P18092 }};	/*Useragent - SAMSUNG-GT-B7350/1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA515144 = {S72683, 113, { &P13498,&P5448,&P17183,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-GT-B7350/BUJG5 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA156118 = {S72684, 141, { &P18015,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; zt180 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018136 = {S72685, 63, { &P18015,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; zt180 Build/ERE27)*/
const Device UA646066 = {S72686, 45, { &P12004,&P5890,&P24665,&P18092 }};	/*Useragent - NOKIAN95 8GB/UC Browser8.0.4.121/28/444/UCWEB*/
const Device UA696305 = {S72687, 41, { &P12004,&P5890,&P24665,&P18092 }};	/*Useragent - NOKIAN95/UC Browser8.0.4.121/28/444/UCWEB*/
const Device UA628367 = {S72688, 86, { &P16384,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s5380k) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA814503 = {S72689, 86, { &P16384,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s5380k) UCBrowser8.2.0.132/69/352/UCWEB Mobile*/
const Device UA751521 = {S72690, 156, { &P24002,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; Android Edition StarText Build/V1.0.0B06) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA895490 = {S72691, 156, { &P24002,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; Android Edition StarText Build/V1.0.0B07) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1964986 = {S72692, 141, { &P21338,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Symphony W10 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995642 = {S72693, 136, { &P17283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB860 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996866 = {S72694, 136, { &P17283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB860 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2194419 = {S72695, 136, { &P17283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB860 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258065 = {S72696, 136, { &P17283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB860 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2367286 = {S72697, 136, { &P17283,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MB860 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2402351 = {S72698, 58, { &P17283,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; MB860 Build/JZO54K)*/
const Device UA2023987 = {S72699, 139, { &P21426,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; STM722HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027209 = {S72700, 140, { &P21426,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; STM722HCZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027499 = {S72701, 139, { &P21426,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; STM722HZ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2234651 = {S72702, 171, { &P18304,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2194816 = {S72703, 143, { &P23900,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ELONEX1012ET Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2230331 = {S72704, 135, { &P21306,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOMO11Speed Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2385123 = {S72705, 154, { &P18442,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2562207 = {S72706, 154, { &P18442,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2698146 = {S72707, 87, { &P18442,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; GT-N7100 Build/JRO03C.N7100ZCALK5;720*1280)*/
const Device UA2752651 = {S72708, 87, { &P18442,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-N7100 Build/JZO54K.N7100ZCDMD3;720*1280)*/
const Device UA2791824 = {S72709, 87, { &P18442,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; GT-N7100 Build/JRO03C.N7100ZCALL4;720*1280)*/
const Device UA2425008 = {S72710, 133, { &P23084,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; da-dk; EN-TAB419 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2426480 = {S72711, 131, { &P20818,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMID705 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2483431 = {S72712, 132, { &P20818,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PMID705X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2847102 = {S72713, 60, { &P20818,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; PMID705 Build/IMM76D)*/
const Device UA2462002 = {S72714, 59, { &P15364,&P7133,&P18068,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE; Windows NT 6.1; Trident/4.0)*/
const Device UA2658914 = {S72715, 63, { &P15364,&P7133,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; Windows NT 6.1; Trident/5.0)*/
const Device UA2850080 = {S72716, 63, { &P15364,&P7133,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; Windows NT 6.1; Trident/5.0)*/
const Device UA2559701 = {S72717, 135, { &P23451,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T720 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987392 = {S72718, 57, { &P23451,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; T720 Build/IMM76D)*/
const Device UA2567700 = {S72719, 139, { &P18386,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I757M Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667616 = {S72720, 147, { &P20509,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MyPhone A888 Duo Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958340 = {S72721, 157, { &P23617,&P18066,&P18062,&P18092 }};	/*Useragent - RunboX5-W Linux/3.0.13 Android/4.0.4 Release/05.20.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2738646 = {S72722, 143, { &P22737,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Micromax A115 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2753057 = {S72723, 142, { &P19862,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax A110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2975012 = {S72724, 137, { &P18722,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A830S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020269 = {S72725, 137, { &P18722,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A830K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020923 = {S72726, 137, { &P18722,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A830L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023588 = {S72727, 93, { &P18673,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S7562) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3080908 = {S72728, 135, { &P22585,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SPH-L720 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080485 = {S72729, 138, { &P17691,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I957 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3083286 = {S72730, 131, { &P17691,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957R Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094534 = {S72731, 131, { &P17691,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA63868 = {S72732, 84, { &P11847,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW380i/R11CA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA422723 = {S72733, 130, { &P15364,&P7133,&P22423,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.4.1.0 Chrome/17.0.963.56 Safari/535.11*/
const Device UA422786 = {S72734, 123, { &P15364,&P7133,&P22423,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.4.1.0 Chrome/17.0.963.56 Safari/535.11*/
const Device UA842477 = {S72735, 171, { &P13123,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2026515 = {S72736, 171, { &P13123,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.2.139/145/355*/
const Device UA2597929 = {S72737, 175, { &P13123,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5360 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA859322 = {S72738, 138, { &P20853,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Scroll Extreme Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2765027 = {S72739, 141, { &P20853,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Scroll Extreme ll Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2236426 = {S72740, 140, { &P15354,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST17i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2525333 = {S72741, 140, { &P15354,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST17a Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2040603 = {S72742, 93, { &P17595,&P17296,&P20755,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:15.0) Gecko/X Firefox/15.0.1 SeaMonkey/2.12.1*/
const Device UA2354709 = {S72743, 136, { &P22164,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; ST97216-1 Build/ST97216-1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2431206 = {S72744, 229, { &P18672,&P5629,&P18369,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2504832 = {S72745, 112, { &P22717,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia109/04.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.35*/
const Device UA2708380 = {S72746, 142, { &P15767,&P18117,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Maxthon (4.0.4.1000) Safari/537.4*/
const Device UA2557371 = {S72747, 131, { &P20582,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; AN10G2I Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2560821 = {S72748, 134, { &P24524,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; V812 Core4 Build/MASTER) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2664984 = {S72749, 139, { &P18280,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC One S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2702311 = {S72750, 135, { &P18280,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2749441 = {S72751, 57, { &P18280,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; One S Build/JDQ39)*/
const Device UA2956955 = {S72752, 140, { &P18280,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC One S Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3126810 = {S72753, 139, { &P18280,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC_One_S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2718807 = {S72754, 139, { &P18449,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST27i Build/6.2.A.0.400) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2768440 = {S72755, 132, { &P18449,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST27i Build/6.2.A.0.400) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2773111 = {S72756, 165, { &P12657,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515m Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2937471 = {S72757, 165, { &P12657,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515a Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2938600 = {S72758, 154, { &P24072,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-mx; ALCATEL ONE TOUCH 6033A Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA3021138 = {S72759, 150, { &P18552,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT897 Build/7.7.1Q-6_SPR-89_ASA-36) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3062548 = {S72760, 146, { &P20283,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AZ210B Build/AZ210B_ICS_02.34I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3081678 = {S72761, 146, { &P20283,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AZ210A Build/AZ210A_ICS_02.34I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3060137 = {S72762, 143, { &P12280,&P22882,&P24341,&P18092 }};	/*Useragent - MQQBrowser/44 Mozilla/5.0 (iPhone 5; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A4449d Safari/7534.48.3*/
const Device UA3109550 = {S72763, 141, { &P12280,&P22882,&P24341,&P18092 }};	/*Useragent - MQQBrowser/44 Mozilla/5.0 (iPhone 5; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari/7534.48.3*/
const Device UA3129326 = {S72764, 143, { &P12280,&P22882,&P24341,&P18092 }};	/*Useragent - MQQBrowser/44 Mozilla/5.0 (iPhone 5; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Safari/7534.48.3*/
const Device UA3080911 = {S72765, 126, { &P16598,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; A500 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3082162 = {S72766, 133, { &P16598,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; A500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA52404 = {S72767, 143, { &P16815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; in-id; CSL_Spice_MI700 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA62674 = {S72768, 141, { &P16815,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; CSL_Spice_MI700 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67612 = {S72769, 137, { &P14821,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; LG-LU3000 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67654 = {S72770, 136, { &P14821,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-LU3000 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67905 = {S72771, 138, { &P14821,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; LG-LU3000 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018763 = {S72772, 62, { &P14821,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LG-LU3000 Build/FRG83G)*/
const Device UA654448 = {S72773, 138, { &P21218,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; ZTE-U N721 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA741652 = {S72774, 138, { &P21218,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; ZTE_U N721 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA772344 = {S72775, 132, { &P21218,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; N721 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA734836 = {S72776, 71, { &P16246,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i677;7.10;7720)*/
const Device UA925788 = {S72777, 71, { &P16246,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i677;7.10;8773)*/
const Device UA2693684 = {S72778, 71, { &P16246,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i677;7.10;8107)*/
const Device UA2362374 = {S72779, 118, { &P14454,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE66) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/355/UCWEB Mobile*/
const Device UA2842820 = {S72780, 119, { &P14454,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiae66-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/84/352/UCWEB Mobile*/
const Device UA2347155 = {S72781, 126, { &P16384,&P17484,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5380k) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2363051 = {S72782, 154, { &P18465,&P18066,&P19803,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2366543 = {S72783, 71, { &P14842,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.1-update1; zh-cn; E10i; 240*320) UCWEB7.9.0.94/139/352*/
const Device UA2376085 = {S72784, 137, { &P18173,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; ISW11F Build/FIK700) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423568 = {S72785, 119, { &P14359,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5228) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2555714 = {S72786, 41, { &P14359,&P5129,&P24673,&P18092 }};	/*Useragent - Nokia5228UCBrowser/8.7.1.234/50/444/UCWEB*/
const Device UA2454471 = {S72787, 148, { &P15985,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6200 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2480626 = {S72788, 148, { &P15985,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P6200 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616571 = {S72789, 148, { &P15985,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6200 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2632464 = {S72790, 159, { &P15985,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P6200 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467859 = {S72791, 102, { &P18463,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; GT-I8750)*/
const Device UA2479968 = {S72792, 103, { &P18463,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-T899M)*/
const Device UA2566884 = {S72793, 138, { &P19854,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R760 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2580151 = {S72794, 138, { &P23500,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; BLU DASH 3.2 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2615547 = {S72795, 158, { &P15882,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5110 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2753398 = {S72796, 131, { &P23835,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; TAB1050 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2755084 = {S72797, 138, { &P13109,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC-X710a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2771843 = {S72798, 166, { &P17283,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; MB860 Build/4.5.1A-129_OLY-145) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2784047 = {S72799, 164, { &P17283,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.4; MB860 Build/4.5.2A-74_OLE-31) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2958032 = {S72800, 155, { &P17283,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.7; MB860 Build/4.5.141) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2959702 = {S72801, 152, { &P19748,&P5043,&P24678,&P18092 }};	/*Useragent - Nokia205/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia205) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3013216 = {S72802, 156, { &P19748,&P5043,&P24678,&P18092 }};	/*Useragent - Nokia205.1/2.0 (03.19) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia205.1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2973915 = {S72803, 133, { &P20690,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; V360 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2971172 = {S72804, 91, { &P15364,&P16110,&P23490,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Maxthon/4.1.0.4000)*/
const Device UA3016905 = {S72805, 84, { &P15364,&P16110,&P23490,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; Maxthon/4.1.0.4000)*/
const Device UA3028035 = {S72806, 151, { &P18283,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9300 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3034629 = {S72807, 151, { &P18283,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9300 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3107888 = {S72808, 151, { &P18283,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I939 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3108548 = {S72809, 152, { &P18283,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I939D Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3132946 = {S72810, 139, { &P18447,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.1.141) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA2883 = {S72811, 250, { &P12042,&P17485,&P3487,&P18092 }};	/*Useragent - SonyEricssonS302/R1BB Browser/OpenWave/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1, SonyEricssonS302/R1BB Browser/OpenWave/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1, SonyEricssonS302/R1BB Browser/OpenWave/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2884 = {S72812, 166, { &P12042,&P17485,&P3487,&P18092 }};	/*Useragent - SonyEricssonS302/R1BB Browser/OpenWave/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1, SonyEricssonS302/R1BB Browser/OpenWave/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22220 = {S72813, 126, { &P12042,&P17485,&P3487,&P18092 }};	/*Useragent - SonyEricssonS302/R1AB Browser/OpenWave/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.FENGPROD.104 (GUI)*/
const Device UA40010 = {S72814, 82, { &P12042,&P17485,&P3487,&P18092 }};	/*Useragent - SonyEricssonS302/R1BB Browser/OpenWave/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA30962 = {S72815, 78, { &P12376,&P6217,&P3418,&P18092 }};	/*Useragent - BlackBerry8230f/4.6.1.309 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/294*/
const Device UA23976 = {S72816, 73, { &P15187,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C3500/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA570409 = {S72817, 162, { &P15364,&P17285,&P18147,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe; SLCC1; .NET CLR 2.0.50727; .NET CLR 1.1.4322; .NET CLR 3.5.30729; .NET CLR 3.0.30729)*/
const Device UA2805497 = {S72818, 188, { &P15364,&P17285,&P18147,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; InfoPath.2; .NET CLR 3.0.30729; WWTClient2)*/
const Device UA770210 = {S72819, 132, { &P21220,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; 102P Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2494710 = {S72820, 56, { &P21220,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; 102P Build/GRJ90)*/
const Device UA1995925 = {S72821, 121, { &P15364,&P18110,&P22460,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11*/
const Device UA1996394 = {S72822, 119, { &P15364,&P18110,&P22460,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11*/
const Device UA2433097 = {S72823, 143, { &P19851,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; Micromax A68 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2460852 = {S72824, 155, { &P12897,&P5843,&P24675,&P18092 }};	/*Useragent - NokiaC2-00/2.0 (03.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac2-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2510294 = {S72825, 166, { &P15919,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P3100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2558199 = {S72826, 166, { &P15919,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3125041 = {S72827, 170, { &P15919,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-P3100 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2529488 = {S72828, 128, { &P21817,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; VP10 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2702168 = {S72829, 128, { &P21817,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; VP10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2580603 = {S72830, 91, { &P15989,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i2) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2661403 = {S72831, 90, { &P15989,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2633548 = {S72832, 151, { &P22037,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3113 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2773275 = {S72833, 150, { &P22037,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P3113 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2669342 = {S72834, 129, { &P18507,&P17979,&P18062,&P18092 }};	/*Useragent - samsung-I8160/1.0 Linux/2.6.35.7 Android/4.0.4 Release/01.29.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2666829 = {S72835, 179, { &P18604,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Smartphone Android by SFR STARADDICT II Build/IceCreamSandwich) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2670024 = {S72836, 158, { &P18604,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; STARADDICT II Plus Build/GXI_ZTEV1.0.0B02) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2667807 = {S72837, 138, { &P16273,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T679 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2751910 = {S72838, 139, { &P16273,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T679 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2676687 = {S72839, 136, { &P14123,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P970 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709486 = {S72840, 137, { &P14123,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P970h Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2710723 = {S72841, 136, { &P21261,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714678 = {S72842, 129, { &P21261,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2754850 = {S72843, 130, { &P21012,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I815 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755406 = {S72844, 151, { &P18505,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MB886 Build/7.7.1Q-144_MB886_MR1-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783713 = {S72845, 151, { &P18283,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9300 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2876519 = {S72846, 151, { &P18283,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9300 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3001603 = {S72847, 152, { &P18283,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I939D Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3114882 = {S72848, 158, { &P15575,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3115555 = {S72849, 157, { &P15575,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3115606 = {S72850, 157, { &P15575,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA24479 = {S72851, 108, { &P11685,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-D900-ORANGE/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA44219 = {S72852, 106, { &P11685,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-D900-VODA/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA64410 = {S72853, 101, { &P11685,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-D900/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA54212 = {S72854, 45, { &P14151,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-710A/1.0 ObigoInternetBrowser/Q03C*/
const Device UA57949 = {S72855, 85, { &P14151,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-710A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA59112 = {S72856, 170, { &P14151,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-710/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C, Alcatel-OT-710/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA63194 = {S72857, 84, { &P14151,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-710/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA63506 = {S72858, 85, { &P14151,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-710D/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA645257 = {S72859, 83, { &P15364,&P5690,&P23681,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2 x64; en-US; rv:1.9a1) Gecko/X Firefox/1.6a1*/
const Device UA656350 = {S72860, 60, { &P15364,&P5690,&P23681,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:1.9a1) Gecko/X Firefox/1.6a1*/
const Device UA884015 = {S72861, 79, { &P15364,&P5690,&P23681,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9a1) Gecko/X Firefox/1.6a1*/
const Device UA862921 = {S72862, 159, { &P13042,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Incredible S Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1959546 = {S72863, 147, { &P21716,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-pt; Optimus Barcelona Build/OPT_B01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1990541 = {S72864, 155, { &P12444,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; U20i Build/3.0.1.A.0.145; MiniCM7-2.2.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022519 = {S72865, 141, { &P12444,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; U20i Build/MiniCM7 Pro V7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2430898 = {S72866, 155, { &P18465,&P18066,&P19816,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2445997 = {S72867, 60, { &P19974,&P5043,&P18517,&P18092 }};	/*Useragent - Nokia111/2.0 (03.24) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2478871 = {S72868, 62, { &P19974,&P5043,&P18517,&P18092 }};	/*Useragent - Nokia111.1/2.0 (03.24) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2702020 = {S72869, 103, { &P12588,&P5055,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac6-01) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2716728 = {S72870, 94, { &P12588,&P5055,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC6-01) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2935267 = {S72871, 32, { &P12588,&P5055,&P24676,&P18092 }};	/*Useragent - NokiaC6-01/UCWEB8.9.0.253/51/999*/
const Device UA2754458 = {S72872, 114, { &P15364,&P7133,&P21635,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36 slurp*/
const Device UA2784939 = {S72873, 115, { &P15364,&P7133,&P21635,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36 slurp*/
const Device UA2799799 = {S72874, 108, { &P15364,&P7133,&P21635,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36 slurp*/
const Device UA2876200 = {S72875, 115, { &P15364,&P7133,&P21635,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36 slurp*/
const Device UA2899734 = {S72876, 108, { &P15364,&P7133,&P21635,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36 slurp*/
const Device UA2879655 = {S72877, 139, { &P23647,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; de-de; Cynus T5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3021001 = {S72878, 93, { &P18672,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5302) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3082970 = {S72879, 128, { &P24400,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SM-T315 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3095648 = {S72880, 131, { &P15767,&P22882,&P20784,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/11A465 Safari/7534.48.3*/
const Device UA3129901 = {S72881, 133, { &P15767,&P22882,&P20784,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/11A501 Safari/7534.48.3*/
const Device UA63727 = {S72882, 101, { &P11805,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-M150/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA43478 = {S72883, 136, { &P14784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_Hero Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA304600 = {S72884, 141, { &P14784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Hero CDMA Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA331248 = {S72885, 132, { &P14784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Hero Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA574692 = {S72886, 132, { &P14784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Hero Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA677148 = {S72887, 132, { &P14784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; Hero Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA857232 = {S72888, 142, { &P14784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC Hero CDMA Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018766 = {S72889, 138, { &P14784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; HTC Hero Build/unknown) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019241 = {S72890, 136, { &P14784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC Hero Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019767 = {S72891, 56, { &P14784,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; Hero Build/GRH78)*/
const Device UA2023742 = {S72892, 145, { &P14784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Hero Build/GRI40; LiGux-Beta4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027291 = {S72893, 120, { &P14784,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Hero Build/GRH78) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA458818 = {S72894, 139, { &P21208,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; MID7026 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA654990 = {S72895, 143, { &P17937,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Motorola Triumph Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021495 = {S72896, 143, { &P17937,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Motorola Triumph Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2659208 = {S72897, 144, { &P21148,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PocketBook A10 3G Build/GRJ90) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1994204 = {S72898, 128, { &P18733,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M532 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2229240 = {S72899, 71, { &P17595,&P17297,&P21577,&P18092 }};	/*Useragent - Mac OS X/10.7.5 (11G56); ExchangeWebServices/2.2 (127); Mail/5.3 (1283)*/
const Device UA2353554 = {S72900, 71, { &P17595,&P17297,&P21577,&P18092 }};	/*Useragent - Mac OS X/10.7.5 (11G63); ExchangeWebServices/2.2 (127); Mail/5.3 (1283)*/
const Device UA2421511 = {S72901, 72, { &P17595,&P17297,&P21577,&P18092 }};	/*Useragent - Mac OS X/10.7.5 (11G63b); ExchangeWebServices/2.2 (127); Mail/5.3 (1283)*/
const Device UA2743658 = {S72902, 73, { &P17595,&P17297,&P21577,&P18092 }};	/*Useragent - Mac OS X/10.7.4 (11E2617); ExchangeWebServices/2.1 (125); Mail/5.3 (1280)*/
const Device UA2533366 = {S72903, 205, { &P17262,&P5629,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Kindle Fire Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.19/283,1.9.19.283,2013-01-17 12:06, -0500, us*/
const Device UA2437201 = {S72904, 153, { &P17172,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Transformer Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2714089 = {S72905, 159, { &P17172,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Transformer Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2855352 = {S72906, 152, { &P17172,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Transformer Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2960416 = {S72907, 165, { &P17172,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Transformer Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/tf101*/
const Device UA2784245 = {S72908, 95, { &P17968,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Android2.x) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2974089 = {S72909, 127, { &P18704,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT200 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2973492 = {S72910, 132, { &P13644,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; EVO Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022842 = {S72911, 132, { &P13644,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; EVO Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3081079 = {S72912, 145, { &P18114,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Transformer Prime TF201 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3082766 = {S72913, 129, { &P22873,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N5120 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3067621 = {S72914, 149, { &P22749,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SonySO-04E Build/10.3.1.B.0.224) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3093747 = {S72915, 145, { &P22749,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SO-04E Build/10.3.1.B.0.224) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3114931 = {S72916, 67, { &P22749,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SO-04E Build/10.3.1.B.0.224)*/
const Device UA3083244 = {S72917, 162, { &P15364,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; en-us; SAMSUNG SM-N900V Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Version/1.5 Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3094603 = {S72918, 155, { &P15364,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; en-ca; SM-N900W8 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Version/1.5 Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3109208 = {S72919, 162, { &P15364,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; en-us; SAMSUNG SM-N900P Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Version/1.5 Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3126756 = {S72920, 170, { &P15364,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; ko-kr; SAMSUNG SM-N900K/KKUBMI8 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Version/1.5 Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3127034 = {S72921, 162, { &P15364,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; ko-kr; SAMSUNG SM-N900L Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Version/1.5 Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3128606 = {S72922, 175, { &P15364,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; en-gb; SAMSUNG SM-N9005/N9005XXUBMI7 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Version/1.5 Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA5040 = {S72923, 42, { &P14675,&P17483,&P3625,&P18092 }};	/*Useragent - LG265/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA9918 = {S72924, 49, { &P14675,&P17483,&P3625,&P18092 }};	/*Useragent - LG265-parrot/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA64502 = {S72925, 46, { &P14675,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-LG265/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA638657 = {S72926, 100, { &P15364,&P18110,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.2.149.27 Safari/525.13*/
const Device UA677962 = {S72927, 139, { &P24556,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; CT704 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1952440 = {S72928, 134, { &P21209,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; XT550 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2933521 = {S72929, 134, { &P21209,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-co; XT550 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Versi?n/4.0 Mobile Safari/533.1*/
const Device UA2933530 = {S72930, 58, { &P21209,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; XT550 Build/GRK39F)*/
const Device UA1972437 = {S72931, 140, { &P17971,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; tr-tr; PLANET Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2366864 = {S72932, 79, { &P15798,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_Explorer_A310e; 320*480) UCWEB7.9.0.94/139/352*/
const Device UA2384781 = {S72933, 79, { &P15798,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_Explorer_A310e; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2386826 = {S72934, 79, { &P15798,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_Explorer_A310e; 320*480) UCWEB7.9.0.94/139/355*/
const Device UA2401525 = {S72935, 149, { &P20001,&P17979,&P18062,&P18092 }};	/*Useragent - Lenovo S720/S100 Linux/3.0.13 Android/4.0.3 Release/12.12.2011 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30*/
const Device UA2732851 = {S72936, 103, { &P11907,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae52-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2737545 = {S72937, 94, { &P11907,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE52-1) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2753861 = {S72938, 140, { &P20285,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ616-12) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2773850 = {S72939, 145, { &P20285,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ616 Build/7.7.1-141-6-PAST-UMTS-LA) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2973664 = {S72940, 145, { &P18114,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Transformer Prime TF201 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3065719 = {S72941, 137, { &P22599,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA10141 = {S72942, 132, { &P17732,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; V-T100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA378330 = {S72943, 135, { &P17732,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; V-T100 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18551 = {S72944, 146, { &P14115,&P5448,&P17183,&P18092 }};	/*Useragent - SonyEricssonM1a/R1BA Browser/Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA60788 = {S72945, 151, { &P12575,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; Android Edition StarTrail Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA725619 = {S72946, 100, { &P14635,&P5843,&P17338,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (08.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac3-00/UC Browser7.9.0.102/69/352*/
const Device UA613297 = {S72947, 157, { &P12216,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6300 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA734465 = {S72948, 156, { &P12216,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6300 Build/GRJ22; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1853487 = {S72949, 69, { &P15364,&P18110,&P17387,&P18092 }};	/*Useragent - websitepulse checker/1.1 (compatible; MSIE 5.5; Netscape 4.75; Linux)*/
const Device UA895285 = {S72950, 129, { &P21162,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MP948 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2030406 = {S72951, 140, { &P15079,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MK16i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA924776 = {S72952, 75, { &P15364,&P17285,&P22316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:14.0) Gecko/X Firefox/14.0.1 SeaMonkey/2.11*/
const Device UA2202181 = {S72953, 139, { &P12280,&P18432,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3 Sleipnir/2.1.1m*/
const Device UA2237425 = {S72954, 139, { &P12280,&P18432,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3 Sleipnir/2.1.2m*/
const Device UA2237717 = {S72955, 139, { &P12280,&P18432,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Safari/7534.48.3 Sleipnir/2.1.2m*/
const Device UA2357070 = {S72956, 141, { &P12280,&P18432,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 Sleipnir/2.1.2m*/
const Device UA2357098 = {S72957, 141, { &P12280,&P18432,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 Sleipnir/2.1.3m*/
const Device UA2386725 = {S72958, 141, { &P12280,&P18432,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3 Sleipnir/2.1.3m*/
const Device UA2406822 = {S72959, 141, { &P12280,&P18432,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3 Sleipnir/2.1.4m*/
const Device UA2406823 = {S72960, 141, { &P12280,&P18432,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 Sleipnir/2.1.4m*/
const Device UA2505651 = {S72961, 141, { &P12280,&P18432,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 Sleipnir/2.1.5m*/
const Device UA2202908 = {S72962, 81, { &P17968,&P5096,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2.1; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2203463 = {S72963, 79, { &P17968,&P5096,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2203466 = {S72964, 81, { &P17968,&P5096,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2.2; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2238375 = {S72965, 81, { &P17968,&P5096,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2.3; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2361272 = {S72966, 137, { &P14017,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P990 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2428314 = {S72967, 136, { &P20437,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; FJL21 Build/V23R39X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2537916 = {S72968, 165, { &P15649,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3021047 = {S72969, 169, { &P15649,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5570 Build/FROYO) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2567135 = {S72970, 134, { &P18280,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; One S Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667657 = {S72971, 138, { &P18280,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One S Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2659661 = {S72972, 142, { &P23514,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; GL07S Build/HuaweiU9700L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3119422 = {S72973, 64, { &P23514,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GL07S Build/HuaweiU9700L)*/
const Device UA2676944 = {S72974, 137, { &P19894,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R830 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2754654 = {S72975, 144, { &P20007,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT886 Build/6.7.3-179_YIC-10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755734 = {S72976, 154, { &P20088,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E-51 Build/HuaweiU8666E-51) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773464 = {S72977, 148, { &P20088,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E Build/HuaweiU8666E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892572 = {S72978, 148, { &P20088,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666N Build/HuaweiU8666N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3081397 = {S72979, 135, { &P22594,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SGH-M919 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA4975 = {S72980, 41, { &P15364,&P17484,&P3625,&P18092 }};	/*Useragent - sam-u450 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA20837 = {S72981, 41, { &P15364,&P17484,&P3625,&P18092 }};	/*Useragent - sam-r351 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA30856 = {S72982, 41, { &P15364,&P17484,&P3625,&P18092 }};	/*Useragent - sam-r550 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA81668 = {S72983, 42, { &P15364,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-U340C UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA63098 = {S72984, 65, { &P13608,&P17483,&P17602,&P18092 }};	/*Useragent - LG-A133/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA53635 = {S72985, 83, { &P11911,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonZ530c/R6AC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA55768 = {S72986, 83, { &P11911,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonZ530i/R6DA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA30803 = {S72987, 47, { &P12343,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-Glide/ UP.Browser/7.2.7.2.541 (GUI) MMP/2.0*/
const Device UA1953471 = {S72988, 156, { &P13038,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-mx; ALCATEL_one_touch_910A Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1968364 = {S72989, 155, { &P13038,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; sl-si; ALCATEL_one_touch_910 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA54639 = {S72990, 102, { &P14248,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; Dell_Grappa Build/CUPCAKE) AppleWebKit/528.5+ Version/3.1.2*/
const Device UA179449 = {S72991, 63, { &P15364,&P7127,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.8.131 Version/11.10*/
const Device UA202268 = {S72992, 63, { &P15364,&P7127,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.8.131 Version/11.11*/
const Device UA1841584 = {S72993, 70, { &P15364,&P18146,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; SunOS i86pc; en-US; rv:1.9.1) Gecko/X Firefox/3.5*/
const Device UA512337 = {S72994, 50, { &P15364,&P7127,&P4618,&P18092 }};	/*Useragent - Mozilla/2.0 (compatible; MSIE 3.0; Windows NT 5.0)*/
const Device UA671444 = {S72995, 50, { &P15364,&P7127,&P4618,&P18092 }};	/*Useragent - Mozilla/3.0 (compatible; MSIE 3.0; Windows NT 5.0)*/
const Device UA2700467 = {S72996, 63, { &P15364,&P7127,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; Windows NT 5.0; Trident/1.0)*/
const Device UA605422 = {S72997, 136, { &P18384,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM101SH Build/S0028) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA747568 = {S72998, 136, { &P18384,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM101SH Build/S0018) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA748693 = {S72999, 136, { &P18384,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM101SH Build/S0020) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034617 = {S73000, 60, { &P18384,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; SBM101SH Build/S0028)*/
const Device UA2508287 = {S73001, 136, { &P18384,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM101SH Build/S0030) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2879918 = {S73002, 136, { &P18384,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; SBM101SH Build/S0032) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA909518 = {S73003, 129, { &P23921,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KM10C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422354 = {S73004, 118, { &P12029,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN82) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2470289 = {S73005, 149, { &P15767,&P18432,&P21789,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1 (like Version/10A523 Mobile/6.0.1 Safari/8536.25)*/
const Device UA2486504 = {S73006, 149, { &P15767,&P18432,&P21789,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1 (like Version/10A550 Mobile/6.0.2 Safari/8536.25)*/
const Device UA2566444 = {S73007, 141, { &P18183,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ615-12) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2595205 = {S73008, 139, { &P20096,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7568 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2697659 = {S73009, 61, { &P20096,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-S7568 Build/IMM76D)*/
const Device UA2675838 = {S73010, 136, { &P13074,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P930 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2715175 = {S73011, 151, { &P20529,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A89 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2734313 = {S73012, 112, { &P18558,&P20706,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia309/07.55; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.36*/
const Device UA2735571 = {S73013, 140, { &P20280,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; vivo X1St Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2859658 = {S73014, 139, { &P20280,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; vivo X1S Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996748 = {S73015, 62, { &P20280,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; vivo X1St Build/JOP40D)*/
const Device UA2769028 = {S73016, 145, { &P21264,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PadFone Infinity Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774120 = {S73017, 138, { &P21264,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PadFone Infinity Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2868093 = {S73018, 141, { &P12280,&P18432,&P22879,&P18092 }};	/*Useragent - MQQBrowser/43 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3*/
const Device UA2876348 = {S73019, 142, { &P12280,&P18432,&P22879,&P18092 }};	/*Useragent - MQQBrowser/43 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3*/
const Device UA2879250 = {S73020, 141, { &P12280,&P18432,&P22879,&P18092 }};	/*Useragent - MQQBrowser/43 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari/7534.48.3*/
const Device UA2960631 = {S73021, 140, { &P12280,&P18432,&P22879,&P18092 }};	/*Useragent - MQQBrowser/43 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3*/
const Device UA3006563 = {S73022, 139, { &P12280,&P18432,&P22879,&P18092 }};	/*Useragent - MQQBrowser/43 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Safari/7534.48.3*/
const Device UA24519 = {S73023, 122, { &P13879,&P17483,&P3459,&P18092 }};	/*Useragent - LG-T320/V100 Obigo/Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA48970 = {S73024, 123, { &P13879,&P17483,&P3459,&P18092 }};	/*Useragent - LG-T320g/V100 Obigo/Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA369400 = {S73025, 131, { &P21665,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; BLU Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA709496 = {S73026, 155, { &P14837,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-I5800 Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA910205 = {S73027, 129, { &P21245,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TP10A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2355927 = {S73028, 129, { &P15767,&P18432,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3*/
const Device UA2357081 = {S73029, 131, { &P15767,&P18432,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3*/
const Device UA2428552 = {S73030, 129, { &P15767,&P18432,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A407 Safari/7534.48.3*/
const Device UA2395706 = {S73031, 150, { &P13123,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; es-es; GT-S5360L Build/ICE CREAM SANDWICH) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2415169 = {S73032, 142, { &P13123,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2418663 = {S73033, 142, { &P13123,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.6; pt-pt; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2456800 = {S73034, 136, { &P21231,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; L-02E Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2462437 = {S73035, 151, { &P19748,&P5043,&P24673,&P18092 }};	/*Useragent - Nokia205/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia205) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA3088498 = {S73036, 155, { &P19748,&P5043,&P24673,&P18092 }};	/*Useragent - Nokia205.1/2.0 (03.19) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia205.1) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2485703 = {S73037, 142, { &P21262,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AIRIS TM500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2791344 = {S73038, 142, { &P21262,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; AIRIS_TM500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2715173 = {S73039, 140, { &P22623,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Micromax A51 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2936653 = {S73040, 145, { &P22623,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Micromax A51 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2763692 = {S73041, 141, { &P18225,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; U9200 Build/HuaweiU9200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2782238 = {S73042, 146, { &P20705,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2937405 = {S73043, 145, { &P20705,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTCONE Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2894416 = {S73044, 89, { &P17595,&P17297,&P24329,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:22.0) Gecko/X Firefox/22.0 SeaMonkey/2.19*/
const Device UA2975014 = {S73045, 97, { &P21475,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A35) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA2898732 = {S73046, 94, { &P20134,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3081349 = {S73047, 137, { &P22999,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A870K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3082008 = {S73048, 137, { &P22999,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A870S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083433 = {S73049, 137, { &P22999,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A870L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA394945 = {S73050, 93, { &P15364,&P17017,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; BSD Four; C) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.3 Safari/533.3*/
const Device UA649922 = {S73051, 152, { &P13685,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LS670 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA737011 = {S73052, 145, { &P13685,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LS670 Build/GWK74; X-CM7.1.01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA846352 = {S73053, 152, { &P13685,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LS670 Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020088 = {S73054, 148, { &P13685,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-LS670 Build/GWK74; X-CM7.1.01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA657011 = {S73055, 142, { &P12726,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; SCH-i569 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023134 = {S73056, 142, { &P12726,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SCH-i569 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026470 = {S73057, 143, { &P21826,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-STYLE 1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2375973 = {S73058, 145, { &P21826,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-STYLE 1 i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026655 = {S73059, 138, { &P18722,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IM-A830L Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2028872 = {S73060, 138, { &P18722,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IM-A830S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2330877 = {S73061, 138, { &P18722,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IM-A830K Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3071710 = {S73062, 135, { &P18722,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SKY-A830 Build/LTE) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2032472 = {S73063, 135, { &P12848,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MB526 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2238416 = {S73064, 135, { &P12848,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MB526 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2704709 = {S73065, 141, { &P23373,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; QMobile_A7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3037936 = {S73066, 156, { &P22594,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3032606 = {S73067, 152, { &P18648,&P5043,&P24679,&P18092 }};	/*Useragent - Nokia306/2.0 (05.93) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia306) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA64025 = {S73068, 78, { &P18055,&P5208,&P4521,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Polaris 6.2; BREW 3.1.5; U; en)/240X400 LGE LG-LG8575*/
const Device UA409620 = {S73069, 134, { &P18262,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SH-04D Build/S1230) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA901320 = {S73070, 134, { &P18262,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SH-04D Build/S4130) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027110 = {S73071, 58, { &P18262,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SH-04D Build/S4130)*/
const Device UA2030925 = {S73072, 134, { &P18262,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SH-04D Build/S8100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA905930 = {S73073, 136, { &P20301,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; MIDC700PR001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA912138 = {S73074, 138, { &P20301,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; MIDC700PR001 Build/Polaroid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2443702 = {S73075, 136, { &P20301,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; MIDC700PR002 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2504327 = {S73076, 136, { &P20301,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; MIDC700PR001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1993405 = {S73077, 134, { &P20274,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IdeaTab_A1107 Build/MR1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2178671 = {S73078, 137, { &P21330,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PMP5197DULTRA Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2237628 = {S73079, 168, { &P15989,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST21i Build/11.0.A.0.16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2257854 = {S73080, 168, { &P15989,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST21i2 Build/11.0.A.1.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2425454 = {S73081, 139, { &P21651,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; TAB-PROTAB25XXL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2483668 = {S73082, 140, { &P21651,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TAB-PROTAB25XXL8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2594869 = {S73083, 141, { &P15364,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; G1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2768364 = {S73084, 150, { &P15364,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; E1-B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3007919 = {S73085, 152, { &P15364,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 3G7271 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3047191 = {S73086, 155, { &P15364,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AMAZE TAB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102993 = {S73087, 148, { &P15364,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SIMM-X722 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2703509 = {S73088, 142, { &P22810,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GOCLEVER TAB R106 Build/FR-MX68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2716906 = {S73089, 98, { &P15364,&P16110,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (6.2.9200.0);en;kml:2.2;client:Free;type:default)*/
const Device UA2799069 = {S73090, 98, { &P15364,&P16110,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1580(Windows;Microsoft Windows (6.2.9200.0);de;kml:2.2;client:Free;type:default)*/
const Device UA2944329 = {S73091, 98, { &P15364,&P16110,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1871(Windows;Microsoft Windows (6.2.9200.0);en;kml:2.2;client:Free;type:default)*/
const Device UA3018787 = {S73092, 98, { &P15364,&P16110,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1888(Windows;Microsoft Windows (6.2.9200.0);en;kml:2.2;client:Free;type:default)*/
const Device UA2754733 = {S73093, 167, { &P12731,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2796509 = {S73094, 180, { &P12731,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; Motorola Electrify Build/4.5.1A_SUN_USC_19.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453*/
const Device UA2756561 = {S73095, 138, { &P23905,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA10060 = {S73096, 88, { &P15526,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HD_mini_T5555; Windows Phone 6.5.3.5)*/
const Device UA28150 = {S73097, 92, { &P15526,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_HD_mini_T5555; Windows Phone 6.5.3.5)*/
const Device UA69727 = {S73098, 118, { &P15526,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD_mini/1.41.162.1 (10904); Windows Phone 6.5.3.5)*/
const Device UA438841 = {S73099, 118, { &P15526,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD_mini/1.11.164.1 (87733); Windows Phone 6.5.3.5)*/
const Device UA438858 = {S73100, 118, { &P15526,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD_mini/1.11.162.1 (87652); Windows Phone 6.5.3.5)*/
const Device UA438859 = {S73101, 96, { &P15526,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_HD_mini-orange-LS; Windows Phone 6.5.3.5)*/
const Device UA527638 = {S73102, 118, { &P15526,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD_mini/1.10.161.1 (86611); Windows Phone 6.5.3.5)*/
const Device UA776786 = {S73103, 118, { &P15526,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD_mini/1.36.184.2 (94538); Windows Phone 6.5.3.5)*/
const Device UA925717 = {S73104, 82, { &P15526,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HD_mini; Windows Phone 6.5.3.5)*/
const Device UA925722 = {S73105, 118, { &P15526,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Vodafone/1.0/HTC_HD_mini/1.41.161.1 (10904); Windows Phone 6.5.3.5)*/
const Device UA516901 = {S73106, 28, { &P18254,&P17017,&P21801,&P18092 }};	/*Useragent - Akregator/1.6.5; syndication*/
const Device UA623045 = {S73107, 28, { &P18254,&P17017,&P21801,&P18092 }};	/*Useragent - Akregator/1.6.6; syndication*/
const Device UA1996902 = {S73108, 139, { &P16553,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T959V Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1996948 = {S73109, 79, { &P15364,&P7133,&P21173,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; en; rv:2.0) Gecko/X Firefox/4.0 Opera 12.01*/
const Device UA2002927 = {S73110, 66, { &P15364,&P7133,&P21173,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; en) Opera 12.01*/
const Device UA2029562 = {S73111, 159, { &P15881,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Incredible 2 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2186465 = {S73112, 159, { &P15881,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Incredible 2 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2354708 = {S73113, 159, { &P15881,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Incredible 2 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2366817 = {S73114, 159, { &P15881,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Incredible 2 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2363931 = {S73115, 83, { &P21026,&P17979,&P18062,&P18092 }};	/*Useragent - ZTE U930_TD/1.0 Linux/2.6.39 Android/4.0 Release/3.5.2012 Browser/AppleWebKit534.30*/
const Device UA2954416 = {S73116, 85, { &P21026,&P17979,&P18062,&P18092 }};	/*Useragent - ZTEU930HD_TD/1.0 Linux/3.0.8 Android/4.0 Release/10.18.2012 Browser/AppleWebKit534.30*/
const Device UA2428176 = {S73117, 129, { &P15198,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Genio Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2429596 = {S73118, 155, { &P14711,&P5204,&P24675,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac2-01) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2552264 = {S73119, 132, { &P21480,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; PMP3270B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2950754 = {S73120, 61, { &P21480,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; PMP3270B Build/IML74K)*/
const Device UA2558984 = {S73121, 140, { &P23300,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; GLADIATOR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583172 = {S73122, 137, { &P18320,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2585175 = {S73123, 130, { &P18320,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667013 = {S73124, 180, { &P18604,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Smartphone Android by SFR STARADDICT II Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2705728 = {S73125, 159, { &P18604,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; STARADDICT II Plus Build/GXI_ZTEV1.0.0B02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958953 = {S73126, 159, { &P18604,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; STARADDICT II Plus Build/GXI_ZTEV1.0.0B03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2670613 = {S73127, 137, { &P19854,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R760 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2718810 = {S73128, 140, { &P20816,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; CINK SLIM Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2796542 = {S73129, 138, { &P22851,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT919 Build/2_290_2012) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2967582 = {S73130, 92, { &P13163,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaN97) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA2973691 = {S73131, 156, { &P14714,&P5843,&P24678,&P18092 }};	/*Useragent - NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax3-02) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3057126 = {S73132, 160, { &P14714,&P5843,&P24678,&P18092 }};	/*Useragent - NokiaX3-02.5/5.0 (07.58) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax3-02.5) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2967143 = {S73133, 160, { &P16362,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710BST Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2967550 = {S73134, 161, { &P16362,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710VMUB Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2967881 = {S73135, 157, { &P16362,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA17397 = {S73136, 101, { &P11919,&P16000,&P3772,&P18092 }};	/*Useragent - Opera 8.54 SonyEricssonM600i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 513) Opera 8.65 [en]*/
const Device UA43830 = {S73137, 100, { &P11919,&P16000,&P3772,&P18092 }};	/*Useragent - Opera 7.0 SonyEricssonM600i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 513) Opera 8.65 [en]*/
const Device UA61993 = {S73138, 90, { &P11919,&P16000,&P3772,&P18092 }};	/*Useragent - SonyEricssonM600i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 306) Opera 8.60 [en]*/
const Device UA63651 = {S73139, 90, { &P11919,&P16000,&P3772,&P18092 }};	/*Useragent - SonyEricssonM600i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 513) Opera 8.65 [en]*/
const Device UA54470 = {S73140, 141, { &P14223,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-ve; UM840 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA69380 = {S73141, 92, { &P16177,&P5448,&P4779,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA687328 = {S73142, 72, { &P16177,&P5448,&P4779,&P18092 }};	/*Useragent - Sprint T7380 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA183413 = {S73143, 10, { &P14033,&P5129,&P4030,&P18092 }};	/*Useragent - NokiaC5-03*/
const Device UA400769 = {S73144, 146, { &P14800,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; XT800W Build/TTSKT_U_80.33.41R) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA787304 = {S73145, 131, { &P14800,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; XT800 Build/TITA_M2_17.19.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1900115 = {S73146, 159, { &P14800,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MOT-XT800/TITA_M2_17.19.0; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018567 = {S73147, 70, { &P14800,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; XT800W Build/TTSKT_U_80.33.41R)*/
const Device UA2019389 = {S73148, 159, { &P14800,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MOT-XT800/TITA_M2_17.16.0; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022105 = {S73149, 143, { &P14800,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; XT800 Build/TITA_M2_17.19.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA547903 = {S73150, 86, { &P15364,&P17017,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; monitis - premium monitoring service; http://www.monitis.com)*/
const Device UA2141598 = {S73151, 132, { &P15364,&P5690,&P23738,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/527+ (KHTML, like Gecko) QtWeb Internet Browser/1.7 http://www.QtWeb.net*/
const Device UA2383640 = {S73152, 180, { &P12108,&P5843,&P24672,&P18092 }};	/*Useragent - Nokia7230/5.0 (06.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia7230) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile*/
const Device UA2413413 = {S73153, 138, { &P18697,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I415 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2421280 = {S73154, 130, { &P20246,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MID77C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2423201 = {S73155, 134, { &P20246,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID77C One Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2426196 = {S73156, 139, { &P20401,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HEU004D Build/HEU004D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2533275 = {S73157, 93, { &P18321,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5300) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2566926 = {S73158, 145, { &P18348,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2862579 = {S73159, 151, { &P18348,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G X710s Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566935 = {S73160, 138, { &P18594,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LS860 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2596588 = {S73161, 138, { &P23446,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Dialog i35 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2670627 = {S73162, 137, { &P18304,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9070 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2737279 = {S73163, 138, { &P18304,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9070P Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676485 = {S73164, 139, { &P20006,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30a Build/9.1.A.0.489) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2733795 = {S73165, 142, { &P12280,&P18432,&P22589,&P18092 }};	/*Useragent - MQQBrowser/42 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3*/
const Device UA2737799 = {S73166, 143, { &P12280,&P18432,&P22589,&P18092 }};	/*Useragent - MQQBrowser/421 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3*/
const Device UA2752419 = {S73167, 142, { &P12280,&P18432,&P22589,&P18092 }};	/*Useragent - MQQBrowser/421 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari/7534.48.3*/
const Device UA2752685 = {S73168, 141, { &P12280,&P18432,&P22589,&P18092 }};	/*Useragent - MQQBrowser/421 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3*/
const Device UA2768724 = {S73169, 142, { &P12280,&P18432,&P22589,&P18092 }};	/*Useragent - MQQBrowser/421 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3*/
const Device UA2859235 = {S73170, 141, { &P12280,&P18432,&P22589,&P18092 }};	/*Useragent - MQQBrowser/42 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3*/
const Device UA2861465 = {S73171, 141, { &P12280,&P18432,&P22589,&P18092 }};	/*Useragent - MQQBrowser/42 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari/7534.48.3*/
const Device UA2749277 = {S73172, 153, { &P22963,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030A Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3067176 = {S73173, 134, { &P20480,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; JY-G3 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA26126 = {S73174, 62, { &P12148,&P5776,&P4030,&P18092 }};	/*Useragent - Nokia6600f/2.0 (05.60) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA43641 = {S73175, 73, { &P12259,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S3550/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA50288 = {S73176, 138, { &P13218,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; Devour Build/DRC92) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA89792 = {S73177, 139, { &P13218,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; Devour Build/DRC92) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA244011 = {S73178, 135, { &P17937,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Triumph Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1934000 = {S73179, 59, { &P17937,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; Triumph Build/GRJ22)*/
const Device UA1974460 = {S73180, 153, { &P17937,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Triumph Build/GRJ22; Starship_Triumph) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA456955 = {S73181, 136, { &P17283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB860 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA582054 = {S73182, 136, { &P17283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MB860 Build/4.5.91) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA622346 = {S73183, 140, { &P17283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MB860 Build/4.5.91) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0 Mobile Safari/534.30*/
const Device UA820199 = {S73184, 136, { &P17283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MB860 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA892413 = {S73185, 136, { &P17283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB860 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA908383 = {S73186, 136, { &P17283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB860 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1964141 = {S73187, 136, { &P17283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB860 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455550 = {S73188, 138, { &P17283,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB860 Build/6.7.7_95) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2641479 = {S73189, 58, { &P17283,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MB860 Build/IMM76L)*/
const Device UA781662 = {S73190, 139, { &P14969,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC Desire S.M.S; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA810634 = {S73191, 140, { &P14969,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_DesireS_S510e; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA1907804 = {S73192, 141, { &P14969,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Desire_S_S510e; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA683410 = {S73193, 137, { &P24555,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; MTS-SP100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023199 = {S73194, 125, { &P24555,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; MTS-SP100 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA721825 = {S73195, 133, { &P23996,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A502 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA879346 = {S73196, 134, { &P23996,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; de-de; A502+ Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2009128 = {S73197, 122, { &P15364,&P5690,&P22434,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Comodo_Dragon/21.0.2.0*/
const Device UA2546530 = {S73198, 129, { &P15364,&P5690,&P22434,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Comodo_Dragon/21.0.2.0*/
const Device UA2453155 = {S73199, 131, { &P20190,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; U30GT-M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2424091 = {S73200, 70, { &P18442,&P18111,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.1.1; zh-cn; GT-N7100; 720*1280) UCWEB7.9.0.94/139/444*/
const Device UA2564204 = {S73201, 92, { &P11880,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA5320) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2602364 = {S73202, 155, { &P18464,&P18066,&P21708,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2670755 = {S73203, 137, { &P18316,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673540 = {S73204, 137, { &P18316,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2674207 = {S73205, 137, { &P18316,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735030 = {S73206, 138, { &P18316,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160LV Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735210 = {S73207, 140, { &P18316,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F160L/LV Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2970270 = {S73208, 114, { &P15364,&P7133,&P21934,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36 slurp*/
const Device UA3020307 = {S73209, 107, { &P15364,&P7133,&P21934,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.72 Safari/537.36 slurp*/
const Device UA3026029 = {S73210, 114, { &P15364,&P7133,&P21934,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 slurp*/
const Device UA3028236 = {S73211, 107, { &P15364,&P7133,&P21934,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 slurp*/
const Device UA20424 = {S73212, 121, { &P15364,&P22839,&P4223,&P18092 }};	/*Useragent - Zte-tu235B_TD/1.0 ThreadX/4.0b Larena/2.40 Release/10.10.2010 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA58813 = {S73213, 116, { &P15364,&P22839,&P4223,&P18092 }};	/*Useragent - Zte-tu230/1.0 ThreadX/4.0b Larena/2.40 Release/8.15.2009 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2303348 = {S73214, 116, { &P15364,&P22839,&P4223,&P18092 }};	/*Useragent - Zte-tu219_TD/1.0 ThreadX/4.0b L2K/4.0 Release/3.15.2011 Browser/NetFront3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1s*/
const Device UA2520647 = {S73215, 92, { &P15364,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-F116-Moldova-Moldtelecom/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2726720 = {S73216, 71, { &P15364,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-S50/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2978469 = {S73217, 80, { &P15364,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-Beeline_C100/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA3084714 = {S73218, 81, { &P15364,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-F116(ITA_H3G)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA3109037 = {S73219, 74, { &P15364,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-Z222/1.0.5 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA249243 = {S73220, 41, { &P11854,&P5890,&P24663,&P18092 }};	/*Useragent - NokiaE71/UC Browser7.9.0.102/28/355/UCWEB*/
const Device UA440425 = {S73221, 29, { &P11854,&P5890,&P24663,&P18092 }};	/*Useragent - NokiaE71/UCWEB7.9.0.94/28/999*/
const Device UA368942 = {S73222, 147, { &P17935,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PG06100 Build/GRI40; CM7.2.0-RM) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA735221 = {S73223, 150, { &P17935,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PG06100 Build/GRI40; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA858069 = {S73224, 154, { &P17935,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PG06100 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626893 = {S73225, 138, { &P23761,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; S170 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2642319 = {S73226, 62, { &P23761,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; S170 Build/GINGERBREAD)*/
const Device UA2715818 = {S73227, 76, { &P17595,&P17296,&P20794,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:7.0.1) Gecko/X Postbox/3.0.8*/
const Device UA663892 = {S73228, 125, { &P14887,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; GT-I9008L Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA722126 = {S73229, 137, { &P14887,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; GT-I9008L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1964122 = {S73230, 130, { &P14887,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; GT-I9008L Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2031931 = {S73231, 130, { &P14887,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; zh-cn; GT-I9008L Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2546151 = {S73232, 137, { &P14887,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; zh-cn; GT-I9008L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2978573 = {S73233, 61, { &P14887,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; GT-I9008L Build/FROYO)*/
const Device UA1958299 = {S73234, 133, { &P21239,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; M9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028907 = {S73235, 124, { &P21239,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; M9 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1977734 = {S73236, 131, { &P22099,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; PLAYTAB Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1983137 = {S73237, 133, { &P15102,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ605 Build/I.7.1-39.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2659296 = {S73238, 130, { &P22256,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; MW0831 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2734567 = {S73239, 140, { &P22979,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9128V Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865704 = {S73240, 213, { &P22979,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SAMSUNG-GT-I9128_TD/1.0 Android/4.1.2 Release/11.15.2012 Browser/AppleWebKit534.30 Build/JRO03L) ApplelWebkit/534.30 (KHTML,like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3072981 = {S73241, 62, { &P22979,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9128V Build/JZO54K)*/
const Device UA2773900 = {S73242, 159, { &P18223,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.0.400) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2901186 = {S73243, 138, { &P22970,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599V Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3007890 = {S73244, 131, { &P22970,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599V Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2975549 = {S73245, 130, { &P18475,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3020720 = {S73246, 131, { &P18475,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320T Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3037081 = {S73247, 156, { &P18283,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9300 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3081121 = {S73248, 141, { &P18465,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire2 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3083496 = {S73249, 139, { &P18465,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA9815 = {S73250, 68, { &P13442,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHE900/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61591 = {S73251, 137, { &P12749,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-ca; LG-CX670 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2462055 = {S73252, 166, { &P15919,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P3100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2624526 = {S73253, 166, { &P15919,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2630434 = {S73254, 167, { &P15919,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3100B Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2423529 = {S73255, 151, { &P18644,&P5043,&P24673,&P18092 }};	/*Useragent - Nokia112/2.0 (03.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia112) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2675105 = {S73256, 139, { &P23905,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100P Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2753420 = {S73257, 91, { &P15364,&P16110,&P22767,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Maxthon/4.0.6.2000)*/
const Device UA2937216 = {S73258, 170, { &P15364,&P16110,&P22767,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729; Maxthon/4.0.6.2000)*/
const Device UA2998623 = {S73259, 84, { &P15364,&P16110,&P22767,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; Maxthon/4.0.6.2000)*/
const Device UA2727635 = {S73260, 140, { &P24511,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; SHV-E270L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2800400 = {S73261, 136, { &P21006,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; L-05D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3089408 = {S73262, 58, { &P21006,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; L-05D Build/JZO54K)*/
const Device UA2974088 = {S73263, 148, { &P18227,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U9510E Build/HuaweiU9510E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976059 = {S73264, 127, { &P24047,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; nl-nl; M785 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3110816 = {S73265, 128, { &P24047,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; nl-nl; M785K Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3081058 = {S73266, 142, { &P15438,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Sensation Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3131331 = {S73267, 145, { &P18114,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Transformer Prime TF201 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA628938 = {S73268, 98, { &P12059,&P5204,&P17347,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2690/UC Browser8.0.3.107/69/444*/
const Device UA2412434 = {S73269, 251, { &P12059,&P5204,&P17347,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/69/444*/
const Device UA671893 = {S73270, 133, { &P23991,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; m7206 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022150 = {S73271, 57, { &P23991,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; m7206 Build/GRI40)*/
const Device UA628573 = {S73272, 138, { &P19758,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; PS47 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033192 = {S73273, 143, { &P24004,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Yan 8.1 Android 4.0 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2333396 = {S73274, 132, { &P24004,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Yan 8.1A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2417873 = {S73275, 131, { &P24004,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Yan8.1G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2520320 = {S73276, 131, { &P24004,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Yan 8.1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2508766 = {S73277, 39, { &P15364,&P18580,&P21570,&P18092 }};	/*Useragent - Mail/53 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2937138 = {S73278, 51, { &P15364,&P18580,&P21570,&P18092 }};	/*Useragent - webMailReader/1.3.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2428469 = {S73279, 139, { &P16273,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T679 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2700461 = {S73280, 139, { &P16273,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T679 Build/JZO86L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2983096 = {S73281, 61, { &P16273,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SGH-T679 Build/JZO54K)*/
const Device UA2557738 = {S73282, 145, { &P22690,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; POLYTRON W2500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2792576 = {S73283, 145, { &P22690,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; POLYTRON_W2500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580237 = {S73284, 136, { &P21903,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U705T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2763966 = {S73285, 58, { &P21903,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; U705T Build/IMM76D)*/
const Device UA2658856 = {S73286, 127, { &P24554,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N78 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2719710 = {S73287, 143, { &P22654,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C5502 Build/10.1.1.A.1.89) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719712 = {S73288, 144, { &P22654,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C5502 Build/10.1.1.A.1.273) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719715 = {S73289, 144, { &P22654,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C5502 Build/10.1.1.A.1.210) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719717 = {S73290, 144, { &P22654,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C5502 Build/10.1.1.A.1.164) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719725 = {S73291, 135, { &P22654,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; C5502 Build/10.1.1.A.1.164) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2719720 = {S73292, 144, { &P22654,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C5502 Build/10.1.1.A.1.191) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2939169 = {S73293, 66, { &P22654,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; C5502 Build/10.1.1.A.1.310)*/
const Device UA2999910 = {S73294, 144, { &P22654,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; C5502 Build/10.1.1.A.1.310) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2839787 = {S73295, 101, { &P15364,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.68; en-us; 8100/2.7.0.68) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2845855 = {S73296, 103, { &P15364,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.105; en-us; 8300/2.7.0.105) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2951125 = {S73297, 103, { &P15364,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.105; en-us; 8100/2.7.0.105) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2952173 = {S73298, 101, { &P15364,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.92; en-us; 8100/2.7.0.92) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2955022 = {S73299, 101, { &P15364,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.6.0.53; en-us; 8110/2.6.0.53) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2982274 = {S73300, 103, { &P15364,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.110; en-us; 8800/2.7.0.110) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2983542 = {S73301, 103, { &P15364,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.106; en-us; 8110/2.7.0.106) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3043036 = {S73302, 103, { &P15364,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.105; en-us; 8700/2.7.0.105) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3043170 = {S73303, 103, { &P15364,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.110; en-us; 8700/2.7.0.110) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3070685 = {S73304, 103, { &P15364,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.105; en-us; 8110/2.7.0.105) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3086437 = {S73305, 101, { &P15364,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 6.2.0.94; en-us; 9105/6.2.0.94) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3099462 = {S73306, 103, { &P15364,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.137; en-us; 9850/5.1.0.137) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3118742 = {S73307, 101, { &P15364,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.92; en-us; 8120/2.7.0.92) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2893388 = {S73308, 129, { &P23801,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; P774A Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2975554 = {S73309, 130, { &P22041,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6810 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2963425 = {S73310, 143, { &P24049,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; BLU DASH 4.5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3083256 = {S73311, 129, { &P20420,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; I10A-LE Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA44286 = {S73312, 46, { &P13819,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-MT375/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA903061 = {S73313, 138, { &P12482,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-T959 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA911669 = {S73314, 138, { &P12482,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T959 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2205630 = {S73315, 166, { &P18283,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2530068 = {S73316, 171, { &P21087,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; Lenovo A690 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/33482*/
const Device UA2565936 = {S73317, 155, { &P18465,&P18066,&P21418,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2583414 = {S73318, 139, { &P20124,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970M Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2600965 = {S73319, 140, { &P20124,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Grand X(M) Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2604499 = {S73320, 138, { &P20124,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2704185 = {S73321, 141, { &P20124,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ZTE Grand X Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2861423 = {S73322, 138, { &P20124,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ZTE V970 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2598985 = {S73323, 139, { &P21028,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Smartfren Tab 7 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2965384 = {S73324, 88, { &P15364,&P17285,&P24342,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; QQBrowser/7.3.11251.400)*/
const Device UA2770336 = {S73325, 139, { &P22932,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; AT-AS40D Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2774080 = {S73326, 145, { &P18556,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook HD+ Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2876894 = {S73327, 132, { &P18556,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; BN NookHD+ Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2879911 = {S73328, 133, { &P18673,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3031066 = {S73329, 165, { &P12657,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515m Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3035654 = {S73330, 156, { &P13107,&P5043,&P24679,&P18092 }};	/*Useragent - NokiaC2-06/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-06) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3096290 = {S73331, 154, { &P18635,&P21637,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus 4 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA23814 = {S73332, 70, { &P13670,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_S740 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA34318 = {S73333, 111, { &P13670,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Smartphone; 240x320; HTC S740; OpVer 32.174.1.611*/
const Device UA34317 = {S73334, 111, { &P13670,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Smartphone; 240x320; HTC S740; OpVer 32.117.1.612*/
const Device UA54350 = {S73335, 94, { &P13670,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Swisscom/1.0/HTC_S740/1.14.166.0*/
const Device UA55953 = {S73336, 111, { &P13670,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Smartphone; 240x320; HTC S740; OpVer 32.117.2.732*/
const Device UA61038 = {S73337, 94, { &P13670,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_S740/1.14.172.1*/
const Device UA68096 = {S73338, 95, { &P14904,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Mazaa)*/
const Device UA251976 = {S73339, 63, { &P15364,&P7127,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.9.168 Version/11.52*/
const Device UA2418477 = {S73340, 83, { &P20123,&P17979,&P18062,&P18092 }};	/*Useragent - ZTEU795_TD/1.0 Linux/2.6.39 Android/4.0 Release/6.10.2012 Browser/AppleWebKit534.30*/
const Device UA2764325 = {S73341, 84, { &P20123,&P17979,&P18062,&P18092 }};	/*Useragent - ZTEU795+_TD/1.0 Linux/3.0.13 Android/4.0 Release/7.10.2012 Browser/AppleWebKit534.30*/
const Device UA2526680 = {S73342, 140, { &P20248,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZTE V889M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3070268 = {S73343, 62, { &P20248,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ZTE V889M Build/IMM76D)*/
const Device UA2515659 = {S73344, 43, { &P17595,&P17297,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.0.X (Intel Mac OS X 10.7.5)*/
const Device UA2536411 = {S73345, 43, { &P17595,&P17297,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.1.X (Intel Mac OS X 10.7.5)*/
const Device UA2584238 = {S73346, 43, { &P17595,&P17297,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.2.X (Intel Mac OS X 10.7.5)*/
const Device UA2739941 = {S73347, 43, { &P17595,&P17297,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.4.X (Intel Mac OS X 10.7.5)*/
const Device UA2869371 = {S73348, 43, { &P17595,&P17297,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.5.X (Intel Mac OS X 10.7.5)*/
const Device UA3019791 = {S73349, 43, { &P17595,&P17297,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.6.X (Intel Mac OS X 10.7.5)*/
const Device UA3109410 = {S73350, 43, { &P17595,&P17297,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.7.X (Intel Mac OS X 10.7.5)*/
const Device UA2532389 = {S73351, 136, { &P21160,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; P4013 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2582117 = {S73352, 150, { &P23016,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Spice Mi-351 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2666887 = {S73353, 130, { &P15767,&P17090,&P22901,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.3 Mobile/8L1 Safari/6533.18.5*/
const Device UA2667057 = {S73354, 130, { &P15767,&P17090,&P22901,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.3 Mobile/8J2 Safari/6533.18.5*/
const Device UA2667669 = {S73355, 130, { &P15767,&P17090,&P22901,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.3 Mobile/8H7 Safari/6533.18.5*/
const Device UA2752981 = {S73356, 140, { &P18183,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ615-12) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2756040 = {S73357, 133, { &P19788,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; E350 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869845 = {S73358, 133, { &P19788,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; E350 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2966966 = {S73359, 102, { &P18218,&P21697,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 610) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile*/
const Device UA2791677 = {S73360, 58, { &P15364,&P5448,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; WindowsCE; Trident/2.0)*/
const Device UA2850905 = {S73361, 63, { &P15364,&P5448,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; 68K; Tablet PC; Trident/2.0)*/
const Device UA2963662 = {S73362, 152, { &P15241,&P5843,&P24678,&P18092 }};	/*Useragent - Nokia300/5.0 (07.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia300) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3130995 = {S73363, 139, { &P20289,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3080330 = {S73364, 138, { &P13109,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC-X710a Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083597 = {S73365, 136, { &P22651,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E435 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083930 = {S73366, 137, { &P22651,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E435f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126237 = {S73367, 137, { &P22651,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E435g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3130726 = {S73368, 138, { &P12968,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.1.96) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3130728 = {S73369, 130, { &P22228,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SM-T210R Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3132863 = {S73370, 129, { &P22228,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SM-T210 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA15655 = {S73371, 61, { &P11829,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia6301/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA30654 = {S73372, 60, { &P14222,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; XV6175.1)*/
const Device UA70923 = {S73373, 149, { &P21142,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-pt; Optimus Monte Carlo Build/OPT_B02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA696065 = {S73374, 149, { &P21142,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-pt; Optimus Monte Carlo Build/OPT_B03) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1944934 = {S73375, 149, { &P21142,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-pt; Optimus Monte Carlo Build/OPT_B04) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025978 = {S73376, 149, { &P21142,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-pt; Optimus Monte Carlo Build/OPT_B05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034632 = {S73377, 149, { &P21142,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-pt; Optimus Monte Carlo Build/OPT_B06) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA348688 = {S73378, 116, { &P15364,&P18110,&P22453,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.2 Chrome/15.0.900.2 Safari/535.2*/
const Device UA349877 = {S73379, 114, { &P15364,&P18110,&P22453,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.2 Chrome/15.0.900.2 Safari/535.2*/
const Device UA645692 = {S73380, 93, { &P15364,&P5690,&P22482,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6*/
const Device UA697567 = {S73381, 95, { &P15364,&P5690,&P22482,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6.2*/
const Device UA890878 = {S73382, 148, { &P21140,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-pt; Optimus_Madrid Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2421712 = {S73383, 128, { &P20234,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EXP8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2450444 = {S73384, 128, { &P20234,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; EXP8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422653 = {S73385, 141, { &P20131,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Thrill 430x Build/FSV04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2557420 = {S73386, 141, { &P20131,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Thrill 430x Build/FSV06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2766113 = {S73387, 141, { &P20131,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Thrill 430x Build/FSV10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2560369 = {S73388, 92, { &P14359,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5228) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2558246 = {S73389, 128, { &P23232,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; F7HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2661459 = {S73390, 142, { &P21873,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GOCLEVER TAB M713G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2672586 = {S73391, 129, { &P22904,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MP888 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2755194 = {S73392, 126, { &P15602,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A501 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2769379 = {S73393, 138, { &P23159,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-S960L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA24744 = {S73394, 30, { &P13398,&P17484,&P3430,&P18092 }};	/*Useragent - SAMSUNG-GT-B7610/1.0 Opera 9.5*/
const Device UA702361 = {S73395, 94, { &P13398,&P17484,&P3430,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-GT-B7610/NBUIK3 Browser/Opera/9.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA68575 = {S73396, 101, { &P17968,&P5477,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.1-update1; Linux; Opera Mobi/ADR-1105231029; U; en) Presto/2.7.81 Version/11.00*/
const Device UA721919 = {S73397, 144, { &P21989,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; LENCM900HZ Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA845131 = {S73398, 172, { &P16745,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; T-Mobile myTouch 3G Slide Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2404187 = {S73399, 137, { &P20966,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; T-01D Build/V09R40A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3127202 = {S73400, 137, { &P20966,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; T-01D Build/V11R42A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2431590 = {S73401, 170, { &P15492,&P17484,&P17373,&P18092 }};	/*Useragent - SAMSUNG-GT-C3322/C3322DDLF3 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0 Opera/9.80 (J2ME/MIDP; Opera Mini/6.5.28159/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2534670 = {S73402, 166, { &P12280,&P19826,&P23594,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko, M5WebKit/1.0.1, M5HybridSDK/1.0.0) Version/5.1.0 Mobile/6.1 Mammoth/5.1.0*/
const Device UA2566081 = {S73403, 170, { &P12280,&P19826,&P23594,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko, M5WebKit/1.0.1, M5HybridSDK/1.0.0) Version/5.1.0 Mobile/6.1.2 Mammoth/5.1.0*/
const Device UA2672550 = {S73404, 170, { &P12280,&P19826,&P23594,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko, M5WebKit/1.0.2, M5HybridSDK/1.0.0) Version/5.1.1 Mobile/6.1.3 Mammoth/5.1.1*/
const Device UA2780896 = {S73405, 170, { &P12280,&P19826,&P23594,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko, M5WebKit/1.0.2, M5HybridSDK/1.0.0) Version/5.1.1 Mobile/6.1.2 Mammoth/5.1.1*/
const Device UA2862373 = {S73406, 166, { &P12280,&P19826,&P23594,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko, M5WebKit/1.0.2, M5HybridSDK/1.0.0) Version/5.1.1 Mobile/6.1 Mammoth/5.1.1*/
const Device UA2868156 = {S73407, 170, { &P12280,&P19826,&P23594,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko, M5WebKit/1.0.2, M5HybridSDK/1.0.0) Version/5.1.1 Mobile/6.1.4 Mammoth/5.1.1*/
const Device UA2570160 = {S73408, 105, { &P18436,&P16110,&P21918,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; chromeframe/25.0.1364.172)*/
const Device UA2559761 = {S73409, 131, { &P21983,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; DIT8070 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2567159 = {S73410, 133, { &P17595,&P18207,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Version/6.0.2 Safari/8536.26.17 Sleipnir/4.1*/
const Device UA2601176 = {S73411, 135, { &P17595,&P18207,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) Version/6.0.3 Safari/8536.28.10 Sleipnir/4.1.2*/
const Device UA2679267 = {S73412, 135, { &P17595,&P18207,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) Version/6.0.4 Safari/8536.29.13 Sleipnir/4.1.2*/
const Device UA2637359 = {S73413, 149, { &P15985,&P6437,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.2; GT-P6200 Build/HTJ85B) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.53463*/
const Device UA2774059 = {S73414, 148, { &P15985,&P6437,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.2; GT-P6200 Build/HTJ85B) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2755412 = {S73415, 136, { &P18234,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P895 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867965 = {S73416, 138, { &P18234,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P895qb Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879903 = {S73417, 138, { &P13123,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3089656 = {S73418, 138, { &P13123,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3031504 = {S73419, 159, { &P18564,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X+ Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2974086 = {S73420, 137, { &P22599,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2961368 = {S73421, 154, { &P17475,&P5768,&P24678,&P18092 }};	/*Useragent - Nokia6300/2.0 (07.00) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6300) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3056835 = {S73422, 92, { &P17475,&P5768,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6300) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2972275 = {S73423, 92, { &P14454,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE66) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA2974995 = {S73424, 138, { &P22729,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT918 Build/2_26D_2009) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3081116 = {S73425, 141, { &P20399,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP8010 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068396 = {S73426, 133, { &P20156,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; S500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA61983 = {S73427, 70, { &P12274,&P5448,&P3430,&P18092 }};	/*Useragent - SAMSUNG-SGH-i900/1.0 (compatible; MSIE 6.0; Windows CE; PPC) Opera 9.5*/
const Device UA54392 = {S73428, 65, { &P14858,&P17483,&P17602,&P18092 }};	/*Useragent - LG-C100/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA227121 = {S73429, 145, { &P12871,&P17295,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7 HTC_WildfireS-orange-LS; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA902021 = {S73430, 74, { &P15364,&P18110,&P21170,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:13.0) Gecko/X Firefox/13.0 SeaMonkey/2.10*/
const Device UA2188641 = {S73431, 85, { &P14010,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s5263) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2029984 = {S73432, 141, { &P24101,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR930L 4G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2760633 = {S73433, 138, { &P24101,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR930L Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2233385 = {S73434, 171, { &P13974,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2234487 = {S73435, 171, { &P13974,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2385233 = {S73436, 119, { &P14984,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5250) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/444/UCWEB Mobile*/
const Device UA2256939 = {S73437, 140, { &P21100,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Q-Smart S12 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2987548 = {S73438, 140, { &P21100,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Q-Smart S12 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2406756 = {S73439, 119, { &P14753,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; Nokia N96) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile*/
const Device UA2698186 = {S73440, 41, { &P14753,&P5890,&P24673,&P18092 }};	/*Useragent - Nokia N96UCBrowser/8.7.1.234/28/444/UCWEB*/
const Device UA2984188 = {S73441, 92, { &P14753,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokian96-1) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2466653 = {S73442, 149, { &P21402,&P17979,&P18062,&P18092 }};	/*Useragent - Lenovo S890/S100 Linux/3.0.13 Android/4.0.3 Release/12.12.2011 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30*/
const Device UA2510607 = {S73443, 137, { &P12280,&P19826,&P18355,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Safari/7534.48.3 Sleipnir/2.2m*/
const Device UA2511048 = {S73444, 137, { &P12280,&P19826,&P18355,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 Safari/7534.48.3 Sleipnir/2.2m*/
const Device UA2533372 = {S73445, 139, { &P12280,&P19826,&P18355,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.2m*/
const Device UA2536831 = {S73446, 139, { &P12280,&P19826,&P18355,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B145 Safari/7534.48.3 Sleipnir/2.2m*/
const Device UA2581940 = {S73447, 137, { &P12280,&P19826,&P18355,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 Safari/7534.48.3 Sleipnir/2.2m*/
const Device UA2606832 = {S73448, 139, { &P12280,&P19826,&P18355,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.2m*/
const Device UA2566941 = {S73449, 128, { &P18704,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT200 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2568288 = {S73450, 137, { &P12280,&P17090,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 4_3_3 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2599484 = {S73451, 137, { &P12280,&P17090,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 4_3_5 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2857742 = {S73452, 137, { &P12280,&P17090,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 4_3_1 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA3071937 = {S73453, 135, { &P12280,&P17090,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 4_3 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2580256 = {S73454, 153, { &P24037,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Allview_TX1_Quasar 4.0.4; ro-ro; ALLVIEW_TX1_Quasar Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2666896 = {S73455, 147, { &P24037,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Allview_TX1_Quasar 4.0.4; ru-ru; Perfeo_7777W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2935926 = {S73456, 159, { &P24037,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Allview_TX1_Quasar 4.0.4; ru-ru; Inch Regulus (ITWGN7004) Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3057521 = {S73457, 138, { &P24037,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Allview_TX1_Quasar 4.1.2; ru-ru; M73 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3086691 = {S73458, 153, { &P24037,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Allview_TX1_Quasar 4.0.4; bg-bg; ALLVIEW_TX1_Quasar Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2672454 = {S73459, 137, { &P20973,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LW770 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2773660 = {S73460, 156, { &P22609,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6312 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2784102 = {S73461, 156, { &P18595,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8190 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2800093 = {S73462, 156, { &P18595,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; GT-I8190 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2751322 = {S73463, 142, { &P21703,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ARCHOS 70 Titanium Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2754626 = {S73464, 134, { &P18299,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP4100 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA126239 = {S73465, 75, { &P18254,&P7480,&P17190,&P18091 }};	/*Useragent - Huaweisymantecspider (compatible; MSIE 8.0; DSE-support@huaweisymantec.com)*/
const Device UA2382866 = {S73466, 82, { &P17595,&P17297,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5; U; en) Presto/2.10.229 Version/11.62*/
const Device UA779192 = {S73467, 80, { &P15373,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5530)/UC Browser8.2.0.132/50/355/UCWEB Mobile*/
const Device UA1947509 = {S73468, 31, { &P15373,&P5129,&P24667,&P18092 }};	/*Useragent - Nokia5530/UCWEB8.2.0.116/50/999*/
const Device UA2403471 = {S73469, 69, { &P17324,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; GT-I9001; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA912341 = {S73470, 190, { &P18283,&P18066,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, gb*/
const Device UA1966200 = {S73471, 190, { &P18283,&P18066,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I535 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, us*/
const Device UA1978560 = {S73472, 191, { &P18283,&P18066,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; GT-I9300T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, au*/
const Device UA1994432 = {S73473, 190, { &P18283,&P18066,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T999 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, us*/
const Device UA2028152 = {S73474, 137, { &P20472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-12D Build/V15R28C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030762 = {S73475, 137, { &P20472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-12D Build/V18R30C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2427282 = {S73476, 137, { &P20472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-12D Build/V28R34B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2755437 = {S73477, 137, { &P20472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-12D Build/V30R35B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079697 = {S73478, 137, { &P20472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-12D Build/V31R36B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2233478 = {S73479, 132, { &P21835,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; M-MP815I Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2349222 = {S73480, 135, { &P20343,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZP900 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2416857 = {S73481, 136, { &P20343,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZP900S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2382060 = {S73482, 147, { &P21632,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-mobile i-STYLE 4 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2382824 = {S73483, 148, { &P21632,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-mobile i-STYLE 4i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2505356 = {S73484, 120, { &P14983,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaC7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2459057 = {S73485, 121, { &P12671,&P5055,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; Nokia700) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile*/
const Device UA2424749 = {S73486, 129, { &P20155,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; T-909 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2552934 = {S73487, 127, { &P20155,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; T-909 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2956954 = {S73488, 130, { &P20155,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; T-909S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2427402 = {S73489, 145, { &P20545,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Maxwell Build/v1.0.0.ICS.maxwell.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2484217 = {S73490, 139, { &P20545,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Maxwell Build/v1.0.2 X-09:29) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2762964 = {S73491, 138, { &P20545,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Maxwell Build/1.0.2.X-17:05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2427719 = {S73492, 136, { &P21366,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; SKYTEX-SP712 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2488393 = {S73493, 92, { &P11945,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233s) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2580164 = {S73494, 89, { &P11945,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; vi; gt-s5233s) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2580961 = {S73495, 85, { &P11945,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233s) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0*/
const Device UA2872453 = {S73496, 93, { &P11945,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; ?en-US; gt-s5233s) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2671580 = {S73497, 158, { &P16362,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2708401 = {S73498, 161, { &P16362,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710BST Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2567373 = {S73499, 135, { &P18299,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP4100 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2579882 = {S73500, 138, { &P23822,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Idea Zeal Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3022192 = {S73501, 135, { &P18564,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; One X+ Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024511 = {S73502, 138, { &P18564,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One X+ Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3081341 = {S73503, 137, { &P14959,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9103 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA54473 = {S73504, 141, { &P14224,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-mx; HTC_IncredibleC Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90109 = {S73505, 121, { &P15364,&P22857,&P3807,&P18092 }};	/*Useragent - LCT_X880B_01/1.0 Nucleus/1.13.21 MTK6225/07A Release/5.1.2008 Browser/TelecaQ03C1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA632367 = {S73506, 122, { &P15364,&P22857,&P3807,&P18092 }};	/*Useragent - Beetel GD777/1.0 Nucleus/1.13.21 MTK6236/11A Release/03.30.2011 Browser/TelecaQ03C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA844306 = {S73507, 121, { &P15364,&P22857,&P3807,&P18092 }};	/*Useragent - LCT_D800B_01/1.0 Nucleus/1.13.21 MTK6225/07A Release/5.1.2008 Browser/TelecaQ03C1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2265963 = {S73508, 73, { &P15364,&P22857,&P3807,&P18092 }};	/*Useragent - LENOVO-E156/S005 Nucleus/1.15 MTK/23C Release/07.01.2010 Browser/Obigo03C*/
const Device UA2697298 = {S73509, 44, { &P15364,&P22857,&P3807,&P18092 }};	/*Useragent - azumi l2 OBIGO/Q03C (MTK; Nucleus; U; es-GT)*/
const Device UA2791437 = {S73510, 125, { &P15364,&P22857,&P3807,&P18092 }};	/*Useragent - HUAWEI-G5520/1.0 MTK6252C/Nucleus 1.0 Release/05.21.2012 Browser/OBIGO-Q03C WAP-Browser 1.0  Profile/MIDP-2.0 Config/CLDC-1.1*/
const Device UA2843936 = {S73511, 111, { &P15364,&P22857,&P3807,&P18092 }};	/*Useragent - Azumi Tzuki_Slim Obigo/Q03C (MTK; Nucleus; es-CL) 		 Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA92870 = {S73512, 56, { &P15364,&P17188,&P17387,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 4.0; obot)*/
const Device UA141088 = {S73513, 124, { &P15364,&P24094,&P18407,&P18092 }};	/*Useragent - Opera/9.70 (Linux mips ; U; CE-HTML/1.0 (<profilelist><ui_profile name="PHILIPS_OLS_2010"/></profilelist>); en) Presto/2.2.1*/
const Device UA2032372 = {S73514, 133, { &P24009,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; M005 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2424239 = {S73515, 137, { &P18276,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-08D Build/V05R11A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3126188 = {S73516, 137, { &P18276,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-08D Build/V07R13A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2429591 = {S73517, 151, { &P18643,&P5043,&P24675,&P18092 }};	/*Useragent - Nokia311/5.0 (03.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia311) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2676192 = {S73518, 151, { &P20017,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2706668 = {S73519, 150, { &P20017,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677298 = {S73520, 132, { &P13644,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; EVO Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2774073 = {S73521, 188, { &P13157,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2893198 = {S73522, 182, { &P13157,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2772758 = {S73523, 157, { &P19861,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA416874 = {S73524, 136, { &P20144,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-tw; GT-I5508 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA638738 = {S73525, 122, { &P20144,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-I5508 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA779258 = {S73526, 134, { &P20144,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-I5508 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019514 = {S73527, 58, { &P20144,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; GT-I5508 Build/FROYO)*/
const Device UA2025846 = {S73528, 127, { &P20144,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-I5508 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1887308 = {S73529, 72, { &P15364,&P7127,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:5.0) Gecko/X Firefox/5.0 SmartLinksAddon*/
const Device UA2593541 = {S73530, 72, { &P15364,&P7127,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; eo; rv:5.0) Gecko/X Firefox/5.0*/
const Device UA1901127 = {S73531, 74, { &P13815,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.1; zh-cn; HTC_Wildfire; 240*320) UCWEB7.9.3.103/139/800*/
const Device UA2299578 = {S73532, 74, { &P13815,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.1; zh-cn; HTC_Wildfire; 240*320) UCWEB7.9.3.103/140/999*/
const Device UA2384261 = {S73533, 73, { &P13815,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.1; zh-cn; HTC_Wildfire; 240*320) UCWEB7.9.0.94/139/352*/
const Device UA2401215 = {S73534, 73, { &P13815,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.1; zh-cn; HTC_Wildfire; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2427217 = {S73535, 73, { &P13815,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.1; zh-cn; HTC_Wildfire; 240*320) UCWEB7.9.0.94/139/355*/
const Device UA2457176 = {S73536, 73, { &P13815,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.7; zh-cn; HTC_Wildfire; 240*320) UCWEB7.9.0.94/139/352*/
const Device UA2721358 = {S73537, 74, { &P13815,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.1; zh-cn; HTC_Wildfire; 240*320) UCWEB7.9.4.145/139/800*/
const Device UA2778094 = {S73538, 73, { &P13815,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.7; zh-cn; HTC_Wildfire; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2199382 = {S73539, 125, { &P15655,&P17484,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s3850) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile*/
const Device UA2242236 = {S73540, 37, { &P18254,&P18580,&P18245,&P18092 }};	/*Useragent - Bing/3.03 CFNetwork/609 Darwin/13.0.0*/
const Device UA2262469 = {S73541, 44, { &P18254,&P18580,&P18245,&P18092 }};	/*Useragent - StumbleUpon/2.52 CFNetwork/609 Darwin/13.0.0*/
const Device UA2418058 = {S73542, 91, { &P24027,&P5629,&P15760,&P18092 }};	/*Useragent - CoolPad8020_CMCC_TD/1.0 Linux/2.6.35 Android/2.3 Release/6.15.2012 Browser/AppleWebkit533.1*/
const Device UA2597596 = {S73543, 133, { &P24027,&P5629,&P15760,&P18092 }};	/*Useragent - CoolPad8020+_CMCC_TD/1.08 Linux/2.6.35 Android/2.3.5 Release/03.29.2012 Mozilla/5.0 AppleWebKit/533.1 Version/4.0 Mobile Safari/533.1*/
const Device UA2504597 = {S73544, 141, { &P15759,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; HTC Vision Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2599295 = {S73545, 140, { &P15759,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC Vision Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952649 = {S73546, 62, { &P15759,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC Vision Build/JDQ39)*/
const Device UA2556801 = {S73547, 138, { &P23063,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; W110 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2567332 = {S73548, 130, { &P15876,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5100 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2635895 = {S73549, 131, { &P15876,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P5100 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2637943 = {S73550, 137, { &P15876,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5100 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2707620 = {S73551, 140, { &P22934,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.X; de-de; Xelio 10 Pro Build/ODYS_Xelio) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3031266 = {S73552, 158, { &P18296,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One V Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3021335 = {S73553, 93, { &P22872,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S5282) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3080015 = {S73554, 134, { &P23667,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; JY-G4 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3068326 = {S73555, 141, { &P21374,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 101 Titanium Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA4147 = {S73556, 135, { &P14997,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ru-ru; Liquid Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA857021 = {S73557, 134, { &P14997,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; hd-us; Liquid Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2745988 = {S73558, 140, { &P14997,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; Liquid Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20153 = {S73559, 52, { &P15311,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-VE440/00.72 UP.Browser/7.2.7.5.634 (GUI) MMP/2.0*/
const Device UA21641 = {S73560, 57, { &P15311,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-VE440-parrot/1.0 UP.Browser/7.2.7.5.656 (GUI) MMP/2.0*/
const Device UA49225 = {S73561, 139, { &P12964,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC-S710d/2.13.1401.4) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72958 = {S73562, 139, { &P12964,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-cn; HTC-S710d/3.06.1401.0) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA666596 = {S73563, 127, { &P12964,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC S710d) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA683087 = {S73564, 125, { &P12964,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC S710d Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA697305 = {S73565, 139, { &P12964,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC-S710d/3.06.1401.3) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA757357 = {S73566, 139, { &P12964,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC-S710d/3.07.1401.3) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1986032 = {S73567, 150, { &P12964,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; HTC_IncredibleS_S710d Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024504 = {S73568, 130, { &P12964,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC S710d Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2032723 = {S73569, 150, { &P12964,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; HTC IncredibleS S710d Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA285904 = {S73570, 130, { &P15364,&P17285,&P22445,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Iron/7.0.520.0 Chrome/7.0.520.0 Safari/534.7*/
const Device UA471470 = {S73571, 89, { &P15364,&P7133,&P18199,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.8.1.23) Gecko/X Thunderbird/2.0.0.23*/
const Device UA551291 = {S73572, 87, { &P17595,&P17294,&P19773,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.4; de; rv:1.9.1.19) Gecko/X SeaMonkey/2.0.14*/
const Device UA2185335 = {S73573, 148, { &P21205,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Andi107 Build/iBalliBall_Andi107) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2338453 = {S73574, 114, { &P23936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn;YL-Coolpad_7230/2.3.006.X.7230; 480*800; CUCC/3.0) CoolpadWebkit/533.1*/
const Device UA2578195 = {S73575, 145, { &P23936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_7230-B Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2763443 = {S73576, 158, { &P23936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Coolpad 7230 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) FlyFlow/2.2 Version/4.0 Mobile Safari/533.1*/
const Device UA2835749 = {S73577, 143, { &P23936,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_7230 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2692398 = {S73578, 153, { &P24547,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.9; en-us; SPHS on Hsdroid Build/MocorDroid2.2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2702435 = {S73579, 153, { &P24547,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SPHS on Hsdroid Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2748270 = {S73580, 153, { &P24547,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPHS on Hsdroid Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2533213 = {S73581, 70, { &P17346,&P18117,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPod4,1; iPhone OS 5.1.1; en_US; en) AppleWebKit/420+*/
const Device UA2700578 = {S73582, 68, { &P17346,&P18117,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPod4,1; iPhone OS 5.1; en_US; en) AppleWebKit/420+*/
const Device UA2506557 = {S73583, 136, { &P15365,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Magic Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099870 = {S73584, 58, { &P15365,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Magic Build/IMM76D)*/
const Device UA2584101 = {S73585, 137, { &P18438,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E973 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2601374 = {S73586, 138, { &P18438,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F200K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667284 = {S73587, 138, { &P18438,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F200S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2857763 = {S73588, 130, { &P18438,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E973 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2602913 = {S73589, 130, { &P15767,&P19826,&P21783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.3m*/
const Device UA2679634 = {S73590, 132, { &P15767,&P19826,&P21783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.3.1m*/
const Device UA2716114 = {S73591, 132, { &P15767,&P19826,&P21783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.3.2m*/
const Device UA2860960 = {S73592, 130, { &P15767,&P19826,&P21783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.3m*/
const Device UA2865851 = {S73593, 132, { &P15767,&P19826,&P21783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.3.2m*/
const Device UA2659425 = {S73594, 130, { &P23899,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT8000 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2674740 = {S73595, 125, { &P16598,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; A500 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3021289 = {S73596, 133, { &P16598,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; A500 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675357 = {S73597, 135, { &P21088,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Stealth 9.7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2755066 = {S73598, 138, { &P22848,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT920 Build/2_290_2009) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2975524 = {S73599, 136, { &P19917,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P765 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2961306 = {S73600, 154, { &P24724,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PAP3540DUO Build/PrestigioPAP3540DUO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999624 = {S73601, 129, { &P24367,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5200 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3028311 = {S73602, 135, { &P18645,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-I9305 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3036003 = {S73603, 143, { &P18645,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SAMSUNG-SGH-I747 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3036014 = {S73604, 135, { &P18645,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SPH-L710 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA68553 = {S73605, 35, { &P14188,&P5129,&P4777,&P18092 }};	/*Useragent - Nokia5233/UC Browser7.7.1.88/50/444*/
const Device UA79860 = {S73606, 31, { &P15364,&P7133,&P4811,&P18092 }};	/*Useragent - Opera/9 (Windows NT 6.1) Presto*/
const Device UA172071 = {S73607, 34, { &P15364,&P7133,&P4811,&P18092 }};	/*Useragent - Opera/9.00 (Windows NT 6.1; U; en)*/
const Device UA525415 = {S73608, 34, { &P15364,&P7133,&P4811,&P18092 }};	/*Useragent - Opera/9.02 (Windows NT 6.1; U; en)*/
const Device UA710676 = {S73609, 33, { &P15364,&P7133,&P4811,&P18092 }};	/*Useragent - Opera/9.0 (Windows NT 6.1; U; en)*/
const Device UA171170 = {S73610, 33, { &P15364,&P17350,&P18404,&P18092 }};	/*Useragent - Opera 9.4 (Windows NT 5.3; U; en)*/
const Device UA822347 = {S73611, 94, { &P15364,&P18110,&P18197,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.11) Gecko/X Lightning/1.0b1 Thunderbird/3.0.6*/
const Device UA899349 = {S73612, 90, { &P15364,&P22196,&P18565,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) KHTML/4.8.3 (like Gecko) Konqueror/4.8 Fedora/4.8.3-1.fc17*/
const Device UA1988714 = {S73613, 88, { &P15364,&P22196,&P18565,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) KHTML/4.8.3 (like Gecko) Konqueror/4.8 Fedora/4.8.3-1.fc17*/
const Device UA2415323 = {S73614, 142, { &P21875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GFIVE Bravo Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2891278 = {S73615, 172, { &P21875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GFIVE Bravo(G95) Build/GFIVE_BRAVO_G95-V1.0-2012.12.21) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3042844 = {S73616, 140, { &P21875,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; GFIVE G95 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2427138 = {S73617, 135, { &P20862,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IRULU-AL101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2560181 = {S73618, 168, { &P11787,&P5204,&P24676,&P18092 }};	/*Useragent - Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3120classic) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2566682 = {S73619, 138, { &P18304,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9070 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2582928 = {S73620, 135, { &P23960,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T1-B Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2675069 = {S73621, 140, { &P23910,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT30at Build/7.0.B.1.152) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2718792 = {S73622, 154, { &P12059,&P5204,&P24677,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2690) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2754710 = {S73623, 135, { &P19784,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LIFETAB_P9514 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2792478 = {S73624, 138, { &P23234,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E976 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994696 = {S73625, 60, { &P23234,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E976 Build/JZO54K)*/
const Device UA3039276 = {S73626, 155, { &P21648,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9505 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3059361 = {S73627, 139, { &P17172,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Transformer TF101 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3080974 = {S73628, 139, { &P17172,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Transformer TF101 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3096011 = {S73629, 112, { &P21003,&P20706,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia310/07.35; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA16191 = {S73630, 90, { &P13544,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-255/1.0 ObigoInternetBrowser/Q03C, Alcatel-OT-255/1.0 ObigoInternetBrowser/Q03C*/
const Device UA24090 = {S73631, 45, { &P13544,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-255A/1.0 ObigoInternetBrowser/Q03C*/
const Device UA53725 = {S73632, 44, { &P13544,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-255/1.0 ObigoInternetBrowser/Q03C*/
const Device UA62809 = {S73633, 45, { &P13544,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-255D/1.0 ObigoInternetBrowser/Q03C*/
const Device UA63017 = {S73634, 83, { &P11715,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW550c/R4CB Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64521 = {S73635, 83, { &P11715,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW550i/R4CB Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA656745 = {S73636, 100, { &P15228,&P5043,&P17347,&P18092 }};	/*Useragent - NokiaC2-03/2.0 (06.51) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-03/UC Browser8.0.3.107/69/444*/
const Device UA493693 = {S73637, 83, { &P15364,&P18110,&P22480,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.3) Gecko/X Firefox/3.0.3 Flock/2.0*/
const Device UA924067 = {S73638, 141, { &P12280,&P18117,&P20786,&P18092 }};	/*Useragent - MQQBrowser/35 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA1957635 = {S73639, 140, { &P12280,&P18117,&P20786,&P18092 }};	/*Useragent - MQQBrowser/35 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3*/
const Device UA1978056 = {S73640, 139, { &P12280,&P18117,&P20786,&P18092 }};	/*Useragent - MQQBrowser/35 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3*/
const Device UA2520561 = {S73641, 133, { &P12280,&P18117,&P20786,&P18092 }};	/*Useragent - MQQBrowser/35 Mozilla/5.0 ( ; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA2686632 = {S73642, 138, { &P12280,&P18117,&P20786,&P18092 }};	/*Useragent - MQQBrowser/35 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3*/
const Device UA2167501 = {S73643, 82, { &P12581,&P5055,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; Nokia500)/UC Browser8.5.0.183/51/355/UCWEB Mobile*/
const Device UA2651272 = {S73644, 127, { &P12581,&P5055,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokia500) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile  3gpp-gba*/
const Device UA2026220 = {S73645, 161, { &P21243,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Motorola MOT-XT681 Build/IRMXCT_9_02.19.05RPS) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028503 = {S73646, 149, { &P21243,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Motorola MOT-XT681 Build/IRMXCT_9_02.19.05RPS) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2257073 = {S73647, 147, { &P21243,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; Motorola MOT-XT681 Build/IRM90B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2574457 = {S73648, 154, { &P21243,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Motorola MOT-XT681 Build/IRMXCT_9_02.19.05RPS) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2589493 = {S73649, 161, { &P21243,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Motorola MOT-XT681 Build/IRMXCT_9_02.30.00RPS) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2816370 = {S73650, 161, { &P21243,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Motorola MOT-XT681 Build/IRMXCT_9_02.24.00RPS) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2818781 = {S73651, 154, { &P21243,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Motorola MOT-XT681 Build/IRMXCT_9_02.30.00RPS) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2822134 = {S73652, 149, { &P21243,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Motorola MOT-XT681 Build/IRMXCT_9_02.30.00RPS) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2356940 = {S73653, 53, { &P18088,&P18432,&P24670,&P18092 }};	/*Useragent - IUC(U;iOS 6.0.1;Zh-cn;320*480;)/UCWEB8.4.1.169/42/997*/
const Device UA2577596 = {S73654, 164, { &P15487,&P17342,&P17454,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S5750E-BOUYGUES/S5750EAGLE1; U; Bada/2.0; fr-fr) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile WQVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA2390281 = {S73655, 80, { &P15364,&P17285,&P22555,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:17.0) Gecko/17.0 Firefox/17.0 IceDragon/17.0.1.0*/
const Device UA2456113 = {S73656, 137, { &P20512,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Fly F350 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3017984 = {S73657, 137, { &P20512,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Fly F350s build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2528629 = {S73658, 137, { &P22085,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ThL W5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528796 = {S73659, 134, { &P20156,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; S500 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2567715 = {S73660, 137, { &P18653,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P870 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2607498 = {S73661, 130, { &P20991,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ME371MG Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22*/
const Device UA2657664 = {S73662, 139, { &P21112,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; OPPOX907 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2763203 = {S73663, 128, { &P21112,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; X907 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2861129 = {S73664, 135, { &P21112,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; X907 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2756902 = {S73665, 137, { &P19853,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7102 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2772772 = {S73666, 137, { &P19853,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7102 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868985 = {S73667, 137, { &P19853,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; GT-N7102 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783083 = {S73668, 138, { &P23169,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT914 Build/2_28B_2005) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879654 = {S73669, 157, { &P15575,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2892676 = {S73670, 156, { &P15575,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2958266 = {S73671, 156, { &P15575,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2958638 = {S73672, 157, { &P15575,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2998469 = {S73673, 156, { &P15575,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3001022 = {S73674, 156, { &P15575,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9108 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3006079 = {S73675, 157, { &P15575,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3049545 = {S73676, 156, { &P15575,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2975049 = {S73677, 140, { &P22209,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Titanium S5 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974458 = {S73678, 138, { &P22935,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9080L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA738111 = {S73679, 155, { &P14923,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GTablet Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1964861 = {S73680, 154, { &P14923,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GTablet Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA912031 = {S73681, 124, { &P15364,&P5690,&P22432,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11 Comodo_Dragon/20.0.1.0*/
const Device UA1979914 = {S73682, 129, { &P20585,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AN7G3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2181677 = {S73683, 88, { &P20373,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-T116a NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1 firmware/T116a T04*/
const Device UA2232258 = {S73684, 162, { &P18317,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer Pad TF300T Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2355660 = {S73685, 163, { &P18317,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer Pad TF300T Build/JRO03R; CyanogenMod-1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2384818 = {S73686, 162, { &P18317,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Transformer Pad TF300T Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2485712 = {S73687, 154, { &P21852,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Spice_Mi-495 Build/SpiceSpice_Mi-495) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2503129 = {S73688, 129, { &P21267,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A798t Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2554895 = {S73689, 134, { &P21267,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A798t Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2595391 = {S73690, 143, { &P21267,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A798t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2748090 = {S73691, 65, { &P21267,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo A798t Build/IMM76D)*/
const Device UA2534581 = {S73692, 141, { &P20738,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Andromax U Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2562077 = {S73693, 176, { &P15767,&P19826,&P22305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_1 like Mac OS X; zh-cn Model:iPad2,2) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7543.48.3 UCBrowser/1.0.0.192/43/355*/
const Device UA2583700 = {S73694, 139, { &P13141,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I727R Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602605 = {S73695, 138, { &P13141,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9210 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2666961 = {S73696, 139, { &P13141,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E110S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2953323 = {S73697, 139, { &P13141,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SHV-E110S Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2728844 = {S73698, 155, { &P18722,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830L Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2791289 = {S73699, 155, { &P18722,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830K Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2854081 = {S73700, 165, { &P18722,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830S Build/JZO54K; CyanogenMod-JB_LR_VGV_V2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985956 = {S73701, 175, { &P18722,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830K Build/JZO54K; CyanogenMod-Cyanogenmod Version 10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2995912 = {S73702, 194, { &P18722,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830 Build/Ho?ng Minh H?u - Ho?ng H?i Bily; CyanogenMod-JB_LR_VGV_SLIM_V1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3071243 = {S73703, 175, { &P18722,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A830L Build/JZO54K; CyanogenMod-Cyanogenmod Version 10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2715421 = {S73704, 135, { &P17595,&P17297,&P22466,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Iron/26.0.1450.0 Chrome/26.0.1450.0 Safari/537.36*/
const Device UA2974666 = {S73705, 137, { &P14959,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9103 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA7209 = {S73706, 117, { &P13371,&P17485,&P3976,&P18092 }};	/*Useragent - Mozilla/4.0 SonyEricssonC902v/R3DA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.2*/
const Device UA631204 = {S73707, 125, { &P15364,&P17285,&P24195,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.12.12934.201*/
const Device UA2690426 = {S73708, 113, { &P15364,&P17285,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)*/
const Device UA2722005 = {S73709, 71, { &P15364,&P17285,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; SV1; QQBrowser/6.12)*/
const Device UA2722430 = {S73710, 79, { &P15364,&P17285,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; QQBrowser/6.12)*/
const Device UA2847038 = {S73711, 113, { &P15364,&P17285,&P24195,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)*/
const Device UA3118081 = {S73712, 198, { &P15364,&P17285,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; QQDownload 718; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; QQBrowser/6.12)*/
const Device UA640777 = {S73713, 46, { &P15364,&P18146,&P17825,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/3.5; SunOS)*/
const Device UA895974 = {S73714, 168, { &P16356,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-N7000 Build/IMM76I; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA908866 = {S73715, 168, { &P16356,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-N7000 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA924482 = {S73716, 62, { &P15364,&P17285,&P18515,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:14.0) Gecko/X Thunderbird/14.0*/
const Device UA2029466 = {S73717, 140, { &P21509,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; Philips W632 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2459568 = {S73718, 120, { &P11964,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE75-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2405714 = {S73719, 74, { &P18141,&P19824,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 2.01) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2563202 = {S73720, 97, { &P12142,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; SonyEricssonU5) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2506962 = {S73721, 152, { &P21622,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-note 2 Build/ICS_IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2506983 = {S73722, 144, { &P15438,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; HTC Sensation Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2749135 = {S73723, 143, { &P15438,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC Sensation Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781588 = {S73724, 66, { &P15438,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; HTC Sensation Build/JDQ39E)*/
const Device UA2895546 = {S73725, 144, { &P15438,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC Sensation Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2533925 = {S73726, 143, { &P21850,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Spice Mi-435 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558465 = {S73727, 176, { &P12139,&P5204,&P24676,&P18092 }};	/*Useragent - Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5310xpressmusic) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2729872 = {S73728, 136, { &P23007,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TM400 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2797656 = {S73729, 142, { &P23007,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AIRIS TM400 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2755270 = {S73730, 137, { &P21628,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3036129 = {S73731, 152, { &P15767,&P19826,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_1_3 like Mac OS X; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT Mobile*/
const Device UA3061099 = {S73732, 152, { &P15767,&P19826,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_1_3 like Mac OS X; en-GB) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IT Mobile*/
const Device UA2800374 = {S73733, 136, { &P21011,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; L-01E Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3002441 = {S73734, 58, { &P21011,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; L-01E Build/JZO54K)*/
const Device UA3026354 = {S73735, 158, { &P17625,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST25i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3062252 = {S73736, 158, { &P17625,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST25i Build/6.0.B.3.188) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3080989 = {S73737, 138, { &P14225,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Desire HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3067296 = {S73738, 143, { &P20816,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WIKO-CINK SLIM Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA9479 = {S73739, 79, { &P15364,&P17017,&P4223,&P18092 }};	/*Useragent - INQ/Q1-218H/1.0.74 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA9480 = {S73740, 85, { &P15364,&P17017,&P4223,&P18092 }};	/*Useragent - INQ/Q1-218H/1.0.74parrot Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87022 = {S73741, 57, { &P15364,&P17017,&P4223,&P18092 }};	/*Useragent - Mozilla/5.0 (PDA; NF35WMPRO/1.0; like Gecko) NetFront/3.5*/
const Device UA90740 = {S73742, 64, { &P15364,&P17017,&P4223,&P18092 }};	/*Useragent - MD-Cube/R01 NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA90741 = {S73743, 152, { &P15364,&P17017,&P4223,&P18092 }};	/*Useragent - MD-302/1.0/B000 Browser/NetFront/3.5 MMS/Obigo-MMS/Q05A SyncML/HW-SyncML/1.0 Java/HWJa/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Player/QTV-Player/5.3*/
const Device UA90786 = {S73744, 98, { &P15364,&P17017,&P4223,&P18092 }};	/*Useragent - Mozilla/5.0 (DigitalMediaPlayer; Walkman/NWZ-X1000 Series/1.00; like Gecko; wireless) NetFront/3.5*/
const Device UA690170 = {S73745, 86, { &P15364,&P17017,&P4223,&P18092 }};	/*Useragent - Mozilla/5.0 (rv:1.3) like Gecko/X Profile/MIDP-2.0 Configuration/CLDC-1.1 NetFront/3.5*/
const Device UA22855 = {S73746, 115, { &P13832,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S7550B/1.0 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA23413 = {S73747, 105, { &P13832,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S7550/S7550XXJD2 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA30368 = {S73748, 84, { &P12406,&P17603,&P3534,&P18092 }};	/*Useragent - PANTECH-C740/ISUS09012008 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA31168 = {S73749, 75, { &P12406,&P17603,&P3534,&P18092 }};	/*Useragent - PANTECH-C740/R01 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA180111 = {S73750, 135, { &P17622,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HERO200 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA202270 = {S73751, 135, { &P17622,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HERO200 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA249252 = {S73752, 135, { &P17622,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HERO200 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA319009 = {S73753, 135, { &P17622,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HERO200 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021832 = {S73754, 59, { &P17622,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; HERO200 Build/GRI40)*/
const Device UA2028556 = {S73755, 135, { &P17622,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ar-eg; HERO200 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA898229 = {S73756, 94, { &P15364,&P18110,&P18101,&P18091 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:7.0.2) Gecko/X Firefox/12.0.1 http://repskan.com/bots.repskan*/
const Device UA2024108 = {S73757, 162, { &P17937,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; WX435 Build/IML77; CyanogenMod-9.0.0.Triumph) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2366719 = {S73758, 151, { &P17937,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; WX435 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2010564 = {S73759, 136, { &P21017,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; P-07D Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2138059 = {S73760, 80, { &P15364,&P7127,&P20904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.7.5) Gecko/X Netscape/8.1.2*/
const Device UA2170484 = {S73761, 133, { &P18466,&P18110,&P19988,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.0 Safari/535.19 Silk-Accelerated=false*/
const Device UA2257524 = {S73762, 132, { &P23897,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; STM805ND Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2404666 = {S73763, 143, { &P20091,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5880D Build/PMP5880D_v1.0.0_X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2772286 = {S73764, 156, { &P20983,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9308 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2750797 = {S73765, 136, { &P24549,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; E2001 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2792990 = {S73766, 57, { &P15364,&P17188,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; WinNT4.0; Trident/5.0)*/
const Device UA2851165 = {S73767, 57, { &P15364,&P17188,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; WinNT4.0; Trident/1.0)*/
const Device UA2974101 = {S73768, 136, { &P20587,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P715 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976691 = {S73769, 137, { &P19853,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7102 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022957 = {S73770, 137, { &P19853,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7102 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023209 = {S73771, 137, { &P19853,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; GT-N7102 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3081089 = {S73772, 137, { &P19853,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7102 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081377 = {S73773, 137, { &P19853,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7102 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126604 = {S73774, 137, { &P19853,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; GT-N7102 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA23606 = {S73775, 158, { &P11680,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-KM900/V10o Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA23607 = {S73776, 158, { &P11680,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-KM900/V10v Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA23608 = {S73777, 158, { &P11680,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-KM900/V10j Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA33382 = {S73778, 158, { &P11680,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-KM900/V10m Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA61672 = {S73779, 131, { &P11680,&P17483,&P3728,&P18092 }};	/*Useragent - LG-KM900/V10g Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA62348 = {S73780, 158, { &P11680,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-KM900/V10w Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA62346 = {S73781, 158, { &P11680,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-KM900/V10l Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA64457 = {S73782, 158, { &P11680,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-KM900/V10t Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA590208 = {S73783, 87, { &P17976,&P5096,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.9.201 Version/11.50*/
const Device UA745391 = {S73784, 92, { &P17976,&P5096,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2.1; Linux; Opera Tablet/ADR-X; U; es-ES) Presto/2.9.201 Version/11.50*/
const Device UA2020855 = {S73785, 89, { &P17976,&P5096,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2.2; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.9.201 Version/11.50*/
const Device UA1953912 = {S73786, 124, { &P15767,&P17090,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.9.2*/
const Device UA1968166 = {S73787, 124, { &P15767,&P17090,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.9.1*/
const Device UA1969801 = {S73788, 124, { &P15767,&P17090,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.9.4*/
const Device UA1995977 = {S73789, 124, { &P15767,&P17090,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.9.5*/
const Device UA2006117 = {S73790, 124, { &P15767,&P17090,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.9.6*/
const Device UA2007385 = {S73791, 124, { &P15767,&P17090,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.9.6*/
const Device UA2157158 = {S73792, 124, { &P15767,&P17090,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.9.1*/
const Device UA2260714 = {S73793, 124, { &P15767,&P17090,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.9.7*/
const Device UA2260732 = {S73794, 124, { &P15767,&P17090,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.9.7*/
const Device UA2334396 = {S73795, 124, { &P15767,&P17090,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.9.2*/
const Device UA2337817 = {S73796, 124, { &P15767,&P17090,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.9.5*/
const Device UA2426591 = {S73797, 124, { &P15767,&P17090,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.9.8*/
const Device UA2456559 = {S73798, 124, { &P15767,&P17090,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.9.8*/
const Device UA2653291 = {S73799, 124, { &P15767,&P17090,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 Flipboard/1.9.8*/
const Device UA2695245 = {S73800, 124, { &P15767,&P17090,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 Flipboard/1.9.8*/
const Device UA2188120 = {S73801, 133, { &P18466,&P18110,&P18616,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.1 Safari/535.19 Silk-Accelerated=false*/
const Device UA2466327 = {S73802, 128, { &P20270,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; U20GT Build/U20GT) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2468867 = {S73803, 145, { &P21618,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile IQ 5A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934395 = {S73804, 67, { &P21618,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; i-mobile IQ 5A Build/IMM76D)*/
const Device UA2510034 = {S73805, 177, { &P12871,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2702524 = {S73806, 177, { &P12871,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; HTC Wildfire S A510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3107710 = {S73807, 181, { &P12871,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC_Wildfire_S_A510e Build/GRJ90) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2957510 = {S73808, 142, { &P24700,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Studio 5.3s Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3019163 = {S73809, 141, { &P22968,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HUAWEI P6-U06 Build/HuaweiP6-U06) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3020335 = {S73810, 148, { &P22968,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HUAWEI P6-U06 Build/HuaweiP6-U06) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA64886 = {S73811, 105, { &P11750,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonC905a/R1BB Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.0*/
const Device UA69533 = {S73812, 88, { &P15224,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-G922/MTK Release/7.15.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA908463 = {S73813, 62, { &P15364,&P18146,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; SunOS i86pc; rv:13.0) Gecko/X Firefox/13.0.1*/
const Device UA912261 = {S73814, 50, { &P15364,&P18120,&P18108,&P18092 }};	/*Useragent - Pulp/2.5.2 (Mac; http://www.acrylicapps.com/pulp/)*/
const Device UA1952166 = {S73815, 144, { &P21052,&P5629,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2205883 = {S73816, 166, { &P11751,&P17483,&P24672,&P18092 }};	/*Useragent - LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-kp500 teleca) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile*/
const Device UA2223198 = {S73817, 136, { &P21312,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; A19 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2310165 = {S73818, 39, { &P13443,&P5890,&P3439,&P18092 }};	/*Useragent - SamsungI8910/SymbianOS/9.1 Series60/3.0*/
const Device UA2332546 = {S73819, 149, { &P21620,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; i-mobile i-STYLE 2i  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2376034 = {S73820, 147, { &P21620,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; i-mobile i-STYLE 2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2423551 = {S73821, 69, { &P18304,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GT-I9070; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2660113 = {S73822, 224, { &P22792,&P17017,&P18246,&P18092 }};	/*Useragent - G607_MAIN_V1.0 (MRE\2.5.00(2072) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\1 gsensor\0 keyboard\reduced) INTEX_TURBO_3.5_V07_INDIAN Release/2013.01.04 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US*/
const Device UA2734631 = {S73823, 224, { &P22792,&P17017,&P18246,&P18092 }};	/*Useragent - G607_MAIN_V1.0 (MRE\2.5.00(2072) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\1 gsensor\0 keyboard\reduced) INTEX_TURBO_3.5_V08_INDIAN Release/2013.03.08 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US*/
const Device UA2958063 = {S73824, 224, { &P22792,&P17017,&P18246,&P18092 }};	/*Useragent - G607_MAIN_V1.0 (MRE\2.5.00(2072) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\1 gsensor\0 keyboard\reduced) INTEX_TURBO_3.5_V10_INDIAN Release/2013.04.26 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US*/
const Device UA2664665 = {S73825, 145, { &P18324,&P5629,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/Sleipnir 2.4.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2669685 = {S73826, 147, { &P18324,&P5629,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/Sleipnir 2.4.0.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2752019 = {S73827, 147, { &P18324,&P5629,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/Sleipnir 2.4.0.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2799910 = {S73828, 147, { &P18324,&P5629,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/Sleipnir 2.4.0.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2937505 = {S73829, 159, { &P18673,&P18066,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300*/
const Device UA2708967 = {S73830, 130, { &P20160,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2718784 = {S73831, 156, { &P15919,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-P3100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2893326 = {S73832, 156, { &P15919,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-P3100 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2736336 = {S73833, 136, { &P22685,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E460 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3073916 = {S73834, 129, { &P22685,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E460 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2772761 = {S73835, 174, { &P18472,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2754821 = {S73836, 127, { &P18704,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT200 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755263 = {S73837, 138, { &P19902,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2945328 = {S73838, 69, { &P15364,&P7133,&P23585,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:23.0) Gecko/X Thunderbird/23.0*/
const Device UA2875080 = {S73839, 140, { &P24164,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; CROSS A7S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892166 = {S73840, 140, { &P23169,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT914 Build/2_28B_2005) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957660 = {S73841, 62, { &P23169,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT914 Build/2_32A_2012)*/
const Device UA2957991 = {S73842, 140, { &P23169,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT914 Build/2_32A_2012) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997213 = {S73843, 140, { &P23169,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT914 Build/2_28B_2012) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3090907 = {S73844, 140, { &P23169,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT914 Build/2_32A_2035) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2948027 = {S73845, 158, { &P20947,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTCEVOV4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2974365 = {S73846, 150, { &P20993,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI P2-6011 Build/HuaweiP2-6011) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3132852 = {S73847, 139, { &P17172,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA44314 = {S73848, 110, { &P12100,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-S5233T/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA252236 = {S73849, 120, { &P15364,&P5690,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.68.1420 Chrome/14.0.835.202 Safari/535.1*/
const Device UA314669 = {S73850, 140, { &P15364,&P5690,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) RockMelt/0.9.50.549 Chrome/10.0.648.205 Safari/534.16*/
const Device UA332364 = {S73851, 120, { &P15364,&P5690,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.24 (KHTML, like Gecko) RockMelt/0.9.58.494 Chrome/11.0.696.71 Safari/534.24*/
const Device UA337086 = {S73852, 119, { &P15364,&P5690,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.64.319 Chrome/13.0.782.215 Safari/535.1*/
const Device UA635397 = {S73853, 121, { &P15364,&P5690,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML, like Gecko) RockMelt/0.9.62.351 Chrome/12.0.742.122 Safari/534.30*/
const Device UA741061 = {S73854, 138, { &P15364,&P5690,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) RockMelt/0.9.48.51 Chrome/9.0.597.107 Safari/534.13*/
const Device UA345458 = {S73855, 155, { &P14872,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SPH-D700 Build/GRJ22; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA365000 = {S73856, 161, { &P14872,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SPH-D700 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA656144 = {S73857, 85, { &P15547,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-c6712) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA685991 = {S73858, 85, { &P15547,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-c6712) UCBrowser8.2.0.132/69/352/UCWEB Mobile*/
const Device UA1842558 = {S73859, 88, { &P17595,&P18120,&P4536,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X; Opera Mobi/27; U; en) Presto/2.4.18 Version/10.00*/
const Device UA2257269 = {S73860, 138, { &P23303,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Ex Note Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2348528 = {S73861, 116, { &P14862,&P5204,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2700c-2/09.97; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2373968 = {S73862, 157, { &P18224,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Ascend G300 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2402910 = {S73863, 135, { &P20480,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JY-G3 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2434292 = {S73864, 74, { &P18141,&P19825,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 2.02) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2567693 = {S73865, 141, { &P20285,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ616-12) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2600028 = {S73866, 146, { &P20285,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ616 Build/7.7.1-141-6-PAST-UMTS-LA) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2601951 = {S73867, 93, { &P20146,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9082) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2671657 = {S73868, 137, { &P18641,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-M830 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709753 = {S73869, 142, { &P20076,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xperia Acro S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3062631 = {S73870, 138, { &P20076,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26w Build/6.2.B.1.96) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3115036 = {S73871, 131, { &P20076,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26w Build/6.2.B.1.96) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2773884 = {S73872, 156, { &P21261,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2755339 = {S73873, 130, { &P15876,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2772684 = {S73874, 134, { &P12605,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT875 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3022601 = {S73875, 136, { &P20708,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IdeaTabA1000-F Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023764 = {S73876, 139, { &P20708,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XOLO A1000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3030108 = {S73877, 136, { &P20708,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IdeaTabA1000-G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3132943 = {S73878, 144, { &P22972,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I537 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA62391 = {S73879, 73, { &P14405,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-GT-B3313/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA200229 = {S73880, 132, { &P21196,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; 003P Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023998 = {S73881, 56, { &P21196,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; 003P Build/GRI40)*/
const Device UA670837 = {S73882, 119, { &P15364,&P18110,&P22455,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11*/
const Device UA673993 = {S73883, 121, { &P15364,&P18110,&P22455,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11*/
const Device UA682245 = {S73884, 143, { &P21340,&P5096,&P3508,&P18092 }};	/*Useragent - HUAWEI_T8300_TD/1.0 OPhone/2.5 (Linux;Android 2.2) Release/03.20.2011 Browser/WAP2.0(AppleWebKit/533.1) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA763092 = {S73885, 128, { &P21340,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HUAWEI T8300 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2019941 = {S73886, 140, { &P21340,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HUAWEI T8300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026696 = {S73887, 133, { &P21340,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HUAWEI T8300 Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2040904 = {S73888, 80, { &P17595,&P17297,&P18523,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:15.0) Gecko/X Thunderbird/15.0.1*/
const Device UA2384119 = {S73889, 71, { &P15501,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.1-update1; zh-cn; E16i; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2567389 = {S73890, 142, { &P19997,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo P700i Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667240 = {S73891, 141, { &P19997,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Lenovo P700 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2676482 = {S73892, 130, { &P12889,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3031593 = {S73893, 161, { &P18462,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2963943 = {S73894, 93, { &P11854,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae71-1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3067000 = {S73895, 92, { &P11854,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE71) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3018014 = {S73896, 103, { &P14748,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae72-1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3059789 = {S73897, 94, { &P14748,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE72-1) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3034487 = {S73898, 165, { &P15798,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Explorer A310e Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA391 = {S73899, 68, { &P17457,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V3iv/08.B5.10R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA15394 = {S73900, 67, { &P17457,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V3i/08.B4.34R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA15401 = {S73901, 69, { &P17457,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V3ire/08.00.33R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA15404 = {S73902, 68, { &P17457,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V3i /08.03.05R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA15717 = {S73903, 77, { &P11696,&P5448,&P22272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) MOT-Q9/01.07.28R*/
const Device UA15720 = {S73904, 77, { &P11696,&P5448,&P22272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) MOT-Q9/01.06.08R*/
const Device UA15724 = {S73905, 77, { &P11696,&P5448,&P22272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) MOT-Q9/01.07.05R*/
const Device UA15722 = {S73906, 77, { &P11696,&P5448,&P22272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) MOT-Q9/01.07.37R*/
const Device UA18556 = {S73907, 116, { &P11696,&P5448,&P22272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) 320x240; VZW; Motorola-Q9m; Windows Mobile 6.0 Standard*/
const Device UA60951 = {S73908, 87, { &P12601,&P17485,&P3499,&P18092 }};	/*Useragent - SonyEricssonW150a/R2AD032 TelecaBrowser/Q07C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61644 = {S73909, 104, { &P11824,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-T919/T919UVHI5 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA625625 = {S73910, 74, { &P15364,&P17285,&P21171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:12.0) Gecko/X Firefox/12.0 SeaMonkey/2.9.1*/
const Device UA2813579 = {S73911, 68, { &P15364,&P17285,&P21171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:12.0) Gecko/X SeaMonkey/2.9.1*/
const Device UA824918 = {S73912, 78, { &P15364,&P18110,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.20) Gecko/X Firefox(2.0.0.20)*/
const Device UA1860023 = {S73913, 73, { &P15364,&P18110,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:2.0b10pre) Gecko/X Firefox-4.0/4.0b10pre*/
const Device UA2127188 = {S73914, 61, { &P15364,&P18110,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; ; Linux x86_64; rv:1.8.1.6) Gecko/X Firefox*/
const Device UA2032864 = {S73915, 140, { &P24030,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; th-TH; OPPOX9015 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2374074 = {S73916, 131, { &P20188,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; U19GT-W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2415268 = {S73917, 134, { &P20453,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NABI2-NV7A Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2484195 = {S73918, 137, { &P20453,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NABI2-NV7A-UK Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3071146 = {S73919, 137, { &P20453,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NABI2-NV7A-IE Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2417986 = {S73920, 129, { &P20522,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MP843 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2423218 = {S73921, 136, { &P20821,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB192 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2484106 = {S73922, 137, { &P20821,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB0192 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2432464 = {S73923, 139, { &P20264,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; bq Curie Build/1.0.2 X-19:30) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2526465 = {S73924, 151, { &P13633,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; A953 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2840809 = {S73925, 151, { &P13633,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; A953 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2753462 = {S73926, 133, { &P17172,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Transformer Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2782812 = {S73927, 139, { &P17172,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Transformer TF101 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2957782 = {S73928, 139, { &P17172,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Transformer TF101 Build/MASTER) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2967610 = {S73929, 102, { &P15795,&P21697,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 710) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile*/
const Device UA2975745 = {S73930, 130, { &P18702,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8013 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974317 = {S73931, 139, { &P16319,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; U8860 Build/HuaweiU8860) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA64160 = {S73932, 109, { &P12023,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-M8910/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA13665 = {S73933, 31, { &P13510,&P5282,&P3579,&P18092 }};	/*Useragent - ZTE-E520/1.0 SMIT-Browser/2.0.0*/
const Device UA89889 = {S73934, 138, { &P16332,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; Galaxy Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA90338 = {S73935, 156, { &P16332,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6 - Galaxo V1.6.3.3; en-gb; Galaxy Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA484122 = {S73936, 156, { &P16332,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6 - Galaxo V1.6.3.4; en-gb; Galaxy Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA50341 = {S73937, 151, { &P13233,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; T-Mobile myTouch 3G Build/COC10) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA88667 = {S73938, 151, { &P13233,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; T-Mobile myTouch 3G Build/CRB57) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2018113 = {S73939, 149, { &P13233,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; es-; T-Mobile myTouch 3G Build/COC10) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA481549 = {S73940, 122, { &P17595,&P18120,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/525.18 (KHTML, like Gecko, Safari/525.20) OmniWeb/v622.6.0.X*/
const Device UA558184 = {S73941, 122, { &P17595,&P18120,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/528.16 (KHTML, like Gecko, Safari/528.16) OmniWeb/v622.8.0.X*/
const Device UA701560 = {S73942, 122, { &P17595,&P18120,&P23602,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/525.18 (KHTML, like Gecko, Safari/525.20) OmniWeb/v622.4.0.X*/
const Device UA2185322 = {S73943, 138, { &P16757,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SCH-I500 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261569 = {S73944, 138, { &P16757,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I500 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2223882 = {S73945, 83, { &P20383,&P5629,&P15760,&P18092 }};	/*Useragent - ZTEU880s_TD/1.0 Linux/2.6.35 Android/2.3 Release/2.10.2012 Browser/AppleWebKit533.1*/
const Device UA2413353 = {S73946, 130, { &P20383,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE U880s Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2553320 = {S73947, 130, { &P20383,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE U880s Build GWK74) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2657062 = {S73948, 87, { &P20383,&P5629,&P15760,&P18092 }};	/*Useragent - ZTEU880s2_TD/1.0 Linux/2.6.35 Android/2.3.5 Release/03.29.2012 Browser/AppleWebKit533.1*/
const Device UA2466427 = {S73949, 69, { &P15919,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; GT-P3100; 600*976) UCWEB7.9.0.94/139/444*/
const Device UA2560092 = {S73950, 93, { &P11964,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaE75-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2765730 = {S73951, 93, { &P11964,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaE75-2) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2584143 = {S73952, 153, { &P20175,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI 1S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2667544 = {S73953, 154, { &P20175,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI 1SC Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2600240 = {S73954, 138, { &P21856,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax P600 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0  Safari/534.30*/
const Device UA2625428 = {S73955, 192, { &P12879,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, -0500, us*/
const Device UA2660427 = {S73956, 58, { &P15364,&P5448,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; WindowsCE; Trident/1.0)*/
const Device UA2791732 = {S73957, 63, { &P15364,&P5448,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; 68K; Tablet PC; Trident/1.0)*/
const Device UA2670606 = {S73958, 139, { &P20738,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Andromax U Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2955549 = {S73959, 132, { &P20738,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Andromax U Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2677292 = {S73960, 137, { &P18484,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-US730 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2732881 = {S73961, 142, { &P22742,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; GOCLEVER TAB M723G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2967282 = {S73962, 102, { &P18623,&P21697,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 510) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile*/
const Device UA2868692 = {S73963, 140, { &P24150,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; vivo Xplay Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3008860 = {S73964, 62, { &P24150,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; vivo Xplay Build/JDQ39)*/
const Device UA3022338 = {S73965, 128, { &P24366,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SM-T311 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3080244 = {S73966, 134, { &P20944,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T-Hub2 Build/TVA301TELBG3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3081689 = {S73967, 143, { &P11747,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire 500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094235 = {S73968, 152, { &P11747,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire 500 dual sim Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3115060 = {S73969, 143, { &P11747,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC_Desire_500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA18557 = {S73970, 117, { &P14998,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) 320x240; VZW; Motorola-Q9c; Windows Mobile 6.1 Standard*/
const Device UA54709 = {S73971, 117, { &P12312,&P17485,&P3976,&P18092 }};	/*Useragent - Mozilla/4.0 SonyEricssonC905v/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3*/
const Device UA397137 = {S73972, 104, { &P14862,&P5204,&P17338,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2700c-2/UC Browser7.9.0.102/70/355*/
const Device UA2810588 = {S73973, 254, { &P14862,&P5204,&P17338,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/69/352*/
const Device UA121632 = {S73974, 65, { &P21640,&P5448,&P17470,&P18092 }};	/*Useragent - Mozilla/6.0 (compatible; MSIE 7.01; Windows CE; 760x412; Samsung)*/
const Device UA330419 = {S73975, 139, { &P15767,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8L1 Safari/6531.22.7*/
const Device UA540710 = {S73976, 139, { &P15767,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8J2 Safari/6531.22.7*/
const Device UA729372 = {S73977, 139, { &P15767,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8J3 Safari/6531.22.7*/
const Device UA1897586 = {S73978, 141, { &P15767,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2260684 = {S73979, 141, { &P15767,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2288235 = {S73980, 249, { &P15767,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3,Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2442077 = {S73981, 139, { &P15767,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2495000 = {S73982, 141, { &P15767,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2495142 = {S73983, 141, { &P15767,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA797800 = {S73984, 107, { &P15364,&P22740,&P22743,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux armv7l; it-IT; rv:1.9.2a1pre) Gecko/X Firefox/3.5 Maemo Browser 1.5.6 RX-51 N900*/
const Device UA616668 = {S73985, 130, { &P24019,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; PC1007 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA912025 = {S73986, 130, { &P24019,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; PC1007 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA688575 = {S73987, 143, { &P15881,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Incredible 2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA802915 = {S73988, 143, { &P15881,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Incredible 2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA889762 = {S73989, 143, { &P15881,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Incredible 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1937889 = {S73990, 142, { &P15881,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Incredible 2 Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028617 = {S73991, 155, { &P15881,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Incredible 2 Build/Kangdemnation_V0.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032893 = {S73992, 143, { &P15881,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Incredible 2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2222311 = {S73993, 147, { &P15881,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Incredible 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2503670 = {S73994, 153, { &P15881,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Droid Incredible 2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2693256 = {S73995, 155, { &P15881,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Incredible 2 Build/Kangdemnation_V0.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2042024 = {S73996, 80, { &P17595,&P18207,&P18523,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:15.0) Gecko/X Thunderbird/15.0.1*/
const Device UA2028544 = {S73997, 142, { &P13724,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i909 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2417073 = {S73998, 143, { &P13724,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-en; SGH-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2817295 = {S73999, 135, { &P13724,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i909 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2029078 = {S74000, 136, { &P21448,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SH530U Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2505062 = {S74001, 138, { &P20073,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; ZTE V880E Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2384234 = {S74002, 209, { &P23343,&P22857,&P18373,&P18092 }};	/*Useragent - Lemon T99 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA2427971 = {S74003, 137, { &P20505,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; starTIM tab 7 Build/TIM_01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2530071 = {S74004, 75, { &P15364,&P16110,&P22564,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:16.0) Gecko/X Firefox/16.0 epic/16.0*/
const Device UA2560820 = {S74005, 68, { &P15364,&P16110,&P22564,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; rv:16.0) Gecko/X Firefox/16.0 Epic/16.0*/
const Device UA2567323 = {S74006, 139, { &P18716,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730M Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2676584 = {S74007, 141, { &P21119,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SoftwinerEvb Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709358 = {S74008, 142, { &P21119,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Softwinerf761 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712200 = {S74009, 134, { &P21119,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SoftwinerEvb Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2994675 = {S74010, 142, { &P21119,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Softwinerf900 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2774165 = {S74011, 158, { &P23771,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2796550 = {S74012, 148, { &P23174,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GenxDroid7 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2896470 = {S74013, 60, { &P15364,&P18146,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; SunOS i86pc; rv:22.0) Gecko/X Firefox/22.0*/
const Device UA2942331 = {S74014, 141, { &P24132,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; iris504Q Build/iris504Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3039410 = {S74015, 156, { &P22382,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8262 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA4616 = {S74016, 20, { &P13319,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-N281/Obigo/Q03*/
const Device UA4617 = {S74017, 61, { &P13319,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-N281-Miami-Orange/ORANGE_ENG_EN_P107A3V1.0.7 Obigo/Q03C*/
const Device UA7374 = {S74018, 65, { &P13319,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-N281-LeTactile-Orange/ORANGE_OFR_FR_P107A3V1.0.8 Obigo/Q03C*/
const Device UA7375 = {S74019, 61, { &P13319,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-N281-Miami-Orange/ORANGE_OSP_SP_P107A3V1.0.3 Obigo/Q03C*/
const Device UA7377 = {S74020, 61, { &P13319,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-N281-Miami-Orange/ORANGE_OSP_SP_P107A3V1.0.7 Obigo/Q03C*/
const Device UA21646 = {S74021, 61, { &P13319,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-N281-Miami-Orange/ORANGE_ENG_EN_P107A3V1.0.4 Obigo/Q03C*/
const Device UA63870 = {S74022, 61, { &P13319,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-N281-Miami-Orange/ORANGE_ENG_EN_P107A3V1.0.8 Obigo/Q03C*/
const Device UA64803 = {S74023, 17, { &P17692,&P22839,&P17470,&P18092 }};	/*Useragent - ZTE-G_R221/WAP2.0*/
const Device UA247879 = {S74024, 135, { &P14183,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-I997 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370694 = {S74025, 59, { &P14183,&P24694,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; SGH-I997 Build/MIUI)*/
const Device UA379924 = {S74026, 135, { &P14183,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-I997 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026040 = {S74027, 135, { &P14183,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.9; en-us; SGH-I997 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA901954 = {S74028, 122, { &P15364,&P5690,&P22431,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5 Comodo_Dragon/19.2.0.0*/
const Device UA1978391 = {S74029, 129, { &P23935,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; PP803 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2414476 = {S74030, 144, { &P20073,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; tmn smart a15 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2500096 = {S74031, 140, { &P20073,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; ZTE V880E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2457714 = {S74032, 136, { &P19864,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 80XSK Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2559777 = {S74033, 137, { &P12545,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-TSM_T6/1.0 Linux/Apps_2.6.32.9-perf Android/2.3.7 Release/01.19.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2706106 = {S74034, 137, { &P12545,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-TSM_T6/1.0 Linux/Apps_2.6.32.9-perf Android/2.3.7 Release/01.24.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2878929 = {S74035, 210, { &P12545,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; sprd-TSM_T6/1.0 Android/2.3.7 Release/04.22.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2566769 = {S74036, 146, { &P18114,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Transformer Prime TF201 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2581240 = {S74037, 133, { &P21890,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; TAD-10021 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2582095 = {S74038, 137, { &P23301,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; OV-BaseCore10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2677171 = {S74039, 140, { &P19764,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6502 Build/10.1.A.1.434) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2729896 = {S74040, 220, { &P18088,&P22843,&P18246,&P18092 }};	/*Useragent - LAVA50_11B_HW (MRE\3.0.00(2500) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) LAVA_DISCOVER135_S208_X Release/2012.12.18 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US*/
const Device UA2878269 = {S74041, 220, { &P18088,&P22843,&P18246,&P18092 }};	/*Useragent - LAVA50_11B_HW (MRE\3.0.00(2500) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) LAVA_DISCOVER132_S210_X Release/2012.12.14 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US*/
const Device UA3015259 = {S74042, 343, { &P18088,&P22843,&P18246,&P18092 }};	/*Useragent - VSUN50_FTE_11B_HW (MRE/3.0.00(512);MAUI/C11_S422_X;BDATE/2013/06/05 15:05;LCD/X;CHIP/MT6250;KEY/Normal;TOUCH/1;CAMERA/1;SENSOR/0;DEV/VSUN50_FTE_11B_HW;WAP Browser/MAUI ();GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/005;GAMELOFT/006;MOBIM/007;KKFUN/008;MT) LAVA_C11_S422_X Release/2013.06.05 WAP Browser/MAUI Profile/  Q03C1-2.40 en-US*/
const Device UA3015692 = {S74043, 343, { &P18088,&P22843,&P18246,&P18092 }};	/*Useragent - VSUN50_FTE_11B_HW (MRE/3.0.00(512);MAUI/C11_S418_X;BDATE/2013/05/10 13:45;LCD/X;CHIP/MT6250;KEY/Normal;TOUCH/1;CAMERA/1;SENSOR/0;DEV/VSUN50_FTE_11B_HW;WAP Browser/MAUI ();GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/005;GAMELOFT/006;MOBIM/007;KKFUN/008;MT) LAVA_C11_S418_X Release/2013.05.10 WAP Browser/MAUI Profile/  Q03C1-2.40 en-US*/
const Device UA3074308 = {S74044, 220, { &P18088,&P22843,&P18246,&P18092 }};	/*Useragent - LAVA50_11B_HW (MRE\3.0.00(2500) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) LAVA_DISCOVER135_S214_X Release/2013.04.28 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US*/
const Device UA3110597 = {S74045, 383, { &P18088,&P22843,&P18246,&P18092 }};	/*Useragent - FARSIGHTED60A_12864_COSMOS_BU1_11B_HW (MRE/3.1.00(1536);MAUI/DISCOVER135_S106_X;BDATE/2013/08/09 11:20;LCD/X;CHIP/MT6260;KEY/Reduced;TOUCH/1;CAMERA/0;SENSOR/0;DEV/FARSIGHTED60A_12864_COSMOS_BU1_11B_HW;WAP Browser/MAUI (HTTP PGDL);GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/) LAVA_DISCOVER135_S106_X Release/2013.08.09 WAP Browser/MAUI (HTTP PGDL) Profile/  Q03C1-2.40 en-US*/
const Device UA2772677 = {S74046, 147, { &P16356,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA63470 = {S74047, 109, { &P12431,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-S7220/S7220ACIB1 SHP/VPP/R5 NetFront/3.5 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA11299 = {S74048, 62, { &P13721,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia6555c/2.0 (03.41) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA62143 = {S74049, 62, { &P13721,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia6555b/2.0 (03.31) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63314 = {S74050, 61, { &P13721,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia6555/2.0 (03.46) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA50149 = {S74051, 135, { &P18061,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; de-; sdk Build/CUPCAKE) AppleWebkit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA61989 = {S74052, 137, { &P18061,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; sdk Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2721882 = {S74053, 137, { &P18061,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; sdk Build/CUPCAKE) AppleWebkit/528.5  (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2801418 = {S74054, 144, { &P18061,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; google_sdk Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA220040 = {S74055, 139, { &P15881,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Incredible 2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2258617 = {S74056, 139, { &P15881,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Incredible 2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA236458 = {S74057, 124, { &P15364,&P5690,&P22406,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.2.0.0 Chrome/12.0.742.112 Safari/534.30*/
const Device UA382153 = {S74058, 73, { &P16329,&P18179,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PLAYSTATION 3 4.10) AppleWebKit/531.22.8 (KHTML, like Gecko)*/
const Device UA2222007 = {S74059, 73, { &P16329,&P18179,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PLAYSTATION 3 4.10) AppleWebKit/531.22.8 (KHTML	 like Gecko)*/
const Device UA760782 = {S74060, 74, { &P15364,&P18146,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; SunOS i86pc; en-US; rv:1.9.0.8) Gecko/X Firefox/3.0.8*/
const Device UA453165 = {S74061, 141, { &P18341,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; HKPHONE H8-3G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA630187 = {S74062, 133, { &P18341,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; H8-3G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033670 = {S74063, 134, { &P18341,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; H8-3G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA456664 = {S74064, 13, { &P15364,&P17017,&P18333,&P18092 }};	/*Useragent - iTunes/10.5.2*/
const Device UA608447 = {S74065, 11, { &P15364,&P17017,&P18333,&P18092 }};	/*Useragent - iTunes/10.6*/
const Device UA639141 = {S74066, 42, { &P15364,&P17017,&P18333,&P18092 }};	/*Useragent - iTunes (unknown version) CFNetwork/520.2.6*/
const Device UA639569 = {S74067, 11, { &P15364,&P17017,&P18333,&P18092 }};	/*Useragent - iTunes/10.0*/
const Device UA1976566 = {S74068, 13, { &P15364,&P17017,&P18333,&P18092 }};	/*Useragent - iTunes/10.1.6*/
const Device UA642439 = {S74069, 71, { &P20319,&P22784,&P4780,&P18092 }};	/*Useragent - MicromaxX444/Q03C MAUI-browser; Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA648049 = {S74070, 98, { &P14697,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Schubert)*/
const Device UA2496331 = {S74071, 106, { &P14697,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; 7 Schubert T9292)*/
const Device UA1967036 = {S74072, 137, { &P21096,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-09D Build/V19R26A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996267 = {S74073, 137, { &P21096,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-09D Build/V20R27B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2367215 = {S74074, 137, { &P21096,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-09D Build/V21R28A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2040382 = {S74075, 81, { &P15364,&P5690,&P22553,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:15.0) Gecko/X Firefox/15.0.2.0 IceDragon/15.0.2.0*/
const Device UA2200093 = {S74076, 152, { &P18113,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XT912 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258032 = {S74077, 152, { &P18113,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT912 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2365772 = {S74078, 142, { &P15837,&P6909,&P24668,&P18092 }};	/*Useragent - NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-05) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2239390 = {S74079, 77, { &P15364,&P17285,&P22554,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:16.1) Gecko/X Firefox/16.1 IceDragon/16.1.0.1*/
const Device UA2362532 = {S74080, 138, { &P12788,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PG86100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2400232 = {S74081, 138, { &P12788,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PG86100 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2482580 = {S74082, 142, { &P12788,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Evo 3D CDMA Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2383141 = {S74083, 137, { &P23053,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN700W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2446007 = {S74084, 128, { &P23053,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN700W Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2498726 = {S74085, 59, { &P23053,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GN700W Build/IMM76D)*/
const Device UA2692885 = {S74086, 150, { &P23053,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn;GiONEE-GN700W/GN700W_T15 Build/IMM76D) AppleWebKit534.30(KHTML,like Gecko)Version/4.0 Mobile Safari/534.30*/
const Device UA2693965 = {S74087, 150, { &P23053,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn;GiONEE-GN700W/GN700W_T16 Build/IMM76D) AppleWebKit534.30(KHTML,like Gecko)Version/4.0 Mobile Safari/534.30*/
const Device UA2830285 = {S74088, 150, { &P23053,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn;GiONEE-GN700W/GN700W_T11 Build/IMM76D) AppleWebKit534.30(KHTML,like Gecko)Version/4.0 Mobile Safari/534.30*/
const Device UA2400631 = {S74089, 154, { &P18673,&P18066,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2466300 = {S74090, 155, { &P18673,&P18066,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562i Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423739 = {S74091, 138, { &P14183,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-I997 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2560810 = {S74092, 137, { &P14183,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I997 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2753782 = {S74093, 136, { &P22685,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E460 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782934 = {S74094, 142, { &P22997,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Celkon A107 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2875375 = {S74095, 132, { &P23219,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; PMP5870C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3062103 = {S74096, 93, { &P11929,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5230) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3080656 = {S74097, 137, { &P19894,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R830 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA22439 = {S74098, 77, { &P14168,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_Touch_T2223 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA23995 = {S74099, 82, { &P14168,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_Touch_Viva_T2223 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA42184 = {S74100, 117, { &P14168,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC_Touch_Viva_T2223; OpVer 86.157.15.753*/
const Device UA55075 = {S74101, 47, { &P14281,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-VX8360/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA60872 = {S74102, 180, { &P12589,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaC5-04.1/10.1.031; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.9 3gpp-gba*/
const Device UA371156 = {S74103, 130, { &P21979,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; tegav2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2338758 = {S74104, 130, { &P21979,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; tegav2 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA606464 = {S74105, 125, { &P15364,&P5690,&P22427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.152 Safari/535.19 Comodo_Dragon/18.1.2.0*/
const Device UA626648 = {S74106, 81, { &P15364,&P5690,&P18200,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; fr-FR; rv:1.7.5) Gecko/X Thunderbird/1.0*/
const Device UA659668 = {S74107, 133, { &P23839,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; de-de; SPX-5 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA709090 = {S74108, 60, { &P15364,&P18146,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; SunOS i86pc; rv:12.0) Gecko/X Firefox/12.0*/
const Device UA760861 = {S74109, 137, { &P23998,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; PTAB4300 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2455715 = {S74110, 68, { &P13452,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2; zh-cn; GT-P1000; 600*1024) UCWEB7.9.0.94/139/444*/
const Device UA2457087 = {S74111, 70, { &P13452,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; GT-P1000; 600*1024) UCWEB7.9.0.94/139/444*/
const Device UA899663 = {S74112, 122, { &P15364,&P16110,&P22458,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5*/
const Device UA901400 = {S74113, 115, { &P15364,&P16110,&P22458,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5*/
const Device UA2462050 = {S74114, 175, { &P15798,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2456377 = {S74115, 140, { &P22048,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; BLU_P100i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2468233 = {S74116, 133, { &P18680,&P18110,&P19805,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.6 Safari/535.19 Silk-Accelerated=true*/
const Device UA2597126 = {S74117, 131, { &P22678,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MAXWEST Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2865156 = {S74118, 138, { &P22678,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MAXWEST Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3014501 = {S74119, 148, { &P22678,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MAXWEST_Orbit4150 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2656175 = {S74120, 144, { &P20827,&P17979,&P18062,&P18092 }};	/*Useragent - MT6515-MT6515-C909/1.0 Linux/Apps_4.0.32.9-perf Android/4.0.3 Release/11.20.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2658046 = {S74121, 144, { &P20827,&P17979,&P18062,&P18092 }};	/*Useragent - MT6515-MT6515-H003/1.0 Linux/Apps_4.0.32.9-perf Android/4.0.3 Release/11.30.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2730717 = {S74122, 144, { &P20827,&P17979,&P18062,&P18092 }};	/*Useragent - MT6515-MT6515-H003/1.0 Linux/Apps_4.0.32.9-perf Android/4.0.3 Release/01.11.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2779591 = {S74123, 144, { &P20827,&P17979,&P18062,&P18092 }};	/*Useragent - MT6515-MT6515-H003/1.0 Linux/Apps_4.0.32.9-perf Android/4.0.3 Release/01.12.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3047782 = {S74124, 144, { &P20827,&P17979,&P18062,&P18092 }};	/*Useragent - MT6515-MT6515-H003/1.0 Linux/Apps_4.0.32.9-perf Android/4.0.3 Release/03.20.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3073529 = {S74125, 144, { &P20827,&P17979,&P18062,&P18092 }};	/*Useragent - MT6515-MT6515-C909/1.0 Linux/Apps_4.0.32.9-perf Android/4.0.3 Release/03.05.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2703597 = {S74126, 139, { &P22954,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; LG-E455f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2995727 = {S74127, 61, { &P22954,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-E455f Build/JZO54K)*/
const Device UA2770411 = {S74128, 161, { &P23131,&P18066,&P18062,&P18092 }};	/*Useragent - ultrafone 303 Linux/3.0.13 Android/4.0.4 Release/04.11.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA3088425 = {S74129, 144, { &P23131,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ultrafone 303 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3033045 = {S74130, 156, { &P18621,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3033332 = {S74131, 156, { &P18621,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E610 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3078972 = {S74132, 157, { &P18621,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3079801 = {S74133, 157, { &P18621,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612f Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3080012 = {S74134, 130, { &P18475,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080596 = {S74135, 131, { &P18475,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320T Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3115569 = {S74136, 114, { &P14540,&P5843,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-01/08.70; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA234707 = {S74137, 129, { &P15364,&P7133,&P22407,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.0.0.0 Chrome/13.0.782.107 Safari/535.1*/
const Device UA234728 = {S74138, 122, { &P15364,&P7133,&P22407,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.0.0.0 Chrome/13.0.782.107 Safari/535.1*/
const Device UA807445 = {S74139, 104, { &P15364,&P17285,&P22516,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.8.1.8pre) Gecko/X Firefox/2.0.0.8 Navigator/9.0.0.1*/
const Device UA812982 = {S74140, 78, { &P15364,&P5690,&P21927,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) chromeframe/7.0.517.44*/
const Device UA2250176 = {S74141, 224, { &P15364,&P5690,&P21927,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB0.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.04506.648; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729) chromeframe/7.0.517.44*/
const Device UA609415 = {S74142, 139, { &P14872,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  SPH-D700 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA910309 = {S74143, 138, { &P14872,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D700 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2029036 = {S74144, 138, { &P14872,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SPH-D700 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1988423 = {S74145, 172, { &P20134,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA1981085 = {S74146, 129, { &P20563,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; AT500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1994203 = {S74147, 130, { &P20563,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; AT500a Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1982160 = {S74148, 155, { &P21357,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; iBall Andi3e Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1996044 = {S74149, 148, { &P21093,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; MEDION LIFE P4012 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025449 = {S74150, 130, { &P23877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; LT-TLA Build/LTLMR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257313 = {S74151, 154, { &P18465,&P18066,&P18690,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.2 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2559656 = {S74152, 137, { &P22067,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; id-id; IMO Z7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986578 = {S74153, 59, { &P22067,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; IMO Z7 Build/JRO03C)*/
const Device UA2565729 = {S74154, 137, { &P21737,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; IRIS355 Build/IRIS355) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2663539 = {S74155, 135, { &P21660,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Novo7 Venus Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2778885 = {S74156, 64, { &P21660,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Novo7 Venus Build/JRO03C)*/
const Device UA2676280 = {S74157, 130, { &P22037,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3113 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2713809 = {S74158, 130, { &P22037,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P3113 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708112 = {S74159, 143, { &P23838,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; StarTrail III Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2713775 = {S74160, 141, { &P23107,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; TA-FONE A37 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2960834 = {S74161, 144, { &P23107,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; TA-FONE A37 HD Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2736808 = {S74162, 147, { &P23110,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; Aqua Glow Build/MocorDroid2.3.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2974115 = {S74163, 143, { &P20816,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WIKO-CINK SLIM Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3083199 = {S74164, 129, { &P21772,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; D2-721G Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA51234 = {S74165, 82, { &P11628,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-E250i/E250iJBIB2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA86478 = {S74166, 60, { &P11628,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-E250/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2720561 = {S74167, 95, { &P17595,&P18120,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en; rv:1.7.12) Gecko/X Firefox/1.0.7 Madfox/3.0*/
const Device UA2168821 = {S74168, 84, { &P14983,&P5055,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; NokiaC7-00)/UC Browser8.5.0.183/51/355/UCWEB Mobile*/
const Device UA2440987 = {S74169, 32, { &P14983,&P5055,&P24671,&P18092 }};	/*Useragent - NokiaC7-00/UCWEB8.5.0.163/51/999*/
const Device UA2552023 = {S74170, 129, { &P14983,&P5055,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiac7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile  3gpp-gba*/
const Device UA2425253 = {S74171, 66, { &P17625,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.7; zh-cn; ST25i; 480*854) UCWEB7.9.0.94/139/444*/
const Device UA2843657 = {S74172, 66, { &P17625,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; ST25i; 480*854) UCWEB7.9.0.94/139/444*/
const Device UA2431768 = {S74173, 171, { &P19862,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax A110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2470458 = {S74174, 154, { &P18466,&P18066,&P19805,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.6 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2507041 = {S74175, 146, { &P21804,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Allview P5-Mini Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987183 = {S74176, 138, { &P21804,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ro-ro; P5-Mini Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567586 = {S74177, 131, { &P21012,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I815 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583125 = {S74178, 131, { &P20011,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Next7P12 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2603301 = {S74179, 134, { &P20011,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Next7P12-8G Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2674223 = {S74180, 130, { &P20968,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Le Pan S Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2734933 = {S74181, 159, { &P23832,&P18066,&P18062,&P18092 }};	/*Useragent - skk Phoenix Linux/3.0.13 Android/4.0.4 Release/03.13.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2936875 = {S74182, 142, { &P23832,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; skk Phoenix Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2863169 = {S74183, 144, { &P23386,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Vandroid T3-A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3039329 = {S74184, 161, { &P19862,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax A110 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA40130 = {S74185, 143, { &P12695,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ME860 Build/OLHKT_U4_0.50.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40161 = {S74186, 143, { &P12695,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; ME860 Build/OLHKT_U4_0.56.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40168 = {S74187, 182, { &P12695,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; Moto_ME860 Build/Blur_Version.4.0.560.ME860.Retail.en.05 Flex/P019) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48379 = {S74188, 143, { &P12695,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME860 Build/OLYGC_U4_1.27.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55718 = {S74189, 182, { &P12695,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Moto_ME860 Build/Blur_Version.4.0.500.ME860.Retail.en.05 Flex/P017) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61362 = {S74190, 143, { &P12695,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME860 Build/OLYGC_U4_1.21.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72780 = {S74191, 143, { &P12695,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME860 Build/OLYGC_U4_1.33.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA75480 = {S74192, 143, { &P12695,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME860 Build/OLYGC_U4_1.31.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020905 = {S74193, 131, { &P12695,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME860 Build/OLYGC_U4_1.33.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA171147 = {S74194, 137, { &P16541,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; SHW-M220L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018895 = {S74195, 61, { &P16541,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SHW-M220L Build/FROYO)*/
const Device UA652952 = {S74196, 71, { &P15364,&P18146,&P20900,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; SunOS sun4u; en-US; rv:1.0.1) Gecko/X Netscape/7.0*/
const Device UA2024831 = {S74197, 136, { &P21165,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ZP300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2507750 = {S74198, 139, { &P21977,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; ENERGY s10 DUAL Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2535270 = {S74199, 93, { &P15649,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5570) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2566631 = {S74200, 127, { &P19991,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A511 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567113 = {S74201, 134, { &P20129,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MI 2 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2595731 = {S74202, 144, { &P13094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LNV-Lenovo_A600e Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747925 = {S74203, 144, { &P13094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LNV-Lenovo A600e Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2844096 = {S74204, 135, { &P13094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LNV-Lenovo A600e Build/ICS) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2934434 = {S74205, 66, { &P13094,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LNV-Lenovo A600e Build/ICS)*/
const Device UA2774327 = {S74206, 158, { &P20947,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTCEVOV4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2754020 = {S74207, 162, { &P12972,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; SGH-T989 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2997549 = {S74208, 163, { &P12972,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; SGH-T989D Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2755058 = {S74209, 140, { &P23910,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT30at Build/7.0.B.1.152) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783284 = {S74210, 153, { &P23175,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5021E Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2975552 = {S74211, 126, { &P16598,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974644 = {S74212, 138, { &P18292,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ609 Build/7.7.1-85_MZ609-31) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974760 = {S74213, 133, { &P19775,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; A210 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3078467 = {S74214, 126, { &P19775,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; A210 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3066219 = {S74215, 134, { &P18605,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTL21 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083495 = {S74216, 145, { &P19891,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I717 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3116585 = {S74217, 132, { &P15767,&P19826,&P24462,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.1.0.64021 Mobile/10B141 Safari/7534.48.3*/
const Device UA3116586 = {S74218, 134, { &P15767,&P19826,&P24462,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.1.0.64021 Mobile/10B147 Safari/7534.48.3*/
const Device UA3116594 = {S74219, 134, { &P15767,&P19826,&P24462,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.1.1.64324 Mobile/10B329 Safari/7534.48.3*/
const Device UA3129530 = {S74220, 134, { &P15767,&P19826,&P24462,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.1.1.64324 Mobile/10B146 Safari/7534.48.3*/
const Device UA46719 = {S74221, 74, { &P12313,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-B130-ORANGE/B130ABHI2 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA55777 = {S74222, 67, { &P12313,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-B130/B130XAHF4 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA87394 = {S74223, 62, { &P12313,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-B130L/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA219787 = {S74224, 101, { &P15364,&P17188,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA327837 = {S74225, 188, { &P15364,&P7133,&P4779,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/28/351*/
const Device UA331065 = {S74226, 188, { &P15364,&P7133,&P4779,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/50/351*/
const Device UA421178 = {S74227, 188, { &P15364,&P7133,&P4779,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/28/355*/
const Device UA697708 = {S74228, 188, { &P15364,&P7133,&P4779,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/50/355*/
const Device UA726527 = {S74229, 189, { &P15364,&P7133,&P4779,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/159/352*/
const Device UA759050 = {S74230, 188, { &P15364,&P7133,&P4779,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/50/352*/
const Device UA832267 = {S74231, 189, { &P15364,&P7133,&P4779,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/159/444*/
const Device UA860275 = {S74232, 188, { &P15364,&P7133,&P4779,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/27/351*/
const Device UA1896775 = {S74233, 189, { &P15364,&P7133,&P4779,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/160/352*/
const Device UA1900713 = {S74234, 189, { &P15364,&P7133,&P4779,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/159/351*/
const Device UA2142148 = {S74235, 189, { &P15364,&P7133,&P4779,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/159/350*/
const Device UA2293249 = {S74236, 188, { &P15364,&P7133,&P4779,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.8.0.95/27/352*/
const Device UA436422 = {S74237, 30, { &P15364,&P22725,&P22726,&P18092 }};	/*Useragent - Mozilla/5.0 (Nokia; Qt; MeeGo)*/
const Device UA875886 = {S74238, 120, { &P15364,&P5690,&P22437,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Comodo_Dragon/1.0.0.5 Safari/532.0*/
const Device UA2207233 = {S74239, 142, { &P15364,&P5690,&P22437,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Comodo_Dragon/1.0.0.9 Chrome/ Version/3.2.1 Safari/532.0*/
const Device UA2999085 = {S74240, 120, { &P15364,&P5690,&P22437,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Comodo_Dragon/1.0.0.5 Safari/532.0*/
const Device UA1953308 = {S74241, 77, { &P15364,&P18110,&P21940,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1.16) Gecko/X Icedove/3.0.11*/
const Device UA2148608 = {S74242, 75, { &P15364,&P18110,&P21940,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.16) Gecko/X Icedove/3.0.11*/
const Device UA1986425 = {S74243, 13, { &P15364,&P17483,&P17660,&P18092 }};	/*Useragent - LGCMF 0.8.1.5*/
const Device UA2335132 = {S74244, 18, { &P15364,&P17483,&P17660,&P18092 }};	/*Useragent - LGMC-LGEAS/5.074IV*/
const Device UA2482461 = {S74245, 17, { &P15364,&P17483,&P17660,&P18092 }};	/*Useragent - LGMC-LGEAS/5.23IV*/
const Device UA2543965 = {S74246, 13, { &P15364,&P17483,&P17660,&P18092 }};	/*Useragent - LGCMF 0.8.1.7*/
const Device UA2593644 = {S74247, 18, { &P15364,&P17483,&P17660,&P18092 }};	/*Useragent - LGMC-LGEAS/5.09L1A*/
const Device UA2954445 = {S74248, 17, { &P15364,&P17483,&P17660,&P18092 }};	/*Useragent - LGMC-LGEAS/6.44.4*/
const Device UA2978552 = {S74249, 18, { &P15364,&P17483,&P17660,&P18092 }};	/*Useragent - LGE-bryce-VS910 4G*/
const Device UA2999647 = {S74250, 17, { &P15364,&P17483,&P17660,&P18092 }};	/*Useragent - LGMC-LGEAS/6.44.9*/
const Device UA2437634 = {S74251, 141, { &P18372,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SO-04D Build/7.0.D.1.130) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2168372 = {S74252, 71, { &P20034,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X294/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2391565 = {S74253, 69, { &P15364,&P7133,&P19821,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:18.0) Gecko/X Thunderbird/18.0*/
const Device UA2460779 = {S74254, 153, { &P23893,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TP10.1-1500DC Dark grey metal Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2565565 = {S74255, 142, { &P21738,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; iris502 Build/LAVAIRIS502) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2567947 = {S74256, 137, { &P18234,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P895 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2702388 = {S74257, 139, { &P18234,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P895qb Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567692 = {S74258, 139, { &P22591,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583425 = {S74259, 137, { &P20937,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR910L Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2600908 = {S74260, 137, { &P20937,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR910L Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2795425 = {S74261, 147, { &P20129,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2875471 = {S74262, 148, { &P20129,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2S Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2958023 = {S74263, 148, { &P20129,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2A Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2958486 = {S74264, 148, { &P20129,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2C Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2958494 = {S74265, 149, { &P20129,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2SC Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA22901 = {S74266, 75, { &P12048,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) T-Mobile Dash*/
const Device UA307980 = {S74267, 140, { &P18081,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SKY IM-A600S Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018916 = {S74268, 64, { &P18081,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SKY IM-A600S Build/FRG83)*/
const Device UA2019396 = {S74269, 138, { &P18081,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-; SKY IM-A600S Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022238 = {S74270, 134, { &P21473,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT319 Build/V4.37B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023001 = {S74271, 134, { &P21473,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; XT319 Build/V4.39A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025921 = {S74272, 134, { &P21473,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; XT319 Build/V4.50A) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2643478 = {S74273, 127, { &P21473,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; XT319 Build/V4.50A) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2721812 = {S74274, 134, { &P21473,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; XT319 Build/V4.560) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA399256 = {S74275, 43, { &P15364,&P21445,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows ME; U; en) Presto/2.1.1*/
const Device UA414809 = {S74276, 43, { &P15364,&P21445,&P18337,&P18092 }};	/*Useragent - Opera/9.60 (Windows ME; U; en) Presto/2.1.1*/
const Device UA2278073 = {S74277, 43, { &P15364,&P21445,&P18337,&P18092 }};	/*Useragent - Opera/9.62 (Windows ME; U; en) Presto/2.1.1*/
const Device UA3040917 = {S74278, 43, { &P15364,&P21445,&P18337,&P18092 }};	/*Useragent - Opera/9.63 (Windows ME; U; de) Presto/2.1.1*/
const Device UA649547 = {S74279, 113, { &P17475,&P5768,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia6300/07.21; Profile/MIDP-2.0 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA581259 = {S74280, 137, { &P20000,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; pt-br; MZ607 Build/1.6.0_218.11-MZ607) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2026256 = {S74281, 140, { &P20000,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; pt-br; MZ607 Build/1.6.0M_272.15_MZ607) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2816548 = {S74282, 142, { &P20000,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; pt-br; MZ607 Build/1.6.0M_272.15_MZ607) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA606828 = {S74283, 43, { &P17595,&P17297,&P21584,&P18092 }};	/*Useragent - MacOutlook/14.2.0.X (Intel Mac OS X 10.7.3)*/
const Device UA613759 = {S74284, 43, { &P17595,&P17297,&P21584,&P18092 }};	/*Useragent - MacOutlook/14.2.1.X (Intel Mac OS X 10.7.3)*/
const Device UA628117 = {S74285, 43, { &P17595,&P17297,&P21584,&P18092 }};	/*Useragent - MacOutlook/14.2.2.X (Intel Mac OS X 10.7.4)*/
const Device UA630540 = {S74286, 43, { &P17595,&P17297,&P21584,&P18092 }};	/*Useragent - MacOutlook/14.2.2.X (Intel Mac OS X 10.7.3)*/
const Device UA798864 = {S74287, 43, { &P17595,&P17297,&P21584,&P18092 }};	/*Useragent - MacOutlook/14.2.1.X (Intel Mac OS X 10.7.4)*/
const Device UA1991778 = {S74288, 43, { &P17595,&P17297,&P21584,&P18092 }};	/*Useragent - MacOutlook/14.2.3.X (Intel Mac OS X 10.7.4)*/
const Device UA2234795 = {S74289, 43, { &P17595,&P17297,&P21584,&P18092 }};	/*Useragent - MacOutlook/14.2.4.X (Intel Mac OS X 10.7.5)*/
const Device UA2389776 = {S74290, 43, { &P17595,&P17297,&P21584,&P18092 }};	/*Useragent - MacOutlook/14.2.5.X (Intel Mac OS X 10.7.5)*/
const Device UA2416622 = {S74291, 43, { &P17595,&P17297,&P21584,&P18092 }};	/*Useragent - MacOutlook/14.2.3.X (Intel Mac OS X 10.7.5)*/
const Device UA2831111 = {S74292, 43, { &P17595,&P17297,&P21584,&P18092 }};	/*Useragent - MacOutlook/14.2.2.X (Intel Mac OS X 10.7.5)*/
const Device UA613592 = {S74293, 73, { &P15364,&P18110,&P21171,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:12.0) Gecko/X Firefox/12.0 SeaMonkey/2.9*/
const Device UA781083 = {S74294, 132, { &P24006,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; TAB-1012 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1995233 = {S74295, 143, { &P21111,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS12M Build/6.7.5-25_LE-8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2230737 = {S74296, 145, { &P21111,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS12M Build/6.7.5-50_MR1-11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2233834 = {S74297, 190, { &P15575,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, gb*/
const Device UA2354966 = {S74298, 190, { &P15575,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, gb*/
const Device UA2390634 = {S74299, 87, { &P15364,&P16110,&P22555,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:17.0) Gecko/17.0 Firefox/17.0 IceDragon/17.0.1.0*/
const Device UA2431078 = {S74300, 171, { &P18304,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2451079 = {S74301, 182, { &P13035,&P5043,&P24671,&P18092 }};	/*Useragent - NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-02) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2421585 = {S74302, 73, { &P18652,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; Micromax_A87; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2533202 = {S74303, 168, { &P14557,&P5843,&P24676,&P18092 }};	/*Useragent - Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6303classic) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2585171 = {S74304, 139, { &P16044,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cnm TOUCHPAD 9.7 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2601584 = {S74305, 147, { &P16044,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Versus TouchPad 7 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667299 = {S74306, 144, { &P16044,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CnM TOUCHPAD 9 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668270 = {S74307, 138, { &P16044,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Vs TOUCHPAD 9.7 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2857052 = {S74308, 147, { &P16044,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Versus TouchPad 9 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2659779 = {S74309, 116, { &P12473,&P5204,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2730c-1/10.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2774151 = {S74310, 159, { &P18447,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.1.140) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3032402 = {S74311, 129, { &P24458,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; KUNO4 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA24758 = {S74312, 67, { &P15754,&P17017,&P3807,&P18092 }};	/*Useragent - Huawei/1.0/0HuaweiG3610/WAP2.0/Obigo-Browser/Q03C MMS/Obigo-MMS/1.2*/
const Device UA58672 = {S74313, 136, { &P15754,&P17017,&P3807,&P18092 }};	/*Useragent - Huawei/1.0/0HuaweiG3610/WAP2.0/Obigo-Browser/Q03C MMS/Obigo-MMS/1.2, Huawei/1.0/0HuaweiG3610/WAP2.0/Obigo-Browser/Q03C MMS/Obigo-MMS/1.2*/
const Device UA31165 = {S74314, 66, { &P12404,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KM555/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA222292 = {S74315, 101, { &P15364,&P7127,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA1932348 = {S74316, 67, { &P12798,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.4;Zh_cn;LT18i;480*854;)UCWEB7.8.0.95/139/351*/
const Device UA2439532 = {S74317, 116, { &P12798,&P5629,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.3.4;Zh_cn;LT18i;480*854;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA923994 = {S74318, 198, { &P18645,&P18066,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, us*/
const Device UA1994342 = {S74319, 191, { &P18645,&P18066,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; SGH-I747M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, ca*/
const Device UA2167324 = {S74320, 190, { &P18645,&P18066,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-L710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, us*/
const Device UA2445587 = {S74321, 191, { &P18645,&P18066,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-R530U Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, us*/
const Device UA1997072 = {S74322, 64, { &P15364,&P18145,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64; rv:14.0) Gecko/X Firefox/14.0.1*/
const Device UA2362058 = {S74323, 133, { &P18465,&P18110,&P19815,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Safari/535.19 Silk-Accelerated=false*/
const Device UA2401868 = {S74324, 80, { &P20093,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-E2250/E2250XXLG2 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2405061 = {S74325, 134, { &P14997,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S100 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2566896 = {S74326, 149, { &P17835,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook Tablet Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2702622 = {S74327, 133, { &P17835,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; NookTablet Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2597111 = {S74328, 156, { &P15837,&P6909,&P24676,&P18092 }};	/*Useragent - NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-05) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2675553 = {S74329, 117, { &P15364,&P5690,&P19995,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22 Sleipnir/4.0.1*/
const Device UA2676934 = {S74330, 137, { &P20013,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F180L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707462 = {S74331, 137, { &P20013,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F180S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713936 = {S74332, 137, { &P20013,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F180K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2772769 = {S74333, 159, { &P18564,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X+ Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2964049 = {S74334, 164, { &P21648,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9505 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/jfltexx*/
const Device UA2974369 = {S74335, 133, { &P20156,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; S500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974885 = {S74336, 138, { &P14854,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020946 = {S74337, 138, { &P14854,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021776 = {S74338, 138, { &P14854,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976690 = {S74339, 140, { &P23910,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT30at Build/7.0.B.1.152) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3082758 = {S74340, 136, { &P22929,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_E10310 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3131713 = {S74341, 137, { &P16356,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA30927 = {S74342, 152, { &P12371,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; SonyEricssonSO-01B Build/R1EA029) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA528581 = {S74343, 35, { &P15364,&P18110,&P22484,&P18092 }};	/*Useragent - Opera/9.10 (X11; Linux i686; U; en)*/
const Device UA368205 = {S74344, 123, { &P15364,&P7133,&P22420,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.1.0.0 Chrome/17.0.963.38 Safari/535.11*/
const Device UA368204 = {S74345, 130, { &P15364,&P7133,&P22420,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.1.0.0 Chrome/17.0.963.38 Safari/535.11*/
const Device UA369749 = {S74346, 138, { &P15873,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ViewPhone3 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA640509 = {S74347, 142, { &P21997,&P5629,&P15760,&P18092 }};	/*Useragent - Spice Mi280 Linux/2.6.35 Android/2.3.4 Release/12.09.2011 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1*/
const Device UA718526 = {S74348, 139, { &P21997,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Spice Mi280 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA903768 = {S74349, 77, { &P15364,&P17285,&P21170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:13.0) Gecko/X Firefox/13.0.1 SeaMonkey/2.10.1*/
const Device UA2028631 = {S74350, 132, { &P12889,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P7300 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2261653 = {S74351, 132, { &P12889,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7300 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2384863 = {S74352, 180, { &P18510,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; ALCATEL ONE TOUCH 918N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2033666 = {S74353, 129, { &P22242,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ME171 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2168910 = {S74354, 200, { &P23069,&P22857,&P18373,&P18092 }};	/*Useragent - MOT-EX440 MIDP-2.0/CLDC-1.1 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA2239095 = {S74355, 137, { &P12972,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; SGH-T989 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2425070 = {S74356, 131, { &P23287,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MID1018 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2442173 = {S74357, 130, { &P18103,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; FZ-A1B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2459209 = {S74358, 135, { &P22093,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; DPS Dream 7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2567264 = {S74359, 131, { &P22041,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6810 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2668244 = {S74360, 141, { &P12280,&P18117,&P22589,&P18092 }};	/*Useragent - MQQBrowser/42 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA2737643 = {S74361, 142, { &P12280,&P18117,&P22589,&P18092 }};	/*Useragent - MQQBrowser/421 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA2780215 = {S74362, 140, { &P12280,&P18117,&P22589,&P18092 }};	/*Useragent - MQQBrowser/42 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3*/
const Device UA3073370 = {S74363, 140, { &P12280,&P18117,&P22589,&P18092 }};	/*Useragent - MQQBrowser/421 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3*/
const Device UA2676935 = {S74364, 140, { &P20285,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ616-12) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708692 = {S74365, 145, { &P20285,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ616 Build/7.7.1-141-6-PAST-UMTS-LA) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2673941 = {S74366, 137, { &P20975,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-AS730 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2710448 = {S74367, 92, { &P14453,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE51) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2718551 = {S74368, 161, { &P18507,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I8160 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3021817 = {S74369, 126, { &P18465,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KFOT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022686 = {S74370, 134, { &P18465,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KF012 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA5316 = {S74371, 76, { &P12248,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-E1360B-Bouygues/E1360BAGJC1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA24453 = {S74372, 67, { &P12248,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-E1360B/E1360BXAID3 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA45937 = {S74373, 67, { &P12248,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-E1360M/E1360MXEID2 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA64033 = {S74374, 74, { &P12248,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-E1360B-ORANGE/E1360BBVID1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA64057 = {S74375, 65, { &P12248,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-E1360/E1360JAIC1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA1842205 = {S74376, 78, { &P12248,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-E1360B/ARID1/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA5792 = {S74377, 67, { &P12289,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia1680c_CMCC/2.0 (05.61) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA30253 = {S74378, 62, { &P12289,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia1680c/2.0 (05.63) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA70012 = {S74379, 53, { &P15159,&P17017,&P3534,&P18092 }};	/*Useragent - Huawei-U2800/U2800ACBXB119 Browser/Obigo-Browser/Q05A*/
const Device UA250315 = {S74380, 122, { &P15364,&P7133,&P22411,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.1.1.0 Chrome/14.0.835.202 Safari/535.1*/
const Device UA250322 = {S74381, 129, { &P15364,&P7133,&P22411,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.1.1.0 Chrome/14.0.835.202 Safari/535.1*/
const Device UA396301 = {S74382, 74, { &P20372,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-Z221/1.3.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA399147 = {S74383, 106, { &P15775,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800; SFR)*/
const Device UA399153 = {S74384, 111, { &P15775,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800; T-Mobile)*/
const Device UA399166 = {S74385, 101, { &P15775,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800)*/
const Device UA399169 = {S74386, 109, { &P15775,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800; Orange)*/
const Device UA399170 = {S74387, 111, { &P15775,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800; Vodafone)*/
const Device UA925851 = {S74388, 101, { &P15775,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Nokia 800)*/
const Device UA925850 = {S74389, 109, { &P15775,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800; Meteor)*/
const Device UA925882 = {S74390, 115, { &P15775,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 800; 800 BOUYGUES)*/
const Device UA2383344 = {S74391, 156, { &P14682,&P5843,&P24668,&P18092 }};	/*Useragent - Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia6303iclassic) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2505195 = {S74392, 133, { &P22152,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAC-10011 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2674203 = {S74393, 133, { &P18113,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT912 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709183 = {S74394, 161, { &P13974,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5670 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2713884 = {S74395, 161, { &P13974,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-S5670 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2713954 = {S74396, 151, { &P18442,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2714811 = {S74397, 151, { &P18442,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2737302 = {S74398, 151, { &P18442,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-N719 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2764274 = {S74399, 137, { &P22950,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E977 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3083147 = {S74400, 138, { &P14854,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093615 = {S74401, 138, { &P14854,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114923 = {S74402, 138, { &P14854,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA61664 = {S74403, 87, { &P12079,&P17485,&P4203,&P18092 }};	/*Useragent - SonyEricssonT303c/R2CC001 TelecaBrowser/Q04C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64506 = {S74404, 86, { &P12079,&P17485,&P4203,&P18092 }};	/*Useragent - SonyEricssonT303/R2CC001 TelecaBrowser/Q04C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2128824 = {S74405, 66, { &P15364,&P5690,&P20418,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.50 [en]*/
const Device UA624531 = {S74406, 123, { &P13838,&P5776,&P24663,&P18092 }};	/*Useragent - Nokia6600/1.0 (5.53.0) SymbianOS/7.0s Series60/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.9.1.120/27/351/UCWEB*/
const Device UA911391 = {S74407, 139, { &P15850,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LU6200 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2840807 = {S74408, 139, { &P15850,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-lu6200 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1963425 = {S74409, 95, { &P21021,&P17979,&P18062,&P18092 }};	/*Useragent - HTCT328t_TD/1.0 Android/4.0 release/2012 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2168025 = {S74410, 130, { &P15767,&P18117,&P20777,&P18092 }};	/*Useragent - MQQBrowser/2.6 Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA2260140 = {S74411, 128, { &P15767,&P18117,&P20777,&P18092 }};	/*Useragent - MQQBrowser/2.6 Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3*/
const Device UA2192568 = {S74412, 51, { &P18088,&P18432,&P24673,&P18092 }};	/*Useragent - IUC(U;iOS 6.0;Zh-cn;320*480;)/UCWEB8.7.1.201/44/997*/
const Device UA2230045 = {S74413, 154, { &P14967,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LG-P500 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2530017 = {S74414, 154, { &P14967,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P500 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2365677 = {S74415, 135, { &P23160,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KB901 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2367016 = {S74416, 140, { &P23699,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SIMM-X720 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2687268 = {S74417, 133, { &P23699,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SIMM-X720 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2790346 = {S74418, 128, { &P23699,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; X720 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2503063 = {S74419, 139, { &P16263,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Huawei MediaPad Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2701945 = {S74420, 138, { &P16263,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; dtab01 Build/HuaweiMediaPad) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2865018 = {S74421, 148, { &P16263,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 10 LINK Build/HuaweiMediaPad) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2567479 = {S74422, 74, { &P18141,&P21717,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 2.06) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2671095 = {S74423, 136, { &P20937,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR910L Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673227 = {S74424, 136, { &P20937,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR910L Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2737929 = {S74425, 187, { &P13157,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2717883 = {S74426, 158, { &P18448,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; ST26i Build/11.2.A.0.21) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2869297 = {S74427, 158, { &P18448,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; ST26i Build/11.2.A.0.31) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2734222 = {S74428, 136, { &P23256,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; N7102 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2753335 = {S74429, 144, { &P18348,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867583 = {S74430, 150, { &P18348,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G X710s Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774291 = {S74431, 159, { &P12798,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT18i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2894723 = {S74432, 159, { &P12798,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LT18i Build/4.1.A.0.562) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2796774 = {S74433, 160, { &P18635,&P18598,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA3074968 = {S74434, 161, { &P18635,&P18598,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2879288 = {S74435, 101, { &P22579,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 925)*/
const Device UA2941025 = {S74436, 111, { &P22579,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 925; Vodafone)*/
const Device UA3093382 = {S74437, 109, { &P22579,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 925; 1326-2)*/
const Device UA3107056 = {S74438, 109, { &P22579,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 925; ANZ892)*/
const Device UA2894361 = {S74439, 140, { &P23680,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AURUS III Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA3080710 = {S74440, 138, { &P20289,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC6435LVW Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA88788 = {S74441, 88, { &P16151,&P18110,&P3976,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Linux 2.6.22) NetFront/3.4 Kindle/2.1 (screen 824x1200; rotate)*/
const Device UA265067 = {S74442, 88, { &P16151,&P18110,&P3976,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Linux 2.6.22) NetFront/3.4 Kindle/2.5 (screen 824x1200; rotate)*/
const Device UA723696 = {S74443, 142, { &P20142,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5838 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020921 = {S74444, 130, { &P20142,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5838 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2031469 = {S74445, 135, { &P20142,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5838 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2722446 = {S74446, 142, { &P20142,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5838 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA724409 = {S74447, 144, { &P21210,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3;en-us; ViewSonic-ViewPad7e build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022942 = {S74448, 67, { &P21210,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.1; ViewPad7e Build/GINGERBREAD)*/
const Device UA2428003 = {S74449, 147, { &P21615,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-MOBILE i-STYLE 5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2503173 = {S74450, 148, { &P21615,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-MOBILE i-STYLE 5i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2564830 = {S74451, 131, { &P20832,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MIDC497 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2673630 = {S74452, 126, { &P18465,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KFOT Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2712062 = {S74453, 134, { &P18465,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KF012 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2772928 = {S74454, 157, { &P18471,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-66) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3029927 = {S74455, 152, { &P19748,&P5043,&P24679,&P18092 }};	/*Useragent - Nokia205/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia205) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3082870 = {S74456, 137, { &P19854,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R760 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3062277 = {S74457, 92, { &P13977,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaN73) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA61745 = {S74458, 104, { &P14317,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonW902/R3EA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.3*/
const Device UA63880 = {S74459, 117, { &P14317,&P17485,&P3976,&P18092 }};	/*Useragent - Mozilla/4.0 SonyEricssonW902v/R3EE Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.3*/
const Device UA244647 = {S74460, 137, { &P17595,&P17296,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.64.347 Chrome/13.0.782.218 Safari/535.1*/
const Device UA252830 = {S74461, 138, { &P17595,&P17296,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.68.1565 Chrome/14.0.835.202 Safari/535.1*/
const Device UA348757 = {S74462, 137, { &P17595,&P17296,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_3) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.71.128 Chrome/14.0.835.202 Safari/535.1*/
const Device UA589615 = {S74463, 137, { &P17595,&P17296,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_4) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.72.698 Chrome/14.0.835.202 Safari/535.1*/
const Device UA630337 = {S74464, 138, { &P17595,&P17296,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/534.24 (KHTML, like Gecko) RockMelt/0.9.58.494 Chrome/11.0.696.71 Safari/534.24*/
const Device UA640077 = {S74465, 139, { &P17595,&P17296,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.30 (KHTML, like Gecko) RockMelt/0.9.62.313 Chrome/12.0.742.122 Safari/534.30*/
const Device UA900333 = {S74466, 147, { &P17595,&P17296,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/534.13 (KHTML, like Gecko) RockMelt/0.9.48.51 Chrome/9.0.597.107 Safari/534.13*/
const Device UA914800 = {S74467, 147, { &P17595,&P17296,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.13 (KHTML, like Gecko) RockMelt/0.9.48.59 Chrome/9.0.597.107 Safari/534.13*/
const Device UA915820 = {S74468, 138, { &P17595,&P17296,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.24 (KHTML, like Gecko) RockMelt/0.9.58.494 Chrome/11.0.696.71 Safari/534.24*/
const Device UA1915655 = {S74469, 137, { &P17595,&P17296,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.71.128 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2134383 = {S74470, 149, { &P17595,&P17296,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/534.16 (KHTML, like Gecko) RockMelt/0.9.50.518 Chrome/10.0.648.205 Safari/534.16*/
const Device UA1895876 = {S74471, 76, { &P15364,&P17285,&P18194,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:6.0) Gecko/X Thunderbird/6.0 Lightning/1.0b5*/
const Device UA689328 = {S74472, 130, { &P21439,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5368 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA729050 = {S74473, 121, { &P21439,&P5629,&P15760,&P18092 }};	/*Useragent - SAMSUNG-GT-S5368_TD/1.0 Android/2.3.6 Release/12.15.2010 Browser/AppleWebKit533.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2021762 = {S74474, 142, { &P21439,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5368 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033886 = {S74475, 135, { &P21439,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5368 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA812257 = {S74476, 154, { &P13070,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-E739 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA764187 = {S74477, 137, { &P20238,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ko-kr; LG-LU5400 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1973675 = {S74478, 136, { &P21734,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-E920 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2412867 = {S74479, 129, { &P21734,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-E920 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2524682 = {S74480, 132, { &P21734,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; E920 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3086655 = {S74481, 142, { &P21734,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; DOEASY E920 B Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2218161 = {S74482, 132, { &P18731,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ONDA MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2402131 = {S74483, 143, { &P18731,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ONDA MID Build/ICS.g12refM1006.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2237165 = {S74484, 118, { &P13554,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN79) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile*/
const Device UA2835443 = {S74485, 119, { &P13554,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian79-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile*/
const Device UA2362840 = {S74486, 81, { &P17976,&P18598,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.2; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2389403 = {S74487, 83, { &P17976,&P18598,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.2.1; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2958027 = {S74488, 83, { &P17976,&P18598,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.2.2; Linux; Opera Tablet/ADY-X) Presto/2.11.355 Version/12.10*/
const Device UA2413780 = {S74489, 95, { &P20292,&P17979,&P18062,&P18092 }};	/*Useragent - HTCT528t_TD/1.0 Android/4.0 release/2012 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2535972 = {S74490, 138, { &P21853,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Y3 Build/HCL) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2767541 = {S74491, 146, { &P21853,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Connect-3G Build/HCL) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622311 = {S74492, 61, { &P12879,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-P7510 Build/JOP40D)*/
const Device UA2625381 = {S74493, 127, { &P12879,&P18598,&P19802,&P18092 }};	/*Useragent - FaithlifeStudyBible/2.2.10 (Linux; U; Android 4.2.1; en-US; GT-P7510 Build/cm_p4wifi-userdebug 4.2.1 JOP40D eng..X.X test-keys)*/
const Device UA2628121 = {S74494, 151, { &P12879,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7510 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MantanoReader*/
const Device UA2633279 = {S74495, 91, { &P12879,&P18598,&P19802,&P18092 }};	/*Useragent - com.google.android.youtube/4.3.9(Linux; U; Android 4.2.2; en_US; GT-P7510 Build/JDQ39) gzip*/
const Device UA2635313 = {S74496, 131, { &P12879,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7510 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3088031 = {S74497, 61, { &P12879,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P7510 Build/JDQ39E)*/
const Device UA2666917 = {S74498, 159, { &P20175,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI 1S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2733356 = {S74499, 160, { &P20175,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI 1SC Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2676438 = {S74500, 137, { &P12942,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I927 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2782446 = {S74501, 139, { &P23206,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cloud Z5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2799226 = {S74502, 93, { &P22872,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S5282) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2799959 = {S74503, 135, { &P20427,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SHL21 Build/S5170) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103230 = {S74504, 57, { &P20427,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SHL21 Build/S5170)*/
const Device UA2801268 = {S74505, 149, { &P22968,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HUAWEI P6-U06 Build/HuaweiP6-U06) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2888714 = {S74506, 137, { &P23588,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Glass 1 Build/XRR82) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2888715 = {S74507, 138, { &P23588,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Glass 1 Build/XRR72D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2888716 = {S74508, 143, { &P23588,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Glass 1 Build/IMM76L; XE7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2895069 = {S74509, 156, { &P15919,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-P3100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2961886 = {S74510, 156, { &P15919,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; GT-P3100 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3029197 = {S74511, 163, { &P22597,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I545 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltevzw*/
const Device UA3066939 = {S74512, 163, { &P22597,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I545 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltevzw*/
const Device UA3115724 = {S74513, 163, { &P22597,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I545 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltevzw*/
const Device UA3080126 = {S74514, 133, { &P21963,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Q800 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083830 = {S74515, 138, { &P14854,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093655 = {S74516, 138, { &P14854,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SHW-M250S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3095531 = {S74517, 138, { &P14854,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111329 = {S74518, 138, { &P14854,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA63009 = {S74519, 43, { &P14470,&P17017,&P3807,&P18092 }};	/*Useragent - Huawei/1.0/G6600/Browser/obigo-browser/q03c*/
const Device UA420999 = {S74520, 62, { &P13868,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;HD7;7.10;7720)*/
const Device UA698591 = {S74521, 68, { &P13868,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;HD7 T9292;7.10;8107)*/
const Device UA748092 = {S74522, 62, { &P13868,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;HD7;7.10;8107)*/
const Device UA925702 = {S74523, 68, { &P13868,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;HD7 T9292;7.10;7740)*/
const Device UA925740 = {S74524, 68, { &P13868,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;HD7 T9292;7.10;7720)*/
const Device UA925780 = {S74525, 68, { &P13868,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;HD7 T9292;7.10;8773)*/
const Device UA925900 = {S74526, 64, { &P13868,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;T9296;7.10;8107)*/
const Device UA925920 = {S74527, 62, { &P13868,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;HD7;7.10;7740)*/
const Device UA862554 = {S74528, 140, { &P15364,&P17838,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Ubuntu/9.10 Chromium/9.0.592.0 Chrome/9.0.592.0 Safari/534.13*/
const Device UA630459 = {S74529, 81, { &P14983,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaC7-00)/UC Browser8.2.0.132/50/355/UCWEB Mobile*/
const Device UA708585 = {S74530, 77, { &P15364,&P18110,&P15761,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux armv61; en-US; rv:1.9.1b2pre) Gecko/X Fennec/1.0a1*/
const Device UA1839374 = {S74531, 88, { &P15364,&P18110,&P15761,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux armv7l; en-US; rv:1.9.2.1) Gecko/X Firefox/3.6.1 Fennec/1.0.1*/
const Device UA1892078 = {S74532, 151, { &P15364,&P7127,&P18068,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 2.0; Windows NT 5.0; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)*/
const Device UA2660496 = {S74533, 63, { &P15364,&P7127,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; Windows NT 5.0; Trident/3.0)*/
const Device UA2791728 = {S74534, 63, { &P15364,&P7127,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; Windows NT 5.0; Trident/3.0)*/
const Device UA1992304 = {S74535, 155, { &P13123,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028653 = {S74536, 155, { &P13123,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022674 = {S74537, 137, { &P12784,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-gb; ZTE-Skate Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2527078 = {S74538, 140, { &P12784,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Skate Pro Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2652846 = {S74539, 142, { &P12784,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Skate Acqua Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455694 = {S74540, 159, { &P13018,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Neo V Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2486687 = {S74541, 128, { &P18733,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M532 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2566961 = {S74542, 136, { &P23190,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Marquis_MP977 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2581274 = {S74543, 97, { &P12375,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIANokia N81) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2661155 = {S74544, 101, { &P12375,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIANokia N81 8GB) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2659723 = {S74545, 137, { &P21189,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MPDC88 BT IPS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2712513 = {S74546, 157, { &P18450,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST23i Build/11.0.A.5.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2753778 = {S74547, 130, { &P15985,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2877584 = {S74548, 131, { &P15985,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2792759 = {S74549, 138, { &P24098,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AS870 4G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2795749 = {S74550, 138, { &P23251,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1 ;en-us; ZTE V807 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30;*/
const Device UA3049859 = {S74551, 139, { &P23251,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ZTE V807 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2974659 = {S74552, 136, { &P22956,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P875 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021235 = {S74553, 129, { &P24365,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SM-T211 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3096279 = {S74554, 150, { &P18437,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3114748 = {S74555, 150, { &P18437,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8020 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3225 = {S74556, 146, { &P14784,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android SenseHERO 2.1; en-gb; HTC Hero Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA18526 = {S74557, 147, { &P14784,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android VillainROM-v13; en-gb; HTC Hero Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA25866 = {S74558, 143, { &P14784,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Kimera 2.1; en-gb; HTC Hero Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA53741 = {S74559, 148, { &P14784,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android VillainROM9.0.0; en-gb; HTC Hero Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA65421 = {S74560, 148, { &P14784,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Hero_2.19_Hiapk; zh-cn; HTC Hero Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA65431 = {S74561, 148, { &P14784,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Hero_2.16_Hiapk; zh-cn; HTC Hero Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018444 = {S74562, 70, { &P14784,&P17979,&P20428,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android VillainROM9.0.0; HTC Hero Build/ERE27)*/
const Device UA2020735 = {S74563, 134, { &P14784,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Hero_2.19_Hiapk; zh-cn; HTC Hero Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA7319 = {S74564, 80, { &P12226,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) HTC-PPC6850 BM BMC*/
const Device UA10613 = {S74565, 109, { &P12226,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_Touch_Pro_T7272-orange/PPC; 480x640; OpVer 31.190.2.734 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA24587 = {S74566, 81, { &P12226,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_Touch_Pro_T7272 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA32617 = {S74567, 99, { &P12226,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Touch_Pro/1.90.172.4*/
const Device UA32620 = {S74568, 99, { &P12226,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Touch_Pro/5.11.162.1*/
const Device UA32621 = {S74569, 99, { &P12226,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Touch_Pro/1.90.161.5*/
const Device UA32739 = {S74570, 99, { &P12226,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Swisscom/1.0/HTC_Touch_Pro/1.90.166.6*/
const Device UA32769 = {S74571, 75, { &P12226,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_Touch_Pro Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA37923 = {S74572, 102, { &P12226,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Touch_Pro/%ROM_VERSION%*/
const Device UA51347 = {S74573, 99, { &P12226,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Touch_Pro/5.10.161.1*/
const Device UA76194 = {S74574, 81, { &P12226,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_Touch_Pro_T7276 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA48741 = {S74575, 139, { &P21015,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SO-03C Build/4.0.D.2.61) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA603073 = {S74576, 141, { &P21015,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SO-03C Build/4.0.1.C.1.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA669515 = {S74577, 140, { &P21015,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SO-03C Build/4.0.1.C.1.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2387201 = {S74578, 141, { &P21015,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SO-03C Build/4.0.1.C.1.31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2878182 = {S74579, 65, { &P21015,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; SO-03C Build/4.0.1.C.1.31)*/
const Device UA49606 = {S74580, 205, { &P13060,&P5629,&P15760,&P18092 }};	/*Useragent - MOT-MT870_TD/1.0 Android/2.3 (Linux; U; Android 2.3.3; zh-cn) Release/5.31.2011 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71760 = {S74581, 136, { &P13060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MT870 Build/04.30.51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019939 = {S74582, 124, { &P13060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MT870 Build/04.30.54) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022869 = {S74583, 124, { &P13060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MT870 Build/04.30.59) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2023941 = {S74584, 129, { &P13060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MT870 Build/04.30.62) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2033998 = {S74585, 136, { &P13060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MT870 Build/04.30.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2518610 = {S74586, 129, { &P13060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MT870 Build/04.30.59) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2815674 = {S74587, 136, { &P13060,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MT870 Build/04.30.59) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA149496 = {S74588, 131, { &P22546,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; HP eStation/1.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA336028 = {S74589, 99, { &P15364,&P7127,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.218 Safari/535.1*/
const Device UA230099 = {S74590, 140, { &P15869,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; en-us; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA316899 = {S74591, 139, { &P15869,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.1; en-us; Xoom Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA334475 = {S74592, 140, { &P15869,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; en-gb; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402730 = {S74593, 140, { &P15869,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; fr-fr; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402752 = {S74594, 140, { &P15869,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; es-us; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402783 = {S74595, 140, { &P15869,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; en-nz; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402842 = {S74596, 140, { &P15869,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; es-es; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402881 = {S74597, 140, { &P15869,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; en-ca; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA402864 = {S74598, 140, { &P15869,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; ar-eg; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA403084 = {S74599, 140, { &P15869,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; zh-tw; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA403095 = {S74600, 140, { &P15869,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; zh-cn; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA403285 = {S74601, 140, { &P15869,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; nl-nl; Xoom Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA663158 = {S74602, 42, { &P13706,&P5129,&P24665,&P18092 }};	/*Useragent - Nokia5235/UC Browser8.0.4.121/50/444/UCWEB*/
const Device UA867097 = {S74603, 88, { &P15364,&P5690,&P22489,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8b5) Gecko/X Flock/0.4 Firefox/1.0+*/
const Device UA2431079 = {S74604, 165, { &P15649,&P5096,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2562915 = {S74605, 129, { &P23752,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CT705 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2600027 = {S74606, 142, { &P21703,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 70 Titanium Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2737554 = {S74607, 161, { &P17324,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2738095 = {S74608, 161, { &P17324,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2869282 = {S74609, 161, { &P17324,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2993997 = {S74610, 161, { &P17324,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2783982 = {S74611, 151, { &P18552,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT897 Build/7.7.1Q-6_SPR-125_ASA-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783404 = {S74612, 73, { &P17595,&P18207,&P24714,&P18092 }};	/*Useragent - Mac OS X/10.8.4 (12E55); ExchangeWebServices/3.0.1 (158); Mail/6.5 (1508)*/
const Device UA2879929 = {S74613, 75, { &P17595,&P18207,&P24714,&P18092 }};	/*Useragent - Mac OS X/10.8.4 (12E3067); ExchangeWebServices/3.0.1 (158); Mail/6.5 (1508)*/
const Device UA2895087 = {S74614, 142, { &P23359,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; QMobile_A20 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3064895 = {S74615, 166, { &P14225,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3079794 = {S74616, 156, { &P14225,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; Desire HD Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA14832 = {S74617, 174, { &P13517,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; de-de; Garmin-Asus A50 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1; A50-V4.1.29-user-20100608*/
const Device UA49315 = {S74618, 75, { &P12715,&P22857,&P4812,&P18092 }};	/*Useragent - Spice M6800 Opera/9.80 (MTK; Nucleus; U; en-US) Presto/2.4.18 Version/10.00*/
const Device UA61409 = {S74619, 76, { &P12715,&P22857,&P4812,&P18092 }};	/*Useragent - Spice M6800  Opera/9.80 (MTK; Nucleus; U; en-US) Presto/2.4.18 Version/10.00*/
const Device UA61227 = {S74620, 84, { &P12660,&P6476,&P4223,&P18092 }};	/*Useragent - ZTE-T54 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 firmware/T54 T04*/
const Device UA640198 = {S74621, 135, { &P18082,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; KM-S120 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018747 = {S74622, 59, { &P18082,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; KM-S120 Build/FRG83)*/
const Device UA215952 = {S74623, 81, { &P17595,&P17295,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.9.168 Version/11.50*/
const Device UA682336 = {S74624, 135, { &P14969,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Desire S Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA725347 = {S74625, 134, { &P14969,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; DesireS Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA806205 = {S74626, 123, { &P14969,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Desire S Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA837965 = {S74627, 139, { &P14969,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Desire S Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026225 = {S74628, 128, { &P14969,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Desire S Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2228371 = {S74629, 129, { &P24031,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T1004 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2386045 = {S74630, 118, { &P16003,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; gt-s5233a) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2419377 = {S74631, 115, { &P16003,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; fa; gt-s5233a) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile*/
const Device UA2461329 = {S74632, 171, { &P18214,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2456489 = {S74633, 174, { &P15767,&P18432,&P22305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6 like Mac OS X; zh-cn Model:iPad3,3) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7543.48.3 UCBrowser/1.0.0.192/43/355*/
const Device UA2433085 = {S74634, 167, { &P11787,&P5204,&P24673,&P18092 }};	/*Useragent - Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia3120classic) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2528341 = {S74635, 148, { &P20731,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4(6577); en-us; Blaze S180  Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2537988 = {S74636, 173, { &P14202,&P5477,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E15i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3015831 = {S74637, 177, { &P14202,&P5477,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E15i Build/2.1.1.A.0.6) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.276 Mobile*/
const Device UA2556455 = {S74638, 146, { &P23137,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I407 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2567220 = {S74639, 144, { &P19855,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT885 Build/6.7.3-135_YIC-8) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567575 = {S74640, 138, { &P14959,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9103 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2581080 = {S74641, 164, { &P12132,&P17485,&P24676,&P18092 }};	/*Useragent - SonyEricssonJ10i2/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj10i2) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2850351 = {S74642, 159, { &P12132,&P17485,&P24676,&P18092 }};	/*Useragent - SonyEricssonJ10i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; fa; sonyericssonj10i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2662642 = {S74643, 139, { &P23774,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Excite_352g Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3058385 = {S74644, 139, { &P23774,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Excite 352g Build/SKV06) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2674448 = {S74645, 137, { &P18594,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LS860 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735118 = {S74646, 133, { &P18594,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Mach Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2879218 = {S74647, 136, { &P23383,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; H7100 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2973486 = {S74648, 133, { &P17172,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Transformer Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021740 = {S74649, 139, { &P17172,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Transformer TF101 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974726 = {S74650, 137, { &P20103,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; EK-GC100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024109 = {S74651, 137, { &P20103,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EK-GC100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA61726 = {S74652, 102, { &P11775,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-D500E/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA63894 = {S74653, 101, { &P11775,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-D500/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA26741 = {S74654, 88, { &P13626,&P17603,&P4223,&P18092 }};	/*Useragent - PantechP9020/JEUS04262010 NetFront/3.5 OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA446126 = {S74655, 100, { &P13626,&P17603,&P4223,&P18092 }};	/*Useragent - PantechP9020/JEUSX NetFront/3.5 OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1 Novarra-Vision/8.0*/
const Device UA88447 = {S74656, 110, { &P15364,&P17017,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; 78; CentOS; US-en) AppleWebKit/527+ (KHTML, like Gecko) Bolt/0.862 Version/3.0 Safari/523.15*/
const Device UA237090 = {S74657, 38, { &P15364,&P17017,&P17315,&P18092 }};	/*Useragent - Version/3.0 Mobile/1A477d Safari/419.3*/
const Device UA344871 = {S74658, 124, { &P17595,&P17297,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_2; en-US) AppleWebKit/533.2 (KHTML, like Gecko) Chrome/5.0.342.9 Safari/533.2*/
const Device UA450745 = {S74659, 124, { &P17595,&P17297,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_0; en-US) AppleWebKit/533.2 (KHTML, like Gecko) Chrome/5.0.342.7 Safari/533.2*/
const Device UA835135 = {S74660, 124, { &P17595,&P17297,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_3; en-US) AppleWebKit/533.2 (KHTML, like Gecko) Chrome/5.0.342.7 Safari/533.2*/
const Device UA878325 = {S74661, 124, { &P17595,&P17297,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/533.2 (KHTML, like Gecko) Chrome/5.0.342.9 Safari/533.2*/
const Device UA2420881 = {S74662, 124, { &P17595,&P17297,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-US) AppleWebKit/533.2 (KHTML, like Gecko) Chrome/5.0.342.9 Safari/533.2*/
const Device UA368865 = {S74663, 131, { &P21139,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHV-E150S Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA539875 = {S74664, 112, { &P15364,&P18118,&P19772,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.21) Gecko/X SeaMonkey/1.1.15 (Ubuntu-1.1.15+nobinonly-0ubuntu2)*/
const Device UA905527 = {S74665, 128, { &P18248,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A700 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2167763 = {S74666, 139, { &P18497,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M886 Build/HuaweiM886) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2383571 = {S74667, 186, { &P12473,&P5204,&P24671,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia2730c-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2567590 = {S74668, 140, { &P15972,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One XL Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667580 = {S74669, 140, { &P15972,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One XL Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2585256 = {S74670, 137, { &P20240,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E971 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2860392 = {S74671, 130, { &P20240,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E971 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2676938 = {S74672, 130, { &P19879,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I915 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2733604 = {S74673, 142, { &P18645,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/2.3;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/SPH-L710;FBSV/4.1.2;]*/
const Device UA2750941 = {S74674, 142, { &P18645,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.1;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/SPH-L710;FBSV/4.1.2;]*/
const Device UA2772259 = {S74675, 144, { &P18645,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.2.1;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/SPH-L710;FBSV/4.1.2;]*/
const Device UA2876982 = {S74676, 142, { &P18645,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.3;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/SPH-L710;FBSV/4.1.2;]*/
const Device UA2970830 = {S74677, 142, { &P18645,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.4;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/SPH-L710;FBSV/4.1.2;]*/
const Device UA3029026 = {S74678, 142, { &P18645,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.5;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/SPH-L710;FBSV/4.1.2;]*/
const Device UA3094310 = {S74679, 169, { &P18645,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.6.1;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/SPH-L710;FBSV/4.1.2;FBCA/armeabi-v7a:armeabi;]*/
const Device UA2973669 = {S74680, 137, { &P22600,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300K Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2962538 = {S74681, 152, { &P21003,&P20706,&P24678,&P18092 }};	/*Useragent - Nokia310/2.0 (07.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia310) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3020564 = {S74682, 137, { &P22999,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A870S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021661 = {S74683, 137, { &P22999,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A870K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022512 = {S74684, 137, { &P22999,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A870L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975877 = {S74685, 134, { &P18299,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP4100 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3027497 = {S74686, 93, { &P15649,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3029471 = {S74687, 94, { &P15649,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570I) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3081380 = {S74688, 139, { &P16319,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; U8860 Build/HuaweiU8860) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA98115 = {S74689, 123, { &P15364,&P17017,&P18107,&P18143 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE or Firefox mutant; not on Windows server; +http://ws.daum.net/aboutWebSearch.html) Daumoa/2.0*/
const Device UA366430 = {S74690, 98, { &P15364,&P7127,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7*/
const Device UA606162 = {S74691, 158, { &P16168,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NookColor Build/IMM76D; CyanogenMod-CyanogenMod) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA794044 = {S74692, 148, { &P16168,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NookColor Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2382132 = {S74693, 142, { &P15991,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2413459 = {S74694, 89, { &P15364,&P17285,&P21959,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.3a3pre) Gecko/X Minefield/3.7a3pre*/
const Device UA2459538 = {S74695, 134, { &P20856,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; SAMBM816ND Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2513849 = {S74696, 37, { &P15795,&P5448,&P20770,&P18092 }};	/*Useragent - NOKIA-Lumia 710/UCWEB2.8.0.249/47/355*/
const Device UA2580094 = {S74697, 63, { &P15364,&P5690,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; Windows NT 5.1; Trident/5.0)*/
const Device UA2792903 = {S74698, 63, { &P15364,&P5690,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; Windows NT 5.2; Trident/1.0)*/
const Device UA2667042 = {S74699, 135, { &P22633,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; eZee'Tab805 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3102670 = {S74700, 135, { &P22633,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; eZee Tab805 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2865633 = {S74701, 140, { &P22848,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT920 Build/2_310_2009) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2866107 = {S74702, 140, { &P22848,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT920 Build/2_290_2009) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2866228 = {S74703, 140, { &P22848,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT920 Build/2_290_2014) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956858 = {S74704, 140, { &P22848,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT920 Build/2_310_2014) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049420 = {S74705, 62, { &P22848,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT920 Build/2_310_2014)*/
const Device UA3091773 = {S74706, 140, { &P22848,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT920 Build/2_32D_2009) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125365 = {S74707, 140, { &P22848,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT920 Build/2_32D_2014) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2900833 = {S74708, 114, { &P14540,&P5843,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-01/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA2967514 = {S74709, 134, { &P24062,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; L3 Build/TECNOL3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3124901 = {S74710, 56, { &P24062,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; L3 Build/TECNOL3)*/
const Device UA8337 = {S74711, 123, { &P14909,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia7510a/2.0 (06.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA9539 = {S74712, 67, { &P14909,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia7510a_CMCC/2.0 (06.65) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA9890 = {S74713, 128, { &P14909,&P5843,&P4037,&P18092 }};	/*Useragent - Mozilla/5.0 ( Nokia7510a-b/2.0 (03.90) Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA64710 = {S74714, 131, { &P14909,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia7510Supernova/2.0 (06.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA50843 = {S74715, 173, { &P14519,&P5129,&P3690,&P18092 }};	/*Useragent - NokiaN97mini/30.0.004 (SymbianOS/9.4; Series60/5.0 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebkit/533.4 (KHTML, like Gecko) NokiaBrowser/7.3.1.28 3gpp-gba*/
const Device UA63482 = {S74716, 156, { &P14519,&P5129,&P3690,&P18092 }};	/*Useragent - NokiaN97mini/12.0.110 (SymbianOS/9.4; Series60/5.0 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebkit/525 (KHTML, like Gecko) BrowserNG/7.1.4*/
const Device UA68330 = {S74717, 38, { &P14519,&P5129,&P3690,&P18092 }};	/*Useragent - NokiaN97 mini/UCBrowser7.7.0.81/50/800*/
const Device UA69524 = {S74718, 13, { &P14519,&P5129,&P3690,&P18092 }};	/*Useragent - NokiaN97 mini*/
const Device UA62467 = {S74719, 40, { &P14413,&P17486,&P17602,&P18092 }};	/*Useragent - MOT-EX115 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA67695 = {S74720, 137, { &P14823,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; LG-LU3100 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67763 = {S74721, 135, { &P14823,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; LG-LU3100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA557971 = {S74722, 138, { &P14823,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-LU3100 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA69406 = {S74723, 31, { &P12088,&P5129,&P24663,&P18092 }};	/*Useragent - NokiaX6-00/UCWEB7.9.0.94/50/999*/
const Device UA379627 = {S74724, 43, { &P12088,&P5129,&P24663,&P18092 }};	/*Useragent - NokiaX6-00/UC Browser7.9.0.102/50/355/UCWEB*/
const Device UA653380 = {S74725, 85, { &P15364,&P18140,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; Edition Linux Mint; en) Presto/2.10.229 Version/11.61*/
const Device UA724481 = {S74726, 136, { &P13669,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-MS690 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA739831 = {S74727, 145, { &P13669,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-MS690 Build/GRJ90-LG-MS690) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA787437 = {S74728, 140, { &P13669,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LG Optimus M Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1902311 = {S74729, 145, { &P13669,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-MS690 Build/GRJ90-LG-MS690) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1902545 = {S74730, 136, { &P13669,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-MS690 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA863449 = {S74731, 144, { &P12562,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID2 Global Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025299 = {S74732, 144, { &P12562,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID2 Global Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2360165 = {S74733, 137, { &P12562,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID2 GLOBAL Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1978628 = {S74734, 114, { &P21353,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; YL-Coolpad_5832/2.3.011.X.5832; 480*320; CTC/2.0) CoolpadWebkit/533.1*/
const Device UA2025446 = {S74735, 132, { &P21353,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 5832 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029264 = {S74736, 125, { &P21353,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 5832 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2029296 = {S74737, 120, { &P21353,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 5832 Build/GWK74) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2230494 = {S74738, 133, { &P21139,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E150S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2354900 = {S74739, 138, { &P20075,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xperia S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2436508 = {S74740, 148, { &P12280,&P18432,&P21788,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.0.3/X (like Mobile/10A525 Safari/8536.25)*/
const Device UA2437638 = {S74741, 148, { &P12280,&P18432,&P21788,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.0.3/X (like Mobile/10A525 Safari/8536.25)*/
const Device UA2566353 = {S74742, 138, { &P21384,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F240L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2566624 = {S74743, 144, { &P20816,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WIKO-CINK SLIM Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583568 = {S74744, 131, { &P20107,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5580C Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2663712 = {S74745, 132, { &P21190,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MPDC8 BT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2676099 = {S74746, 140, { &P18183,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 Build/7.7.1-128_MZ615-12) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2738268 = {S74747, 153, { &P18283,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2770049 = {S74748, 153, { &P18283,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I535 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2798827 = {S74749, 153, { &P18283,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T999 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2973470 = {S74750, 134, { &P18605,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTL21 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3019950 = {S74751, 135, { &P16598,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; A500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3071311 = {S74752, 57, { &P16598,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; A500 Build/JOP40D)*/
const Device UA3061493 = {S74753, 156, { &P18595,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8190 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3108796 = {S74754, 156, { &P18595,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; GT-I8190 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3081117 = {S74755, 130, { &P18702,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8013 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080158 = {S74756, 141, { &P11747,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire V Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111381 = {S74757, 142, { &P11747,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire VC Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080246 = {S74758, 134, { &P18299,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP4100 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA13019 = {S74759, 143, { &P13993,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-A1200eam/1.0/R541L7_G_11.00.16R Mozilla/4.0 (compatible; MSIE 6.0; Linux; A1200eam; 781) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00*/
const Device UA18051 = {S74760, 139, { &P13993,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-A1200w/1.0/R541L7_G_11.10.11R Mozilla/4.0 (compatible; MSIE 6.0; Linux; A1200w; 781) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00*/
const Device UA20343 = {S74761, 151, { &P13993,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-A1200i/R532L4_G_11.40.1AR Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola A1200i;nnn) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [es-la]*/
const Device UA50250 = {S74762, 131, { &P13993,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-A1200r/1.0 LinuxOS/2.4.20 Release/8.22.2006 Browser/Opera8.00 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R532_G_11.00.59P*/
const Device UA56292 = {S74763, 86, { &P13993,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-A1200eam/1.0/R541L7_G_11.00.13R Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00*/
const Device UA63259 = {S74764, 135, { &P13993,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-A1200e/R541_G_11.xx.yyR Mozilla/4.0 (compatible; MSIE 6.0 Linux; A1200e;nnn) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00[yy]*/
const Device UA67322 = {S74765, 130, { &P13993,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-A1200/1.0 LinuxOS/2.4.20 Release/6.20.2006 Browser/Opera8.00 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R532_G_11.00.53P*/
const Device UA67388 = {S74766, 131, { &P13993,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-A1200e/1.0 LinuxOS/2.4.20 Release/1.31.2007 Browser/Opera8.00 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R541_G_11.56.14R*/
const Device UA135403 = {S74767, 146, { &P13993,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-A1200/R532_G_11.00.53P Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola A1200; 1862) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [en]*/
const Device UA140782 = {S74768, 147, { &P13993,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-A1200/R532H2_G_11.20.08P Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola A1200; 781) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [en]*/
const Device UA667005 = {S74769, 146, { &P13993,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-A1200/R532_G_11.00.44P Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola A1200; 1555) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [en]*/
const Device UA53999 = {S74770, 53, { &P14086,&P5284,&P3976,&P18092 }};	/*Useragent - sam-r810 NetFront/3.4 Mozilla/5.0 like Gecko/20060426*/
const Device UA2428292 = {S74771, 153, { &P12059,&P5204,&P24675,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia2690) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2506793 = {S74772, 140, { &P21941,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; Alldro3Speed DUO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2654500 = {S74773, 103, { &P13144,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.393; en-us; 9860/5.1.0.393) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2655302 = {S74774, 103, { &P13144,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.276; en-us; 9860/5.1.0.276) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2655844 = {S74775, 103, { &P13144,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.0.0.496; en-us; 9860/5.0.0.496) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2842565 = {S74776, 103, { &P13144,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.546; en-us; 9860/5.1.0.546) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2951447 = {S74777, 103, { &P13144,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.0.0.734; en-us; 9860/5.0.0.734) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2951520 = {S74778, 103, { &P13144,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.532; en-us; 9860/5.1.0.532) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2662772 = {S74779, 144, { &P22668,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BLU VIVO 4.65 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3119313 = {S74780, 144, { &P22668,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BLU_VIVO 4.65 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2704871 = {S74781, 146, { &P22809,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ro-ro; ALLVIEW_P5_Quad Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2739965 = {S74782, 135, { &P17595,&P18207,&P22708,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) Version/6.0.4 Safari/8536.29.13 Sleipnir/4.2.1*/
const Device UA2796893 = {S74783, 133, { &P17595,&P18207,&P22708,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) Version/6.0.5 Safari/8536.30.1 Sleipnir/4.2.1*/
const Device UA2783285 = {S74784, 159, { &P12798,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LT18i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2769845 = {S74785, 133, { &P23187,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; V370 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3013224 = {S74786, 132, { &P23187,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; V370 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2901236 = {S74787, 156, { &P18283,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9300 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3080318 = {S74788, 144, { &P23799,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ONE TOUCH 4011X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093713 = {S74789, 156, { &P20146,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9082 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA20179 = {S74790, 102, { &P17473,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-D600 /1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA63913 = {S74791, 102, { &P17473,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-D600E/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA64829 = {S74792, 101, { &P17473,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-D600/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA21429 = {S74793, 79, { &P15279,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-T239/T239UVIC6 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA158639 = {S74794, 121, { &P15364,&P18110,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux 1.6; es-es; AirLife100) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA1858439 = {S74795, 121, { &P15364,&P18110,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux 1.6; en-us; AirLife100) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA2804803 = {S74796, 121, { &P15364,&P18110,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux 1.6; en-gb; AirLife100) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA607569 = {S74797, 156, { &P12681,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S 4G Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA889039 = {S74798, 160, { &P12681,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S 4G Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA910398 = {S74799, 160, { &P12681,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S 4G Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2014550 = {S74800, 163, { &P18114,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer Prime TF201 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2030149 = {S74801, 163, { &P18114,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer Prime TF201 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2354453 = {S74802, 163, { &P18114,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Transformer Prime TF201 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2030220 = {S74803, 156, { &P13054,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6425LVW Build/IML74K; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2232091 = {S74804, 81, { &P15364,&P16110,&P17201,&P18163 }};	/*Useragent - Mozilla/5.0/Nutch-1.4 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0)*/
const Device UA2424398 = {S74805, 142, { &P22858,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U8836D Build/HuaweiU8836D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2508473 = {S74806, 171, { &P18214,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3125050 = {S74807, 175, { &P18214,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S7500 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2567521 = {S74808, 143, { &P15438,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Sensation Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583229 = {S74809, 139, { &P15438,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Sensation Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2565859 = {S74810, 98, { &P19862,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax_A110) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2671262 = {S74811, 170, { &P18113,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2707655 = {S74812, 167, { &P12871,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2866458 = {S74813, 167, { &P12871,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; HTC Wildfire S A510e Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752384 = {S74814, 136, { &P17710,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; LG-P925g Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2975797 = {S74815, 152, { &P22960,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2971183 = {S74816, 94, { &P11907,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE52-1) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3023351 = {S74817, 134, { &P18464,&P18110,&P24339,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Safari/535.19 Silk-Accelerated=false*/
const Device UA3027858 = {S74818, 164, { &P18645,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3029905 = {S74819, 156, { &P18645,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9305 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3065391 = {S74820, 156, { &P18645,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SPH-L710 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3079940 = {S74821, 164, { &P18645,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3082738 = {S74822, 137, { &P22941,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6812 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094802 = {S74823, 138, { &P22941,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6812B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA1121 = {S74824, 62, { &P11891,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia5070b/2.0 (04.20) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA17043 = {S74825, 61, { &P11891,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia5070/2.0 (04.22) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA7575 = {S74826, 111, { &P13361,&P17483,&P3534,&P18092 }};	/*Useragent - LG-KF750-v10c Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA8587 = {S74827, 117, { &P13361,&P17483,&P3534,&P18092 }};	/*Useragent - LG-KF750-Orange/v10c Browser/Obigo-Q05A/3.12MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA8899 = {S74828, 111, { &P13361,&P17483,&P3534,&P18092 }};	/*Useragent - LG-KF750-v10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA20255 = {S74829, 111, { &P13361,&P17483,&P3534,&P18092 }};	/*Useragent - LG/KF750/v10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA24603 = {S74830, 111, { &P13361,&P17483,&P3534,&P18092 }};	/*Useragent - LG-KF750/v10a Browser/Obigo-Q05A/3.11 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA53787 = {S74831, 138, { &P13361,&P17483,&P3534,&P18092 }};	/*Useragent - Mozilla/4.0 (Vodafone/1.0/LG-KF750/V10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA63392 = {S74832, 138, { &P13361,&P17483,&P3534,&P18092 }};	/*Useragent - Mozilla/4.0 (Vodafone/1.0/LG-KF750/V10b Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA87433 = {S74833, 111, { &P13361,&P17483,&P3534,&P18092 }};	/*Useragent - LG-KF750-v10b Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA1950125 = {S74834, 136, { &P12280,&P17090,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari Version/5.1*/
const Device UA1956782 = {S74835, 136, { &P12280,&P17090,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari Version/5.1*/
const Device UA2319604 = {S74836, 136, { &P12280,&P17090,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8G4 Safari Version/5.1*/
const Device UA2441992 = {S74837, 136, { &P12280,&P17090,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8F190 Safari Version/5.1*/
const Device UA2639920 = {S74838, 144, { &P12280,&P17090,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3*/
const Device UA2685921 = {S74839, 136, { &P12280,&P17090,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 Safari Version/5.1*/
const Device UA1970909 = {S74840, 137, { &P21252,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; Fly_IQ235 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2425762 = {S74841, 175, { &P12139,&P5204,&P24673,&P18092 }};	/*Useragent - Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5310xpressmusic) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2505414 = {S74842, 89, { &P17976,&P18111,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1.1; Linux; Opera Tablet/ADY-X; U; ru) Presto/2.9.201 Version/11.50*/
const Device UA2696808 = {S74843, 89, { &P17976,&P18111,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1.2; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.9.201 Version/11.50*/
const Device UA2799510 = {S74844, 83, { &P15364,&P5690,&P22767,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; Maxthon/4.0.6.2000)*/
const Device UA2900016 = {S74845, 165, { &P22594,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-M919 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/jfltetmo*/
const Device UA2774087 = {S74846, 130, { &P16044,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Touchpad Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2799043 = {S74847, 141, { &P12280,&P18117,&P22879,&P18092 }};	/*Useragent - MQQBrowser/43 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA2892879 = {S74848, 140, { &P12280,&P18117,&P22879,&P18092 }};	/*Useragent - MQQBrowser/43 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3*/
const Device UA3050168 = {S74849, 138, { &P12280,&P18117,&P22879,&P18092 }};	/*Useragent - MQQBrowser/43 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3*/
const Device UA3074193 = {S74850, 139, { &P12280,&P18117,&P22879,&P18092 }};	/*Useragent - MQQBrowser/43 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3*/
const Device UA3083149 = {S74851, 144, { &P21629,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Lenovo S920_ROW Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA69273 = {S74852, 31, { &P14033,&P5129,&P24663,&P18092 }};	/*Useragent - NokiaC5-03/UCWEB7.9.0.94/50/999*/
const Device UA249661 = {S74853, 43, { &P14033,&P5129,&P24663,&P18092 }};	/*Useragent - NokiaC5-03/UC Browser7.9.0.102/50/355/UCWEB*/
const Device UA96715 = {S74854, 71, { &P17595,&P18120,&P17213,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Mac_PowerPC Mac OS X; en) Opera 8.02*/
const Device UA876639 = {S74855, 43, { &P17595,&P18120,&P17213,&P18092 }};	/*Useragent - Opera/8.51 (Macintosh; PPC Mac OS X; U; de)*/
const Device UA1872658 = {S74856, 54, { &P17595,&P18120,&P17213,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X; U; en) Opera 8.0*/
const Device UA2138465 = {S74857, 42, { &P17595,&P18120,&P17213,&P18092 }};	/*Useragent - Opera/8.0 (Macintosh; PPC Mac OS X; U; en)*/
const Device UA2210941 = {S74858, 42, { &P17595,&P18120,&P17213,&P18092 }};	/*Useragent - Opera/8.5 (Macintosh; PPC Mac OS X; U; en)*/
const Device UA221860 = {S74859, 133, { &P11702,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA689281 = {S74860, 133, { &P11702,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019705 = {S74861, 133, { &P11702,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; MB300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027234 = {S74862, 57, { &P11702,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; MB300 Build/GRJ22)*/
const Device UA479533 = {S74863, 33, { &P18254,&P17285,&P18245,&P18091 }};	/*Useragent - SuperBot/4.7.0.74 (Windows Vista)*/
const Device UA3018311 = {S74864, 130, { &P18254,&P17285,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.4) Gecko/X http://ow.ly web crawler (.NET CLR 3.5.30729);c2VvLWNyYXdsZXI=*/
const Device UA3020641 = {S74865, 100, { &P18254,&P17285,&P18245,&P18091 }};	/*Useragent - yacybot (webportal-global; x86 Windows Vista 6.0; java 1.7.0_25; Europe/en) http://yacy.net/bot.html*/
const Device UA2028323 = {S74866, 155, { &P15992,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; ZTE-Mimosa X Build/ICS_P943FV1.0.0B06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2652018 = {S74867, 155, { &P15992,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; ZTE-Mimosa X Build/ICS_P943FV1.0.0B07) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032711 = {S74868, 137, { &P21345,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-11D Build/V21R36A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2235821 = {S74869, 137, { &P21345,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-11D Build/V24R40A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2507920 = {S74870, 137, { &P21345,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; F-11D Build/V26R42B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2383348 = {S74871, 132, { &P18475,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2478125 = {S74872, 133, { &P18475,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320T Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2390922 = {S74873, 137, { &P15364,&P17816,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.11 (KHTML, like Gecko) Ubuntu/10.04 Chromium/23.0.1271.64 Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2428431 = {S74874, 139, { &P15364,&P17816,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Ubuntu/10.04 Chromium/23.0.1271.64 Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2533426 = {S74875, 135, { &P18280,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; One S Build/JOP40G) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2536592 = {S74876, 134, { &P18280,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; One S Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2850492 = {S74877, 139, { &P18280,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; HTC One S Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2854322 = {S74878, 138, { &P18280,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One S Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2487270 = {S74879, 133, { &P13644,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; EVO Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2536299 = {S74880, 133, { &P13644,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; EVO Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2598320 = {S74881, 140, { &P23284,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Next7C12F Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2602548 = {S74882, 156, { &P18174,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; VS920 4G Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2782461 = {S74883, 159, { &P18174,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; VS920 4G Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2679714 = {S74884, 69, { &P15364,&P7133,&P22226,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:21.0) Gecko/X Thunderbird/21.0*/
const Device UA2791978 = {S74885, 56, { &P15364,&P22310,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; Win3.11; Trident/1.0)*/
const Device UA2792763 = {S74886, 56, { &P15364,&P22310,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; Win3.11; Trident/2.0)*/
const Device UA2714782 = {S74887, 158, { &P18302,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT25i Build/4.1.B.0.631) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2773535 = {S74888, 152, { &P19770,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2797104 = {S74889, 152, { &P19770,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798772 = {S74890, 152, { &P19770,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879358 = {S74891, 152, { &P19770,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5020X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782974 = {S74892, 159, { &P12968,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.0.200) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2879537 = {S74893, 136, { &P23521,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2944091 = {S74894, 166, { &P14225,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2961364 = {S74895, 156, { &P14225,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; Desire HD Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3016734 = {S74896, 156, { &P14225,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Desire HD Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA5150 = {S74897, 52, { &P14225,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_wwe / 1.0 / htc_ace / 2.2.1*/
const Device UA5151 = {S74898, 51, { &P14225,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player twm_tw / 1.0 / htc_ace / 2.2.1*/
const Device UA5164 = {S74899, 55, { &P14225,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_asia_wwe / 1.0 / htc_ace / 2.2*/
const Device UA28088 = {S74900, 50, { &P14225,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_wwe / 1.0 / htc_ace / 2.2*/
const Device UA39625 = {S74901, 57, { &P14225,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_asia_wwe / 1.0 / htc_ace / 2.3.3*/
const Device UA42585 = {S74902, 56, { &P14225,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player cingular_us / 1.0 / htc_ace / 2.3.3*/
const Device UA45724 = {S74903, 49, { &P14225,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player h3g_uk / 1.0 / htc_ace / 2.2*/
const Device UA45715 = {S74904, 51, { &P14225,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player tmo_de / 1.0 / htc_ace / 2.2.1*/
const Device UA45760 = {S74905, 56, { &P14225,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player cingular_us / 1.0 / htc_ace / 2.2.1*/
const Device UA52962 = {S74906, 52, { &P14225,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_wwe / 1.0 / htc_ace / 2.3.3*/
const Device UA53089 = {S74907, 52, { &P14225,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_tur / 1.0 / htc_ace / 2.3.3*/
const Device UA53136 = {S74908, 54, { &P14225,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player hutch_aus / 1.0 / htc_ace / 2.3.3*/
const Device UA69731 = {S74909, 54, { &P14225,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player orange_uk / 1.0 / htc_ace / 2.3.3*/
const Device UA438807 = {S74910, 56, { &P14225,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player vodafone_au / 1.0 / htc_ace / 2.3.3*/
const Device UA438812 = {S74911, 56, { &P14225,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player vodafone_uk / 1.0 / htc_ace / 2.3.5*/
const Device UA438832 = {S74912, 56, { &P14225,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player vodafone_au / 1.0 / htc_ace / 2.3.5*/
const Device UA678281 = {S74913, 52, { &P14225,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_wwe / 1.0 / htc_ace / 2.3.5*/
const Device UA2312272 = {S74914, 56, { &P14225,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player cingular_us / 1.0 / htc_ace / 2.3.5*/
const Device UA2979536 = {S74915, 54, { &P14225,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player telus_wwe / 1.0 / htc_ace / 2.3.5*/
const Device UA63692 = {S74916, 84, { &P13550,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonV630iv/R1JC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA645893 = {S74917, 100, { &P14711,&P5204,&P17347,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-01/UC Browser8.0.3.107/69/444*/
const Device UA703239 = {S74918, 73, { &P15364,&P18144,&P24473,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/4.1; OpenBSD) KHTML/4.1.4 (like Gecko)*/
const Device UA777774 = {S74919, 112, { &P15364,&P5690,&P23600,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Fluid/0.9.4 Safari/525.13*/
const Device UA2203691 = {S74920, 28, { &P18254,&P17017,&P21800,&P18092 }};	/*Useragent - Akregator/4.9.1; syndication*/
const Device UA2236462 = {S74921, 28, { &P18254,&P17017,&P21800,&P18092 }};	/*Useragent - Akregator/4.9.2; syndication*/
const Device UA2387204 = {S74922, 28, { &P18254,&P17017,&P21800,&P18092 }};	/*Useragent - Akregator/4.9.3; syndication*/
const Device UA2406905 = {S74923, 28, { &P18254,&P17017,&P21800,&P18092 }};	/*Useragent - Akregator/4.9.4; syndication*/
const Device UA2486944 = {S74924, 28, { &P18254,&P17017,&P21800,&P18092 }};	/*Useragent - Akregator/4.9.5; syndication*/
const Device UA2380322 = {S74925, 131, { &P21388,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2419675 = {S74926, 178, { &P18583,&P5043,&P24671,&P18092 }};	/*Useragent - Nokia110/2.0 (03.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia110) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2423730 = {S74927, 122, { &P12329,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN86 8MP) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2580219 = {S74928, 80, { &P15364,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0*/
const Device UA2779192 = {S74929, 96, { &P15364,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; 11a h700tecno) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2851912 = {S74930, 86, { &P15364,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; t23) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2857891 = {S74931, 90, { &P15364,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; es-LA; mozilla) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2934788 = {S74932, 87, { &P15364,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; id; generic) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2952355 = {S74933, 91, { &P15364,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US;  mozilla) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2953322 = {S74934, 88, { &P15364,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; s8000) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2986266 = {S74935, 86, { &P15364,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; t50) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2988814 = {S74936, 95, { &P15364,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; beetel gd777) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA3045656 = {S74937, 92, { &P15364,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sjt_e861a) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA3045877 = {S74938, 90, { &P15364,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; fr-FR; mozilla) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA3071817 = {S74939, 90, { &P15364,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; fr-FR; generic) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA3086547 = {S74940, 87, { &P15364,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; bird) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA3087266 = {S74941, 90, { &P15364,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; zen-a60) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA3100158 = {S74942, 87, { &P15364,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; vi; generic) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA3100354 = {S74943, 86, { &P15364,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; inq) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA3119014 = {S74944, 81, { &P15364,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mstar) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0*/
const Device UA3119819 = {S74945, 90, { &P15364,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile, ,*/
const Device UA2596617 = {S74946, 139, { &P23298,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; OV-SteelCore10+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2794119 = {S74947, 144, { &P23298,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; OV-STEELCORE 10+ (Z) Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2676489 = {S74948, 138, { &P13109,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC-X710a Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2753607 = {S74949, 140, { &P21141,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Lenovo P770 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2998568 = {S74950, 140, { &P21141,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Lenovo P770 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879798 = {S74951, 140, { &P21542,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6503 Build/10.3.A.0.423) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799321 = {S74952, 145, { &P23138,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; TRUE BEYOND 3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3074344 = {S74953, 67, { &P23138,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; TRUE BEYOND 3G Build/JZO54K)*/
const Device UA2943145 = {S74954, 152, { &P19973,&P5043,&P24678,&P18092 }};	/*Useragent - Nokia114/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia114) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3021467 = {S74955, 138, { &P14854,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022559 = {S74956, 138, { &P14854,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SHW-M250S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022955 = {S74957, 138, { &P14854,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023834 = {S74958, 138, { &P14854,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022058 = {S74959, 136, { &P24101,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ADR930L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021613 = {S74960, 142, { &P12280,&P19826,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 UCBrowser/9.2.0.312 Mobile*/
const Device UA3058874 = {S74961, 140, { &P12280,&P19826,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 UCBrowser/9.2.0.312 Mobile*/
const Device UA3059080 = {S74962, 140, { &P12280,&P19826,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 UCBrowser/9.2.0.312 Mobile*/
const Device UA3079245 = {S74963, 140, { &P12280,&P19826,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144 UCBrowser/9.2.0.312 Mobile*/
const Device UA3079797 = {S74964, 142, { &P12280,&P19826,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 UCBrowser/9.2.0.312 Mobile*/
const Device UA3080999 = {S74965, 142, { &P12280,&P19826,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 UCBrowser/9.2.0.312 Mobile*/
const Device UA3031364 = {S74966, 164, { &P13337,&P17485,&P24679,&P18092 }};	/*Useragent - SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj108i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3033036 = {S74967, 156, { &P14714,&P5843,&P24679,&P18092 }};	/*Useragent - NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax3-02) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3080178 = {S74968, 133, { &P20584,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Z120 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3132764 = {S74969, 149, { &P18318,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Transformer Pad TF700T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA430173 = {S74970, 30, { &P15364,&P17017,&P23044,&P18092 }};	/*Useragent - NSPlayer/9.0.0.4504 WMFSDK/9.0*/
const Device UA438973 = {S74971, 30, { &P15364,&P17017,&P23044,&P18092 }};	/*Useragent - NSPlayer/9.0.0.3268 WMFSDK/9.0*/
const Device UA442463 = {S74972, 30, { &P15364,&P17017,&P23044,&P18092 }};	/*Useragent - NSPlayer/9.0.0.2980 WMFSDK/9.0*/
const Device UA459848 = {S74973, 30, { &P15364,&P17017,&P23044,&P18092 }};	/*Useragent - NSPlayer/9.0.0.3270 WMFSDK/9.0*/
const Device UA699068 = {S74974, 30, { &P15364,&P17017,&P23044,&P18092 }};	/*Useragent - NSPlayer/9.0.0.3267 WMFSDK/9.0*/
const Device UA726093 = {S74975, 19, { &P15364,&P17017,&P23044,&P18092 }};	/*Useragent - NSPlayer/9.0.0.3268*/
const Device UA2206838 = {S74976, 30, { &P15364,&P17017,&P23044,&P18092 }};	/*Useragent - NSPlayer/9.0.0.3265 WMFSDK/9.0*/
const Device UA2766423 = {S74977, 48, { &P15364,&P17017,&P23044,&P18092 }};	/*Useragent - NSPlayer/9.0.0.4509 WMFSDK/9.0 SE 2.X MetaSr 1.0*/
const Device UA2933036 = {S74978, 21, { &P15364,&P17017,&P23044,&P18092 }};	/*Useragent - NSPlayer/9.00.00.4504*/
const Device UA454598 = {S74979, 225, { &P12280,&P18117,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA741806 = {S74980, 227, { &P12280,&P18117,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA798003 = {S74981, 225, { &P12280,&P18117,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA2222483 = {S74982, 227, { &P12280,&P18117,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA789450 = {S74983, 112, { &P15241,&P5843,&P17438,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia300/07.44; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.7*/
const Device UA684997 = {S74984, 135, { &P20362,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; EasyPad 970 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA788668 = {S74985, 86, { &P17968,&P5062,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 1.6; Linux; Opera Mobi/ADR-X; U; en) Presto/2.10.254 Version/12.00*/
const Device UA2228223 = {S74986, 125, { &P20822,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; N-08D Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2382580 = {S74987, 178, { &P15241,&P5843,&P24672,&P18092 }};	/*Useragent - Nokia300/5.0 (07.03) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia300) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile*/
const Device UA2404823 = {S74988, 131, { &P23515,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TD-1010 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2426665 = {S74989, 135, { &P20881,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483053 = {S74990, 137, { &P21936,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ThL W6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2538207 = {S74991, 167, { &P18450,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST23i Build/11.0.A.5.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2559608 = {S74992, 129, { &P23875,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; TR730 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2567325 = {S74993, 155, { &P20088,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E-51 Build/HuaweiU8666E-51) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583175 = {S74994, 149, { &P20088,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E Build/HuaweiU8666E) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2707384 = {S74995, 192, { &P22037,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, -0500, us*/
const Device UA2708966 = {S74996, 138, { &P18564,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One X+ Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2770237 = {S74997, 156, { &P19853,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7102 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2796459 = {S74998, 156, { &P19853,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7102 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2738018 = {S74999, 93, { &P14984,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5250) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2768660 = {S75000, 143, { &P23408,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 97B TITANIUM Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2895317 = {S75001, 140, { &P21542,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6503 Build/10.3.A.0.423) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3016838 = {S75002, 133, { &P21542,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6503 Build/10.3.A.0.423) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2897354 = {S75003, 155, { &P18465,&P18066,&P23482,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA3031260 = {S75004, 150, { &P18702,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8013 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2958258 = {S75005, 145, { &P20705,&P18598,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC One Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3083189 = {S75006, 154, { &P12605,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080826 = {S75007, 126, { &P19991,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A511 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA62147 = {S75008, 83, { &P11998,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonZ550i/R6CA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA30668 = {S75009, 96, { &P14062,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-X991-Rio-orange/X991_V1_Z2_FRES_D18F108 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C*/
const Device UA30669 = {S75010, 96, { &P14062,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-X991-Rio-orange/X991_V1_Z2_FRES_D18F109 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C*/
const Device UA401783 = {S75011, 85, { &P18141,&P18204,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 1.61) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2*/
const Device UA609363 = {S75012, 141, { &P12681,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  Nexus S 4G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA631627 = {S75013, 140, { &P12681,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Nexus S 4G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA757194 = {S75014, 141, { &P12681,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  Nexus S 4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA1973475 = {S75015, 140, { &P12681,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Nexus S 4G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA615168 = {S75016, 65, { &P15364,&P18110,&P18367,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:12.0) Gecko/X Thunderbird/12.0.1*/
const Device UA815652 = {S75017, 54, { &P15364,&P17017,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (compatible; U) Presto/2.7.39 Version/11.00*/
const Device UA2165409 = {S75018, 80, { &P13706,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5235)/UC Browser8.5.0.183/50/355/UCWEB Mobile*/
const Device UA2339952 = {S75019, 138, { &P21435,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V788D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2374314 = {S75020, 134, { &P21435,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; V788D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2574489 = {S75021, 131, { &P21435,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V788D Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2561921 = {S75022, 144, { &P21466,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Titan TV S320 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2591505 = {S75023, 22, { &P18141,&P22712,&P17470,&P18092 }};	/*Useragent - libhttp/1.80 (PS Vita)*/
const Device UA2704753 = {S75024, 22, { &P18141,&P22712,&P17470,&P18092 }};	/*Useragent - libhttp/2.06 (PS Vita)*/
const Device UA2714728 = {S75025, 22, { &P18141,&P22712,&P17470,&P18092 }};	/*Useragent - libhttp/2.10 (PS Vita)*/
const Device UA2716669 = {S75026, 22, { &P18141,&P22712,&P17470,&P18092 }};	/*Useragent - libhttp/2.11 (PS Vita)*/
const Device UA2771790 = {S75027, 22, { &P18141,&P22712,&P17470,&P18092 }};	/*Useragent - libhttp/2.12 (PS Vita)*/
const Device UA2843078 = {S75028, 22, { &P18141,&P22712,&P17470,&P18092 }};	/*Useragent - libhttp/2.00 (PS Vita)*/
const Device UA3031367 = {S75029, 22, { &P18141,&P22712,&P17470,&P18092 }};	/*Useragent - libhttp/2.60 (PS Vita)*/
const Device UA3083044 = {S75030, 22, { &P18141,&P22712,&P17470,&P18092 }};	/*Useragent - libhttp/2.61 (PS Vita)*/
const Device UA2672078 = {S75031, 138, { &P22848,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT920 Build/2_290_2009) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3107078 = {S75032, 131, { &P22848,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT920 Build/2_32D_2009) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2715154 = {S75033, 156, { &P20081,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7108 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2782435 = {S75034, 156, { &P20081,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7108 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2738126 = {S75035, 91, { &P15989,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i2) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2783937 = {S75036, 90, { &P15989,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2754499 = {S75037, 145, { &P18114,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Transformer Prime TF201 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3082980 = {S75038, 152, { &P24072,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081239 = {S75039, 134, { &P20213,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; N9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081056 = {S75040, 133, { &P24068,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; N986 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080838 = {S75041, 128, { &P19971,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MT7001 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA11317 = {S75042, 138, { &P21193,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; netphone 701 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55313 = {S75043, 140, { &P21193,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Netphone 701 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA22326 = {S75044, 163, { &P15425,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-GT-B7300C_CMCC/1.0 Release/01.24.2009 Browser/IE6 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA46460 = {S75045, 94, { &P15425,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-GT-B7300Orange/BVIJ1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA63033 = {S75046, 86, { &P15425,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-GT-B7300/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA312878 = {S75047, 100, { &P15364,&P7127,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.60 Safari/534.24*/
const Device UA347333 = {S75048, 121, { &P15364,&P5690,&P22414,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.0.2.0 Chrome/16.0.912.59 Safari/535.7*/
const Device UA676547 = {S75049, 142, { &P12897,&P5843,&P24667,&P18092 }};	/*Useragent - NokiaC2-00/2.0 (03.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac2-00) UCBrowser8.2.1.144/70/352/UCWEB Mobile*/
const Device UA829293 = {S75050, 97, { &P15364,&P18134,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1.1) Gecko/X Linux Mint/7 (Gloria) Firefox/3.5.1*/
const Device UA899662 = {S75051, 130, { &P21159,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MID84C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1949478 = {S75052, 143, { &P21476,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ko-kr; SHW-M340L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2205477 = {S75053, 175, { &P15798,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2194823 = {S75054, 132, { &P22094,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; M-MP722I Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2202585 = {S75055, 81, { &P17976,&P6437,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.2; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2531154 = {S75056, 83, { &P17976,&P6437,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.2.2; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2603412 = {S75057, 83, { &P17976,&P6437,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.2.1; Linux; Opera Tablet/ADY-X) Presto/2.11.355 Version/12.10*/
const Device UA2701925 = {S75058, 91, { &P17976,&P6437,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.2; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10,gzip(gfe)*/
const Device UA2370888 = {S75059, 62, { &P15364,&P16110,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; U; en) Presto/2.5.24 Version/10.53*/
const Device UA2474443 = {S75060, 62, { &P15364,&P16110,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; U; en) Presto/2.5.22 Version/10.51*/
const Device UA2525744 = {S75061, 62, { &P15364,&P16110,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; U; en) Presto/2.5.24 Version/10.54*/
const Device UA2685319 = {S75062, 62, { &P15364,&P16110,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; U; en) Presto/2.5.22 Version/10.50*/
const Device UA2723625 = {S75063, 62, { &P15364,&P16110,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; U; en) Presto/2.5.24 Version/10.52*/
const Device UA2426766 = {S75064, 153, { &P20829,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; BC9710AM Build/IMM76D) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2507058 = {S75065, 154, { &P20829,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; BC9710AM  Build/IMM76D) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2834887 = {S75066, 132, { &P20829,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; BC9710AM Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2486680 = {S75067, 43, { &P17968,&P17979,&P18107,&P18092 }};	/*Useragent - Mozilla/ (Android; Mobile;) Gecko/ Firefox/*/
const Device UA2505757 = {S75068, 143, { &P21885,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GSmart GS202+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2557236 = {S75069, 144, { &P23878,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; Movistar Motion Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2567243 = {S75070, 129, { &P19783,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ME172V Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2570466 = {S75071, 140, { &P21969,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; TAC-90012 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2671078 = {S75072, 79, { &P18254,&P17188,&P18245,&P18091 }};	/*Useragent - Mozilla/5.001 (windows; U; NT4.0; en-US; rv:1.0) Gecko/X - crawler@dhoondho.com*/
const Device UA2712061 = {S75073, 133, { &P20988,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PLT7035-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2976015 = {S75074, 142, { &P21020,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Micromax A116 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023426 = {S75075, 133, { &P21020,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; A116 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3039895 = {S75076, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (xokYZOXYGKSLMXnZxQkcjAxr4E3a8+0X3xBe/nw/XKA=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3039897 = {S75077, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (iGtY0hJmov5Fk7HDajUIFHSFd99uX/+KKi5qRkOtGC0=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3039898 = {S75078, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (X6aa0cxTmTyzcR7tJg3bMFMUGU3xsdq29+eu6XdAkHE=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3039899 = {S75079, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (k2d90rHgrieCxWZ7j7LEwgJtAttkMnZE2P+0KY+tAnI=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3039901 = {S75080, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (SfAMt3UVsKT+8oRuuNpdWI4PkbnvCpKxcacjaYU/lm0=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3039902 = {S75081, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (TC0JkyfLsQ88z/VhLsmfJsVs8ox+9y/8X9lg+iiM47E=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3039903 = {S75082, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (guKh6oaFgD+xMc5oGFJI1fq3vzOdjGaqg6BZG0redCA=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3039911 = {S75083, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (dfIqXopg5WI32DlAoLIVAGgDN2xkKJEYxAkRgUGHN9Y=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3039942 = {S75084, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Ls7mTa8UpuMpngc0acQipSbiVKIFJVQxlPjHo2joYE0=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040061 = {S75085, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (0bMPZuK/pZ/MGsoXWZnpxDZS+/EYc5Dxlku2joAQv8U=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040064 = {S75086, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Mqv5sOlSrVW+dUzqWqRlD6AARXq7ovO3vtJH/+u/Rvc=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040072 = {S75087, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (08NM6fWO0spYbbrISyT640jdADTaDrcwt7LJUmRJA84=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040078 = {S75088, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ETTxHyN01okH+hHakfiU9rtc9S5CI4VpReEtqpkuang=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040082 = {S75089, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (GnkUPP2lBJm/9lZYoP7uGAYRR0ms99IG/Q5qufwgfOQ=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040091 = {S75090, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (HJGPZi6aUMHsUq4HW5Pl1O87AJ1GCmyvBmmNLkeH6Eo=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040097 = {S75091, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (62Xq1n+WrUMgx2SQALk+4WgRJH1O4FXJTIniPlv6xpU=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040098 = {S75092, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (6tk33y5hhueCSio09/ceEZwcEtv31GHeKbrx8GSD8sM=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040100 = {S75093, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (9McWXSTeEmMMfA6OsMV9kep3k6cuVZuHY6NWIDd8I2c=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040101 = {S75094, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (BiJcJ/NnWWNRCJbpE1IDAUARApQI13bBpXjUKIJ3NX0=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040102 = {S75095, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (A0EOyAcA49CvJQmybQ9R7LGJppl1vSQUAMNKtbYbklA=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040103 = {S75096, 137, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 () Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040109 = {S75097, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (g6nvFfchIynZi8ANz6XPf+CbzWUMVLJqJcEp+hN8JyU=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040125 = {S75098, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (th82DnmgCskucwSlw5YfKzAa7euPJMHwiXiw3NS2uio=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040187 = {S75099, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (jYgJA+CQFZBTIh8fIZ8yjQSipb2lRoSHvR9EkMfGUKc=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040224 = {S75100, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (IzWk1OkDfyfiLtNbMheKn0Z/1RN3ZhU2J+0yhWKGggQ=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040237 = {S75101, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (O7daEPPR2YhzQal6PXK9cr2EkanWTzKcqJRvh58/wgI=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040246 = {S75102, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (/pvbdYxWRjdoWSNGyIEiz24A/TxU5NIpGCrI1c+6BO8=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040247 = {S75103, 191, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (/pvbdYxWRjdoWSNGyIEiz24A/TxU5NIpGCrI1c+6BO8=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2,gzip(gfe)*/
const Device UA3040248 = {S75104, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (TfYrHRyrSD4Kn2M0+GHEmoJTgX6OLxy4kW1ZuuQLIDM=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040283 = {S75105, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (q2LVBpKdjbTyqvLqnu3cXX5gjjEx1Jq+0EvSmPd3Vpw=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040291 = {S75106, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (XkKA8LspQ7HAPCh3iWpgQ8bGpHzXRROGeMnxx60F5Mc=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040304 = {S75107, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (l7dXj3d5pf5os4ewfPWqDq6O2BKbpVb83l7mJzmt/u4=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040327 = {S75108, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (rt4R8kudpZt4cdRTDJpP4Dml+DiDYOmChp3nTiVQ9us=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040341 = {S75109, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (bTReBGAmyU0p9EZguECS3zebxG/O7ntBasc9XCGRZt4=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040353 = {S75110, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (uze4z7Jwafdyx1Rmk3l3JfpPUIl/6aSpXF7DxANxl8I=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040358 = {S75111, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (7256e72oXTNB/RApA4sEF/AKB7dnIX8dCCLChdrqcgs=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040364 = {S75112, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (seO21gpOtp8uTe8EjZClilM5rgzyrlWqV92C7FPS+dA=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040367 = {S75113, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (VKixsQPwnrpBozTTnkCTM3GrVtsL7a+mT9Gyk8jkqcY=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040372 = {S75114, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (Y6J0A4AypvhLrYtEDuplyc/ynkWaSb7/uFFTeDesFfw=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040373 = {S75115, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (tmcE6pNlnWk/bsOM7/ghgQFOf6UC9tGzNg3xgrBPB0k=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040384 = {S75116, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (sKBlqb3ROanUFyNonRs/B7i7iApx2iN1aOE8r0n9ULY=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040386 = {S75117, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (oSFMNOWJ7mee1o3PNlu10IDyy3flDA9esJ/3OXAKdeg=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA3040389 = {S75118, 181, { &P15364,&P24348,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (wRP81FUXe8n0nEyvvzSY5n8m1S3JRPPA/egWmkaZT40=) Chromium/20.0.1132.57 Chrome/20.0.1132.57 Safari/535.2*/
const Device UA15759 = {S75119, 158, { &P11838,&P17486,&P20417,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola MOTOROKR E6; 2647) MOT-MOTOROKR E6/R533_G_11.12.10P Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [vi]*/
const Device UA24706 = {S75120, 146, { &P11838,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-MOTOROKRE6/R533_G_11.12.08P Mozilla/4.0 (compatible; MSIE 6.0; Linux; MOTOROKRE6; 781) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [en]*/
const Device UA67299 = {S75121, 132, { &P11838,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-MOTOROKRE6/1.0 LinuxOS/2.4.20 Release/8.4.2006 Browser/Opera8.00 Profile/MIDP2.0 Configuration/CLDC1.1 Software/R533_G_11.11.04P*/
const Device UA67748 = {S75122, 134, { &P11838,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-MOTOROKRE6e/1.0 LinuxOS/2.4.20 Release/31.3.2007 Browser/Opera8.00 Profile/MIDP2.0 Configuration/CLDC1.1 Software/R535_G_11.81.02P*/
const Device UA459991 = {S75123, 149, { &P11838,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-MOTOROKR E6/1.0/R533_G_11.12.06P Mozilla/4.0 (compatible; MSIE 6.0; Linux; MOTOROKR E6; 781) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [*/
const Device UA19424 = {S75124, 55, { &P14168,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Viva_T2223 Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA419188 = {S75125, 50, { &P14168,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_T2223 Opera/9.50 (Windows NT 5.1; U; pt)*/
const Device UA477861 = {S75126, 55, { &P14168,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Viva_T2223 Opera/9.50 (Windows NT 5.1; U; ru)*/
const Device UA1839018 = {S75127, 58, { &P14168,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Viva_T2223 Opera/9.50 (Windows NT 5.1; U; es-ES)*/
const Device UA1867183 = {S75128, 55, { &P14168,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Viva_T2223 Opera/9.50 (Windows NT 5.1; U; it)*/
const Device UA44002 = {S75129, 42, { &P13766,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-LG230 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA45015 = {S75130, 41, { &P13766,&P17483,&P3625,&P18092 }};	/*Useragent - LG-LG230 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA68944 = {S75131, 35, { &P14188,&P5129,&P4779,&P18092 }};	/*Useragent - Nokia5233/UC Browser7.8.0.95/50/351*/
const Device UA160948 = {S75132, 71, { &P17346,&P17145,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8C148 (iPod; U; CPU OS 4_2_1 like Mac OS X; en_us)*/
const Device UA788578 = {S75133, 83, { &P15364,&P7133,&P19772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.8.1.6) Gecko/X SeaMonkey/1.1.4*/
const Device UA899776 = {S75134, 78, { &P15364,&P5690,&P22306,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:2.0b11) Gecko/X Firefox/4.0b11 SeaMonkey/2.1b2*/
const Device UA2639290 = {S75135, 89, { &P15364,&P5690,&P22306,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-CA; rv:1.9.3a3pre) Gecko/X SeaMonkey/2.1a1pre*/
const Device UA562545 = {S75136, 13, { &P15364,&P17017,&P17282,&P18092 }};	/*Useragent - Firefox/1.0.4*/
const Device UA1890915 = {S75137, 71, { &P15364,&P17017,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; U; Warp 4.5; en-US; rv:1.7.12) Gecko/X Firefox/1.0.7*/
const Device UA1903153 = {S75138, 70, { &P15364,&P17017,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0 (SkyOS; U; Intel x86; en-US; rv:1.7.5) Gecko/X Firefox/1.0*/
const Device UA790494 = {S75139, 142, { &P14711,&P5204,&P24667,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac2-01) UCBrowser8.2.1.144/70/355/UCWEB Mobile*/
const Device UA1985117 = {S75140, 70, { &P18321,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;GT-S5300;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA2347549 = {S75141, 147, { &P21619,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-mobile i-style 3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2593616 = {S75142, 148, { &P21619,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-mobile i-style 3i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2367200 = {S75143, 141, { &P20231,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A21 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2426795 = {S75144, 133, { &P20231,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A21 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2422244 = {S75145, 73, { &P18649,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; Micromax_A57; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2422725 = {S75146, 163, { &P13308,&P17485,&P24673,&P18092 }};	/*Useragent - SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonj105i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2567255 = {S75147, 133, { &P21603,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AMID-9743G Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567858 = {S75148, 136, { &P12280,&P17146,&P22901,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.3 Mobile/9A405 Safari/7534.48.3*/
const Device UA2703024 = {S75149, 134, { &P12280,&P17146,&P22901,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.3 Mobile/9A334 Safari/7534.48.3*/
const Device UA2671117 = {S75150, 126, { &P15602,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A501 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676086 = {S75151, 140, { &P18084,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2703492 = {S75152, 153, { &P22960,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030D Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2958466 = {S75153, 159, { &P20146,&P18111,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300*/
const Device UA2749142 = {S75154, 213, { &P23428,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SAMSUNG-GT-I8268_TD/1.0 Android/4.1.2 Release/11.15.2012 Browser/AppleWebKit534.30 Build/JRO03L) ApplelWebkit/534.30 (KHTML,like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2891810 = {S75155, 139, { &P23428,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8268 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2895196 = {S75156, 156, { &P20081,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7108 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2963220 = {S75157, 156, { &P20081,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7108 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2879591 = {S75158, 168, { &P12280,&P17090,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5 UCBrowser/9.1.0.297*/
const Device UA3036661 = {S75159, 156, { &P18283,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2975704 = {S75160, 137, { &P24098,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AS870 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3027862 = {S75161, 163, { &P22582,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I337 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jflteatt*/
const Device UA3062765 = {S75162, 163, { &P22582,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I337 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jflteatt*/
const Device UA3116176 = {S75163, 163, { &P22582,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I337 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jflteatt*/
const Device UA3130720 = {S75164, 137, { &P22596,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970C Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132958 = {S75165, 136, { &P22596,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA2095 = {S75166, 81, { &P12089,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN80-1/3.0 (4.0632.0.10) Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA67143 = {S75167, 67, { &P12089,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN80/1.0.0 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA67826 = {S75168, 155, { &P14868,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; Android Dev Phone 1 Build/PLAT-RC33) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA88609 = {S75169, 151, { &P14868,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; Android Dev Phone 1 Build/CRB21) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA88620 = {S75170, 151, { &P14868,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; Android Dev Phone 1 Build/CRB43) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA479415 = {S75171, 150, { &P14868,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; Android Dev Phone 1 Build/CRC1) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA510134 = {S75172, 118, { &P15364,&P6596,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; CrOS i686 0.9.130; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.344 Safari/534.10*/
const Device UA863512 = {S75173, 118, { &P15364,&P6596,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; CrOS i686 0.9.128; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.343 Safari/534.10*/
const Device UA695644 = {S75174, 37, { &P17968,&P6545,&P4755,&P18092 }};	/*Useragent - stagefright/1.1 (Linux;Android 3.0.1)*/
const Device UA1991599 = {S75175, 137, { &P17968,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0UI; en-us; MID Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022836 = {S75176, 161, { &P17968,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0UI; en-us; MID serials 3.0ui honeycomb Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2365428 = {S75177, 147, { &P17968,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0UI; en-in; iBall 3G 7316 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3118386 = {S75178, 107, { &P17968,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0; en-us) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA760123 = {S75179, 116, { &P17595,&P17294,&P23678,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) iCab/4.9 Safari/533.16*/
const Device UA2428148 = {S75180, 146, { &P20168,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Maxwell Plus Build/1.0.3 X-14:07) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2375384 = {S75181, 135, { &P22976,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6358 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2399250 = {S75182, 80, { &P22976,&P5629,&P15760,&P18092 }};	/*Useragent - SAMSUNG-GT-S6358_TD/1.0 Android/2.3.6 Release/1.15.2012 Browser/AppleWebKit533.1*/
const Device UA2589741 = {S75183, 142, { &P22976,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6358 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2400882 = {S75184, 130, { &P20123,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U795 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2416215 = {S75185, 130, { &P20123,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U795 Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2746734 = {S75186, 61, { &P20123,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ZTE U795 Build/IMM76D)*/
const Device UA2984655 = {S75187, 140, { &P20123,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U795+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422582 = {S75188, 151, { &P18648,&P5043,&P24673,&P18092 }};	/*Useragent - Nokia306/2.0 (03.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia306) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2462059 = {S75189, 127, { &P23868,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ST8 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2481742 = {S75190, 139, { &P21950,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; iDxD10 3G Build/ICS.g02ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2502350 = {S75191, 130, { &P21950,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; iDxD10 3G Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2667183 = {S75192, 130, { &P21950,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; iDxD10 3G Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2563073 = {S75193, 95, { &P12329,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAN86 8MP) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2559812 = {S75194, 171, { &P14877,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2565295 = {S75195, 171, { &P14877,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2660051 = {S75196, 171, { &P14877,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2567708 = {S75197, 139, { &P18292,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ609 Build/7.7.1-85_MZ609-31) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2582531 = {S75198, 131, { &P22057,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; el-gr; Tab1020 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2660856 = {S75199, 60, { &P21003,&P20706,&P17470,&P18092 }};	/*Useragent - Nokia310/2.0 (07.35) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2668222 = {S75200, 145, { &P22631,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Fly F40 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2769858 = {S75201, 153, { &P19771,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035X Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2799129 = {S75202, 153, { &P19771,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035E Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2964928 = {S75203, 153, { &P19771,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035D Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3021460 = {S75204, 136, { &P21022,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E615 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022615 = {S75205, 137, { &P21022,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E615f Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3080590 = {S75206, 138, { &P18072,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080650 = {S75207, 138, { &P18072,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094769 = {S75208, 138, { &P18072,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3063055 = {S75209, 139, { &P12681,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus S 4G Build/JRO03R) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA30755 = {S75210, 107, { &P12335,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-A167/A167UCIB3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA475806 = {S75211, 83, { &P15364,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0)*/
const Device UA925870 = {S75212, 97, { &P15364,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0) AdMob;Octagon*/
const Device UA2998614 = {S75213, 95, { &P15364,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; 909)*/
const Device UA3022821 = {S75214, 106, { &P15364,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; HTCPO881; SPRINT)*/
const Device UA706293 = {S75215, 89, { &P17976,&P5629,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.3; Linux; Opera Tablet/ADR-X; U; de) Presto/2.9.201 Version/11.50*/
const Device UA917217 = {S75216, 89, { &P17976,&P5629,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.2; Linux; Opera Tablet/ADR-X; U; en) Presto/2.9.201 Version/11.50*/
const Device UA1922249 = {S75217, 89, { &P17976,&P5629,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.4; Linux; Opera Tablet/ADR-X; U; de) Presto/2.9.201 Version/11.50*/
const Device UA2022111 = {S75218, 89, { &P17976,&P5629,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.1; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.9.201 Version/11.50*/
const Device UA2025204 = {S75219, 87, { &P17976,&P5629,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3; Linux; Opera Tablet/ADR-X; U; en) Presto/2.9.201 Version/11.50*/
const Device UA2033972 = {S75220, 92, { &P17976,&P5629,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.6; Linux; Opera Tablet/ADR-X; U; en-GB) Presto/2.9.201 Version/11.50*/
const Device UA2351985 = {S75221, 103, { &P15364,&P7133,&P21778,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.3.5 Safari/534.34*/
const Device UA2362446 = {S75222, 96, { &P15364,&P7133,&P21778,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.3.5 Safari/534.34*/
const Device UA2428430 = {S75223, 132, { &P22052,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7501 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2458114 = {S75224, 120, { &P15845,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaC5-06) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2509051 = {S75225, 105, { &P12871,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Wildfire_S_A510e) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2698977 = {S75226, 105, { &P12871,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; HTC_Wildfire_S_A510e) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2560872 = {S75227, 164, { &P13308,&P17485,&P24676,&P18092 }};	/*Useragent - SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj105i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2655759 = {S75228, 137, { &P21446,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; SH837W Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2951620 = {S75229, 59, { &P21446,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SH837W Build/JZO54K)*/
const Device UA2698997 = {S75230, 139, { &P22605,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I829 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2989962 = {S75231, 130, { &P22605,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I829 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2701570 = {S75232, 171, { &P15364,&P22887,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/23.0.1271.95 Chrome/23.0.1271.95 Safari/537.6+ Ubuntu/13.04 (3.6.1-2ubuntu1) Epiphany/3.6.1*/
const Device UA2749715 = {S75233, 169, { &P15364,&P22887,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/23.0.1271.95 Chrome/23.0.1271.95 Safari/537.6+ Ubuntu/13.04 (3.6.1-2ubuntu1) Epiphany/3.6.1*/
const Device UA2753703 = {S75234, 138, { &P18386,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I757M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2895421 = {S75235, 134, { &P18464,&P18110,&P23482,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Safari/535.19 Silk-Accelerated=false*/
const Device UA2974467 = {S75236, 138, { &P22738,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT916 Build/2_26D_2014) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3080417 = {S75237, 130, { &P22041,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6810 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA7768 = {S75238, 66, { &P11703,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-L6/0A.52.45R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA11232 = {S75239, 67, { &P11703,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-L6i/0A.65.07R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22274 = {S75240, 149, { &P15417,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Nokia6121c/5.11; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA113625 = {S75241, 34, { &P15364,&P7133,&P22484,&P18092 }};	/*Useragent - Opera/9.10 (Windows NT 6.1; U; en)*/
const Device UA648869 = {S75242, 85, { &P14010,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s5263) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA700819 = {S75243, 85, { &P14010,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5263) UCBrowser8.2.0.132/69/352/UCWEB Mobile*/
const Device UA2027718 = {S75244, 149, { &P20452,&P17979,&P18062,&P18092 }};	/*Useragent - Lenovo S880/S100 Linux/3.0.13 Android/4.0.3 Release/12.12.2011 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30*/
const Device UA2190405 = {S75245, 134, { &P21128,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; S153G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2233270 = {S75246, 134, { &P22002,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; IQ255 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2344606 = {S75247, 141, { &P17952,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; SHW-M110S Build/JELLYBEAN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2364814 = {S75248, 140, { &P20206,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ZTE N880E Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2476823 = {S75249, 140, { &P20206,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE N880E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2574722 = {S75250, 131, { &P20206,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ZTE N880E Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2829283 = {S75251, 131, { &P20206,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE N880E Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2958402 = {S75252, 136, { &P20206,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N880E Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2435434 = {S75253, 138, { &P20200,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS GAMEPAD Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2484451 = {S75254, 127, { &P23868,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ST8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2985850 = {S75255, 56, { &P23868,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; ST8 Build/JRO03C)*/
const Device UA2511663 = {S75256, 40, { &P17968,&P17979,&P24441,&P18092 }};	/*Useragent - Dolphin http client/9.3.1(200) (Android)*/
const Device UA2672040 = {S75257, 40, { &P17968,&P17979,&P24441,&P18092 }};	/*Useragent - Dolphin http client/9.3.2(203) (Android)*/
const Device UA2566819 = {S75258, 144, { &P15854,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ607-14) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567214 = {S75259, 139, { &P22590,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717M Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2664790 = {S75260, 135, { &P22022,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; eZee'TAB707 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2704833 = {S75261, 134, { &P23429,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; miTab FUNK Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2782970 = {S75262, 157, { &P18295,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC One X Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2716001 = {S75263, 93, { &P18672,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5302) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2976108 = {S75264, 136, { &P22804,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P713 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023392 = {S75265, 138, { &P22804,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P713GO Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3050744 = {S75266, 104, { &P15364,&P16110,&P21712,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.9.1 Safari/534.34*/
const Device UA3111098 = {S75267, 114, { &P15786,&P5043,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-05/08.69; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA14226 = {S75268, 135, { &P15364,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 640x480; HTC_Advantage/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1) Opera 8.65 [en]*/
const Device UA415682 = {S75269, 76, { &P15364,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x240) Opera 8.65 [en]*/
const Device UA863713 = {S75270, 76, { &P15364,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320) Opera 8.65 [nl]*/
const Device UA872483 = {S75271, 76, { &P15364,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 320x320) Opera 8.65 [en]*/
const Device UA882050 = {S75272, 76, { &P15364,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x240) Opera 8.60 [en]*/
const Device UA1857303 = {S75273, 79, { &P15364,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320) Opera 8.65 [zh-cn]*/
const Device UA2516797 = {S75274, 76, { &P15364,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 480x640) Opera 8.60 [en]*/
const Device UA2785760 = {S75275, 79, { &P15364,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Sprint:PPC-6700) Opera 8.65 [en]*/
const Device UA3097377 = {S75276, 75, { &P15364,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 320x320)Opera 8.65 [en]*/
const Device UA23015 = {S75277, 151, { &P15554,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; fr-fr; Android Edition Starnaute Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1991717 = {S75278, 153, { &P15554,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; fr-fr; Android Edition Starnaute Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA249860 = {S75279, 53, { &P18088,&P17017,&P24666,&P18092 }};	/*Useragent - IUC(U;iOS 4.3.5;Zh-cn;320*480;)/UCWEB8.1.0.104/41/997*/
const Device UA353111 = {S75280, 53, { &P18088,&P17017,&P24666,&P18092 }};	/*Useragent - IUC(U;iOS 5.0.1;Zh-cn;320*480;)/UCWEB8.1.4.125/42/997*/
const Device UA1925089 = {S75281, 51, { &P18088,&P17017,&P24666,&P18092 }};	/*Useragent - IUC(U;iOS 4.1;Zh-cn;320*480;)/UCWEB8.1.2.111/41/800*/
const Device UA2215432 = {S75282, 53, { &P18088,&P17017,&P24666,&P18092 }};	/*Useragent - IUC(U;iOS 4.2.1;Zh-cn;320*480;)/UCWEB8.1.2.111/41/997*/
const Device UA2334257 = {S75283, 53, { &P18088,&P17017,&P24666,&P18092 }};	/*Useragent - IUC(U;iOS 5.1.1;Zh-cn;320*480;)/UCWEB8.1.2.111/42/997*/
const Device UA715644 = {S75284, 90, { &P14929,&P5156,&P4779,&P18092 }};	/*Useragent - BlackBerry9780/6.5.0.54 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/159/352*/
const Device UA1906311 = {S75285, 91, { &P14929,&P5156,&P4779,&P18092 }};	/*Useragent - BlackBerry9780/6.6.0.207 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/159/444*/
const Device UA443993 = {S75286, 137, { &P15364,&P5690,&P17242,&P18091 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; maxamine.com-robot) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.127 Safari/533.4*/
const Device UA605283 = {S75287, 124, { &P15364,&P16110,&P22457,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19*/
const Device UA772276 = {S75288, 117, { &P15364,&P16110,&P22457,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19*/
const Device UA2176335 = {S75289, 134, { &P23892,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; Hd1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2848296 = {S75290, 134, { &P23892,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; Hd1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2203000 = {S75291, 189, { &P15364,&P7133,&P18527,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/84/352*/
const Device UA2203526 = {S75292, 189, { &P15364,&P7133,&P18527,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/69/352*/
const Device UA2229358 = {S75293, 189, { &P15364,&P7133,&P18527,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/70/352*/
const Device UA2361729 = {S75294, 189, { &P15364,&P7133,&P18527,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/69/444*/
const Device UA2366715 = {S75295, 189, { &P15364,&P7133,&P18527,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/84/444*/
const Device UA2385001 = {S75296, 189, { &P15364,&P7133,&P18527,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/69/405*/
const Device UA2419384 = {S75297, 199, { &P15364,&P7133,&P18527,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/69/352  3gpp-gba*/
const Device UA2421316 = {S75298, 189, { &P15364,&P7133,&P18527,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/70/444*/
const Device UA2456442 = {S75299, 189, { &P15364,&P7133,&P18527,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/70/405*/
const Device UA2653077 = {S75300, 199, { &P15364,&P7133,&P18527,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/69/444  3gpp-gba*/
const Device UA2839003 = {S75301, 199, { &P15364,&P7133,&P18527,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/84/352  3gpp-gba*/
const Device UA2387596 = {S75302, 120, { &P14983,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaC7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile*/
const Device UA2424610 = {S75303, 163, { &P15555,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; myTouch_4G_Slide Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2529696 = {S75304, 163, { &P15555,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MyTouch 4G Slide Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567019 = {S75305, 126, { &P14111,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Arc Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA3046197 = {S75306, 140, { &P14111,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT15a Build/4.1.B.0.587) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2773159 = {S75307, 130, { &P22873,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5120 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2898734 = {S75308, 156, { &P16356,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N7000 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3032361 = {S75309, 164, { &P18614,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT890 Build/9.8.2I-50_SMI-26) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2974931 = {S75310, 137, { &P18469,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabS2109A-F Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2968609 = {S75311, 92, { &P15791,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia603) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3068432 = {S75312, 130, { &P14905,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3081106 = {S75313, 138, { &P18292,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ609 Build/7.7.1-85_MZ609-31) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA20098 = {S75314, 133, { &P11959,&P5338,&P3772,&P18092 }};	/*Useragent - MOT-RAZRV3XXR_J/97.04.30R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003290) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.60 [en]*/
const Device UA541245 = {S75315, 127, { &P11959,&P5338,&P3772,&P18092 }};	/*Useragent - MOT-RAZRV3XXR_J/97.04.30R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; X) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [en]*/
const Device UA48989 = {S75316, 120, { &P13882,&P17484,&P3834,&P18092 }};	/*Useragent - SAMSUNG-GT-S8500T/1.0 SHP/VPP/R5 Dolfin/2.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 S8500DVJF3*/
const Device UA50229 = {S75317, 138, { &P13201,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; Eris Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA140556 = {S75318, 144, { &P12280,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; like iPhone; U; en-US) AppleWebKit/528.5+ (KHTML, like Gecko, Safari/528.5+) Version/4.0 Kindle/3.0 (screen 600x800; rotate)*/
const Device UA156750 = {S75319, 133, { &P17346,&P17137,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18 Twitter for iPhone*/
const Device UA293357 = {S75320, 133, { &P17346,&P17137,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7D11 Twitter for iPhone*/
const Device UA400084 = {S75321, 134, { &P17346,&P17137,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7C145 Twitter for iPhone*/
const Device UA223920 = {S75322, 71, { &P18088,&P5448,&P4779,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA253792 = {S75323, 72, { &P15364,&P22470,&P17825,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/3.5; Darwin) KHTML/3.5.6 (like Gecko)*/
const Device UA369652 = {S75324, 127, { &P23874,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; S8600 Build/HuaweiS8600) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA746962 = {S75325, 137, { &P23874,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HW-HUAWEI_S8600/S8600V100R001C92B119; 480*800;CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2029642 = {S75326, 132, { &P23874,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; S8600 Build/HuaweiS8600) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2992965 = {S75327, 139, { &P23874,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; S8600 Build/HuaweiS8600) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA647424 = {S75328, 71, { &P17595,&P18577,&P17317,&P18092 }};	/*Useragent - Safari/6533.21.1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBook5%2C1)*/
const Device UA651046 = {S75329, 71, { &P17595,&P18577,&P17317,&P18092 }};	/*Useragent - Safari/6533.21.1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (Macmini3%2C1)*/
const Device UA661143 = {S75330, 74, { &P17595,&P18577,&P17317,&P18092 }};	/*Useragent - Safari/6533.21.1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro4%2C1)*/
const Device UA701620 = {S75331, 74, { &P17595,&P18577,&P17317,&P18092 }};	/*Useragent - Safari/6533.21.1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookAir3%2C1)*/
const Device UA767491 = {S75332, 71, { &P17595,&P18577,&P17317,&P18092 }};	/*Useragent - Safari/6533.21.1 CFNetwork/454.12.4 Darwin/10.8.0 (x86_64) (iMac12%2C1)*/
const Device UA872000 = {S75333, 70, { &P17595,&P18577,&P17317,&P18092 }};	/*Useragent - Safari/6533.21.1 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacPro3%2C1)*/
const Device UA900369 = {S75334, 61, { &P17968,&P17979,&P22244,&P18092 }};	/*Useragent - Flipboard/1.8.4 beta 1/65,1.8.4,2012-05-30 18:15, us, Android*/
const Device UA900787 = {S75335, 57, { &P17968,&P17979,&P22244,&P18092 }};	/*Useragent - Flipboard/1.8.4 beta 1/65,1.8.4,2012-05-30 18:15, Android*/
const Device UA901344 = {S75336, 61, { &P17968,&P17979,&P22244,&P18092 }};	/*Useragent - Flipboard/1.8.4 beta 1/65,1.8.4,2012-05-30 18:15, gb, Android*/
const Device UA2027230 = {S75337, 54, { &P17968,&P17979,&P22244,&P18092 }};	/*Useragent - Flipboard/1.8.4/65,1.8.4,2012-05-30 18:17, gb, Android*/
const Device UA2033026 = {S75338, 64, { &P17968,&P17979,&P22244,&P18092 }};	/*Useragent - Flipboard/1.8.4 beta 3/86,1.8.4.86,2012-06-14 14:31, gb, Android*/
const Device UA2222501 = {S75339, 50, { &P17968,&P17979,&P22244,&P18092 }};	/*Useragent - Flipboard/1.8.4/65,1.8.4,2012-05-30 18:17, Android*/
const Device UA2223155 = {S75340, 57, { &P17968,&P17979,&P22244,&P18092 }};	/*Useragent - Flipboard/1.8.4/73,1.8.4.73,2012-06-06 12:32, us, Android*/
const Device UA2818227 = {S75341, 53, { &P17968,&P17979,&P22244,&P18092 }};	/*Useragent - Flipboard/1.8.4/73,1.8.4.73,2012-06-06 12:32, Android*/
const Device UA909556 = {S75342, 140, { &P21014,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Qmobile-S11 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029467 = {S75343, 153, { &P24573,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; TURKCELL MaxiPRO5 Build/HuaweiU8860) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2391577 = {S75344, 98, { &P15655,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s3850) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2461332 = {S75345, 166, { &P18442,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-N7100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2469424 = {S75346, 166, { &P18442,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-N7100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2487411 = {S75347, 138, { &P16362,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SPH-D710 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2563805 = {S75348, 137, { &P16362,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-D710 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2487942 = {S75349, 99, { &P12080,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3653) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2528246 = {S75350, 133, { &P21144,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; F8 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2843482 = {S75351, 137, { &P21144,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; FPT F8 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2536811 = {S75352, 131, { &P23506,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GEM7020 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2664414 = {S75353, 93, { &P18442,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7100) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2556262 = {S75354, 131, { &P20752,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; AN10CG3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2560136 = {S75355, 136, { &P23964,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.4; zh-tw; HD88+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2660063 = {S75356, 136, { &P23964,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.4; en-us; HD88+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2764760 = {S75357, 136, { &P23964,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.4; en-gb; HD88+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3071647 = {S75358, 136, { &P23964,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.4; zh-cn; HD88+ Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567004 = {S75359, 142, { &P11747,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire V Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567138 = {S75360, 138, { &P18469,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabS2109A-F Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2598749 = {S75361, 103, { &P14983,&P5055,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac7-00) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2855908 = {S75362, 32, { &P14983,&P5055,&P24676,&P18092 }};	/*Useragent - NokiaC7-00/UCWEB8.9.0.253/51/999*/
const Device UA2602287 = {S75363, 161, { &P15575,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2667135 = {S75364, 161, { &P15575,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2667910 = {S75365, 161, { &P15575,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9108 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2669396 = {S75366, 162, { &P15575,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9100G Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2853776 = {S75367, 161, { &P15575,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.1.258 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2784787 = {S75368, 131, { &P15767,&P18432,&P22329,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Rockmelt/3.0.3.7 Mobile/10A525 Safari/8536.25*/
const Device UA2737847 = {S75369, 139, { &P22978,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6313T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2738258 = {S75370, 138, { &P22602,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-S738C Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2754165 = {S75371, 141, { &P11747,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire V Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868274 = {S75372, 142, { &P11747,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire VC Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2936974 = {S75373, 90, { &P15364,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; C5302) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2954337 = {S75374, 88, { &P15364,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; A27) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2956825 = {S75375, 93, { &P15364,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; IRIS_455) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2958334 = {S75376, 90, { &P15364,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; C2104) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2986772 = {S75377, 92, { &P15364,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Nexus_S) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3049009 = {S75378, 92, { &P15364,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; LG-E455) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3056605 = {S75379, 99, { &P15364,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Connect-2G-2.0) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3075870 = {S75380, 105, { &P15364,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Symphony_Xplorer_W35) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3087711 = {S75381, 95, { &P15364,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; XOLO_A1000) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2974929 = {S75382, 143, { &P19855,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT885 Build/6.7.3-135_YIC-8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3080257 = {S75383, 126, { &P20269,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A211 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094737 = {S75384, 158, { &P15989,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.4.27) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3110919 = {S75385, 158, { &P15989,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.0.16) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3111718 = {S75386, 158, { &P15989,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.6.8) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3113257 = {S75387, 158, { &P15989,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.4.22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3116527 = {S75388, 158, { &P15989,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.1.8) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127386 = {S75389, 158, { &P15989,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i2 Build/11.0.A.6.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA14020 = {S75390, 100, { &P11784,&P16000,&P3772,&P18092 }};	/*Useragent - Opera 7.0 SonyEricssonW950i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 513) Opera 8.65 [en]*/
const Device UA62560 = {S75391, 90, { &P11784,&P16000,&P3772,&P18092 }};	/*Useragent - SonyEricssonW950i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 323) Opera 8.60 [en]*/
const Device UA64251 = {S75392, 90, { &P11784,&P16000,&P3772,&P18092 }};	/*Useragent - SonyEricssonW950i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 598) Opera 8.65 [en]*/
const Device UA77034 = {S75393, 101, { &P11784,&P16000,&P3772,&P18092 }};	/*Useragent - Opera 8.54 SonyEricssonW950i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 513) Opera 8.65 [en]*/
const Device UA21396 = {S75394, 191, { &P11991,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/5.0 Vodafone/1.0/LG-GM750/V10b Browser/IE8.12 MMS/LGMMSv1.0/1.2 Java/LGVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA20519 = {S75395, 74, { &P15364,&P5448,&P22272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) T-Mobile_Juno*/
const Device UA49247 = {S75396, 74, { &P15364,&P5448,&P22272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) Vodafone 1520*/
const Device UA86859 = {S75397, 75, { &P15364,&P5448,&P22272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) Sprint:MotoQ9c*/
const Device UA87101 = {S75398, 75, { &P15364,&P5448,&P22272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) Alltel:MotoQ9c*/
const Device UA342440 = {S75399, 142, { &P17951,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Milestone Build/R.U.R.1920) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623133 = {S75400, 142, { &P17951,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; iw-il; Milestone Build/R.U.R.1920) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA646005 = {S75401, 66, { &P17951,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; Milestone Build/R.U.R.1920)*/
const Device UA738511 = {S75402, 142, { &P17951,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; Milestone Build/R.U.R.1920) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA788887 = {S75403, 72, { &P17951,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; Milestone Build/SHOLS_U2_05.26.3)*/
const Device UA2019118 = {S75404, 142, { &P17951,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; es-es; Milestone Build/R.U.R.1920) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019590 = {S75405, 142, { &P17951,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Milestone Build/R.U.R.1920) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021007 = {S75406, 136, { &P17951,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Milestone Build/SHOLS_U2_05.26.3) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA136947 = {S75407, 124, { &P17595,&P18120,&P18211,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/5.2.1.1588 Safari/532.4*/
const Device UA217718 = {S75408, 92, { &P15364,&P7127,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; Edition United States Local; en) Presto/2.9.168 Version/11.50*/
const Device UA396178 = {S75409, 32, { &P18095,&P18094,&P22726,&P18092 }};	/*Useragent - Mozilla/5.0 (Nokia; Qt; Symbian)*/
const Device UA699852 = {S75410, 92, { &P17595,&P17294,&P22480,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.4; en-US; rv:1.9) Gecko/X Firefox/3.0 Flock/2.0b1*/
const Device UA906090 = {S75411, 64, { &P15364,&P18145,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64; rv:13.0) Gecko/X Firefox/13.0.1*/
const Device UA1997334 = {S75412, 138, { &P19910,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E617G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2228133 = {S75413, 144, { &P12765,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A100 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2554494 = {S75414, 151, { &P12765,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A100 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2722743 = {S75415, 144, { &P12765,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A100 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2850321 = {S75416, 166, { &P12765,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A100 Build/JZO54K; CyanogenMod-Android 4.1.2 CM10 build) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2235067 = {S75417, 131, { &P22124,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; TAB 804 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2490825 = {S75418, 99, { &P12137,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5620) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2503847 = {S75419, 132, { &P20523,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; GT-P3105 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2621763 = {S75420, 61, { &P20523,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P3105 Build/IMM76D)*/
const Device UA2531126 = {S75421, 151, { &P19747,&P5043,&P24673,&P18092 }};	/*Useragent - Nokia206/2.0 (03.59) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia206) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA3102541 = {S75422, 155, { &P19747,&P5043,&P24673,&P18092 }};	/*Useragent - Nokia206.1/2.0 (03.58) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia206.1) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2538024 = {S75423, 165, { &P18241,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; LG-E400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3022572 = {S75424, 169, { &P18241,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-E400 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2581107 = {S75425, 138, { &P21739,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iris454 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2586550 = {S75426, 126, { &P14111,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Arc Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2671807 = {S75427, 144, { &P22621,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax P360 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2715469 = {S75428, 94, { &P20134,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2781830 = {S75429, 77, { &P23009,&P22784,&P3680,&P18092 }};	/*Useragent - Micromax X321/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2879726 = {S75430, 159, { &P12798,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-CN; LT18i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2955667 = {S75431, 150, { &P15201,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_Sensation_Z710e Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956321 = {S75432, 147, { &P15201,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC_Sensation_4G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2992053 = {S75433, 150, { &P15201,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC Sensation Z710e Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2992079 = {S75434, 147, { &P15201,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Sensation 4G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2965785 = {S75435, 164, { &P20146,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I9082 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2973489 = {S75436, 137, { &P22872,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5282 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022223 = {S75437, 152, { &P19771,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022373 = {S75438, 152, { &P19771,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023395 = {S75439, 152, { &P19771,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3083249 = {S75440, 138, { &P18747,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A840SP Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083372 = {S75441, 137, { &P18747,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A840S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3096286 = {S75442, 157, { &P18595,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA5106 = {S75443, 150, { &P13349,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-hk; HTC Aria A6380 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA5110 = {S75444, 150, { &P13349,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-au; HTC_Aria_A6380 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA63074 = {S75445, 97, { &P14475,&P17483,&P3534,&P18092 }};	/*Useragent - LG-A165/V100 Obigo/Q05A MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA24722 = {S75446, 41, { &P15744,&P17484,&P3625,&P18092 }};	/*Useragent - sam-r580 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA90282 = {S75447, 31, { &P17648,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F01C(c500;TB;W24H16)*/
const Device UA132017 = {S75448, 31, { &P17648,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F01B(c500;TB;W24H16)*/
const Device UA659136 = {S75449, 80, { &P13706,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5235)/UC Browser8.2.0.132/50/355/UCWEB Mobile*/
const Device UA2258433 = {S75450, 122, { &P14682,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia6303iclassic/10.80; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA2230309 = {S75451, 135, { &P23981,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; 9300+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2423708 = {S75452, 161, { &P14743,&P17485,&P24673,&P18092 }};	/*Useragent - SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonj20i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2487849 = {S75453, 69, { &P15364,&P7133,&P19981,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:19.0) Gecko/X Thunderbird/19.0*/
const Device UA2561066 = {S75454, 162, { &P15575,&P18066,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2566818 = {S75455, 139, { &P18474,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9210T Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2585268 = {S75456, 152, { &P20129,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; MI 2 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2674224 = {S75457, 138, { &P19902,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530M Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2974731 = {S75458, 134, { &P23179,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ZP980 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975991 = {S75459, 135, { &P24102,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SM-C101 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3026604 = {S75460, 135, { &P18442,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-N7100 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3028292 = {S75461, 135, { &P18442,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-N7105 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3031049 = {S75462, 135, { &P18442,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SPH-L900 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3031322 = {S75463, 143, { &P18442,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SAMSUNG-SGH-I317 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3038885 = {S75464, 141, { &P18442,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Galaxy Note II Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3080709 = {S75465, 141, { &P19997,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo P700i Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114671 = {S75466, 140, { &P19997,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Lenovo P700 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3130938 = {S75467, 129, { &P23955,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5210 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA79176 = {S75468, 75, { &P16696,&P22857,&P4812,&P18092 }};	/*Useragent - Spice M6900 Opera/9.80 (MTK; Nucleus; U; en-US) Presto/2.4.18 Version/10.00*/
const Device UA336275 = {S75469, 76, { &P16696,&P22857,&P4812,&P18092 }};	/*Useragent - Spice M6900  Opera/9.80 (MTK; Nucleus; U; en-US) Presto/2.4.18 Version/10.00*/
const Device UA151009 = {S75470, 154, { &P14800,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT800W Build/TTSKT_U_80.10.24R) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA640513 = {S75471, 154, { &P14800,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT800W Build/TTSKT_U_80.10.28R) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA647199 = {S75472, 137, { &P14800,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT800 Build/TITA_M2_16.22.7) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA680272 = {S75473, 151, { &P14800,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT800 Build/TITA_M2_16.22.7) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA703158 = {S75474, 137, { &P14800,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT800 Build/TITA_M2_16.35.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1865434 = {S75475, 151, { &P14800,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT800 Build/TITA_M2_16.35.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018312 = {S75476, 151, { &P14800,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT800 Build/TITA_M2_16.30.3) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018465 = {S75477, 76, { &P14800,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; XT800W Build/TTSKT_U_80.10.28R)*/
const Device UA2019966 = {S75478, 137, { &P14800,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT800 Build/TITA_M2_16.30.3) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2493839 = {S75479, 151, { &P14800,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT800 Build/TITA_M2_20.08.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2685253 = {S75480, 142, { &P14800,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT800 Build/TITA_M2_16.35.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA175462 = {S75481, 132, { &P15364,&P17285,&P22447,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Iron/9.0.600.2 Chrome/9.0.600.2 Safari/534.13*/
const Device UA627065 = {S75482, 69, { &P15364,&P17017,&P18272,&P18092 }};	/*Useragent - Browser/Opera  Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA830700 = {S75483, 93, { &P15364,&P22198,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-GB; rv:1.7.13) Gecko/X Fedora/1.0.8-1.1.fc4 Firefox/1.0.8*/
const Device UA2199123 = {S75484, 139, { &P20128,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT-AS43D2 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2433170 = {S75485, 139, { &P20128,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT-AS43D2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2202772 = {S75486, 130, { &P15767,&P18432,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3 Sleipnir/2.1.1m*/
const Device UA2237426 = {S75487, 130, { &P15767,&P18432,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3 Sleipnir/2.1.2m*/
const Device UA2358008 = {S75488, 132, { &P15767,&P18432,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 Sleipnir/2.1.3m*/
const Device UA2387355 = {S75489, 130, { &P15767,&P18432,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3 Sleipnir/2.1.3m*/
const Device UA2406826 = {S75490, 132, { &P15767,&P18432,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 Sleipnir/2.1.4m*/
const Device UA2408251 = {S75491, 132, { &P15767,&P18432,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 Sleipnir/2.1.5m*/
const Device UA2427057 = {S75492, 167, { &P14557,&P5843,&P24673,&P18092 }};	/*Useragent - Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6303classic) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2425761 = {S75493, 132, { &P20733,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAB-1030 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2467441 = {S75494, 132, { &P20733,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TAB-1030 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2566963 = {S75495, 146, { &P18476,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I577 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2566770 = {S75496, 131, { &P14905,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7100 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2637795 = {S75497, 131, { &P14905,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P7100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2601345 = {S75498, 134, { &P23311,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Sweet M971 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2861393 = {S75499, 63, { &P23311,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Sweet M971 Build/JRO03H)*/
const Device UA2755054 = {S75500, 138, { &P19844,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80 COBALT Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2879607 = {S75501, 142, { &P23693,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PNDPP48GP Build/ICS.PNDPP48GP.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3020976 = {S75502, 97, { &P22622,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Micromax_A72) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3048718 = {S75503, 139, { &P24425,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ONE TOUCH TAB 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3093243 = {S75504, 140, { &P24425,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ONE TOUCH TAB 7 Build/GSDT011) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3096463 = {S75505, 111, { &P15364,&P18148,&P24641,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-914_im_india_269) U2/1.0.0 UCBrowser/3.1.1.343 U2/1.0.0 Mobile*/
const Device UA3111046 = {S75506, 111, { &P15364,&P18148,&P24641,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-885_im_india_249) U2/1.0.0 UCBrowser/3.1.1.343 U2/1.0.0 Mobile*/
const Device UA61788 = {S75507, 66, { &P14330,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GX300/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA66960 = {S75508, 10, { &P13552,&P5129,&P4030,&P18092 }};	/*Useragent - NokiaC6-00*/
const Device UA316694 = {S75509, 133, { &P18024,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; A601 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA734859 = {S75510, 133, { &P18024,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; A601 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA627442 = {S75511, 42, { &P15373,&P5129,&P24665,&P18092 }};	/*Useragent - Nokia5530/UC Browser8.0.4.121/50/444/UCWEB*/
const Device UA643270 = {S75512, 30, { &P15373,&P5129,&P24665,&P18092 }};	/*Useragent - Nokia5530/UCWEB8.0.3.99/50/999*/
const Device UA578668 = {S75513, 114, { &P14714,&P5843,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02/05.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA615278 = {S75514, 132, { &P15364,&P7133,&P22428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19 Comodo_Dragon/18.3.0.0*/
const Device UA615348 = {S75515, 125, { &P15364,&P7133,&P22428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19 Comodo_Dragon/18.3.0.0*/
const Device UA887934 = {S75516, 140, { &P21068,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE C1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1969678 = {S75517, 131, { &P21068,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE C1 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2027516 = {S75518, 126, { &P21068,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE C1 Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2030078 = {S75519, 81, { &P21706,&P5629,&P15760,&P18092 }};	/*Useragent - SAMSUNG-GT-I8250_TD/1.0 Android/2.3.6 Release/11.11.2011 Browser/AppleWebKit533.1*/
const Device UA2395837 = {S75520, 135, { &P21706,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8250 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2546221 = {S75521, 142, { &P21706,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8250 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2362979 = {S75522, 140, { &P21077,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; MID8127 Build/GreenDemon_v1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2456568 = {S75523, 71, { &P12444,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.1-update1; zh-cn; U20i; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2432540 = {S75524, 226, { &P18442,&P18111,&P18369,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.31270/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2482559 = {S75525, 136, { &P22673,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Etel i3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2531651 = {S75526, 51, { &P17595,&P18120,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 (iOS 4S; U; CPU like Mac OS X; zh-cn)*/
const Device UA2531701 = {S75527, 50, { &P17595,&P18120,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 (iOS 4; U; CPU like Mac OS X; zh-cn)*/
const Device UA2560294 = {S75528, 50, { &P17595,&P18120,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 (iOS 5; U; CPU like Mac OS X; zh-cn)*/
const Device UA2567496 = {S75529, 133, { &P13644,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; EVO Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2582913 = {S75530, 84, { &P23326,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-T96 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1 firmware/T96 T02*/
const Device UA2774325 = {S75531, 159, { &P15989,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21i Build/11.0.A.4.22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2869562 = {S75532, 159, { &P15989,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21i2 Build/11.0.A.6.5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2753980 = {S75533, 135, { &P20119,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_S9714 Build/JRO03R) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2768704 = {S75534, 139, { &P24070,&P18598,&P19802,&P18092 }};	/*Useragent - OPPO_R815T/1.0 Linux/3.4.0 Android/4.2.1  Release/12.24.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2 System/Android 4.2.1;*/
const Device UA2937084 = {S75535, 136, { &P24070,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; R815T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049375 = {S75536, 58, { &P24070,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; R815T Build/JOP40D)*/
const Device UA3039299 = {S75537, 148, { &P15985,&P6437,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.2; GT-P6200 Build/HTJ85B) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2975967 = {S75538, 130, { &P14905,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3093652 = {S75539, 141, { &P16292,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.1; Galaxy Nexus Build/ITL41F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094707 = {S75540, 141, { &P16292,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.2; Galaxy Nexus Build/ICL53F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094786 = {S75541, 141, { &P16292,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114967 = {S75542, 114, { &P15837,&P6909,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-05/08.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA24382 = {S75543, 73, { &P15684,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-C3560/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24392 = {S75544, 45, { &P14103,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP502-V10b Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA24501 = {S75545, 40, { &P14103,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP502 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA55016 = {S75546, 93, { &P12013,&P5448,&P4578,&P18092 }};	/*Useragent - SAMSUNG-SGH-I607/I607FG1 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 320x240)*/
const Device UA368667 = {S75547, 127, { &P13042,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Incredible S Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA661921 = {S75548, 139, { &P13042,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Incredible S Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034624 = {S75549, 132, { &P13042,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Incredible S Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2293838 = {S75550, 139, { &P13042,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Incredible S Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA601156 = {S75551, 85, { &P12137,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s5620) UCBrowser8.2.0.132/69/452/UCWEB Mobile*/
const Device UA636882 = {S75552, 85, { &P12137,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s5620) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA1992399 = {S75553, 133, { &P22135,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SmartTab1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2203509 = {S75554, 93, { &P17595,&P17295,&P20755,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:15.0) Gecko/X Firefox/15.0.1 SeaMonkey/2.12.1*/
const Device UA2431917 = {S75555, 229, { &P18214,&P5629,&P17373,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/6.5.28192/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2422805 = {S75556, 129, { &P21280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MP969 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2485538 = {S75557, 131, { &P20526,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TOUCH96 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2566535 = {S75558, 136, { &P14574,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus S Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667492 = {S75559, 137, { &P14574,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus S Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2655794 = {S75560, 154, { &P17625,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xperia U Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2714736 = {S75561, 137, { &P22909,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Flame2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2717072 = {S75562, 152, { &P18583,&P5043,&P24677,&P18092 }};	/*Useragent - Nokia110/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia110) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2718911 = {S75563, 151, { &P19860,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A9+ Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2738128 = {S75564, 159, { &P19860,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A9+ Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3083765 = {S75565, 139, { &P21507,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C1605 Build/11.1.A.0.68) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2167352 = {S75566, 54, { &P17579,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SN3S UP.Browser/6.2_7.2.7.1.K.6.212 (GUI) MMP/2.0*/
const Device UA1904546 = {S75567, 137, { &P17187,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; BLU Magic Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA371356 = {S75568, 72, { &P14802,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i917.;7.10;7720)*/
const Device UA437093 = {S75569, 71, { &P14802,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i917;7.10;7720)*/
const Device UA396219 = {S75570, 70, { &P11747,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2; zh-cn; HTC_Desire; 480*800) UCWEB7.9.3.103/139/800*/
const Device UA660615 = {S75571, 70, { &P11747,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2; zh-cn; HTC_Desire; 480*800) UCWEB7.9.3.103/139/999*/
const Device UA2147138 = {S75572, 72, { &P11747,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.7; zh-cn; HTC_Desire; 480*800) UCWEB7.9.3.103/139/800*/
const Device UA2158577 = {S75573, 70, { &P11747,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2; zh-cn; HTC_Desire; 480*800) UCWEB7.9.4.145/139/800*/
const Device UA2296958 = {S75574, 71, { &P11747,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2; zh-cn; HTC_Desire; 480*800) UCWEB7.9.3.103/139/1002*/
const Device UA2297559 = {S75575, 70, { &P11747,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2; zh-cn; HTC_Desire; 480*800) UCWEB7.9.1.100/139/800*/
const Device UA2502614 = {S75576, 69, { &P11747,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2; zh-cn; HTC_Desire; 480*800) UCWEB7.9.0.94/139/352*/
const Device UA2503373 = {S75577, 73, { &P11747,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC_Desire_V; 480*800) UCWEB7.9.0.94/139/355*/
const Device UA2578072 = {S75578, 73, { &P11747,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC_Desire_V; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2746557 = {S75579, 73, { &P11747,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC_Desire_V; 480*800) UCWEB7.9.0.94/139/352*/
const Device UA2838474 = {S75580, 69, { &P11747,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2; zh-cn; HTC_Desire; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA430254 = {S75581, 64, { &P15364,&P7133,&P23788,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Windows NT 6.1; en-us; dream) DoggCatcher*/
const Device UA623717 = {S75582, 185, { &P15364,&P17285,&P24197,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0;	Trident/4.0; SLCC2;	.NET CLR 2.0.50727; .NET CLR 3.5.30729;	.NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E) QQBrowser/6.0*/
const Device UA626154 = {S75583, 114, { &P15364,&P17285,&P24197,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.0*/
const Device UA898697 = {S75584, 64, { &P15364,&P17285,&P24197,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0) QQBrowser/6.0*/
const Device UA714066 = {S75585, 168, { &P14114,&P17296,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-gb; Silk/1.0.13.81_X)Maxthon AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16 Silk-Accelerated=true*/
const Device UA1985819 = {S75586, 150, { &P23985,&P5096,&P3508,&P18092 }};	/*Useragent - HUAWEI_T8600_TD/1.0 OPhone/2.5 (Linux;Android 2.2) Release/11.01.2011 Browser/WAP2.0(AppleWebKit/533.1) Mobile Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2022120 = {S75587, 140, { &P23985,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HUAWEI T8600 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024068 = {S75588, 128, { &P23985,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HUAWEI T8600 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2034156 = {S75589, 133, { &P23985,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HUAWEI T8600 Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2257209 = {S75590, 151, { &P13974,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5670 Build/GRI40; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2475652 = {S75591, 162, { &P13974,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT-S5670 Build/GRI40; CyanogenMod 7.2.0 Stable) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030173 = {S75592, 132, { &P24033,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ETAB 701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2189576 = {S75593, 157, { &P13123,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2827867 = {S75594, 157, { &P13123,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2223461 = {S75595, 138, { &P21535,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2551246 = {S75596, 139, { &P21535,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S6T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2645202 = {S75597, 129, { &P21535,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S6 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2355405 = {S75598, 146, { &P18300,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T769 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2508048 = {S75599, 104, { &P18254,&P18110,&P3409,&P18091 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.0.5) Gecko/X Gentoo Firefox/3.0.5 (Dot TK - spider 4.1)*/
const Device UA2532307 = {S75600, 165, { &P20183,&P18066,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; zh-cn; MI-ONE Plus Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2536888 = {S75601, 151, { &P18442,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2560135 = {S75602, 151, { &P18442,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2659254 = {S75603, 151, { &P18442,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; SCH-N719 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2858603 = {S75604, 151, { &P18442,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-N719 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2557583 = {S75605, 137, { &P22095,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ThL W7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2562548 = {S75606, 133, { &P22993,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TM-MID720 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3118842 = {S75607, 62, { &P22993,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; TM-MID720 Build/JRO03H)*/
const Device UA2652624 = {S75608, 178, { &P15785,&P5843,&P24671,&P18092 }};	/*Useragent - Nokia201/2.0 (11.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia201) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2676846 = {S75609, 130, { &P21012,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I815 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2973888 = {S75610, 152, { &P20749,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974072 = {S75611, 152, { &P22963,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3064899 = {S75612, 156, { &P22609,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6312 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3081084 = {S75613, 130, { &P20107,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5580C Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA8984 = {S75614, 103, { &P13368,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-SGH-T669B/T669BMJK1 SHP/VPP/R5 Dolfin/1.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA64526 = {S75615, 125, { &P12032,&P5843,&P4030,&P18092 }};	/*Useragent - Nokia6260s-1/2.0 (07.20) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA198855 = {S75616, 81, { &P17595,&P17295,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.8.131 Version/11.11*/
const Device UA2209421 = {S75617, 81, { &P17595,&P17295,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.8.119 Version/11.10*/
const Device UA644402 = {S75618, 171, { &P15364,&P17483,&P18373,&P18092 }};	/*Useragent - LG-350 MIDP-2.0/CLDC-1.1 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; U; pt-BZ) Presto/2.5.28 Version/10.10*/
const Device UA675701 = {S75619, 150, { &P23861,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; en-us; EBW404B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2236852 = {S75620, 123, { &P15364,&P18110,&P19813,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) CasperJS/1.0.0-RC2+Phantomjs/1.7.0 Safari/534.34*/
const Device UA2467223 = {S75621, 142, { &P23976,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.4; zh-tw; iTouch-HD88 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2579289 = {S75622, 142, { &P23976,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.4; en-us; iTouch-HD88 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654833 = {S75623, 142, { &P23976,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.4; en-gb; iTouch-HD88 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3086054 = {S75624, 142, { &P23976,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.4; zh-cn; iTouch-HD88 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2530757 = {S75625, 88, { &P17968,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; spice) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2558576 = {S75626, 90, { &P17968,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mozilla) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2567319 = {S75627, 139, { &P17691,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I957 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2584096 = {S75628, 132, { &P17691,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957R Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2988838 = {S75629, 132, { &P17691,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2661049 = {S75630, 162, { &P23830,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Q-Smart model S6 Build/S6_S406_YA860_V01.01B26) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2704450 = {S75631, 162, { &P23830,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Q-Smart model S6 Build/S6_S406_YA860_V01.01B30) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2863731 = {S75632, 162, { &P23830,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Q-Smart model S6 Build/S6_S406_YA860_V01.01B31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3105866 = {S75633, 162, { &P23830,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Q-Smart model S6 Build/S6_S406_YA860_V01.01B32) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3120304 = {S75634, 162, { &P23830,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Q-Smart model S6 Build/S6_S406_YA860_V01.01B28) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2676352 = {S75635, 133, { &P17172,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Transformer Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2707868 = {S75636, 139, { &P17172,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Transformer TF101 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2737563 = {S75637, 164, { &P13042,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC Incredible S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2733246 = {S75638, 120, { &P17595,&P21721,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2754488 = {S75639, 138, { &P18474,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9210T Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2796194 = {S75640, 156, { &P22872,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5282 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2947364 = {S75641, 159, { &P12798,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT18i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3079601 = {S75642, 159, { &P12798,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LT18i Build/4.1.A.0.562) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2960198 = {S75643, 133, { &P18645,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; SPH-L710 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2963051 = {S75644, 141, { &P18645,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3057009 = {S75645, 134, { &P18645,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-ca; SGH-I747M Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3121699 = {S75646, 133, { &P18645,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-I9305 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3125407 = {S75647, 141, { &P18645,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3080124 = {S75648, 136, { &P22805,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E455 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081347 = {S75649, 136, { &P24101,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ADR930L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083048 = {S75650, 139, { &P12798,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT18i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110549 = {S75651, 139, { &P12798,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LT18i Build/4.1.A.0.562) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA21627 = {S75652, 82, { &P12772,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-F100(Zimbabwe)/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24622 = {S75653, 70, { &P12772,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-F100/0.1 ACS-NF/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA32930 = {S75654, 87, { &P12772,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-F100(Jordan Orange)/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49402 = {S75655, 77, { &P12772,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-F100(DGT)/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64562 = {S75656, 72, { &P12772,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-F100/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA242088 = {S75657, 122, { &P15364,&P5690,&P22409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.4.0.0 Chrome/13.0.782.220 Safari/535.1*/
const Device UA654825 = {S75658, 100, { &P15228,&P5043,&P17338,&P18092 }};	/*Useragent - NokiaC2-03/2.0 (06.51) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-03/UC Browser7.9.0.102/69/352*/
const Device UA2227278 = {S75659, 117, { &P23545,&P17017,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-US; mauie800) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2383904 = {S75660, 196, { &P14682,&P5843,&P24672,&P18092 }};	/*Useragent - Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia6303iclassic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile*/
const Device UA2437422 = {S75661, 121, { &P15364,&P16110,&P19987,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 Sleipnir/3.8.4*/
const Device UA2505998 = {S75662, 87, { &P15364,&P16110,&P19987,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Sleipnir/3.8.4)*/
const Device UA2528995 = {S75663, 114, { &P15364,&P16110,&P19987,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 Sleipnir/3.8.4*/
const Device UA2457076 = {S75664, 66, { &P13018,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; MT11i; 480*854) UCWEB7.9.0.94/139/444*/
const Device UA2695861 = {S75665, 66, { &P13018,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; MT11i; 480*854) UCWEB7.9.0.94/139/444*/
const Device UA2426942 = {S75666, 139, { &P12280,&P18432,&P18355,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari/7534.48.3 Sleipnir/2.2m*/
const Device UA2428862 = {S75667, 139, { &P12280,&P18432,&P18355,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3 Sleipnir/2.2m*/
const Device UA2429357 = {S75668, 139, { &P12280,&P18432,&P18355,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 Sleipnir/2.2m*/
const Device UA2468912 = {S75669, 137, { &P12280,&P18432,&P18355,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Safari/7534.48.3 Sleipnir/2.2m*/
const Device UA2583141 = {S75670, 132, { &P20145,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; NS-13T001 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2676031 = {S75671, 135, { &P19784,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LIFETAB_P9514 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2737389 = {S75672, 131, { &P15767,&P18432,&P20782,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/10A523 Safari/7534.48.3*/
const Device UA2752690 = {S75673, 129, { &P15767,&P18432,&P20782,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/10A403 Safari/7534.48.3*/
const Device UA2768642 = {S75674, 131, { &P15767,&P18432,&P20782,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/10A550 Safari/7534.48.3*/
const Device UA2868656 = {S75675, 132, { &P15767,&P18432,&P20782,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/10A8426 Safari/7534.48.3*/
const Device UA2956740 = {S75676, 129, { &P15767,&P18432,&P20782,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/10A407 Safari/7534.48.3*/
const Device UA2754414 = {S75677, 139, { &P16319,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; U8860 Build/HuaweiU8860) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755156 = {S75678, 138, { &P16044,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cnm TOUCHPAD 9.7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2773867 = {S75679, 146, { &P16044,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Versus TouchPad 7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799779 = {S75680, 146, { &P16044,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Versus TouchPad 9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877917 = {S75681, 143, { &P16044,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CnM TouchPad 7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937528 = {S75682, 143, { &P16044,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CnM TouchPad 9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893823 = {S75683, 115, { &P15364,&P17285,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291*/
const Device UA3062313 = {S75684, 115, { &P15364,&P17285,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.319*/
const Device UA2961524 = {S75685, 135, { &P24068,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1;en_us; N986 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30;*/
const Device UA3078454 = {S75686, 57, { &P24068,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; N986 Build/JOP40D)*/
const Device UA3080329 = {S75687, 135, { &P21037,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LIFETAB_S9512 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3083938 = {S75688, 140, { &P23910,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT30at Build/7.0.B.1.152) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA78236 = {S75689, 121, { &P15501,&P5477,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.1-update1;Zh_cn;E16i;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/444*/
const Device UA379872 = {S75690, 72, { &P15501,&P5477,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.1-update1;Zh_cn;E16i;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA69096 = {S75691, 80, { &P12274,&P5448,&P4779,&P18092 }};	/*Useragent - SAMSUNG-SGH-i900/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA627173 = {S75692, 140, { &P18707,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; de-de; MD_LIFETAB_P9516 Build/HTK75C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1938141 = {S75693, 142, { &P19903,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-W999 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2412115 = {S75694, 135, { &P19903,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-W999 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2235095 = {S75695, 168, { &P17625,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2242377 = {S75696, 63, { &P12280,&P18432,&P22071,&P18092 }};	/*Useragent - Instagram 3.1.2 (iPhone; iPhone OS 6.0; en_US) AppleWebKit/420+*/
const Device UA2356683 = {S75697, 65, { &P12280,&P18432,&P22071,&P18092 }};	/*Useragent - Instagram 3.1.2 (iPhone; iPhone OS 6.0.1; en_US) AppleWebKit/420+*/
const Device UA2259235 = {S75698, 84, { &P15364,&P16110,&P22554,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:16.1) Gecko/X Firefox/16.1 IceDragon/16.1.0.1*/
const Device UA2437608 = {S75699, 155, { &P17595,&P18120,&P19966,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.4.3.X (Second Life Release; default skin) Safari/533.3*/
const Device UA2423668 = {S75700, 139, { &P20849,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Luna TAB374 Build/LunaTAB374) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2499588 = {S75701, 180, { &P17475,&P5768,&P24671,&P18092 }};	/*Useragent - Nokia6300/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia6300) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2505577 = {S75702, 144, { &P23812,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Magicon Mnote Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2791448 = {S75703, 130, { &P23812,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Magicon Mnote Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2511344 = {S75704, 95, { &P15364,&P18110,&P20916,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.21 (KHTML, like Gecko) rekonq/2.1 Safari/537.21*/
const Device UA2512581 = {S75705, 95, { &P15364,&P18110,&P20916,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) rekonq/2.1 Safari/534.34*/
const Device UA2570339 = {S75706, 142, { &P23850,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Q7A Build/ICS_IMM76D_B206) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2583209 = {S75707, 140, { &P18453,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT25i Build/9.0.D.0.484) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2672152 = {S75708, 141, { &P22632,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Titanium S1 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2707714 = {S75709, 144, { &P22627,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IRIS 455 Build/LAVAIRIS455) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2754168 = {S75710, 143, { &P19855,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT885 Build/6.7.3-135_YIC-8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774286 = {S75711, 156, { &P18713,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799255 = {S75712, 156, { &P18713,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P769 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2894726 = {S75713, 156, { &P18713,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P768 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2796329 = {S75714, 132, { &P18442,&P17979,&P18062,&P18092 }};	/*Useragent - samsung-GT-N7100/1.0 Linux/2.6.35.7 Android/4.0.4 Release/03.27.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2952758 = {S75715, 125, { &P18442,&P17979,&P18062,&P18092 }};	/*Useragent - N7100-N7100/1.0 Linux/2.6.35.7 Android/4.0 Release/01.21.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3045158 = {S75716, 125, { &P18442,&P17979,&P18062,&P18092 }};	/*Useragent - N7100-N7100/1.0 Linux/2.6.35.7 Android/4.0 Release/01.20.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2782276 = {S75717, 137, { &P23172,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; TAB750 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2885918 = {S75718, 140, { &P22598,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6506 Build/10.3.A.0.423) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2973657 = {S75719, 141, { &P19997,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo P700i Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022958 = {S75720, 140, { &P19997,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Lenovo P700 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3029683 = {S75721, 135, { &P15575,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-I9100 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3030446 = {S75722, 136, { &P15575,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-I9100G Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3080254 = {S75723, 140, { &P21402,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Lenovo S890 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080907 = {S75724, 133, { &P24129,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; A900 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA30351 = {S75725, 77, { &P12306,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-F107(3UK)/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA76707 = {S75726, 140, { &P18084,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; zh-tw; Transformer TF101G Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA78521 = {S75727, 140, { &P18084,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; en-us; Transformer TF101G Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA328984 = {S75728, 141, { &P18084,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; zh-tw; Transformer TF101G Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA397348 = {S75729, 140, { &P18084,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; it-it; Transformer TF101G Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA702499 = {S75730, 140, { &P18084,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; en-gb; Transformer TF101G Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA721815 = {S75731, 140, { &P18084,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; de-de; Transformer TF101G Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA737071 = {S75732, 140, { &P18084,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; en-ca; Transformer TF101G Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA1909351 = {S75733, 140, { &P18084,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; ru-ru; Transformer TF101G Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA2296400 = {S75734, 141, { &P18084,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; en-us; Transformer TF101G Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA170833 = {S75735, 140, { &P18034,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; SKY IM-A630K Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018509 = {S75736, 138, { &P18034,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-; SKY IM-A630K Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018928 = {S75737, 64, { &P18034,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; SKY IM-A630K Build/FRG83)*/
const Device UA902205 = {S75738, 171, { &P15575,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA905972 = {S75739, 171, { &P15575,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2023806 = {S75740, 172, { &P15575,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352*/
const Device UA2023944 = {S75741, 171, { &P15575,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2028149 = {S75742, 172, { &P15575,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2816308 = {S75743, 171, { &P15575,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2370805 = {S75744, 50, { &P15364,&P17188,&P17200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 4.0)*/
const Device UA2658835 = {S75745, 57, { &P15364,&P17188,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; WinNT4.0; Trident/4.0)*/
const Device UA3117730 = {S75746, 90, { &P15364,&P17188,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; AOL 9.7; AOLBuild 4343.19; Windows NT 4.0; Trident/5.0)*/
const Device UA2428067 = {S75747, 138, { &P12496,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-I897 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2582688 = {S75748, 137, { &P12496,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I897 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2506926 = {S75749, 142, { &P21411,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Q-Smart S20 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2676990 = {S75750, 144, { &P18348,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2736611 = {S75751, 150, { &P18348,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G X710s Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2704272 = {S75752, 138, { &P20864,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A13-MID Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2853379 = {S75753, 60, { &P20864,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; A13-MID Build/JRO03C)*/
const Device UA2784444 = {S75754, 155, { &P18241,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-E400 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752980 = {S75755, 137, { &P19865,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A850L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2756069 = {S75756, 137, { &P19865,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A850S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769432 = {S75757, 137, { &P19865,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A850K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784111 = {S75758, 139, { &P18593,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S6010 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057675 = {S75759, 140, { &P18593,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-S6010L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3083369 = {S75760, 152, { &P22961,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080586 = {S75761, 143, { &P18466,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3082938 = {S75762, 152, { &P22964,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094440 = {S75763, 159, { &P22964,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL_ONE_TOUCH_6010X_Orange Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3132851 = {S75764, 137, { &P19856,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LS970 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA54959 = {S75765, 93, { &P14438,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE50-2/3.0 (07.36.0.0) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62666 = {S75766, 93, { &P14438,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE50-1/3.0 (06.41.3.0) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20875 = {S75767, 105, { &P12159,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C5510/C5510DFII1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA58671 = {S75768, 121, { &P12159,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C5510H/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1/*MzU0NzUyMDMwMTEwNDg4*/
const Device UA663299 = {S75769, 48, { &P23144,&P18575,&P17470,&P18092 }};	/*Useragent - Facebook/4110.0 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA673497 = {S75770, 139, { &P14223,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-ve; UM840 Build/HuaweiUM840) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA397063 = {S75771, 78, { &P13042,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_Incredible_S; 480*800) UCWEB7.9.3.103/139/800*/
const Device UA568008 = {S75772, 78, { &P13042,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_Incredible_S; 480*800) UCWEB7.9.3.103/139/999*/
const Device UA570115 = {S75773, 78, { &P13042,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_Incredible_S; 480*800) UCWEB7.9.1.100/139/800*/
const Device UA627268 = {S75774, 78, { &P13042,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_Incredible_S; 480*800) UCWEB7.9.3.103/139/800*/
const Device UA1928449 = {S75775, 80, { &P13042,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_Incredible_S; 480*800) UCWEB7.9.3.103/139/32160*/
const Device UA1982463 = {S75776, 78, { &P13042,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_Incredible_S; 480*800) UCWEB7.9.4.145/139/800*/
const Device UA2155795 = {S75777, 78, { &P13042,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_Incredible_S; 480*800) UCWEB7.9.4.145/139/800*/
const Device UA2466292 = {S75778, 77, { &P13042,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; HTC_Incredible_S; 480*800) UCWEB7.9.0.94/139/352*/
const Device UA2483040 = {S75779, 77, { &P13042,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; HTC_Incredible_S; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2543897 = {S75780, 78, { &P13042,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; HTC_Incredible_S; 480*800) UCWEB7.9.4.145/139/800*/
const Device UA2978058 = {S75781, 77, { &P13042,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_Incredible_S; 480*800) UCWEB7.9.0.94/139/800*/
const Device UA2373811 = {S75782, 89, { &P13324,&P5055,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian8-00) UCBrowser8.3.0.154/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA2375696 = {S75783, 73, { &P16329,&P22223,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PLAYSTATION 3 4.22) AppleWebKit/531.22.8 (KHTML, like Gecko)*/
const Device UA2386238 = {S75784, 133, { &P16168,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; NookColor Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2427014 = {S75785, 138, { &P18282,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; MITO A200 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2526634 = {S75786, 133, { &P18282,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A200 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2670260 = {S75787, 136, { &P20416,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; F-04E Build/V14R31C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2558272 = {S75788, 92, { &P14753,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; Nokia N96) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2508844 = {S75789, 166, { &P16356,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-N7000 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2580238 = {S75790, 166, { &P16356,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-N7000 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2527305 = {S75791, 89, { &P17968,&P18111,&P19961,&P18092 }};	/*Useragent - Opera/12.02 (Android 4.1; Linux; Opera Mobi/ADR-X; U; en-US) Presto/2.9.201 Version/12.02*/
const Device UA2559758 = {S75792, 148, { &P21965,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MAXWEST_ORBIT6100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958144 = {S75793, 148, { &P21965,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; WESTMAX_ORBIT6100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566932 = {S75794, 161, { &P18233,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL_one_touch_995 Build/IceCreamSandwich) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2579208 = {S75795, 140, { &P12725,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE N880G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2647816 = {S75796, 140, { &P12725,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE N880F Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2581248 = {S75797, 93, { &P15845,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaC5-06) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2658190 = {S75798, 135, { &P22397,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; A880 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2670109 = {S75799, 153, { &P18113,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/2.3;FBBV/X;FBDM/{density=1.5,width=540,height=960};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/DROID RAZR;FBSV/4.1.2;]*/
const Device UA2737255 = {S75800, 153, { &P18113,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.1;FBBV/X;FBDM/{density=1.5,width=540,height=960};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/DROID RAZR;FBSV/4.1.2;]*/
const Device UA2771220 = {S75801, 155, { &P18113,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.2.1;FBBV/X;FBDM/{density=1.5,width=540,height=960};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/DROID RAZR;FBSV/4.1.2;]*/
const Device UA2896465 = {S75802, 153, { &P18113,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.3;FBBV/X;FBDM/{density=1.5,width=540,height=960};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/DROID RAZR;FBSV/4.1.2;]*/
const Device UA3018608 = {S75803, 153, { &P18113,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.4;FBBV/X;FBDM/{density=1.5,width=540,height=960};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/DROID RAZR;FBSV/4.1.2;]*/
const Device UA3028239 = {S75804, 153, { &P18113,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.5;FBBV/X;FBDM/{density=1.5,width=540,height=960};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/DROID RAZR;FBSV/4.1.2;]*/
const Device UA3093302 = {S75805, 180, { &P18113,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.6.1;FBBV/X;FBDM/{density=1.5,width=540,height=960};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/DROID RAZR;FBSV/4.1.2;FBCA/armeabi-v7a:armeabi;]*/
const Device UA2673244 = {S75806, 137, { &P19853,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7102 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2711693 = {S75807, 137, { &P19853,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7102 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2736683 = {S75808, 137, { &P19853,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; GT-N7102 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2784146 = {S75809, 142, { &P14225,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire HD Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867924 = {S75810, 138, { &P14225,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Desire HD Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2870898 = {S75811, 142, { &P14225,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire HD Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2872153 = {S75812, 150, { &P23327,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; iris 401e Build/LAVA-iris401e-S109) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2963686 = {S75813, 150, { &P23327,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; iris 401e Build/LAVA-iris401e-S111) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3059898 = {S75814, 150, { &P23327,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; iris 401e Build/LAVA-iris401e-S114) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2896345 = {S75815, 161, { &P18304,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9070 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2975713 = {S75816, 126, { &P19991,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A511 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2975555 = {S75817, 131, { &P22920,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; WL-101GQC Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2963506 = {S75818, 164, { &P13337,&P17485,&P24678,&P18092 }};	/*Useragent - SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj108i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3081113 = {S75819, 126, { &P18733,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M532 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3065483 = {S75820, 126, { &P16598,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA20331 = {S75821, 41, { &P15070,&P17484,&P3625,&P18092 }};	/*Useragent - sam-r420 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA82310 = {S75822, 50, { &P17014,&P7472,&P4901,&P18092 }};	/*Useragent - Alcatel-OH5/1.0 UP.Browser/6.1.0.7.7 (GUI) MMP/1.0*/
const Device UA704531 = {S75823, 129, { &P21665,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-es; Blu Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA712577 = {S75824, 139, { &P21665,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; BLU Build/1.0.6330.0359) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028730 = {S75825, 139, { &P21665,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; BLU Build/1.0.7264.0392) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA536594 = {S75826, 42, { &P17595,&P18120,&P4811,&P18092 }};	/*Useragent - Opera/9.0 (Macintosh; PPC Mac OS X; U; en)*/
const Device UA656517 = {S75827, 43, { &P17595,&P18120,&P4811,&P18092 }};	/*Useragent - Opera/9.00 (Macintosh; PPC Mac OS X; U; en)*/
const Device UA758412 = {S75828, 43, { &P17595,&P18120,&P4811,&P18092 }};	/*Useragent - Opera/9.01 (Macintosh; PPC Mac OS X; U; en)*/
const Device UA3040877 = {S75829, 43, { &P17595,&P18120,&P4811,&P18092 }};	/*Useragent - Opera/9.02 (Macintosh; PPC Mac OS X; U; en)*/
const Device UA789266 = {S75830, 138, { &P24030,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; OPPOX9015 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA904021 = {S75831, 64, { &P15364,&P17285,&P18514,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:13.0) Gecko/X Thunderbird/13.0.1*/
const Device UA1961472 = {S75832, 51, { &P14188,&P5890,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 (Nokia5233;SymbianOS/9.1 Series60/3.0)*/
const Device UA2203825 = {S75833, 129, { &P15767,&P18432,&P20777,&P18092 }};	/*Useragent - MQQBrowser/2.6 Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3*/
const Device UA2357208 = {S75834, 131, { &P15767,&P18432,&P20777,&P18092 }};	/*Useragent - MQQBrowser/2.6 Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3*/
const Device UA2223233 = {S75835, 140, { &P18226,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; U9500 Build/HuaweiU9500) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2431150 = {S75836, 229, { &P15575,&P5629,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2432257 = {S75837, 230, { &P15575,&P5629,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2435643 = {S75838, 230, { &P15575,&P5629,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2362382 = {S75839, 119, { &P11689,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian70-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2426393 = {S75840, 117, { &P11689,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian70) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2422729 = {S75841, 118, { &P12061,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE65) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile*/
const Device UA2504128 = {S75842, 92, { &P12061,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiae65-1) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2777835 = {S75843, 40, { &P12061,&P5890,&P24673,&P18092 }};	/*Useragent - NOKIAE65UCBrowser/8.7.0.218/28/352/UCWEB*/
const Device UA2601088 = {S75844, 134, { &P20427,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHL21 Build/S2140) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2536791 = {S75845, 91, { &P15791,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia603) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2582714 = {S75846, 157, { &P19902,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530M Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658070 = {S75847, 158, { &P19902,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-R530M Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3018583 = {S75848, 165, { &P19902,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530M Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/d2mtr*/
const Device UA2590151 = {S75849, 115, { &P15364,&P16110,&P17237,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/528.8 (KHTML, like Gecko) Chrome/2.0.156.1 Safari/528.8*/
const Device UA2894396 = {S75850, 155, { &P23664,&P18066,&P18062,&P18092 }};	/*Useragent - TSM-T8i Linux/3.0.13 Android/4.0.4 Release/05.29.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA3074185 = {S75851, 138, { &P23664,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TSM-T8i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892171 = {S75852, 146, { &P23756,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SM-G730A Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3109481 = {S75853, 114, { &P13107,&P5043,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-06/07.48; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA16960 = {S75854, 67, { &P18088,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC) Opera 8.65 [en]*/
const Device UA26646 = {S75855, 67, { &P18088,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC) Opera 8.65 [ru]*/
const Device UA27300 = {S75856, 70, { &P18088,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; (null)) Opera 8.65 [en]*/
const Device UA33636 = {S75857, 69, { &P18088,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone) Opera 8.65*/
const Device UA69633 = {S75858, 76, { &P18088,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320) Opera 8.65 [en]*/
const Device UA73736 = {S75859, 67, { &P18088,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC) Opera 8.65 [de]*/
const Device UA86987 = {S75860, 63, { &P18088,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; G500) Opera 8.65*/
const Device UA87685 = {S75861, 83, { &P18088,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 240x320) Opera 8.65 [en]*/
const Device UA87748 = {S75862, 76, { &P18088,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320) Opera 8.60 [en]*/
const Device UA284828 = {S75863, 112, { &P18088,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; 320x320; XV6900; Window Mobile 6.1 Professional;) Opera 8.65 [en]*/
const Device UA17235 = {S75864, 9, { &P15364,&P17017,&P17215,&P18092 }};	/*Useragent - Opera/9.5*/
const Device UA301252 = {S75865, 10, { &P15364,&P17017,&P17215,&P18092 }};	/*Useragent - Opera/9.50*/
const Device UA1957708 = {S75866, 161, { &P12731,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB855 Build/4.5.1A-1_SUN-198_6; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2818010 = {S75867, 152, { &P12731,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Photonic Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA727046 = {S75868, 155, { &P13685,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LS670 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033597 = {S75869, 158, { &P13685,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LS670 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2233968 = {S75870, 156, { &P13685,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LS670 Build/IMM76L; CyanogenMod-9.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1960730 = {S75871, 138, { &P12280,&P17145,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E200 Safari Version/5.1*/
const Device UA1993336 = {S75872, 138, { &P12280,&P17145,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E401 Safari Version/5.1*/
const Device UA2166640 = {S75873, 139, { &P12280,&P17145,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_10 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E600 Safari Version/5.1*/
const Device UA2393227 = {S75874, 138, { &P12280,&P17145,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari Version/5.1*/
const Device UA2589172 = {S75875, 139, { &P12280,&P17145,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148a Safari Version/5.1*/
const Device UA2933755 = {S75876, 144, { &P19968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Haier HW-W910 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3070198 = {S75877, 66, { &P19968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Haier HW-W910 Build/IMM76I)*/
const Device UA2162646 = {S75878, 87, { &P12581,&P5055,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokia500) UCBrowser8.3.0.154/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA2356421 = {S75879, 95, { &P15364,&P18110,&P20913,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) rekonq/1.3 Safari/534.34*/
const Device UA2456501 = {S75880, 140, { &P20258,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE N881E Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2652931 = {S75881, 131, { &P20258,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE N881E Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2843269 = {S75882, 62, { &P20258,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ZTE N881E Build/IMM76I)*/
const Device UA2466279 = {S75883, 136, { &P18296,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; One V Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2933920 = {S75884, 58, { &P18296,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; One V Build/JZO54K)*/
const Device UA3032711 = {S75885, 141, { &P18296,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC_One_VX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2505907 = {S75886, 140, { &P20738,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Andromax U Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2530770 = {S75887, 92, { &P15209,&P5768,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6233) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2858323 = {S75888, 156, { &P15209,&P5768,&P24676,&P18092 }};	/*Useragent - Nokia6233/2.0 (05.60) Profile/MIDP-2.0 Configuration/CLDC-1.1 , UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6233) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2675393 = {S75889, 143, { &P19855,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT885 Build/6.7.3-135_YIC-8) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2698877 = {S75890, 138, { &P22954,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E455f Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2782181 = {S75891, 143, { &P23027,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Spice Mi-450 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2992441 = {S75892, 136, { &P23027,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Mi450 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2942472 = {S75893, 139, { &P24732,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-S735C Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2945953 = {S75894, 188, { &P13157,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2975990 = {S75895, 139, { &P21507,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C1605 Build/11.1.A.0.68) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024868 = {S75896, 161, { &P18507,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I8160 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3033116 = {S75897, 93, { &P18673,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S7562) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3080325 = {S75898, 138, { &P18386,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I757M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA22455 = {S75899, 87, { &P12035,&P17485,&P4203,&P18092 }};	/*Useragent - SonyEricssonT280a/R1CB002 TelecaBrowser/Q04C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64542 = {S75900, 87, { &P12035,&P17485,&P4203,&P18092 }};	/*Useragent - SonyEricssonT280i/R1CB002 TelecaBrowser/Q04C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA5293 = {S75901, 161, { &P13427,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 1.0/SamsungSGHi560/I560DTHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA6458 = {S75902, 170, { &P13427,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Vodafone/1.0/SamsungSGHi560/I560BUGK1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA22328 = {S75903, 170, { &P13427,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Vodafone/1.0/SamsungSGHi560/I560BUHF1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA28385 = {S75904, 161, { &P13427,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 1.0/SamsungSGHi560/I560JDHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA28387 = {S75905, 161, { &P13427,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 1.0/SamsungSGHi560/I560DFHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA28388 = {S75906, 161, { &P13427,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 1.0/SamsungSGHi560/I560XXGL1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA28386 = {S75907, 161, { &P13427,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 1.0/SamsungSGHi560/I560XEGL2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA36947 = {S75908, 161, { &P13427,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 1.0/SamsungSGHi560/I560JVHB3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA45915 = {S75909, 161, { &P13427,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 1.0/SamsungSGHi560/I560XXHE1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA46509 = {S75910, 170, { &P13427,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Vodafone/1.0/SamsungSGHi560/I560BUGJ5 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA46508 = {S75911, 170, { &P13427,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Vodafone/1.0/SamsungSGHi560/I560BUHD3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA51028 = {S75912, 170, { &P13427,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Vodafone/1.0/SamsungSGHi560/I560BUHB1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA59815 = {S75913, 170, { &P13427,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Vodafone/1.0/SamsungSGHi560/I560JBHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA61825 = {S75914, 170, { &P13427,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Vodafone/1.0/SamsungSGHi560/I560AEHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA61824 = {S75915, 170, { &P13427,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Vodafone/1.0/SamsungSGHi560/I560JBHB1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA62583 = {S75916, 161, { &P13427,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 1.0/SamsungSGHi560/I560JCHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA62868 = {S75917, 161, { &P13427,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 1.0/SamsungSGHi560/I560DTHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA64823 = {S75918, 161, { &P13427,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 1.0/SamsungSGHi560/I560JAHD2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA69556 = {S75919, 155, { &P13427,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung-SGH-i560/BGHA1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA88550 = {S75920, 170, { &P13427,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Vodafone/1.0/SamsungSGHi560/I560BUHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA54614 = {S75921, 140, { &P12331,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; E130 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA503742 = {S75922, 62, { &P12331,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; E130 Build/ERE27)*/
const Device UA13183 = {S75923, 68, { &P13904,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-E2210BT/E2210BZDIG1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA13181 = {S75924, 67, { &P13904,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-E2210B/E2210BXAIJ1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA22445 = {S75925, 59, { &P13904,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-E2210L/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA43887 = {S75926, 58, { &P13904,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-E2210/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA64876 = {S75927, 76, { &P13904,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-E2210B-Bouygues/E2210BAGIF3 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA49448 = {S75928, 73, { &P13029,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-C3750/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA399143 = {S75929, 101, { &P15795,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 710)*/
const Device UA399159 = {S75930, 111, { &P15795,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 710; Vodafone)*/
const Device UA925728 = {S75931, 109, { &P15795,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 710; Orange)*/
const Device UA925736 = {S75932, 111, { &P15795,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 710; T-Mobile)*/
const Device UA925837 = {S75933, 109, { &P15795,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 710; Meteor)*/
const Device UA1959636 = {S75934, 129, { &P20579,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; AT570 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2496667 = {S75935, 58, { &P20579,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; AT570 Build/IML74K)*/
const Device UA1984440 = {S75936, 131, { &P23777,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CMP743d Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2558071 = {S75937, 131, { &P23777,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CMP743d Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029532 = {S75938, 130, { &P22032,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; ODYS-Q Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029974 = {S75939, 130, { &P21166,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE U880E Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2033183 = {S75940, 125, { &P21166,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE U880E Build/GWK74) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2033781 = {S75941, 84, { &P21166,&P5629,&P15760,&P18092 }};	/*Useragent - ZTEU880E_TD/1.0 Linux/2.6.35 Android/2.3 Release/12.15.2011 Browser/AppleWebKit533.1*/
const Device UA2396353 = {S75942, 130, { &P21166,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE U880E Build GWK74) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2319534 = {S75943, 50, { &P18254,&P7133,&P18245,&P18092 }};	/*Useragent - Mozilla/3.0 (compatible; WebMon 1.0.12; Windows 7)*/
const Device UA2388286 = {S75944, 103, { &P18254,&P7133,&P18245,&P18092 }};	/*Useragent - RssBandit/1.9.0.1002 (.NET CLR 2.0.50727.5466; WinNT 6.1.7601 Service Pack 1; http://www.rssbandit.org)*/
const Device UA2427684 = {S75945, 76, { &P18254,&P7133,&P18245,&P18092 }};	/*Useragent - SharpReader/0.9.7.0 (.NET CLR 2.0.50727.5466; WinNT 6.1.7601 Service Pack 1)*/
const Device UA2386187 = {S75946, 51, { &P17595,&P18120,&P20789,&P18092 }};	/*Useragent - MQQBrowser/38 (iOS 4S; U; CPU like Mac OS X; zh-cn)*/
const Device UA2389825 = {S75947, 50, { &P17595,&P18120,&P20789,&P18092 }};	/*Useragent - MQQBrowser/38 (iOS 4; U; CPU like Mac OS X; zh-cn)*/
const Device UA2426507 = {S75948, 50, { &P17595,&P18120,&P20789,&P18092 }};	/*Useragent - MQQBrowser/38 (iOS 5; U; CPU like Mac OS X; zh-cn)*/
const Device UA2431767 = {S75949, 166, { &P18283,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2430122 = {S75950, 126, { &P14202,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; X8 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2694812 = {S75951, 133, { &P14202,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; X8 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2664591 = {S75952, 143, { &P22689,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; Brava DM-993 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2754584 = {S75953, 145, { &P18114,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Transformer Prime TF201 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2877892 = {S75954, 150, { &P18114,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ASUS Transformer Prime TF201 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2769655 = {S75955, 137, { &P18456,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T699 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2779925 = {S75956, 134, { &P23187,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; V370 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2793522 = {S75957, 135, { &P24568,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; N9810 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3080374 = {S75958, 135, { &P21648,&P21637,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-I9505 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.30 Mobile Safari/537.36*/
const Device UA3131332 = {S75959, 141, { &P21648,&P21637,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-I9505G Build/JWR66Y.S003) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3083644 = {S75960, 137, { &P19861,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I605 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA17281 = {S75961, 69, { &P11912,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V360v/08.B7.AFR MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62890 = {S75962, 68, { &P11912,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V360/08.B7.2ER MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA50260 = {S75963, 72, { &P13208,&P5208,&P3499,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Teleca Q7; Brew 3.1.5; U; en) 240X400 LGE UX700*/
const Device UA90105 = {S75964, 31, { &P17744,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P07B(c500;TB;W24H16)*/
const Device UA397209 = {S75965, 70, { &P13974,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2.1;Zh_cn;GT-S5670;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA564170 = {S75966, 119, { &P13974,&P5096,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.2.1;Zh_cn;GT-S5670;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355*/
const Device UA578663 = {S75967, 114, { &P14669,&P5843,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-00/04.90; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA660076 = {S75968, 30, { &P14358,&P5890,&P24668,&P18092 }};	/*Useragent - NokiaE63/UCWEB8.3.0.133/28/999*/
const Device UA2478046 = {S75969, 79, { &P14358,&P5890,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokiae63-1) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA628537 = {S75970, 138, { &P21291,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; 009Z Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023176 = {S75971, 62, { &P21291,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; 009Z Build/GINGERBREAD)*/
const Device UA717518 = {S75972, 82, { &P15364,&P22308,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 3.51; en-GB; rv:1.9.0.1) Gecko/X Firefox/3.0.1*/
const Device UA1948318 = {S75973, 137, { &P22007,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; GPLUS GN105 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1949809 = {S75974, 138, { &P22007,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; GIONEE GN105 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023964 = {S75975, 121, { &P22007,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GN105 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2033911 = {S75976, 133, { &P22007,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GN105 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2394637 = {S75977, 126, { &P22007,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GN105 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2817941 = {S75978, 57, { &P22007,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GN105 Build/GRJ22)*/
const Device UA2028705 = {S75979, 135, { &P21301,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; M031 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2167125 = {S75980, 126, { &P21301,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; M031 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2719693 = {S75981, 245, { &P21301,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; M031 Build/IML74K) AppleWebKit/530.17 (KHTML, like Gecko) FlyFlow/2.3 Version/4.0 Mobile Safari/530.17 baidubrowser/023_1.41.3.2_diordna_069_046/uzieM_51_3.0.4_130M/Xe/1EXCEB87A57CEEE0B84F875E1A%7CX/1*/
const Device UA2724908 = {S75982, 126, { &P21301,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; M031 Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2376100 = {S75983, 94, { &P21705,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch)*/
const Device UA2580156 = {S75984, 93, { &P21705,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM;Touch)*/
const Device UA2422252 = {S75985, 147, { &P20000,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MZ607 Build/7.7.1-122-4-FLEM-WIFI-LA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2424884 = {S75986, 143, { &P20181,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0; Swipe_Fablet_F1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2425076 = {S75987, 64, { &P19779,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; A1+; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2435626 = {S75988, 216, { &P17337,&P6546,&P18525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Opera/9.80 (Android; Opera Mini/6.0.25270/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2623903 = {S75989, 214, { &P17337,&P6546,&P18525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Opera/9.80 (Android; Opera Mini/6.0.25270/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2460844 = {S75990, 139, { &P20923,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Mitab FLY Build/ICS.g08ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2531826 = {S75991, 93, { &P19976,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaE6-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2511985 = {S75992, 93, { &P18304,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I9070) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2580676 = {S75993, 101, { &P12581,&P5055,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia500) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3045763 = {S75994, 30, { &P12581,&P5055,&P24676,&P18092 }};	/*Useragent - Nokia500/UCWEB8.9.0.253/51/999*/
const Device UA2583396 = {S75995, 131, { &P18475,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2603864 = {S75996, 132, { &P18475,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320T Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2676087 = {S75997, 129, { &P15876,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5100 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735170 = {S75998, 130, { &P15876,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P5100 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2673052 = {S75999, 141, { &P16263,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SpringBoard Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2673562 = {S76000, 145, { &P16263,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI MediaPad Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708672 = {S76001, 145, { &P16263,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MediaPad 7 Lite Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2692855 = {S76002, 86, { &P18673,&P18110,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.0.4; zh-cn; GT-S7562 Build/IMM76I.S7562ZCALI1;480*800)*/
const Device UA2705206 = {S76003, 86, { &P18673,&P18110,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.0.4; zh-cn; GT-S7562 Build/IMM76I.S7562ZCAMB2;480*800)*/
const Device UA2729163 = {S76004, 88, { &P18673,&P18110,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.0.4; zh-cn; GT-S7562i Build/IMM76I.S7562iZCALL1;480*800)*/
const Device UA2950877 = {S76005, 88, { &P18673,&P18110,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.0.4; zh-cn; GT-S7562i Build/IMM76I.S7562iZCALI6;480*800)*/
const Device UA2951300 = {S76006, 86, { &P18673,&P18110,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.0.4; zh-cn; GT-S7562 Build/IMM76I.S7562ZCALK2;480*800)*/
const Device UA2957980 = {S76007, 86, { &P18673,&P18110,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.0.4; zh-cn; GT-S7562 Build/IMM76I.S7562ZCAME1;480*800)*/
const Device UA3086151 = {S76008, 86, { &P18673,&P18110,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.0.4; zh-cn; GT-S7562 Build/IMM76I.S7562ZCALL1;480*800)*/
const Device UA3092468 = {S76009, 86, { &P18673,&P18110,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.0.4; zh-cn; GT-S7562 Build/IMM76I.S7562ZCAMG1;480*800)*/
const Device UA2702532 = {S76010, 108, { &P23358,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; id301)*/
const Device UA2708080 = {S76011, 108, { &P23358,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; id300)*/
const Device UA3050006 = {S76012, 118, { &P23358,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; id300; Vodafone)*/
const Device UA2748453 = {S76013, 172, { &P23055,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; GIO-GiONEE_C800/CBD7602LT_0301_V0327; 540*960; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2779977 = {S76014, 172, { &P23055,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; GIO-GiONEE_C800/CBD7602LT_0301_V0335; 540*960; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2889731 = {S76015, 172, { &P23055,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GIO-GiONEE_C800/CBD7602GT_0301_V0324; 540*960; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957844 = {S76016, 172, { &P23055,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; GIO-GiONEE_C800/CBD7602LT_0301_V0358; 540*960; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3107952 = {S76017, 172, { &P23055,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; GIO-GiONEE_C800/CBD7602LT_0301_V0361; 540*960; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2750638 = {S76018, 153, { &P12848,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MB526 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3012470 = {S76019, 159, { &P12848,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MB526 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/mb526*/
const Device UA2771648 = {S76020, 162, { &P17324,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-I9001 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2782857 = {S76021, 162, { &P17324,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; GT-I9001 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453*/
const Device UA2867554 = {S76022, 162, { &P17324,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.3; GT-I9001 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2797130 = {S76023, 143, { &P23023,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Spice Mi-505 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2895065 = {S76024, 112, { &P15241,&P5843,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia300/07.03; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA3059885 = {S76025, 94, { &P14033,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC5-03) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3094694 = {S76026, 102, { &P13123,&P18110,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-S5360) U2/1.0.0 UCBrowser/9.3.0.321 Mobile*/
const Device UA3110591 = {S76027, 140, { &P17346,&P18432,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A523 Safari/8536.25*/
const Device UA3114497 = {S76028, 138, { &P17346,&P18432,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A403 Safari/8536.25*/
const Device UA3128079 = {S76029, 138, { &P17346,&P18432,&P24397,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/30.0.1599.12 Mobile/10A406 Safari/8536.25*/
const Device UA17084 = {S76030, 148, { &P14931,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; HTC Click-Orange Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA266555 = {S76031, 58, { &P15364,&P21445,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows ME; U; en) Presto/2.2.15 Version/10.10*/
const Device UA355191 = {S76032, 70, { &P15364,&P18110,&P22643,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) KHTML/4.7.4 (like Gecko) Konqueror/4.7*/
const Device UA359426 = {S76033, 68, { &P15364,&P18110,&P22643,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) KHTML/4.7.4 (like Gecko) Konqueror/4.7*/
const Device UA428932 = {S76034, 123, { &P15364,&P5690,&P22424,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11 Comodo_Dragon/17.5.2.0*/
const Device UA2330173 = {S76035, 130, { &P15364,&P5690,&P22424,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11 Comodo_Dragon/17.5.2.0*/
const Device UA1995951 = {S76036, 141, { &P22029,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Symphony_W25 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027927 = {S76037, 137, { &P20815,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Scroll Engage Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2261304 = {S76038, 142, { &P13035,&P5043,&P24670,&P18092 }};	/*Useragent - NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac2-02) UCBrowser8.4.0.159/70/444/UCWEB Mobile*/
const Device UA2386012 = {S76039, 32, { &P12059,&P5204,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia2690/10.10)*/
const Device UA2412881 = {S76040, 192, { &P15364,&P17285,&P18362,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; OfficeLiveConnector.1.3; OfficeLivePatch.1.3) Sleipnir/2.8.4*/
const Device UA2472333 = {S76041, 173, { &P15364,&P17285,&P18362,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/4.0; GTB6; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; Sleipnir/2.8.5)*/
const Device UA2496953 = {S76042, 151, { &P15364,&P17285,&P18362,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; FDM; Sleipnir/2.8.5)*/
const Device UA2461345 = {S76043, 131, { &P20985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MIDC410 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2656235 = {S76044, 131, { &P20985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; MIDC410 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2481222 = {S76045, 141, { &P21441,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOMO9 STAR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528914 = {S76046, 140, { &P21441,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOMO9star Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2505311 = {S76047, 143, { &P21614,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Philips W536 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558595 = {S76048, 71, { &P22617,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X325/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2566527 = {S76049, 136, { &P21037,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LIFETAB_S9512 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2696484 = {S76050, 44, { &P12080,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SAMSUNG-GT-S3653/S3653JPJD1)*/
const Device UA2580743 = {S76051, 144, { &P21885,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; GSmart GS202+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2670141 = {S76052, 152, { &P21003,&P20706,&P24676,&P18092 }};	/*Useragent - Nokia310/2.0 (07.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia310) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2702194 = {S76053, 93, { &P13554,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian79-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2708732 = {S76054, 92, { &P13554,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN79) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2756481 = {S76055, 146, { &P19863,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2975558 = {S76056, 130, { &P20107,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5580C Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974245 = {S76057, 126, { &P20269,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A211 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023145 = {S76058, 133, { &P16598,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; A500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023378 = {S76059, 126, { &P16598,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; A500 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3020866 = {S76060, 136, { &P20864,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021234 = {S76061, 136, { &P20864,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID-A13 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021721 = {S76062, 129, { &P20864,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA20486 = {S76063, 153, { &P13553,&P5267,&P3834,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8500R/S8500RUXJF3; U; Bada/1.0; en-ca) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA48973 = {S76064, 141, { &P14176,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; IS04 Build/TEK008) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA54296 = {S76065, 141, { &P14176,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; IS04 Build/FEK100) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA58118 = {S76066, 141, { &P14176,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; IS04 Build/FEK200) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2383528 = {S76067, 102, { &P17595,&P17297,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5; U; Edition MacAppStore; en) Presto/2.9.168 Version/11.51*/
const Device UA2145577 = {S76068, 85, { &P15364,&P7133,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; es-ES; rv:1.8.0.12) Gecko/X Firefox/1.5.0.12*/
const Device UA608903 = {S76069, 141, { &P21066,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Polaroid PMID701i Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2683977 = {S76070, 70, { &P21066,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; Polaroid PMID701i Build/IML74K)*/
const Device UA2990725 = {S76071, 133, { &P21066,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; PMID701DC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2161012 = {S76072, 107, { &P15364,&P17486,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; MOT-Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2427143 = {S76073, 91, { &P15209,&P5768,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6233) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2566964 = {S76074, 142, { &P15438,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Sensation Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567687 = {S76075, 136, { &P18470,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 101G10 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583964 = {S76076, 137, { &P19911,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E970 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2859037 = {S76077, 130, { &P19911,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E970 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2664694 = {S76078, 159, { &P22645,&P18066,&P18062,&P18092 }};	/*Useragent - G51 Wynncom Linux/3.0.13 Android/4.0.4 Release/01.23.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2704617 = {S76079, 77, { &P22620,&P22784,&P3680,&P18092 }};	/*Useragent - Micromax X272/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2719587 = {S76080, 142, { &P22655,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; sv-se; C5503 Build/10.3.A.0.317) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719597 = {S76081, 143, { &P22655,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; xh-us; C5503 Build/10.3.1.A.0.15) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719708 = {S76082, 142, { &P22655,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; sv-se; C5503 Build/10.3.A.0.356) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3030021 = {S76083, 66, { &P22655,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; C5503 Build/10.3.1.A.0.244)*/
const Device UA3107804 = {S76084, 144, { &P22655,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; sv-se; C5503 Build/10.3.1.A.0.244) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752349 = {S76085, 140, { &P22978,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; GT-S6313T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3101883 = {S76086, 62, { &P22978,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-S6313T Build/JZO54K)*/
const Device UA2774294 = {S76087, 171, { &P12605,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2969150 = {S76088, 92, { &P14453,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE51) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3059128 = {S76089, 152, { &P21003,&P20706,&P24679,&P18092 }};	/*Useragent - Nokia310/2.0 (07.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia310) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3081104 = {S76090, 136, { &P22097,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTabS2110AF Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3109379 = {S76091, 136, { &P22097,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTabS2110AH Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3065521 = {S76092, 142, { &P19843,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE Blade III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3083020 = {S76093, 142, { &P19843,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE BLADE III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083826 = {S76094, 138, { &P19843,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Blade III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127186 = {S76095, 141, { &P19843,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BLADE III_IL Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3095181 = {S76096, 156, { &P15575,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3095844 = {S76097, 156, { &P15575,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-I9100 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3114360 = {S76098, 157, { &P15575,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9100G Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA8107 = {S76099, 138, { &P11723,&P17486,&P3581,&P18092 }};	/*Useragent - MOT-ZN5/R6637_G_81.11.2CR_128 Mozilla/5.0 (compatible; MSIE 6.0; Linux; MOTOZINE ZN5) Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony/1.0*/
const Device UA63991 = {S76100, 132, { &P11723,&P17486,&P3581,&P18092 }};	/*Useragent - MOT-ZN5/R6637_G_81.03.05R Mozilla/4.0 (compatible; OSS/1.0; Linux MOTOZINE ZN5) Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony 1.0*/
const Device UA12334 = {S76101, 56, { &P12292,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_3G_T3232 Opera/9.50 (Windows NT 5.1; U; zh-tw)*/
const Device UA33048 = {S76102, 53, { &P12292,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_3G_T3232 Opera/9.50 (Windows NT 5.1; U; de)*/
const Device UA63936 = {S76103, 53, { &P12292,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_3G_T3232 Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA88178 = {S76104, 53, { &P12292,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_3G_T3232 Opera/9.50 (Windows NT 5.1; U; pt)*/
const Device UA789096 = {S76105, 53, { &P12292,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_3G_T3232 Opera/9.50 (Windows NT 5.1; U; nl)*/
const Device UA2242351 = {S76106, 53, { &P12292,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_3G_T3232 Opera/9.50 (Windows NT 5.1; U; ru)*/
const Device UA24355 = {S76107, 13, { &P14755,&P17483,&P17660,&P18092 }};	/*Useragent - LG/GD580/v1.0*/
const Device UA61619 = {S76108, 13, { &P14755,&P17483,&P17660,&P18092 }};	/*Useragent - LG-GD580/v1.0*/
const Device UA41301 = {S76109, 123, { &P14711,&P5204,&P17431,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-01/11.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.0.9.17.2*/
const Device UA49456 = {S76110, 149, { &P13032,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-us; Movistar Link Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA257474 = {S76111, 86, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.1)*/
const Device UA356019 = {S76112, 86, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.3)*/
const Device UA361800 = {S76113, 79, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.0.4)*/
const Device UA362702 = {S76114, 86, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.4)*/
const Device UA368282 = {S76115, 86, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.5)*/
const Device UA441024 = {S76116, 87, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.15)*/
const Device UA451128 = {S76117, 89, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.0rc3)*/
const Device UA586147 = {S76118, 79, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.0.5)*/
const Device UA588537 = {S76119, 86, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.7)*/
const Device UA588618 = {S76120, 86, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.6)*/
const Device UA590382 = {S76121, 79, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.0.9)*/
const Device UA591548 = {S76122, 86, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.9)*/
const Device UA603964 = {S76123, 80, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.0.16)*/
const Device UA604205 = {S76124, 87, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.16)*/
const Device UA609234 = {S76125, 80, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.0.17)*/
const Device UA609252 = {S76126, 87, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.17)*/
const Device UA650123 = {S76127, 86, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.8)*/
const Device UA787155 = {S76128, 87, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.12)*/
const Device UA813522 = {S76129, 80, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.0.15)*/
const Device UA820487 = {S76130, 87, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.11)*/
const Device UA823612 = {S76131, 87, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.14)*/
const Device UA834468 = {S76132, 80, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.0.14)*/
const Device UA850276 = {S76133, 87, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.13)*/
const Device UA1958447 = {S76134, 87, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.10)*/
const Device UA2305464 = {S76135, 86, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.0.2)*/
const Device UA2339848 = {S76136, 80, { &P15364,&P7133,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; Sleipnir/3.0.10)*/
const Device UA653118 = {S76137, 104, { &P12473,&P5204,&P17347,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2730c-1/UC Browser8.0.3.107/69/444*/
const Device UA441072 = {S76138, 125, { &P15364,&P7133,&P22426,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.140 Safari/535.19 Comodo_Dragon/18.0.3.0*/
const Device UA441254 = {S76139, 132, { &P15364,&P7133,&P22426,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.140 Safari/535.19 Comodo_Dragon/18.0.3.0*/
const Device UA643163 = {S76140, 114, { &P14635,&P5843,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-00/08.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA663768 = {S76141, 99, { &P17595,&P17297,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:7.0.1) Gecko/X Firefox/7.0.1 AlexaToolbar/alxf-2.15*/
const Device UA676247 = {S76142, 9, { &P18088,&P22725,&P17470,&P18092 }};	/*Useragent - MeeGo/6.0*/
const Device UA2455435 = {S76143, 133, { &P23466,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TM-MID880 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2508814 = {S76144, 104, { &P18436,&P16110,&P21915,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; chromeframe/24.0.1312.57)*/
const Device UA2567702 = {S76145, 130, { &P20750,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AN10DG3 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2676284 = {S76146, 138, { &P16044,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CnM Touchpad 9.7 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2711808 = {S76147, 146, { &P16044,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Versus TouchPad 7 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734612 = {S76148, 143, { &P16044,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CnM TouchPad 7 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735189 = {S76149, 143, { &P16044,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CnM TouchPad 9 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2768141 = {S76150, 146, { &P16044,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Versus TouchPad 9 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2755114 = {S76151, 137, { &P14959,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9103 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2756413 = {S76152, 137, { &P13141,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9210 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2797908 = {S76153, 138, { &P13141,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I727R Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877847 = {S76154, 138, { &P13141,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E110S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2997044 = {S76155, 138, { &P13141,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SHV-E110S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2757256 = {S76156, 69, { &P15364,&P7133,&P22768,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:22.0) Gecko/X Thunderbird/22.0*/
const Device UA2781195 = {S76157, 137, { &P24586,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 86VEBC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994061 = {S76158, 59, { &P24586,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; 86VEBC Build/IMM76D)*/
const Device UA3036368 = {S76159, 155, { &P18465,&P18066,&P24339,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA3081628 = {S76160, 132, { &P20518,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A10 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109623 = {S76161, 125, { &P20518,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A10 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA659687 = {S76162, 115, { &P14482,&P5843,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-01/07.15; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.11*/
const Device UA368822 = {S76163, 165, { &P18298,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-XT928/6.5.1_GC-159-DNRCG-8; 720*1280; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370774 = {S76164, 148, { &P18298,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT928 Build/6.5.1_GC-159-DNRCG-8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA787166 = {S76165, 166, { &P18298,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-XT928/6.5.1_GC-179-DNRCG-12; 720*1280; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA805703 = {S76166, 165, { &P18298,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-XT928/6.5.1_GC-179-DNRCG-9; 720*1280; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1939174 = {S76167, 166, { &P18298,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-XT928/6.5.1_GC-122-DNRCG-15; 720*1280; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1946331 = {S76168, 136, { &P18298,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT928 Build/6.5.1_GC-159-DNRCG-8) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2026317 = {S76169, 149, { &P18298,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT928 Build/6.5.1_GC-179-DNRCG-12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031860 = {S76170, 137, { &P18298,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT928 Build/6.5.1_GC-179-DNRCG-12) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA369119 = {S76171, 135, { &P21116,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM010SH Build/S0012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA624837 = {S76172, 135, { &P21116,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM010SH Build/S0010) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA635417 = {S76173, 135, { &P21116,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM010SH Build/S0014) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA895486 = {S76174, 135, { &P21116,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM010SH Build/S0016) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2669394 = {S76175, 135, { &P21116,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM010SH Build/S0018) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2894708 = {S76176, 135, { &P21116,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; DM010SH Build/S0020) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA855685 = {S76177, 80, { &P17595,&P18584,&P17470,&P18092 }};	/*Useragent - WebProcess/7534.53.11 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookPro8%2C1)*/
const Device UA1942555 = {S76178, 87, { &P17595,&P18584,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader/1725 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookPro8%2C2)*/
const Device UA1967666 = {S76179, 80, { &P17595,&P18584,&P17470,&P18092 }};	/*Useragent - WebProcess/7534.53.11 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookAir4%2C2)*/
const Device UA1967967 = {S76180, 85, { &P17595,&P18584,&P17470,&P18092 }};	/*Useragent - ThumbnailService/39400.005 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookPro8%2C2)*/
const Device UA2312864 = {S76181, 82, { &P17595,&P18584,&P17470,&P18092 }};	/*Useragent - ocspd (unknown version) CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookPro8%2C1)*/
const Device UA2318710 = {S76182, 75, { &P17595,&P18584,&P17470,&P18092 }};	/*Useragent - WebProcess/7534.53.11 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (iMac12%2C1)*/
const Device UA2500234 = {S76183, 89, { &P17595,&P18584,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookPro8%2C1)*/
const Device UA768099 = {S76184, 133, { &P14025,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; C8600 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2425804 = {S76185, 139, { &P14025,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-ve; C8600 Build/HuaweiC8600) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA895379 = {S76186, 114, { &P15767,&P17146,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.1*/
const Device UA903354 = {S76187, 114, { &P15767,&P17146,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.2*/
const Device UA910864 = {S76188, 114, { &P15767,&P17146,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.4*/
const Device UA1996357 = {S76189, 114, { &P15767,&P17146,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.5*/
const Device UA1999890 = {S76190, 114, { &P15767,&P17146,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.6*/
const Device UA2202971 = {S76191, 114, { &P15767,&P17146,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.7*/
const Device UA2362791 = {S76192, 114, { &P15767,&P17146,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.8*/
const Device UA2554002 = {S76193, 112, { &P15767,&P17146,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Flipboard/1.9.8*/
const Device UA1956660 = {S76194, 135, { &P18717,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PMP5097CPRO Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2507525 = {S76195, 138, { &P20346,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MIDC407 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3014789 = {S76196, 138, { &P20346,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MIDC407 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2652936 = {S76197, 130, { &P20754,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; AN7CG3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2663153 = {S76198, 145, { &P22642,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; GIONEE_Ctrl_V3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666346 = {S76199, 108, { &P19799,&P18110,&P24409,&P18092 }};	/*Useragent - Mozilla/5.0 (DTV) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.1.1 AQUOSBrowser/1.1 (US01DTV;V;0001;0001)*/
const Device UA2667099 = {S76200, 29, { &P18254,&P17017,&P22569,&P18092 }};	/*Useragent - Akregator/4.10.1; syndication*/
const Device UA2705807 = {S76201, 29, { &P18254,&P17017,&P22569,&P18092 }};	/*Useragent - Akregator/4.10.2; syndication*/
const Device UA2782048 = {S76202, 29, { &P18254,&P17017,&P22569,&P18092 }};	/*Useragent - Akregator/4.10.3; syndication*/
const Device UA2874772 = {S76203, 29, { &P18254,&P17017,&P22569,&P18092 }};	/*Useragent - Akregator/4.10.4; syndication*/
const Device UA3020622 = {S76204, 29, { &P18254,&P17017,&P22569,&P18092 }};	/*Useragent - Akregator/4.10.5; syndication*/
const Device UA2677159 = {S76205, 139, { &P20124,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Grand X(M) Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708204 = {S76206, 140, { &P20124,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ZTE Grand X Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709131 = {S76207, 138, { &P20124,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970M Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713676 = {S76208, 137, { &P20124,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V970 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714942 = {S76209, 137, { &P15364,&P22887,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu/13.04 Chromium/27.0.1453.6 Chrome/27.0.1453.6 Safari/537.36*/
const Device UA2751515 = {S76210, 135, { &P15364,&P22887,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu/13.04 Chromium/27.0.1453.6 Chrome/27.0.1453.6 Safari/537.36*/
const Device UA2798846 = {S76211, 160, { &P11747,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire V Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3002884 = {S76212, 161, { &P11747,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire VC Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2769952 = {S76213, 138, { &P14854,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783118 = {S76214, 138, { &P14854,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SHW-M250S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783472 = {S76215, 138, { &P14854,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2866909 = {S76216, 138, { &P14854,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2780038 = {S76217, 142, { &P19792,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire U Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2876096 = {S76218, 151, { &P19792,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire U dual sim Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2879326 = {S76219, 140, { &P20926,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; AT-AS40SE Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879571 = {S76220, 148, { &P23494,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Connect-2G-2.0 Build/HCL ME Tablet V2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3063837 = {S76221, 162, { &P19862,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Micromax A110Q Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3082198 = {S76222, 168, { &P12280,&P17090,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5 UCBrowser/9.3.0.321*/
const Device UA8579 = {S76223, 107, { &P17330,&P5448,&P4808,&P18092 }};	/*Useragent - HTC-ST7377/1.59.502.3 (67150) Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA10062 = {S76224, 107, { &P17330,&P5448,&P4808,&P18092 }};	/*Useragent - HTC-ST7377/2.10.502.4 (93577) Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA24373 = {S76225, 70, { &P13671,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-M3510/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61144 = {S76226, 71, { &P13671,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-M3510L/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA379110 = {S76227, 61, { &P20065,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX210/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA2242582 = {S76228, 62, { &P20065,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX210+/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA675239 = {S76229, 31, { &P11929,&P5129,&P24670,&P18092 }};	/*Useragent - Nokia5230/UCWEB8.4.0.150/50/999*/
const Device UA896236 = {S76230, 80, { &P11929,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-in; Nokia5230)/UC Browser8.4.0.159/50/352/UCWEB Mobile*/
const Device UA676544 = {S76231, 71, { &P15364,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux; U;2.2.2;zh_cn;Coolpad W706+;320*480;)UCWEB7.6.0.75/139/32411*/
const Device UA718528 = {S76232, 68, { &P15364,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux; U;2.3.4;zh_cn;Lenovo P70;320*480;)UCWEB7.6.0.75/139/32101*/
const Device UA798348 = {S76233, 70, { &P15364,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux; U;2.2.2;zh_cn;Coolpad W706;320*480;)UCWEB7.6.0.75/139/32411*/
const Device UA1893404 = {S76234, 57, { &P15364,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux;U;2.1;Zh_cn;E239;320*480;)UCWEB7.6.0.75/139/800*/
const Device UA2408930 = {S76235, 57, { &P15364,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux;U;2.1;Zh_cn;D539;320*480;)UCWEB7.6.0.75/139/800*/
const Device UA2742550 = {S76236, 62, { &P15364,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux; U;2.2.2;zh_cn;5855;320*480;)UCWEB7.6.0.75/139/32521*/
const Device UA2033485 = {S76237, 157, { &P15759,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Vision Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258030 = {S76238, 157, { &P15759,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Vision Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2220365 = {S76239, 145, { &P18661,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N860 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2870354 = {S76240, 146, { &P18661,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N860 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2583196 = {S76241, 153, { &P18706,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_9p017 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667618 = {S76242, 154, { &P18706,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_ov5647 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2706289 = {S76243, 150, { &P18706,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_9p017 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2733278 = {S76244, 153, { &P18706,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_253_6628 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2779952 = {S76245, 152, { &P18706,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_5640 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2794201 = {S76246, 148, { &P18706,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_253 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2657624 = {S76247, 140, { &P23559,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; AMOI N821 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099995 = {S76248, 62, { &P23559,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; AMOI N821 Build/JRO03C)*/
const Device UA2672425 = {S76249, 145, { &P19855,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT885 Build/9.8.3O-148_IRD-3) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2704585 = {S76250, 139, { &P20240,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Optimus G Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2754793 = {S76251, 138, { &P22592,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717R Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2994767 = {S76252, 131, { &P22228,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SM-T210 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3080380 = {S76253, 152, { &P23672,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083259 = {S76254, 137, { &P18469,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabS2109A-F Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA34481 = {S76255, 135, { &P11878,&P17486,&P20449,&P18092 }};	/*Useragent - MOT-Z6/R60_G_80.xx.yyI Mozilla/4.0 (compatible; MSIE 6.0 Linux; MOTOROKR Z6;nnn) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.50[yy]*/
const Device UA35449 = {S76256, 85, { &P11878,&P17486,&P20449,&P18092 }};	/*Useragent - MOTOROKR Z6/R60_G_80.33.03R Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.50[es-LA]*/
const Device UA62146 = {S76257, 140, { &P11878,&P17486,&P20449,&P18092 }};	/*Useragent - MOTOROKR Z6/R60_G_80.xx.yyI Mozilla/4.0 (compatible; MSIE 6.0 Linux; MOTOROKR Z6;nnn) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.50[yy]*/
const Device UA68167 = {S76258, 140, { &P11878,&P17486,&P20449,&P18092 }};	/*Useragent - MOT-MOTOROKRZ6_CMCC/1.0 LinuxOS/2.6.10 Release/04.30.2007 Browser/Opera8.50 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R60_G_80.31.11R*/
const Device UA64837 = {S76259, 141, { &P12323,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Pulse Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA557630 = {S76260, 133, { &P12323,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-in; Pulse Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018140 = {S76261, 63, { &P12323,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Pulse Build/ERE27)*/
const Device UA55202 = {S76262, 98, { &P12099,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-J800/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA44119 = {S76263, 40, { &P13786,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP570 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA48467 = {S76264, 130, { &P12725,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; N880 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61467 = {S76265, 132, { &P12725,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; N880 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61468 = {S76266, 136, { &P12725,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; BLADE_N880 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1899880 = {S76267, 158, { &P12725,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en_us; N880 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.2.0 Mobile (Dragonfruit)*/
const Device UA2024011 = {S76268, 156, { &P12725,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; ZTE-C_N880/N880V1.0.0B05; 480*800; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA87551 = {S76269, 4, { &P18088,&P18588,&P18246,&P18092 }};	/*Useragent - MAUI*/
const Device UA1970266 = {S76270, 154, { &P14123,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; LG-P970 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA877203 = {S76271, 61, { &P15364,&P6729,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win95; rv:1.8.1) Gecko/X Firefox/2.0*/
const Device UA2210575 = {S76272, 64, { &P15364,&P6729,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Win95; en-US; rv:1.8.1) Gecko/X BonEcho/2.0*/
const Device UA888336 = {S76273, 117, { &P15364,&P17285,&P17387,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618)*/
const Device UA1969842 = {S76274, 142, { &P15881,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Incredible 2 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2395141 = {S76275, 146, { &P15881,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Incredible 2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2842604 = {S76276, 152, { &P15881,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Droid Incredible 2 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2026930 = {S76277, 149, { &P21322,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; AMOI_N808 Build/ALPS.GB2.MP.V1.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032844 = {S76278, 150, { &P21322,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; GPLUS-N808 Build/ALPS.GB2.MP.V1.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2520475 = {S76279, 138, { &P21322,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; AMOI N808 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2743760 = {S76280, 131, { &P21322,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; AMOI N808 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2435429 = {S76281, 104, { &P18436,&P16110,&P21914,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; chromeframe/23.0.1271.97)*/
const Device UA2457584 = {S76282, 67, { &P12968,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; LT26i; 720*1280) UCWEB7.9.0.94/139/444*/
const Device UA2504642 = {S76283, 67, { &P12968,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.7; zh-cn; LT26i; 720*1280) UCWEB7.9.0.94/139/444*/
const Device UA2461330 = {S76284, 168, { &P17625,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2475375 = {S76285, 104, { &P15364,&P16110,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.3 (KHTML, like Gecko) Chrome/1, 6, 0, 4 Safari/535.3*/
const Device UA2545329 = {S76286, 118, { &P15364,&P16110,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/1.0.154.36 Safari/525.19*/
const Device UA2567572 = {S76287, 151, { &P20094,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825-1 Build/HuaweiU8825-1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2859648 = {S76288, 149, { &P20094,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583081 = {S76289, 141, { &P18465,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Amazon Kindle Fire Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2988398 = {S76290, 141, { &P18465,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Amazon Kindle Fire Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2579968 = {S76291, 174, { &P23794,&P5096,&P3508,&P18092 }};	/*Useragent - HUAWEI Y310-5000_TD/1.0 Linux/2.6.35 Android/2.3.5 Release/7.16.2012 Mozilla/5.0 (Linux; U; Android 2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2581318 = {S76292, 161, { &P20529,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A89 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3058015 = {S76293, 165, { &P20529,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A89 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2584867 = {S76294, 139, { &P14854,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2601596 = {S76295, 139, { &P14854,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SHW-M250S Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667242 = {S76296, 139, { &P14854,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250K Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2953585 = {S76297, 139, { &P14854,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250L Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2676231 = {S76298, 135, { &P14574,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus S Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735641 = {S76299, 136, { &P14574,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus S Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708543 = {S76300, 155, { &P15649,&P5096,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5570 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2717886 = {S76301, 98, { &P19862,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax A110) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2737926 = {S76302, 161, { &P14877,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2796575 = {S76303, 161, { &P14877,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2993999 = {S76304, 161, { &P14877,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-CN; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2739352 = {S76305, 155, { &P18465,&P18066,&P22587,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2753851 = {S76306, 130, { &P15919,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P3100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2867198 = {S76307, 137, { &P15919,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P3100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773116 = {S76308, 139, { &P20289,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC6435LVW Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868414 = {S76309, 137, { &P20289,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; HTC6435LVW Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2975471 = {S76310, 146, { &P20283,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AZ210A Build/AZ210A_ICS_02.34I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022189 = {S76311, 146, { &P20283,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AZ210B Build/AZ210B_ICS_02.34I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA56366 = {S76312, 92, { &P13952,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-806/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A SVN/04E*/
const Device UA57047 = {S76313, 85, { &P13952,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-806A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA62512 = {S76314, 85, { &P13952,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-806D/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA63705 = {S76315, 84, { &P13952,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-806/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA87079 = {S76316, 121, { &P13394,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-SGH-J700V/BUHA3/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA88894 = {S76317, 121, { &P13394,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-SGH-J700V/JBHC1/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA98987 = {S76318, 121, { &P13394,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-SGH-J700V/BUHG1/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA261802 = {S76319, 121, { &P13394,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-SGH-J700V/BUHB2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA639541 = {S76320, 121, { &P13394,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-SGH-J700V/BUHB1/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA860642 = {S76321, 121, { &P13394,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-SGH-J700V/AMHB2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA2127844 = {S76322, 121, { &P13394,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-SGH-J700V/AEHE1/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA396276 = {S76323, 79, { &P15364,&P17285,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 6.0; Microsoft Office Outlook 12.0.6562; Pro)*/
const Device UA437173 = {S76324, 79, { &P15364,&P17285,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 6.0; Microsoft Office Outlook 12.0.6607; Pro)*/
const Device UA2013889 = {S76325, 79, { &P15364,&P17285,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 6.0; Microsoft Office Outlook 12.0.6662; Pro)*/
const Device UA2131586 = {S76326, 79, { &P15364,&P17285,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 6.0; Microsoft Office Outlook 12.0.4518; Pro)*/
const Device UA2280779 = {S76327, 79, { &P15364,&P17285,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 6.0; Microsoft Office Outlook 12.0.6554; Pro)*/
const Device UA3083638 = {S76328, 79, { &P15364,&P17285,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 6.0; Microsoft Office Outlook 12.0.6683; Pro)*/
const Device UA2028670 = {S76329, 158, { &P15575,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029166 = {S76330, 157, { &P15575,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029760 = {S76331, 158, { &P15575,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029774 = {S76332, 157, { &P15575,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2260236 = {S76333, 157, { &P15575,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2446282 = {S76334, 86, { &P15575,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Adr (Linux; U; 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD.ZCLC2;480*800)*/
const Device UA2590467 = {S76335, 85, { &P15575,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Adr (Linux; U; 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD.ZSKI3;480*800)*/
const Device UA2817311 = {S76336, 158, { &P15575,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2817864 = {S76337, 157, { &P15575,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9108 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2821493 = {S76338, 86, { &P15575,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Adr (Linux; U; 2.3.5; zh-cn; GT-I9100G Build/GINGERBREAD.ZCKI1;480*800)*/
const Device UA2029104 = {S76339, 138, { &P20378,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE N855D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2686497 = {S76340, 131, { &P20378,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE N855D Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2164101 = {S76341, 71, { &P20028,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X445/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2218792 = {S76342, 135, { &P20175,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MI 1S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2480774 = {S76343, 136, { &P20175,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MI 1SC Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2257787 = {S76344, 135, { &P22165,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I-Buddy 7.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2456679 = {S76345, 17, { &P23024,&P22843,&P17470,&P18092 }};	/*Useragent - KKT38_WAP_Browser*/
const Device UA2461164 = {S76346, 137, { &P22115,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TOUCH90 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2530663 = {S76347, 90, { &P17625,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; ST25i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2986768 = {S76348, 90, { &P17625,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; ST25a) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2557828 = {S76349, 151, { &P21261,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2669253 = {S76350, 53, { &P18088,&P19826,&P24670,&P18092 }};	/*Useragent - IUC(U;iOS 6.1.3;Zh-cn;320*480;)/UCWEB8.4.1.169/42/997*/
const Device UA2601744 = {S76351, 136, { &P20119,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_S9714 Build/JRO03R) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2622668 = {S76352, 193, { &P17337,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.14/271,1.9.14.271,2012-12-20 13:41, -0500, ca*/
const Device UA2624270 = {S76353, 192, { &P17337,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0200, za*/
const Device UA2625472 = {S76354, 193, { &P17337,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, -0400, ca*/
const Device UA2627417 = {S76355, 193, { &P17337,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, -0400, ca*/
const Device UA2771216 = {S76356, 137, { &P23006,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; AIRIS TM350 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3073975 = {S76357, 134, { &P23006,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; TM350 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2799104 = {S76358, 129, { &P24111,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T8700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2871489 = {S76359, 145, { &P23223,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; PMP5588C Build/E8HD-ELITE-1280x768) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2975562 = {S76360, 152, { &P21968,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3083479 = {S76361, 133, { &P20976,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; W100 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080763 = {S76362, 134, { &P15767,&P18432,&P24411,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/10A523 Safari/7534.48.3*/
const Device UA3080767 = {S76363, 135, { &P15767,&P18432,&P24411,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/10A8426 Safari/7534.48.3*/
const Device UA3080771 = {S76364, 132, { &P15767,&P18432,&P24411,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/10A403 Safari/7534.48.3*/
const Device UA3115748 = {S76365, 134, { &P15767,&P18432,&P24411,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/10A550 Safari/7534.48.3*/
const Device UA3116567 = {S76366, 132, { &P15767,&P18432,&P24411,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/10A407 Safari/7534.48.3*/
const Device UA3116569 = {S76367, 132, { &P15767,&P18432,&P24411,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/10A406 Safari/7534.48.3*/
const Device UA3116573 = {S76368, 135, { &P15767,&P18432,&P24411,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.0.2.62956 Mobile/10A8500 Safari/7534.48.3*/
const Device UA29907 = {S76369, 71, { &P12280,&P17084,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8B117 (iPhone; U; CPU OS 4_1 like Mac OS X; en_us)*/
const Device UA38559 = {S76370, 51, { &P13690,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-V860/00.72 UP.Browser/7.2.7.5.630 (GUI) MMP/2.0*/
const Device UA38560 = {S76371, 51, { &P13690,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-V860/00.72 UP.Browser/7.2.7.5.665 (GUI) MMP/2.0*/
const Device UA138910 = {S76372, 143, { &P15364,&P17658,&P4808,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; MSIE 6.0; DoCoMo; Nokia; SymbianOS; NetFront; XENIUM; PPC; ScanAlert; +http://www.mcafeesecure.com/bot.jsp )*/
const Device UA169960 = {S76373, 71, { &P15364,&P18110,&P17828,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/4.5; Linux) KHTML/4.5.5 (like Gecko)*/
const Device UA2384059 = {S76374, 102, { &P17595,&P17297,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5; U; Edition MacAppStore; en) Presto/2.9.168 Version/11.52*/
const Device UA260232 = {S76375, 62, { &P15364,&P7127,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.5.24 Version/10.53*/
const Device UA382264 = {S76376, 62, { &P15364,&P7127,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.5.22 Version/10.51*/
const Device UA2262432 = {S76377, 62, { &P15364,&P7127,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.5.22 Version/10.50*/
const Device UA2263534 = {S76378, 62, { &P15364,&P7127,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.5.24 Version/10.54*/
const Device UA558118 = {S76379, 82, { &P15364,&P18118,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.10) Gecko/X Ubuntu Firefox/3.6.10*/
const Device UA576546 = {S76380, 137, { &P14017,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Optimus 2X Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021890 = {S76381, 61, { &P14017,&P24694,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; Optimus 2X Build/MIUI)*/
const Device UA2024039 = {S76382, 140, { &P14017,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG Optimus 2X Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031684 = {S76383, 173, { &P14017,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB860 Build/MIUI-2.2.17.0-OLYMPUS-EN; LG-P990 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA888642 = {S76384, 74, { &P15364,&P17285,&P22535,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:10.0.3) Gecko/X Firefox/10.0.3 Epic/10.0.3*/
const Device UA925860 = {S76385, 101, { &P18218,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 610)*/
const Device UA2685276 = {S76386, 111, { &P18218,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 610; Vodafone)*/
const Device UA1970008 = {S76387, 132, { &P20466,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; B79 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1973063 = {S76388, 137, { &P21999,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; RPSPG3201 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030530 = {S76389, 137, { &P21272,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; AQUA 4.0 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031902 = {S76390, 134, { &P20140,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GEM10312BK Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2385095 = {S76391, 134, { &P20140,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GEM10312BK Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2457504 = {S76392, 178, { &P15785,&P5843,&P24672,&P18092 }};	/*Useragent - Nokia201/2.0 (11.64) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia201) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2455144 = {S76393, 133, { &P20475,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ENERGY i8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2552527 = {S76394, 141, { &P20475,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Viettel I8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2456028 = {S76395, 155, { &P14186,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-S5830 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2550399 = {S76396, 155, { &P14186,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-S5830 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2531370 = {S76397, 167, { &P12968,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT26i Build/6.1.A.2.55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2537244 = {S76398, 132, { &P23770,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ID720WTA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2538286 = {S76399, 168, { &P18302,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MT25i Build/4.1.B.0.631) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3058314 = {S76400, 172, { &P18302,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT25i Build/4.1.B.0.631) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2666226 = {S76401, 129, { &P15767,&P18432,&P20781,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/10A403 Safari/7534.48.3*/
const Device UA2667604 = {S76402, 131, { &P15767,&P18432,&P20781,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/10A523 Safari/7534.48.3*/
const Device UA2708227 = {S76403, 131, { &P15767,&P18432,&P20781,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/10A550 Safari/7534.48.3*/
const Device UA2858783 = {S76404, 129, { &P15767,&P18432,&P20781,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/10A407 Safari/7534.48.3*/
const Device UA2990674 = {S76405, 132, { &P15767,&P18432,&P20781,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/10A8426 Safari/7534.48.3*/
const Device UA2708668 = {S76406, 142, { &P14225,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire HD Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712828 = {S76407, 138, { &P14225,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Desire HD Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735515 = {S76408, 142, { &P14225,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire HD Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2772875 = {S76409, 93, { &P20146,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9082) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2736374 = {S76410, 144, { &P23420,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Symphony_W125 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2901105 = {S76411, 161, { &P19862,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax A110 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3033147 = {S76412, 188, { &P15794,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2974122 = {S76413, 140, { &P20452,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Lenovo S880 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020924 = {S76414, 138, { &P18072,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021326 = {S76415, 138, { &P18072,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022323 = {S76416, 138, { &P18072,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975032 = {S76417, 129, { &P20750,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AN10DG3 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3034129 = {S76418, 153, { &P20175,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI 1S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3108976 = {S76419, 154, { &P20175,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI 1SC Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3081388 = {S76420, 142, { &P22749,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-04E Build/10.1.1.D.2.31) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3131645 = {S76421, 137, { &P18509,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3921 = {S76422, 133, { &P17714,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Stream Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019124 = {S76423, 57, { &P17714,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; Stream Build/MASTER)*/
const Device UA5797 = {S76424, 122, { &P12292,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC_Touch_3G_T3232-Orange; OpVer 33.171.1.6116*/
const Device UA20129 = {S76425, 102, { &P12292,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HTC_Jade/1.40.164.2 (21259)*/
const Device UA30264 = {S76426, 122, { &P12292,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC_Touch_3G_T3232-Orange; OpVer 33.171.1.6112*/
const Device UA37193 = {S76427, 122, { &P12292,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC_Touch_3G_T3232-Orange; OpVer 33.171.2.7311*/
const Device UA66433 = {S76428, 80, { &P12292,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_Touch_3G_T3232 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA7867 = {S76429, 103, { &P11708,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK618i/R1JG Release/Feb-15-2007 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA32968 = {S76430, 135, { &P12810,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-es; Light Pro Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA63302 = {S76431, 148, { &P18008,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; tr-tr; Turkcell T10 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018904 = {S76432, 70, { &P18008,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Turkcell T10 Build/ERE27)*/
const Device UA59857 = {S76433, 138, { &P15699,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Incredible Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA310419 = {S76434, 138, { &P15699,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Incredible Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA652564 = {S76435, 138, { &P15699,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Incredible Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA693665 = {S76436, 138, { &P15699,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Incredible Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA736089 = {S76437, 148, { &P15699,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Droid Incredible Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1861081 = {S76438, 138, { &P15699,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; Incredible Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1895791 = {S76439, 138, { &P15699,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Incredible Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025138 = {S76440, 157, { &P15699,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Incredible Build/FRF91; liquid.htcinc.X.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA69015 = {S76441, 97, { &P18036,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; MWP6885)*/
const Device UA232224 = {S76442, 103, { &P17595,&P17296,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; Mac App Store Edition; en) Presto/2.7.62 Version/11.01*/
const Device UA325028 = {S76443, 80, { &P17595,&P17296,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en) Presto/2.7.62 Version/11.00*/
const Device UA252043 = {S76444, 131, { &P12496,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; I897 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA322479 = {S76445, 131, { &P12496,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; I897 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA331483 = {S76446, 131, { &P12496,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; I897 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018703 = {S76447, 135, { &P12496,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SGH-I897 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023146 = {S76448, 119, { &P12496,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; I897 Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA563309 = {S76449, 134, { &P12896,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P999 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA639583 = {S76450, 134, { &P12896,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P999 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA836098 = {S76451, 155, { &P12896,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P999 Build/MIUI; LG-P990 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020209 = {S76452, 148, { &P12896,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-P999 Build/MIUIandroid 1.8.19) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032108 = {S76453, 134, { &P12896,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.9; en-us; LG-P999 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA630373 = {S76454, 133, { &P21122,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; P-04D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031876 = {S76455, 57, { &P21122,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; P-04D Build/GRJ90)*/
const Device UA2028722 = {S76456, 144, { &P21630,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; i-mobile i-note WiFi Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2552784 = {S76457, 146, { &P21630,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-note WiFi 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2728890 = {S76458, 153, { &P21630,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-NOTE WIFI 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2836016 = {S76459, 75, { &P21630,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; i-mobile i-note WiFi 7 Build/IMM76D)*/
const Device UA2951736 = {S76460, 75, { &P21630,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; i-mobile i-NOTE WIFI 2 Build/IMM76D)*/
const Device UA2383911 = {S76461, 118, { &P12029,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN82) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/355/UCWEB Mobile*/
const Device UA2257097 = {S76462, 143, { &P22182,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; RPSPE4301 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2430252 = {S76463, 137, { &P14017,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P990 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2402035 = {S76464, 66, { &P14993,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; LT15i; 480*854) UCWEB7.9.0.94/139/352*/
const Device UA2481301 = {S76465, 66, { &P14993,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; LT15i; 480*854) UCWEB7.9.0.94/139/444*/
const Device UA2567950 = {S76466, 138, { &P18488,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-L300 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2670612 = {S76467, 50, { &P15364,&P18580,&P22329,&P18092 }};	/*Useragent - Rockmelt/3.0.0.328 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2672790 = {S76468, 156, { &P15802,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amaze 4G Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3011041 = {S76469, 161, { &P15802,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amaze 4G Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/ruby*/
const Device UA2737232 = {S76470, 144, { &P21466,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Titan TV S320 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2740006 = {S76471, 137, { &P18682,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-M950 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2755413 = {S76472, 136, { &P19917,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P765 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2899153 = {S76473, 133, { &P20146,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2974721 = {S76474, 135, { &P21037,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LIFETAB_S9512 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3018275 = {S76475, 143, { &P19862,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Micromax A110Q Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3078847 = {S76476, 136, { &P19862,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Micromax A110Q Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA16632 = {S76477, 65, { &P11883,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-E2100/E2100JAIC1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA10103 = {S76478, 135, { &P12323,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; nl-; Pulse Build/CRB17) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA10105 = {S76479, 137, { &P12323,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-gb; Pulse Build/CRB17) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA20308 = {S76480, 104, { &P15063,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-T659/T659UVIG9 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA32737 = {S76481, 62, { &P12751,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-C160L/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA64663 = {S76482, 61, { &P12751,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-C160/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA250064 = {S76483, 43, { &P13552,&P5129,&P24663,&P18092 }};	/*Useragent - NokiaC6-00/UC Browser7.9.0.102/50/355/UCWEB*/
const Device UA421159 = {S76484, 31, { &P13552,&P5129,&P24663,&P18092 }};	/*Useragent - NokiaC6-00/UCWEB7.9.0.94/50/800*/
const Device UA507453 = {S76485, 52, { &P15364,&P18110,&P22526,&P18092 }};	/*Useragent - Midori/0.1.9 (X11; Linux i686; U; en-us) WebKit/532+*/
const Device UA2587103 = {S76486, 59, { &P15364,&P18110,&P22526,&P18092 }};	/*Useragent - Midori/0.1.10 (X11; Linux i686; U; en-us) WebKit/(531).(2)+*/
const Device UA2815481 = {S76487, 58, { &P15364,&P18110,&P22526,&P18092 }};	/*Useragent - Midori/0.1.10 (X11; Linux i686; U; en-us) WebKit/(531).(2)*/
const Device UA3084570 = {S76488, 54, { &P15364,&P18110,&P22526,&P18092 }};	/*Useragent - Midori/0.1.7 (X11; Linux x86_64; U; en-us) WebKit/532+*/
const Device UA3117355 = {S76489, 55, { &P15364,&P18110,&P22526,&P18092 }};	/*Useragent - Midori/0.1.20 (X11; Linux i686; U; fr-fr) WebKit/532.1+*/
const Device UA638300 = {S76490, 158, { &P20370,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ZTE-C_N760/N760V1.0.0B03; 320*480; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA761421 = {S76491, 158, { &P20370,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE-C_N760/N760V1.0.0B04; 320*480; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA773940 = {S76492, 158, { &P20370,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ZTE-C_N760/N760V1.0.0B02; 320*480; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA820418 = {S76493, 126, { &P20370,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ZTE-C N760 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2021394 = {S76494, 126, { &P20370,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE-C N760 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2024605 = {S76495, 158, { &P20370,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE-C_N760/N760V1.0.0B05; 320*480; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026077 = {S76496, 131, { &P20370,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ZTE-C N760 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2412613 = {S76497, 131, { &P20370,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE-C N760 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2787221 = {S76498, 158, { &P20370,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE-C_N760/N760V1.0.0B07; 320*480; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA796635 = {S76499, 134, { &P22207,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; DATAM803HC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2340405 = {S76500, 136, { &P21164,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ZP200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044109 = {S76501, 137, { &P21164,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ZP200+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2353983 = {S76502, 139, { &P19848,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SHW-M440S Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2457534 = {S76503, 139, { &P19848,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M440S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2422775 = {S76504, 118, { &P13602,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; gt-s5233w) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2552193 = {S76505, 115, { &P13602,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; vi; gt-s5233w) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2399085 = {S76506, 141, { &P19759,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo K860 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2457373 = {S76507, 137, { &P20740,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; AVA757 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2533470 = {S76508, 166, { &P18645,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; SPH-L710 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2560168 = {S76509, 174, { &P18645,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2601702 = {S76510, 166, { &P18645,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; SPH-L710 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2700281 = {S76511, 167, { &P18645,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; SGH-I747M Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2851872 = {S76512, 166, { &P18645,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9305 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2553799 = {S76513, 103, { &P12140,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.442; en-us; 9550/4.2.0.442) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2654113 = {S76514, 103, { &P12140,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.487; en-us; 9550/4.2.0.487) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2934036 = {S76515, 103, { &P12140,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.323; en-us; 9550/4.2.0.323) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2983758 = {S76516, 103, { &P12140,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.124; en-us; 9550/4.2.0.124) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2556663 = {S76517, 142, { &P21655,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; POV_TAB_NAVI7_3G_M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2749767 = {S76518, 142, { &P21655,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; POV_TAB-NAVI7-3G-M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2700981 = {S76519, 20, { &P17346,&P17779,&P21178,&P18092 }};	/*Useragent - iPod4,1/6.1 (10B144)*/
const Device UA2703008 = {S76520, 43, { &P17346,&P17779,&P21178,&P18092 }};	/*Useragent - server-bag [iPhone OS,6.1.3,10B329,iPod4,1]*/
const Device UA2733486 = {S76521, 22, { &P17346,&P17779,&P21178,&P18092 }};	/*Useragent - iPod4,1/6.1.3 (10B329)*/
const Device UA2855362 = {S76522, 22, { &P17346,&P17779,&P21178,&P18092 }};	/*Useragent - iPod4,1/6.1.2 (10B146)*/
const Device UA2955530 = {S76523, 22, { &P17346,&P17779,&P21178,&P18092 }};	/*Useragent - iPod5,1/6.1.3 (10B329)*/
const Device UA2676371 = {S76524, 136, { &P18653,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P870 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2751479 = {S76525, 60, { &P15364,&P18146,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; SunOS i86pc; rv:21.0) Gecko/X Firefox/21.0*/
const Device UA2799332 = {S76526, 152, { &P18557,&P20706,&P24677,&P18092 }};	/*Useragent - Nokia308/2.0 (05.85) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia308) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2754645 = {S76527, 150, { &P19762,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI MT1-U06 Build/HuaweiMT1-U06) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2770417 = {S76528, 126, { &P18465,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KFOT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2868013 = {S76529, 134, { &P18465,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KF012 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774114 = {S76530, 146, { &P23102,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8681 Build/HuaweiU8681) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2878647 = {S76531, 146, { &P23716,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI A199 Build/HuaweiA199) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998115 = {S76532, 137, { &P23716,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI A199 Build/HuaweiA199) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3006956 = {S76533, 159, { &P23716,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_A199/A199V100R001C92B123; 720*1280; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA3074131 = {S76534, 68, { &P23716,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HUAWEI A199 Build/HuaweiA199)*/
const Device UA3079019 = {S76535, 159, { &P23716,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_A199/A199V100R001C92B128; 720*1280; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA3088613 = {S76536, 163, { &P23716,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HW-HUAWEI_A199/A199V100R001C92B121SP05; 720*1280; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Mobile Safari/534.30*/
const Device UA2899090 = {S76537, 161, { &P18513,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6802 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3029131 = {S76538, 161, { &P16356,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-N7000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3061433 = {S76539, 161, { &P16356,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; GT-N7000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3030991 = {S76540, 170, { &P14682,&P5843,&P24679,&P18092 }};	/*Useragent - Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6303iclassic) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3082733 = {S76541, 129, { &P20750,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AN10DG3 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3132021 = {S76542, 144, { &P18640,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA64781 = {S76543, 61, { &P13900,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia3100/1.0 (06.31) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA572659 = {S76544, 104, { &P12473,&P5204,&P17338,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2730c-1/UC Browser7.9.0.102/69/352*/
const Device UA539145 = {S76545, 126, { &P15364,&P17350,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en_US) AppleWebKit/530.19.2 (KHTML, like Gecko) Dreamweaver/11.5.0.5315 Version/4.0.2 Safari/530.19.2*/
const Device UA2977941 = {S76546, 114, { &P15364,&P17350,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT based; en-US) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Safari/528.17*/
const Device UA687204 = {S76547, 97, { &P15364,&P7133,&P22333,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.0.4) Gecko/X Firefox/3.0.4 CometBird/3.0.4*/
const Device UA588622 = {S76548, 68, { &P17968,&P17979,&P20434,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:12.0a2) Gecko/12.0a2 Firefox/12.0a2*/
const Device UA597128 = {S76549, 66, { &P17968,&P17979,&P20434,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:12.0a2) Gecko/12.0 Firefox/12.0a2*/
const Device UA601654 = {S76550, 74, { &P17968,&P17979,&P20434,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:12.0) Gecko/12.0 Firefox/12.0 Fennec/12.0*/
const Device UA924296 = {S76551, 62, { &P17968,&P17979,&P20434,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:12.0) Gecko/12.0 Firefox/12.0*/
const Device UA797626 = {S76552, 81, { &P15364,&P18110,&P20460,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:10.0.2) Gecko/X Firefox/10.0.2 SeaMonkey/2.7.2*/
const Device UA2812145 = {S76553, 64, { &P15364,&P18110,&P20460,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:10.0.2) Gecko/X SeaMonkey/2.7.2*/
const Device UA624644 = {S76554, 43, { &P15547,&P17484,&P17347,&P18092 }};	/*Useragent - samsung-gt-c6712/UC Browser8.0.3.107/69/444*/
const Device UA877566 = {S76555, 85, { &P12080,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s3653) UCBrowser8.3.0.154/69/352/UCWEB Mobile*/
const Device UA904917 = {S76556, 85, { &P12080,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s3653) UCBrowser8.3.1.161/69/405/UCWEB Mobile*/
const Device UA1957971 = {S76557, 116, { &P14482,&P5843,&P17438,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-01.5/07.32; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.7*/
const Device UA2304162 = {S76558, 115, { &P14482,&P5843,&P17438,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-01/06.00; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15*/
const Device UA2032783 = {S76559, 132, { &P12765,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; th-th; A100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2666320 = {S76560, 138, { &P12765,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; MITO A100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2483405 = {S76561, 128, { &P20838,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SHT21 Build/SB221) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2534595 = {S76562, 128, { &P20838,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SHT21 Build/SC170) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2600114 = {S76563, 128, { &P20838,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SHT21 Build/S2150) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2667969 = {S76564, 138, { &P21645,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TANK 4.5 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2755258 = {S76565, 138, { &P18292,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ609 Build/7.7.1-85_MZ609-31) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2765146 = {S76566, 142, { &P23533,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xtouch X405 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2966871 = {S76567, 159, { &P18283,&P18111,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300*/
const Device UA2943409 = {S76568, 139, { &P24149,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Aqua I-5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3027715 = {S76569, 113, { &P14826,&P5843,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC1-01/06.15; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA452166 = {S76570, 141, { &P15364,&P17818,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ Ubuntu/11.04 (3.0.4-1ubuntu1) Epiphany/3.0.4*/
const Device UA637470 = {S76571, 72, { &P15364,&P17285,&P22311,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:6.0) Gecko/X Firefox/6.0 SeaMonkey/2.3.1*/
const Device UA2026673 = {S76572, 140, { &P22133,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Z710t Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034332 = {S76573, 131, { &P22133,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Z710t Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2385571 = {S76574, 101, { &P18591,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 810)*/
const Device UA2480885 = {S76575, 162, { &P12082,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Mini Pro Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2702740 = {S76576, 176, { &P12082,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; X10 Build/JZO54K; CyanogenMod-AlienMOD_Solar_Flare_SeriesV) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2496706 = {S76577, 131, { &P18714,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE V889D Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2520352 = {S76578, 140, { &P18714,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE V889D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2577998 = {S76579, 142, { &P18714,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; ZTE-U V889D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2504243 = {S76580, 154, { &P20530,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MOT-XT788 Build/IRPMCT_B_02.60.00RPD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555878 = {S76581, 145, { &P20530,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MOT-XT788 Build/IRPMCT_B_02.60.00RPD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2660720 = {S76582, 154, { &P20530,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MOT-XT788 Build/IRPMCT_B_02.64.00RPD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2510304 = {S76583, 167, { &P18296,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC One V Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2736486 = {S76584, 135, { &P23840,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; W120 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2784097 = {S76585, 137, { &P21283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869296 = {S76586, 138, { &P21283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2894037 = {S76587, 138, { &P21283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2961138 = {S76588, 156, { &P13107,&P5043,&P24678,&P18092 }};	/*Useragent - NokiaC2-06/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-06) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3020694 = {S76589, 132, { &P23162,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B15 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3081102 = {S76590, 135, { &P23190,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Marquis_MP977 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3114908 = {S76591, 150, { &P15767,&P22882,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari/7534.48.3 TeamStream HD (iPad2,1 370 1.8.7)*/
const Device UA3129208 = {S76592, 150, { &P15767,&P22882,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Safari/7534.48.3 TeamStream HD (iPad3,4 411 1.9)*/
const Device UA3131324 = {S76593, 127, { &P23702,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ME173X Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3132954 = {S76594, 128, { &P23702,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ME173X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA62867 = {S76595, 39, { &P11821,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GB270 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA23253 = {S76596, 70, { &P14366,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHB100/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62059 = {S76597, 111, { &P14366,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHB100/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA38538 = {S76598, 103, { &P14619,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-SGH-A597/A597UCJG1 SHP/VPP/R5 Jasmine/1.0 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA63741 = {S76599, 84, { &P14537,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-799/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA2370893 = {S76600, 85, { &P14537,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-799A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA66983 = {S76601, 139, { &P17960,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; Next2 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA181827 = {S76602, 120, { &P15364,&P17188,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 4.0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA683149 = {S76603, 87, { &P17595,&P17296,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Macintosh;Mac OS X (10.6.8);en;kml:2.2;client:Free;type:default)*/
const Device UA1937209 = {S76604, 87, { &P17595,&P17296,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Macintosh;Mac OS X (10.6.8);de;kml:2.2;client:Free;type:default)*/
const Device UA2148825 = {S76605, 87, { &P17595,&P17296,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Macintosh;Mac OS X (10.6.8);fr;kml:2.2;client:Free;type:default)*/
const Device UA720485 = {S76606, 116, { &P14714,&P5843,&P17433,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02.5/06.73; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.1.21.8*/
const Device UA809662 = {S76607, 114, { &P14714,&P5843,&P17433,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02/07.16; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.1.21.8*/
const Device UA817011 = {S76608, 127, { &P23994,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A500 Build/GRJ90) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA836391 = {S76609, 132, { &P23994,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo A500/S100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025320 = {S76610, 139, { &P23994,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A500 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027661 = {S76611, 132, { &P23994,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A500 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA908350 = {S76612, 77, { &P23545,&P17017,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; mauie800) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2033749 = {S76613, 155, { &P14842,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; E10i Build/3.0.1.A.0.145; MiniCM7-2.2.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1989458 = {S76614, 79, { &P13163,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaN97)/UC Browser8.5.0.183/50/352/UCWEB Mobile*/
const Device UA2160794 = {S76615, 52, { &P12449,&P5890,&P21761,&P18092 }};	/*Useragent - iBrowser/2.5 (NokiaE5-00;SymbianOS/9.3 Series60/3.2)*/
const Device UA2594719 = {S76616, 178, { &P18221,&P5843,&P24671,&P18092 }};	/*Useragent - Nokia302/5.0 (14.78) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia302) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2712979 = {S76617, 106, { &P20082,&P17484,&P17369,&P18092 }};	/*Useragent - SAMSUNG-GT-E2350B Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.29475/29.3417; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2502527 = {S76618, 140, { &P23927,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; Moii E801 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099726 = {S76619, 62, { &P23927,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Moii E801 Build/IMM76I)*/
const Device UA2529745 = {S76620, 143, { &P22608,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Spice Mi-530 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2573075 = {S76621, 71, { &P15364,&P6704,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.8.1.24) Gecko/X Firefox/3.6*/
const Device UA2596929 = {S76622, 157, { &P15060,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-S5660 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2706334 = {S76623, 156, { &P15060,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-S5660 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088476 = {S76624, 157, { &P15060,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-S5660 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2669225 = {S76625, 134, { &P23815,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Z7C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2676420 = {S76626, 133, { &P19788,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; E350 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2780938 = {S76627, 133, { &P19788,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; E350 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2705030 = {S76628, 95, { &P23122,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; Karbonn_A6) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2794034 = {S76629, 87, { &P23122,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A6) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2773956 = {S76630, 139, { &P23855,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; WM8850-mid Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782632 = {S76631, 132, { &P23855,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; WM8850-mid Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2796238 = {S76632, 135, { &P23855,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; WM8850:X6-10V Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3083946 = {S76633, 140, { &P17346,&P18432,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A523 Safari/8536.25*/
const Device UA3096036 = {S76634, 138, { &P17346,&P18432,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/10A403 Safari/8536.25*/
const Device UA3096289 = {S76635, 155, { &P20705,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA21755 = {S76636, 84, { &P13373,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonZ310iv/R1JC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA27457 = {S76637, 83, { &P13373,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonZ310a/R8BA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62493 = {S76638, 83, { &P13373,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonZ310i/R6BC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA50373 = {S76639, 112, { &P13246,&P17483,&P4732,&P18092 }};	/*Useragent - LG-GD710/V10f; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.14812; U; en) Opera 9.50*/
const Device UA54413 = {S76640, 122, { &P14205,&P17483,&P3459,&P18092 }};	/*Useragent - LG-C320/V100 Obigo/Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA350988 = {S76641, 115, { &P17595,&P17296,&P23600,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.52.7 (KHTML, like Gecko) Fluid/0.9.6 Safari/534.52.7*/
const Device UA841229 = {S76642, 115, { &P17595,&P17296,&P23600,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Fluid/0.9.6 Safari/534.57.2*/
const Device UA881462 = {S76643, 125, { &P17595,&P17296,&P23600,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Fluid/0.9.6 Safari/533.21.1*/
const Device UA1892292 = {S76644, 111, { &P17595,&P17296,&P23600,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.50 (KHTML, like Gecko) Fluid/0.9.6 Safari/534.50*/
const Device UA1906200 = {S76645, 117, { &P17595,&P17296,&P23600,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.51.22 (KHTML, like Gecko) Fluid/0.9.6 Safari/534.51.22*/
const Device UA1991037 = {S76646, 115, { &P17595,&P17296,&P23600,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.55.3 (KHTML, like Gecko) Fluid/0.9.6 Safari/534.55.3*/
const Device UA2278159 = {S76647, 127, { &P17595,&P17296,&P23600,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) Fluid/0.9.6 Safari/533.20.25*/
const Device UA2757529 = {S76648, 121, { &P17595,&P17296,&P23600,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; nl-nl) AppleWebKit/532.3+ (KHTML, like Gecko) Fluid/0.9.6 Safari/532.3+*/
const Device UA2859553 = {S76649, 115, { &P17595,&P17296,&P23600,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.58.2 (KHTML, like Gecko) Fluid/0.9.6 Safari/534.58.2*/
const Device UA2937056 = {S76650, 115, { &P17595,&P17296,&P23600,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.59.8 (KHTML, like Gecko) Fluid/0.9.6 Safari/534.59.8*/
const Device UA600500 = {S76651, 85, { &P12080,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s3653) UCBrowser8.2.0.132/69/352/UCWEB Mobile*/
const Device UA605011 = {S76652, 85, { &P12080,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s3653) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2161078 = {S76653, 38, { &P12080,&P17484,&P24667,&P18092 }};	/*Useragent - samsung-gt-s3653/UCWEB8.2.0.116/69/352*/
const Device UA895758 = {S76654, 80, { &P15364,&P17285,&P20904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.7.5) Gecko/X Netscape/8.1.3*/
const Device UA2238870 = {S76655, 97, { &P17595,&P18207,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:12.0) Gecko/X Firefox/12.0 AlexaToolbar/alxf-2.17*/
const Device UA2200538 = {S76656, 142, { &P23898,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; ELONEX765ET Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2290647 = {S76657, 127, { &P17595,&P17295,&P24749,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; en-us) AppleWebKit/528.16 (KHTML, like Gecko) Stainless/0.5.3 Safari/525.20.1*/
const Device UA2420563 = {S76658, 66, { &P18223,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.7; zh-cn; LT22i; 540*960) UCWEB7.9.0.94/139/352*/
const Device UA2466894 = {S76659, 66, { &P18223,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; LT22i; 540*960) UCWEB7.9.0.94/139/444*/
const Device UA2426874 = {S76660, 33, { &P12897,&P5843,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (NokiaC2-00/03.45)*/
const Device UA2581715 = {S76661, 143, { &P22992,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CloudPad 700d Build/FSV02) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2617593 = {S76662, 53, { &P17337,&P17979,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.5023 Android/3.1 Device/samsung-GT-P7500*/
const Device UA2660567 = {S76663, 56, { &P15364,&P22310,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Win3.11; Trident/5.0)*/
const Device UA2669265 = {S76664, 146, { &P22646,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GiONEE_Dream_D1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2670401 = {S76665, 185, { &P18113,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (DROID RAZR, 42, 1.9.1)*/
const Device UA2702527 = {S76666, 136, { &P19790,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; L-04E Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2737216 = {S76667, 136, { &P22679,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3088802 = {S76668, 129, { &P22679,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2774061 = {S76669, 188, { &P15794,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2780200 = {S76670, 142, { &P23328,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; GOCLEVER TAB M813G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2783355 = {S76671, 139, { &P22641,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C1504 Build/11.3.A.0.47) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2970558 = {S76672, 97, { &P14519,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaN97 mini) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3033070 = {S76673, 161, { &P18382,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire C Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2971188 = {S76674, 92, { &P13554,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN79) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3946 = {S76675, 76, { &P12226,&P5448,&P3430,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Opera/9.5) HTC-PPC6850 BM BMC*/
const Device UA5206 = {S76676, 54, { &P12226,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Pro_T7272 Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA28163 = {S76677, 57, { &P12226,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Pro_T7272 Opera/9.50 (Windows NT 5.1; U; zh-tw)*/
const Device UA32738 = {S76678, 62, { &P12226,&P5448,&P3430,&P18092 }};	/*Useragent - Swisscom/1.0/HTC_Touch_Pro/ Opera/9.50 (Windows NT 5.1; U; de)*/
const Device UA45783 = {S76679, 54, { &P12226,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Pro_T7272 Opera/9.50 (Windows NT 5.1; U; pt)*/
const Device UA262569 = {S76680, 54, { &P12226,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Pro_T7272 Opera/9.50 (Windows NT 5.1; U; nl)*/
const Device UA536885 = {S76681, 54, { &P12226,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Pro_T7272 Opera/9.50 (Windows NT 5.1; U; it)*/
const Device UA710961 = {S76682, 61, { &P12226,&P5448,&P3430,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_Touch_Pro Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA1857486 = {S76683, 54, { &P12226,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Pro_T7272 Opera/9.50 (Windows NT 5.1; U; nb)*/
const Device UA30442 = {S76684, 141, { &P12316,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; E400 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA30445 = {S76685, 69, { &P12316,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; E400 Build/ECLAIR)*/
const Device UA2018595 = {S76686, 63, { &P12316,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; E400 Build/ECLAIR)*/
const Device UA26916 = {S76687, 121, { &P15518,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S5230C/1.0 SHP/R5 Release/02.01.2009 Browser/NetFront3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA49682 = {S76688, 131, { &P13072,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; it-it; TT101 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA64056 = {S76689, 92, { &P14570,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Maple_S520 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA462661 = {S76690, 94, { &P15364,&P5690,&P18245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; rv:1.7.3) Gecko/X Waterunicorn/0.10 StumbleUpon/1.998*/
const Device UA476312 = {S76691, 103, { &P15364,&P5690,&P18245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.4b) Gecko/X Mozilla Firebird/0.5+ StumbleUpon/1.63*/
const Device UA2495973 = {S76692, 51, { &P15364,&P5690,&P18245,&P18092 }};	/*Useragent - Mozilla/3.0 (compatible; WebMon 1.0.12; Windows XP)*/
const Device UA1941333 = {S76693, 146, { &P16452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; GSmart G1345 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027847 = {S76694, 128, { &P20817,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ANS-01 Build/2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2424350 = {S76695, 130, { &P20817,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ANS-01 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2837348 = {S76696, 130, { &P20817,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; ANS-01 Build/RK2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2028315 = {S76697, 141, { &P12964,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-S710d/4.09.1401.0) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031732 = {S76698, 141, { &P12964,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-S710d/4.08.1401.7) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031910 = {S76699, 129, { &P12964,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC S710d) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032818 = {S76700, 131, { &P12964,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC S710d Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2034560 = {S76701, 126, { &P12964,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC S710d Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2450244 = {S76702, 141, { &P12964,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-S710d/4.10.1401.2) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2165722 = {S76703, 135, { &P21533,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; vivo E1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2722491 = {S76704, 128, { &P21533,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; vivo E1 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2170992 = {S76705, 148, { &P22043,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2402583 = {S76706, 148, { &P22043,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P5113 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2612260 = {S76707, 155, { &P22043,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2612781 = {S76708, 155, { &P22043,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2615756 = {S76709, 148, { &P22043,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2624006 = {S76710, 146, { &P22043,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-P5113 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2349190 = {S76711, 141, { &P21732,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; K-Touch_W650 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2352195 = {S76712, 119, { &P12456,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian73-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile*/
const Device UA2536347 = {S76713, 91, { &P12707,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia701) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2528157 = {S76714, 139, { &P21538,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo Y3t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984795 = {S76715, 130, { &P21538,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo Y3t Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2987291 = {S76716, 61, { &P21538,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; vivo Y3t Build/IMM76D)*/
const Device UA2584779 = {S76717, 71, { &P17595,&P18207,&P21579,&P18092 }};	/*Useragent - Mac OS X/10.8.3 (12D78); ExchangeWebServices/3.0 (157); Mail/6.3 (1503)*/
const Device UA2941352 = {S76718, 136, { &P21909,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-03E Build/10.1.E.0.269) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2670604 = {S76719, 138, { &P17691,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I957 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2709239 = {S76720, 131, { &P17691,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957R Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2734624 = {S76721, 131, { &P17691,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I957M Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2753780 = {S76722, 130, { &P14905,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755277 = {S76723, 137, { &P18507,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8160 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2870018 = {S76724, 138, { &P18507,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8160P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773015 = {S76725, 136, { &P14574,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus S Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783061 = {S76726, 147, { &P16362,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2937683 = {S76727, 151, { &P16362,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710VMUB Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2937991 = {S76728, 150, { &P16362,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710BST Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2934606 = {S76729, 140, { &P19878,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I497 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2974653 = {S76730, 144, { &P22906,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A390_ROW Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3061189 = {S76731, 136, { &P18645,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I747 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.17 Mobile Safari/537.36*/
const Device UA3080019 = {S76732, 145, { &P18645,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I747 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.30 Mobile Safari/537.36*/
const Device UA3131736 = {S76733, 137, { &P18645,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L710 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3081337 = {S76734, 130, { &P21202,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-P500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA67816 = {S76735, 72, { &P14867,&P5208,&P3499,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Teleca Q7; Brew 3.1.5; U; en) 240X400 LGE UX840*/
const Device UA667816 = {S76736, 95, { &P17595,&P17297,&P18394,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.7.4; U; Edition MAS; en) Presto/2.10.229 Version/11.63*/
const Device UA2218050 = {S76737, 96, { &P15803,&P5843,&P17338,&P18092 }};	/*Useragent - Nokia200/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia200/UC Browser7.9.0.102/69/352*/
const Device UA2361462 = {S76738, 120, { &P11907,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE52-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile*/
const Device UA2262264 = {S76739, 158, { &P20134,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2350051 = {S76740, 139, { &P21437,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-F120S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2350576 = {S76741, 166, { &P18283,&P18111,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-I9300 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2556500 = {S76742, 166, { &P18283,&P18111,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9300 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2455456 = {S76743, 136, { &P18739,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; EVOPad_R8010 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2652417 = {S76744, 134, { &P18739,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; BlissR8015 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2658159 = {S76745, 136, { &P18739,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R801T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2699410 = {S76746, 140, { &P18739,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-Th; OPPOR8015 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2459467 = {S76747, 143, { &P20250,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Celkon A 200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2537196 = {S76748, 149, { &P20143,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Edison 3G Build/1.1.11-1015 X-15:24) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2570221 = {S76749, 69, { &P15364,&P7133,&P21711,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:20.0) Gecko/X Thunderbird/20.0*/
const Device UA2663267 = {S76750, 151, { &P22704,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-STYLE Q 5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2674755 = {S76751, 138, { &P15438,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Sensation Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676349 = {S76752, 142, { &P15438,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Sensation Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2703385 = {S76753, 130, { &P23295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U807 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2764952 = {S76754, 139, { &P23295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U807 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985899 = {S76755, 130, { &P23295,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U807 Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2986882 = {S76756, 61, { &P23295,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ZTE U807 Build/IMM76D)*/
const Device UA2709297 = {S76757, 138, { &P18488,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L300 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2783375 = {S76758, 151, { &P20231,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A21 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2997972 = {S76759, 159, { &P20231,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A21 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2733600 = {S76760, 156, { &P19853,&P18066,&P18062,&P18092 }};	/*Useragent - GT-N7102 Linux/3.0.13 Android/4.1.9 Release/02.01.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2976009 = {S76761, 126, { &P18733,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M532 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974450 = {S76762, 136, { &P22950,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E977 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3538 = {S76763, 103, { &P15758,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-J750AFHA1/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA24461 = {S76764, 103, { &P15758,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-J750BVGG1/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA25120 = {S76765, 103, { &P15758,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-J750MJHL1/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA26056 = {S76766, 103, { &P15758,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-J750AFHC1/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA61965 = {S76767, 103, { &P15758,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-J750MJGK4/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA64068 = {S76768, 98, { &P15758,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-J750/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA64069 = {S76769, 103, { &P15758,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-J750BNGI4/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA65558 = {S76770, 103, { &P15758,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-J750MJGJ4/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA105086 = {S76771, 62, { &P15364,&P7127,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.2.15 Version/10.10*/
const Device UA249228 = {S76772, 41, { &P13977,&P5890,&P24663,&P18092 }};	/*Useragent - NokiaN73/UC Browser7.9.0.102/28/355/UCWEB*/
const Device UA351440 = {S76773, 141, { &P15759,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; HTC Vision Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA603588 = {S76774, 141, { &P15759,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Vision Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA643456 = {S76775, 140, { &P15759,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Vision Build/GRI40) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA829280 = {S76776, 141, { &P15759,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Vision Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1979062 = {S76777, 141, { &P15759,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Vision Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025434 = {S76778, 63, { &P15759,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Vision Build/IMM76L)*/
const Device UA655462 = {S76779, 100, { &P15364,&P5690,&P23668,&P18092 }};	/*Useragent - Mozilla/7.0 (Windows; U; Windows NT 5.1; pl; rv:1.7.1.7) Gecko/X Firefox/3.7.7 (.NET CLR 3.2.37777)"*/
const Device UA1993971 = {S76780, 140, { &P21491,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Micromax A80 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2223079 = {S76781, 131, { &P21491,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; A80 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2367889 = {S76782, 141, { &P12280,&P18117,&P20789,&P18092 }};	/*Useragent - MQQBrowser/38 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA2424655 = {S76783, 142, { &P12280,&P18117,&P20789,&P18092 }};	/*Useragent - MQQBrowser/381 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA2503822 = {S76784, 140, { &P12280,&P18117,&P20789,&P18092 }};	/*Useragent - MQQBrowser/38 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3*/
const Device UA2704066 = {S76785, 171, { &P16362,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SPH-D710, 42, 1.9.1)*/
const Device UA2457748 = {S76786, 129, { &P18464,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2564301 = {S76787, 93, { &P13648,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaE7-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2567565 = {S76788, 135, { &P20480,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JY-G3 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2751477 = {S76789, 164, { &P20183,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752311 = {S76790, 64, { &P15364,&P5690,&P22169,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:21.0) Gecko/X Firefox/21.0 slurp*/
const Device UA2866429 = {S76791, 70, { &P15364,&P5690,&P22169,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:21.0) Gecko/X Firefox/21.0 slurp slurp*/
const Device UA2898274 = {S76792, 95, { &P17968,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Android2.x) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3023377 = {S76793, 134, { &P12605,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT875 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975537 = {S76794, 132, { &P20518,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A10 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974095 = {S76795, 140, { &P21402,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Lenovo S890 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2970813 = {S76796, 92, { &P12707,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia701) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA368780 = {S76797, 136, { &P18741,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; OPPOT703 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024496 = {S76798, 120, { &P18741,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; T703 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2496366 = {S76799, 125, { &P18741,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; T703 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2817616 = {S76800, 56, { &P18741,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; T703 Build/GRJ22)*/
const Device UA444791 = {S76801, 88, { &P17595,&P17296,&P20917,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.1b3) Gecko/X Firefox/3.1b3*/
const Device UA545565 = {S76802, 135, { &P16088,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; KU9500 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA571309 = {S76803, 134, { &P16088,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SU950 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA787579 = {S76804, 134, { &P16088,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; KU9500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019000 = {S76805, 133, { &P16088,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SU950 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1968178 = {S76806, 162, { &P21938,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; YL-Coolpad_5820/5820_V6.20; 320*480; CTC/2.0) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2020753 = {S76807, 162, { &P21938,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; YL-Coolpad_5820/5820_V3.50; 320*480; CTC/2.0) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2020876 = {S76808, 162, { &P21938,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; YL-Coolpad_5820/5820_V5.23; 320*480; CTC/2.0) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2026072 = {S76809, 141, { &P21938,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad 5820 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027833 = {S76810, 147, { &P21938,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; YL-Coolpad_5820/v1.0; 320*480) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2030008 = {S76811, 162, { &P21938,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; YL-Coolpad_5820/5820_V6.10; 320*480; CTC/2.0) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2030501 = {S76812, 162, { &P21938,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; YL-Coolpad_5820/5820_V5.70; 320*480; CTC/2.0) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2721199 = {S76813, 134, { &P21938,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad 5820 Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1969273 = {S76814, 140, { &P21471,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T320e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027523 = {S76815, 131, { &P21471,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T320e Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1993469 = {S76816, 135, { &P17595,&P18207,&P22457,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19*/
const Device UA2178073 = {S76817, 135, { &P17595,&P18207,&P22457,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_1) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19*/
const Device UA2236939 = {S76818, 135, { &P17595,&P18207,&P22457,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19*/
const Device UA2708215 = {S76819, 135, { &P17595,&P18207,&P22457,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19*/
const Device UA2939104 = {S76820, 135, { &P17595,&P18207,&P22457,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19*/
const Device UA2420725 = {S76821, 138, { &P21537,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2837908 = {S76822, 60, { &P21537,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; vivo S9 Build/IMM76D)*/
const Device UA2981982 = {S76823, 129, { &P21537,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S9 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2424083 = {S76824, 129, { &P23637,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MK808 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2727553 = {S76825, 58, { &P23637,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; MK808 Build/JRO03H)*/
const Device UA2424382 = {S76826, 33, { &P14826,&P5843,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (NokiaC1-01/06.15)*/
const Device UA2455861 = {S76827, 142, { &P21614,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Philips W536 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2485710 = {S76828, 136, { &P20891,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Tablet Hexen Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2705750 = {S76829, 131, { &P20891,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Hexen10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2540514 = {S76830, 134, { &P20760,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Event Build/IML77) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2553772 = {S76831, 122, { &P21155,&P17979,&P18062,&P18092 }};	/*Useragent - Lenovo-A800/S100 Linux/3.0.13 Android/4.0 Release/07.19.2012 Browser/AppleWebKit533.1 Profile/ Configuration/ Safari/533.1*/
const Device UA3118480 = {S76832, 149, { &P21155,&P17979,&P18062,&P18092 }};	/*Useragent - Lenovo A800/S100 Linux/3.0.13 Android/4.0.3 Release/12.12.2011 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30*/
const Device UA2564952 = {S76833, 173, { &P12280,&P19826,&P18693,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B146 Safari/7534.48.3 UCBrowser8.9.0.238/44/355*/
const Device UA2597663 = {S76834, 171, { &P12280,&P19826,&P18693,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B144 Safari/7534.48.3 UCBrowser8.9.0.238/44/355*/
const Device UA2607051 = {S76835, 173, { &P12280,&P19826,&P18693,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B329 Safari/7534.48.3 UCBrowser8.9.0.238/44/355*/
const Device UA2704356 = {S76836, 171, { &P12280,&P19826,&P18693,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B142 Safari/7534.48.3 UCBrowser8.9.0.238/44/355*/
const Device UA2731676 = {S76837, 171, { &P12280,&P19826,&P18693,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B143 Safari/7534.48.3 UCBrowser8.9.0.238/44/355*/
const Device UA2602611 = {S76838, 139, { &P14225,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Desire HD Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2604170 = {S76839, 143, { &P14225,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire HD Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2857798 = {S76840, 143, { &P14225,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire HD Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2664288 = {S76841, 134, { &P12280,&P19826,&P21178,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/6.1 Mobile/9B176 Safari/7534.48.3*/
const Device UA2736897 = {S76842, 135, { &P12280,&P19826,&P21178,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.1 Mobile/10B329 Safari/8536.25*/
const Device UA2997423 = {S76843, 135, { &P12280,&P19826,&P21178,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.1 Mobile/10B350 Safari/8536.25*/
const Device UA2677183 = {S76844, 138, { &P19844,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 80 COBALT Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2677301 = {S76845, 138, { &P18386,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I757M Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2800681 = {S76846, 136, { &P21793,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9205 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2942639 = {S76847, 138, { &P24165,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; 3G7334i Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3032063 = {S76848, 156, { &P22585,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L720 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3024986 = {S76849, 162, { &P21648,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-gb; GT-I9505 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltexx*/
const Device UA3066945 = {S76850, 162, { &P21648,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9505 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltexx*/
const Device UA3129663 = {S76851, 162, { &P21648,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-gb; GT-I9505 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltexx*/
const Device UA3081062 = {S76852, 137, { &P21384,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F240L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3130584 = {S76853, 137, { &P21607,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; VS870 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA18787 = {S76854, 111, { &P11918,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-F700/F700XXHB1 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA63307 = {S76855, 120, { &P11918,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-F700-Vodafone/F700ACGK2 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA16305 = {S76856, 61, { &P11871,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia6125/2.0 (03.71) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21726 = {S76857, 46, { &P15324,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-AX310/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA31037 = {S76858, 46, { &P15324,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-LW310/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA62271 = {S76859, 79, { &P16680,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SCH-W169/W169DDDK3 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49508 = {S76860, 144, { &P13089,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Huawei 858 Build/HuaweiU8160) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61056 = {S76861, 115, { &P12617,&P5208,&P4902,&P18092 }};	/*Useragent - POLARIS/6.1 (BREW 3.1.5; U; en-us; LG; POLARIS/6.1/WAP) Boost LG102 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA67028 = {S76862, 136, { &P14808,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; IS01 Build/SA180) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA67248 = {S76863, 136, { &P14808,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; IS01 Build/S9200) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA67369 = {S76864, 136, { &P14808,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; IS01 Build/SC030) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA67806 = {S76865, 136, { &P14808,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; IS01 Build/S2151) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA68201 = {S76866, 136, { &P14808,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; IS01 Build/S3082) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA90745 = {S76867, 135, { &P14808,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; IS01 Build/S2151) AppleWebKit/528.5 (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA421737 = {S76868, 136, { &P14808,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; IS01 Build/S9210) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2018341 = {S76869, 136, { &P14808,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; IS01 Build/S6191) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2018871 = {S76870, 136, { &P14808,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; IS01 Build/S5090) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA89018 = {S76871, 64, { &P17330,&P5448,&P3430,&P18092 }};	/*Useragent - HTC-ST7377/1.59.502.3 (67150) Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA436319 = {S76872, 134, { &P21988,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; PD_Novel Build/MID) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024090 = {S76873, 58, { &P21988,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; PD_Novel Build/MID)*/
const Device UA632392 = {S76874, 72, { &P15364,&P17285,&P22357,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:12.0) Gecko/X Firefox/12.0 PaleMoon/12.0*/
const Device UA793089 = {S76875, 92, { &P15364,&P17285,&P22357,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:12.0) Gecko/X Firefox/12.0-x64 PaleMoon/12.0-x64*/
const Device UA693552 = {S76876, 122, { &P14887,&P17979,&P3508,&P18092 }};	/*Useragent - SAMSUNG-GT-I9008L_TD/1.0 Android/2.2.1 Release/12.15.2010 Browser/AppleWebKit533.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2033128 = {S76877, 136, { &P22083,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; eZee' Tab702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257625 = {S76878, 135, { &P22083,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-pt; eZee'Tab702 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2383001 = {S76879, 135, { &P23376,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M9000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2383117 = {S76880, 179, { &P13123,&P5629,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5360 Build/GINGERBREAD; 240*320) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444*/
const Device UA2425277 = {S76881, 174, { &P20954,&P5629,&P15760,&P18092 }};	/*Useragent - K-Touch_T760_TD/T760_MocorDroid2.2_W11.41_V1.00 Release/06.26.2012 Mozilla/5.0 (Linux; U; Android 2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2430495 = {S76882, 130, { &P15767,&P18432,&P18355,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 Sleipnir/2.2m*/
const Device UA2507875 = {S76883, 128, { &P15767,&P18432,&P18355,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3 Sleipnir/2.2m*/
const Device UA2697229 = {S76884, 130, { &P15767,&P18432,&P18355,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A550 Safari/7534.48.3 Sleipnir/2.2m*/
const Device UA2505707 = {S76885, 140, { &P21304,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Fly_IQ450 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558461 = {S76886, 138, { &P23449,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; Dialog i43 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2599117 = {S76887, 61, { &P15364,&P6704,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Win 9x 4.90; Trident/4.0)*/
const Device UA2730135 = {S76888, 55, { &P15364,&P6704,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Win98; Trident/4.0)*/
const Device UA2667228 = {S76889, 143, { &P24744,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Infinix X450 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2705507 = {S76890, 121, { &P14682,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia6303iclassic/10.80; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2713005 = {S76891, 40, { &P17968,&P17979,&P24442,&P18092 }};	/*Useragent - Dolphin http client/9.4.1(208) (Android)*/
const Device UA2783573 = {S76892, 40, { &P17968,&P17979,&P24442,&P18092 }};	/*Useragent - Dolphin http client/9.4.2(212) (Android)*/
const Device UA2736652 = {S76893, 140, { &P20294,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC_T528w Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2891591 = {S76894, 140, { &P20294,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC T528w Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3087931 = {S76895, 62, { &P20294,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; HTC T528w Build/JRO03H)*/
const Device UA2892550 = {S76896, 138, { &P23564,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; HS-U950 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA10007 = {S76897, 100, { &P12412,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S5550/JCJB3 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA53971 = {S76898, 92, { &P12412,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-S5550U SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA52056 = {S76899, 31, { &P13433,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N02A(c100;TB;W24H16)*/
const Device UA90280 = {S76900, 31, { &P13433,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N02C(c500;TB;W24H16)*/
const Device UA55082 = {S76901, 86, { &P13398,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-GT-B7610/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA760500 = {S76902, 102, { &P13398,&P5448,&P4808,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-GT-B7610/NBUIK3 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA2126386 = {S76903, 102, { &P13398,&P5448,&P4808,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-GT-B7610/NBUIK1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA2270245 = {S76904, 102, { &P13398,&P5448,&P4808,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-GT-B7610/NBUIL1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA316554 = {S76905, 100, { &P15364,&P17188,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA590181 = {S76906, 82, { &P15364,&P17285,&P18191,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:10.0.1) Gecko/X Thunderbird/10.0.1 Lightning/1.2.1*/
const Device UA591822 = {S76907, 85, { &P15364,&P5690,&P20460,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64; rv:10.0.2) Gecko/X Firefox/10.0.2 SeaMonkey/2.7.2*/
const Device UA832312 = {S76908, 63, { &P15364,&P5690,&P20460,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:10.0.2) Gecko/X SeaMonkey/2.7.2*/
const Device UA2312743 = {S76909, 78, { &P15364,&P5690,&P20460,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:10.0.1) Gecko/X Firefox/10.0.1 SeaMonkey/2.7.1*/
const Device UA636338 = {S76910, 141, { &P21530,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; vivo V1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028555 = {S76911, 129, { &P21530,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; vivo V1 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2475249 = {S76912, 134, { &P21530,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; vivo V1 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA761081 = {S76913, 82, { &P12871,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.3;Zh_cn;HTC Wildfire S A510e;320*480;)UCWEB7.8.0.95/139/355*/
const Device UA1940621 = {S76914, 82, { &P12871,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.5;Zh_cn;HTC Wildfire S A510e;320*480;)UCWEB7.8.0.95/139/355*/
const Device UA2021980 = {S76915, 131, { &P12871,&P5629,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.3.3;Zh_cn;HTC Wildfire S A510e;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA1956973 = {S76916, 133, { &P13042,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_IncredibleS_S710e; zh-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2335600 = {S76917, 133, { &P13042,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_IncredibleS_S710e; en-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2261593 = {S76918, 151, { &P13941,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S7 Build/Smartfren.andromaxtab7.0.v.37.3) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2344043 = {S76919, 151, { &P13941,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; S7 Build/Smartfren.andromaxtab7.0.v.27.3) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2352715 = {S76920, 133, { &P23867,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; E620 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2404276 = {S76921, 150, { &P21626,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-STYLE Q6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422242 = {S76922, 137, { &P20956,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; FPT HD Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2428498 = {S76923, 148, { &P21257,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; ATLANTIS Build/NGMNGM_ATLANTIS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2578594 = {S76924, 31, { &P15785,&P5843,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia201/11.81)*/
const Device UA2659755 = {S76925, 115, { &P17595,&P17295,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10_5_8) AppleWebKit/537.26+ (KHTML, like Gecko) Version/6.0.2 Safari/536.26.17*/
const Device UA2855754 = {S76926, 159, { &P17595,&P17295,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/534.50.2 (KHTML, like Gecko) (like Version/6.0 Safari/534.50.2) Roccat/3.0 Roccat/534.50.2*/
const Device UA3079735 = {S76927, 115, { &P17595,&P17295,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10_5_8) AppleWebKit/536.30.1 (KHTML, like Gecko) Version/6.0.5 Safari/536.30.1*/
const Device UA2667720 = {S76928, 138, { &P23140,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Andi 3.5i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2768520 = {S76929, 156, { &P22745,&P18066,&P18062,&P18092 }};	/*Useragent - FONE 500 Linux/3.0.13 Android/4.0.4 Release/02.27.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2754242 = {S76930, 137, { &P18469,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabS2109A-F Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754328 = {S76931, 126, { &P16598,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2768110 = {S76932, 98, { &P12375,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIANokia N81) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2852564 = {S76933, 93, { &P12375,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian81-3) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2878948 = {S76934, 102, { &P12375,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIANokia N81 8GB) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA3121121 = {S76935, 92, { &P12375,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaN81) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2769440 = {S76936, 131, { &P18475,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320T Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2769578 = {S76937, 130, { &P18475,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3029629 = {S76938, 198, { &P23771,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8190N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb*/
const Device UA2798608 = {S76939, 153, { &P18706,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_ov5647 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799271 = {S76940, 152, { &P18706,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_9p017 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2866840 = {S76941, 152, { &P18706,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_253_6628 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893704 = {S76942, 147, { &P18706,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_253 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2958115 = {S76943, 151, { &P18706,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_5640 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2898419 = {S76944, 144, { &P23799,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ONE TOUCH 4011X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3020052 = {S76945, 137, { &P23799,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ONE TOUCH 4011X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2901215 = {S76946, 161, { &P15991,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6102 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3020921 = {S76947, 136, { &P20227,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-L40G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3017807 = {S76948, 147, { &P20129,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3031011 = {S76949, 148, { &P20129,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2S Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3059106 = {S76950, 149, { &P20129,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2SC Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3060463 = {S76951, 148, { &P20129,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2A Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3125291 = {S76952, 148, { &P20129,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2C Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3081693 = {S76953, 138, { &P23184,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; iOCEAN X7 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA9988 = {S76954, 101, { &P15278,&P5208,&P4902,&P18092 }};	/*Useragent - Mozilla/4.0 (BREW 3.1.5; U; en-us; Samsung; PLS_M330; POLARIS/6.1/WAP) MMP/2.0 Configuration/CLDC-1.1*/
const Device UA21427 = {S76955, 101, { &P15278,&P5208,&P4902,&P18092 }};	/*Useragent - Mozilla/4.0 (BREW 3.1.5; U; en-us; Samsung; SPH_M330; POLARIS/6.1/WAP) MMP/2.0 Configuration/CLDC-1.1*/
const Device UA55848 = {S76956, 90, { &P16218,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN91/3.0 (1.10.030) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63660 = {S76957, 92, { &P16218,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN91-2/3.0 (3.10.023) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64497 = {S76958, 92, { &P16218,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN91-1/3.0 (2.10.013) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA712770 = {S76959, 100, { &P14669,&P5843,&P17338,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-00/UC Browser7.9.0.102/69/352*/
const Device UA3085097 = {S76960, 252, { &P14669,&P5843,&P17338,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/69/352*/
const Device UA148838 = {S76961, 10, { &P12193,&P5129,&P4030,&P18092 }};	/*Useragent - NokiaC5-00*/
const Device UA230042 = {S76962, 61, { &P16530,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX285/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA447473 = {S76963, 50, { &P15364,&P17017,&P23790,&P18092 }};	/*Useragent - System Center Operations Manager 2007 6.0.6278.100*/
const Device UA576439 = {S76964, 49, { &P15364,&P17017,&P23790,&P18092 }};	/*Useragent - System Center Operations Manager 2007 6.1.7221.81*/
const Device UA671831 = {S76965, 48, { &P15364,&P17017,&P23790,&P18092 }};	/*Useragent - System Center Operations Manager 2007 6.1.7221.0*/
const Device UA658603 = {S76966, 140, { &P15767,&P18117,&P21786,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) 1Password/3.6.5/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA735652 = {S76967, 138, { &P15767,&P18117,&P21786,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) 1Password/3.6.5/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA2233713 = {S76968, 110, { &P17595,&P17297,&P22890,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) iCab/5.0 Safari/533.16*/
const Device UA1962917 = {S76969, 128, { &P20219,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; T05A Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2400871 = {S76970, 128, { &P20219,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T05A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1983106 = {S76971, 154, { &P15060,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S5660 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033053 = {S76972, 186, { &P15060,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S5660 Build/IMM76L; CyanogenMod-CyanogenMod9 RC1 by MaclawStudios) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029792 = {S76973, 131, { &P21423,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; DaVinci Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2354922 = {S76974, 142, { &P22692,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-pt; moche smart a8 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2427438 = {S76975, 147, { &P18556,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD+ Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2452416 = {S76976, 130, { &P18103,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; FZ-A1B Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2461992 = {S76977, 138, { &P18304,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9070 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2482558 = {S76978, 95, { &P18612,&P17979,&P18062,&P18092 }};	/*Useragent - HTCT329t_TD/1.0 Android/4.0 release/2012 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2549174 = {S76979, 143, { &P20151,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; TOOKY A9PLUS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2647751 = {S76980, 144, { &P20151,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; TOOKY A19PLUS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2694003 = {S76981, 140, { &P20151,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; TOOKY T83 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2762350 = {S76982, 142, { &P20151,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; TOOKY A981+ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892591 = {S76983, 140, { &P20151,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; TOOKY T88 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580248 = {S76984, 107, { &P18510,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_918N) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2596759 = {S76985, 93, { &P14877,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; GT-I9000) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2901218 = {S76986, 153, { &P20175,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI 1S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3004432 = {S76987, 154, { &P20175,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI 1SC Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2892316 = {S76988, 175, { &P22906,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Lenovo A390_ROW Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33482*/
const Device UA3033936 = {S76989, 160, { &P16356,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-N7000 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/n7000*/
const Device UA3062484 = {S76990, 160, { &P16356,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-N7000 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/n7000*/
const Device UA3128220 = {S76991, 160, { &P16356,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-N7000 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/n7000*/
const Device UA3060453 = {S76992, 114, { &P15364,&P7133,&P24142,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36 slurp*/
const Device UA3063575 = {S76993, 114, { &P15364,&P7133,&P24142,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.62 Safari/537.36 slurp*/
const Device UA3080248 = {S76994, 114, { &P15364,&P7133,&P24142,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36 slurp*/
const Device UA3115409 = {S76995, 114, { &P15364,&P7133,&P24142,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36 slurp*/
const Device UA3083603 = {S76996, 131, { &P24103,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; C2 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA8858 = {S76997, 66, { &P12501,&P5843,&P4030,&P18092 }};	/*Useragent - Nokia6750-1c/2.0(ATT.5.93) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA9116 = {S76998, 127, { &P12501,&P5843,&P4030,&P18092 }};	/*Useragent - Nokia6750-1b/2.0(ATT.4.99) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA30985 = {S76999, 46, { &P12385,&P6476,&P3625,&P18092 }};	/*Useragent - LGE-MT310/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA133275 = {S77000, 122, { &P15364,&P5690,&P18211,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/5.2.1.1588 Safari/532.4*/
const Device UA651734 = {S77001, 101, { &P15364,&P5690,&P18211,&P18092 }};	/*Useragent - GoogleEarth/5.2.1.1588(Windows;Microsoft Windows (5.1.2600.3);en-US;kml:2.2;client:Free;type:default)*/
const Device UA2242333 = {S77002, 101, { &P15364,&P5690,&P18211,&P18092 }};	/*Useragent - GoogleEarth/5.2.1.1588(Windows;Microsoft Windows (5.1.2600.3);de-DE;kml:2.2;client:Free;type:default)*/
const Device UA183739 = {S77003, 138, { &P17980,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; vi-vn; Q-mobile S10 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019146 = {S77004, 62, { &P17980,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; Q-mobile S10 Build/FRF91)*/
const Device UA200521 = {S77005, 141, { &P17595,&P17296,&P22447,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Iron/9.0.600.2 Chrome/9.0.600.2 Safari/534.13*/
const Device UA647824 = {S77006, 141, { &P17595,&P17296,&P22447,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Iron/9.0.600.2 Chrome/9.0.600.2 Safari/534.13*/
const Device UA1868443 = {S77007, 141, { &P17595,&P17296,&P22447,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Iron/9.0.600.2 Chrome/9.0.600.2 Safari/534.13*/
const Device UA1889608 = {S77008, 78, { &P18095,&P18094,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (S60; SymbOS; Opera Mobi/SYB-X; U; fa) Presto/2.8.149 Version/11.10*/
const Device UA533788 = {S77009, 58, { &P18254,&P18110,&P22937,&P18092 }};	/*Useragent - Liferea/1.6.4 (Linux; en_US.UTF-8; http://liferea.sf.net/)*/
const Device UA545515 = {S77010, 57, { &P18254,&P18110,&P22937,&P18092 }};	/*Useragent - Liferea/1.6.5 (Linux; en_US.utf8; http://liferea.sf.net/)*/
const Device UA557870 = {S77011, 59, { &P18254,&P18110,&P22937,&P18092 }};	/*Useragent - Liferea/1.6.6b (Linux; en_US.UTF-8; http://liferea.sf.net/)*/
const Device UA1872528 = {S77012, 57, { &P18254,&P18110,&P22937,&P18092 }};	/*Useragent - Liferea/1.6.2 (Linux; en_US.utf8; http://liferea.sf.net/)*/
const Device UA2242428 = {S77013, 57, { &P18254,&P18110,&P22937,&P18092 }};	/*Useragent - Liferea/1.6.4 (Linux; en_US.utf8; http://liferea.sf.net/)*/
const Device UA2275977 = {S77014, 57, { &P18254,&P18110,&P22937,&P18092 }};	/*Useragent - Liferea/1.6.3 (Linux; en_US.utf8; http://liferea.sf.net/)*/
const Device UA2290274 = {S77015, 58, { &P18254,&P18110,&P22937,&P18092 }};	/*Useragent - Liferea/1.6.2 (Linux; en_US.UTF-8; http://liferea.sf.net/)*/
const Device UA2391681 = {S77016, 58, { &P18254,&P18110,&P22937,&P18092 }};	/*Useragent - Liferea/1.6.4 (Linux; fr_FR.UTF-8; http://liferea.sf.net/)*/
const Device UA2472314 = {S77017, 57, { &P18254,&P18110,&P22937,&P18092 }};	/*Useragent - Liferea/1.6.5 (Linux; fr_FR.utf8; http://liferea.sf.net/)*/
const Device UA2472311 = {S77018, 57, { &P18254,&P18110,&P22937,&P18092 }};	/*Useragent - Liferea/1.6.4 (Linux; fr_FR.utf8; http://liferea.sf.net/)*/
const Device UA1952991 = {S77019, 135, { &P21531,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; vivo S3 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027865 = {S77020, 128, { &P21531,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; vivo S3 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2030087 = {S77021, 159, { &P21397,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; LNV-Lenovo_S850e/S-3-03; 480*800; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2414072 = {S77022, 134, { &P21397,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S850e Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2414799 = {S77023, 143, { &P21397,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S850e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2644154 = {S77024, 129, { &P21397,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S850e Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2384298 = {S77025, 151, { &P21127,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar_Touch_S02i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658063 = {S77026, 150, { &P21127,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar Touch S02 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2482953 = {S77027, 155, { &P14482,&P5843,&P24673,&P18092 }};	/*Useragent - NokiaC3-01/5.0 (07.51) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac3-01) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2747294 = {S77028, 159, { &P14482,&P5843,&P24673,&P18092 }};	/*Useragent - NokiaC3-01.5/5.0 (06.75) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac3-01.5) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2490339 = {S77029, 128, { &P15767,&P19826,&P18355,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/7534.48.3 Sleipnir/2.2m*/
const Device UA2533341 = {S77030, 130, { &P15767,&P19826,&P18355,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Sleipnir/2.2m*/
const Device UA2606872 = {S77031, 130, { &P15767,&P19826,&P18355,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.2m*/
const Device UA2605725 = {S77032, 132, { &P23366,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; W30 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2676988 = {S77033, 138, { &P18716,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730M Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2700164 = {S77034, 136, { &P20175,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI 1S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2791357 = {S77035, 127, { &P20175,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI 1S Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2850115 = {S77036, 137, { &P20175,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI 1SC Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2710645 = {S77037, 151, { &P18283,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9300 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2732913 = {S77038, 151, { &P18283,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9300 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2710956 = {S77039, 101, { &P20743,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 720)*/
const Device UA2994966 = {S77040, 109, { &P20743,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 720; ANZ885)*/
const Device UA2713547 = {S77041, 161, { &P14433,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9003 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2753039 = {S77042, 136, { &P22613,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E430 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754804 = {S77043, 132, { &P20518,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A10 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869211 = {S77044, 125, { &P20518,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A10 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754803 = {S77045, 135, { &P20015,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SC-03E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769149 = {S77046, 142, { &P15438,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Sensation Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2772781 = {S77047, 138, { &P15438,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Sensation Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773608 = {S77048, 136, { &P20864,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782899 = {S77049, 129, { &P20864,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2879332 = {S77050, 136, { &P20864,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; mid-A13 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2796538 = {S77051, 151, { &P19853,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7102 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2872630 = {S77052, 151, { &P19853,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7102 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2958668 = {S77053, 140, { &P24638,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Orbit Z50 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3083370 = {S77054, 132, { &P23182,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PMP7280C3G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3111376 = {S77055, 145, { &P15767,&P22882,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 7_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/11A465 Safari/7534.48.3*/
const Device UA3130655 = {S77056, 147, { &P15767,&P22882,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 7_0_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/11A501 Safari/7534.48.3*/
const Device UA18805 = {S77057, 61, { &P11934,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia6103/2.0 (04.90) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA857889 = {S77058, 48, { &P12280,&P17145,&P17470,&P18092 }};	/*Useragent - Facebook 4110.0 (iPhone; iPhone OS 4.2.1; en_US)*/
const Device UA1953239 = {S77059, 48, { &P12280,&P17145,&P17470,&P18092 }};	/*Useragent - Facebook 4110.0 (iPhone; iPhone OS 4.2.6; en_US)*/
const Device UA1957071 = {S77060, 49, { &P12280,&P17145,&P17470,&P18092 }};	/*Useragent - Facebook 4110.0 (iPhone; iPhone OS 4.2.10; en_US)*/
const Device UA1958567 = {S77061, 48, { &P12280,&P17145,&P17470,&P18092 }};	/*Useragent - Facebook 4110.0 (iPhone; iPhone OS 4.2.8; en_US)*/
const Device UA27950 = {S77062, 95, { &P12192,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch_Diamond2_T5353; Windows Phone 6.5)*/
const Device UA90342 = {S77063, 95, { &P12192,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch_Diamond2_T5360; Windows Phone 6.5)*/
const Device UA925908 = {S77064, 99, { &P12192,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch_Diamond2_T5353; Windows Phone 6.5.3.5)*/
const Device UA162255 = {S77065, 142, { &P17998,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XCD 28 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1905666 = {S77066, 128, { &P17998,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XCD 28 Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA165962 = {S77067, 141, { &P15364,&P17285,&P22400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Comodo_Dragon/8.0.0.4 Chrome/8.0.552.215 Safari/534.10*/
const Device UA253824 = {S77068, 120, { &P20126,&P17485,&P18245,&P18091 }};	/*Useragent - SonyEricssonK800i/R1ED Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 (Mediobot/1.0 +http://bot.medio.com)*/
const Device UA759837 = {S77069, 140, { &P22251,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; MW0812 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1966047 = {S77070, 145, { &P22251,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; MW0812_V2.0 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2360391 = {S77071, 82, { &P17595,&P18207,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; en) Presto/2.10.229 Version/11.64*/
const Device UA2459894 = {S77072, 145, { &P20245,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; miTab LION Build/ICS.g08refem611.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2482217 = {S77073, 136, { &P23977,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; VICTORY Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2483519 = {S77074, 148, { &P20836,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Wynncom_G1 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2879240 = {S77075, 137, { &P20836,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3;en-us;[Wynncom G11] build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2509838 = {S77076, 170, { &P18382,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Desire C Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2715760 = {S77077, 160, { &P19779,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A1+ Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2753216 = {S77078, 130, { &P22041,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6810 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2753967 = {S77079, 160, { &P18233,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL_one_touch_995 Build/IceCreamSandwich) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879481 = {S77080, 161, { &P18233,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL_one_touch_995S Build/IceCreamSandwich) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3017009 = {S77081, 158, { &P18295,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One X Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3033807 = {S77082, 158, { &P18295,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3006005 = {S77083, 130, { &P15767,&P18117,&P20784,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/9B206 Safari/7534.48.3*/
const Device UA3108193 = {S77084, 128, { &P15767,&P18117,&P20784,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/9B176 Safari/7534.48.3*/
const Device UA3132873 = {S77085, 132, { &P13644,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3081100 = {S77086, 127, { &P24624,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; A1-811 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA697007 = {S77087, 37, { &P15015,&P17486,&P3807,&P18092 }};	/*Useragent - MOT-WX345/03.38.01 Obigo/Q03C MMP/2.0*/
const Device UA52798 = {S77088, 10, { &P12088,&P5129,&P4030,&P18092 }};	/*Useragent - NokiaX6-00*/
const Device UA68239 = {S77089, 35, { &P12088,&P5129,&P4030,&P18092 }};	/*Useragent - NokiaX6-00/UCBrowser7.7.0.81/50/999*/
const Device UA268367 = {S77090, 128, { &P15364,&P17285,&P22444,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Iron/5.0.381.0 Chrome/5.0.381 Safari/533.4*/
const Device UA330716 = {S77091, 102, { &P12280,&P17779,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA664749 = {S77092, 148, { &P17595,&P17296,&P17392,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-US) AppleWebKit/534.10 (KHTML, like Gecko) RockMelt/0.8.40.147 Chrome/8.0.552.231 Safari/534.10*/
const Device UA744515 = {S77093, 145, { &P17595,&P17296,&P17392,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.7 (KHTML, like Gecko) RockMelt/0.8.36.116 Chrome/7.0.517.44 Safari/534.7*/
const Device UA860079 = {S77094, 145, { &P17595,&P17296,&P17392,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; en-US) AppleWebKit/534.7 (KHTML, like Gecko) RockMelt/0.8.36.116 Chrome/7.0.517.44 Safari/534.7*/
const Device UA896233 = {S77095, 148, { &P17595,&P17296,&P17392,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.10 (KHTML, like Gecko) RockMelt/0.8.40.147 Chrome/8.0.552.231 Safari/534.10*/
const Device UA2267597 = {S77096, 145, { &P17595,&P17296,&P17392,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.3 (KHTML, like Gecko) RockMelt/0.8.34.820 Chrome/6.0.472.63 Safari/534.3*/
const Device UA697214 = {S77097, 97, { &P15364,&P5208,&P4521,&P18092 }};	/*Useragent - Mozilla/5.0 (Sonim-XP3400/E343B_1900B04_B_N1900;U;REX/1.0;BREW/3.1.5;240*320;CTC/2.0) POLARIS/6.2*/
const Device UA784899 = {S77098, 95, { &P15364,&P5208,&P4521,&P18092 }};	/*Useragent - Mozilla/5.0 (Sonim-XP3400/E343B_2100B00_N1900;U;REX/1.0;BREW/3.1.5;240*320;CTC/2.0) POLARIS/6.2*/
const Device UA2813437 = {S77099, 97, { &P15364,&P5208,&P4521,&P18092 }};	/*Useragent - Mozilla/5.0 (Sonim-XP3400/E343B_1900B03_B_N1900;U;REX/1.0;BREW/3.1.5;240*320;CTC/2.0) POLARIS/6.2*/
const Device UA2159822 = {S77100, 116, { &P15837,&P6909,&P17438,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-05/08.60a; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15*/
const Device UA2188963 = {S77101, 128, { &P19775,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A210 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2230304 = {S77102, 60, { &P15364,&P6729,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows 95; U; ru) Presto/2.10.289 Version/12.02*/
const Device UA2385279 = {S77103, 154, { &P20535,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar Touch Kem 350 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426869 = {S77104, 66, { &P18302,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; MT25i; 480*854) UCWEB7.9.0.94/139/444*/
const Device UA2426910 = {S77105, 131, { &P20840,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EGL26BL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2428506 = {S77106, 139, { &P14225,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Desire HD Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2599505 = {S77107, 138, { &P14225,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Desire HD Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2457681 = {S77108, 139, { &P20248,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V889M Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2536056 = {S77109, 95, { &P15501,&P5477,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E16i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2580188 = {S77110, 93, { &P15575,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; GT-I9100) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2703054 = {S77111, 94, { &P15575,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I9100G) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2671144 = {S77112, 145, { &P18114,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Transformer Prime TF201 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2717080 = {S77113, 105, { &P14188,&P5129,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) Nokia5233*/
const Device UA3057945 = {S77114, 26, { &P14188,&P5129,&P17470,&P18092 }};	/*Useragent - Nokia5233;c2VvLWNyYXdsZXI=*/
const Device UA2753211 = {S77115, 143, { &P20816,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WIKO-CINK SLIM Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754334 = {S77116, 143, { &P15854,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ607-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2773817 = {S77117, 130, { &P18454,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGPT12 Build/TISU0105) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2937397 = {S77118, 129, { &P18454,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGPT13 Build/TID0129) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2974662 = {S77119, 143, { &P24048,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly s Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020628 = {S77120, 152, { &P22785,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire 600 dual sim Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020870 = {S77121, 143, { &P22785,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire 600 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2977087 = {S77122, 136, { &P24061,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-S7270 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2977097 = {S77123, 130, { &P24061,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-S7270L Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3092929 = {S77124, 137, { &P24061,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-S7270L Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3108264 = {S77125, 129, { &P24061,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-S7270 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3116405 = {S77126, 105, { &P24612,&P17484,&P18328,&P18092 }};	/*Useragent - SAMSUNG GT-S3330 Opera/9.80 (J2ME/MIDP; Opera Mini/7.1.32840/31.1475; U; en) Presto/2.8.119 Version/11.10*/
const Device UA3080075 = {S77127, 136, { &P20227,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-L40G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083646 = {S77128, 140, { &P15438,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; HTC Sensation Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA61738 = {S77129, 105, { &P12005,&P5776,&P3756,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Series90/1.1 Nokia7710/4.10.0 Profile/MIDP-2.0 Configuration/CLDC-1.0)*/
const Device UA13471 = {S77130, 44, { &P15364,&P5222,&P4626,&P18092 }};	/*Useragent - XV6875.1 Opera/9.70 (Windows NT  5.1; U; en)*/
const Device UA44038 = {S77131, 120, { &P13772,&P17979,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (HW-HUAWEI_C8500/C8500V100R001C92B234;U;Android/2.1;240*320;CTC/2.0) AppleWebKit/530.17 Mobile Safari/530.17*/
const Device UA49951 = {S77132, 46, { &P13117,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-UN150/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA430462 = {S77133, 113, { &P15364,&P7133,&P22440,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/530.9 (KHTML, like Gecko) Iron/2.0.178.0 Safari/530.9*/
const Device UA719632 = {S77134, 113, { &P15364,&P7133,&P22440,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/530.1 (KHTML, like Gecko) Iron/2.0.168.0 Safari/530.1*/
const Device UA454617 = {S77135, 65, { &P15364,&P18110,&P18329,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:11.0) Gecko/X Thunderbird/11.0.1*/
const Device UA690993 = {S77136, 134, { &P21067,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; OnePAD 970 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA907289 = {S77137, 129, { &P22814,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PC906 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1990778 = {S77138, 129, { &P22814,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PC906 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2029816 = {S77139, 140, { &P18620,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; MID7022 Build/GreenDemon_v1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2430234 = {S77140, 139, { &P18216,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Sony Tablet P Build/TISU0144) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2257444 = {S77141, 137, { &P22033,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SP-140 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555758 = {S77142, 137, { &P22681,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; UMI X1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2783869 = {S77143, 153, { &P15364,&P7133,&P22776,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.5.3.X (Second Life Release; default skin) Safari/533.3*/
const Device UA2679664 = {S77144, 136, { &P18653,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P870 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2711303 = {S77145, 137, { &P22609,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6312 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2995525 = {S77146, 130, { &P22609,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6312 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2799359 = {S77147, 158, { &P12798,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT18i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2865825 = {S77148, 158, { &P12798,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; LT18i Build/4.1.A.0.562) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2774140 = {S77149, 161, { &P18382,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire C Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2755536 = {S77150, 138, { &P20517,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CINK PEAX Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892863 = {S77151, 138, { &P20517,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; CINK PEAX Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2796926 = {S77152, 148, { &P23407,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BLU Quattro 5.7HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022867 = {S77153, 138, { &P22590,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021135 = {S77154, 140, { &P24480,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; GT-I8262B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3083144 = {S77155, 140, { &P18662,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC J Z321e Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3057498 = {S77156, 103, { &P13552,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac6-00) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3060838 = {S77157, 94, { &P13552,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC6-00) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3083769 = {S77158, 138, { &P21507,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C1605 Build/11.3.A.2.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2256 = {S77159, 132, { &P11938,&P17486,&P17470,&P18092 }};	/*Useragent - MOT-E8/R6713_G_71.03.05R Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 [es-co]*/
const Device UA2257 = {S77160, 124, { &P11938,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.14.1ER BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA18814 = {S77161, 126, { &P11938,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.14.1ER_B BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA18816 = {S77162, 124, { &P11938,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.01.2DR BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA18817 = {S77163, 124, { &P11938,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.02.07R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA18819 = {S77164, 124, { &P11938,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.14.0EI BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA18823 = {S77165, 125, { &P11938,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/Final Myth Rokr E8 BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA18824 = {S77166, 129, { &P11938,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/Final Myth Rokr E8 SP1 BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA18825 = {S77167, 124, { &P11938,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.04.06R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20643 = {S77168, 122, { &P11938,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/Nik_Stallion_E8 BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20644 = {S77169, 124, { &P11938,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.14.1AR BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA35808 = {S77170, 124, { &P11938,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.14.1FI BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA35807 = {S77171, 124, { &P11938,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.01.2ER BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA39243 = {S77172, 122, { &P11938,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6718_LIZU_V1.0 BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA39248 = {S77173, 124, { &P11938,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.14.18R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54916 = {S77174, 126, { &P11938,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.14.1CR_A BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62478 = {S77175, 124, { &P11938,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.01.8BR BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62580 = {S77176, 124, { &P11938,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.01.2FR BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62891 = {S77177, 124, { &P11938,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8/R6713_G_71.14.1CR BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA89097 = {S77178, 131, { &P11938,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-E8_CMCC/R6718_G_71.10.13R_C BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2672 = {S77179, 78, { &P12558,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_Touch Cruise Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA2677 = {S77180, 99, { &P12558,&P5448,&P22270,&P18092 }};	/*Useragent - HTC-P3650-orange/PPC; 240x320; OpVer 28.319.2.732 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA21957 = {S77181, 129, { &P12558,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC_P3650/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22386 = {S77182, 84, { &P12558,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_Touch_Cruise_T4242 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA61757 = {S77183, 71, { &P12558,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_P3650 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA55429 = {S77184, 105, { &P13603,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-T479B/T479UGKB1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA24737 = {S77185, 70, { &P12558,&P5448,&P22271,&P18092 }};	/*Useragent - HTC_P3650 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA44518 = {S77186, 105, { &P12558,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC P3650; OpVer 28.129.15.751*/
const Device UA60651 = {S77187, 122, { &P12558,&P5448,&P22271,&P18092 }};	/*Useragent - HTC_P3650t/1.27.621.1 Mozilla/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA87424 = {S77188, 98, { &P12558,&P5448,&P22271,&P18092 }};	/*Useragent - HTC-P3650-orange/PPC; 240x320; OpVer 28.129.2.732 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA87846 = {S77189, 128, { &P12558,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC_P3650/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62408 = {S77190, 110, { &P15689,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-GT-C3630/1.0 SHP/VPP/R5 NetFront/3.4 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA69186 = {S77191, 85, { &P13999,&P5448,&P4779,&P18092 }};	/*Useragent - HTC_HD2_T8585 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA724386 = {S77192, 81, { &P13999,&P5448,&P4779,&P18092 }};	/*Useragent - HD2_T8585 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA2293938 = {S77193, 116, { &P13999,&P5448,&P4779,&P18092 }};	/*Useragent - PPC; HTC_HD2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA243895 = {S77194, 129, { &P15364,&P7133,&P22408,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.2.0.0 Chrome/13.0.782.218 Safari/535.1*/
const Device UA244638 = {S77195, 122, { &P15364,&P7133,&P22408,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.2.0.0 Chrome/13.0.782.218 Safari/535.1*/
const Device UA644225 = {S77196, 134, { &P13941,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; nl-nl; MID-S7 Build/GR144) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2486287 = {S77197, 142, { &P13941,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Connspeed_AS7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2831576 = {S77198, 130, { &P13941,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; epade S7 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2954230 = {S77199, 142, { &P13941,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Hiya Smart S7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2981578 = {S77200, 137, { &P13941,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; epade S7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2437615 = {S77201, 141, { &P19846,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SO-05D Build/7.0.D.1.130) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2225146 = {S77202, 96, { &P18506,&P5043,&P17347,&P18092 }};	/*Useragent - Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia305/UC Browser8.0.3.107/69/444*/
const Device UA2261432 = {S77203, 131, { &P21990,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; chagall Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2421902 = {S77204, 145, { &P22125,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; INFINITY_LOTUS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2427412 = {S77205, 152, { &P22125,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GIONEE INFINITY_LOTUS Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2557832 = {S77206, 97, { &P18652,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A87) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2730249 = {S77207, 138, { &P23251,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ZTE V807 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2773710 = {S77208, 154, { &P22582,&P18598,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2897548 = {S77209, 93, { &P18673,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S7562) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2899411 = {S77210, 161, { &P21020,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Micromax A116 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2957905 = {S77211, 62, { &P22851,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT919 Build/2_290_2012)*/
const Device UA2958455 = {S77212, 140, { &P22851,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT919 Build/2_290_2012) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3001946 = {S77213, 140, { &P22851,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT919 Build/2_290_2005) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088190 = {S77214, 140, { &P22851,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT919 Build/2_290_2004) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2973650 = {S77215, 137, { &P21385,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F240S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976021 = {S77216, 134, { &P20213,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; N9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3026339 = {S77217, 158, { &P12968,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT26i Build/6.2.B.0.200) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3108656 = {S77218, 157, { &P12968,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT26i Build/6.2.B.1.96) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3064595 = {S77219, 157, { &P18296,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC One V Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3096088 = {S77220, 113, { &P23522,&P20706,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia501/10.0.2; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA1972991 = {S77221, 152, { &P11702,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB300 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2746505 = {S77222, 87, { &P17968,&P18111,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1.1; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.9.201 Version/11.50*/
const Device UA2766157 = {S77223, 87, { &P17968,&P18111,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1.2; Linux; Opera Mobi/ADR-X; U; nb) Presto/2.9.201 Version/11.50*/
const Device UA2390395 = {S77224, 98, { &P12080,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s3653) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2382196 = {S77225, 178, { &P12989,&P5843,&P24672,&P18092 }};	/*Useragent - Nokia303/5.0 (14.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia303) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2384933 = {S77226, 151, { &P20290,&P18066,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A11 Build/KarbonnA11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469869 = {S77227, 171, { &P13974,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2485212 = {S77228, 171, { &P13974,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2504580 = {S77229, 147, { &P17174,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 10 FHD Build/HuaweiMediaPad) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2561455 = {S77230, 130, { &P21895,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LT8036 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2567579 = {S77231, 140, { &P15397,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST18i Build/4.1.B.0.587) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602705 = {S77232, 147, { &P23366,&P5629,&P21729,&P18092 }};	/*Useragent - OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; W30 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2605958 = {S77233, 156, { &P18680,&P18066,&P21707,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2666360 = {S77234, 164, { &P15555,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; myTouch 4G Slide Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768743 = {S77235, 165, { &P15555,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MyTouch 4G Slide Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2670466 = {S77236, 122, { &P12280,&P18432,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/2.0*/
const Device UA2736514 = {S77237, 124, { &P12280,&P18432,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/2.0.1*/
const Device UA2799483 = {S77238, 124, { &P12280,&P18432,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/2.0.2*/
const Device UA2863696 = {S77239, 124, { &P12280,&P18432,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Flipboard/2.0.1*/
const Device UA2937988 = {S77240, 124, { &P12280,&P18432,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Flipboard/2.0.2*/
const Device UA2964080 = {S77241, 124, { &P12280,&P18432,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/2.0.3*/
const Device UA3010687 = {S77242, 124, { &P12280,&P18432,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/2.0.4*/
const Device UA3055485 = {S77243, 124, { &P12280,&P18432,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Flipboard/2.0.4*/
const Device UA2670884 = {S77244, 139, { &P16319,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; U8860 Build/HuaweiU8860) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2718569 = {S77245, 161, { &P16436,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I8150 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2896859 = {S77246, 162, { &P20289,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC6435LVW Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/dlx*/
const Device UA2769097 = {S77247, 142, { &P22914,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Cherry Sonic  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2794951 = {S77248, 154, { &P20877,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; ALCATEL ONE TOUCH 8000D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3021448 = {S77249, 93, { &P18321,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5300) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3031136 = {S77250, 156, { &P20983,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9308 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA8585 = {S77251, 107, { &P13684,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Toshiba/TG01/02/ PPC; 480x800*/
const Device UA10075 = {S77252, 115, { &P13684,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Toshiba-TG01-orange/ 02/ PPC; 480x800*/
const Device UA16278 = {S77253, 104, { &P13684,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Toshiba/TG01/ PPC; 480x800*/
const Device UA45233 = {S77254, 115, { &P13684,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Toshiba-TG01-orange/ 01/ PPC; 480x800*/
const Device UA89116 = {S77255, 90, { &P13684,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Toshiba/TG01*/
const Device UA13545 = {S77256, 132, { &P11760,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GT500-v10h Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA13546 = {S77257, 132, { &P11760,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GT500-v10k Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA13547 = {S77258, 132, { &P11760,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GT500-v10j Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA13548 = {S77259, 132, { &P11760,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GT500-v10f Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA13549 = {S77260, 132, { &P11760,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GT500-v10b Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA20466 = {S77261, 132, { &P11760,&P17483,&P3728,&P18092 }};	/*Useragent - LG/GT500/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA36746 = {S77262, 141, { &P11760,&P17483,&P3728,&P18092 }};	/*Useragent - LG/GT500-Bouygues/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63592 = {S77263, 132, { &P11760,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GT500/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA66690 = {S77264, 132, { &P11760,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GT500-v10e Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA66691 = {S77265, 132, { &P11760,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GT500-v10i Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA66692 = {S77266, 132, { &P11760,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GT500-v10d Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA66693 = {S77267, 132, { &P11760,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GT500-v10c Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA49928 = {S77268, 123, { &P13107,&P5043,&P17433,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-06/06.52; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.1.21.8.4*/
const Device UA627931 = {S77269, 80, { &P17595,&P17296,&P18367,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:12.0) Gecko/X Thunderbird/12.0.1*/
const Device UA631736 = {S77270, 103, { &P15364,&P18145,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA677646 = {S77271, 102, { &P15364,&P18145,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA734235 = {S77272, 70, { &P15364,&P7133,&P17208,&P18163 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:2.0.1) Gecko/X Firefox/6.0.1/Nutch-1.4*/
const Device UA905663 = {S77273, 141, { &P20183,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MI-ONE Plus Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2029240 = {S77274, 142, { &P20183,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  MI-ONE Plus Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA2445432 = {S77275, 136, { &P20183,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MI-ONE Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1882291 = {S77276, 143, { &P17595,&P18120,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (BD-C5500; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5*/
const Device UA2155013 = {S77277, 144, { &P17595,&P18120,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Stopper; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA2336448 = {S77278, 144, { &P17595,&P18120,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en_US) AppleWebKit/533.19.4 (KHTML, like Gecko) Dreamweaver/12.0.0.5808 Version/5.0.3 Safari/533.19.4*/
const Device UA1957945 = {S77279, 73, { &P20206,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; ZTE_N880E; 480*800) UCWEB7.9.3.103/139/32873*/
const Device UA2683790 = {S77280, 73, { &P20206,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; ZTE_N880E; 480*800) UCWEB7.9.3.103/139/32873*/
const Device UA1995986 = {S77281, 138, { &P17324,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9001 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2233035 = {S77282, 174, { &P15501,&P5477,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E16i Build/2.1.1.A.0.53) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2234222 = {S77283, 177, { &P12871,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2832583 = {S77284, 177, { &P12871,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; HTC Wildfire S A510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2168485 = {S77285, 57, { &P20030,&P22784,&P21795,&P18092 }};	/*Useragent - MicromaxX454/MstarBrowser1.0/MIDP2.0Configuration/CLDC1.1*/
const Device UA2353883 = {S77286, 118, { &P14453,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE51) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/355/UCWEB Mobile*/
const Device UA2262899 = {S77287, 117, { &P20082,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2350B/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA2347047 = {S77288, 128, { &P20379,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Kurio10 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2350670 = {S77289, 30, { &P23087,&P22835,&P17470,&P18092 }};	/*Useragent - Spice_M-5900/WapBrowserProfile*/
const Device UA2353982 = {S77290, 136, { &P23664,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; TSM-T8i Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2422801 = {S77291, 159, { &P18442,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N7100 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469481 = {S77292, 159, { &P18442,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I317 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2486819 = {S77293, 159, { &P18442,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L900 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2456932 = {S77294, 103, { &P12181,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.128; en-us; 9530/4.2.0.128) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2503182 = {S77295, 103, { &P12181,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.179; en-us; 9530/4.2.0.179) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2578231 = {S77296, 103, { &P12181,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.456; en-us; 9530/4.2.0.456) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2462486 = {S77297, 144, { &P19781,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 97 TITANIUMHD Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2486952 = {S77298, 151, { &P15785,&P5843,&P24675,&P18092 }};	/*Useragent - Nokia201/2.0 (11.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia201) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2505851 = {S77299, 165, { &P15798,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Explorer A310e Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659699 = {S77300, 58, { &P15364,&P22308,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; WinNT3.51; Trident/5.0)*/
const Device UA2791865 = {S77301, 58, { &P15364,&P22308,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; WinNT3.51; Trident/5.0)*/
const Device UA2708669 = {S77302, 137, { &P21350,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8730 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3102374 = {S77303, 130, { &P21350,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8730 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2753874 = {S77304, 142, { &P19843,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE BLADE III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783411 = {S77305, 138, { &P19843,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Blade III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2997349 = {S77306, 141, { &P19843,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BLADE III_IL Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755074 = {S77307, 136, { &P20937,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR910L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2938710 = {S77308, 136, { &P20937,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR910L Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769701 = {S77309, 135, { &P22845,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; eZee'TAB706 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2894854 = {S77310, 162, { &P20134,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5830i Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2974108 = {S77311, 136, { &P14574,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus S Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975561 = {S77312, 136, { &P22929,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_E10310 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974322 = {S77313, 133, { &P21963,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Q800 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3081082 = {S77314, 131, { &P22920,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; WL-101GQC Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080311 = {S77315, 137, { &P22601,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300L Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081613 = {S77316, 133, { &P17172,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Transformer Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126536 = {S77317, 139, { &P17172,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Transformer TF101 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094337 = {S77318, 156, { &P16356,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N7000 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA5861 = {S77319, 101, { &P12325,&P5448,&P4808,&P18092 }};	/*Useragent - HP_KB1/0.90 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC;320x240*/
const Device UA30502 = {S77320, 106, { &P12325,&P5448,&P4808,&P18092 }};	/*Useragent - HP_KB1/1.02_1.02 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC;320x240*/
const Device UA1994086 = {S77321, 150, { &P11758,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia6820/2.0 (4.83) Profile/MIDP-1.0 Configuration/CLDC-1.0 AppEngine-Google; (+http://code.google.com/appengine; appid: s~google.com:mobile-checker)*/
const Device UA19429 = {S77322, 87, { &P14588,&P17485,&P3499,&P18092 }};	/*Useragent - SonyEricssonW100a/R1CA009 TelecaBrowser/Q07C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA259593 = {S77323, 58, { &P15364,&P6704,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows 98; U; en) Presto/2.5.24 Version/10.53*/
const Device UA548988 = {S77324, 58, { &P15364,&P6704,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows 98; U; en) Presto/2.5.24 Version/10.52*/
const Device UA705208 = {S77325, 58, { &P15364,&P6704,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows 98; U; en) Presto/2.5.22 Version/10.51*/
const Device UA802839 = {S77326, 58, { &P15364,&P6704,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows 98; U; en) Presto/2.5.24 Version/10.54*/
const Device UA2131149 = {S77327, 58, { &P15364,&P6704,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows 98; U; en) Presto/2.5.22 Version/10.50*/
const Device UA496014 = {S77328, 83, { &P17595,&P18120,&P22499,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.5b) Gecko/X Camino/0.7+*/
const Device UA834954 = {S77329, 139, { &P22010,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0UI; en-in; i7218 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA886752 = {S77330, 122, { &P15364,&P5690,&P22429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 Comodo_Dragon/19.0.3.0*/
const Device UA1973414 = {S77331, 136, { &P15842,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; X310e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2023332 = {S77332, 138, { &P18002,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A760S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2030414 = {S77333, 136, { &P22035,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; VORTEX COLOR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2232557 = {S77334, 96, { &P15364,&P5690,&P21778,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.3.5 Safari/534.34*/
const Device UA2177291 = {S77335, 141, { &P21094,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A789 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2401797 = {S77336, 132, { &P15985,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2449364 = {S77337, 132, { &P15985,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P6200 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2462056 = {S77338, 177, { &P12871,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2530413 = {S77339, 180, { &P18510,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; ALCATEL ONE TOUCH 918N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3021401 = {S77340, 184, { &P18510,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; ALCATEL_ONE_TOUCH_918N Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2531091 = {S77341, 140, { &P21413,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Qmobile S13 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2585177 = {S77342, 130, { &P15919,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P3100 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2633924 = {S77343, 131, { &P15919,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P3100 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2636988 = {S77344, 137, { &P15919,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P3100 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2737615 = {S77345, 148, { &P21911,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; AMAZE S200 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2738037 = {S77346, 148, { &P22910,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MyPhone A818g Duo Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3120845 = {S77347, 70, { &P22910,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; MyPhone A818g Duo Build/JRO03C)*/
const Device UA2754794 = {S77348, 130, { &P18702,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8013 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2886813 = {S77349, 146, { &P23567,&P5629,&P15760,&P18092 }};	/*Useragent - Bmobile_AX530 Linux/2.6.35.7 Android/2.3.6 Release/10.10.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1*/
const Device UA2942548 = {S77350, 135, { &P22126,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Torque Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2976053 = {S77351, 142, { &P22749,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-04E Build/10.1.1.D.2.31) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974076 = {S77352, 138, { &P22591,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976701 = {S77353, 138, { &P22592,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717R Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2969594 = {S77354, 142, { &P19843,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE Blade III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36*/
const Device UA2976058 = {S77355, 142, { &P19843,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE BLADE III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022991 = {S77356, 138, { &P19843,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Blade III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3027546 = {S77357, 156, { &P19853,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7102 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3036653 = {S77358, 156, { &P19853,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7102 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA4254 = {S77359, 75, { &P15364,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) T-Mobile_Atlas*/
const Device UA9813 = {S77360, 119, { &P15364,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; MDA Touch/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21651 = {S77361, 73, { &P15364,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) ASUSP750/1.0*/
const Device UA21824 = {S77362, 95, { &P15364,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Vodafone/1.0/VPA_Touch/2.24.162.52*/
const Device UA22313 = {S77363, 124, { &P15364,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; MDA Touch Plus/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA38661 = {S77364, 94, { &P15364,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Vodafone/1.0/SFR_v3650/1.25.163.3*/
const Device UA57958 = {S77365, 74, { &P15364,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) i-mate JASJAM*/
const Device UA87372 = {S77366, 81, { &P15364,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) i-mate 9502; 480x640*/
const Device UA487801 = {S77367, 91, { &P15364,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 480x640; X650/2.0 Profile*/
const Device UA32183 = {S77368, 145, { &P12478,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; es-es; Orange_Boston Build/DONUT) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA32717 = {S77369, 138, { &P12478,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; pt-pt; Boston Build/DONUT) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA32037 = {S77370, 74, { &P12101,&P7063,&P3606,&P18092 }};	/*Useragent - BlackBerry8703e/4.1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105*/
const Device UA219589 = {S77371, 141, { &P14574,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Google Nexus S Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA576587 = {S77372, 134, { &P14574,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus S Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA638747 = {S77373, 134, { &P14574,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Nexus S Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA661834 = {S77374, 141, { &P14574,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Google Nexus S Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA778065 = {S77375, 141, { &P14574,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Google Nexus S Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1937829 = {S77376, 129, { &P14574,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Google Nexus S Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA567327 = {S77377, 225, { &P13988,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; zh-cn; HTCA9188_TD/1.0 Android/2.3 release/4.1.2011 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019832 = {S77378, 135, { &P13988,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; zh-cn; HTC A9188 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020508 = {S77379, 123, { &P13988,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; zh-cn; HTC A9188 Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2025967 = {S77380, 128, { &P13988,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; zh-cn; HTC A9188 Build/GRI40) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA767856 = {S77381, 142, { &P15228,&P5043,&P24667,&P18092 }};	/*Useragent - NokiaC2-03/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac2-03) UCBrowser8.2.1.144/70/355/UCWEB Mobile*/
const Device UA646338 = {S77382, 61, { &P20039,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX288/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA776268 = {S77383, 70, { &P15060,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.3;Zh_cn;GT-S5660;320*480;)UCWEB7.8.0.95/139/352*/
const Device UA2030019 = {S77384, 70, { &P15060,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;GT-S5660;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA869807 = {S77385, 100, { &P17595,&P17295,&P22480,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.0.1) Gecko/X Firefox/3.0.1 Flock/2.0b2*/
const Device UA899550 = {S77386, 82, { &P15364,&P18110,&P23681,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9a1) Gecko/X Firefox/1.6a1 (Swiftfox)*/
const Device UA1963099 = {S77387, 57, { &P20031,&P22784,&P21795,&P18092 }};	/*Useragent - MicromaxX276/MstarBrowser1.0/MIDP2.0Congiguration/CLDC1.1*/
const Device UA2222569 = {S77388, 115, { &P14848,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-00/11.00; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA2028712 = {S77389, 133, { &P22000,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TAB466EUK Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2233694 = {S77390, 169, { &P12616,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; WT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2040384 = {S77391, 93, { &P17595,&P17297,&P20755,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:15.0) Gecko/X Firefox/15.0.1 SeaMonkey/2.12.1*/
const Device UA2225023 = {S77392, 139, { &P21531,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S3+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3042470 = {S77393, 130, { &P21531,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S3+ Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2226653 = {S77394, 130, { &P21321,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9228 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2258089 = {S77395, 139, { &P21321,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9228 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2981364 = {S77396, 61, { &P21321,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9228 Build/IMM76D)*/
const Device UA2401307 = {S77397, 142, { &P14482,&P5843,&P24668,&P18092 }};	/*Useragent - NokiaC3-01/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac3-01) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2386086 = {S77398, 31, { &P18506,&P5043,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia305/03.60)*/
const Device UA2398575 = {S77399, 110, { &P18113,&P17979,&P18245,&P18091 }};	/*Useragent - YahooMobileFinance/1.0 (Android Finance; 1.1.5) (cdma_spyder;motorola;DROID RAZR;4.0.4/6.7.2-180_DHD-16_M4-31)*/
const Device UA2423290 = {S77400, 106, { &P15364,&P17285,&P18106,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/99.0 Safari/537.1 YottaaMonitor*/
const Device UA3016245 = {S77401, 107, { &P15364,&P17285,&P18106,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/99.0 Safari/537.11 YottaaMonitor*/
const Device UA2558038 = {S77402, 139, { &P21367,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; ARCHOS 97 XENON Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2664697 = {S77403, 137, { &P12788,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; PG86100 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2843585 = {S77404, 138, { &P12788,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; PG86100 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2750376 = {S77405, 140, { &P23561,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; AMOI N828 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088954 = {S77406, 140, { &P23561,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; AMOI N828 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879678 = {S77407, 129, { &P20420,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; I10A-LE Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3062604 = {S77408, 139, { &P18451,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26ii Build/6.2.B.1.96) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3032693 = {S77409, 152, { &P13644,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3026828 = {S77410, 157, { &P18295,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; One X Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/evita*/
const Device UA3062481 = {S77411, 157, { &P18295,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; One X Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/evita*/
const Device UA3093605 = {S77412, 165, { &P18295,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC One X Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/endeavoru*/
const Device UA3125319 = {S77413, 165, { &P18295,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-gb; HTC One X Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/endeavoru*/
const Device UA3127362 = {S77414, 157, { &P18295,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; One X Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/evita*/
const Device UA3080901 = {S77415, 137, { &P21385,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F240S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083271 = {S77416, 126, { &P18733,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M532 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3083535 = {S77417, 137, { &P23691,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5280 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA62237 = {S77418, 73, { &P14388,&P17486,&P3534,&P18092 }};	/*Useragent - Motorola-V950 Obigo/Q05A1 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA251365 = {S77419, 122, { &P15364,&P5690,&P22412,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.2.2.0 Chrome/14.0.835.202 Safari/535.1*/
const Device UA676311 = {S77420, 31, { &P14188,&P5129,&P24668,&P18092 }};	/*Useragent - Nokia5233/UCWEB8.3.0.133/50/800*/
const Device UA2375852 = {S77421, 88, { &P14188,&P5129,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokia5233) UCBrowser8.3.0.154/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA905944 = {S77422, 139, { &P17968,&P24691,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; aokp_tenderloin Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2336541 = {S77423, 137, { &P21899,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Dell V04B Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3017030 = {S77424, 143, { &P21899,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Dell Streak Pro Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1996081 = {S77425, 130, { &P23931,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; BB EH7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2402049 = {S77426, 66, { &P18303,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.7; zh-cn; MT27i; 480*854) UCWEB7.9.0.94/139/352*/
const Device UA2842578 = {S77427, 66, { &P18303,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; MT27i; 480*854) UCWEB7.9.0.94/139/352*/
const Device UA2402193 = {S77428, 148, { &P14849,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I800 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2433073 = {S77429, 229, { &P18513,&P5629,&P17373,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/6.5.27431/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2466199 = {S77430, 150, { &P21400,&P17979,&P18062,&P18092 }};	/*Useragent - Lenovo S880i/S100 Linux/3.0.13 Android/4.0.3 Release/12.12.2011 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30*/
const Device UA2468435 = {S77431, 138, { &P19853,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7102 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2698786 = {S77432, 138, { &P19853,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; GT-N7102 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3121358 = {S77433, 138, { &P19853,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7102 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2531016 = {S77434, 176, { &P14225,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2660026 = {S77435, 166, { &P14225,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; Desire HD Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2663425 = {S77436, 166, { &P14225,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2580956 = {S77437, 139, { &P23879,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; 201K Build/117.1.1c00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2852001 = {S77438, 61, { &P23879,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; 201K Build/117.1.1c00)*/
const Device UA2940893 = {S77439, 139, { &P23879,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; 201K Build/118.0.1f00) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666678 = {S77440, 140, { &P22640,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Fly_IQ446 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2707285 = {S77441, 64, { &P21542,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; C6503 Build/10.1.A.1.434)*/
const Device UA2863331 = {S77442, 142, { &P21542,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; C6503 Build/10.1.A.1.434) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734395 = {S77443, 134, { &P18464,&P18110,&P22587,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Safari/535.19 Silk-Accelerated=false*/
const Device UA2734675 = {S77444, 159, { &P13124,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PantechP9070 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2735049 = {S77445, 145, { &P21264,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PadFone Infinity Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2737539 = {S77446, 138, { &P21264,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PadFone Infinity Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2784827 = {S77447, 158, { &P23905,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100P Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2784885 = {S77448, 158, { &P23905,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100P Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2755522 = {S77449, 135, { &P18470,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 101G10 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2769359 = {S77450, 124, { &P18017,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; K1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2772496 = {S77451, 132, { &P18017,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Ideapad K1 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2892925 = {S77452, 124, { &P18017,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; K1 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3020414 = {S77453, 143, { &P18466,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2967671 = {S77454, 171, { &P12605,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2974500 = {S77455, 136, { &P22955,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P714 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3082968 = {S77456, 133, { &P21472,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A111 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081330 = {S77457, 126, { &P20556,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A701 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3081196 = {S77458, 136, { &P18687,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P705 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094559 = {S77459, 137, { &P18687,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P708g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA9549 = {S77460, 98, { &P13953,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-F400/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA89725 = {S77461, 11, { &P16871,&P22784,&P17470,&P18092 }};	/*Useragent - MICROMAX Q5*/
const Device UA89939 = {S77462, 10, { &P16871,&P22784,&P17470,&P18092 }};	/*Useragent - MICROMAXQ5*/
const Device UA158702 = {S77463, 142, { &P13036,&P5267,&P3834,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8500L/1.0; U; Bada/1.0; es) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA190363 = {S77464, 143, { &P15364,&P17285,&P22402,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Comodo_Dragon/10.0.0.2 Chrome/10.0.648.204 Safari/534.16*/
const Device UA290135 = {S77465, 136, { &P15364,&P7133,&P17392,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) RockMelt/0.8.34.841 Chrome/6.0.472.63 Safari/534.3*/
const Device UA294083 = {S77466, 136, { &P15364,&P7133,&P17392,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) RockMelt/0.8.36.116 Chrome/7.0.517.44 Safari/534.7*/
const Device UA669247 = {S77467, 139, { &P15364,&P7133,&P17392,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) RockMelt/0.8.40.147 Chrome/8.0.552.231 Safari/534.10*/
const Device UA369555 = {S77468, 147, { &P15555,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; myTouch_4G_Slide Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA807695 = {S77469, 151, { &P15555,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC myTouch_4G_Slide Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027145 = {S77470, 147, { &P15555,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; myTouch 4G Slide Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031484 = {S77471, 147, { &P15555,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; myTouch_4G_Slide Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2758840 = {S77472, 147, { &P15555,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; myTouch_4G_Slide Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA462486 = {S77473, 87, { &P15364,&P7133,&P21960,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:2.0b2pre) Gecko/X Minefield/4.0b2pre*/
const Device UA480282 = {S77474, 75, { &P15364,&P7133,&P21960,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0b4pre) Gecko/X Minefield/4.0b4pre*/
const Device UA632771 = {S77475, 146, { &P15575,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867119 = {S77476, 147, { &P15575,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2870627 = {S77477, 147, { &P15575,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA722969 = {S77478, 115, { &P14714,&P5843,&P17438,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02/05.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15*/
const Device UA1962349 = {S77479, 116, { &P14714,&P5843,&P17438,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02.5/07.32; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.7*/
const Device UA1908498 = {S77480, 108, { &P17595,&P17295,&P22890,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) iCab/5.0 Safari/533.16*/
const Device UA2350299 = {S77481, 137, { &P13452,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2613271 = {S77482, 59, { &P13452,&P24697,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P1000 Build/MIUI)*/
const Device UA2618948 = {S77483, 135, { &P13452,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-; GT-P1000 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2619791 = {S77484, 140, { &P13452,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P1000 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 ????????? Safari/534.30*/
const Device UA2621252 = {S77485, 66, { &P13452,&P24697,&P18062,&P18092 }};	/*Useragent - Biblia/2.2.7 (Linux; U; Android 4.0.4; es-ES; GT-P1000 Build/MIUI)*/
const Device UA2383461 = {S77486, 145, { &P21871,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; POLYTRON W3430 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2598606 = {S77487, 145, { &P21871,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; POLYTRON W3430 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2396168 = {S77488, 142, { &P21867,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; zh-cn; K-Touch W806+ Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2478038 = {S77489, 135, { &P21867,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch W806+ Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2776441 = {S77490, 66, { &P21867,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; K-Touch W806+ Build/IMM76I)*/
const Device UA2982138 = {S77491, 144, { &P21867,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch W806+ Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2562721 = {S77492, 97, { &P14764,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; SonyEricssonU1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2952460 = {S77493, 98, { &P14764,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; Sony EricssonU1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2536469 = {S77494, 64, { &P15364,&P5690,&P18689,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:19.0) Gecko/X Firefox/19.0 slurp*/
const Device UA2597817 = {S77495, 70, { &P15364,&P5690,&P18689,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:19.0) Gecko/X Firefox/19.0 slurp slurp*/
const Device UA2540497 = {S77496, 140, { &P21264,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PadFone Infinity Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2540503 = {S77497, 146, { &P21264,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PadFone Infinity Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2601335 = {S77498, 156, { &P18283,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9300 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2584861 = {S77499, 140, { &P13018,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT11i Build/4.1.B.0.587) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2861106 = {S77500, 140, { &P13018,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MT11i Build/4.1.A.0.562) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2596554 = {S77501, 162, { &P14743,&P17485,&P24676,&P18092 }};	/*Useragent - SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj20i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2712039 = {S77502, 144, { &P22822,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; KPN Smart 300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2773811 = {S77503, 138, { &P14225,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Desire HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2944085 = {S77504, 158, { &P17625,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST25i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2944845 = {S77505, 158, { &P17625,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST25i Build/6.0.B.3.188) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3020868 = {S77506, 94, { &P15649,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570I) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3024190 = {S77507, 93, { &P15649,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3022605 = {S77508, 93, { &P20146,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9082) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3033336 = {S77509, 156, { &P14848,&P5843,&P24679,&P18092 }};	/*Useragent - NokiaX3-00/5.0 (11.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax3-00) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3059300 = {S77510, 137, { &P15575,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-gb; GT-i9100 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3061778 = {S77511, 59, { &P15575,&P21637,&P23997,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.3; GT-I9100 Build/JSS15Q)*/
const Device UA3110730 = {S77512, 137, { &P15575,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-gb; GT-I9100 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3126693 = {S77513, 137, { &P15575,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; es-es; GT-i9100 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3083127 = {S77514, 145, { &P18114,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Transformer Prime TF201 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3063442 = {S77515, 135, { &P22126,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Torque Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA28107 = {S77516, 83, { &P12220,&P5448,&P15792,&P18092 }};	/*Useragent - SAMSUNG-SGH-i600/WM534 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8)*/
const Device UA51835 = {S77517, 77, { &P12220,&P5448,&P15792,&P18092 }};	/*Useragent - SAMSUNG-SGH-i600ORANGE/BVGD2 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8)*/
const Device UA54182 = {S77518, 81, { &P12220,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) SAMSUNG-SGH-i600/1.0*/
const Device UA60956 = {S77519, 72, { &P12220,&P5448,&P15792,&P18092 }};	/*Useragent - SAMSUNG-SGH-i600TIM/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8)*/
const Device UA61077 = {S77520, 82, { &P12220,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) SAMSUNG-SGH-i600V/1.0*/
const Device UA61646 = {S77521, 83, { &P12220,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) SAMSUNG-SGH-i600/WM534*/
const Device UA63059 = {S77522, 61, { &P13334,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GD900 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA22170 = {S77523, 17, { &P13855,&P22839,&P17470,&P18092 }};	/*Useragent - ZTE-G S213/WAP2.0*/
const Device UA64422 = {S77524, 17, { &P13855,&P22839,&P17470,&P18092 }};	/*Useragent - ZTE-G-S213/WAP2.0*/
const Device UA2022801 = {S77525, 139, { &P14919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Pyramid Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2579098 = {S77526, 139, { &P14919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC_Pyramid Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2587292 = {S77527, 133, { &P14919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-tw; HTC_Pyramid Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari*/
const Device UA2588294 = {S77528, 139, { &P14919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_Pyramid Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3116818 = {S77529, 139, { &P14919,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-tw; HTC Pyramid Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA68795 = {S77530, 100, { &P14763,&P5776,&P4777,&P18092 }};	/*Useragent - NokiaN72/5.0819.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.7.1.88/27/444*/
const Device UA769953 = {S77531, 64, { &P15364,&P18144,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64; rv:9.0.1) Gecko/X Firefox/9.0.1*/
const Device UA889381 = {S77532, 133, { &P22028,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TM-MID780 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1993547 = {S77533, 81, { &P13324,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaN8-00)/UC Browser8.5.0.183/50/352/UCWEB Mobile*/
const Device UA2025690 = {S77534, 126, { &P23888,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; 5860A Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2029048 = {S77535, 133, { &P23888,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; 5860A Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2412362 = {S77536, 126, { &P23888,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; 5860A Build GRJ90) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2759322 = {S77537, 57, { &P23888,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; 5860A Build/GRJ90)*/
const Device UA2235446 = {S77538, 169, { &P12798,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2154812 = {S77539, 138, { &P18219,&P5843,&P24670,&P18092 }};	/*Useragent - Nokia202/2.0 (20.28) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia202) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2384996 = {S77540, 57, { &P15216,&P7116,&P21731,&P18092 }};	/*Useragent - OneBrowser/for Blackberry3.0.0 (BlackBerry8520/5.0.0.592)*/
const Device UA2421628 = {S77541, 139, { &P20830,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MMU002D Build/MMU002D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422925 = {S77542, 155, { &P21140,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-pt; Optimus_Madrid Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2537089 = {S77543, 151, { &P18283,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9300 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2557330 = {S77544, 151, { &P18283,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9300 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2657906 = {S77545, 159, { &P23439,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LNV-Lenovo_A765e/S-3-02; 480*854; CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2728503 = {S77546, 143, { &P23439,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A765e Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2951655 = {S77547, 65, { &P23439,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo A765e Build/IMM76I)*/
const Device UA2658940 = {S77548, 140, { &P21563,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC_Z560e Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2792337 = {S77549, 140, { &P21563,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC Z560e Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044521 = {S77550, 131, { &P21563,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC Z560e Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2676590 = {S77551, 135, { &P20119,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_S9714 Build/JRO03R) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2716276 = {S77552, 159, { &P12616,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; WT19i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2958432 = {S77553, 159, { &P12616,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; WT19i Build/4.0.2.A.0.58) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2800381 = {S77554, 156, { &P22607,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S5303 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2738341 = {S77555, 153, { &P18673,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2784754 = {S77556, 159, { &P18448,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST26i Build/11.2.A.0.21) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2774069 = {S77557, 156, { &P18243,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P880 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2769389 = {S77558, 139, { &P18453,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT25i Build/9.0.D.0.484) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869197 = {S77559, 139, { &P23410,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; iris458Q Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892409 = {S77560, 170, { &P22597,&P18598,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I545 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SCH-I545, 47, 1.9.7)*/
const Device UA2945603 = {S77561, 94, { &P15649,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570I) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2945609 = {S77562, 93, { &P15649,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3022379 = {S77563, 90, { &P15989,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3026883 = {S77564, 91, { &P15989,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i2) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3105736 = {S77565, 87, { &P15989,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; id; ST21i) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Ponsel*/
const Device UA3033945 = {S77566, 163, { &P22594,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-M919 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltetmo*/
const Device UA3063529 = {S77567, 163, { &P22594,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-M919 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltetmo*/
const Device UA3130198 = {S77568, 163, { &P22594,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-M919 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltetmo*/
const Device UA3036544 = {S77569, 97, { &P21475,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A35) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3081343 = {S77570, 138, { &P22756,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C1604 Build/11.3.A.2.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA9410 = {S77571, 98, { &P13412,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-GT-S7330/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA30806 = {S77572, 51, { &P14497,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-Lightpipe/ UP.Browser/7.2.7.2.531 (GUI) MMP/2.0*/
const Device UA53633 = {S77573, 55, { &P14497,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-Lightpipe/1002 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0*/
const Device UA63294 = {S77574, 55, { &P14497,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-Lightpipe/1003 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0*/
const Device UA250513 = {S77575, 131, { &P17595,&P17296,&P22452,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1*/
const Device UA409844 = {S77576, 123, { &P15364,&P5690,&P22423,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.4.1.0 Chrome/17.0.963.56 Safari/535.11*/
const Device UA827421 = {S77577, 161, { &P13201,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ERIS Build/GRI40; GSB.v4.5-FINAL_ODEXED_CM7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030020 = {S77578, 151, { &P13201,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ERIS Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA447727 = {S77579, 188, { &P15364,&P7133,&P4759,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.6.1.82/69/351*/
const Device UA523541 = {S77580, 188, { &P15364,&P7133,&P4759,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.6.1.82/50/351*/
const Device UA529275 = {S77581, 188, { &P15364,&P7133,&P4759,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.6.1.82/28/351*/
const Device UA545394 = {S77582, 188, { &P15364,&P7133,&P4759,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.6.1.82/50/355*/
const Device UA2132987 = {S77583, 188, { &P15364,&P7133,&P4759,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.6.1.82/50/352*/
const Device UA2278946 = {S77584, 188, { &P15364,&P7133,&P4759,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.6.1.82/28/352*/
const Device UA447905 = {S77585, 89, { &P17968,&P5096,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2.2; Linux; Opera Mobi/ADR-X; U; en-GB) Presto/2.7.81 Version/11.00*/
const Device UA793880 = {S77586, 84, { &P17968,&P5096,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2; Linux; Opera Mobi/ADR-X; U; de) Presto/2.7.81 Version/11.00*/
const Device UA855579 = {S77587, 89, { &P17968,&P5096,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2.1; Linux; Opera Mobi/ADR-X; U; es-ES) Presto/2.7.81 Version/11.00*/
const Device UA471298 = {S77588, 113, { &P15364,&P7133,&P22442,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/528.7 (KHTML, like Gecko) Iron/1.0.155.0 Safari/528.7*/
const Device UA623200 = {S77589, 136, { &P21699,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; TS1 Duo Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1863346 = {S77590, 100, { &P15364,&P5690,&P20444,&P18092 }};	/*Useragent - Mozilla/8.0 (Windows; U; Windows NT 5.1; pl; rv:1.8.1.8) Gecko/X Firefox/3.8.8 (.NET CLR 3.3.38888)"*/
const Device UA699282 = {S77591, 49, { &P12193,&P5890,&P17347,&P18092 }};	/*Useragent - nokiac5-00.2/UC Browser8.0.3.107/69/444  3gpp-gba*/
const Device UA2392782 = {S77592, 47, { &P12193,&P5890,&P17347,&P18092 }};	/*Useragent - nokiac5-00/UC Browser8.0.3.107/69/444  3gpp-gba*/
const Device UA785731 = {S77593, 64, { &P15364,&P7127,&P18367,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:12.0) Gecko/X Thunderbird/12.0.1*/
const Device UA812173 = {S77594, 99, { &P17595,&P17296,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:7.0.1) Gecko/X Firefox/7.0.1 AlexaToolbar/alxf-2.15*/
const Device UA1960516 = {S77595, 84, { &P12588,&P5055,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; NokiaC6-01)/UC Browser8.5.0.183/51/352/UCWEB Mobile*/
const Device UA2339580 = {S77596, 32, { &P12588,&P5055,&P24671,&P18092 }};	/*Useragent - NokiaC6-01/UCWEB8.5.0.163/51/800*/
const Device UA1971811 = {S77597, 133, { &P22000,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TAB466EUK Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1984159 = {S77598, 199, { &P24465,&P22784,&P18373,&P18092 }};	/*Useragent - MicromaxX640ASTRO36_TD/v3 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA2159925 = {S77599, 162, { &P12139,&P5204,&P24668,&P18092 }};	/*Useragent - Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia5310xpressmusic) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2170829 = {S77600, 166, { &P13526,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; T-Mobile myTouch 4G Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2365059 = {S77601, 166, { &P13526,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; T-Mobile myTouch 4G Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2421920 = {S77602, 157, { &P13526,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; myTouch 4G Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422885 = {S77603, 166, { &P13526,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; T-Mobile myTouch 4G Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2503026 = {S77604, 157, { &P13526,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; myTouch 4g Build/JZO54K; CyanogenMod-CM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2428425 = {S77605, 135, { &P23933,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; DISTAB9000R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2458329 = {S77606, 140, { &P22027,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Videocon A20 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2467679 = {S77607, 130, { &P22178,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 792TPC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2671079 = {S77608, 140, { &P20479,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenco CoolTab-70 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2736680 = {S77609, 97, { &P22622,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Micromax_A72) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2774166 = {S77610, 161, { &P18462,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2769027 = {S77611, 143, { &P23021,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Spice Mi-535 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2782763 = {S77612, 150, { &P16292,&P18598,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2867748 = {S77613, 151, { &P16292,&P18598,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2885656 = {S77614, 140, { &P19764,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6502 Build/10.3.A.0.423) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2975935 = {S77615, 137, { &P18747,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A840S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021241 = {S77616, 138, { &P18747,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A840SP Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021262 = {S77617, 93, { &P18513,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6802) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3115108 = {S77618, 156, { &P21261,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3094454 = {S77619, 161, { &P15575,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109572 = {S77620, 161, { &P15575,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3110401 = {S77621, 161, { &P15575,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-I9100 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127412 = {S77622, 162, { &P15575,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9100G Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA62124 = {S77623, 61, { &P16153,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia6267/2.0 (03.25) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA54576 = {S77624, 112, { &P14239,&P5208,&P4628,&P18092 }};	/*Useragent - POLARIS/6.01 (BREW 3.1.5; U; en-us; LG; LX370; POLARIS/6.01/AMB) MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63635 = {S77625, 61, { &P14529,&P6909,&P4030,&P18092 }};	/*Useragent - Nokia3120/1.0 (06.31) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA213303 = {S77626, 133, { &P12519,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB501 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA567891 = {S77627, 133, { &P12519,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB501 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA574310 = {S77628, 133, { &P12519,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB501 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA234844 = {S77629, 122, { &P15364,&P5690,&P22407,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.0.0.0 Chrome/13.0.782.107 Safari/535.1*/
const Device UA396257 = {S77630, 136, { &P21251,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; eGlide, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA745249 = {S77631, 168, { &P15575,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100 Build/IMM76I; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA908040 = {S77632, 168, { &P15575,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9100 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA906510 = {S77633, 78, { &P15364,&P17285,&P22360,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:12.2.1) Gecko/X Firefox/12.2.1 PaleMoon/12.2.1*/
const Device UA909540 = {S77634, 98, { &P15364,&P17285,&P22360,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:12.2.1) Gecko/X Firefox/12.2.1-x64 PaleMoon/12.2.1-x64*/
const Device UA1952084 = {S77635, 138, { &P22096,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; SPNG708 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027867 = {S77636, 129, { &P20595,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U18GT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2527741 = {S77637, 130, { &P20595,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; U18GT2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2817479 = {S77638, 131, { &P20595,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U18GT-W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2034535 = {S77639, 140, { &P23274,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; TAB-PROTAB2-IPS9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2232830 = {S77640, 171, { &P14877,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2762248 = {S77641, 171, { &P14877,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2790646 = {S77642, 171, { &P14877,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2031763 = {S77643, 140, { &P11747,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261547 = {S77644, 140, { &P11747,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2937189 = {S77645, 145, { &P11747,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Desire L by HTC Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2430723 = {S77646, 158, { &P12502,&P17485,&P24673,&P18092 }};	/*Useragent - SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; fa; sonyericssonw20i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2469511 = {S77647, 159, { &P12473,&P5204,&P24675,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia2730c-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2510221 = {S77648, 37, { &P18218,&P15924,&P20770,&P18092 }};	/*Useragent - NOKIA-Lumia 610/UCWEB2.8.0.249/47/355*/
const Device UA2567025 = {S77649, 131, { &P19874,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-i705 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2672260 = {S77650, 137, { &P14225,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Desire HD Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714372 = {S77651, 138, { &P14225,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Desire HD Build/JOP40G) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676339 = {S77652, 141, { &P11747,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire V Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2711001 = {S77653, 142, { &P11747,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire VC Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2797005 = {S77654, 133, { &P12280,&P18432,&P22329,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Rockmelt/3.0.3.7 Mobile/10A525 Safari/8536.25*/
const Device UA2849949 = {S77655, 58, { &P15364,&P17188,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; WinNT4.0; Trident/4.0)*/
const Device UA2731646 = {S77656, 101, { &P21407,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 928)*/
const Device UA2794662 = {S77657, 159, { &P14969,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; HTC Desire S Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2869320 = {S77658, 159, { &P14969,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Desire S Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2754070 = {S77659, 141, { &P19997,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo P700i Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2872986 = {S77660, 140, { &P19997,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Lenovo P700 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2770881 = {S77661, 162, { &P14877,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-I9000 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2771452 = {S77662, 162, { &P14877,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.3; GT-I9000 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2769537 = {S77663, 138, { &P14854,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773571 = {S77664, 138, { &P14854,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893005 = {S77665, 138, { &P14854,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2771850 = {S77666, 150, { &P20094,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825-1 Build/HuaweiU8825-1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2796682 = {S77667, 148, { &P20094,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2772623 = {S77668, 136, { &P18621,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E610 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877593 = {S77669, 136, { &P18621,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E612 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2938007 = {S77670, 137, { &P18621,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E612f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2781919 = {S77671, 139, { &P23128,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-us; M4TEL SS660 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2879203 = {S77672, 144, { &P23256,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Note II N7102 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2895412 = {S77673, 93, { &P18672,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5302) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2898180 = {S77674, 156, { &P22872,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5282 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3032081 = {S77675, 152, { &P12989,&P5843,&P24679,&P18092 }};	/*Useragent - Nokia303/5.0 (14.87) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia303) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3080828 = {S77676, 137, { &P21373,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F240K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA737965 = {S77677, 91, { &P15216,&P6173,&P4779,&P18092 }};	/*Useragent - BlackBerry8520/4.2.0.135 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/159/444*/
const Device UA648056 = {S77678, 87, { &P17976,&P6437,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.2; Linux; Opera Tablet/ADR-X; U; fr) Presto/2.9.201 Version/11.50*/
const Device UA706871 = {S77679, 89, { &P17976,&P6437,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.2.1; Linux; Opera Tablet/ADR-X; U; pl) Presto/2.9.201 Version/11.50*/
const Device UA804186 = {S77680, 89, { &P17976,&P6437,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.2.2; Linux; Opera Tablet/ADR-X; U; en) Presto/2.9.201 Version/11.50*/
const Device UA1955908 = {S77681, 89, { &P17976,&P6437,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.2.6; Linux; Opera Tablet/ADR-X; U; en) Presto/2.9.201 Version/11.50*/
const Device UA615223 = {S77682, 125, { &P15364,&P5690,&P22428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19 Comodo_Dragon/18.3.0.0*/
const Device UA719985 = {S77683, 130, { &P23823,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; M10 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA849657 = {S77684, 129, { &P21866,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; ViewPad7x Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2022899 = {S77685, 131, { &P21866,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; ViewPad7X Build/HTK55D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2814713 = {S77686, 129, { &P21866,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; ViewPad7X Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2356553 = {S77687, 139, { &P18222,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IS12S Build/6.1.D.1.74) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2223002 = {S77688, 161, { &P21347,&P5629,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; HUAWEI U8661 Build/HuaweiU8661) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2235004 = {S77689, 171, { &P14433,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2840814 = {S77690, 171, { &P14433,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2436475 = {S77691, 79, { &P15767,&P17779,&P17373,&P18092 }};	/*Useragent - Opera/9.80 (iPad; Opera Mini/6.5.2/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2466222 = {S77692, 66, { &P15047,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; SK17i; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2482884 = {S77693, 66, { &P15047,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; SK17i; 320*480) UCWEB7.9.0.94/139/352*/
const Device UA2457992 = {S77694, 154, { &P18442,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2655277 = {S77695, 154, { &P18442,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; SCH-N719 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2656658 = {S77696, 154, { &P18442,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2953422 = {S77697, 154, { &P18442,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-N719 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2458618 = {S77698, 142, { &P22003,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GOCLEVER TAB A93.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2461098 = {S77699, 219, { &P15876,&P18066,&P17373,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/6.5.28192/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2622823 = {S77700, 217, { &P15876,&P18066,&P17373,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/6.5.28192/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2530271 = {S77701, 142, { &P15880,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ME865 Build/6.7.2_GC-384) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957791 = {S77702, 142, { &P15880,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ME865 Build/6.7.2_GC-404) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2560468 = {S77703, 103, { &P15798,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Explorer_A310e) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2583393 = {S77704, 146, { &P18556,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD+ Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2566773 = {S77705, 139, { &P20608,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Edison Build/1.1.8 X-16:29) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2672447 = {S77706, 133, { &P18737,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; X909 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676848 = {S77707, 138, { &P22592,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717R Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2678416 = {S77708, 145, { &P18476,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I577 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2699867 = {S77709, 136, { &P23959,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; PROG40C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2719350 = {S77710, 151, { &P18666,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A25 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2755493 = {S77711, 130, { &P12879,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7510 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2938246 = {S77712, 130, { &P12879,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P7510 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2769812 = {S77713, 137, { &P22948,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P716 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2848797 = {S77714, 142, { &P23440,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Vodafone 975 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2957359 = {S77715, 133, { &P24078,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; E3 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3074267 = {S77716, 55, { &P24078,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; E3 Build/JOP40D)*/
const Device UA3002712 = {S77717, 85, { &P19779,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; id; A1+) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Ponsel*/
const Device UA3020936 = {S77718, 88, { &P19779,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A1+) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3019668 = {S77719, 143, { &P20004,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; Karbonn A27+ Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3021577 = {S77720, 140, { &P20004,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Karbonn A26 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3078001 = {S77721, 142, { &P20004,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Karbonn A26 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3026172 = {S77722, 93, { &P20146,&P18111,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9082) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3080942 = {S77723, 130, { &P22052,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7501 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3083132 = {S77724, 143, { &P19855,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT885 Build/6.7.3-135_YIC-8) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA64520 = {S77725, 101, { &P11820,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E251/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA87391 = {S77726, 102, { &P11820,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E251L/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA15956 = {S77727, 132, { &P13537,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GT405/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA15959 = {S77728, 139, { &P13537,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GT405-Orange/V10f Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA24732 = {S77729, 43, { &P15658,&P17017,&P3807,&P18092 }};	/*Useragent - Huawei/1.0/G7010/Browser/obigo-browser/q03c*/
const Device UA46961 = {S77730, 43, { &P15658,&P17017,&P3807,&P18092 }};	/*Useragent - Huawei/G7010/1.0/Browser/obigo-browser/q03c*/
const Device UA30863 = {S77731, 52, { &P12358,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-VE240/00.72 UP.Browser/7.2.7.5.541 (GUI) MMP/2.0*/
const Device UA32459 = {S77732, 52, { &P12358,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-VE240/00.72 UP.Browser/7.2.7.5.548 (GUI) MMP/2.0*/
const Device UA32460 = {S77733, 52, { &P12358,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-VE240/00.72 UP.Browser/7.2.7.5.599 (GUI) MMP/2.0*/
const Device UA55238 = {S77734, 47, { &P14301,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-VX5500/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA451561 = {S77735, 136, { &P14852,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; LG-SU370 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA899739 = {S77736, 122, { &P15364,&P5690,&P22430,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5 Comodo_Dragon/19.1.0.0*/
const Device UA1967559 = {S77737, 86, { &P16384,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s5380k) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2029206 = {S77738, 131, { &P18061,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; rk30sdk Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2452845 = {S77739, 144, { &P18061,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Full AOSP on Rk29sdk Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2191498 = {S77740, 135, { &P21288,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T06A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2520746 = {S77741, 130, { &P21288,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AT107F Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2360237 = {S77742, 71, { &P20039,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X288/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2527704 = {S77743, 155, { &P21595,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar Touch Kem 432m Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580245 = {S77744, 92, { &P18241,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; LG-E400) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2607493 = {S77745, 70, { &P15795,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 710;7.10;8862)*/
const Device UA2656103 = {S77746, 70, { &P15795,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;Nokia;Lumia 710;7.10;8858)*/
const Device UA2677155 = {S77747, 130, { &P22041,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6810 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2658999 = {S77748, 63, { &P15364,&P5448,&P22273,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; PPC; Tablet PC; Trident/4.0)*/
const Device UA2663317 = {S77749, 156, { &P15786,&P5043,&P24676,&P18092 }};	/*Useragent - NokiaC2-05/2.0 (08.69) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-05) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2729987 = {S77750, 93, { &P15919,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-P3100) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2737381 = {S77751, 165, { &P19998,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2862926 = {S77752, 167, { &P19998,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI C8812E Build/HuaweiC8812E) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2944708 = {S77753, 140, { &P20439,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SOL21 Build/9.1.D.0.401) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2769504 = {S77754, 138, { &P20871,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782664 = {S77755, 164, { &P18472,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3015918 = {S77756, 155, { &P18472,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-79) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3078228 = {S77757, 157, { &P18472,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-79-1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2975536 = {S77758, 145, { &P18114,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Transformer Prime TF201 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021675 = {S77759, 135, { &P23190,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Marquis_MP977 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA33292 = {S77760, 117, { &P12883,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2330B/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA44003 = {S77761, 70, { &P15599,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-T100/1.0 ACS-NF/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA72940 = {S77762, 117, { &P17475,&P5768,&P17338,&P18092 }};	/*Useragent - Nokia6300/2.0 (05.50) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6300/UC Browser7.9.0.102/70/352 Novarra-Vision/8.0*/
const Device UA371359 = {S77763, 103, { &P15851,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Radar; Orange)*/
const Device UA371361 = {S77764, 95, { &P15851,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Radar)*/
const Device UA688401 = {S77765, 101, { &P15851,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Radar C110e)*/
const Device UA765354 = {S77766, 98, { &P15851,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Radar 4G)*/
const Device UA439165 = {S77767, 28, { &P18182,&P6830,&P17470,&P18092 }};	/*Useragent - ZDM/4.0; Windows Mobile 7.0;*/
const Device UA682679 = {S77768, 86, { &P17595,&P17297,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:2.0b1) Gecko/X Firefox/4.0b1*/
const Device UA611542 = {S77769, 137, { &P20929,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; nl-nl; AT-AS43D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1945856 = {S77770, 152, { &P12519,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB501 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2368320 = {S77771, 135, { &P23695,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SurfTab_8.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2456332 = {S77772, 33, { &P14711,&P5204,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (NokiaC2-01/11.10)*/
const Device UA2563772 = {S77773, 171, { &P17324,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2580507 = {S77774, 171, { &P17324,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2580828 = {S77775, 171, { &P17324,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2600282 = {S77776, 171, { &P17324,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2583347 = {S77777, 133, { &P20518,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A10 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2708341 = {S77778, 126, { &P20518,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A10 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2595639 = {S77779, 137, { &P21523,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; SH930W Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2637644 = {S77780, 151, { &P22043,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2735223 = {S77781, 155, { &P22043,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA2676418 = {S77782, 137, { &P14877,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9000 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2746722 = {S77783, 103, { &P15236,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.137; en-us; 9930/5.1.0.137) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2747467 = {S77784, 103, { &P15236,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.553; en-us; 9930/5.1.0.553) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2846682 = {S77785, 103, { &P15236,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.429; en-us; 9930/5.1.0.429) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2983640 = {S77786, 103, { &P15236,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.230; en-us; 9930/5.1.0.230) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3124553 = {S77787, 103, { &P15236,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 5.1.0.678; en-us; 9930/5.1.0.678) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2773182 = {S77788, 145, { &P21489,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Infected Rezound Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2797407 = {S77789, 149, { &P21489,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Infected HTC Rezound Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877044 = {S77790, 148, { &P21489,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Rezound on TRON 2.1 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3021455 = {S77791, 88, { &P18666,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A25) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3031516 = {S77792, 160, { &P18223,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3083557 = {S77793, 134, { &P20480,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JY-G3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081060 = {S77794, 131, { &P20145,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; NS-13T001 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA20491 = {S77795, 79, { &P15095,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-T349/T349UVIC7 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA42381 = {S77796, 121, { &P14842,&P5477,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.1-update1;Zh_cn;E10i;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA1906101 = {S77797, 72, { &P14842,&P5477,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.1-update1;Zh_cn;E10i;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA53792 = {S77798, 52, { &P15364,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-RASPB/00.72 UP.Browser/7.2.7.5.622 (GUI) MMP/2.0*/
const Device UA60580 = {S77799, 52, { &P15364,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-VEGAS/00.72 UP.Browser/7.2.6.1.835 (GUI) MMP/2.0*/
const Device UA60985 = {S77800, 52, { &P15364,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-RASPB/00.72 UP.Browser/7.2.7.5.598 (GUI) MMP/2.0*/
const Device UA64231 = {S77801, 52, { &P15364,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-VEGAS/00.72 UP.Browser/7.2.6.1.841 (GUI) MMP/2.0*/
const Device UA89995 = {S77802, 52, { &P15364,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-DBLDV/00.72 UP.Browser/7.2.7.5.599 (GUI) MMP/2.0*/
const Device UA2320694 = {S77803, 60, { &P15364,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-A-A7/00.05 UP.Browser/7.2.7.2.542f (GUI) MMP/2.0 Push/PO*/
const Device UA2933482 = {S77804, 52, { &P15364,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-CRUSH/00.72 UP.Browser/7.2.7.5.678 (GUI) MMP/2.0*/
const Device UA59650 = {S77805, 59, { &P14919,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_europe / 1.0 / htc_pyramid / 2.3.3*/
const Device UA61547 = {S77806, 54, { &P14919,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player tmous / 1.0 / htc_pyramid / 2.3.3*/
const Device UA69973 = {S77807, 54, { &P14919,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player tmous / 1.0 / htc_pyramid / 2.3.4*/
const Device UA71537 = {S77808, 58, { &P14919,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player orange_uk / 1.0 / htc_pyramid / 2.3.3*/
const Device UA72723 = {S77809, 60, { &P14919,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player telstra_wwe / 1.0 / htc_pyramid / 2.3.3*/
const Device UA438860 = {S77810, 60, { &P14919,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player vodafone_uk / 1.0 / htc_pyramid / 2.3.4*/
const Device UA568642 = {S77811, 59, { &P14919,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_europe / 1.0 / htc_pyramid / 2.3.4*/
const Device UA1969927 = {S77812, 59, { &P14919,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_europe / 1.0 / htc_pyramid / 4.0.3*/
const Device UA2222871 = {S77813, 54, { &P14919,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player tmous / 1.0 / htc_pyramid / 4.0.3*/
const Device UA2743250 = {S77814, 61, { &P14919,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_asia_wwe / 1.0 / htc_pyramid / 4.0.3*/
const Device UA2374210 = {S77815, 54, { &P17581,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SH3J UP.Browser/6.2_7.2.7.1.K.5.212 (GUI) MMP/2.0*/
const Device UA616250 = {S77816, 148, { &P15364,&P17017,&P18373,&P18092 }};	/*Useragent - NexianNX-G967/MTK  Release/01.01.2011/MAUI Browser/Opera Profile/MIDP2.0 Configuration/CLDC-1 Opera/9.80 (MTK; U; id-ID) Presto/2.5.28 Version/10.10*/
const Device UA616272 = {S77817, 148, { &P15364,&P17017,&P18373,&P18092 }};	/*Useragent - NexianNX-G963/MTK  Release/01.01.2011/MAUI Browser/Opera Profile/MIDP2.0 Configuration/CLDC-1 Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA616273 = {S77818, 147, { &P15364,&P17017,&P18373,&P18092 }};	/*Useragent - NexianNX-G963/MTK Release/01.01.2011/MAUI Browser/Opera Profile/MIDP2.0 Configuration/CLDC-1 Opera/9.80 (MTK; U; id-ID) Presto/2.5.28 Version/10.10*/
const Device UA616319 = {S77819, 148, { &P15364,&P17017,&P18373,&P18092 }};	/*Useragent - NexianNX-G963/MTK  Release/01.01.2011/MAUI Browser/Opera Profile/MIDP2.0 Configuration/CLDC-1 Opera/9.80 (MTK; U; id-ID) Presto/2.5.28 Version/10.10*/
const Device UA616323 = {S77820, 148, { &P15364,&P17017,&P18373,&P18092 }};	/*Useragent - NexianNX-G967/MTK  Release/01.01.2011/MAUI Browser/Opera Profile/MIDP2.0 Configuration/CLDC-1 Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA895248 = {S77821, 194, { &P15364,&P7133,&P24670,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.4.0.159/50/351/UCWEB*/
const Device UA895425 = {S77822, 194, { &P15364,&P7133,&P24670,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.4.0.159/50/352/UCWEB*/
const Device UA898736 = {S77823, 194, { &P15364,&P7133,&P24670,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.4.0.159/28/352/UCWEB*/
const Device UA902918 = {S77824, 194, { &P15364,&P7133,&P24670,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.4.0.159/28/351/UCWEB*/
const Device UA1953462 = {S77825, 194, { &P15364,&P7133,&P24670,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.4.0.159/50/355/UCWEB*/
const Device UA1953656 = {S77826, 194, { &P15364,&P7133,&P24670,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.4.0.159/50/444/UCWEB*/
const Device UA2224109 = {S77827, 194, { &P15364,&P7133,&P24670,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.4.0.159/28/444/UCWEB*/
const Device UA2325519 = {S77828, 194, { &P15364,&P7133,&P24670,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.4.0.159/28/354/UCWEB*/
const Device UA2372028 = {S77829, 194, { &P15364,&P7133,&P24670,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0;SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.4.0.159/50/354/UCWEB*/
const Device UA2368084 = {S77830, 138, { &P18557,&P20706,&P24668,&P18092 }};	/*Useragent - Nokia308/2.0 (05.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia308) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2403152 = {S77831, 112, { &P19974,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia111/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2424510 = {S77832, 114, { &P19974,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia111.1/03.24; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2403174 = {S77833, 143, { &P20073,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tmn smart a15 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2422422 = {S77834, 139, { &P20073,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V880E Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2480535 = {S77835, 147, { &P13157,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Sensation XE Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955663 = {S77836, 158, { &P13157,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC_SensationXE_Beats_Z715e Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997665 = {S77837, 170, { &P13157,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC Sensation XE with Beats Audio Z715e Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422970 = {S77838, 132, { &P20106,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A5+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2482522 = {S77839, 71, { &P20327,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X336/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2635219 = {S77840, 158, { &P22037,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2636393 = {S77841, 151, { &P22037,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2866238 = {S77842, 155, { &P22037,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA2566751 = {S77843, 127, { &P20556,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A701 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2700273 = {S77844, 137, { &P21379,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SH631W Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2772195 = {S77845, 162, { &P18214,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-S7500 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2766105 = {S77846, 138, { &P24583,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; EK-GC110 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2773954 = {S77847, 139, { &P21605,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C2104 Build/15.0.A.1.31) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2791587 = {S77848, 137, { &P23234,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E976 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2799687 = {S77849, 146, { &P22597,&P18598,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I545 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2975968 = {S77850, 143, { &P15854,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ607-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022564 = {S77851, 138, { &P18386,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I757M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020937 = {S77852, 93, { &P15649,&P5096,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5570) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA2975719 = {S77853, 137, { &P23215,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SHV-E330S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973999 = {S77854, 93, { &P14984,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5250) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3061933 = {S77855, 160, { &P18382,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire C Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3029912 = {S77856, 141, { &P19778,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PantechP9090 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3054711 = {S77857, 134, { &P19778,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PantechP9090 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3034501 = {S77858, 154, { &P12108,&P5843,&P24679,&P18092 }};	/*Useragent - Nokia7230/5.0 (10.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7230) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3083445 = {S77859, 138, { &P23687,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; LIFE PLAY Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083537 = {S77860, 130, { &P13644,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; EVO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080507 = {S77861, 144, { &P20007,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT886 Build/6.7.3-179_YIC-10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA25964 = {S77862, 62, { &P12143,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia6208c/2.0 (04.51) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA64676 = {S77863, 12, { &P11656,&P17483,&P17660,&P18092 }};	/*Useragent - LG/U990/v1.0*/
const Device UA21666 = {S77864, 90, { &P13564,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-305/1.0 ObigoInternetBrowser/Q03C, Alcatel-OT-305/1.0 ObigoInternetBrowser/Q03C*/
const Device UA51702 = {S77865, 45, { &P13564,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-305A/1.0 ObigoInternetBrowser/Q03C*/
const Device UA55262 = {S77866, 44, { &P13564,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-305/1.0 ObigoInternetBrowser/Q03C*/
const Device UA421559 = {S77867, 104, { &P14708,&P5204,&P17338,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5130c-2/UC Browser7.9.0.102/70/355*/
const Device UA573913 = {S77868, 98, { &P14708,&P5204,&P17338,&P18092 }};	/*Useragent - Nokia5130/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5130/UC Browser7.9.0.102/69/352*/
const Device UA499098 = {S77869, 97, { &P15364,&P5690,&P22478,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.13) Gecko/X Firefox/2.0.0.13 Flock/1.1.1*/
const Device UA637103 = {S77870, 135, { &P13452,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; GT-P1000 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA763495 = {S77871, 135, { &P13452,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-P1000 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA811543 = {S77872, 138, { &P13452,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-P1000 Build/MIUI-XJ) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025743 = {S77873, 62, { &P13452,&P24694,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; GT-P1000 Build/MIUI-XJ)*/
const Device UA2607902 = {S77874, 143, { &P13452,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Android 2.3.5; en-gb; GT-P1000 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2612780 = {S77875, 131, { &P13452,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-P1000 Build/MIUI-XJ) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2624637 = {S77876, 146, { &P13452,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ru-ru; GT-P1000 Build/MIUI-XJ) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-9.1.0 Mobile Safari/534.30*/
const Device UA2496848 = {S77877, 126, { &P11751,&P17483,&P24667,&P18092 }};	/*Useragent - LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-kp500 teleca) UCBrowser8.2.1.144/70/352/UCWEB Mobile*/
const Device UA1886593 = {S77878, 72, { &P16177,&P5448,&P4777,&P18092 }};	/*Useragent - Sprint T7380 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2212284 = {S77879, 92, { &P16177,&P5448,&P4777,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2042674 = {S77880, 103, { &P15364,&P18145,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2205860 = {S77881, 102, { &P15364,&P18145,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2226623 = {S77882, 135, { &P23894,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V1277 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2260710 = {S77883, 108, { &P15767,&P17141,&P17596,&P18163 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko)/Nutch-1.5.1*/
const Device UA2358637 = {S77884, 57, { &P15364,&P7127,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0) Presto/2.12.388 Version/12.10*/
const Device UA2384778 = {S77885, 143, { &P15877,&P18111,&P18112,&P18091 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID SPYDER Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2670045 = {S77886, 142, { &P15877,&P18111,&P18112,&P18091 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID SPYDER Build/HDish) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422839 = {S77887, 155, { &P15837,&P6909,&P24673,&P18092 }};	/*Useragent - NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiax2-05) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2455884 = {S77888, 148, { &P15919,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3100 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2613458 = {S77889, 155, { &P15919,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; GT-P3100 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2613937 = {S77890, 148, { &P15919,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3100 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615915 = {S77891, 148, { &P15919,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3100 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2620256 = {S77892, 155, { &P15919,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; GT-P3100 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2621104 = {S77893, 148, { &P15919,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/SPS05T; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2468394 = {S77894, 131, { &P21515,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GEM8113 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2480793 = {S77895, 137, { &P22681,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; UMI X1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2530140 = {S77896, 143, { &P20261,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Philips W832 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558284 = {S77897, 137, { &P22634,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; WEXLER-TAB-7T Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2782291 = {S77898, 88, { &P18088,&P17017,&P20920,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/4.6.32741/29.3551; U; zh) Presto/2.8.119 Version/11.10*/
const Device UA2567390 = {S77899, 131, { &P17337,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7500 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2633925 = {S77900, 131, { &P17337,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P7500 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567580 = {S77901, 131, { &P22052,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7501 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2615956 = {S77902, 183, { &P15932,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:34, lv*/
const Device UA2625495 = {S77903, 192, { &P15932,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +0000, gb*/
const Device UA2674981 = {S77904, 139, { &P13018,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT11i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2994213 = {S77905, 139, { &P13018,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MT11i Build/4.1.A.0.562) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3102117 = {S77906, 139, { &P13018,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT11a Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676852 = {S77907, 134, { &P18299,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP4100 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2768788 = {S77908, 164, { &P14202,&P5477,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E15i Build/2.0.1.A.0.47) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2799738 = {S77909, 163, { &P14202,&P5477,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E15i Build/2.1.1.A.0.6) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2754802 = {S77910, 138, { &P22590,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773889 = {S77911, 152, { &P13644,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2784795 = {S77912, 158, { &P19777,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105P Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2869250 = {S77913, 157, { &P19777,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9105 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2878683 = {S77914, 152, { &P22962,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030A Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3102813 = {S77915, 145, { &P22962,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030A Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2898806 = {S77916, 147, { &P20283,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AZ210A Build/AZ210A_ICS_02.34I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA3010074 = {S77917, 147, { &P20283,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AZ210B Build/AZ210B_ICS_02.34I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2975572 = {S77918, 141, { &P21374,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 101 Titanium Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022806 = {S77919, 137, { &P19856,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-LS970 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021749 = {S77920, 137, { &P22651,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E435g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022050 = {S77921, 137, { &P22651,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E435f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022543 = {S77922, 136, { &P22651,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E435 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2997748 = {S77923, 137, { &P24098,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AS870 4G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3058817 = {S77924, 130, { &P24098,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AS870 4G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3083051 = {S77925, 143, { &P15854,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ607-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3081692 = {S77926, 146, { &P19863,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA4024 = {S77927, 69, { &P15435,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GD330-CH/V100 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22358 = {S77928, 66, { &P15435,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GD330/V100 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA4209 = {S77929, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/XEHE1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA4210 = {S77930, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/XXGK4 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA6134 = {S77931, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/DZGL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA20721 = {S77932, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/JTHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA24122 = {S77933, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/DZHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA24123 = {S77934, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/XXHA1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA24320 = {S77935, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/XBHB3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA31125 = {S77936, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/JPHH2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA31127 = {S77937, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/DDHB4 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA31126 = {S77938, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/DZHF1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA31128 = {S77939, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/XEGL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA31130 = {S77940, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/AKHE2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA31131 = {S77941, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/DBGL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA31132 = {S77942, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/XEGK3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA36500 = {S77943, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/NPGJ2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA37375 = {S77944, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/PMHE3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA37376 = {S77945, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/JVGK3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA37377 = {S77946, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung-SGH-i550/AOGL2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA45055 = {S77947, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/AKHA1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA50701 = {S77948, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/JVGJ4 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA50703 = {S77949, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/AIGI4 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA50702 = {S77950, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/DBHD2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA50704 = {S77951, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/XXGL2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA55034 = {S77952, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/JPGL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA62241 = {S77953, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/DDHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA64139 = {S77954, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/DBHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA64580 = {S77955, 155, { &P17163,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i550/JVHB2 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA23418 = {S77956, 41, { &P15603,&P17484,&P3625,&P18092 }};	/*Useragent - sam-r460 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA1894368 = {S77957, 114, { &P14635,&P5843,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-00/04.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.6*/
const Device UA183721 = {S77958, 132, { &P23863,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; LT-NA7 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA772338 = {S77959, 151, { &P12784,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Skate Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026851 = {S77960, 151, { &P12784,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Skate Build/IML74K; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA880738 = {S77961, 79, { &P15364,&P18118,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; pl; rv:1.8.0.1) Gecko/X Firefox/1.5.0.1 Ubuntu*/
const Device UA1964821 = {S77962, 37, { &P15795,&P5448,&P24630,&P18092 }};	/*Useragent - NOKIA-Lumia 710/UCWEB2.2.0.157/47/355*/
const Device UA2007715 = {S77963, 88, { &P15364,&P7133,&P22552,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:14.0) Gecko/X Firefox/14.0.3.0 IceDragon/14.0.3.0*/
const Device UA2206075 = {S77964, 133, { &P18465,&P18110,&P18616,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.1 Safari/535.19 Silk-Accelerated=false*/
const Device UA2232448 = {S77965, 136, { &P23889,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PAD703 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2504385 = {S77966, 136, { &P23889,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAD703 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2257337 = {S77967, 134, { &P21386,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo A698t Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2375940 = {S77968, 129, { &P21386,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo A698t Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2447549 = {S77969, 143, { &P21386,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo A698t Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2826931 = {S77970, 134, { &P21386,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo A698t Build IML74K) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2294355 = {S77971, 127, { &P17595,&P17295,&P24748,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.27.1 (KHTML, like Gecko) Stainless/0.4 Safari/525.20.1*/
const Device UA2354229 = {S77972, 154, { &P21592,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar Touch Kem 432 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2427780 = {S77973, 158, { &P12258,&P17483,&P24673,&P18092 }};	/*Useragent - LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-gs290) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2425129 = {S77974, 144, { &P20825,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VS TOUCHTAB 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467428 = {S77975, 137, { &P20227,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-L40G Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2529286 = {S77976, 153, { &P17475,&P5768,&P24675,&P18092 }};	/*Useragent - Nokia6300/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia6300) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2557212 = {S77977, 131, { &P20513,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; A5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2994855 = {S77978, 137, { &P20513,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KPT A5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566940 = {S77979, 139, { &P20517,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CINK PEAX Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2596250 = {S77980, 155, { &P18630,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A800S Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2600639 = {S77981, 138, { &P21537,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; vivo S9 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2749995 = {S77982, 139, { &P21537,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; vivo S9t Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3071624 = {S77983, 60, { &P21537,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; vivo S9 Build/JRO03C)*/
const Device UA2671726 = {S77984, 114, { &P15364,&P7133,&P19839,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.43 Safari/537.31 slurp*/
const Device UA2677112 = {S77985, 114, { &P15364,&P7133,&P19839,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 slurp*/
const Device UA2738759 = {S77986, 152, { &P20868,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M040 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2714930 = {S77987, 92, { &P12280,&P17146,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 5_0_1; zh-CN; iPh3,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile*/
const Device UA2737330 = {S77988, 92, { &P12280,&P17146,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 5_0_1; zh-CN; iPh4,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile*/
const Device UA2714987 = {S77989, 92, { &P12029,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN82) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2715204 = {S77990, 137, { &P22927,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_E7310 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2734632 = {S77991, 152, { &P21968,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030D Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2783056 = {S77992, 141, { &P23148,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ONE TOUCH TAB 7HD Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2799408 = {S77993, 150, { &P21261,&P18598,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9500 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3013909 = {S77994, 153, { &P21261,&P18598,&P22879,&P18092 }};	/*Useragent - MQQBrowser/4.3/Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2800623 = {S77995, 141, { &P23148,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ONE TOUCH TAB 7HD Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2962194 = {S77996, 161, { &P16356,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; GT-N7000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2963726 = {S77997, 161, { &P16356,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-N7000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2879014 = {S77998, 141, { &P23646,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Zeepad 7.0 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2893171 = {S77999, 64, { &P15364,&P5690,&P22170,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:22.0) Gecko/X Firefox/22.0 slurp*/
const Device UA2998011 = {S78000, 70, { &P15364,&P5690,&P22170,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:22.0) Gecko/X Firefox/22.0 slurp slurp*/
const Device UA2961824 = {S78001, 154, { &P12108,&P5843,&P24678,&P18092 }};	/*Useragent - Nokia7230/5.0 (10.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7230) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3021224 = {S78002, 138, { &P14225,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Desire HD Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021452 = {S78003, 144, { &P14225,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire HD Build/ViperDHD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021874 = {S78004, 134, { &P20944,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T-Hub2 Build/TVA301TELBG3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3080543 = {S78005, 140, { &P20452,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Lenovo S880 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3036411 = {S78006, 97, { &P20729,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A27) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA24298 = {S78007, 96, { &P14553,&P17485,&P3976,&P18092 }};	/*Useragent - Mozilla/4.0 SonyEricssonV640iv/R1DA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA44221 = {S78008, 137, { &P12493,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; fr-fr; U8230 Build/CRB17) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA308427 = {S78009, 48, { &P15364,&P18110,&P22520,&P18092 }};	/*Useragent - Midori/0.3 (X11; Linux; U; en-us) WebKit/534.21+*/
const Device UA393642 = {S78010, 54, { &P15364,&P18110,&P22520,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux) AppleWebKit/531.2+ Midori/0.3*/
const Device UA517318 = {S78011, 77, { &P15364,&P18110,&P22520,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux; en-us; rv:1.9.0.2) Gecko/X Firefox/3.8 Midori/0.3*/
const Device UA525594 = {S78012, 96, { &P15364,&P18110,&P22520,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux; en-us) AppleWebKit/531+ (KHTML, like Gecko) Safari/531.2+ Midori/0.3*/
const Device UA894092 = {S78013, 47, { &P15364,&P18110,&P22520,&P18092 }};	/*Useragent - Midori/0.3 (X11; Linux; U; en-us) WebKit/531.2+*/
const Device UA2142180 = {S78014, 48, { &P15364,&P18110,&P22520,&P18092 }};	/*Useragent - Midori/0.3 (X11; Linux; U; de-de) WebKit/534.21+*/
const Device UA369635 = {S78015, 148, { &P20286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MT917 Build/6.5.1_GC-146-DNRTD-8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA668054 = {S78016, 218, { &P20286,&P5629,&P15760,&P18092 }};	/*Useragent - MOT-MT917_TD/1.0 Android/2.3 (Linux; U; Android 2.3.5; zh-cn) Release/12.09.2011 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2758702 = {S78017, 148, { &P20286,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MT917 Build/6.5.1_GC-146-DNRTD-9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA663864 = {S78018, 150, { &P21111,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ja-jp; IS12M Build/6.5.1-134_SWK-68_LE-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA856676 = {S78019, 136, { &P18235,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; zh-cn; S2005A-H Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2495048 = {S78020, 130, { &P18235,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; S2005A-H Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2519625 = {S78021, 139, { &P18235,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; S2005A-H Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2218681 = {S78022, 135, { &P20478,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; JY-G2 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2379475 = {S78023, 135, { &P20478,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JY-G2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2559260 = {S78024, 159, { &P12258,&P17483,&P24676,&P18092 }};	/*Useragent - LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gs290) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2567248 = {S78025, 137, { &P19917,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P765 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583339 = {S78026, 127, { &P16598,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A500 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2584953 = {S78027, 148, { &P23309,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Celkon A75 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2595285 = {S78028, 133, { &P17615,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SHV-E230S Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2659314 = {S78029, 133, { &P17615,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E230S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2748326 = {S78030, 133, { &P17615,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E230K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2763544 = {S78031, 133, { &P17615,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SHV-E230K Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2792321 = {S78032, 133, { &P17615,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E230L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2673254 = {S78033, 136, { &P18234,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P895 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673938 = {S78034, 140, { &P21542,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6503 Build/10.1.A.1.434) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676831 = {S78035, 135, { &P18470,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 101G10 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2696508 = {S78036, 114, { &P15364,&P16110,&P24195,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)*/
const Device UA2951053 = {S78037, 80, { &P15364,&P16110,&P24195,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; QQBrowser/6.12)*/
const Device UA2982824 = {S78038, 72, { &P15364,&P16110,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.2; SV1; QQBrowser/6.12)*/
const Device UA3118223 = {S78039, 195, { &P15364,&P16110,&P24195,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; QQBrowser/6.12)*/
const Device UA2796763 = {S78040, 152, { &P19747,&P5043,&P24677,&P18092 }};	/*Useragent - Nokia206/2.0 (03.59) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia206) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2754623 = {S78041, 133, { &P20690,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; V360 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2768118 = {S78042, 104, { &P23331,&P18110,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; U;  CE-HTML/1.0 NETTV/4.1.1 PHILIPSTV/2.1.1; en) Presto/2.10.287 Version/12.00*/
const Device UA2868225 = {S78043, 134, { &P17595,&P17297,&P22778,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.1 Safari/7536.29.13 Sleipnir/4.3.0.1*/
const Device UA2901068 = {S78044, 131, { &P17595,&P17297,&P22778,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.1 Safari/7536.30.1 Sleipnir/4.3.1*/
const Device UA2876323 = {S78045, 129, { &P20420,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; I10A-LE Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2974107 = {S78046, 134, { &P20480,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; JY-G3 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2968672 = {S78047, 135, { &P15989,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-in; ST21i Build/11.0.A.4.27) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3000520 = {S78048, 135, { &P15989,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-in; ST21i2 Build/11.0.A.6.8) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3008874 = {S78049, 135, { &P15989,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-in; ST21i2 Build/11.0.A.1.8) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3009614 = {S78050, 135, { &P15989,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-in; ST21i Build/11.0.A.0.16) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3091614 = {S78051, 135, { &P15989,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-in; ST21i Build/11.0.A.4.22) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2976682 = {S78052, 126, { &P18733,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M532 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3026816 = {S78053, 165, { &P21351,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HUAWEI C8813 Build/HuaweiC8813) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3059926 = {S78054, 167, { &P21351,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI C8813Q Build/HuaweiC8813Q) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3034273 = {S78055, 137, { &P18442,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-N7105 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079350 = {S78056, 137, { &P18442,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-N7105 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079586 = {S78057, 137, { &P18442,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L900 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108794 = {S78058, 137, { &P18442,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I317 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108942 = {S78059, 137, { &P18442,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-N7105 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3130586 = {S78060, 128, { &P21769,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ME301T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA20096 = {S78061, 131, { &P17425,&P5338,&P3772,&P18092 }};	/*Useragent - MOT-RAZRV3xxR/97.04.2BR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003290) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.60 [en]*/
const Device UA2279086 = {S78062, 125, { &P17425,&P5338,&P3772,&P18092 }};	/*Useragent - MOT-RAZRV3xxR/97.04.2BR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; X) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [en]*/
const Device UA21944 = {S78063, 62, { &P12024,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia7310c/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA21947 = {S78064, 67, { &P12024,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia7310c_CMCC/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63720 = {S78065, 13, { &P13537,&P17483,&P17660,&P18092 }};	/*Useragent - LG/GT405/v1.0*/
const Device UA338716 = {S78066, 147, { &P15438,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC/Sensation/1.45.161.1; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA409486 = {S78067, 147, { &P15438,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC/Sensation/1.45.162.1; de-de) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA571043 = {S78068, 147, { &P15438,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC/Sensation/1.45.166.1; de-ch) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA812860 = {S78069, 142, { &P15438,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Sensation_Z710a; en-au) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA1896531 = {S78070, 147, { &P15438,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC/Sensation/1.35.161.2; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA1904138 = {S78071, 147, { &P15438,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC/Sensation/1.45.163.4; fr-fr) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA680007 = {S78072, 43, { &P18364,&P5129,&P24665,&P18092 }};	/*Useragent - NokiaC5-05/UC Browser8.0.4.121/50/444/UCWEB*/
const Device UA694932 = {S78073, 101, { &P18218,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 610)*/
const Device UA925858 = {S78074, 111, { &P18218,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 610; Vodafone)*/
const Device UA772776 = {S78075, 79, { &P14454,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE66)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA1946774 = {S78076, 30, { &P14454,&P5890,&P24667,&P18092 }};	/*Useragent - NokiaE66/UCWEB8.2.0.116/28/800*/
const Device UA1968503 = {S78077, 127, { &P12082,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; X10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2403572 = {S78078, 127, { &P12082,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; X10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2024100 = {S78079, 86, { &P17968,&P18066,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.3; Linux; Opera Mobi/ADR-X; U; en) Presto/2.7.81 Version/11.00*/
const Device UA2476389 = {S78080, 86, { &P17968,&P18066,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.4; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.7.81 Version/11.00*/
const Device UA2236231 = {S78081, 124, { &P12142,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU5) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/352/UCWEB Mobile*/
const Device UA2384452 = {S78082, 125, { &P14980,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-b7722) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2369225 = {S78083, 65, { &P15364,&P5690,&P20404,&P18092 }};	/*Useragent - Opera/12.80 (Windows NT 5.1; U; en) Presto/2.10.289 Version/12.08*/
const Device UA2427181 = {S78084, 134, { &P20922,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Dslide 704 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2605946 = {S78085, 130, { &P20446,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; F-05E Build/V13R36C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2558740 = {S78086, 154, { &P22618,&P22857,&P18373,&P18092 }};	/*Useragent - Micromax Q76/Q03C   Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA2578757 = {S78087, 142, { &P21402,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo S890 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2594396 = {S78088, 133, { &P21402,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo S890 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2746982 = {S78089, 64, { &P21402,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo S890 Build/IMM76D)*/
const Device UA2604078 = {S78090, 135, { &P21636,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C525c Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2674731 = {S78091, 130, { &P14905,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7100 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2654335 = {S78092, 178, { &P18557,&P20706,&P24671,&P18092 }};	/*Useragent - Nokia308/2.0 (05.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia308) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2700046 = {S78093, 160, { &P12125,&P5204,&P24676,&P18092 }};	/*Useragent - Nokia6500s-1/2.0 (08.30) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6500s-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2737442 = {S78094, 98, { &P12142,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; SonyEricssonU5) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2737809 = {S78095, 93, { &P14359,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5228) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2754342 = {S78096, 148, { &P18227,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U9510E Build/HuaweiU9510E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754347 = {S78097, 136, { &P18653,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P870 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2767829 = {S78098, 135, { &P22836,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SC-04E Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA3050948 = {S78099, 128, { &P22836,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SC-04E Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2968867 = {S78100, 102, { &P15775,&P21697,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 800) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile*/
const Device UA2772530 = {S78101, 139, { &P21507,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C1605 Build/11.1.A.0.68) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2870792 = {S78102, 143, { &P23363,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Vodafone 975N Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2899131 = {S78103, 156, { &P21628,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8552 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2974099 = {S78104, 130, { &P22052,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7501 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2966840 = {S78105, 158, { &P18280,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC VLE_U Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3013259 = {S78106, 158, { &P18280,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3079654 = {S78107, 158, { &P18280,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2968085 = {S78108, 94, { &P19976,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaE6-00) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA2974652 = {S78109, 144, { &P23799,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ONE TOUCH 4011X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2996811 = {S78110, 141, { &P24721,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PAP4505DUO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA55877 = {S78111, 61, { &P11957,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia6133/2.0 (05.60) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20726 = {S78112, 51, { &P13655,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-W840/00.72 UP.Browser/7.2.7.5.673 (GUI) MMP/2.0*/
const Device UA26963 = {S78113, 51, { &P13655,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-W840/00.72 UP.Browser/7.2.7.5.663 (GUI) MMP/2.0*/
const Device UA21998 = {S78114, 85, { &P12548,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-807A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA47688 = {S78115, 84, { &P12548,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-807/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA60565 = {S78116, 92, { &P12548,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-807/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A SVN/05S*/
const Device UA60566 = {S78117, 92, { &P12548,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-807/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A SVN/04N*/
const Device UA61132 = {S78118, 85, { &P12548,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-807D/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA64601 = {S78119, 37, { &P14232,&P17483,&P3866,&P18092 }};	/*Useragent - LG-KE970 MIC/1.1.14 MIDP-2.0/CLDC-1.1*/
const Device UA174032 = {S78120, 140, { &P15364,&P5690,&P22401,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Comodo_Dragon/9.0.0.3 Chrome/9.0.597.98 Safari/534.13*/
const Device UA413427 = {S78121, 30, { &P15364,&P6704,&P18405,&P18092 }};	/*Useragent - Opera/9.27 (Windows 98; U; en)*/
const Device UA534709 = {S78122, 30, { &P15364,&P6704,&P18405,&P18092 }};	/*Useragent - Opera/9.25 (Windows 98; U; en)*/
const Device UA798831 = {S78123, 30, { &P15364,&P6704,&P18405,&P18092 }};	/*Useragent - Opera/9.21 (Windows 98; U; en)*/
const Device UA1879832 = {S78124, 30, { &P15364,&P6704,&P18405,&P18092 }};	/*Useragent - Opera/9.24 (Windows 98; U; en)*/
const Device UA2132965 = {S78125, 30, { &P15364,&P6704,&P18405,&P18092 }};	/*Useragent - Opera/9.22 (Windows 98; U; en)*/
const Device UA2278942 = {S78126, 30, { &P15364,&P6704,&P18405,&P18092 }};	/*Useragent - Opera/9.26 (Windows 98; U; en)*/
const Device UA659062 = {S78127, 101, { &P15364,&P7133,&P22479,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.8.1.17; ns) Gecko/X Firefox/2.0.0.17 Flock/1.2.6*/
const Device UA791320 = {S78128, 32, { &P14033,&P5129,&P24670,&P18092 }};	/*Useragent - NokiaC5-03/UCWEB8.4.0.150/50/800*/
const Device UA895104 = {S78129, 81, { &P14033,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-in; NokiaC5-03)/UC Browser8.4.0.159/50/352/UCWEB Mobile*/
const Device UA794631 = {S78130, 67, { &P12616,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.4;Zh_cn;WT19i;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA2022295 = {S78131, 116, { &P12616,&P5629,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.3.4;Zh_cn;WT19i;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA2350016 = {S78132, 125, { &P13677,&P17484,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5253) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/405/UCWEB Mobile*/
const Device UA2257149 = {S78133, 138, { &P23059,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Smart_one Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2359844 = {S78134, 205, { &P16036,&P5477,&P4861,&P18089 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; Nexus One Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 (compatible; Googlebot-Mobile/2.1; +http://www.google.com/bot.html)*/
const Device UA2384637 = {S78135, 131, { &P23318,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; LT-7009 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2850444 = {S78136, 131, { &P23318,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT-7009 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2660691 = {S78137, 63, { &P15364,&P16110,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; Windows NT 6.2; Trident/3.0)*/
const Device UA2666622 = {S78138, 138, { &P23189,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Surfer 8.31 3G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2673271 = {S78139, 151, { &P18505,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MB886 Build/7.7.1Q-144_MB886_MR1-22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2711452 = {S78140, 139, { &P20816,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CINK SLIM Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2732573 = {S78141, 139, { &P23518,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2;zh-cn; ZTE V987 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30;*/
const Device UA2747316 = {S78142, 130, { &P23249,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; N80IPS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2780111 = {S78143, 161, { &P20192,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; Orange Daytona Build/HuaweiG510-0200) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24 TT/4.1.1-eng.lX.X.X*/
const Device UA2863582 = {S78144, 152, { &P20192,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HUAWEI G510-0200 Build/HuaweiG510-0200) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24 TT/4.1.1*/
const Device UA2937958 = {S78145, 152, { &P20192,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HUAWEI G510-0100 Build/HuaweiG510-0100) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24 TT/4.1.1*/
const Device UA2800147 = {S78146, 63, { &P15364,&P18110,&P23212,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:17.0) Gecko/X Icedove/17.0.5*/
const Device UA3018055 = {S78147, 61, { &P15364,&P18110,&P23212,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:17.0) Gecko/X Icedove/17.0.7*/
const Device UA2901231 = {S78148, 156, { &P19853,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7102 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2964693 = {S78149, 156, { &P19853,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7102 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2944471 = {S78150, 138, { &P23642,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-M840 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2975024 = {S78151, 134, { &P20480,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JY-G3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3063302 = {S78152, 151, { &P20529,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A89 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3061352 = {S78153, 159, { &P22209,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Titanium S5 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3027713 = {S78154, 157, { &P18295,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC One X Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA61690 = {S78155, 77, { &P13370,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-M3200/M3200JCHJ1 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21136 = {S78156, 127, { &P12146,&P17483,&P3459,&P18092 }};	/*Useragent - LG-GU285g Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA23890 = {S78157, 127, { &P12146,&P17483,&P3459,&P18092 }};	/*Useragent - LG-GU285f Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA62729 = {S78158, 106, { &P12146,&P17483,&P3459,&P18092 }};	/*Useragent - LG-GU285/v10a Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA23779 = {S78159, 77, { &P18088,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA51434 = {S78160, 126, { &P18088,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) 480x800; XV6975; Window Mobile 6.5 Professional;*/
const Device UA75837 = {S78161, 172, { &P18088,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Mozilla/5.0 (HW-HUAWEI_C8300/C8300V100R001C00B102;U;Windows Mobile/6.5;320*480;CTC/2.0) IE/6.0*/
const Device UA43681 = {S78162, 145, { &P13724,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; SCH-i909 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA857438 = {S78163, 145, { &P13724,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; GT-I9088 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1920634 = {S78164, 131, { &P13724,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; SCH-i909 Build/ECLAIR) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2742830 = {S78165, 136, { &P13724,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; SCH-i909 Build/ECLAIR) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA113211 = {S78166, 67, { &P18254,&P17350,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT; MS Search 4.0 Robot)*/
const Device UA182192 = {S78167, 126, { &P15364,&P18110,&P22448,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Iron/10.0.650.0 Chrome/10.0.650.0 Safari/534.16*/
const Device UA700574 = {S78168, 128, { &P15364,&P18110,&P22448,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Iron/10.0.650.0 Chrome/10.0.650.0 Safari/534.16*/
const Device UA237670 = {S78169, 74, { &P14969,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_Desire_S; 480*800) UCWEB7.9.3.103/139/800*/
const Device UA452097 = {S78170, 73, { &P14969,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_Desire_S; 480*800) UCWEB7.9.0.94/139/800*/
const Device UA572565 = {S78171, 75, { &P14969,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_Desire_S; 480*800) UCWEB7.9.3.103/139/1002*/
const Device UA628417 = {S78172, 74, { &P14969,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_Desire_S; 480*800) UCWEB7.9.3.103/139/800*/
const Device UA642490 = {S78173, 74, { &P14969,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_Desire_S; 480*800) UCWEB7.9.3.103/139/999*/
const Device UA758166 = {S78174, 74, { &P14969,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_Desire_S; 480*800) UCWEB7.9.1.100/139/800*/
const Device UA1994150 = {S78175, 74, { &P14969,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_Desire_S; 480*800) UCWEB7.9.4.145/139/800*/
const Device UA2316987 = {S78176, 74, { &P14969,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_Desire_S; 480*800) UCWEB7.9.4.145/139/800*/
const Device UA2456358 = {S78177, 73, { &P14969,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_Desire_S; 480*800) UCWEB7.9.0.94/139/352*/
const Device UA2505032 = {S78178, 73, { &P14969,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_Desire_S; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA423768 = {S78179, 140, { &P15699,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Incredible Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA748923 = {S78180, 147, { &P15699,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Droid Incredible Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA763465 = {S78181, 141, { &P15699,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Incredible Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA821365 = {S78182, 141, { &P15699,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Incredible Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA836990 = {S78183, 141, { &P15699,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Incredible Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1971029 = {S78184, 147, { &P15699,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Droid Incredible Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025476 = {S78185, 147, { &P15699,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Droid Incredible Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034001 = {S78186, 147, { &P15699,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Droid Incredible Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA738976 = {S78187, 89, { &P15364,&P18110,&P22479,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.14) Gecko/X Firefox/2.0.0.14 Flock/1.2.1*/
const Device UA576332 = {S78188, 140, { &P24574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Spice Mi-350 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2216096 = {S78189, 140, { &P24574,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Spice Mi-350 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA625363 = {S78190, 131, { &P21244,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHV-E140S Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA791298 = {S78191, 137, { &P23984,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; id-id; T1C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1977096 = {S78192, 132, { &P21224,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; N-07D Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2890769 = {S78193, 54, { &P21224,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; N-07D Build/AX)*/
const Device UA2401624 = {S78194, 67, { &P15364,&P5690,&P18429,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:17.0) Gecko/17.0 Firefox/17.0 slurp*/
const Device UA2407102 = {S78195, 64, { &P15364,&P5690,&P18429,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:17.0) Gecko/X Firefox/17.0 slurp*/
const Device UA2555593 = {S78196, 70, { &P15364,&P5690,&P18429,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:17.0) Gecko/X Firefox/17.0 slurp slurp*/
const Device UA2486392 = {S78197, 164, { &P18317,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Transformer Pad TF300T Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2565034 = {S78198, 147, { &P18317,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; TF300T Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2669694 = {S78199, 154, { &P18317,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; TF300T Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781815 = {S78200, 155, { &P18317,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; TF300T Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2554223 = {S78201, 138, { &P22104,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HS-E926 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2746779 = {S78202, 129, { &P22104,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HS-E926 Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2983060 = {S78203, 60, { &P22104,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HS-E926 Build/IMM76I)*/
const Device UA3043141 = {S78204, 129, { &P22104,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; HS-E926 Build IMM76I) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2566774 = {S78205, 137, { &P14017,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P990 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2625628 = {S78206, 150, { &P17168,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7310 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2633816 = {S78207, 149, { &P17168,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-P7310 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2637174 = {S78208, 150, { &P17168,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7310 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2957590 = {S78209, 150, { &P17168,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7310 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2676749 = {S78210, 138, { &P18292,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ609 Build/7.7.1-85_MZ609-31) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2705237 = {S78211, 140, { &P23076,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; @TAB-NINE.7-X2-B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2782956 = {S78212, 155, { &P18621,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; LG-E612 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2754233 = {S78213, 133, { &P20156,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; S500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755047 = {S78214, 134, { &P18605,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTL21 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2974604 = {S78215, 137, { &P22601,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300L Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA1685 = {S78216, 98, { &P11914,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-E2510/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA17273 = {S78217, 112, { &P11914,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-E2510-ORANGE/E2510CHIB1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA19676 = {S78218, 136, { &P15006,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SCH-W899 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70718 = {S78219, 136, { &P15006,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-W899 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022692 = {S78220, 124, { &P15006,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-W899 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2546114 = {S78221, 129, { &P15006,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-W899 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA21428 = {S78222, 124, { &P16980,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-SGH-I627/UCIC2 Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA54861 = {S78223, 80, { &P15364,&P22857,&P17470,&P18092 }};	/*Useragent - Beetel GT415/2.0 (5.0614.0) NucleusOS/1.0 Profile/MIDP-2.0Configuration/CLDC-1.0*/
const Device UA88671 = {S78224, 116, { &P15364,&P22857,&P17470,&P18092 }};	/*Useragent - MX8/1.00 Nucleus RTOS/V1.11.19 MTK6225/07B Release/04.25.2008 Browser/Teleca Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA89713 = {S78225, 122, { &P15364,&P22857,&P17470,&P18092 }};	/*Useragent - BIRD K298/1.00 Nucleus RTOS/V1.11.19 MTK6223/08A Release/07.01.2008 Browser/Teleca Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA129762 = {S78226, 118, { &P15364,&P22857,&P17470,&P18092 }};	/*Useragent - BIRD /1.00 Nucleus RTOS/V1.11.19 MTK6235/08B Release/04.09.2009 Browser/Teleca Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA686438 = {S78227, 80, { &P15364,&P5690,&P20903,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; ja-JP; rv:1.7.5) Gecko/X Netscape/8.0.4*/
const Device UA585617 = {S78228, 76, { &P15364,&P17285,&P22339,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:9.0.1) Gecko/X Firefox/9.0.1 CometBird/9.0.1*/
const Device UA808005 = {S78229, 30, { &P15364,&P21445,&P17213,&P18092 }};	/*Useragent - Opera/8.50 (Windows ME; U; en)*/
const Device UA857381 = {S78230, 30, { &P15364,&P21445,&P17213,&P18092 }};	/*Useragent - Opera/8.52 (Windows ME; U; en)*/
const Device UA2129054 = {S78231, 61, { &P15364,&P21445,&P17213,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows ME; pl) Opera 8.02*/
const Device UA853848 = {S78232, 102, { &P14802,&P18148,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 8.0; Trident/5.0; IEMobile/9.0; SAMSUNG; SGH-i917)*/
const Device UA2857968 = {S78233, 117, { &P14802,&P18148,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 8.0; Trident/5.0; IEMobile/9.0; WpsLondonTest; SAMSUNG; SGH-i917)*/
const Device UA891727 = {S78234, 79, { &P13977,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-in; NokiaN73)/UC Browser8.4.0.159/28/352/UCWEB Mobile*/
const Device UA1961987 = {S78235, 78, { &P16003,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; gt-s5233a) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA1954220 = {S78236, 131, { &P22008,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; SPNG808 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2232461 = {S78237, 77, { &P23545,&P17017,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; mauie800) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2235144 = {S78238, 171, { &P16356,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2222391 = {S78239, 129, { &P21222,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT1S0 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2230810 = {S78240, 135, { &P21273,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Aqua S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2354536 = {S78241, 154, { &P12216,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ADR6300 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950533 = {S78242, 154, { &P12216,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ADR6300 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423521 = {S78243, 87, { &P17968,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; spice) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2461384 = {S78244, 132, { &P15919,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P3100 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2563402 = {S78245, 131, { &P15919,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P3100 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2469663 = {S78246, 142, { &P20261,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Philips W832 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2531066 = {S78247, 161, { &P13123,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000*/
const Device UA2565297 = {S78248, 166, { &P18595,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-I8190 Build/JRO03H) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2662590 = {S78249, 166, { &P18595,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I8190 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2581070 = {S78250, 99, { &P21269,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Intex_Aqua_3.2) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2597882 = {S78251, 135, { &P17346,&P18432,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 60 like Mac OS X) AppleWebKit/536.26 (KHTML like Gecko) Mobile/10A403 Safari/7534.48.3 Sleipnir/2.1.2m*/
const Device UA2659180 = {S78252, 140, { &P23447,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Newman N2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2733020 = {S78253, 227, { &P22826,&P22828,&P18246,&P18092 }};	/*Useragent - FARSIGHTED50_6464_11B_HW (MRE\2.5.00(800) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\1 gsensor\0 keyboard\normal) ZEN_P32_Software_V1.2 Release/2013.01.31 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US*/
const Device UA2733225 = {S78254, 227, { &P22826,&P22828,&P18246,&P18092 }};	/*Useragent - FARSIGHTED50_6464_11B_HW (MRE\2.5.00(800) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\1 gsensor\0 keyboard\normal) ZEN_P32_Software_V1.5 Release/2013.03.01 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US*/
const Device UA3072119 = {S78255, 227, { &P22826,&P22828,&P18246,&P18092 }};	/*Useragent - FARSIGHTED50_6464_11B_HW (MRE\2.5.00(800) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\1 gsensor\0 keyboard\normal) ZEN_P32_Software_V1.5 Release/2013.03.01 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 hi-IN*/
const Device UA2774164 = {S78256, 164, { &P18614,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT890 Build/9.8.2I-50_SMI-26) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2754566 = {S78257, 136, { &P21022,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E615 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868523 = {S78258, 137, { &P21022,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E615f Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2768951 = {S78259, 130, { &P23721,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CMP749 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2895570 = {S78260, 161, { &P19862,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax A110 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2962517 = {S78261, 156, { &P16356,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-N7000 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2963641 = {S78262, 156, { &P16356,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-N7000 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3063148 = {S78263, 94, { &P11905,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6120c) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3020616 = {S78264, 64, { &P15364,&P5690,&P22171,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:23.0) Gecko/X Firefox/23.0 slurp*/
const Device UA3077119 = {S78265, 70, { &P15364,&P5690,&P22171,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:23.0) Gecko/X Firefox/23.0 slurp slurp*/
const Device UA3058845 = {S78266, 156, { &P16356,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; GT-N7000 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3065193 = {S78267, 156, { &P16356,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-N7000 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3027750 = {S78268, 111, { &P22717,&P5843,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia109/04.12; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA63318 = {S78269, 104, { &P11817,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-T819/T819UVHB2 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA27264 = {S78270, 84, { &P12161,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW350c/R10BA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA36743 = {S78271, 122, { &P12161,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW350c_CMCC/R10BA SEMC-SW/R10BA Release/04.23.2008 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54684 = {S78272, 84, { &P12161,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW350a/R11BA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53634 = {S78273, 79, { &P13997,&P17017,&P4223,&P18092 }};	/*Useragent - INQ/B1-200H/1.0.39 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA56586 = {S78274, 79, { &P13997,&P17017,&P4223,&P18092 }};	/*Useragent - INQ/B1-200H/1.0.36 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA56587 = {S78275, 79, { &P13997,&P17017,&P4223,&P18092 }};	/*Useragent - INQ/B1-200H/1.0.54 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62283 = {S78276, 79, { &P13997,&P17017,&P4223,&P18092 }};	/*Useragent - INQ/B1-200H/1.0.63 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62339 = {S78277, 79, { &P13997,&P17017,&P4223,&P18092 }};	/*Useragent - INQ/B1-200H/1.0.56 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62340 = {S78278, 79, { &P13997,&P17017,&P4223,&P18092 }};	/*Useragent - INQ/B1-200H/1.0.38 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA790574 = {S78279, 143, { &P17595,&P17295,&P22492,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.0.4568 Chrome/7.0.517.440 Safari/534.7*/
const Device UA416259 = {S78280, 66, { &P15364,&P18110,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux i686; en) Opera 8.51*/
const Device UA432814 = {S78281, 66, { &P15364,&P18110,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux i686; en) Opera 8.52*/
const Device UA684867 = {S78282, 66, { &P15364,&P18110,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux i686; en) Opera 8.54*/
const Device UA881756 = {S78283, 47, { &P15364,&P18110,&P18098,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; U; en) Opera 8.52*/
const Device UA616467 = {S78284, 133, { &P18388,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; a9100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA616470 = {S78285, 133, { &P18388,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; A9100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA616473 = {S78286, 133, { &P18388,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; A9100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA616479 = {S78287, 131, { &P18388,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; nl-; A9100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA616481 = {S78288, 58, { &P18388,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; A9100C Build/GRJ90)*/
const Device UA616482 = {S78289, 134, { &P18388,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; A9100C Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1903698 = {S78290, 67, { &P13505,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.2; zh-cn; ME525; 480*854) UCWEB7.9.3.103/139/800*/
const Device UA2154290 = {S78291, 67, { &P13505,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.2; zh-cn; MB525; 480*854) UCWEB7.9.4.145/139/800*/
const Device UA2394129 = {S78292, 67, { &P13505,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.7; zh-cn; MB525; 480*854) UCWEB7.9.4.145/139/800*/
const Device UA2496140 = {S78293, 68, { &P13505,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; ME525+; 480*854) UCWEB7.9.4.145/139/800*/
const Device UA2033140 = {S78294, 173, { &P14202,&P5477,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E15i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA1952922 = {S78295, 136, { &P17948,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SC-02C Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2481137 = {S78296, 128, { &P20269,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; A211 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2355100 = {S78297, 136, { &P18603,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; L-02D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2365107 = {S78298, 153, { &P20603,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-C800 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2368079 = {S78299, 178, { &P18557,&P20706,&P24672,&P18092 }};	/*Useragent - Nokia308/2.0 (05.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia308) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2428743 = {S78300, 135, { &P20870,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Dslide 703R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2469140 = {S78301, 130, { &P20354,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EGS005 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2505296 = {S78302, 139, { &P14854,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2556886 = {S78303, 139, { &P14854,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2986396 = {S78304, 139, { &P14854,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2520156 = {S78305, 138, { &P21564,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; KM-S330 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2560775 = {S78306, 148, { &P21511,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Starmobile ENERGY Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2599142 = {S78307, 139, { &P20235,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Cynus T1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2664027 = {S78308, 137, { &P22085,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ThL W5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2673884 = {S78309, 111, { &P18095,&P18148,&P24633,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-914_im_india_269) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile*/
const Device UA2738017 = {S78310, 111, { &P18095,&P18148,&P24633,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-885_im_india_249) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile*/
const Device UA2751542 = {S78311, 111, { &P18095,&P18148,&P24633,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; zh-CN; NOKIA; RM-822_apac_prc_204) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile*/
const Device UA2860259 = {S78312, 111, { &P18095,&P18148,&P24633,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-821_im_india_443) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile*/
const Device UA2674747 = {S78313, 148, { &P20088,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E Build/HuaweiU8666E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677239 = {S78314, 154, { &P20088,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8666E-51 Build/HuaweiU8666E-51) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2796384 = {S78315, 164, { &P18645,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2799105 = {S78316, 156, { &P18645,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SPH-L710 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2751130 = {S78317, 146, { &P15767,&P17090,&P22593,&P18092 }};	/*Useragent - Mozilla/5.0(iPad; U;CPU OS 4_3 like Mac OS X; zh-CN; iPad2,1) AppleWebKit/534.46 (KHTML, like Gecko) UCBrowser/2.0.1.280 U3/0.8.0 Safari/7543.48.3*/
const Device UA2799371 = {S78318, 148, { &P18295,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2960484 = {S78319, 165, { &P22585,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L720 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/jfltespr*/
const Device UA2974892 = {S78320, 134, { &P23667,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; JY-G4 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3083325 = {S78321, 130, { &P18556,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; BN NookHD+ Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA67017 = {S78322, 8, { &P14748,&P5890,&P4030,&P18092 }};	/*Useragent - NokiaE72*/
const Device UA68515 = {S78323, 35, { &P14748,&P5890,&P4030,&P18092 }};	/*Useragent - NokiaE72-1/UCBrowser7.7.0.81/28/800*/
const Device UA69527 = {S78324, 10, { &P14748,&P5890,&P4030,&P18092 }};	/*Useragent - NokiaE72-1*/
const Device UA426918 = {S78325, 88, { &P15364,&P5690,&P24198,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0) QQBrowser/6.11.12135.201*/
const Device UA821457 = {S78326, 125, { &P15364,&P5690,&P24198,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.11.12388.201*/
const Device UA2321240 = {S78327, 123, { &P15364,&P5690,&P24198,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1 QQBrowser/6.11.11981.201*/
const Device UA2519428 = {S78328, 100, { &P15364,&P5690,&P24198,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; qdesk 2.3.1186.202) QQBrowser/6.11.12388.201*/
const Device UA650203 = {S78329, 158, { &P12056,&P5204,&P24667,&P18092 }};	/*Useragent - Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia7210supernova) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA600634 = {S78330, 142, { &P16074,&P5768,&P24667,&P18092 }};	/*Useragent - Nokia3110c/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia3110c) UCBrowser8.2.0.132/69/352/UCWEB Mobile*/
const Device UA659159 = {S78331, 63, { &P15364,&P18145,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD i386; rv:9.0.1) Gecko/X Firefox/9.0.1*/
const Device UA678485 = {S78332, 50, { &P15364,&P17188,&P17384,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.0; Windows NT 4.0)*/
const Device UA2658819 = {S78333, 57, { &P15364,&P17188,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; WinNT4.0; Trident/5.0)*/
const Device UA880126 = {S78334, 154, { &P17622,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HERO200 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA831729 = {S78335, 149, { &P15575,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA872381 = {S78336, 149, { &P15575,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9100 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1950968 = {S78337, 149, { &P15575,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2866770 = {S78338, 150, { &P15575,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2958390 = {S78339, 151, { &P15575,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-S959G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2999591 = {S78340, 150, { &P15575,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA903512 = {S78341, 141, { &P21120,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; KIRA N9010 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1954143 = {S78342, 128, { &P22117,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; DEOX Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1996473 = {S78343, 144, { &P21879,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; BioniQ 700 Build/ICS.f08refem32.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2167533 = {S78344, 102, { &P17595,&P18207,&P22482,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6.2*/
const Device UA2257676 = {S78345, 155, { &P18466,&P18066,&P19814,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.2 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2387123 = {S78346, 117, { &P15518,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; gt-s5230) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2836431 = {S78347, 114, { &P15518,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; fa; gt-s5230) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile*/
const Device UA2842176 = {S78348, 122, { &P15518,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-s5230) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2437587 = {S78349, 136, { &P20854,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Scroll Evoke Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2461757 = {S78350, 138, { &P22115,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TOUCH90 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2530792 = {S78351, 161, { &P20231,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A21 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2665771 = {S78352, 169, { &P20231,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Karbonn A21 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3057656 = {S78353, 165, { &P20231,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A21 Build/IMM76I) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2531901 = {S78354, 130, { &P21194,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U; Android 4.0.4; en-us; MID102C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2556692 = {S78355, 139, { &P21431,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE V955 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952157 = {S78356, 61, { &P21431,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ZTE V955 Build/IMM76I)*/
const Device UA2567707 = {S78357, 138, { &P19853,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7102 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2707748 = {S78358, 138, { &P19853,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; GT-N7102 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583198 = {S78359, 138, { &P20013,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F180L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2600982 = {S78360, 138, { &P20013,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F180K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667181 = {S78361, 138, { &P20013,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F180S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2677177 = {S78362, 137, { &P14959,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9103 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2706592 = {S78363, 131, { &P21299,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; CTAB712 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2707402 = {S78364, 140, { &P20174,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; Cink King Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994124 = {S78365, 140, { &P20174,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; CINK KING Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708548 = {S78366, 130, { &P15767,&P18117,&P20782,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA2937182 = {S78367, 128, { &P15767,&P18117,&P20782,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/9B176 Safari/7534.48.3*/
const Device UA2773107 = {S78368, 165, { &P13042,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Incredible S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2755065 = {S78369, 138, { &P22591,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2756566 = {S78370, 126, { &P16598,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; A500 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2773524 = {S78371, 147, { &P19861,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I605 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2869729 = {S78372, 142, { &P23591,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; TRUE BEYOND TAB 3G Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2999594 = {S78373, 71, { &P23591,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; TRUE BEYOND TAB 3G Build/JRO03C)*/
const Device UA2879887 = {S78374, 137, { &P23642,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-M840 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2974713 = {S78375, 134, { &P15877,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT910 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3083212 = {S78376, 151, { &P20002,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 992D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA33431 = {S78377, 116, { &P12917,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2330/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA62533 = {S78378, 77, { &P14266,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-M2710/M2710DDIF4 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA789303 = {S78379, 102, { &P15364,&P7133,&P22482,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6.1 GTB7.1*/
const Device UA634043 = {S78380, 141, { &P15759,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  HTC Vision Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA911777 = {S78381, 140, { &P15759,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Vision Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA665905 = {S78382, 64, { &P15364,&P7133,&P18100,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:11.0) Gecko/X Firefox/11.0 slurp*/
const Device UA674013 = {S78383, 71, { &P15364,&P7133,&P18100,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:11.0) Gecko/X Firefox/11.0 slurp*/
const Device UA849425 = {S78384, 77, { &P15364,&P7133,&P18100,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:11.0) Gecko/X Firefox/11.0 slurp slurp*/
const Device UA1993430 = {S78385, 171, { &P18321,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA1974598 = {S78386, 129, { &P21281,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MP959 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029276 = {S78387, 146, { &P20516,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Woxter Tablet PC 97IPS Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2414571 = {S78388, 146, { &P20516,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Woxter Tablet PC 97IPS Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2416015 = {S78389, 151, { &P20516,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Woxter Tablet PC 97IPS DUAL Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2031548 = {S78390, 150, { &P21382,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; LNV-Lenovo_A560e Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2775760 = {S78391, 143, { &P21382,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; LNV-Lenovo_A560e Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3098846 = {S78392, 74, { &P21382,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; LNV-Lenovo_A560e Build/GINGERBREAD)*/
const Device UA2261425 = {S78393, 125, { &P12137,&P17484,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5620) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/405/UCWEB Mobile*/
const Device UA2481683 = {S78394, 31, { &P18221,&P5843,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia302/14.26)*/
const Device UA2510611 = {S78395, 37, { &P15775,&P5448,&P20770,&P18092 }};	/*Useragent - NOKIA-Lumia 800/UCWEB2.8.0.249/47/355*/
const Device UA2564251 = {S78396, 165, { &P18621,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; LG-E612 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2730141 = {S78397, 165, { &P18621,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; LG-E610 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2764931 = {S78398, 166, { &P18621,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; LG-E612f Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2613774 = {S78399, 158, { &P15876,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5100 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2716634 = {S78400, 159, { &P13123,&P5629,&P22281,&P18092 }};	/*Useragent - OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2869283 = {S78401, 164, { &P15201,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Sensation 4G Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2872875 = {S78402, 167, { &P15201,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Sensation Z710e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2738089 = {S78403, 158, { &P18451,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT26ii Build/6.1.A.2.55) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2794229 = {S78404, 158, { &P18451,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LT26ii Build/6.1.A.2.45) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2997616 = {S78405, 159, { &P18442,&P18111,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300*/
const Device UA2860469 = {S78406, 136, { &P23252,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; M6589 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865772 = {S78407, 135, { &P23146,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HYUNDAI T7S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2879134 = {S78408, 152, { &P23672,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3051480 = {S78409, 145, { &P23672,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2879690 = {S78410, 136, { &P23231,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E988 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3008156 = {S78411, 129, { &P23231,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E988 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2973905 = {S78412, 132, { &P23182,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PMP7280C3G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3018266 = {S78413, 136, { &P24114,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E440 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3058054 = {S78414, 129, { &P24114,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E440 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3026610 = {S78415, 153, { &P18635,&P21637,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-US; Nexus 4 Build/JWR66V) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3062769 = {S78416, 153, { &P18635,&P21637,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-US; Nexus 4 Build/JWR66Y) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3094026 = {S78417, 161, { &P13123,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5360 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3096255 = {S78418, 114, { &P12897,&P5843,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-00/03.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA4274 = {S78419, 122, { &P12108,&P5843,&P17431,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia7230/10.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.0.9.17.1*/
const Device UA70471 = {S78420, 125, { &P12108,&P5843,&P17431,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia7230-1c/10.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.0.9.17.1*/
const Device UA64194 = {S78421, 72, { &P14581,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-R101/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA89893 = {S78422, 32, { &P17748,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH07B(c500;TB;W24H16)*/
const Device UA154110 = {S78423, 139, { &P15364,&P7133,&P22399,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Comodo_Dragon/6.0.0.10 Chrome/6.0.472.63 Safari/534.3*/
const Device UA196107 = {S78424, 137, { &P18085,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SHW-M290K Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA221358 = {S78425, 140, { &P16552,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Micromax_A70 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA421844 = {S78426, 152, { &P21053,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonS51SE Build/4.0.2.A.0.61) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA603757 = {S78427, 152, { &P21053,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonS51SE Build/4.0.2.A.0.74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA660917 = {S78428, 152, { &P21053,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonS51SE Build/4.0.2.A.0.58) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1959792 = {S78429, 140, { &P21053,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; S51SE Build/4.0.2.A.0.74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2146417 = {S78430, 84, { &P15364,&P17285,&P18197,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9pre) Gecko/X Thunderbird/3.0a1*/
const Device UA765200 = {S78431, 142, { &P13035,&P5043,&P24667,&P18092 }};	/*Useragent - NokiaC2-02/2.0 (06.52) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac2-02) UCBrowser8.2.1.144/70/355/UCWEB Mobile*/
const Device UA624610 = {S78432, 52, { &P15364,&P22308,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.05; Windows NT 3.51)*/
const Device UA2581312 = {S78433, 58, { &P15364,&P22308,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; WinNT3.51; Trident/1.0)*/
const Device UA671590 = {S78434, 133, { &P21282,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; MP700 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA777507 = {S78435, 67, { &P14738,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;7 Mozart;7.10;7740)*/
const Device UA925711 = {S78436, 64, { &P14738,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;T8697;7.10;7720)*/
const Device UA925741 = {S78437, 67, { &P14738,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;7 Mozart;7.10;7720)*/
const Device UA925767 = {S78438, 64, { &P14738,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;T8697;7.10;8107)*/
const Device UA925809 = {S78439, 67, { &P14738,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;7 Mozart;7.10;8107)*/
const Device UA925820 = {S78440, 73, { &P14738,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;7 Mozart T8698;7.10;7740)*/
const Device UA925916 = {S78441, 73, { &P14738,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;7 Mozart T8698;7.10;8107)*/
const Device UA2034648 = {S78442, 64, { &P14738,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;T8697;7.10;8773)*/
const Device UA2419389 = {S78443, 73, { &P14738,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;7 Mozart T8698;7.10;8773)*/
const Device UA1992592 = {S78444, 146, { &P15805,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Raider X710e Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2032378 = {S78445, 141, { &P15805,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Holiday Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2232940 = {S78446, 163, { &P13452,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2616045 = {S78447, 166, { &P13452,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2621231 = {S78448, 164, { &P13452,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2621439 = {S78449, 164, { &P13452,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-P1000T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/405*/
const Device UA2623077 = {S78450, 164, { &P13452,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-P1000R Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2216520 = {S78451, 75, { &P18182,&P5448,&P24644,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; SLCC2)/UCWEB7.4.0.57/31/999*/
const Device UA2410586 = {S78452, 88, { &P18182,&P5448,&P24644,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; .NET CLR 2.0.50727)/UCWEB7.4.0.57/31/999*/
const Device UA2545392 = {S78453, 73, { &P18182,&P5448,&P24644,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; SV1)/UCWEB7.4.0.57/31/999*/
const Device UA2775486 = {S78454, 80, { &P18182,&P5448,&P24644,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; InfoPath.2)/UCWEB7.4.0.57/31/999*/
const Device UA3070894 = {S78455, 78, { &P18182,&P5448,&P24644,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; .NET4.0E)/UCWEB7.4.0.57/31/999*/
const Device UA3084963 = {S78456, 73, { &P18182,&P5448,&P24644,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB7.4.0.57/31/999*/
const Device UA2257541 = {S78457, 130, { &P21284,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MP1000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257719 = {S78458, 143, { &P21036,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Spice Mi-425 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2384380 = {S78459, 125, { &P15197,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-b5310) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2431368 = {S78460, 68, { &P12280,&P18117,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 (iPhone4,1; iPhone OS 5.1.1; en_US) AppleWebKit/420+*/
const Device UA2844596 = {S78461, 66, { &P12280,&P18117,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 (iPhone4,1; iPhone OS 5.1; en_US) AppleWebKit/420+*/
const Device UA2461053 = {S78462, 132, { &P19852,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7511 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2466561 = {S78463, 126, { &P24551,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; M2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2864572 = {S78464, 133, { &P24551,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; M2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985274 = {S78465, 138, { &P24551,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; zh-cn; M2 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2467600 = {S78466, 64, { &P15364,&P5690,&P18521,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:18.0) Gecko/X Firefox/18.0 slurp*/
const Device UA2508147 = {S78467, 70, { &P15364,&P5690,&P18521,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:18.0) Gecko/X Firefox/18.0 slurp slurp*/
const Device UA2504940 = {S78468, 136, { &P21448,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SH530U Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2509856 = {S78469, 154, { &P21152,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 930D Build/1.2.10.4) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2530070 = {S78470, 158, { &P20471,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V1 Build/HCL ME Tablet V1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734529 = {S78471, 174, { &P20471,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CONNECT 2G Build/HCL ME Tablet CONNECT 2G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2672668 = {S78472, 154, { &P13644,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2715306 = {S78473, 129, { &P21769,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ME301T Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2754340 = {S78474, 136, { &P20713,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID7065 Build/ICS.MID7065.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2796773 = {S78475, 127, { &P23005,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TBDC1093 Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3096560 = {S78476, 158, { &P18280,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3131392 = {S78477, 127, { &P23203,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; A1-810 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA11506 = {S78478, 106, { &P12530,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) PPC; 240x320; HTC P3450; OpVer 23.114.15.752*/
const Device UA24376 = {S78479, 129, { &P12530,&P5448,&P17173,&P18092 }};	/*Useragent - PPC; 240x320; HTC_P3450/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA26645 = {S78480, 105, { &P12530,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) PPC; 240x320; HTC P3450; OpVer 23.114.3.634*/
const Device UA40310 = {S78481, 99, { &P12530,&P5448,&P17173,&P18092 }};	/*Useragent - HTC-P3450-orange/PPC; 240x320; OpVer 23.114.2.741 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA40312 = {S78482, 105, { &P12530,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) PPC; 240x320; HTC P3450; OpVer 23.116.1.611*/
const Device UA57053 = {S78483, 82, { &P16537,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-E3213K/E3213KJPKD1 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA130989 = {S78484, 111, { &P15364,&P24094,&P17215,&P18092 }};	/*Useragent - Opera/9.50 (Linux mips ; U; CE-HTML/1.0 (<profilelist><ui_profile name="PHILIPS_OLS_2008"/></profilelist>); en)*/
const Device UA197288 = {S78485, 133, { &P14604,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB200 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA574165 = {S78486, 133, { &P14604,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB200 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA876261 = {S78487, 133, { &P14604,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; MB200 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA319666 = {S78488, 163, { &P17228,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; PocketEdge ERMINE-0.9; Android 2.2.1; en-us; edgejr Build/FRG83) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA402261 = {S78489, 74, { &P17968,&P17979,&P18186,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile; rv:11.0) Gecko/11.0 Firefox/11.0 Fennec/11.0*/
const Device UA774134 = {S78490, 134, { &P22262,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; T9199+ Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1994081 = {S78491, 134, { &P22262,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; T9199+ Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA924439 = {S78492, 63, { &P15364,&P18110,&P18515,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:14.0) Gecko/X Thunderbird/14.0*/
const Device UA1957581 = {S78493, 131, { &P21297,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TAB-711 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026904 = {S78494, 151, { &P15364,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Precedent Build/AreaRom Test)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2644038 = {S78495, 150, { &P15364,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Precedent Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2952961 = {S78496, 167, { &P15364,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; HYPERION 8 Mod by echcon04 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2957613 = {S78497, 158, { &P15364,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2028113 = {S78498, 143, { &P20122,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; Evodroid 7.2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2424197 = {S78499, 135, { &P20122,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; eVodroid U7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2655210 = {S78500, 136, { &P20122,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; Evodroid W10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2429887 = {S78501, 140, { &P12681,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus S 4G Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2582971 = {S78502, 139, { &P12681,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus S 4G Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2502375 = {S78503, 137, { &P21299,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-be; Lenco TAB-712 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2567498 = {S78504, 137, { &P21022,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E615 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2708083 = {S78505, 138, { &P21022,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E615f Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2672822 = {S78506, 132, { &P20818,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMID705X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2859168 = {S78507, 131, { &P20818,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMID705 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3048449 = {S78508, 60, { &P20818,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; PMID705 Build/JRO03C)*/
const Device UA2716675 = {S78509, 143, { &P22627,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; IRIS 455 Build/LAVAIRIS455) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2754278 = {S78510, 159, { &P17625,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.7; ST25i Build/6.0.B.3.184) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2766800 = {S78511, 141, { &P20931,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; iBall Slide i9702 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2768007 = {S78512, 141, { &P20931,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; iBall Slide 6309i Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2998609 = {S78513, 141, { &P20931,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; iBall Slide 6318i Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2768776 = {S78514, 148, { &P23121,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Intex Aqua Superb Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2771079 = {S78515, 136, { &P20227,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-L40G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773352 = {S78516, 136, { &P20587,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P715 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782080 = {S78517, 141, { &P23201,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Karbonn S2 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999626 = {S78518, 69, { &P17595,&P22878,&P21570,&P18092 }};	/*Useragent - Mail/1508 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookPro10%2C1)*/
const Device UA3051450 = {S78519, 68, { &P17595,&P22878,&P21570,&P18092 }};	/*Useragent - Mail/1508 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (MacBookAir5%2C2)*/
const Device UA3088917 = {S78520, 63, { &P17595,&P22878,&P21570,&P18092 }};	/*Useragent - Mail/1508 CFNetwork/596.4.3 Darwin/12.4.0 (x86_64) (iMac13%2C1)*/
const Device UA3080638 = {S78521, 138, { &P22727,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SGP312 Build/10.3.1.C.0.136) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3093504 = {S78522, 67, { &P22727,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SGP312 Build/10.3.1.C.0.136)*/
const Device UA55224 = {S78523, 106, { &P13305,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E370-VODA/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA62127 = {S78524, 101, { &P13305,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E370/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA352692 = {S78525, 77, { &P15364,&P18140,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; Linux Mint; en) Presto/2.10.229 Version/11.60*/
const Device UA414627 = {S78526, 93, { &P15364,&P17350,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.1.2 Safari/525.21*/
const Device UA2411839 = {S78527, 111, { &P15364,&P17350,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT; U; en) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Iris/1.1.7 Safari/525.20*/
const Device UA448002 = {S78528, 98, { &P15364,&P17285,&P20441,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; de; rv:1.9.0.15) Gecko/X Firefox 2.1 (.NET CLR 3.5.30729)*/
const Device UA454482 = {S78529, 110, { &P15364,&P17285,&P20441,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; de; rv:1.9.0.15) Gecko/X Firefox 2.1 (.NET CLR 3.5.30729)	Firefox 2.1*/
const Device UA565937 = {S78530, 11, { &P16725,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M6700*/
const Device UA573779 = {S78531, 66, { &P14781,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;mwp6985;7.10;7720)*/
const Device UA609097 = {S78532, 66, { &P14781,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;mwp6985;7.10;8107)*/
const Device UA925831 = {S78533, 66, { &P14781,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;mwp6985;7.10;8773)*/
const Device UA711095 = {S78534, 87, { &P17976,&P6546,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.1; Linux; Opera Tablet/ADR-X; U; fr) Presto/2.9.201 Version/11.50*/
const Device UA626878 = {S78535, 43, { &P14010,&P17484,&P17347,&P18092 }};	/*Useragent - samsung-gt-s5263/UC Browser8.0.3.107/69/444*/
const Device UA1958838 = {S78536, 136, { &P17595,&P18207,&P22455,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11*/
const Device UA2235576 = {S78537, 136, { &P17595,&P18207,&P22455,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11*/
const Device UA2750250 = {S78538, 136, { &P17595,&P18207,&P22455,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11*/
const Device UA3001525 = {S78539, 136, { &P17595,&P18207,&P22455,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11*/
const Device UA2234643 = {S78540, 165, { &P18241,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; LG-E400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2241406 = {S78541, 131, { &P18385,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2241484 = {S78542, 60, { &P18385,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Nexus 7 Build/IMM76D)*/
const Device UA2242101 = {S78543, 130, { &P18385,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; Nexus 7 Build/XtmTR) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2870045 = {S78544, 131, { &P18385,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus 7 Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2361500 = {S78545, 145, { &P21333,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; INO_ONE Build/INO_ONEINO_ONE) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2524089 = {S78546, 136, { &P20287,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT535 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556979 = {S78547, 133, { &P23950,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; F9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583569 = {S78548, 138, { &P14877,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9000 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583420 = {S78549, 127, { &P20269,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A211 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2849802 = {S78550, 58, { &P15364,&P22308,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; WinNT3.51; Trident/5.0)*/
const Device UA2708220 = {S78551, 137, { &P18653,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P870 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2713847 = {S78552, 140, { &P22750,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XOLO B700 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957911 = {S78553, 110, { &P18718,&P18148,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-846_im_mea3_237) U2/1.0.0 UCBrowser/3.0.0.285 U2/1.0.0 Mobile*/
const Device UA2971045 = {S78554, 111, { &P18718,&P18148,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-846_im_india_233) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile*/
const Device UA2775337 = {S78555, 98, { &P17595,&P7127,&P18108,&P18092 }};	/*Useragent - Mozilla/4.7C-CCK-MCD {C-UDP; EBM-APPLE} (Macintosh; I; PPC)','Mozilla/4.8 [en] (Windows NT 5.0; U)*/
const Device UA2963057 = {S78556, 170, { &P14682,&P5843,&P24678,&P18092 }};	/*Useragent - Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6303iclassic) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2966396 = {S78557, 165, { &P13042,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Incredible S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2974999 = {S78558, 140, { &P18662,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC J Z321e Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023024 = {S78559, 150, { &P20094,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825-1 Build/HuaweiU8825-1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3025617 = {S78560, 148, { &P20094,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3059414 = {S78561, 93, { &P15991,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6102) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3083269 = {S78562, 128, { &P23135,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; U30GT 2 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3131653 = {S78563, 150, { &P18505,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA43570 = {S78564, 75, { &P13704,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KM570-Bouygues/V10c Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA58500 = {S78565, 73, { &P13704,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KM570-orange/V10e Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA62909 = {S78566, 66, { &P13704,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KM570/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA54414 = {S78567, 117, { &P14206,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) LG-GW550 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA58447 = {S78568, 105, { &P14206,&P5448,&P4808,&P18092 }};	/*Useragent - LG-GW550 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA64222 = {S78569, 66, { &P13365,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GU220/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA68853 = {S78570, 80, { &P12274,&P5448,&P4777,&P18092 }};	/*Useragent - SAMSUNG-SGH-i900/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA93296 = {S78571, 113, { &P17346,&P17134,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7A341*/
const Device UA93943 = {S78572, 100, { &P17346,&P17134,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko)*/
const Device UA165748 = {S78573, 113, { &P15364,&P17285,&P22442,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/528.7 (KHTML, like Gecko) Iron/1.0.155.0 Safari/528.7*/
const Device UA397216 = {S78574, 42, { &P13706,&P5129,&P24663,&P18092 }};	/*Useragent - Nokia5235/UC Browser7.9.0.102/50/351/UCWEB*/
const Device UA895055 = {S78575, 79, { &P11854,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-in; NokiaE71)/UC Browser8.4.0.159/28/352/UCWEB Mobile*/
const Device UA2154576 = {S78576, 30, { &P11854,&P5890,&P24670,&P18092 }};	/*Useragent - NokiaE71/UCWEB8.4.0.150/28/800*/
const Device UA2545807 = {S78577, 79, { &P11854,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiae71-1) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA785020 = {S78578, 68, { &P15364,&P6729,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win95; en-US; rv:1.7.5) Gecko/X Firefox/1.0*/
const Device UA911292 = {S78579, 138, { &P14183,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I997 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1970835 = {S78580, 35, { &P14519,&P5129,&P24671,&P18092 }};	/*Useragent - NokiaN97 mini/UCWEB8.5.0.163/50/800*/
const Device UA2223474 = {S78581, 84, { &P14519,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaN97 mini)/UC Browser8.5.0.183/50/355/UCWEB Mobile*/
const Device UA1994894 = {S78582, 154, { &P19751,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; BLU TOUCH BOOK 7.0 Build/1.0.8265.0448) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024786 = {S78583, 140, { &P21563,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC_Z560e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027919 = {S78584, 131, { &P21563,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Z560e Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2028822 = {S78585, 140, { &P21563,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Z560e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2216405 = {S78586, 62, { &P15364,&P16110,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; U; en) Presto/2.2.15 Version/10.10*/
const Device UA2229018 = {S78587, 116, { &P14708,&P5204,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia5130c-2/07.95; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2231217 = {S78588, 128, { &P18282,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A200 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2424747 = {S78589, 128, { &P18282,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; a200 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2382714 = {S78590, 87, { &P12581,&P5055,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokia500) UCBrowser8.4.0.159/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA2407282 = {S78591, 154, { &P21140,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Optimus_Madrid Build/IceCreamSandwich) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2425299 = {S78592, 82, { &P15364,&P7133,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; WOW64; Opera Mobi/46154) Presto/2.11.355 Version/12.10*/
const Device UA2445106 = {S78593, 130, { &P20399,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; PantechP8010 Build/IMM76D) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2972241 = {S78594, 130, { &P20399,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; PantechP8010 Build/JZO54K) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2520569 = {S78595, 121, { &P21267,&P17979,&P18062,&P18092 }};	/*Useragent - Lenovo-A798t_TD/S100 Linux/3.0.13 Android/4.0 Release/05.16.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2*/
const Device UA2654757 = {S78596, 103, { &P14384,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.482; en-us; 9520/4.2.0.482) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2696081 = {S78597, 103, { &P14384,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.189; en-us; 9520/4.2.0.189) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2727876 = {S78598, 103, { &P14384,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.323; en-us; 9520/4.2.0.323) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2777669 = {S78599, 103, { &P14384,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.423; en-us; 9520/4.2.0.423) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3118632 = {S78600, 103, { &P14384,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.220; en-us; 9520/4.2.0.220) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2654893 = {S78601, 103, { &P15257,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.422; en-us; 8530/4.2.0.422) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2655422 = {S78602, 103, { &P15257,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.451; en-us; 8530/4.2.0.451) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2655846 = {S78603, 103, { &P15257,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.298; en-us; 8530/4.2.0.298) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2791104 = {S78604, 103, { &P15257,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.335; en-us; 8530/4.2.0.335) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2673167 = {S78605, 139, { &P18719,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C1505 Build/11.3.A.0.47) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713168 = {S78606, 136, { &P19917,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P765 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2732866 = {S78607, 137, { &P22759,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; eZee' Tab 804 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2865794 = {S78608, 135, { &P22759,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; eZee'Tab804 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2879663 = {S78609, 140, { &P23495,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; TAC-70072 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2973893 = {S78610, 138, { &P23184,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; iOCEAN X7 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020725 = {S78611, 170, { &P22872,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5282 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3050594 = {S78612, 166, { &P22872,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-S5282 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3029027 = {S78613, 115, { &P14862,&P5204,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2700c-2/09.97; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3080755 = {S78614, 136, { &P24382,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E986 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083934 = {S78615, 133, { &P20868,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M040 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081271 = {S78616, 134, { &P23193,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCL21 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA53784 = {S78617, 145, { &P14025,&P17979,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (HW-C8600/C8600V100R001C92B225;U;Android/2.1;320*480;CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA304867 = {S78618, 79, { &P14222,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5; XV6175.1)*/
const Device UA735593 = {S78619, 43, { &P13324,&P5055,&P24665,&P18092 }};	/*Useragent - NokiaN8-00/UC Browser8.0.4.121/50/444/UCWEB*/
const Device UA467172 = {S78620, 90, { &P15364,&P7133,&P22480,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; fr; rv:1.9.0.5) Gecko/X Firefox/3.0.5 Flock/2.0.3*/
const Device UA586620 = {S78621, 127, { &P15767,&P17779,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU 0S 5_0_1 like Mac OS X) AppleWebkit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3*/
const Device UA2751495 = {S78622, 124, { &P15767,&P17779,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU 6_1_3 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA2853446 = {S78623, 61, { &P15767,&P17779,&P17776,&P18092 }};	/*Useragent - YelpWebView/1.0 ipad/5_1_1 YelpApp/6.5.0 (x-screen-size 1.0;)*/
const Device UA2997105 = {S78624, 61, { &P15767,&P17779,&P17776,&P18092 }};	/*Useragent - YelpWebView/1.0 ipad/5_1_1 YelpApp/6.7.0 (x-screen-size 1.0;)*/
const Device UA3092951 = {S78625, 61, { &P15767,&P17779,&P17776,&P18092 }};	/*Useragent - YelpWebView/1.0 ipad/5_1_1 YelpApp/7.0.1 (x-screen-size 1.0;)*/
const Device UA895542 = {S78626, 123, { &P12280,&P17146,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.1*/
const Device UA904060 = {S78627, 123, { &P12280,&P17146,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.2*/
const Device UA911634 = {S78628, 123, { &P12280,&P17146,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.4*/
const Device UA1995589 = {S78629, 123, { &P12280,&P17146,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.5*/
const Device UA2002937 = {S78630, 123, { &P12280,&P17146,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.6*/
const Device UA2172908 = {S78631, 123, { &P12280,&P17146,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Flipboard/1.9.6*/
const Device UA2173877 = {S78632, 121, { &P12280,&P17146,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Flipboard/1.9.6*/
const Device UA2204766 = {S78633, 123, { &P12280,&P17146,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.7*/
const Device UA2260775 = {S78634, 123, { &P12280,&P17146,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Flipboard/1.9.7*/
const Device UA2362542 = {S78635, 123, { &P12280,&P17146,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.9.8*/
const Device UA2653652 = {S78636, 123, { &P12280,&P17146,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Flipboard/1.9.8*/
const Device UA1944768 = {S78637, 63, { &P20328,&P22784,&P3807,&P18092 }};	/*Useragent - MicromaxX271/1.1.2011 Browser/OBIGO_Q03C MMS/OBIGO_Q03C_MMS_V02*/
const Device UA1960054 = {S78638, 130, { &P23999,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TAB464 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2449969 = {S78639, 130, { &P23999,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; TAB464 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2234302 = {S78640, 169, { &P13018,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; MT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2227229 = {S78641, 138, { &P18674,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7005 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2351380 = {S78642, 121, { &P12707,&P5055,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; Nokia701) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/51/355/UCWEB Mobile*/
const Device UA2403518 = {S78643, 133, { &P20807,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; B92 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2403705 = {S78644, 136, { &P23546,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AD683G Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2456215 = {S78645, 73, { &P19956,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-C3260/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2485052 = {S78646, 135, { &P22075,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; eZee'Tab803 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2559162 = {S78647, 91, { &P16072,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAN78) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2603682 = {S78648, 130, { &P15767,&P18117,&P20781,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/9B206 Safari/7534.48.3*/
const Device UA2709252 = {S78649, 128, { &P15767,&P18117,&P20781,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/9B176 Safari/7534.48.3*/
const Device UA2659450 = {S78650, 99, { &P14912,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-c3222) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2730306 = {S78651, 223, { &P14912,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-c3222) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile, SAMSUNG-GT-C3222/C3222DDJL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA2672125 = {S78652, 137, { &P19865,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A850K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677704 = {S78653, 137, { &P19865,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A850L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707578 = {S78654, 137, { &P19865,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A850S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677161 = {S78655, 139, { &P18452,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT29i Build/7.0.A.3.212) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2751002 = {S78656, 140, { &P22855,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; Fly IQ430 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2769684 = {S78657, 136, { &P22855,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; IQ430 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2755063 = {S78658, 131, { &P20145,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; NS-13T001 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2781800 = {S78659, 157, { &P16273,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T679 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2859808 = {S78660, 156, { &P16273,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T679 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2888660 = {S78661, 141, { &P21648,&P21637,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-I9505G Build/JWR66N.S005) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2995276 = {S78662, 138, { &P24508,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LG-E445 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022097 = {S78663, 133, { &P19775,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; A210 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3039414 = {S78664, 140, { &P18372,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-04D Build/9.1.C.0.475) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA49905 = {S78665, 138, { &P14007,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; ZTE-U X850 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA679819 = {S78666, 136, { &P14007,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; ZTE X850 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA676601 = {S78667, 139, { &P15364,&P17285,&P17392,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.10 (KHTML, like Gecko) RockMelt/0.8.40.147 Chrome/8.0.552.231 Safari/534.10*/
const Device UA882476 = {S78668, 136, { &P15364,&P17285,&P17392,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.7 (KHTML, like Gecko) RockMelt/0.8.36.128 Chrome/7.0.517.44 Safari/534.7*/
const Device UA701131 = {S78669, 102, { &P16246,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i677)*/
const Device UA859587 = {S78670, 77, { &P23792,&P22283,&P4392,&P18092 }};	/*Useragent - Mozilla/4.0 (Brew MP 1.0.2; U; en-us; Kyocera; NetFront/4.1/AMB) Sprint E4233*/
const Device UA1965732 = {S78671, 85, { &P15197,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-b5310) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2758305 = {S78672, 85, { &P15197,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-b5310) UCBrowser8.3.1.161/70/405/UCWEB Mobile*/
const Device UA1994573 = {S78673, 144, { &P13526,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyTouch 4g Build/AwesomeICS) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2229084 = {S78674, 140, { &P13526,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyTouch 4G Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2483133 = {S78675, 144, { &P13526,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC myTouch 4g Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2545333 = {S78676, 149, { &P13526,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T-Mobile myTouch 4G Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2024295 = {S78677, 155, { &P15178,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-LW690 Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034032 = {S78678, 182, { &P15178,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-LW690 Build/GWK74; CyanogenMod 7.2 ported by: Primed and sammyz) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030733 = {S78679, 153, { &P13094,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Lenovo A60 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2381716 = {S78680, 162, { &P13094,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A60+ Build/LenovoLePhone) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2667275 = {S78681, 140, { &P17968,&P5096,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile*/
const Device UA2353918 = {S78682, 147, { &P15919,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616131 = {S78683, 147, { &P15919,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616218 = {S78684, 145, { &P15919,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617412 = {S78685, 145, { &P15919,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2366095 = {S78686, 135, { &P22093,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; DPS Dream 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2384453 = {S78687, 157, { &P18672,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2461328 = {S78688, 167, { &P18296,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC One V Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2409043 = {S78689, 89, { &P15364,&P17285,&P21958,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2a1pre) Gecko/X Minefield/3.6a1pre*/
const Device UA2466488 = {S78690, 143, { &P20233,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VSD220 Build/IMM76D.UI23E802_VSC) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2483493 = {S78691, 141, { &P20467,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Viettel I9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2845361 = {S78692, 133, { &P20467,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2485702 = {S78693, 68, { &P15364,&P17285,&P22564,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:16.0) Gecko/X Firefox/16.0 epic/16.0*/
const Device UA2988244 = {S78694, 75, { &P15364,&P17285,&P22564,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:16.0) Gecko/X Firefox/16.0 Epic/16.0*/
const Device UA2516620 = {S78695, 146, { &P19891,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SAMSUNG-SGH-I717 Build/JOP40G) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2534053 = {S78696, 145, { &P19891,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I717 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2532273 = {S78697, 158, { &P19861,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I605 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2752049 = {S78698, 158, { &P19861,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I605 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2561868 = {S78699, 121, { &P18295,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_One_X; en-us) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA3072664 = {S78700, 121, { &P18295,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_One_X; en-au) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2566954 = {S78701, 139, { &P18707,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MD_LIFETAB_P9516 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583320 = {S78702, 131, { &P13452,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2633169 = {S78703, 143, { &P13452,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000 Build/GINGERBREAD) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2637431 = {S78704, 132, { &P13452,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000L Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2598673 = {S78705, 93, { &P17324,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; GT-I9001) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2674738 = {S78706, 133, { &P12605,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT875 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2678414 = {S78707, 127, { &P18704,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT200 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2749232 = {S78708, 135, { &P18296,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One V Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2767960 = {S78709, 135, { &P21395,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; Lenovo-A3000-H/S100) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2769574 = {S78710, 137, { &P22609,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6312 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773906 = {S78711, 162, { &P19862,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax A110 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2782582 = {S78712, 137, { &P14183,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I997 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892602 = {S78713, 137, { &P14183,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-I997 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2796044 = {S78714, 153, { &P23151,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAP4044DUO Build/PrestigioPAP4044DUO) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2974078 = {S78715, 148, { &P17835,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook Tablet Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3093686 = {S78716, 132, { &P13644,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; EVO Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3095253 = {S78717, 132, { &P13644,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; EVO Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083669 = {S78718, 93, { &P15991,&P5629,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6102) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA4813 = {S78719, 139, { &P12192,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) HTC_Touch_Diamond2_T5353 Mozilla/4.0*/
const Device UA12444 = {S78720, 128, { &P12192,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_Touch_Diamond2/2.16.162.3 (67041)*/
const Device UA24504 = {S78721, 102, { &P12192,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA38807 = {S78722, 128, { &P12192,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_Touch_Diamond2/2.16.163.3 (67041)*/
const Device UA38808 = {S78723, 128, { &P12192,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Vodafone/1.0/HTC_Touch_Diamond2/2.16.164.1 (67041)*/
const Device UA75495 = {S78724, 144, { &P12192,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 480x800; HTC_Touch_Diamond2_T5353-Orange; OpVer 93.172.15.752*/
const Device UA20455 = {S78725, 66, { &P13549,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GB230/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA172380 = {S78726, 80, { &P17595,&P17295,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.7.62 Version/11.01*/
const Device UA731342 = {S78727, 80, { &P17595,&P17295,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.7.62 Version/11.00*/
const Device UA370651 = {S78728, 156, { &P13452,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-P1000 Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2608161 = {S78729, 161, { &P13452,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; GT-P1000 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA821702 = {S78730, 31, { &P20055,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX231/Q03C MAUI-browser/*/
const Device UA848280 = {S78731, 97, { &P12970,&P18148,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 8.0; Trident/5.0; IEMobile/9.0; Asus;Galaxy6)*/
const Device UA2858000 = {S78732, 112, { &P12970,&P18148,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 8.0; Trident/5.0; IEMobile/9.0; WpsLondonTest; Asus;Galaxy6)*/
const Device UA1989992 = {S78733, 128, { &P20376,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; T10A Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2477636 = {S78734, 128, { &P20376,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T10A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2200132 = {S78735, 95, { &P17595,&P18207,&P20792,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; Edition MAS; en) Presto/2.10.229 Version/11.66*/
const Device UA2025731 = {S78736, 155, { &P18386,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I757M Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2384727 = {S78737, 73, { &P18720,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; Micromax_A50; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2505227 = {S78738, 136, { &P20732,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; YP-GP1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2529936 = {S78739, 140, { &P21884,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SpiceMi1010 Build/SpiceMi1010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2566036 = {S78740, 127, { &P15767,&P17146,&P22901,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.3 Mobile/9A405 Safari/7534.48.3*/
const Device UA3100525 = {S78741, 125, { &P15767,&P17146,&P22901,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mercury/7.3 Mobile/9A334 Safari/7534.48.3*/
const Device UA2583226 = {S78742, 140, { &P14993,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT15i Build/4.1.B.0.587) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2606485 = {S78743, 156, { &P20983,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9308 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2644638 = {S78744, 87, { &P17968,&P18111,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1.1; Linux; Opera Mobi/ADR-X; U; en) Presto/2.8.149 Version/11.10*/
const Device UA2854949 = {S78745, 87, { &P17968,&P18111,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1.2; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.8.149 Version/11.10*/
const Device UA2796245 = {S78746, 159, { &P18223,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT22i Build/6.1.1.B.1.54) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2794240 = {S78747, 156, { &P13035,&P5043,&P24677,&P18092 }};	/*Useragent - NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-02) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2752458 = {S78748, 94, { &P15845,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC5-06) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2752818 = {S78749, 141, { &P23242,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; SKY HD9500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781942 = {S78750, 146, { &P23242,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; SKY HD9500 Plus Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2773898 = {S78751, 156, { &P18283,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2860351 = {S78752, 148, { &P20848,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Xenta TAB13-201 Build/Xenta TAB13-201) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2975075 = {S78753, 140, { &P21542,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6503 Build/10.1.A.1.434) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3031511 = {S78754, 157, { &P12972,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3059445 = {S78755, 158, { &P12972,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3025625 = {S78756, 150, { &P21261,&P18598,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9500 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3081470 = {S78757, 139, { &P22656,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SOL22 Build/10.2.F.3.43) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081230 = {S78758, 136, { &P18653,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P870 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA269517 = {S78759, 73, { &P18182,&P5448,&P24642,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB7.0.0.41/31/352*/
const Device UA444202 = {S78760, 73, { &P18182,&P5448,&P24642,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB7.0.2.37/31/800*/
const Device UA468454 = {S78761, 73, { &P18182,&P5448,&P24642,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB7.0.0.41/30/351*/
const Device UA838466 = {S78762, 82, { &P18182,&P5448,&P24642,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)/UCWEB7.0.0.41/30/351*/
const Device UA2242404 = {S78763, 73, { &P18182,&P5448,&P24642,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB7.0.0.33/31/800*/
const Device UA561968 = {S78764, 164, { &P22341,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MOT-XT800+/TTUPG_M6_1.200.36.7; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA712314 = {S78765, 164, { &P22341,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MOT-XT800+/TTUPG_M6_2.200.15.1; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1882274 = {S78766, 162, { &P22341,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MOT-XT800+/TTUPG_M6_2.200.13; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1941514 = {S78767, 162, { &P22341,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MOT-XT800+/TTUPG_M6_2.200.21; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018746 = {S78768, 148, { &P22341,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; XT800+ Build/TTUPG_M6_2.200.15.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019059 = {S78769, 148, { &P22341,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; XT800+ Build/TTUPG_M6_1.200.36.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023139 = {S78770, 136, { &P22341,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; XT800+ Build/TTUPG_M6_2.200.15.1) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2023339 = {S78771, 146, { &P22341,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; XT800+ Build/TTUPG_M6_2.200.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA623166 = {S78772, 113, { &P15803,&P5843,&P17438,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia200/10.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15*/
const Device UA674235 = {S78773, 157, { &P16752,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SPICE Mi-720 Build/Spice_Mi-720_IND_V08_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA847403 = {S78774, 97, { &P12970,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Asus;Galaxy6)*/
const Device UA2857966 = {S78775, 112, { &P12970,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; WpsLondonTest; Asus;Galaxy6)*/
const Device UA861565 = {S78776, 158, { &P13251,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Dell Streak Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2695826 = {S78777, 158, { &P13251,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Dell Streak Build/GRI40; PureCyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028700 = {S78778, 135, { &P18631,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PMP5080CPRO Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2360713 = {S78779, 145, { &P23473,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; RMD-1035 Build/ICS.g12refM1005RU.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2383579 = {S78780, 178, { &P18644,&P5043,&P24672,&P18092 }};	/*Useragent - Nokia112/2.0 (03.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia112) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2383676 = {S78781, 129, { &P22355,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; iDxD7 3G Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2596988 = {S78782, 129, { &P22355,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; iDxD7 3G Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2425282 = {S78783, 137, { &P20932,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D2-911 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2508337 = {S78784, 97, { &P13815,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; HTC_Wildfire) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2585170 = {S78785, 138, { &P19865,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A850K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2600030 = {S78786, 138, { &P19865,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A850L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2600038 = {S78787, 138, { &P19865,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A850S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2599323 = {S78788, 95, { &P20004,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Karbonn_A2) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2599840 = {S78789, 143, { &P21733,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 3 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2699037 = {S78790, 131, { &P23164,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-7204 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2796769 = {S78791, 98, { &P21020,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Micromax A116) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2738308 = {S78792, 153, { &P18442,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2867977 = {S78793, 153, { &P18442,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L900 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA3073520 = {S78794, 153, { &P18442,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2739207 = {S78795, 94, { &P11964,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE75-1) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2997819 = {S78796, 94, { &P11964,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE75-2) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2770064 = {S78797, 137, { &P22599,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2896171 = {S78798, 156, { &P20983,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9308 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2849083 = {S78799, 156, { &P15972,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One XL Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2891964 = {S78800, 132, { &P23755,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; tr-tr; GT-N5105 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3023818 = {S78801, 93, { &P18283,&P18111,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9300) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA63753 = {S78802, 59, { &P13353,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-E2100B/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA10059 = {S78803, 63, { &P17330,&P5448,&P4626,&P18092 }};	/*Useragent - HTC-ST7377/2.10.502.4 (93577) Opera/9.7 (Windows NT 5.1; U; en)*/
const Device UA68411 = {S78804, 47, { &P11684,&P5129,&P4777,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UC Browser7.7.1.88/50/444*/
const Device UA1892492 = {S78805, 48, { &P11684,&P5129,&P4777,&P18092 }};	/*Useragent - nokia5800d-1/UC Browser7.7.1.88/69/444  3gpp-gba*/
const Device UA2526655 = {S78806, 54, { &P17512,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-CA3I UP.Browser/6.2_7.2.7.1.K.5.214 (GUI) MMP/2.0*/
const Device UA2589449 = {S78807, 100, { &P18267,&P5043,&P17347,&P18092 }};	/*Useragent - NokiaX2-02/2.0 (11.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-02/UC Browser8.0.3.107/69/444*/
const Device UA457083 = {S78808, 79, { &P15364,&P5690,&P22314,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; WOW64; rv:11.0) Gecko/X Firefox/11.0 SeaMonkey/2.8*/
const Device UA658579 = {S78809, 79, { &P15364,&P5690,&P22314,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:11.0) Gecko/X Firefox/11.0 SeaMonkey/2.8*/
const Device UA795935 = {S78810, 59, { &P15364,&P5690,&P22314,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko/X SeaMonkey/2.8*/
const Device UA845815 = {S78811, 61, { &P15364,&P18110,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv6l; U; en) Presto/2.8.115 Version/11.10*/
const Device UA857584 = {S78812, 136, { &P15389,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4;  U8800 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19*/
const Device UA911409 = {S78813, 135, { &P15389,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U8800 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2573842 = {S78814, 135, { &P15389,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; U8800 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA899898 = {S78815, 172, { &P15649,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352*/
const Device UA1986204 = {S78816, 171, { &P15649,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352*/
const Device UA2024570 = {S78817, 171, { &P15649,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2409250 = {S78818, 95, { &P15364,&P5690,&P22486,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.4) Gecko/X Firefox/2.0.0.4 Flock/0.9.0*/
const Device UA2225967 = {S78819, 139, { &P18307,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Malata SMBA1002 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2488236 = {S78820, 139, { &P18307,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Malata SMBA1002 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2422379 = {S78821, 124, { &P14764,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/352/UCWEB Mobile*/
const Device UA2449462 = {S78822, 73, { &P20100,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-E3309/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2458426 = {S78823, 141, { &P17108,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Inspire 4G Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2582475 = {S78824, 140, { &P17108,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Inspire 4G Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2459816 = {S78825, 137, { &P21773,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; SP704CE Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 TabletSafari/534.30*/
const Device UA2531184 = {S78826, 171, { &P18507,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2634070 = {S78827, 131, { &P17323,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P6800 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2634481 = {S78828, 131, { &P17323,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6800 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2636708 = {S78829, 138, { &P17323,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6800 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667111 = {S78830, 88, { &P20515,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; A54) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2676931 = {S78831, 136, { &P20864,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2710090 = {S78832, 129, { &P20864,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735405 = {S78833, 136, { &P20864,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; mid-A13 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2704735 = {S78834, 153, { &P18296,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One V Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2870153 = {S78835, 154, { &P18296,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One V Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2707288 = {S78836, 140, { &P23181,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NEBULA6.9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2707883 = {S78837, 156, { &P21356,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; TECNO N3 Build/master) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2718752 = {S78838, 93, { &P22607,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S5303) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2751404 = {S78839, 136, { &P21903,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; U705T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998169 = {S78840, 58, { &P21903,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; U705T Build/JZO54K)*/
const Device UA2756321 = {S78841, 134, { &P20480,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JY-G3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2885871 = {S78842, 138, { &P19862,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; Micromax A110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2893301 = {S78843, 170, { &P18645,&P18598,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I747 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA3014619 = {S78844, 162, { &P18645,&P18598,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L710 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2936930 = {S78845, 140, { &P23271,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-be; Lenco CoolTab-80 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2973910 = {S78846, 138, { &P20517,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CINK PEAX Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3033056 = {S78847, 162, { &P21020,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Micromax A116 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3022181 = {S78848, 138, { &P19902,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974070 = {S78849, 144, { &P24066,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Lenovo A706_ROW Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3015917 = {S78850, 92, { &P12280,&P19826,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 6_1_2; zh-CN; iPh4,1) U2/1.0.0 UCBrowser/9.1.0.287 U2/1.0.0 Mobile*/
const Device UA3016388 = {S78851, 92, { &P12280,&P19826,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 6_1_2; zh-CN; iPh3,1) U2/1.0.0 UCBrowser/9.1.0.287 U2/1.0.0 Mobile*/
const Device UA3017572 = {S78852, 92, { &P12280,&P19826,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 6_1_2; zh-CN; iPh5,2) U2/1.0.0 UCBrowser/9.1.0.287 U2/1.0.0 Mobile*/
const Device UA3021821 = {S78853, 139, { &P15972,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One XL Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022952 = {S78854, 139, { &P15972,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One XL Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3027498 = {S78855, 91, { &P15989,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i2) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3028850 = {S78856, 90, { &P15989,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3060873 = {S78857, 150, { &P20094,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825-1 Build/HuaweiU8825-1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3108737 = {S78858, 148, { &P20094,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3080371 = {S78859, 134, { &P19985,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SH-02E Build/S6210) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080865 = {S78860, 161, { &P21020,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Micromax A116 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3132631 = {S78861, 139, { &P18223,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA29010 = {S78862, 43, { &P20126,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonK800iv/R1KG SEMC/NewsReader/1.0*/
const Device UA50162 = {S78863, 62, { &P20126,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonK800i/R8BF Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA258454 = {S78864, 56, { &P18254,&P18570,&P17470,&P18091 }};	/*Useragent - SpiderSolitaireFree/1.5 CFNetwork/485.13.9 Darwin/11.0.0*/
const Device UA2334628 = {S78865, 57, { &P18254,&P18570,&P17470,&P18091 }};	/*Useragent - SpiderSolitaireFree/2.1.3 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2337087 = {S78866, 100, { &P12897,&P5843,&P17347,&P18092 }};	/*Useragent - NokiaC2-00/2.0 (03.99) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-00/UC Browser8.0.3.107/69/444*/
const Device UA2261419 = {S78867, 89, { &P13324,&P5055,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokian8-00) UCBrowser8.2.1.144/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA2310446 = {S78868, 32, { &P13324,&P5055,&P24667,&P18092 }};	/*Useragent - NokiaN8-00/UCWEB8.2.0.116/51/999*/
const Device UA644075 = {S78869, 116, { &P12056,&P5204,&P17347,&P18092 }};	/*Useragent - Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7210supernova/UC Browser8.0.3.107/69/444*/
const Device UA665438 = {S78870, 153, { &P21107,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-01D Build/4.0.1.C.1.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA669126 = {S78871, 153, { &P21107,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-01D Build/4.0.1.C.0.86) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1951784 = {S78872, 141, { &P21107,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SO-01D Build/4.0.1.C.1.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2387324 = {S78873, 153, { &P21107,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonSO-01D Build/4.0.1.C.1.31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA803093 = {S78874, 79, { &P13163,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaN97)/UC Browser8.2.0.132/50/355/UCWEB Mobile*/
const Device UA1963994 = {S78875, 85, { &P14980,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-b7722) UCBrowser8.3.1.161/69/405/UCWEB Mobile*/
const Device UA1994128 = {S78876, 85, { &P14980,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-b7722) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2028471 = {S78877, 134, { &P18238,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; E330 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2256907 = {S78878, 171, { &P17324,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2401653 = {S78879, 171, { &P17324,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2776058 = {S78880, 171, { &P17324,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.0.181/145/352*/
const Device UA2837517 = {S78881, 171, { &P17324,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2237681 = {S78882, 157, { &P18513,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2833595 = {S78883, 155, { &P18513,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2242617 = {S78884, 57, { &P20038,&P22784,&P21795,&P18092 }};	/*Useragent - MicromaxX278/MstarBrowser1.0/MIDP2.0Configuration/CLDC1.1*/
const Device UA2262148 = {S78885, 154, { &P15575,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2466288 = {S78886, 154, { &P15575,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2842496 = {S78887, 155, { &P15575,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2354897 = {S78888, 141, { &P22066,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZIGO EON5I Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556129 = {S78889, 136, { &P22066,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EON5I Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2421922 = {S78890, 73, { &P18382,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC_Desire_C; 320*480) UCWEB7.9.0.94/139/352*/
const Device UA2423239 = {S78891, 73, { &P18382,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC_Desire_C; 320*480) UCWEB7.9.0.94/139/355*/
const Device UA2427653 = {S78892, 73, { &P18382,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC_Desire_C; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2422838 = {S78893, 131, { &P23847,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; ADT 2.5 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2425032 = {S78894, 139, { &P20132,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST70104-1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2428373 = {S78895, 158, { &P20134,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2842550 = {S78896, 86, { &P20134,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Adr (Linux; U; 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD.ZCLA4;320*480)*/
const Device UA2952149 = {S78897, 86, { &P20134,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Adr (Linux; U; 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD.ZCLL3;320*480)*/
const Device UA2465949 = {S78898, 118, { &P15364,&P16110,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.2.149.27 Safari/525.13*/
const Device UA2657996 = {S78899, 118, { &P15364,&P16110,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.4.154.25 Safari/525.19*/
const Device UA2844145 = {S78900, 103, { &P15364,&P16110,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/0.0.0.0 Safari/534.24*/
const Device UA2546134 = {S78901, 142, { &P23282,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I659 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2696295 = {S78902, 143, { &P23282,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M460D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2557842 = {S78903, 155, { &P16356,&P18066,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-N7000 Build/IML74K; XPath_Offline) AppleWebKit/533.1 (KHTML, like Gecko) Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2602244 = {S78904, 136, { &P23809,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CT910 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2603415 = {S78905, 143, { &P22060,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Celkon A 225 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2613410 = {S78906, 136, { &P15919,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2614707 = {S78907, 136, { &P15919,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; GT-P3100 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7*/
const Device UA2671107 = {S78908, 137, { &P21506,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2701247 = {S78909, 135, { &P20255,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; R8113 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2707035 = {S78910, 152, { &P18644,&P5043,&P24677,&P18092 }};	/*Useragent - Nokia112/2.0 (03.32) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia112) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2782631 = {S78911, 137, { &P18722,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A830K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783356 = {S78912, 137, { &P18722,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A830L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2796965 = {S78913, 137, { &P18722,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A830S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2852255 = {S78914, 55, { &P15364,&P6729,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Win95; Trident/4.0)*/
const Device UA2866653 = {S78915, 139, { &P15820,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-SU640 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3018085 = {S78916, 139, { &P24105,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; Archos 80 Xenon Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3095731 = {S78917, 155, { &P21648,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9505 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3114407 = {S78918, 154, { &P12280,&P22882,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 7_0 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/11A465 Safari/7534.48.3*/
const Device UA3130925 = {S78919, 156, { &P12280,&P22882,&P18486,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 7_0_2 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/11A501 Safari/7534.48.3*/
const Device UA84 = {S78920, 89, { &P13265,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Samsung-SCHI910 PPC 240x400*/
const Device UA6007 = {S78921, 86, { &P13265,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) VZW:SCH-i910 PPC 240x400*/
const Device UA6368 = {S78922, 93, { &P13265,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Samsung-SCHI910 BMC PPC 240x400*/
const Device UA2390 = {S78923, 69, { &P15364,&P5448,&P17173,&P18092 }};	/*Useragent - Treo750 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA4257 = {S78924, 76, { &P15364,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) T-Mobile_Atlas*/
const Device UA22620 = {S78925, 120, { &P15364,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) PPC; MDA Touch/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA47346 = {S78926, 71, { &P15364,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) HTC/X02HT*/
const Device UA86903 = {S78927, 122, { &P15364,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) 320x320; VZW; UTStar-XV6800; Window Mobile 6.0 Professional;*/
const Device UA2268296 = {S78928, 89, { &P15364,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Holleycomm-H8800/2.0 WAP2.0*/
const Device UA62794 = {S78929, 83, { &P11858,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonZ710i/R1JC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61700 = {S78930, 83, { &P12016,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW900i/R5BC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA50308 = {S78931, 109, { &P13223,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux ARM; en-us) AppleWebKit/525.1+ (KHTML, like Gecko) Safari/525 Mobile/MotorolaEvoke*/
const Device UA675986 = {S78932, 61, { &P15364,&P6704,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; en) Opera 8.53*/
const Device UA713801 = {S78933, 61, { &P15364,&P6704,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; ru) Opera 8.50*/
const Device UA2437801 = {S78934, 61, { &P15364,&P6704,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; en) Opera 8.52*/
const Device UA2437824 = {S78935, 61, { &P15364,&P6704,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; ru) Opera 8.54*/
const Device UA603070 = {S78936, 64, { &P17976,&P17979,&P18566,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:13.0) Gecko/13.0 Firefox/13.0a2*/
const Device UA614791 = {S78937, 74, { &P17976,&P17979,&P18566,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:13.0) Gecko/13.0 Firefox/13.0 Fennec/13.0*/
const Device UA695525 = {S78938, 118, { &P15364,&P16110,&P22455,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11*/
const Device UA820579 = {S78939, 125, { &P15364,&P16110,&P22455,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11*/
const Device UA626843 = {S78940, 129, { &P21819,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; 101N Build/AX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027308 = {S78941, 53, { &P21819,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; 101N Build/AX)*/
const Device UA654435 = {S78942, 148, { &P14604,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB200 Build/GRJ22; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1970941 = {S78943, 142, { &P18080,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; IM-A725L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2176715 = {S78944, 131, { &P21144,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; F8 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2219272 = {S78945, 128, { &P12973,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A101 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2505375 = {S78946, 158, { &P12973,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2426464 = {S78947, 138, { &P20152,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; OPPOT29 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2468618 = {S78948, 169, { &P18318,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ASUS Transformer Pad TF700T Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2599229 = {S78949, 168, { &P18318,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ASUS Transformer Pad TF700T Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3019386 = {S78950, 177, { &P18318,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ASUS Transformer Pad TF700T Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1.2/tf700t*/
const Device UA2499092 = {S78951, 157, { &P12249,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MiniPro Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2669175 = {S78952, 64, { &P15364,&P5690,&P22167,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:20.0) Gecko/X Firefox/20.0 slurp*/
const Device UA2707085 = {S78953, 70, { &P15364,&P5690,&P22167,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:20.0) Gecko/X Firefox/20.0 slurp slurp*/
const Device UA2798684 = {S78954, 93, { &P18283,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9300) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2738171 = {S78955, 159, { &P16292,&P18598,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; Galaxy Nexus Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752482 = {S78956, 160, { &P16292,&P18598,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Galaxy Nexus Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2964574 = {S78957, 72, { &P12280,&P19826,&P24532,&P18092 }};	/*Useragent - Instagram 4.0.2 (iPhone4,1; iPhone OS 6_1_3; en_US; en) AppleWebKit/420+*/
const Device UA2966314 = {S78958, 72, { &P12280,&P19826,&P24532,&P18092 }};	/*Useragent - Instagram 4.0.2 (iPhone5,2; iPhone OS 6_1_4; en_US; en) AppleWebKit/420+*/
const Device UA2897726 = {S78959, 161, { &P18507,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I8160 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2975804 = {S78960, 134, { &P20480,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; JY-G3 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3092424 = {S78961, 135, { &P20480,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; JY-G3N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022370 = {S78962, 148, { &P17835,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook Tablet Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3024092 = {S78963, 132, { &P17835,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; NookTablet Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2975576 = {S78964, 138, { &P18707,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MD_LIFETAB_P9516 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3060988 = {S78965, 155, { &P13644,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; EVO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jewel*/
const Device UA49053 = {S78966, 47, { &P15364,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-AX8370/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA84058 = {S78967, 46, { &P15364,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-MX200/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA84060 = {S78968, 65, { &P15364,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-MX500/1.0 UP.Browser/6.2.3.2 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA86658 = {S78969, 46, { &P15364,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-AX245/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA86688 = {S78970, 46, { &P15364,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-MX510/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA87425 = {S78971, 42, { &P15364,&P17483,&P3625,&P18092 }};	/*Useragent - LG285/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA87619 = {S78972, 47, { &P15364,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-MX8550/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA3049779 = {S78973, 46, { &P15364,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-UN160/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA66655 = {S78974, 71, { &P12682,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_P3490 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA87384 = {S78975, 77, { &P12682,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Sprint MP6950SP*/
const Device UA87099 = {S78976, 126, { &P16951,&P5448,&P22270,&P18092 }};	/*Useragent - Treo800w/v0100 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE, PPC; 320x320) (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA412741 = {S78977, 42, { &P15364,&P18110,&P18397,&P18092 }};	/*Useragent - Opera/6.11 (Linux 2.4.10-4GB i686; U) [en]*/
const Device UA412744 = {S78978, 42, { &P15364,&P18110,&P18397,&P18092 }};	/*Useragent - Opera/6.11 (Linux 2.4.18-4GB i686; U) [en]*/
const Device UA683240 = {S78979, 73, { &P15364,&P18110,&P18397,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Linux 2.4.20-4GB i686) Opera 6.12 [de]*/
const Device UA794048 = {S78980, 44, { &P15364,&P18110,&P18397,&P18092 }};	/*Useragent - Opera/6.12 (Linux 2.4.18-14cpq i686; U) [en]*/
const Device UA825201 = {S78981, 42, { &P15364,&P18110,&P18397,&P18092 }};	/*Useragent - Opera/6.12 (Linux 2.4.20-4GB i686; U) [en]*/
const Device UA848417 = {S78982, 68, { &P15364,&P18110,&P18397,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Linux 2.4.4 i686) Opera 6.11 [en]*/
const Device UA572918 = {S78983, 127, { &P13051,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-T U880 Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1983086 = {S78984, 132, { &P13051,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-T U880 Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2034345 = {S78985, 132, { &P13051,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-T U880 Build FRG83G) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA714527 = {S78986, 100, { &P15364,&P22191,&P19773,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1.15) Gecko/X Fedora/2.0.10-1.fc12 SeaMonkey/2.0.10*/
const Device UA835902 = {S78987, 98, { &P15364,&P22191,&P19773,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.19) Gecko/X Fedora/2.0.14-1.fc14 SeaMonkey/2.0.14*/
const Device UA717033 = {S78988, 131, { &P21370,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHV-E140L Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA719900 = {S78989, 138, { &P23900,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-gb; ELONEX1012ET Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1973274 = {S78990, 50, { &P18095,&P5890,&P21761,&P18092 }};	/*Useragent - iBrowser/2.5 (NokiaNXX;SymbianOS/9.3 Series60/3.2)*/
const Device UA2032607 = {S78991, 134, { &P23926,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GN100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2445397 = {S78992, 127, { &P23926,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GN100 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2686769 = {S78993, 58, { &P23926,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; GN100 Build/GRK39F)*/
const Device UA2232930 = {S78994, 135, { &P21112,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; X9017 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2336886 = {S78995, 134, { &P21112,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; X907 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2664452 = {S78996, 153, { &P23720,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; O+ 8.3 Android Build/OplusOplus_A100p) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2699373 = {S78997, 153, { &P23720,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; O+_8.3_Android Build/OplusOplus_A100p) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2666422 = {S78998, 131, { &P21990,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; chagall Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2673355 = {S78999, 138, { &P22590,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717M Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2693159 = {S79000, 150, { &P15841,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/2.1.0.131)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2732763 = {S79001, 151, { &P15841,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/10.0.0.336)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2879138 = {S79002, 151, { &P15841,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-us; PlayBook Build/10.1.0.103)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3103525 = {S79003, 151, { &P15841,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayBook; U; Android 2.3.3; en-gb; PlayBook Build/10.1.0.123)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2754411 = {S79004, 137, { &P20240,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Optimus G Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754734 = {S79005, 138, { &P23969,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2772993 = {S79006, 157, { &P18296,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC One V Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2774328 = {S79007, 164, { &P22582,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2773844 = {S79008, 154, { &P22856,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0151 Build/HuaweiY300-0151) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2974896 = {S79009, 151, { &P20002,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 992D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2997390 = {S79010, 152, { &P19771,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3005494 = {S79011, 152, { &P19771,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035D Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3049514 = {S79012, 152, { &P19771,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035E Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3022998 = {S79013, 141, { &P24635,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Orbit 5400 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3081690 = {S79014, 138, { &P18707,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MD_LIFETAB_P9516 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3083067 = {S79015, 136, { &P20864,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094533 = {S79016, 129, { &P20864,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA32198 = {S79017, 79, { &P12479,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-T339/T339UVHD8 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA67688 = {S79018, 161, { &P17228,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; PocketEdge DINGO-1.13; Android 1.6; en-us; edgejr Build/DRD20) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA104417 = {S79019, 48, { &P15364,&P21442,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 2000)*/
const Device UA2462649 = {S79020, 48, { &P15364,&P21442,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Windows 2000)*/
const Device UA391110 = {S79021, 161, { &P13700,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MOT-ME811/SHDWR_X6_2.200.25; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA763006 = {S79022, 161, { &P13700,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MOT-ME811/SHDWR_X6_2.200.12; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA801674 = {S79023, 161, { &P13700,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MOT-ME811/SHDWR_X6_2.200.33; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022257 = {S79024, 161, { &P13700,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MOT-ME811/SHDWR_X6_2.200.36; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028948 = {S79025, 135, { &P21359,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; iBall i6012 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2230777 = {S79026, 135, { &P21359,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; iBall i6012 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2145455 = {S79027, 62, { &P15364,&P16110,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; U; en) Presto/2.2.15 Version/10.00*/
const Device UA2759222 = {S79028, 62, { &P15364,&P16110,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; U; en) Presto/2.2.15 Version/10.01*/
const Device UA2933185 = {S79029, 48, { &P15364,&P16110,&P4812,&P18092 }};	/*Useragent - Opera/10.00 (Windows NT 6.2; U; en) Presto/2.2.0*/
const Device UA2348751 = {S79030, 150, { &P15364,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; MG-391 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2933687 = {S79031, 147, { &P15364,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; W80 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042920 = {S79032, 155, { &P15364,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A326 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2403648 = {S79033, 182, { &P14848,&P5843,&P24672,&P18092 }};	/*Useragent - NokiaX3-00/5.0 (08.54) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax3-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile*/
const Device UA2431506 = {S79034, 37, { &P15795,&P5448,&P20765,&P18092 }};	/*Useragent - NOKIA-Lumia 710/UCWEB2.7.0.223/47/355*/
const Device UA2672412 = {S79035, 158, { &P12972,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T989 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA3101404 = {S79036, 158, { &P12972,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T989 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2537305 = {S79037, 116, { &P15364,&P5690,&P19963,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17 Sleipnir/3.9.1*/
const Device UA2571061 = {S79038, 91, { &P14545,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-x330t) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2704793 = {S79039, 90, { &P14545,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-x330) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2599870 = {S79040, 159, { &P12040,&P17483,&P24676,&P18092 }};	/*Useragent - LG-GW300/V100 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gw300) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2671991 = {S79041, 141, { &P19997,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo P700i Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708721 = {S79042, 140, { &P19997,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Lenovo P700 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2705297 = {S79043, 134, { &P21186,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MPDC100 BT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2716535 = {S79044, 151, { &P19996,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A15 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2733090 = {S79045, 135, { &P23086,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Fusion Bolt Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2955284 = {S79046, 64, { &P23086,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; Fusion Bolt Build/JRO03C)*/
const Device UA2754164 = {S79047, 129, { &P20750,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AN10DG3 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2769437 = {S79048, 148, { &P17835,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook Tablet Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2937840 = {S79049, 132, { &P17835,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; NookTablet Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2877951 = {S79050, 135, { &P23214,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; sv-se; C6833 Build/14.1.B.1.278) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2877953 = {S79051, 133, { &P23214,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-; C6833 Build/14.1.B.1.278) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2877960 = {S79052, 135, { &P23214,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; sv-se; C6833 Build/14.1.B.1.320) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2877958 = {S79053, 135, { &P23214,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; sv-se; C6833 Build/14.1.B.1.298) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3109007 = {S79054, 64, { &P23214,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; C6833 Build/14.1.B.1.510)*/
const Device UA3125438 = {S79055, 139, { &P23214,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SonyC6833 Build/14.1.B.1.510) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3125560 = {S79056, 135, { &P23214,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; C6833 Build/14.1.B.1.510) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2879642 = {S79057, 77, { &P23493,&P22784,&P3680,&P18092 }};	/*Useragent - Micromax X351/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2898812 = {S79058, 91, { &P23545,&P17017,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mauie800) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2901098 = {S79059, 114, { &P12897,&P5843,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-00/03.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA2975538 = {S79060, 146, { &P19863,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974309 = {S79061, 134, { &P19985,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SH-02E Build/S4260) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022554 = {S79062, 142, { &P15438,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Sensation Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023438 = {S79063, 138, { &P15438,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Sensation Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2977682 = {S79064, 109, { &P17595,&P18120,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.1 Safari/533.3*/
const Device UA3017139 = {S79065, 137, { &P24063,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; IdeaTab S6000-F Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3057493 = {S79066, 136, { &P24063,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; IdeaTab S6000-H Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3026431 = {S79067, 187, { &P13157,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3027870 = {S79068, 158, { &P22641,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; C1504 Build/11.3.A.0.47) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3080190 = {S79069, 158, { &P22641,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; C1504 Build/11.3.A.0.59) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3029878 = {S79070, 135, { &P24170,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; T1-E Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3083339 = {S79071, 138, { &P22591,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA24621 = {S79072, 62, { &P13270,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia5140i/2.0 (03.80) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21324 = {S79073, 141, { &P12519,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB501 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA44392 = {S79074, 149, { &P12519,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB501 Build/GroovyG's 2.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA44396 = {S79075, 152, { &P12519,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MB501 Build/ZPSKT_U3_21.24.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA27963 = {S79076, 51, { &P12194,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch2_T3320 Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA63192 = {S79077, 51, { &P12194,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch2_T3330 Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA30854 = {S79078, 41, { &P12356,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-R211 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA1959973 = {S79079, 188, { &P15364,&P7133,&P18524,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/69/352*/
const Device UA1993351 = {S79080, 188, { &P15364,&P7133,&P18524,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/70/352*/
const Device UA2163638 = {S79081, 188, { &P15364,&P7133,&P18524,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/69/444*/
const Device UA2375844 = {S79082, 188, { &P15364,&P7133,&P18524,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/70/444*/
const Device UA2445618 = {S79083, 198, { &P15364,&P7133,&P18524,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/69/352  3gpp-gba*/
const Device UA2445739 = {S79084, 198, { &P15364,&P7133,&P18524,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/70/352  3gpp-gba*/
const Device UA2649330 = {S79085, 198, { &P15364,&P7133,&P18524,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/69/444  3gpp-gba*/
const Device UA2225585 = {S79086, 138, { &P18506,&P5043,&P24667,&P18092 }};	/*Useragent - Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia305) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2230766 = {S79087, 64, { &P15364,&P5690,&P18428,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:16.0) Gecko/X Firefox/16.0 slurp*/
const Device UA2354898 = {S79088, 70, { &P15364,&P5690,&P18428,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:16.0) Gecko/X Firefox/16.0 slurp slurp*/
const Device UA2387215 = {S79089, 145, { &P17968,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Full Android on Grouper Build/JOP40C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2404450 = {S79090, 127, { &P17968,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Build/JOP40C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2937123 = {S79091, 129, { &P17968,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Build/JDQ39E) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2428055 = {S79092, 68, { &P18241,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; LG-E400; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2424817 = {S79093, 31, { &P18643,&P5043,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia311/03.81)*/
const Device UA2469871 = {S79094, 173, { &P14202,&P5477,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E15i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2429137 = {S79095, 155, { &P18464,&P18066,&P19804,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2527223 = {S79096, 158, { &P11950,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Mini Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2696983 = {S79097, 157, { &P11950,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia X10 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2600161 = {S79098, 87, { &P17595,&P17297,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Macintosh;Mac OS X (10.7.5);en;kml:2.2;client:Free;type:default)*/
const Device UA2847803 = {S79099, 87, { &P17595,&P17297,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.2.8415(Macintosh;Mac OS X (10.7.5);en;kml:2.2;client:Free;type:default)*/
const Device UA2861884 = {S79100, 87, { &P17595,&P17297,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Macintosh;Mac OS X (10.7.5);de;kml:2.2;client:Free;type:default)*/
const Device UA2621910 = {S79101, 132, { &P13452,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P1000 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2623868 = {S79102, 61, { &P13452,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-P1000 Build/JOP40D)*/
const Device UA2994763 = {S79103, 131, { &P13452,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P1000 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2996116 = {S79104, 61, { &P13452,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P1000 Build/JDQ39E)*/
const Device UA2705525 = {S79105, 138, { &P22660,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-sa; LG-P778 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734593 = {S79106, 139, { &P15364,&P17819,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu/11.10 Chromium/25.0.1364.172 Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2751274 = {S79107, 141, { &P15364,&P17819,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.22 (KHTML, like Gecko) Ubuntu/11.10 Chromium/25.0.1364.172 Chrome/25.0.1364.172 Safari/537.22*/
const Device UA2738177 = {S79108, 132, { &P23155,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PRAXIS 13 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2784879 = {S79109, 156, { &P14574,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3088618 = {S79110, 156, { &P14574,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus S Build/JRO03E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2868301 = {S79111, 143, { &P23461,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-us; Spice_Mi-422 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879672 = {S79112, 138, { &P15991,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2891566 = {S79113, 110, { &P18435,&P21460,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.3; ARM; Trident/7.0; Touch; .NET4.0E; .NET4.0C; Tablet PC 2.0)*/
const Device UA2964025 = {S79114, 137, { &P23691,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5280 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3077858 = {S79115, 130, { &P23691,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5280 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2975671 = {S79116, 136, { &P22805,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E455 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2972145 = {S79117, 133, { &P20145,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; NS-13T001 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2999714 = {S79118, 135, { &P24102,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SM-C101 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3034474 = {S79119, 161, { &P21020,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Micromax A116 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3083384 = {S79120, 139, { &P15972,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One XL Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094805 = {S79121, 139, { &P15972,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One XL Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA62187 = {S79122, 103, { &P11661,&P5795,&P3978,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Series80/2.0 Nokia9500/4.51 Profile/MIDP-2.0 Configuration/CLDC-1.1)*/
const Device UA24634 = {S79123, 100, { &P17165,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-T639/UVGG1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA64399 = {S79124, 74, { &P14594,&P6476,&P17470,&P18092 }};	/*Useragent - LG-LX260 POLARIS-LX260/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA351777 = {S79125, 88, { &P15364,&P17285,&P21902,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/16.0.912.63)*/
const Device UA357715 = {S79126, 162, { &P15364,&P17285,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.2; chromeframe/16.0.912.63; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; InfoPath.2)*/
const Device UA366523 = {S79127, 130, { &P15364,&P17285,&P21902,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/16.0.912.75) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7*/
const Device UA367699 = {S79128, 165, { &P15364,&P17285,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/16.0.912.75; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA368133 = {S79129, 123, { &P15364,&P17285,&P21902,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/16.0.912.77) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA380861 = {S79130, 194, { &P15364,&P17285,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB7.2; chromeframe/16.0.912.77; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA398445 = {S79131, 107, { &P15364,&P17285,&P21902,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; BOIE9;ENUS; chromeframe/16.0.912.75)*/
const Device UA635465 = {S79132, 123, { &P15364,&P17285,&P21902,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/16.0.912.63) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA1935717 = {S79133, 163, { &P15364,&P17285,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/16.0.912.63; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)*/
const Device UA2245671 = {S79134, 214, { &P15364,&P17285,&P21902,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; chromeframe/16.0.912.77; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; MDDC; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C; BRI/1; X; Version/11.00302)*/
const Device UA397645 = {S79135, 116, { &P15364,&P18110,&P22454,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7*/
const Device UA397801 = {S79136, 114, { &P15364,&P18110,&P22454,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7*/
const Device UA436721 = {S79137, 52, { &P18088,&P17017,&P24663,&P18092 }};	/*Useragent - IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB7.9.0.94/41/997*/
const Device UA1895816 = {S79138, 52, { &P18088,&P17017,&P24663,&P18092 }};	/*Useragent - IUC(U;iOS 4.2.1;Zh-cn;320*480;)/UCWEB7.9.0.94/41/997*/
const Device UA1911756 = {S79139, 52, { &P18088,&P17017,&P24663,&P18092 }};	/*Useragent - IUC(U;iOS 5.0.1;Zh-cn;320*480;)/UCWEB7.9.0.94/41/997*/
const Device UA2400976 = {S79140, 41, { &P18088,&P17017,&P24663,&P18092 }};	/*Useragent - MAUI_WAP_Browser/UCWEB7.9.0.114/109/33068*/
const Device UA2450380 = {S79141, 41, { &P18088,&P17017,&P24663,&P18092 }};	/*Useragent - MAUI_WAP_Browser/UCWEB7.9.0.114/109/33224*/
const Device UA2030522 = {S79142, 167, { &P13999,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC HD2 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2383496 = {S79143, 167, { &P13999,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC HD2 Build/JRO03R; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032716 = {S79144, 137, { &P21025,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; IS11LG Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034594 = {S79145, 157, { &P16954,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Dell Venue Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2169939 = {S79146, 74, { &P15364,&P22470,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Darwin i386; en-US; rv:1.9.0.8) Gecko/X Firefox/3.0.8*/
const Device UA2237960 = {S79147, 79, { &P17968,&P5062,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 1.6; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2362797 = {S79148, 155, { &P18466,&P18066,&P19815,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2658214 = {S79149, 153, { &P18295,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC One X with Beats Audio Build/GRJ90) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2385875 = {S79150, 202, { &P12139,&P5204,&P24672,&P18092 }};	/*Useragent - Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5310xpressmusic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2386547 = {S79151, 134, { &P18721,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A90S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2421519 = {S79152, 140, { &P14922,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-LU6800 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2431869 = {S79153, 135, { &P18675,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SH-01E Build/SA310) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2483149 = {S79154, 141, { &P22004,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ALLVIEW P6 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2560166 = {S79155, 102, { &P11932,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA6210 Navigator) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2566644 = {S79156, 137, { &P18438,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E973 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583691 = {S79157, 143, { &P21020,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Micromax A116 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2676562 = {S79158, 124, { &P15364,&P16110,&P19995,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22 Sleipnir/4.0.1*/
const Device UA2863150 = {S79159, 117, { &P15364,&P16110,&P19995,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22 Sleipnir/4.0.1*/
const Device UA2669408 = {S79160, 161, { &P16292,&P18598,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2704460 = {S79161, 162, { &P16292,&P18598,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2675837 = {S79162, 130, { &P18702,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8013 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2736725 = {S79163, 93, { &P22609,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6312) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2774137 = {S79164, 158, { &P20076,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26w Build/6.1.A.2.45) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3022700 = {S79165, 136, { &P18281,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P940 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022816 = {S79166, 138, { &P18281,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-KU5400 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3058088 = {S79167, 137, { &P18281,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-SU540 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3032263 = {S79168, 162, { &P19862,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax A110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2974964 = {S79169, 137, { &P23167,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-H9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2968246 = {S79170, 135, { &P24129,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; A900 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022177 = {S79171, 143, { &P16044,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CnM TouchPad 9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022335 = {S79172, 138, { &P16044,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cnm TOUCHPAD 9.7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022529 = {S79173, 143, { &P16044,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CnM TouchPad 7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023678 = {S79174, 146, { &P16044,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Versus TouchPad 7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3093101 = {S79175, 164, { &P24065,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ar-ae; SAMSUNG GT-S7272 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA5185 = {S79176, 78, { &P12220,&P5448,&P17173,&P18092 }};	/*Useragent - SAMSUNG-SGH-i600ORANGE/BVGD2 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA14360 = {S79177, 84, { &P12220,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) SAMSUNG-SGH-i600/WM534*/
const Device UA22616 = {S79178, 70, { &P12220,&P5448,&P17173,&P18092 }};	/*Useragent - SAMSUNG-SGH-i600/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA54290 = {S79179, 83, { &P12220,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) SAMSUNG-SGH-i600V/1.0*/
const Device UA32088 = {S79180, 136, { &P12331,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; E130 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA9137 = {S79181, 97, { &P13391,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-880/1.0 model-orange Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA31498 = {S79182, 85, { &P13391,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-880A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA64438 = {S79183, 84, { &P13391,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-OT-880/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA60518 = {S79184, 120, { &P17968,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA222604 = {S79185, 128, { &P17968,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Build/IRK40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA318109 = {S79186, 35, { &P17968,&P6546,&P4459,&P18092 }};	/*Useragent - stagefright/1.1 (Linux;Android 3.1)*/
const Device UA833218 = {S79187, 121, { &P17968,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Build/HTJ70C) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA69390 = {S79188, 82, { &P15207,&P5448,&P4779,&P18092 }};	/*Useragent - HTC_P3400i-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA562506 = {S79189, 93, { &P15207,&P5448,&P4779,&P18092 }};	/*Useragent - HTC_P3400-Mozilla/4.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA514752 = {S79190, 91, { &P15364,&P5690,&P22332,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1) Gecko/X Firefox/3.5 CometBird/3.5*/
const Device UA656605 = {S79191, 85, { &P15197,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-b5310) UCBrowser8.2.1.144/69/352/UCWEB Mobile*/
const Device UA1963507 = {S79192, 85, { &P15197,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-b5310) UCBrowser8.2.0.132/69/352/UCWEB Mobile*/
const Device UA699021 = {S79193, 136, { &P21818,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; IT-M809RC Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA848524 = {S79194, 99, { &P13868,&P18148,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 8.0; Trident/5.0; IEMobile/9.0; HTC; HD7 T9292)*/
const Device UA2857965 = {S79195, 114, { &P13868,&P18148,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 8.0; Trident/5.0; IEMobile/9.0; WpsLondonTest; HTC; HD7 T9292)*/
const Device UA883702 = {S79196, 138, { &P21216,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; Micromax A78 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2411292 = {S79197, 131, { &P21216,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; A78 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1890739 = {S79198, 112, { &P15364,&P7127,&P17335,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-en) AppleWebKit/533.16 (KHTML, like Gecko) Version/4.1 Safari/533.16*/
const Device UA1988792 = {S79199, 136, { &P21233,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; TECNO T1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2767162 = {S79200, 140, { &P21233,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; T1 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2211688 = {S79201, 97, { &P15364,&P17285,&P22478,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.8.1.13) Gecko/X Firefox/2.0.0.13 Flock/1.1.1*/
const Device UA2361780 = {S79202, 114, { &P15364,&P7133,&P18439,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11 slurp*/
const Device UA2405267 = {S79203, 114, { &P15364,&P7133,&P18439,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.95 Safari/537.11 slurp*/
const Device UA2434787 = {S79204, 114, { &P15364,&P7133,&P18439,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11 slurp*/
const Device UA2401263 = {S79205, 120, { &P15845,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaC5-06) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/444/UCWEB Mobile*/
const Device UA2422730 = {S79206, 163, { &P21360,&P18066,&P18062,&P18092 }};	/*Useragent - iBall Andi4.3J+ Linux/3.0.13 Android/4.0.4 Release/10.25.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2696940 = {S79207, 146, { &P21360,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iBall Andi4.3J+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2468970 = {S79208, 149, { &P20300,&P18066,&P21729,&P18092 }};	/*Useragent - OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; W60 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2510614 = {S79209, 37, { &P18623,&P5448,&P20770,&P18092 }};	/*Useragent - NOKIA-Lumia 510/UCWEB2.8.0.249/47/355*/
const Device UA2583169 = {S79210, 131, { &P20035,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2671148 = {S79211, 131, { &P18475,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320T Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676502 = {S79212, 130, { &P18475,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7320 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2672920 = {S79213, 144, { &P20007,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT886 Build/6.7.3-179_YIC-10) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2703331 = {S79214, 137, { &P22660,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P778 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2784840 = {S79215, 154, { &P18635,&P18598,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; Nexus 4 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2782301 = {S79216, 138, { &P23143,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT-7245 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2782580 = {S79217, 139, { &P13018,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT11i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2957622 = {S79218, 139, { &P13018,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MT11i Build/4.1.A.0.562) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2873371 = {S79219, 134, { &P22836,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SC-04E Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3104016 = {S79220, 127, { &P22836,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SC-04E Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2942652 = {S79221, 133, { &P15575,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3121881 = {S79222, 134, { &P15575,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-I9100G Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2943128 = {S79223, 139, { &P23680,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AURUS III Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2967599 = {S79224, 78, { &P15364,&P22784,&P3680,&P18092 }};	/*Useragent - Micromax X088/R2AE  SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA3092965 = {S79225, 77, { &P15364,&P22784,&P3680,&P18092 }};	/*Useragent - Micromax X455/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA3126519 = {S79226, 78, { &P15364,&P22784,&P3680,&P18092 }};	/*Useragent - Micromax X455i/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2996854 = {S79227, 137, { &P24102,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; SM-C101 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3020291 = {S79228, 59, { &P24102,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; SM-C101 Build/JDQ39)*/
const Device UA3037205 = {S79229, 152, { &P20868,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M040 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3083832 = {S79230, 123, { &P21239,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; M9 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080662 = {S79231, 132, { &P13644,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; EVO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA548994 = {S79232, 91, { &P15364,&P22201,&P20803,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/4.3; Linux) KHTML/4.3.1 (like Gecko) Fedora/4.3.1-3.fc11*/
const Device UA702996 = {S79233, 143, { &P21331,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; de-de; Odys-Loox Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1893571 = {S79234, 39, { &P17595,&P18120,&P22891,&P18092 }};	/*Useragent - iCab/4.0 (Macintosh; U; Intel Mac OS X)*/
const Device UA2030349 = {S79235, 148, { &P21127,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Mobiistar Touch S02 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2217831 = {S79236, 82, { &P13991,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Mail; 1.4.4) (mecha; HTC; ADR6400L; 2.3.4/GRJ22)*/
const Device UA2385723 = {S79237, 190, { &P13337,&P17485,&P24672,&P18092 }};	/*Useragent - SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonj108i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2461249 = {S79238, 165, { &P18241,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; LG-E400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2428121 = {S79239, 126, { &P20263,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; C925 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2437233 = {S79240, 47, { &P15364,&P18580,&P22326,&P18092 }};	/*Useragent - Rockmelt/2.1.0.1310 CFNetwork/609 Darwin/13.0.0*/
const Device UA2560313 = {S79241, 140, { &P21971,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Aqua Trendy Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2584163 = {S79242, 142, { &P24137,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; myTab 10 Dual Core Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2675279 = {S79243, 130, { &P12879,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7510 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2766785 = {S79244, 130, { &P12879,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P7510 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2666597 = {S79245, 154, { &P23280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar Touch Kem 402 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955977 = {S79246, 152, { &P23280,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-; Mobiistar Touch Kem 402 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666996 = {S79247, 139, { &P23061,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; TECNO_B3 Build/TECNO_B3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2701425 = {S79248, 171, { &P20192,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2850493 = {S79249, 100, { &P20192,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; HUAWEI G510-0010 Build/G510-0010V100R001C17B168;480*854)*/
const Device UA2988034 = {S79250, 100, { &P20192,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; HUAWEI G510-0010 Build/G510-0010V100R001C17B173;480*854)*/
const Device UA2713953 = {S79251, 156, { &P23728,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; O+ 8.52 Android Build/Oplus8.52_Android) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2753389 = {S79252, 138, { &P18641,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-M830 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2774056 = {S79253, 150, { &P18702,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8013 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2772300 = {S79254, 134, { &P22836,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SC-04E Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799131 = {S79255, 146, { &P22594,&P18598,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-M919 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2849110 = {S79256, 64, { &P15364,&P16110,&P24163,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; MRA 8.0 (build 5784)) Presto/*/
const Device UA2866854 = {S79257, 169, { &P20705,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (HTC One, 47, 1.9.7)*/
const Device UA2937855 = {S79258, 167, { &P20705,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTCONE Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (HTCONE, 47, 1.9.7)*/
const Device UA2975988 = {S79259, 126, { &P20556,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A701 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022691 = {S79260, 136, { &P22097,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTabS2110AF Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023923 = {S79261, 136, { &P22097,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTabS2110AH Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2975799 = {S79262, 131, { &P20145,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; NS-13T001 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3065198 = {S79263, 158, { &P18302,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT25i Build/4.1.B.0.631) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3081241 = {S79264, 138, { &P22592,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717R Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065538 = {S79265, 137, { &P18679,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-B5330 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3111847 = {S79266, 138, { &P18679,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-B5330L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080078 = {S79267, 133, { &P24069,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; R815 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3916 = {S79268, 143, { &P17714,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Stream Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA441281 = {S79269, 125, { &P15364,&P5690,&P22426,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.140 Safari/535.19 Comodo_Dragon/18.0.3.0*/
const Device UA611091 = {S79270, 62, { &P15364,&P18145,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64; rv:12.0) Gecko/X Firefox/12.0*/
const Device UA706722 = {S79271, 113, { &P15364,&P16110,&P22454,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7*/
const Device UA779143 = {S79272, 120, { &P15364,&P16110,&P22454,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7*/
const Device UA1991342 = {S79273, 81, { &P12588,&P5055,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-in; NokiaC6-01)/UC Browser8.2.0.132/50/444/UCWEB Mobile*/
const Device UA2372061 = {S79274, 89, { &P12588,&P5055,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiac6-01) UCBrowser8.2.1.144/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA852428 = {S79275, 97, { &P15364,&P7127,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:2.0.1) Gecko/X Firefox/4.0.1,gzip(gfe) (via translate.google.com)*/
const Device UA2148882 = {S79276, 87, { &P15364,&P5567,&P4536,&P18092 }};	/*Useragent - Mozilla/5.0 Opera/9.80 (KDDI-KC46; BREW; Opera Mobi; U; ja) Presto/2.4.18 Version/10.00*/
const Device UA2370944 = {S79277, 87, { &P15364,&P5567,&P4536,&P18092 }};	/*Useragent - Mozilla/5.0 Opera/9.80 (KDDI-CA3K; BREW; Opera Mobi; U; ja) Presto/2.4.18 Version/10.00*/
const Device UA2409834 = {S79278, 87, { &P15364,&P5567,&P4536,&P18092 }};	/*Useragent - Mozilla/5.0 Opera/9.80 (KDDI-TS3Y; BREW; Opera Mobi; U; ja) Presto/2.4.18 Version/10.00*/
const Device UA2517894 = {S79279, 87, { &P15364,&P5567,&P4536,&P18092 }};	/*Useragent - Mozilla/5.0 Opera/9.80 (KDDI-SN3V; BREW; Opera Mobi; U; ja) Presto/2.4.18 Version/10.00*/
const Device UA3041672 = {S79280, 87, { &P15364,&P5567,&P4536,&P18092 }};	/*Useragent - Mozilla/5.0 Opera/9.80 (KDDI-KC4A; BREW; Opera Mobi; U; ja) Presto/2.4.18 Version/10.00*/
const Device UA2162223 = {S79281, 101, { &P15364,&P7127,&P22343,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9.2.32) Gecko/X Firefox/3.6.32 (Palemoon/3.6.32)*/
const Device UA2386227 = {S79282, 83, { &P18510,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; ALCATEL_ONE_TOUCH_918N; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2420567 = {S79283, 83, { &P18510,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; ALCATEL_ONE_TOUCH_918N; 320*480) UCWEB7.9.0.94/139/355*/
const Device UA2578447 = {S79284, 83, { &P18510,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; ALCATEL_ONE_TOUCH_918N; 320*480) UCWEB7.9.0.94/139/352*/
const Device UA2401266 = {S79285, 119, { &P12456,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian73-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2423843 = {S79286, 142, { &P20426,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; W200 Wil Fone Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2424003 = {S79287, 136, { &P23130,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; H2000+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2459470 = {S79288, 129, { &P17686,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; MZ601 Build/JB_MR1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2599850 = {S79289, 128, { &P17686,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; MZ601 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2581756 = {S79290, 91, { &P12061,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAE65) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2753983 = {S79291, 130, { &P20107,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5580C Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755286 = {S79292, 135, { &P23190,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Marquis_MP977 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2773185 = {S79293, 137, { &P20103,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; EK-GC100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2896879 = {S79294, 137, { &P20103,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EK-GC100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3022977 = {S79295, 139, { &P22756,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C1604 Build/11.1.A.0.68) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022561 = {S79296, 140, { &P24095,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5306 Build/12.0.A.1.284) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3061492 = {S79297, 93, { &P18442,&P18111,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7100) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3083561 = {S79298, 138, { &P19848,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M440S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083600 = {S79299, 137, { &P23167,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-H9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA15377 = {S79300, 64, { &P11889,&P5926,&P4030,&P18092 }};	/*Useragent - Nokia8800e-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA9427 = {S79301, 139, { &P13409,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; E110 Build/Cupcake) AppleWebKit/528.5+ (KHTML, like  Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA63903 = {S79302, 40, { &P14554,&P17486,&P17602,&P18092 }};	/*Useragent - MOT-EX128 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA69132 = {S79303, 102, { &P11689,&P5776,&P4779,&P18092 }};	/*Useragent - NokiaN70-1/5.0737.3.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/27/351*/
const Device UA231470 = {S79304, 101, { &P11689,&P5776,&P4779,&P18092 }};	/*Useragent - NokiaN70/ 5.0741.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/27/351*/
const Device UA163558 = {S79305, 119, { &P15364,&P17188,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 4.0; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10*/
const Device UA695415 = {S79306, 41, { &P13163,&P5129,&P24665,&P18092 }};	/*Useragent - NokiaN97/UC Browser8.0.4.121/50/444/UCWEB*/
const Device UA605254 = {S79307, 153, { &P13074,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P930 Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA709641 = {S79308, 153, { &P13074,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P930 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA909261 = {S79309, 153, { &P13074,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P930 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA690583 = {S79310, 115, { &P14540,&P5843,&P17438,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-01/08.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15*/
const Device UA664377 = {S79311, 146, { &P16352,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Huawei-U9000 Build/HuaweiU9000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1904040 = {S79312, 71, { &P15364,&P18145,&P20918,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; FreeBSD i386; en-US; rv:1.7) Gecko/X Firefox/0.9.1*/
const Device UA2172108 = {S79313, 154, { &P13074,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LG-P930 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423594 = {S79314, 158, { &P13074,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P930 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2179637 = {S79315, 140, { &P17996,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; MID1126 Build/GreenDemon_v1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2664181 = {S79316, 72, { &P12280,&P17146,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPhone4,1; iPhone OS 5.0.1; en_US; en) AppleWebKit/420+*/
const Device UA2530980 = {S79317, 141, { &P12280,&P17146,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3*/
const Device UA2535976 = {S79318, 140, { &P12280,&P17146,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA2580795 = {S79319, 166, { &P16356,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-N7000 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2603909 = {S79320, 134, { &P20097,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CT1010 Build/IMM76I.v007) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2607546 = {S79321, 143, { &P21812,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GSmart Rio R1 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2607552 = {S79322, 136, { &P21812,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Rio R1 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2619871 = {S79323, 229, { &P13452,&P5629,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3030; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2622958 = {S79324, 229, { &P13452,&P5629,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2657663 = {S79325, 142, { &P20183,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI-ONE Plus Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099853 = {S79326, 133, { &P20183,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI-ONE Plus Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2658987 = {S79327, 172, { &P23205,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GOCLEVER TAB I71 Build/GRI40) AppleWebKit/533.1 (KHTML like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.0.181/145/352*/
const Device UA2661406 = {S79328, 138, { &P23853,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Phoenix Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2676742 = {S79329, 138, { &P14854,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SHW-M250S Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713235 = {S79330, 138, { &P14854,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713873 = {S79331, 138, { &P14854,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250K Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2736201 = {S79332, 138, { &P14854,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M250L Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2958411 = {S79333, 164, { &P19862,&P18066,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300*/
const Device UA2707045 = {S79334, 133, { &P23513,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SC-1009JB Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2737755 = {S79335, 93, { &P11880,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA5320) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2779421 = {S79336, 95, { &P11880,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5320d-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2769558 = {S79337, 145, { &P18300,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2770713 = {S79338, 111, { &P18095,&P18148,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-914_im_india_269) U2/1.0.0 UCBrowser/3.0.0.285 U2/1.0.0 Mobile*/
const Device UA2784990 = {S79339, 111, { &P18095,&P18148,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-885_im_india_249) U2/1.0.0 UCBrowser/3.0.0.285 U2/1.0.0 Mobile*/
const Device UA2771018 = {S79340, 163, { &P23905,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.4; GT-I9100P Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2937924 = {S79341, 163, { &P23905,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; GT-I9100P Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2999450 = {S79342, 163, { &P23905,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-I9100P Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2866235 = {S79343, 130, { &P17103,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; TM1004 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2945151 = {S79344, 156, { &P18645,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SPH-L710 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2945355 = {S79345, 164, { &P18645,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2966355 = {S79346, 156, { &P18645,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9305 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2945365 = {S79347, 95, { &P14202,&P5477,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E15i) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA2973479 = {S79348, 137, { &P21384,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F240L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2958938 = {S79349, 144, { &P24112,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; th-th; i-mobile IQ X Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022490 = {S79350, 152, { &P22961,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3028296 = {S79351, 164, { &P13042,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC Incredible S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3063895 = {S79352, 155, { &P21794,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9152 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3054788 = {S79353, 138, { &P24382,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; tr-tr; LG-E986 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3094349 = {S79354, 161, { &P19862,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax A110 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3096249 = {S79355, 156, { &P21628,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8552 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA11611 = {S79356, 141, { &P21270,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; ME722 Build/MLS2GC_0.24.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11620 = {S79357, 141, { &P21270,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; ME722 Build/MLS2GC_1.10.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40430 = {S79358, 141, { &P21270,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME722 Build/MLS2GC_2.10.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55619 = {S79359, 140, { &P21270,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME722 Build/MLS2GC_2.5.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55645 = {S79360, 140, { &P21270,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME722 Build/MLS2GC_2.6.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020723 = {S79361, 129, { &P21270,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; ME722 Build/MLS2GC_0.24.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022312 = {S79362, 128, { &P21270,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME722 Build/MLS2GC_2.6.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2025950 = {S79363, 133, { &P21270,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME722 Build/MLS2GC_2.6.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA53937 = {S79364, 127, { &P14071,&P6757,&P3525,&P18092 }};	/*Useragent - Dell_Mini 3iX/1.0 (Linux; Android 1.5) Browser/WAP 2.0 AppleWebKit/528.5+ Version/3.1.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA128808 = {S79365, 142, { &P21409,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; md-us; p7901a Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA647198 = {S79366, 47, { &P13324,&P5055,&P17347,&P18092 }};	/*Useragent - nokian8-00/UC Browser8.0.3.107/69/444  3gpp-gba*/
const Device UA737456 = {S79367, 97, { &P15364,&P7133,&P22332,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.1.3) Gecko/X Firefox/3.5.3 CometBird/3.5.3*/
const Device UA2000888 = {S79368, 176, { &P19891,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I717 Build/JRO03H; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2007184 = {S79369, 176, { &P19891,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I717 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2199439 = {S79370, 176, { &P19891,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I717 Build/JRO03R; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2345289 = {S79371, 154, { &P14557,&P5843,&P24668,&P18092 }};	/*Useragent - Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia6303classic) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2316811 = {S79372, 129, { &P20127,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT270 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2375202 = {S79373, 126, { &P20894,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; M3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3086414 = {S79374, 138, { &P20894,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MOVI M3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2399019 = {S79375, 130, { &P21352,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB469 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2469227 = {S79376, 169, { &P12616,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; WT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2416992 = {S79377, 135, { &P21843,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; EasyPad 971 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2424515 = {S79378, 160, { &P15364,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Transformer Pad Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2506818 = {S79379, 159, { &P15364,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; olympus Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2763117 = {S79380, 159, { &P15364,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; sunfire Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2552404 = {S79381, 143, { &P21400,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo S880i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2552442 = {S79382, 134, { &P21400,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo S880i Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2693717 = {S79383, 65, { &P21400,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo S880i Build/IMM76D)*/
const Device UA2556186 = {S79384, 157, { &P21599,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Mobiistar Touch S06 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2567705 = {S79385, 134, { &P14997,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S100 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2676428 = {S79386, 143, { &P20816,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WIKO-CINK SLIM Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2704728 = {S79387, 88, { &P20529,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A89) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2727481 = {S79388, 129, { &P18283,&P17979,&P18112,&P18092 }};	/*Useragent - sprd-GT_I9300/1.0 Linux/2.6.35.7 Android/4.1.1 Release/09.28.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2747074 = {S79389, 128, { &P18283,&P17979,&P18112,&P18092 }};	/*Useragent - MTK-GT_I9300/1.0 Linux/2.6.35.7 Android/4.1.9 Release/11.01.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2778219 = {S79390, 128, { &P18283,&P17979,&P18112,&P18092 }};	/*Useragent - MTK-GT_I9300/1.0 Linux/2.6.35.7 Android/4.1.1 Release/10.24.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2790333 = {S79391, 128, { &P18283,&P17979,&P18112,&P18092 }};	/*Useragent - MTK-GT_I9300/1.0 Linux/2.6.35.7 Android/4.1.1 Release/10.13.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2950969 = {S79392, 128, { &P18283,&P17979,&P18112,&P18092 }};	/*Useragent - MTK-GT_I9300/1.0 Linux/2.6.35.7 Android/4.1.1 Release/10.09.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2984334 = {S79393, 128, { &P18283,&P17979,&P18112,&P18092 }};	/*Useragent - MTK-GT_I9300/1.0 Linux/2.6.35.7 Android/4.1.1 Release/10.22.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2738469 = {S79394, 92, { &P15791,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia603) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2774144 = {S79395, 157, { &P16362,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2797441 = {S79396, 161, { &P16362,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710VMUB Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2892917 = {S79397, 160, { &P16362,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710BST Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2873250 = {S79398, 187, { &P13157,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3021168 = {S79399, 137, { &P21373,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F240K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022882 = {S79400, 130, { &P18702,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N8013 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021587 = {S79401, 93, { &P14186,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3031249 = {S79402, 160, { &P22867,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5303 Build/12.0.A.1.257) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3080455 = {S79403, 135, { &P22860,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Q700 Build/XOLOQ700) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3131716 = {S79404, 137, { &P15575,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3094358 = {S79405, 156, { &P22382,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8262 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA16376 = {S79406, 61, { &P11876,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia7390/2.0 (04.51) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA26227 = {S79407, 98, { &P12488,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-L170/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA11572 = {S79408, 111, { &P13623,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Snap_S523/V1.22.841.5 (53520) Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA26706 = {S79409, 91, { &P13623,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Snap_S523 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA26707 = {S79410, 111, { &P13623,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Snap_S523/V2.03.841.1 (65854) Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA24143 = {S79411, 53, { &P11736,&P17486,&P3402,&P18092 }};	/*Useragent - MOT-W230/0.0.31 UP.Browser/6.3.0.6.c.19 (GUI) MMP/2.0*/
const Device UA54026 = {S79412, 67, { &P15364,&P17017,&P3499,&P18092 }};	/*Useragent - modu T/1.0 Browser/Obigo/Q7 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54787 = {S79413, 45, { &P15364,&P17017,&P3499,&P18092 }};	/*Useragent - Mozilla/4.1 (compatible; Teleca/Q07/GS-CX700)*/
const Device UA90179 = {S79414, 98, { &P15364,&P17017,&P3499,&P18092 }};	/*Useragent - Alex-uo/1.0 Qualcom/6290 Release/3.20.2010 Browser/Obigo Q7 Profile/MIDP-2.0 Configuration/CLDC1.1*/
const Device UA250314 = {S79415, 114, { &P15364,&P18110,&P22452,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1*/
const Device UA250474 = {S79416, 116, { &P15364,&P18110,&P22452,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1*/
const Device UA532652 = {S79417, 112, { &P15364,&P7133,&P22512,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.4) Gecko/X Firefox/3.6.4 Wyzo/3.6.4.1 SearchToolbar/1.2*/
const Device UA2022671 = {S79418, 141, { &P18082,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; KM-S120 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995536 = {S79419, 136, { &P17308,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROIDX Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2033367 = {S79420, 137, { &P18744,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOR803 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033995 = {S79421, 121, { &P18744,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; R803 Build/GRK39F) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2476025 = {S79422, 126, { &P18744,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; R803 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2232593 = {S79423, 137, { &P12788,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PG86100 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2403265 = {S79424, 137, { &P12788,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PG86100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2427923 = {S79425, 141, { &P12788,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Evo 3D CDMA Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2527439 = {S79426, 140, { &P12788,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Evo 3D GSM Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2257399 = {S79427, 130, { &P21026,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ZTE U930 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2789052 = {S79428, 61, { &P21026,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; ZTE U930 Build/IML74K)*/
const Device UA3100039 = {S79429, 136, { &P21026,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U930A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3120061 = {S79430, 132, { &P21026,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ZTE U930HD Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2356506 = {S79431, 91, { &P11945,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5233s) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2390406 = {S79432, 91, { &P11945,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5233s) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2456596 = {S79433, 88, { &P11945,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; vi; gt-s5233s) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2457783 = {S79434, 88, { &P11945,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; vi; gt-s5233s) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2504792 = {S79435, 84, { &P11945,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5233s) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0*/
const Device UA2431557 = {S79436, 80, { &P18088,&P5567,&P4732,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/4.2.99/28.3392; U; id) Presto/2.8.119 Version/11.10*/
const Device UA2456003 = {S79437, 129, { &P23178,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; iDxD8 3G Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2665867 = {S79438, 129, { &P23178,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; iDxD8 3G Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2456385 = {S79439, 109, { &P15364,&P16110,&P18106,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/98 Safari/537.4 (StatusCake)*/
const Device UA2581366 = {S79440, 93, { &P12236,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA6700s) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2535733 = {S79441, 93, { &P13974,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5670) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2657578 = {S79442, 137, { &P22674,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-be; Lenco TAB-813 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2665467 = {S79443, 134, { &P23185,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GoTab Gti8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2689627 = {S79444, 136, { &P20946,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; P-02D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2795081 = {S79445, 135, { &P24493,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; BS471 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2873715 = {S79446, 168, { &P23695,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1;engb;Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 4.1.1; de-de; ST80208-2 Build/JRO03HSafari/534.30;SurfTab_8.0*/
const Device UA3012002 = {S79447, 168, { &P23695,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1;engb;Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 4.1.1; de-de; ST80216-2 Build/JRO03HSafari/534.30;SurfTab_8.0*/
const Device UA2937413 = {S79448, 152, { &P19771,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035A Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2994204 = {S79449, 152, { &P19771,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035X Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3048548 = {S79450, 152, { &P19771,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2937463 = {S79451, 132, { &P21948,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID07-D2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3037217 = {S79452, 138, { &P21948,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MID0714 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078587 = {S79453, 131, { &P21948,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MID0738 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3021567 = {S79454, 138, { &P19902,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530M Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974237 = {S79455, 134, { &P20414,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SmartTabII10 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3039313 = {S79456, 159, { &P18449,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST27i Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3021255 = {S79457, 136, { &P19917,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P765 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022264 = {S79458, 129, { &P24367,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5200 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3082763 = {S79459, 138, { &P20517,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CINK PEAX Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081675 = {S79460, 145, { &P18300,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3132714 = {S79461, 126, { &P15869,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xoom Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3061094 = {S79462, 94, { &P13324,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaN8-00) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3081398 = {S79463, 134, { &P20480,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; JY-G3 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3132870 = {S79464, 138, { &P18564,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One X+ Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA6159 = {S79465, 28, { &P13451,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-B3410R/B3410UXJC4*/
const Device UA50369 = {S79466, 74, { &P13451,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-B3410/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA7366 = {S79467, 84, { &P17058,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-SGH-I907/UCHI5 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA46929 = {S79468, 86, { &P12550,&P5055,&P3804,&P18092 }};	/*Useragent - SonyEricssonP910a/R3A SEMC-Browser/Symbian/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA60591 = {S79469, 86, { &P12550,&P5055,&P3804,&P18092 }};	/*Useragent - SonyEricssonP910c/R2A SEMC-Browser/Symbian/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA64474 = {S79470, 86, { &P12550,&P5055,&P3804,&P18092 }};	/*Useragent - SonyEricssonP910i/R5B SEMC-Browser/Symbian/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA50356 = {S79471, 124, { &P11732,&P17484,&P4732,&P18092 }};	/*Useragent - SAMSUNG-SGH-I637/UCIJ2; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.14812; U; en-US) Opera 9.50*/
const Device UA1943965 = {S79472, 100, { &P14711,&P5204,&P17338,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (11.20) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-01/UC Browser7.9.0.102/69/352*/
const Device UA281198 = {S79473, 147, { &P16140,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Spice_Mi300 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA298509 = {S79474, 151, { &P16140,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; CSL_Spice_MI300 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA281666 = {S79475, 113, { &P15364,&P17285,&P22440,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/530.9 (KHTML, like Gecko) Iron/2.0.178.0 Safari/530.9*/
const Device UA304003 = {S79476, 113, { &P15364,&P17285,&P22440,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/530.1 (KHTML, like Gecko) Iron/2.0.168.0 Safari/530.1*/
const Device UA368215 = {S79477, 123, { &P15364,&P5690,&P22420,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.1.0.0 Chrome/17.0.963.38 Safari/535.11*/
const Device UA668639 = {S79478, 85, { &P15364,&P18140,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; Edition Linux Mint; en) Presto/2.10.229 Version/11.62*/
const Device UA2517121 = {S79479, 83, { &P15364,&P5690,&P20907,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:0.9.4.1) Gecko/X Netscape6/6.2.2*/
const Device UA903524 = {S79480, 100, { &P15364,&P18346,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5*/
const Device UA2349679 = {S79481, 176, { &P14225,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2383386 = {S79482, 166, { &P14225,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2384449 = {S79483, 154, { &P18673,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2424345 = {S79484, 69, { &P18214,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GT-S7500; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2443216 = {S79485, 131, { &P18473,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC X720d Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2594753 = {S79486, 141, { &P18473,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-X720d/1.34.1401.1) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2700915 = {S79487, 141, { &P18473,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-X720d/2.19.1401.1) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2758849 = {S79488, 141, { &P18473,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-X720d/1.27.1401.1) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2808927 = {S79489, 141, { &P18473,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC-X720d/1.28.1401.1) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2464717 = {S79490, 180, { &P12108,&P5843,&P24671,&P18092 }};	/*Useragent - Nokia7230/5.0 (10.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia7230) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2456226 = {S79491, 131, { &P20292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T528t Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2521883 = {S79492, 140, { &P20292,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T528t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985245 = {S79493, 62, { &P20292,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC T528t Build/IMM76D)*/
const Device UA2480633 = {S79494, 156, { &P18386,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I757M Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2482398 = {S79495, 133, { &P23452,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAC-80011 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2487074 = {S79496, 133, { &P21889,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; TAC-90011 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2526721 = {S79497, 139, { &P15860,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-SU880 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2535290 = {S79498, 142, { &P21160,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MEDION P4013 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2564824 = {S79499, 97, { &P18649,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A57) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2662940 = {S79500, 135, { &P23370,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Oysters T34 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2677282 = {S79501, 138, { &P22591,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2695108 = {S79502, 140, { &P23559,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; AMOI N821 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2729907 = {S79503, 157, { &P23559,&P18066,&P18062,&P18092 }};	/*Useragent - AMOI N821 Linux/3.0.13 Android/4.0.4 Release/12.19.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2732449 = {S79504, 141, { &P23436,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; CAPHG11-01 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2855905 = {S79505, 141, { &P23436,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; CAPHG11-01 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2711476 = {S79506, 161, { &P18442,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7105 Build/JDQ39; Carbon-CARBON-JB-v1.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2866477 = {S79507, 162, { &P18442,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L900 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937167 = {S79508, 162, { &P18442,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T889 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996546 = {S79509, 162, { &P18442,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7105 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2750760 = {S79510, 111, { &P15364,&P7133,&P22527,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.6 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/537.6 Midori/0.5*/
const Device UA2896330 = {S79511, 141, { &P24155,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; fnac 10 v2 Build/1.0.0_X-15:17) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3039512 = {S79512, 141, { &P24155,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; fnac 10 v2 Build/1.0.1_X-14:35) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3020417 = {S79513, 130, { &P12879,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7510 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3020297 = {S79514, 145, { &P20558,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ608 Build/7.7.1-141-7-FLEM-UMTS-LA) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021348 = {S79515, 143, { &P20558,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ608-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2975999 = {S79516, 138, { &P22592,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717R Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3019387 = {S79517, 131, { &P15767,&P18432,&P20784,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/10A523 Safari/7534.48.3*/
const Device UA3078725 = {S79518, 131, { &P15767,&P18432,&P20784,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/10A550 Safari/7534.48.3*/
const Device UA3078955 = {S79519, 129, { &P15767,&P18432,&P20784,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/10A403 Safari/7534.48.3*/
const Device UA3080170 = {S79520, 139, { &P20439,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SOL21 Build/9.1.D.0.401) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA22252 = {S79521, 148, { &P14143,&P17017,&P3807,&P18092 }};	/*Useragent - Huawei/1.0/0HuaweiG6620/SP01 Browser/Obigo-Browser/Q03C MMS/Obigo-MMS/1.2, Huawei/1.0/0HuaweiG6620/SP01 Browser/Obigo-Browser/Q03C MMS/Obigo-MMS/1.2*/
const Device UA22253 = {S79522, 73, { &P14143,&P17017,&P3807,&P18092 }};	/*Useragent - Huawei/1.0/0HuaweiG6620/SP01 Browser/Obigo-Browser/Q03C MMS/Obigo-MMS/1.2*/
const Device UA43224 = {S79523, 68, { &P14143,&P17017,&P3807,&P18092 }};	/*Useragent - Huawei-G6620/1.0/0/SP01 Browser/Obigo-Browser/Q03C MMS/Obigo-MMS/1.2*/
const Device UA68828 = {S79524, 34, { &P14358,&P5890,&P4777,&P18092 }};	/*Useragent - NokiaE63/UC Browser7.7.1.88/28/444*/
const Device UA450620 = {S79525, 36, { &P14358,&P5890,&P4777,&P18092 }};	/*Useragent - nokiae63-1/UC Browser7.7.1.88/69/444*/
const Device UA354515 = {S79526, 99, { &P15364,&P7127,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7*/
const Device UA2221968 = {S79527, 99, { &P15364,&P7127,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2*/
const Device UA2394535 = {S79528, 106, { &P15364,&P7127,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2*/
const Device UA1888688 = {S79529, 25, { &P15364,&P17350,&P18333,&P18092 }};	/*Useragent - iTunes/9.0.2 (Windows; N)*/
const Device UA628409 = {S79530, 142, { &P18267,&P5043,&P24667,&P18092 }};	/*Useragent - NokiaX2-02/2.0 (10.91) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiax2-02) UCBrowser8.2.1.144/70/355/UCWEB Mobile*/
const Device UA852966 = {S79531, 79, { &P13554,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN79)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA2151750 = {S79532, 30, { &P13554,&P5890,&P24667,&P18092 }};	/*Useragent - NOKIAN79/UCWEB8.2.0.116/28/800*/
const Device UA2026595 = {S79533, 171, { &P14186,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352*/
const Device UA2031435 = {S79534, 171, { &P14186,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2376373 = {S79535, 172, { &P14186,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5830L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2809281 = {S79536, 171, { &P14186,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352*/
const Device UA2816527 = {S79537, 171, { &P14186,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2216567 = {S79538, 122, { &P16518,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2222/E2222JVKJ3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA2229194 = {S79539, 146, { &P22163,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Explay Infinity Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2381760 = {S79540, 132, { &P23270,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SC-72MID Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2502753 = {S79541, 132, { &P23270,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SC-72MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2857962 = {S79542, 139, { &P23270,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SC-72MID Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423567 = {S79543, 131, { &P23852,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; ADT 2.0 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2484761 = {S79544, 122, { &P13979,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; Nokia6220c-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2469937 = {S79545, 133, { &P18466,&P18110,&P19817,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.6 Safari/535.19 Silk-Accelerated=false*/
const Device UA2563379 = {S79546, 174, { &P15501,&P5477,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E16i Build/2.1.1.A.0.53) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3021158 = {S79547, 178, { &P15501,&P5477,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E16i Build/2.1.1.A.0.53) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2602982 = {S79548, 131, { &P23310,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; ADT 2.6 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2676579 = {S79549, 129, { &P15919,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P3100 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2658758 = {S79550, 56, { &P15364,&P22310,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Win3.11; Trident/1.0)*/
const Device UA2658762 = {S79551, 63, { &P15364,&P16110,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 2.0; Windows NT 6.2; Trident/4.0)*/
const Device UA2791642 = {S79552, 63, { &P15364,&P16110,&P18068,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 1.0; Windows NT 6.2; Trident/4.0)*/
const Device UA2665961 = {S79553, 153, { &P23723,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; O+ 8.35 Android Build/OplusOplus_A200) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2849401 = {S79554, 153, { &P23723,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; O+_8.35_Android Build/OplusOplus_A200) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2666866 = {S79555, 167, { &P15438,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Sensation Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2958808 = {S79556, 163, { &P15438,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Sensation Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2676058 = {S79557, 138, { &P19854,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R760X Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2704234 = {S79558, 171, { &P18509,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-MS770 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (LG-MS770, 42, 1.9.1)*/
const Device UA2708044 = {S79559, 136, { &P20587,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P715 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713692 = {S79560, 140, { &P22705,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Fly IQ451 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2737925 = {S79561, 160, { &P12473,&P5204,&P24677,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2730c-1) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2753051 = {S79562, 126, { &P19991,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A511 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2754005 = {S79563, 162, { &P14433,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-I9003 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2999188 = {S79564, 162, { &P14433,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; GT-I9003 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2773966 = {S79565, 138, { &P22738,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT916 Build/2_26D_2014) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773664 = {S79566, 138, { &P18072,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2797582 = {S79567, 138, { &P18072,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877298 = {S79568, 138, { &P18072,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782004 = {S79569, 146, { &P23125,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Orbit 5700T Build/ICS_IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3017982 = {S79570, 94, { &P13648,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaE7-00) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3064793 = {S79571, 151, { &P20231,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A21 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3058580 = {S79572, 165, { &P14877,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9000 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/galaxysmtd*/
const Device UA3080920 = {S79573, 165, { &P14877,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-I9000 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/galaxysmtd*/
const Device UA3094151 = {S79574, 136, { &P18281,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P940 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094674 = {S79575, 138, { &P18281,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-KU5400 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3112539 = {S79576, 137, { &P18281,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-SU540 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA24092 = {S79577, 88, { &P15666,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-G868/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA616287 = {S79578, 88, { &P15666,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-G868T/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP-2.0Configuration/CLDC-1.0*/
const Device UA42207 = {S79579, 121, { &P12444,&P5477,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.1-update1;Zh_cn;U20i;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA1905295 = {S79580, 72, { &P12444,&P5477,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.1-update1;Zh_cn;U20i;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA101805 = {S79581, 113, { &P15364,&P5690,&P22442,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/528.7 (KHTML, like Gecko) Iron/1.0.155.0 Safari/528.7*/
const Device UA287008 = {S79582, 148, { &P16135,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT701 Build/STCU_41.07.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018075 = {S79583, 148, { &P16135,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT701 Build/STCU_31.05.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018735 = {S79584, 148, { &P16135,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XT701 Build/STCU_31.05.4) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2022401 = {S79585, 134, { &P16135,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT701 Build/STCU_41.07.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA395969 = {S79586, 132, { &P22518,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; AN7 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA448363 = {S79587, 48, { &P15364,&P18110,&P24163,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; de) Presto/2.1.1*/
const Device UA584014 = {S79588, 114, { &P13035,&P5043,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-02/06.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA728198 = {S79589, 132, { &P22252,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; Novo7Basic Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA701248 = {S79590, 89, { &P17595,&P18120,&P22490,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.8.0.1) Gecko/X Flock/0.5.13.2*/
const Device UA804963 = {S79591, 80, { &P15798,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.5;Zh_cn;HTC Explorer A310e;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA895660 = {S79592, 81, { &P11905,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-in; NOKIA6120c)/UC Browser8.4.0.159/28/352/UCWEB Mobile*/
const Device UA2223048 = {S79593, 79, { &P11905,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokia6120c) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2177733 = {S79594, 154, { &P18465,&P18066,&P18617,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.1 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2261581 = {S79595, 138, { &P20235,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cynus T1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2392367 = {S79596, 95, { &P15364,&P16110,&P22481,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.0.16) Gecko/X Firefox/3.0.16 Flock/2.5.6*/
const Device UA2421358 = {S79597, 132, { &P20150,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MFC155FR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2581952 = {S79598, 132, { &P24139,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; CTP828BT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2658677 = {S79599, 147, { &P23794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HUAWEI Y310-5000 Build/HuaweiY310-5000) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2764567 = {S79600, 147, { &P23794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.5; zh-cn; HUAWEI Y310-5000 Build HuaweiY310-5000) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2852413 = {S79601, 154, { &P23794,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HUAWEI Y310-5000 Build/HuaweiY310-5000) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2671463 = {S79602, 150, { &P20094,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825-1 Build/HuaweiU8825-1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707391 = {S79603, 148, { &P20094,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675831 = {S79604, 53, { &P15364,&P6729,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows 95) Presto/2.12.388 Version/12.15*/
const Device UA2701544 = {S79605, 136, { &P23304,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R813T Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2855263 = {S79606, 58, { &P23304,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; R813T Build/IMM76D)*/
const Device UA2769198 = {S79607, 145, { &P22933,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; Avea inTouch 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892336 = {S79608, 135, { &P18737,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; X909T Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092343 = {S79609, 134, { &P18737,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; X909 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2993879 = {S79610, 128, { &P24597,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; AT10-A Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3061489 = {S79611, 142, { &P16869,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Micromax A76 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3081264 = {S79612, 142, { &P16869,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Micromax A74 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3061661 = {S79613, 94, { &P18364,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC5-05) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3093865 = {S79614, 156, { &P22872,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5282 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3115335 = {S79615, 159, { &P12968,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26i Build/6.2.B.0.200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA33289 = {S79616, 58, { &P13644,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player sprint / 1.0 / htc_supersonic / 2.3.3*/
const Device UA48913 = {S79617, 56, { &P13644,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player sprint / 1.0 / htc_supersonic / 2.2*/
const Device UA623957 = {S79618, 58, { &P13644,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player sprint / 1.0 / htc_supersonic / 2.3.5*/
const Device UA197621 = {S79619, 54, { &P17509,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-CA3F UP.Browser/6.2_7.2.7.1.K.4.310 (GUI) MMP/2.0*/
const Device UA718922 = {S79620, 51, { &P15364,&P18110,&P24473,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/4.1; Linux; X11)*/
const Device UA2517490 = {S79621, 92, { &P15364,&P5690,&P22511,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Wyzo/3.5.6*/
const Device UA530436 = {S79622, 86, { &P17595,&P17294,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:2.0) Gecko/X Firefox/4.0 TenFourFox/7450*/
const Device UA546923 = {S79623, 52, { &P15364,&P17017,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; rv:2.0) Gecko/X Firefox/4.0*/
const Device UA578650 = {S79624, 121, { &P12139,&P5204,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia5310XpressMusic/10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA609510 = {S79625, 95, { &P17595,&P17296,&P18394,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; Edition MAS; en) Presto/2.10.229 Version/11.63*/
const Device UA637189 = {S79626, 133, { &P21591,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; 101DL Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032405 = {S79627, 57, { &P21591,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; 101DL Build/GRJ90)*/
const Device UA666787 = {S79628, 59, { &P15364,&P18145,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD i386; rv:8.0) Gecko/X Firefox/8.0*/
const Device UA653076 = {S79629, 43, { &P15655,&P17484,&P17347,&P18092 }};	/*Useragent - samsung-gt-s3850/UC Browser8.0.3.107/69/444*/
const Device UA908779 = {S79630, 138, { &P20853,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Scroll Extreme Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2024983 = {S79631, 139, { &P20853,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  Scroll Extreme Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA1996334 = {S79632, 43, { &P17595,&P18207,&P21584,&P18092 }};	/*Useragent - MacOutlook/14.2.3.X (Intel Mac OS X 10.8.0)*/
const Device UA2182911 = {S79633, 43, { &P17595,&P18207,&P21584,&P18092 }};	/*Useragent - MacOutlook/14.2.3.X (Intel Mac OS X 10.8.1)*/
const Device UA2232713 = {S79634, 43, { &P17595,&P18207,&P21584,&P18092 }};	/*Useragent - MacOutlook/14.2.4.X (Intel Mac OS X 10.8.2)*/
const Device UA2242457 = {S79635, 43, { &P17595,&P18207,&P21584,&P18092 }};	/*Useragent - MacOutlook/14.2.3.X (Intel Mac OS X 10.8.2)*/
const Device UA2389169 = {S79636, 43, { &P17595,&P18207,&P21584,&P18092 }};	/*Useragent - MacOutlook/14.2.5.X (Intel Mac OS X 10.8.2)*/
const Device UA2027021 = {S79637, 128, { &P21394,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Lenovo A668t Build/GWK74) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2031062 = {S79638, 104, { &P21394,&P5629,&P15760,&P18092 }};	/*Useragent - LenovoA668t_TD/1.0 Android/2.3 (Linux; Android 2.3) Release/08.20.2011 Browser/WAP 2.0 AppleWebKit/533.1*/
const Device UA2337332 = {S79639, 133, { &P21394,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Lenovo A668t Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2743557 = {S79640, 140, { &P21394,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Lenovo A668t Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2190380 = {S79641, 132, { &P21131,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; S15 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2351878 = {S79642, 138, { &P23907,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; PROG160 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2360881 = {S79643, 132, { &P20149,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P6211 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616508 = {S79644, 61, { &P20149,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P6211 Build/IMM76D)*/
const Device UA2384831 = {S79645, 132, { &P21334,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A8 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2639575 = {S79646, 87, { &P15364,&P7127,&P21955,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9b2pre) Gecko/X Minefield/3.0b2pre*/
const Device UA2469308 = {S79647, 163, { &P13337,&P17485,&P24675,&P18092 }};	/*Useragent - SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonj108i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2536684 = {S79648, 97, { &P18720,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Micromax_A50) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2597883 = {S79649, 104, { &P14225,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Desire_HD_A9191) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2731451 = {S79650, 94, { &P14225,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; Desire_HD) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2634111 = {S79651, 130, { &P15985,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P6200 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.28 Safari/537.31*/
const Device UA2673615 = {S79652, 130, { &P15985,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2667403 = {S79653, 167, { &P19861,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I605 Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3021642 = {S79654, 172, { &P19861,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I605 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/i605*/
const Device UA2676922 = {S79655, 136, { &P20227,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-L40G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2704214 = {S79656, 133, { &P23314,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fa-ir; G3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2935170 = {S79657, 139, { &P23314,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Jiayu G3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879599 = {S79658, 157, { &P18582,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT28h Build/6.1.E.2.68) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2881401 = {S79659, 156, { &P18582,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT28h Build/6.1.E.3.7) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2756298 = {S79660, 155, { &P14207,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.4; ADR6350 Build/GRJ22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453*/
const Device UA2877899 = {S79661, 137, { &P24733,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; TAB950 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892374 = {S79662, 153, { &P24072,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033A Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2936994 = {S79663, 137, { &P22610,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3056617 = {S79664, 138, { &P22610,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810B Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3077121 = {S79665, 130, { &P22610,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2974228 = {S79666, 92, { &P12029,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN82) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA2997989 = {S79667, 137, { &P22916,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E450f Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3049139 = {S79668, 137, { &P22916,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E450g Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3088374 = {S79669, 136, { &P22916,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E450 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3092355 = {S79670, 137, { &P22916,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E450B Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3019683 = {S79671, 98, { &P12142,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; SonyEricssonU5) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3081097 = {S79672, 136, { &P22660,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P778 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3059266 = {S79673, 127, { &P24458,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; KUNO4 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3059545 = {S79674, 93, { &P22872,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S5282) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3083203 = {S79675, 135, { &P15575,&P21637,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-I9100 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.30 Mobile Safari/537.36*/
const Device UA3083181 = {S79676, 139, { &P18592,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One VX Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081035 = {S79677, 138, { &P22590,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I717M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3092211 = {S79678, 72, { &P15364,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile: SPH-L720; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA3092193 = {S79679, 72, { &P15364,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet: GT-P3113; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA3092234 = {S79680, 72, { &P15364,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet: SCH-I915; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA3092249 = {S79681, 72, { &P15364,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile: SCH-I605; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA3092254 = {S79682, 71, { &P15364,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile: Nexus 4; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA3107813 = {S79683, 71, { &P15364,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile: LG-P769; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA64404 = {S79684, 85, { &P13217,&P17603,&P3534,&P18092 }};	/*Useragent - PANTECH-P7000/JDUS10052009 Browser/Obigo/Q05A Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA155263 = {S79685, 139, { &P15364,&P5690,&P22399,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Comodo_Dragon/6.0.0.10 Chrome/6.0.472.63 Safari/534.3*/
const Device UA671422 = {S79686, 100, { &P12897,&P5843,&P17338,&P18092 }};	/*Useragent - NokiaC2-00/2.0 (03.82) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-00/UC Browser7.9.0.102/69/352*/
const Device UA517825 = {S79687, 12, { &P15364,&P17017,&P3581,&P18092 }};	/*Useragent - Symphony M70*/
const Device UA552424 = {S79688, 12, { &P15364,&P17017,&P3581,&P18092 }};	/*Useragent - Symphony D48*/
const Device UA817939 = {S79689, 12, { &P15364,&P17017,&P3581,&P18092 }};	/*Useragent - Symphony D30*/
const Device UA846008 = {S79690, 81, { &P14225,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.5;Zh_cn;HTC Desire HD A9191;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA2024374 = {S79691, 71, { &P14225,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.3;Zh_cn;Desire HD;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA566170 = {S79692, 39, { &P17595,&P18120,&P22890,&P18092 }};	/*Useragent - iCab/5.0 (Macintosh; U; Intel Mac OS X)*/
const Device UA2541861 = {S79693, 37, { &P17595,&P18120,&P22890,&P18092 }};	/*Useragent - iCab/5.0 (Macintosh; U; PPC Mac OS X)*/
const Device UA648891 = {S79694, 117, { &P14862,&P5204,&P17436,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2700c-2/07.80; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.3.23.12*/
const Device UA2217912 = {S79695, 60, { &P15364,&P6729,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows 95; U; ru) Presto/2.10.289 Version/12.00*/
const Device UA2551577 = {S79696, 60, { &P15364,&P6729,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows 95; U; ru) Presto/2.10.289 Version/12.01*/
const Device UA2028762 = {S79697, 150, { &P21624,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; i-mobile i-style Q1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029026 = {S79698, 151, { &P21624,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; th-th; i-mobile i-style Q1i Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2400862 = {S79699, 189, { &P13308,&P17485,&P24672,&P18092 }};	/*Useragent - SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; sonyericssonj105i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile*/
const Device UA2424087 = {S79700, 90, { &P14545,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-x330t) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2437330 = {S79701, 89, { &P14545,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-x330) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2747482 = {S79702, 90, { &P14545,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-x330t) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2469280 = {S79703, 163, { &P13452,&P5096,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2622344 = {S79704, 166, { &P13452,&P5096,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355*/
const Device UA2622600 = {S79705, 164, { &P13452,&P5096,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-P1000L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355*/
const Device UA2624282 = {S79706, 164, { &P13452,&P5096,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2529354 = {S79707, 136, { &P12605,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; XT875 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2663818 = {S79708, 135, { &P12605,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT875 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2566758 = {S79709, 139, { &P20174,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CINK KING Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2672045 = {S79710, 138, { &P20264,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; bq Curie Build/1.1.0 X-14:50) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2567502 = {S79711, 134, { &P20156,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; S500 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2581016 = {S79712, 166, { &P14225,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000*/
const Device UA2734566 = {S79713, 156, { &P14225,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Desire HD Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2878437 = {S79714, 160, { &P14225,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire HD Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000*/
const Device UA2603259 = {S79715, 156, { &P20081,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7108 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3047236 = {S79716, 156, { &P20081,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7108 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2604687 = {S79717, 134, { &P18464,&P18110,&P21707,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Safari/535.19 Silk-Accelerated=false*/
const Device UA2660265 = {S79718, 88, { &P20231,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A21) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2851157 = {S79719, 96, { &P20231,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Karbonn_A21) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2667017 = {S79720, 171, { &P21020,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; Micromax A116 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3021651 = {S79721, 175, { &P21020,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Micromax_A116 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2677243 = {S79722, 139, { &P14993,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT15i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712742 = {S79723, 153, { &P18706,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_ov5647 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714163 = {S79724, 152, { &P18706,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1809c_v75_gq1008_9p017 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735150 = {S79725, 147, { &P18706,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_253 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2862873 = {S79726, 152, { &P18706,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; e1809c_v75_jbl_253_6628 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2719634 = {S79727, 133, { &P22655,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C5503 Build/10.3.A.0.317) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3022275 = {S79728, 142, { &P22655,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C5503 Build/10.3.1.A.0.244) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735024 = {S79729, 138, { &P22942,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-D700 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2737922 = {S79730, 94, { &P19976,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaE6-00) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2752636 = {S79731, 158, { &P13123,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2753036 = {S79732, 148, { &P17835,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook Tablet Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2943214 = {S79733, 159, { &P23477,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT926 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/xt926*/
const Device UA2781608 = {S79734, 139, { &P22975,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; ALLVIEW A4ALL Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2782968 = {S79735, 158, { &P18295,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One X Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799019 = {S79736, 158, { &P18295,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One X Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2867013 = {S79737, 186, { &P22582,&P18598,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SAMSUNG-SGH-I337, 47, 1.9.7)*/
const Device UA2968094 = {S79738, 134, { &P21636,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C525c Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2975556 = {S79739, 137, { &P22390,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5310 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3029917 = {S79740, 93, { &P14186,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3032601 = {S79741, 140, { &P11751,&P17483,&P24679,&P18092 }};	/*Useragent - LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-kp500 teleca) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3033081 = {S79742, 151, { &P19853,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7102 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3033277 = {S79743, 151, { &P19853,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7102 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3130737 = {S79744, 137, { &P16362,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131558 = {S79745, 141, { &P16362,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710VMUB Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132035 = {S79746, 140, { &P16362,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710BST Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA18859 = {S79747, 61, { &P14895,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia6263/2.0 (05.20) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA7657 = {S79748, 139, { &P13386,&P5448,&P4808,&P18092 }};	/*Useragent - LG-GM730/v10a WM/6.1 IEMobile/6.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA9016 = {S79749, 192, { &P13386,&P5448,&P4808,&P18092 }};	/*Useragent - IEMobile/v6.0 LG-GM730/V20b Browser/IE8.12 MMS/LGMMSv1.0/1.2 Java/LGVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1/Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA13252 = {S79750, 71, { &P15207,&P5448,&P22271,&P18092 }};	/*Useragent - HTC_P3400i-Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA88189 = {S79751, 29, { &P15364,&P17017,&P4924,&P18092 }};	/*Useragent - C5005/001.00 ACS-NetFront/3.2*/
const Device UA88402 = {S79752, 29, { &P15364,&P17017,&P4924,&P18092 }};	/*Useragent - C5330/001.00 ACS-NetFront/3.2*/
const Device UA88831 = {S79753, 29, { &P15364,&P17017,&P4924,&P18092 }};	/*Useragent - C5320/001.00 ACS-NetFront/3.2*/
const Device UA88990 = {S79754, 29, { &P15364,&P17017,&P4924,&P18092 }};	/*Useragent - C5588/001.00 ACS-NetFront/3.2*/
const Device UA89747 = {S79755, 29, { &P15364,&P17017,&P4924,&P18092 }};	/*Useragent - C5110/001.00 ACS-Netfront/3.2*/
const Device UA89750 = {S79756, 29, { &P15364,&P17017,&P4924,&P18092 }};	/*Useragent - C6100/001.00 ACS-Netfront/3.2*/
const Device UA219956 = {S79757, 139, { &P17595,&P17296,&P22445,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Iron/7.0.520.1 Chrome/7.0.520.1 Safari/534.7*/
const Device UA914813 = {S79758, 139, { &P17595,&P17296,&P22445,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Iron/7.0.520.1 Chrome/7.0.520.1 Safari/534.7*/
const Device UA657468 = {S79759, 100, { &P16074,&P5768,&P17347,&P18092 }};	/*Useragent - Nokia3110c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia3110c/UC Browser8.0.3.107/69/444*/
const Device UA695138 = {S79760, 100, { &P13035,&P5043,&P17347,&P18092 }};	/*Useragent - NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-02/UC Browser8.0.3.107/69/444*/
const Device UA614941 = {S79761, 80, { &P17595,&P17297,&P18367,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:12.0) Gecko/X Thunderbird/12.0.1*/
const Device UA2373472 = {S79762, 146, { &P12473,&P5204,&P24667,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia2730c-1) UCBrowser8.2.0.132/69/452/UCWEB Mobile*/
const Device UA2228733 = {S79763, 145, { &P15809,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Rhyme S510b Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2340273 = {S79764, 149, { &P12871,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Wildfire S A510e Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2417275 = {S79765, 132, { &P21136,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ca; HS-7DTB5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2425636 = {S79766, 141, { &P19860,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A9+ Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2486853 = {S79767, 133, { &P19860,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A9+ Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2698806 = {S79768, 145, { &P19860,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A9+(Flare S100) Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2428150 = {S79769, 131, { &P23845,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; ADT 1.0 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2527483 = {S79770, 155, { &P18686,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A810S Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2779553 = {S79771, 180, { &P18686,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A810K Build/VEGAVIET.COM ROM JELLYBEAN V1.0; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2530341 = {S79772, 171, { &P13974,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2558964 = {S79773, 171, { &P13974,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2560782 = {S79774, 91, { &P15206,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAE90) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2567343 = {S79775, 141, { &P20452,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Lenovo S880 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2570571 = {S79776, 143, { &P19843,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE BLADE III Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2583368 = {S79777, 143, { &P19843,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE Blade III Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668790 = {S79778, 139, { &P19843,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Blade III Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583218 = {S79779, 133, { &P20231,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A21 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667167 = {S79780, 141, { &P20231,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A21 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2603303 = {S79781, 139, { &P14854,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667481 = {S79782, 139, { &P14854,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2861468 = {S79783, 139, { &P14854,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2630299 = {S79784, 192, { &P15919,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0530, in*/
const Device UA2631286 = {S79785, 192, { &P15919,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0545, np*/
const Device UA2633832 = {S79786, 193, { &P15919,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0700, th*/
const Device UA2635801 = {S79787, 199, { &P15919,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0800, ph*/
const Device UA2684434 = {S79788, 132, { &P15364,&P5690,&P23737,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/527+ (KHTML, like Gecko) QtWeb Internet Browser/1.5 http://www.QtWeb.net*/
const Device UA2667907 = {S79789, 137, { &P22755,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; DM014SH Build/S0014) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2704602 = {S79790, 137, { &P22755,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; DM014SH Build/S0010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2780328 = {S79791, 59, { &P22755,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; DM014SH Build/S0014)*/
const Device UA2677692 = {S79792, 145, { &P21489,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Infected Rezound Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2718979 = {S79793, 149, { &P21489,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Infected HTC Rezound Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2752153 = {S79794, 148, { &P21489,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Rezound on TRON 2.1 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2860443 = {S79795, 148, { &P21489,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Rezound on TRON-2.0 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735362 = {S79796, 187, { &P15794,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2799114 = {S79797, 159, { &P22209,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Titanium S5 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2741795 = {S79798, 101, { &P22713,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 521)*/
const Device UA2796834 = {S79799, 167, { &P23122,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; Karbonn A6 Build/MocorDroid4.0.3) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2754189 = {S79800, 140, { &P20452,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Lenovo S880 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784790 = {S79801, 156, { &P22594,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2872217 = {S79802, 138, { &P23674,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; SX7-PEARL.GmbH Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2944408 = {S79803, 159, { &P16292,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; Galaxy Nexus Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2965692 = {S79804, 160, { &P16292,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Galaxy Nexus Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2947625 = {S79805, 136, { &P20713,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID7065 Build/ICS.MID7065.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Safari/537.36*/
const Device UA3031199 = {S79806, 147, { &P12879,&P6546,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.1; GT-P7510 Build/HMJ37) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3037010 = {S79807, 155, { &P18464,&P18066,&P24334,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA3060451 = {S79808, 92, { &P12581,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia500) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3062717 = {S79809, 140, { &P16869,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Micromax A76 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3081573 = {S79810, 140, { &P16869,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Micromax A74 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3094493 = {S79811, 155, { &P20146,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9082 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA20411 = {S79812, 84, { &P13929,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-X990/X990_V2_Z13_EN_D18F101 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C*/
const Device UA20414 = {S79813, 83, { &P13929,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-X990-VIVO/VIVO_X990_V1.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C*/
const Device UA23566 = {S79814, 84, { &P13929,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-X990/X990_V2_Z14_PT_D18F100 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C*/
const Device UA33127 = {S79815, 84, { &P13929,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-X990/X990_V2_Z21_ES_D18F103 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C*/
const Device UA51079 = {S79816, 84, { &P13929,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-X990/X990_V2_Z15_PT_D18F108 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C*/
const Device UA58808 = {S79817, 89, { &P13929,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-X990-VIVO/X990_V2_Z10_PT_D18F100 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C*/
const Device UA60725 = {S79818, 84, { &P13929,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-X990/X990_V2_Z18_EN_D18F100 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C*/
const Device UA255549 = {S79819, 99, { &P15364,&P7127,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA367651 = {S79820, 121, { &P15364,&P17285,&P22417,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.2.1.0 Chrome/16.0.912.75 Safari/535.7*/
const Device UA368357 = {S79821, 128, { &P15364,&P17285,&P22417,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.2.1.0 Chrome/16.0.912.75 Safari/535.7*/
const Device UA643208 = {S79822, 28, { &P18254,&P17017,&P21799,&P18092 }};	/*Useragent - Akregator/4.8.3; syndication*/
const Device UA1990765 = {S79823, 28, { &P18254,&P17017,&P21799,&P18092 }};	/*Useragent - Akregator/4.8.4; syndication*/
const Device UA2159349 = {S79824, 28, { &P18254,&P17017,&P21799,&P18092 }};	/*Useragent - Akregator/4.8.2; syndication*/
const Device UA2239607 = {S79825, 28, { &P18254,&P17017,&P21799,&P18092 }};	/*Useragent - Akregator/4.8.5; syndication*/
const Device UA2644641 = {S79826, 33, { &P18254,&P17017,&P21799,&P18092 }};	/*Useragent - Akregator/4.8.5; syndication SUSE*/
const Device UA691250 = {S79827, 161, { &P14913,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SPH-M820 Build/GINGERBREAD; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA783214 = {S79828, 153, { &P12731,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; MB855 Build/4.5.1A-1_SUN-198_6)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032354 = {S79829, 167, { &P12731,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Motorola Electrify Build/4.5.1A_SUN_USC_19.0)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2771219 = {S79830, 155, { &P12731,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2997656 = {S79831, 168, { &P12731,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Motorola Electrify Build/4.5.1A_SUN_USC_19.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA1952609 = {S79832, 143, { &P21671,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; eGlide Steal Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1993473 = {S79833, 136, { &P24010,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; MIDC800PR001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2261580 = {S79834, 128, { &P20556,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A701 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2457336 = {S79835, 134, { &P20556,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A701 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2339534 = {S79836, 139, { &P21336,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HS-EG906 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2832970 = {S79837, 130, { &P21336,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HS-EG906 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3042574 = {S79838, 61, { &P21336,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HS-EG906 Build/IMM76D)*/
const Device UA2348890 = {S79839, 152, { &P15880,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB865 Build/JRO03C; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2889463 = {S79840, 152, { &P15880,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MB865 Build/JRO03C; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2367797 = {S79841, 42, { &P18254,&P18580,&P18245,&P18091 }};	/*Useragent - Tweetbot/26000 CFNetwork/609 Darwin/13.0.0*/
const Device UA2455041 = {S79842, 42, { &P18254,&P18580,&P18245,&P18091 }};	/*Useragent - Tweetbot/25100 CFNetwork/609 Darwin/13.0.0*/
const Device UA2458889 = {S79843, 42, { &P18254,&P18580,&P18245,&P18091 }};	/*Useragent - Tweetbot/26200 CFNetwork/609 Darwin/13.0.0*/
const Device UA2603527 = {S79844, 46, { &P18254,&P18580,&P18245,&P18091 }};	/*Useragent - Tweetbot/27301 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2737664 = {S79845, 46, { &P18254,&P18580,&P18245,&P18091 }};	/*Useragent - Tweetbot/28000 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2853015 = {S79846, 46, { &P18254,&P18580,&P18245,&P18091 }};	/*Useragent - Tweetbot/27100 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2457769 = {S79847, 159, { &P13042,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Incredible S Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2537307 = {S79848, 123, { &P15364,&P16110,&P19963,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17 Sleipnir/3.9.1*/
const Device UA2602938 = {S79849, 116, { &P15364,&P16110,&P19963,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17 Sleipnir/3.9.1*/
const Device UA2558898 = {S79850, 90, { &P18302,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MT25i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2581860 = {S79851, 138, { &P23388,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TSINTECH Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2660191 = {S79852, 107, { &P19868,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_985N) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2783717 = {S79853, 93, { &P18321,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5300) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA3014853 = {S79854, 139, { &P16292,&P21637,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Galaxy Nexus Build/JWR66V) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2765589 = {S79855, 144, { &P23036,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Viettel V8502 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2799277 = {S79856, 160, { &P17625,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST25i Build/6.1.1.B.1.10) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799081 = {S79857, 147, { &P18471,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-66) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3019230 = {S79858, 147, { &P18471,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-78) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3092114 = {S79859, 149, { &P18471,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-78-1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2879380 = {S79860, 129, { &P20216,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M1062 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3023154 = {S79861, 93, { &P13974,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5670) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3021536 = {S79862, 131, { &P23896,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; S5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2962543 = {S79863, 93, { &P18283,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9300) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3022800 = {S79864, 134, { &P18471,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT907 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2963096 = {S79865, 161, { &P17324,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3009064 = {S79866, 161, { &P17324,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3012468 = {S79867, 161, { &P17324,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3018230 = {S79868, 161, { &P17324,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2975580 = {S79869, 138, { &P22591,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022276 = {S79870, 132, { &P21603,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AMID-9743G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3080830 = {S79871, 133, { &P21048,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; W732 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3059010 = {S79872, 157, { &P18556,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; BN NookHD+ Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.2/ovation*/
const Device UA3062131 = {S79873, 157, { &P18556,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; BN NookHD+ Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.2/ovation*/
const Device UA3127364 = {S79874, 157, { &P18556,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; BN NookHD+ Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.2/ovation*/
const Device UA3083179 = {S79875, 134, { &P20414,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SmartTabII10 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA4543 = {S79876, 111, { &P14709,&P17484,&P3636,&P18092 }};	/*Useragent - SAMSUNG-GT-M8800/M880BXXJA2 SHP/VPP/R5 Jasmine/0.8 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA63690 = {S79877, 60, { &P12128,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia5140/2.0 (3.10) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62242 = {S79878, 64, { &P13255,&P17483,&P3459,&P18092 }};	/*Useragent - LG-GR700/V10g Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA432574 = {S79879, 148, { &P18020,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SKY IM-A650S Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA495074 = {S79880, 70, { &P18020,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SKY IM-A650S Build/ERE27)*/
const Device UA2742346 = {S79881, 146, { &P18020,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-; SKY IM-A650S Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA249241 = {S79882, 43, { &P11905,&P5890,&P24663,&P18092 }};	/*Useragent - NOKIA6120c/UC Browser7.9.0.102/28/355/UCWEB*/
const Device UA396977 = {S79883, 63, { &P15364,&P7127,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; ru) Presto/2.9.168 Version/11.51*/
const Device UA635919 = {S79884, 76, { &P15364,&P17285,&P22356,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:11.0) Gecko/X Firefox/11.0.1 PaleMoon/11.0.1*/
const Device UA856384 = {S79885, 96, { &P15364,&P17285,&P22356,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:11.0) Gecko/X Firefox/11.0.1-x64 PaleMoon/11.0.1-x64*/
const Device UA572703 = {S79886, 47, { &P12193,&P5890,&P17338,&P18092 }};	/*Useragent - nokiac5-00/UC Browser7.9.0.102/69/352  3gpp-gba*/
const Device UA573861 = {S79887, 49, { &P12193,&P5890,&P17338,&P18092 }};	/*Useragent - nokiac5-00.2/UC Browser7.9.0.102/69/352  3gpp-gba*/
const Device UA578630 = {S79888, 114, { &P14711,&P5204,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-01/11.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA805807 = {S79889, 84, { &P14519,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaN97 mini)/UC Browser8.2.0.132/50/355/UCWEB Mobile*/
const Device UA1948665 = {S79890, 35, { &P14519,&P5129,&P24667,&P18092 }};	/*Useragent - NokiaN97 mini/UCWEB8.2.0.116/50/800*/
const Device UA700847 = {S79891, 134, { &P21253,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; LAVA S12 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA718309 = {S79892, 85, { &P18141,&P22215,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 1.00) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2*/
const Device UA1956278 = {S79893, 135, { &P21532,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; vivo Y1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2816862 = {S79894, 128, { &P21532,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; vivo Y1 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2029131 = {S79895, 149, { &P21240,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile i-style Q2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2352520 = {S79896, 153, { &P21240,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile i-STYLE Q2 DUO Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2261363 = {S79897, 145, { &P17346,&P18432,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2367406 = {S79898, 147, { &P17346,&P18432,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2352508 = {S79899, 145, { &P21962,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Ken Brown Synkom Build/A777) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2505397 = {S79900, 149, { &P21962,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Ken Brown Synkom Build/tablet-c) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2424495 = {S79901, 99, { &P20126,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonk800i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2457225 = {S79902, 99, { &P20126,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonk800i) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2482878 = {S79903, 96, { &P20126,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; fa; sonyericssonk800i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2843032 = {S79904, 96, { &P20126,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; fa; sonyericssonk800i) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2469138 = {S79905, 151, { &P18219,&P5843,&P24675,&P18092 }};	/*Useragent - Nokia202/2.0 (20.28) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia202) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2531122 = {S79906, 171, { &P16436,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2537576 = {S79907, 53, { &P15364,&P6729,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows 95) Presto/2.12.388 Version/12.14*/
const Device UA2558720 = {S79908, 90, { &P18450,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST23i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2581758 = {S79909, 170, { &P19779,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A1+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3021934 = {S79910, 174, { &P19779,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A1+ Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2659386 = {S79911, 154, { &P18296,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC One V Build/IML74K) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2671261 = {S79912, 93, { &P15364,&P22276,&P21710,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) KHTML/4.10.1 (like Gecko) Konqueror/4.10 Fedora/4.10.1-1.fc17*/
const Device UA2717592 = {S79913, 136, { &P22945,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; OnePAD 1100x2 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2738718 = {S79914, 153, { &P20146,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2739994 = {S79915, 158, { &P19862,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2753164 = {S79916, 126, { &P18733,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M532 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755083 = {S79917, 133, { &P14997,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S100 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2767298 = {S79918, 140, { &P22729,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT918 Build/2_26C_2009) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2866686 = {S79919, 140, { &P22729,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT918 Build/2_26D_2009) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957642 = {S79920, 62, { &P22729,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT918 Build/2_26D_2009)*/
const Device UA3124561 = {S79921, 140, { &P22729,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT918 Build/2_32D_2009) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2773277 = {S79922, 156, { &P12788,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PG86100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2862262 = {S79923, 152, { &P23239,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3076893 = {S79924, 145, { &P23239,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2866112 = {S79925, 139, { &P23797,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; GT-1240 Build/GT-1240-v1.4-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2997971 = {S79926, 139, { &P23797,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; GT-1240 Build/GT-1240-v2.2-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3088447 = {S79927, 139, { &P23797,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; GT-1240 Build/GT-1240-v2.1-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3107745 = {S79928, 139, { &P23797,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; GT-1240 Build/GT-1240-v2.4-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2894622 = {S79929, 93, { &P18321,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5300) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2937284 = {S79930, 137, { &P21869,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; ZP950+ Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2991122 = {S79931, 137, { &P21869,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ZP950H Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2945350 = {S79932, 93, { &P14186,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3021500 = {S79933, 130, { &P21202,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-P500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974651 = {S79934, 145, { &P19867,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I437 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975702 = {S79935, 131, { &P23177,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TAB-840_G Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3025613 = {S79936, 161, { &P18672,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5302 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3081333 = {S79937, 148, { &P17835,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook Tablet Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3130725 = {S79938, 138, { &P20146,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131086 = {S79939, 137, { &P20146,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA48912 = {S79940, 110, { &P13867,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-C3510T/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA724957 = {S79941, 138, { &P21846,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; TAB-PLAYTABPRO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1962699 = {S79942, 142, { &P21846,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; TAB-PLAYTABPRO(4R) Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1995941 = {S79943, 80, { &P14984,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5250)/UC Browser8.5.0.183/50/352/UCWEB Mobile*/
const Device UA2810061 = {S79944, 31, { &P14984,&P5129,&P24671,&P18092 }};	/*Useragent - Nokia5250/UCWEB8.5.0.163/50/800*/
const Device UA2031864 = {S79945, 130, { &P21279,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MP1047 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2368037 = {S79946, 155, { &P18680,&P18066,&P19803,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2417663 = {S79947, 196, { &P14682,&P5843,&P24671,&P18092 }};	/*Useragent - Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia6303iclassic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2423435 = {S79948, 129, { &P23912,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; D976D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2511894 = {S79949, 166, { &P18283,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2664137 = {S79950, 70, { &P17346,&P17090,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPod4,1; iPhone OS 4.3.5; en_US; en) AppleWebKit/420+*/
const Device UA2698861 = {S79951, 70, { &P17346,&P17090,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPod4,1; iPhone OS 4.3.3; en_US; en) AppleWebKit/420+*/
const Device UA2559813 = {S79952, 161, { &P19860,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A9+ Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2599528 = {S79953, 169, { &P19860,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Karbonn A9+ Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3107784 = {S79954, 181, { &P19860,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; CELKON_A9+ Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2599654 = {S79955, 155, { &P13074,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-P930 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2866728 = {S79956, 158, { &P13074,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-P930 Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2676664 = {S79957, 129, { &P20750,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AN10DG3 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2712113 = {S79958, 80, { &P15364,&P16110,&P22317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:21.0) Gecko/X Firefox/21.0 SeaMonkey/2.18*/
const Device UA2754474 = {S79959, 126, { &P20269,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A211 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755453 = {S79960, 135, { &P21037,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LIFETAB_S9512 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2767353 = {S79961, 132, { &P22982,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; Primo 91 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2799246 = {S79962, 147, { &P16362,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-D710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3050819 = {S79963, 150, { &P16362,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-D710BST Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3074328 = {S79964, 151, { &P16362,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-D710VMUB Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2867487 = {S79965, 154, { &P23151,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PAP4044DUO Build/PrestigioPAP4044DUO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2943790 = {S79966, 165, { &P21351,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HUAWEI C8813 Build/HuaweiC8813) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2972712 = {S79967, 167, { &P21351,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI C8813Q Build/HuaweiC8813Q) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3077604 = {S79968, 167, { &P21351,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HUAWEI C8813D Build/HuaweiC8813D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2951579 = {S79969, 120, { &P17595,&P21721,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11*/
const Device UA2976683 = {S79970, 133, { &P20868,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M040 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974709 = {S79971, 154, { &P20541,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAP4500TDUO Build/PrestigioPAP4500TDUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3092209 = {S79972, 152, { &P20541,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAP4500DUO Build/PrestigioPAP4500DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3031509 = {S79973, 159, { &P20289,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2975743 = {S79974, 133, { &P14997,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S100 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974106 = {S79975, 133, { &P20584,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Z120 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3064791 = {S79976, 160, { &P18462,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC Desire X Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3080044 = {S79977, 169, { &P18462,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC Desire X dual sim Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3031562 = {S79978, 138, { &P24513,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530X Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3109150 = {S79979, 131, { &P24513,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530X Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3081480 = {S79980, 157, { &P18604,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; STARADDICT II Plus Build/GXI_ZTEV1.0.0B03) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126982 = {S79981, 178, { &P18604,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Smartphone Android by SFR STARADDICT II Build/IceCreamSandwich) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080539 = {S79982, 138, { &P20816,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CINK SLIM Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3132973 = {S79983, 134, { &P18635,&P21637,&P24394,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus 4 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1650.11 Mobile Safari/537.36*/
const Device UA3130590 = {S79984, 130, { &P12879,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3094130 = {S79985, 161, { &P19862,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax A110 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3094455 = {S79986, 161, { &P18513,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6802 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3096277 = {S79987, 150, { &P23773,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8010 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA401361 = {S79988, 123, { &P15364,&P5690,&P22422,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.3.0.0 Chrome/17.0.963.46 Safari/535.11*/
const Device UA884927 = {S79989, 91, { &P14533,&P6173,&P4779,&P18092 }};	/*Useragent - BlackBerry9630/4.2.0.424 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/159/444*/
const Device UA458361 = {S79990, 148, { &P15539,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Glacier Build/Zips_Glacier) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA696046 = {S79991, 139, { &P15539,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Glacier Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA895043 = {S79992, 142, { &P15539,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.5; en-us; HTC Glacier Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2024766 = {S79993, 142, { &P15539,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Glacier Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2475068 = {S79994, 142, { &P15539,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Glacier Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1984025 = {S79995, 153, { &P21091,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Celkon_A97 Build/Celkon_A97Celkon_A97) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2017665 = {S79996, 72, { &P15364,&P17285,&P22363,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:15.0) Gecko/X Firefox/15.0 PaleMoon/15.0*/
const Device UA2180555 = {S79997, 92, { &P15364,&P17285,&P22363,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:15.0) Gecko/X Firefox/15.0-x64 PaleMoon/15.0-x64*/
const Device UA2033436 = {S79998, 129, { &P21236,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VIGO7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2554984 = {S79999, 138, { &P18583,&P5043,&P24668,&P18092 }};	/*Useragent - Nokia110/2.0 (03.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia110) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2319345 = {S80000, 32, { &P13324,&P5055,&P24670,&P18092 }};	/*Useragent - NokiaN8-00/UCWEB8.4.0.150/51/800*/
const Device UA2551955 = {S80001, 89, { &P13324,&P5055,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokian8-00) UCBrowser8.4.0.159/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA2258404 = {S80002, 113, { &P12108,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia7230/10.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2692960 = {S80003, 116, { &P12108,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia7230-1c/10.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2262248 = {S80004, 157, { &P14186,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2384356 = {S80005, 157, { &P14186,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2746391 = {S80006, 157, { &P14186,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2378343 = {S80007, 165, { &P20094,&P18066,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2414264 = {S80008, 136, { &P20462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; PTL21 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2421126 = {S80009, 130, { &P20256,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TAB275 Build/TAB275) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2530966 = {S80010, 161, { &P16036,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus One Build/VJ CM10.1; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2667030 = {S80011, 161, { &P16036,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus One Build/VJ CM10.1; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580601 = {S80012, 95, { &P12444,&P5477,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; U20i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2596536 = {S80013, 93, { &P15919,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-P3100) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2704178 = {S80014, 137, { &P21880,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; DM-992 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057516 = {S80015, 138, { &P21880,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ar-eg; DM-992a Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2765307 = {S80016, 139, { &P21417,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IdeaTabA2207A-H Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2715723 = {S80017, 147, { &P23123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Intex Aqua Swadesh Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2768486 = {S80018, 144, { &P22861,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; Andi4v Build/MocorDroid2.3.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2769168 = {S80019, 136, { &P19856,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-LS970 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784707 = {S80020, 162, { &P21020,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Micromax A116 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2800238 = {S80021, 129, { &P23180,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; TB311 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2865069 = {S80022, 136, { &P23478,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; ZP810 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3020706 = {S80023, 128, { &P23135,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; U30GT 2 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3093514 = {S80024, 139, { &P22756,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C1604 Build/11.1.A.0.64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081465 = {S80025, 136, { &P22874,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9502 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3096347 = {S80026, 157, { &P16356,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA4440 = {S80027, 111, { &P14432,&P17483,&P3534,&P18092 }};	/*Useragent - LG-KF700/v10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA4442 = {S80028, 118, { &P14432,&P17483,&P3534,&P18092 }};	/*Useragent - LG-KF700-Orange/v10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA32758 = {S80029, 114, { &P14432,&P17483,&P3534,&P18092 }};	/*Useragent - LG/KF700-O2/v10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA54989 = {S80030, 107, { &P14432,&P17483,&P3534,&P18092 }};	/*Useragent - LG-KF700C Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA62635 = {S80031, 111, { &P14432,&P17483,&P3534,&P18092 }};	/*Useragent - LG/KF700/v10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA9021 = {S80032, 136, { &P14042,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SonyEricssonX2/R3AA; Profile/MIDP-2.1; Configuration/CLDC-1.1; Windows Phone 6.5.3.5)*/
const Device UA64609 = {S80033, 98, { &P13760,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-Z170/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA64849 = {S80034, 61, { &P11865,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia6610I/1.0 (4.10) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA18834 = {S80035, 40, { &P15364,&P17017,&P3487,&P18092 }};	/*Useragent - HQ1 UP.Browser/7.2.6.1.591 (GUI) MMP/2.0*/
const Device UA21512 = {S80036, 52, { &P15364,&P17017,&P3487,&P18092 }};	/*Useragent - odkyom2k/ZQ1007 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0*/
const Device UA32928 = {S80037, 51, { &P15364,&P17017,&P3487,&P18092 }};	/*Useragent - wskyok4kc/1004 UP.Browser/7.2.6.1.838 (GUI) MMP/2.0*/
const Device UA69563 = {S80038, 79, { &P15364,&P17017,&P3487,&P18092 }};	/*Useragent - http://mms.revol.us/uaprofs/kyo-e1100/1003 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0*/
const Device UA87774 = {S80039, 53, { &P15364,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-E1000/1.0.12 UP.Browser/7.2.6.1.537 (GUI) MMP/2.0*/
const Device UA230574 = {S80040, 86, { &P15364,&P17017,&P3487,&P18092 }};	/*Useragent - http://mms.revol.us/uaprofs/M1400-AAR.xml/AAR1004 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0*/
const Device UA297244 = {S80041, 79, { &P15364,&P17017,&P3487,&P18092 }};	/*Useragent - http://mms.revol.us/uaprofs/s4000-VQ/10022 UP.Browser/7.2.6.1.838 (GUI) MMP/2.0*/
const Device UA812553 = {S80042, 79, { &P15364,&P17017,&P3487,&P18092 }};	/*Useragent - http://mms.revol.us/uaprofs/kyo-e2000/1013 UP.Browser/7.2.6.1.567 (GUI) MMP/2.0*/
const Device UA182934 = {S80043, 54, { &P17516,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-CA3J UP.Browser/6.2_7.2.7.1.K.5.207 (GUI) MMP/2.0*/
const Device UA264800 = {S80044, 141, { &P15364,&P7133,&P22398,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Comodo_Dragon/4.1.1.12 Chrome/4.1.249.1042 Safari/532.5*/
const Device UA368859 = {S80045, 137, { &P15575,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA611633 = {S80046, 137, { &P15575,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA645544 = {S80047, 135, { &P15575,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-gb; GT-I9100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030713 = {S80048, 59, { &P15575,&P24697,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; GT-I9100 Build/MIUI)*/
const Device UA2382370 = {S80049, 140, { &P15575,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; GT-I9100 Build/MIUI.es) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA572715 = {S80050, 100, { &P13035,&P5043,&P17338,&P18092 }};	/*Useragent - NokiaC2-02/2.0 (06.52) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-02/UC Browser7.9.0.102/69/352*/
const Device UA753424 = {S80051, 131, { &P15364,&P17285,&P22511,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.1.6) Gecko/X FireDownload/2.0.1 Firefox/3.5.6 Wyzo/3.5.6.1 FireTorrent/2.0.1*/
const Device UA678997 = {S80052, 47, { &P12449,&P5890,&P17347,&P18092 }};	/*Useragent - nokiae5-00/UC Browser8.0.3.107/69/444  3gpp-gba*/
const Device UA690919 = {S80053, 45, { &P12581,&P5055,&P17347,&P18092 }};	/*Useragent - nokia500/UC Browser8.0.3.107/69/444  3gpp-gba*/
const Device UA786323 = {S80054, 66, { &P15842,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;PI39100;7.10;7720)*/
const Device UA925701 = {S80055, 70, { &P15842,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;TITAN X310e;7.10;7720)*/
const Device UA925775 = {S80056, 70, { &P15842,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;TITAN X310e;7.10;7740)*/
const Device UA925784 = {S80057, 70, { &P15842,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;TITAN X310e;7.10;8107)*/
const Device UA925865 = {S80058, 70, { &P15842,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;TITAN X310e;7.10;8773)*/
const Device UA1953601 = {S80059, 126, { &P11751,&P17483,&P24668,&P18092 }};	/*Useragent - LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-kp500 teleca) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA1973340 = {S80060, 136, { &P21332,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-E860 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2589897 = {S80061, 129, { &P21332,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-E860 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1981010 = {S80062, 157, { &P22381,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SN10T1 Build/IML77; CyanogenMod-TeamDRH) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028137 = {S80063, 132, { &P22120,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; SPNG9716 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032610 = {S80064, 136, { &P20518,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; GIONEE A10 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2258534 = {S80065, 131, { &P20518,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; A10 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2167290 = {S80066, 74, { &P15364,&P16110,&P22535,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; rv:10.0.3) Gecko/X Firefox/10.0.3 Epic/10.0.3*/
const Device UA2258394 = {S80067, 117, { &P17595,&P18207,&P23600,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Fluid/0.9.6 Safari/536.26.14*/
const Device UA2668942 = {S80068, 117, { &P17595,&P18207,&P23600,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/536.28.10 (KHTML, like Gecko) Fluid/0.9.6 Safari/536.28.10*/
const Device UA3001023 = {S80069, 115, { &P17595,&P18207,&P23600,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) Fluid/0.9.6 Safari/536.30.1*/
const Device UA2260760 = {S80070, 138, { &P18648,&P5043,&P24668,&P18092 }};	/*Useragent - Nokia306/2.0 (03.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia306) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2362476 = {S80071, 88, { &P18254,&P18110,&P18099,&P18091 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:10.0.7) Gecko/X YextBot/Java Firefox/10.0.7*/
const Device UA2384123 = {S80072, 125, { &P14985,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s8530) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2383132 = {S80073, 142, { &P14848,&P5843,&P24668,&P18092 }};	/*Useragent - NokiaX3-00/5.0 (11.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax3-00) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2385420 = {S80074, 132, { &P20004,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A2 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2577484 = {S80075, 190, { &P13337,&P17485,&P24671,&P18092 }};	/*Useragent - SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonj108i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2425460 = {S80076, 143, { &P20931,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; iBall Slide 3G 7334 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2527033 = {S80077, 141, { &P20931,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; iBall Slide i9702 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3085440 = {S80078, 149, { &P20931,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; iBall Slide 3G 7307 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2427588 = {S80079, 132, { &P18277,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; N-04D Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2461814 = {S80080, 180, { &P18510,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; ALCATEL ONE TOUCH 918N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2482106 = {S80081, 152, { &P20136,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Star Mobile Bright Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2530934 = {S80082, 169, { &P18223,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT22i Build/6.1.1.B.1.54) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2545390 = {S80083, 122, { &P18221,&P22723,&P22724,&P18092 }};	/*Useragent - Nokia3020/5.0 (14.53) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA2601195 = {S80084, 146, { &P20558,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ608 Build/7.7.1-141-7-FLEM-UMTS-LA) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2601591 = {S80085, 144, { &P20558,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ608-14) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2625631 = {S80086, 192, { &P17323,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +0800, my*/
const Device UA2659394 = {S80087, 92, { &P14782,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAE61i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2661670 = {S80088, 91, { &P14782,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAE61) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2676509 = {S80089, 132, { &P20518,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A10 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735076 = {S80090, 125, { &P20518,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A10 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2700131 = {S80091, 88, { &P19860,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A9+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2730603 = {S80092, 96, { &P19860,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Karbonn_A9+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3091491 = {S80093, 95, { &P19860,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; CELKON_A9+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2706078 = {S80094, 43, { &P17595,&P17296,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.2.X (Intel Mac OS X 10.6.8)*/
const Device UA2749170 = {S80095, 43, { &P17595,&P17296,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.1.X (Intel Mac OS X 10.6.8)*/
const Device UA2752831 = {S80096, 43, { &P17595,&P17296,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.4.X (Intel Mac OS X 10.6.8)*/
const Device UA2964081 = {S80097, 43, { &P17595,&P17296,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.5.X (Intel Mac OS X 10.6.8)*/
const Device UA3019450 = {S80098, 43, { &P17595,&P17296,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.6.X (Intel Mac OS X 10.6.8)*/
const Device UA3110901 = {S80099, 43, { &P17595,&P17296,&P21586,&P18092 }};	/*Useragent - MacOutlook/14.3.7.X (Intel Mac OS X 10.6.8)*/
const Device UA2714244 = {S80100, 139, { &P23905,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100P Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2867970 = {S80101, 93, { &P18442,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7100) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2754348 = {S80102, 140, { &P22209,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Titanium S5 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773270 = {S80103, 156, { &P22597,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2783712 = {S80104, 159, { &P15047,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SK17i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2896726 = {S80105, 152, { &P17595,&P21721,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100 (Edition Next)*/
const Device UA3028625 = {S80106, 137, { &P17595,&P21721,&P24175,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 OPR/15.0.1147.153*/
const Device UA2897744 = {S80107, 161, { &P18214,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S7500 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2941733 = {S80108, 140, { &P24135,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; MITO A355 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956262 = {S80109, 136, { &P22804,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P713 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3016355 = {S80110, 138, { &P22804,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P713GO Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2964605 = {S80111, 152, { &P18558,&P20706,&P24678,&P18092 }};	/*Useragent - Nokia309/2.0 (05.85) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia309) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3033173 = {S80112, 150, { &P22606,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3020862 = {S80113, 136, { &P22948,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P716 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023823 = {S80114, 159, { &P14969,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Desire S Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3092621 = {S80115, 159, { &P14969,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; HTC Desire S Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA17037 = {S80116, 128, { &P11890,&P5338,&P3772,&P18092 }};	/*Useragent - MOT-MOTOZ9/9E.03.3AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003352) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.60 [en]*/
const Device UA17040 = {S80117, 128, { &P11890,&P5338,&P3772,&P18092 }};	/*Useragent - MOT-MOTOZ9/9E.01.16R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003306) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.60 [en]*/
const Device UA2210578 = {S80118, 122, { &P11890,&P5338,&P3772,&P18092 }};	/*Useragent - MOT-MOTOZ9/9E.02.0FR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; X) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [es]*/
const Device UA44694 = {S80119, 35, { &P11929,&P5129,&P4777,&P18092 }};	/*Useragent - Nokia5230/UC Browser7.7.1.88/50/444*/
const Device UA2290654 = {S80120, 37, { &P11929,&P5129,&P4777,&P18092 }};	/*Useragent - Nokia5230-c/UC Browser7.7.2.91/50/355*/
const Device UA2527105 = {S80121, 56, { &P14787,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3R UP.Browser/6.2_7.2.7.1.K.4.1.203 (GUI) MMP/2.0*/
const Device UA74600 = {S80122, 58, { &P13054,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player verizon_wwe / 1.0 / htc_mecha / 2.3.4*/
const Device UA547945 = {S80123, 58, { &P13054,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player verizon_wwe / 1.0 / htc_mecha / 2.2.1*/
const Device UA749140 = {S80124, 58, { &P13054,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player verizon_wwe / 1.0 / htc_vigor / 2.3.4*/
const Device UA2319229 = {S80125, 58, { &P13054,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player verizon_wwe / 1.0 / htc_vigor / 4.0.3*/
const Device UA2791208 = {S80126, 58, { &P13054,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player verizon_wwe / 1.0 / htc_mecha / 4.0.4*/
const Device UA202406 = {S80127, 54, { &P17580,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-CA3H UP.Browser/6.2_7.2.7.1.K.5.207 (GUI) MMP/2.0*/
const Device UA157477 = {S80128, 144, { &P21560,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; MID7005 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA433374 = {S80129, 78, { &P15364,&P5448,&P4737,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone)/UC Browser7.2.2.51*/
const Device UA502559 = {S80130, 87, { &P15364,&P5448,&P4737,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 176x220)/UC Browser7.2.2.51*/
const Device UA515301 = {S80131, 74, { &P15364,&P5448,&P4737,&P18092 }};	/*Useragent - T-Mobile_Maple (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.2.2.51*/
const Device UA520237 = {S80132, 90, { &P15364,&P5448,&P4737,&P18092 }};	/*Useragent - Sprint:SPH-i325 320x240 (compatible; MSIE 4.01; Windows CE; Smartphone)/UC Browser7.2.2.51*/
const Device UA675318 = {S80133, 71, { &P15364,&P5448,&P4737,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.2.2.51*/
const Device UA474769 = {S80134, 26, { &P18088,&P17017,&P24644,&P18092 }};	/*Useragent - J2ME/UCWEB7.4.0.57/139/999*/
const Device UA694188 = {S80135, 26, { &P18088,&P17017,&P24644,&P18092 }};	/*Useragent - J2ME/UCWEB7.4.0.57/139/800*/
const Device UA549383 = {S80136, 135, { &P15364,&P5690,&P22512,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.4) Gecko/X FireDownload/2.0.1 Firefox/3.6.4 Wyzo/3.6.4.1 SearchToolbar/1.2 KAT*/
const Device UA527317 = {S80137, 19, { &P15364,&P17017,&P23038,&P18092 }};	/*Useragent - NSPlayer/4.1.0.3856*/
const Device UA578670 = {S80138, 116, { &P14862,&P5204,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2700c-2/07.80; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA658400 = {S80139, 46, { &P14188,&P5129,&P17347,&P18092 }};	/*Useragent - nokia5233/UC Browser8.0.3.107/69/444  3gpp-gba*/
const Device UA717986 = {S80140, 85, { &P17595,&P17297,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:5.0.1) Gecko/X Firefox/5.0.1 Facicons*/
const Device UA853919 = {S80141, 136, { &P19938,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-i589 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021068 = {S80142, 124, { &P19938,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-i589 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2028568 = {S80143, 136, { &P19938,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; zh-cn; SCH-i589 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2167051 = {S80144, 79, { &P12581,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia500)/UC Browser8.5.0.183/50/444/UCWEB Mobile*/
const Device UA2028393 = {S80145, 138, { &P21536,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo E3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2546707 = {S80146, 129, { &P21536,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo E3 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2202734 = {S80147, 76, { &P15364,&P16110,&P22364,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; rv:15.0) Gecko/X Firefox/15.1.1 PaleMoon/15.1.1*/
const Device UA2203317 = {S80148, 96, { &P15364,&P16110,&P22364,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:15.0) Gecko/X Firefox/15.1.1-x64 PaleMoon/15.1.1-x64*/
const Device UA2692857 = {S80149, 152, { &P12784,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Skate Build/JRO03C; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2699923 = {S80150, 156, { &P12784,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Skate Build/JRO03C; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2368028 = {S80151, 131, { &P20750,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AN10DG3 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2385241 = {S80152, 129, { &P17926,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AT100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2427189 = {S80153, 146, { &P20105,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A919 Duo Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2428286 = {S80154, 91, { &P11945,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; gt-s5233s) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2506441 = {S80155, 84, { &P11945,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; gt-s5233s) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0*/
const Device UA2697057 = {S80156, 91, { &P11945,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; gt-s5233s) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2483510 = {S80157, 136, { &P21110,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; A8809 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2530226 = {S80158, 140, { &P21906,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MITO A220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583216 = {S80159, 138, { &P14225,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Desire HD Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667437 = {S80160, 139, { &P14225,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Desire HD Build/JOP40G) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2746809 = {S80161, 133, { &P23460,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I-Buddy 3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2673409 = {S80162, 161, { &P15575,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2713596 = {S80163, 162, { &P15575,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-S959G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2851712 = {S80164, 161, { &P15575,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000*/
const Device UA2632666 = {S80165, 192, { &P22043,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, -0600, us*/
const Device UA2661490 = {S80166, 63, { &P15364,&P7133,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; Windows NT 6.1; Trident/1.0)*/
const Device UA2676266 = {S80167, 138, { &P20517,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CINK PEAX Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676832 = {S80168, 148, { &P17835,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook Tablet Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676986 = {S80169, 138, { &P18474,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9210T Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2703122 = {S80170, 169, { &P12280,&P18117,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3 XiaoMi/MiuiBrowser/1.0*/
const Device UA2706166 = {S80171, 130, { &P24138,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; CTP828 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2796125 = {S80172, 88, { &P18095,&P18109,&P18414,&P18092 }};	/*Useragent - Opera/9.80 (Series 60; Opera Mini/7.6.32889/29.3551; U; zh) Presto/2.8.119 Version/11.10*/
const Device UA2732257 = {S80173, 138, { &P21201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N7200   Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955886 = {S80174, 134, { &P21201,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; vi-vn; N7200 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2737060 = {S80175, 158, { &P20004,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Karbonn A2 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2865392 = {S80176, 150, { &P20004,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A2 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2769695 = {S80177, 136, { &P18281,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P940 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2872160 = {S80178, 138, { &P18281,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-KU5400 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2958510 = {S80179, 137, { &P18281,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-SU540 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774092 = {S80180, 136, { &P22679,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2945145 = {S80181, 156, { &P18595,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8190 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2960413 = {S80182, 156, { &P18595,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; GT-I8190 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3021496 = {S80183, 152, { &P24072,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3080253 = {S80184, 132, { &P21603,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AMID-9743G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3038878 = {S80185, 158, { &P18113,&P24162,&P23997,&P18091 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; XT912 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/spyder*/
const Device UA3065422 = {S80186, 158, { &P18113,&P24162,&P23997,&P18091 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; XT912 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/spyder*/
const Device UA3111750 = {S80187, 158, { &P18113,&P24162,&P23997,&P18091 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; XT912 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/spyder*/
const Device UA3083481 = {S80188, 142, { &P21885,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GSmart GS202+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA5998 = {S80189, 155, { &P12347,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-G810/XEHA3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA24538 = {S80190, 155, { &P12347,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; Series60/3.1 Samsung/SGH-G810/JVHE1 Profile/MIDP-2.0 Configuration/CLDC-1.1 U; ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA30812 = {S80191, 155, { &P12347,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; Series60/3.1 Samsung/SGH-G810/DUHF2 Profile/MIDP-2.0 Configuration/CLDC-1.1 U; ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA30813 = {S80192, 155, { &P12347,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; Series60/3.1 Samsung/SGH-G810/JCHE2 Profile/MIDP-2.0 Configuration/CLDC-1.1 U; ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA30814 = {S80193, 155, { &P12347,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; Series60/3.1 Samsung/SGH-G810/JPHE1 Profile/MIDP-2.0 Configuration/CLDC-1.1 U; ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA30815 = {S80194, 155, { &P12347,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; Series60/3.1 Samsung/SGH-G810/DZHD4 Profile/MIDP-2.0 Configuration/CLDC-1.1 U; ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA36478 = {S80195, 155, { &P12347,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; Series60/3.1 Samsung/SGH-G810/JXHE1 Profile/MIDP-2.0 Configuration/CLDC-1.1 U; ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA62917 = {S80196, 155, { &P12347,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; Series60/3.1 Samsung/SGH-G810/DBHD4 Profile/MIDP-2.0 Configuration/CLDC-1.1 U; ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA29959 = {S80197, 73, { &P12280,&P17144,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8A400 (iPhone; U; CPU OS 4_0_2 like Mac OS X; en_us)*/
const Device UA106334 = {S80198, 71, { &P12280,&P17144,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8A293 (iPhone; U; CPU OS 4_0 like Mac OS X; en_us)*/
const Device UA116173 = {S80199, 73, { &P12280,&P17144,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8A306 (iPhone; U; CPU OS 4_0_1 like Mac OS X; en_us)*/
const Device UA32173 = {S80200, 82, { &P12476,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry8910_CMCC/5.0.0.608 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/204*/
const Device UA64380 = {S80201, 77, { &P12476,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry8910/5.0.0.461 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/612*/
const Device UA69296 = {S80202, 34, { &P14358,&P5890,&P4779,&P18092 }};	/*Useragent - NokiaE63/UC Browser7.8.0.95/28/355*/
const Device UA90738 = {S80203, 11, { &P16796,&P22784,&P17470,&P18092 }};	/*Useragent - MicromaxQ75*/
const Device UA2400593 = {S80204, 56, { &P17582,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SN3Q UP.Browser/6.2_7.2.7.1.K.6.1.102 (GUI) MMP/2.0*/
const Device UA770425 = {S80205, 87, { &P15364,&P18110,&P22311,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 on x86_64; rv:6.0.2) Gecko/X Firefox/6.0.2 SeaMonkey/2.3.3*/
const Device UA896358 = {S80206, 172, { &P20134,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2659441 = {S80207, 176, { &P20134,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5830i Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA624424 = {S80208, 112, { &P15803,&P5843,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia200/10.58; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA701423 = {S80209, 109, { &P17595,&P18120,&P23603,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-US) AppleWebKit/125.4 (KHTML, like Gecko, Safari) OmniWeb/v563.15*/
const Device UA1874817 = {S80210, 109, { &P17595,&P18120,&P23603,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-US) AppleWebKit/125.4 (KHTML, like Gecko, Safari) OmniWeb/v563.57*/
const Device UA2300074 = {S80211, 109, { &P17595,&P18120,&P23603,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-US) AppleWebKit/125.4 (KHTML, like Gecko, Safari) OmniWeb/v563.66*/
const Device UA2681110 = {S80212, 109, { &P17595,&P18120,&P23603,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-US) AppleWebKit/125.4 (KHTML, like Gecko, Safari) OmniWeb/v563.34*/
const Device UA804547 = {S80213, 81, { &P11907,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE52-1)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA1893328 = {S80214, 76, { &P15364,&P6704,&P20906,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win 9x 4.90; de-DE; rv:0.9.2) Gecko/X Netscape6/6.1*/
const Device UA1994845 = {S80215, 129, { &P22102,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; M-285 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2404662 = {S80216, 136, { &P21005,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; F-10D Build/V20R47F) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2028236 = {S80217, 154, { &P12698,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; LG-C660 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2202990 = {S80218, 93, { &P17595,&P18207,&P20755,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:15.0) Gecko/X Firefox/15.0.1 SeaMonkey/2.12.1*/
const Device UA2168416 = {S80219, 148, { &P23937,&P18066,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Y2 Build/HCL ME Tablet Y2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2202778 = {S80220, 131, { &P15767,&P18117,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.1.1m*/
const Device UA2237721 = {S80221, 131, { &P15767,&P18117,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.1.2m*/
const Device UA2357867 = {S80222, 131, { &P15767,&P18117,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.1.3m*/
const Device UA2844275 = {S80223, 131, { &P15767,&P18117,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.1.4m*/
const Device UA2337505 = {S80224, 132, { &P21387,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A360/S100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2789439 = {S80225, 133, { &P21387,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A360 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2817023 = {S80226, 140, { &P21387,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A360 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2365409 = {S80227, 139, { &P21540,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2830482 = {S80228, 130, { &P21540,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S12 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2403243 = {S80229, 158, { &P21033,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U1 Build/HCL ME Tablet U1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2424191 = {S80230, 33, { &P15988,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SAMSUNG-GT-C3312)*/
const Device UA2470291 = {S80231, 158, { &P12280,&P18432,&P21789,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1 (like Version/10A551 Mobile/6.0.2 Safari/8536.25)*/
const Device UA2470501 = {S80232, 158, { &P12280,&P18432,&P21789,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.1 (like Version/10A523 Mobile/6.0.1 Safari/8536.25)*/
const Device UA2483085 = {S80233, 143, { &P20237,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; HW-01E Build/HuaweiU9501L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2485813 = {S80234, 135, { &P14014,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; S120 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2599702 = {S80235, 93, { &P15060,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5660) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2583295 = {S80236, 127, { &P18733,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M532 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2612051 = {S80237, 148, { &P22041,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6810 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615317 = {S80238, 148, { &P22041,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6810 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615844 = {S80239, 148, { &P22041,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6810 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617375 = {S80240, 152, { &P22041,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P6810 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2708507 = {S80241, 131, { &P17835,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; NookTablet Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2719414 = {S80242, 148, { &P17835,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook Tablet Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2734717 = {S80243, 148, { &P17835,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Barnes & Noble Nook Tablet Build/JOP40G) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2755839 = {S80244, 138, { &P22929,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; LIFETAB_E10310 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2892641 = {S80245, 67, { &P22929,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; LIFETAB_E10310 Build/JRO03H)*/
const Device UA2765076 = {S80246, 128, { &P20187,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Novo7Fire Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2778929 = {S80247, 133, { &P20187,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Novo7Fire Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2772683 = {S80248, 144, { &P23157,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Smartbook U25GT-S701 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2867310 = {S80249, 132, { &P16044,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Touchpad Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2999318 = {S80250, 61, { &P16044,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; TouchPad Build/JDQ39E)*/
const Device UA2893501 = {S80251, 111, { &P18444,&P18148,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; zh-CN; NOKIA; RM-826_apac_prc_202) U2/1.0.0 UCBrowser/3.0.0.285 U2/1.0.0 Mobile*/
const Device UA2958319 = {S80252, 110, { &P18444,&P18148,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-820_nam_att_100) U2/1.0.0 UCBrowser/3.0.0.285 U2/1.0.0 Mobile*/
const Device UA3017988 = {S80253, 111, { &P18444,&P18148,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-826_im_india_201) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile*/
const Device UA2870066 = {S80254, 143, { &P23254,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Celkon A119Q Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2901223 = {S80255, 160, { &P18382,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire C Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3021504 = {S80256, 136, { &P18653,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P870 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3062996 = {S80257, 156, { &P22607,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S5303 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3049082 = {S80258, 154, { &P24728,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; PAP5044DUO Build/PrestigioPAP5044DUO) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3083373 = {S80259, 144, { &P19855,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT885 Build/9.8.3O-148_IRD-3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083156 = {S80260, 136, { &P22853,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC 802w Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083165 = {S80261, 136, { &P19917,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P765 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA2905 = {S80262, 87, { &P11870,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) HTCS620;Smartphone;320x240*/
const Device UA18808 = {S80263, 141, { &P11935,&P17483,&P3728,&P18092 }};	/*Useragent - LG/KC910i-Bouygues/V10a Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA22304 = {S80264, 160, { &P11935,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-KC910i/V10b Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA24527 = {S80265, 132, { &P11935,&P17483,&P3728,&P18092 }};	/*Useragent - LG/KC910i/V10a Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA26464 = {S80266, 133, { &P11935,&P17483,&P3728,&P18092 }};	/*Useragent - LG-KC910i-v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA53607 = {S80267, 160, { &P11935,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-KC910i/V10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA53777 = {S80268, 140, { &P11935,&P17483,&P3728,&P18092 }};	/*Useragent - LG-KC910i-Orange/V10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA62536 = {S80269, 132, { &P11935,&P17483,&P3728,&P18092 }};	/*Useragent - LG-KC910i/V10a Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA26631 = {S80270, 41, { &P13614,&P22839,&P4902,&P18092 }};	/*Useragent - ztef350/1.0 Polaris/6.1.0.0 (GUI) MMP/2.0*/
const Device UA32807 = {S80271, 55, { &P12773,&P17017,&P3807,&P18092 }};	/*Useragent - R260/P107D10/Obigo Q03C/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA172390 = {S80272, 134, { &P14657,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ko-kr; LU2300 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA205012 = {S80273, 135, { &P14657,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; LU2300 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019152 = {S80274, 59, { &P14657,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.2; LU2300 Build/FRG83G)*/
const Device UA1905909 = {S80275, 74, { &P13815,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2.1;Zh_cn;HTC Wildfire;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA2019226 = {S80276, 123, { &P13815,&P5096,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.2.1;Zh_cn;HTC Wildfire;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA587624 = {S80277, 69, { &P15364,&P7133,&P22353,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:9.1) Gecko/X Firefox/9.1 PaleMoon/9.1*/
const Device UA2023753 = {S80278, 153, { &P15501,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E16i Build/4.0.1.A.0.283; GingerElite) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025649 = {S80279, 157, { &P15501,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E16i Build/3.0.1.A.0.145; Ginger Ultimate) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2442095 = {S80280, 144, { &P15501,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; E16i Build/3; GingerElite V3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2039730 = {S80281, 64, { &P15364,&P5690,&P18345,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:15.0) Gecko/X Firefox/15.0 slurp*/
const Device UA2040599 = {S80282, 66, { &P15364,&P5690,&P18345,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:15.0) Gecko/X Firefox/15.0.1 slurp*/
const Device UA2200234 = {S80283, 72, { &P15364,&P5690,&P18345,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:15.0) Gecko/X Firefox/15.0.1 slurp slurp*/
const Device UA2227395 = {S80284, 140, { &P12280,&P17146,&P20788,&P18092 }};	/*Useragent - MQQBrowser/37 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA2347435 = {S80285, 141, { &P12280,&P17146,&P20788,&P18092 }};	/*Useragent - MQQBrowser/37 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3*/
const Device UA2354386 = {S80286, 141, { &P12280,&P17146,&P20788,&P18092 }};	/*Useragent - MQQBrowser/371 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA2384718 = {S80287, 142, { &P12280,&P17146,&P20788,&P18092 }};	/*Useragent - MQQBrowser/371 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3*/
const Device UA2353670 = {S80288, 131, { &P21326,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TAB-831 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2383767 = {S80289, 147, { &P21861,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; Xteam 4.0 Smartpad 810c Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422232 = {S80290, 170, { &P21861,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1-SDK_V1.00_X.X release-keys; it-it; Xteam Smartpad 810c Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2982625 = {S80291, 147, { &P21861,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; Xteam 4.1 Smartpad 810c Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2991245 = {S80292, 143, { &P21861,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; Xteam Smartpad 810c Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2404141 = {S80293, 130, { &P17168,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; GT-P7310 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2488239 = {S80294, 132, { &P17168,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P7310 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2631217 = {S80295, 131, { &P17168,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7310 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2428510 = {S80296, 143, { &P21084,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; NW-Z1000Series Build/E200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781965 = {S80297, 143, { &P21084,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; NW-Z1000Series Build/E201) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2487486 = {S80298, 136, { &P23871,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; A350 TV Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2530581 = {S80299, 169, { &P12616,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; WT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2540502 = {S80300, 121, { &P21264,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; PadFone Infinity) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2554459 = {S80301, 60, { &P19973,&P5043,&P17470,&P18092 }};	/*Useragent - Nokia114/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2583083 = {S80302, 153, { &P20192,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Orange Daytona Build/HuaweiG510-0200) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2708504 = {S80303, 155, { &P20192,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI G510-0200 Build/HuaweiG510-0200) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567199 = {S80304, 135, { &P18605,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTL21 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583106 = {S80305, 137, { &P19861,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I605 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2727291 = {S80306, 148, { &P13006,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; folio100 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2579856 = {S80307, 142, { &P23472,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GOCLEVER TAB M703G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2586648 = {S80308, 145, { &P21611,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2601332 = {S80309, 165, { &P19998,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2712139 = {S80310, 167, { &P19998,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI C8812E Build/HuaweiC8812E) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2612784 = {S80311, 136, { &P15876,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7*/
const Device UA2617187 = {S80312, 136, { &P15876,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P5100 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7*/
const Device UA2662954 = {S80313, 166, { &P18496,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Incredible 4G LTE Build/IMM76D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665533 = {S80314, 140, { &P22644,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Celkon A 59 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2712607 = {S80315, 91, { &P23545,&P17017,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mauie800) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2797168 = {S80316, 91, { &P23545,&P17017,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mauie800) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2738073 = {S80317, 135, { &P18740,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; U7015 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2774134 = {S80318, 156, { &P18240,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P700 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3010659 = {S80319, 159, { &P17595,&P18120,&P22777,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.4.2.34167 (Firestorm-Release; firestorm skin) Safari/533.3*/
const Device UA2796162 = {S80320, 146, { &P22585,&P18598,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L720 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2941777 = {S80321, 133, { &P15919,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3053101 = {S80322, 133, { &P15919,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3031735 = {S80323, 148, { &P12879,&P6437,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.2; GT-P7510 Build/HTJ85B) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2953441 = {S80324, 139, { &P24557,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9128 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2993673 = {S80325, 130, { &P24557,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9128 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3074358 = {S80326, 61, { &P24557,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9128 Build/JZO54K)*/
const Device UA3023526 = {S80327, 138, { &P22590,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021231 = {S80328, 144, { &P19855,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT885 Build/9.8.3O-148_IRD-3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021490 = {S80329, 133, { &P21028,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; StarTab 715 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022519 = {S80330, 137, { &P21028,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Smartfren Tab 7 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3058069 = {S80331, 134, { &P21028,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; StarTab 715x Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3059740 = {S80332, 175, { &P13123,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5360 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3125479 = {S80333, 172, { &P13123,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi; GT-S5360 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3081392 = {S80334, 131, { &P24078,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; E3 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3130939 = {S80335, 138, { &P21627,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC first Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA4369 = {S80336, 82, { &P12638,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SAMSUNG-SGH-i780/1.0*/
const Device UA48684 = {S80337, 80, { &P12638,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-SGH-i780TIM/PAIHI4_IC3 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA55536 = {S80338, 83, { &P12638,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-SGH-i780ORANGE/PBVHK1_IB1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA63703 = {S80339, 70, { &P12638,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-SGH-i780/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA32097 = {S80340, 45, { &P13369,&P17484,&P4977,&P18092 }};	/*Useragent - Samsung-SPHM350 Access-NetFront/3.1.5 MMP/2.0*/
const Device UA62381 = {S80341, 79, { &P14402,&P17484,&P4970,&P18092 }};	/*Useragent - Samsung-SPHM520 AU-MIC-M520/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62711 = {S80342, 14, { &P12115,&P17483,&P17660,&P18092 }};	/*Useragent - LG/KU990i/v1.0*/
const Device UA1966898 = {S80343, 139, { &P23919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; R800i Build/4.1.H.0.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2481564 = {S80344, 154, { &P23919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SonyEricssonR800i Build/4.0.2.A.0.62) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2845573 = {S80345, 142, { &P23919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; R800i Build/4.0.2.A.0.62) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA746626 = {S80346, 150, { &P15390,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P925 Build/GRJ90; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2816988 = {S80347, 165, { &P15390,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P925 Build/GRJ90; CyanogenMod 7.2+ v1.6.0 P925) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA774512 = {S80348, 138, { &P13251,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Dell Streak Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1951705 = {S80349, 70, { &P20380,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-R1/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA1985308 = {S80350, 135, { &P22011,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; APPTB701BV2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2235200 = {S80351, 170, { &P13815,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2261602 = {S80352, 138, { &P18747,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A840S Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2352124 = {S80353, 75, { &P17595,&P18120,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10A406e (Apple TV; U; CPU OS 6_0 like Mac OS X; en_us)*/
const Device UA2457936 = {S80354, 76, { &P17595,&P18120,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10A831 (Apple TV; U; CPU OS 6_0_1 like Mac OS X; en_us)*/
const Device UA2668432 = {S80355, 77, { &P17595,&P18120,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10B329a (Apple TV; U; CPU OS 6_1_3 like Mac OS X; en_us)*/
const Device UA2933312 = {S80356, 75, { &P17595,&P18120,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.9B830 (Apple?TV; U; CPU OS 5_1_1 like Mac OS X; en_us)*/
const Device UA3008739 = {S80357, 76, { &P17595,&P18120,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.10B809 (Apple TV; U; CPU OS 6_1_4 like Mac OS X; en_us)*/
const Device UA2412797 = {S80358, 70, { &P15919,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.3;Zh_cn;GT-P3100;600*976;)UCWEB7.8.0.95/139/444*/
const Device UA2448863 = {S80359, 70, { &P15919,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.4;Zh_cn;GT-P3100;600*976;)UCWEB7.8.0.95/139/444*/
const Device UA2620479 = {S80360, 119, { &P15919,&P18066,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android4.0.3;Zh_cn;GT-P3100;600*976;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA2626891 = {S80361, 119, { &P15919,&P18066,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android4.0.4;Zh_cn;GT-P3100;600*976;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA2414831 = {S80362, 138, { &P22097,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; IdeaTabS2110AF Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3043982 = {S80363, 67, { &P22097,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; IdeaTabS2110AF Build/IML74K)*/
const Device UA2424777 = {S80364, 83, { &P17976,&P6545,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.0.1; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2748191 = {S80365, 83, { &P17976,&P6545,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.0UI; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2428333 = {S80366, 135, { &P20474,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; A606 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483119 = {S80367, 139, { &P20468,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Viettel I6 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2562206 = {S80368, 160, { &P12135,&P17483,&P24676,&P18092 }};	/*Useragent - LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gd510) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2564712 = {S80369, 73, { &P15364,&P18145,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (X11; FreeBSD 9.1-RELEASE amd64) Presto/2.12.388 Version/12.14*/
const Device UA2567236 = {S80370, 137, { &P20240,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E971 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2659022 = {S80371, 137, { &P23004,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; TECNO D3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2952109 = {S80372, 61, { &P23004,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; TECNO D3 Build/GRK39F)*/
const Device UA2664142 = {S80373, 147, { &P21640,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Samsung GT-S9081 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734431 = {S80374, 147, { &P21640,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; Samsung GT-S9081 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2676091 = {S80375, 130, { &P20107,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5580C Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2699590 = {S80376, 63, { &P15364,&P7133,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; Windows NT 6.1; Trident/5.0)*/
const Device UA2705108 = {S80377, 47, { &P15364,&P18580,&P18589,&P18092 }};	/*Useragent - Skype/4.6.0.306 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2768258 = {S80378, 144, { &P20837,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IRIS_501 Build/LAVAIRIS501) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2770877 = {S80379, 134, { &P23179,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ZP980 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2770908 = {S80380, 139, { &P15972,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One XL Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2872870 = {S80381, 139, { &P15972,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One XL Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2966210 = {S80382, 159, { &P15047,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SK17i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2975785 = {S80383, 152, { &P22964,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021735 = {S80384, 159, { &P22964,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL_ONE_TOUCH_6010X_Orange Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975807 = {S80385, 133, { &P21472,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A111 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3083288 = {S80386, 131, { &P23177,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TAB-840_G Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3079233 = {S80387, 132, { &P21261,&P17979,&P18112,&P18092 }};	/*Useragent - samsung-GT-I9500/1.0 Linux/2.6.35.7 Android/4.1.2 Release/07.18.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3079400 = {S80388, 94, { &P12088,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaX6-00) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA8510 = {S80389, 64, { &P14534,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GR501/V10c Obigo/Q7.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA12797 = {S80390, 48, { &P17051,&P17484,&P5001,&P18092 }};	/*Useragent - SCH-U750P/1.0 NetFront/3.0.22.2.17 (GUI) MMP/2.0*/
const Device UA20430 = {S80391, 48, { &P15364,&P17017,&P3534,&P18092 }};	/*Useragent - Mozilla/4.1 (compatible; Teleca/Q05A/GS-CX10000)*/
const Device UA21743 = {S80392, 164, { &P15364,&P17017,&P3534,&P18092 }};	/*Useragent - MTC/1.0/0MTC 733/B100 Browser/Obigo-Browser/Q05A MMS/Obigo-MMS/Q05A SyncML/HW-SyncML/1.0 Java/HWJa/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Player/QTV-Player/5.3*/
const Device UA33439 = {S80393, 85, { &P15364,&P17017,&P3534,&P18092 }};	/*Useragent - Panama-orange/Obigo Browser/Q05A Browser/Q03C MMS/1.0/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53896 = {S80394, 76, { &P15364,&P17017,&P3534,&P18092 }};	/*Useragent - TCL-Q3/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA30335 = {S80395, 85, { &P12300,&P5448,&P22270,&P18092 }};	/*Useragent - HTC-P4600/%ROM_VERSION% Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA30336 = {S80396, 82, { &P12300,&P5448,&P22270,&P18092 }};	/*Useragent - HTC-P4600/5.11.502.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA30337 = {S80397, 75, { &P12300,&P5448,&P22270,&P18092 }};	/*Useragent - HTC-P4600/1.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA33182 = {S80398, 138, { &P12880,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; ZTE_GV821 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA50326 = {S80399, 144, { &P13668,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; HTC Dream G1 Build/DRD35) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA52801 = {S80400, 141, { &P13668,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-sg; HTC Dream Build/DRD20) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA67080 = {S80401, 141, { &P13668,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; HTC Dream Build/DRC83) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA309791 = {S80402, 131, { &P18047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Z71 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA730786 = {S80403, 131, { &P18047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Z71 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019107 = {S80404, 131, { &P18047,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Z71 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA452462 = {S80405, 116, { &P12473,&P5204,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2730c-1/10.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA763031 = {S80406, 76, { &P15364,&P16110,&P22339,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; rv:9.0.1) Gecko/X Firefox/9.0.1 CometBird/9.0.1*/
const Device UA866986 = {S80407, 133, { &P21203,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SX-SP715A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2540553 = {S80408, 83, { &P21203,&P18066,&P18062,&P18092 }};	/*Useragent - Logos/2.2.8.1 (Linux; U; Android 4.0.3; en-US; SX-SP715A Build/ALPHA2 4.0.3 X.O A1)*/
const Device UA2540614 = {S80409, 62, { &P21203,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; SX-SP715A Build/IML74K)*/
const Device UA2540604 = {S80410, 131, { &P21203,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; SX-SP715A Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA892545 = {S80411, 153, { &P17835,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NookTablet Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1983736 = {S80412, 149, { &P17835,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NookTablet Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2232828 = {S80413, 171, { &P16436,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2030796 = {S80414, 135, { &P21992,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.6; A9550 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2383415 = {S80415, 185, { &P12258,&P17483,&P24672,&P18092 }};	/*Useragent - LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-gs290) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2354824 = {S80416, 156, { &P18474,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-I9210T Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2419913 = {S80417, 156, { &P18474,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; GT-I9210T Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2391366 = {S80418, 98, { &P15547,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-c6712) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2363720 = {S80419, 130, { &P20256,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TAB275 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2431262 = {S80420, 37, { &P18218,&P15924,&P20765,&P18092 }};	/*Useragent - NOKIA-Lumia 610/UCWEB2.7.0.223/47/355*/
const Device UA2459561 = {S80421, 133, { &P21887,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAD-97052 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2469507 = {S80422, 133, { &P21887,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAC-97052 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2461246 = {S80423, 95, { &P15364,&P18110,&P20915,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) rekonq/2.0 Safari/534.34*/
const Device UA2506487 = {S80424, 133, { &P21890,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; TAD-10021 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2527499 = {S80425, 157, { &P15079,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Pro Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2529894 = {S80426, 155, { &P19999,&P18066,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Karbonn A30 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2530360 = {S80427, 163, { &P13452,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2633727 = {S80428, 165, { &P13452,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444*/
const Device UA2634820 = {S80429, 164, { &P13452,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2553107 = {S80430, 56, { &P18088,&P22828,&P18246,&P18092 }};	/*Useragent - ZEN P8 /Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA2695379 = {S80431, 57, { &P18088,&P22828,&P18246,&P18092 }};	/*Useragent - ZEN P8i /Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA2559290 = {S80432, 71, { &P22616,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X396/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2670747 = {S80433, 160, { &P18233,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL_one_touch_995 Build/IceCreamSandwich) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2752242 = {S80434, 96, { &P12329,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN86 8MP) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2766630 = {S80435, 139, { &P23294,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; ZTE N909 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3073897 = {S80436, 130, { &P23294,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; ZTE N909 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2768163 = {S80437, 149, { &P23103,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Starmobile DIAMOND Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3014802 = {S80438, 152, { &P23103,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; Starmobile DIAMOND V3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2770948 = {S80439, 142, { &P22395,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6606 Build/10.1.1.B.0.166) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774001 = {S80440, 136, { &P16263,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; dtab01 Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2799401 = {S80441, 146, { &P16263,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 10 LINK Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2782019 = {S80442, 135, { &P23033,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Spice Mi-725 Build/Mi725) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2784768 = {S80443, 144, { &P18512,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT881 Build/6.7.3-130_YUS-11) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2891524 = {S80444, 143, { &P20143,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Edison 3G Build/2.0.0 X-17:50) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3023688 = {S80445, 145, { &P20143,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Edison 2 3G Build/1.0.1_X-10:06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3062119 = {S80446, 145, { &P20143,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Edison 2 3G Build/1.0.2_X-08:25) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2974805 = {S80447, 137, { &P20240,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Optimus G Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3060938 = {S80448, 93, { &P15649,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3080665 = {S80449, 94, { &P15649,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570I) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3080831 = {S80450, 154, { &P20541,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAP4500TDUO Build/PrestigioPAP4500TDUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3131179 = {S80451, 132, { &P22754,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HP Slate 7 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA2753424 = {S80452, 73, { &P13558,&P7063,&P3606,&P18092 }};	/*Useragent - BlackBerry7290/4.1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/104*/
const Device UA14001 = {S80453, 61, { &P11783,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia7270/2.0 (03.24) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA16239 = {S80454, 67, { &P11866,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia3208c_CMCC/2.0 (09.25) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA16240 = {S80455, 62, { &P11866,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia3208c/2.0 (10.50) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA34453 = {S80456, 79, { &P11866,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia3208c/2.0 (10.50) Profile/MIDP-2.1 Configuration/CLDC-1.1 Nokia3208c/10.50*/
const Device UA26456 = {S80457, 84, { &P13597,&P17603,&P3534,&P18092 }};	/*Useragent - PANTECH-C520/IQUS04212008 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA26459 = {S80458, 84, { &P13597,&P17603,&P3534,&P18092 }};	/*Useragent - PANTECH-C520/IQUS04132009 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA69174 = {S80459, 35, { &P11929,&P5129,&P4779,&P18092 }};	/*Useragent - Nokia5230/UC Browser7.8.0.95/50/355*/
const Device UA135885 = {S80460, 54, { &P17568,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SH3F UP.Browser/6.2_7.2.7.1.K.4.306 (GUI) MMP/2.0*/
const Device UA567638 = {S80461, 62, { &P13077,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.3;Zh_cn;C8650;320*480;)UCWEB7.6.0.75/139/32046*/
const Device UA575152 = {S80462, 80, { &P17595,&P22328,&P17470,&P18092 }};	/*Useragent - WebProcess/7534.51.22 CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookPro8%2C1)*/
const Device UA1904862 = {S80463, 80, { &P17595,&P22328,&P17470,&P18092 }};	/*Useragent - WebProcess/7534.51.22 CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookAir4%2C2)*/
const Device UA1936517 = {S80464, 74, { &P17595,&P22328,&P17470,&P18092 }};	/*Useragent - WebProcess/7534.52.7 CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (iMac12%2C1)*/
const Device UA1939338 = {S80465, 87, { &P17595,&P22328,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader/1725 CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookPro8%2C1)*/
const Device UA2639686 = {S80466, 82, { &P17595,&P22328,&P17470,&P18092 }};	/*Useragent - ocspd (unknown version) CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookPro8%2C1)*/
const Device UA2653849 = {S80467, 89, { &P17595,&P22328,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookPro8%2C1)*/
const Device UA2720580 = {S80468, 92, { &P17595,&P22328,&P17470,&P18092 }};	/*Useragent - XProtectUpdater (unknown version) CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookPro8%2C1)*/
const Device UA597184 = {S80469, 62, { &P15364,&P5690,&P18329,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko/X Thunderbird/11.0*/
const Device UA1990919 = {S80470, 71, { &P15364,&P5690,&P18329,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:11.0) Gecko/X Thunderbird/11.0.1*/
const Device UA725576 = {S80471, 133, { &P22123,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; 7DTB2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1954073 = {S80472, 146, { &P21722,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; STARTEXT II Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, comme Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1956745 = {S80473, 50, { &P15364,&P17017,&P23791,&P18092 }};	/*Useragent - System Center 2012 - Operations Manager 7.0.8560.0*/
const Device UA2579408 = {S80474, 50, { &P15364,&P17017,&P23791,&P18092 }};	/*Useragent - System Center 2012 - Operations Manager 7.0.9538.0*/
const Device UA2242408 = {S80475, 71, { &P17595,&P17296,&P21578,&P18092 }};	/*Useragent - Mac OS X/10.6.8 (10K549); ExchangeWebServices/1.3 (61); Mail/4.6 (1085)*/
const Device UA2342615 = {S80476, 182, { &P16074,&P5768,&P24671,&P18092 }};	/*Useragent - Nokia3110c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia3110c) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2340724 = {S80477, 151, { &P12798,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-gb; SonyEricssonLT18i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2759800 = {S80478, 139, { &P12798,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-gb; LT18i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2421961 = {S80479, 71, { &P20055,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X231/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2468728 = {S80480, 150, { &P17835,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Barnes & Noble Nook Tablet Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2507600 = {S80481, 134, { &P17835,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; NookTablet Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2729091 = {S80482, 132, { &P17835,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; NookTablet Build/JOP40C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2531115 = {S80483, 168, { &P12798,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT18i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2597449 = {S80484, 63, { &P15364,&P5690,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; Windows NT 5.2; Trident/3.0)*/
const Device UA2851187 = {S80485, 63, { &P15364,&P5690,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; Windows NT 5.1; Trident/4.0)*/
const Device UA2659307 = {S80486, 135, { &P21301,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; M031 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2662489 = {S80487, 92, { &P14967,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; LG-P500) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2675380 = {S80488, 138, { &P20517,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Cink Peax Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2767760 = {S80489, 138, { &P20517,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Cink Peax Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3109118 = {S80490, 140, { &P20517,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; CINK PEAX 2 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2703290 = {S80491, 142, { &P20731,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Blaze S180  Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2754275 = {S80492, 126, { &P20556,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A701 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2783938 = {S80493, 129, { &P22943,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; E270BSA Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2892668 = {S80494, 145, { &P23712,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; th-th; i-mobile IQ XA Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3023359 = {S80495, 129, { &P21772,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; D2-721G Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3064344 = {S80496, 158, { &P18223,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT22i Build/6.2.A.1.100) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3108069 = {S80497, 158, { &P18223,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT22i Build/6.2.A.0.400) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3027947 = {S80498, 137, { &P24595,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; fnac 5 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3093822 = {S80499, 153, { &P20018,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993D Build/ICECREAM) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109299 = {S80500, 152, { &P20018,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993 Build/ICECREAM) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083184 = {S80501, 134, { &P23150,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Q1000 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA53916 = {S80502, 75, { &P13103,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-K1/08.02.81R MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/1.0*/
const Device UA26657 = {S80503, 85, { &P12638,&P5448,&P3772,&P18092 }};	/*Useragent - SAMSUNG-SGH-i780ORANGE/PBVIC2 (compatible; MSIE 6.0; Windows CE; PPC) Opera 8.65 [fr]*/
const Device UA63686 = {S80504, 71, { &P12638,&P5448,&P3772,&P18092 }};	/*Useragent - SAMSUNG-SGH-i780/1.0 (compatible; MSIE 6.0; Windows CE; PPC) Opera 8.65*/
const Device UA87867 = {S80505, 74, { &P12638,&P5448,&P3772,&P18092 }};	/*Useragent - SAMSUNG-SGH-i780V/MJHA3 (compatible; MSIE 6.0; Windows CE; PPC) Opera 8.65*/
const Device UA14116 = {S80506, 67, { &P11790,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia3610a_CMCC/2.0 (03.56) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA14119 = {S80507, 62, { &P11790,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia3610a/2.0 (03.56) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA18547 = {S80508, 107, { &P14516,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-Z400-Vodafone/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA63430 = {S80509, 98, { &P14516,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-Z400/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA63804 = {S80510, 105, { &P11975,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonT707a/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3*/
const Device UA2223358 = {S80511, 56, { &P17643,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SN3T UP.Browser/6.2_7.2.7.1.K.6.1.102 (GUI) MMP/2.0*/
const Device UA305549 = {S80512, 12, { &P16620,&P22784,&P17470,&P18092 }};	/*Useragent - MicromaxX510*/
const Device UA420959 = {S80513, 131, { &P20500,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Cosmo Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA687308 = {S80514, 131, { &P20500,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ru-ru; Cosmo Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2440254 = {S80515, 134, { &P20500,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ru-ru; Cosmo Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA506285 = {S80516, 154, { &P21601,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; us-us; POV-Mobii-7 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1855979 = {S80517, 154, { &P21601,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; us-us; POV Mobii 7 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019096 = {S80518, 154, { &P21601,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.1; es-es; POV-Mobii-7 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019237 = {S80519, 150, { &P21601,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.3; es-es; MERM7UH Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA797058 = {S80520, 77, { &P15364,&P18140,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux x86_64; U; Linux Mint; en) Presto/2.2.15 Version/10.10*/
const Device UA704781 = {S80521, 72, { &P15364,&P17017,&P22315,&P18092 }};	/*Useragent - Mozilla/5.0 (BeOS; U; BeOS BePC; en-US; rv:1.9a1) Gecko/X SeaMonkey/1.5a*/
const Device UA833528 = {S80522, 42, { &P15364,&P21445,&P18098,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows ME; U; en) Opera 8.51*/
const Device UA2684209 = {S80523, 61, { &P15364,&P21445,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows ME; ru) Opera 8.50*/
const Device UA857065 = {S80524, 142, { &P19977,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; IM-A775C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995660 = {S80525, 37, { &P18218,&P15924,&P24630,&P18092 }};	/*Useragent - NOKIA-Lumia 610/UCWEB2.2.0.157/47/355*/
const Device UA2354816 = {S80526, 166, { &P18442,&P18111,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-N7100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2697181 = {S80527, 166, { &P18442,&P18111,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-N7100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2400753 = {S80528, 150, { &P20887,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1811_v75_jbaol1_g88 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2549041 = {S80529, 144, { &P20887,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1811_v75_hjy1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2652546 = {S80530, 147, { &P20887,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1811_v75_jyuany1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2833935 = {S80531, 150, { &P20887,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; e1811_v75_jbaol1_g58 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2402367 = {S80532, 64, { &P20120,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; A45; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2404069 = {S80533, 141, { &P20540,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PAP4300 DUO Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2422531 = {S80534, 176, { &P15767,&P18117,&P22305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_1 like Mac OS X; zh-cn Model:iPad3,3) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7543.48.3 UCBrowser/1.0.0.192/43/355*/
const Device UA2467246 = {S80535, 74, { &P19862,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; Micromax_A110; 480*854) UCWEB7.9.0.94/139/444*/
const Device UA2459602 = {S80536, 129, { &P22100,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M1070 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2483750 = {S80537, 147, { &P15919,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622875 = {S80538, 154, { &P15919,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2623220 = {S80539, 145, { &P15919,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2637334 = {S80540, 147, { &P15919,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2637797 = {S80541, 148, { &P15919,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3100B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2638032 = {S80542, 145, { &P15919,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2564380 = {S80543, 156, { &P16074,&P5768,&P24676,&P18092 }};	/*Useragent - Nokia3110c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3110c) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2527260 = {S80544, 194, { &P11787,&P5204,&P24672,&P18092 }};	/*Useragent - Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia3120classic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2534134 = {S80545, 158, { &P16356,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2535833 = {S80546, 138, { &P23434,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Onyx Athena 10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2537448 = {S80547, 139, { &P22162,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AUXUS CoreX4 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2603254 = {S80548, 156, { &P19853,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-N7102 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2993770 = {S80549, 156, { &P19853,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7102 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2599679 = {S80550, 146, { &P21891,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; WALTON Primo_G1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658853 = {S80551, 139, { &P21891,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Primo_G1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2669433 = {S80552, 128, { &P22521,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; WF7C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2678418 = {S80553, 100, { &P15364,&P7133,&P22718,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; XBLWP7; ZuneWP7) UCBrowser/2.9.0.263*/
const Device UA2713445 = {S80554, 103, { &P15798,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC Explorer A310e) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2800211 = {S80555, 103, { &P15798,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC Explorer A310e) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2715886 = {S80556, 169, { &P20729,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A27 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2753219 = {S80557, 137, { &P18747,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A840S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892971 = {S80558, 138, { &P18747,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A840SP Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754333 = {S80559, 138, { &P20174,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CINK KING Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754638 = {S80560, 138, { &P22729,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT918 Build/2_26D_2009) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2781512 = {S80561, 152, { &P19748,&P5043,&P24677,&P18092 }};	/*Useragent - Nokia205/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia205) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2769953 = {S80562, 134, { &P15877,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT910 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2866639 = {S80563, 137, { &P23165,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; H7500+ Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2896600 = {S80564, 157, { &P18450,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST23i Build/11.0.A.5.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2900601 = {S80565, 214, { &P24157,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; sprd-iNote-mini/1.0 Android/2.3.5 Release/05.18.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3028285 = {S80566, 190, { &P22606,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N5110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us*/
const Device UA3037943 = {S80567, 150, { &P18437,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3022516 = {S80568, 138, { &P21507,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C1605 Build/11.3.A.2.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3025424 = {S80569, 144, { &P18635,&P21637,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3062763 = {S80570, 144, { &P18635,&P21637,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3027502 = {S80571, 156, { &P19847,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S6012 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3081081 = {S80572, 145, { &P20558,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ608 Build/7.7.1-141-7-FLEM-UMTS-LA) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3114303 = {S80573, 143, { &P20558,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ608-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA62431 = {S80574, 101, { &P14741,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-M610/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA49514 = {S80575, 115, { &P13044,&P22283,&P4223,&P18092 }};	/*Useragent - MOT-EX130/1.0 REXL4/1.0 BrewMP/1.0.3 Release/6.15.2011 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA60810 = {S80576, 65, { &P12577,&P17483,&P17602,&P18092 }};	/*Useragent - LG-A200/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA76184 = {S80577, 134, { &P17160,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; en-us; Slider SL101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA78161 = {S80578, 134, { &P17160,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; de-de; Slider SL101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA629672 = {S80579, 134, { &P17160,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; zh-tw; Slider SL101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA697153 = {S80580, 134, { &P17160,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; en-gb; Slider SL101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA2145357 = {S80581, 135, { &P17160,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; zh-tw; Slider SL101 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA2149979 = {S80582, 134, { &P17160,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; zh-tw; Slider SL101 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA2218200 = {S80583, 54, { &P17577,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3S UP.Browser/6.2_7.2.7.1.K.5.212 (GUI) MMP/2.0*/
const Device UA336684 = {S80584, 137, { &P17108,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Inspire 4G Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA576621 = {S80585, 141, { &P17108,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Inspire 4G Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA787403 = {S80586, 141, { &P17108,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Inspire 4G Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA354043 = {S80587, 156, { &P15438,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Sensation Build/GRJ22; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA575798 = {S80588, 156, { &P15438,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Sensation Build/GRJ22; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA573829 = {S80589, 41, { &P15364,&P17017,&P17338,&P18092 }};	/*Useragent - mstar platform/UC Browser7.9.0.102/69/352*/
const Device UA736051 = {S80590, 32, { &P15364,&P17017,&P17338,&P18092 }};	/*Useragent - opera/UC Browser7.9.0.102/69/352*/
const Device UA1904820 = {S80591, 32, { &P15364,&P17017,&P17338,&P18092 }};	/*Useragent - mstar/UC Browser7.9.0.102/69/352*/
const Device UA1951226 = {S80592, 43, { &P15364,&P17017,&P17338,&P18092 }};	/*Useragent - r620/UC Browser7.9.0.102/69/352 TRUSTED/1.0*/
const Device UA2481114 = {S80593, 66, { &P15517,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; MT15i; 480*854) UCWEB7.9.0.94/139/444*/
const Device UA2594812 = {S80594, 66, { &P15517,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; MT15i; 480*854) UCWEB7.9.0.94/139/444*/
const Device UA894960 = {S80595, 81, { &P13552,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-in; NokiaC6-00)/UC Browser8.4.0.159/50/352/UCWEB Mobile*/
const Device UA2221328 = {S80596, 32, { &P13552,&P5129,&P24670,&P18092 }};	/*Useragent - NokiaC6-00/UCWEB8.4.0.150/50/800*/
const Device UA2375452 = {S80597, 89, { &P13552,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiac6-00) UCBrowser8.4.0.159/69/352/UCWEB Mobile  3gpp-gba*/
const Device UA895075 = {S80598, 81, { &P12088,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaX6-00)/UC Browser8.4.0.159/50/352/UCWEB Mobile*/
const Device UA2025662 = {S80599, 145, { &P21250,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; NGM Vanity Smart Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2258608 = {S80600, 169, { &P15397,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; ST18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2261657 = {S80601, 140, { &P17108,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Inspire 4G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2419431 = {S80602, 140, { &P17108,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Inspire 4G Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2032336 = {S80603, 132, { &P23827,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; W806 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033593 = {S80604, 125, { &P23827,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; W806 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2840196 = {S80605, 173, { &P23827,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U;AliyunOS 1.6.1.16-RT-X.X; zh-cn; W806 Build/AliyunOs-2012;Android 2.3 Compatible) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2400783 = {S80606, 140, { &P12108,&P5843,&P24670,&P18092 }};	/*Useragent - Nokia7230/5.0 (10.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia7230) UCBrowser8.4.0.159/70/444/UCWEB Mobile*/
const Device UA2383573 = {S80607, 120, { &P11964,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE75-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile*/
const Device UA2352085 = {S80608, 151, { &P21294,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; MODECOM FreeTAB 9702 IPS X2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2426969 = {S80609, 159, { &P23974,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Telenor_One_Touch_S Build/IceCreamSandwich) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2455635 = {S80610, 155, { &P15649,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-S5570 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2551114 = {S80611, 155, { &P15649,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-S5570 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2465234 = {S80612, 79, { &P15364,&P16110,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 6.2; Microsoft Office Outlook 12.0.6662; Pro)*/
const Device UA2641967 = {S80613, 79, { &P15364,&P16110,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 6.2; Microsoft Office Outlook 12.0.4518; Pro)*/
const Device UA2811374 = {S80614, 79, { &P15364,&P16110,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 6.2; Microsoft Office Outlook 12.0.6607; Pro)*/
const Device UA3095514 = {S80615, 79, { &P15364,&P16110,&P18569,&P18092 }};	/*Useragent - Microsoft Office/12.0 (Windows NT 6.2; Microsoft Office Outlook 12.0.6683; Pro)*/
const Device UA2599687 = {S80616, 93, { &P15118,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; NokiaX7-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2584865 = {S80617, 141, { &P20183,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MI-ONE Plus Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567391 = {S80618, 131, { &P19879,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I915 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2596844 = {S80619, 126, { &P15364,&P7133,&P22780,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth Pro/7.0.2.8415 Safari/532.4*/
const Device UA2602607 = {S80620, 140, { &P15047,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SK17i Build/4.1.B.0.587) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2855291 = {S80621, 140, { &P15047,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SK17a Build/4.1.B.0.431) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668033 = {S80622, 141, { &P23291,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; POV_TV-HDMI-200BT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2675694 = {S80623, 131, { &P20145,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; NS-13T001 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2713650 = {S80624, 165, { &P21351,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HUAWEI C8813 Build/HuaweiC8813) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2881213 = {S80625, 140, { &P12280,&P19826,&P24445,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Janetter/1.5.2*/
const Device UA2937719 = {S80626, 140, { &P12280,&P19826,&P24445,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3 Janetter/1.5.2*/
const Device UA2870579 = {S80627, 98, { &P19862,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Micromax A110) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2755425 = {S80628, 136, { &P19917,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P765 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773859 = {S80629, 145, { &P20558,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ608 Build/7.7.1-141-7-FLEM-UMTS-LA) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2783458 = {S80630, 143, { &P20558,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ608-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2851336 = {S80631, 157, { &P18021,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A770K Build/JZO54K; CyanogenMod-CM10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049767 = {S80632, 167, { &P18021,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A770K Build/update by fengdian; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121644 = {S80633, 162, { &P18021,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A770K Build/->Wangsai_008; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2784716 = {S80634, 137, { &P12496,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I897 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2788699 = {S80635, 151, { &P13215,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-I5700 Build/GWK74; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2869809 = {S80636, 140, { &P23259,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAC-70061 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3024168 = {S80637, 172, { &P22594,&P18598,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-M919 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SGH-M919, 52, 1.9.9.2)*/
const Device UA3031489 = {S80638, 147, { &P17926,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3032069 = {S80639, 157, { &P22584,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337M Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3021815 = {S80640, 134, { &P23193,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCL21 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021726 = {S80641, 138, { &P22756,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C1604 Build/11.3.A.2.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2970227 = {S80642, 135, { &P24075,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Q600 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3017314 = {S80643, 91, { &P12004,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian95) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3079808 = {S80644, 92, { &P12004,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN95) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3080627 = {S80645, 96, { &P12004,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN95 8GB) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3096320 = {S80646, 153, { &P18635,&P21637,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-US; Nexus 4 Build/JWR66Y) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA18864 = {S80647, 133, { &P11840,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-MOTORAZRV9/9C.14.36R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 14003276) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA18871 = {S80648, 134, { &P11840,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-MOTORAZRV9/9C.14.1CR8 BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 14003262) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA18876 = {S80649, 133, { &P11840,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-MOTORAZRV9/9C.13.43R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 14183245) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA39304 = {S80650, 133, { &P11840,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-MOTORAZRV9/9C.13.37R3 BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 14183237) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [en]*/
const Device UA1879783 = {S80651, 126, { &P11840,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-MOTORAZRV9/9C.13.37R3 BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; X) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [en]*/
const Device UA8726 = {S80652, 121, { &P13498,&P5448,&P4536,&P18092 }};	/*Useragent - SAMSUNG-GT-B7350Orange/BVJI1 Opera/9.80 (Windows Mobile; Windows CE; Opera Mobi/ORS-75XXX; U) Presto/2,4,13 Version/10.00*/
const Device UA10410 = {S80653, 113, { &P13498,&P5448,&P4536,&P18092 }};	/*Useragent - SAMSUNG-GT-B7350/1.0 Opera/9.80 (Windows Mobile; Windows CE; Opera Mobi/ORS-75XXX; U) Presto/2,4,13 Version/10.00*/
const Device UA38497 = {S80654, 118, { &P13452,&P5096,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.2;Zh_cn;GT-P1000;600*1024;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355*/
const Device UA560581 = {S80655, 69, { &P13452,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2;Zh_cn;GT-P1000;600*1024;)UCWEB7.8.0.95/139/352*/
const Device UA2616051 = {S80656, 119, { &P13452,&P5096,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.2;Zh_cn;GT-P1000T;600*1024;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/444*/
const Device UA516563 = {S80657, 130, { &P21292,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; 003Z Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA733057 = {S80658, 132, { &P21292,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ja-jp; 003Z Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA121718 = {S80659, 113, { &P15364,&P17285,&P22441,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Iron/3.0.197.0 Safari/532.0*/
const Device UA250344 = {S80660, 122, { &P15364,&P5690,&P22411,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.1.1.0 Chrome/14.0.835.202 Safari/535.1*/
const Device UA636384 = {S80661, 143, { &P20058,&P22784,&P18373,&P18092 }};	/*Useragent - MicromaxX660/   Browser/Opera profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA753131 = {S80662, 142, { &P15483,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7 HTC_ChaCha-orange-LS; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA1995265 = {S80663, 142, { &P15767,&P17090,&P20776,&P18092 }};	/*Useragent - MQQBrowser/2.5.1 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3*/
const Device UA2007375 = {S80664, 142, { &P15767,&P17090,&P20776,&P18092 }};	/*Useragent - MQQBrowser/2.5.1 Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3*/
const Device UA2327759 = {S80665, 140, { &P15767,&P17090,&P20776,&P18092 }};	/*Useragent - MQQBrowser/2.5 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 Safari/7534.48.3*/
const Device UA902254 = {S80666, 169, { &P12616,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; WT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA1966847 = {S80667, 39, { &P20029,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX274/Q03C MAUI-browser/CLDC-1.1*/
const Device UA1973237 = {S80668, 139, { &P19848,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M440S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2366325 = {S80669, 150, { &P13337,&P17485,&P24670,&P18092 }};	/*Useragent - SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-in; sonyericssonj108i) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2362290 = {S80670, 83, { &P23535,&P17979,&P18062,&P18092 }};	/*Useragent - LenovoS899t_TD/1.0 Android/4.0 Release/02.01.2012 Browser/WAP2.0 appleWebkit/534.30*/
const Device UA2422227 = {S80671, 134, { &P23928,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; BOULM727MC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2424424 = {S80672, 127, { &P24510,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N10 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2489174 = {S80673, 99, { &P15547,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-c6712) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2540017 = {S80674, 138, { &P18458,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I200 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2562546 = {S80675, 93, { &P18214,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S7500) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2652882 = {S80676, 101, { &P14880,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.78; en-us; 8320/2.7.0.78) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2951280 = {S80677, 103, { &P14880,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.105; en-us; 8320/2.7.0.105) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2983636 = {S80678, 103, { &P14880,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.106; en-us; 8320/2.7.0.106) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3043192 = {S80679, 103, { &P14880,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.110; en-us; 8320/2.7.0.110) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA3100562 = {S80680, 101, { &P14880,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.92; en-us; 8320/2.7.0.92) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2666117 = {S80681, 175, { &P22998,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; TECNO_N7 Build/TECNOTECNO_N7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/33567*/
const Device UA2672678 = {S80682, 139, { &P15397,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST18i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676743 = {S80683, 126, { &P19991,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A511 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676833 = {S80684, 136, { &P21022,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E615 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735014 = {S80685, 137, { &P21022,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E615f Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677004 = {S80686, 134, { &P18605,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTL21 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713679 = {S80687, 127, { &P18605,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTL21 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2698844 = {S80688, 162, { &P11646,&P17485,&P24676,&P18092 }};	/*Useragent - SonyEricssonW595/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw595) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2799257 = {S80689, 93, { &P18513,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6802) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2749686 = {S80690, 136, { &P20184,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; X506S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2753007 = {S80691, 134, { &P20480,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; JY-G3 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2938031 = {S80692, 135, { &P20480,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; JY-G3N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2766317 = {S80693, 152, { &P21857,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24 TT/4.1.1*/
const Device UA2769902 = {S80694, 136, { &P22859,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; IQ445 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078573 = {S80695, 140, { &P22859,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; Fly_IQ445 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2852970 = {S80696, 74, { &P23525,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-S3800W/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA3023118 = {S80697, 145, { &P18300,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973673 = {S80698, 139, { &P20439,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SOL21 Build/9.1.D.0.401) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2970994 = {S80699, 94, { &P11964,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE75-1) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA2995043 = {S80700, 215, { &P15364,&P24371,&P24415,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.4; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.4 Release/04.11.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2998082 = {S80701, 215, { &P15364,&P24371,&P24415,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.4; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.4 Release/04.19.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3068860 = {S80702, 215, { &P15364,&P24371,&P24415,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.4; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.4 Release/04.02.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3068863 = {S80703, 125, { &P15364,&P24371,&P24415,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.4; en-us; hsdroid) antivirusHttpClient/2.0 (KHTML, like Gecko) Version/Version 2.3/antivirus*/
const Device UA3068861 = {S80704, 215, { &P15364,&P24371,&P24415,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.4; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.4 Release/05.13.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3068869 = {S80705, 215, { &P15364,&P24371,&P24415,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.4; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.4 Release/04.26.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3068868 = {S80706, 215, { &P15364,&P24371,&P24415,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.4; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.4 Release/05.06.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3068872 = {S80707, 215, { &P15364,&P24371,&P24415,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.4; zh-cn; sprd-SPHS-on-Hsdroid/1.0 Android/4.4 Release/06.03.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3068871 = {S80708, 205, { &P15364,&P24371,&P24415,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.4; zh-cn; sprd-N3L-U/1.0 Android/4.4 Release/04.23.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2999210 = {S80709, 136, { &P22948,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P716 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3108552 = {S80710, 129, { &P22948,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P716 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3019844 = {S80711, 161, { &P24636,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ORBIT 4600 Build/ORBIT-4600-V1.0-2013.06.23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3126740 = {S80712, 161, { &P24636,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ORBIT 4600 Build/ORBIT-4600-V1.0-2013.08.13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3027796 = {S80713, 159, { &P19861,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I605 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i605*/
const Device UA3064021 = {S80714, 159, { &P19861,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I605 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i605*/
const Device UA3128086 = {S80715, 159, { &P19861,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I605 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i605*/
const Device UA3034250 = {S80716, 163, { &P22585,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L720 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltespr*/
const Device UA3063524 = {S80717, 163, { &P22585,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L720 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltespr*/
const Device UA3127914 = {S80718, 163, { &P22585,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L720 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltespr*/
const Device UA3059470 = {S80719, 147, { &P21948,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; MID0714PGE01.133 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3083364 = {S80720, 137, { &P18595,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I8190 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093879 = {S80721, 138, { &P16044,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cnm TOUCHPAD 9.7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126264 = {S80722, 146, { &P16044,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Versus TouchPad 9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126733 = {S80723, 143, { &P16044,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CnM TouchPad 9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126917 = {S80724, 146, { &P16044,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Versus TouchPad 7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA38124 = {S80725, 119, { &P14877,&P5629,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.3.3;Zh_cn;GT-I9000;480*800;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA1904768 = {S80726, 70, { &P14877,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.3;Zh_cn;GT-I9000;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA61701 = {S80727, 40, { &P14314,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GT360 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA63405 = {S80728, 122, { &P14513,&P17483,&P3459,&P18092 }};	/*Useragent - LG-P525/V100 Obigo/Q7.3 MMS/LG-MMS-V1.1/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63485 = {S80729, 54, { &P12274,&P5049,&P3430,&P18092 }};	/*Useragent - SAMSUNG-SGH-i900/1.0 (Windows NT 5.1; U; en) Opera 9.5*/
const Device UA68693 = {S80730, 54, { &P17578,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-HI3H UP.Browser/6.2_7.2.7.1.K.5.207 (GUI) MMP/2.0*/
const Device UA68743 = {S80731, 82, { &P15207,&P5448,&P4777,&P18092 }};	/*Useragent - HTC_P3400i-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA770008 = {S80732, 93, { &P15207,&P5448,&P4777,&P18092 }};	/*Useragent - HTC_P3400-Mozilla/4.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA746714 = {S80733, 116, { &P14482,&P5843,&P17433,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-01.5/06.73; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.1.21.8*/
const Device UA1959672 = {S80734, 114, { &P14482,&P5843,&P17433,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-01/07.16; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.1.21.8*/
const Device UA2359993 = {S80735, 54, { &P17569,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SH3E UP.Browser/6.2_7.2.7.1.K.3.358 (GUI) MMP/2.0*/
const Device UA199775 = {S80736, 133, { &P17595,&P17296,&P22449,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.24 (KHTML, like Gecko) Iron/11.0.700.2 Chrome/11.0.700.2 Safari/534.24*/
const Device UA316877 = {S80737, 133, { &P17595,&P17296,&P22449,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_7) AppleWebKit/534.24 (KHTML, like Gecko) Iron/11.0.700.2 Chrome/11.0.700.2 Safari/534.24*/
const Device UA2463259 = {S80738, 56, { &P17637,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SN3R UP.Browser/6.2_7.2.7.1.K.6.1.103 (GUI) MMP/2.0*/
const Device UA857085 = {S80739, 78, { &P15364,&P18145,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; FreeBSD amd64; en-US; rv:1.8.0.8) Gecko/X Firefox/1.5.0.8*/
const Device UA1972266 = {S80740, 171, { &P15991,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2660767 = {S80741, 175, { &P15991,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6102 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA628907 = {S80742, 97, { &P15364,&P5690,&P20901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.4) Gecko/X Netscape/7.1 (ax; CDonDemand-Dom)*/
const Device UA564122 = {S80743, 146, { &P17346,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8L1 Safari/6531.22.7*/
const Device UA689218 = {S80744, 146, { &P17346,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8J2 Safari/6531.22.7*/
const Device UA2143107 = {S80745, 147, { &P17346,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2370097 = {S80746, 147, { &P17346,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2438805 = {S80747, 147, { &P17346,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2786259 = {S80748, 147, { &P17346,&P17090,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_3_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA848463 = {S80749, 101, { &P18176,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 900)*/
const Device UA2166658 = {S80750, 126, { &P15364,&P7133,&P20433,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth Pro/6.2.2.6613 Safari/532.4*/
const Device UA1967073 = {S80751, 87, { &P12581,&P5055,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokia500) UCBrowser8.2.1.144/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA2000845 = {S80752, 66, { &P15364,&P5690,&P21173,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 5.1; en) Opera 12.01*/
const Device UA2007362 = {S80753, 72, { &P15364,&P5690,&P21173,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; ru; rv:2.0) Gecko/X Firefox/4.0 Opera 12.01*/
const Device UA2262147 = {S80754, 165, { &P13974,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5670 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2327873 = {S80755, 81, { &P22977,&P5629,&P15760,&P18092 }};	/*Useragent - SAMSUNG-GT-B9062_TD/1.0 Android/2.3.6 Release/09.15.2011 Browser/AppleWebKit533.1*/
const Device UA2520522 = {S80756, 135, { &P22977,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-B9062 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2808624 = {S80757, 142, { &P22977,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-B9062 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2670275 = {S80758, 92, { &P15364,&P5690,&P21808,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.19pre) Gecko/X Prism zdesktop/7.2.2*/
const Device UA2413921 = {S80759, 127, { &P21041,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; D97 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2421667 = {S80760, 149, { &P15798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Explorer A310e Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2728169 = {S80761, 143, { &P15798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Explorer Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3098864 = {S80762, 71, { &P15798,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC Explorer A310e Build/IMM76L)*/
const Device UA2422413 = {S80763, 79, { &P17835,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Barnes & Noble Nook Tablet Build/JZO54K)*/
const Device UA2528303 = {S80764, 157, { &P17835,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Barnes & Noble Nook Tablet Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2456737 = {S80765, 159, { &P19862,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2554331 = {S80766, 138, { &P21485,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; EK-GC120 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2598068 = {S80767, 138, { &P24461,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Celkon A79 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2600012 = {S80768, 105, { &P15364,&P18144,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.68 Safari/537.17*/
const Device UA2852647 = {S80769, 104, { &P15364,&P18144,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD i386) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.68 Safari/537.17*/
const Device UA2607327 = {S80770, 158, { &P21640,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-us; GT-I8245 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0.X Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2856606 = {S80771, 166, { &P21640,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; en-gb; SAMSUNG GT-I8245 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0.X Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2668784 = {S80772, 143, { &P20076,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xperia Acro S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2895545 = {S80773, 140, { &P20076,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26w Build/6.2.B.0.200) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2676690 = {S80774, 135, { &P20015,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SC-03E Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676987 = {S80775, 133, { &P14997,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S100 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2698682 = {S80776, 41, { &P12933,&P17484,&P3625,&P18092 }};	/*Useragent - sam-r631 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA2737703 = {S80777, 111, { &P15364,&P5690,&P22527,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.6 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/537.6 Midori/0.5*/
const Device UA2783924 = {S80778, 158, { &P18303,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; MT27i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2756277 = {S80779, 138, { &P18707,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MD_LIFETAB_P9516 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2800103 = {S80780, 158, { &P18280,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC VLE_U Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2869417 = {S80781, 158, { &P18280,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2869691 = {S80782, 158, { &P18280,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One S Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2945607 = {S80783, 157, { &P18295,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC One X Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2942911 = {S80784, 136, { &P24128,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SH-07E Build/S5310) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022939 = {S80785, 136, { &P24128,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SH-07E Build/S7120) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2944040 = {S80786, 97, { &P18435,&P21460,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.3; Windows NT 6.3; ARM; Trident/7.0; rv:11.0) like Gecko/X Firefox/12.0*/
const Device UA3026059 = {S80787, 81, { &P18435,&P21460,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.3; ARM; Trident/7.0; rv:11.0) like Gecko/X Firefox/12.0*/
const Device UA2974102 = {S80788, 138, { &P19848,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M440S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2976698 = {S80789, 127, { &P17926,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AT100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3030922 = {S80790, 103, { &P12449,&P5890,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae5-00) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3080125 = {S80791, 132, { &P18280,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; One S Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080255 = {S80792, 143, { &P18645,&P21637,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SAMSUNG-SGH-I747 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.30 Mobile Safari/537.36*/
const Device UA24387 = {S80793, 52, { &P14258,&P17486,&P3402,&P18092 }};	/*Useragent - MOT-W375/0.0.64 UP.Browser/6.3.0.6.c.9 (GUI) MMP/2.0*/
const Device UA51080 = {S80794, 143, { &P12990,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HTC Bee Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA90446 = {S80795, 148, { &P12990,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HTC Bee Build/ERE27) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA2018315 = {S80796, 65, { &P12990,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; HTC Bee Build/ERE27)*/
const Device UA19958 = {S80797, 79, { &P15014,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-F930(Yoigo)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA43846 = {S80798, 77, { &P15014,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-F930(3UK)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20723 = {S80799, 75, { &P11958,&P5448,&P4578,&P18092 }};	/*Useragent - Palm750/v0000 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)*/
const Device UA24371 = {S80800, 75, { &P11958,&P5448,&P4578,&P18092 }};	/*Useragent - Palm750/v0003 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)*/
const Device UA54952 = {S80801, 75, { &P11958,&P5448,&P4578,&P18092 }};	/*Useragent - Palm750/v0100 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)*/
const Device UA33305 = {S80802, 11, { &P16858,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M5445*/
const Device UA45656 = {S80803, 115, { &P11950,&P5629,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.3.3;Zh_cn;X10i;480*854;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355*/
const Device UA699585 = {S80804, 66, { &P11950,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.3;Zh_cn;X10i;480*854;)UCWEB7.8.0.95/139/444*/
const Device UA48472 = {S80805, 131, { &P18083,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; pt-br; V9C Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA69993 = {S80806, 65, { &P13143,&P17483,&P17602,&P18092 }};	/*Useragent - LG-A250/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2412644 = {S80807, 56, { &P17572,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SN3N UP.Browser/6.2_7.2.7.1.K.4.1.204 (GUI) MMP/2.0*/
const Device UA397480 = {S80808, 41, { &P14454,&P5890,&P24665,&P18092 }};	/*Useragent - NokiaE66/UC Browser8.0.4.121/28/351/UCWEB*/
const Device UA1905970 = {S80809, 29, { &P14454,&P5890,&P24665,&P18092 }};	/*Useragent - NokiaE66/UCWEB8.0.3.99/28/800*/
const Device UA412259 = {S80810, 141, { &P17968,&P18149,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; cm_tenderloin Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA454605 = {S80811, 141, { &P17968,&P18149,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; cm_tenderloin Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA711718 = {S80812, 140, { &P17968,&P18149,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; cm_tenderloin Build/GWK74) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA799415 = {S80813, 141, { &P17968,&P18149,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; cm_tenderloin Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2027137 = {S80814, 141, { &P17968,&P18149,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; cm_tenderloin Build/IMM76I) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2684739 = {S80815, 141, { &P17968,&P18149,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; cm_tenderloin Build/IMM76L) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7*/
const Device UA435612 = {S80816, 44, { &P17595,&P17292,&P18333,&P18092 }};	/*Useragent - iTunes/4.2 (Macintosh; U; PPC Mac OS X 10.2)*/
const Device UA573933 = {S80817, 81, { &P15201,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_Sensation_Z710e; 540*960) UCWEB7.9.3.103/139/800*/
const Device UA737237 = {S80818, 81, { &P15201,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; HTC_Sensation_Z710e; 540*960) UCWEB7.9.3.103/139/800*/
const Device UA2466432 = {S80819, 80, { &P15201,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC_Sensation_Z710e; 540*960) UCWEB7.9.0.94/139/352*/
const Device UA2503468 = {S80820, 80, { &P15201,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; HTC_Sensation_Z710e; 540*960) UCWEB7.9.0.94/139/352*/
const Device UA2527653 = {S80821, 80, { &P15201,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC_Sensation_Z710e; 540*960) UCWEB7.9.0.94/139/444*/
const Device UA2980650 = {S80822, 81, { &P15201,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_Sensation_Z710e; 540*960) UCWEB7.9.4.145/139/800*/
const Device UA633764 = {S80823, 86, { &P17595,&P17294,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:8.0) Gecko/X Firefox/8.0 TenFourFox/7450*/
const Device UA861743 = {S80824, 97, { &P15364,&P17843,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; pl-PL; rv:1.9.0.7) Gecko/X Kubuntu/8.10 (intrepid) Firefox/3.0.9*/
const Device UA1887011 = {S80825, 69, { &P15364,&P17843,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Kubuntu 8.4; de; rv:1.9) Gecko/X Firefox/3.0.1*/
const Device UA900493 = {S80826, 165, { &P15649,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352*/
const Device UA912943 = {S80827, 102, { &P15364,&P18346,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA1906516 = {S80828, 90, { &P15364,&P18110,&P17558,&P18091 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.23) Gecko/X YextBot/Java Firefox/3.6.23*/
const Device UA2258380 = {S80829, 126, { &P17595,&P17297,&P24727,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) FluidApp Version/1.6.1 Safari/534.57.7*/
const Device UA2453891 = {S80830, 126, { &P17595,&P17297,&P24727,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) FluidApp Version/1.6.1 Safari/534.56.5*/
const Device UA2033526 = {S80831, 121, { &P21309,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-tw; myPadP4_V1.4) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2448391 = {S80832, 121, { &P21309,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-tw; myPadP4_V1.6) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2814325 = {S80833, 130, { &P21309,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; zh-tw; myPadP4 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2175683 = {S80834, 133, { &P18465,&P18110,&P19988,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.0 Safari/535.19 Silk-Accelerated=false*/
const Device UA2456681 = {S80835, 69, { &P16436,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GT-I8150; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2401486 = {S80836, 134, { &P20413,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; CELKON CT2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2511628 = {S80837, 135, { &P17595,&P18207,&P19995,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Version/6.0.2 Safari/8536.26.17 Sleipnir/4.0.1*/
const Device UA2659648 = {S80838, 162, { &P12026,&P17485,&P24676,&P18092 }};	/*Useragent - SonyEricssonC510/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonc510) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2558282 = {S80839, 144, { &P21740,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; IRIS_430 Build/LAVAIRIS430) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558904 = {S80840, 139, { &P22168,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AUXUS CoreX2 3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2567005 = {S80841, 131, { &P13452,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2632059 = {S80842, 131, { &P13452,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P1000 Build/JRO03R) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22*/
const Device UA2633888 = {S80843, 138, { &P13452,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2634797 = {S80844, 132, { &P13452,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000N Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583695 = {S80845, 138, { &P20103,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; EK-GC100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2704296 = {S80846, 138, { &P20103,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EK-GC100 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2877601 = {S80847, 132, { &P20523,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; tr-tr; GT-P3105 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2651488 = {S80848, 143, { &P19968,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Haier HW-W910 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2659225 = {S80849, 156, { &P17346,&P18117,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 511 like Mac OS X; en-us) AppleWebKit/534.35 (KHTML like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7055 Mobile*/
const Device UA2673163 = {S80850, 138, { &P22729,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT918 Build/2_26C_2009) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3103307 = {S80851, 131, { &P22729,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT918 Build/2_32D_2009) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2694390 = {S80852, 135, { &P16798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; W960 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952450 = {S80853, 141, { &P16798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KONKA W960 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3070373 = {S80854, 126, { &P16798,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; W960 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2699594 = {S80855, 135, { &P23622,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; ELEMENT 8V3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2700209 = {S80856, 141, { &P23154,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XOLO_X1000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2770097 = {S80857, 139, { &P14993,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT15i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774125 = {S80858, 136, { &P22097,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTabS2110AF Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3074117 = {S80859, 136, { &P22097,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTabS2110AH Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2782249 = {S80860, 161, { &P12605,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2795944 = {S80861, 136, { &P24493,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; BS471 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2868373 = {S80862, 173, { &P18509,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-MS770 Build/JZO54K; CyanogenMod-10.0.0.l0.BobZhome.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2975540 = {S80863, 133, { &P24068,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; N986 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3094730 = {S80864, 140, { &P21542,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6503 Build/10.1.A.1.434) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3131309 = {S80865, 141, { &P18462,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA30165 = {S80866, 64, { &P13662,&P17483,&P3459,&P18092 }};	/*Useragent - LG-GW370/V10c Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA90688 = {S80867, 129, { &P21541,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; TPA60W Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA131652 = {S80868, 34, { &P15364,&P17285,&P22484,&P18092 }};	/*Useragent - Opera/9.10 (Windows NT 6.0; U; en)*/
const Device UA253992 = {S80869, 44, { &P20126,&P17485,&P17338,&P18092 }};	/*Useragent - sonyericssonk800i/UC Browser7.9.0.102/70/352*/
const Device UA254008 = {S80870, 44, { &P20126,&P17485,&P17338,&P18092 }};	/*Useragent - sonyericssonk800i/UC Browser7.9.0.102/69/352*/
const Device UA631175 = {S80871, 91, { &P15364,&P22194,&P17829,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/4.6; Linux) KHTML/4.6.5 (like Gecko) Fedora/4.6.5-8.fc14*/
const Device UA440418 = {S80872, 80, { &P15364,&P22857,&P4536,&P18092 }};	/*Useragent - Opera/9.80 (MTK; Nucleus; Opera Mobi/2167; U; en-US) Presto/2.4.18 Version/10.00*/
const Device UA556027 = {S80873, 54, { &P15364,&P5690,&P4393,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; U; Opera Mobi; de; CEB-42)*/
const Device UA624793 = {S80874, 36, { &P11945,&P17484,&P17347,&P18092 }};	/*Useragent - gt-s5233s/UC Browser8.0.3.107/69/352*/
const Device UA1937797 = {S80875, 36, { &P11945,&P17484,&P17347,&P18092 }};	/*Useragent - gt-s5233s/UC Browser8.0.3.107/69/351*/
const Device UA2786294 = {S80876, 36, { &P11945,&P17484,&P17347,&P18092 }};	/*Useragent - gt-s5233s/UC Browser8.0.3.107/70/352*/
const Device UA644048 = {S80877, 85, { &P14980,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-b7722) UCBrowser8.2.1.144/69/352/UCWEB Mobile*/
const Device UA663979 = {S80878, 85, { &P14980,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-b7722) UCBrowser8.2.0.132/69/352/UCWEB Mobile*/
const Device UA690026 = {S80879, 62, { &P15364,&P7133,&P17698,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:8.0) Gecko/X Firefox/8.0 slurp*/
const Device UA704784 = {S80880, 69, { &P15364,&P7133,&P17698,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:8.0) Gecko/X Firefox/8.0 slurp*/
const Device UA712015 = {S80881, 73, { &P15364,&P7133,&P17698,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:8.0.1) Gecko/X Firefox/8.0.1 slurp*/
const Device UA1904646 = {S80882, 75, { &P15364,&P7133,&P17698,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:8.0) Gecko/X Firefox/8.0 slurp slurp*/
const Device UA1930593 = {S80883, 66, { &P15364,&P7133,&P17698,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:8.0.1) Gecko/X Firefox/8.0.1 slurp*/
const Device UA2156695 = {S80884, 100, { &P18267,&P5043,&P17338,&P18092 }};	/*Useragent - NokiaX2-02/2.0 (11.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-02/UC Browser7.9.0.102/69/352*/
const Device UA891434 = {S80885, 109, { &P15364,&P18144,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; OpenBSD i386; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/5.0.359.0 Safari/533.3*/
const Device UA1996169 = {S80886, 135, { &P22021,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Kogan-Agora Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2582978 = {S80887, 141, { &P22021,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KHPHN05ANDA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2030540 = {S80888, 133, { &P22993,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TM-MID720 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032593 = {S80889, 158, { &P19998,&P18066,&P20776,&P18092 }};	/*Useragent - MQQBrowser/25/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HW-HUAWEI_C8812/C8812V100R001C92B913SP02; 480*800; CTC/2.0) AppleWebKit/534.30 Mobile Safari/534.30*/
const Device UA2033157 = {S80890, 162, { &P19998,&P18066,&P20776,&P18092 }};	/*Useragent - MQQBrowser/25/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2382514 = {S80891, 79, { &P11689,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian70-1) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2402034 = {S80892, 70, { &P18296,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC_One_V; 480*800) UCWEB7.9.0.94/139/352*/
const Device UA2456160 = {S80893, 70, { &P18296,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC_One_V; 480*800) UCWEB7.9.0.94/139/355*/
const Device UA2456730 = {S80894, 70, { &P18296,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC_One_V; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2427196 = {S80895, 135, { &P23523,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; I9377 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2561200 = {S80896, 138, { &P23523,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-i9377 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2458430 = {S80897, 146, { &P20090,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848 Duo Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2485907 = {S80898, 147, { &P20090,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848i Duo Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2483525 = {S80899, 129, { &P11932,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA6210 Navigator) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2529403 = {S80900, 132, { &P23902,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT5 Build/GT3385) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2580300 = {S80901, 93, { &P16356,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-N7000) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2659212 = {S80902, 147, { &P22630,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; INHON PAPILIO G1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952489 = {S80903, 69, { &P22630,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; INHON PAPILIO G1 Build/JRO03C)*/
const Device UA2661338 = {S80904, 58, { &P15364,&P22308,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; WinNT3.51; Trident/2.0)*/
const Device UA2674201 = {S80905, 139, { &P21507,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C1605 Build/11.1.A.0.68) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2694846 = {S80906, 141, { &P13960,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch V9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2787099 = {S80907, 132, { &P13960,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; CHUWI V9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2980413 = {S80908, 137, { &P13960,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ThL V9 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2707350 = {S80909, 139, { &P23855,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; WM8850-mid Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708268 = {S80910, 132, { &P23855,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; WM8850-mid Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2718532 = {S80911, 140, { &P18654,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; WX04K Build/324.4.0000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867883 = {S80912, 140, { &P18654,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; WX04K Build/324.5.0000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058952 = {S80913, 140, { &P18654,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; WX04K Build/325.3.0000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2733595 = {S80914, 157, { &P23728,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; O+ 8.52 Android Build/Oplus8.52_Android) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734160 = {S80915, 98, { &P19862,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Micromax_A110) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2796392 = {S80916, 156, { &P19847,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S6012 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2751059 = {S80917, 144, { &P22989,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; i-mobile IQ 4 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2754185 = {S80918, 130, { &P22052,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7501 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2784755 = {S80919, 159, { &P18453,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT25i Build/9.1.A.1.140) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2781141 = {S80920, 76, { &P15364,&P18110,&P22317,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:21.0) Gecko/X Firefox/21.0 SeaMonkey/2.18*/
const Device UA3010609 = {S80921, 159, { &P18471,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT907 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/xt907*/
const Device UA2869898 = {S80922, 140, { &P15767,&P17090,&P20783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/8J2 Safari/7534.48.3*/
const Device UA2897085 = {S80923, 140, { &P15767,&P17090,&P20783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/8L1 Safari/7534.48.3*/
const Device UA2888662 = {S80924, 155, { &P18464,&P18066,&P23483,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2894853 = {S80925, 90, { &P15989,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2944695 = {S80926, 91, { &P15989,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i2) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2945466 = {S80927, 141, { &P12280,&P18117,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 UCBrowser/9.1.0.287 Mobile*/
const Device UA3013776 = {S80928, 141, { &P12280,&P18117,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 UCBrowser/9.1.0.287 Mobile*/
const Device UA3021250 = {S80929, 131, { &P24103,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; C2 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021193 = {S80930, 139, { &P13054,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ADR6425LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023938 = {S80931, 93, { &P22609,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6312) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3115506 = {S80932, 157, { &P18295,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One X Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA105 = {S80933, 40, { &P13273,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KC550 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA63730 = {S80934, 89, { &P12129,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia5500d/2.0 (04.60) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA27379 = {S80935, 129, { &P12173,&P5448,&P22270,&P18092 }};	/*Useragent - LG-CT810/V10x IEMobile/7.11 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA136402 = {S80936, 54, { &P17570,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SN3J UP.Browser/6.2_7.2.7.1.K.3.352 (GUI) MMP/2.0*/
const Device UA62042 = {S80937, 68, { &P14363,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHE490/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA327774 = {S80938, 79, { &P15364,&P17843,&P17829,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/4.6; Linux) KHTML/4.6.5 (like Gecko) Kubuntu*/
const Device UA2020105 = {S80939, 70, { &P17324,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.3;Zh_cn;GT-I9001;480*800;)UCWEB7.8.0.95/139/351*/
const Device UA2033657 = {S80940, 70, { &P17324,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.5;Zh_cn;GT-I9001;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA601171 = {S80941, 76, { &P15364,&P17285,&P18329,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:11.0) Gecko/X Thunderbird/11.0 Lightning/1.3*/
const Device UA600159 = {S80942, 134, { &P18424,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; FPT F6 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622486 = {S80943, 58, { &P18424,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; FPT F6 Build/GRJ90)*/
const Device UA749176 = {S80944, 93, { &P17968,&P5477,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.1-update1; Linux; Opera Mobi/ADR-X; U; ja) Presto/2.9.201 Version/11.50*/
const Device UA796347 = {S80945, 134, { &P17595,&P17295,&P24746,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Stainless/0.7.5 like Version/4.0 Safari/530.17*/
const Device UA2270820 = {S80946, 144, { &P17595,&P17295,&P24746,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Stainless/0.7.5 like Version/4.0 Safari/530.17*/
const Device UA1934531 = {S80947, 67, { &P15147,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.4;Zh_cn;ST15i;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA2027372 = {S80948, 152, { &P24026,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; M.T.T. Smart Multimedia Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2164708 = {S80949, 47, { &P14358,&P5890,&P21761,&P18092 }};	/*Useragent - iBrowser/2.5 (E63-1;SymbianOS/9.2 Series60/3.1)*/
const Device UA2261500 = {S80950, 138, { &P18630,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A800S Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2366525 = {S80951, 138, { &P13999,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; NexusHD2 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2380116 = {S80952, 137, { &P13999,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC HD2 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2456049 = {S80953, 137, { &P13999,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC HD2 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2420705 = {S80954, 140, { &P17475,&P5768,&P24670,&P18092 }};	/*Useragent - Nokia6300/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia6300) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2426718 = {S80955, 139, { &P15364,&P17350,&P19966,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; ; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.4.2.X (Second Life Release; default skin) Safari/533.3*/
const Device UA2484940 = {S80956, 151, { &P21295,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MODECOM FreeTab 1002 IPS X2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2490932 = {S80957, 155, { &P18464,&P18066,&P19837,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2558430 = {S80958, 92, { &P14417,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5236) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2567372 = {S80959, 132, { &P21334,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A8 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2699609 = {S80960, 56, { &P15364,&P22310,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; Win3.11; Trident/5.0)*/
const Device UA2857010 = {S80961, 49, { &P15364,&P22310,&P17384,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows 3.11)*/
const Device UA2676998 = {S80962, 143, { &P15854,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ607-14) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2701966 = {S80963, 155, { &P20598,&P18066,&P18062,&P18092 }};	/*Useragent - Miracle Linux/3.0.13 Android/4.0.4 Release/11.19.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA3048367 = {S80964, 146, { &P20598,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; Enspert_Miracle Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2718798 = {S80965, 92, { &P12707,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia701) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2774336 = {S80966, 165, { &P18125,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2800679 = {S80967, 156, { &P21628,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8552 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2901214 = {S80968, 165, { &P15798,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Explorer A310e Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2937201 = {S80969, 161, { &P16356,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7000 Build/VeTW3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2994470 = {S80970, 162, { &P16356,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA3015877 = {S80971, 163, { &P16356,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N7000 Build/VeTW3.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA3021590 = {S80972, 132, { &P18017,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Ideapad K1 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022972 = {S80973, 124, { &P18017,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; K1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2964515 = {S80974, 149, { &P24041,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Karbonn A50 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2969133 = {S80975, 93, { &P14359,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5228) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA2976007 = {S80976, 145, { &P18556,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD+ Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3025941 = {S80977, 161, { &P17324,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3080074 = {S80978, 161, { &P17324,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126086 = {S80979, 161, { &P17324,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3036276 = {S80980, 134, { &P22653,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTL22 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3083253 = {S80981, 137, { &P18002,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A760S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080131 = {S80982, 137, { &P24105,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Archos 80 Xenon Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3083493 = {S80983, 148, { &P17835,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook Tablet Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080372 = {S80984, 152, { &P21833,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3130730 = {S80985, 138, { &P19902,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131308 = {S80986, 137, { &P15575,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132937 = {S80987, 137, { &P15575,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA62362 = {S80988, 81, { &P14234,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK300i/R2BA SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA157449 = {S80989, 61, { &P16703,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX265/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA550625 = {S80990, 141, { &P15364,&P17285,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.4) Gecko/X  Firefoxlib/5.0\0318  control/5.00318  X  ( .NET CLR 3.5.30729; .NET4.0C)*/
const Device UA589998 = {S80991, 169, { &P15364,&P17285,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.1.16) Gecko/X AskTbFWV5/3.14.0.19709  Firefox[xSP_2:a6bb6b87c1124af321e1acd56a3bc3d0_208]  X  (.NET CLR 3.5.30729)*/
const Device UA2259853 = {S80992, 101, { &P15364,&P17285,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.25) Gecko/X  FirefoX X  (.NET CLR 3.5.30729)*/
const Device UA561993 = {S80993, 97, { &P11689,&P5776,&P24662,&P18092 }};	/*Useragent - NokiaN70-1/5.0737.3.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UCWEB7.8.0.87/27/352*/
const Device UA2541600 = {S80994, 96, { &P11689,&P5776,&P24662,&P18092 }};	/*Useragent - NokiaN70/ 5.0741.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UCWEB7.8.0.87/27/352*/
const Device UA2023067 = {S80995, 154, { &P11950,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; X10i Build/3.0.1.G.0.75; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA662850 = {S80996, 95, { &P15364,&P18135,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.5) Gecko/X Linux Mint/8 (Helena) Firefox/3.5.5*/
const Device UA756003 = {S80997, 97, { &P15364,&P18135,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1.8) Gecko/X Linux Mint/8 (Helena) Firefox/3.5.8*/
const Device UA2265772 = {S80998, 97, { &P15364,&P18135,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1.7) Gecko/X Linux Mint/8 (Helena) Firefox/3.5.7*/
const Device UA1992203 = {S80999, 147, { &P17346,&P18117,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2156879 = {S81000, 145, { &P17346,&P18117,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2002389 = {S81001, 141, { &P12280,&P18117,&P20787,&P18092 }};	/*Useragent - MQQBrowser/36 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA2185510 = {S81002, 140, { &P12280,&P18117,&P20787,&P18092 }};	/*Useragent - MQQBrowser/36 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3*/
const Device UA2346590 = {S81003, 139, { &P12280,&P18117,&P20787,&P18092 }};	/*Useragent - MQQBrowser/36 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3*/
const Device UA2224650 = {S81004, 140, { &P12709,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MK16a Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2361764 = {S81005, 114, { &P12108,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia7230/06.90; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA2486553 = {S81006, 191, { &P18113,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (DROID RAZR, 39, 1.8.2)*/
const Device UA2229727 = {S81007, 134, { &P17595,&P17295,&P22461,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1200.0 Iron/21.0.1200.0  Safari/537.1*/
const Device UA2367115 = {S81008, 130, { &P13124,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; PantechP9070 Build/IMM76I) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2383615 = {S81009, 134, { &P18680,&P18110,&P19814,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.2 Safari/535.19 Silk-Accelerated=false*/
const Device UA2384218 = {S81010, 119, { &P14359,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5228) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile*/
const Device UA2386074 = {S81011, 141, { &P21526,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cherry w500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2402939 = {S81012, 154, { &P20534,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAP4040_DUO Build/PrestigioPAP4040DUO) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2413654 = {S81013, 74, { &P20099,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-C3313T/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2431003 = {S81014, 118, { &P16072,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN78) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile*/
const Device UA2527409 = {S81015, 137, { &P21379,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SH631W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469720 = {S81016, 169, { &P14682,&P5843,&P24675,&P18092 }};	/*Useragent - Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia6303iclassic) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2487843 = {S81017, 99, { &P15655,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3850) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2502759 = {S81018, 131, { &P18612,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T329t Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2655370 = {S81019, 140, { &P18612,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T329t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986538 = {S81020, 62, { &P18612,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HTC T329t Build/IMM76D)*/
const Device UA2578260 = {S81021, 145, { &P21654,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; POV_TAB-PROTAB30-IPS9 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2659081 = {S81022, 135, { &P17346,&P18117,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 511 like Mac OS X) AppleWebKit/534.46 (KHTML like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.1.1m*/
const Device UA2668070 = {S81023, 134, { &P24140,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; CTP989RX Build/CTP989RX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2676335 = {S81024, 134, { &P19985,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SH-02E Build/S2130) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712288 = {S81025, 161, { &P19861,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I605 Build/JDQ39; Carbon-CARBON-JB-v1.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752688 = {S81026, 162, { &P19861,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I605 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2737319 = {S81027, 155, { &P18687,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; LG-P705 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2770904 = {S81028, 163, { &P18213,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-S6500D Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2774126 = {S81029, 162, { &P18213,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-S6500 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2773870 = {S81030, 132, { &P23162,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B15 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2851510 = {S81031, 59, { &P15364,&P22308,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; WinNT3.51; Trident/3.0)*/
const Device UA2895907 = {S81032, 159, { &P14969,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HTC Desire S Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3007802 = {S81033, 159, { &P14969,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; HTC Desire S Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2901191 = {S81034, 139, { &P20134,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2944804 = {S81035, 155, { &P21648,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9505 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2999676 = {S81036, 154, { &P24538,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; ALCATEL ONE TOUCH 8008X Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2 Mobile Safari/534.30*/
const Device UA3092295 = {S81037, 152, { &P24538,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; he-il; orange infinity 8008X Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2 Mobile Safari/534.30*/
const Device UA3012742 = {S81038, 140, { &P20436,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Cyclone Voyager 2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3057160 = {S81039, 138, { &P20436,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-uk; cyclone voyager Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3057337 = {S81040, 145, { &P20436,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Cyclone Voyager Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3021553 = {S81041, 137, { &P24391,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8260 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3078152 = {S81042, 130, { &P24391,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8260 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3024242 = {S81043, 97, { &P21475,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A35) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3033565 = {S81044, 105, { &P12193,&P5890,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac5-00.2) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3057806 = {S81045, 103, { &P12193,&P5890,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac5-00) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3037939 = {S81046, 136, { &P24655,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; IdeaTab A3000-H Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3092196 = {S81047, 136, { &P24655,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; IdeaTab A3000-F Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3081273 = {S81048, 134, { &P24568,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; N9810 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083754 = {S81049, 130, { &P14908,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6210 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA24559 = {S81050, 65, { &P15240,&P17483,&P17602,&P18092 }};	/*Useragent - LG-A230/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA43583 = {S81051, 46, { &P13707,&P5107,&P3625,&P18092 }};	/*Useragent - TXT8040/T05_0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA90895 = {S81052, 69, { &P17346,&P17146,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.9A334 (iPod; U; CPU OS 5_0 like Mac OS X; en_us)*/
const Device UA256643 = {S81053, 71, { &P17346,&P17146,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.9A405 (iPod; U; CPU OS 5_0_1 like Mac OS X; en_us)*/
const Device UA170861 = {S81054, 10, { &P14983,&P5055,&P4030,&P18092 }};	/*Useragent - NokiaC7-00*/
const Device UA245100 = {S81055, 109, { &P15364,&P6596,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 15.1011.43) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.51 Safari/535.2*/
const Device UA250642 = {S81056, 109, { &P15364,&P6596,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 15.1011.62) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.83 Safari/535.2*/
const Device UA252415 = {S81057, 111, { &P15364,&P6596,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 15.1011.107) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.116 Safari/535.2*/
const Device UA252984 = {S81058, 111, { &P15364,&P6596,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 15.1011.118) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.117 Safari/535.2*/
const Device UA255759 = {S81059, 111, { &P15364,&P6596,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 15.1011.137) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA629175 = {S81060, 106, { &P15364,&P6596,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 15.950.0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.862.1 Safari/535.2*/
const Device UA640287 = {S81061, 106, { &P15364,&P6596,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 15.916.0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.855.0 Safari/535.2*/
const Device UA1901644 = {S81062, 109, { &P15364,&P6596,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 15.997.2011) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.872.0 Safari/535.2*/
const Device UA543933 = {S81063, 138, { &P16762,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; md-us; Spice Mi-310 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA689835 = {S81064, 154, { &P15065,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID Pro Build/VZW; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3071716 = {S81065, 157, { &P15065,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID Pro Build/GRKLVL; CM7 by vanillaLVL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1898731 = {S81066, 136, { &P21451,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SH8188U Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1965217 = {S81067, 146, { &P15555,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; myTouch_4G_Slide Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2933847 = {S81068, 150, { &P15555,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC MyTouch 4G Slide Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1993783 = {S81069, 128, { &P19991,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A511 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2257906 = {S81070, 168, { &P11950,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2208429 = {S81071, 128, { &P15364,&P7133,&P22456,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Iron/6.0.475.1 Chrome/6.0.475.1 Safari/534*/
const Device UA2257388 = {S81072, 171, { &P13452,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2616322 = {S81073, 171, { &P13452,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2617054 = {S81074, 171, { &P13452,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2620985 = {S81075, 166, { &P13452,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; GT-P1000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/352*/
const Device UA2621118 = {S81076, 171, { &P13452,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2623884 = {S81077, 172, { &P13452,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-P1000N Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2353593 = {S81078, 135, { &P21011,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; L-01E Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2353770 = {S81079, 133, { &P21030,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAC-97032 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2467690 = {S81080, 120, { &P12236,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA6700s) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2452596 = {S81081, 127, { &P23992,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; N12 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2530943 = {S81082, 165, { &P18687,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; LG-P705 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2531007 = {S81083, 95, { &P14842,&P5477,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E10i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2556939 = {S81084, 147, { &P15699,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Droid Incredible Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102308 = {S81085, 147, { &P15699,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Droid Incredible Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583421 = {S81086, 135, { &P19985,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SH-02E Build/S2130) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2596507 = {S81087, 157, { &P15517,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Neo Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2596535 = {S81088, 157, { &P12942,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I927 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2765487 = {S81089, 156, { &P12942,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I927 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3008682 = {S81090, 157, { &P12942,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I927 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2602800 = {S81091, 161, { &P16356,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N7000 Build/JDQ39; Carbon-Carbon-JB-v1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892267 = {S81092, 162, { &P16356,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-N7000 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665110 = {S81093, 140, { &P23743,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; MITO t510 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666273 = {S81094, 179, { &P12871,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC Wildfire S A510e Build/kantry123 v2.0; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778423 = {S81095, 165, { &P12871,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Wildfire S A510e Build/ASNX; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044923 = {S81096, 152, { &P12871,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Marvelc Build/ASNX; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2705812 = {S81097, 98, { &P21020,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Micromax_A116) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2716969 = {S81098, 166, { &P18113,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT912 Build/JDQ39; LiquidSmooth-Liquid-JB-v2.2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2771509 = {S81099, 165, { &P18113,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT912 Build/JDQ39E; LiquidSmooth-Liquid-JB-v2.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2772895 = {S81100, 147, { &P17926,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT100 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2765915 = {S81101, 184, { &P18622,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/VEGAVIETTEAM_HOANGMINHHUU; CyanogenMod-JB_LR_VGV_V4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2779755 = {S81102, 165, { &P18622,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/JZO54K; CyanogenMod-JB_LR_VGV_V3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2860161 = {S81103, 164, { &P18622,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/IMM76L; CyanogenMod-CM9_OPTIMUS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3120280 = {S81104, 168, { &P18622,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A820L Build/Hoang Nguyen; CyanogenMod-EF84L_ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2772886 = {S81105, 145, { &P19867,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I437 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879711 = {S81106, 152, { &P23694,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Symphony Xplorer W35 Build/UNKNOWN) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2898568 = {S81107, 151, { &P20529,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A89 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2938022 = {S81108, 133, { &P24524,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; V812 Core4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2941933 = {S81109, 133, { &P16356,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-N7000 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2974725 = {S81110, 134, { &P23477,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT926 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021163 = {S81111, 135, { &P22860,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Q700 Build/XOLOQ700) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023105 = {S81112, 140, { &P18465,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Amazon Kindle Fire Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3050240 = {S81113, 160, { &P15364,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; L36h Build/10.1.1.A.1.253) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3053934 = {S81114, 156, { &P15364,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; ZTE U956 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3091193 = {S81115, 169, { &P15364,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI D2-2010 Build/HuaweiD2-2010) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3081383 = {S81116, 138, { &P24480,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094958 = {S81117, 102, { &P15991,&P18110,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-S6102) U2/1.0.0 UCBrowser/9.3.0.321 Mobile*/
const Device UA4032 = {S81118, 101, { &P12068,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-J150/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA18639 = {S81119, 102, { &P12068,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-J150B/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA13599 = {S81120, 83, { &P11762,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW710i/R1JC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24498 = {S81121, 124, { &P12077,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-S3600i/S3600iJPIL1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA69308 = {S81122, 31, { &P14748,&P5890,&P24663,&P18092 }};	/*Useragent - NokiaE72-1/UCWEB7.9.0.94/28/999*/
const Device UA342758 = {S81123, 43, { &P14748,&P5890,&P24663,&P18092 }};	/*Useragent - NokiaE72-1/UC Browser7.9.0.102/28/355/UCWEB*/
const Device UA87653 = {S81124, 117, { &P12280,&P6274,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Mobile/5F136*/
const Device UA96805 = {S81125, 104, { &P12280,&P6274,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko)*/
const Device UA452749 = {S81126, 77, { &P15364,&P17350,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 7.1; ru; rv:1.9.3.15) Gecko/X Firefox/7.0*/
const Device UA2230700 = {S81127, 38, { &P15364,&P17350,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT) Firefox/7.0.1*/
const Device UA723636 = {S81128, 78, { &P16003,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; gt-s5233a) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2474831 = {S81129, 31, { &P16003,&P17484,&P24667,&P18092 }};	/*Useragent - gt-s5233a/UCWEB8.2.0.116/69/352*/
const Device UA1955668 = {S81130, 79, { &P11689,&P5890,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian70-1) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA1952087 = {S81131, 79, { &P14454,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE66)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA1991855 = {S81132, 30, { &P14454,&P5890,&P24671,&P18092 }};	/*Useragent - NokiaE66/UCWEB8.5.0.163/28/800*/
const Device UA1996177 = {S81133, 79, { &P15364,&P16110,&P22361,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:12.3) Gecko/X Firefox/12.3 PaleMoon/12.3*/
const Device UA1996363 = {S81134, 92, { &P15364,&P16110,&P22361,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:12.3) Gecko/X Firefox/12.3-x64 PaleMoon/12.3-x64*/
const Device UA2669709 = {S81135, 99, { &P17595,&P18207,&P21808,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en-US; rv:1.9.2.16pre) Gecko/X Prism zdesktop/7.2.2*/
const Device UA2384875 = {S81136, 139, { &P21952,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; Allview2SpeedDuo Build/SV1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2417804 = {S81137, 134, { &P22650,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; Dslide 702 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2559762 = {S81138, 134, { &P22650,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Dslide 702 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2453280 = {S81139, 138, { &P16589,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; KM-S300 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2531574 = {S81140, 148, { &P23464,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-us; Spice Mi-424 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2549847 = {S81141, 141, { &P21492,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Micromax A84 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2579296 = {S81142, 148, { &P21518,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-ph; Cloudfone_Excite320g Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2579524 = {S81143, 136, { &P21446,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SH837W Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2604561 = {S81144, 135, { &P22746,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N9100 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2637114 = {S81145, 130, { &P17323,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P6800 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Safari/537.31*/
const Device UA2711926 = {S81146, 130, { &P17323,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6800 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2659827 = {S81147, 84, { &P23295,&P17979,&P18062,&P18092 }};	/*Useragent - ZTEU807_TD/1.0 Linux/3.0.13 Android/4.0 Release/10.15.2012 Browser/AppleWebKit534.30*/
const Device UA2664986 = {S81148, 171, { &P19862,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; Micromax A110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3022841 = {S81149, 175, { &P19862,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax_A110 Build/JRO03C) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2672461 = {S81150, 133, { &P20156,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; S500 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2715056 = {S81151, 159, { &P19763,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; C6602 Build/10.1.A.1.434) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2799368 = {S81152, 161, { &P19763,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; C6602 Build/10.1.1.A.1.253) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2781777 = {S81153, 135, { &P22852,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; eZee'Tab709 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2782905 = {S81154, 139, { &P18452,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT29i Build/7.0.A.3.212) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2864731 = {S81155, 11, { &P23332,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M5364*/
const Device UA2898006 = {S81156, 140, { &P11751,&P17483,&P24678,&P18092 }};	/*Useragent - LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-kp500 teleca) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2898792 = {S81157, 157, { &P18296,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC One V Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2937545 = {S81158, 142, { &P23498,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; sr-rs; ZTE Blade G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3023662 = {S81159, 160, { &P22622,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Micromax A72 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3060752 = {S81160, 157, { &P18295,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One X Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3080495 = {S81161, 140, { &P22632,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Titanium S1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3132859 = {S81162, 138, { &P23772,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA18506 = {S81163, 53, { &P14086,&P5284,&P4223,&P18092 }};	/*Useragent - sam-r810 NetFront/3.5 Mozilla/5.0 like Gecko/20060426*/
const Device UA63576 = {S81164, 105, { &P17609,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-GT-S5628/ SHP/VPP/R5 Dolfin/1.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA182032 = {S81165, 134, { &P17953,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-za; RBM_HD Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA656624 = {S81166, 119, { &P20164,&P17605,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-S3778V/1.0 RTK-E/1.0 DF3G/1.0 Release/04.04.2011 Browser/NetFront4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA791272 = {S81167, 144, { &P14225,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Desire HD Build/GRI40)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2938028 = {S81168, 155, { &P14225,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2998036 = {S81169, 145, { &P14225,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA3049928 = {S81170, 145, { &P14225,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Desire HD Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA893237 = {S81171, 139, { &P20930,&P5629,&P15760,&P18092 }};	/*Useragent - AT-AS28D Linux/2.6.35 Android/2.3.5 Release/04.20.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1*/
const Device UA2812075 = {S81172, 136, { &P20930,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; nl-nl; AT-AS28D Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1956191 = {S81173, 51, { &P17595,&P18120,&P20786,&P18092 }};	/*Useragent - MQQBrowser/35 (iOS 4S; U; CPU like Mac OS X; zh-cn)*/
const Device UA1958520 = {S81174, 50, { &P17595,&P18120,&P20786,&P18092 }};	/*Useragent - MQQBrowser/35 (iOS 4; U; CPU like Mac OS X; zh-cn)*/
const Device UA2334892 = {S81175, 52, { &P17595,&P18120,&P20786,&P18092 }};	/*Useragent - MQQBrowser/35 (iOS 3GS; U; CPU like Mac OS X; zh-cn)*/
const Device UA2002003 = {S81176, 135, { &P13505,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MB525 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2022045 = {S81177, 160, { &P22341,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MOT-XT800+/TTUPG_M7_2.45.0; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024590 = {S81178, 160, { &P22341,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-XT800+/TTUPG_M7_2.52.0; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027224 = {S81179, 137, { &P22341,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT800+ Build/TTUPG_M7_2.55.0) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2495225 = {S81180, 160, { &P22341,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-XT800+/TTUPG_M7_2.55.0; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2808712 = {S81181, 144, { &P22341,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; XT800+ Build/TTUPG_M7_2.52.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2149249 = {S81182, 121, { &P15364,&P16110,&P22415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.1.1.0 Chrome/16.0.912.63 Safari/535.7*/
const Device UA2494130 = {S81183, 128, { &P15364,&P16110,&P22415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.1.1.0 Chrome/16.0.912.63 Safari/535.7*/
const Device UA2162959 = {S81184, 71, { &P20032,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X291/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2197739 = {S81185, 130, { &P23766,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-7230 Build/M7513) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2204355 = {S81186, 137, { &P18499,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ISW11SC Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2257054 = {S81187, 140, { &P23082,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A828 TV Duo Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2258709 = {S81188, 90, { &P15364,&P5690,&P20806,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.0 Safari/534.34*/
const Device UA2332553 = {S81189, 131, { &P17595,&P18207,&P22454,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7*/
const Device UA2350719 = {S81190, 131, { &P17595,&P18207,&P22454,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7*/
const Device UA2794506 = {S81191, 131, { &P17595,&P18207,&P22454,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7*/
const Device UA3014954 = {S81192, 131, { &P17595,&P18207,&P22454,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7*/
const Device UA2369723 = {S81193, 74, { &P18141,&P19823,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 2.00) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA2431770 = {S81194, 170, { &P18382,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Desire C Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2461122 = {S81195, 139, { &P11751,&P17483,&P24675,&P18092 }};	/*Useragent - LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-kp500 teleca) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2535756 = {S81196, 159, { &P18295,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC One X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2557833 = {S81197, 90, { &P12616,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; WT19i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2604670 = {S81198, 164, { &P20183,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2603947 = {S81199, 156, { &P15575,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; GT-I9100 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2667413 = {S81200, 156, { &P15575,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2667906 = {S81201, 157, { &P15575,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9100G Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2663162 = {S81202, 139, { &P21495,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E220S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2675776 = {S81203, 138, { &P23905,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100P Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2695750 = {S81204, 128, { &P15575,&P17979,&P18112,&P18092 }};	/*Useragent - MTk-GT_I9100/1.0 Linux/2.6.35.7 Android/4.1.1 Release/10.12.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2707048 = {S81205, 81, { &P15575,&P17979,&P18112,&P18092 }};	/*Useragent - SAMSUNG-GT-I9108_TD/1.0 Android/4.1.2 Release/12.6.2012 Browser/AppleWebKit534.30*/
const Device UA2714843 = {S81206, 93, { &P18595,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8190) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2796342 = {S81207, 158, { &P18223,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT22i Build/6.2.A.0.400) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3013126 = {S81208, 158, { &P18223,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT22i Build/6.2.A.1.100) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752964 = {S81209, 138, { &P20517,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CINK PEAX Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2754230 = {S81210, 136, { &P18243,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-P880 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2770630 = {S81211, 137, { &P18636,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS930 4G Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773482 = {S81212, 140, { &P23015,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; lenovo A820 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2971844 = {S81213, 164, { &P21020,&P18111,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; Micromax A116 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300*/
const Device UA2879488 = {S81214, 136, { &P21794,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9152 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893495 = {S81215, 129, { &P22228,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SM-T210 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2945904 = {S81216, 130, { &P22228,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SM-T210R Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2964355 = {S81217, 156, { &P15919,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-P3100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3011586 = {S81218, 156, { &P15919,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-P3100 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2993080 = {S81219, 139, { &P18297,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I8530 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997190 = {S81220, 131, { &P24413,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; NS-14T004 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3027683 = {S81221, 92, { &P11945,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233s) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3058299 = {S81222, 89, { &P11945,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; vi; gt-s5233s) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3062374 = {S81223, 160, { &P11747,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire V Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3081616 = {S81224, 134, { &P15877,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT910 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080459 = {S81225, 137, { &P22952,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F260S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094276 = {S81226, 127, { &P17686,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MZ601 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3111850 = {S81227, 127, { &P17686,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MZ601 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094975 = {S81228, 138, { &P19902,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R530M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083935 = {S81229, 138, { &P24732,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-S735C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA20143 = {S81230, 52, { &P17463,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-VE465/00.72 UP.Browser/7.2.7.5.616 (GUI) MMP/2.0*/
const Device UA20151 = {S81231, 52, { &P17463,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-VE465/00.72 UP.Browser/7.2.7.5.659 (GUI) MMP/2.0*/
const Device UA20152 = {S81232, 52, { &P17463,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-VE465/00.72 UP.Browser/7.2.7.5.541 (GUI) MMP/2.0*/
const Device UA22624 = {S81233, 50, { &P17463,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-VE465/1.0 UP.Browser/7.2.7.5.593 (GUI) MMP/2.0*/
const Device UA30972 = {S81234, 52, { &P17463,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-VE465/00.72 UP.Browser/7.2.7.5.643 (GUI) MMP/2.0*/
const Device UA32856 = {S81235, 57, { &P17463,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-VE465-parrot/1.0 UP.Browser/7.2.7.5.593 (GUI) MMP/2.0*/
const Device UA50633 = {S81236, 52, { &P17463,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-VE465/00.72 UP.Browser/7.2.7.5.599 (GUI) MMP/2.0*/
const Device UA67291 = {S81237, 142, { &P14796,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; 1000ET Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA72822 = {S81238, 143, { &P21270,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME722 Build/4.5.3-109_MS2-3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA75739 = {S81239, 136, { &P21270,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME722 Build/4.5.3-77) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1991333 = {S81240, 143, { &P21270,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME722 Build/4.5.3-109_MS2-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021715 = {S81241, 131, { &P21270,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME722 Build/4.5.3-109_MS2-3) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2030392 = {S81242, 136, { &P21270,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME722 Build/4.5.3-109_MS2-5) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2031799 = {S81243, 131, { &P21270,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ME722 Build/4.5.3-109_MS2-5) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA90085 = {S81244, 43, { &P16600,&P22784,&P17470,&P18092 }};	/*Useragent - MicromaxX600/MIDP2.0/CLDC1.1/Screen-240X400*/
const Device UA292616 = {S81245, 116, { &P18059,&P5208,&P4902,&P18092 }};	/*Useragent - POLARIS/6.1 (BREW 3.1.5; U; en-us; LG; POLARIS/6.1/WAP) Sprint LX101 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA661832 = {S81246, 142, { &P12797,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; SCH-i579 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022608 = {S81247, 130, { &P12797,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; SCH-i579 Build/GINGERBREAD) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2980370 = {S81248, 142, { &P12797,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i579 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA679372 = {S81249, 141, { &P21092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; ATP7483 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA742296 = {S81250, 144, { &P21092,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; Excellent8 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA715202 = {S81251, 136, { &P17346,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9B206 Safari/6533.18.5*/
const Device UA722894 = {S81252, 134, { &P17346,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9B176 Safari/6533.18.5*/
const Device UA2949935 = {S81253, 148, { &P17346,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA3098731 = {S81254, 148, { &P17346,&P18117,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/9B206 Safari/6533.18.5*/
const Device UA924292 = {S81255, 105, { &P15364,&P18145,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA1994381 = {S81256, 104, { &P15364,&P18145,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA2191389 = {S81257, 132, { &P19877,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T869 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2529118 = {S81258, 168, { &P18302,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MT25i Build/4.1.B.0.631) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2260187 = {S81259, 138, { &P18583,&P5043,&P24670,&P18092 }};	/*Useragent - Nokia110/2.0 (03.04) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia110) UCBrowser8.4.0.159/70/444/UCWEB Mobile*/
const Device UA2353713 = {S81260, 157, { &P18214,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2395625 = {S81261, 137, { &P21432,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE-N788 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2524231 = {S81262, 130, { &P21432,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE N788 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2406570 = {S81263, 169, { &P12798,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355*/
const Device UA2411139 = {S81264, 6, { &P18088,&P18588,&P18589,&P18092 }};	/*Useragent - Kindle*/
const Device UA2422806 = {S81265, 154, { &P18283,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556780 = {S81266, 154, { &P18283,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2594959 = {S81267, 154, { &P18283,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423841 = {S81268, 131, { &P20752,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AN10CG3 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2429445 = {S81269, 66, { &P17346,&P18432,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 (iPod4,1; iPhone OS 6.0.1; en_US) AppleWebKit/420+*/
const Device UA2435656 = {S81270, 64, { &P17346,&P18432,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 (iPod4,1; iPhone OS 6.0; en_US) AppleWebKit/420+*/
const Device UA2469229 = {S81271, 166, { &P15919,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P3100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2632723 = {S81272, 166, { &P15919,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-P3100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2478322 = {S81273, 125, { &P20364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; ELF-II Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2691182 = {S81274, 136, { &P20364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ELF-II Build/ICS.ELF-II.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2841491 = {S81275, 125, { &P20364,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ELF-II Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2485091 = {S81276, 155, { &P13035,&P5043,&P24675,&P18092 }};	/*Useragent - NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac2-02) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2505542 = {S81277, 138, { &P21894,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; pt-pt; AEG_AX410 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2558861 = {S81278, 95, { &P13979,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; Nokia6220c-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2580265 = {S81279, 175, { &P18652,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; Micromax A87 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3020559 = {S81280, 179, { &P18652,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A87 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2694635 = {S81281, 33, { &P15228,&P5043,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (NokiaC2-03/07.63)*/
const Device UA2600840 = {S81282, 133, { &P18017,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Ideapad K1 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2603905 = {S81283, 125, { &P18017,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; K1 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2662735 = {S81284, 102, { &P21274,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Intex_Aqua_Marvel) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2864142 = {S81285, 103, { &P21274,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Intex_Aqua_Marvel+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2674344 = {S81286, 102, { &P18218,&P21697,&P24633,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 610) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile*/
const Device UA2701123 = {S81287, 97, { &P20005,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A65) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2709099 = {S81288, 138, { &P21263,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TAB7250 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2753252 = {S81289, 137, { &P14877,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2768874 = {S81290, 93, { &P15649,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2900726 = {S81291, 94, { &P15649,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570I) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2769438 = {S81292, 135, { &P22760,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ThL W8 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782831 = {S81293, 156, { &P18621,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E610 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2868306 = {S81294, 156, { &P18621,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2863796 = {S81295, 71, { &P23263,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X322/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2945346 = {S81296, 157, { &P12968,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT26i Build/6.1.A.2.55) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2973386 = {S81297, 157, { &P12968,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LT26i Build/6.1.A.2.45) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3124121 = {S81298, 158, { &P12968,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LT26i Build/6.1.A.0.452) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3023139 = {S81299, 171, { &P18113,&P24702,&P19802,&P18091 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT912 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/spyder*/
const Device UA2957484 = {S81300, 171, { &P24688,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Spice Mi-515 Build/4.1.003.X.7295+; 540*960; CUCC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2962185 = {S81301, 156, { &P14848,&P5843,&P24678,&P18092 }};	/*Useragent - NokiaX3-00/5.0 (11.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax3-00) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3032826 = {S81302, 165, { &P18645,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3021663 = {S81303, 133, { &P20976,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; W100 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3016598 = {S81304, 139, { &P24425,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ONE TOUCH TAB 7 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3024053 = {S81305, 93, { &P13974,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5670) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3029908 = {S81306, 132, { &P22707,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Flytouch9 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3094290 = {S81307, 133, { &P22707,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Flytouch 9 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA7198 = {S81308, 78, { &P14450,&P5026,&P3410,&P18092 }};	/*Useragent - BlackBerry8830m/4.5.0.135 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/104*/
const Device UA24515 = {S81309, 84, { &P13661,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-708/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA30164 = {S81310, 85, { &P13661,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-708A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA58661 = {S81311, 170, { &P13661,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-708/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C, Alcatel-OT-708/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA63221 = {S81312, 97, { &P13661,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-708/1.0 model-orange Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA50997 = {S81313, 87, { &P17083,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_S200*/
const Device UA63315 = {S81314, 97, { &P14501,&P17483,&P3534,&P18092 }};	/*Useragent - LG-A155/V100 Obigo/Q05A MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA222785 = {S81315, 138, { &P19780,&P18110,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.127 Large Screen Safari/533.4 GoogleTV/b54202*/
const Device UA327811 = {S81316, 102, { &P15364,&P18346,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA577982 = {S81317, 114, { &P14682,&P5843,&P17347,&P18092 }};	/*Useragent - Nokia6303iclassic/5.0 (09.83) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303iclassic/UC Browser8.0.3.107/69/352*/
const Device UA572047 = {S81318, 31, { &P14983,&P5055,&P24665,&P18092 }};	/*Useragent - NokiaC7-00/UCWEB8.0.3.99/51/999*/
const Device UA772494 = {S81319, 43, { &P14983,&P5055,&P24665,&P18092 }};	/*Useragent - NokiaC7-00/UC Browser8.0.4.121/50/444/UCWEB*/
const Device UA651339 = {S81320, 73, { &P20166,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-C3332/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA788608 = {S81321, 77, { &P23545,&P17017,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; mauie800) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA883427 = {S81322, 153, { &P15364,&P7133,&P18728,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.3.0.X (Second Life Release; default skin) Safari/533.3*/
const Device UA1902478 = {S81323, 159, { &P13700,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; MOT-ME811/SHDWR_X7_2.67.0; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022327 = {S81324, 159, { &P13700,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-ME811/SHDWR_X7_2.71.0; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025154 = {S81325, 159, { &P13700,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; MOT-ME811/SHDWR_X7_2.74.0; 480*854; CTC/2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1955526 = {S81326, 130, { &P22017,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TAB467 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2745585 = {S81327, 130, { &P22017,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAB467 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2261439 = {S81328, 85, { &P15197,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-b5310) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA1989804 = {S81329, 134, { &P18314,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M032 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2034405 = {S81330, 71, { &P18283,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.4;Zh_cn;GT-I9300;720*1280;)UCWEB7.8.0.95/139/444*/
const Device UA2029020 = {S81331, 135, { &P21319,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOMO11 bird Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2443983 = {S81332, 135, { &P21319,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MOMO11 bird Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2505126 = {S81333, 135, { &P21319,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOMO11bird3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029661 = {S81334, 138, { &P16436,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I8150 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2262092 = {S81335, 33, { &P14540,&P5843,&P21724,&P18092 }};	/*Useragent - OneBrowser/3.0 (NokiaX2-01/08.63)*/
const Device UA2339090 = {S81336, 133, { &P20924,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; miTab AIR Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2357115 = {S81337, 39, { &P15795,&P5448,&P20764,&P18092 }};	/*Useragent - NOKIA-Lumia 710/UCWEB2.6.0.213/47/32079*/
const Device UA2384754 = {S81338, 135, { &P21972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; Tablet 723B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2467137 = {S81339, 66, { &P18449,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.7; zh-cn; ST27i; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2426065 = {S81340, 136, { &P20921,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; D2-711 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2430691 = {S81341, 138, { &P20508,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cynus T2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2460259 = {S81342, 53, { &P15364,&P6729,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows 95) Presto/2.12.388 Version/12.12*/
const Device UA2528955 = {S81343, 141, { &P13526,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; myTouch 4g Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2561596 = {S81344, 140, { &P13526,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; myTouch 4g Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997235 = {S81345, 141, { &P13526,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; myTouch 4G Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2562551 = {S81346, 129, { &P22814,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; PC906 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2604122 = {S81347, 145, { &P20007,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT886 Build/6.7.3-179_YIC-10) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2634739 = {S81348, 151, { &P15882,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-nz; GT-P5110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2863455 = {S81349, 155, { &P15882,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA2955297 = {S81350, 151, { &P15882,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA3092364 = {S81351, 155, { &P15882,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; GT-P5110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA2670602 = {S81352, 126, { &P16598,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A500 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2671089 = {S81353, 140, { &P21141,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Lenovo P770 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2674752 = {S81354, 146, { &P23102,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8681 Build/HuaweiU8681) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676845 = {S81355, 139, { &P15972,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One XL Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714706 = {S81356, 139, { &P15972,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One XL Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2752008 = {S81357, 143, { &P23105,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Starmobile Hit Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2752794 = {S81358, 88, { &P20529,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A89) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2754490 = {S81359, 140, { &P21402,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Lenovo S890 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755346 = {S81360, 137, { &P18641,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-M830 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2769106 = {S81361, 145, { &P17174,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MediaPad 10 FHD Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2773558 = {S81362, 151, { &P18125,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID4 Build/6.7.2-180_DR4-16_M2-37) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2861237 = {S81363, 155, { &P14207,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ADR6350 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937228 = {S81364, 156, { &P14207,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ADR6350 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2893022 = {S81365, 156, { &P21648,&P18066,&P18062,&P18092 }};	/*Useragent - GT-I9505 Linux/3.0.13 Android/4.2.2 Release/05.22.2013 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2988927 = {S81366, 179, { &P20729,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; Micromax A27 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3020724 = {S81367, 183, { &P20729,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A27 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2944990 = {S81368, 158, { &P12968,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT26i Build/6.2.B.0.200) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2961339 = {S81369, 85, { &P18436,&P21460,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (MSIE 9.0; Windows NT 6.3; WOW64; Trident/7.0; Touch; rv:11.0) like Gecko*/
const Device UA3056607 = {S81370, 90, { &P18436,&P21460,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (MSIE 9.0; Windows NT 6.3; Win64; x64; Trident/7.0; Touch; rv:11.0) like Gecko*/
const Device UA2974368 = {S81371, 139, { &P22636,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Aquaris Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3018882 = {S81372, 92, { &P12671,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia700) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3020805 = {S81373, 72, { &P18283,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile: SCH-I535; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA3078341 = {S81374, 72, { &P18283,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile: SGH-T999; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA3064722 = {S81375, 158, { &P12798,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT18i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3062408 = {S81376, 167, { &P12871,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3094800 = {S81377, 126, { &P18465,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KFOT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA11269 = {S81378, 84, { &P12511,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW880iv/R1JC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA17913 = {S81379, 143, { &P14531,&P17483,&P3534,&P18092 }};	/*Useragent - LG-KF310-Orange/V10b Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21857 = {S81380, 131, { &P14531,&P17483,&P3534,&P18092 }};	/*Useragent - LG-KF310 Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24725 = {S81381, 163, { &P14531,&P17483,&P3534,&P18092 }};	/*Useragent - Mozilla/4.0 (Vodafone/1.0/LG-KF310/V10b Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1)*/
const Device UA58617 = {S81382, 163, { &P14531,&P17483,&P3534,&P18092 }};	/*Useragent - Mozilla/4.0 (Vodafone/1.0/LG-KF310/V10c Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1)*/
const Device UA69267 = {S81383, 90, { &P14845,&P5448,&P4779,&P18092 }};	/*Useragent - HTC_Touch_HD_T8282 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA201735 = {S81384, 38, { &P18254,&P18576,&P18245,&P18092 }};	/*Useragent - Bing/2.2 CFNetwork/459 Darwin/10.0.0d3*/
const Device UA252423 = {S81385, 137, { &P15364,&P17817,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Ubuntu/10.10 Chromium/15.0.874.106 Chrome/15.0.874.106 Safari/535.2*/
const Device UA252483 = {S81386, 135, { &P15364,&P17817,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.2 (KHTML, like Gecko) Ubuntu/10.10 Chromium/15.0.874.106 Chrome/15.0.874.106 Safari/535.2*/
const Device UA355242 = {S81387, 135, { &P13868,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; HTC HD7 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025587 = {S81388, 135, { &P13868,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; HTC HD7 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2812407 = {S81389, 141, { &P13868,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; HTC HD7 Pro/+ Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA573003 = {S81390, 34, { &P14519,&P5129,&P24665,&P18092 }};	/*Useragent - NokiaN97 mini/UCWEB8.0.3.99/50/800*/
const Device UA634458 = {S81391, 46, { &P14519,&P5129,&P24665,&P18092 }};	/*Useragent - NokiaN97 mini/UC Browser8.0.4.121/50/444/UCWEB*/
const Device UA863151 = {S81392, 144, { &P21497,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ME632 Build/5.5.1Q-131_HKT-8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2399969 = {S81393, 145, { &P21497,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ME632 Build/5.5.1Q-131_HKT-16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2808290 = {S81394, 142, { &P21497,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ME632 Build/5.5.2Q-CHN-111) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1955159 = {S81395, 72, { &P21233,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.5;Zh_cn;TECNO T1;320*480;)UCWEB7.8.0.95/139/32929*/
const Device UA2228309 = {S81396, 140, { &P13025,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WT19a Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2501988 = {S81397, 145, { &P13025,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LiveWithWalkman Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2027370 = {S81398, 142, { &P21398,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S680 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028864 = {S81399, 128, { &P21398,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S680 Build/IML74K) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2336683 = {S81400, 133, { &P21398,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S680 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2028261 = {S81401, 145, { &P17954,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MID1125 Build/CaribbeanSunset_v1.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2455131 = {S81402, 158, { &P22043,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2619158 = {S81403, 147, { &P22043,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2220169 = {S81404, 132, { &P21395,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo A300/S100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2445504 = {S81405, 139, { &P21395,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A300 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2814939 = {S81406, 132, { &P21395,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A300 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2237675 = {S81407, 146, { &P21052,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2457649 = {S81408, 119, { &P14417,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5236) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2424732 = {S81409, 182, { &P15837,&P6909,&P24672,&P18092 }};	/*Useragent - NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-05) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2457445 = {S81410, 128, { &P18282,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A200 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2989706 = {S81411, 145, { &P18282,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; E-Boda Essential A200 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2467841 = {S81412, 136, { &P22671,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; eZee'Tab1001 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2560151 = {S81413, 141, { &P22658,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; Tablet726H Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2567503 = {S81414, 138, { &P20075,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xperia S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2580576 = {S81415, 93, { &P14433,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I9003) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2594738 = {S81416, 313, { &P23587,&P22857,&P18373,&P18092 }};	/*Useragent - HUAWEI_G6800 (MRE\2.3.00(25600) resolution\X chipset\MT6255 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\qwerty) MAUI/MAUI.11B.W12.12.MP.V2 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; sr-SR) Presto/2.5.28 Version/10.10*/
const Device UA2729101 = {S81417, 313, { &P23587,&P22857,&P18373,&P18092 }};	/*Useragent - HUAWEI_G6800 (MRE\2.3.00(25600) resolution\X chipset\MT6255 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\qwerty) MAUI/MAUI.11B.W12.12.MP.V2 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; hr-HR) Presto/2.5.28 Version/10.10*/
const Device UA3043536 = {S81418, 313, { &P23587,&P22857,&P18373,&P18092 }};	/*Useragent - HUAWEI_G6800 (MRE\2.3.00(25600) resolution\X chipset\MT6255 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\qwerty) MAUI/MAUI.11B.W12.12.MP.V2 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; ro-RO) Presto/2.5.28 Version/10.10*/
const Device UA3048577 = {S81419, 313, { &P23587,&P22857,&P18373,&P18092 }};	/*Useragent - HUAWEI_G6800 (MRE\2.3.00(25600) resolution\X chipset\MT6255 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\qwerty) MAUI/MAUI.11B.W12.12.MP.V2 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; es-ES) Presto/2.5.28 Version/10.10*/
const Device UA2671097 = {S81420, 136, { &P12496,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I897 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2780880 = {S81421, 137, { &P12496,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-I897 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676600 = {S81422, 126, { &P20556,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A701 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2718813 = {S81423, 88, { &P19779,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A1+) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2737633 = {S81424, 140, { &P18449,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST27i Build/6.2.A.0.400) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2880226 = {S81425, 152, { &P18648,&P5043,&P24677,&P18092 }};	/*Useragent - Nokia306/2.0 (05.93) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia306) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2901206 = {S81426, 131, { &P17595,&P17296,&P22778,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.59.8 (KHTML, like Gecko) Version/5.1 Safari/6534.59.8 Sleipnir/4.3.1*/
const Device UA2893039 = {S81427, 138, { &P20733,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-be; Lenco TAB-1030 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2974994 = {S81428, 137, { &P18002,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A760S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020972 = {S81429, 137, { &P20075,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Xperia S Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975798 = {S81430, 134, { &P22653,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTL22 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022407 = {S81431, 144, { &P20608,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Edison Build/1.1.11-1015 X-10:30) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3013353 = {S81432, 141, { &P12280,&P18117,&P24341,&P18092 }};	/*Useragent - MQQBrowser/44 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA3107456 = {S81433, 140, { &P12280,&P18117,&P24341,&P18092 }};	/*Useragent - MQQBrowser/44 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3*/
const Device UA3059060 = {S81434, 93, { &P18672,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5302) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3080452 = {S81435, 137, { &P22954,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E455f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081109 = {S81436, 125, { &P23001,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M97 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3111736 = {S81437, 152, { &P18506,&P5043,&P24680,&P18092 }};	/*Useragent - Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia305) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3130729 = {S81438, 151, { &P12605,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132878 = {S81439, 136, { &P23776,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGP321 Build/10.3.1.A.0.244) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA62335 = {S81440, 117, { &P14244,&P17484,&P3430,&P18092 }};	/*Useragent - SAMSUNG-SGH-L810-Vodafone/L810BUHG3 SHP/VPP/R5 Opera/9.5 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA11259 = {S81441, 98, { &P13578,&P5448,&P22272,&P18092 }};	/*Useragent - HTC-P3470-orange/PPC; 240x320; OpVer 29.115.2.732 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA53492 = {S81442, 98, { &P13578,&P5448,&P22272,&P18092 }};	/*Useragent - HTC-P3470-orange/PPC; 240x320; OpVer 29.113.2.731 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA62998 = {S81443, 70, { &P13578,&P5448,&P22272,&P18092 }};	/*Useragent - HTC_P3470 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA22870 = {S81444, 62, { &P13717,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-C276L/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA43680 = {S81445, 61, { &P13717,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-C276/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA24069 = {S81446, 72, { &P15660,&P5298,&P3499,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Teleca Q7; Brew 4.0.3; U; en) 240X400 LGE LG840*/
const Device UA24666 = {S81447, 65, { &P15732,&P17483,&P17602,&P18092 }};	/*Useragent - LG-S365/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA561401 = {S81448, 114, { &P14669,&P5843,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-00/04.80; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.6*/
const Device UA348430 = {S81449, 131, { &P17595,&P17296,&P22453,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.1 Chrome/15.0.900.2 Safari/535.2*/
const Device UA360156 = {S81450, 135, { &P21452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; DM011SH Build/S0006) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA624068 = {S81451, 135, { &P21452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; DM011SH Build/S0018) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024464 = {S81452, 135, { &P21452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; DM011SH Build/S0010) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2659676 = {S81453, 135, { &P21452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; DM011SH Build/S0020) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3012678 = {S81454, 135, { &P21452,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; DM011SH Build/S0022) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA626359 = {S81455, 46, { &P14188,&P5129,&P17338,&P18092 }};	/*Useragent - nokia5233/UC Browser7.9.0.102/69/352  3gpp-gba*/
const Device UA785668 = {S81456, 136, { &P21673,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; eGlide Prism Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2842670 = {S81457, 136, { &P21673,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; eGlide Prism Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA795206 = {S81458, 80, { &P14984,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5250)/UC Browser8.2.0.132/50/355/UCWEB Mobile*/
const Device UA2311371 = {S81459, 31, { &P14984,&P5129,&P24667,&P18092 }};	/*Useragent - Nokia5250/UCWEB8.2.0.116/50/800*/
const Device UA845326 = {S81460, 71, { &P15364,&P7133,&P18171,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:13.0) Gecko/X Firefox/13.0 slurp*/
const Device UA904790 = {S81461, 79, { &P15364,&P7133,&P18171,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:13.0) Gecko/X Firefox/13.0.1 slurp slurp*/
const Device UA907779 = {S81462, 73, { &P15364,&P7133,&P18171,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:13.0) Gecko/X Firefox/13.0.1 slurp*/
const Device UA908162 = {S81463, 66, { &P15364,&P7133,&P18171,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:13.0) Gecko/X Firefox/13.0.1 slurp*/
const Device UA1985367 = {S81464, 64, { &P15364,&P7133,&P18171,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:13.0) Gecko/X Firefox/13.0 slurp*/
const Device UA899377 = {S81465, 134, { &P21179,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MOMO8 bird Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1995696 = {S81466, 134, { &P21179,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MOMO8 bird Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1966757 = {S81467, 486, { &P15364,&P18110,&P22398,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.12) Gecko/X Firefox/3.6.12","Mozilla/5.0 (Windows; U; Windows NT 5.1; pl-PL; rv:1.8.1.24pre) Gecko/X K-Meleon/1.5.4","Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/540.0 (KHTML,like Gecko) Chrome/9.1.0.0 Safari/540.0","Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Comodo_Dragon/4.1.1.11 Chrome/4.1.249.1042 Safari/532.5","Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US; rv:1.9.0.*/
const Device UA2640933 = {S81468, 135, { &P15364,&P18110,&P22398,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Comodo_Dragon/4.1.1.11 Chrome/4.1.249.1042 Safari/532.5*/
const Device UA2023250 = {S81469, 154, { &P15184,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P920 Build/GRJ90; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2000094 = {S81470, 65, { &P12280,&P18117,&P22070,&P18092 }};	/*Useragent - Instagram 3.0.0 (iPhone; iPhone OS 5.1.1; en_US) AppleWebKit/420+*/
const Device UA2200395 = {S81471, 63, { &P12280,&P18117,&P22070,&P18092 }};	/*Useragent - Instagram 3.0.0 (iPhone; iPhone OS 5.1; en_US) AppleWebKit/420+*/
const Device UA2029941 = {S81472, 132, { &P17168,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7310 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2610671 = {S81473, 132, { &P17168,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7310 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2613476 = {S81474, 77, { &P17168,&P18111,&P18112,&P18092 }};	/*Useragent - AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; GT-P7310 Build/JRO03H)*/
const Device UA2616529 = {S81475, 145, { &P17168,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7310 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.0.9 Mobile Safari/534.30*/
const Device UA2616859 = {S81476, 145, { &P17168,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7310 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2617344 = {S81477, 132, { &P17168,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7310 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2618070 = {S81478, 61, { &P17168,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P7310 Build/JZO54K)*/
const Device UA2032597 = {S81479, 138, { &P18021,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A770K Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2203468 = {S81480, 81, { &P17976,&P5096,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2237126 = {S81481, 83, { &P17976,&P5096,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2.2; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2237667 = {S81482, 83, { &P17976,&P5096,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2.1; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2267491 = {S81483, 89, { &P15364,&P17285,&P21957,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2a1pre) Gecko/X Minefield/3.2a1pre*/
const Device UA2337903 = {S81484, 144, { &P21344,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; el-gr; novaPad 100 C124 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2342575 = {S81485, 143, { &P21994,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Teclast P85??(A9D3) Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2645250 = {S81486, 154, { &P21994,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; P85(R8A1) Build/IMM76D; 1024*720) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2827347 = {S81487, 72, { &P21994,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Teclast P85??(A9D3) Build/IMM76D)*/
const Device UA2425961 = {S81488, 43, { &P17346,&P17779,&P18431,&P18092 }};	/*Useragent - server-bag [iPhone OS,6.0.1,10A523,iPod4,1]*/
const Device UA2595260 = {S81489, 22, { &P17346,&P17779,&P18431,&P18092 }};	/*Useragent - iPod4,1/6.0.1 (10A523)*/
const Device UA2827573 = {S81490, 20, { &P17346,&P17779,&P18431,&P18092 }};	/*Useragent - iPod4,1/6.0 (10A403)*/
const Device UA2695388 = {S81491, 140, { &P21653,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; POV_TAB-PROTAB25 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2580617 = {S81492, 137, { &P23806,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; S18 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2675894 = {S81493, 150, { &P19762,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI MT1-U06 Build/HuaweiMT1-U06) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676873 = {S81494, 145, { &P18114,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Transformer Prime TF201 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708964 = {S81495, 132, { &P18017,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Ideapad K1 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2713175 = {S81496, 124, { &P18017,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; K1 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2734939 = {S81497, 142, { &P23828,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; Q-Smart S16 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2798714 = {S81498, 156, { &P18304,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9070 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2753695 = {S81499, 133, { &P21963,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Q800 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2756851 = {S81500, 138, { &P22935,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9080L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769355 = {S81501, 138, { &P22592,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717R Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769772 = {S81502, 145, { &P18635,&P18598,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2851838 = {S81503, 58, { &P15364,&P22308,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; WinNT3.51; Trident/3.0)*/
const Device UA2866627 = {S81504, 157, { &P23742,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; O+ 8.15 Android Build/Oplus8.15_Android) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2953470 = {S81505, 133, { &P23896,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; S5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958999 = {S81506, 133, { &P23896,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; S5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994542 = {S81507, 139, { &P23896,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; K-Touch S5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099772 = {S81508, 138, { &P23896,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; zh-tw; iNO S5 Build/1.0.492_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2960038 = {S81509, 134, { &P21312,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; A19 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2960299 = {S81510, 139, { &P24587,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I869 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3074891 = {S81511, 130, { &P24587,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I869 Build/JZO54K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3039311 = {S81512, 159, { &P18736,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3021248 = {S81513, 133, { &P24077,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; H120 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3063035 = {S81514, 159, { &P12798,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LT18i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3035896 = {S81515, 105, { &P24617,&P17484,&P17369,&P18092 }};	/*Useragent - SAMSUNG-GT-E2200 Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.32208/30.3697; U; en) Presto/2.8.119 Version/11.10*/
const Device UA3083230 = {S81516, 137, { &P20508,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cynus T2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA7066 = {S81517, 154, { &P12495,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE66-2/102.07.81; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA14824 = {S81518, 77, { &P13516,&P5208,&P4521,&P18092 }};	/*Useragent - Mozilla/4.0(compatible; Polaris 6.2; Brew 3.1.5; U; en) 240X400 LGE LG-AX8575*/
const Device UA49828 = {S81519, 138, { &P13092,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; in-id; ZTE-U V852 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA62565 = {S81520, 122, { &P14422,&P17483,&P3459,&P18092 }};	/*Useragent - LG-P520/V100 Obigo/Q7.3 MMS/LG-MMS-V1.1/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA156552 = {S81521, 146, { &P16744,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SHW-M130L Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018217 = {S81522, 68, { &P16744,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SHW-M130L Build/ECLAIR)*/
const Device UA855228 = {S81523, 137, { &P21163,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; ViewPad 10S Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458989 = {S81524, 136, { &P21609,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-es; MID7105SR Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA459120 = {S81525, 60, { &P21609,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; MID7105SR Build/FRF85B)*/
const Device UA644712 = {S81526, 109, { &P15364,&P18145,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; FreeBSD i386; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/4.0.207.0 Safari/532.0*/
const Device UA2221470 = {S81527, 139, { &P21211,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; CELKON_A_90 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2232804 = {S81528, 137, { &P14123,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P970 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2344703 = {S81529, 137, { &P22141,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Andi_4.3a Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2391409 = {S81530, 98, { &P12137,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s5620) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2384235 = {S81531, 157, { &P15575,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2594213 = {S81532, 157, { &P15575,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652880 = {S81533, 157, { &P15575,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2746393 = {S81534, 158, { &P15575,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2422412 = {S81535, 88, { &P23020,&P22835,&P17470,&P18092 }};	/*Useragent - SpiceM5365/MTK Release/01.01.2012 Browser/wap2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2422560 = {S81536, 135, { &P20862,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IRULU-AL101 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2469226 = {S81537, 176, { &P14225,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2481563 = {S81538, 141, { &P20355,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; STARNAUTE II Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2487851 = {S81539, 80, { &P15364,&P18140,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux x86_64; Edition Linux Mint) Presto/2.12.388 Version/12.13*/
const Device UA2560660 = {S81540, 135, { &P21823,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTX21 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2583382 = {S81541, 112, { &P18113,&P17979,&P17470,&P18092 }};	/*Useragent - stagefright/1.2 (Linux;Android ; Release/4.1.2, Manufacturer/motorola, Model/DROID RAZR, Build/9.8.2O-72_VZW-16)*/
const Device UA2583784 = {S81542, 137, { &P12496,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I897 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2666884 = {S81543, 138, { &P12496,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-I897 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2600449 = {S81544, 137, { &P21187,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MPDC99 BT IPS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2668357 = {S81545, 152, { &P20868,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M040 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2662384 = {S81546, 89, { &P20169,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; N320) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2663359 = {S81547, 88, { &P20120,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A45) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2752792 = {S81548, 136, { &P21829,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; F-02E Build/V17R48A) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2670607 = {S81549, 126, { &P20269,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A211 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2672274 = {S81550, 142, { &P19843,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE Blade III Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707327 = {S81551, 138, { &P19843,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BLADE III Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675857 = {S81552, 139, { &P18453,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT25i Build/9.0.D.0.484) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2700421 = {S81553, 142, { &P23296,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Lenovo A590 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2714571 = {S81554, 73, { &P15364,&P17285,&P22317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:21.0) Gecko/X Firefox/21.0 SeaMonkey/2.18*/
const Device UA2717874 = {S81555, 158, { &P11950,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; X10i Build/3.0.1.G.0.75) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2736513 = {S81556, 156, { &P20515,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; A54 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2738147 = {S81557, 94, { &P13648,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaE7-00) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2739608 = {S81558, 153, { &P15575,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2956670 = {S81559, 154, { &P15575,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100G Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2752894 = {S81560, 140, { &P20180,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XOLO_A800 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2767441 = {S81561, 139, { &P22917,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Ktouch_A11 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2784889 = {S81562, 159, { &P15397,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST18i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2772863 = {S81563, 147, { &P23204,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Tablet Rebel 8GB Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2782401 = {S81564, 139, { &P18088,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Skyfire2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2793716 = {S81565, 135, { &P15877,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT910 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957106 = {S81566, 58, { &P15877,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; XT910 Build/JDQ39E)*/
const Device UA2801271 = {S81567, 137, { &P22958,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-US780 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2866734 = {S81568, 145, { &P23248,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GIONEE_Gpad_G2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879920 = {S81569, 133, { &P24159,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; M-MP876S2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2892230 = {S81570, 132, { &P23413,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PLT7223G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2962030 = {S81571, 156, { &P15837,&P6909,&P24678,&P18092 }};	/*Useragent - NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-05) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2967547 = {S81572, 159, { &P20076,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26w Build/6.2.B.0.200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3021230 = {S81573, 127, { &P17686,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MZ601 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022273 = {S81574, 127, { &P17686,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MZ601 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3024198 = {S81575, 88, { &P19779,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A1+) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3083363 = {S81576, 138, { &P20244,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; bq Elcano Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094373 = {S81577, 146, { &P23198,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI H868C Build/HuaweiH868C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA9420 = {S81578, 141, { &P16957,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; XCD35 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA24130 = {S81579, 84, { &P12365,&P17603,&P3534,&P18092 }};	/*Useragent - PANTECH-C530/ITUS10062008 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA30902 = {S81580, 75, { &P12365,&P17603,&P3534,&P18092 }};	/*Useragent - PANTECH-C530/R01 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA460439 = {S81581, 96, { &P12365,&P17603,&P3534,&P18092 }};	/*Useragent - PANTECH-C530/ITUSX Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1 Novarra-Vision/8.0*/
const Device UA24613 = {S81582, 90, { &P15720,&P5448,&P22270,&P18092 }};	/*Useragent - HPiPAQ910/1.0/Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) (PPC; 320x240)*/
const Device UA87592 = {S81583, 136, { &P15720,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) HPiPAQ910/1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 320x240)*/
const Device UA53905 = {S81584, 29, { &P20211,&P17483,&P3459,&P18092 }};	/*Useragent - LGE-GW370B Obigo/Q7.3 MMP/2.0*/
const Device UA64041 = {S81585, 52, { &P14567,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-WX404/00.72 UP.Browser/7.2.7.5.705 (GUI) MMP/2.0*/
const Device UA64869 = {S81586, 72, { &P14712,&P5208,&P3499,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Teleca Q7; Brew 3.1.5; U; en) 240X400 LGE LG700*/
const Device UA113633 = {S81587, 89, { &P17012,&P5448,&P3430,&P18092 }};	/*Useragent - Opera/9.5 (Microsoft Windows; Windows CE; Opera Mobi/9.5; U; en) VZW:SCH-i770 PPC 320x320*/
const Device UA253958 = {S81588, 120, { &P13296,&P5448,&P4779,&P18092 }};	/*Useragent - SonyEricssonX1i/R3AA Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA1895651 = {S81589, 75, { &P13296,&P5448,&P4779,&P18092 }};	/*Useragent - X1i Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA367631 = {S81590, 123, { &P15364,&P7133,&P24201,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1 QQBrowser/6.10.11509.201*/
const Device UA381287 = {S81591, 165, { &P15364,&P7133,&P24201,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C) QQBrowser/6.10.11509.201*/
const Device UA584028 = {S81592, 186, { &P15364,&P7133,&P24201,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.10.11509.201*/
const Device UA1942590 = {S81593, 203, { &P15364,&P7133,&P24201,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E) QQBrowser/6.10.11509.201*/
const Device UA1944627 = {S81594, 201, { &P15364,&P7133,&P24201,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; Tablet PC 2.0) QQBrowser/6.10.11509.201*/
const Device UA2310062 = {S81595, 95, { &P15364,&P7133,&P24201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0) QQBrowser/6.10.11509.201*/
const Device UA2310541 = {S81596, 188, { &P15364,&P7133,&P24201,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; .NET4.0E; MDDC) QQBrowser/6.10.11509.201*/
const Device UA2312804 = {S81597, 125, { &P15364,&P7133,&P24201,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.10.11528.201*/
const Device UA578644 = {S81598, 116, { &P12125,&P5204,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia6500s-1/10.00; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA707219 = {S81599, 47, { &P14983,&P5055,&P17347,&P18092 }};	/*Useragent - nokiac7-00/UC Browser8.0.3.107/69/444  3gpp-gba*/
const Device UA707383 = {S81600, 122, { &P21219,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android 2.1-update1;pandigital6gs)AppleWebKit/530.17(KHTML,like Gecko)Version/4.0 Mobile Safari/530.17*/
const Device UA2162531 = {S81601, 79, { &P12456,&P5890,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokian73-1) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA1966969 = {S81602, 145, { &P12258,&P17483,&P24668,&P18092 }};	/*Useragent - LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-in; lg-gs290) UCBrowser8.3.0.154/69/405/UCWEB Mobile*/
const Device UA1993866 = {S81603, 175, { &P15798,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA1885770 = {S81604, 31, { &P15364,&P22832,&P18246,&P18092 }};	/*Useragent - Videocon_V1545_Maui Wap Browser*/
const Device UA2651165 = {S81605, 31, { &P15364,&P22832,&P18246,&P18092 }};	/*Useragent - Videocon V1511_Maui Wap Browser*/
const Device UA2986533 = {S81606, 31, { &P15364,&P22832,&P18246,&P18092 }};	/*Useragent - Videocon_V1541_Maui Wap Browser*/
const Device UA3075204 = {S81607, 31, { &P15364,&P22832,&P18246,&P18092 }};	/*Useragent - Videocon_V1521_Maui Wap Browser*/
const Device UA1984935 = {S81608, 73, { &P15364,&P7127,&P22340,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:11.0) Gecko/X Firefox/11.0 CometBird/11.0*/
const Device UA2027319 = {S81609, 91, { &P23938,&P5629,&P15760,&P18092 }};	/*Useragent - CoolPad8022_CMCC_TD/1.0 Linux/2.6.35 Android/2.3 Release/5.10.2012 Browser/AppleWebkit533.1*/
const Device UA2041164 = {S81610, 80, { &P17595,&P17295,&P18523,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:15.0) Gecko/X Thunderbird/15.0.1*/
const Device UA2236040 = {S81611, 136, { &P21422,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KATBL10A08DA Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2331353 = {S81612, 141, { &P15539,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Glacier Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2413540 = {S81613, 141, { &P15539,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Glacier Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2431940 = {S81614, 223, { &P12568,&P5096,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; GT-B7510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2405008 = {S81615, 53, { &P15364,&P6729,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows 95) Presto/2.12.388 Version/12.11*/
const Device UA2415338 = {S81616, 133, { &P22691,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; M-MP1010i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2418525 = {S81617, 107, { &P17595,&P18207,&P22504,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en; rv:1.9.0.19) Gecko/X Camino/2.0.9 (like Firefox/3.0.19)*/
const Device UA2461247 = {S81618, 167, { &P12968,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT26i Build/6.1.A.2.45) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2483913 = {S81619, 142, { &P21796,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Philips W336 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2485243 = {S81620, 131, { &P23379,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TOUCH97 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2653007 = {S81621, 145, { &P17686,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MZ601 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2509067 = {S81622, 171, { &P14433,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2660354 = {S81623, 171, { &P14433,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2554018 = {S81624, 139, { &P18072,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2555825 = {S81625, 139, { &P18072,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2934504 = {S81626, 139, { &P18072,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2581304 = {S81627, 95, { &P13800,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA5630d-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2559013 = {S81628, 88, { &P19996,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A15) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2564828 = {S81629, 161, { &P19996,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A15 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2565217 = {S81630, 166, { &P19847,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-S6012 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3057827 = {S81631, 170, { &P19847,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S6012 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2579797 = {S81632, 131, { &P23896,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; S5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2583344 = {S81633, 131, { &P20154,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5770D Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2597450 = {S81634, 154, { &P23718,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; O+_8.12_Android Build/OplusOplus_A75) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666533 = {S81635, 133, { &P20729,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Videocon A27 Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2667579 = {S81636, 134, { &P23182,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; PMP7280C3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2670745 = {S81637, 134, { &P20480,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JY-G3 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2737234 = {S81638, 150, { &P23739,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Hyper S250 Build/ICS_IMM76I_B479) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747189 = {S81639, 40, { &P14682,&P5843,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia6303iclassic/10.80)*/
const Device UA2753215 = {S81640, 140, { &P18662,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC J Z321e Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769773 = {S81641, 138, { &P18682,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-M950 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2773709 = {S81642, 145, { &P18556,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD+ Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2893363 = {S81643, 269, { &P18088,&P22791,&P4780,&P18092 }};	/*Useragent - HONGYU50_COSMOS_11B_HW (MRE\3.0.00(2500) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) CHERRY_W5_PH_C722L_CQ01_X Release/2013.05.02 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2937595 = {S81644, 270, { &P18088,&P22791,&P4780,&P18092 }};	/*Useragent - HONGYU50_COSMOS_11B_HW (MRE\3.0.00(2500) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) Cherry_W12_Ph_C722L_CQ01_X Release/2013.04.26 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA3050067 = {S81645, 412, { &P18088,&P22791,&P4780,&P18092 }};	/*Useragent - HONGYU50_COSMOS_11B_HW (MRE/3.0.00(2500);MAUI/Y_W20_C737E_S06_X;BDATE/2013/05/04 11:46;LCD/X;CHIP/MT6250;KEY/Reduced;TOUCH/1;CAMERA/0;SENSOR/0;DEV/HONGYU50_COSMOS_11B_HW;WAP Browser/MAUI (HTTP PGDL;HTTPS);GMOBI/001;MBOUNCE/002;MOMAGIC/003;INDEX/004;SPICEI2I/005;GAMELO) CHERRY_W20_C737E_S06_X Release/2013.05.04 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA3032695 = {S81646, 156, { &P12788,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PG86100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2965984 = {S81647, 98, { &P19862,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax_A110) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA2997663 = {S81648, 139, { &P23518,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1;zh-cn; ZTE V987 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30;*/
const Device UA3048480 = {S81649, 139, { &P23518,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; ZTE V987 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3021745 = {S81650, 95, { &P23122,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; Karbonn_A6) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3033066 = {S81651, 157, { &P21628,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3035724 = {S81652, 155, { &P23163,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; SCH-I959 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3062528 = {S81653, 171, { &P12972,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-T989 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/hercules*/
const Device UA3080162 = {S81654, 171, { &P12972,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-T989 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/hercules*/
const Device UA3127570 = {S81655, 171, { &P12972,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-T989 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/hercules*/
const Device UA3081471 = {S81656, 137, { &P21028,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Smartfren Tab 7 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA1603 = {S81657, 93, { &P17170,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-VF527/1.0/EVN_PS MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.416 (GUI)*/
const Device UA1602 = {S81658, 93, { &P17170,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-VF527/1.0/EVN_PW MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.416 (GUI)*/
const Device UA17203 = {S81659, 93, { &P17170,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-VF527/1.0/EVN_PV MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.416 (GUI)*/
const Device UA24663 = {S81660, 93, { &P17170,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-VF527/1.0/EVN_PK MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.416 (GUI)*/
const Device UA30703 = {S81661, 93, { &P17170,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-VF527/1.0/EVN_PI MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.410 (GUI)*/
const Device UA55038 = {S81662, 93, { &P17170,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-VF527/1.0/E5N_PN MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.413 (GUI)*/
const Device UA55039 = {S81663, 93, { &P17170,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-VF527/1.0/EVN_PL MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.416 (GUI)*/
const Device UA62276 = {S81664, 93, { &P17170,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-VF527/1.0/E5N_PO MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.416 (GUI)*/
const Device UA63511 = {S81665, 93, { &P17170,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-VF527/1.0/EVN_PH MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.410 (GUI)*/
const Device UA64704 = {S81666, 93, { &P17170,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-VF527/1.0/EVN_PJ MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.410 (GUI)*/
const Device UA44312 = {S81667, 61, { &P11651,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia6170/2.0 (03.24) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23001 = {S81668, 173, { &P11971,&P17485,&P4203,&P18092 }};	/*Useragent - SonyEricssonK330/R1DA002 TelecaBrowser/Q04C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1,SonyEricssonK330/R1DA002 TelecaBrowser/Q04C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63764 = {S81669, 86, { &P11971,&P17485,&P4203,&P18092 }};	/*Useragent - SonyEricssonK330/R1DA002 TelecaBrowser/Q04C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88762 = {S81670, 87, { &P11971,&P17485,&P4203,&P18092 }};	/*Useragent - SonyEricssonK330a/R1CE002 TelecaBrowser/Q04C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA14855 = {S81671, 70, { &P13518,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-R100/1.0 ACS-NF/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA182169 = {S81672, 59, { &P16897,&P22784,&P17470,&P18092 }};	/*Useragent - MicromaxX228+/3.83; Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA646549 = {S81673, 126, { &P15364,&P5096,&P22237,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Android 2.2;en-us;i7 Build/IML74K) AppleWebKit/533.16(KHTML, like Gecko) Chrome/11.0.696.34 Safari/533.16*/
const Device UA742522 = {S81674, 93, { &P15364,&P17813,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.3) Gecko/X Ubuntu/8.04 (hardy) Firefox/3.5.3*/
const Device UA822180 = {S81675, 83, { &P15364,&P18144,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; OpenBSD i386; en-US; rv:1.7.10) Gecko/X (No IDN) Firefox/1.0.6*/
const Device UA1979451 = {S81676, 159, { &P16092,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB870 Build/4.5.1A-DTN-130; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2340153 = {S81677, 160, { &P16092,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB870 Build/ParanoidAndroid; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1901540 = {S81678, 60, { &P18490,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxQ22/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA2025124 = {S81679, 67, { &P15397,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.4;Zh_cn;ST18i;480*854;)UCWEB7.8.0.95/139/444*/
const Device UA2369312 = {S81680, 133, { &P17595,&P18207,&P19987,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Version/5.1 Safari/7534.51.22 Sleipnir/3.8.3*/
const Device UA2425984 = {S81681, 135, { &P17595,&P18207,&P19987,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Version/6.0.2 Safari/8536.26.17 Sleipnir/3.8.5*/
const Device UA2340184 = {S81682, 156, { &P23983,&P18066,&P18062,&P18092 }};	/*Useragent - ELIYA S1 Linux/3.0.13 Android/4.0.4 Release/07.20.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2385855 = {S81683, 182, { &P14714,&P5843,&P24672,&P18092 }};	/*Useragent - NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax3-02) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile*/
const Device UA2382458 = {S81684, 96, { &P18643,&P5043,&P17347,&P18092 }};	/*Useragent - Nokia311/5.0 (03.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia311/UC Browser8.0.3.107/69/444*/
const Device UA2386105 = {S81685, 132, { &P24100,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; CAL21 Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2422979 = {S81686, 134, { &P21508,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GEM10313BK Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2424457 = {S81687, 132, { &P20733,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB-1030 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2504752 = {S81688, 132, { &P20733,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TAB-1030 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2469287 = {S81689, 171, { &P13452,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2621125 = {S81690, 172, { &P13452,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-P1000N Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.0.211/145/352*/
const Device UA2621220 = {S81691, 171, { &P13452,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355*/
const Device UA2624860 = {S81692, 172, { &P13452,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-P1000T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2626154 = {S81693, 166, { &P13452,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; GT-P1000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2626240 = {S81694, 172, { &P13452,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-P1000M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2626363 = {S81695, 171, { &P13452,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2637596 = {S81696, 171, { &P13452,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/444*/
const Device UA2704448 = {S81697, 156, { &P18635,&P18598,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2567218 = {S81698, 141, { &P18662,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC J Z321e Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2581183 = {S81699, 156, { &P14693,&P5843,&P24676,&P18092 }};	/*Useragent - NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac1-02) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2597341 = {S81700, 134, { &P15897,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; XT530 Build/V4.480) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3099773 = {S81701, 134, { &P15897,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-us; XT531 Build/V4.480) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2601661 = {S81702, 161, { &P15575,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-i9100 Build/JDQ39; Carbon-Carbon-JB-v1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2793430 = {S81703, 161, { &P15575,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-i9100 Build/JDQ39; Carbon-CARBON-JB-v1.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937224 = {S81704, 162, { &P15575,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-i9100 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2603598 = {S81705, 146, { &P21888,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; WALTON Primo_R1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2663330 = {S81706, 146, { &P21888,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; WALTON_Primo_R1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2633937 = {S81707, 151, { &P15932,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3110 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2636276 = {S81708, 151, { &P15932,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P3110 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2784837 = {S81709, 150, { &P15932,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P3110 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799840 = {S81710, 150, { &P15932,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3110 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2702877 = {S81711, 130, { &P23816,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 796TPC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2712358 = {S81712, 140, { &P12280,&P17146,&P22589,&P18092 }};	/*Useragent - MQQBrowser/42 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA2734125 = {S81713, 141, { &P12280,&P17146,&P22589,&P18092 }};	/*Useragent - MQQBrowser/42 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3*/
const Device UA2752491 = {S81714, 141, { &P12280,&P17146,&P22589,&P18092 }};	/*Useragent - MQQBrowser/421 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA2781852 = {S81715, 142, { &P12280,&P17146,&P22589,&P18092 }};	/*Useragent - MQQBrowser/421 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3*/
const Device UA2713118 = {S81716, 152, { &P20749,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735160 = {S81717, 140, { &P22738,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT916 Build/2_26C_2014) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865893 = {S81718, 140, { &P22738,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT916 Build/2_26D_2014) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998828 = {S81719, 62, { &P22738,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; XT916 Build/2_26D_2014)*/
const Device UA2754191 = {S81720, 126, { &P18733,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M532 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2769383 = {S81721, 137, { &P21385,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F240S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2872747 = {S81722, 133, { &P23278,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; TID-80042 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2892418 = {S81723, 142, { &P24096,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Orange Kivo Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2898919 = {S81724, 154, { &P18635,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; Nexus 4 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2970756 = {S81725, 93, { &P11880,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA5320) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3032710 = {S81726, 146, { &P18466,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2972129 = {S81727, 135, { &P24076,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; PAD720 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3026143 = {S81728, 161, { &P16436,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I8150 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA41838 = {S81729, 119, { &P14433,&P5096,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.2.1;Zh_cn;GT-I9003;480*800;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA650470 = {S81730, 70, { &P14433,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2.1;Zh_cn;GT-I9003;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA42356 = {S81731, 119, { &P14186,&P5096,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.2.1;Zh_cn;GT-S5830;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA576305 = {S81732, 70, { &P14186,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2.1;Zh_cn;GT-S5830;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA64144 = {S81733, 39, { &P14576,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GM200 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA119280 = {S81734, 102, { &P17012,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) VZW:SCH-i770 PPC 320x320*/
const Device UA147472 = {S81735, 9, { &P13706,&P5129,&P4030,&P18092 }};	/*Useragent - Nokia5235*/
const Device UA367732 = {S81736, 123, { &P15364,&P5690,&P24201,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1 QQBrowser/6.10.11509.201*/
const Device UA367756 = {S81737, 88, { &P15364,&P5690,&P24201,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0) QQBrowser/6.10.11509.201*/
const Device UA367860 = {S81738, 80, { &P15364,&P5690,&P24201,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) QQBrowser/6.10.11509.201*/
const Device UA585232 = {S81739, 75, { &P15364,&P5690,&P24201,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1) QQBrowser/6.10.11509.201*/
const Device UA760473 = {S81740, 92, { &P17595,&P17296,&P19773,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.1.15) Gecko/X SeaMonkey/2.0.10*/
const Device UA583605 = {S81741, 114, { &P14848,&P5843,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-00/08.54; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA672646 = {S81742, 131, { &P21334,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; vi-vn; A8 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020833 = {S81743, 131, { &P21334,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; th-th; A8 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA912204 = {S81744, 135, { &P13505,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MB525 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2520695 = {S81745, 136, { &P13505,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MB525+ Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1985120 = {S81746, 153, { &P16553,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-T959V Build/VUVKJ6; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032227 = {S81747, 143, { &P22696,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ru-ru; Fly_IQ280 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033744 = {S81748, 134, { &P21226,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; E860 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2160932 = {S81749, 62, { &P11684,&P5890,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 (Nokia5800XpressMusic;SymbianOS/9.1 Series60/3.0)*/
const Device UA2333497 = {S81750, 135, { &P16598,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Picasso Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2431682 = {S81751, 229, { &P16356,&P5629,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2428365 = {S81752, 118, { &P15206,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE90) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile*/
const Device UA2426121 = {S81753, 142, { &P20133,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; 201HW Build/HuaweiU9201L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2454406 = {S81754, 127, { &P17595,&P18207,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_8_2; en-us) AppleWebKit/531.22.7 (KHTML, like Gecko) Version/4.0.5 Safari/531.22.7*/
const Device UA2529828 = {S81755, 151, { &P23049,&P5629,&P15760,&P18092 }};	/*Useragent - Spice Mi-315 Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Spice Build/SpiceMi315) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2539588 = {S81756, 114, { &P15364,&P7133,&P19811,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22 slurp*/
const Device UA2567361 = {S81757, 115, { &P15364,&P7133,&P19811,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22 slurp*/
const Device UA2584902 = {S81758, 115, { &P15364,&P7133,&P19811,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22 slurp*/
const Device UA2566951 = {S81759, 136, { &P20119,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LIFETAB_S9714 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2608190 = {S81760, 80, { &P17337,&P17979,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.4.4) (GT-P7500;samsung;GT-P7500;3.2/HTJ85B)*/
const Device UA2665813 = {S81761, 138, { &P23166,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; FABLET F3 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2733140 = {S81762, 154, { &P22611,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-S7572 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2794112 = {S81763, 86, { &P22611,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-S7572 Build/JZO54K.S7572ZCAMB2;480*800)*/
const Device UA2996105 = {S81764, 86, { &P22611,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-S7572 Build/JZO54K.S7572ZCAMD2;480*800)*/
const Device UA2733720 = {S81765, 178, { &P16292,&P18598,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (Galaxy Nexus, 42, 1.9.1)*/
const Device UA2863087 = {S81766, 179, { &P16292,&P18598,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Galaxy Nexus Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (Galaxy Nexus, 42, 1.9.1)*/
const Device UA2774283 = {S81767, 149, { &P18454,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGPT12 Build/TJS0166) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2937807 = {S81768, 149, { &P18454,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGPT13 Build/TJS0166) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2755553 = {S81769, 141, { &P21374,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 101 Titanium Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2799248 = {S81770, 136, { &P22748,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E425 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867814 = {S81771, 137, { &P22748,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E425f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3088711 = {S81772, 137, { &P22748,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E425g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2800660 = {S81773, 152, { &P19771,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035A Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867948 = {S81774, 152, { &P19771,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893420 = {S81775, 152, { &P19771,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 5035X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2901220 = {S81776, 156, { &P22609,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6312 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2958298 = {S81777, 137, { &P24114,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E440 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3130935 = {S81778, 136, { &P20146,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9082 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131411 = {S81779, 144, { &P18614,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT890 Build/9.8.2I-50_SMI-26) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA64654 = {S81780, 111, { &P13748,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-M7603/1.0 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA63291 = {S81781, 52, { &P14496,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KS365/V100 Obigo/WAP2.0 Profile/MIDP-2.0/CLDC-1.1*/
const Device UA24543 = {S81782, 98, { &P13835,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-Z240/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA54226 = {S81783, 52, { &P14157,&P17484,&P4902,&P18092 }};	/*Useragent - SCH-S239/ POLARIS/6.15 (GUI; compatible; UP.Browser)*/
const Device UA61723 = {S81784, 113, { &P14315,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-E1410/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA63827 = {S81785, 72, { &P14315,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-E1410/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA69180 = {S81786, 92, { &P14168,&P5448,&P4779,&P18092 }};	/*Useragent - HTC_Touch_Viva_T2223 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA89892 = {S81787, 31, { &P17554,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P04B(c500;TB;W24H16)*/
const Device UA141117 = {S81788, 58, { &P15364,&P7127,&P18245,&P18091 }};	/*Useragent - Mozilla/4.74 [en] (Windows NT 5.0; U; maxamine.com--robot)*/
const Device UA1895698 = {S81789, 99, { &P15364,&P7127,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; Windows NT 5.0; phpwebbrainBot/0.1 - http://www.monsterli.ch/phpwebbrain/)*/
const Device UA248938 = {S81790, 43, { &P13324,&P5055,&P24663,&P18092 }};	/*Useragent - NokiaN8-00/UC Browser7.9.0.102/50/355/UCWEB*/
const Device UA460018 = {S81791, 73, { &P15364,&P17811,&P17810,&P18092 }};	/*Useragent - LENOVO-P60/(2009.11.24) S004/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA622937 = {S81792, 81, { &P15364,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-S520/S100 LMP/LML Release/2010.01.25 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA639228 = {S81793, 44, { &P15364,&P17811,&P17810,&P18092 }};	/*Useragent - LENOVO-A332/(2009.12.16) S003/WAP2.0 Profile*/
const Device UA641901 = {S81794, 81, { &P15364,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-i350/S111 LMP/LML Release/2010.10.12 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA645601 = {S81795, 44, { &P15364,&P17811,&P17810,&P18092 }};	/*Useragent - LENOVO-A312/(2009.12.09) S002/WAP2.0 Profile*/
const Device UA1899398 = {S81796, 80, { &P15364,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-TD16/S100 LMP/LML Release/2010.7.30 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA2264520 = {S81797, 73, { &P15364,&P17811,&P17810,&P18092 }};	/*Useragent - LENOVO-P60/(2010.02.23) S005/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA2805197 = {S81798, 81, { &P15364,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-P301/S303 LMP/LML Release/2010.12.29 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA866011 = {S81799, 87, { &P15211,&P17603,&P3534,&P18092 }};	/*Useragent - PantechP2030/JSUSX Browser/Obigo/Q05A OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA875174 = {S81800, 81, { &P15364,&P18110,&P22309,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 on x86_64; rv:5.0) Gecko/X Firefox/5.0 SeaMonkey/2.2*/
const Device UA2819764 = {S81801, 69, { &P15364,&P18110,&P22309,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:2.2a1pre) Gecko/X SeaMonkey/2.2a1pre*/
const Device UA1962436 = {S81802, 140, { &P12280,&P17146,&P20786,&P18092 }};	/*Useragent - MQQBrowser/35 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA1978131 = {S81803, 141, { &P12280,&P17146,&P20786,&P18092 }};	/*Useragent - MQQBrowser/35 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3*/
const Device UA2350171 = {S81804, 78, { &P16003,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; gt-s5233a) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2527284 = {S81805, 159, { &P23920,&P18066,&P18062,&P18092 }};	/*Useragent - Benten A668 Linux/3.0.13 Android/4.0.4 Release/10.24.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2257479 = {S81806, 145, { &P23860,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; SKnetworksE860 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2341352 = {S81807, 133, { &P15003,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M380W Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2545845 = {S81808, 133, { &P15003,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M380S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2350029 = {S81809, 119, { &P11880,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA5320) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile*/
const Device UA2469879 = {S81810, 168, { &P13018,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MT11i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2703328 = {S81811, 92, { &P15364,&P16110,&P21808,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.2.19pre) Gecko/X Prism zdesktop/7.2.2*/
const Device UA2505662 = {S81812, 130, { &P18220,&P5055,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; Nokia808 PureView) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/51/355/UCWEB Mobile*/
const Device UA2565242 = {S81813, 161, { &P18666,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A25 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3020438 = {S81814, 165, { &P18666,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A25 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2783304 = {S81815, 141, { &P15767,&P18432,&P24406,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 OverDrive Media Console/2.6.5*/
const Device UA2601586 = {S81816, 135, { &P23160,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KB901 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2605533 = {S81817, 168, { &P18448,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; ST26i Build/11.2.A.0.21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2657724 = {S81818, 125, { &P18442,&P17979,&P18112,&P18092 }};	/*Useragent - SND-N7100/1.0 Linux/2.6.35.7 Android/4.1.1 Release/12.13.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2750118 = {S81819, 125, { &P18442,&P17979,&P18112,&P18092 }};	/*Useragent - SND-N7100/1.0 Linux/2.6.35.7 Android/4.1.2 Release/01.24.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2864420 = {S81820, 125, { &P18442,&P17979,&P18112,&P18092 }};	/*Useragent - SND-N7100/1.0 Linux/2.6.35.7 Android/4.1.2 Release/03.08.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2936480 = {S81821, 125, { &P18442,&P17979,&P18112,&P18092 }};	/*Useragent - SND-N7100/1.0 Linux/2.6.35.7 Android/4.1.1 Release/03.04.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2704123 = {S81822, 170, { &P18471,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XT907 Build/9.8.1Q_27-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (XT907, 42, 1.9.1)*/
const Device UA2862116 = {S81823, 168, { &P18471,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-66) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (XT907, 42, 1.9.1)*/
const Device UA3017986 = {S81824, 168, { &P18471,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-78) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (XT907, 42, 1.9.1)*/
const Device UA2784688 = {S81825, 158, { &P12616,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; WT19i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2729700 = {S81826, 139, { &P23547,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1 ;zh-cn; ZTE V889S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30;*/
const Device UA2848684 = {S81827, 140, { &P23547,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; ZTE V889S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752593 = {S81828, 98, { &P14764,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; SonyEricssonU1) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2865718 = {S81829, 99, { &P14764,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Sony EricssonU1) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2782828 = {S81830, 157, { &P18304,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9070 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2783065 = {S81831, 139, { &P22756,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C1604 Build/11.1.A.0.68) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782338 = {S81832, 147, { &P18509,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-MS770 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2877105 = {S81833, 160, { &P22622,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Micromax A72 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2954271 = {S81834, 119, { &P17595,&P21721,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17*/
const Device UA3032801 = {S81835, 156, { &P20226,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3020792 = {S81836, 130, { &P14908,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6210 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022788 = {S81837, 130, { &P13452,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P1000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3039306 = {S81838, 154, { &P12059,&P5204,&P24679,&P18092 }};	/*Useragent - Nokia2690/2.0 (09.70) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2690) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3062656 = {S81839, 155, { &P22854,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; GT-I9508 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3081292 = {S81840, 143, { &P13064,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SAMSUNG-SGH-I727 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083175 = {S81841, 140, { &P20543,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A660 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081054 = {S81842, 145, { &P18556,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD+ Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3083662 = {S81843, 145, { &P23137,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I407 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA5340 = {S81844, 174, { &P15377,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) LG-KS20/V10a Browser/IEMobile/7.6 MMS/LG-MMS-WINCE-V1.0/1.2 Java/LGVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA36973 = {S81845, 178, { &P15377,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) LG-KS20/TIM/V10b Browser/IEMobile/7.6 MMS/LG-MMS-WINCE-V1.0/1.2 Java/LGVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA45955 = {S81846, 187, { &P15377,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Vodafone/1.0/LG-KS20/V10b Browser/IEMobile/7.6 MMS/LG-MMS-WINCE-V1.0/1.2 Java/LGVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA9337 = {S81847, 104, { &P13408,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-A885/A885TLIK1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA29856 = {S81848, 145, { &P12272,&P5448,&P4808,&P18092 }};	/*Useragent - LG-GW820/V10g IEMobile/8.12 Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA43422 = {S81849, 55, { &P12871,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_tur / 1.0 / htc_marvel / 2.3.3*/
const Device UA43493 = {S81850, 57, { &P12871,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player orange_uk / 1.0 / htc_marvel / 2.3.3*/
const Device UA59443 = {S81851, 58, { &P12871,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_europe / 1.0 / htc_marvel / 2.3.3*/
const Device UA71943 = {S81852, 53, { &P12871,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player tmous / 1.0 / htc_marvel / 2.3.3*/
const Device UA79690 = {S81853, 59, { &P12871,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player telstra_wwe / 1.0 / htc_marvel / 2.3.3*/
const Device UA438808 = {S81854, 59, { &P12871,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player vodafone_uk / 1.0 / htc_marvel / 2.3.3*/
const Device UA438878 = {S81855, 59, { &P12871,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player vodafone_uk / 1.0 / htc_marvel / 2.3.5*/
const Device UA2150692 = {S81856, 58, { &P12871,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_europe / 1.0 / htc_marvel / 2.3.5*/
const Device UA2573542 = {S81857, 53, { &P12871,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player tmous / 1.0 / htc_marvel / 2.3.5*/
const Device UA2807263 = {S81858, 48, { &P12871,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player HTC / 1.0 / marvelc / 2.3.4*/
const Device UA55122 = {S81859, 30, { &P14675,&P17483,&P4628,&P18092 }};	/*Useragent - LGE-LG265 POLARIS/6.01 MMP/2.0*/
const Device UA73267 = {S81860, 138, { &P13772,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HW-HUAWEI_C8500/C8500V100R001C92B660; 240*320; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA307196 = {S81861, 133, { &P13772,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; C8500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020270 = {S81862, 121, { &P13772,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; C8500 Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2022023 = {S81863, 122, { &P13772,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; C8500 Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2411014 = {S81864, 146, { &P21863,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; cs-cz; GSmart G1355 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA440745 = {S81865, 136, { &P15364,&P17285,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/18.0.1025.142) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19*/
const Device UA440954 = {S81866, 129, { &P15364,&P17285,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/18.0.1025.142) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.142 Safari/535.19*/
const Device UA453843 = {S81867, 157, { &P15364,&P17285,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/18.0.1025.142; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618)*/
const Device UA457675 = {S81868, 129, { &P15364,&P17285,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/18.0.1025.151) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19*/
const Device UA457719 = {S81869, 136, { &P15364,&P17285,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/18.0.1025.151) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19*/
const Device UA457766 = {S81870, 165, { &P15364,&P17285,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/18.0.1025.151; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)*/
const Device UA605641 = {S81871, 90, { &P15364,&P17285,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/18.0.1025.162)*/
const Device UA605645 = {S81872, 97, { &P15364,&P17285,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/18.0.1025.162)*/
const Device UA606043 = {S81873, 129, { &P15364,&P17285,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/18.0.1025.162) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19*/
const Device UA606146 = {S81874, 162, { &P15364,&P17285,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.3; chromeframe/18.0.1025.162; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA606898 = {S81875, 200, { &P15364,&P17285,&P21905,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/18.0.1025.162; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; InfoPath.2; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA614920 = {S81876, 129, { &P15364,&P17285,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/18.0.1025.168) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA614986 = {S81877, 136, { &P15364,&P17285,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/18.0.1025.168) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA628787 = {S81878, 15, { &P21797,&P17979,&P21556,&P18092 }};	/*Useragent - HTC-Android/1.0*/
const Device UA656466 = {S81879, 108, { &P13337,&P17485,&P17347,&P18092 }};	/*Useragent - SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonj108i/UC Browser8.0.3.107/69/444*/
const Device UA670278 = {S81880, 139, { &P15364,&P5690,&P17392,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) RockMelt/0.8.40.147 Chrome/8.0.552.231 Safari/534.10*/
const Device UA750009 = {S81881, 136, { &P15364,&P5690,&P17392,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.7 (KHTML, like Gecko) RockMelt/0.8.36.116 Chrome/7.0.517.44 Safari/534.7*/
const Device UA2268610 = {S81882, 136, { &P15364,&P5690,&P17392,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) RockMelt/0.8.34.820 Chrome/6.0.472.63 Safari/534.3*/
const Device UA689337 = {S81883, 138, { &P21298,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; QW TB-1007 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA924775 = {S81884, 131, { &P15364,&P17285,&P22433,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11 Comodo_Dragon/20.1.1.0*/
const Device UA925474 = {S81885, 124, { &P15364,&P17285,&P22433,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11 Comodo_Dragon/20.1.1.0*/
const Device UA925852 = {S81886, 71, { &P18076,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i667;7.10;8112)*/
const Device UA2523327 = {S81887, 71, { &P18076,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i667;7.10;8773)*/
const Device UA1867553 = {S81888, 66, { &P15364,&P6704,&P20905,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win98; en-US; m18) Gecko/X Netscape6/6.01*/
const Device UA2034510 = {S81889, 70, { &P18513,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;GT-S6802;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA2352322 = {S81890, 123, { &P13648,&P5055,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; NokiaE7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/51/355/UCWEB Mobile*/
const Device UA2384523 = {S81891, 137, { &P20173,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; Limo X2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2505783 = {S81892, 149, { &P20173,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1_jelly_bean; Limo X2+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2385763 = {S81893, 135, { &P21016,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LGL21 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2469881 = {S81894, 171, { &P16436,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2599308 = {S81895, 53, { &P15364,&P18580,&P18369,&P18092 }};	/*Useragent - OperaMini/7.0.5.45389 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2425105 = {S81896, 147, { &P20548,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; bq Livingstone 2 Build/1.1.7 X-10:33) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2455921 = {S81897, 58, { &P12848,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; MB526 Build/JZO54K)*/
const Device UA2690808 = {S81898, 58, { &P12848,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; MB526 Build/JRO03L)*/
const Device UA2838755 = {S81899, 136, { &P12848,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MB526 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2458912 = {S81900, 138, { &P18622,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A820L Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2505012 = {S81901, 66, { &P18450,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; ST23i; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2750596 = {S81902, 174, { &P12972,&P5629,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (SGH-T989, 42, 1.9.1)*/
const Device UA2563130 = {S81903, 91, { &P14438,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAE50) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2554775 = {S81904, 142, { &P23811,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; ZTE-U V889F Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2854937 = {S81905, 140, { &P23811,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE V889F Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3017105 = {S81906, 135, { &P21449,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SH-04E Build/S6181) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2568187 = {S81907, 152, { &P15767,&P19826,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_1_2 like Mac OS X; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile*/
const Device UA2669306 = {S81908, 152, { &P15767,&P19826,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_1_3 like Mac OS X; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile*/
const Device UA2706370 = {S81909, 152, { &P15767,&P19826,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_1_3 like Mac OS X; en-GB) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile*/
const Device UA2583253 = {S81910, 134, { &P21048,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; W732 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583326 = {S81911, 137, { &P20227,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-L40G Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2659028 = {S81912, 178, { &P19748,&P5043,&P24671,&P18092 }};	/*Useragent - Nokia205/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia205) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2667399 = {S81913, 139, { &P12280,&P17090,&P22901,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.3 Mobile/8J2 Safari/6533.18.5*/
const Device UA2702724 = {S81914, 139, { &P12280,&P17090,&P22901,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.3 Mobile/8L1 Safari/6533.18.5*/
const Device UA2863724 = {S81915, 139, { &P12280,&P17090,&P22901,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Mercury/7.3 Mobile/8H7 Safari/6533.18.5*/
const Device UA2674208 = {S81916, 132, { &P20529,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A89 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677000 = {S81917, 137, { &P18469,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabS2109A-F Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2704574 = {S81918, 151, { &P19853,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7102 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2737702 = {S81919, 151, { &P19853,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7102 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2706188 = {S81920, 93, { &P19847,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S6012) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2737695 = {S81921, 93, { &P15919,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-P3100) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2768499 = {S81922, 93, { &P22609,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6312) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2783453 = {S81923, 127, { &P17926,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AT100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2938244 = {S81924, 138, { &P23628,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Andi5Li Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3020272 = {S81925, 138, { &P20244,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; bq Elcano Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3063671 = {S81926, 93, { &P22872,&P18111,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S5282) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3080659 = {S81927, 130, { &P12879,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7510 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3093679 = {S81928, 135, { &P22653,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; HTL22 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3109717 = {S81929, 156, { &P20705,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3127827 = {S81930, 155, { &P20705,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTCONE Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA4762 = {S81931, 128, { &P12178,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-EM30/R6716_G_71.01.02R_B BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA26632 = {S81932, 126, { &P12178,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-EM30/R6716_G_71.03.04R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA26633 = {S81933, 128, { &P12178,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-EM30/R6716_G_71.04.06R_C BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA27523 = {S81934, 126, { &P12178,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-EM30/R6716_G_71.03.06R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA27527 = {S81935, 130, { &P12178,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-EM30/Imperfecto_Lite_v10.3 BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA27526 = {S81936, 126, { &P12178,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-EM30/R6716_G_71.01.02R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA36764 = {S81937, 127, { &P12178,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-EM30/IMPERFECTO_V.2.0_C BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA45505 = {S81938, 126, { &P12178,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-EM30/R6716_G_71.00.19R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54380 = {S81939, 128, { &P12178,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-EM30/R6716_G_71.04.06R_A BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63258 = {S81940, 126, { &P12178,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-EM30/R6716_G_71.03.02R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63891 = {S81941, 128, { &P12178,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-EM30/R6716_G_71.01.02R_A BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA5470 = {S81942, 160, { &P12263,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-; LG-GW620 Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA5472 = {S81943, 162, { &P12263,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; LG-GW620 Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA17912 = {S81944, 107, { &P13965,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-A237/A237UCHG1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA24488 = {S81945, 148, { &P12221,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Nokia6290/3.03; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA79609 = {S81946, 152, { &P12221,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3 U; [en]; Series60/3.2 Nokia6290/1.00 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA68758 = {S81947, 36, { &P12449,&P5890,&P4777,&P18092 }};	/*Useragent - NOKIAE5-00/UC Browser7.7.1.88/28/444*/
const Device UA569423 = {S81948, 46, { &P12449,&P5890,&P4777,&P18092 }};	/*Useragent - nokiae5-00/UC Browser7.7.1.88/69/444  3gpp-gba*/
const Device UA64576 = {S81949, 96, { &P23012,&P17485,&P3976,&P18092 }};	/*Useragent - Mozilla/4.0 SonyEricssonW890iv/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA202347 = {S81950, 54, { &P17588,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3V UP.Browser/6.2_7.2.7.1.K.6.210 (GUI) MMP/2.0*/
const Device UA349061 = {S81951, 131, { &P17595,&P17297,&P22453,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.1 Chrome/15.0.900.2 Safari/535.2*/
const Device UA422338 = {S81952, 131, { &P17595,&P17297,&P22453,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.1 Chrome/15.0.900.2 Safari/535.2*/
const Device UA851239 = {S81953, 131, { &P17595,&P17297,&P22453,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.1 Chrome/15.0.900.2 Safari/535.2*/
const Device UA2401042 = {S81954, 131, { &P17595,&P17297,&P22453,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.1 Chrome/15.0.900.2 Safari/535.2*/
const Device UA363320 = {S81955, 124, { &P18061,&P5096,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; sdk Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Maxthon*/
const Device UA2411526 = {S81956, 107, { &P15364,&P7127,&P18197,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; es-ES; rv:1.9.1.7) Gecko/X Thunderbird/3.0.1 ThunderBrowse/3.2.8.1*/
const Device UA836931 = {S81957, 146, { &P16752,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SPICE Mi-720 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA924803 = {S81958, 91, { &P17595,&P17296,&P22316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:14.0) Gecko/X Firefox/14.0.1 SeaMonkey/2.11*/
const Device UA1927882 = {S81959, 142, { &P22974,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5578 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2589277 = {S81960, 135, { &P22974,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5578 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1934860 = {S81961, 62, { &P18741,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux; U;2.3.4;zh_cn;T703;320*480;)UCWEB7.6.0.75/139/32534*/
const Device UA1990029 = {S81962, 137, { &P18745,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOX905 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2686803 = {S81963, 126, { &P18745,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; X905 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2005194 = {S81964, 86, { &P15364,&P7133,&P18532,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.6.2)*/
const Device UA2007010 = {S81965, 114, { &P15364,&P7133,&P18532,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Sleipnir/3.6.2*/
const Device UA2008088 = {S81966, 121, { &P15364,&P7133,&P18532,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.60 Safari/537.1 Sleipnir/3.6.1*/
const Device UA2008105 = {S81967, 86, { &P15364,&P7133,&P18532,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.6.1)*/
const Device UA2027956 = {S81968, 129, { &P21236,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-nz; VIGO7 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2417128 = {S81969, 129, { &P21236,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; VIGO7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2030902 = {S81970, 54, { &P18306,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; N-05D Build/AX)*/
const Device UA2032146 = {S81971, 130, { &P18306,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; N-05D Build/AX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2382780 = {S81972, 138, { &P18686,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A810K Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2423037 = {S81973, 138, { &P18686,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A810S Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2423127 = {S81974, 145, { &P20843,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Scroll Basic Plus v.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2426998 = {S81975, 68, { &P15028,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.2; zh-cn; LG-P350; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2558819 = {S81976, 195, { &P18465,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; KFOT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-17 12:06, +0000, gb*/
const Device UA2701070 = {S81977, 188, { &P18465,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-17 12:06, -0500, us*/
const Device UA2504861 = {S81978, 155, { &P15786,&P5043,&P24673,&P18092 }};	/*Useragent - NokiaC2-05/2.0 (08.69) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac2-05) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2537620 = {S81979, 143, { &P21020,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Micromax A116 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2537797 = {S81980, 93, { &P14186,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5830) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2562722 = {S81981, 91, { &P12671,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia700) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2566683 = {S81982, 135, { &P20414,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SmartTabII10 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2593615 = {S81983, 134, { &P21335,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch W619 Build/GRK39F) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2594017 = {S81984, 141, { &P21335,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; zh-cn; K-Touch W619 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2776404 = {S81985, 141, { &P21335,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; K-Touch_W619 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2600839 = {S81986, 137, { &P20864,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2603899 = {S81987, 130, { &P20864,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A13-MID Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2675517 = {S81988, 138, { &P20608,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Edison Build/1.1.8 X-16:29) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676326 = {S81989, 137, { &P20075,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xperia S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712246 = {S81990, 139, { &P20174,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Cink King Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2732144 = {S81991, 139, { &P21132,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Andi4.5h Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2732670 = {S81992, 155, { &P18464,&P18066,&P22588,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2800569 = {S81993, 159, { &P13018,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; MT11i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2796807 = {S81994, 152, { &P18721,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A90S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2749971 = {S81995, 138, { &P20075,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Xperia S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2863997 = {S81996, 139, { &P20075,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Xperia S Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2773359 = {S81997, 144, { &P19855,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT885 Build/9.8.3O-148_IRD-3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782445 = {S81998, 127, { &P17686,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MZ601 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2784086 = {S81999, 127, { &P17686,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MZ601 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2791696 = {S82000, 54, { &P15364,&P6729,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Win95; Trident/5.0)*/
const Device UA2864629 = {S82001, 134, { &P24553,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; X21 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2868690 = {S82002, 151, { &P18673,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2896468 = {S82003, 152, { &P18673,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562i Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2898775 = {S82004, 93, { &P20146,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9082) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2901234 = {S82005, 161, { &P18672,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5302 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3060430 = {S82006, 160, { &P17625,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST25i Build/6.1.1.B.1.10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2975994 = {S82007, 137, { &P14877,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3026079 = {S82008, 167, { &P23122,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; Karbonn A6 Build/MocorDroid4.0.3) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3057388 = {S82009, 142, { &P12280,&P18432,&P24341,&P18092 }};	/*Useragent - MQQBrowser/44 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3*/
const Device UA3077719 = {S82010, 140, { &P12280,&P18432,&P24341,&P18092 }};	/*Useragent - MQQBrowser/44 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3*/
const Device UA3077900 = {S82011, 141, { &P12280,&P18432,&P24341,&P18092 }};	/*Useragent - MQQBrowser/44 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 Safari/7534.48.3*/
const Device UA3078516 = {S82012, 141, { &P12280,&P18432,&P24341,&P18092 }};	/*Useragent - MQQBrowser/44 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3*/
const Device UA3080286 = {S82013, 138, { &P15972,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One XL Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3109396 = {S82014, 156, { &P18645,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SPH-L710 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3111580 = {S82015, 164, { &P18645,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SAMSUNG-SGH-I747 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126641 = {S82016, 164, { &P18645,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126912 = {S82017, 156, { &P18645,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9305 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA22631 = {S82018, 82, { &P14345,&P16000,&P4615,&P18092 }};	/*Useragent - NokiaN-Gage/1.0 SymbianOS/6.1 Series60/1.2 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA62680 = {S82019, 84, { &P14345,&P16000,&P4615,&P18092 }};	/*Useragent - NokiaN-GageQD/1.0 SymbianOS/6.1 Series60/1.2 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA24244 = {S82020, 110, { &P14584,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-S8000L/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA64248 = {S82021, 110, { &P14584,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-S8000B/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA2023316 = {S82022, 141, { &P16459,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ru-ru; GSmart G1315 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA157704 = {S82023, 141, { &P16168,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; LogicPD Zoom2 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1900029 = {S82024, 137, { &P16168,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; NookColor Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA196512 = {S82025, 132, { &P13201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ERIS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA217080 = {S82026, 132, { &P13201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ERIS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA251533 = {S82027, 132, { &P13201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ERIS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA539849 = {S82028, 140, { &P13201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ERIS-GSBv3.3 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA642880 = {S82029, 132, { &P13201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Eris Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1870787 = {S82030, 132, { &P13201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ERIS Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018328 = {S82031, 140, { &P13201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; GSBv1.7-ERIS Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018385 = {S82032, 140, { &P13201,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; ERIS GSBv2.1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA347242 = {S82033, 121, { &P15364,&P17285,&P22414,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.0.2.0 Chrome/16.0.912.59 Safari/535.7*/
const Device UA351620 = {S82034, 128, { &P15364,&P17285,&P22414,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.0.2.0 Chrome/16.0.912.59 Safari/535.7*/
const Device UA779907 = {S82035, 146, { &P21861,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; it-it; SMARTPAD810C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA454865 = {S82036, 142, { &P15575,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA773751 = {S82037, 142, { &P15575,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA704344 = {S82038, 114, { &P14635,&P5843,&P17437,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-00/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.4.0.34.8*/
const Device UA614840 = {S82039, 75, { &P15364,&P22740,&P20434,&P18092 }};	/*Useragent - Mozilla/5.0 (Maemo; Linux armv7l; rv:12.0) Gecko/X Firefox/12.0 Fennec/12.0*/
const Device UA663680 = {S82040, 149, { &P21436,&P5096,&P3508,&P18092 }};	/*Useragent - ZTE-TU830_TD/1.0 OPhone/2.5 (Linux; Android 2.2) Release/08.15.2011 Browser/WAP2.0 (AppleWebKit/533.1) Mobile Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA670003 = {S82041, 126, { &P21436,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-T U830 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2020929 = {S82042, 138, { &P21436,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-T U830 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026538 = {S82043, 131, { &P21436,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-T U830 Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2154668 = {S82044, 52, { &P14748,&P5890,&P21761,&P18092 }};	/*Useragent - iBrowser/2.5 (NokiaE72-1;SymbianOS/9.3 Series60/3.2)*/
const Device UA2458996 = {S82045, 86, { &P17968,&P17979,&P18395,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/6.6.29182/28.3234; U; zh) Presto/2.8.119 Version/11.10*/
const Device UA2259467 = {S82046, 95, { &P15364,&P5690,&P22483,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.0.7) Gecko/X Firefox/1.5.0.7 Flock/0.7.6*/
const Device UA2462698 = {S82047, 81, { &P15364,&P5690,&P22483,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.0.9) Gecko/X Flock/0.7.9.1*/
const Device UA2294468 = {S82048, 133, { &P15364,&P5690,&P23734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/527+ (KHTML, like Gecko)  QtWeb Internet Browser/3.0 http://www.QtWeb.net*/
const Device UA2495372 = {S82049, 132, { &P15364,&P5690,&P23734,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/527+ (KHTML, like Gecko) QtWeb Internet Browser/3.0 http://www.QtWeb.net*/
const Device UA2349023 = {S82050, 140, { &P16036,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus One Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2384590 = {S82051, 140, { &P16036,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3073240 = {S82052, 141, { &P16036,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus One Build/Slim VJ) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2352189 = {S82053, 149, { &P21616,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile i-style Q3 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2526273 = {S82054, 150, { &P21616,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile i-style Q3i Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2364911 = {S82055, 133, { &P17976,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Tablet PC Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2413272 = {S82056, 133, { &P17976,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Tablet PC Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2775093 = {S82057, 141, { &P17976,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID_Serials Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2421749 = {S82058, 144, { &P20826,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; VS TOUCHTAB 9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2458249 = {S82059, 139, { &P20928,&P5629,&P15760,&P18092 }};	/*Useragent - AT-AQ28D Linux/2.6.35 Android/2.3.5 Release/11.15.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1*/
const Device UA2459258 = {S82060, 132, { &P13452,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P1000 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2622192 = {S82061, 133, { &P13452,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P1000N Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2623528 = {S82062, 138, { &P13452,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P1000 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2624705 = {S82063, 133, { &P13452,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P1000L Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2632331 = {S82064, 131, { &P13452,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P1000 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2632633 = {S82065, 137, { &P13452,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P1000 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2536111 = {S82066, 134, { &P22044,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Energy x10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2578513 = {S82067, 71, { &P23037,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X293/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2729337 = {S82068, 129, { &P23335,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; U25GT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2662239 = {S82069, 92, { &P15028,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; LG-P350) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2664846 = {S82070, 138, { &P23817,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ro-ro; COSMOTE_MY_TAB Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2672071 = {S82071, 137, { &P20103,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; EK-GC100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735787 = {S82072, 137, { &P20103,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EK-GC100 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2674445 = {S82073, 138, { &P18707,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MD_LIFETAB_P9516 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2706814 = {S82074, 90, { &P18448,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; ST26i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2711477 = {S82075, 181, { &P12871,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Wildfire S A510e Build/JDQ39; CyanogenMod-Sense 5T by djolivier) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088856 = {S82076, 190, { &P12871,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Wildfire S A510e Build/CM10.1-Release-Candidate-v2.1; CyanogenMod-CM10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3107709 = {S82077, 173, { &P12871,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Wildfire S A510e Build/CM10.1-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/marvel*/
const Device UA2718761 = {S82078, 137, { &P18653,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P870 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2769887 = {S82079, 152, { &P22962,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867868 = {S82080, 143, { &P16321,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Infinix X500 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2868571 = {S82081, 139, { &P18582,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xperia ion Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2888373 = {S82082, 139, { &P18582,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT28h Build/6.2.B.0.211) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868822 = {S82083, 138, { &P21065,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; CINK FIVE Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892574 = {S82084, 134, { &P21065,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CINK+ Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2959657 = {S82085, 156, { &P21117,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; HS-9DTB4 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2892654 = {S82086, 140, { &P24687,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HTC-802d/1.11.1401.2) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956824 = {S82087, 139, { &P24687,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HTC 802d Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049008 = {S82088, 140, { &P24687,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HTC-802d/1.02.1401.1) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2893596 = {S82089, 156, { &P20705,&P18598,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC One Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2895416 = {S82090, 158, { &P18302,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT25i Build/4.1.B.0.631) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2944226 = {S82091, 151, { &P20231,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A21 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3011179 = {S82092, 159, { &P20231,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A21 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2975977 = {S82093, 138, { &P20816,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CINK SLIM Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2977460 = {S82094, 142, { &P15364,&P7133,&P22437,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Comodo_Dragon/1.0.0.9 Chrome/ Version/3.2.1 Safari/532.0*/
const Device UA3000066 = {S82095, 137, { &P20146,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-I9082 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3093661 = {S82096, 138, { &P20608,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Edison Build/1.1.8 X-16:29) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3111218 = {S82097, 157, { &P12968,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; LT26i Build/6.2.B.1.96) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3111707 = {S82098, 158, { &P12968,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT26i Build/6.2.B.0.200) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA64440 = {S82099, 61, { &P14253,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia6100/1.0 (04.01) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA25782 = {S82100, 59, { &P12317,&P17483,&P3534,&P18092 }};	/*Useragent - LG-TU515 Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54164 = {S82101, 65, { &P12317,&P17483,&P3534,&P18092 }};	/*Useragent - LG-CU515/V1.0g Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA464601 = {S82102, 80, { &P15364,&P7127,&P20903,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.7.5) Gecko/X Netscape/8.0.1*/
const Device UA478027 = {S82103, 79, { &P15364,&P7133,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.7.7) Gecko/X Firefox/1.0.3*/
const Device UA541221 = {S82104, 132, { &P20288,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us; HTC_P515E) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA565326 = {S82105, 70, { &P15364,&P7133,&P22311,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:6.0) Gecko/X Firefox/6.0 SeaMonkey/2.3*/
const Device UA572001 = {S82106, 37, { &P12456,&P5890,&P17338,&P18092 }};	/*Useragent - nokian73-1/UC Browser7.9.0.102/69/352*/
const Device UA615376 = {S82107, 136, { &P12280,&P18117,&P20784,&P18092 }};	/*Useragent - MQQBrowser/33 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3*/
const Device UA629971 = {S82108, 136, { &P12280,&P18117,&P20784,&P18092 }};	/*Useragent - MQQBrowser/33 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3*/
const Device UA685667 = {S82109, 138, { &P12280,&P18117,&P20784,&P18092 }};	/*Useragent - MQQBrowser/33 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA678342 = {S82110, 142, { &P13644,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/GRJ90)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA810199 = {S82111, 142, { &P13644,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PC36100 Build/GRI40)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2782942 = {S82112, 143, { &P13644,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA3103167 = {S82113, 143, { &P13644,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; PC36100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA916007 = {S82114, 73, { &P15364,&P7133,&P17208,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:6.0.2) Gecko/X Firefox/6.0.2 slurp*/
const Device UA1896992 = {S82115, 69, { &P15364,&P7133,&P17208,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:6.0) Gecko/X Firefox/6.0 slurp*/
const Device UA1898179 = {S82116, 79, { &P15364,&P7133,&P17208,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:6.0.2) Gecko/X Firefox/6.0.2 slurp slurp*/
const Device UA1898742 = {S82117, 66, { &P15364,&P7133,&P17208,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:6.0.2) Gecko/X Firefox/6.0.2 slurp*/
const Device UA1902455 = {S82118, 133, { &P22485,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; T3 Build/1.8.1371.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1951001 = {S82119, 133, { &P22485,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; T3 Build/1.9.1425.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018371 = {S82120, 132, { &P22485,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; T3 Build/1.2.867.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018541 = {S82121, 133, { &P22485,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; T3 Build/1.3.1018.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018802 = {S82122, 133, { &P22485,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; T3 Build/1.7.1310.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022223 = {S82123, 133, { &P22485,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; T3 Build/1.6.1250.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2351352 = {S82124, 122, { &P12329,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN86 8MP) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile*/
const Device UA2664286 = {S82125, 91, { &P12456,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAE73) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2728583 = {S82126, 92, { &P12456,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokian73-1) U2/1.0.0 UCBrowser/8.8.0.227 U2/1.0.0 Mobile*/
const Device UA2484181 = {S82127, 143, { &P22145,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Videocon A30 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2535585 = {S82128, 137, { &P20963,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SBM200SH Build/S0016) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2507155 = {S82129, 151, { &P18558,&P20706,&P24673,&P18092 }};	/*Useragent - Nokia309/2.0 (05.85) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia309) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2527800 = {S82130, 143, { &P21260,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NGM Absolute Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528308 = {S82131, 137, { &P22698,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ThL A1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558214 = {S82132, 166, { &P18472,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; DROID RAZR HD Build/9.8.1Q_39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2673582 = {S82133, 175, { &P18472,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2567195 = {S82134, 145, { &P18512,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT881 Build/6.7.3-130_YUS-11) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2567969 = {S82135, 131, { &P19852,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7511 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2579666 = {S82136, 141, { &P21493,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Fly IQ444; Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2951430 = {S82137, 140, { &P21493,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Fly_IQ444 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2644786 = {S82138, 138, { &P23068,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ThL V12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747869 = {S82139, 138, { &P23068,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ThL V12 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658399 = {S82140, 140, { &P21501,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; P5 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2660336 = {S82141, 88, { &P20170,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A95) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2698923 = {S82142, 99, { &P20170,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Celkon*A95*Pro) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2675414 = {S82143, 102, { &P15795,&P21697,&P24633,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 710) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile*/
const Device UA2737535 = {S82144, 104, { &P14358,&P5890,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaE63*/
const Device UA2744838 = {S82145, 178, { &P12989,&P5843,&P24671,&P18092 }};	/*Useragent - Nokia303/5.0 (14.38) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia303) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2755342 = {S82146, 143, { &P18466,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755408 = {S82147, 151, { &P20002,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 992D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2795732 = {S82148, 134, { &P23194,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Engage 7HD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2797026 = {S82149, 140, { &P12788,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Evo 3D CDMA Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877357 = {S82150, 139, { &P12788,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Evo 3D GSM Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879362 = {S82151, 135, { &P12788,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; PG86100 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798392 = {S82152, 129, { &P18320,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PadFone Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2867630 = {S82153, 136, { &P18320,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PadFone Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869651 = {S82154, 136, { &P18320,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PadFone Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879510 = {S82155, 129, { &P18320,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PadFone Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2799351 = {S82156, 152, { &P23239,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2800680 = {S82157, 159, { &P23239,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL_ONE_TOUCH_6030X_Orange Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2872325 = {S82158, 34, { &P23144,&P22882,&P17470,&P18092 }};	/*Useragent - iOS/7.0 (11A4372q) dataaccessd/1.0*/
const Device UA3059571 = {S82159, 34, { &P23144,&P22882,&P17470,&P18092 }};	/*Useragent - iOS/7.0 (11A4449d) dataaccessd/1.0*/
const Device UA3109451 = {S82160, 32, { &P23144,&P22882,&P17470,&P18092 }};	/*Useragent - iOS/7.0 (11A465) dataaccessd/1.0*/
const Device UA3130593 = {S82161, 34, { &P23144,&P22882,&P17470,&P18092 }};	/*Useragent - iOS/7.0.2 (11A501) dataaccessd/1.0*/
const Device UA2879246 = {S82162, 139, { &P22715,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android OUYA 4.1.2; en-us; OUYA  Build/JZO54L-OUYA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2953129 = {S82163, 137, { &P24508,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E445 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2958974 = {S82164, 140, { &P21532,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; vivo Y19t Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3000659 = {S82165, 131, { &P21532,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; vivo Y19t Build/JOP40D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3109085 = {S82166, 138, { &P21532,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; vivo Y11 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022604 = {S82167, 140, { &P20183,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MI-ONE Plus Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974444 = {S82168, 144, { &P20104,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; 201M Build/9.8.2Q-34_SMJ-102) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2967121 = {S82169, 156, { &P14574,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3021725 = {S82170, 133, { &P21048,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; W732 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022888 = {S82171, 93, { &P22607,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S5303) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3024500 = {S82172, 170, { &P18465,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Amazon Kindle Fire Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/otter*/
const Device UA3063537 = {S82173, 170, { &P18465,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Amazon Kindle Fire Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/otter*/
const Device UA3079319 = {S82174, 156, { &P15837,&P6909,&P24679,&P18092 }};	/*Useragent - NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-05) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3093663 = {S82175, 140, { &P16869,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Micromax A76 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109452 = {S82176, 140, { &P16869,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Micromax A74 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3096345 = {S82177, 158, { &P18295,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3111730 = {S82178, 49, { &P15767,&P23096,&P17470,&P18092 }};	/*Useragent - browseripad/40889 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3127665 = {S82179, 55, { &P15767,&P23096,&P17470,&P18092 }};	/*Useragent - CitrixReceiver-iPad/179 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3131327 = {S82180, 59, { &P15767,&P23096,&P17470,&P18092 }};	/*Useragent - Perfect%20Browser-iPad/10.0 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA5565 = {S82181, 129, { &P14162,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV6/96.66.74R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11033063) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA5566 = {S82182, 129, { &P14162,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV6/99.41.08R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163225) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA5567 = {S82183, 129, { &P14162,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV6/98.30.80R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12093102) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA5573 = {S82184, 130, { &P14162,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV6/96.80.55AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11073083) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA29729 = {S82185, 129, { &P14162,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV6/99.41.0BR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163234) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA29727 = {S82186, 131, { &P14162,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV6v/98.20.33BR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12043064) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA29731 = {S82187, 129, { &P14162,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV6/96.80.51R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11073071) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA29733 = {S82188, 129, { &P14162,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV6/96.A0.0ER BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11073121) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA29738 = {S82189, 130, { &P14162,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV6/98.30.72AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12003079) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA29742 = {S82190, 129, { &P14162,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV6/99.41.05R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163189) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA46369 = {S82191, 130, { &P14162,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV6/96.66.71AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11033039) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA54268 = {S82192, 132, { &P14162,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV6vb/99.41.04ER BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163189) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA58081 = {S82193, 130, { &P14162,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV6v/98.50.11R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12093118) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA74338 = {S82194, 128, { &P14162,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV6/98.60.02R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12093118) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [he]*/
const Device UA24405 = {S82195, 41, { &P15688,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-R500 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA68603 = {S82196, 130, { &P14973,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; LG-V909 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA69581 = {S82197, 9, { &P14984,&P5129,&P4030,&P18092 }};	/*Useragent - Nokia5250*/
const Device UA2456001 = {S82198, 102, { &P17595,&P17297,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5; U; Edition MacAppStore; en) Presto/2.9.168 Version/11.50*/
const Device UA567845 = {S82199, 142, { &P23857,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-es; AIRIS_OnePAD700 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA613280 = {S82200, 88, { &P17595,&P17296,&P21171,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:12.0) Gecko/X Firefox/12.0 SeaMonkey/2.9*/
const Device UA888001 = {S82201, 135, { &P21444,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; PLOYER-MOMO Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2548284 = {S82202, 142, { &P21444,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; PLOYER-MOMO Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2810287 = {S82203, 64, { &P21444,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.3; PLOYER-MOMO Build/IML74K)*/
const Device UA1953974 = {S82204, 85, { &P14985,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s8530) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2028472 = {S82205, 171, { &P18513,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2233391 = {S82206, 138, { &P18643,&P5043,&P24670,&P18092 }};	/*Useragent - Nokia311/5.0 (03.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia311) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2355740 = {S82207, 65, { &P12280,&P18117,&P22071,&P18092 }};	/*Useragent - Instagram 3.1.2 (iPhone; iPhone OS 5.1.1; en_US) AppleWebKit/420+*/
const Device UA2790200 = {S82208, 63, { &P12280,&P18117,&P22071,&P18092 }};	/*Useragent - Instagram 3.1.2 (iPhone; iPhone OS 5.1; en_US) AppleWebKit/420+*/
const Device UA2339452 = {S82209, 132, { &P12889,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7300 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2612264 = {S82210, 118, { &P12889,&P18111,&P18112,&P18092 }};	/*Useragent - Logos/1.1.3 (Linux; U; Android 4.1.1; en-US; GT-P7300 Build/cm_p5-userdebug 4.1.1 JRO03H eng.kalltkaffe.X.X test-keys)*/
const Device UA2612416 = {S82211, 61, { &P12889,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GT-P7300 Build/JRO03H)*/
const Device UA2617138 = {S82212, 132, { &P12889,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617266 = {S82213, 119, { &P12889,&P18111,&P18112,&P18092 }};	/*Useragent - Biblia/2.1.2 (Linux; U; Android 4.1.2; es-ES; GT-P7300 Build/cm_p5-userdebug 4.1.2 JZO54K eng.kalltkaffe.X.X test-keys)*/
const Device UA2617349 = {S82214, 61, { &P12889,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P7300 Build/JZO54K)*/
const Device UA2618518 = {S82215, 118, { &P12889,&P18111,&P18112,&P18092 }};	/*Useragent - Vyrso/2.1.2 (Linux; U; Android 4.1.1; en-US; GT-P7300 Build/cm_p5-userdebug 4.1.1 JRO03H eng.safariking.X.X test-keys)*/
const Device UA2357297 = {S82216, 39, { &P18218,&P15924,&P20764,&P18092 }};	/*Useragent - NOKIA-Lumia 610/UCWEB2.6.0.213/47/32079*/
const Device UA2437216 = {S82217, 40, { &P17968,&P17979,&P24439,&P18092 }};	/*Useragent - Dolphin http client/9.1.0(195) (Android)*/
const Device UA2453563 = {S82218, 141, { &P21259,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; NGM EXPLORER Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2469350 = {S82219, 129, { &P20127,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT270 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2528457 = {S82220, 138, { &P21996,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; S100 AMAZE Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2564068 = {S82221, 167, { &P18448,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST26i Build/11.0.A.7.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2567083 = {S82222, 137, { &P18662,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ISW13HT Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667408 = {S82223, 141, { &P18662,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC J Z321e Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2580599 = {S82224, 101, { &P15483,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_ChaCha_A810e) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2764632 = {S82225, 101, { &P15483,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; HTC_ChaCha_A810e) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2599913 = {S82226, 139, { &P23176,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MTV_volt Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2660408 = {S82227, 99, { &P14031,&P5477,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT-I5503) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2664849 = {S82228, 140, { &P22790,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; Masstel M15 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2673966 = {S82229, 137, { &P14877,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9000 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708460 = {S82230, 132, { &P22754,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HP Slate 7 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2719120 = {S82231, 132, { &P20453,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NABI2-NV7A Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2864938 = {S82232, 135, { &P20453,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NABI2-NV7A-UK Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2877858 = {S82233, 169, { &P20005,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A65 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2750902 = {S82234, 125, { &P23946,&P17979,&P18062,&P18092 }};	/*Useragent - SND-I7100/1.0 Linux/2.6.35.7 Android/4.0.3 Release/03.05.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2860424 = {S82235, 125, { &P23946,&P17979,&P18062,&P18092 }};	/*Useragent - SND-I7100/1.0 Linux/2.6.35.7 Android/4.0.3 Release/02.26.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3086916 = {S82236, 125, { &P23946,&P17979,&P18062,&P18092 }};	/*Useragent - SND-I7100/1.0 Linux/2.6.35.7 Android/4.0.3 Release/01.21.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2784883 = {S82237, 159, { &P13018,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT11i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2768603 = {S82238, 163, { &P22984,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GOCLEVER-TERRA9o Build/GOCLEVER-TERRA9o-GU13045-0325) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2770509 = {S82239, 140, { &P15160,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TP7-1000DC light Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3079383 = {S82240, 140, { &P23214,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6833 Build/14.1.B.1.510) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2894131 = {S82241, 156, { &P18435,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; ARM; Trident/7.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA3005552 = {S82242, 185, { &P18435,&P16110,&P18618,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2; ARM; Trident/7.0; .NET4.0E; .NET4.0C; Tablet PC 2.0; Microsoft Outlook 15.0.4505; Microsoft Outlook 15.0.4505; ms-office; MSOffice 15)*/
const Device UA2960664 = {S82243, 170, { &P16553,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T959V Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/galaxys4gmtd*/
const Device UA3021607 = {S82244, 174, { &P22622,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Micromax_A72 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3059262 = {S82245, 162, { &P14969,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire SV Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3060400 = {S82246, 161, { &P14969,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3061610 = {S82247, 152, { &P21472,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; A111 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3027689 = {S82248, 98, { &P19862,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax A110) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3058637 = {S82249, 152, { &P15364,&P20706,&P24679,&P18092 }};	/*Useragent - Nokia210/2.0 (04.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia210) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3059531 = {S82250, 149, { &P15364,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; P2 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3106497 = {S82251, 153, { &P15364,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-CN; R809T Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3059747 = {S82252, 176, { &P15803,&P5843,&P24055,&P18092 }};	/*Useragent - Nokia200/2.0 (11.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311*/
const Device UA3081045 = {S82253, 124, { &P18017,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; K1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094549 = {S82254, 132, { &P18017,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Ideapad K1 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080995 = {S82255, 136, { &P22727,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGP312 Build/10.3.1.C.0.136) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA63757 = {S82256, 77, { &P14790,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN77-1/3.0838.22.0.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA14287 = {S82257, 141, { &P17953,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-za; RBM C Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA40942 = {S82258, 140, { &P17953,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; de-at; RBM3 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA49016 = {S82259, 140, { &P17953,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; hu-hu; RBM2 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA19892 = {S82260, 85, { &P20279,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) HTC6175*/
const Device UA89692 = {S82261, 89, { &P20279,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) USCCHTC6175*/
const Device UA67100 = {S82262, 110, { &P16948,&P5448,&P4737,&P18092 }};	/*Useragent - SAMSUNG-GT-B7320/1.0 MIDP/2.1 CLDC/1.1 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.2.2.51*/
const Device UA63222 = {S82263, 47, { &P13318,&P6476,&P3625,&P18092 }};	/*Useragent - LGE-VX8300/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA90523 = {S82264, 135, { &P18271,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; DM009SH Build/S0300) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA450897 = {S82265, 135, { &P18271,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; DM009SH Build/S0400) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA573941 = {S82266, 135, { &P18271,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; DM009SH Build/S0800) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA771720 = {S82267, 135, { &P18271,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; DM009SH Build/S0700) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019144 = {S82268, 135, { &P18271,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; DM009SH Build/S0500) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA684677 = {S82269, 120, { &P15364,&P7127,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.205 Safari/534.16*/
const Device UA606861 = {S82270, 103, { &P15364,&P18346,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.162 Safari/535.19*/
const Device UA505425 = {S82271, 82, { &P15364,&P7133,&P20797,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.1.10) Gecko/X Postbox/1.0.1*/
const Device UA1849588 = {S82272, 94, { &P15364,&P17285,&P22332,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; fr; rv:1.9.1.3) Gecko/X Firefox/3.5.3 CometBird/3.5.3*/
const Device UA564077 = {S82273, 132, { &P15389,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; u8800 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA575486 = {S82274, 132, { &P15389,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; u8800 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1939238 = {S82275, 157, { &P15389,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Huawei Ideos X5 1.12.9(ret4rt) Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021740 = {S82276, 142, { &P15389,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Huawei Ideos X5 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA697111 = {S82277, 62, { &P15364,&P5690,&P22313,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:7.0a1) Gecko/X SeaMonkey/2.4a1*/
const Device UA2144966 = {S82278, 86, { &P15364,&P5690,&P22313,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:7.0) Gecko/X Firefox/7.0 SeaMonkey/2.4 Lightning/1.0b7*/
const Device UA609050 = {S82279, 155, { &P12731,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB855 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA744780 = {S82280, 155, { &P12731,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB855 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA904859 = {S82281, 151, { &P12731,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB855 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA640087 = {S82282, 69, { &P15364,&P17285,&P22345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:2.0) Gecko/X Firefox/4.0 PaleMoon/4.0*/
const Device UA822680 = {S82283, 71, { &P15364,&P18110,&P23696,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8b5) Gecko/X Firefox/1.4.1*/
const Device UA2027324 = {S82284, 138, { &P21949,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F100L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2166599 = {S82285, 96, { &P18506,&P5043,&P17338,&P18092 }};	/*Useragent - Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia305/UC Browser7.9.0.102/69/352*/
const Device UA2226948 = {S82286, 142, { &P13042,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Incredible S Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2457282 = {S82287, 142, { &P13042,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Incredible S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2364927 = {S82288, 151, { &P18296,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; One V Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2367107 = {S82289, 136, { &P18600,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-03D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2383309 = {S82290, 136, { &P22513,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; PW1100S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2386310 = {S82291, 133, { &P22024,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; dL1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2389719 = {S82292, 141, { &P17905,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SO-03D Build/6.1.F.0.106) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2400634 = {S82293, 140, { &P23463,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Spice Mi-320 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2423649 = {S82294, 159, { &P12135,&P17483,&P24673,&P18092 }};	/*Useragent - LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-gd510) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2426724 = {S82295, 132, { &P20107,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5580C Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2426819 = {S82296, 140, { &P12280,&P17146,&P20790,&P18092 }};	/*Useragent - MQQBrowser/39 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA2467812 = {S82297, 141, { &P12280,&P17146,&P20790,&P18092 }};	/*Useragent - MQQBrowser/39 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3*/
const Device UA2486804 = {S82298, 141, { &P12280,&P17146,&P20790,&P18092 }};	/*Useragent - MQQBrowser/391 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA2848968 = {S82299, 142, { &P12280,&P17146,&P20790,&P18092 }};	/*Useragent - MQQBrowser/391 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3*/
const Device UA2461309 = {S82300, 90, { &P23545,&P17017,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; mauie800) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2529103 = {S82301, 90, { &P23545,&P17017,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; mauie800) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2467025 = {S82302, 95, { &P15364,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; 11a h700tecno) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2656205 = {S82303, 87, { &P15364,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; s8000) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2696003 = {S82304, 94, { &P15364,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; beetel gd777) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2728374 = {S82305, 91, { &P15364,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sjt_e861a) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2844397 = {S82306, 99, { &P15364,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; 11a wbg3509a_0101) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2878173 = {S82307, 95, { &P15364,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; 11a h700tecno) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2951503 = {S82308, 89, { &P15364,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; es-la; mozilla) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2471703 = {S82309, 114, { &P15364,&P7133,&P18629,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17 slurp*/
const Device UA2487436 = {S82310, 114, { &P15364,&P7133,&P18629,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.56 Safari/537.17 slurp*/
const Device UA2512206 = {S82311, 114, { &P15364,&P7133,&P18629,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17 slurp*/
const Device UA2481419 = {S82312, 127, { &P21041,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D97 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2530724 = {S82313, 166, { &P18679,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-B5330 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2534839 = {S82314, 180, { &P19868,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; ALCATEL ONE TOUCH 985N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3021083 = {S82315, 184, { &P19868,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; ALCATEL_ONE_TOUCH_985N Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2672862 = {S82316, 159, { &P20947,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCEVOV4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2562972 = {S82317, 171, { &P13452,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2629927 = {S82318, 171, { &P13452,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2632709 = {S82319, 171, { &P13452,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2635659 = {S82320, 171, { &P13452,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2567569 = {S82321, 134, { &P20156,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S500 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2580505 = {S82322, 165, { &P13974,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5670 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3078570 = {S82323, 169, { &P13974,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5670 Build/FROYO) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2583258 = {S82324, 139, { &P19848,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M440S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2670727 = {S82325, 130, { &P17337,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7500 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2666650 = {S82326, 87, { &P17595,&P17296,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.3.8542(Macintosh;Mac OS X (10.6.8);en;kml:2.2;client:Free;type:default)*/
const Device UA2671875 = {S82327, 133, { &P20097,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CT1010 Build/IMM76I.v007) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2672913 = {S82328, 133, { &P20690,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; V360 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673952 = {S82329, 136, { &P20075,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Xperia S Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2706768 = {S82330, 129, { &P18320,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PadFone Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2711817 = {S82331, 136, { &P18320,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PadFone Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2860983 = {S82332, 129, { &P18320,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PadFone Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2711815 = {S82333, 74, { &P17595,&P21721,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:20.0) Gecko/X Firefox/20.0*/
const Device UA2715779 = {S82334, 92, { &P11945,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233s) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2769328 = {S82335, 92, { &P11945,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233s) U2/1.0.0 UCBrowser/9.0.0.295 U2/1.0.0 Mobile*/
const Device UA2784900 = {S82336, 92, { &P11945,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233s) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2878723 = {S82337, 89, { &P11945,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; vi; gt-s5233s) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2737694 = {S82338, 88, { &P18666,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A25) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2751257 = {S82339, 112, { &P23119,&P20706,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia307/07.55; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.36*/
const Device UA2754250 = {S82340, 137, { &P22600,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300K Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2767188 = {S82341, 150, { &P23142,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROIDZ Sport Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2769560 = {S82342, 138, { &P20608,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Edison Build/1.1.8 X-16:29) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2770414 = {S82343, 134, { &P20213,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; N9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2954916 = {S82344, 99, { &P15364,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-SGH-S275M/1.0 SHP/VPP/R5 NetFront/4.2 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA3059888 = {S82345, 106, { &P15364,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-SGH-T189N/T189NUVMG1 SHP/VPP/R5 NetFront/4.2 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA3086241 = {S82346, 73, { &P15364,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-E1265/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2961812 = {S82347, 144, { &P24066,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Lenovo A706_ROW Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2974314 = {S82348, 138, { &P20174,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CINK KING Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974451 = {S82349, 140, { &P18372,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-04D Build/9.1.C.0.475) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023137 = {S82350, 139, { &P15397,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST18i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020161 = {S82351, 144, { &P24454,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iball Andi4di Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3023486 = {S82352, 151, { &P19860,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A9+ Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3028263 = {S82353, 129, { &P24386,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SH-08E Build/S7200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3082696 = {S82354, 129, { &P24386,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; SH-08E Build/S8220) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3030365 = {S82355, 152, { &P21963,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Q800 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3083063 = {S82356, 136, { &P14574,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus S Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3079961 = {S82357, 167, { &P15364,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; HUAWEI P6-C00 Build/HuaweiP6-C00) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3080434 = {S82358, 149, { &P15364,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; P2 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3093172 = {S82359, 167, { &P15364,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; HUAWEI P6-T00 Build/HuaweiP6-T00) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3093217 = {S82360, 171, { &P15364,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-CN; HUAWEI G610-U00 Build/HuaweiG610-U00) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA295468 = {S82361, 146, { &P18030,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SHW-M100S Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019208 = {S82362, 68, { &P18030,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SHW-M100S Build/ECLAIR)*/
const Device UA151651 = {S82363, 139, { &P13201,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; FroyoEris Build/unknown) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA274997 = {S82364, 137, { &P13201,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; FroyoEris Build/unknown) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA291620 = {S82365, 132, { &P13201,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; ERIS Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA523680 = {S82366, 146, { &P13201,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Eris FroshedYo v11 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2128631 = {S82367, 132, { &P13201,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Eris Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA250299 = {S82368, 99, { &P15364,&P7127,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA387788 = {S82369, 43, { &P15364,&P17017,&P17596,&P18092 }};	/*Useragent - Safari (unknown version) CFNetwork/454.11.5*/
const Device UA703983 = {S82370, 42, { &P15364,&P17017,&P17596,&P18092 }};	/*Useragent - Safari (unknown version) CFNetwork/520.2.6*/
const Device UA880948 = {S82371, 44, { &P15364,&P17017,&P17596,&P18092 }};	/*Useragent - Safari (unknown version) CFNetwork/454.10.13*/
const Device UA445552 = {S82372, 64, { &P15364,&P17017,&P17394,&P18092 }};	/*Useragent - Mozilla/5.0 (Sample; MFC Simulator/4.2; like Gecko) NetFront/4.2*/
const Device UA526915 = {S82373, 65, { &P15364,&P7133,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; en) Opera 8.51*/
const Device UA838462 = {S82374, 65, { &P15364,&P7133,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; en) Opera 8.50*/
const Device UA1906577 = {S82375, 65, { &P15364,&P7133,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; en) Opera 8.53*/
const Device UA2264140 = {S82376, 65, { &P15364,&P7133,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; en) Opera 8.54*/
const Device UA664342 = {S82377, 149, { &P12280,&P18117,&P21786,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) 1Password/3.6.5/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA823387 = {S82378, 147, { &P12280,&P18117,&P21786,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) 1Password/3.6.2/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA697870 = {S82379, 139, { &P21444,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PLOYER-MOMO Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024256 = {S82380, 63, { &P21444,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; PLOYER-MOMO Build/GRJ22)*/
const Device UA2460395 = {S82381, 135, { &P18655,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SH-09D Build/SB220) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2205839 = {S82382, 45, { &P23144,&P18580,&P23081,&P18092 }};	/*Useragent - InstacastHD/1.1.3 CFNetwork/609 Darwin/13.0.0*/
const Device UA2229771 = {S82383, 152, { &P12605,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XT875 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2352438 = {S82384, 152, { &P12605,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XT875 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2355141 = {S82385, 152, { &P12605,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT875 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2234741 = {S82386, 133, { &P21244,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E140S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2335922 = {S82387, 75, { &P14545,&P17483,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; kau; lg-x330t) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2518745 = {S82388, 77, { &P14545,&P17483,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; lg-x330t) UCBrowser8.3.0.154/69/352/UCWEB Mobile*/
const Device UA2354788 = {S82389, 130, { &P15767,&P17146,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA2423870 = {S82390, 132, { &P20241,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5570C Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2504609 = {S82391, 139, { &P21367,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ARCHOS 97 XENON Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2505829 = {S82392, 67, { &P15364,&P17188,&P20918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; WinNT4.0; rv:1.7.3) Gecko/X Firefox/0.10.1*/
const Device UA2520839 = {S82393, 127, { &P23058,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN305 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2574849 = {S82394, 136, { &P23058,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN305 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2692770 = {S82395, 144, { &P23058,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn;GiONEE-GN305/Phone Build/IMM76D) AppleWebKit534.30(KHTML,like Gecko)Version/4.0 Mobile Safari/534.30*/
const Device UA2981943 = {S82396, 122, { &P23058,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN305 Build/IMM76D) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2563995 = {S82397, 164, { &P18385,&P18598,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us ; Nexus 7 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2851213 = {S82398, 165, { &P18385,&P18598,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; Nexus 7 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2583397 = {S82399, 136, { &P21330,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PMP5197DULTRA Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2600980 = {S82400, 135, { &P20343,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZP900 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2863063 = {S82401, 136, { &P20343,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZP900S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2768448 = {S82402, 53, { &P18088,&P19826,&P24675,&P18092 }};	/*Useragent - IUC(U;iOS 6.1.3;Zh-cn;320*480;)/UCWEB8.8.0.212/44/997*/
const Device UA2660576 = {S82403, 137, { &P21318,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; el-gr; NovaPad80C104 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2660920 = {S82404, 70, { &P19975,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 505;7.10;8858)*/
const Device UA2937198 = {S82405, 70, { &P19975,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 505;7.10;8862)*/
const Device UA2667592 = {S82406, 158, { &P18114,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ASUS Eee Pad Transformer Prime TF201 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2668824 = {S82407, 146, { &P18114,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Transformer Prime TF201 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2670750 = {S82408, 133, { &P15877,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT910 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673945 = {S82409, 135, { &P21037,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LIFETAB_S9512 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2677185 = {S82410, 137, { &P13141,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9210 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734870 = {S82411, 138, { &P13141,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I727R Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2994398 = {S82412, 138, { &P13141,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E110S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708638 = {S82413, 62, { &P15364,&P18144,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64; rv:20.0) Gecko/X Firefox/20.0*/
const Device UA2711078 = {S82414, 136, { &P24549,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; E2001 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2783369 = {S82415, 158, { &P13018,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT11i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2729183 = {S82416, 26, { &P18442,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-I317/100.40102*/
const Device UA2840583 = {S82417, 26, { &P18442,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-T889/100.40101*/
const Device UA2952628 = {S82418, 26, { &P18442,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SPH-L900/100.40102*/
const Device UA2984374 = {S82419, 26, { &P18442,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-N7100/100.40102*/
const Device UA3079520 = {S82420, 152, { &P18442,&P17484,&P17606,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.5;FBBV/X;FBDM/{density=2.0,width=720,height=1280};FBLC/en_US;FBCR/AT&amp-T;FBPN/com.facebook.katana;FBDV/SAMSUNG-SGH-I317;FBSV/4.1.2;]*/
const Device UA2730225 = {S82421, 93, { &P14753,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian96-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2738150 = {S82422, 93, { &P14753,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; Nokia N96) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2738236 = {S82423, 104, { &P11854,&P5890,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaE71*/
const Device UA2772697 = {S82424, 155, { &P13644,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; PC36100 Build/GRJ90) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2774160 = {S82425, 170, { &P18505,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2795590 = {S82426, 121, { &P12280,&P18117,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0*/
const Device UA2867511 = {S82427, 123, { &P12280,&P18117,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0.2*/
const Device UA2891694 = {S82428, 123, { &P12280,&P18117,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0.1*/
const Device UA3011410 = {S82429, 123, { &P12280,&P18117,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0.3*/
const Device UA3018006 = {S82430, 123, { &P12280,&P18117,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/2.0.4*/
const Device UA2864336 = {S82431, 11, { &P23333,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M5385*/
const Device UA2896731 = {S82432, 152, { &P20868,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M040 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2879573 = {S82433, 141, { &P18451,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26ii Build/6.2.B.0.200) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2958074 = {S82434, 140, { &P24127,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; Revo_Lead Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022520 = {S82435, 137, { &P18687,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P708g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023532 = {S82436, 136, { &P18687,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P705 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2972109 = {S82437, 136, { &P24156,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; 202F Build/V16R45B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3023522 = {S82438, 136, { &P24156,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; 202F Build/V17R47F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092370 = {S82439, 58, { &P24156,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; 202F Build/V17R47F)*/
const Device UA3016239 = {S82440, 94, { &P15845,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC5-06) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3064703 = {S82441, 169, { &P20005,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A65 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3081277 = {S82442, 138, { &P22591,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3062368 = {S82443, 142, { &P15438,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Sensation Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3083763 = {S82444, 138, { &P15438,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Sensation Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA68658 = {S82445, 35, { &P11880,&P5890,&P4030,&P18092 }};	/*Useragent - Nokia 5320/UCBrowser7.7.0.81/28/800*/
const Device UA69517 = {S82446, 10, { &P11880,&P5890,&P4030,&P18092 }};	/*Useragent - Nokia 5320*/
const Device UA87727 = {S82447, 9, { &P11880,&P5890,&P4030,&P18092 }};	/*Useragent - NOKIA5320*/
const Device UA255981 = {S82448, 119, { &P15364,&P17285,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.71.128 Chrome/14.0.835.202 Safari/535.1*/
const Device UA342023 = {S82449, 119, { &P15364,&P17285,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.64.361 Chrome/13.0.782.218 Safari/535.1*/
const Device UA398670 = {S82450, 126, { &P15364,&P17285,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.72.698 Chrome/14.0.835.202 Safari/535.1*/
const Device UA638307 = {S82451, 126, { &P15364,&P17285,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.64.319 Chrome/13.0.782.215 Safari/535.1*/
const Device UA872445 = {S82452, 120, { &P15364,&P17285,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.24 (KHTML, like Gecko) RockMelt/0.9.56.357 Chrome/11.0.696.71 Safari/534.24*/
const Device UA875379 = {S82453, 139, { &P15364,&P17285,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.13 (KHTML, like Gecko) RockMelt/0.9.46.126 Chrome/9.0.597.107 Safari/534.13*/
const Device UA895539 = {S82454, 140, { &P15364,&P17285,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) RockMelt/0.9.50.518 Chrome/10.0.648.205 Safari/534.16*/
const Device UA915876 = {S82455, 121, { &P15364,&P17285,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.30 (KHTML, like Gecko) RockMelt/0.9.62.313 Chrome/12.0.742.122 Safari/534.30*/
const Device UA301641 = {S82456, 12, { &P16633,&P22784,&P17470,&P18092 }};	/*Useragent - MicromaxX505*/
const Device UA694777 = {S82457, 74, { &P15364,&P7133,&P22306,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:2.0.1) Gecko/X Firefox/4.0.1 SeaMonkey/2.1*/
const Device UA789401 = {S82458, 100, { &P15364,&P22202,&P18198,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.24) Gecko/X Fedora/3.1.16-1.fc14 Thunderbird/3.1.16*/
const Device UA835775 = {S82459, 95, { &P15364,&P7127,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:11.0) Gecko/X Firefox/11.0,gzip(gfe) (via translate.google.com)*/
const Device UA903629 = {S82460, 155, { &P12814,&P24691,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I777 Build/Task650 & Ktoonsez AOKP) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2025474 = {S82461, 70, { &P15575,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.3;Zh_cn;GT-I9100;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA2809581 = {S82462, 71, { &P15575,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.3;Zh_cn;GT-I9100G;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA2828914 = {S82463, 70, { &P15575,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.4;Zh_cn;GT-I9100;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA2146764 = {S82464, 47, { &P15364,&P5690,&P18105,&P18092 }};	/*Useragent - Opera/9.10 (Windows NT 5.1; U; pl) Version/10.9*/
const Device UA2212429 = {S82465, 62, { &P15364,&P5690,&P18105,&P18092 }};	/*Useragent - Opera/9.10 (Windows NT 5.1; U; pl) Presto/2.1.15 Version/10.10*/
const Device UA2336062 = {S82466, 64, { &P15364,&P5690,&P18105,&P18092 }};	/*Useragent - Opera/9.12 (Windows NT 5.1; U; ru) Presto/8.0.6.61 Version/10.10*/
const Device UA2179280 = {S82467, 159, { &P20262,&P18066,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Karbonn_A18 Build/KarbonnA18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2224286 = {S82468, 136, { &P23937,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Y2 Build/HCL ME Tablet Y2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2231684 = {S82469, 137, { &P21089,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE T22 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2460936 = {S82470, 136, { &P18647,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T-02D Build/V19R50B) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2386774 = {S82471, 178, { &P18648,&P5043,&P24672,&P18092 }};	/*Useragent - Nokia306/2.0 (03.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia306) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2401341 = {S82472, 158, { &P17968,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Galaxy Precedent Build/AreaRom Test)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2481059 = {S82473, 171, { &P19862,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax A110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2464745 = {S82474, 140, { &P21491,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en_us; Micromax A80 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2467472 = {S82475, 171, { &P17324,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2506191 = {S82476, 171, { &P17324,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2506561 = {S82477, 171, { &P17324,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2848225 = {S82478, 171, { &P17324,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2529713 = {S82479, 133, { &P23819,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; StarPAD 9 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2530079 = {S82480, 148, { &P22672,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-ph; Cloudfone_Excite350e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2567395 = {S82481, 137, { &P18243,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P880 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2579909 = {S82482, 109, { &P23555,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; ALCATEL; ONE TOUCH 5040X)*/
const Device UA2583038 = {S82483, 140, { &P21889,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; TAC-90011 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2607545 = {S82484, 144, { &P21812,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GSmart Rio R1 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2649896 = {S82485, 141, { &P12798,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; LT18i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2653074 = {S82486, 65, { &P12798,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Xperia Arc S Build/JZO54K)*/
const Device UA2706560 = {S82487, 151, { &P12798,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-gb; SonyEricssonLT18i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2675267 = {S82488, 139, { &P12616,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WT19i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2693655 = {S82489, 172, { &P18436,&P16110,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2; WOW64; Trident/4.0; Touch; .NET4.0E; .NET4.0C; Tablet PC 2.0; .NET CLR 3.5.30729; .NET CLR 2.0.50727; .NET CLR 3.0.30729)*/
const Device UA2737067 = {S82490, 151, { &P18673,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2767442 = {S82491, 152, { &P18673,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562i Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2798924 = {S82492, 156, { &P20141,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S5301 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2754635 = {S82493, 134, { &P20414,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SmartTabII10 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755415 = {S82494, 137, { &P20075,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xperia S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774319 = {S82495, 147, { &P12879,&P6546,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.1; GT-P7510 Build/HMJ37) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2780099 = {S82496, 141, { &P18611,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC-T329d/1.15.1401.1) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934943 = {S82497, 127, { &P18611,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T329d) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2868747 = {S82498, 143, { &P21438,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F120K/30a Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108278 = {S82499, 143, { &P21438,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F120K/30b Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2901174 = {S82500, 164, { &P20183,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3013161 = {S82501, 103, { &P15798,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Explorer_A310e) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3020515 = {S82502, 103, { &P15798,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Explorer_A310e) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3022638 = {S82503, 137, { &P18679,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-B5330 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023199 = {S82504, 138, { &P18679,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-B5330L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024183 = {S82505, 98, { &P21020,&P18111,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Micromax A116) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3065672 = {S82506, 141, { &P15767,&P18432,&P24712,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 OverDrive Media Console/3.0.1*/
const Device UA3080979 = {S82507, 126, { &P18282,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3115838 = {S82508, 106, { &P15492,&P17484,&P18328,&P18092 }};	/*Useragent - SAMSUNG-GT-C3322i Opera/9.80 (J2ME/MIDP; Opera Mini/7.1.33886/31.1475; U; en) Presto/2.8.119 Version/11.10*/
const Device UA64067 = {S82509, 79, { &P11894,&P17486,&P3402,&P18092 }};	/*Useragent - MOT-W388/0.1.41 UP.Browser/6.3.0.6.c.24 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54821 = {S82510, 83, { &P14546,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK790a/R1KG Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA6486 = {S82511, 175, { &P12478,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; CMLeclair1.3; es-es; Boston Build/ERE27; Orange_Boston Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32180 = {S82512, 178, { &P12478,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; CMLeclair1.2; pt-pt; CMLBoston Build/ERE27; Orange_Boston Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32182 = {S82513, 178, { &P12478,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; 2.1CMLMod1.5; es-es; CMLMod1.5 Build/ERE27; Orange_Boston Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32188 = {S82514, 182, { &P12478,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; firetrap Sugar1.2; pt-pt; Sugar1.2 Build/ERE27; Orange_Boston Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA525642 = {S82515, 135, { &P12478,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Boston Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA533228 = {S82516, 134, { &P12478,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Boston Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA23504 = {S82517, 110, { &P15610,&P17483,&P4848,&P18092 }};	/*Useragent - LG-KE990/v10a Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA60870 = {S82518, 110, { &P15610,&P17483,&P4848,&P18092 }};	/*Useragent - LG/KE990/v10a Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61778 = {S82519, 39, { &P14327,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KF510 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA62568 = {S82520, 73, { &P14424,&P17486,&P4203,&P18092 }};	/*Useragent - Motorola-w385 Obigo/Q04C1 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA69405 = {S82521, 80, { &P15210,&P5448,&P4779,&P18092 }};	/*Useragent - SAMSUNG-GT-i8000/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA203949 = {S82522, 54, { &P17576,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SH38 UP.Browser/6.2_7.2.7.1.K.3.352 (GUI) MMP/2.0*/
const Device UA140531 = {S82523, 91, { &P18254,&P6704,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; Win 9x 4.90; .NET CLR 1.1.4322; MSIECrawler)*/
const Device UA485600 = {S82524, 78, { &P18254,&P6704,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; .NET CLR 1.1.4322; MSIECrawler)*/
const Device UA678767 = {S82525, 59, { &P18254,&P6704,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; MSIECrawler)*/
const Device UA1853907 = {S82526, 72, { &P18254,&P6704,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; Win 9x 4.90; MSIECrawler)*/
const Device UA3057918 = {S82527, 76, { &P18254,&P6704,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; MSIECrawler);c2VvLWNyYXdsZXI=*/
const Device UA228343 = {S82528, 139, { &P12530,&P5448,&P4779,&P18092 }};	/*Useragent - PPC; 240x320; HTC_P3450/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA656344 = {S82529, 98, { &P17475,&P5768,&P17347,&P18092 }};	/*Useragent - Nokia6300/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6300/UC Browser8.0.3.107/69/444*/
const Device UA588927 = {S82530, 76, { &P17595,&P18584,&P17415,&P18092 }};	/*Useragent - Safari/7534.53.10 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookPro8%2C1)*/
const Device UA652057 = {S82531, 76, { &P17595,&P18584,&P17415,&P18092 }};	/*Useragent - Safari/7534.53.10 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookAir4%2C1)*/
const Device UA664939 = {S82532, 72, { &P17595,&P18584,&P17415,&P18092 }};	/*Useragent - Safari/7534.53.10 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacPro3%2C1)*/
const Device UA707865 = {S82533, 69, { &P17595,&P18584,&P17415,&P18092 }};	/*Useragent - Safari/7534.55.3 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (iMac8%2C1)*/
const Device UA761324 = {S82534, 70, { &P17595,&P18584,&P17415,&P18092 }};	/*Useragent - Safari/7534.55.3 CFNetwork/520.3.2 Darwin/11.3.0 (i386) (MacBook4%2C1)*/
const Device UA772792 = {S82535, 73, { &P17595,&P18584,&P17415,&P18092 }};	/*Useragent - Safari/7534.53.10 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (Macmini5%2C1)*/
const Device UA491233 = {S82536, 67, { &P17595,&P17294,&P18333,&P18092 }};	/*Useragent - iTunes/9.2.1 (Macintosh; PPC Mac OS X 10.4.11) AppleWebKit/533.19.4*/
const Device UA628292 = {S82537, 69, { &P17595,&P17294,&P18333,&P18092 }};	/*Useragent - iTunes/9.2.1 (Macintosh; Intel Mac OS X 10.4.11) AppleWebKit/533.19.4*/
const Device UA669677 = {S82538, 116, { &P14862,&P5204,&P17437,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2700c-2/09.97; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.4.0.34.8*/
const Device UA2026913 = {S82539, 159, { &P12562,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID2 Global Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033680 = {S82540, 163, { &P12562,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID2 Global Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027662 = {S82541, 131, { &P20557,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; MZ606 Build/I.7.1-34) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2030881 = {S82542, 140, { &P22177,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0UI; en-in; 3G7307 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031811 = {S82543, 142, { &P21228,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Lenovo A750 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2475085 = {S82544, 133, { &P21228,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo A750 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3042608 = {S82545, 142, { &P21228,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A750 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033423 = {S82546, 136, { &P12788,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PG86100 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2414065 = {S82547, 137, { &P12788,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCEVO3D Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034258 = {S82548, 135, { &P21534,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; vivo V2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2644240 = {S82549, 128, { &P21534,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; vivo V2 Build/GRJ90) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2234908 = {S82550, 152, { &P21596,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Mobiistar_Touch_Lai_5.0 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2338478 = {S82551, 144, { &P20121,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; CloudfoneThrill Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2338760 = {S82552, 161, { &P24022,&P5629,&P15760,&P18092 }};	/*Useragent - Spice Mi-285 Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Spice Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2354034 = {S82553, 157, { &P15649,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2481374 = {S82554, 158, { &P15649,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2365520 = {S82555, 113, { &P15364,&P5690,&P22525,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.22 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/535.22 Midori/0.4*/
const Device UA2409874 = {S82556, 68, { &P15364,&P5690,&P22525,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en-us) Midori/0.4*/
const Device UA2749844 = {S82557, 115, { &P15364,&P5690,&P22525,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.6 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/537.6 Midori/"0"."4"*/
const Device UA2381521 = {S82558, 118, { &P16003,&P17484,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; gt-s5233a) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile*/
const Device UA3098987 = {S82559, 115, { &P16003,&P17484,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; fa; gt-s5233a) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile*/
const Device UA2405952 = {S82560, 153, { &P15364,&P5690,&P19966,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.4.2.X (Second Life Release; default skin) Safari/533.3*/
const Device UA2469283 = {S82561, 169, { &P13018,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; MT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2428103 = {S82562, 86, { &P15364,&P18145,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (X11; FreeBSD 8.1-RELEASE i386; Edition Next) Presto/2.12.388 Version/12.10*/
const Device UA2438138 = {S82563, 59, { &P15364,&P18110,&P23764,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; U; Chumby; Linux) Flash Lite 3.1.7*/
const Device UA2459027 = {S82564, 125, { &P20464,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; N-06D Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2503090 = {S82565, 133, { &P21370,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; SHV-E140L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2505762 = {S82566, 78, { &P15364,&P18118,&P20918,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-AU; rv:1.7) Gecko/X Firefox/0.9.3 (Ubuntu)*/
const Device UA2528405 = {S82567, 71, { &P18283,&P18111,&P24673,&P18092 }};	/*Useragent - JUC (Linux; U; 4.1.1; zh-cn; GT-I9300; 720*1280) UCWEB8.7.4.225/145/800*/
const Device UA3064799 = {S82568, 93, { &P18283,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9300) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA2670005 = {S82569, 161, { &P12972,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2566929 = {S82570, 138, { &P18697,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I415 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2569590 = {S82571, 155, { &P18464,&P18066,&P21167,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2579754 = {S82572, 154, { &P21600,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar Touch Kem 462 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583127 = {S82573, 139, { &P15850,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LU6200 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583231 = {S82574, 138, { &P18002,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A760S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2599314 = {S82575, 131, { &P23506,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GEM7020 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2599948 = {S82576, 136, { &P22126,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Torque Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2666986 = {S82577, 127, { &P12973,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A101 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667108 = {S82578, 158, { &P12973,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2704360 = {S82579, 138, { &P20855,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Scroll Excel 3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2705080 = {S82580, 171, { &P21141,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; Lenovo P770 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33482*/
const Device UA2732254 = {S82581, 153, { &P23540,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MITO A222 Build/MITO-A222-V00.00.01) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734598 = {S82582, 161, { &P14183,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I997 Build/JDQ39; Carbon-CARBON-JB-v1.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937211 = {S82583, 162, { &P14183,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I997 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2988799 = {S82584, 161, { &P14183,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I997 Build/JDQ39; Carbon-Carbon-JB-v1.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2767908 = {S82585, 150, { &P23145,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROIDZ Force Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2771074 = {S82586, 136, { &P20240,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E971 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2796053 = {S82587, 146, { &P23139,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; WALTON_Primo-N1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2863760 = {S82588, 130, { &P15767,&P17146,&P20783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/9A405 Safari/7534.48.3*/
const Device UA2865731 = {S82589, 139, { &P23503,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; MITOA800 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2872358 = {S82590, 93, { &P22872,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S5282) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA3020413 = {S82591, 132, { &P18554,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; BN Nook HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021960 = {S82592, 144, { &P18554,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023108 = {S82593, 130, { &P18437,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N8000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022178 = {S82594, 153, { &P20018,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993D Build/ICECREAM) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023933 = {S82595, 152, { &P20018,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993 Build/ICECREAM) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3080842 = {S82596, 142, { &P22737,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Micromax A115 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081032 = {S82597, 137, { &P22396,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5312 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111515 = {S82598, 138, { &P22396,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5312B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA54375 = {S82599, 145, { &P13882,&P5267,&P3834,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8500T/1.0; U; Bada/1.0; en-us) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA21122 = {S82600, 77, { &P16471,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-F101(RDS)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2156324 = {S82601, 75, { &P16471,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-F101(Optus) NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2805883 = {S82602, 78, { &P16471,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-F101(Tigo)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22446 = {S82603, 43, { &P12252,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-X760/X760_Z10NL_GD_B07A202 Obigo/Q03C*/
const Device UA24635 = {S82604, 63, { &P12252,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-X760-vegas-orange/X760_Z15ES_FS_B07A104_ORANGE Obigo/Q03C*/
const Device UA28486 = {S82605, 43, { &P12252,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-X760/X760_Z10UK_GD_B07A201 Obigo/Q03C*/
const Device UA45956 = {S82606, 63, { &P12252,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-X760-vegas-orange/X760_Z15UK_FS_B07A105_ORANGE Obigo/Q03C*/
const Device UA64543 = {S82607, 63, { &P12252,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-X760-vegas-orange/X760_Z15UK_FS_B07A106_ORANGE Obigo/Q03C*/
const Device UA64744 = {S82608, 43, { &P12252,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-X760/X760_Z10UK_GD_B07A202 Obigo/Q03C*/
const Device UA32991 = {S82609, 116, { &P12817,&P17484,&P3834,&P18092 }};	/*Useragent - SAMSUNG-GT-I6712/1.0 SHP/VPP/R5 Dolfin/2.0 NexPlayer/3.0 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B*/
const Device UA53596 = {S82610, 65, { &P13990,&P17483,&P3459,&P18092 }};	/*Useragent - LG-GW382f/V10d Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA616392 = {S82611, 77, { &P18095,&P18094,&P22575,&P18092 }};	/*Useragent - Opera/9.80 (S60; SymbOS; Opera Tablet/9174; U; en) Presto/2.7.81 Version/10.5*/
const Device UA2793248 = {S82612, 26, { &P15575,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-I9100/100.40102*/
const Device UA2153987 = {S82613, 130, { &P21238,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; VIGO10 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2257353 = {S82614, 133, { &P22142,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; da-dk; EN-TAB410 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2355028 = {S82615, 138, { &P14433,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9003 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2449964 = {S82616, 138, { &P14433,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9003 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2361090 = {S82617, 140, { &P15767,&P17090,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3*/
const Device UA2368450 = {S82618, 140, { &P15767,&P17090,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3*/
const Device UA2389822 = {S82619, 154, { &P18283,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2486404 = {S82620, 174, { &P15501,&P5477,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E16i Build/2.1.1.A.0.53) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2455280 = {S82621, 133, { &P23469,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MID9106CM Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2467202 = {S82622, 138, { &P20857,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Scroll Excel 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2482485 = {S82623, 225, { &P14916,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung gt-c3303i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile, SAMSUNG-GT-C3303i/C3303iDDKF2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA2483564 = {S82624, 99, { &P14916,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung gt-c3303i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2512529 = {S82625, 176, { &P18113,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2579874 = {S82626, 151, { &P15241,&P5843,&P24675,&P18092 }};	/*Useragent - Nokia300/5.0 (07.44) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia300) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2539944 = {S82627, 134, { &P21603,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AMID-9743G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2564236 = {S82628, 164, { &P18280,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One S Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937707 = {S82629, 165, { &P18280,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One S Build/JDQ39E; LiquidSmooth-Liquid-JB-v2.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2564899 = {S82630, 165, { &P14186,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3058118 = {S82631, 169, { &P14186,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5830 Build/FROYO) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2600332 = {S82632, 128, { &P17686,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MZ601 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2602441 = {S82633, 128, { &P17686,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MZ601 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2602987 = {S82634, 138, { &P18622,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A820L Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2657444 = {S82635, 145, { &P23457,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GIONEE_Ctrl_V4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2663890 = {S82636, 134, { &P23170,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AX5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666971 = {S82637, 133, { &P20729,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A27 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2677051 = {S82638, 126, { &P18733,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M532 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2710483 = {S82639, 136, { &P22613,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E430 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2749851 = {S82640, 136, { &P20478,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; JY-G2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867202 = {S82641, 158, { &P22641,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; C1504 Build/11.3.A.0.47) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2870130 = {S82642, 149, { &P20182,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xolo_X900 Build/AZ510_ICS_02.36I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2773124 = {S82643, 137, { &P20124,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ZTE V970 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937915 = {S82644, 137, { &P20124,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZTE V970 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773606 = {S82645, 139, { &P20439,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SOL21 Build/9.1.D.0.395) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2794535 = {S82646, 160, { &P18622,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A820L Build/HOANG MINH H?U_VEGAVIETTEAM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3046349 = {S82647, 163, { &P18622,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A820L Build/HOA?NG MINH H??rU_VEGAVIETTEAM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2944561 = {S82648, 165, { &P19998,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2961188 = {S82649, 167, { &P19998,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI C8812E Build/HuaweiC8812E) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2944510 = {S82650, 167, { &P12871,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2955950 = {S82651, 132, { &P24482,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-Masstel-M11/1.0 Linux/2.6.35.7 Android/2.3.5 Release/03.21.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2956663 = {S82652, 132, { &P24482,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-Masstel-M11/1.0 Linux/2.6.35.7 Android/2.3.5 Release/04.03.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3000570 = {S82653, 132, { &P24482,&P5629,&P15760,&P18092 }};	/*Useragent - sprd-Masstel-M11/1.0 Linux/2.6.35.7 Android/2.3.5 Release/05.16.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2975564 = {S82654, 131, { &P21334,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A8 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3036637 = {S82655, 167, { &P15877,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3021491 = {S82656, 136, { &P14017,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P990 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021833 = {S82657, 137, { &P24105,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Archos 80 Xenon Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3030387 = {S82658, 157, { &P22382,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3032951 = {S82659, 156, { &P18680,&P18066,&P24339,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA3048365 = {S82660, 78, { &P18436,&P21460,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.3; WOW64; Trident/6.0; Touch)*/
const Device UA3090829 = {S82661, 83, { &P18436,&P21460,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.3; Win64; x64; Trident/6.0; Touch)*/
const Device UA3083225 = {S82662, 141, { &P23254,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Celkon A119Q Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083940 = {S82663, 134, { &P18740,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; U7015 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3088906 = {S82664, 128, { &P12280,&P22882,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 7_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293*/
const Device UA3103598 = {S82665, 141, { &P12280,&P22882,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 7_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA3107591 = {S82666, 145, { &P12280,&P22882,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 7_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7*/
const Device UA20274 = {S82667, 89, { &P14246,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) HPiPAQ610/1.0 (PPC; 240x320)*/
const Device UA58674 = {S82668, 135, { &P14246,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) HPiPAQ610/1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320)*/
const Device UA41580 = {S82669, 161, { &P21487,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en; LG-P993 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA57325 = {S82670, 164, { &P21487,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; LG-P993 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2812052 = {S82671, 129, { &P21487,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; LG-P993 Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA42677 = {S82672, 118, { &P15028,&P5096,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.2.2;Zh_cn;LG-P350;240*320;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355*/
const Device UA421550 = {S82673, 69, { &P15028,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2.2;Zh_cn;LG-P350;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA67967 = {S82674, 35, { &P14188,&P5129,&P4759,&P18092 }};	/*Useragent - Nokia5233/UC Browser7.6.1.82/50/351*/
const Device UA132689 = {S82675, 147, { &P13251,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Dell Streak Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA581182 = {S82676, 43, { &P12588,&P5055,&P24665,&P18092 }};	/*Useragent - NokiaC6-01/UC Browser8.0.4.121/50/352/UCWEB*/
const Device UA497166 = {S82677, 98, { &P15364,&P7133,&P22481,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; pl; rv:1.9.0.14; onet) Gecko/X Firefox/3.0.14 Flock/2.5.3*/
const Device UA716197 = {S82678, 76, { &P15364,&P7127,&P20459,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:9.0.1) Gecko/X Firefox/9.0.1 SeaMonkey/2.6.1*/
const Device UA734623 = {S82679, 142, { &P21410,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; bq Verne Plus Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1907546 = {S82680, 145, { &P21410,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; bq Verne Plus 3G Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA798879 = {S82681, 108, { &P21401,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LNV-Lenovo_A68e/Lenovo A68e_S011_X; 480*320; CTC/2.0) Wangel/23*/
const Device UA1919626 = {S82682, 152, { &P21401,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo A68e Build/Lenovo_A68e_S016_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023500 = {S82683, 152, { &P21401,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo A68e Build/Lenovo_A68e_S018_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026375 = {S82684, 152, { &P21401,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo A68e Build/Lenovo_A68e_S019_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA893286 = {S82685, 139, { &P21877,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; TAB-PROTAB2.4XL Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1997252 = {S82686, 172, { &P16292,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03H; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2012244 = {S82687, 172, { &P16292,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2166884 = {S82688, 85, { &P14985,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s8530) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2203465 = {S82689, 87, { &P17968,&P5477,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.1-update1; Linux; Opera Mobi/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2222538 = {S82690, 136, { &P21566,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MIDC010PR001 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2385246 = {S82691, 182, { &P13107,&P5043,&P24672,&P18092 }};	/*Useragent - NokiaC2-06/2.0 (07.48) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-06) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile*/
const Device UA2462051 = {S82692, 171, { &P16356,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2416063 = {S82693, 128, { &P20269,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A211 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2426234 = {S82694, 132, { &P23934,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; BB EH7LE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2839861 = {S82695, 132, { &P23934,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; BB_EH7LE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2428291 = {S82696, 98, { &P15655,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s3850) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2527752 = {S82697, 140, { &P23345,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; Onyx Calypso 9.7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2530937 = {S82698, 153, { &P13644,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EVO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2531113 = {S82699, 169, { &P12798,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2531548 = {S82700, 168, { &P13018,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MT11i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3108110 = {S82701, 172, { &P13018,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT11i Build/4.1.B.0.587) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2557287 = {S82702, 132, { &P23806,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ar-eg; S18 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2844094 = {S82703, 125, { &P23806,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; S18 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2558942 = {S82704, 140, { &P20495,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; OmegaS200 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580135 = {S82705, 158, { &P21362,&P18066,&P21729,&P18092 }};	/*Useragent - OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SYMPHONY_W90 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659272 = {S82706, 160, { &P18300,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T769 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957594 = {S82707, 175, { &P18300,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-T769 Build/JZO54K; LiquidSmooth-LiquidAllStar) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2664327 = {S82708, 136, { &P22649,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; IQ443 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2869690 = {S82709, 172, { &P12056,&P5204,&P24677,&P18092 }};	/*Useragent - Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7210supernova) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2754173 = {S82710, 132, { &P21603,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AMID-9743G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2784897 = {S82711, 160, { &P12841,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC PH39100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2773874 = {S82712, 135, { &P21869,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZP950+ Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879171 = {S82713, 134, { &P21869,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZP950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2999702 = {S82714, 135, { &P21869,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZP950H Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784841 = {S82715, 158, { &P18450,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST23i Build/11.0.A.5.5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2866057 = {S82716, 166, { &P22609,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-S6312 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3020949 = {S82717, 170, { &P22609,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6312 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2878849 = {S82718, 135, { &P23497,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; P-03E Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892726 = {S82719, 138, { &P18472,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Razr HD Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3077162 = {S82720, 143, { &P18472,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Droid Razr HD Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2943642 = {S82721, 164, { &P13042,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC Incredible S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2943780 = {S82722, 88, { &P19779,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A1+) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2944276 = {S82723, 161, { &P18321,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5300 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2997075 = {S82724, 146, { &P21261,&P18598,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3021445 = {S82725, 92, { &P18241,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; LG-E400) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3022207 = {S82726, 131, { &P24078,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; E3 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020938 = {S82727, 97, { &P18652,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A87) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3034036 = {S82728, 140, { &P22585,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SPH-L720 Build/JWR66V.S014) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3065402 = {S82729, 187, { &P15794,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3080250 = {S82730, 133, { &P24077,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; H120 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094759 = {S82731, 134, { &P21909,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-03E Build/10.1.E.0.305) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA30657 = {S82732, 107, { &P12332,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-T429/T429UVHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA21413 = {S82733, 87, { &P16182,&P17485,&P3600,&P18092 }};	/*Useragent - SonyEricssonK600i/R2BA Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21451 = {S82734, 86, { &P16182,&P17485,&P3600,&P18092 }};	/*Useragent - SonyEricssonK600c/R1K Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23427 = {S82735, 117, { &P15605,&P6476,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2230M/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA49989 = {S82736, 76, { &P15605,&P6476,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2230M/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA26128 = {S82737, 105, { &P12149,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonZ780i/R3DA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.2*/
const Device UA251042 = {S82738, 131, { &P17595,&P17297,&P22452,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1*/
const Device UA709619 = {S82739, 131, { &P17595,&P17297,&P22452,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1*/
const Device UA745781 = {S82740, 131, { &P17595,&P17297,&P22452,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1*/
const Device UA2350878 = {S82741, 131, { &P17595,&P17297,&P22452,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1*/
const Device UA512145 = {S82742, 97, { &P15364,&P17285,&P22333,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.7) Gecko/X Firefox/3.0.7 CometBird/3.0.7*/
const Device UA575483 = {S82743, 81, { &P15364,&P22740,&P18160,&P18092 }};	/*Useragent - Mozilla/5.0 (Maemo; Linux armv7l; rv:10.0a1) Gecko/X Firefox/10.0a1 Fennec/10.0a1*/
const Device UA597159 = {S82744, 81, { &P15364,&P22740,&P18160,&P18092 }};	/*Useragent - Mozilla/5.0 (Maemo; Linux armv7l; rv:10.0.2) Gecko/X Firefox/10.0.2 Fennec/10.0.2*/
const Device UA2640183 = {S82745, 81, { &P15364,&P22740,&P18160,&P18092 }};	/*Useragent - Mozilla/5.0 (Maemo; Linux armv7l; rv:10.0.1) Gecko/X Firefox/10.0.1 Fennec/10.0.1*/
const Device UA924069 = {S82746, 82, { &P17595,&P17296,&P18191,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:10.0.1) Gecko/X Thunderbird/10.0.1*/
const Device UA608808 = {S82747, 136, { &P13644,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PC36100 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA679875 = {S82748, 143, { &P13644,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PC36100 Build/MIUIANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA872498 = {S82749, 136, { &P13644,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC EVO Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA899441 = {S82750, 139, { &P13644,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Evo 4G Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2445785 = {S82751, 132, { &P13644,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EVO Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1942560 = {S82752, 83, { &P15238,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-S5270K/S5270KDDKK1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA647275 = {S82753, 60, { &P15364,&P17188,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0; rv:9.0.1) Gecko/X Firefox/9.0.1*/
const Device UA1882289 = {S82754, 115, { &P15364,&P7127,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en) AppleWebKit/522.12.1 (KHTML, like Gecko) Version/3.0.1 Safari/522.12.2*/
const Device UA1958036 = {S82755, 174, { &P14923,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TeamDRH ICS for GTablet Build/IML77; CyanogenMod-TeamDRH) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2261426 = {S82756, 85, { &P14980,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-b7722) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2202192 = {S82757, 140, { &P12280,&P18117,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.1.1m*/
const Device UA2237427 = {S82758, 140, { &P12280,&P18117,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.1.2m*/
const Device UA2367351 = {S82759, 140, { &P12280,&P18117,&P19962,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.1.3m*/
const Device UA2262215 = {S82760, 151, { &P15649,&P5096,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2347660 = {S82761, 171, { &P18645,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-L710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SPH-L710, 34, 1.6.0)*/
const Device UA2795219 = {S82762, 187, { &P18645,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SAMSUNG-SGH-I747, 42, 1.9.1)*/
const Device UA2365961 = {S82763, 135, { &P23949,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; H9500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2367609 = {S82764, 134, { &P18680,&P18110,&P19815,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Safari/535.19 Silk-Accelerated=false*/
const Device UA2381863 = {S82765, 132, { &P19774,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PMP3370B Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2383906 = {S82766, 125, { &P14182,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5333) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2422888 = {S82767, 139, { &P15364,&P17819,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Ubuntu/11.10 Chromium/23.0.1271.64 Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2424290 = {S82768, 137, { &P15364,&P17819,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.11 (KHTML, like Gecko) Ubuntu/11.10 Chromium/23.0.1271.64 Chrome/23.0.1271.64 Safari/537.11*/
const Device UA2430119 = {S82769, 37, { &P15775,&P5448,&P20765,&P18092 }};	/*Useragent - NOKIA-Lumia 800/UCWEB2.7.0.223/47/355*/
const Device UA2457107 = {S82770, 100, { &P13157,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC_Sensation_XE_with_Beats_Audio_Z715e; 540*960) UCWEB7.9.0.94/139/352*/
const Device UA2482061 = {S82771, 100, { &P13157,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC_Sensation_XE_with_Beats_Audio_Z715e; 540*960) UCWEB7.9.0.94/139/444*/
const Device UA2574612 = {S82772, 101, { &P13157,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC_Sensation_XE_with_Beats_Audio_Z715e; 540*960) UCWEB7.9.4.145/139/800*/
const Device UA2492440 = {S82773, 57, { &P15364,&P7127,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0) Presto/2.12.388 Version/12.13*/
const Device UA2697094 = {S82774, 178, { &P15241,&P5843,&P24671,&P18092 }};	/*Useragent - Nokia300/5.0 (07.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia300) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2537991 = {S82775, 93, { &P12456,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian73-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2770267 = {S82776, 92, { &P12456,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE73) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2601027 = {S82777, 141, { &P20089,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SO-01E Build/9.0.G.1.108) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2584088 = {S82778, 135, { &P20117,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IQ440 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2603887 = {S82779, 93, { &P14367,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA6710s) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2600806 = {S82780, 87, { &P23359,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A2) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2601729 = {S82781, 138, { &P21373,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F240K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2661567 = {S82782, 93, { &P16356,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7000) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2666774 = {S82783, 138, { &P18722,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IM-A830L Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2779914 = {S82784, 138, { &P18722,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IM-A830S Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2860694 = {S82785, 138, { &P18722,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IM-A830K Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2670913 = {S82786, 136, { &P19917,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P765 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676385 = {S82787, 140, { &P18465,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Amazon Kindle Fire Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2707991 = {S82788, 137, { &P18722,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IM-A830L Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2710101 = {S82789, 137, { &P18722,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IM-A830S Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734697 = {S82790, 134, { &P18722,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SKY-A830 Build/LTE) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2859962 = {S82791, 137, { &P18722,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IM-A830K Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2719410 = {S82792, 165, { &P18652,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A87 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2737796 = {S82793, 90, { &P17625,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; ST25i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2753367 = {S82794, 158, { &P18304,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2773570 = {S82795, 131, { &P23896,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; S5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784753 = {S82796, 167, { &P15877,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2784821 = {S82797, 161, { &P15802,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC_Amaze_4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2868896 = {S82798, 161, { &P15802,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Amaze 4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2937366 = {S82799, 170, { &P22585,&P18598,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L720 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SPH-L720, 47, 1.9.7)*/
const Device UA3022376 = {S82800, 183, { &P21475,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A35 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2974717 = {S82801, 139, { &P22656,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SOL22 Build/10.2.F.3.43) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3037953 = {S82802, 158, { &P18724,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-H Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3023232 = {S82803, 137, { &P14183,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I997 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020935 = {S82804, 148, { &P20182,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xolo_X900 Build/AZ510_ICS_02.36I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3082964 = {S82805, 144, { &P15809,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Rhyme S510b Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080493 = {S82806, 148, { &P20182,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xolo_X900 Build/AZ510_ICS_02.36I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3078986 = {S82807, 157, { &P15575,&P5629,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3079130 = {S82808, 156, { &P15575,&P5629,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3092982 = {S82809, 157, { &P15575,&P5629,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3108706 = {S82810, 156, { &P15575,&P5629,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3109617 = {S82811, 135, { &P15364,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SM-N900V Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3115045 = {S82812, 136, { &P15364,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SM-N900W8 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126955 = {S82813, 135, { &P15364,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SM-N900P Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127354 = {S82814, 146, { &P15364,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; AOSP on geeb_att_us Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3130587 = {S82815, 139, { &P18736,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA5319 = {S82816, 67, { &P13639,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-E1310M/E1310MXEIE2 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA61743 = {S82817, 67, { &P13639,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-E1310E/E1310EXBIE1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA88884 = {S82818, 78, { &P13639,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-E1310B/BUIE1/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA778332 = {S82819, 78, { &P13639,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SAMSUNG-E1310B/AMIE1/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA16321 = {S82820, 95, { &P15210,&P5448,&P17183,&P18092 }};	/*Useragent - SAMSUNG-GT-i8000Orange/NXXJE3 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA61802 = {S82821, 86, { &P15210,&P5448,&P17183,&P18092 }};	/*Useragent - SAMSUNG-GT-i8000/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA52788 = {S82822, 33, { &P14662,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH06A3(c500;TB;W24H14)*/
const Device UA69109 = {S82823, 96, { &P12192,&P5448,&P4779,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA2804354 = {S82824, 103, { &P12192,&P5448,&P4779,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95/31/352*/
const Device UA457104 = {S82825, 81, { &P15364,&P18110,&P18201,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.0.10) Gecko/X Thunderbird/1.5.0.10*/
const Device UA552575 = {S82826, 165, { &P21553,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; NX-A7500 Build/I700T_P4_11A_TSNE_FT_R_0_01_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA570671 = {S82827, 165, { &P21553,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; id-id; NX-A7500 Build/I700T_P7_T01_TSNE_FT_R_0_06_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA733290 = {S82828, 165, { &P21553,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; NX-A7500 Build/I700T_P4_11A_TSNE_FT_R_0_05_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA572336 = {S82829, 108, { &P13337,&P17485,&P17338,&P18092 }};	/*Useragent - SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonj108i/UC Browser7.9.0.102/69/352*/
const Device UA653353 = {S82830, 112, { &P12989,&P5843,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia303/13.44; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA899167 = {S82831, 88, { &P15364,&P18118,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; Ubuntu 7.04; de-CH; rv:1.8.1.5) Gecko/X Firefox/2.0.0.5*/
const Device UA689505 = {S82832, 47, { &P15364,&P5690,&P18404,&P18092 }};	/*Useragent - Opera/9.40 (Windows NT 5.1; U; pl) Version/10.9*/
const Device UA1894522 = {S82833, 62, { &P15364,&P5690,&P18404,&P18092 }};	/*Useragent - Opera/9.40 (Windows NT 5.1; U; pl) Presto/2.4.15 Version/10.10*/
const Device UA2586797 = {S82834, 56, { &P15364,&P5690,&P18404,&P18092 }};	/*Useragent - Opera/9.45 (Windows; U; Windows NT 5.1; ru; rv:1.8.1.68)*/
const Device UA1897224 = {S82835, 133, { &P19971,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; nl-nl; DIT7000 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031580 = {S82836, 134, { &P19971,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; T7006 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026474 = {S82837, 89, { &P17976,&P18066,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.3; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.8.149 Version/11.10*/
const Device UA2444918 = {S82838, 89, { &P17976,&P18066,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.4; Linux; Opera Tablet/ADR-X; U; en) Presto/2.8.149 Version/11.10*/
const Device UA2257082 = {S82839, 135, { &P23939,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; COUPLE Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2366444 = {S82840, 171, { &P15060,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2552912 = {S82841, 171, { &P15060,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2452744 = {S82842, 128, { &P23887,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T15A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2547032 = {S82843, 128, { &P23887,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; T15A Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2455027 = {S82844, 70, { &P15876,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; GT-P5100; 800*1232) UCWEB7.9.0.94/139/352*/
const Device UA2653545 = {S82845, 140, { &P14433,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.8; es-es; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2506916 = {S82846, 102, { &P13324,&P5055,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokian8-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2555578 = {S82847, 32, { &P13324,&P5055,&P24675,&P18092 }};	/*Useragent - NokiaN8-00/UCWEB8.8.0.212/51/800*/
const Device UA2596301 = {S82848, 139, { &P22088,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AllviewSpeedDuo Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2536107 = {S82849, 189, { &P18680,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-17 12:06, -0500, us*/
const Device UA2558290 = {S82850, 142, { &P23208,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; AigoPad M60 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583126 = {S82851, 136, { &P23546,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AD683G Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583318 = {S82852, 135, { &P20480,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; JY-G3 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583963 = {S82853, 133, { &P20529,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A89 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2595222 = {S82854, 95, { &P21998,&P17979,&P18062,&P18092 }};	/*Useragent - HTCT327t_TD/1.0 Android/4.0 release/2012 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2800573 = {S82855, 161, { &P18700,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-B5512 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2714943 = {S82856, 170, { &P18510,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; ALCATEL ONE TOUCH 918N Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2715705 = {S82857, 159, { &P15397,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; ST18i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2716665 = {S82858, 156, { &P18679,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-B5330 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2737467 = {S82859, 152, { &P16598,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A500 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2867128 = {S82860, 155, { &P13974,&P5096,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5670 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2754172 = {S82861, 132, { &P20529,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A89 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2767799 = {S82862, 151, { &P22967,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Cloudfone Excite 400d Build/FSV04) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2784084 = {S82863, 136, { &P22916,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E450 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2796800 = {S82864, 137, { &P22916,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E450f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2961652 = {S82865, 167, { &P15201,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Sensation Z710e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3019020 = {S82866, 164, { &P15201,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Sensation 4G Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2963278 = {S82867, 147, { &P23323,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Archos 50 Platinum Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3024237 = {S82868, 155, { &P18687,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; LG-P705 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3030381 = {S82869, 164, { &P13308,&P17485,&P24679,&P18092 }};	/*Useragent - SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj105i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3083605 = {S82870, 134, { &P17283,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; MB860 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3062320 = {S82871, 93, { &P15649,&P5096,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5570) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3076623 = {S82872, 141, { &P23973,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; Andromax-c Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079918 = {S82873, 94, { &P14983,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC7-00) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3093714 = {S82874, 93, { &P18673,&P18066,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S7562) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3110430 = {S82875, 114, { &P14714,&P5843,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02/07.16; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA3114907 = {S82876, 116, { &P14714,&P5843,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA9553 = {S82877, 84, { &P13425,&P17603,&P3534,&P18092 }};	/*Useragent - PANTECH-C790/JAUS08312009 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24073 = {S82878, 75, { &P13836,&P19954,&P3976,&P18092 }};	/*Useragent - SEC-SGHE840-ORANGE/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA44853 = {S82879, 68, { &P13836,&P19954,&P3976,&P18092 }};	/*Useragent - SEC-SGHE840/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24667 = {S82880, 66, { &P15013,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GB220/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA31038 = {S82881, 54, { &P12393,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-M2000/AAN1004 UP.Browser/7.2.7.2.530 (GUI) MMP/2.0*/
const Device UA31041 = {S82882, 54, { &P12393,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-M2000/AAN1002 UP.Browser/7.2.7.2.530 (GUI) MMP/2.0*/
const Device UA54398 = {S82883, 53, { &P12393,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-M2000/ZQ1000 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0*/
const Device UA63824 = {S82884, 53, { &P12393,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-M2000/ZQ1003 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0*/
const Device UA64743 = {S82885, 13, { &P11765,&P17483,&P17660,&P18092 }};	/*Useragent - LG/KC910/v1.0*/
const Device UA2445908 = {S82886, 51, { &P17537,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC3O UP.Browser/6.2.0.15.1.1.1.2 (GUI) MMP/2.0*/
const Device UA90106 = {S82887, 45, { &P17591,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC3Z UP.Browser/6.2.0.16.1 (GUI) MMP/2.0*/
const Device UA172763 = {S82888, 54, { &P17574,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SN3P UP.Browser/6.2_7.2.7.1.K.5.207 (GUI) MMP/2.0*/
const Device UA367186 = {S82889, 160, { &P17262,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Kindle Fire Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Mobile Maxthon*/
const Device UA667116 = {S82890, 152, { &P17262,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Kindle Fire Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1945293 = {S82891, 175, { &P17262,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Kindle Fire Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Mobile Maxthon Mobile Maxthon*/
const Device UA378124 = {S82892, 107, { &P15364,&P7127,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; ) AppleWebKit/534.12 (KHTML, like Gecko) Maxthon/3.0 Safari/534.12*/
const Device UA2360979 = {S82893, 111, { &P15364,&P7127,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12*/
const Device UA3117951 = {S82894, 111, { &P15364,&P7127,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/537.1 (KHTML, like Gecko) Maxthon/3.0 Chrome/22.0.1229.79 Safari/537.1*/
const Device UA378841 = {S82895, 144, { &P17595,&P17296,&P24746,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) Stainless/0.7.5 like Version/4.0 Safari/530.17*/
const Device UA506488 = {S82896, 144, { &P17595,&P17296,&P24746,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.19.4 (KHTML, like Gecko) Stainless/0.7.5 like Version/4.0 Safari/530.17*/
const Device UA2164037 = {S82897, 134, { &P17595,&P17296,&P24746,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Stainless/0.7.5 like Version/4.0 Safari/530.17*/
const Device UA635875 = {S82898, 135, { &P21451,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; SH8188U Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2372271 = {S82899, 135, { &P21451,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SH8188U Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA711121 = {S82900, 99, { &P17595,&P17297,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:6.0.2) Gecko/X Firefox/6.0.2 AlexaToolbar/alxf-2.15*/
const Device UA2033316 = {S82901, 142, { &P22386,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; fr-fr; AN7BG2DT Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028905 = {S82902, 126, { &P22818,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; M1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3042308 = {S82903, 124, { &P22818,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; M1 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2167010 = {S82904, 138, { &P18497,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M886 Build/HuaweiM886) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2235603 = {S82905, 76, { &P17595,&P17290,&P18171,&P18092 }};	/*Useragent - cPro.iDevice [Mozilla/5.0 (Macintosh; Intel Mac OS X 10) Gecko/X Firefox/13]*/
const Device UA2382751 = {S82906, 171, { &P18672,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2384366 = {S82907, 118, { &P15209,&P5768,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokia6233) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile*/
const Device UA2470749 = {S82908, 170, { &P11747,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Desire V Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2451712 = {S82909, 178, { &P18644,&P5043,&P24671,&P18092 }};	/*Useragent - Nokia112/2.0 (03.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia112) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2467050 = {S82910, 153, { &P17935,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PG06100 Build/GRI40; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469201 = {S82911, 168, { &P18302,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MT25i Build/4.1.B.0.631) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2479213 = {S82912, 142, { &P21403,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo S686 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2822056 = {S82913, 64, { &P21403,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; Lenovo S686 Build/IMM76I)*/
const Device UA3117687 = {S82914, 133, { &P21403,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo S686 Build/IMM76I) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2527090 = {S82915, 69, { &P18700,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GT-B5512; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2499515 = {S82916, 82, { &P13094,&P18110,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Adr (Linux; U; 2.3.6; zh-cn; Lenovo A60+ Build/A60+_S312_X;320*480)*/
const Device UA2836144 = {S82917, 82, { &P13094,&P18110,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Adr (Linux; U; 2.3.6; zh-cn; Lenovo A60+ Build/A60+_S314_X;320*480)*/
const Device UA2849283 = {S82918, 82, { &P13094,&P18110,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Adr (Linux; U; 2.3.6; zh-cn; Lenovo A60+ Build/A60+_S319_X;320*480)*/
const Device UA2951200 = {S82919, 82, { &P13094,&P18110,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Adr (Linux; U; 2.3.6; zh-cn; Lenovo A60+ Build/A60+_S315_X;320*480)*/
const Device UA2580137 = {S82920, 141, { &P21656,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; POV_TV-HDMI-KB-01 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2580448 = {S82921, 93, { &P21356,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; TECNO_N3) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2580635 = {S82922, 151, { &P18421,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; FPT Tablet III Build/ARES_00.01.61.H1127) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2583298 = {S82923, 164, { &P18295,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One X Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2676174 = {S82924, 130, { &P22052,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7501 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2673351 = {S82925, 140, { &P20452,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Lenovo S880 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708931 = {S82926, 160, { &P13815,&P5096,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; HTC Wildfire Build/FRG83D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2717868 = {S82927, 160, { &P18462,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; HTC Desire X Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2722662 = {S82928, 69, { &P15364,&P22784,&P4780,&P18092 }};	/*Useragent - MicromaxX259/Q03C MAUI-Browser Profile/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA3014736 = {S82929, 71, { &P15364,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X328/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA3078144 = {S82930, 71, { &P15364,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X282/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2729788 = {S82931, 147, { &P23302,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Mobiistar Touch S30 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2996573 = {S82932, 145, { &P23302,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-; Mobiistar Touch S30 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2738176 = {S82933, 140, { &P11751,&P17483,&P24677,&P18092 }};	/*Useragent - LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-kp500 teleca) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2739794 = {S82934, 159, { &P20134,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2784793 = {S82935, 148, { &P12879,&P6437,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.2; GT-P7510 Build/HTJ85B) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2768887 = {S82936, 96, { &P20004,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Karbonn_A2+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2781442 = {S82937, 147, { &P23423,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Filla Sapphire 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2795861 = {S82938, 138, { &P20711,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID1065 Build/ICS.MID1065.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2796230 = {S82939, 24, { &P23337,&P22843,&P18246,&P18092 }};	/*Useragent - LAVA.KKT40s.Maui Browser*/
const Device UA3021268 = {S82940, 126, { &P17686,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; MZ601 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2968175 = {S82941, 133, { &P15575,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3009233 = {S82942, 133, { &P15575,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3022837 = {S82943, 133, { &P15390,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P925 Build/ICS) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3032182 = {S82944, 156, { &P23776,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGP321 Build/10.1.1.A.1.307) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3059983 = {S82945, 178, { &P14540,&P5843,&P24055,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.71) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311*/
const Device UA3082429 = {S82946, 140, { &P19759,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo K860 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3082983 = {S82947, 137, { &P18637,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9260 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3106292 = {S82948, 139, { &P15767,&P22882,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 7_0 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA3111312 = {S82949, 159, { &P15767,&P22882,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 7_0 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5*/
const Device UA3112157 = {S82950, 132, { &P15767,&P22882,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari/6533.18.5 iLunascape/3212*/
const Device UA3129992 = {S82951, 134, { &P15767,&P22882,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Safari/6533.18.5 iLunascape/3212*/
const Device UA4791 = {S82952, 135, { &P12189,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-; U8220 Build/CRB17) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA27766 = {S82953, 137, { &P12189,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; U8220 Build/CRB17) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA25402 = {S82954, 104, { &P12113,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-A886/A886UXIJ9 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA49383 = {S82955, 138, { &P16930,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; CSL-MI350 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA300685 = {S82956, 54, { &P17699,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SN3O UP.Browser/6.2_7.2.7.1.K.5.207 (GUI) MMP/2.0*/
const Device UA174919 = {S82957, 60, { &P18088,&P5448,&P22371,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.9)*/
const Device UA333316 = {S82958, 96, { &P15364,&P17285,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/13.0.782.112)*/
const Device UA567390 = {S82959, 125, { &P15364,&P17285,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/13.0.782.215) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.215 Safari/535.1*/
const Device UA745172 = {S82960, 89, { &P15364,&P17285,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/13.0.782.107)*/
const Device UA771145 = {S82961, 125, { &P15364,&P17285,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/13.0.782.112) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1*/
const Device UA890790 = {S82962, 125, { &P15364,&P17285,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/13.0.782.220) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1*/
const Device UA2142093 = {S82963, 132, { &P15364,&P17285,&P21898,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/13.0.782.112) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1*/
const Device UA2498324 = {S82964, 220, { &P15364,&P17285,&P21898,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/13.0.782.107; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; InfoPath.2; .NET CLR 1.1.4322; .NET4.0C; .NET4.0E; MS-RTC LM 8; .NET CLR 3.0.30729)*/
const Device UA679436 = {S82965, 41, { &P13554,&P5890,&P24665,&P18092 }};	/*Useragent - NOKIAN79/UC Browser8.0.4.121/28/444/UCWEB*/
const Device UA630477 = {S82966, 101, { &P15851,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Radar C110e)*/
const Device UA714740 = {S82967, 98, { &P15851,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Radar 4G)*/
const Device UA925743 = {S82968, 114, { &P15851,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Radar C110e; 1.08.161.01)*/
const Device UA925773 = {S82969, 103, { &P15851,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Radar; Orange)*/
const Device UA925776 = {S82970, 114, { &P15851,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Radar C110e; 1.08.164.02)*/
const Device UA925818 = {S82971, 114, { &P15851,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Radar C110e; 2.01.164.03)*/
const Device UA925849 = {S82972, 95, { &P15851,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Radar)*/
const Device UA2216293 = {S82973, 49, { &P11684,&P5129,&P17347,&P18092 }};	/*Useragent - nokia5800d-1/UC Browser8.0.3.107/69/444  3gpp-gba*/
const Device UA802510 = {S82974, 129, { &P14404,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S5260/1.0 SHP/VPP/R5 Dolfin/2.0 NexPlayer/3.0 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B NetFront/3.5*/
const Device UA903213 = {S82975, 171, { &P18214,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA1923184 = {S82976, 78, { &P12657,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; HTC_EVO_3D_X515m; 540*960) UCWEB7.9.3.103/139/800*/
const Device UA2456083 = {S82977, 77, { &P12657,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC_EVO_3D_X515m; 540*960) UCWEB7.9.0.94/139/352*/
const Device UA2504694 = {S82978, 77, { &P12657,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC_EVO_3D_X515m; 540*960) UCWEB7.9.0.94/139/444*/
const Device UA2641344 = {S82979, 78, { &P12657,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; HTC_EVO_3D_X515m; 540*960) UCWEB7.9.4.145/139/800*/
const Device UA2653445 = {S82980, 77, { &P12657,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC_EVO_3D_X515m; 540*960) UCWEB7.9.0.94/139/355*/
const Device UA2839977 = {S82981, 77, { &P12657,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; HTC_EVO_3D_X515m; 540*960) UCWEB7.9.0.94/139/352*/
const Device UA1952085 = {S82982, 126, { &P11751,&P17483,&P24670,&P18092 }};	/*Useragent - LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-kp500 teleca) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA1980333 = {S82983, 79, { &P14453,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE51)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2027392 = {S82984, 67, { &P12798,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.4;Zh_cn;LT18i;480*854;)UCWEB7.8.0.95/139/444*/
const Device UA2028991 = {S82985, 67, { &P17625,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.7;Zh_cn;ST25i;480*854;)UCWEB7.8.0.95/139/444*/
const Device UA2201116 = {S82986, 84, { &P15364,&P16110,&P20755,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:15.0) Gecko/X Firefox/15.0.1 SeaMonkey/2.12.1*/
const Device UA2375117 = {S82987, 148, { &P12567,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ja-jp; Panasonic SV-ME1000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2849686 = {S82988, 146, { &P12567,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Panasonic UN-W700 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2404746 = {S82989, 155, { &P16074,&P5768,&P24673,&P18092 }};	/*Useragent - Nokia3110c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia3110c) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2402776 = {S82990, 135, { &P21302,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IQ441 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2469862 = {S82991, 171, { &P14433,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2430434 = {S82992, 138, { &P18593,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S6010 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2465485 = {S82993, 182, { &P14482,&P5843,&P24672,&P18092 }};	/*Useragent - NokiaC3-01/5.0 (06.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac3-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2466235 = {S82994, 131, { &P18632,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; AN10BG3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2536786 = {S82995, 148, { &P20175,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2764382 = {S82996, 149, { &P20175,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1SC Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2556096 = {S82997, 144, { &P21713,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NWZ-Z1000Series Build/E200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879009 = {S82998, 144, { &P21713,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NWZ-Z1000Series Build/E201) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2671451 = {S82999, 166, { &P12731,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2583294 = {S83000, 134, { &P18721,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A90S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583191 = {S83001, 127, { &P18733,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M532 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2849805 = {S83002, 54, { &P15364,&P6729,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Win95; Trident/4.0)*/
const Device UA2660090 = {S83003, 97, { &P11747,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_Desire_V) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2850850 = {S83004, 98, { &P11747,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_Desire_VC) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2665123 = {S83005, 136, { &P22666,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Penta WS707C Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2670751 = {S83006, 136, { &P14017,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P990 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2672192 = {S83007, 138, { &P14854,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713170 = {S83008, 138, { &P14854,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M250K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676292 = {S83009, 138, { &P20174,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CINK KING Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677226 = {S83010, 140, { &P20183,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MI-ONE Plus Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2696836 = {S83011, 136, { &P23290,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; IMO S79 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2700752 = {S83012, 129, { &P15767,&P19826,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/7534.48.3*/
const Device UA2705041 = {S83013, 131, { &P15767,&P19826,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3*/
const Device UA2795230 = {S83014, 131, { &P15767,&P19826,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3*/
const Device UA2733047 = {S83015, 31, { &P19747,&P5043,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia206/03.59)*/
const Device UA2737409 = {S83016, 157, { &P18448,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST26i Build/11.0.A.7.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2768460 = {S83017, 158, { &P18448,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST26i Build/11.0.A.3.28) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2800231 = {S83018, 153, { &P13452,&P5096,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-US; GT-P1000 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2738080 = {S83019, 154, { &P18385,&P18598,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; Nexus 7 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2755411 = {S83020, 137, { &P21373,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F240K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2756334 = {S83021, 137, { &P21384,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F240L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769609 = {S83022, 139, { &P15397,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST18i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773242 = {S83023, 170, { &P12814,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; SAMSUNG-SGH-I777 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2773866 = {S83024, 138, { &P22591,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782848 = {S83025, 137, { &P14877,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9000 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879899 = {S83026, 92, { &P11945,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233s) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3009962 = {S83027, 89, { &P11945,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; vi; gt-s5233s) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2890061 = {S83028, 171, { &P21858,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI Y300-0000 Build/HuaweiY300-0000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3046626 = {S83029, 100, { &P21858,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; HUAWEI Y300-0000 Build/Y300-0000V100R001C17B172;480*800)*/
const Device UA3100443 = {S83030, 100, { &P21858,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.1; zh-cn; HUAWEI Y300-0000 Build/Y300-0000V100R001C17B170;480*800)*/
const Device UA3022697 = {S83031, 136, { &P20240,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E971 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023580 = {S83032, 162, { &P15649,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5570I Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3030087 = {S83033, 161, { &P15649,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5570 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3059298 = {S83034, 102, { &P14188,&P5129,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5233) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3081085 = {S83035, 144, { &P20104,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; 201M Build/9.8.2Q-34_SMJ-102) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110203 = {S83036, 161, { &P16292,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3111276 = {S83037, 160, { &P16292,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Galaxy Nexus Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA18507 = {S83038, 126, { &P13100,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) SP; 240x320; HTC_S710/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA18511 = {S83039, 87, { &P13100,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) HTCS710;Smartphone;240x320*/
const Device UA30981 = {S83040, 118, { &P12381,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonW595s/R3EG Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 model-orange JavaPlatform/JP-8.3.3*/
const Device UA69222 = {S83041, 80, { &P15425,&P5448,&P4779,&P18092 }};	/*Useragent - SAMSUNG-GT-B7300/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA564431 = {S83042, 100, { &P13488,&P5448,&P4779,&P18092 }};	/*Useragent - SonyEricssonM1i/R1BA Browser/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA703020 = {S83043, 37, { &P14358,&P5890,&P17347,&P18092 }};	/*Useragent - nokiae63-1/UC Browser8.0.3.107/69/444*/
const Device UA860188 = {S83044, 149, { &P14255,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; U8150 Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1856155 = {S83045, 127, { &P18254,&P7480,&P5001,&P18089 }};	/*Useragent - Mozilla/4.0 (MobilePhone SCP-5500/US/1.0) NetFront/3.0 MMP/2.0 FAKE (compatible; Googlebot/2.1; http://www.google.com/bot.html)*/
const Device UA1909033 = {S83046, 122, { &P18254,&P7480,&P5001,&P18089 }};	/*Useragent - Mozilla/4.0 (MobilePhone SCP-5500/US/1.0) NetFront/3.0 MMP/2.0 (compatible; Googlebot/2.1; http://www.google.com/bot.html)*/
const Device UA1948732 = {S83047, 118, { &P15364,&P7127,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; zh-CN) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19.1*/
const Device UA1987052 = {S83048, 138, { &P16273,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T679 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2026544 = {S83049, 143, { &P21343,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; el-gr; NovaPad 80 C124 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2424828 = {S83050, 164, { &P15028,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P350 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2179585 = {S83051, 140, { &P15767,&P17090,&P20777,&P18092 }};	/*Useragent - MQQBrowser/2.6 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3*/
const Device UA2230437 = {S83052, 140, { &P15767,&P17090,&P20777,&P18092 }};	/*Useragent - MQQBrowser/2.6 Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3*/
const Device UA2230618 = {S83053, 145, { &P20262,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Karbonn_A18 Build/KarbonnA18) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2238818 = {S83054, 137, { &P19909,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P935 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2242380 = {S83055, 166, { &P11751,&P17483,&P24671,&P18092 }};	/*Useragent - LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-kp500 teleca) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2390410 = {S83056, 98, { &P14010,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s5263) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2375226 = {S83057, 134, { &P20494,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-za; Titan Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2983151 = {S83058, 151, { &P20494,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; m865 Build/TITAN PRIME; TITAN PRIME) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2383116 = {S83059, 71, { &P18295,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; HTC_One_X; 720*1184) UCWEB7.9.0.94/139/352*/
const Device UA2420939 = {S83060, 71, { &P18295,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC_One_X; 720*1184) UCWEB7.9.0.94/139/352*/
const Device UA2653481 = {S83061, 71, { &P18295,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; HTC_One_X; 720*1184) UCWEB7.9.0.94/139/444*/
const Device UA2694693 = {S83062, 71, { &P18295,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC_One_X; 720*1184) UCWEB7.9.0.94/139/444*/
const Device UA2384297 = {S83063, 154, { &P16356,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-N7000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2384469 = {S83064, 154, { &P16356,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2400978 = {S83065, 132, { &P23379,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TOUCH 97 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2408985 = {S83066, 65, { &P15364,&P7127,&P18362,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) Sleipnir/2.8.1*/
const Device UA2444367 = {S83067, 85, { &P15364,&P7127,&P18362,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; .NET CLR 1.1.4322; Sleipnir/2.8.5)*/
const Device UA2545145 = {S83068, 109, { &P15364,&P7127,&P18362,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; FDM) Sleipnir/2.8.4*/
const Device UA2979682 = {S83069, 84, { &P15364,&P7127,&P18362,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; .NET CLR 1.1.4322) Sleipnir/2.8.3*/
const Device UA2416934 = {S83070, 134, { &P20184,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; X506 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2482268 = {S83071, 69, { &P18507,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GT-I8160; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2505384 = {S83072, 158, { &P12040,&P17483,&P24673,&P18092 }};	/*Useragent - LG-GW300/V100 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-gw300) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2557228 = {S83073, 134, { &P22149,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; i88 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583273 = {S83074, 140, { &P21507,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C1605 Build/11.1.A.0.64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2580089 = {S83075, 135, { &P18740,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U7015 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2693748 = {S83076, 135, { &P18740,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U7011 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2583184 = {S83077, 135, { &P21302,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IQ441 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2599236 = {S83078, 159, { &P14551,&P17483,&P24676,&P18092 }};	/*Useragent - LG-GW305/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gw305) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2622472 = {S83079, 61, { &P17337,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-P7500 Build/JOP40D)*/
const Device UA2625440 = {S83080, 145, { &P17337,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-P7500 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 GSA/2.2.10.X*/
const Device UA2627450 = {S83081, 78, { &P17337,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7500 Build/JOP40D) SPG/v1.0.3*/
const Device UA2635245 = {S83082, 130, { &P17337,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; GT-P7500 Build/JOP40G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2869167 = {S83083, 131, { &P17337,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3048175 = {S83084, 61, { &P17337,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P7500 Build/JDQ39E)*/
const Device UA2660876 = {S83085, 160, { &P12513,&P17485,&P24676,&P18092 }};	/*Useragent - SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw20) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2666376 = {S83086, 138, { &P21385,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F240S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2669215 = {S83087, 137, { &P21525,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SBM203SH Build/S0018) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2702517 = {S83088, 100, { &P14916,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-c3303i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2779612 = {S83089, 226, { &P14916,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-c3303i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile, SAMSUNG-GT-C3303i/C3303iDDKF2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA2751522 = {S83090, 144, { &P13999,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; Infinity HD2+ Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2938254 = {S83091, 155, { &P13999,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; TouchWizMOD_HD2 Build/TouchWizMOD_HD2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3077525 = {S83092, 142, { &P13999,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; ITOUCH HD2S Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2754722 = {S83093, 154, { &P20541,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAP4500TDUO Build/PrestigioPAP4500TDUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879437 = {S83094, 152, { &P20541,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAP4500DUO Build/PrestigioPAP4500DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769932 = {S83095, 137, { &P20075,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Xperia S Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773118 = {S83096, 130, { &P13452,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P1000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2800098 = {S83097, 157, { &P12972,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2892829 = {S83098, 158, { &P12972,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989D Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2782260 = {S83099, 165, { &P17937,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; WX435 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2850198 = {S83100, 165, { &P17937,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; WX435 Build/IMM76I; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2792603 = {S83101, 56, { &P15364,&P22310,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Win3.11; Trident/3.0)*/
const Device UA2794946 = {S83102, 135, { &P18480,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N850L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3108787 = {S83103, 134, { &P18480,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N850 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2870625 = {S83104, 136, { &P15767,&P18117,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/537.4 Maxthon*/
const Device UA2944955 = {S83105, 114, { &P15786,&P5043,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-05/08.79; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA2945597 = {S83106, 161, { &P16436,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I8150 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2945980 = {S83107, 160, { &P11747,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire V Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3107068 = {S83108, 161, { &P11747,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire VC Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2958172 = {S83109, 145, { &P19776,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Lenovo K900_ROW Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA3032061 = {S83110, 158, { &P23772,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3031335 = {S83111, 159, { &P15397,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST18i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2969389 = {S83112, 131, { &P20145,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; NS-13T001 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Safari/537.36*/
const Device UA3021723 = {S83113, 142, { &P21885,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GSmart GS202+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2977095 = {S83114, 60, { &P24061,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-S7270 Build/JDQ39)*/
const Device UA2977098 = {S83115, 138, { &P24061,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-S7270 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999013 = {S83116, 137, { &P22396,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5312 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3108045 = {S83117, 138, { &P22396,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5312B Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3023792 = {S83118, 129, { &P21853,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Connect-3G Build/HCL) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3057747 = {S83119, 121, { &P21853,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Y3 Build/HCL) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3020797 = {S83120, 136, { &P24655,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; IdeaTab A3000-F Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022136 = {S83121, 136, { &P24655,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; IdeaTab A3000-H Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3083073 = {S83122, 136, { &P18662,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ISW13HT Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA58 = {S83123, 128, { &P12096,&P5448,&P17173,&P18092 }};	/*Useragent - PPC; 240x320; HTC_TyTN/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA24332 = {S83124, 128, { &P12096,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) PPC; 240x320; HTC_TyTN\1.0 Profile\MIDP-2.0 Configuration\CLDC-1.1*/
const Device UA64908 = {S83125, 70, { &P12096,&P5448,&P17173,&P18092 }};	/*Useragent - HTC_TyTN Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA21774 = {S83126, 64, { &P12003,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia7070d-2/2.0 (06.20) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA32710 = {S83127, 40, { &P12515,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP265 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA54506 = {S83128, 47, { &P14231,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-VX9900/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA454786 = {S83129, 125, { &P15364,&P7133,&P24198,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.11.12388.201*/
const Device UA591408 = {S83130, 123, { &P15364,&P7133,&P24198,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1 QQBrowser/6.11.11981.201*/
const Device UA600636 = {S83131, 191, { &P15364,&P7133,&P24198,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0) QQBrowser/6.11.12388.201*/
const Device UA813391 = {S83132, 193, { &P15364,&P7133,&P24198,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.11.12388.201*/
const Device UA823291 = {S83133, 176, { &P15364,&P7133,&P24198,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0) QQBrowser/6.11.12388.201*/
const Device UA2333045 = {S83134, 181, { &P15364,&P7133,&P24198,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; QQDownload 679; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C) QQBrowser/6.11.12135.201*/
const Device UA456574 = {S83135, 74, { &P17976,&P17979,&P20434,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:12.0) Gecko/12.0 Firefox/12.0 Fennec/12.0*/
const Device UA475726 = {S83136, 68, { &P15364,&P7127,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) SEOChat::Bot v1.1*/
const Device UA659777 = {S83137, 89, { &P15364,&P7127,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; ZoomSpider.net bot; .NET CLR 1.1.4322)*/
const Device UA706093 = {S83138, 55, { &P15364,&P17486,&P4222,&P18092 }};	/*Useragent - MOT-A-D5/00.02 UP.Browser/7.0.2.2.c.1.126 (GUI) MMP/2.0*/
const Device UA2274027 = {S83139, 55, { &P15364,&P17486,&P4222,&P18092 }};	/*Useragent - MOT-A-3D/01.04 UP.Browser/7.0.2.2.c.1.108 (GUI) MMP/2.0*/
const Device UA2306883 = {S83140, 55, { &P15364,&P17486,&P4222,&P18092 }};	/*Useragent - MOT-A-C8/00.04 UP.Browser/7.0.2.2.c.1.126 (GUI) MMP/2.0*/
const Device UA687474 = {S83141, 96, { &P15364,&P17838,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.11) Gecko/X Ubuntu/9.10 (karmic) Firefox/3.0.11*/
const Device UA867450 = {S83142, 92, { &P15364,&P18110,&P22483,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.0.4) Gecko/X Firefox/1.5.0.4 Flock/0.7.0.17.1*/
const Device UA2130904 = {S83143, 46, { &P15364,&P21445,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows ME)*/
const Device UA2219196 = {S83144, 79, { &P13554,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN79)/UC Browser8.5.0.183/28/444/UCWEB Mobile*/
const Device UA1988536 = {S83145, 113, { &P21643,&P22857,&P18373,&P18092 }};	/*Useragent - i-mobile S 301T   Browser/Opera  Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; th-TH) Presto/2.5.28 Version/10.10*/
const Device UA2026125 = {S83146, 138, { &P15820,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-SU640 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2030174 = {S83147, 131, { &P20189,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U18GT-H Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2155982 = {S83148, 52, { &P14033,&P5890,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 (NokiaC5-03;SymbianOS/9.1 Series60/3.0)*/
const Device UA2481746 = {S83149, 95, { &P13677,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; fa; samsung-gt-s5253) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2383510 = {S83150, 188, { &P14743,&P17485,&P24672,&P18092 }};	/*Useragent - SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonj20i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile*/
const Device UA2403549 = {S83151, 118, { &P15791,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia603) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/444/UCWEB Mobile*/
const Device UA2415696 = {S83152, 137, { &P20982,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Sen S3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2419300 = {S83153, 136, { &P21379,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SH631W Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2421639 = {S83154, 142, { &P23872,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; connspeed_AS5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2427139 = {S83155, 92, { &P16871,&P22784,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxq5) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2695671 = {S83156, 92, { &P16871,&P22784,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxq5) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2466792 = {S83157, 70, { &P18213,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GT-S6500D; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2458775 = {S83158, 128, { &P20888,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; D709 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2485888 = {S83159, 153, { &P12108,&P5843,&P24675,&P18092 }};	/*Useragent - Nokia7230/5.0 (10.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia7230) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2536649 = {S83160, 90, { &P12798,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; LT18i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2557977 = {S83161, 143, { &P18305,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; LG-P936/V20c Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778693 = {S83162, 138, { &P18305,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; LG-P936 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2579570 = {S83163, 228, { &P22788,&P22831,&P18246,&P18092 }};	/*Useragent - TECHAIN50_11B_HW (MRE\3.0.00(2500) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) Masscom_P19i_T716V_M00_S03_X Release/2012.11.16 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 vi-VN*/
const Device UA2730970 = {S83164, 228, { &P22788,&P22831,&P18246,&P18092 }};	/*Useragent - TECHAIN50_11B_HW (MRE\3.0.00(2500) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) Masscom_P19i_T716V_M00_S03_X Release/2012.11.29 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 vi-VN*/
const Device UA2749667 = {S83165, 228, { &P22788,&P22831,&P18246,&P18092 }};	/*Useragent - TECHAIN50_11B_HW (MRE\3.0.00(2500) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) Masscom_P19i_T716V_M00_S04_X Release/2013.01.04 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 vi-VN*/
const Device UA2592685 = {S83166, 139, { &P23699,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SIMM-X720 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2894619 = {S83167, 128, { &P23699,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; X720 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2676818 = {S83168, 117, { &P15364,&P17285,&P19995,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.152 Safari/537.22 Sleipnir/4.0.1*/
const Device UA2607122 = {S83169, 134, { &P15390,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P925 Build/ICS) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2633685 = {S83170, 131, { &P15985,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2636611 = {S83171, 131, { &P15985,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P6200 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2660204 = {S83172, 88, { &P21064,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A44) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2673373 = {S83173, 126, { &P18733,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M532 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676309 = {S83174, 137, { &P20240,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Optimus G Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2702623 = {S83175, 139, { &P18072,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120K Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2708391 = {S83176, 139, { &P18072,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2733217 = {S83177, 139, { &P18072,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120L Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2702653 = {S83178, 138, { &P14872,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-D700 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996831 = {S83179, 139, { &P14872,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-D700 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708060 = {S83180, 145, { &P20104,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; 201M Build/9.8.2Q-34_SMJ-102) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2869750 = {S83181, 164, { &P15501,&P5477,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E16i Build/2.1.1.A.0.53) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2792764 = {S83182, 63, { &P15364,&P17285,&P17384,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 4.0; Windows NT 6.0; Trident/2.0)*/
const Device UA2799247 = {S83183, 142, { &P13644,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EVO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2869891 = {S83184, 141, { &P12280,&P17146,&P22879,&P18092 }};	/*Useragent - MQQBrowser/43 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3*/
const Device UA2879253 = {S83185, 140, { &P12280,&P17146,&P22879,&P18092 }};	/*Useragent - MQQBrowser/43 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA2943894 = {S83186, 164, { &P13308,&P17485,&P24678,&P18092 }};	/*Useragent - SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj105i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3022796 = {S83187, 132, { &P23171,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; G2S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2966723 = {S83188, 170, { &P18505,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3023223 = {S83189, 142, { &P22737,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Micromax A115 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3018271 = {S83190, 107, { &P18510,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_918N) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3021548 = {S83191, 107, { &P18510,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_918N) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3030927 = {S83192, 141, { &P12280,&P18117,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 UCBrowser/9.2.0.312 Mobile*/
const Device UA3108535 = {S83193, 141, { &P12280,&P18117,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 UCBrowser/9.2.0.312 Mobile*/
const Device UA3060255 = {S83194, 93, { &P13706,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5235) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3060595 = {S83195, 88, { &P19779,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A1+) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3083598 = {S83196, 130, { &P13452,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P1000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3081288 = {S83197, 131, { &P15102,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ605 Build/I.7.1-39.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3081083 = {S83198, 138, { &P22592,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717R Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA16002 = {S83199, 115, { &P11849,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-F330-TEN/F330AFHB1 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA43675 = {S83200, 98, { &P11849,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-F330/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA7564 = {S83201, 61, { &P13352,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2660/2.0 (05.20) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA7566 = {S83202, 66, { &P13352,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2660_CMCC/2.0 (05.25) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA20903 = {S83203, 107, { &P15143,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-T229/T229UVHC4 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA55024 = {S83204, 69, { &P14274,&P6476,&P4902,&P18092 }};	/*Useragent - SHARP-STX2/MC.40 POLARIS/6.15 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA67747 = {S83205, 148, { &P14887,&P5477,&P4861,&P18092 }};	/*Useragent - SAMSUNG_GT-I9008_TD/1.0 OPhone/2.0 (Linux; Android 2.1) Release/5.1.2010 Browser/WAP2.0 (AppleWebKit/530.17) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA1900446 = {S83206, 100, { &P14848,&P5843,&P17338,&P18092 }};	/*Useragent - NokiaX3-00/5.0 (08.54) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax3-00/UC Browser7.9.0.102/69/352*/
const Device UA393431 = {S83207, 54, { &P17639,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SH3K UP.Browser/6.2_7.2.7.1.K.5.207 (GUI) MMP/2.0*/
const Device UA90734 = {S83208, 61, { &P16566,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX266/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA301594 = {S83209, 31, { &P17745,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P10A(c100;TB;W24H15)*/
const Device UA349313 = {S83210, 111, { &P18254,&P17017,&P18245,&P18091 }};	/*Useragent - yacybot (freeworld/global; amd64 Windows Server 2008 R2 6.1; java 1.6.0_29; Europe/de) http://yacy.net/bot.html*/
const Device UA748903 = {S83211, 76, { &P18254,&P17017,&P18245,&P18091 }};	/*Useragent - feedfinder/1.371 Python-urllib/1.17 +http://www.aaronsw.com/2002/feedfinder/*/
const Device UA2031251 = {S83212, 136, { &P21158,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; ZTE V875 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2178006 = {S83213, 138, { &P20983,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9308 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2236869 = {S83214, 121, { &P15791,&P5055,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; Nokia603) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/51/352/UCWEB Mobile*/
const Device UA2262074 = {S83215, 157, { &P18321,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2366601 = {S83216, 152, { &P23459,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; POLY PAD 9716HD Build/ICS.g12refM1005TR.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2383201 = {S83217, 136, { &P20359,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZP300+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2454718 = {S83218, 140, { &P23947,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Alpha GTR Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2487375 = {S83219, 155, { &P18465,&P18066,&P19817,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.6 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2505220 = {S83220, 144, { &P20880,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NEO_N002 Build/NeoNEO_N002) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2844041 = {S83221, 66, { &P20880,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; NEO_N002 Build/NeoNEO_N002)*/
const Device UA2530090 = {S83222, 11, { &P23572,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M5910*/
const Device UA2549568 = {S83223, 139, { &P22107,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HS-EG950 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2982284 = {S83224, 61, { &P22107,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; HS-EG950 Build/IMM76I)*/
const Device UA2584174 = {S83225, 141, { &P21141,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Lenovo P770 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583960 = {S83226, 140, { &P12616,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WT19i Build/4.1.B.0.587) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602398 = {S83227, 135, { &P20213,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N9500 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2674320 = {S83228, 102, { &P18623,&P21697,&P24633,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 510) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile*/
const Device UA2698870 = {S83229, 93, { &P20141,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S5301) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2708743 = {S83230, 137, { &P18456,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T699 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2733110 = {S83231, 141, { &P22808,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; WALTON Primo-D1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2751623 = {S83232, 143, { &P22822,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KPN Smart 300 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2753319 = {S83233, 136, { &P18240,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P700 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784796 = {S83234, 159, { &P13054,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR6425LVW Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2779359 = {S83235, 157, { &P15850,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-lu6200 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2783723 = {S83236, 140, { &P22656,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SOL22 Build/10.2.F.3.43) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2784104 = {S83237, 157, { &P19859,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328w Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2792208 = {S83238, 86, { &P17968,&P18111,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1.2; Linux; Opera Mobi/ADR-X; U; ru) Presto/2.7.81 Version/11.00*/
const Device UA2841631 = {S83239, 86, { &P17968,&P18111,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.1.1; Linux; Opera Mobi/ADR-X; U; en) Presto/2.7.81 Version/11.00*/
const Device UA2870077 = {S83240, 152, { &P15575,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100G Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2997557 = {S83241, 151, { &P15575,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2999600 = {S83242, 151, { &P15575,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2954182 = {S83243, 171, { &P19861,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I605 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SCH-I605, 42, 1.9.1)*/
const Device UA3037955 = {S83244, 160, { &P12841,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC PH39100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3022595 = {S83245, 130, { &P20154,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5770D Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2974118 = {S83246, 152, { &P21833,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023513 = {S83247, 140, { &P22632,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Titanium S1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021444 = {S83248, 146, { &P20090,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848i Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022849 = {S83249, 145, { &P20090,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848 Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023518 = {S83250, 132, { &P20231,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A21 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022219 = {S83251, 137, { &P20508,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cynus T2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021774 = {S83252, 125, { &P23001,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M97 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022845 = {S83253, 96, { &P20004,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Karbonn_A2+) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3032480 = {S83254, 149, { &P20991,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ME371MG Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3062165 = {S83255, 154, { &P20705,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; HTC One Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3063980 = {S83256, 152, { &P19775,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; A210 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3040397 = {S83257, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (ZmEufAdvOJ6+jpzS29ugr8sbtxJwbNEwTvj+UjrNdn8=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040398 = {S83258, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (VAGxVr3aE18uG9UGNhxfYQKo8MfRq62rziav6QZT9EQ=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040399 = {S83259, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (oVaX3xu8lS6+dxqSTxJyCyJ+2By0urQwPAPZDhxZX8Q=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040400 = {S83260, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (J0uUUVb1lF9anOEASNkv+dk9J+E03avcau9WUGCrjqA=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040402 = {S83261, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (ekLz1M/BXVWgI0h9U0OQLG8D1dQihLpRf5wxUVxHV/g=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040403 = {S83262, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (seO21gpOtp8uTe8EjZClilM5rgzyrlWqV92C7FPS+dA=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040404 = {S83263, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (jwD2y6YFkctosT5FELDx07yy1LfQUWIvcaRaHNV882U=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040405 = {S83264, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (/fePx6eQbNOJc18VKF+tUCcbSw7dqUiyUeAWMr1Oe2E=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040406 = {S83265, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (kSamzH/nGjnKspyMoq+MPnFVFkSdI+FUde17GDT6ReQ=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040407 = {S83266, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (QOeq2ctm9O3zMuuvTkrEUt0UCx8jzBmtsr8W7cqsYZ8=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040408 = {S83267, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (WKKNEiGPExM+NZzLS3xwOi2UAiCvCuH+EZWhrH36F8w=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040409 = {S83268, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (Y+zhCxawR0sYLoffHaQoaUxQwBeXpqpbhXezjKAZqVc=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040410 = {S83269, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (HRfVdPGb2DXDoQv7j0jiZ7kumjupqYEZuuv+uUjYaXY=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040411 = {S83270, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (aQw7//dlIYE6iKTsPsmlK510hNgZeoA88gEzyy33PN8=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040412 = {S83271, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (i4DjSJEBwdLJDDtV37i4IpGJXfG7l1GcnLuPNHSOyBQ=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040413 = {S83272, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (6PNbJqYPVfeNgfBReG2mSHUyWmtwktcmfHhI0M3t0O4=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040414 = {S83273, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (GZTg5JsTFXdJASbcqxtJ4kc227JaAEjA9qCSMp3LpEM=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040415 = {S83274, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (EnLPfK3gBP87LW9Hd6JIkOZwJI246mpl+xkiH+y8vqA=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040416 = {S83275, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (H3Md1XIJnTbE+bu2rA68p1zm1RLiEeeTztPpy0c371E=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040418 = {S83276, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (YJxLL7kX0A4Or/0URhJklkH+e4kL6zwjoChsP/AYteg=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040417 = {S83277, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (toiqvGz7d4VX6nP0TXtuSfxAmWEMBplIGl6f4IRb+PA=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040419 = {S83278, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (ZzcZ9AYfEOcJ9lhSYF9ZdA3wxO7FaGsplEU1IZWRH/Q=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040420 = {S83279, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (OGxyVI07puJEIltprVOx+nKUtFBW0X1nDLz6GIOU7TM=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040421 = {S83280, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (2+5mbYbRy06RbV9AqKfDxK1OxQTSP/3ogwQnfKgRpe8=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040422 = {S83281, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (dPBmLrfYvwIweCRKYVi6ehCtlkvkj5qzhl3wX2U7t2E=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040423 = {S83282, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (o8u5P/w47yiT9EAgMQTYbosdzSOTsLwW+Jodpmz0mIg=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040424 = {S83283, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (wwV7b+ehRKit4Z/Dcpf5Iyt+XWId08eJmLj0BeAM1ik=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040425 = {S83284, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (iGHKmJCWM+S9hhlEe6w0Freohib2zGpsMp1/y8TZBkw=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040426 = {S83285, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (/HDwNKUP3bXS6+ZAwbPIIV7okuayQT2skTvAork+ngM=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040427 = {S83286, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (sPHxgDKpO8H7xz7UJx+863KHtKvj0XxHSH5oNWkOt+Y=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040428 = {S83287, 137, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 () Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3040401 = {S83288, 181, { &P15364,&P24349,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (XPokjOYxFHsUdRJ3O2EuYjHwfjohEnE9z01PffnbHEI=) Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/535.2*/
const Device UA3061697 = {S83289, 97, { &P22622,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Micromax_A72) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3081237 = {S83290, 137, { &P20124,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ZTE V970 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA4878 = {S83291, 93, { &P12277,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Alltel HTC Snap*/
const Device UA26915 = {S83292, 86, { &P12277,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Snap Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA29867 = {S83293, 91, { &P12277,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Snap_S522 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA15621 = {S83294, 165, { &P11826,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/HPiPAQDataMessenger/1.00.00 Browser/VF-Browser/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA15620 = {S83295, 98, { &P11826,&P5448,&P22270,&P18092 }};	/*Useragent - HPiPAQDataMessenger/1.0/Mozilla/4.0  (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC;320x240*/
const Device UA58660 = {S83296, 87, { &P15048,&P17483,&P3487,&P18092 }};	/*Useragent - LG-A140/1.0/EUN_NK MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.1.582 (GUI)*/
const Device UA27939 = {S83297, 106, { &P12191,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) garmin-asus-Nuvifone-M10/2.0*/
const Device UA40017 = {S83298, 68, { &P13694,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHJ600/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61116 = {S83299, 76, { &P13694,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHJ600E-ORANGE/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA832918 = {S83300, 69, { &P13694,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHJ600E/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA640773 = {S83301, 100, { &P14848,&P5843,&P17347,&P18092 }};	/*Useragent - NokiaX3-00/5.0 (08.54) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax3-00/UC Browser8.0.3.107/69/444*/
const Device UA371363 = {S83302, 69, { &P14640,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;OMNIA7;7.10;8107)*/
const Device UA755437 = {S83303, 69, { &P14640,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;OMNIA7;7.10;7740)*/
const Device UA768350 = {S83304, 69, { &P14640,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;OMNIA7;7.10;7720)*/
const Device UA925857 = {S83305, 69, { &P14640,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;OMNIA7;7.10;8773)*/
const Device UA676487 = {S83306, 41, { &P12581,&P5055,&P24665,&P18092 }};	/*Useragent - Nokia500/UC Browser8.0.4.121/50/444/UCWEB*/
const Device UA460431 = {S83307, 64, { &P15364,&P18146,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (X11; SunOS i86pc; U; en) Presto/2.2.15 Version/10.10*/
const Device UA647638 = {S83308, 64, { &P15364,&P18146,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (X11; SunOS i86pc; U; en) Presto/2.2.15 Version/10.11*/
const Device UA645269 = {S83309, 71, { &P16237,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i937;7.10;7720)*/
const Device UA925904 = {S83310, 71, { &P16237,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i937;7.10;8773)*/
const Device UA832570 = {S83311, 54, { &P18088,&P17017,&P18402,&P18092 }};	/*Useragent - Opera/9.50 (J2ME/MIDP; Opera Mini/2.99/27.1659; U; en)*/
const Device UA890523 = {S83312, 143, { &P22026,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; NVSBL VORTEX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2338853 = {S83313, 81, { &P14983,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaC7-00)/UC Browser8.5.0.183/50/355/UCWEB Mobile*/
const Device UA1987306 = {S83314, 81, { &P11907,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE52-1)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA1987946 = {S83315, 162, { &P17968,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; generic Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2202076 = {S83316, 71, { &P15364,&P16110,&P18523,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:15.0) Gecko/X Thunderbird/15.0.1*/
const Device UA2503224 = {S83317, 171, { &P18507,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2350376 = {S83318, 170, { &P18382,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Desire C Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2393508 = {S83319, 79, { &P11905,&P5890,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokia6120c) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2423655 = {S83320, 137, { &P20228,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2428298 = {S83321, 98, { &P12137,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s5620) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2429482 = {S83322, 135, { &P20117,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IQ440 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2457541 = {S83323, 144, { &P20514,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ZEN U4 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2504970 = {S83324, 159, { &P12125,&P5204,&P24673,&P18092 }};	/*Useragent - Nokia6500s-1/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6500s-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2505419 = {S83325, 83, { &P23566,&P17979,&P18062,&P18092 }};	/*Useragent - ZTE U950_TD/1.0 Linux/3.1.10 Android/4.0 Release/8.1.2012 Browser/AppleWebKit534.30*/
const Device UA2561028 = {S83326, 90, { &P13018,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; MT11i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2565154 = {S83327, 91, { &P13445,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAN85) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2579951 = {S83328, 81, { &P17976,&P5477,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.1; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2580205 = {S83329, 89, { &P17976,&P5477,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.1-update1; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2581622 = {S83330, 166, { &P21356,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; TECNO N3 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2598327 = {S83331, 98, { &P20330,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; Micromax_P350) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2665741 = {S83332, 156, { &P20075,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Xperia S Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057268 = {S83333, 163, { &P20075,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Xperia S Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/nozomi*/
const Device UA2675053 = {S83334, 145, { &P18300,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T769 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675836 = {S83335, 136, { &P14183,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I997 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2703862 = {S83336, 141, { &P22652,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Celkon_A67 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2763506 = {S83337, 159, { &P18002,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A760S Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3046561 = {S83338, 158, { &P18002,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A760S Build/CM10-RUCRO; CyanogenMod-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2750528 = {S83339, 93, { &P22626,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Iris_349) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3107625 = {S83340, 94, { &P22626,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Iris_349+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2769163 = {S83341, 139, { &P18225,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; U9200 Build/HuaweiU9200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2772861 = {S83342, 130, { &P18702,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N8013 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2795772 = {S83343, 143, { &P23474,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PHICOMM i600 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2961362 = {S83344, 135, { &P23474,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; i600 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2809549 = {S83345, 51, { &P19932,&P17484,&P4521,&P18092 }};	/*Useragent - SCH-W279/ POLARIS/6.2 (GUI; compatible; UP.Browser)*/
const Device UA2867307 = {S83346, 129, { &P21853,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Connect-3G Build/HCL) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2867838 = {S83347, 121, { &P21853,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Y3 Build/HCL) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2867762 = {S83348, 148, { &P23425,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; iBuddyConnect?_3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956135 = {S83349, 136, { &P24570,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZP910 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022055 = {S83350, 136, { &P18662,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ISW13HT Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975057 = {S83351, 136, { &P20226,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-E975 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3037930 = {S83352, 158, { &P19902,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3022060 = {S83353, 127, { &P20127,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT270 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023431 = {S83354, 136, { &P22874,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9502 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020565 = {S83355, 140, { &P21094,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A789 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023587 = {S83356, 93, { &P15649,&P5096,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5570) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3027684 = {S83357, 155, { &P18241,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-E400 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3083079 = {S83358, 127, { &P23801,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; P774A Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3081195 = {S83359, 137, { &P18622,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A820L Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080010 = {S83360, 132, { &P18554,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; BN Nook HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3109653 = {S83361, 144, { &P18554,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3062114 = {S83362, 90, { &P15989,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3080877 = {S83363, 91, { &P15989,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i2) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3080748 = {S83364, 134, { &P24493,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; BS471 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081618 = {S83365, 138, { &P22590,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081289 = {S83366, 138, { &P22978,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6313T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094834 = {S83367, 138, { &P18386,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I757M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083335 = {S83368, 132, { &P23171,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; G2S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094527 = {S83369, 140, { &P14225,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire HD Build/SVHD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109870 = {S83370, 146, { &P15364,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-CN; X Build/HMX) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3112883 = {S83371, 167, { &P15364,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; HUAWEI P6-T00 Build/HuaweiP6-T00) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3112884 = {S83372, 149, { &P15364,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; P2 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127146 = {S83373, 171, { &P15364,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-CN; HUAWEI G610-U00 Build/HuaweiG610-U00) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3115337 = {S83374, 157, { &P16356,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3115439 = {S83375, 157, { &P16356,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-N7000 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA14877 = {S83376, 95, { &P14880,&P5026,&P4948,&P18092 }};	/*Useragent - BlackBerry8320/4.5.0.81 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100 UP.Browser/5.0.3.3*/
const Device UA30348 = {S83377, 34, { &P12304,&P6476,&P17470,&P18092 }};	/*Useragent - LG9200/1.0 TELECA/Q7.2-1.1 MMP/2.0*/
const Device UA41792 = {S83378, 54, { &P14046,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player optus_au / 1.0 / htc_vivo / 2.3.3*/
const Device UA57669 = {S83379, 53, { &P14046,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_wwe / 1.0 / htc_vivo / 2.3.3*/
const Device UA69776 = {S83380, 58, { &P14046,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player verizon_wwe / 1.0 / htc_vivow / 2.3.3*/
const Device UA70762 = {S83381, 58, { &P14046,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player verizon_wwe / 1.0 / htc_vivow / 2.3.4*/
const Device UA61151 = {S83382, 71, { &P12644,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-R54/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2816848 = {S83383, 89, { &P12644,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-R54(South_Africa_MTN)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA68613 = {S83384, 34, { &P11854,&P5890,&P4777,&P18092 }};	/*Useragent - NokiaE71/UC Browser7.7.1.88/28/444*/
const Device UA765659 = {S83385, 36, { &P11854,&P5890,&P4777,&P18092 }};	/*Useragent - nokiae71-1/UC Browser7.7.1.88/69/444*/
const Device UA370351 = {S83386, 142, { &P17324,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; hr-hr; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2400249 = {S83387, 61, { &P17324,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9001 Build/IMM76L)*/
const Device UA2656512 = {S83388, 139, { &P17324,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-I9001 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2815709 = {S83389, 139, { &P17324,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-I9001 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA687840 = {S83390, 137, { &P21549,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-pk; DE88Plus Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018742 = {S83391, 134, { &P21549,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; in-id; DE88Plus Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA501249 = {S83392, 98, { &P15364,&P17285,&P22481,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; pl; rv:1.9.0.14; onet) Gecko/X Firefox/3.0.14 Flock/2.5.3*/
const Device UA1858509 = {S83393, 83, { &P15364,&P7127,&P20907,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-GB; rv:0.9.4.1) Gecko/X Netscape6/6.2.3*/
const Device UA585459 = {S83394, 77, { &P15364,&P18110,&P18192,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:9.0) Gecko/X Thunderbird/9.0 Lightning/1.1.1*/
const Device UA589369 = {S83395, 95, { &P15364,&P7133,&P20460,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:10.0) Gecko/X Firefox/10.0 SeaMonkey/2.7 Lightning/1.2.1*/
const Device UA830986 = {S83396, 70, { &P15364,&P7133,&P20460,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:10.0.2) Gecko/X SeaMonkey/2.7.2*/
const Device UA688656 = {S83397, 45, { &P18254,&P17017,&P22244,&P18092 }};	/*Useragent - Flipboard/1.8.4/49,1.8.4,2012-05-11 16:30, gb*/
const Device UA899590 = {S83398, 45, { &P18254,&P17017,&P22244,&P18092 }};	/*Useragent - Flipboard/1.8.4/51,1.8.4,2012-05-16 03:48, gb*/
const Device UA900165 = {S83399, 45, { &P18254,&P17017,&P22244,&P18092 }};	/*Useragent - Flipboard/1.8.4/45,1.8.4,2012-05-09 22:44, gb*/
const Device UA900512 = {S83400, 45, { &P18254,&P17017,&P22244,&P18092 }};	/*Useragent - Flipboard/1.8.4/60,1.8.4,2012-05-24 22:32, sg*/
const Device UA1964760 = {S83401, 45, { &P18254,&P17017,&P22244,&P18092 }};	/*Useragent - Flipboard/1.8.4/54,1.8.4,2012-05-16 21:29, us*/
const Device UA1988105 = {S83402, 45, { &P18254,&P17017,&P22244,&P18092 }};	/*Useragent - Flipboard/1.8.4/55,1.8.4,2012-05-19 00:44, ca*/
const Device UA724234 = {S83403, 109, { &P17595,&P17295,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:5.0) Gecko/X Firefox/5.0,gzip(gfe) (via translate.google.com)*/
const Device UA789730 = {S83404, 134, { &P15936,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; XT316 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA911663 = {S83405, 172, { &P12280,&P18117,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3 UCBrowser8.3.1.173/42/355*/
const Device UA2721495 = {S83406, 170, { &P12280,&P18117,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3 UCBrowser8.3.1.173/42/355*/
const Device UA1957398 = {S83407, 176, { &P19891,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I717 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1970534 = {S83408, 176, { &P19891,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I717 Build/IMM76I; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029554 = {S83409, 130, { &P23999,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TAB464 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2397774 = {S83410, 130, { &P23999,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB464 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2157120 = {S83411, 61, { &P20033,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX368/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA2217822 = {S83412, 70, { &P16869,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax Q34/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2221509 = {S83413, 65, { &P15364,&P18110,&P20793,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; N12; 480*764) UCWEB1.0.1.113/147/800*/
const Device UA2453446 = {S83414, 72, { &P15364,&P18110,&P20793,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; P76a(K3G5); 800*444) UCWEB1.0.1.113/147/800*/
const Device UA3070383 = {S83415, 68, { &P15364,&P18110,&P20793,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; CW-V17; 800*432) UCWEB1.0.1.113/147/800*/
const Device UA2235412 = {S83416, 171, { &P12280,&P18432,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10A403 Safari/7534.48.3 UCBrowser8.5.0.195/44/355*/
const Device UA2367790 = {S83417, 173, { &P12280,&P18432,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10A523 Safari/7534.48.3 UCBrowser8.5.0.195/44/355*/
const Device UA2328390 = {S83418, 89, { &P14983,&P5055,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiac7-00) UCBrowser8.4.0.159/69/352/UCWEB Mobile  3gpp-gba*/
const Device UA2355686 = {S83419, 160, { &P14183,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I997 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2405769 = {S83420, 153, { &P15364,&P17285,&P19966,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.4.2.X (Second Life Release; default skin) Safari/533.3*/
const Device UA2408691 = {S83421, 74, { &P15364,&P6729,&P20902,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win95; en-US; rv:1.7.2) Gecko/X Netscape/7.2 (ax)*/
const Device UA2413944 = {S83422, 121, { &P21386,&P17979,&P18062,&P18092 }};	/*Useragent - Lenovo-A698t_TD/S100 Linux/3.0.13 Android/4.0 Release/01.05.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2*/
const Device UA2423769 = {S83423, 158, { &P12513,&P17485,&P24673,&P18092 }};	/*Useragent - SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonw20) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2428288 = {S83424, 98, { &P12080,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s3653) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2432721 = {S83425, 90, { &P23351,&P22784,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromax) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2527668 = {S83426, 90, { &P23351,&P22784,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromax) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2461336 = {S83427, 167, { &P18450,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST23i Build/11.0.A.5.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2482078 = {S83428, 70, { &P20059,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X269/Q03C MAUI-Browser Profile/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA2482501 = {S83429, 131, { &P22818,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; M1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2816224 = {S83430, 130, { &P22818,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Android 2.3.5;zh-cn;GO M1 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3044357 = {S83431, 134, { &P22818,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; GO M1 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2503144 = {S83432, 140, { &P18225,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; U9200 Build/HuaweiU9200) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2506855 = {S83433, 133, { &P23883,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; X10.Dual+ Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2529287 = {S83434, 132, { &P23883,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; X10.Dual Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2532223 = {S83435, 141, { &P21880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; Brava DM-992 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2583176 = {S83436, 138, { &P18507,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8160 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2563482 = {S83437, 69, { &P18088,&P22710,&P4201,&P18092 }};	/*Useragent - AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b*/
const Device UA2579780 = {S83438, 139, { &P21848,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PolyPad 1010IPS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2580285 = {S83439, 144, { &P21378,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; tablet Fnac 8 Build/1.0.4 X-11:52) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2704493 = {S83440, 144, { &P21378,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; tablet Fnac 8 Build/1.1.0 X-12:19) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2846371 = {S83441, 144, { &P21378,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; tablet Fnac 8 Build/1.0.1 X-10:20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2663145 = {S83442, 141, { &P23120,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Aqua Style Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2674440 = {S83443, 136, { &P19856,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-LS970 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2715801 = {S83444, 138, { &P22599,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300S Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2734225 = {S83445, 226, { &P17346,&P19826,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA2779782 = {S83446, 226, { &P17346,&P19826,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA2988533 = {S83447, 224, { &P17346,&P19826,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B144,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA2749838 = {S83448, 156, { &P18456,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T699 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865393 = {S83449, 157, { &P18456,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T699 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2774332 = {S83450, 166, { &P15880,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2754628 = {S83451, 137, { &P21028,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Smartfren Tab 7 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2866941 = {S83452, 133, { &P21028,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; StarTab 715 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755997 = {S83453, 136, { &P14017,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P990 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2771243 = {S83454, 136, { &P19911,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-E970 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2772408 = {S83455, 137, { &P23167,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-H9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2781827 = {S83456, 126, { &P24146,&P5629,&P15760,&P18092 }};	/*Useragent - FPT-FPT-F2/1.0 Linux/2.6.35.7 Android/2.3.5 Release/03.30.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2939593 = {S83457, 209, { &P24146,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; FPT-FPT-F2/1.0 Android/2.3.5 Release/05.23.2013 Browser/AppleWebKit533.1 Build/MocorDroid2.3.5) AppleWebKit533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2781917 = {S83458, 97, { &P23226,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Videocon_A15) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2797113 = {S83459, 134, { &P23136,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Videocon A27i Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2799106 = {S83460, 145, { &P17968,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2869279 = {S83461, 145, { &P17968,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2869789 = {S83462, 152, { &P17968,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2849214 = {S83463, 156, { &P21033,&P18066,&P20775,&P18092 }};	/*Useragent - MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U1 Build/HCL ME Tablet U1) AppleWebKit/534.30 (KHTML like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3035763 = {S83464, 105, { &P19955,&P17484,&P17373,&P18092 }};	/*Useragent - SAMSUNG GT-S3570 Opera/9.80 (J2ME/MIDP; Opera Mini/6.5.30607/30.3697; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2944899 = {S83465, 98, { &P19862,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax A110) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3021659 = {S83466, 93, { &P18442,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7100) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3022970 = {S83467, 145, { &P20105,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A919 Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974710 = {S83468, 137, { &P21028,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Smartfren Tab 7 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2997765 = {S83469, 129, { &P23313,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; M8 pro Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2999485 = {S83470, 136, { &P23160,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; KB901 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3013091 = {S83471, 89, { &P16568,&P5567,&P18525,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/6.0.3/27.2338; U; en) Presto/2.8.119 240X400 Pantech CDM8999*/
const Device UA3023735 = {S83472, 161, { &P14433,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9003 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3094771 = {S83473, 137, { &P18679,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-B5330 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094498 = {S83474, 133, { &P14997,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S100 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081619 = {S83475, 145, { &P20105,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A919 Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081681 = {S83476, 152, { &P23175,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5021E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094345 = {S83477, 142, { &P24112,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; i-mobile IQ X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA94 = {S83478, 49, { &P12472,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-A455/1.0 UP.Browser/7.2.7.5.646 (GUI) MMP/2.0*/
const Device UA32158 = {S83479, 51, { &P12472,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-A455/00.72 UP.Browser/7.2.7.5.606 (GUI) MMP/2.0*/
const Device UA22221 = {S83480, 173, { &P15404,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; MB861 Build/Blur_Version.4.1.150.MB861.KT.en.KR Flex/P010) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA22222 = {S83481, 143, { &P15404,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; MB861 Build/OLYKT_U4_1.15.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59582 = {S83482, 143, { &P15404,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; MB861 Build/OLYKT_U4_1.18.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59583 = {S83483, 173, { &P15404,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; MB861 Build/Blur_Version.4.1.180.MB861.KT.en.KR Flex/P010) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59584 = {S83484, 143, { &P15404,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; MB861 Build/OLYKT_U4_1.21.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59585 = {S83485, 173, { &P15404,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; MB861 Build/Blur_Version.4.1.210.MB861.KT.en.KR Flex/P010) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59892 = {S83486, 104, { &P15739,&P17485,&P4223,&P18092 }};	/*Useragent - SonyEricssonW20a/R7DA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2*/
const Device UA27942 = {S83487, 106, { &P12191,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) garmin-asus-Nuvifone-M10/1.0*/
const Device UA42119 = {S83488, 121, { &P11950,&P5477,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.1-update1;Zh_cn;X10i;480*854;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA751843 = {S83489, 72, { &P11950,&P5477,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.1-update1;Zh_cn;X10i;480*854;)UCWEB7.8.0.95/139/444*/
const Device UA54091 = {S83490, 156, { &P14115,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SonyEricssonM1a/R1BA; Windows Phone 6.5; Profile/MIDP-2.1; Configuration/CLDC-1.1; Windows Phone 6.5.3.5)*/
const Device UA69314 = {S83491, 30, { &P15373,&P5129,&P24663,&P18092 }};	/*Useragent - Nokia5530/UCWEB7.9.0.94/50/999*/
const Device UA343638 = {S83492, 42, { &P15373,&P5129,&P24663,&P18092 }};	/*Useragent - Nokia5530/UC Browser7.9.0.102/50/355/UCWEB*/
const Device UA89929 = {S83493, 41, { &P16808,&P22784,&P17470,&P18092 }};	/*Useragent - MicromaxQ7/MIDP2.0/CLDC1.1/Screen-320X240*/
const Device UA391066 = {S83494, 54, { &P17876,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3W UP.Browser/6.2_7.2.7.1.K.6.210 (GUI) MMP/2.0*/
const Device UA1943153 = {S83495, 93, { &P15364,&P7127,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:8.0) Gecko/X Firefox/8.0,gzip(gfe) (via translate.google.com)*/
const Device UA560376 = {S83496, 85, { &P15526,&P5448,&P4779,&P18092 }};	/*Useragent - HD_mini_T5555 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA2805498 = {S83497, 89, { &P15526,&P5448,&P4779,&P18092 }};	/*Useragent - HTC_HD_mini_T5555 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA566064 = {S83498, 77, { &P16087,&P5448,&P4779,&P18092 }};	/*Useragent - 480x800 LGE VS750 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA590206 = {S83499, 75, { &P15364,&P7133,&P18099,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:10.0.1) Gecko/X Firefox/10.0.1 slurp*/
const Device UA591141 = {S83500, 68, { &P15364,&P7133,&P18099,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:10.0.2) Gecko/X Firefox/10.0.2 slurp*/
const Device UA591193 = {S83501, 81, { &P15364,&P7133,&P18099,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:10.0.2) Gecko/X Firefox/10.0.2 slurp slurp*/
const Device UA1953333 = {S83502, 71, { &P15364,&P7133,&P18099,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:10.0) Gecko/X Firefox/10.0 slurp*/
const Device UA874606 = {S83503, 135, { &P17595,&P17295,&P22457,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.19 (KHTML, like Gecko) Iron/18.0.1050.0 Chrome/18.0.1050.0 Safari/535.19*/
const Device UA899062 = {S83504, 146, { &P21427,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Woxter Tablet PC 100CX Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA911303 = {S83505, 159, { &P17835,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK BNTV250 Build/GINGERBREAD 1.4.3)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1983300 = {S83506, 160, { &P17835,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK BNTV250A Build/GINGERBREAD 1.4.3)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1989530 = {S83507, 140, { &P23274,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TAB-PROTAB2-IPS9 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1994728 = {S83508, 137, { &P23989,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; Yummy Duo Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2223593 = {S83509, 137, { &P21355,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KM-E100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2257032 = {S83510, 65, { &P17976,&P17979,&P20445,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:14.0) Gecko/14.0 Firefox/2.0.1.1*/
const Device UA2749965 = {S83511, 71, { &P17976,&P17979,&P20445,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:2.0.2.4) Gecko/2.0.2.4 Firefox/2.0.2.4*/
const Device UA2344415 = {S83512, 146, { &P23943,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; tr-tr; Vodafone 958 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2350286 = {S83513, 137, { &P13074,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LG-P930 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2354821 = {S83514, 137, { &P13074,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P930 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2421674 = {S83515, 175, { &P18652,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; Micromax A87 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2355146 = {S83516, 132, { &P22041,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6810 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2615084 = {S83517, 132, { &P22041,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P6810 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2621266 = {S83518, 138, { &P22041,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6810 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2506979 = {S83519, 181, { &P21489,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Rezound Build/IML74K; CyanogenMod-(/'o')/ _|____|_ 1.9.4 Linaro) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936740 = {S83520, 111, { &P17595,&P18207,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/536.30.1 (KHTML, like Gecko) Version/5 Safari/533.16*/
const Device UA2409373 = {S83521, 90, { &P15364,&P17285,&P22514,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.8.1.6) Gecko/X Firefox/2.0 Wyzo/0.5.3*/
const Device UA2459482 = {S83522, 135, { &P20363,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; one touch T20 Build/MAIN) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2482625 = {S83523, 130, { &P23599,&P18110,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-50EX645; CC/USA; en) Presto/2.10.250 Version/11.60*/
const Device UA2528563 = {S83524, 120, { &P14367,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA6710s) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2538202 = {S83525, 164, { &P15985,&P6437,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6200 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2554297 = {S83526, 35, { &P12473,&P5204,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia2730c-1/10.45)*/
const Device UA2598604 = {S83527, 139, { &P21019,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E270K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2673231 = {S83528, 135, { &P23190,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Marquis_MP977 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2674449 = {S83529, 137, { &P18507,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8160 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2680969 = {S83530, 88, { &P22385,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-I8262D Build/JZO54K.I8262DZCAMC2;480*800)*/
const Device UA2680970 = {S83531, 88, { &P22385,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-I8262D Build/JZO54K.I8262DZCAMA9;480*800)*/
const Device UA2680984 = {S83532, 88, { &P22385,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-I8262D Build/JZO54K.I8262DZCAMB2;480*800)*/
const Device UA2681021 = {S83533, 155, { &P22385,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2850083 = {S83534, 63, { &P15364,&P16110,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; Windows NT 6.2; Trident/4.0)*/
const Device UA2727729 = {S83535, 94, { &P20094,&P18110,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Adr (Linux; U; 4.0.4; zh-cn; HUAWEI U8825D Build/U8825DV100R001C17B951;480*800)*/
const Device UA2745901 = {S83536, 94, { &P20094,&P18110,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Adr (Linux; U; 4.0.4; zh-cn; HUAWEI U8825D Build/U8825DV100R001C17B935;480*800)*/
const Device UA2848143 = {S83537, 94, { &P20094,&P18110,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Adr (Linux; U; 4.0.4; zh-cn; HUAWEI U8825D Build/U8825DV100R001C17B956;480*800)*/
const Device UA3070137 = {S83538, 94, { &P20094,&P18110,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Adr (Linux; U; 4.0.4; zh-cn; HUAWEI U8825D Build/U8825DV100R001C17B933;480*800)*/
const Device UA2749368 = {S83539, 147, { &P21004,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VERTU Ti Build/4.0.4.A.0.1.72) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2753700 = {S83540, 136, { &P22929,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_E10310 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2768114 = {S83541, 138, { &P19856,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-LS970 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2783701 = {S83542, 159, { &P18449,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST27i Build/6.2.A.0.400) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2782827 = {S83543, 172, { &P15877,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2782662 = {S83544, 139, { &P12616,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WT19i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2997628 = {S83545, 156, { &P18505,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MB886 Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2799591 = {S83546, 136, { &P22955,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P714 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2944810 = {S83547, 161, { &P14433,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9003 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3091924 = {S83548, 161, { &P14433,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; GT-I9003 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2975711 = {S83549, 137, { &P18637,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9260 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022814 = {S83550, 134, { &P23160,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KB901 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3009918 = {S83551, 156, { &P14482,&P5843,&P24678,&P18092 }};	/*Useragent - NokiaC3-01/5.0 (07.51) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac3-01) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3020421 = {S83552, 137, { &P12496,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I897 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3058999 = {S83553, 165, { &P16263,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI MediaPad Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3078666 = {S83554, 165, { &P16263,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MediaPad 7 Lite Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3021880 = {S83555, 133, { &P24129,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; A900 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3064712 = {S83556, 156, { &P22941,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6812 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3061810 = {S83557, 159, { &P13018,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; MT11i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3062245 = {S83558, 161, { &P14877,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109270 = {S83559, 161, { &P14877,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3081225 = {S83560, 137, { &P22853,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC 802w Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109351 = {S83561, 114, { &P14711,&P5204,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-01/11.40; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA3113418 = {S83562, 142, { &P12280,&P22882,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X; zh-CN) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 UCBrowser/9.3.0.326 Mobile*/
const Device UA3128355 = {S83563, 144, { &P12280,&P22882,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X; zh-CN) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 UCBrowser/9.3.0.326 Mobile*/
const Device UA17357 = {S83564, 106, { &P11813,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-F488/1.0 SHP/VPP/R5 NetFront/3.4 Qtv/5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA31867 = {S83565, 112, { &P13902,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHM310L/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA63235 = {S83566, 111, { &P13902,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHM310/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA20540 = {S83567, 111, { &P13559,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-T356-parrot/T356UWJG3 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA61577 = {S83568, 104, { &P13559,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-T356/T356TLJE4 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA21905 = {S83569, 101, { &P14313,&P5208,&P4902,&P18092 }};	/*Useragent - Mozilla/4.0 (BREW 3.1.5; U; en-us; Samsung; PLS_M240; POLARIS/6.1/WAP) MMP/2.0 Configuration/CLDC-1.1*/
const Device UA61657 = {S83570, 101, { &P14313,&P5208,&P4902,&P18092 }};	/*Useragent - Mozilla/4.0 (BREW 3.1.5; U; en-us; Samsung; SPH_M240; POLARIS/6.1/WAP) MMP/2.0 Configuration/CLDC-1.1*/
const Device UA22254 = {S83571, 107, { &P13779,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-A437/A437UCGDA Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA90777 = {S83572, 136, { &P17585,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; it-it; OliPad 100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA258516 = {S83573, 115, { &P17585,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; it-; OliPad 100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA258519 = {S83574, 134, { &P17585,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; it-; OliPad 100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023496 = {S83575, 117, { &P17585,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; it-it; OliPad 100) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA237873 = {S83576, 132, { &P17595,&P17296,&P22451,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.0 Chrome/13.0.800.0 Safari/535.1*/
const Device UA286819 = {S83577, 122, { &P15364,&P18110,&P22445,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Iron/7.0.520.0 Chrome/7.0.520.0 Safari/534.7*/
const Device UA1852018 = {S83578, 124, { &P15364,&P18110,&P22445,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Iron/7.0.520.0 Chrome/7.0.520.0 Safari/534.7*/
const Device UA331480 = {S83579, 132, { &P22381,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; SN10T1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA346287 = {S83580, 81, { &P15364,&P7133,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; Opera Mobi/23731; U; en) Presto/2.9.201 Version/11.50*/
const Device UA353643 = {S83581, 99, { &P15364,&P18346,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.63 Safari/535.7*/
const Device UA909872 = {S83582, 151, { &P15911,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT862 Build/IML74K; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2158013 = {S83583, 30, { &P11854,&P5890,&P24668,&P18092 }};	/*Useragent - NokiaE71/UCWEB8.3.0.133/28/800*/
const Device UA2776012 = {S83584, 79, { &P11854,&P5890,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiae71-1) UCBrowser8.3.1.161/70/352/UCWEB Mobile*/
const Device UA895892 = {S83585, 81, { &P14748,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-in; NokiaE72-1)/UC Browser8.4.0.159/28/352/UCWEB Mobile*/
const Device UA1971969 = {S83586, 32, { &P14748,&P5890,&P24670,&P18092 }};	/*Useragent - NokiaE72-1/UCWEB8.4.0.150/28/999*/
const Device UA1992576 = {S83587, 132, { &P15767,&P17146,&P20776,&P18092 }};	/*Useragent - MQQBrowser/2.5.1 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA2023481 = {S83588, 140, { &P13724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-I9000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030342 = {S83589, 140, { &P13724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; SGH-I9000 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026461 = {S83590, 131, { &P20496,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; W50 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026992 = {S83591, 137, { &P21303,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; Fly_IQ275 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2758545 = {S83592, 133, { &P21303,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; IQ275 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2262019 = {S83593, 165, { &P14186,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2237850 = {S83594, 161, { &P15798,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2421551 = {S83595, 160, { &P18482,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; A75 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2402159 = {S83596, 77, { &P15483,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_ChaCha_A810e; 320*480) UCWEB7.9.0.94/139/352*/
const Device UA2502766 = {S83597, 77, { &P15483,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_ChaCha_A810e; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2841726 = {S83598, 77, { &P15483,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_ChaCha_A810e; 320*480) UCWEB7.9.0.94/139/352*/
const Device UA2421326 = {S83599, 139, { &P19977,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; IM-A775C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752691 = {S83600, 60, { &P15364,&P5690,&P20794,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:7.0.1) Gecko/X Postbox/3.0.8*/
const Device UA2458223 = {S83601, 98, { &P14912,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung gt-c3222) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2529398 = {S83602, 222, { &P14912,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung gt-c3222) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile, SAMSUNG-GT-C3222/C3222DDJL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA2470559 = {S83603, 32, { &P15364,&P17017,&P21915,&P18092 }};	/*Useragent - contype chromeframe/24.0.1312.52*/
const Device UA2490198 = {S83604, 32, { &P15364,&P17017,&P21915,&P18092 }};	/*Useragent - contype chromeframe/24.0.1312.56*/
const Device UA2510864 = {S83605, 32, { &P15364,&P17017,&P21915,&P18092 }};	/*Useragent - contype chromeframe/24.0.1312.57*/
const Device UA2537546 = {S83606, 90, { &P15147,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; ST15i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2565995 = {S83607, 169, { &P15397,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; ST18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2579799 = {S83608, 135, { &P23862,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HS-F1 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2580963 = {S83609, 135, { &P22647,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; Ezee'TAB901 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2599617 = {S83610, 148, { &P21911,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; AMAZE S200 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2653113 = {S83611, 92, { &P21527,&P17979,&P18062,&P18092 }};	/*Useragent - CoolPad8150D_CMCC_TD/1.0 Linux/3.0.8 Android/4.0 Release/10.15.2012 Browser/AppleWebkit534.3*/
const Device UA2657828 = {S83612, 62, { &P15364,&P24094,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; ; en) Presto/2.10.287 Version/12.00*/
const Device UA2659114 = {S83613, 171, { &P20001,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Lenovo S720 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/33482*/
const Device UA2676983 = {S83614, 145, { &P18556,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD+ Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2696587 = {S83615, 133, { &P23262,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TAC-70031 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2705985 = {S83616, 140, { &P22663,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S11t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719585 = {S83617, 144, { &P22655,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C5503 Build/10.1.1.A.1.253) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719627 = {S83618, 144, { &P22655,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C5503 Build/10.1.1.A.1.200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719628 = {S83619, 144, { &P22655,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C5503 Build/10.1.1.A.1.210) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719649 = {S83620, 144, { &P22655,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C5503 Build/10.1.1.A.1.191) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719667 = {S83621, 143, { &P22655,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C5503 Build/10.1.1.A.0.72) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719675 = {S83622, 143, { &P22655,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C5503 Build/10.1.1.A.1.89) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719680 = {S83623, 144, { &P22655,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; sv-se; C5503 Build/10.1.1.A.1.259) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719689 = {S83624, 144, { &P22655,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C5503 Build/10.1.1.A.1.164) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719704 = {S83625, 150, { &P22655,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; C5503 Build/LAGAN-JB-APPS-X-1000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2961719 = {S83626, 66, { &P22655,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; C5503 Build/10.1.1.A.1.310)*/
const Device UA2738808 = {S83627, 90, { &P18450,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST23i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2783071 = {S83628, 143, { &P22749,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-04E Build/10.1.1.D.0.179) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784794 = {S83629, 165, { &P18645,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2798768 = {S83630, 149, { &P20137,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Xtreamer_Mobile_AiKi Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2872522 = {S83631, 135, { &P20137,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Aiki-5 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2863268 = {S83632, 171, { &P12972,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T989 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SGH-T989, 42, 1.9.1)*/
const Device UA2989873 = {S83633, 171, { &P12972,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T989 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SGH-T989, 42, 1.9.1)*/
const Device UA2869200 = {S83634, 151, { &P20146,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9082 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3050969 = {S83635, 152, { &P20146,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9082i Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2874773 = {S83636, 136, { &P23231,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E988 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893102 = {S83637, 167, { &P23694,&P18111,&P21729,&P18092 }};	/*Useragent - OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Symphony Xplorer W35 Build/UNKNOWN) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2898724 = {S83638, 156, { &P18680,&P18066,&P23482,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2944454 = {S83639, 155, { &P18241,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-E400 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2944513 = {S83640, 93, { &P18283,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9300) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3023128 = {S83641, 134, { &P18253,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT894 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3030828 = {S83642, 162, { &P19862,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Micromax A110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3021133 = {S83643, 138, { &P15850,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LU6200 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974265 = {S83644, 137, { &P20124,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ZTE V970 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975978 = {S83645, 140, { &P19846,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-05D Build/9.1.C.0.475) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2981907 = {S83646, 128, { &P15364,&P24094,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46HX750; CC/USA; en) Presto/2.10.250 Version/11.60*/
const Device UA2981970 = {S83647, 128, { &P15364,&P24094,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46EX650; CC/VNM; vi) Presto/2.10.250 Version/11.60*/
const Device UA3039765 = {S83648, 158, { &P15364,&P24094,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; Opera TV Store/4510; U; el) Presto/2.10.250 Version/11.60 Model/Sony-KDL-55HX750 SonyCEBrowser/1.0 (KDL-55HX750; CTV/PKG1.910EUA; GRC)*/
const Device UA3039783 = {S83649, 106, { &P15364,&P24094,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; Sony BRAVIA/PKG1.901GAA (KDL-46EX650,,en,MY); en) Presto/2.10.250 Version/11.60*/
const Device UA3070612 = {S83650, 128, { &P15364,&P24094,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL55HX850; CC/USA; en) Presto/2.10.250 Version/11.60*/
const Device UA3088062 = {S83651, 75, { &P15364,&P24094,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; Opera TV/19; U; pt-BR) Presto/2.9.201 Version/11.60*/
const Device UA3098821 = {S83652, 128, { &P15364,&P24094,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40HX853; CC/GBR; en) Presto/2.10.250 Version/11.60*/
const Device UA3020802 = {S83653, 137, { &P18622,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A820L Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023670 = {S83654, 98, { &P19862,&P18111,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Micromax A110) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3026819 = {S83655, 159, { &P12258,&P17483,&P24679,&P18092 }};	/*Useragent - LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gs290) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3027166 = {S83656, 160, { &P20289,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC6435LVW Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/dlx*/
const Device UA3083918 = {S83657, 160, { &P20289,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC6435LVW Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/dlx*/
const Device UA3128239 = {S83658, 160, { &P20289,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; HTC6435LVW Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/dlx*/
const Device UA3032611 = {S83659, 163, { &P14183,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I997 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/infuse4g*/
const Device UA3093708 = {S83660, 163, { &P14183,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I997 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/infuse4g*/
const Device UA3033261 = {S83661, 162, { &P14743,&P17485,&P24679,&P18092 }};	/*Useragent - SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj20i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3059077 = {S83662, 170, { &P18673,&P18066,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S7562 Build/IMM76I) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3093678 = {S83663, 127, { &P17926,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AT100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094115 = {S83664, 134, { &P23160,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KB901 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081226 = {S83665, 137, { &P20289,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; HTC6435LVW Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA62368 = {S83666, 61, { &P11999,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia6086/2.0 (06.00) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24262 = {S83667, 79, { &P12091,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-T456/T456UZJB4 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA67736 = {S83668, 73, { &P12141,&P5035,&P3418,&P18092 }};	/*Useragent - BlackBerry8900/4.6.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100*/
const Device UA2153730 = {S83669, 78, { &P15364,&P7127,&P22730,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 5.0; Avant Browser)*/
const Device UA2154687 = {S83670, 78, { &P15364,&P7127,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 5.0; Avant Browser)*/
const Device UA2945221 = {S83671, 161, { &P15364,&P7127,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.0; Avant Browser; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; Media Center PC 6.0; MS-RTC LM 8)*/
const Device UA401062 = {S83672, 62, { &P20318,&P22784,&P18246,&P18092 }};	/*Useragent - Micromax X450/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA546346 = {S83673, 26, { &P15364,&P17017,&P24163,&P18092 }};	/*Useragent - Opera/9.81 (SaferSurf; de)*/
const Device UA723322 = {S83674, 135, { &P22252,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Novo7Basic Build/ICS_MR0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA762419 = {S83675, 134, { &P22252,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Novo7Basic Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026073 = {S83676, 129, { &P22381,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SN10T1 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033566 = {S83677, 129, { &P22381,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; SN10T1 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2158773 = {S83678, 79, { &P12456,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian73-1) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA1995966 = {S83679, 95, { &P15364,&P18110,&P20910,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) rekonq/1.0 Safari/534.34*/
const Device UA2202190 = {S83680, 114, { &P15364,&P5690,&P19986,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Sleipnir/3.7.3*/
const Device UA2231058 = {S83681, 114, { &P15364,&P5690,&P19986,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4 Sleipnir/3.7.4*/
const Device UA2235556 = {S83682, 137, { &P21489,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Rezound Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2372971 = {S83683, 91, { &P12193,&P5890,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokiac5-00.2) UCBrowser8.3.0.154/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA2644501 = {S83684, 89, { &P12193,&P5890,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokiac5-00) UCBrowser8.3.0.154/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA2469288 = {S83685, 164, { &P14967,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2421024 = {S83686, 156, { &P15989,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; ST21i2 Build/11.0.A.1.8) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2424574 = {S83687, 156, { &P15989,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; ST21i Build/11.0.A.0.16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2424288 = {S83688, 130, { &P24012,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; TA7300 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2429426 = {S83689, 158, { &P18224,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Ascend G300 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2458731 = {S83690, 138, { &P12482,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-T959 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2663826 = {S83691, 137, { &P12482,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T959 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2485533 = {S83692, 161, { &P20231,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A21 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2507564 = {S83693, 169, { &P20231,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Karbonn A21 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2487850 = {S83694, 100, { &P16384,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5380k) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2729905 = {S83695, 100, { &P16384,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5380d) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2489963 = {S83696, 62, { &P15364,&P18110,&P21767,&P18092 }};	/*Useragent - OneBrowser/Mini1.1 (Linux; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2529115 = {S83697, 144, { &P22687,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ZEN U5 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2534868 = {S83698, 143, { &P15438,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; HTC Sensation Build/JOP40G) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2537616 = {S83699, 138, { &P15438,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Sensation Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2601952 = {S83700, 142, { &P15438,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Sensation Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2555291 = {S83701, 139, { &P14969,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Desire S Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2864568 = {S83702, 149, { &P14969,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Desire S Build/JellyFireBean v2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987565 = {S83703, 138, { &P14969,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Desire S Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654428 = {S83704, 132, { &P21838,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; MID9738G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2578747 = {S83705, 138, { &P21868,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ALLVIEW3CONECT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2659359 = {S83706, 171, { &P21155,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Lenovo A800 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/33482*/
const Device UA2659572 = {S83707, 93, { &P13474,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAX5-01) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2666773 = {S83708, 139, { &P18072,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668690 = {S83709, 139, { &P18072,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2702625 = {S83710, 139, { &P18072,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2674983 = {S83711, 134, { &P23160,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KB901 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2799054 = {S83712, 99, { &P12080,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3653) U2/1.0.0 UCBrowser/9.0.0.295 U2/1.0.0 Mobile*/
const Device UA2748745 = {S83713, 143, { &P23643,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASUS Tablet P1801-T Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2752372 = {S83714, 152, { &P15241,&P5843,&P24677,&P18092 }};	/*Useragent - Nokia300/5.0 (07.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia300) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2753345 = {S83715, 140, { &P18465,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Amazon Kindle Fire Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2763986 = {S83716, 137, { &P23564,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HS-U950 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2769729 = {S83717, 153, { &P15919,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA3121519 = {S83718, 153, { &P15919,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2784829 = {S83719, 159, { &P12616,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WT19i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2771202 = {S83720, 133, { &P20868,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M040 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782972 = {S83721, 156, { &P18243,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P880 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2935264 = {S83722, 139, { &P24098,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; AS870 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103398 = {S83723, 61, { &P24098,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; AS870 4G Build/JZO54K)*/
const Device UA2943847 = {S83724, 93, { &P22872,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S5282) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2955765 = {S83725, 149, { &P24737,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; STARXTREM Build/GM_ZTEV1.0.0B03) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058325 = {S83726, 149, { &P24737,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; STARXTREM Build/GM_ZTEV1.0.0B05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956692 = {S83727, 141, { &P19842,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3013691 = {S83728, 168, { &P11787,&P5204,&P24678,&P18092 }};	/*Useragent - Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3120classic) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3022203 = {S83729, 134, { &P17283,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; MB860 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021754 = {S83730, 139, { &P13018,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT11i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3025853 = {S83731, 157, { &P12968,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT26i Build/6.1.A.2.55) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3108977 = {S83732, 157, { &P12968,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LT26i Build/6.1.A.2.45) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3025942 = {S83733, 99, { &P15547,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-c6712) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3083284 = {S83734, 140, { &P24096,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Orange Kivo Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080368 = {S83735, 137, { &P22953,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F220K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3131479 = {S83736, 144, { &P21611,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA8890 = {S83737, 126, { &P13359,&P6712,&P4030,&P18092 }};	/*Useragent - Nokia3711fold/5.0 (03.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA18171 = {S83738, 69, { &P14020,&P5448,&P22271,&P18092 }};	/*Useragent - HTC_S730 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA44297 = {S83739, 72, { &P13822,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-R109/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA518329 = {S83740, 32, { &P13429,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH03B(c500;TB;W30H18)*/
const Device UA67479 = {S83741, 143, { &P14900,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; Lenovo 3GW100 Build/FRG83) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA705169 = {S83742, 122, { &P14900,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; 3GW100 Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2028238 = {S83743, 127, { &P14900,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; 3GW100 Build/FRG83) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA553312 = {S83744, 54, { &P17594,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC3X UP.Browser/6.2_7.2.7.1.K.5.207 (GUI) MMP/2.0*/
const Device UA261606 = {S83745, 77, { &P15364,&P5690,&P4536,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; Opera Mobi/49; U; en) Presto/2.4.18 Version/10.00*/
const Device UA851797 = {S83746, 154, { &P15201,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Sensation 4G Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031868 = {S83747, 171, { &P15201,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Sensation 4G with Beats Audio Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2868809 = {S83748, 155, { &P15201,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Sensation 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2999479 = {S83749, 158, { &P15201,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA393297 = {S83750, 135, { &P22255,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Adam Build/Beast2.2.1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA555817 = {S83751, 129, { &P22255,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Adam Build/NIAX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019192 = {S83752, 129, { &P22255,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; Adam Build/NIPX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019263 = {S83753, 152, { &P22255,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Adam Build/Cromnia[Beta3]ClemsynKernel) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA661582 = {S83754, 97, { &P15842,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; PI39100)*/
const Device UA731701 = {S83755, 101, { &P15842,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; TITAN X310e)*/
const Device UA2027002 = {S83756, 171, { &P13974,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2033958 = {S83757, 171, { &P13974,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352*/
const Device UA2027762 = {S83758, 133, { &P21482,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-mx; MID7108SC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2500737 = {S83759, 133, { &P21482,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; MID7108SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2778460 = {S83760, 133, { &P21482,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.5; es-es; MID7108SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2028276 = {S83761, 132, { &P14849,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I800 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2030336 = {S83762, 150, { &P20964,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Turkcell Maxi Plus 5 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2159079 = {S83763, 90, { &P22799,&P22840,&P4780,&P18092 }};	/*Useragent - MyPhoneTW26-3D DUO/MTKRelease/2012/02/15Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0*/
const Device UA2228099 = {S83764, 157, { &P15991,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2234299 = {S83765, 169, { &P15147,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; ST15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2368487 = {S83766, 139, { &P20925,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT-AS45SE Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2375399 = {S83767, 71, { &P16356,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.3;Zh_cn;GT-N7000;800*1280;)UCWEB7.8.0.95/139/444*/
const Device UA2476660 = {S83768, 71, { &P16356,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.4;Zh_cn;GT-N7000;800*1280;)UCWEB7.8.0.95/139/444*/
const Device UA2381976 = {S83769, 132, { &P21202,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-P500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2420151 = {S83770, 126, { &P15302,&P17485,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; sonyericssonwt13i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2833856 = {S83771, 123, { &P15302,&P17485,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; fa; sonyericssonwt13i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2455539 = {S83772, 161, { &P18061,&P5096,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 BaDoinkDownloader_Android_2.0*/
const Device UA2455579 = {S83773, 161, { &P18061,&P5096,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; sdk Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 BaDoinkDownloader_Android_2.0*/
const Device UA2507029 = {S83774, 151, { &P18583,&P5043,&P24675,&P18092 }};	/*Useragent - Nokia110/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia110) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2504954 = {S83775, 128, { &P20838,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHT21 Build/SB221) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2537208 = {S83776, 90, { &P12798,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT18i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2583313 = {S83777, 134, { &P18737,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; X909 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2654016 = {S83778, 163, { &P18476,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I577 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2671278 = {S83779, 136, { &P18281,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P940 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2780899 = {S83780, 138, { &P18281,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-KU5400 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673236 = {S83781, 151, { &P18125,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID4 Build/6.7.2-180_DR4-16_M2-37) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675676 = {S83782, 130, { &P20154,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5770D Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2702742 = {S83783, 151, { &P23529,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; galaxy prime plus Build/1.0.459_X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2732695 = {S83784, 133, { &P21191,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MPDC77 BT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2733268 = {S83785, 133, { &P20867,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; miTab NEO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2799274 = {S83786, 93, { &P18214,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S7500) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2754074 = {S83787, 126, { &P17686,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; MZ601 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755555 = {S83788, 131, { &P23314,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; G3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2764083 = {S83789, 140, { &P18613,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T329w Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2776725 = {S83790, 140, { &P18613,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC_T329w Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2769659 = {S83791, 140, { &P21155,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A800 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2770228 = {S83792, 152, { &P22960,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773154 = {S83793, 159, { &P15539,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.4; HTC Glacier Build/GRJ22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2945791 = {S83794, 151, { &P19860,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A9+ Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3007541 = {S83795, 159, { &P19860,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A9+ Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2963496 = {S83796, 137, { &P24105,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Archos 80 Xenon Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3058896 = {S83797, 168, { &P14557,&P5843,&P24679,&P18092 }};	/*Useragent - Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6303classic) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3081041 = {S83798, 130, { &P19852,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7511 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094556 = {S83799, 136, { &P24065,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-S7272 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094050 = {S83800, 93, { &P18672,&P5629,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5302) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3094450 = {S83801, 133, { &P22653,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTL22 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3132016 = {S83802, 136, { &P21935,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E980 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA2674 = {S83803, 87, { &P12558,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC_P3650) Opera 8.65 [sv]*/
const Device UA2675 = {S83804, 87, { &P12558,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC_P3650) Opera 8.65 [en]*/
const Device UA21955 = {S83805, 87, { &P12558,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC_P3650) Opera 8.65 [pt]*/
const Device UA21956 = {S83806, 87, { &P12558,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC_P3650) Opera 8.65 [ru]*/
const Device UA21961 = {S83807, 87, { &P12558,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC_P3650) Opera 8.65 [nl]*/
const Device UA36090 = {S83808, 90, { &P12558,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC_P3650) Opera 8.65 [es-es]*/
const Device UA36088 = {S83809, 87, { &P12558,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC_P3650) Opera 8.65 [de]*/
const Device UA44517 = {S83810, 90, { &P12558,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC_P3650) Opera 8.65 [zh-tw]*/
const Device UA86918 = {S83811, 87, { &P12558,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC_P3650) Opera 8.65 [it]*/
const Device UA22363 = {S83812, 110, { &P15439,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-SGH-i200/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49413 = {S83813, 119, { &P15439,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-SGH-i200-ORANGE/BVHE6 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA32909 = {S83814, 80, { &P14768,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-F160(Zambia)/1.0 NetFront/3.4 QTV5.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA1931558 = {S83815, 77, { &P14768,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-F160(MTN)/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA401257 = {S83816, 123, { &P15364,&P17285,&P22422,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.3.0.0 Chrome/17.0.963.46 Safari/535.11*/
const Device UA590233 = {S83817, 130, { &P15364,&P17285,&P22422,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.3.0.0 Chrome/17.0.963.46 Safari/535.11*/
const Device UA646276 = {S83818, 126, { &P13051,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE-T U880 Build/GWK74) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1967201 = {S83819, 131, { &P13051,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE-T U880 Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2375235 = {S83820, 131, { &P13051,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE-T U880 Build GWK74) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA648612 = {S83821, 152, { &P15767,&P17090,&P21785,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) 1Password/3.5.9/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA915570 = {S83822, 152, { &P15767,&P17090,&P21785,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) 1Password/3.5.9/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA653492 = {S83823, 75, { &P15364,&P22473,&P17470,&P18092 }};	/*Useragent - curl/7.16.4 (i386-apple-darwin9.0) libcurl/7.16.4 OpenSSL/0.9.7l zlib/1.2.3*/
const Device UA1884687 = {S83824, 78, { &P15364,&P22473,&P17470,&P18092 }};	/*Useragent - curl/7.16.3 (powerpc-apple-darwin9.0) libcurl/7.16.3 OpenSSL/0.9.7l zlib/1.2.3*/
const Device UA925704 = {S83825, 65, { &P14637,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;LG;LG-E900;7.10;7740)*/
const Device UA925760 = {S83826, 66, { &P14637,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;LG;LG-E900h;7.10;7720)*/
const Device UA925806 = {S83827, 65, { &P14637,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;LG;LG-E900;7.10;8107)*/
const Device UA925835 = {S83828, 66, { &P14637,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;LG;LG-E900h;7.10;8107)*/
const Device UA925872 = {S83829, 65, { &P14637,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;LG;LG-E900;7.10;7720)*/
const Device UA2034646 = {S83830, 65, { &P14637,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;LG;LG-E900;7.10;8773)*/
const Device UA2818622 = {S83831, 66, { &P14637,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;LG;LG-E900h;7.10;8773)*/
const Device UA1951037 = {S83832, 153, { &P21961,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ro-ro; Allview_P2 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025898 = {S83833, 171, { &P17324,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352*/
const Device UA2027394 = {S83834, 171, { &P17324,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2029626 = {S83835, 171, { &P17324,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2545318 = {S83836, 171, { &P17324,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2349478 = {S83837, 171, { &P18700,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-B5512 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2261186 = {S83838, 158, { &P12056,&P5204,&P24670,&P18092 }};	/*Useragent - Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia7210supernova) UCBrowser8.4.0.159/70/444/UCWEB Mobile*/
const Device UA2714618 = {S83839, 126, { &P12280,&P17779,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU 6_1_3 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3*/
const Device UA2257121 = {S83840, 162, { &P17968,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; generic Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2257176 = {S83841, 164, { &P15985,&P6437,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6200 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2616227 = {S83842, 165, { &P15985,&P6437,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6200L Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2270435 = {S83843, 100, { &P15364,&P5690,&P22510,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.9) Gecko/X Firefox/3.0.9 Wyzo/3.0.3 FBSMTWB*/
const Device UA2642243 = {S83844, 92, { &P15364,&P5690,&P22510,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.9.0.9) Gecko/X Firefox/3.0.9 Wyzo/3.0.3*/
const Device UA2337428 = {S83845, 105, { &P15364,&P5448,&P3487,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) DV300 UP.Browser/7.2.7.2.559 (GUI)  MMP/2.0*/
const Device UA2952532 = {S83846, 105, { &P15364,&P5448,&P3487,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) DV150 UP.Browser/7.2.7.2.559 (GUI)  MMP/2.0*/
const Device UA2342354 = {S83847, 135, { &P21031,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ZP500 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2594238 = {S83848, 136, { &P21031,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZP500+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2387829 = {S83849, 118, { &P12707,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia701) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile*/
const Device UA2384341 = {S83850, 157, { &P18304,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2447746 = {S83851, 51, { &P23052,&P22828,&P17470,&P18092 }};	/*Useragent - ZEN M8/1.0; Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2456981 = {S83852, 163, { &P19998,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2595794 = {S83853, 165, { &P19998,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8812E Build/HuaweiC8812E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467768 = {S83854, 134, { &P20868,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M040 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2472027 = {S83855, 40, { &P17968,&P17979,&P24440,&P18092 }};	/*Useragent - Dolphin http client/9.2.0(197) (Android)*/
const Device UA2531692 = {S83856, 170, { &P13815,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3078680 = {S83857, 174, { &P13815,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; HTC_Wildfire Build/FRG83D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2567322 = {S83858, 140, { &P20738,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Andromax U Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA3000024 = {S83859, 152, { &P15919,&P18111,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.3.2.300*/
const Device UA2702032 = {S83860, 146, { &P13157,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC Sensation XE Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2704526 = {S83861, 166, { &P14225,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Desire HD Build/JellyMIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2767504 = {S83862, 167, { &P14225,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Desire HD Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2707392 = {S83863, 64, { &P17968,&P5629,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Android 2.3.6; WOW64; rv:17.0) Gecko/X Firefox/17.0*/
const Device UA2769492 = {S83864, 134, { &P23193,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCL21 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2772671 = {S83865, 137, { &P18622,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A820L Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798679 = {S83866, 132, { &P20231,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A21 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2999225 = {S83867, 140, { &P20231,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A21 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799399 = {S83868, 97, { &P22864,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A26) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2958196 = {S83869, 164, { &P19862,&P18111,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; Micromax A110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300*/
const Device UA2937706 = {S83870, 150, { &P23395,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T5-A Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3015814 = {S83871, 143, { &P23395,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T5-A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2942925 = {S83872, 172, { &P12056,&P5204,&P24678,&P18092 }};	/*Useragent - Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7210supernova) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2944560 = {S83873, 160, { &P22622,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Micromax A72 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2957139 = {S83874, 141, { &P17625,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; ST25i Build/6.2.A.0.400) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2963497 = {S83875, 92, { &P12280,&P18117,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 5_1_1; zh-CN; iPh3,1) U2/1.0.0 UCBrowser/9.1.0.287 U2/1.0.0 Mobile*/
const Device UA2966486 = {S83876, 92, { &P12280,&P18117,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 5_1_1; zh-CN; iPh4,1) U2/1.0.0 UCBrowser/9.1.0.287 U2/1.0.0 Mobile*/
const Device UA2975676 = {S83877, 152, { &P23672,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022557 = {S83878, 136, { &P22853,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC 802w Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3011506 = {S83879, 165, { &P12798,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Xperia Arc S Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/anzu*/
const Device UA3045341 = {S83880, 160, { &P12798,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Xperia Arc S Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3001201 = {S83881, 142, { &P24504,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Lenovo A830 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121407 = {S83882, 142, { &P24504,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-tw; Lenovo A830 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3122127 = {S83883, 64, { &P24504,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; Lenovo A830 Build/JOP40D)*/
const Device UA3021393 = {S83884, 99, { &P12137,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5620) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3026613 = {S83885, 93, { &P21261,&P18598,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.2; en-US; GT-I9500) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3061220 = {S83886, 137, { &P22941,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6812 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3078696 = {S83887, 138, { &P22941,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6812B Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3079748 = {S83888, 137, { &P22582,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I337 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125448 = {S83889, 137, { &P22582,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I337 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3128219 = {S83890, 137, { &P22582,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I337 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3111604 = {S83891, 102, { &P15795,&P21697,&P24641,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 710) U2/1.0.0 UCBrowser/3.1.1.343 U2/1.0.0 Mobile*/
const Device UA3132936 = {S83892, 128, { &P19783,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ME172V Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA5146 = {S83893, 57, { &P11747,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_asia_wwe / 1.0 / htc_bravo / 2.2*/
const Device UA28055 = {S83894, 52, { &P11747,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_wwe / 1.0 / htc_bravo / 2.2*/
const Device UA28058 = {S83895, 50, { &P11747,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player o2_de / 1.0 / htc_bravo / 2.2*/
const Device UA28059 = {S83896, 54, { &P11747,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player orange_uk / 1.0 / htc_bravo / 2.2*/
const Device UA28060 = {S83897, 51, { &P11747,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player tmo_uk / 1.0 / htc_bravo / 2.2*/
const Device UA36844 = {S83898, 54, { &P11747,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player orange_es / 1.0 / htc_bravo / 2.2*/
const Device UA45693 = {S83899, 54, { &P11747,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player telus_wwe / 1.0 / htc_bravo / 2.2*/
const Device UA45700 = {S83900, 51, { &P11747,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player h3g_uk / 1.0 / htc_bravo / 2.2*/
const Device UA45706 = {S83901, 58, { &P11747,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player virgin_mobile / 1.0 / htc_bravo / 2.2*/
const Device UA45707 = {S83902, 56, { &P11747,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player vodafone_uk / 1.0 / htc_bravo / 2.2*/
const Device UA53388 = {S83903, 54, { &P11747,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_wwe / 1.0 / htc_bravo / 2.3.3*/
const Device UA70802 = {S83904, 54, { &P11747,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_wwe / 1.0 / htc_bravo / 2.2.2*/
const Device UA71421 = {S83905, 56, { &P11747,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player telstra_wwe / 1.0 / htc_bravo / 2.2*/
const Device UA438875 = {S83906, 58, { &P11747,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player vodafone_uk / 1.0 / htc_bravo / 2.2.2*/
const Device UA438873 = {S83907, 58, { &P11747,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player vodafone_tr / 1.0 / htc_bravo / 2.2.2*/
const Device UA16432 = {S83908, 146, { &P14334,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android UbuntDroid-v4.5; en-us; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59811 = {S83909, 144, { &P14334,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android UbuntDroid-v6; en-us; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1941737 = {S83910, 165, { &P14334,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android StockROM on Crack 1.2 (2.2.2 EC07); en-us; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021488 = {S83911, 167, { &P14334,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android StockROM on Crack 1.1.2 (2.2.2 EC07); en-us; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023792 = {S83912, 163, { &P14334,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Ice Cream Froyo 1.2 (2.2.2 EC07); en-us; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025947 = {S83913, 140, { &P14334,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android IcyPop1.0; en-us; SPH-M910 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67419 = {S83914, 144, { &P13242,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0.1; es-us; A853 Build/SHLA_U2_02.02.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA67830 = {S83915, 142, { &P13242,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0; pt-br; A853 Build/SHLA_U2_01.03.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA646485 = {S83916, 142, { &P13242,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0; es-us; A853 Build/SHLA_U2_01.03.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1851549 = {S83917, 144, { &P13242,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.0.1; es-us; A853 Build/SHLA_U2_02.01.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA87720 = {S83918, 39, { &P13491,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP210 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA753509 = {S83919, 116, { &P15364,&P17188,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 4.0; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA192630 = {S83920, 123, { &P15364,&P17285,&P22403,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.24 (KHTML, like Gecko) Comodo_Dragon/11.4.0.0 Chrome/11.0.696.71 Safari/534.24*/
const Device UA316970 = {S83921, 130, { &P15364,&P17285,&P22403,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.24 (KHTML, like Gecko) Comodo_Dragon/11.4.0.0 Chrome/11.0.696.71 Safari/534.24*/
const Device UA628226 = {S83922, 35, { &P15518,&P17484,&P17347,&P18092 }};	/*Useragent - gt-s5230/UC Browser8.0.3.107/69/444*/
const Device UA769314 = {S83923, 40, { &P15518,&P17484,&P17347,&P18092 }};	/*Useragent - samsung-s5230/UC Browser8.0.3.107/69/352*/
const Device UA2518151 = {S83924, 35, { &P15518,&P17484,&P17347,&P18092 }};	/*Useragent - gt-s5230/UC Browser8.0.3.107/70/352*/
const Device UA659836 = {S83925, 159, { &P12562,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID2 GLOBAL Build/S273; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA865854 = {S83926, 87, { &P17976,&P5096,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2; Linux; Opera Tablet/ADR-X; U; de) Presto/2.8.149 Version/11.10*/
const Device UA2019119 = {S83927, 92, { &P17976,&P5096,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2.2; Linux; Opera Tablet/ADR-X; U; en-GB) Presto/2.8.149 Version/11.10*/
const Device UA2019367 = {S83928, 92, { &P17976,&P5096,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2.1; Linux; Opera Tablet/ADR-X; U; es-ES) Presto/2.8.149 Version/11.10*/
const Device UA2443965 = {S83929, 150, { &P13337,&P17485,&P24668,&P18092 }};	/*Useragent - SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-in; sonyericssonj108i) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA1953983 = {S83930, 127, { &P20382,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE-T U960s Build/GWK74) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2032942 = {S83931, 132, { &P20382,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE-T U960s Build/GWK74) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2373008 = {S83932, 132, { &P20382,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE-T U960s Build GWK74) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2227773 = {S83933, 138, { &P14969,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Desire S Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2367043 = {S83934, 138, { &P14969,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Desire S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2234925 = {S83935, 165, { &P14433,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2220370 = {S83936, 54, { &P12449,&P5890,&P20777,&P18092 }};	/*Useragent - MQQBrowser/2.6 (NokiaE5-00;SymbianOS/9.3 Series60/3.2)*/
const Device UA2669959 = {S83937, 99, { &P17595,&P17297,&P21808,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.2.16pre) Gecko/X Prism zdesktop/7.2.2*/
const Device UA2417039 = {S83938, 142, { &P21404,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo A580 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2687601 = {S83939, 133, { &P21404,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo A580 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2419477 = {S83940, 144, { &P21547,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Symphony S101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426804 = {S83941, 169, { &P15397,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; ST18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2432687 = {S83942, 229, { &P14186,&P5629,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2467341 = {S83943, 146, { &P23634,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; ELEMENT10 1 Build/ICS.g08refem611.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2486903 = {S83944, 147, { &P19863,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2558285 = {S83945, 94, { &P16591,&P22784,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq50) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2567510 = {S83946, 136, { &P20015,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SC-03E Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2570262 = {S83947, 163, { &P13123,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-S5360 Build/GINGERBREAD) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2706000 = {S83948, 162, { &P13123,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-S5360 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Mobile Safari/537.31 OPR/14.0.1074.54070*/
const Device UA2579764 = {S83949, 156, { &P21543,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Telenor_Smart_HD Build/IceCreamSandwich) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2583173 = {S83950, 137, { &P14183,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I997 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2704756 = {S83951, 138, { &P14183,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-I997 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583188 = {S83952, 137, { &P18281,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P940 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2863841 = {S83953, 139, { &P18281,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-KU5400 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2600838 = {S83954, 140, { &P18302,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT25i Build/4.1.B.0.631) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2600984 = {S83955, 130, { &P17168,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7310 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2634223 = {S83956, 131, { &P17168,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P7310 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2635879 = {S83957, 129, { &P17168,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; GT-P7310 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2666442 = {S83958, 155, { &P16292,&P18066,&P23369,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76B) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229 Mobile Safari/537.4 pss-webkit-request*/
const Device UA2670734 = {S83959, 140, { &P18662,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC J Z321e Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676583 = {S83960, 145, { &P20558,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ608 Build/7.7.1-141-7-FLEM-UMTS-LA) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2709485 = {S83961, 143, { &P20558,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOOM 2 ME Build/7.7.1-128_MZ608-14) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2696002 = {S83962, 137, { &P23070,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ThL W2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2707084 = {S83963, 138, { &P18072,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708563 = {S83964, 138, { &P18072,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2750942 = {S83965, 138, { &P18072,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E120K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2732147 = {S83966, 157, { &P15575,&P5629,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2751545 = {S83967, 156, { &P15575,&P5629,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2780777 = {S83968, 157, { &P15575,&P5629,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2733814 = {S83969, 146, { &P22915,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Lemon P9 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2734489 = {S83970, 155, { &P20729,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; CROSS A27 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2799208 = {S83971, 162, { &P18300,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; SGH-T769 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2869842 = {S83972, 141, { &P23498,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZTE Blade G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2939223 = {S83973, 138, { &P18513,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2975583 = {S83974, 140, { &P19759,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo K860 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022410 = {S83975, 137, { &P20075,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xperia S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2975987 = {S83976, 134, { &P21909,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-03E Build/10.1.E.0.269) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023527 = {S83977, 132, { &P20529,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A89 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3015428 = {S83978, 98, { &P12375,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIANokia N81) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3019373 = {S83979, 102, { &P12375,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIANokia N81 8GB) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3021506 = {S83980, 80, { &P18645,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile: SAMSUNG-SGH-I747; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA3022508 = {S83981, 72, { &P18645,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile: SPH-L710; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA3023315 = {S83982, 93, { &P18513,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6802) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3061278 = {S83983, 159, { &P12616,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; WT19i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3038896 = {S83984, 150, { &P22874,&P18598,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9502 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3083381 = {S83985, 138, { &P19902,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530M Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083589 = {S83986, 130, { &P20154,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5770D Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126896 = {S83987, 152, { &P15803,&P5843,&P24680,&P18092 }};	/*Useragent - Nokia200/2.0 (11.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia200) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA22863 = {S83988, 68, { &P15520,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-T2/1.0 ACS-NF/3.4 QTV2.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24404 = {S83989, 56, { &P15687,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-R38.0/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0*/
const Device UA33285 = {S83990, 86, { &P12891,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-G869/MTK Release/01.01.2011 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1*/
const Device UA2329314 = {S83991, 91, { &P12141,&P7149,&P4779,&P18092 }};	/*Useragent - BlackBerry8900/5.2.0.104 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/159/444*/
const Device UA785153 = {S83992, 52, { &P15364,&P17017,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (SaferSurf rv:10.0) Gecko/X Firefox/10.0*/
const Device UA1991937 = {S83993, 57, { &P15364,&P17017,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; rv:1.9b4) Gecko/X Firefox/10.0a4*/
const Device UA1993177 = {S83994, 38, { &P16463,&P17484,&P24630,&P18092 }};	/*Useragent - SAMSUNG-GT-I8350/UCWEB2.2.0.157/47/355*/
const Device UA2313989 = {S83995, 32, { &P14983,&P5055,&P24668,&P18092 }};	/*Useragent - NokiaC7-00/UCWEB8.3.0.133/51/999*/
const Device UA2520544 = {S83996, 89, { &P14983,&P5055,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokiac7-00) UCBrowser8.3.0.154/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA1993151 = {S83997, 85, { &P12142,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU5)/UC Browser8.5.0.183/50/352/UCWEB Mobile*/
const Device UA1995045 = {S83998, 155, { &P15991,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2009038 = {S83999, 81, { &P15364,&P5690,&P22552,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:14.0) Gecko/X Firefox/14.0.3.0 IceDragon/14.0.3.0*/
const Device UA2343261 = {S84000, 153, { &P18224,&P5629,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U8818 Build/HuaweiU8818) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033843 = {S84001, 137, { &P23924,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; HWA580 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2223490 = {S84002, 150, { &P13308,&P17485,&P24670,&P18092 }};	/*Useragent - SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-in; sonyericssonj105i) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2174479 = {S84003, 146, { &P20333,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax P500(Funbook) Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2195655 = {S84004, 125, { &P14010,&P17484,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5263) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile*/
const Device UA2356038 = {S84005, 67, { &P17346,&P18432,&P22071,&P18092 }};	/*Useragent - Instagram 3.1.2 (iPod touch; iPhone OS 6.0; en_US) AppleWebKit/420+*/
const Device UA2362871 = {S84006, 69, { &P17346,&P18432,&P22071,&P18092 }};	/*Useragent - Instagram 3.1.2 (iPod touch; iPhone OS 6.0.1; en_US) AppleWebKit/420+*/
const Device UA2347796 = {S84007, 159, { &P23917,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Explay_Advance Build/ExplayExplay_Advance) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2375915 = {S84008, 88, { &P23022,&P22835,&P17470,&P18092 }};	/*Useragent - SpiceM5665/MTK Release/01.01.2012 Browser/wap2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2398729 = {S84009, 136, { &P22677,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ThL W3 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2481171 = {S84010, 140, { &P20230,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-LU6500 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2482885 = {S84011, 154, { &P18283,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556799 = {S84012, 154, { &P18283,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2529454 = {S84013, 143, { &P21796,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Philips W336 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2554344 = {S84014, 102, { &P19765,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; SCH-I930)*/
const Device UA2555972 = {S84015, 149, { &P21519,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-ph; Cloudfone Excite 350i Build/FSV02) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2567073 = {S84016, 116, { &P15364,&P17285,&P19963,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17 Sleipnir/3.9.1*/
const Device UA2858119 = {S84017, 123, { &P15364,&P17285,&P19963,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17 Sleipnir/3.9.1*/
const Device UA2578305 = {S84018, 141, { &P21984,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; zh-cn; K-Touch W719 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984421 = {S84019, 65, { &P21984,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; K-Touch W719 Build/IMM76I)*/
const Device UA2597644 = {S84020, 132, { &P21202,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-P500 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2601608 = {S84021, 135, { &P19864,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 80XSK Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2695365 = {S84022, 33, { &P14848,&P5843,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (NokiaX3-00/11.00)*/
const Device UA2706950 = {S84023, 129, { &P18702,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N8013 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2738465 = {S84024, 155, { &P14186,&P5096,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5830 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2767644 = {S84025, 136, { &P23478,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; ZP810 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2901699 = {S84026, 134, { &P21909,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-03E Build/10.1.E.0.269) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2769110 = {S84027, 144, { &P22749,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-04E Build/10.1.1.D.0.179) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2795796 = {S84028, 143, { &P20837,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; IRIS_501 Build/LAVAIRIS501) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2799045 = {S84029, 150, { &P12972,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2799622 = {S84030, 136, { &P18687,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P705 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2870443 = {S84031, 137, { &P18687,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P708g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2859868 = {S84032, 144, { &P23724,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; TurboX-G310   Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878742 = {S84033, 141, { &P24161,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Orange Zali Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2944988 = {S84034, 158, { &P18223,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT22i Build/6.2.A.1.100) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2962572 = {S84035, 158, { &P18223,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT22i Build/6.2.A.0.400) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2944365 = {S84036, 155, { &P20705,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; HTC One Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2946037 = {S84037, 161, { &P14877,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2963495 = {S84038, 161, { &P14877,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3121705 = {S84039, 161, { &P14877,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-CN; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3014445 = {S84040, 86, { &P17595,&P17295,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:22.0) Gecko/X Firefox/22.0 TenFourFox/G5*/
const Device UA2965329 = {S84041, 166, { &P15880,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3022415 = {S84042, 143, { &P21071,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Novo 10 Hero QuadCore Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023551 = {S84043, 144, { &P15809,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Rhyme S510b Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2971555 = {S84044, 132, { &P21261,&P18598,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.2; en-gb; GT-I9500 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2994870 = {S84045, 136, { &P22748,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E425 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3049243 = {S84046, 137, { &P22748,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E425f Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3014974 = {S84047, 152, { &P15364,&P20706,&P24678,&P18092 }};	/*Useragent - Nokia210/2.0 (04.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia210) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3017196 = {S84048, 157, { &P15364,&P17285,&P22777,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.4.2.34167 (Firestorm-Release; firestorm skin) Safari/533.3*/
const Device UA3063309 = {S84049, 167, { &P15201,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Sensation Z710e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3094617 = {S84050, 134, { &P21636,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C525c Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081676 = {S84051, 131, { &P21334,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A8 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3096349 = {S84052, 165, { &P18114,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Transformer Prime TF201 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3108649 = {S84053, 140, { &P15767,&P22882,&P21787,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA3128584 = {S84054, 142, { &P15767,&P22882,&P21787,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA8217 = {S84055, 96, { &P11724,&P5026,&P4948,&P18092 }};	/*Useragent - BlackBerry8310/4.5.0.182 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102 UP.Browser/5.0.3.3*/
const Device UA8614 = {S84056, 76, { &P16862,&P22835,&P17470,&P18092 }};	/*Useragent - Spice S707/2008.07.09 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA10159 = {S84057, 76, { &P16862,&P22835,&P17470,&P18092 }};	/*Useragent - Spice S707/2008.09.26 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA38657 = {S84058, 76, { &P16862,&P22835,&P17470,&P18092 }};	/*Useragent - Spice S707/2008.05.29 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA44298 = {S84059, 76, { &P16862,&P22835,&P17470,&P18092 }};	/*Useragent - Spice S707/2008.05.16 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53918 = {S84060, 76, { &P16862,&P22835,&P17470,&P18092 }};	/*Useragent - Spice S707/2009.03.03 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53919 = {S84061, 76, { &P16862,&P22835,&P17470,&P18092 }};	/*Useragent - Spice S707/2009.05.31 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53920 = {S84062, 76, { &P16862,&P22835,&P17470,&P18092 }};	/*Useragent - Spice S707/2009.04.28 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53921 = {S84063, 76, { &P16862,&P22835,&P17470,&P18092 }};	/*Useragent - Spice S707/2008.09.02 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53922 = {S84064, 76, { &P16862,&P22835,&P17470,&P18092 }};	/*Useragent - Spice S707/2009.04.10 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53923 = {S84065, 76, { &P16862,&P22835,&P17470,&P18092 }};	/*Useragent - Spice S707/2008.08.20 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53924 = {S84066, 76, { &P16862,&P22835,&P17470,&P18092 }};	/*Useragent - Spice S707/2009.07.15 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53925 = {S84067, 76, { &P16862,&P22835,&P17470,&P18092 }};	/*Useragent - Spice S707/2009.06.29 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53926 = {S84068, 76, { &P16862,&P22835,&P17470,&P18092 }};	/*Useragent - Spice S707/2008.09.27 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53927 = {S84069, 76, { &P16862,&P22835,&P17470,&P18092 }};	/*Useragent - Spice S707/2008.12.03 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64563 = {S84070, 76, { &P16862,&P22835,&P17470,&P18092 }};	/*Useragent - Spice S707/2009.03.25 LMP/SH Release Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA57944 = {S84071, 142, { &P15341,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; it-it; PM1107 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA363397 = {S84072, 152, { &P15341,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.1.19; en-us; PMP3084B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA23435 = {S84073, 56, { &P14424,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-JFJKN/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0*/
const Device UA24007 = {S84074, 56, { &P14424,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-JFJK0/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0*/
const Device UA87640 = {S84075, 53, { &P14424,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-W385/1.0 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0*/
const Device UA55127 = {S84076, 101, { &P17204,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-C450/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA88652 = {S84077, 13, { &P16929,&P22784,&P17470,&P18092 }};	/*Useragent - Micromax-X225*/
const Device UA89891 = {S84078, 31, { &P17734,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N04B(c500;TB;W24H16)*/
const Device UA359655 = {S84079, 138, { &P12280,&P17146,&P20779,&P18092 }};	/*Useragent - MQQBrowser/28 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA587057 = {S84080, 136, { &P12280,&P17146,&P20779,&P18092 }};	/*Useragent - MQQBrowser/28 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Safari/7534.48.3*/
const Device UA730080 = {S84081, 138, { &P12280,&P17146,&P20779,&P18092 }};	/*Useragent - MQQBrowser/28 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3*/
const Device UA2150577 = {S84082, 140, { &P12280,&P17146,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2801 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA428866 = {S84083, 101, { &P15364,&P18346,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11*/
const Device UA590346 = {S84084, 68, { &P15364,&P5690,&P18099,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:10.0.2) Gecko/X Firefox/10.0.2 slurp*/
const Device UA1939222 = {S84085, 64, { &P15364,&P5690,&P18099,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:10.0) Gecko/X Firefox/10.0 slurp*/
const Device UA1960740 = {S84086, 68, { &P15364,&P5690,&P18099,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:10.0.1) Gecko/X Firefox/10.0.1 slurp*/
const Device UA1972021 = {S84087, 74, { &P15364,&P5690,&P18099,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:10.0.2) Gecko/X Firefox/10.0.2 slurp slurp*/
const Device UA2496284 = {S84088, 88, { &P17595,&P17295,&P21171,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:12.0) Gecko/X Firefox/12.0 SeaMonkey/2.9*/
const Device UA625580 = {S84089, 199, { &P20066,&P22784,&P18373,&P18092 }};	/*Useragent - MicromaxX55 ASTRO36_TD/v3 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Profile/MIDP-2.0 Configuration/CLDC-1.1 Sync/SyncClient1.1 Opera/9.80 (MTK; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA627178 = {S84090, 133, { &P21308,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; CTP810 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA632262 = {S84091, 89, { &P15364,&P7133,&P22347,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:6.0) Gecko/X Firefox/6.0-x64 PaleMoon/6.0-x64*/
const Device UA800912 = {S84092, 165, { &P15364,&P7133,&P18727,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/1.6.1.1691 (Phoenix Viewer Release Mesh; Firebird skin) Safari/533.3*/
const Device UA836021 = {S84093, 78, { &P15364,&P22740,&P17599,&P18092 }};	/*Useragent - Mozilla/5.0 (Maemo; Linux armv7l; rv:7.0.1) Gecko/X Firefox/7.0.1 Fennec/7.0.1*/
const Device UA3069486 = {S84094, 72, { &P15364,&P22740,&P17599,&P18092 }};	/*Useragent - Mozilla/5.0 (Maemo; Linux armv7l; rv:7.0) Gecko/X Firefox/7.0 Fennec/7.0*/
const Device UA896409 = {S84095, 83, { &P12004,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN95 8GB)/UC Browser8.4.0.159/28/352/UCWEB Mobile*/
const Device UA1994000 = {S84096, 79, { &P12004,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-in; NOKIAN95)/UC Browser8.4.0.159/28/352/UCWEB Mobile*/
const Device UA2261424 = {S84097, 77, { &P12004,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian95) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2375609 = {S84098, 81, { &P12004,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian95_8gb) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA1943694 = {S84099, 32, { &P14983,&P5055,&P24667,&P18092 }};	/*Useragent - NokiaC7-00/UCWEB8.2.0.116/51/800*/
const Device UA2374185 = {S84100, 89, { &P14983,&P5055,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiac7-00) UCBrowser8.2.1.144/69/444/UCWEB Mobile  3gpp-gba*/
const Device UA1961920 = {S84101, 137, { &P24738,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; Fly_IQ270 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1996328 = {S84102, 69, { &P15364,&P16110,&P18515,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:14.0) Gecko/X Thunderbird/14.0*/
const Device UA2131292 = {S84103, 78, { &P15364,&P22471,&P17470,&P18092 }};	/*Useragent - curl/7.16.3 (powerpc-apple-darwin8.0) libcurl/7.16.3 OpenSSL/0.9.7l zlib/1.2.3*/
const Device UA2462919 = {S84104, 78, { &P15364,&P22471,&P17470,&P18092 }};	/*Useragent - curl/7.13.1 (powerpc-apple-darwin8.0) libcurl/7.13.1 OpenSSL/0.9.7l zlib/1.2.3*/
const Device UA2189034 = {S84105, 189, { &P15364,&P7133,&P18520,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.5.0.185/84/352*/
const Device UA2226190 = {S84106, 189, { &P15364,&P7133,&P18520,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.5.0.185/82/352*/
const Device UA2346868 = {S84107, 189, { &P15364,&P7133,&P18520,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.5.0.185/82/405*/
const Device UA2377660 = {S84108, 189, { &P15364,&P7133,&P18520,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.5.0.185/83/352*/
const Device UA2828507 = {S84109, 199, { &P15364,&P7133,&P18520,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.5.0.185/82/352  3gpp-gba*/
const Device UA2242591 = {S84110, 97, { &P18563,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Motorola_MC659B_312*/
const Device UA2338306 = {S84111, 99, { &P18563,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Motorola_MC659B_40211*/
const Device UA2261528 = {S84112, 135, { &P21125,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; L-06D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2420363 = {S84113, 139, { &P21125,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; L-06DJOJO Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2402189 = {S84114, 135, { &P17717,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-SU760 Build/ICS) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2466986 = {S84115, 135, { &P17717,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-SU760 Build/ICS) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2404280 = {S84116, 126, { &P12351,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s3653w) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2462061 = {S84117, 171, { &P20267,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; P300(Funbook) Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2434317 = {S84118, 134, { &P15767,&P18432,&P22326,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Rockmelt/2.1.0.1310 Mobile/10A525 Safari/8536.25*/
const Device UA2435899 = {S84119, 136, { &P12280,&P18432,&P22326,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Rockmelt/2.1.0.1310 Mobile/10A525 Safari/8536.25*/
const Device UA2452351 = {S84120, 136, { &P21446,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SH837W Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2509404 = {S84121, 141, { &P12280,&P19826,&P20790,&P18092 }};	/*Useragent - MQQBrowser/391 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B142 Safari/7534.48.3*/
const Device UA2534853 = {S84122, 140, { &P12280,&P19826,&P20790,&P18092 }};	/*Useragent - MQQBrowser/391 Mozilla/5.0 (iPhone 5; CPU iPhone OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 Safari/7534.48.3*/
const Device UA2860785 = {S84123, 143, { &P12280,&P19826,&P20790,&P18092 }};	/*Useragent - MQQBrowser/391 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3*/
const Device UA2556390 = {S84124, 155, { &P23837,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Essentielb-Black Diamond Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2559399 = {S84125, 32, { &P14983,&P5055,&P24675,&P18092 }};	/*Useragent - NokiaC7-00/UCWEB8.8.0.212/51/800*/
const Device UA2763602 = {S84126, 102, { &P14983,&P5055,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac7-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2559993 = {S84127, 93, { &P16871,&P22784,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq5) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2601530 = {S84128, 139, { &P23156,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOLO A700 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2622470 = {S84129, 92, { &P17337,&P17979,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.0 Android (15/4.0.4; 160dpi; 800x1232; samsung; GT-P7500; GT-P7500; p3; en_US)*/
const Device UA2637438 = {S84130, 86, { &P17337,&P17979,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.4 Android (17/4.2.2; 160dpi; 800x1232; samsung; GT-P7500; p4; p3; en_CA)*/
const Device UA2637386 = {S84131, 151, { &P15882,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2637663 = {S84132, 151, { &P15882,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2995556 = {S84133, 155, { &P15882,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA2659129 = {S84134, 117, { &P23304,&P17979,&P18062,&P18092 }};	/*Useragent - OPPO_R813T/1.0 Linux/3.0.35.7 Android/4.0 Release/11.15.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2*/
const Device UA2659653 = {S84135, 93, { &P18507,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I8160) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2662143 = {S84136, 96, { &P14839,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA6720c-1b) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2698954 = {S84137, 93, { &P14839,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA6720c) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2668170 = {S84138, 147, { &P22686,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Primo_F1 Build/WALTONPrimo_F1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2709591 = {S84139, 152, { &P12765,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2769968 = {S84140, 156, { &P20096,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-S7568 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2877902 = {S84141, 160, { &P23008,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax A91 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752459 = {S84142, 144, { &P22812,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CELKON CT 888 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752486 = {S84143, 164, { &P13337,&P17485,&P24677,&P18092 }};	/*Useragent - SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj108i) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2767433 = {S84144, 153, { &P24073,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033M Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3019753 = {S84145, 164, { &P18113,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT912 Build/JDQ39E; Carbon-CARBON-JB-NIGHTLY-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892613 = {S84146, 144, { &P23708,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ultrafone_501 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2898798 = {S84147, 138, { &P18672,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2961184 = {S84148, 133, { &P24068,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; N986 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3028231 = {S84149, 197, { &P22584,&P18598,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-ca; SGH-I337M Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, ca*/
const Device UA3017823 = {S84150, 151, { &P18606,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3059605 = {S84151, 158, { &P18606,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3027505 = {S84152, 152, { &P16598,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A500 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3068882 = {S84153, 134, { &P12280,&P24384,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D44 Safari/9537.53*/
const Device UA3068884 = {S84154, 134, { &P12280,&P24384,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D46 Safari/9537.53*/
const Device UA3068885 = {S84155, 134, { &P12280,&P24384,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D47 Safari/9537.53*/
const Device UA3068886 = {S84156, 134, { &P12280,&P24384,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D48 Safari/9537.53*/
const Device UA3068887 = {S84157, 134, { &P12280,&P24384,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D49 Safari/9537.53*/
const Device UA3068888 = {S84158, 134, { &P12280,&P24384,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D50 Safari/9537.53*/
const Device UA3068890 = {S84159, 134, { &P12280,&P24384,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D52 Safari/9537.53*/
const Device UA3068889 = {S84160, 134, { &P12280,&P24384,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D51 Safari/9537.53*/
const Device UA3095037 = {S84161, 102, { &P18673,&P18110,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-S7562) U2/1.0.0 UCBrowser/9.3.0.321 Mobile*/
const Device UA3131433 = {S84162, 140, { &P16362,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710BST Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132029 = {S84163, 137, { &P16362,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA94200 = {S84164, 97, { &P16083,&P5448,&P22270,&P18092 }};	/*Useragent - MOT-Q11/01.00.49R-01 Software/WM6.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA19435 = {S84165, 96, { &P14180,&P5448,&P22270,&P18092 }};	/*Useragent - Touch_HD_T8285/V1.59.841.6 (48753) Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA58122 = {S84166, 76, { &P14180,&P5448,&P22270,&P18092 }};	/*Useragent - Touch_HD_T8285 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA2463213 = {S84167, 96, { &P14180,&P5448,&P22270,&P18092 }};	/*Useragent - Touch_HD_T8285/V1.59.841.5 (48753) Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA27378 = {S84168, 67, { &P12173,&P17483,&P4223,&P18092 }};	/*Useragent - LG-CT810/V10h NetFront/v3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA47624 = {S84169, 135, { &P17337,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-gb; GT-P7500 Build/HONEYCOMB) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA60515 = {S84170, 131, { &P17337,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; GT-P7500 Build/HRI83) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA55270 = {S84171, 47, { &P14339,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-VX8560/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA61842 = {S84172, 47, { &P14339,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-VX8550/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA89915 = {S84173, 31, { &P17733,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N06B(c100;TB;W24H16)*/
const Device UA234861 = {S84174, 54, { &P17505,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-CA3C UP.Browser/6.2_7.2.7.1.K.2.234 (GUI) MMP/2.0*/
const Device UA865653 = {S84175, 154, { &P14574,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus S Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA622911 = {S84176, 39, { &P15364,&P17017,&P23784,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; ipMonitor 9.0)*/
const Device UA867839 = {S84177, 53, { &P14188,&P5890,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8 (Nokia5233;SymbianOS/9.1 Series60/3.0)*/
const Device UA882346 = {S84178, 188, { &P15364,&P7133,&P18516,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/69/352*/
const Device UA893538 = {S84179, 188, { &P15364,&P7133,&P18516,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/70/352*/
const Device UA899383 = {S84180, 188, { &P15364,&P7133,&P18516,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/69/444*/
const Device UA1982655 = {S84181, 188, { &P15364,&P7133,&P18516,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/69/405*/
const Device UA2335159 = {S84182, 198, { &P15364,&P7133,&P18516,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/69/352  3gpp-gba*/
const Device UA2372707 = {S84183, 188, { &P15364,&P7133,&P18516,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.1.161/70/405*/
const Device UA2743536 = {S84184, 188, { &P15364,&P7133,&P18516,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.1.161/70/352*/
const Device UA2758997 = {S84185, 198, { &P15364,&P7133,&P18516,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/69/444  3gpp-gba*/
const Device UA901788 = {S84186, 89, { &P17595,&P17297,&P21170,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:13.0) Gecko/X Firefox/13.0 SeaMonkey/2.10*/
const Device UA924951 = {S84187, 91, { &P17595,&P17295,&P22316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:14.0) Gecko/X Firefox/14.0.1 SeaMonkey/2.11*/
const Device UA2139370 = {S84188, 47, { &P15364,&P5690,&P18403,&P18092 }};	/*Useragent - Opera/9.30 (Windows NT 5.1; U; pl) Version/10.9*/
const Device UA2409582 = {S84189, 62, { &P15364,&P5690,&P18403,&P18092 }};	/*Useragent - Opera/9.30 (Windows NT 5.1; U; pl) Presto/2.3.15 Version/10.10*/
const Device UA2357985 = {S84190, 32, { &P15364,&P17017,&P21914,&P18092 }};	/*Useragent - contype chromeframe/23.0.1271.64*/
const Device UA2405900 = {S84191, 32, { &P15364,&P17017,&P21914,&P18092 }};	/*Useragent - contype chromeframe/23.0.1271.95*/
const Device UA2433621 = {S84192, 32, { &P15364,&P17017,&P21914,&P18092 }};	/*Useragent - contype chromeframe/23.0.1271.97*/
const Device UA2368283 = {S84193, 117, { &P15364,&P7480,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11 iTunes/10.7 Downcast/2.7.13.1002*/
const Device UA2503627 = {S84194, 157, { &P15767,&P17090,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3 TeamStream HD (iPad2,1 284 1.7)*/
const Device UA2654325 = {S84195, 157, { &P15767,&P17090,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3 TeamStream HD (iPad2,1 284 1.7)*/
const Device UA2528709 = {S84196, 119, { &P14782,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE61i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile*/
const Device UA2878197 = {S84197, 118, { &P14782,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE61) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile*/
const Device UA3043929 = {S84198, 93, { &P14782,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiae61i-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA3070881 = {S84199, 41, { &P14782,&P5890,&P24673,&P18092 }};	/*Useragent - NOKIAE61iUCBrowser/8.7.1.234/28/352/UCWEB*/
const Device UA2466243 = {S84200, 151, { &P21294,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MODECOM FreeTAB 9702 IPS X2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2487855 = {S84201, 156, { &P18680,&P18066,&P19838,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2501326 = {S84202, 137, { &P21446,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; SH837W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2503298 = {S84203, 148, { &P14908,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P6210 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2613429 = {S84204, 148, { &P14908,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6210 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615651 = {S84205, 148, { &P14908,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6210 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2505078 = {S84206, 69, { &P21356,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; TECNO_N3; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2530452 = {S84207, 171, { &P16356,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2537446 = {S84208, 90, { &P15397,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; ST18i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2552315 = {S84209, 110, { &P22109,&P5629,&P15760,&P18092 }};	/*Useragent - HS-T930_TD/1.0 Android/2.3 Release/07.07.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2583151 = {S84210, 141, { &P20540,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PAP4300 DUO Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2581218 = {S84211, 153, { &P20453,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NABI2-NV7A Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2603904 = {S84212, 140, { &P15147,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST15i Build/4.1.B.0.587) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583187 = {S84213, 138, { &P21506,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2596018 = {S84214, 114, { &P20255,&P17979,&P18062,&P18092 }};	/*Useragent - OPPO_R811/1.0 Linux/3.0.13 Android/4.0 Release/10.15.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2*/
const Device UA2603372 = {S84215, 94, { &P18296,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_One_V) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2635848 = {S84216, 137, { &P13452,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Mobile Safari/537.31*/
const Device UA2676298 = {S84217, 130, { &P13452,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2659482 = {S84218, 88, { &P18482,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; A75) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2849843 = {S84219, 63, { &P15364,&P17285,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; Windows NT 6.0; Trident/5.0)*/
const Device UA2674742 = {S84220, 134, { &P20480,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; JY-G3 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675277 = {S84221, 125, { &P14111,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Arc Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2706947 = {S84222, 136, { &P17595,&P17295,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17; Maxthon (%s);*/
const Device UA2708676 = {S84223, 133, { &P23319,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TM-MID730 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2727540 = {S84224, 103, { &P12054,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.380; en-us; 9500/4.2.0.380) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2835503 = {S84225, 103, { &P12054,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.0.0.174; en-us; 9500/4.0.0.174) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2982704 = {S84226, 103, { &P12054,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.198; en-us; 9500/4.2.0.198) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2984135 = {S84227, 103, { &P12054,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.2.0.436; en-us; 9500/4.2.0.436) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2734985 = {S84228, 96, { &P21881,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Karbonn_A4_) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2735948 = {S84229, 155, { &P18702,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N8013 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA2766907 = {S84230, 151, { &P18702,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N8013 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2798884 = {S84231, 159, { &P17625,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST25i Build/6.1.1.B.1.10) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752325 = {S84232, 93, { &P14782,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE61i) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2781577 = {S84233, 92, { &P14782,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE61) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2855894 = {S84234, 94, { &P14782,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae61i-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2784842 = {S84235, 153, { &P19907,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C5155 Build/IML77) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2755421 = {S84236, 133, { &P21048,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; W732 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2757297 = {S84237, 154, { &P22785,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Desire 600 dual sim Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2757298 = {S84238, 76, { &P22785,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; HTC Desire 600 dual sim Build/JZO54K)*/
const Device UA2769696 = {S84239, 139, { &P22636,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Aquaris Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773666 = {S84240, 138, { &P19848,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M440S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2781796 = {S84241, 158, { &P15991,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2782733 = {S84242, 153, { &P20018,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993D Build/ICECREAM) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867411 = {S84243, 152, { &P20018,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993 Build/ICECREAM) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783697 = {S84244, 146, { &P20090,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848i Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799265 = {S84245, 145, { &P20090,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848 Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2865906 = {S84246, 137, { &P23272,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-ch; Lenco TAB-900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2879029 = {S84247, 162, { &P24078,&P18598,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; E3 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33931*/
const Device UA2892345 = {S84248, 152, { &P23711,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; iBall Andi4.5d quadro Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2939888 = {S84249, 138, { &P18304,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2944441 = {S84250, 159, { &P12616,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; WT19i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056807 = {S84251, 159, { &P12616,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; WT19i Build/4.0.2.A.0.58) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2954925 = {S84252, 151, { &P18437,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; GT-N8000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2994346 = {S84253, 155, { &P18437,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N8000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA3022018 = {S84254, 140, { &P21181,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S560 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022792 = {S84255, 134, { &P21815,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; P-02E Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3062534 = {S84256, 145, { &P19867,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I437 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3095254 = {S84257, 146, { &P24545,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Starmobile KNIGHT Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA9015 = {S84258, 43, { &P13385,&P17483,&P3625,&P18092 }};	/*Useragent - LG8700/1.1 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA86972 = {S84259, 43, { &P13385,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-LG8700 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA87427 = {S84260, 43, { &P13385,&P17483,&P3625,&P18092 }};	/*Useragent - LG8700/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA13088 = {S84261, 150, { &P17670,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; Videocon_V7400 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA62761 = {S84262, 92, { &P11861,&P5776,&P3756,&P18092 }};	/*Useragent - Nokia6260/2.0 (3.0448.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA32786 = {S84263, 142, { &P12769,&P5096,&P3508,&P18092 }};	/*Useragent - MOT-MT620_TD/1.0 OPhone/2.5 (Linux; Android 2.2) Release/03.31.2011 Browser/WAP2.0 (AppleWebKit/533.1) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA61504 = {S84264, 63, { &P12732,&P17017,&P3807,&P18092 }};	/*Useragent - Huawei/1.0/HUAWEI_G6005/Obigo Browser/Q03C Browser/Q03C MMS/1.2*/
const Device UA68519 = {S84265, 85, { &P13999,&P5448,&P4777,&P18092 }};	/*Useragent - HTC_HD2_T8585 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA557809 = {S84266, 81, { &P13999,&P5448,&P4777,&P18092 }};	/*Useragent - HD2_T8585 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2758543 = {S84267, 81, { &P13999,&P5448,&P4777,&P18092 }};	/*Useragent - HTC_HD2_T8585/480x800 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA670941 = {S84268, 46, { &P15767,&P17146,&P17470,&P18092 }};	/*Useragent - Facebook 4110.0 (iPad; iPhone OS 5.0.1; en_US)*/
const Device UA2309356 = {S84269, 44, { &P15767,&P17146,&P17470,&P18092 }};	/*Useragent - Facebook 4100.0 (iPad; iPhone OS 5.0; en_US)*/
const Device UA160174 = {S84270, 63, { &P18254,&P7480,&P17470,&P18189 }};	/*Useragent - FeedFetcher-Google-CoOp; (+http://www.google.com/coop/cse/cref)*/
const Device UA784715 = {S84271, 49, { &P18254,&P18575,&P18245,&P18092 }};	/*Useragent - StumbleUpon/2.60 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA2545745 = {S84272, 42, { &P18254,&P18575,&P18245,&P18092 }};	/*Useragent - Bing/3.03 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA638773 = {S84273, 49, { &P11684,&P5129,&P17338,&P18092 }};	/*Useragent - nokia5800d-1/UC Browser7.9.0.102/69/352  3gpp-gba*/
const Device UA713553 = {S84274, 112, { &P11787,&P5204,&P17347,&P18092 }};	/*Useragent - Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia3120classic/UC Browser8.0.3.107/69/444*/
const Device UA659110 = {S84275, 134, { &P15539,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Glacier; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA811277 = {S84276, 89, { &P15364,&P7133,&P21955,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1 x64; en-US; rv:1.9.0.10) Gecko/X Minefield/3.0.10*/
const Device UA2472131 = {S84277, 87, { &P15364,&P7133,&P21955,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9a4pre) Gecko/X Minefield/3.0a4pre*/
const Device UA837925 = {S84278, 145, { &P12258,&P17483,&P24667,&P18092 }};	/*Useragent - LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-gs290) UCBrowser8.2.1.144/69/352/UCWEB Mobile*/
const Device UA1951400 = {S84279, 146, { &P21428,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Woxter Tablet PC 85CXi Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2030197 = {S84280, 139, { &P24020,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; vi-vn; HKPhone 4S-3G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2041165 = {S84281, 40, { &P17968,&P17979,&P24438,&P18092 }};	/*Useragent - Dolphin http client/8.8.1(171) (Android)*/
const Device UA2043153 = {S84282, 40, { &P17968,&P17979,&P24438,&P18092 }};	/*Useragent - Dolphin http client/8.8.0(168) (Android)*/
const Device UA2205470 = {S84283, 40, { &P17968,&P17979,&P24438,&P18092 }};	/*Useragent - Dolphin http client/8.8.2(177) (Android)*/
const Device UA2231034 = {S84284, 44, { &P15364,&P18580,&P22322,&P18092 }};	/*Useragent - Rockmelt/2.0.1.4 CFNetwork/609 Darwin/13.0.0*/
const Device UA2233791 = {S84285, 95, { &P15364,&P18110,&P20912,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) rekonq/1.2 Safari/534.34*/
const Device UA2350867 = {S84286, 167, { &P12968,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT26i Build/6.1.A.2.45) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2352037 = {S84287, 119, { &P14753,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; Nokia N96) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile*/
const Device UA2354055 = {S84288, 166, { &P18510,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; ALCATEL ONE TOUCH 918N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2394106 = {S84289, 65, { &P15779,&P17483,&P17602,&P18092 }};	/*Useragent - LG-S367/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2427790 = {S84290, 132, { &P20527,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A6 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2433589 = {S84291, 156, { &P18680,&P18066,&P19816,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2456137 = {S84292, 132, { &P21038,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; U900 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2457711 = {S84293, 122, { &P13800,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA5630d-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2458048 = {S84294, 150, { &P20137,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Xtreamer_Mobile_AiKi Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2468615 = {S84295, 138, { &P14872,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SPH-D700 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2663719 = {S84296, 137, { &P14872,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-D700 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2532627 = {S84297, 144, { &P21378,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; tablet Fnac 8 Build/1.0.4 X-11:52) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2528697 = {S84298, 135, { &P21815,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; P-02E Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2555814 = {S84299, 134, { &P23866,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; F10 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2675221 = {S84300, 158, { &P18509,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-MS770 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2579966 = {S84301, 151, { &P23758,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9430 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580318 = {S84302, 89, { &P11950,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; X10i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2602993 = {S84303, 145, { &P21333,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; INO_ONE Build/INO_ONEINO_ONE) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2625125 = {S84304, 191, { &P15985,&P6437,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0800, id*/
const Device UA2626331 = {S84305, 190, { &P15985,&P6437,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0800, my*/
const Device UA2667008 = {S84306, 133, { &P20453,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NABI2-NV7A Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22*/
const Device UA2673242 = {S84307, 137, { &P20264,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; bq Curie Build/1.1.0 X-14:50) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2674227 = {S84308, 133, { &P20868,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M040 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709389 = {S84309, 163, { &P21068,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI-ONE C1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2709868 = {S84310, 166, { &P13644,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; EVO Build/JRO03C; Carbon-CARBON-JB-EXP-10APR2013) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121631 = {S84311, 169, { &P13644,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; PC36100 Build/JDQ39E; Carbon-CARBON-JB-UNOFFICIAL-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2713897 = {S84312, 133, { &P23265,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; da-dk; TAB-80012 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2736271 = {S84313, 148, { &P23103,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Starmobile DIAMOND Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2737296 = {S84314, 158, { &P18449,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST27i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2737622 = {S84315, 161, { &P13452,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2737819 = {S84316, 93, { &P15649,&P5096,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5570) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2754353 = {S84317, 137, { &P20264,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; bq Curie Build/1.1.0 X-14:50) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2755275 = {S84318, 137, { &P20508,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cynus T2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769698 = {S84319, 138, { &P18386,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I757M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769875 = {S84320, 130, { &P13452,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2773002 = {S84321, 138, { &P15850,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LU6200 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2870073 = {S84322, 151, { &P20081,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7108 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2958414 = {S84323, 151, { &P20081,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7108 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2998921 = {S84324, 159, { &P16356,&P18111,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300*/
const Device UA2952116 = {S84325, 133, { &P21613,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Aurora-II Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2987663 = {S84326, 128, { &P21613,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Aurora-II Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3018417 = {S84327, 139, { &P15364,&P17819,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.11 (KHTML, like Gecko) Ubuntu/11.10 Chromium/27.0.1453.93 Chrome/27.0.1453.93 Safari/537.36*/
const Device UA2958474 = {S84328, 60, { &P20744,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia301/2.0 (02.33) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2967831 = {S84329, 141, { &P12280,&P17146,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 UCBrowser/9.1.0.287 Mobile*/
const Device UA3019705 = {S84330, 141, { &P12280,&P17146,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 UCBrowser/9.1.0.287 Mobile*/
const Device UA3022279 = {S84331, 130, { &P19874,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-i705 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021322 = {S84332, 137, { &P18636,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS930 4G Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3029967 = {S84333, 95, { &P15501,&P5477,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E16i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3021136 = {S84334, 134, { &P23150,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Q1000 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021038 = {S84335, 146, { &P23198,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI H868C Build/HuaweiH868C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021552 = {S84336, 136, { &P18472,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Razr HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021915 = {S84337, 134, { &P18472,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT925 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3063696 = {S84338, 150, { &P24078,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; E3 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3094461 = {S84339, 138, { &P24108,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; W8 beyond Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083486 = {S84340, 143, { &P21802,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GSmart Maya M1 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094504 = {S84341, 139, { &P24132,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; iris504Q Build/iris504Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094308 = {S84342, 126, { &P18008,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ST10 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA8550 = {S84343, 168, { &P14872,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android noobnl Andromeda Galaxy ROM v1.0.3; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA8725 = {S84344, 158, { &P14872,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Epic_Experience_v1.2.07a; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA9982 = {S84345, 149, { &P14872,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Baked-Snack-1.2; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10402 = {S84346, 158, { &P14872,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android SyndicateROM_1.0.1_(Sai); en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10403 = {S84347, 157, { &P14872,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Epic_Experience_v1.2.03; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10405 = {S84348, 157, { &P14872,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Epic_Experience_v1.2.07; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10406 = {S84349, 157, { &P14872,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Epic_Experience_v1.2.05; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10407 = {S84350, 157, { &P14872,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Epic_Experience_v1.2.04; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10409 = {S84351, 157, { &P14872,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Epic_Experience_v1.2.08; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA38803 = {S84352, 168, { &P14872,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android noobnl Andromeda Galaxy ROM v1.1.1; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA44040 = {S84353, 169, { &P14872,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android noobnl_Andromeda_Galaxy_ROM_v1.1.1a; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA44216 = {S84354, 163, { &P14872,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android SyndicateROM_1.0.2_(Shuriken); en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA44215 = {S84355, 157, { &P14872,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Epic_Experience_v1.2.09; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA52520 = {S84356, 152, { &P14872,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Heroic.Epic_v1.0.3; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA58663 = {S84357, 154, { &P14872,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Heroic.Epic_v1.0.4.1; en-us; SPH-D700 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10073 = {S84358, 86, { &P13684,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Toshiba/TG01/02; Windows Phone 6.5)*/
const Device UA10076 = {S84359, 94, { &P13684,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Toshiba-TG01-orange/ 02; Windows Phone 6.5)*/
const Device UA10074 = {S84360, 83, { &P13684,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Toshiba_TG01; Windows Phone 6.5)*/
const Device UA11421 = {S84361, 119, { &P14085,&P17979,&P3508,&P18092 }};	/*Useragent - LG_P503/1.0 Android/2.2 Release/5.20.2010 Browser/KHTML (Mozilla/5.0 AppleWebKit/533.1 Version/4.0 Mobile Safari/533.1)*/
const Device UA54134 = {S84362, 45, { &P14127,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KF350_CMCC Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA64177 = {S84363, 40, { &P14127,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KF350 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA68352 = {S84364, 92, { &P14168,&P5448,&P4777,&P18092 }};	/*Useragent - HTC_Touch_Viva_T2223 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA155394 = {S84365, 50, { &P15364,&P17017,&P3692,&P18092 }};	/*Useragent - WinWAP/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA251465 = {S84366, 132, { &P17595,&P17297,&P22451,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.0 Chrome/13.0.800.0 Safari/535.1*/
const Device UA718253 = {S84367, 132, { &P17595,&P17297,&P22451,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.0 Chrome/13.0.800.0 Safari/535.1*/
const Device UA839772 = {S84368, 132, { &P17595,&P17297,&P22451,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.0 Chrome/13.0.800.0 Safari/535.1*/
const Device UA2383477 = {S84369, 132, { &P17595,&P17297,&P22451,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.0 Chrome/13.0.800.0 Safari/535.1*/
const Device UA252704 = {S84370, 138, { &P17595,&P17295,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.68.1565 Chrome/14.0.835.202 Safari/535.1*/
const Device UA670142 = {S84371, 147, { &P17595,&P17295,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.13 (KHTML, like Gecko) RockMelt/0.9.48.59 Chrome/9.0.597.107 Safari/534.13*/
const Device UA670602 = {S84372, 138, { &P17595,&P17295,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.24 (KHTML, like Gecko) RockMelt/0.9.56.357 Chrome/11.0.696.71 Safari/534.24*/
const Device UA915865 = {S84373, 139, { &P17595,&P17295,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.30 (KHTML, like Gecko) RockMelt/0.9.62.313 Chrome/12.0.742.122 Safari/534.30*/
const Device UA1897500 = {S84374, 137, { &P17595,&P17295,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.64.347 Chrome/13.0.782.218 Safari/535.1*/
const Device UA680891 = {S84375, 118, { &P15364,&P17188,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 4.0; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13*/
const Device UA379757 = {S84376, 45, { &P12004,&P5890,&P24663,&P18092 }};	/*Useragent - NOKIAN95 8GB/UC Browser7.9.0.102/28/355/UCWEB*/
const Device UA397226 = {S84377, 41, { &P12004,&P5890,&P24663,&P18092 }};	/*Useragent - NOKIAN95/UC Browser7.9.0.102/28/355/UCWEB*/
const Device UA781151 = {S84378, 62, { &P15364,&P17017,&P4809,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Mac_PowerPC) Opera 5.0 [en]*/
const Device UA540926 = {S84379, 137, { &P21521,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ko-kr; SHW-M290S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA568122 = {S84380, 136, { &P21204,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; myPad P2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA640216 = {S84381, 83, { &P15364,&P7133,&P18196,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:5.0) Gecko/X Thunderbird/5.0 Lightning/1.0b4*/
const Device UA652395 = {S84382, 55, { &P12697,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player sprint / 1.0 / htc_shooter / 2.3.4*/
const Device UA2395317 = {S84383, 55, { &P12697,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player Sprint / 1.0 / htc_shooter / 4.0.3*/
const Device UA2519790 = {S84384, 62, { &P12697,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player virgin_mobile / 1.0 / htc_shooter / 4.0.3*/
const Device UA2863144 = {S84385, 52, { &P12697,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc / 1.0 / htc_shooter / 4.0.3*/
const Device UA659462 = {S84386, 87, { &P17595,&P17297,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Macintosh;Mac OS X (10.7.2);en;kml:2.2;client:Free;type:default)*/
const Device UA712317 = {S84387, 87, { &P17595,&P17297,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Macintosh;Mac OS X (10.7.3);en;kml:2.2;client:Free;type:default)*/
const Device UA792107 = {S84388, 87, { &P17595,&P17297,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Macintosh;Mac OS X (10.7.4);en;kml:2.2;client:Free;type:default)*/
const Device UA2383319 = {S84389, 87, { &P17595,&P17297,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Macintosh;Mac OS X (10.7.5);en;kml:2.2;client:Free;type:default)*/
const Device UA692208 = {S84390, 129, { &P16036,&P18110,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA777052 = {S84391, 74, { &P20391,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-Z331/1.5.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA825518 = {S84392, 67, { &P13018,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.4;Zh_cn;MT11i;480*854;)UCWEB7.8.0.95/139/444*/
const Device UA1856047 = {S84393, 97, { &P15364,&P7127,&P22479,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.8.1.16) Gecko/X Firefox/2.0.0.16 Flock/1.2.4*/
const Device UA2330488 = {S84394, 150, { &P13308,&P17485,&P24668,&P18092 }};	/*Useragent - SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-in; sonyericssonj105i) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA902735 = {S84395, 138, { &P16757,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I500 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2025119 = {S84396, 143, { &P16757,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SCH-I500 Build/GINGERBREAD) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1903373 = {S84397, 115, { &P15364,&P22283,&P4223,&P18092 }};	/*Useragent - MOT-EX132/1.0 REXL4/1.0 BrewMP/1.0.3 Release/6.15.2011 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA1973758 = {S84398, 82, { &P12784,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-T3 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1 firmware/T3 T04*/
const Device UA2851335 = {S84399, 82, { &P12784,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-T3 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1 firmware/T3 M01*/
const Device UA1978823 = {S84400, 56, { &P15364,&P17017,&P18565,&P18092 }};	/*Useragent - Mozilla/5.0 (X11) KHTML/4.8.4 (like Gecko) Konqueror/4.8*/
const Device UA2031981 = {S84401, 132, { &P22092,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; RMD-1030 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2223877 = {S84402, 132, { &P22092,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; RMD-1030 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2333345 = {S84403, 97, { &P15364,&P7133,&P18106,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/555.5 (KHTML, like Gecko) Chrome/99.0.999.0 Safari/555.5*/
const Device UA2334722 = {S84404, 97, { &P15364,&P7133,&P18106,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/99.9.999 Safari/534.24*/
const Device UA2728110 = {S84405, 113, { &P15364,&P7133,&P18106,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/99.0 Safari/537.1 YottaaMonitor*/
const Device UA2820976 = {S84406, 125, { &P15364,&P7133,&P18106,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; de) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/NA Chrome anonymized by Abelssoft X*/
const Device UA3020918 = {S84407, 114, { &P15364,&P7133,&P18106,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/99.0 Safari/537.11 YottaaMonitor*/
const Device UA2360554 = {S84408, 75, { &P17595,&P17295,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:18.0) Gecko/18.0 Firefox/18.0*/
const Device UA2379774 = {S84409, 165, { &P12784,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Skate Build/IML74K; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2413151 = {S84410, 58, { &P20236,&P17483,&P3459,&P18092 }};	/*Useragent - LG-A341 Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2878125 = {S84411, 65, { &P20236,&P17483,&P3459,&P18092 }};	/*Useragent - LG-A341-parrot Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2485002 = {S84412, 170, { &P13815,&P5096,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2426331 = {S84413, 154, { &P17346,&P18432,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3 TeamStream (iPod4,1 293 2.2.2)*/
const Device UA2845768 = {S84414, 152, { &P17346,&P18432,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3 TeamStream (iPod4,1 293 2.2.2)*/
const Device UA2469202 = {S84415, 166, { &P18679,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-B5330 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2459448 = {S84416, 139, { &P23156,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOLO A700 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2480177 = {S84417, 136, { &P21523,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SH930W Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2480393 = {S84418, 138, { &P15364,&P5629,&P3807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; OBIGO_Q03C Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2499315 = {S84419, 37, { &P23018,&P17486,&P3807,&P18092 }};	/*Useragent - MOT-EX430/03.03.10 Obigo/Q03C MMP/2.0*/
const Device UA2522328 = {S84420, 79, { &P22703,&P5629,&P15760,&P18092 }};	/*Useragent - SAMSUNG-GT-S6108_TD/1.0 Android/2.3.6 Release/1.5.2012 Browser/AppleWebKit533.1*/
const Device UA2817568 = {S84421, 135, { &P22703,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6108 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2527787 = {S84422, 70, { &P18673,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.4;Zh_cn;GT-S7562;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA2555154 = {S84423, 160, { &P23974,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hu-hu; Telenor_One_Touch_S Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2584855 = {S84424, 140, { &P18226,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; U9500 Build/HuaweiU9500) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2650549 = {S84425, 139, { &P21129,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; SCH-i929 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2663009 = {S84426, 139, { &P23834,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; SmartPro Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2674439 = {S84427, 145, { &P19867,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I437 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676088 = {S84428, 134, { &P20414,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SmartTabII10 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676880 = {S84429, 132, { &P21603,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AMID-9743G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2700540 = {S84430, 138, { &P21490,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A860L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2703062 = {S84431, 138, { &P21481,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A860S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2706787 = {S84432, 131, { &P22476,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V70 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2734648 = {S84433, 153, { &P22807,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; WALTON Primo C1 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2752527 = {S84434, 87, { &P17595,&P18207,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1580(Macintosh;Mac OS X (10.8.3);en;kml:2.2;client:Free;type:default)*/
const Device UA2961927 = {S84435, 87, { &P17595,&P18207,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1580(Macintosh;Mac OS X (10.8.4);en;kml:2.2;client:Free;type:default)*/
const Device UA3057594 = {S84436, 87, { &P17595,&P18207,&P22779,&P18092 }};	/*Useragent - GoogleEarth/7.1.1.1888(Macintosh;Mac OS X (10.8.4);en;kml:2.2;client:Free;type:default)*/
const Device UA2769901 = {S84437, 138, { &P19902,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530M Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774158 = {S84438, 159, { &P18225,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; U9200 Build/HuaweiU9200) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2777803 = {S84439, 166, { &P18673,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2781172 = {S84440, 159, { &P23420,&P18111,&P21729,&P18092 }};	/*Useragent - OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Symphony_W125 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2784838 = {S84441, 150, { &P18437,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8000 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2801259 = {S84442, 144, { &P22971,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire 200 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2863872 = {S84443, 155, { &P23717,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; O+ 8.6 Android Build/Oplus8.6_Android) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2872752 = {S84444, 150, { &P22874,&P18598,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9502 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2890863 = {S84445, 136, { &P20821,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB192 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2989316 = {S84446, 137, { &P20821,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB0192 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2940670 = {S84447, 138, { &P21020,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-in; Micromax A116 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2943391 = {S84448, 156, { &P22607,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S5303 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2963222 = {S84449, 97, { &P13815,&P5096,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; HTC_Wildfire) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3027013 = {S84450, 97, { &P13815,&P5096,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; HTC_Wildfire) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA2963488 = {S84451, 62, { &P15364,&P18144,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64; rv:22.0) Gecko/X Firefox/22.0*/
const Device UA2974443 = {S84452, 137, { &P20264,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; bq Curie Build/1.1.0 X-14:50) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021456 = {S84453, 135, { &P23546,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AD683G Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022270 = {S84454, 126, { &P19775,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2994233 = {S84455, 61, { &P17324,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I9001 Build/JDQ39E)*/
const Device UA3087947 = {S84456, 138, { &P17324,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; de-de; GT-I9001 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3002424 = {S84457, 136, { &P21432,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; N7889 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3017456 = {S84458, 151, { &P24396,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ultrafone 304 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3021195 = {S84459, 140, { &P12788,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Evo 3D CDMA Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022893 = {S84460, 135, { &P12788,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; PG86100 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023850 = {S84461, 139, { &P12788,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Evo 3D GSM Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024448 = {S84462, 135, { &P22597,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SCH-I545 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3062198 = {S84463, 152, { &P18721,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A90S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3059364 = {S84464, 93, { &P18513,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6802) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3081048 = {S84465, 137, { &P20264,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; bq Curie Build/1.1.0 X-14:50) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094438 = {S84466, 135, { &P21829,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; F-02E Build/V17R48A) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3062358 = {S84467, 139, { &P15397,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST18i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3080309 = {S84468, 135, { &P23094,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; F-06E Build/V21R48D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094803 = {S84469, 138, { &P21495,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E220S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093910 = {S84470, 137, { &P20075,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Xperia S Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3095116 = {S84471, 162, { &P19862,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Micromax A110Q Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3096489 = {S84472, 150, { &P12879,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7510 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA20089 = {S84473, 74, { &P11958,&P5448,&P22271,&P18092 }};	/*Useragent - Palm750/v0005 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA69078 = {S84474, 54, { &P17573,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SH3B UP.Browser/6.2_7.2.7.1.K.3.352 (GUI) MMP/2.0*/
const Device UA93065 = {S84475, 93, { &P17740,&P17658,&P18392,&P18091 }};	/*Useragent - DoCoMo/2.0 P05A(c100;TB;W24H15) (compatible; BaiduMobaider/1.0; +http://www.baidu.jp/spider/)*/
const Device UA153200 = {S84476, 54, { &P17571,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SN3I UP.Browser/6.2_7.2.7.1.K.3.352 (GUI) MMP/2.0*/
const Device UA226449 = {S84477, 123, { &P15364,&P17285,&P22405,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.1.0.0 Chrome/12.0.742.91 Safari/534.30*/
const Device UA234520 = {S84478, 130, { &P15364,&P17285,&P22405,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.1.0.0 Chrome/12.0.742.91 Safari/534.30*/
const Device UA369990 = {S84479, 140, { &P12955,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ZTE Blade Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2401542 = {S84480, 140, { &P12955,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; ST3_Blade Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2643148 = {S84481, 140, { &P12955,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZTE Blade Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA409321 = {S84482, 55, { &P15364,&P18576,&P17470,&P18092 }};	/*Useragent - Million%20HD%20WPs/8.80.1 CFNetwork/459 Darwin/10.0.0d3*/
const Device UA543317 = {S84483, 46, { &P15364,&P18576,&P17470,&P18092 }};	/*Useragent - WOW%20WPs/8.80.1 CFNetwork/459 Darwin/10.0.0d3*/
const Device UA547427 = {S84484, 49, { &P15364,&P18576,&P17470,&P18092 }};	/*Useragent - InstapaperPro/3.0.3 CFNetwork/459 Darwin/10.0.0d3*/
const Device UA670870 = {S84485, 44, { &P15364,&P18576,&P17470,&P18092 }};	/*Useragent - GQMagazine/1.0 CFNetwork/459 Darwin/10.0.0d3*/
const Device UA755682 = {S84486, 46, { &P15364,&P18576,&P17470,&P18092 }};	/*Useragent - AtomicLite/5.6.0 CFNetwork/459 Darwin/10.0.0d3*/
const Device UA789230 = {S84487, 47, { &P15364,&P18576,&P17470,&P18092 }};	/*Useragent - Y!%20Search/2.0.3 CFNetwork/459 Darwin/10.0.0d3*/
const Device UA824466 = {S84488, 115, { &P15364,&P7127,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.6 Safari/532.0*/
const Device UA572222 = {S84489, 43, { &P14010,&P17484,&P17338,&P18092 }};	/*Useragent - samsung-gt-s5263/UC Browser7.9.0.102/69/352*/
const Device UA575078 = {S84490, 136, { &P18731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ONDA MID Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021479 = {S84491, 60, { &P18731,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; ONDA MID Build/GRJ22)*/
const Device UA2023014 = {S84492, 124, { &P18731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ONDA MID Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2024044 = {S84493, 129, { &P18731,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ONDA MID Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA625255 = {S84494, 127, { &P21453,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; ja-jp; A01SH Build/HTK55D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA848615 = {S84495, 125, { &P21453,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; A01SH Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA680782 = {S84496, 119, { &P15364,&P7127,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.237 Safari/534.10*/
const Device UA2164744 = {S84497, 37, { &P15775,&P5448,&P24630,&P18092 }};	/*Useragent - NOKIA-Lumia 800/UCWEB2.2.0.157/47/355*/
const Device UA2168009 = {S84498, 105, { &P15364,&P18144,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA2644563 = {S84499, 104, { &P15364,&P18144,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD i386) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11*/
const Device UA2200539 = {S84500, 81, { &P15364,&P17285,&P22553,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:15.0) Gecko/X Firefox/15.0.2.0 IceDragon/15.0.2.0*/
const Device UA2211904 = {S84501, 101, { &P15364,&P7127,&P22516,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; fr; rv:1.8.1.9pre) Gecko/X Firefox/2.0.0.9 Navigator/9.0.0.3*/
const Device UA2242336 = {S84502, 109, { &P22108,&P17979,&P18062,&P18092 }};	/*Useragent - HS-T96_TD/1.0 Android/4.0 Release/5.10.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2258911 = {S84503, 64, { &P15364,&P17285,&P18428,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:16.0) Gecko/X Firefox/16.0 slurp*/
const Device UA2366783 = {S84504, 71, { &P15364,&P17285,&P18428,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:16.0) Gecko/X Firefox/16.0 slurp*/
const Device UA2260239 = {S84505, 62, { &P15364,&P18144,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64; rv:16.0) Gecko/X Firefox/16.0*/
const Device UA2748138 = {S84506, 169, { &P12280,&P17084,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/5.1 Mobile/8B117 Safari/7534.48.3 UCBrowser8.5.1.228/41/355*/
const Device UA2484714 = {S84507, 168, { &P12798,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT18i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2409221 = {S84508, 84, { &P15364,&P5690,&P19994,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 1.1.4322) Sleipnir/2.7.0*/
const Device UA2438967 = {S84509, 167, { &P15364,&P5690,&P19994,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30) Sleipnir/2.7.1*/
const Device UA2445139 = {S84510, 104, { &P15364,&P5690,&P19994,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727) Sleipnir/2.7.2*/
const Device UA2436693 = {S84511, 134, { &P20498,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP7100D3G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2457622 = {S84512, 155, { &P15575,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2842883 = {S84513, 154, { &P15575,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467026 = {S84514, 68, { &P14967,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; LG-P500; 320*480) UCWEB7.9.0.94/139/355*/
const Device UA2654278 = {S84515, 68, { &P14967,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.2; zh-cn; LG-P500; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2480812 = {S84516, 150, { &P20157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en_us; Cloudfone Excite350g Build/FSV05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2482828 = {S84517, 132, { &P20873,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; Consul 7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2534677 = {S84518, 170, { &P11747,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Desire V Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2562881 = {S84519, 90, { &P18303,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; MT27i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2578367 = {S84520, 156, { &P15850,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; LG-lu6200 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2653966 = {S84521, 156, { &P15850,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-LU6200 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2579052 = {S84522, 169, { &P23412,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; TM-9741 Build/V1.1.ICECREAMSANDWICH.rus.teXet.X.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 MYI/1.10*/
const Device UA2583780 = {S84523, 141, { &P21181,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S560 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2600081 = {S84524, 138, { &P21502,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LEMON P7  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2616510 = {S84525, 183, { &P22037,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, us*/
const Device UA2632258 = {S84526, 192, { &P22037,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; GT-P3113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.22/288,1.9.22.288,2013-01-31 13:09, -0500, pe*/
const Device UA2659541 = {S84527, 91, { &P14877,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2; en-US; GT-I9000) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2660445 = {S84528, 97, { &P18382,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_Desire_C) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2666787 = {S84529, 136, { &P22666,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Penta WS707C Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2701370 = {S84530, 156, { &P15575,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2701383 = {S84531, 156, { &P15575,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2730024 = {S84532, 157, { &P15575,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2850551 = {S84533, 156, { &P15575,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2731446 = {S84534, 133, { &P23382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; YPY_AB10D Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2878800 = {S84535, 134, { &P23382,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; YPY_AB10DC Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2736924 = {S84536, 82, { &P21321,&P17979,&P18112,&P18092 }};	/*Useragent - SAMSUNG-GT-I9228_TD/1.0 Android/4.1.2 Release/12.24.2012 Browser/AppleWebKit534.30*/
const Device UA2737098 = {S84537, 97, { &P11747,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire V) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2783609 = {S84538, 97, { &P11747,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire V) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2751895 = {S84539, 161, { &P15060,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5660 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2994766 = {S84540, 161, { &P15060,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-S5660 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3048255 = {S84541, 161, { &P15060,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-CN; GT-S5660 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2799011 = {S84542, 137, { &P22601,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300L Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2851881 = {S84543, 63, { &P15364,&P16110,&P4551,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 5.0; Windows NT 6.2; Trident/5.0)*/
const Device UA2857078 = {S84544, 142, { &P19750,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDZ Dash Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3016901 = {S84545, 144, { &P19750,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDZ Dash X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867380 = {S84546, 150, { &P20993,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI P2-6011 Build/HuaweiP2-6011) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893394 = {S84547, 99, { &P12137,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5620) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2898730 = {S84548, 93, { &P15649,&P5096,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5570) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2939576 = {S84549, 142, { &P23693,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PNDPP48GP Build/ICS.PNDPP48GP.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2941836 = {S84550, 159, { &P12258,&P17483,&P24678,&P18092 }};	/*Useragent - LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gs290) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2960443 = {S84551, 176, { &P12139,&P5204,&P24678,&P18092 }};	/*Useragent - Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5310xpressmusic) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2968157 = {S84552, 165, { &P13064,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2968277 = {S84553, 146, { &P12765,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3002733 = {S84554, 32, { &P23324,&P22832,&P18246,&P18092 }};	/*Useragent - Videocon_V1539N_Maui Wap Browser*/
const Device UA3021053 = {S84555, 140, { &P20001,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S720 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024178 = {S84556, 163, { &P22582,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3093618 = {S84557, 137, { &P14877,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093914 = {S84558, 138, { &P21019,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E270K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3078911 = {S84559, 156, { &P15364,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; ZTE U956 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3108503 = {S84560, 160, { &P15364,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; L36h Build/10.1.1.A.1.253) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3108849 = {S84561, 165, { &P15364,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI C8815 Build/HuaweiC8815) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3111002 = {S84562, 161, { &P18507,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I8160 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3113524 = {S84563, 114, { &P14669,&P5843,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-00/08.25; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA2431 = {S84564, 130, { &P11965,&P5338,&P3772,&P18092 }};	/*Useragent - MOT-MOTOQA1/A2.00.52R1 BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003388) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.60 [es]*/
const Device UA489823 = {S84565, 123, { &P11965,&P5338,&P3772,&P18092 }};	/*Useragent - MOT-MOTOQA1/A2.00.52R1 BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; X) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.60 [en]*/
const Device UA2891 = {S84566, 72, { &P12045,&P17486,&P3430,&P18092 }};	/*Useragent - MOT-A3100 Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/16465; U; pt-BR)*/
const Device UA2893 = {S84567, 69, { &P12045,&P17486,&P3430,&P18092 }};	/*Useragent - MOT-A3100 Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/16263; U; en)*/
const Device UA2894 = {S84568, 69, { &P12045,&P17486,&P3430,&P18092 }};	/*Useragent - MOT-A3100 Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/16108; U; en)*/
const Device UA22874 = {S84569, 69, { &P12045,&P17486,&P3430,&P18092 }};	/*Useragent - MOT-A3100 Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/16747; U; en)*/
const Device UA22876 = {S84570, 73, { &P12045,&P17486,&P3430,&P18092 }};	/*Useragent - MOT-A3100/ Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/15405; U; zh-tw)*/
const Device UA22877 = {S84571, 73, { &P12045,&P17486,&P3430,&P18092 }};	/*Useragent - MOT-A3100/ Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/16747; U; zh-tw)*/
const Device UA22878 = {S84572, 73, { &P12045,&P17486,&P3430,&P18092 }};	/*Useragent - MOT-A3100/ Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/16263; U; zh-tw)*/
const Device UA22880 = {S84573, 72, { &P12045,&P17486,&P3430,&P18092 }};	/*Useragent - MOT-A3100 Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/16465; U; es-ES)*/
const Device UA36222 = {S84574, 72, { &P12045,&P17486,&P3430,&P18092 }};	/*Useragent - MOT-A3100 Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/16263; U; es-ES)*/
const Device UA44650 = {S84575, 69, { &P12045,&P17486,&P3430,&P18092 }};	/*Useragent - MOT-A3100 Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/15405; U; en)*/
const Device UA54103 = {S84576, 136, { &P12439,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; E120 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA30331 = {S84577, 61, { &P12299,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia6102/2.0 (04.50) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64841 = {S84578, 62, { &P12299,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia6102i/2.0 (04.68) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA9242 = {S84579, 68, { &P12154,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHE570/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA27964 = {S84580, 87, { &P12194,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch2_T3320; Windows Phone 6.5)*/
const Device UA296638 = {S84581, 87, { &P12194,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch2_T3330; Windows Phone 6.5)*/
const Device UA29880 = {S84582, 145, { &P14042,&P5448,&P4808,&P18092 }};	/*Useragent - SonyEricssonX2/R1AA Browser/Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA55283 = {S84583, 47, { &P14209,&P17484,&P5001,&P18092 }};	/*Useragent - SCH-U430/1.0 NetFront/3.0.22.2.12 (GUI) MMP/2.0*/
const Device UA62964 = {S84584, 78, { &P14464,&P5448,&P4578,&P18092 }};	/*Useragent - HPiPAQhw6900/1.0/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x240)*/
const Device UA89658 = {S84585, 54, { &P17511,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-CA3G UP.Browser/6.2_7.2.7.1.K.2.234 (GUI) MMP/2.0*/
const Device UA473140 = {S84586, 86, { &P15364,&P7133,&P21928,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0) chromeframe/6.0.472.63*/
const Device UA2978628 = {S84587, 187, { &P15364,&P7133,&P21928,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MS-RTC LM 8) chromeframe/6.0.472.63*/
const Device UA3041591 = {S84588, 187, { &P15364,&P7133,&P21928,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MS-RTC LM 8) chromeframe/6.0.472.63*/
const Device UA634688 = {S84589, 43, { &P11907,&P5890,&P24665,&P18092 }};	/*Useragent - NokiaE52-1/UC Browser8.0.4.121/28/444/UCWEB*/
const Device UA694887 = {S84590, 136, { &P15364,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; CSL-MI410 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA724023 = {S84591, 133, { &P15364,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; swiper Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026002 = {S84592, 133, { &P15364,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 3GC101 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2775749 = {S84593, 130, { &P15364,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; HD7 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA712855 = {S84594, 137, { &P14849,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SCH-I800 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2024630 = {S84595, 132, { &P14849,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I800 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1996634 = {S84596, 78, { &P17595,&P18207,&P18515,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:14.0) Gecko/X Thunderbird/14.0*/
const Device UA2034226 = {S84597, 163, { &P22153,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Andi2 Build/iBall_HERA_00.04.79.17.43.H1019.M01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2042760 = {S84598, 92, { &P15364,&P16110,&P22363,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:15.0) Gecko/X Firefox/15.0-x64 PaleMoon/15.0-x64*/
const Device UA2186810 = {S84599, 79, { &P15364,&P16110,&P22363,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:15.0) Gecko/X Firefox/15.0 PaleMoon/15.0*/
const Device UA2863269 = {S84600, 166, { &P13644,&P5629,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (PC36100, 42, 1.9.1)*/
const Device UA2257391 = {S84601, 166, { &P15919,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P3100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2447625 = {S84602, 166, { &P15919,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2337850 = {S84603, 130, { &P23430,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; miTab URBAN Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2552938 = {S84604, 135, { &P23430,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; miTab URBAN Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2746373 = {S84605, 158, { &P20075,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xperia S Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2383967 = {S84606, 120, { &P12236,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA6700s) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile*/
const Device UA2403926 = {S84607, 171, { &P20330,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; Micromax P350 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2407225 = {S84608, 42, { &P18088,&P7480,&P17470,&P18092 }};	/*Useragent - Apple/4.0 Downcast/2.7.14.1133 iTunes/10.7*/
const Device UA2457556 = {S84609, 190, { &P18595,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I8190 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/177,1.9.6.177,2012-10-17 16:38, gb*/
const Device UA2469519 = {S84610, 141, { &P19999,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A30 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2527425 = {S84611, 31, { &P18583,&P5043,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia110/03.26)*/
const Device UA2529818 = {S84612, 150, { &P23876,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2531177 = {S84613, 168, { &P18303,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; MT27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2554813 = {S84614, 22, { &P23269,&P17017,&P17470,&P18092 }};	/*Useragent - A352 WAP-Browser/1.0.0*/
const Device UA2559789 = {S84615, 160, { &P19998,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2659248 = {S84616, 162, { &P19998,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8812E Build/HuaweiC8812E) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2583076 = {S84617, 145, { &P20168,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Maxwell Plus Build/1.0.3 X-14:07) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583370 = {S84618, 134, { &P18113,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT912 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2600231 = {S84619, 146, { &P22944,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Starmobile FLIRT Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2615294 = {S84620, 193, { &P13452,&P5629,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:34, my*/
const Device UA2617017 = {S84621, 194, { &P13452,&P5629,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-P1000L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.7/210,1.9.7.210,2012-10-25 19:53, ar*/
const Device UA2617464 = {S84622, 193, { &P13452,&P5629,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.7/209,1.9.7.209,2012-10-24 23:31, au*/
const Device UA2624204 = {S84623, 203, { &P13452,&P5629,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000N Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, -0400, tt*/
const Device UA2626026 = {S84624, 203, { &P13452,&P5629,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-P1000M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, -0400, ca*/
const Device UA2622938 = {S84625, 216, { &P12889,&P6546,&P18525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Opera/9.80 (Android; Opera Mini/6.0.25270/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2623269 = {S84626, 214, { &P12889,&P6546,&P18525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Opera/9.80 (Android; Opera Mini/6.0.25270/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2660539 = {S84627, 90, { &P13018,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MT11i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2667477 = {S84628, 150, { &P20841,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 10 Build/1.1.11-1015 X-10:30) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2707343 = {S84629, 153, { &P20841,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 10 3G Build/1.1.11-1015 X-16:17) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2713316 = {S84630, 137, { &P18458,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I200 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2717754 = {S84631, 165, { &P18649,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A57 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752780 = {S84632, 93, { &P22607,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S5303) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2769108 = {S84633, 145, { &P20105,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A919 Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769534 = {S84634, 152, { &P21968,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2771082 = {S84635, 133, { &P15390,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P925 Build/ICS) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2781733 = {S84636, 93, { &P14417,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5236) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2798075 = {S84637, 134, { &P19985,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SH-02E Build/S4260) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879582 = {S84638, 99, { &P15655,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3850) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2944088 = {S84639, 160, { &P22862,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax A88 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2966040 = {S84640, 148, { &P21769,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ME301T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2975705 = {S84641, 136, { &P22660,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P778 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022857 = {S84642, 138, { &P20737,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2996464 = {S84643, 138, { &P24687,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-tw; HTC 802d Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3106984 = {S84644, 140, { &P24687,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HTC-802d/2.17.1401.2) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3020184 = {S84645, 103, { &P11932,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6210 Navigator) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3059448 = {S84646, 174, { &P21857,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3021820 = {S84647, 93, { &P13974,&P5096,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5670) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3023098 = {S84648, 137, { &P18686,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A810S Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023397 = {S84649, 137, { &P18686,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A810K Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023743 = {S84650, 129, { &P21772,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; D2-721G Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3023811 = {S84651, 88, { &P18666,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A25) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3063769 = {S84652, 154, { &P22860,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Q700 Build/XOLOQ700) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3063527 = {S84653, 97, { &P20729,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A27) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3025944 = {S84654, 137, { &P24377,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; F-07E Build/V19R38A) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3060428 = {S84655, 165, { &P19998,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3092756 = {S84656, 167, { &P19998,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI C8812E Build/HuaweiC8812E) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3094453 = {S84657, 161, { &P21020,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Micromax A116 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3094637 = {S84658, 102, { &P18672,&P18110,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-S5302) U2/1.0.0 UCBrowser/9.3.0.321 Mobile*/
const Device UA3115610 = {S84659, 156, { &P22597,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I545 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3131720 = {S84660, 136, { &P21793,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9205 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA2389 = {S84661, 75, { &P11958,&P5448,&P17173,&P18092 }};	/*Useragent - Palm750/v0004 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA20090 = {S84662, 75, { &P11958,&P5448,&P17173,&P18092 }};	/*Useragent - Palm750/v0101 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA20091 = {S84663, 75, { &P11958,&P5448,&P17173,&P18092 }};	/*Useragent - Palm750/v0005 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA20697 = {S84664, 41, { &P15126,&P17484,&P3625,&P18092 }};	/*Useragent - sam-r310 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA22436 = {S84665, 85, { &P15469,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-Z150/1.0 SHP/VPP/R5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA70686 = {S84666, 98, { &P12059,&P5204,&P17338,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2690/UC Browser7.9.0.102/70/352*/
const Device UA2518426 = {S84667, 251, { &P12059,&P5204,&P17338,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/69/352*/
const Device UA75293 = {S84668, 116, { &P14993,&P5629,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.3.4;Zh_cn;LT15i;480*854;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA2019444 = {S84669, 67, { &P14993,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.2;Zh_cn;LT15i;480*854;)UCWEB7.8.0.95/139/355*/
const Device UA2022912 = {S84670, 67, { &P14993,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.4;Zh_cn;LT15i;480*854;)UCWEB7.8.0.95/139/444*/
const Device UA2023296 = {S84671, 67, { &P14993,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.3;Zh_cn;LT15i;480*854;)UCWEB7.8.0.95/139/444*/
const Device UA90399 = {S84672, 54, { &P17590,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SH3L UP.Browser/6.2_7.2.7.1.K.6.210 (GUI) MMP/2.0*/
const Device UA453125 = {S84673, 62, { &P15364,&P18146,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; SunOS sun4u; rv:9.0.1) Gecko/X Firefox/9.0.1*/
const Device UA583660 = {S84674, 44, { &P17595,&P17297,&P21582,&P18092 }};	/*Useragent - MacOutlook/14.14.0.X (Intel Mac OS X 10.7.2)*/
const Device UA659108 = {S84675, 44, { &P17595,&P17297,&P21582,&P18092 }};	/*Useragent - MacOutlook/14.14.0.X (Intel Mac OS X 10.7.3)*/
const Device UA2334046 = {S84676, 44, { &P17595,&P17297,&P21582,&P18092 }};	/*Useragent - MacOutlook/14.14.0.X (Intel Mac OS X 10.7.4)*/
const Device UA2830226 = {S84677, 44, { &P17595,&P17297,&P21582,&P18092 }};	/*Useragent - MacOutlook/14.14.0.X (Intel Mac OS X 10.7.5)*/
const Device UA911142 = {S84678, 76, { &P17595,&P18578,&P18537,&P18092 }};	/*Useragent - Reeder/1010.79.00 CFNetwork/520.4.3 Darwin/11.4.0 (x86_64) (MacBookAir4%2C2)*/
const Device UA911152 = {S84679, 76, { &P17595,&P18578,&P18537,&P18092 }};	/*Useragent - Reeder/1010.79.00 CFNetwork/520.4.3 Darwin/11.4.0 (x86_64) (MacBookPro8%2C1)*/
const Device UA1955794 = {S84680, 72, { &P17595,&P18578,&P18537,&P18092 }};	/*Useragent - Reeder/1010.79.00 CFNetwork/520.4.3 Darwin/11.4.0 (x86_64) (MacPro3%2C1)*/
const Device UA1966582 = {S84681, 73, { &P17595,&P18578,&P18537,&P18092 }};	/*Useragent - Reeder/1010.79.00 CFNetwork/520.4.3 Darwin/11.4.0 (x86_64) (MacBook5%2C1)*/
const Device UA1984728 = {S84682, 71, { &P17595,&P18578,&P18537,&P18092 }};	/*Useragent - Reeder/1010.79.00 CFNetwork/520.4.3 Darwin/11.4.0 (x86_64) (iMac11%2C1)*/
const Device UA1933528 = {S84683, 100, { &P11793,&P5768,&P17347,&P18092 }};	/*Useragent - Nokia3500c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia3500c/UC Browser8.0.3.107/69/352*/
const Device UA691285 = {S84684, 80, { &P15210,&P5448,&P4777,&P18092 }};	/*Useragent - SAMSUNG-GT-i8000/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA885142 = {S84685, 71, { &P16356,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;GT-N7000;800*1280;)UCWEB7.8.0.95/139/444*/
const Device UA2026746 = {S84686, 120, { &P16356,&P5629,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.3.6;Zh_cn;GT-N7000;800*1280;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA921601 = {S84687, 140, { &P15767,&P17146,&P21786,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) 1Password/3.6.1/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA924491 = {S84688, 73, { &P15364,&P7133,&P18330,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:14.0) Gecko/X Firefox/14.0.1 slurp*/
const Device UA1995189 = {S84689, 66, { &P15364,&P7133,&P18330,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:14.0) Gecko/X Firefox/14.0.1 slurp*/
const Device UA1995654 = {S84690, 79, { &P15364,&P7133,&P18330,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:14.0) Gecko/X Firefox/14.0.1 slurp slurp*/
const Device UA1939830 = {S84691, 130, { &P17994,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; N-04C Build/AX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033008 = {S84692, 54, { &P17994,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; N-04C Build/AX)*/
const Device UA1988212 = {S84693, 85, { &P14182,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s5333) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2811137 = {S84694, 85, { &P14182,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5333) UCBrowser8.3.1.161/69/405/UCWEB Mobile*/
const Device UA2033606 = {S84695, 133, { &P20474,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; A606 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034396 = {S84696, 134, { &P21341,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Mi500; en-us; Mi500 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027358 = {S84697, 141, { &P21664,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; th-th; i-mobile i698 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028362 = {S84698, 142, { &P13503,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire Z Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2401914 = {S84699, 138, { &P13503,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Desire Z Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2503584 = {S84700, 142, { &P13503,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire Z Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2029730 = {S84701, 158, { &P20134,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2427685 = {S84702, 86, { &P20134,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Adr (Linux; U; 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD.ZCLF1;320*480)*/
const Device UA2817367 = {S84703, 86, { &P20134,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Adr (Linux; U; 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD.ZCLA4;320*480)*/
const Device UA2138986 = {S84704, 85, { &P15364,&P17285,&P21929,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0) chromeframe/5.0.317.0*/
const Device UA2813298 = {S84705, 224, { &P15364,&P17285,&P21929,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; OfficeLiveConnector.1.4; .NET CLR 3.0.30729; OfficeLivePatch.1.3; FDM) chromeframe/5.0.342.0*/
const Device UA2349206 = {S84706, 96, { &P17595,&P18207,&P22578,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; Edition Next; en) Presto/2.11.310 Version/12.50*/
const Device UA2168263 = {S84707, 84, { &P13648,&P5055,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; NokiaE7-00)/UC Browser8.5.0.183/51/355/UCWEB Mobile*/
const Device UA2167601 = {S84708, 48, { &P23026,&P22828,&P17470,&P18092 }};	/*Useragent - ZEN M28; Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2354336 = {S84709, 186, { &P12135,&P17483,&P24672,&P18092 }};	/*Useragent - LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-gd510) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile*/
const Device UA2382172 = {S84710, 182, { &P14714,&P5843,&P24671,&P18092 }};	/*Useragent - NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax3-02) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2383317 = {S84711, 114, { &P14848,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-00/11.00; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2415147 = {S84712, 128, { &P20252,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V96A Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2438680 = {S84713, 78, { &P15364,&P18110,&P21960,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; en-US; rv:2.0b2pre) Gecko/X Minefield/4.0b2pre*/
const Device UA2587420 = {S84714, 78, { &P15364,&P18110,&P21960,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 (x86_64); rv:2.0b4pre) Gecko/X Minefield/4.0b4pre*/
const Device UA2447490 = {S84715, 141, { &P20286,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MT917 Build/6.7.2_GC-205-DNRTD-4) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2549675 = {S84716, 150, { &P20286,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MT917 Build/6.7.2_GC-205-DNRTD-4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3098289 = {S84717, 151, { &P20286,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MT917 Build/6.7.2_GC-131-DNRTD-20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455957 = {S84718, 100, { &P15794,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_Sensation_XL_with_Beats_Audio_X315e; 480*800) UCWEB7.9.0.94/139/352*/
const Device UA2465985 = {S84719, 100, { &P15794,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC_Sensation_XL_with_Beats_Audio_X315e; 480*800) UCWEB7.9.0.94/139/352*/
const Device UA2528398 = {S84720, 100, { &P15794,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC_Sensation_XL_with_Beats_Audio_X315e; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2506731 = {S84721, 155, { &P14848,&P5843,&P24675,&P18092 }};	/*Useragent - NokiaX3-00/5.0 (11.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiax3-00) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2583563 = {S84722, 141, { &P19759,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo K860 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2594899 = {S84723, 64, { &P13746,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 1.6; zh-cn; GT540; 320*480) UCWEB7.9.0.94/139/352*/
const Device UA2653379 = {S84724, 72, { &P13746,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.1-update1; zh-cn; GT540; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2658564 = {S84725, 136, { &P23919,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; R800i Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2600045 = {S84726, 137, { &P23782,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; PHCASIATAB10B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2600250 = {S84727, 169, { &P16292,&P18598,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us ; Galaxy Nexus Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2660424 = {S84728, 170, { &P16292,&P18598,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; Galaxy Nexus Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2797454 = {S84729, 143, { &P12280,&P18432,&P24406,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 OverDrive Media Console/2.6.5*/
const Device UA2611075 = {S84730, 79, { &P13452,&P5096,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.4.5) (GT-P1000;samsung;GT-P1000;2.2/FROYO)*/
const Device UA2660905 = {S84731, 131, { &P21990,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; chagall Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2666405 = {S84732, 131, { &P22928,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID7011 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2703165 = {S84733, 92, { &P18621,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; LG-E612) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2734699 = {S84734, 138, { &P18072,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120L Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2751582 = {S84735, 138, { &P18072,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2991281 = {S84736, 138, { &P18072,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SHV-E120L Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2992135 = {S84737, 138, { &P18072,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120K Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734750 = {S84738, 166, { &P21117,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HS-9DTB4 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/405*/
const Device UA2734905 = {S84739, 136, { &P22651,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E435 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3087739 = {S84740, 137, { &P22651,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E435f Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2737619 = {S84741, 90, { &P18448,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; ST26i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2750297 = {S84742, 155, { &P18301,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; VS840 4G Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752648 = {S84743, 137, { &P22600,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300K Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2755518 = {S84744, 130, { &P19852,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7511 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2856955 = {S84745, 156, { &P19856,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-LS970 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2972105 = {S84746, 172, { &P19856,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-LS970 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/geehrc4g_spr_us*/
const Device UA2769175 = {S84747, 134, { &P20343,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZP900 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892772 = {S84748, 135, { &P20343,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZP900S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769494 = {S84749, 134, { &P18253,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT894 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773291 = {S84750, 130, { &P20154,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5770D Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2866969 = {S84751, 159, { &P20183,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI-ONE Plus Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2799404 = {S84752, 142, { &P21020,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Micromax A116 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2852971 = {S84753, 90, { &P17595,&P17296,&P23668,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; es-ES; rv:1.9.2.12) Gecko/X Firefox/3.7.12*/
const Device UA2874509 = {S84754, 135, { &P22653,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTL22 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2877433 = {S84755, 138, { &P18072,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877827 = {S84756, 138, { &P18072,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892456 = {S84757, 138, { &P18072,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2944941 = {S84758, 187, { &P15794,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2959379 = {S84759, 155, { &P24164,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; CROSS A7S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2968507 = {S84760, 164, { &P18640,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2968783 = {S84761, 148, { &P24134,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Celkon A8+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3022416 = {S84762, 130, { &P13452,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3026020 = {S84763, 159, { &P21602,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; C5302 Build/12.0.A.1.211) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3093394 = {S84764, 159, { &P21602,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; C5302 Build/12.0.A.2.245) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3058066 = {S84765, 178, { &P14826,&P5843,&P24055,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311*/
const Device UA3093971 = {S84766, 134, { &P18471,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT907 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA19556 = {S84767, 86, { &P12255,&P17485,&P3600,&P18092 }};	/*Useragent - SonyEricssonZ520a/R3F Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22429 = {S84768, 86, { &P12255,&P17485,&P3600,&P18092 }};	/*Useragent - SonyEricssonZ520c/R3J Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA28493 = {S84769, 86, { &P12255,&P17485,&P3600,&P18092 }};	/*Useragent - SonyEricssonZ520i/R3F Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22181 = {S84770, 41, { &P15395,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-R210 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA48936 = {S84771, 40, { &P13873,&P17486,&P17602,&P18092 }};	/*Useragent - MOT-EX245 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA66982 = {S84772, 32, { &P14805,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N906i(c100;TB;W24H16)*/
const Device UA160422 = {S84773, 35, { &P14805,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N906imyu(c100;TB;W24H16)*/
const Device UA561944 = {S84774, 33, { &P14805,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N906iL(c100;TB;W24H16)*/
const Device UA253978 = {S84775, 99, { &P14042,&P5448,&P4779,&P18092 }};	/*Useragent - SonyEricssonX2/R3AA Browser/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA290402 = {S84776, 126, { &P13668,&P17350,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en_US) AppleWebKit/530.19.2 (KHTML, like Gecko) Dreamweaver/11.0.0.4909 Version/4.0.2 Safari/530.19.2*/
const Device UA453146 = {S84777, 143, { &P13503,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Desire Z Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028736 = {S84778, 148, { &P13503,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC_DesireZ_A7272 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2745586 = {S84779, 139, { &P13503,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Desire Z Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA467215 = {S84780, 58, { &P15364,&P21445,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows ME; U; en) Presto/2.2.15 Version/10.00*/
const Device UA1862895 = {S84781, 58, { &P15364,&P21445,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (Windows ME; U; en) Presto/2.2.15 Version/10.01*/
const Device UA664287 = {S84782, 146, { &P14923,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; TeamDRH ICS for GTablet Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA673928 = {S84783, 146, { &P14923,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TeamDRH ICS for GTablet Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA903846 = {S84784, 80, { &P17595,&P17296,&P18514,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:13.0) Gecko/X Thunderbird/13.0.1*/
const Device UA1983290 = {S84785, 130, { &P24019,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PC1007 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2473877 = {S84786, 130, { &P24019,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PC1007 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2022646 = {S84787, 142, { &P12600,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SCH-i559 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2686150 = {S84788, 135, { &P12600,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SCH-i559 Build/GINGERBREAD) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2318436 = {S84789, 138, { &P18221,&P5843,&P24670,&P18092 }};	/*Useragent - Nokia302/5.0 (14.53) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia302) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2423542 = {S84790, 175, { &P18649,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; Micromax A57 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2386147 = {S84791, 117, { &P14763,&P5776,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian72) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2469228 = {S84792, 168, { &P12616,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; WT19i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2429149 = {S84793, 129, { &P15767,&P18117,&P18355,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.2m*/
const Device UA2446890 = {S84794, 137, { &P20366,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE T82 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2456110 = {S84795, 128, { &P20528,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; R97B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2458580 = {S84796, 118, { &P12707,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia701) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2463828 = {S84797, 135, { &P20974,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; W900 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950596 = {S84798, 135, { &P20974,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; W900 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2467170 = {S84799, 119, { &P12487,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA5700) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile*/
const Device UA3118178 = {S84800, 91, { &P12487,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5700) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2486588 = {S84801, 164, { &P15985,&P6437,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6200 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2621263 = {S84802, 165, { &P15985,&P6437,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6200L Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/352*/
const Device UA2530329 = {S84803, 146, { &P22630,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; INHON PAPILIO G1 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2560094 = {S84804, 93, { &P18283,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-I9300) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2583228 = {S84805, 121, { &P15767,&P18117,&P21569,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Incredimail/1.0.4.3811*/
const Device UA2602042 = {S84806, 132, { &P17835,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; NookTablet Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2749870 = {S84807, 149, { &P17835,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Barnes & Noble Nook Tablet Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2603543 = {S84808, 138, { &P20013,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F180L Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2666983 = {S84809, 138, { &P20013,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F180S Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2708474 = {S84810, 138, { &P20013,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F180K Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2623783 = {S84811, 166, { &P15919,&P18111,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P3100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2633804 = {S84812, 166, { &P15919,&P18111,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-P3100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2660835 = {S84813, 97, { &P19869,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; Micromax_A30) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2709057 = {S84814, 142, { &P22676,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Celkon A119 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2719727 = {S84815, 142, { &P22654,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; C5502 Build/10.3.A.0.295) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3126857 = {S84816, 65, { &P22654,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; C5502 Build/10.3.1.A.2.67)*/
const Device UA2729914 = {S84817, 159, { &P12502,&P17485,&P24676,&P18092 }};	/*Useragent - SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; fa; sonyericssonw20i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2869508 = {S84818, 93, { &P18304,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I9070) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2738181 = {S84819, 105, { &P11929,&P5129,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) Nokia5230*/
const Device UA2739852 = {S84820, 156, { &P18680,&P18066,&P22587,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2752556 = {S84821, 164, { &P12657,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC EVO 3D X515m Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2768487 = {S84822, 97, { &P22903,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A62) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2773724 = {S84823, 131, { &P23177,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TAB-840_G Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2869264 = {S84824, 148, { &P20182,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xolo_X900 Build/AZ510_ICS_02.36I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2795096 = {S84825, 148, { &P22995,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; Celkon A58 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2796075 = {S84826, 153, { &P21261,&P18598,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892353 = {S84827, 147, { &P23713,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; Fly IQ238 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2893393 = {S84828, 137, { &P22382,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3012601 = {S84829, 165, { &P13991,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6400L Build/IMM76L; CyanogenMod-TouchWiz 5.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3023126 = {S84830, 133, { &P23146,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HYUNDAI T7S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023228 = {S84831, 136, { &P18438,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E973 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3002952 = {S84832, 138, { &P22681,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; UMI X1S Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022456 = {S84833, 139, { &P14993,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT15i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3062460 = {S84834, 158, { &P13018,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT11i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3037096 = {S84835, 164, { &P20183,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3083650 = {S84836, 126, { &P17686,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; MZ601 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3081464 = {S84837, 134, { &P23090,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i9400 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110677 = {S84838, 102, { &P18623,&P21697,&P24641,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 510) U2/1.0.0 UCBrowser/3.1.1.343 U2/1.0.0 Mobile*/
const Device UA3109398 = {S84839, 112, { &P22717,&P5843,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia109/04.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA3132843 = {S84840, 126, { &P16598,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A500 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA9264 = {S84841, 155, { &P13401,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 NokiaE66-3/300.21.012; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA24583 = {S84842, 40, { &P13963,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KS660 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA51989 = {S84843, 64, { &P13963,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KS660_CMCC Release/01.02.2009 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA56049 = {S84844, 145, { &P13200,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; Docomo HT-03A Build/DRD08) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA56780 = {S84845, 102, { &P11689,&P5776,&P4715,&P18092 }};	/*Useragent - NokiaN70-1/5.0737.3.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.4.0.65/27/352*/
const Device UA292563 = {S84846, 101, { &P11689,&P5776,&P4715,&P18092 }};	/*Useragent - NokiaN70/ 5.0741.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.4.0.65/27/351*/
const Device UA64610 = {S84847, 70, { &P13590,&P17483,&P3499,&P18092 }};	/*Useragent - Mozilla/5.0(compatible; Teleca Q7; 4.0.3; U; en) 240X480 LGE LG-LG9600*/
const Device UA69104 = {S84848, 88, { &P11978,&P5448,&P4779,&P18092 }};	/*Useragent - HTC_Touch2_T3333 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA2332407 = {S84849, 54, { &P17587,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3X UP.Browser/6.2_7.2.7.1.K.7.113 (GUI) MMP/2.0*/
const Device UA90612 = {S84850, 133, { &P15364,&P18110,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; arm; en, Nuvifone) AppleWebKit/523.13 (KHTML, like Gecko) Version/3.0 Mobile Safari/419.3 Qt Qtopia Iris/1.1.2*/
const Device UA290073 = {S84851, 129, { &P15364,&P18110,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; arm; en, Nuvi) AppleWebKit/523.13 (KHTML, like Gecko) Version/3.0 Mobile Safari/419.3 Qt Qtopia Iris/1.1.2*/
const Device UA176439 = {S84852, 59, { &P16588,&P22784,&P17470,&P18092 }};	/*Useragent - MicromaxX226+/3.83; Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA844787 = {S84853, 135, { &P14979,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; md-us; A70BHT Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA397413 = {S84854, 133, { &P16168,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; NookColor Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA851443 = {S84855, 133, { &P16168,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NookColor Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1958931 = {S84856, 134, { &P16168,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nook Color Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2031187 = {S84857, 134, { &P16168,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nook Color Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA515285 = {S84858, 59, { &P15364,&P5448,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (WindowsCE 6.0; rv:2.0.1) Gecko/X Firefox/4.0.1*/
const Device UA671640 = {S84859, 90, { &P17595,&P17297,&P18368,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:8.0) Gecko/X Thunderbird/8.0 Lightning/1.0*/
const Device UA651338 = {S84860, 115, { &P14669,&P5843,&P17438,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-00/04.90; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15*/
const Device UA613171 = {S84861, 128, { &P15767,&P18117,&P20775,&P18092 }};	/*Useragent - MQQBrowser/2.4 Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3*/
const Device UA680550 = {S84862, 130, { &P15767,&P18117,&P20775,&P18092 }};	/*Useragent - MQQBrowser/2.4 Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA729632 = {S84863, 102, { &P15364,&P7133,&P22516,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.8.1.5pre) Gecko/X Firefox/2.0.0.4 Navigator/9.0b2*/
const Device UA905745 = {S84864, 141, { &P12280,&P18117,&P20785,&P18092 }};	/*Useragent - MQQBrowser/34 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA1995526 = {S84865, 138, { &P12280,&P18117,&P20785,&P18092 }};	/*Useragent - MQQBrowser/34 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3*/
const Device UA2163901 = {S84866, 139, { &P12280,&P18117,&P20785,&P18092 }};	/*Useragent - MQQBrowser/34 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3*/
const Device UA2332124 = {S84867, 140, { &P12280,&P18117,&P20785,&P18092 }};	/*Useragent - MQQBrowser/34 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B208 Safari/7534.48.3*/
const Device UA1962384 = {S84868, 130, { &P15364,&P5690,&P22456,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Iron/6.0.475.1 Chrome/6.0.475.1 Safari/X.534*/
const Device UA2409182 = {S84869, 109, { &P15364,&P5690,&P22456,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Iron/6.0.475 Safari/534*/
const Device UA1988263 = {S84870, 146, { &P21621,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; i-mobile i-note S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1990916 = {S84871, 108, { &P15364,&P17350,&P18208,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; ; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.2.2.6613 Safari/532.4*/
const Device UA2167030 = {S84872, 147, { &P12280,&P18432,&P21786,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.6.5/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA2223006 = {S84873, 135, { &P21101,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ZP100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2578124 = {S84874, 135, { &P21101,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZP100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2845418 = {S84875, 140, { &P21101,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZOPO ZP100 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2227736 = {S84876, 133, { &P21059,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A90 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2334640 = {S84877, 97, { &P15364,&P17285,&P22486,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.8.1.7) Gecko/X Firefox/2.0.0.7 Flock/0.9.1.2*/
const Device UA2382513 = {S84878, 118, { &P16072,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN78) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile*/
const Device UA2404033 = {S84879, 138, { &P17324,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9001 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2469200 = {S84880, 169, { &P18223,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT22i Build/6.1.1.B.1.54) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2413775 = {S84881, 135, { &P22146,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; Tablet816HD Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2419967 = {S84882, 136, { &P21650,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; TAB-PROTAB26 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422869 = {S84883, 74, { &P20330,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; Micromax_P350; 480*764) UCWEB7.9.0.94/139/444*/
const Device UA2424327 = {S84884, 134, { &P18464,&P18110,&P19816,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.4 Safari/535.19 Silk-Accelerated=false*/
const Device UA2427855 = {S84885, 171, { &P14877,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355*/
const Device UA2460054 = {S84886, 171, { &P14877,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2442909 = {S84887, 153, { &P12663,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; LG-E730 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2493634 = {S84888, 89, { &P15364,&P7133,&P21957,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2a1pre) Gecko/X Minefield/3.2a1pre*/
const Device UA2503013 = {S84889, 132, { &P14908,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6210 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2613625 = {S84890, 132, { &P14908,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P6210 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2615932 = {S84891, 138, { &P14908,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P6210 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2622768 = {S84892, 138, { &P14908,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6210 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2555901 = {S84893, 142, { &P23968,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; S930 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2695641 = {S84894, 66, { &P23968,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 4.0.3; S930 Build/MocorDroid2.3.5)*/
const Device UA2562489 = {S84895, 64, { &P15364,&P17285,&P18689,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:19.0) Gecko/X Firefox/19.0 slurp*/
const Device UA2850556 = {S84896, 71, { &P15364,&P17285,&P18689,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:19.0) Gecko/X Firefox/19.0 slurp*/
const Device UA2579844 = {S84897, 139, { &P15060,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S5660 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580827 = {S84898, 141, { &P21498,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LEMON P100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2583074 = {S84899, 138, { &P21028,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Smartfren Tab 7 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2597197 = {S84900, 93, { &P14433,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-I9003) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2597708 = {S84901, 138, { &P16321,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Xolo_X500 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2841209 = {S84902, 143, { &P16321,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; cs-cz; EVOLVE_FX500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2598388 = {S84903, 103, { &P16595,&P5477,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; Micromax_A60) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2603533 = {S84904, 138, { &P18747,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A840S Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2603758 = {S84905, 166, { &P16292,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Galaxy Nexus Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2667568 = {S84906, 187, { &P13157,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2675426 = {S84907, 102, { &P15775,&P21697,&P24633,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 800) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile*/
const Device UA2706945 = {S84908, 137, { &P18622,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A820L Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708267 = {S84909, 126, { &P12973,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A101 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2712726 = {S84910, 157, { &P12973,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2727789 = {S84911, 127, { &P15364,&P18144,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+*/
const Device UA2983671 = {S84912, 126, { &P15364,&P18144,&P17830,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD i386) AppleWebKit/535.22+ (KHTML, like Gecko) Chromium/17.0.963.56 Chrome/17.0.963.56 Safari/535.22+*/
const Device UA2728066 = {S84913, 156, { &P18072,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; SHV-E120S Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747013 = {S84914, 156, { &P18072,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; SHV-E120L Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2796890 = {S84915, 159, { &P15147,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; ST15i Build/4.0.2.A.0.84) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2738115 = {S84916, 158, { &P15147,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST15i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2755262 = {S84917, 152, { &P21833,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774000 = {S84918, 134, { &P19985,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SH-02E Build/S4100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2793465 = {S84919, 163, { &P18295,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2782967 = {S84920, 148, { &P19782,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B1-A71 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2779723 = {S84921, 140, { &P23345,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; Onyx Calypso 9.7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2784085 = {S84922, 136, { &P22950,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E977 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2855242 = {S84923, 154, { &P18240,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P700 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865564 = {S84924, 136, { &P22805,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E455 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2874809 = {S84925, 138, { &P18564,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One X+ Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2886516 = {S84926, 74, { &P17595,&P18207,&P23479,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:25.0) Gecko/X Firefox/25.0*/
const Device UA2898653 = {S84927, 134, { &P23700,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; N9101 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2944423 = {S84928, 156, { &P19847,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S6012 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3021084 = {S84929, 126, { &P18282,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2966961 = {S84930, 159, { &P13018,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT11i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3032708 = {S84931, 149, { &P18556,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BNTV600 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3021824 = {S84932, 133, { &P24069,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; R815 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3018427 = {S84933, 94, { &P15118,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaX7-00) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3023687 = {S84934, 138, { &P22941,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6812B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023928 = {S84935, 137, { &P22941,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6812 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3032071 = {S84936, 151, { &P18673,&P18066,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3059291 = {S84937, 152, { &P18673,&P18066,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562i Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3034497 = {S84938, 172, { &P12056,&P5204,&P24679,&P18092 }};	/*Useragent - Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7210supernova) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3081191 = {S84939, 136, { &P20713,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID7065 Build/ICS.MID7065.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080484 = {S84940, 135, { &P17948,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SC-02C Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094307 = {S84941, 136, { &P23164,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-7204 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3115123 = {S84942, 159, { &P16292,&P21637,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Galaxy Nexus Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3131398 = {S84943, 128, { &P23975,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B1-710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3509 = {S84944, 141, { &P12189,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; es-ve; U8220 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA24085 = {S84945, 145, { &P12189,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; U8220PLUS Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32549 = {S84946, 154, { &P12189,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; Pulse with FLB-MOD Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA17126 = {S84947, 122, { &P16045,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SonyEricssonX1a/R3AA Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA50310 = {S84948, 141, { &P18004,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; CBW Blaze Build/DONUT) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA649683 = {S84949, 32, { &P17755,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F09A3(c500;TB;W24H16)*/
const Device UA149673 = {S84950, 54, { &P17507,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-CA3E UP.Browser/6.2_7.2.7.1.K.2.234 (GUI) MMP/2.0*/
const Device UA458366 = {S84951, 78, { &P15364,&P7133,&P22534,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:9.0.1) Gecko/X Firefox/9.0.1 Epic/9.0.1*/
const Device UA562538 = {S84952, 125, { &P17595,&P17297,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_0; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA652566 = {S84953, 125, { &P17595,&P17297,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_2; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA796664 = {S84954, 125, { &P17595,&P17297,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_3; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA1961439 = {S84955, 125, { &P17595,&P17297,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA2651914 = {S84956, 125, { &P17595,&P17297,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA656733 = {S84957, 64, { &P15364,&P7133,&P18101,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:12.0) Gecko/X Firefox/12.0 slurp*/
const Device UA660631 = {S84958, 71, { &P15364,&P7133,&P18101,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.0) Gecko/X Firefox/12.0 slurp*/
const Device UA813278 = {S84959, 77, { &P15364,&P7133,&P18101,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:12.0) Gecko/X Firefox/12.0 slurp slurp*/
const Device UA665968 = {S84960, 43, { &P12137,&P17484,&P17347,&P18092 }};	/*Useragent - samsung-gt-s5620/UC Browser8.0.3.107/69/352*/
const Device UA707564 = {S84961, 71, { &P17595,&P17297,&P21572,&P18092 }};	/*Useragent - Mac OS X/10.7.4 (11E53); ExchangeWebServices/2.1 (125); Mail/5.2 (1278)*/
const Device UA710533 = {S84962, 72, { &P17595,&P17297,&P21572,&P18092 }};	/*Useragent - Mac OS X/10.7.3 (11D50d); ExchangeWebServices/2.1 (125); Mail/5.2 (1257)*/
const Device UA722811 = {S84963, 72, { &P17595,&P17297,&P21572,&P18092 }};	/*Useragent - Mac OS X/10.7.3 (11D50b); ExchangeWebServices/2.1 (125); Mail/5.2 (1257)*/
const Device UA819221 = {S84964, 71, { &P17595,&P17297,&P21572,&P18092 }};	/*Useragent - Mac OS X/10.7.3 (11D50); ExchangeWebServices/2.1 (125); Mail/5.2 (1257)*/
const Device UA1992759 = {S84965, 73, { &P17595,&P17297,&P21572,&P18092 }};	/*Useragent - Mac OS X/10.7.3 (11D2001); ExchangeWebServices/2.1 (125); Mail/5.2 (1257)*/
const Device UA899478 = {S84966, 64, { &P15364,&P5690,&P18171,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:13.0) Gecko/X Firefox/13.0 slurp*/
const Device UA905704 = {S84967, 66, { &P15364,&P5690,&P18171,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:13.0) Gecko/X Firefox/13.0.1 slurp*/
const Device UA2320487 = {S84968, 72, { &P15364,&P5690,&P18171,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:13.0) Gecko/X Firefox/13.0.1 slurp slurp*/
const Device UA2328040 = {S84969, 70, { &P15364,&P5690,&P18171,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:13.0) Gecko/X Firefox/13.0 slurp slurp*/
const Device UA1979875 = {S84970, 174, { &P15501,&P5477,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E16i Build/2.1.1.A.0.53) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA1874556 = {S84971, 119, { &P15364,&P18346,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 4.10; zh-CN) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19.1*/
const Device UA2023461 = {S84972, 165, { &P22383,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.16; en-gb; PMP3074B Build/ECLAIR.eng.root.X.X) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2031158 = {S84973, 152, { &P22383,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.19; hr-hr; PMP3074B Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2032099 = {S84974, 133, { &P22435,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; TAB465EUK Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2221963 = {S84975, 47, { &P11854,&P5890,&P21761,&P18092 }};	/*Useragent - iBrowser/2.5 (E71-1;SymbianOS/9.2 Series60/3.1)*/
const Device UA2332125 = {S84976, 96, { &P15364,&P7133,&P21780,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.2.0 Safari/534.34*/
const Device UA2372606 = {S84977, 103, { &P15364,&P7133,&P21780,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.2.0 Safari/534.34*/
const Device UA2351386 = {S84978, 124, { &P12375,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIANokia N81) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile*/
const Device UA2384433 = {S84979, 133, { &P23746,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; O+ TABLET Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2420921 = {S84980, 70, { &P20060,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax Q23/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2455474 = {S84981, 57, { &P16598,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; A500 Build/JZO54K)*/
const Device UA2456165 = {S84982, 128, { &P16598,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; A500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2466851 = {S84983, 139, { &P20511,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Fly_IQ442 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2468391 = {S84984, 143, { &P21713,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NWZ-Z1000Series Build/E200) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2503385 = {S84985, 141, { &P20482,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GOCLEVER TAB A104 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2531432 = {S84986, 169, { &P13018,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; MT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2696153 = {S84987, 161, { &P11660,&P17485,&P24673,&P18092 }};	/*Useragent - SonyEricssonC902/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonc902) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2560609 = {S84988, 171, { &P20332,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax P275 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2567949 = {S84989, 134, { &P12605,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT875 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2597873 = {S84990, 137, { &P23941,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; S58 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2792661 = {S84991, 200, { &P23771,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8190N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.14/271,1.9.14.271,2012-12-20 15:49, +0000, gb*/
const Device UA2675710 = {S84992, 134, { &P22727,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGP312 Build/10.1.C.0.344) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676061 = {S84993, 133, { &P21963,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Q800 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2693936 = {S84994, 168, { &P18630,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A800S Build/CM9 V2 by atulaphanI; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2834569 = {S84995, 154, { &P18630,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A800S Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3107907 = {S84996, 165, { &P18630,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A800S Build/IMM76L; CyanogenMod-Rom_Cook_CM9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2711780 = {S84997, 158, { &P15397,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST18i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2733704 = {S84998, 143, { &P22751,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Celkon*A86 Build/Celkon_A86) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2734266 = {S84999, 148, { &P23122,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Karbonn A6 Build/MocorDroid4.0.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2734938 = {S85000, 130, { &P21117,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HS-9DTB4 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2865749 = {S85001, 134, { &P21117,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HS-9DTB4-4GB Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2750618 = {S85002, 139, { &P22844,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; FLY F45s Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2751595 = {S85003, 154, { &P23717,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; O+ 8.6 Android Build/Oplus8.6_Android) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2751619 = {S85004, 140, { &P15767,&P17090,&P20782,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/8J2 Safari/7534.48.3*/
const Device UA2769704 = {S85005, 140, { &P15767,&P17090,&P20782,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/8L1 Safari/7534.48.3*/
const Device UA2753368 = {S85006, 158, { &P18513,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2769388 = {S85007, 136, { &P18662,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ISW13HT Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773810 = {S85008, 152, { &P20749,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783512 = {S85009, 136, { &P14574,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Nexus S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783707 = {S85010, 156, { &P22585,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L720 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2860744 = {S85011, 135, { &P23957,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; FPT V Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2864666 = {S85012, 163, { &P13042,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; HTC Incredible S Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2937961 = {S85013, 163, { &P13042,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HTC Incredible S Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2866931 = {S85014, 139, { &P23237,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOLO X910 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2869726 = {S85015, 131, { &P15767,&P18117,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.4.2m*/
const Device UA2964566 = {S85016, 131, { &P15767,&P18117,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.4.4m*/
const Device UA3092946 = {S85017, 131, { &P15767,&P18117,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.4.5m*/
const Device UA2898424 = {S85018, 148, { &P20544,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Maxwell 2 Lite Build/1.0.0_X-14:29) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3125665 = {S85019, 148, { &P20544,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Maxwell 2 Lite Build/1.0.1_X-09:59) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2941538 = {S85020, 145, { &P18113,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2975567 = {S85021, 129, { &P22606,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N5110 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2997516 = {S85022, 154, { &P24541,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ALCATEL ONE TOUCH 8008W Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2 Mobile Safari/534.30*/
const Device UA3021048 = {S85023, 97, { &P18649,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A57) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3057284 = {S85024, 94, { &P18649,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; id; Micromax_A57) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Ponsel*/
const Device UA3023271 = {S85025, 98, { &P19862,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Micromax_A110) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3023208 = {S85026, 144, { &P21629,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Lenovo S920_ROW Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3063693 = {S85027, 161, { &P21882,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax P362 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3116402 = {S85028, 105, { &P24615,&P17484,&P18328,&P18092 }};	/*Useragent - SAMSUNG GT-S3332 Opera/9.80 (J2ME/MIDP; Opera Mini/7.1.32840/31.1475; U; en) Presto/2.8.119 Version/11.10*/
const Device UA3081680 = {S85029, 138, { &P19858,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KYL21 Build/019.0.3b00) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083357 = {S85030, 134, { &P21815,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; P-02E Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065540 = {S85031, 144, { &P20253,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; i-mobile IQ 5.1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3109344 = {S85032, 145, { &P20253,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; i-mobile IQ 5.1A Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3095057 = {S85033, 158, { &P17625,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST25i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3115104 = {S85034, 158, { &P17625,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST25i Build/6.0.B.3.188) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3095128 = {S85035, 165, { &P15798,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Explorer A310e Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3095744 = {S85036, 156, { &P18283,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9300 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3131085 = {S85037, 136, { &P20226,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131322 = {S85038, 138, { &P22602,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-S738C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131598 = {S85039, 130, { &P17337,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA56148 = {S85040, 32, { &P17760,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH06B(c500;TB;W24H16)*/
const Device UA673384 = {S85041, 84, { &P17968,&P5062,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Android 1.6; Linux; Opera Mobi/ADR-X; U; en) Presto/2.7.81 Version/11.00*/
const Device UA68163 = {S85042, 136, { &P17951,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Milestone Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA379886 = {S85043, 136, { &P17951,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Milestone Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA655518 = {S85044, 136, { &P17951,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Milestone Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018666 = {S85045, 136, { &P17951,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; Milestone Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022021 = {S85046, 124, { &P17951,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; Milestone Build/MIUI) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA117838 = {S85047, 83, { &P18088,&P22740,&P18373,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; Maemo; Opera Mobi/4; U; en-GB) Presto/2.5.28 Version/10.1*/
const Device UA133252 = {S85048, 143, { &P15364,&P17816,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Ubuntu/10.04 Chromium/5.0.375.127 Chrome/5.0.375.127 Safari/533.4*/
const Device UA438971 = {S85049, 145, { &P15364,&P17816,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Ubuntu/10.04 Chromium/5.0.375.127 Chrome/5.0.375.127 Safari/533.4*/
const Device UA241249 = {S85050, 93, { &P13999,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; HD2)*/
const Device UA759380 = {S85051, 93, { &P13999,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0;  ; LEO70)*/
const Device UA301994 = {S85052, 143, { &P22127,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; Lenovo 3GW101 Build/FRG83) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA690712 = {S85053, 122, { &P22127,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; 3GW101 Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2031779 = {S85054, 127, { &P22127,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; 3GW101 Build/FRG83) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2023320 = {S85055, 152, { &P13633,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; MotoA953 Build/GWK74; CyanogenMod-72) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031993 = {S85056, 161, { &P13633,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MotoA953 Build/MILS2_U6_4.1-22; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA415512 = {S85057, 147, { &P17346,&P17145,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA495341 = {S85058, 148, { &P17346,&P17145,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8C148 Safari/6531.22.7*/
const Device UA562693 = {S85059, 87, { &P17976,&P6546,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.1; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.8.149 Version/11.10*/
const Device UA684034 = {S85060, 62, { &P15364,&P5690,&P17698,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:8.0) Gecko/X Firefox/8.0 slurp*/
const Device UA689811 = {S85061, 66, { &P15364,&P5690,&P17698,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:8.0.1) Gecko/X Firefox/8.0.1 slurp*/
const Device UA1920526 = {S85062, 68, { &P15364,&P5690,&P17698,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:8.0) Gecko/X Firefox/8.0 slurp slurp*/
const Device UA1985473 = {S85063, 72, { &P22485,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.5;Zh_cn;TECNO T3;320*480;)UCWEB7.8.0.95/139/32929*/
const Device UA2025929 = {S85064, 141, { &P24543,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GOCLEVER TAB A103 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2480521 = {S85065, 141, { &P24543,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB A103 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2127727 = {S85066, 31, { &P11880,&P5890,&P24644,&P18092 }};	/*Useragent - Nokia 5320/UCWEB7.4.0.57/28/800*/
const Device UA2223377 = {S85067, 136, { &P21149,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGT-X40 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2227001 = {S85068, 165, { &P21240,&P18066,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-style Q2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2366289 = {S85069, 169, { &P21240,&P18066,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-STYLE Q2 DUO Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257456 = {S85070, 136, { &P20410,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; MAGPAD9.7Pro Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2354277 = {S85071, 142, { &P21341,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Spice Mi-500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2384382 = {S85072, 138, { &P20869,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A878 Duo Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2385705 = {S85073, 53, { &P15364,&P6729,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows 95) Presto/2.12.388 Version/12.10*/
const Device UA2402267 = {S85074, 70, { &P18672,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;GT-S5302;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA2413120 = {S85075, 91, { &P15364,&P17350,&P22441,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U;) AppleWebKit/532.0 (KHTML, like Gecko) Iron/3.0.197.0 Safari/532.0*/
const Device UA2425693 = {S85076, 154, { &P22875,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654501 = {S85077, 154, { &P22875,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9220 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2436503 = {S85078, 226, { &P18679,&P18066,&P18369,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-B5330 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.30496/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2455033 = {S85079, 150, { &P15364,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; MG-391 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2697611 = {S85080, 147, { &P15364,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; A20 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2457931 = {S85081, 161, { &P12026,&P17485,&P24673,&P18092 }};	/*Useragent - SonyEricssonC510/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonc510) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2467239 = {S85082, 146, { &P20259,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Starmobile ASTRA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2508152 = {S85083, 139, { &P20517,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CINK PEAX Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2527992 = {S85084, 137, { &P23880,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A868 Duo Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2555350 = {S85085, 139, { &P16436,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I8150 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695325 = {S85086, 61, { &P16436,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I8150 Build/JZO54K)*/
const Device UA2557385 = {S85087, 139, { &P23856,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; TH-1127M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2563588 = {S85088, 165, { &P14433,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2565353 = {S85089, 166, { &P16362,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; SPH-D710 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2731286 = {S85090, 169, { &P16362,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; SPH-D710BST Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2675218 = {S85091, 141, { &P18372,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SO-04D Build/7.0.D.1.137) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583325 = {S85092, 146, { &P18300,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-T769 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2584100 = {S85093, 130, { &P12889,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7300 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2633356 = {S85094, 131, { &P12889,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P7300 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2601156 = {S85095, 149, { &P23202,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; aigoPad M80E Build/E8HD-ELITE-1280x768) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2653302 = {S85096, 132, { &P22548,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB 9 V8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2695961 = {S85097, 44, { &P15655,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SAMSUNG-GT-S3850/S3850JPKK2)*/
const Device UA2702783 = {S85098, 129, { &P20579,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; AT570 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2953836 = {S85099, 58, { &P20579,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; AT570 Build/JRO03C)*/
const Device UA2734686 = {S85100, 137, { &P18722,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A830L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734824 = {S85101, 137, { &P18722,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A830K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735628 = {S85102, 137, { &P18722,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A830S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2753062 = {S85103, 137, { &P18002,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A760S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755175 = {S85104, 138, { &P20174,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Cink King Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2770952 = {S85105, 152, { &P22961,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773527 = {S85106, 134, { &P23477,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT926 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2791491 = {S85107, 175, { &P15991,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2 jellybean customized # ariel-tbc; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2784622 = {S85108, 132, { &P23182,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PMP7280C3G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2793246 = {S85109, 138, { &P18456,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T699 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2851692 = {S85110, 64, { &P15364,&P7127,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 5.0; Trident/2.0)*/
const Device UA2957114 = {S85111, 154, { &P18324,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-VM696 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3021594 = {S85112, 134, { &P23090,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i9400 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021320 = {S85113, 145, { &P21489,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Infected Rezound Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2962911 = {S85114, 173, { &P23255,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI G520-5000 Build/HuaweiG520-5000) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3006210 = {S85115, 173, { &P23255,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI G520-0000 Build/HuaweiG520-0000) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3023779 = {S85116, 154, { &P20542,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAP5000TDUO Build/PrestigioPAP5000TDUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3031331 = {S85117, 157, { &P22872,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5282 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3020844 = {S85118, 131, { &P15102,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ605 Build/I.7.1-39.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021954 = {S85119, 133, { &P21642,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NXM805ND_EN Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3015471 = {S85120, 138, { &P18072,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36*/
const Device UA3021245 = {S85121, 138, { &P18072,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023390 = {S85122, 138, { &P18072,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3017799 = {S85123, 88, { &P19860,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A9+) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3057985 = {S85124, 95, { &P19860,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; CELKON_A9+) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3019801 = {S85125, 161, { &P15802,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC_Amaze_4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3092541 = {S85126, 161, { &P15802,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Amaze 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3023035 = {S85127, 147, { &P20931,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; iBall Slide 3G7271 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024143 = {S85128, 141, { &P20931,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; iBall Slide 3G 7334 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3046305 = {S85129, 135, { &P22229,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; NABIXD-NV10A Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA3081376 = {S85130, 137, { &P21481,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A860S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080938 = {S85131, 154, { &P20542,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAP5000TDUO Build/PrestigioPAP5000TDUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093627 = {S85132, 142, { &P19968,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Haier HW-W910 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA24541 = {S85133, 68, { &P13350,&P19954,&P3976,&P18092 }};	/*Useragent - SEC-SGHF210/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA51845 = {S85134, 69, { &P13350,&P19954,&P3976,&P18092 }};	/*Useragent - SEC-SGHF210L/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA52028 = {S85135, 62, { &P13220,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonK770i/R8BC Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA605141 = {S85136, 48, { &P12280,&P17090,&P17470,&P18092 }};	/*Useragent - Facebook 4110.0 (iPhone; iPhone OS 4.3.3; en_US)*/
const Device UA669262 = {S85137, 48, { &P12280,&P17090,&P17470,&P18092 }};	/*Useragent - Facebook 4110.0 (iPhone; iPhone OS 4.3.5; en_US)*/
const Device UA1957300 = {S85138, 48, { &P12280,&P17090,&P17470,&P18092 }};	/*Useragent - Facebook 4110.0 (iPhone; iPhone OS 4.3.1; en_US)*/
const Device UA2242237 = {S85139, 46, { &P12280,&P17090,&P17470,&P18092 }};	/*Useragent - Facebook 4110.0 (iPhone; iPhone OS 4.3; en_US)*/
const Device UA2395045 = {S85140, 48, { &P12280,&P17090,&P17470,&P18092 }};	/*Useragent - Facebook 4110.0 (iPhone; iPhone OS 4.3.2; en_US)*/
const Device UA64629 = {S85141, 21, { &P14687,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GB190 Obigo/WAP2.0*/
const Device UA54587 = {S85142, 133, { &P14584,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S8000C/1.0 SHP/R5 Release/02.01.2009 Browser/NetFront3.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA55137 = {S85143, 107, { &P14584,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S8000H/1.0 SHP/R5 NetFront/3.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA62458 = {S85144, 98, { &P14412,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-A412/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA185544 = {S85145, 62, { &P16558,&P22784,&P18246,&P18092 }};	/*Useragent - Micromax X395/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA330784 = {S85146, 67, { &P13078,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux;U;2.2.2;Zh_cn;ZTE-U V880;480*800;)UCWEB7.6.0.75/139/32014*/
const Device UA398802 = {S85147, 86, { &P18095,&P18109,&P4860,&P18092 }};	/*Useragent - Opera/9.80 (Series 60; Opera Mini/6.24095/27.1169; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2211453 = {S85148, 84, { &P18095,&P18109,&P4860,&P18092 }};	/*Useragent - Opera/9.80 (Series 60; Opera Mini/6.24455/24.871; U; en) Presto/2.5.25 Version/10.54*/
const Device UA682355 = {S85149, 83, { &P15364,&P7133,&P18329,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:11.0) Gecko/X Thunderbird/11.0 Lightning/1.3*/
const Device UA459167 = {S85150, 138, { &P17835,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nook Tablet Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA572734 = {S85151, 106, { &P12132,&P17485,&P17338,&P18092 }};	/*Useragent - SonyEricssonJ10i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonj10i/UC Browser7.9.0.102/69/352*/
const Device UA696384 = {S85152, 108, { &P12132,&P17485,&P17338,&P18092 }};	/*Useragent - SonyEricssonJ10i2/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonj10i2/UC Browser7.9.0.102/69/352*/
const Device UA583915 = {S85153, 120, { &P11787,&P5204,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia3120classic/09.41; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA761850 = {S85154, 85, { &P14985,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s8530) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA852460 = {S85155, 80, { &P15364,&P18146,&P18199,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; SunOS sun4u; en-US; rv:1.8.1.4) Gecko/X Thunderbird/2.0.0.4*/
const Device UA903267 = {S85156, 177, { &P12871,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2031906 = {S85157, 177, { &P12871,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; HTC Wildfire S A510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA925202 = {S85158, 78, { &P17595,&P17296,&P18515,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:14.0) Gecko/X Thunderbird/14.0*/
const Device UA1941866 = {S85159, 138, { &P21313,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; Viettel i5 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1964295 = {S85160, 169, { &P12798,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352*/
const Device UA1993749 = {S85161, 154, { &P14969,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Desire S Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027114 = {S85162, 154, { &P14969,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Desire S Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033816 = {S85163, 154, { &P14969,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Desire S Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1995699 = {S85164, 96, { &P17595,&P17296,&P22578,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; Edition Next; en) Presto/2.11.310 Version/12.50*/
const Device UA2000863 = {S85165, 35, { &P14540,&P5843,&P21759,&P18092 }};	/*Useragent - iBrowser/Mini2.8 (NokiaX2-01/08.63)*/
const Device UA2030485 = {S85166, 136, { &P22131,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; PB-S3 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2166306 = {S85167, 80, { &P17595,&P18207,&P18367,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:12.0) Gecko/X Thunderbird/12.0.1*/
const Device UA2236806 = {S85168, 141, { &P22058,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ALLVIEW P4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2261329 = {S85169, 172, { &P12280,&P18117,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3 UCBrowser8.5.0.195/42/355*/
const Device UA2337696 = {S85170, 78, { &P13602,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; gt-s5233w) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2372819 = {S85171, 75, { &P13602,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; vi; gt-s5233w) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2371817 = {S85172, 131, { &P20709,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID1042 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2382146 = {S85173, 136, { &P12110,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Legend Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2530112 = {S85174, 137, { &P12110,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Legend2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2384558 = {S85175, 152, { &P22140,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; POLY PAD 9708HD Build/ICS.g12refM1005TR.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2455151 = {S85176, 137, { &P20839,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; H9500S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483376 = {S85177, 71, { &P18442,&P18111,&P24673,&P18092 }};	/*Useragent - JUC (Linux; U; 4.1.1; zh-cn; GT-N7100; 720*1280) UCWEB8.7.4.225/145/800*/
const Device UA3059363 = {S85178, 93, { &P18442,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7100) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA2544928 = {S85179, 114, { &P15364,&P24094,&P24304,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips ; U; HbbTV/1.1.1 (; Sharp; ; ; ; ) CE-HTML/1.0 NETTV/3.0.0; en) Presto/2.6.33 Version/10.70*/
const Device UA2651536 = {S85180, 110, { &P15364,&P24094,&P24304,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; HbbTV/1.1.1 (; Opera Software; videowebtv; ; ; ); en) Presto/2.10.287 Version/12.00*/
const Device UA3039672 = {S85181, 151, { &P15364,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL838; 51.3.31.2; t12; ) ; ToshibaTP/1.2.0 (+DRM) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039677 = {S85182, 200, { &P15364,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL838; 55.4.02.0; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039691 = {S85183, 151, { &P15364,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL838; 53.2.17.1; t12; ) ; ToshibaTP/1.2.0 (+DRM) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039692 = {S85184, 147, { &P15364,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL838; 48.4.72.8; t12; ) ; ToshibaTP/1.1.0 () ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039694 = {S85185, 200, { &P15364,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL838; 55.4.02.0; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D) ; pl) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039704 = {S85186, 200, { &P15364,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL838; 55.4.02.0; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D) ; en) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039719 = {S85187, 200, { &P15364,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL838; 73.0.51.8; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039730 = {S85188, 151, { &P15364,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL838; 51.3.31.2; t12; ) ; ToshibaTP/1.2.0 (+DRM) ; it) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039753 = {S85189, 151, { &P15364,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL838; 51.3.31.2; t12; ) ; ToshibaTP/1.2.0 (+DRM) ; pt) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039761 = {S85190, 200, { &P15364,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL838; 85.3.88.6; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3100340 = {S85191, 130, { &P15364,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mips; U;HbbTV/1.1.1 (+RTSP;DMM;Dreambox;0.1a;1.0;) CE-HTML/1.0; en) AppleWebKit/535.19 no/Volksbox QtWebkit/2.2*/
const Device UA2583367 = {S85192, 154, { &P20534,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAP4040_DUO Build/PrestigioPAP4040DUO) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2598354 = {S85193, 168, { &P18451,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT26ii Build/6.1.A.2.55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2600010 = {S85194, 146, { &P20105,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A919 Duo Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2603537 = {S85195, 140, { &P15517,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT15i Build/4.1.B.0.587) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667763 = {S85196, 162, { &P20134,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-S5830i Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2670735 = {S85197, 131, { &P21334,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A8 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2672682 = {S85198, 131, { &P23314,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; G3 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675518 = {S85199, 151, { &P20002,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 992D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2797197 = {S85200, 165, { &P22612,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI T8951 Build/HuaweiT8951) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2732268 = {S85201, 153, { &P20877,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ALCATEL ONE TOUCH 8000D Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2752068 = {S85202, 97, { &P18382,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire C) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2868433 = {S85203, 97, { &P18382,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire C) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2752202 = {S85204, 132, { &P20689,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; N-05E Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768698 = {S85205, 143, { &P19842,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; X920e Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2996909 = {S85206, 134, { &P19842,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X920e Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2768944 = {S85207, 96, { &P13800,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA5630d-1) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2769320 = {S85208, 144, { &P20104,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; 201M Build/9.8.2Q-34_SMJ-102) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2772390 = {S85209, 159, { &P18449,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.7; ST27i Build/6.0.B.3.184) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2772485 = {S85210, 143, { &P16053,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; UNUSUAL Vortex Dual Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2773820 = {S85211, 126, { &P18282,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2773958 = {S85212, 138, { &P20244,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; bq Elcano Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2781499 = {S85213, 168, { &P23003,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; TECNO Q1 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/33567*/
const Device UA2795968 = {S85214, 137, { &P20598,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Miracle Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2799784 = {S85215, 146, { &P23247,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; WALTON_Primo-X1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2800661 = {S85216, 132, { &P20453,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NABI2-NV7A Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2885445 = {S85217, 140, { &P20089,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-01E Build/9.1.C.0.475) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879729 = {S85218, 138, { &P23654,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; AVEO X6 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892883 = {S85219, 147, { &P12972,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T989 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2939140 = {S85220, 165, { &P16292,&P18598,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2944808 = {S85221, 93, { &P13974,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5670) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2962001 = {S85222, 158, { &P12798,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT18i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2962539 = {S85223, 142, { &P24688,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Spice Mi-515 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2965153 = {S85224, 139, { &P24052,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; SKY_NANO Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3021970 = {S85225, 137, { &P23691,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5280 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2974650 = {S85226, 135, { &P17948,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SC-02C Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2991266 = {S85227, 130, { &P23376,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3;vi-vn; M9000 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3003354 = {S85228, 167, { &P23376,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; vi_vn; M9000 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 NetFrontLifeBrowser/2.2.1 Mobile (Dragonfruit)*/
const Device UA3021807 = {S85229, 98, { &P21020,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Micromax_A116) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3028931 = {S85230, 158, { &P18449,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST27i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3061109 = {S85231, 95, { &P14202,&P5477,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E15i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3083374 = {S85232, 140, { &P23498,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZTE Blade G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3063714 = {S85233, 93, { &P20146,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9082) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3111304 = {S85234, 155, { &P21794,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9152 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3128088 = {S85235, 156, { &P18267,&P5043,&P24680,&P18092 }};	/*Useragent - NokiaX2-02/2.0 (11.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; NokiaX2-02) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3132774 = {S85236, 136, { &P22871,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9295 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA11250 = {S85237, 119, { &P13617,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; MDA Vario/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49305 = {S85238, 128, { &P13617,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; MDA Vario/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20132 = {S85239, 52, { &P14566,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP170-orange/V10b Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA64006 = {S85240, 40, { &P14566,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP170 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA54896 = {S85241, 151, { &P12415,&P17486,&P4223,&P18092 }};	/*Useragent - Mot-EX300/B000 Browser/NetFront/3.5 MMS/Obigo-MMS/Q05A SyncML/HW-SyncML/1.0 Java/HWJa/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Player/QTV-Player/5.3*/
const Device UA55050 = {S85242, 65, { &P13630,&P17483,&P3499,&P18092 }};	/*Useragent - LG-GU295/V10i Teleca/Q7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63007 = {S85243, 112, { &P13605,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-I6220/1.0 SHP/VPP/R5 Jasmine/1.0 NexPlayer/2.12 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA160959 = {S85244, 63, { &P23417,&P22843,&P17470,&P18092 }};	/*Useragent - LAVA-A10/SW1.0.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA180244 = {S85245, 59, { &P15767,&P18575,&P17470,&P18092 }};	/*Useragent - Perfect%20Browser-iPad/2.6 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA207981 = {S85246, 57, { &P15767,&P18575,&P17470,&P18092 }};	/*Useragent - MobileRSSFree-iPad/3.1.4 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA318479 = {S85247, 50, { &P15767,&P18575,&P17470,&P18092 }};	/*Useragent - browseripad/23286 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA1889831 = {S85248, 53, { &P15767,&P18575,&P17470,&P18092 }};	/*Useragent - MobileRSS-iPad/3.1.4 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA447320 = {S85249, 142, { &P15364,&P5690,&P21930,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; InfoPath.1) chromeframe/4.0*/
const Device UA2541804 = {S85250, 182, { &P15364,&P5690,&P21930,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET4.0C; .NET4.0E) chromeframe/4.0*/
const Device UA515001 = {S85251, 88, { &P15364,&P6704,&P22479,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.8.1.16) Gecko/X Firefox/2.0.0.16 Flock/1.2.4*/
const Device UA696057 = {S85252, 94, { &P15364,&P6704,&P22479,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win 9x 4.90; en-US; rv:1.8.1.16) Gecko/X Firefox/2.0.0.16 Flock/1.2.4*/
const Device UA578646 = {S85253, 113, { &P12108,&P5843,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia7230/06.90; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA709247 = {S85254, 114, { &P14482,&P5843,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-01/05.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA1970076 = {S85255, 67, { &P13077,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; C8650; 320*480) UCWEB7.9.4.145/139/800*/
const Device UA773494 = {S85256, 159, { &P14993,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; LT15i Build/4.0.2.A.0.42; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1944387 = {S85257, 150, { &P21550,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGT-A20 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1993734 = {S85258, 150, { &P12132,&P17485,&P24670,&P18092 }};	/*Useragent - SonyEricssonJ10i2/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonj10i2) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2336931 = {S85259, 148, { &P12132,&P17485,&P24670,&P18092 }};	/*Useragent - SonyEricssonJ10i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonj10i) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2235634 = {S85260, 191, { &P18295,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, gb*/
const Device UA2643868 = {S85261, 156, { &P14682,&P5843,&P24670,&P18092 }};	/*Useragent - Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia6303iclassic) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2172432 = {S85262, 154, { &P18466,&P18066,&P19989,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.0 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2257788 = {S85263, 159, { &P14161,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LG-VM670 Build/JRO03L; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2428625 = {S85264, 159, { &P14161,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-VM670 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2340602 = {S85265, 171, { &P18213,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.0.181/145/352*/
const Device UA2383779 = {S85266, 172, { &P18213,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6500D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2341584 = {S85267, 146, { &P20894,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; M3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2351241 = {S85268, 134, { &P13633,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A953 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2355163 = {S85269, 134, { &P13633,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A953 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2362990 = {S85270, 39, { &P15775,&P5448,&P20764,&P18092 }};	/*Useragent - NOKIA-Lumia 800/UCWEB2.6.0.213/47/32079*/
const Device UA2421825 = {S85271, 180, { &P19868,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; ALCATEL ONE TOUCH 985N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2379483 = {S85272, 182, { &P13107,&P5043,&P24671,&P18092 }};	/*Useragent - NokiaC2-06/2.0 (07.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-06) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2413444 = {S85273, 137, { &P19789,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KM-S200 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2650358 = {S85274, 137, { &P19789,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KM-S220 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2422635 = {S85275, 190, { &P22114,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; jblend) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile/HS-T39_TD/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0 Config/CLDC-1.1*/
const Device UA2506411 = {S85276, 294, { &P22114,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; jblend) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile/HS-T39_TD/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0 Config/CLDC-1.1, /HS-T39_TD/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0 Config/CLDC-1.1*/
const Device UA2790607 = {S85277, 293, { &P22114,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; jblend) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile/HS-T39_TD/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0 Config/CLDC-1.1,/HS-T39_TD/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0 Config/CLDC-1.1*/
const Device UA2467677 = {S85278, 137, { &P23858,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; sv-se; Tablet 816HDC Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2471725 = {S85279, 144, { &P20507,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 7 Build/1.0.3 X-12:51) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2481605 = {S85280, 101, { &P14188,&P5129,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia5233/51.1.002/sw_platform=S60;sw_platform_version=5.0;java_build_version=1.4.43)*/
const Device UA2495731 = {S85281, 155, { &P15364,&P5096,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 Mozilla/5.0 (Linux; U; Android 2.2.2; in-id; A850 Build/SDB-P2_1.3.20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2983277 = {S85282, 150, { &P15364,&P5096,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; GFIVE A57 Build/S21) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2503127 = {S85283, 68, { &P17595,&P19835,&P21570,&P18092 }};	/*Useragent - Mail/1499 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (MacBookPro8%2C1)*/
const Device UA2553919 = {S85284, 63, { &P17595,&P19835,&P21570,&P18092 }};	/*Useragent - Mail/1499 CFNetwork/596.2.3 Darwin/12.2.0 (x86_64) (iMac12%2C2)*/
const Device UA2534323 = {S85285, 133, { &P20529,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A89 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2560870 = {S85286, 90, { &P17625,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST25i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2579992 = {S85287, 147, { &P22137,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PIRANHA BUSINESS TAB 7" Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2583696 = {S85288, 135, { &P23894,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V1277 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602610 = {S85289, 139, { &P19854,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R760X Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2662566 = {S85290, 96, { &P21474,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.9; en-US; Celkon_A_85) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2738131 = {S85291, 161, { &P13123,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; GT-S5360 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2748787 = {S85292, 72, { &P12280,&P17090,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPhone3,1; iPhone OS 4.3.3; en_US; en) AppleWebKit/420+*/
const Device UA2854283 = {S85293, 72, { &P12280,&P17090,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPhone3,1; iPhone OS 4.3.5; en_US; en) AppleWebKit/420+*/
const Device UA2769376 = {S85294, 130, { &P17337,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2770250 = {S85295, 139, { &P13054,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ADR6425LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784886 = {S85296, 161, { &P18462,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire X Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2773562 = {S85297, 140, { &P20183,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MI-ONE Plus Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799275 = {S85298, 162, { &P19763,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6602 Build/10.1.1.A.1.253) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2800377 = {S85299, 145, { &P17174,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 10 FHD Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2862677 = {S85300, 154, { &P18621,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; LG-E610 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956261 = {S85301, 154, { &P18621,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; LG-E612 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2866726 = {S85302, 126, { &P20999,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-S5292/1.0 Linux/2.6.35.7 Android/4.0.4 Release/04.01.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2958991 = {S85303, 126, { &P20999,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-S5292/1.0 Linux/2.6.35.7 Android/4.0.4 Release/04.16.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2944410 = {S85304, 155, { &P23163,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; SCH-I959 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2939555 = {S85305, 130, { &P24180,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TBD753 Build/Zeki.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2945204 = {S85306, 159, { &P22209,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Titanium S5 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2958192 = {S85307, 139, { &P21532,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; vivo Y11 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2963920 = {S85308, 162, { &P19862,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Micromax A110Q Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2966412 = {S85309, 157, { &P19856,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LS970 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2966495 = {S85310, 146, { &P24109,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; STX ULTRA Build/STXSTX_ULTRA) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2975983 = {S85311, 138, { &P19858,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KYL21 Build/019.0.3b00) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2999856 = {S85312, 163, { &P13042,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HTC Incredible S Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3007839 = {S85313, 163, { &P13042,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; HTC Incredible S Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3018479 = {S85314, 105, { &P12871,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Wildfire_S_A510e) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3020691 = {S85315, 105, { &P12871,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Wildfire_S_A510e) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3020523 = {S85316, 134, { &P24568,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; N9810 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3029067 = {S85317, 188, { &P18639,&P21637,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 10 Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us*/
const Device UA3109613 = {S85318, 188, { &P18639,&P21637,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 10 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.8/620,2.0.8.620,2013-09-06 15:06, -0400, us*/
const Device UA3062904 = {S85319, 153, { &P13452,&P5096,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-US; GT-P1000 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3080458 = {S85320, 140, { &P21181,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S560 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3061845 = {S85321, 88, { &P18666,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A25) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3080018 = {S85322, 137, { &P21490,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A860L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080412 = {S85323, 129, { &P21990,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; chagall Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3093788 = {S85324, 156, { &P22609,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6312 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3108403 = {S85325, 71, { &P15364,&P7133,&P22772,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:24.0) Gecko/X Firefox/24.0 slurp*/
const Device UA3109457 = {S85326, 157, { &P19861,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I605 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3110284 = {S85327, 169, { &P18113,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3110604 = {S85328, 164, { &P22582,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3110658 = {S85329, 141, { &P12280,&P22882,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari/7534.48.3 Sleipnir/2.4.5m*/
const Device UA3130059 = {S85330, 143, { &P12280,&P22882,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Safari/7534.48.3 Sleipnir/2.4.5m*/
const Device UA2909 = {S85331, 70, { &P11870,&P5448,&P17173,&P18092 }};	/*Useragent - HTC_S620 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA86900 = {S85332, 88, { &P11870,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) HTCS620;Smartphone;320x240*/
const Device UA88347 = {S85333, 75, { &P11870,&P5448,&P17173,&P18092 }};	/*Useragent - HTC-Excalibur Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA33396 = {S85334, 42, { &P13825,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KG290UAE Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA62694 = {S85335, 39, { &P13825,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KG290 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA37311 = {S85336, 165, { &P14832,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; LG-KH5200 Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA67518 = {S85337, 163, { &P14832,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; LG-KH5200 Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA148785 = {S85338, 32, { &P17551,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P905i(c100;TB;W24H15)*/
const Device UA56777 = {S85339, 32, { &P14722,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N08A3(c500;TB;W24H16)*/
const Device UA59494 = {S85340, 15, { &P12548,&P7472,&P17470,&P18092 }};	/*Useragent - alcatel-ot-807a*/
const Device UA89917 = {S85341, 31, { &P14747,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F06B(c500;TB;W24H16)*/
const Device UA79417 = {S85342, 119, { &P14433,&P5629,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.3.4;Zh_cn;GT-I9003;480*800;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA2019805 = {S85343, 70, { &P14433,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.4;Zh_cn;GT-I9003;480*800;)UCWEB7.8.0.95/139/352*/
const Device UA2027075 = {S85344, 70, { &P14433,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;GT-I9003;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA625921 = {S85345, 76, { &P15364,&P7133,&P18194,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:6.0) Gecko/X Thunderbird/6.0 Lightning/1.0b5*/
const Device UA579071 = {S85346, 37, { &P11905,&P5890,&P17347,&P18092 }};	/*Useragent - nokia6120c/UC Browser8.0.3.107/69/444*/
const Device UA829562 = {S85347, 77, { &P15364,&P7127,&P23696,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.8b4) Gecko/X Firefox/1.4*/
const Device UA903425 = {S85348, 166, { &P15575,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-I9100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA1953913 = {S85349, 167, { &P15575,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-I9100G Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2033021 = {S85350, 167, { &P15575,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-I9100T Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA903461 = {S85351, 80, { &P17595,&P17297,&P18514,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:13.0) Gecko/X Thunderbird/13.0.1*/
const Device UA2354784 = {S85352, 133, { &P22816,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S22 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2366602 = {S85353, 118, { &P12061,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE65) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile*/
const Device UA2694504 = {S85354, 119, { &P12061,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiae65-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2415846 = {S85355, 140, { &P20238,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ko-kr; LG-LU5400 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2451609 = {S85356, 135, { &P20008,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT687 Build/V2.27D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2454318 = {S85357, 131, { &P17595,&P18207,&P22452,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1*/
const Device UA2764137 = {S85358, 131, { &P17595,&P18207,&P22452,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1*/
const Device UA3001765 = {S85359, 131, { &P17595,&P18207,&P22452,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1*/
const Device UA2458290 = {S85360, 146, { &P18307,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Malata SMBA1002 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469204 = {S85361, 165, { &P14433,&P5096,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2481937 = {S85362, 166, { &P13077,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8650+ Build/V100R001C92B861) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2504599 = {S85363, 153, { &P13077,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HW-HUAWEI_C8650/C8650V100R001C92B825; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2489702 = {S85364, 26, { &P18088,&P17979,&P20428,&P18092 }};	/*Useragent - Instapaper for Android 1.2*/
const Device UA2503340 = {S85365, 88, { &P23050,&P22835,&P17470,&P18092 }};	/*Useragent - SpiceM5395/MTK Release/01.01.2012 Browser/wap2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2523864 = {S85366, 43, { &P17595,&P18207,&P21585,&P18092 }};	/*Useragent - MacOutlook/14.0.0.X (Intel Mac OS X 10.8.2)*/
const Device UA2668257 = {S85367, 43, { &P17595,&P18207,&P21585,&P18092 }};	/*Useragent - MacOutlook/14.0.0.X (Intel Mac OS X 10.8.3)*/
const Device UA3000064 = {S85368, 43, { &P17595,&P18207,&P21585,&P18092 }};	/*Useragent - MacOutlook/14.0.0.X (Intel Mac OS X 10.8.4)*/
const Device UA2526774 = {S85369, 127, { &P15809,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_Rhyme_S510b; en-hk) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2583323 = {S85370, 141, { &P17905,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SO-03D Build/6.1.F.0.106) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2579734 = {S85371, 226, { &P22789,&P22831,&P18246,&P18092 }};	/*Useragent - TECHAIN50_11B_HW (MRE\3.0.00(2500) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) Masscom_P12_T738_M00_S03_X Release/2012.11.16 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 vi-VN*/
const Device UA2779966 = {S85372, 226, { &P22789,&P22831,&P18246,&P18092 }};	/*Useragent - TECHAIN50_11B_HW (MRE\3.0.00(2500) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) Masscom_P12_T738_M00_S04_X Release/2013.01.11 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 vi-VN*/
const Device UA2588383 = {S85373, 84, { &P15364,&P17350,&P19773,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 9.0; en-US; rv:1.9.1.11) Gecko/X SeaMonkey/2.0.6*/
const Device UA2627643 = {S85374, 150, { &P12889,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7300 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2630939 = {S85375, 149, { &P12889,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7300 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2661590 = {S85376, 89, { &P20438,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; N400) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2663701 = {S85377, 160, { &P20004,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2850941 = {S85378, 168, { &P20004,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; Karbonn A2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3058240 = {S85379, 172, { &P20004,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Karbonn_A2 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2699772 = {S85380, 91, { &P13964,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2; en-US; GT-I5510) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2712278 = {S85381, 139, { &P18302,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT25i Build/4.1.B.0.631) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2737846 = {S85382, 157, { &P20251,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328d Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2737859 = {S85383, 154, { &P14967,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; LG-P500 Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2738223 = {S85384, 154, { &P12108,&P5843,&P24677,&P18092 }};	/*Useragent - Nokia7230/5.0 (10.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7230) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2751935 = {S85385, 144, { &P21802,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GSmart Maya M1 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2752084 = {S85386, 88, { &P17595,&P17294,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:20.0) Gecko/X Firefox/20.0 TenFourFox/7450*/
const Device UA2752436 = {S85387, 94, { &P12236,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6700s) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2773991 = {S85388, 133, { &P20584,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Z120 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799365 = {S85389, 161, { &P14969,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2869507 = {S85390, 162, { &P14969,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire SV Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2801225 = {S85391, 40, { &P23144,&P23096,&P17470,&P18092 }};	/*Useragent - Tumblr/117 CFNetwork/646.4 Darwin/14.0.0*/
const Device UA3111551 = {S85392, 42, { &P23144,&P23096,&P17470,&P18092 }};	/*Useragent - Facebook/X CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA2866776 = {S85393, 140, { &P23237,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XOLO X910 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2900612 = {S85394, 168, { &P23255,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI G520-5000 Build/HuaweiG520-5000) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2938210 = {S85395, 168, { &P23255,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI G520-0000 Build/HuaweiG520-0000) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2938188 = {S85396, 144, { &P19776,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Lenovo K900_ROW Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2940159 = {S85397, 127, { &P21052,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-us; A9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2944324 = {S85398, 156, { &P20096,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-S7568 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3020718 = {S85399, 134, { &P23894,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V1277 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2994270 = {S85400, 166, { &P22607,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-S5303 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3020703 = {S85401, 170, { &P22607,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S5303 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3020516 = {S85402, 93, { &P21628,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8552) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3023958 = {S85403, 88, { &P20529,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A89) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3023932 = {S85404, 136, { &P24114,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E440 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3065444 = {S85405, 97, { &P22864,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A26) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3054514 = {S85406, 163, { &P18595,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I8190 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/golden*/
const Device UA3059305 = {S85407, 172, { &P19891,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I717 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/quincyatt*/
const Device UA3094710 = {S85408, 146, { &P20090,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848i Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127092 = {S85409, 145, { &P20090,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848 Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094303 = {S85410, 136, { &P14017,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P990 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083438 = {S85411, 138, { &P15850,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LU6200 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093511 = {S85412, 136, { &P22584,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SGH-I337M Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083355 = {S85413, 141, { &P13124,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PantechP9070 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3108625 = {S85414, 137, { &P21261,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3114672 = {S85415, 134, { &P21261,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i9500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3115557 = {S85416, 156, { &P20146,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9082 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA18510 = {S85417, 127, { &P13100,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) SP; 240x320; HTC_S710/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20551 = {S85418, 94, { &P13100,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Vodafone/1.0/HTC_s710/1.22.172.3*/
const Device UA39876 = {S85419, 94, { &P13100,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Vodafone/1.0/HTC_s710/1.25.163.1*/
const Device UA87450 = {S85420, 94, { &P13100,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Vodafone/1.0/HTC_s710/1.15.163.2*/
const Device UA19886 = {S85421, 110, { &P15364,&P22832,&P17470,&P18092 }};	/*Useragent - Videocon V1760/V1760_00_V10A_IND_01_09_15_10/04.09.2009 Browser/Teleca Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA41566 = {S85422, 99, { &P15364,&P22832,&P17470,&P18092 }};	/*Useragent - Videocon V1533/22.11.2007 Browser/wap2.0 Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA76256 = {S85423, 99, { &P15364,&P22832,&P17470,&P18092 }};	/*Useragent - Videocon V1635/22.11.2007 Browser/wap2.0 Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA133878 = {S85424, 90, { &P15364,&P22832,&P17470,&P18092 }};	/*Useragent - Videocon_V1655_Java/V1655-JAVA-91-V10a-MAR-30-2010 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21419 = {S85425, 101, { &P15274,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-M2310/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA63066 = {S85426, 83, { &P11672,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonZ770i/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA54286 = {S85427, 17, { &P14170,&P22839,&P17470,&P18092 }};	/*Useragent - ZTE-G-S215/WAP2.0*/
const Device UA56165 = {S85428, 145, { &P14656,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-sg; GT-I8520 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA56770 = {S85429, 32, { &P14721,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P906i(c100;TB;W24H15)*/
const Device UA63967 = {S85430, 104, { &P14561,&P17483,&P17470,&P18092 }};	/*Useragent - LG-KD876/v1.0 Arena/L2.3.3 Release/5.2.2008 Browser/Teleca/QLine Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA68345 = {S85431, 146, { &P14918,&P5267,&P3834,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S7230B/1.0; U; Bada/1.0; pt-br) AppleWebKit/533.1 (KHTML, like Gecko) Dolfin/2.0 Mobile WQVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA79159 = {S85432, 136, { &P17989,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Cricket Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA295314 = {S85433, 134, { &P17989,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Cricket Build/FRG22D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA639611 = {S85434, 82, { &P17595,&P17295,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.5.8; U; en) Presto/2.10.229 Version/11.60*/
const Device UA823558 = {S85435, 62, { &P15364,&P21445,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows ME) Opera 6.01 [en]*/
const Device UA826326 = {S85436, 61, { &P15364,&P21445,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows ME) Opera 6.0 [de]*/
const Device UA894984 = {S85437, 31, { &P15364,&P21445,&P17466,&P18092 }};	/*Useragent - Opera/6.05 (Windows ME; U) [de]*/
const Device UA433739 = {S85438, 30, { &P15364,&P6704,&P17215,&P18092 }};	/*Useragent - Opera/9.50 (Windows 98; U; en)*/
const Device UA434609 = {S85439, 30, { &P15364,&P6704,&P17215,&P18092 }};	/*Useragent - Opera/9.52 (Windows 98; U; en)*/
const Device UA520562 = {S85440, 30, { &P15364,&P6704,&P17215,&P18092 }};	/*Useragent - Opera/9.51 (Windows 98; U; en)*/
const Device UA656149 = {S85441, 137, { &P15364,&P17979,&P21556,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.05; en-us; p10t Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA893816 = {S85442, 127, { &P15364,&P17979,&P21556,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0; en-us; dream) AppleWebKit/5.25.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA3116749 = {S85443, 126, { &P15364,&P17979,&P21556,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0; en-us; dream) AppleWebKit/525.10  (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA709438 = {S85444, 87, { &P17595,&P17297,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Macintosh;Mac OS X (10.7.2);en;kml:2.2;client:Free;type:default)*/
const Device UA882833 = {S85445, 87, { &P17595,&P17297,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Macintosh;Mac OS X (10.7.3);en;kml:2.2;client:Free;type:default)*/
const Device UA1991662 = {S85446, 87, { &P17595,&P17297,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Macintosh;Mac OS X (10.7.4);en;kml:2.2;client:Free;type:default)*/
const Device UA2454972 = {S85447, 87, { &P17595,&P17297,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Macintosh;Mac OS X (10.7.5);en;kml:2.2;client:Free;type:default)*/
const Device UA896004 = {S85448, 134, { &P12841,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_PH39100/1.63.502.4; en-us) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA903784 = {S85449, 75, { &P15364,&P22740,&P18566,&P18092 }};	/*Useragent - Mozilla/5.0 (Maemo; Linux armv7l; rv:13.0) Gecko/X Firefox/13.0 Fennec/13.0*/
const Device UA907978 = {S85450, 77, { &P15364,&P22740,&P18566,&P18092 }};	/*Useragent - Mozilla/5.0 (Maemo; Linux armv7l; rv:13.0) Gecko/X Firefox/13.0.1 Fennec/13.0*/
const Device UA2154800 = {S85451, 87, { &P17595,&P17297,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.7373(Macintosh;Mac OS X (10.7.4);en;kml:2.2;client:Free;type:default)*/
const Device UA2317126 = {S85452, 87, { &P17595,&P17297,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Macintosh;Mac OS X (10.7.4);en;kml:2.2;client:Free;type:default)*/
const Device UA2361277 = {S85453, 87, { &P17595,&P17297,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.7373(Macintosh;Mac OS X (10.7.5);en;kml:2.2;client:Free;type:default)*/
const Device UA2481398 = {S85454, 87, { &P17595,&P17297,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Macintosh;Mac OS X (10.7.5);en;kml:2.2;client:Free;type:default)*/
const Device UA2202689 = {S85455, 120, { &P17346,&P18432,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Flipboard/1.9.7*/
const Device UA2367706 = {S85456, 122, { &P17346,&P18432,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Flipboard/1.9.8*/
const Device UA2214945 = {S85457, 119, { &P14222,&P5448,&P4737,&P18092 }};	/*Useragent - 320x240; VZW; UTStar-XV6175.1; Windows Mobile 6.5 Standard; (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.2.2.51*/
const Device UA2807335 = {S85458, 116, { &P14222,&P5448,&P4737,&P18092 }};	/*Useragent - 320x240; VZW; UTStar-XV6175; Window Mobile 6.1 Standard; (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.2.2.51*/
const Device UA2263599 = {S85459, 89, { &P15364,&P5690,&P21958,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2a1pre) Gecko/X Minefield/3.6a1pre*/
const Device UA2340767 = {S85460, 130, { &P17965,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-01D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2360009 = {S85461, 133, { &P12788,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/EVO_3D/3.28.161.3; es-es) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2413150 = {S85462, 133, { &P12788,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/EVO_3D/3.28.161.3; de-de) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2684837 = {S85463, 133, { &P12788,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC/EVO_3D/3.28.165.2; it-it) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2385057 = {S85464, 123, { &P19976,&P5055,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; NokiaE6-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/51/355/UCWEB Mobile*/
const Device UA2400836 = {S85465, 125, { &P14404,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5260) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2408507 = {S85466, 90, { &P15364,&P5690,&P22514,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.6) Gecko/X Firefox/2.0 Wyzo/0.5.3*/
const Device UA2503582 = {S85467, 153, { &P11625,&P5768,&P24673,&P18092 }};	/*Useragent - Nokia5300/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5300) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2505784 = {S85468, 139, { &P17835,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-us; NookTablet Build/JOP40C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2855480 = {S85469, 140, { &P17835,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; NookTablet Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2598092 = {S85470, 161, { &P14743,&P17485,&P24675,&P18092 }};	/*Useragent - SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonj20i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2527417 = {S85471, 135, { &P23467,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; iDx5 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558389 = {S85472, 91, { &P23351,&P22784,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromax) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2565861 = {S85473, 172, { &P18213,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6500D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2934794 = {S85474, 171, { &P18213,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2600425 = {S85475, 136, { &P23862,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; HS-F1 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2613558 = {S85476, 120, { &P15876,&P18066,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android4.0.3;Zh_cn;GT-P5100;800*1232;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA2614632 = {S85477, 71, { &P15876,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.3;Zh_cn;GT-P5100;800*1232;)UCWEB7.8.0.95/139/351*/
const Device UA2623154 = {S85478, 71, { &P15876,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.4;Zh_cn;GT-P5100;800*1232;)UCWEB7.8.0.95/139/444*/
const Device UA2636264 = {S85479, 151, { &P15919,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2860108 = {S85480, 151, { &P15919,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2660697 = {S85481, 92, { &P21023,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; LG-E405) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2664881 = {S85482, 92, { &P23414,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua_Sx) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2709318 = {S85483, 159, { &P15991,&P5629,&P22281,&P18092 }};	/*Useragent - OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2670912 = {S85484, 137, { &P18002,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A760S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676365 = {S85485, 126, { &P18282,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; a200 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2677300 = {S85486, 133, { &P15390,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P925 Build/ICS) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2694157 = {S85487, 126, { &P16356,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-N7000/1.0 Linux/2.6.35.7 Android/4.0.9 Release/08.15.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2696168 = {S85488, 126, { &P16356,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-N7000/1.0 Linux/2.6.35.7 Android/4.0.9 Release/10.11.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA3099645 = {S85489, 126, { &P16356,&P17979,&P18062,&P18092 }};	/*Useragent - sprd-N7000/1.0 Linux/2.6.35.7 Android/4.0.9 Release/09.13.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2703370 = {S85490, 154, { &P22875,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9220 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2709168 = {S85491, 86, { &P22875,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.1.2; zh-cn; GT-I9220 Build/JZO54K.I9220ZCLSF;800*1280)*/
const Device UA2705816 = {S85492, 160, { &P13914,&P5843,&P24676,&P18092 }};	/*Useragent - Nokia5330-1d/5.0 (06.88) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5330-1d) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2714569 = {S85493, 137, { &P22942,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-D700 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2800527 = {S85494, 139, { &P15364,&P17350,&P22776,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; ; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.5.3.X (Second Life Release; default skin) Safari/533.3*/
const Device UA2730342 = {S85495, 170, { &P18673,&P18066,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S7562 Build/IMM76I) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2951276 = {S85496, 166, { &P18673,&P18066,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2732966 = {S85497, 137, { &P23843,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; FPT IV Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2733038 = {S85498, 139, { &P15364,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; redhookbay Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2860769 = {S85499, 139, { &P15364,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Transformer Prime Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA3092369 = {S85500, 136, { &P15364,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Mi-510 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2754232 = {S85501, 133, { &P20156,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S500 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2755283 = {S85502, 135, { &P22760,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ThL W8 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2767314 = {S85503, 161, { &P13644,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EVO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (EVO, 42, 1.9.1)*/
const Device UA2772675 = {S85504, 138, { &P20816,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CINK SLIM Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2779701 = {S85505, 148, { &P23281,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Mobiistar Touch S08 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3100825 = {S85506, 146, { &P23281,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-; Mobiistar Touch S08 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2794761 = {S85507, 138, { &P18679,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-B5330 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3061203 = {S85508, 140, { &P17968,&P18598,&P22824,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.9909AP Mobile*/
const Device UA2877372 = {S85509, 144, { &P22906,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A390_ROW Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799021 = {S85510, 151, { &P20983,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9308 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2800684 = {S85511, 136, { &P22956,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P875 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2860260 = {S85512, 136, { &P18072,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; E120L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2870741 = {S85513, 137, { &P22999,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A870K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879430 = {S85514, 137, { &P22999,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A870S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937553 = {S85515, 137, { &P22999,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A870L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2870896 = {S85516, 137, { &P22755,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; DM014SH Build/S1004) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3030669 = {S85517, 137, { &P22755,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; DM014SH Build/S1008) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879198 = {S85518, 161, { &P22737,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Micromax A115 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2967268 = {S85519, 155, { &P22854,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; GT-I9508 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2893044 = {S85520, 100, { &P23409,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HUAWEI; 4Afrika)*/
const Device UA2894298 = {S85521, 164, { &P12657,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC EVO 3D X515m Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2961139 = {S85522, 168, { &P14557,&P5843,&P24678,&P18092 }};	/*Useragent - Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6303classic) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2963741 = {S85523, 103, { &P13324,&P5055,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian8-00) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3023023 = {S85524, 137, { &P22853,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC 802w Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3033345 = {S85525, 159, { &P13054,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR6425LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2969841 = {S85526, 93, { &P14753,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; Nokia N96) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3023394 = {S85527, 137, { &P19910,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E617G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3015944 = {S85528, 96, { &P12329,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN86 8MP) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3023027 = {S85529, 126, { &P18008,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ST10 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021677 = {S85530, 92, { &P12061,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE65) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3049620 = {S85531, 74, { &P15492,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-C3322i/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA3059093 = {S85532, 168, { &P11787,&P5204,&P24679,&P18092 }};	/*Useragent - Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3120classic) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3060398 = {S85533, 159, { &P12814,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I777 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i777*/
const Device UA3080188 = {S85534, 159, { &P12814,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I777 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i777*/
const Device UA3127922 = {S85535, 159, { &P12814,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I777 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/i777*/
const Device UA3094795 = {S85536, 128, { &P12879,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-P7510 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3093626 = {S85537, 140, { &P20183,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MI-ONE Plus Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081227 = {S85538, 143, { &P22749,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SO-04E Build/10.3.1.B.0.224) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3131304 = {S85539, 136, { &P19911,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E970 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131337 = {S85540, 137, { &P16349,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Sony Tablet S Build/TISU0143) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA16179 = {S85541, 102, { &P13543,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-M7500/M7500ACHL1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA20523 = {S85542, 91, { &P15099,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-R225/ES-ATT-USA-P103D13V1.1.5 OBIGO/Q03C WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA58875 = {S85543, 91, { &P15099,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-R225/ES-ATT-USA-P103D13V1.1.7 OBIGO/Q03C WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20911 = {S85544, 43, { &P15145,&P17017,&P3807,&P18092 }};	/*Useragent - Huawei/1.0/G6603/Browser/obigo-browser/q03c*/
const Device UA21034 = {S85545, 146, { &P12833,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/Vodafone1230/ENX_P180A1V1.1.0 IE Mobile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21035 = {S85546, 129, { &P12833,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/Vodafone1230/IE Mobile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA29380 = {S85547, 136, { &P12833,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/Vodafone1231/LSO_P180A1V1.0.1/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA33031 = {S85548, 146, { &P12833,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/Vodafone1231/ESL_P180A1V1.0.9 IE Mobile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA46101 = {S85549, 129, { &P12833,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/Vodafone1231/IE Mobile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA46100 = {S85550, 146, { &P12833,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/Vodafone1231/PTG_P180A1V1.1.0 IE Mobile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA59158 = {S85551, 146, { &P12833,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/Vodafone1230/INX_P180A1V1.1.4 IE Mobile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21171 = {S85552, 107, { &P11974,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-T739/T739UVHB2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA24579 = {S85553, 41, { &P15714,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-R600 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA24720 = {S85554, 67, { &P15743,&P17017,&P3807,&P18092 }};	/*Useragent - Huawei/1.0/0HuaweiG3511/WAP2.0/Obigo-Browser/Q03C MMS/Obigo-MMS/1.2*/
const Device UA29381 = {S85555, 67, { &P11897,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GR500FD/V10a Teleca/Q7.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA44052 = {S85556, 86, { &P13774,&P6476,&P4223,&P18092 }};	/*Useragent - ZTE-T930 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 firmware/T930 T06*/
const Device UA51628 = {S85557, 34, { &P13949,&P22839,&P17470,&P18092 }};	/*Useragent - ZTE-G N290/WAP2.0 Profile/MIDP-2.0*/
const Device UA56792 = {S85558, 31, { &P14725,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P01A(c100;TB;W24H15)*/
const Device UA56807 = {S85559, 35, { &P14188,&P5129,&P4715,&P18092 }};	/*Useragent - Nokia5233/UC Browser7.4.0.65/50/351*/
const Device UA69179 = {S85560, 47, { &P11684,&P5129,&P4779,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UC Browser7.8.0.95/50/355*/
const Device UA69206 = {S85561, 34, { &P13977,&P5890,&P4779,&P18092 }};	/*Useragent - NokiaN73/UC Browser7.8.0.95/28/351*/
const Device UA89916 = {S85562, 32, { &P16984,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH08B(c500;TB;W24H16)*/
const Device UA90554 = {S85563, 32, { &P16984,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH08C(c500;TB;W24H14)*/
const Device UA2270851 = {S85564, 32, { &P16984,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH08A(c500;TB;W24H14)*/
const Device UA692276 = {S85565, 120, { &P12139,&P5204,&P17347,&P18092 }};	/*Useragent - Nokia5310XpressMusic/2.0 (09.42) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5310xpressmusic/UC Browser8.0.3.107/69/444*/
const Device UA616398 = {S85566, 94, { &P17595,&P18120,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X; Opera Tablet/23731; U; en) Presto/2.9.201 Version/11.50*/
const Device UA2307907 = {S85567, 92, { &P17595,&P18120,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X; Opera Mobi/23731; U; en) Presto/2.9.201 Version/11.50*/
const Device UA677290 = {S85568, 108, { &P12132,&P17485,&P17347,&P18092 }};	/*Useragent - SonyEricssonJ10i2/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonj10i2/UC Browser8.0.3.107/69/444*/
const Device UA1916067 = {S85569, 75, { &P21381,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; Lenovo_A520; 480*800) UCWEB7.9.3.103/139/32707*/
const Device UA2398527 = {S85570, 171, { &P18304,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2222727 = {S85571, 53, { &P21032,&P18615,&P20434,&P18092 }};	/*Useragent - Mozilla/5.0 (Mobile; rv:12.0) Gecko/12.0 Firefox/12.0*/
const Device UA2261294 = {S85572, 194, { &P14557,&P5843,&P24672,&P18092 }};	/*Useragent - Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia6303classic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/352/UCWEB Mobile*/
const Device UA2351371 = {S85573, 119, { &P14417,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5236) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/352/UCWEB Mobile*/
const Device UA2380035 = {S85574, 154, { &P11787,&P5204,&P24668,&P18092 }};	/*Useragent - Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia3120classic) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2384536 = {S85575, 157, { &P13974,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2424085 = {S85576, 157, { &P13974,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2401798 = {S85577, 128, { &P24129,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A900 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2405551 = {S85578, 62, { &P15364,&P5690,&P19821,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:18.0) Gecko/X Thunderbird/18.0*/
const Device UA2481630 = {S85579, 67, { &P15985,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 3.2; zh-cn; GT-P6200; 600*976) UCWEB7.9.0.94/139/444*/
const Device UA2426588 = {S85580, 64, { &P18482,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; A75; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2458274 = {S85581, 165, { &P13974,&P5096,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5670 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2535072 = {S85582, 175, { &P12605,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2537250 = {S85583, 66, { &P12280,&P19826,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 (iPhone4,1; iPhone OS 6.1; en_US) AppleWebKit/420+*/
const Device UA2601095 = {S85584, 68, { &P12280,&P19826,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 (iPhone4,1; iPhone OS 6.1.2; en_US) AppleWebKit/420+*/
const Device UA2766380 = {S85585, 68, { &P12280,&P19826,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 (iPhone4,1; iPhone OS 6.1.3; en_US) AppleWebKit/420+*/
const Device UA2551681 = {S85586, 179, { &P15991,&P5629,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6102 Build/GINGERBREAD; 240*320) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444*/
const Device UA2564221 = {S85587, 168, { &P12616,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; WT19i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2603895 = {S85588, 137, { &P14574,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Nexus S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583571 = {S85589, 129, { &P23873,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BC1003 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2584099 = {S85590, 147, { &P23102,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8681 Build/HuaweiU8681) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2585174 = {S85591, 134, { &P20868,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M040 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2597161 = {S85592, 152, { &P15575,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100G Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2701163 = {S85593, 151, { &P15575,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2986378 = {S85594, 151, { &P15575,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2598050 = {S85595, 143, { &P21993,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MAXIMUS max903 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2664991 = {S85596, 73, { &P20729,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; Micromax_A27; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2667066 = {S85597, 134, { &P21642,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NXM805ND_EN Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2674221 = {S85598, 141, { &P21374,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 101 Titanium Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2675392 = {S85599, 132, { &P20231,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A21 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2862797 = {S85600, 140, { &P20231,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A21 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676481 = {S85601, 131, { &P15102,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ605 Build/I.7.1-39.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2698836 = {S85602, 141, { &P23814,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; U2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2708390 = {S85603, 147, { &P20129,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2767203 = {S85604, 148, { &P20129,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2S Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2714152 = {S85605, 158, { &P16356,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2757208 = {S85606, 131, { &P21334,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A8 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2768645 = {S85607, 135, { &P23477,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT926 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878999 = {S85608, 136, { &P23477,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT926 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768780 = {S85609, 153, { &P16356,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2782434 = {S85610, 138, { &P23184,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; iOCEAN X7 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782628 = {S85611, 137, { &P20947,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Evo V 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782888 = {S85612, 130, { &P19874,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-i705 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2782909 = {S85613, 157, { &P12973,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798995 = {S85614, 126, { &P12973,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2877370 = {S85615, 151, { &P23846,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A12 Build/RheaTK) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2796666 = {S85616, 138, { &P23139,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Primo-N1 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2798559 = {S85617, 136, { &P22651,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E435 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892759 = {S85618, 137, { &P22651,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E435f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2855407 = {S85619, 137, { &P24585,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; ZP800H Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2868624 = {S85620, 148, { &P20175,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3050508 = {S85621, 149, { &P20175,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1SC Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2941510 = {S85622, 156, { &P16074,&P5768,&P24678,&P18092 }};	/*Useragent - Nokia3110c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3110c) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2943731 = {S85623, 93, { &P18513,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6802) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2964274 = {S85624, 161, { &P15649,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5570 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2964719 = {S85625, 162, { &P15649,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5570I Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2966947 = {S85626, 161, { &P14186,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5830 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3016338 = {S85627, 161, { &P14186,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-S5830 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3020858 = {S85628, 136, { &P23164,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-7204 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022216 = {S85629, 137, { &P22952,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F260S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021730 = {S85630, 153, { &P20534,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAP4040_DUO Build/PrestigioPAP4040DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020405 = {S85631, 93, { &P22609,&P18111,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6312) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3022403 = {S85632, 145, { &P23137,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I407 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022715 = {S85633, 139, { &P12616,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WT19i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023890 = {S85634, 99, { &P12080,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3653) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3079605 = {S85635, 156, { &P18679,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-B5330 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3082754 = {S85636, 134, { &P12605,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT875 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3076830 = {S85637, 93, { &P14454,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae66-1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3081203 = {S85638, 92, { &P14454,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE66) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3094636 = {S85639, 151, { &P20529,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A89 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3095649 = {S85640, 156, { &P18595,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8190 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA63906 = {S85641, 98, { &P12172,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-J200/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA62965 = {S85642, 87, { &P14705,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN93i-1/10.0.025 SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20192 = {S85643, 49, { &P14174,&P17485,&P3866,&P18092 }};	/*Useragent - SonyEricssonT250i/R5CA005 TelecaBrowser/1.1.14.20*/
const Device UA26743 = {S85644, 66, { &P13627,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GM600/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA48891 = {S85645, 63, { &P15974,&P22839,&P17470,&P18092 }};	/*Useragent - ZTE-C_S265/VE_S265_NBTL25FV1.0.0B17 SMIT-BROWSER/V2.0.0 CTC/1.0*/
const Device UA63939 = {S85646, 98, { &P14560,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-A411/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA572242 = {S85647, 114, { &P14682,&P5843,&P17338,&P18092 }};	/*Useragent - Nokia6303iclassic/5.0 (09.83) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303iclassic/UC Browser7.9.0.102/69/352*/
const Device UA90555 = {S85648, 136, { &P17994,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; N-04C Build/A1010101) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019760 = {S85649, 54, { &P17994,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; N-04C Build/AX)*/
const Device UA90563 = {S85650, 43, { &P16609,&P22784,&P17470,&P18092 }};	/*Useragent - MicromaxX550/MIDP2.0/CLDC1.1/Screen-240X400*/
const Device UA243947 = {S85651, 122, { &P15364,&P5690,&P22408,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.2.0.0 Chrome/13.0.782.218 Safari/535.1*/
const Device UA263379 = {S85652, 45, { &P15364,&P17188,&P5001,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows; U; NT4.0;) NetFront/3.0*/
const Device UA278847 = {S85653, 69, { &P17346,&P17084,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8B117 (iPod; U; CPU OS 4_1 like Mac OS X; en_us)*/
const Device UA388508 = {S85654, 149, { &P23884,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update2; en-; ZiiO7 Build/v1.00.13-ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018458 = {S85655, 151, { &P23884,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update2; en-us; ZiiO7 Build/v1.00.28-ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA643816 = {S85656, 47, { &P15364,&P18570,&P22244,&P18092 }};	/*Useragent - Flipboard/1.8.3 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA561836 = {S85657, 60, { &P16545,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxQ80/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA752490 = {S85658, 103, { &P15364,&P18346,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 4.10; zh-CN; rv:1.9.0.8) Gecko/X Firefox/3.0.8 (.NET CLR 3.5.30729)*/
const Device UA882139 = {S85659, 62, { &P15364,&P7133,&P17207,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:5.0) Gecko/X Firefox/5.0 slurp*/
const Device UA915826 = {S85660, 69, { &P15364,&P7133,&P17207,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:5.0) Gecko/X Firefox/5.0 slurp*/
const Device UA898111 = {S85661, 79, { &P15364,&P17285,&P22359,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:12.1) Gecko/X Firefox/12.1 PaleMoon/12.1*/
const Device UA1978904 = {S85662, 90, { &P15364,&P17017,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (en-US) AppleWebKit/536.8 (KHTML, like Gecko) Chrome/19.0.1084.36 Safari/536.8*/
const Device UA925822 = {S85663, 102, { &P18076,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i667)*/
const Device UA1941400 = {S85664, 33, { &P15364,&P17017,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 Gecko/X Firefox/6.0.2*/
const Device UA1950795 = {S85665, 137, { &P21861,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; Mediacom 810C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2034049 = {S85666, 136, { &P21861,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; SmartPad810c Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2979567 = {S85667, 137, { &P21861,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; Mediacom 810C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422177 = {S85668, 82, { &P17595,&P18207,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; en) Presto/2.10.229 Version/11.61*/
const Device UA1975436 = {S85669, 31, { &P20329,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX261/Q03C MAUI-browser/*/
const Device UA1996759 = {S85670, 70, { &P17595,&P18207,&P21571,&P18092 }};	/*Useragent - Mac OS X/10.8 (12A269); ExchangeWebServices/3.0 (157); Mail/6.0 (1485)*/
const Device UA2000062 = {S85671, 71, { &P17595,&P18207,&P21571,&P18092 }};	/*Useragent - Mac OS X/10.8.1 (12B19); ExchangeWebServices/3.0 (157); Mail/6.0 (1486)*/
const Device UA2021883 = {S85672, 143, { &P21521,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SHW-M290S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2685925 = {S85673, 143, { &P21521,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SHW-M290S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2578200 = {S85674, 139, { &P17976,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-za; Android Tablet PC Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2027227 = {S85675, 140, { &P20381,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-C_N780/N780V1.0.0B06) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028198 = {S85676, 140, { &P20381,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-C_N780/N780V1.0.0B04) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032067 = {S85677, 140, { &P20381,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-C_N780/N780V1.0.0B05) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2575056 = {S85678, 131, { &P20381,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-C N780 Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2028387 = {S85679, 131, { &P20470,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID9742 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2157992 = {S85680, 96, { &P15364,&P5690,&P21780,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.2.0 Safari/534.34*/
const Device UA2262221 = {S85681, 166, { &P13591,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; ZTE-BLADE Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2171632 = {S85682, 33, { &P15803,&P5843,&P21759,&P18092 }};	/*Useragent - iBrowser/Mini2.8 (Nokia200/11.56)*/
const Device UA2380404 = {S85683, 141, { &P22459,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; K-Touch_W610 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2384296 = {S85684, 154, { &P17625,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652881 = {S85685, 154, { &P17625,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; ST25i Build/6.0.B.3.188) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2385299 = {S85686, 123, { &P15118,&P5055,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; NokiaX7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/51/444/UCWEB Mobile*/
const Device UA2421095 = {S85687, 71, { &P21388,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.2; zh-cn; Karbonn_A1; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2433562 = {S85688, 66, { &P15767,&P18432,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 (iPad2,1; iPhone OS 6.0.1; en_US) AppleWebKit/420+*/
const Device UA2454120 = {S85689, 129, { &P24031,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T1004 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2466084 = {S85690, 139, { &P15364,&P18066,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; G1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2467491 = {S85691, 133, { &P23394,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TM-8041HD Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2486815 = {S85692, 135, { &P12605,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; XT875 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2600580 = {S85693, 134, { &P12605,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT875 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2505135 = {S85694, 157, { &P21610,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Explay Surfer 7.02 Build/ICS.g12refM703A1HZ1.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2528945 = {S85695, 158, { &P21610,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3;  Explay Surfer 7.02 Build/ICS.g12refM703A1HZ1.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133  Safari/535.19*/
const Device UA2508291 = {S85696, 38, { &P16463,&P17484,&P20770,&P18092 }};	/*Useragent - SAMSUNG-GT-I8350/UCWEB2.8.0.249/47/355*/
const Device UA2525390 = {S85697, 98, { &P22118,&P5629,&P15760,&P18092 }};	/*Useragent - HS-U850/1.0 Linux/2.6 Android/2.3 Browser/AppleWebKit533.1 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA2933828 = {S85698, 136, { &P22118,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-U850 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2555650 = {S85699, 154, { &P21597,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar Touch Lai 502 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556613 = {S85700, 100, { &P12581,&P5055,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia500) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2597535 = {S85701, 92, { &P12487,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA5700) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2559790 = {S85702, 147, { &P20129,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2561905 = {S85703, 167, { &P18295,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; HTC One X Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2565742 = {S85704, 171, { &P15060,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2660184 = {S85705, 171, { &P15060,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2583321 = {S85706, 138, { &P17324,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9001 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2732533 = {S85707, 138, { &P17324,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-i9001 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2596568 = {S85708, 140, { &P22154,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TM-7026 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 MYI/1.10*/
const Device UA2597973 = {S85709, 148, { &P21517,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-ph; Cloudfone_Excite320e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2599748 = {S85710, 175, { &P18649,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; Micromax A57 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3058016 = {S85711, 179, { &P18649,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A57 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2601522 = {S85712, 141, { &P21526,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cherry w500 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2603861 = {S85713, 135, { &P23193,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCL21 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2625512 = {S85714, 162, { &P22037,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3113, 39, 1.8.2)*/
const Device UA2635338 = {S85715, 164, { &P22037,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3113, 42, 1.9.1)*/
const Device UA2629518 = {S85716, 151, { &P15919,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2631111 = {S85717, 149, { &P15919,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2637044 = {S85718, 151, { &P15919,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2995677 = {S85719, 155, { &P15919,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA2655574 = {S85720, 140, { &P22682,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MITO A322 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658255 = {S85721, 130, { &P24551,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; M2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2838819 = {S85722, 140, { &P24551,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; M2 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2671844 = {S85723, 140, { &P20183,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MI-ONE Plus Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677002 = {S85724, 127, { &P17926,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AT100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2715761 = {S85725, 156, { &P18673,&P18066,&P22281,&P18092 }};	/*Useragent - OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2870695 = {S85726, 170, { &P19868,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; ALCATEL ONE TOUCH 985N Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752841 = {S85727, 92, { &P15206,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE90) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2770731 = {S85728, 130, { &P14908,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6210 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2770728 = {S85729, 132, { &P23171,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; G2S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2771460 = {S85730, 126, { &P19775,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2773727 = {S85731, 26, { &P22597,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SCH-I545/101.40202*/
const Device UA2796995 = {S85732, 143, { &P19862,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Micromax A110Q Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2944321 = {S85733, 153, { &P20175,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI 1S Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3006233 = {S85734, 154, { &P20175,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI 1SC Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2878788 = {S85735, 75, { &P17595,&P21721,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.9.0) Presto/2.12.388 Version/12.15*/
const Device UA2879239 = {S85736, 171, { &P22913,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Karbonn A25 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33756*/
const Device UA2935903 = {S85737, 148, { &P24737,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; STARXTREM Build/GM_ZTEV1.0.0B03) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2944706 = {S85738, 98, { &P21020,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Micromax A116) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2952640 = {S85739, 152, { &P24540,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ALCATEL ONE TOUCH 4005D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2956546 = {S85740, 137, { &P20839,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; H9500S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2983777 = {S85741, 137, { &P20839,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; H9500S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2963978 = {S85742, 161, { &P14969,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; HTC Desire SV Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3002964 = {S85743, 160, { &P14969,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC Desire S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3023733 = {S85744, 137, { &P22954,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E455f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023712 = {S85745, 139, { &P15047,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SK17i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023109 = {S85746, 137, { &P18679,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-B5330 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3059979 = {S85747, 170, { &P22872,&P18111,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5282 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3060257 = {S85748, 92, { &P18241,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; LG-E400) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3083386 = {S85749, 136, { &P18499,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ISW11SC Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081396 = {S85750, 137, { &P21028,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Smartfren Tab 7 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094555 = {S85751, 127, { &P20127,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT270 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3083847 = {S85752, 132, { &P18113,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; XT912 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3078597 = {S85753, 137, { &P16356,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-N7000 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3113085 = {S85754, 137, { &P16356,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-N7000 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3081200 = {S85755, 93, { &P15373,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5530) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3093826 = {S85756, 161, { &P15991,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6102 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3111330 = {S85757, 138, { &P22607,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S5303B Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3115064 = {S85758, 137, { &P22607,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S5303 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA64615 = {S85759, 60, { &P11810,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia3200/1.0 (5.34) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA29379 = {S85760, 67, { &P12257,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-A137/A137UCHG3 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA32497 = {S85761, 12, { &P12491,&P17017,&P17470,&P18092 }};	/*Useragent - V6202/WAP2.0*/
const Device UA62859 = {S85762, 101, { &P12118,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E200/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA49533 = {S85763, 115, { &P15492,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-C3322/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA54448 = {S85764, 40, { &P14215,&P17486,&P17602,&P18092 }};	/*Useragent - MOT-EX122 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA63155 = {S85765, 28, { &P15364,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHF300/1.0 NetFront/3.2*/
const Device UA69649 = {S85766, 68, { &P15364,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHD836/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA86957 = {S85767, 68, { &P15364,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHX836/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88827 = {S85768, 68, { &P15364,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHE496/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87674 = {S85769, 86, { &P17456,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) VZW:SCH-i760 PPC 240x320*/
const Device UA2757868 = {S85770, 83, { &P14218,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-C3300i/C3300iXXKD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA2803957 = {S85771, 83, { &P14218,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-C3300K/C3300KXXJK2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA252264 = {S85772, 164, { &P15364,&P17285,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/15.0.874.106; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)*/
const Device UA254764 = {S85773, 166, { &P15364,&P17285,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/15.0.874.121; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)*/
const Device UA257391 = {S85774, 123, { &P15364,&P17285,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; chromeframe/15.0.874.121; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.04506)*/
const Device UA350460 = {S85775, 185, { &P15364,&P17285,&P21901,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; chromeframe/15.0.874.121; GTB7.2; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729)*/
const Device UA624158 = {S85776, 89, { &P15364,&P17285,&P21901,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/15.0.874.121)*/
const Device UA643025 = {S85777, 96, { &P15364,&P17285,&P21901,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/15.0.874.121)*/
const Device UA663048 = {S85778, 125, { &P15364,&P17285,&P21901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/15.0.874.121) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA709701 = {S85779, 132, { &P15364,&P17285,&P21901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/15.0.874.121) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2*/
const Device UA253799 = {S85780, 106, { &P15364,&P17843,&P17812,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/3.4; Linux) KHTML/3.4.3 (like Gecko) (Kubuntu package 4:3.4.3-0ubuntu1)*/
const Device UA288932 = {S85781, 54, { &P17635,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SN3L UP.Browser/6.2_7.2.7.1.K.4.306 (GUI) MMP/2.0*/
const Device UA305283 = {S85782, 12, { &P16819,&P22784,&P17470,&P18092 }};	/*Useragent - MicromaxX330*/
const Device UA377434 = {S85783, 75, { &P17595,&P23099,&P17317,&P18092 }};	/*Useragent - Safari/6533.21.1 CFNetwork/454.11.12 Darwin/10.7.0 (i386) (MacBookPro5%2C5)*/
const Device UA526352 = {S85784, 73, { &P17595,&P23099,&P17317,&P18092 }};	/*Useragent - Safari/6533.20.27 CFNetwork/454.11.12 Darwin/10.7.0 (i386) (MacBook2%2C1)*/
const Device UA628489 = {S85785, 70, { &P17595,&P23099,&P17317,&P18092 }};	/*Useragent - Safari/6533.21.1 CFNetwork/454.11.12 Darwin/10.7.0 (i386) (iMac10%2C1)*/
const Device UA746774 = {S85786, 75, { &P17595,&P23099,&P17317,&P18092 }};	/*Useragent - Safari/6533.21.1 CFNetwork/454.11.12 Darwin/10.7.0 (i386) (MacBookAir3%2C1)*/
const Device UA752610 = {S85787, 71, { &P17595,&P23099,&P17317,&P18092 }};	/*Useragent - Safari/6533.21.1 CFNetwork/454.11.12 Darwin/10.7.0 (i386) (MacPro4%2C1)*/
const Device UA875717 = {S85788, 72, { &P17595,&P23099,&P17317,&P18092 }};	/*Useragent - Safari/6533.21.1 CFNetwork/454.11.12 Darwin/10.7.0 (i386) (Macmini4%2C1)*/
const Device UA531985 = {S85789, 75, { &P15364,&P7133,&P22345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:2.0.1) Gecko/X Firefox/4.0.6 PaleMoon/4.0.6*/
const Device UA818953 = {S85790, 93, { &P15364,&P7133,&P22345,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.0) Gecko/X Firefox/4.0.5-x64 PaleMoon/4.0.5-x64*/
const Device UA571298 = {S85791, 83, { &P15364,&P5690,&P22311,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:6.0.2) Gecko/X Firefox/6.0.2 SeaMonkey/2.3.3*/
const Device UA875666 = {S85792, 92, { &P15364,&P5690,&P22311,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:6.0.2) Gecko/X Firefox/6.0.2 SeaMonkey/2.3.3 Lightning/1.0b5*/
const Device UA2213676 = {S85793, 62, { &P15364,&P5690,&P22311,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:6.0.2) Gecko/X SeaMonkey/2.3.3*/
const Device UA680653 = {S85794, 43, { &P17595,&P17296,&P21584,&P18092 }};	/*Useragent - MacOutlook/14.2.2.X (Intel Mac OS X 10.6.8)*/
const Device UA2393160 = {S85795, 43, { &P17595,&P17296,&P21584,&P18092 }};	/*Useragent - MacOutlook/14.2.3.X (Intel Mac OS X 10.6.8)*/
const Device UA2457905 = {S85796, 43, { &P17595,&P17296,&P21584,&P18092 }};	/*Useragent - MacOutlook/14.2.5.X (Intel Mac OS X 10.6.8)*/
const Device UA2828324 = {S85797, 43, { &P17595,&P17296,&P21584,&P18092 }};	/*Useragent - MacOutlook/14.2.4.X (Intel Mac OS X 10.6.8)*/
const Device UA761483 = {S85798, 139, { &P21380,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; i-mobile i693 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA895056 = {S85799, 81, { &P18364,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-in; NokiaC5-05)/UC Browser8.4.0.159/50/352/UCWEB Mobile*/
const Device UA924785 = {S85800, 78, { &P17595,&P17297,&P18515,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:14.0) Gecko/X Thunderbird/14.0*/
const Device UA1865803 = {S85801, 92, { &P13894,&P5448,&P4737,&P18092 }};	/*Useragent - SAMSUNG-GT-B6520/1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.2.2.51*/
const Device UA1943916 = {S85802, 59, { &P18088,&P24739,&P18246,&P18092 }};	/*Useragent - KarbonnK15/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA2027250 = {S85803, 180, { &P18510,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; ALCATEL ONE TOUCH 918N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA1979333 = {S85804, 132, { &P24013,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; NEC-102 Build/AX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2421578 = {S85805, 96, { &P17595,&P17297,&P22578,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.7.5; U; Edition Next; en) Presto/2.11.310 Version/12.50*/
const Device UA2028080 = {S85806, 133, { &P22259,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; JY-G1 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2651361 = {S85807, 134, { &P22259,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; JY-G1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2235096 = {S85808, 171, { &P20267,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; P300(Funbook) Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2933516 = {S85809, 141, { &P24543,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GOCLEVER TAB A103 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3085806 = {S85810, 141, { &P24543,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; cs-cz; GOCLEVER TAB A103 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2257501 = {S85811, 142, { &P21489,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Rezound Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2399313 = {S85812, 138, { &P21489,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Rezound Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3120908 = {S85813, 142, { &P21489,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Rezound Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2373355 = {S85814, 139, { &P16436,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I8150 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2726420 = {S85815, 61, { &P16436,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I8150 Build/IMM76L)*/
const Device UA2934293 = {S85816, 139, { &P16436,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I8150 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2410551 = {S85817, 57, { &P15364,&P5629,&P17600,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3) Gecko/X Firefox/8.0*/
const Device UA2428708 = {S85818, 171, { &P18700,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-B5512 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355*/
const Device UA2439527 = {S85819, 89, { &P15364,&P7127,&P21957,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9.2a1pre) Gecko/X Minefield/3.2a1pre*/
const Device UA2457964 = {S85820, 94, { &P12142,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; fa; sonyericssonu5i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2504777 = {S85821, 46, { &P12142,&P17485,&P24673,&P18092 }};	/*Useragent - SonyEricssonU5UCBrowser/8.7.1.234/50/444/UCWEB*/
const Device UA2528252 = {S85822, 94, { &P12142,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; fa; sonyericssonu5i) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2654003 = {S85823, 97, { &P12142,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonu5i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2460944 = {S85824, 192, { &P18639,&P18598,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Nexus 10 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0000, gb*/
const Device UA2706678 = {S85825, 191, { &P18639,&P18598,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Nexus 10 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, +0100, gb*/
const Device UA2482514 = {S85826, 143, { &P20883,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2934222 = {S85827, 144, { &P20883,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 2A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2483180 = {S85828, 158, { &P14551,&P17483,&P24673,&P18092 }};	/*Useragent - LG-GW305/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-gw305) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2486347 = {S85829, 162, { &P18385,&P18598,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 7 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (Nexus 7, 39, 1.8.2)*/
const Device UA2954135 = {S85830, 161, { &P18385,&P18598,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (Nexus 7, 42, 1.9.1)*/
const Device UA2503306 = {S85831, 154, { &P19872,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I847 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2529920 = {S85832, 141, { &P23869,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Advance TV Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708686 = {S85833, 167, { &P16092,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2597487 = {S85834, 138, { &P23471,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; vivo E5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2600182 = {S85835, 90, { &P18449,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; ST27i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2604717 = {S85836, 131, { &P17168,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7310 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2633775 = {S85837, 131, { &P17168,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P7310 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2616213 = {S85838, 186, { &P13452,&P5096,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-us; GT-P1000L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, mx*/
const Device UA2617012 = {S85839, 188, { &P13452,&P5096,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-us; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.7/210,1.9.7.210,2012-10-25 19:53, ve*/
const Device UA2624249 = {S85840, 194, { &P13452,&P5096,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.16/276,1.9.16.276,2012-12-21 16:37, +0800, ph*/
const Device UA2626098 = {S85841, 195, { &P13452,&P5096,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-P1000R Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, -0500, jm*/
const Device UA2626809 = {S85842, 197, { &P13452,&P5096,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pt-br; GT-P1000L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, -0200, br*/
const Device UA2631654 = {S85843, 195, { &P13452,&P5096,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-us; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, -0500, ec*/
const Device UA2636258 = {S85844, 199, { &P13452,&P5096,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.19/285,1.9.19.285,2013-01-24 11:19, +0800, china*/
const Device UA2664034 = {S85845, 92, { &P21022,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LG-E615) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2667651 = {S85846, 136, { &P18747,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; vi-vn; IM-A840S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 MobileSafari/534.30*/
const Device UA2854079 = {S85847, 137, { &P18747,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; IM-A840S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2674841 = {S85848, 138, { &P19848,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M440S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677288 = {S85849, 140, { &P18372,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SO-04D Build/7.0.D.1.137) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707310 = {S85850, 89, { &P17595,&P18207,&P22317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:21.0) Gecko/X Firefox/21.0 SeaMonkey/2.18*/
const Device UA2713935 = {S85851, 139, { &P15147,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST15i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2750216 = {S85852, 141, { &P22811,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ONIDA i101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752016 = {S85853, 170, { &P14682,&P5843,&P24677,&P18092 }};	/*Useragent - Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6303iclassic) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2768824 = {S85854, 147, { &P20801,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; Optimus_San_Remo Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA2769621 = {S85855, 135, { &P17948,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SC-02C Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2772760 = {S85856, 160, { &P18303,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT27i Build/6.1.1.B.1.54) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2773137 = {S85857, 134, { &P23160,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KB901 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2797447 = {S85858, 152, { &P19973,&P5043,&P24677,&P18092 }};	/*Useragent - Nokia114/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia114) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2799330 = {S85859, 146, { &P18466,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2800153 = {S85860, 137, { &P20081,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7108 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879492 = {S85861, 137, { &P20081,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7108 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2830284 = {S85862, 123, { &P15364,&P17017,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; maui) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile /AJSC/1.5*/
const Device UA2889142 = {S85863, 117, { &P15364,&P17017,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-US; sunmicro) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2982350 = {S85864, 116, { &P15364,&P17017,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; generic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA3098781 = {S85865, 116, { &P15364,&P17017,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; karbonn) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2869654 = {S85866, 160, { &P22862,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax A88 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2892820 = {S85867, 178, { &P18604,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Smartphone Android by SFR STARADDICT II Build/IceCreamSandwich) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2894450 = {S85868, 157, { &P18604,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; STARADDICT II Plus Build/GXI_ZTEV1.0.0B03) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879187 = {S85869, 136, { &P21792,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9200 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2933373 = {S85870, 195, { &P18254,&P17820,&P19840,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/12.04 Chromium/18.0.1025.151 Chrome/18.0.1025.151 Safari/535.19 (compatible; PhantomJS/1.5.0; +http://cs.harding.edu)*/
const Device UA2943783 = {S85871, 156, { &P18304,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9070 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2956616 = {S85872, 145, { &P21871,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; POLYTRON_W3430 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3031492 = {S85873, 155, { &P19784,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LIFETAB_P9514 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3023543 = {S85874, 142, { &P21882,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Micromax P362 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3015915 = {S85875, 162, { &P18304,&P5629,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.3.2.300*/
const Device UA3023033 = {S85876, 146, { &P18496,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Incredible 4G LTE Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022268 = {S85877, 90, { &P17625,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; ST25i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3065801 = {S85878, 97, { &P22903,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A62) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3025430 = {S85879, 127, { &P18385,&P21637,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus 7 Build/JWR66V) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.47 Safari/537.22*/
const Device UA3026144 = {S85880, 135, { &P16356,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-N7000 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3029910 = {S85881, 89, { &P20129,&P18111,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; zh-CN; MI 2) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3080167 = {S85882, 90, { &P20129,&P18111,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; zh-CN; MI 2S) U2/1.0.0 UCBrowser/9.2.4.329 U2/1.0.0 Mobile*/
const Device UA3033271 = {S85883, 151, { &P20146,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9082 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3078929 = {S85884, 152, { &P20146,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9082i Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3049631 = {S85885, 128, { &P24400,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SM-T315 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3094114 = {S85886, 136, { &P18438,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E973 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094809 = {S85887, 133, { &P14997,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; S100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080076 = {S85888, 138, { &P15850,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LU6200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094501 = {S85889, 153, { &P20534,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAP4040_DUO Build/PrestigioPAP4040DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083220 = {S85890, 136, { &P24508,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E445 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3115120 = {S85891, 157, { &P20146,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9082 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3115338 = {S85892, 158, { &P23771,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3130937 = {S85893, 129, { &P22922,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; M470BSA Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA9883 = {S85894, 68, { &P13446,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHX820/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA11505 = {S85895, 129, { &P12530,&P5448,&P22270,&P18092 }};	/*Useragent - PPC; 240x320; HTC_P3450/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA88159 = {S85896, 71, { &P12530,&P5448,&P22270,&P18092 }};	/*Useragent - HTC-Touch/Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA44068 = {S85897, 62, { &P15207,&P5448,&P4578,&P18092 }};	/*Useragent - HTC_P3400-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)*/
const Device UA50204 = {S85898, 142, { &P18061,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; google_sdk Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA54731 = {S85899, 134, { &P18061,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-US; sdk Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Vesion/3.1.2 Mobile Safari/525.20.1*/
const Device UA118957 = {S85900, 135, { &P18061,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; sdk Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA54436 = {S85901, 72, { &P14212,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-T108/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA197462 = {S85902, 36, { &P13552,&P5129,&P4777,&P18092 }};	/*Useragent - NokiaC6-00/UC Browser7.7.1.88/50/444*/
const Device UA2212659 = {S85903, 46, { &P13552,&P5129,&P4777,&P18092 }};	/*Useragent - nokiac6-00/UC Browser7.7.1.88/69/444  3gpp-gba*/
const Device UA87821 = {S85904, 115, { &P17346,&P6274,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Mobile/5F137*/
const Device UA87856 = {S85905, 117, { &P17346,&P6274,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 2_1_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Mobile/5F138*/
const Device UA500370 = {S85906, 102, { &P17346,&P6274,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 2_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko)*/
const Device UA692260 = {S85907, 104, { &P17346,&P6274,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 2_1_1 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko)*/
const Device UA249918 = {S85908, 43, { &P14983,&P5055,&P24663,&P18092 }};	/*Useragent - NokiaC7-00/UC Browser7.9.0.102/50/355/UCWEB*/
const Device UA369212 = {S85909, 138, { &P17262,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Kindle Fire Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA448585 = {S85910, 157, { &P20600,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v1.02_14.13-EN-2011.03.25; nl-nl; TAB210 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA544119 = {S85911, 148, { &P20600,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android v1.02_15.14-EN@X; nl-nl; TAB210 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA462572 = {S85912, 135, { &P15364,&P7133,&P22511,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.1.6) Gecko/X FireDownload/2.0.1 Firefox/3.5.6 Wyzo/3.5.6.1 FireTorrent/2.0.1 KAT*/
const Device UA601730 = {S85913, 92, { &P15364,&P7133,&P21807,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.19pre) Gecko/X Prism zdesktop/7.1.4*/
const Device UA628381 = {S85914, 136, { &P21307,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; SH8118U Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA689566 = {S85915, 125, { &P15364,&P17285,&P24199,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.13.13719.201*/
const Device UA733405 = {S85916, 133, { &P19980,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; X903 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022962 = {S85917, 121, { &P19980,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; X903 Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2027376 = {S85918, 126, { &P19980,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; X903 Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA743402 = {S85919, 151, { &P22133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Z710t Build/HTC_U8500L_R2D_EC07) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1921542 = {S85920, 139, { &P22133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC Z710t Build/HTC_U8500L_R2D_EC07) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1973945 = {S85921, 139, { &P22133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Z710t Build/HTC_U8500L_R2D_EC05) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2021030 = {S85922, 99, { &P22133,&P5629,&P15760,&P18092 }};	/*Useragent - HTCZ710t_TD/1.0 Android/2.3 release/6.1.2011 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2025309 = {S85923, 151, { &P22133,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; HTC Z710t Build/HTC_U8500L_R2D_EC05) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA893331 = {S85924, 79, { &P12581,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-in; Nokia500)/UC Browser8.4.0.159/50/352/UCWEB Mobile*/
const Device UA2211478 = {S85925, 122, { &P15364,&P18110,&P22443,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/532.3 (KHTML, like Gecko) Iron/4.0.227.0 Chrome/4.0.227.0 Safari/532.3*/
const Device UA2392387 = {S85926, 131, { &P15364,&P18110,&P22443,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US) AppleWebKit/532.3 (KHTML, like Gecko) Iron/4.0.227.0 Chrome/4.0.227.0 Safari/532.3*/
const Device UA1940649 = {S85927, 37, { &P11854,&P5890,&P17347,&P18092 }};	/*Useragent - nokiae71-1/UC Browser8.0.3.107/69/444*/
const Device UA1955095 = {S85928, 135, { &P12848,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MB526 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2029320 = {S85929, 149, { &P21468,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; AMOI_N807 Build/ALPS.GB2.MP.V1.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033849 = {S85930, 128, { &P16534,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Vega Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2034597 = {S85931, 169, { &P16553,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959V Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2154097 = {S85932, 51, { &P11929,&P5890,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 (Nokia5230;SymbianOS/9.1 Series60/3.0)*/
const Device UA2165190 = {S85933, 143, { &P13036,&P17342,&P17454,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8500L/1.0; U; Bada/2.0; es) AppleWebKit/534.20 (KHTML, like Gecko) Dolfin/3.0 Mobile WVGA SMM-MMS/1.2.0 OPN-B*/
const Device UA2165428 = {S85934, 53, { &P21719,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player sprint_wwe / 1.0 / jewel / 4.0.3*/
const Device UA2579593 = {S85935, 46, { &P21719,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc / 1.0 / jewel / 4.1.1*/
const Device UA2692167 = {S85936, 53, { &P21719,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player sprint_wwe / 1.0 / jewel / 4.0.4*/
const Device UA2223068 = {S85937, 71, { &P20042,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X312/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2371944 = {S85938, 154, { &P15649,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S5570 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2504977 = {S85939, 118, { &P13445,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN85) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2396427 = {S85940, 71, { &P20324,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X331/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2405782 = {S85941, 130, { &P21071,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Novo10 Hero Build/X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2530970 = {S85942, 145, { &P21071,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Novo 10 Hero QuadCore Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2409558 = {S85943, 78, { &P15364,&P6704,&P22304,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win 9x 4.90; en-US; rv:1.8.0.1) Gecko/X SeaMonkey/1.0*/
const Device UA2419631 = {S85944, 130, { &P24536,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VL-110 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2420386 = {S85945, 150, { &P20205,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; OPTIMA Smart Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2423505 = {S85946, 136, { &P20875,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GoTab Epic V Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2424665 = {S85947, 64, { &P20265,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; A52; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2425821 = {S85948, 143, { &P20253,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 5 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2469880 = {S85949, 168, { &P11950,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2428271 = {S85950, 92, { &P16808,&P22784,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxq7) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2653806 = {S85951, 92, { &P16808,&P22784,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxq7) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2429605 = {S85952, 98, { &P13677,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s5253) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2433212 = {S85953, 137, { &P17595,&P18207,&P19963,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Version/6.0.2 Safari/8536.26.17 Sleipnir/3.9.0.2*/
const Device UA2455699 = {S85954, 131, { &P22065,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; SmartTab2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2457774 = {S85955, 163, { &P19968,&P17485,&P24673,&P18092 }};	/*Useragent - SonyEricssonW910i/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonw910i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2482206 = {S85956, 151, { &P20175,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2728191 = {S85957, 152, { &P20175,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1SC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483126 = {S85958, 88, { &P17968,&P18598,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.2.1; Linux; Opera Mobi/ADR-X; U; en) Presto/2.10.254 Version/12.00*/
const Device UA3101625 = {S85959, 91, { &P17968,&P18598,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.2.2; Linux; Opera Mobi/ADR-X; U; en-GB) Presto/2.10.254 Version/12.00*/
const Device UA2507880 = {S85960, 99, { &P14010,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5263) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2526330 = {S85961, 146, { &P21631,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-note WiFi 9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2777459 = {S85962, 75, { &P21631,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; i-mobile i-note WiFi 9 Build/IMM76D)*/
const Device UA2577306 = {S85963, 142, { &P21700,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Penta IS701C Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2585169 = {S85964, 141, { &P21094,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A789 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2596564 = {S85965, 130, { &P24019,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; PC1007 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2748323 = {S85966, 130, { &P24019,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; PC1007 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2603920 = {S85967, 137, { &P22097,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTabS2110AF Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2658191 = {S85968, 136, { &P21379,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SH631W Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2673055 = {S85969, 138, { &P23969,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709237 = {S85970, 146, { &P20090,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848i Duo Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2711691 = {S85971, 145, { &P20090,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848 Duo Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2716573 = {S85972, 138, { &P20244,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; bq Elcano Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2719331 = {S85973, 130, { &P15767,&P17146,&P20782,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.1 Mobile/9A405 Safari/7534.48.3*/
const Device UA2798119 = {S85974, 168, { &P18113,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2748734 = {S85975, 93, { &P11747,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2; en-US; HTC_Desire) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2765139 = {S85976, 95, { &P11747,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; HTC_Desire) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2751752 = {S85977, 169, { &P21475,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A35 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2869923 = {S85978, 93, { &P16356,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7000) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2765587 = {S85979, 141, { &P20361,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Infinix X401 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2768912 = {S85980, 156, { &P14714,&P5843,&P24677,&P18092 }};	/*Useragent - NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax3-02) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2772673 = {S85981, 139, { &P18226,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; U9500 Build/HuaweiU9500) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782352 = {S85982, 136, { &P14969,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Desire S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2862026 = {S85983, 177, { &P15364,&P22887,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.32 (KHTML, like Gecko) Chromium/25.0.1349.2 Chrome/25.0.1349.2 Safari/537.32 Ubuntu/13.04 (3.7.91-1ubuntu1~raring1) Epiphany/3.7.91*/
const Device UA2862213 = {S85984, 179, { &P15364,&P22887,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.32 (KHTML, like Gecko) Chromium/25.0.1349.2 Chrome/25.0.1349.2 Safari/537.32 Ubuntu/13.04 (3.7.91-1ubuntu1~raring1) Epiphany/3.7.91*/
const Device UA2958210 = {S85985, 177, { &P15364,&P22887,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.32 (KHTML, like Gecko) Chromium/25.0.1349.2 Chrome/25.0.1349.2 Safari/537.32 Ubuntu/13.04 (3.8.2-1ubuntu1~raring1) Epiphany/3.8.2*/
const Device UA2864105 = {S85986, 130, { &P21629,&P17979,&P19802,&P18092 }};	/*Useragent - Lenovo-S920/S102 Linux/3.4.0 Android/4.2 Release/12.26.2012 Browser/AppleWebKit534.30 Profile/ Configuration/ Mobile Safari/534.30*/
const Device UA2869790 = {S85987, 160, { &P19998,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2998799 = {S85988, 162, { &P19998,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8812E Build/HuaweiC8812E) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2876100 = {S85989, 158, { &P18451,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LT26ii Build/6.1.A.2.45) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3016837 = {S85990, 158, { &P18451,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT26ii Build/6.1.A.2.55) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2896029 = {S85991, 160, { &P21351,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI C8813 Build/HuaweiC8813) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2958221 = {S85992, 162, { &P21351,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI C8813Q Build/HuaweiC8813Q) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2948595 = {S85993, 119, { &P15364,&P5690,&P17381,&P18091 }};	/*Useragent - Mozilla/5.0 (GobyBot/1.0; Windows; U; Windows NT 5.1; en-US; +http://www.goby.com/; rv:1.8.0.1) Gecko/X Firefox/1.5.0.1*/
const Device UA2963612 = {S85994, 93, { &P18442,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7100) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2997872 = {S85995, 144, { &P12567,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Panasonic P51 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2 Mobile Safari/534.30*/
const Device UA3003691 = {S85996, 135, { &P24387,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; L-05E Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.2 Mobile Safari/534.30*/
const Device UA3122864 = {S85997, 135, { &P24387,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ja-jp; L-05E Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3023379 = {S85998, 140, { &P20183,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MI-ONE Plus Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3059446 = {S85999, 156, { &P18687,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P705 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3108413 = {S86000, 157, { &P18687,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P705g Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3014849 = {S86001, 92, { &P16072,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN78) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3023802 = {S86002, 97, { &P20005,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A65) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3021464 = {S86003, 99, { &P15655,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3850) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3022844 = {S86004, 157, { &P18604,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; STARADDICT II Plus Build/GXI_ZTEV1.0.0B03) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023430 = {S86005, 178, { &P18604,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Smartphone Android by SFR STARADDICT II Build/IceCreamSandwich) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023572 = {S86006, 107, { &P19868,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_985N) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3055661 = {S86007, 107, { &P19868,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_985N) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3081103 = {S86008, 136, { &P24528,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E987 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093495 = {S86009, 140, { &P18465,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Amazon Kindle Fire Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3081285 = {S86010, 137, { &P20240,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Optimus G Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3063885 = {S86011, 132, { &P20529,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A89 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3113029 = {S86012, 143, { &P23402,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; BLU STUDIO 5.0 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126390 = {S86013, 140, { &P23402,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Studio 5.0s Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094701 = {S86014, 159, { &P22209,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Titanium S5 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3125953 = {S86015, 133, { &P23836,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-ca; 801a Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3131313 = {S86016, 137, { &P23642,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-M840 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA24626 = {S86017, 98, { &P13548,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-L770/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA33267 = {S86018, 120, { &P13548,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-L770-Vodafone/L770BUHE1 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA53311 = {S86019, 107, { &P13548,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-L770-ORANGE/BVIG2 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA13663 = {S86020, 67, { &P13509,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GC900F/V10d Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA25393 = {S86021, 107, { &P12111,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-A256/A256UXIF8 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA48889 = {S86022, 123, { &P12884,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; zh-tw; YL-Coolpad_D530TW/1.01.012.100927.D530TW; 320*480; CTC/2.0) CoolpadWebkit/530.18*/
const Device UA560956 = {S86023, 118, { &P12884,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; zh-tw; YL-Coolpad_D530TW/1.01.012.X.D530TW; 320*480; CTC/2.0) CoolpadWebkit/530.18*/
const Device UA2020570 = {S86024, 118, { &P12884,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; D530 Build/ERD79) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2020667 = {S86025, 132, { &P12884,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; D530 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA56317 = {S86026, 54, { &P17567,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SH3D UP.Browser/6.2_7.2.7.1.K.4.306 (GUI) MMP/2.0*/
const Device UA87002 = {S86027, 143, { &P17346,&P17129,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 2_0 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5A345 Safari/525.20*/
const Device UA2265673 = {S86028, 143, { &P17346,&P17129,&P4571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 2_0 like Mac OS X; de-de) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5A347 Safari/525.20*/
const Device UA87905 = {S86029, 24, { &P18088,&P19949,&P17470,&P18092 }};	/*Useragent - NEC-N600/1.0 HopenOS/2.0*/
const Device UA264136 = {S86030, 55, { &P12300,&P5448,&P3430,&P18092 }};	/*Useragent - HTC-P4600/5.11.502.2 Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA265029 = {S86031, 48, { &P12300,&P5448,&P3430,&P18092 }};	/*Useragent - HTC-P4600/1.2 Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA300073 = {S86032, 32, { &P17743,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P07A3(c500;TB;W24H15)*/
const Device UA454805 = {S86033, 75, { &P15364,&P22740,&P18186,&P18092 }};	/*Useragent - Mozilla/5.0 (Maemo; Linux armv7l; rv:11.0) Gecko/X Firefox/11.0 Fennec/11.0*/
const Device UA599813 = {S86034, 90, { &P17976,&P6545,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.0.1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00*/
const Device UA2775703 = {S86035, 90, { &P17976,&P6545,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.0UI; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00*/
const Device UA2889104 = {S86036, 88, { &P17976,&P6545,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.0; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00*/
const Device UA634279 = {S86037, 80, { &P17595,&P17295,&P18367,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:12.0) Gecko/X Thunderbird/12.0.1*/
const Device UA662989 = {S86038, 32, { &P15364,&P17017,&P24663,&P18092 }};	/*Useragent - UC Browser7.9.0.102/50/355/UCWEB*/
const Device UA1949834 = {S86039, 32, { &P15364,&P17017,&P24663,&P18092 }};	/*Useragent - UC Browser7.9.1.120/27/352/UCWEB*/
const Device UA870169 = {S86040, 137, { &P17716,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; LG-LU3700 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA898586 = {S86041, 81, { &P13324,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaN8-00)/UC Browser8.4.0.159/50/352/UCWEB Mobile*/
const Device UA1909906 = {S86042, 121, { &P19978,&P17979,&P3508,&P18092 }};	/*Useragent - SAMSUNG-GT-I9018_TD/1.0 Android/2.2.2 Release/12.15.2010 Browser/AppleWebKit533.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2026068 = {S86043, 137, { &P23990,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; SCH-P739 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2981305 = {S86044, 130, { &P23990,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; SCH-P739 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2026166 = {S86045, 140, { &P23085,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_C8860E/C8860EV100R001C92B892; 480*854; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2685928 = {S86046, 141, { &P23085,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8860E Build/HuaweiC8860E) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2817029 = {S86047, 140, { &P23085,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_C8860E/C8860EV100R001C92B871; 480*854; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA3099320 = {S86048, 140, { &P23085,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_C8860E/C8860EV100R001C92B897; 480*854; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2426127 = {S86049, 150, { &P18511,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; URBANO PROGRESSO Build/011.0.3100) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2154039 = {S86050, 79, { &P12456,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian73-1) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2245626 = {S86051, 69, { &P22667,&P7133,&P17193,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/3.1; Xbox)*/
const Device UA2257385 = {S86052, 133, { &P21388,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; A1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2817620 = {S86053, 126, { &P21388,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2382925 = {S86054, 116, { &P12473,&P5204,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2730c-1/10.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2400615 = {S86055, 132, { &P21501,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; P5 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2409463 = {S86056, 78, { &P15364,&P22308,&P19772,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; WinNT3.51; en-US; rv:1.8.1.8) Gecko/X SeaMonkey/1.1.5*/
const Device UA2411215 = {S86057, 116, { &P18088,&P7133,&P17370,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/5.0 (Windows NT 6.1; WOW64) AppleWebKit/23.411; U; en) Presto/2.5.25 Version/10.54*/
const Device UA2413371 = {S86058, 117, { &P13914,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia5330-1d/06.88; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA2432523 = {S86059, 88, { &P18095,&P18109,&P18395,&P18092 }};	/*Useragent - Opera/9.80 (Series 60; Opera Mini/6.6.29184/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2526568 = {S86060, 117, { &P14545,&P17483,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; lg-x330t) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile*/
const Device UA2469499 = {S86061, 91, { &P15209,&P5768,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia6233) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2579511 = {S86062, 155, { &P15209,&P5768,&P24675,&P18092 }};	/*Useragent - Nokia6233/2.0 (05.60) Profile/MIDP-2.0 Configuration/CLDC-1.1 , UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia6233) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2504941 = {S86063, 151, { &P19973,&P5043,&P24673,&P18092 }};	/*Useragent - Nokia114/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia114) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2506015 = {S86064, 138, { &P21544,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V887 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2506598 = {S86065, 146, { &P18554,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2527249 = {S86066, 138, { &P21300,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenco TAB-1014 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2557681 = {S86067, 137, { &P23340,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; DJC Touchtab3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2583277 = {S86068, 141, { &P21155,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A800 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2656091 = {S86069, 160, { &P16356,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T879 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2819487 = {S86070, 160, { &P16356,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7000 Build/JRO03L; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2669541 = {S86071, 139, { &P20737,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730G Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2673235 = {S86072, 140, { &P21402,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Lenovo S890 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673569 = {S86073, 137, { &P20508,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cynus T2 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676656 = {S86074, 139, { &P13054,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ADR6425LVW Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2706028 = {S86075, 87, { &P23354,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A3) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2707963 = {S86076, 138, { &P22952,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F260S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2731902 = {S86077, 162, { &P11698,&P17485,&P24676,&P18092 }};	/*Useragent - SonyEricssonW995/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw995) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2735853 = {S86078, 152, { &P22960,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030D Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2750865 = {S86079, 93, { &P15575,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9100) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2752628 = {S86080, 137, { &P22599,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300S Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2753365 = {S86081, 158, { &P21020,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; Micromax A116 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2755383 = {S86082, 162, { &P16436,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-I8150 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453*/
const Device UA2755501 = {S86083, 160, { &P12798,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.4; LT18i Build/4.0.2.A.0.62) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453*/
const Device UA2781390 = {S86084, 132, { &P18437,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; fr-fr; GT-N8000 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2781687 = {S86085, 154, { &P20887,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; e1811_v77_gq2000_ov5647 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2796348 = {S86086, 134, { &P18471,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT907 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2796695 = {S86087, 151, { &P18283,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2799266 = {S86088, 160, { &P18324,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.7; LG-VM696 Build/ZV5.GWK74) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2861171 = {S86089, 133, { &P14202,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; X8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867799 = {S86090, 152, { &P22964,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892744 = {S86091, 159, { &P22964,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL_ONE_TOUCH_6010X_Orange Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868802 = {S86092, 145, { &P23615,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-au; PNDPP41DG7BLK^ Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936790 = {S86093, 140, { &P24692,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Spice MI-353 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3059976 = {S86094, 159, { &P15972,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One XL Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3022020 = {S86095, 144, { &P18512,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT881 Build/6.7.3-130_YUS-11) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022830 = {S86096, 141, { &P23254,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Celkon A119Q Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3018367 = {S86097, 142, { &P12280,&P18432,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 UCBrowser/9.1.0.287 Mobile*/
const Device UA3020486 = {S86098, 142, { &P12280,&P18432,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 UCBrowser/9.1.0.287 Mobile*/
const Device UA3091925 = {S86099, 142, { &P12280,&P18432,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 UCBrowser/9.1.0.287 Mobile*/
const Device UA3022505 = {S86100, 135, { &P21448,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SH530U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024150 = {S86101, 135, { &P23094,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; F-06E Build/V21R48D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3063766 = {S86102, 159, { &P17625,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST25i Build/6.1.1.B.1.10) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3049176 = {S86103, 127, { &P24624,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; A1-811 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3097276 = {S86104, 150, { &P24078,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; E3 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA62622 = {S86105, 61, { &P11970,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia6126/2.0 (04.28) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23329 = {S86106, 73, { &P15588,&P6476,&P3976,&P18092 }};	/*Useragent - SAMSUNG-GT-M2510/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23658 = {S86107, 77, { &P15453,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Sprint MP6900SP*/
const Device UA26789 = {S86108, 22, { &P13634,&P17483,&P17470,&P18092 }};	/*Useragent - LG-GS500/V100 OBIGO/Q7*/
const Device UA135933 = {S86109, 31, { &P13432,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F01A(c100;TB;W24H17)*/
const Device UA55244 = {S86110, 46, { &P14303,&P6476,&P3625,&P18092 }};	/*Useragent - TXT8010/T05_0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA88432 = {S86111, 8, { &P13554,&P5890,&P4030,&P18092 }};	/*Useragent - NOKIAN79*/
const Device UA88881 = {S86112, 69, { &P12996,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-T90/1.0 ACS-NF/3.4 QTV2.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA89942 = {S86113, 31, { &P17736,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N07B(c500;TB;W24H16)*/
const Device UA396152 = {S86114, 54, { &P17875,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3U UP.Browser/6.2_7.2.7.1.K.5.207 (GUI) MMP/2.0*/
const Device UA159308 = {S86115, 32, { &P17737,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N905i(c100;TB;W24H16)*/
const Device UA496940 = {S86116, 35, { &P17737,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N905imyu(c100;TB;W24H16)*/
const Device UA203023 = {S86117, 122, { &P15364,&P17285,&P18209,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.0.3.2197 Safari/532.4*/
const Device UA1963406 = {S86118, 82, { &P17595,&P17297,&P18191,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:10.0.1) Gecko/X Thunderbird/10.0.1*/
const Device UA601459 = {S86119, 86, { &P15364,&P7133,&P22314,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:11.0) Gecko/X Firefox/11.0 SeaMonkey/2.8 Lightning/1.3*/
const Device UA685726 = {S86120, 74, { &P21396,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; Lenovo_A65; 320*480) UCWEB7.9.3.103/139/32458*/
const Device UA691096 = {S86121, 27, { &P15364,&P17350,&P4811,&P18092 }};	/*Useragent - Opera/9.02 (Windows; U; nl)*/
const Device UA2409414 = {S86122, 23, { &P15364,&P17350,&P4811,&P18092 }};	/*Useragent - Opera/9.00 (Windows; U)*/
const Device UA692716 = {S86123, 68, { &P13094,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux; U;2.3.3;zh_cn;Lenovo A60;320*480;)UCWEB7.6.0.75/139/32399*/
const Device UA827262 = {S86124, 159, { &P16036,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus One Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031936 = {S86125, 159, { &P16036,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus One Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1950354 = {S86126, 86, { &P12351,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s3653w) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2373438 = {S86127, 86, { &P12351,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s3653w) UCBrowser8.3.1.161/70/465/UCWEB Mobile*/
const Device UA1987699 = {S86128, 155, { &P12787,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; ALCATEL_one_touch_908 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3086717 = {S86129, 155, { &P12787,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; el-gr; ALCATEL ONE TOUCH 908 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2009121 = {S86130, 122, { &P15364,&P17285,&P22434,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Comodo_Dragon/21.0.2.0*/
const Device UA2009138 = {S86131, 129, { &P15364,&P17285,&P22434,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Comodo_Dragon/21.0.2.0*/
const Device UA2029291 = {S86132, 138, { &P20075,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xperia S Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2032288 = {S86133, 131, { &P21157,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MID114C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2160947 = {S86134, 148, { &P21335,&P18110,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; YunOs 1.0.0.3; zh-cn; K-Touch W619 Build/AliyunOs-2012) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2201264 = {S86135, 112, { &P15364,&P7133,&P18427,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4 slurp*/
const Device UA2231817 = {S86136, 112, { &P15364,&P7133,&P18427,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 slurp*/
const Device UA2354830 = {S86137, 105, { &P15364,&P7133,&P18427,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4 slurp*/
const Device UA2455906 = {S86138, 161, { &P20265,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A52 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2422350 = {S86139, 136, { &P20135,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Enjoy 7 Plus Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2424346 = {S86140, 135, { &P22688,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; eZee'Tab705 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2433176 = {S86141, 37, { &P18623,&P5448,&P20765,&P18092 }};	/*Useragent - NOKIA-Lumia 510/UCWEB2.7.0.223/47/355*/
const Device UA2467310 = {S86142, 143, { &P15364,&P17350,&P19965,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; ; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.3.1.31155 (Firestorm-Release; firestorm skin) Safari/533.3*/
const Device UA2506703 = {S86143, 162, { &P13308,&P17485,&P24675,&P18092 }};	/*Useragent - SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonj105i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2527512 = {S86144, 136, { &P22015,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; M9108 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2537479 = {S86145, 50, { &P15364,&P18580,&P22327,&P18092 }};	/*Useragent - Rockmelt/2.2.0.241 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2555055 = {S86146, 51, { &P21742,&P22846,&P17470,&P18092 }};	/*Useragent - Lemon_T109; Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2556157 = {S86147, 142, { &P11833,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  SonyEricssonP1i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2598744 = {S86148, 70, { &P17346,&P17146,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPod4,1; iPhone OS 5.0.1; en_US; en) AppleWebKit/420+*/
const Device UA2564370 = {S86149, 137, { &P22945,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; OnePAD 1100x2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2580949 = {S86150, 93, { &P16436,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I8150) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2588346 = {S86151, 89, { &P15364,&P5448,&P21957,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows CE 6.0; en-US; rv:1.9.2a1pre) Gecko/X Minefield/3.2a1pre*/
const Device UA2596711 = {S86152, 154, { &P12784,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Skate Build/JRO03C; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2988274 = {S86153, 154, { &P12784,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; Skate Build/JRO03C; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2598776 = {S86154, 94, { &P13591,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; ZTE-BLADE) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2599111 = {S86155, 90, { &P18223,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT22i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2601115 = {S86156, 159, { &P20183,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI-ONE Plus Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2602191 = {S86157, 133, { &P23238,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TOUCH102B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2623099 = {S86158, 192, { &P17168,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7310 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0000, gb*/
const Device UA2660169 = {S86159, 93, { &P12568,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; GT-B7510) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2676587 = {S86160, 138, { &P15850,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LU6200 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676639 = {S86161, 140, { &P21155,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A800 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2699599 = {S86162, 93, { &P12061,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae65-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2738035 = {S86163, 92, { &P12061,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE65) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2703040 = {S86164, 166, { &P20515,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; A54 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3058056 = {S86165, 170, { &P20515,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; A54 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2706999 = {S86166, 134, { &P21869,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZP950 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709134 = {S86167, 135, { &P21869,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZP950+ Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714869 = {S86168, 99, { &P15655,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3850) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2732108 = {S86169, 147, { &P22944,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Starmobile FLIRT Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2733219 = {S86170, 139, { &P19902,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530M Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2764068 = {S86171, 140, { &P19902,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SCH-R530M Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752369 = {S86172, 150, { &P21261,&P18598,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9500 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2752592 = {S86173, 92, { &P12671,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia700) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2755404 = {S86174, 140, { &P21181,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S560 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769371 = {S86175, 133, { &P20097,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CT1010 Build/IMM76I.v007) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2770935 = {S86176, 138, { &P21507,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C1605 Build/11.3.A.2.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2771276 = {S86177, 62, { &P15364,&P5690,&P22768,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:22.0) Gecko/X Thunderbird/22.0*/
const Device UA2772870 = {S86178, 132, { &P20807,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B92 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783943 = {S86179, 140, { &P21094,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A789 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799009 = {S86180, 144, { &P18554,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2896090 = {S86181, 132, { &P18554,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; BN Nook HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2799791 = {S86182, 138, { &P23243,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; FPT F11 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2861360 = {S86183, 146, { &P23077,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; @TAB-SEVEN-X1-B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2889991 = {S86184, 154, { &P24562,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar Touch Kem 351 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049316 = {S86185, 152, { &P24562,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-; Mobiistar Touch Kem 351 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2893304 = {S86186, 99, { &P12080,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3653) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2962129 = {S86187, 155, { &P18621,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; LG-E612 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3020719 = {S86188, 140, { &P24096,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Orange Kivo Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3016608 = {S86189, 167, { &P15877,&P22211,&P19802,&P18091 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT910 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/umts_spyder*/
const Device UA3023194 = {S86190, 135, { &P21853,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Connect-3G-2.0 Build/HCL) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3024139 = {S86191, 136, { &P18663,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SH-10D Build/S7080) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3061302 = {S86192, 155, { &P22874,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; GT-I9502 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3058103 = {S86193, 144, { &P24378,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Lenovo P780_ROW Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3059197 = {S86194, 103, { &P13324,&P5055,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian8-00) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3094788 = {S86195, 148, { &P15201,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Sensation Z710e Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094768 = {S86196, 145, { &P21489,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Infected Rezound Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3061929 = {S86197, 140, { &P19846,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-05D Build/9.1.C.0.475) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3083666 = {S86198, 140, { &P20183,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MI-ONE Plus Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083371 = {S86199, 136, { &P20226,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-E975 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065192 = {S86200, 141, { &P22858,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U8836D Build/HuaweiU8836D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3094445 = {S86201, 136, { &P20240,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Optimus G Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093376 = {S86202, 93, { &P13974,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5670) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3081642 = {S86203, 94, { &P11907,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE52-1) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3109072 = {S86204, 168, { &P12280,&P22882,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 7_0 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5*/
const Device UA3130774 = {S86205, 170, { &P12280,&P22882,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 7_0_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C134 NetNanny Mobile/1.0 Safari/6533.18.5*/
const Device UA62940 = {S86206, 75, { &P11885,&P5530,&P4836,&P18092 }};	/*Useragent - MOT-Z3/08.02.08R MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/1.0*/
const Device UA36762 = {S86207, 143, { &P12178,&P17486,&P17671,&P18092 }};	/*Useragent - MOT-MOTOROKREM30_CMCC/1.0 LinuxOS/2.6.10 Release/07.18.2008 Browser/OSS1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R6716_G_71.04.06R_B*/
const Device UA49103 = {S86208, 138, { &P12178,&P17486,&P17671,&P18092 }};	/*Useragent - MOT-MOTOROKREM30/1.0 LinuxOS/2.6.10 Release/07.18.2008 Browser/OSS1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R6716_G_71.01.23R_A*/
const Device UA18676 = {S86209, 52, { &P13891,&P17483,&P17660,&P18092 }};	/*Useragent - LGKU380/v1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49059 = {S86210, 53, { &P13891,&P17483,&P17660,&P18092 }};	/*Useragent - LG/KU380/v10c Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54196 = {S86211, 53, { &P13891,&P17483,&P17660,&P18092 }};	/*Useragent - LG-KU380/v1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA32935 = {S86212, 133, { &P12863,&P17483,&P3459,&P18092 }};	/*Useragent - Mozilla/4.0 (Vodafone/1.0/LG-GU280/v10b Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA33139 = {S86213, 127, { &P12863,&P17483,&P3459,&P18092 }};	/*Useragent - LG-GU280f Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA33422 = {S86214, 106, { &P12863,&P17483,&P3459,&P18092 }};	/*Useragent - LG-GU280/v09a Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA48021 = {S86215, 133, { &P12863,&P17483,&P3459,&P18092 }};	/*Useragent - Mozilla/4.0 (Vodafone/1.0/LG-GU280/v10a Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA59884 = {S86216, 113, { &P12863,&P17483,&P3459,&P18092 }};	/*Useragent - LG-GU280-Orange/v10e Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA30905 = {S86217, 73, { &P12367,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S5150/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA32936 = {S86218, 136, { &P12797,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; SCH-i579 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40284 = {S86219, 118, { &P14967,&P5096,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.2.2;Zh_cn;LG-P500;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355*/
const Device UA40290 = {S86220, 116, { &P14967,&P5096,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.2;Zh_cn;LG-P500;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355*/
const Device UA833097 = {S86221, 69, { &P14967,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2.2;Zh_cn;LG-P500;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA2020405 = {S86222, 67, { &P14967,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2;Zh_cn;LG-P500;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA2020696 = {S86223, 69, { &P14967,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2.1;Zh_cn;LG-P500;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA41280 = {S86224, 121, { &P14540,&P5843,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-01/08.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.2.26.6*/
const Device UA63769 = {S86225, 39, { &P18058,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GM205 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA68755 = {S86226, 34, { &P13977,&P5890,&P4777,&P18092 }};	/*Useragent - NokiaN73/UC Browser7.7.1.88/28/444*/
const Device UA68745 = {S86227, 36, { &P13324,&P5055,&P4777,&P18092 }};	/*Useragent - NokiaN8-00/UC Browser7.7.1.88/50/444*/
const Device UA656154 = {S86228, 46, { &P13324,&P5055,&P4777,&P18092 }};	/*Useragent - nokian8-00/UC Browser7.7.1.88/69/444  3gpp-gba*/
const Device UA69473 = {S86229, 29, { &P14454,&P5890,&P24663,&P18092 }};	/*Useragent - NokiaE66/UCWEB7.9.0.94/28/999*/
const Device UA379664 = {S86230, 41, { &P14454,&P5890,&P24663,&P18092 }};	/*Useragent - NokiaE66/UC Browser7.9.0.102/28/355/UCWEB*/
const Device UA252034 = {S86231, 76, { &P15364,&P18140,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; Linux Mint; en) Presto/2.9.168 Version/11.52*/
const Device UA307863 = {S86232, 140, { &P15364,&P17285,&P22401,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Comodo_Dragon/9.0.0.3 Chrome/9.0.597.98 Safari/534.13*/
const Device UA667527 = {S86233, 98, { &P12108,&P5843,&P17347,&P18092 }};	/*Useragent - Nokia7230/5.0 (06.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7230/UC Browser8.0.3.107/69/444*/
const Device UA574468 = {S86234, 65, { &P14726,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;LG;LG-C900;7.10;7720)*/
const Device UA589133 = {S86235, 117, { &P17595,&P17297,&P23600,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.53.11 (KHTML, like Gecko) Fluid/0.9.6 Safari/534.53.11*/
const Device UA746112 = {S86236, 115, { &P17595,&P17297,&P23600,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) Fluid/0.9.6 Safari/534.56.5*/
const Device UA2382887 = {S86237, 115, { &P17595,&P17297,&P23600,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Fluid/0.9.6 Safari/534.57.7*/
const Device UA613764 = {S86238, 88, { &P17595,&P17297,&P21171,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:12.0) Gecko/X Firefox/12.0 SeaMonkey/2.9*/
const Device UA747525 = {S86239, 121, { &P14682,&P5843,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia6303iclassic/07.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA905711 = {S86240, 169, { &P13018,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; MT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2499179 = {S86241, 85, { &P14182,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s5333) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA1958543 = {S86242, 74, { &P18720,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;Micromax A50;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA2217743 = {S86243, 146, { &P12135,&P17483,&P24667,&P18092 }};	/*Useragent - LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-gd510) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2032883 = {S86244, 156, { &P14202,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; E15i Build/IMM76I; CyanogenMod-MiniCM9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2338032 = {S86245, 156, { &P14202,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; E15i Build/IMM76L; CyanogenMod-MiniCM9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2172434 = {S86246, 37, { &P14540,&P5843,&P23405,&P18092 }};	/*Useragent - MQQBrowser/Mini2.8 (NokiaX2-01/08.63)*/
const Device UA2242384 = {S86247, 87, { &P17595,&P18207,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Macintosh;Mac OS X (10.8.2);en;kml:2.2;client:Free;type:default)*/
const Device UA2291380 = {S86248, 87, { &P18088,&P17017,&P24515,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/9.80 (J2ME/22.478; U; en) Presto/2.5.25 Version/10.54*/
const Device UA2335788 = {S86249, 121, { &P14557,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia6303classic/10.12; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA2384650 = {S86250, 139, { &P18474,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9210T Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2454896 = {S86251, 139, { &P18474,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-I9210T Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2420167 = {S86252, 129, { &P21975,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TM901 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2747249 = {S86253, 129, { &P21975,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TM901 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2420874 = {S86254, 147, { &P20521,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; Artes D821 Build/ICS.g12refM805AST.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422035 = {S86255, 158, { &P13634,&P17483,&P24673,&P18092 }};	/*Useragent - LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-gs500) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2426538 = {S86256, 152, { &P18283,&P18111,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9300 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2846754 = {S86257, 152, { &P18283,&P18111,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9300 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2470752 = {S86258, 162, { &P12765,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2428285 = {S86259, 98, { &P15547,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-c6712) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2444432 = {S86260, 143, { &P17952,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3042949 = {S86261, 143, { &P17952,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2458827 = {S86262, 157, { &P15364,&P17285,&P19965,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.3.1.31155 (Firestorm-Release; firestorm skin) Safari/533.3*/
const Device UA2459603 = {S86263, 161, { &P11953,&P17485,&P24673,&P18092 }};	/*Useragent - SonyEricssonU10i/R7BA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonu10i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2466443 = {S86264, 139, { &P15850,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LU6200 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2761534 = {S86265, 139, { &P15850,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LG-LU6200 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2470405 = {S86266, 131, { &P20742,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TOUCH87 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2487336 = {S86267, 137, { &P14017,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; LG-P990 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2603023 = {S86268, 137, { &P14017,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-P990 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2505882 = {S86269, 130, { &P21076,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; MID801 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2558201 = {S86270, 164, { &P14967,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2706570 = {S86271, 166, { &P19891,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2580674 = {S86272, 161, { &P17968,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MID Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2852405 = {S86273, 172, { &P17968,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; INFINITY_POWER Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2708883 = {S86274, 135, { &P21878,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; 201F Build/V33R12A) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2603906 = {S86275, 138, { &P18674,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7005 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2658836 = {S86276, 58, { &P15364,&P22308,&P17191,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; WinNT3.51; Trident/1.0)*/
const Device UA2659128 = {S86277, 90, { &P18448,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST26i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2659673 = {S86278, 90, { &P15047,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; SK17i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2661990 = {S86279, 93, { &P18700,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-B5512) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2676305 = {S86280, 128, { &P23873,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BC1003 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2695610 = {S86281, 157, { &P18303,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xperia Sola Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2698845 = {S86282, 136, { &P20462,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; PTL21 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752692 = {S86283, 159, { &P20183,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI-ONE Plus Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752644 = {S86284, 154, { &P16049,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LG-P698 Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2758984 = {S86285, 130, { &P18437,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-N8000 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2762737 = {S86286, 104, { &P11684,&P5129,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia5800d-1/60.0.003/sw_platform=S60;sw_platform_version=5.0;java_build_version=1.4.43)*/
const Device UA2769941 = {S86287, 137, { &P18637,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9260 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2793872 = {S86288, 174, { &P18317,&P18598,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ASUS Transformer Pad TF300T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA2860818 = {S86289, 170, { &P18317,&P18598,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ASUS Transformer Pad TF300T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2794106 = {S86290, 97, { &P22623,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; Micromax_A51) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2867174 = {S86291, 95, { &P15364,&P17017,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; jbed-fastbcc) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2892902 = {S86292, 88, { &P15364,&P17017,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; spice) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA3072896 = {S86293, 91, { &P15364,&P17017,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sunmicro) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2867477 = {S86294, 130, { &P23721,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CMP749 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2944035 = {S86295, 141, { &P20089,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-01E Build/9.1.C.0.475) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2874504 = {S86296, 162, { &P19862,&P18598,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Micromax A110Q Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2878808 = {S86297, 152, { &P19767,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ALCATEL ONE TOUCH 8008D Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2892242 = {S86298, 138, { &P24131,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Tab700M Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892335 = {S86299, 155, { &P18125,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3121578 = {S86300, 157, { &P18125,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID4 Build/9.8.2O-72_VZW-18-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2954735 = {S86301, 148, { &P24045,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-ph; MyPhone A919i Duo Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2992748 = {S86302, 132, { &P20463,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; PMP7100D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3020854 = {S86303, 152, { &P23175,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5021E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3062420 = {S86304, 155, { &P18621,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; LG-E612 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3079391 = {S86305, 155, { &P20705,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; HTC One Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3064592 = {S86306, 137, { &P19856,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-LS970 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3081336 = {S86307, 133, { &P21642,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NXM805ND_EN Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3095636 = {S86308, 157, { &P18296,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC One V Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3115503 = {S86309, 165, { &P12657,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515m Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA2683 = {S86310, 73, { &P12034,&P5810,&P3417,&P18092 }};	/*Useragent - BlackBerry7130/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/179*/
const Device UA21746 = {S86311, 39, { &P15333,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KF600 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA49939 = {S86312, 40, { &P15333,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KF600d Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA23733 = {S86313, 11, { &P15621,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M5700*/
const Device UA49137 = {S86314, 94, { &P13213,&P17603,&P3534,&P18092 }};	/*Useragent - PantechP2000/JGUS03292010 Browser/Obigo/Q05A OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA68893 = {S86315, 118, { &P11669,&P5776,&P4777,&P18092 }};	/*Useragent - Nokia7610/2.0 (7.0642.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.7.1.88/27/444*/
const Device UA69357 = {S86316, 34, { &P11854,&P5890,&P4779,&P18092 }};	/*Useragent - NokiaE71/UC Browser7.8.0.95/28/355*/
const Device UA89089 = {S86317, 64, { &P16444,&P5448,&P3430,&P18092 }};	/*Useragent - HTC-ST6356/2.31.502.4 (66023) Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA201692 = {S86318, 141, { &P16760,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Spice Mi-270 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019411 = {S86319, 65, { &P16760,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; Spice Mi-270 Build/MASTER)*/
const Device UA2021202 = {S86320, 129, { &P16760,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Spice Mi-270 Build/MASTER) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA208379 = {S86321, 87, { &P17595,&P17296,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Macintosh;Mac OS X (10.6.8);en;kml:2.2;client:Free;type:default)*/
const Device UA2138993 = {S86322, 87, { &P17595,&P17296,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.1.2032(Macintosh;Mac OS X (10.6.8);en;kml:2.2;client:Free;type:default)*/
const Device UA294975 = {S86323, 31, { &P17738,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P01B(c500;TB;W24H16)*/
const Device UA407883 = {S86324, 132, { &P11758,&P6909,&P18245,&P18089 }};	/*Useragent - Nokia6820/2.0 (4.83) Profile/MIDP-1.0 Configuration/CLDC-1.0 (compatible; Mediapartners-Google/2.1; +http://www.google.com/bot.html)*/
const Device UA430018 = {S86325, 75, { &P15364,&P17017,&P24473,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/4.1; DragonFly) KHTML/4.1.4 (like Gecko)*/
const Device UA550018 = {S86326, 118, { &P15364,&P7127,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.2.149.27 Safari/525.13*/
const Device UA561980 = {S86327, 83, { &P11734,&P5448,&P4779,&P18092 }};	/*Useragent - HTC_TyTN_II Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA564280 = {S86328, 81, { &P11734,&P5448,&P4779,&P18092 }};	/*Useragent - HTC_P4550 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA738439 = {S86329, 121, { &P16287,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; 8810 Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2028280 = {S86330, 126, { &P16287,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; 8810 Build/FRG83G) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2033536 = {S86331, 133, { &P16287,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; 8810 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2546523 = {S86332, 126, { &P16287,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.2.2; zh-cn; 8810 Build FRG83G) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA903877 = {S86333, 93, { &P17595,&P17296,&P21170,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:13.0) Gecko/X Firefox/13.0.1 SeaMonkey/2.10.1*/
const Device UA1993969 = {S86334, 135, { &P23426,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; TM-5200 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2001280 = {S86335, 122, { &P15364,&P17285,&P22436,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 Comodo_Dragon/21.1.1.0*/
const Device UA2001864 = {S86336, 129, { &P15364,&P17285,&P22436,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 Comodo_Dragon/21.1.1.0*/
const Device UA2442375 = {S86337, 68, { &P15985,&P6437,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android3.2;Zh_cn;GT-P6200;600*976;)UCWEB7.8.0.95/139/352*/
const Device UA2609740 = {S86338, 117, { &P15985,&P6437,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android3.2;Zh_cn;GT-P6200;600*976;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA2032587 = {S86339, 149, { &P21554,&P17979,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 Mozilla/5.0 (Linux; U; Android A5000; in-id; A5000 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2155889 = {S86340, 129, { &P15767,&P17145,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari Version/5.1*/
const Device UA2413383 = {S86341, 149, { &P15767,&P17145,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; zh-cn Model:iPad1,1) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B176 Safari/7543.48.3*/
const Device UA2415809 = {S86342, 145, { &P12258,&P17483,&P24670,&P18092 }};	/*Useragent - LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-in; lg-gs290) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2237388 = {S86343, 147, { &P21263,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; AIRIS OnePAD 725 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2239519 = {S86344, 135, { &P21306,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MOMO11Speed Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2594271 = {S86345, 35, { &P14708,&P5204,&P21724,&P18092 }};	/*Useragent - OneBrowser/3.0 (Nokia5130c-2/07.95)*/
const Device UA2339451 = {S86346, 83, { &P24023,&P5629,&P15760,&P18092 }};	/*Useragent - K-TouchT580_TD/1.0 Android/2.3 Release/02.01.2012 Browser/WAP2.0 appleWebkit/534.30*/
const Device UA2478423 = {S86347, 160, { &P21463,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; cherry superion+ Build/gingerbread_house_r84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2352834 = {S86348, 80, { &P12355,&P5567,&P17370,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/5.0/27.2239; U; en) Presto/2.8.119 320X240 LG VN250*/
const Device UA2383503 = {S86349, 165, { &P18621,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; LG-E612 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2422057 = {S86350, 145, { &P20266,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; Explay A350 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2451908 = {S86351, 147, { &P20000,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ607 Build/7.7.1-122-4-FLEM-WIFI-LA) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2456312 = {S86352, 33, { &P18323,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SAMSUNG-GT-S5222)*/
const Device UA2458101 = {S86353, 99, { &P11941,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonw580i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2727466 = {S86354, 99, { &P11941,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonw580i) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2460828 = {S86355, 136, { &P21085,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; J-Q88B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2467211 = {S86356, 118, { &P14438,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE50) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile*/
const Device UA2469243 = {S86357, 168, { &P18303,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; MT27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2503821 = {S86358, 180, { &P17262,&P5629,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Kindle Fire Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (Kindle Fire, 35, 1.7.0)*/
const Device UA2483049 = {S86359, 75, { &P21269,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; Intex_Aqua_3.2; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2487481 = {S86360, 135, { &P20480,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; JY-G3 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2504341 = {S86361, 138, { &P21500,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Lemon P2  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2584847 = {S86362, 139, { &P20244,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Elcano Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2600571 = {S86363, 90, { &P12616,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; WT19i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2600707 = {S86364, 90, { &P12968,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT26i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2603187 = {S86365, 57, { &P15364,&P17188,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 4.0) Presto/2.12.388 Version/12.14*/
const Device UA2730110 = {S86366, 164, { &P19968,&P17485,&P24676,&P18092 }};	/*Useragent - SonyEricssonW910i/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw910i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2662789 = {S86367, 140, { &P23855,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; WM8850-mid Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2667726 = {S86368, 161, { &P16356,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-N7000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2669395 = {S86369, 161, { &P16356,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; GT-N7000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2676067 = {S86370, 153, { &P20018,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993D Build/ICECREAM) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2955690 = {S86371, 152, { &P20018,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993 Build/ICECREAM) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2704001 = {S86372, 138, { &P21488,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A860K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2715051 = {S86373, 153, { &P20175,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI 1S Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2879182 = {S86374, 156, { &P16074,&P5768,&P24677,&P18092 }};	/*Useragent - Nokia3110c/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3110c) U2/1.0.0 UCBrowser/9.0.0.295 U2/1.0.0 Mobile*/
const Device UA2881399 = {S86375, 160, { &P14969,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; HTC Desire S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2894722 = {S86376, 161, { &P14969,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; HTC Desire SV Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752557 = {S86377, 152, { &P20438,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; N400 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752693 = {S86378, 64, { &P15364,&P17285,&P22169,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:21.0) Gecko/X Firefox/21.0 slurp*/
const Device UA2869338 = {S86379, 162, { &P21068,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; MI-ONE C1 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2769439 = {S86380, 137, { &P21028,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Smartfren Tab 7 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2770053 = {S86381, 153, { &P20534,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAP4040_DUO Build/PrestigioPAP4040DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774089 = {S86382, 139, { &P15047,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SK17i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2794400 = {S86383, 137, { &P22095,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ThL W7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2797448 = {S86384, 128, { &P23873,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BC1003 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2799965 = {S86385, 170, { &P13064,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; SAMSUNG-SGH-I727 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2809153 = {S86386, 199, { &P12280,&P17084,&P19840,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7 (compatible; PhantomJS/1.5.0; +http://cs.harding.edu)*/
const Device UA2850263 = {S86387, 145, { &P23344,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; OPSSON IVO6600 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2863315 = {S86388, 145, { &P23224,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Videocon A22 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2879035 = {S86389, 132, { &P23755,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5105 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2879593 = {S86390, 141, { &P23692,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Primux_Delta Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2892635 = {S86391, 149, { &P20289,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC6435LVW Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2943812 = {S86392, 141, { &P23972,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Woxter Nimbus 97Q Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2944228 = {S86393, 88, { &P18666,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A25) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2944645 = {S86394, 154, { &P18385,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; Nexus 7 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2962072 = {S86395, 140, { &P23181,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; NEBULA6.9 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2964073 = {S86396, 132, { &P21853,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; Connect-3G Build/HCL) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3019988 = {S86397, 124, { &P21853,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; Y3 Build/HCL) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3033547 = {S86398, 157, { &P12532,&P6437,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.2.1; HTC Flyer P510e Build/HTK75C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3061412 = {S86399, 172, { &P15877,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3028535 = {S86400, 93, { &P21261,&P18598,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.2; en-US; GT-I9500) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3021803 = {S86401, 134, { &P21823,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTX21 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020865 = {S86402, 142, { &P24112,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; i-mobile IQ X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024142 = {S86403, 97, { &P22623,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; Micromax_A51) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3007542 = {S86404, 64, { &P15364,&P17285,&P22171,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:23.0) Gecko/X Firefox/23.0 slurp*/
const Device UA3010415 = {S86405, 133, { &P16362,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3077183 = {S86406, 136, { &P16362,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; SPH-D710BST Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3030207 = {S86407, 153, { &P18639,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 10 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.2/manta*/
const Device UA3079544 = {S86408, 153, { &P18639,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 10 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.2/manta*/
const Device UA3094275 = {S86409, 143, { &P23712,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; i-mobile IQ XA Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3132979 = {S86410, 127, { &P18385,&P21637,&P24394,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus 7 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1650.11 Safari/537.36*/
const Device UA3095061 = {S86411, 150, { &P19792,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire U dual sim Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3096308 = {S86412, 157, { &P18295,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC One X Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3115121 = {S86413, 157, { &P18673,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA54230 = {S86414, 96, { &P13372,&P17485,&P3976,&P18092 }};	/*Useragent - Mozilla/4.0 SonyEricssonK850iv/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA24762 = {S86415, 98, { &P15756,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-A561/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA45758 = {S86416, 56, { &P15539,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player tmobile / 1.0 / htc_glacier / 2.2.1*/
const Device UA421075 = {S86417, 56, { &P15539,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player tmobile / 1.0 / htc_glacier / 2.3.4*/
const Device UA67879 = {S86418, 34, { &P13977,&P5890,&P4759,&P18092 }};	/*Useragent - NokiaN73/UC Browser7.6.1.82/28/351*/
const Device UA67979 = {S86419, 102, { &P11689,&P5776,&P4759,&P18092 }};	/*Useragent - NokiaN70-1/5.0638.3.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.6.1.82/27/354*/
const Device UA308984 = {S86420, 101, { &P11689,&P5776,&P4759,&P18092 }};	/*Useragent - NokiaN70/ 5.0741.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.6.1.82/27/351*/
const Device UA780052 = {S86421, 88, { &P17968,&P5062,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 1.6; Linux; Opera Mobi/ADR-X; U; en-GB) Presto/2.8.149 Version/11.10*/
const Device UA179450 = {S86422, 84, { &P18088,&P22740,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; Maemo; Opera Mobi/9; U; en-GB) Presto/2.7.81 Version/11.00*/
const Device UA300762 = {S86423, 124, { &P15364,&P18110,&P22446,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Iron/8.0.555.0 Chrome/8.0.555.0 Safari/534.10*/
const Device UA505821 = {S86424, 126, { &P15364,&P18110,&P22446,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Iron/8.0.555.0 Chrome/8.0.555.0 Safari/534.10*/
const Device UA368828 = {S86425, 135, { &P16092,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID X2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2683652 = {S86426, 134, { &P16092,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB870 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA455992 = {S86427, 85, { &P12280,&P17779,&P18328,&P18092 }};	/*Useragent - Opera/9.80 (iPhone; Opera Mini/7.1.32694/27.1407; U; en) Presto/2.8.119 Version/11.10*/
const Device UA513243 = {S86428, 101, { &P15364,&P17816,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.8pre) Gecko/X Ubuntu/10.04 (lucid) Namoroka/3.6.8pre*/
const Device UA571680 = {S86429, 100, { &P14714,&P5843,&P17338,&P18092 }};	/*Useragent - NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax3-02/UC Browser7.9.0.102/69/352*/
const Device UA578416 = {S86430, 43, { &P15197,&P17484,&P17347,&P18092 }};	/*Useragent - samsung-gt-b5310/UC Browser8.0.3.107/69/444*/
const Device UA688329 = {S86431, 126, { &P21399,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo P70 Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA1943375 = {S86432, 138, { &P21399,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo P70 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026157 = {S86433, 138, { &P21399,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo P70 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033315 = {S86434, 131, { &P21399,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo P70 Build/GRJ22) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA720797 = {S86435, 36, { &P13602,&P17484,&P17347,&P18092 }};	/*Useragent - gt-s5233w/UC Browser8.0.3.107/69/352*/
const Device UA753417 = {S86436, 36, { &P13602,&P17484,&P17347,&P18092 }};	/*Useragent - gt-s5233w/UC Browser8.0.3.107/69/444*/
const Device UA2166850 = {S86437, 26, { &P16362,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SPH-D710/100.40004*/
const Device UA2026807 = {S86438, 171, { &P16356,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2033022 = {S86439, 171, { &P16356,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA1892165 = {S86440, 100, { &P13488,&P5448,&P4777,&P18092 }};	/*Useragent - SonyEricssonM1i/R1BA Browser/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA1908120 = {S86441, 124, { &P15364,&P18146,&P18194,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; SunOS i86pc; rv:6.0) Gecko/X Thunderbird/6.0 OracleBeehiveExtension/1.0.0.4-OracleInternal Lightning/1.0b5*/
const Device UA1978227 = {S86442, 65, { &P14433,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux; U;2.3.6;zh_cn;GT-I9003;480*800;)UCWEB7.6.0.75/139/7694*/
const Device UA2526700 = {S86443, 187, { &P18113,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT912 Build/IMM76D; CyanogenMod-Full Throttle Retribution V2 10/16/12) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2591924 = {S86444, 190, { &P18113,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Droid Razr Build/IMM76D; CyanogenMod-Full Throttle CM9 Stronger Then All) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2829814 = {S86445, 151, { &P18113,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT912 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032317 = {S86446, 141, { &P13097,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; in-id; Axioo-VIGO350 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2344029 = {S86447, 198, { &P12056,&P5204,&P24671,&P18092 }};	/*Useragent - Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia7210supernova) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2344181 = {S86448, 153, { &P23974,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; hu-hu; Telenor_One_Touch_S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2418483 = {S86449, 124, { &P14764,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/444/UCWEB Mobile*/
const Device UA2366820 = {S86450, 168, { &P18450,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST23i Build/11.0.A.2.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2374660 = {S86451, 129, { &P23604,&P18110,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-55EX640; CC/USA; en) Presto/2.8.115 Version/11.10*/
const Device UA3039550 = {S86452, 139, { &P23604,&P18110,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-55EX640; CC/USA; en) Presto/2.8.115 Version/11.10,gzip(gfe)*/
const Device UA2378286 = {S86453, 135, { &P21229,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; iball Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2380101 = {S86454, 138, { &P20597,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F100S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2652468 = {S86455, 35, { &P14862,&P5204,&P21724,&P18092 }};	/*Useragent - OneBrowser/3.0 (Nokia2700c-2/09.97)*/
const Device UA2416673 = {S86456, 147, { &P15919,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615344 = {S86457, 147, { &P15919,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615938 = {S86458, 145, { &P15919,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2626491 = {S86459, 148, { &P15919,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3100B Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422480 = {S86460, 137, { &P21608,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MIDC409 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2425896 = {S86461, 137, { &P20290,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A11 Build/KarbonnA11) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2559531 = {S86462, 54, { &P15364,&P18580,&P21568,&P18092 }};	/*Useragent - GmailHybrid/2.0.1.7057 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2577560 = {S86463, 43, { &P15364,&P18580,&P21568,&P18092 }};	/*Useragent - GmailPush/2.0.1 CFNetwork/609 Darwin/13.0.0*/
const Device UA2495681 = {S86464, 116, { &P15364,&P7127,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; ja-JP) AppleWebKit/529 (KHTML, like Gecko, Safari/529.0) Lunascape/4.9.9.96*/
const Device UA2504844 = {S86465, 160, { &P19902,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530M Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2528351 = {S86466, 139, { &P22166,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; POLY PAD_8216HD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2554748 = {S86467, 157, { &P18507,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2848659 = {S86468, 85, { &P18507,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Adr (Linux; U; 2.3.6; zh-cn; GT-I8160 Build/GINGERBREAD.ZCLK3;480*800)*/
const Device UA2555500 = {S86469, 143, { &P20861,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Scroll Essential II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2559424 = {S86470, 140, { &P23035,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Viettel_i67 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2583560 = {S86471, 152, { &P20002,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 992D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2600971 = {S86472, 137, { &P22497,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; w30hd(QuadCore) Build/W30hd) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2601531 = {S86473, 142, { &P22858,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U8836D Build/HuaweiU8836D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2666534 = {S86474, 138, { &P20124,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ZTE V970 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2619577 = {S86475, 95, { &P17337,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Messenger; 1.8.3) (GT-P7500; samsung; GT-P7500; 4.0.4/IMM76D)*/
const Device UA2621539 = {S86476, 89, { &P17337,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Messenger; 1.8.3) (p4; samsung; GT-P7500; 4.1.1/JRO03L)*/
const Device UA2677280 = {S86477, 130, { &P13452,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2665913 = {S86478, 156, { &P18307,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Malata SMBA1002 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3073672 = {S86479, 165, { &P18307,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Malata SMBA1002 Build/JDQ39; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2675276 = {S86480, 134, { &P20343,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZP900 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675278 = {S86481, 132, { &P19860,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A9+ Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2861725 = {S86482, 140, { &P19860,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A9+ Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2684450 = {S86483, 154, { &P12663,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; LG-E730 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3044578 = {S86484, 148, { &P12663,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; LG-E730 Build/GRJ22; CM7-JBTaste) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2707955 = {S86485, 99, { &P19761,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HUAWEI; W1-U00)*/
const Device UA2708344 = {S86486, 145, { &P22664,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GIONEE_Ctrl_V2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2709752 = {S86487, 137, { &P21385,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F240S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2867002 = {S86488, 165, { &P19863,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2751922 = {S86489, 104, { &P13977,&P5890,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaN73*/
const Device UA2752197 = {S86490, 92, { &P16072,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN78) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2752425 = {S86491, 88, { &P19996,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A15) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2768449 = {S86492, 152, { &P17926,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AT100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA2859713 = {S86493, 148, { &P17926,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; AT100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2773816 = {S86494, 137, { &P18216,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Sony Tablet P Build/TISU0144) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2782647 = {S86495, 137, { &P12841,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Vivid 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799251 = {S86496, 146, { &P23230,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Huawei-H867G Build/HuaweiH867G) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2858339 = {S86497, 152, { &P15364,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; 8190 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2937364 = {S86498, 152, { &P15364,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; 8720 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.292 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3049138 = {S86499, 160, { &P15364,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo A630t Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.292 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2863727 = {S86500, 82, { &P23228,&P17979,&P18062,&P18092 }};	/*Useragent - SAMSUNG-GT-I8258_TD/1.0 Android/4.0.4 Release/09.24.2012 Browser/AppleWebKit534.30*/
const Device UA2866036 = {S86501, 154, { &P20529,&P18066,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A89 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.289*/
const Device UA2895251 = {S86502, 93, { &P19847,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S6012) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2938029 = {S86503, 150, { &P18304,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2945154 = {S86504, 158, { &P18303,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; MT27i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3021380 = {S86505, 137, { &P20947,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Evo V 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021197 = {S86506, 142, { &P23161,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Xtreamer Joyz Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2995970 = {S86507, 130, { &P23413,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PLT7223G Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3020268 = {S86508, 90, { &P18302,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MT25i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3015666 = {S86509, 156, { &P15786,&P5043,&P24678,&P18092 }};	/*Useragent - NokiaC2-05/2.0 (08.69) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-05) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3021570 = {S86510, 133, { &P18721,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A90S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022725 = {S86511, 137, { &P24391,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8260 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023542 = {S86512, 138, { &P24732,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-S735C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023791 = {S86513, 93, { &P15575,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; GT-I9100) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3080395 = {S86514, 151, { &P23846,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A12 Build/RheaTK) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3094183 = {S86515, 137, { &P18674,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7005 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3060879 = {S86516, 162, { &P12026,&P17485,&P24679,&P18092 }};	/*Useragent - SonyEricssonC510/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonc510) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3061126 = {S86517, 137, { &P22597,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I545 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3083916 = {S86518, 137, { &P22597,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I545 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3094548 = {S86519, 127, { &P20127,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT270 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094842 = {S86520, 140, { &P21094,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LENOVO A789 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083586 = {S86521, 138, { &P24150,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; vivo Xplay Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094496 = {S86522, 133, { &P24053,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SHL22 Build/S6290) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094974 = {S86523, 137, { &P24149,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Aqua I-5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3078665 = {S86524, 178, { &P12897,&P5843,&P24055,&P18092 }};	/*Useragent - NokiaC2-00/2.0 (03.99) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311*/
const Device UA3130949 = {S86525, 137, { &P19920,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS870 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA8037 = {S86526, 60, { &P18088,&P5448,&P22303,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows CE; PPC; 240x320)*/
const Device UA147398 = {S86527, 46, { &P18088,&P5448,&P22303,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows CE)*/
const Device UA63649 = {S86528, 13, { &P14531,&P17483,&P17660,&P18092 }};	/*Useragent - LG/KF310/v2.0*/
const Device UA21854 = {S86529, 69, { &P12009,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia6212 classic/2.0 (06.20) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA62933 = {S86530, 120, { &P13542,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW830i_CMCC/R1KG SEMC-SW/R1KG Release/04.18.2007 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64499 = {S86531, 83, { &P13542,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW830i/R1KG Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23966 = {S86532, 39, { &P12359,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KF300 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA30865 = {S86533, 51, { &P12359,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KF300-Orange/V10c Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA58641 = {S86534, 84, { &P12359,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KF300e/CMCC Release/1.4.2008 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24554 = {S86535, 147, { &P14467,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Basic EB28 2.2.2; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA33040 = {S86536, 145, { &P14467,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Platinum_2.2.2; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43294 = {S86537, 145, { &P14467,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android SpeedRomBetaV2; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43514 = {S86538, 150, { &P14467,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Ultimate EB28 2.2.2; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59235 = {S86539, 144, { &P14467,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android UbuntDroid-v6; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73016 = {S86540, 144, { &P14467,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Vanilla_2.2.2; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019778 = {S86541, 158, { &P14467,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android TransGinger 2.2.2 EF09 v1.4; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023110 = {S86542, 160, { &P14467,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android TransGinger 2.2.2 EF09 v2.0mg; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2451158 = {S86543, 183, { &P14467,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android TrasGingerOnCrack EF09 2.2.2 1.3.4 Vampirefo Edition; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2641501 = {S86544, 142, { &P14467,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Blaze 2.2.2; en-us; SPH-M920 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43545 = {S86545, 68, { &P13699,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHE210/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA44285 = {S86546, 86, { &P13818,&P6476,&P4223,&P18092 }};	/*Useragent - ZTE-T870 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 firmware/T870 T06*/
const Device UA69129 = {S86547, 100, { &P14763,&P5776,&P4779,&P18092 }};	/*Useragent - NokiaN72/5.0819.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/27/351*/
const Device UA88639 = {S86548, 11, { &P16933,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M5252*/
const Device UA166250 = {S86549, 31, { &P17649,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F02A(c100;TB;W24H17)*/
const Device UA281562 = {S86550, 148, { &P18034,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-kr; SKY IM-A630K Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018514 = {S86551, 70, { &P18034,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; SKY IM-A630K Build/ERE27)*/
const Device UA377749 = {S86552, 139, { &P12530,&P5448,&P4777,&P18092 }};	/*Useragent - PPC; 240x320; HTC_P3450/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA427041 = {S86553, 138, { &P18307,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Malata SMBA1002 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033248 = {S86554, 146, { &P18307,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Malata SMBA1002 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257510 = {S86555, 139, { &P18307,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Malata SMBA1002 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA725210 = {S86556, 93, { &P17595,&P17297,&P18198,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.2.13) Gecko/X Thunderbird/3.1.7*/
const Device UA681626 = {S86557, 42, { &P14984,&P5129,&P24665,&P18092 }};	/*Useragent - Nokia5250/UC Browser8.0.4.121/50/444/UCWEB*/
const Device UA608919 = {S86558, 171, { &P15575,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en_us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA694049 = {S86559, 171, { &P15575,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA783821 = {S86560, 172, { &P15575,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2025026 = {S86561, 172, { &P15575,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.2.139/145/355*/
const Device UA2034391 = {S86562, 171, { &P15575,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA645663 = {S86563, 60, { &P15364,&P17188,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0; rv:8.0.1) Gecko/X Firefox/8.0.1*/
const Device UA657746 = {S86564, 59, { &P15364,&P18145,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD i386; rv:6.0) Gecko/X Firefox/6.0*/
const Device UA925800 = {S86565, 71, { &P16463,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;GT-I8350;7.10;7720)*/
const Device UA925803 = {S86566, 71, { &P16463,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;GT-I8350;7.10;8107)*/
const Device UA2412416 = {S86567, 71, { &P16463,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;GT-I8350;7.10;8773)*/
const Device UA1954620 = {S86568, 70, { &P18095,&P18109,&P18122,&P18092 }};	/*Useragent - Opera/9.80 (Series 60; Opera Mini; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2384662 = {S86569, 142, { &P15881,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Incredible 2 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2652232 = {S86570, 142, { &P15881,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Incredible 2 Build/JRO03R) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2029449 = {S86571, 134, { &P21179,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MOMO8 bird Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2032924 = {S86572, 134, { &P21179,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MOMO8 bird Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2352126 = {S86573, 136, { &P18658,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ISW16SH Build/S9181) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2032591 = {S86574, 140, { &P21489,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; zh-cn; HTC Rezound Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2458446 = {S86575, 138, { &P21489,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Rezound Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2200276 = {S86576, 155, { &P18466,&P18066,&P18616,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.1 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2226695 = {S86577, 132, { &P14905,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P7100 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2427063 = {S86578, 132, { &P14905,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2227989 = {S86579, 146, { &P20284,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xenta TAB9-200 Build/Xenta TAB9-200) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2234924 = {S86580, 168, { &P13018,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MT11i Build/4.1.B.0.431) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2343137 = {S86581, 75, { &P17595,&P19834,&P17470,&P18092 }};	/*Useragent - WebProcess/8536.25 CFNetwork/596.1 Darwin/12.1.0 (x86_64) (MacBookPro9%2C2)*/
const Device UA2654746 = {S86582, 88, { &P17595,&P19834,&P17470,&P18092 }};	/*Useragent - iCoreService (unknown version) CFNetwork/596.1 Darwin/12.1.1 (x86_64) (MacBookPro10%2C2)*/
const Device UA2346952 = {S86583, 140, { &P12798,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LT18i Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2421443 = {S86584, 142, { &P12798,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xperia Arc S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2458097 = {S86585, 141, { &P20131,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Thrill 430x Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2469864 = {S86586, 171, { &P18507,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2468154 = {S86587, 140, { &P21652,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; POV_TAB-PROTAB27 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2482184 = {S86588, 131, { &P18632,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AN10BG3 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2486406 = {S86589, 180, { &P19868,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; ALCATEL ONE TOUCH 985N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2506444 = {S86590, 158, { &P12258,&P17483,&P24675,&P18092 }};	/*Useragent - LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-gs290) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2557595 = {S86591, 140, { &P22001,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Celkon A 77 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2583225 = {S86592, 136, { &P17948,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SC-02C Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2585173 = {S86593, 154, { &P20018,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993D Build/ICECREAM) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2855529 = {S86594, 153, { &P20018,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 993 Build/ICECREAM) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2598145 = {S86595, 90, { &P15147,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST15i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2600334 = {S86596, 131, { &P12889,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7300 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2635822 = {S86597, 131, { &P12889,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P7300 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2601554 = {S86598, 132, { &P16168,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; NookColor Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2604732 = {S86599, 127, { &P18008,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ST10 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2654452 = {S86600, 154, { &P18673,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747286 = {S86601, 155, { &P18673,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562i Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659540 = {S86602, 95, { &P11950,&P5477,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; X10i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2668205 = {S86603, 64, { &P15364,&P17285,&P22167,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:20.0) Gecko/X Firefox/20.0 slurp*/
const Device UA2670903 = {S86604, 137, { &P17324,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9001 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2700177 = {S86605, 93, { &P23004,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; TECNO_D3) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2702667 = {S86606, 222, { &P16518,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-e2222) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile,SAMSUNG-GT-E2222/E2222JVKJ3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA2792068 = {S86607, 99, { &P16518,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-e2222) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2709098 = {S86608, 165, { &P20709,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID1042 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444*/
const Device UA2713172 = {S86609, 138, { &P22738,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT916 Build/2_26C_2014) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2737496 = {S86610, 88, { &P20231,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A21) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2869317 = {S86611, 152, { &P21963,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Q800 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2768687 = {S86612, 164, { &P13308,&P17485,&P24677,&P18092 }};	/*Useragent - SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj105i) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA3058606 = {S86613, 197, { &P18283,&P18066,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb*/
const Device UA2773294 = {S86614, 136, { &P22660,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P778 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783624 = {S86615, 135, { &P23130,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; H2000+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784000 = {S86616, 134, { &P23894,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V1277 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784037 = {S86617, 127, { &P20127,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT270 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2796205 = {S86618, 137, { &P23431,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; miTab DIAMOND Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2797693 = {S86619, 136, { &P20714,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID8065 Build/ICS.MID8065.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2799833 = {S86620, 157, { &P13991,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6400L Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2861656 = {S86621, 71, { &P23264,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X327/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2936552 = {S86622, 92, { &P15767,&P17779,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPd OS 6_1_3; en-US; iPd4,1) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2867955 = {S86623, 143, { &P23011,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; iris405 Build/LAVAIRIS405) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2868429 = {S86624, 139, { &P15364,&P17350,&P23733,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT based; en-US) AppleWebKit/534.34 (KHTML, like Gecko)  QtWeb Internet Browser/3.8.4 http://www.QtWeb.net*/
const Device UA2869299 = {S86625, 140, { &P12280,&P18117,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.4.2m*/
const Device UA2966912 = {S86626, 140, { &P12280,&P18117,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.4.4m*/
const Device UA3079675 = {S86627, 140, { &P12280,&P18117,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.4.5m*/
const Device UA2869688 = {S86628, 141, { &P17968,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3060767 = {S86629, 157, { &P22609,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6312 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA2967892 = {S86630, 160, { &P18303,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT27i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3021132 = {S86631, 137, { &P22953,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F220K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022640 = {S86632, 134, { &P18740,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; U7015 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2970545 = {S86633, 137, { &P18021,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A770k Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36*/
const Device UA3021329 = {S86634, 137, { &P18021,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A770K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3001066 = {S86635, 139, { &P21630,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; i-note WiFi 1.1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3092529 = {S86636, 164, { &P21630,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; i-note WiFi 1.1 Build/i-mobile i-note WiFi 1.1 v0.2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3022329 = {S86637, 134, { &P24493,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; BS471 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022603 = {S86638, 99, { &P21269,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Intex_Aqua_3.2) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3017551 = {S86639, 162, { &P21648,&P18598,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9505 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA3022482 = {S86640, 132, { &P20807,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B92 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020653 = {S86641, 155, { &P19910,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; LG-E617G/E617G20a-BLM-XX Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3029361 = {S86642, 158, { &P12616,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; WT19i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3029486 = {S86643, 186, { &P12972,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-T989 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/BLACKLiquid-v2.9/hercules*/
const Device UA3060960 = {S86644, 175, { &P15991,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6102 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3062913 = {S86645, 135, { &P23546,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AD683G Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3094270 = {S86646, 134, { &P20226,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; LG-E975 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094531 = {S86647, 153, { &P21140,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Optimus_Madrid Build/IceCreamSandwich) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094808 = {S86648, 135, { &P23165,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; H7500+ Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080391 = {S86649, 92, { &P13163,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaN97) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3083839 = {S86650, 143, { &P12972,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SAMSUNG-SGH-T989 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3092553 = {S86651, 90, { &P15364,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; C2104) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3125927 = {S86652, 90, { &P15364,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; C1904) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3126075 = {S86653, 99, { &P15364,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Connect-2G-2.0) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3094675 = {S86654, 137, { &P24606,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E470f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA9992 = {S86655, 52, { &P12465,&P17484,&P4902,&P18092 }};	/*Useragent - SCH-F339/ POLARIS/6.15 (GUI; compatible; UP.Browser)*/
const Device UA32116 = {S86656, 77, { &P12465,&P17484,&P4902,&P18092 }};	/*Useragent - SAMSUNG-SCH-F339/1.0 POLARIS/6.15.WEB Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA49058 = {S86657, 102, { &P13890,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-SGH-T566/T566UXJG5 SHP/VPP/R5 Dolfin/1.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA67110 = {S86658, 33, { &P14742,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH05A3(c500;TB;W24H14)*/
const Device UA158236 = {S86659, 31, { &P17719,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N01B(c500;TB;W24H16)*/
const Device UA167223 = {S86660, 61, { &P16593,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX560/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA305473 = {S86661, 55, { &P15364,&P17188,&P20906,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; WinNT4.0; en-US) Netscape6/6.1*/
const Device UA318166 = {S86662, 54, { &P17566,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SH3I UP.Browser/6.2_7.2.7.1.K.5.207 (GUI) MMP/2.0*/
const Device UA370658 = {S86663, 155, { &P16053,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VS660 Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA459836 = {S86664, 58, { &P15364,&P21445,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows ME; U; en) Presto/2.5.24 Version/10.53*/
const Device UA461557 = {S86665, 58, { &P15364,&P21445,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows ME; U; en) Presto/2.5.24 Version/10.54*/
const Device UA713391 = {S86666, 58, { &P15364,&P21445,&P18096,&P18092 }};	/*Useragent - Opera/9.80 (Windows ME; U; en) Presto/2.5.22 Version/10.50*/
const Device UA1936345 = {S86667, 100, { &P14482,&P5843,&P17347,&P18092 }};	/*Useragent - NokiaC3-01/5.0 (06.05) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac3-01/UC Browser8.0.3.107/69/444*/
const Device UA626901 = {S86668, 99, { &P13868,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; HD7 T9292)*/
const Device UA633584 = {S86669, 93, { &P13868,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; HD7)*/
const Device UA925723 = {S86670, 95, { &P13868,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; T9296)*/
const Device UA726486 = {S86671, 72, { &P15364,&P7133,&P18193,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:7.0) Gecko/X Thunderbird/7.0*/
const Device UA737268 = {S86672, 145, { &P15178,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-LW690 Build/GRJ90-LG-LW690) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1900568 = {S86673, 145, { &P15178,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LG-LW690 Build/GRJ90-LG-LW690) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA789390 = {S86674, 165, { &P13077,&P5629,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8650+ Build/V100R001C92B861) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA847630 = {S86675, 74, { &P17976,&P17979,&P18186,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet; rv:11.0) Gecko/11.0 Firefox/11.0 Fennec/11.0*/
const Device UA848599 = {S86676, 138, { &P12280,&P17146,&P20782,&P18092 }};	/*Useragent - MQQBrowser/31 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA856671 = {S86677, 138, { &P12280,&P17146,&P20782,&P18092 }};	/*Useragent - MQQBrowser/31 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3*/
const Device UA882538 = {S86678, 78, { &P15364,&P7127,&P23682,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; ; rv:1.8.0.7) Gecko/X Firefox/1.9.0.1*/
const Device UA897744 = {S86679, 80, { &P15373,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5530)/UC Browser8.4.0.159/50/352/UCWEB Mobile*/
const Device UA2372896 = {S86680, 31, { &P15373,&P5129,&P24670,&P18092 }};	/*Useragent - Nokia5530/UCWEB8.4.0.150/50/999*/
const Device UA923913 = {S86681, 162, { &P18645,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I747 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2023473 = {S86682, 154, { &P18645,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-L710 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032286 = {S86683, 156, { &P22091,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; th-th; WellcoM_A90; Build/ALPS.GB.FDD2.MP.V2.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2168219 = {S86684, 130, { &P15767,&P17146,&P20777,&P18092 }};	/*Useragent - MQQBrowser/2.6 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA2169791 = {S86685, 37, { &P14862,&P5204,&P21759,&P18092 }};	/*Useragent - iBrowser/Mini2.8 (Nokia2700c-2/09.98)*/
const Device UA2416432 = {S86686, 63, { &P12280,&P18432,&P22070,&P18092 }};	/*Useragent - Instagram 3.0.0 (iPhone; iPhone OS 6.0; en_US) AppleWebKit/420+*/
const Device UA2260449 = {S86687, 37, { &P14826,&P5843,&P23405,&P18092 }};	/*Useragent - MQQBrowser/Mini2.8 (NokiaC1-01/04.40)*/
const Device UA2310455 = {S86688, 125, { &P11766,&P5776,&P24663,&P18092 }};	/*Useragent - Nokia3230/2.0 (5.0614.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.9.1.120/27/351/UCWEB*/
const Device UA2349683 = {S86689, 170, { &P11747,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Desire V Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2366991 = {S86690, 135, { &P18253,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT894 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2420625 = {S86691, 129, { &P11932,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA6210 Navigator) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile*/
const Device UA2421061 = {S86692, 147, { &P21265,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; AIRIS OnePAD 730 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455533 = {S86693, 132, { &P14849,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I800 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2466187 = {S86694, 154, { &P18442,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556393 = {S86695, 154, { &P18442,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2466191 = {S86696, 135, { &P20343,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ZP900 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2839148 = {S86697, 136, { &P20343,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ZP900S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2466583 = {S86698, 69, { &P12568,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.2; zh-cn; GT-B7510; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2480779 = {S86699, 142, { &P21978,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GSmart GS202 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2484018 = {S86700, 156, { &P17172,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Transformer Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2487852 = {S86701, 99, { &P13677,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5253) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2518463 = {S86702, 132, { &P15364,&P17285,&P23738,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/527+ (KHTML, like Gecko) QtWeb Internet Browser/1.7 http://www.QtWeb.net*/
const Device UA2528708 = {S86703, 122, { &P11834,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA6760s-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2531902 = {S86704, 91, { &P15518,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5230) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2597890 = {S86705, 96, { &P15518,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-s5230) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2659517 = {S86706, 88, { &P15518,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; ru; gt-s5230) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2730075 = {S86707, 88, { &P15518,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; fa; gt-s5230) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2856735 = {S86708, 91, { &P15518,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; es-LA; gt-s5230) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2986182 = {S86709, 88, { &P15518,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; vi; gt-s5230) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2565858 = {S86710, 162, { &P12765,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2764333 = {S86711, 162, { &P12765,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; A100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2580548 = {S86712, 91, { &P13452,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2; en-US; GT-P1000) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2584097 = {S86713, 138, { &P20508,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cynus T2 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2587395 = {S86714, 76, { &P15364,&P16110,&P22334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; rv:2.0.1) Gecko/X Firefox/4.0.1 CometBird/4.0.1*/
const Device UA2593956 = {S86715, 131, { &P21309,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; myPadP4 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2596324 = {S86716, 159, { &P13634,&P17483,&P24676,&P18092 }};	/*Useragent - LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gs500) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2624317 = {S86717, 150, { &P17323,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P6800 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2632478 = {S86718, 149, { &P17323,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P6800 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2633614 = {S86719, 130, { &P17168,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P7310 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.28 Safari/537.31*/
const Device UA2710426 = {S86720, 129, { &P17168,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7310 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2663323 = {S86721, 141, { &P22648,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ONIDA i099 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2671485 = {S86722, 152, { &P21833,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030E Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675396 = {S86723, 139, { &P15047,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SK17i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2682977 = {S86724, 80, { &P15364,&P7127,&P22315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9a1) Gecko/X SeaMonkey/1.5a*/
const Device UA2706345 = {S86725, 99, { &P14090,&P5477,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT-I5801) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2733721 = {S86726, 175, { &P20289,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC6435LVW Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (HTC6435LVW, 42, 1.9.1)*/
const Device UA2734681 = {S86727, 140, { &P23154,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOLO_X1000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2737721 = {S86728, 138, { &P22600,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300K Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2748481 = {S86729, 134, { &P18595,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I8190 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3004298 = {S86730, 139, { &P18595,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I8190 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2750767 = {S86731, 145, { &P22988,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; i-mobile i-note WIFI3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2752074 = {S86732, 117, { &P11684,&P5129,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) Nokia5800 XpressMusic*/
const Device UA2768286 = {S86733, 96, { &P22209,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Titanium_S5) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2768362 = {S86734, 145, { &P23104,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Starmobile GOAL Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2769860 = {S86735, 131, { &P18466,&P18066,&P23371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/19.77.34.5 Mobile Safari/537.22*/
const Device UA2780619 = {S86736, 129, { &P23736,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; iDnD7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2783522 = {S86737, 152, { &P22963,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783722 = {S86738, 137, { &P18458,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I200 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783840 = {S86739, 136, { &P18438,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E973 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2859097 = {S86740, 68, { &P17595,&P21691,&P21570,&P18092 }};	/*Useragent - Mail/1503 CFNetwork/596.3.3 Darwin/12.3.0 (x86_64) (MacBookPro9%2C2)*/
const Device UA2800657 = {S86741, 137, { &P19856,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LS970 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867285 = {S86742, 156, { &P19777,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9105 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2868422 = {S86743, 158, { &P21605,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; C2104 Build/15.0.A.1.31) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2892894 = {S86744, 147, { &P20146,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2935781 = {S86745, 142, { &P15794,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Sensation XL Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2944038 = {S86746, 151, { &P19996,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A15 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3025861 = {S86747, 93, { &P23000,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; TECNO_P3) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA2964176 = {S86748, 197, { &P18595,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8190 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.5/580,2.0.5.580,2013-06-26 22:40, +0100, gb*/
const Device UA2998185 = {S86749, 148, { &P20847,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; Xenta TAB10-211 Build/Xenta TAB10-211) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3023696 = {S86750, 93, { &P16356,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7000) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3022597 = {S86751, 134, { &P19864,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 80XSK Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021443 = {S86752, 145, { &P15805,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Raider X710e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022711 = {S86753, 140, { &P15805,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Holiday Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3018432 = {S86754, 33, { &P15364,&P7480,&P17470,&P18091 }};	/*Useragent - Xaldon WebSpider;c2VvLWNyYXdsZXI=*/
const Device UA3057949 = {S86755, 32, { &P15364,&P7480,&P17470,&P18091 }};	/*Useragent - Test Spider 0.2;c2VvLWNyYXdsZXI=*/
const Device UA3057955 = {S86756, 45, { &P15364,&P7480,&P17470,&P18091 }};	/*Useragent - Norbert the Spider(Burf.com);c2VvLWNyYXdsZXI=*/
const Device UA3059011 = {S86757, 161, { &P11747,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire V Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3022498 = {S86758, 130, { &P17337,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3061927 = {S86759, 93, { &P15575,&P18111,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9100) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3059072 = {S86760, 137, { &P19861,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I605 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3094190 = {S86761, 137, { &P19861,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SCH-I605 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3060466 = {S86762, 162, { &P21351,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI C8813Q Build/HuaweiC8813Q) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3060877 = {S86763, 160, { &P21351,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI C8813 Build/HuaweiC8813) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3094931 = {S86764, 136, { &P20240,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E971 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3130946 = {S86765, 136, { &P22727,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGP312 Build/10.3.1.C.0.136) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3132934 = {S86766, 137, { &P22382,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA2673 = {S86767, 69, { &P16213,&P5448,&P22271,&P18092 }};	/*Useragent - Xda_orbit_2; 240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA2676 = {S86768, 68, { &P16213,&P5448,&P22271,&P18092 }};	/*Useragent - Xda_orbit_2/240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA22697 = {S86769, 38, { &P15512,&P17483,&P17602,&P18092 }};	/*Useragent - LG-X335 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA22941 = {S86770, 80, { &P15537,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK500c/R2L SEMC-Browser/4.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA60295 = {S86771, 81, { &P15537,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK500c/R2AA SEMC-Browser/4.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA60899 = {S86772, 81, { &P15537,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK500c/R2AT SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62581 = {S86773, 81, { &P15537,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK500i/R2AA SEMC-Browser/4.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62582 = {S86774, 80, { &P15537,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK500i/R2L SEMC-Browser/4.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63415 = {S86775, 81, { &P15537,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK500i/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23514 = {S86776, 52, { &P17423,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-R974_/00.72 UP.Browser/7.2.7.5.656 (GUI) MMP/2.0*/
const Device UA60975 = {S86777, 52, { &P17423,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-R974_/00.72 UP.Browser/7.2.7.5.671 (GUI) MMP/2.0*/
const Device UA145041 = {S86778, 52, { &P17423,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-R974_/00.72 UP.Browser/7.2.7.5.658 (GUI) MMP/2.0*/
const Device UA24006 = {S86779, 73, { &P15651,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-GT-M2520/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA78564 = {S86780, 127, { &P14673,&P17605,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-E2652W/E2652WZHKF2 RTK-E/1.0 MMP/1.0 Release/Jun 27 2011 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA52337 = {S86781, 120, { &P13452,&P5629,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.3.3;Zh_cn;GT-P1000;600*1024;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA2018992 = {S86782, 71, { &P13452,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.3;Zh_cn;GT-P1000;600*1024;)UCWEB7.8.0.95/139/352*/
const Device UA2609722 = {S86783, 120, { &P13452,&P5629,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.3.6;Zh_cn;GT-P1000;600*1024;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA2610553 = {S86784, 71, { &P13452,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;GT-P1000;600*1024;)UCWEB7.8.0.95/139/444*/
const Device UA2637902 = {S86785, 71, { &P13452,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.4;Zh_cn;GT-P1000;600*1024;)UCWEB7.8.0.95/139/444*/
const Device UA693791 = {S86786, 47, { &P15364,&P18570,&P22246,&P18092 }};	/*Useragent - Flipboard/1.7.2 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA470871 = {S86787, 68, { &P15364,&P5690,&P21960,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:2.0b5pre) Gecko/X Minefield/4.0b5pre*/
const Device UA1891571 = {S86788, 84, { &P15364,&P5690,&P21960,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; Windows NT 5.1; en-US; rv:2.0b2pre) Gecko/X Minefield/4.0b2pre*/
const Device UA572448 = {S86789, 47, { &P14983,&P5055,&P17338,&P18092 }};	/*Useragent - nokiac7-00/UC Browser7.9.0.102/69/352  3gpp-gba*/
const Device UA584980 = {S86790, 131, { &P17595,&P17295,&P22454,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.7 (KHTML, like Gecko) Iron/16.0.950.0 Chrome/16.0.950.0 Safari/535.7*/
const Device UA736976 = {S86791, 67, { &P15106,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux; U;2.3.3;zh_cn;HTC C510e;320*480;)UCWEB7.6.0.75/139/32317*/
const Device UA1938366 = {S86792, 67, { &P15106,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux; U;2.3.5;zh_cn;HTC C510e;320*480;)UCWEB7.6.0.75/139/32317*/
const Device UA2522438 = {S86793, 26, { &P12972,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-T989/100.40004*/
const Device UA1954377 = {S86794, 141, { &P22160,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; iberry AUXUS AX02 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032284 = {S86795, 147, { &P17168,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7310 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617431 = {S86796, 151, { &P17168,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P7310 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2220905 = {S86797, 71, { &P20036,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X246/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2257376 = {S86798, 136, { &P22139,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; Tablet-9738B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2260809 = {S86799, 147, { &P12280,&P18117,&P21787,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) 1Password/3.7/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA2366628 = {S86800, 158, { &P14919,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Pyramid Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2383747 = {S86801, 135, { &P21278,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; QW TB-9718D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2384334 = {S86802, 162, { &P15501,&P5477,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.53) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2420677 = {S86803, 122, { &P13800,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA5630d-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile*/
const Device UA2421588 = {S86804, 138, { &P19746,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VIVO 4.3 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2425492 = {S86805, 79, { &P16595,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.1-update1; zh-cn; Micromax_A60; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2426905 = {S86806, 143, { &P20825,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS TOUCHTAB 7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2428296 = {S86807, 99, { &P16384,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s5380k) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2428760 = {S86808, 138, { &P15802,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Amaze 4G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2439323 = {S86809, 89, { &P15364,&P5690,&P21957,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2a1pre) Gecko/X Minefield/3.2a1pre*/
const Device UA2458119 = {S86810, 133, { &P19996,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A15 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2467906 = {S86811, 139, { &P20180,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOLO_A800 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2483915 = {S86812, 118, { &P15791,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia603) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2503175 = {S86813, 136, { &P23064,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ThL W1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2787276 = {S86814, 136, { &P23064,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ThL W1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2504275 = {S86815, 163, { &P19863,&P18066,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2526862 = {S86816, 83, { &P21433,&P17979,&P18062,&P18092 }};	/*Useragent - ZTE U985_TD/1.0 Linux/3.1.10 Android/4.0 Release/8.1.2012 Browser/AppleWebKit534.30*/
const Device UA2529910 = {S86817, 138, { &P21305,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; FLY IQ230 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2554122 = {S86818, 31, { &P18557,&P20706,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia308/05.80)*/
const Device UA2565842 = {S86819, 100, { &P13206,&P17485,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk530i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2580490 = {S86820, 166, { &P15201,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Sensation 4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2860637 = {S86821, 169, { &P15201,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation Z710e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2583324 = {S86822, 138, { &P20235,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cynus T1 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583346 = {S86823, 127, { &P17686,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; MZ601 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2594297 = {S86824, 133, { &P21501,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ro-ro; P5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2603304 = {S86825, 139, { &P16553,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T959V Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2610694 = {S86826, 154, { &P15919,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3100 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2630500 = {S86827, 147, { &P15919,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2673958 = {S86828, 130, { &P17168,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7310 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2658175 = {S86829, 148, { &P22639,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; CELKON A83 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2662072 = {S86830, 130, { &P19778,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; PantechP9090 Build/IMM76I) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA3124293 = {S86831, 130, { &P19778,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; PantechP9090 Build/JZO54K) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2671027 = {S86832, 137, { &P18747,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A840S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675077 = {S86833, 126, { &P17686,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; MZ601 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2675922 = {S86834, 136, { &P18240,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P700 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677385 = {S86835, 62, { &P15364,&P5690,&P22226,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:21.0) Gecko/X Thunderbird/21.0*/
const Device UA2705403 = {S86836, 43, { &P15364,&P18580,&P23031,&P18092 }};	/*Useragent - Dolphin/7.3 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2869837 = {S86837, 99, { &P12137,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5620) U2/1.0.0 UCBrowser/9.0.0.295 U2/1.0.0 Mobile*/
const Device UA2709750 = {S86838, 135, { &P12788,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; PG86100 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713723 = {S86839, 139, { &P12788,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Evo 3D CDMA Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2869812 = {S86840, 99, { &P15547,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-c6712) U2/1.0.0 UCBrowser/9.0.0.295 U2/1.0.0 Mobile*/
const Device UA2737531 = {S86841, 94, { &P18296,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC One V) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2869276 = {S86842, 94, { &P18296,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC One V) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2738141 = {S86843, 90, { &P18302,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MT25i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2750756 = {S86844, 112, { &P23119,&P20706,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia307/07.55; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2768729 = {S86845, 141, { &P22996,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Celkon A62 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768930 = {S86846, 161, { &P18645,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2796673 = {S86847, 153, { &P18645,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L710 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2769585 = {S86848, 137, { &P18674,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7005 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2771308 = {S86849, 140, { &P17905,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SO-03D Build/6.1.F.0.106) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2781439 = {S86850, 146, { &P23104,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Starmobile GOAL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2783293 = {S86851, 159, { &P14993,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT15i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799405 = {S86852, 152, { &P19767,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ALCATEL ONE TOUCH 8008D Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799519 = {S86853, 157, { &P18301,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS840 4G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2858162 = {S86854, 159, { &P21756,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Orange_infinity_996 Build/IceCreamSandwich) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2866426 = {S86855, 96, { &P15364,&P7127,&P21781,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.4.1 Safari/534.34*/
const Device UA3054261 = {S86856, 96, { &P15364,&P7127,&P21781,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/537.21 (KHTML, like Gecko) QupZilla/1.4.3 Safari/537.21*/
const Device UA2867459 = {S86857, 134, { &P18472,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT925 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892758 = {S86858, 136, { &P18472,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Razr HD Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3028631 = {S86859, 197, { &P15575,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb*/
const Device UA2894216 = {S86860, 143, { &P23708,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ultrafone_501 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3039495 = {S86861, 159, { &P12616,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WT19i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3021060 = {S86862, 136, { &P14017,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-P990 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020953 = {S86863, 130, { &P22662,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Enjoy 71 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3017040 = {S86864, 163, { &P24062,&P18111,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; L3 Build/TECNOL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/33567*/
const Device UA3022586 = {S86865, 126, { &P12973,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023407 = {S86866, 157, { &P12973,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3079798 = {S86867, 96, { &P20004,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Karbonn A2+) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3023816 = {S86868, 88, { &P20529,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A89) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3080576 = {S86869, 97, { &P22622,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Micromax A72) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3033273 = {S86870, 168, { &P23255,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI G520-0000 Build/HuaweiG520-0000) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3078975 = {S86871, 168, { &P23255,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI G520-5000 Build/HuaweiG520-5000) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3065677 = {S86872, 143, { &P12280,&P18432,&P24712,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 OverDrive Media Console/3.0.1*/
const Device UA3058872 = {S86873, 115, { &P14708,&P5204,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia5130c-2/07.95; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3130733 = {S86874, 139, { &P18564,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One X+ Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3093684 = {S86875, 141, { &P21614,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Philips W536 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093787 = {S86876, 135, { &P21446,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SH837W Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093664 = {S86877, 133, { &P20156,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S500 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3069053 = {S86878, 79, { &P15436,&P17485,&P24505,&P18092 }};	/*Useragent - Opera/8.01 (J2ME/MIDP; Opera Mini/1.0.1479/HiFi; SonyEricsson P900; no; U; ssr)*/
const Device UA3079222 = {S86879, 95, { &P23122,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; Karbonn_A6) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3109060 = {S86880, 138, { &P18072,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109185 = {S86881, 138, { &P18072,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126436 = {S86882, 138, { &P18072,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E120K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110928 = {S86883, 97, { &P15364,&P18110,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; A1+) U2/1.0.0 UCBrowser/9.3.0.321 Mobile*/
const Device UA11921 = {S86884, 136, { &P12311,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11929 = {S86885, 148, { &P12311,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1-DL05-Beta-6; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55901 = {S86886, 150, { &P12311,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1-DL05-Beta-5.5; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026488 = {S86887, 168, { &P12311,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Android 2.2.2|UberMoment 1.4.1.2 DL25; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032255 = {S86888, 161, { &P12311,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Android 2.2.2|UberMoment 1.4.1; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24599 = {S86889, 105, { &P13885,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-G400/1.0 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA64496 = {S86890, 106, { &P13885,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-G400L/1.0 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA63273 = {S86891, 74, { &P14459,&P7063,&P3606,&P18092 }};	/*Useragent - BlackBerry7130e/4.1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/104*/
const Device UA31178 = {S86892, 109, { &P14864,&P17484,&P3834,&P18092 }};	/*Useragent - SAMSUNG-GT-S8500C/1.0 SHP/VPP/R5 Dolfin/2.0 Nextreaming SMM-MMS/1.2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA61133 = {S86893, 115, { &P14864,&P17484,&P3834,&P18092 }};	/*Useragent - SAMSUNG-GT-S8500/S8500XXJF2 SHP/VPP/R5 Dolfin/2.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA50231 = {S86894, 32, { &P13202,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH02A(c100;TB;W24H16)*/
const Device UA50334 = {S86895, 49, { &P13231,&P17484,&P3976,&P18092 }};	/*Useragent - Mozilla/5.0 440x240 Samsung SCH-U940 NetFront/3.4*/
const Device UA64059 = {S86896, 101, { &P14571,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-M140/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA69198 = {S86897, 81, { &P13374,&P5448,&P4779,&P18092 }};	/*Useragent - HTC_P3700 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA279267 = {S86898, 67, { &P17595,&P17293,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; PPC Mac OS X 10.3.9; Tasman 1.0)*/
const Device UA315045 = {S86899, 143, { &P18077,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; EV-S110 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA624642 = {S86900, 100, { &P14714,&P5843,&P17347,&P18092 }};	/*Useragent - NokiaX3-02/5.0 (05.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax3-02/UC Browser8.0.3.107/69/444*/
const Device UA382217 = {S86901, 146, { &P16292,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; Galaxy Nexus Build/ICL53F) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75 Mobile Safari/535.7*/
const Device UA399047 = {S86902, 146, { &P16292,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; Galaxy Nexus Build/ITL41F) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75 Mobile Safari/535.7*/
const Device UA411748 = {S86903, 146, { &P16292,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM30B) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA440709 = {S86904, 146, { &P16292,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76B) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA441070 = {S86905, 146, { &P16292,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Galaxy Nexus Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA610081 = {S86906, 146, { &P16292,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76I) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA900541 = {S86907, 146, { &P16292,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75 Mobile Safari/535.7*/
const Device UA434053 = {S86908, 80, { &P15364,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; T-01B; Windows Phone 6.5.3.5)*/
const Device UA492684 = {S86909, 84, { &P15364,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; KDDI-TS01; Windows Phone 6.5.3.5)*/
const Device UA496803 = {S86910, 80, { &P15364,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; T-01A_6.5; Windows Phone 6.5)*/
const Device UA925695 = {S86911, 75, { &P15364,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; H-21; Windows Phone 6.5)*/
const Device UA925841 = {S86912, 83, { &P15364,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Toshiba/X02T; Windows Phone 6.5)*/
const Device UA550717 = {S86913, 90, { &P14845,&P5448,&P4777,&P18092 }};	/*Useragent - HTC_Touch_HD_T8282 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA599465 = {S86914, 39, { &P15364,&P18570,&P21570,&P18092 }};	/*Useragent - Mail/53 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA635415 = {S86915, 81, { &P15364,&P5690,&P20458,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:8.0.1) Gecko/X Firefox/8.0.1 SeaMonkey/2.5*/
const Device UA696676 = {S86916, 88, { &P15364,&P5690,&P20458,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:8.0.1) Gecko/X Firefox/8.0.1 SeaMonkey/2.5 Lightning/1.0*/
const Device UA756482 = {S86917, 60, { &P15364,&P5690,&P20458,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:8.0.1) Gecko/X SeaMonkey/2.5*/
const Device UA677542 = {S86918, 43, { &P18364,&P5129,&P24663,&P18092 }};	/*Useragent - NokiaC5-05/UC Browser7.9.0.102/50/355/UCWEB*/
const Device UA718190 = {S86919, 148, { &P15364,&P17819,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.4+ (KHTML, like Gecko) Version/5.0 Safari/535.4+ Ubuntu/11.10 (3.2.0-1~webupd8~oneiric) Epiphany/3.2.0*/
const Device UA2823686 = {S86920, 159, { &P15364,&P17819,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; fr-fr) AppleWebKit/531.2+ (KHTML, like Gecko) Version/5.0 Safari/531.2+ Ubuntu/11.10 (3.2.0-1~webupd8~oneiric) Epiphany/2.30.6*/
const Device UA725722 = {S86921, 144, { &P21846,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; es-es; PlayTabPro Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032826 = {S86922, 142, { &P21846,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; TAB-PLAYTABPRO Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA729580 = {S86923, 131, { &P17595,&P17295,&P22452,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1*/
const Device UA792870 = {S86924, 173, { &P14202,&P5477,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E15i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2765519 = {S86925, 177, { &P14202,&P5477,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E15i Build/2.1.1.A.0.6) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA1940553 = {S86926, 138, { &P22348,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; iDx7 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1942092 = {S86927, 61, { &P20320,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX256/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA1955524 = {S86928, 146, { &P22135,&P18066,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SmartTab1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1967333 = {S86929, 141, { &P13526,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Mytouch 4G Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1985478 = {S86930, 140, { &P13526,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC MyTouch4G Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1981087 = {S86931, 79, { &P12029,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN82)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA2366432 = {S86932, 137, { &P18677,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SBM104SH Build/S0030) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2032170 = {S86933, 156, { &P13286,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID2 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257675 = {S86934, 152, { &P13286,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID2 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2373375 = {S86935, 77, { &P14763,&P5776,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian72) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2167801 = {S86936, 146, { &P16336,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; U8800Pro Build/HuaweiU8800Pro) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2218360 = {S86937, 60, { &P15805,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player cingular_us / 1.0 / htc_holiday / 4.0.3*/
const Device UA2439570 = {S86938, 60, { &P15805,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player cingular_us / 1.0 / htc_holiday / 2.3.4*/
const Device UA2339186 = {S86939, 87, { &P19998,&P18110,&P20776,&P18092 }};	/*Useragent - MQQBrowser/25 (Linux; U; 4.0.3; zh-cn; HUAWEI C8812 Build/C8812V100R001C92B926;480*800)*/
const Device UA2643086 = {S86940, 87, { &P19998,&P18110,&P20776,&P18092 }};	/*Useragent - MQQBrowser/25 (Linux; U; 4.0.3; zh-cn; HUAWEI C8812 Build/C8812V100R001C92B921;480*800)*/
const Device UA2353943 = {S86941, 155, { &P12616,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; WT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2404692 = {S86942, 98, { &P14980,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-b7722) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2654379 = {S86943, 96, { &P14980,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; fa; samsung-gt-b7722i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2481006 = {S86944, 163, { &P14877,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA3070381 = {S86945, 165, { &P14877,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2402952 = {S86946, 125, { &P14972,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s3370) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2469230 = {S86947, 161, { &P20265,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A52 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2466915 = {S86948, 94, { &P16600,&P22784,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxx600) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2727274 = {S86949, 94, { &P16600,&P22784,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxx600) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2467394 = {S86950, 89, { &P17595,&P17295,&P20457,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:17.0) Gecko/X Firefox/17.0 SeaMonkey/2.14.1*/
const Device UA2484449 = {S86951, 165, { &P18621,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; LG-E612 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2486824 = {S86952, 132, { &P20463,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP7100D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2503107 = {S86953, 137, { &P20239,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Enjoy 10 Plus Build/1.14ST) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2504396 = {S86954, 157, { &P16356,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2554904 = {S86955, 157, { &P16356,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2506367 = {S86956, 135, { &P23383,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; H7100 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2522579 = {S86957, 149, { &P20075,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-Xperia S Build/GINGERBREAD_V13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2758483 = {S86958, 145, { &P20075,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-Xperia S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2561259 = {S86959, 168, { &P18449,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; ST27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2567027 = {S86960, 137, { &P14017,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P990 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2578935 = {S86961, 103, { &P15234,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.4.0.560; en-us; 9330/4.4.0.560) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2983075 = {S86962, 103, { &P15234,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 4.4.0.562; en-us; 9330/4.4.0.562) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2579815 = {S86963, 156, { &P21944,&P18066,&P18062,&P18092 }};	/*Useragent - Orbit 5300 Linux/3.0.13 Android/4.0 Release/11.19.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2595708 = {S86964, 150, { &P22148,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; PIRANHA BUSINESS II TAB 9" Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2836064 = {S86965, 151, { &P22148,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; PIRANHA BUSINESS II TAB 9.0 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2596098 = {S86966, 110, { &P22113,&P5629,&P15760,&P18092 }};	/*Useragent - HS-T830_TD/1.0 Android/2.3 Release/09.27.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2598006 = {S86967, 87, { &P23381,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A5) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2599420 = {S86968, 160, { &P21334,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A8 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2659473 = {S86969, 93, { &P16356,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-N7000) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2666850 = {S86970, 141, { &P20001,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S720 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668981 = {S86971, 159, { &P12798,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-CN; LT18i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2675565 = {S86972, 137, { &P19856,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LS970 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2678425 = {S86973, 135, { &P23546,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AD683G Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707938 = {S86974, 136, { &P14872,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-D700 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712727 = {S86975, 141, { &P13124,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PantechP9070 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2737890 = {S86976, 154, { &P15028,&P5096,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-US; LG-P350 Build/FRG83) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2737910 = {S86977, 151, { &P20146,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9082 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2799225 = {S86978, 150, { &P21334,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A8 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752551 = {S86979, 103, { &P11932,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6210 Navigator) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2769430 = {S86980, 130, { &P21202,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-P500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2783295 = {S86981, 150, { &P20160,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2783789 = {S86982, 142, { &P21623,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 6 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893024 = {S86983, 143, { &P21623,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 6A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784252 = {S86984, 133, { &P15184,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P920 Build/ICS) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2794681 = {S86985, 160, { &P21756,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; Orange_infinity_996 Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2854306 = {S86986, 173, { &P12280,&P19826,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_2 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B146 Safari/7534.48.3 UCBrowser8.5.1.228/44/355*/
const Device UA2891192 = {S86987, 173, { &P12280,&P19826,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10B329 Safari/7534.48.3 UCBrowser8.5.1.228/44/355*/
const Device UA2855850 = {S86988, 133, { &P21188,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MPDC97 BT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2937419 = {S86989, 64, { &P15364,&P17285,&P22170,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:22.0) Gecko/X Firefox/22.0 slurp*/
const Device UA2958217 = {S86990, 151, { &P21628,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8552 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3122743 = {S86991, 154, { &P21628,&P18111,&P22879,&P18092 }};	/*Useragent - MQQBrowser/4.3/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8552 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2965387 = {S86992, 157, { &P14959,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9103 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2970816 = {S86993, 98, { &P14764,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; SonyEricssonU1) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA2999854 = {S86994, 59, { &P12280,&P22882,&P17470,&P18092 }};	/*Useragent - Appcelerator Titanium/1.7.2 (iPhone/7.0; iPhone OS; en_US;)*/
const Device UA3003465 = {S86995, 165, { &P14433,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9003 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/galaxysl*/
const Device UA3012774 = {S86996, 147, { &P16976,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT300 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3079600 = {S86997, 149, { &P16976,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT300SE Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3014704 = {S86998, 93, { &P15575,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9100) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3016629 = {S86999, 136, { &P24101,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ADR930L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3021336 = {S87000, 138, { &P21019,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E270K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022262 = {S87001, 146, { &P24545,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Starmobile KNIGHT Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022620 = {S87002, 133, { &P15184,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P920 Build/ICS) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022642 = {S87003, 137, { &P21490,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A860L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022887 = {S87004, 92, { &P12280,&P18117,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 5_1_1; zh-CN; iPh4,1) U2/1.0.0 UCBrowser/9.2.0.312 U2/1.0.0 Mobile*/
const Device UA3058893 = {S87005, 92, { &P12280,&P18117,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 5_1_1; zh-CN; iPh3,1) U2/1.0.0 UCBrowser/9.2.0.312 U2/1.0.0 Mobile*/
const Device UA3023375 = {S87006, 137, { &P18595,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I8190 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3062754 = {S87007, 157, { &P22756,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; C1604 Build/11.3.A.2.1) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3027905 = {S87008, 130, { &P20446,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; F-05E Build/V08R32D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3028096 = {S87009, 128, { &P19971,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MT7001 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3038052 = {S87010, 153, { &P20175,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI 1S Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3081053 = {S87011, 137, { &P20075,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xperia S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110307 = {S87012, 132, { &P20231,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A21 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083647 = {S87013, 148, { &P24158,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Telenor Smart Pro 2 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111649 = {S87014, 136, { &P21525,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SBM203SH Build/S0022) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093563 = {S87015, 126, { &P19775,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3115776 = {S87016, 112, { &P19974,&P5043,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia111/03.32; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA3130948 = {S87017, 137, { &P21628,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA17225 = {S87018, 53, { &P15364,&P17017,&P4222,&P18092 }};	/*Useragent - OPWV-SDK UP.Browser/7.0.2.3.119 (GUI) MMP/2.0 Push/PO*/
const Device UA21345 = {S87019, 87, { &P15364,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-CF75/10 UP.Browser/7.0.2.2.d.5(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23886 = {S87020, 86, { &P15364,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-S65/50 UP.Browser/7.0.2.2.d.3(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA26699 = {S87021, 87, { &P15364,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-CF75/22 UP.Browser/7.0.2.2.d.7(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA26761 = {S87022, 88, { &P15364,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-CX65/25 UP.Browser/7.0.0.1.c.3 (GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA43648 = {S87023, 87, { &P15364,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-ME75/22 UP.Browser/7.0.2.2.d.7(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53466 = {S87024, 90, { &P15364,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-C65/43 UP.Browser/7.0.2.2.d.1.100(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA60871 = {S87025, 86, { &P15364,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-C72/22 UP.Browser/7.0.2.2.d.7(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61477 = {S87026, 87, { &P15364,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-ME75/19 UP.Browser/7.0.2.2.d.7(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA81339 = {S87027, 87, { &P15364,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-CF75/07 UP.Browser/7.0.2.2.d.5(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA81433 = {S87028, 86, { &P15364,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-C72/16 UP.Browser/7.0.2.2.d.7(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87571 = {S87029, 87, { &P15364,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-C65/12 UP.Browser/7.0.0.1.c.3 (GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA48847 = {S87030, 75, { &P12718,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry8980/5.0.0.978 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/1*/
const Device UA76081 = {S87031, 83, { &P12718,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry8980_CMCC/5.0.0.1055 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/204*/
const Device UA30349 = {S87032, 145, { &P12305,&P5448,&P4808,&P18092 }};	/*Useragent - LG-GW825/V10c IEMobile/8.12 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA61471 = {S87033, 136, { &P12726,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-i569 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023821 = {S87034, 124, { &P12726,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; SCH-i569 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA62382 = {S87035, 72, { &P14403,&P22839,&P4223,&P18092 }};	/*Useragent - zte-S2x/1.0 NetFront/3.5 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA67933 = {S87036, 47, { &P11684,&P5129,&P4759,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UC Browser7.6.1.82/50/352*/
const Device UA67964 = {S87037, 35, { &P11929,&P5129,&P4759,&P18092 }};	/*Useragent - Nokia5230/UC Browser7.6.1.82/50/351*/
const Device UA68353 = {S87038, 37, { &P11929,&P5129,&P4759,&P18092 }};	/*Useragent - Nokia5230-c/UC Browser7.6.1.82/50/355*/
const Device UA89943 = {S87039, 31, { &P17727,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F08B(c500;TB;W24H16)*/
const Device UA90735 = {S87040, 59, { &P16871,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxQ5/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA202534 = {S87041, 38, { &P12004,&P5890,&P4777,&P18092 }};	/*Useragent - NOKIAN95 8GB/UC Browser7.7.1.88/28/444*/
const Device UA321194 = {S87042, 34, { &P12004,&P5890,&P4777,&P18092 }};	/*Useragent - NOKIAN95/UC Browser7.7.1.88/28/444*/
const Device UA565855 = {S87043, 34, { &P12004,&P5890,&P4777,&P18092 }};	/*Useragent - nokian95/UC Browser7.7.1.88/69/444*/
const Device UA2804701 = {S87044, 38, { &P12004,&P5890,&P4777,&P18092 }};	/*Useragent - nokian95_8gb/UC Browser7.7.1.88/69/444*/
const Device UA616388 = {S87045, 82, { &P17465,&P7133,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; Opera Tablet/11648; U; en) Presto/2.7.81 Version/11.00*/
const Device UA458739 = {S87046, 135, { &P17950,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MID7015A Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458973 = {S87047, 59, { &P17950,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; MID7015A Build/FRF85B)*/
const Device UA575381 = {S87048, 61, { &P20321,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX270/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA695675 = {S87049, 62, { &P15364,&P5690,&P20459,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:9.0.1) Gecko/X SeaMonkey/2.6.1*/
const Device UA1938028 = {S87050, 90, { &P15364,&P5690,&P20459,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:9.0.1) Gecko/X Firefox/9.0.1 SeaMonkey/2.6.1 Lightning/1.1*/
const Device UA2309122 = {S87051, 76, { &P15364,&P5690,&P20459,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; rv:9.0.1) Gecko/X Firefox/9.0.1 SeaMonkey/2.6.1*/
const Device UA606801 = {S87052, 125, { &P15364,&P17285,&P22427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.152 Safari/535.19 Comodo_Dragon/18.1.2.0*/
const Device UA607195 = {S87053, 132, { &P15364,&P17285,&P22427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.152 Safari/535.19 Comodo_Dragon/18.1.2.0*/
const Device UA688549 = {S87054, 79, { &P14453,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE51)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA705706 = {S87055, 144, { &P21073,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; Ultimate10 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024275 = {S87056, 141, { &P21073,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; nl-nl; ATP7526 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA777494 = {S87057, 137, { &P13286,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROID2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA819128 = {S87058, 137, { &P13286,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032263 = {S87059, 137, { &P13286,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2742974 = {S87060, 137, { &P13286,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID2 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2979868 = {S87061, 154, { &P14557,&P5843,&P24667,&P18092 }};	/*Useragent - Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia6303classic) UCBrowser8.2.0.132/69/452/UCWEB Mobile*/
const Device UA854570 = {S87062, 136, { &P17595,&P17295,&P22455,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.11 (KHTML, like Gecko)  Iron/17.0.1000.0 Chrome/17.0.1000.0 Safari/535.11*/
const Device UA857613 = {S87063, 104, { &P23556,&P5096,&P3508,&P18092 }};	/*Useragent - LenovoA66t_TD/1.0 Android/2.2 (Linux; Android 2.2) Release/12.20.2010 Browser/WAP 2.0 AppleWebKit/530.17*/
const Device UA1922090 = {S87064, 128, { &P23556,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Lenovo A66t Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA901889 = {S87065, 49, { &P23144,&P18570,&P23081,&P18092 }};	/*Useragent - InstacastHD/1.1.3 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA924001 = {S87066, 168, { &P14183,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I997 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1888882 = {S87067, 83, { &P11734,&P5448,&P4777,&P18092 }};	/*Useragent - HTC_TyTN_II Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA1896490 = {S87068, 135, { &P13278,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; es-es; X10mini Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022243 = {S87069, 135, { &P13278,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; X10mini Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1915813 = {S87070, 147, { &P17346,&P17146,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA2439305 = {S87071, 145, { &P17346,&P17146,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 5_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16*/
const Device UA1990222 = {S87072, 146, { &P21429,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Woxter Tablet PC 75CXi Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1992543 = {S87073, 69, { &P15364,&P5690,&P22538,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:12.0) Gecko/X Firefox/12.0 Kylo/1.1.1*/
const Device UA1999156 = {S87074, 154, { &P18283,&P18066,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025057 = {S87075, 136, { &P20499,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Magnum2X Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032696 = {S87076, 149, { &P15991,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2869967 = {S87077, 150, { &P15991,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2352261 = {S87078, 137, { &P18676,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SBM106SH Build/S0020) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2039813 = {S87079, 50, { &P17595,&P18120,&P20787,&P18092 }};	/*Useragent - MQQBrowser/36 (iOS 4; U; CPU like Mac OS X; zh-cn)*/
const Device UA2042436 = {S87080, 51, { &P17595,&P18120,&P20787,&P18092 }};	/*Useragent - MQQBrowser/36 (iOS 4S; U; CPU like Mac OS X; zh-cn)*/
const Device UA2257459 = {S87081, 164, { &P14967,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2211026 = {S87082, 80, { &P15364,&P17285,&P17825,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/3.5; Windows NT 6.0) KHTML/3.5.6 (like Gecko)*/
const Device UA2934063 = {S87083, 96, { &P18221,&P5843,&P17347,&P18092 }};	/*Useragent - Nokia302/5.0 (14.78) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia302/UC Browser8.0.3.107/69/444*/
const Device UA2222963 = {S87084, 135, { &P21165,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ZP300 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2227392 = {S87085, 249, { &P15785,&P5843,&P18326,&P18092 }};	/*Useragent - Nokia201/2.0 (11.21) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.2.1.144/70/355*/
const Device UA2342987 = {S87086, 135, { &P15880,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MB865 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2354809 = {S87087, 87, { &P17595,&P18207,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.7373(Macintosh;Mac OS X (10.8.2);en;kml:2.2;client:Free;type:default)*/
const Device UA2549146 = {S87088, 87, { &P17595,&P18207,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Macintosh;Mac OS X (10.8.2);en;kml:2.2;client:Free;type:default)*/
const Device UA2405881 = {S87089, 99, { &P16384,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s5380k) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2595421 = {S87090, 99, { &P16384,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s5380d) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2455732 = {S87091, 166, { &P18679,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-B5330 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2401445 = {S87092, 113, { &P15364,&P7133,&P22525,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.22 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/535.22 Midori/0.4*/
const Device UA2417773 = {S87093, 137, { &P20396,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE R22 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2418374 = {S87094, 44, { &P17595,&P18207,&P18379,&P18092 }};	/*Useragent - MacOutlook/14.10.0.X (Intel Mac OS X 10.8.2)*/
const Device UA2989547 = {S87095, 44, { &P17595,&P18207,&P18379,&P18092 }};	/*Useragent - MacOutlook/14.10.0.X (Intel Mac OS X 10.8.3)*/
const Device UA3009611 = {S87096, 44, { &P17595,&P18207,&P18379,&P18092 }};	/*Useragent - MacOutlook/14.10.0.X (Intel Mac OS X 10.8.4)*/
const Device UA2458655 = {S87097, 153, { &P16503,&P5204,&P24673,&P18092 }};	/*Useragent - Nokia7310/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia7310) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2485535 = {S87098, 165, { &P14186,&P5096,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2463035 = {S87099, 81, { &P18481,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-C3520/C3520DDKK4 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA2467287 = {S87100, 138, { &P12697,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; shooteru Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2570936 = {S87101, 138, { &P12697,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; shooteru Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2467569 = {S87102, 134, { &P23802,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; W30(QuadCore) Build/W30) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2469209 = {S87103, 133, { &P22156,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GEM10312S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2504427 = {S87104, 133, { &P22156,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GEM10312S Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2481237 = {S87105, 137, { &P20986,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Sen S2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483846 = {S87106, 133, { &P20893,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; LETAB1003 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2485249 = {S87107, 138, { &P20289,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; HTC6435LVW Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2502530 = {S87108, 130, { &P21287,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; MP1043 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2506289 = {S87109, 134, { &P17595,&P23108,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PowerPC MorphOS 3.1; Odyssey Web Browser; rv:1.18) AppleWebKit/536.9 (KHTML, like Gecko) OWB/1.18 Safari/536.9*/
const Device UA2580744 = {S87110, 136, { &P17595,&P23108,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PowerPC MorphOS 3.1; Odyssey Web Browser; rv:1.19) AppleWebKit/537.25 (KHTML, like Gecko) OWB/1.19 Safari/537.25*/
const Device UA2735007 = {S87111, 136, { &P17595,&P23108,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PowerPC MorphOS 3.1; Odyssey Web Browser; rv:1.20) AppleWebKit/537.33 (KHTML, like Gecko) OWB/1.20 Safari/537.33*/
const Device UA2950797 = {S87112, 96, { &P15785,&P5843,&P17347,&P18092 }};	/*Useragent - Nokia201/2.0 (11.64) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia201/UC Browser8.0.3.107/69/444*/
const Device UA2564827 = {S87113, 168, { &P11950,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2565709 = {S87114, 169, { &P14969,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2567666 = {S87115, 156, { &P18680,&P18066,&P21418,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2601589 = {S87116, 133, { &P20807,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; B92 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2713628 = {S87117, 136, { &P21005,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; F-10D Build/V21R48A) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2610958 = {S87118, 87, { &P13452,&P5629,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.4.5) (GT-P1000;samsung;GT-P1000;2.3.3/GINGERBREAD)*/
const Device UA2638861 = {S87119, 74, { &P15364,&P5690,&P22319,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; de; rv:1.8.1.8) Gecko/X SeaMonkey*/
const Device UA2651132 = {S87120, 190, { &P13308,&P17485,&P24671,&P18092 }};	/*Useragent - SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonj105i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2653301 = {S87121, 155, { &P18747,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A840S Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2779724 = {S87122, 191, { &P18747,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A840S Build/Version Metro Sky By Hanzoo1504 Vietnamese; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2657744 = {S87123, 130, { &P23639,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MK808B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2660713 = {S87124, 88, { &P20265,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A52) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2662644 = {S87125, 89, { &P21514,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; N350) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2676936 = {S87126, 137, { &P21028,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Smartfren Tab 7 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2861135 = {S87127, 133, { &P21028,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; StarTab 715 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2704261 = {S87128, 93, { &P18679,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-B5330) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2706103 = {S87129, 113, { &P18221,&P22723,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia3020/14.53; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2714082 = {S87130, 144, { &P21333,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; INO_ONE Build/INO_ONEINO_ONE) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2863466 = {S87131, 141, { &P21333,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; INO_ONE_PLUS Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2737532 = {S87132, 153, { &P13964,&P5096,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-US; GT-I5510 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2739356 = {S87133, 155, { &P18680,&P18066,&P22588,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2752609 = {S87134, 97, { &P20005,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A65) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2765142 = {S87135, 120, { &P14557,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia6303classic/09.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2770985 = {S87136, 130, { &P13452,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2772283 = {S87137, 148, { &P20947,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCEVOV4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2773129 = {S87138, 134, { &P19864,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 80XSK Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2778866 = {S87139, 177, { &P17036,&P5629,&P15760,&P18092 }};	/*Useragent - DESAY_TS808_TD/1.0 Android/2.3.5 (Linux; Android 2.3.5) Release/7.25.2012 Mozilla/5.0(Linux;U;Android 2.3.5) AppleWebKit/533.1(KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2872533 = {S87140, 159, { &P15047,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; SK17i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2782780 = {S87141, 144, { &P21333,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; INO_ONE Build/INO_ONEINO_ONE) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782911 = {S87142, 146, { &P19863,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799103 = {S87143, 158, { &P18724,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-H Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2938017 = {S87144, 158, { &P18724,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-F Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3015131 = {S87145, 160, { &P18737,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Find 5 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/find5*/
const Device UA2957808 = {S87146, 138, { &P23719,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; DIT4350 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879164 = {S87147, 93, { &P21628,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8552) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2891111 = {S87148, 137, { &P24528,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E987 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2893131 = {S87149, 160, { &P23011,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; iris405 Build/LAVAIRIS405) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2942488 = {S87150, 130, { &P24130,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; MD-717 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2945475 = {S87151, 98, { &P19862,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Micromax A110) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3021870 = {S87152, 127, { &P20127,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT270 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA2964357 = {S87153, 156, { &P22382,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8262 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3022680 = {S87154, 130, { &P24100,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CAL21 Build/AX) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2996977 = {S87155, 144, { &P21629,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Lenovo S920_ROW Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3009343 = {S87156, 140, { &P23683,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; Prism II Build/HuaweiU8686) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24 TT/4.1.1*/
const Device UA3011279 = {S87157, 133, { &P24129,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; A900 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3013005 = {S87158, 164, { &P12841,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Vivid 4G Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/holiday*/
const Device UA3022221 = {S87159, 95, { &P20004,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Karbonn_A2) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3022200 = {S87160, 138, { &P21495,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E220S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021431 = {S87161, 136, { &P14969,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Desire S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023986 = {S87162, 140, { &P21980,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; UBISLATE7C+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024176 = {S87163, 141, { &P21980,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; UBISLATE 7C+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024215 = {S87164, 141, { &P15364,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; AOSP on Toro Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078731 = {S87165, 148, { &P15364,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; AOSP on geeb_att_us Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092507 = {S87166, 139, { &P15364,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; VirtualBox Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3080691 = {S87167, 93, { &P21628,&P18111,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8552) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3026917 = {S87168, 92, { &P15209,&P5768,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6233) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3060417 = {S87169, 130, { &P18702,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N8013 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Safari/537.36*/
const Device UA3113633 = {S87170, 137, { &P18458,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I200 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094508 = {S87171, 130, { &P13452,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094369 = {S87172, 142, { &P22822,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KPN Smart 300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081194 = {S87173, 135, { &P21523,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SH930W Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094013 = {S87174, 134, { &P19864,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 80XSK Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094723 = {S87175, 130, { &P19874,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-i705 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3065259 = {S87176, 135, { &P21330,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PMP5197DULTRA Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Safari/537.36*/
const Device UA3093349 = {S87177, 143, { &P11747,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire 500 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3130583 = {S87178, 136, { &P24039,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9190 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131303 = {S87179, 129, { &P16976,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT300SE Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3132707 = {S87180, 127, { &P16976,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT300 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA17130 = {S87181, 110, { &P16045,&P17485,&P3430,&P18092 }};	/*Useragent - Opera/9.5 (Microsoft Windows; PPC; Opera Mobi; U) SonyEricssonX1a/R3AA Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA5474 = {S87182, 202, { &P12263,&P5062,&P3441,&P18092 }};	/*Useragent - LG-GW620 Mozilla/5.0 (Linux; U; Android 1.6) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Java/Jbed/7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA53626 = {S87183, 62, { &P12041,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia3510i/1.0 (05.30) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA54548 = {S87184, 52, { &P14237,&P17484,&P4902,&P18092 }};	/*Useragent - SCH-M369/ POLARIS/6.15 (GUI; compatible; UP.Browser)*/
const Device UA62295 = {S87185, 75, { &P13421,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_TouchDual Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA63121 = {S87186, 66, { &P14479,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GD350/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA1872481 = {S87187, 45, { &P14188,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia5233/SymbianOS/9.1 Series60/3.0 3gpp-gba*/
const Device UA173462 = {S87188, 130, { &P16534,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; Vega Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA255053 = {S87189, 100, { &P15364,&P18346,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.120 Safari/535.2*/
const Device UA1901613 = {S87190, 64, { &P15216,&P6173,&P3417,&P18092 }};	/*Useragent - BlackBerry8520/4.2.0.135 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA451125 = {S87191, 33, { &P17595,&P7480,&P22897,&P18091 }};	/*Useragent - RSSOwl/2.1.2.X (Macintosh; U; en)*/
const Device UA2224085 = {S87192, 33, { &P17595,&P7480,&P22897,&P18091 }};	/*Useragent - RSSOwl/2.1.4.X (Macintosh; U; en)*/
const Device UA2528111 = {S87193, 33, { &P17595,&P7480,&P22897,&P18091 }};	/*Useragent - RSSOwl/2.1.6.X (Macintosh; U; en)*/
const Device UA476301 = {S87194, 101, { &P15364,&P18110,&P17260,&P18091 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2) Gecko Firefox/3.5 (ITABot; robot@itasoftware.com)*/
const Device UA684199 = {S87195, 66, { &P15364,&P5690,&P17203,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:2.0.1) Gecko/X Firefox/4.0.1 slurp*/
const Device UA1870230 = {S87196, 62, { &P15364,&P5690,&P17203,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:2.0) Gecko/X Firefox/4.0 slurp*/
const Device UA731573 = {S87197, 102, { &P16237,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i937)*/
const Device UA778013 = {S87198, 151, { &P17308,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/4.5.1_57_DX8-51)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2800240 = {S87199, 152, { &P17308,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA924787 = {S87200, 78, { &P17595,&P17295,&P18515,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:14.0) Gecko/X Thunderbird/14.0*/
const Device UA1991745 = {S87201, 132, { &P14612,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A777 Build/A777) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2028251 = {S87202, 134, { &P14612,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A7 ART Build/A777) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2314004 = {S87203, 144, { &P14612,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GOCLEVER TAB T75 Build/A777) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2338604 = {S87204, 81, { &P11964,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE75-1)/UC Browser8.5.0.183/28/444/UCWEB Mobile*/
const Device UA2024538 = {S87205, 140, { &P21719,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; htc_jewel Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028336 = {S87206, 140, { &P21719,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; htc_jewel Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030114 = {S87207, 143, { &P21719,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; htc_jewel 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033649 = {S87208, 65, { &P20120,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;A45;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA2257485 = {S87209, 133, { &P22065,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SmartTab2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2258496 = {S87210, 169, { &P15047,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; SK17i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2346579 = {S87211, 132, { &P23942,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; JMI T970 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2355670 = {S87212, 135, { &P18296,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; One V Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2414117 = {S87213, 157, { &P21094,&P18066,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A789 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2419925 = {S87214, 141, { &P20482,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB A104 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2441042 = {S87215, 54, { &P13977,&P5890,&P21761,&P18092 }};	/*Useragent - iBrowser/2.5 (NokiaN73(08);SymbianOS/9.1 Series60/3.0)*/
const Device UA2442558 = {S87216, 57, { &P13977,&P5890,&P21761,&P18092 }};	/*Useragent - iBrowser/2.5 (NokiaN73(Z5.01);SymbianOS/9.1 Series60/3.0)*/
const Device UA2589020 = {S87217, 54, { &P13977,&P5890,&P21761,&P18092 }};	/*Useragent - iBrowser/2.5 (NokiaN73(12);SymbianOS/9.1 Series60/3.0)*/
const Device UA2455416 = {S87218, 137, { &P24563,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; S21 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2458776 = {S87219, 143, { &P20736,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB A104.2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2484563 = {S87220, 155, { &P14714,&P5843,&P24675,&P18092 }};	/*Useragent - NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiax3-02) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2485208 = {S87221, 169, { &P15047,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; SK17i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2505538 = {S87222, 137, { &P22067,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; IMO Z7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2536785 = {S87223, 151, { &P20983,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9308 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2583224 = {S87224, 131, { &P20241,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5570C Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2583778 = {S87225, 138, { &P18216,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Sony Tablet P Build/TISU0144) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2601152 = {S87226, 134, { &P21933,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M729 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2604125 = {S87227, 136, { &P18470,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 101G10 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2610209 = {S87228, 81, { &P15985,&P17979,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.3.10) (GT-P6200;samsung;GT-P6200;3.2/HTJ85B)*/
const Device UA2668588 = {S87229, 165, { &P21351,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HUAWEI C8813 Build/HuaweiC8813) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2668820 = {S87230, 178, { &P18113,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2675326 = {S87231, 134, { &P23894,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V1277 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676580 = {S87232, 111, { &P18718,&P18148,&P24633,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-846_im_india_233) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile*/
const Device UA2710435 = {S87233, 130, { &P19874,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-i705 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2718566 = {S87234, 134, { &P21909,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-03E Build/10.1.E.0.250) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2737857 = {S87235, 92, { &P15209,&P5768,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6233) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2738093 = {S87236, 155, { &P14433,&P5096,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-I9003 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752013 = {S87237, 159, { &P12258,&P17483,&P24677,&P18092 }};	/*Useragent - LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gs290) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2752591 = {S87238, 156, { &P13107,&P5043,&P24677,&P18092 }};	/*Useragent - NokiaC2-06/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-06) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2892826 = {S87239, 152, { &P18221,&P5843,&P24677,&P18092 }};	/*Useragent - Nokia302/5.0 (14.78) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia302) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2765814 = {S87240, 139, { &P23388,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TSINTECH Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867686 = {S87241, 150, { &P20513,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A5 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2768855 = {S87242, 94, { &P15118,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaX7-00) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2769030 = {S87243, 136, { &P22804,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P713 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2772535 = {S87244, 131, { &P15102,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ605 Build/I.7.1-39.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2780149 = {S87245, 89, { &P16598,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A500) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2799263 = {S87246, 122, { &P17346,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.2*/
const Device UA2955030 = {S87247, 120, { &P17346,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0*/
const Device UA2962029 = {S87248, 122, { &P17346,&P19826,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Flipboard/2.0.4*/
const Device UA2799331 = {S87249, 155, { &P21648,&P18598,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9505 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2868089 = {S87250, 161, { &P21882,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax P362 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2868958 = {S87251, 140, { &P23271,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenco CoolTab-80 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2869043 = {S87252, 147, { &P20931,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; iBall Slide 3G7271 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892768 = {S87253, 141, { &P20931,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; iBall Slide 3G 7334 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2879484 = {S87254, 137, { &P24098,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AS870 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2944553 = {S87255, 190, { &P18702,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N8013 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.5/580,2.0.5.580,2013-06-26 22:40, -0400, us*/
const Device UA2964215 = {S87256, 114, { &P14669,&P5843,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-00/04.90; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA3021373 = {S87257, 137, { &P18674,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7005 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021595 = {S87258, 138, { &P15850,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LU6200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2963054 = {S87259, 133, { &P18509,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; LG-MS770 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3021654 = {S87260, 136, { &P18499,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ISW11SC Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022518 = {S87261, 134, { &P19985,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SH-02E Build/S4100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2966602 = {S87262, 157, { &P18582,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT28h Build/6.1.E.3.7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3021377 = {S87263, 135, { &P23165,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; H7500+ Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2968808 = {S87264, 165, { &P13064,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3023584 = {S87265, 92, { &P15028,&P5096,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; LG-P350) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3022509 = {S87266, 93, { &P19847,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S6012) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3059864 = {S87267, 157, { &P21506,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3021785 = {S87268, 135, { &P23130,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; H2000+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022866 = {S87269, 139, { &P15147,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST15i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021052 = {S87270, 94, { &P12236,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6700s) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3031208 = {S87271, 163, { &P19862,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Micromax A110Q Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3023485 = {S87272, 143, { &P23712,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; i-mobile IQ XA Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3063831 = {S87273, 156, { &P21356,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; TECNO N3 Build/master) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3033255 = {S87274, 156, { &P22871,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9295 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3057930 = {S87275, 167, { &P15364,&P5690,&P17193,&P18089 }};	/*Useragent - Googlebot/2.1 (+http://www.googlebot.com/bot.html; MSIE 7.0; AOL 9.1; AOLBuild 4334.5009; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.;c2VvLWNyYXdsZXI=*/
const Device UA3057951 = {S87276, 167, { &P15364,&P5690,&P17193,&P18089 }};	/*Useragent - Googlebot/2.1 (+http://www.googlebot.com/bot.html; MSIE 7.0; Windows NT 5.1; IEMB3; InfoPath.2; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3;c2VvLWNyYXdsZXI=*/
const Device UA3078126 = {S87277, 167, { &P15364,&P5690,&P17193,&P18089 }};	/*Useragent - Googlebot/2.1 (+http://www.googlebot.com/bot.html; MSIE 7.0; Windows NT 5.1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; Media Center PC 4.0; .NET CLR 2.0.5;c2VvLWNyYXdsZXI=*/
const Device UA3058902 = {S87278, 140, { &P15767,&P17090,&P20784,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/8J2 Safari/7534.48.3*/
const Device UA3079696 = {S87279, 140, { &P15767,&P17090,&P20784,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/8L1 Safari/7534.48.3*/
const Device UA3060584 = {S87280, 164, { &P12132,&P17485,&P24679,&P18092 }};	/*Useragent - SonyEricssonJ10i2/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj10i2) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3094116 = {S87281, 140, { &P20001,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S720 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094797 = {S87282, 134, { &P23124,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A919i Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA64091 = {S87283, 104, { &P13733,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-A747/A747UCHB3 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA24304 = {S87284, 41, { &P13896,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-U350 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA33260 = {S87285, 135, { &P14574,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Nexus S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA42050 = {S87286, 135, { &P14574,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Nexus S Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52322 = {S87287, 133, { &P14574,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60904 = {S87288, 135, { &P14574,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Nexus S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019084 = {S87289, 136, { &P14574,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Nexus S Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020374 = {S87290, 133, { &P14574,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus S Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49365 = {S87291, 43, { &P16365,&P17017,&P3807,&P18092 }};	/*Useragent - Huawei/1.0/G6150/Browser/obigo-browser/q03c*/
const Device UA56771 = {S87292, 34, { &P14358,&P5890,&P4715,&P18092 }};	/*Useragent - NokiaE63/UC Browser7.4.0.65/28/352*/
const Device UA62735 = {S87293, 23, { &P14443,&P19954,&P3816,&P18092 }};	/*Useragent - SEC-SGHX660/1.0 TSS/2.5*/
const Device UA66858 = {S87294, 28, { &P18088,&P22832,&P18246,&P18092 }};	/*Useragent - Videocon_VC1605_MAUI_Wap 2.0*/
const Device UA90731 = {S87295, 38, { &P18088,&P22832,&P18246,&P18092 }};	/*Useragent - Videocon_V1675_Wap MAUI WAP Browser2.0*/
const Device UA2809916 = {S87296, 31, { &P18088,&P22832,&P18246,&P18092 }};	/*Useragent - Videocon V1510 MAUI WAP Browser*/
const Device UA452419 = {S87297, 98, { &P12108,&P5843,&P17338,&P18092 }};	/*Useragent - Nokia7230/5.0 (06.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7230/UC Browser7.9.0.102/69/352*/
const Device UA398389 = {S87298, 47, { &P12142,&P17485,&P24665,&P18092 }};	/*Useragent - SonyEricssonU5/UC Browser8.0.4.121/50/352/UCWEB*/
const Device UA590039 = {S87299, 94, { &P17595,&P17295,&P20460,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:10.0.1) Gecko/X Firefox/10.0.1 SeaMonkey/2.7.1*/
const Device UA599655 = {S87300, 45, { &P23144,&P18570,&P18376,&P18092 }};	/*Useragent - Instacast/1.5 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA601844 = {S87301, 128, { &P15767,&P18117,&P20774,&P18092 }};	/*Useragent - MQQBrowser/2.3 Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3*/
const Device UA757129 = {S87302, 130, { &P15767,&P18117,&P20774,&P18092 }};	/*Useragent - MQQBrowser/2.3 Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA685191 = {S87303, 71, { &P15364,&P16110,&P18367,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:12.0) Gecko/X Thunderbird/12.0.1*/
const Device UA816150 = {S87304, 45, { &P12581,&P5055,&P17338,&P18092 }};	/*Useragent - nokia500/UC Browser7.9.0.102/69/352  3gpp-gba*/
const Device UA911154 = {S87305, 121, { &P17346,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.4*/
const Device UA2003851 = {S87306, 121, { &P17346,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.6*/
const Device UA2235459 = {S87307, 121, { &P17346,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.7*/
const Device UA2403522 = {S87308, 121, { &P17346,&P18117,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.9.8*/
const Device UA2682477 = {S87309, 83, { &P15364,&P16110,&P22337,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:7.0.1) Gecko/X Firefox/7.0.1 CometBird/7.0.1*/
const Device UA2026995 = {S87310, 135, { &P22078,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; eZee'Tab701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1990693 = {S87311, 137, { &P20267,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; P300(Funbook) Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2024289 = {S87312, 136, { &P22105,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HS-EG900 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2420754 = {S87313, 190, { &P16356,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, gb*/
const Device UA2375585 = {S87314, 85, { &P12375,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIANokia N81)/UC Browser8.5.0.183/28/444/UCWEB Mobile*/
const Device UA2375667 = {S87315, 89, { &P12375,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIANokia N81 8GB)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2161304 = {S87316, 52, { &P13552,&P5890,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 (NokiaC6-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA2397974 = {S87317, 104, { &P15364,&P7133,&P18519,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.6.1 Safari/534.34*/
const Device UA2456088 = {S87318, 182, { &P11793,&P5768,&P24672,&P18092 }};	/*Useragent - Nokia3500c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia3500c) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2476143 = {S87319, 116, { &P15364,&P16110,&P17334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/525.28 (KHTML, like Gecko) Version/3.2.2 Safari/525.28.1*/
const Device UA2483300 = {S87320, 148, { &P20172,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; Mobiistar Touch S05 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2485193 = {S87321, 134, { &P21933,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M729 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2487361 = {S87322, 62, { &P15364,&P5690,&P19981,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:19.0) Gecko/X Thunderbird/19.0*/
const Device UA2556176 = {S87323, 131, { &P21365,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; NEXO 3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2863287 = {S87324, 162, { &P20134,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2579900 = {S87325, 163, { &P12132,&P17485,&P24675,&P18092 }};	/*Useragent - SonyEricssonJ10i2/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonj10i2) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2656760 = {S87326, 161, { &P12132,&P17485,&P24675,&P18092 }};	/*Useragent - SonyEricssonJ10i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; sonyericssonj10i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2579983 = {S87327, 139, { &P21496,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; FLY F410 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2581357 = {S87328, 177, { &P15201,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Sensation Z710e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2764424 = {S87329, 174, { &P15201,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Sensation 4G Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2606282 = {S87330, 131, { &P21202,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-P500 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2673965 = {S87331, 130, { &P19852,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7511 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2657718 = {S87332, 155, { &P21906,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MITO A220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659457 = {S87333, 94, { &P20069,&P22784,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx78) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2987620 = {S87334, 106, { &P20069,&P22784,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx78) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile TRUSTED/1.0*/
const Device UA2706258 = {S87335, 170, { &P21033,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; U1 Build/HCL ME Tablet U1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444*/
const Device UA2671091 = {S87336, 133, { &P21048,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; W732 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2672175 = {S87337, 137, { &P21384,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F240L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2674739 = {S87338, 138, { &P12681,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus S 4G Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675526 = {S87339, 153, { &P20534,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAP4040_DUO Build/PrestigioPAP4040DUO) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2849858 = {S87340, 57, { &P15364,&P17188,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; WinNT4.0; Trident/3.0)*/
const Device UA2701015 = {S87341, 70, { &P15767,&P17090,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPad2,1; iPhone OS 4.3.5; en_US; en) AppleWebKit/420+*/
const Device UA2703574 = {S87342, 93, { &P12126,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA6730c) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2767102 = {S87343, 95, { &P12126,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA6730c-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2705201 = {S87344, 137, { &P20740,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; AVA757 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2707928 = {S87345, 121, { &P21853,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Y3 Build/HCL) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2793622 = {S87346, 129, { &P21853,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Connect-3G Build/HCL) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708560 = {S87347, 137, { &P21373,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F240K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734252 = {S87348, 168, { &P18635,&P18598,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (Nexus 4, 42, 1.9.1)*/
const Device UA2737761 = {S87349, 151, { &P20265,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A52 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2738003 = {S87350, 151, { &P20120,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A45 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2745651 = {S87351, 182, { &P15837,&P6909,&P24671,&P18092 }};	/*Useragent - NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax2-05) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2752463 = {S87352, 159, { &P18303,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT27i Build/6.1.1.B.1.54) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752546 = {S87353, 150, { &P21052,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A9 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2767366 = {S87354, 143, { &P23577,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PHICOMM i370 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2769651 = {S87355, 135, { &P23546,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AD683G Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769650 = {S87356, 144, { &P15809,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Rhyme S510b Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2781937 = {S87357, 72, { &P12280,&P19826,&P24531,&P18092 }};	/*Useragent - Instagram 3.5.1 (iPhone4,1; iPhone OS 6_1_3; en_US; en) AppleWebKit/420+*/
const Device UA2784792 = {S87358, 159, { &P20289,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799508 = {S87359, 147, { &P12972,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T989 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2800236 = {S87360, 130, { &P18437,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N8000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2937377 = {S87361, 130, { &P18437,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-N8000 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2849022 = {S87362, 142, { &P23303,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EX'NOTE ??? Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2849446 = {S87363, 140, { &P23303,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; EX'NOTE ? Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865352 = {S87364, 139, { &P24136,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; MALATA_I50 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2868989 = {S87365, 157, { &P18493,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; LG-MS840 Build/GRK39F) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2869018 = {S87366, 127, { &P20127,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT270 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2935115 = {S87367, 139, { &P21376,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; LG-F120L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2938430 = {S87368, 128, { &P23956,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SM-T310 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2939379 = {S87369, 138, { &P18214,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2939875 = {S87370, 137, { &P19779,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-in; A1+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2953761 = {S87371, 166, { &P20146,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9082 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2998733 = {S87372, 152, { &P17968,&P18598,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; X Build/HMX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA3059371 = {S87373, 159, { &P16319,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; U8860 Build/HuaweiU8860) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3015351 = {S87374, 93, { &P14782,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE61i) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3077785 = {S87375, 92, { &P14782,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE61) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3022963 = {S87376, 144, { &P21333,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; INO_ONE Build/INO_ONEINO_ONE) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3016294 = {S87377, 159, { &P14993,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT15i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3059199 = {S87378, 160, { &P22209,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Titanium S5 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3019931 = {S87379, 150, { &P24158,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; sr-rs; Telenor Smart Pro 2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA3021693 = {S87380, 181, { &P23122,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; Karbonn_A6 Build/MocorDroid4.0.3) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3027008 = {S87381, 136, { &P22660,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P778 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3027979 = {S87382, 103, { &P15798,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC Explorer A310e) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3063989 = {S87383, 103, { &P15798,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC Explorer A310e) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3064717 = {S87384, 159, { &P15397,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; ST18i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3058971 = {S87385, 138, { &P17595,&P21721,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.22 (KHTML, like Gecko) Maxthon/4.0.6.3000 Chrome/25.0.1364.99 Safari/537.22*/
const Device UA3093607 = {S87386, 88, { &P19860,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A9+) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3125931 = {S87387, 95, { &P19860,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; CELKON A9+) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3060253 = {S87388, 107, { &P18510,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_918N) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3061259 = {S87389, 103, { &P15798,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Explorer_A310e) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3094092 = {S87390, 140, { &P22021,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KHPHN05ANDA Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093528 = {S87391, 137, { &P18021,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A770K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093681 = {S87392, 136, { &P23550,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE T81 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094729 = {S87393, 133, { &P18721,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A90S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080940 = {S87394, 152, { &P23151,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAP4044DUO Build/PrestigioPAP4044DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3095063 = {S87395, 144, { &P20253,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; i-mobile IQ 5.3 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080299 = {S87396, 92, { &P13554,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN79) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3094708 = {S87397, 133, { &P15184,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P920 Build/ICS) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3096890 = {S87398, 154, { &P20705,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; HTC One Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3110374 = {S87399, 166, { &P14225,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3129306 = {S87400, 144, { &P23756,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SM-G730A Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA18066 = {S87401, 70, { &P11806,&P5704,&P4836,&P18092 }};	/*Useragent - MOT-E770v/85.97.C3P MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA67379 = {S87402, 69, { &P11806,&P5704,&P4836,&P18092 }};	/*Useragent - MOT-E770/85.97.F1P MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20197 = {S87403, 53, { &P17319,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-X640/1.0 UP.Browser/6.2.2.6 (GUI) MMP/1.0*/
const Device UA22617 = {S87404, 54, { &P17319,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-X640C/1.0 UP.Browser/6.2.2.6 (GUI) MMP/1.0*/
const Device UA20247 = {S87405, 116, { &P13791,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-ZV60-Vodafone/BUGE4 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA21638 = {S87406, 135, { &P18043,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; ISW11HT Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA22427 = {S87407, 61, { &P15465,&P17483,&P17602,&P18092 }};	/*Useragent - LG-CU500 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23385 = {S87408, 40, { &P12065,&P17483,&P17602,&P18092 }};	/*Useragent - LG-TE365 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA24571 = {S87409, 107, { &P13617,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; MDA Vario/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1)*/
const Device UA26647 = {S87410, 107, { &P13617,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; MDA Vario/1.2 Profile/MIDP-2.0 Configuration/CLDC-1.1)*/
const Device UA61659 = {S87411, 107, { &P13617,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; MDA Vario/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1)*/
const Device UA24691 = {S87412, 51, { &P15738,&P6712,&P3402,&P18092 }};	/*Useragent - Nokia-1006 UP.Browser/6.3.0.8.c.1.104 (GUI) MMP/2.0*/
const Device UA48792 = {S87413, 151, { &P17563,&P22857,&P4536,&P18092 }};	/*Useragent - NexianNX-G965/MTK Release/21.04.2011 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1  Opera/9.80 (MTK; Nucleus; U; en-US) Presto/2.4.18 Version/10.00*/
const Device UA49622 = {S87414, 150, { &P17563,&P22857,&P4536,&P18092 }};	/*Useragent - NexianNX-G965/MTK Release/21.04.2011 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1 Opera/9.80 (MTK; Nucleus; U; id-ID) Presto/2.4.18 Version/10.00*/
const Device UA49708 = {S87415, 150, { &P17563,&P22857,&P4536,&P18092 }};	/*Useragent - NexianNX-G965/MTK Release/21.04.2011 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1 Opera/9.80 (MTK; Nucleus; U; en-US) Presto/2.4.18 Version/10.00*/
const Device UA616314 = {S87416, 151, { &P17563,&P22857,&P4536,&P18092 }};	/*Useragent - NexianNX-G965/MTK Release/21.04.2011 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1  Opera/9.80 (MTK; Nucleus; U; id-ID) Presto/2.4.18 Version/10.00*/
const Device UA48943 = {S87417, 80, { &P15364,&P5208,&P4223,&P18092 }};	/*Useragent - Mozilla/5.0(compatible; Access NetFront/3.5; BREW 3.1.5; en,es)/240x320 PCD C721*/
const Device UA53780 = {S87418, 161, { &P14024,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; Orange Tactile internet 2 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA61754 = {S87419, 47, { &P14319,&P17484,&P5001,&P18092 }};	/*Useragent - SCH-U490/1.0 NetFront/3.0.22.2.16 (GUI) MMP/2.0*/
const Device UA64284 = {S87420, 15, { &P14587,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHC260/1.0*/
const Device UA81333 = {S87421, 78, { &P15364,&P17017,&P3402,&P18092 }};	/*Useragent - BenQ-CF61/1.00/WAP2.0/MIDP2.0/CLDC1.0 UP.Browser/6.3.0.4.c.1.102 (GUI) MMP/2.0*/
const Device UA89123 = {S87422, 78, { &P15364,&P17017,&P3402,&P18092 }};	/*Useragent - BenQ-EF51/1.00/WAP2.0/MIDP2.0/CLDC1.0 UP.Browser/6.3.0.4.c.1.102 (GUI) MMP/2.0*/
const Device UA1837097 = {S87423, 45, { &P15364,&P17017,&P3402,&P18092 }};	/*Useragent - BENQ UP.Browser/6.3.0.4.c.1.102 (GUI) MMP/2.0*/
const Device UA90276 = {S87424, 31, { &P17757,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N01C(c500;TB;W24H16)*/
const Device UA90524 = {S87425, 31, { &P17724,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F05C(c500;TB;W24H16)*/
const Device UA140095 = {S87426, 31, { &P17552,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P02B(c500;TB;W24H16)*/
const Device UA352794 = {S87427, 79, { &P15364,&P5690,&P24200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) QQBrowser/6.9.11079.201*/
const Device UA361769 = {S87428, 87, { &P15364,&P5690,&P24200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0) QQBrowser/6.9.11153.201*/
const Device UA364090 = {S87429, 122, { &P15364,&P5690,&P24200,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1 QQBrowser/6.9.11206.201*/
const Device UA570117 = {S87430, 95, { &P14763,&P5776,&P24662,&P18092 }};	/*Useragent - NokiaN72/5.0819.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UCWEB7.8.0.87/27/352*/
const Device UA724394 = {S87431, 71, { &P15364,&P18110,&P20459,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:9.0) Gecko/X Firefox/9.0 SeaMonkey/2.6*/
const Device UA924814 = {S87432, 91, { &P17595,&P17297,&P22316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:14.0) Gecko/X Firefox/14.0.1 SeaMonkey/2.11*/
const Device UA1870716 = {S87433, 81, { &P15364,&P7133,&P20901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; de-DE; rv:1.4) Gecko/X Netscape/7.1 (ax)*/
const Device UA1995474 = {S87434, 84, { &P20124,&P17979,&P18062,&P18092 }};	/*Useragent - ZTE U970_TD/1.0 Linux/2.6.39 Android/4.0 Release/2.21.2012 Browser/AppleWebKit534.30*/
const Device UA2000186 = {S87435, 77, { &P15364,&P22740,&P18628,&P18092 }};	/*Useragent - Mozilla/5.0 (Maemo; Linux armv7l; rv:14.0) Gecko/X Firefox/14.0.1 Fennec/14.0*/
const Device UA2028231 = {S87436, 151, { &P14784,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Hero Build/GRH78; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028813 = {S87437, 91, { &P24024,&P5629,&P15760,&P18092 }};	/*Useragent - CoolPad8180_CMCC_TD/1.0 Linux/2.6.35 Android/2.3 Release/1.20.2012 Browser/AppleWebkit533.1*/
const Device UA2029785 = {S87438, 154, { &P14186,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5830 Build/MIUI; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2362607 = {S87439, 135, { &P14186,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5830 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2419348 = {S87440, 130, { &P21275,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M1050S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2423098 = {S87441, 140, { &P12280,&P17146,&P20789,&P18092 }};	/*Useragent - MQQBrowser/38 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA2503371 = {S87442, 141, { &P12280,&P17146,&P20789,&P18092 }};	/*Useragent - MQQBrowser/38 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3*/
const Device UA2844049 = {S87443, 141, { &P12280,&P17146,&P20789,&P18092 }};	/*Useragent - MQQBrowser/381 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA2423109 = {S87444, 143, { &P20826,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS TOUCHTAB 9 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2425732 = {S87445, 159, { &P17595,&P18120,&P19965,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.3.1.31155 (Firestorm-Release; firestorm skin) Safari/533.3*/
const Device UA2449328 = {S87446, 130, { &P23877,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LT-TLA Build/LTLMR1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2455403 = {S87447, 135, { &P21163,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; ViewPad 10S Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2457933 = {S87448, 152, { &P18442,&P18111,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2778568 = {S87449, 152, { &P18442,&P18111,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2465975 = {S87450, 133, { &P20469,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N51 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2466136 = {S87451, 132, { &P22147,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; M-MP706i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2505074 = {S87452, 139, { &P20533,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; POLY PAD_8208HD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2506730 = {S87453, 169, { &P17625,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST25i Build/6.1.1.B.1.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2506739 = {S87454, 160, { &P18114,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Eee Pad Transformer Prime TF201 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2850998 = {S87455, 147, { &P18114,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Transformer Prime TF201 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2538810 = {S87456, 32, { &P15364,&P17017,&P21918,&P18092 }};	/*Useragent - contype chromeframe/25.0.1364.97*/
const Device UA2570106 = {S87457, 33, { &P15364,&P17017,&P21918,&P18092 }};	/*Useragent - contype chromeframe/25.0.1364.152*/
const Device UA2584291 = {S87458, 33, { &P15364,&P17017,&P21918,&P18092 }};	/*Useragent - contype chromeframe/25.0.1364.172*/
const Device UA2553941 = {S87459, 44, { &P15547,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SAMSUNG-GT-C6712/C6712DDKK3)*/
const Device UA2559101 = {S87460, 164, { &P15028,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P350 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3058143 = {S87461, 168, { &P15028,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-US; LG-P350 Build/FRG83) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2602396 = {S87462, 135, { &P12848,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MB526 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2598163 = {S87463, 147, { &P20894,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M3 Enjoy TV Box Build/V1.03.02MC03_X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2643616 = {S87464, 126, { &P20894,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M3 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2706156 = {S87465, 122, { &P15767,&P18432,&P21569,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Incredimail/1.0.5.4111*/
const Device UA2616197 = {S87466, 166, { &P15876,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P5100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2624754 = {S87467, 166, { &P15876,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P5100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2638688 = {S87468, 72, { &P15364,&P6704,&P20907,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win98; en-US; rv:0.9.4) Gecko/X Netscape6/6.2.1*/
const Device UA2662694 = {S87469, 143, { &P22758,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Celkon A 220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2671120 = {S87470, 139, { &P18226,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; U9500 Build/HuaweiU9500) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2672458 = {S87471, 144, { &P18512,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT881 Build/6.7.3-130_YUS-11) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2672609 = {S87472, 133, { &P15184,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P920 Build/ICS) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2672740 = {S87473, 43, { &P15364,&P18580,&P24428,&P18092 }};	/*Useragent - Dolphin/6.3 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2676655 = {S87474, 134, { &P23193,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCL21 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709598 = {S87475, 136, { &P18243,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-P880 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714392 = {S87476, 144, { &P19855,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT885 Build/9.8.3O-148_IRD-3) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2719212 = {S87477, 112, { &P22717,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia109/04.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2721019 = {S87478, 74, { &P15364,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE R270/P107D11/OPERA MINI 4.3/OBIGO Q03C/MIDP-2.0 CONFIGURATION/CLDC-1.1*/
const Device UA2758994 = {S87479, 83, { &P15364,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G X993/X993_V1_Z1_BR_D1EF104 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C*/
const Device UA3044504 = {S87480, 21, { &P15364,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G R255/Obigo Q03C*/
const Device UA2737479 = {S87481, 105, { &P15364,&P18144,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.63 Safari/537.31*/
const Device UA2738207 = {S87482, 156, { &P16362,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; SPH-D710 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2751873 = {S87483, 157, { &P23156,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; XOLO A700 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2869529 = {S87484, 161, { &P20837,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; IRIS_501 Build/LAVAIRIS501) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752882 = {S87485, 94, { &P14367,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6710s) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2799022 = {S87486, 156, { &P18679,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-B5330 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2780288 = {S87487, 156, { &P23435,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CloudPad 700w Build/FSV03) AppleWebKit/531.21.10(KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA2782930 = {S87488, 133, { &P18721,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A90S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2800433 = {S87489, 160, { &P13124,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; PantechP9070 Build/GRJ90) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2856029 = {S87490, 146, { &P20168,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; bq Maxwell Plus Build/2.0.0 X-11:45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2866097 = {S87491, 126, { &P15364,&P7133,&P23666,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth Pro/7.1.1.1580 Safari/532.4*/
const Device UA2866596 = {S87492, 97, { &P14839,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6720c-1b) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2936956 = {S87493, 94, { &P14839,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6720c) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2868033 = {S87494, 88, { &P23245,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A3+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2879085 = {S87495, 154, { &P23575,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ALCATEL ONE TOUCH 8000A Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879137 = {S87496, 131, { &P23641,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; TAB-7D3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2938534 = {S87497, 106, { &P15364,&P18144,&P21635,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.110 Safari/537.36*/
const Device UA2940515 = {S87498, 138, { &P19862,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.1; en-in; Micromax A110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2956719 = {S87499, 134, { &P21660,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Novo7 Venus Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2961251 = {S87500, 155, { &P18687,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; LG-P705 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2964893 = {S87501, 158, { &P19845,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE N9120 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3023536 = {S87502, 134, { &P20255,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; R8113 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022707 = {S87503, 135, { &P18600,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-03D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022589 = {S87504, 143, { &P21802,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GSmart Maya M1 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2994959 = {S87505, 137, { &P22999,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A870S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3003768 = {S87506, 137, { &P22999,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A870L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3050569 = {S87507, 137, { &P22999,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A870K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2996758 = {S87508, 152, { &P24538,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ALCATEL ONE TOUCH 8008X Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3060209 = {S87509, 150, { &P24538,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Orange Infinity 8008X Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2998201 = {S87510, 135, { &P18471,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT907 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3011976 = {S87511, 136, { &P18471,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT907 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3011282 = {S87512, 99, { &P14031,&P5477,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT-I5503) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3027383 = {S87513, 156, { &P16074,&P5768,&P24679,&P18092 }};	/*Useragent - Nokia3110c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3110c) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3049828 = {S87514, 136, { &P20146,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9082 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3080134 = {S87515, 137, { &P20013,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F180S Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3063290 = {S87516, 158, { &P18564,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One X+ Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3094266 = {S87517, 142, { &P21882,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Micromax P362 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093932 = {S87518, 134, { &P22640,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IQ446 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3079008 = {S87519, 97, { &P18652,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A87) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3079835 = {S87520, 97, { &P22864,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A26) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA4253 = {S87521, 71, { &P12097,&P5448,&P17173,&P18092 }};	/*Useragent - HTC_P4350 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA11586 = {S87522, 88, { &P13457,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonGeneric A1/R8BB Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA10138 = {S87523, 138, { &P17725,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; th-th; WellcoM-A800 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78680 = {S87524, 141, { &P17725,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; th-th; WellcoM-A800 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10237 = {S87525, 68, { &P13481,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-R6/1.0 ACS-NF/3.4 QTV2.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62529 = {S87526, 83, { &P14512,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonV630i/R1KG Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA13052 = {S87527, 132, { &P13755,&P17486,&P17470,&P18092 }};	/*Useragent - MOT-U9/R6632_G_81.17.04R Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 [es-co]*/
const Device UA13054 = {S87528, 124, { &P13755,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-U9/R6632_G_81.24.07R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA13056 = {S87529, 124, { &P13755,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-U9/R6632_G_81.27.0DR BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA43854 = {S87530, 124, { &P13755,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-U9/R6632_G_81.27.0CR BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49100 = {S87531, 126, { &P13755,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-U9/R6632_G_81.12.15R_B BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49101 = {S87532, 124, { &P13755,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-U9/R6632_G_81.11.48R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54807 = {S87533, 124, { &P13755,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-U9/R6632_G_81.27.05R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA56884 = {S87534, 124, { &P13755,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-U9/R6632_G_81.11.29R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62253 = {S87535, 124, { &P13755,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-U9/R6632_G_81.17.04R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63086 = {S87536, 124, { &P13755,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-U9/R6632_G_81.12.18R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63940 = {S87537, 124, { &P13755,&P17486,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-U9/R6632_G_81.27.0ER BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA18654 = {S87538, 5, { &P14161,&P18588,&P18589,&P18092 }};	/*Useragent - VM670*/
const Device UA19290 = {S87539, 91, { &P11956,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN75-3/3.0(20.1.172) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20855 = {S87540, 39, { &P13568,&P17486,&P3807,&P18092 }};	/*Useragent - Mot-WX295/1.0 ObigoInternetBrowser/Q03C*/
const Device UA32042 = {S87541, 109, { &P12454,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-I6230/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA56144 = {S87542, 31, { &P14649,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P03B(c500;TB;W24H16)*/
const Device UA62296 = {S87543, 66, { &P14396,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GB255/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA76417 = {S87544, 73, { &P14396,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GB255-parrot/V100 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA70915 = {S87545, 117, { &P14877,&P5096,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.2;Zh_cn;GT-I9000;480*800;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355*/
const Device UA2018957 = {S87546, 70, { &P14877,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2.1;Zh_cn;GT-I9000;480*800;)UCWEB7.8.0.95/139/355*/
const Device UA2020708 = {S87547, 68, { &P14877,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2;Zh_cn;GT-I9000;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA90736 = {S87548, 38, { &P16838,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxQ1+/Q03C MAUI-browser/MIDP-2.0*/
const Device UA193989 = {S87549, 52, { &P18088,&P17017,&P24662,&P18092 }};	/*Useragent - IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB7.8.0.87/41/997*/
const Device UA324217 = {S87550, 52, { &P18088,&P17017,&P24662,&P18092 }};	/*Useragent - IUC(U;iOS 4.2.1;Zh-cn;320*480;)/UCWEB7.8.0.87/41/997*/
const Device UA553840 = {S87551, 52, { &P18088,&P17017,&P24662,&P18092 }};	/*Useragent - IUC(U;iOS 3.1.3;Zh-cn;320*480;)/UCWEB7.8.0.87/39/800*/
const Device UA555109 = {S87552, 50, { &P18088,&P17017,&P24662,&P18092 }};	/*Useragent - IUC(U;iOS 4.1;Zh-cn;320*480;)/UCWEB7.8.0.87/41/800*/
const Device UA428934 = {S87553, 123, { &P15364,&P17285,&P22424,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11 Comodo_Dragon/17.5.2.0*/
const Device UA822981 = {S87554, 130, { &P15364,&P17285,&P22424,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11 Comodo_Dragon/17.5.2.0*/
const Device UA663155 = {S87555, 47, { &P12588,&P5055,&P17347,&P18092 }};	/*Useragent - nokiac6-01/UC Browser8.0.3.107/69/444  3gpp-gba*/
const Device UA679493 = {S87556, 85, { &P19901,&P17484,&P4841,&P18092 }};	/*Useragent - SAMSUNG-SCH-F689/1.0 POLARIS/5.30.WAP CTC/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA895882 = {S87557, 80, { &P13706,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-in; Nokia5235)/UC Browser8.4.0.159/50/352/UCWEB Mobile*/
const Device UA908522 = {S87558, 98, { &P15364,&P16110,&P22360,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:12.2.1) Gecko/X Firefox/12.2.1-x64 PaleMoon/12.2.1-x64*/
const Device UA1959874 = {S87559, 78, { &P15364,&P16110,&P22360,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; rv:12.2.1) Gecko/X Firefox/12.2.1 PaleMoon/12.2.1*/
const Device UA1937256 = {S87560, 141, { &P21658,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; th-th; i-mobile i695 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1968604 = {S87561, 79, { &P12029,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN82)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2020593 = {S87562, 152, { &P15389,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; u8800 Build/GRJ22; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2383301 = {S87563, 149, { &P15389,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; iw-il; U8800 Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025087 = {S87564, 130, { &P20591,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; K8GT_H Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029494 = {S87565, 130, { &P20591,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; K8GT_W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2146917 = {S87566, 78, { &P15364,&P22308,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; WinNT3.51; en-US; rv:1.8.1.7) Gecko/X Firefox/2.0.0.7*/
const Device UA2154483 = {S87567, 31, { &P20331,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX207/Q03C MAUI-browser/*/
const Device UA2164233 = {S87568, 77, { &P15518,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; gt-s5230) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2980178 = {S87569, 74, { &P15518,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; ru; gt-s5230) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2198290 = {S87570, 35, { &P18267,&P5043,&P21759,&P18092 }};	/*Useragent - iBrowser/Mini2.8 (NokiaX2-02/11.57)*/
const Device UA2242367 = {S87571, 87, { &P17595,&P17296,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.7373(Macintosh;Mac OS X (10.6.8);en;kml:2.2;client:Free;type:default)*/
const Device UA2445104 = {S87572, 87, { &P17595,&P17296,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Macintosh;Mac OS X (10.6.8);en;kml:2.2;client:Free;type:default)*/
const Device UA2495301 = {S87573, 87, { &P17595,&P17296,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.1.6014(Macintosh;Mac OS X (10.6.8);en;kml:2.2;client:Free;type:default)*/
const Device UA2336305 = {S87574, 137, { &P12879,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; GT-P7510 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2616605 = {S87575, 145, { &P12879,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; GT-P7510 Build/HTJ85B) AppleWebKit/534.30 (KHTML, like Gecko) Dolphin/INT-1.1.0 Mobile Safari/534.30*/
const Device UA2351434 = {S87576, 77, { &P14763,&P5776,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokian72) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2374780 = {S87577, 60, { &P11905,&P5890,&P21761,&P18092 }};	/*Useragent - iBrowser/2.5 (Nokia6120c(B20.03);SymbianOS/9.2 Series60/3.1)*/
const Device UA2543951 = {S87578, 60, { &P11905,&P5890,&P21761,&P18092 }};	/*Useragent - iBrowser/2.5 (Nokia6120c(B20.01);SymbianOS/9.2 Series60/3.1)*/
const Device UA3117394 = {S87579, 60, { &P11905,&P5890,&P21761,&P18092 }};	/*Useragent - iBrowser/2.5 (Nokia6120c(B21.03);SymbianOS/9.2 Series60/3.1)*/
const Device UA2381943 = {S87580, 146, { &P17968,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; M5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2423839 = {S87581, 139, { &P18386,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I757M Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2425236 = {S87582, 134, { &P20813,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GoTab GBT9 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2430404 = {S87583, 138, { &P12280,&P18117,&P18355,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.2m*/
const Device UA2457577 = {S87584, 135, { &P20098,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GN800 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2469929 = {S87585, 88, { &P17051,&P5567,&P17370,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/5.0/27.2260; U; en) Presto/2.8.119 320X240 Samsung SCH-U750*/
const Device UA2486901 = {S87586, 135, { &P23946,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; I7100 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2505909 = {S87587, 159, { &P12135,&P17483,&P24675,&P18092 }};	/*Useragent - LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-gd510) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2555970 = {S87588, 150, { &P20129,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2696398 = {S87589, 71, { &P20129,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Adr (Linux; U; 4.1.1; zh-cn; MI 2 Build/JRO03L;720*1280)*/
const Device UA2580116 = {S87590, 138, { &P23189,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Surfer 8.31 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2596212 = {S87591, 117, { &P18738,&P17979,&P18062,&P18092 }};	/*Useragent - OPPO_R817T/1.0 Linux/3.0.35.7 Android/4.0 Release/11.15.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2*/
const Device UA2598438 = {S87592, 168, { &P11747,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; HTC Desire Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2660089 = {S87593, 165, { &P11747,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2602076 = {S87594, 146, { &P20090,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848 Duo Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2603863 = {S87595, 147, { &P20090,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A848i Duo Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2624674 = {S87596, 189, { &P17337,&P6546,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.16/276,1.9.16.276,2012-12-21 16:37, +1300, nz*/
const Device UA2624936 = {S87597, 190, { &P17337,&P6546,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500D Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, -0800, ca*/
const Device UA2629543 = {S87598, 166, { &P15876,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P5100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2631103 = {S87599, 166, { &P15876,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P5100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2658800 = {S87600, 142, { &P23531,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; MITO_A78 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2671092 = {S87601, 140, { &P17905,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SO-03D Build/6.1.F.0.106) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2672023 = {S87602, 135, { &P22760,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ThL W8 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2695173 = {S87603, 32, { &P12108,&P5843,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia7230/10.81)*/
const Device UA2731789 = {S87604, 92, { &P18687,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; LG-P705) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2735336 = {S87605, 57, { &P15364,&P17188,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 4.0) Presto/2.12.388 Version/12.15*/
const Device UA2737781 = {S87606, 152, { &P19904,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; C5170 Build/IML77) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752285 = {S87607, 92, { &P14438,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE50) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2752341 = {S87608, 158, { &P14993,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT15i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2763987 = {S87609, 70, { &P18623,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 510;7.10;8858)*/
const Device UA2992740 = {S87610, 70, { &P18623,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 510;7.10;8862)*/
const Device UA2768172 = {S87611, 88, { &P19860,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A9+) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2892609 = {S87612, 156, { &P22875,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9220 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2769358 = {S87613, 140, { &P20540,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PAP4300 DUO Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2781976 = {S87614, 93, { &P21261,&P18598,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.2; en-US; GT-I9500) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2782576 = {S87615, 140, { &P22632,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Titanium S1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782867 = {S87616, 137, { &P18559,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LG730 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784529 = {S87617, 137, { &P20235,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cynus T1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798397 = {S87618, 134, { &P20478,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JY-G2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799575 = {S87619, 137, { &P22601,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300L Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2846985 = {S87620, 42, { &P13337,&P17485,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SonyEricssonJ108i/R7DA052)*/
const Device UA2855676 = {S87621, 143, { &P23628,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iBall Andi5L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2856901 = {S87622, 166, { &P20146,&P18111,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9082 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2863071 = {S87623, 189, { &P12605,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (DROID BIONIC, 42, 1.9.1)*/
const Device UA2869995 = {S87624, 135, { &P19985,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SH-02E Build/S4260) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2879401 = {S87625, 93, { &P21628,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8552) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2963513 = {S87626, 159, { &P20183,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI-ONE Plus Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2892373 = {S87627, 137, { &P20087,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; JK8000 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2944582 = {S87628, 141, { &P18372,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-04D Build/9.1.C.0.475) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2962037 = {S87629, 159, { &P23199,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Coolpad7295 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3015352 = {S87630, 161, { &P23199,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Coolpad 7295+ Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2998298 = {S87631, 112, { &P23705,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; C811 4G) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA3003416 = {S87632, 143, { &P12731,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3123407 = {S87633, 156, { &P12731,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-us; Motorola Electrify Build/4.5.1A_SUN_USC_19.0) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3059381 = {S87634, 159, { &P18496,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LVW Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3033549 = {S87635, 147, { &P23203,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; A1-810 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3018677 = {S87636, 144, { &P21463,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Superion TV 2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3059258 = {S87637, 157, { &P18301,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS840 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3021656 = {S87638, 176, { &P22597,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I545 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/jfltevzw*/
const Device UA3022385 = {S87639, 140, { &P24378,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Lenovo P780 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023142 = {S87640, 144, { &P24378,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Lenovo P780_ROW Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023837 = {S87641, 139, { &P24132,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; iris504Q Build/iris504Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3027674 = {S87642, 147, { &P18465,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Amazon Kindle Fire Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3029324 = {S87643, 156, { &P18679,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-B5330 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3059309 = {S87644, 162, { &P21020,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Micromax A116 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3083365 = {S87645, 137, { &P21949,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F100L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109368 = {S87646, 142, { &P21623,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 6 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094850 = {S87647, 142, { &P23161,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Xtreamer Joyz Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065264 = {S87648, 134, { &P20255,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; R8113 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3094368 = {S87649, 136, { &P20598,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Miracle Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3079313 = {S87650, 156, { &P20146,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9082 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3079407 = {S87651, 159, { &P20183,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI-ONE Plus Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109508 = {S87652, 157, { &P16362,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3109659 = {S87653, 161, { &P16362,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D710VMUB Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3109547 = {S87654, 159, { &P18223,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT22i Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3112880 = {S87655, 167, { &P21351,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI C8813Q Build/HuaweiC8813Q) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3113321 = {S87656, 165, { &P21351,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HUAWEI C8813 Build/HuaweiC8813) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA61931 = {S87657, 63, { &P11889,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia8800SI/2.0 (04.20) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA10143 = {S87658, 148, { &P17735,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; cherry mobile-SUPERION Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24529 = {S87659, 98, { &P15704,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-J400/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA27655 = {S87660, 98, { &P15704,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-J408/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA32191 = {S87661, 175, { &P14660,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia6122c_CMCC/5.21 (SymbianOS/9.2; U; Series60/3.1 Nokia6122c_CMCC/5.21; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) Mozilla/5.0 AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA56172 = {S87662, 44, { &P14660,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia6122cCN01.02/SymbianOS/9.1 Series60/3.0*/
const Device UA62375 = {S87663, 108, { &P14142,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E1390/E1390DDIG3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA62184 = {S87664, 98, { &P14380,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-P910/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA67946 = {S87665, 34, { &P14358,&P5890,&P4759,&P18092 }};	/*Useragent - NokiaE63/UC Browser7.6.1.82/28/351*/
const Device UA88272 = {S87666, 47, { &P17575,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SN3G UP.Browser/6.2.0.14.1.2 (GUI) MMP/2.0*/
const Device UA88425 = {S87667, 86, { &P17012,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) VZW:SCH-i770 PPC 320x320*/
const Device UA656305 = {S87668, 60, { &P18088,&P17017,&P22573,&P18092 }};	/*Useragent - Opera/8.01 (J2ME/MIDP; Opera Mini/1.1.5005/1662; en; U; ssr)*/
const Device UA331789 = {S87669, 100, { &P15364,&P18346,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1*/
const Device UA329709 = {S87670, 124, { &P15364,&P17285,&P22406,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.2.0.0 Chrome/12.0.742.112 Safari/534.30*/
const Device UA451753 = {S87671, 131, { &P15364,&P17285,&P22406,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.2.0.0 Chrome/12.0.742.112 Safari/534.30*/
const Device UA413793 = {S87672, 43, { &P17595,&P18120,&P22484,&P18092 }};	/*Useragent - Opera/9.10 (Macintosh; PPC Mac OS X; U; en)*/
const Device UA435478 = {S87673, 34, { &P15364,&P17188,&P18405,&P18092 }};	/*Useragent - Opera/9.25 (Windows NT 4.0; U; en)*/
const Device UA567104 = {S87674, 87, { &P17976,&P6437,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.2; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.8.149 Version/11.10*/
const Device UA2021014 = {S87675, 89, { &P17976,&P6437,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.2.1; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.8.149 Version/11.10*/
const Device UA643046 = {S87676, 59, { &P15364,&P6729,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Windows 95; U; en) Presto/2.9.168 Version/11.52*/
const Device UA713949 = {S87677, 51, { &P14188,&P5890,&P21746,&P18092 }};	/*Useragent - iBrowser/2.6 (Nokia5233;SymbianOS/9.1 Series60/3.0)*/
const Device UA773902 = {S87678, 62, { &P15364,&P5690,&P17207,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:5.0) Gecko/X Firefox/5.0 slurp*/
const Device UA1842802 = {S87679, 74, { &P15364,&P18144,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; OpenBSD amd64; en-US; rv:1.8.0.9) Gecko/X Firefox/3.5*/
const Device UA910464 = {S87680, 86, { &P15364,&P7133,&P18528,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.5.0)*/
const Device UA912731 = {S87681, 121, { &P15364,&P7133,&P18528,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 Sleipnir/3.5.1*/
const Device UA1995587 = {S87682, 86, { &P15364,&P7133,&P18528,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; Sleipnir/3.5.2)*/
const Device UA2162745 = {S87683, 114, { &P15364,&P7133,&P18528,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 Sleipnir/3.5.1*/
const Device UA911764 = {S87684, 131, { &P15364,&P17285,&P22432,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11 Comodo_Dragon/20.0.1.0*/
const Device UA912026 = {S87685, 124, { &P15364,&P17285,&P22432,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11 Comodo_Dragon/20.0.1.0*/
const Device UA925811 = {S87686, 102, { &P21640,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; Monument)*/
const Device UA925877 = {S87687, 66, { &P18309,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;PI86100;7.10;8112)*/
const Device UA1961226 = {S87688, 154, { &P19854,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-R760 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1961334 = {S87689, 167, { &P22062,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Onda Vi40 Elite - Top Notch Tablets Edition Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1981587 = {S87690, 134, { &P21078,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; Dslide 700 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2024663 = {S87691, 54, { &P18277,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; N-04D Build/AX)*/
const Device UA2028879 = {S87692, 130, { &P18277,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; N-04D Build/AX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025644 = {S87693, 130, { &P22017,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TAB467 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2475527 = {S87694, 130, { &P22017,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB467 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2214858 = {S87695, 46, { &P15767,&P17145,&P17470,&P18092 }};	/*Useragent - Facebook 4020.0 (iPad; iPhone OS 4.2.1; en_US)*/
const Device UA2223256 = {S87696, 131, { &P22099,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PLAYTAB Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2353298 = {S87697, 137, { &P24028,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; ZTE N789 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2355116 = {S87698, 135, { &P22762,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; L-01D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2399949 = {S87699, 137, { &P23158,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IMO S88 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2426235 = {S87700, 129, { &P20940,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T-10B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2457598 = {S87701, 93, { &P16591,&P22784,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxq50) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2458155 = {S87702, 142, { &P21084,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NW-Z1000Series Build/E200) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2462004 = {S87703, 171, { &P12056,&P5204,&P24675,&P18092 }};	/*Useragent - Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia7210supernova) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2467578 = {S87704, 71, { &P18295,&P18111,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.1.1; zh-cn; HTC_One_X; 720*1184) UCWEB7.9.0.94/139/444*/
const Device UA2484120 = {S87705, 71, { &P18295,&P18111,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.1.1; zh-cn; HTC_One_X; 720*1184) UCWEB7.9.0.94/139/352*/
const Device UA2507233 = {S87706, 142, { &P20742,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TOUCH87 Build/ICS.g08refem618.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2515242 = {S87707, 134, { &P18464,&P18110,&P19838,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.7 Safari/535.19 Silk-Accelerated=false*/
const Device UA2555290 = {S87708, 155, { &P23322,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TM-7043XD Build/V1.1.JELLYBEAN.rus.teXet.X.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2565996 = {S87709, 197, { &P15794,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2584851 = {S87710, 139, { &P18626,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I547C Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2580506 = {S87711, 197, { &P13157,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2603266 = {S87712, 141, { &P19764,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6502 Build/10.1.A.1.434) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2618094 = {S87713, 55, { &P15876,&P17979,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.5610 Android/4.0.3 Device/samsung-GT-P5100*/
const Device UA2644693 = {S87714, 96, { &P18219,&P5843,&P17347,&P18092 }};	/*Useragent - Nokia202/2.0 (20.36) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia202/UC Browser8.0.3.107/69/444*/
const Device UA2664531 = {S87715, 146, { &P21004,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VERTU Ti Build/4.0.4.A.0.1.72) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2667378 = {S87716, 163, { &P13042,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HTC Incredible S Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2669050 = {S87717, 163, { &P13042,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; HTC Incredible S Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2674960 = {S87718, 140, { &P21094,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A789 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675853 = {S87719, 137, { &P12841,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Vivid 4G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676643 = {S87720, 137, { &P18216,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Sony Tablet P Build/TISU0144) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2682089 = {S87721, 122, { &P15364,&P16110,&P22450,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Iron/12.0.750.0 Chrome/12.0.750.0 Safari/534.30*/
const Device UA2981106 = {S87722, 115, { &P15364,&P16110,&P22450,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/534.30 (KHTML, like Gecko) Iron/12.0.750.0 Chrome/12.0.750.0 Safari/534.30*/
const Device UA2696614 = {S87723, 135, { &P12110,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; novo7Legend Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3091180 = {S87724, 142, { &P12110,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; NGM_Legend2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2699270 = {S87725, 138, { &P24584,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; Q-Smart S2 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2700047 = {S87726, 71, { &P22619,&P22784,&P4780,&P18092 }};	/*Useragent - Micromax X340/Q03C MAUI-Browser Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2708693 = {S87727, 136, { &P22497,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; w30hd(QuadCore) Build/W30hd) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2749352 = {S87728, 146, { &P23188,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; OlivePad V-T210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752290 = {S87729, 162, { &P21269,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Intex Aqua 3.2 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2770020 = {S87730, 138, { &P22590,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717M Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2779560 = {S87731, 93, { &P18304,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9070) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2781755 = {S87732, 105, { &P12871,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC Wildfire S A510e) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2879507 = {S87733, 105, { &P12871,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC Wildfire S A510e) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2783923 = {S87734, 140, { &P20183,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MI-ONE Plus Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784894 = {S87735, 159, { &P18451,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26ii Build/6.1.A.2.55) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799516 = {S87736, 163, { &P18009,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTCEVODesign4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2893367 = {S87737, 156, { &P18009,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PH44100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2867027 = {S87738, 99, { &P22158,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; INTEX_CLOUD_X1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2870894 = {S87739, 145, { &P15805,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Raider X710e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892911 = {S87740, 140, { &P15805,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Holiday Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2872172 = {S87741, 134, { &P20944,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T-Hub2 Build/TVA301TELBG3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2878488 = {S87742, 139, { &P23553,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZTE V985 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892956 = {S87743, 156, { &P18713,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P760 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2894333 = {S87744, 156, { &P18713,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P768 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2999060 = {S87745, 156, { &P18713,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P769 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2893047 = {S87746, 152, { &P19771,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2901201 = {S87747, 152, { &P19771,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937819 = {S87748, 152, { &P19771,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5035E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893053 = {S87749, 141, { &P23254,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Celkon A119Q Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2945143 = {S87750, 156, { &P21356,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; TECNO N3 Build/master) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2963972 = {S87751, 152, { &P21472,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; A111 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2965084 = {S87752, 97, { &P22622,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Micromax A72) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2965333 = {S87753, 154, { &P17160,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Slider SL101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3022319 = {S87754, 137, { &P21949,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F100L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2977081 = {S87755, 143, { &P24048,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly s Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2977083 = {S87756, 136, { &P24048,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly s Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3023240 = {S87757, 129, { &P21990,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; chagall Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022149 = {S87758, 139, { &P18302,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT25i Build/4.1.B.0.631) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020086 = {S87759, 162, { &P15364,&P18598,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; V4 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33931*/
const Device UA3107636 = {S87760, 175, { &P15364,&P18598,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; Lenovo S820_ROW Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33482*/
const Device UA3023200 = {S87761, 138, { &P23687,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; LIFE PLAY Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023957 = {S87762, 88, { &P20515,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; A54) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3024188 = {S87763, 159, { &P21141,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; Lenovo P770 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3064147 = {S87764, 152, { &P12765,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3057852 = {S87765, 155, { &P20729,&P18111,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; CROSS A27 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079778 = {S87766, 155, { &P24365,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SM-T211 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3058595 = {S87767, 176, { &P12139,&P5204,&P24679,&P18092 }};	/*Useragent - Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5310xpressmusic) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3079519 = {S87768, 155, { &P18283,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9300 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3059329 = {S87769, 91, { &P14545,&P17483,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-x330t) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3061012 = {S87770, 90, { &P14545,&P17483,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-x330) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3062126 = {S87771, 156, { &P14482,&P5843,&P24679,&P18092 }};	/*Useragent - NokiaC3-01/5.0 (07.51) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac3-01) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3093986 = {S87772, 134, { &P21011,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; L-01E Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094845 = {S87773, 138, { &P23680,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AURUS III Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3081033 = {S87774, 127, { &P23313,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; M8 pro Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094149 = {S87775, 137, { &P19910,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E617G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094094 = {S87776, 136, { &P24165,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; 3G7334i Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3082948 = {S87777, 137, { &P21949,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F100L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094530 = {S87778, 136, { &P24611,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E989 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094503 = {S87779, 142, { &P12567,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Panasonic P51 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3095050 = {S87780, 158, { &P18223,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT22i Build/6.2.A.1.100) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3113319 = {S87781, 137, { &P18593,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S6010 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126220 = {S87782, 138, { &P18593,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S6010L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3131733 = {S87783, 137, { &P12972,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA16019 = {S87784, 100, { &P11983,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-L708/ZTIB1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA20032 = {S87785, 40, { &P15024,&P17486,&P17602,&P18092 }};	/*Useragent - MOT-EX126 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA20034 = {S87786, 112, { &P15025,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHE215L/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA20696 = {S87787, 106, { &P15136,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S6700T/1.0 SHP/VPP/R5 NetFront/3.5 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA2302999 = {S87788, 105, { &P15136,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S6700/1.0 SHP/VPP/R5 NetFront/3.5 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA20838 = {S87789, 98, { &P15137,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SWD-V100/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA67225 = {S87790, 78, { &P14824,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-S5200/S5200DDIK3 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA67615 = {S87791, 124, { &P13686,&P17603,&P4732,&P18092 }};	/*Useragent - PantechP9050/JJUS09132010; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.21329; U; es) Opera 9.50*/
const Device UA445639 = {S87792, 117, { &P13686,&P17603,&P4732,&P18092 }};	/*Useragent - PantechP9050/JJUSX; Mozilla/5.0 (Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mini/att/4.2.21329; U; es) Opera 9.50*/
const Device UA69117 = {S87793, 36, { &P12193,&P5129,&P4779,&P18092 }};	/*Useragent - NokiaC5-00/UC Browser7.8.0.95/28/352*/
const Device UA701817 = {S87794, 38, { &P12193,&P5129,&P4779,&P18092 }};	/*Useragent - NokiaC5-00.2/UC Browser7.8.0.95/28/355*/
const Device UA195248 = {S87795, 145, { &P16156,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; phiremod for Nook Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA218955 = {S87796, 133, { &P13286,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA684181 = {S87797, 133, { &P13286,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA729690 = {S87798, 133, { &P13286,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID2 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA251362 = {S87799, 122, { &P15364,&P17285,&P22412,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.2.2.0 Chrome/14.0.835.202 Safari/535.1*/
const Device UA421726 = {S87800, 129, { &P15364,&P17285,&P22412,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.2.2.0 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2394815 = {S87801, 137, { &P12657,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC EVO 3D Build/MIUI) AppleWebKit/533.1 (KHTML	 like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA361836 = {S87802, 192, { &P15364,&P7133,&P24200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.9.11153.201*/
const Device UA363581 = {S87803, 122, { &P15364,&P7133,&P24200,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1 QQBrowser/6.9.11206.201*/
const Device UA2309181 = {S87804, 185, { &P15364,&P7133,&P24200,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.9.11206.201*/
const Device UA520860 = {S87805, 37, { &P17595,&P18120,&P22889,&P18092 }};	/*Useragent - iCab/4.8 (Macintosh; U; PPC Mac OS X)*/
const Device UA756450 = {S87806, 39, { &P17595,&P18120,&P22889,&P18092 }};	/*Useragent - iCab/4.8 (Macintosh; U; Intel Mac OS X)*/
const Device UA842519 = {S87807, 60, { &P15364,&P6729,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (Windows 95; U; ru) Presto/2.10.229 Version/11.61*/
const Device UA599485 = {S87808, 114, { &P15364,&P18118,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Ubuntu; en-us) AppleWebKit/525.13 (KHTML, like Gecko) Version/2.2 Firefox/525.13	Safari 2.2*/
const Device UA3069491 = {S87809, 103, { &P15364,&P18118,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Ubuntu; en-us) AppleWebKit/525.13 (KHTML, like Gecko) Version/2.2 Firefox/525.13*/
const Device UA727131 = {S87810, 53, { &P14188,&P5890,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 (Nokia5233;SymbianOS/9.1 Series60/3.0)*/
const Device UA771491 = {S87811, 155, { &P12605,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT875 Build/IML74K; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA924239 = {S87812, 151, { &P12605,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT875 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA777357 = {S87813, 115, { &P18267,&P5043,&P17438,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-02/10.90; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15*/
const Device UA906575 = {S87814, 166, { &P18283,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA925765 = {S87815, 96, { &P18182,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; 0000)*/
const Device UA925903 = {S87816, 121, { &P18182,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; <manufacturer>; <model> [;<operator])*/
const Device UA2499837 = {S87817, 97, { &P18182,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Nokia)*/
const Device UA1897974 = {S87818, 142, { &P15017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-pt; Sapo a5 Build/tejosunhsine) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031520 = {S87819, 137, { &P15017,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; ZTE-LIBRA Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2000874 = {S87820, 168, { &P15575,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9100 Build/JRO03H; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2007208 = {S87821, 168, { &P15575,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2745168 = {S87822, 169, { &P15575,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9100G Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257751 = {S87823, 168, { &P12798,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT18i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2275164 = {S87824, 12, { &P15364,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG_KIES*/
const Device UA3109176 = {S87825, 180, { &P15364,&P17484,&P17606,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.6.1;FBBV/X;FBDM/{density=3.0,width=1080,height=1920};FBLC/en_US;FBCR/AT&amp-T;FBPN/com.facebook.katana;FBDV/SAMSUNG-SGH-I337;FBSV/4.2.2;FBCA/armeabi-v7a:armeabi;]*/
const Device UA2337198 = {S87826, 137, { &P17963,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Next7P Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2376054 = {S87827, 140, { &P15297,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT15a Build/4.1.B.0.431) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3043654 = {S87828, 92, { &P12456,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; ar-sa; nokian73-1) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2396010 = {S87829, 135, { &P22016,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; X710d Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2400208 = {S87830, 126, { &P17595,&P17296,&P24727,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) FluidApp Version/1.6.1 Safari/534.57.2*/
const Device UA2794367 = {S87831, 126, { &P17595,&P17296,&P24727,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.59.8 (KHTML, like Gecko) FluidApp Version/1.6.1 Safari/534.59.8*/
const Device UA2403826 = {S87832, 136, { &P18278,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; F-05D Build/V08R31C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2442541 = {S87833, 140, { &P17595,&P18207,&P22511,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en-US; rv:1.9.1.6) Gecko/X FireDownload/1.2pre Firefox/3.5.6 Wyzo/3.5.6.1 FireTorrent/1.1pre*/
const Device UA2486086 = {S87834, 155, { &P16074,&P5768,&P24675,&P18092 }};	/*Useragent - Nokia3110c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia3110c) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2762209 = {S87835, 187, { &P12132,&P17485,&P24672,&P18092 }};	/*Useragent - SonyEricssonJ10i2/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; fa; sonyericssonj10i2) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile*/
const Device UA2950678 = {S87836, 185, { &P12132,&P17485,&P24672,&P18092 }};	/*Useragent - SonyEricssonJ10i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; fa; sonyericssonj10i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile*/
const Device UA2494005 = {S87837, 129, { &P15364,&P16110,&P22413,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Comodo_Dragon/15.0.3.0 Chrome/15.0.874.102 Safari/535.2*/
const Device UA2639867 = {S87838, 122, { &P15364,&P16110,&P22413,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.2 (KHTML, like Gecko) Comodo_Dragon/15.0.3.0 Chrome/15.0.874.102 Safari/535.2*/
const Device UA2530276 = {S87839, 132, { &P12889,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P7300 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2631219 = {S87840, 131, { &P12889,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7300 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2833399 = {S87841, 119, { &P11689,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian70-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2553940 = {S87842, 133, { &P21514,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; N350 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2554656 = {S87843, 153, { &P13094,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Lenovo A60 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2790582 = {S87844, 162, { &P13094,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A60+ Build/LenovoLePhone) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2567830 = {S87845, 132, { &P17595,&P17297,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/6.0.2 Safari/7536.26.17 Sleipnir/4.1*/
const Device UA2669665 = {S87846, 134, { &P17595,&P17297,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/6.0.3 Safari/7536.28.10 Sleipnir/4.1.2*/
const Device UA2583271 = {S87847, 142, { &P21614,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Philips W536 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2594501 = {S87848, 131, { &P18061,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; rk30sdk Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2600940 = {S87849, 139, { &P23969,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190T Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2623317 = {S87850, 193, { &P15985,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6200L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0700, th*/
const Device UA2626484 = {S87851, 192, { &P15985,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0800, my*/
const Device UA2657138 = {S87852, 131, { &P23767,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-7240 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2676297 = {S87853, 139, { &P15767,&P17141,&P19810,&P18092 }};	/*Useragent - Mozilla/5.0 (iPaddy; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/534.52 (KHTML, like Gecko) NX/2.1.0.8.21 NintendoBrowser/1.0.0.7494.US*/
const Device UA2707808 = {S87854, 141, { &P20091,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5880D Build/PMP5880D_v1.0.0_X) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708382 = {S87855, 144, { &P18554,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708862 = {S87856, 138, { &P20737,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735096 = {S87857, 139, { &P22756,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C1604 Build/11.1.A.0.64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2728085 = {S87858, 133, { &P22152,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAC-10011 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2735610 = {S87859, 159, { &P18672,&P5629,&P22281,&P18092 }};	/*Useragent - OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2879228 = {S87860, 158, { &P22756,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; C1604 Build/11.1.A.0.68) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2768111 = {S87861, 94, { &P13474,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAX5-01) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2769652 = {S87862, 144, { &P20168,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Maxwell Plus Build/1.0.3 X-14:07) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2771840 = {S87863, 140, { &P19759,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo K860 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774111 = {S87864, 140, { &P18372,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SO-04D Build/7.0.D.1.137) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2781526 = {S87865, 159, { &P23199,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Coolpad7295 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2782797 = {S87866, 141, { &P20091,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5880D Build/PMP5880D_v1.0.0_X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2783700 = {S87867, 157, { &P18582,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT28h Build/6.1.E.3.7) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2783985 = {S87868, 195, { &P18635,&P18598,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Nexus 4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.2/552,2.0.2.552,2013-05-22 23:05, +0100, gb*/
const Device UA2799087 = {S87869, 136, { &P22497,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; w30hd(QuadCore) Build/W30hd) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2799207 = {S87870, 137, { &P19910,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E617G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3058122 = {S87871, 170, { &P12605,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT875 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/targa*/
const Device UA2870149 = {S87872, 137, { &P23286,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TECNO F7 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2900571 = {S87873, 160, { &P22394,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6603 Build/10.3.A.0.423) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2934097 = {S87874, 79, { &P21545,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; HUAWEI_Y210-2010; 320*480) UCWEB7.9.0.94/139/33572*/
const Device UA3086266 = {S87875, 75, { &P21545,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; HUAWEI_Y210S; 320*480) UCWEB7.9.0.94/139/33572*/
const Device UA2940287 = {S87876, 92, { &P15209,&P5768,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6233) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2944996 = {S87877, 97, { &P21475,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A35) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2945032 = {S87878, 97, { &P20729,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A27) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2958678 = {S87879, 146, { &P18283,&P18598,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3022227 = {S87880, 141, { &P13124,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PantechP9070 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2999587 = {S87881, 184, { &P24547,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; SPHS on Hsdroid Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33745*/
const Device UA3006957 = {S87882, 182, { &P24547,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us ; SPHS on Hsdroid Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33745*/
const Device UA3022495 = {S87883, 97, { &P22864,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A26) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3022506 = {S87884, 145, { &P23255,&P18598,&P19802,&P18092 }};	/*Useragent - HUAWEI_G520-T10_TD/1.0 Android/4.2.1 (Linux; U; Android 4.2.1; zh-cn) Release/04.01.2013 Browser/WAP2.0 (AppleWebKit/534.30) Mobile Safari/534.30*/
const Device UA3022906 = {S87885, 137, { &P20983,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9308 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023851 = {S87886, 140, { &P20543,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A660 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3025167 = {S87887, 127, { &P24419,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ME302C Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3025640 = {S87888, 156, { &P24149,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Aqua I-5 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3026944 = {S87889, 168, { &P18113,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3065060 = {S87890, 155, { &P22585,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SPH-L720 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3030755 = {S87891, 157, { &P23680,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; AURUS III Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3034290 = {S87892, 101, { &P12581,&P5055,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia500) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3058458 = {S87893, 148, { &P18385,&P21637,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA3093566 = {S87894, 152, { &P18385,&P21637,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA3080984 = {S87895, 136, { &P22809,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; P5_Quad Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3112266 = {S87896, 143, { &P21071,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Novo 10 Hero QuadCore Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3079275 = {S87897, 160, { &P12513,&P17485,&P24679,&P18092 }};	/*Useragent - SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw20) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3093323 = {S87898, 92, { &P14453,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE51) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3108430 = {S87899, 88, { &P19860,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A9+) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3109334 = {S87900, 95, { &P19860,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; CELKON_A9+) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3109147 = {S87901, 156, { &P14826,&P5843,&P24680,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; NokiaC1-01) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3129702 = {S87902, 156, { &P14635,&P5843,&P24680,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (08.71) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; NokiaC3-00) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA50462 = {S87903, 62, { &P11941,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW580i/R8BE Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA32765 = {S87904, 110, { &P12763,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-GT-C6620/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA757403 = {S87905, 99, { &P14635,&P5843,&P4779,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (07.20) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac3-00/UC Browser7.8.0.95/70/355*/
const Device UA54656 = {S87906, 140, { &P14252,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GSMART G1310 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78108 = {S87907, 140, { &P14252,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; GSmart G1310 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022551 = {S87908, 141, { &P14252,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ru-ru; GSmart G1310 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43549 = {S87909, 138, { &P13700,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) CellcomLBR/1.0/e811/CELLCOM_LBR_P180A2V1.0.1/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63105 = {S87910, 53, { &P14590,&P17483,&P17660,&P18092 }};	/*Useragent - LG-KU250/v1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64336 = {S87911, 53, { &P14590,&P17483,&P17660,&P18092 }};	/*Useragent - LG/KU250/v1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63767 = {S87912, 37, { &P14539,&P17483,&P3866,&P18092 }};	/*Useragent - LG-KE850 MIC/1.1.14 MIDP-2.0/CLDC-1.1*/
const Device UA90398 = {S87913, 54, { &P17638,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC42 UP.Browser/6.2_7.2.7.1.K.6.210 (GUI) MMP/2.0*/
const Device UA349011 = {S87914, 155, { &P12814,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-I777 Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA355744 = {S87915, 152, { &P13257,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Droid Build/MIUI-1.11.25.0-SHOLES-EN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA569082 = {S87916, 151, { &P13257,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Droid Build/MIUI-1.8.26.0-SHOLES-EN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA806527 = {S87917, 152, { &P13257,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Droid Build/MIUI-1.12.30.0-SHOLES-EN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA834596 = {S87918, 151, { &P13257,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Droid Build/MIUI-2.2.10.0-SHOLES-EN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019910 = {S87919, 151, { &P13257,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Droid Build/MIUI-1.9.30.0-SHOLES-EN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA447440 = {S87920, 34, { &P15364,&P17285,&P4811,&P18092 }};	/*Useragent - Opera/9.02 (Windows NT 6.0; U; en)*/
const Device UA1839776 = {S87921, 34, { &P15364,&P17285,&P4811,&P18092 }};	/*Useragent - Opera/9.00 (Windows NT 6.0; U; en)*/
const Device UA458843 = {S87922, 134, { &P21225,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; MID7010 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA524519 = {S87923, 118, { &P15364,&P7127,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/1.0.154.53 Safari/525.19*/
const Device UA572088 = {S87924, 37, { &P11905,&P5890,&P17338,&P18092 }};	/*Useragent - nokia6120c/UC Browser7.9.0.102/69/352*/
const Device UA649101 = {S87925, 37, { &P11689,&P5890,&P17347,&P18092 }};	/*Useragent - nokian70-1/UC Browser8.0.3.107/69/444*/
const Device UA655413 = {S87926, 139, { &P21198,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; Dell Streak 10 Pro Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA771026 = {S87927, 114, { &P18267,&P5043,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-02/10.90; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA1886672 = {S87928, 81, { &P15364,&P17285,&P22306,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:2.0.1) Gecko/X Firefox/4.0.1 SeaMonkey/2.1*/
const Device UA775874 = {S87929, 120, { &P13296,&P5448,&P4777,&P18092 }};	/*Useragent - SonyEricssonX1i/R3AA Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2682123 = {S87930, 75, { &P13296,&P5448,&P4777,&P18092 }};	/*Useragent - X1i Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA789626 = {S87931, 109, { &P22106,&P17979,&P3508,&P18092 }};	/*Useragent - HS-T92_TD/1.0 Android/2.2 Release/12.10.2011 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA872143 = {S87932, 73, { &P15364,&P7133,&P17203,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0.1) Gecko/X Firefox/4.0.1 slurp*/
const Device UA1871457 = {S87933, 69, { &P15364,&P7133,&P17203,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0) Gecko/X Firefox/4.0 slurp*/
const Device UA1914010 = {S87934, 87, { &P15364,&P17979,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Linux armv7l; rv:10.0a1) Gecko/X Firefox/10.0a1 MobiFilter/10.0a1*/
const Device UA2032093 = {S87935, 80, { &P15364,&P17979,&P18099,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Linux armv7l; rv:10.0) Gecko/X Firefox/10.0 Spearmint/10.0*/
const Device UA1958641 = {S87936, 133, { &P21841,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ASTROPLUS Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1994206 = {S87937, 150, { &P21114,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; De-Sensed HTC Vivid Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1997529 = {S87938, 73, { &P17595,&P18579,&P18430,&P18092 }};	/*Useragent - Safari/8536.25 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacBookAir4%2C2)*/
const Device UA2496813 = {S87939, 73, { &P17595,&P18579,&P18430,&P18092 }};	/*Useragent - Safari/8536.25 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacBookPro9%2C2)*/
const Device UA2546126 = {S87940, 68, { &P17595,&P18579,&P18430,&P18092 }};	/*Useragent - Safari/8536.25 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (iMac12%2C1)*/
const Device UA2034225 = {S87941, 132, { &P22260,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; th-th; 697i Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2156636 = {S87942, 82, { &P15791,&P5055,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; Nokia603)/UC Browser8.5.0.183/51/352/UCWEB Mobile*/
const Device UA2339502 = {S87943, 80, { &P14359,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5228)/UC Browser8.5.0.183/50/444/UCWEB Mobile*/
const Device UA2222518 = {S87944, 135, { &P17937,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WX435 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2257096 = {S87945, 168, { &P18303,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; MT27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2350925 = {S87946, 132, { &P20349,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Novo7Flame Build/X.qm) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2518434 = {S87947, 124, { &P12835,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-S3770K/S3770KDDKL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA2383454 = {S87948, 249, { &P18506,&P5043,&P18516,&P18092 }};	/*Useragent - Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.1.161/69/355*/
const Device UA2398008 = {S87949, 98, { &P15364,&P16110,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.2.6613(Windows;Microsoft Windows (6.2.9200.0);en;kml:2.2;client:Free;type:default)*/
const Device UA2400059 = {S87950, 135, { &P20976,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; W100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2401558 = {S87951, 126, { &P14851,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5753e) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2419098 = {S87952, 133, { &P22691,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M-MP1010i Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2456685 = {S87953, 81, { &P18323,&P17484,&P17374,&P18092 }};	/*Useragent - SAMSUNG-GT-S5222 Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2473292 = {S87954, 100, { &P15364,&P7127,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:7.0a1) Gecko/X Firefox/7.0a1 (.NET CLR 3.5.30729)*/
const Device UA2475646 = {S87955, 110, { &P22112,&P5629,&P15760,&P18092 }};	/*Useragent - HS-T860_TD/1.0 Android/2.3 Release/05.31.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2484466 = {S87956, 64, { &P15364,&P17285,&P18521,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:18.0) Gecko/X Firefox/18.0 slurp*/
const Device UA2484776 = {S87957, 135, { &P18253,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; XT894 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2854741 = {S87958, 134, { &P18253,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT894 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2485718 = {S87959, 99, { &P20126,&P17485,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonk800i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2596577 = {S87960, 99, { &P20126,&P17485,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; sonyericssonk800i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2495060 = {S87961, 73, { &P20382,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.7; zh-cn; ZTE-T_U960s; 480*800) UCWEB7.9.3.103/139/800*/
const Device UA2502839 = {S87962, 83, { &P19868,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; ALCATEL_ONE_TOUCH_985N; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2595332 = {S87963, 83, { &P19868,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; ALCATEL_ONE_TOUCH_985N; 320*480) UCWEB7.9.0.94/139/355*/
const Device UA2503830 = {S87964, 68, { &P15184,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; LG-P920; 480*800) UCWEB7.9.0.94/139/352*/
const Device UA2727035 = {S87965, 68, { &P15184,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.2; zh-cn; LG-P920; 480*800) UCWEB7.9.0.94/139/352*/
const Device UA2526849 = {S87966, 156, { &P17691,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I957 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2579788 = {S87967, 150, { &P20172,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Mobiistar Touch S05 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2579808 = {S87968, 117, { &P21903,&P17979,&P18062,&P18092 }};	/*Useragent - OPPO_U705T/1.0 Linux/3.0.35.7 Android/4.0 Release/11.15.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2*/
const Device UA2580230 = {S87969, 93, { &P16808,&P22784,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq7) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2712283 = {S87970, 157, { &P18661,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N860 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2597326 = {S87971, 151, { &P18673,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2850614 = {S87972, 152, { &P18673,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562i Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2600344 = {S87973, 126, { &P23868,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ST8 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2604175 = {S87974, 136, { &P23130,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; H2000+ Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2644540 = {S87975, 133, { &P22351,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-ae; I200 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2657342 = {S87976, 73, { &P22981,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-E2262/1.0 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2667768 = {S87977, 157, { &P19859,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328w Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2668105 = {S87978, 141, { &P19999,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A30 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668864 = {S87979, 156, { &P18673,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-S7562 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2794663 = {S87980, 157, { &P18673,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-S7562i Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2672052 = {S87981, 32, { &P15364,&P17017,&P21919,&P18092 }};	/*Useragent - contype chromeframe/26.0.1410.43*/
const Device UA2674906 = {S87982, 32, { &P15364,&P17017,&P21919,&P18092 }};	/*Useragent - contype chromeframe/26.0.1410.64*/
const Device UA2672293 = {S87983, 140, { &P19759,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo K860 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673181 = {S87984, 154, { &P20541,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAP4500TDUO Build/PrestigioPAP4500TDUO) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676744 = {S87985, 140, { &P20089,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SO-01E Build/9.0.G.1.108) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2695808 = {S87986, 103, { &P11728,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.105; en-us; 8820/2.7.0.105) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2777598 = {S87987, 103, { &P11728,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.109; en-us; 8820/2.7.0.109) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2982677 = {S87988, 103, { &P11728,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 2.7.0.106; en-us; 8820/2.7.0.106) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2696051 = {S87989, 64, { &P20529,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; A89; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2730085 = {S87990, 64, { &P20529,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; A89; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2707540 = {S87991, 191, { &P22114,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; jblend) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile/HS-T39_TD/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0 Config/CLDC-1.1*/
const Device UA2851080 = {S87992, 294, { &P22114,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; jblend) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile/HS-T39_TD/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0 Config/CLDC-1.1,/HS-T39_TD/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0 Config/CLDC-1.1*/
const Device UA2707724 = {S87993, 133, { &P20923,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Mitab FLY Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2732578 = {S87994, 88, { &P22638,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; W20) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2736927 = {S87995, 157, { &P20134,&P5629,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2739202 = {S87996, 93, { &P21356,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; TECNO N3) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2767400 = {S87997, 141, { &P22676,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Celkon A119 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2768500 = {S87998, 126, { &P21383,&P5629,&P21729,&P18092 }};	/*Useragent - OneBrowser/3.2/Lenovo-A288t_TD/S100 Linux/2.6.35 Android/2.3.5 Release/02.29.2012 Browser/AppleWebkit533.1 Mobile Safari/533.1*/
const Device UA2769037 = {S87999, 134, { &P21823,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTX21 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2795400 = {S88000, 146, { &P20801,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Optimus_San_Remo Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2869919 = {S88001, 155, { &P20705,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; HTC One Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2848955 = {S88002, 158, { &P15989,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Tipo Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2868130 = {S88003, 157, { &P18280,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC One S Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2893073 = {S88004, 141, { &P20131,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Thrill 430x Build/FSV14) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2939347 = {S88005, 148, { &P24158,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Telenor Smart Pro 2 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2944275 = {S88006, 158, { &P22641,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; C1504 Build/11.3.A.0.47) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2945206 = {S88007, 152, { &P16598,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A500 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3115755 = {S88008, 153, { &P15364,&P17285,&P23486,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.6.6.X (Second Life Release; default skin) Safari/533.3*/
const Device UA2963735 = {S88009, 142, { &P15798,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-in; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3022558 = {S88010, 137, { &P21481,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A860S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2965432 = {S88011, 135, { &P17625,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.7; en-in; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3051490 = {S88012, 135, { &P17625,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.7; en-in; ST25i Build/6.0.B.3.188) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3024018 = {S88013, 128, { &P20014,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-01E Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021324 = {S88014, 135, { &P21446,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SH837W Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2987154 = {S88015, 149, { &P18437,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N8000 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3056710 = {S88016, 155, { &P18437,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N8000 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1/n8000*/
const Device UA2998660 = {S88017, 148, { &P18280,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC One S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3102892 = {S88018, 148, { &P18280,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC VLE_U Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2999825 = {S88019, 88, { &P19996,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A15) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3013603 = {S88020, 24, { &P15364,&P24739,&P18246,&P18092 }};	/*Useragent - Maui Browser Karbonn K73*/
const Device UA3015230 = {S88021, 101, { &P18220,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia808 PureView) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3020567 = {S88022, 137, { &P20013,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F180S Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3062355 = {S88023, 158, { &P18449,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; ST27i Build/6.2.A.1.100) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3079607 = {S88024, 158, { &P18303,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; MT27i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3025168 = {S88025, 176, { &P12973,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3027671 = {S88026, 173, { &P22209,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Titanium_S5 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3048830 = {S88027, 169, { &P22209,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; Titanium S5 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444*/
const Device UA3030592 = {S88028, 156, { &P23521,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3060642 = {S88029, 139, { &P24606,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; LG-E470f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3083489 = {S88030, 134, { &P21823,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTX21 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094672 = {S88031, 130, { &P13452,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3110238 = {S88032, 141, { &P18466,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Amazon Kindle Fire HD Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094602 = {S88033, 130, { &P24100,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CAL21 Build/AX) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094930 = {S88034, 134, { &P23894,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V1277 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093771 = {S88035, 130, { &P23755,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126492 = {S88036, 156, { &P14540,&P5843,&P24680,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.71) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; NokiaX2-01) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3131420 = {S88037, 139, { &P18449,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST27i Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132705 = {S88038, 137, { &P18283,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA12448 = {S88039, 51, { &P13502,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch2_T3335 Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA16011 = {S88040, 9, { &P11850,&P5926,&P4030,&P18092 }};	/*Useragent - nokia2626*/
const Device UA20742 = {S88041, 107, { &P15132,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-T329/T329UVFL8 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA63153 = {S88042, 61, { &P14282,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia7250I/1.0 (3.22) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA50355 = {S88043, 74, { &P13392,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-C3510/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA30484 = {S88044, 78, { &P12319,&P5026,&P3410,&P18092 }};	/*Useragent - BlackBerry8130m/4.5.0.127 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/294*/
const Device UA56802 = {S88045, 34, { &P13977,&P5890,&P4715,&P18092 }};	/*Useragent - NokiaN73/UC Browser7.4.0.65/28/351*/
const Device UA62354 = {S88046, 39, { &P16119,&P17486,&P3807,&P18092 }};	/*Useragent - Mot-WX290/1.0 ObigoInternetBrowser/Q03C*/
const Device UA64250 = {S88047, 63, { &P14585,&P22839,&P3601,&P18092 }};	/*Useragent - ZTE-F912/1.0 ACS-NF/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA657185 = {S88048, 54, { &P17867,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SH3H UP.Browser/6.2_7.2.7.1.K.5.207 (GUI) MMP/2.0*/
const Device UA103294 = {S88049, 10, { &P11907,&P5890,&P4030,&P18092 }};	/*Useragent - NOKIAE52-1*/
const Device UA292189 = {S88050, 116, { &P17595,&P17294,&P22889,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; en) AppleWebKit/533.19.4 (KHTML, like Gecko) iCab/4.8 Safari/533.16*/
const Device UA350960 = {S88051, 189, { &P15364,&P7133,&P17347,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/69/352*/
const Device UA648717 = {S88052, 189, { &P15364,&P7133,&P17347,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/69/444*/
const Device UA814584 = {S88053, 199, { &P15364,&P7133,&P17347,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/69/444  3gpp-gba*/
const Device UA1938531 = {S88054, 189, { &P15364,&P7133,&P17347,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/70/352*/
const Device UA2149248 = {S88055, 199, { &P15364,&P7133,&P17347,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser8.0.3.107/69/352  3gpp-gba*/
const Device UA355875 = {S88056, 44, { &P20126,&P17485,&P17347,&P18092 }};	/*Useragent - sonyericssonk800i/UC Browser8.0.3.107/70/352*/
const Device UA700837 = {S88057, 44, { &P20126,&P17485,&P17347,&P18092 }};	/*Useragent - sonyericssonk800i/UC Browser8.0.3.107/69/352*/
const Device UA710559 = {S88058, 44, { &P20126,&P17485,&P17347,&P18092 }};	/*Useragent - sonyericssonk800i/UC Browser8.0.3.107/69/444*/
const Device UA397183 = {S88059, 41, { &P13163,&P5129,&P24663,&P18092 }};	/*Useragent - NokiaN97/UC Browser7.9.0.102/50/355/UCWEB*/
const Device UA454788 = {S88060, 136, { &P12280,&P18117,&P20782,&P18092 }};	/*Useragent - MQQBrowser/31 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3*/
const Device UA454787 = {S88061, 136, { &P12280,&P18117,&P20782,&P18092 }};	/*Useragent - MQQBrowser/31 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3*/
const Device UA883142 = {S88062, 138, { &P12280,&P18117,&P20782,&P18092 }};	/*Useragent - MQQBrowser/31 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA574491 = {S88063, 77, { &P15364,&P17350,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 7.1; ru; rv:1.9.2.15) Gecko/X Firefox/4.0*/
const Device UA2215030 = {S88064, 79, { &P15364,&P17350,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 7.0; Win64; x64; rv:3.0b2pre) Gecko/X Firefox/4.0b12pre*/
const Device UA654857 = {S88065, 78, { &P17595,&P17295,&P18192,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:9.0) Gecko/X Thunderbird/9.0.1*/
const Device UA671997 = {S88066, 62, { &P16319,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux; U;2.3.6;zh_cn;U8860;480*854;)UCWEB7.6.0.75/139/7675*/
const Device UA2026791 = {S88067, 165, { &P13974,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5670 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2031919 = {S88068, 135, { &P22018,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-sg; Slicepad L8 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2376082 = {S88069, 135, { &P22018,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-sg; Slicepad L8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2232520 = {S88070, 168, { &P12616,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; WT19i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2339808 = {S88071, 138, { &P18644,&P5043,&P24668,&P18092 }};	/*Useragent - Nokia112/2.0 (03.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia112) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2362637 = {S88072, 157, { &P14919,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC_Pyramid Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2427415 = {S88073, 138, { &P20525,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A898 Duo Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2458797 = {S88074, 168, { &P15397,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST18i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2480827 = {S88075, 64, { &P21064,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; A44; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2485805 = {S88076, 173, { &P18295,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (HTC One X, 39, 1.8.2)*/
const Device UA2654619 = {S88077, 119, { &P13536,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA6110) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2951468 = {S88078, 100, { &P13536,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6110navigator) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2538022 = {S88079, 168, { &P15397,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST18i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2553718 = {S88080, 44, { &P12137,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SAMSUNG-GT-S5620/S5620DDKC1)*/
const Device UA2558333 = {S88081, 170, { &P18462,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC Desire X Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2708683 = {S88082, 163, { &P17308,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2578412 = {S88083, 112, { &P18447,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; LT30p-o) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2597647 = {S88084, 135, { &P23745,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Mi-Book Mi7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2598656 = {S88085, 171, { &P18297,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I8530 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2603898 = {S88086, 130, { &P21990,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; chagall Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2657452 = {S88087, 69, { &P15575,&P18111,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.1.2; zh-cn; GT-I9100; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2658953 = {S88088, 212, { &P17835,&P5629,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK BNTV250 Build/GINGERBREAD 1.4.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, -0500, us*/
const Device UA2659514 = {S88089, 93, { &P18297,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I8530) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2661276 = {S88090, 92, { &P14967,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; LG-P500) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2668416 = {S88091, 140, { &P15767,&P17090,&P20781,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/8J2 Safari/7534.48.3*/
const Device UA2793555 = {S88092, 140, { &P15767,&P17090,&P20781,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/8L1 Safari/7534.48.3*/
const Device UA2672314 = {S88093, 137, { &P19902,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530M Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673252 = {S88094, 137, { &P18637,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9260 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675060 = {S88095, 136, { &P22097,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTabS2110AF Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2692107 = {S88096, 139, { &P14433,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9003 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2832606 = {S88097, 61, { &P14433,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9003 Build/JZO54K)*/
const Device UA3085777 = {S88098, 139, { &P14433,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9003 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2870545 = {S88099, 100, { &P16384,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5380k) U2/1.0.0 UCBrowser/9.0.0.295 U2/1.0.0 Mobile*/
const Device UA2713951 = {S88100, 136, { &P20713,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID7065 Build/ICS.MID7065.X) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2726687 = {S88101, 129, { &P21975,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; TM901 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2849589 = {S88102, 129, { &P21975,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; TM901 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2727214 = {S88103, 138, { &P23289,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; ZTE-V6500 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2730620 = {S88104, 154, { &P12731,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; MB855 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3071321 = {S88105, 154, { &P12731,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; MB855 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2750817 = {S88106, 160, { &P19998,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA3072861 = {S88107, 162, { &P19998,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8812E Build/HuaweiC8812E) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2752422 = {S88108, 93, { &P15919,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-P3100) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2767242 = {S88109, 53, { &P15364,&P6704,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows 98) Presto/2.12.388 Version/12.15*/
const Device UA2767288 = {S88110, 193, { &P18472,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (DROID RAZR HD, 42, 1.9.1)*/
const Device UA3024386 = {S88111, 186, { &P18472,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-79) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (DROID RAZR HD, 52, 1.9.9.2)*/
const Device UA2767660 = {S88112, 90, { &P22641,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; C1504) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2768551 = {S88113, 158, { &P18672,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2773857 = {S88114, 129, { &P18061,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; rk30sdk Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2892853 = {S88115, 155, { &P21953,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Ctrl_V1 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2798819 = {S88116, 139, { &P15147,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST15i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798842 = {S88117, 197, { &P16356,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.2/552,2.0.2.552,2013-05-22 23:05, +0100, gb*/
const Device UA2798992 = {S88118, 139, { &P15517,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT15i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2894240 = {S88119, 159, { &P19999,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A30 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2867143 = {S88120, 165, { &P12987,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; SPH-M930BST Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2867475 = {S88121, 156, { &P23004,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; TECNO D3 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2869609 = {S88122, 130, { &P12889,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7300 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2875470 = {S88123, 139, { &P23829,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; G2 Build/ICS.Trio_Stealth_G2.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2943492 = {S88124, 165, { &P22612,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI T8951 Build/HuaweiT8951) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2892891 = {S88125, 142, { &P22654,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5502 Build/10.1.1.A.1.310) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2900553 = {S88126, 97, { &P22622,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Micromax A72) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2938611 = {S88127, 156, { &P16356,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3006643 = {S88128, 156, { &P16356,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3013827 = {S88129, 125, { &P17595,&P17297,&P24729,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) FluidApp Version/1842 Safari/534.57.7*/
const Device UA2943811 = {S88130, 169, { &P20005,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A65 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2944227 = {S88131, 152, { &P18721,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A90S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2945982 = {S88132, 158, { &P12616,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; WT19i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2965170 = {S88133, 153, { &P19907,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C5155 Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2965607 = {S88134, 158, { &P20076,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26w Build/6.1.A.2.45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3022371 = {S88135, 134, { &P23124,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A919i Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020137 = {S88136, 150, { &P15985,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3022567 = {S88137, 132, { &P19860,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A9+ Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023123 = {S88138, 130, { &P19852,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7511 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3026605 = {S88139, 158, { &P15397,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST18i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3027879 = {S88140, 93, { &P13974,&P5096,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5670) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3029123 = {S88141, 135, { &P22582,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SGH-I337 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3032685 = {S88142, 151, { &P20983,&P18066,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9308 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3094801 = {S88143, 136, { &P22660,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P778 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114296 = {S88144, 133, { &P15390,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P925 Build/ICS) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3064146 = {S88145, 142, { &P23838,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; StarTrail III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3094027 = {S88146, 135, { &P23165,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; H7500+ Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066818 = {S88147, 139, { &P13018,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT11i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3068898 = {S88148, 125, { &P15767,&P24384,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D36 Safari/9537.53*/
const Device UA3068900 = {S88149, 125, { &P15767,&P24384,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D41 Safari/9537.53*/
const Device UA3068901 = {S88150, 125, { &P15767,&P24384,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D48 Safari/9537.53*/
const Device UA3068902 = {S88151, 125, { &P15767,&P24384,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D49 Safari/9537.53*/
const Device UA3068904 = {S88152, 125, { &P15767,&P24384,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D52 Safari/9537.53*/
const Device UA3068903 = {S88153, 125, { &P15767,&P24384,&P24305,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11D51 Safari/9537.53*/
const Device UA3078320 = {S88154, 138, { &P15972,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One XL Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3079796 = {S88155, 97, { &P14519,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaN97 mini) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3109624 = {S88156, 143, { &P23546,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; New Andromax-i Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080435 = {S88157, 155, { &P20146,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9082 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3083394 = {S88158, 128, { &P18702,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-N8013 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3093074 = {S88159, 92, { &P12280,&P19826,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 6_1_2; zh-CN; iPh4,1) U2/1.0.0 UCBrowser/9.2.0.312 U2/1.0.0 Mobile*/
const Device UA3108262 = {S88160, 92, { &P12280,&P19826,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 6_1_2; zh-CN; iPh5,2) U2/1.0.0 UCBrowser/9.2.0.312 U2/1.0.0 Mobile*/
const Device UA3115336 = {S88161, 157, { &P18283,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA5143 = {S88162, 60, { &P13815,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_asia_india / 1.0 / htc_buzz / 2.2.1*/
const Device UA5145 = {S88163, 57, { &P13815,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player vodafone_uk / 1.0 / htc_buzz / 2.2.1*/
const Device UA5147 = {S88164, 58, { &P13815,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_asia_wwe / 1.0 / htc_buzz / 2.2.1*/
const Device UA5148 = {S88165, 53, { &P13815,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_wwe / 1.0 / htc_buzz / 2.2.1*/
const Device UA36845 = {S88166, 53, { &P13815,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player h3g_dan / 1.0 / htc_buzz / 2.2.1*/
const Device UA36848 = {S88167, 57, { &P13815,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player telstra_wwe / 1.0 / htc_buzz / 2.2.1*/
const Device UA45695 = {S88168, 63, { &P13815,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player vodafone_de_1und1 / 1.0 / htc_buzz / 2.2.1*/
const Device UA45696 = {S88169, 52, { &P13815,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player tmo_uk / 1.0 / htc_buzz / 2.2.1*/
const Device UA45697 = {S88170, 59, { &P13815,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_wwe_tesco / 1.0 / htc_buzz / 2.2.1*/
const Device UA45699 = {S88171, 52, { &P13815,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player h3g_uk / 1.0 / htc_buzz / 2.2.1*/
const Device UA45702 = {S88172, 58, { &P13815,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player vodafone_spa / 1.0 / htc_buzz / 2.2.1*/
const Device UA45704 = {S88173, 57, { &P13815,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_wwe-afk / 1.0 / htc_buzz / 2.2.1*/
const Device UA77927 = {S88174, 57, { &P13815,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player vodafone_fr / 1.0 / htc_buzz / 2.2.1*/
const Device UA438828 = {S88175, 57, { &P13815,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player vodafone_de / 1.0 / htc_buzz / 2.2.1*/
const Device UA28485 = {S88176, 172, { &P11938,&P17486,&P4112,&P18092 }};	/*Useragent - MOT-E8/R6713_G_71.14.1ER_B MotoWebKit/417.19 (KHTML, like Gecko) BER/2.0 Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/9.0*/
const Device UA23336 = {S88177, 11, { &P15591,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M5170*/
const Device UA23439 = {S88178, 141, { &P15606,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; cs-cz; GSmart G1317D Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA60655 = {S88179, 142, { &P15606,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; cs-cz; GSmart G1317D Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32091 = {S88180, 40, { &P15672,&P17483,&P17470,&P18092 }};	/*Useragent - LGE-LG260 POLARIS-LG260/2.0(GUI) MMP/2.0*/
const Device UA87435 = {S88181, 74, { &P15672,&P17483,&P17470,&P18092 }};	/*Useragent - LG-LG260 POLARIS-LG260/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA42790 = {S88182, 119, { &P15060,&P5096,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.2.1;Zh_cn;GT-S5660;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA2020348 = {S88183, 70, { &P15060,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2.1;Zh_cn;GT-S5660;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA44303 = {S88184, 78, { &P13824,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-ONIDA F970/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA506527 = {S88185, 148, { &P18081,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SKY IM-A600S Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA454456 = {S88186, 94, { &P17595,&P17295,&P18329,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:11.0) Gecko/X Thunderbird/11.0.1 Lightning/1.3*/
const Device UA639955 = {S88187, 130, { &P16146,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT702 Build/SGC03040) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA651915 = {S88188, 144, { &P16146,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT702 Build/SGC03040) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA534789 = {S88189, 135, { &P13278,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; X10mini Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018591 = {S88190, 59, { &P13278,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; X10mini Build/FRG83)*/
const Device UA706772 = {S88191, 140, { &P12562,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID2 GLOBAL Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA741956 = {S88192, 136, { &P12731,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB855 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA803454 = {S88193, 136, { &P12731,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MB855 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA809979 = {S88194, 136, { &P12731,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB855 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1981412 = {S88195, 136, { &P12731,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB855 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033836 = {S88196, 136, { &P12731,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB855 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA754942 = {S88197, 47, { &P12449,&P5890,&P17338,&P18092 }};	/*Useragent - nokiae5-00/UC Browser7.9.0.102/69/352  3gpp-gba*/
const Device UA846714 = {S88198, 62, { &P15364,&P18144,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64; rv:11.0) Gecko/X Firefox/11.0*/
const Device UA855682 = {S88199, 142, { &P22086,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; iDx10 3G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028467 = {S88200, 166, { &P16356,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-N7000 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2643850 = {S88201, 166, { &P16356,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-N7000 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA1905951 = {S88202, 136, { &P19978,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; GT-I9018 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020436 = {S88203, 124, { &P19978,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; GT-I9018 Build/FROYO) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2026527 = {S88204, 129, { &P19978,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; GT-I9018 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2028365 = {S88205, 151, { &P21335,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5.1.12-RT-X.X; zh-cn; K-Touch W619 Build/AliyunOs-2012) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2395942 = {S88206, 151, { &P21335,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5.1.13-RT-X.X; zh-cn; K-Touch W619 Build/AliyunOs-2012) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2029200 = {S88207, 161, { &P12532,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC Flyer P512 Build/JRO03C; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2163988 = {S88208, 51, { &P21242,&P18110,&P17810,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; Lenovo_A366t; 320*480)*/
const Device UA2216518 = {S88209, 77, { &P16087,&P5448,&P4777,&P18092 }};	/*Useragent - 480x800 LGE VS750 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2301658 = {S88210, 113, { &P15364,&P16110,&P22452,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1*/
const Device UA2786064 = {S88211, 120, { &P15364,&P16110,&P22452,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Iron/14.0.850.0 Chrome/14.0.850.0 Safari/535.1*/
const Device UA2335364 = {S88212, 52, { &P18095,&P5890,&P20777,&P18092 }};	/*Useragent - MQQBrowser/2.6 (NokiaNXX;SymbianOS/9.3 Series60/3.2)*/
const Device UA2398563 = {S88213, 147, { &P21093,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MEDION LIFE P4012 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2422360 = {S88214, 99, { &P13206,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonk530i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2469870 = {S88215, 165, { &P18687,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; LG-P705 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2484996 = {S88216, 166, { &P15575,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2507969 = {S88217, 160, { &P21489,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Rezound Build/IML74K; CyanogenMod-RAGE 2.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2602014 = {S88218, 138, { &P18021,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A770K Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2617855 = {S88219, 55, { &P15919,&P17979,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.5610 Android/4.0.4 Device/samsung-GT-P3100*/
const Device UA2625178 = {S88220, 190, { &P17337,&P6437,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0200, za*/
const Device UA2656913 = {S88221, 138, { &P23870,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TCL S710 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2666962 = {S88222, 138, { &P12496,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I897 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667185 = {S88223, 127, { &P19775,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A210 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2670609 = {S88224, 133, { &P18721,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A90S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2672186 = {S88225, 145, { &P20105,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A919 Duo Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2678415 = {S88226, 139, { &P20738,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Andromax U Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2699532 = {S88227, 164, { &P14967,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2792314 = {S88228, 162, { &P14967,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; LG-P500 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2889714 = {S88229, 164, { &P14967,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; LG-P500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2699551 = {S88230, 138, { &P23384,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NB-1458 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2701217 = {S88231, 140, { &P12955,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; ST3_Blade Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2703175 = {S88232, 91, { &P15985,&P6437,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 3.2; en-US; GT-P6200) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2705688 = {S88233, 162, { &P11660,&P17485,&P24676,&P18092 }};	/*Useragent - SonyEricssonC902/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonc902) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2707646 = {S88234, 88, { &P21057,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A7+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2709492 = {S88235, 143, { &P21071,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Novo 10 Hero QuadCore Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2733523 = {S88236, 11, { &P23277,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M6100*/
const Device UA2738068 = {S88237, 93, { &P15575,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; GT-I9100) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2766727 = {S88238, 93, { &P14186,&P5629,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; zh-CN; GT-S5830) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile*/
const Device UA2768346 = {S88239, 163, { &P22998,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; TECNO_N7 Build/TECNOTECNO_N7) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2768717 = {S88240, 96, { &P13979,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; Nokia6220c-1) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2773625 = {S88241, 139, { &P20708,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XOLO A1000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782367 = {S88242, 134, { &P23090,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i9400 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783093 = {S88243, 135, { &P22095,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ThL W7 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783702 = {S88244, 165, { &P13064,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799512 = {S88245, 162, { &P18297,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-I8530 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2864377 = {S88246, 159, { &P22632,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Titanium S1 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2962910 = {S88247, 159, { &P20183,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI-ONE Plus Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2935392 = {S88248, 93, { &P15364,&P5768,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3110c) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2935511 = {S88249, 92, { &P15364,&P5768,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6131) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2986821 = {S88250, 89, { &P15364,&P5768,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; vi; nokia6085) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2954554 = {S88251, 137, { &P23053,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GN700W Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3087505 = {S88252, 59, { &P23053,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GN700W Build/JRO03C)*/
const Device UA2958886 = {S88253, 139, { &P20468,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-nz; innos i6 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3042499 = {S88254, 140, { &P20468,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; i6 Build/BOWAYBOWAY_i6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2963980 = {S88255, 160, { &P18462,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC Desire X Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3124329 = {S88256, 169, { &P18462,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC Desire X dual sim Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2964020 = {S88257, 159, { &P15397,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; ST18i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3061783 = {S88258, 165, { &P19891,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3023539 = {S88259, 136, { &P19911,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-E970 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3013902 = {S88260, 53, { &P21032,&P18615,&P24713,&P18092 }};	/*Useragent - Mozilla/5.0 (Mobile; rv:18.1) Gecko/18.1 Firefox/18.1*/
const Device UA3059000 = {S88261, 156, { &P18713,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P760 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3092648 = {S88262, 156, { &P18713,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P768 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3022045 = {S88263, 129, { &P23236,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AMI-TT4 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3062147 = {S88264, 160, { &P19779,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A1+ Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3028797 = {S88265, 156, { &P18582,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT28h Build/6.1.E.3.7) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3093565 = {S88266, 157, { &P18582,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT28h Build/6.1.E.2.68) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3028857 = {S88267, 95, { &P14202,&P5477,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E15i) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3029186 = {S88268, 90, { &P17625,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; ST25i) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3058895 = {S88269, 178, { &P18267,&P5043,&P24055,&P18092 }};	/*Useragent - NokiaX2-02/2.0 (11.79) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311*/
const Device UA3059544 = {S88270, 183, { &P21475,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A35 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3060580 = {S88271, 156, { &P23776,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGP321 Build/10.3.1.A.0.244) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3109243 = {S88272, 134, { &P18253,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT894 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3062276 = {S88273, 105, { &P15364,&P18066,&P23371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/19.77.34.5 Safari/537.22*/
const Device UA3125855 = {S88274, 105, { &P15364,&P18066,&P23371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/19.77.34.5 Safari/537.22*/
const Device UA3111523 = {S88275, 134, { &P23477,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT926 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3113441 = {S88276, 136, { &P14969,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Desire S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094686 = {S88277, 132, { &P23807,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A50 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110123 = {S88278, 156, { &P22594,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3094778 = {S88279, 138, { &P24558,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8550L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3115118 = {S88280, 150, { &P18702,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N8013 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3130739 = {S88281, 127, { &P17926,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3130942 = {S88282, 129, { &P20991,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ME371MG Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3132648 = {S88283, 126, { &P18282,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A200 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA6616 = {S88284, 225, { &P16176,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-cn; HTCA3366/1.0 Android/2.2 release/06.23.2010 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA8882 = {S88285, 39, { &P13358,&P17486,&P3807,&P18092 }};	/*Useragent - Mot-WX395/1.0 ObigoInternetBrowser/Q03C*/
const Device UA9276 = {S88286, 87, { &P13402,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_E101*/
const Device UA23337 = {S88287, 134, { &P15592,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-tw; myPad P1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA23340 = {S88288, 66, { &P15593,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KM553/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA26365 = {S88289, 46, { &P13588,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-AX265/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA158114 = {S88290, 31, { &P14644,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P02A(c100;TB;W24H15)*/
const Device UA56151 = {S88291, 47, { &P11684,&P5129,&P4715,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UC Browser7.4.0.65/50/351*/
const Device UA68899 = {S88292, 116, { &P11679,&P5776,&P4777,&P18092 }};	/*Useragent - Nokia6630/1.0 (5.03.08) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.7.1.88/27/444*/
const Device UA69287 = {S88293, 36, { &P12088,&P5129,&P4779,&P18092 }};	/*Useragent - NokiaX6-00/UC Browser7.8.0.95/50/355*/
const Device UA86998 = {S88294, 79, { &P14114,&P18110,&P3601,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Linux 2.6.10) NetFront/3.3 Kindle/1.0 (screen 600x800)*/
const Device UA230558 = {S88295, 137, { &P15759,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Vision Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA566840 = {S88296, 145, { &P15759,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Vision Build/MIUI OsiMood) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA576569 = {S88297, 137, { &P15759,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Vision Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018995 = {S88298, 145, { &P15759,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Vision Build/MIUI OsiMood) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA392663 = {S88299, 141, { &P14225,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ja-jp; 001HT Build/GRH78C) AppleWebKit/533.16 (KHTML, like Gecko) Version/4.0 Safari/533.16*/
const Device UA478898 = {S88300, 80, { &P15364,&P17285,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.7.10) Gecko/X Firefox/1.0.6*/
const Device UA645827 = {S88301, 89, { &P15364,&P17285,&P22350,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:8.0) Gecko/X Firefox/8.0-x64 PaleMoon/8.0-x64*/
const Device UA658781 = {S88302, 76, { &P15364,&P17285,&P22350,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:8.0) Gecko/X Firefox/8.0 PaleMoon/8.0*/
const Device UA706081 = {S88303, 152, { &P13078,&P5096,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-U V880 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA761652 = {S88304, 137, { &P15575,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; es-es; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2024021 = {S88305, 137, { &P15575,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; es-es; GT-I9100 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033146 = {S88306, 140, { &P15575,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; es-es; GT-I9100 Build/IMM76I-RR) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA870177 = {S88307, 78, { &P17595,&P18120,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US; rv:1.8.1.13) Gecko/X Firefox*/
const Device UA2308371 = {S88308, 76, { &P17595,&P18120,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; rv:1.8.1.16) Gecko/X Firefox*/
const Device UA906428 = {S88309, 80, { &P17595,&P17295,&P18514,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:13.0) Gecko/X Thunderbird/13.0.1*/
const Device UA907709 = {S88310, 136, { &P12216,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6300 Build/GRJ22) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1931153 = {S88311, 69, { &P14123,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.2; zh-cn; LG-P970; 480*800) UCWEB7.9.3.103/139/800*/
const Device UA2401759 = {S88312, 68, { &P14123,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; LG-P970; 480*800) UCWEB7.9.0.94/139/352*/
const Device UA1950526 = {S88313, 78, { &P13602,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; gt-s5233w) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2410324 = {S88314, 75, { &P13602,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; vi; gt-s5233w) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA1956203 = {S88315, 145, { &P18620,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MID7022 Build/CaribbeanSunset_v1.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1963683 = {S88316, 136, { &P21263,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; es-es; OnePAD 725 Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1995409 = {S88317, 66, { &P15364,&P17285,&P18330,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:14.0) Gecko/X Firefox/14.0.1 slurp*/
const Device UA1995638 = {S88318, 73, { &P15364,&P17285,&P18330,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:14.0) Gecko/X Firefox/14.0.1 slurp*/
const Device UA2026924 = {S88319, 142, { &P14433,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2686815 = {S88320, 139, { &P14433,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-I9003 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2809387 = {S88321, 61, { &P14433,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-I9003 Build/IMM76L)*/
const Device UA2364322 = {S88322, 153, { &P13031,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; U8510 Build/supertoast; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028088 = {S88323, 158, { &P15877,&P18066,&P18338,&P18091 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID SPYDER Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2316546 = {S88324, 131, { &P18699,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID8042 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2332630 = {S88325, 151, { &P15364,&P17285,&P22766,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618; Maxthon/3.0)*/
const Device UA2495279 = {S88326, 110, { &P15364,&P17285,&P22766,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.04506; Maxthon/3.0)*/
const Device UA2411428 = {S88327, 146, { &P12135,&P17483,&P24670,&P18092 }};	/*Useragent - LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-gd510) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2466512 = {S88328, 168, { &P18449,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; ST27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2401732 = {S88329, 163, { &P12871,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2424305 = {S88330, 140, { &P23913,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHP-SH630E Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2468521 = {S88331, 142, { &P19851,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax A68 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2484685 = {S88332, 128, { &P16598,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; A500 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2699208 = {S88333, 127, { &P16598,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; A500 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2533211 = {S88334, 92, { &P16003,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233a) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2580066 = {S88335, 89, { &P16003,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; fa; gt-s5233a) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2764280 = {S88336, 85, { &P16003,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233a) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0*/
const Device UA2545821 = {S88337, 138, { &P22348,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; iDx7 Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2742995 = {S88338, 125, { &P22348,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; iDx7 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2594808 = {S88339, 141, { &P14993,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Arc Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558454 = {S88340, 171, { &P18700,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-B5512 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2705974 = {S88341, 154, { &P14207,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2602399 = {S88342, 135, { &P23376,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M9000 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2653050 = {S88343, 137, { &P22549,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CYA-52 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659535 = {S88344, 166, { &P18582,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT28h Build/6.1.E.3.7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2666879 = {S88345, 139, { &P20128,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT-AS43D2 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667836 = {S88346, 139, { &P20128,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT-AS43D2 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2666963 = {S88347, 143, { &P20253,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 5 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2694482 = {S88348, 32, { &P17475,&P5768,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia6300/07.21)*/
const Device UA2713878 = {S88349, 152, { &P22961,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010D Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2733277 = {S88350, 93, { &P20512,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Fly_F350) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2737675 = {S88351, 176, { &P12973,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752884 = {S88352, 153, { &P15575,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2997832 = {S88353, 153, { &P15575,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2879443 = {S88354, 155, { &P21023,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-E405 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2769886 = {S88355, 128, { &P23135,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; U30GT 2 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2771240 = {S88356, 136, { &P14017,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-P990 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774401 = {S88357, 136, { &P22871,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9295 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2781848 = {S88358, 99, { &P14972,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3370) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2783879 = {S88359, 32, { &P15364,&P17017,&P21920,&P18092 }};	/*Useragent - contype chromeframe/27.0.1453.94*/
const Device UA2872155 = {S88360, 33, { &P15364,&P17017,&P21920,&P18092 }};	/*Useragent - contype chromeframe/27.0.1453.110*/
const Device UA2886597 = {S88361, 33, { &P15364,&P17017,&P21920,&P18092 }};	/*Useragent - contype chromeframe/27.0.1453.116*/
const Device UA2784250 = {S88362, 126, { &P18008,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ST10 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2784835 = {S88363, 129, { &P21990,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; chagall Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2795586 = {S88364, 95, { &P14202,&P5477,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E15i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2795889 = {S88365, 151, { &P16598,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA2991044 = {S88366, 147, { &P16598,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2868937 = {S88367, 159, { &P20169,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; N320 Build/LAVALAVA_N320) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2799008 = {S88368, 156, { &P21026,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; ZTE U930 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2799997 = {S88369, 145, { &P23153,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ultrafone 701HD Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2866227 = {S88370, 88, { &P20515,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; A54) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2876182 = {S88371, 133, { &P20556,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A701 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3088599 = {S88372, 126, { &P20556,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A701 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2879748 = {S88373, 141, { &P23505,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Vodafone 875 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879900 = {S88374, 99, { &P15547,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-c6712) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3003066 = {S88375, 155, { &P20709,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; MID1042 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2892453 = {S88376, 149, { &P18461,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Grand X In Build/GXI_THUV1.0.0B06) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892547 = {S88377, 130, { &P23413,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PLT7223G Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2894487 = {S88378, 137, { &P22610,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2938523 = {S88379, 138, { &P22610,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2943861 = {S88380, 159, { &P13018,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; MT11i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2944117 = {S88381, 134, { &P22847,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SH-06E Build/S6170) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2953517 = {S88382, 157, { &P21543,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sr-rs; Telenor_Smart_HD Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2961134 = {S88383, 91, { &P14545,&P17483,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-x330t) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2965036 = {S88384, 99, { &P14912,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-c3222) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3124610 = {S88385, 223, { &P14912,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-c3222) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile, SAMSUNG-GT-C3222/C3222DDKH2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA3020940 = {S88386, 141, { &P20491,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Thunder S220 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3005751 = {S88387, 175, { &P24066,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; Lenovo A706_ROW Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33482*/
const Device UA3013972 = {S88388, 134, { &P18296,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.3; en-in; HTC One V Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3022545 = {S88389, 128, { &P23873,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BC1003 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3031582 = {S88390, 153, { &P20129,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MI 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3060744 = {S88391, 149, { &P18320,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3107110 = {S88392, 156, { &P18320,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3023813 = {S88393, 136, { &P18655,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SH-09D Build/S7080) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3025616 = {S88394, 138, { &P22660,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-P778 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3026137 = {S88395, 99, { &P19862,&P18598,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Micromax A110Q) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3058584 = {S88396, 160, { &P12135,&P17483,&P24679,&P18092 }};	/*Useragent - LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gd510) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3059172 = {S88397, 176, { &P18643,&P5043,&P24055,&P18092 }};	/*Useragent - Nokia311/5.0 (07.36) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311*/
const Device UA3065361 = {S88398, 139, { &P13054,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ADR6425LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3109273 = {S88399, 135, { &P12814,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SGH-I777 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093656 = {S88400, 137, { &P23647,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Cynus T5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3078273 = {S88401, 136, { &P12109,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; B6000 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3109628 = {S88402, 130, { &P17337,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3108665 = {S88403, 178, { &P14711,&P5204,&P24055,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (11.20) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311*/
const Device UA62894 = {S88404, 79, { &P14437,&P17486,&P3402,&P18092 }};	/*Useragent - MOT-W396/0.1.43 UP.Browser/6.3.0.6.c.27 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA5092 = {S88405, 52, { &P13345,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-WX415/00.72 UP.Browser/7.2.7.5.708 (GUI) MMP/2.0*/
const Device UA7144 = {S88406, 95, { &P14830,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE62/3.0 (2.0618.06.32); SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA55875 = {S88407, 98, { &P14830,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE62-1/3.0 (2.0618.06.17); SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1)*/
const Device UA50443 = {S88408, 62, { &P13183,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonK810i/R8BA Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA719039 = {S88409, 103, { &P14862,&P5204,&P4779,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2700c-2/UC Browser7.8.0.95/70/355*/
const Device UA50379 = {S88410, 35, { &P14188,&P5129,&P4722,&P18092 }};	/*Useragent - Nokia5233/UC Browser7.0.0.41/50/351*/
const Device UA68614 = {S88411, 36, { &P14748,&P5890,&P4777,&P18092 }};	/*Useragent - NokiaE72-1/UC Browser7.7.1.88/28/444*/
const Device UA774445 = {S88412, 46, { &P14748,&P5890,&P4777,&P18092 }};	/*Useragent - nokiae72-1/UC Browser7.7.1.88/69/444  3gpp-gba*/
const Device UA87134 = {S88413, 45, { &P17504,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-CA3A UP.Browser/6.2.0.13.2 (GUI) MMP/2.0*/
const Device UA91545 = {S88414, 33, { &P17891,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/H699/Screen-176X220*/
const Device UA882358 = {S88415, 145, { &P17595,&P17295,&P17392,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.7 (KHTML, like Gecko) RockMelt/0.8.36.116 Chrome/7.0.517.44 Safari/534.7*/
const Device UA896232 = {S88416, 148, { &P17595,&P17295,&P17392,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.10 (KHTML, like Gecko) RockMelt/0.8.40.147 Chrome/8.0.552.231 Safari/534.10*/
const Device UA2258231 = {S88417, 145, { &P17595,&P17295,&P17392,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.3 (KHTML, like Gecko) RockMelt/0.8.34.820 Chrome/6.0.472.63 Safari/534.3*/
const Device UA158280 = {S88418, 150, { &P14611,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Nexian NX-A890 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA616256 = {S88419, 72, { &P14611,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1-update1; Nexian NX-A890 Build/ERE27)*/
const Device UA314214 = {S88420, 143, { &P13668,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Magic/Dream Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA421189 = {S88421, 142, { &P13668,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Sapphire/Dream Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA671739 = {S88422, 142, { &P13668,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Sapphire/dream Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1904949 = {S88423, 142, { &P13668,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Dream/Sapphire Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018313 = {S88424, 143, { &P13668,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC Magic/Dream Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018890 = {S88425, 137, { &P13668,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Dream Build/GRI34) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021800 = {S88426, 142, { &P13668,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Dream/Sapphire Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA601334 = {S88427, 92, { &P17595,&P17297,&P18329,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:11.0) Gecko/X Thunderbird/11.0 Lightning/1.3*/
const Device UA664355 = {S88428, 134, { &P20557,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; MZ606 Build/H.6.2-9_CN-37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2150354 = {S88429, 35, { &P14763,&P5776,&P17347,&P18092 }};	/*Useragent - nokian72/UC Browser8.0.3.107/69/444*/
const Device UA840345 = {S88430, 37, { &P15364,&P17188,&P17213,&P18092 }};	/*Useragent - Opera/8.54 (Windows NT 4.0; U; zh-cn)*/
const Device UA2391961 = {S88431, 37, { &P15364,&P17188,&P17213,&P18092 }};	/*Useragent - Opera/8.50 (Windows NT 4.0; U; zh-cn)*/
const Device UA1934264 = {S88432, 127, { &P20369,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-C N880S Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2024870 = {S88433, 139, { &P20369,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh_TW; ZTE-C N880S Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026765 = {S88434, 132, { &P20369,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-C N880S Build/FRF91) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA1969898 = {S88435, 134, { &P21276,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; QW TB-9718 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2007708 = {S88436, 172, { &P12280,&P18117,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3 UCBrowser8.4.0.159/42/355*/
const Device UA2837214 = {S88437, 172, { &P12280,&P18117,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B208 Safari/7534.48.3 UCBrowser8.4.0.159/42/355*/
const Device UA2033302 = {S88438, 152, { &P20206,&P5629,&P20776,&P18092 }};	/*Useragent - MQQBrowser/25 Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE-N880E Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2159134 = {S88439, 62, { &P11684,&P5890,&P21746,&P18092 }};	/*Useragent - iBrowser/2.6 (Nokia5800XpressMusic;SymbianOS/9.1 Series60/3.0)*/
const Device UA2216822 = {S88440, 47, { &P13552,&P5129,&P17347,&P18092 }};	/*Useragent - nokiac6-00/UC Browser8.0.3.107/69/444  3gpp-gba*/
const Device UA2373409 = {S88441, 70, { &P18731,&P18110,&P20793,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; ONDA_MID; 800*444) UCWEB1.0.1.113/147/800*/
const Device UA2242375 = {S88442, 52, { &P18364,&P5890,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 (NokiaC5-05;SymbianOS/9.1 Series60/3.0)*/
const Device UA2482271 = {S88443, 137, { &P18683,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ISW13F Build/V69R51I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2364306 = {S88444, 49, { &P14358,&P5890,&P20777,&P18092 }};	/*Useragent - MQQBrowser/2.6 (E63-1;SymbianOS/9.2 Series60/3.1)*/
const Device UA2423705 = {S88445, 130, { &P15767,&P18432,&P22322,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Rockmelt/2.0.3.516 Mobile/10A403 Safari/8536.2*/
const Device UA2372678 = {S88446, 81, { &P15364,&P17285,&P20443,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.4) Gecko/X Firefox/3.4.7*/
const Device UA2384124 = {S88447, 139, { &P14872,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-D700 Build/JRO03R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3043413 = {S88448, 61, { &P14872,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; SPH-D700 Build/JZO54K)*/
const Device UA2408918 = {S88449, 85, { &P15526,&P5448,&P4777,&P18092 }};	/*Useragent - HD_mini_T5555 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2413226 = {S88450, 141, { &P21228,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Lenovo A750 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2422037 = {S88451, 99, { &P13220,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonk770i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2841556 = {S88452, 99, { &P13220,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonk770i) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2453998 = {S88453, 128, { &P12765,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A100 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2842227 = {S88454, 128, { &P12765,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2456797 = {S88455, 157, { &P13054,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ADR6425LVW Build/IML74K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2482676 = {S88456, 63, { &P20513,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; A5; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2524135 = {S88457, 149, { &P21181,&P17979,&P18062,&P18092 }};	/*Useragent - Lenovo S560/S100 Linux/3.0.13 Android/4.0.3 Release/12.12.2011 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30*/
const Device UA2526146 = {S88458, 118, { &P15209,&P5768,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokia6233) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2529491 = {S88459, 143, { &P20493,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Flare (S100) Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555305 = {S88460, 133, { &P18464,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2571158 = {S88461, 62, { &P15364,&P5690,&P21711,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:20.0) Gecko/X Thunderbird/20.0*/
const Device UA2792148 = {S88462, 154, { &P13659,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P509 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580031 = {S88463, 58, { &P15364,&P22308,&P4618,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 3.0; WinNT3.51; Trident/5.0)*/
const Device UA2580710 = {S88464, 160, { &P18471,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; XT907 Build/9.8.1Q_27-2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2738060 = {S88465, 158, { &P18471,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-66) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2582034 = {S88466, 100, { &P20126,&P17485,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk800i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2593889 = {S88467, 142, { &P20475,&P17979,&P18062,&P18092 }};	/*Useragent - MALATA I8 Linux/3.0.13 Android/4.0.4 Release/07.24.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30*/
const Device UA2600009 = {S88468, 139, { &P19858,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KYL21 Build/018.0.3a90) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2601244 = {S88469, 138, { &P18316,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F160L Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2733216 = {S88470, 138, { &P18316,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-F160K Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2604124 = {S88471, 128, { &P17926,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AT100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2635582 = {S88472, 151, { &P15932,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; GT-P3110 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2780832 = {S88473, 151, { &P15932,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2658828 = {S88474, 136, { &P22994,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-E830 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2661578 = {S88475, 163, { &P14877,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2663898 = {S88476, 162, { &P11636,&P17485,&P24676,&P18092 }};	/*Useragent - SonyEricssonW705/R1GA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw705) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2663943 = {S88477, 154, { &P16503,&P5204,&P24676,&P18092 }};	/*Useragent - Nokia7310/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7310) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2665835 = {S88478, 95, { &P19779,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; Karbonn_A1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2666735 = {S88479, 137, { &P18283,&P24698,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I535 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2667126 = {S88480, 161, { &P14186,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-CN; GT-S5830 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2667127 = {S88481, 164, { &P13042,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC Incredible S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2667832 = {S88482, 135, { &P21261,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i9500 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2672020 = {S88483, 136, { &P18662,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ISW13HT Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677216 = {S88484, 135, { &P21330,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PMP5197DULTRA Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2694177 = {S88485, 101, { &P13241,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 3.4.0.59; en-us; 8830/3.4.0.59) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2983710 = {S88486, 101, { &P13241,&P5764,&P24666,&P18092 }};	/*Useragent - UCWEB/2.0(BlackBerry; U; 3.4.0.55; en-us; 8830/3.4.0.55) U2/1.0.0 UCBrowser/8.1.0.216 U2/1.0.0 Mobile*/
const Device UA2701511 = {S88487, 146, { &P23854,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; SKnetworksEG950 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2704515 = {S88488, 138, { &P21564,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ko-kr; KM-S330 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708349 = {S88489, 139, { &P15517,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT15i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708784 = {S88490, 137, { &P19910,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E617G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709048 = {S88491, 136, { &P23776,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGP321 Build/10.1.1.A.1.253) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2713872 = {S88492, 131, { &P23896,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; S5 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2749960 = {S88493, 73, { &P21475,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; Micromax_A35; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2750688 = {S88494, 88, { &P23366,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; W30) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2751991 = {S88495, 162, { &P18213,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6500D Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752076 = {S88496, 94, { &P23118,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua_Star) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2764843 = {S88497, 163, { &P15850,&P18066,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-LU6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2769586 = {S88498, 137, { &P22865,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AT-AS53N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2772876 = {S88499, 137, { &P17324,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9001 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774116 = {S88500, 135, { &P23165,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; H7500+ Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2778404 = {S88501, 156, { &P18281,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; LG-KU5400 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879584 = {S88502, 97, { &P20729,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A27) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2782640 = {S88503, 133, { &P21642,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NXM805ND_EN Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2782854 = {S88504, 137, { &P18722,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IM-A830L Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3049857 = {S88505, 134, { &P18722,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SKY-A830 Build/LTE) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783183 = {S88506, 142, { &P21885,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GSmart GS202+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784628 = {S88507, 134, { &P23981,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; 9300+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2795431 = {S88508, 154, { &P17475,&P5768,&P24677,&P18092 }};	/*Useragent - Nokia6300/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6300) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2796667 = {S88509, 155, { &P17625,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2996374 = {S88510, 155, { &P17625,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; ST25i Build/6.0.B.3.188) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2797498 = {S88511, 140, { &P20543,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A660 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798604 = {S88512, 137, { &P22853,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC 802w Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799249 = {S88513, 139, { &P18302,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT25i Build/4.1.B.0.631) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799684 = {S88514, 160, { &P18449,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST27i Build/6.1.1.B.1.10) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799792 = {S88515, 146, { &P23153,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ultrafone 701HD Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2800233 = {S88516, 165, { &P19891,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2878298 = {S88517, 130, { &P23462,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; TAB714 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2861686 = {S88518, 143, { &P20861,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Scroll Essential II Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3008999 = {S88519, 149, { &P15364,&P5690,&P22774,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; SlimBrowser/7.00.035)*/
const Device UA2893372 = {S88520, 158, { &P21973,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Aqua flash Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2867836 = {S88521, 149, { &P20841,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 10 Build/1.1.11-1015 X-10:30) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2873964 = {S88522, 152, { &P20841,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 10 3G Build/1.1.11-1015 X-16:17) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2868375 = {S88523, 134, { &P21104,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; X507S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2998424 = {S88524, 133, { &P21104,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; X507 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2878375 = {S88525, 74, { &P17595,&P21721,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:19.0) Gecko/X Firefox/19.0*/
const Device UA2878447 = {S88526, 65, { &P15364,&P18110,&P21777,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:19.0) Gecko/X K9Browser/19.0.0*/
const Device UA2879062 = {S88527, 156, { &P15880,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2880679 = {S88528, 157, { &P20134,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2892139 = {S88529, 89, { &P16598,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A500) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2892419 = {S88530, 147, { &P18595,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; GT-I8190 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3050912 = {S88531, 159, { &P15575,&P18111,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300*/
const Device UA2892669 = {S88532, 128, { &P23702,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ME173X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2892674 = {S88533, 151, { &P20096,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7568 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2937491 = {S88534, 128, { &P20187,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Novo7Fire Build/Xtm) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2944069 = {S88535, 167, { &P23122,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; Karbonn A6 Build/MocorDroid4.0.3) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2956898 = {S88536, 127, { &P18455,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC T528d) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958330 = {S88537, 170, { &P23201,&P18598,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; Karbonn S2 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33756*/
const Device UA3023257 = {S88538, 135, { &P21005,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; F-10D Build/V22R49C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021580 = {S88539, 140, { &P21526,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cherry w500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2987062 = {S88540, 96, { &P15364,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; Dell_Streak) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2990498 = {S88541, 88, { &P15364,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; A55) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3119078 = {S88542, 93, { &P15364,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; SGH-T839) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3018195 = {S88543, 159, { &P14743,&P17485,&P24678,&P18092 }};	/*Useragent - SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; fa; sonyericssonj20i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3008244 = {S88544, 114, { &P14714,&P5843,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02/07.16; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA3018574 = {S88545, 116, { &P14714,&P5843,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA3079581 = {S88546, 159, { &P18719,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C1505 Build/11.3.A.0.47) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3015516 = {S88547, 140, { &P12280,&P17146,&P24341,&P18092 }};	/*Useragent - MQQBrowser/44 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA3057616 = {S88548, 141, { &P12280,&P17146,&P24341,&P18092 }};	/*Useragent - MQQBrowser/44 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3*/
const Device UA3027073 = {S88549, 88, { &P20231,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A21) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3060464 = {S88550, 159, { &P20738,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Andromax U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3019614 = {S88551, 138, { &P24480,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262B Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3020977 = {S88552, 144, { &P20253,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; i-mobile IQ 5.1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023401 = {S88553, 145, { &P20253,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; i-mobile IQ 5.1A Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023348 = {S88554, 154, { &P14967,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; LG-P500 Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3060552 = {S88555, 159, { &P20183,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI-ONE Plus Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3036682 = {S88556, 71, { &P15364,&P16110,&P22171,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:23.0) Gecko/X Firefox/23.0 slurp*/
const Device UA3059014 = {S88557, 130, { &P15767,&P17146,&P20784,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.3 Mobile/9A405 Safari/7534.48.3*/
const Device UA3059055 = {S88558, 148, { &P20175,&P18066,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3125404 = {S88559, 149, { &P20175,&P18066,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1SC Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3063300 = {S88560, 135, { &P18600,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-03D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3112702 = {S88561, 133, { &P23146,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HYUNDAI T7S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094273 = {S88562, 130, { &P17168,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7310 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080168 = {S88563, 96, { &P20004,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Karbonn_A2+) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3109492 = {S88564, 152, { &P19775,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; A210 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3110040 = {S88565, 97, { &P22864,&P5629,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A26) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3113938 = {S88566, 106, { &P22864,&P5629,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; Micromax A26) U2/1.0.0 UCBrowser/9.3.0.321 Mobile*/
const Device UA3113939 = {S88567, 167, { &P12871,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3125661 = {S88568, 142, { &P12280,&P19826,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_2 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 UCBrowser/9.3.0.326 Mobile*/
const Device UA3126811 = {S88569, 142, { &P12280,&P19826,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_4 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B350 UCBrowser/9.3.0.326 Mobile*/
const Device UA3127031 = {S88570, 142, { &P12280,&P19826,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 UCBrowser/9.3.0.326 Mobile*/
const Device UA13067 = {S88571, 112, { &P13762,&P17483,&P3534,&P18092 }};	/*Useragent - LG/HB620T/v10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA24673 = {S88572, 112, { &P13762,&P17483,&P3534,&P18092 }};	/*Useragent - LG-HB620T/v10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA89862 = {S88573, 125, { &P13762,&P17483,&P3534,&P18092 }};	/*Useragent - Vodafone/1.0/LG-HB620T/V10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2262914 = {S88574, 125, { &P13762,&P17483,&P3534,&P18092 }};	/*Useragent - Vodafone/1.0/LG-HB620T/V10b Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA16692 = {S88575, 101, { &P13915,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E740/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA24485 = {S88576, 46, { &P15275,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-LG370/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA32819 = {S88577, 65, { &P12775,&P17483,&P17602,&P18092 }};	/*Useragent - LG-A225/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA43759 = {S88578, 68, { &P12545,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-T6/1.0 ACS-NF/3.4 QTV2.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA47015 = {S88579, 116, { &P15047,&P5629,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.3.3;Zh_cn;SK17i;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355*/
const Device UA2030320 = {S88580, 67, { &P15047,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.4;Zh_cn;SK17i;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA49318 = {S88581, 45, { &P12986,&P17017,&P17470,&P18092 }};	/*Useragent - Nexian NX-G507/MTK Release/10.01.2009 Browser*/
const Device UA299701 = {S88582, 31, { &P14803,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F03B(c500;TB;W24H16)*/
const Device UA67292 = {S88583, 31, { &P14797,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P03A(c100;TB;W24H15)*/
const Device UA90283 = {S88584, 31, { &P17721,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F02C(c500;TB;W24H16)*/
const Device UA90286 = {S88585, 32, { &P17763,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH02C(c500;TB;W24H16)*/
const Device UA196053 = {S88586, 138, { &P16757,&P17979,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android UKB 2.0; en-us; SCH-I500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA458642 = {S88587, 97, { &P18309,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; PI86100)*/
const Device UA460281 = {S88588, 100, { &P17595,&P17294,&P22481,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.4; en-US; rv:1.9.0.16) Gecko/X Firefox/3.0.16 Flock/2.5.6*/
const Device UA578482 = {S88589, 140, { &P15438,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Sensation Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA651772 = {S88590, 136, { &P15438,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Sensation Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA629120 = {S88591, 31, { &P18088,&P17017,&P17347,&P18092 }};	/*Useragent - j2me/UC Browser8.0.3.107/69/444*/
const Device UA693893 = {S88592, 31, { &P18088,&P17017,&P17347,&P18092 }};	/*Useragent - maui/UC Browser8.0.3.107/69/444*/
const Device UA708673 = {S88593, 31, { &P18088,&P17017,&P17347,&P18092 }};	/*Useragent - maui/UC Browser8.0.3.107/69/352*/
const Device UA724411 = {S88594, 31, { &P18088,&P17017,&P17347,&P18092 }};	/*Useragent - j2me/UC Browser8.0.3.107/69/351*/
const Device UA652945 = {S88595, 130, { &P15767,&P17146,&P20773,&P18092 }};	/*Useragent - MQQBrowser/2.2 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA837632 = {S88596, 157, { &P15364,&P7133,&P18729,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.0.1.27000 (Firestorm-Release; firestorm skin) Safari/533.3*/
const Device UA1955734 = {S88597, 86, { &P12351,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s3653w) UCBrowser8.2.0.132/69/352/UCWEB Mobile*/
const Device UA1966814 = {S88598, 86, { &P12351,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s3653w) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA1993361 = {S88599, 138, { &P14433,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9003 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2025703 = {S88600, 131, { &P21325,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; V11 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2817167 = {S88601, 136, { &P21325,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ThL V11 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031690 = {S88602, 135, { &P21006,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; L-05D Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2032157 = {S88603, 153, { &P14186,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Final 2.3.9; en-gb; GT-S5830 Build/GINGER->_<-BREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2655295 = {S88604, 159, { &P14186,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android WP8; en-us; GT-S5830 Build/GRWK74; WP8.0_by_maniaczek1111) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2351311 = {S88605, 84, { &P19976,&P5055,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; NokiaE6-00)/UC Browser8.5.0.183/51/355/UCWEB Mobile*/
const Device UA2234070 = {S88606, 135, { &P15364,&P17819,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.1 (KHTML, like Gecko) Ubuntu/11.10 Chromium/21.0.1180.89 Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2362901 = {S88607, 130, { &P18051,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; F-01D Build/V08R31A) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2776794 = {S88608, 170, { &P11747,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Desire Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2425861 = {S88609, 171, { &P20330,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; Micromax P350 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2442289 = {S88610, 76, { &P23062,&P22832,&P4780,&P18092 }};	/*Useragent - Videocon_V1570_Maui Wap Browser/1.0; Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2451477 = {S88611, 182, { &P14848,&P5843,&P24671,&P18092 }};	/*Useragent - NokiaX3-00/5.0 (11.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax3-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2453723 = {S88612, 99, { &P17595,&P17296,&P21808,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.2.16pre) Gecko/X Prism zdesktop/7.2.1*/
const Device UA2455512 = {S88613, 157, { &P14433,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2457581 = {S88614, 157, { &P16436,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2474861 = {S88615, 140, { &P21872,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; GOCLEVER TAB A73 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2760789 = {S88616, 140, { &P21872,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-us; GOCLEVER TAB A73 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2480718 = {S88617, 68, { &P16049,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; LG-P698; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2483264 = {S88618, 152, { &P23725,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; O+_8.5_Android Build/OplusOplus_A51) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2506923 = {S88619, 164, { &P14123,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P970 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2525508 = {S88620, 131, { &P20196,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID7048 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2553252 = {S88621, 69, { &P18679,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; GT-B5330; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2536787 = {S88622, 151, { &P18283,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2537480 = {S88623, 133, { &P15767,&P18432,&P22327,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Rockmelt/2.2.0.241 Mobile/10A525 Safari/8536.25*/
const Device UA2558124 = {S88624, 154, { &P19853,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7102 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2703971 = {S88625, 158, { &P15539,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2586660 = {S88626, 142, { &P21629,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Lenovo S920 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3017908 = {S88627, 146, { &P21629,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Lenovo S920_ROW Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2603902 = {S88628, 142, { &P20261,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Philips W832 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2604076 = {S88629, 131, { &P18731,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ONDA MID Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2634798 = {S88630, 131, { &P13452,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P1000 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2635230 = {S88631, 130, { &P13452,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P1000 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2672378 = {S88632, 140, { &P20540,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PAP4300 DUO Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707334 = {S88633, 139, { &P15079,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MK16i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2799044 = {S88634, 99, { &P13677,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5253) U2/1.0.0 UCBrowser/9.0.0.295 U2/1.0.0 Mobile*/
const Device UA2733368 = {S88635, 151, { &P18283,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2798685 = {S88636, 151, { &P20518,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A10 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752500 = {S88637, 161, { &P18297,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I8530 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2868545 = {S88638, 155, { &P21022,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LG-E615 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2768966 = {S88639, 157, { &P19779,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A1+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2769873 = {S88640, 143, { &P21071,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Novo 10 Hero QuadCore Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2783077 = {S88641, 138, { &P20737,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784766 = {S88642, 134, { &P17283,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; MB860 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784769 = {S88643, 134, { &P23383,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; H7100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2792277 = {S88644, 134, { &P23306,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; E11 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2795050 = {S88645, 143, { &P22989,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 4 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2797641 = {S88646, 144, { &P20253,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; i-mobile IQ 5.1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937456 = {S88647, 145, { &P20253,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; i-mobile IQ 5.1A Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799224 = {S88648, 158, { &P13141,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I727R Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2998332 = {S88649, 75, { &P15364,&P5690,&P23489,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Maxthon/4.0.6.2000)*/
const Device UA2856440 = {S88650, 146, { &P23250,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; NoVa Pad 70C124 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2866500 = {S88651, 95, { &P21911,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.1; en-US; AMAZE_S200) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2892218 = {S88652, 147, { &P18476,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I577 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984863 = {S88653, 139, { &P18476,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I577 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892660 = {S88654, 90, { &P22641,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; C1504) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2956620 = {S88655, 161, { &P20253,&P18111,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; i-mobile IQ 5.1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102708 = {S88656, 162, { &P20253,&P18111,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; i-mobile IQ 5.1A Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2964279 = {S88657, 159, { &P18223,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT22i Build/6.1.1.B.1.54) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3030820 = {S88658, 158, { &P18707,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MD_LIFETAB_P9516 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3022117 = {S88659, 130, { &P13452,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P1000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021432 = {S88660, 136, { &P22809,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; P5_Quad Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2986536 = {S88661, 95, { &P15364,&P5477,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; U20a) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2989587 = {S88662, 96, { &P15364,&P5477,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; XT800) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3045213 = {S88663, 99, { &P15364,&P5477,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT-I5800) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2991129 = {S88664, 131, { &P15767,&P19826,&P20777,&P18092 }};	/*Useragent - MQQBrowser/2.6 Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3*/
const Device UA3071541 = {S88665, 131, { &P15767,&P19826,&P20777,&P18092 }};	/*Useragent - MQQBrowser/2.6 Mozilla/5.0 (iPad; CPU OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3*/
const Device UA3100046 = {S88666, 129, { &P15767,&P19826,&P20777,&P18092 }};	/*Useragent - MQQBrowser/2.6 Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B141 Safari/7534.48.3*/
const Device UA3007877 = {S88667, 155, { &P18283,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9300 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3012199 = {S88668, 133, { &P19861,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; SCH-I605 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3015591 = {S88669, 153, { &P19904,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C5170 Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3062124 = {S88670, 159, { &P18302,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT25i Build/4.1.B.0.631) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3079658 = {S88671, 152, { &P20529,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A89 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3022618 = {S88672, 136, { &P24382,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E986 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3032715 = {S88673, 74, { &P18141,&P24346,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 2.60) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA3065416 = {S88674, 156, { &P18509,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LG-MS770 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3110454 = {S88675, 131, { &P20145,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; NS-13T001 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3095261 = {S88676, 141, { &P20261,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Philips W832 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083186 = {S88677, 139, { &P20927,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT-AS45IPS Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3062268 = {S88678, 98, { &P19862,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax_A110) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3094616 = {S88679, 168, { &P13157,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Sensation XE with Beats Audio Z715e Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3113423 = {S88680, 133, { &P15767,&P18117,&P24411,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Coast/1.0.1.62563 Mobile/9B206 Safari/7534.48.3*/
const Device UA3116560 = {S88681, 131, { &P15767,&P18117,&P24411,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Coast/1.0.1.62563 Mobile/9B176 Safari/7534.48.3*/
const Device UA3077792 = {S88682, 93, { &P15060,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5660) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3079791 = {S88683, 159, { &P23199,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Coolpad7295 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3125774 = {S88684, 161, { &P23199,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Coolpad 7295+ Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3093988 = {S88685, 139, { &P12616,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WT19i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3079348 = {S88686, 93, { &P22382,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8262) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3110306 = {S88687, 134, { &P17283,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MB860 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093677 = {S88688, 141, { &P21738,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; iris502 Build/LAVAIRIS502) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA7333 = {S88689, 107, { &P13307,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-ZV40-Vodafone/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA25505 = {S88690, 87, { &P12130,&P17485,&P3600,&P18092 }};	/*Useragent - SonyEricssonK608i/R2BA Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA43915 = {S88691, 87, { &P11802,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KG195/V01 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1 LG-KG195/V01 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA18512 = {S88692, 79, { &P14184,&P17484,&P4924,&P18092 }};	/*Useragent - SAMSUNG-SGH-T539/T539UVHA1 Profile/MIDP-2.0 Configuration/CLDC-1.1 NetFront/3.2*/
const Device UA55125 = {S88693, 111, { &P14286,&P17484,&P3430,&P18092 }};	/*Useragent - SAMSUNG-S8000/S8000BOIK6 SHP/VPP/R5 Opera/9.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA26801 = {S88694, 41, { &P13638,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-A870 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA162055 = {S88695, 31, { &P14646,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F02B(c500;TB;W24H16)*/
const Device UA56162 = {S88696, 32, { &P14655,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N06A3(c500;TB;W24H16)*/
const Device UA68356 = {S88697, 80, { &P13750,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SCH-W259/W259DDDF2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA87069 = {S88698, 47, { &P17503,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-CA39 UP.Browser/6.2.0.13.1.5 (GUI) MMP/2.0*/
const Device UA88376 = {S88699, 70, { &P12499,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Treo700wx*/
const Device UA89914 = {S88700, 31, { &P17759,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N05B(c500;TB;W24H16)*/
const Device UA90284 = {S88701, 31, { &P17722,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F03C(c500;TB;W24H16)*/
const Device UA173015 = {S88702, 14, { &P12142,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonU5*/
const Device UA359525 = {S88703, 140, { &P16005,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; ZTE Crescent Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA440662 = {S88704, 130, { &P15767,&P17146,&P20774,&P18092 }};	/*Useragent - MQQBrowser/2.3 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA809469 = {S88705, 80, { &P16595,&P5477,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.1-update1;Zh_cn;Micromax A60;240*320;)UCWEB7.8.0.95/140/444*/
const Device UA572451 = {S88706, 35, { &P15518,&P17484,&P17338,&P18092 }};	/*Useragent - gt-s5230/UC Browser7.9.0.102/69/352*/
const Device UA574366 = {S88707, 48, { &P12193,&P5890,&P4777,&P18092 }};	/*Useragent - nokiac5-00.2/UC Browser7.7.1.88/69/444  3gpp-gba*/
const Device UA1886143 = {S88708, 36, { &P12193,&P5890,&P4777,&P18092 }};	/*Useragent - nokiac5-00/UC Browser7.7.1.88/69/444*/
const Device UA575301 = {S88709, 44, { &P17595,&P17297,&P18379,&P18092 }};	/*Useragent - MacOutlook/14.10.0.X (Intel Mac OS X 10.7.2)*/
const Device UA2324979 = {S88710, 44, { &P17595,&P17297,&P18379,&P18092 }};	/*Useragent - MacOutlook/14.10.0.X (Intel Mac OS X 10.7.4)*/
const Device UA2827749 = {S88711, 44, { &P17595,&P17297,&P18379,&P18092 }};	/*Useragent - MacOutlook/14.10.0.X (Intel Mac OS X 10.7.5)*/
const Device UA589939 = {S88712, 78, { &P15364,&P7127,&P20460,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:10.0.1) Gecko/X Firefox/10.0.1 SeaMonkey/2.7.1*/
const Device UA2164130 = {S88713, 85, { &P12142,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU5)/UC Browser8.2.0.132/50/444/UCWEB Mobile*/
const Device UA634311 = {S88714, 155, { &P12972,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-T989 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1964514 = {S88715, 155, { &P12972,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-T989 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023550 = {S88716, 158, { &P12972,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-T989 Build/GRI40; CyanogenMod-DARKSIDE) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA698031 = {S88717, 64, { &P17669,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;T7575;7.10;7720)*/
const Device UA723653 = {S88718, 97, { &P15364,&P7127,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:9.0.1) Gecko/X Firefox/9.0.1,gzip(gfe) (via translate.google.com)*/
const Device UA745272 = {S88719, 136, { &P21430,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; pt-br; ZTE V860 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA819202 = {S88720, 80, { &P14359,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5228)/UC Browser8.2.0.132/50/355/UCWEB Mobile*/
const Device UA840851 = {S88721, 135, { &P17937,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; WX435 Build/GWK74) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA843626 = {S88722, 136, { &P17937,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; WX435 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1992595 = {S88723, 135, { &P17937,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; WX435 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2421518 = {S88724, 48, { &P15364,&P18575,&P22243,&P18092 }};	/*Useragent - Flipboard/1.9.8 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA919129 = {S88725, 95, { &P15364,&P7133,&P20458,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:8.0.1) Gecko/X Firefox/8.0.1 SeaMonkey/2.5 Lightning/1.0*/
const Device UA1895662 = {S88726, 126, { &P23613,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40EX520; CC/XGA; en) Presto/2.7.61 Version/11.00*/
const Device UA2545105 = {S88727, 77, { &P15364,&P17188,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; WinNT4.0; en-US; rv:1.9.2.28) Gecko/X Firefox/3.6.28*/
const Device UA1991386 = {S88728, 81, { &P15845,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaC5-06)/UC Browser8.2.0.132/50/355/UCWEB Mobile*/
const Device UA1999901 = {S88729, 40, { &P17968,&P17979,&P24437,&P18092 }};	/*Useragent - Dolphin http client/8.7.0(164) (Android)*/
const Device UA2165180 = {S88730, 80, { &P11880,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA5320)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2219680 = {S88731, 118, { &P15364,&P17188,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 4.0; zh-CN) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19.1*/
const Device UA2219984 = {S88732, 64, { &P11684,&P5890,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8 (Nokia5800XpressMusic;SymbianOS/9.1 Series60/3.0)*/
const Device UA2350512 = {S88733, 165, { &P18687,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; LG-P705 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2351508 = {S88734, 224, { &P17346,&P18432,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA2466438 = {S88735, 226, { &P17346,&P18432,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA2378547 = {S88736, 133, { &P22817,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; DEM752HCF Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2378634 = {S88737, 138, { &P12945,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A780L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2401594 = {S88738, 64, { &P15364,&P17188,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 4.0; U; en) Presto/2.10.289 Version/12.02*/
const Device UA2414569 = {S88739, 154, { &P15575,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2479306 = {S88740, 154, { &P15575,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2481849 = {S88741, 154, { &P18283,&P18111,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2658054 = {S88742, 154, { &P18283,&P18111,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2554560 = {S88743, 143, { &P22665,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; maximus max901 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2583121 = {S88744, 126, { &P16598,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; A500 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2780752 = {S88745, 166, { &P18645,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2623613 = {S88746, 174, { &P15919,&P18111,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3100 Build/JRO03C; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444*/
const Device UA2889538 = {S88747, 69, { &P15919,&P18111,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.1.2; zh-cn; GT-P3100; 600*976) UCWEB7.9.0.94/139/444*/
const Device UA2642448 = {S88748, 152, { &P12987,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M930BST Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2879322 = {S88749, 153, { &P12987,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M930BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2655972 = {S88750, 31, { &P23578,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N01E(c500;TB;W24H16)*/
const Device UA2701361 = {S88751, 96, { &P13746,&P5477,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT540) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2701446 = {S88752, 143, { &P22683,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Beeline E600 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2705505 = {S88753, 153, { &P15877,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT910 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892780 = {S88754, 156, { &P15877,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT910 Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708379 = {S88755, 129, { &P21990,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; chagall Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708783 = {S88756, 131, { &P16168,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; NookColor Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2731325 = {S88757, 133, { &P23962,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; E6 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2737533 = {S88758, 158, { &P15047,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; SK17i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2879191 = {S88759, 151, { &P21057,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A7+ Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2769032 = {S88760, 134, { &P21815,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; P-02E Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773821 = {S88761, 127, { &P21222,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT1S0 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2773920 = {S88762, 130, { &P22662,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Enjoy 71 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2781841 = {S88763, 140, { &P23126,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CRCB 2093 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2782583 = {S88764, 133, { &P21660,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Novo7 Venus Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2783695 = {S88765, 164, { &P17308,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.4; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2792983 = {S88766, 141, { &P23014,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Allview_AX1_Shine Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2794742 = {S88767, 106, { &P14748,&P5890,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaE72-1*/
const Device UA2795845 = {S88768, 168, { &P22980,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; TECNO D5 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/33567*/
const Device UA2795964 = {S88769, 147, { &P23103,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Starmobile DIAMOND Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3014803 = {S88770, 150, { &P23103,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Starmobile DIAMOND V3 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2796753 = {S88771, 96, { &P22209,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Titanium S5) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2799468 = {S88772, 133, { &P21933,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M729 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2800509 = {S88773, 139, { &P22656,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SOL22 Build/10.2.F.3.43) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868542 = {S88774, 157, { &P14959,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9103 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2857818 = {S88775, 178, { &P19747,&P5043,&P24671,&P18092 }};	/*Useragent - Nokia206/2.0 (03.59) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia206) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2864952 = {S88776, 158, { &P15539,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; HTC Glacier Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2865940 = {S88777, 55, { &P17968,&P18598,&P23216,&P18092 }};	/*Useragent - Mozilla/5.0 (Android 4.2; rv:19.0) Gecko/X Firefox/19.0*/
const Device UA2868298 = {S88778, 137, { &P22872,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5282 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869925 = {S88779, 165, { &P16263,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI MediaPad Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2893082 = {S88780, 165, { &P16263,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MediaPad 7 Lite Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2878930 = {S88781, 67, { &P23527,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-T55 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2892943 = {S88782, 168, { &P16362,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-D710 Build/JDQ39E; LiquidSmooth-Liquid-JB-v2.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937212 = {S88783, 152, { &P24539,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2943868 = {S88784, 93, { &P22609,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6312) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2957471 = {S88785, 154, { &P23199,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad7295 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2999380 = {S88786, 156, { &P23199,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 7295+ Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2957657 = {S88787, 154, { &P23015,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Lenovo A820 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2998496 = {S88788, 155, { &P23015,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Lenovo A820t Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2958864 = {S88789, 138, { &P24528,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E987 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958979 = {S88790, 156, { &P22941,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6812 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2963280 = {S88791, 128, { &P19996,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; A15 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2964352 = {S88792, 95, { &P14202,&P5477,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E15i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3009735 = {S88793, 144, { &P24628,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; LG-LG-Optimus Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3077917 = {S88794, 146, { &P24628,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; LG-Optimus/mus Build/VDT-Rom) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3021662 = {S88795, 152, { &P23151,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAP4044DUO Build/PrestigioPAP4044DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3058909 = {S88796, 150, { &P15985,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3017072 = {S88797, 147, { &P15364,&P18598,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; R821T Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA3108269 = {S88798, 148, { &P15364,&P18598,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; R823T Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA3020166 = {S88799, 93, { &P14417,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5236) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3023212 = {S88800, 133, { &P24559,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Z992 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3064150 = {S88801, 155, { &P22594,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SGH-M919 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3027376 = {S88802, 135, { &P19861,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SCH-I605 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3065015 = {S88803, 160, { &P13815,&P5096,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; HTC Wildfire Build/FRG83D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3057821 = {S88804, 130, { &P21117,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HS-9DTB4 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3058769 = {S88805, 196, { &P21261,&P18598,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0530, in*/
const Device UA3059096 = {S88806, 141, { &P12280,&P17146,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 UCBrowser/9.2.0.312 Mobile*/
const Device UA3059592 = {S88807, 141, { &P12280,&P17146,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 UCBrowser/9.2.0.312 Mobile*/
const Device UA3059928 = {S88808, 97, { &P22903,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A62) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3114080 = {S88809, 135, { &P23130,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; H2000+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3090616 = {S88810, 176, { &P15803,&P5843,&P22881,&P18092 }};	/*Useragent - Nokia200/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291*/
const Device UA3093739 = {S88811, 160, { &P18382,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire C Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3093966 = {S88812, 152, { &P24073,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033M Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094069 = {S88813, 137, { &P18686,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A810S Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109236 = {S88814, 137, { &P18686,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A810K Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3108049 = {S88815, 147, { &P15364,&P18598,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; R821T Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3108479 = {S88816, 166, { &P15364,&P18598,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; HUAWEI G610-U00 Build/HuaweiG610-U00) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3125823 = {S88817, 162, { &P15364,&P18598,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HUAWEI P6-T00 Build/HuaweiP6-T00) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA9537 = {S88818, 99, { &P13421,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Vodafone/1.0/HTC_TouchDual/1.31.172.16*/
const Device UA22093 = {S88819, 74, { &P13421,&P5448,&P22271,&P18092 }};	/*Useragent - HTC_TouchDual Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA86894 = {S88820, 99, { &P13421,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Vodafone/1.0/HTC_TouchDual/1.31.163.09*/
const Device UA26363 = {S88821, 87, { &P13587,&P17485,&P3600,&P18092 }};	/*Useragent - SonyEricssonK750c/R1CA Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA43622 = {S88822, 77, { &P13711,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW205a/R1CA001 Browser/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62028 = {S88823, 92, { &P14360,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-Z140/2.0 SHP/VPP/R5 SMB3.1 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA22207 = {S88824, 66, { &P15400,&P17017,&P4528,&P18092 }};	/*Useragent - Nexian NX-G508/MTK Release/10.01.2009 Browser/MAUI profile/No Java*/
const Device UA23726 = {S88825, 61, { &P14463,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonT610/R201 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA25991 = {S88826, 100, { &P12147,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-A736/UXHB1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA53647 = {S88827, 37, { &P13642,&P17483,&P17470,&P18092 }};	/*Useragent - LG-KG800 MIC/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA54794 = {S88828, 29, { &P14263,&P17017,&P4924,&P18092 }};	/*Useragent - C5589/001.00 ACS-NetFront/3.2*/
const Device UA55045 = {S88829, 75, { &P14277,&P17484,&P4628,&P18092 }};	/*Useragent - Samsung-SPHM630 Polaris/6.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64175 = {S88830, 153, { &P14578,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/SGH-L870/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA72571 = {S88831, 158, { &P14578,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 SymbianOS/9.3; U; Series60/3.2 Samsung/SGH-L870-ORANGE/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA64434 = {S88832, 92, { &P14667,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-X930-T-Mobile/X930_V1_Z1_EN_B18F103 Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA68695 = {S88833, 36, { &P12193,&P5129,&P4777,&P18092 }};	/*Useragent - NokiaC5-00/UC Browser7.7.1.88/28/444*/
const Device UA773419 = {S88834, 38, { &P12193,&P5129,&P4777,&P18092 }};	/*Useragent - NokiaC5-00.2/UC Browser7.7.1.88/28/444*/
const Device UA379836 = {S88835, 46, { &P14519,&P5129,&P24663,&P18092 }};	/*Useragent - NokiaN97 mini/UC Browser7.9.0.102/50/355/UCWEB*/
const Device UA2144355 = {S88836, 34, { &P14519,&P5129,&P24663,&P18092 }};	/*Useragent - NokiaN97 mini/UCWEB7.9.0.94/50/800*/
const Device UA455943 = {S88837, 85, { &P18141,&P17095,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 1.66) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2*/
const Device UA475007 = {S88838, 97, { &P15364,&P5690,&P22333,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; ru; rv:1.9.0.11) Gecko/X Firefox/3.0.11 CometBird/3.0.11*/
const Device UA2812219 = {S88839, 74, { &P15364,&P16110,&P18191,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:10.0) Gecko/X Thunderbird/10.0*/
const Device UA647608 = {S88840, 64, { &P15364,&P5690,&P18101,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:12.0) Gecko/X Firefox/12.0 slurp*/
const Device UA802957 = {S88841, 70, { &P15364,&P5690,&P18101,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:12.0) Gecko/X Firefox/12.0 slurp slurp*/
const Device UA649707 = {S88842, 88, { &P17976,&P5096,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2.1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.7.81 Version/11.00*/
const Device UA2018614 = {S88843, 86, { &P17976,&P5096,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.2; Linux; Opera Tablet/ADR-X; U; de) Presto/2.7.81 Version/11.00*/
const Device UA649856 = {S88844, 146, { &P21588,&P5629,&P20776,&P18092 }};	/*Useragent - MQQBrowser/25 Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; 5860 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA718782 = {S88845, 142, { &P21678,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; th-th; i-mobile i691 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA812886 = {S88846, 95, { &P15364,&P17017,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0(GM_UserLogon)AppleWebKit/535.1 (KHTML, like Gecko) Chrome/16.0.912.758 Safari/535.1*/
const Device UA833654 = {S88847, 64, { &P15364,&P17285,&P18100,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:11.0) Gecko/X Firefox/11.0 slurp*/
const Device UA1985827 = {S88848, 71, { &P15364,&P17285,&P18100,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:11.0) Gecko/X Firefox/11.0 slurp*/
const Device UA1885244 = {S88849, 83, { &P15364,&P18346,&P17202,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows; U; Windows NT 4.1; fr-FR; rv:1.8.1.3) Gecko/X Firefox/2.0.0.3*/
const Device UA1959505 = {S88850, 136, { &P22101,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; Tablet-9728B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026621 = {S88851, 171, { &P14433,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2028853 = {S88852, 171, { &P14433,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2026531 = {S88853, 137, { &P21552,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGT SMA2+ Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029512 = {S88854, 149, { &P21114,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; De-Sensed HTC Vivid Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2034550 = {S88855, 143, { &P24008,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; vi-vn; HKPhone revo Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2200545 = {S88856, 131, { &P15767,&P18432,&P20776,&P18092 }};	/*Useragent - MQQBrowser/2.5.1 Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3*/
const Device UA2840546 = {S88857, 133, { &P15767,&P18432,&P20776,&P18092 }};	/*Useragent - MQQBrowser/2.5.1 Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3*/
const Device UA2221934 = {S88858, 75, { &P14545,&P17483,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; kau; lg-x330t) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2686693 = {S88859, 77, { &P14545,&P17483,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; lg-x330t) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2350384 = {S88860, 145, { &P20980,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A97i Build/CelkonA97i_Celkon) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2352190 = {S88861, 162, { &P12765,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2384337 = {S88862, 149, { &P13452,&P5096,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2410314 = {S88863, 136, { &P23864,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; DATAM819HD_C Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2820219 = {S88864, 136, { &P23864,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; DATAM819HD_A Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2412599 = {S88865, 129, { &P20812,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT330 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2424767 = {S88866, 40, { &P12056,&P5204,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia7210Supernova/07.2)*/
const Device UA2445267 = {S88867, 63, { &P19946,&P17483,&P3459,&P18092 }};	/*Useragent - LG-A340/V10d Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2448296 = {S88868, 139, { &P19979,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E170K Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2455945 = {S88869, 144, { &P22065,&P18111,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 4.1; en-us; SmartTab2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2463946 = {S88870, 89, { &P22800,&P22840,&P4780,&P18092 }};	/*Useragent - MyPhoneQTW88TVDUO/MTKRelease/2012/04/17Browser/MAUIProfile/MIDP-2.0Configuration/CLDC-1.0*/
const Device UA2464936 = {S88871, 185, { &P12258,&P17483,&P24671,&P18092 }};	/*Useragent - LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-gs290) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2469030 = {S88872, 138, { &P21132,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Andi4.5h Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2481035 = {S88873, 161, { &P18666,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A25 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2482856 = {S88874, 131, { &P20351,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; 101 CXi Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2482882 = {S88875, 139, { &P23559,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AMOI N821 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2487067 = {S88876, 167, { &P14557,&P5843,&P24675,&P18092 }};	/*Useragent - Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia6303classic) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2502371 = {S88877, 133, { &P21137,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HS-10DTB2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2504750 = {S88878, 127, { &P21323,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V9S Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2507889 = {S88879, 139, { &P20244,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Elcano Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2527377 = {S88880, 133, { &P23470,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TM-MID790 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555016 = {S88881, 149, { &P21494,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cloudfone Thrill 500g Build/FSV01) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2558643 = {S88882, 162, { &P18721,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A90S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2559835 = {S88883, 157, { &P12788,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PG86100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2563595 = {S88884, 90, { &P12856,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t300) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2778894 = {S88885, 87, { &P12856,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; vi; lg-t300) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2661554 = {S88886, 99, { &P15198,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; Pt-BR; samsung-gt-s3650) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2601076 = {S88887, 133, { &P20807,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B92 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2605960 = {S88888, 129, { &P20844,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB462 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2610878 = {S88889, 84, { &P15919,&P17979,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.4.5) (espressorf;samsung;GT-P3100;4.0.4/IMM76D)*/
const Device UA2633102 = {S88890, 192, { &P15876,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0800, my*/
const Device UA2634359 = {S88891, 188, { &P15876,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-P5100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0100*/
const Device UA2637734 = {S88892, 151, { &P15876,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2637834 = {S88893, 151, { &P15876,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2738443 = {S88894, 130, { &P13452,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P1000 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2654786 = {S88895, 159, { &P20253,&P18066,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile IQ 5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2667065 = {S88896, 141, { &P15805,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Holiday Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667311 = {S88897, 146, { &P15805,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Raider X710e Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667532 = {S88898, 159, { &P14969,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HTC Desire S Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2667575 = {S88899, 159, { &P14969,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; HTC Desire S Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2702965 = {S88900, 139, { &P22735,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Primo G2 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2706667 = {S88901, 136, { &P20240,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E971 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2738248 = {S88902, 137, { &P20947,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Evo V 4G Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708673 = {S88903, 130, { &P21202,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-P500 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2710203 = {S88904, 140, { &P23015,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Lenovo A820 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2710642 = {S88905, 149, { &P20137,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Xtreamer_Mobile_AiKi Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3108322 = {S88906, 151, { &P20137,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xtreamer_Mobile_Aiki5s Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712368 = {S88907, 135, { &P18470,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 101G10 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2718524 = {S88908, 107, { &P18254,&P7133,&P19839,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 slurp*/
const Device UA2718963 = {S88909, 138, { &P22935,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9080L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2729806 = {S88910, 90, { &P14255,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; Ideos) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2731064 = {S88911, 175, { &P18672,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5302 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2749884 = {S88912, 140, { &P21068,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI-ONE C1 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752654 = {S88913, 161, { &P20332,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax P275 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2765169 = {S88914, 136, { &P23244,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-05E Build/S1160) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2861812 = {S88915, 136, { &P23244,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; SH-05E Build/S3270) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768860 = {S88916, 158, { &P19862,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-in; Micromax A110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2779399 = {S88917, 193, { &P13157,&P18066,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2779696 = {S88918, 131, { &P18702,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N8013 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2779809 = {S88919, 143, { &P21738,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; iris502 Build/LAVAIRIS502) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2780972 = {S88920, 137, { &P18240,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P700 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2781171 = {S88921, 143, { &P23580,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; PHICOMM i800 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2782552 = {S88922, 144, { &P18554,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2784074 = {S88923, 146, { &P12765,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A100 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2790537 = {S88924, 156, { &P18466,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (KFTT, 35, 1.7.0)*/
const Device UA3043820 = {S88925, 163, { &P18466,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; KFTT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (KFTT, 35, 1.7.0)*/
const Device UA2791818 = {S88926, 58, { &P15364,&P22308,&P17193,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; WinNT3.51; Trident/1.0)*/
const Device UA2859550 = {S88927, 101, { &P21545,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; HUAWEI_Y210-0200) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2994031 = {S88928, 101, { &P21545,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; HUAWEI_Y210-0100) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2866509 = {S88929, 159, { &P21980,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; UBISLATE7C+ Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2868625 = {S88930, 157, { &P21506,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2892834 = {S88931, 159, { &P21881,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Karbonn A4  Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2937910 = {S88932, 151, { &P22875,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9220 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2952033 = {S88933, 111, { &P23283,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; Huawei; H883G; HuaweiH883G)*/
const Device UA2958013 = {S88934, 146, { &P23449,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Dialog i43(ICS) Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2961057 = {S88935, 138, { &P18321,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2997580 = {S88936, 136, { &P22956,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P875 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3014618 = {S88937, 165, { &P19891,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3024015 = {S88938, 139, { &P20738,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Andromax U Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022261 = {S88939, 93, { &P14186,&P5096,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5830) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3080670 = {S88940, 156, { &P22396,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5312 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3030342 = {S88941, 94, { &P18296,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC One V) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3083246 = {S88942, 94, { &P18296,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC One V) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3065348 = {S88943, 151, { &P20518,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A10 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3060685 = {S88944, 178, { &P14669,&P5843,&P24055,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311*/
const Device UA3095529 = {S88945, 130, { &P18437,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N8000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3093685 = {S88946, 138, { &P23381,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A5Classic Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094188 = {S88947, 146, { &P21802,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GSmart Maya M1 v2 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3064442 = {S88948, 98, { &P19862,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Micromax_A110) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3079227 = {S88949, 142, { &P22997,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Celkon A107+ Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3116525 = {S88950, 136, { &P22854,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9508 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110311 = {S88951, 93, { &P15649,&P5629,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3110706 = {S88952, 94, { &P15649,&P5629,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5570I) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA1854 = {S88953, 96, { &P12138,&P5026,&P4948,&P18092 }};	/*Useragent - BlackBerry8130/4.5.0.169 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/126 UP.Browser/5.0.3.3*/
const Device UA3925 = {S88954, 140, { &P12439,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; E120 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA27586 = {S88955, 108, { &P12182,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-T219S/T219UVGJ1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA12950 = {S88956, 72, { &P17559,&P5208,&P3499,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Teleca Q7; Brew 3.1.5; U; en) 240X400 LGE AX840*/
const Device UA13678 = {S88957, 125, { &P11763,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-K3/99.31.04R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12153168) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA13681 = {S88958, 127, { &P11763,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-K3m/99.51.05AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12173212) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA13679 = {S88959, 125, { &P11763,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-K3/99.13.11R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12133142) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA66723 = {S88960, 125, { &P11763,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-K3/99.13.14R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12143162) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA66724 = {S88961, 125, { &P11763,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-K3/99.41.08R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163225) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA66730 = {S88962, 125, { &P11763,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-K3/99.41.03R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163189) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA70133 = {S88963, 127, { &P11763,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-K3m/99.41.04AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163189) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA443224 = {S88964, 118, { &P11763,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-K3/99.13.11R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; X) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA16698 = {S88965, 124, { &P13918,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia6303ci/5.0 (07.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA21027 = {S88966, 116, { &P15152,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) LG-C570 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA64146 = {S88967, 105, { &P12051,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S5600/1.0 SHP/VPP/R5 NetFront/3.5 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA24568 = {S88968, 44, { &P14465,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-X990/X990_V2_Z12_ESFR_D18F100 Obigo/Q03C*/
const Device UA54301 = {S88969, 82, { &P14465,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-X990/X990_V2_Z5_FR_D13F104  Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C*/
const Device UA62970 = {S88970, 82, { &P14465,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-X990/X990_V1_Z2_ID_BFFC101  Profile/MIDP-1.0 Configuration/CLDC-1.0 Obigo/Q03C*/
const Device UA50299 = {S88971, 50, { &P13221,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-ZN4/00.72 UP.Browser/7.2.7.5.597 (GUI) MMP/2.0*/
const Device UA87751 = {S88972, 50, { &P13221,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-ZN4/00.72 UP.Browser/7.2.7.5.540 (GUI) MMP/2.0*/
const Device UA53538 = {S88973, 134, { &P14146,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; ZTE-RACER Build/chun) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2474420 = {S88974, 137, { &P14146,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; el-gr; ZTE-RACER Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA53861 = {S88975, 47, { &P14049,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-VX8370/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA56156 = {S88976, 32, { &P17758,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P08A3(c500;TB;W24H15)*/
const Device UA64839 = {S88977, 105, { &P14709,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-GT-M8800/1.0 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA318151 = {S88978, 54, { &P17634,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SH3G UP.Browser/6.2_7.2.7.1.K.5.207 (GUI) MMP/2.0*/
const Device UA351451 = {S88979, 77, { &P18254,&P7480,&P17470,&P19831 }};	/*Useragent - Mozilla/5.0 (compatible; EtaoSpider/1.0; http://open.etao.com/dev/EtaoSpider)*/
const Device UA2040267 = {S88980, 9, { &P18254,&P7480,&P17470,&P19831 }};	/*Useragent - 360Spider*/
const Device UA411991 = {S88981, 151, { &P18114,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Transformer Prime TF201 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA387738 = {S88982, 184, { &P15364,&P7133,&P24643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UCWEB7.2.2.54/139/352*/
const Device UA422890 = {S88983, 130, { &P15364,&P17285,&P22423,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.4.1.0 Chrome/17.0.963.56 Safari/535.11*/
const Device UA437545 = {S88984, 123, { &P15364,&P17285,&P22423,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.4.1.0 Chrome/17.0.963.56 Safari/535.11*/
const Device UA498996 = {S88985, 90, { &P15364,&P18110,&P17381,&P18091 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.0.7) Gecko/X Firefox/1.5.0.7 SnapPreviewBot*/
const Device UA550619 = {S88986, 74, { &P15364,&P18110,&P18373,&P18092 }};	/*Useragent - Opera/9.80 (Linux i686; Opera Mobi/7457; U; en) Presto/2.5.28 Version/10.1*/
const Device UA599341 = {S88987, 91, { &P17595,&P18120,&P18200,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; fr-FR; rv:1.7.10) Gecko/X Thunderbird/1.0.6*/
const Device UA800593 = {S88988, 69, { &P15364,&P17285,&P22354,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:9.2) Gecko/X Firefox/9.2 PaleMoon/9.2*/
const Device UA702988 = {S88989, 140, { &P21424,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; bq Voltaire Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA744306 = {S88990, 137, { &P20365,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-gb; ViewPad 10e Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA791987 = {S88991, 44, { &P16384,&P17484,&P17338,&P18092 }};	/*Useragent - samsung-gt-s5380k/UC Browser7.9.0.102/69/352*/
const Device UA924288 = {S88992, 88, { &P15364,&P7133,&P22551,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:13.0) Gecko/X Firefox/13.0.3.0 IceDragon/13.0.3.0*/
const Device UA1961252 = {S88993, 131, { &P18708,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U30GT-H Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2027745 = {S88994, 134, { &P21752,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M030 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2028684 = {S88995, 135, { &P21007,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; P-06D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2034090 = {S88996, 135, { &P21967,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; el-gr; Turbo-X Ice Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2519578 = {S88997, 67, { &P18252,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.7; zh-cn; XT615; 480*854) UCWEB7.9.4.145/139/999*/
const Device UA2320257 = {S88998, 75, { &P14545,&P17483,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; kau; lg-x330t) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2812491 = {S88999, 77, { &P14545,&P17483,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; lg-x330t) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2353097 = {S89000, 118, { &P15206,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE90) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile*/
const Device UA2360564 = {S89001, 158, { &P20134,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2374278 = {S89002, 129, { &P23601,&P18110,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-46EX640; CC/USA; en) Presto/2.8.115 Version/11.10*/
const Device UA2379086 = {S89003, 136, { &P23336,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; I_7520 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2399137 = {S89004, 194, { &P14557,&P5843,&P24671,&P18092 }};	/*Useragent - Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia6303classic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2410298 = {S89005, 61, { &P20048,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX283/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA2412677 = {S89006, 129, { &P15364,&P5690,&P18354,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152) Sleipnir/2.5.13*/
const Device UA2472467 = {S89007, 174, { &P15364,&P5690,&P18354,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729) Sleipnir/2.5.12*/
const Device UA2418132 = {S89008, 126, { &P21233,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; T1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2455622 = {S89009, 170, { &P21033,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; U1 Build/HCL ME Tablet U1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2504765 = {S89010, 120, { &P13474,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAX5-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2469211 = {S89011, 91, { &P16003,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; gt-s5233a) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2469291 = {S89012, 91, { &P16003,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; gt-s5233a) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2480866 = {S89013, 146, { &P20513,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3002215 = {S89014, 161, { &P20513,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; id-id; CROSS A5 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2483295 = {S89015, 132, { &P13278,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; X10.mini Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2486038 = {S89016, 143, { &P11747,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; HTC Desire Build/VJ CM10.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2554039 = {S89017, 112, { &P18223,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; LT22i-o) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2861937 = {S89018, 138, { &P19870,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-L710 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2576872 = {S89019, 159, { &P15850,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; LG-lu6200 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2688063 = {S89020, 155, { &P15850,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; LG-LU6200 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2619085 = {S89021, 130, { &P17168,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2; en-gb; GT-P7310 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2623213 = {S89022, 61, { &P17168,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-P7310 Build/JOP40D)*/
const Device UA2624171 = {S89023, 124, { &P17168,&P18598,&P19802,&P18092 }};	/*Useragent - Biblia/2.2.8.1 (Linux; U; Android 4.2.1; es-US; GT-P7310 Build/cm_p5wifi-userdebug 4.2.1 JOP40D eng.christian.X.X test-keys)*/
const Device UA2631475 = {S89024, 132, { &P17168,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-P7310 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2634649 = {S89025, 131, { &P17168,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7310 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2637652 = {S89026, 60, { &P17168,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P7310 Build/JDQ39)*/
const Device UA2622973 = {S89027, 104, { &P15876,&P17979,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.0 Android (15/4.0.3; 160dpi; 800x1232; samsung; GT-P5100; espresso10rf; espresso10; en_GB)*/
const Device UA2659985 = {S89028, 90, { &P15397,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST18i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2664837 = {S89029, 130, { &P15767,&P17146,&P20781,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) MQQBrowser/3.0 Mobile/9A405 Safari/7534.48.3*/
const Device UA2667306 = {S89030, 139, { &P20511,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Fly_IQ442 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667150 = {S89031, 166, { &P14225,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2766013 = {S89032, 156, { &P14225,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; Desire HD Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2671477 = {S89033, 134, { &P20478,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JY-G2 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673260 = {S89034, 134, { &P12848,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MB526 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2674426 = {S89035, 135, { &P17948,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SC-02C Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2697797 = {S89036, 171, { &P15060,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2984789 = {S89037, 171, { &P15060,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2698807 = {S89038, 90, { &P18582,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT28h) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2706997 = {S89039, 133, { &P21660,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Novo7 Venus Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2712199 = {S89040, 134, { &P23383,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; H7100 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712820 = {S89041, 134, { &P23090,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i9400 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714231 = {S89042, 135, { &P23130,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; H2000+ Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714270 = {S89043, 161, { &P19862,&P18066,&P22281,&P18092 }};	/*Useragent - OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2738081 = {S89044, 137, { &P14959,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9103 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2879274 = {S89045, 161, { &P20267,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; P300(Funbook) Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752812 = {S89046, 170, { &P18645,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I747 Build/JROO3C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2999522 = {S89047, 162, { &P18645,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-I9305 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2765624 = {S89048, 92, { &P12980,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; LG-E510) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2769546 = {S89049, 154, { &P20542,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAP5000TDUO Build/PrestigioPAP5000TDUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769767 = {S89050, 128, { &P20014,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-01E Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2780912 = {S89051, 154, { &P18639,&P18598,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 10 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA3087433 = {S89052, 150, { &P18639,&P18598,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 10 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2782464 = {S89053, 140, { &P22021,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KHPHN05ANDA Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782495 = {S89054, 130, { &P12889,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2782527 = {S89055, 133, { &P23146,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HYUNDAI T7S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2786239 = {S89056, 60, { &P15364,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX50/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA3041357 = {S89057, 60, { &P15364,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxQ56/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA3084837 = {S89058, 61, { &P15364,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX490/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA2795676 = {S89059, 65, { &P23374,&P17483,&P3459,&P18092 }};	/*Useragent - LG-LGC440/V100 Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2799280 = {S89060, 156, { &P14017,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P990 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2858118 = {S89061, 140, { &P23570,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GOCLEVER TAB R70 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2864821 = {S89062, 69, { &P23570,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; GOCLEVER TAB R70 Build/JRO03H)*/
const Device UA2872188 = {S89063, 159, { &P21602,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; C5302 Build/12.0.A.1.211) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2867214 = {S89064, 142, { &P21882,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Micromax P362 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867603 = {S89065, 138, { &P23276,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; POV_TAB-PL1015 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2868580 = {S89066, 139, { &P15079,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MK16i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869186 = {S89067, 159, { &P21991,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Aqua Wonder Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2891335 = {S89068, 140, { &P18612,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC T329t Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892205 = {S89069, 142, { &P23707,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; Masstel M25 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2944626 = {S89070, 90, { &P17625,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; ST25i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2956917 = {S89071, 128, { &P23135,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; U30GT 2 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2957138 = {S89072, 150, { &P16273,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2962197 = {S89073, 137, { &P23215,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SHV-E330S Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2962583 = {S89074, 142, { &P24161,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Orange Zali Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2963414 = {S89075, 96, { &P20004,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Karbonn A2+) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3021589 = {S89076, 139, { &P20927,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT-AS45IPS Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021946 = {S89077, 134, { &P21261,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i9500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023102 = {S89078, 135, { &P21829,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; F-02E Build/V17R48A) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2971887 = {S89079, 161, { &P13974,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-S5670 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3016639 = {S89080, 161, { &P13974,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5670 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3000311 = {S89081, 152, { &P21301,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M031 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3011277 = {S89082, 164, { &P12132,&P17485,&P24678,&P18092 }};	/*Useragent - SonyEricssonJ10i2/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonj10i2) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3124115 = {S89083, 159, { &P12132,&P17485,&P24678,&P18092 }};	/*Useragent - SonyEricssonJ10i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; fa; sonyericssonj10i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3013030 = {S89084, 89, { &P16598,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A500) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3021098 = {S89085, 137, { &P18747,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A840S Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023781 = {S89086, 137, { &P19847,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S6012 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022212 = {S89087, 99, { &P14010,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5263) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3022868 = {S89088, 127, { &P23801,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; P774A Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3029078 = {S89089, 98, { &P21020,&P18598,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Micromax A116) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3029235 = {S89090, 160, { &P18462,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; HTC Desire X Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3032716 = {S89091, 93, { &P16871,&P22784,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq5) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3033069 = {S89092, 151, { &P18283,&P18066,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3044914 = {S89093, 156, { &P18283,&P18111,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-I9300 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3078674 = {S89094, 170, { &P18283,&P18111,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9300 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3058326 = {S89095, 136, { &P14574,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus S Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3109064 = {S89096, 136, { &P14574,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus S Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058734 = {S89097, 136, { &P24065,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-S7272 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3094366 = {S89098, 137, { &P12496,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I897 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111158 = {S89099, 135, { &P21448,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SH530U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3113436 = {S89100, 139, { &P23148,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ONE TOUCH TAB 7HD Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3079674 = {S89101, 90, { &P18302,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MT25i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3115436 = {S89102, 161, { &P18462,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Desire X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3131296 = {S89103, 137, { &P21350,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8730 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA2351 = {S89104, 86, { &P11685,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-D900E-Bouygues/D900EAGIE1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63719 = {S89105, 77, { &P11685,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-D900E/D900EXAID4 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24349 = {S89106, 74, { &P12101,&P5810,&P3417,&P18092 }};	/*Useragent - BlackBerry8703e/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105*/
const Device UA61758 = {S89107, 53, { &P14320,&P17486,&P3402,&P18092 }};	/*Useragent - MOT-W377/0.0.32 UP.Browser/6.3.0.6.c.19 (GUI) MMP/2.0*/
const Device UA23079 = {S89108, 35, { &P20085,&P17484,&P4381,&P18092 }};	/*Useragent - Samsung-SPHM320 Polaris/5.0 MMP/2.0*/
const Device UA87058 = {S89109, 33, { &P16329,&P18178,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PLAYSTATION 3; 2.00)*/
const Device UA89879 = {S89110, 11, { &P16838,&P22784,&P17470,&P18092 }};	/*Useragent - Micromax Q1*/
const Device UA275297 = {S89111, 138, { &P15364,&P5690,&P22380,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Comodo_Dragon/4.0.1.6 Chrome/4.0.249.78 Safari/532.5*/
const Device UA1940215 = {S89112, 60, { &P15364,&P6729,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows 95; U; ru) Presto/2.10.229 Version/11.60*/
const Device UA484037 = {S89113, 79, { &P15364,&P17188,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; WinNT4.0; en-US; rv:1.8.0.12) Gecko/X Firefox/1.5.0.12*/
const Device UA845795 = {S89114, 84, { &P15364,&P17188,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 4.0; en-US; rv:1.8.0.2) Gecko/X Firefox/1.5.0.2;*/
const Device UA736620 = {S89115, 81, { &P15364,&P5690,&P21174,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 11.11*/
const Device UA854514 = {S89116, 92, { &P15364,&P5690,&P21174,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; ru) Opera 11.11 (ru,ru-RU;q=0.9,en;q=0.8)*/
const Device UA2211511 = {S89117, 66, { &P15364,&P5690,&P21174,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; de) Opera 11.11*/
const Device UA563031 = {S89118, 143, { &P21477,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; cherry-MagnumHD Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA567636 = {S89119, 78, { &P15364,&P5690,&P20902,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; ja-JP; rv:1.7.2) Gecko/X Netscape/7.2*/
const Device UA2207092 = {S89120, 83, { &P15364,&P5690,&P20902,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.7.2) Gecko/X Netscape/7.2 (ax)*/
const Device UA674109 = {S89121, 106, { &P15364,&P18145,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA792052 = {S89122, 105, { &P15364,&P18145,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19*/
const Device UA615525 = {S89123, 125, { &P15364,&P17285,&P22428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19 Comodo_Dragon/18.3.0.0*/
const Device UA615526 = {S89124, 132, { &P15364,&P17285,&P22428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19 Comodo_Dragon/18.3.0.0*/
const Device UA799414 = {S89125, 97, { &P15364,&P18110,&P21780,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.2.0 Safari/534.34*/
const Device UA706571 = {S89126, 136, { &P21450,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; SH8128U Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2379107 = {S89127, 79, { &P15791,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia603)/UC Browser8.2.0.132/50/355/UCWEB Mobile*/
const Device UA896649 = {S89128, 81, { &P12588,&P5055,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-in; NokiaC6-01)/UC Browser8.4.0.159/50/352/UCWEB Mobile*/
const Device UA2497059 = {S89129, 89, { &P12588,&P5055,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiac6-01) UCBrowser8.4.0.159/69/352/UCWEB Mobile  3gpp-gba*/
const Device UA1865592 = {S89130, 102, { &P15364,&P17188,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 4.0; zh-CN; rv:1.9.0.8) Gecko/X Firefox/3.0.8 (.NET CLR 3.5.30729)*/
const Device UA1883825 = {S89131, 81, { &P13374,&P5448,&P4777,&P18092 }};	/*Useragent - HTC_P3700 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA1945024 = {S89132, 32, { &P11880,&P5890,&P24667,&P18092 }};	/*Useragent - Nokia 5320/UCWEB8.2.0.116/28/800*/
const Device UA2162485 = {S89133, 80, { &P11880,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA5320)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA2023934 = {S89134, 169, { &P15397,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; ST18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2423847 = {S89135, 168, { &P17625,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2162157 = {S89136, 133, { &P17595,&P17295,&P22458,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/536.5 (KHTML, like Gecko) Iron/19.0.1100.0 Chrome/19.0.1100.0 Safari/536.5*/
const Device UA2216662 = {S89137, 126, { &P23629,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46NX723; CC/GBR; en) Presto/2.7.61 Version/11.00*/
const Device UA2720897 = {S89138, 126, { &P23629,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40NX720; CC/XGA; en) Presto/2.7.61 Version/11.00*/
const Device UA2742987 = {S89139, 126, { &P23629,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46NX720; CC/USA; en) Presto/2.7.61 Version/11.00*/
const Device UA2261448 = {S89140, 84, { &P12142,&P17485,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; sonyericssonu5i) UCBrowser8.2.1.144/69/352/UCWEB Mobile*/
const Device UA2518670 = {S89141, 146, { &P12135,&P17483,&P24668,&P18092 }};	/*Useragent - LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-in; lg-gd510) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2341614 = {S89142, 138, { &P22501,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MC7403A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654536 = {S89143, 131, { &P22501,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-eg; MC7403A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2344768 = {S89144, 133, { &P23892,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Hd1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2360218 = {S89145, 54, { &P14748,&P5890,&P20777,&P18092 }};	/*Useragent - MQQBrowser/2.6 (NokiaE72-1;SymbianOS/9.3 Series60/3.2)*/
const Device UA2366843 = {S89146, 154, { &P18283,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2375502 = {S89147, 137, { &P20815,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Scroll Engage Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2412447 = {S89148, 142, { &P14848,&P5843,&P24670,&P18092 }};	/*Useragent - NokiaX3-00/5.0 (08.54) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax3-00) UCBrowser8.4.0.159/70/352/UCWEB Mobile*/
const Device UA2418812 = {S89149, 202, { &P12139,&P5204,&P24671,&P18092 }};	/*Useragent - Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5310xpressmusic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2420635 = {S89150, 161, { &P18649,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Micromax A57 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2420790 = {S89151, 151, { &P20521,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; tr-tr; Artes D821 Build/ICS.g12refM805AST.X) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2455072 = {S89152, 137, { &P23907,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PROG160 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2459123 = {S89153, 91, { &P11658,&P5776,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6680) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2478017 = {S89154, 142, { &P19895,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I519 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2485093 = {S89155, 151, { &P18557,&P20706,&P24675,&P18092 }};	/*Useragent - Nokia308/2.0 (05.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia308) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2486244 = {S89156, 136, { &P21936,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ThL W6 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2496545 = {S89157, 133, { &P22435,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TAB465EUK Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2507201 = {S89158, 138, { &P22497,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; w30hd(QuadCore) Build/W30hd) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2549712 = {S89159, 142, { &P22005,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; SmartPad870 Build/E8HD-T364-768) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2984215 = {S89160, 147, { &P22005,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; SmartPad870 Build/E8HD-T364-1280x768) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2553543 = {S89161, 137, { &P23913,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; SH630E Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2559574 = {S89162, 165, { &P21023,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; LG-E405 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2566158 = {S89163, 141, { &P19846,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SO-05D Build/7.0.D.1.130) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2578285 = {S89164, 153, { &P18282,&P5629,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; MITO A200 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2584853 = {S89165, 137, { &P18499,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ISW11SC Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602986 = {S89166, 130, { &P20752,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AN10CG3 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2660337 = {S89167, 162, { &P11953,&P17485,&P24676,&P18092 }};	/*Useragent - SonyEricssonU10i/R7BA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonu10i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2670828 = {S89168, 138, { &P12280,&P18117,&P21783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.3m*/
const Device UA2716712 = {S89169, 140, { &P12280,&P18117,&P21783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.3.2m*/
const Device UA2706839 = {S89170, 136, { &P12482,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T959 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2864663 = {S89171, 137, { &P12482,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-T959 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708508 = {S89172, 137, { &P20124,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ZTE V970 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708616 = {S89173, 138, { &P23523,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-i9377 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2779987 = {S89174, 135, { &P23523,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; I9377 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2729601 = {S89175, 131, { &P23456,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; 1030TPC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2729929 = {S89176, 92, { &P16049,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; LG-P698) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2749860 = {S89177, 142, { &P22689,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Brava DM-993 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2867304 = {S89178, 152, { &P18737,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; X909 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2768702 = {S89179, 147, { &P21052,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2769297 = {S89180, 95, { &P20004,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Karbonn A2) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2769616 = {S89181, 141, { &P22858,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U8836D Build/HuaweiU8836D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2779902 = {S89182, 135, { &P18253,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT894 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2792078 = {S89183, 131, { &P23147,&P17979,&P18112,&P18092 }};	/*Useragent - Lenovo-S720i/S103 Linux/3.4.0 Android/4.1 Release/10.29.2012 Browser/AppleWebKit534.30 Profile/ Configuration/ Mobile Safari/534.30*/
const Device UA2796596 = {S89184, 131, { &P23573,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MID7052 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2867342 = {S89185, 138, { &P23247,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Primo-X1 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2868481 = {S89186, 158, { &P23716,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI A199 Build/HuaweiA199) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2892312 = {S89187, 161, { &P15794,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Sensation XL Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892878 = {S89188, 169, { &P22903,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A62 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2944374 = {S89189, 158, { &P13018,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT11i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2954448 = {S89190, 151, { &P12871,&P24693,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Wildfire S A510e Build/AOKP-jb-m1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957462 = {S89191, 153, { &P23575,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ALCATEL ONE TOUCH 8000A Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2958311 = {S89192, 152, { &P15575,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9100G Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2995503 = {S89193, 151, { &P15575,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2958538 = {S89194, 154, { &P14123,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LG-P970 Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2959917 = {S89195, 157, { &P19859,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328w Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2960307 = {S89196, 160, { &P19779,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A1+ Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2963725 = {S89197, 157, { &P18280,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC One S Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2972713 = {S89198, 162, { &P21068,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; MI-ONE C1 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2980937 = {S89199, 136, { &P21122,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; P-04D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996580 = {S89200, 133, { &P24069,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; R815 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3011953 = {S89201, 167, { &P20094,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3054952 = {S89202, 167, { &P20094,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI C8825D Build/HuaweiC8825D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3011540 = {S89203, 101, { &P12581,&P5055,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia500) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3022258 = {S89204, 142, { &P21623,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 6 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3059102 = {S89205, 157, { &P18488,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3022155 = {S89206, 131, { &P24524,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; V812 Core4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021384 = {S89207, 137, { &P20508,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Cynus T2 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3033034 = {S89208, 157, { &P18641,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-M830 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3022699 = {S89209, 140, { &P19999,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A30 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3025129 = {S89210, 153, { &P18385,&P21637,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-US; Nexus 7 Build/JWR66V) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3061762 = {S89211, 153, { &P18385,&P21637,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-US; Nexus 7 Build/JWR66Y) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3025179 = {S89212, 155, { &P21953,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Ctrl_V1 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3064003 = {S89213, 159, { &P18223,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT22i Build/6.1.1.B.1.54) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3027682 = {S89214, 159, { &P22632,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Titanium S1 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3031636 = {S89215, 94, { &P16591,&P22784,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq50) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3094506 = {S89216, 144, { &P20168,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Maxwell Plus Build/1.0.3 X-14:07) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3080598 = {S89217, 137, { &P18630,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A800S Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3064175 = {S89218, 92, { &P12707,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia701) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3079444 = {S89219, 163, { &P13042,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; HTC Incredible S Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3108377 = {S89220, 163, { &P13042,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HTC Incredible S Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3082756 = {S89221, 172, { &P13064,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I727 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/skyrocket*/
const Device UA3128155 = {S89222, 172, { &P13064,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I727 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/skyrocket*/
const Device UA3130953 = {S89223, 154, { &P21857,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA2521 = {S89224, 67, { &P15298,&P6712,&P4030,&P18092 }};	/*Useragent - Nokia1681c_CMCC/2.0 (06.82) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA21407 = {S89225, 62, { &P15298,&P6712,&P4030,&P18092 }};	/*Useragent - Nokia1681c/2.0 (06.82) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA62093 = {S89226, 84, { &P13844,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonZ555i/R10CA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA7538 = {S89227, 146, { &P13333,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 LGKT610/V20A; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA64491 = {S89228, 146, { &P13333,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 LGKT610/v10A; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA26506 = {S89229, 112, { &P13605,&P17484,&P3636,&P18092 }};	/*Useragent - SAMSUNG-GT-I6220/1.0 SHP/VPP/R5 Jasmine/0.8 NexPlayer/2.12 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA63249 = {S89230, 33, { &P12209,&P17483,&P17660,&P18092 }};	/*Useragent - LG-KS360/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA64461 = {S89231, 40, { &P14672,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KF900 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA68373 = {S89232, 36, { &P12088,&P5129,&P4777,&P18092 }};	/*Useragent - NokiaX6-00/UC Browser7.7.1.88/50/444*/
const Device UA279591 = {S89233, 122, { &P15364,&P17285,&P18211,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/5.2.1.1588 Safari/532.4*/
const Device UA641302 = {S89234, 142, { &P16088,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ko-kr; KU9500 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1871661 = {S89235, 141, { &P16088,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; SU950 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2021183 = {S89236, 133, { &P18266,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; P76TI Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021962 = {S89237, 121, { &P18266,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; P76TI Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2809532 = {S89238, 57, { &P18266,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; P76TI Build/GRJ22)*/
const Device UA461944 = {S89239, 92, { &P17595,&P17295,&P18197,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.1.9) Gecko/X Thunderbird/3.0.4*/
const Device UA878587 = {S89240, 80, { &P17595,&P17297,&P18194,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:6.0.1) Gecko/X Thunderbird/6.0.1*/
const Device UA764246 = {S89241, 99, { &P14540,&P5843,&P4777,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (07.11) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-01/UC Browser7.7.1.88/69/444*/
const Device UA603693 = {S89242, 136, { &P12280,&P18117,&P20783,&P18092 }};	/*Useragent - MQQBrowser/32 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3*/
const Device UA604159 = {S89243, 136, { &P12280,&P18117,&P20783,&P18092 }};	/*Useragent - MQQBrowser/32 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3*/
const Device UA720262 = {S89244, 138, { &P12280,&P18117,&P20783,&P18092 }};	/*Useragent - MQQBrowser/32 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA630331 = {S89245, 74, { &P20370,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; ZTE-C_N760; 320*480) UCWEB7.9.3.103/139/32139*/
const Device UA649769 = {S89246, 80, { &P13398,&P5448,&P4779,&P18092 }};	/*Useragent - SAMSUNG-GT-B7610/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA771980 = {S89247, 62, { &P15364,&P18145,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64; rv:11.0) Gecko/X Firefox/11.0*/
const Device UA853608 = {S89248, 69, { &P15364,&P16110,&P22354,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; rv:9.2) Gecko/X Firefox/9.2 PaleMoon/9.2*/
const Device UA855671 = {S89249, 89, { &P15364,&P16110,&P22354,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:9.2) Gecko/X Firefox/9.2-x64 PaleMoon/9.2-x64*/
const Device UA925813 = {S89250, 103, { &P18088,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Microsoft; Virtual)*/
const Device UA1890267 = {S89251, 139, { &P21675,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; th-th; i-mobile i692 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1936859 = {S89252, 98, { &P11699,&P5768,&P17347,&P18092 }};	/*Useragent - Nokia5200/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia5200/UC Browser8.0.3.107/69/352*/
const Device UA1963632 = {S89253, 139, { &P21719,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; htc_jewel Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1964335 = {S89254, 139, { &P21719,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; htc_jewel Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA1992641 = {S89255, 142, { &P22130,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; hu-hu; xTAB-70i Build/IceCreamSandwich) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2684722 = {S89256, 139, { &P22130,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; hu-hu; xTAB-70i Build/ICS.xTAB70i.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029603 = {S89257, 130, { &P21524,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5;zh-cn;SHP-SH320T/2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2814281 = {S89258, 144, { &P21524,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; SHP-SH320T/AD_230_E350_01006) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2743486 = {S89259, 70, { &P16436,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;GT-I8150;480*800;)UCWEB7.8.0.95/139/355*/
const Device UA2167971 = {S89260, 139, { &P13724,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I9000 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2170773 = {S89261, 35, { &P14826,&P5843,&P21759,&P18092 }};	/*Useragent - iBrowser/Mini2.8 (NokiaC1-01/04.40)*/
const Device UA2202194 = {S89262, 114, { &P15364,&P17285,&P19986,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Sleipnir/3.7.3*/
const Device UA2258843 = {S89263, 114, { &P15364,&P17285,&P19986,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4 Sleipnir/3.7.4*/
const Device UA2197420 = {S89264, 32, { &P15364,&P17017,&P21913,&P18092 }};	/*Useragent - contype chromeframe/22.0.1229.79*/
const Device UA2238010 = {S89265, 32, { &P15364,&P17017,&P21913,&P18092 }};	/*Useragent - contype chromeframe/22.0.1229.94*/
const Device UA2316062 = {S89266, 81, { &P15364,&P17285,&P20444,&P18092 }};	/*Useragent - Mozilla/4.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.2) Gecko/X Firefox/3.8.9*/
const Device UA2365491 = {S89267, 140, { &P18283,&P18066,&P20791,&P18092 }};	/*Useragent - MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2467299 = {S89268, 152, { &P18283,&P18066,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2374805 = {S89269, 73, { &P16329,&P22222,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PLAYSTATION 3 4.15) AppleWebKit/531.22.8 (KHTML, like Gecko)*/
const Device UA2381689 = {S89270, 156, { &P16036,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus One Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747600 = {S89271, 158, { &P16036,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus One Build/CM10; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2386603 = {S89272, 99, { &P12351,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s3653w) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2404960 = {S89273, 141, { &P12521,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SO-02D Build/6.1.F.0.106) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2423639 = {S89274, 153, { &P21592,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Kem 432 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2424643 = {S89275, 148, { &P15798,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Explorer A310e Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2433539 = {S89276, 66, { &P17346,&P18117,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 (iPod4,1; iPhone OS 5.1.1; en_US) AppleWebKit/420+*/
const Device UA2458758 = {S89277, 125, { &P12399,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU8i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/352/UCWEB Mobile*/
const Device UA2478422 = {S89278, 138, { &P19847,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S6012 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2480210 = {S89279, 147, { &P18666,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A25 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2481526 = {S89280, 166, { &P18712,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Next8P12 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2484179 = {S89281, 57, { &P15364,&P17188,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 4.0) Presto/2.12.388 Version/12.12*/
const Device UA2510135 = {S89282, 138, { &P19861,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SCH-I605 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2604375 = {S89283, 137, { &P19861,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I605 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2555000 = {S89284, 139, { &P12784,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Skate Pro Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2555322 = {S89285, 152, { &P18471,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2571193 = {S89286, 140, { &P20006,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT30a Build/7.0.A.3.195) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2580186 = {S89287, 138, { &P20141,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S5301 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2594743 = {S89288, 130, { &P23608,&P18110,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-60EX645; CC/USA; en) Presto/2.10.250 Version/11.60*/
const Device UA2600036 = {S89289, 135, { &P20478,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JY-G2 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2601605 = {S89290, 148, { &P20143,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Edison 3G Build/1.1.11-1015 X-15:24) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2602005 = {S89291, 133, { &P20274,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IdeaTab_A1107 Build/MR1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2603481 = {S89292, 141, { &P21542,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C6503 Build/10.1.A.1.434) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2604169 = {S89293, 138, { &P21384,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F240L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2618468 = {S89294, 219, { &P13452,&P5096,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21608/28.2957; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2623874 = {S89295, 221, { &P13452,&P5096,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2625778 = {S89296, 145, { &P13452,&P5096,&P17371,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2; GT-P1000 Build/FROYO) Opera/9.80 (Android; Opera Mini/5.1.21927/28.3445; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2652939 = {S89297, 155, { &P24649,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android GFIVE-2.3; en-us; GFIVE A68 Build/A68-V1.0-2012.08.08) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2655049 = {S89298, 67, { &P18582,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; LT28h; 720*1280) UCWEB7.9.0.94/139/444*/
const Device UA2656324 = {S89299, 139, { &P23870,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; TCL S710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2665073 = {S89300, 95, { &P11834,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA6760s-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2659643 = {S89301, 94, { &P18295,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; HTC_One_X) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2659727 = {S89302, 164, { &P12980,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-E510 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2661008 = {S89303, 164, { &P12980,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; LG-E510 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2664533 = {S89304, 64, { &P20515,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; A54; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2698804 = {S89305, 101, { &P19975,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 505)*/
const Device UA2699877 = {S89306, 87, { &P21052,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A9) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2703795 = {S89307, 136, { &P23842,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; V8503 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2707141 = {S89308, 126, { &P18239,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A510 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708042 = {S89309, 126, { &P19775,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A210 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2712949 = {S89310, 139, { &P22636,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Aquaris Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713520 = {S89311, 132, { &P20807,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B92 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2716107 = {S89312, 143, { &P18466,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Amazon Kindle Fire HD Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2795993 = {S89313, 132, { &P17595,&P17297,&P22708,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.1 Safari/7536.29.13 Sleipnir/4.2.1*/
const Device UA2749940 = {S89314, 92, { &P21953,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Ctrl_V1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2751602 = {S89315, 156, { &P20146,&P18111,&P22281,&P18092 }};	/*Useragent - OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2752226 = {S89316, 138, { &P22601,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SHV-E300L Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2752243 = {S89317, 92, { &P18241,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; LG-E400) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2766198 = {S89318, 138, { &P18174,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; VS920 4G Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2768425 = {S89319, 160, { &P18720,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Micromax A50 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2768661 = {S89320, 106, { &P11905,&P5890,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NOKIA6120c*/
const Device UA2770275 = {S89321, 69, { &P15364,&P16110,&P22768,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:22.0) Gecko/X Thunderbird/22.0*/
const Device UA2937720 = {S89322, 156, { &P14848,&P5843,&P24677,&P18092 }};	/*Useragent - NokiaX3-00/5.0 (11.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax3-00) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2782846 = {S89323, 135, { &P18737,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Find 5 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783078 = {S89324, 134, { &P23376,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M9000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783419 = {S89325, 135, { &P21330,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PMP5197DULTRA Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2784993 = {S89326, 103, { &P16463,&P17350,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 7.10; en-US; SAMSUNG; GT-I8350) U2/1.0.0 UCBrowser/3.0.0.285 U2/1.0.0 Mobile*/
const Device UA2795371 = {S89327, 97, { &P18649,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A57) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2800087 = {S89328, 141, { &P20491,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Thunder S220 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2865231 = {S89329, 143, { &P23221,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Videocon A51 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865351 = {S89330, 139, { &P21321,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9228 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867437 = {S89331, 168, { &P12871,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2867593 = {S89332, 146, { &P23198,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI H868C Build/HuaweiH868C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867973 = {S89333, 138, { &P22756,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C1604 Build/11.3.A.2.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868544 = {S89334, 159, { &P15517,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT15i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2869182 = {S89335, 166, { &P18224,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI U8815 Build/HuaweiU8815) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2873091 = {S89336, 125, { &P23001,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M97 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2877779 = {S89337, 137, { &P20013,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F180S Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2996988 = {S89338, 137, { &P20013,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F180K Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2878679 = {S89339, 131, { &P23556,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A66/S001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2879149 = {S89340, 147, { &P19856,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-LS970 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2892222 = {S89341, 139, { &P22991,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HTC 802t Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892816 = {S89342, 95, { &P23122,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; Karbonn A6) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2900611 = {S89343, 151, { &P22875,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9220 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2998522 = {S89344, 151, { &P22875,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9220 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2944705 = {S89345, 141, { &P19846,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-05D Build/9.1.C.0.475) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2944807 = {S89346, 153, { &P13452,&P5096,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-US; GT-P1000 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2961755 = {S89347, 152, { &P18737,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; X909 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2966039 = {S89348, 128, { &P23232,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; F7HD Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2966605 = {S89349, 136, { &P19999,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; Karbonn A30 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3012941 = {S89350, 157, { &P19920,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS870 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3013824 = {S89351, 157, { &P20705,&P18598,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA3013949 = {S89352, 93, { &P18595,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8190) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3016426 = {S89353, 92, { &P15206,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE90) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3058497 = {S89354, 91, { &P13964,&P5096,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2; en-US; GT-I5510) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3021826 = {S89355, 168, { &P13157,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Sensation XE with Beats Audio Z715e Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024473 = {S89356, 138, { &P24513,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530X Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3025158 = {S89357, 97, { &P22903,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A62) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3026523 = {S89358, 97, { &P18382,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire C) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3093377 = {S89359, 97, { &P18382,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire C) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3026826 = {S89360, 93, { &P16356,&P18111,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7000) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3028113 = {S89361, 150, { &P21052,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A9 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3030046 = {S89362, 96, { &P22209,&P18111,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Titanium S5) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3056592 = {S89363, 140, { &P24558,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-I8550L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3093653 = {S89364, 145, { &P20801,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Optimus_San_Remo Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094433 = {S89365, 128, { &P20014,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-01E Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3110845 = {S89366, 134, { &P19985,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SH-02E Build/S4100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109408 = {S89367, 139, { &P14993,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT15i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3112417 = {S89368, 129, { &P21990,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; chagall Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3081668 = {S89369, 92, { &P15791,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia603) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3094511 = {S89370, 156, { &P20983,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9308 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3094611 = {S89371, 137, { &P24609,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E440g Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094620 = {S89372, 135, { &P19856,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; LG-LS970 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109424 = {S89373, 165, { &P18472,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-79) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3114637 = {S89374, 158, { &P12798,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT18i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3113422 = {S89375, 151, { &P18442,&P18111,&P24683,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.5 Mobile Safari/533.1*/
const Device UA3130589 = {S89376, 138, { &P22970,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T599N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131679 = {S89377, 148, { &P22968,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HUAWEI P6-U06 Build/HuaweiP6-U06) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132064 = {S89378, 137, { &P16356,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-N7000 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132841 = {S89379, 137, { &P16356,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132605 = {S89380, 126, { &P19775,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A210 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3132935 = {S89381, 146, { &P17989,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N8000_USA_Cricket Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132947 = {S89382, 146, { &P17989,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V8000_USA_Cricket Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA401 = {S89383, 76, { &P16338,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-L72/08.21.09R MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/1.0*/
const Device UA62470 = {S89384, 75, { &P16338,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-L9/08.21.09R MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/1.0*/
const Device UA10415 = {S89385, 92, { &P13485,&P5027,&P4948,&P18092 }};	/*Useragent - BlackBerry8330/4.3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/126 UP.Browser/5.0.3.3*/
const Device UA8521 = {S89386, 122, { &P13451,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-B3410/B3410DDJC3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA17954 = {S89387, 98, { &P13987,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-A727/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA20578 = {S89388, 15, { &P14455,&P7472,&P17470,&P18092 }};	/*Useragent - alcatel-ot-606a*/
const Device UA22360 = {S89389, 61, { &P15436,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonP900/R101 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA22081 = {S89390, 53, { &P14527,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia6220/2.0 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA23256 = {S89391, 81, { &P15580,&P17484,&P4970,&P18092 }};	/*Useragent - Samsung-SPHA900P AU-MIC-A900P/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23894 = {S89392, 79, { &P15580,&P17484,&P4970,&P18092 }};	/*Useragent - Samsung-SPHA900 AU-MIC-A900/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88759 = {S89393, 34, { &P15580,&P17484,&P4970,&P18092 }};	/*Useragent - Samsung-SPHA900 AU-MIC/2.0 MMP/2.0*/
const Device UA49409 = {S89394, 55, { &P13016,&P17017,&P3807,&P18092 }};	/*Useragent - R236/P130D20/Obigo Q03C/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA50489 = {S89395, 34, { &P14358,&P5890,&P4737,&P18092 }};	/*Useragent - NokiaE63/UC Browser7.2.2.51/28/351*/
const Device UA56354 = {S89396, 32, { &P14700,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F08A3(c500;TB;W24H16)*/
const Device UA67877 = {S89397, 36, { &P12193,&P5129,&P4759,&P18092 }};	/*Useragent - NokiaC5-00/UC Browser7.6.1.82/28/352*/
const Device UA2979258 = {S89398, 38, { &P12193,&P5129,&P4759,&P18092 }};	/*Useragent - NokiaC5-00.2/UC Browser7.6.1.82/28/352*/
const Device UA89100 = {S89399, 142, { &P13296,&P5448,&P3430,&P18092 }};	/*Useragent - Vodafone/1.0/SEX1i/R3AA Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/15431; U; DE) UP.Link/6.3.1.20.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA554526 = {S89400, 124, { &P13296,&P5448,&P3430,&P18092 }};	/*Useragent - Vodafone/1.0/SEX1i/R3AA Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/15431; U; UK)  Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA838726 = {S89401, 123, { &P13296,&P5448,&P3430,&P18092 }};	/*Useragent - Vodafone/1.0/SEX1i/R1AA Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/2141; U; UK)  Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA101935 = {S89402, 33, { &P17884,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/X260/Screen-176X220*/
const Device UA253779 = {S89403, 8, { &P15364,&P17017,&P17242,&P18092 }};	/*Useragent - Chrome/5*/
const Device UA823509 = {S89404, 122, { &P15364,&P17017,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; U; WebPositive/533.4; Haiku) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.55 Safari/533.4*/
const Device UA256035 = {S89405, 120, { &P12280,&P17146,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X) (compatible; Yeti-Mobile/0.1; +http://help.naver.com/robots/)*/
const Device UA325496 = {S89406, 137, { &P17595,&P17296,&P22444,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Iron/5.0.381.0 Chrome/5.0.381 Safari/533.4*/
const Device UA835973 = {S89407, 137, { &P17595,&P17296,&P22444,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Iron/5.0.381.0 Chrome/5.0.381 Safari/533.4*/
const Device UA914901 = {S89408, 137, { &P17595,&P17296,&P22444,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Iron/5.0.381.0 Chrome/5.0.381 Safari/533.4*/
const Device UA572022 = {S89409, 108, { &P13308,&P17485,&P17338,&P18092 }};	/*Useragent - SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonj105i/UC Browser7.9.0.102/69/352*/
const Device UA420804 = {S89410, 88, { &P12194,&P5448,&P4779,&P18092 }};	/*Useragent - HTC_Touch2_T3320 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA437188 = {S89411, 41, { &P12581,&P5055,&P24663,&P18092 }};	/*Useragent - Nokia500/UC Browser7.9.0.102/50/355/UCWEB*/
const Device UA552110 = {S89412, 130, { &P18038,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; N-06C Build/AX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA885842 = {S89413, 54, { &P18038,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.3; N-06C Build/AX)*/
const Device UA688379 = {S89414, 70, { &P12568,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2.2;Zh_cn;GT-B7510;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA571118 = {S89415, 99, { &P13035,&P5043,&P4777,&P18092 }};	/*Useragent - NokiaC2-02/2.0 (06.52) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-02/UC Browser7.7.1.88/69/444*/
const Device UA2936707 = {S89416, 60, { &P15364,&P5690,&P24686,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:10.0.2) Gecko/X Yoono/1.8.40*/
const Device UA600310 = {S89417, 88, { &P17595,&P17296,&P22314,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:11.0) Gecko/X Firefox/11.0 SeaMonkey/2.8*/
const Device UA647450 = {S89418, 92, { &P15364,&P16110,&P22357,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:12.0) Gecko/X Firefox/12.0-x64 PaleMoon/12.0-x64*/
const Device UA1970673 = {S89419, 72, { &P15364,&P16110,&P22357,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; rv:12.0) Gecko/X Firefox/12.0 PaleMoon/12.0*/
const Device UA1913050 = {S89420, 157, { &P12681,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus S 4G Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA675854 = {S89421, 144, { &P20144,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; GT-I5508 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2020046 = {S89422, 130, { &P20144,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; GT-I5508 Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2027194 = {S89423, 135, { &P20144,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; GT-I5508 Build/ERE27) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA715233 = {S89424, 106, { &P15364,&P24094,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux mips; en-US) AppleWebKit/531.2+ (KHTML, like Gecko) Version/4.0 Safari/531.2+*/
const Device UA868065 = {S89425, 161, { &P12280,&P17090,&P21785,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) 1Password/3.5.9/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA868895 = {S89426, 161, { &P12280,&P17090,&P21785,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) 1Password/3.5.9/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA899074 = {S89427, 79, { &P13554,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-in; NOKIAN79)/UC Browser8.4.0.159/28/352/UCWEB Mobile*/
const Device UA2759046 = {S89428, 79, { &P13554,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian79-1) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA1837162 = {S89429, 157, { &P15364,&P5448,&P4578,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; SPV M700; OpVer 19.123.2.733) OrangeBot-Mobile 2008.0 (mobilesearch.support@orange-ftgroup.com)*/
const Device UA1892765 = {S89430, 126, { &P15364,&P7133,&P20430,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth Pro/6.0.3.2197 Safari/532.4*/
const Device UA1894815 = {S89431, 126, { &P23620,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL32CX520; CC/XGA; en) Presto/2.7.61 Version/11.00*/
const Device UA1933400 = {S89432, 141, { &P20503,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Stellar Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1935000 = {S89433, 104, { &P18254,&P7480,&P4551,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; NetNose-Crawler 2.0; A New Search Experience: http://www.netnose.com)*/
const Device UA1945895 = {S89434, 91, { &P23987,&P17979,&P3508,&P18092 }};	/*Useragent - CoolPad8013_CMCC_TD/1.0 Linux/2.6.32 Android/2.2 Release/6.30.2011 Browser/AppleWebkit533.1*/
const Device UA1972878 = {S89435, 134, { &P21276,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; QW TB-9718 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1986672 = {S89436, 134, { &P17835,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NookTablet Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2021191 = {S89437, 130, { &P23915,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; U2 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022268 = {S89438, 118, { &P23915,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; U2 Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2685407 = {S89439, 123, { &P23915,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; U2 Build/FRG83) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2028942 = {S89440, 153, { &P13077,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HW-HUAWEI_C8650/C8650V100R001C92B825; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2029948 = {S89441, 166, { &P13077,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8650+ Build/V100R001C92B861) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030365 = {S89442, 136, { &P21539,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; vivo S1 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2167235 = {S89443, 138, { &P15767,&P18432,&P21786,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.6.5/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA2746739 = {S89444, 140, { &P15767,&P18432,&P21786,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.6.5/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA2310587 = {S89445, 67, { &P18741,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; T703; 320*480) UCWEB7.9.0.94/139/32534*/
const Device UA2325078 = {S89446, 86, { &P14851,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s5753e) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2340135 = {S89447, 133, { &P20466,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; B79 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2350343 = {S89448, 123, { &P15364,&P16110,&P24194,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.7 (KHTML, like Gecko) Chrome/20.0.1099.0 Safari/536.7 QQBrowser/6.14.15493.201*/
const Device UA2362732 = {S89449, 125, { &P14980,&P17484,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-b7722) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile*/
const Device UA2384893 = {S89450, 136, { &P18253,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT894 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2399979 = {S89451, 158, { &P23917,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Explay_Advance Build/ExplayExplay_Advance) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2410395 = {S89452, 147, { &P12889,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7300 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2423483 = {S89453, 139, { &P11747,&P24698,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Desire Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2830518 = {S89454, 139, { &P11747,&P24698,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC Desire Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2428297 = {S89455, 98, { &P14010,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s5263) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2445941 = {S89456, 131, { &P20191,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; U30GT-W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2454401 = {S89457, 111, { &P15364,&P17188,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12*/
const Device UA2455736 = {S89458, 132, { &P18478,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6201 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2466333 = {S89459, 136, { &P22681,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; UMI X1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2480534 = {S89460, 131, { &P21309,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; myPadP4 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2480906 = {S89461, 156, { &P19779,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A1+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2501612 = {S89462, 67, { &P15147,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.4;Zh_cn;ST15i;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA2517538 = {S89463, 126, { &P23625,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40EX720; CC/XGA; en) Presto/2.7.61 Version/11.00*/
const Device UA2527543 = {S89464, 148, { &P23475,&P18066,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Y1 Build/HCL ME Tablet Y1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2534637 = {S89465, 171, { &P20330,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; Micromax P350 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2535010 = {S89466, 171, { &P20267,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; P300(Funbook) Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2554106 = {S89467, 44, { &P13677,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SAMSUNG-GT-S5253/S5253DDKJ2)*/
const Device UA2600981 = {S89468, 138, { &P19910,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E617G Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2604128 = {S89469, 127, { &P18282,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A200 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2616963 = {S89470, 80, { &P17323,&P17979,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.4.6) (GT-P6800;samsung;GT-P6800;3.2/HTJ85B)*/
const Device UA2622109 = {S89471, 79, { &P17323,&P17979,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.4.6) (p6800;Samsung;GT-P6800;4.1.2/JZO54K)*/
const Device UA2654519 = {S89472, 137, { &P20982,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; Sen S3 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659098 = {S89473, 101, { &P12707,&P5055,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia701) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2659764 = {S89474, 95, { &P14834,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; Nokia5730S-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2661885 = {S89475, 100, { &P11941,&P17485,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw580i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2667997 = {S89476, 157, { &P20251,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328d Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2671956 = {S89477, 138, { &P23156,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOLO A700 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675532 = {S89478, 137, { &P18674,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N7005 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676877 = {S89479, 141, { &P22858,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U8836D Build/HuaweiU8836D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2687905 = {S89480, 27, { &P15364,&P22846,&P18246,&P18092 }};	/*Useragent - Lemon B369_Maui Wap Browser*/
const Device UA2953315 = {S89481, 23, { &P15364,&P22846,&P18246,&P18092 }};	/*Useragent - Lemon B499_Maui Browser*/
const Device UA2704528 = {S89482, 172, { &P18581,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2708236 = {S89483, 144, { &P15809,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Rhyme S510b Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2731743 = {S89484, 140, { &P23796,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Byond B66 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2733011 = {S89485, 134, { &P12110,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; novo7Legend Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2864046 = {S89486, 137, { &P12110,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Legend2 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2737246 = {S89487, 156, { &P18507,&P5629,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2752402 = {S89488, 164, { &P12987,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; SPH-M930BST Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752739 = {S89489, 92, { &P13445,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN85) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2765371 = {S89490, 143, { &P23726,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; Turbo-X G400 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781802 = {S89491, 168, { &P11787,&P5204,&P24677,&P18092 }};	/*Useragent - Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3120classic) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2768847 = {S89492, 163, { &P12444,&P5477,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; U20i Build/2.1.1.A.0.6) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2773406 = {S89493, 137, { &P20508,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Cynus T2 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773853 = {S89494, 131, { &P21501,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; P5 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773993 = {S89495, 134, { &P20117,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IQ440 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774117 = {S89496, 136, { &P20200,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS GAMEPAD Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2783190 = {S89497, 135, { &P21448,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SH530U Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2789872 = {S89498, 154, { &P18747,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7 (chip mod); vi-vn; IM-A840S Build/i_am_chip_pi) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2797015 = {S89499, 135, { &P21446,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SH837W Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2797452 = {S89500, 135, { &P18470,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ARCHOS 101G10 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2799424 = {S89501, 159, { &P18736,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2800094 = {S89502, 153, { &P19904,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C5170 Build/IML77) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2800665 = {S89503, 130, { &P21117,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HS-9DTB4 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2848375 = {S89504, 143, { &P21978,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GSmart GS202 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2863398 = {S89505, 101, { &P12581,&P5055,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia500) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2865733 = {S89506, 167, { &P20094,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI C8825D Build/HuaweiC8825D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2877813 = {S89507, 167, { &P20094,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2865210 = {S89508, 142, { &P23610,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; SENCOR ELEMENT 8V2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2869504 = {S89509, 158, { &P13437,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; T-Mobile G2 Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2867218 = {S89510, 151, { &P18581,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2868258 = {S89511, 94, { &P12126,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6730c) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2957008 = {S89512, 96, { &P12126,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6730c-1) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2869424 = {S89513, 137, { &P18686,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A810S Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892929 = {S89514, 137, { &P18686,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A810K Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2875251 = {S89515, 134, { &P18225,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; U9200 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3019792 = {S89516, 96, { &P13677,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; fa; samsung-gt-s5253) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2890864 = {S89517, 142, { &P18321,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2891761 = {S89518, 143, { &P15649,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; it-it; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2939221 = {S89519, 155, { &P15649,&P5096,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5570 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2939553 = {S89520, 150, { &P22854,&P18598,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9508 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2941582 = {S89521, 156, { &P18679,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-B5330 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2958043 = {S89522, 103, { &P15364,&P16110,&P21781,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.21 (KHTML, like Gecko) QupZilla/1.4.3 Safari/537.21*/
const Device UA3050716 = {S89523, 96, { &P15364,&P16110,&P21781,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.21 (KHTML, like Gecko) QupZilla/1.4.3 Safari/537.21*/
const Device UA2963974 = {S89524, 156, { &P22941,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6812 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2966562 = {S89525, 128, { &P20529,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; A89 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3023803 = {S89526, 135, { &P21523,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SH930W Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023202 = {S89527, 134, { &P17283,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MB860 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2973696 = {S89528, 93, { &P16871,&P22784,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq5) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2998572 = {S89529, 140, { &P24566,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2;  i-mobile i-STYLE 7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022959 = {S89530, 137, { &P19856,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LS970 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3015102 = {S89531, 92, { &P12280,&P17146,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 5_0_1; zh-CN; iPh3,1) U2/1.0.0 UCBrowser/9.1.0.287 U2/1.0.0 Mobile*/
const Device UA3020617 = {S89532, 92, { &P12280,&P17146,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 5_0_1; zh-CN; iPh4,1) U2/1.0.0 UCBrowser/9.1.0.287 U2/1.0.0 Mobile*/
const Device UA3017566 = {S89533, 93, { &P21356,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; TECNO_N3) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3023017 = {S89534, 141, { &P21738,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; iris502 Build/LAVAIRIS502) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021459 = {S89535, 99, { &P22158,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; INTEX_CLOUD_X1) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3022158 = {S89536, 138, { &P24108,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; W8 beyond Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023248 = {S89537, 158, { &P21973,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Aqua flash Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3079812 = {S89538, 155, { &P14433,&P5096,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-I9003 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3026615 = {S89539, 161, { &P14969,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; HTC Desire SV Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3026694 = {S89540, 155, { &P22597,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SCH-I545 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3061690 = {S89541, 159, { &P18303,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT27i Build/6.1.1.B.1.54) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056705 = {S89542, 129, { &P22873,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N5120 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3093330 = {S89543, 161, { &P22627,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; IRIS 455 Build/LAVAIRIS455) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3079669 = {S89544, 144, { &P21333,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; INO_ONE Build/INO_ONEINO_ONE) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3110880 = {S89545, 102, { &P15649,&P18110,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-S5570) U2/1.0.0 UCBrowser/9.3.0.321 Mobile*/
const Device UA3112122 = {S89546, 156, { &P18713,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P760 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3127491 = {S89547, 156, { &P18713,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P769 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA64259 = {S89548, 61, { &P12049,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia7600/2.0 (04.05) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA6266 = {S89549, 152, { &P13740,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I7110/DJHK7 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA43772 = {S89550, 152, { &P13740,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I7110/XNHL1 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA43773 = {S89551, 152, { &P13740,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I7110/XNHJ8 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA56312 = {S89552, 152, { &P13740,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I7110/XNHL2 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA56314 = {S89553, 152, { &P13740,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I7110/XXHI5 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA62914 = {S89554, 152, { &P13740,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I7110/XXHI3 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA62673 = {S89555, 174, { &P13514,&P17486,&P4112,&P18092 }};	/*Useragent - MOT-VE66/R6743_G_74.31.03RPL Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MotoWebKit/417.19 (KHTML, like Gecko) BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/9.0*/
const Device UA23973 = {S89556, 66, { &P15239,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GB250/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA28415 = {S89557, 84, { &P12241,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-660/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA89164 = {S89558, 85, { &P12241,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-660A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA51475 = {S89559, 74, { &P17473,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-D600/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA62665 = {S89560, 61, { &P17473,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-D608/1.0/WAP2.0 Profile/MIDP-2.0 Configuration/CL*/
const Device UA54310 = {S89561, 61, { &P13444,&P17486,&P17671,&P18092 }};	/*Useragent - MOT-W220/1.0 Release/6.30.2006 Browser/CMCS1.0 Software/0.280*/
const Device UA55219 = {S89562, 40, { &P14300,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP175 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA67344 = {S89563, 146, { &P14816,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; Videocon-V7500 Build/DONUT) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA161637 = {S89564, 10, { &P12236,&P5890,&P3756,&P18092 }};	/*Useragent - NOKIA6700s*/
const Device UA168098 = {S89565, 33, { &P17747,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH906i(c100;TB;W24H16)*/
const Device UA501037 = {S89566, 35, { &P17747,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH906iTV(c100;TB;W24H16)*/
const Device UA207178 = {S89567, 71, { &P17595,&P17296,&P21576,&P18092 }};	/*Useragent - Mac OS X/10.6.8 (10K549); ExchangeWebServices/1.3 (61); Mail/4.5 (1084)*/
const Device UA2209872 = {S89568, 71, { &P17595,&P17296,&P21576,&P18092 }};	/*Useragent - Mac OS X/10.6.7 (10J869); ExchangeWebServices/1.3 (61); Mail/4.5 (1084)*/
const Device UA295717 = {S89569, 132, { &P17973,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; en-us;pandigitalc1hr) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA369629 = {S89570, 129, { &P22907,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Lenovo A390e Build/FRG83G) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2021722 = {S89571, 172, { &P22907,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; LNV-Lenovo_A390e/Lenovo_A390e_S020_X; 320*480; CTC/2.0) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2024229 = {S89572, 172, { &P22907,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; LNV-Lenovo_A390e/Lenovo_A390e_S024_X; 320*480; CTC/2.0) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA447507 = {S89573, 121, { &P15364,&P18346,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 4.10; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.204 Safari/534.16*/
const Device UA826564 = {S89574, 92, { &P17595,&P17296,&P18329,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:11.0) Gecko/X Thunderbird/11.0 Lightning/1.3*/
const Device UA551737 = {S89575, 90, { &P17595,&P17296,&P22306,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:2.0.1) Gecko/X Firefox/4.0.1 SeaMonkey/2.1*/
const Device UA2977617 = {S89576, 88, { &P17595,&P17296,&P22306,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:2.0b11) Gecko/X Firefox/ SeaMonkey/2.1b2*/
const Device UA572132 = {S89577, 42, { &P12142,&P17485,&P17338,&P18092 }};	/*Useragent - sonyericssonu5i/UC Browser7.9.0.102/69/352*/
const Device UA648730 = {S89578, 102, { &P17595,&P17294,&P22482,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.4; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6.1*/
const Device UA1861711 = {S89579, 103, { &P15364,&P17817,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; ru; rv:1.9.2.15pre) Gecko/X Ubuntu/10.10 (maverick) Namoroka/3.6.15pre*/
const Device UA718799 = {S89580, 158, { &P22563,&P18110,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Bigdroid_2.0; zh-cn; AM7-A1/FRF85B; 1024*600) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17 ThunderSoft_BigDroid/2.0*/
const Device UA842712 = {S89581, 81, { &P11964,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE75-1)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA1946677 = {S89582, 70, { &P21052,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; KPT_A9; 480*800) UCWEB7.9.3.103/139/32503*/
const Device UA2695663 = {S89583, 63, { &P21052,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; A9; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2025062 = {S89584, 130, { &P20604,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; TAB264 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2030256 = {S89585, 155, { &P16092,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-18)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2937439 = {S89586, 156, { &P16092,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2144214 = {S89587, 123, { &P15364,&P5690,&P17191,&P18165 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; heritrix/1.3.0 http://www.cs.washington.edu/research/networking/websys/)*/
const Device UA2543287 = {S89588, 70, { &P20142,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; GT-S5838; 320*480) UCWEB7.9.4.145/139/999*/
const Device UA2148434 = {S89589, 126, { &P23623,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46EX520; CC/XGA; en) Presto/2.7.61 Version/11.00*/
const Device UA2167777 = {S89590, 82, { &P12707,&P5055,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; Nokia701)/UC Browser8.5.0.183/51/352/UCWEB Mobile*/
const Device UA2303355 = {S89591, 55, { &P12657,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player sprint / 1.0 / htc_kingdom / 2.3.4*/
const Device UA2442362 = {S89592, 64, { &P12657,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player us_cellular_wwe / 1.0 / htc_kingdom / 2.3.4*/
const Device UA2496596 = {S89593, 54, { &P12657,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player boost / 1.0 / htc_kingdom / 4.0.3*/
const Device UA2361542 = {S89594, 142, { &P15877,&P18111,&P18338,&P18091 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID SPYDER Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2374960 = {S89595, 130, { &P21954,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; 760TPC Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2402354 = {S89596, 122, { &P13979,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; Nokia6220c-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile*/
const Device UA2411656 = {S89597, 129, { &P23597,&P18110,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-40EX640; CC/USA; en) Presto/2.8.115 Version/11.10*/
const Device UA2413874 = {S89598, 149, { &P12824,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; U8180 Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2467078 = {S89599, 155, { &P19859,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2469187 = {S89600, 153, { &P15767,&P18432,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_0_1 like Mac OS X; en-GB) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M Mobile*/
const Device UA2469471 = {S89601, 153, { &P15767,&P18432,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_0_1 like Mac OS X; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7414M Mobile*/
const Device UA2504290 = {S89602, 135, { &P20095,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 101F Build/V49R39H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2577634 = {S89603, 121, { &P14682,&P5843,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia6303iclassic/09.83; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.33*/
const Device UA2526317 = {S89604, 135, { &P21278,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; QW TB-9718D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2527639 = {S89605, 134, { &P20593,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Cube U9GT2 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2560093 = {S89606, 95, { &P16600,&P22784,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx600) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2566660 = {S89607, 134, { &P18464,&P18110,&P21418,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.8 Safari/535.19 Silk-Accelerated=false*/
const Device UA2594117 = {S89608, 139, { &P22087,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ar-eg; G-TiDE E50 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2595878 = {S89609, 160, { &P22054,&P18066,&P18062,&P18092 }};	/*Useragent - Swingtel SX3 Linux/3.0.13 Android/4.0.4 Release/11.27.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2597133 = {S89610, 134, { &P21844,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Celkon CT7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2623618 = {S89611, 98, { &P17323,&P17979,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.0 Android (15/4.0.4; 160dpi; 800x1232; samsung; GT-P6800; GT-P6800; smdk4210; en_US)*/
const Device UA2632797 = {S89612, 95, { &P17323,&P17979,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 Android (17/4.2.1; 213dpi; 800x1205; Samsung; GT-P6800; p6800; smdk4210; en_US)*/
const Device UA2656791 = {S89613, 159, { &P20883,&P18066,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile IQ 2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659403 = {S89614, 90, { &P14993,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; LT15i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2659438 = {S89615, 91, { &P18451,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT26ii) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2661839 = {S89616, 166, { &P18509,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LG-MS770 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2663403 = {S89617, 171, { &P14969,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC Desire SV Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2854208 = {S89618, 170, { &P14969,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC Desire S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2667432 = {S89619, 135, { &P23090,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i9400 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667581 = {S89620, 145, { &P18554,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2668482 = {S89621, 142, { &P21374,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 101 Titanium Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2777576 = {S89622, 33, { &P14669,&P5843,&P21724,&P18092 }};	/*Useragent - OneBrowser/3.0 (NokiaX2-00/08.35)*/
const Device UA2699796 = {S89623, 87, { &P21501,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; P5) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2703343 = {S89624, 91, { &P14090,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2; en-US; GT-I5801) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2705981 = {S89625, 139, { &P23195,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; DIGMA iDsD10 3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2706413 = {S89626, 141, { &P23514,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GL07S Build/HuaweiU9700L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2706951 = {S89627, 127, { &P20127,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT270 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2707575 = {S89628, 133, { &P20156,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S500 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709093 = {S89629, 132, { &P22024,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; dL1 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2710088 = {S89630, 135, { &P21448,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SH530U Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713949 = {S89631, 139, { &P18225,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; U9200 Build/HuaweiU9200) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2733964 = {S89632, 165, { &P18699,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID8042 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444*/
const Device UA2750036 = {S89633, 164, { &P20931,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iBall Slide 3G7271 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2763377 = {S89634, 166, { &P16356,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-N7000 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2767899 = {S89635, 91, { &P14545,&P17483,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-x330t) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2894939 = {S89636, 91, { &P14545,&P17483,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-x330t) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2768596 = {S89637, 106, { &P14033,&P5129,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaC5-03*/
const Device UA2779972 = {S89638, 139, { &P24503,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E270S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2782376 = {S89639, 137, { &P18021,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A770K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783092 = {S89640, 140, { &P20001,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S720 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784531 = {S89641, 142, { &P23161,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Xtreamer Joyz Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2790402 = {S89642, 135, { &P21346,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; NovoCrystal Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2799023 = {S89643, 157, { &P22584,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337M Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2866022 = {S89644, 160, { &P21850,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Spice Mi-435 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2867318 = {S89645, 96, { &P23116,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Aqua.Active) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2867393 = {S89646, 133, { &P12110,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; novo7Legend Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2879341 = {S89647, 136, { &P12110,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Legend2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3028629 = {S89648, 190, { &P15932,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb*/
const Device UA2869725 = {S89649, 133, { &P24077,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; H120 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2891781 = {S89650, 147, { &P19843,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZTE Blade III Pro Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2894217 = {S89651, 156, { &P18507,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2937755 = {S89652, 151, { &P20134,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2941026 = {S89653, 186, { &P12972,&P22212,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-T989 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2956876 = {S89654, 177, { &P23121,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; Intex Aqua Superb Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33829*/
const Device UA2962513 = {S89655, 155, { &P21794,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9152 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2965913 = {S89656, 156, { &P18582,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT28h Build/6.1.E.3.7) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3019198 = {S89657, 157, { &P18582,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT28h Build/6.1.E.2.68) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2966289 = {S89658, 93, { &P21628,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8552) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2966346 = {S89659, 156, { &P16362,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; SPH-D710 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3012166 = {S89660, 159, { &P16362,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; SPH-D710BST Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2966473 = {S89661, 136, { &P21231,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; L-02E Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2971287 = {S89662, 128, { &P20231,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; A21 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3048221 = {S89663, 139, { &P21437,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-F120S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3023416 = {S89664, 139, { &P12681,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus S 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3008813 = {S89665, 133, { &P15919,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3012747 = {S89666, 133, { &P21628,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-I8552 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3022502 = {S89667, 137, { &P18630,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A800S Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3016289 = {S89668, 148, { &P14908,&P6437,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.2; GT-P6210 Build/HTJ85B) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3017442 = {S89669, 142, { &P22749,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-04E Build/10.1.1.D.2.31) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3022492 = {S89670, 134, { &P23478,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ZP810 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020930 = {S89671, 139, { &P24609,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-sa; LG-E440g Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3022265 = {S89672, 127, { &P21222,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT1S0 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023313 = {S89673, 92, { &P21023,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; LG-E405) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3058380 = {S89674, 95, { &P23122,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; Karbonn A6) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3079836 = {S89675, 157, { &P23156,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; XOLO A700 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3059257 = {S89676, 99, { &P14912,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-c3222) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3059742 = {S89677, 142, { &P12280,&P18432,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 UCBrowser/9.2.0.312 Mobile*/
const Device UA3078767 = {S89678, 142, { &P12280,&P18432,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_2 like Mac OS X; zh-CN) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A551 UCBrowser/9.2.0.312 Mobile*/
const Device UA3093959 = {S89679, 134, { &P18472,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT925 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110805 = {S89680, 133, { &P18314,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M032 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3065665 = {S89681, 74, { &P18141,&P24347,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 2.61) AppleWebKit/536.26 (KHTML, like Gecko)*/
const Device UA3092251 = {S89682, 130, { &P22025,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; VL-879 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3108952 = {S89683, 160, { &P16292,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-CN; Galaxy Nexus Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3094309 = {S89684, 135, { &P20437,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; FJL21 Build/V39R48C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094796 = {S89685, 132, { &P23477,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; XT926 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126340 = {S89686, 139, { &P15047,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SK17i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109568 = {S89687, 107, { &P19862,&P17979,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; Micromax A110) U2/1.0.0 UCBrowser/9.3.0.321 Mobile*/
const Device UA3110187 = {S89688, 157, { &P18471,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-78) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3115426 = {S89689, 157, { &P18509,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS770 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3127320 = {S89690, 152, { &P18643,&P5043,&P24680,&P18092 }};	/*Useragent - Nokia311/5.0 (07.36) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia311) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3131753 = {S89691, 137, { &P18559,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LG730 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132942 = {S89692, 138, { &P15972,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One XL Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA21128 = {S89693, 141, { &P15160,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; el-gr; Light Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA64303 = {S89694, 64, { &P13541,&P17483,&P3534,&P18092 }};	/*Useragent - LG-CU575/V10d Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24672 = {S89695, 104, { &P15385,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-A827/A827UCHC1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA30750 = {S89696, 80, { &P12334,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-T459A/T459UEIK2 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53444 = {S89697, 68, { &P13975,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHD840/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA69430 = {S89698, 36, { &P13324,&P5055,&P4779,&P18092 }};	/*Useragent - NokiaN8-00/UC Browser7.8.0.95/50/355*/
const Device UA2212874 = {S89699, 46, { &P13324,&P5055,&P4779,&P18092 }};	/*Useragent - nokian8-00/UC Browser7.8.0.95/69/351  3gpp-gba*/
const Device UA1906549 = {S89700, 143, { &P21371,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; de-de; ViewSonic-V350 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA89743 = {S89701, 80, { &P17805,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-P50/S041 LMP/LML Release/2009.12.10 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA122030 = {S89702, 80, { &P17805,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-P50/S045 LMP/LML Release/2010.03.08 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA2263547 = {S89703, 83, { &P17805,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-P50-VN/S010 LMP/LML Release/2010.03.03 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA2265801 = {S89704, 83, { &P17805,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-P50-VN/S014 LMP/LML Release/2010.05.06 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA89890 = {S89705, 31, { &P17726,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F07B(c500;TB;W24H16)*/
const Device UA90009 = {S89706, 31, { &P17742,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P06B(c500;TB;W24H16)*/
const Device UA311142 = {S89707, 36, { &P11905,&P5890,&P4759,&P18092 }};	/*Useragent - NOKIA6120c/UC Browser7.6.1.82/28/352*/
const Device UA253784 = {S89708, 9, { &P15364,&P17017,&P17249,&P18092 }};	/*Useragent - Chrome/10*/
const Device UA1889275 = {S89709, 95, { &P15364,&P17017,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible) AppleWebKit/534.14 (KHTML, like Gecko) Chrome/10.0.602.0 Safari/534.14*/
const Device UA762323 = {S89710, 46, { &P15767,&P17090,&P17470,&P18092 }};	/*Useragent - Facebook 4100.0 (iPad; iPhone OS 4.3.5; en_US)*/
const Device UA1994753 = {S89711, 46, { &P15767,&P17090,&P17470,&P18092 }};	/*Useragent - Facebook 4110.0 (iPad; iPhone OS 4.3.3; en_US)*/
const Device UA461162 = {S89712, 91, { &P15364,&P22203,&P24474,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/4.2; Linux) KHTML/4.2.1 (like Gecko) Fedora/4.2.1-4.fc11*/
const Device UA487869 = {S89713, 120, { &P15364,&P5208,&P24642,&P18092 }};	/*Useragent - Mozilla/5.0(HW-C5700/C5700C58B107;U;REX/1.0;BREW/3.1.5;Profile/MIDP-2.0 Configuration/CLDC-1.1;240*320;CTC/2.0)ucweb/7.0*/
const Device UA510590 = {S89714, 122, { &P15364,&P5208,&P24642,&P18092 }};	/*Useragent - Mozilla/5.0 (HW-C5720/C5720C58B105;U;REX/1.0;BREW/3.1.5;Profile/MIDP-2.0 Configuration/CLDC-1.1;240*320;CTC/2.0) Ucweb/7.0*/
const Device UA724361 = {S89715, 122, { &P15364,&P5208,&P24642,&P18092 }};	/*Useragent - Mozilla/5.0 (HW-C5700/C5700C58B401;U;REX/1.0;BREW/3.1.5;Profile/MIDP-2.0 Configuration/CLDC-1.1;240*320;CTC/2.0) ucweb/7.0*/
const Device UA2134989 = {S89716, 122, { &P15364,&P5208,&P24642,&P18092 }};	/*Useragent - Mozilla/5.0 (HW-C5720/C5720C58B106;U;REX/1.0;BREW/3.1.5;Profile/MIDP-2.0 Configuration/CLDC-1.1;240*320;CTC/2.0) Ucweb/7.0*/
const Device UA2140939 = {S89717, 120, { &P15364,&P5208,&P24642,&P18092 }};	/*Useragent - Mozilla/5.0(HW-C5710/C5710C58B105;U;REX/1.0;BREW/3.1.5;Profile/MIDP-2.0 Configuration/CLDC-1.1;240*320;CTC/2.0)ucweb/7.0*/
const Device UA501908 = {S89718, 75, { &P17595,&P17295,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3;) Gecko/X Firefox/2.0.0.17*/
const Device UA505606 = {S89719, 89, { &P15364,&P17285,&P18201,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.8.0.10) Gecko/X Thunderbird/1.5.0.10*/
const Device UA570467 = {S89720, 138, { &P21316,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; 008Z Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA630664 = {S89721, 138, { &P12280,&P17146,&P20784,&P18092 }};	/*Useragent - MQQBrowser/33 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3*/
const Device UA661766 = {S89722, 138, { &P12280,&P17146,&P20784,&P18092 }};	/*Useragent - MQQBrowser/33 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA696695 = {S89723, 44, { &P14851,&P17484,&P17347,&P18092 }};	/*Useragent - samsung-gt-s5753e/UC Browser8.0.3.107/69/444*/
const Device UA715634 = {S89724, 139, { &P17595,&P17295,&P22445,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Iron/7.0.520.1 Chrome/7.0.520.1 Safari/534.7*/
const Device UA720806 = {S89725, 79, { &P15364,&P7133,&P18097,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:9.0.1) Gecko/X Firefox/9.0.1 slurp slurp*/
const Device UA1940183 = {S89726, 66, { &P15364,&P7133,&P18097,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:9.0.1) Gecko/X Firefox/9.0.1 slurp*/
const Device UA729069 = {S89727, 81, { &P13648,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaE7-00)/UC Browser8.2.0.132/50/355/UCWEB Mobile*/
const Device UA845347 = {S89728, 96, { &P15364,&P16110,&P22356,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:11.0) Gecko/X Firefox/11.0.1-x64 PaleMoon/11.0.1-x64*/
const Device UA2332999 = {S89729, 76, { &P15364,&P16110,&P22356,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; rv:11.0) Gecko/X Firefox/11.0.1 PaleMoon/11.0.1*/
const Device UA858492 = {S89730, 37, { &P15364,&P18144,&P18405,&P18092 }};	/*Useragent - Opera/9.22 (X11; OpenBSD i386; U; en)*/
const Device UA899725 = {S89731, 122, { &P15364,&P17285,&P22430,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5 Comodo_Dragon/19.1.0.0*/
const Device UA899746 = {S89732, 129, { &P15364,&P17285,&P22430,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5 Comodo_Dragon/19.1.0.0*/
const Device UA2024077 = {S89733, 144, { &P21368,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; JOYPAD_C81 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2411627 = {S89734, 137, { &P21368,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; JOYPAD C81 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA2210950 = {S89735, 28, { &P16569,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxM2/Q03C MAUI-browser*/
const Device UA2214683 = {S89736, 85, { &P12243,&P5448,&P4737,&P18092 }};	/*Useragent - HTC_Snap_S521 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.2.2.51*/
const Device UA2258620 = {S89737, 164, { &P16049,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P698 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2339387 = {S89738, 146, { &P20158,&P5629,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; A7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2452949 = {S89739, 147, { &P20158,&P5629,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; A7* Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2384497 = {S89740, 161, { &P20262,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Karbonn_A18 Build/KarbonnA18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2386175 = {S89741, 40, { &P16463,&P17484,&P20764,&P18092 }};	/*Useragent - SAMSUNG-GT-I8350/UCWEB2.6.0.213/47/32079*/
const Device UA2413203 = {S89742, 135, { &P21164,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ZP200 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2424168 = {S89743, 99, { &P15302,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonwt13i) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2430095 = {S89744, 99, { &P15302,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonwt13i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2528203 = {S89745, 96, { &P15302,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; fa; sonyericssonwt13i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA3043444 = {S89746, 96, { &P15302,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; fa; sonyericssonwt13i) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2424304 = {S89747, 136, { &P21345,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; F-11D Build/V24R40A) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2471953 = {S89748, 155, { &P18680,&P18066,&P19805,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.6 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2482001 = {S89749, 75, { &P14031,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.1-update1; zh-cn; GT-I5503; 240*320) UCWEB7.9.0.94/139/355*/
const Device UA2487224 = {S89750, 151, { &P18648,&P5043,&P24675,&P18092 }};	/*Useragent - Nokia306/2.0 (03.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia306) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2487314 = {S89751, 73, { &P15364,&P18145,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (X11; FreeBSD 9.1-RELEASE amd64) Presto/2.12.388 Version/12.12*/
const Device UA2496116 = {S89752, 83, { &P15364,&P18110,&P21959,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.3a1pre) Gecko/X Minefield/3.7a1pre*/
const Device UA2681943 = {S89753, 81, { &P15364,&P18110,&P21959,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.3a1pre) Gecko/X Minefield/3.7a1pre*/
const Device UA2502408 = {S89754, 144, { &P23433,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; BLU P100 Build/1.0.8388.0486) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2595707 = {S89755, 66, { &P18448,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; ST26i; 480*854) UCWEB7.9.0.94/139/444*/
const Device UA2505190 = {S89756, 137, { &P18075,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nb-no; KIS II Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2522522 = {S89757, 187, { &P14743,&P17485,&P24671,&P18092 }};	/*Useragent - SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; sonyericssonj20i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2527596 = {S89758, 155, { &P14969,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Desire S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2655421 = {S89759, 155, { &P14969,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2528346 = {S89760, 81, { &P18657,&P17484,&P17374,&P18092 }};	/*Useragent - SAMSUNG-GT-C3262 Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2535415 = {S89761, 135, { &P21660,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Novo7 Venus Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2548537 = {S89762, 80, { &P23751,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-C3782/C3782XXLE1 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2566033 = {S89763, 174, { &P13042,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC Incredible S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2578293 = {S89764, 90, { &P18254,&P16110,&P18245,&P18092 }};	/*Useragent - RssBandit/1.9.0.1002 (.NET CLR 2.0.50727.6400; WinNT 6.2.9200.0; http://www.rssbandit.org)*/
const Device UA2595381 = {S89765, 63, { &P18254,&P16110,&P18245,&P18092 }};	/*Useragent - SharpReader/0.9.7.0 (.NET CLR 2.0.50727.6400; WinNT 6.2.9200.0)*/
const Device UA2580747 = {S89766, 192, { &P23773,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N8010 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0000, gb*/
const Device UA2860577 = {S89767, 192, { &P23773,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N8010 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, +0100, gb*/
const Device UA2582425 = {S89768, 100, { &P13220,&P17485,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk770i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2613266 = {S89769, 90, { &P17337,&P17979,&P22069,&P18092 }};	/*Useragent - Instagram 1.1.3 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7500; GT-P7500; p3; en_GB)*/
const Device UA2643756 = {S89770, 34, { &P15364,&P16110,&P17215,&P18092 }};	/*Useragent - Opera/9.50 (Windows NT 6.2; U; en)*/
const Device UA3117839 = {S89771, 34, { &P15364,&P16110,&P17215,&P18092 }};	/*Useragent - Opera/9.52 (Windows NT 6.2; U; en)*/
const Device UA2662389 = {S89772, 171, { &P14031,&P5477,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; GT-I5503 Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3107128 = {S89773, 175, { &P14031,&P5477,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; GT-I5503 Build/ERE27) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.276 Mobile*/
const Device UA2666708 = {S89774, 140, { &P17968,&P18598,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile*/
const Device UA2668012 = {S89775, 157, { &P18295,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HTC One X Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2716153 = {S89776, 156, { &P18283,&P18111,&P22281,&P18092 }};	/*Useragent - OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2676373 = {S89777, 139, { &P20708,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XOLO A1000 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676847 = {S89778, 137, { &P20508,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Cynus T2 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2702811 = {S89779, 149, { &P18702,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N8013 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3007922 = {S89780, 155, { &P18702,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-N8013 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1/n8013*/
const Device UA2712201 = {S89781, 127, { &P20127,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT270 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2714565 = {S89782, 139, { &P22641,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C1504 Build/11.3.A.0.47) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714664 = {S89783, 89, { &P17595,&P17296,&P22317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:21.0) Gecko/X Firefox/21.0 SeaMonkey/2.18*/
const Device UA2729421 = {S89784, 67, { &P17968,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; T730; 320*480) UCWEB7.9.0.94/139/33183*/
const Device UA2730384 = {S89785, 162, { &P19970,&P17485,&P24676,&P18092 }};	/*Useragent - SonyEricssonG502/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssong502) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2733409 = {S89786, 179, { &P18604,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Smartphone Android by SFR STARADDICT II Build/IceCreamSandwich) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2734776 = {S89787, 158, { &P18604,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; STARADDICT II Plus Build/GXI_ZTEV1.0.0B02) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2957172 = {S89788, 157, { &P20705,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2752274 = {S89789, 155, { &P12568,&P5096,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-US; GT-B7510 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2755285 = {S89790, 134, { &P21302,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IQ441 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2768407 = {S89791, 99, { &P12399,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; SonyEricssonU8i) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2769350 = {S89792, 145, { &P20000,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ607 Build/7.7.1-122-4-FLEM-WIFI-LA) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2770273 = {S89793, 138, { &P23156,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOLO A700 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2773924 = {S89794, 138, { &P19858,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KYL21 Build/019.0.3b00) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782343 = {S89795, 137, { &P20983,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9308 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2782998 = {S89796, 136, { &P23164,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-7204 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783449 = {S89797, 132, { &P22024,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; dL1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783753 = {S89798, 136, { &P18499,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ISW11SC Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784001 = {S89799, 189, { &P18639,&P18598,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Nexus 10 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.2/552,2.0.2.552,2013-05-22 23:05, +0100, gb*/
const Device UA2792087 = {S89800, 91, { &P14372,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gm360) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2792800 = {S89801, 159, { &P14372,&P17483,&P24676,&P18092 }};	/*Useragent - LG-GM360/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; es-LA; lg-gm360) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2793863 = {S89802, 154, { &P12216,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2794095 = {S89803, 95, { &P21973,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Aqua_flash) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2799429 = {S89804, 159, { &P16319,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; U8860 Build/HuaweiU8860) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2800206 = {S89805, 136, { &P22805,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E455 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2864908 = {S89806, 141, { &P21503,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lemon P101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2866828 = {S89807, 135, { &P20155,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; T-909S Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2958286 = {S89808, 127, { &P20155,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T-909 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2867130 = {S89809, 137, { &P17324,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9001 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867215 = {S89810, 134, { &P23667,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; JY-G4 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869413 = {S89811, 136, { &P22874,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9502 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869876 = {S89812, 149, { &P13054,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2892457 = {S89813, 134, { &P23523,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; I9377 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3049577 = {S89814, 137, { &P23523,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-i9377 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892730 = {S89815, 145, { &P23137,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I407 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2944407 = {S89816, 93, { &P18304,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I9070) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2944905 = {S89817, 93, { &P15575,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9100) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2962000 = {S89818, 159, { &P15147,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; ST15i Build/4.0.2.A.0.84) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2962573 = {S89819, 88, { &P20529,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A89) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3020419 = {S89820, 134, { &P20098,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GN800 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022501 = {S89821, 136, { &P21525,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SBM203SH Build/S0022) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2983008 = {S89822, 98, { &P21026,&P5629,&P15760,&P18092 }};	/*Useragent - HS-U930/1.0 Linux/2.6 Android/2.3 Browser/AppleWebKit533.1 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA2983263 = {S89823, 136, { &P21026,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-U930 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2987532 = {S89824, 158, { &P18450,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Xperia Miro Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3004990 = {S89825, 135, { &P18448,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-in; ST26i Build/11.2.A.0.31) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3104589 = {S89826, 135, { &P18448,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-in; ST26i Build/11.2.A.0.21) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3025623 = {S89827, 89, { &P11950,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; X10i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3018244 = {S89828, 148, { &P24545,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Starmobile KNIGHT Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3021330 = {S89829, 148, { &P15201,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Sensation Z710e Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021368 = {S89830, 124, { &P21052,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A9 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021390 = {S89831, 146, { &P18713,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P769 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3024441 = {S89832, 90, { &P12616,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; WT19i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3059932 = {S89833, 161, { &P12280,&P19826,&P22825,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.9909IP Mobile*/
const Device UA3027016 = {S89834, 95, { &P12444,&P5477,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; U20i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3057390 = {S89835, 99, { &P14090,&P5477,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT-I5801) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3058883 = {S89836, 164, { &P18724,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; IdeaTab A2107A-H Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3059304 = {S89837, 183, { &P20729,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A27 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3114666 = {S89838, 134, { &P21878,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; 201F Build/V35R13E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3062916 = {S89839, 93, { &P23000,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; TECNO_P3) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3083243 = {S89840, 158, { &P24132,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; iris504Q Build/iris504Q) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3079076 = {S89841, 174, { &P22622,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Micromax_A72 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3081125 = {S89842, 159, { &P21980,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; UBISLATE7C+ Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3093688 = {S89843, 160, { &P21980,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; UBISLATE 7C+ Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3095481 = {S89844, 158, { &P18302,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT25i Build/4.1.B.0.631) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3095743 = {S89845, 155, { &P23163,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; SCH-I959 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3111528 = {S89846, 153, { &P20175,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI 1S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3111892 = {S89847, 156, { &P23521,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9195 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3112543 = {S89848, 187, { &P13157,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3131406 = {S89849, 137, { &P18673,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7562 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA8645 = {S89850, 62, { &P13487,&P5204,&P3780,&P18092 }};	/*Useragent - Nokia6300i/2.0 (03.41) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA22873 = {S89851, 66, { &P15522,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GB280/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA23500 = {S89852, 64, { &P15609,&P17483,&P4203,&P18092 }};	/*Useragent - LG-CU400/V1.0 Obigo/Q04C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA28978 = {S89853, 69, { &P16210,&P5448,&P22271,&P18092 }};	/*Useragent - Xda_Stellar; 240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA64335 = {S89854, 68, { &P15364,&P19954,&P3976,&P18092 }};	/*Useragent - SEC-SGHP520/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA50376 = {S89855, 146, { &P13668,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HTC Dream Build/EPE54B) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA56135 = {S89856, 147, { &P13668,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HTC Dream Build/Super E) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA51677 = {S89857, 45, { &P13952,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-806A/1.0 ObigoInternetBrowser/Q03C*/
const Device UA54985 = {S89858, 46, { &P14272,&P17484,&P5001,&P18092 }};	/*Useragent - SCH-U410/1.0 NetFront/3.0.22.2.6 (GUI) MMP/2.0*/
const Device UA63386 = {S89859, 101, { &P14245,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E590/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA67901 = {S89860, 41, { &P12807,&P17486,&P17602,&P18092 }};	/*Useragent - MOT-EX124g Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA69085 = {S89861, 80, { &P15425,&P5448,&P4777,&P18092 }};	/*Useragent - SAMSUNG-GT-B7300/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA153076 = {S89862, 116, { &P15364,&P7127,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.41 Safari/534.7*/
const Device UA300401 = {S89863, 143, { &P12991,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-ru; MTC 916 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2020605 = {S89864, 76, { &P14031,&P5477,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.1-update1;Zh_cn;GT-I5503;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA753910 = {S89865, 58, { &P15364,&P5690,&P22309,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:5.0) Gecko/X SeaMonkey/2.2*/
const Device UA1890184 = {S89866, 77, { &P15364,&P5690,&P22309,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:5.0) Gecko/X Firefox/5.0 SeaMonkey/2.2*/
const Device UA583647 = {S89867, 85, { &P15364,&P17285,&P18192,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:9.0) Gecko/X Thunderbird/9.0.1 Lightning/1.1.1*/
const Device UA616252 = {S89868, 58, { &P13126,&P17017,&P18246,&P18092 }};	/*Useragent - Nexian NX-G733/MTK Release/10.01.2009/Browser/MAUI Profile*/
const Device UA616291 = {S89869, 58, { &P13126,&P17017,&P18246,&P18092 }};	/*Useragent - Nexian NX-G733/MTK Release/10.01.2009 Browser/MAUI Profile*/
const Device UA2393837 = {S89870, 72, { &P13078,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.2; zh-cn; ZTE-U_V880; 480*800) UCWEB7.9.4.145/139/800*/
const Device UA690963 = {S89871, 43, { &P14972,&P17484,&P17347,&P18092 }};	/*Useragent - samsung-gt-s3370/UC Browser8.0.3.107/69/444*/
const Device UA1961369 = {S89872, 74, { &P13094,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; Lenovo_A60; 320*480) UCWEB7.9.3.103/139/32399*/
const Device UA694319 = {S89873, 134, { &P21513,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; fr-fr; MP434 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA697932 = {S89874, 64, { &P14632,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;T8788;7.10;7720)*/
const Device UA730788 = {S89875, 29, { &P12622,&P17017,&P4924,&P18092 }};	/*Useragent - C5120/001.00 ACS-Netfront/3.2*/
const Device UA765194 = {S89876, 172, { &P16292,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76I; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA905334 = {S89877, 172, { &P16292,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy Nexus Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA776458 = {S89878, 109, { &P17595,&P18120,&P20804,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Qt/4.6.0 Safari/532.4*/
const Device UA779762 = {S89879, 80, { &P12638,&P5448,&P4779,&P18092 }};	/*Useragent - SAMSUNG-SGH-i780/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA901976 = {S89880, 122, { &P15364,&P17285,&P22431,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5 Comodo_Dragon/19.2.0.0*/
const Device UA902229 = {S89881, 129, { &P15364,&P17285,&P22431,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5 Comodo_Dragon/19.2.0.0*/
const Device UA924843 = {S89882, 66, { &P15364,&P5690,&P18330,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:14.0) Gecko/X Firefox/14.0.1 slurp*/
const Device UA1995184 = {S89883, 72, { &P15364,&P5690,&P18330,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:14.0) Gecko/X Firefox/14.0.1 slurp slurp*/
const Device UA1969562 = {S89884, 83, { &P12329,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN86 8MP)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA2361804 = {S89885, 82, { &P17595,&P18207,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; en) Presto/2.10.229 Version/11.62*/
const Device UA2337011 = {S89886, 140, { &P21364,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-us; onetouch T60 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2760032 = {S89887, 150, { &P15028,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; LG-P350 Build/GRI40; BeautySense 4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2807674 = {S89888, 138, { &P15028,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; Optimus Me Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026682 = {S89889, 138, { &P23919,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; R800i Build/4.1.H.0.4) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2479993 = {S89890, 135, { &P23919,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; R800i Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2028490 = {S89891, 135, { &P21444,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PLOYER-MOMO Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2423121 = {S89892, 141, { &P21444,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PLOYER-MOMO Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2034473 = {S89893, 137, { &P22384,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; PTT1026 Build/ICS.b02ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2722027 = {S89894, 128, { &P22384,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; PTT1026 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2153570 = {S89895, 53, { &P11929,&P5890,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8 (Nokia5230;SymbianOS/9.1 Series60/3.0)*/
const Device UA2258077 = {S89896, 143, { &P13042,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Incredible S Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2500094 = {S89897, 143, { &P13042,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Incredible S Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2332200 = {S89898, 76, { &P21242,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; Lenovo_A366t; 320*480) UCWEB7.9.3.103/139/32908*/
const Device UA2349408 = {S89899, 135, { &P18651,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SH-01D Build/S8230) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2356098 = {S89900, 155, { &P18465,&P18066,&P19814,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.2 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2383494 = {S89901, 139, { &P12955,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST3_Blade Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2387194 = {S89902, 39, { &P18623,&P5448,&P20764,&P18092 }};	/*Useragent - NOKIA-Lumia 510/UCWEB2.6.0.213/47/32079*/
const Device UA2401918 = {S89903, 135, { &P15364,&P17817,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/10.10 Chromium/22.0.1229.79 Chrome/22.0.1229.79 Safari/537.4*/
const Device UA2445407 = {S89904, 158, { &P20367,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ALCATEL one touch 986 Build/C986-2SALCN1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950117 = {S89905, 159, { &P20367,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ALCATEL one touch 986+ Build/C986-2SALCN1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2457535 = {S89906, 157, { &P13123,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2469285 = {S89907, 167, { &P18295,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; HTC One X Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2483130 = {S89908, 137, { &P20357,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SmartPad7503G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2485050 = {S89909, 140, { &P21650,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; POV_TAB-PROTAB26 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2506462 = {S89910, 135, { &P21261,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i9500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2506974 = {S89911, 135, { &P18113,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; XT912 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2793480 = {S89912, 134, { &P18113,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT912 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2554226 = {S89913, 131, { &P23415,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; TM-9747 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2579329 = {S89914, 131, { &P18715,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID8048 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2580473 = {S89915, 164, { &P14123,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P970 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2583185 = {S89916, 137, { &P20075,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Xperia S Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2583269 = {S89917, 140, { &P20439,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SOL21 Build/9.0.F.2.128) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2598783 = {S89918, 53, { &P15364,&P6704,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows 98) Presto/2.12.388 Version/12.14*/
const Device UA2599557 = {S89919, 169, { &P15147,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; ST15i Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2602017 = {S89920, 135, { &P23949,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; H9500 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2611754 = {S89921, 148, { &P14905,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; GT-P7100 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2612819 = {S89922, 148, { &P14905,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; GT-P7100 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616334 = {S89923, 148, { &P14905,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; GT-P7100 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2621098 = {S89924, 263, { &P14905,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; it-it; GT-P7100 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.3.1-89 (Android 4.1.2; samsung GT-P7100 Build/cm_p3-userdebug 4.1.2 JZO54K eng..X.X test-keys)*/
const Device UA2621947 = {S89925, 166, { &P15876,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P5100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2624095 = {S89926, 166, { &P15876,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P5100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2622454 = {S89927, 95, { &P15985,&P17979,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.0 Android (13/3.2; 160dpi; 600x976; samsung; GT-P6200; GT-P6200; smdkc210; en_US)*/
const Device UA2708427 = {S89928, 130, { &P12889,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7300 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2653619 = {S89929, 139, { &P21533,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo E1t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996616 = {S89930, 26, { &P19861,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SCH-I605/100.40102*/
const Device UA2659677 = {S89931, 172, { &P17952,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3086562 = {S89932, 172, { &P17952,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2671113 = {S89933, 136, { &P14017,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-P990 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2675510 = {S89934, 134, { &P19864,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 80XSK Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2695105 = {S89935, 83, { &P15364,&P16110,&P20902,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.7.2) Gecko/X Netscape/7.2 (ax)*/
const Device UA2699219 = {S89936, 93, { &P15060,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5660) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2703704 = {S89937, 142, { &P22761,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Wynncom_G58 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2707870 = {S89938, 136, { &P14574,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Nexus S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708500 = {S89939, 98, { &P21736,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; LAVA_iris_351) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2709615 = {S89940, 172, { &P12605,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2712888 = {S89941, 138, { &P20174,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Cink King Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714380 = {S89942, 128, { &P24536,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VL-110 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2724096 = {S89943, 139, { &P18714,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V889D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2762675 = {S89944, 141, { &P18714,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE-U V889D Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2733270 = {S89945, 97, { &P21851,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Spice_MI-352) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2734263 = {S89946, 160, { &P20289,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC6435LVW Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2734633 = {S89947, 145, { &P15805,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Raider X710e Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734754 = {S89948, 140, { &P15805,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Holiday Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735559 = {S89949, 138, { &P18386,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I757M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735633 = {S89950, 137, { &P20013,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F180K Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2860948 = {S89951, 137, { &P20013,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F180L Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2863811 = {S89952, 137, { &P20013,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F180S Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2751481 = {S89953, 139, { &P18654,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; WX04K Build/324.4.0000) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2752373 = {S89954, 155, { &P14574,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Nexus S Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2763785 = {S89955, 142, { &P23341,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BLU Touch Book 9.7 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2799112 = {S89956, 160, { &P13018,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.4; MT11i Build/4.0.2.A.0.62) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2773929 = {S89957, 142, { &P22737,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Micromax A115 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2781725 = {S89958, 137, { &P23272,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenco TAB-900 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2781890 = {S89959, 162, { &P15798,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2782040 = {S89960, 144, { &P22933,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Avea inTouch 2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2782815 = {S89961, 131, { &P21546,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; V972 Core4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2783188 = {S89962, 134, { &P17283,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MB860 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2795700 = {S89963, 95, { &P22747,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; PIONEER_P1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2796072 = {S89964, 133, { &P23704,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M500W Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2800230 = {S89965, 165, { &P12814,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I777 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2860095 = {S89966, 170, { &P15991,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2 jellybean # ariel-tbc /UCHIHA; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2861216 = {S89967, 98, { &P21882,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Micromax_P362) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2863673 = {S89968, 135, { &P23502,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; Fly IQ237 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2866407 = {S89969, 152, { &P18466,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; KFTT Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2866574 = {S89970, 93, { &P18595,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8190) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2879702 = {S89971, 154, { &P21094,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A789 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2892496 = {S89972, 151, { &P21064,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A44 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2999523 = {S89973, 96, { &P15364,&P17285,&P21781,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.21 (KHTML, like Gecko) QupZilla/1.4.3 Safari/537.21*/
const Device UA2958100 = {S89974, 156, { &P15837,&P6909,&P24677,&P18092 }};	/*Useragent - NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax2-05) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2945150 = {S89975, 159, { &P17625,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST25i Build/6.1.1.B.1.10) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2955867 = {S89976, 152, { &P24073,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033M Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2957736 = {S89977, 97, { &P22862,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Micromax_A88) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2958349 = {S89978, 133, { &P24171,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; S735 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2961371 = {S89979, 156, { &P18509,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LG-MS770 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2961998 = {S89980, 151, { &P23846,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A12 Build/RheaTK) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2964713 = {S89981, 97, { &P22903,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A62) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3056361 = {S89982, 150, { &P15985,&P6437,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 UCBrowser/2.3.2.300*/
const Device UA2999614 = {S89983, 143, { &P20129,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3121162 = {S89984, 144, { &P20129,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2S Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2999619 = {S89985, 137, { &P18621,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E610 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3073683 = {S89986, 137, { &P18621,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E612 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3005241 = {S89987, 159, { &P23015,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Lenovo A820 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3010024 = {S89988, 160, { &P23015,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Lenovo A820t Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3023456 = {S89989, 139, { &P15079,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MK16i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020572 = {S89990, 141, { &P22858,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U8836D Build/HuaweiU8836D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022281 = {S89991, 137, { &P24149,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Aqua I-5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023878 = {S89992, 160, { &P23008,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax A91 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3054828 = {S89993, 148, { &P20090,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ph; MyPhone A848i Duo Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3105989 = {S89994, 148, { &P20090,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MyPhone A848g Duo Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3095522 = {S89995, 134, { &P22847,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SH-06E Build/S7190) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3079414 = {S89996, 97, { &P18649,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A57) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3079459 = {S89997, 98, { &P21020,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Micromax_A116) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3083585 = {S89998, 133, { &P20427,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHL21 Build/S5170) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3080203 = {S89999, 97, { &P20005,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A65) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3093238 = {S90000, 151, { &P23207,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A4+ Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3093415 = {S90001, 93, { &P13974,&P5096,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5670) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3093550 = {S90002, 151, { &P20231,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A21 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3094135 = {S90003, 155, { &P22854,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; GT-I9508 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3112423 = {S90004, 129, { &P21853,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Connect-3G Build/HCL) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3108355 = {S90005, 156, { &P15786,&P5043,&P24679,&P18092 }};	/*Useragent - NokiaC2-05/2.0 (08.69) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac2-05) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3108973 = {S90006, 145, { &P12972,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-T989 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3126252 = {S90007, 145, { &P12972,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-T989 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3115114 = {S90008, 158, { &P18564,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One X+ Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3131329 = {S90009, 150, { &P18552,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA61991 = {S90010, 60, { &P12152,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia6822/2.0 (4.52) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21872 = {S90011, 72, { &P12013,&P5448,&P17173,&P18092 }};	/*Useragent - SAMSUNG-SGH-I607/UCHA1 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA23965 = {S90012, 105, { &P15640,&P17484,&P3636,&P18092 }};	/*Useragent - SAMSUNG-GT-S5600L/1.0 SHP/VPP/R5 Jasmine/0.8 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA24760 = {S90013, 98, { &P15755,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-Z230/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA61687 = {S90014, 80, { &P14165,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonS700i/R3F SEMC-Browser/4.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62506 = {S90015, 80, { &P14165,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonS700i/R3B SEMC-Browser/4.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63587 = {S90016, 80, { &P14165,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonS700i/R3M SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62609 = {S90017, 101, { &P14426,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-M110/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA50147 = {S90018, 81, { &P11700,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW810i/R4EA Java/SEMC-Java/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA51230 = {S90019, 82, { &P11700,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW810iv/R4CE Java/SEMC-Java/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA83706 = {S90020, 16, { &P11700,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW810*/
const Device UA50232 = {S90021, 10, { &P14782,&P5890,&P4030,&P18092 }};	/*Useragent - NokiaE61-1*/
const Device UA67209 = {S90022, 8, { &P14782,&P5890,&P4030,&P18092 }};	/*Useragent - NOKIAE61*/
const Device UA68137 = {S90023, 9, { &P14782,&P5890,&P4030,&P18092 }};	/*Useragent - NOKIAE61i*/
const Device UA54246 = {S90024, 117, { &P14218,&P17605,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C3300K/1.0 RTK-E/1.0 DF/1.0 Release/22.04.2010 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA56134 = {S90025, 31, { &P17720,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N03A(c100;TB;W24H16)*/
const Device UA64119 = {S90026, 70, { &P13199,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHB520/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64578 = {S90027, 111, { &P13199,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHB520/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA66557 = {S90028, 73, { &P16392,&P5448,&P22271,&P18092 }};	/*Useragent - HTC-8900/1.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA684986 = {S90029, 99, { &P14826,&P5843,&P4777,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (05.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac1-01/UC Browser7.7.1.88/69/444*/
const Device UA90077 = {S90030, 133, { &P17622,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; HERO200 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA141234 = {S90031, 135, { &P17622,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HERO200 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA298722 = {S90032, 136, { &P17622,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HERO200 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA292576 = {S90033, 71, { &P18088,&P5448,&P4715,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65*/
const Device UA305959 = {S90034, 11, { &P16869,&P22784,&P17470,&P18092 }};	/*Useragent - MicromaxQ3+*/
const Device UA2409818 = {S90035, 68, { &P13964,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2;Zh_cn;GT-I5510;240*400;)UCWEB7.8.0.95/139/444*/
const Device UA2541005 = {S90036, 81, { &P15364,&P17285,&P20901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; de-DE; rv:1.4) Gecko/X Netscape/7.1 (ax)*/
const Device UA545868 = {S90037, 80, { &P17595,&P17295,&P22181,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:2.2a1pre) Gecko/X Firefox/4.2a1pre*/
const Device UA547678 = {S90038, 120, { &P17595,&P17296,&P22889,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) iCab/4.8 Safari/533.16*/
const Device UA710855 = {S90039, 120, { &P17595,&P17296,&P22889,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) iCab/4.8 Safari/533.16*/
const Device UA1962418 = {S90040, 110, { &P17595,&P17296,&P22889,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) iCab/4.8 Safari/533.16*/
const Device UA717973 = {S90041, 98, { &P11625,&P5768,&P17347,&P18092 }};	/*Useragent - Nokia5300/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia5300/UC Browser8.0.3.107/69/444*/
const Device UA583929 = {S90042, 94, { &P17595,&P17297,&P18192,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:9.0) Gecko/X Thunderbird/9.0.1 Lightning/1.1.1*/
const Device UA654984 = {S90043, 78, { &P15364,&P22740,&P20445,&P18092 }};	/*Useragent - Mozilla/5.0 (Maemo; Linux armv7l; rv:2.0.1) Gecko/X Firefox/4.0.1 Fennec/2.0.1*/
const Device UA726542 = {S90044, 133, { &P11747,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC Desire; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2166339 = {S90045, 141, { &P11747,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC Desire Mefisto; en-gb) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA728369 = {S90046, 85, { &P15364,&P17017,&P4860,&P18092 }};	/*Useragent - opera/9.80 (j2me/midp; opera mini/6.24093/26.958; u; en) presto/2.8.119 version/10.54*/
const Device UA785745 = {S90047, 64, { &P15364,&P5690,&P18100,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko/X Firefox/11.0 slurp*/
const Device UA839044 = {S90048, 70, { &P15364,&P5690,&P18100,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko/X Firefox/11.0 slurp slurp*/
const Device UA831495 = {S90049, 69, { &P15364,&P17285,&P18105,&P18092 }};	/*Useragent - Opera/9.52 (Windows NT 6.0; U; Opera/9.52 (X11; Linux x86_64; U); en)*/
const Device UA2032394 = {S90050, 144, { &P22019,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; LINSAY COSMOS F-10HD Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2034179 = {S90051, 84, { &P18510,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;ALCATEL ONE TOUCH 918N;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA2176714 = {S90052, 136, { &P17595,&P17295,&P22460,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1150.1  Iron/20.0.1150.1 Safari/536.11*/
const Device UA2686641 = {S90053, 162, { &P12139,&P5204,&P24670,&P18092 }};	/*Useragent - Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia5310xpressmusic) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2342899 = {S90054, 135, { &P21017,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; P-07D Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2350172 = {S90055, 175, { &P16595,&P5477,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; Micromax A60 Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2370480 = {S90056, 88, { &P12191,&P5448,&P4779,&P18092 }};	/*Useragent - garmin-asus-Nuvifone-M10/2.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA2494079 = {S90057, 88, { &P12191,&P5448,&P4779,&P18092 }};	/*Useragent - garmin-asus-Nuvifone-M10/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA2382405 = {S90058, 126, { &P15302,&P17485,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; sonyericssonwt13i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile*/
const Device UA2982524 = {S90059, 123, { &P15302,&P17485,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; fa; sonyericssonwt13i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile*/
const Device UA2384295 = {S90060, 155, { &P13018,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; MT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2395748 = {S90061, 126, { &P18017,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; K1 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2398992 = {S90062, 143, { &P21994,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Teclast P85??(A9D3) Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2402128 = {S90063, 161, { &P18652,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Micromax A87 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2423613 = {S90064, 128, { &P20270,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U20GT Build/U20GT) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2435983 = {S90065, 38, { &P16463,&P17484,&P20765,&P18092 }};	/*Useragent - SAMSUNG-GT-I8350/UCWEB2.7.0.223/47/355*/
const Device UA2457802 = {S90066, 135, { &P18663,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SH-10D Build/SB010) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2485608 = {S90067, 155, { &P18174,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; VS920 4G Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2506925 = {S90068, 146, { &P20509,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A888 Duo Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2579514 = {S90069, 151, { &P18644,&P5043,&P24675,&P18092 }};	/*Useragent - Nokia112/2.0 (03.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia112) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2550949 = {S90070, 194, { &P11787,&P5204,&P24671,&P18092 }};	/*Useragent - Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia3120classic) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2553372 = {S90071, 138, { &P17595,&P18207,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.68.1565 Chrome/14.0.835.202 Safari/535.1*/
const Device UA3050673 = {S90072, 138, { &P17595,&P18207,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.68.1565 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2580666 = {S90073, 161, { &P15991,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000*/
const Device UA2583967 = {S90074, 134, { &P15877,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT910 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2714154 = {S90075, 161, { &P17149,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2600342 = {S90076, 92, { &P21205,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Andi107) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2600633 = {S90077, 136, { &P23741,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; PHCASIATAB9D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2600973 = {S90078, 132, { &P15102,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ605 Build/I.7.1-39.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2604079 = {S90079, 140, { &P15079,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MK16i Build/4.1.B.0.587) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2607338 = {S90080, 134, { &P21963,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Q800 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2617615 = {S90081, 55, { &P17323,&P17979,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.6939 Android/4.0.4 Device/samsung-GT-P6800*/
const Device UA2624355 = {S90082, 150, { &P14905,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; nl-nl; GT-P7100 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2628776 = {S90083, 150, { &P14905,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; nl-nl; GT-P7100 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2631467 = {S90084, 149, { &P14905,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; nl-nl; GT-P7100 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2631582 = {S90085, 152, { &P17337,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2636712 = {S90086, 151, { &P17337,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA3101820 = {S90087, 155, { &P17337,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA2653960 = {S90088, 81, { &P15988,&P17484,&P17374,&P18092 }};	/*Useragent - SAMSUNG-GT-C3312 Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2660106 = {S90089, 167, { &P18661,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; N860 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2660251 = {S90090, 92, { &P13536,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA6110) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2668866 = {S90091, 159, { &P16292,&P18598,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; Galaxy Nexus Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2704927 = {S90092, 160, { &P16292,&P18598,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-CN; Galaxy Nexus Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2672979 = {S90093, 140, { &P21181,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S560 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2693138 = {S90094, 125, { &P14985,&P17484,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s8530) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2702219 = {S90095, 138, { &P23196,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; DIGMA iDsD8 3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2702776 = {S90096, 90, { &P12824,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; U8180) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2708969 = {S90097, 127, { &P21222,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT1S0 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2709757 = {S90098, 149, { &P20841,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 10 Build/1.1.11-1015 X-10:30) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2863006 = {S90099, 152, { &P20841,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 10 3G Build/1.1.11-1015 X-16:17) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2712649 = {S90100, 135, { &P23165,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; H7500+ Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2733170 = {S90101, 167, { &P20094,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2733701 = {S90102, 167, { &P20094,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI C8825D Build/HuaweiC8825D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2735013 = {S90103, 129, { &P18437,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N8000 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2751289 = {S90104, 147, { &P20931,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; iBall Slide 3G7271 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2865933 = {S90105, 141, { &P20931,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; iBall Slide 3G 7334 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2752811 = {S90106, 165, { &P18113,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2764460 = {S90107, 87, { &P14202,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; X8) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2952668 = {S90108, 89, { &P14202,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; E15i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2768705 = {S90109, 158, { &P18214,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2778190 = {S90110, 150, { &P18227,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI U9510E Build/HuaweiU9510E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781708 = {S90111, 93, { &P14186,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2783294 = {S90112, 156, { &P18687,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-P705 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2783693 = {S90113, 137, { &P22954,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E455f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799254 = {S90114, 159, { &P15147,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST15i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2794967 = {S90115, 168, { &P21356,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; TECNO N3 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/33567*/
const Device UA2797299 = {S90116, 136, { &P20226,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-E975 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798902 = {S90117, 137, { &P14959,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9103 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3099598 = {S90118, 126, { &P21144,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; F8 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2858174 = {S90119, 138, { &P23778,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; my-mm; SGT-X41 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2862096 = {S90120, 171, { &P21402,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; Lenovo S890 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/33482*/
const Device UA2882264 = {S90121, 159, { &P14993,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LT15i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2868092 = {S90122, 138, { &P21019,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E270K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869207 = {S90123, 158, { &P18606,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2879527 = {S90124, 151, { &P18606,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2869709 = {S90125, 147, { &P23323,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Archos 50 Platinum Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877476 = {S90126, 132, { &P19860,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A9+ Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893018 = {S90127, 160, { &P18505,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2893952 = {S90128, 161, { &P20170,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A95 Build/CelkonCelkon_A95) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2937469 = {S90129, 136, { &P23957,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; FPT V Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958018 = {S90130, 138, { &P18151,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; Paladin Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2960815 = {S90131, 156, { &P21026,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; ZTE U930 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2962032 = {S90132, 46, { &P15364,&P18580,&P18245,&P18091 }};	/*Useragent - Tweetbot/28201 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3058735 = {S90133, 46, { &P15364,&P18580,&P18245,&P18091 }};	/*Useragent - Tweetbot/28300 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2962186 = {S90134, 103, { &P15798,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC Explorer A310e) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2964024 = {S90135, 161, { &P21020,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Micromax A116 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3022606 = {S90136, 131, { &P20004,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A2 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2991800 = {S90137, 130, { &P22873,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5120 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3000045 = {S90138, 150, { &P24078,&P18598,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; E3 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3009062 = {S90139, 160, { &P12513,&P17485,&P24678,&P18092 }};	/*Useragent - SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw20) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3011511 = {S90140, 132, { &P21648,&P18598,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.2; en-gb; GT-I9505 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3011677 = {S90141, 140, { &P23498,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZTE Blade G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3023021 = {S90142, 128, { &P24400,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SM-T315 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023237 = {S90143, 123, { &P21239,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; M9 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023413 = {S90144, 140, { &P22021,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KHPHN05ANDA Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3079874 = {S90145, 159, { &P21991,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Aqua Wonder Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3079622 = {S90146, 158, { &P15147,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST15i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3026387 = {S90147, 150, { &P21334,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A8 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3080072 = {S90148, 161, { &P13452,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3029539 = {S90149, 155, { &P15649,&P5096,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5570 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3030089 = {S90150, 90, { &P22641,&P18111,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; C1504) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3050075 = {S90151, 136, { &P24570,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; ZP910 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3052036 = {S90152, 138, { &P15575,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3090791 = {S90153, 138, { &P15575,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3058530 = {S90154, 93, { &P22382,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8262) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3059053 = {S90155, 99, { &P21842,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-e2202) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3059107 = {S90156, 150, { &P22854,&P18598,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9508 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3059596 = {S90157, 169, { &P19762,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI MT1-U06 Build/HuaweiMT1-U06) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3111517 = {S90158, 141, { &P20091,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5880D Build/PMP5880D_v1.0.0_X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3061716 = {S90159, 93, { &P15575,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9100) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3094432 = {S90160, 143, { &P19891,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SAMSUNG-SGH-I717 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3062197 = {S90161, 93, { &P14359,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5228) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3083718 = {S90162, 158, { &P22756,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; C1604 Build/11.1.A.0.68) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109411 = {S90163, 140, { &P19999,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A30 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093911 = {S90164, 131, { &P23704,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M500W Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3095525 = {S90165, 152, { &P20868,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M040 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3113637 = {S90166, 130, { &P23218,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PMP5780D Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3115504 = {S90167, 156, { &P22585,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L720 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3131611 = {S90168, 138, { &P18606,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132965 = {S90169, 131, { &P18606,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA64029 = {S90170, 53, { &P14896,&P17486,&P3402,&P18092 }};	/*Useragent - MOT-W270/0.0.42 UP.Browser/6.3.0.6.c.23 (GUI) MMP/2.0*/
const Device UA4821 = {S90171, 94, { &P12194,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Touch2_T3320 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA54643 = {S90172, 94, { &P12194,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Touch2_T3330 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA51936 = {S90173, 73, { &P12164,&P5810,&P3417,&P18092 }};	/*Useragent - BlackBerry8707/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/245*/
const Device UA18555 = {S90174, 116, { &P14998,&P5448,&P22272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) 320x240; VZW; Motorola-Q9c; Windows Mobile 6.0 Standard*/
const Device UA22082 = {S90175, 59, { &P15381,&P22839,&P17470,&P18092 }};	/*Useragent - ZTE-R220/1.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62037 = {S90176, 115, { &P14362,&P17484,&P3834,&P18092 }};	/*Useragent - SAMSUNG-GT-C5180/C5180ZWJL3 SHP/VPP/R5 Dolfin/2.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA64411 = {S90177, 107, { &P14597,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-T219/T219UVFL7 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA30667 = {S90178, 101, { &P11757,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E390/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA32462 = {S90179, 45, { &P12485,&P17483,&P3625,&P18092 }};	/*Useragent - LG-LX165/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA50459 = {S90180, 32, { &P13259,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P903i(c100;TB;W24H12)*/
const Device UA52782 = {S90181, 32, { &P14600,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F905i(c100;TB;W24H17)*/
const Device UA56160 = {S90182, 32, { &P14762,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N706i(c100;TB;W24H16)*/
const Device UA67126 = {S90183, 33, { &P14762,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N706i2(c100;TB;W24H16)*/
const Device UA67210 = {S90184, 54, { &P14783,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SN3F UP.Browser/6.2_7.2.7.1.K.2.234 (GUI) MMP/2.0*/
const Device UA68171 = {S90185, 34, { &P15373,&P5129,&P4777,&P18092 }};	/*Useragent - Nokia5530/UCBrowser7.7.0.81/50/999*/
const Device UA68629 = {S90186, 35, { &P15373,&P5129,&P4777,&P18092 }};	/*Useragent - Nokia5530/UC Browser7.7.1.88/50/444*/
const Device UA90303 = {S90187, 111, { &P18086,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; Microsoft; XDeviceEmulator)*/
const Device UA151772 = {S90188, 50, { &P16225,&P5222,&P3430,&P18092 }};	/*Useragent - Opera/9.5 (Windows NT 5.1; U; en) SAMSUNG SCH-i920*/
const Device UA250456 = {S90189, 122, { &P15364,&P17285,&P22411,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.1.1.0 Chrome/14.0.835.202 Safari/535.1*/
const Device UA1904165 = {S90190, 129, { &P15364,&P17285,&P22411,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.1.1.0 Chrome/14.0.835.202 Safari/535.1*/
const Device UA759041 = {S90191, 137, { &P22391,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; tr-tr; HT_PAD1051 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA350743 = {S90192, 124, { &P15364,&P5690,&P24208,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.8.10793.201*/
const Device UA351384 = {S90193, 99, { &P15364,&P5690,&P24208,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727) QQBrowser/6.8.10898.201*/
const Device UA352870 = {S90194, 87, { &P15364,&P5690,&P24208,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0) QQBrowser/6.8.10898.201*/
const Device UA698950 = {S90195, 154, { &P13123,&P5629,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1947007 = {S90196, 154, { &P13123,&P5629,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA379663 = {S90197, 41, { &P13554,&P5890,&P24663,&P18092 }};	/*Useragent - NOKIAN79/UC Browser7.9.0.102/28/355/UCWEB*/
const Device UA675227 = {S90198, 42, { &P14359,&P5129,&P24665,&P18092 }};	/*Useragent - Nokia5228/UC Browser8.0.4.121/50/444/UCWEB*/
const Device UA713352 = {S90199, 99, { &P14635,&P5843,&P4759,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (07.20) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac3-00/UC Browser7.6.1.82/69/351*/
const Device UA573495 = {S90200, 37, { &P14358,&P5890,&P17338,&P18092 }};	/*Useragent - nokiae63-1/UC Browser7.9.0.102/69/352*/
const Device UA589197 = {S90201, 79, { &P15364,&P17285,&P20460,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:10.0) Gecko/X Firefox/10.0 SeaMonkey/2.7*/
const Device UA651200 = {S90202, 66, { &P15364,&P5690,&P18097,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:9.0.1) Gecko/X Firefox/9.0.1 slurp*/
const Device UA780707 = {S90203, 72, { &P15364,&P5690,&P18097,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:9.0.1) Gecko/X Firefox/9.0.1 slurp slurp*/
const Device UA703630 = {S90204, 140, { &P17595,&P17297,&P22511,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.1.6) Gecko/X FireDownload/1.2pre Firefox/3.5.6 Wyzo/3.5.6.1 FireTorrent/1.1pre*/
const Device UA721813 = {S90205, 45, { &P12707,&P5055,&P17347,&P18092 }};	/*Useragent - nokia701/UC Browser8.0.3.107/69/444  3gpp-gba*/
const Device UA760051 = {S90206, 95, { &P15364,&P7127,&P22482,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6.1*/
const Device UA889246 = {S90207, 153, { &P12788,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PG86100 Build/IML74K; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1996180 = {S90208, 153, { &P12788,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PG86100 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027996 = {S90209, 156, { &P12788,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Evo 3D GSM Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027582 = {S90210, 131, { &P22439,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-eg; SC3071N Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2445034 = {S90211, 131, { &P22439,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; SC3071n Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2031988 = {S90212, 148, { &P22129,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; PB-S1 Build/ALPS.GB.FDD2.MP.V3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2787083 = {S90213, 154, { &P12706,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-P690 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2160617 = {S90214, 44, { &P15302,&P17485,&P17347,&P18092 }};	/*Useragent - sonyericssonwt13i/UC Browser8.0.3.107/69/444*/
const Device UA2654567 = {S90215, 53, { &P15364,&P18570,&P18369,&P18092 }};	/*Useragent - OperaMini/7.0.5.45389 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2207320 = {S90216, 75, { &P15364,&P7133,&P20918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.6) Gecko/X Firefox/0.8*/
const Device UA2209245 = {S90217, 73, { &P15364,&P7133,&P20918,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; rv:1.7.3) Gecko/X Firefox/0.10.1*/
const Device UA2212232 = {S90218, 80, { &P13398,&P5448,&P4777,&P18092 }};	/*Useragent - SAMSUNG-GT-B7610/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2761021 = {S90219, 154, { &P14557,&P5843,&P24670,&P18092 }};	/*Useragent - Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia6303classic) UCBrowser8.4.0.159/70/444/UCWEB Mobile*/
const Device UA2337199 = {S90220, 147, { &P24015,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; en-ph; ICE+Turbo Build/CloudfoneICE+Turbo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2351442 = {S90221, 186, { &P12513,&P17485,&P24672,&P18092 }};	/*Useragent - SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonw20) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile*/
const Device UA2383435 = {S90222, 33, { &P18267,&P5043,&P21724,&P18092 }};	/*Useragent - OneBrowser/3.0 (NokiaX2-02/11.57)*/
const Device UA2385621 = {S90223, 69, { &P17346,&P18117,&P22071,&P18092 }};	/*Useragent - Instagram 3.1.2 (iPod touch; iPhone OS 5.1.1; en_US) AppleWebKit/420+*/
const Device UA2403510 = {S90224, 171, { &P18283,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I535 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SCH-I535, 36, 1.7.1)*/
const Device UA2825431 = {S90225, 171, { &P18283,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (GT-I9300, 34, 1.6.0)*/
const Device UA2400821 = {S90226, 124, { &P12142,&P17485,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; sonyericssonu5i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile*/
const Device UA2477816 = {S90227, 121, { &P12142,&P17485,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; fa; sonyericssonu5i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile*/
const Device UA2403750 = {S90228, 139, { &P21941,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; Allview3SpeedDuo Build/SV1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2410788 = {S90229, 96, { &P18219,&P5843,&P17338,&P18092 }};	/*Useragent - Nokia202/2.0 (20.28) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia202/UC Browser7.9.0.102/69/352*/
const Device UA2416330 = {S90230, 155, { &P15575,&P18066,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423850 = {S90231, 155, { &P18680,&P18066,&P18690,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.2 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2424279 = {S90232, 136, { &P21289,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; FreeTAB 9701 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2427157 = {S90233, 162, { &P18300,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-T769 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2547876 = {S90234, 76, { &P21663,&P18110,&P20793,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; NOVO7_Tornados; 800*444) UCWEB1.0.1.113/147/800*/
const Device UA2450919 = {S90235, 125, { &P15197,&P17484,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-b5310) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile*/
const Device UA2457954 = {S90236, 99, { &P13183,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonk810i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2840927 = {S90237, 99, { &P13183,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonk810i) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2462594 = {S90238, 73, { &P15364,&P18144,&P17825,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/3.5; OpenBSD) KHTML/3.5.9 (like Gecko)*/
const Device UA2466237 = {S90239, 132, { &P20149,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6211 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2469421 = {S90240, 165, { &P18385,&P18598,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; Nexus 7 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2479840 = {S90241, 145, { &P23473,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; RMD-1035 Build/ICS.g12refM1005RU.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2482071 = {S90242, 157, { &P22875,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2487233 = {S90243, 168, { &P18448,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST26i Build/11.0.A.3.28) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2528558 = {S90244, 137, { &P23550,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE T81 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2554666 = {S90245, 159, { &P13042,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Incredible S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2842358 = {S90246, 159, { &P13042,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Incredible S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2558713 = {S90247, 95, { &P13443,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; SamsungI8910) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2559870 = {S90248, 164, { &P23546,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; AD683G Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2577900 = {S90249, 156, { &P13018,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; MT11i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2836749 = {S90250, 156, { &P13018,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; MT11i Build/4.1.B.0.431) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2581461 = {S90251, 139, { &P20737,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2595353 = {S90252, 137, { &P21455,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; DM013SH Build/S0006) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2596016 = {S90253, 45, { &P23091,&P22831,&P17470,&P18092 }};	/*Useragent - P16i; Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2602995 = {S90254, 140, { &P12788,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Evo 3D CDMA Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667995 = {S90255, 139, { &P12788,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Evo 3D GSM Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2615924 = {S90256, 90, { &P17337,&P17979,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7500; GT-P7500; p3; en_GB)*/
const Device UA2622963 = {S90257, 95, { &P13452,&P17979,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.0 Android (8/2.2; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; GT-P1000; en_GB)*/
const Device UA2622966 = {S90258, 98, { &P13452,&P17979,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.0 Android (10/2.3.3; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; gt-p1000; en_GB)*/
const Device UA2623449 = {S90259, 99, { &P15919,&P17979,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.0 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P3100; espressorf; espresso; en_US)*/
const Device UA2653698 = {S90260, 134, { &P18464,&P18110,&P19815,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Safari/535.19 Silk-Accelerated=false*/
const Device UA2654672 = {S90261, 145, { &P22675,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-uk; GOCLEVER NETBOOK R103 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2666766 = {S90262, 135, { &P23981,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; 9300+ Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667033 = {S90263, 130, { &P18437,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N8000 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667307 = {S90264, 138, { &P20173,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Limo X2+ Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2863217 = {S90265, 137, { &P20173,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; Limo X2 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2670899 = {S90266, 144, { &P20168,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Maxwell Plus Build/1.0.3 X-14:07) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2674348 = {S90267, 15, { &P15364,&P17979,&P17470,&P18092 }};	/*Useragent - Podkicker/1.2.4*/
const Device UA2677222 = {S90268, 140, { &P21526,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cherry w500 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2682200 = {S90269, 97, { &P15364,&P16110,&P22479,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.8.1.16) Gecko/X Firefox/2.0.0.16 Flock/1.2.4*/
const Device UA2684941 = {S90270, 151, { &P24649,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; GFIVE A68 Build/A68-V1.0-2012.05.29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2722667 = {S90271, 151, { &P24649,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; GFIVE A68 Build/A68-V1.0-2012.03.17) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2696104 = {S90272, 147, { &P15798,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Explorer A310e Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2828094 = {S90273, 146, { &P15798,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Explorer A310 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2696974 = {S90274, 68, { &P18687,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; LG-P705; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2700165 = {S90275, 142, { &P22683,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Beeline E600 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2700323 = {S90276, 145, { &P18324,&P5629,&P21783,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/Sleipnir 2.3.9) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2701018 = {S90277, 145, { &P18324,&P5629,&P21783,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/Sleipnir 2.3.8) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2700388 = {S90278, 88, { &P20529,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A89) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2702864 = {S90279, 161, { &P13999,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC HD2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2713163 = {S90280, 137, { &P20081,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7108 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2997710 = {S90281, 137, { &P20081,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N7108 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713575 = {S90282, 134, { &P23981,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; 9300+ Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713580 = {S90283, 140, { &P15880,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ME865 Build/6.7.2_GC-384) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734626 = {S90284, 138, { &P20132,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST70104-1 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734654 = {S90285, 134, { &P21112,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; X9017 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2750957 = {S90286, 133, { &P21112,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; X907 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734901 = {S90287, 133, { &P21933,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M729 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2738004 = {S90288, 131, { &P15767,&P18117,&P21783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.3.2m*/
const Device UA2751371 = {S90289, 129, { &P15767,&P18117,&P21783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.3m*/
const Device UA2749279 = {S90290, 133, { &P23416,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; TM-9747BT Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2762535 = {S90291, 39, { &P14557,&P5843,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia6303classic/10.12)*/
const Device UA2773162 = {S90292, 152, { &P20541,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAP4500DUO Build/PrestigioPAP4500DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783586 = {S90293, 140, { &P21703,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 70 Titanium Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2790564 = {S90294, 140, { &P23375,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HUAWEI Y 220T Build/HuaweiY220T) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2848498 = {S90295, 71, { &P23375,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; HUAWEI Y 220T Build/HuaweiY220T)*/
const Device UA2792930 = {S90296, 152, { &P22990,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-STYLE Q 5A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2796875 = {S90297, 129, { &P23236,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AMI-TT4 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2797001 = {S90298, 159, { &P18496,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LVW Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2860815 = {S90299, 128, { &P20187,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Novo7Fire Build/Xtm) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2865931 = {S90300, 99, { &P21269,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Intex Aqua 3.2) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2867231 = {S90301, 138, { &P21489,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Rezound Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867238 = {S90302, 158, { &P12968,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.7; LT26i Build/6.0.A.3.62) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2869206 = {S90303, 157, { &P18641,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-M830 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2869728 = {S90304, 141, { &P22862,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Micromax A88 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877860 = {S90305, 138, { &P20128,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT-AS43D2 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879310 = {S90306, 138, { &P20128,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT-AS43D2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879211 = {S90307, 169, { &P22864,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A26 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2879227 = {S90308, 92, { &P23011,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; iris405) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2879568 = {S90309, 149, { &P16976,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT300SE Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2897415 = {S90310, 147, { &P16976,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT300 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2879710 = {S90311, 138, { &P22969,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT915 Build/2_32A_2008) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2898857 = {S90312, 151, { &P22611,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-S7572 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2936371 = {S90313, 137, { &P22952,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F260S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2936773 = {S90314, 61, { &P16436,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-I8150 Build/JDQ39E)*/
const Device UA3072117 = {S90315, 138, { &P16436,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I8150 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2960652 = {S90316, 155, { &P22874,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; GT-I9502 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2945156 = {S90317, 152, { &P12765,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2971888 = {S90318, 156, { &P22020,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9268 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3003608 = {S90319, 155, { &P18507,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I8160 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3059597 = {S90320, 161, { &P16292,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3015527 = {S90321, 144, { &P18639,&P18598,&P23369,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Nexus 10 Build/JOP40D) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229 Safari/537.4 pss-webkit-request*/
const Device UA3060878 = {S90322, 147, { &P17926,&P6437,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.2.1; AT100 Build/HTK55D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3015678 = {S90323, 157, { &P18458,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3021823 = {S90324, 134, { &P23252,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; M6589 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3059702 = {S90325, 166, { &P19863,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3021366 = {S90326, 138, { &P20925,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT-AS45SE Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020493 = {S90327, 157, { &P21597,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; Mobiistar Touch Lai 502 HD Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3020624 = {S90328, 136, { &P20200,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS GAMEPAD Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021977 = {S90329, 170, { &P21628,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8552 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3049355 = {S90330, 166, { &P21628,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I8552 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3023471 = {S90331, 127, { &P24624,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; A1-811 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3024815 = {S90332, 97, { &P11747,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire V) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3093590 = {S90333, 97, { &P11747,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire V) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3034443 = {S90334, 138, { &P15649,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3104177 = {S90335, 139, { &P15649,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3056402 = {S90336, 141, { &P19778,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PantechP9090 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3058928 = {S90337, 93, { &P12568,&P5096,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; GT-B7510) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3059066 = {S90338, 93, { &P16808,&P22784,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq7) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3062712 = {S90339, 137, { &P20508,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Cynus T2 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3093645 = {S90340, 136, { &P20200,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS GAMEPAD Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3078649 = {S90341, 152, { &P15575,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9100G Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3125568 = {S90342, 151, { &P15575,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3079356 = {S90343, 174, { &P19779,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A1+ Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3079487 = {S90344, 162, { &P19970,&P17485,&P24679,&P18092 }};	/*Useragent - SonyEricssonG502/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssong502) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3079734 = {S90345, 93, { &P22382,&P18111,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8262) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3114924 = {S90346, 141, { &P24688,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Spice Mi-515 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3115105 = {S90347, 102, { &P15775,&P21697,&P24641,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 800) U2/1.0.0 UCBrowser/3.1.1.343 U2/1.0.0 Mobile*/
const Device UA3110015 = {S90348, 76, { &P17346,&P22882,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.11A465 (iPod touch; U; CPU OS 7_0 like Mac OS X; en_us)*/
const Device UA3131024 = {S90349, 78, { &P17346,&P22882,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.11A501 (iPod touch; U; CPU OS 7_0_2 like Mac OS X; en_us)*/
const Device UA3114790 = {S90350, 128, { &P16044,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Touchpad Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3128162 = {S90351, 128, { &P16044,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; TouchPad Build/JLS36G) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3130936 = {S90352, 138, { &P23227,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I467 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA2434 = {S90353, 128, { &P16054,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320; HTC_P3600/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA1866096 = {S90354, 119, { &P11742,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-C5212/C5212DDIK1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA2810497 = {S90355, 124, { &P11742,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-C5212i/C5212iJVJI1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA10239 = {S90356, 81, { &P13483,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x240; HP iPAQ hw6500/1.0)*/
const Device UA52436 = {S90357, 80, { &P13483,&P5448,&P4578,&P18092 }};	/*Useragent - HP iPAQ hw6500/1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x240)*/
const Device UA69564 = {S90358, 77, { &P13483,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x240; HP iPAQ hw6500)*/
const Device UA16693 = {S90359, 52, { &P13916,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-C261/0.1.66 UP.Browser/6.2.3.9.c.9 (GUI) MMP/2.0*/
const Device UA18860 = {S90360, 136, { &P15068,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHB308/1.0 Release/2.28.2008 Browser/Openwave6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA20324 = {S90361, 111, { &P15068,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHB300/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA20579 = {S90362, 44, { &P12112,&P17484,&P4970,&P18092 }};	/*Useragent - SCH-U450-parrot/1.0 AU-MIC/2.0 (GUI) MMP/2.0*/
const Device UA25394 = {S90363, 37, { &P12112,&P17484,&P4970,&P18092 }};	/*Useragent - SCH-U450/1.0 AU-MIC/2.0 (GUI) MMP/2.0*/
const Device UA21653 = {S90364, 88, { &P20393,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-T165+ NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 firmware/T165+ T05*/
const Device UA24686 = {S90365, 11, { &P15581,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M5570*/
const Device UA43784 = {S90366, 119, { &P13745,&P5208,&P4628,&P18092 }};	/*Useragent - POLARIS/6.01 (BREW 3.1.5; U; en-us; LG; POLARIS/6.01/WAP) Sprint LX290S MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA49875 = {S90367, 142, { &P13097,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; in-id; Axioo-VIGO350 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA64643 = {S90368, 60, { &P14689,&P17017,&P3976,&P18092 }};	/*Useragent - Mozilla/4.0 (MobilePhone PL3800/US/1.0) NetFront/3.4 MMP/2.0*/
const Device UA67865 = {S90369, 138, { &P14878,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; SH-10B Build/S1110) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA90744 = {S90370, 138, { &P14878,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; SH-10B Build/S7023) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA443833 = {S90371, 138, { &P14878,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; SH-10B Build/S9190) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA1936933 = {S90372, 138, { &P14878,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; SH-10B Build/SB080) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA68992 = {S90373, 36, { &P14983,&P5055,&P4777,&P18092 }};	/*Useragent - NokiaC7-00/UC Browser7.7.1.88/50/444*/
const Device UA556308 = {S90374, 46, { &P14983,&P5055,&P4777,&P18092 }};	/*Useragent - nokiac7-00/UC Browser7.7.1.88/69/444  3gpp-gba*/
const Device UA571844 = {S90375, 116, { &P12056,&P5204,&P17338,&P18092 }};	/*Useragent - Nokia7210Supernova/2.0 (06.51) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7210supernova/UC Browser7.9.0.102/69/352*/
const Device UA2977461 = {S90376, 110, { &P15364,&P22740,&P22744,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux armv7l; fr-FR; rv:1.9.2a1pre) Gecko/X Firefox/3.5 Maemo Browser 1.4.1.22 RX-51 N900*/
const Device UA250605 = {S90377, 143, { &P15201,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Sensation 4G Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA687185 = {S90378, 147, { &P15201,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MIUI.us Sensation 4G Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA252702 = {S90379, 138, { &P17595,&P17297,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.68.1565 Chrome/14.0.835.202 Safari/535.1*/
const Device UA641449 = {S90380, 139, { &P17595,&P17297,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/534.30 (KHTML, like Gecko) RockMelt/0.9.62.351 Chrome/12.0.742.122 Safari/534.30*/
const Device UA1943872 = {S90381, 137, { &P17595,&P17297,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.72.615 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2147338 = {S90382, 138, { &P17595,&P17297,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.68.1565 Chrome/14.0.835.202 Safari/535.1*/
const Device UA2258328 = {S90383, 137, { &P17595,&P17297,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.64.347 Chrome/13.0.782.218 Safari/535.1*/
const Device UA2982288 = {S90384, 138, { &P17595,&P17297,&P17391,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.68.1565 Chrome/14.0.835.202 Safari/535.1*/
const Device UA420117 = {S90385, 127, { &P15364,&P17350,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT; Avant Browser; Avant Browser; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2)*/
const Device UA455685 = {S90386, 79, { &P15364,&P17285,&P22314,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:11.0) Gecko/X Firefox/11.0 SeaMonkey/2.8*/
const Device UA525568 = {S90387, 74, { &P11727,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-D780/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA567222 = {S90388, 90, { &P12292,&P5448,&P4779,&P18092 }};	/*Useragent - HTC_Touch_3G_T3232 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA671691 = {S90389, 86, { &P17595,&P17294,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:5.0) Gecko/X Firefox/5.0 TenFourFox/7400*/
const Device UA710702 = {S90390, 136, { &P20557,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; zh-cn; MZ606 Build/H.6.2-9_CN-18) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1924955 = {S90391, 136, { &P20557,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; zh-cn; MZ606 Build/H.6.2_CN-18-2) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA771694 = {S90392, 87, { &P17595,&P17295,&P18210,&P18092 }};	/*Useragent - GoogleEarth/6.1.0.5001(Macintosh;Mac OS X (10.5.8);en;kml:2.2;client:Free;type:default)*/
const Device UA827702 = {S90393, 84, { &P17595,&P17294,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:7.0) Gecko/X Firefox/7.0 TenFourFox/G3*/
const Device UA1995828 = {S90394, 156, { &P20134,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025122 = {S90395, 157, { &P12453,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Liberty Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027149 = {S90396, 157, { &P12453,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Liberty Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2413315 = {S90397, 153, { &P12453,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Liberty Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027554 = {S90398, 141, { &P21821,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TAB-PROTAB2XXL(4) Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2034252 = {S90399, 138, { &P21821,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TAB-PROTAB2XXL Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2029641 = {S90400, 135, { &P24016,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; GIONEE A9 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2040791 = {S90401, 102, { &P23025,&P17603,&P4392,&P18092 }};	/*Useragent - PantechP6030/JZUSX BMP/1.0.4 DeviceId/X NetFront/4.1 OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2161440 = {S90402, 60, { &P15364,&P6729,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows 95; U; ru) Presto/2.10.229 Version/11.64*/
const Device UA2223405 = {S90403, 147, { &P19998,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2339277 = {S90404, 52, { &P12088,&P5890,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 (NokiaX6-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA2351671 = {S90405, 167, { &P21388,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; Karbonn_A1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2360225 = {S90406, 82, { &P12671,&P5055,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; Nokia700)/UC Browser8.5.0.183/51/352/UCWEB Mobile*/
const Device UA2376008 = {S90407, 142, { &P22161,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iberry AUXUS AX03G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2384353 = {S90408, 156, { &P18720,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; Micromax A50 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2402873 = {S90409, 132, { &P13006,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; folio100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2415565 = {S90410, 152, { &P23053,&P18066,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN700W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2432514 = {S90411, 223, { &P14186,&P5096,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2433055 = {S90412, 95, { &P15364,&P16110,&P24354,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; QQBrowser/7.0.4350.400)*/
const Device UA2846234 = {S90413, 88, { &P15364,&P16110,&P24354,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; QQBrowser/7.0.4350.400)*/
const Device UA2455789 = {S90414, 137, { &P20481,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DII PAD Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2457408 = {S90415, 138, { &P16436,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8150 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2457533 = {S90416, 153, { &P13078,&P5096,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-U V880 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2466282 = {S90417, 136, { &P20839,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; H9500S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2481790 = {S90418, 96, { &P15364,&P7127,&P21778,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.3.5 Safari/534.34*/
const Device UA2483028 = {S90419, 161, { &P19970,&P17485,&P24673,&P18092 }};	/*Useragent - SonyEricssonG502/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssong502) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2484809 = {S90420, 138, { &P18622,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A820L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2485812 = {S90421, 143, { &P21617,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2502252 = {S90422, 155, { &P21087,&P5629,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A690 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2502723 = {S90423, 136, { &P19971,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MT7003 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2808913 = {S90424, 141, { &P19971,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; WEXLER.BOOK_T7008 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2505583 = {S90425, 134, { &P14014,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S120 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2511621 = {S90426, 99, { &P14980,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-b7722) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2851896 = {S90427, 97, { &P14980,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; fa; samsung-gt-b7722i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2526575 = {S90428, 111, { &P15364,&P18145,&P22525,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD) AppleWebKit/534.26 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/534.26 Midori/0.4*/
const Device UA2528071 = {S90429, 82, { &P21094,&P18110,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Adr (Linux; U; 4.0.4; zh-cn; Lenovo A789 Build/A789_S228_X;480*800)*/
const Device UA2776557 = {S90430, 82, { &P21094,&P18110,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Adr (Linux; U; 4.0.4; zh-cn; Lenovo A789 Build/A789_S221_X;480*800)*/
const Device UA2542524 = {S90431, 143, { &P18008,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; SmartQT10 Build/GB.f02ref.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3089895 = {S90432, 145, { &P18008,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HUAWEI Y310-T10 Build/HuaweiY310-T10) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2554489 = {S90433, 157, { &P21094,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A789 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2577625 = {S90434, 142, { &P15786,&P5043,&P24668,&P18092 }};	/*Useragent - NokiaC2-05/2.0 (08.69) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-05) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2579541 = {S90435, 120, { &P13648,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaE7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2595768 = {S90436, 131, { &P22014,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; TM-9740 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2602606 = {S90437, 136, { &P21448,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SH530U Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2654298 = {S90438, 171, { &P18318,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ASUS Transformer Pad TF700T Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2657250 = {S90439, 137, { &P18442,&P24698,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L900 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2791962 = {S90440, 149, { &P18442,&P24698,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T889 Build/MIUI By AminLAmo) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2662451 = {S90441, 100, { &P15875,&P17485,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonck15i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2666987 = {S90442, 132, { &P21501,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; P5 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668589 = {S90443, 156, { &P20096,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-S7568 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2671971 = {S90444, 136, { &P14017,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P990 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676642 = {S90445, 138, { &P23447,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Newman N2 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2680973 = {S90446, 88, { &P22385,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Adr (Linux; U; 4.1.2; zh-cn; GT-I8262D Build/JZO54K.I8262DZCAMB2;480*800)*/
const Device UA2680966 = {S90447, 88, { &P22385,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Adr (Linux; U; 4.1.2; zh-cn; GT-I8262D Build/JZO54K.I8262DZCAMC2;480*800)*/
const Device UA2681019 = {S90448, 88, { &P22385,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Adr (Linux; U; 4.1.2; zh-cn; GT-I8262D Build/JZO54K.I8262DZCAMA9;480*800)*/
const Device UA2681020 = {S90449, 155, { &P22385,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2712069 = {S90450, 132, { &P23162,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B15 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712205 = {S90451, 134, { &P23376,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M9000 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713947 = {S90452, 137, { &P22865,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AT-AS53N Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2715885 = {S90453, 140, { &P22209,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Titanium S5 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2715999 = {S90454, 159, { &P18513,&P5629,&P22281,&P18092 }};	/*Useragent - OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3062100 = {S90455, 142, { &P22654,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C5502 Build/10.3.1.A.0.244) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2727063 = {S90456, 201, { &P15364,&P17483,&P18246,&P18092 }};	/*Useragent - MIKI50_6464_11B_24MP_HW (MRE\2.5.00(800) resolution\X chipset\MT6250 touch\0 tpannel\0 camera\1 gsensor\0 keyboard\qwerty) RLG_G51TV_V01_X Release/2012.09.08 WAP Browser/MAUI Profile/  Q03C1-2.40 en-US*/
const Device UA2951790 = {S90457, 201, { &P15364,&P17483,&P18246,&P18092 }};	/*Useragent - MIKI50_6464_11B_24MP_HW (MRE\2.5.00(800) resolution\X chipset\MT6250 touch\0 tpannel\0 camera\1 gsensor\0 keyboard\qwerty) RLG_G51TV_V01_X Release/2012.09.19 WAP Browser/MAUI Profile/  Q03C1-2.40 en-US*/
const Device UA2738775 = {S90458, 156, { &P19999,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Karbonn A30 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2749428 = {S90459, 141, { &P23517,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; konka W830 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2750084 = {S90460, 145, { &P20841,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; tablet Fnac 10 Build/2.1.0 X-18:06) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2752126 = {S90461, 154, { &P23546,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; AD683G Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2752423 = {S90462, 161, { &P20330,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; Micromax P350 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2762600 = {S90463, 156, { &P23919,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; R800i Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2764971 = {S90464, 136, { &P21069,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; P-05D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2769252 = {S90465, 93, { &P19777,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9105) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2769503 = {S90466, 141, { &P21614,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Philips W536 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2772866 = {S90467, 140, { &P20089,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SO-01E Build/9.0.G.1.108) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2777418 = {S90468, 142, { &P20087,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo K800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2779189 = {S90469, 143, { &P23541,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE Grand Era Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2779951 = {S90470, 138, { &P14967,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985597 = {S90471, 138, { &P14967,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2938405 = {S90472, 95, { &P15501,&P5477,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E16i) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2783307 = {S90473, 140, { &P21526,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cherry w500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783691 = {S90474, 135, { &P21523,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SH930W Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783880 = {S90475, 134, { &P23124,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A919i Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784103 = {S90476, 131, { &P20527,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A6 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2795886 = {S90477, 157, { &P18474,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; GT-I9210T Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867488 = {S90478, 150, { &P20527,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A6 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2796994 = {S90479, 134, { &P22653,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTL22 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798551 = {S90480, 135, { &P21829,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; F-02E Build/V17R48A) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799256 = {S90481, 157, { &P19856,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LS970 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799360 = {S90482, 150, { &P22606,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5110 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799494 = {S90483, 163, { &P18581,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; SCH-S720C Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2799839 = {S90484, 158, { &P19902,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R530M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2800574 = {S90485, 132, { &P20807,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; B92 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2860367 = {S90486, 143, { &P23161,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Xtreamer Joyz Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA3100610 = {S90487, 129, { &P20187,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Novo7Fire Build/Xtm) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2867852 = {S90488, 139, { &P18453,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; LT25i Build/9.1.A.1.140) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937900 = {S90489, 137, { &P18453,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Xperia V Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868103 = {S90490, 137, { &P15802,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Amaze 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879466 = {S90491, 140, { &P19999,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A30 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2891062 = {S90492, 143, { &P22608,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Spice Mi-530 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892685 = {S90493, 159, { &P20183,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2893031 = {S90494, 150, { &P12803,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I405 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2938921 = {S90495, 128, { &P20838,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; SHT21 Build/S5140) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2944152 = {S90496, 159, { &P13634,&P17483,&P24678,&P18092 }};	/*Useragent - LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gs500) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2944361 = {S90497, 138, { &P23705,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C811 4G Build/C811M020) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2963615 = {S90498, 152, { &P21963,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Q800 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2987751 = {S90499, 115, { &P21879,&P18110,&P17250,&P18092 }};	/*Useragent - PAD-FMD700P Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2995762 = {S90500, 116, { &P21879,&P18110,&P17250,&P18092 }};	/*Useragent - PAD-FMD700HD Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2994952 = {S90501, 93, { &P18442,&P18111,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; zh-CN; GT-N7100) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile*/
const Device UA2994833 = {S90502, 150, { &P15364,&P18111,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.1; id-id; T753 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3046740 = {S90503, 155, { &P15364,&P18111,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; MITO t300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999695 = {S90504, 138, { &P18687,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P708g Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3048065 = {S90505, 137, { &P18687,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P705 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3004366 = {S90506, 164, { &P21911,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; AMAZE S200 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA3004848 = {S90507, 162, { &P12026,&P17485,&P24678,&P18092 }};	/*Useragent - SonyEricssonC510/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonc510) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3023415 = {S90508, 173, { &P20004,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn_A2+ Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3020974 = {S90509, 69, { &P18471,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile: XT907; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA3021387 = {S90510, 138, { &P24480,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262B Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023665 = {S90511, 155, { &P21739,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; iris454 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3023742 = {S90512, 93, { &P18700,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-B5512) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3027716 = {S90513, 88, { &P20231,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A21) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3029330 = {S90514, 157, { &P18280,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC One S Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056154 = {S90515, 123, { &P21239,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; M9 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3057868 = {S90516, 138, { &P24631,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; LG-F240 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058870 = {S90517, 100, { &P13206,&P17485,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk530i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3111382 = {S90518, 137, { &P17324,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9001 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094608 = {S90519, 137, { &P18701,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-T100K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3079353 = {S90520, 155, { &P21023,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-E405 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3078711 = {S90521, 95, { &P20004,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Karbonn_A2) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3112540 = {S90522, 128, { &P23873,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BC1003 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3079985 = {S90523, 95, { &P15501,&P5477,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E16i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3126443 = {S90524, 144, { &P18554,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094139 = {S90525, 134, { &P21104,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; X507S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109543 = {S90526, 168, { &P15201,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3111301 = {S90527, 156, { &P16356,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-N7000 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126299 = {S90528, 156, { &P16356,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; GT-N7000 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3115786 = {S90529, 151, { &P18283,&P18111,&P24683,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9300 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.5 Mobile Safari/533.1*/
const Device UA3132662 = {S90530, 138, { &P20871,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810P Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA16932 = {S90531, 116, { &P14970,&P17605,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-E3210/1.0 RTK-E/1.0 DF/1.0 Release/12.14.2010 Browser/NetFront4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21934 = {S90532, 130, { &P12021,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-VE538/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.2, Browser/UP.Browser/7.2.7.2 (GUI) MMP/2.0 Push/PO*/
const Device UA21935 = {S90533, 158, { &P12021,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-VE538/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.2, Browser/UP.Browser/7.2.7.2, Browser/UP.Browser/7.2.7.2 (GUI) MMP/2.0 Push/PO*/
const Device UA21936 = {S90534, 186, { &P12021,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-VE538/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.2, Browser/UP.Browser/7.2.7.2, Browser/UP.Browser/7.2.7.2, Browser/UP.Browser/7.2.7.2 (GUI) MMP/2.0 Push/PO*/
const Device UA21937 = {S90535, 214, { &P12021,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-VE538/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.2, Browser/UP.Browser/7.2.7.2, Browser/UP.Browser/7.2.7.2, Browser/UP.Browser/7.2.7.2, Browser/UP.Browser/7.2.7.2 (GUI) MMP/2.0 Push/PO*/
const Device UA63187 = {S90536, 102, { &P12021,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-VE538/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.2 (GUI) MMP/2.0 Push/PO*/
const Device UA22238 = {S90537, 21, { &P15408,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP110 Obigo/WAP2.0*/
const Device UA24531 = {S90538, 13, { &P12197,&P17483,&P17660,&P18092 }};	/*Useragent - LG/KF755/v1.0*/
const Device UA50254 = {S90539, 17, { &P13206,&P17485,&P17470,&P18092 }};	/*Useragent - sonyericssonk530i*/
const Device UA50331 = {S90540, 62, { &P13206,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonK530i/R8BB Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2588712 = {S90541, 205, { &P15513,&P5055,&P3804,&P18092 }};	/*Useragent - NokiaT7-00_TD/CMCC (SymbianOS/3; Release/028.004; Series60/5.2 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko)NokiaBrowser/7.3.1.37 Mobile Safari/533.4 3gpp-gba*/
const Device UA54790 = {S90542, 68, { &P14262,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHD830/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63501 = {S90543, 88, { &P14522,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-G821/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP-2.0 configuration/CLDC-1.0*/
const Device UA89845 = {S90544, 88, { &P14522,&P17017,&P4780,&P18092 }};	/*Useragent - Nexian-NXG821/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA667266 = {S90545, 133, { &P14997,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Liquid Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018754 = {S90546, 133, { &P14997,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Liquid Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019655 = {S90547, 133, { &P14997,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Liquid Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA690271 = {S90548, 42, { &P14753,&P5890,&P24665,&P18092 }};	/*Useragent - Nokia N96/UC Browser8.0.4.121/28/444/UCWEB*/
const Device UA447471 = {S90549, 168, { &P17999,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; MOT-XT301/CNACT_X2_01.20.3; 240*320; CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA828866 = {S90550, 168, { &P17999,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; MOT-XT301/CNACT_X2_01.21.5; 240*320; CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2023462 = {S90551, 168, { &P17999,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; MOT-XT301/CNACT_X2_01.51.0; 240*320; CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA515176 = {S90552, 99, { &P17595,&P17294,&P22510,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.4; en-US; rv:1.9.0.9) Gecko/X Firefox/3.0.9 Wyzo/3.0.3*/
const Device UA1897536 = {S90553, 88, { &P17595,&P17295,&P22311,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:6.0) Gecko/X Firefox/6.0 SeaMonkey/2.3.1*/
const Device UA807396 = {S90554, 61, { &P15364,&P18110,&P18194,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:6.0) Gecko/X Thunderbird/6.0*/
const Device UA664334 = {S90555, 104, { &P14738,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; 7 Mozart T8698)*/
const Device UA925718 = {S90556, 98, { &P14738,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; 7 Mozart)*/
const Device UA925758 = {S90557, 106, { &P14738,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; 7 Mozart; Orange)*/
const Device UA925909 = {S90558, 95, { &P14738,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; T8697)*/
const Device UA713868 = {S90559, 60, { &P15364,&P17350,&P17191,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 7.0) XSpider 7*/
const Device UA747295 = {S90560, 91, { &P12226,&P5448,&P4779,&P18092 }};	/*Useragent - HTC_Touch_Pro_T7272 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA759372 = {S90561, 37, { &P15364,&P18144,&P4811,&P18092 }};	/*Useragent - Opera/9.01 (X11; OpenBSD i386; U; en)*/
const Device UA902531 = {S90562, 89, { &P17595,&P17295,&P21170,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:13.0) Gecko/X Firefox/13.0 SeaMonkey/2.10*/
const Device UA2028560 = {S90563, 165, { &P15649,&P5096,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2853120 = {S90564, 169, { &P15649,&P5096,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5570 Build/FROYO) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2024207 = {S90565, 88, { &P17976,&P5477,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00*/
const Device UA2029334 = {S90566, 96, { &P17976,&P5477,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.1-update1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.10.254 Version/12.00*/
const Device UA2203461 = {S90567, 81, { &P17976,&P6546,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.1; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2217448 = {S90568, 148, { &P14743,&P17485,&P24670,&P18092 }};	/*Useragent - SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonj20i) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2223895 = {S90569, 131, { &P18027,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID7042 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2257667 = {S90570, 208, { &P18113,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, us*/
const Device UA2314488 = {S90571, 79, { &P12707,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-in; Nokia701)/UC Browser8.2.0.132/50/444/UCWEB Mobile*/
const Device UA2333873 = {S90572, 72, { &P15364,&P16110,&P22314,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; rv:11.0) Gecko/X Firefox/11.0 SeaMonkey/2.8*/
const Device UA2355174 = {S90573, 135, { &P15364,&P17816,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/10.04 Chromium/22.0.1229.79 Chrome/22.0.1229.79 Safari/537.4*/
const Device UA2832660 = {S90574, 137, { &P15364,&P17816,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/10.04 Chromium/22.0.1229.79 Chrome/22.0.1229.79 Safari/537.4*/
const Device UA2372016 = {S90575, 96, { &P18583,&P5043,&P17338,&P18092 }};	/*Useragent - Nokia110/2.0 (03.04) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia110/UC Browser7.9.0.102/69/352*/
const Device UA2376117 = {S90576, 131, { &P20188,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U19GT-W Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2519092 = {S90577, 26, { &P13064,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-I727/100.40004*/
const Device UA2410832 = {S90578, 74, { &P13153,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-E2220/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA2413286 = {S90579, 47, { &P15364,&P5890,&P21761,&P18092 }};	/*Useragent - iBrowser/2.5 (E66-1;SymbianOS/9.2 Series60/3.1)*/
const Device UA2809332 = {S90580, 45, { &P15364,&P5890,&P21761,&P18092 }};	/*Useragent - iBrowser/2.5 (N79;SymbianOS/9.3 Series60/3.2)*/
const Device UA2425132 = {S90581, 143, { &P20880,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NEO_N002 Build/NeoNEO_N002) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2455603 = {S90582, 132, { &P15364,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; i697 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA3099610 = {S90583, 142, { &P15364,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; SY-Z19M4 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75 Mobile Safari/535.7*/
const Device UA2466171 = {S90584, 138, { &P16273,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T679 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2481781 = {S90585, 70, { &P13591,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.2; zh-cn; ZTE-BLADE; 480*800) UCWEB7.9.0.94/139/355*/
const Device UA2483782 = {S90586, 138, { &P16757,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SCH-I500 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2485210 = {S90587, 175, { &P12139,&P5204,&P24675,&P18092 }};	/*Useragent - Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia5310xpressmusic) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2502360 = {S90588, 135, { &P21231,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; L-02E Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2528513 = {S90589, 135, { &P20255,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; R8111 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2530316 = {S90590, 40, { &P17968,&P17979,&P24421,&P18092 }};	/*Useragent - Dolphin http client/1.3.1(197) (Android)*/
const Device UA2553915 = {S90591, 33, { &P16074,&P5768,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia3110c/07.21)*/
const Device UA2558340 = {S90592, 169, { &P17625,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST25i Build/6.1.1.B.1.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2558961 = {S90593, 168, { &P15539,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2564219 = {S90594, 168, { &P15147,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST15i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2579883 = {S90595, 139, { &P16321,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xolo_X500 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2590525 = {S90596, 109, { &P22106,&P5629,&P15760,&P18092 }};	/*Useragent - HS-T92_TD/1.0 Android/2.3 Release/12.10.2011 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2593510 = {S90597, 125, { &P14182,&P17484,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5333) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2600967 = {S90598, 128, { &P21222,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT1S0 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2605522 = {S90599, 129, { &P20014,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-01E Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2633951 = {S90600, 151, { &P15985,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2783287 = {S90601, 150, { &P15985,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6200 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2655763 = {S90602, 68, { &P18451,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; LT26ii; 720*1280) UCWEB7.9.0.94/139/444*/
const Device UA2657597 = {S90603, 115, { &P18740,&P17979,&P18062,&P18092 }};	/*Useragent - OPPO_U701T/1.0 Linux/3.0.13 Android/4.0 Release/10.15.2012 Browser/AppleWebKit534.30 Mobile Safari/534.30 MBBMS/2.2*/
const Device UA2659594 = {S90604, 161, { &P20120,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A45 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3058321 = {S90605, 165, { &P20120,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A45 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2662014 = {S90606, 124, { &P15767,&P17146,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B141 Safari/8536.25*/
const Device UA3091807 = {S90607, 150, { &P15767,&P17146,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/6.0 Mobile/9A405 Safari/7534.48.3 YaBrowser/2.1.1364.172*/
const Device UA2662062 = {S90608, 170, { &P18720,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; Micromax A50 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3124951 = {S90609, 174, { &P18720,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Micromax_A50 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2667827 = {S90610, 127, { &P18248,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A700 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667912 = {S90611, 156, { &P16356,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-N7000 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2733907 = {S90612, 156, { &P16356,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; GT-N7000 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2670731 = {S90613, 134, { &P20117,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IQ440 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676995 = {S90614, 131, { &P21501,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; P5 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677062 = {S90615, 134, { &P21815,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; P-02E Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2701266 = {S90616, 97, { &P14969,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Desire_S) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2706841 = {S90617, 127, { &P17686,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MZ601 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2709136 = {S90618, 127, { &P17686,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MZ601 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2707798 = {S90619, 136, { &P21446,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SH837W Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2736801 = {S90620, 152, { &P15575,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100G Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2747115 = {S90621, 57, { &P11751,&P17483,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1)*/
const Device UA2750114 = {S90622, 88, { &P19937,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; A73) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2763913 = {S90623, 71, { &P21431,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; ZTE_V955; 480*854) UCWEB7.9.0.94/139/33253*/
const Device UA2764468 = {S90624, 155, { &P14123,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-P970 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3055986 = {S90625, 160, { &P14123,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-P970 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/p970*/
const Device UA2764936 = {S90626, 175, { &P18321,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5300 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2814243 = {S90627, 171, { &P18321,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2766796 = {S90628, 92, { &P11868,&P5768,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6280) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2852773 = {S90629, 161, { &P12803,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I405 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000*/
const Device UA2768674 = {S90630, 106, { &P13552,&P5129,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaC6-00*/
const Device UA2768781 = {S90631, 26, { &P22585,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SPH-L720/101.40202*/
const Device UA2778554 = {S90632, 128, { &P18254,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A-AZ;FBAV/2.0;FBDM/{density=1.5,width=800,height=1280};FBLC/en_US;FBCR/;FBPN/com.facebook.katana;FBDV/KFTT;FBSV/4.0.3;]*/
const Device UA2863942 = {S90633, 155, { &P18254,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.1;FBBV/X;FBDM/{density=1.5,width=540,height=960};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/DROID BIONIC;FBSV/4.1.2;]*/
const Device UA2779565 = {S90634, 140, { &P23795,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Byond B65 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2779802 = {S90635, 142, { &P23141,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROIDZ Atom Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2882550 = {S90636, 154, { &P15184,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; LG-P920 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2781681 = {S90637, 26, { &P22582,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-I337/101.40202*/
const Device UA2781739 = {S90638, 93, { &P12487,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA5700) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2782425 = {S90639, 136, { &P23550,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE T81 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2790388 = {S90640, 106, { &P13324,&P5055,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (NokiaN8-00/111.040.1511/sw_platform=S60;sw_platform_version=5.3;java_build_version=2.3.24)*/
const Device UA2796797 = {S90641, 128, { &P20835,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 754TPC Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2798775 = {S90642, 138, { &P14225,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Desire HD Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2800409 = {S90643, 148, { &P21769,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ME301T Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2849418 = {S90644, 153, { &P23519,&P18066,&P18062,&P18092 }};	/*Useragent - iDxD4 Linux/3.0.13 Android/4.0.4 Release/12.04.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2964600 = {S90645, 89, { &P20129,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; zh-CN; MI 2) U2/1.0.0 UCBrowser/9.1.1.309 U2/1.0.0 Mobile*/
const Device UA2890794 = {S90646, 138, { &P22638,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; W20 Build/Symphony_W20) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3086775 = {S90647, 62, { &P22638,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; W20 Build/Symphony_W20)*/
const Device UA2892675 = {S90648, 102, { &P18176,&P21697,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 900) U2/1.0.0 UCBrowser/3.0.0.285 U2/1.0.0 Mobile*/
const Device UA2937402 = {S90649, 143, { &P13746,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LGL35G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2937765 = {S90650, 141, { &P19842,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2956688 = {S90651, 156, { &P22980,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; TECNO D5 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2958624 = {S90652, 91, { &P23351,&P22784,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromax) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2962578 = {S90653, 97, { &P18382,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire C) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2963724 = {S90654, 158, { &P18449,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST27i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2964437 = {S90655, 95, { &P15501,&P5477,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E16i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2977102 = {S90656, 136, { &P24061,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-S7270 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA2982855 = {S90657, 136, { &P20944,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; T-Hub2 Build/TVA301TELBG3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2987119 = {S90658, 89, { &P15364,&P18110,&P20787,&P18092 }};	/*Useragent - MQQBrowser/3.6/Adr (Linux; U; 4.0.4; zh-cn; vivo S7t Build/ALPS.ICS2.TDD.MP.V1.2;480*800)*/
const Device UA3046234 = {S90659, 88, { &P15364,&P18110,&P20787,&P18092 }};	/*Useragent - MQQBrowser/3.6/Adr (Linux; U; 4.2.1; zh-cn; vivo X1St Build/ALPS.JB2.TDD.MP.V1;720*1280)*/
const Device UA2991739 = {S90660, 152, { &P20877,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ALCATEL ONE TOUCH 8000D Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3007228 = {S90661, 151, { &P20158,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A7* Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3011555 = {S90662, 80, { &P24612,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-S3330/S3330JVMC5 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA3016309 = {S90663, 148, { &P21304,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; ru-ru; Fly IQ450 Quattro Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3016910 = {S90664, 153, { &P21963,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Q800 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3019453 = {S90665, 133, { &P20145,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; NS-13T001 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3019811 = {S90666, 154, { &P18017,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Ideapad K1 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1/k1*/
const Device UA3022612 = {S90667, 133, { &P24542,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARNOVA 90G3 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3024200 = {S90668, 97, { &P18652,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A87) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3108778 = {S90669, 93, { &P18700,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-B5512) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3039987 = {S90670, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (/jRFM94SIl+0rwrScZlAfRpeVd1zX9LVAc31pppjmOQ=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3039988 = {S90671, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (CJZKkKysm/LkU+fZlIfebm7a9VNMuopZW8bqizzu1lI=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3039989 = {S90672, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (NKlZzYOCS63Iq04D5xmpu6GTWaUlr6F5E/V88R9Z2ro=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3039990 = {S90673, 203, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (PSjeKGWdCylYhWGfjYvF0dKDtfIL4BxWi9vri2hRbT0=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2 AlexaToolbar/alxg-3.1*/
const Device UA3039991 = {S90674, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (JAMkPgy34jtg5uC/PTfQye1NRFZmn+kq6LJvRXI+l8A=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3039992 = {S90675, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (1j9Klv6ZovFXBK/nyrgG/iH+jY/bvSpyuNsuzCAL2kc=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3039994 = {S90676, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (H9vuO55J/W/eNCjf7LhP6UHNesJkTSkpq/3U3wuSATg=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3039995 = {S90677, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (QDsFFh/GywLoYHpNZElVJa1IXkbKb5uqLgqYi6JsZ0w=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3039996 = {S90678, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (HtfMH4ztVEkcy+JAR7uq5dxwuGNyDc+BklZPJOWo44o=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3039997 = {S90679, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (GnkUPP2lBJm/9lZYoP7uGAYRR0ms99IG/Q5qufwgfOQ=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3039999 = {S90680, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (F87GmG3U5UzGPls4nzXh8E0YdQDCbyzs7f3KnzyYCWY=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3039998 = {S90681, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (BiJcJ/NnWWNRCJbpE1IDAUARApQI13bBpXjUKIJ3NX0=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040000 = {S90682, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (H3Md1XIJnTbE+bu2rA68p1zm1RLiEeeTztPpy0c371E=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040001 = {S90683, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (LmSzCLvkVtv8go56pkt0CddiEHAq3YDx2qKsCpola4o=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040004 = {S90684, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (7zeHZopFpbXZh+fsx0qv92Y120A/VprIybj2ogv4BEc=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040154 = {S90685, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (xR36Tq9pb/2KNCkXC3Nu1HBrhQMzwBt+vlts4pxjmLs=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040155 = {S90686, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (q2LVBpKdjbTyqvLqnu3cXX5gjjEx1Jq+0EvSmPd3Vpw=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040157 = {S90687, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (XT28UluKEur58RliQm9ffzA9Ed0PI9ZHkAaWZudRZQc=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040158 = {S90688, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (seO21gpOtp8uTe8EjZClilM5rgzyrlWqV92C7FPS+dA=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040159 = {S90689, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (mEHSren5PNQEk2sVQQOzhnX0SPrYAQ6x+xwMaR4JcZg=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040162 = {S90690, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (rqZhywrLlRmI6qan4QYva0VIv9pNu29JPwgQ3VhVnlU=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040163 = {S90691, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (g3kuz6HvwD1E+pHG1Nwxvc1pwnRrGwY8opYv4mY7fRs=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040164 = {S90692, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (uehqSy5ycXDRBKP+jbwUI1YPVd1yTBKlCXkDiKHDrj4=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040165 = {S90693, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (z+fvGPOyIDB7AlyW1GAPcCji/Jny6v/TzN9NQqOAIiI=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040166 = {S90694, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (nuS3iPfF+4oBZXsWfuw2wYSgxrH/3AxDEPP29Ubp/qM=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040167 = {S90695, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (bTReBGAmyU0p9EZguECS3zebxG/O7ntBasc9XCGRZt4=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040168 = {S90696, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (xokYZOXYGKSLMXnZxQkcjAxr4E3a8+0X3xBe/nw/XKA=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040169 = {S90697, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (eLxYEzaXKSGyVCtDG8d1ouh030x3nVxlEbl8rNaF8D8=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040170 = {S90698, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (xfMcA+UNukSoKxiQo4oK63DE5u5v+y6FWE3ME12jJoM=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040171 = {S90699, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ps12oxi4NLcjkolTgLW+makwfh+qo4+Hyxd7hoj3zhs=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040173 = {S90700, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (t8KTdxr3CCQJs0qCrxcYV8EfKDYQAn9KVgD6PdMFVoo=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040172 = {S90701, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (lDmtjpKAQjTZX57klDy8yWr9AokW7gHOaG78SEH4bIM=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040175 = {S90702, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (nUzOdO5G4Zk1kpPNdPN0jyVHPVu1wLsdp7Vr9j1GedI=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040176 = {S90703, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (qrxHXoiLgqDAfXtTk8py4SJH9t6DzTJXbyXeGN77MgA=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040177 = {S90704, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (6y+XVtkcmUcScnuMY4hWaj5HPWngNx88XntxfGm+3rs=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040178 = {S90705, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (RO9oj+u2gYTRUfZifbLi9HK8Bk0GiQbZWK6TsLivua8=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040179 = {S90706, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (6e5YUf22go55jLeYCrxeLl4VBU4hHaD1vd6RkSY0CUE=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040180 = {S90707, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (jFStrNCKwLCtSNpH0kaAT8gmOTLStyWawS580arwYhU=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040182 = {S90708, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (ZmEufAdvOJ6+jpzS29ugr8sbtxJwbNEwTvj+UjrNdn8=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3040184 = {S90709, 181, { &P15364,&P24348,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (UuCyn/MDeKIqEOTue1DYHMGflhX4w4+4cfNzdo9/ADA=) Chromium/22.0.1229.94 Chrome/22.0.1229.94 Safari/535.2*/
const Device UA3079260 = {S90710, 157, { &P18679,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-B5330 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3057533 = {S90711, 165, { &P18214,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-S7500 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/trebon*/
const Device UA3121006 = {S90712, 156, { &P18214,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-S7500 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057879 = {S90713, 151, { &P22611,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-S7572 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3059167 = {S90714, 152, { &P15575,&P18066,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100G Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3060323 = {S90715, 159, { &P13634,&P17483,&P24679,&P18092 }};	/*Useragent - LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gs500) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3109213 = {S90716, 138, { &P24127,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Revo_Lead Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3112421 = {S90717, 134, { &P21302,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IQ441 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114930 = {S90718, 137, { &P20947,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Evo V 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3113026 = {S90719, 134, { &P24042,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ARNOVA 101 G4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3079109 = {S90720, 160, { &P21850,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Spice Mi-435 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3079131 = {S90721, 156, { &P23199,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad 7295+ Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3079708 = {S90722, 154, { &P23199,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad7295 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3114552 = {S90723, 136, { &P22991,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC 802t Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3079307 = {S90724, 93, { &P21628,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8552) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3079680 = {S90725, 93, { &P22609,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6312) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3093282 = {S90726, 87, { &P15364,&P18598,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.2; en-US; P2) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3094505 = {S90727, 128, { &P18437,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-N8000 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094908 = {S90728, 93, { &P20146,&P18111,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9082) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3111494 = {S90729, 132, { &P15767,&P22882,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 Safari/7534.48.3 Sleipnir/2.4.5m*/
const Device UA3112103 = {S90730, 157, { &P12973,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3116524 = {S90731, 126, { &P12973,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3112545 = {S90732, 161, { &P16356,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; GT-N7000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3113537 = {S90733, 161, { &P16356,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-N7000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3131413 = {S90734, 138, { &P19894,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830C Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132966 = {S90735, 137, { &P19894,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-R830 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA224 = {S90736, 107, { &P15127,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-F270/F270DDHG6 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA20698 = {S90737, 109, { &P15127,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-F270i/F270iDDIA1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA5003 = {S90738, 85, { &P13999,&P5448,&P17183,&P18092 }};	/*Useragent - HTC_Leo Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA16165 = {S90739, 91, { &P13999,&P5448,&P17183,&P18092 }};	/*Useragent - HTC_HD2_T8585 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA38015 = {S90740, 134, { &P13999,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) PPC; HTC_HD2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA31257 = {S90741, 77, { &P11859,&P5890,&P4030,&P18092 }};	/*Useragent - NokiaN92-2/2.0 (5.0748.2.1.4) S60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54788 = {S90742, 77, { &P11859,&P5890,&P4030,&P18092 }};	/*Useragent - NokiaN92-1/2.0 (5.0748.2.0.4) S60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23362 = {S90743, 46, { &P15600,&P6476,&P3625,&P18092 }};	/*Useragent - CDM8975/T05_0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA24557 = {S90744, 39, { &P15710,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP199 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA26967 = {S90745, 64, { &P13658,&P17483,&P3459,&P18092 }};	/*Useragent - LG-GT950/V10g Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA30947 = {S90746, 47, { &P12373,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-S4000/ UP.Browser/7.2.6.1.794 (GUI) MMP/2.0*/
const Device UA50626 = {S90747, 52, { &P12373,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-S4000/10033 UP.Browser/7.2.6.1.644 (GUI) MMP/2.0*/
const Device UA40779 = {S90748, 114, { &P13746,&P5062,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android1.6;Zh_cn;GT540;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355*/
const Device UA2019231 = {S90749, 65, { &P13746,&P5062,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android1.6;Zh_cn;GT540;320*480;)UCWEB7.8.0.95/139/351*/
const Device UA40256 = {S90750, 118, { &P14967,&P5629,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.3.3;Zh_cn;LG-P500;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355*/
const Device UA1935761 = {S90751, 69, { &P14967,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.3;Zh_cn;LG-P500;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA58799 = {S90752, 134, { &P20502,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Orbit Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58800 = {S90753, 133, { &P20502,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Orbit Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA90036 = {S90754, 31, { &P17741,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P05B(c500;TB;W24H16)*/
const Device UA333733 = {S90755, 135, { &P16757,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I500 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA346126 = {S90756, 135, { &P16757,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SCH-I500 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA641525 = {S90757, 142, { &P16757,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SCH-I500 Build/MIUIAndroid) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA338914 = {S90758, 122, { &P15364,&P17285,&P22409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.4.0.0 Chrome/13.0.782.220 Safari/535.1*/
const Device UA725046 = {S90759, 129, { &P15364,&P17285,&P22409,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.4.0.0 Chrome/13.0.782.220 Safari/535.1*/
const Device UA359500 = {S90760, 48, { &P17595,&P17779,&P20779,&P18092 }};	/*Useragent - MQQBrowser/28 (iOS; U; CPU like Mac OS X; zh-cn)*/
const Device UA368150 = {S90761, 123, { &P15364,&P7133,&P22419,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.0.3.0 Chrome/17.0.963.38 Safari/535.11*/
const Device UA368178 = {S90762, 130, { &P15364,&P7133,&P22419,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.0.3.0 Chrome/17.0.963.38 Safari/535.11*/
const Device UA532513 = {S90763, 63, { &P15364,&P17017,&P17466,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Mac_PowerPC) Opera 6.0  [en]*/
const Device UA821186 = {S90764, 23, { &P15364,&P17017,&P17466,&P18092 }};	/*Useragent - Opera/6.01 (X11; U; nn)*/
const Device UA441258 = {S90765, 125, { &P15364,&P17285,&P22426,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.140 Safari/535.19 Comodo_Dragon/18.0.3.0*/
const Device UA441471 = {S90766, 132, { &P15364,&P17285,&P22426,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.140 Safari/535.19 Comodo_Dragon/18.0.3.0*/
const Device UA772451 = {S90767, 102, { &P17595,&P17295,&P22482,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6.1*/
const Device UA514725 = {S90768, 71, { &P15364,&P18110,&P20907,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:0.9.4) Gecko/X Netscape6/6.2*/
const Device UA553050 = {S90769, 72, { &P18254,&P18110,&P18407,&P18091 }};	/*Useragent - Opera/9.70 (Linux armv7l ; U; turbotabbee/TSV2.0/1.02Q; fr) Presto/2.2.1*/
const Device UA573511 = {S90770, 89, { &P17976,&P5629,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.7; Linux; Opera Tablet/ADR-X; U; en) Presto/2.8.149 Version/11.10*/
const Device UA634198 = {S90771, 92, { &P17976,&P5629,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.3; Linux; Opera Tablet/ADR-X; U; en-GB) Presto/2.8.149 Version/11.10*/
const Device UA2020075 = {S90772, 87, { &P17976,&P5629,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3; Linux; Opera Tablet/ADR-X; U; en) Presto/2.8.149 Version/11.10*/
const Device UA673309 = {S90773, 43, { &P11684,&P5129,&P24668,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UCWEB8.3.0.133/50/800*/
const Device UA2332855 = {S90774, 44, { &P11684,&P5129,&P24668,&P18092 }};	/*Useragent - Nokia5800w XpressMusic/UCWEB8.3.0.133/50/800*/
const Device UA2372921 = {S90775, 91, { &P11684,&P5129,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokia5800d-1) UCBrowser8.3.0.154/69/352/UCWEB Mobile  3gpp-gba*/
const Device UA632725 = {S90776, 68, { &P20370,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux; U;2.3.4;zh_cn;ZTE-C N760;320*480;)UCWEB7.6.0.75/139/32139*/
const Device UA742898 = {S90777, 138, { &P21414,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; bq Darwin Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA743199 = {S90778, 126, { &P23611,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40CX520; CC/XGA; en) Presto/2.7.61 Version/11.00*/
const Device UA746166 = {S90779, 126, { &P23618,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46EX523; CC/USA; en) Presto/2.7.61 Version/11.00*/
const Device UA806399 = {S90780, 150, { &P13308,&P17485,&P24667,&P18092 }};	/*Useragent - SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-in; sonyericssonj105i) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA854305 = {S90781, 102, { &P17595,&P17295,&P22481,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.0.16) Gecko/X Firefox/3.0.16 Flock/2.5.6*/
const Device UA859160 = {S90782, 118, { &P15364,&P17188,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 4.0; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.1.249.1064 Safari/532.5*/
const Device UA2127695 = {S90783, 116, { &P15364,&P17188,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 4.0; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.249.78 Safari/532.5*/
const Device UA2028348 = {S90784, 168, { &P11950,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA1865301 = {S90785, 111, { &P15364,&P5690,&P23684,&P18092 }};	/*Useragent - GoogleEarth/5.1.3533.1731(Windows;Microsoft Windows XP (Service Pack 3);en-US;kml:2.2;client:Free;type:default)*/
const Device UA2264142 = {S90786, 111, { &P15364,&P5690,&P23684,&P18092 }};	/*Useragent - GoogleEarth/5.1.3535.3218(Windows;Microsoft Windows XP (Service Pack 3);en-US;kml:2.2;client:Free;type:default)*/
const Device UA2027641 = {S90787, 171, { &P15060,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352*/
const Device UA2979900 = {S90788, 171, { &P15060,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352*/
const Device UA2028983 = {S90789, 129, { &P22242,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ME171 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2030721 = {S90790, 129, { &P21453,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; A01SH Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2174872 = {S90791, 35, { &P12897,&P5843,&P21759,&P18092 }};	/*Useragent - iBrowser/Mini2.8 (NokiaC2-00/03.82)*/
const Device UA2171354 = {S90792, 53, { &P14188,&P5890,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1 (Nokia5233;SymbianOS/9.1 Series60/3.0)*/
const Device UA2193465 = {S90793, 66, { &P15364,&P17285,&P18345,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:15.0) Gecko/X Firefox/15.0.1 slurp*/
const Device UA2231976 = {S90794, 73, { &P15364,&P17285,&P18345,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:15.0) Gecko/X Firefox/15.0.1 slurp*/
const Device UA2199851 = {S90795, 71, { &P17595,&P18207,&P21574,&P18092 }};	/*Useragent - Mac OS X/10.8.2 (12C54); ExchangeWebServices/3.0 (157); Mail/6.1 (1498)*/
const Device UA2207578 = {S90796, 123, { &P17595,&P17295,&P17335,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_4; en-gb) AppleWebKit/528.4+ (KHTML, like Gecko) Version/4.1 Safari/526.11.2*/
const Device UA2269948 = {S90797, 123, { &P17595,&P17295,&P17335,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-us) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.1 Safari/530.18*/
const Device UA2274311 = {S90798, 118, { &P17595,&P17295,&P17335,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_8; en-us) AppleWebKit/532.0+ (KHTML, like Gecko) Version/4.1 Safari/531.9*/
const Device UA2260444 = {S90799, 157, { &P18321,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2366128 = {S90800, 160, { &P21052,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2402957 = {S90801, 119, { &P16871,&P22784,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; micromaxq5) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2371842 = {S90802, 52, { &P13324,&P5890,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 (NokiaN8-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA2373013 = {S90803, 84, { &P12142,&P17485,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; sonyericssonu5i) UCBrowser8.3.0.154/69/352/UCWEB Mobile*/
const Device UA2382626 = {S90804, 163, { &P13964,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I5510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2652589 = {S90805, 171, { &P14031,&P5477,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; GT-I5503 Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2410671 = {S90806, 58, { &P23057,&P22832,&P17470,&P18092 }};	/*Useragent - Videocon V1528/3.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2502035 = {S90807, 118, { &P13445,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN85) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile*/
const Device UA2454092 = {S90808, 151, { &P12798,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT18i Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455920 = {S90809, 151, { &P23937,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Y2 Build/HCL ME Tablet Y2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2458423 = {S90810, 91, { &P13847,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-lg900g) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2467921 = {S90811, 131, { &P20425,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N-03E Build/AX) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2472080 = {S90812, 123, { &P19755,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T508 Build/M) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2477552 = {S90813, 138, { &P21010,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMID4311 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2816139 = {S90814, 136, { &P15785,&P5843,&P24670,&P18092 }};	/*Useragent - Nokia201/2.0 (11.56) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; kau; nokia201) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2483002 = {S90815, 138, { &P18701,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-T100K Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2487621 = {S90816, 151, { &P18221,&P5843,&P24675,&P18092 }};	/*Useragent - Nokia302/5.0 (14.53) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia302) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2497035 = {S90817, 152, { &P14255,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4 ICS; en-us; U8150 Build/GRK39F; ICSIdeos 2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2503583 = {S90818, 126, { &P15767,&P17145,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Flipboard/1.9.8*/
const Device UA2823869 = {S90819, 126, { &P15767,&P17145,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Flipboard/1.9.6*/
const Device UA2506765 = {S90820, 137, { &P20505,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; starTIM tab 7 Build/TIM_01) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2525485 = {S90821, 149, { &P13452,&P5096,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2617052 = {S90822, 150, { &P13452,&P5096,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.2; pt-br; GT-P1000L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2617328 = {S90823, 150, { &P13452,&P5096,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-P1000N Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2618034 = {S90824, 147, { &P13452,&P5096,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.2; en-; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2618681 = {S90825, 151, { &P13452,&P5096,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2623516 = {S90826, 150, { &P13452,&P5096,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-P1000T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2528896 = {S90827, 134, { &P13644,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; EVO Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2697278 = {S90828, 161, { &P11636,&P17485,&P24673,&P18092 }};	/*Useragent - SonyEricssonW705/R1GA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonw705) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2555558 = {S90829, 157, { &P21610,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Explay Surfer 7.02 Build/ICS.g12refM703A1HZ1.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2580118 = {S90830, 172, { &P12280,&P18117,&P18693,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B206 Safari/7534.48.3 UCBrowser8.9.0.238/42/355*/
const Device UA2594312 = {S90831, 130, { &P23607,&P18110,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-46EX645; CC/USA; en) Presto/2.10.250 Version/11.60*/
const Device UA2603263 = {S90832, 157, { &P12968,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LT26i Build/6.1.A.2.45) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2603529 = {S90833, 142, { &P20091,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5880D Build/PMP5880D_v1.0.0_X) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2607774 = {S90834, 28, { &P17337,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (GT-P7500 HMJ37)*/
const Device UA2616006 = {S90835, 157, { &P13452,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2619379 = {S90836, 155, { &P13452,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2620542 = {S90837, 157, { &P13452,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2655687 = {S90838, 98, { &P22987,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung gt-c3303) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2698420 = {S90839, 222, { &P22987,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung gt-c3303) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile, SAMSUNG-GT-C3303/C3303DDJG4 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA2661083 = {S90840, 162, { &P19904,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; C5170 Build/IML77) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2666965 = {S90841, 128, { &P20127,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT270 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667174 = {S90842, 133, { &P19860,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A9+ Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2988370 = {S90843, 141, { &P19860,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A9+ Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667188 = {S90844, 140, { &P20708,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XOLO A1000 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668983 = {S90845, 165, { &P22612,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI T8951 Build/HuaweiT8951) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2670603 = {S90846, 137, { &P18021,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A770K Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2699289 = {S90847, 150, { &P22148,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PIRANHA BUSINESS II TAB 9" Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2699932 = {S90848, 90, { &P18633,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; U8185) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2891368 = {S90849, 161, { &P18300,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T769 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2705808 = {S90850, 145, { &P19855,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT885 Build/9.8.3O-148_IRD-3) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2707432 = {S90851, 165, { &P20196,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID7048 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444*/
const Device UA2708788 = {S90852, 132, { &P20274,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IdeaTab_A1107 Build/MR1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2729873 = {S90853, 90, { &P18303,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MT27i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2732727 = {S90854, 139, { &P23498,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V880G Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2737448 = {S90855, 134, { &P23477,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT926 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2737952 = {S90856, 134, { &P20213,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; N9500 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2751050 = {S90857, 97, { &P23016,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Spice_Mi-351) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2751517 = {S90858, 163, { &P19862,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2762189 = {S90859, 155, { &P14854,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; SHW-M250K Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3098263 = {S90860, 155, { &P14854,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; SHW-M250S Build/IMM76D; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2764914 = {S90861, 167, { &P18685,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; U8655 Build/HuaweiU8655; CyanogenMod 7.2 Unofficial) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2768397 = {S90862, 155, { &P20290,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A11 Build/KarbonnA11) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2769733 = {S90863, 125, { &P14111,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Arc Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2772026 = {S90864, 135, { &P18600,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-03D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2781862 = {S90865, 33, { &P17595,&P7480,&P22895,&P18091 }};	/*Useragent - RSSOwl/2.2.0.X (Macintosh; U; en)*/
const Device UA2782255 = {S90866, 156, { &P22394,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; C6603 Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2793998 = {S90867, 96, { &P21980,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; UBISLATE7C+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2794168 = {S90868, 142, { &P21329,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Penta IS703C Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2799004 = {S90869, 137, { &P21481,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A860S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799071 = {S90870, 130, { &P18731,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ONDA MID Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2799281 = {S90871, 130, { &P17168,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7310 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2799704 = {S90872, 163, { &P15809,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; HTC Rhyme S510b Build/GRJ90) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2847301 = {S90873, 130, { &P20595,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; U18GT2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2889619 = {S90874, 154, { &P11699,&P5768,&P24676,&P18092 }};	/*Useragent - Nokia5200/2.0 (05.53) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5200) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2866719 = {S90875, 141, { &P20934,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; 5860E Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2867109 = {S90876, 136, { &P23234,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E976 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867152 = {S90877, 159, { &P18303,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.7; MT27i Build/6.0.B.3.184) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453*/
const Device UA2894835 = {S90878, 158, { &P18452,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT29i Build/9.1.B.1.67) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2877582 = {S90879, 137, { &P19746,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VIVO 4.3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2878302 = {S90880, 154, { &P18637,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9260 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2954598 = {S90881, 154, { &P18637,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9260 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879693 = {S90882, 134, { &P20480,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; JY-G3 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892381 = {S90883, 147, { &P20868,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; M040 Build/JRO03H) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2935936 = {S90884, 155, { &P24564,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; mobiistar touch BEAN 454 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937055 = {S90885, 142, { &P20467,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Goophone i9 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2938117 = {S90886, 155, { &P19891,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2943927 = {S90887, 114, { &P14826,&P5843,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC1-01/06.15; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA2944986 = {S90888, 160, { &P13815,&P5096,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; HTC Wildfire Build/FRG83D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2953554 = {S90889, 228, { &P20172,&P17017,&P18246,&P18092 }};	/*Useragent - CKT50_COSMOS_11B_HW (MRE\2.5.00(2500) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\0 gsensor\0 keyboard\reduced) COCOA-S05A_FLY_L2EN_203_X Release/2012.12.28 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US*/
const Device UA2955973 = {S90890, 164, { &P17262,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; Kindle Fire Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2956341 = {S90891, 176, { &P15699,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Infected HTC Incredible Build/JDQ39; CyanogenMod-INFECTION) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956993 = {S90892, 137, { &P20963,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SBM200SH Build/S1002) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2961367 = {S90893, 93, { &P13974,&P5096,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5670) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2961827 = {S90894, 97, { &P22864,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A26) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2962721 = {S90895, 165, { &P19863,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2963266 = {S90896, 94, { &P18296,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC One V) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2964361 = {S90897, 93, { &P19847,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S6012) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2968787 = {S90898, 160, { &P12135,&P17483,&P24678,&P18092 }};	/*Useragent - LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gd510) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2996431 = {S90899, 131, { &P21383,&P5629,&P20769,&P18092 }};	/*Useragent - Lenovo-A288t_TD/S100 Linux/2.6.35 Android/2.3.5 Release/02.29.2012 Browser/AppleWebkit533.1 Mobile Safari/533.1 UCBrowser/2.3.2.289*/
const Device UA3002903 = {S90900, 167, { &P21657,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI C8950D Build/HuaweiC8950D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3010716 = {S90901, 165, { &P21657,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI T8950 Build/HuaweiT8950) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3011244 = {S90902, 138, { &P24740,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; Fly IQ246 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3015827 = {S90903, 164, { &P12605,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT875 Build/JDQ39E; Carbon-CARBON-JB-NIGHTLY-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3021042 = {S90904, 130, { &P23218,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PMP5780D Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023689 = {S90905, 136, { &P18320,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PadFone Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023757 = {S90906, 129, { &P18320,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PadFone Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3026161 = {S90907, 97, { &P18382,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_Desire_C) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3091806 = {S90908, 97, { &P18382,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_Desire_C) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3079542 = {S90909, 89, { &P11950,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; X10i) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3028166 = {S90910, 165, { &P22612,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI T8951 Build/HuaweiT8951) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3060990 = {S90911, 165, { &P19863,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3080482 = {S90912, 169, { &P19792,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire U dual sim Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3034260 = {S90913, 157, { &P23477,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; XT926 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/xt926*/
const Device UA3093416 = {S90914, 157, { &P23477,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; XT926 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/xt926*/
const Device UA3060347 = {S90915, 170, { &P18460,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-1 Build/HuaweiU8950-1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3058611 = {S90916, 151, { &P18666,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A25 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3058841 = {S90917, 164, { &P12657,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC EVO 3D X515m Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3080922 = {S90918, 155, { &P15575,&P21637,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-I9100 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3059713 = {S90919, 151, { &P21628,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8552 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3061013 = {S90920, 151, { &P22875,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9220 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3094677 = {S90921, 137, { &P18722,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IM-A830L Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3078165 = {S90922, 137, { &P24582,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE T790 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3078968 = {S90923, 92, { &P15028,&P5096,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; LG-P350) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3079061 = {S90924, 156, { &P18002,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; IM-A760S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3079634 = {S90925, 165, { &P19863,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3092684 = {S90926, 105, { &P12871,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Wildfire_S_A510e) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3093263 = {S90927, 159, { &P19999,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A30 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3093623 = {S90928, 103, { &P11932,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6210 Navigator) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3109155 = {S90929, 93, { &P22626,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Iris_349) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3126116 = {S90930, 94, { &P22626,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Iris_349+) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3109394 = {S90931, 160, { &P11747,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire V Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3112137 = {S90932, 159, { &P14969,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC Desire S Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2895 = {S90933, 60, { &P12045,&P5448,&P22270,&P18092 }};	/*Useragent - MOT-A3100/ (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA22875 = {S90934, 59, { &P12045,&P5448,&P22270,&P18092 }};	/*Useragent - MOT-A3100 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA14947 = {S90935, 73, { &P12164,&P7063,&P3606,&P18092 }};	/*Useragent - BlackBerry8707/4.1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/125*/
const Device UA61476 = {S90936, 60, { &P12727,&P5926,&P3605,&P18092 }};	/*Useragent - Nokia6061/2.0 (3.03) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA18088 = {S90937, 52, { &P15995,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player sprint / 1.0 / htc_speedy / 2.2*/
const Device UA49341 = {S90938, 54, { &P15995,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player sprint / 1.0 / htc_speedy / 2.3.3*/
const Device UA687498 = {S90939, 54, { &P15995,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player sprint / 1.0 / htc_speedy / 2.3.4*/
const Device UA22026 = {S90940, 57, { &P15105,&P17017,&P18246,&P18092 }};	/*Useragent - NexianNX-G353/MTK Release/01.01.2010 Browser/MAUI Profile*/
const Device UA22735 = {S90941, 96, { &P12210,&P5026,&P4948,&P18092 }};	/*Useragent - BlackBerry8120/4.5.0.174 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/303 UP.Browser/5.0.3.3*/
const Device UA30506 = {S90942, 67, { &P12328,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia1682c_CMCC/2.0 (06.86) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA62151 = {S90943, 62, { &P12328,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia1682c/2.0 (06.86) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA66925 = {S90944, 138, { &P14733,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; pl-pl; Era G1 Build/DRC92) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA67009 = {S90945, 32, { &P14746,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N705i(c100;TB;W24H16)*/
const Device UA2292575 = {S90946, 35, { &P14746,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N705imyu(c100;TB;W24H16)*/
const Device UA87679 = {S90947, 14, { &P12375,&P5890,&P4030,&P18092 }};	/*Useragent - NOKIANokia N81*/
const Device UA286127 = {S90948, 54, { &P17703,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3O UP.Browser/6.2_7.2.7.1.K.4.306 (GUI) MMP/2.0*/
const Device UA325867 = {S90949, 133, { &P17595,&P17295,&P22449,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.24 (KHTML, like Gecko) Iron/11.0.700.2 Chrome/11.0.700.2 Safari/534.24*/
const Device UA639018 = {S90950, 92, { &P17595,&P17295,&P20459,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:9.0.1) Gecko/X Firefox/9.0.1 SeaMonkey/2.6.1*/
const Device UA647514 = {S90951, 36, { &P16003,&P17484,&P17347,&P18092 }};	/*Useragent - gt-s5233a/UC Browser8.0.3.107/69/352*/
const Device UA695903 = {S90952, 43, { &P14182,&P17484,&P17347,&P18092 }};	/*Useragent - samsung-gt-s5333/UC Browser8.0.3.107/69/444*/
const Device UA731430 = {S90953, 42, { &P12142,&P17485,&P17347,&P18092 }};	/*Useragent - sonyericssonu5i/UC Browser8.0.3.107/69/352*/
const Device UA1937514 = {S90954, 42, { &P12142,&P17485,&P17347,&P18092 }};	/*Useragent - sonyericssonu5i/UC Browser8.0.3.107/69/444*/
const Device UA734254 = {S90955, 102, { &P16463,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; GT-I8350)*/
const Device UA925898 = {S90956, 103, { &P16463,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; GT-I8350T)*/
const Device UA751397 = {S90957, 69, { &P22007,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; GN105; 320*480) UCWEB7.9.3.103/139/32702*/
const Device UA756747 = {S90958, 96, { &P15241,&P5843,&P17347,&P18092 }};	/*Useragent - Nokia300/5.0 (06.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia300/UC Browser8.0.3.107/69/444*/
const Device UA2329228 = {S90959, 79, { &P11689,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokian70-1) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2034522 = {S90960, 165, { &P14433,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA1995358 = {S90961, 82, { &P15364,&P16110,&P22316,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:14.0) Gecko/X Firefox/14.0.1 SeaMonkey/2.11*/
const Device UA2031983 = {S90962, 179, { &P15184,&P17979,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android AcuraTeam 3D MAX V3.1; es-es; LG-P920 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2242304 = {S90963, 65, { &P20225,&P17483,&P17602,&P18092 }};	/*Useragent - LG-A235/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2293821 = {S90964, 101, { &P17330,&P5448,&P4779,&P18092 }};	/*Useragent - HTC-ST7377/2.10.502.4 (93577) Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA2301441 = {S90965, 101, { &P17330,&P5448,&P4779,&P18092 }};	/*Useragent - HTC-ST7377/1.59.502.3 (67150) Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA2327213 = {S90966, 74, { &P15364,&P17017,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; spice) UCBrowser8.3.0.154/69/405/UCWEB Mobile*/
const Device UA2685026 = {S90967, 73, { &P15364,&P17017,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; j2me) UCBrowser8.3.0.154/70/352/UCWEB Mobile*/
const Device UA2351741 = {S90968, 165, { &P11747,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2455042 = {S90969, 168, { &P11747,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; HTC Desire Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2381188 = {S90970, 169, { &P17968,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus one Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2388012 = {S90971, 149, { &P19998,&P18066,&P20791,&P18092 }};	/*Useragent - MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2457630 = {S90972, 161, { &P19998,&P18066,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2395760 = {S90973, 67, { &P18303,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.7;Zh_cn;MT27i;480*854;)UCWEB7.8.0.95/139/444*/
const Device UA2401786 = {S90974, 158, { &P18382,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC Desire C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2429696 = {S90975, 136, { &P18173,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ISW11F Build/FIK700) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2438946 = {S90976, 140, { &P15364,&P17285,&P19994,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; WOW64; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.0.04506; Media Center PC 5.0) Sleipnir/2.7.2*/
const Device UA2465907 = {S90977, 132, { &P23942,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; JMI T970 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2467244 = {S90978, 57, { &P15364,&P17188,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 4.0) Presto/2.12.388 Version/12.11*/
const Device UA2480432 = {S90979, 138, { &P21376,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F120L Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2483869 = {S90980, 146, { &P18455,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One SC T528d Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2696874 = {S90981, 92, { &P11689,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokian70-1) U2/1.0.0 UCBrowser/8.8.0.227 U2/1.0.0 Mobile*/
const Device UA2505386 = {S90982, 70, { &P18437,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; GT-N8000; 800*1232) UCWEB7.9.0.94/139/444*/
const Device UA2526594 = {S90983, 165, { &P21516,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Cloudfone Thrill 400g Build/VENUS_00.03.150.H1105) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2527261 = {S90984, 33, { &P14824,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SAMSUNG-GT-E2252)*/
const Device UA2527598 = {S90985, 134, { &P22022,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; eZeeTab707 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2546345 = {S90986, 59, { &P15364,&P24094,&P18374,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips ; U; en) Presto/2.6.33 Version/10.70*/
const Device UA2559788 = {S90987, 151, { &P19853,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7102 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2563479 = {S90988, 135, { &P12280,&P18432,&P22327,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Rockmelt/2.2.0.241 Mobile/10A525 Safari/8536.25*/
const Device UA2565743 = {S90989, 168, { &P14993,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT15i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2579301 = {S90990, 60, { &P22717,&P5843,&P4037,&P18092 }};	/*Useragent - Nokia109/2.0 (04.10) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2600358 = {S90991, 130, { &P18708,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U30GT-H Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2603545 = {S90992, 139, { &P14225,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Desire HD Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2610612 = {S90993, 95, { &P15985,&P17979,&P22069,&P18092 }};	/*Useragent - Instagram 1.1.1 Android (13/3.2; 160dpi; 600x976; samsung; GT-P6200; GT-P6200; smdkc210; en_GB)*/
const Device UA2624473 = {S90994, 190, { &P17323,&P6437,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.16/276,1.9.16.276,2012-12-21 16:37, +0300, kw*/
const Device UA2635376 = {S90995, 129, { &P17323,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P6800 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.28 Safari/537.31*/
const Device UA2707579 = {S90996, 129, { &P17323,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-p6800 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2642182 = {S90997, 133, { &P18348,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_Velocity_4G_X710s; en-hk) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2721629 = {S90998, 133, { &P18348,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC_Velocity_4G_X710s; zh-tw) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2866380 = {S90999, 93, { &P15364,&P18110,&P21808,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US; rv:1.9.2.15pre) Gecko/X Prism zdesktop/7.2.2*/
const Device UA2660606 = {S91000, 166, { &P18304,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9070 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3108329 = {S91001, 170, { &P18304,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9070 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA2661219 = {S91002, 154, { &P20146,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2667525 = {S91003, 136, { &P20359,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZP300+ Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2671032 = {S91004, 137, { &P15820,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-SU640 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673058 = {S91005, 136, { &P18499,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ISW11SC Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676602 = {S91006, 197, { &P18635,&P18598,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Nexus 4 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, +0100, gb*/
const Device UA2700950 = {S91007, 136, { &P23781,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGT-A22 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2701363 = {S91008, 162, { &P20094,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2850573 = {S91009, 162, { &P20094,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8825D Build/HuaweiC8825D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2702750 = {S91010, 89, { &P18721,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A90S) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2706480 = {S91011, 100, { &P11948,&P17485,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk550i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2707354 = {S91012, 91, { &P21207,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Andi3n) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2707867 = {S91013, 138, { &P18222,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IS12S Build/6.1.D.1.91) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708968 = {S91014, 136, { &P14969,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Desire S Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713870 = {S91015, 133, { &P18314,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M032 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2715160 = {S91016, 144, { &P20104,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; 201M Build/9.8.2Q-34_SMJ-102) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734658 = {S91017, 142, { &P11747,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Desire Build/VJ CM10.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2736354 = {S91018, 66, { &P18254,&P16110,&P17470,&P18092 }};	/*Useragent - X-Client/AppexWin8Microsoft.BingNews X-Client-AppVersion/2.0.0.273*/
const Device UA2747575 = {S91019, 139, { &P20124,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ZTE V970M Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2749858 = {S91020, 140, { &P23154,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOLO_X1000 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2791469 = {S91021, 166, { &P15575,&P18111,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-I9100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2767323 = {S91022, 97, { &P23008,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax_A91) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2768721 = {S91023, 159, { &P22394,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; C6603 Build/10.1.A.1.434) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2868305 = {S91024, 161, { &P22394,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; C6603 Build/10.1.1.A.1.253) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2781398 = {S91025, 93, { &P12456,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian73-1) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA3019366 = {S91026, 92, { &P12456,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE73) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA2781486 = {S91027, 150, { &P18482,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; A75 Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2893469 = {S91028, 157, { &P18661,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; N860 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2781723 = {S91029, 154, { &P12706,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LG-P690 Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2782577 = {S91030, 139, { &P20738,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Andromax U Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783227 = {S91031, 137, { &P21949,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F100L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2833369 = {S91032, 137, { &P21107,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; SO-01D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2791969 = {S91033, 93, { &P15876,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; GT-P5100) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2795004 = {S91034, 152, { &P18558,&P20706,&P24677,&P18092 }};	/*Useragent - Nokia309/2.0 (05.85) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia309) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2895019 = {S91035, 158, { &P15079,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MK16i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2797581 = {S91036, 134, { &P21016,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LGL21 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798629 = {S91037, 143, { &P21802,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GSmart Maya M1 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799068 = {S91038, 97, { &P23008,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax A91) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2799121 = {S91039, 129, { &P18708,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U30GT-H Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2799126 = {S91040, 137, { &P15802,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Amaze 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799869 = {S91041, 137, { &P23582,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; fnac 4.5 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2800107 = {S91042, 161, { &P16292,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2840702 = {S91043, 31, { &P18219,&P5843,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia202/20.28)*/
const Device UA2841619 = {S91044, 31, { &P15241,&P5843,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia300/07.57)*/
const Device UA2848390 = {S91045, 148, { &P23750,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; bg-bg; Privileg MID-7M Build/Privileg_MID-7M) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2854481 = {S91046, 116, { &P12287,&P5043,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia6700c-1/13.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2857529 = {S91047, 98, { &P21358,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; iBall_Andi3.5) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2865134 = {S91048, 97, { &P18652,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A87) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2957901 = {S91049, 90, { &P13018,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; MT11i) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2867244 = {S91050, 128, { &P23899,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT8000 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2867413 = {S91051, 133, { &P21472,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A111 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893844 = {S91052, 160, { &P21738,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; iris502 Build/LAVAIRIS502) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2869819 = {S91053, 144, { &P22998,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TECNO_N7 Build/TECNOTECNO_N7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879691 = {S91054, 100, { &P16384,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5380k) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3009676 = {S91055, 100, { &P16384,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5380d) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2879880 = {S91056, 190, { &P18645,&P22212,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I747 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2938035 = {S91057, 150, { &P18300,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T769 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2938140 = {S91058, 100, { &P18503,&P18148,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; en-US; HTC;  8S by HTC) U2/1.0.0 UCBrowser/3.0.0.285 U2/1.0.0 Mobile*/
const Device UA3018462 = {S91059, 100, { &P18503,&P18148,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; en-US; HTC;  8S by HTC) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile*/
const Device UA2952651 = {S91060, 162, { &P14877,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2957753 = {S91061, 158, { &P18564,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC One X+ Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2958224 = {S91062, 155, { &P13064,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2963264 = {S91063, 159, { &P21602,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; C5302 Build/12.0.A.1.211) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2965396 = {S91064, 160, { &P21141,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Lenovo P770 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2999282 = {S91065, 111, { &P15364,&P16110,&P22527,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.6 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/537.6 Midori/0.5*/
const Device UA3013465 = {S91066, 136, { &P24508,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E445 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3015217 = {S91067, 157, { &P18174,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS920 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3018910 = {S91068, 96, { &P13800,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA5630d-1) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3061015 = {S91069, 144, { &P18017,&P6437,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.2.1; K1 Build/HTK75C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3021510 = {S91070, 132, { &P20274,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IdeaTab_A1107 Build/MR1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022119 = {S91071, 176, { &P22582,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I337 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/jflteatt*/
const Device UA3022961 = {S91072, 144, { &P18554,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Barnes & Noble Nook HD Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3024530 = {S91073, 159, { &P15147,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; ST15i Build/4.0.2.A.0.84) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3027506 = {S91074, 100, { &P16384,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5380k) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3059123 = {S91075, 100, { &P16384,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5380d) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3034249 = {S91076, 93, { &P19847,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S6012) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3081014 = {S91077, 161, { &P22737,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Micromax A115 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3079382 = {S91078, 180, { &P14862,&P5204,&P24055,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311*/
const Device UA3059067 = {S91079, 175, { &P18672,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5302 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3078561 = {S91080, 156, { &P22611,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-S7572 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3079173 = {S91081, 90, { &P15364,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; C2104) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3092261 = {S91082, 94, { &P15364,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; AURUS_III) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3109199 = {S91083, 130, { &P22662,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Enjoy 71 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3093440 = {S91084, 93, { &P14984,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5250) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3094922 = {S91085, 156, { &P19853,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7102 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3106532 = {S91086, 129, { &P22606,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N5110 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3108447 = {S91087, 175, { &P15649,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5570 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3109069 = {S91088, 176, { &P15649,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5570I Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3110065 = {S91089, 161, { &P16436,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I8150 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3110902 = {S91090, 137, { &P19853,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-N7102 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114794 = {S91091, 132, { &P19860,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A9+ Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3130732 = {S91092, 138, { &P19845,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE N9120 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA13241 = {S91093, 95, { &P15944,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Smartphone; 176x220; HPiPAQ510/1.0*/
const Device UA64364 = {S91094, 80, { &P14495,&P16000,&P4615,&P18092 }};	/*Useragent - Nokia3650/1.0 SymbianOS/6.1 Series60/1.2 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA68643 = {S91095, 9, { &P14495,&P16000,&P4615,&P18092 }};	/*Useragent - Nokia3650*/
const Device UA62142 = {S91096, 60, { &P11837,&P6909,&P4030,&P18092 }};	/*Useragent - Nokia7210/1.0 (3.08) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA19965 = {S91097, 138, { &P15017,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; pt-pt; a5 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA20240 = {S91098, 162, { &P14872,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Genocide_0.3a_EB13_OC/UV_Kernel; en-us; SPH-D700 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA21356 = {S91099, 151, { &P14872,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Bonsai/Kernel_v3.0.0; en-us; SPH-D700 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA21357 = {S91100, 158, { &P14872,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Frankenstein E=Bee13 v1.0.1; en-us; SPH-D700 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA21358 = {S91101, 169, { &P14872,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Genocide Kernel 0.2a EB13 1.2GHz OC/UV; en-us; SPH-D700 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24074 = {S91102, 151, { &P14872,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Bonsai/Kernel_v4.0.1; en-us; SPH-D700 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32757 = {S91103, 151, { &P14872,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Bonsai/Kernel_v4.1.1; en-us; SPH-D700 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA33223 = {S91104, 143, { &P14872,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Bonsai/SPH-D700_v4.1.3; en-us Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA43327 = {S91105, 154, { &P14872,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android BeigePhone Epic 4G v1.0; en-us; SPH-D700 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA24536 = {S91106, 72, { &P13921,&P17486,&P4203,&P18092 }};	/*Useragent - Motorola-V3m Obigo/Q04C1 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24646 = {S91107, 76, { &P13921,&P17486,&P4203,&P18092 }};	/*Useragent - Motorola-V3m-Red Obigo/Q04C1 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA41847 = {S91108, 80, { &P13860,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-CG990/CG990_Z4_IN_BFFC107 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C*/
const Device UA45227 = {S91109, 80, { &P13860,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-CG990/CG990_Z0_ID_BFFC102 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C*/
const Device UA45228 = {S91110, 80, { &P13860,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-CG990/CG990_Z1_ID_BFFC102 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C*/
const Device UA60917 = {S91111, 81, { &P13860,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-CG990/CG990_Z11_SZ_BFFC100 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C*/
const Device UA60924 = {S91112, 81, { &P13860,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-CG990/CG990_Z10_NG_BFFC100 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C*/
const Device UA61934 = {S91113, 80, { &P13860,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-CG990/CG990_Z4_IN_BFFC101 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C*/
const Device UA44084 = {S91114, 118, { &P13742,&P5448,&P22270,&P18092 }};	/*Useragent - ZTE Bluebelt/v2/1.0.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA60572 = {S91115, 115, { &P13742,&P5448,&P22270,&P18092 }};	/*Useragent - ZTE Bluebelt/1.0.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA55992 = {S91116, 35, { &P14645,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH905iTV(c100;TB;W24H16)*/
const Device UA56125 = {S91117, 33, { &P14645,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH905i(c100;TB;W24H16)*/
const Device UA62057 = {S91118, 45, { &P14365,&P17484,&P4628,&P18092 }};	/*Useragent - Samsung-SCHU440 Infraware-Polaris/6.0 MMP/2.0*/
const Device UA68069 = {S91119, 36, { &P12449,&P5890,&P4759,&P18092 }};	/*Useragent - NOKIAE5-00/UC Browser7.6.1.82/28/352*/
const Device UA68023 = {S91120, 34, { &P11854,&P5890,&P4759,&P18092 }};	/*Useragent - NokiaE71/UC Browser7.6.1.82/28/351*/
const Device UA563798 = {S91121, 54, { &P17626,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-HI3E UP.Browser/6.2_7.2.7.1.K.2.234 (GUI) MMP/2.0*/
const Device UA186754 = {S91122, 45, { &P17595,&P18120,&P24163,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X; U; en)*/
const Device UA350585 = {S91123, 145, { &P13054,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; HTC Mecha Build/MIUI-Icecream) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA681586 = {S91124, 136, { &P13054,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; HTC Mecha Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA812834 = {S91125, 160, { &P13054,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1 / 2.3.7 Hybrid; en-us; HTC Mecha Build/MIUI-Icecream) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA497000 = {S91126, 162, { &P15364,&P17285,&P21926,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618) chromeframe/8.0.552.224*/
const Device UA2979893 = {S91127, 249, { &P15364,&P17285,&P21926,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB5; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; InfoPath.2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; OfficeLiveConnector.1.5; OfficeLivePatch.1.3) chromeframe/8.0.552.224*/
const Device UA557645 = {S91128, 89, { &P15364,&P17285,&P22346,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:5.0) Gecko/X Firefox/5.0-x64 PaleMoon/5.0-x64*/
const Device UA915886 = {S91129, 76, { &P15364,&P17285,&P22346,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:5.0) Gecko/X Firefox/5.0 PaleMoon/5.0*/
const Device UA574376 = {S91130, 114, { &P15228,&P5043,&P17437,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-03/06.51; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.4.0.34.8*/
const Device UA2025017 = {S91131, 177, { &P12871,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.2.139/145/355*/
const Device UA2025630 = {S91132, 177, { &P12871,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; HTC Wildfire S A510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2890209 = {S91133, 181, { &P12871,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC_Wildfire_S_A510e Build/GRJ90) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA717743 = {S91134, 134, { &P22438,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; TAB460 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA746655 = {S91135, 43, { &P14404,&P17484,&P17347,&P18092 }};	/*Useragent - samsung-gt-s5260/UC Browser8.0.3.107/69/444*/
const Device UA887599 = {S91136, 122, { &P15364,&P17285,&P22429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 Comodo_Dragon/19.0.3.0*/
const Device UA889818 = {S91137, 129, { &P15364,&P17285,&P22429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 Comodo_Dragon/19.0.3.0*/
const Device UA910801 = {S91138, 40, { &P17968,&P17979,&P24435,&P18092 }};	/*Useragent - Dolphin http client/8.5.1(152) (Android)*/
const Device UA1898459 = {S91139, 76, { &P15364,&P7127,&P22311,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:6.0.2) Gecko/X Firefox/6.0.2 SeaMonkey/2.3.3*/
const Device UA1901729 = {S91140, 114, { &P14711,&P5204,&P17437,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-01/10.50; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.4.0.34.8*/
const Device UA1948051 = {S91141, 52, { &P12449,&P5890,&P21754,&P18092 }};	/*Useragent - iBrowser/2.4 (NokiaE5-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA1954918 = {S91142, 139, { &P14161,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-VM670 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2022397 = {S91143, 160, { &P14161,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; LG-VM670 Build/ICS40; IceCreamSandwich-Final) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1986422 = {S91144, 69, { &P15364,&P18110,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Linux; Opera Mobi/2; U; en) Presto/2.10.254 Version/12.00*/
const Device UA1975832 = {S91145, 98, { &P11689,&P5776,&P23404,&P18092 }};	/*Useragent - MQQBrowser/Mini2.7 (NokiaN70-1/5.0609.2.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1 )*/
const Device UA2020558 = {S91146, 133, { &P14255,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; U8150 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2541950 = {S91147, 133, { &P14255,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Ideos Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021907 = {S91148, 157, { &P15575,&P5629,&P20775,&P18092 }};	/*Useragent - MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2441632 = {S91149, 81, { &P15575,&P5629,&P20775,&P18092 }};	/*Useragent - MQQBrowser/24 (Linux; U; 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD.ZQKJ6;480*800)*/
const Device UA2028143 = {S91150, 127, { &P12082,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; X10 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2169798 = {S91151, 35, { &P14669,&P5843,&P21759,&P18092 }};	/*Useragent - iBrowser/Mini2.8 (NokiaX2-00/08.25)*/
const Device UA2245613 = {S91152, 64, { &P15364,&P16110,&P24197,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.2) QQBrowser/6.0*/
const Device UA2320295 = {S91153, 132, { &P12848,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; MB526 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2722773 = {S91154, 125, { &P12848,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MB526 Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2335115 = {S91155, 152, { &P24573,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TURKCELL MaxiPRO5 Build/HuaweiU8860) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2338588 = {S91156, 85, { &P15198,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s3650) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2366962 = {S91157, 173, { &P14842,&P5477,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E10i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2373775 = {S91158, 70, { &P18304,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;GT-I9070;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA2933629 = {S91159, 35, { &P14635,&P5843,&P21759,&P18092 }};	/*Useragent - iBrowser/Mini2.8 (NokiaC3-00/08.65)*/
const Device UA2413790 = {S91160, 103, { &P16305,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; ZTE; MegaFon SP-W1)*/
const Device UA2425023 = {S91161, 157, { &P16356,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2428709 = {S91162, 89, { &P12856,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-t300) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2456485 = {S91163, 89, { &P12856,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-t300) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2846037 = {S91164, 86, { &P12856,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; vi; lg-t300) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2433661 = {S91165, 62, { &P15364,&P18144,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64; rv:17.0) Gecko/X Firefox/17.0*/
const Device UA2439376 = {S91166, 120, { &P15364,&P16110,&P22453,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.1 Chrome/15.0.900.1 Safari/535.2*/
const Device UA2818527 = {S91167, 81, { &P15845,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaC5-06)/UC Browser8.5.0.183/50/355/UCWEB Mobile*/
const Device UA2501593 = {S91168, 154, { &P18283,&P18111,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-I9300 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2850500 = {S91169, 154, { &P18283,&P18111,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2502632 = {S91170, 151, { &P18241,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; LG-E400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2507743 = {S91171, 133, { &P21613,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Aurora-II Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2529917 = {S91172, 148, { &P20221,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Starmobile CRYSTAL Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2530080 = {S91173, 139, { &P21892,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; X6-7A MID Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2531302 = {S91174, 173, { &P12280,&P18432,&P18693,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0_1 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10A523 Safari/7534.48.3 UCBrowser8.9.0.238/44/355*/
const Device UA2554830 = {S91175, 154, { &P20983,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9308 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556858 = {S91176, 137, { &P18645,&P24698,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L710 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2558093 = {S91177, 89, { &P13602,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; vi; gt-s5233w) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2560467 = {S91178, 92, { &P13602,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233w) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2855084 = {S91179, 92, { &P13602,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; fr-FR; gt-s5233w) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2559654 = {S91180, 91, { &P13734,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5603) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2565463 = {S91181, 163, { &P18556,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BNTV600 Build/2.0.6.6008.magnesium02.ovation.rrdf.s1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2577824 = {S91182, 69, { &P18297,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GT-I8530; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2580163 = {S91183, 134, { &P18314,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M032 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2583567 = {S91184, 146, { &P21489,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Infected Rezound Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2584856 = {S91185, 137, { &P14017,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-P990 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2603538 = {S91186, 138, { &P21949,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F100L Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2619366 = {S91187, 53, { &P15985,&P17979,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.5023 Android/3.2 Device/samsung-GT-P6200*/
const Device UA2630527 = {S91188, 151, { &P12879,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P7510 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2633122 = {S91189, 151, { &P12879,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2993519 = {S91190, 155, { &P12879,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA2663763 = {S91191, 167, { &P20076,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT26w Build/6.1.A.2.55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2663991 = {S91192, 142, { &P22608,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Spice Mi-530 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2694240 = {S91193, 145, { &P12528,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Tablet PC szenio 8100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2987365 = {S91194, 136, { &P12528,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N8100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695661 = {S91195, 101, { &P18623,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 510)*/
const Device UA2706568 = {S91196, 136, { &P20200,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS GAMEPAD Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2706772 = {S91197, 130, { &P19879,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I915 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2727218 = {S91198, 180, { &P20134,&P5629,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD; 320*480) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444*/
const Device UA2730875 = {S91199, 93, { &P13452,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; GT-P1000) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2733425 = {S91200, 135, { &P22875,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.9; I9220 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2863965 = {S91201, 138, { &P22875,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-i9220 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2734478 = {S91202, 148, { &P20175,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2744132 = {S91203, 136, { &P22671,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; eZee'Tab1001 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2745218 = {S91204, 136, { &P22671,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; eZee'Tab1001 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2747508 = {S91205, 225, { &P18088,&P22840,&P18246,&P18092 }};	/*Useragent - TINNO50_6464_W1224_11B_HW (MRE\2.5.00(800) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\1 gsensor\0 keyboard\normal) F7013A_T28TV_X_V10 Release/2012.10.20 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/  Q03C1-2.40 en-US*/
const Device UA2768227 = {S91206, 155, { &P15880,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MB865 Build/unknown; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2778614 = {S91207, 144, { &P21320,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; SNM-LT25c Build/9.0.E.1.80) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2779556 = {S91208, 92, { &P17968,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; PIONEER) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2852443 = {S91209, 96, { &P17968,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; huaqin73_gb) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2781799 = {S91210, 96, { &P13443,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; SamsungI8910) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2782810 = {S91211, 138, { &P18222,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IS12S Build/6.1.D.1.91) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937424 = {S91212, 150, { &P20158,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A7 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2797446 = {S91213, 157, { &P18636,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; VS930 4G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2798823 = {S91214, 126, { &P18248,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A700 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2798898 = {S91215, 133, { &P18008,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HYUNDAI T10 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2800151 = {S91216, 137, { &P22646,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Dream_D1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2800683 = {S91217, 130, { &P19877,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T869 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2865082 = {S91218, 135, { &P20350,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; AirTab M975w Build/M975W) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2958533 = {S91219, 163, { &P14842,&P5477,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E10i Build/2.1.1.A.0.6) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2866614 = {S91220, 163, { &P15483,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC ChaCha A810e Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2957452 = {S91221, 160, { &P12616,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.4; WT19i Build/4.0.2.A.0.62) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453*/
const Device UA2937519 = {S91222, 137, { &P16757,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I500 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868895 = {S91223, 165, { &P19891,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I717 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2937394 = {S91224, 97, { &P21475,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A35) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2892605 = {S91225, 143, { &P12216,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6300 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2935156 = {S91226, 96, { &P18557,&P20706,&P17347,&P18092 }};	/*Useragent - Nokia308/2.0 (05.85) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia308/UC Browser8.0.3.107/69/444*/
const Device UA2937975 = {S91227, 152, { &P21472,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A111 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2937081 = {S91228, 113, { &P15785,&P5043,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2010/11.95; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2937493 = {S91229, 171, { &P21968,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; ALCATEL ONE TOUCH 4030D Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2944694 = {S91230, 97, { &P18652,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A87) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2945799 = {S91231, 156, { &P18679,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-B5330 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2952920 = {S91232, 144, { &P21097,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Mobiistar S01 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958232 = {S91233, 148, { &P18324,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/ZV5.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2958491 = {S91234, 139, { &P17108,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Inspire 4G Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2958496 = {S91235, 143, { &P17108,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Inspire 4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2961465 = {S91236, 161, { &P21882,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax P362 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2991844 = {S91237, 154, { &P18195,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; YP-G70 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996787 = {S91238, 155, { &P18195,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; YP-G70 Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998648 = {S91239, 150, { &P17324,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; de-de; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA3009461 = {S91240, 142, { &P24112,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; i-mobile IQ X Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3010241 = {S91241, 176, { &P18506,&P5043,&P22881,&P18092 }};	/*Useragent - Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291*/
const Device UA3014022 = {S91242, 133, { &P18304,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-I9070 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3015062 = {S91243, 96, { &P13979,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; Nokia6220c-1) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3015533 = {S91244, 90, { &P15364,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; C2104) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3123766 = {S91245, 89, { &P15364,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; zh-CN; M040) U2/1.0.0 UCBrowser/9.1.1.309 U2/1.0.0 Mobile*/
const Device UA3015912 = {S91246, 153, { &P15364,&P5690,&P23486,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.6.1.X (Second Life Release; default skin) Safari/533.3*/
const Device UA3016126 = {S91247, 178, { &P18267,&P5043,&P22881,&P18092 }};	/*Useragent - NokiaX2-02/2.0 (11.79) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291*/
const Device UA3079719 = {S91248, 152, { &P22754,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HP Slate 7 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3017481 = {S91249, 152, { &P20231,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A21 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3079576 = {S91250, 159, { &P15147,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST15i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3022296 = {S91251, 137, { &P14877,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9000 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023306 = {S91252, 134, { &P23217,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP7170B3GCE Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3078523 = {S91253, 132, { &P23217,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP7170B3G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023428 = {S91254, 90, { &P18303,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; MT27i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3023679 = {S91255, 143, { &P23223,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; PMP5588C Build/E8HD-ELITE-1280x768) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3026035 = {S91256, 88, { &P20515,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; A54) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3110600 = {S91257, 180, { &P22597,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.6.1;FBBV/X;FBDM/{density=3.0,width=1080,height=1920};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/SCH-I545;FBSV/4.2.2;FBCA/armeabi-v7a:armeabi;]*/
const Device UA3109847 = {S91258, 131, { &P21501,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; P5 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066940 = {S91259, 157, { &P18280,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; One S Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/ville*/
const Device UA3078374 = {S91260, 132, { &P18734,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AN10BG2I Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3126566 = {S91261, 137, { &P20983,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9308 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3083928 = {S91262, 139, { &P18696,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LRA Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3093485 = {S91263, 160, { &P19856,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; LG-LS970 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/ls970*/
const Device UA3129259 = {S91264, 160, { &P19856,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; LG-LS970 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/ls970*/
const Device UA3110679 = {S91265, 112, { &P19972,&P5043,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia113/03.32; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA3128651 = {S91266, 153, { &P18385,&P21637,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-US; Nexus 7 Build/JWR66Y) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.1.344 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3130736 = {S91267, 137, { &P18301,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS840 4G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132847 = {S91268, 136, { &P18301,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS840PP Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131801 = {S91269, 129, { &P24367,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5200 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA2224 = {S91270, 20, { &P17542,&P17017,&P17470,&P18092 }};	/*Useragent - Fly DS160, Fly DS160*/
const Device UA18725 = {S91271, 9, { &P17542,&P17017,&P17470,&P18092 }};	/*Useragent - Fly DS160*/
const Device UA70007 = {S91272, 38, { &P17542,&P17017,&P17470,&P18092 }};	/*Useragent - Fly_DS170 Configuration/WAP2.0/Browser*/
const Device UA2671 = {S91273, 69, { &P16213,&P5448,&P22270,&P18092 }};	/*Useragent - Xda_orbit_2/240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA44368 = {S91274, 68, { &P16213,&P5448,&P22270,&P18092 }};	/*Useragent - Xda Orbit; 240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA5097 = {S91275, 86, { &P13348,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-GT-B7620/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA9298 = {S91276, 67, { &P13405,&P6712,&P4617,&P18092 }};	/*Useragent - Nokia2322c_CMCC/2.0 (08.20) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA16362 = {S91277, 86, { &P16860,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-SCH-M715/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA21827 = {S91278, 121, { &P15351,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) ASUS-GalaxyMini/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA33299 = {S91279, 102, { &P15364,&P5208,&P4902,&P18092 }};	/*Useragent - Mozilla/5.0 (YL-COOLPAD_E600/10.14.E600;U;REX/1.0;BREW/3.1.5SP01;240*320;CTC/2.0) POLARIS/V6.15.090923*/
const Device UA2134256 = {S91280, 97, { &P15364,&P5208,&P4902,&P18092 }};	/*Useragent - Mozilla/5.0 (YL-Coolpad_D510/03.17.D510;U;REX/1.0;BREW/3.1.5SP01;240*320;CTC/2.0) POLARIS/v6.15.X*/
const Device UA2291915 = {S91281, 97, { &P15364,&P5208,&P4902,&P18092 }};	/*Useragent - Mozilla/5.0 (YL-COOLPAD_E600/10.14.E600;U;REX/1.0;BREW/3.1.5SP01;240*320;CTC/2.0) POLARIS/V6.15.X*/
const Device UA62233 = {S91282, 40, { &P16413,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP501 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA84096 = {S91283, 8, { &P13318,&P5107,&P17470,&P18092 }};	/*Useragent - VX8300v1*/
const Device UA178674 = {S91284, 133, { &P11702,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB300 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA787658 = {S91285, 36, { &P15209,&P5768,&P17347,&P18092 }};	/*Useragent - nokia6233/UC Browser8.0.3.107/69/444*/
const Device UA2588314 = {S91286, 64, { &P14533,&P6173,&P3417,&P18092 }};	/*Useragent - BlackBerry9630/4.2.0.451 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA452328 = {S91287, 71, { &P15364,&P18110,&P22313,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:7.0) Gecko/X Firefox/7.0 SeaMonkey/2.4*/
const Device UA522040 = {S91288, 119, { &P15364,&P18346,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 4.10; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/1.0.154.53 Safari/525.19*/
const Device UA677973 = {S91289, 96, { &P15364,&P18110,&P22482,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6.2*/
const Device UA537567 = {S91290, 77, { &P15364,&P5690,&P22334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:2.0) Gecko/X Firefox/4.0 CometBird/4.0*/
const Device UA2258891 = {S91291, 73, { &P15364,&P5690,&P22334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:2.0) Gecko/X Firefox/4.0 CometBrowser/4.0*/
const Device UA572647 = {S91292, 106, { &P12502,&P17485,&P17338,&P18092 }};	/*Useragent - SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonw20i/UC Browser7.9.0.102/69/352*/
const Device UA588762 = {S91293, 76, { &P15364,&P17285,&P22353,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:9.1) Gecko/X Firefox/9.1 PaleMoon/9.1*/
const Device UA1948664 = {S91294, 89, { &P15364,&P17285,&P22353,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:9.1) Gecko/X Firefox/9.1-x64 PaleMoon/9.1-x64*/
const Device UA690537 = {S91295, 88, { &P15364,&P22195,&P22643,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) KHTML/4.7.2 (like Gecko) Konqueror/4.7 Fedora/4.7.2-5.fc16*/
const Device UA760363 = {S91296, 90, { &P15364,&P22195,&P22643,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) KHTML/4.7.2 (like Gecko) Konqueror/4.7 Fedora/4.7.2-5.fc16*/
const Device UA830187 = {S91297, 92, { &P15364,&P22186,&P18097,&P18092 }};	/*Useragent - Mozilla/5.1 (X11; U; Linux i686; en-US; rv:1.9.0.1) Gecko/X Fedora/3.0.1-1.fc9 Firefox/9.0.1*/
const Device UA833617 = {S91298, 138, { &P12280,&P18117,&P20779,&P18092 }};	/*Useragent - MQQBrowser/28 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA1985731 = {S91299, 136, { &P12280,&P18117,&P20779,&P18092 }};	/*Useragent - MQQBrowser/28 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3*/
const Device UA1987560 = {S91300, 136, { &P12280,&P18117,&P20779,&P18092 }};	/*Useragent - MQQBrowser/28 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Safari/7534.48.3*/
const Device UA1906411 = {S91301, 153, { &P17108,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Inspire 4G Build/GRI40; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1944291 = {S91302, 82, { &P15364,&P17285,&P22352,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:9.0.1) Gecko/X Firefox/9.0.1 PaleMoon/9.0.1*/
const Device UA1945760 = {S91303, 95, { &P15364,&P17285,&P22352,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:9.0.1) Gecko/X Firefox/9.0.1-x64 PaleMoon/9.0.1-x64*/
const Device UA1961150 = {S91304, 139, { &P21068,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MI-ONE C1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2018307 = {S91305, 136, { &P20158,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Elocity A7 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018838 = {S91306, 131, { &P20158,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; th-th; A7 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019582 = {S91307, 154, { &P12567,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; Panasonic SV-ME970 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2517411 = {S91308, 154, { &P12567,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; Panasonic SV-MV100 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2027572 = {S91309, 172, { &P15649,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2934895 = {S91310, 176, { &P15649,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5570I Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2023710 = {S91311, 136, { &P21265,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; es-es; OnePAD 730 Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025955 = {S91312, 134, { &P21164,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; ZP200 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026685 = {S91313, 129, { &P20580,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; AT830 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2157402 = {S91314, 54, { &P14033,&P5890,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8 (NokiaC5-03;SymbianOS/9.1 Series60/3.0)*/
const Device UA2157976 = {S91315, 47, { &P14748,&P5890,&P17347,&P18092 }};	/*Useragent - nokiae72-1/UC Browser8.0.3.107/69/444  3gpp-gba*/
const Device UA2165934 = {S91316, 80, { &P17595,&P18207,&P18514,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:13.0) Gecko/X Thunderbird/13.0.1*/
const Device UA2217417 = {S91317, 81, { &P15364,&P18110,&P21956,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1b1pre) Gecko/X Minefield/3.1b1pre*/
const Device UA2315962 = {S91318, 81, { &P15364,&P16110,&P21171,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:12.0) Gecko/X Firefox/12.0 SeaMonkey/2.9.1*/
const Device UA2353363 = {S91319, 137, { &P15364,&P17819,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/11.10 Chromium/22.0.1229.79 Chrome/22.0.1229.79 Safari/537.4*/
const Device UA2455705 = {S91320, 135, { &P15364,&P17819,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/11.10 Chromium/22.0.1229.79 Chrome/22.0.1229.79 Safari/537.4*/
const Device UA2362408 = {S91321, 159, { &P17595,&P18120,&P19964,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.2.2.29837 (Firestorm-Release; firestorm skin) Safari/533.3*/
const Device UA2372833 = {S91322, 167, { &P22062,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Onda Vi40 Elite - Top Notch Tablets Edition Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2385197 = {S91323, 121, { &P12671,&P5055,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; Nokia700) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/51/355/UCWEB Mobile*/
const Device UA2384584 = {S91324, 151, { &P12280,&P17090,&P20788,&P18092 }};	/*Useragent - MQQBrowser/371 Mozilla/5.0 (iPhone 4; U; CPU iPhone OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3*/
const Device UA2692985 = {S91325, 150, { &P12280,&P17090,&P20788,&P18092 }};	/*Useragent - MQQBrowser/37 Mozilla/5.0 (iPhone 4; U; CPU iPhone OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3*/
const Device UA2425398 = {S91326, 92, { &P11689,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokian70-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2431369 = {S91327, 90, { &P11689,&P5890,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokian70) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2399265 = {S91328, 137, { &P16436,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I8150 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2727162 = {S91329, 160, { &P20158,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2404486 = {S91330, 91, { &P13602,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5233w) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2426853 = {S91331, 91, { &P13602,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5233w) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2482714 = {S91332, 88, { &P13602,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; vi; gt-s5233w) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2594507 = {S91333, 88, { &P13602,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; vi; gt-s5233w) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA3118387 = {S91334, 91, { &P13602,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; fr-fr; gt-s5233w) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2410799 = {S91335, 72, { &P21613,&P18110,&P20793,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; Aurora-II; 1024*552) UCWEB1.0.1.113/147/800*/
const Device UA2411324 = {S91336, 137, { &P20242,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; LG-KU8800 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2424400 = {S91337, 151, { &P13974,&P5096,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5670 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2430094 = {S91338, 139, { &P20852,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Accord711 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2430822 = {S91339, 93, { &P20069,&P22784,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxx78) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2449942 = {S91340, 136, { &P23427,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; TM-3200R Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2451340 = {S91341, 132, { &P20727,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; T580 Build/X_8461_V0526) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2455534 = {S91342, 140, { &P17691,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I957 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2457473 = {S91343, 160, { &P20183,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2466059 = {S91344, 139, { &P23947,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Alpha GTR Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2480778 = {S91345, 141, { &P20133,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 201HW Build/HuaweiU9201L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2485537 = {S91346, 169, { &P15147,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; ST15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2503142 = {S91347, 127, { &P12375,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; N81 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2506961 = {S91348, 147, { &P20731,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4(6577); Blaze S180  Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2530305 = {S91349, 136, { &P22085,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ThL W5 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2531144 = {S91350, 153, { &P20541,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAP4500DUO Build/PrestigioPAP4500DUO) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2547809 = {S91351, 145, { &P21837,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Woxter Tablet PC 50BL Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2554721 = {S91352, 153, { &P21396,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A65 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2555664 = {S91353, 69, { &P20175,&P18110,&P24673,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; MI 1S; 480*854) UCWEB8.7.4.225/145/32693*/
const Device UA2558376 = {S91354, 100, { &P14060,&P17485,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw150i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2952266 = {S91355, 97, { &P14060,&P17485,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; fa; sonyericssonw150i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2583804 = {S91356, 163, { &P18321,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-S5300 Build/GINGERBREAD) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2706840 = {S91357, 162, { &P18321,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-S5300 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Mobile Safari/537.31 OPR/14.0.1074.54070*/
const Device UA2727260 = {S91358, 161, { &P20120,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A45 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2595058 = {S91359, 138, { &P21836,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TM-4504 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597606 = {S91360, 167, { &P23156,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; XOLO A700 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2599951 = {S91361, 139, { &P23447,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Newman N2 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2601609 = {S91362, 143, { &P21885,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GSmart GS202+ Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602037 = {S91363, 139, { &P20926,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT-AS40SE Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602440 = {S91364, 143, { &P11747,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Desire Build/VJ CM10.1) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2613889 = {S91365, 130, { &P22042,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build/IMM76D) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2617793 = {S91366, 132, { &P22042,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2619060 = {S91367, 61, { &P22042,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P3108 Build/IMM76D)*/
const Device UA2621212 = {S91368, 130, { &P22042,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build IMM76D) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA2622539 = {S91369, 152, { &P22042,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build/IMM76D; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2659511 = {S91370, 95, { &P20033,&P22784,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx368) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2659558 = {S91371, 92, { &P14294,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia5238) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2686671 = {S91372, 93, { &P17595,&P18207,&P18198,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en-US; rv:1.9.2.13) Gecko/X Thunderbird/3.1.7*/
const Device UA2696897 = {S91373, 144, { &P20181,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-us; Swipe_Fablet_F1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2700995 = {S91374, 93, { &P19938,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; SCH-I589) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2708907 = {S91375, 140, { &P20001,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo S720 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709357 = {S91376, 136, { &P22660,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P778 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2729739 = {S91377, 139, { &P23034,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; FLY F41s Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747963 = {S91378, 140, { &P16553,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T959V Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2748129 = {S91379, 153, { &P15184,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P920 Build/GRJ90; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2748710 = {S91380, 90, { &P15047,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; SK17i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2751413 = {S91381, 151, { &P20081,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7108 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2994593 = {S91382, 151, { &P20081,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7108 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2751879 = {S91383, 137, { &P22875,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9220 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2767702 = {S91384, 137, { &P22875,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-i9220 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2751949 = {S91385, 151, { &P20096,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7568 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2759847 = {S91386, 72, { &P21441,&P18110,&P20793,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; MOMO9_STAR; 800*432) UCWEB1.0.1.113/147/800*/
const Device UA2985054 = {S91387, 71, { &P21441,&P18110,&P20793,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; MOMO9star; 800*432) UCWEB1.0.1.146/147/800*/
const Device UA2894214 = {S91388, 156, { &P18509,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LG-MS770 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2769171 = {S91389, 134, { &P22640,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IQ446 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769535 = {S91390, 137, { &P22162,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AUXUS CoreX4 3G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2769775 = {S91391, 71, { &P15364,&P16110,&P22169,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:21.0) Gecko/X Firefox/21.0 slurp*/
const Device UA2769885 = {S91392, 140, { &P19846,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SO-05D Build/7.0.D.1.137) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2779990 = {S91393, 92, { &P21737,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; IRIS355) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2784247 = {S91394, 138, { &P15160,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TP7-1000DC light Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2790404 = {S91395, 164, { &P18622,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Tomatoes Version II; vi-vn; IM-A820L Build/Tomatoes Version II) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2844644 = {S91396, 159, { &P18622,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Tomatoes Final; vi-vn; IM-A820L Build/Tomatoes Version II) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2859225 = {S91397, 136, { &P23315,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-E930 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2861200 = {S91398, 134, { &P20556,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A701 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2863371 = {S91399, 127, { &P20556,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A701 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2865416 = {S91400, 165, { &P21274,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Intex Aqua Marvel Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2866775 = {S91401, 132, { &P23389,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; LT7035-D Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2867400 = {S91402, 131, { &P23388,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TSINTECH Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2867966 = {S91403, 134, { &P21112,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; X9017 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892887 = {S91404, 133, { &P21112,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; X907 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868139 = {S91405, 92, { &P18621,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; LG-E612) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2870025 = {S91406, 106, { &P12088,&P5129,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaX6-00*/
const Device UA2870317 = {S91407, 135, { &P22909,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Flame2 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879478 = {S91408, 135, { &P23504,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; UMI X2 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2890050 = {S91409, 146, { &P23769,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 7020 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2934427 = {S91410, 143, { &P20736,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GOCLEVER TAB A104.2 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2937218 = {S91411, 140, { &P20292,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC T528t Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2938583 = {S91412, 159, { &P20001,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo S720 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2958456 = {S91413, 93, { &P20141,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S5301) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA3000528 = {S91414, 156, { &P23003,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; TECNO Q1 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2956851 = {S91415, 120, { &P18254,&P7133,&P17201,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Win64; x64; Trident/6.0) (TARGUSinfo Web Crawler www.targusinfo.com)*/
const Device UA2957886 = {S91416, 173, { &P21857,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2959383 = {S91417, 136, { &P22209,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; Titanium S5 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2960707 = {S91418, 139, { &P24169,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Primo H2 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996425 = {S91419, 144, { &P23242,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SKY HD9500 Plus Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2997316 = {S91420, 139, { &P23242,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SKY HD9500 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3068993 = {S91421, 109, { &P18445,&P18148,&P22782,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows Phone 8.0; ARM; Trident/7.0; Touch; rv:11.0; IEMobile/11.0; NOKIA; Lumia 920) like Gecko*/
const Device UA2997552 = {S91422, 163, { &P23207,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A4+ Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33756*/
const Device UA2998024 = {S91423, 137, { &P21309,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; myPad P4 Lite Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2999862 = {S91424, 140, { &P22033,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; de-de; SP-140 DC Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3003627 = {S91425, 114, { &P14711,&P5204,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-01/11.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA3006197 = {S91426, 135, { &P21605,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-in; C2104 Build/15.0.A.1.31) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3125652 = {S91427, 135, { &P21605,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-in; C2104 Build/15.0.A.2.17) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3013059 = {S91428, 157, { &P23496,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D2-927G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA3061404 = {S91429, 158, { &P15364,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TA-M72CBT Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3017604 = {S91430, 159, { &P15517,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT15i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3021099 = {S91431, 137, { &P18458,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I200 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023891 = {S91432, 88, { &P23245,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A3+) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3048456 = {S91433, 152, { &P24072,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033A Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3056145 = {S91434, 133, { &P22382,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-I8262 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3056937 = {S91435, 96, { &P21474,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.9; en-US; Celkon_A_85) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3058185 = {S91436, 137, { &P18385,&P21637,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon*/
const Device UA3108769 = {S91437, 137, { &P18385,&P21637,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon*/
const Device UA3058589 = {S91438, 156, { &P22603,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-I739 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3059198 = {S91439, 149, { &P24365,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SM-T211 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3060689 = {S91440, 162, { &P21068,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; MI-ONE C1 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3061037 = {S91441, 152, { &P20541,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAP4500DUO Build/PrestigioPAP4500DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3061505 = {S91442, 137, { &P14959,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9103 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3064421 = {S91443, 93, { &P14186,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3109431 = {S91444, 188, { &P15794,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3092841 = {S91445, 178, { &P14635,&P5843,&P24055,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (08.71) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311*/
const Device UA3111212 = {S91446, 138, { &P20925,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT-AS45SE Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3115330 = {S91447, 150, { &P22606,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3131732 = {S91448, 137, { &P18636,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; VS930 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132908 = {S91449, 144, { &P23698,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I527 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA2481 = {S91450, 70, { &P11980,&P5448,&P22271,&P18092 }};	/*Useragent - HTC_P3300 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA5202 = {S91451, 111, { &P12224,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)  PPC; 240x320; Toshiba  G810/RL1_081018_092ES2_61*/
const Device UA28145 = {S91452, 89, { &P12224,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)  PPC; 240x320; Toshiba G810*/
const Device UA36894 = {S91453, 111, { &P12224,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)  PPC; 240x320; Toshiba  G810/RL1_081105_093ES2_61*/
const Device UA45777 = {S91454, 90, { &P12224,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)  PPC; 240x320; Toshiba  G810*/
const Device UA45778 = {S91455, 158, { &P12224,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)  PPC; 240x320; Toshiba G810/ ver RL1_20090425_096IL1_61/ Windows CE; IE Mobile 7.11; Mozilla/4.0*/
const Device UA6438 = {S91456, 158, { &P12311,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android SDX.FROYO.DL25.OMEGA1_2.2.1; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA6439 = {S91457, 146, { &P12311,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android UbuntDroid_v5.1; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA11923 = {S91458, 147, { &P12311,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Basic_DL05_2.2.1; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA37680 = {S91459, 155, { &P12311,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android SDX_FROYO_DL25_RC4_2.2.1; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40583 = {S91460, 145, { &P12311,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android KeyLime09 v1.0; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA50966 = {S91461, 151, { &P12311,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android FROYO_DL25_RC4_2.2.1; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55904 = {S91462, 153, { &P12311,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Ginger Fresh EB28 V1.0; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55906 = {S91463, 145, { &P12311,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android KeyLime28 v3.3; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55908 = {S91464, 153, { &P12311,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Ginger Fresh EB28 V1.5; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55909 = {S91465, 188, { &P12311,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android KeyLime28 v4.0ro.build.date=2011. 06. 21. (i 22:11:41 KST; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55910 = {S91466, 147, { &P12311,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android KeyLime28 v3.2.0; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA71859 = {S91467, 151, { &P12311,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android VULGARom Naked v1.1a; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369210 = {S91468, 144, { &P12311,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android ICM v3.3 EB28; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022447 = {S91469, 146, { &P12311,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android JROM EF09 2.2.2; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA88557 = {S91470, 75, { &P11646,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW595/R3DA Profile/MIDP-2.1 Configuration/CLDC-1.1 UNTRUSTED/1.0*/
const Device UA22024 = {S91471, 73, { &P14308,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Dopod838Pro*/
const Device UA32157 = {S91472, 39, { &P12471,&P17486,&P3807,&P18092 }};	/*Useragent - Mot-WX390/1.0 ObigoInternetBrowser/Q03C*/
const Device UA43712 = {S91473, 85, { &P13728,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-MDTOUCHMINI(3ITA)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64595 = {S91474, 101, { &P13227,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-D820/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA45816 = {S91475, 59, { &P14333,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-505/1.0 model-Bouygues ObigoInternetBrowser/Q03C*/
const Device UA59619 = {S91476, 59, { &P14333,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-505/1.0 version 010 03 ObigoInternetBrowser/Q03C*/
const Device UA61804 = {S91477, 44, { &P14333,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-505/1.0 ObigoInternetBrowser/Q03C*/
const Device UA50427 = {S91478, 102, { &P11689,&P5776,&P4737,&P18092 }};	/*Useragent - NokiaN70-1/5.0737.3.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.2.2.51/27/351*/
const Device UA50445 = {S91479, 16, { &P13256,&P17811,&P17810,&P18092 }};	/*Useragent - LENOVO-i921/240A*/
const Device UA63204 = {S91480, 63, { &P14487,&P22839,&P3601,&P18092 }};	/*Useragent - ZTE-F900/1.0 ACS-NF/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64659 = {S91481, 74, { &P14690,&P7063,&P3606,&P18092 }};	/*Useragent - BlackBerry7100i/4.1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/103*/
const Device UA66645 = {S91482, 52, { &P16413,&P17483,&P17660,&P18092 }};	/*Useragent - LGKP501/v1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA543879 = {S91483, 99, { &P14540,&P5843,&P4759,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-01/UC Browser7.6.1.82/69/351*/
const Device UA68821 = {S91484, 36, { &P14033,&P5129,&P4777,&P18092 }};	/*Useragent - NokiaC5-03/UC Browser7.7.1.88/50/444*/
const Device UA69145 = {S91485, 88, { &P11978,&P5448,&P4777,&P18092 }};	/*Useragent - HTC_Touch2_T3333 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA282102 = {S91486, 61, { &P15364,&P6704,&P22728,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; Avant Browser)*/
const Device UA1867245 = {S91487, 76, { &P15364,&P6704,&P22728,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; Avant Browser; Avant Browser)*/
const Device UA101100 = {S91488, 49, { &P17885,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/V908/MIDP2.0/CLDC1.1/Screen-240x320*/
const Device UA153616 = {S91489, 139, { &P15364,&P17285,&P22399,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Comodo_Dragon/6.0.0.10 Chrome/6.0.472.63 Safari/534.3*/
const Device UA177483 = {S91490, 206, { &P18254,&P17285,&P17193,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 3.5.21022; .NET CLR 3.5.30729; .NET CLR 3.0.30729) (TARGUSinfo Web Crawler www.targusinfo.com)*/
const Device UA242051 = {S91491, 123, { &P15364,&P5690,&P24203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.2.8687.201*/
const Device UA1898335 = {S91492, 78, { &P15364,&P5690,&P24203,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) QQBrowser/6.2.8687.201*/
const Device UA1900378 = {S91493, 86, { &P15364,&P5690,&P24203,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0) QQBrowser/6.2.8687.201*/
const Device UA280309 = {S91494, 118, { &P13668,&P17350,&P17315,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; en_US) AppleWebKit/522.7 (KHTML, like Gecko) Dreamweaver/10.0.0.4117 Version/3.0 Safari/522.7*/
const Device UA2019690 = {S91495, 74, { &P14969,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.3;Zh_cn;HTC Desire S;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA2544397 = {S91496, 74, { &P14969,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.5;Zh_cn;HTC Desire S;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA435327 = {S91497, 106, { &P15364,&P6596,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 0.0.0) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.57 Safari/534.24*/
const Device UA1877966 = {S91498, 109, { &P15364,&P6596,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; CrOS i686 0.11.257) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.57 Safari/534.24*/
const Device UA2825349 = {S91499, 115, { &P15364,&P6596,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; CrOS i686 0.0.0; en-US) AppleWebKit/534.21 (KHTML, like Gecko) Chrome/11.0.676.0 Safari/534.21*/
const Device UA507140 = {S91500, 145, { &P21372,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; en-us; i-mobile i651 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1863272 = {S91501, 149, { &P21372,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; th-th; i-mobile i651 Build/ECLAIR) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA554607 = {S91502, 77, { &P15364,&P17285,&P22309,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:5.0) Gecko/X Firefox/5.0 SeaMonkey/2.2*/
const Device UA574281 = {S91503, 35, { &P23545,&P17017,&P17338,&P18092 }};	/*Useragent - mauie800/UC Browser7.9.0.102/69/352*/
const Device UA632286 = {S91504, 64, { &P11684,&P5890,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 (Nokia5800XpressMusic;SymbianOS/9.1 Series60/3.0)*/
const Device UA660837 = {S91505, 83, { &P15364,&P17285,&P20459,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:9.0.1) Gecko/X Firefox/9.0.1 SeaMonkey/2.6.1*/
const Device UA714014 = {S91506, 225, { &P17346,&P18117,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA1983938 = {S91507, 223, { &P17346,&P18117,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA718376 = {S91508, 134, { &P20095,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; TAB101F Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA818923 = {S91509, 138, { &P18221,&P5843,&P24667,&P18092 }};	/*Useragent - Nokia302/5.0 (14.53) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia302) UCBrowser8.2.1.144/70/355/UCWEB Mobile*/
const Device UA1917381 = {S91510, 134, { &P12896,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; LG-P999 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033642 = {S91511, 136, { &P12896,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P999 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1932158 = {S91512, 126, { &P15364,&P7133,&P20431,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 6.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth Pro/6.1.0.5001 Safari/532.4*/
const Device UA1935514 = {S91513, 148, { &P12478,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; pt-pt; Boston Build/GRJ22; CM7.1_EAv7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1954718 = {S91514, 79, { &P12061,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE65)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2329760 = {S91515, 60, { &P15364,&P6729,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows 95; U; ru) Presto/2.10.229 Version/11.62*/
const Device UA2032407 = {S91516, 173, { &P12444,&P5477,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; U20i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352*/
const Device UA2020853 = {S91517, 78, { &P15483,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.3;Zh_cn;HTC ChaCha A810e;320*480;)UCWEB7.8.0.95/139/352*/
const Device UA2371266 = {S91518, 78, { &P15483,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.5;Zh_cn;HTC ChaCha A810e;320*480;)UCWEB7.8.0.95/139/352*/
const Device UA2166684 = {S91519, 80, { &P14753,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; Nokia N96)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2169790 = {S91520, 140, { &P12280,&P17146,&P20787,&P18092 }};	/*Useragent - MQQBrowser/36 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA2340568 = {S91521, 141, { &P12280,&P17146,&P20787,&P18092 }};	/*Useragent - MQQBrowser/36 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3*/
const Device UA2220720 = {S91522, 8, { &P15791,&P5055,&P3804,&P18092 }};	/*Useragent - Nokia603*/
const Device UA2237706 = {S91523, 69, { &P15364,&P5690,&P20799,&P18092 }};	/*Useragent - Mozilla/5.2 (Windows; U; Windows NT 5.2; en-EN) Gecko/X Firefox/1.8.7*/
const Device UA2257738 = {S91524, 132, { &P18306,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; N-05D Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258216 = {S91525, 104, { &P18436,&P16110,&P21913,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; Touch; chromeframe/22.0.1229.94)*/
const Device UA2260402 = {S91526, 77, { &P15364,&P18110,&P21942,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.24) Gecko/X Icedove/3.1.16*/
const Device UA2806947 = {S91527, 75, { &P15364,&P18110,&P21942,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.24) Gecko/X Icedove/3.1.16*/
const Device UA2307057 = {S91528, 71, { &P12980,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; LG-E510; 320*480) UCWEB7.9.3.103/139/32430*/
const Device UA2441089 = {S91529, 96, { &P15241,&P5843,&P17338,&P18092 }};	/*Useragent - Nokia300/5.0 (07.03) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia300/UC Browser7.9.0.102/69/352*/
const Device UA2341862 = {S91530, 135, { &P20972,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SH-07D Build/S7190) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2365559 = {S91531, 37, { &P18267,&P5043,&P23405,&P18092 }};	/*Useragent - MQQBrowser/Mini2.8 (NokiaX2-02/11.57)*/
const Device UA2375070 = {S91532, 52, { &P11907,&P5890,&P21761,&P18092 }};	/*Useragent - iBrowser/2.5 (NokiaE52-1;SymbianOS/9.3 Series60/3.2)*/
const Device UA2383391 = {S91533, 86, { &P14851,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s5753e) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2420196 = {S91534, 140, { &P21441,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MOMO9 STAR Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2422036 = {S91535, 155, { &P14693,&P5843,&P24673,&P18092 }};	/*Useragent - NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokiac1-02) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2424642 = {S91536, 150, { &P15028,&P5096,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; LG-P350 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2462439 = {S91537, 31, { &P19748,&P5043,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia205/03.18)*/
const Device UA2466121 = {S91538, 134, { &P20361,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; X401 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2481183 = {S91539, 71, { &P15759,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_Vision; 480*800) UCWEB7.9.0.94/139/352*/
const Device UA2951083 = {S91540, 71, { &P15759,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.3; zh-cn; HTC_Vision; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2485466 = {S91541, 139, { &P19902,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SCH-R530M Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2486405 = {S91542, 169, { &P14969,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2487457 = {S91543, 68, { &P15364,&P17979,&P23665,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Linux armv7l; rv:21.0) Gecko/21.0 Firefox/21.0*/
const Device UA2507373 = {S91544, 140, { &P18564,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; HTC One X+ Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2528592 = {S91545, 136, { &P22681,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; UMI X1 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2528633 = {S91546, 118, { &P12671,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia700) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2531215 = {S91547, 135, { &P21869,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZP950 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2550636 = {S91548, 117, { &P14545,&P17483,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; lg-x330t) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2553231 = {S91549, 165, { &P18699,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID8042 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2553989 = {S91550, 165, { &P13025,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LiveWithWalkman Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580412 = {S91551, 151, { &P20081,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7108 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2580947 = {S91552, 53, { &P15364,&P6729,&P22540,&P18092 }};	/*Useragent - Opera/9.80 (Windows 95) Presto/2.12.388 Version/12.13*/
const Device UA2866201 = {S91553, 92, { &P15364,&P17285,&P21808,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.19pre) Gecko/X Prism zdesktop/7.2.2*/
const Device UA2596961 = {S91554, 168, { &P14183,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; SGH-I997 Build/JOP40D; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2598074 = {S91555, 139, { &P21735,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; G-TIDE E8  Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2600322 = {S91556, 134, { &P20690,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; V360 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2601162 = {S91557, 136, { &P22677,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ThL W3 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2603766 = {S91558, 139, { &P21320,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT25c Build/9.0.E.1.80) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2625714 = {S91559, 150, { &P22041,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-au; GT-P6810 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2631749 = {S91560, 149, { &P22041,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-au; GT-P6810 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2655270 = {S91561, 159, { &P21617,&P18066,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile IQ 1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2661223 = {S91562, 154, { &P20146,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2667631 = {S91563, 133, { &P20498,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP7100D3G Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2668617 = {S91564, 135, { &P21869,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZP950 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2859350 = {S91565, 136, { &P21869,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZP950+ Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2673560 = {S91566, 125, { &P23868,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ST8 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2676925 = {S91567, 139, { &P20439,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SOL21 Build/9.0.F.2.128) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2696377 = {S91568, 146, { &P17968,&P18066,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/ICS.f06ref848L.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2699614 = {S91569, 32, { &P23558,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH03E(c500;TB;W24H16)*/
const Device UA2700575 = {S91570, 100, { &P18220,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; Nokia808 PureView) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2701056 = {S91571, 138, { &P23879,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; 201K Build/117.1.1c00) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2702538 = {S91572, 133, { &P22816,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S22 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2706533 = {S91573, 97, { &P18462,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; HTC_Desire_X) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2707328 = {S91574, 137, { &P20235,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cynus T1 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709705 = {S91575, 138, { &P20816,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CINK SLIM Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2710096 = {S91576, 136, { &P18438,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E973 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2710422 = {S91577, 128, { &P20014,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-01E Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2713129 = {S91578, 141, { &P23008,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax A91 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713246 = {S91579, 137, { &P20983,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9308 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2730006 = {S91580, 169, { &P21087,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; Lenovo A690 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2730269 = {S91581, 87, { &P20158,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A7) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2733054 = {S91582, 138, { &P18686,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A810K Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2866342 = {S91583, 138, { &P18686,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A810S Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2750700 = {S91584, 90, { &P19763,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; C6602) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2765701 = {S91585, 137, { &P23172,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; TAB750 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2767763 = {S91586, 151, { &P22611,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-S7572 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2768292 = {S91587, 160, { &P21267,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo A798t Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2768373 = {S91588, 159, { &P21141,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; Lenovo P770 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2773910 = {S91589, 135, { &P22634,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; WEXLER-TAB-7T Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2778013 = {S91590, 60, { &P18088,&P17017,&P24506,&P18092 }};	/*Useragent - Opera/8.01 (J2ME/MIDP; Opera Mini/10.1.000/1724; en; U; ssr)*/
const Device UA2778703 = {S91591, 149, { &P15798,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Explorer A310e Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2780792 = {S91592, 69, { &P20146,&P18111,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.1.2; zh-cn; GT-I9082; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2781482 = {S91593, 165, { &P16595,&P5477,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; Micromax A60 Build/ERE27) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2796341 = {S91594, 157, { &P21021,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328t Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2799369 = {S91595, 159, { &P15079,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MK16i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799517 = {S91596, 153, { &P18665,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N861 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2863119 = {S91597, 156, { &P16356,&P5629,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2866051 = {S91598, 156, { &P16356,&P5629,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2865061 = {S91599, 69, { &P22607,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; GT-S5303; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2894490 = {S91600, 155, { &P21987,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Gpad_G1 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2866356 = {S91601, 163, { &P11950,&P5477,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; X10i Build/2.1.A.0.435) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2869420 = {S91602, 162, { &P19862,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Micromax A110 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2877173 = {S91603, 138, { &P15850,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-lu6200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877660 = {S91604, 140, { &P18224,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Ascend G300 Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2878805 = {S91605, 135, { &P15880,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MB865 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2893396 = {S91606, 127, { &P23001,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M97 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2938047 = {S91607, 138, { &P24169,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Primo H2 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2938230 = {S91608, 155, { &P19891,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2938598 = {S91609, 137, { &P17346,&P19826,&P22879,&P18092 }};	/*Useragent - MQQBrowser/43 Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3*/
const Device UA2953683 = {S91610, 153, { &P22874,&P18598,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9502 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958287 = {S91611, 158, { &P18642,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Huawei-U8665 Build/HuaweiU8665B037) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2960136 = {S91612, 95, { &P23122,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; Karbonn A6) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3058548 = {S91613, 209, { &P18113,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us*/
const Device UA2961996 = {S91614, 158, { &P15147,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST15i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2962151 = {S91615, 100, { &P13206,&P17485,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk530i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2962196 = {S91616, 88, { &P20515,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; A54) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2963190 = {S91617, 94, { &P16591,&P22784,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq50) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2967278 = {S91618, 156, { &P19917,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P765 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2981423 = {S91619, 81, { &P15364,&P16110,&P20917,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.1b2) Gecko/X Firefox/3.1b2*/
const Device UA2990928 = {S91620, 151, { &P18510,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ALCATEL ONE TOUCH 918N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3046441 = {S91621, 138, { &P18510,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; BEN918N Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998069 = {S91622, 131, { &P24741,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; FLY F8s Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2998184 = {S91623, 137, { &P15364,&P5690,&P23733,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.34 (KHTML, like Gecko)  QtWeb Internet Browser/3.8.4 http://www.QtWeb.net*/
const Device UA3000017 = {S91624, 99, { &P19862,&P18598,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Micromax_A110Q) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3000719 = {S91625, 147, { &P20259,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Starmobile ASTRA Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3000872 = {S91626, 151, { &P15364,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S7t Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3001934 = {S91627, 147, { &P15364,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; 8720 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3011964 = {S91628, 99, { &P19862,&P18598,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Micromax A110Q) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3020704 = {S91629, 135, { &P21330,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PMP5197DULTRA Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3020730 = {S91630, 93, { &P22941,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6812) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3021089 = {S91631, 133, { &P14997,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; S100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021259 = {S91632, 71, { &P18385,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Tablet: Nexus 7; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA3022006 = {S91633, 146, { &P21646,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; BLU STUDIO 5.3 II Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022023 = {S91634, 142, { &P23838,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; StarTrail III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022698 = {S91635, 130, { &P17168,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7310 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023382 = {S91636, 138, { &P24150,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; vivo Xplay Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023848 = {S91637, 144, { &P20253,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; i-mobile IQ 5.3 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024195 = {S91638, 99, { &P13677,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5253) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3027308 = {S91639, 134, { &P22847,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SH-06E Build/S7190) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3029474 = {S91640, 155, { &P21022,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LG-E615 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3029607 = {S91641, 158, { &P15539,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; HTC Glacier Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3058334 = {S91642, 97, { &P14839,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6720c-1b) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3107719 = {S91643, 94, { &P14839,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6720c) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3093267 = {S91644, 93, { &P22607,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S5303) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3093354 = {S91645, 151, { &P20158,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A7* Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3108945 = {S91646, 159, { &P20518,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Karbonn A10 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3060260 = {S91647, 170, { &P20146,&P18111,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9082 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3079679 = {S91648, 163, { &P23716,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI A199 Build/HuaweiA199) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3079610 = {S91649, 175, { &P18513,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6802 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3079802 = {S91650, 165, { &P18666,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A25 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3080144 = {S91651, 93, { &P21261,&P18598,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.2; en-US; GT-I9500) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3080574 = {S91652, 90, { &P17625,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; ST25i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3115612 = {S91653, 148, { &P15985,&P6437,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.2; GT-P6200 Build/HTJ85B) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3108736 = {S91654, 151, { &P21057,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A7+ Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3094581 = {S91655, 152, { &P21963,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Q800 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3094695 = {S91656, 161, { &P18672,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5302 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3108361 = {S91657, 175, { &P13974,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-S5670 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3125958 = {S91658, 175, { &P13974,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5670 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3126351 = {S91659, 136, { &P23248,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Gpad_G2 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109496 = {S91660, 157, { &P22382,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3115501 = {S91661, 150, { &P21261,&P18598,&P24683,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9500 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.5 Mobile Safari/533.1*/
const Device UA3115742 = {S91662, 88, { &P19779,&P5629,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A1+) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3126989 = {S91663, 114, { &P14482,&P5843,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-01/07.51; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA3128637 = {S91664, 116, { &P14482,&P5843,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-01.5/06.75; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA5829 = {S91665, 128, { &P12310,&P17486,&P3772,&P18092 }};	/*Useragent - MOT-ZN300/13.02.0BR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 17243441) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.65 [pt]*/
const Device UA30365 = {S91666, 128, { &P12310,&P17486,&P3772,&P18092 }};	/*Useragent - MOT-ZN300/13.02.14R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 17243462) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.65 [en]*/
const Device UA64235 = {S91667, 96, { &P14583,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE70-1/3.0 (1.0610.05.06) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA59111 = {S91668, 82, { &P12490,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK700/R1KG Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA13240 = {S91669, 96, { &P15944,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Smartphone; 176x220; HPiPAQ510/1.0*/
const Device UA18553 = {S91670, 115, { &P15177,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; i-mate K-JAM PPC; 240x320; i-mate JAMin PPC; 240x320; PPC; 240x320)*/
const Device UA65999 = {S91671, 88, { &P15177,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; i-mate K-JAM PPC; 240x320; PPC; 240x320)*/
const Device UA23657 = {S91672, 111, { &P15174,&P17483,&P3534,&P18092 }};	/*Useragent - LG-KT520/v10a Browser/Obigo-Q05A/3.11 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA33377 = {S91673, 111, { &P15174,&P17483,&P3534,&P18092 }};	/*Useragent - LG/KT520/v10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA22359 = {S91674, 104, { &P17070,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-A637/A637UCHH2 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA62807 = {S91675, 101, { &P14452,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E730/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA24441 = {S91676, 68, { &P12106,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-T509S/T509UVID1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA24442 = {S91677, 67, { &P12106,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-T509/T509UVFC4 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA26131 = {S91678, 104, { &P12150,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-A796/A796UXJB1 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA26136 = {S91679, 57, { &P12558,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Cruise_T4242 Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA26137 = {S91680, 60, { &P12558,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Cruise_T4242 Opera/9.50 (Windows NT 5.1; U; zh-tw)*/
const Device UA26138 = {S91681, 57, { &P12558,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Cruise_T4242 Opera/9.50 (Windows NT 5.1; U; it)*/
const Device UA90024 = {S91682, 57, { &P12558,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Cruise_T4242 Opera/9.50 (Windows NT 5.1; U; nb)*/
const Device UA26460 = {S91683, 57, { &P13598,&P17017,&P3487,&P18092 }};	/*Useragent - UTSTARCOM-CDM-8964VM UP.Browser/7.2.7.2.398 (GUI) MMP/2.0*/
const Device UA43632 = {S91684, 119, { &P13714,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) 320x320; VZW; UTStar-SMT5800; Window Mobile 6.1 Standard;*/
const Device UA56066 = {S91685, 14, { &P14764,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonU1*/
const Device UA253812 = {S91686, 16, { &P14764,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonU10i*/
const Device UA67135 = {S91687, 100, { &P14763,&P5776,&P4715,&P18092 }};	/*Useragent - NokiaN72/5.0706.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.4.0.65/27/352*/
const Device UA64573 = {S91688, 111, { &P14681,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHC520/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA69336 = {S91689, 36, { &P12449,&P5890,&P4779,&P18092 }};	/*Useragent - NOKIAE5-00/UC Browser7.8.0.95/28/355*/
const Device UA89739 = {S91690, 88, { &P17562,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-G801/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA616281 = {S91691, 88, { &P17562,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-G801/MTK Release/5.12.2011 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA324105 = {S91692, 140, { &P13503,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_DesireZ_A7272; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA336338 = {S91693, 137, { &P15364,&P17819,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/11.10 Chromium/13.0.782.215 Chrome/13.0.782.215 Safari/535.1*/
const Device UA337563 = {S91694, 135, { &P15364,&P17819,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/11.10 Chromium/13.0.782.215 Chrome/13.0.782.215 Safari/535.1*/
const Device UA710127 = {S91695, 137, { &P15364,&P17819,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/535.1 (KHTML, like Gecko) Ubuntu/11.10 Chromium/13.0.782.215 Chrome/13.0.782.215 Safari/535.1*/
const Device UA1936360 = {S91696, 159, { &P23919,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; R800i Build/4.0.2.A.0.42; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033093 = {S91697, 134, { &P22263,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; T9199C Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA448274 = {S91698, 11, { &P23376,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M9000*/
const Device UA514469 = {S91699, 84, { &P13508,&P5448,&P4737,&P18092 }};	/*Useragent - SAMSUNG-C6625/1.0 (compatible; MSIE 4.01; Windows CE; Smartphone)/UC Browser7.2.2.51*/
const Device UA542900 = {S91700, 80, { &P15364,&P18118,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.7) Gecko/X Ubuntu Firefox/3.5.7*/
const Device UA543451 = {S91701, 96, { &P12192,&P5448,&P4777,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2300411 = {S91702, 100, { &P16074,&P5768,&P17338,&P18092 }};	/*Useragent - Nokia3110c/2.0 (07.01) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia3110c/UC Browser7.9.0.102/69/352*/
const Device UA687163 = {S91703, 174, { &P15501,&P5477,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E16i Build/2.1.1.A.0.53) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2855728 = {S91704, 178, { &P15501,&P5477,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E16i Build/2.1.1.A.0.53) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA666254 = {S91705, 101, { &P15842,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; TITAN X310e)*/
const Device UA925817 = {S91706, 97, { &P15842,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; PI39100)*/
const Device UA678452 = {S91707, 102, { &P16404,&P17603,&P4392,&P18092 }};	/*Useragent - PantechP5000/JTUSX BMP/1.0.2 DeviceId/X NetFront/4.1 OMC/1.5.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA691419 = {S91708, 140, { &P15767,&P17090,&P20774,&P18092 }};	/*Useragent - MQQBrowser/2.3 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3*/
const Device UA745253 = {S91709, 140, { &P15767,&P17090,&P20774,&P18092 }};	/*Useragent - MQQBrowser/2.3 Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3*/
const Device UA2333370 = {S91710, 140, { &P15767,&P17090,&P20774,&P18092 }};	/*Useragent - MQQBrowser/2.3 Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 Safari/7534.48.3*/
const Device UA762783 = {S91711, 26, { &P14872,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SPH-D700/100.20306*/
const Device UA865883 = {S91712, 77, { &P15364,&P17285,&P4536,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; Opera Mobi/49; U; en) Presto/2.4.18 Version/10.00*/
const Device UA888761 = {S91713, 61, { &P15364,&P5448,&P17213,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE) Opera 8.0 [en]*/
const Device UA895595 = {S91714, 142, { &P14207,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6350 Build/GRJ22)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2867681 = {S91715, 143, { &P14207,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA1899054 = {S91716, 88, { &P17595,&P17295,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:7.0b1) Gecko/X Firefox/7.0b1 TenFourFox/G5*/
const Device UA1936534 = {S91717, 143, { &P21672,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; th-th; i-mobile I-Note Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024355 = {S91718, 160, { &P15079,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MK16i Build/4.0.1.A.0.283; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1988914 = {S91719, 134, { &P21101,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; ZP100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2161851 = {S91720, 85, { &P12137,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5620) UCBrowser8.4.0.159/69/405/UCWEB Mobile*/
const Device UA2027024 = {S91721, 128, { &P21363,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; T01A Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2339954 = {S91722, 128, { &P21363,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T01A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2031555 = {S91723, 133, { &P13222,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Ally Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2168351 = {S91724, 34, { &P12059,&P5204,&P21759,&P18092 }};	/*Useragent - iBrowser/Mini2.8 (Nokia2690/10.10)*/
const Device UA2203001 = {S91725, 88, { &P15364,&P16110,&P22553,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:15.0) Gecko/X Firefox/15.0.2.0 IceDragon/15.0.2.0*/
const Device UA2260192 = {S91726, 154, { &P22875,&P18066,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9220 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2789539 = {S91727, 77, { &P13365,&P17483,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; lg-gu220) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2375895 = {S91728, 141, { &P24020,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; vi-vn; HKPhone 4S-3G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2408782 = {S91729, 128, { &P15364,&P17285,&P22456,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Iron/6.0.475 Chrome/6.0.475.0 Safari/X.534*/
const Device UA2416228 = {S91730, 251, { &P12059,&P5204,&P18527,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/70/352*/
const Device UA2424737 = {S91731, 117, { &P13365,&P17483,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; lg-gu220) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2456434 = {S91732, 130, { &P20014,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-01E Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2466557 = {S91733, 33, { &P14673,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SAMSUNG-GT-E2652)*/
const Device UA2478027 = {S91734, 157, { &P16356,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2503508 = {S91735, 141, { &P20543,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A660 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2507404 = {S91736, 133, { &P24553,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; X21 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2517431 = {S91737, 90, { &P15364,&P16110,&P22529,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.2.6) Gecko/X Epic/1.0 Firefox/3.6.6*/
const Device UA2527565 = {S91738, 157, { &P14186,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2541342 = {S91739, 65, { &P17595,&P18120,&P22888,&P18092 }};	/*Useragent - Mozilla/4.5 (compatible; iCab 2.9.1; Macintosh; U; PPC; Mac OS X)*/
const Device UA2979881 = {S91740, 40, { &P17595,&P18120,&P22888,&P18092 }};	/*Useragent - iCab/2.9.5 (Macintosh; U; PPC; Mac OS X)*/
const Device UA2554383 = {S91741, 142, { &P22669,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GSmart G1362 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2556014 = {S91742, 137, { &P20878,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MIDC901 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2556119 = {S91743, 90, { &P14545,&P17483,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-x330t) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2556526 = {S91744, 111, { &P20742,&P18110,&P17250,&P18092 }};	/*Useragent - TOUCH87 Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2558623 = {S91745, 159, { &P18295,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2579072 = {S91746, 130, { &P23566,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U950 Build/IMM76L) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2985998 = {S91747, 61, { &P23566,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ZTE U950 Build/IMM76L)*/
const Device UA2581146 = {S91748, 152, { &P20934,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; 5860E Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000*/
const Device UA2597794 = {S91749, 164, { &P16049,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P698 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2601583 = {S91750, 138, { &P12841,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Vivid 4G Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2991748 = {S91751, 145, { &P21802,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GSmart Maya M1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2628264 = {S91752, 136, { &P15882,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-us; GT-P5110 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7*/
const Device UA2632274 = {S91753, 136, { &P15882,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P5110 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7*/
const Device UA2637013 = {S91754, 136, { &P15882,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5110 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2622452 = {S91755, 188, { &P22043,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.15/274,1.9.15.274,2012-12-21 15:38, +0000*/
const Device UA2623770 = {S91756, 192, { &P22043,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, -0500, ca*/
const Device UA2652757 = {S91757, 181, { &P14202,&P5477,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; E15i Build/2.1.1.A.0.6; 320*480) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/352*/
const Device UA2656533 = {S91758, 157, { &P19999,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Karbonn A30 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2863239 = {S91759, 173, { &P18280,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC VLE_U Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (HTC VLE_U, 42, 1.9.1)*/
const Device UA2659976 = {S91760, 53, { &P14188,&P5890,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4 (Nokia5233;SymbianOS/9.1 Series60/3.0)*/
const Device UA2668103 = {S91761, 138, { &P20983,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9308 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668407 = {S91762, 156, { &P22875,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9220 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2671044 = {S91763, 134, { &P21823,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTX21 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2671987 = {S91764, 134, { &P21302,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IQ441 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2673374 = {S91765, 137, { &P21949,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F100L Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2676059 = {S91766, 134, { &P17283,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MB860 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2686292 = {S91767, 93, { &P15364,&P16110,&P22480,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.0.5) Gecko/X Firefox/3.0.5 Flock/2.0.3*/
const Device UA2694480 = {S91768, 59, { &P18254,&P18110,&P22938,&P18091 }};	/*Useragent - Liferea/1.8.10 (Linux; fr_FR.UTF-8; http://liferea.sf.net/)*/
const Device UA2696373 = {S91769, 91, { &P11868,&P5768,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6280) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2706431 = {S91770, 96, { &P13437,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; T-Mobile_G2) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2706872 = {S91771, 129, { &P18061,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; rk30sdk Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2714904 = {S91772, 133, { &P21642,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NXM805ND_EN Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2715888 = {S91773, 160, { &P20134,&P5629,&P22281,&P18092 }};	/*Useragent - OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2729510 = {S91774, 140, { &P22947,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; Poly Pad_7408IPS Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3044579 = {S91775, 154, { &P16356,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2730806 = {S91776, 95, { &P20413,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; CELKON_CT2) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2732575 = {S91777, 103, { &P13648,&P5055,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae7-00) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2735512 = {S91778, 136, { &P23164,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-7204 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2766052 = {S91779, 143, { &P18442,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N7100 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2780206 = {S91780, 89, { &P21893,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; MSD7) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2782671 = {S91781, 137, { &P18641,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-M830 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783006 = {S91782, 137, { &P22952,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F260S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784092 = {S91783, 153, { &P21140,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Optimus_Madrid Build/IceCreamSandwich) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784625 = {S91784, 131, { &P23155,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PRAXIS 13 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2795470 = {S91785, 101, { &P18220,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia808 PureView) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2796252 = {S91786, 96, { &P11834,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6760s-1) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2797657 = {S91787, 142, { &P20253,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 5 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798821 = {S91788, 142, { &P20837,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IRIS_501 Build/LAVAIRIS501) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2800432 = {S91789, 133, { &P24542,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARNOVA 90G3 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2805229 = {S91790, 59, { &P15364,&P17188,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.0; Maxthon)*/
const Device UA3099022 = {S91791, 106, { &P15364,&P17188,&P22770,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; MAXTHON)*/
const Device UA2845425 = {S91792, 160, { &P15364,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Mini Plus 4G Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2850054 = {S91793, 131, { &P18473,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC X720d Build/JRO03C) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3086467 = {S91794, 127, { &P18473,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC X720d) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2853829 = {S91795, 143, { &P11833,&P17485,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  SonyEricssonP1i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2853089 = {S91796, 141, { &P23288,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; iDream_A75 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2866117 = {S91797, 138, { &P23258,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TPC0707 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867074 = {S91798, 138, { &P23617,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; RunboX5-W Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867615 = {S91799, 136, { &P19909,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P935 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868409 = {S91800, 141, { &P21738,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; iris502 Build/LAVAIRIS502) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869069 = {S91801, 158, { &P18449,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; ST27i Build/6.2.A.0.400) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3007716 = {S91802, 158, { &P18449,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; ST27i Build/6.2.A.1.100) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2877849 = {S91803, 141, { &P13124,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PantechP9070 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879344 = {S91804, 131, { &P23704,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M500W Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2879548 = {S91805, 157, { &P19920,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-MS870 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2879658 = {S91806, 131, { &P24078,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; E3 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2880399 = {S91807, 134, { &P23150,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Q1000 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2891706 = {S91808, 163, { &P21911,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; AMAZE S200 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2892682 = {S91809, 146, { &P21646,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; BLU STUDIO 5.3 II Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893038 = {S91810, 131, { &P24103,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; C2 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2936548 = {S91811, 171, { &P23828,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; Q-Smart S16 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33829*/
const Device UA2943917 = {S91812, 142, { &P23683,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Prism II Build/HuaweiU8686) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2952660 = {S91813, 162, { &P15364,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; baoxue Build/F7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33738*/
const Device UA3073527 = {S91814, 163, { &P15364,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; 668 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33736*/
const Device UA2953581 = {S91815, 154, { &P23388,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TSINTECH Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956282 = {S91816, 107, { &P18510,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL ONE TOUCH 918N) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA3002719 = {S91817, 107, { &P18510,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL ONE TOUCH 918N) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2958206 = {S91818, 161, { &P22627,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; IRIS 455 Build/LAVAIRIS455) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2959701 = {S91819, 151, { &P18666,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A25 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2963326 = {S91820, 93, { &P16356,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7000) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2963609 = {S91821, 158, { &P15397,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST18i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2965911 = {S91822, 154, { &P22860,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Q700 Build/XOLOQ700) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2974331 = {S91823, 140, { &P20262,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.3; en-us; Karbonn_A18 Build/KarbonnA18) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2993688 = {S91824, 137, { &P24585,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; es-es; ZP800H Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996069 = {S91825, 155, { &P15989,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; ST21i2 Build/11.0.A.1.8) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2996917 = {S91826, 155, { &P15989,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; ST21i Build/11.0.A.0.16) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2998020 = {S91827, 164, { &P15767,&P17144,&P18215,&P18092 }};	/*Useragent - Mozilla/5.0 AppleCoreMedia/1.0.0.8J2 (iPad; U; CPU OS 4_0 like Mac OS X; en-us) AppleWebKit/531.21.10 (HTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10*/
const Device UA3009342 = {S91828, 137, { &P11747,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.3; en-in; HTC Desire V Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3012201 = {S91829, 93, { &P22607,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S5303) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3013577 = {S91830, 150, { &P14908,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6210 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3078645 = {S91831, 156, { &P14017,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P990 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3021328 = {S91832, 93, { &P22626,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Iris_349) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3021575 = {S91833, 131, { &P23704,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHW-M500W Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021653 = {S91834, 132, { &P23807,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A50 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021837 = {S91835, 134, { &P18505,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; MB886 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022187 = {S91836, 133, { &P20156,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S500 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022306 = {S91837, 139, { &P18592,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One VX Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022786 = {S91838, 143, { &P23546,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; New Andromax-i Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3079725 = {S91839, 152, { &P20438,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; N400 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3029048 = {S91840, 194, { &P18635,&P21637,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-gb; Nexus 4 Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb*/
const Device UA3029591 = {S91841, 156, { &P22020,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9268 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3029627 = {S91842, 160, { &P21738,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; iris502 Build/LAVAIRIS502) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3044439 = {S91843, 104, { &P15364,&P16110,&P19818,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.8.1 Safari/534.34*/
const Device UA3048318 = {S91844, 161, { &P21988,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-US; PD_Novel Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3049059 = {S91845, 128, { &P20888,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; D709 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3078394 = {S91846, 131, { &P20888,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PMID709 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3050423 = {S91847, 159, { &P18382,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC Desire C Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058663 = {S91848, 150, { &P20513,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A5 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109042 = {S91849, 163, { &P14202,&P5477,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E15i Build/2.1.1.A.0.6) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3111554 = {S91850, 137, { &P14183,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I997 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066127 = {S91851, 135, { &P18442,&P21637,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-N7100 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.24 Mobile Safari/537.36*/
const Device UA3080326 = {S91852, 135, { &P18442,&P21637,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-N7105 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.30 Mobile Safari/537.36*/
const Device UA3108057 = {S91853, 180, { &P14708,&P5204,&P24055,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311*/
const Device UA3093517 = {S91854, 158, { &P15047,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; SK17i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3093609 = {S91855, 99, { &P21269,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Intex Aqua 3.2) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3109962 = {S91856, 157, { &P15575,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3114668 = {S91857, 155, { &P18283,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; GT-I9300 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3115437 = {S91858, 162, { &P21020,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Micromax A116 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3125591 = {S91859, 97, { &P15364,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax_A34) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3125965 = {S91860, 108, { &P15364,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_4010D) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3132861 = {S91861, 137, { &P22595,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919V Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132951 = {S91862, 140, { &P22598,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6506 Build/10.3.A.0.423) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA82 = {S91863, 43, { &P17561,&P17483,&P3625,&P18092 }};	/*Useragent - LG9100/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA22180 = {S91864, 50, { &P17561,&P17483,&P3625,&P18092 }};	/*Useragent - LG9100-parrot/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA5033 = {S91865, 47, { &P13331,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-VX8500/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA17122 = {S91866, 139, { &P15366,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; HTC Tattoo Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA38996 = {S91867, 138, { &P15366,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC Tattoo Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020333 = {S91868, 138, { &P15366,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; HTC Tattoo Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA19882 = {S91869, 115, { &P12262,&P17484,&P3430,&P18092 }};	/*Useragent - SAMSUNG-S8300/S8300BOIH3 SHP/VPP/R5 Opera/9.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA21303 = {S91870, 79, { &P15189,&P17484,&P4924,&P18092 }};	/*Useragent - SAMSUNG-SGH-T629/T629UVFG8 Profile/MIDP-2.0 Configuration/CLDC-1.1 NetFront/3.2*/
const Device UA21304 = {S91871, 80, { &P15189,&P17484,&P4924,&P18092 }};	/*Useragent - SAMSUNG-SGH-T629R/T629UVFL2 Profile/MIDP-2.0 Configuration/CLDC-1.1 NetFront/3.2*/
const Device UA21272 = {S91872, 44, { &P12530,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_P3450 Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA22010 = {S91873, 77, { &P15639,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) HTC-6900-MR2 BM*/
const Device UA23865 = {S91874, 82, { &P12075,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-SGH-I616/1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA43634 = {S91875, 73, { &P13716,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-GT-M2513/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA52779 = {S91876, 75, { &P12077,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-S3600i/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA54406 = {S91877, 51, { &P14201,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-W766/00.72 UP.Browser/7.2.7.5.630 (GUI) MMP/2.0*/
const Device UA54407 = {S91878, 51, { &P14201,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-W766/00.72 UP.Browser/7.2.7.5.606 (GUI) MMP/2.0*/
const Device UA83578 = {S91879, 61, { &P16256,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonT68/R201A Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA2408793 = {S91880, 31, { &P14752,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 L03A(c100;TB;W20H11)*/
const Device UA562861 = {S91881, 54, { &P17877,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-HI3D UP.Browser/6.2_7.2.7.1.K.2.234 (GUI) MMP/2.0*/
const Device UA87945 = {S91882, 45, { &P17633,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SH36 UP.Browser/6.2.0.14.2 (GUI) MMP/2.0*/
const Device UA428050 = {S91883, 86, { &P18088,&P17017,&P4860,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/6.24093/27.1324; U; ru) Presto/2.8.119 Version/11.10*/
const Device UA646283 = {S91884, 84, { &P18088,&P17017,&P4860,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/6.24288/25.729; U; en) Presto/2.5.25 Version/10.54*/
const Device UA2020496 = {S91885, 69, { &P17337,&P6546,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android3.1;Zh_cn;GT-P7500;800*1280;)UCWEB7.8.0.95/139/355*/
const Device UA2608081 = {S91886, 118, { &P17337,&P6546,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android3.1;Zh_cn;GT-P7500;800*1280;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA384963 = {S91887, 138, { &P15364,&P7133,&P22380,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Comodo_Dragon/4.0.1.6 Chrome/4.0.249.78 Safari/532.5*/
const Device UA428489 = {S91888, 128, { &P15767,&P18117,&P20773,&P18092 }};	/*Useragent - MQQBrowser/2.2 Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3*/
const Device UA1986794 = {S91889, 130, { &P15767,&P18117,&P20773,&P18092 }};	/*Useragent - MQQBrowser/2.2 Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA439669 = {S91890, 64, { &P15364,&P17017,&P17826,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/4.4) KHTML/4.4.5 (like Gecko)*/
const Device UA451301 = {S91891, 79, { &P15364,&P18145,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (X11; FreeBSD 8.2-RELEASE amd64; U; en) Presto/2.9.168 Version/11.51*/
const Device UA535449 = {S91892, 239, { &P12280,&P17145,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_8 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E401,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA766996 = {S91893, 239, { &P12280,&P17145,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E200,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA806076 = {S91894, 240, { &P12280,&P17145,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_10 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E600,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA863146 = {S91895, 239, { &P12280,&P17145,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA2208370 = {S91896, 240, { &P12280,&P17145,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148a,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA2144883 = {S91897, 100, { &P14482,&P5843,&P17338,&P18092 }};	/*Useragent - NokiaC3-01/5.0 (06.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac3-01/UC Browser7.9.0.102/69/352*/
const Device UA573477 = {S91898, 33, { &P18254,&P17017,&P21798,&P18092 }};	/*Useragent - Akregator/4.7.2; syndication SUSE*/
const Device UA1918325 = {S91899, 28, { &P18254,&P17017,&P21798,&P18092 }};	/*Useragent - Akregator/4.7.3; syndication*/
const Device UA588183 = {S91900, 140, { &P15767,&P17090,&P20773,&P18092 }};	/*Useragent - MQQBrowser/2.2 Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3*/
const Device UA684039 = {S91901, 140, { &P15767,&P17090,&P20773,&P18092 }};	/*Useragent - MQQBrowser/2.2 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3*/
const Device UA629832 = {S91902, 35, { &P14763,&P5776,&P17338,&P18092 }};	/*Useragent - nokian72/UC Browser7.9.0.102/69/352*/
const Device UA1988426 = {S91903, 136, { &P17595,&P23111,&P18108,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PowerPC MorphOS 2.7; Odyssey Web Browser; rv:1.17) AppleWebKit/535.14 (KHTML, like Gecko) OWB/1.17 Safari/535.14*/
const Device UA1902969 = {S91904, 114, { &P14669,&P5843,&P17437,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-00/04.90; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.4.0.34.8*/
const Device UA720451 = {S91905, 134, { &P15364,&P5690,&P23730,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko)  QtWeb Internet Browser/3.3 http://www.QtWeb.net*/
const Device UA2322417 = {S91906, 61, { &P15364,&P18144,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD i386; rv:12.0) Gecko/X Firefox/12.0*/
const Device UA883047 = {S91907, 90, { &P17595,&P17294,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:6.0.1) Gecko/X Firefox/6.0.1 TenFourFox/7450*/
const Device UA1904460 = {S91908, 168, { &P18254,&P7480,&P17191,&P19831 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0 compatible; Asterias Crawler v4; +http://www.singingfish.com/help/spider.html; webmaster@singingfish.com); SpiderThread Revision: 3.10*/
const Device UA2029207 = {S91909, 156, { &P21033,&P18066,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U1 Build/HCL ME Tablet U1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029874 = {S91910, 150, { &P14186,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-us; GT-S5830L Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2999641 = {S91911, 150, { &P14186,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA3088512 = {S91912, 151, { &P14186,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-S5830M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2031292 = {S91913, 138, { &P22151,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; GPLUS-M600 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031460 = {S91914, 153, { &P18714,&P5629,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V889D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2040762 = {S91915, 81, { &P15364,&P17285,&P22552,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:14.0) Gecko/X Firefox/14.0.3.0 IceDragon/14.0.3.0*/
const Device UA2165943 = {S91916, 85, { &P14764,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU1)/UC Browser8.5.0.183/50/352/UCWEB Mobile*/
const Device UA2365356 = {S91917, 83, { &P13979,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; Nokia6220c-1)/UC Browser8.5.0.183/28/444/UCWEB Mobile*/
const Device UA2228049 = {S91918, 150, { &P13526,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; T-Mobile myTouch 4G Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2262247 = {S91919, 32, { &P12059,&P5204,&P21724,&P18092 }};	/*Useragent - OneBrowser/3.0 (Nokia2690/10.10)*/
const Device UA2334385 = {S91920, 143, { &P18282,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A200 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2362536 = {S91921, 77, { &P23351,&P22784,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; micromax) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2372385 = {S91922, 51, { &P15373,&P5890,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 (Nokia5530;SymbianOS/9.1 Series60/3.0)*/
const Device UA2384472 = {S91923, 33, { &P14826,&P5843,&P21724,&P18092 }};	/*Useragent - OneBrowser/3.0 (NokiaC1-01/04.40)*/
const Device UA2384746 = {S91924, 162, { &P21226,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; E860 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2386185 = {S91925, 37, { &P18623,&P5448,&P24630,&P18092 }};	/*Useragent - NOKIA-Lumia 510/UCWEB2.2.0.157/47/355*/
const Device UA2397694 = {S91926, 126, { &P14060,&P17485,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; sonyericssonw150i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2404149 = {S91927, 131, { &P24100,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CAL21 Build/AX) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2417935 = {S91928, 137, { &P23303,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Ex Note Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2420045 = {S91929, 74, { &P18652,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.5;Zh_cn;Micromax A87;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA2426411 = {S91930, 116, { &P14532,&P17483,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; lg-t325) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2455391 = {S91931, 155, { &P15147,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; ST15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2765260 = {S91932, 155, { &P15147,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; ST15i Build/4.0.2.A.0.84) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2456707 = {S91933, 157, { &P18295,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; One X Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2843199 = {S91934, 161, { &P18295,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC One X Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2463936 = {S91935, 58, { &P13505,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.1; MB525 Build/JRO03L)*/
const Device UA2817176 = {S91936, 136, { &P13505,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB525 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2466267 = {S91937, 66, { &P12824,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.2; zh-cn; U8180; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2477602 = {S91938, 137, { &P21924,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MTP-710 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2482383 = {S91939, 135, { &P21642,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NXM805ND_EN Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2486117 = {S91940, 162, { &P18721,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A90S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2487580 = {S91941, 133, { &P18464,&P18110,&P19805,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.6 Safari/535.19 Silk-Accelerated=true*/
const Device UA2503391 = {S91942, 135, { &P21306,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MOMO11Speed Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2504800 = {S91943, 131, { &P20832,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MIDC497 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2527952 = {S91944, 157, { &P14959,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2528822 = {S91945, 142, { &P15767,&P17145,&P20779,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) MQQBrowser/2.8 Mobile/8C148 Safari/7534.48.3*/
const Device UA2530060 = {S91946, 154, { &P23714,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; O+_8.9_Android Build/OplusOplus_A1000) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2548424 = {S91947, 110, { &P22116,&P5629,&P15760,&P18092 }};	/*Useragent - HS-TG88_TD/1.0 Android/2.3 Release/05.31.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2554942 = {S91948, 157, { &P18513,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6352 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2596066 = {S91949, 168, { &P18449,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; ST27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2559363 = {S91950, 99, { &P14985,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8530) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2560665 = {S91951, 93, { &P11689,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian70-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2600137 = {S91952, 91, { &P11689,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian70) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2574514 = {S91953, 67, { &P18302,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.4;Zh_cn;MT25i;480*854;)UCWEB7.8.0.95/139/444*/
const Device UA2766553 = {S91954, 166, { &P13064,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2620169 = {S91955, 163, { &P17337,&P6546,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7500 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2654395 = {S91956, 163, { &P12871,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Wildfire S A510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2842990 = {S91957, 152, { &P12871,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2659519 = {S91958, 101, { &P13042,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; HTC_Incredible_S) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2661763 = {S91959, 166, { &P20141,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-S5301 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2667164 = {S91960, 138, { &P20508,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Cynus T2 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA3043328 = {S91961, 96, { &P18644,&P5043,&P17347,&P18092 }};	/*Useragent - Nokia112/2.0 (03.32) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia112/UC Browser8.0.3.107/69/444*/
const Device UA2695462 = {S91962, 24, { &P20195,&P17483,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (LG-T375)*/
const Device UA2701881 = {S91963, 160, { &P21351,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI C8813 Build/HuaweiC8813) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2712424 = {S91964, 136, { &P18621,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E610 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712731 = {S91965, 132, { &P20807,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; B92 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713398 = {S91966, 132, { &P23171,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; G2S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2934933 = {S91967, 44, { &P14010,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SAMSUNG-GT-S5263/S5263JPKB4)*/
const Device UA2728032 = {S91968, 57, { &P18669,&P7106,&P21731,&P18092 }};	/*Useragent - OneBrowser/for Blackberry3.0.0 (BlackBerry9320/7.1.0.398)*/
const Device UA2730789 = {S91969, 174, { &P12987,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SPH-M930BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2934699 = {S91970, 171, { &P12987,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SPH-M930 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2736956 = {S91971, 139, { &P20244,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; bq Elcano Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2758863 = {S91972, 113, { &P18221,&P22723,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia3020/14.53; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.1.62.6*/
const Device UA2763741 = {S91973, 265, { &P18088,&P22840,&P4780,&P18092 }};	/*Useragent - TINNO50_6464_W1224_11B_HW (MRE\2.5.00(1536) resolution\X chipset\MT6250 touch\1 tpannel\1 camera\1 gsensor\0 keyboard\reduced) F5100A_T29TV_X_V12 Release/2012.11.17 WAP Browser/MAUI (HTTP PGDL; HTTPS) Profile/Profile/MIDP-2.0 Configuration/CLDC-1.1 Q03C1-2.40 en-US*/
const Device UA2764996 = {S91974, 169, { &P19891,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I717 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000*/
const Device UA2766605 = {S91975, 140, { &P23895,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TP10.1-1500DC kb Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2769813 = {S91976, 133, { &P20427,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHL21 Build/S2140) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2783163 = {S91977, 139, { &P20927,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT-AS45IPS Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2795075 = {S91978, 93, { &P20837,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; IRIS_501) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2796257 = {S91979, 95, { &P20471,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; CONNECT_2G) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2797636 = {S91980, 134, { &P23478,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ZP810 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798402 = {S91981, 135, { &P22677,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ThL W3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798561 = {S91982, 137, { &P13999,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NexusHD2 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798605 = {S91983, 133, { &P18314,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M032 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799278 = {S91984, 157, { &P12532,&P6437,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.2.1; HTC Flyer P510e Build/HTK75C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799327 = {S91985, 148, { &P14908,&P6437,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.2; GT-P6210 Build/HTJ85B) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2800150 = {S91986, 132, { &P20274,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IdeaTab_A1107 Build/MR1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2800656 = {S91987, 135, { &P22860,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Q700 Build/XOLOQ700) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2864822 = {S91988, 166, { &P18712,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Next8P12 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444*/
const Device UA2865880 = {S91989, 154, { &P15985,&P6437,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-US; GT-P6200 Build/HTJ85B) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2869122 = {S91990, 137, { &P19865,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A850K Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2957610 = {S91991, 137, { &P19865,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A850L Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869894 = {S91992, 152, { &P23175,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 5021E Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2870020 = {S91993, 146, { &P18496,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Incredible 4G LTE Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877720 = {S91994, 134, { &P24493,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; BS471 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892607 = {S91995, 137, { &P14872,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-D700 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892860 = {S91996, 138, { &P18385,&P18598,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon*/
const Device UA2897413 = {S91997, 160, { &P19863,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2936774 = {S91998, 142, { &P24563,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; Q-Smart S21 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2937912 = {S91999, 158, { &P23716,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI A199 Build/HuaweiA199) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2945161 = {S92000, 160, { &P18462,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; HTC Desire X Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2952390 = {S92001, 44, { &P15364,&P18580,&P17470,&P18091 }};	/*Useragent - Spider/2.3.0 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3092271 = {S92002, 44, { &P15364,&P18580,&P17470,&P18091 }};	/*Useragent - Spider/2.4.3 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA2957116 = {S92003, 154, { &P20231,&P18066,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A21 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.289*/
const Device UA2957198 = {S92004, 138, { &P19878,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I497 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3048518 = {S92005, 89, { &P18721,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A90S) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2959700 = {S92006, 158, { &P22756,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; C1604 Build/11.1.A.0.68) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2961746 = {S92007, 159, { &P21991,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Aqua Wonder Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2961826 = {S92008, 157, { &P20251,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328d Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2962182 = {S92009, 161, { &P18297,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I8530 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2962220 = {S92010, 161, { &P20837,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; IRIS_501 Build/LAVAIRIS501) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2962753 = {S92011, 93, { &P14186,&P5096,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5830) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2967509 = {S92012, 88, { &P19860,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A9+) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2978772 = {S92013, 92, { &P15364,&P16110,&P22531,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.2.13) Gecko/X Epic/1.2 Firefox/3.6.10*/
const Device UA2999694 = {S92014, 93, { &P22941,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6812) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3003751 = {S92015, 128, { &P19860,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; A9+ Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3010167 = {S92016, 162, { &P15809,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HTC Rhyme S510b Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3059182 = {S92017, 159, { &P17172,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61647*/
const Device UA3012715 = {S92018, 140, { &P24401,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; moii E996 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3012963 = {S92019, 133, { &P22609,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-S6312 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3012967 = {S92020, 173, { &P22998,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; TECNO_N7 Build/TECNOTECNO_N7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/352*/
const Device UA3015550 = {S92021, 88, { &P17595,&P17294,&P22170,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:22.0) Gecko/X Firefox/22.0 TenFourFox/7450*/
const Device UA3015677 = {S92022, 162, { &P19843,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE Blade III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3078669 = {S92023, 162, { &P19843,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE BLADE III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3060715 = {S92024, 147, { &P13105,&P6437,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.2.1; G100W Build/HTK55D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3017100 = {S92025, 153, { &P18665,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; N861 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3018108 = {S92026, 93, { &P22382,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8262) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3019990 = {S92027, 139, { &P19862,&P18598,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.1; en-in; Micromax A110Q Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3020431 = {S92028, 156, { &P18621,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E610 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3021246 = {S92029, 142, { &P22822,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KPN Smart 300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021576 = {S92030, 141, { &P21614,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Philips W536 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023677 = {S92031, 136, { &P20598,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Miracle Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024227 = {S92032, 175, { &P13054,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ADR6425LVW Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/vigor*/
const Device UA3027534 = {S92033, 90, { &P22641,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; C1504) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3057258 = {S92034, 88, { &P15364,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A12) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3057558 = {S92035, 147, { &P20868,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; M040 Build/JRO03H) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3057819 = {S92036, 132, { &P20453,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NABI2-NV7A Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3057973 = {S92037, 143, { &P24476,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; QMobile_A700 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3109094 = {S92038, 160, { &P23021,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Spice Mi-535 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3061197 = {S92039, 156, { &P22875,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9220 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3059098 = {S92040, 160, { &P20705,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.20.1364.172 Mobile Safari/537.22*/
const Device UA3059710 = {S92041, 150, { &P21794,&P18598,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9152 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3061262 = {S92042, 150, { &P23163,&P18598,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; SCH-I959 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3061518 = {S92043, 107, { &P19868,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_985N) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3093232 = {S92044, 161, { &P20837,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; IRIS_501 Build/LAVAIRIS501) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3078967 = {S92045, 179, { &P15798,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC_Explorer_A310e Build/GRJ90) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3079246 = {S92046, 164, { &P22998,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TECNO_N7 Build/TECNOTECNO_N7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3079710 = {S92047, 156, { &P16356,&P5629,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3108661 = {S92048, 156, { &P16356,&P5629,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3080139 = {S92049, 94, { &P15845,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaC5-06) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3082178 = {S92050, 157, { &P12605,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; XT875 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/targa*/
const Device UA3127360 = {S92051, 157, { &P12605,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; XT875 Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/targa*/
const Device UA3092645 = {S92052, 155, { &P22860,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Q700 Build/XOLOQ700) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3095995 = {S92053, 141, { &P17346,&P17146,&P24345,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) CriOS/29.0.1547.11 Mobile/9A405 Safari/7534.48.3*/
const Device UA3127257 = {S92054, 129, { &P20752,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AN10CG3 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3109941 = {S92055, 152, { &P21472,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; A111 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3111717 = {S92056, 139, { &P23154,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOLO_X1000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114916 = {S92057, 132, { &P18471,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; XT907 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3128483 = {S92058, 160, { &P14862,&P5204,&P24680,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia2700c-2) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3131595 = {S92059, 135, { &P21606,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LGL86C Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132421 = {S92060, 125, { &P14111,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Arc Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3501 = {S92061, 71, { &P13277,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_P6500 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA37968 = {S92062, 105, { &P13277,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC P6500; OpVer 27.157.1.614*/
const Device UA32733 = {S92063, 73, { &P12034,&P7063,&P3606,&P18092 }};	/*Useragent - BlackBerry7130/4.1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/120*/
const Device UA55326 = {S92064, 174, { &P12067,&P17486,&P4112,&P18092 }};	/*Useragent - MOT-EM35/R6743_G_74.12.01ROX Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MotoWebKit/417.19 (KHTML, like Gecko) BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/9.0*/
const Device UA13025 = {S92065, 104, { &P13730,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-P960/P960ASHE1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA14530 = {S92066, 67, { &P13512,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-A117/A117UCGH2 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA15988 = {S92067, 61, { &P14564,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia7900/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA19016 = {S92068, 96, { &P14845,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Touch_HD_T8282 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA24606 = {S92069, 95, { &P15176,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S5503/ SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA21920 = {S92070, 73, { &P12018,&P5448,&P22271,&P18092 }};	/*Useragent - HTC-8500/1.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA23749 = {S92071, 68, { &P15625,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHX830/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23982 = {S92072, 12, { &P15643,&P22835,&P17470,&P18092 }};	/*Useragent - SPICE M5161n*/
const Device UA49168 = {S92073, 26, { &P15643,&P22835,&P17470,&P18092 }};	/*Useragent - SPICE M5161n, SPICE M5161n*/
const Device UA24308 = {S92074, 46, { &P15672,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-LG260/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA26640 = {S92075, 87, { &P13615,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_F900*/
const Device UA26800 = {S92076, 106, { &P13637,&P17483,&P3459,&P18092 }};	/*Useragent - LG/BL42/v10a Browser/Teleca-Q7.3 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA44213 = {S92077, 131, { &P13637,&P17483,&P3459,&P18092 }};	/*Useragent - LG-BL42k/v10b Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA30270 = {S92078, 102, { &P12294,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHA127/A127UCGG5 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA50022 = {S92079, 163, { &P21561,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; LG-P355 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2682790 = {S92080, 160, { &P21561,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en; LG-P355 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA56044 = {S92081, 74, { &P11827,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-D880/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA61841 = {S92082, 56, { &P14338,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-CHARL/00.62 UP.Browser/6.2.3.4.c.1.124 (GUI) MMP/2.0*/
const Device UA62974 = {S92083, 98, { &P14466,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-A701/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA63361 = {S92084, 93, { &P14575,&P19950,&P3625,&P18092 }};	/*Useragent - SAGEM-myX5-2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.6.d.3 (GUI) MMP/1.0*/
const Device UA64135 = {S92085, 98, { &P14575,&P19950,&P3625,&P18092 }};	/*Useragent - SAGEM-myX5-2v/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.3.3.g.2.103 (GUI) MMP/2.0*/
const Device UA459888 = {S92086, 97, { &P14575,&P19950,&P3625,&P18092 }};	/*Useragent - SAGEM-myX5-2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.6.d.3.100 (GUI) MMP/1.0*/
const Device UA87151 = {S92087, 45, { &P17525,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-HI3B UP.Browser/6.2.0.13.2 (GUI) MMP/2.0*/
const Device UA1870085 = {S92088, 31, { &P17723,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F04C(c500;TB;W30H20)*/
const Device UA90525 = {S92089, 47, { &P17628,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-PT36 UP.Browser/6.2.0.17.2.4 (GUI) MMP/2.0*/
const Device UA160412 = {S92090, 119, { &P15364,&P17017,&P17241,&P18092 }};	/*Useragent - AppleWebKit/532.7 (KHTML, like Gecko) Chrome/4.0.271.1 Safari/532.7 TencentTraveler/5.0 QQBrowser/5.0.6642.400 (webkit)*/
const Device UA253778 = {S92091, 8, { &P15364,&P17017,&P17241,&P18092 }};	/*Useragent - Chrome/4*/
const Device UA750250 = {S92092, 46, { &P15364,&P17017,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 AppleWebKit/532.8 Chrome/4.0.288.0*/
const Device UA207086 = {S92093, 80, { &P15364,&P7133,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; Opera Mobi/14316; U; en) Presto/2.7.81 Version/11.00*/
const Device UA312493 = {S92094, 141, { &P17595,&P17295,&P22447,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Iron/9.0.600.2 Chrome/9.0.600.2 Safari/534.13*/
const Device UA2307005 = {S92095, 112, { &P14557,&P5843,&P17347,&P18092 }};	/*Useragent - Nokia6303classic/2.0 (09.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303classic/UC Browser8.0.3.107/69/352*/
const Device UA363703 = {S92096, 130, { &P15767,&P17146,&P20772,&P18092 }};	/*Useragent - MQQBrowser/2.1 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA1940031 = {S92097, 128, { &P15767,&P17146,&P20772,&P18092 }};	/*Useragent - MQQBrowser/2.1 Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Safari/7534.48.3*/
const Device UA368701 = {S92098, 133, { &P14007,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; Racer Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019466 = {S92099, 137, { &P14007,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-au; ZTE Racer Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020272 = {S92100, 133, { &P14007,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Racer Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA560323 = {S92101, 105, { &P15364,&P18145,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.124 Safari/534.30*/
const Device UA2259968 = {S92102, 103, { &P15364,&P18145,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.91 Safari/534.30*/
const Device UA429374 = {S92103, 129, { &P12705,&P5156,&P3519,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9788; zh-CN) AppleWebKit/534.8+ (KHTML, like Gecko) Version/6.0.0.673 Mobile Safari/534.8+*/
const Device UA463907 = {S92104, 135, { &P15364,&P5690,&P22491,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Flock/3.0.0.3737 Chrome/4.1.249.1071 Safari/532.5*/
const Device UA2263726 = {S92105, 133, { &P15364,&P5690,&P22491,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Flock/3.0.0.3989 Chrome/5.0.375.75 Safari/533.4*/
const Device UA496764 = {S92106, 121, { &P15364,&P7133,&P17558,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.13) Gecko/X Firefox/3.6.13 ( .NET CLR 3.5.30729; .NET4.0C) slurp*/
const Device UA547974 = {S92107, 89, { &P15364,&P7133,&P17558,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.17) Gecko/X Firefox/3.6.17 slurp*/
const Device UA1854359 = {S92108, 89, { &P15364,&P7133,&P17558,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.13) Gecko/X Firefox/3.6.13 slurp*/
const Device UA531510 = {S92109, 67, { &P14969,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.3;Zh_cn;HTC Desire S;480*800;)UCWEB7.6.0.75/139/800*/
const Device UA1897385 = {S92110, 69, { &P14969,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.3;Zh_cn;HTC Desire S;480*800;)UCWEB7.6.0.75/139/32154*/
const Device UA2758278 = {S92111, 69, { &P14969,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.5;Zh_cn;HTC Desire S;480*800;)UCWEB7.6.0.75/139/32154*/
const Device UA572394 = {S92112, 43, { &P14985,&P17484,&P17338,&P18092 }};	/*Useragent - samsung-gt-s8530/UC Browser7.9.0.102/69/352*/
const Device UA684847 = {S92113, 169, { &P12616,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; WT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA646625 = {S92114, 134, { &P17935,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PG06100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA723756 = {S92115, 134, { &P17935,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PG06100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA710646 = {S92116, 60, { &P15364,&P23114,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:8.0.1) Gecko/X Firefox/8.0.1*/
const Device UA713088 = {S92117, 85, { &P18141,&P22217,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 1.60) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2*/
const Device UA815082 = {S92118, 99, { &P17595,&P17297,&P21807,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.2.16pre) Gecko/X Prism zdesktop/7.1.4*/
const Device UA2549729 = {S92119, 28, { &P12803,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNGSCHI405/2.3.6-EAS-1.2*/
const Device UA868333 = {S92120, 88, { &P17976,&P6545,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.0.1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.7.81 Version/11.00*/
const Device UA893619 = {S92121, 79, { &P13163,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaN97)/UC Browser8.4.0.159/50/352/UCWEB Mobile*/
const Device UA910382 = {S92122, 71, { &P15364,&P16110,&P18514,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:13.0) Gecko/X Thunderbird/13.0.1*/
const Device UA1952487 = {S92123, 89, { &P12588,&P5055,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiac6-01) UCBrowser8.3.0.154/69/352/UCWEB Mobile  3gpp-gba*/
const Device UA1959109 = {S92124, 85, { &P14182,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s5333) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2023092 = {S92125, 161, { &P21551,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; id-id; E139 Build/I700T_P6_20_AC76_FTN_R_0_04_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2226069 = {S92126, 155, { &P19854,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-R760 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2307224 = {S92127, 62, { &P15364,&P5690,&P18397,&P18092 }};	/*Useragent - Opera/6.10 (Windows NT 5.1; U; pl) Presto/2.1.15 Version/10.10*/
const Device UA2368925 = {S92128, 90, { &P15518,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5230) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2423221 = {S92129, 95, { &P15518,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-s5230) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2428325 = {S92130, 90, { &P15518,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5230) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2430944 = {S92131, 87, { &P15518,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; fa; gt-s5230) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2504998 = {S92132, 87, { &P15518,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; ru; gt-s5230) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2597081 = {S92133, 90, { &P15518,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; es-la; gt-s5230) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2695096 = {S92134, 87, { &P15518,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; fa; gt-s5230) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA3118277 = {S92135, 87, { &P15518,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; vi; gt-s5230) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2372559 = {S92136, 133, { &P21818,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IT-M809RC Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2376470 = {S92137, 139, { &P19859,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC T328w Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2384340 = {S92138, 147, { &P20265,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A52 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2404000 = {S92139, 114, { &P23532,&P17017,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; id; tcl-c616) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2813904 = {S92140, 142, { &P14714,&P5843,&P24670,&P18092 }};	/*Useragent - NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiax3-02) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2422015 = {S92141, 128, { &P22136,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T1Ci Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2426826 = {S92142, 157, { &P18700,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-B5512 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2433388 = {S92143, 223, { &P14433,&P5096,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2446491 = {S92144, 131, { &P21423,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DaVinci Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2450590 = {S92145, 140, { &P22159,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AUXUS AX01 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2454333 = {S92146, 161, { &P12879,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7510 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2455208 = {S92147, 135, { &P16001,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SH-06D Build/S9030) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2455607 = {S92148, 145, { &P15985,&P6437,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2616199 = {S92149, 146, { &P15985,&P6437,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; th-th; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2621407 = {S92150, 143, { &P15985,&P6437,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; en-; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2456417 = {S92151, 33, { &P22987,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SAMSUNG GT-C3303)*/
const Device UA2472426 = {S92152, 84, { &P15364,&P18110,&P20445,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686 on x86_64; rv:2.0.1) Gecko/X Firefox/4.0.1 Fennec/2.0.1*/
const Device UA2479892 = {S92153, 147, { &P21840,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Woxter Tablet PC 85 HDS Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2481628 = {S92154, 63, { &P23359,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; A2; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2484828 = {S92155, 177, { &P15201,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Sensation Z710e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2487223 = {S92156, 171, { &P20332,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax P275 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2487845 = {S92157, 69, { &P15364,&P16110,&P19981,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:19.0) Gecko/X Thunderbird/19.0*/
const Device UA2520558 = {S92158, 160, { &P15483,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Chacha A810 Build/ASN01; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2527751 = {S92159, 98, { &P15364,&P22324,&P17193,&P18092 }};	/*Useragent - Microsoft Internet Explorer; MSIE 7.3.2; Windows NT 5.4; Mozilla Network 2.0 XP Professional WOW64*/
const Device UA2529327 = {S92160, 135, { &P23090,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i9400 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2529673 = {S92161, 129, { &P15198,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Genio Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2545571 = {S92162, 149, { &P13775,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB520 Build/GWK74; CyanogenMod-72) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2548058 = {S92163, 119, { &P15364,&P17188,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA2550185 = {S92164, 97, { &P18182,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Nokia)*/
const Device UA2553253 = {S92165, 165, { &P21023,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; LG-E405 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2654911 = {S92166, 161, { &P11646,&P17485,&P24673,&P18092 }};	/*Useragent - SonyEricssonW595/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonw595) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2555993 = {S92167, 143, { &P24552,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB A972BK Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2556704 = {S92168, 146, { &P11747,&P24693,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; HTC Desire Build/AOKP VJ 2.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2556843 = {S92169, 149, { &P20158,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; A7* Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2560059 = {S92170, 37, { &P18176,&P5448,&P20770,&P18092 }};	/*Useragent - NOKIA-Lumia 900/UCWEB2.8.0.249/47/355*/
const Device UA2583314 = {S92171, 146, { &P20104,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 201M Build/7.7.1Q-164_SMJ-165) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2594095 = {S92172, 133, { &P22056,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TM-MID755 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2596134 = {S92173, 132, { &P20154,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5770D Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2599274 = {S92174, 98, { &P12399,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V5; en-US; SonyEricssonU8i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2600261 = {S92175, 140, { &P15354,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST17i Build/4.1.B.0.587) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2658402 = {S92176, 88, { &P17976,&P17979,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android android4.0; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA3044817 = {S92177, 82, { &P17976,&P17979,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Android v2.2; Linux; Opera Tablet/ADR-X) Presto/2.11.355 Version/12.10*/
const Device UA2666890 = {S92178, 136, { &P18600,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-03D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2666997 = {S92179, 122, { &P21853,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Y3 Build/HCL) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667200 = {S92180, 132, { &P20527,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A6 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667487 = {S92181, 138, { &P12482,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T959 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2669372 = {S92182, 128, { &P20127,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT270 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2677297 = {S92183, 137, { &P16436,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8150 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2694811 = {S92184, 172, { &P17952,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SHW-M110S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2706714 = {S92185, 90, { &P14993,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT15i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2707464 = {S92186, 131, { &P21030,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAC-97032 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708233 = {S92187, 144, { &P22998,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TECNO_N7 Build/TECNOTECNO_N7) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709087 = {S92188, 132, { &P20498,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP7100D3G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2709489 = {S92189, 141, { &P20491,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Thunder S220 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714192 = {S92190, 136, { &P19911,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-E970 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2729803 = {S92191, 90, { &P15079,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; MK16i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2730744 = {S92192, 90, { &P18449,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST27i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2734411 = {S92193, 169, { &P19763,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; C6602 Build/10.1.A.1.434) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2749549 = {S92194, 95, { &P20959,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Celkon_A88) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2767902 = {S92195, 93, { &P14433,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I9003) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2779176 = {S92196, 165, { &P20864,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; A13-MID Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2791681 = {S92197, 165, { &P20864,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A13-MID Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2781792 = {S92198, 105, { &P13706,&P5129,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) Nokia5235*/
const Device UA2781950 = {S92199, 90, { &P17625,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST25i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2794156 = {S92200, 156, { &P12972,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; SGH-T989 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2794494 = {S92201, 139, { &P20244,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; bq Elcano Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2795979 = {S92202, 97, { &P18462,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; HTC Desire X) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2958075 = {S92203, 97, { &P18462,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; HTC Desire X) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2797391 = {S92204, 131, { &P21030,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAC-97032 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2798887 = {S92205, 128, { &P20844,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB462 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2798921 = {S92206, 136, { &P21525,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SBM203SH Build/S0018) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798931 = {S92207, 134, { &P18740,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; U7015 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799036 = {S92208, 142, { &P19781,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 97 TITANIUMHD Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2801264 = {S92209, 148, { &P22968,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HUAWEI P6-U06 Build/HuaweiP6-U06) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2863234 = {S92210, 169, { &P12788,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PG86100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (PG86100, 42, 1.9.1)*/
const Device UA2857305 = {S92211, 149, { &P18321,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-S5300 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2858913 = {S92212, 63, { &P21064,&P17483,&P3459,&P18092 }};	/*Useragent - LG-A447/V100 Obigo/Q7.3 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2954572 = {S92213, 143, { &P21378,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; tablet Fnac 8 Build/1.1.0 X-12:19) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2879363 = {S92214, 163, { &P20980,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; A97i Build/CelkonA97i_Celkon) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2957682 = {S92215, 159, { &P21094,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo A789 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2866547 = {S92216, 148, { &P20529,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A89 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2866822 = {S92217, 132, { &P23217,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP7170B3G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2867047 = {S92218, 137, { &P19853,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-N7102 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869036 = {S92219, 141, { &P23011,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; iris405 Build/LAVAIRIS405) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869246 = {S92220, 149, { &P20991,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ME371MG Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2870044 = {S92221, 158, { &P23772,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2876180 = {S92222, 131, { &P16168,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; NookColor Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2877171 = {S92223, 137, { &P18630,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A800S Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2897698 = {S92224, 152, { &P23672,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2898277 = {S92225, 140, { &P18372,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-04D Build/9.1.C.0.475) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2933865 = {S92226, 60, { &P18241,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; LG-E400 Build/IMM76L)*/
const Device UA3119586 = {S92227, 138, { &P18241,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-E400 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2935757 = {S92228, 177, { &P12841,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC PH39100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (HTC PH39100, 42, 1.9.1)*/
const Device UA3000086 = {S92229, 93, { &P18679,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-B5330) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2937105 = {S92230, 154, { &P18296,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC One V Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2940914 = {S92231, 133, { &P21950,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; iDxD10 3G Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2998096 = {S92232, 155, { &P19917,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LG-P765 Build/IMM76L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2956839 = {S92233, 153, { &P21826,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; LifeStyle 10DCIPS(BO-LF10DCI) Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2957754 = {S92234, 97, { &P22903,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A62) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2958284 = {S92235, 71, { &P15364,&P16110,&P22170,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:22.0) Gecko/X Firefox/22.0 slurp*/
const Device UA2966557 = {S92236, 158, { &P11747,&P5096,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-US; HTC Desire Build/FRG83G) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3124356 = {S92237, 155, { &P11747,&P5096,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-US; HTC Desire Build/FRF91) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2968178 = {S92238, 135, { &P18302,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-in; MT25i Build/4.1.B.0.631) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2989131 = {S92239, 88, { &P20158,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A7*) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2998301 = {S92240, 90, { &P12616,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; WT19i) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2998145 = {S92241, 137, { &P22875,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-i9220 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3049622 = {S92242, 137, { &P22875,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9220 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2998655 = {S92243, 148, { &P19843,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ZTE Blade III Pro Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3012404 = {S92244, 93, { &P15575,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; GT-I9100) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3015872 = {S92245, 156, { &P13074,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P930 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3016107 = {S92246, 157, { &P18559,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LG730 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3020625 = {S92247, 145, { &P24687,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One dual 802d Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3058356 = {S92248, 136, { &P24687,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC 802d Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023751 = {S92249, 93, { &P15575,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; GT-I9100) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3027612 = {S92250, 161, { &P14186,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5830 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3039511 = {S92251, 130, { &P18556,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; BN NookHD+ Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3043176 = {S92252, 104, { &P12581,&P5055,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia500/111.021.0028/sw_platform=S60;sw_platform_version=5.3;java_build_version=2.3.24)*/
const Device UA3048754 = {S92253, 93, { &P18304,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9070) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA3050766 = {S92254, 80, { &P24615,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-S3332/S3332JVMD1 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA3051043 = {S92255, 132, { &P22594,&P18598,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.2; en-us; SGH-M919 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3058421 = {S92256, 107, { &P18510,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL ONE TOUCH 918N) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3126060 = {S92257, 107, { &P18510,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL ONE TOUCH 918N) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3092820 = {S92258, 173, { &P23255,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI G520-5000 Build/HuaweiG520-5000) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3060072 = {S92259, 103, { &P14748,&P5890,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae72-1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3069627 = {S92260, 128, { &P15364,&P16110,&P22417,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.2.1.0 Chrome/16.0.912.75 Safari/535.7*/
const Device UA3117201 = {S92261, 121, { &P15364,&P16110,&P22417,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.7 (KHTML, like Gecko) Comodo_Dragon/16.2.1.0 Chrome/16.0.912.75 Safari/535.7*/
const Device UA3078700 = {S92262, 93, { &P22607,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S5303) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3090596 = {S92263, 162, { &P14959,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9103 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/i9103*/
const Device UA3092687 = {S92264, 92, { &P12029,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN82) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3092909 = {S92265, 88, { &P20515,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; A54) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3110302 = {S92266, 133, { &P24542,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARNOVA 90G3 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094231 = {S92267, 129, { &P23236,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AMI-TT4 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3116521 = {S92268, 112, { &P15364,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia208/03.39; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.55*/
const Device UA3127683 = {S92269, 93, { &P13974,&P5629,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5670) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3130956 = {S92270, 128, { &P24366,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SM-T311 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3131867 = {S92271, 137, { &P12972,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132663 = {S92272, 136, { &P22685,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E460 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA18797 = {S92273, 61, { &P11933,&P5768,&P4617,&P18092 }};	/*Useragent - Nokia6136/2.0 (04.01) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2465 = {S92274, 52, { &P11977,&P17483,&P17660,&P18092 }};	/*Useragent - LG-GW520/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA7378 = {S92275, 34, { &P13319,&P22839,&P17470,&P18092 }};	/*Useragent - ZTE-G N281/WAP2.0 Profile/MIDP-2.0*/
const Device UA8735 = {S92276, 87, { &P13502,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch2_T3335; Windows Phone 6.5)*/
const Device UA43909 = {S92277, 141, { &P18005,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-vn; V8401 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA21664 = {S92278, 39, { &P15319,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP215 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA23900 = {S92279, 56, { &P15636,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-SPARK/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0*/
const Device UA24680 = {S92280, 66, { &P15735,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GM310/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA26368 = {S92281, 188, { &P15364,&P7133,&P4715,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.4.0.65/70/352*/
const Device UA493615 = {S92282, 188, { &P15364,&P7133,&P4715,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.4.0.65/27/351*/
const Device UA771438 = {S92283, 188, { &P15364,&P7133,&P4715,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.4.0.65/50/355*/
const Device UA1849046 = {S92284, 188, { &P15364,&P7133,&P4715,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.4.0.65/28/352*/
const Device UA2267900 = {S92285, 188, { &P15364,&P7133,&P4715,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.4.0.65/50/351*/
const Device UA31367 = {S92286, 104, { &P12418,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-A657/A657UCIB5 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA56781 = {S92287, 62, { &P13337,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonJ108i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA651893 = {S92288, 115, { &P14635,&P5843,&P17436,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-00/07.20; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.3.23.12*/
const Device UA48984 = {S92289, 29, { &P13881,&P22839,&P17470,&P18092 }};	/*Useragent - ZTE-R230/2.0 Profile/MIDP-2.0*/
const Device UA50218 = {S92290, 32, { &P13198,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH04A(c100;TB;W24H16)*/
const Device UA68320 = {S92291, 36, { &P13324,&P5055,&P4759,&P18092 }};	/*Useragent - NokiaN8-00/UC Browser7.6.1.82/50/355*/
const Device UA2135207 = {S92292, 46, { &P13324,&P5055,&P4759,&P18092 }};	/*Useragent - nokian8-00/UC Browser7.6.1.82/69/351  3gpp-gba*/
const Device UA68427 = {S92293, 34, { &P13163,&P5129,&P4777,&P18092 }};	/*Useragent - NokiaN97/UC Browser7.7.1.88/50/444*/
const Device UA2978092 = {S92294, 36, { &P13163,&P5129,&P4777,&P18092 }};	/*Useragent - nokian97-1/UC Browser7.7.1.88/69/444*/
const Device UA87222 = {S92295, 45, { &P17589,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SN3E UP.Browser/6.2.0.13.2 (GUI) MMP/2.0*/
const Device UA87305 = {S92296, 12, { &P16274,&P17483,&P17660,&P18092 }};	/*Useragent - LG/U900/v1.0*/
const Device UA385441 = {S92297, 54, { &P17873,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3Q UP.Browser/6.2_7.2.7.1.K.4.306 (GUI) MMP/2.0*/
const Device UA270844 = {S92298, 77, { &P17968,&P17979,&P4536,&P18092 }};	/*Useragent - Opera/9.80 (Android; Linux; Opera Mobi/49; U; en) Presto/2.4.18 Version/10.00*/
const Device UA370649 = {S92299, 136, { &P23948,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; vi-vn; Gemei G2 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2758207 = {S92300, 60, { &P23948,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; Gemei G2 Build/GRJ22)*/
const Device UA433726 = {S92301, 36, { &P15364,&P18110,&P17215,&P18092 }};	/*Useragent - Opera/9.50 (Linux armv5tejl ; U; en)*/
const Device UA460843 = {S92302, 47, { &P11684,&P5129,&P4722,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UC Browser7.0.0.41/50/351*/
const Device UA566468 = {S92303, 90, { &P12292,&P5448,&P4777,&P18092 }};	/*Useragent - HTC_Touch_3G_T3232 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA567683 = {S92304, 56, { &P15364,&P23114,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:6.0) Gecko/X Firefox/6.0*/
const Device UA572933 = {S92305, 60, { &P15364,&P23114,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:6.0.2) Gecko/X Firefox/6.0.2*/
const Device UA572153 = {S92306, 36, { &P11945,&P17484,&P17338,&P18092 }};	/*Useragent - gt-s5233s/UC Browser7.9.0.102/69/444*/
const Device UA774643 = {S92307, 36, { &P11945,&P17484,&P17338,&P18092 }};	/*Useragent - gt-s5233s/UC Browser7.9.0.102/69/351*/
const Device UA575421 = {S92308, 47, { &P12375,&P5890,&P24665,&P18092 }};	/*Useragent - NOKIANokia N81/UC Browser8.0.3.107/28/351/UCWEB*/
const Device UA2311937 = {S92309, 51, { &P12375,&P5890,&P24665,&P18092 }};	/*Useragent - NOKIANokia N81 8GB/UC Browser8.0.4.121/28/352/UCWEB*/
const Device UA605565 = {S92310, 188, { &P15364,&P7133,&P18326,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.2.1.144/69/352*/
const Device UA660470 = {S92311, 188, { &P15364,&P7133,&P18326,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.2.1.144/69/444*/
const Device UA1989526 = {S92312, 188, { &P15364,&P7133,&P18326,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.2.0.132/69/452*/
const Device UA2259689 = {S92313, 188, { &P15364,&P7133,&P18326,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.2.1.144/69/405*/
const Device UA2315724 = {S92314, 198, { &P15364,&P7133,&P18326,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.2.1.144/69/444  3gpp-gba*/
const Device UA2331327 = {S92315, 188, { &P15364,&P7133,&P18326,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.2.0.132/70/352*/
const Device UA2589229 = {S92316, 188, { &P15364,&P7133,&P18326,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.2.1.144/70/352*/
const Device UA667447 = {S92317, 90, { &P17595,&P17295,&P18368,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:8.0) Gecko/X Thunderbird/8.0 Lightning/1.0*/
const Device UA696551 = {S92318, 37, { &P12456,&P5890,&P17347,&P18092 }};	/*Useragent - nokian73-1/UC Browser8.0.3.107/69/352*/
const Device UA704794 = {S92319, 37, { &P11854,&P5890,&P17338,&P18092 }};	/*Useragent - nokiae71-1/UC Browser7.9.0.102/69/352*/
const Device UA710263 = {S92320, 90, { &P15364,&P7133,&P21803,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.0.1) Gecko/X Prism (zdesktop/1.0.4)*/
const Device UA814983 = {S92321, 68, { &P20369,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux;U;2.2.2;Zh_cn;ZTE-C N880S;480*800;)UCWEB7.6.0.75/139/32026*/
const Device UA905513 = {S92322, 81, { &P14983,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaC7-00)/UC Browser8.4.0.159/50/352/UCWEB Mobile*/
const Device UA898977 = {S92323, 148, { &P14743,&P17485,&P24668,&P18092 }};	/*Useragent - SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonj20i) UCBrowser8.3.0.154/69/352/UCWEB Mobile*/
const Device UA1989048 = {S92324, 155, { &P18295,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC One X Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027415 = {S92325, 159, { &P18295,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; HTC One X Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA920475 = {S92326, 11, { &P15364,&P17017,&P17698,&P18092 }};	/*Useragent - Firefox/8.0*/
const Device UA1945306 = {S92327, 50, { &P15364,&P17017,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (SaferSurf rv:8.0) Gecko/X Firefox/8.0*/
const Device UA1967306 = {S92328, 176, { &P14225,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2032838 = {S92329, 166, { &P14225,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2020729 = {S92330, 138, { &P16539,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Micromax A85 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029227 = {S92331, 161, { &P20120,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A45 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2026518 = {S92332, 149, { &P15649,&P5096,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026711 = {S92333, 69, { &P17323,&P6437,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android3.2;Zh_cn;GT-P6800;800*1232;)UCWEB7.8.0.95/139/352*/
const Device UA2612140 = {S92334, 118, { &P17323,&P6437,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android3.2;Zh_cn;GT-P6800;800*1232;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA2028721 = {S92335, 131, { &P23508,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-uk; GEM8012 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2031765 = {S92336, 146, { &P15539,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Glacier Build/GRJ22)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2892617 = {S92337, 147, { &P15539,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2262225 = {S92338, 150, { &P12132,&P17485,&P24668,&P18092 }};	/*Useragent - SonyEricssonJ10i2/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonj10i2) UCBrowser8.3.0.154/69/352/UCWEB Mobile*/
const Device UA2340427 = {S92339, 143, { &P12562,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID2 Global Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2352764 = {S92340, 53, { &P15364,&P17350,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows) Gecko Firefox/12. -CareMediCom-*/
const Device UA2695237 = {S92341, 252, { &P14540,&P5843,&P18527,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/69/444*/
const Device UA2404701 = {S92342, 120, { &P13648,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaE7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile*/
const Device UA2433065 = {S92343, 229, { &P15060,&P5629,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2459976 = {S92344, 229, { &P15060,&P5629,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2455376 = {S92345, 150, { &P14967,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; en-in; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2455723 = {S92346, 135, { &P21972,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Tablet 723B Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2456367 = {S92347, 153, { &P20535,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Kem 350 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2456880 = {S92348, 146, { &P15555,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; myTouch_4G_Slide Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2789081 = {S92349, 156, { &P12110,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Legend Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2475047 = {S92350, 112, { &P15364,&P5690,&P3499,&P18092 }};	/*Useragent - TE1303 Mozilla/5.0 (Windows; U; Windows NT 5.1; en; like Gecko) Obigo/Q7 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2484900 = {S92351, 137, { &P21489,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Rezound Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2496743 = {S92352, 77, { &P23066,&P22832,&P4780,&P18092 }};	/*Useragent - Videocon V1548_Maui Wap Browser/3.83; Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2502597 = {S92353, 68, { &P17323,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 3.2; zh-cn; GT-P6800; 800*1232) UCWEB7.9.0.94/139/352*/
const Device UA2527779 = {S92354, 145, { &P22675,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GOCLEVER NETBOOK R103 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2529127 = {S92355, 129, { &P24556,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; CT704 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2555417 = {S92356, 141, { &P23811,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE-U V889F Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2595107 = {S92357, 120, { &P19976,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaE6-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/352/UCWEB Mobile*/
const Device UA2597690 = {S92358, 171, { &P12972,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2604498 = {S92359, 135, { &P23383,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; H7100 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2617500 = {S92360, 79, { &P12889,&P17979,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.4.6) (GT-P7300;samsung;GT-P7300;3.1/HMJ37)*/
const Device UA2610622 = {S92361, 86, { &P15876,&P17979,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.4.5) (espresso10rf;samsung;GT-P5100;4.0.3/IML74K)*/
const Device UA2627551 = {S92362, 150, { &P14908,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P6210 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2634672 = {S92363, 150, { &P14908,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P6210 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2629871 = {S92364, 162, { &P13452,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-P1000M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000*/
const Device UA2630598 = {S92365, 161, { &P13452,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; ru-ru; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000*/
const Device UA2630599 = {S92366, 161, { &P13452,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000*/
const Device UA2632003 = {S92367, 162, { &P13452,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; GT-P1000L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000*/
const Device UA2635106 = {S92368, 162, { &P13452,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; th-th; GT-P1000T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000*/
const Device UA2631176 = {S92369, 61, { &P17323,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-P6800 Build/JOP40D)*/
const Device UA2890995 = {S92370, 131, { &P17323,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-p6800 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2645687 = {S92371, 157, { &P14202,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; E15i Build/JRO03L; CyanogenMod-MiniCM10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2790668 = {S92372, 157, { &P14202,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; E15i Build/JZO54K; CyanogenMod-MiniCM10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2652769 = {S92373, 136, { &P22425,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; el-gr; A7710 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2657294 = {S92374, 71, { &P15364,&P18145,&P19819,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64) KHTML/4.9.3 (like Gecko) Konqueror/4.9*/
const Device UA2659953 = {S92375, 186, { &P12973,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2665402 = {S92376, 161, { &P20518,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A10 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2665652 = {S92377, 91, { &P23417,&P22843,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lava-a10) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2667407 = {S92378, 131, { &P22662,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Enjoy 71 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2668584 = {S92379, 162, { &P21068,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; MI-ONE C1 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2671028 = {S92380, 126, { &P18008,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ST10 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2691411 = {S92381, 93, { &P15364,&P17350,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; ; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.1 Safari/533.3*/
const Device UA2701373 = {S92382, 151, { &P20096,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7568 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2705226 = {S92383, 140, { &P13054,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ADR6425LVW Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2708374 = {S92384, 142, { &P21623,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 6 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709493 = {S92385, 137, { &P21028,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Smartfren Tab 7 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2710640 = {S92386, 134, { &P21261,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i9500 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2732944 = {S92387, 136, { &P22344,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; eZee'Tab 971 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2734584 = {S92388, 162, { &P18581,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2734647 = {S92389, 137, { &P18622,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A820L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734755 = {S92390, 137, { &P18630,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A800S Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2750654 = {S92391, 156, { &P16436,&P5629,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2750724 = {S92392, 136, { &P23275,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; POV_TAB-P825 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2764354 = {S92393, 156, { &P16553,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-T959V Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2764445 = {S92394, 98, { &P20332,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax_P275) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2783102 = {S92395, 134, { &P21261,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i9500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2794356 = {S92396, 106, { &P13324,&P5055,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaN8-00*/
const Device UA2795880 = {S92397, 156, { &P18437,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N8000 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2799963 = {S92398, 138, { &P23381,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A5Classic Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2825350 = {S92399, 155, { &P23914,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; R800x Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2842372 = {S92400, 139, { &P23378,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; Informer 708 3G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2850268 = {S92401, 139, { &P23378,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Informer 708 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2844277 = {S92402, 67, { &P20076,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; LT26w; 720*1280) UCWEB7.9.0.94/139/352*/
const Device UA2848157 = {S92403, 90, { &P17976,&P18598,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.2.1; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.10.254 Version/12.00*/
const Device UA2855213 = {S92404, 212, { &P18447,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT30p Build/JZO54K; CyanogenMod-CM10 base (X unofficial build by quarx, compiled by sevenrock)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2951979 = {S92405, 159, { &P18447,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Xperia T Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2855383 = {S92406, 149, { &P21239,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; M9 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2859218 = {S92407, 141, { &P24563,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Q-Smart S21 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2871703 = {S92408, 155, { &P17595,&P18120,&P22776,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.5.3.X (Second Life Release; default skin) Safari/533.3*/
const Device UA2960185 = {S92409, 93, { &P13974,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5670) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2865469 = {S92410, 89, { &P11950,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; X10i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2866738 = {S92411, 124, { &P18465,&P18066,&P23371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KFOT Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/19.77.34.5 Safari/537.22*/
const Device UA2877108 = {S92412, 136, { &P23564,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HS-U950 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2878828 = {S92413, 142, { &P21526,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Cherry w500 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879511 = {S92414, 133, { &P20976,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; W100 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2891625 = {S92415, 174, { &P15364,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; S7100 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33745*/
const Device UA2891784 = {S92416, 139, { &P23710,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Wynncom G32 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2938429 = {S92417, 152, { &P19859,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2945657 = {S92418, 158, { &P15539,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; HTC Glacier Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2962974 = {S92419, 100, { &P20126,&P17485,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk800i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2965915 = {S92420, 89, { &P11950,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; X10i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2984253 = {S92421, 165, { &P18460,&P18066,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI U8950D Build/HuaweiU8950D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984894 = {S92422, 158, { &P15541,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Xperia Play Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994965 = {S92423, 135, { &P24542,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; ARNOVA 90G3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3005177 = {S92424, 137, { &P22622,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-us; Micromax A72 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3006054 = {S92425, 142, { &P24494,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; Q-Smart S53 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3015876 = {S92426, 92, { &P14438,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE50) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3016696 = {S92427, 139, { &P17346,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.4m*/
const Device UA3110398 = {S92428, 139, { &P17346,&P19826,&P18353,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.4.5m*/
const Device UA3017729 = {S92429, 157, { &P14877,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3021585 = {S92430, 136, { &P20937,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR910L Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022144 = {S92431, 137, { &P17324,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9001 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023381 = {S92432, 133, { &P21933,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M729 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023418 = {S92433, 133, { &P24053,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SHL22 Build/S6290) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023907 = {S92434, 136, { &P24165,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; 3G7334i Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024022 = {S92435, 133, { &P18008,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HYUNDAI T10 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3028232 = {S92436, 158, { &P14993,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT15i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3049018 = {S92437, 173, { &P15364,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Xperia Live with Walkman Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3123343 = {S92438, 151, { &P15364,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; EPAD Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.1/tf201*/
const Device UA3058851 = {S92439, 145, { &P22594,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.5;FBBV/X;FBDM/{density=3.0,width=1080,height=1920};FBLC/en_US;FBCR/T-Mobile;FBPN/com.facebook.katana;FBDV/SGH-M919;FBSV/4.2.2;]*/
const Device UA3109281 = {S92440, 172, { &P22594,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.6.1;FBBV/X;FBDM/{density=3.0,width=1080,height=1920};FBLC/en_US;FBCR/T-Mobile;FBPN/com.facebook.katana;FBDV/SGH-M919;FBSV/4.2.2;FBCA/armeabi-v7a:armeabi;]*/
const Device UA3059046 = {S92441, 156, { &P14693,&P5843,&P24679,&P18092 }};	/*Useragent - NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac1-02) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3059559 = {S92442, 103, { &P14983,&P5055,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac7-00) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3060992 = {S92443, 97, { &P13815,&P5096,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; HTC_Wildfire) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3064438 = {S92444, 145, { &P21948,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID0714PGE01.133 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3110522 = {S92445, 137, { &P20597,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F100S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3066899 = {S92446, 155, { &P19917,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-P765 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3073345 = {S92447, 126, { &P20513,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; A5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078401 = {S92448, 93, { &P18679,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-B5330) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3079168 = {S92449, 90, { &P17625,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST25i) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3080393 = {S92450, 137, { &P18509,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-MS770 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3088819 = {S92451, 140, { &P22969,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; XT915 Build/2_32A_2013) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103395 = {S92452, 140, { &P22969,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; XT915 Build/2_32A_2008) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3094588 = {S92453, 152, { &P21472,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A111 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3092849 = {S92454, 92, { &P21023,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; LG-E405) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3093408 = {S92455, 94, { &P19976,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaE6-00) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3106804 = {S92456, 102, { &P15364,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-SGH-T289/T289UVME3 SHP/VPP/R5 Dolfin/1.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA3110572 = {S92457, 134, { &P23957,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; FPT V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109369 = {S92458, 138, { &P20737,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-S730G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109713 = {S92459, 161, { &P18382,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire C Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3127211 = {S92460, 138, { &P23156,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOLO A700 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3128942 = {S92461, 158, { &P12968,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT26i Build/6.1.A.2.55) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3132945 = {S92462, 138, { &P24044,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-R740C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA4021 = {S92463, 157, { &P13310,&P17483,&P3459,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-GT350/V10a Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA5091 = {S92464, 91, { &P13617,&P5690,&P17215,&P18092 }};	/*Useragent - Opera/9.50 (Windows NT 5.1; MDA_Vario_V/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1; U; de)*/
const Device UA19956 = {S92465, 69, { &P17613,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-B270i/B270iBGID1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA20662 = {S92466, 46, { &P14270,&P17484,&P5001,&P18092 }};	/*Useragent - SCH-U540/1.0 NetFront/3.0.22.1.5 (GUI) MMP/2.0*/
const Device UA54923 = {S92467, 46, { &P14270,&P17484,&P5001,&P18092 }};	/*Useragent - SCH-U540/1.0 NetFront/3.0.22.2.5 (GUI) MMP/2.0*/
const Device UA55064 = {S92468, 117, { &P14280,&P17484,&P3834,&P18092 }};	/*Useragent - SAMSUNG-GT-S5530/S5530XXJG8 SHP/VPP/R5 Dolfin/2.0 QTV/5.3 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1 OPN-B*/
const Device UA31787 = {S92469, 34, { &P14763,&P5776,&P4722,&P18092 }};	/*Useragent - nokian72/UC Browser7.0.0.41/69/351*/
const Device UA50154 = {S92470, 100, { &P14763,&P5776,&P4722,&P18092 }};	/*Useragent - NokiaN72/5.0819.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.0.0.41/27/352*/
const Device UA40328 = {S92471, 60, { &P17497,&P17662,&P3601,&P18092 }};	/*Useragent - Mozilla/4.08 (740SC;SoftBank;SN354715035737340) NetFront/3.3*/
const Device UA40330 = {S92472, 42, { &P17497,&P17662,&P3601,&P18092 }};	/*Useragent - Mozilla/4.08 (740SC;SoftBank) NetFront/3.3*/
const Device UA55529 = {S92473, 60, { &P17497,&P17662,&P3601,&P18092 }};	/*Useragent - Mozilla/4.08 (740SC;SoftBank;SN354715035305338) NetFront/3.3*/
const Device UA57036 = {S92474, 60, { &P17497,&P17662,&P3601,&P18092 }};	/*Useragent - Mozilla/4.08 (740SC;SoftBank;SN354715035629794) NetFront/3.3*/
const Device UA77204 = {S92475, 60, { &P17497,&P17662,&P3601,&P18092 }};	/*Useragent - Mozilla/4.08 (740SC;SoftBank;SN354715034163506) NetFront/3.3*/
const Device UA349527 = {S92476, 46, { &P17497,&P17662,&P3601,&P18092 }};	/*Useragent - SoftBank/1.0/740SC/SCJ001 Browser/NetFront/3.3*/
const Device UA49091 = {S92477, 73, { &P13897,&P22835,&P17470,&P18092 }};	/*Useragent - SPICE-M6464/1.0 IAC/M6235 IAC/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49539 = {S92478, 148, { &P13897,&P22835,&P17470,&P18092 }};	/*Useragent - SPICE-M6464/1.0 IAC/M6235 IAC/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1, SPICE-M6464/1.0 IAC/M6235 IAC/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53931 = {S92479, 98, { &P14067,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-Z560/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA82189 = {S92480, 107, { &P14067,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-Z560-Vodafone/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA67224 = {S92481, 71, { &P11782,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-S3600/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA69432 = {S92482, 31, { &P11907,&P5890,&P24663,&P18092 }};	/*Useragent - NokiaE52-1/UCWEB7.9.0.94/28/999*/
const Device UA648411 = {S92483, 43, { &P11907,&P5890,&P24663,&P18092 }};	/*Useragent - NokiaE52-1/UC Browser7.9.0.102/28/444/UCWEB*/
const Device UA79291 = {S92484, 65, { &P15262,&P17483,&P17602,&P18092 }};	/*Useragent - LG-A255/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA86533 = {S92485, 85, { &P17456,&P5448,&P22371,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.9) VZW:SCH-i760 PPC 240x320*/
const Device UA156767 = {S92486, 131, { &P17346,&P17144,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A293 Twitter for iPhone*/
const Device UA163570 = {S92487, 133, { &P17346,&P17144,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8A400 Twitter for iPhone*/
const Device UA201058 = {S92488, 36, { &P11905,&P5890,&P4777,&P18092 }};	/*Useragent - NOKIA6120c/UC Browser7.7.1.88/28/444*/
const Device UA253927 = {S92489, 99, { &P14042,&P5448,&P4777,&P18092 }};	/*Useragent - SonyEricssonX2/R3AA Browser/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA317164 = {S92490, 138, { &P15364,&P6545,&P18245,&P18091 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Bottle of Smoke Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA491605 = {S92491, 80, { &P15364,&P5448,&P4715,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)/UC Browser7.4.0.65*/
const Device UA2127872 = {S92492, 110, { &P15364,&P5448,&P4715,&P18092 }};	/*Useragent - 480x800; XV6875.1; Window Mobile 6.5 Professional; (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65*/
const Device UA538717 = {S92493, 61, { &P20317,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX262/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA2312550 = {S92494, 27, { &P16553,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-T959V/100.20306*/
const Device UA552175 = {S92495, 43, { &P17595,&P17296,&P21585,&P18092 }};	/*Useragent - MacOutlook/14.0.0.X (Intel Mac OS X 10.6.8)*/
const Device UA816363 = {S92496, 97, { &P15364,&P7127,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:6.0.2) Gecko/X Firefox/6.0.2,gzip(gfe) (via translate.google.com)*/
const Device UA854095 = {S92497, 99, { &P17595,&P17296,&P21807,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.2.16pre) Gecko/X Prism zdesktop/7.1.4*/
const Device UA573333 = {S92498, 106, { &P14743,&P17485,&P17338,&P18092 }};	/*Useragent - SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonj20i/UC Browser7.9.0.102/69/352*/
const Device UA644428 = {S92499, 114, { &P14540,&P5843,&P17437,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-01/07.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.4.0.34.8*/
const Device UA642643 = {S92500, 142, { &P22496,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; hu-hu; xTAB-100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA700337 = {S92501, 47, { &P13552,&P5129,&P17338,&P18092 }};	/*Useragent - nokiac6-00/UC Browser7.9.0.102/69/352  3gpp-gba*/
const Device UA893422 = {S92502, 85, { &P12142,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU5)/UC Browser8.4.0.159/50/352/UCWEB Mobile*/
const Device UA1901132 = {S92503, 106, { &P11646,&P17485,&P17338,&P18092 }};	/*Useragent - SonyEricssonW595/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonw595/UC Browser7.9.0.102/69/352*/
const Device UA1949227 = {S92504, 138, { &P15106,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_Salsa_C510b; en-au) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA1951016 = {S92505, 152, { &P21396,&P5629,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A65 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1958952 = {S92506, 79, { &P15206,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE90)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA2026892 = {S92507, 163, { &P13452,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA1990309 = {S92508, 92, { &P15364,&P16110,&P22359,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:12.1) Gecko/X Firefox/12.1-x64 PaleMoon/12.1-x64*/
const Device UA2641442 = {S92509, 72, { &P15364,&P16110,&P22359,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; rv:12.1) Gecko/X Firefox/12.1 PaleMoon/12.1*/
const Device UA2024836 = {S92510, 166, { &P13591,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; ZTE-BLADE Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2008108 = {S92511, 114, { &P15364,&P5690,&P18532,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.60 Safari/537.1 Sleipnir/3.6.1*/
const Device UA2023926 = {S92512, 129, { &P20708,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A1000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2554272 = {S92513, 130, { &P20708,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MA1000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2028171 = {S92514, 129, { &P21161,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; MP747 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2150483 = {S92515, 54, { &P12088,&P5890,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 (NokiaX6-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA2445585 = {S92516, 83, { &P12329,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN86 8MP)/UC Browser8.5.0.183/28/444/UCWEB Mobile*/
const Device UA3041707 = {S92517, 162, { &P12139,&P5204,&P24667,&P18092 }};	/*Useragent - Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia5310xpressmusic) UCBrowser8.2.0.132/69/452/UCWEB Mobile*/
const Device UA2160981 = {S92518, 52, { &P14033,&P5890,&P21746,&P18092 }};	/*Useragent - iBrowser/2.6 (NokiaC5-03;SymbianOS/9.1 Series60/3.0)*/
const Device UA2162929 = {S92519, 89, { &P13077,&P18110,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 (Linux; U; 2.3.6; zh-cn; HUAWEI C8650+ Build/C8650+V100R001C92B875;320*480)*/
const Device UA2328100 = {S92520, 93, { &P13077,&P18110,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 (Linux; U; 2.3.6; zh-cn; HUAWEI C8650+ Build/C8650+V100R001C92B866SP01;320*480)*/
const Device UA2168137 = {S92521, 81, { &P12236,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA6700s)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2192581 = {S92522, 64, { &P15364,&P18144,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64; rv:15.0) Gecko/X Firefox/15.0.1*/
const Device UA2807963 = {S92523, 149, { &P15364,&P7127,&P22512,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9.2.4) Gecko/X FireDownload/2.0.1 Firefox/3.6.4 Wyzo/3.6.4.1 FireTorrent/2.0.1 SearchToolbar/1.2*/
const Device UA2241249 = {S92524, 152, { &P18385,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03R; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2241862 = {S92525, 152, { &P18385,&P24701,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO54K; LiquidSmooth-Liquid) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2262909 = {S92526, 44, { &P11996,&P17484,&P17338,&P18092 }};	/*Useragent - samsung-gt-s5560i/UC Browser7.9.0.102/69/352*/
const Device UA2285654 = {S92527, 91, { &P15364,&P17832,&P3409,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; qcv; rv:1.9.0.5) Gecko/X Ubuntu/7.10 (gutsy) Firefox/3.0.5*/
const Device UA2302000 = {S92528, 69, { &P13999,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.7; zh-cn; HTC_HD2; 480*800) UCWEB7.9.3.103/139/800*/
const Device UA2777555 = {S92529, 68, { &P13999,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_HD2; 480*800) UCWEB7.9.0.94/139/352*/
const Device UA2844155 = {S92530, 68, { &P13999,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_HD2; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2543988 = {S92531, 189, { &P15602,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Iconia A501 Build/IMM76L; CyanogenMod-Tegraowners ICS ROM v170 (thor & digetx)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2383192 = {S92532, 138, { &P14856,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-SU660 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2402208 = {S92533, 139, { &P21719,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; htc_jewel Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2452337 = {S92534, 139, { &P21719,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; htc_jewel Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2403923 = {S92535, 120, { &P19976,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaE6-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile*/
const Device UA2410499 = {S92536, 226, { &P20027,&P22857,&P18373,&P18092 }};	/*Useragent - MicromaxX650 ASTRO36_TD/v3 MAUI/10A1032MP_ASTRO_W1052 Release/31.12.2010 Browser/Opera Sync/SyncClient1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.80 (MTK; Nucleus; Opera Mobi/4000; U; en-US) Presto/2.5.28 Version/10.10*/
const Device UA2413220 = {S92537, 139, { &P18113,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT912 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA2425810 = {S92538, 155, { &P18296,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC One V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2452566 = {S92539, 136, { &P20410,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MAGPAD9.7Pro Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2466342 = {S92540, 140, { &P20413,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; CELKON CT2 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2484092 = {S92541, 138, { &P20519,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TOUCH90W Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2504808 = {S92542, 137, { &P21804,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; P5-Mini Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2525552 = {S92543, 117, { &P21323,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ru-ru; V9S_V1.4) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2526667 = {S92544, 173, { &P11950,&P5477,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; X10i Build/2.1.A.0.435) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2529915 = {S92545, 135, { &P15365,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Magic Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2551952 = {S92546, 143, { &P22175,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; AraTop AS-09 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2562759 = {S92547, 138, { &P18507,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8160 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2578309 = {S92548, 145, { &P22059,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; ru-ru; Explay Star Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2597327 = {S92549, 152, { &P19859,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2598588 = {S92550, 158, { &P18283,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2598727 = {S92551, 174, { &P12657,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC EVO 3D X515m Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2599237 = {S92552, 162, { &P20438,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; N400 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2600686 = {S92553, 138, { &P15802,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Amaze 4G Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602038 = {S92554, 137, { &P20200,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS GAMEPAD Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2604121 = {S92555, 139, { &P12681,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus S 4G Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2618184 = {S92556, 147, { &P15876,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2620774 = {S92557, 147, { &P15876,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2640212 = {S92558, 80, { &P15364,&P16110,&P17282,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.7.12) Gecko/X Firefox/1.0.7*/
const Device UA2652661 = {S92559, 165, { &P20709,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID1042 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2661171 = {S92560, 71, { &P20004,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; Karbonn_A2; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2662598 = {S92561, 136, { &P23299,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; OV-BaseCore7 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2663802 = {S92562, 165, { &P21022,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LG-E615 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3044444 = {S92563, 87, { &P12707,&P5055,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokia701) UCBrowser8.3.1.161/70/352/UCWEB Mobile  3gpp-gba*/
const Device UA2694507 = {S92564, 165, { &P15060,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5660 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2990645 = {S92565, 138, { &P18557,&P20706,&P24670,&P18092 }};	/*Useragent - Nokia308/2.0 (05.85) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia308) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2706793 = {S92566, 145, { &P20259,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Starmobile ASTRA Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707307 = {S92567, 128, { &P20153,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AN7HG3 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2707326 = {S92568, 134, { &P21011,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; L-01E Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707417 = {S92569, 135, { &P21112,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; X9017 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2707810 = {S92570, 103, { &P16463,&P17350,&P24633,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 7.10; en-US; SAMSUNG; GT-I8350) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile*/
const Device UA2708615 = {S92571, 134, { &P18253,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT894 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2709084 = {S92572, 137, { &P18488,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-L300 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2870183 = {S92573, 99, { &P14010,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5263) U2/1.0.0 UCBrowser/9.0.0.295 U2/1.0.0 Mobile*/
const Device UA2714574 = {S92574, 137, { &P12496,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I897 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2727977 = {S92575, 19, { &P17968,&P17979,&P4861,&P18092 }};	/*Useragent - eBayAndroid/2.1.1.0*/
const Device UA2733632 = {S92576, 19, { &P17968,&P17979,&P4861,&P18092 }};	/*Useragent - eBayAndroid/2.1.3.1*/
const Device UA2729986 = {S92577, 124, { &P13157,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_Sensation_XE_with_Beats_Audio_Z715e) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2733906 = {S92578, 141, { &P16362,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/2.3;FBBV/X;FBDM/{density=1.5,width=480,height=800};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/SPH-D710;FBSV/4.0.4;]*/
const Device UA2865701 = {S92579, 141, { &P16362,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.1;FBBV/X;FBDM/{density=1.5,width=480,height=800};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/SPH-D710;FBSV/4.0.4;]*/
const Device UA2751218 = {S92580, 92, { &P12280,&P17090,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 4_3_3; zh-CN; iPh3,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile*/
const Device UA2766961 = {S92581, 138, { &P19865,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A850L Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2863510 = {S92582, 138, { &P19865,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A850K Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2781882 = {S92583, 148, { &P19860,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A9+ Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2781881 = {S92584, 93, { &P14294,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5238) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2782578 = {S92585, 136, { &P13074,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-P930 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2784107 = {S92586, 132, { &P20498,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP7100D3G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2796243 = {S92587, 125, { &P23868,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ST8 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2797638 = {S92588, 134, { &P12848,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MB526 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798450 = {S92589, 134, { &P22705,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IQ451 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799426 = {S92590, 155, { &P19784,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LIFETAB_P9514 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799807 = {S92591, 136, { &P20598,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Miracle Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2826768 = {S92592, 178, { &P18648,&P5043,&P24671,&P18092 }};	/*Useragent - Nokia306/2.0 (03.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia306) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA3070405 = {S92593, 57, { &P15722,&P5156,&P21731,&P18092 }};	/*Useragent - OneBrowser/for Blackberry3.0.0 (BlackBerry9300/6.0.0.668)*/
const Device UA2863117 = {S92594, 159, { &P18304,&P5629,&P22281,&P18092 }};	/*Useragent - OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3001166 = {S92595, 155, { &P19917,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-P765 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2866469 = {S92596, 26, { &P22594,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-M919/101.40202*/
const Device UA2869277 = {S92597, 156, { &P14959,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-I9103 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2877434 = {S92598, 152, { &P21112,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; X907 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2876321 = {S92599, 157, { &P18300,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T769 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2895959 = {S92600, 146, { &P20283,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AZ210A Build/AZ210A_ICS_02.34I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2999852 = {S92601, 101, { &P17968,&P5477,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; Android2.x) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2937609 = {S92602, 151, { &P18637,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9260 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3015107 = {S92603, 70, { &P17346,&P19826,&P24532,&P18092 }};	/*Useragent - Instagram 4.0.2 (iPod4,1; iPhone OS 6_1_3; en_US; en) AppleWebKit/420+*/
const Device UA2957611 = {S92604, 136, { &P20511,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; IQ442 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2958295 = {S92605, 137, { &P24148,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; s4502M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2961363 = {S92606, 154, { &P14967,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; LG-P500 Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2962042 = {S92607, 176, { &P12973,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2962345 = {S92608, 135, { &P23094,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; F-06E Build/V17R45B) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2962576 = {S92609, 97, { &P11747,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire V) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2963585 = {S92610, 107, { &P18510,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL ONE TOUCH 918N) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2963638 = {S92611, 161, { &P13452,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2964359 = {S92612, 90, { &P18302,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MT25i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2983665 = {S92613, 74, { &P17595,&P21721,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:17.0) Gecko/X Firefox/17.0*/
const Device UA2993512 = {S92614, 155, { &P24565,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; Mobiistar Touch Bean 452 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997895 = {S92615, 50, { &P17595,&P18120,&P22879,&P18092 }};	/*Useragent - MQQBrowser/43 (iOS 5; U; CPU like Mac OS X; zh-cn)*/
const Device UA2999097 = {S92616, 51, { &P17595,&P18120,&P22879,&P18092 }};	/*Useragent - MQQBrowser/43 (iOS 4S; U; CPU like Mac OS X; zh-cn)*/
const Device UA3003437 = {S92617, 132, { &P22597,&P18598,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.2; en-us; SCH-I545 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3004364 = {S92618, 155, { &P20470,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; MID9742 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3007668 = {S92619, 137, { &P20937,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ADR910L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA3010953 = {S92620, 146, { &P21302,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; ru-ru; Fly IQ4410 Quad Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3014950 = {S92621, 155, { &P18699,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; MID8042 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3016606 = {S92622, 140, { &P22582,&P18598,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3018300 = {S92623, 92, { &P13445,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN85) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3020293 = {S92624, 163, { &P18556,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nook HD+ Build/JDQ39E; Carbon-CARBON-JB-UNOFFICIAL-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3020693 = {S92625, 88, { &P20120,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A45) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3020723 = {S92626, 136, { &P24528,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E987 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021791 = {S92627, 139, { &P18696,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LRA Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022968 = {S92628, 153, { &P21140,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Optimus_Madrid Build/IceCreamSandwich) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3108605 = {S92629, 162, { &P15809,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HTC Rhyme S510b Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3083240 = {S92630, 155, { &P14574,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Nexus S Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3028063 = {S92631, 114, { &P15364,&P16110,&P21934,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 slurp*/
const Device UA3029925 = {S92632, 164, { &P12987,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; SPH-M930BST Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3058922 = {S92633, 149, { &P23955,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5210 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3060352 = {S92634, 177, { &P12059,&P5204,&P24055,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311*/
const Device UA3061552 = {S92635, 140, { &P12788,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Evo 3D CDMA Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3077321 = {S92636, 139, { &P24718,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PAP5300DUO Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3078449 = {S92637, 105, { &P12871,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC Wildfire S A510e) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3108395 = {S92638, 105, { &P12871,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC Wildfire S A510e) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3078485 = {S92639, 151, { &P24533,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ph; Cloudfone Excite 501d Build/SKV10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108786 = {S92640, 156, { &P22980,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; TECNO D5 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3090239 = {S92641, 172, { &P15364,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Connect-2G-2.0 Build/HCL+ME+Tablet+V2) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3121345 = {S92642, 152, { &P15364,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; W125 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3092640 = {S92643, 104, { &P11868,&P5768,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6280) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile TRUSTED/1.0*/
const Device UA3092985 = {S92644, 160, { &P18449,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST27i Build/6.1.1.B.1.10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3093265 = {S92645, 93, { &P11880,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA5320) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3093414 = {S92646, 164, { &P22584,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I337M Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltecan*/
const Device UA3125209 = {S92647, 164, { &P22584,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I337M Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/jfltecan*/
const Device UA3109026 = {S92648, 134, { &P15365,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Magic Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110570 = {S92649, 142, { &P22627,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; IRIS 455 Build/LAVAIRIS455) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127015 = {S92650, 134, { &P18505,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; MB886 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110554 = {S92651, 93, { &P21261,&P18598,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.2; zh-CN; GT-I9500) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3115050 = {S92652, 156, { &P19847,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S6012 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3115607 = {S92653, 159, { &P15989,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST21i Build/11.0.A.4.22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3131759 = {S92654, 132, { &P23187,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; V370 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132925 = {S92655, 135, { &P18468,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; IdeaTabA2109A Build/JRO03R) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA21753 = {S92656, 97, { &P11997,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-C417 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA62988 = {S92657, 107, { &P11997,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-C417/C417UCFK3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA4996 = {S92658, 101, { &P13428,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E830/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA26342 = {S92659, 57, { &P13585,&P5768,&P3625,&P18092 }};	/*Useragent - Nokia6275/2.0 (BL290V0100.nep) UP.Browser/6.2.3.8 MMP/2.0*/
const Device UA20301 = {S92660, 54, { &P15062,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-V3s/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0*/
const Device UA21245 = {S92661, 68, { &P16213,&P5448,&P17173,&P18092 }};	/*Useragent - Xda orbit; 240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA21747 = {S92662, 43, { &P15334,&P17484,&P4203,&P18092 }};	/*Useragent - Samsung-SPHM510 AU-OBIGO/Q04C1-1.22 MMP/2.0*/
const Device UA29863 = {S92663, 86, { &P12274,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-SGH-i900/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA31981 = {S92664, 72, { &P12448,&P5208,&P4030,&P18092 }};	/*Useragent - Mozilla/4.1 (compatible; MSIE 6.0; Brew 3.1.5; U; en)/320x240 NOKIA 7705*/
const Device UA43617 = {S92665, 68, { &P13709,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHD520/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA58591 = {S92666, 119, { &P15028,&P17979,&P3508,&P18092 }};	/*Useragent - LG_P350/1.0 Android/2.2 Release/5.20.2010 Browser/KHTML (Mozilla/5.0 AppleWebKit/533.1 Version/4.0 Mobile Safari/533.1)*/
const Device UA60733 = {S92667, 84, { &P16378,&P17603,&P3534,&P18092 }};	/*Useragent - PANTECH-C610/IRUS08252008 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA1885286 = {S92668, 77, { &P16378,&P17603,&P3534,&P18092 }};	/*Useragent - PANTECH-C610/IRUSX Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61555 = {S92669, 79, { &P12753,&P17484,&P4970,&P18092 }};	/*Useragent - Samsung-SPHM500 AU-MIC-M500/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA644821 = {S92670, 31, { &P14766,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F04B(c500;TB;W24H16)*/
const Device UA69244 = {S92671, 36, { &P14033,&P5129,&P4779,&P18092 }};	/*Useragent - NokiaC5-03/UC Browser7.8.0.95/50/351*/
const Device UA250135 = {S92672, 122, { &P15364,&P17285,&P22410,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.0.1.0 Chrome/14.0.835.163 Safari/535.1*/
const Device UA1902277 = {S92673, 129, { &P15364,&P17285,&P22410,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/14.0.1.0 Chrome/14.0.835.163 Safari/535.1*/
const Device UA334454 = {S92674, 122, { &P15364,&P17285,&P22407,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.0.0.0 Chrome/13.0.782.107 Safari/535.1*/
const Device UA566224 = {S92675, 129, { &P15364,&P17285,&P22407,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.0.0.0 Chrome/13.0.782.107 Safari/535.1*/
const Device UA513489 = {S92676, 89, { &P15364,&P7133,&P20902,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.7.2) Gecko/X Netscape/7.2 (ax;nscd1)*/
const Device UA687493 = {S92677, 99, { &P14635,&P5843,&P4777,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (08.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac3-00/UC Browser7.7.1.88/69/444*/
const Device UA555806 = {S92678, 46, { &P13324,&P5055,&P4742,&P18092 }};	/*Useragent - nokian8-00/UC Browser7.5.1.77/69/351  3gpp-gba*/
const Device UA606851 = {S92679, 128, { &P17968,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029233 = {S92680, 171, { &P13974,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2987675 = {S92681, 175, { &P13974,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; GT-S5670 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA657982 = {S92682, 50, { &P17346,&P18117,&P17470,&P18092 }};	/*Useragent - Facebook 4110.0 (iPod touch; iPhone OS 5.1; en_US)*/
const Device UA882055 = {S92683, 52, { &P17346,&P18117,&P17470,&P18092 }};	/*Useragent - Facebook 4110.0 (iPod touch; iPhone OS 5.1.1; en_US)*/
const Device UA1997064 = {S92684, 69, { &P17346,&P18117,&P17470,&P18092 }};	/*Useragent - Instagram 2.5.1 (iPod touch; iPhone OS 5.1.1; en_US) AppleWebKit/420+*/
const Device UA1975856 = {S92685, 149, { &P12280,&P17146,&P21786,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) 1Password/3.6.5/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA808991 = {S92686, 96, { &P15364,&P17814,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.3) Gecko/X Ubuntu/8.10 (intrepid) Firefox/3.6.3*/
const Device UA866543 = {S92687, 79, { &P15364,&P7127,&P23681,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9a1) Gecko/X Firefox/1.6a1*/
const Device UA901838 = {S92688, 34, { &P23144,&P17779,&P23079,&P18092 }};	/*Useragent - Instacast/2.1 (like iTunes/10.1.2)*/
const Device UA2282588 = {S92689, 62, { &P15364,&P17188,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 4.0; U; de) Presto/2.7.62 Version/11.00*/
const Device UA1869161 = {S92690, 118, { &P15364,&P17188,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 4.0; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/1.0.154.53 Safari/525.19*/
const Device UA2299868 = {S92691, 116, { &P12473,&P5204,&P17437,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2730c-1/09.41; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.4.0.34.8*/
const Device UA1942804 = {S92692, 94, { &P16373,&P17342,&P3834,&P18092 }};	/*Useragent - SAMSUNG-GT-S7250/1.0 Bada/2.0 Dolfin/2.0 SMM-MMS/1.2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA1963174 = {S92693, 131, { &P14903,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; ko-kr; LG-V901 Build/HTK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2019329 = {S92694, 136, { &P22469,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.0; es-us; E700R Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2020955 = {S92695, 80, { &P16290,&P17979,&P3508,&P18092 }};	/*Useragent - HUAWEIU8520/1.0 Linux/2.6.35 Android/2.2.2 Release/5.26.2011 Browser/WebKit533.1*/
const Device UA2145658 = {S92696, 39, { &P12004,&P5890,&P17338,&P18092 }};	/*Useragent - nokian95_8gb/UC Browser7.9.0.102/69/352*/
const Device UA2303432 = {S92697, 35, { &P12004,&P5890,&P17338,&P18092 }};	/*Useragent - nokian95/UC Browser7.9.0.102/70/352*/
const Device UA2346283 = {S92698, 180, { &P16092,&P5629,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (DROID X2, 34, 1.6.0)*/
const Device UA2368006 = {S92699, 39, { &P18176,&P5448,&P20764,&P18092 }};	/*Useragent - NOKIA-Lumia 900/UCWEB2.6.0.213/47/32079*/
const Device UA2371888 = {S92700, 81, { &P19976,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-in; NokiaE6-00)/UC Browser8.2.0.132/50/444/UCWEB Mobile*/
const Device UA2376021 = {S92701, 128, { &P22108,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HS-T96 Build/IML74K) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2383388 = {S92702, 177, { &P15201,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Sensation Z710e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2829946 = {S92703, 174, { &P15201,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Sensation 4G Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2384362 = {S92704, 249, { &P15803,&P5843,&P18516,&P18092 }};	/*Useragent - Nokia200/2.0 (11.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.1.161/69/355*/
const Device UA2408970 = {S92705, 81, { &P15364,&P18110,&P21958,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2a1pre) Gecko/X Minefield/3.6a1pre*/
const Device UA2413744 = {S92706, 115, { &P14693,&P5843,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC1-02/03.40; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.5.16.11*/
const Device UA2413830 = {S92707, 130, { &P21238,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-nz; VIGO10 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2421965 = {S92708, 142, { &P11793,&P5768,&P24670,&P18092 }};	/*Useragent - Nokia3500c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia3500c) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2424641 = {S92709, 163, { &P16595,&P5477,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; Micromax A60 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2454242 = {S92710, 132, { &P23942,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JMI T970 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2459377 = {S92711, 141, { &P18224,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Ascend G300 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2695877 = {S92712, 122, { &P13443,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; SamsungI8910) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2485005 = {S92713, 168, { &P14993,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT15i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2485092 = {S92714, 92, { &P16871,&P22784,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; micromaxq5) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2485541 = {S92715, 90, { &P23351,&P22784,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; micromax) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2500482 = {S92716, 157, { &P15575,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2824637 = {S92717, 157, { &P15575,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2504692 = {S92718, 67, { &P13964,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2; zh-cn; GT-I5510; 240*400) UCWEB7.9.0.94/139/355*/
const Device UA2505861 = {S92719, 125, { &P20464,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N-06D Build/AX) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2507559 = {S92720, 138, { &P12841,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Vivid 4G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2536650 = {S92721, 139, { &P22162,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AUXUS CoreX4 3G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2536889 = {S92722, 151, { &P22875,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9220 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2554285 = {S92723, 144, { &P22125,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; INFINITY_LOTUS Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2583788 = {S92724, 139, { &P18222,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IS12S Build/6.1.D.1.91) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2600190 = {S92725, 136, { &P21985,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nb-no; 726EP Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2600478 = {S92726, 165, { &P14574,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; Nexus S Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2602006 = {S92727, 146, { &P20000,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ607 Build/7.7.1-122-4-FLEM-WIFI-LA) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2603901 = {S92728, 154, { &P21140,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Optimus_Madrid Build/IceCreamSandwich) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2623048 = {S92729, 186, { &P12879,&P6437,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.16/276,1.9.16.276,2012-12-21 16:37, +0000*/
const Device UA2635329 = {S92730, 164, { &P22043,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P5113, 42, 1.9.1)*/
const Device UA2625651 = {S92731, 150, { &P15985,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P6200 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2712730 = {S92732, 129, { &P12889,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7300 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2652357 = {S92733, 139, { &P15397,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ST18 Build/4.1.B.0.587) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2833721 = {S92734, 140, { &P15397,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xperia Ray Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2659470 = {S92735, 147, { &P20868,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; M040 Build/JRO03H) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2767752 = {S92736, 160, { &P13054,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2860737 = {S92737, 161, { &P18304,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2666985 = {S92738, 143, { &P21466,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Titan TV S320 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667484 = {S92739, 136, { &P23889,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PAD703 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2676836 = {S92740, 140, { &P19846,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SO-05D Build/7.0.D.1.137) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2677693 = {S92741, 153, { &P21140,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Optimus_Madrid Build/IceCreamSandwich) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2697561 = {S92742, 139, { &P23381,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A5Classic Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2698928 = {S92743, 166, { &P12972,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; SGH-T989 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2706022 = {S92744, 166, { &P18125,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2707385 = {S92745, 142, { &P21885,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GSmart GS202+ Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708512 = {S92746, 137, { &P22646,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Dream_D1 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713330 = {S92747, 133, { &P23146,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HYUNDAI T7S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2937484 = {S92748, 142, { &P22655,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5503 Build/10.1.1.A.1.310) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2733424 = {S92749, 135, { &P23946,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; I7100 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2741563 = {S92750, 136, { &P22929,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_E10310 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2748494 = {S92751, 150, { &P23538,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; NetTAB SPACE III Build/NetTAB SPACE III) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2765358 = {S92752, 89, { &P19936,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A100) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2767240 = {S92753, 137, { &P18316,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-F160K Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2992311 = {S92754, 137, { &P18316,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F160L Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2767894 = {S92755, 151, { &P20983,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9308 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2783450 = {S92756, 135, { &P22927,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_E7310 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2783696 = {S92757, 134, { &P24549,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; E2001 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2794809 = {S92758, 96, { &P21991,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Aqua_Wonder) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2795701 = {S92759, 159, { &P23919,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; R800i Build/4.0.2.A.0.42) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2797333 = {S92760, 129, { &P17323,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-p6800 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2799230 = {S92761, 156, { &P20226,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799708 = {S92762, 138, { &P20926,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT-AS40SE Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2827413 = {S92763, 129, { &P21029,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KUNO3 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2861050 = {S92764, 137, { &P18686,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A810S Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2994006 = {S92765, 137, { &P18686,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A810K Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2864649 = {S92766, 165, { &P19863,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2864831 = {S92767, 90, { &P12798,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT18i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2893046 = {S92768, 152, { &P20934,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; 5860E Build/GWK74) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2866576 = {S92769, 166, { &P12731,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2867208 = {S92770, 145, { &P15555,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; myTouch_4G_Slide Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868512 = {S92771, 125, { &P24510,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N10 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2869318 = {S92772, 161, { &P11747,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire V Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2876103 = {S92773, 134, { &P23383,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; H7100 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2876807 = {S92774, 161, { &P19898,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; SCH-I699 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2877368 = {S92775, 138, { &P23561,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; AMOI N828 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877879 = {S92776, 137, { &P12482,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T959 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937375 = {S92777, 152, { &P21240,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile i-STYLE Q2 DUO Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3000615 = {S92778, 148, { &P21240,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile i-STYLE Q2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2941576 = {S92779, 137, { &P21936,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; ThL W6 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2950617 = {S92780, 138, { &P22838,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; hr-hr; MOVE Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2955579 = {S92781, 145, { &P17174,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MediaPad 10 FHD Build/HuaweiMediaPad) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2956151 = {S92782, 57, { &P15364,&P17350,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows) Gecko/X Firefox/17.05 -CareMediCom-*/
const Device UA3125512 = {S92783, 57, { &P15364,&P17350,&P18429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows) Gecko/X Firefox/17.08 -CareMediCom-*/
const Device UA2957040 = {S92784, 97, { &P23017,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Spice_Mi-502) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2958187 = {S92785, 143, { &P19907,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C5155 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2958260 = {S92786, 143, { &P23712,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; i-mobile IQ XA Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2958403 = {S92787, 139, { &P20878,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; MIDC901 Build/MIDC901) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2959673 = {S92788, 152, { &P22385,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2960671 = {S92789, 160, { &P23008,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax A91 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2961825 = {S92790, 88, { &P20231,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A21) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2962874 = {S92791, 157, { &P22756,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; C1604 Build/11.3.A.2.1) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2997511 = {S92792, 159, { &P16319,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; U8860 Build/HuaweiU8860) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453*/
const Device UA2997935 = {S92793, 148, { &P18296,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC One V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3000140 = {S92794, 148, { &P19845,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ZTE N9120 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3000726 = {S92795, 156, { &P18002,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; IM-A760S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3009015 = {S92796, 137, { &P18382,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.3; en-in; HTC Desire C Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3013145 = {S92797, 151, { &P20158,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A7* Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3017135 = {S92798, 165, { &P19862,&P18598,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-in; Micromax A110Q Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300*/
const Device UA3022014 = {S92799, 138, { &P23381,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A5Classic Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023210 = {S92800, 150, { &P19792,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Desire U dual sim Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023267 = {S92801, 142, { &P22627,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; IRIS 455 Build/LAVAIRIS455) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3027887 = {S92802, 135, { &P22594,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SGH-M919 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3047558 = {S92803, 167, { &P23248,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; Gpad_G2 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33886*/
const Device UA3049883 = {S92804, 146, { &P18021,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A770K Build/->Wangsai_008) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3086469 = {S92805, 139, { &P18021,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A770K Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058310 = {S92806, 155, { &P20202,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-L38C Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3058501 = {S92807, 143, { &P22585,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.5;FBBV/X;FBDM/{density=3.0,width=1080,height=1920};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/SPH-L720;FBSV/4.2.2;]*/
const Device UA3126048 = {S92808, 170, { &P22585,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.6.1;FBBV/X;FBDM/{density=3.0,width=1080,height=1920};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/SPH-L720;FBSV/4.2.2;FBCA/armeabi-v7a:armeabi;]*/
const Device UA3060187 = {S92809, 148, { &P18471,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.5;FBBV/X;FBDM/{density=1.5,width=540,height=888};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/XT907;FBSV/4.1.2;]*/
const Device UA3126539 = {S92810, 175, { &P18471,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.6.1;FBBV/X;FBDM/{density=1.5,width=540,height=888};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/XT907;FBSV/4.1.2;FBCA/armeabi-v7a:armeabi;]*/
const Device UA3060434 = {S92811, 148, { &P23702,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ME173X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3074219 = {S92812, 138, { &P22496,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; hu-hu; xTAB-100dci-3G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3125763 = {S92813, 135, { &P22496,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; DTR97 Build/xTAB-100 QCR) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3095255 = {S92814, 137, { &P15802,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Amaze 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3102149 = {S92815, 140, { &P18613,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC_T329w Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3120362 = {S92816, 140, { &P18613,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC T329w Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108566 = {S92817, 155, { &P20587,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-P715 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109570 = {S92818, 98, { &P19862,&P18111,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Micromax A110) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3129561 = {S92819, 91, { &P23522,&P20706,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia501) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3130744 = {S92820, 129, { &P24365,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SM-T211 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3131317 = {S92821, 136, { &P21794,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9152 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA8034 = {S92822, 73, { &P11716,&P5448,&P22303,&P18092 }};	/*Useragent - HTC-8100/1.2 Mozilla/4.0 (compatible; MSIE 5.5; Windows CE; PPC; 240x320)*/
const Device UA10323 = {S92823, 85, { &P13494,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) SAMSUNG-SGH-i640V/BUGI4*/
const Device UA38682 = {S92824, 85, { &P13494,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) SAMSUNG-SGH-i640V/BUGK1*/
const Device UA52453 = {S92825, 85, { &P13494,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) SAMSUNG-SGH-i640V/BUGJ2*/
const Device UA11643 = {S92826, 125, { &P14949,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; MDA_compact_IV/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA16158 = {S92827, 85, { &P15468,&P17485,&P3990,&P18092 }};	/*Useragent - SonyEricssonZ800/R1X Browser/SEMC-Browser/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22434 = {S92828, 86, { &P15468,&P17485,&P3990,&P18092 }};	/*Useragent - SonyEricssonZ800i/R1X Browser/SEMC-Browser/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA50132 = {S92829, 68, { &P11825,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-E398/0E.20.99R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA18664 = {S92830, 61, { &P15001,&P6712,&P4030,&P18092 }};	/*Useragent - Nokia5611/2.0 (09.70) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA20267 = {S92831, 142, { &P15288,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-C3110C/1.0 Release/9.05.2008 Browser/Openwave6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA21502 = {S92832, 113, { &P15288,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-C3110/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA90355 = {S92833, 101, { &P15288,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-C3110C/1.0 Release/9.05.2008 Browser/Openwave6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21500 = {S92834, 54, { &P15287,&P17017,&P3402,&P18092 }};	/*Useragent - UTSTARCOM-CDM7126/1.0 UP.Browser/6.3.0.7 (GUI) MMP/2.0*/
const Device UA21908 = {S92835, 41, { &P11698,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW995/R1HA SEMC/NewsReader/1.0*/
const Device UA24676 = {S92836, 12, { &P15734,&P17483,&P17660,&P18092 }};	/*Useragent - LG/U960/v1.0*/
const Device UA39041 = {S92837, 136, { &P11908,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-SGH-i908/1.0 Release/03.07 Browser/IE6 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA42701 = {S92838, 42, { &P17508,&P17662,&P3601,&P18092 }};	/*Useragent - Mozilla/4.08 (821SC;SoftBank) NetFront/3.3*/
const Device UA58638 = {S92839, 86, { &P17508,&P17662,&P3601,&P18092 }};	/*Useragent - SoftBank/1.0/821SC/SCJ001 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA69478 = {S92840, 36, { &P14748,&P5890,&P4779,&P18092 }};	/*Useragent - NokiaE72-1/UC Browser7.8.0.95/28/355*/
const Device UA394377 = {S92841, 54, { &P17543,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC3R UP.Browser/6.2_7.2.7.1.K.4.310 (GUI) MMP/2.0*/
const Device UA124205 = {S92842, 49, { &P17888,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/H999/MIDP2.0/CLDC1.1/Screen-320X240*/
const Device UA242910 = {S92843, 189, { &P15364,&P7133,&P17338,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/69/352*/
const Device UA783083 = {S92844, 189, { &P15364,&P7133,&P17338,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/70/352*/
const Device UA2587466 = {S92845, 199, { &P15364,&P7133,&P17338,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.9.0.102/69/352  3gpp-gba*/
const Device UA304113 = {S92846, 118, { &P15364,&P7127,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13*/
const Device UA344396 = {S92847, 225, { &P12280,&P17146,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA649150 = {S92848, 227, { &P12280,&P17146,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA2313371 = {S92849, 227, { &P12280,&P17146,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA433280 = {S92850, 71, { &P17595,&P23100,&P17317,&P18092 }};	/*Useragent - Safari/6533.19.4 CFNetwork/454.11.5 Darwin/10.6.0 (i386) (MacBook7%2C1)*/
const Device UA751047 = {S92851, 74, { &P17595,&P23100,&P17317,&P18092 }};	/*Useragent - Safari/6533.19.4 CFNetwork/454.11.5 Darwin/10.6.0 (i386) (MacBookPro7%2C1)*/
const Device UA869278 = {S92852, 69, { &P17595,&P23100,&P17317,&P18092 }};	/*Useragent - Safari/6533.19.4 CFNetwork/454.11.5 Darwin/10.6.0 (i386) (iMac11%2C2)*/
const Device UA1862984 = {S92853, 74, { &P17595,&P23100,&P17317,&P18092 }};	/*Useragent - Safari/6533.19.4 CFNetwork/454.11.5 Darwin/10.6.0 (i386) (MacBookAir3%2C1)*/
const Device UA450350 = {S92854, 88, { &P12194,&P5448,&P4777,&P18092 }};	/*Useragent - HTC_Touch2_T3320 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA452657 = {S92855, 42, { &P14984,&P5129,&P24663,&P18092 }};	/*Useragent - Nokia5250/UC Browser7.9.0.102/50/355/UCWEB*/
const Device UA526411 = {S92856, 30, { &P15364,&P6704,&P4811,&P18092 }};	/*Useragent - Opera/9.00 (Windows 98; U; en)*/
const Device UA3069278 = {S92857, 30, { &P15364,&P6704,&P4811,&P18092 }};	/*Useragent - Opera/9.02 (Windows 98; U; en)*/
const Device UA576094 = {S92858, 67, { &P15364,&P17285,&P18368,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:8.0) Gecko/X Thunderbird/8.0*/
const Device UA593895 = {S92859, 133, { &P17968,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA2749439 = {S92860, 128, { &P17968,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; MID Build/MID) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7*/
const Device UA710417 = {S92861, 99, { &P15228,&P5043,&P4777,&P18092 }};	/*Useragent - NokiaC2-03/2.0 (06.51) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-03/UC Browser7.7.1.88/69/444*/
const Device UA1935800 = {S92862, 108, { &P19968,&P17485,&P17347,&P18092 }};	/*Useragent - SonyEricssonW910i/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonw910i/UC Browser8.0.3.107/69/352*/
const Device UA727394 = {S92863, 106, { &P12502,&P17485,&P17347,&P18092 }};	/*Useragent - SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonw20i/UC Browser8.0.3.107/69/444*/
const Device UA769311 = {S92864, 52, { &P15364,&P17017,&P18100,&P18092 }};	/*Useragent - Mozilla/5.0 (SaferSurf rv:11.0) Gecko/X Firefox/11.0*/
const Device UA2803290 = {S92865, 63, { &P15364,&P6704,&P20418,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows 98) Opera 7.50 [ru"]*/
const Device UA892900 = {S92866, 146, { &P16356,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-N7000 Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892636 = {S92867, 147, { &P16356,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA1925700 = {S92868, 92, { &P17976,&P6545,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.0.1; Linux; Opera Tablet/ADR-X; U; en-GB) Presto/2.9.201 Version/11.50*/
const Device UA1936695 = {S92869, 100, { &P13107,&P5043,&P17347,&P18092 }};	/*Useragent - NokiaC2-06/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-06/UC Browser8.0.3.107/69/444*/
const Device UA1940903 = {S92870, 96, { &P12989,&P5843,&P17347,&P18092 }};	/*Useragent - Nokia303/5.0 (13.44) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia303/UC Browser8.0.3.107/70/355*/
const Device UA1963979 = {S92871, 171, { &P14877,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2029307 = {S92872, 171, { &P14877,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2811015 = {S92873, 78, { &P15209,&P5768,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokia6233) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA1977876 = {S92874, 146, { &P18283,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957722 = {S92875, 147, { &P18283,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA1991076 = {S92876, 169, { &P15147,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; ST15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA1991241 = {S92877, 155, { &P18280,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC One S Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030027 = {S92878, 159, { &P18280,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC One S Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2440152 = {S92879, 67, { &P15517,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.4;Zh_cn;MT15i;480*854;)UCWEB7.8.0.95/139/444*/
const Device UA2026247 = {S92880, 154, { &P12945,&P5629,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A780 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030009 = {S92881, 137, { &P21948,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; pl-pl; MID07 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2166157 = {S92882, 86, { &P12351,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s3653w) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2232875 = {S92883, 39, { &P15795,&P5448,&P20762,&P18092 }};	/*Useragent - NOKIA-Lumia 710/UCWEB2.5.0.202/47/32079*/
const Device UA2258512 = {S92884, 110, { &P17595,&P18120,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5*/
const Device UA2343809 = {S92885, 37, { &P12897,&P5843,&P23405,&P18092 }};	/*Useragent - MQQBrowser/Mini2.8 (NokiaC2-00/03.82)*/
const Device UA2353319 = {S92886, 165, { &P21616,&P18066,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-style Q3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2362258 = {S92887, 91, { &P16003,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5233a) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2391547 = {S92888, 91, { &P16003,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5233a) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2423654 = {S92889, 88, { &P16003,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; fa; gt-s5233a) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2527361 = {S92890, 88, { &P16003,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; fa; gt-s5233a) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2398686 = {S92891, 157, { &P18214,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2406748 = {S92892, 155, { &P18464,&P18066,&P19803,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2418081 = {S92893, 137, { &P18678,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SBM102SH Build/S1010) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2421272 = {S92894, 53, { &P14188,&P5890,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2 (Nokia5233;SymbianOS/9.1 Series60/3.0)*/
const Device UA2423407 = {S92895, 127, { &P12375,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N81 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2436062 = {S92896, 88, { &P18088,&P17017,&P18526,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/6.7.29879/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2681537 = {S92897, 46, { &P13324,&P5055,&P4737,&P18092 }};	/*Useragent - nokian8-00/UC Browser7.2.2.51/70/352  3gpp-gba*/
const Device UA2444668 = {S92898, 78, { &P20206,&P18110,&P20776,&P18092 }};	/*Useragent - MQQBrowser/25 (Linux; U; 2.3.6; zh-cn; ZTE N880E Build/N880EV1.0.0B06;480*800)*/
const Device UA2837443 = {S92899, 172, { &P14090,&P5477,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; GT-I5801 Build/ECLAIR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2460505 = {S92900, 37, { &P18176,&P5448,&P20765,&P18092 }};	/*Useragent - NOKIA-Lumia 900/UCWEB2.7.0.223/47/355*/
const Device UA2486512 = {S92901, 79, { &P18254,&P18118,&P18429,&P18091 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:17.0) Gecko/X Firefox/17.0 AdometryBot*/
const Device UA2493381 = {S92902, 78, { &P17595,&P17295,&P18374,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; PPC Mac OS X 10.5.8; U; en) Presto/2.6.31 Version/10.70*/
const Device UA2504746 = {S92903, 94, { &P16620,&P22784,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxx510) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2504826 = {S92904, 135, { &P18442,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N7100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2507081 = {S92905, 155, { &P15837,&P6909,&P24675,&P18092 }};	/*Useragent - NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiax2-05) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2527804 = {S92906, 135, { &P20352,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M-270 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2528527 = {S92907, 139, { &P21304,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Fly_IQ450 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2554160 = {S92908, 131, { &P14714,&P5843,&P17436,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02/leCEX92_MP2.2.1_05.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.3.23.12*/
const Device UA2657558 = {S92909, 170, { &P18462,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC Desire X Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2555510 = {S92910, 144, { &P21618,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 5A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2583319 = {S92911, 138, { &P17324,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9001 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2591883 = {S92912, 135, { &P20462,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PTL21 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2594856 = {S92913, 131, { &P17346,&P17146,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25*/
const Device UA2790276 = {S92914, 131, { &P17346,&P17146,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A403 Safari/8536.25*/
const Device UA2596173 = {S92915, 135, { &P23758,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i9430 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2602397 = {S92916, 150, { &P20137,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Xtreamer_Mobile_AiKi Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602915 = {S92917, 69, { &P15364,&P16110,&P21711,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:20.0) Gecko/X Thunderbird/20.0*/
const Device UA2603993 = {S92918, 137, { &P21085,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; J-Q88B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2607680 = {S92919, 28, { &P12879,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (GT-P7510 HMJ37)*/
const Device UA2622187 = {S92920, 217, { &P17323,&P6437,&P17373,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Opera/9.80 (Android; Opera Mini/6.5.27431/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2622749 = {S92921, 217, { &P12889,&P6437,&P17373,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Opera/9.80 (Android; Opera Mini/6.5.27431/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2658308 = {S92922, 131, { &P22680,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; RMD-755 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2659249 = {S92923, 147, { &P21112,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; X907 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2661676 = {S92924, 169, { &P18449,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST27i Build/6.1.1.B.1.10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2666718 = {S92925, 131, { &P21480,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PMP3270B Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667474 = {S92926, 145, { &P15809,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Rhyme S510b Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668238 = {S92927, 161, { &P18304,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9070 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2668990 = {S92928, 132, { &P21030,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAC-97032 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2680972 = {S92929, 152, { &P22385,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2686400 = {S92930, 132, { &P23451,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; T720 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2696020 = {S92931, 158, { &P15575,&P5629,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2844716 = {S92932, 157, { &P15575,&P5629,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2696145 = {S92933, 129, { &P23885,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; N70-S Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2701468 = {S92934, 162, { &P16598,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A500 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2706666 = {S92935, 138, { &P14225,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Desire HD Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708426 = {S92936, 135, { &P22677,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ThL W3 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2711181 = {S92937, 114, { &P15364,&P16110,&P19839,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 slurp*/
const Device UA2711813 = {S92938, 133, { &P21226,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; E860 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2711922 = {S92939, 137, { &P20173,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Limo X2+ Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2716738 = {S92940, 69, { &P15364,&P16110,&P22226,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:21.0) Gecko/X Thunderbird/21.0*/
const Device UA2727640 = {S92941, 34, { &P12077,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SAMSUNG GT-S3600i)*/
const Device UA2730190 = {S92942, 124, { &P15794,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_Sensation_XL_with_Beats_Audio_X315e) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2732681 = {S92943, 135, { &P22635,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; KATBL10A16E Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2734837 = {S92944, 135, { &P18278,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; F-05D Build/V08R31C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735544 = {S92945, 125, { &P18248,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; A700 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2736774 = {S92946, 90, { &P22756,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; C1604) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2748005 = {S92947, 147, { &P23342,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; Funtab Build/ICS.g12refM806A1SPISY.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2749267 = {S92948, 96, { &P13094,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Lenovo_A60+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2749411 = {S92949, 152, { &P15767,&P18432,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A523 Safari/8536.25 XiaoMi/MiuiBrowser/1.0*/
const Device UA2759092 = {S92950, 129, { &P23419,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; ru-ru; TM-7025 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2789791 = {S92951, 131, { &P23419,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; TM-7025 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2767408 = {S92952, 156, { &P14959,&P5629,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2778888 = {S92953, 169, { &P17108,&P22211,&P19802,&P18091 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; HTC Inspire 4G Build/JOP40D; CyanogenMod-BlueBot-V3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2780952 = {S92954, 140, { &P12280,&P18432,&P24445,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A525 Safari/7534.48.3 Janetter/1.5.1*/
const Device UA2937507 = {S92955, 138, { &P12280,&P18432,&P24445,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A405 Safari/7534.48.3 Janetter/1.5.2*/
const Device UA2784253 = {S92956, 129, { &P23767,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-7240 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2790712 = {S92957, 153, { &P22986,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; GT-5660 Build/GRWK74; CyanogenMod-7.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2793602 = {S92958, 94, { &P23156,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; XOLO_A700) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2793827 = {S92959, 152, { &P18314,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M032 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2797230 = {S92960, 128, { &P20153,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AN7HG3 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2797583 = {S92961, 138, { &P23447,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Newman N2 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798393 = {S92962, 130, { &P16168,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; NookColor Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2799032 = {S92963, 155, { &P18302,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; MT25i Build/4.1.B.0.631) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2799041 = {S92964, 129, { &P20752,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AN10CG3 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2800286 = {S92965, 141, { &P23408,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 97B TITANIUM Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2834172 = {S92966, 113, { &P15364,&P17017,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; j2me) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/352/UCWEB Mobile*/
const Device UA2837699 = {S92967, 114, { &P15364,&P17017,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; spice) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/352/UCWEB Mobile*/
const Device UA2834786 = {S92968, 141, { &P24572,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GPLUS N930 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3043178 = {S92969, 63, { &P24572,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GPLUS N930 Build/IMM76D)*/
const Device UA2849656 = {S92970, 170, { &P12280,&P18117,&P23594,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko, M5WebKit/1.0.1, M5HybridSDK/1.0.0) Version/5.1.0 Mobile/5.1.1 Mammoth/5.1.0*/
const Device UA3073214 = {S92971, 170, { &P12280,&P18117,&P23594,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko, M5WebKit/1.0.2, M5HybridSDK/1.0.0) Version/5.1.1 Mobile/5.1.1 Mammoth/5.1.1*/
const Device UA2851780 = {S92972, 162, { &P11944,&P17485,&P24676,&P18092 }};	/*Useragent - SonyEricssonT715/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssont715) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2854149 = {S92973, 149, { &P13123,&P18598,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-S5360 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2958581 = {S92974, 155, { &P15802,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; HTC Ruby Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2865853 = {S92975, 161, { &P14031,&P5477,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; GT-I5503 Build/ERE27) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2866851 = {S92976, 138, { &P22866,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AT-AS43D3 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868371 = {S92977, 127, { &P20510,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CT920 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2868642 = {S92978, 141, { &P23008,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax A91 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868983 = {S92979, 137, { &P18174,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; VS920 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869047 = {S92980, 137, { &P19870,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-L710 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869497 = {S92981, 134, { &P21011,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; L-01E Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2870147 = {S92982, 139, { &P20006,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT30a Build/7.0.A.3.195) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877420 = {S92983, 133, { &P21226,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; E860 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879197 = {S92984, 139, { &P11747,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Desire Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879417 = {S92985, 141, { &P23643,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Tablet P1801-T Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2890088 = {S92986, 100, { &P15364,&P17485,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw660i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2993013 = {S92987, 162, { &P15364,&P17485,&P24676,&P18092 }};	/*Useragent - SonyEricssonC702/R3EK Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonc702) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2892366 = {S92988, 134, { &P18505,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; MB886 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2897844 = {S92989, 128, { &P20446,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; F-05E Build/V16R39A) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2893671 = {S92990, 152, { &P21472,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; A111 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2896969 = {S92991, 93, { &P21261,&P18598,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.2; en-US; GT-I9500) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2936924 = {S92992, 151, { &P21628,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8552 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2944650 = {S92993, 168, { &P18113,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2949808 = {S92994, 132, { &P13006,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; folio100 Build/HTJ85B) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3098220 = {S92995, 131, { &P13006,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fr-fr; folio100 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951028 = {S92996, 167, { &P13070,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-E739 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957631 = {S92997, 158, { &P20708,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; XOLO A1000 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2958153 = {S92998, 147, { &P22584,&P18598,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-ca; SGH-I337M Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2958369 = {S92999, 157, { &P23771,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8190N Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2958375 = {S93000, 152, { &P20280,&P18598,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; vivo X1St Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2961745 = {S93001, 90, { &P18448,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; ST26i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2961999 = {S93002, 164, { &P12987,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; SPH-M930BST Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2963516 = {S93003, 156, { &P22875,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9220 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2982574 = {S93004, 74, { &P17595,&P21721,&P18428,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:16.0) Gecko/X Firefox/16.0*/
const Device UA2993462 = {S93005, 134, { &P21963,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Q800 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2997822 = {S93006, 155, { &P18645,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2999938 = {S93007, 136, { &P17625,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-in; ST25i Build/6.1.1.B.1.10) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3000346 = {S93008, 98, { &P23131,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ultrafone_303) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3003293 = {S93009, 138, { &P21020,&P18598,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.1; en-in; Micromax A116 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3003850 = {S93010, 161, { &P19938,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; SCH-I589 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3004137 = {S93011, 140, { &P21963,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; XOLO Q800 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3004784 = {S93012, 146, { &P12788,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PG86100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3006673 = {S93013, 140, { &P24096,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Orange Kivo Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3006952 = {S93014, 145, { &P22930,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; D2-727 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3007970 = {S93015, 132, { &P22585,&P18598,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.2; en-us; SPH-L720 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3012782 = {S93016, 158, { &P14551,&P17483,&P24678,&P18092 }};	/*Useragent - LG-GW305/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gw305) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3014743 = {S93017, 132, { &P20290,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; A11 Build/KarbonnA11) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3016185 = {S93018, 99, { &P21269,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Intex Aqua 3.2) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3016694 = {S93019, 103, { &P14983,&P5055,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac7-00) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3022145 = {S93020, 135, { &P23165,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; H7500+ Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022381 = {S93021, 137, { &P12841,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Vivid 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023245 = {S93022, 142, { &P19968,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Haier HW-W910 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023273 = {S93023, 133, { &P18314,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M032 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023516 = {S93024, 139, { &P23154,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOLO_X1000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3031939 = {S93025, 93, { &P14358,&P5890,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae63-1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3128663 = {S93026, 92, { &P14358,&P5890,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE63) U2/1.0.0 UCBrowser/9.2.0.336 U2/1.0.0 Mobile*/
const Device UA3049034 = {S93027, 158, { &P14433,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA3049484 = {S93028, 156, { &P23295,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; ZTE U807 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.292 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3050323 = {S93029, 140, { &P19847,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; GT-S6012B Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058689 = {S93030, 161, { &P13974,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5670 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3058743 = {S93031, 89, { &P16598,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A500) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3060585 = {S93032, 156, { &P18507,&P5629,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3078668 = {S93033, 152, { &P22717,&P5843,&P24679,&P18092 }};	/*Useragent - Nokia109/2.0 (04.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia109) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3079405 = {S93034, 141, { &P23154,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XOLO_X1000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3082982 = {S93035, 135, { &P22597,&P21637,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SCH-I545 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.30 Mobile Safari/537.36*/
const Device UA3092752 = {S93036, 155, { &P23546,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AD683G Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3092420 = {S93037, 97, { &P19869,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; Micromax_A30) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3093397 = {S93038, 93, { &P18700,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-B5512) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3093482 = {S93039, 98, { &P12142,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; SonyEricssonU5) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3109426 = {S93040, 157, { &P22584,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I337M Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3110553 = {S93041, 169, { &P20005,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A65 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3110915 = {S93042, 167, { &P23122,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; Karbonn A6 Build/MocorDroid4.0.3) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3111227 = {S93043, 159, { &P18447,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30p Build/9.1.A.1.141) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3112104 = {S93044, 141, { &P23021,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Spice Mi-535 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3112958 = {S93045, 102, { &P20146,&P18110,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-I9082) U2/1.0.0 UCBrowser/9.3.0.321 Mobile*/
const Device UA3127227 = {S93046, 159, { &P21602,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; C5302 Build/12.0.A.1.211) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127342 = {S93047, 159, { &P21602,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; C5302 Build/12.0.A.2.254) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3131416 = {S93048, 138, { &P20076,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26w Build/6.2.B.1.96) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA5098 = {S93049, 78, { &P13348,&P17484,&P3430,&P18092 }};	/*Useragent - SAMSUNG-GT-B7620/1.0 Browser/Opera/9.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64688 = {S93050, 60, { &P14386,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia6610/1.0 (5.52) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA20829 = {S93051, 81, { &P13610,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-F952(METFONE)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63043 = {S93052, 77, { &P13610,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-F952(VMS)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA24636 = {S93053, 88, { &P15726,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-G889/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA64202 = {S93054, 81, { &P12239,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonJ300i/R2BA SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA43831 = {S93055, 68, { &P13751,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHP310/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA52057 = {S93056, 32, { &P13434,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 D905i(c100;TB;W24H17)*/
const Device UA53943 = {S93057, 44, { &P14073,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-300/1.0 ObigoInternetBrowser/Q03C*/
const Device UA54189 = {S93058, 141, { &P18012,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; V8402 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA60847 = {S93059, 156, { &P12582,&P5096,&P3508,&P18092 }};	/*Useragent - SonyEricssonWT18i_TD/1.0 OPhone/2.5 (Linux; Android 2.2) Release/5.23.2011 Browser/WAP2.0 (AppleWebKit/533.1) Mobile Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63379 = {S93060, 115, { &P14507,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-M7600L/1.0 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA68470 = {S93061, 102, { &P14802,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i917)*/
const Device UA925714 = {S93062, 103, { &P14802,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i917.)*/
const Device UA96140 = {S93063, 12, { &P16249,&P17483,&P17660,&P18092 }};	/*Useragent - LG/U250/v1.0*/
const Device UA97610 = {S93064, 85, { &P15364,&P5690,&P18353,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; MEGAUPLOAD 1.0) Sleipnir/2.41*/
const Device UA2462915 = {S93065, 109, { &P15364,&P5690,&P18353,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; GTB6; .NET CLR 1.1.4322; .NET CLR 2.0.50727) Sleipnir/2.49*/
const Device UA436233 = {S93066, 59, { &P15364,&P6729,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Windows 95; U; ru) Presto/2.9.168 Version/11.50*/
const Device UA459005 = {S93067, 139, { &P22547,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; ru-ru; MID7027 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA697077 = {S93068, 43, { &P15198,&P17484,&P17347,&P18092 }};	/*Useragent - samsung-gt-s3650/UC Browser8.0.3.107/69/444*/
const Device UA760010 = {S93069, 64, { &P15364,&P7133,&P17213,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1; en) Opera 8.0*/
const Device UA896390 = {S93070, 79, { &P14454,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-in; NokiaE66)/UC Browser8.4.0.159/28/352/UCWEB Mobile*/
const Device UA904151 = {S93071, 129, { &P15364,&P16110,&P22431,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5 Comodo_Dragon/19.2.0.0*/
const Device UA906828 = {S93072, 122, { &P15364,&P16110,&P22431,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.56 Safari/536.5 Comodo_Dragon/19.2.0.0*/
const Device UA1932650 = {S93073, 106, { &P11646,&P17485,&P17347,&P18092 }};	/*Useragent - SonyEricssonW595/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonw595/UC Browser8.0.3.107/69/352*/
const Device UA1945933 = {S93074, 95, { &P15803,&P5843,&P4777,&P18092 }};	/*Useragent - Nokia200/2.0 (10.58) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia200/UC Browser7.7.1.88/69/444*/
const Device UA1955429 = {S93075, 139, { &P15483,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; HTC_ChaCha_A810b; en-au) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA1957109 = {S93076, 136, { &P15448,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SGH-T839 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2157923 = {S93077, 138, { &P12989,&P5843,&P24667,&P18092 }};	/*Useragent - Nokia303/5.0 (14.38) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia303) UCBrowser8.2.1.144/70/355/UCWEB Mobile*/
const Device UA1974907 = {S93078, 164, { &P15028,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P350 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352*/
const Device UA1987336 = {S93079, 155, { &P13669,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-MS690 Build/GWK74; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2609583 = {S93080, 163, { &P13452,&P5096,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en_us ; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2631560 = {S93081, 165, { &P13452,&P5096,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/352*/
const Device UA2634320 = {S93082, 167, { &P13452,&P5096,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-US; GT-P1000 Build/FROYO) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2720523 = {S93083, 147, { &P21860,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ViewSonic-ViewPad4 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025095 = {S93084, 70, { &P18214,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;GT-S7500;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA2028682 = {S93085, 135, { &P22344,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; eZee'Tab971 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2691304 = {S93086, 135, { &P22344,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; eZee'Tab971 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2028790 = {S93087, 153, { &P13094,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Lenovo A60 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028969 = {S93088, 155, { &P18513,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030376 = {S93089, 172, { &P21223,&P5629,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_C8810 Build/C8810V100R001C92B862) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031431 = {S93090, 171, { &P18700,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-B5512 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2159792 = {S93091, 86, { &P14851,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s5753e) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2222924 = {S93092, 145, { &P15919,&P18066,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2611758 = {S93093, 145, { &P15919,&P18066,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617480 = {S93094, 143, { &P15919,&P18066,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2236451 = {S93095, 39, { &P18218,&P15924,&P20762,&P18092 }};	/*Useragent - NOKIA-Lumia 610/UCWEB2.5.0.202/47/32079*/
const Device UA2809822 = {S93096, 151, { &P22545,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0UI; es-us; Space Livepad 7.2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2337732 = {S93097, 250, { &P12059,&P5204,&P18524,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/70/352*/
const Device UA2574006 = {S93098, 26, { &P19891,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-I717/100.40004*/
const Device UA2372195 = {S93099, 204, { &P14916,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung gt-c3303k) UCBrowser8.3.1.161/70/465/UCWEB Mobile, SAMSUNG-GT-C3303K/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA2393986 = {S93100, 204, { &P14916,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung gt-c3303i) UCBrowser8.3.1.161/70/465/UCWEB Mobile, SAMSUNG-GT-C3303i/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA2372844 = {S93101, 62, { &P12798,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.4;Zh_cn;LT18i;480*854;)UCWEB7.6.0.75/139/32322*/
const Device UA2384836 = {S93102, 152, { &P18253,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT894 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2391436 = {S93103, 155, { &P18465,&P18066,&P19815,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.3 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2395710 = {S93104, 134, { &P22159,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; AUXUS AX01 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2422153 = {S93105, 78, { &P17595,&P18207,&P19821,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:18.0) Gecko/X Thunderbird/18.0*/
const Device UA2423706 = {S93106, 64, { &P15364,&P17285,&P18429,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:17.0) Gecko/X Firefox/17.0 slurp*/
const Device UA2426780 = {S93107, 63, { &P15767,&P18432,&P22071,&P18092 }};	/*Useragent - Instagram 3.1.2 (iPad; iPhone OS 6.0.1; en_US) AppleWebKit/420+*/
const Device UA2836442 = {S93108, 61, { &P15767,&P18432,&P22071,&P18092 }};	/*Useragent - Instagram 3.1.2 (iPad; iPhone OS 6.0; en_US) AppleWebKit/420+*/
const Device UA2453495 = {S93109, 157, { &P18672,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2472765 = {S93110, 97, { &P11958,&P5448,&P4779,&P18092 }};	/*Useragent - Palm750/v0100 Mozilla/4.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA2805396 = {S93111, 85, { &P11958,&P5448,&P4779,&P18092 }};	/*Useragent - Palm750/v0005 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA2473741 = {S93112, 131, { &P21296,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; FreeTAB Build/RK2906) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2484775 = {S93113, 97, { &P12209,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-ks360 teleca) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2497146 = {S93114, 73, { &P21094,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; Lenovo_A789; 480*800) UCWEB7.9.4.145/139/800*/
const Device UA2506639 = {S93115, 134, { &P21104,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; X507 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2643578 = {S93116, 130, { &P20610,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; nl-nl; TAB224 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2519663 = {S93117, 176, { &P22539,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ZTPAD ZT-282 (Slatedroid Market Script) Build/ICS.f04ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2648343 = {S93118, 149, { &P22539,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ZTPAD ZT-282 Build/ICS.f04ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2520683 = {S93119, 133, { &P20543,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; A660 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2527323 = {S93120, 62, { &P16168,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; NookColor Build/JZO54K)*/
const Device UA2554670 = {S93121, 61, { &P20075,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Xperia S Build/JZO54K)*/
const Device UA2577686 = {S93122, 140, { &P22132,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SOWA_S7700 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2588468 = {S93123, 145, { &P15364,&P5629,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; D23 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2812782 = {S93124, 150, { &P15364,&P5629,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; TCL_A966 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2594948 = {S93125, 33, { &P14635,&P5843,&P21724,&P18092 }};	/*Useragent - OneBrowser/3.0 (NokiaC3-00/08.65)*/
const Device UA2600207 = {S93126, 164, { &P18635,&P18598,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us ; Nexus 4 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2602330 = {S93127, 140, { &P13054,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ADR6425LVW Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2615819 = {S93128, 53, { &P12889,&P17979,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.4559 Android/3.1 Device/samsung-GT-P7300*/
const Device UA2624167 = {S93129, 86, { &P12889,&P17979,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.0 Android (16/4.1.1; 160dpi; 1280x752; samsung; GT-P7300; p5; p3; en_US)*/
const Device UA2624215 = {S93130, 90, { &P12889,&P17979,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.0 Android (13/3.2; 160dpi; 800x1232; samsung; GT-P7300; GT-P7300; p3; en_GB)*/
const Device UA2642517 = {S93131, 133, { &P12280,&P17084,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B117 Safari Version/5.1*/
const Device UA2840941 = {S93132, 143, { &P12280,&P17084,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/5.1 Mobile/8B117 Safari/7534.48.3*/
const Device UA2652698 = {S93133, 156, { &P15364,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; X1 Build/HCL ME Tablet X1) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3121225 = {S93134, 159, { &P15364,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; in-id; AF9190 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2659516 = {S93135, 165, { &P14123,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P970 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2660121 = {S93136, 100, { &P13179,&P17485,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw350i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2667106 = {S93137, 138, { &P16436,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I8150 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668496 = {S93138, 142, { &P20931,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; iBall Slide 3G 7334 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2670780 = {S93139, 135, { &P18600,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-03D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2696179 = {S93140, 112, { &P15989,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; ST21i-o) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2705936 = {S93141, 129, { &P24536,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VL-110 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2707373 = {S93142, 138, { &P19858,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KYL21 Build/019.0.3b00) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708121 = {S93143, 134, { &P24005,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ZT ICS Build/ICS.f04ref.X) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2710731 = {S93144, 147, { &P20143,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Edison 3G Build/1.1.11-1015 X-15:24) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2711121 = {S93145, 129, { &P20752,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AN10CG3 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2712462 = {S93146, 137, { &P12482,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T959 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713514 = {S93147, 140, { &P21703,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 70 Titanium Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2713571 = {S93148, 138, { &P20180,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOLO_A800 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713948 = {S93149, 140, { &P19999,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Karbonn A30 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714307 = {S93150, 132, { &P21508,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GEM10313BK Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2733007 = {S93151, 126, { &P24510,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N10 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2733362 = {S93152, 135, { &P24005,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ZT ICS Build/ICS.f04ref.X) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2733493 = {S93153, 132, { &P23314,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; G3 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2748349 = {S93154, 95, { &P16620,&P22784,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx510) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2762110 = {S93155, 168, { &P12824,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; U8180 Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2764576 = {S93156, 90, { &P15517,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; MT15i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2765536 = {S93157, 87, { &P21334,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A8) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2767044 = {S93158, 140, { &P23498,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; ZTE V880G Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2781927 = {S93159, 156, { &P12616,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; WT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2783100 = {S93160, 134, { &P20098,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GN800 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2785829 = {S93161, 89, { &P15364,&P7133,&P18201,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.8.0.10) Gecko/X Thunderbird/1.5.0.10*/
const Device UA2791545 = {S93162, 136, { &P23753,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; PHCASIATAB8B Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2794151 = {S93163, 127, { &P18239,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A510 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2794495 = {S93164, 159, { &P17172,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Transformer TF101 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Safari/537.31 OPR/14.0.1074.54070*/
const Device UA2795934 = {S93165, 94, { &P23381,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A5Classic) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2796200 = {S93166, 139, { &P16321,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xolo_X500 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2869176 = {S93167, 176, { &P12139,&P5204,&P24677,&P18092 }};	/*Useragent - Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5310xpressmusic) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2797635 = {S93168, 140, { &P23514,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GL07S Build/HuaweiU9700L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798626 = {S93169, 136, { &P12942,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I927 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799133 = {S93170, 142, { &P21378,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; tablet Fnac 8 Build/1.1.0 X-12:19) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2800089 = {S93171, 133, { &P23086,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Fusion Bolt Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2845270 = {S93172, 139, { &P23450,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; TOOKY A6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2853438 = {S93173, 183, { &P20729,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A27 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2862317 = {S93174, 161, { &P22875,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9220 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2865730 = {S93175, 168, { &P23255,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI G520-5000 Build/HuaweiG520-5000) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2865854 = {S93176, 168, { &P23255,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI G520-0000 Build/HuaweiG520-0000) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA3049635 = {S93177, 167, { &P18295,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; One X Build/JDQ39E; Carbon-CARBON-JB-UNOFFICIAL-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2866807 = {S93178, 26, { &P21648,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-I9505/101.40202*/
const Device UA2866977 = {S93179, 155, { &P12216,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.4; ADR6300 Build/GRJ22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2867276 = {S93180, 126, { &P20252,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V96A Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2868246 = {S93181, 159, { &P11950,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.3; X10i Build/3.0.1.G.0.75) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453*/
const Device UA2869094 = {S93182, 138, { &P16553,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T959V Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2870897 = {S93183, 164, { &P18640,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2876248 = {S93184, 105, { &P20109,&P17484,&P18369,&P18092 }};	/*Useragent - SAMSUNG-GT-S3572 Opera/9.80 (J2ME/MIDP; Opera Mini/7.0.30356/30.3214; U; pt) Presto/2.8.119 Version/11.10*/
const Device UA2877367 = {S93185, 134, { &P23949,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; H9500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877867 = {S93186, 134, { &P15880,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; MB865 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879330 = {S93187, 138, { &P22925,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; M-MP5303G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937722 = {S93188, 156, { &P19898,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I699 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2950691 = {S93189, 135, { &P15364,&P17818,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.4 (KHTML, like Gecko) Ubuntu/11.04 Chromium/22.0.1229.79 Chrome/22.0.1229.79 Safari/537.4*/
const Device UA2952964 = {S93190, 154, { &P21628,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8552 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955960 = {S93191, 171, { &P23255,&P18111,&P20787,&P18092 }};	/*Useragent - MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI G520-0000 Build/HuaweiG520-0000) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956889 = {S93192, 152, { &P21003,&P20706,&P24677,&P18092 }};	/*Useragent - Nokia310/2.0 (07.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia310) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2957326 = {S93193, 150, { &P16479,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-R720 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2958078 = {S93194, 151, { &P22604,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I879 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2958229 = {S93195, 152, { &P23771,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8190N Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2958561 = {S93196, 170, { &P23000,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; TECNO_P3 Build/master) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2960689 = {S93197, 88, { &P19996,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A15) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2960809 = {S93198, 93, { &P12456,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian73-1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3126117 = {S93199, 92, { &P12456,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE73) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA2960816 = {S93200, 97, { &P23226,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Videocon A15) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2961819 = {S93201, 161, { &P22737,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Micromax A115 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2963419 = {S93202, 155, { &P21023,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-E405 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2963721 = {S93203, 157, { &P23156,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; XOLO A700 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2992478 = {S93204, 142, { &P12280,&P19826,&P20788,&P18092 }};	/*Useragent - MQQBrowser/37 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3*/
const Device UA3047202 = {S93205, 143, { &P12280,&P19826,&P20788,&P18092 }};	/*Useragent - MQQBrowser/371 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3*/
const Device UA2994811 = {S93206, 170, { &P21648,&P18598,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9505 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (GT-I9505, 42, 1.9.1)*/
const Device UA2995026 = {S93207, 174, { &P18318,&P18598,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; ASUS Transformer Pad TF700T Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA2999534 = {S93208, 155, { &P20587,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-P715 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3016083 = {S93209, 137, { &P13999,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NexusHD2 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.64 Mobile Safari/537.36*/
const Device UA3018757 = {S93210, 129, { &P18721,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-in; A90S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3019454 = {S93211, 152, { &P22717,&P5843,&P24678,&P18092 }};	/*Useragent - Nokia109/2.0 (04.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia109) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3020081 = {S93212, 159, { &P12502,&P17485,&P24678,&P18092 }};	/*Useragent - SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; fa; sonyericssonw20i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3020570 = {S93213, 135, { &P20437,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; FJL21 Build/V37R47A) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020942 = {S93214, 94, { &P13474,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAX5-01) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3021369 = {S93215, 140, { &P23498,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZTE Blade G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022211 = {S93216, 138, { &P14225,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Desire HD Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022339 = {S93217, 130, { &P23755,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023620 = {S93218, 136, { &P23550,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE T81 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024023 = {S93219, 134, { &P22640,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IQ446 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3058824 = {S93220, 99, { &P14404,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; es-LA; samsung-gt-s5260) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3031948 = {S93221, 137, { &P17172,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Transformer TF101 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3043270 = {S93222, 114, { &P15364,&P18346,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 4.10; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)*/
const Device UA3071922 = {S93223, 114, { &P15364,&P18346,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.10; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)*/
const Device UA3057308 = {S93224, 157, { &P18679,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-B5330 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3059708 = {S93225, 152, { &P22385,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3074252 = {S93226, 136, { &P20213,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; N9500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078660 = {S93227, 112, { &P24736,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Nokia 909)*/
const Device UA3080396 = {S93228, 160, { &P23017,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Spice Mi-502 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3081019 = {S93229, 93, { &P16356,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7000) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3081128 = {S93230, 93, { &P22941,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6812) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3111445 = {S93231, 159, { &P12798,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT18i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3087564 = {S93232, 154, { &P21858,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0000 Build/HuaweiY300-0000) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3088232 = {S93233, 141, { &P18303,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; MT27i Build/6.2.A.0.400) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088450 = {S93234, 150, { &P15649,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA3121637 = {S93235, 151, { &P15649,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA3092730 = {S93236, 99, { &P22158,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; INTEX_CLOUD_X1) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3093373 = {S93237, 133, { &P20518,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; A10 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3094794 = {S93238, 137, { &P12841,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Vivid 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3102484 = {S93239, 138, { &P22999,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A870S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3103963 = {S93240, 138, { &P22999,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A870K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3103909 = {S93241, 57, { &P24069,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; R815 Build/JOP40D)*/
const Device UA3121645 = {S93242, 135, { &P24069,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; th-th; R815 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108341 = {S93243, 176, { &P15785,&P5843,&P24055,&P18092 }};	/*Useragent - Nokia201/2.0 (11.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311*/
const Device UA3109461 = {S93244, 158, { &P20947,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTCEVOV4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3110889 = {S93245, 161, { &P21882,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax P362 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3113903 = {S93246, 147, { &P23203,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; A1-810 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3114554 = {S93247, 164, { &P20183,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126714 = {S93248, 90, { &P15989,&P18066,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3127410 = {S93249, 91, { &P15989,&P18066,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST21i2) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3130731 = {S93250, 145, { &P18645,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131431 = {S93251, 140, { &P23910,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT30at Build/9.1.F.1.120) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132600 = {S93252, 136, { &P23697,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9192 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132941 = {S93253, 129, { &P18639,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 10 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA4877 = {S93254, 91, { &P13322,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) HTC_Snap_S510*/
const Device UA8942 = {S93255, 62, { &P13364,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-X670-Lisbon-orange/X670_V1_OG_Z2_FRES_DFFA107 Obigo/Q03C*/
const Device UA8943 = {S93256, 62, { &P13364,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-X670-Lisbon-orange/X670_V1_OG_Z2_FRES_DFFA110 Obigo/Q03C*/
const Device UA21269 = {S93257, 62, { &P13364,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-X670-Lisboa-orange/X670_V1_OG_Z3_ESFR_DFFA109 Obigo/Q03C*/
const Device UA14225 = {S93258, 131, { &P13798,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 640x480; HTC_X7500/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1) Opera 8.65 [fr]*/
const Device UA44181 = {S93259, 131, { &P13798,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 640x480; HTC_X7500/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1) Opera 8.65 [en]*/
const Device UA44182 = {S93260, 131, { &P13798,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 640x480; HTC_X7500/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1) Opera 8.65 [nl]*/
const Device UA21376 = {S93261, 44, { &P15270,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-600/1.0 ObigoInternetBrowser/Q03C*/
const Device UA21661 = {S93262, 82, { &P14091,&P5448,&P22270,&P18092 }};	/*Useragent - PANTECH-C810/M044.33 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA30829 = {S93263, 82, { &P14091,&P5448,&P22270,&P18092 }};	/*Useragent - PANTECH-C810/CB.WM01 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA54023 = {S93264, 82, { &P14091,&P5448,&P22270,&P18092 }};	/*Useragent - PANTECH-C810/CT.WM02 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA59313 = {S93265, 82, { &P14091,&P5448,&P22270,&P18092 }};	/*Useragent - PANTECH-C810/M043.32 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA22224 = {S93266, 47, { &P15405,&P17484,&P5001,&P18092 }};	/*Useragent - SCH-U650/1.0 NetFront/3.0.22.2.15 (GUI) MMP/2.0*/
const Device UA22248 = {S93267, 111, { &P15364,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHB310/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA24489 = {S93268, 124, { &P15364,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHB300-ORANGE/B300AFHG1 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA87393 = {S93269, 111, { &P15364,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHC516/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA87476 = {S93270, 111, { &P15364,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHC506/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA23879 = {S93271, 54, { &P15632,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-PM225/1.0 UP.Browser/6.2.3.7.e.1.101 (GUI) MMP/2.0*/
const Device UA23998 = {S93272, 42, { &P14427,&P17484,&P3534,&P18092 }};	/*Useragent - Samsung-SPHM800 AU-OBIGO/Q05A-3.13 MMP/2.0*/
const Device UA26453 = {S93273, 94, { &P16789,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG-SCH-i225*/
const Device UA47632 = {S93274, 10, { &P15589,&P22835,&P17470,&P18092 }};	/*Useragent - Spice QT68*/
const Device UA49327 = {S93275, 135, { &P12991,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; ru-ru; MTC 916 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA55142 = {S93276, 101, { &P13939,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-X700/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA62204 = {S93277, 37, { &P14383,&P17483,&P3866,&P18092 }};	/*Useragent - LG-KE770 MIC/1.1.14 MIDP-2.0/CLDC-1.1*/
const Device UA62877 = {S93278, 30, { &P14458,&P22839,&P3579,&P18092 }};	/*Useragent - ZTE-C70/1.0 SMIT-Browser/2.0.0*/
const Device UA67268 = {S93279, 31, { &P14793,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P04A(c100;TB;W24H15)*/
const Device UA556606 = {S93280, 116, { &P11658,&P5776,&P4777,&P18092 }};	/*Useragent - Nokia6680/1.0 (5.04.40) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.7.1.88/27/444*/
const Device UA1875219 = {S93281, 137, { &P14931,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; HTC click Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019176 = {S93282, 137, { &P14931,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; HTC click Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA68882 = {S93283, 36, { &P12588,&P5055,&P4777,&P18092 }};	/*Useragent - NokiaC6-01/UC Browser7.7.1.88/50/444*/
const Device UA559899 = {S93284, 46, { &P12588,&P5055,&P4777,&P18092 }};	/*Useragent - nokiac6-01/UC Browser7.7.1.88/69/444  3gpp-gba*/
const Device UA87283 = {S93285, 45, { &P17555,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-MA33 UP.Browser/6.2.0.13.2 (GUI) MMP/2.0*/
const Device UA89647 = {S93286, 47, { &P17636,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SN3M UP.Browser/6.2.0.14.1.2 (GUI) MMP/2.0*/
const Device UA89695 = {S93287, 89, { &P17564,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-G912/MTK Release/27.11.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA90279 = {S93288, 31, { &P17739,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P01C(c500;TB;W24H16)*/
const Device UA176621 = {S93289, 43, { &P16555,&P22784,&P17470,&P18092 }};	/*Useragent - MicromaxX410/MIDP2.0/CLDC1.1/Screen-240X320*/
const Device UA192101 = {S93290, 101, { &P15364,&P22192,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.0.6002.2);en-US;kml:2.2;client:Free;type:default)*/
const Device UA197369 = {S93291, 98, { &P15364,&P22192,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.0.6002.2);en;kml:2.2;client:Free;type:default)*/
const Device UA198151 = {S93292, 101, { &P15364,&P22192,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.0.6002.2);en-GB;kml:2.2;client:Free;type:default)*/
const Device UA325720 = {S93293, 101, { &P15364,&P22192,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.0.6001.1);en-US;kml:2.2;client:Free;type:default)*/
const Device UA419893 = {S93294, 98, { &P15364,&P22192,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.0.6002.2);de;kml:2.2;client:Free;type:default)*/
const Device UA2292847 = {S93295, 101, { &P15364,&P22192,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Windows;Microsoft Windows (6.0.6002.2);de-DE;kml:2.2;client:Free;type:default)*/
const Device UA284885 = {S93296, 31, { &P17731,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N04A(c100;TB;W24H16)*/
const Device UA516890 = {S93297, 93, { &P15364,&P7133,&P21925,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0) chromeframe/9.0.597.94*/
const Device UA524230 = {S93298, 87, { &P15364,&P7133,&P21925,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0) chromeframe/9.0.597.107*/
const Device UA2275964 = {S93299, 143, { &P15364,&P7133,&P21925,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.107 Safari/534.13 chromeframe/9.0.597.107*/
const Device UA363790 = {S93300, 140, { &P15767,&P17090,&P20772,&P18092 }};	/*Useragent - MQQBrowser/2.1 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3*/
const Device UA723161 = {S93301, 140, { &P15767,&P17090,&P20772,&P18092 }};	/*Useragent - MQQBrowser/2.1 Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3*/
const Device UA452242 = {S93302, 43, { &P14182,&P17484,&P17338,&P18092 }};	/*Useragent - samsung-gt-s5333/UC Browser7.9.0.102/69/352*/
const Device UA502773 = {S93303, 100, { &P15364,&P18110,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PRS-950; U; Linux; en-us) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2 PRS-950/2.0*/
const Device UA547308 = {S93304, 60, { &P16874,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxQ55/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA561166 = {S93305, 100, { &P14640,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; SAMSUNG; OMNIA7)*/
const Device UA925738 = {S93306, 108, { &P14640,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; SAMSUNG; OMNIA7; Orange)*/
const Device UA564436 = {S93307, 122, { &P14971,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-B5722/B5722JPJJ1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA848086 = {S93308, 71, { &P15364,&P16110,&P18329,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:11.0) Gecko/X Thunderbird/11.0.1*/
const Device UA662666 = {S93309, 169, { &P13018,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; MT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA705970 = {S93310, 64, { &P15364,&P7127,&P18329,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:11.0) Gecko/X Thunderbird/11.0.1*/
const Device UA711147 = {S93311, 104, { &P11808,&P5204,&P17347,&P18092 }};	/*Useragent - Nokia5610d-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5610d-1/UC Browser8.0.3.107/69/444*/
const Device UA2302241 = {S93312, 116, { &P14708,&P5204,&P17437,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia5130c-2/07.91; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.4.0.34.8*/
const Device UA767756 = {S93313, 26, { &P18088,&P17017,&P17349,&P18092 }};	/*Useragent - J2ME/UCWEB7.3.1.56/139/999*/
const Device UA770632 = {S93314, 32, { &P15364,&P21442,&P4810,&P18092 }};	/*Useragent - Opera/7.0 (Windows 2000; U) [en]*/
const Device UA2774773 = {S93315, 44, { &P15364,&P21442,&P4810,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows 2000; U) Opera 7.0 [en]*/
const Device UA792039 = {S93316, 80, { &P15364,&P7133,&P20904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.7.5) Gecko/X Netscape/8.1.3*/
const Device UA829992 = {S93317, 77, { &P15518,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; gt-s5230) UCBrowser8.2.0.132/69/352/UCWEB Mobile*/
const Device UA2814323 = {S93318, 30, { &P15518,&P17484,&P24667,&P18092 }};	/*Useragent - gt-s5230/UCWEB8.2.0.116/69/352*/
const Device UA2319378 = {S93319, 80, { &P14984,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5250)/UC Browser8.4.0.159/50/352/UCWEB Mobile*/
const Device UA1891109 = {S93320, 76, { &P15364,&P17285,&P18196,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:5.0) Gecko/X Thunderbird/5.0 Lightning/1.0b4*/
const Device UA1899933 = {S93321, 92, { &P17976,&P6545,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.0.1; Linux; Opera Tablet/ADR-X; U; en-GB) Presto/2.8.149 Version/11.10*/
const Device UA1918222 = {S93322, 62, { &P15364,&P17285,&P17698,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:8.0) Gecko/X Firefox/8.0 slurp*/
const Device UA1938979 = {S93323, 44, { &P14831,&P17484,&P17347,&P18092 }};	/*Useragent - samsung-gt-s7233e/UC Browser8.0.3.107/69/444*/
const Device UA1940036 = {S93324, 106, { &P11660,&P17485,&P17347,&P18092 }};	/*Useragent - SonyEricssonC902/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonc902/UC Browser8.0.3.107/69/444*/
const Device UA2610903 = {S93325, 165, { &P15985,&P6437,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6200L Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2613689 = {S93326, 164, { &P15985,&P6437,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6200 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2026240 = {S93327, 132, { &P19980,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; X903 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027885 = {S93328, 144, { &P21052,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029725 = {S93329, 164, { &P21151,&P5629,&P20776,&P18092 }};	/*Useragent - MQQBrowser/25 Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LNV-Lenovo A790e Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030209 = {S93330, 154, { &P15575,&P18066,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2260279 = {S93331, 155, { &P15575,&P18066,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2145297 = {S93332, 43, { &P15197,&P17484,&P17338,&P18092 }};	/*Useragent - samsung-gt-b5310/UC Browser7.9.0.102/69/351*/
const Device UA2159854 = {S93333, 85, { &P15198,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s3650) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2212800 = {S93334, 126, { &P23649,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40EX525; CC/BRA; en) Presto/2.7.61 Version/11.00*/
const Device UA2218326 = {S93335, 74, { &P19998,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HUAWEI_C8812; 480*800) UCWEB7.9.4.145/139/800*/
const Device UA2444515 = {S93336, 145, { &P13634,&P17483,&P24668,&P18092 }};	/*Useragent - LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-gs500) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2411310 = {S93337, 85, { &P14764,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU1)/UC Browser8.2.0.132/50/444/UCWEB Mobile*/
const Device UA2392543 = {S93338, 61, { &P15364,&P7127,&P17387,&P18091 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0) METASpider*/
const Device UA2404900 = {S93339, 167, { &P18295,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC One X Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355*/
const Device UA2409761 = {S93340, 62, { &P16886,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX1i_R/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA2414846 = {S93341, 67, { &P12616,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.4;Zh_cn;WT19i;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA2442282 = {S93342, 79, { &P16871,&P22784,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; micromaxq5) UCBrowser8.3.0.154/69/352/UCWEB Mobile*/
const Device UA2455074 = {S93343, 60, { &P15364,&P6704,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (Windows 98; U; en) Presto/2.10.289 Version/12.02*/
const Device UA2865171 = {S93344, 79, { &P15767,&P17779,&P18525,&P18092 }};	/*Useragent - Opera/9.80 (iPad; Opera Mini/6.0.1/29.3345; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2472825 = {S93345, 146, { &P15364,&P17285,&P18354,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; InfoPath.1) Sleipnir/2.5.17*/
const Device UA2478236 = {S93346, 136, { &P22033,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SP-140 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2481941 = {S93347, 149, { &P21626,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile i-STYLE Q6 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2504675 = {S93348, 138, { &P14969,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Desire S Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2519543 = {S93349, 251, { &P14540,&P5843,&P18516,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/69/444*/
const Device UA2524655 = {S93350, 117, { &P12287,&P5043,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia6700c-1/13.10; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.2.68.14*/
const Device UA2555031 = {S93351, 128, { &P23824,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M870 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2558372 = {S93352, 156, { &P11793,&P5768,&P24676,&P18092 }};	/*Useragent - Nokia3500c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3500c) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2579179 = {S93353, 155, { &P18450,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; ST23i Build/11.0.A.5.5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2592259 = {S93354, 186, { &P12135,&P17483,&P24671,&P18092 }};	/*Useragent - LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-gd510) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2594038 = {S93355, 143, { &P21945,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ja-jp; edenTABET701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2601241 = {S93356, 138, { &P18637,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9260 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602989 = {S93357, 138, { &P15820,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-SU640 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2618227 = {S93358, 92, { &P13452,&P5096,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Messenger; 1.8.3) (GT-P1000; samsung; GT-P1000; 2.2/FROYO)*/
const Device UA2621797 = {S93359, 90, { &P17337,&P17979,&P22073,&P18092 }};	/*Useragent - Instagram 3.3.3 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7500; GT-P7500; p3; en_US)*/
const Device UA2648019 = {S93360, 128, { &P22084,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; iDx10 Build/iDx10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2652828 = {S93361, 72, { &P17595,&P18577,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBook5%2C1)*/
const Device UA2790030 = {S93362, 75, { &P17595,&P18577,&P18694,&P18092 }};	/*Useragent - Reeder/1020.09.00 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro7%2C1)*/
const Device UA2653526 = {S93363, 166, { &P15364,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us  Build/HCL ME Tablet Y2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2837808 = {S93364, 161, { &P15364,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Z7H Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2654303 = {S93365, 40, { &P15364,&P17017,&P23787,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; ipMonitor 10.7)*/
const Device UA2656313 = {S93366, 149, { &P20231,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A21 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2835416 = {S93367, 157, { &P20231,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Karbonn A21 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2656723 = {S93368, 129, { &P23347,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; V900A Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2658142 = {S93369, 170, { &P22661,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; TM-9743W Build/V1.1.ICECREAMSANDWICH.rus.teXet.X.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 MYI/1.10*/
const Device UA2661418 = {S93370, 167, { &P18554,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BNTV400 Build/2.0.6.6008.magnesium02.hummingbird.rrdf.s1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2666875 = {S93371, 141, { &P21402,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Lenovo S890 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667031 = {S93372, 137, { &P14872,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-D700 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667553 = {S93373, 129, { &P21352,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB469 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2668276 = {S93374, 142, { &P20491,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Thunder S220 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2669549 = {S93375, 156, { &P20146,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9082 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2675512 = {S93376, 141, { &P21614,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Philips W536 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2816740 = {S93377, 85, { &P14404,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5260) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2693666 = {S93378, 185, { &P13634,&P17483,&P24672,&P18092 }};	/*Useragent - LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-gs500) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2696536 = {S93379, 137, { &P23431,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; miTab DIAMOND Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2708205 = {S93380, 135, { &P23889,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PAD703 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709135 = {S93381, 136, { &P12942,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-I927 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2711002 = {S93382, 130, { &P20241,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5570C Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2728009 = {S93383, 148, { &P15364,&P5629,&P21753,&P18092 }};	/*Useragent - iBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Mtag353 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2728105 = {S93384, 101, { &P18623,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 510)*/
const Device UA2729938 = {S93385, 154, { &P20146,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734297 = {S93386, 151, { &P22875,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9220 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2734491 = {S93387, 158, { &P16362,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-D710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2734806 = {S93388, 125, { &P24510,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N10 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735021 = {S93389, 130, { &P18731,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ONDA MID Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735168 = {S93390, 164, { &P12987,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M930BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2748354 = {S93391, 141, { &P23377,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; NuqleoFusion Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2748393 = {S93392, 139, { &P23459,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; POLY PAD 9716HD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2749049 = {S93393, 146, { &P21640,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Samsung GT-S9081 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2848919 = {S93394, 138, { &P21640,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I425 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2755104 = {S93395, 138, { &P22592,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717R Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2767687 = {S93396, 157, { &P11747,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; HTC Desire Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2779231 = {S93397, 90, { &P20076,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT26w) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2779419 = {S93398, 174, { &P19779,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A1+ Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2782253 = {S93399, 153, { &P18471,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-66) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2792709 = {S93400, 56, { &P15364,&P22310,&P17190,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 8.0; Win3.11; Trident/1.0)*/
const Device UA2793205 = {S93401, 88, { &P18083,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; V9C) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2793515 = {S93402, 140, { &P23569,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; SINUDYNE_SY970G2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2800232 = {S93403, 158, { &P18707,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MD_LIFETAB_P9516 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2825589 = {S93404, 189, { &P12132,&P17485,&P24671,&P18092 }};	/*Useragent - SonyEricssonJ10i2/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; sonyericssonj10i2) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2981594 = {S93405, 185, { &P12132,&P17485,&P24671,&P18092 }};	/*Useragent - SonyEricssonJ10i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; fa; sonyericssonj10i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2834126 = {S93406, 126, { &P20126,&P17485,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; sonyericssonk800i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile*/
const Device UA2852213 = {S93407, 162, { &P14186,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-us; GT-S5830M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000*/
const Device UA3044728 = {S93408, 161, { &P14186,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000*/
const Device UA2857247 = {S93409, 51, { &P17595,&P18120,&P22589,&P18092 }};	/*Useragent - MQQBrowser/42 (iOS 4S; U; CPU like Mac OS X; zh-cn)*/
const Device UA2857794 = {S93410, 50, { &P17595,&P18120,&P22589,&P18092 }};	/*Useragent - MQQBrowser/42 (iOS 4; U; CPU like Mac OS X; zh-cn)*/
const Device UA2860791 = {S93411, 93, { &P23388,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; TSINTECH) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2867394 = {S93412, 126, { &P18239,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A510 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2867499 = {S93413, 139, { &P12681,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus S 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867728 = {S93414, 135, { &P20511,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; IQ442 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2868836 = {S93415, 135, { &P20437,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; FJL21 Build/V37R47A) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869311 = {S93416, 145, { &P15555,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; MyTouch 4G Slide Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869871 = {S93417, 135, { &P20359,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZP300+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2873523 = {S93418, 139, { &P17625,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST25i Build/6.2.A.0.400) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2876093 = {S93419, 155, { &P20705,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2877474 = {S93420, 127, { &P22242,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ME171 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2878993 = {S93421, 108, { &P21968,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; ALCATEL_ONE_TOUCH_4030D) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2879343 = {S93422, 138, { &P24150,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; vivo Xplay Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879441 = {S93423, 133, { &P24069,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; R815 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879563 = {S93424, 132, { &P23807,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A50 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879634 = {S93425, 139, { &P13526,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; myTouch 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2889490 = {S93426, 74, { &P17595,&P21721,&P18521,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:18.0) Gecko/X Firefox/18.0*/
const Device UA2890205 = {S93427, 156, { &P13974,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-S5670 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892196 = {S93428, 96, { &P14834,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; Nokia5730S-1) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2892395 = {S93429, 154, { &P17283,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB860 Build/4.5.1A-129_OLY-145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2892548 = {S93430, 142, { &P23838,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; StarTrail III Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892822 = {S93431, 128, { &P24536,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VL-110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2936682 = {S93432, 154, { &P20530,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MOT-XT788 Build/IRPMCT_B_02.74.00RPD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2936775 = {S93433, 138, { &P22397,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; HW A880 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2938016 = {S93434, 102, { &P15851,&P17350,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 7.10; en-US; HTC; Radar C110e) U2/1.0.0 UCBrowser/3.0.0.285 U2/1.0.0 Mobile*/
const Device UA3107626 = {S93435, 102, { &P15851,&P17350,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 7.10; en-US; HTC; Radar C110e) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile*/
const Device UA2950753 = {S93436, 117, { &P17595,&P21721,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2957069 = {S93437, 163, { &P24103,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; AliyunOS 2.0; Android 4.0 Compatible; en-us; C2 Build/AliyunOs-2012) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2957239 = {S93438, 162, { &P20094,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2957410 = {S93439, 172, { &P22584,&P18598,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-ca; SGH-I337M Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SGH-I337M, 47, 1.9.7)*/
const Device UA2963508 = {S93440, 150, { &P24078,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; E3 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2963727 = {S93441, 159, { &P18303,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT27i Build/6.1.1.B.1.54) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2987024 = {S93442, 160, { &P15158,&P5843,&P24676,&P18092 }};	/*Useragent - Nokia2710c-2/2.0 (06.13) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia2710c-2) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2999880 = {S93443, 155, { &P11747,&P5096,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-US; HTC Desire Build/FRF91) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2994161 = {S93444, 107, { &P19868,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL ONE TOUCH 985N) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA3121205 = {S93445, 107, { &P19868,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL ONE TOUCH 985N) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2999297 = {S93446, 157, { &P18295,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; HTC One X Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2997470 = {S93447, 139, { &P23242,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SKY HD9500 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3050142 = {S93448, 144, { &P23242,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SKY HD9500 Plus Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2998057 = {S93449, 159, { &P15575,&P18066,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.289*/
const Device UA2998160 = {S93450, 153, { &P24539,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; ALCATEL ONE TOUCH 6010 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA3010201 = {S93451, 134, { &P18450,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-in; ST23i Build/11.0.A.5.5) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3010840 = {S93452, 154, { &P14967,&P5096,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-US; LG-P500 Build/FRG83) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056824 = {S93453, 152, { &P14967,&P5096,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-US; LG-P500 Build/FRF91) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3011298 = {S93454, 160, { &P23017,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Spice Mi-502 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3021227 = {S93455, 137, { &P15802,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Amaze 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021571 = {S93456, 134, { &P21302,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IQ441 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021947 = {S93457, 137, { &P22607,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S5303 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022209 = {S93458, 145, { &P20000,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ607 Build/7.7.1-122-4-FLEM-WIFI-LA) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022956 = {S93459, 134, { &P21011,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; L-01E Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023535 = {S93460, 137, { &P17324,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9001 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024021 = {S93461, 130, { &P12889,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3024111 = {S93462, 139, { &P23148,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ONE TOUCH TAB 7HD Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3025925 = {S93463, 169, { &P18635,&P21637,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (Nexus 4, 52, 1.9.9.2)*/
const Device UA3079602 = {S93464, 167, { &P18635,&P21637,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 4 Build/JWR66Y) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (Nexus 4, 54, 2.0.1)*/
const Device UA3043378 = {S93465, 52, { &P15364,&P5890,&P20784,&P18092 }};	/*Useragent - MQQBrowser/3.3 (NokiaNXX;SymbianOS/9.1 Series60/3.0)*/
const Device UA3099163 = {S93466, 47, { &P15364,&P5890,&P20784,&P18092 }};	/*Useragent - MQQBrowser/3.3 (E66;SymbianOS/9.1 Series60/3.0)*/
const Device UA3049783 = {S93467, 162, { &P21657,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI C8950D Build/HuaweiC8950D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3088668 = {S93468, 160, { &P21657,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8950 Build/HuaweiT8950) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3050172 = {S93469, 156, { &P18712,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Next8P12 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3051829 = {S93470, 132, { &P11680,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; KM9001RK Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3057282 = {S93471, 138, { &P22991,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; HTC 802t Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057598 = {S93472, 129, { &P21948,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MID0738 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3058968 = {S93473, 112, { &P12108,&P5843,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia7230/10.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3059376 = {S93474, 158, { &P23716,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI A199 Build/HuaweiA199) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3059637 = {S93475, 149, { &P21793,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9205 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3132900 = {S93476, 139, { &P16292,&P21637,&P24394,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Galaxy Nexus Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1650.11 Mobile Safari/537.36*/
const Device UA3073854 = {S93477, 136, { &P22955,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P714 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3079619 = {S93478, 88, { &P20529,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A89) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3079803 = {S93479, 231, { &P24618,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; zh-cn; SAMSUNG-GT-I8558_TD/1.0 Android/4.2.2 Release/04.15.2013 Browser/AppleWebKit535.19 Build/JDQ39) ApplelWebkit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3086436 = {S93480, 154, { &P15364,&P18111,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MEEG_101 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099932 = {S93481, 151, { &P15364,&P18111,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HMIA5 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3093426 = {S93482, 156, { &P22941,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6812 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3094158 = {S93483, 139, { &P23154,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XOLO_X1000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3094268 = {S93484, 134, { &P22705,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IQ451 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3096444 = {S93485, 163, { &P22582,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3118518 = {S93486, 137, { &P23130,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; H2000+ Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3109319 = {S93487, 97, { &P18649,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A57) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3109629 = {S93488, 158, { &P22641,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; C1504 Build/11.3.A.0.47) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3110976 = {S93489, 137, { &P24583,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; EK-GC110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111566 = {S93490, 158, { &P13018,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT11i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3125128 = {S93491, 88, { &P20120,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A45) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3126515 = {S93492, 187, { &P15794,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126879 = {S93493, 106, { &P21475,&P5629,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; Micromax A35) U2/1.0.0 UCBrowser/9.3.0.321 Mobile*/
const Device UA3127553 = {S93494, 97, { &P21475,&P5629,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A35) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3132854 = {S93495, 133, { &P18665,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; N861 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA9403 = {S93496, 104, { &P11953,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonU10a/R7BA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2*/
const Device UA61605 = {S93497, 104, { &P11953,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonU10i/R7BA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2*/
const Device UA11385 = {S93498, 69, { &P15293,&P5448,&P3430,&P18092 }};	/*Useragent - HTC MAX 4G Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/1409; U; ru)*/
const Device UA55166 = {S93499, 101, { &P13607,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-J210/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA16424 = {S93500, 159, { &P13875,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Android 2.1-update1;zh-cn;ZTE-C_R750/R750V1.0.0B05;320*480;CTC/2.0) AppleWebkit/530.17(KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA41072 = {S93501, 159, { &P13875,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Android 2.1-update1;zh-cn;ZTE-C_R750/R750V1.0.0B04;320*480;CTC/2.0) AppleWebkit/530.17(KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA48940 = {S93502, 159, { &P13875,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Android 2.1-update1;zh-cn;ZTE-C_R750/R750V1.0.0B03;320*480;CTC/2.0) AppleWebkit/530.17(KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA56750 = {S93503, 163, { &P13875,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Android 2.1-update1;zh-cn;ZTE-C_R750/VSF_R750V1.0.0B01;320*480;CTC/2.0) AppleWebkit/530.17(KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA16768 = {S93504, 62, { &P13937,&P17486,&P17671,&P18092 }};	/*Useragent - MOT-C168i/1.0 Release/7.24.2006 Browser/CMCS1.0 Software/0.080*/
const Device UA51107 = {S93505, 25, { &P13937,&P17486,&P17671,&P18092 }};	/*Useragent - MOT-C168/ WAP.Browser/1.0*/
const Device UA60787 = {S93506, 65, { &P14060,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW150i/R2AD034 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA253923 = {S93507, 17, { &P14060,&P17485,&P17470,&P18092 }};	/*Useragent - sonyericssonw150i*/
const Device UA50153 = {S93508, 62, { &P11924,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonS500i/R8BE Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA50473 = {S93509, 17, { &P11924,&P17485,&P17470,&P18092 }};	/*Useragent - sonyericssons500i*/
const Device UA20141 = {S93510, 73, { &P15639,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) HTC-6900-MR1*/
const Device UA23963 = {S93511, 69, { &P15639,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) HTC-6900*/
const Device UA20998 = {S93512, 130, { &P16054,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) PPC; 240x320; HTC_P3600i/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21000 = {S93513, 129, { &P16054,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) PPC; 240x320; HTC_P3600/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21202 = {S93514, 129, { &P15167,&P5448,&P4808,&P18092 }};	/*Useragent - LG-GM730f/IEMobile/v6.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA21308 = {S93515, 21, { &P15191,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP115 Obigo/WAP2.0*/
const Device UA21729 = {S93516, 12, { &P15326,&P17483,&P17660,&P18092 }};	/*Useragent - LG/U890/v1.0*/
const Device UA22351 = {S93517, 111, { &P15432,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHB200/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA23171 = {S93518, 83, { &P14364,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK510a/R4EA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23663 = {S93519, 83, { &P14364,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK510c/R4EA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23009 = {S93520, 83, { &P15551,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonZ558c/R4FA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87468 = {S93521, 83, { &P15551,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonZ558i/R4FA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23776 = {S93522, 86, { &P12274,&P5448,&P17183,&P18092 }};	/*Useragent - SAMSUNG-SGH-i900/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA63446 = {S93523, 89, { &P14446,&P5776,&P3756,&P18092 }};	/*Useragent - Nokia6682/2.0 (4.65.0) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24716 = {S93524, 12, { &P15296,&P17483,&P17660,&P18092 }};	/*Useragent - LG/U830/v1.0*/
const Device UA34399 = {S93525, 13, { &P13680,&P17483,&P17660,&P18092 }};	/*Useragent - LG/U8330/v1.0*/
const Device UA42859 = {S93526, 142, { &P16967,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; Dell Aero Build/MASTER) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA50107 = {S93527, 142, { &P24003,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; Google Ion Build/DRC83) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA51759 = {S93528, 144, { &P20801,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; phiremod Vibrant Build/XWJS5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61510 = {S93529, 82, { &P12736,&P17486,&P3807,&P18092 }};	/*Useragent - Mot-EX212/18.01 Browser/Obigo/Q03C MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62083 = {S93530, 47, { &P14370,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-VX8350/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA64906 = {S93531, 127, { &P12096,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320; HTC_TyTN/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA67972 = {S93532, 36, { &P12088,&P5129,&P4759,&P18092 }};	/*Useragent - NokiaX6-00/UC Browser7.6.1.82/50/351*/
const Device UA87124 = {S93533, 47, { &P17631,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SH33 UP.Browser/6.2.0.13.1.5 (GUI) MMP/2.0*/
const Device UA176163 = {S93534, 34, { &P12004,&P5890,&P4759,&P18092 }};	/*Useragent - NOKIAN95/UC Browser7.6.1.82/28/352*/
const Device UA390617 = {S93535, 38, { &P12004,&P5890,&P4759,&P18092 }};	/*Useragent - NOKIAN95 8GB/UC Browser7.6.1.82/28/352*/
const Device UA176465 = {S93536, 71, { &P18088,&P5448,&P4759,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.6.1.82*/
const Device UA333831 = {S93537, 38, { &P12004,&P5890,&P4779,&P18092 }};	/*Useragent - NOKIAN95 8GB/UC Browser7.8.0.95/28/355*/
const Device UA727839 = {S93538, 34, { &P12004,&P5890,&P4779,&P18092 }};	/*Useragent - NOKIAN95/UC Browser7.8.0.95/28/352*/
const Device UA258735 = {S93539, 138, { &P17585,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; OliPad 100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368823 = {S93540, 137, { &P18235,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; S2005A-H Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA403358 = {S93541, 143, { &P18235,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; S2005A-H Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/5.0 Mobile Safari/533.16*/
const Device UA575086 = {S93542, 80, { &P15364,&P7133,&P22537,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:5.0) Gecko/X Firefox/5.0 Kylo/1.0.1.76141*/
const Device UA1949352 = {S93543, 78, { &P17595,&P17295,&P18191,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:10.0) Gecko/X Thunderbird/10.0*/
const Device UA590304 = {S93544, 143, { &P17968,&P5629,&P21557,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/2.1.0 Safari/533.1*/
const Device UA828970 = {S93545, 143, { &P17968,&P5629,&P21557,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/2.1.0 Safari/533.1*/
const Device UA831924 = {S93546, 143, { &P17968,&P5629,&P21557,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/2.1.0 Safari/533.1*/
const Device UA1970068 = {S93547, 143, { &P17968,&P5629,&P21557,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/2.1.0 Safari/533.1*/
const Device UA623795 = {S93548, 74, { &P15364,&P7127,&P20458,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:8.0.1) Gecko/X Firefox/8.0.1 SeaMonkey/2.5*/
const Device UA648855 = {S93549, 75, { &P23994,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; Lenovo_A500; 320*480) UCWEB7.9.3.103/139/32564*/
const Device UA760175 = {S93550, 70, { &P18088,&P17017,&P17368,&P18091 }};	/*Useragent - Opera/8.01 (J2ME/MIDP; MXit WebBot/6.2.1/1.8.5.168; U;) Opera Mini/3.1*/
const Device UA2262224 = {S93551, 70, { &P18088,&P17017,&P17368,&P18091 }};	/*Useragent - Opera/8.01 (J2ME/MIDP; MXit WebBot/6.1.0/1.8.5.168; U;) Opera Mini/3.1*/
const Device UA925781 = {S93552, 97, { &P18309,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Radiant)*/
const Device UA925796 = {S93553, 97, { &P18309,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; PI86100)*/
const Device UA1864970 = {S93554, 108, { &P14029,&P17484,&P3834,&P18092 }};	/*Useragent - SAMSUNG-GT-S5330/1.0 SHP/VPP/R5 Dolfin/2.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA1900944 = {S93555, 126, { &P23626,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40EX723; CC/GBR; en) Presto/2.7.61 Version/11.00*/
const Device UA1935626 = {S93556, 106, { &P11683,&P17485,&P17347,&P18092 }};	/*Useragent - SonyEricssonC905/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonc905/UC Browser8.0.3.107/69/352*/
const Device UA1955171 = {S93557, 148, { &P18498,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LG855 Build/CTMod 3.68)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1956906 = {S93558, 77, { &P15518,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; gt-s5230) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA1958898 = {S93559, 80, { &P14753,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; Nokia N96)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA1976234 = {S93560, 135, { &P14433,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; GT-I9003 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1984259 = {S93561, 169, { &P15047,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; SK17i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2019642 = {S93562, 158, { &P21233,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; id-id; T1 Build/I700T_P4_10_AC73_RE_R_0_03_1010_X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023642 = {S93563, 150, { &P23923,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; SonyEricssonZ1i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034257 = {S93564, 131, { &P23923,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Z1i Build/4.0.2.A.0.42) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2026642 = {S93565, 147, { &P14969,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire S Build/GRJ90)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2957938 = {S93566, 148, { &P14969,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2027991 = {S93567, 131, { &P23783,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pt-br; GT-8230 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029085 = {S93568, 136, { &P20293,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; it-it; MID7034C Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033582 = {S93569, 160, { &P18482,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; A75 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2034109 = {S93570, 129, { &P22381,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SN10T1 Build/IML77) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2128443 = {S93571, 158, { &P15364,&P17814,&P4810,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; de; rv:1.9.0.6) Gecko/X Ubuntu/8.10 (intrepid) Mozilla/4.0 (compatible; MSIE 6.0; MSIE 5.5; Windows NT 5.1) Opera 7.03 [de]*/
const Device UA2148840 = {S93572, 85, { &P11905,&P5890,&P4780,&P18092 }};	/*Useragent - Nokia6120c/3.83; Profile/MIDP-2.0 Configuration/CLDC-1.1;Sansui_H50T_Maui Wap Browser*/
const Device UA2168076 = {S93573, 80, { &P14782,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE61i)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2546097 = {S93574, 79, { &P14782,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE61)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2256898 = {S93575, 167, { &P18661,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; N860 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2261416 = {S93576, 77, { &P23351,&P22784,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; micromax) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2338903 = {S93577, 80, { &P14417,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-in; Nokia5236)/UC Browser8.2.0.132/50/444/UCWEB Mobile*/
const Device UA2340839 = {S93578, 155, { &P18466,&P18066,&P19988,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.0 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2352559 = {S93579, 141, { &P17108,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Inspire 4G Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2982417 = {S93580, 141, { &P17108,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Inspire 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2366173 = {S93581, 173, { &P12444,&P5477,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; U20i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2474977 = {S93582, 124, { &P16455,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-S5610K/S5610KJPKL1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA2589835 = {S93583, 148, { &P12502,&P17485,&P24668,&P18092 }};	/*Useragent - SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonw20i) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2436468 = {S93584, 36, { &P23144,&P17779,&P23080,&P18092 }};	/*Useragent - Instacast/3.0.3 (like iTunes/10.1.2)*/
const Device UA2575145 = {S93585, 35, { &P15803,&P5843,&P23405,&P18092 }};	/*Useragent - MQQBrowser/Mini2.8 (Nokia200/11.56)*/
const Device UA2451535 = {S93586, 144, { &P21111,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IS12M Build/6.7.5-50_MR1-11) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2459531 = {S93587, 166, { &P13591,&P5096,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; ZTE-BLADE Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2460289 = {S93588, 131, { &P18277,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N-04D Build/AX) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2462411 = {S93589, 57, { &P19776,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.6; K900 Build/GRK39F)*/
const Device UA2462414 = {S93590, 133, { &P19776,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; K900 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2467965 = {S93591, 105, { &P15364,&P18144,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11*/
const Device UA2469392 = {S93592, 166, { &P16362,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; SPH-D710 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2474033 = {S93593, 101, { &P15364,&P17838,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.2pre) Gecko/X Ubuntu/9.10 (karmic) Namoroka/3.6.2pre*/
const Device UA2499957 = {S93594, 135, { &P15364,&P17816,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.1 (KHTML, like Gecko) Ubuntu/10.04 Chromium/21.0.1180.89 Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2503892 = {S93595, 99, { &P13051,&P17979,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/ZTE-TU880_TD/1.0 Linux/2.6.32 Android/2.2 Release/5.25.2011 Browser/AppleWebKit533.1*/
const Device UA2503916 = {S93596, 131, { &P18306,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N-05D Build/AX) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2505400 = {S93597, 136, { &P20740,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AVA757 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2520602 = {S93598, 68, { &P12968,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.4;Zh_cn;LT26i;720*1280;)UCWEB7.8.0.95/139/355*/
const Device UA2527058 = {S93599, 34, { &P14916,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SAMSUNG GT-C3303i)*/
const Device UA2527717 = {S93600, 134, { &P23467,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; iDx5 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2528221 = {S93601, 98, { &P14353,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s5250) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2529402 = {S93602, 99, { &P13206,&P17485,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonk530i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2541114 = {S93603, 77, { &P15364,&P18146,&P19772,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; SunOS i86pc; en-US; rv:1.8.1.12) Gecko/X SeaMonkey/1.1.8*/
const Device UA2557047 = {S93604, 65, { &P20169,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; N320; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2558510 = {S93605, 143, { &P20837,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IRIS_501 Build/LAVAIRIS501) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2578075 = {S93606, 103, { &P18254,&P5690,&P18245,&P18092 }};	/*Useragent - RssBandit/1.9.0.1002 (.NET CLR 2.0.50727.3643; WinNT 5.1.2600 Service Pack 3; http://www.rssbandit.org)*/
const Device UA2578258 = {S93607, 157, { &P18214,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2580558 = {S93608, 171, { &P13123,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2581047 = {S93609, 162, { &P17968,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; generic Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2594089 = {S93610, 142, { &P15006,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-W899 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2604127 = {S93611, 136, { &P21523,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SH930W Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2626936 = {S93612, 130, { &P14905,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P7100 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.37 Safari/537.22*/
const Device UA2634074 = {S93613, 130, { &P14905,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7100 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2652994 = {S93614, 141, { &P20289,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; HTC6435LVW Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2654185 = {S93615, 33, { &P15837,&P6909,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (NokiaX2-05/08.60)*/
const Device UA2659219 = {S93616, 157, { &P20134,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2659251 = {S93617, 154, { &P21094,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A789 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2659891 = {S93618, 169, { &P14993,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LT15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2662300 = {S93619, 173, { &P11950,&P5477,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; X10i Build/2.1.A.0.435) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2662734 = {S93620, 168, { &P19902,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530M Build/JDQ39; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2668830 = {S93621, 156, { &P20123,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; ZTE U795 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2675501 = {S93622, 152, { &P20541,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAP4500DUO Build/PrestigioPAP4500DUO) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2694288 = {S93623, 138, { &P23424,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TM-5204 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2694869 = {S93624, 131, { &P22135,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; SmartTab1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2698628 = {S93625, 82, { &P13014,&P22839,&P17394,&P18092 }};	/*Useragent - ZTE-E821S-Movistar/1.0 NetFront/4.2 QTV5.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2698965 = {S93626, 142, { &P21290,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; FreeTAB 9701 HD X1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2703255 = {S93627, 138, { &P18747,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A840S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2707337 = {S93628, 94, { &P17952,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; SHW-M110S) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2708274 = {S93629, 130, { &P22662,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Enjoy 71 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708958 = {S93630, 137, { &P15802,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Amaze 4G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709410 = {S93631, 131, { &P20527,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A6 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2710089 = {S93632, 138, { &P20244,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Elcano Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713107 = {S93633, 161, { &P16436,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I8150 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2717042 = {S93634, 136, { &P23550,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE T81 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2730746 = {S93635, 143, { &P21623,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 6 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2736864 = {S93636, 148, { &P21052,&P5629,&P22281,&P18092 }};	/*Useragent - OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2751703 = {S93637, 160, { &P21351,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI C8813 Build/HuaweiC8813) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2757883 = {S93638, 129, { &P15364,&P24184,&P24473,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/4.1; Linux 2.6.27.7-134.fc10.x86_64; X11; x86_64) KHTML/4.1.3 (like Gecko) Fedora/4.1.3-4.fc10*/
const Device UA2762993 = {S93639, 142, { &P20237,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HW-01E Build/HuaweiU9501L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2764355 = {S93640, 157, { &P18582,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia ion Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2765385 = {S93641, 137, { &P23164,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-7204 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2768779 = {S93642, 153, { &P18509,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-MS770 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2782845 = {S93643, 138, { &P20925,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT-AS45SE Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2792755 = {S93644, 88, { &P16957,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2; en-US; XCD35) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2795444 = {S93645, 93, { &P23003,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; TECNO_Q1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2795888 = {S93646, 103, { &P23123,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Intex_Aqua_Swadesh) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2796428 = {S93647, 148, { &P19996,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A15 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2797637 = {S93648, 137, { &P21488,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A860K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798403 = {S93649, 144, { &P23153,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ultrafone 701HD Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798982 = {S93650, 134, { &P21256,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Wilco Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799367 = {S93651, 174, { &P21857,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2843694 = {S93652, 128, { &P23638,&P24094,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40HX750; CC/VNM; vi) Presto/2.10.250 Version/11.60*/
const Device UA2847731 = {S93653, 159, { &P15158,&P5843,&P24673,&P18092 }};	/*Useragent - Nokia2710c-2/2.0 (06.13) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia2710c-2) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2847890 = {S93654, 168, { &P15147,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST15i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2851409 = {S93655, 88, { &P20518,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A10) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2854269 = {S93656, 99, { &P21842,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-e2202) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2857159 = {S93657, 155, { &P23296,&P18111,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Lenovo A590 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2861865 = {S93658, 155, { &P19911,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-E970 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3056526 = {S93659, 167, { &P19911,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-E970 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/geeb_att_us*/
const Device UA2864571 = {S93660, 164, { &P17172,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Transformer TF101 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA2865151 = {S93661, 169, { &P13526,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; myTouch 4G Build/JDQ39; LiquidSmooth-Liquid-JB-v2.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2865675 = {S93662, 131, { &P23759,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; RMD-757 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2867028 = {S93663, 136, { &P21987,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Gpad_G1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867229 = {S93664, 140, { &P21980,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; UBISLATE7C+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867780 = {S93665, 135, { &P20332,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax P275 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2868166 = {S93666, 147, { &P20143,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Edison 3G Build/1.1.11-1015 X-15:24) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2868695 = {S93667, 139, { &P24168,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAP4055DUO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869204 = {S93668, 156, { &P22209,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Titanium S5 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2877853 = {S93669, 136, { &P22809,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; P5_Quad Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879235 = {S93670, 137, { &P17324,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9001 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879449 = {S93671, 160, { &P18462,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC Desire X Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2891797 = {S93672, 172, { &P24744,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Infinix X450 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/33567*/
const Device UA2892120 = {S93673, 158, { &P18321,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2893951 = {S93674, 147, { &P18737,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; X909 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2934917 = {S93675, 150, { &P23464,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Spice Mi-424 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2935203 = {S93676, 94, { &P15575,&P18066,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; zh-CN; GT-I9100G) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile*/
const Device UA2937834 = {S93677, 156, { &P16436,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2946016 = {S93678, 144, { &P23799,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ONE TOUCH 4011X Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2955257 = {S93679, 93, { &P22980,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; TECNO_D5) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2957511 = {S93680, 150, { &P18735,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-R820 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2962566 = {S93681, 97, { &P20005,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A65) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2997852 = {S93682, 147, { &P18673,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3016888 = {S93683, 163, { &P18009,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTCEVODesign4G Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3021160 = {S93684, 130, { &P17168,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7310 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021371 = {S93685, 152, { &P21598,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Kem 452 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3057860 = {S93686, 157, { &P21598,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar_Touch_KEM_452_HDMI Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021672 = {S93687, 72, { &P22597,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile: SCH-I545; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA3022184 = {S93688, 131, { &P21501,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; P5 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022217 = {S93689, 137, { &P18447,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Xperia T Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022330 = {S93690, 129, { &P20752,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AN10CG3 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022840 = {S93691, 137, { &P22396,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5312 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023921 = {S93692, 138, { &P23156,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOLO A700 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3028310 = {S93693, 99, { &P14980,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-b7722) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3034289 = {S93694, 136, { &P18295,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; HTC One X Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3057756 = {S93695, 88, { &P21064,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A44) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3058087 = {S93696, 151, { &P24535,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Cloudfone Excite 402d Build/FSV08) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3058866 = {S93697, 99, { &P19862,&P18598,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Micromax_A110Q) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3059104 = {S93698, 161, { &P18504,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LS840 Build/ZVK.IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3059562 = {S93699, 152, { &P20280,&P18598,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; vivo X1St Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3061600 = {S93700, 143, { &P23223,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; PMP5588C Build/E8HD-ELITE-1280x768) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Safari/537.36*/
const Device UA3078543 = {S93701, 137, { &P24606,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E470f Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3086192 = {S93702, 138, { &P21544,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4;es-us; ZTE V887 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3093828 = {S93703, 127, { &P21222,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT1S0 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3094181 = {S93704, 89, { &P21472,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; A111) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3102181 = {S93705, 88, { &P20265,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A52) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA3103061 = {S93706, 154, { &P22860,&P18598,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Q700 Build/XOLOQ700) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3103166 = {S93707, 157, { &P20076,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT26w Build/6.1.A.2.55) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3121782 = {S93708, 157, { &P20076,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LT26w Build/6.1.A.2.45) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3110257 = {S93709, 156, { &P23776,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGP321 Build/10.3.1.A.0.244) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3110453 = {S93710, 127, { &P24458,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; KUNO4 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3110502 = {S93711, 132, { &P20146,&P18598,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.2; en-gb; GT-I9082 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3116532 = {S93712, 151, { &P19860,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A9+ Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3131321 = {S93713, 130, { &P17168,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7310 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA900 = {S93714, 89, { &P17612,&P5448,&P3430,&P18092 }};	/*Useragent - SAMSUNG-SCH-M710/(null)ID4 (compatible; MSIE 6.0; Windows CE; PPC) Opera 9.5 Presto/2.2.1*/
const Device UA68015 = {S93715, 100, { &P14763,&P5776,&P4759,&P18092 }};	/*Useragent - NokiaN72/5.0706.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.6.1.82/27/354*/
const Device UA5311 = {S93716, 75, { &P12243,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_Snap_S521 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA10058 = {S93717, 107, { &P16444,&P5448,&P4808,&P18092 }};	/*Useragent - HTC-ST6356/2.31.502.4 (66023) Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA14223 = {S93718, 71, { &P13798,&P5448,&P17173,&P18092 }};	/*Useragent - HTC_X7500 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA56592 = {S93719, 129, { &P13798,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) PPC; 640x480; HTC_X7500/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA18446 = {S93720, 165, { &P14058,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Android 2.1-update1;zh-cn;ZTE-C_N600/ZTE-C_N600V1.0.0B03;240*320;CTC/2.0) AppleWebkit/530.17(KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA21299 = {S93721, 177, { &P14058,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; ZTE-C_N600+/ZTE-C_N600PLUSV1.0.0B02; 240*320; CTC/2.0) AppleWebkit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA53902 = {S93722, 165, { &P14058,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U;Android 2.1-update1;zh-cn;ZTE-C_N600/ZTE-C_N600V1.0.0B02;240*320;CTC/2.0) AppleWebkit/530.17(KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2022196 = {S93723, 132, { &P14058,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; ZTE-C N600 Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA22425 = {S93724, 46, { &P14999,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP270V/V10g Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA42029 = {S93725, 45, { &P14999,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP270/V10g Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA87324 = {S93726, 45, { &P14999,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP270-V10a Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA23720 = {S93727, 62, { &P15620,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia3555b/2.0 (05.07) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA20996 = {S93728, 129, { &P16054,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC_P3600/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21041 = {S93729, 147, { &P16054,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_Trinity_P3600 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC_P3600/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA39912 = {S93730, 130, { &P16054,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC_P3600i/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87356 = {S93731, 73, { &P16054,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_Trinity Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA88747 = {S93732, 71, { &P16054,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_P3600 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA21395 = {S93733, 64, { &P11991,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; LG_GM750/1.0)*/
const Device UA23013 = {S93734, 101, { &P13816,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E360/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA24377 = {S93735, 53, { &P12157,&P17483,&P17660,&P18092 }};	/*Useragent - LG/KU970/v10a Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA26150 = {S93736, 13, { &P12157,&P17483,&P17660,&P18092 }};	/*Useragent - LG-KU970/v1.0*/
const Device UA25569 = {S93737, 71, { &P16216,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN71-1/2.0613 Series60/3.0  Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA26373 = {S93738, 34, { &P13590,&P17483,&P17470,&P18092 }};	/*Useragent - LG9600/1.0 TELECA/Q7.0-1.6 MMP/2.0*/
const Device UA33383 = {S93739, 44, { &P12688,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-706/1.0 ObigoInternetBrowser/Q03C*/
const Device UA61345 = {S93740, 45, { &P12688,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-706A/1.0 ObigoInternetBrowser/Q03C*/
const Device UA51093 = {S93741, 68, { &P13933,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V235/0A.30.6CR MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA50403 = {S93742, 34, { &P13977,&P5890,&P4737,&P18092 }};	/*Useragent - NokiaN73/UC Browser7.2.2.51/28/351*/
const Device UA52784 = {S93743, 94, { &P19968,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; LG; GW910)*/
const Device UA54531 = {S93744, 76, { &P14235,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonZ320i/R1F003 Browser/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87643 = {S93745, 76, { &P14235,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonZ320a/R1C003 Browser/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA56336 = {S93746, 98, { &P14697,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Schubert)*/
const Device UA59230 = {S93747, 167, { &P14058,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; ZTE-C_N600+/ZTE-C_N600PLUSV1.0.0B03; 240*320; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020064 = {S93748, 125, { &P14058,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; ZTE-C N600+ Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA61153 = {S93749, 42, { &P12646,&P22832,&P17470,&P18092 }};	/*Useragent - VIDEOCON V-1448/WAP2.0 Profile/MT6253/QVGA*/
const Device UA67294 = {S93750, 29, { &P14358,&P5890,&P24645,&P18092 }};	/*Useragent - NokiaE63/UCWEB7.5.0.66/28/999*/
const Device UA2328991 = {S93751, 99, { &P14669,&P5843,&P4777,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-00/UC Browser7.7.1.88/69/444*/
const Device UA68696 = {S93752, 110, { &P14345,&P16000,&P4777,&P18092 }};	/*Useragent - NokiaN-GageQD/1.0 SymbianOS/6.1 Series60/1.2 Profile/MIDP-1.0 Configuration/CLDC-1.0/UC Browser7.7.1.88/26/444*/
const Device UA68817 = {S93753, 34, { &P13554,&P5890,&P4777,&P18092 }};	/*Useragent - NOKIAN79/UC Browser7.7.1.88/28/444*/
const Device UA2804744 = {S93754, 36, { &P13554,&P5890,&P4777,&P18092 }};	/*Useragent - nokian79-1/UC Browser7.7.1.88/69/444*/
const Device UA69313 = {S93755, 36, { &P13552,&P5129,&P4779,&P18092 }};	/*Useragent - NokiaC6-00/UC Browser7.8.0.95/50/355*/
const Device UA477690 = {S93756, 58, { &P17878,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3I UP.Browser/6.2_7.2.7.1.K.1.5.1.120 (GUI) MMP/2.0*/
const Device UA132643 = {S93757, 54, { &P17565,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SN3K UP.Browser/6.2_7.2.7.1.K.3.352 (GUI) MMP/2.0*/
const Device UA211961 = {S93758, 237, { &P12280,&P17090,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA317574 = {S93759, 237, { &P12280,&P17090,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA217066 = {S93760, 137, { &P12681,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Nexus S 4G Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA712495 = {S93761, 137, { &P12681,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus S 4G Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA327466 = {S93762, 89, { &P15364,&P17285,&P21897,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/12.0.742.122)*/
const Device UA668641 = {S93763, 96, { &P15364,&P17285,&P21897,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/12.0.742.100)*/
const Device UA1890061 = {S93764, 127, { &P15364,&P17285,&P21897,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/12.0.742.122) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA2139417 = {S93765, 134, { &P15364,&P17285,&P21897,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/12.0.742.112) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30*/
const Device UA2288685 = {S93766, 127, { &P15364,&P17285,&P21897,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/12.0.742.100) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.100 Safari/534.30*/
const Device UA2290844 = {S93767, 127, { &P15364,&P17285,&P21897,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/12.0.742.112) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30*/
const Device UA713861 = {S93768, 43, { &P12137,&P17484,&P17338,&P18092 }};	/*Useragent - samsung-gt-s5620/UC Browser7.9.0.102/69/444*/
const Device UA628002 = {S93769, 41, { &P12061,&P5890,&P24665,&P18092 }};	/*Useragent - NOKIAE65/UC Browser8.0.4.121/28/352/UCWEB*/
const Device UA425396 = {S93770, 48, { &P17595,&P17779,&P20782,&P18092 }};	/*Useragent - MQQBrowser/31 (iOS; U; CPU like Mac OS X; zh-cn)*/
const Device UA463435 = {S93771, 141, { &P15364,&P17285,&P22398,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Comodo_Dragon/4.1.1.12 Chrome/4.1.249.1042 Safari/532.5*/
const Device UA466014 = {S93772, 74, { &P17595,&P18585,&P17316,&P18092 }};	/*Useragent - Safari/6531.21.10 CFNetwork/454.9.4 Darwin/10.3.2 (i386) (MacBookPro7%2C1)*/
const Device UA752573 = {S93773, 70, { &P17595,&P18585,&P17316,&P18092 }};	/*Useragent - Safari/6531.22.7 CFNetwork/454.9.4 Darwin/10.3.2 (i386) (MacBook7%2C1)*/
const Device UA585407 = {S93774, 76, { &P15364,&P5690,&P18192,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:9.0) Gecko/X Thunderbird/9.0 Lightning/1.1.1*/
const Device UA2313274 = {S93775, 69, { &P15364,&P5690,&P18192,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:9.0) Gecko/X Thunderbird/9.0.1*/
const Device UA613137 = {S93776, 140, { &P15767,&P17090,&P20775,&P18092 }};	/*Useragent - MQQBrowser/2.4 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3*/
const Device UA687248 = {S93777, 140, { &P15767,&P17090,&P20775,&P18092 }};	/*Useragent - MQQBrowser/2.4 Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3*/
const Device UA817600 = {S93778, 140, { &P15767,&P17090,&P20775,&P18092 }};	/*Useragent - MQQBrowser/2.4 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 Safari/7534.48.3*/
const Device UA649449 = {S93779, 88, { &P15364,&P18140,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.6) Gecko/X Firefox/2.0.0.6 (Linux Mint)*/
const Device UA675066 = {S93780, 136, { &P17346,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.0.2 Mobile/9A405 Safari/6533.18.5*/
const Device UA2949213 = {S93781, 148, { &P17346,&P17146,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5*/
const Device UA682565 = {S93782, 54, { &P14033,&P5890,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 (NokiaC5-03;SymbianOS/9.1 Series60/3.0)*/
const Device UA689283 = {S93783, 79, { &P15364,&P5690,&P22339,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:9.0.1) Gecko/X Firefox/9.0.1 CometBrowser/9.0.1*/
const Device UA2312399 = {S93784, 83, { &P15364,&P5690,&P22339,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:9.0.1) Gecko/X Firefox/9.0.1 CometBird/9.0.1*/
const Device UA752884 = {S93785, 127, { &P13772,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; C8500 Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2025072 = {S93786, 132, { &P13772,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; C8500 Build/ERE27) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2032343 = {S93787, 132, { &P13772,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla 5.0 (Linux; U; Android 2.1-update1; zh-cn; C8500 Build ERE27) UC AppleWebKit 534.31 (KHTML, like Gecko) Mobile Safari 534.31*/
const Device UA786382 = {S93788, 64, { &P15364,&P17285,&P18101,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:12.0) Gecko/X Firefox/12.0 slurp*/
const Device UA2328511 = {S93789, 71, { &P15364,&P17285,&P18101,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:12.0) Gecko/X Firefox/12.0 slurp*/
const Device UA798771 = {S93790, 113, { &P15364,&P6596,&P17241,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; CrOS i686 9.10.0; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Chrome/4.0.253.0 Safari/532.5*/
const Device UA837007 = {S93791, 136, { &P13505,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MB525 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2025034 = {S93792, 136, { &P13505,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB525 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2722335 = {S93793, 58, { &P13505,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; MB525 Build/IMM76L)*/
const Device UA925846 = {S93794, 102, { &P16246,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i677)*/
const Device UA925876 = {S93795, 97, { &P18561,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG-E740h)*/
const Device UA925897 = {S93796, 96, { &P18561,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; LG-E740)*/
const Device UA1850505 = {S93797, 143, { &P16140,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; Spice_Mi300 Build/DONUT) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA1883631 = {S93798, 63, { &P15364,&P18110,&P20411,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; en) Presto/2.2.15 Version/10.20*/
const Device UA1899478 = {S93799, 92, { &P17595,&P17297,&P22311,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:6.0.2) Gecko/X Firefox/6.0.2 SeaMonkey/2.3.3*/
const Device UA1939840 = {S93800, 98, { &P11667,&P5768,&P17347,&P18092 }};	/*Useragent - Nokia6085/2.0 (06.00) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6085/UC Browser8.0.3.107/69/444*/
const Device UA1941311 = {S93801, 114, { &P13107,&P5043,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-06/06.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA1972327 = {S93802, 81, { &P12236,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA6700s)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA1974384 = {S93803, 171, { &P13157,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation XE with Beats Audio Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050096 = {S93804, 172, { &P13157,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2001291 = {S93805, 122, { &P15364,&P16110,&P22436,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 Comodo_Dragon/21.1.1.0*/
const Device UA2402780 = {S93806, 129, { &P15364,&P16110,&P22436,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1 Comodo_Dragon/21.1.1.0*/
const Device UA2028575 = {S93807, 182, { &P22081,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; eZee'Tab7c Build/V0.15-Storex-V971-114.GINGERBREAD.eng.chenxiang.X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026500 = {S93808, 143, { &P12765,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A100 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2032316 = {S93809, 131, { &P21052,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; th-th; A9 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2150235 = {S93810, 80, { &P13078,&P18110,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 (Linux; U; 2.2.2; zh-cn; ZTE-U V880 Build/UNI_CN_V880 2.1;480*800)*/
const Device UA2221921 = {S93811, 99, { &P18267,&P5043,&P4777,&P18092 }};	/*Useragent - NokiaX2-02/2.0 (10.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-02/UC Browser7.7.1.88/69/444*/
const Device UA2299815 = {S93812, 35, { &P14545,&P17483,&P17338,&P18092 }};	/*Useragent - lg-x330t/UC Browser7.9.0.102/69/352*/
const Device UA2318886 = {S93813, 133, { &P21203,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SX-SP715A Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2335397 = {S93814, 141, { &P24548,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GOCLEVER TAB A971 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2336768 = {S93815, 146, { &P12513,&P17485,&P24670,&P18092 }};	/*Useragent - SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonw20) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2863121 = {S93816, 175, { &P13054,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (ADR6425LVW, 42, 1.9.1)*/
const Device UA2413701 = {S93817, 132, { &P23896,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; S5 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2420756 = {S93818, 125, { &P17108,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC-A9192/1.0; en-gb) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2550839 = {S93819, 125, { &P17108,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HTC-A9192/1.0; en-us) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2442355 = {S93820, 78, { &P13602,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; gt-s5233w) UCBrowser8.2.1.144/69/352/UCWEB Mobile*/
const Device UA2443988 = {S93821, 175, { &P15798,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2953344 = {S93822, 179, { &P15798,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC_Explorer_A310e Build/GRJ90) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2446168 = {S93823, 170, { &P18382,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Desire C Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2466812 = {S93824, 136, { &P20966,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; T-01D Build/V09R40A) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2478319 = {S93825, 252, { &P14826,&P5843,&P18520,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.5.0.185/83/352*/
const Device UA2495344 = {S93826, 54, { &P12088,&P5890,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8 (NokiaX6-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA2502854 = {S93827, 65, { &P18666,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;A25;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA2503536 = {S93828, 135, { &P21088,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Stealth 9.7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2529819 = {S93829, 161, { &P19996,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A15 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2549095 = {S93830, 26, { &P20195,&P17483,&P21758,&P18092 }};	/*Useragent - iBrowser/Mini2.5 (LG-T375)*/
const Device UA2557914 = {S93831, 159, { &P20183,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2558288 = {S93832, 164, { &P20492,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cruize W280 Build/Cherry_Cruize_W280_2012.12.30) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2560767 = {S93833, 100, { &P15302,&P17485,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonwt13i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2779020 = {S93834, 97, { &P15302,&P17485,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; fa; sonyericssonwt13i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2577790 = {S93835, 128, { &P23658,&P24094,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL55HX853; CC/GBR; en) Presto/2.10.250 Version/11.60*/
const Device UA2578778 = {S93836, 153, { &P18714,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V889D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2580474 = {S93837, 160, { &P21052,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2596947 = {S93838, 147, { &P21640,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Samsung GT-S9081 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2598684 = {S93839, 161, { &P20265,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A52 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2601597 = {S93840, 138, { &P19902,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R530M Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2602988 = {S93841, 141, { &P22021,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KHPHN05ANDA Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2605735 = {S93842, 137, { &P14969,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Desire S Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2618708 = {S93843, 149, { &P13452,&P5096,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2621553 = {S93844, 151, { &P13452,&P5096,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2623260 = {S93845, 151, { &P13452,&P5096,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2635524 = {S93846, 163, { &P13452,&P5096,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android HDVs_Android2.2.1; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2635523 = {S93847, 92, { &P13452,&P5096,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Adr (Linux; U; HDVs_Android2.2.1; zh-cn; GT-P1000 Build/HDVs-V2-1.4;600*1024)*/
const Device UA2633782 = {S93848, 131, { &P20523,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3105 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2638745 = {S93849, 134, { &P15364,&P16110,&P22448,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Iron/10.0.650.1 Chrome/10.0.650.1 Safari/534.16*/
const Device UA2651269 = {S93850, 72, { &P21432,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; ZTE_N788; 320*480) UCWEB7.9.3.103/139/32846*/
const Device UA2651964 = {S93851, 138, { &P18072,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; SHV-E120S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2652048 = {S93852, 145, { &P22515,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; Andy-3G-4.0 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2655897 = {S93853, 53, { &P15364,&P6704,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (Windows 98) Presto/2.12.388 Version/12.11*/
const Device UA2658609 = {S93854, 154, { &P20096,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7568 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2666717 = {S93855, 133, { &P19996,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A15 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667244 = {S93856, 141, { &P20131,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Thrill 430x Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667691 = {S93857, 157, { &P21021,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328t Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2667874 = {S93858, 129, { &P20153,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AN7HG3 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2676879 = {S93859, 145, { &P20104,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 201M Build/7.7.1Q-164_SMJ-165) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2683527 = {S93860, 142, { &P11793,&P5768,&P24668,&P18092 }};	/*Useragent - Nokia3500c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia3500c) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2686879 = {S93861, 155, { &P18072,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; SGV-E120L Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2696619 = {S93862, 126, { &P14202,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; X8 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2707000 = {S93863, 145, { &P20000,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ607 Build/7.7.1-122-4-FLEM-WIFI-LA) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2707393 = {S93864, 133, { &P20584,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Z120 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708261 = {S93865, 135, { &P21446,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SH837W Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708421 = {S93866, 138, { &P20925,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT-AS45SE Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708664 = {S93867, 139, { &P15354,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST17i Build/4.1.B.0.587) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708719 = {S93868, 141, { &P12798,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xperia Arc S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709760 = {S93869, 132, { &P19996,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A15 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2710831 = {S93870, 133, { &P23086,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Fusion Bolt Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2713253 = {S93871, 141, { &P20261,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Philips W832 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2726589 = {S93872, 161, { &P17968,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MID Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2833621 = {S93873, 161, { &P17968,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2726950 = {S93874, 148, { &P18702,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N8013 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2729047 = {S93875, 143, { &P23432,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cloudpad 970g Build/FSV05) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2730837 = {S93876, 135, { &P23454,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-au; KATBL08A16E Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2733034 = {S93877, 138, { &P16436,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8150 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2734634 = {S93878, 130, { &P14908,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6210 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2734849 = {S93879, 129, { &P23767,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-7240 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2751575 = {S93880, 90, { &P23919,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; R800i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2764417 = {S93881, 140, { &P23947,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Alpha GTR Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2765475 = {S93882, 140, { &P23325,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-en; MAPAN MX88R Build/MAPAN MX88R) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2766436 = {S93883, 110, { &P16745,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; T-Mobile_myTouch_3G_Slide) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2769435 = {S93884, 135, { &P20416,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; F-04E Build/V14R31C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2769798 = {S93885, 141, { &P20261,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Philips W832 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2774573 = {S93886, 85, { &P15364,&P16110,&P19773,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.1.19) Gecko/X SeaMonkey/2.0.14*/
const Device UA2779616 = {S93887, 60, { &P14017,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; LG-P990 Build/IMM76L)*/
const Device UA2782842 = {S93888, 149, { &P20964,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Turkcell Maxi Plus 5 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2786063 = {S93889, 39, { &P15364,&P18145,&P4811,&P18092 }};	/*Useragent - Opera/9.01 (X11; FreeBSD 6 i386; U; en)*/
const Device UA2794363 = {S93890, 184, { &P23199,&P18111,&P20787,&P18092 }};	/*Useragent - MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad7295 Build/4.1.015.X.7295; 540*960; CUCC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2797304 = {S93891, 88, { &P17595,&P17295,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:21.0) Gecko/X Firefox/21.0 TenFourFox/7450*/
const Device UA2798853 = {S93892, 135, { &P23889,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PAD703 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2989910 = {S93893, 174, { &P17308,&P5629,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (DROIDX, 42, 1.9.1)*/
const Device UA3118048 = {S93894, 67, { &P14090,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2; zh-cn; GT-I5801; 240*400) UCWEB7.9.0.94/139/444*/
const Device UA2846042 = {S93895, 61, { &P23571,&P22828,&P18246,&P18092 }};	/*Useragent - ZEN M2s CODE/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA2848369 = {S93896, 138, { &P23338,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; miTab BROOKLYN Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2854757 = {S93897, 163, { &P21911,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; AMAZE S200 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2859165 = {S93898, 94, { &P20180,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; XOLO_A800) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2863099 = {S93899, 170, { &P20146,&P18111,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9082 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2869280 = {S93900, 150, { &P20107,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5580C Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2877168 = {S93901, 129, { &P21990,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; chagall Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2878833 = {S93902, 137, { &P14967,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-P500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2879308 = {S93903, 160, { &P22867,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5303 Build/12.0.A.1.257) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2879309 = {S93904, 138, { &P23559,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AMOI N821 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879459 = {S93905, 131, { &P21415,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Quantum 7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2889834 = {S93906, 96, { &P15539,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; HTC_Glacier) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2892658 = {S93907, 155, { &P23715,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Starmobile Engage7 3G Build/ICS_IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2935301 = {S93908, 165, { &P18715,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID8048 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444*/
const Device UA2937808 = {S93909, 156, { &P14959,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2938260 = {S93910, 147, { &P21112,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; X907 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2938275 = {S93911, 134, { &P24160,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; SPC Niobe7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2944651 = {S93912, 163, { &P22582,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2956366 = {S93913, 130, { &P16332,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; hr-hr; Galaxy Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2956998 = {S93914, 154, { &P23556,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A66 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2957320 = {S93915, 160, { &P18552,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2960482 = {S93916, 150, { &P21052,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A9 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2962957 = {S93917, 95, { &P20004,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Karbonn A2) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2987238 = {S93918, 155, { &P23743,&P18111,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; MITO t510 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2990765 = {S93919, 153, { &P24547,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPHS on Hsdroid Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2993639 = {S93920, 137, { &P22954,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E455f Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2998023 = {S93921, 148, { &P19902,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-R530M Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2998058 = {S93922, 146, { &P21891,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; WALTON_Primo-G1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998221 = {S93923, 89, { &P21472,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; A111) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2998791 = {S93924, 142, { &P21119,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Softwinern01 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2999355 = {S93925, 154, { &P24073,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; ALCATEL ONE TOUCH 6033M Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.1 Mobile Safari/534.30*/
const Device UA3000329 = {S93926, 147, { &P15767,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ipad mini Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3004473 = {S93927, 162, { &P22209,&P18111,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Titanium S5 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300*/
const Device UA3004781 = {S93928, 148, { &P23771,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8190N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3007083 = {S93929, 133, { &P22607,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-gb; GT-S5303 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3013616 = {S93930, 136, { &P18324,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.7; en-us; LG-VM696 Build/ZV5.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3015404 = {S93931, 133, { &P19847,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-gb; GT-S6012 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3015472 = {S93932, 94, { &P14367,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6710s) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3016390 = {S93933, 101, { &P12707,&P5055,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia701) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3021035 = {S93934, 137, { &P21949,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F100L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021242 = {S93935, 134, { &P21104,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; X507S Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022489 = {S93936, 137, { &P23647,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Cynus T5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022571 = {S93937, 136, { &P23163,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I959 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023829 = {S93938, 144, { &P20168,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Maxwell Plus Build/1.0.3 X-14:07) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3027011 = {S93939, 138, { &P18465,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Amazon Kindle Fire Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3039383 = {S93940, 129, { &P22873,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N5120 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3057989 = {S93941, 89, { &P20169,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; N320) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3058255 = {S93942, 74, { &P18113,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile: DROID RAZR; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA3058813 = {S93943, 165, { &P18466,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Amazon Kindle Fire HD Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.2/tate*/
const Device UA3072857 = {S93944, 136, { &P22950,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E977 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3079499 = {S93945, 94, { &P20069,&P22784,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx78) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3083505 = {S93946, 99, { &P19862,&P18598,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Micromax_A110Q) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3093026 = {S93947, 169, { &P15649,&P5096,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5570 Build/FROYO) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3093095 = {S93948, 94, { &P13648,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaE7-00) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3093591 = {S93949, 99, { &P14031,&P5477,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT-I5503) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3108433 = {S93950, 170, { &P22609,&P18111,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6312 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3109433 = {S93951, 150, { &P18437,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3110396 = {S93952, 137, { &P24608,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E467f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111852 = {S93953, 137, { &P18636,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS930 4G Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3115433 = {S93954, 157, { &P21628,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3122753 = {S93955, 144, { &P23756,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SM-G730A Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3129701 = {S93956, 160, { &P14708,&P5204,&P24680,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia5130c-2) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3131310 = {S93957, 136, { &P21792,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9200 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131816 = {S93958, 137, { &P24097,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919N Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132610 = {S93959, 140, { &P12841,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC PH39100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA66998 = {S93960, 61, { &P14743,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA4876 = {S93961, 46, { &P12713,&P17484,&P5001,&P18092 }};	/*Useragent - SCH-U700/1.0 NetFront/3.0.22.2.7 (GUI) MMP/2.0*/
const Device UA13038 = {S93962, 87, { &P13749,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_E200*/
const Device UA40782 = {S93963, 97, { &P13749,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_E200 1.049c.00*/
const Device UA40783 = {S93964, 96, { &P13749,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_E200 2.001.00*/
const Device UA43801 = {S93965, 96, { &P13749,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_E200 2.002.00*/
const Device UA56364 = {S93966, 96, { &P13749,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_E200 2.004.00*/
const Device UA25504 = {S93967, 61, { &P14414,&P17483,&P17602,&P18092 }};	/*Useragent - LG-TU720 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA43836 = {S93968, 83, { &P13753,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-F106(Vinaphone)/1.0 NetFront/3.5 QTV2.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA44994 = {S93969, 91, { &P13848,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Snap_S526 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA48991 = {S93970, 83, { &P13883,&P22846,&P4223,&P18092 }};	/*Useragent - Lemon_W100/1.0SH NetFront/3.5 SMM-MMS/1.2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA52058 = {S93971, 32, { &P13435,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH03A(c100;TB;W24H16)*/
const Device UA54839 = {S93972, 41, { &P14267,&P17017,&P3625,&P18092 }};	/*Useragent - SPH-M310 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA55144 = {S93973, 39, { &P14289,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP320 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA55217 = {S93974, 73, { &P14299,&P17483,&P4970,&P18092 }};	/*Useragent - LG-LX550 AU-MIC-LX550/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61235 = {S93975, 68, { &P12662,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-T7/1.0 ACS-NF/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA69448 = {S93976, 99, { &P14540,&P5843,&P4779,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-01/UC Browser7.8.0.95/70/355*/
const Device UA90275 = {S93977, 31, { &P17553,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P02C(c500;TB;W24H16)*/
const Device UA236504 = {S93978, 117, { &P15364,&P18110,&P22451,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.0 Chrome/13.0.800.0 Safari/535.1*/
const Device UA238539 = {S93979, 115, { &P15364,&P18110,&P22451,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.0 Chrome/13.0.800.0 Safari/535.1*/
const Device UA339364 = {S93980, 118, { &P15364,&P18110,&P22450,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.30 (KHTML, like Gecko) Iron/12.0.750.0 Chrome/12.0.750.0 Safari/534.30*/
const Device UA341494 = {S93981, 116, { &P15364,&P18110,&P22450,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.30 (KHTML, like Gecko) Iron/12.0.750.0 Chrome/12.0.750.0 Safari/534.30*/
const Device UA660499 = {S93982, 141, { &P13526,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; HTC myTouch 4G Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA399142 = {S93983, 81, { &P15795,&P7133,&P17200,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; NOKIA; Lumia 710)*/
const Device UA419677 = {S93984, 30, { &P15364,&P6704,&P22484,&P18092 }};	/*Useragent - Opera/9.10 (Windows 98; U; en)*/
const Device UA2807530 = {S93985, 63, { &P12141,&P7149,&P15830,&P18092 }};	/*Useragent - BlackBerry8900/5.2.0.96 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA691524 = {S93986, 170, { &P13815,&P5096,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA3100081 = {S93987, 174, { &P13815,&P5096,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; HTC_Wildfire Build/FRG83D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA633092 = {S93988, 133, { &P17308,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; DROIDX Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022735 = {S93989, 133, { &P17308,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; DROIDX Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA633942 = {S93990, 48, { &P17595,&P17779,&P20784,&P18092 }};	/*Useragent - MQQBrowser/33 (iOS; U; CPU like Mac OS X; zh-cn)*/
const Device UA675130 = {S93991, 117, { &P15364,&P6596,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; CrOS i686 0.0.0; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.151 Safari/534.16*/
const Device UA1860261 = {S93992, 118, { &P15364,&P6596,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; CrOS i686 0.10.146; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.642.2 Safari/534.16*/
const Device UA675212 = {S93993, 131, { &P17595,&P17295,&P22453,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.1 Chrome/15.0.900.2 Safari/535.2*/
const Device UA693796 = {S93994, 112, { &P15364,&P18131,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.6) Gecko/X Linux Mint/4.0 (Daryna) Firefox/2.0.0.6 (Linux Mint)*/
const Device UA726599 = {S93995, 67, { &P15364,&P5690,&P18368,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; WOW64; rv:8.0) Gecko/X Thunderbird/8.0*/
const Device UA2147518 = {S93996, 120, { &P15364,&P5690,&P18368,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:8.0) Gecko/X Thunderbird/8.0 OracleBeehiveExtension/1.0.0.4-OracleInternal Lightning/1.0*/
const Device UA756355 = {S93997, 53, { &P18088,&P17017,&P24665,&P18092 }};	/*Useragent - IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB8.0.4.105/41/800*/
const Device UA784991 = {S93998, 53, { &P18088,&P17017,&P24665,&P18092 }};	/*Useragent - IUC(U;iOS 4.2.1;Zh-cn;320*480;)/UCWEB8.0.4.105/41/800*/
const Device UA800685 = {S93999, 51, { &P18088,&P17017,&P24665,&P18092 }};	/*Useragent - IUC(U;iOS 4.1;Zh-cn;320*480;)/UCWEB8.0.4.105/41/800*/
const Device UA764233 = {S94000, 115, { &P15879,&P5890,&P22720,&P18092 }};	/*Useragent - Mozilla/5.0 (MeeGo; NokiaN950-00/00) AppleWebKit/534.13 (KHTML, like Gecko) NokiaBrowser/8.5.0 Mobile Safari/534.13*/
const Device UA911746 = {S94001, 131, { &P15364,&P16110,&P22432,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11 Comodo_Dragon/20.0.1.0*/
const Device UA912053 = {S94002, 124, { &P15364,&P16110,&P22432,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.47 Safari/536.11 Comodo_Dragon/20.0.1.0*/
const Device UA912477 = {S94003, 27, { &P18254,&P17017,&P18418,&P18092 }};	/*Useragent - Mozilla/5.0 NewsFox/1.0.8.4*/
const Device UA924478 = {S94004, 131, { &P15364,&P16110,&P22433,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11 Comodo_Dragon/20.1.1.0*/
const Device UA924892 = {S94005, 124, { &P15364,&P16110,&P22433,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11 Comodo_Dragon/20.1.1.0*/
const Device UA925712 = {S94006, 102, { &P16463,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; SAMSUNG; GT-I8350)*/
const Device UA925839 = {S94007, 103, { &P16463,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; SAMSUNG; GT-I8350T)*/
const Device UA925895 = {S94008, 109, { &P16463,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; SAMSUNG; Omnia W; Orange)*/
const Device UA925720 = {S94009, 64, { &P18073,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;T9295;7.10;7720)*/
const Device UA925737 = {S94010, 64, { &P18073,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;T9295;7.10;8107)*/
const Device UA925810 = {S94011, 72, { &P14630,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i917R;7.10;8107)*/
const Device UA925913 = {S94012, 72, { &P14630,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i917R;7.10;7720)*/
const Device UA2520703 = {S94013, 72, { &P14630,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i917R;7.10;8773)*/
const Device UA1940539 = {S94014, 61, { &P18477,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX333/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA1967661 = {S94015, 131, { &P20594,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U10GT_A Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1974633 = {S94016, 54, { &P21008,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.5; N-01D Build/AX)*/
const Device UA2024814 = {S94017, 130, { &P21008,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; N-01D Build/AX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1989864 = {S94018, 85, { &P15198,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s3650) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2332283 = {S94019, 85, { &P15198,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s3650) UCBrowser8.2.0.132/69/352/UCWEB Mobile*/
const Device UA2610837 = {S94020, 171, { &P13452,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2617385 = {S94021, 172, { &P13452,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-P1000T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2623852 = {S94022, 171, { &P13452,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352*/
const Device UA2625000 = {S94023, 172, { &P13452,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-P1000N Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352*/
const Device UA2020141 = {S94024, 172, { &P20370,&P5629,&P20771,&P18092 }};	/*Useragent - MQQBrowser/20 Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ZTE-C_N760/N760V1.0.0B03; 320*480; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022996 = {S94025, 172, { &P20370,&P5629,&P20771,&P18092 }};	/*Useragent - MQQBrowser/20 Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE-C_N760/N760V1.0.0B04; 320*480; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022100 = {S94026, 151, { &P15483,&P5629,&P20771,&P18092 }};	/*Useragent - MQQBrowser/20 Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC A810e Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023609 = {S94027, 137, { &P16954,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Dell Venue Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026097 = {S94028, 133, { &P24456,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-us; Tuna Build/IFK77E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.67*/
const Device UA2026118 = {S94029, 133, { &P22127,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 3GW101 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2743744 = {S94030, 126, { &P22127,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 3GW101 Build/MIUI) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2026554 = {S94031, 163, { &P18317,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Transformer Prime TF300T Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2813086 = {S94032, 161, { &P18317,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Transformer Pad TF300T Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2028259 = {S94033, 155, { &P15649,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2339753 = {S94034, 156, { &P15649,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570I Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034047 = {S94035, 139, { &P22079,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; eZee'Tab10c Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2133314 = {S94036, 97, { &P15364,&P7127,&P22483,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.8.0.5) Gecko/X Firefox/1.5.0.5 Flock/0.7.4.1*/
const Device UA2279220 = {S94037, 73, { &P15364,&P7133,&P22181,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.2a1pre) Gecko/X Firefox/4.2a1pre*/
const Device UA2683181 = {S94038, 79, { &P15364,&P7133,&P22181,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.2a1pre; Gecko/X Firefox/4.2a1pre)*/
const Device UA2330262 = {S94039, 86, { &P20126,&P17485,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; sonyericssonk800i) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2339568 = {S94040, 157, { &P14959,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2361024 = {S94041, 137, { &P15364,&P19822,&P18629,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.17 (KHTML, like Gecko) Ubuntu/12.10 Chromium/24.0.1312.2 Chrome/24.0.1312.2 Safari/537.17*/
const Device UA2372839 = {S94042, 81, { &P15364,&P5690,&P17369,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/4.4.0.60 (Windows XP)/886; U; en) Presto/2.4.15*/
const Device UA2377107 = {S94043, 138, { &P18643,&P5043,&P24667,&P18092 }};	/*Useragent - Nokia311/5.0 (03.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia311) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2384476 = {S94044, 146, { &P22065,&P18066,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SmartTab2 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2408904 = {S94045, 30, { &P18088,&P17017,&P4737,&P18092 }};	/*Useragent - maui/UC Browser7.2.2.51/69/352*/
const Device UA2409342 = {S94046, 108, { &P15364,&P17350,&P18210,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; ; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.1.0.5001 Safari/532.4*/
const Device UA2469876 = {S94047, 90, { &P15518,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; gt-s5230) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2487021 = {S94048, 90, { &P15518,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; gt-s5230) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2847688 = {S94049, 95, { &P15518,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-s5230) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2475243 = {S94050, 76, { &P15364,&P16110,&P22538,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:12.0) Gecko/X Firefox/12.0 Kylo/1.1.1*/
const Device UA2482970 = {S94051, 158, { &P12505,&P17483,&P24673,&P18092 }};	/*Useragent - LG-GX500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-gx500) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2503836 = {S94052, 160, { &P20004,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2506792 = {S94053, 140, { &P23869,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Advance TV Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2510377 = {S94054, 134, { &P17595,&P17297,&P19995,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/6.0.2 Safari/7536.26.17 Sleipnir/4.0.1*/
const Device UA2511338 = {S94055, 99, { &P15197,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-b5310) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2525580 = {S94056, 133, { &P23746,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; O+ TABLET Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2530183 = {S94057, 136, { &P20956,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; FPT HD Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2546137 = {S94058, 152, { &P21435,&P5629,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V788D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2549211 = {S94059, 129, { &P21375,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; P711F Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2553163 = {S94060, 128, { &P23632,&P24094,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL55HX750; CC/USA; en) Presto/2.10.250 Version/11.60*/
const Device UA2554771 = {S94061, 170, { &P21874,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android GFIVE-4.0; en-us; GFIVE Aurora(A79) Build/Aurora-A79-V1.0-2012.09.29) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2555295 = {S94062, 129, { &P20216,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M1062 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2586730 = {S94063, 60, { &P16573,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxQ66/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA2593358 = {S94064, 185, { &P12513,&P17485,&P24671,&P18092 }};	/*Useragent - SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; sonyericssonw20) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2596104 = {S94065, 134, { &P20521,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Artes D821 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2596459 = {S94066, 53, { &P15364,&P6704,&P20412,&P18092 }};	/*Useragent - Opera/9.80 (Windows 98) Presto/2.12.388 Version/12.12*/
const Device UA2599585 = {S94067, 167, { &P20947,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTCEVOV4G Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2637486 = {S94068, 101, { &P22037,&P17979,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.4 Android (16/4.1.1; 160dpi; 600x976; samsung; GT-P3113; espressowifi; espresso; en_US)*/
const Device UA2643349 = {S94069, 171, { &P13526,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; T-Mobile myTouch 4G Build/IMM76L; CyanogenMod-mimicry) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659157 = {S94070, 144, { &P21741,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lemon Tab LT29 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2659529 = {S94071, 170, { &P21119,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; SoftwinerEvb Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2659686 = {S94072, 104, { &P15201,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_Sensation_Z710e) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2660100 = {S94073, 178, { &P18061,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Full AOSP on Rk29sdk Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2666759 = {S94074, 130, { &P18702,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N8013 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2666914 = {S94075, 138, { &P14872,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D700 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667047 = {S94076, 138, { &P16273,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T679 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667121 = {S94077, 155, { &P20541,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAP4500TDUO Build/PrestigioPAP4500TDUO) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667433 = {S94078, 138, { &P22162,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AUXUS CoreX4 3G Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667505 = {S94079, 156, { &P21026,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; ZTE U930 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2667719 = {S94080, 156, { &P22875,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9220 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2668269 = {S94081, 138, { &P13999,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NexusHD2 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668852 = {S94082, 134, { &P15184,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P920 Build/ICS) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2685397 = {S94083, 117, { &P15364,&P17350,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT based; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.A.B.C Safari/525.13*/
const Device UA2696700 = {S94084, 69, { &P18445,&P18148,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 8.0;NOKIA;Lumia 920;8.0;10211)*/
const Device UA2706287 = {S94085, 138, { &P21385,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F240S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2706769 = {S94086, 102, { &P18176,&P21697,&P24633,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 7.10; en-US; NOKIA; Lumia 900) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile*/
const Device UA2708264 = {S94087, 128, { &P21352,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB469 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2711687 = {S94088, 133, { &P12110,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; novo7Legend Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2711809 = {S94089, 135, { &P22634,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; WEXLER-TAB-7T Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2714049 = {S94090, 137, { &P13999,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NexusHD2 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2727970 = {S94091, 160, { &P18482,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; A75 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355*/
const Device UA2729095 = {S94092, 134, { &P20521,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; Artes D821 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2732862 = {S94093, 132, { &P22000,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TAB466EUK Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2733893 = {S94094, 137, { &P24733,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; TAB950 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734565 = {S94095, 167, { &P15880,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2734745 = {S94096, 128, { &P20844,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB462 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2734839 = {S94097, 142, { &P20507,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 7 Build/1.0.3 X-12:51) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735516 = {S94098, 127, { &P20216,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M1062 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2738024 = {S94099, 135, { &P22760,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ThL W8 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2748922 = {S94100, 92, { &P12706,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; LG-P690) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2753180 = {S94101, 139, { &P20439,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SOL21 Build/9.1.D.0.395) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2768531 = {S94102, 130, { &P19877,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T869 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2769493 = {S94103, 134, { &P21011,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; L-01E Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2779979 = {S94104, 171, { &P21988,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us ; PD_Novel Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444*/
const Device UA2782811 = {S94105, 126, { &P18248,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; A700 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2783721 = {S94106, 145, { &P20259,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Starmobile ASTRA Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2792790 = {S94107, 165, { &P18666,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A25 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2796842 = {S94108, 148, { &P20231,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A21 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2799051 = {S94109, 136, { &P14017,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P990 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2843855 = {S94110, 156, { &P21845,&P18066,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-STYLE Q4 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2848083 = {S94111, 135, { &P23634,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; ELEMENT10 1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3099392 = {S94112, 168, { &P15047,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; SK17i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2848314 = {S94113, 157, { &P21468,&P18066,&P18062,&P18092 }};	/*Useragent - AMOI N807 Linux/3.0.13 Android/4.0.4 Release/11.06.2012 Browser/AppleWebKit534.30 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/534.30 Android 4.0.1;*/
const Device UA2850049 = {S94114, 131, { &P23292,&P17979,&P18062,&P18092 }};	/*Useragent - GIPO-W919/1.0 Linux/2.6.35.7 Android/Android_4.0 Release/12.03.2012 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2861284 = {S94115, 152, { &P16263,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; ja-jp; dtab01 Build/HuaweiMediaPad) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24 TT/4.1.2-X.X*/
const Device UA2867617 = {S94116, 130, { &P21480,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PMP3270B Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2868107 = {S94117, 134, { &P23252,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; M6589 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868378 = {S94118, 141, { &P23254,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Celkon A119Q Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868897 = {S94119, 165, { &P13064,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2868929 = {S94120, 139, { &P23154,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOLO_X1000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869261 = {S94121, 138, { &P21495,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E220S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869927 = {S94122, 159, { &P18302,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT25i Build/4.1.B.0.631) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2870161 = {S94123, 159, { &P20076,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26w Build/6.2.B.0.200) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2877769 = {S94124, 134, { &P15389,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U8800 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877829 = {S94125, 134, { &P20478,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; JY-G2 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879379 = {S94126, 147, { &P17926,&P6437,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.2.1; AT100 Build/HTK55D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2879400 = {S94127, 137, { &P16273,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T679 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2885406 = {S94128, 40, { &P17595,&P23096,&P18534,&P18092 }};	/*Useragent - Reeder/1.5.7 CFNetwork/655 Darwin/14.0.0*/
const Device UA2891649 = {S94129, 95, { &P21982,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Celkon_A98) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2892445 = {S94130, 98, { &P22737,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Micromax_A115) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2892632 = {S94131, 143, { &P23712,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; i-mobile IQ XA Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2935749 = {S94132, 131, { &P22966,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MID7051 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2953983 = {S94133, 132, { &P20190,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; U30GT-MS Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2956772 = {S94134, 90, { &P18223,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; LT22i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2957526 = {S94135, 156, { &P13123,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2957899 = {S94136, 147, { &P18300,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T769 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2958101 = {S94137, 156, { &P22875,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2958448 = {S94138, 151, { &P23428,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8268 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2963977 = {S94139, 151, { &P20518,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A10 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2989490 = {S94140, 139, { &P21977,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ENERGY s10 DUAL Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2993626 = {S94141, 146, { &P22004,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ro-ro; ALLVIEW_P6_Quad Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2994865 = {S94142, 138, { &P21019,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E270K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2997808 = {S94143, 151, { &P16553,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA3001648 = {S94144, 147, { &P12605,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3004195 = {S94145, 152, { &P20438,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; N400 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3010251 = {S94146, 155, { &P14433,&P5096,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-I9003 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3010718 = {S94147, 159, { &P21628,&P18111,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8552 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300*/
const Device UA3011600 = {S94148, 159, { &P12040,&P17483,&P24678,&P18092 }};	/*Useragent - LG-GW300/V100 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gw300) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3015427 = {S94149, 96, { &P11834,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6760s-1) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3015452 = {S94150, 90, { &P12798,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT18i) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3015665 = {S94151, 159, { &P22632,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Titanium S1 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3016630 = {S94152, 155, { &P21953,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Ctrl_V1 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3017571 = {S94153, 159, { &P22997,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Celkon A107 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3019019 = {S94154, 154, { &P12706,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LG-P690 Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3019873 = {S94155, 93, { &P14294,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5238) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3022599 = {S94156, 138, { &P23680,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AURUS III Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023400 = {S94157, 137, { &P12482,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T959 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023621 = {S94158, 136, { &P13074,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-P930 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023628 = {S94159, 125, { &P14111,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Arc Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023658 = {S94160, 144, { &P22998,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TECNO_N7 Build/TECNOTECNO_N7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023924 = {S94161, 136, { &P19909,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P935 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3051113 = {S94162, 132, { &P15649,&P5096,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.2.1; en-gb; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3058234 = {S94163, 89, { &P21472,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; A111) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3059018 = {S94164, 178, { &P15228,&P5043,&P24055,&P18092 }};	/*Useragent - NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311*/
const Device UA3059295 = {S94165, 157, { &P18636,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; VS930 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3080269 = {S94166, 88, { &P20231,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A21) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3092038 = {S94167, 137, { &P24609,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E440g Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3093368 = {S94168, 170, { &P22382,&P18111,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8262 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3093604 = {S94169, 98, { &P14764,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; SonyEricssonU1) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3093689 = {S94170, 88, { &P21064,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A44) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3095535 = {S94171, 135, { &P21005,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; F-10D Build/V22R49C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3102938 = {S94172, 169, { &P21405,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; Lenovo A278t Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3108823 = {S94173, 136, { &P22727,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGP312 Build/10.3.1.C.0.136) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3109771 = {S94174, 168, { &P18113,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3111579 = {S94175, 93, { &P15649,&P5096,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5570) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3113527 = {S94176, 160, { &P18462,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC Desire X Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3113797 = {S94177, 135, { &P24595,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; fnac 5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3113838 = {S94178, 188, { &P13157,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3115431 = {S94179, 148, { &P12879,&P6437,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.2; GT-P7510 Build/HTJ85B) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3132618 = {S94180, 141, { &P16292,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132913 = {S94181, 139, { &P13054,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR6425LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA62309 = {S94182, 100, { &P12064,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-L768/ZUHK3 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA2908 = {S94183, 82, { &P12048,&P5448,&P4578,&P18092 }};	/*Useragent - T-Mobile Dash Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 320x240)*/
const Device UA2273181 = {S94184, 94, { &P12048,&P5448,&P4578,&P18092 }};	/*Useragent - T-Mobile Dash Mozilla/4.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 320x240)*/
const Device UA4559 = {S94185, 79, { &P13315,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Xda_Guide/240x320*/
const Device UA10232 = {S94186, 145, { &P12311,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Amstel v2.0; en-us; SPH-M900 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10235 = {S94187, 167, { &P12311,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android noobnl's Radioactive FroYo v1.8.6; en-us; SPH-M900 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10230 = {S94188, 157, { &P12311,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android SDX-Samdroid turbo v4.4; en-us; SPH-M900 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA25701 = {S94189, 152, { &P12311,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Ultimate Dream 3.2; en-us; SPH-M900 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA30239 = {S94190, 167, { &P12311,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android noobnl-Radioactive-Froyo-v2.0pre3; en-us; SPH-M900 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA30236 = {S94191, 151, { &P12311,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android bourineROM v1.1.5; en-us; SPH-M900 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA30241 = {S94192, 166, { &P12311,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android noobnl Radioactive FroYo v1.8.22; en-us; SPH-M900 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA30243 = {S94193, 167, { &P12311,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android noobnl's Radioactive FroYo v1.8.7; en-us; SPH-M900 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA22397 = {S94194, 76, { &P15453,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Sprint MP6900SP*/
const Device UA23108 = {S94195, 83, { &P13768,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonZ550c/R6BA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA44018 = {S94196, 83, { &P13768,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonZ550a/R6GA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64351 = {S94197, 61, { &P13618,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia7200/2.0 (3.110) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA64179 = {S94198, 100, { &P12302,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-J808/ZWHI1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA39996 = {S94199, 122, { &P13746,&P5477,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.1-update1;Zh_cn;GT540;320*480;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355*/
const Device UA2021203 = {S94200, 73, { &P13746,&P5477,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.1-update1;Zh_cn;GT540;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA44507 = {S94201, 73, { &P12018,&P5448,&P22303,&P18092 }};	/*Useragent - HTC-8500/1.2 Mozilla/4.0 (compatible; MSIE 5.5; Windows CE; PPC; 240x320)*/
const Device UA49217 = {S94202, 65, { &P12959,&P17483,&P17602,&P18092 }};	/*Useragent - LG-A258/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA61168 = {S94203, 62, { &P12651,&P6476,&P4977,&P18092 }};	/*Useragent - Mozilla/4.0 (MobilePhone SCP-3100/CA/1.0) NetFront/3.1 MMP/2.0*/
const Device UA86335 = {S94204, 62, { &P12651,&P6476,&P4977,&P18092 }};	/*Useragent - Mozilla/4.0 (MobilePhone SCP-3100/US/1.0) NetFront/3.1 MMP/2.0*/
const Device UA63588 = {S94205, 89, { &P12986,&P17017,&P4780,&P18092 }};	/*Useragent - Nexian NX-G505/MTK Release/10.1.2009 browser/MAUI Profile/MIDP-2.0 configuration/CLDC-1.0*/
const Device UA64268 = {S94206, 70, { &P14586,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-T106/1.0 ACS-NF/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA67262 = {S94207, 139, { &P13189,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; T-Mobile G1 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA108124 = {S94208, 33, { &P17893,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/H677/Screen-176X220*/
const Device UA203956 = {S94209, 116, { &P15364,&P18110,&P22449,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.24 (KHTML, like Gecko) Iron/11.0.700.2 Chrome/11.0.700.2 Safari/534.24*/
const Device UA315846 = {S94210, 118, { &P15364,&P18110,&P22449,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Iron/11.0.700.2 Chrome/11.0.700.2 Safari/534.24*/
const Device UA563749 = {S94211, 96, { &P14637,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; LG; LG-E900)*/
const Device UA925725 = {S94212, 97, { &P14637,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; LG; LG-E900h)*/
const Device UA925873 = {S94213, 104, { &P14637,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; LG; LG-E900; Orange)*/
const Device UA723224 = {S94214, 57, { &P15139,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player tmous / 1.0 / htc_doubleshot / 2.3.4*/
const Device UA729557 = {S94215, 47, { &P12588,&P5055,&P17338,&P18092 }};	/*Useragent - nokiac6-01/UC Browser7.9.0.102/69/352  3gpp-gba*/
const Device UA732839 = {S94216, 71, { &P15364,&P18145,&P22643,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64) KHTML/4.7.3 (like Gecko) Konqueror/4.7*/
const Device UA775690 = {S94217, 106, { &P11953,&P17485,&P17347,&P18092 }};	/*Useragent - SonyEricssonU10i/R7BA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonu10i/UC Browser8.0.3.107/69/444*/
const Device UA793199 = {S94218, 138, { &P12280,&P17146,&P20781,&P18092 }};	/*Useragent - MQQBrowser/30 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA1967113 = {S94219, 138, { &P12280,&P17146,&P20781,&P18092 }};	/*Useragent - MQQBrowser/30 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3*/
const Device UA837001 = {S94220, 86, { &P17579,&P5567,&P4536,&P18092 }};	/*Useragent - Mozilla/5.0 Opera/9.5 (KDDI-SN3S; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00*/
const Device UA847524 = {S94221, 36, { &P15364,&P18146,&P18405,&P18092 }};	/*Useragent - Opera/9.24 (X11; SunOS i86pc; U; en)*/
const Device UA2282732 = {S94222, 91, { &P15364,&P22193,&P17828,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/4.5; Linux) KHTML/4.5.5 (like Gecko) Fedora/4.5.5-2.fc13*/
const Device UA1962575 = {S94223, 52, { &P17346,&P17145,&P17470,&P18092 }};	/*Useragent - Facebook 4110.0 (iPod touch; iPhone OS 4.2.1; en_US)*/
const Device UA1950601 = {S94224, 158, { &P20134,&P5629,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1991352 = {S94225, 146, { &P14923,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TeamDRH ICS for GTablet Build/IML77) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA1996872 = {S94226, 91, { &P17595,&P18207,&P22316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:14.0) Gecko/X Firefox/14.0.1 SeaMonkey/2.11*/
const Device UA2021690 = {S94227, 143, { &P22389,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; pt-br; Ypy 7 - TB07FTA Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023073 = {S94228, 123, { &P22389,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; TB07FTA; pt-br) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027664 = {S94229, 161, { &P20265,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A52 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2029010 = {S94230, 157, { &P13123,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030479 = {S94231, 138, { &P12798,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; LT18i Build/4.1.B.0.431) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2367408 = {S94232, 169, { &P14969,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2155600 = {S94233, 81, { &P15118,&P5129,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaX7-00)/UC Browser8.2.0.132/50/355/UCWEB Mobile*/
const Device UA2160902 = {S94234, 110, { &P15364,&P5448,&P4759,&P18092 }};	/*Useragent - 480x800; XV6875.1; Window Mobile 6.5 Professional; (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.6.1.82*/
const Device UA2277390 = {S94235, 80, { &P15364,&P5448,&P4759,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)/UC Browser7.6.1.82*/
const Device UA2262659 = {S94236, 88, { &P12298,&P5567,&P17371,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/5.1/27.2239; U; en) Presto/2.8.119 400X240 Samsung SCH-U820*/
const Device UA2338761 = {S94237, 146, { &P20134,&P5629,&P20791,&P18092 }};	/*Useragent - MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2951230 = {S94238, 158, { &P20134,&P5629,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2391908 = {S94239, 79, { &P15364,&P5690,&P20906,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:0.9.2) Gecko/X Netscape6/6.1*/
const Device UA2395568 = {S94240, 57, { &P12004,&P5890,&P21761,&P18092 }};	/*Useragent - iBrowser/2.5 (NokiaN95(08.01);SymbianOS/9.2 Series60/3.1)*/
const Device UA2409201 = {S94241, 34, { &P15364,&P17285,&P17213,&P18092 }};	/*Useragent - Opera/8.51 (Windows NT 6.0; U; en)*/
const Device UA2414437 = {S94242, 128, { &P22089,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 932i Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2426898 = {S94243, 156, { &P12616,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; WT19i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455404 = {S94244, 156, { &P13815,&P5096,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.1; en-in; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2463766 = {S94245, 49, { &P11854,&P5890,&P20777,&P18092 }};	/*Useragent - MQQBrowser/2.6 (E71-1;SymbianOS/9.2 Series60/3.1)*/
const Device UA2466419 = {S94246, 128, { &P21548,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; X906 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2467222 = {S94247, 90, { &P23417,&P22843,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lava-a10) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2494526 = {S94248, 138, { &P21438,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F120K Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2498301 = {S94249, 81, { &P22983,&P5629,&P15760,&P18092 }};	/*Useragent - SAMSUNG-GT-S7508_TD/1.0 Android/2.3.6 Release/11.30.2011 Browser/AppleWebKit533.1*/
const Device UA2505061 = {S94250, 157, { &P15364,&P5690,&P19965,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.3.1.31155 (Firestorm-Release; firestorm skin) Safari/533.3*/
const Device UA2505083 = {S94251, 133, { &P23469,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID9106CM Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2505739 = {S94252, 147, { &P21257,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ATLANTIS Build/NGMNGM_ATLANTIS) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2505742 = {S94253, 132, { &P20829,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; BC9710AM Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2517675 = {S94254, 64, { &P15364,&P7127,&P18353,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) Sleipnir/2.49*/
const Device UA2526178 = {S94255, 153, { &P21598,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Kem 452 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2527398 = {S94256, 75, { &P15364,&P5690,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; Opera Mobi/46154) Presto/2.11.355 Version/12.10*/
const Device UA2527754 = {S94257, 65, { &P21514,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; N350; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2528501 = {S94258, 135, { &P21110,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A8809 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2548265 = {S94259, 156, { &P12444,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; U20i Build/IMM76L; CyanogenMod-MiniCM9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2686379 = {S94260, 156, { &P12444,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; U20i Build/IMM76I; CyanogenMod-MiniCM9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2553205 = {S94261, 179, { &P18321,&P5629,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5300 Build/GINGERBREAD; 240*320) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444*/
const Device UA2554623 = {S94262, 82, { &P15364,&P16110,&P18692,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; WOW64; Opera Mobi/46154) Presto/2.11.355 Version/12.10*/
const Device UA2554714 = {S94263, 161, { &P17972,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; pandigitalsprnova1/sourceidDLX) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2563811 = {S94264, 140, { &P20927,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT-AS45IPS Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2578060 = {S94265, 138, { &P20592,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-SU870 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2579886 = {S94266, 156, { &P18302,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; MT25i Build/4.1.B.0.631) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2580293 = {S94267, 171, { &P12280,&P17090,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 Mozilla/5.0 (iPhone 4; U; CPU iPhone OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3 tencent(securitypay)*/
const Device UA2580328 = {S94268, 162, { &P14997,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; S100 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2583168 = {S94269, 131, { &P24100,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CAL21 Build/AX) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2594720 = {S94270, 66, { &P15079,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; MK16i; 480*854) UCWEB7.9.0.94/139/444*/
const Device UA2598309 = {S94271, 167, { &P18295,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC One X Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2599242 = {S94272, 173, { &P12444,&P5477,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; U20i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2601200 = {S94273, 135, { &P21011,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; L-01E Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2607490 = {S94274, 70, { &P15851,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;Radar C110e;7.10;8862)*/
const Device UA2607495 = {S94275, 67, { &P15851,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;Radar 4G;7.10;8862)*/
const Device UA2652707 = {S94276, 130, { &P23598,&P18110,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-55EX645; CC/CAN; en) Presto/2.10.250 Version/11.60*/
const Device UA2658805 = {S94277, 80, { &P17595,&P17297,&P18689,&P18090 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:19.0) Gecko/X Firefox/19.0 slurp*/
const Device UA2665122 = {S94278, 169, { &P21980,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; UBISLATE7C+ Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2668260 = {S94279, 158, { &P18451,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LT26ii Build/6.1.A.2.45) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2668938 = {S94280, 161, { &P19898,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; SCH-I699 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2692237 = {S94281, 170, { &P14225,&P5629,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Desire HD Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (Desire HD, 34, 1.6.0)*/
const Device UA3072413 = {S94282, 170, { &P14225,&P5629,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (Desire HD, 42, 1.9.1)*/
const Device UA2696853 = {S94283, 127, { &P21323,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; V9S Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2699862 = {S94284, 100, { &P13183,&P17485,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk810i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2707865 = {S94285, 138, { &P20511,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Fly_IQ442 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713804 = {S94286, 137, { &P16436,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I8150 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713877 = {S94287, 147, { &P20221,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Starmobile CRYSTAL Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714076 = {S94288, 135, { &P22681,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; UMI X1 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714302 = {S94289, 126, { &P18248,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A700 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2721523 = {S94290, 52, { &P13324,&P5890,&P21746,&P18092 }};	/*Useragent - iBrowser/2.6 (NokiaN8-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA2729136 = {S94291, 133, { &P21890,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAD-10021 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2732075 = {S94292, 137, { &P23267,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-be; Lenco TAB-832 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2732870 = {S94293, 137, { &P20853,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Scroll Extreme Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2733009 = {S94294, 136, { &P23165,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; H7500+ Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2734628 = {S94295, 137, { &P23523,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-i9377 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735165 = {S94296, 128, { &P20256,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TAB275 Build/TAB275) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735923 = {S94297, 136, { &P20598,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Miracle Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2738219 = {S94298, 149, { &P15919,&P18111,&P22281,&P18092 }};	/*Useragent - OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2764601 = {S94299, 175, { &P18513,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S6802 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2765619 = {S94300, 130, { &P23767,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-7240 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2767673 = {S94301, 140, { &P24037,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALLVIEW_TX1_Quasar Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2768728 = {S94302, 160, { &P12135,&P17483,&P24677,&P18092 }};	/*Useragent - LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gd510) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2773408 = {S94303, 130, { &P21224,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N-07D Build/AX) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2779029 = {S94304, 170, { &P18283,&P18111,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9300 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2792170 = {S94305, 131, { &P23334,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TM-7024 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2793141 = {S94306, 107, { &P21152,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ALCATEL_ONE_TOUCH_930D) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2797901 = {S94307, 135, { &P21005,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; F-10D Build/V21R48A) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798628 = {S94308, 133, { &P22633,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; eZee Tab805 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2799525 = {S94309, 171, { &P20017,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799733 = {S94310, 156, { &P20603,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; LG-C800 Build/GRK39F) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2819818 = {S94311, 86, { &P17595,&P18207,&P22309,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:5.0) Gecko/X Firefox/5.0 SeaMonkey/2.2*/
const Device UA3046344 = {S94312, 184, { &P14183,&P5096,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (SAMSUNG-SGH-I997, 42, 1.9.1)*/
const Device UA2826691 = {S94313, 157, { &P14186,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2831033 = {S94314, 157, { &P14186,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2852398 = {S94315, 93, { &P12110,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2; en-US; HTC_Legend) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2861741 = {S94316, 140, { &P15767,&P19826,&P21786,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.6.5/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA2865604 = {S94317, 140, { &P23748,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Byond B61 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867536 = {S94318, 157, { &P11747,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC Desire V Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2868035 = {S94319, 162, { &P19854,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; SCH-R760 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2868168 = {S94320, 130, { &P17168,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7310 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2869506 = {S94321, 157, { &P14877,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9000 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2875559 = {S94322, 142, { &P20507,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 7 Build/1.0.3 X-12:51) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2877354 = {S94323, 131, { &P20004,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A2 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877422 = {S94324, 128, { &P21352,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB469 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2879364 = {S94325, 137, { &P22953,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F220K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2891014 = {S94326, 156, { &P13123,&P5629,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2893953 = {S94327, 157, { &P21628,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2934182 = {S94328, 155, { &P19872,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SGH-I847 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934527 = {S94329, 139, { &P23967,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HS-EG909 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2934898 = {S94330, 119, { &P15364,&P18110,&P19818,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) CasperJS/1.0.2+Phantomjs/1.8.1 Safari/534.34*/
const Device UA2937374 = {S94331, 95, { &P23174,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GenxDroid7) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2937517 = {S94332, 135, { &P21869,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ZP950+ Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2943401 = {S94333, 134, { &P24568,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; N9810 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2958091 = {S94334, 152, { &P21020,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; Micromax A116 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2959338 = {S94335, 83, { &P15364,&P17285,&P22767,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Maxthon/4.0.6.2000)*/
const Device UA2960477 = {S94336, 156, { &P20123,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; ZTE U795 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2962170 = {S94337, 97, { &P18649,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A57) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2965657 = {S94338, 88, { &P20120,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A45) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2970566 = {S94339, 150, { &P21334,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A8 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2990370 = {S94340, 154, { &P15364,&P18066,&P20787,&P18092 }};	/*Useragent - MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S7t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997940 = {S94341, 153, { &P23672,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030X Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2999335 = {S94342, 163, { &P20158,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A7* Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33756*/
const Device UA3004467 = {S94343, 137, { &P17595,&P21721,&P22321,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/535.7 (KHTML, like Gecko) RockMelt/0.16.91.483 Chrome/16.0.912.77 Safari/535.7*/
const Device UA3009395 = {S94344, 128, { &P23178,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; iDxD8 3G Build/JB) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3010986 = {S94345, 151, { &P23207,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A4+ Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3012415 = {S94346, 156, { &P22396,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5312 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3012462 = {S94347, 132, { &P21312,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; A19 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3013686 = {S94348, 99, { &P22158,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; INTEX CLOUD X1) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3014216 = {S94349, 105, { &P12871,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC Wildfire S A510e) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3014228 = {S94350, 133, { &P16356,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3014333 = {S94351, 131, { &P13644,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-us; PC36100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3019024 = {S94352, 171, { &P21968,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; ALCATEL ONE TOUCH 4030D Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3019029 = {S94353, 161, { &P22627,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; IRIS 455 Build/LAVAIRIS455) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3022059 = {S94354, 183, { &P22903,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A62 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3022213 = {S94355, 140, { &P23693,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PNDPP48GP Build/ICS.PNDPP48GP.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022496 = {S94356, 183, { &P22864,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A26 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3022831 = {S94357, 141, { &P23021,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Spice Mi-535 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023119 = {S94358, 135, { &P20732,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; YP-GP1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023899 = {S94359, 136, { &P24508,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E445 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3025178 = {S94360, 88, { &P20265,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A52) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3028280 = {S94361, 100, { &P14060,&P17485,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw150i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3033037 = {S94362, 91, { &P15518,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5230) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3060579 = {S94363, 96, { &P15518,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-s5230) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3049957 = {S94364, 155, { &P19867,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I437 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3053584 = {S94365, 139, { &P23181,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; NEBULA6.9 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA3055856 = {S94366, 129, { &P21472,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-in; A111 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3056099 = {S94367, 150, { &P22550,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; Multilaser Titanium Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057093 = {S94368, 180, { &P24041,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; Karbonn A50 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33756*/
const Device UA3057929 = {S94369, 167, { &P15364,&P17285,&P17190,&P18089 }};	/*Useragent - Googlebot/2.1 (+http://www.googlebot.com/bot.html; MSIE 8.0; Windows NT 6.0; Trident/4.0; GTB6; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET C;c2VvLWNyYXdsZXI=*/
const Device UA3059033 = {S94370, 155, { &P20119,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_S9714 Build/JRO03R) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3059105 = {S94371, 157, { &P21283,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3059380 = {S94372, 160, { &P19863,&P18066,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3059384 = {S94373, 173, { &P18016,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ThinkPad Tablet Build/ThinkPadTablet_A400_03) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3079104 = {S94374, 162, { &P13054,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; ADR6425LVW Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/vigor*/
const Device UA3079347 = {S94375, 159, { &P18635,&P21637,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus 4 Build/JWR66Y) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.20.1364.172 Mobile Safari/537.22*/
const Device UA3083504 = {S94376, 103, { &P16595,&P5477,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; Micromax_A60) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3084054 = {S94377, 157, { &P12972,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SGH-T989 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3127297 = {S94378, 157, { &P12972,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T989 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3093448 = {S94379, 93, { &P12568,&P5096,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; GT-B7510) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3093486 = {S94380, 93, { &P14753,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; Nokia N96) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3109477 = {S94381, 94, { &P18296,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_One_V) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3109658 = {S94382, 148, { &P18385,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 7 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3110849 = {S94383, 155, { &P22585,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SPH-L720 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3113636 = {S94384, 135, { &P20732,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; YP-GP1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114165 = {S94385, 161, { &P14433,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9003 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3114722 = {S94386, 98, { &P21020,&P18111,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Micromax A116) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3127139 = {S94387, 114, { &P14826,&P5843,&P24470,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC1-01/06.15; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.7.0.0.11*/
const Device UA96 = {S94388, 44, { &P14232,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KE970/V10e Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA50382 = {S94389, 17, { &P13216,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonK750i*/
const Device UA8898 = {S94390, 87, { &P16676,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_E100*/
const Device UA51852 = {S94391, 110, { &P16676,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_E100 / 2.004.00_IT_vodafone*/
const Device UA8991 = {S94392, 121, { &P13382,&P5448,&P15792,&P18092 }};	/*Useragent - SAMSUNG-SGH-i710/MSIE4.01/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8)*/
const Device UA10052 = {S94393, 87, { &P13374,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_P3700 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA61803 = {S94394, 83, { &P14332,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW600i/R7CA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA16643 = {S94395, 60, { &P11884,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia7250/1.0 (3.14) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA62322 = {S94396, 115, { &P14039,&P17484,&P3834,&P18092 }};	/*Useragent - SAMSUNG-GT-S5520/S5520ZWKA1 SHP/VPP/R5 Dolfin/2.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA83421 = {S94397, 17, { &P11948,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonK550i*/
const Device UA20348 = {S94398, 111, { &P15074,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHM600/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA20876 = {S94399, 77, { &P12212,&P5530,&P4836,&P18092 }};	/*Useragent - MOT-W510/08.11.04R MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/1.0*/
const Device UA21307 = {S94400, 40, { &P15190,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP260 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA22440 = {S94401, 12, { &P15472,&P17483,&P17660,&P18092 }};	/*Useragent - LG/U880/v1.0*/
const Device UA22696 = {S94402, 107, { &P15511,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-T729/T729UVGF9 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA32625 = {S94403, 91, { &P12509,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-VF533/1.0/EVN_ZO MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.1.423 (GUI)*/
const Device UA32626 = {S94404, 91, { &P12509,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-VF533/1.0/EVN_ZN MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.1.423 (GUI)*/
const Device UA32624 = {S94405, 91, { &P12509,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-VF533/1.0/EVN_ZP MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.1.423 (GUI)*/
const Device UA51351 = {S94406, 91, { &P12509,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-VF533/1.0/EVN_ZJ MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.1.423 (GUI)*/
const Device UA51352 = {S94407, 91, { &P12509,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-VF533/1.0/EVN_ZI MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.1.423 (GUI)*/
const Device UA32821 = {S94408, 109, { &P12776,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; MDA Compact/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1)*/
const Device UA87729 = {S94409, 137, { &P13161,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-A810/1.0 LinuxOS/2.4.20 Release/6.10.2008 Browser/Opera8.00 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/GSM_E28.24.1_G_11.02.18R*/
const Device UA62221 = {S94410, 12, { &P14385,&P17483,&P17660,&P18092 }};	/*Useragent - LG/U400/v1.0*/
const Device UA67042 = {S94411, 12, { &P13800,&P5890,&P4030,&P18092 }};	/*Useragent - NOKIA5630d-1*/
const Device UA67313 = {S94412, 146, { &P14642,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; Motorola-XT502 Build/DONUT) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA68104 = {S94413, 36, { &P14983,&P5055,&P4759,&P18092 }};	/*Useragent - NokiaC7-00/UC Browser7.6.1.82/50/352*/
const Device UA2720221 = {S94414, 46, { &P14983,&P5055,&P4759,&P18092 }};	/*Useragent - nokiac7-00/UC Browser7.6.1.82/69/351  3gpp-gba*/
const Device UA122311 = {S94415, 81, { &P17797,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-P629/S118 LMP/LML Release/2010.04.22 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA134571 = {S94416, 33, { &P17892,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/H688/Screen-176X220*/
const Device UA301325 = {S94417, 125, { &P16156,&P17979,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Honeycomb; en-us; Build/Nook) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA304184 = {S94418, 128, { &P18061,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0; en-us; sdk Build/HONEYCOMB) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA306182 = {S94419, 140, { &P11950,&P24695,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; X10i Build/Cyanogen6.1.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA312089 = {S94420, 144, { &P11950,&P24695,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; X10i Build/CyanogenMod 6.1.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370934 = {S94421, 122, { &P21483,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; ASUS T20 Build/FRF91) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2020005 = {S94422, 141, { &P21483,&P5096,&P3508,&P18092 }};	/*Useragent - ASUS_T20_TD/1.0 OPhone/2.5 (Linux; Android 2.2) Release/12.23.2009 Browser/WAP2.0 (AppleWebKit/533.1) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA380763 = {S94423, 123, { &P15364,&P17285,&P22420,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.1.0.0 Chrome/17.0.963.38 Safari/535.11*/
const Device UA520257 = {S94424, 142, { &P16146,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; XT702 Build/SGC_U6_10.13.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021033 = {S94425, 130, { &P16146,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; XT702 Build/SGC_U6_10.13.0) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA538849 = {S94426, 104, { &P15364,&P22205,&P19772,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US; rv:1.8.1.13) Gecko/X Fedora/1.1.9-2.fc7 SeaMonkey/1.1.9*/
const Device UA583722 = {S94427, 113, { &P11625,&P5768,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia5300/07.20; Profile/MIDP-2.0 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA645509 = {S94428, 94, { &P17595,&P17296,&P18197,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.1.16) Gecko/X Thunderbird/3.0.11*/
const Device UA682833 = {S94429, 137, { &P12216,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6300 Build/GRJ22) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026273 = {S94430, 59, { &P12216,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; ADR6300 Build/GRJ22)*/
const Device UA2692922 = {S94431, 110, { &P17595,&P17297,&P22889,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) iCab/4.8 Safari/533.16*/
const Device UA896989 = {S94432, 109, { &P17595,&P17296,&P18106,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Mac OS X 10_6_1; en-US) AppleWebKit/530.5 (KHTML, like Gecko) Chrome/ Safari/530.5*/
const Device UA925732 = {S94433, 74, { &P14886,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;USCCHTC-PC93100;7.10;7720)*/
const Device UA925769 = {S94434, 74, { &P14886,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;USCCHTC-PC93100;7.10;8107)*/
const Device UA925917 = {S94435, 70, { &P14886,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;7 Pro T7576;7.10;8107)*/
const Device UA2834993 = {S94436, 70, { &P14886,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;7 Pro T7576;7.10;8773)*/
const Device UA925821 = {S94437, 67, { &P14651,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;7 Trophy;7.10;8107)*/
const Device UA925840 = {S94438, 73, { &P14651,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;7 Trophy T8686;7.10;8107)*/
const Device UA925911 = {S94439, 67, { &P14651,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;7 Trophy;7.10;7720)*/
const Device UA925914 = {S94440, 67, { &P14651,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;7 Trophy;7.10;7740)*/
const Device UA1934563 = {S94441, 141, { &P22257,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; TR718DC_V21, Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1987703 = {S94442, 138, { &P22269,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; KATBL08AND Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032182 = {S94443, 139, { &P22269,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; KATBL08ANDC Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2010501 = {S94444, 74, { &P17595,&P19834,&P18537,&P18092 }};	/*Useragent - Reeder/1010.79.00 CFNetwork/596.1 Darwin/12.1.0 (x86_64) (MacBookAir5%2C2)*/
const Device UA2183086 = {S94445, 69, { &P17595,&P19834,&P18537,&P18092 }};	/*Useragent - Reeder/1010.79.00 CFNetwork/596.1 Darwin/12.1.0 (x86_64) (iMac12%2C2)*/
const Device UA2186712 = {S94446, 74, { &P17595,&P19834,&P18537,&P18092 }};	/*Useragent - Reeder/1010.79.00 CFNetwork/596.1 Darwin/12.1.0 (x86_64) (MacBookPro5%2C5)*/
const Device UA2026385 = {S94447, 146, { &P21406,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn;Lenovo S760 Build/GINGERBREAD) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2028338 = {S94448, 155, { &P14186,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028339 = {S94449, 155, { &P14186,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029071 = {S94450, 157, { &P22875,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029935 = {S94451, 159, { &P13042,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Incredible S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2341198 = {S94452, 159, { &P13042,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Incredible S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030676 = {S94453, 153, { &P21396,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A65 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2413415 = {S94454, 68, { &P12968,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.7;Zh_cn;LT26i;720*1280;)UCWEB7.8.0.95/139/444*/
const Device UA2033678 = {S94455, 136, { &P20858,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Scroll Excel Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2145039 = {S94456, 46, { &P11929,&P5129,&P17338,&P18092 }};	/*Useragent - nokia5230/UC Browser7.9.0.102/69/352  3gpp-gba*/
const Device UA2223263 = {S94457, 253, { &P14862,&P5204,&P18524,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/70/352*/
const Device UA2290286 = {S94458, 91, { &P12226,&P5448,&P4777,&P18092 }};	/*Useragent - HTC_Touch_Pro_T7272 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2349453 = {S94459, 167, { &P18295,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC One X Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2350034 = {S94460, 164, { &P14123,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P970 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2404749 = {S94461, 98, { &P14985,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s8530) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2409775 = {S94462, 46, { &P11929,&P5129,&P17347,&P18092 }};	/*Useragent - nokia5230/UC Browser8.0.3.107/69/444  3gpp-gba*/
const Device UA2425684 = {S94463, 53, { &P11929,&P5890,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2 (Nokia5230;SymbianOS/9.1 Series60/3.0)*/
const Device UA2450183 = {S94464, 74, { &P18649,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.5;Zh_cn;Micromax A57;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA2454030 = {S94465, 136, { &P21421,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KATBL10A16DA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2455122 = {S94466, 141, { &P21587,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Q-Smart S18 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2455362 = {S94467, 151, { &P14186,&P5096,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2474959 = {S94468, 82, { &P22042,&P17979,&P18062,&P18092 }};	/*Useragent - SAMSUNG-GT-P3108_TD/1.0 Android/4.0.3 Release/02.15.2012 Browser/AppleWebKit534.30*/
const Device UA2475801 = {S94469, 137, { &P21083,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SBM107SH Build/S0010) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2478326 = {S94470, 69, { &P17346,&P17145,&P22070,&P18092 }};	/*Useragent - Instagram 3.0.1 (iPod touch; iPhone OS 4.2.1; en_US) AppleWebKit/420+*/
const Device UA2479282 = {S94471, 137, { &P18241,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E400 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2482515 = {S94472, 133, { &P22254,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M-MP101S2 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2484037 = {S94473, 128, { &P18248,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; A700 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2485817 = {S94474, 164, { &P16049,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P698 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2505949 = {S94475, 138, { &P14959,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-I9103 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2507344 = {S94476, 129, { &P23347,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V900A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2524523 = {S94477, 136, { &P13452,&P24697,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000 Build/MIUI) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2527692 = {S94478, 138, { &P21271,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AQUA 5.0 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2527839 = {S94479, 138, { &P20260,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; onetouch EVO7 Build/MAIN_01) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2528831 = {S94480, 140, { &P21845,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-STYLE Q4 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2530562 = {S94481, 78, { &P17595,&P18207,&P19981,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:19.0) Gecko/X Thunderbird/19.0*/
const Device UA2574114 = {S94482, 116, { &P18481,&P17605,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-C3520/1.0 RTK-E/1.0 DF/1.0 Release/07.15.2011 Browser/NetFront4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2574821 = {S94483, 129, { &P20563,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT500 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2809417 = {S94484, 130, { &P20563,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT500a Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2578323 = {S94485, 31, { &P18644,&P5043,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia112/03.26)*/
const Device UA2580668 = {S94486, 160, { &P18482,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; A75 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2589678 = {S94487, 149, { &P23926,&P5629,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GN100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2595224 = {S94488, 68, { &P23414,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; Aqua_Sx; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2595257 = {S94489, 68, { &P21023,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; LG-E405; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2598631 = {S94490, 166, { &P13591,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; ZTE-BLADE Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2604123 = {S94491, 138, { &P20289,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; HTC6435LVW Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2622181 = {S94492, 132, { &P14905,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P7100 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2631256 = {S94493, 131, { &P14905,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7100 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2633905 = {S94494, 166, { &P15932,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P3110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2634787 = {S94495, 166, { &P15932,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-P3110 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2646292 = {S94496, 132, { &P22416,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Classpad Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2652308 = {S94497, 131, { &P23418,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; TM-7023 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2658980 = {S94498, 139, { &P23447,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Newman N2 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2660329 = {S94499, 169, { &P23919,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; R800i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2660999 = {S94500, 173, { &P15483,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC ChaCha A810e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2667186 = {S94501, 135, { &P21016,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LGL21 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667243 = {S94502, 138, { &P19746,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VIVO 4.3 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667485 = {S94503, 148, { &P20221,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Starmobile CRYSTAL Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668278 = {S94504, 146, { &P20259,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Starmobile ASTRA Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668629 = {S94505, 139, { &P20248,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V889M Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2680965 = {S94506, 157, { &P22385,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I8262D Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2701253 = {S94507, 73, { &P21351,&P18111,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.1.1; zh-cn; HUAWEI_C8813; 480*854) UCWEB7.9.0.94/139/800*/
const Device UA2703572 = {S94508, 91, { &P12089,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAN80) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2703972 = {S94509, 159, { &P14969,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2705418 = {S94510, 92, { &P11692,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIA3250) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2707138 = {S94511, 134, { &P22640,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IQ446 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708663 = {S94512, 135, { &P20359,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZP300+ Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709419 = {S94513, 135, { &P21523,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SH930W Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2710094 = {S94514, 137, { &P21132,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Andi4.5h Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712456 = {S94515, 136, { &P19909,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P935 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2716658 = {S94516, 62, { &P15364,&P17285,&P22226,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:21.0) Gecko/X Thunderbird/21.0*/
const Device UA2728063 = {S94517, 191, { &P19849,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-S7560M Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:38, ca*/
const Device UA2730685 = {S94518, 174, { &P18724,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; IdeaTab A2107A-H Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2733129 = {S94519, 80, { &P17595,&P17296,&P22167,&P18090 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:20.0) Gecko/X Firefox/20.0 slurp*/
const Device UA2734645 = {S94520, 137, { &P20289,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; HTC6435LVW Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734958 = {S94521, 129, { &P21990,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; chagall Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735003 = {S94522, 135, { &P20332,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax P275 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735222 = {S94523, 138, { &P20248,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V889M Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2746613 = {S94524, 136, { &P22671,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; eZee'Tab1001 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2814774 = {S94525, 136, { &P22671,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; eZee'Tab1001 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2764583 = {S94526, 99, { &P13215,&P5477,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT-I5700) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2767833 = {S94527, 135, { &P19790,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; L-04E Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2879463 = {S94528, 100, { &P12351,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3653w) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2777931 = {S94529, 170, { &P19779,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A1+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2778960 = {S94530, 101, { &P12657,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_EVO_3D_X515m) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2779233 = {S94531, 100, { &P23010,&P17485,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk790i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2783878 = {S94532, 159, { &P13437,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.4; T-Mobile G2 Build/GRJ22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2844728 = {S94533, 172, { &P18195,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; YP-G70 Build/IMM76L; CyanogenMod-Electric Catfish Mk 0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2792265 = {S94534, 141, { &P23339,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HIMAX PURE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2793850 = {S94535, 93, { &P18437,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N8000) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2797082 = {S94536, 136, { &P23158,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IMO S88 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2797699 = {S94537, 137, { &P15820,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-SU640 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798980 = {S94538, 144, { &P18017,&P6437,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.2.1; K1 Build/HTK75C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2840691 = {S94539, 128, { &P20186,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Loox Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2849084 = {S94540, 144, { &P23305,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-at; RAPAX090-0508 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2852268 = {S94541, 159, { &P21623,&P18111,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.1.1; th-th; i-mobile IQ 6 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2867284 = {S94542, 135, { &P24586,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 86VEBC Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868421 = {S94543, 159, { &P15972,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC One XL Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2869217 = {S94544, 141, { &P21160,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MEDION P4013 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2870042 = {S94545, 157, { &P18174,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS920 4G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2877922 = {S94546, 142, { &P21466,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Titan TV S320 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2878288 = {S94547, 139, { &P15364,&P18111,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; G1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2878492 = {S94548, 96, { &P19999,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Karbonn_A30) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2879105 = {S94549, 90, { &P12968,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT26i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2879479 = {S94550, 137, { &P18747,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A840S Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879559 = {S94551, 129, { &P21865,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB-730 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2898560 = {S94552, 99, { &P14010,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5263) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2934629 = {S94553, 75, { &P23769,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; Coolpad_7020; 320*480) UCWEB7.9.0.94/139/33619*/
const Device UA2935046 = {S94554, 135, { &P21011,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; L-01E Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3048697 = {S94555, 152, { &P12989,&P5843,&P24677,&P18092 }};	/*Useragent - Nokia303/5.0 (14.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia303) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2938126 = {S94556, 136, { &P20711,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID1065 Build/ICS.MID1065.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2942291 = {S94557, 136, { &P20937,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ADR910L Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2944318 = {S94558, 137, { &P23215,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SHV-E330S Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2946166 = {S94559, 158, { &P13437,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; T-Mobile G2 Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2957742 = {S94560, 138, { &P20132,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST70104-1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2958079 = {S94561, 150, { &P23163,&P18598,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; SCH-I959 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2958396 = {S94562, 147, { &P20255,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R811 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2959928 = {S94563, 152, { &P18314,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M032 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2960189 = {S94564, 99, { &P21842,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-e2202) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2969482 = {S94565, 159, { &P15079,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MK16i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2970167 = {S94566, 155, { &P21794,&P18598,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9152 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2992141 = {S94567, 93, { &P21117,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; HS-9DTB4) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2994713 = {S94568, 132, { &P23222,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PMP3470B Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2996610 = {S94569, 156, { &P23715,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Starmobile Engage7 3G Build/ICS_IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3001195 = {S94570, 142, { &P20124,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; ru-ru; ZTE Grand X Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3001760 = {S94571, 150, { &P18214,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA3007263 = {S94572, 159, { &P14993,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LT15i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3011181 = {S94573, 169, { &P19792,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire U dual sim Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3011664 = {S94574, 141, { &P20124,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ZTE Grand X Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3014135 = {S94575, 133, { &P16362,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; SPH-D710 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3014412 = {S94576, 107, { &P19868,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL ONE TOUCH 985N) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3014621 = {S94577, 96, { &P22209,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Titanium S5) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3015043 = {S94578, 153, { &P21794,&P18598,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9152 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3015830 = {S94579, 159, { &P19999,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A30 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3017036 = {S94580, 176, { &P18643,&P5043,&P22881,&P18092 }};	/*Useragent - Nokia311/5.0 (07.36) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291*/
const Device UA3019202 = {S94581, 156, { &P24149,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Aqua I-5 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3021057 = {S94582, 183, { &P20005,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A65 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3021232 = {S94583, 129, { &P18708,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U30GT-H Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021652 = {S94584, 137, { &P18509,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-MS770 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021644 = {S94585, 141, { &P20261,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Philips W832 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022052 = {S94586, 93, { &P13536,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6110) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3022140 = {S94587, 129, { &P21990,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; chagall Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022372 = {S94588, 142, { &P20253,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 5 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023926 = {S94589, 127, { &P23313,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; M8 pro Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3027969 = {S94590, 93, { &P11689,&P5890,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian70-1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3059019 = {S94591, 91, { &P11689,&P5890,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian70) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3048782 = {S94592, 136, { &P24382,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E986 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3049109 = {S94593, 101, { &P21857,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; HUAWEI_Y300-0100) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3049900 = {S94594, 150, { &P12841,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC PH39100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3056554 = {S94595, 90, { &P15397,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; ST18i) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3057075 = {S94596, 159, { &P22382,&P18111,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8262 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300*/
const Device UA3057539 = {S94597, 160, { &P19998,&P18066,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3057919 = {S94598, 158, { &P15364,&P5690,&P17558,&P18091 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.6) Gecko/X diffbot/0.3.7 (http://www.diffbot.com) Firefox/3.6.6 (.NET CLR 3.5.30729);c2VvLWNyYXdsZXI=*/
const Device UA3058924 = {S94599, 96, { &P22209,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Titanium_S5) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3059126 = {S94600, 154, { &P21094,&P18066,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A789 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3059863 = {S94601, 148, { &P19783,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ME172V Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3060351 = {S94602, 171, { &P20017,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 997D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3078620 = {S94603, 95, { &P16600,&P22784,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx600) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3079461 = {S94604, 156, { &P23004,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; TECNO D3 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3079485 = {S94605, 139, { &P15147,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST15i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3079577 = {S94606, 156, { &P16436,&P5629,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3080025 = {S94607, 159, { &P12040,&P17483,&P24679,&P18092 }};	/*Useragent - LG-GW300/V100 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gw300) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3093257 = {S94608, 97, { &P22623,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; Micromax_A51) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3103411 = {S94609, 129, { &P22043,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5113 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3108789 = {S94610, 90, { &P12616,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; WT19i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3108792 = {S94611, 95, { &P12444,&P5477,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; U20i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3109337 = {S94612, 91, { &P13964,&P5096,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2; en-US; GT-I5510) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3109358 = {S94613, 154, { &P22860,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Q700 Build/XOLOQ700) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109524 = {S94614, 97, { &P20729,&P5629,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A27) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3110905 = {S94615, 143, { &P23138,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; TRUE BEYOND 3G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111516 = {S94616, 155, { &P22874,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; GT-I9502 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3111556 = {S94617, 136, { &P14017,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-P990 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114796 = {S94618, 160, { &P22622,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Micromax A72 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3125490 = {S94619, 127, { &P24458,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; KUNO4 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3131812 = {S94620, 141, { &P20399,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PantechP8010 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132637 = {S94621, 133, { &P20760,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Event Build/IML77) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA726 = {S94622, 105, { &P11851,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC P5500; OpVer 25.131.1.6109*/
const Device UA16016 = {S94623, 99, { &P11851,&P5448,&P22271,&P18092 }};	/*Useragent - HTC-P5500-orange/PPC; 240x320; OpVer 25.131.2.7309 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA31883 = {S94624, 70, { &P11851,&P5448,&P22271,&P18092 }};	/*Useragent - HTC_P5500 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA87299 = {S94625, 122, { &P11851,&P5448,&P22271,&P18092 }};	/*Useragent - HTC P5500/1.31.621.09 Mozilla/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA12794 = {S94626, 101, { &P13504,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-X540/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA12998 = {S94627, 39, { &P14961,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KM335 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA20263 = {S94628, 102, { &P13794,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E350E/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA21291 = {S94629, 101, { &P13794,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E350/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA20444 = {S94630, 85, { &P15364,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-600A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA88649 = {S94631, 94, { &P15364,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-GLAMPHONE-ELLE-A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA88833 = {S94632, 46, { &P15364,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-S321A/1.0 ObigoInternetBrowser/Q03C*/
const Device UA88862 = {S94633, 86, { &P15364,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-S626A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA89119 = {S94634, 85, { &P15364,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-C820/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA90320 = {S94635, 45, { &P15364,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-360A/1.0 ObigoInternetBrowser/Q03C*/
const Device UA21630 = {S94636, 46, { &P15306,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-AX585/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA22338 = {S94637, 128, { &P15429,&P5448,&P22271,&P18092 }};	/*Useragent - PPC; 240x320; HTC_P3451/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA22419 = {S94638, 21, { &P15461,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP130 Obigo/WAP2.0*/
const Device UA78459 = {S94639, 33, { &P15461,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP130-orange/V10c Obigo/WAP2.0*/
const Device UA23087 = {S94640, 86, { &P12865,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-C75/11 UP.Browser/7.0.2.2.d.5(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA33141 = {S94641, 86, { &P12865,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-C75/13 UP.Browser/7.0.2.2.d.7(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61920 = {S94642, 86, { &P12865,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-C75/24 UP.Browser/7.0.2.2.d.7(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64277 = {S94643, 86, { &P12865,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-C75/22 UP.Browser/7.0.2.2.d.7(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23432 = {S94644, 71, { &P16031,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_X7510 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA24528 = {S94645, 61, { &P14515,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonT630/R601 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA26418 = {S94646, 35, { &P13592,&P17017,&P4924,&P18092 }};	/*Useragent - HUAWEI-M328/001.00 ACS-NetFront/3.2*/
const Device UA27292 = {S94647, 117, { &P12077,&P17605,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S3600i/1.0 RTK-E/1.0 DF/1.0 Release/01.09.2009 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA30226 = {S94648, 98, { &P13666,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-A711/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA33378 = {S94649, 41, { &P16677,&P17483,&P3625,&P18092 }};	/*Useragent - LG-LG100 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA50404 = {S94650, 34, { &P11854,&P5890,&P4737,&P18092 }};	/*Useragent - NokiaE71/UC Browser7.2.2.51/28/352*/
const Device UA53840 = {S94651, 116, { &P14045,&P17605,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C3518/1.0 RTK-E/1.0 DF/1.0 Release/22.12.2009 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61488 = {S94652, 171, { &P14045,&P17605,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C3518/1.0 RTK-E/1.0 DF/1.0 Release/22.12.2009 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA55197 = {S94653, 46, { &P14296,&P17484,&P5001,&P18092 }};	/*Useragent - SCH-U900/1.0 NetFront/3.0.22.2.9 (GUI) MMP/2.0*/
const Device UA56785 = {S94654, 34, { &P11854,&P5890,&P4715,&P18092 }};	/*Useragent - NokiaE71/UC Browser7.4.0.65/28/352*/
const Device UA56791 = {S94655, 142, { &P24003,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; Google Ion Build/CRB43) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA62327 = {S94656, 45, { &P14589,&P17484,&P4628,&P18092 }};	/*Useragent - Samsung-SCHR351 Infraware-Polaris/6.0 MMP/2.0*/
const Device UA67125 = {S94657, 31, { &P14761,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F03A(c100;TB;W24H17)*/
const Device UA2287599 = {S94658, 46, { &P13324,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN8-00/SymbianOS/9.1 Series60/3.0 3gpp-gba*/
const Device UA68573 = {S94659, 15, { &P14939,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHX160/1.0*/
const Device UA68917 = {S94660, 116, { &P11860,&P5776,&P4777,&P18092 }};	/*Useragent - Nokia6681/2.0 (6.09.00) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.7.1.88/27/444*/
const Device UA1900989 = {S94661, 112, { &P14557,&P5843,&P17338,&P18092 }};	/*Useragent - Nokia6303classic/2.0 (09.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303classic/UC Browser7.9.0.102/69/352*/
const Device UA80410 = {S94662, 70, { &P16106,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; O2 Xda II;PPC;240x320)*/
const Device UA80412 = {S94663, 75, { &P16106,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; O2 Xda 2mini; PPC; 240x320)*/
const Device UA80413 = {S94664, 84, { &P16106,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; O2 Xda 2s;PPC;240x320; PPC; 240x320)*/
const Device UA87441 = {S94665, 72, { &P16106,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; O2 Xda 2i; PPC; 240x320)*/
const Device UA837702 = {S94666, 90, { &P11689,&P16000,&P20417,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Nokia N70/2.0537.1.7; 6169) Opera 8.00 [en]*/
const Device UA87165 = {S94667, 47, { &P17641,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SN3B UP.Browser/6.2.0.13.1.5 (GUI) MMP/2.0*/
const Device UA87964 = {S94668, 45, { &P17528,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC3B UP.Browser/6.2.0.13.2 (GUI) MMP/2.0*/
const Device UA849161 = {S94669, 138, { &P15364,&P17815,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.8 (KHTML, like Gecko) Ubuntu/9.04 Chromium/7.0.529.0 Chrome/7.0.529.0 Safari/534.8*/
const Device UA2260397 = {S94670, 140, { &P15364,&P17815,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.9 (KHTML, like Gecko) Ubuntu/9.04 Chromium/7.0.530.0 Chrome/7.0.530.0 Safari/534.9*/
const Device UA359497 = {S94671, 143, { &P17968,&P5629,&P21558,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/1.9.1 Safari/533.1*/
const Device UA1940899 = {S94672, 143, { &P17968,&P5629,&P21558,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en_gb) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/1.9.4 Safari/533.1*/
const Device UA1940944 = {S94673, 143, { &P17968,&P5629,&P21558,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/1.9.4 Safari/533.1*/
const Device UA2022123 = {S94674, 143, { &P17968,&P5629,&P21558,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en_gb) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/1.9.4 Safari/533.1*/
const Device UA402827 = {S94675, 132, { &P15869,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Xoom Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA797449 = {S94676, 92, { &P15364,&P5690,&P21807,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.19pre) Gecko/X Prism zdesktop/7.1.4*/
const Device UA672769 = {S94677, 18, { &P15364,&P17017,&P22949,&P18092 }};	/*Useragent - Apple-PubSub/65.24*/
const Device UA697191 = {S94678, 15, { &P15364,&P17017,&P22949,&P18092 }};	/*Useragent - Apple-PubSub/59*/
const Device UA785872 = {S94679, 81, { &P15364,&P7133,&P21174,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; U; en; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Opera 11.11*/
const Device UA845756 = {S94680, 66, { &P15364,&P7133,&P21174,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; en) Opera 11.11*/
const Device UA824532 = {S94681, 132, { &P18282,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A200 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA826489 = {S94682, 42, { &P17490,&P18378,&P4811,&P18092 }};	/*Useragent - Opera/9.00 (Nintendo Wii; U; ; 1309-9; en)*/
const Device UA912205 = {S94683, 129, { &P15767,&P18117,&P18361,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.0m*/
const Device UA2157950 = {S94684, 127, { &P15767,&P18117,&P18361,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3 Sleipnir/2.0m*/
const Device UA1878523 = {S94685, 78, { &P15364,&P18145,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (X11; FreeBSD 8.2-RELEASE i386; U; en) Presto/2.8.131 Version/11.10*/
const Device UA1883919 = {S94686, 86, { &P17512,&P5567,&P4536,&P18092 }};	/*Useragent - Mozilla/5.0 Opera/9.5 (KDDI-CA3I; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00*/
const Device UA1898281 = {S94687, 101, { &P12558,&P5448,&P4779,&P18092 }};	/*Useragent - HTC_P3650 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; HTC_P3650)/UC Browser7.8.0.95*/
const Device UA2774737 = {S94688, 94, { &P12558,&P5448,&P4779,&P18092 }};	/*Useragent - HTC_Touch_Cruise_T4242 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA1957453 = {S94689, 144, { &P17595,&P17017,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Macintosh; en-US; Valve Steam GameOverlay/X; ) AppleWebKit/535.15 (KHTML, like Gecko) Chrome/18.0.989.0 Safari/535.11*/
const Device UA1959544 = {S94690, 157, { &P13123,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1984854 = {S94691, 155, { &P14225,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Desire HD Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1987069 = {S94692, 74, { &P20381,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.2; zh-cn; ZTE-C_N780; 320*480) UCWEB7.9.3.103/139/32580*/
const Device UA1992702 = {S94693, 165, { &P18241,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; LG-E400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2019394 = {S94694, 138, { &P21601,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.0; es-es; Mobii 7 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2028685 = {S94695, 159, { &P15798,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2950141 = {S94696, 159, { &P15483,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC ChaCha A810e Build/GRJ90; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031244 = {S94697, 69, { &P18241,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;LG-E400;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA2032782 = {S94698, 140, { &P21834,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; DROIDZ Mate Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2149989 = {S94699, 44, { &P14060,&P17485,&P17347,&P18092 }};	/*Useragent - sonyericssonw150i/UC Browser8.0.3.107/69/444*/
const Device UA2158991 = {S94700, 128, { &P15767,&P18117,&P20772,&P18092 }};	/*Useragent - MQQBrowser/2.1 Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3*/
const Device UA2314084 = {S94701, 130, { &P15767,&P18117,&P20772,&P18092 }};	/*Useragent - MQQBrowser/2.1 Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA2347235 = {S94702, 78, { &P15209,&P5768,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokia6233) UCBrowser8.4.0.159/70/444/UCWEB Mobile*/
const Device UA2223359 = {S94703, 53, { &P11929,&P5890,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1 (Nokia5230;SymbianOS/9.1 Series60/3.0)*/
const Device UA2291796 = {S94704, 79, { &P15364,&P7127,&P20900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; de-DE; rv:1.0.2) Gecko/X Netscape/7.01*/
const Device UA2303668 = {S94705, 68, { &P15364,&P18110,&P3807,&P18092 }};	/*Useragent - LANIX-LX11/R01/MT53 Browser/2.0/Obigo/Q03C Profile/MIDP-2.0/CLDC-1.1*/
const Device UA2335790 = {S94706, 50, { &P12581,&P5890,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 (Nokia500;SymbianOS/9.1 Series60/3.0)*/
const Device UA2373213 = {S94707, 54, { &P13324,&P5890,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8 (NokiaN8-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA2384326 = {S94708, 79, { &P12061,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE65)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA2404483 = {S94709, 118, { &P12671,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia700) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile*/
const Device UA2413124 = {S94710, 69, { &P20364,&P18110,&P20793,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; ELF-II; 1024*552) UCWEB1.0.1.113/147/800*/
const Device UA2413623 = {S94711, 132, { &P21425,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; STM803HC Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2417839 = {S94712, 139, { &P14917,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-KU5900 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2420879 = {S94713, 120, { &P16591,&P22784,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; micromaxq50) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2425768 = {S94714, 172, { &P18113,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2426922 = {S94715, 175, { &P19936,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A100/Micromax_A100_V1.0; 480*854;  CTC/2.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2439260 = {S94716, 133, { &P15364,&P17285,&P19992,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506) Sleipnir/2.6.1*/
const Device UA2441253 = {S94717, 52, { &P12088,&P5890,&P21746,&P18092 }};	/*Useragent - iBrowser/2.6 (NokiaX6-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA2453284 = {S94718, 161, { &P15798,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2455497 = {S94719, 151, { &P14433,&P5096,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2455807 = {S94720, 148, { &P22135,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SmartTab1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2463872 = {S94721, 71, { &P15364,&P18145,&P18565,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64) KHTML/4.8.4 (like Gecko) Konqueror/4.8*/
const Device UA2502494 = {S94722, 128, { &P23636,&P24094,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46HX853; CC/GBR; en) Presto/2.10.250 Version/11.60*/
const Device UA2506747 = {S94723, 136, { &P21455,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DM013SH Build/S0006) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2519881 = {S94724, 143, { &P15364,&P16110,&P22402,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Comodo_Dragon/10.0.0.2 Chrome/10.0.648.204 Safari/534.16*/
const Device UA2520447 = {S94725, 105, { &P15364,&P7127,&P18350,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; Sleipnir/2.9.1)*/
const Device UA2526640 = {S94726, 64, { &P20170,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; A95; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2526878 = {S94727, 154, { &P18673,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2527215 = {S94728, 138, { &P18686,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A810S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2529270 = {S94729, 72, { &P17595,&P17295,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:19.0) Gecko/X Firefox/19.0*/
const Device UA2529618 = {S94730, 138, { &P20081,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7108 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2552909 = {S94731, 31, { &P15803,&P5843,&P21724,&P18092 }};	/*Useragent - OneBrowser/3.0 (Nokia200/11.56)*/
const Device UA2578988 = {S94732, 162, { &P13042,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Incredible S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2591905 = {S94733, 150, { &P21624,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; i-mobile i-style Q1i Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2594843 = {S94734, 154, { &P20081,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-N7108 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2595798 = {S94735, 67, { &P15364,&P16110,&P18368,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:8.0) Gecko/X Thunderbird/8.0*/
const Device UA2599952 = {S94736, 141, { &P12521,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SO-02D Build/6.1.F.0.106) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2600346 = {S94737, 133, { &P22024,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; dL1 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2617819 = {S94738, 93, { &P15985,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Messenger; 1.8.3) (GT-P6200; samsung; GT-P6200; 3.2/HTJ85B)*/
const Device UA2623748 = {S94739, 164, { &P12879,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7510, 39, 1.8.2)*/
const Device UA2640001 = {S94740, 79, { &P15364,&P7133,&P20900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.0.2) Gecko/X Netscape/7.02*/
const Device UA2643619 = {S94741, 101, { &P15364,&P16110,&P22343,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.2.32) Gecko/X Firefox/3.6.32 (Palemoon/3.6.32)*/
const Device UA2652655 = {S94742, 147, { &P21840,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Woxter Tablet PC 85 HDS Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2654092 = {S94743, 128, { &P21311,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; CMP741E Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2660756 = {S94744, 162, { &P19907,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; C5155 Build/IML77) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2664427 = {S94745, 162, { &P21472,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A111 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2666716 = {S94746, 136, { &P22634,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; WEXLER-TAB-7T Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667441 = {S94747, 138, { &P13999,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; NexusHD2 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2669405 = {S94748, 156, { &P15575,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2676918 = {S94749, 130, { &P24100,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CAL21 Build/AX) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2693495 = {S94750, 146, { &P21649,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; POV_TAB-PROTAB30-IPS10 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2697917 = {S94751, 159, { &P13914,&P5843,&P24673,&P18092 }};	/*Useragent - Nokia5330-1d/5.0 (06.88) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5330-1d) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2701200 = {S94752, 91, { &P16024,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t310i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA3100070 = {S94753, 88, { &P16024,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; vi; lg-t310i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2701355 = {S94754, 94, { &P16545,&P22784,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq80) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2701448 = {S94755, 160, { &P22612,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8951 Build/HuaweiT8951) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2701575 = {S94756, 147, { &P18666,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; A25 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2706616 = {S94757, 130, { &P20523,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3105 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2708763 = {S94758, 134, { &P18314,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M032 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2708938 = {S94759, 156, { &P18464,&P18066,&P21707,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.9 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2709112 = {S94760, 154, { &P20542,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAP5000TDUO Build/PrestigioPAP5000TDUO) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712725 = {S94761, 138, { &P23381,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A5Classic Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713567 = {S94762, 127, { &P22242,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ME171 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2714832 = {S94763, 135, { &P22909,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Flame2 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2729908 = {S94764, 90, { &P15079,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MK16i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2732967 = {S94765, 143, { &P20507,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 7 Build/1.0.3 X-12:51) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2733522 = {S94766, 135, { &P21992,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.6; A9550 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2735186 = {S94767, 142, { &P20837,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IRIS_501 Build/LAVAIRIS501) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2748886 = {S94768, 89, { &P21996,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; S100) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2749917 = {S94769, 130, { &P18061,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; rk30sdk Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2751246 = {S94770, 147, { &P20868,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; M040 Build/JRO03H) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2765490 = {S94771, 100, { &P15106,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; HTC_Salsa_C510e) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2779046 = {S94772, 140, { &P11804,&P17483,&P24676,&P18092 }};	/*Useragent - LG-GW525 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gw525 teleca) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2793929 = {S94773, 89, { &P21472,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A111) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2798973 = {S94774, 155, { &P23728,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; O+ 8.52 Android Build/Oplus8.52_Android) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799679 = {S94775, 154, { &P17160,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Slider SL101 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799971 = {S94776, 62, { &P15364,&P17285,&P22768,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:22.0) Gecko/X Thunderbird/22.0*/
const Device UA2822112 = {S94777, 80, { &P23747,&P17484,&P17394,&P18092 }};	/*Useragent - SAMSUNG-GT-C3780/C3780JVLF2 NetFront/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2838532 = {S94778, 122, { &P11834,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA6760s-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile*/
const Device UA2845857 = {S94779, 154, { &P20096,&P18066,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7568 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2855553 = {S94780, 137, { &P19911,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-E970 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2860586 = {S94781, 157, { &P18455,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC T528d Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2861844 = {S94782, 131, { &P20187,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Novo7Fire Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2862336 = {S94783, 95, { &P21911,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; AMAZE_S200) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2863813 = {S94784, 101, { &P19975,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; Lumia 505)*/
const Device UA2867212 = {S94785, 135, { &P20267,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; P300(Funbook) Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2867298 = {S94786, 93, { &P13536,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6110) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2869415 = {S94787, 152, { &P20529,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A89 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2869638 = {S94788, 160, { &P21141,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Lenovo P770 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2875476 = {S94789, 124, { &P23853,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Phoenix Build/X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2877355 = {S94790, 137, { &P16436,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I8150 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877787 = {S94791, 137, { &P12496,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I897 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877965 = {S94792, 140, { &P23214,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6833 Build/14.1.B.1.320) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879173 = {S94793, 135, { &P23165,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; H7500+ Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879295 = {S94794, 141, { &P12798,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xperia Arc S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879412 = {S94795, 127, { &P23072,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; T-17B Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2889229 = {S94796, 80, { &P23754,&P5629,&P15760,&P18092 }};	/*Useragent - SAMSUNG-GT-S5698_TD/1.0 Android/2.3.6 Release/1.15.2012 Browser/AppleWebKit533.1*/
const Device UA2892059 = {S94797, 106, { &P18364,&P5129,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaC5-05*/
const Device UA2892462 = {S94798, 140, { &P24037,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALLVIEW_TX1_Quasar Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2892876 = {S94799, 132, { &P19996,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A15 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2935326 = {S94800, 93, { &P15649,&P5629,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; zh-CN; GT-S5570) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile*/
const Device UA2937477 = {S94801, 131, { &P15003,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M380W Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2938134 = {S94802, 97, { &P22862,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Micromax A88) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2944236 = {S94803, 91, { &P15518,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5230) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2963261 = {S94804, 96, { &P15518,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-s5230) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3013891 = {S94805, 91, { &P15518,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; es-LA; gt-s5230) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3092044 = {S94806, 88, { &P15518,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; fa; gt-s5230) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2955218 = {S94807, 140, { &P23514,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GL07S Build/HuaweiU9700L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2956596 = {S94808, 198, { &P22594,&P18598,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-M919 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.27/385,1.9.27.385,2013-03-31 15:31, +0000, us*/
const Device UA2957169 = {S94809, 139, { &P22590,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717M Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2957427 = {S94810, 164, { &P15364,&P18111,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; i-mobile i-style 8 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2960002 = {S94811, 147, { &P18636,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; VS930 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2962225 = {S94812, 158, { &P14993,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT15i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2964262 = {S94813, 154, { &P14207,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; ADR6350 Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2964353 = {S94814, 93, { &P18700,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-B5512) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2964722 = {S94815, 155, { &P14574,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Nexus S Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2964918 = {S94816, 138, { &P24407,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; KYY21 Build/100.0.2480) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2967059 = {S94817, 162, { &P13123,&P5629,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-S5360 Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60015*/
const Device UA2986887 = {S94818, 144, { &P23541,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; ZTE Grand Era Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2995504 = {S94819, 138, { &P14207,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; ADR6350 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2997571 = {S94820, 111, { &P15364,&P18145,&P22527,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD) AppleWebKit/535.22 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/535.22 Midori/0.5*/
const Device UA2998259 = {S94821, 162, { &P23829,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; G2 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33886*/
const Device UA2999089 = {S94822, 150, { &P14872,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2999543 = {S94823, 151, { &P24557,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9128 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3002118 = {S94824, 93, { &P16808,&P22784,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq7) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3006108 = {S94825, 133, { &P12972,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; SGH-T989 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3012805 = {S94826, 95, { &P14842,&P5477,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E10i) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3014246 = {S94827, 96, { &P21881,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Karbonn A4 ) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3014488 = {S94828, 148, { &P21493,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Fly_IQ444 Quattro Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3015554 = {S94829, 133, { &P22941,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-S6812 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3016921 = {S94830, 89, { &P21472,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; A111) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3018711 = {S94831, 88, { &P15364,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A12) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3019973 = {S94832, 93, { &P12487,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA5700) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3020630 = {S94833, 138, { &P24503,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E270S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020845 = {S94834, 96, { &P23116,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Aqua.Active) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3022021 = {S94835, 138, { &P22978,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6313T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022092 = {S94836, 138, { &P23237,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOLO X910 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022156 = {S94837, 137, { &P24583,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; EK-GC110 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022382 = {S94838, 139, { &P15517,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT15i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022710 = {S94839, 137, { &P18593,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S6010 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022694 = {S94840, 136, { &P22497,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; w30hd(QuadCore) Build/W30hd) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022975 = {S94841, 130, { &P19877,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T869 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023001 = {S94842, 135, { &P22681,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; UMI X1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023517 = {S94843, 135, { &P22927,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_E7310 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023615 = {S94844, 134, { &P23376,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M9000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023842 = {S94845, 136, { &P22991,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC 802t Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024096 = {S94846, 142, { &P23978,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WALKMAN Build/icx1227_E111) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3025851 = {S94847, 92, { &P14967,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; LG-P500) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3039391 = {S94848, 160, { &P20240,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Optimus G Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/geeb*/
const Device UA3048459 = {S94849, 152, { &P22964,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010X Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3049354 = {S94850, 154, { &P21596,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; mobiistar touch LAI 504 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050494 = {S94851, 74, { &P17595,&P21721,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:12.0) Gecko/X Firefox/12.0*/
const Device UA3055168 = {S94852, 135, { &P12616,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-in; WT19i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3061429 = {S94853, 157, { &P19859,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328w Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3062288 = {S94854, 99, { &P14353,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5250) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3064013 = {S94855, 132, { &P20807,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B92 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3078735 = {S94856, 149, { &P21794,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9152 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3079784 = {S94857, 152, { &P19859,&P18066,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3110119 = {S94858, 156, { &P22871,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9295 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3092745 = {S94859, 147, { &P16976,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT300 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3093223 = {S94860, 183, { &P22864,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A26 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3094920 = {S94861, 124, { &P21052,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A9 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3095750 = {S94862, 140, { &P20124,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ZTE Grand X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3108425 = {S94863, 170, { &P22607,&P18066,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S5303 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3109131 = {S94864, 175, { &P19862,&P18066,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax_A110 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3109169 = {S94865, 164, { &P13042,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC Incredible S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109776 = {S94866, 93, { &P18283,&P18111,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9300) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3111000 = {S94867, 98, { &P19862,&P18066,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax A110) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3111154 = {S94868, 159, { &P21141,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; Lenovo P770 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3113801 = {S94869, 137, { &P12482,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T959 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3115314 = {S94870, 152, { &P13644,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; EVO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3115608 = {S94871, 159, { &P18736,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One SV Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3116522 = {S94872, 93, { &P14186,&P5629,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5830) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3130742 = {S94873, 129, { &P18385,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3131442 = {S94874, 127, { &P24419,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ME302C Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3132948 = {S94875, 168, { &P15794,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA2482 = {S94876, 129, { &P11980,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) PPC; 240x320; HTC_P3300/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA4549 = {S94877, 120, { &P13617,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; MDA Vario/3.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA5174 = {S94878, 31, { &P17694,&P22839,&P4924,&P18092 }};	/*Useragent - ZTE-F866/1.0 ACS-NF/3.2 Qtv/4.3*/
const Device UA31496 = {S94879, 71, { &P12425,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-PEBL U6/08.84.2DR MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23421 = {S94880, 56, { &P14524,&P17017,&P18246,&P18092 }};	/*Useragent - NexianNX-NXT901/MTKRelease/11-20-2010Browser/MAUIProfile*/
const Device UA566648 = {S94881, 103, { &P14708,&P5204,&P4779,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5130c-2/UC Browser7.8.0.95/69/401*/
const Device UA44227 = {S94882, 29, { &P13810,&P6476,&P4902,&P18092 }};	/*Useragent - LG-LG510/1.0 Polaris/6.15.WEB*/
const Device UA48894 = {S94883, 123, { &P12940,&P17483,&P3459,&P18092 }};	/*Useragent - LG-T315i/V100 Obigo/Q7.3 MMS/LG-MMS-V1.1/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA50417 = {S94884, 110, { &P14345,&P16000,&P4737,&P18092 }};	/*Useragent - NokiaN-GageQD/1.0 SymbianOS/6.1 Series60/1.2 Profile/MIDP-1.0 Configuration/CLDC-1.0/UC Browser7.2.2.51/26/352*/
const Device UA53676 = {S94885, 65, { &P13246,&P17483,&P3499,&P18092 }};	/*Useragent - LG-GD710/V10f Teleca/Q7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA67436 = {S94886, 119, { &P14824,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-S5200/S5200DDIK3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA60564 = {S94887, 30, { &P12547,&P22839,&P3579,&P18092 }};	/*Useragent - ZTE-C77/1.0 SMIT-Browser/2.0.0*/
const Device UA63185 = {S94888, 65, { &P14486,&P17483,&P3534,&P18092 }};	/*Useragent - LG-CU915/V1.0h Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA80609 = {S94889, 9, { &P12487,&P5890,&P4030,&P18092 }};	/*Useragent - Nokia5700*/
const Device UA87074 = {S94890, 104, { &P12280,&P17129,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_0 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko)*/
const Device UA88382 = {S94891, 88, { &P11658,&P5776,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Nokia 6680/5.04.07; 9399) Opera 8.65 [en]*/
const Device UA90006 = {S94892, 31, { &P17756,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 L04B(c100;TB;W24H12)*/
const Device UA203426 = {S94893, 127, { &P22255,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Adam Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA560341 = {S94894, 128, { &P22255,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-gb; Adam Build/NIHCPX) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA427427 = {S94895, 112, { &P15767,&P18117,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.8.1*/
const Device UA601648 = {S94896, 112, { &P15767,&P18117,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.8.2*/
const Device UA603579 = {S94897, 112, { &P15767,&P18117,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.8.4*/
const Device UA615685 = {S94898, 114, { &P15767,&P18117,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.8.4*/
const Device UA708530 = {S94899, 112, { &P15767,&P18117,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.8.3*/
const Device UA773416 = {S94900, 114, { &P15767,&P18117,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.8.3*/
const Device UA455678 = {S94901, 88, { &P17595,&P17297,&P22314,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:11.0) Gecko/X Firefox/11.0 SeaMonkey/2.8*/
const Device UA2408580 = {S94902, 78, { &P15364,&P18145,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (X11; FreeBSD 8.2-RELEASE amd64; U; en) Presto/2.7.62 Version/11.01*/
const Device UA557833 = {S94903, 98, { &P14651,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; 7 Trophy)*/
const Device UA925739 = {S94904, 104, { &P14651,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; 7 Trophy T8686)*/
const Device UA566093 = {S94905, 127, { &P11980,&P5448,&P4779,&P18092 }};	/*Useragent - PPC; 240x320; HTC_P3300/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA606328 = {S94906, 138, { &P12280,&P17146,&P20783,&P18092 }};	/*Useragent - MQQBrowser/32 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3*/
const Device UA838053 = {S94907, 138, { &P12280,&P17146,&P20783,&P18092 }};	/*Useragent - MQQBrowser/32 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA646703 = {S94908, 78, { &P17595,&P17296,&P18192,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:9.0) Gecko/X Thunderbird/9.0.1*/
const Device UA655693 = {S94909, 135, { &P12814,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; SGH-I777 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022727 = {S94910, 133, { &P12814,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-us; SGH-I777 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032557 = {S94911, 137, { &P12814,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-I777 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA724518 = {S94912, 44, { &P12351,&P17484,&P17347,&P18092 }};	/*Useragent - samsung-gt-s3653w/UC Browser8.0.3.107/69/444*/
const Device UA730181 = {S94913, 115, { &P17595,&P17295,&P23600,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) Fluid/0.9.6 Safari/534.50.2*/
const Device UA762727 = {S94914, 74, { &P18182,&P5448,&P24666,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB8.1.0.104/31/800*/
const Device UA1934298 = {S94915, 74, { &P18182,&P5448,&P24666,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB8.1.0.104/40/999*/
const Device UA2308654 = {S94916, 74, { &P18182,&P5448,&P24666,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB8.1.0.104/31/999*/
const Device UA2309748 = {S94917, 74, { &P18182,&P5448,&P24666,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB8.1.0.104/40/800*/
const Device UA776267 = {S94918, 42, { &P13677,&P17484,&P4742,&P18092 }};	/*Useragent - samsung-gt-s5253/UC Browser7.5.1.77/70/351*/
const Device UA878105 = {S94919, 77, { &P15364,&P7133,&P18105,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; U; ru) Presto/2.9.168 Version/11.X-09-15 14:14:37*/
const Device UA1903240 = {S94920, 44, { &P13220,&P17485,&P17338,&P18092 }};	/*Useragent - sonyericssonk770i/UC Browser7.9.0.102/69/352*/
const Device UA1903890 = {S94921, 67, { &P15364,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2;Zh_cn;generic;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA2020933 = {S94922, 74, { &P15364,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2.2;Zh_cn;Vodafone 858;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA1951327 = {S94923, 173, { &P14842,&P5477,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E10i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2335521 = {S94924, 165, { &P14186,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA1982994 = {S94925, 81, { &P15364,&P5690,&P22551,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:13.0) Gecko/X Firefox/13.0.3.0 IceDragon/13.0.3.0*/
const Device UA2023059 = {S94926, 134, { &P22494,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; nl-nl; TAB450 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025032 = {S94927, 138, { &P22496,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; hu-hu; xTAB-100 Build/ICS.b02ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2025375 = {S94928, 156, { &P23988,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; ZTE-C_N700/N700V1.0.0B04; 240*400; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029365 = {S94929, 155, { &P19859,&P18066,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2352196 = {S94930, 166, { &P16362,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; SPH-D710 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2142032 = {S94931, 122, { &P17595,&P17296,&P24722,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.50 (KHTML, like Gecko) FluidApp/1.2 Safari/534.50*/
const Device UA2258658 = {S94932, 126, { &P17595,&P17296,&P24722,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) FluidApp/1.2 Safari/533.21.1*/
const Device UA2258668 = {S94933, 126, { &P17595,&P17296,&P24722,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) FluidApp/1.2 Safari/533.21.1*/
const Device UA2208964 = {S94934, 29, { &P15364,&P17017,&P18568,&P18092 }};	/*Useragent - Microsoft Office Outlook 2010*/
const Device UA2215585 = {S94935, 61, { &P15364,&P18110,&P24645,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.3;Zh_cn;C8800;480*800;)UCWEB7.5.0.66/139/7713*/
const Device UA2261509 = {S94936, 133, { &P21639,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M-MP860S2 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2337676 = {S94937, 155, { &P14969,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Desire S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2339743 = {S94938, 155, { &P14969,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2364687 = {S94939, 157, { &P15364,&P7133,&P23488,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.1.1.28744 (Firestorm-Release; firestorm skin) Safari/533.3*/
const Device UA2370577 = {S94940, 43, { &P14404,&P17484,&P17338,&P18092 }};	/*Useragent - samsung-gt-s5260/UC Browser7.9.0.102/69/352*/
const Device UA2384338 = {S94941, 137, { &P17283,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MB860 Build/6.7.7_95) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2408676 = {S94942, 44, { &P15364,&P18118,&P22484,&P18092 }};	/*Useragent - Opera/9.12 (X11; Linux i686; U; en) (Ubuntu)*/
const Device UA2410608 = {S94943, 129, { &P20579,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT570 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2423144 = {S94944, 90, { &P13734,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5603) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2430370 = {S94945, 90, { &P13734,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5603) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2425020 = {S94946, 147, { &P20120,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A45 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2438568 = {S94947, 127, { &P15364,&P5690,&P19992,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30) Sleipnir/2.6.0*/
const Device UA2455623 = {S94948, 161, { &P14202,&P5477,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E15i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2481455 = {S94949, 135, { &P21163,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ViewPad 10S Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2504607 = {S94950, 135, { &P22015,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M9108 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2506553 = {S94951, 132, { &P20523,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3105 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2520922 = {S94952, 135, { &P15649,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; GT-S5570 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2527053 = {S94953, 33, { &P18657,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SAMSUNG-GT-C3262)*/
const Device UA2527432 = {S94954, 135, { &P23925,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; W2430 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2528175 = {S94955, 137, { &P21254,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Polaris Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2528580 = {S94956, 139, { &P20294,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC T528w Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2528852 = {S94957, 134, { &P20438,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N400 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2530337 = {S94958, 137, { &P21355,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; KM-E100 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2547810 = {S94959, 132, { &P23421,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; TM-7037W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2552778 = {S94960, 147, { &P21064,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A44 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2553572 = {S94961, 81, { &P17595,&P18207,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; en) Presto/2.9.168 Version/11.51*/
const Device UA2554296 = {S94962, 57, { &P15364,&P17188,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 4.0) Presto/2.12.388 Version/12.10*/
const Device UA2559576 = {S94963, 184, { &P18113,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2560316 = {S94964, 136, { &P22095,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ThL W7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2574478 = {S94965, 141, { &P12765,&P18066,&P21753,&P18092 }};	/*Useragent - iBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2579099 = {S94966, 140, { &P20467,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Viettel I9 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2589482 = {S94967, 56, { &P12142,&P5890,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 (SonyEricssonU5;SymbianOS/9.1 Series60/3.0)*/
const Device UA2597944 = {S94968, 166, { &P18595,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I8190 Build/JZO54K; CyanogenMod-JellyTime_R30) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2598073 = {S94969, 165, { &P14017,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LG-P990 Build/IMM76L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2599246 = {S94970, 141, { &P21947,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; orbit 4300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2603540 = {S94971, 136, { &P20332,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax P275 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2604119 = {S94972, 135, { &P21823,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTX21 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2613998 = {S94973, 145, { &P15985,&P6437,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2617010 = {S94974, 146, { &P15985,&P6437,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 3.2; th-th; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2631466 = {S94975, 192, { &P14908,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, -0600, us*/
const Device UA2625246 = {S94976, 52, { &P17337,&P17979,&P22080,&P18092 }};	/*Useragent - WhatsApp/2.9.369 Android/3.2 Device/samsung-GT-P7500*/
const Device UA2652942 = {S94977, 128, { &P23619,&P24094,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40EX650; CC/VNM; vi) Presto/2.10.250 Version/11.60*/
const Device UA2654069 = {S94978, 157, { &P19898,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I699 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2657968 = {S94979, 136, { &P18195,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; YP-G70 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2660833 = {S94980, 164, { &P15184,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; LG-P920 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2663564 = {S94981, 100, { &P16373,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s7250d) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2667107 = {S94982, 128, { &P22814,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PC906 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2671912 = {S94983, 133, { &P17595,&P17296,&P21784,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.58.2 (KHTML, like Gecko) Version/5.1.8 Safari/6534.58.2 Sleipnir/4.1.2*/
const Device UA2698846 = {S94984, 169, { &P19999,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Karbonn A30 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2700964 = {S94985, 163, { &P21351,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI C8813 Build/HuaweiC8813) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2701371 = {S94986, 152, { &P21021,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328t Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2701470 = {S94987, 159, { &P12505,&P17483,&P24676,&P18092 }};	/*Useragent - LG-GX500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gx500) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2706352 = {S94988, 111, { &P18444,&P18148,&P24633,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; en-US; NOKIA; RM-826_im_india_201) U2/1.0.0 UCBrowser/2.9.0.263 U2/1.0.0 Mobile*/
const Device UA2708047 = {S94989, 137, { &P18747,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A840S Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712145 = {S94990, 152, { &P21112,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; X907 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2713517 = {S94991, 135, { &P21448,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SH530U Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713566 = {S94992, 137, { &P22162,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AUXUS CoreX4 3G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2713940 = {S94993, 142, { &P21378,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; tablet Fnac 8 Build/1.1.0 X-12:19) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2720899 = {S94994, 54, { &P13324,&P5890,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 (NokiaN8-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA2726760 = {S94995, 91, { &P19775,&P17979,&P18062,&P18092 }};	/*Useragent - Lenovo A2105 Linux/3.0 Android/4.0 Release/04.12.2012 DXBrowser/2.0 Profile/ Configuration/*/
const Device UA2734244 = {S94996, 173, { &P18295,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (HTC One X, 42, 1.9.1)*/
const Device UA2735145 = {S94997, 140, { &P20543,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A660 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735532 = {S94998, 135, { &P20267,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; P300(Funbook) Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2747183 = {S94999, 135, { &P23387,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; eZeeTab1003 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2748486 = {S95000, 94, { &P18213,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6500D) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2749153 = {S95001, 91, { &P13824,&P22839,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; zte-f970) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2762764 = {S95002, 41, { &P14743,&P17485,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SonyEricssonJ20i/R7CA064)*/
const Device UA2762953 = {S95003, 130, { &P22178,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 792TPC Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2765297 = {S95004, 91, { &P21273,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua_S) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2767766 = {S95005, 93, { &P16871,&P22784,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq5) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2779991 = {S95006, 150, { &P23960,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; T1-B Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2782010 = {S95007, 158, { &P14186,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2786672 = {S95008, 96, { &P15364,&P17818,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.6) Gecko/X Ubuntu/11.04 (natty) Firefox/3.6.6*/
const Device UA2795237 = {S95009, 155, { &P21857,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2797196 = {S95010, 130, { &P20425,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N-03E Build/AX) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2798855 = {S95011, 138, { &P21304,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Fly_IQ450 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799809 = {S95012, 135, { &P24585,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ZP800H Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2839149 = {S95013, 135, { &P21123,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; EM543 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2845120 = {S95014, 42, { &P20126,&P17485,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SonyEricssonK800i/R8BF003)*/
const Device UA2849727 = {S95015, 135, { &P23320,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-pt; eZee Tab972 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2856893 = {S95016, 69, { &P20141,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; GT-S5301; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2862302 = {S95017, 138, { &P23392,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; Ritmix RMD745 Build/FR-MX68) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2864734 = {S95018, 175, { &P17283,&P22212,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; MB860 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2866928 = {S95019, 137, { &P18447,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Xperia T Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867066 = {S95020, 130, { &P23218,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PMP5780D Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2867404 = {S95021, 160, { &P20262,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Karbonn_A18 Build/KarbonnA18) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2867550 = {S95022, 127, { &P20216,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M1062 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2867849 = {S95023, 137, { &P16436,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I8150 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867851 = {S95024, 130, { &P19879,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-I915 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2868038 = {S95025, 135, { &P22681,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; UMI X1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869560 = {S95026, 147, { &P16976,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT300 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2869796 = {S95027, 139, { &P17108,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Inspire 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869926 = {S95028, 156, { &P14123,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P970 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2872362 = {S95029, 133, { &P20427,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHL21 Build/S5170) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877773 = {S95030, 137, { &P20869,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A878 Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877811 = {S95031, 134, { &P23478,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZP810 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879166 = {S95032, 140, { &P22742,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GOCLEVER TAB M723G Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2879189 = {S95033, 131, { &P24524,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; V812 Core4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2879248 = {S95034, 138, { &P15759,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Vision Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879294 = {S95035, 157, { &P18559,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LG730 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2892643 = {S95036, 153, { &P23714,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; O+_8.9_Android Build/OplusOplus_A1000) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892916 = {S95037, 158, { &P19845,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE N9120 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2893011 = {S95038, 137, { &P18509,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-MS770 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893332 = {S95039, 126, { &P23232,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; F7HD Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2893810 = {S95040, 115, { &P15364,&P16110,&P21635,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36 slurp*/
const Device UA2896740 = {S95041, 160, { &P22612,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8951 Build/HuaweiT8951) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2897411 = {S95042, 148, { &P19783,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ME172V Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2949857 = {S95043, 37, { &P17595,&P18120,&P23675,&P18092 }};	/*Useragent - iCab/4.7 (Macintosh; U; PPC Mac OS X)*/
const Device UA2953440 = {S95044, 139, { &P18635,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus 4 Build/rs_v0.6) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2953701 = {S95045, 94, { &P15575,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9100G) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2957072 = {S95046, 92, { &P22651,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; LG-E435) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2957727 = {S95047, 126, { &P22485,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; th-th; T3 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2958226 = {S95048, 152, { &P18714,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V889D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2958313 = {S95049, 156, { &P18304,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2958636 = {S95050, 151, { &P22603,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I739 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2962728 = {S95051, 156, { &P22603,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-I739 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2964668 = {S95052, 151, { &P21057,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A7+ Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2984463 = {S95053, 140, { &P21998,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T327t Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987580 = {S95054, 136, { &P21880,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DM-992 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2995843 = {S95055, 82, { &P22042,&P17979,&P18112,&P18092 }};	/*Useragent - SAMSUNG-GT-P3108_TD/1.0 Android/4.1.2 Release/12.21.2012 Browser/AppleWebKit534.30*/
const Device UA2997016 = {S95056, 149, { &P18496,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6410LVW Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2997343 = {S95057, 147, { &P18174,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VS920 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2997934 = {S95058, 153, { &P18009,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCEVODesign4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2998153 = {S95059, 159, { &P24653,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A810 Build/HOANG MINH H?U_VEGAVIETTEAM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998590 = {S95060, 151, { &P15802,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC_Amaze_4G Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2999184 = {S95061, 148, { &P13124,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PantechP9070 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2999767 = {S95062, 152, { &P19862,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3000776 = {S95063, 133, { &P23183,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T1-D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3002800 = {S95064, 156, { &P18214,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3003369 = {S95065, 134, { &P20947,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.3; en-us; HTCEVOV4G Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3005967 = {S95066, 136, { &P13018,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.4; en-in; MT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3007306 = {S95067, 161, { &P19898,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; SCH-I699 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3007443 = {S95068, 129, { &P16598,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; A500 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3009916 = {S95069, 159, { &P20004,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A2+ Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3010927 = {S95070, 134, { &P19845,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; ZTE N9120 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3011666 = {S95071, 80, { &P17595,&P17296,&P22170,&P18090 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:22.0) Gecko/X Firefox/22.0 slurp*/
const Device UA3011942 = {S95072, 163, { &P14202,&P5477,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E15i Build/2.1.1.A.0.6) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3012590 = {S95073, 94, { &P23118,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua Star) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3013309 = {S95074, 173, { &P21857,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3013621 = {S95075, 129, { &P12765,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; A100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3013890 = {S95076, 158, { &P24132,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; iris504Q Build/iris504Q) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3015822 = {S95077, 88, { &P20265,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A52) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3018822 = {S95078, 162, { &P18507,&P5629,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-I8160 Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60015*/
const Device UA3019897 = {S95079, 160, { &P21738,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; iris502 Build/LAVAIRIS502) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3021033 = {S95080, 152, { &P24073,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033M Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021061 = {S95081, 128, { &P20153,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AN7HG3 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021592 = {S95082, 138, { &P24127,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Revo_Lead Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021790 = {S95083, 139, { &P24168,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAP4055DUO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021805 = {S95084, 143, { &P23138,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; TRUE BEYOND 3G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021950 = {S95085, 133, { &P20427,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHL21 Build/S2140) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022364 = {S95086, 152, { &P20541,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAP4500DUO Build/PrestigioPAP4500DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023041 = {S95087, 143, { &P23402,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; BLU STUDIO 5.0 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023631 = {S95088, 139, { &P20006,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT30a Build/7.0.A.3.195) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023718 = {S95089, 134, { &P23981,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; 9300+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024024 = {S95090, 141, { &P20091,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5880D Build/PMP5880D_v1.0.0_X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3034271 = {S95091, 136, { &P20240,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Optimus G Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3050106 = {S95092, 143, { &P21020,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Micromax A116 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3057482 = {S95093, 160, { &P18713,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-P769 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/p769*/
const Device UA3058782 = {S95094, 94, { &P16545,&P22784,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq80) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3059293 = {S95095, 148, { &P24366,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SM-T311 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3078260 = {S95096, 133, { &P18471,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; XT907 Build/9.8.1Q-78) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3079488 = {S95097, 131, { &P20265,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; A52 Build/A52) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079891 = {S95098, 93, { &P19847,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-S6012) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3115611 = {S95099, 149, { &P24365,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SM-T211 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3093392 = {S95100, 97, { &P11747,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_Desire_V) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3108738 = {S95101, 134, { &P20518,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; A10 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3109303 = {S95102, 138, { &P24503,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E270S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109554 = {S95103, 164, { &P18614,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT890 Build/9.8.2I-50_SMI-26) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3109647 = {S95104, 175, { &P21020,&P18111,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Micromax_A116 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3110061 = {S95105, 102, { &P14186,&P18110,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-S5830) U2/1.0.0 UCBrowser/9.3.0.321 Mobile*/
const Device UA3110312 = {S95106, 88, { &P18666,&P5629,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A25) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3110544 = {S95107, 139, { &P18302,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT25i Build/4.1.B.0.631) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110732 = {S95108, 137, { &P24425,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ONE TOUCH TAB 7 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3110929 = {S95109, 140, { &P21980,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; UBISLATE7C+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111572 = {S95110, 93, { &P22382,&P18111,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8262) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3112471 = {S95111, 93, { &P18442,&P18111,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N7100) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3112887 = {S95112, 140, { &P24161,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Orange Zali Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114558 = {S95113, 139, { &P20738,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Andromax U Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3115046 = {S95114, 93, { &P22872,&P18111,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S5282) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3115173 = {S95115, 131, { &P21052,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3115178 = {S95116, 156, { &P22607,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S5303 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3116534 = {S95117, 99, { &P19862,&P18598,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Micromax A110Q) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3128417 = {S95118, 154, { &P12059,&P5204,&P24680,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia2690) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3131264 = {S95119, 42, { &P15364,&P23096,&P18245,&P18092 }};	/*Useragent - Bing/4.1.1 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA5163 = {S95120, 55, { &P15759,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_wwe / 1.0 / htc_vision / 2.2.1*/
const Device UA28082 = {S95121, 53, { &P15759,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_wwe / 1.0 / htc_vision / 2.2*/
const Device UA38837 = {S95122, 60, { &P15759,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_asia_wwe / 1.0 / htc_vision / 2.3.3*/
const Device UA38843 = {S95123, 55, { &P15759,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_wwe / 1.0 / htc_vision / 2.3.3*/
const Device UA76876 = {S95124, 62, { &P15759,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_asia_india / 1.0 / htc_vision / 2.3.3*/
const Device UA13655 = {S95125, 84, { &P11874,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-i300_VN/S102 LMP/LML Release/2009.08.13 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA16373 = {S95126, 84, { &P11874,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-i300_VN/S101 LMP/LML Release/2009.08.03 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA34476 = {S95127, 81, { &P11874,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-i300/S003 LMP/LML Release/2009.03.19 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA20718 = {S95128, 40, { &P13719,&P17485,&P4203,&P18092 }};	/*Useragent - SonyEricssonJ230c/R5J TelecaBrowser/4.08*/
const Device UA62927 = {S95129, 40, { &P13719,&P17485,&P4203,&P18092 }};	/*Useragent - SonyEricssonJ230i/R5N TelecaBrowser/4.08*/
const Device UA61868 = {S95130, 65, { &P14342,&P17483,&P3534,&P18092 }};	/*Useragent - LG-CE110/V1.0i Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22602 = {S95131, 13, { &P15494,&P17483,&P17660,&P18092 }};	/*Useragent - LG/U8360/v1.0*/
const Device UA23258 = {S95132, 103, { &P15582,&P17483,&P3728,&P18092 }};	/*Useragent - LG-KB775F Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA31171 = {S95133, 153, { &P12409,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510M/UBHL2 Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA31172 = {S95134, 147, { &P12409,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Samsung/I8510M Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA57755 = {S95135, 107, { &P12442,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-T439/T439UVGI7 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA32872 = {S95136, 78, { &P12785,&P5448,&P4578,&P18092 }};	/*Useragent - HPiPAQrw6815/1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)*/
const Device UA43584 = {S95137, 60, { &P15364,&P17017,&P4977,&P18092 }};	/*Useragent - Mozilla/4.0 (MobilePhone PL6600/US/1.0) NetFront/3.1 MMP/2.0*/
const Device UA84582 = {S95138, 61, { &P15364,&P17017,&P4977,&P18092 }};	/*Useragent - Mozilla/4.0 (MobilePhone VI-2300/US/1.0) NetFront/3.1 MMP/2.0*/
const Device UA62513 = {S95139, 112, { &P14416,&P17483,&P3728,&P18092 }};	/*Useragent - LG-KS500/v10a Browser/Teleca-Q7.1/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA64707 = {S95140, 139, { &P14416,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/4.0 (Vodafone/1.0/LG-KS500/V10b Browser/Teleca-Q7.1/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA62775 = {S95141, 39, { &P14448,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GS205 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA64118 = {S95142, 99, { &P11813,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-SGH-F488I/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA67592 = {S95143, 142, { &P21988,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; PD_Novel Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA70224 = {S95144, 65, { &P15233,&P17483,&P17602,&P18092 }};	/*Useragent - LG-A130/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA87189 = {S95145, 47, { &P17593,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3D UP.Browser/6.2.0.13.1.5 (GUI) MMP/2.0*/
const Device UA111877 = {S95146, 13, { &P17981,&P17017,&P17470,&P18092 }};	/*Useragent - Q-mobile Q131*/
const Device UA337631 = {S95147, 122, { &P15364,&P17285,&P22408,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.2.0.0 Chrome/13.0.782.218 Safari/535.1*/
const Device UA1897364 = {S95148, 129, { &P15364,&P17285,&P22408,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Comodo_Dragon/13.2.0.0 Chrome/13.0.782.218 Safari/535.1*/
const Device UA347782 = {S95149, 123, { &P12280,&P17146,&P22246,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.7.1*/
const Device UA351533 = {S95150, 121, { &P12280,&P17146,&P22246,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.7*/
const Device UA352603 = {S95151, 121, { &P12280,&P17146,&P22246,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Flipboard/1.7.1*/
const Device UA366220 = {S95152, 123, { &P12280,&P17146,&P22246,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.7.2*/
const Device UA643873 = {S95153, 123, { &P12280,&P17146,&P22246,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Flipboard/1.7.2*/
const Device UA366481 = {S95154, 40, { &P17968,&P17979,&P23031,&P18092 }};	/*Useragent - Dolphin http client/7.3.0(116) (Android)*/
const Device UA621916 = {S95155, 45, { &P17968,&P17979,&P23031,&P18092 }};	/*Useragent - Dolphin http client/7.3.1 Beta(118) (Android)*/
const Device UA369719 = {S95156, 146, { &P21860,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-tw; ViewSonic-ViewPad4 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2839923 = {S95157, 146, { &P21860,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; Viewsonic-ViewPad4 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA379188 = {S95158, 53, { &P14188,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (Nokia5233;SymbianOS/9.1 Series60/3.0)*/
const Device UA412753 = {S95159, 126, { &P17595,&P17296,&P17237,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_0; en-US) AppleWebKit/528.10 (KHTML, like Gecko) Chrome/2.0.157.2 Safari/528.10*/
const Device UA858457 = {S95160, 124, { &P17595,&P17296,&P17237,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-US) AppleWebKit/530.6 (KHTML, like Gecko) Chrome/2.0.174.0 Safari/530.6*/
const Device UA427277 = {S95161, 31, { &P15364,&P6704,&P18398,&P18092 }};	/*Useragent - Opera/4.02 (Windows 98; U) [en]*/
const Device UA448584 = {S95162, 71, { &P13042,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux;U;2.2.1;Zh_cn;HTC Incredible S;480*800;)UCWEB7.6.0.75/139/800*/
const Device UA1992344 = {S95163, 74, { &P13042,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux; U;2.3.5;zh_cn;HTC Incredible S;480*800;)UCWEB7.6.0.75/139/32485*/
const Device UA768733 = {S95164, 171, { &P16356,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA483310 = {S95165, 95, { &P15364,&P7127,&P22481,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9.0.16) Gecko/X Firefox/3.0.16 Flock/2.5.6*/
const Device UA507443 = {S95166, 81, { &P15364,&P5690,&P23703,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.7.5) Gecko/X Thunderbird/0.9*/
const Device UA508564 = {S95167, 145, { &P20501,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; Nova A100 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA559068 = {S95168, 44, { &P17595,&P17296,&P18379,&P18092 }};	/*Useragent - MacOutlook/14.10.0.X (Intel Mac OS X 10.6.8)*/
const Device UA569313 = {S95169, 49, { &P14358,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (E63-1;SymbianOS/9.1 Series60/3.0)*/
const Device UA2213539 = {S95170, 47, { &P14358,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (E63;SymbianOS/9.1 Series60/3.0)*/
const Device UA719343 = {S95171, 135, { &P14877,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; GT-I9000 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA763655 = {S95172, 137, { &P14877,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9000 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034543 = {S95173, 137, { &P14877,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; GT-I9000 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA757076 = {S95174, 47, { &P13648,&P5055,&P17347,&P18092 }};	/*Useragent - nokiae7-00/UC Browser8.0.3.107/69/444  3gpp-gba*/
const Device UA758459 = {S95175, 126, { &P23624,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40CX523; CC/GBR; en) Presto/2.7.61 Version/11.00*/
const Device UA763670 = {S95176, 109, { &P15364,&P18110,&P18209,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.0.3.2197 Safari/532.4*/
const Device UA806381 = {S95177, 65, { &P15575,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.4;Zh_cn;GT-I9100;480*800;)UCWEB7.6.0.75/139/32092*/
const Device UA924138 = {S95178, 76, { &P17595,&P18579,&P18537,&P18092 }};	/*Useragent - Reeder/1010.79.00 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacBookAir4%2C2)*/
const Device UA1953979 = {S95179, 73, { &P17595,&P18579,&P18537,&P18092 }};	/*Useragent - Reeder/1010.79.00 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacBook5%2C1)*/
const Device UA1966465 = {S95180, 71, { &P17595,&P18579,&P18537,&P18092 }};	/*Useragent - Reeder/1010.79.00 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (iMac12%2C2)*/
const Device UA1978235 = {S95181, 76, { &P17595,&P18579,&P18537,&P18092 }};	/*Useragent - Reeder/1010.79.00 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacBookPro6%2C2)*/
const Device UA2336090 = {S95182, 72, { &P17595,&P18579,&P18537,&P18092 }};	/*Useragent - Reeder/1010.79.00 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacPro3%2C1)*/
const Device UA925874 = {S95183, 102, { &P16237,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i937)*/
const Device UA1859549 = {S95184, 80, { &P15364,&P6704,&P18199,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.8.1.24) Gecko/X Thunderbird/2.0.0.24*/
const Device UA1935216 = {S95185, 87, { &P17976,&P5477,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.9.201 Version/11.50*/
const Device UA2022333 = {S95186, 95, { &P17976,&P5477,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.1-update1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.9.201 Version/11.50*/
const Device UA1952999 = {S95187, 44, { &P16384,&P17484,&P17347,&P18092 }};	/*Useragent - samsung-gt-s5380k/UC Browser8.0.3.107/69/352*/
const Device UA1958482 = {S95188, 167, { &P13644,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PC36100 Build/IMM76I; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033451 = {S95189, 167, { &P13644,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PC36100 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1970087 = {S95190, 171, { &P14433,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2033935 = {S95191, 171, { &P14433,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA1995076 = {S95192, 79, { &P16072,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN78)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2020489 = {S95193, 140, { &P22232,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.1.5; fr-fr; A101B2-LZ Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2023183 = {S95194, 155, { &P11747,&P24694,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; HTC Desire Build/MIUI-XJ) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Mobile Maxthon*/
const Device UA2025102 = {S95195, 141, { &P22176,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; AraTop AS-03 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3049739 = {S95196, 144, { &P15649,&P5096,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2145934 = {S95197, 126, { &P23633,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL26EX320; CC/GBR; en) Presto/2.7.61 Version/11.00*/
const Device UA2147756 = {S95198, 67, { &P20295,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux; U;2.3.4;zh_cn;HTC S610d;480*800;)UCWEB7.6.0.75/139/32449*/
const Device UA2166546 = {S95199, 86, { &P12399,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU8i)/UC Browser8.5.0.183/50/352/UCWEB Mobile*/
const Device UA2229549 = {S95200, 103, { &P15364,&P18144,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA2261417 = {S95201, 37, { &P13554,&P5890,&P17338,&P18092 }};	/*Useragent - nokian79-1/UC Browser7.9.0.102/69/352*/
const Device UA2285408 = {S95202, 82, { &P17595,&P18120,&P22508,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.8b2) Gecko Camino/0.9+*/
const Device UA2592474 = {S95203, 37, { &P14635,&P5843,&P23405,&P18092 }};	/*Useragent - MQQBrowser/Mini2.8 (NokiaC3-00/07.20)*/
const Device UA2444067 = {S95204, 54, { &P13552,&P5890,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8 (NokiaC6-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA2445725 = {S95205, 89, { &P22667,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; Xbox)*/
const Device UA2455374 = {S95206, 153, { &P21388,&P5096,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; Karbonn_A1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2466089 = {S95207, 155, { &P15047,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; SK17i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2482329 = {S95208, 99, { &P15875,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonck15i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2528849 = {S95209, 138, { &P18174,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; VS920 4G Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2529610 = {S95210, 89, { &P12856,&P17483,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-t300) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2544175 = {S95211, 52, { &P13552,&P5890,&P21746,&P18092 }};	/*Useragent - iBrowser/2.6 (NokiaC6-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA2553039 = {S95212, 154, { &P18679,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-B5330 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2553396 = {S95213, 153, { &P20290,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A11 Build/KarbonnA11) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2554588 = {S95214, 81, { &P15575,&P18110,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Adr (Linux; U; 4.0.4; zh-cn; GT-I9100G Build/IMM76D.ZCLPL;480*800)*/
const Device UA2583007 = {S95215, 132, { &P22614,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ja-jp; N-04E Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2587386 = {S95216, 54, { &P13324,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (NokiaN8-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA2588813 = {S95217, 132, { &P19971,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; T700 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2589303 = {S95218, 58, { &P12142,&P5890,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 (SonyEricssonU5;SymbianOS/9.1 Series60/3.0)*/
const Device UA2594649 = {S95219, 157, { &P20452,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S880 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2596515 = {S95220, 130, { &P23609,&P18110,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-40EX645; CC/CAN; en) Presto/2.10.250 Version/11.60*/
const Device UA2608337 = {S95221, 175, { &P13452,&P5096,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO; 600*1024) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/800*/
const Device UA2616101 = {S95222, 172, { &P13452,&P5096,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO; 600*1024) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/352*/
const Device UA2643828 = {S95223, 126, { &P15364,&P5690,&P20433,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; N; Windows NT 5.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth Pro/6.2.2.6613 Safari/532.4*/
const Device UA2652224 = {S95224, 43, { &P17595,&P17297,&P21585,&P18092 }};	/*Useragent - MacOutlook/14.0.0.X (Intel Mac OS X 10.7.5)*/
const Device UA2652639 = {S95225, 153, { &P18621,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; LG-E612 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2657167 = {S95226, 138, { &P18630,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A800S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2661713 = {S95227, 160, { &P18083,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; V9C Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2662510 = {S95228, 163, { &P13964,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I5510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2662780 = {S95229, 169, { &P20452,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; Lenovo S880 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2665592 = {S95230, 164, { &P14207,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; ADR6350 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2667062 = {S95231, 129, { &P17965,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-01D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2667114 = {S95232, 153, { &P20541,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAP4500DUO Build/PrestigioPAP4500DUO) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667182 = {S95233, 134, { &P21226,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; E860 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667305 = {S95234, 135, { &P15389,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U8800 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667692 = {S95235, 167, { &P15201,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC Sensation Z710e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2667828 = {S95236, 134, { &P18238,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; E330 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668045 = {S95237, 156, { &P21321,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9228 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2668616 = {S95238, 133, { &P23171,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; G2S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2674848 = {S95239, 133, { &P20427,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHL21 Build/S2140) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2696766 = {S95240, 152, { &P21630,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile i-NOTE WIFI 2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2700737 = {S95241, 160, { &P17968,&P24693,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PIXELDROID HD2 Build/PIXEL_AOKP_JELLY_BEAN) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2700827 = {S95242, 167, { &P18280,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; HTC One S Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2703450 = {S95243, 169, { &P15079,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; MK16i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2704539 = {S95244, 168, { &P16319,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; U8860 Build/HuaweiU8860) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2705738 = {S95245, 91, { &P14107,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAN76) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2706472 = {S95246, 163, { &P14183,&P5096,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2707191 = {S95247, 130, { &P18061,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; rk30sdk Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2707433 = {S95248, 138, { &P18630,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A800S Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2708910 = {S95249, 133, { &P18225,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; U9200 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2728566 = {S95250, 126, { &P23814,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; U2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2728955 = {S95251, 133, { &P20475,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ENERGY i8 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2730570 = {S95252, 139, { &P13685,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-LS670 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2734764 = {S95253, 162, { &P18507,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-I8160 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Mobile Safari/537.31 OPR/14.0.1074.54070*/
const Device UA2734887 = {S95254, 136, { &P17324,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9001 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734924 = {S95255, 140, { &P22632,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Titanium S1 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735058 = {S95256, 137, { &P21481,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A860S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2736658 = {S95257, 107, { &P15364,&P5690,&P19839,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 slurp*/
const Device UA2736944 = {S95258, 134, { &P23758,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i9430 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2748020 = {S95259, 148, { &P12784,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; Skate Build/MIUI-2013-01-03-Faia) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2751554 = {S95260, 145, { &P15555,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; myTouch_4G_Slide Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2879470 = {S95261, 99, { &P14985,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8530) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2763475 = {S95262, 160, { &P20076,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Acro S Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2766731 = {S95263, 143, { &P20883,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 2 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2768328 = {S95264, 136, { &P20714,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID8065 Build/ICS.MID8065.X) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2778789 = {S95265, 126, { &P23313,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; M8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2779034 = {S95266, 90, { &P12968,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; LT26i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2798975 = {S95267, 131, { &P21052,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799323 = {S95268, 138, { &P20248,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V889M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799334 = {S95269, 166, { &P19863,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2799806 = {S95270, 126, { &P21817,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VP10 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2844074 = {S95271, 40, { &P11689,&P5890,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (NokiaN70-1/5.0737.3.0.1)*/
const Device UA2849245 = {S95272, 132, { &P23810,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; C12 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2850181 = {S95273, 133, { &P21994,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; P85(A9D3) Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2854600 = {S95274, 93, { &P13123,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; GT-S5360) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2855248 = {S95275, 138, { &P23568,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lovme X20 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2856010 = {S95276, 128, { &P20216,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M1062 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2856092 = {S95277, 129, { &P20563,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ja-jp; AT500 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2857987 = {S95278, 140, { &P23380,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; Boreal(FJA-7A-P) Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2859261 = {S95279, 183, { &P21475,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A35 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2860040 = {S95280, 94, { &P23140,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Andi_3.5i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2860663 = {S95281, 162, { &P16362,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-D710 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2860802 = {S95282, 95, { &P23309,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Celkon_A75) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2866910 = {S95283, 145, { &P20509,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A888 Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867312 = {S95284, 154, { &P18450,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; ST23i Build/11.0.A.5.5) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2867604 = {S95285, 141, { &P20544,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Maxwell_Lite Build/1.0.2-X-19:27) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2867619 = {S95286, 170, { &P19891,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; SAMSUNG-SGH-I717 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2868106 = {S95287, 78, { &P17595,&P18207,&P22768,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:22.0) Gecko/X Thunderbird/22.0*/
const Device UA2869012 = {S95288, 122, { &P15767,&P18432,&P22883,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Incredimail/1.2.1.4897*/
const Device UA2877859 = {S95289, 137, { &P21949,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F100L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2878477 = {S95290, 94, { &P21502,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; LEMON_P7_) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2879207 = {S95291, 136, { &P18305,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P936 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879245 = {S95292, 124, { &P21052,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A9 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2879395 = {S95293, 137, { &P16436,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8150 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879442 = {S95294, 157, { &P18458,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I200 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2890457 = {S95295, 146, { &P22686,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Primo_F1 Build/WALTONPrimo_F1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2890929 = {S95296, 126, { &P21383,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Lenovo-A288t_TD/S100 Linux/2.6.35 Android/2.3.5 Release/02.29.2012 Browser/AppleWebkit533.1 Mobile Safari/533.1*/
const Device UA2890962 = {S95297, 166, { &P22704,&P18066,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; th-th; i-mobile i-STYLE Q 5 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892436 = {S95298, 133, { &P20862,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IRULU-AL101 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2892773 = {S95299, 138, { &P20511,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Fly_IQ442 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892990 = {S95300, 138, { &P23237,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOLO X910 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893071 = {S95301, 133, { &P18238,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; E330 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2897407 = {S95302, 127, { &P20812,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT330 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2937511 = {S95303, 134, { &P21101,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ZP100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2954359 = {S95304, 140, { &P20117,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; Fly_IQ440 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956563 = {S95305, 91, { &P14763,&P5776,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian72) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2957550 = {S95306, 150, { &P18672,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2957773 = {S95307, 170, { &P22872,&P18111,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5282 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2958332 = {S95308, 147, { &P18301,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; VS840 4G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2962569 = {S95309, 158, { &P18449,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; ST27i Build/6.2.A.1.100) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2966181 = {S95310, 165, { &P18645,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I747 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2977980 = {S95311, 116, { &P15364,&P18110,&P17237,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686 (x86_64); en-US) AppleWebKit/530.7 (KHTML, like Gecko) Chrome/2.0.175.0 Safari/530.7*/
const Device UA2992902 = {S95312, 157, { &P23015,&P18111,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Lenovo A820 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2993965 = {S95313, 96, { &P15364,&P16110,&P21897,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.2; WOW64; Trident/5.0; chromeframe/12.0.742.112)*/
const Device UA2998275 = {S95314, 140, { &P24504,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Lenovo A830 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2998415 = {S95315, 156, { &P14186,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2999257 = {S95316, 128, { &P23824,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M870 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2999525 = {S95317, 143, { &P19904,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C5170 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3003769 = {S95318, 139, { &P24582,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; ZTE T790 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3004367 = {S95319, 147, { &P18641,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-M830 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3004535 = {S95320, 169, { &P20729,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A27 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3004882 = {S95321, 162, { &P21134,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; iBall Andi4.5q Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3006604 = {S95322, 164, { &P21383,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A288t Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3010714 = {S95323, 95, { &P16600,&P22784,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx600) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3013528 = {S95324, 164, { &P15501,&P5477,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E16i Build/2.1.1.A.0.53) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3016337 = {S95325, 100, { &P14831,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s7233e) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3018390 = {S95326, 97, { &P18462,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; HTC Desire X) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3020692 = {S95327, 129, { &P23767,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-7240 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3020843 = {S95328, 132, { &P22024,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; dL1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021465 = {S95329, 148, { &P24158,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Telenor Smart Pro 2 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021560 = {S95330, 144, { &P23153,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ultrafone 701HD Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021773 = {S95331, 134, { &P21006,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; L-05D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022331 = {S95332, 138, { &P22866,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AT-AS43D3 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022852 = {S95333, 133, { &P23086,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Fusion Bolt Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022997 = {S95334, 135, { &P20416,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; F-04E Build/V14R31C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023100 = {S95335, 142, { &P19781,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 97 TITANIUMHD Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3046475 = {S95336, 92, { &P18715,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; MID8048) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3049459 = {S95337, 150, { &P20408,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ZTE V768 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA3049618 = {S95338, 132, { &P16044,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; TouchPad Build/JDQ39E) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3049910 = {S95339, 146, { &P21935,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E980 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3051156 = {S95340, 138, { &P14186,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3058072 = {S95341, 133, { &P20556,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A701 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3059203 = {S95342, 159, { &P18582,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT28h Build/6.2.B.0.211) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3061786 = {S95343, 149, { &P21792,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9200 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3078048 = {S95344, 176, { &P22594,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-M919 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/jfltetmo*/
const Device UA3078588 = {S95345, 151, { &P18637,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9260 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3079882 = {S95346, 162, { &P18283,&P18598,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-I535 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA3080694 = {S95347, 90, { &P18303,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; MT27i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3084094 = {S95348, 157, { &P18621,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E612f Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3127843 = {S95349, 156, { &P18621,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; LG-E610 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3089112 = {S95350, 174, { &P23107,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; TA-FONE A37 HD Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33756*/
const Device UA3103247 = {S95351, 129, { &P18715,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID8048 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3108974 = {S95352, 169, { &P18241,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-E400 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3109171 = {S95353, 94, { &P11964,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NokiaE75-1) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3109305 = {S95354, 134, { &P23478,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ZP810 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109478 = {S95355, 96, { &P22209,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Titanium_S5) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3111353 = {S95356, 159, { &P13018,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; MT11i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3115006 = {S95357, 140, { &P24700,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Studio 5.3s Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3115049 = {S95358, 155, { &P24365,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SM-T211 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3117936 = {S95359, 114, { &P15364,&P16110,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Version/3.1.2 Safari/525.21*/
const Device UA3131391 = {S95360, 158, { &P23772,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA7288 = {S95361, 111, { &P13281,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHM300/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA11162 = {S95362, 155, { &P14467,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android SMB Cricket Transform; en-us; SPH-M920 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA55139 = {S95363, 164, { &P14467,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Shiftr182 Point-Blank ROM v2.3; en-us; SPH-M920 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA12326 = {S95364, 70, { &P16210,&P5448,&P22270,&P18092 }};	/*Useragent - Xda_Stellar/ 240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA61867 = {S95365, 83, { &P14341,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK800c/R8BF Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA51988 = {S95366, 83, { &P13962,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK310a/R4DA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA18173 = {S95367, 141, { &P14025,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; C8600 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2021265 = {S95368, 127, { &P14025,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; C8600 Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA22357 = {S95369, 79, { &P15434,&P17484,&P4924,&P18092 }};	/*Useragent - SAMSUNG-SGH-T519/T519UVFG8 Profile/MIDP-2.0 Configuration/CLDC-1.1 NetFront/3.2*/
const Device UA23746 = {S95370, 135, { &P15624,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (iDEN; U; CPU iDEN Android Eclair Creekside; en-us) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA23975 = {S95371, 47, { &P15642,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-VX5400/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA31504 = {S95372, 79, { &P12427,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-T109/T109UVHG7 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA44693 = {S95373, 39, { &P17983,&P17017,&P17470,&P18092 }};	/*Useragent - Haier EC-6060/Bakrie Telecom/BOLT v2.31*/
const Device UA54289 = {S95374, 95, { &P14172,&P19950,&P4893,&P18092 }};	/*Useragent - SAGEM-my700x/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.f.1.161 (GUI)*/
const Device UA63712 = {S95375, 95, { &P14172,&P19950,&P4893,&P18092 }};	/*Useragent - SAGEM-my700x/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.f.1.162 (GUI)*/
const Device UA62943 = {S95376, 12, { &P14461,&P17483,&P17660,&P18092 }};	/*Useragent - LG/U970/v1.0*/
const Device UA67008 = {S95377, 118, { &P11669,&P5776,&P4715,&P18092 }};	/*Useragent - Nokia7610/2.0 (7.0642.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.4.0.65/27/351*/
const Device UA67540 = {S95378, 80, { &P14732,&P5448,&P4737,&P18092 }};	/*Useragent - SAMSUNG-GT-B7330/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.2.2.51*/
const Device UA68051 = {S95379, 36, { &P13552,&P5129,&P4759,&P18092 }};	/*Useragent - NokiaC6-00/UC Browser7.6.1.82/50/351*/
const Device UA87238 = {S95380, 45, { &P17632,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SH34 UP.Browser/6.2.0.13.4 (GUI) MMP/2.0*/
const Device UA140340 = {S95381, 43, { &P18254,&P18585,&P18245,&P18092 }};	/*Useragent - Bing/1.3.0 CFNetwork/485.10.2 Darwin/10.3.1*/
const Device UA2262365 = {S95382, 47, { &P18254,&P18585,&P18245,&P18092 }};	/*Useragent - StumbleUpon/2.52 CFNetwork/467.12 Darwin/10.3.1*/
const Device UA249770 = {S95383, 89, { &P15364,&P17285,&P21900,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/14.0.835.202)*/
const Device UA342948 = {S95384, 96, { &P15364,&P17285,&P21900,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; chromeframe/14.0.835.202)*/
const Device UA343262 = {S95385, 125, { &P15364,&P17285,&P21900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/14.0.835.202) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA343291 = {S95386, 132, { &P15364,&P17285,&P21900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; chromeframe/14.0.835.202) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1*/
const Device UA409516 = {S95387, 161, { &P15364,&P17285,&P21900,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; GTB7.1; chromeframe/14.0.835.186; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; .NET4.0C)*/
const Device UA637378 = {S95388, 125, { &P15364,&P17285,&P21900,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; chromeframe/14.0.835.186) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1*/
const Device UA1960174 = {S95389, 78, { &P15364,&P18346,&P22730,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 4.1; Avant Browser)*/
const Device UA2165971 = {S95390, 78, { &P15364,&P18346,&P22730,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 7.0; Trident 4.0; Windows NT 4.1; Avant Browser)*/
const Device UA869732 = {S95391, 120, { &P15364,&P18346,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 4.10; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10*/
const Device UA302908 = {S95392, 11, { &P16874,&P22784,&P17470,&P18092 }};	/*Useragent - MICROMAXQ55*/
const Device UA316581 = {S95393, 101, { &P15364,&P18346,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA360234 = {S95394, 145, { &P18114,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; zh-tw; Transformer Prime TF201 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA361735 = {S95395, 145, { &P18114,&P18110,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Linux Ventana; en-us; Transformer Prime TF201 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/8.0 Safari/534.13*/
const Device UA369991 = {S95396, 72, { &P11747,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2.2;Zh_cn;HTC Desire;480*800;)UCWEB7.8.0.95/139/355*/
const Device UA2019462 = {S95397, 70, { &P11747,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2;Zh_cn;HTC Desire;480*800;)UCWEB7.8.0.95/139/352*/
const Device UA399569 = {S95398, 135, { &P15364,&P7133,&P22491,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Flock/3.0.6.4257 Chrome/5.0.375.1281 Safari/533.4*/
const Device UA436930 = {S95399, 53, { &P11929,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (Nokia5230;SymbianOS/9.1 Series60/3.0)*/
const Device UA515635 = {S95400, 109, { &P17595,&P17296,&P22482,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6.2 GTB7.1*/
const Device UA533311 = {S95401, 35, { &P16003,&P17484,&P4759,&P18092 }};	/*Useragent - gt-s5233a/UC Browser7.6.1.82/69/352*/
const Device UA544718 = {S95402, 35, { &P16003,&P17484,&P4759,&P18092 }};	/*Useragent - gt-s5233a/UC Browser7.6.1.82/69/351*/
const Device UA549797 = {S95403, 65, { &P14202,&P18110,&P24661,&P18092 }};	/*Useragent - JUC(Linux;U;2.1-update1;Zh_cn;E15i;320*480;)UCWEB7.7.0.85/139/800*/
const Device UA1891664 = {S95404, 62, { &P14202,&P18110,&P24661,&P18092 }};	/*Useragent - JUC(Linux;U;2.1-update1;En;E15i;320*480;)UCWEB7.7.0.85/139/999*/
const Device UA557183 = {S95405, 42, { &P12260,&P17484,&P4777,&P18092 }};	/*Useragent - samsung-gt-s5350/UC Browser7.7.1.88/69/444*/
const Device UA561063 = {S95406, 135, { &P13659,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P509 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA571964 = {S95407, 36, { &P15209,&P5768,&P17338,&P18092 }};	/*Useragent - nokia6233/UC Browser7.9.0.102/69/352*/
const Device UA572815 = {S95408, 67, { &P15364,&P7133,&P20795,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:2.0.1) Gecko/X Postbox/2.5.3*/
const Device UA573876 = {S95409, 59, { &P15364,&P6729,&P20451,&P18092 }};	/*Useragent - Opera/9.80 (Windows 95; U; ru) Presto/2.9.168 Version/11.51*/
const Device UA616237 = {S95410, 58, { &P15400,&P17017,&P18246,&P18092 }};	/*Useragent - Nexian NX-G509/MTK Release/10.01.2009 Browser/MAUI Profile*/
const Device UA643937 = {S95411, 117, { &P14708,&P5204,&P17436,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia5130c-2/06.94; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.3.23.12*/
const Device UA648603 = {S95412, 66, { &P15364,&P17285,&P18097,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:9.0.1) Gecko/X Firefox/9.0.1 slurp*/
const Device UA787483 = {S95413, 171, { &P18214,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA811417 = {S95414, 86, { &P17595,&P17297,&P22309,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:5.0) Gecko/X Firefox/5.0 SeaMonkey/2.2*/
const Device UA816303 = {S95415, 76, { &P15364,&P5690,&P24188,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:2.1.1) Gecko/X Firefox/4.0.2pre Fennec/4.0.1*/
const Device UA906136 = {S95416, 50, { &P17595,&P18120,&P20785,&P18092 }};	/*Useragent - MQQBrowser/34 (iOS 4; U; CPU like Mac OS X; zh-cn)*/
const Device UA908649 = {S95417, 51, { &P17595,&P18120,&P20785,&P18092 }};	/*Useragent - MQQBrowser/34 (iOS 4S; U; CPU like Mac OS X; zh-cn)*/
const Device UA1863604 = {S95418, 112, { &P15364,&P5690,&P20461,&P18092 }};	/*Useragent - GoogleEarth/5.0.11733.9347(Windows;Microsoft Windows XP (Service Pack 3);en-US;kml:2.2;client:Free;type:default)*/
const Device UA2801878 = {S95419, 112, { &P15364,&P5690,&P20461,&P18092 }};	/*Useragent - GoogleEarth/5.0.11337.1968(Windows;Microsoft Windows XP (Service Pack 3);en-US;kml:2.2;client:Free;type:default)*/
const Device UA1895513 = {S95420, 69, { &P13402,&P5448,&P4779,&P18092 }};	/*Useragent - acer_E101 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA1905144 = {S95421, 48, { &P15364,&P6704,&P22267,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.5; Windows 98; )*/
const Device UA1939863 = {S95422, 35, { &P12004,&P5890,&P17347,&P18092 }};	/*Useragent - nokian95/UC Browser8.0.3.107/69/444*/
const Device UA2309244 = {S95423, 39, { &P12004,&P5890,&P17347,&P18092 }};	/*Useragent - nokian95_8gb/UC Browser8.0.3.107/69/444*/
const Device UA3073964 = {S95424, 152, { &P14183,&P5096,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA1996310 = {S95425, 112, { &P18506,&P5043,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia305/03.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA2003250 = {S95426, 69, { &P17346,&P18117,&P22070,&P18092 }};	/*Useragent - Instagram 3.0.0 (iPod touch; iPhone OS 5.1.1; en_US) AppleWebKit/420+*/
const Device UA2021514 = {S95427, 137, { &P15210,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; es-es; GT-I8000 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023365 = {S95428, 48, { &P17968,&P17979,&P18245,&P18091 }};	/*Useragent - Android/1.0 (cdma_spyder 6.7.2-180_DHD-16_M4-31)*/
const Device UA2027055 = {S95429, 137, { &P21994,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; P85(R8A1) Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028504 = {S95430, 61, { &P21994,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; P85(R8A1) Build/GRJ22)*/
const Device UA2032249 = {S95431, 125, { &P21994,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; P85(R8A1) Build/GRJ22) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2027476 = {S95432, 152, { &P16356,&P18066,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-N7000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2365111 = {S95433, 152, { &P16356,&P18066,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027738 = {S95434, 155, { &P18214,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029004 = {S95435, 157, { &P16356,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2339946 = {S95436, 157, { &P16356,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029784 = {S95437, 169, { &P12798,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2029966 = {S95438, 163, { &P19998,&P18066,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030091 = {S95439, 157, { &P16436,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032073 = {S95440, 149, { &P17149,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2997560 = {S95441, 150, { &P17149,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2167199 = {S95442, 146, { &P22135,&P18066,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SmartTab1 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2261295 = {S95443, 125, { &P11755,&P5776,&P24663,&P18092 }};	/*Useragent - Nokia6670/2.0 (6.0522.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.9.1.120/27/351/UCWEB*/
const Device UA2329323 = {S95444, 138, { &P15241,&P5843,&P24670,&P18092 }};	/*Useragent - Nokia300/5.0 (07.44) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia300) UCBrowser8.4.0.159/70/352/UCWEB Mobile*/
const Device UA2321196 = {S95445, 51, { &P15802,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player tmous / 1.0 / htc_ruby / 2.3.4*/
const Device UA2371768 = {S95446, 51, { &P15802,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player tmous / 1.0 / htc_ruby / 4.0.3*/
const Device UA2350819 = {S95447, 164, { &P12706,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P690 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2371758 = {S95448, 137, { &P14823,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ko-kr; LG-LU3100 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2382313 = {S95449, 168, { &P15147,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST15i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2383474 = {S95450, 168, { &P15397,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST18i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2418725 = {S95451, 139, { &P15575,&P24697,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-I9100 Build/MIUI.es) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2431838 = {S95452, 229, { &P13974,&P5629,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; fa-fa; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/5.1.24639/28.3392; U; fa) Presto/2.8.119 Version/11.10*/
const Device UA2448771 = {S95453, 157, { &P18513,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2453627 = {S95454, 138, { &P20130,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SONY-Q8 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2455583 = {S95455, 128, { &P15602,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A501 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2467254 = {S95456, 149, { &P20175,&P18066,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2482840 = {S95457, 155, { &P11793,&P5768,&P24673,&P18092 }};	/*Useragent - Nokia3500c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia3500c) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2478789 = {S95458, 154, { &P18283,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2481263 = {S95459, 146, { &P20158,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2481568 = {S95460, 135, { &P17346,&P18432,&P17776,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Version/5.1 Safari/7534.48.3*/
const Device UA2481693 = {S95461, 154, { &P18645,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2483326 = {S95462, 53, { &P14188,&P5890,&P20784,&P18092 }};	/*Useragent - MQQBrowser/3.3 (Nokia5233;SymbianOS/9.1 Series60/3.0)*/
const Device UA2483458 = {S95463, 152, { &P22875,&P18066,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9220 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2496975 = {S95464, 204, { &P15028,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Froyo 222 (V10G) by carlostaringa01; es-es; LG-P350 Optimus Me Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA2500133 = {S95465, 157, { &P15364,&P17285,&P19964,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.2.2.29837 (Firestorm-Release; firestorm skin) Safari/533.3*/
const Device UA2502543 = {S95466, 128, { &P23630,&P24094,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40HX753; CC/GBR; en) Presto/2.10.250 Version/11.60*/
const Device UA2505016 = {S95467, 155, { &P12968,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26i Build/6.1.A.2.45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2506637 = {S95468, 175, { &P18649,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; Micromax A57 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2506795 = {S95469, 135, { &P15877,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; XT910 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2526987 = {S95470, 128, { &P21331,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; Loox Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2527696 = {S95471, 66, { &P18466,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; KFTT; 800*1280) UCWEB7.9.0.94/139/444*/
const Device UA2528373 = {S95472, 135, { &P21256,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Wilco Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2546848 = {S95473, 135, { &P22079,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; eZee'Tab10c Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2553068 = {S95474, 139, { &P21952,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Allview2SpeedDuo Build/SV1.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2553176 = {S95475, 158, { &P11747,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC Desire V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2555261 = {S95476, 94, { &P20033,&P22784,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxx368) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2558691 = {S95477, 100, { &P12351,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3653w) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2577718 = {S95478, 33, { &P15988,&P17484,&P21724,&P18092 }};	/*Useragent - OneBrowser/3.0 (SAMSUNG-GT-C3312)*/
const Device UA2578639 = {S95479, 159, { &P19862,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2597978 = {S95480, 168, { &P15047,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; SK17i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2600616 = {S95481, 181, { &P16263,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MediaPad 7 Lite Build/HuaweiMediaPad) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2601243 = {S95482, 153, { &P21833,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030E Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2617189 = {S95483, 174, { &P15919,&P18066,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build/IMM76D; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/355*/
const Device UA2618057 = {S95484, 174, { &P15919,&P18066,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3100 Build/IML74K; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/352*/
const Device UA2632872 = {S95485, 189, { &P12879,&P6546,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; fr-fr; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0100, fr*/
const Device UA2630722 = {S95486, 149, { &P15985,&P6437,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.3.3000*/
const Device UA2637499 = {S95487, 150, { &P15985,&P6437,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; pt-br; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.3.3000*/
const Device UA2636873 = {S95488, 131, { &P20149,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6211 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2650288 = {S95489, 71, { &P20383,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.7; zh-cn; ZTE_U880s; 320*480) UCWEB7.9.3.103/139/800*/
const Device UA2652823 = {S95490, 131, { &P15767,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MiPad 4 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2653097 = {S95491, 171, { &P20170,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A95 Build/CelkonCelkon_A95) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2655158 = {S95492, 165, { &P21383,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A288t Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2659218 = {S95493, 152, { &P12968,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26i Build/6.1.A.2.45) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2659684 = {S95494, 172, { &P15809,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Rhyme S510b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2660148 = {S95495, 159, { &P18673,&P18066,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2665047 = {S95496, 171, { &P16273,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2666683 = {S95497, 146, { &P20509,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A888 Duo Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667032 = {S95498, 139, { &P20925,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT-AS45SE Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667588 = {S95499, 143, { &P21623,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 6 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668246 = {S95500, 151, { &P13505,&P24694,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-CN; MB525 Build/MIUI) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2683030 = {S95501, 51, { &P15364,&P18346,&P18337,&P18092 }};	/*Useragent - Opera/9.64 (Windows NT 4.10; U; zh-cn) Presto/2.1.1*/
const Device UA2685009 = {S95502, 155, { &P15364,&P5629,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; TCL A996 Build/REL_1.2.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2703910 = {S95503, 157, { &P20292,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC T528t Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2706973 = {S95504, 135, { &P21005,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; F-10D Build/V21R48A) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707135 = {S95505, 135, { &P20416,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; F-04E Build/V14R31C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707874 = {S95506, 161, { &P18507,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I8160 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2709762 = {S95507, 145, { &P20509,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MyPhone A888 Duo Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2711412 = {S95508, 187, { &P15794,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2713176 = {S95509, 133, { &P18253,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT894 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734451 = {S95510, 158, { &P20146,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2734558 = {S95511, 137, { &P16273,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T679 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734728 = {S95512, 140, { &P21980,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; UBISLATE7C+ Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735064 = {S95513, 161, { &P16273,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2735426 = {S95514, 129, { &P18708,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U30GT-H Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735573 = {S95515, 136, { &P16757,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I500 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735622 = {S95516, 136, { &P21987,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Gpad_G1 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2745421 = {S95517, 152, { &P13775,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB520 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2746620 = {S95518, 179, { &P18672,&P5629,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5302 Build/GINGERBREAD; 240*320) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444*/
const Device UA2749145 = {S95519, 92, { &P11864,&P5768,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6151) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2749672 = {S95520, 97, { &P21338,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Symphony_W10) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2750082 = {S95521, 136, { &P22681,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; UMI X1 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2750850 = {S95522, 162, { &P16553,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2751866 = {S95523, 135, { &P22095,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ThL W7 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2751951 = {S95524, 142, { &P21882,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Micromax P362 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2754390 = {S95525, 138, { &P22591,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717D Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2763886 = {S95526, 112, { &P18448,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; ST26i-o) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA2766764 = {S95527, 159, { &P23199,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Coolpad7295 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2777848 = {S95528, 136, { &P13644,&P24698,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; PC36100 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2780431 = {S95529, 133, { &P23528,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; MAGPAD9.1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2782022 = {S95530, 167, { &P12605,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2783007 = {S95531, 140, { &P20131,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Thrill 430x Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2792356 = {S95532, 149, { &P20529,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A89 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2794016 = {S95533, 137, { &P18240,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P700 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2799522 = {S95534, 158, { &P18448,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST26i Build/11.0.A.7.5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2800046 = {S95535, 142, { &P23978,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WALKMAN Build/icx1227_E111) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2841915 = {S95536, 54, { &P15364,&P18575,&P18369,&P18092 }};	/*Useragent - OperaMini/7.0.5.45389 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA2847290 = {S95537, 139, { &P22088,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AllviewSpeedDuo Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2847737 = {S95538, 137, { &P22678,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MAXWEST Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2850661 = {S95539, 175, { &P19862,&P18066,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax_A110 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2857222 = {S95540, 162, { &P19971,&P17485,&P24676,&P18092 }};	/*Useragent - SonyEricssonT700/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssont700) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2857831 = {S95541, 90, { &P15897,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; XT530) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2860466 = {S95542, 129, { &P23515,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TD-1010 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2860734 = {S95543, 154, { &P23199,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Coolpad7295 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2861938 = {S95544, 141, { &P15880,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ME865 Build/6.7.2_GC-384) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2865046 = {S95545, 129, { &P21865,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB-730 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2865274 = {S95546, 138, { &P20869,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A878 Duo Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2865886 = {S95547, 53, { &P14188,&P5890,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5 (Nokia5233;SymbianOS/9.1 Series60/3.0)*/
const Device UA2866876 = {S95548, 39, { &P15364,&P17017,&P18618,&P18092 }};	/*Useragent - Outlook/15.0 (15.0.4505.1002; MSI; x64)*/
const Device UA2867279 = {S95549, 131, { &P24571,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; JXD-P1000 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2867610 = {S95550, 156, { &P13018,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; MT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2868420 = {S95551, 158, { &P23905,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100P Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2869776 = {S95552, 137, { &P20173,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Limo X2+ Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877239 = {S95553, 147, { &P20221,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Starmobile CRYSTAL Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879103 = {S95554, 145, { &P20801,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Optimus_San_Remo Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879316 = {S95555, 134, { &P24570,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZP910 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879338 = {S95556, 132, { &P23846,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A12 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879387 = {S95557, 158, { &P12968,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT26i Build/6.2.B.0.200) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2879555 = {S95558, 131, { &P23262,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAC-70031 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2879618 = {S95559, 153, { &P20129,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MI 2 Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2879676 = {S95560, 156, { &P13074,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P930 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2889470 = {S95561, 141, { &P20183,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MI-ONE Plus Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2890778 = {S95562, 58, { &P15364,&P17017,&P18546,&P18092 }};	/*Useragent - Lynx/2.8.7rel.1 libwww-FM/2.14 SSL-MM/1.4.1 OpenSSL/1.0.1e*/
const Device UA2892377 = {S95563, 137, { &P23003,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TECNO Q1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892458 = {S95564, 169, { &P20884,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; Lenovo A298t Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.292 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2892459 = {S95565, 104, { &P12004,&P5890,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NOKIAN95*/
const Device UA2892899 = {S95566, 139, { &P15354,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST17i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2897409 = {S95567, 157, { &P22872,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5282 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2899257 = {S95568, 140, { &P19846,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-05D Build/9.1.C.0.475) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2935116 = {S95569, 161, { &P17108,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Inspire 4G Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000*/
const Device UA2937825 = {S95570, 136, { &P21953,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Ctrl_V1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937863 = {S95571, 173, { &P23255,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI G520-5000 Build/HuaweiG520-5000) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2938060 = {S95572, 139, { &P18696,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LRA Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2938905 = {S95573, 128, { &P24366,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SM-T311 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2940356 = {S95574, 140, { &P15160,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; TP7-1000DC light Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951097 = {S95575, 152, { &P15364,&P18066,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; MG-555 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2953576 = {S95576, 165, { &P20470,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID9742 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444*/
const Device UA2955773 = {S95577, 89, { &P21963,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Q800) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2956804 = {S95578, 138, { &P18456,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T699 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2957342 = {S95579, 155, { &P13074,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; LG-P930 Build/GRJ90) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2958499 = {S95580, 161, { &P21020,&P18598,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Micromax A116 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2961416 = {S95581, 165, { &P19863,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2962724 = {S95582, 89, { &P16598,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A500) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2963077 = {S95583, 157, { &P21021,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328t Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2963422 = {S95584, 152, { &P19907,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; C5155 Build/IML77) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2986800 = {S95585, 159, { &P21020,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; Micromax A116 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2988796 = {S95586, 131, { &P20160,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2994679 = {S95587, 112, { &P15364,&P16110,&P18430,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; es-US ) AppleWebKit/540.0 (KHTML like Gecko) Version/6.0 Safari/8900.00*/
const Device UA2996475 = {S95588, 96, { &P22632,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Titanium_S1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2998445 = {S95589, 162, { &P18460,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U8950D Build/HuaweiU8950D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2998468 = {S95590, 159, { &P13042,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Incredible S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2999791 = {S95591, 136, { &P22678,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MAXWEST Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3000352 = {S95592, 155, { &P13064,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3004832 = {S95593, 112, { &P22717,&P5843,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia109/04.12; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA3008997 = {S95594, 168, { &P23286,&P18111,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; TECNO F7 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/33567*/
const Device UA3009428 = {S95595, 166, { &P22584,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I337M Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/jfltecan*/
const Device UA3009725 = {S95596, 99, { &P16518,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-e2222) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3009910 = {S95597, 93, { &P22626,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Iris 349) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3010499 = {S95598, 170, { &P18510,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; ALCATEL ONE TOUCH 918N Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3011648 = {S95599, 178, { &P14826,&P5843,&P22881,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291*/
const Device UA3013948 = {S95600, 93, { &P21261,&P18598,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.2; en-US; GT-I9500) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3015136 = {S95601, 155, { &P21739,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; iris454 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3015772 = {S95602, 157, { &P20292,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC T528t Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3018309 = {S95603, 118, { &P15364,&P7480,&P17191,&P18091 }};	/*Useragent - Mozilla/5.0 (compatible; BecomeBot/3.0; MSIE 6.0 compatible; +http://www.become.com/site_owners.html);c2VvLWNyYXdsZXI=*/
const Device UA3020517 = {S95604, 137, { &P12841,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Vivid 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020518 = {S95605, 138, { &P20926,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT-AS40SE Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020922 = {S95606, 133, { &P20427,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHL21 Build/S5170) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021225 = {S95607, 138, { &P24407,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; KYY21 Build/100.0.2480) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021544 = {S95608, 131, { &P21334,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A8 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021650 = {S95609, 130, { &P21224,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N-07D Build/AX) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021658 = {S95610, 137, { &P21132,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Andi4.5h Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021751 = {S95611, 135, { &P23889,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PAD703 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021787 = {S95612, 130, { &P12889,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7300 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021925 = {S95613, 130, { &P18478,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6201 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022099 = {S95614, 136, { &P21953,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Ctrl_V1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022408 = {S95615, 132, { &P21508,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GEM10313BK Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022737 = {S95616, 136, { &P20366,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE T82 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022839 = {S95617, 136, { &P23986,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SBM206SH Build/S0008) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023233 = {S95618, 127, { &P22242,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ME171 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023216 = {S95619, 136, { &P18713,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-P760 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023763 = {S95620, 142, { &P12567,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Panasonic P51 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023836 = {S95621, 185, { &P22158,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; INTEX_CLOUD_X1 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3028245 = {S95622, 137, { &P20289,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; HTC6435LVW Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3034248 = {S95623, 141, { &P18466,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Amazon Kindle Fire HD Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3049573 = {S95624, 132, { &P23866,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; F10 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3050475 = {S95625, 155, { &P18715,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; MID8048 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3051285 = {S95626, 139, { &P15802,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Amaze 4G Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3056938 = {S95627, 132, { &P21794,&P18598,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.2; en-gb; GT-I9152 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3058083 = {S95628, 137, { &P22991,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC 802t Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3058483 = {S95629, 95, { &P23174,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GenxDroid7) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3058767 = {S95630, 90, { &P15364,&P18111,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; C2104) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3058847 = {S95631, 160, { &P22612,&P18066,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8951 Build/HuaweiT8951) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3061763 = {S95632, 95, { &P20004,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Karbonn A2) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3132962 = {S95633, 137, { &P18283,&P18111,&P24394,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1650.11 Mobile Safari/537.36*/
const Device UA3074316 = {S95634, 148, { &P22602,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-S738C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3075850 = {S95635, 142, { &P12567,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Panasonic P51 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3077490 = {S95636, 137, { &P22862,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.1; en-in; Micromax A88 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3078531 = {S95637, 146, { &P21302,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; Fly IQ4411 Quad Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3079310 = {S95638, 100, { &P15875,&P17485,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonck15i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3079800 = {S95639, 164, { &P22972,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I537 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3084942 = {S95640, 83, { &P15364,&P16110,&P20459,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:9.0.1) Gecko/X Firefox/9.0.1 SeaMonkey/2.6.1*/
const Device UA3092066 = {S95641, 134, { &P20556,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; A701 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092717 = {S95642, 183, { &P22903,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A62 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3093276 = {S95643, 93, { &P18595,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8190) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3108720 = {S95644, 88, { &P20120,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A45) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3108930 = {S95645, 89, { &P11950,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; X10i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3109125 = {S95646, 82, { &P16886,&P22784,&P3680,&P18092 }};	/*Useragent - Micromax X1i Ultra/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA3109367 = {S95647, 155, { &P18241,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-E400 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109391 = {S95648, 88, { &P23245,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A3+) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3109414 = {S95649, 181, { &P12871,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC_Wildfire_S_A510e Build/GRJ90) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3109842 = {S95650, 155, { &P20705,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; HTC One Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109932 = {S95651, 134, { &P23252,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; M6589 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110039 = {S95652, 152, { &P16598,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A500 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3110314 = {S95653, 123, { &P22485,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; T3 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3110474 = {S95654, 155, { &P16168,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; NookColor Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.2/encore*/
const Device UA3110877 = {S95655, 131, { &P21334,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A8 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111093 = {S95656, 156, { &P18437,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N8000 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3112968 = {S95657, 155, { &P14433,&P5096,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-I9003 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3113039 = {S95658, 156, { &P12788,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PG86100 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3115063 = {S95659, 137, { &P18447,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Xperia T Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126257 = {S95660, 155, { &P23728,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; O+ 8.52 Android Build/Oplus8.52_Android) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126291 = {S95661, 156, { &P12897,&P5843,&P24680,&P18092 }};	/*Useragent - NokiaC2-00/2.0 (03.99) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; NokiaC2-00) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3127388 = {S95662, 156, { &P14669,&P5843,&P24680,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; NokiaX2-00) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3129102 = {S95663, 162, { &P19862,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax A110 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3129652 = {S95664, 163, { &P21121,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; tr-tr; SAMSUNG SM-N9000Q Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Version/1.5 Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3131342 = {S95665, 129, { &P17687,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ604 Build/I.7.1-45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA2470 = {S95666, 109, { &P14493,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SAMSUNG SGH-X620/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA21440 = {S95667, 102, { &P14493,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-X620C/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA63234 = {S95668, 101, { &P14493,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-X620/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA37999 = {S95669, 42, { &P13308,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonJ105a/R1GA SEMC/NewsReader/1.0*/
const Device UA253810 = {S95670, 62, { &P13308,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA62262 = {S95671, 96, { &P14392,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaE60-1/3.0 (2.0618.06.05) SymbianOS/9.1 Series60/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA9406 = {S95672, 62, { &P13410,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia3555c/2.0 (03.41) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA11645 = {S95673, 168, { &P13647,&P17485,&P3487,&P18092 }};	/*Useragent - SonyEricssonW395c/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1, SonyEricssonW395c/R1FB Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA26863 = {S95674, 83, { &P13647,&P17485,&P3487,&P18092 }};	/*Useragent - SonyEricssonW395c/R1DA Browser/OpenWave/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA14953 = {S95675, 68, { &P13522,&P19954,&P3976,&P18092 }};	/*Useragent - SEC-SGHE950/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62571 = {S95676, 66, { &P11867,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-L2/0A.53.10R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21716 = {S95677, 41, { &P15321,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-R410 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA24656 = {S95678, 79, { &P14444,&P17484,&P4970,&P18092 }};	/*Useragent - Samsung-SPHA920 AU-MIC-A920/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA26421 = {S95679, 106, { &P13594,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-M8800H/1.0 SHP/VPP/R5 NetFront/3.5 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA26790 = {S95680, 53, { &P13635,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-S1310-V/1001 UP.Browser/7.2.6.1.840 (GUI) MMP/2.0*/
const Device UA33128 = {S95681, 46, { &P12860,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-MX505/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA73186 = {S95682, 22, { &P12274,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-i900/JHID1*/
const Device UA49488 = {S95683, 67, { &P13039,&P17017,&P3807,&P18092 }};	/*Useragent - Huawei/1.0/0HuaweiG3510/WAP2.0/Obigo-Browser/Q03C MMS/Obigo-MMS/1.2*/
const Device UA53837 = {S95684, 80, { &P12703,&P5448,&P22303,&P18092 }};	/*Useragent - HTC-2125/1.2 Mozilla/4.0 (compatible; MSIE 5.5; Windows CE; Smartphone; 240x320)*/
const Device UA72994 = {S95685, 137, { &P12703,&P5448,&P22303,&P18092 }};	/*Useragent - HTC-2125/1.2 Mozilla/4.0 (compatible; MSIE 5.5; Windows CE; Smartphone; 240x320) (compatible; MSIE 4.01; Windows CE; Smartphone; 320x240)*/
const Device UA61334 = {S95686, 79, { &P12683,&P17484,&P4970,&P18092 }};	/*Useragent - Samsung-SPHM620 AU-MIC-M620/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88761 = {S95687, 81, { &P12683,&P17484,&P4970,&P18092 }};	/*Useragent - Samsung-SPHM620W AU-MIC-M620W/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61928 = {S95688, 43, { &P17907,&P17017,&P3807,&P18092 }};	/*Useragent - Huawei/1.0/G6605/Browser/obigo-browser/q03c*/
const Device UA64346 = {S95689, 39, { &P14591,&P17486,&P3807,&P18092 }};	/*Useragent - Mot-WX260/1.0 ObigoInternetBrowser/Q03C*/
const Device UA67708 = {S95690, 144, { &P14642,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Motorola-XT502 Build/FRG83) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018449 = {S95691, 142, { &P14642,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Motorola-XT502 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA87127 = {S95692, 47, { &P17630,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SA3A UP.Browser/6.2.0.13.1.4 (GUI) MMP/2.0*/
const Device UA90367 = {S95693, 97, { &P15487,&P17484,&P3834,&P18092 }};	/*Useragent - SAMSUNG-GT-S5750E/1.0 SHP/VPP/R5 Dolfin/2.0 SMM-MMS/1.2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA134868 = {S95694, 54, { &P17584,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3P UP.Browser/6.2_7.2.7.1.K.4.306 (GUI) MMP/2.0*/
const Device UA225279 = {S95695, 36, { &P11905,&P5890,&P4779,&P18092 }};	/*Useragent - NOKIA6120c/UC Browser7.8.0.95/28/351*/
const Device UA497084 = {S95696, 68, { &P17595,&P23101,&P17317,&P18092 }};	/*Useragent - Safari/6533.19.4 CFNetwork/454.11.5 Darwin/10.5.0 (i386) (iMac7%2C1)*/
const Device UA515172 = {S95697, 74, { &P17595,&P23101,&P17317,&P18092 }};	/*Useragent - Safari/6533.18.5 CFNetwork/454.11.5 Darwin/10.5.0 (i386) (MacBookAir3%2C2)*/
const Device UA554885 = {S95698, 70, { &P17595,&P23101,&P17317,&P18092 }};	/*Useragent - Safari/6533.21.1 CFNetwork/454.11.5 Darwin/10.5.0 (i386) (MacPro3%2C1)*/
const Device UA746610 = {S95699, 74, { &P17595,&P23101,&P17317,&P18092 }};	/*Useragent - Safari/6533.19.4 CFNetwork/454.11.5 Darwin/10.5.0 (i386) (MacBookPro7%2C1)*/
const Device UA807912 = {S95700, 71, { &P17595,&P23101,&P17317,&P18092 }};	/*Useragent - Safari/6533.19.4 CFNetwork/454.11.5 Darwin/10.5.0 (i386) (MacBook6%2C1)*/
const Device UA2282841 = {S95701, 99, { &P15364,&P5690,&P22531,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.13) Gecko/X Epic/1.2 Firefox/3.6.10 GTB7.1*/
const Device UA3097714 = {S95702, 89, { &P17595,&P17296,&P20797,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-GB; rv:1.9.1.10) Gecko/X Postbox/1.0.1*/
const Device UA556284 = {S95703, 74, { &P15364,&P7127,&P22306,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:2.0.1) Gecko/X Firefox/4.0.1 SeaMonkey/2.1*/
const Device UA571801 = {S95704, 43, { &P14980,&P17484,&P17338,&P18092 }};	/*Useragent - samsung-gt-b7722/UC Browser7.9.0.102/69/352*/
const Device UA2308944 = {S95705, 106, { &P19971,&P17485,&P17347,&P18092 }};	/*Useragent - SonyEricssonT700/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssont700/UC Browser8.0.3.107/69/444*/
const Device UA665480 = {S95706, 60, { &P15364,&P17188,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0; rv:5.0.1) Gecko/X Firefox/5.0.1*/
const Device UA743072 = {S95707, 143, { &P16039,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-tw; MOT-A1680 Build/DRD35) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA790652 = {S95708, 44, { &P14662,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; SH06A3)*/
const Device UA813647 = {S95709, 96, { &P15364,&P5690,&P22356,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:11.0) Gecko/X Firefox/11.0.1-x64 PaleMoon/11.0.1-x64*/
const Device UA837731 = {S95710, 78, { &P19899,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Sprint:SCH-i830; PPC; 240x320)*/
const Device UA1899604 = {S95711, 79, { &P19899,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Sprint; SCH-i830; PPC; 240x320)*/
const Device UA846518 = {S95712, 89, { &P17595,&P17294,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.4; en-US; rv:1.9.0.4) Gecko/X Firefox/2.0.0.18*/
const Device UA912226 = {S95713, 138, { &P12280,&P18117,&P18361,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.0m*/
const Device UA925815 = {S95714, 69, { &P16963,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;DELL;Venue Pro;7.10;7720)*/
const Device UA925824 = {S95715, 69, { &P16963,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;DELL;Venue Pro;7.10;8107)*/
const Device UA925883 = {S95716, 102, { &P18076,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i667)*/
const Device UA1855036 = {S95717, 120, { &P17595,&P17296,&P17335,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_1; nl-nl) AppleWebKit/532.3+ (KHTML, like Gecko) Version/4.1 Safari/531.9*/
const Device UA2320735 = {S95718, 121, { &P17595,&P17296,&P17335,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; nb-no) AppleWebKit/533.16 (KHTML, like Gecko) Version/4.1 Safari/533.16*/
const Device UA1900272 = {S95719, 44, { &P12351,&P17484,&P17338,&P18092 }};	/*Useragent - samsung-gt-s3653w/UC Browser7.9.0.102/69/352*/
const Device UA1907666 = {S95720, 46, { &P21401,&P18110,&P17810,&P18092 }};	/*Useragent - JUC(Linux; U;2.3.4;zh_cn;Lenovo A68e;320*480;)*/
const Device UA1957344 = {S95721, 150, { &P12280,&P17090,&P20786,&P18092 }};	/*Useragent - MQQBrowser/35 Mozilla/5.0 (iPhone 4; U; CPU iPhone OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3*/
const Device UA1960152 = {S95722, 140, { &P15837,&P6909,&P24667,&P18092 }};	/*Useragent - NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; kau; nokiax2-05) UCBrowser8.2.1.144/70/355/UCWEB Mobile*/
const Device UA1967529 = {S95723, 85, { &P12375,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIANokia N81)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA1989641 = {S95724, 32, { &P14748,&P5890,&P24668,&P18092 }};	/*Useragent - NokiaE72-1/UCWEB8.3.0.133/28/800*/
const Device UA2950963 = {S95725, 89, { &P14748,&P5890,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiae72-1) UCBrowser8.3.1.161/70/352/UCWEB Mobile  3gpp-gba*/
const Device UA2019563 = {S95726, 134, { &P12478,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Boston Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020359 = {S95727, 134, { &P12478,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Boston Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019729 = {S95728, 149, { &P22545,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; es-us; Space Livepad 7.2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019874 = {S95729, 146, { &P18047,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; pt-pt; Z71 Build/GRJ22; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019906 = {S95730, 156, { &P13123,&P5629,&P20775,&P18092 }};	/*Useragent - MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020858 = {S95731, 156, { &P13123,&P5629,&P20775,&P18092 }};	/*Useragent - MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.3.6; vi-vn; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021785 = {S95732, 110, { &P15364,&P17979,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android/2.1.2 ; zh-CN) AppleWebKit/533.1 elite/fzz Safari/533.1 Version/4.0 P160UNA/1.0*/
const Device UA2853827 = {S95733, 20, { &P15364,&P17979,&P4861,&P18092 }};	/*Useragent - eBayAndroid/2.1.2.15*/
const Device UA2024657 = {S95734, 146, { &P21428,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Woxter Tablet PC 85CXi Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2028593 = {S95735, 155, { &P18321,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029936 = {S95736, 153, { &P13078,&P5096,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-U V880 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032650 = {S95737, 153, { &P15798,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Explorer A310e Build/GRJ90)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3049364 = {S95738, 154, { &P15798,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2032665 = {S95739, 158, { &P15575,&P5629,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033545 = {S95740, 157, { &P15575,&P5629,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033828 = {S95741, 158, { &P15575,&P5629,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2688452 = {S95742, 26, { &P16356,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-N7000/100.40004*/
const Device UA2644202 = {S95743, 37, { &P14708,&P5204,&P21759,&P18092 }};	/*Useragent - iBrowser/Mini2.8 (Nokia5130c-2/07.95)*/
const Device UA2180502 = {S95744, 35, { &P15988,&P17484,&P21759,&P18092 }};	/*Useragent - iBrowser/Mini2.8 (SAMSUNG-GT-C3312)*/
const Device UA2302877 = {S95745, 44, { &P12581,&P5055,&P4777,&P18092 }};	/*Useragent - nokia500/UC Browser7.7.1.88/69/444  3gpp-gba*/
const Device UA2366159 = {S95746, 168, { &P18223,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT22i Build/6.1.B.0.544) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2384834 = {S95747, 103, { &P15364,&P18144,&P18427,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4*/
const Device UA2401205 = {S95748, 119, { &P14782,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE61i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile*/
const Device UA2422701 = {S95749, 69, { &P15364,&P16110,&P19821,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:18.0) Gecko/X Thunderbird/18.0*/
const Device UA2426715 = {S95750, 90, { &P11996,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5560) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2467218 = {S95751, 96, { &P11996,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; fa; samsung-gt-s5560i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2654140 = {S95752, 90, { &P11996,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5560) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2951117 = {S95753, 87, { &P11996,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; fa; gt-s5560) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2440892 = {S95754, 50, { &P13163,&P5890,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 (NokiaN97;SymbianOS/9.1 Series60/3.0)*/
const Device UA2441390 = {S95755, 51, { &P13706,&P5890,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 (Nokia5235;SymbianOS/9.1 Series60/3.0)*/
const Device UA2455655 = {S95756, 155, { &P12968,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; LT26i Build/6.1.A.2.45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2458425 = {S95757, 132, { &P22138,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ja-jp; N-02E Build/AX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2463095 = {S95758, 31, { &P20070,&P22784,&P18246,&P18092 }};	/*Useragent - Micromax X268/Q03C MAUI-browser*/
const Device UA2466164 = {S95759, 152, { &P13591,&P5096,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; ZTE-BLADE Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2466198 = {S95760, 154, { &P18303,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; MT27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2480660 = {S95761, 138, { &P14959,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9103 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2484711 = {S95762, 164, { &P15028,&P5096,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P350 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2492774 = {S95763, 142, { &P21484,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; SCH-i899 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2504953 = {S95764, 93, { &P16874,&P22784,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxq55) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2505280 = {S95765, 153, { &P15575,&P18066,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100G Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2505399 = {S95766, 139, { &P23927,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Moii E801 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2521436 = {S95767, 103, { &P18088,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; Microsoft; Virtual)*/
const Device UA2528030 = {S95768, 136, { &P20228,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB710 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2529375 = {S95769, 175, { &P18652,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; Micromax A87 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2530692 = {S95770, 142, { &P21290,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; FreeTAB 9701 HD X1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2555174 = {S95771, 140, { &P21653,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; POV_TAB-PROTAB25 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2557414 = {S95772, 94, { &P16600,&P22784,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; micromaxx600) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2557893 = {S95773, 151, { &P12989,&P5843,&P24675,&P18092 }};	/*Useragent - Nokia303/5.0 (14.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia303) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2558096 = {S95774, 133, { &P20807,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B92 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2577804 = {S95775, 149, { &P13964,&P5096,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I5510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2581042 = {S95776, 173, { &P20262,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; Karbonn_A18 Build/KarbonnA18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2589659 = {S95777, 87, { &P15364,&P7127,&P18198,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.9.2.28) Gecko/X Thunderbird/3.1.20*/
const Device UA2594515 = {S95778, 141, { &P21875,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GFIVE Bravo Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2596153 = {S95779, 144, { &P23437,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; iBuddy Connect Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2600617 = {S95780, 167, { &P11747,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; HTC Desire Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2626879 = {S95781, 217, { &P17337,&P6437,&P17373,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Opera/9.80 (Android; Opera Mini/6.5.26969/28.3445; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2641766 = {S95782, 81, { &P15364,&P17483,&P4780,&P18092 }};	/*Useragent - CSLG32/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA2653557 = {S95783, 71, { &P20175,&P18110,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Adr (Linux; U; 4.0.4; zh-cn; MI 1S Build/IMM76D;480*854)*/
const Device UA2654109 = {S95784, 155, { &P12798,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2658413 = {S95785, 131, { &P23465,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TOUCH85 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2659252 = {S95786, 156, { &P16356,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2659611 = {S95787, 91, { &P11653,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sgh-f480) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2855967 = {S95788, 92, { &P11653,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sgh-f480v) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2659633 = {S95789, 168, { &P12824,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; U8180 Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2660352 = {S95790, 165, { &P15060,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5660 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2660441 = {S95791, 165, { &P18243,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; LG-P880 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2660647 = {S95792, 80, { &P17595,&P17296,&P18689,&P18090 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:19.0) Gecko/X Firefox/19.0 slurp*/
const Device UA2662362 = {S95793, 175, { &P18224,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HUAWEI U8815 Build/HuaweiU8815) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2663163 = {S95794, 166, { &P20935,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; ZTE-Z990G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2665409 = {S95795, 138, { &P20508,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Cynus T2 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2667596 = {S95796, 157, { &P20294,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC T528w Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2667624 = {S95797, 126, { &P18248,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; A700 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2695279 = {S95798, 75, { &P19859,&P18110,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Adr (Linux; U; 4.0.3; zh-cn; HTC T328w Build/IML74K;480*800)*/
const Device UA2695903 = {S95799, 136, { &P21289,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; pl-pl; FreeTAB 9701 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2701374 = {S95800, 164, { &P21383,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A288t Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2704006 = {S95801, 138, { &P21373,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F240K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2707086 = {S95802, 136, { &P23158,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IMO S88 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707809 = {S95803, 134, { &P15389,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U8800 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707916 = {S95804, 134, { &P21016,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LGL21 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2708146 = {S95805, 134, { &P21472,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A111 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2727270 = {S95806, 154, { &P18223,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xperia P Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2733032 = {S95807, 129, { &P20256,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TAB275 Build/TAB275) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2733074 = {S95808, 132, { &P21052,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A9 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2734860 = {S95809, 138, { &P20926,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT-AS40SE Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735225 = {S95810, 142, { &P21466,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Titan TV S320 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735355 = {S95811, 138, { &P16553,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T959V Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2736259 = {S95812, 128, { &P23899,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT8000 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2737824 = {S95813, 142, { &P21466,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Titan TV S320 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2738021 = {S95814, 137, { &P23167,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-H9500 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2739954 = {S95815, 138, { &P22590,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717M Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2746068 = {S95816, 155, { &P13141,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; SHV-E110s Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2747261 = {S95817, 149, { &P19996,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A15 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2748621 = {S95818, 88, { &P20106,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A5+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2750650 = {S95819, 137, { &P17324,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9001 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2750954 = {S95820, 152, { &P19859,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2750959 = {S95821, 133, { &P21472,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A111 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2751009 = {S95822, 154, { &P21094,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A789 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2751764 = {S95823, 134, { &P20872,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGP311 Build/10.1.C.0.344) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2752067 = {S95824, 132, { &P23182,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PMP7280C3G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2762381 = {S95825, 129, { &P21179,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MOMO8 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2767779 = {S95826, 162, { &P15575,&P24698,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9100 Build/MIUIv5) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2768764 = {S95827, 168, { &P14557,&P5843,&P24677,&P18092 }};	/*Useragent - Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6303classic) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2779050 = {S95828, 97, { &P17998,&P5477,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; XCD_28) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2780841 = {S95829, 126, { &P21817,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VP10 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2790432 = {S95830, 47, { &P14358,&P5890,&P20784,&P18092 }};	/*Useragent - MQQBrowser/3.3 (E63;SymbianOS/9.1 Series60/3.0)*/
const Device UA2792174 = {S95831, 92, { &P14123,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; LG-P970) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2793647 = {S95832, 140, { &P18719,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C1505 Build/11.3.A.0.47) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2794287 = {S95833, 156, { &P22875,&P5629,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2816913 = {S95834, 77, { &P15364,&P17017,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; sunmicro) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA3098793 = {S95835, 76, { &P15364,&P17017,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; generic) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2841812 = {S95836, 41, { &P11953,&P17485,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SonyEricssonU10i/R7BA084)*/
const Device UA2847447 = {S95837, 96, { &P18583,&P5043,&P17347,&P18092 }};	/*Useragent - Nokia110/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia110/UC Browser8.0.3.107/69/444*/
const Device UA2850477 = {S95838, 95, { &P20058,&P22784,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx660) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2850494 = {S95839, 159, { &P11680,&P17483,&P24676,&P18092 }};	/*Useragent - LG-KM900/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-km900) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2851110 = {S95840, 137, { &P23778,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGT-X41 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2855189 = {S95841, 132, { &P23357,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID1040C Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2856364 = {S95842, 145, { &P23261,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MAPAN MX913 DC Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2856577 = {S95843, 175, { &P18304,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9070 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2863116 = {S95844, 156, { &P18513,&P5629,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6352 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2863461 = {S95845, 160, { &P22612,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8951 Build/HuaweiT8951) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2867381 = {S95846, 148, { &P21616,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile i-style Q3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868351 = {S95847, 138, { &P22978,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6313T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2869215 = {S95848, 157, { &P20947,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTCEVOV4G Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2870309 = {S95849, 136, { &P22948,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P716 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877665 = {S95850, 147, { &P23103,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Starmobile DIAMOND Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879329 = {S95851, 137, { &P21490,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A860L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879541 = {S95852, 142, { &P21733,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 3 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892236 = {S95853, 163, { &P19902,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R530M Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892376 = {S95854, 140, { &P15877,&P18111,&P22234,&P18091 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID SPYDER Build/HDish) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892523 = {S95855, 142, { &P22822,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KPN Smart 300 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892615 = {S95856, 139, { &P18303,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MT27i Build/6.2.A.0.400) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892647 = {S95857, 146, { &P22910,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MyPhone A818g Duo Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892751 = {S95858, 136, { &P21089,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE T22 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2896881 = {S95859, 138, { &P22657,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASP-4500Z Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2936042 = {S95860, 93, { &P23061,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; TECNO_B3) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2937021 = {S95861, 106, { &P14983,&P5055,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaC7-00*/
const Device UA2938229 = {S95862, 136, { &P18713,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-P760 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2950537 = {S95863, 137, { &P15645,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Z105 Build/malataZ105) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2952024 = {S95864, 76, { &P21267,&P18110,&P24668,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; Lenovo A798t; 480*782) UCWEB8.3.1.151/145/33220*/
const Device UA2953197 = {S95865, 91, { &P14799,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2; en-US; GT-I5500) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2955009 = {S95866, 136, { &P20122,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; Evodroid W10 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2958370 = {S95867, 154, { &P20183,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE Plus Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2958545 = {S95868, 168, { &P20192,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2958575 = {S95869, 164, { &P20884,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A298t Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2961366 = {S95870, 158, { &P21973,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Aqua flash Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2964284 = {S95871, 157, { &P20947,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTCEVOV4G Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2966938 = {S95872, 92, { &P15028,&P5096,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; LG-P350) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA2985670 = {S95873, 139, { &P23298,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; OV-SteelCore10+ Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2987657 = {S95874, 178, { &P19973,&P5043,&P24671,&P18092 }};	/*Useragent - Nokia114/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia114) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2989436 = {S95875, 93, { &P15060,&P5629,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; zh-CN; GT-S5660) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile*/
const Device UA2990038 = {S95876, 131, { &P20463,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP7100D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2990768 = {S95877, 138, { &P12884,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; MID5301 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2992535 = {S95878, 164, { &P24580,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PIRANHA BUSINESS III TAB 10.1 ZEUS WHITE Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2995019 = {S95879, 89, { &P17595,&P17297,&P22317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:21.0) Gecko/X Firefox/21.0 SeaMonkey/2.18*/
const Device UA2995753 = {S95880, 93, { &P22998,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; TECNO_N7) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2996107 = {S95881, 92, { &P15364,&P17779,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPd OS 6_1_3; zh-CN; iPd5,1) U2/1.0.0 UCBrowser/9.0.1.284 U2/1.0.0 Mobile*/
const Device UA2999205 = {S95882, 144, { &P20202,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-L38C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2999361 = {S95883, 148, { &P15201,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Sensation Z710e Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3000575 = {S95884, 154, { &P20183,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI-ONE Plus Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3002232 = {S95885, 149, { &P12968,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; LT26i Build/6.2.B.0.200) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3002275 = {S95886, 156, { &P14693,&P5843,&P24678,&P18092 }};	/*Useragent - NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac1-02) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3004083 = {S95887, 133, { &P18595,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-I8190 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3005449 = {S95888, 147, { &P18559,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-LG730 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3013306 = {S95889, 153, { &P23150,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Q1000 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3013379 = {S95890, 63, { &P18556,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; BN NookHD+ Build/JDQ39E)*/
const Device UA3015497 = {S95891, 161, { &P21321,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9228 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3015756 = {S95892, 88, { &P23245,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A3+) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3016632 = {S95893, 93, { &P15575,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; GT-I9100) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3017483 = {S95894, 90, { &P12798,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; LT18i) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3018189 = {S95895, 133, { &P24053,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SHL22 Build/S6290) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3020339 = {S95896, 91, { &P13964,&P5096,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2; en-US; GT-I5510) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3021145 = {S95897, 134, { &P23949,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; H9500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021423 = {S95898, 136, { &P24611,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E989 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022590 = {S95899, 137, { &P20597,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F100S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022727 = {S95900, 137, { &P12496,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I897 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022733 = {S95901, 132, { &P22926,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M-MP75S23G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023110 = {S95902, 137, { &P16436,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I8150 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023406 = {S95903, 136, { &P14574,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Nexus S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3030139 = {S95904, 100, { &P15302,&P17485,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonwt13i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3034541 = {S95905, 138, { &P14433,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3041525 = {S95906, 130, { &P15364,&P16110,&P22420,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.1.0.0 Chrome/17.0.963.38 Safari/535.11*/
const Device UA3054206 = {S95907, 138, { &P12765,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; gaba A100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3056143 = {S95908, 76, { &P18435,&P21460,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.3; ARM; Trident/6.0; Touch)*/
const Device UA3057435 = {S95909, 99, { &P12399,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; SonyEricssonU8i) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3057731 = {S95910, 158, { &P21794,&P18598,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9152 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300*/
const Device UA3057896 = {S95911, 142, { &P22752,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax P560 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3058455 = {S95912, 94, { &P23118,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua_Star) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3059720 = {S95913, 152, { &P24150,&P18598,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; vivo Xplay Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3076203 = {S95914, 93, { &P22396,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S5312) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3078354 = {S95915, 137, { &P18559,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LG730 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3078772 = {S95916, 100, { &P11941,&P17485,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw580i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3079120 = {S95917, 136, { &P22660,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P778 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3079165 = {S95918, 150, { &P21792,&P18598,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9200 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3081217 = {S95919, 153, { &P23150,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Q1000 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3092891 = {S95920, 148, { &P20175,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; MI 1S Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3094182 = {S95921, 97, { &P18382,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_Desire_C) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3103073 = {S95922, 141, { &P23643,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ASUS Tablet P1801-T Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3108672 = {S95923, 90, { &P22641,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; C1504) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3108907 = {S95924, 181, { &P23122,&P18066,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; Karbonn_A6 Build/MocorDroid4.0.3) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3108953 = {S95925, 93, { &P14186,&P5096,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5830) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3109046 = {S95926, 175, { &P14186,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5830 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3109120 = {S95927, 179, { &P18652,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A87 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3109302 = {S95928, 153, { &P20175,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI 1S Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109453 = {S95929, 93, { &P22609,&P18111,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6312) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3109489 = {S95930, 97, { &P22903,&P5629,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A62) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3110428 = {S95931, 155, { &P22597,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SCH-I545 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3110879 = {S95932, 138, { &P20926,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AT-AS40SE Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111095 = {S95933, 158, { &P15397,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST18i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3111482 = {S95934, 154, { &P21858,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0000 Build/HuaweiY300-0000) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3112676 = {S95935, 90, { &P22756,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; C1604) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3113439 = {S95936, 139, { &P24168,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAP4055DUO Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114915 = {S95937, 132, { &P18017,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Ideapad K1 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3116529 = {S95938, 156, { &P21356,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; TECNO N3 Build/master) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3116536 = {S95939, 158, { &P12616,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; WT19i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126266 = {S95940, 133, { &P18008,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HYUNDAI T10 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126297 = {S95941, 133, { &P20556,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A701 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127184 = {S95942, 137, { &P23139,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Primo-N1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3130989 = {S95943, 157, { &P22872,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5282 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3131316 = {S95944, 137, { &P21506,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131318 = {S95945, 145, { &P13064,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA22552 = {S95946, 88, { &P12039,&P16000,&P3772,&P18092 }};	/*Useragent - MOTORIZR-Z8/46.00.00 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 381) Opera 8.65 [en]*/
const Device UA2907 = {S95947, 123, { &P17925,&P5448,&P4578,&P18092 }};	/*Useragent - dopod C720/5.1.342/WAP1.2 Profile/MIDP2.0 Configuration/CLDC1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone)*/
const Device UA44659 = {S95948, 79, { &P17925,&P5448,&P4578,&P18092 }};	/*Useragent - DopodC720W/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 320x240)*/
const Device UA9292 = {S95949, 61, { &P13403,&P17483,&P17602,&P18092 }};	/*Useragent - LG-TU500 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA11267 = {S95950, 79, { &P12970,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) ASUS-GalaxyII/1.0*/
const Device UA11600 = {S95951, 47, { &P13629,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-363/010_02 ObigoInternetBrowser/Q03C*/
const Device UA20158 = {S95952, 45, { &P13629,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-363A/1.0 ObigoInternetBrowser/Q03C*/
const Device UA26758 = {S95953, 57, { &P13629,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-363/1.0 model-orange ObigoInternetBrowser/Q03C*/
const Device UA55610 = {S95954, 44, { &P13629,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-363/1.0 ObigoInternetBrowser/Q03C*/
const Device UA51964 = {S95955, 90, { &P13959,&P5776,&P3756,&P18092 }};	/*Useragent - Nokia6620/2.0 (2.14.1) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA20200 = {S95956, 83, { &P13892,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW830c/R1KG Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54858 = {S95957, 83, { &P13892,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW830a/R1KG Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20421 = {S95958, 119, { &P20385,&P22839,&P4223,&P18092 }};	/*Useragent - Zte-tu218_TD/1.0 ThreadX/4.0b Larena/2.40 Release/3.15.2010 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA842475 = {S95959, 79, { &P20385,&P22839,&P4223,&P18092 }};	/*Useragent - Zte-tu218_TD/1.0 ThreadX/4.0b Larena/2.40 Release/3.15.2010 Browser/NetFront3.5*/
const Device UA20689 = {S95960, 87, { &P15122,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) acer_P300*/
const Device UA22087 = {S95961, 146, { &P15384,&P5477,&P4861,&P18092 }};	/*Useragent - SonyEricssonA8i_TD/1.0 Ophone/2.0 (Linux; Android 2.1) Release/3.2.2010 Browser/WAP 2.0 AppleWebKit/530.17 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA22447 = {S95962, 107, { &P15473,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-T619/T619UVFG8 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA22469 = {S95963, 121, { &P14030,&P5222,&P4626,&P18092 }};	/*Useragent - Mozilla/5.0 (HTC-T9199/1.08.706.1;U;Windows Mobile/6.5;Profile/MIDP-2.0 Configuration/CLDC-1.1;480*800;CTC/2.0) Opera/9.7*/
const Device UA59894 = {S95964, 122, { &P14030,&P5222,&P4626,&P18092 }};	/*Useragent - Mozilla/5.0 (HTC-T9199/1.18.1401.8;U;Windows Mobile/6.5;Profile/MIDP-2.0 Configuration/CLDC-1.1;480*800;CTC/2.0) Opera/9.7*/
const Device UA24734 = {S95965, 111, { &P15750,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHB220/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA24757 = {S95966, 73, { &P15753,&P17486,&P3534,&P18092 }};	/*Useragent - Motorola-VE20 Obigo/Q05A1 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA50142 = {S95967, 17, { &P13179,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW350i*/
const Device UA25399 = {S95968, 90, { &P15364,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) T-Mobile_LEO*/
const Device UA449635 = {S95969, 87, { &P15364,&P5448,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) KDDI-TS01*/
const Device UA26757 = {S95970, 64, { &P13555,&P17483,&P3534,&P18092 }};	/*Useragent - LG-CB630/V10d Obigo/Q05A Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA26839 = {S95971, 45, { &P13643,&P17484,&P4628,&P18092 }};	/*Useragent - Samsung-SCHR330 Infraware-Polaris/6.0 MMP/2.0*/
const Device UA30948 = {S95972, 85, { &P12374,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-V8301/MB6801_V1_Z1_VN_F1BPa101 Profile/MIDP-2.0 Configuration/CLDC-1.1 Obigo/Q03C*/
const Device UA32789 = {S95973, 57, { &P12770,&P17017,&P18246,&P18092 }};	/*Useragent - NexianNX-G331/MTK Release/02.09.2009 Browser/MAUI Profile*/
const Device UA59358 = {S95974, 137, { &P15365,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; HTC_Magic Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59468 = {S95975, 137, { &P15365,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_Magic Build/GRH78) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA50449 = {S95976, 35, { &P14188,&P5129,&P4737,&P18092 }};	/*Useragent - Nokia5233/UC Browser7.2.2.51/50/352*/
const Device UA52047 = {S95977, 145, { &P21676,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; en-us; i-mobile i858 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA278380 = {S95978, 149, { &P21676,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair; th-th; i-mobile i858 Build/ECLAIR) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA52823 = {S95979, 67, { &P16193,&P5448,&P22270,&P18092 }};	/*Useragent - Xda_Serra/480x640 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA62195 = {S95980, 39, { &P14382,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GB165 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA796789 = {S95981, 114, { &P14540,&P5843,&P17433,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-01/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.1.21.8*/
const Device UA61243 = {S95982, 29, { &P12665,&P17483,&P3499,&P18092 }};	/*Useragent - LGE-GU290V Obigo/Q7.0 MMP/2.0*/
const Device UA68055 = {S95983, 140, { &P17976,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.0; nl-nl; Tablet PC Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA81535 = {S95984, 11, { &P17339,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHC200*/
const Device UA432671 = {S95985, 54, { &P17702,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3N UP.Browser/6.2_7.2.7.1.K.3.352 (GUI) MMP/2.0*/
const Device UA158682 = {S95986, 77, { &P15364,&P22843,&P17470,&P18092 }};	/*Useragent - Lava-A9/S016 LMP/LML Release/2010.03.04 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA273120 = {S95987, 152, { &P13891,&P17483,&P3857,&P18092 }};	/*Useragent - Vodafone/1.0/LG-KU380/V10a Browser/Obigo-Q05A1/2.16 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/2.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA341244 = {S95988, 100, { &P15364,&P18346,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1*/
const Device UA347259 = {S95989, 124, { &P15364,&P7133,&P24208,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.8.10793.201*/
const Device UA350797 = {S95990, 175, { &P15364,&P7133,&P24208,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0) QQBrowser/6.8.10793.201*/
const Device UA351421 = {S95991, 192, { &P15364,&P7133,&P24208,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.8.10793.201*/
const Device UA348162 = {S95992, 135, { &P14183,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; SGH-I997 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025588 = {S95993, 135, { &P14183,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-I997 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA445006 = {S95994, 71, { &P18088,&P22832,&P3807,&P18092 }};	/*Useragent - Videocon_V1603_SW_V10a/MAUI.09A.W09.52.MP.V4/wap2.0/OBIGO_Q03C/MIDP 1.0*/
const Device UA2802888 = {S95995, 89, { &P18088,&P22832,&P3807,&P18092 }};	/*Useragent - Videocon_V1611_SW/1.0 MTK/MAUI.09A.W09.52.MP.V3.F15 Release/11.19.2010 Browser/OBIGO_Q03C*/
const Device UA455356 = {S95996, 88, { &P17595,&P17295,&P22314,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:11.0) Gecko/X Firefox/11.0 SeaMonkey/2.8*/
const Device UA545313 = {S95997, 86, { &P17582,&P5567,&P4536,&P18092 }};	/*Useragent - Mozilla/5.0 Opera/9.5 (KDDI-SN3Q; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00*/
const Device UA572324 = {S95998, 86, { &P17595,&P17296,&P22313,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:7.0) Gecko/X Firefox/7.0 SeaMonkey/2.4*/
const Device UA1902524 = {S95999, 98, { &P11625,&P5768,&P17338,&P18092 }};	/*Useragent - Nokia5300/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia5300/UC Browser7.9.0.102/69/352*/
const Device UA596019 = {S96000, 51, { &P11929,&P5890,&P21746,&P18092 }};	/*Useragent - iBrowser/2.6 (Nokia5230;SymbianOS/9.1 Series60/3.0)*/
const Device UA599801 = {S96001, 121, { &P12280,&P18117,&P22246,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Flipboard/1.7.2*/
const Device UA817081 = {S96002, 123, { &P12280,&P18117,&P22246,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.7.2*/
const Device UA889545 = {S96003, 121, { &P12280,&P18117,&P22246,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.7.2*/
const Device UA1985745 = {S96004, 123, { &P12280,&P18117,&P22246,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.7.1*/
const Device UA2314809 = {S96005, 73, { &P14800,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.1-update1; zh-cn; XT800; 480*854) UCWEB7.9.4.145/139/800*/
const Device UA689805 = {S96006, 135, { &P14872,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SPH-D700 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA753485 = {S96007, 196, { &P12280,&P17144,&P4201,&P18091 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_1 like Mac OS X; fr-fr) OrangeBot-Mobile AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A306 Safari/(mobilesearch.support@orange-ftgroup.com)*/
const Device UA760961 = {S96008, 48, { &P17595,&P17779,&P20783,&P18092 }};	/*Useragent - MQQBrowser/32 (iOS; U; CPU like Mac OS X; zh-cn)*/
const Device UA2803205 = {S96009, 133, { &P13746,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; pl-pl; GT540 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA822525 = {S96010, 31, { &P15364,&P17017,&P18405,&P18092 }};	/*Useragent - Opera/9.25 (OpenSolaris; U; en)*/
const Device UA836019 = {S96011, 66, { &P15364,&P7127,&P18191,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:10.0.2) Gecko/X Thunderbird/10.0.2*/
const Device UA1888220 = {S96012, 76, { &P17595,&P18120,&P22509,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; rv:1.9.2) Gecko/X Camino/3.0*/
const Device UA1957270 = {S96013, 170, { &P13815,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2023044 = {S96014, 132, { &P16356,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-es; N7000 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2609270 = {S96015, 118, { &P17337,&P6437,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android3.2;Zh_cn;GT-P7500;800*1232;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA2609886 = {S96016, 69, { &P17337,&P6437,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android3.2;Zh_cn;GT-P7500;800*1232;)UCWEB7.8.0.95/139/355*/
const Device UA2023830 = {S96017, 133, { &P16168,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NookColor Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2027167 = {S96018, 164, { &P18113,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-31)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088773 = {S96019, 165, { &P18113,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2027468 = {S96020, 203, { &P17835,&P5629,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; NOOK BNTV250 Build/GINGERBREAD 1.4.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, us*/
const Device UA2028346 = {S96021, 155, { &P16356,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031006 = {S96022, 168, { &P18283,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SCH-I535 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031360 = {S96023, 168, { &P18283,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-T999 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2043767 = {S96024, 144, { &P18028,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Milestone PLUS Build/45.1.28) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2043769 = {S96025, 144, { &P18028,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Milestone PLUS Build/45.1.27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2043770 = {S96026, 144, { &P18028,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Milestone PLUS Build/45.1.15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2160883 = {S96027, 70, { &P20734,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux; U;2.3.5;zh_cn;Philips W626;320*480;)UCWEB7.6.0.75/139/32543*/
const Device UA2213710 = {S96028, 133, { &P13386,&P5448,&P4779,&P18092 }};	/*Useragent - LG-GM730/v10a WM/6.1 IEMobile/6.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA2260205 = {S96029, 136, { &P15364,&P17813,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Ubuntu/8.04 Chromium/11.0.696.68 Chrome/11.0.696.68 Safari/534.24*/
const Device UA2285781 = {S96030, 134, { &P15364,&P17813,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.24 (KHTML, like Gecko) Ubuntu/8.04 Chromium/11.0.696.68 Chrome/11.0.696.68 Safari/534.24*/
const Device UA2271750 = {S96031, 59, { &P17595,&P17296,&P18108,&P18091 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; BilliosBot)*/
const Device UA2373943 = {S96032, 69, { &P15364,&P17285,&P22538,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:12.0) Gecko/X Firefox/12.0 Kylo/1.1.1*/
const Device UA2385698 = {S96033, 154, { &P18283,&P18066,&P20787,&P18092 }};	/*Useragent - MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2400884 = {S96034, 160, { &P20513,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2403547 = {S96035, 118, { &P11679,&P5776,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokia6630) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2415194 = {S96036, 155, { &P12616,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; WT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2420931 = {S96037, 152, { &P23459,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; POLY PAD 9716HD Build/ICS.g12refM1005TR.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2422296 = {S96038, 135, { &P17283,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MB860 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2436653 = {S96039, 229, { &P18513,&P5629,&P18369,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2448350 = {S96040, 142, { &P22543,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; FP-001A-XON Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2452319 = {S96041, 151, { &P15649,&P5096,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2453243 = {S96042, 101, { &P13157,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.3;Zh_cn;HTC Sensation XE with Beats Audio Z715e;540*960;)UCWEB7.8.0.95/139/444*/
const Device UA2465663 = {S96043, 117, { &P23351,&P22784,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; micromax) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2479747 = {S96044, 132, { &P18712,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Next8P12 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2480240 = {S96045, 146, { &P18482,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; A75 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2483047 = {S96046, 99, { &P23010,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonk790i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2484385 = {S96047, 91, { &P13602,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; gt-s5233w) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2555813 = {S96048, 91, { &P13602,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; gt-s5233w) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2502430 = {S96049, 155, { &P15397,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; ST18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2505392 = {S96050, 99, { &P11948,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonk550i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2517077 = {S96051, 92, { &P14168,&P5448,&P4759,&P18092 }};	/*Useragent - HTC_Touch_Viva_T2223 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.6.1.82*/
const Device UA2526529 = {S96052, 76, { &P21440,&P18110,&P20793,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; MOMO9_plus_III; 800*444) UCWEB1.0.1.113/147/800*/
const Device UA2543792 = {S96053, 52, { &P14983,&P5890,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 (NokiaC7-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA2577212 = {S96054, 119, { &P16871,&P22784,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; micromaxq5) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2579523 = {S96055, 137, { &P23340,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DJC Touchtab3 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2594741 = {S96056, 166, { &P19868,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; ALCATEL ONE TOUCH 985N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2596996 = {S96057, 93, { &P16545,&P22784,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxq80) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2598676 = {S96058, 164, { &P12706,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P690 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2601552 = {S96059, 143, { &P23978,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WALKMAN Build/icx1227_E110) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2604833 = {S96060, 62, { &P15364,&P17285,&P21711,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:20.0) Gecko/X Thunderbird/20.0*/
const Device UA2616507 = {S96061, 147, { &P15876,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617158 = {S96062, 147, { &P15876,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2623764 = {S96063, 164, { &P22037,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3113, 39, 1.8.2)*/
const Device UA2625563 = {S96064, 164, { &P22037,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P3113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3113, 39, 1.8.2)*/
const Device UA2637860 = {S96065, 164, { &P15882,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P5110, 42, 1.9.1)*/
const Device UA2629941 = {S96066, 151, { &P15932,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2637735 = {S96067, 151, { &P15932,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2631782 = {S96068, 61, { &P15985,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-P6200 Build/JOP40D)*/
const Device UA2994642 = {S96069, 131, { &P15985,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-p6200 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2641059 = {S96070, 83, { &P15364,&P18110,&P21957,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2a1pre) Gecko/X Minefield/3.2a1pre*/
const Device UA2642177 = {S96071, 53, { &P12329,&P5890,&P21761,&P18092 }};	/*Useragent - iBrowser/2.5 (NokiaN868MP;SymbianOS/9.3 Series60/3.2)*/
const Device UA2644264 = {S96072, 151, { &P21432,&P5629,&P20776,&P18092 }};	/*Useragent - MQQBrowser/25 Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE-N788 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652726 = {S96073, 162, { &P21632,&P5629,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-mobile i-STYLE 4 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652941 = {S96074, 81, { &P15575,&P18110,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.0.4; zh-cn; GT-I9100G Build/IMM76D.ZQLR7;480*800)*/
const Device UA2653703 = {S96075, 170, { &P21861,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1-SDK_V1.00_X.X release-keys; Xteam Smartpad 810c Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2654052 = {S96076, 155, { &P21021,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328t Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2657521 = {S96077, 158, { &P12040,&P17483,&P24675,&P18092 }};	/*Useragent - LG-GW300/V100 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-gw300) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2657581 = {S96078, 160, { &P24581,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; PIRANHA BUSINESS TAB 10.1 ZEUS WHITE Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2659405 = {S96079, 162, { &P13123,&P5629,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.2.2.259*/
const Device UA2659446 = {S96080, 165, { &P20196,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID7048 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2661004 = {S96081, 160, { &P20513,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2662618 = {S96082, 165, { &P14123,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LG-P970 Build/IMM76L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2664859 = {S96083, 165, { &P19917,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LG-P765 Build/IMM76L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2666235 = {S96084, 162, { &P20934,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; 5860E Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2666721 = {S96085, 132, { &P20004,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A2 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667128 = {S96086, 137, { &P21089,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE T22 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667189 = {S96087, 142, { &P21160,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MEDION P4013 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667241 = {S96088, 135, { &P17283,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MB860 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667280 = {S96089, 137, { &P14959,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9103 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667302 = {S96090, 138, { &P14969,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Desire S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667551 = {S96091, 139, { &P20180,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOLO_A800 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667894 = {S96092, 164, { &P12657,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC EVO 3D X515m Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2668328 = {S96093, 136, { &P23776,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGP321 Build/10.1.1.A.0.72) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2668579 = {S96094, 161, { &P22875,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9220 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2693187 = {S96095, 144, { &P22135,&P18111,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 4.1; en-us; SmartTab1 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2694562 = {S96096, 33, { &P15492,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SAMSUNG GT-C3322)*/
const Device UA2695289 = {S96097, 47, { &P11854,&P5890,&P20784,&P18092 }};	/*Useragent - MQQBrowser/3.3 (E71;SymbianOS/9.1 Series60/3.0)*/
const Device UA2695561 = {S96098, 154, { &P12945,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A780 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2696167 = {S96099, 72, { &P21474,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.9; zh-cn; Celkon_A_85; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2699096 = {S96100, 53, { &P11929,&P5890,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4 (Nokia5230;SymbianOS/9.1 Series60/3.0)*/
const Device UA2700148 = {S96101, 171, { &P20837,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; IRIS_501 Build/LAVAIRIS501) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2702446 = {S96102, 134, { &P14014,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S120 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2702708 = {S96103, 170, { &P19997,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Lenovo P700i Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2703798 = {S96104, 160, { &P21267,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo A798t Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2705931 = {S96105, 164, { &P12698,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-C660 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2706141 = {S96106, 169, { &P21383,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; Lenovo A288t Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2706895 = {S96107, 136, { &P21525,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SBM203SH Build/S0018) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2711138 = {S96108, 141, { &P22608,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Spice Mi-530 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712066 = {S96109, 137, { &P14872,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D700 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2720142 = {S96110, 80, { &P14732,&P5448,&P4777,&P18092 }};	/*Useragent - SAMSUNG-GT-B7330/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2727195 = {S96111, 174, { &P17262,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; Kindle Fire Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2728269 = {S96112, 155, { &P22682,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MITO A322 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2733218 = {S96113, 139, { &P22657,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ASP-4500Z Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2734546 = {S96114, 158, { &P13991,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6400L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2734636 = {S96115, 136, { &P22809,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; P5_Quad Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734649 = {S96116, 130, { &P23222,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PMP3470B Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735428 = {S96117, 134, { &P23949,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; H9500 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2745241 = {S96118, 76, { &P19998,&P18110,&P24668,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HUAWEI C8812; 480*800) UCWEB8.3.1.151/145/33265*/
const Device UA2747492 = {S96119, 140, { &P20852,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; Accord711 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2748259 = {S96120, 156, { &P19898,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I699 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2748541 = {S96121, 92, { &P12698,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; LG-C660) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2749864 = {S96122, 139, { &P18564,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC One X+ Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2749989 = {S96123, 160, { &P20527,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A6 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2751401 = {S96124, 71, { &P23536,&P5690,&P22577,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; Edition ASUS) Presto/2.12.388 Version/12.15*/
const Device UA2763880 = {S96125, 129, { &P23762,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; iDj7n Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2764245 = {S96126, 171, { &P20192,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2764383 = {S96127, 147, { &P20255,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R811 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2767451 = {S96128, 77, { &P15364,&P18145,&P22225,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64; rv:20.0) Gecko/X Firefox/20.0 SeaMonkey/2.17*/
const Device UA2768238 = {S96129, 159, { &P20183,&P5629,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2775748 = {S96130, 132, { &P21226,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; E860 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2777709 = {S96131, 136, { &P20982,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Sen S3 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2779431 = {S96132, 105, { &P18182,&P5448,&P3487,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) WB250 UP.Browser/7.2.7.2.559 (GUI)  MMP/2.0*/
const Device UA2787614 = {S96133, 73, { &P20378,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; ZTE_N855D; 320*480) UCWEB7.9.3.103/139/33031*/
const Device UA2790751 = {S96134, 128, { &P22521,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; WF7C Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2794486 = {S96135, 150, { &P16957,&P5096,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-US; XCD35 Build/FRF91) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2811762 = {S96136, 56, { &P12142,&P5890,&P21746,&P18092 }};	/*Useragent - iBrowser/2.6 (SonyEricssonU5;SymbianOS/9.1 Series60/3.0)*/
const Device UA2816741 = {S96137, 139, { &P18745,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; OPPOX905 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2819441 = {S96138, 39, { &P14862,&P5204,&P23405,&P18092 }};	/*Useragent - MQQBrowser/Mini2.8 (Nokia2700c-2/07.80)*/
const Device UA2844881 = {S96139, 139, { &P21417,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabA2207A-H Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2853118 = {S96140, 97, { &P21357,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; iBall_Andi3e) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2859779 = {S96141, 131, { &P20349,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Novo7Flame Build/X.tm) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2860031 = {S96142, 127, { &P20252,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V96A Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2862990 = {S96143, 95, { &P17968,&P18066,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; zh-CN; Android2.x) U2/1.0.0 UCBrowser/8.8.3.278 U2/1.0.0 Mobile*/
const Device UA2865570 = {S96144, 138, { &P21507,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C1605 Build/11.3.A.2.1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2866371 = {S96145, 145, { &P18556,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook HD+ Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2866676 = {S96146, 158, { &P15575,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2866853 = {S96147, 134, { &P20511,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; IQ442 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867146 = {S96148, 134, { &P21878,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; 201F Build/V33R12A) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2867441 = {S96149, 130, { &P20349,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Novo7Flame Build/X.tm) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2867463 = {S96150, 132, { &P23802,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; W30(QuadCore) Build/W30) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2867957 = {S96151, 153, { &P16356,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2868010 = {S96152, 136, { &P20290,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A11 Build/KarbonnA11) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868774 = {S96153, 134, { &P22762,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; L-01D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868819 = {S96154, 146, { &P22850,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI H881C Build/HuaweiH881C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2868941 = {S96155, 162, { &P17149,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; SCH-I510 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2877040 = {S96156, 141, { &P21796,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Philips W336 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879034 = {S96157, 152, { &P21261,&P18598,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9500 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2879163 = {S96158, 137, { &P20597,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F100S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879222 = {S96159, 140, { &P22003,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB A93.2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2879386 = {S96160, 155, { &P21330,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PMP5197DULTRA Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2890544 = {S96161, 169, { &P18241,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-E400 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2891511 = {S96162, 165, { &P18027,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID7042 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444*/
const Device UA2892757 = {S96163, 132, { &P24553,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; X21 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892823 = {S96164, 133, { &P18314,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M032 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892854 = {S96165, 153, { &P15919,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2892960 = {S96166, 129, { &P23800,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; EL72B Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2892961 = {S96167, 135, { &P23064,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ThL W1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892987 = {S96168, 137, { &P18088,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Skyfire2 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893170 = {S96169, 155, { &P23546,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AD683G Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2893333 = {S96170, 139, { &P23148,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ONE TOUCH TAB 7HD Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2893672 = {S96171, 133, { &P21472,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A111 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2934490 = {S96172, 139, { &P24511,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E270L Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2935627 = {S96173, 136, { &P23064,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ThL W1 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2937115 = {S96174, 154, { &P18385,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39E; Carbon-CARBON-JB-v1.7) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2937160 = {S96175, 94, { &P23110,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua_Glow) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2937894 = {S96176, 131, { &P22691,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M-MP1010i Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2938361 = {S96177, 135, { &P21006,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; L-05D Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2941393 = {S96178, 155, { &P14123,&P5096,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-US; LG-P970 Build/FRG83G) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2943895 = {S96179, 143, { &P24048,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly s Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2951248 = {S96180, 139, { &P22592,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717R Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2952790 = {S96181, 125, { &P18680,&P18066,&P23371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KFJWI Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/19.77.34.5 Safari/537.22*/
const Device UA2955337 = {S96182, 151, { &P22875,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9220 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2956555 = {S96183, 139, { &P22818,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; Monga M1 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956658 = {S96184, 104, { &P12581,&P5055,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) Nokia500*/
const Device UA2957050 = {S96185, 153, { &P21628,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8552 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2957079 = {S96186, 138, { &P22953,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F220K Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2958544 = {S96187, 172, { &P21152,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ALCATEL ONE TOUCH 930D Build/1.2.10.4) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2959732 = {S96188, 152, { &P24150,&P18598,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; vivo Xplay Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2960630 = {S96189, 140, { &P23107,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; TA-FONE A37 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2961141 = {S96190, 93, { &P20983,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; zh-CN; GT-I9308) U2/1.0.0 UCBrowser/9.1.1.309 U2/1.0.0 Mobile*/
const Device UA2962958 = {S96191, 159, { &P15047,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; SK17i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2966792 = {S96192, 154, { &P20480,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JY-G3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2977353 = {S96193, 34, { &P14147,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH901iC(c100;TB;W24H12)*/
const Device UA2982075 = {S96194, 76, { &P17595,&P21721,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:15.0) Gecko/X Firefox/15.0.1*/
const Device UA2983557 = {S96195, 158, { &P18680,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (KFJWI, 35, 1.7.0)*/
const Device UA2987487 = {S96196, 175, { &P17172,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; Transformer TF101 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2988153 = {S96197, 154, { &P22604,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I879 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2989278 = {S96198, 131, { &P13006,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; folio100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2989698 = {S96199, 155, { &P23194,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Engage 7HD Build/Engage7HD.Starmobile.V012.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2990237 = {S96200, 140, { &P23855,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; WM8850-mid Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2991155 = {S96201, 154, { &P18686,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; IM-A810S Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2991775 = {S96202, 97, { &P21839,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Penta_WS802C) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2994671 = {S96203, 94, { &P20134,&P5629,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; zh-CN; GT-S5830i) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile*/
const Device UA2995926 = {S96204, 160, { &P23015,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Lenovo A820t Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.292 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2996027 = {S96205, 93, { &P18283,&P18111,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; zh-CN; GT-I9300) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile*/
const Device UA2997377 = {S96206, 140, { &P18453,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; LT25i Build/9.1.A.1.140) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2997430 = {S96207, 131, { &P21076,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; MID8011 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2997673 = {S96208, 148, { &P23772,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-us; GT-I8190L Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2998089 = {S96209, 141, { &P23021,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Spice Mi-535 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2998230 = {S96210, 144, { &P20980,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A97i Build/CelkonA97i_Celkon) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2998610 = {S96211, 132, { &P22926,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M-MP75S23G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2998632 = {S96212, 140, { &P22037,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon*/
const Device UA2999588 = {S96213, 145, { &P18497,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; M886 Build/HuaweiM886) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA3000139 = {S96214, 146, { &P18465,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KFOT Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3001709 = {S96215, 88, { &P23207,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A4+) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3001901 = {S96216, 150, { &P21794,&P18598,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9152 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3003145 = {S96217, 178, { &P14711,&P5204,&P22881,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291*/
const Device UA3003329 = {S96218, 165, { &P18649,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A57 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3075981 = {S96219, 160, { &P12125,&P5204,&P24678,&P18092 }};	/*Useragent - Nokia6500s-1/2.0 (08.30) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6500s-1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3006603 = {S96220, 182, { &P13157,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3009896 = {S96221, 160, { &P21850,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Spice Mi-435 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3010489 = {S96222, 90, { &P22641,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; C1504) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3010847 = {S96223, 159, { &P21980,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; UBISLATE7C+ Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3012645 = {S96224, 156, { &P18637,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9260 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3012704 = {S96225, 159, { &P21094,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo A789 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3013550 = {S96226, 156, { &P18437,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N8000 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3014016 = {S96227, 161, { &P18462,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Desire X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3014407 = {S96228, 154, { &P15184,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; LG-P920 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3016278 = {S96229, 155, { &P22594,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SGH-M919 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3017872 = {S96230, 156, { &P23428,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I8268 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3018855 = {S96231, 93, { &P21356,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; TECNO N3) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3019032 = {S96232, 95, { &P12444,&P5477,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; U20i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3019701 = {S96233, 158, { &P11950,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; X10i Build/3.0.1.G.0.75) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3020932 = {S96234, 137, { &P12945,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A780L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022611 = {S96235, 132, { &P20807,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; B92 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022802 = {S96236, 137, { &P15802,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Amaze 4G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023592 = {S96237, 127, { &P23072,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; T-17B Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3024356 = {S96238, 131, { &P20882,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; OP0118-12 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3030267 = {S96239, 129, { &P21963,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-in; Q800 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3034668 = {S96240, 140, { &P15438,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; HTC Sensation Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3048896 = {S96241, 135, { &P18505,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MB886 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3050949 = {S96242, 141, { &P20233,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VSD220 Build/IMM76D.UI23ED12_VSC) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3052243 = {S96243, 139, { &P18581,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-us; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3052715 = {S96244, 134, { &P23156,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; XOLO A700 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3055183 = {S96245, 138, { &P18507,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3056629 = {S96246, 160, { &P23461,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; en-US; Spice_Mi-422 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3057027 = {S96247, 175, { &P21629,&P18598,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; Lenovo S920_ROW Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33482*/
const Device UA3058188 = {S96248, 76, { &P16292,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile: Galaxy Nexus; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA3060359 = {S96249, 159, { &P20183,&P5629,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3062129 = {S96250, 155, { &P18283,&P21637,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-I9300 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3073898 = {S96251, 138, { &P21495,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E220S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3074353 = {S96252, 114, { &P17595,&P17294,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_4) AppleWebKit/537.13  (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2*/
const Device UA3078108 = {S96253, 142, { &P21466,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Titan TV S320 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3079805 = {S96254, 162, { &P20094,&P18066,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3079811 = {S96255, 159, { &P15047,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; SK17i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3079843 = {S96256, 173, { &P20534,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAP4040_DUO Build/PrestigioPAP4040DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3080234 = {S96257, 167, { &P22968,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; HUAWEI P6-U06 Build/HuaweiP6-U06) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3089053 = {S96258, 69, { &P22872,&P18111,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.1.2; zh-cn; GT-S5282; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA3090981 = {S96259, 129, { &P19904,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; C5170 Build/IML77) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3091639 = {S96260, 162, { &P13452,&P5629,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.3.2.300*/
const Device UA3093192 = {S96261, 156, { &P17799,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; zh-cn; SCH-P709 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA3093236 = {S96262, 175, { &P19862,&P18111,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Micromax_A110 Build/JRO03C) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3093275 = {S96263, 99, { &P21269,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Intex_Aqua_3.2) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3108617 = {S96264, 170, { &P18442,&P18111,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N7100 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3108676 = {S96265, 89, { &P16598,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A500) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3108845 = {S96266, 96, { &P21474,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.9; en-US; Celkon_A_85) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3109246 = {S96267, 144, { &P23153,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ultrafone 701HD Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109386 = {S96268, 137, { &P13054,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; ADR6425LVW Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109418 = {S96269, 90, { &P12616,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; WT19i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3109436 = {S96270, 158, { &P18449,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST27i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109933 = {S96271, 138, { &P23237,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOLO X910 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110071 = {S96272, 102, { &P13974,&P18110,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-S5670) U2/1.0.0 UCBrowser/9.3.0.321 Mobile*/
const Device UA3110301 = {S96273, 93, { &P18513,&P5629,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S6802) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3110872 = {S96274, 87, { &P23359,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A2) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3110903 = {S96275, 134, { &P21449,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SH-04E Build/S6181) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111923 = {S96276, 156, { &P18679,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-B5330 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3113388 = {S96277, 102, { &P22872,&P18110,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-S5282) U2/1.0.0 UCBrowser/9.3.0.321 Mobile*/
const Device UA3114492 = {S96278, 137, { &P19856,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LS970 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114556 = {S96279, 132, { &P20274,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IdeaTab_A1107 Build/MR1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3114978 = {S96280, 147, { &P12879,&P6546,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.1; GT-P7510 Build/HMJ37) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3116518 = {S96281, 161, { &P22627,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; IRIS 455 Build/LAVAIRIS455) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3130961 = {S96282, 136, { &P22679,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131438 = {S96283, 130, { &P18437,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8000 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3132919 = {S96284, 140, { &P18582,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT28at Build/6.1.C.1.111) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA4258 = {S96285, 62, { &P12097,&P5448,&P4578,&P18092 }};	/*Useragent - HTC_P4350-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)*/
const Device UA24335 = {S96286, 61, { &P12097,&P5448,&P4578,&P18092 }};	/*Useragent - HTCP4350-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)*/
const Device UA50366 = {S96287, 62, { &P14698,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW880i/R8BA Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA15211 = {S96288, 141, { &P13529,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-M628/1.0 Release/2.28.2008 Browser/Openwave6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA44564 = {S96289, 146, { &P13529,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-M628_CMCC/1.0 Release/2.28.2008 Browser/Openwave6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA16226 = {S96290, 107, { &P11863,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-T319/T319UVFD4 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA16225 = {S96291, 108, { &P11863,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-T319R/T319UVFG3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA20120 = {S96292, 62, { &P15030,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP505 Teleca/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21028 = {S96293, 76, { &P16100,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonZ250i/R1F003 Browser/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA32978 = {S96294, 76, { &P16100,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonZ250a/R1C003 Browser/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21663 = {S96295, 77, { &P14091,&P5448,&P22271,&P18092 }};	/*Useragent - PANTECH-C810/R01 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA22297 = {S96296, 82, { &P15419,&P5448,&P22270,&P18092 }};	/*Useragent - PANTECH-C820/WM28.31 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA22609 = {S96297, 23, { &P14185,&P19954,&P3816,&P18092 }};	/*Useragent - SEC-SGHX680/1.0 TSS/2.5*/
const Device UA24164 = {S96298, 116, { &P15446,&P17484,&P3625,&P18092 }};	/*Useragent - Samsung-SGH-B320/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA24374 = {S96299, 41, { &P15682,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-R300 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA24595 = {S96300, 111, { &P15716,&P17483,&P4848,&P18092 }};	/*Useragent - LG/KE990d/v10a Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA30193 = {S96301, 58, { &P12288,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-S319/1.0 model-orange ObigoInternetBrowser/Q03C*/
const Device UA46649 = {S96302, 45, { &P12288,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-S319/1.0 ObigoInternetBrowser/Q03C*/
const Device UA33318 = {S96303, 26, { &P15364,&P17017,&P4948,&P18092 }};	/*Useragent - UP.Browser/5.0/Spice G6550*/
const Device UA785577 = {S96304, 114, { &P14635,&P5843,&P17433,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC3-00/08.63; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.1.21.8*/
const Device UA49870 = {S96305, 85, { &P13096,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-W755/MTK Release/01.01.2011 Browser/MAUI Profile/MIDP2.0Configuration/CLDC-1*/
const Device UA54367 = {S96306, 101, { &P14191,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E690/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA54479 = {S96307, 55, { &P14226,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-laylo/AAR10044 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0*/
const Device UA88985 = {S96308, 55, { &P14226,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-M1400/AAZ10043 UP.Browser/7.2.7.2.536 (GUI) MMP/2.0*/
const Device UA89080 = {S96309, 54, { &P14226,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-laylo/AAR1004 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0*/
const Device UA1938785 = {S96310, 55, { &P14226,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-M1400/AAR10042 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0*/
const Device UA56167 = {S96311, 142, { &P14657,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; LU2300 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA56153 = {S96312, 32, { &P14652,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N07A3(c500;TB;W24H16)*/
const Device UA56787 = {S96313, 35, { &P11929,&P5129,&P4715,&P18092 }};	/*Useragent - Nokia5230/UC Browser7.4.0.65/50/351*/
const Device UA61846 = {S96314, 98, { &P17616,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-J208/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA62170 = {S96315, 85, { &P14378,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-Z540/1.0 SHP/VPP/R5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA62552 = {S96316, 99, { &P14420,&P17484,&P4924,&P18092 }};	/*Useragent - SAMSUNG-SGH-ZV10/1.0 SHP/VPP/R5  NetFront/3.2 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA63626 = {S96317, 60, { &P14508,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia2650/1.0 (6.19) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA66000 = {S96318, 68, { &P16137,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Xda Mini S; 240x320)*/
const Device UA79855 = {S96319, 14, { &P15364,&P17017,&P4810,&P18092 }};	/*Useragent - Opera/7 Presto*/
const Device UA820065 = {S96320, 65, { &P15364,&P17017,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.23; Mac_PowerPC) Opera 7.54  [en]*/
const Device UA87559 = {S96321, 16, { &P17804,&P17811,&P17810,&P18092 }};	/*Useragent - LENOVO-i966/66Mt*/
const Device UA90010 = {S96322, 31, { &P17729,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F09B(c100;TB;W20H09)*/
const Device UA90012 = {S96323, 32, { &P17761,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH09B(c500;TB;W24H16)*/
const Device UA341393 = {S96324, 123, { &P15364,&P7133,&P24203,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.2.8853.201*/
const Device UA2682939 = {S96325, 135, { &P14924,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; A70CHT Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA371177 = {S96326, 89, { &P18088,&P17979,&P17470,&P18092 }};	/*Useragent - Devicescape-Agent/4.0.136 (Android - 671a80e2-00db-3dbe-39ec-c247c7ebcb1d) tmobile_wispr1*/
const Device UA2019745 = {S96327, 71, { &P18088,&P17979,&P17470,&P18092 }};	/*Useragent - Devicescape-Agent/4.0.136 (Android - X-0000-0000-0000-X) tmobile_wispr1*/
const Device UA2022026 = {S96328, 71, { &P18088,&P17979,&P17470,&P18092 }};	/*Useragent - Devicescape-Agent/4.0.137 (Android - X-0000-0000-0000-X) tmobile_wispr1*/
const Device UA420407 = {S96329, 50, { &P18254,&P18585,&P17470,&P18092 }};	/*Useragent - MobileRSSFree/3.4 CFNetwork/485.10.2 Darwin/10.3.1*/
const Device UA561715 = {S96330, 46, { &P18254,&P18585,&P17470,&P18092 }};	/*Useragent - MobileRSS/3.4 CFNetwork/485.10.2 Darwin/10.3.1*/
const Device UA779531 = {S96331, 43, { &P18254,&P18585,&P17470,&P18092 }};	/*Useragent - RSSRadio/1.35 CFNetwork/485.2 Darwin/10.3.1*/
const Device UA589890 = {S96332, 150, { &P15201,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Sensation 4G Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75 Mobile Safari/535.7*/
const Device UA1961108 = {S96333, 153, { &P15201,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; HTC Sensation Z710e Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA470349 = {S96334, 140, { &P17595,&P17294,&P22511,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.4; en-US; rv:1.9.1.6) Gecko/X FireDownload/1.2pre Firefox/3.5.6 Wyzo/3.5.6.1 FireTorrent/1.1pre*/
const Device UA2145859 = {S96335, 63, { &P14186,&P18110,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.4;Zh_cn;GT-S5830;320*480;)UCWEB7.8.1.96/139/800*/
const Device UA2292735 = {S96336, 36, { &P12456,&P5890,&P4779,&P18092 }};	/*Useragent - nokian73-1/UC Browser7.8.0.95/69/352*/
const Device UA702406 = {S96337, 59, { &P15364,&P18145,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD i386; rv:2.0) Gecko/X Firefox/4.0*/
const Device UA730873 = {S96338, 89, { &P17595,&P17296,&P20796,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.2.13) Gecko/X Postbox/2.1.4*/
const Device UA829519 = {S96339, 68, { &P15364,&P17285,&P18099,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:10.0.2) Gecko/X Firefox/10.0.2 slurp*/
const Device UA852046 = {S96340, 137, { &P17595,&P17295,&P22444,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Iron/5.0.381.0 Chrome/5.0.381 Safari/533.4*/
const Device UA1905032 = {S96341, 142, { &P12567,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ja-jp; Panasonic UT-PB1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2323187 = {S96342, 79, { &P15364,&P18145,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (X11; FreeBSD 9.0-RELEASE i386; U; en) Presto/2.10.229 Version/11.60*/
const Device UA2544569 = {S96343, 171, { &P16436,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2021341 = {S96344, 131, { &P21458,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ko-kr; SHW-M430W Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2025036 = {S96345, 138, { &P22086,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; iDx10 3G Build/ICS.b02ref.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2027748 = {S96346, 155, { &P13974,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028375 = {S96347, 155, { &P13974,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028953 = {S96348, 131, { &P21224,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N-07D Build/AX) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2029159 = {S96349, 143, { &P15881,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Incredible 2 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2726974 = {S96350, 143, { &P15881,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Incredible 2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2130684 = {S96351, 83, { &P15364,&P17285,&P20907,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:0.9.4.1) Gecko/X Netscape6/6.2.3*/
const Device UA2162477 = {S96352, 61, { &P15364,&P18110,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.5;Zh_cn;ThL V7;480*800;)UCWEB7.8.1.96/139/800*/
const Device UA2288062 = {S96353, 68, { &P15364,&P18110,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;2.1-update1;Zh_cn;OMAP_SS;480*800;)UCWEB7.8.1.96/139/999*/
const Device UA2644413 = {S96354, 146, { &P15699,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Droid Incredible Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2265327 = {S96355, 46, { &P15364,&P18110,&P17388,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/2.1.2; X11)*/
const Device UA2298369 = {S96356, 75, { &P15364,&P22283,&P3499,&P18092 }};	/*Useragent - Coolpad-W702/BrewMP1.0.3/Qualcom6270/ObigoQ7/MIDP-2.1Configuration/CLDC-1.1*/
const Device UA2336597 = {S96357, 85, { &P14972,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s3370) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2337956 = {S96358, 70, { &P19935,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;A818 Duo;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA2338731 = {S96359, 135, { &P21319,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MOMO11 bird Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2350909 = {S96360, 165, { &P12568,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; GT-B7510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2380863 = {S96361, 154, { &P18465,&P18066,&P19989,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.0 Mobile Safari/535.19 Silk-Accelerated=true*/
const Device UA2400458 = {S96362, 249, { &P18643,&P5043,&P18516,&P18092 }};	/*Useragent - Nokia311/5.0 (03.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.1.161/69/355*/
const Device UA2400675 = {S96363, 140, { &P12280,&P18432,&P20786,&P18092 }};	/*Useragent - MQQBrowser/35 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3*/
const Device UA2418152 = {S96364, 157, { &P15364,&P5690,&P19964,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/4.2.2.29837 (Firestorm-Release; firestorm skin) Safari/533.3*/
const Device UA2451727 = {S96365, 157, { &P18304,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2461079 = {S96366, 170, { &P16292,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; Galaxy Nexus Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2475772 = {S96367, 161, { &P15767,&P17145,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/7534.48.3 TeamStream HD (iPad1,1 232 1.6.1)*/
const Device UA2482701 = {S96368, 172, { &P23849,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; MTS-SP101 Build/HuaweiC8511) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2484218 = {S96369, 138, { &P12942,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I927 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2502621 = {S96370, 152, { &P22140,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; POLY PAD 9708HD Build/ICS.g12refM1005TR.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2518176 = {S96371, 63, { &P11639,&P7149,&P15830,&P18092 }};	/*Useragent - BlackBerry9000/5.2.0.89 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2528227 = {S96372, 99, { &P13179,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonw350i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2529240 = {S96373, 166, { &P19847,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-S6012 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2529687 = {S96374, 99, { &P13220,&P17485,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonk770i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2531173 = {S96375, 137, { &P22634,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; WEXLER-TAB-7T Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2542245 = {S96376, 57, { &P15364,&P22784,&P21795,&P18092 }};	/*Useragent - MicromaxX275/MstarBrowser1.0/MIDP2.0Configuration/CLDC1.1*/
const Device UA2546391 = {S96377, 82, { &P17595,&P18207,&P18191,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:10.0.2) Gecko/X Thunderbird/10.0.2*/
const Device UA2552403 = {S96378, 140, { &P21669,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; eGlide XL PRO II Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2552908 = {S96379, 157, { &P20170,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A95 Build/CelkonCelkon_A95) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2556136 = {S96380, 134, { &P20735,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; X403 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2561348 = {S96381, 159, { &P18280,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC VLE_U Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2578881 = {S96382, 71, { &P20983,&P18110,&P24673,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; GT-I9308; 720*1280) UCWEB8.7.4.225/145/800*/
const Device UA2579073 = {S96383, 158, { &P14372,&P17483,&P24673,&P18092 }};	/*Useragent - LG-GM360/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-gm360) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2595396 = {S96384, 161, { &P11698,&P17485,&P24673,&P18092 }};	/*Useragent - SonyEricssonW995/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonw995) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2599604 = {S96385, 71, { &P15364,&P16110,&P18689,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:19.0) Gecko/X Firefox/19.0 slurp*/
const Device UA2601694 = {S96386, 71, { &P23536,&P5690,&P21172,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; Edition ASUS) Presto/2.12.388 Version/12.14*/
const Device UA2603530 = {S96387, 139, { &P20132,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST70104-1 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2619345 = {S96388, 82, { &P12879,&P17979,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.4.6) (GT-P7510;samsung;GT-P7510;4.0.4/IMM76D)*/
const Device UA2613417 = {S96389, 147, { &P22041,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6810 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617355 = {S96390, 95, { &P15985,&P17979,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 Android (13/3.2; 160dpi; 600x976; samsung; GT-P6200; GT-P6200; smdkc210; en_US)*/
const Device UA2619769 = {S96391, 163, { &P19852,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P7511 Build/IMM76L; CyanogenMod-Ganbarou V1.00 GT) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2627418 = {S96392, 152, { &P15919,&P18111,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2637538 = {S96393, 152, { &P15919,&P18111,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2634131 = {S96394, 131, { &P15985,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P6200 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2630467 = {S96395, 166, { &P22037,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P3113 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2648806 = {S96396, 147, { &P15364,&P5629,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; A8T Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652585 = {S96397, 60, { &P14123,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P970 Build/JZO54K)*/
const Device UA2652878 = {S96398, 154, { &P18449,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; ST27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2655431 = {S96399, 152, { &P18673,&P18066,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2655458 = {S96400, 99, { &P16985,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; vodafone_555_blue) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2657374 = {S96401, 138, { &P12942,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SGH-I927 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2658962 = {S96402, 153, { &P21852,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Spice_Mi-495 Build/SpiceSpice_Mi-495) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2659253 = {S96403, 151, { &P20123,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U795 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2660008 = {S96404, 154, { &P11625,&P5768,&P24676,&P18092 }};	/*Useragent - Nokia5300/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5300) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2660126 = {S96405, 170, { &P21700,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; Penta IS701C Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2660599 = {S96406, 174, { &P18645,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2660897 = {S96407, 169, { &P18303,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MT27i Build/6.1.1.B.1.54) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2666946 = {S96408, 140, { &P20927,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT-AS45IPS Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667347 = {S96409, 137, { &P19909,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P935 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2667723 = {S96410, 137, { &P21987,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Gpad_G1 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668104 = {S96411, 143, { &P20837,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IRIS_501 Build/LAVAIRIS501) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2694928 = {S96412, 145, { &P23468,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Cloudfone ICE 2Gs Build/FSV06) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2695662 = {S96413, 152, { &P23053,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GN700W Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695706 = {S96414, 136, { &P23475,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Y1 Build/HCL ME Tablet Y1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2696143 = {S96415, 91, { &P12174,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sgh-f480i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2698538 = {S96416, 118, { &P11658,&P5776,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokia6680) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/405/UCWEB Mobile*/
const Device UA2699068 = {S96417, 164, { &P12663,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-E730 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2701156 = {S96418, 162, { &P15991,&P5629,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.2.2.259*/
const Device UA2702830 = {S96419, 135, { &P21815,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; P-02E Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2703360 = {S96420, 174, { &P19869,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; Micromax A30 Build/REL_2.3.12) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2703518 = {S96421, 154, { &P18635,&P18598,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; Nexus 4 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2708219 = {S96422, 134, { &P21278,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; QW TB-9718D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2708722 = {S96423, 142, { &P19781,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 97 TITANIUMHD Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2713519 = {S96424, 134, { &P20478,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; JY-G2 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713572 = {S96425, 149, { &P20964,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Turkcell Maxi Plus 5 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714866 = {S96426, 131, { &P21052,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A9 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2719756 = {S96427, 95, { &P15364,&P7133,&P24189,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.5) Gecko/X Firefox/3.6.5pre Fennec/1.1*/
const Device UA2722676 = {S96428, 95, { &P15364,&P6729,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows 95) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2*/
const Device UA2722852 = {S96429, 157, { &P20452,&P18066,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S880 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2725953 = {S96430, 128, { &P22089,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; 932i Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2729286 = {S96431, 76, { &P18556,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Barnes & Noble Nook HD+ Build/JZO54K)*/
const Device UA2729780 = {S96432, 151, { &P20394,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE U790 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2730050 = {S96433, 91, { &P16218,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAN91) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2731404 = {S96434, 166, { &P18493,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; LG-MS840 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2734484 = {S96435, 159, { &P18324,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; LG-VM696 Build/ZV5.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2734547 = {S96436, 140, { &P22003,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB A93.2 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2734742 = {S96437, 163, { &P20134,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-S5830i Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Mobile Safari/537.31 OPR/14.0.1074.54070*/
const Device UA2734925 = {S96438, 126, { &P19991,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A511 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735502 = {S96439, 131, { &P22000,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TAB466EUK Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735558 = {S96440, 141, { &P21796,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Philips W336 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735567 = {S96441, 107, { &P15364,&P17285,&P19839,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.64 Safari/537.31 slurp*/
const Device UA2735713 = {S96442, 131, { &P22691,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M-MP1010i Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735878 = {S96443, 140, { &P22021,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KHPHN05ANDA Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2742217 = {S96444, 106, { &P15364,&P22832,&P3499,&P18092 }};	/*Useragent - VG1515_Videocon/1.0 Qualcom/6240 Release/3.10.2011 Browser/Obigo Q7 Profile/MIDP-2.0 Configuration/CLDC1.1*/
const Device UA2746354 = {S96445, 129, { &P23133,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; U30GT Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2746379 = {S96446, 70, { &P18283,&P18110,&P24667,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; GT-I9300; 480*800) UCWEB8.2.2.135/145/800*/
const Device UA2750051 = {S96447, 136, { &P22085,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ThL W5 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2750844 = {S96448, 141, { &P20544,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Maxwell_Lite Build/1.0.2-X-19:27) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2751446 = {S96449, 137, { &P20597,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F100S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2757571 = {S96450, 142, { &P17863,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SCH-I400 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2763147 = {S96451, 66, { &P14255,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.1; zh-cn; Ideos; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2763190 = {S96452, 221, { &P16518,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung gt-e2222) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile,SAMSUNG-GT-E2222/E2222JVKJ3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA2763904 = {S96453, 131, { &P23465,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; TOUCH85 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2766835 = {S96454, 161, { &P14872,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2767583 = {S96455, 136, { &P18687,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P705 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2767811 = {S96456, 80, { &P17595,&P18207,&P22167,&P18090 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:20.0) Gecko/X Firefox/20.0 slurp*/
const Device UA2779069 = {S96457, 91, { &P23546,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; AD683G) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2781292 = {S96458, 126, { &P23232,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; F7HD Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2790203 = {S96459, 149, { &P20494,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; m865 Build/Ultimate CM7; TITAN V7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2792711 = {S96460, 99, { &P14877,&P5477,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT-I9000) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2793096 = {S96461, 135, { &P20098,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GN800 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2797494 = {S96462, 152, { &P20542,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAP5000DUO Build/PrestigioPAP5000DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2799808 = {S96463, 170, { &P12942,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; SAMSUNG-SGH-I927 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2805273 = {S96464, 42, { &P15655,&P17484,&P4759,&P18092 }};	/*Useragent - samsung-gt-s3850/UC Browser7.6.1.82/69/352*/
const Device UA2839220 = {S96465, 131, { &P21060,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID9042 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2841223 = {S96466, 40, { &P12139,&P5204,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia5310XpressMusic/10)*/
const Device UA2842601 = {S96467, 136, { &P18241,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-E400 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2842708 = {S96468, 151, { &P23475,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Y1 Build/HCL ME Tablet Y1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2848972 = {S96469, 141, { &P21980,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; UBISLATE7C+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2852967 = {S96470, 99, { &P12482,&P5477,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; SGH-T959) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2853164 = {S96471, 97, { &P16991,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; Vodafone_858) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2855430 = {S96472, 131, { &P23334,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; TM-7024 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2855470 = {S96473, 137, { &P23438,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; Digma iDj7 3G Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2857740 = {S96474, 129, { &P20835,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 754TPC Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2864906 = {S96475, 139, { &P23381,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A5Classic Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2865968 = {S96476, 91, { &P23351,&P22784,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromax) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2867280 = {S96477, 137, { &P21132,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Andi4.5h Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868630 = {S96478, 138, { &P20180,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOLO_A800 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868906 = {S96479, 161, { &P15501,&P5477,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.53) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2876350 = {S96480, 134, { &P20255,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; R8113 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2878926 = {S96481, 62, { &P15364,&P18144,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64; rv:21.0) Gecko/X Firefox/21.0*/
const Device UA2879396 = {S96482, 144, { &P13157,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Sensation XE Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879508 = {S96483, 145, { &P20104,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 201M Build/7.7.1Q-164_SMJ-165) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879565 = {S96484, 173, { &P20534,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAP4040_DUO Build/PrestigioPAP4040DUO) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2879567 = {S96485, 157, { &P18488,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L300 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2893013 = {S96486, 153, { &P21963,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Q800 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2893354 = {S96487, 138, { &P16321,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xolo_X500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2897412 = {S96488, 149, { &P18556,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BNTV600 Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2935369 = {S96489, 94, { &P23849,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; MTS-SP101) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2936291 = {S96490, 92, { &P21738,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; iris502) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2937344 = {S96491, 153, { &P23194,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Engage 7HD Build/Engage7HD.Starmobile.V012.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2937350 = {S96492, 139, { &P21973,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Aqua flash Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937421 = {S96493, 163, { &P15649,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-S5570I Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2937896 = {S96494, 137, { &P23251,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ZTE V807 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2938042 = {S96495, 153, { &P14877,&P5096,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-US; GT-I9000 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2948867 = {S96496, 119, { &P15364,&P24094,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux mips64; en-US; rv:1.8.1.14) Gecko/X Mandriva/2.0.0.14-3.3mdv2008.1 (2008.1) Firefox/2.0.0.14*/
const Device UA2953525 = {S96497, 139, { &P16036,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus One Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956364 = {S96498, 174, { &P22622,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Micromax_A72 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2956471 = {S96499, 131, { &P23835,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; TAB1050 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2957089 = {S96500, 98, { &P21856,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax_P600) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2957378 = {S96501, 160, { &P15972,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; One XL Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2963322 = {S96502, 150, { &P20513,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A5 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2980509 = {S96503, 122, { &P13153,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2220/E2220XXLA2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA2982720 = {S96504, 99, { &P23564,&P17979,&P18062,&P18092 }};	/*Useragent - HS-U950/1.0 Linux/3.0 Android/4.0 Browser/AppleWebKit534.30 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA2985016 = {S96505, 69, { &P15364,&P22846,&P4780,&P18092 }};	/*Useragent - Lemon B489_Maui Browser/3.83; Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2987069 = {S96506, 138, { &P23176,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MTV_volt Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2990109 = {S96507, 144, { &P18554,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Barnes & Noble Nook HD Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2993513 = {S96508, 66, { &P18448,&P18111,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.1.2; zh-cn; ST26i; 480*854) UCWEB7.9.0.94/139/444*/
const Device UA2994964 = {S96509, 71, { &P18076,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i667;7.10;8862)*/
const Device UA2996941 = {S96510, 147, { &P13991,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6400L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2997080 = {S96511, 138, { &P20244,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Elcano Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2997163 = {S96512, 143, { &P12896,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P999 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2997830 = {S96513, 152, { &P19777,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9105P Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2997831 = {S96514, 156, { &P18513,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6352 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2998166 = {S96515, 137, { &P20986,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; Sen S2 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2998942 = {S96516, 151, { &P20123,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U795 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2999131 = {S96517, 158, { &P18466,&P18066,&P24406,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 OverDrive Media Console/2.6.5*/
const Device UA2999333 = {S96518, 150, { &P21648,&P18598,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9505 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2999356 = {S96519, 151, { &P22020,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9268 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3004226 = {S96520, 133, { &P18283,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3007106 = {S96521, 148, { &P21903,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U705T Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3008383 = {S96522, 161, { &P22752,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax P560 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3014797 = {S96523, 93, { &P14433,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I9003) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3015491 = {S96524, 160, { &P21267,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo A798t Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3016186 = {S96525, 157, { &P23680,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; AURUS III Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3019736 = {S96526, 93, { &P22941,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6812) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3020794 = {S96527, 137, { &P18622,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A820L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021140 = {S96528, 131, { &P21052,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A9 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021152 = {S96529, 128, { &P17965,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-01D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021638 = {S96530, 136, { &P23158,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IMO S88 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021875 = {S96531, 152, { &P21596,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; mobiistar touch LAI 504 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021895 = {S96532, 129, { &P17323,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-p6800 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022087 = {S96533, 137, { &P19746,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VIVO 4.3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022277 = {S96534, 136, { &P22681,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; UMI X1S Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022332 = {S96535, 138, { &P20436,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Cyclone Voyager 2 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023432 = {S96536, 140, { &P20131,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Thrill 430x Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023462 = {S96537, 134, { &P23957,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; FPT V Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024273 = {S96538, 140, { &P22676,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Celkon A119 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024350 = {S96539, 136, { &P12110,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Legend2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3029309 = {S96540, 134, { &P21878,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; 201F Build/V35R13E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3030756 = {S96541, 195, { &P20705,&P18598,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC One Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb*/
const Device UA3046816 = {S96542, 76, { &P17595,&P21721,&P18330,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:14.0) Gecko/X Firefox/14.0.1*/
const Device UA3047331 = {S96543, 167, { &P18642,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; U8665 Build/HuaweiU8665; CyanogenMod 7.2 Unofficial) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3048619 = {S96544, 60, { &P18243,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; LG-P880 Build/JDQ39E)*/
const Device UA3057672 = {S96545, 96, { &P13443,&P5129,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; SamsungI8910) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3057746 = {S96546, 154, { &P21858,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0000 Build/HuaweiY300-0000) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3058090 = {S96547, 131, { &P21415,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Quantum 7 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3058190 = {S96548, 93, { &P22396,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S5312) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3059008 = {S96549, 143, { &P19891,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SAMSUNG-SGH-I717 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3059084 = {S96550, 102, { &P21274,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Intex_Aqua_Marvel) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3060697 = {S96551, 88, { &P12516,&P17017,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; s8003) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3077079 = {S96552, 146, { &P24682,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Celkon A118 Build/CelkonA118) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3078204 = {S96553, 167, { &P15364,&P17285,&P17193,&P18089 }};	/*Useragent - Googlebot/2.1 (+http://www.googlebot.com/bot.html; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 3.5.30729; .NET CLR 3.0.3;c2VvLWNyYXdsZXI=*/
const Device UA3078289 = {S96554, 88, { &P18482,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; A75) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3078621 = {S96555, 95, { &P20033,&P22784,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx368) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3078644 = {S96556, 161, { &P24039,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9190 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.20.1364.172 Mobile Safari/537.22*/
const Device UA3079180 = {S96557, 88, { &P15518,&P17017,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; s5230) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3079258 = {S96558, 155, { &P15802,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; HTC Ruby Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3079579 = {S96559, 162, { &P21628,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8552 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.20.1364.172 Mobile Safari/537.22*/
const Device UA3131038 = {S96560, 156, { &P18283,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9300 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3090144 = {S96561, 132, { &P20705,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; HTC One Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3091879 = {S96562, 154, { &P22972,&P18598,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I537 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3092896 = {S96563, 150, { &P21648,&P18598,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9505 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3093128 = {S96564, 156, { &P21022,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E615 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3094840 = {S96565, 135, { &P14877,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-I9000 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3096433 = {S96566, 153, { &P15767,&P19826,&P24468,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 6_1_3 like Mac OS X; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.10080IT Mobile*/
const Device UA3103369 = {S96567, 136, { &P24625,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E410 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3107980 = {S96568, 175, { &P24378,&P18598,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; Lenovo P780_ROW Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33482*/
const Device UA3108113 = {S96569, 99, { &P18283,&P17979,&P24717,&P18092 }};	/*Useragent - Instagram 4.1.2 Android (16/4.1.2; 320dpi; 720x1280; samsung/Verizon; SCH-I535; d2vzw; qcom; en_US)*/
const Device UA3108450 = {S96570, 93, { &P22396,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S5312) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3108486 = {S96571, 90, { &P12798,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; LT18i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3108722 = {S96572, 94, { &P15118,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; NokiaX7-00) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3110066 = {S96573, 164, { &P12987,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; SPH-M930BST Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3110424 = {S96574, 93, { &P20146,&P18598,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.2; en-US; GT-I9082) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3110651 = {S96575, 145, { &P15805,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Raider X710e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111101 = {S96576, 159, { &P18223,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT22i Build/6.1.1.B.1.54) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3111518 = {S96577, 145, { &P20000,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ607 Build/7.7.1-122-4-FLEM-WIFI-LA) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3111921 = {S96578, 102, { &P15575,&P18110,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-I9100) U2/1.0.0 UCBrowser/9.3.0.321 Mobile*/
const Device UA3114234 = {S96579, 152, { &P18721,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A90S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3114559 = {S96580, 132, { &P22816,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S22 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3114721 = {S96581, 107, { &P21020,&P17979,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; Micromax A116) U2/1.0.0 UCBrowser/9.3.0.321 Mobile*/
const Device UA3114792 = {S96582, 140, { &P23693,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PNDPP48GP Build/ICS.PNDPP48GP.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3120661 = {S96583, 142, { &P13094,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LNV-Lenovo A600e Build/ICS) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3126956 = {S96584, 136, { &P20937,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ADR910L Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127214 = {S96585, 146, { &P24045,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; MyPhone A919i Duo Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127397 = {S96586, 136, { &P22497,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; w30hd(QuadCore) Build/W30hd) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3131750 = {S96587, 141, { &P23505,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Vodafone 875 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132846 = {S96588, 129, { &P22606,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N5110 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3132959 = {S96589, 139, { &P18582,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT28h Build/6.2.B.0.211) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA2133618 = {S96590, 103, { &P14862,&P5204,&P4759,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2700c-2/UC Browser7.6.1.82/69/352*/
const Device UA537262 = {S96591, 97, { &P12059,&P5204,&P4759,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2690/UC Browser7.6.1.82/69/352*/
const Device UA5427 = {S96592, 107, { &P12256,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-L310/L310XEHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA11965 = {S96593, 96, { &P11728,&P5026,&P4948,&P18092 }};	/*Useragent - BlackBerry8820/4.5.0.182 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102 UP.Browser/5.0.3.3*/
const Device UA28815 = {S96594, 92, { &P11729,&P5810,&P4948,&P18092 }};	/*Useragent - BlackBerry8100/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100 UP.Browser/5.0.3.3*/
const Device UA19009 = {S96595, 96, { &P14845,&P5448,&P17183,&P18092 }};	/*Useragent - HTC_Touch_HD_T8282 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA20027 = {S96596, 38, { &P15023,&P17483,&P3866,&P18092 }};	/*Useragent - LG-ME970d MIC/1.1.14 MIDP-2.0/CLDC-1.1*/
const Device UA21447 = {S96597, 115, { &P15284,&P5448,&P17470,&P18092 }};	/*Useragent - LG-GT810H Netfront/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA22350 = {S96598, 81, { &P14291,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK508c/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23002 = {S96599, 81, { &P14291,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK508i/R2AA SEMC-Browser/4.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA55179 = {S96600, 81, { &P14291,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK508i/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22355 = {S96601, 101, { &P15433,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-X650/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA23080 = {S96602, 61, { &P13877,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E1130/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA41073 = {S96603, 70, { &P13877,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E1130B/E1130BXXJD1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA23177 = {S96604, 79, { &P15570,&P6476,&P4970,&P18092 }};	/*Useragent - Samsung-SPHM610 AU-MIC-M610/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24518 = {S96605, 45, { &P15701,&P22832,&P17470,&P18092 }};	/*Useragent - Videocon_1405_SwL110C_03A9_V1_0_B_Wap Browser*/
const Device UA24535 = {S96606, 86, { &P15706,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-G730/MTK Release/10.01.2009 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1*/
const Device UA44318 = {S96607, 66, { &P13827,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GM210/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA60573 = {S96608, 101, { &P13849,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E500/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA50411 = {S96609, 47, { &P11684,&P5129,&P4737,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UC Browser7.2.2.51/50/352*/
const Device UA53907 = {S96610, 53, { &P14061,&P17483,&P17660,&P18092 }};	/*Useragent - LG/KU385/v1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA56170 = {S96611, 71, { &P13671,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-M3510/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA61882 = {S96612, 45, { &P14344,&P17484,&P4628,&P18092 }};	/*Useragent - Samsung-SCHR561 Infraware-Polaris/6.0 MMP/2.0*/
const Device UA63131 = {S96613, 12, { &P14481,&P17483,&P17660,&P18092 }};	/*Useragent - LG/U300/v1.0*/
const Device UA66648 = {S96614, 68, { &P16205,&P5448,&P22270,&P18092 }};	/*Useragent - Xda_Ignito/480x640 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA89712 = {S96615, 71, { &P21392,&P17017,&P3807,&P18092 }};	/*Useragent - K451/1.0 MTK/MAUI.08A.W08.28.MP.V2 Release/7.30.2009 Browser/OBIGO-Q03C*/
const Device UA105667 = {S96616, 49, { &P17889,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/H877/MIDP2.0/CLDC1.1/Screen-320X240*/
const Device UA121230 = {S96617, 48, { &P17901,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/E62/MIDP2.0/CLDC1.1/Screen-240X400*/
const Device UA143775 = {S96618, 12, { &P17982,&P17017,&P17470,&P18092 }};	/*Useragent - Q-mobile M56*/
const Device UA399149 = {S96619, 81, { &P15775,&P7133,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; NOKIA; Lumia 800)*/
const Device UA438840 = {S96620, 84, { &P12243,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Snap_S521; Windows Phone 6.5)*/
const Device UA452165 = {S96621, 44, { &P11941,&P17485,&P17338,&P18092 }};	/*Useragent - sonyericssonw580i/UC Browser7.9.0.102/70/352*/
const Device UA572308 = {S96622, 44, { &P11941,&P17485,&P17338,&P18092 }};	/*Useragent - sonyericssonw580i/UC Browser7.9.0.102/69/352*/
const Device UA512055 = {S96623, 71, { &P14787,&P5567,&P3430,&P18092 }};	/*Useragent - Mozilla/5.0 Opera/9.5 (KDDI-TS3R; BREW; Opera Mobi; U; ja) Presto/2.2.1*/
const Device UA523190 = {S96624, 99, { &P17595,&P17295,&P22510,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.0.9) Gecko/X Firefox/3.0.9 Wyzo/3.0.3*/
const Device UA531439 = {S96625, 67, { &P15364,&P7133,&P4810,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1) Opera 7.54  [en]*/
const Device UA570492 = {S96626, 141, { &P15364,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; de-de; MZ601 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA749792 = {S96627, 141, { &P15364,&P17296,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Intel Mac OSX 10_6_3; 3.2; en-gb; MZ601 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA574108 = {S96628, 46, { &P14984,&P5129,&P17338,&P18092 }};	/*Useragent - nokia5250/UC Browser7.9.0.102/69/352  3gpp-gba*/
const Device UA583751 = {S96629, 97, { &P15364,&P17816,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-GB; rv:1.9.2.24) Gecko/X Ubuntu/10.04 (lucid) Firefox/2.0.0.6*/
const Device UA613190 = {S96630, 130, { &P15767,&P17146,&P20775,&P18092 }};	/*Useragent - MQQBrowser/2.4 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA2328301 = {S96631, 128, { &P15767,&P17146,&P20775,&P18092 }};	/*Useragent - MQQBrowser/2.4 Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Safari/7534.48.3*/
const Device UA664586 = {S96632, 53, { &P11929,&P5890,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 (Nokia5230;SymbianOS/9.1 Series60/3.0)*/
const Device UA714512 = {S96633, 42, { &P14359,&P5129,&P24663,&P18092 }};	/*Useragent - Nokia5228/UC Browser7.9.0.102/50/355/UCWEB*/
const Device UA731755 = {S96634, 44, { &P11941,&P17485,&P17347,&P18092 }};	/*Useragent - sonyericssonw580i/UC Browser8.0.3.107/69/444*/
const Device UA1937637 = {S96635, 44, { &P11941,&P17485,&P17347,&P18092 }};	/*Useragent - sonyericssonw580i/UC Browser8.0.3.107/70/352*/
const Device UA899639 = {S96636, 79, { &P14453,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NokiaE51)/UC Browser8.4.0.159/28/352/UCWEB Mobile*/
const Device UA895329 = {S96637, 81, { &P11907,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-in; NokiaE52-1)/UC Browser8.4.0.159/28/352/UCWEB Mobile*/
const Device UA1899288 = {S96638, 98, { &P12192,&P5448,&P24663,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB7.9.0.94/31/800*/
const Device UA1988305 = {S96639, 112, { &P18219,&P5843,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia202/20.28; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA2010563 = {S96640, 122, { &P15364,&P16110,&P22434,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Comodo_Dragon/21.0.2.0*/
const Device UA2010815 = {S96641, 129, { &P15364,&P16110,&P22434,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Comodo_Dragon/21.0.2.0*/
const Device UA2019911 = {S96642, 135, { &P19786,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; fj_vortex Build/GRH55) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020739 = {S96643, 148, { &P22253,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; Ainol Novo8 Advanced Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022339 = {S96644, 183, { &P22253,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Ainol Novo8 Advanced Build/v1012_30_809_D5_CHDJ_A38_v1012.60-C_CH@X) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022879 = {S96645, 134, { &P22563,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; AM7-A1 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025004 = {S96646, 172, { &P14090,&P5477,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; GT-I5801 Build/ECLAIR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2027944 = {S96647, 167, { &P21388,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; Karbonn_A1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2029610 = {S96648, 139, { &P16273,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T679 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3045173 = {S96649, 139, { &P16273,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T679 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2131682 = {S96650, 71, { &P15364,&P18110,&P20900,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.0.2) Gecko/X Netscape/7.01*/
const Device UA2213116 = {S96651, 73, { &P15364,&P5690,&P22335,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:5.0) Gecko/X Firefox/5.0 CometBrowser/5.0*/
const Device UA2218964 = {S96652, 109, { &P15364,&P18110,&P18208,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth/6.2.2.6613 Safari/532.4*/
const Device UA2222551 = {S96653, 86, { &P20126,&P17485,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; sonyericssonk800i) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2242226 = {S96654, 122, { &P14721,&P7480,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P906i(c100;TB;W24H15) AppEngine-Google; (+http://code.google.com/appengine; appid: s~google.com:mobile-checker)*/
const Device UA2242265 = {S96655, 135, { &P17504,&P7480,&P3625,&P18092 }};	/*Useragent - KDDI-CA3A UP.Browser/6.2.0.13.2 (GUI) MMP/2.0 AppEngine-Google; (+http://code.google.com/appengine; appid: s~google.com:mobile-checker)*/
const Device UA2261443 = {S96656, 74, { &P15518,&P17017,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; s5230) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2262130 = {S96657, 176, { &P17661,&P7480,&P3976,&P18092 }};	/*Useragent - SoftBank/1.0/920SH/SHJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1 AppEngine-Google; (+http://code.google.com/appengine; appid: s~google.com:mobile-checker)*/
const Device UA2333302 = {S96658, 83, { &P13800,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA5630d-1)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA2376379 = {S96659, 55, { &P15216,&P7116,&P21765,&P18092 }};	/*Useragent - iBrowser/for Blackberry2.0.0 (BlackBerry8520/5.0.0.592)*/
const Device UA2349709 = {S96660, 146, { &P20981,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax P250(Funbook) Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2371895 = {S96661, 163, { &P19998,&P18066,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2373048 = {S96662, 146, { &P21427,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Woxter Tablet PC 100CX Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2374636 = {S96663, 54, { &P18364,&P5890,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8 (NokiaC5-05;SymbianOS/9.1 Series60/3.0)*/
const Device UA2384850 = {S96664, 165, { &P14574,&P18111,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; Nexus S Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2405129 = {S96665, 98, { &P15197,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-b5310) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2403529 = {S96666, 117, { &P23351,&P22784,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; micromax) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2404817 = {S96667, 98, { &P14972,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s3370) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2413297 = {S96668, 79, { &P14438,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE50)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2420280 = {S96669, 135, { &P12788,&P24697,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PG86100 Build/MIUI) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2422656 = {S96670, 125, { &P20822,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N-08D Build/AX) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2436597 = {S96671, 79, { &P18088,&P5567,&P4719,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/4.3.4/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2437778 = {S96672, 49, { &P15364,&P21442,&P17384,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows 2000)*/
const Device UA2462438 = {S96673, 151, { &P19748,&P5043,&P24675,&P18092 }};	/*Useragent - Nokia205/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia205) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2496681 = {S96674, 130, { &P21486,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; MOMO15 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2504633 = {S96675, 54, { &P15364,&P5690,&P21168,&P18092 }};	/*Useragent - Netscape 4.8 (Win XP): Mozilla/4.8 (Windows NT 5.1; U)*/
const Device UA2506729 = {S96676, 167, { &P11787,&P5204,&P24675,&P18092 }};	/*Useragent - Nokia3120classic/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia3120classic) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2506794 = {S96677, 160, { &P18385,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Nexus 7 Build/JOP40D; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2507296 = {S96678, 158, { &P13634,&P17483,&P24675,&P18092 }};	/*Useragent - LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-gs500) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2523547 = {S96679, 136, { &P23924,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HWA580 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2525676 = {S96680, 154, { &P11950,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; en-in; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2525944 = {S96681, 146, { &P18083,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; V9C Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2554851 = {S96682, 125, { &P14864,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s8500) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2579510 = {S96683, 161, { &P18666,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A25 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2602004 = {S96684, 150, { &P20964,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Turkcell Maxi Plus 5 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2607474 = {S96685, 68, { &P13868,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;HD7 T9292;7.10;8862)*/
const Device UA2610070 = {S96686, 29, { &P12889,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (GT-P7300 HTJ85B)*/
const Device UA2607701 = {S96687, 131, { &P17168,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-gb; GT-P7310 Build/HRI83) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2617820 = {S96688, 100, { &P13452,&P5629,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Messenger; 1.8.3) (GT-P1000; samsung; GT-P1000; 2.3.3/GINGERBREAD)*/
const Device UA2610351 = {S96689, 164, { &P15985,&P6437,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en_us ; GT-P6200 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2633446 = {S96690, 168, { &P15985,&P6437,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-US; GT-P6200 Build/HTJ85B) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2613546 = {S96691, 90, { &P17337,&P17979,&P22070,&P18092 }};	/*Useragent - Instagram 3.0.5 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7500; GT-P7500; p3; en_US)*/
const Device UA2622205 = {S96692, 186, { &P12889,&P6437,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.15/274,1.9.15.274,2012-12-21 15:38, +0800*/
const Device UA2631765 = {S96693, 191, { &P12889,&P6437,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7300B Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0700, th*/
const Device UA2628732 = {S96694, 210, { &P15882,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; fr-fr; GT-P5110 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +0100, fr*/
const Device UA2633141 = {S96695, 210, { &P15882,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-ca; GT-P5110 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, -0800, ca*/
const Device UA2635093 = {S96696, 216, { &P15882,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; GT-P5110 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0100, fr*/
const Device UA2635667 = {S96697, 209, { &P15882,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; GT-P5110 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.27/381,1.9.27.381,2013-03-21 11:39, +0100, fr*/
const Device UA2631379 = {S96698, 156, { &P15919,&P18111,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-P3100 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3108561 = {S96699, 170, { &P15919,&P18111,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-P3100 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA2653767 = {S96700, 144, { &P20251,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328d) AndroidWebKit/534.30 (KHTML, Like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2659122 = {S96701, 154, { &P20183,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE Plus Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2659554 = {S96702, 165, { &P20290,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A11 Build/KarbonnA11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2659593 = {S96703, 161, { &P21059,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; A90 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2659591 = {S96704, 166, { &P18002,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; IM-A760S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2681784 = {S96705, 85, { &P15526,&P5448,&P4759,&P18092 }};	/*Useragent - HD_mini_T5555 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.6.1.82*/
const Device UA2688570 = {S96706, 168, { &P12496,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I897 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2693487 = {S96707, 31, { &P18506,&P5043,&P21724,&P18092 }};	/*Useragent - OneBrowser/3.0 (Nokia305/03.60)*/
const Device UA2695497 = {S96708, 66, { &P23919,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; R800i; 480*854) UCWEB7.9.0.94/139/444*/
const Device UA2702091 = {S96709, 168, { &P15517,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MT15i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2702820 = {S96710, 140, { &P15759,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Vision Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2708423 = {S96711, 137, { &P13999,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; NexusHD2 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2713810 = {S96712, 140, { &P18224,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Ascend G300 Build/JOP40C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2717786 = {S96713, 78, { &P17595,&P18207,&P22226,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:21.0) Gecko/X Thunderbird/21.0*/
const Device UA2720210 = {S96714, 72, { &P15364,&P18110,&P20904,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.7.5) Gecko/X Netscape/8.1.3*/
const Device UA2720571 = {S96715, 83, { &P15364,&P17813,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.3) Gecko/X Ubuntu/8.04 Firefox/4.0*/
const Device UA2814719 = {S96716, 251, { &P14826,&P5843,&P18516,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/70/444*/
const Device UA2726973 = {S96717, 112, { &P15364,&P18346,&P22373,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.10) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/3.0 Chrome/18.0.966.0 Safari/535.12*/
const Device UA2728067 = {S96718, 156, { &P18465,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (KFOT, 35, 1.7.0)*/
const Device UA2728894 = {S96719, 99, { &P11941,&P17485,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonw580i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2729963 = {S96720, 172, { &P14877,&P5477,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; GT-I9000 Build/ECLAIR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2732826 = {S96721, 71, { &P15364,&P16110,&P22167,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:20.0) Gecko/X Firefox/20.0 slurp*/
const Device UA2734823 = {S96722, 133, { &P13633,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A953 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734826 = {S96723, 131, { &P20004,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A2 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734842 = {S96724, 144, { &P20333,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax P500(Funbook) Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2734915 = {S96725, 132, { &P23802,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; W30(QuadCore) Build/W30) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735281 = {S96726, 137, { &P15575,&P24698,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9100 Build/MIUIv5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735333 = {S96727, 128, { &P20835,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 754TPC Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735542 = {S96728, 142, { &P20253,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 5 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735911 = {S96729, 137, { &P23582,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; fnac 4.5 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2737008 = {S96730, 140, { &P22742,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GOCLEVER TAB M723G Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2746585 = {S96731, 157, { &P17625,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; ST25i Build/6.1.1.B.1.10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2746942 = {S96732, 128, { &P23621,&P24094,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL32EX650; CC/VNM; vi) Presto/2.10.250 Version/11.60*/
const Device UA2748509 = {S96733, 160, { &P11808,&P5204,&P24676,&P18092 }};	/*Useragent - Nokia5610d-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5610d-1) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2748758 = {S96734, 167, { &P21388,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; Karbonn_A1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2750903 = {S96735, 169, { &P15364,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID3 Build/5.5.1_84_D3G-66_M2-10) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2751553 = {S96736, 135, { &P13074,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-P930 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2762558 = {S96737, 110, { &P18435,&P16110,&P17190,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2; ARM; Trident/4.0; Touch; .NET4.0E; .NET4.0C; Tablet PC 2.0)*/
const Device UA2764801 = {S96738, 183, { &P13157,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2764950 = {S96739, 94, { &P16874,&P22784,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq55) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2778984 = {S96740, 152, { &P20251,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328d Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2778989 = {S96741, 155, { &P21267,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A798t Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2781014 = {S96742, 132, { &P22816,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S22 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2790536 = {S96743, 100, { &P12449,&P5890,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (NokiaE5-00/101.003/sw_platform=S60;sw_platform_version=3.2;java_build_version=1.0.5)*/
const Device UA2791983 = {S96744, 168, { &P23855,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; WM8850-mid Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2792061 = {S96745, 159, { &P16356,&P18066,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2794538 = {S96746, 139, { &P22729,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT918 Build/2_26C_2009) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2810325 = {S96747, 108, { &P17595,&P17295,&P23678,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10_5_8) AppleWebKit/534.50.2 (KHTML, like Gecko) iCab/4.9 Safari/533.16*/
const Device UA2817508 = {S96748, 78, { &P13042,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.4;Zh_cn;HTC Incredible S;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA2834696 = {S96749, 136, { &P22258,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB307 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2841012 = {S96750, 140, { &P17691,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I957 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2843988 = {S96751, 74, { &P20094,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; HUAWEI_U8825D; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2844447 = {S96752, 178, { &P19973,&P5043,&P24672,&P18092 }};	/*Useragent - Nokia114/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia114) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2845701 = {S96753, 129, { &P21280,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MP969 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2851657 = {S96754, 93, { &P18437,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-N8000) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2852077 = {S96755, 94, { &P18295,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; HTC_One_X) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2853665 = {S96756, 150, { &P16950,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; Streak 7 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2853764 = {S96757, 97, { &P23463,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; Spice_Mi-320) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2854979 = {S96758, 95, { &P21075,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Q-Smart_S1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2860025 = {S96759, 137, { &P14433,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9003 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2860726 = {S96760, 154, { &P19907,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; C5155 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2860852 = {S96761, 131, { &P15003,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M380W Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2861580 = {S96762, 184, { &P16292,&P18066,&P18106,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76B) AppleWebKit/[WEBKIT_VERSION] (KHTML, like Gecko) Chrome/[CHROME_VERSION] Mobile Safari/[WEBKIT_VERSION] pss-webkit-request*/
const Device UA2864791 = {S96763, 132, { &P23217,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP7170B3G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2866581 = {S96764, 151, { &P13644,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2867719 = {S96765, 159, { &P18581,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2867954 = {S96766, 149, { &P18721,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; A90S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2869285 = {S96767, 173, { &P18016,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ThinkPad Tablet Build/ThinkPadTablet_A400_03) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2869693 = {S96768, 152, { &P18665,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; N861 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2878350 = {S96769, 162, { &P21625,&P5629,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-mobile i-STYLE 6 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2879188 = {S96770, 130, { &P20241,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5570C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2879360 = {S96771, 135, { &P22085,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ThL W5 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879458 = {S96772, 156, { &P18241,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; LG-E400 Build/GRK39F) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2888791 = {S96773, 218, { &P15364,&P17285,&P21930,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.5.30729; OfficeLiveConnector.1.4; .NET CLR 3.0.30729; OfficeLivePatch.1.3; FDM) chromeframe/4.0*/
const Device UA2890172 = {S96774, 151, { &P23090,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9400 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2892371 = {S96775, 137, { &P18622,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A820L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892403 = {S96776, 123, { &P20822,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N-08D Build/AX) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2892596 = {S96777, 140, { &P15880,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ME865 Build/6.7.2_GC-384) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892794 = {S96778, 138, { &P23300,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GLADIATOR Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893173 = {S96779, 141, { &P23440,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Vodafone 975 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893177 = {S96780, 136, { &P14123,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P970 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2896426 = {S96781, 162, { &P15438,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Sensation Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2937513 = {S96782, 157, { &P22756,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; C1604 Build/11.3.A.2.1) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2938171 = {S96783, 162, { &P12803,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; SCH-I405 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2952202 = {S96784, 166, { &P18509,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LG-MS770 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2952281 = {S96785, 89, { &P20980,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; A97i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2957569 = {S96786, 160, { &P23226,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Videocon A15 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2958450 = {S96787, 151, { &P23295,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U807 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2960945 = {S96788, 92, { &P13602,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233w) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3014077 = {S96789, 89, { &P13602,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; vi; gt-s5233w) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2985669 = {S96790, 139, { &P18239,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; TCL A510 Build/TCL_A510) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2988052 = {S96791, 147, { &P24578,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PIRANHA ULTRA TAB 10.1" Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2988386 = {S96792, 154, { &P22611,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-S7572 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2989293 = {S96793, 93, { &P14433,&P5629,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; zh-CN; GT-I9003) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile*/
const Device UA2991482 = {S96794, 88, { &P15975,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; V9A) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2992088 = {S96795, 53, { &P18088,&P19826,&P24668,&P18092 }};	/*Useragent - IUC(U;iOS 6.1.3;Zh-cn;320*480;)/UCWEB8.3.1.151/42/997*/
const Device UA2994773 = {S96796, 93, { &P22915,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Lemon_P9) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2994844 = {S96797, 98, { &P22621,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax_P360) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2995240 = {S96798, 131, { &P21137,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HS-10DTB2 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2995455 = {S96799, 206, { &P22582,&P18598,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.27/385,1.9.27.385,2013-03-31 15:31, +0000, us*/
const Device UA2996683 = {S96800, 142, { &P15881,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Incredible 2 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2996899 = {S96801, 132, { &P22816,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S22 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2998556 = {S96802, 150, { &P12867,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-MS910 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2998668 = {S96803, 132, { &P21660,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Novo7 Venus Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2998751 = {S96804, 154, { &P18640,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT901 Build/9.8.2Q-50_SLS-13) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2998985 = {S96805, 164, { &P21151,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LNV-Lenovo A790e Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2999083 = {S96806, 138, { &P24628,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-Optimus Build/VDTv6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2999878 = {S96807, 157, { &P22609,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6312 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3000327 = {S96808, 146, { &P22596,&P18598,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R970 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3003279 = {S96809, 135, { &P22641,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.1; en-in; C1504 Build/11.3.A.0.47) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3003838 = {S96810, 90, { &P22756,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; C1604) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3004475 = {S96811, 134, { &P20427,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHL21 Build/S5170) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3006484 = {S96812, 90, { &P18303,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; MT27i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3008678 = {S96813, 161, { &P16292,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9050 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3009727 = {S96814, 94, { &P20069,&P22784,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx78) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3010282 = {S96815, 152, { &P21112,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; X907 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3010595 = {S96816, 102, { &P21274,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Intex Aqua Marvel) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3011514 = {S96817, 157, { &P23381,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A5Classic Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3011862 = {S96818, 169, { &P19762,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI MT1-U06 Build/HuaweiMT1-U06) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3012646 = {S96819, 137, { &P24481,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S7560 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3012894 = {S96820, 158, { &P20004,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Karbonn A2 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3013823 = {S96821, 162, { &P15575,&P5629,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.3; GT-I9100 Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60015*/
const Device UA3014371 = {S96822, 152, { &P21472,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A111 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3014409 = {S96823, 95, { &P20033,&P22784,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx368) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3015531 = {S96824, 151, { &P23807,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; A50 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3016193 = {S96825, 92, { &P23414,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua Sx) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3016231 = {S96826, 136, { &P16292,&P18598,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3016257 = {S96827, 136, { &P12616,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.4; en-in; WT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3016836 = {S96828, 161, { &P12972,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; SGH-T989 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3017874 = {S96829, 152, { &P19904,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; C5170 Build/IML77) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3018862 = {S96830, 155, { &P22916,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-E450 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3021148 = {S96831, 132, { &P20498,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP7100D3G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3105707 = {S96832, 180, { &P14708,&P5204,&P22881,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291*/
const Device UA3021192 = {S96833, 145, { &P20801,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Optimus_San_Remo Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022007 = {S96834, 135, { &P24585,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ZP800H Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022225 = {S96835, 137, { &P14959,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9103 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023694 = {S96836, 138, { &P20511,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Fly_IQ442 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023701 = {S96837, 136, { &P24529,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9506 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023758 = {S96838, 155, { &P23728,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; O+ 8.52 Android Build/Oplus8.52_Android) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023775 = {S96839, 142, { &P20837,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IRIS_501 Build/LAVAIRIS501) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024027 = {S96840, 134, { &P22705,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IQ451 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024121 = {S96841, 131, { &P21030,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAC-97032 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3024414 = {S96842, 136, { &P20004,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; Karbonn A2+ Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3027946 = {S96843, 205, { &P18645,&P18066,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us*/
const Device UA3028180 = {S96844, 197, { &P19861,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I605 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us*/
const Device UA3030085 = {S96845, 156, { &P16362,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; SPH-D710 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3042252 = {S96846, 93, { &P15364,&P17188,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (WinNT4.0) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2*/
const Device UA3047903 = {S96847, 103, { &P20931,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; iBall_Slide_3G7271) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3048083 = {S96848, 151, { &P18282,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA3048473 = {S96849, 215, { &P19763,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Z Build/JZO54K; CyanogenMod-CM10 base (X unofficial build by quarx, compiled by sevenrock)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049313 = {S96850, 160, { &P18630,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A800S Build/HOANG MINH H?U_VEGAVIETTEAM) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049371 = {S96851, 136, { &P23304,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; R813T Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3053713 = {S96852, 153, { &P12973,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3057076 = {S96853, 93, { &P22382,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8262) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3057813 = {S96854, 141, { &P23008,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax A91 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3058182 = {S96855, 152, { &P23575,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ALCATEL ONE TOUCH 8000A Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3058253 = {S96856, 87, { &P23359,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A2) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3058550 = {S96857, 173, { &P21857,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HUAWEI Y300-0100 Build/HuaweiY300-0100) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3058674 = {S96858, 197, { &P20146,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0530, in*/
const Device UA3069575 = {S96859, 43, { &P11939,&P17485,&P17347,&P18092 }};	/*Useragent - sonyericssons312/UC Browser8.0.3.107/69/444*/
const Device UA3073349 = {S96860, 152, { &P22856,&P18110,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; HUAWEI Y300-0151 Build/HuaweiY300-0151) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24 TT/4.1.1*/
const Device UA3074399 = {S96861, 161, { &P22622,&P5629,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Micromax A72 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.3.2.300*/
const Device UA3078203 = {S96862, 167, { &P15364,&P5690,&P17190,&P18089 }};	/*Useragent - Googlebot/2.1 (+http://www.googlebot.com/bot.html; MSIE 8.0; Windows NT 5.1; Trident/4.0; FunWebProducts; GTB6.5; .NET CLR 1.1.4322; .NET CLR 2.0.5072;c2VvLWNyYXdsZXI=*/
const Device UA3078631 = {S96863, 99, { &P16518,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-e2222) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3078708 = {S96864, 151, { &P22603,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I739 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3079005 = {S96865, 156, { &P18464,&P18066,&P24339,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.4 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA3079255 = {S96866, 157, { &P15850,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LG-LU6200 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3079379 = {S96867, 164, { &P22906,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Lenovo A390_ROW Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3079830 = {S96868, 137, { &P14877,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-gb; GT-I9000 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3086010 = {S96869, 132, { &P14849,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I800 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3087858 = {S96870, 134, { &P24493,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; BS471 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3091809 = {S96871, 80, { &P17595,&P17296,&P22171,&P18090 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:23.0) Gecko/X Firefox/23.0 slurp*/
const Device UA3091837 = {S96872, 142, { &P23317,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Rikomagic MK802IIIS Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3092344 = {S96873, 135, { &P22838,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pl-pl; MOVE Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092460 = {S96874, 112, { &P15785,&P5043,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2010/11.95; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3092993 = {S96875, 103, { &P13552,&P5129,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac6-00) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3093277 = {S96876, 158, { &P18582,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT28h Build/6.2.B.0.211) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3093465 = {S96877, 90, { &P18302,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MT25i) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3100921 = {S96878, 97, { &P15364,&P17483,&P3534,&P18092 }};	/*Useragent - LG-A390/V100 Obigo/Q05A MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA3108362 = {S96879, 93, { &P20146,&P18598,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.2; en-US; GT-I9082) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3108513 = {S96880, 139, { &P24608,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; LG-E467f Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108851 = {S96881, 92, { &P13445,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN85) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3109872 = {S96882, 93, { &P21628,&P18111,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8552) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3110293 = {S96883, 138, { &P24407,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; KYY21 Build/205.0.0d00) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110316 = {S96884, 159, { &P12798,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LT18i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3110986 = {S96885, 126, { &P15602,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; A501 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3111911 = {S96886, 138, { &P20436,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Cyclone Voyager 2 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3112460 = {S96887, 156, { &P18509,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LG-MS770 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3112542 = {S96888, 140, { &P20131,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Thrill 430x Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3113225 = {S96889, 165, { &P13042,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Incredible S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3113526 = {S96890, 161, { &P13452,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3116583 = {S96891, 134, { &P15767,&P18432,&P24462,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.1.0.64021 Mobile/10A523 Safari/7534.48.3*/
const Device UA3116588 = {S96892, 135, { &P15767,&P18432,&P24462,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.1.0.64021 Mobile/10A8426 Safari/7534.48.3*/
const Device UA3116591 = {S96893, 135, { &P15767,&P18432,&P24462,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.1.0.64021 Mobile/10A8500 Safari/7534.48.3*/
const Device UA3116592 = {S96894, 134, { &P15767,&P18432,&P24462,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.1.0.64021 Mobile/10A550 Safari/7534.48.3*/
const Device UA3116595 = {S96895, 132, { &P15767,&P18432,&P24462,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Coast/1.1.1.64324 Mobile/10A403 Safari/7534.48.3*/
const Device UA3125723 = {S96896, 69, { &P18634,&P18148,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 8.0;NOKIA;Lumia 822;8.0;10327)*/
const Device UA3126526 = {S96897, 153, { &P23714,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; O+_8.9_Android Build/OplusOplus_A1000) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127467 = {S96898, 146, { &P18466,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; KFTT Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3128013 = {S96899, 151, { &P19853,&P18111,&P24683,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-N7102 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.5 Mobile Safari/533.1*/
const Device UA3128092 = {S96900, 149, { &P20991,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ME371MG Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3132428 = {S96901, 138, { &P18724,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-H Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3132906 = {S96902, 137, { &P19870,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-L710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132922 = {S96903, 137, { &P18682,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-M950 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132933 = {S96904, 136, { &P20240,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E971 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA11598 = {S96905, 68, { &P14807,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-E790/0E.30.34R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA15611 = {S96906, 73, { &P14807,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-E1 iTunes/0E.30.42R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA31632 = {S96907, 72, { &P14807,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-E1 - RokR/00.00.00 MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54117 = {S96908, 78, { &P14807,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-E1 ROKR iTunes/0E.30.48R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61673 = {S96909, 80, { &P14807,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-E1 Walkman Phone/0E.30.48R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64178 = {S96910, 71, { &P14807,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-ROKR_E1/0E.30.48R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2890 = {S96911, 65, { &P15364,&P5448,&P3430,&P18092 }};	/*Useragent - A3100 Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/16108; U; en)*/
const Device UA52824 = {S96912, 70, { &P15364,&P5448,&P3430,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; *HTC/X05HT*) Opera 9.50*/
const Device UA444055 = {S96913, 102, { &P15364,&P5448,&P3430,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; SHARP/WS020SH; PPC; 480x800) Opera Mobi; Opera 9.5 [ja]*/
const Device UA23520 = {S96914, 41, { &P11660,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonC902/R3EG SEMC/NewsReader/1.0*/
const Device UA37579 = {S96915, 191, { &P12457,&P17605,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2550/1.0 RTK-E/1.0 DF/1.0 Release/23.04.2009 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzUzMDkzMDQ2NTEwMjE4 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA14358 = {S96916, 83, { &P13828,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK790c/R8BA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA55018 = {S96917, 101, { &P13828,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK790/R2B Release/Feb-07-2007 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54206 = {S96918, 87, { &P13547,&P17485,&P3600,&P18092 }};	/*Useragent - SonyEricssonW700c/R1DB Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA17947 = {S96919, 124, { &P16905,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M-940 Gold/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1, Spice M-940 Gold/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA53879 = {S96920, 61, { &P16905,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M-940 Gold/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA18644 = {S96921, 66, { &P15928,&P6712,&P4030,&P18092 }};	/*Useragent - Nokia5132_CMCC/2.0 (07.91) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA22448 = {S96922, 61, { &P15474,&P17483,&P17602,&P18092 }};	/*Useragent - LG-TU550 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22614 = {S96923, 107, { &P15497,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-T409/T409UVGE3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA23671 = {S96924, 79, { &P15618,&P17484,&P4970,&P18092 }};	/*Useragent - Samsung-SPHZ400 AU-MIC-Z400/2.0 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA24569 = {S96925, 46, { &P15712,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-AX260/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA24733 = {S96926, 86, { &P15749,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-G920/MTK Release/01.01.2010 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1*/
const Device UA28392 = {S96927, 104, { &P12234,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-A766/A766UXIA4 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA658312 = {S96928, 114, { &P14669,&P5843,&P17433,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-00/08.17; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.1.21.8*/
const Device UA69344 = {S96929, 103, { &P12473,&P5204,&P4779,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2730c-1/UC Browser7.8.0.95/70/401*/
const Device UA69645 = {S96930, 122, { &P13770,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-C6112/C6112DDJB2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA44369 = {S96931, 67, { &P16213,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Xda Orbit; 240x320)*/
const Device UA44924 = {S96932, 71, { &P13843,&P22839,&P4924,&P18092 }};	/*Useragent - ZTE-F165/1.0 ACS-NF/3.2 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49660 = {S96933, 57, { &P13069,&P17017,&P18246,&P18092 }};	/*Useragent - NexianNX-G235/MTK Release/10.01.2010 Browser/MAUI profile*/
const Device UA52826 = {S96934, 66, { &P17544,&P19949,&P17470,&P18092 }};	/*Useragent - NEC-N6206/1.0 Release/06.15.2006 Browser/CMS2.0.0 Profile/MIDP-2.0*/
const Device UA56072 = {S96935, 100, { &P14634,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; Taylor)*/
const Device UA61789 = {S96936, 14, { &P14331,&P17017,&P17470,&P18092 }};	/*Useragent - i-mobileIE3250*/
const Device UA62562 = {S96937, 79, { &P15334,&P17484,&P4970,&P18092 }};	/*Useragent - Samsung-SPHM510 AU-MIC-M510/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63129 = {S96938, 39, { &P14480,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KM380 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA64401 = {S96939, 57, { &P14595,&P22832,&P17470,&P18092 }};	/*Useragent - Videocon_V1688(Twist)_WAP2.0_MIDP2.0_Configration/CLDC1.1*/
const Device UA69484 = {S96940, 35, { &P15373,&P5129,&P4779,&P18092 }};	/*Useragent - Nokia5530/UC Browser7.8.0.95/50/355*/
const Device UA1904674 = {S96941, 99, { &P13035,&P5043,&P4779,&P18092 }};	/*Useragent - NokiaC2-02/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-02/UC Browser7.8.0.95/70/355*/
const Device UA83583 = {S96942, 21, { &P16228,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonT100/R101*/
const Device UA88277 = {S96943, 47, { &P17534,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC3I UP.Browser/6.2.0.14.1.2 (GUI) MMP/2.0*/
const Device UA243535 = {S96944, 31, { &P18088,&P17017,&P17338,&P18092 }};	/*Useragent - j2me/UC Browser7.9.0.102/69/352*/
const Device UA2301346 = {S96945, 41, { &P18088,&P17017,&P17338,&P18092 }};	/*Useragent - maui/UC Browser7.9.0.102/69/352 /AJSC/1.5*/
const Device UA329351 = {S96946, 125, { &P18061,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; sdk Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA565124 = {S96947, 132, { &P18061,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; google_sdk Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA380910 = {S96948, 102, { &P15364,&P18145,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA1950830 = {S96949, 101, { &P15364,&P18145,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA357829 = {S96950, 135, { &P16362,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SPH-D710 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369712 = {S96951, 127, { &P13042,&P5629,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.3.5;Zh_cn;HTC Incredible S;480*800;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA2019175 = {S96952, 78, { &P13042,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.3;Zh_cn;HTC Incredible S;480*800;)UCWEB7.8.0.95/139/351*/
const Device UA2023128 = {S96953, 78, { &P13042,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.5;Zh_cn;HTC Incredible S;480*800;)UCWEB7.8.0.95/139/352*/
const Device UA403263 = {S96954, 132, { &P15869,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/ITL41F) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7*/
const Device UA403283 = {S96955, 132, { &P15869,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM30C) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA403293 = {S96956, 132, { &P15869,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/HLK75F) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7*/
const Device UA425012 = {S96957, 131, { &P15869,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Xoom Build/IML77) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA863698 = {S96958, 131, { &P15869,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xoom Build/IMM76) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7*/
const Device UA400965 = {S96959, 104, { &P18254,&P18110,&P18206,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; C) AppleWebKit/533.3 (KHTML, like Gecko) PhantomJS/1.2.0 Safari/533.3*/
const Device UA810038 = {S96960, 48, { &P15767,&P18585,&P17470,&P18092 }};	/*Useragent - browseripad/23286 CFNetwork/467.12 Darwin/10.3.1*/
const Device UA826776 = {S96961, 58, { &P15767,&P18585,&P17470,&P18092 }};	/*Useragent - Perfect%20Browser-iPad/2.91 CFNetwork/467.12 Darwin/10.3.1*/
const Device UA2758330 = {S96962, 91, { &P12141,&P6173,&P4779,&P18092 }};	/*Useragent - BlackBerry8900/4.2.0.127 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/159/444*/
const Device UA515867 = {S96963, 103, { &P15364,&P5448,&P4841,&P18092 }};	/*Useragent - YL-COOLPADN900/03.15 POLARIS/5.30 CTC/1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)*/
const Device UA537717 = {S96964, 94, { &P15364,&P5448,&P4841,&P18092 }};	/*Useragent - YL-COOLPADN900/03.15 POLARIS/5.30 CTC/1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)*/
const Device UA568686 = {S96965, 54, { &P14983,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (NokiaC7-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA591312 = {S96966, 94, { &P17595,&P17297,&P20460,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:10.0.2) Gecko/X Firefox/10.0.2 SeaMonkey/2.7.2*/
const Device UA616403 = {S96967, 82, { &P17465,&P5690,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; Opera Tablet/11648; U; en) Presto/2.7.81 Version/11.00*/
const Device UA634203 = {S96968, 16, { &P24378,&P17811,&P17810,&P18092 }};	/*Useragent - LENOVO-P780/176A*/
const Device UA728552 = {S96969, 35, { &P13365,&P17483,&P17338,&P18092 }};	/*Useragent - lg-gu220/UC Browser7.9.0.102/69/352*/
const Device UA745121 = {S96970, 44, { &P13220,&P17485,&P17347,&P18092 }};	/*Useragent - sonyericssonk770i/UC Browser8.0.3.107/69/444*/
const Device UA806730 = {S96971, 44, { &P13220,&P17485,&P17347,&P18092 }};	/*Useragent - sonyericssonk770i/UC Browser8.0.3.107/69/352*/
const Device UA747429 = {S96972, 126, { &P23645,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL32CX523; CC/GBR; en) Presto/2.7.61 Version/11.00*/
const Device UA776766 = {S96973, 71, { &P15364,&P17285,&P22534,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:9.0.1) Gecko/X Firefox/9.0.1 Epic/9.0.1*/
const Device UA791719 = {S96974, 56, { &P15767,&P18575,&P18245,&P18092 }};	/*Useragent - Bing%20for%20iPad/1.1.2 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA794899 = {S96975, 90, { &P15364,&P17285,&P22530,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.7) Gecko/X Epic/1.1 Firefox/3.6.7*/
const Device UA898942 = {S96976, 84, { &P14519,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-in; NokiaN97 mini)/UC Browser8.4.0.159/50/352/UCWEB Mobile*/
const Device UA1976273 = {S96977, 35, { &P14519,&P5129,&P24670,&P18092 }};	/*Useragent - NokiaN97 mini/UCWEB8.4.0.150/50/999*/
const Device UA902257 = {S96978, 164, { &P14967,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA903338 = {S96979, 168, { &P12482,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959 Build/IMM76I; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1900286 = {S96980, 58, { &P15364,&P18146,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; SunOS sun4u; rv:7.0) Gecko/X Firefox/7.0*/
const Device UA1900385 = {S96981, 77, { &P15364,&P17285,&P22313,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:7.0) Gecko/X Firefox/7.0 SeaMonkey/2.4*/
const Device UA1902491 = {S96982, 126, { &P23650,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46HX823; CC/GBR; en) Presto/2.7.61 Version/11.00*/
const Device UA1903588 = {S96983, 36, { &P12174,&P17484,&P17338,&P18092 }};	/*Useragent - sgh-f480i/UC Browser7.9.0.102/69/352*/
const Device UA2021684 = {S96984, 138, { &P22391,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HT_PAD1051 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1953444 = {S96985, 72, { &P21388,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2.2;Zh_cn;Karbonn_A1;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA1977264 = {S96986, 138, { &P13644,&P24697,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Evo 4G Build/MIUI) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2013742 = {S96987, 160, { &P18385,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2029921 = {S96988, 160, { &P18385,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03H; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2024121 = {S96989, 158, { &P15397,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ST18i Build/4.0.A.2.368; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027041 = {S96990, 171, { &P14031,&P5477,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; GT-I5503 Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2934844 = {S96991, 175, { &P14031,&P5477,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; GT-I5503 Build/ERE27) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2028945 = {S96992, 160, { &P20183,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028994 = {S96993, 148, { &P12918,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LS855 Build/CTMod 3.68)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029500 = {S96994, 139, { &P22084,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; iDx10 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2150478 = {S96995, 51, { &P22907,&P18110,&P17810,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.2; zh-cn; Lenovo_A390e; 320*480)*/
const Device UA2223453 = {S96996, 129, { &P20585,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AN7G3 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2260054 = {S96997, 97, { &P15364,&P17188,&P22516,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; WinNT4.0; en-US; rv:1.8.1.12) Gecko/X Firefox/2.0.0.12 Navigator/9.0.0.6*/
const Device UA2262843 = {S96998, 83, { &P15364,&P18145,&P19961,&P18092 }};	/*Useragent - Opera/9.80 (X11; FreeBSD 9.0-RELEASE-p3 amd64; U; en) Presto/2.10.289 Version/12.02*/
const Device UA2306746 = {S96999, 67, { &P15809,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux; U;2.3.5;zh_cn;HTC S510b;480*800;)UCWEB7.6.0.75/139/32500*/
const Device UA2337496 = {S97000, 79, { &P15206,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE90)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2355178 = {S97001, 198, { &P14225,&P5629,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, gb*/
const Device UA2393710 = {S97002, 78, { &P15364,&P7127,&P22578,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; Edition Next; de) Presto/2.11.310 Version/12.50*/
const Device UA2415038 = {S97003, 131, { &P21924,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MTP-710 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2425220 = {S97004, 39, { &P15364,&P18580,&P24427,&P18092 }};	/*Useragent - Dolphin/6.2 CFNetwork/609 Darwin/13.0.0*/
const Device UA2443146 = {S97005, 26, { &P16273,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-T679/100.20306*/
const Device UA2445901 = {S97006, 84, { &P12142,&P17485,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; sonyericssonu5i) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2449290 = {S97007, 154, { &P16356,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2456689 = {S97008, 250, { &P18506,&P5043,&P18527,&P18092 }};	/*Useragent - Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/69/444*/
const Device UA2466762 = {S97009, 136, { &P15911,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT862 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2475456 = {S97010, 137, { &P12249,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MiniPro Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2475792 = {S97011, 124, { &P18088,&P5690,&P18122,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/(Windows; U; Windows NT 5.1; en-US) AppleWebKit/23.411; U; en) Presto/2.5.25 Version/10.54*/
const Device UA2477502 = {S97012, 133, { &P21237,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TOUCHMATE Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2502491 = {S97013, 157, { &P14877,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2502533 = {S97014, 146, { &P19937,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; A73 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2516977 = {S97015, 50, { &P15364,&P18145,&P17389,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/2.2.2; FreeBSD)*/
const Device UA2525551 = {S97016, 52, { &P15364,&P7133,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0. (AAS-2.2.2; U; Windows NT 6.1)*/
const Device UA2527254 = {S97017, 125, { &P15198,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s3650) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2528696 = {S97018, 99, { &P13216,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonk750i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2529570 = {S97019, 173, { &P12444,&P5477,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; U20i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2544295 = {S97020, 154, { &P18230,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; A1_07 Build/GINGERBREAD; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2545466 = {S97021, 54, { &P14519,&P5890,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 (NokiaN97mini;SymbianOS/9.1 Series60/3.0)*/
const Device UA2554549 = {S97022, 146, { &P13157,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Sensation XE Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2554802 = {S97023, 81, { &P22875,&P18110,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Adr (Linux; U; 4.0.4; zh-cn; GT-I9220 Build/IMM76D.ZCLPR;800*1280)*/
const Device UA2588514 = {S97024, 43, { &P16384,&P17484,&P4759,&P18092 }};	/*Useragent - samsung-gt-s5380k/UC Browser7.6.1.82/69/351*/
const Device UA2589241 = {S97025, 71, { &P19859,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC_T328w; 480*800) UCWEB7.9.4.145/139/800*/
const Device UA2592977 = {S97026, 139, { &P22150,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-tw; TOOKY A110 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2593846 = {S97027, 137, { &P20130,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SONY-Q8 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2594771 = {S97028, 161, { &P12444,&P5477,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; U20i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2598610 = {S97029, 88, { &P23532,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; id; tcl-c616) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2986368 = {S97030, 91, { &P23532,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; tcl-c616) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2601777 = {S97031, 129, { &P20446,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; F-05E Build/V13R36C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2602994 = {S97032, 136, { &P18278,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; F-05D Build/V08R31C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2605924 = {S97033, 138, { &P14183,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-I997 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2607485 = {S97034, 73, { &P14738,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;7 Mozart T8698;7.10;8862)*/
const Device UA2609162 = {S97035, 28, { &P13452,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (GT-P1000 FROYO)*/
const Device UA2620836 = {S97036, 29, { &P13452,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (GT-P1000 IMM76L)*/
const Device UA2611161 = {S97037, 147, { &P15985,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6200 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2611400 = {S97038, 147, { &P17323,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6800 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2612960 = {S97039, 154, { &P17323,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6800 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2615816 = {S97040, 272, { &P17323,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-P6800 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 TwonkyBeamBrowser/3.2.1-65 (Android 4.0.4; Samsung GT-P6800 Build/cm_p6800-userdebug 4.0.4 IMM76L eng.locerra.X.X test-keys)*/
const Device UA2615127 = {S97041, 157, { &P13452,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2621259 = {S97042, 158, { &P13452,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.3; th-th; GT-P1000T Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2617241 = {S97043, 38, { &P12879,&P17979,&P18062,&P18092 }};	/*Useragent - Android/4.0.4; samsung GT-P7510; 2.3.2*/
const Device UA2633019 = {S97044, 147, { &P12879,&P17979,&P18062,&P18092 }};	/*Useragent - Rdio.Net; Android/2.5; Android/4.0.4; DM=samsung; DN=GT-P7510; DC=samsung; DF=samsung/GT-P7510/GT-P7510:4.0.4/IMM76D/VLLH3:user/release-keys; DH=p3*/
const Device UA2621980 = {S97045, 95, { &P15985,&P17979,&P22073,&P18092 }};	/*Useragent - Instagram 3.3.3 Android (13/3.2; 160dpi; 600x976; samsung; GT-P6200; GT-P6200; smdkc210; en_GB)*/
const Device UA2622160 = {S97046, 98, { &P17323,&P17979,&P22073,&P18092 }};	/*Useragent - Instagram 3.3.3 Android (15/4.0.4; 160dpi; 800x1232; samsung; GT-P6800; GT-P6800; smdk4210; en_US)*/
const Device UA2630805 = {S97047, 153, { &P13452,&P5096,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000*/
const Device UA2639586 = {S97048, 113, { &P15364,&P17814,&P21955,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.0.6) Gecko/X Ubuntu/8.10 (intrepid) Minefield/3.0 MEGAUPLOAD 2.0*/
const Device UA2640790 = {S97049, 50, { &P12061,&P5890,&P21761,&P18092 }};	/*Useragent - iBrowser/2.5 (NokiaE65;SymbianOS/9.1 Series60/3.0)*/
const Device UA2657239 = {S97050, 172, { &P18213,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6500D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2657244 = {S97051, 133, { &P21634,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M-MP875S2 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2661618 = {S97052, 165, { &P18240,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; LG-P700 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2667611 = {S97053, 157, { &P21068,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI-ONE C1 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2668275 = {S97054, 136, { &P21448,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SH530U Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2668722 = {S97055, 143, { &P20073,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tmn smart a15 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2686711 = {S97056, 76, { &P17595,&P18207,&P20795,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:2.0.1) Gecko/X Postbox/2.5.3*/
const Device UA2701362 = {S97057, 152, { &P18714,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V889D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2701372 = {S97058, 156, { &P18507,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2702373 = {S97059, 176, { &P13094,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; Lenovo A60+ Build/LenovoLePhone) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2703665 = {S97060, 161, { &P16292,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9050 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2708343 = {S97061, 138, { &P20525,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A898 Duo Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2711689 = {S97062, 137, { &P20525,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A898 Duo Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2712638 = {S97063, 127, { &P22814,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PC906 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2713944 = {S97064, 137, { &P12697,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; shooteru Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2714485 = {S97065, 136, { &P21089,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE T22 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2721910 = {S97066, 80, { &P17595,&P18207,&P18329,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:11.0) Gecko/X Thunderbird/11.0.1*/
const Device UA2722705 = {S97067, 145, { &P20239,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MSI Enjoy 10 Plus Build/1.3ST) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2723662 = {S97068, 132, { &P23422,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ru-ru; TM-9737W Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2727085 = {S97069, 128, { &P23673,&P24094,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46HX753; CC/GBR; en) Presto/2.10.250 Version/11.60*/
const Device UA2731288 = {S97070, 98, { &P11843,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; SAMSUNGSGH-i450) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2734653 = {S97071, 139, { &P22624,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Burst S280 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734739 = {S97072, 138, { &P21304,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Fly_IQ450 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735231 = {S97073, 134, { &P24549,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; E2001 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735907 = {S97074, 137, { &P18701,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-T100K Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2747056 = {S97075, 159, { &P15820,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-SU640 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2748214 = {S97076, 70, { &P17952,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; SHW-M110S; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2748463 = {S97077, 182, { &P21152,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ALCATEL ONE TOUCH 930D Build/1.2.10.4) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2749951 = {S97078, 134, { &P23370,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Oysters T34 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2750125 = {S97079, 138, { &P20240,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Optimus G Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2750415 = {S97080, 155, { &P11747,&P5096,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-CN; HTC Desire Build/FRF91) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2750806 = {S97081, 130, { &P18478,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6201 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2750882 = {S97082, 135, { &P20155,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; T-909S Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2751015 = {S97083, 136, { &P20290,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A11 Build/KarbonnA11) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2752353 = {S97084, 136, { &P23189,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Surfer 8.31 3G Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2762670 = {S97085, 131, { &P18708,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; U30GT-H Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2767722 = {S97086, 142, { &P22608,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Spice Mi-530 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2768054 = {S97087, 157, { &P15850,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LG-LU6200 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2779903 = {S97088, 145, { &P20262,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Karbonn_A18 Build/KarbonnA18) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2780784 = {S97089, 156, { &P19898,&P5629,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-I699 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2781699 = {S97090, 160, { &P19997,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo P700i Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2789697 = {S97091, 128, { &P23677,&P24094,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40EX653; CC/GBR; en) Presto/2.10.250 Version/11.60*/
const Device UA2790352 = {S97092, 130, { &P23906,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; I10-LE Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2790901 = {S97093, 73, { &P21338,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; Symphony_W10; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2791622 = {S97094, 131, { &P21212,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; N90DC_S Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2791920 = {S97095, 141, { &P13054,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; ADR6425LVW Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2795065 = {S97096, 100, { &P20126,&P17485,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk800i) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2795596 = {S97097, 162, { &P18581,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; SCH-S720C Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2818856 = {S97098, 182, { &P14693,&P5843,&P24671,&P18092 }};	/*Useragent - NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac1-02) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2827240 = {S97099, 126, { &P14851,&P17484,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5753e) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2844593 = {S97100, 68, { &P18281,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.7; zh-cn; LG-P940; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2845737 = {S97101, 60, { &P14017,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; LG-P990 Build/IMM76L)*/
const Device UA2849201 = {S97102, 26, { &P22037,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-P3113/100.40101*/
const Device UA2850716 = {S97103, 151, { &P23295,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; ZTE U807 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2851221 = {S97104, 154, { &P12414,&P5843,&P24676,&P18092 }};	/*Useragent - Nokia7020/2.0 (08.11) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7020) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2854375 = {S97105, 99, { &P14784,&P5477,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; HTC_Hero) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2854682 = {S97106, 98, { &P20267,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; P300(Funbook)) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2863131 = {S97107, 139, { &P13054,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ADR6425LVW Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2863600 = {S97108, 97, { &P13815,&P5096,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; HTC Wildfire) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2864405 = {S97109, 125, { &P21216,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A78 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2865081 = {S97110, 90, { &P18223,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT22i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2867442 = {S97111, 160, { &P14993,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.4; LT15i Build/4.0.2.A.0.62) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2867548 = {S97112, 130, { &P18478,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6201 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2869712 = {S97113, 93, { &P16356,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-N7000) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2877299 = {S97114, 142, { &P21617,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877817 = {S97115, 134, { &P18655,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SH-09D Build/S4060) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2877997 = {S97116, 133, { &P13386,&P5448,&P4777,&P18092 }};	/*Useragent - LG-GM730/v10a WM/6.1 IEMobile/6.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2878419 = {S97117, 143, { &P19781,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 97 TITANIUMHD Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2879177 = {S97118, 144, { &P20516,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Woxter Tablet PC 97IPS Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2891046 = {S97119, 100, { &P15364,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-sgh-t528g) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA3099964 = {S97120, 99, { &P15364,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-c3322) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2892571 = {S97121, 152, { &P20877,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ALCATEL ONE TOUCH 8000D Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892795 = {S97122, 134, { &P24549,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; E2001 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892875 = {S97123, 135, { &P23241,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GN708W Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2894213 = {S97124, 147, { &P13105,&P6437,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.2.1; G100W Build/HTK55D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2896028 = {S97125, 160, { &P20540,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PAP4300 DUO Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2897843 = {S97126, 154, { &P20872,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGP311 Build/10.1.C.0.370) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2937193 = {S97127, 152, { &P18739,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; R801 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2937815 = {S97128, 135, { &P23336,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; I_7520 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2938135 = {S97129, 156, { &P23286,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; TECNO F7 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2943865 = {S97130, 128, { &P23975,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B1-710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2952204 = {S97131, 140, { &P18295,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; HTC One X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2952344 = {S97132, 162, { &P11683,&P17485,&P24676,&P18092 }};	/*Useragent - SonyEricssonC905/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonc905) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2953484 = {S97133, 161, { &P18385,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2956279 = {S97134, 97, { &P21850,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Spice_Mi-435) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2965242 = {S97135, 156, { &P22853,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HTC 802w Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2968581 = {S97136, 135, { &P18278,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; F-05D Build/V11R40A) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2982529 = {S97137, 65, { &P19779,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;A1+;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA2984837 = {S97138, 165, { &P24007,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad 5210S Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2985881 = {S97139, 73, { &P15364,&P18145,&P21814,&P18092 }};	/*Useragent - Opera/9.80 (X11; FreeBSD 9.1-RELEASE amd64) Presto/2.12.388 Version/12.11*/
const Device UA2986461 = {S97140, 172, { &P20004,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Karbonn_A2 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2987328 = {S97141, 150, { &P20976,&P18066,&P21729,&P18092 }};	/*Useragent - OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; W100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2992048 = {S97142, 156, { &P21503,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Lemon P101 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2992485 = {S97143, 93, { &P14186,&P5096,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; zh-CN; GT-S5830) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile*/
const Device UA2993533 = {S97144, 92, { &P20587,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; LG-P715) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2994000 = {S97145, 134, { &P23478,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ZP810 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2994994 = {S97146, 71, { &P16463,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;GT-I8350;7.10;8862)*/
const Device UA2995430 = {S97147, 143, { &P21802,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GSmart Maya M1 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2997207 = {S97148, 151, { &P21026,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ZTE U930 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2997484 = {S97149, 129, { &P21698,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VL-107L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2998281 = {S97150, 151, { &P20729,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; A27 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2998425 = {S97151, 150, { &P18321,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2998553 = {S97152, 139, { &P13025,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WT19a Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2998713 = {S97153, 148, { &P18280,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC VLE_U Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2999175 = {S97154, 140, { &P21873,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB M713G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2999196 = {S97155, 159, { &P12657,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC EVO 3D X515m Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2999316 = {S97156, 168, { &P21858,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI Y300-0000 Build/HuaweiY300-0000) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3000423 = {S97157, 148, { &P24070,&P18598,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; R815T Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3001048 = {S97158, 113, { &P15785,&P5043,&P18567,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2010/11.95; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.2.0.0.36*/
const Device UA3005018 = {S97159, 144, { &P16092,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-us; DROID X2 Build/4.5.1A-DTN-200-18) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3012657 = {S97160, 155, { &P22805,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-E455 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3016168 = {S97161, 89, { &P20169,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; N320) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3016573 = {S97162, 155, { &P22585,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SPH-L720 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3016839 = {S97163, 158, { &P15047,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; SK17i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3017198 = {S97164, 155, { &P20290,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A11 Build/KarbonnA11) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3020627 = {S97165, 134, { &P15365,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Magic Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020632 = {S97166, 170, { &P22941,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6812 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3021040 = {S97167, 132, { &P22816,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S22 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021081 = {S97168, 142, { &P21733,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 3 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021233 = {S97169, 134, { &P21011,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; L-01E Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021583 = {S97170, 142, { &P21617,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021700 = {S97171, 172, { &P20289,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC6435LVW Build/JDQ39E; Carbon-CARBON-JB-UNOFFICIAL-X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3021832 = {S97172, 130, { &P20425,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N-03E Build/AX) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021971 = {S97173, 130, { &P16168,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; NookColor Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022485 = {S97174, 132, { &P23846,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A12 Build/RheaTK) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022486 = {S97175, 139, { &P15354,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST17i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022563 = {S97176, 137, { &P16436,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I8150 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022828 = {S97177, 137, { &P18722,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IM-A830L Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023373 = {S97178, 149, { &P20841,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 10 Build/1.1.11-1015 X-10:30) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023988 = {S97179, 137, { &P23247,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Primo-X1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3027660 = {S97180, 159, { &P18324,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; LG-VM696 Build/ZV5.GWK74) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3042577 = {S97181, 162, { &P18364,&P5129,&P3690,&P18092 }};	/*Useragent - NokiaC5-05/23.5.015 (SymbianOS/9.4; U; Series60/5.0 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Safari/525 3gpp-gba*/
const Device UA3045630 = {S97182, 130, { &P22860,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ITQ700 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3047144 = {S97183, 131, { &P21639,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M-MP860S2 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3049467 = {S97184, 154, { &P21611,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3050645 = {S97185, 150, { &P12572,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-D600 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA3052166 = {S97186, 134, { &P18661,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-us; N860 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3056815 = {S97187, 158, { &P18564,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC One X+ Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3057026 = {S97188, 183, { &P18622,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; vi-vn; IM-A820L Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 MoKee OpenSource/MOKEE_4.2.2_A820L_V1/ef65l*/
const Device UA3057526 = {S97189, 136, { &P24076,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; es-es; PAD720 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057719 = {S97190, 139, { &P17108,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Inspire 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3059087 = {S97191, 101, { &P15483,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_ChaCha_A810e) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3060069 = {S97192, 168, { &P21858,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI Y300-0000 Build/HuaweiY300-0000) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3073821 = {S97193, 152, { &P15364,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; 8720 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3077953 = {S97194, 72, { &P22594,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile: SGH-M919; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA3078555 = {S97195, 176, { &P19862,&P18598,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Micromax_A110Q Build/JOP40D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3078825 = {S97196, 92, { &P12280,&P17146,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (iOS; U; iPh OS 5_0_1; zh-CN; iPh3,1) U2/1.0.0 UCBrowser/9.2.0.312 U2/1.0.0 Mobile*/
const Device UA3078939 = {S97197, 160, { &P23200,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Coolpad 5890 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3079321 = {S97198, 159, { &P20001,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo S720 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3079532 = {S97199, 101, { &P12707,&P5055,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia701) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3079582 = {S97200, 149, { &P22228,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SM-T210 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3131021 = {S97201, 158, { &P18724,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; IdeaTab A2107A-H Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3088171 = {S97202, 158, { &P23716,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI A199 Build/HuaweiA199) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA3092319 = {S97203, 80, { &P17595,&P18207,&P22171,&P18090 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:23.0) Gecko/X Firefox/23.0 slurp*/
const Device UA3092278 = {S97204, 133, { &P21853,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Connect-3G-2.0 Build/HCL) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3092690 = {S97205, 173, { &P20004,&P18066,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn_A2+ Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3092892 = {S97206, 157, { &P22941,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6812 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3101140 = {S97207, 138, { &P12942,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I927 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103338 = {S97208, 147, { &P18510,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; ALCATEL ONE TOUCH 918N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3108317 = {S97209, 186, { &P12056,&P5204,&P24055,&P18092 }};	/*Useragent - Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311*/
const Device UA3108461 = {S97210, 93, { &P18507,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I8160) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3108519 = {S97211, 99, { &P15492,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-c3322) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3108554 = {S97212, 139, { &P24656,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; HTC 609d Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108610 = {S97213, 94, { &P13591,&P5096,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; ZTE-BLADE) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3108779 = {S97214, 95, { &P14842,&P5477,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E10i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3108933 = {S97215, 96, { &P12329,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN86 8MP) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3109095 = {S97216, 170, { &P19847,&P18066,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S6012 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3109098 = {S97217, 153, { &P23150,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Q1000 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109177 = {S97218, 92, { &P15206,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE90) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3109404 = {S97219, 159, { &P12616,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; WT19i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109672 = {S97220, 161, { &P15649,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5570 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109938 = {S97221, 159, { &P20183,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI-ONE Plus Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109967 = {S97222, 157, { &P22756,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; C1604 Build/11.3.A.2.1) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3110476 = {S97223, 155, { &P18621,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; LG-E612 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3111185 = {S97224, 156, { &P22396,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5312 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3111849 = {S97225, 157, { &P21598,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar_Touch_KEM_452_HDMI Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3112491 = {S97226, 94, { &P12236,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6700s) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3112706 = {S97227, 139, { &P20006,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT30a Build/7.0.A.3.195) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3113264 = {S97228, 102, { &P18513,&P18110,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-S6802) U2/1.0.0 UCBrowser/9.3.0.321 Mobile*/
const Device UA3126206 = {S97229, 140, { &P21526,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cherry w500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126284 = {S97230, 154, { &P12879,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; GT-P7510 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.2/p4wifi*/
const Device UA3127138 = {S97231, 157, { &P12968,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LT26i Build/6.1.A.2.55) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127166 = {S97232, 132, { &P20807,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; B92 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127213 = {S97233, 146, { &P21646,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; BLU STUDIO 5.3 II Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127751 = {S97234, 158, { &P16044,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; TouchPad Build/JLS36G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 CyanogenMod/10.2/tenderloin*/
const Device UA3131618 = {S97235, 138, { &P21283,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6310N Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131806 = {S97236, 137, { &P23293,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-LS720 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132624 = {S97237, 146, { &P15880,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132886 = {S97238, 137, { &P18488,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA4432 = {S97239, 64, { &P14414,&P17483,&P3534,&P18092 }};	/*Useragent - LG-TU720/V0.9 Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA45061 = {S97240, 92, { &P11724,&P5033,&P4948,&P18092 }};	/*Useragent - BlackBerry8310/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102 UP.Browser/5.0.3.3*/
const Device UA18558 = {S97241, 142, { &P14117,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; GT540R Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA388951 = {S97242, 173, { &P14117,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-ca; LG-GT540R Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA19012 = {S97243, 46, { &P14845,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_HD_T8282 Opera/9.7 (Windows NT 5.1; U; en)*/
const Device UA19017 = {S97244, 52, { &P14845,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_HD_T8282 Opera/9.7 (Windows NT 5.1; U; en)*/
const Device UA34451 = {S97245, 53, { &P14845,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Touch_HD_T8282 Opera/9.70 (Windows NT 5.1; U; en)*/
const Device UA19966 = {S97246, 98, { &P15018,&P19944,&P3625,&P18092 }};	/*Useragent - SHARP-TQ-GX17/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.3.2.d.1.112 (GUI) MMP/2.0*/
const Device UA20405 = {S97247, 101, { &P15084,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E340/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA20913 = {S97248, 98, { &P15146,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-A717/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA21244 = {S97249, 71, { &P11980,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_P3300 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA35935 = {S97250, 129, { &P11980,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; HTC_P3300/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21919 = {S97251, 73, { &P12018,&P5448,&P15792,&P18092 }};	/*Useragent - HTC-8500/1.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8)*/
const Device UA24774 = {S97252, 80, { &P14415,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonZ1010/R1K SEMC-Browser/4.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54782 = {S97253, 80, { &P14415,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonZ1010/R1H SEMC-Browser/4.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62491 = {S97254, 78, { &P14415,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonZ1010/R1G SEMC-Browser/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA25427 = {S97255, 99, { &P12121,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-W7/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.2 (GUI) MMP/2.0 Push/PO*/
const Device UA32655 = {S97256, 57, { &P12510,&P17017,&P3625,&P18092 }};	/*Useragent - KWC-M1000/1.0.07 UP.Browser/6.2.3.9.g.1.110 (GUI) MMP/2.0*/
const Device UA51358 = {S97257, 57, { &P12510,&P17017,&P3625,&P18092 }};	/*Useragent - KWC-M1000/1.0.06 UP.Browser/6.2.3.9.g.1.110 (GUI) MMP/2.0*/
const Device UA36228 = {S97258, 70, { &P14686,&P5448,&P17173,&P18092 }};	/*Useragent - HTC_S621 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA64628 = {S97259, 69, { &P14686,&P5448,&P17173,&P18092 }};	/*Useragent - HTCS621-Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA43717 = {S97260, 153, { &P16202,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; U; Series60/3.2 Nokia6650fold/04.18; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA50243 = {S97261, 81, { &P13820,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonK320i/R4GC Java/SEMC-Java/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA51200 = {S97262, 74, { &P13683,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-B3310i/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53779 = {S97263, 29, { &P14023,&P22839,&P17470,&P18092 }};	/*Useragent - ZTE-R231/2.0 Profile/MIDP-2.0*/
const Device UA55864 = {S97264, 45, { &P15278,&P17484,&P4902,&P18092 }};	/*Useragent - Samsung-SPHM330 Infraware-Polaris/6.1 MMP/2.0*/
const Device UA56773 = {S97265, 110, { &P14345,&P16000,&P4715,&P18092 }};	/*Useragent - NokiaN-GageQD/1.0 SymbianOS/6.1 Series60/1.2 Profile/MIDP-1.0 Configuration/CLDC-1.0/UC Browser7.4.0.65/26/351*/
const Device UA63171 = {S97266, 110, { &P14484,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-S7070C/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA67175 = {S97267, 58, { &P14770,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3H UP.Browser/6.2_7.2.7.1.K.1.5.1.120 (GUI) MMP/2.0*/
const Device UA67343 = {S97268, 52, { &P14815,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-BLAZE/00.72 UP.Browser/7.2.7.5.678 (GUI) MMP/2.0*/
const Device UA67715 = {S97269, 7, { &P12856,&P17483,&P17470,&P18092 }};	/*Useragent - lg-t300*/
const Device UA85249 = {S97270, 36, { &P16755,&P17017,&P17470,&P18092 }};	/*Useragent - PT-GF200 CLDC/CLDC-1.0 MIDP/MIDP-1.0*/
const Device UA87240 = {S97271, 47, { &P17501,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-CA38 UP.Browser/6.2.0.13.1.5 (GUI) MMP/2.0*/
const Device UA87929 = {S97272, 45, { &P17645,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-ST33 UP.Browser/6.2.0.13.2 (GUI) MMP/2.0*/
const Device UA348627 = {S97273, 31, { &P14188,&P5129,&P24666,&P18092 }};	/*Useragent - Nokia5233/UCWEB8.1.0.104/50/800*/
const Device UA379621 = {S97274, 54, { &P12449,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (NokiaE5-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA541038 = {S97275, 70, { &P15364,&P7127,&P22334,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:2.0) Gecko/X Firefox/4.0 CometBird/4.0*/
const Device UA543379 = {S97276, 52, { &P18088,&P17017,&P24661,&P18092 }};	/*Useragent - IUC(U;iOS 4.2.1;Zh-cn;320*480;)/UCWEB7.7.1.90/41/997*/
const Device UA747294 = {S97277, 52, { &P18088,&P17017,&P24661,&P18092 }};	/*Useragent - IUC(U;iOS 4.3.3;Zh-cn;320*480;)/UCWEB7.7.1.90/41/997*/
const Device UA570503 = {S97278, 54, { &P14033,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (NokiaC5-03;SymbianOS/9.1 Series60/3.0)*/
const Device UA574078 = {S97279, 64, { &P15364,&P18144,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64; rv:7.0.1) Gecko/X Firefox/7.0.1*/
const Device UA586252 = {S97280, 131, { &P15364,&P7133,&P18192,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:9.0) Gecko/X Thunderbird/9.0.1 OracleBeehiveExtension/1.0.0.4-OracleInternal Lightning/1.1.1*/
const Device UA648350 = {S97281, 104, { &P12513,&P17485,&P17347,&P18092 }};	/*Useragent - SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonw20/UC Browser8.0.3.107/69/352*/
const Device UA2262885 = {S97282, 45, { &P17595,&P18575,&P18534,&P18092 }};	/*Useragent - Reeder/1.5.5 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA681886 = {S97283, 90, { &P17595,&P17297,&P20458,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:8.0.1) Gecko/X Firefox/8.0.1 SeaMonkey/2.5*/
const Device UA689119 = {S97284, 108, { &P13308,&P17485,&P17347,&P18092 }};	/*Useragent - SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonj105i/UC Browser8.0.3.107/69/352*/
const Device UA1978562 = {S97285, 48, { &P23144,&P18585,&P17470,&P18092 }};	/*Useragent - Facebook/4110.0 CFNetwork/485.10.2 Darwin/10.3.1*/
const Device UA744097 = {S97286, 104, { &P12513,&P17485,&P17338,&P18092 }};	/*Useragent - SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonw20/UC Browser7.9.0.102/69/352*/
const Device UA755733 = {S97287, 35, { &P23545,&P17017,&P17347,&P18092 }};	/*Useragent - mauie800/UC Browser8.0.3.107/69/444*/
const Device UA784845 = {S97288, 69, { &P15364,&P17285,&P22349,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:7.0) Gecko/X Firefox/7.0 PaleMoon/7.0*/
const Device UA834404 = {S97289, 58, { &P15364,&P18146,&P17207,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; SunOS sun4v; rv:5.0) Gecko/X Firefox/5.0*/
const Device UA891191 = {S97290, 122, { &P15364,&P16110,&P22429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 Comodo_Dragon/19.0.3.0*/
const Device UA893329 = {S97291, 129, { &P15364,&P16110,&P22429,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 Comodo_Dragon/19.0.3.0*/
const Device UA906857 = {S97292, 140, { &P12280,&P17146,&P20785,&P18092 }};	/*Useragent - MQQBrowser/34 Mozilla/5.0 (iPhone 4; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA1965228 = {S97293, 141, { &P12280,&P17146,&P20785,&P18092 }};	/*Useragent - MQQBrowser/34 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Safari/7534.48.3*/
const Device UA1841424 = {S97294, 103, { &P15364,&P17285,&P22482,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6.1 FBSMTWB*/
const Device UA1855001 = {S97295, 112, { &P15364,&P5690,&P17335,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; fi-FI) AppleWebKit/528.16 (KHTML, like Gecko) Version/4.1 Safari/528.16*/
const Device UA1855003 = {S97296, 107, { &P15364,&P5690,&P17335,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en) AppleWebKit/526.9 (KHTML, like Gecko) Version/4.1 Safari/526.8*/
const Device UA1888888 = {S97297, 87, { &P17595,&P17296,&P18211,&P18092 }};	/*Useragent - GoogleEarth/5.2.1.1588(Macintosh;Mac OS X (10.6.8);en;kml:2.2;client:Free;type:default)*/
const Device UA1902072 = {S97298, 45, { &P12707,&P5055,&P17338,&P18092 }};	/*Useragent - nokia701/UC Browser7.9.0.102/69/352  3gpp-gba*/
const Device UA1903233 = {S97299, 104, { &P11808,&P5204,&P17338,&P18092 }};	/*Useragent - Nokia5610d-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5610d-1/UC Browser7.9.0.102/69/352*/
const Device UA1948872 = {S97300, 89, { &P15364,&P5690,&P22353,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:9.1) Gecko/X Firefox/9.1-x64 PaleMoon/9.1-x64*/
const Device UA1952818 = {S97301, 167, { &P18296,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC One V Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA1962496 = {S97302, 173, { &P11950,&P5477,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; X10i Build/2.1.A.0.435) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352*/
const Device UA2020167 = {S97303, 133, { &P22268,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-tw; MH350 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021797 = {S97304, 136, { &P22239,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; EB-W51GJ Build/2.00a) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022294 = {S97305, 136, { &P22239,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; EB-W51GJ Build/2.01d) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023271 = {S97306, 142, { &P21510,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; zh-tw; MioPad 6 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025506 = {S97307, 145, { &P17996,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; MID1126 Build/CaribbeanSunset_v1.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2033785 = {S97308, 133, { &P21876,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; MyPad 103 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2167099 = {S97309, 153, { &P12616,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; WT19i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2169526 = {S97310, 33, { &P18506,&P5043,&P21759,&P18092 }};	/*Useragent - iBrowser/Mini2.8 (Nokia305/03.60)*/
const Device UA2210612 = {S97311, 86, { &P17588,&P5567,&P4536,&P18092 }};	/*Useragent - Mozilla/5.0 Opera/9.5 (KDDI-TS3V; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00*/
const Device UA2240912 = {S97312, 131, { &P18385,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Nexus 7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2285492 = {S97313, 36, { &P11905,&P5890,&P4742,&P18092 }};	/*Useragent - nokia6120c/UC Browser7.5.1.77/69/352*/
const Device UA2287879 = {S97314, 80, { &P12638,&P5448,&P4777,&P18092 }};	/*Useragent - SAMSUNG-SGH-i780/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2301370 = {S97315, 108, { &P15364,&P18138,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.13) Gecko/X Linux Mint/11 (Katya) Firefox/3.6.13 (Swiftfox)*/
const Device UA2303256 = {S97316, 86, { &P17581,&P5567,&P4536,&P18092 }};	/*Useragent - Mozilla/5.0 Opera/9.5 (KDDI-SH3J; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00*/
const Device UA2317384 = {S97317, 77, { &P14763,&P5776,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; nokian72) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2321308 = {S97318, 102, { &P17595,&P18110,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Linux; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA2329661 = {S97319, 81, { &P15364,&P17285,&P22551,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:13.0) Gecko/X Firefox/13.0.3.0 IceDragon/13.0.3.0*/
const Device UA2331986 = {S97320, 251, { &P14540,&P5843,&P18524,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/70/352*/
const Device UA2350372 = {S97321, 170, { &P21700,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; Penta IS701C Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2352203 = {S97322, 40, { &P16463,&P17484,&P20762,&P18092 }};	/*Useragent - SAMSUNG-GT-I8350/UCWEB2.5.0.202/47/32079*/
const Device UA2372087 = {S97323, 52, { &P12588,&P5055,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 (NokiaC6-01;SymbianOS/9.1 Series60/3.0)*/
const Device UA2375660 = {S97324, 90, { &P11932,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA6210 Navigator)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2392457 = {S97325, 36, { &P14033,&P5129,&P4722,&P18092 }};	/*Useragent - NokiaC5-03/UC Browser7.0.0.41/50/351*/
const Device UA2834305 = {S97326, 116, { &P12856,&P17483,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; lg-t300) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/405/UCWEB Mobile*/
const Device UA2403348 = {S97327, 63, { &P15767,&P18117,&P22071,&P18092 }};	/*Useragent - Instagram 3.1.2 (iPad; iPhone OS 5.1.1; en_US) AppleWebKit/420+*/
const Device UA2437735 = {S97328, 101, { &P15364,&P22202,&P21956,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1b2) Gecko/X Fedora/3.1-0.4.beta2.fc11 Minefield/3.1b2*/
const Device UA2445159 = {S97329, 67, { &P15079,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.4;Zh_cn;MK16i;480*854;)UCWEB7.8.0.95/139/444*/
const Device UA2445290 = {S97330, 132, { &P13006,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; folio100 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2448901 = {S97331, 54, { &P14983,&P5890,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1 (NokiaC7-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA2455442 = {S97332, 174, { &P12280,&P17145,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.1 Mobile/8C148 Safari/7534.48.3 UCBrowser8.5.0.195/41/355*/
const Device UA2455672 = {S97333, 106, { &P15364,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Bold) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.0.1337 Mobile Safari/537.10+*/
const Device UA2455799 = {S97334, 147, { &P20158,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; A7* Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2475547 = {S97335, 54, { &P13324,&P5890,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1 (NokiaN8-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA2494522 = {S97336, 68, { &P16319,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; U8860; 480*854) UCWEB7.9.4.145/139/7675*/
const Device UA2502567 = {S97337, 167, { &P16321,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; X500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2502649 = {S97338, 140, { &P24572,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GPLUS N930 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2502967 = {S97339, 168, { &P20413,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; CELKON CT2 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2507024 = {S97340, 93, { &P16591,&P22784,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; micromaxq50) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2516922 = {S97341, 139, { &P17595,&P17296,&P22443,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Iron/4.0.275.2 Chrome/4.0.275.2 Safari/532.5*/
const Device UA2519821 = {S97342, 73, { &P21444,&P18110,&P20793,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; PLOYER-MOMO; 800*444) UCWEB1.0.1.113/147/800*/
const Device UA2530484 = {S97343, 137, { &P14123,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; LG-P970 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2531190 = {S97344, 133, { &P17595,&P17296,&P19995,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/6534.57.2 Sleipnir/4.0.1*/
const Device UA2536529 = {S97345, 139, { &P20495,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; OmegaS200 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2543723 = {S97346, 133, { &P22493,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-in; MTABneo Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2553291 = {S97347, 147, { &P22137,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; tr-tr; PIRANHA BUSINESS TAB 7" Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2553505 = {S97348, 152, { &P20330,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Micromax P350 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2557951 = {S97349, 62, { &P15364,&P17285,&P19981,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:19.0) Gecko/X Thunderbird/19.0*/
const Device UA2588109 = {S97350, 53, { &P15373,&P5890,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 (Nokia5530;SymbianOS/9.1 Series60/3.0)*/
const Device UA2590537 = {S97351, 134, { &P21847,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; ro-ro; EVOTAB FUN Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2594816 = {S97352, 164, { &P17997,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; MID7012 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2594908 = {S97353, 141, { &P20886,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Android T3i Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2595700 = {S97354, 154, { &P21112,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; OPPOX907 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2601606 = {S97355, 131, { &P21224,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N-07D Build/AX) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2617603 = {S97356, 36, { &P17337,&P17979,&P4459,&P18092 }};	/*Useragent - Android/3.1; samsung GT-P7500; 2.3.2*/
const Device UA2629226 = {S97357, 166, { &P15985,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6200 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2646922 = {S97358, 151, { &P20175,&P18066,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI 1S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2653487 = {S97359, 157, { &P18304,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2653981 = {S97360, 75, { &P19862,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.4;Zh_cn;Micromax A110;480*854;)UCWEB7.8.0.95/139/444*/
const Device UA2657020 = {S97361, 161, { &P12502,&P17485,&P24675,&P18092 }};	/*Useragent - SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; sonyericssonw20i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2659797 = {S97362, 174, { &P12848,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; MB526 Build/4.5.1-134_DFP-231) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2660641 = {S97363, 162, { &P21226,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; E860 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2660648 = {S97364, 165, { &P12568,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; GT-B7510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2661668 = {S97365, 171, { &P12867,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-MS910 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2682555 = {S97366, 99, { &P14206,&P5448,&P4737,&P18092 }};	/*Useragent - LG-GW550 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.2.2.51*/
const Device UA2682698 = {S97367, 126, { &P23657,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL32EX421; CC/XLA; en) Presto/2.7.61 Version/11.00*/
const Device UA2684572 = {S97368, 129, { &P23419,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; ru-ru; TM-7025 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2692516 = {S97369, 254, { &P14862,&P5204,&P18527,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/70/352*/
const Device UA2694318 = {S97370, 162, { &P21619,&P5629,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.6; th-th; i-mobile i-style 3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2697442 = {S97371, 77, { &P15364,&P18145,&P20456,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64; rv:18.0) Gecko/X Firefox/18.0 SeaMonkey/2.15*/
const Device UA2698580 = {S97372, 139, { &P23617,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; RunboX5-W Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2703557 = {S97373, 166, { &P18437,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-N8000 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2703695 = {S97374, 167, { &P20180,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; XOLO_A800 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2704243 = {S97375, 171, { &P20170,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A95 Build/CelkonCelkon_A95) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2705853 = {S97376, 62, { &P15364,&P18144,&P18689,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64; rv:19.0) Gecko/X Firefox/19.0*/
const Device UA2706949 = {S97377, 142, { &P23978,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WALKMAN Build/icx1227_E110) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707083 = {S97378, 134, { &P21992,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.6; A9550 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2707140 = {S97379, 140, { &P12521,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SO-02D Build/6.1.F.0.106) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2709345 = {S97380, 153, { &P20175,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI 1S Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2712211 = {S97381, 139, { &P20927,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT-AS45IPS Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2715319 = {S97382, 130, { &P20425,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N-03E Build/AX) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2716467 = {S97383, 134, { &P21493,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IQ444 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2728163 = {S97384, 90, { &P13863,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-gx200) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2730558 = {S97385, 143, { &P21466,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Titan TV S320 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2733273 = {S97386, 131, { &P22614,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; N-04E Build/AX) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2733297 = {S97387, 157, { &P18612,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC T329t Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2734554 = {S97388, 165, { &P17283,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB860 Build/4.5.1A-129_OLY-145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2734730 = {S97389, 134, { &P18296,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; One V Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735015 = {S97390, 134, { &P22762,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; L-01D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735148 = {S97391, 140, { &P20131,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Thrill 430x Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735379 = {S97392, 137, { &P18630,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A800S Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735662 = {S97393, 128, { &P17965,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-01D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2736501 = {S97394, 134, { &P21104,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; X507S Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2739406 = {S97395, 135, { &P22927,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_E7310 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2741788 = {S97396, 147, { &P22715,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; OUYA Console Build/JZO54L-OUYA) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2748245 = {S97397, 147, { &P18739,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; R801 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2748967 = {S97398, 54, { &P14033,&P5890,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4 (NokiaC5-03;SymbianOS/9.1 Series60/3.0)*/
const Device UA2749876 = {S97399, 137, { &P12482,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T959 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2750835 = {S97400, 131, { &P21415,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Quantum 7 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2762822 = {S97401, 96, { &P19863,&P18110,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Adr (Linux; U; 4.0.4; zh-cn; HUAWEI U9508 Build/U9508V100R001CHNC00B018;720*1280)*/
const Device UA2765402 = {S97402, 132, { &P22691,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M-MP1010i Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2766520 = {S97403, 73, { &P15364,&P18118,&P22167,&P18090 }};	/*Useragent - Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:20.0) Gecko/X Firefox/20.0 slurp*/
const Device UA2766849 = {S97404, 153, { &P18451,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26ii Build/6.1.A.2.45) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2767007 = {S97405, 159, { &P21094,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo A789 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2767209 = {S97406, 148, { &P21240,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile i-STYLE Q2 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2767920 = {S97407, 133, { &P18238,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; E330 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2775482 = {S97408, 61, { &P20134,&P18110,&P24661,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.6;En;GT-S5830i;320*480;)UCWEB7.7.0.85/139/999*/
const Device UA2775510 = {S97409, 101, { &P17330,&P5448,&P4777,&P18092 }};	/*Useragent - HTC-ST7377/1.59.502.3 (67150) Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2775780 = {S97410, 71, { &P20983,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; GT-I9308; 720*1280) UCWEB7.9.4.145/139/800*/
const Device UA2776491 = {S97411, 139, { &P20251,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC T328d Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2777721 = {S97412, 33, { &P14912,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SAMSUNG GT-C3222)*/
const Device UA2779400 = {S97413, 172, { &P16553,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SGH-T959V Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2779913 = {S97414, 138, { &P20597,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F100S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2780969 = {S97415, 168, { &P22756,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; C1604 Build/11.1.A.0.68) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2781312 = {S97416, 158, { &P19779,&P5629,&P22281,&P18092 }};	/*Useragent - OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A1+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2781736 = {S97417, 159, { &P18449,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST27i Build/6.1.1.B.1.10) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2785718 = {S97418, 35, { &P11929,&P5129,&P4722,&P18092 }};	/*Useragent - Nokia5230/UC Browser7.0.0.41/50/351*/
const Device UA2788739 = {S97419, 80, { &P21435,&P18110,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 (Linux; U; 2.3.6; zh-cn; ZTE V788D Build/UNI_CN_V788D 2.0;320*480)*/
const Device UA2790733 = {S97420, 133, { &P20300,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; W60 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2792013 = {S97421, 169, { &P21411,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Q-Smart S20 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2820429 = {S97422, 92, { &P21347,&P18110,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8/Adr (Linux; U; 2.3.6; zh-cn; HUAWEI U8661 Build/U8661V100R001C17B827;320*480)*/
const Device UA2837092 = {S97423, 128, { &P21613,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Aurora-II Build/X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2838679 = {S97424, 151, { &P21520,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cloudfone Thrill 430g Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2843155 = {S97425, 77, { &P15364,&P16110,&P22309,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:5.0) Gecko/X Firefox/5.0 SeaMonkey/2.2*/
const Device UA2845203 = {S97426, 147, { &P20548,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Livingstone 2 Build/1.1.7 X-10:33) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2846576 = {S97427, 161, { &P19996,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A15 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2847121 = {S97428, 138, { &P16292,&P18111,&P23481,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Galaxy Nexus Build/JZO54K) AppleWebKit/537.9 (KHTML, like Gecko) Chrome/23.0.1260.0 Mobile Safari/537.9*/
const Device UA2848014 = {S97429, 70, { &P16356,&P18111,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.1.2; zh-cn; GT-N7000; 800*1280) UCWEB7.9.0.94/139/444*/
const Device UA2848983 = {S97430, 142, { &P20493,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Flare (S100) Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2849209 = {S97431, 135, { &P20478,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; JY-G2 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2850416 = {S97432, 91, { &P13197,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAN93) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2851517 = {S97433, 136, { &P12784,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; es-es; Skate Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2851831 = {S97434, 100, { &P15809,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Rhyme_S510b) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2851982 = {S97435, 166, { &P18437,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-N8000 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2852891 = {S97436, 92, { &P15184,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; LG-P920) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2855236 = {S97437, 176, { &P21269,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Intex_Aqua_3.2 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2861616 = {S97438, 165, { &P23257,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI T8833 Build/HuaweiT8833) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2861618 = {S97439, 158, { &P13077,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; C8650 Build/HuaweiC8650) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2861676 = {S97440, 127, { &P23072,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; T-17B Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2861720 = {S97441, 144, { &P13157,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Sensation XE Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2864439 = {S97442, 142, { &P21733,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; i-mobile IQ 3 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2864896 = {S97443, 127, { &P20510,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CT920 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2865683 = {S97444, 157, { &P21648,&P18598,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9505 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2866655 = {S97445, 154, { &P20947,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTCEVOV4G Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2868499 = {S97446, 153, { &P19788,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; E350 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2869639 = {S97447, 158, { &P17172,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Transformer TF101 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2870264 = {S97448, 128, { &P18051,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; F-01D Build/V10R32D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2877780 = {S97449, 136, { &P18683,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ISW13F Build/V69R51I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879383 = {S97450, 163, { &P20816,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WIKO-CINK SLIM Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2879444 = {S97451, 139, { &P22624,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Burst S280 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2879521 = {S97452, 128, { &P17965,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-01D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2892095 = {S97453, 104, { &P13163,&P5129,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaN97*/
const Device UA2892546 = {S97454, 129, { &P22041,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P6810 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2892569 = {S97455, 150, { &P18385,&P18598,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.3.2.289*/
const Device UA2892678 = {S97456, 142, { &P20883,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892835 = {S97457, 137, { &P18701,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-T100K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892946 = {S97458, 139, { &P20731,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Blaze S180 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893639 = {S97459, 135, { &P17308,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROIDX Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2895250 = {S97460, 134, { &P23007,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TM400 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2896738 = {S97461, 170, { &P18460,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI U8950-1 Build/HuaweiU8950-1) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2897695 = {S97462, 158, { &P18474,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9210T Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2933122 = {S97463, 85, { &P13421,&P5448,&P4779,&P18092 }};	/*Useragent - HTC_TouchDual Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA2937371 = {S97464, 142, { &P21466,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Titan TV S320 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937500 = {S97465, 137, { &P16263,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Huawei MediaPad Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2937632 = {S97466, 156, { &P21022,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E615 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2944958 = {S97467, 136, { &P23776,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGP321 Build/10.3.1.A.0.236) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2953043 = {S97468, 159, { &P16356,&P18111,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2953061 = {S97469, 90, { &P13077,&P5629,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.3; zh-CN; C8650) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile*/
const Device UA2955581 = {S97470, 165, { &P23909,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MID7032 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444*/
const Device UA2957031 = {S97471, 166, { &P18652,&P5629,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Micromax A87 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.3.2.289*/
const Device UA2957382 = {S97472, 158, { &P11747,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.7; HTC Desire Build/GRI40) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2957904 = {S97473, 152, { &P21021,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328t Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2958087 = {S97474, 170, { &P14183,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; SAMSUNG-SGH-I997 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2977299 = {S97475, 65, { &P15364,&P7133,&P18107,&P18092 }};	/*Useragent - VPNGate/5.0 (Windows NT 6.1; WOW64; rv:22.0) Gecko/X Firefox/2637*/
const Device UA3041952 = {S97476, 153, { &P15364,&P7133,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.6) Gecko/X  Firefox[xSP_2:1e4a1b03d1b6cd8a174a826f76e009f4_259]  X  (.NET CLR 3.5.30729) FBSMTWB*/
const Device UA2978834 = {S97477, 74, { &P15364,&P17484,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-E3217B/1.0 NetFront/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2982900 = {S97478, 158, { &P12453,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Liberty Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2983581 = {S97479, 99, { &P13554,&P5890,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (NokiaN79-1/32.001/sw_platform=S60;sw_platform_version=3.2;java_build_version=1.0.4)*/
const Device UA2983680 = {S97480, 132, { &P15364,&P16110,&P22447,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Iron/9.0.600.2 Chrome/9.0.600.2 Safari/534.13*/
const Device UA2983980 = {S97481, 69, { &P19898,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; SCH-I699; 480*800) UCWEB7.9.0.94/139/800*/
const Device UA2985280 = {S97482, 158, { &P21267,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A798t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985385 = {S97483, 153, { &P20894,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MOVI M3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2987065 = {S97484, 128, { &P23968,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; S930 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2987232 = {S97485, 99, { &P13861,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; es-LA; samsung-gt-m3710) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2987844 = {S97486, 141, { &P15364,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; G1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2987874 = {S97487, 99, { &P18460,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; HUAWEI_U8950-1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2988859 = {S97488, 136, { &P20921,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; D2-711 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2989041 = {S97489, 87, { &P17968,&P18598,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.2.2; Linux; Opera Mobi/ADR-X; U; en) Presto/2.9.201 Version/11.50*/
const Device UA2990952 = {S97490, 93, { &P15060,&P5096,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; zh-CN; GT-S5660) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile*/
const Device UA2991244 = {S97491, 90, { &P18252,&P5629,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; zh-CN; XT615) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile*/
const Device UA2995337 = {S97492, 141, { &P24546,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GOCLEVER TAB R104 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2997033 = {S97493, 139, { &P22591,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-I717D Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2997266 = {S97494, 155, { &P23414,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Aqua Sx Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2997648 = {S97495, 130, { &P14905,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7100 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2997795 = {S97496, 160, { &P20471,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; V1 Build/HCL+ME+Tablet+V1) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2997880 = {S97497, 156, { &P22875,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9220 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2997889 = {S97498, 159, { &P20004,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A2+ Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2998042 = {S97499, 80, { &P17595,&P17297,&P22170,&P18090 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:22.0) Gecko/X Firefox/22.0 slurp*/
const Device UA2998373 = {S97500, 90, { &P18449,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; ST27i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2998495 = {S97501, 147, { &P18739,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; R801 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2998806 = {S97502, 134, { &P21992,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.6; A9550 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2999044 = {S97503, 155, { &P22651,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-E435 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2999073 = {S97504, 153, { &P12968,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; LT26i Build/6.2.B.0.200) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2999989 = {S97505, 94, { &P23327,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; iris_401e) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3000157 = {S97506, 149, { &P18385,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3003905 = {S97507, 100, { &P14060,&P17485,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw150i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3006028 = {S97508, 173, { &P22906,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Lenovo A390_ROW Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/352*/
const Device UA3009308 = {S97509, 160, { &P23200,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Coolpad 5890 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3010666 = {S97510, 155, { &P19917,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LG-P765 Build/IMM76L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3012044 = {S97511, 156, { &P23003,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; TECNO Q1 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3012737 = {S97512, 165, { &P22596,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SCH-R970 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/jflteusc*/
const Device UA3013220 = {S97513, 101, { &P17968,&P5477,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; Android2.x) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3014148 = {S97514, 93, { &P12061,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae65-1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3113022 = {S97515, 92, { &P12061,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAE65) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3014150 = {S97516, 97, { &P23008,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax A91) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3015353 = {S97517, 155, { &P15802,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; HTC Ruby Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3016279 = {S97518, 90, { &P18582,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT28h) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3020050 = {S97519, 163, { &P20980,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; A97i Build/CelkonA97i_Celkon) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3021430 = {S97520, 140, { &P22003,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB A93.2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021555 = {S97521, 137, { &P19853,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-N7102 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022033 = {S97522, 134, { &P21256,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Wilco Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022152 = {S97523, 136, { &P21355,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; KM-E100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022322 = {S97524, 135, { &P20137,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Aiki-5 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022573 = {S97525, 134, { &P20255,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; R8111 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023376 = {S97526, 130, { &P20523,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023461 = {S97527, 92, { &P13602,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233w) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3107884 = {S97528, 89, { &P13602,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; vi; gt-s5233w) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3023521 = {S97529, 137, { &P18002,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A760S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3027002 = {S97530, 138, { &P24511,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E270L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3027719 = {S97531, 132, { &P23477,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; XT926 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3029086 = {S97532, 136, { &P22584,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SGH-I337M Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3029613 = {S97533, 165, { &P17283,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; MB860 Build/4.5.1A-129_OLY-145) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3029857 = {S97534, 137, { &P12482,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T959 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3040775 = {S97535, 73, { &P15364,&P7480,&P18245,&P18093 }};	/*Useragent - facebookexternalhit/1.0 ( http://www.facebook.com/externalhit_uatext.php)*/
const Device UA3041104 = {S97536, 74, { &P18088,&P17017,&P24505,&P18092 }};	/*Useragent - Opera/8.01 (J2ME/MIDP; Opera Mini/1.0.1520/HiFi; mobile phone; no; U; ssr)*/
const Device UA3042727 = {S97537, 116, { &P14532,&P17483,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; lg-t325) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA3045498 = {S97538, 94, { &P20206,&P5629,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; zh-CN; ZTE_N880E) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile*/
const Device UA3048046 = {S97539, 133, { &P23183,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T1-D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3048891 = {S97540, 140, { &P18303,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MT27i Build/6.2.A.0.400) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3050107 = {S97541, 152, { &P22663,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S11t Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3054889 = {S97542, 156, { &P23295,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; ZTE U807 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056562 = {S97543, 143, { &P23699,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; X720 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3056620 = {S97544, 139, { &P16553,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-T959V Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3056962 = {S97545, 138, { &P24611,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E989 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3057925 = {S97546, 132, { &P18088,&P5690,&P17470,&P18089 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/10.53 (Windows NT 5.1; U; pl; compatible; Googlebot/20.2497; U; id) Presto/2.5.25;c2VvLWNyYXdsZXI=*/
const Device UA3058116 = {S97547, 175, { &P17324,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I9001 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/ariesve*/
const Device UA3058668 = {S97548, 95, { &P15364,&P5096,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; Karbonn_A1) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3060345 = {S97549, 156, { &P14123,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P970 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3062363 = {S97550, 130, { &P20523,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Safari/537.36*/
const Device UA3069841 = {S97551, 64, { &P15364,&P17188,&P18347,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 4.0; U; ru) Presto/2.10.229 Version/11.64*/
const Device UA3073364 = {S97552, 82, { &P21002,&P17484,&P17374,&P18092 }};	/*Useragent - SAMSUNG-GT-C3312R Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10*/
const Device UA3074011 = {S97553, 158, { &P14183,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA3074043 = {S97554, 146, { &P19911,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-E970 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3074089 = {S97555, 150, { &P18507,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA3074169 = {S97556, 155, { &P20196,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; MID7048 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3077621 = {S97557, 138, { &P16356,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-gb; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3077966 = {S97558, 135, { &P20267,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; P300(Funbook) Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3078347 = {S97559, 141, { &P18472,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-79) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3078581 = {S97560, 176, { &P18506,&P5043,&P24055,&P18092 }};	/*Useragent - Nokia305/2.0 (05.92) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311*/
const Device UA3078773 = {S97561, 155, { &P22853,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; HTC 802w Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3079343 = {S97562, 137, { &P21645,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; TANK 4.5 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3079839 = {S97563, 152, { &P18714,&P5629,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V889D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3080949 = {S97564, 156, { &P12972,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; SGH-T989 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127402 = {S97565, 159, { &P18449,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ST27i Build/6.2.A.1.100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3087300 = {S97566, 131, { &P15767,&P19826,&P20775,&P18092 }};	/*Useragent - MQQBrowser/2.4 Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3*/
const Device UA3089523 = {S97567, 146, { &P20729,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-us; Micromax A27 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3091863 = {S97568, 90, { &P18223,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT22i) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3091964 = {S97569, 143, { &P20545,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; bq Maxwell 2 Build/1.0.0_X-16:07) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3091993 = {S97570, 152, { &P18665,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; N861 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3092347 = {S97571, 77, { &P18472,&P17979,&P22167,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Mobile: DROID RAZR HD; rv:20.0) Gecko/20.0 Firefox/20.0*/
const Device UA3092829 = {S97572, 138, { &P20137,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; he-il; Aiki-4S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3093112 = {S97573, 166, { &P12731,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3093337 = {S97574, 99, { &P14090,&P5477,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT-I5801) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3094374 = {S97575, 130, { &P16044,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Touchpad Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3104699 = {S97576, 133, { &P19777,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-I9105 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3107691 = {S97577, 94, { &P23110,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua_Glow) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3107979 = {S97578, 100, { &P14916,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-c3303i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3108315 = {S97579, 178, { &P15501,&P5477,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E16i Build/2.1.1.A.0.53) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3108323 = {S97580, 137, { &P24608,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E467f Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3108382 = {S97581, 146, { &P21623,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; th-th; i-mobile IQ 6.1 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108389 = {S97582, 99, { &P16971,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-sgh-a877) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3108623 = {S97583, 149, { &P24367,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P5200 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3108889 = {S97584, 170, { &P22941,&P18111,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6812 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3108943 = {S97585, 94, { &P14367,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6710s) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3109110 = {S97586, 132, { &P15877,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; XT910 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109220 = {S97587, 96, { &P20518,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Karbonn_A10) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3109314 = {S97588, 155, { &P20290,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A11 Build/KarbonnA11) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109336 = {S97589, 140, { &P22676,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Celkon A119 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3109621 = {S97590, 129, { &P23767,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-7240 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3110067 = {S97591, 159, { &P18303,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT27i Build/6.1.1.B.1.54) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3110435 = {S97592, 158, { &P18303,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; MT27i Build/6.0.B.3.184) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3110874 = {S97593, 128, { &P17337,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-P7500 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3110968 = {S97594, 159, { &P22632,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Titanium S1 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3111015 = {S97595, 136, { &P23986,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SBM206SH Build/S0008) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111181 = {S97596, 156, { &P18679,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-B5330 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3111303 = {S97597, 159, { &P17625,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST25i Build/6.1.1.B.1.10) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3113025 = {S97598, 130, { &P12889,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3113241 = {S97599, 153, { &P13452,&P5096,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-US; GT-P1000 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3113452 = {S97600, 145, { &P19891,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I717 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3114235 = {S97601, 93, { &P13974,&P5096,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5670) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3115174 = {S97602, 136, { &P20711,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID1065 Build/ICS.MID1065.X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3125271 = {S97603, 96, { &P23116,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Aqua.Active) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3126203 = {S97604, 92, { &P12671,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia700) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3126309 = {S97605, 137, { &P24582,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE T790 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126359 = {S97606, 136, { &P14574,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Nexus S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126469 = {S97607, 137, { &P17324,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9001 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126558 = {S97608, 137, { &P19847,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-S6012 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126567 = {S97609, 153, { &P23717,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; O+ 8.6 Android Build/Oplus8.6_Android) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127183 = {S97610, 152, { &P18644,&P5043,&P24680,&P18092 }};	/*Useragent - Nokia112/2.0 (03.32) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia112) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3127194 = {S97611, 156, { &P22603,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-I739 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127392 = {S97612, 160, { &P23008,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax A91 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127468 = {S97613, 141, { &P15794,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Sensation XL Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127521 = {S97614, 135, { &P20137,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Aiki-5 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3128233 = {S97615, 138, { &P17346,&P22882,&P22224,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/26.0.1410.53 Mobile/11A465 Safari/8536.25*/
const Device UA3130448 = {S97616, 156, { &P15228,&P5043,&P24680,&P18092 }};	/*Useragent - NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; NokiaC2-03) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3131393 = {S97617, 137, { &P24063,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; IdeaTab S6000-F Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3131397 = {S97618, 136, { &P22613,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E430 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132638 = {S97619, 130, { &P20035,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I925 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3132756 = {S97620, 147, { &P15877,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132885 = {S97621, 154, { &P22856,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HUAWEI Y300-0151 Build/HuaweiY300-0151) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132918 = {S97622, 129, { &P22943,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; E270BSA Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA8570 = {S97623, 67, { &P14199,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-X497/X497UCEG3 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA11381 = {S97624, 138, { &P13600,&P17486,&P3581,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible;MSIE 6.0;Linux MOTOROKR Z6W-orange) MOT-Z6w/R6635_G_81.03.2BR Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony 1.0*/
const Device UA11380 = {S97625, 138, { &P13600,&P17486,&P3581,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible;MSIE 6.0;Linux MOTOROKR Z6W-orange) MOT-Z6w/R6635_G_81.01.61R Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony 1.0*/
const Device UA26463 = {S97626, 140, { &P13600,&P17486,&P3581,&P18092 }};	/*Useragent - MOT-MOTOROKR-Z6W/Mozilla/5.0 (compatible;MSIE 6.0;Linux MOTOROKR Z6W)/R6635_G_81.xx.yyI Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony 1.0*/
const Device UA88008 = {S97627, 134, { &P13600,&P17486,&P3581,&P18092 }};	/*Useragent - MOT-Z6w/R6635_G_81.01.64R Mozilla/5.0 (compatible; MSIE 6.0; Linux; MOTOROKR Z6W) Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony/1.0*/
const Device UA15288 = {S97628, 16, { &P13532,&P17017,&P17470,&P18092 }};	/*Useragent - CORAL 400/WAP2.0*/
const Device UA56316 = {S97629, 103, { &P11904,&P17486,&P17671,&P18092 }};	/*Useragent - MOT-ZN200/1.0 Release/09.01.2008 Browser/CMCS1.0 Software/0.34D Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21213 = {S97630, 21, { &P15169,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP135 Obigo/WAP2.0*/
const Device UA21270 = {S97631, 116, { &P15173,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-C3060/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA22177 = {S97632, 65, { &P15393,&P17483,&P3534,&P18092 }};	/*Useragent - LG-KF390 Teleca/Q05A-3.12 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22407 = {S97633, 68, { &P15457,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHU300/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA55000 = {S97634, 102, { &P12052,&P17486,&P17671,&P18092 }};	/*Useragent - MOT-W360/1.0 Release/03.06.2008 Browser/CMCS1.0 Software/1.180 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA50269 = {S97635, 61, { &P12026,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonC510/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA253863 = {S97636, 62, { &P12026,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonC510a/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA23656 = {S97637, 12, { &P15616,&P17017,&P17470,&P18092 }};	/*Useragent - Sonim XP3300*/
const Device UA24516 = {S97638, 70, { &P11980,&P5448,&P4578,&P18092 }};	/*Useragent - HTCP3300-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)*/
const Device UA35933 = {S97639, 71, { &P11980,&P5448,&P4578,&P18092 }};	/*Useragent - HTC_P3300-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)*/
const Device UA71524 = {S97640, 122, { &P11980,&P5448,&P4578,&P18092 }};	/*Useragent - HTC P3300/1.35.621.1 Mozilla/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)*/
const Device UA24623 = {S97641, 86, { &P15724,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-W750/MTK Release/01.01.2010 Browser/MAUI Profile/MIDP2.0 Configuration/CLCD-1*/
const Device UA26639 = {S97642, 70, { &P11870,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_S620 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA27521 = {S97643, 34, { &P12177,&P6476,&P17470,&P18092 }};	/*Useragent - LG9700/1.1 TELECA/Q7.0-1.3 MMP/2.0*/
const Device UA49461 = {S97644, 34, { &P12177,&P6476,&P17470,&P18092 }};	/*Useragent - LG9700/1.0 TELECA/Q7.0-1.3 MMP/2.0*/
const Device UA33081 = {S97645, 60, { &P13558,&P6208,&P4420,&P18092 }};	/*Useragent - BlackBerry7290/4.0.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49853 = {S97646, 152, { &P13094,&P5629,&P20771,&P18092 }};	/*Useragent - MQQBrowser/20 Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Lenovo A60 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA50312 = {S97647, 74, { &P13226,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-E2120/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA53624 = {S97648, 24, { &P13995,&P19954,&P3816,&P18092 }};	/*Useragent - SEC-SGHE720C/1.0 TSS/2.5*/
const Device UA53945 = {S97649, 23, { &P13995,&P19954,&P3816,&P18092 }};	/*Useragent - SEC-SGHE720/1.0 TSS/2.5*/
const Device UA54446 = {S97650, 71, { &P14213,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-T165i/1.0 ACS-NF/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54589 = {S97651, 115, { &P14240,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-M7600B/1.0 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA59515 = {S97652, 71, { &P16145,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; Xda Exec; 640x480)*/
const Device UA87361 = {S97653, 91, { &P16145,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC;JASJAR;640x480; PPC; Xda Exec; 640x480)*/
const Device UA61308 = {S97654, 66, { &P12680,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2692_CMCC/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63223 = {S97655, 117, { &P14971,&P17605,&P3976,&P18092 }};	/*Useragent - SAMSUNG-GT-B5722C/1.0 RTK-E/1.0 DF/1.0 Release/10.14.2009 Browser/NetFront3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63423 = {S97656, 62, { &P17707,&P17017,&P3807,&P18092 }};	/*Useragent - SFR115/g9qA/EFS-SFR-R9SM-P109A17V1.0.4/R221 Obigo/Q03C WAP/2.0*/
const Device UA63681 = {S97657, 87, { &P15505,&P5448,&P4578,&P18092 }};	/*Useragent - Dopod818Pro/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; Dopod818 Pro)*/
const Device UA87415 = {S97658, 90, { &P15505,&P5448,&P4578,&P18092 }};	/*Useragent - Dopod 818 Pro Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; Dopod 818 Pro)*/
const Device UA63729 = {S97659, 39, { &P13583,&P17486,&P3807,&P18092 }};	/*Useragent - Mot-WX280/1.0 ObigoInternetBrowser/Q03C*/
const Device UA63881 = {S97660, 42, { &P12529,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-F309/ UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA67298 = {S97661, 36, { &P12449,&P5890,&P4715,&P18092 }};	/*Useragent - NOKIAE5-00/UC Browser7.4.0.65/28/351*/
const Device UA85783 = {S97662, 77, { &P15364,&P16000,&P4977,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.0; SmartPhone; Symbian OS/1.1.0) NetFront/3.1*/
const Device UA89650 = {S97663, 47, { &P17535,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC3S UP.Browser/6.2.0.15.1.1 (GUI) MMP/2.0*/
const Device UA137992 = {S97664, 34, { &P17898,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/B2010/Screen-128X128*/
const Device UA368179 = {S97665, 123, { &P15364,&P5690,&P22419,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.0.3.0 Chrome/17.0.963.38 Safari/535.11*/
const Device UA2150973 = {S97666, 47, { &P14764,&P17485,&P24665,&P18092 }};	/*Useragent - SonyEricssonU1/UC Browser8.0.4.121/50/444/UCWEB*/
const Device UA2216570 = {S97667, 91, { &P14888,&P17361,&P4779,&P18092 }};	/*Useragent - BlackBerry9650/4.4.0.560 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/159/352*/
const Device UA440423 = {S97668, 133, { &P17160,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Slider SL101 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA452275 = {S97669, 112, { &P11787,&P5204,&P17338,&P18092 }};	/*Useragent - Nokia3120classic/2.0 (07.16) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia3120classic/UC Browser7.9.0.102/69/352*/
const Device UA454045 = {S97670, 132, { &P15364,&P16110,&P22426,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.140 Safari/535.19 Comodo_Dragon/18.0.3.0*/
const Device UA454215 = {S97671, 125, { &P15364,&P16110,&P22426,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.140 Safari/535.19 Comodo_Dragon/18.0.3.0*/
const Device UA541027 = {S97672, 36, { &P12588,&P5055,&P4759,&P18092 }};	/*Useragent - NokiaC6-01/UC Browser7.6.1.82/50/351*/
const Device UA2757772 = {S97673, 46, { &P12588,&P5055,&P4759,&P18092 }};	/*Useragent - nokiac6-01/UC Browser7.6.1.82/69/351  3gpp-gba*/
const Device UA568554 = {S97674, 44, { &P17595,&P17296,&P21581,&P18092 }};	/*Useragent - MacOutlook/14.13.0.X (Intel Mac OS X 10.6.8)*/
const Device UA572408 = {S97675, 81, { &P12097,&P5448,&P4779,&P18092 }};	/*Useragent - HTC_P4350 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA1933978 = {S97676, 104, { &P12125,&P5204,&P17347,&P18092 }};	/*Useragent - Nokia6500s-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6500s-1/UC Browser8.0.3.107/69/352*/
const Device UA682900 = {S97677, 173, { &P13157,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA1975522 = {S97678, 167, { &P13157,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation XE with Beats Audio Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75 Mobile Safari/535.7*/
const Device UA589882 = {S97679, 48, { &P17595,&P17779,&P20781,&P18092 }};	/*Useragent - MQQBrowser/30 (iOS; U; CPU like Mac OS X; zh-cn)*/
const Device UA729112 = {S97680, 43, { &P13677,&P17484,&P17338,&P18092 }};	/*Useragent - samsung-gt-s5253/UC Browser7.9.0.102/69/444*/
const Device UA822452 = {S97681, 69, { &P17490,&P18378,&P18405,&P18092 }};	/*Useragent - Opera/9.23 (Nintendo Wii; U; ; 1038-58; Wii Internet Channel/1.0; en)*/
const Device UA855358 = {S97682, 129, { &P21990,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; tr-tr; chagall Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA869564 = {S97683, 125, { &P15364,&P17285,&P24198,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.15 Safari/535.11 QQBrowser/6.11.12388.201*/
const Device UA2981095 = {S97684, 280, { &P15364,&P17285,&P24198,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/4.0; QQDownload 677; qdesk 2.3.1186.202; EmbeddedWB 14.52 from: http://www.bsalsa.com/ EmbeddedWB 14.52; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C; Alexa Toolbar) QQBrowser/6.11.12388.201*/
const Device UA880356 = {S97685, 64, { &P15364,&P7127,&P18193,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:7.0.1) Gecko/X Thunderbird/7.0.1*/
const Device UA900297 = {S97686, 141, { &P16757,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-I500 Build/MIUI ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1896904 = {S97687, 66, { &P12710,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.3;Zh_cn;HTC A510c;320*480;)UCWEB7.6.0.75/139/32125*/
const Device UA1930326 = {S97688, 130, { &P21979,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-us; tegav2 Build/HLK75D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2020705 = {S97689, 128, { &P21979,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; hd-us; tegav2 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1942213 = {S97690, 60, { &P15364,&P23114,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:9.0.1) Gecko/X Firefox/9.0.1*/
const Device UA1980201 = {S97691, 151, { &P18253,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT894 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1983687 = {S97692, 114, { &P15364,&P5690,&P18528,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 Sleipnir/3.5.1*/
const Device UA1990545 = {S97693, 139, { &P16036,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Nexus One Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2018767 = {S97694, 74, { &P17968,&P17979,&P24188,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Linux armv7l; rv:2.0) Gecko/X Firefox/4.0 Fennec/4.0*/
const Device UA2020142 = {S97695, 131, { &P23829,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; in-id; G2 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027264 = {S97696, 159, { &P15047,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SK17i Build/4.0.2.A.0.42; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029738 = {S97697, 152, { &P18283,&P18066,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032313 = {S97698, 160, { &P19937,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; A73 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2032962 = {S97699, 139, { &P23987,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; CoolPad8013 Build/2.2.060.P1.X.8013) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2033716 = {S97700, 139, { &P23987,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; CoolPad8013 Build/2.2.062.P1.X.8013) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2152364 = {S97701, 102, { &P15364,&P18144,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA2161886 = {S97702, 101, { &P15364,&P18144,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD i386) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.77 Safari/535.7*/
const Device UA2241281 = {S97703, 165, { &P18385,&P18111,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; Nexus 7 Build/JRO03D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2241768 = {S97704, 165, { &P18385,&P18111,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; Nexus 7 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2259965 = {S97705, 75, { &P15364,&P17188,&P20901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; WinNT4.0; en-US; rv:1.4) Gecko/X Netscape/7.1 (ax)*/
const Device UA2260032 = {S97706, 80, { &P15364,&P7133,&P20903,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.7.5) Gecko/X Netscape/8.0.1*/
const Device UA2292693 = {S97707, 36, { &P12456,&P5890,&P4742,&P18092 }};	/*Useragent - nokian73-1/UC Browser7.5.1.77/69/352*/
const Device UA2300222 = {S97708, 47, { &P11907,&P5890,&P17338,&P18092 }};	/*Useragent - nokiae52-1/UC Browser7.9.0.102/69/352  3gpp-gba*/
const Device UA2321430 = {S97709, 75, { &P21396,&P18110,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 (Linux; U; 2.3.5; zh-cn; Lenovo A65 Build/A65_S218_X;320*480)*/
const Device UA2327902 = {S97710, 83, { &P17595,&P18120,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; de-AT; rv:1.9.1.8) Gecko/X Firefox/3.6.6*/
const Device UA2332936 = {S97711, 62, { &P16366,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux; U;2.3.4;zh_cn;S8600;480*800;)UCWEB7.6.0.75/139/7713*/
const Device UA2338124 = {S97712, 155, { &P14433,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2340400 = {S97713, 54, { &P14033,&P5890,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1 (NokiaC5-03;SymbianOS/9.1 Series60/3.0)*/
const Device UA2351990 = {S97714, 184, { &P18113,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2352153 = {S97715, 180, { &P20981,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax P250(Funbook) Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2360145 = {S97716, 65, { &P18482,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.4;Zh_cn;A75;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA2409371 = {S97717, 43, { &P11939,&P17485,&P17338,&P18092 }};	/*Useragent - sonyericssons312/UC Browser7.9.0.102/69/352*/
const Device UA2409786 = {S97718, 34, { &P12814,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SAMSUNG-SGH-I777/100.20306*/
const Device UA2411370 = {S97719, 53, { &P14984,&P5890,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8 (Nokia5250;SymbianOS/9.1 Series60/3.0)*/
const Device UA2413198 = {S97720, 139, { &P21227,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Polaroid Tablet Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2421676 = {S97721, 146, { &P20894,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; in-id; M3 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2422264 = {S97722, 65, { &P12280,&P17146,&P22071,&P18092 }};	/*Useragent - Instagram 3.1.2 (iPhone; iPhone OS 5.0.1; en_US) AppleWebKit/420+*/
const Device UA2454625 = {S97723, 197, { &P13157,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2461171 = {S97724, 98, { &P14980,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-b7722) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2466417 = {S97725, 186, { &P12125,&P5204,&P24672,&P18092 }};	/*Useragent - Nokia6500s-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia6500s-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2472697 = {S97726, 126, { &P23644,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL55HX729; CC/USA; en) Presto/2.7.61 Version/11.00*/
const Device UA2474032 = {S97727, 43, { &P16384,&P17484,&P4777,&P18092 }};	/*Useragent - samsung-gt-s5380k/UC Browser7.7.1.88/69/444*/
const Device UA2481521 = {S97728, 53, { &P18088,&P18432,&P24668,&P18092 }};	/*Useragent - IUC(U;iOS 6.0.1;Zh-cn;320*480;)/UCWEB8.3.1.151/42/997*/
const Device UA2483203 = {S97729, 62, { &P15364,&P17285,&P19821,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:18.0) Gecko/X Thunderbird/18.0*/
const Device UA2484312 = {S97730, 165, { &P12568,&P5096,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; GT-B7510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2485346 = {S97731, 132, { &P17595,&P17297,&P19963,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/6.0.2 Safari/7536.26.17 Sleipnir/3.9*/
const Device UA2485536 = {S97732, 167, { &P18661,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; N860 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2487294 = {S97733, 155, { &P18466,&P18066,&P19817,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; KFTT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.6 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2502343 = {S97734, 94, { &P15364,&P7480,&P17207,&P18092 }};	/*Useragent - Mozilla-Firefox5.0 AppEngine-Google; (+http://code.google.com/appengine; appid: s~zhoulixia86)*/
const Device UA2502591 = {S97735, 33, { &P12897,&P5843,&P21724,&P18092 }};	/*Useragent - OneBrowser/3.0 (NokiaC2-00/03.45)*/
const Device UA2520466 = {S97736, 85, { &P18714,&P18110,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8/Adr (Linux; U; 2.3.6; zh-cn; ZTE V889D Build/UNI_CN_V889D 2.2;480*800)*/
const Device UA2524652 = {S97737, 81, { &P17595,&P18207,&P22542,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; en) Presto/2.9.168 Version/11.52*/
const Device UA2529431 = {S97738, 142, { &P21873,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB M713G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2817178 = {S97739, 140, { &P15837,&P6909,&P24670,&P18092 }};	/*Useragent - NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; kau; nokiax2-05) UCBrowser8.4.0.159/70/352/UCWEB Mobile*/
const Device UA2552777 = {S97740, 155, { &P12798,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2555289 = {S97741, 171, { &P20332,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax P275 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2574021 = {S97742, 26, { &P18300,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-T769/100.20306*/
const Device UA2601196 = {S97743, 134, { &P20427,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHL21 Build/S2140) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2609636 = {S97744, 136, { &P17337,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; GT-P7500 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2613898 = {S97745, 136, { &P17337,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2624183 = {S97746, 139, { &P17337,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/R13.4-rc2) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2633302 = {S97747, 137, { &P17337,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500R Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2613122 = {S97748, 83, { &P13452,&P17979,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.4.5) (GT-P1000N;samsung;GT-P1000N;2.2.1/FROYO)*/
const Device UA2616999 = {S97749, 82, { &P13452,&P17979,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.4.6) (GT-P1000;samsung;GT-P1000;4.1.1/JRO03R)*/
const Device UA2623442 = {S97750, 53, { &P13452,&P17979,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.9245 Android/2.2 Device/samsung-GT-P1000*/
const Device UA2621902 = {S97751, 164, { &P15882,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-ch; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P5110, 36, 1.7.1)*/
const Device UA2637894 = {S97752, 164, { &P15882,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P5110, 42, 1.9.1)*/
const Device UA2621951 = {S97753, 99, { &P15919,&P17979,&P22073,&P18092 }};	/*Useragent - Instagram 3.3.2 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P3100; espressorf; espresso; en_GB)*/
const Device UA2636081 = {S97754, 155, { &P13452,&P5096,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-CN; GT-P1000 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2636674 = {S97755, 153, { &P13452,&P5096,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-CN; GT-P1000 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2638422 = {S97756, 97, { &P15364,&P7133,&P22486,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.8.1.6) Gecko/X Firefox/2.0.0.6 Flock/0.9.0.2*/
const Device UA2655736 = {S97757, 132, { &P20873,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Consul 7 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2659814 = {S97758, 160, { &P20158,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2660874 = {S97759, 99, { &P11939,&P17485,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssons312) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2661022 = {S97760, 161, { &P21057,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A7+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2686111 = {S97761, 89, { &P13552,&P5129,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokiac6-00) UCBrowser8.3.1.161/70/465/UCWEB Mobile  3gpp-gba*/
const Device UA2693424 = {S97762, 137, { &P21836,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TM-4504 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2694105 = {S97763, 128, { &P20528,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; R97B Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2695168 = {S97764, 53, { &P15364,&P6704,&P21951,&P18092 }};	/*Useragent - Opera/9.80 (Windows 98) Presto/2.12.388 Version/12.10*/
const Device UA2695394 = {S97765, 154, { &P15575,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695748 = {S97766, 154, { &P16356,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-N7000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2695967 = {S97767, 153, { &P20206,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE-N880E Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2700140 = {S97768, 159, { &P12513,&P17485,&P24675,&P18092 }};	/*Useragent - SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonw20) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2701382 = {S97769, 151, { &P21026,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; ZTE U930 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2703174 = {S97770, 172, { &P14090,&P5477,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; GT-I5801 Build/ECLAIR) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2703183 = {S97771, 133, { &P20140,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GEM10312BK Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2709155 = {S97772, 142, { &P21617,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 1 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2710152 = {S97773, 152, { &P15767,&P18117,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 5_1_1 like Mac OS X; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile*/
const Device UA2710433 = {S97774, 199, { &P15575,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, +0100, gb*/
const Device UA2722104 = {S97775, 100, { &P15837,&P6909,&P17347,&P18092 }};	/*Useragent - NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-05/UC Browser8.0.3.107/69/444*/
const Device UA2722463 = {S97776, 165, { &P17346,&P17145,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/7534.48.3 TeamStream (iPod2,1 232 1.6.2)*/
const Device UA2729678 = {S97777, 133, { &P23171,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; G2S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2730021 = {S97778, 154, { &P12798,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2732922 = {S97779, 140, { &P20731,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Blaze S180 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2732925 = {S97780, 138, { &P11747,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire Build/CM10) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2734643 = {S97781, 137, { &P19865,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A850K Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734783 = {S97782, 130, { &P21224,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N-07D Build/AX) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734852 = {S97783, 134, { &P20098,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GN800 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734872 = {S97784, 133, { &P24542,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARNOVA 90G3 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2735639 = {S97785, 138, { &P21320,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT25c Build/9.0.E.1.80) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735860 = {S97786, 134, { &P23946,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; I7100 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735903 = {S97787, 134, { &P24549,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; E2001 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2736102 = {S97788, 135, { &P20437,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; FJL21 Build/V35R45B) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2746948 = {S97789, 171, { &P21988,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us ; PD_Novel Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2748068 = {S97790, 132, { &P22818,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M1 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2748244 = {S97791, 159, { &P13042,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Incredible S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2750004 = {S97792, 134, { &P22940,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Xelio 7 pro Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2750826 = {S97793, 134, { &P23478,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZP810 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2751143 = {S97794, 162, { &P15991,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-S6102 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Mobile Safari/537.31 OPR/14.0.1074.54070*/
const Device UA2761276 = {S97795, 157, { &P18507,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2762869 = {S97796, 98, { &P20123,&P17979,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/ZTEU795_TD/1.0 Linux/2.6.39 Android/4.0 Release/6.10.2012 Browser/AppleWebKit534.30*/
const Device UA2764359 = {S97797, 169, { &P15517,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; MT15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2766513 = {S97798, 155, { &P14186,&P5096,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-CN; GT-S5830 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2767378 = {S97799, 158, { &P18673,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S7562 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2767479 = {S97800, 166, { &P13064,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-I727 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2769429 = {S97801, 130, { &P24100,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CAL21 Build/AX) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2775561 = {S97802, 73, { &P20295,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; HTC_S610d; 480*800) UCWEB7.9.4.145/139/32449*/
const Device UA2780019 = {S97803, 134, { &P18314,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M032 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2781400 = {S97804, 135, { &P24585,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ZP800H Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2786219 = {S97805, 139, { &P12173,&P5448,&P4779,&P18092 }};	/*Useragent - LG-CT810/V10x IEMobile/7.11 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA2789578 = {S97806, 134, { &P15364,&P16110,&P22492,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.3.4641 Chrome/7.0.517.450 Safari/534.7*/
const Device UA2791721 = {S97807, 152, { &P20206,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE N880E Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2792419 = {S97808, 173, { &P20980,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; A97i Build/CelkonA97i_Celkon) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2793067 = {S97809, 199, { &P20146,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.19/284,1.9.19.284,2013-01-12 05:21, +0530, in*/
const Device UA2793114 = {S97810, 95, { &P11747,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; HTC_Desire) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2793149 = {S97811, 163, { &P13240,&P17485,&P24676,&P18092 }};	/*Useragent - SonyEricssonW890i/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw890i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2794701 = {S97812, 124, { &P20822,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N-08D Build/AX) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2795067 = {S97813, 138, { &P17346,&P19826,&P22589,&P18092 }};	/*Useragent - MQQBrowser/421 Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3*/
const Device UA2803258 = {S97814, 92, { &P13894,&P5448,&P4777,&P18092 }};	/*Useragent - SAMSUNG-GT-B6520/1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2832498 = {S97815, 143, { &P22669,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GSmart G1362 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2833015 = {S97816, 142, { &P23393,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; eZee'Tab704 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2835458 = {S97817, 76, { &P23053,&P18110,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Adr (Linux; U; 4.0.4; zh-cn; GN700W Build/GN700W_T16;480*800)*/
const Device UA2837610 = {S97818, 64, { &P21057,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; A7+; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2838088 = {S97819, 150, { &P22362,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; tr-tr; SmartTab10 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2838136 = {S97820, 138, { &P21437,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F120S Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2847796 = {S97821, 160, { &P23412,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TM-9741 Build/V1.1.ICECREAMSANDWICH.rus.teXet.X.X) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2848350 = {S97822, 141, { &P21411,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Q-Smart S20 Build/IMM76I) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2850640 = {S97823, 159, { &P17519,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; T8830Pro Build/HuaweiT8830Pro) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2853378 = {S97824, 176, { &P12731,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2855674 = {S97825, 93, { &P19935,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A818_Duo) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2858158 = {S97826, 159, { &P15517,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-CN; MT15i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2860622 = {S97827, 138, { &P18701,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-T100K Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2861522 = {S97828, 133, { &P21752,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M030 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2861543 = {S97829, 186, { &P15794,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HTC Sensation XL with Beats Audio X315e Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2862223 = {S97830, 163, { &P15798,&P5629,&P22281,&P18092 }};	/*Useragent - OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2863209 = {S97831, 164, { &P20884,&P5629,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A298t Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2863703 = {S97832, 134, { &P21104,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; X507 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2865779 = {S97833, 154, { &P12798,&P5629,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2866582 = {S97834, 170, { &P18472,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2868628 = {S97835, 157, { &P14877,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9000 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2878807 = {S97836, 80, { &P17595,&P18207,&P22169,&P18090 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:21.0) Gecko/X Firefox/21.0 slurp*/
const Device UA2879205 = {S97837, 131, { &P22000,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TAB466EUK Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2879212 = {S97838, 162, { &P19843,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE BLADE III Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2879448 = {S97839, 156, { &P22037,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; GT-P3113 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2892884 = {S97840, 133, { &P23370,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Oysters T34 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2892999 = {S97841, 152, { &P23151,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAP4044DUO Build/PrestigioPAP4044DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2934372 = {S97842, 128, { &P23232,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; F7HD Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2935149 = {S97843, 136, { &P23299,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; OV-BaseCore7 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2935188 = {S97844, 92, { &P14123,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; LG-P970) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2937033 = {S97845, 90, { &P18223,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; LT22i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2937239 = {S97846, 93, { &P22610,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6810) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2937485 = {S97847, 153, { &P23717,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; O+ 8.6 Android Build/Oplus8.6_Android) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937727 = {S97848, 136, { &P20878,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MIDC901 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2938044 = {S97849, 132, { &P21059,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A90 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2939202 = {S97850, 123, { &P21239,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; M9 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2939330 = {S97851, 142, { &P24112,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; i-mobile IQ X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2943064 = {S97852, 140, { &P24096,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Orange Kivo Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2944270 = {S97853, 137, { &P18595,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I8190 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2950874 = {S97854, 146, { &P14973,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-V909 Build/HMJ37; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2951184 = {S97855, 167, { &P12280,&P17090,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3 TeamStream (iPhone3,1 293 2.2.2)*/
const Device UA2952580 = {S97856, 146, { &P24577,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PIRANHA ULTRA II TAB 7 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2956314 = {S97857, 162, { &P15809,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HTC Rhyme S510b Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2957358 = {S97858, 92, { &P16218,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN91) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2957588 = {S97859, 159, { &P18223,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.7; LT22i Build/6.0.B.1.564) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2957774 = {S97860, 90, { &P12798,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; LT18i) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2957813 = {S97861, 137, { &P22991,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC 802t Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2958076 = {S97862, 94, { &P18295,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; HTC One X) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2958363 = {S97863, 142, { &P22627,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; IRIS 455 Build/LAVAIRIS455) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2959377 = {S97864, 143, { &P23546,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; New Andromax-i Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2959862 = {S97865, 159, { &P20738,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Andromax U Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2977059 = {S97866, 132, { &P19799,&P18110,&P19800,&P18092 }};	/*Useragent - Mozilla/5.0 (DTV) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.1.5 AQUOSBrowser/1.0 (AS00DTV;V;0001;0001)AQUOS-AS/2.0 LC-70LE950X*/
const Device UA2977068 = {S97867, 133, { &P19799,&P18110,&P19800,&P18092 }};	/*Useragent - Mozilla/5.0 (DTV) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.5 AQUOSBrowser/1.0 (AS00DTV;V;0001;0001) AQUOS-AS/2.0 LC-80LE840X*/
const Device UA3118720 = {S97868, 156, { &P19799,&P18110,&P19800,&P18092 }};	/*Useragent - Mozilla/5.0 (DTV) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.4 AQUOSBrowser/1.0 (JP00DTV;B;0001;0001) InettvBrowser/2.2 (08001F;DTV10VSFC3;0001;0001)*/
const Device UA2982002 = {S97869, 185, { &P12502,&P17485,&P24672,&P18092 }};	/*Useragent - SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; fa; sonyericssonw20i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile*/
const Device UA2982435 = {S97870, 126, { &P24551,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M2 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2982666 = {S97871, 33, { &P13392,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SAMSUNG GT-C3510)*/
const Device UA2983023 = {S97872, 171, { &P19862,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax A110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2985999 = {S97873, 178, { &P19748,&P5043,&P24672,&P18092 }};	/*Useragent - Nokia205/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia205) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2986923 = {S97874, 124, { &P15364,&P6590,&P17394,&P18092 }};	/*Useragent - NetFront/4.2 (BREW 5.0.1.2; U; en-us; SONIM; NetFront/4.2/AMB) Sprint XP3410 MMP/2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2988198 = {S97875, 154, { &P19996,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; MITO A15 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2989034 = {S97876, 169, { &P21141,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; Lenovo P770 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/352*/
const Device UA2990052 = {S97877, 187, { &P13064,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SAMSUNG-SGH-I727, 42, 1.9.1)*/
const Device UA2991993 = {S97878, 158, { &P13042,&P5629,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Incredible S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2993905 = {S97879, 92, { &P12089,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN80) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2993916 = {S97880, 170, { &P22609,&P18111,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6312 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2994057 = {S97881, 93, { &P18700,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-B5512) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2994253 = {S97882, 95, { &P22751,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Celkon*A86) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2994720 = {S97883, 92, { &P21987,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Gpad_G1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2995358 = {S97884, 170, { &P22607,&P18066,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-S5303 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2995771 = {S97885, 147, { &P21853,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Y3 Build/HCL) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2996108 = {S97886, 103, { &P13324,&P5055,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian8-00) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2996599 = {S97887, 154, { &P23156,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XOLO A700 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2996932 = {S97888, 157, { &P16263,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; de-de; Huawei MediaPad Build/JDQ39E; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2996974 = {S97889, 92, { &P21739,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; iris454) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2997406 = {S97890, 155, { &P21739,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; iris454 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2997491 = {S97891, 162, { &P14090,&P5477,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; GT-I5801 Build/ECLAIR) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2998127 = {S97892, 156, { &P22611,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-S7572 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2998698 = {S97893, 147, { &P19920,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-MS870 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2998736 = {S97894, 156, { &P14887,&P5096,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-CN; GT-I9008L Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2998795 = {S97895, 154, { &P12798,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2999039 = {S97896, 152, { &P20251,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328d Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2999231 = {S97897, 74, { &P17595,&P21721,&P22772,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:24.0) Gecko/X Firefox/24.0*/
const Device UA2999403 = {S97898, 160, { &P23257,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8833 Build/HuaweiT8833) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2999576 = {S97899, 164, { &P15555,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.4; myTouch_4G_Slide Build/GRJ22) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2999730 = {S97900, 143, { &P20760,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Event Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3000019 = {S97901, 129, { &P18699,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID8042 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3000334 = {S97902, 163, { &P15809,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Rhyme S510b Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3000839 = {S97903, 153, { &P17625,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3000871 = {S97904, 156, { &P16292,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9050 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3001252 = {S97905, 144, { &P21853,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Y3 Build/HCL) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA3001959 = {S97906, 150, { &P20280,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; vivo X1 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3004876 = {S97907, 158, { &P15079,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MK16i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3006111 = {S97908, 144, { &P20253,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; i-mobile IQ 5.3 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3008386 = {S97909, 90, { &P12616,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; WT19i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3009737 = {S97910, 155, { &P21022,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LG-E615 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3010059 = {S97911, 92, { &P18621,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; LG-E612) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3010246 = {S97912, 156, { &P14959,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-I9103 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3010418 = {S97913, 153, { &P23235,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; VT75C Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3011959 = {S97914, 157, { &P18452,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; LT29i Build/9.1.B.1.67) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3012589 = {S97915, 93, { &P20837,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; IRIS_501) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3013497 = {S97916, 156, { &P20515,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; A54 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3013687 = {S97917, 161, { &P21129,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; SCH-i929 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3014824 = {S97918, 156, { &P22611,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-S7572 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3016182 = {S97919, 154, { &P15985,&P6437,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-US; GT-P6200 Build/HTJ85B) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3016988 = {S97920, 161, { &P18700,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-B5512 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3018369 = {S97921, 152, { &P21301,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M031 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3018481 = {S97922, 163, { &P14183,&P5096,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3018853 = {S97923, 103, { &P15364,&P5477,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; Micromax A60) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3018993 = {S97924, 155, { &P20587,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-P715 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3020563 = {S97925, 138, { &P21320,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT25c Build/9.0.E.1.80) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3020801 = {S97926, 137, { &P18701,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-T100K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021972 = {S97927, 134, { &P15880,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; MB865 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022374 = {S97928, 179, { &P22626,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Iris_349 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3022467 = {S97929, 134, { &P23758,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i9430 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023857 = {S97930, 140, { &P24161,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Orange Zali Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3030062 = {S97931, 107, { &P15364,&P5690,&P21934,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.95 Safari/537.36 slurp*/
const Device UA3041226 = {S97932, 77, { &P15364,&P17188,&P20907,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; WinNT4.0; en-US; rv:0.9.4.1) Gecko/X Netscape6/6.2.2*/
const Device UA3044565 = {S97933, 131, { &P23093,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MIDC801 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3045293 = {S97934, 161, { &P20529,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A89 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA3046472 = {S97935, 53, { &P18088,&P19826,&P24667,&P18092 }};	/*Useragent - IUC(U;iOS 6.1.3;Zh-cn;320*480;)/UCWEB8.2.1.132/42/997*/
const Device UA3046606 = {S97936, 158, { &P16036,&P5096,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 XiaoMi/MiuiBrowser/1.0*/
const Device UA3046930 = {S97937, 148, { &P24550,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; GOCLEVER HYBRID Build/GOCLEVER HYBRID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3049192 = {S97938, 151, { &P24534,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Cloudfone Excite 500d Build/FSV08) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3049269 = {S97939, 161, { &P21129,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; SCH-i929 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3049546 = {S97940, 154, { &P19860,&P18066,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A9+ Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300*/
const Device UA3050117 = {S97941, 135, { &P21309,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; myPad P4 Lite Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3050210 = {S97942, 136, { &P21739,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; iris454 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3056655 = {S97943, 153, { &P24070,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-CN; R815T Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056740 = {S97944, 143, { &P22785,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire 600 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3057205 = {S97945, 178, { &P14669,&P5843,&P22881,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291*/
const Device UA3057454 = {S97946, 167, { &P14225,&P5629,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60015*/
const Device UA3057965 = {S97947, 53, { &P14188,&P5890,&P20787,&P18092 }};	/*Useragent - MQQBrowser/3.6 (Nokia5233;SymbianOS/9.1 Series60/3.0)*/
const Device UA3058327 = {S97948, 94, { &P18296,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_One_V) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3058544 = {S97949, 137, { &P14872,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-D700 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3058715 = {S97950, 97, { &P18462,&P18111,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; HTC Desire X) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3059006 = {S97951, 149, { &P18250,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Droid RAZR MAXX HD Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3061016 = {S97952, 151, { &P22020,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9268 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3061401 = {S97953, 145, { &P14111,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Arc Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3073875 = {S97954, 150, { &P23090,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; i9400 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA3074241 = {S97955, 151, { &P20399,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; PantechP8010 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3074678 = {S97956, 138, { &P12972,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3075157 = {S97957, 128, { &P20265,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-in; A52 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3076550 = {S97958, 133, { &P19856,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; LG-LS970 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3077941 = {S97959, 141, { &P24688,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Spice Mi-515 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3078835 = {S97960, 157, { &P20251,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328d Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3078841 = {S97961, 156, { &P22853,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HTC 802w Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3078885 = {S97962, 155, { &P22651,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-E435 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3078945 = {S97963, 156, { &P21935,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E980 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3079553 = {S97964, 152, { &P18314,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M032 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3087878 = {S97965, 137, { &P23251,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ZTE V807 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3088667 = {S97966, 148, { &P24640,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Orbit TAB Phone 9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3092207 = {S97967, 95, { &P21911,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.1; en-US; AMAZE_S200) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3092230 = {S97968, 164, { &P21020,&P18598,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-in; Micromax A116 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300*/
const Device UA3092316 = {S97969, 97, { &P11747,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC_Desire_V) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3092688 = {S97970, 157, { &P21068,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI-ONE C1 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3092786 = {S97971, 151, { &P24557,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9128 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3093020 = {S97972, 157, { &P18300,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T769 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3093140 = {S97973, 176, { &P18583,&P5043,&P24055,&P18092 }};	/*Useragent - Nokia110/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311*/
const Device UA3093306 = {S97974, 155, { &P18687,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-P705 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3093542 = {S97975, 159, { &P20004,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Karbonn A26 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3101472 = {S97976, 141, { &P22132,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; SOWA_S6600 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102279 = {S97977, 150, { &P22874,&P18598,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9502 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA3103151 = {S97978, 134, { &P21262,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TM500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3103201 = {S97979, 142, { &P18652,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-in; Micromax A87 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3105016 = {S97980, 138, { &P22737,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-in; Micromax A115 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3106273 = {S97981, 132, { &P20202,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-us; LG-L38C Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3106671 = {S97982, 157, { &P15759,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; HTC Vision Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3106963 = {S97983, 131, { &P21950,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; iDxD10 3G Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3107401 = {S97984, 91, { &P13452,&P5096,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2; en-US; GT-P1000) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3107903 = {S97985, 140, { &P24037,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALLVIEW_TX1_Quasar Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3108037 = {S97986, 170, { &P23477,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT926 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/xt926*/
const Device UA3108111 = {S97987, 96, { &P21881,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Karbonn_A4_) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3108431 = {S97988, 96, { &P13800,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA5630d-1) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3108492 = {S97989, 100, { &P13220,&P17485,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk770i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3108633 = {S97990, 97, { &P23017,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Spice_Mi-502) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3108837 = {S97991, 156, { &P14959,&P5629,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3108924 = {S97992, 177, { &P14202,&P5477,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-US; E15i Build/2.1.1.A.0.6) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3109129 = {S97993, 176, { &P19862,&P18598,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Micromax_A110Q Build/JOP40D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3109157 = {S97994, 96, { &P13979,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; Nokia6220c-1) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3109388 = {S97995, 163, { &P22998,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; TECNO_N7 Build/TECNOTECNO_N7) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109486 = {S97996, 167, { &P15201,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Sensation Z710e Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109968 = {S97997, 159, { &P19999,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A30 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3110710 = {S97998, 155, { &P22594,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SGH-M919 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3110878 = {S97999, 152, { &P21596,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; mobiistar touch LAI 504 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111688 = {S98000, 159, { &P16869,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; Micromax A76 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3112957 = {S98001, 89, { &P21472,&P18111,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; A111) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3114551 = {S98002, 128, { &P17965,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SC-01D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3115057 = {S98003, 130, { &P19877,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T869 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3115270 = {S98004, 157, { &P23156,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; XOLO A700 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3118983 = {S98005, 162, { &P13685,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; LS670 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3121438 = {S98006, 137, { &P23139,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Primo-N1 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3124919 = {S98007, 133, { &P21346,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; NovoCrystal Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3126033 = {S98008, 97, { &P20005,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Micromax A65) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3126045 = {S98009, 89, { &P20169,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; N320) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3126078 = {S98010, 89, { &P19775,&P18598,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; A210) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3126399 = {S98011, 134, { &P20098,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GN800 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126428 = {S98012, 152, { &P12765,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126476 = {S98013, 98, { &P21020,&P18598,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Micromax_A116) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3126493 = {S98014, 144, { &P22998,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TECNO_N7 Build/TECNOTECNO_N7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126522 = {S98015, 142, { &P23978,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; WALKMAN Build/icx1227_E111) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126524 = {S98016, 134, { &P19790,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; L-04E Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126537 = {S98017, 137, { &P12496,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I897 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3126957 = {S98018, 133, { &P21933,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M729 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127010 = {S98019, 129, { &P22476,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V70 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3127065 = {S98020, 134, { &P12109,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; B6000 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127255 = {S98021, 142, { &P19781,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ARCHOS 97 TITANIUMHD Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3127357 = {S98022, 171, { &P12605,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID BIONIC Build/9.8.2O-72_VZW-22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3127426 = {S98023, 162, { &P19862,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Micromax A110 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3127444 = {S98024, 156, { &P20226,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E975 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3127479 = {S98025, 102, { &P21628,&P18110,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-I8552) U2/1.0.0 UCBrowser/9.3.0.321 Mobile*/
const Device UA3127833 = {S98026, 141, { &P12280,&P18117,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X; zh-CN) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 UCBrowser/9.3.0.326 Mobile*/
const Device UA3128113 = {S98027, 150, { &P15985,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3128145 = {S98028, 158, { &P20076,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26w Build/6.2.B.1.96) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3128318 = {S98029, 158, { &P18296,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC One V Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3128416 = {S98030, 97, { &P18382,&P18066,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire C) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3131386 = {S98031, 148, { &P23702,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ME173X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3131394 = {S98032, 134, { &P21636,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C525c Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131928 = {S98033, 138, { &P18295,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132424 = {S98034, 148, { &P15201,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132623 = {S98035, 130, { &P23773,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8010 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3132858 = {S98036, 141, { &P19842,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132905 = {S98037, 168, { &P13157,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132978 = {S98038, 134, { &P24368,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5215 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA15439 = {S98039, 107, { &P11818,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-X495/X495UVEL1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA15762 = {S98040, 68, { &P11839,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHE770/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA1757 = {S98041, 79, { &P16943,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-J770/J770AHHI2 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20156 = {S98042, 12, { &P15040,&P18110,&P17470,&P18092 }};	/*Useragent - Philips X116*/
const Device UA20403 = {S98043, 45, { &P13899,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-303A/1.0 ObigoInternetBrowser/Q03C*/
const Device UA49096 = {S98044, 44, { &P13899,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-303/1.0 ObigoInternetBrowser/Q03C*/
const Device UA21249 = {S98045, 82, { &P11981,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) i-mate 8502; 240x320*/
const Device UA21248 = {S98046, 82, { &P11981,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) i-mate 8502/ 240x320*/
const Device UA22027 = {S98047, 23, { &P15560,&P19954,&P3816,&P18092 }};	/*Useragent - SEC-SGHE530/1.0 TSS/2.5*/
const Device UA23091 = {S98048, 24, { &P15560,&P19954,&P3816,&P18092 }};	/*Useragent - SEC-SGHE530C/1.0 TSS/2.5*/
const Device UA22349 = {S98049, 40, { &P15430,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP275 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA63476 = {S98050, 22, { &P12132,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonJ10i2/R7CA*/
const Device UA79361 = {S98051, 100, { &P12132,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonJ10i/R7CA Browser/NetFront Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2*/
const Device UA22423 = {S98052, 98, { &P15464,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-Z720/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA60654 = {S98053, 107, { &P15464,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-Z720-Vodafone/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA33115 = {S98054, 87, { &P13693,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonF500i/R2E SEMC-Browser/4.0 Profile/MIDP-1.0 MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA40015 = {S98055, 81, { &P13693,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonF500i/R2AA SEMC-Browser/4.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA55962 = {S98056, 80, { &P13693,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonF500i/R2N SEMC-Browser/4.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64608 = {S98057, 81, { &P13693,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonF500i/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23420 = {S98058, 101, { &P15604,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-X670/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA61121 = {S98059, 119, { &P12637,&P17483,&P3857,&P18092 }};	/*Useragent - LG-KU580-Orange/v10a Browser/Obigo-Q05A1/2.16 MMS/LG-MMS-V1.0/1.2 Java/LGVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA75125 = {S98060, 132, { &P12637,&P17483,&P3857,&P18092 }};	/*Useragent - LG-KU580 Browser/Obigo-Q05A1/2.16 MMS/LG-MMS-V1.0/1.2 Java/LGVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Ext-V-Profile/VSCL-2.0.0*/
const Device UA87314 = {S98061, 150, { &P12637,&P17483,&P3857,&P18092 }};	/*Useragent - Vodafone/1.0/LG-KU580/V10a Browser/Obigo-Q05A1/2.16 MMS/LG-MMS-V1.0/1.2 Java/LGVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Ext-V-Profile/VSCL-2.0.0*/
const Device UA87333 = {S98062, 137, { &P12637,&P17483,&P3857,&P18092 }};	/*Useragent - LG/KU580/V10a Browser/Obigo-Q05A1/2.16 MMS/LG-MMS-V1.0/1.2 Java/LGVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Ext-V-Profile/VSCL-2.0.0*/
const Device UA23926 = {S98063, 79, { &P17083,&P5222,&P17183,&P18092 }};	/*Useragent - acer_S200 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5)*/
const Device UA23927 = {S98064, 83, { &P17083,&P5222,&P17183,&P18092 }};	/*Useragent - acer_S200 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5.3.5)*/
const Device UA23992 = {S98065, 26, { &P15645,&P17484,&P17606,&P18092 }};	/*Useragent - Mozilla/SMB3(Z105)/Samsung*/
const Device UA24524 = {S98066, 111, { &P15703,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHB500/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA24558 = {S98067, 11, { &P15711,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M5750*/
const Device UA24736 = {S98068, 68, { &P15751,&P5208,&P4223,&P18092 }};	/*Useragent - Mozilla/4.0 (BREW 3.1.5; U; en-us; Sanyo; NetFront/3.5.1/AMB) PL3810*/
const Device UA32857 = {S98069, 95, { &P12780,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Vodafone/1.0/HTC_v1415/1.18.161.3*/
const Device UA552739 = {S98070, 113, { &P14682,&P5843,&P4777,&P18092 }};	/*Useragent - Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303iclassic/UC Browser7.7.1.88/69/444*/
const Device UA69489 = {S98071, 99, { &P14669,&P5843,&P4779,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (04.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-00/UC Browser7.8.0.95/70/355*/
const Device UA42189 = {S98072, 116, { &P14912,&P17605,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C3222/1.0 RTK-E/1.0 DF/1.0 Release/11.12.2010 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54968 = {S98073, 41, { &P17540,&P17017,&P18246,&P18092 }};	/*Useragent - Fly_E135/MAUIBrowser/Configuration/WAP2.0*/
const Device UA55186 = {S98074, 140, { &P14293,&P17605,&P3976,&P18092 }};	/*Useragent - SAMSUNG-GT-S3930C_CMCC/1.0 RTK-E/1.0 SHP/VPP/R5 Release/17.08.2008 Browser/NetFront3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA500092 = {S98075, 146, { &P14611,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; in-id; Nexian NX-A890 Build/DONUT) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA67643 = {S98076, 36, { &P12088,&P5129,&P4715,&P18092 }};	/*Useragent - NokiaX6-00/UC Browser7.4.0.65/50/355*/
const Device UA60030 = {S98077, 61, { &P12536,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-C165/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA61161 = {S98078, 90, { &P12648,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHA501 SHP/CDR-2.2 NetFront/3.2 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA61773 = {S98079, 101, { &P14326,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E380/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA61837 = {S98080, 136, { &P14337,&P17483,&P3534,&P18092 }};	/*Useragent - LG-KF311/V10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61980 = {S98081, 56, { &P14357,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-VU204/00.62 UP.Browser/6.2.3.4.c.1.128 (GUI) MMP/2.0*/
const Device UA67991 = {S98082, 36, { &P14033,&P5129,&P4759,&P18092 }};	/*Useragent - NokiaC5-03/UC Browser7.6.1.82/50/351*/
const Device UA69144 = {S98083, 35, { &P13706,&P5129,&P4777,&P18092 }};	/*Useragent - Nokia5235/UC Browser7.7.1.88/50/444*/
const Device UA69177 = {S98084, 118, { &P11669,&P5776,&P4779,&P18092 }};	/*Useragent - Nokia7610/2.0 (6.0522.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.8.0.95/27/351*/
const Device UA87272 = {S98085, 45, { &P17647,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-ST34 UP.Browser/6.2.0.13.2 (GUI) MMP/2.0*/
const Device UA87459 = {S98086, 30, { &P13572,&P5282,&P3579,&P18092 }};	/*Useragent - ZTE-C78/1.0 SMIT-Browser/2.0.0*/
const Device UA709742 = {S98087, 46, { &P15364,&P18585,&P22247,&P18092 }};	/*Useragent - Flipboard/1.5.3 CFNetwork/467.12 Darwin/10.3.1*/
const Device UA541561 = {S98088, 107, { &P13337,&P17485,&P4777,&P18092 }};	/*Useragent - SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonj108i/UC Browser7.7.1.88/69/444*/
const Device UA1852853 = {S98089, 135, { &P15364,&P17285,&P22491,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Flock/3.0.6.4257 Chrome/5.0.375.1281 Safari/533.4*/
const Device UA491718 = {S98090, 158, { &P14900,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; zh-cn; 3GW100 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Lenovo-3GW100/1.0.0*/
const Device UA508195 = {S98091, 164, { &P14900,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; zh-cn; qsd8250_surf Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Lenovo-3GW100/1.0.0*/
const Device UA2018365 = {S98092, 145, { &P14900,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; zh-cn; Lenovo 3GW100 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA514872 = {S98093, 48, { &P23144,&P22474,&P17596,&P18092 }};	/*Useragent - MobileSafari/525.20 CFNetwork/339.3 Darwin/9.4.1*/
const Device UA524327 = {S98094, 35, { &P16003,&P17484,&P4742,&P18092 }};	/*Useragent - gt-s5233a/UC Browser7.5.1.77/69/352*/
const Device UA2131312 = {S98095, 35, { &P16003,&P17484,&P4742,&P18092 }};	/*Useragent - gt-s5233a/UC Browser7.5.1.77/69/351*/
const Device UA533653 = {S98096, 63, { &P15484,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-A-B2/02.01 UP.Browser/7.2.7.1.c.1.102 (GUI) MMP/2.0 Push/PO*/
const Device UA534039 = {S98097, 30, { &P15364,&P21445,&P17215,&P18092 }};	/*Useragent - Opera/9.52 (Windows ME; U; en)*/
const Device UA2135014 = {S98098, 30, { &P15364,&P21445,&P17215,&P18092 }};	/*Useragent - Opera/9.50 (Windows ME; U; en)*/
const Device UA558651 = {S98099, 80, { &P17595,&P18120,&P22507,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US; rv:1.8.1) Gecko/X Camino/1.1a1*/
const Device UA574299 = {S98100, 42, { &P14782,&P5890,&P24663,&P18092 }};	/*Useragent - NOKIAE61i/UC Browser7.9.0.102/28/355/UCWEB*/
const Device UA2302858 = {S98101, 41, { &P14782,&P5890,&P24663,&P18092 }};	/*Useragent - NOKIAE61/UC Browser7.9.0.102/28/355/UCWEB*/
const Device UA575400 = {S98102, 135, { &P12814,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-I777 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA621893 = {S98103, 39, { &P17968,&P17979,&P24433,&P18092 }};	/*Useragent - Dolphin http client/8.2.0(62) (Android)*/
const Device UA622259 = {S98104, 40, { &P17968,&P17979,&P24433,&P18092 }};	/*Useragent - Dolphin http client/8.2.1(141) (Android)*/
const Device UA622287 = {S98105, 40, { &P17968,&P17979,&P24433,&P18092 }};	/*Useragent - Dolphin http client/8.2.0(139) (Android)*/
const Device UA622326 = {S98106, 40, { &P17968,&P17979,&P24433,&P18092 }};	/*Useragent - Dolphin http client/8.2.2(144) (Android)*/
const Device UA656054 = {S98107, 31, { &P11929,&P5129,&P24666,&P18092 }};	/*Useragent - Nokia5230/UCWEB8.1.0.104/50/800*/
const Device UA1933790 = {S98108, 43, { &P14864,&P17484,&P17347,&P18092 }};	/*Useragent - samsung-gt-s8500/UC Browser8.0.3.107/69/352*/
const Device UA719649 = {S98109, 77, { &P15364,&P17350,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 7.1; ru; rv:1.9.3.15) Gecko/X Firefox/6.0*/
const Device UA811660 = {S98110, 107, { &P15364,&P22740,&P17213,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; ; Linux armv5tejl; U) Opera 8.02 [en_US] Maemo browser 0.4.34 N770/SU-18*/
const Device UA830791 = {S98111, 126, { &P15364,&P5690,&P18193,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:7.0.1) Gecko/X Thunderbird/7.0.1 OracleBeehiveExtension/1.0.0.4-OracleInternal Lightning/1.0b7*/
const Device UA856317 = {S98112, 139, { &P15364,&P17819,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.30 (KHTML, like Gecko) Ubuntu/11.10 Chromium/12.0.742.112 Chrome/12.0.742.112 Safari/534.30*/
const Device UA2260026 = {S98113, 135, { &P15364,&P17819,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.30 (KHTML, like Gecko) Ubuntu/11.10 Chromium/12.0.742.91 Chrome/12.0.742.91 Safari/534.30*/
const Device UA1838472 = {S98114, 82, { &P15364,&P5690,&P22532,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.7) Gecko/X Epic/ Firefox/*/
const Device UA1868150 = {S98115, 86, { &P17574,&P5567,&P4536,&P18092 }};	/*Useragent - Mozilla/5.0 Opera/9.5 (KDDI-SN3P; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00*/
const Device UA1892232 = {S98116, 107, { &P19968,&P17485,&P4777,&P18092 }};	/*Useragent - SonyEricssonW910i/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonw910i/UC Browser7.7.1.88/69/444*/
const Device UA1933936 = {S98117, 48, { &P17595,&P17017,&P17698,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh OS) Gecko/X Firefox/8.0.0*/
const Device UA1948120 = {S98118, 170, { &P17932,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Galaxy Mini (SGH-T499) Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1992511 = {S98119, 35, { &P14540,&P5843,&P21764,&P18092 }};	/*Useragent - iBrowser/Mini2.7 (NokiaX2-01/08.63)*/
const Device UA2034402 = {S98120, 166, { &P18114,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Transformer Prime TF201 Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2026863 = {S98121, 149, { &P16757,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I500 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3088884 = {S98122, 150, { &P16757,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; SCH-I500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA2027078 = {S98123, 149, { &P13974,&P5096,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5670 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028213 = {S98124, 164, { &P18510,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; ALCATEL ONE TOUCH 918N Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030493 = {S98125, 168, { &P18645,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-L710 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2030905 = {S98126, 176, { &P18645,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I747 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2384660 = {S98127, 193, { &P15575,&P5629,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, gb*/
const Device UA2352201 = {S98128, 190, { &P16362,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:34, us*/
const Device UA2034601 = {S98129, 147, { &P18295,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC One X Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121392 = {S98130, 148, { &P18295,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC One X Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2139580 = {S98131, 32, { &P14183,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SAMSUNG-SGH-I997/100.202*/
const Device UA2192134 = {S98132, 88, { &P15364,&P16110,&P22552,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:14.0) Gecko/X Firefox/14.0.3.0 IceDragon/14.0.3.0*/
const Device UA2212750 = {S98133, 80, { &P17595,&P18570,&P17470,&P18092 }};	/*Useragent - WebProcess/7534.48.3 CFNetwork/520.0.13 Darwin/11.0.0 (x86_64) (MacBookPro8%2C1)*/
const Device UA2217305 = {S98134, 78, { &P13847,&P17483,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; lg-lg900g) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2242470 = {S98135, 34, { &P13782,&P17483,&P17338,&P18092 }};	/*Useragent - lg-t310/UC Browser7.9.0.102/69/352*/
const Device UA2315620 = {S98136, 128, { &P17595,&P17297,&P24722,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_3; en-us) AppleWebKit/534.53.11 (KHTML, like Gecko) FluidApp/1.2 Safari/534.53.11*/
const Device UA2838073 = {S98137, 126, { &P17595,&P17297,&P24722,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_5; en-us) AppleWebKit/534.57.7 (KHTML, like Gecko) FluidApp/1.2 Safari/534.57.7*/
const Device UA2338629 = {S98138, 148, { &P12502,&P17485,&P24670,&P18092 }};	/*Useragent - SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonw20i) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2366758 = {S98139, 168, { &P14993,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT15i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2382918 = {S98140, 171, { &P14959,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2383347 = {S98141, 252, { &P14826,&P5843,&P18527,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/70/352*/
const Device UA2383778 = {S98142, 165, { &P14123,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P970 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2392689 = {S98143, 46, { &P14984,&P5129,&P17347,&P18092 }};	/*Useragent - nokia5250/UC Browser8.0.3.107/69/444  3gpp-gba*/
const Device UA2394248 = {S98144, 77, { &P16087,&P5448,&P4715,&P18092 }};	/*Useragent - 480x800 LGE VS750 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65*/
const Device UA2443871 = {S98145, 137, { &P12528,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; md-us; disgo8100 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2476409 = {S98146, 135, { &P22083,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; eZee'Tab702 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2484143 = {S98147, 87, { &P15364,&P17285,&P24354,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; QQBrowser/7.0.4350.400)*/
const Device UA2484715 = {S98148, 184, { &P18113,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2485207 = {S98149, 174, { &P13042,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC Incredible S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2485378 = {S98150, 90, { &P13365,&P17483,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-gu220) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2487588 = {S98151, 166, { &P18645,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; SPH-L710 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2505462 = {S98152, 99, { &P14698,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonw880i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2528045 = {S98153, 157, { &P16436,&P5629,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2528636 = {S98154, 94, { &P20058,&P22784,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxx660) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2530132 = {S98155, 166, { &P20935,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; ZTE-Z990G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2540665 = {S98156, 105, { &P21285,&P19795,&P19797,&P18092 }};	/*Useragent - Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.10+ (KHTML, like Gecko) Version/10.0.0.1337 Mobile Safari/537.10+*/
const Device UA2543577 = {S98157, 81, { &P12004,&P5890,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian95_8gb) UCBrowser8.3.0.154/69/352/UCWEB Mobile*/
const Device UA2552919 = {S98158, 87, { &P17595,&P18207,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Macintosh;Mac OS X (10.8.2);en;kml:2.2;client:Free;type:default)*/
const Device UA2555279 = {S98159, 71, { &P18283,&P18110,&P24673,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; GT-I9300; 720*1280) UCWEB8.7.4.225/145/800*/
const Device UA2555705 = {S98160, 153, { &P12414,&P5843,&P24673,&P18092 }};	/*Useragent - Nokia7020/2.0 (08.11) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia7020) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2557497 = {S98161, 170, { &P12280,&P18432,&P23594,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko, M5WebKit/1.0.1, M5HybridSDK/1.0.0) Version/5.1.0 Mobile/6.0.1 Mammoth/5.1.0*/
const Device UA2560315 = {S98162, 91, { &P11996,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5560) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2659601 = {S98163, 97, { &P11996,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; fa; samsung-gt-s5560i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2562291 = {S98164, 162, { &P16292,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03O) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2580360 = {S98165, 172, { &P18125,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID4 Build/6.7.2-180_DR4-16_M2-37) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2581803 = {S98166, 137, { &P17346,&P19826,&P20791,&P18092 }};	/*Useragent - MQQBrowser/40 Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Safari/7534.48.3*/
const Device UA2589255 = {S98167, 119, { &P14222,&P5448,&P4779,&P18092 }};	/*Useragent - 320x240; VZW; UTStar-XV6175.1; Windows Mobile 6.5 Standard; (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA2593533 = {S98168, 124, { &P12548,&P7472,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; es-la; alcatel-ot-807a) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2595114 = {S98169, 129, { &P22355,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; iDxD7 3G Build/ICS) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2596848 = {S98170, 161, { &P21057,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A7+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2597542 = {S98171, 99, { &P13183,&P17485,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonk810i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2617816 = {S98172, 53, { &P17337,&P17979,&P22076,&P18092 }};	/*Useragent - WhatsApp/2.7.9876 Android/3.2 Device/samsung-GT-P7500*/
const Device UA2615436 = {S98173, 164, { &P17337,&P6437,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7500 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2634452 = {S98174, 165, { &P17337,&P6437,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7500R Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2615947 = {S98175, 95, { &P13452,&P17979,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 Android (8/2.2; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; GT-P1000; en_GB)*/
const Device UA2618148 = {S98176, 98, { &P13452,&P17979,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 Android (10/2.3.3; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; gt-p1000; en_US)*/
const Device UA2616687 = {S98177, 99, { &P15919,&P17979,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P3100; espressorf; espresso; en_GB)*/
const Device UA2624403 = {S98178, 163, { &P17337,&P6546,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7500 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2630673 = {S98179, 164, { &P17337,&P6546,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7500R Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2628394 = {S98180, 163, { &P17337,&P6546,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7500 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2634716 = {S98181, 156, { &P15919,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-P3100 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2636125 = {S98182, 156, { &P15919,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-P3100 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2637239 = {S98183, 151, { &P14908,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6210 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.53005*/
const Device UA2893465 = {S98184, 150, { &P14908,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6210 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2645671 = {S98185, 130, { &P22258,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; TAB307 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2645904 = {S98186, 157, { &P21094,&P18066,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A789 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2651110 = {S98187, 175, { &P18652,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; Micromax A87 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2653719 = {S98188, 65, { &P19936,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; A100; 480*782) UCWEB7.9.0.94/139/355*/
const Device UA2656201 = {S98189, 147, { &P22005,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SmartPad870 Build/E8HD-T364-1280x768) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2663989 = {S98190, 180, { &P20981,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax P250(Funbook) Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2685751 = {S98191, 90, { &P21151,&P18110,&P20776,&P18092 }};	/*Useragent - MQQBrowser/25 (Linux; U; 2.3.5; zh-cn; LNV-Lenovo A790e Build/Lenovo A790e_S021_X;480*800)*/
const Device UA2685953 = {S98192, 73, { &P20251,&P18110,&P24666,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.3; zh-cn; HTC T328d; 480*800) UCWEB8.1.1.122/145/33021*/
const Device UA2695051 = {S98193, 147, { &P20265,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A52 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2695326 = {S98194, 158, { &P21269,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Intex Aqua 3.2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2695562 = {S98195, 173, { &P21223,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_C8810 Build/C8810V100R001C92B862) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2695682 = {S98196, 87, { &P18283,&P18110,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Adr (Linux; U; 4.0.4; zh-cn; GT-I9300 Build/IMM76D.I9300ZNBLH3;720*1280)*/
const Device UA2696075 = {S98197, 116, { &P17332,&P17483,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; lg-c310) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2700528 = {S98198, 160, { &P19937,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; A73 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2701381 = {S98199, 156, { &P16436,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2702369 = {S98200, 136, { &P18647,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T-02D Build/V19R50B) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2703372 = {S98201, 141, { &P21980,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; UBISLATE7C+ Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2705499 = {S98202, 165, { &P15802,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; HTC Ruby Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2706687 = {S98203, 173, { &P14183,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2707100 = {S98204, 137, { &P16757,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I500 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2707554 = {S98205, 137, { &P19917,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P765 Build/JRO03L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2707786 = {S98206, 171, { &P18505,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2726964 = {S98207, 172, { &P12280,&P17146,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3 UCBrowser8.5.0.195/42/355*/
const Device UA2726980 = {S98208, 131, { &P23661,&P24094,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40HX757; CC/COL; es-ES) Presto/2.10.250 Version/11.60*/
const Device UA2729159 = {S98209, 158, { &P14551,&P17483,&P24675,&P18092 }};	/*Useragent - LG-GW305/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-gw305) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2730004 = {S98210, 182, { &P13157,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2731888 = {S98211, 163, { &P14090,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I5801 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2732842 = {S98212, 153, { &P21592,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Kem 432 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2733121 = {S98213, 142, { &P21796,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Philips W336 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2733294 = {S98214, 141, { &P20183,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MI-ONE Plus Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2733363 = {S98215, 163, { &P12657,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-CN; HTC EVO 3D X515m Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2735294 = {S98216, 178, { &P18604,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Smartphone Android by SFR STARADDICT II Build/IceCreamSandwich) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735404 = {S98217, 123, { &P20822,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N-08D Build/AX) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2738025 = {S98218, 100, { &P15302,&P17485,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonwt13i) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2868388 = {S98219, 100, { &P15302,&P17485,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonwt13i) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2738363 = {S98220, 133, { &P22633,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; eZee Tab805 Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2741787 = {S98221, 139, { &P22714,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; LG-E975w Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2743488 = {S98222, 73, { &P15809,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; HTC_S510b; 480*800) UCWEB7.9.4.145/139/32500*/
const Device UA2746463 = {S98223, 165, { &P20470,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID9742 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2748282 = {S98224, 135, { &P23007,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TM400 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2767042 = {S98225, 138, { &P19747,&P5043,&P24670,&P18092 }};	/*Useragent - Nokia206/2.0 (03.59) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia206) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2750798 = {S98226, 163, { &P13644,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; EVO Build/JRO03C; LiquidSmooth-Liquid-JB-v2.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2751038 = {S98227, 138, { &P18606,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PadFone 2 Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2759158 = {S98228, 89, { &P12945,&P18110,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8/Adr (Linux; U; 2.3.5; zh-cn; Lenovo A780 Build/Lenovo_A780_S026_X;480*800)*/
const Device UA2762931 = {S98229, 161, { &P21347,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI U8661 Build/HuaweiU8661) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2762988 = {S98230, 144, { &P11754,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  SonyEricssonP990i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2763081 = {S98231, 155, { &P18612,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T329t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2763853 = {S98232, 95, { &P16530,&P22784,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx285) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2767077 = {S98233, 162, { &P20094,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2767774 = {S98234, 136, { &P20396,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE R22 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2768381 = {S98235, 134, { &P17717,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-SU760 Build/ICS) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2780274 = {S98236, 162, { &P18072,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; SHV-E120L Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2780687 = {S98237, 161, { &P18214,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2791024 = {S98238, 66, { &P14011,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.2; zh-cn; U8500; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2791242 = {S98239, 67, { &P14799,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2; zh-cn; GT-I5500; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2792362 = {S98240, 64, { &P19996,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; A15; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2793849 = {S98241, 166, { &P23375,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; HUAWEI Y 220T Build/HuaweiY220T) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2793862 = {S98242, 138, { &P20075,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xperia S Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2794317 = {S98243, 136, { &P20357,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SmartPad7503G Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2794613 = {S98244, 144, { &P20262,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Karbonn_A18 Build/KarbonnA18) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2799383 = {S98245, 134, { &P23124,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A919i Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2801673 = {S98246, 25, { &P18254,&P17350,&P18245,&P18091 }};	/*Useragent - SuperBot/4.7.0.72 (Win32)*/
const Device UA2804270 = {S98247, 80, { &P14732,&P5448,&P4779,&P18092 }};	/*Useragent - SAMSUNG-GT-B7330/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA2822627 = {S98248, 117, { &P14763,&P5776,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokian72) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2825103 = {S98249, 136, { &P18564,&P17979,&P18062,&P18092 }};	/*Useragent - oneX+ Linux/2.6.35 Android/4.0.1 Release/07.20.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1*/
const Device UA2830525 = {S98250, 154, { &P17625,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2833536 = {S98251, 96, { &P18648,&P5043,&P17347,&P18092 }};	/*Useragent - Nokia306/2.0 (03.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia306/UC Browser8.0.3.107/69/444*/
const Device UA2839354 = {S98252, 128, { &P23676,&P24094,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL22EX553; CC/GBR; en) Presto/2.10.250 Version/11.60*/
const Device UA2841058 = {S98253, 135, { &P15911,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT862 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2841367 = {S98254, 44, { &P14980,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SAMSUNG-GT-B7722/B7722DDKI1)*/
const Device UA2841933 = {S98255, 178, { &P19779,&P5629,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; A1+ Build/MocorDroid2.3.5; 320*480) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444*/
const Device UA2843217 = {S98256, 158, { &P15991,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6102E Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2843680 = {S98257, 100, { &P12193,&P5129,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (NokiaC5-00/071.005/sw_platform=S60;sw_platform_version=3.2;java_build_version=1.0.5)*/
const Device UA2843820 = {S98258, 98, { &P13861,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; es-la; samsung-gt-m3710) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2844695 = {S98259, 87, { &P23411,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; vi; q-mobile) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2849076 = {S98260, 133, { &P23354,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; A3 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2850332 = {S98261, 163, { &P22612,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8951 Build/HuaweiT8951) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2850370 = {S98262, 164, { &P20884,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A298t Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2850553 = {S98263, 156, { &P16292,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9050 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2850574 = {S98264, 156, { &P14186,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2853020 = {S98265, 166, { &P13991,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ADR6400L Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2853850 = {S98266, 151, { &P18421,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; FPT Tablet III Build/ARES_00.01.61.H1127) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2854021 = {S98267, 168, { &P20413,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; CELKON CT2 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2854761 = {S98268, 106, { &P20019,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_916) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2856974 = {S98269, 137, { &P20598,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Miracle Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2857610 = {S98270, 161, { &P19897,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; SCH-I779 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2859317 = {S98271, 86, { &P15364,&P17017,&P21771,&P18092 }};	/*Useragent - verification|card identification|cvn|security code|cvv code|cvc XiaoMi/MiuiBrowser/1.0*/
const Device UA2862163 = {S98272, 147, { &P21112,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; X907 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2862243 = {S98273, 159, { &P23354,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A3 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2862443 = {S98274, 65, { &P12280,&P19826,&P22071,&P18092 }};	/*Useragent - Instagram 3.1.2 (iPhone; iPhone OS 6.1.3; en_US) AppleWebKit/420+*/
const Device UA2863225 = {S98275, 132, { &P23262,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAC-70031 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2863270 = {S98276, 174, { &P15539,&P5629,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (HTC Glacier, 42, 1.9.1)*/
const Device UA2863714 = {S98277, 160, { &P23257,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8833 Build/HuaweiT8833) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2864777 = {S98278, 139, { &P15759,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC Vision Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2865233 = {S98279, 89, { &P20129,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; zh-CN; MI 2) U2/1.0.0 UCBrowser/9.0.0.288 U2/1.0.0 Mobile*/
const Device UA2865606 = {S98280, 139, { &P21304,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Fly_IQ450 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2867502 = {S98281, 137, { &P20075,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xperia S Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868137 = {S98282, 159, { &P13634,&P17483,&P24677,&P18092 }};	/*Useragent - LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gs500) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2868972 = {S98283, 160, { &P15880,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MB865 Build/6.7.2_EDEM-18) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2869431 = {S98284, 134, { &P23946,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; I7100 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2878195 = {S98285, 137, { &P20821,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; TAB192 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2878303 = {S98286, 152, { &P13051,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ZTE-T U880 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2879498 = {S98287, 155, { &P21037,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LIFETAB_S9512 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2879620 = {S98288, 156, { &P18320,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PadFone Build/JRO03L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2891149 = {S98289, 165, { &P20209,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID7036 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444*/
const Device UA2892438 = {S98290, 136, { &P19789,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KM-S200 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892559 = {S98291, 132, { &P23182,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP7280C3G Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2892560 = {S98292, 157, { &P21388,&P5096,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-US; Karbonn_A1 Build/FRF91) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2892589 = {S98293, 142, { &P20073,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tmn smart a15 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892729 = {S98294, 134, { &P16001,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SH-06D Build/S9030) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893036 = {S98295, 149, { &P18511,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; URBANO PROGRESSO Build/011.0.3100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893045 = {S98296, 170, { &P18552,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT897 Build/9.8.2Q-122_XT897_FFW-5) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2893180 = {S98297, 134, { &P18675,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SH-01E Build/S5270) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893669 = {S98298, 157, { &P16356,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2894219 = {S98299, 159, { &P22636,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Aquaris Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2937352 = {S98300, 136, { &P20366,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE T82 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937465 = {S98301, 156, { &P19917,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P765 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2937615 = {S98302, 127, { &P22231,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; T-17P Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2937697 = {S98303, 137, { &P12482,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T959 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937850 = {S98304, 142, { &P21856,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax P600 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937869 = {S98305, 138, { &P21320,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT25c Build/9.0.E.1.80) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937886 = {S98306, 138, { &P20180,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; XOLO_A800 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937887 = {S98307, 128, { &P20256,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TAB275 Build/TAB275) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2937923 = {S98308, 133, { &P14014,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; S120 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937973 = {S98309, 158, { &P15364,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TA-M72CBT Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2937979 = {S98310, 134, { &P20255,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; R8111 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2938139 = {S98311, 132, { &P21508,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GEM10313BK Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2938189 = {S98312, 152, { &P24073,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6033M Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2938190 = {S98313, 150, { &P15985,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6200 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2940355 = {S98314, 144, { &P20333,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax P500(Funbook) Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2942961 = {S98315, 93, { &P11689,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian70-1) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3009437 = {S98316, 91, { &P11689,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian70) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2950344 = {S98317, 137, { &P16273,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T679 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2951330 = {S98318, 101, { &P15364,&P5129,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5235) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2952457 = {S98319, 166, { &P20515,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; A54 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2952585 = {S98320, 91, { &P14790,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NokiaN77) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2953850 = {S98321, 136, { &P17308,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROIDX Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2954432 = {S98322, 166, { &P15798,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC Explorer A310e Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2955264 = {S98323, 135, { &P23336,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; I_7520 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2957213 = {S98324, 93, { &P11692,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA3250) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2957563 = {S98325, 142, { &P20181,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0; Swipe_Fablet_F1 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2958247 = {S98326, 144, { &P23152,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MegaFon_SP-AI Build/1.2.10.4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2958902 = {S98327, 140, { &P23199,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Coolpad7295 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2960946 = {S98328, 165, { &P23257,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI T8833 Build/HuaweiT8833) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2963899 = {S98329, 100, { &P15302,&P17485,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonwt13i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2965098 = {S98330, 166, { &P12731,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2971559 = {S98331, 142, { &P14919,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; HTC pyramid Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2978942 = {S98332, 37, { &P14454,&P5890,&P17347,&P18092 }};	/*Useragent - nokiae66-1/UC Browser8.0.3.107/69/444*/
const Device UA2979980 = {S98333, 87, { &P13771,&P17979,&P3508,&P18092 }};	/*Useragent - TCL_GENESEE_E708_TD/1.0 Android/2.2 Release/01.25.2011 Browser/WAP2.0 AppleWebKit/533.1*/
const Device UA2981619 = {S98334, 178, { &P15575,&P18066,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9100G Build/IMM76D; 480*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/32092*/
const Device UA2982719 = {S98335, 31, { &P18648,&P5043,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia306/03.63)*/
const Device UA2983892 = {S98336, 142, { &P24681,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Celkon CT 9 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984332 = {S98337, 160, { &P15354,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia Active Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984933 = {S98338, 94, { &P18460,&P18110,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Adr (Linux; U; 4.0.4; zh-cn; HUAWEI U8950D Build/U8950DV100R001C17B939;540*960)*/
const Device UA2985127 = {S98339, 97, { &P15364,&P16110,&P20806,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) Qt/4.8.4 Safari/534.34*/
const Device UA2985185 = {S98340, 65, { &P15364,&P18111,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.1.2; zh-cn; L003; 480*800) UCWEB7.9.0.94/139/800*/
const Device UA2986838 = {S98341, 161, { &P14574,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus S Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2987426 = {S98342, 157, { &P24579,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; tr-tr; PIRANHA Ultra Tab Zeus White 10.1 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2987513 = {S98343, 92, { &P15184,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; LG-P920) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2988125 = {S98344, 116, { &P13914,&P5843,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia5330-1d/06.88; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.49*/
const Device UA2988175 = {S98345, 98, { &P20094,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; HUAWEI_U8825D) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2988722 = {S98346, 90, { &P21035,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; V8403) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2988823 = {S98347, 152, { &P23725,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; O+_8.5_Android Build/OplusOplus_A51) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2988847 = {S98348, 136, { &P17307,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; AE910 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2989454 = {S98349, 140, { &P21289,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pl-pl; FreeTAB 9701 IPS Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2990201 = {S98350, 187, { &P19891,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (SAMSUNG-SGH-I717, 42, 1.9.1)*/
const Device UA2991869 = {S98351, 151, { &P18637,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9260 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2993074 = {S98352, 95, { &P20836,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Wynncom_G1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2993451 = {S98353, 70, { &P18437,&P18111,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.1.2; zh-cn; GT-N8000; 800*1232) UCWEB7.9.0.94/139/444*/
const Device UA2993466 = {S98354, 137, { &P23518,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZTE V987 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2993945 = {S98355, 92, { &P21953,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Ctrl_V1) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2994775 = {S98356, 137, { &P22853,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC 802w Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2995410 = {S98357, 156, { &P15876,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-P5100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2996262 = {S98358, 92, { &P14107,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN76) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2996513 = {S98359, 153, { &P18595,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I8190 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2996608 = {S98360, 99, { &P11843,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; SAMSUNGSGH-i450) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2996750 = {S98361, 106, { &P11907,&P5890,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaE52-1*/
const Device UA2997238 = {S98362, 165, { &P22582,&P18598,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2997862 = {S98363, 137, { &P24687,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC 802d Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2997996 = {S98364, 150, { &P22394,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C6603 Build/10.1.A.1.434) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2998446 = {S98365, 160, { &P19863,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2998862 = {S98366, 134, { &P23862,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HS-F1 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2999092 = {S98367, 166, { &P15798,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; HTC Explorer A310e Build/GRJ90) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2999646 = {S98368, 163, { &P13141,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; SGH-I727R Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2999923 = {S98369, 137, { &P24582,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE T790 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3000022 = {S98370, 155, { &P20119,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_S9714 Build/JRO03R) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3000083 = {S98371, 155, { &P18468,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; IdeaTabA2109A Build/JRO03R) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3000480 = {S98372, 138, { &P22853,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC 802w Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3001028 = {S98373, 134, { &P23758,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i9430 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3001158 = {S98374, 156, { &P22980,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; TECNO D5 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3002083 = {S98375, 156, { &P22875,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9220 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3004480 = {S98376, 90, { &P15364,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; C2104) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA3004879 = {S98377, 157, { &P19842,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HTC X920e Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3005480 = {S98378, 155, { &P23439,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A765e Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3005893 = {S98379, 159, { &P21141,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; Lenovo P770 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3007224 = {S98380, 157, { &P22385,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I8262D Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3008745 = {S98381, 157, { &P20294,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HTC T528w Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3009584 = {S98382, 159, { &P13124,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; PantechP9070 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3012414 = {S98383, 155, { &P14123,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; LG-P970 Build/IMM76L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3013464 = {S98384, 157, { &P20292,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HTC T528t Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3013602 = {S98385, 152, { &P22785,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire 600 dual sim Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3014404 = {S98386, 164, { &P18724,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; IdeaTab A2107A-H Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3014485 = {S98387, 177, { &P12059,&P5204,&P22881,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291*/
const Device UA3015313 = {S98388, 153, { &P13964,&P5096,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-US; GT-I5510 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3015519 = {S98389, 152, { &P19860,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A9+ Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3015611 = {S98390, 94, { &P16545,&P22784,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq80) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3016831 = {S98391, 90, { &P17625,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST25i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3017168 = {S98392, 88, { &P18482,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; A75) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3018012 = {S98393, 178, { &P14540,&P5843,&P22881,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.71) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291*/
const Device UA3018084 = {S98394, 156, { &P12972,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; SGH-T989 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3020690 = {S98395, 94, { &P12126,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA6730c) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3021350 = {S98396, 138, { &P15759,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Vision Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021434 = {S98397, 138, { &P19848,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M440S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021539 = {S98398, 126, { &P15602,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; A501 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3021743 = {S98399, 143, { &P22933,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Avea inTouch 2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021917 = {S98400, 138, { &P20248,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V889M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021969 = {S98401, 138, { &P23447,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Newman N2 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3023528 = {S98402, 141, { &P15794,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Sensation XL Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024100 = {S98403, 144, { &P23242,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SKY HD9500 PLUS Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024920 = {S98404, 137, { &P16263,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Huawei MediaPad Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3025251 = {S98405, 90, { &P18223,&P18111,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; LT22i) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3026248 = {S98406, 163, { &P14183,&P5096,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3028249 = {S98407, 93, { &P18595,&P18111,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8190) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3039549 = {S98408, 130, { &P24217,&P18110,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-55EX630; CC/AUS; en) Presto/2.10.250 Version/11.60*/
const Device UA3039553 = {S98409, 106, { &P24217,&P18110,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; U; ; en) Presto/2.10.250 Version/11.60 Sony Bravia/8.011 (KDL-55EX630, , en, US)*/
const Device UA3044777 = {S98410, 119, { &P23298,&P18110,&P17250,&P18092 }};	/*Useragent - OV-SteelCore10+ Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.34 Safari/534.24*/
const Device UA3045378 = {S98411, 137, { &P13074,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-P930 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3045590 = {S98412, 155, { &P22663,&P18066,&P20787,&P18092 }};	/*Useragent - MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; vivo S11t Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3045742 = {S98413, 141, { &P22818,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; M1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3048545 = {S98414, 111, { &P15364,&P17285,&P22527,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.6 (KHTML, like Gecko) Chrome/18.0.1025.133 Safari/537.6 Midori/0.5*/
const Device UA3048866 = {S98415, 142, { &P21020,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Micromax A116 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3049649 = {S98416, 137, { &P23410,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; iris458Q Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3049709 = {S98417, 142, { &P17713,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LGL55C Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA3049751 = {S98418, 136, { &P23189,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Surfer 8.31 3G Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3051967 = {S98419, 136, { &P22632,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; Titanium S1 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3053677 = {S98420, 136, { &P21602,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-in; C5302 Build/12.0.A.1.211) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3055263 = {S98421, 133, { &P22037,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3055357 = {S98422, 156, { &P23206,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Cloud Z5 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056747 = {S98423, 161, { &P14959,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9103 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056870 = {S98424, 157, { &P23771,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8190N Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056995 = {S98425, 90, { &P15147,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST15i) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3057074 = {S98426, 92, { &P16218,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN91) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3058141 = {S98427, 136, { &P23654,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; AVEO X6 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3059059 = {S98428, 90, { &P12856,&P17483,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t300) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3069994 = {S98429, 95, { &P15364,&P6704,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows 98) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2*/
const Device UA3070232 = {S98430, 88, { &P17976,&P18066,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Android 4.0.3; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.7.81 Version/11.00*/
const Device UA3074251 = {S98431, 151, { &P23694,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Symphony Xplorer W35 Build/UNKNOWN) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3074348 = {S98432, 130, { &P24180,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; TBD753 Build/Zeki.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3076600 = {S98433, 134, { &P24512,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; ru-ru; novo9-Spark Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3078190 = {S98434, 149, { &P17337,&P6546,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 UCBrowser/2.3.2.300*/
const Device UA3078391 = {S98435, 158, { &P20708,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; XOLO A1000 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3078474 = {S98436, 161, { &P24062,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; L3 Build/TECNOL3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/352*/
const Device UA3078590 = {S98437, 150, { &P20107,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5580C Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3078788 = {S98438, 90, { &P12968,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LT26i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3079460 = {S98439, 157, { &P18661,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; N860 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3088058 = {S98440, 92, { &P20470,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; MID9742) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3088850 = {S98441, 158, { &P22747,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; PIONEER_P1 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3090189 = {S98442, 137, { &P23008,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; Micromax A91 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3091738 = {S98443, 96, { &P23116,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Aqua.Active) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3092253 = {S98444, 132, { &P18713,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; LG-P769 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3092500 = {S98445, 132, { &P21059,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A90 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3092531 = {S98446, 153, { &P21472,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A111 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3092773 = {S98447, 157, { &P22385,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I8262D Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3092784 = {S98448, 156, { &P23697,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9192 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3092835 = {S98449, 160, { &P21267,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo A798t Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3092974 = {S98450, 91, { &P23417,&P22843,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lava-a10) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3093216 = {S98451, 165, { &P21657,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI T8950 Build/HuaweiT8950) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3093307 = {S98452, 95, { &P11950,&P5477,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; X10i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3093303 = {S98453, 156, { &P22610,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6810 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3094724 = {S98454, 143, { &P22933,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Avea inTouch 2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3102257 = {S98455, 132, { &P23767,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; pt-br; GT-7240R Build/GT72XX) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3106120 = {S98456, 165, { &P18652,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A87 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3106327 = {S98457, 140, { &P21611,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3106409 = {S98458, 160, { &P23027,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Spice Mi-450 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3106565 = {S98459, 129, { &P15364,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; in-id; E1-B Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3107728 = {S98460, 180, { &P23118,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Aqua_Star Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3107798 = {S98461, 171, { &P20004,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; Karbonn A26 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33756*/
const Device UA3107801 = {S98462, 132, { &P18017,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Ideapad K1 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3107832 = {S98463, 138, { &P20132,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST70104-1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3108218 = {S98464, 151, { &P15364,&P20706,&P24673,&P18092 }};	/*Useragent - Nokia210/2.0 (04.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia210) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA3108298 = {S98465, 184, { &P18510,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; ALCATEL_ONE_TOUCH_918N Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3108353 = {S98466, 159, { &P13042,&P18066,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Incredible S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3108363 = {S98467, 151, { &P22875,&P18066,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9220 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3108376 = {S98468, 88, { &P18083,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; V9C) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3108364 = {S98469, 156, { &P22875,&P5629,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3108839 = {S98470, 161, { &P21321,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9228 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3108869 = {S98471, 157, { &P21021,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC T328t Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3108894 = {S98472, 173, { &P22209,&P18111,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Titanium_S5 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3109245 = {S98473, 136, { &P21355,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; KM-E100 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109311 = {S98474, 165, { &P20529,&P18066,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A89 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3109342 = {S98475, 93, { &P21648,&P18598,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.2; en-US; GT-I9505) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3110397 = {S98476, 94, { &P23118,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua_Star) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3110912 = {S98477, 135, { &P22927,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; LIFETAB_E7310 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3110932 = {S98478, 136, { &P24625,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E410 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3111690 = {S98479, 94, { &P13474,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAX5-01) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3114299 = {S98480, 131, { &P22000,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TAB466EUK Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3121312 = {S98481, 144, { &P18241,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; LG-E400 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA3124310 = {S98482, 103, { &P13648,&P5055,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae7-00) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3124923 = {S98483, 139, { &P15160,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; it-it; starlight Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125136 = {S98484, 170, { &P22382,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8262 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3125334 = {S98485, 177, { &P22785,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Desire 600 dual sim Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.20.1364.172 Mobile Safari/537.22*/
const Device UA3125596 = {S98486, 92, { &P23414,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua_Sx) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3125772 = {S98487, 93, { &P15575,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; GT-I9100) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3125979 = {S98488, 90, { &P15147,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; ST15i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3126003 = {S98489, 155, { &P22805,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-E455 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126225 = {S98490, 127, { &P20510,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; CT920 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126296 = {S98491, 141, { &P24049,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; BLU DASH 4.5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126368 = {S98492, 130, { &P23767,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-7240R Build/GT72XX) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126601 = {S98493, 127, { &P24386,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SH-08E Build/S8220) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126723 = {S98494, 137, { &P24052,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SKY_NANO Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126783 = {S98495, 103, { &P15798,&P5629,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC Explorer A310e) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3127131 = {S98496, 137, { &P22875,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9220 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127169 = {S98497, 134, { &P23949,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; H9500 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127236 = {S98498, 134, { &P23217,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP7170B3GCE Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3127256 = {S98499, 159, { &P20183,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; MI-ONE Plus Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127272 = {S98500, 135, { &P24585,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ZP800H Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127492 = {S98501, 157, { &P16362,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-D710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3127632 = {S98502, 160, { &P22867,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5303 Build/12.0.A.1.257) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3128159 = {S98503, 131, { &P15767,&P22882,&P20783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) MQQBrowser/3.2 Mobile/11A465 Safari/7534.48.3*/
const Device UA3130563 = {S98504, 157, { &P18488,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3130747 = {S98505, 137, { &P18594,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-LS860 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131320 = {S98506, 129, { &P17686,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MZ601 Build/I.7.1-45) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3131580 = {S98507, 163, { &P19862,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Micromax A110Q Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3132431 = {S98508, 141, { &P19778,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PantechP9090 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132964 = {S98509, 139, { &P23240,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; USCC-C6721 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA552 = {S98510, 78, { &P11696,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) MOT-Q9/02.00.01R*/
const Device UA58491 = {S98511, 78, { &P11696,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) MOT-Q9/01.09.34I*/
const Device UA2232 = {S98512, 131, { &P11927,&P17017,&P4893,&P18092 }};	/*Useragent - SIE-EF81/39 UP.Browser/7.0.0.1.181 (GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.1.3.h (GUI) MMP/2.0*/
const Device UA18744 = {S98513, 131, { &P11927,&P17017,&P4893,&P18092 }};	/*Useragent - SIE-EF81/48 UP.Browser/7.0.0.1.181 (GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.1.3.h (GUI) MMP/2.0*/
const Device UA21379 = {S98514, 132, { &P11927,&P17017,&P4893,&P18092 }};	/*Useragent - SIE-EF81/58 UP.Browser/7.0.0.1.181 (GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.1.k.10 (GUI) MMP/2.0*/
const Device UA2906 = {S98515, 76, { &P11870,&P5448,&P4578,&P18092 }};	/*Useragent - HTCS620-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 320x240)*/
const Device UA4249 = {S98516, 120, { &P12095,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-U708/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1/*MzU1MjYyMDExMTk4OTk4*/
const Device UA12323 = {S98517, 87, { &P11734,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_P4550 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA12328 = {S98518, 89, { &P11734,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_TyTN_II Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA62803 = {S98519, 61, { &P14451,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonP800/R101 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA20593 = {S98520, 40, { &P15104,&P7205,&P4841,&P18092 }};	/*Useragent - SCH-F699/ Embider/5.30 (GUI; compatible)*/
const Device UA23864 = {S98521, 81, { &P12075,&P5448,&P22271,&P18092 }};	/*Useragent - SAMSUNG-SGH-I616/1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA24560 = {S98522, 52, { &P17455,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-R4541/00.72 UP.Browser/7.2.6.1.841 (GUI) MMP/2.0*/
const Device UA33054 = {S98523, 46, { &P12837,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-MX380/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA33379 = {S98524, 134, { &P12907,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ru-ru; MTC 1055 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49209 = {S98525, 138, { &P22684,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; ru-ru; Beeline E400 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA50103 = {S98526, 137, { &P13166,&P17486,&P20418,&P18092 }};	/*Useragent - MOT-E680i/E680I_G_0D.C5.ABP Mozilla/4.0 (compatible; MSIE 6.0; Motorola; 1320) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 7.50 [zh-tw]*/
const Device UA50121 = {S98527, 147, { &P13166,&P17486,&P20418,&P18092 }};	/*Useragent - MOT-E680i/E680I_G_0D.C5.A4P Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola E680i; 1036) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 7.50 [en]*/
const Device UA50133 = {S98528, 147, { &P13166,&P17486,&P20418,&P18092 }};	/*Useragent - MOT-E680i/E680I_G_0D.C5.A3P Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola E680i; 1030) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 7.50 [en]*/
const Device UA50185 = {S98529, 147, { &P13166,&P17486,&P20418,&P18092 }};	/*Useragent - MOT-E680i/E680I_G_0D.C5.A6P Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola E680i; 1115) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 7.50 [en]*/
const Device UA50188 = {S98530, 147, { &P13166,&P17486,&P20418,&P18092 }};	/*Useragent - MOT-E680i/E680I_G_0D.C5.AAP Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola E680i; 1229) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 7.50 [en]*/
const Device UA82718 = {S98531, 137, { &P13166,&P17486,&P20418,&P18092 }};	/*Useragent - MOT-E680i/E680I_G_0D.C3.A8P Mozilla/4.0 (compatible; MSIE 6.0; Motorola; 1030) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 7.50 [zh-tw]*/
const Device UA50113 = {S98532, 61, { &P13188,&P6712,&P4030,&P18092 }};	/*Useragent - Nokia6310i/1.0 (4.80) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA50146 = {S98533, 36, { &P13180,&P17811,&P17810,&P18092 }};	/*Useragent - LENOVO-A330/(2009.06.05) S007/WAP2.0*/
const Device UA50298 = {S98534, 36, { &P13180,&P17811,&P17810,&P18092 }};	/*Useragent - LENOVO-A330/(2009.08.26) S009/WAP2.0*/
const Device UA67756 = {S98535, 36, { &P13180,&P17811,&P17810,&P18092 }};	/*Useragent - LENOVO-A330/(2009.08.26) S012/WAP2.0*/
const Device UA53681 = {S98536, 105, { &P14005,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S5500H/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1/*MA==*/
const Device UA60978 = {S98537, 95, { &P14005,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S5500/ SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA53835 = {S98538, 71, { &P14043,&P22839,&P4924,&P18092 }};	/*Useragent - ZTE-F252/1.0 ACS-NF/3.2 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA56009 = {S98539, 62, { &P23010,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonK790i/R8BF Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA59143 = {S98540, 175, { &P17770,&P5448,&P4808,&P18092 }};	/*Useragent - dopodT8388/1.0 WindowsMobile/6.5 CEOS/5.2 release/5.0 IE/6.0 WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1 4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA2586829 = {S98541, 183, { &P17770,&P5448,&P4808,&P18092 }};	/*Useragent - dopodT8388/1.0 WindowsMobile/6.5 CEOS/5.2 release/5.0 IE/6.0 WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA62779 = {S98542, 98, { &P11922,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-Z248/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA63805 = {S98543, 74, { &P16856,&P22835,&P17470,&P18092 }};	/*Useragent - Spice QT-57/(2010.06.22) S005/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA68230 = {S98544, 34, { &P13163,&P5129,&P4759,&P18092 }};	/*Useragent - NokiaN97/UC Browser7.6.1.82/50/352*/
const Device UA2136877 = {S98545, 99, { &P14826,&P5843,&P4759,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (04.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac1-01/UC Browser7.6.1.82/69/352*/
const Device UA68713 = {S98546, 30, { &P11929,&P5129,&P24662,&P18092 }};	/*Useragent - Nokia5230/UCWEB7.8.0.87/50/999*/
const Device UA68760 = {S98547, 34, { &P14454,&P5890,&P4777,&P18092 }};	/*Useragent - NokiaE66/UC Browser7.7.1.88/28/351*/
const Device UA87047 = {S98548, 93, { &P15182,&P17484,&P4924,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungSGHX680V/X680BUFG4 Browser/NF/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87474 = {S98549, 43, { &P17619,&P17484,&P4203,&P18092 }};	/*Useragent - Samsung-SPHM530 AU-OBIGO/Q04C1-1.22 MMP/2.0*/
const Device UA88669 = {S98550, 164, { &P13906,&P5129,&P3690,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0 SonyEricssonP100/01; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 Safari/525*/
const Device UA90627 = {S98551, 31, { &P17728,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F08C(c100;TB;W20H09)*/
const Device UA102799 = {S98552, 140, { &P12280,&P17134,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/1A542a Safari/419.3*/
const Device UA103027 = {S98553, 25, { &P18031,&P17017,&P17470,&P18092 }};	/*Useragent - HOSIN_M100/SW1.0.0/WAP2.0*/
const Device UA333930 = {S98554, 35, { &P15209,&P5768,&P4777,&P18092 }};	/*Useragent - nokia6233/UC Browser7.7.1.88/69/444*/
const Device UA438693 = {S98555, 38, { &P15364,&P17285,&P4810,&P18092 }};	/*Useragent - Opera/7.23 (Windows NT 6.0; U) [zh-cn]*/
const Device UA441003 = {S98556, 114, { &P15767,&P17146,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.8.3*/
const Device UA652743 = {S98557, 114, { &P15767,&P17146,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.8.4*/
const Device UA855760 = {S98558, 114, { &P15767,&P17146,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.8.1*/
const Device UA452483 = {S98559, 35, { &P15364,&P22784,&P17338,&P18092 }};	/*Useragent - micromax/UC Browser7.9.0.102/69/352*/
const Device UA462864 = {S98560, 97, { &P15364,&P18134,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.9) Gecko/X Linux Mint/7 (Gloria) Shiretoko/3.5.7*/
const Device UA527173 = {S98561, 40, { &P15364,&P17017,&P23785,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; ipMonitor 10.5)*/
const Device UA538077 = {S98562, 52, { &P18088,&P17017,&P24660,&P18092 }};	/*Useragent - IUC(U;iOS 4.3.1;Zh-cn;320*480;)/UCWEB7.6.0.75/41/997*/
const Device UA738615 = {S98563, 52, { &P18088,&P17017,&P24660,&P18092 }};	/*Useragent - IUC(U;iOS 4.2.1;Zh-cn;320*480;)/UCWEB7.6.0.75/41/997*/
const Device UA574562 = {S98564, 43, { &P13648,&P5055,&P24663,&P18092 }};	/*Useragent - NokiaE7-00/UC Browser7.9.0.102/50/355/UCWEB*/
const Device UA709685 = {S98565, 106, { &P12026,&P17485,&P17347,&P18092 }};	/*Useragent - SonyEricssonC510/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonc510/UC Browser8.0.3.107/69/444*/
const Device UA607405 = {S98566, 132, { &P15364,&P16110,&P22427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.152 Safari/535.19 Comodo_Dragon/18.1.2.0*/
const Device UA608703 = {S98567, 125, { &P15364,&P16110,&P22427,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.152 Safari/535.19 Comodo_Dragon/18.1.2.0*/
const Device UA612348 = {S98568, 120, { &P15767,&P18117,&P18356,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/ Sleipnir/1.9.1m*/
const Device UA615155 = {S98569, 129, { &P15767,&P18117,&P18356,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3 Sleipnir/1.9.2m*/
const Device UA656512 = {S98570, 131, { &P15767,&P18117,&P18356,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/1.9.2m*/
const Device UA616299 = {S98571, 83, { &P15364,&P17017,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nexiannx-g868t) UCBrowser8.2.0.132/69/444/UCWEB Mobile*/
const Device UA758810 = {S98572, 74, { &P15364,&P17017,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; spice) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA661938 = {S98573, 62, { &P15364,&P5690,&P17208,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:6.0) Gecko/X Firefox/6.0 slurp*/
const Device UA669521 = {S98574, 92, { &P15364,&P18342,&P20444,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.2) Gecko/X Ubuntu/9.25 (jaunty) Firefox/3.8*/
const Device UA676608 = {S98575, 122, { &P15364,&P18110,&P22456,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Iron/6.0.475.1 Chrome/6.0.475.1 Safari/534.3*/
const Device UA699503 = {S98576, 104, { &P12135,&P17483,&P17347,&P18092 }};	/*Useragent - LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  lg-gd510/UC Browser8.0.3.107/69/352*/
const Device UA726073 = {S98577, 44, { &P17595,&P17295,&P21582,&P18092 }};	/*Useragent - MacOutlook/14.14.0.X (Intel Mac OS X 10.5.8)*/
const Device UA738083 = {S98578, 54, { &P14748,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (NokiaE72-1;SymbianOS/9.1 Series60/3.0)*/
const Device UA772840 = {S98579, 87, { &P17595,&P17295,&P18211,&P18092 }};	/*Useragent - GoogleEarth/5.2.1.1588(Macintosh;Mac OS X (10.5.8);en;kml:2.2;client:Free;type:default)*/
const Device UA829050 = {S98580, 42, { &P17519,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 830P)*/
const Device UA849167 = {S98581, 58, { &P15364,&P18146,&P17208,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; SunOS sun4u; rv:6.0) Gecko/X Firefox/6.0*/
const Device UA887569 = {S98582, 66, { &P15364,&P7127,&P20418,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0) Opera 7.50 [en]*/
const Device UA1903027 = {S98583, 42, { &P15364,&P7127,&P20418,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; U) Opera 7.50*/
const Device UA907549 = {S98584, 66, { &P15364,&P17285,&P18171,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:13.0) Gecko/X Firefox/13.0.1 slurp*/
const Device UA925082 = {S98585, 40, { &P17968,&P17979,&P24436,&P18092 }};	/*Useragent - Dolphin http client/8.6.1(159) (Android)*/
const Device UA2028810 = {S98586, 40, { &P17968,&P17979,&P24436,&P18092 }};	/*Useragent - Dolphin http client/8.6.0(156) (Android)*/
const Device UA1897965 = {S98587, 66, { &P15364,&P17285,&P17208,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:6.0.2) Gecko/X Firefox/6.0.2 slurp*/
const Device UA1905306 = {S98588, 32, { &P15364,&P22835,&P17338,&P18092 }};	/*Useragent - spice/UC Browser7.9.0.102/69/352*/
const Device UA1934100 = {S98589, 134, { &P21510,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-tw; MioPad 6 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1938418 = {S98590, 113, { &P16503,&P5204,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia7310/09.40; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA2018283 = {S98591, 143, { &P18000,&P6757,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android unknown; fr-fr; A70HB Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2018289 = {S98592, 146, { &P21670,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1.1832; th-th; i-mobile i810 Build/s1832) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018672 = {S98593, 141, { &P21670,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; th-th; i-mobile i810 Build/s1834) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022981 = {S98594, 157, { &P15366,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; HTC Tattoo Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028666 = {S98595, 157, { &P14186,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028906 = {S98596, 155, { &P15575,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032059 = {S98597, 157, { &P16436,&P5629,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2138774 = {S98598, 40, { &P15364,&P23099,&P17470,&P18092 }};	/*Useragent - MDN/1.0 CFNetwork/485.13.9 Darwin/10.7.0*/
const Device UA2810941 = {S98599, 71, { &P14887,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.2; zh-cn; GT-I9008L; 480*800) UCWEB7.9.4.145/139/800*/
const Device UA2157204 = {S98600, 85, { &P14404,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5260) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2209161 = {S98601, 89, { &P15364,&P17285,&P20902,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.7.2) Gecko/X Netscape/7.2 (ax;nscd1)*/
const Device UA2213028 = {S98602, 36, { &P12061,&P5890,&P4777,&P18092 }};	/*Useragent - nokiae65-1/UC Browser7.7.1.88/69/444*/
const Device UA2222567 = {S98603, 131, { &P21051,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GEM8112 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2232552 = {S98604, 39, { &P15775,&P5448,&P20762,&P18092 }};	/*Useragent - NOKIA-Lumia 800/UCWEB2.5.0.202/47/32079*/
const Device UA2242271 = {S98605, 47, { &P14358,&P5890,&P21754,&P18092 }};	/*Useragent - iBrowser/2.4 (E63-1;SymbianOS/9.1 Series60/3.0)*/
const Device UA2259984 = {S98606, 103, { &P15364,&P18145,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.215 Safari/535.1*/
const Device UA2296434 = {S98607, 102, { &P15364,&P18145,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD i386) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.215 Safari/535.1*/
const Device UA2266390 = {S98608, 71, { &P15364,&P18146,&P20917,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; SunOS i86pc; de; rv:1.9.1b3) Gecko/X Firefox/3.1b3*/
const Device UA2296372 = {S98609, 81, { &P13578,&P5448,&P4779,&P18092 }};	/*Useragent - HTC_P3470 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA2342996 = {S98610, 132, { &P21480,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PMP3270B Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2362503 = {S98611, 170, { &P16292,&P18111,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; Galaxy Nexus Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2474427 = {S98612, 76, { &P12856,&P17483,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; lg-t300) UCBrowser8.3.0.154/69/405/UCWEB Mobile*/
const Device UA2378210 = {S98613, 154, { &P20983,&P18066,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9308 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2392516 = {S98614, 102, { &P17595,&P17297,&P22481,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.0.16) Gecko/X Firefox/3.0.16 Flock/2.5.6*/
const Device UA2421819 = {S98615, 117, { &P23417,&P22843,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; lava-a10) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2423420 = {S98616, 120, { &P13474,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAX5-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile*/
const Device UA2442315 = {S98617, 70, { &P18700,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;GT-B5512;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA2455069 = {S98618, 164, { &P14967,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2455458 = {S98619, 125, { &P12399,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU8i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/352/UCWEB Mobile*/
const Device UA2456973 = {S98620, 99, { &P14060,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonw150i) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2457785 = {S98621, 99, { &P14060,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonw150i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2483192 = {S98622, 96, { &P14060,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; fa; sonyericssonw150i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2461322 = {S98623, 229, { &P18304,&P5629,&P17373,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/6.5.27431/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2494379 = {S98624, 70, { &P15364,&P16110,&P22335,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; rv:5.0) Gecko/X Firefox/5.0 CometBird/5.0*/
const Device UA2494984 = {S98625, 73, { &P15364,&P7127,&P21170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:13.0) Gecko/X Firefox/13.0 SeaMonkey/2.10*/
const Device UA2495549 = {S98626, 48, { &P15364,&P18570,&P18589,&P18092 }};	/*Useragent - Skype/4.0.0.1325 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2506364 = {S98627, 160, { &P20513,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2526331 = {S98628, 156, { &P21036,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Spice_Mi-425 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2527291 = {S98629, 147, { &P15767,&P17146,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3 TeamStream HD (iPad2,1 284 1.7)*/
const Device UA2544405 = {S98630, 159, { &P15147,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ST15i Build/4.0.2.A.0.42; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2552851 = {S98631, 159, { &P14031,&P5477,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I5503 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2555587 = {S98632, 91, { &P13088,&P17486,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; mot-ex223) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2557024 = {S98633, 168, { &P15079,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MK16i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2557640 = {S98634, 99, { &P14972,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3370) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2578478 = {S98635, 147, { &P15555,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; myTouch_4G_Slide Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2578823 = {S98636, 136, { &P18600,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SC-03D Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2579654 = {S98637, 165, { &P14574,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; Nexus S Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2589652 = {S98638, 73, { &P17968,&P17979,&P17597,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Linux armv7l;rv:5.0) Gecko/X Firefox/5.0 Fennec/5.0*/
const Device UA2593179 = {S98639, 155, { &P18672,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5302 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2619078 = {S98640, 95, { &P13452,&P17979,&P22069,&P18092 }};	/*Useragent - Instagram 1.1.1 Android (8/2.2; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; GT-P1000; en_GB)*/
const Device UA2619249 = {S98641, 98, { &P13452,&P17979,&P22069,&P18092 }};	/*Useragent - Instagram 1.1.6 Android (10/2.3.3; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; gt-p1000; en_US)*/
const Device UA2617672 = {S98642, 104, { &P15876,&P17979,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 Android (15/4.0.3; 160dpi; 800x1232; samsung; GT-P5100; espresso10rf; espresso10; en_US)*/
const Device UA2634027 = {S98643, 156, { &P15919,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; GT-P3100 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2635954 = {S98644, 156, { &P15919,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P3100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2637601 = {S98645, 151, { &P17323,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2629588 = {S98646, 160, { &P13452,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0(Linux; U; Android 2.3.4; zh-CN; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.0.212 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2633678 = {S98647, 161, { &P13452,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2634695 = {S98648, 161, { &P13452,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2637419 = {S98649, 162, { &P13452,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; GT-P1000T Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.1.258 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2638776 = {S98650, 127, { &P11980,&P5448,&P4777,&P18092 }};	/*Useragent - PPC; 240x320; HTC_P3300/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2640253 = {S98651, 57, { &P12798,&P18110,&P24661,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.4;En;LT18i;480*854;)UCWEB7.7.0.85/139/999*/
const Device UA2647079 = {S98652, 67, { &P15397,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.4;Zh_cn;ST18i;480*854;)UCWEB7.8.0.95/139/444*/
const Device UA2648760 = {S98653, 157, { &P15649,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2652100 = {S98654, 158, { &P21033,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; U1 Build/HCL ME Tablet U1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2652553 = {S98655, 143, { &P12765,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; A100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2652935 = {S98656, 118, { &P17595,&P18207,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.68 Safari/534.24*/
const Device UA2653228 = {S98657, 131, { &P20187,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Novo7Fire Build/X.qm) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2654862 = {S98658, 157, { &P21129,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i929 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2655500 = {S98659, 152, { &P20983,&P18066,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9308 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2657355 = {S98660, 172, { &P18581,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SCH-S720C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2658012 = {S98661, 166, { &P18595,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-I8190 Build/JRO03H) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2659220 = {S98662, 153, { &P13051,&P5096,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-T U880 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2660440 = {S98663, 165, { &P12788,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; PG86100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2666247 = {S98664, 80, { &P17595,&P18207,&P18689,&P18090 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:19.0) Gecko/X Firefox/19.0 slurp*/
const Device UA2693842 = {S98665, 152, { &P20267,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; P300(Funbook) Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2694041 = {S98666, 66, { &P18088,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2; zh-cn; generic; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2694159 = {S98667, 125, { &P12260,&P17484,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5350) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2698654 = {S98668, 155, { &P20947,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Evo V 4G Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2698942 = {S98669, 88, { &P18088,&P22846,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lemon) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2702580 = {S98670, 168, { &P15079,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MK16i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2705089 = {S98671, 161, { &P14433,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9003 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2719662 = {S98672, 136, { &P22655,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5503 Build/10.1.1.A.1.210) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2719678 = {S98673, 143, { &P22655,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5503 Build/10.1.1.A.1.191) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2733200 = {S98674, 155, { &P20542,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PAP5000TDUO Build/PrestigioPAP5000TDUO) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2734202 = {S98675, 161, { &P12280,&P19826,&P22580,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_1_3 like Mac OS X; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/2.0.7961 Mobile*/
const Device UA2735010 = {S98676, 138, { &P23947,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Alpha GTR Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734995 = {S98677, 139, { &P18582,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xperia ion Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735175 = {S98678, 136, { &P21355,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; KM-E100 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2736088 = {S98679, 139, { &P17108,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Inspire 4G Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2736865 = {S98680, 138, { &P19848,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M440S Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2747051 = {S98681, 151, { &P11747,&P5096,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2748803 = {S98682, 167, { &P23460,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; I-Buddy 3 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2751004 = {S98683, 130, { &P20349,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Novo7Flame Build/X.tm) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2751150 = {S98684, 137, { &P19746,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VIVO 4.3 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2764326 = {S98685, 180, { &P20333,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax P500(Funbook) Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2764554 = {S98686, 92, { &P13088,&P17486,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mot-ex223) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2765641 = {S98687, 170, { &P15802,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC_Amaze_4G Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2766760 = {S98688, 156, { &P20394,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; ZTE U790 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2766795 = {S98689, 161, { &P21129,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; SCH-i929 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2767437 = {S98690, 137, { &P15850,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-LU6200 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2767620 = {S98691, 138, { &P17108,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Inspire 4G Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2779080 = {S98692, 165, { &P15364,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; PIONEER Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2780848 = {S98693, 152, { &P21592,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Kem 432 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2781272 = {S98694, 91, { &P13365,&P17483,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gu220) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2781974 = {S98695, 155, { &P20202,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-L38C Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2788051 = {S98696, 79, { &P12216,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Mail; 1.4.5) (inc; HTC; ADR6300; 2.3.4/GRJ22)*/
const Device UA2790067 = {S98697, 139, { &P12697,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; shooteru Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2790470 = {S98698, 165, { &P21151,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; LNV-Lenovo A790e Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2792452 = {S98699, 173, { &P14842,&P5477,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E10i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2793663 = {S98700, 138, { &P21132,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Andi4.5h Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2795593 = {S98701, 141, { &P21738,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; iris502 Build/LAVAIRIS502) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2817827 = {S98702, 78, { &P12174,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; sgh-f480i) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2818686 = {S98703, 95, { &P18506,&P5043,&P4759,&P18092 }};	/*Useragent - Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia305/UC Browser7.6.1.82/69/351*/
const Device UA2831351 = {S98704, 140, { &P21719,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; htc_jewel Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2833433 = {S98705, 179, { &P18513,&P5629,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6802 Build/GINGERBREAD; 320*480) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444*/
const Device UA2834723 = {S98706, 96, { &P15364,&P17285,&P21778,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.3.5 Safari/534.34*/
const Device UA2838746 = {S98707, 161, { &P14842,&P5477,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E10i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2840754 = {S98708, 155, { &P15060,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-S5660 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2841436 = {S98709, 138, { &P15860,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-SU880 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2841727 = {S98710, 118, { &P11658,&P5776,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokia6680) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2842819 = {S98711, 76, { &P17595,&P18207,&P18196,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:5.0) Gecko/X Thunderbird/5.0*/
const Device UA2843196 = {S98712, 152, { &P18739,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOR801 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2843760 = {S98713, 136, { &P20122,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Evodroid W10 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2844987 = {S98714, 186, { &P13914,&P5843,&P24671,&P18092 }};	/*Useragent - Nokia5330-1d/5.0 (06.88) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5330-1d) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2845698 = {S98715, 165, { &P18027,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID7042 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2847099 = {S98716, 166, { &P19847,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-S6012 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2847271 = {S98717, 139, { &P18174,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; VS920 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2851732 = {S98718, 175, { &P16595,&P5477,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; Micromax A60 Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2854024 = {S98719, 166, { &P18731,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; ONDA MID Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2854127 = {S98720, 93, { &P12452,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; NOKIAE55-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2854402 = {S98721, 99, { &P14029,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5330) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2854723 = {S98722, 133, { &P21820,&P6546,&P4459,&P18092 }};	/*Useragent - TBD753B Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2856583 = {S98723, 103, { &P13815,&P5477,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; HTC_Wildfire) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2856845 = {S98724, 173, { &P20192,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2856896 = {S98725, 141, { &P21873,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB M713G Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2857335 = {S98726, 170, { &P23061,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; TECNO_B3 Build/TECNO_B3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/33567*/
const Device UA2857777 = {S98727, 90, { &P13025,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; WT19a) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2857866 = {S98728, 136, { &P22681,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; UMI X1 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2858372 = {S98729, 137, { &P20075,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Xperia S Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2859094 = {S98730, 116, { &P16745,&P5477,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; T-Mobile_myTouch_3G_Slide) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2860334 = {S98731, 152, { &P18737,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; X909 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2860559 = {S98732, 165, { &P21657,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI T8950 Build/HuaweiT8950) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2861173 = {S98733, 138, { &P18622,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A820L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2861438 = {S98734, 164, { &P13018,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1 Jelly Bean; es-es; SonyEricssonMT11i Build/4.1.B.0.587) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2861671 = {S98735, 135, { &P23241,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GN708W Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2861685 = {S98736, 136, { &P20267,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; P300(Funbook) Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2861904 = {S98737, 139, { &P20006,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT30a Build/7.0.A.3.195) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2862672 = {S98738, 152, { &P21852,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Spice_Mi-495 Build/SpiceSpice_Mi-495) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2862893 = {S98739, 126, { &P20252,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V96A Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2863092 = {S98740, 146, { &P15555,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; MyTouch Slide 4G Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2863149 = {S98741, 139, { &P20731,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Blaze S180 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2864264 = {S98742, 144, { &P23473,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; RMD-1035 Build/ICS.g12refM1005RU.X) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2865751 = {S98743, 135, { &P15365,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Magic Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2866105 = {S98744, 138, { &P23617,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; RunboX5-W Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2866252 = {S98745, 156, { &P18464,&P18066,&P22587,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.0 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2867988 = {S98746, 147, { &P21334,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A8 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2869037 = {S98747, 80, { &P17595,&P17297,&P22169,&P18090 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:21.0) Gecko/X Firefox/21.0 slurp*/
const Device UA2869414 = {S98748, 150, { &P23773,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-N8010 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2869983 = {S98749, 153, { &P19861,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I605 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2877243 = {S98750, 137, { &P12697,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; shooteru Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2890462 = {S98751, 184, { &P18510,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; ALCATEL_ONE_TOUCH_918N Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2891446 = {S98752, 97, { &P21039,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Spice_Mi-355) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2892638 = {S98753, 151, { &P18635,&P18598,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 4 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2892743 = {S98754, 137, { &P18316,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F160L Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892953 = {S98755, 134, { &P21007,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; P-06D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893064 = {S98756, 136, { &P20963,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SBM200SH Build/S1002) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2894720 = {S98757, 156, { &P19911,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E970 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2897089 = {S98758, 160, { &P19759,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; Lenovo K860i Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2934232 = {S98759, 37, { &P16003,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (GT-S5233A/S523AJPJG3)*/
const Device UA2936789 = {S98760, 169, { &P19762,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI MT1-U06 Build/HuaweiMT1-U06) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2937050 = {S98761, 90, { &P18448,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST26i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2937131 = {S98762, 156, { &P16292,&P18598,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Galaxy Nexus Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2937550 = {S98763, 137, { &P20592,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-SU870 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937710 = {S98764, 137, { &P21438,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F120K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937715 = {S98765, 138, { &P19979,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHV-E170K Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937750 = {S98766, 160, { &P18451,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT26ii Build/6.2.B.0.200) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2937801 = {S98767, 130, { &P13006,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; folio100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2937865 = {S98768, 130, { &P20733,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAB-1030 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2937872 = {S98769, 159, { &P12681,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Nexus S 4G Build/JRO03R) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2939028 = {S98770, 99, { &P14980,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-b7722) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2940151 = {S98771, 138, { &P24164,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CROSS A7S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2950981 = {S98772, 57, { &P13633,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; A953 Build/JZO54K)*/
const Device UA2951814 = {S98773, 66, { &P18633,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; U8185; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2951946 = {S98774, 137, { &P20410,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; he-il; MAGPAD9.7 Pro Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2952265 = {S98775, 123, { &P21853,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Y3 Build/HCL) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2953333 = {S98776, 95, { &P14202,&P5477,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; zh-CN; E15i) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile*/
const Device UA2953386 = {S98777, 158, { &P14161,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-VM670 Build/JDQ39; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2956809 = {S98778, 88, { &P17595,&P17294,&P22169,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:21.0) Gecko/X Firefox/21.0 TenFourFox/7450*/
const Device UA2957284 = {S98779, 155, { &P14967,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.3; LG-P500 Build/GRI40) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2957396 = {S98780, 137, { &P18474,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9210T Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2957567 = {S98781, 162, { &P12867,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.4; LG-MS910 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2957737 = {S98782, 137, { &P23518,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZTE V987 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2957776 = {S98783, 157, { &P15759,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; HTC Vision Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2957807 = {S98784, 158, { &P15759,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.3; HTC Vision Build/GRI40) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453*/
const Device UA2958214 = {S98785, 137, { &P20525,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A898 Duo Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2958342 = {S98786, 153, { &P22609,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S6312 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2958446 = {S98787, 154, { &P14967,&P5096,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; LG-P500 Build/FRG83) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2958451 = {S98788, 152, { &P12968,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26i Build/6.1.A.2.45) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2960358 = {S98789, 130, { &P20523,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P3105 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2962913 = {S98790, 90, { &P12856,&P17483,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t300) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2963383 = {S98791, 153, { &P20117,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; IQ440 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2977432 = {S98792, 84, { &P15364,&P7133,&P18200,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.7.12) Gecko/X Thunderbird/1.0.7*/
const Device UA2978661 = {S98793, 126, { &P15364,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL32EX520; CC/XGA; en) Presto/2.7.61 Version/11.00*/
const Device UA2983634 = {S98794, 141, { &P18226,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; U9500 Build/HuaweiU9500) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2983779 = {S98795, 165, { &P20094,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2984192 = {S98796, 136, { &P20452,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; S8800 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2985627 = {S98797, 97, { &P16552,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; Micromax_A70) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2986939 = {S98798, 143, { &P23147,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Lenovo S720i Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2988667 = {S98799, 136, { &P12784,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; Skate Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2988892 = {S98800, 93, { &P15649,&P5096,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; zh-CN; GT-S5570) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile*/
const Device UA2989349 = {S98801, 141, { &P22003,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB A93.2 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2989924 = {S98802, 171, { &P13991,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; ADR6400L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (ADR6400L, 42, 1.9.1)*/
const Device UA2990276 = {S98803, 138, { &P23582,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; fnac 4.5 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2991667 = {S98804, 169, { &P12814,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I777 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2993193 = {S98805, 185, { &P18021,&P22212,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; IM-A770K Build/->Wangsai_008; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA2993415 = {S98806, 155, { &P23773,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N8010 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA2993838 = {S98807, 157, { &P20180,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; XOLO_A800 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2993872 = {S98808, 158, { &P13051,&P5096,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-CN; ZTE-T U880 Build/FRG83G) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2994310 = {S98809, 97, { &P22027,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; Videocon_A20) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2995669 = {S98810, 109, { &P14519,&P5129,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) NokiaN97 mini*/
const Device UA2995680 = {S98811, 76, { &P17595,&P21721,&P20794,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:7.0.1) Gecko/X Postbox/3.0.8*/
const Device UA2996048 = {S98812, 169, { &P20884,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; Lenovo A298t Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2996460 = {S98813, 161, { &P21321,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9228 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2996862 = {S98814, 90, { &P18303,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; MT27i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2996900 = {S98815, 157, { &P21598,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar_Touch_KEM_452_HDMI Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2997359 = {S98816, 152, { &P20290,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A11 Build/KarbonnA11) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2997485 = {S98817, 155, { &P21267,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A798t Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2998035 = {S98818, 171, { &P19770,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; ALCATEL ONE TOUCH 5020D Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2998444 = {S98819, 159, { &P17519,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; T8830Pro Build/HuaweiT8830Pro) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2998563 = {S98820, 147, { &P18742,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; R807 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2998694 = {S98821, 126, { &P23824,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; M870 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2998785 = {S98822, 156, { &P22020,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9268 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2998793 = {S98823, 151, { &P20300,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; W60 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2998906 = {S98824, 160, { &P18195,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; YP-G70 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2999063 = {S98825, 156, { &P22596,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2999381 = {S98826, 152, { &P20206,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE N880E Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2999405 = {S98827, 151, { &P22853,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HTC 802w Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2999497 = {S98828, 136, { &P22642,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Ctrl_V3 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2999645 = {S98829, 159, { &P18451,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT26ii Build/6.2.B.0.200) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2999810 = {S98830, 161, { &P12867,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LG-MS910 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3000207 = {S98831, 80, { &P17595,&P18207,&P22170,&P18090 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:22.0) Gecko/X Firefox/22.0 slurp*/
const Device UA3000464 = {S98832, 99, { &P19862,&P18598,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Micromax A110Q) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA3001250 = {S98833, 154, { &P14969,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Desire S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3001781 = {S98834, 154, { &P24504,&P18598,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Lenovo A830 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3003604 = {S98835, 170, { &P20002,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ALCATEL ONE TOUCH 992D Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3005205 = {S98836, 100, { &P13183,&P17485,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk810i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3006106 = {S98837, 163, { &P23716,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI A199 Build/HuaweiA199) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3006172 = {S98838, 161, { &P14225,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3006356 = {S98839, 156, { &P21321,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-I9228 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3006678 = {S98840, 173, { &P20192,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3007714 = {S98841, 166, { &P23709,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI G525-U00 Build/HuaweiG525-U00) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3010801 = {S98842, 157, { &P21068,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; MI-ONE C1 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3010977 = {S98843, 162, { &P11953,&P17485,&P24678,&P18092 }};	/*Useragent - SonyEricssonU10i/R7BA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonu10i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3011234 = {S98844, 155, { &P18621,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-E612 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3012207 = {S98845, 162, { &P18581,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; SCH-S720C Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3012785 = {S98846, 88, { &P21064,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A44) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3012797 = {S98847, 155, { &P20202,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-L38C Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3014236 = {S98848, 165, { &P12814,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I777 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3014710 = {S98849, 92, { &P18241,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; LG-E400) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3015624 = {S98850, 93, { &P18595,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I8190) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3016335 = {S98851, 100, { &P13179,&P17485,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw350i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3016993 = {S98852, 100, { &P14916,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung gt-c3303i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3017245 = {S98853, 97, { &P18462,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; HTC Desire X) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3018863 = {S98854, 90, { &P13018,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MT11i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3019126 = {S98855, 150, { &P21792,&P18598,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9200 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3019702 = {S98856, 95, { &P14842,&P5477,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E10i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3021367 = {S98857, 137, { &P20597,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F100S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021374 = {S98858, 137, { &P18174,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; VS920 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3021673 = {S98859, 174, { &P23245,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A3+ Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3022280 = {S98860, 135, { &P21379,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SH631W Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022384 = {S98861, 142, { &P21856,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax P600 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024112 = {S98862, 125, { &P24047,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; M785 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3024939 = {S98863, 149, { &P18511,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; URBANO PROGRESSO Build/011.0.3100) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3027968 = {S98864, 146, { &P21802,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GSmart Maya M1 v2 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3043616 = {S98865, 135, { &P21936,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; zh-tw; ThL W6 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3044192 = {S98866, 133, { &P15184,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; es-es; LG-P920 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3045513 = {S98867, 147, { &P23202,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-; aigoPad M80E Build/E8HD-ELITE-1280x768) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3046565 = {S98868, 152, { &P16053,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; UNUSUAL Vortex Pocket Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3046624 = {S98869, 93, { &P18712,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Next8P12) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3047744 = {S98870, 166, { &P18679,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-B5330 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3048268 = {S98871, 105, { &P15373,&P5129,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) Nokia5530*/
const Device UA3049119 = {S98872, 156, { &P15397,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; ST18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA3049344 = {S98873, 144, { &P20262,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Karbonn_A18 Build/KarbonnA18) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3049518 = {S98874, 153, { &P14090,&P5096,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-US; GT-I5801 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3049716 = {S98875, 153, { &P23150,&P18598,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Q1000 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3049770 = {S98876, 81, { &P20999,&P17484,&P17374,&P18092 }};	/*Useragent - SAMSUNG-GT-S5292 Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10*/
const Device UA3049870 = {S98877, 166, { &P20931,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; iBall Slide 3G7271 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3050408 = {S98878, 153, { &P20471,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; CONNECT 2G Build/HCL ME Tablet CONNECT 2G) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3052517 = {S98879, 137, { &P24149,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Aqua I-5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3055853 = {S98880, 90, { &P20175,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; zh-CN; MI 1S) U2/1.0.0 UCBrowser/9.1.1.309 U2/1.0.0 Mobile*/
const Device UA3056446 = {S98881, 160, { &P19851,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax A68 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056711 = {S98882, 98, { &P21020,&P18598,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Micromax A116) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3056748 = {S98883, 160, { &P20250,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Celkon A 200 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056843 = {S98884, 178, { &P15228,&P5043,&P22881,&P18092 }};	/*Useragent - NokiaC2-03/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291*/
const Device UA3056911 = {S98885, 161, { &P22621,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax P360 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056961 = {S98886, 152, { &P14997,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; S100 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3057083 = {S98887, 89, { &P21963,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Q800) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3057145 = {S98888, 164, { &P21611,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3057808 = {S98889, 176, { &P22585,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SPH-L720 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/jfltespr*/
const Device UA3058148 = {S98890, 172, { &P22641,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; C1504 Build/11.3.A.0.47) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3058698 = {S98891, 190, { &P23773,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N8010 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, +0100, gb*/
const Device UA3060876 = {S98892, 154, { &P22727,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGP312 Build/10.1.C.0.370) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3070586 = {S98893, 63, { &P15397,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; Xperia Ray Build/JZO54K)*/
const Device UA3070793 = {S98894, 161, { &P16273,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-T679 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3070832 = {S98895, 42, { &P11700,&P17485,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SonyEricssonW810i/R4EA031)*/
const Device UA3071400 = {S98896, 96, { &P19973,&P5043,&P17347,&P18092 }};	/*Useragent - Nokia114/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia114/UC Browser8.0.3.107/69/444*/
const Device UA3071749 = {S98897, 142, { &P15060,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3073401 = {S98898, 139, { &P23810,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Next7C12 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3073919 = {S98899, 92, { &P22748,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; LG-E425) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3073956 = {S98900, 140, { &P22043,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon*/
const Device UA3074004 = {S98901, 145, { &P20935,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; ZTE-Z990G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA3074223 = {S98902, 156, { &P21321,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9228 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3074245 = {S98903, 147, { &P18488,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SPH-L300 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3074356 = {S98904, 144, { &P24066,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Lenovo A706_ROW Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3074407 = {S98905, 151, { &P21532,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; vivo Y11 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3078386 = {S98906, 150, { &P20158,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A7 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3078413 = {S98907, 136, { &P21739,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; iris454 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3078706 = {S98908, 169, { &P21261,&P18598,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9500 Build/JDQ39) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3079558 = {S98909, 162, { &P15438,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Sensation Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3085749 = {S98910, 148, { &P17281,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SCH-I905 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3085902 = {S98911, 70, { &P18623,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;NOKIA;Lumia 510;7.10;8779)*/
const Device UA3086659 = {S98912, 155, { &P23771,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8190N Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088206 = {S98913, 155, { &P16598,&P18066,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A500 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.289*/
const Device UA3088449 = {S98914, 163, { &P18224,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HUAWEI U8815 Build/HuaweiU8815C02B952) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3088474 = {S98915, 148, { &P19849,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-S7560M Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3088547 = {S98916, 186, { &P17835,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Barnes & Noble Nook Tablet Build/JDQ39E; LiquidSmooth-Liquid-JB-v2.4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088795 = {S98917, 152, { &P18680,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWI Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA3089219 = {S98918, 134, { &P22602,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; SCH-S738C Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3091804 = {S98919, 134, { &P21059,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; id-id; A90 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3091871 = {S98920, 155, { &P21472,&P18111,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; A111 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300*/
const Device UA3092453 = {S98921, 134, { &P21878,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; 201F Build/V35R13E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3092518 = {S98922, 156, { &P18437,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-N8000 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3092589 = {S98923, 149, { &P18554,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BNTV400 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3092987 = {S98924, 161, { &P23505,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Vodafone 875 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3093100 = {S98925, 154, { &P23179,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ZP980 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3093385 = {S98926, 160, { &P19851,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax A68 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3100172 = {S98927, 76, { &P21351,&P18111,&P24671,&P18092 }};	/*Useragent - JUC (Linux; U; 4.1.1; zh-cn; HUAWEI C8813; 480*854) UCWEB8.5.1.179/145/33523*/
const Device UA3101664 = {S98928, 142, { &P20181,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0; Swipe_Fablet_F1 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3102948 = {S98929, 134, { &P22855,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; IQ430 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3103069 = {S98930, 160, { &P23919,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.4; R800i Build/4.0.2.A.0.42) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA3103578 = {S98931, 133, { &P23451,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; T720 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3106274 = {S98932, 158, { &P21261,&P18598,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9500 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.289*/
const Device UA3106921 = {S98933, 130, { &P13452,&P5096,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3107663 = {S98934, 135, { &P13018,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-in; MT11i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3107727 = {S98935, 97, { &P23017,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Spice_Mi-502) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3107754 = {S98936, 93, { &P23004,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; TECNO_D3) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3107800 = {S98937, 139, { &P21973,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Aqua flash Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3108056 = {S98938, 155, { &P21267,&P18066,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A798t Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3108078 = {S98939, 80, { &P22667,&P16110,&P17201,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; Xbox; Xbox One)*/
const Device UA3108157 = {S98940, 90, { &P15147,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; ST15i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3108164 = {S98941, 156, { &P20587,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P715 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3108308 = {S98942, 157, { &P22610,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6810 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3108360 = {S98943, 154, { &P16503,&P5204,&P24679,&P18092 }};	/*Useragent - Nokia7310/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7310) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3108370 = {S98944, 159, { &P14993,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LT15i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3108435 = {S98945, 162, { &P11698,&P17485,&P24679,&P18092 }};	/*Useragent - SonyEricssonW995/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw995) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3108499 = {S98946, 99, { &P12273,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-m5650) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3108765 = {S98947, 161, { &P18700,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-B5512 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3108806 = {S98948, 155, { &P18621,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-E612 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3108931 = {S98949, 151, { &P24062,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; L3 Build/TECNOL3) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109040 = {S98950, 96, { &P21881,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Karbonn_A4_) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3109076 = {S98951, 149, { &P18595,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I8190 Build/JRO03H.I8190XXALJL) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3109253 = {S98952, 165, { &P19863,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3109350 = {S98953, 169, { &P21261,&P18598,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9500 Build/JDQ39) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3109648 = {S98954, 101, { &P21857,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; HUAWEI_Y300-0100) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3109929 = {S98955, 156, { &P24149,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; Aqua I-5 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3110546 = {S98956, 136, { &P14017,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P990 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110743 = {S98957, 161, { &P14877,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3111022 = {S98958, 129, { &P18708,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U30GT-H Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3112100 = {S98959, 131, { &P24524,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; V812 Core4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3112265 = {S98960, 132, { &P22024,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; dL1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3113516 = {S98961, 157, { &P18280,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC One S Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3113542 = {S98962, 155, { &P20202,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LG-L38C Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3113525 = {S98963, 155, { &P14574,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Nexus S Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3114896 = {S98964, 158, { &P15539,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; HTC Glacier Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3118182 = {S98965, 125, { &P14912,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung gt-c3222) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA3121510 = {S98966, 166, { &P12280,&P22882,&P23594,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko, M5WebKit/1.0.2, M5HybridSDK/1.0.0) Version/5.1.1 Mobile/7.0 Mammoth/5.1.1*/
const Device UA3121710 = {S98967, 142, { &P22992,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; CloudPad 700d Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3125119 = {S98968, 129, { &P19775,&P18598,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.1; en-in; A210 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3125144 = {S98969, 170, { &P18471,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT907 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/xt907*/
const Device UA3125251 = {S98970, 92, { &P18621,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; LG-E612) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3125361 = {S98971, 157, { &P18295,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC One X Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3125726 = {S98972, 88, { &P23207,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A4+) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3125776 = {S98973, 103, { &P12588,&P5055,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac6-01) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3125868 = {S98974, 98, { &P12375,&P5890,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIANokia N81) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3125915 = {S98975, 90, { &P17625,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST25i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3125954 = {S98976, 90, { &P15047,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; SK17i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3126136 = {S98977, 170, { &P22396,&P18111,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5312 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3126189 = {S98978, 145, { &P13064,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SAMSUNG-SGH-I727 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3126307 = {S98979, 101, { &P15483,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_ChaCha_A810e) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3126317 = {S98980, 184, { &P18472,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.6.1;FBBV/X;FBDM/{density=2.0,width=720,height=1184};FBLC/en_US;FBCR/Verizon Wireless;FBPN/com.facebook.katana;FBDV/DROID RAZR HD;FBSV/4.1.2;FBCA/armeabi-v7a:armeabi;]*/
const Device UA3126404 = {S98981, 95, { &P21358,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.8; en-US; Andi3.5KKe) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3126467 = {S98982, 165, { &P22612,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI T8951 Build/HuaweiT8951) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126481 = {S98983, 136, { &P23163,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-I959 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126511 = {S98984, 137, { &P20592,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-SU870 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126778 = {S98985, 155, { &P18687,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; LG-P705 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127112 = {S98986, 138, { &P19979,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E170K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127140 = {S98987, 158, { &P24132,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; iris504Q Build/iris504Q) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127298 = {S98988, 137, { &P22594,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-M919 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3127341 = {S98989, 157, { &P22609,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S6312 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3127391 = {S98990, 157, { &P23771,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8190N Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127425 = {S98991, 167, { &P15877,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT910 Build/9.8.2O-124_SPUEM-14) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3127532 = {S98992, 158, { &P18295,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC One X Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3128429 = {S98993, 147, { &P20129,&P18111,&P24683,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.5 Mobile Safari/533.1*/
const Device UA3130588 = {S98994, 136, { &P23231,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E988 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3130814 = {S98995, 152, { &P15785,&P5843,&P24680,&P18092 }};	/*Useragent - Nokia201/2.0 (11.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia201) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3130952 = {S98996, 145, { &P13064,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131401 = {S98997, 138, { &P21065,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; CINK FIVE Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131450 = {S98998, 152, { &P20192,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Orange Daytona Build/HuaweiG510-0200) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131583 = {S98999, 137, { &P24391,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8260 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131741 = {S99000, 140, { &P18223,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132382 = {S99001, 145, { &P12657,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC EVO 3D X515m Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA1187 = {S99002, 137, { &P15366,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; HTC Tattoo Build/FLAN) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1189 = {S99003, 139, { &P15366,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; HTC Tattoo Build/ERD35B) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA17090 = {S99004, 138, { &P15366,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; HTC Tattoo Build/ERD72) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA17094 = {S99005, 138, { &P15366,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; HTC Tattoo Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA38999 = {S99006, 146, { &P15366,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; HTC Tattoo Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA21164 = {S99007, 79, { &P11973,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-F110/F110XAHB3 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA5053 = {S99008, 131, { &P13340,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GC900/V10a Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA20171 = {S99009, 158, { &P13340,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-GC900/V10b Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA78053 = {S99010, 158, { &P13340,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-GC900/V10c Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA6937 = {S99011, 165, { &P13815,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android WildPuzzleROM v8.0.7 froyo 2.2; fr-fr; HTC Wildfire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32289 = {S99012, 166, { &P13815,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android WildPuzzleROM v8.0.11 FRF91 2.2; en-gb; HTC Wildfire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32387 = {S99013, 166, { &P13815,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android WildPuzzleROM v8.0.10 FRF91 2.2; ru-ru; HTC Wildfire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1875276 = {S99014, 148, { &P13815,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android SUnAaBh.v1.1; en-gb; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20121 = {S99015, 51, { &P12383,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-E2000/1013 UP.Browser/7.2.6.1.567 (GUI) MMP/2.0*/
const Device UA20415 = {S99016, 51, { &P12383,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-E2000/1022 UP.Browser/7.2.6.1.537 (GUI) MMP/2.0*/
const Device UA21495 = {S99017, 52, { &P12383,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-E2000/10086 UP.Browser/7.2.6.1.537 (GUI) MMP/2.0*/
const Device UA30983 = {S99018, 51, { &P12383,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-E2000/1017 UP.Browser/7.2.6.1.537 (GUI) MMP/2.0*/
const Device UA58809 = {S99019, 51, { &P12383,&P17017,&P3487,&P18092 }};	/*Useragent - KWC-E2000/1016 UP.Browser/7.2.6.1.537 (GUI) MMP/2.0*/
const Device UA20270 = {S99020, 15, { &P15059,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHC300/1.0*/
const Device UA21288 = {S99021, 109, { &P12795,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; MDA compact/3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1)*/
const Device UA22178 = {S99022, 162, { &P15394,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; fr-fr; Tactile_internet-orange-LS Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA22377 = {S99023, 111, { &P15446,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHB320/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA22393 = {S99024, 111, { &P15451,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHM120/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA64452 = {S99025, 89, { &P14055,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KG200j/V01 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1 LG-KG200j/V01 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA24474 = {S99026, 87, { &P15691,&P17017,&P4780,&P18092 }};	/*Useragent - Nexian NX-T909/MTK Release/10.01.2009 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1*/
const Device UA26143 = {S99027, 68, { &P12155,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V980/80.2E.27I MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA26146 = {S99028, 69, { &P12155,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V980M/83.28.38. MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24742 = {S99029, 112, { &P15752,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHB220B/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA27579 = {S99030, 78, { &P12101,&P5026,&P3410,&P18092 }};	/*Useragent - BlackBerry8703e/4.5.0.148 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/104*/
const Device UA83403 = {S99031, 17, { &P12441,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonK310i*/
const Device UA32395 = {S99032, 154, { &P12484,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android SUnAaBh.v1.1; en-gb; HTC_Wildfire_A3333 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA32411 = {S99033, 184, { &P12484,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android WildPuzzleClearROM qmod02 base_1.37.405.1; ru-ru; HTC_Wildfire_A3333 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA73442 = {S99034, 171, { &P12484,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android WildPuzzleROM WWE_1.25.405.1; en-gb; HTC_Wildfire_A3333 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA33493 = {S99035, 71, { &P12096,&P5448,&P22272,&P18092 }};	/*Useragent - HTC_Hermes Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA86564 = {S99036, 127, { &P12096,&P5448,&P22272,&P18092 }};	/*Useragent - PPC; 240x320; HTC_TyTN/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA41273 = {S99037, 71, { &P12422,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V220ENS/0B.D1.32R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA51290 = {S99038, 68, { &P12422,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V220/0B.D1.09R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA52787 = {S99039, 81, { &P13298,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonK510i/R4EA Java/SEMC-Java/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA44033 = {S99040, 54, { &P15674,&P17017,&P3625,&P18092 }};	/*Useragent - KWC-K127/1204 UP.Browser/6.2.3.9.g.1.110 (GUI) MMP/2.0*/
const Device UA56500 = {S99041, 55, { &P15674,&P17017,&P3625,&P18092 }};	/*Useragent - KWC-K127/11031 UP.Browser/6.2.3.9.g.1.110 (GUI) MMP/2.0*/
const Device UA84520 = {S99042, 54, { &P15674,&P17017,&P3625,&P18092 }};	/*Useragent - KWC-K127/1002 UP.Browser/6.2.3.9.g.1.110 (GUI) MMP/2.0*/
const Device UA60008 = {S99043, 13, { &P16572,&P17483,&P17660,&P18092 }};	/*Useragent - LG/U8110/v2.0*/
const Device UA49224 = {S99044, 139, { &P12964,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HTC-S710d/1.30.1401.3) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA70315 = {S99045, 139, { &P12964,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HTC-S710d/1.43.1401.5) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA53464 = {S99046, 69, { &P13982,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-R90/1.0 ACS-NF/3.4 QTV2.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61152 = {S99047, 102, { &P12645,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-M140L/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA61241 = {S99048, 73, { &P12664,&P6476,&P17470,&P18092 }};	/*Useragent - LG-LX400 TELECA-LX400/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62731 = {S99049, 41, { &P14442,&P22832,&P17470,&P18092 }};	/*Useragent - Videocon_V1435/WAP2.0 Profile/MT6253/QCIF*/
const Device UA79564 = {S99050, 84, { &P14442,&P22832,&P17470,&P18092 }};	/*Useragent - Videocon_V1435/WAP2.0 Profile/MT6253/QCIF, Videocon_V1435/WAP2.0 Profile/MT6253/QCIF*/
const Device UA62784 = {S99051, 98, { &P14449,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-U908/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA67084 = {S99052, 116, { &P11679,&P5776,&P4715,&P18092 }};	/*Useragent - Nokia6630/1.0 (5.03.08) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.4.0.65/27/351*/
const Device UA68012 = {S99053, 81, { &P13770,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-C6112/C6112DDJB2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA68093 = {S99054, 80, { &P12274,&P5448,&P4759,&P18092 }};	/*Useragent - SAMSUNG-SGH-i900/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.6.1.82*/
const Device UA68102 = {S99055, 82, { &P15207,&P5448,&P4759,&P18092 }};	/*Useragent - HTC_P3400i-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.6.1.82*/
const Device UA68113 = {S99056, 46, { &P14033,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaC5-03/SymbianOS/9.1 Series60/3.0 3gpp-gba*/
const Device UA80230 = {S99057, 60, { &P14125,&P6478,&P4420,&P18092 }};	/*Useragent - BlackBerry7100/4.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA85854 = {S99058, 77, { &P18095,&P16000,&P18397,&P18092 }};	/*Useragent - Mozilla/4.1 (compatible; MSIE 5.0; Symbian OS; Series 60;424) Opera 6.10 [en]*/
const Device UA89723 = {S99059, 71, { &P21391,&P17017,&P3807,&P18092 }};	/*Useragent - K444/1.0 MTK/MAUI.08A.W08.28.MP.V2 Release/7.30.2009 Browser/OBIGO-Q03C*/
const Device UA104011 = {S99060, 88, { &P15364,&P5690,&P18355,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322) Sleipnir/2.21*/
const Device UA450612 = {S99061, 35, { &P13602,&P17484,&P4777,&P18092 }};	/*Useragent - gt-s5233w/UC Browser7.7.1.88/69/444*/
const Device UA490566 = {S99062, 12, { &P13443,&P17484,&P17606,&P18092 }};	/*Useragent - SamsungI8910*/
const Device UA503997 = {S99063, 141, { &P16039,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; zh-cn; MOT-A1680 Build/DRD35) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA509900 = {S99064, 90, { &P17595,&P18120,&P20902,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.7.2) Gecko/X Netscape/7.2 (ax)*/
const Device UA512719 = {S99065, 116, { &P15364,&P17188,&P17240,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 4.0; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.38 Safari/532.0*/
const Device UA523208 = {S99066, 43, { &P17032,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 934SH)*/
const Device UA552983 = {S99067, 102, { &P15364,&P17813,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.18pre) Gecko/X Ubuntu/8.04 (hardy) Namoroka/3.6.18pre*/
const Device UA2019841 = {S99068, 76, { &P14090,&P5477,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.1-update1;Zh_cn;GT-I5801;240*400;)UCWEB7.8.0.95/140/444*/
const Device UA574312 = {S99069, 61, { &P15364,&P18110,&P18193,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:7.0) Gecko/X Thunderbird/7.0*/
const Device UA624425 = {S99070, 75, { &P12871,&P18110,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.3;Zh_cn;HTC Wildfire S A510e;320*480;)UCWEB7.8.1.96/139/800*/
const Device UA625252 = {S99071, 124, { &P15364,&P7133,&P24207,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.7.10246.201*/
const Device UA849027 = {S99072, 185, { &P15364,&P7133,&P24207,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C) QQBrowser/6.7.10246.201*/
const Device UA660391 = {S99073, 163, { &P15555,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; myTouch_4G_Slide Build/GRJ22; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA685800 = {S99074, 66, { &P15364,&P5690,&P22181,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:2.2a1pre) Gecko/X Firefox/4.2a1pre*/
const Device UA695396 = {S99075, 106, { &P14743,&P17485,&P17347,&P18092 }};	/*Useragent - SonyEricssonJ20i/R7CA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonj20i/UC Browser8.0.3.107/69/444*/
const Device UA707902 = {S99076, 67, { &P14969,&P18110,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.3;Zh_cn;HTC Desire S;480*800;)UCWEB7.8.1.96/139/800*/
const Device UA708646 = {S99077, 74, { &P18182,&P5448,&P24667,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB8.2.0.116/31/999*/
const Device UA726285 = {S99078, 60, { &P15364,&P7127,&P18368,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:8.0) Gecko/X Thunderbird/8.0*/
const Device UA729998 = {S99079, 75, { &P15364,&P18110,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; Opera Mobi/10; U; en) Presto/2.7.81 Version/11.00*/
const Device UA843124 = {S99080, 55, { &P17595,&P18120,&P18098,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X; U; en) Opera 8.51*/
const Device UA896546 = {S99081, 96, { &P17595,&P17295,&P18193,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:7.0.1) Gecko/X Thunderbird/7.0.1 Lightning/1.0b7*/
const Device UA925912 = {S99082, 95, { &P16567,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; Acer; M310)*/
const Device UA1869472 = {S99083, 93, { &P15364,&P7133,&P22488,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.8.1.8) Gecko/X Firefox/2.0.0.8 Flock/1.0*/
const Device UA1899892 = {S99084, 82, { &P15364,&P17285,&P22347,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; WOW64; rv:6.0.2) Gecko/X Firefox/6.0.2 PaleMoon/6.0.2*/
const Device UA1904173 = {S99085, 47, { &P14748,&P5890,&P17338,&P18092 }};	/*Useragent - nokiae72-1/UC Browser7.9.0.102/69/352  3gpp-gba*/
const Device UA1973324 = {S99086, 53, { &P11929,&P5890,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0 (Nokia5230;SymbianOS/9.1 Series60/3.0)*/
const Device UA2684462 = {S99087, 85, { &P14864,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s8500) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2041345 = {S99088, 97, { &P15364,&P16110,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/21.0.1180.89)*/
const Device UA2196266 = {S99089, 90, { &P15364,&P16110,&P21912,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; chromeframe/21.0.1180.89)*/
const Device UA2019347 = {S99090, 144, { &P21672,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; i-mobile I-Note Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019247 = {S99091, 86, { &P17976,&P6437,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.2; Linux; Opera Tablet/ADR-X; U; en) Presto/2.7.81 Version/11.00*/
const Device UA2019955 = {S99092, 88, { &P17976,&P6437,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Android 3.2.1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.7.81 Version/11.00*/
const Device UA2022481 = {S99093, 123, { &P22557,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; zh-cn; HTC A6390 Build/GRI40) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA3117118 = {S99094, 135, { &P22557,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; zh-cn; HTC A6390 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033231 = {S99095, 128, { &P22242,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; ME171 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2024440 = {S99096, 136, { &P13286,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; DROID2 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2028334 = {S99097, 154, { &P18720,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; Micromax A50 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2129412 = {S99098, 89, { &P15364,&P7127,&P18199,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.8.1.24) Gecko/X Thunderbird/2.0.0.24*/
const Device UA2150631 = {S99099, 37, { &P12061,&P5890,&P17347,&P18092 }};	/*Useragent - nokiae65-1/UC Browser8.0.3.107/69/444*/
const Device UA2206877 = {S99100, 165, { &P15364,&P5690,&P23487,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; chrome://navigator/locale/navigator.properties; rv:1.8.1.21) Gecko/X SecondLife/1.23.5.X (Second Life Release; default skin)*/
const Device UA2210746 = {S99101, 87, { &P15439,&P5448,&P4737,&P18092 }};	/*Useragent - SAMSUNG-SGH-i200/1.0 (compatible; MSIE 4.01; Windows CE; Smartphone)/UC Browser7.2.2.51*/
const Device UA2217169 = {S99102, 50, { &P15364,&P5890,&P21754,&P18092 }};	/*Useragent - iBrowser/2.4 (NokiaNXX;SymbianOS/9.1 Series60/3.0)*/
const Device UA2223194 = {S99103, 155, { &P12798,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2223260 = {S99104, 149, { &P14433,&P5096,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2280237 = {S99105, 132, { &P15364,&P17838,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.27 (KHTML, like Gecko) Ubuntu/9.10 Chromium/12.0.711.0 Chrome/12.0.711.0 Safari/534.27*/
const Device UA2286508 = {S99106, 136, { &P15364,&P17838,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.30 (KHTML, like Gecko) Ubuntu/9.10 Chromium/12.0.742.53 Chrome/12.0.742.53 Safari/534.30*/
const Device UA2304028 = {S99107, 59, { &P15364,&P17285,&P24686,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:7.0.1) Gecko/X Yoono/1.8.16*/
const Device UA2318118 = {S99108, 32, { &P12193,&P5129,&P24671,&P18092 }};	/*Useragent - NokiaC5-00/UCWEB8.5.0.163/28/800*/
const Device UA2336026 = {S99109, 251, { &P14711,&P5204,&P18524,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/70/352*/
const Device UA2339386 = {S99110, 84, { &P15118,&P5055,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian3; U; en-us; NokiaX7-00)/UC Browser8.5.0.183/51/352/UCWEB Mobile*/
const Device UA2346795 = {S99111, 87, { &P18283,&P18110,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Adr (Linux; U; 4.0.4; zh-cn; GT-I9300 Build/IMM76D.I9300ZNBLH3;720*1280)*/
const Device UA2372847 = {S99112, 149, { &P12280,&P17084,&P20786,&P18092 }};	/*Useragent - MQQBrowser/35 Mozilla/5.0 (iPhone 3GS; U; CPU iPhone OS 4_1 like Mac OS X; zh-cn) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B117 Safari/7534.48.3*/
const Device UA2373772 = {S99113, 81, { &P15364,&P17813,&P18101,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; de; rv:1.9.0.3) Gecko/X Ubuntu/8.04 Firefox/12.0*/
const Device UA2383067 = {S99114, 174, { &P13042,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC Incredible S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2383436 = {S99115, 171, { &P13123,&P18111,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2409156 = {S99116, 37, { &P13163,&P5129,&P17338,&P18092 }};	/*Useragent - nokian97-1/UC Browser7.9.0.102/69/352*/
const Device UA2409930 = {S99117, 126, { &P23659,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46EX723; CC/GBR; en) Presto/2.7.61 Version/11.00*/
const Device UA2410093 = {S99118, 43, { &P16366,&P17484,&P17338,&P18092 }};	/*Useragent - samsung-gt-s8600/UC Browser7.9.0.102/69/352*/
const Device UA2410738 = {S99119, 71, { &P18296,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.3;Zh_cn;HTC One V;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA2413884 = {S99120, 155, { &P18465,&P18066,&P19988,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFOT Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Silk/2.0 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA2427033 = {S99121, 170, { &P12280,&P18432,&P23595,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko, M5WebKit/1.0.1, M5HybridSDK/1.0.0) Version/5.0.4 Mobile/6.0.1 Mammoth/5.0.4*/
const Device UA2428106 = {S99122, 98, { &P14404,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s5260) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2440219 = {S99123, 54, { &P14983,&P5890,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 (NokiaC7-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA2443285 = {S99124, 154, { &P11787,&P5204,&P24667,&P18092 }};	/*Useragent - Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia3120classic) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2460294 = {S99125, 226, { &P15575,&P18066,&P18369,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2742816 = {S99126, 35, { &P12897,&P5843,&P21764,&P18092 }};	/*Useragent - iBrowser/Mini2.7 (NokiaC2-00/03.45)*/
const Device UA2469513 = {S99127, 90, { &P13734,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; gt-s5603) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2469980 = {S99128, 229, { &P19871,&P5629,&P18369,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6800 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3445; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2472078 = {S99129, 129, { &P19793,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JT-B1 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2494973 = {S99130, 100, { &P15837,&P6909,&P17338,&P18092 }};	/*Useragent - NokiaX2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-05/UC Browser7.9.0.102/69/352*/
const Device UA2495572 = {S99131, 85, { &P16366,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s8600) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2521725 = {S99132, 170, { &P12681,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S 4G Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2525068 = {S99133, 162, { &P15501,&P5477,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.53) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2545446 = {S99134, 86, { &P11996,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5560i) UCBrowser8.2.1.144/69/352/UCWEB Mobile*/
const Device UA2552198 = {S99135, 155, { &P17595,&P18120,&P18728,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.3.4.X (Second Life Release; default skin) Safari/533.3*/
const Device UA2577663 = {S99136, 169, { &P14993,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LT15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2577878 = {S99137, 152, { &P15911,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT862 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3043131 = {S99138, 117, { &P15518,&P17484,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; gt-s5230) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/405/UCWEB Mobile*/
const Device UA2594292 = {S99139, 169, { &P15517,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; MT15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2597333 = {S99140, 155, { &P13107,&P5043,&P24675,&P18092 }};	/*Useragent - NokiaC2-06/2.0 (07.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac2-06) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2621963 = {S99141, 29, { &P17168,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (GT-P7310 IMM76D)*/
const Device UA2607781 = {S99142, 69, { &P12889,&P6546,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android3.1;Zh_cn;GT-P7300;800*1280;)UCWEB7.8.0.95/139/355*/
const Device UA2607915 = {S99143, 118, { &P12889,&P6546,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android3.1;Zh_cn;GT-P7300;800*1280;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA2611557 = {S99144, 151, { &P14905,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7100 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2621968 = {S99145, 167, { &P14905,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; GT-P7100 Build/IMM76L; CyanogenMod-Rs Supremacy v2.0 CM9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616889 = {S99146, 89, { &P15985,&P6437,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (im; 1.8.3.13903); (Linux; U; Android 3.2; GT-P6200 Build/HONEYCOMB_MR2);*/
const Device UA2608290 = {S99147, 28, { &P14908,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (GT-P1010 FROYO)*/
const Device UA2608367 = {S99148, 29, { &P14908,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (GT-P6210 HTJ85B)*/
const Device UA2611876 = {S99149, 83, { &P13452,&P5096,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.4.5.12736); (Linux; U; Android 2.2; GT-P1000 Build/FROYO);*/
const Device UA2620263 = {S99150, 84, { &P13452,&P5096,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 2.2; GT-P1000R Build/FROYO);*/
const Device UA2620315 = {S99151, 84, { &P13452,&P5096,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 2.2; GT-P1000M Build/FROYO);*/
const Device UA2619657 = {S99152, 164, { &P15919,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3100, 36, 1.7.1)*/
const Device UA2620963 = {S99153, 164, { &P15919,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3100, 36, 1.7.1)*/
const Device UA2621561 = {S99154, 104, { &P15876,&P17979,&P22073,&P18092 }};	/*Useragent - Instagram 3.3.3 Android (15/4.0.3; 160dpi; 800x1232; samsung; GT-P5100; espresso10rf; espresso10; en_GB)*/
const Device UA2622152 = {S99155, 95, { &P13452,&P17979,&P22073,&P18092 }};	/*Useragent - Instagram 3.3.3 Android (8/2.2; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; GT-P1000; en_GB)*/
const Device UA2622905 = {S99156, 98, { &P13452,&P17979,&P22073,&P18092 }};	/*Useragent - Instagram 3.3.3 Android (10/2.3.3; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; gt-p1000; en_GB)*/
const Device UA2623592 = {S99157, 132, { &P17323,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P6800 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2633344 = {S99158, 131, { &P14908,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P6210 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2637857 = {S99159, 130, { &P14908,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P6210 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2651779 = {S99160, 81, { &P17595,&P18207,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.8.2; U; en) Presto/2.9.168 Version/11.50*/
const Device UA2656252 = {S99161, 46, { &P14764,&P17485,&P24673,&P18092 }};	/*Useragent - SonyEricssonU1UCBrowser/8.7.1.234/50/444/UCWEB*/
const Device UA2667971 = {S99162, 136, { &P18645,&P24698,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SPH-L710 Build/MIUI) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2686421 = {S99163, 74, { &P23532,&P17017,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; Id; tcl-c616) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2686594 = {S99164, 136, { &P20590,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; novo7_Aurora Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2692963 = {S99165, 151, { &P15047,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SK17i Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2693052 = {S99166, 116, { &P14568,&P17483,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; lg-c105) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2695622 = {S99167, 153, { &P21435,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V788D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2696069 = {S99168, 91, { &P15364,&P17486,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; mot-ex128) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2699234 = {S99169, 165, { &P21023,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; LG-E405 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2701051 = {S99170, 154, { &P15911,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; XT862 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2703401 = {S99171, 179, { &P14183,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SAMSUNG-SGH-I997 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2720821 = {S99172, 69, { &P15483,&P18110,&P20771,&P18092 }};	/*Useragent - MQQBrowser/20 (Linux; U; 2.3.4; zh-cn; HTC A810e Build/GRJ22;480*320)*/
const Device UA2721132 = {S99173, 66, { &P17999,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2.1;Zh_cn;T301;600*800;)UCWEB7.8.0.95/139/444*/
const Device UA2727015 = {S99174, 129, { &P23178,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; iDxD8 3G Build/ICS) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2727567 = {S99175, 33, { &P16518,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SAMSUNG GT-E2222)*/
const Device UA2728455 = {S99176, 140, { &P17108,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Inspire 4G Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2729188 = {S99177, 163, { &P13964,&P5096,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I5510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2730023 = {S99178, 162, { &P15201,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation Z710e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2733382 = {S99179, 165, { &P19863,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2735234 = {S99180, 134, { &P20255,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; R8111 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2735373 = {S99181, 135, { &P18283,&P24698,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I535 Build/MIUI) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2741954 = {S99182, 100, { &P15364,&P22186,&P18197,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.15) Gecko/X Fedora/3.0.10-1.fc12 Thunderbird/3.0.10*/
const Device UA2745473 = {S99183, 180, { &P11625,&P5768,&P24671,&P18092 }};	/*Useragent - Nokia5300/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5300) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2747119 = {S99184, 73, { &P20183,&P18110,&P24673,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; MI-ONE Plus; 480*854) UCWEB8.7.4.225/145/800*/
const Device UA2748567 = {S99185, 173, { &P17308,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; DROIDX Build/4.5.1_57_DX8-51) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2749861 = {S99186, 128, { &P22242,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ME171 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2762806 = {S99187, 157, { &P20183,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; MI-ONE Plus Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2762854 = {S99188, 154, { &P18303,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MT27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2775721 = {S99189, 115, { &P17595,&P17297,&P18106,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/ Safari/533.3*/
const Device UA2778076 = {S99190, 68, { &P18685,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; U8655-1; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2778194 = {S99191, 158, { &P11680,&P17483,&P24673,&P18092 }};	/*Useragent - LG-KM900/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-km900) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2780100 = {S99192, 161, { &P21321,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9228 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2780585 = {S99193, 183, { &P18472,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2780778 = {S99194, 165, { &P15798,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2788212 = {S99195, 182, { &P14482,&P5843,&P24671,&P18092 }};	/*Useragent - NokiaC3-01/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac3-01) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2790924 = {S99196, 118, { &P12089,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN80) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2958169 = {S99197, 100, { &P14851,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5753e) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2795739 = {S99198, 145, { &P23013,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AllviewAX2Frenzy Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2795924 = {S99199, 173, { &P18472,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; DROID RAZR HD Build/9.8.1Q-62_VQW_MR-2) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2803511 = {S99200, 81, { &P13578,&P5448,&P4777,&P18092 }};	/*Useragent - HTC_P3470 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2812142 = {S99201, 87, { &P21223,&P18110,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 (Linux; U; 2.3.6; zh-cn; HUAWEI C8810 Build/C8810V100R001C92B862;480*800)*/
const Device UA2814202 = {S99202, 44, { &P12832,&P17485,&P17347,&P18092 }};	/*Useragent - sonyericssonck13i/UC Browser8.0.3.107/69/444*/
const Device UA2814879 = {S99203, 132, { &P16319,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ru-ru; U8860 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2816693 = {S99204, 122, { &P18481,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-C3520/C3520JPKK3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA2817058 = {S99205, 114, { &P12059,&P5204,&P18273,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2690/09.70; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.0.5.16.11*/
const Device UA2822036 = {S99206, 121, { &P16600,&P22784,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; micromaxx600) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2825660 = {S99207, 80, { &P17595,&P18207,&P18428,&P18090 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:16.0) Gecko/X Firefox/16.0 slurp*/
const Device UA2829107 = {S99208, 161, { &P18652,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; Micromax A87 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2833368 = {S99209, 173, { &P20262,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; Karbonn_A18 Build/KarbonnA18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2834557 = {S99210, 173, { &P15649,&P5096,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-S5570 Build/FROYO; 240*320) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444*/
const Device UA2835670 = {S99211, 40, { &P12056,&P5204,&P21724,&P18092 }};	/*Useragent - OneBrowser/3.0 (Nokia7210Supernova/07.2)*/
const Device UA2837831 = {S99212, 71, { &P18213,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;GT-S6500D;320*480;)UCWEB7.8.0.95/139/351*/
const Device UA2838441 = {S99213, 160, { &P14090,&P5477,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I5801 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2839973 = {S99214, 153, { &P15028,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LG-P350 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2840232 = {S99215, 180, { &P11625,&P5768,&P24672,&P18092 }};	/*Useragent - Nokia5300/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5300) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2840292 = {S99216, 152, { &P15877,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; XT910 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2840451 = {S99217, 53, { &P18088,&P18432,&P24667,&P18092 }};	/*Useragent - IUC(U;iOS 6.0.1;Zh-cn;320*480;)/UCWEB8.2.1.132/42/997*/
const Device UA2842126 = {S99218, 89, { &P15364,&P16110,&P18199,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.8.1.24) Gecko/X Thunderbird/2.0.0.24*/
const Device UA2844063 = {S99219, 92, { &P15364,&P5768,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia3110c) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2844587 = {S99220, 138, { &P18002,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A760S Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2846130 = {S99221, 89, { &P15364,&P22843,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lava-a9) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2846930 = {S99222, 164, { &P23546,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; AD683G Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2847412 = {S99223, 64, { &P20300,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; W60; 480*800) UCWEB7.9.0.94/139/444*/
const Device UA2848787 = {S99224, 171, { &P16273,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2850440 = {S99225, 32, { &P12193,&P5129,&P24675,&P18092 }};	/*Useragent - NokiaC5-00/UCWEB8.8.0.212/28/999*/
const Device UA2850546 = {S99226, 142, { &P18085,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; Galaxy Gio Build/JELLYBEAN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2851731 = {S99227, 163, { &P21229,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; iball Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2852509 = {S99228, 170, { &P21850,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Spice Mi-435 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2856532 = {S99229, 138, { &P19748,&P5043,&P24670,&P18092 }};	/*Useragent - Nokia205/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia205) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2859821 = {S99230, 138, { &P23559,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AMOI N821 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2860377 = {S99231, 151, { &P15364,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; 7260 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2860853 = {S99232, 135, { &P20956,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; FPT HD Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2861066 = {S99233, 161, { &P22394,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; C6603 Build/10.1.A.1.434) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2861250 = {S99234, 157, { &P18295,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC One X Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2861665 = {S99235, 133, { &P21278,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; QW TB-9718D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2861796 = {S99236, 143, { &P22125,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; INFINITY_LOTUS Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2861806 = {S99237, 138, { &P18630,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A800S Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2863060 = {S99238, 158, { &P13051,&P5096,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-CN; ZTE-T U880 Build/FRG83G) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2864042 = {S99239, 139, { &P12280,&P18117,&P24445,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Janetter/1.5.1*/
const Device UA2864645 = {S99240, 95, { &P16600,&P22784,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx600) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2864759 = {S99241, 156, { &P18214,&P5629,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2864799 = {S99242, 90, { &P18449,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; ST27i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2865327 = {S99243, 143, { &P21617,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile IQ 1 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2868921 = {S99244, 156, { &P20123,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; ZTE U795 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2869259 = {S99245, 134, { &P18552,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; XT897 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892259 = {S99246, 155, { &P18448,&P18111,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; ST26i Build/11.2.A.0.21) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2892416 = {S99247, 130, { &P20149,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6211 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2892522 = {S99248, 133, { &P22940,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Xelio 7 pro Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2892824 = {S99249, 134, { &P21017,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; P-07D Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2892869 = {S99250, 134, { &P19790,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; L-04E Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2893387 = {S99251, 156, { &P22610,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6810 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2897416 = {S99252, 150, { &P23952,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GOCLEVER Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2932906 = {S99253, 114, { &P15364,&P17285,&P23684,&P18092 }};	/*Useragent - GoogleEarth/5.1.3533.1731(Windows;Microsoft Windows Vista (Service Pack 1);en-US;kml:2.2;client:Free;type:default)*/
const Device UA2933908 = {S99254, 125, { &P22987,&P17484,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung gt-c3303) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2935942 = {S99255, 161, { &P12841,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC PH39100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2937396 = {S99256, 135, { &P20343,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZP900H Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937527 = {S99257, 136, { &P20346,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MIDC407 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937626 = {S99258, 122, { &P15364,&P7133,&P22708,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.0 Safari/537.36 Sleipnir/4.2.9*/
const Device UA2938068 = {S99259, 162, { &P19842,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Butterfly Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2938216 = {S99260, 138, { &P23274,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TAB-PROTAB2-IPS9 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2938228 = {S99261, 133, { &P21752,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M030 Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2938708 = {S99262, 157, { &P18459,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VS950 4G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2950854 = {S99263, 169, { &P18666,&P5629,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; A25 Build/GRK39F; 240*320) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444*/
const Device UA2951326 = {S99264, 140, { &P24464,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Celkon A900 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2951554 = {S99265, 85, { &P17968,&P17979,&P21767,&P18092 }};	/*Useragent - OneBrowser/Mini1.1 (Android; Linux; Opera Mobi/27; U; en) Presto/2.4.18 Version/10.00*/
const Device UA2952783 = {S99266, 159, { &P20146,&P18111,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9082 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2953429 = {S99267, 136, { &P20416,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; F-04E Build/V14R31C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2954226 = {S99268, 177, { &P18125,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID4 Build/9.8.2O-72_VZW-18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (DROID4, 42, 1.9.1)*/
const Device UA2956311 = {S99269, 160, { &P21036,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Spice Mi-425 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2957681 = {S99270, 152, { &P21592,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Kem 432 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2957923 = {S99271, 132, { &P24553,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; X21 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2958199 = {S99272, 159, { &P18719,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; C1505 Build/11.3.A.0.47) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2958469 = {S99273, 157, { &P18382,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC Desire C Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2958583 = {S99274, 135, { &P21448,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SH530U Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2958590 = {S99275, 145, { &P20509,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MyPhone A888 Duo Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2958713 = {S99276, 133, { &P14997,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; S100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2961640 = {S99277, 135, { &P23094,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; F-06E Build/V19R47F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2977111 = {S99278, 164, { &P24064,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; pl-pl; SAMSUNG GT-S7275 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Version/1.0 Chrome/18.0.1025.308 Mobile Safari/535.19*/
const Device UA2978998 = {S99279, 37, { &P13163,&P5129,&P17347,&P18092 }};	/*Useragent - nokian97-1/UC Browser8.0.3.107/69/444*/
const Device UA2979060 = {S99280, 64, { &P14888,&P17361,&P4473,&P18092 }};	/*Useragent - BlackBerry9650/4.4.0.560 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2979553 = {S99281, 150, { &P13337,&P17485,&P24667,&P18092 }};	/*Useragent - SonyEricssonJ108i/R7EA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-in; sonyericssonj108i) UCBrowser8.2.0.132/69/452/UCWEB Mobile*/
const Device UA2981570 = {S99282, 51, { &P21383,&P18110,&P17810,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; Lenovo_A288t; 320*480)*/
const Device UA2982026 = {S99283, 140, { &P20475,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; I8 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982738 = {S99284, 35, { &P11808,&P5204,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia5610d-1/10.00)*/
const Device UA2982810 = {S99285, 152, { &P20512,&P5629,&P21729,&P18092 }};	/*Useragent - OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Fly F350 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982818 = {S99286, 119, { &P16808,&P22784,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; micromaxq7) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2983200 = {S99287, 139, { &P12280,&P18432,&P20784,&P18092 }};	/*Useragent - MQQBrowser/33 Mozilla/5.0 (iPhone; CPU iPhone OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3*/
const Device UA2983246 = {S99288, 123, { &P11996,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; fa; samsung-gt-s5560i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile*/
const Device UA2984189 = {S99289, 156, { &P21242,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A366t Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2984907 = {S99290, 131, { &P23804,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MID1060 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2985081 = {S99291, 160, { &P18083,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; V9C Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2985975 = {S99292, 155, { &P19777,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9105P Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2986297 = {S99293, 87, { &P17595,&P17295,&P21791,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.5; rv:19.0) Gecko/X Firefox/19.0 SeaMonkey/2.16*/
const Device UA2986340 = {S99294, 90, { &P15354,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; ST17i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2987012 = {S99295, 92, { &P18685,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; U8655-1) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2987125 = {S99296, 197, { &P12146,&P17483,&P24676,&P18092 }};	/*Useragent - LG-GU285/v10a Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lggu280) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2987706 = {S99297, 97, { &P20250,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Celkon_A_200) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2988135 = {S99298, 144, { &P20507,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; tablet Fnac 7 Build/2.0.0 X-10:24) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2988829 = {S99299, 155, { &P23547,&P18111,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; ZTE V889S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2988901 = {S99300, 152, { &P20255,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; R811 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2989064 = {S99301, 153, { &P23280,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Kem 402 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2989710 = {S99302, 93, { &P18509,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; LG-MS770) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2990077 = {S99303, 177, { &P15880,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB865 Build/6.7.2-226-EDICS-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (MB865, 42, 1.9.1)*/
const Device UA2990106 = {S99304, 174, { &P16273,&P5629,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (SGH-T679, 42, 1.9.1)*/
const Device UA2991449 = {S99305, 161, { &P14877,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; GT-I9000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2991621 = {S99306, 149, { &P12280,&P19826,&P21786,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/3.6.5/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA2992189 = {S99307, 135, { &P20357,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SmartPad7503G Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2993139 = {S99308, 107, { &P20981,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Micromax_P250(Funbook)) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2993305 = {S99309, 139, { &P12280,&P19826,&P20784,&P18092 }};	/*Useragent - MQQBrowser/33 Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3*/
const Device UA2993481 = {S99310, 182, { &P13157,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2993833 = {S99311, 153, { &P18451,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT26ii Build/6.1.A.2.45) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2995038 = {S99312, 132, { &P21640,&P5629,&P15760,&P18092 }};	/*Useragent - samsung-mini7100/1.0 Linux/2.6.35.7 Android/2.3.5 Release/03.25.2013 Browser/AppleWebKit533.1 (KHTML, like Gecko) Mozilla/5.0 Mobile*/
const Device UA2995418 = {S99313, 94, { &P16591,&P22784,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq50) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2995681 = {S99314, 96, { &P23106,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; TA-FONE_A34) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2995814 = {S99315, 159, { &P15517,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; MT15i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2996598 = {S99316, 149, { &P12765,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; A100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2997048 = {S99317, 136, { &P22854,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9508 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2997258 = {S99318, 92, { &P13197,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIAN93) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2997350 = {S99319, 141, { &P20736,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GOCLEVER TAB A104.2 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2997506 = {S99320, 138, { &P23947,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Alpha GTR Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2997514 = {S99321, 134, { &P17710,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P925g Build/ICS) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2997593 = {S99322, 146, { &P24045,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; MyPhone A919i Duo Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2997607 = {S99323, 165, { &P22626,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Iris 349 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2998521 = {S99324, 148, { &P21903,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; U705T Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2998626 = {S99325, 158, { &P18661,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; N860 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2999023 = {S99326, 152, { &P18612,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC T329t Build/JRO03H) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2999186 = {S99327, 140, { &P21991,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Aqua Wonder Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3001366 = {S99328, 152, { &P18295,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC One X Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3006701 = {S99329, 160, { &P23535,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; Lenovo S899t Build/S-2-20) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3008708 = {S99330, 151, { &P20120,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A45 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3010027 = {S99331, 157, { &P18612,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HTC T329t Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3010326 = {S99332, 146, { &P23494,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Connect-2G-2.0 Build/HCL ME Tablet V2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3012158 = {S99333, 150, { &P18083,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; V9C Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3013263 = {S99334, 160, { &P19997,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo P700i Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3013557 = {S99335, 93, { &P22627,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; IRIS 455) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3013826 = {S99336, 151, { &P23815,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Z7C Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3014798 = {S99337, 157, { &P11747,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; HTC Desire Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3014899 = {S99338, 173, { &P16745,&P5096,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; T-Mobile myTouch 3G Slide Build/FRG83D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3016474 = {S99339, 162, { &P19970,&P17485,&P24678,&P18092 }};	/*Useragent - SonyEricssonG502/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssong502) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3018397 = {S99340, 155, { &P14186,&P5096,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5830 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3018484 = {S99341, 154, { &P20705,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; HTC One Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3019226 = {S99342, 157, { &P16356,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-N7000 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3019443 = {S99343, 159, { &P23919,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; R800i Build/4.0.2.A.0.42) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3020860 = {S99344, 134, { &P20511,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; IQ442 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022048 = {S99345, 137, { &P20592,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-SU870 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022391 = {S99346, 138, { &P21304,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Fly_IQ450 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022580 = {S99347, 137, { &P19865,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A850S Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022598 = {S99348, 134, { &P18655,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SH-09D Build/S4060) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3028149 = {S99349, 156, { &P12972,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; SGH-T989 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3031425 = {S99350, 91, { &P13734,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5603) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3041163 = {S99351, 121, { &P15364,&P16110,&P22451,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.1 Chrome/13.0.800.1 Safari/535.1*/
const Device UA3043031 = {S99352, 142, { &P13974,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3043538 = {S99353, 102, { &P12088,&P5129,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (NokiaX6-00/40.0.002/sw_platform=S60;sw_platform_version=5.0;java_build_version=1.4.43)*/
const Device UA3044829 = {S99354, 131, { &P20750,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; de-de; AN10DG3 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3046916 = {S99355, 169, { &P21151,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; LNV-Lenovo A790e Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3048176 = {S99356, 160, { &P21039,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Spice Mi-355 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3048278 = {S99357, 166, { &P22607,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-S5303 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/405*/
const Device UA3048529 = {S99358, 152, { &P13094,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Lenovo A60 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3049961 = {S99359, 164, { &P22623,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; Micromax A51 Build/REL_2.3.12) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3050058 = {S99360, 182, { &P15794,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3050151 = {S99361, 152, { &P15364,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; N350 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3050157 = {S99362, 152, { &P21600,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Kem 462 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3050376 = {S99363, 150, { &P16436,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA3050380 = {S99364, 141, { &P22608,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Spice Mi-530 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3050612 = {S99365, 159, { &P20452,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; Lenovo S880 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056132 = {S99366, 161, { &P20267,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; P300(Funbook) Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056236 = {S99367, 89, { &P22860,&P18598,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Q700) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3056303 = {S99368, 172, { &P21152,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ALCATEL ONE TOUCH 930D Build/1.2.10.4) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056511 = {S99369, 161, { &P20837,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; IRIS_501 Build/LAVAIRIS501) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056529 = {S99370, 156, { &P23004,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; TECNO D3 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056761 = {S99371, 164, { &P19968,&P17485,&P24678,&P18092 }};	/*Useragent - SonyEricssonW910i/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw910i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3056858 = {S99372, 94, { &P13591,&P5096,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; ZTE-BLADE) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3056943 = {S99373, 90, { &P12798,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; LT18i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3057231 = {S99374, 143, { &P18509,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/3.4;FBBV/X;FBDM/{density=1.0,width=320,height=480};FBLC/en_US;FBCR/MetroPCS;FBPN/com.facebook.katana;FBDV/LG-MS770;FBSV/4.0.4;]*/
const Device UA3058071 = {S99375, 141, { &P20233,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; VSD220 Build/IMM76D.UI23ED12_VSC) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3058077 = {S99376, 138, { &P16553,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SGH-T959V Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3058096 = {S99377, 126, { &P20177,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; C725 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3058238 = {S99378, 96, { &P22632,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Titanium_S1) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3058291 = {S99379, 147, { &P18385,&P21637,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus 7 Build/JWR66V) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3058392 = {S99380, 90, { &P17625,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST25i) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3058575 = {S99381, 128, { &P20446,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; F-05E Build/V17R40A) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3058787 = {S99382, 163, { &P20980,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; A97i Build/CelkonA97i_Celkon) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3059110 = {S99383, 154, { &P23546,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; AD683G Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3060547 = {S99384, 139, { &P23154,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XOLO_X1000 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3070839 = {S99385, 162, { &P18721,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; A90S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA3070919 = {S99386, 128, { &P18466,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A-AZ;FBAV/2.0;FBDM/{density=1.5,width=1280,height=800};FBLC/en_US;FBCR/;FBPN/com.facebook.katana;FBDV/KFTT;FBSV/4.0.3;]*/
const Device UA3071044 = {S99387, 151, { &P16598,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; A500 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3071377 = {S99388, 92, { &P17997,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; MID7012) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3071540 = {S99389, 136, { &P23058,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GN305 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3071800 = {S99390, 144, { &P20477,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; Full AOSP on Noontec Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3072648 = {S99391, 131, { &P20228,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; TAB7100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3073266 = {S99392, 140, { &P23303,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; EX'NOTE ? Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3073468 = {S99393, 92, { &P18699,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; MID8042) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3073684 = {S99394, 125, { &P15364,&P18144,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64) AppleWebKit/537.32 (KHTML, like Gecko) Chromium/25.0.1349.2 Chrome/25.0.1349.2 Safari/537.32*/
const Device UA3073908 = {S99395, 153, { &P19847,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S6012 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA3075646 = {S99396, 156, { &P22991,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HTC 802t Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3076772 = {S99397, 150, { &P16957,&P5096,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-US; XCD35 Build/FRF91) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3077241 = {S99398, 132, { &P21022,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-gb; LG-E615 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3077290 = {S99399, 176, { &P18219,&P5843,&P22881,&P18092 }};	/*Useragent - Nokia202/2.0 (20.52) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291*/
const Device UA3077536 = {S99400, 155, { &P22651,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-E435 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3077788 = {S99401, 136, { &P22854,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9508 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3077868 = {S99402, 137, { &P24052,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SKY_NANO Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3077989 = {S99403, 106, { &P19792,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; HTC Desire U dual sim) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3078019 = {S99404, 134, { &P24156,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; 202F Build/V17R47F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3078110 = {S99405, 159, { &P22997,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Celkon A107 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3078667 = {S99406, 144, { &P17926,&P6546,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.1; AT100 Build/HMJ37) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3079723 = {S99407, 159, { &P13124,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; PantechP9070 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3085420 = {S99408, 117, { &P17595,&P21721,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1*/
const Device UA3086378 = {S99409, 154, { &P15345,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P506 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3086465 = {S99410, 155, { &P18448,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Xperia J Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3086573 = {S99411, 164, { &P15364,&P18066,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A356 Build/LenovoLePhone) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3087478 = {S99412, 168, { &P20192,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA3088129 = {S99413, 139, { &P15649,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-S5570 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088466 = {S99414, 153, { &P17989,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; X501_USA_Cricket Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA3088679 = {S99415, 154, { &P20452,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S880 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3090323 = {S99416, 133, { &P20515,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-in; A54 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3091659 = {S99417, 89, { &P20438,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; N400) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3091967 = {S99418, 127, { &P23313,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; M8 pro Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3092164 = {S99419, 136, { &P23248,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Gpad_G2 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3092166 = {S99420, 131, { &P20705,&P18598,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.2.2; en-gb; HTC One Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3092436 = {S99421, 149, { &P12280,&P22882,&P21787,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) 1Password/3.7.2/X (like Mobile/8C148 Safari/6533.18.5)*/
const Device UA3092614 = {S99422, 161, { &P23697,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9192 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.20.1364.172 Mobile Safari/537.22*/
const Device UA3092719 = {S99423, 161, { &P20146,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9082 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.20.1364.172 Mobile Safari/537.22*/
const Device UA3098142 = {S99424, 131, { &P15364,&P7106,&P15829,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry Unknown; en) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.206 Mobile Safari/534.11+*/
const Device UA3098820 = {S99425, 57, { &P14523,&P5156,&P21731,&P18092 }};	/*Useragent - OneBrowser/for Blackberry3.0.0 (BlackBerry9800/6.0.0.246)*/
const Device UA3099095 = {S99426, 42, { &P15302,&P17485,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SonyEricssonWT13i/R3AF010)*/
const Device UA3099657 = {S99427, 164, { &P13746,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; LGL35G Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA3100554 = {S99428, 148, { &P15869,&P6546,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; Xoom Build/HMJ25) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 XiaoMi/MiuiBrowser/1.0*/
const Device UA3100607 = {S99429, 110, { &P23362,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; SAMSUNG; SPH-I800; SPRINT)*/
const Device UA3100912 = {S99430, 154, { &P18637,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9260 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3101571 = {S99431, 138, { &P19747,&P5043,&P24668,&P18092 }};	/*Useragent - Nokia206/2.0 (03.58) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia206) UCBrowser8.3.1.161/70/352/UCWEB Mobile*/
const Device UA3101622 = {S99432, 154, { &P18665,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; N861 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA3101707 = {S99433, 137, { &P23231,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E988 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3102067 = {S99434, 90, { &P23235,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; VT75C) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3102278 = {S99435, 162, { &P13123,&P5629,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.3.2.289*/
const Device UA3102487 = {S99436, 93, { &P21261,&P18598,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.2; zh-CN; GT-I9500) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile*/
const Device UA3102756 = {S99437, 141, { &P13685,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; LS670 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA3102882 = {S99438, 150, { &P18513,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6802 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA3102889 = {S99439, 71, { &P21261,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.2.2; zh-cn; GT-I9500; 1080*1920) UCWEB7.9.0.94/139/444*/
const Device UA3103097 = {S99440, 136, { &P22860,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Q700 Build/XOLOQ700) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3103113 = {S99441, 52, { &P23144,&P22878,&P17596,&P18092 }};	/*Useragent - MobileSafari/8536.25 CFNetwork/609.1.4 Darwin/12.4.0*/
const Device UA3103240 = {S99442, 136, { &P18466,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; KFTT Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon*/
const Device UA3103536 = {S99443, 137, { &P23011,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; iris405 Build/LAVAIRIS405) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3103613 = {S99444, 89, { &P20438,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; N400) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA3103678 = {S99445, 145, { &P21606,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LGL86C Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3103749 = {S99446, 166, { &P23003,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; TECNO Q1 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/352*/
const Device UA3104074 = {S99447, 134, { &P22093,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; hu-hu; DPS Dream 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3104387 = {S99448, 160, { &P18746,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; T-Mobile myTouch Q Build/HuaweiU8730) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA3104534 = {S99449, 127, { &P20513,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-us; A5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3105201 = {S99450, 134, { &P23771,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-gb; GT-I8190N Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3106022 = {S99451, 156, { &P22382,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8262 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3107055 = {S99452, 137, { &P24052,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SKY_NANO Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3107079 = {S99453, 174, { &P15364,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; TA-FONE A39 HD Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/33756*/
const Device UA3107875 = {S99454, 126, { &P20252,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V96A Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3107920 = {S99455, 166, { &P21472,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; A111 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3107942 = {S99456, 164, { &P15364,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; AMAZE S250 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA3108125 = {S99457, 89, { &P24648,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; G928) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3108249 = {S99458, 147, { &P20255,&P18066,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R811 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3108252 = {S99459, 157, { &P21068,&P5629,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE C1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3108465 = {S99460, 89, { &P21472,&P18111,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; A111) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3108896 = {S99461, 160, { &P14574,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus S Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/crespo*/
const Device UA3108899 = {S99462, 99, { &P22158,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; INTEX CLOUD X1) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3109134 = {S99463, 133, { &P20427,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHL21 Build/S2140) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3109156 = {S99464, 130, { &P21028,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Tab 7.9 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3109161 = {S99465, 72, { &P12280,&P19826,&P24717,&P18092 }};	/*Useragent - Instagram 4.1.2 (iPhone4,1; iPhone OS 6_1_3; en_US; en) AppleWebKit/420+*/
const Device UA3109290 = {S99466, 152, { &P21600,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Kem 462 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110143 = {S99467, 148, { &P18635,&P21637,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; zh-cn; Nexus 4 Build/JWR66Y) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3110967 = {S99468, 167, { &P12496,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I897 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/captivatemtd*/
const Device UA3115035 = {S99469, 136, { &P20366,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE T82 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3118292 = {S99470, 162, { &P18476,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I577 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3119090 = {S99471, 138, { &P22872,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5282 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3119648 = {S99472, 135, { &P20255,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; R8111 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA3120938 = {S99473, 96, { &P22632,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Titanium S1) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA3124158 = {S99474, 133, { &P19920,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; LG-MS870 Build/IMM76L) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3125288 = {S99475, 137, { &P24606,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E470f Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3125368 = {S99476, 142, { &P22585,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SPH-L720 Build/JWR66V.S014) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125492 = {S99477, 170, { &P15575,&P18111,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I9100 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3125515 = {S99478, 140, { &P11804,&P17483,&P24679,&P18092 }};	/*Useragent - LG-GW525 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gw525 teleca) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3125555 = {S99479, 155, { &P21792,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; GT-I9200 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3125629 = {S99480, 133, { &P21081,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; V790 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3125677 = {S99481, 92, { &P14413,&P17486,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mot-ex115) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3125746 = {S99482, 174, { &P13815,&P5096,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; HTC_Wildfire Build/FRG83D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3125884 = {S99483, 137, { &P12814,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; SGH-I777 Build/JSS15J) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125887 = {S99484, 93, { &P21356,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; TECNO_N3) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3125897 = {S99485, 166, { &P15364,&P18598,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.2.2; id-id; Vandroid Build/S5-V1.0-2013.07.18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125920 = {S99486, 147, { &P18295,&P18598,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; HTC One X Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3125984 = {S99487, 172, { &P20004,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Karbonn_A2 Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3125989 = {S99488, 93, { &P15060,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-S5660) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3126034 = {S99489, 195, { &P15364,&P5629,&P15760,&P18091 }};	/*Useragent - Mozilla/5.0 (Linux;u;Android 2.3.7;zh-cn;) AppleWebKit/533.1 (KHTML,like Gecko) Version/4.0 Mobile Safari/533.1 (compatible; +http://www.baidu.com/search/spider.html)DAPPER-HOST-IP:108.162.199.56*/
const Device UA3126066 = {S99490, 88, { &P20265,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A52) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3126132 = {S99491, 159, { &P22913,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A25 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126131 = {S99492, 107, { &P19868,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL ONE TOUCH 985N) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3126165 = {S99493, 92, { &P18621,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; LG-E612) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3126265 = {S99494, 158, { &P16292,&P21637,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; zh-CN; Galaxy Nexus Build/JWR66Y) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126268 = {S99495, 155, { &P22991,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; HTC 802t Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126342 = {S99496, 152, { &P18583,&P5043,&P24680,&P18092 }};	/*Useragent - Nokia110/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia110) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3126354 = {S99497, 158, { &P18582,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LT28h Build/6.2.B.0.211) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126376 = {S99498, 89, { &P20129,&P18111,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; zh-CN; MI 2) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3126392 = {S99499, 136, { &P24529,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9506 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126489 = {S99500, 164, { &P12657,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC EVO 3D X515m Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126688 = {S99501, 128, { &P20153,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AN7HG3 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126754 = {S99502, 88, { &P20529,&P18066,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A89) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3126819 = {S99503, 158, { &P21973,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Aqua flash Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126882 = {S99504, 165, { &P19998,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HUAWEI C8812 Build/HuaweiC8812) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126894 = {S99505, 90, { &P22641,&P18111,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; C1504) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3127063 = {S99506, 156, { &P22875,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9220 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127079 = {S99507, 96, { &P20004,&P18066,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Karbonn A2+) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3127085 = {S99508, 131, { &P17615,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E230K Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3127159 = {S99509, 160, { &P21738,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; iris502 Build/LAVAIRIS502) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127162 = {S99510, 156, { &P16362,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; SPH-D710 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127165 = {S99511, 164, { &P18724,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; IdeaTab A2107A-H Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127239 = {S99512, 134, { &P21432,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; N7889 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127249 = {S99513, 137, { &P18622,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A820L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127275 = {S99514, 152, { &P18314,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M032 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127318 = {S99515, 133, { &P21853,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Connect-3G-2.0 Build/HCL) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3127390 = {S99516, 93, { &P15575,&P18111,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-I9100) U2/1.0.0 UCBrowser/9.3.0.321 U2/1.0.0 Mobile*/
const Device UA3127846 = {S99517, 149, { &P18556,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; BNTV600 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3128428 = {S99518, 150, { &P22874,&P18598,&P24683,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-I9502 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.5 Mobile Safari/533.1*/
const Device UA3130696 = {S99519, 159, { &P15397,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST18i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3130954 = {S99520, 136, { &P18438,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E973 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3130955 = {S99521, 133, { &P20129,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MI 2 Build/JRO03L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3131707 = {S99522, 159, { &P20289,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC6435LVW Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3132856 = {S99523, 140, { &P21602,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5302 Build/12.0.A.2.254) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA54632 = {S99524, 87, { &P14247,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP220/V01 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1 LG-KP220/V01 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA2509 = {S99525, 139, { &P17261,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; zh-cn; ME501 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA35335 = {S99526, 96, { &P11729,&P5026,&P4948,&P18092 }};	/*Useragent - BlackBerry8100/4.5.0.174 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/142 UP.Browser/5.0.3.3*/
const Device UA7269 = {S99527, 70, { &P13277,&P5448,&P22271,&P18092 }};	/*Useragent - HTC_P6500 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA87374 = {S99528, 104, { &P13277,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC P6500; OpVer 27.110.1.612*/
const Device UA13057 = {S99529, 93, { &P13757,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-VF526/1.0/E1N_HF MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.410 (GUI)*/
const Device UA13058 = {S99530, 93, { &P13757,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-VF526/1.0/EVN_HQ MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.416 (GUI)*/
const Device UA13059 = {S99531, 93, { &P13757,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-VF526/1.0/EVN_HP MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.416 (GUI)*/
const Device UA40794 = {S99532, 93, { &P13757,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-VF526/1.0/EVN_HL MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.412 (GUI)*/
const Device UA43856 = {S99533, 93, { &P13757,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-VF526/1.0/EVN_HF MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.414 (GUI)*/
const Device UA87027 = {S99534, 93, { &P13757,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-VF526/1.0/EVN_HH MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.407 (GUI)*/
const Device UA13961 = {S99535, 137, { &P13921,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-MOTORAZRmaxxV3/98.30.80R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12093102) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA13975 = {S99536, 137, { &P13921,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-MOTORAZRmaxxV3/99.41.0BR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163234) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA13977 = {S99537, 138, { &P13921,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-MOTORAZRmaxxV3/98.30.72AR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12003079) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA13981 = {S99538, 137, { &P13921,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-MOTORAZRmaxxV3/96.80.51R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11073071) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA29423 = {S99539, 137, { &P13921,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-MOTORAZRmaxxV3/96.A0.0CR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 11073121) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA29471 = {S99540, 137, { &P13921,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-MOTORAZRmaxxV3/99.41.08R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12163225) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA16375 = {S99541, 114, { &P13045,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-P260-ORANGE/P260BVGG5 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA49519 = {S99542, 107, { &P13045,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-P260/P260PKGK1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA20133 = {S99543, 112, { &P15035,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-M8800L/1.0 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.9 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA53818 = {S99544, 39, { &P14037,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GB160 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA22243 = {S99545, 101, { &P15410,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E760/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA23254 = {S99546, 102, { &P15578,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-F275L/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA23495 = {S99547, 67, { &P15608,&P17017,&P17470,&P18092 }};	/*Useragent - Fly_Q400/WapBrowserprofile/WIFIprofile/MIDP2.0Configuration/CLDC1.1*/
const Device UA24000 = {S99548, 47, { &P15648,&P17017,&P17470,&P18092 }};	/*Useragent - TNM-V70/1.0 Release/11.22.2008 Browser/Obigo2.0*/
const Device UA24466 = {S99549, 140, { &P15689,&P17605,&P3976,&P18092 }};	/*Useragent - SAMSUNG-GT-C3630C_CMCC/1.0 RTK-E/1.0 SHP/VPP-R5 Release/11.17.2009 Browser/NetFront3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1 SMM-MMS/1.2.0*/
const Device UA28382 = {S99550, 98, { &P12231,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-F406/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA38492 = {S99551, 171, { &P13451,&P17605,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-B3410/1.0 RTK-E/1.0 DF/1.0 Release/22.12.2009 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA44077 = {S99552, 116, { &P13451,&P17605,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-B3410/1.0 RTK-E/1.0 DF/1.0 Release/22.12.2009 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA42613 = {S99553, 101, { &P13872,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 240x320; MOT-Q-UMTS/APVER) Opera 8.65 [en]*/
const Device UA48933 = {S99554, 105, { &P13872,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 240x320; MOT-Q-UMTS/01.07.05R) Opera 8.65 [en]*/
const Device UA56746 = {S99555, 105, { &P13872,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 240x320; MOT-Q-UMTS/01.01.03R) Opera 8.65 [en]*/
const Device UA50311 = {S99556, 71, { &P11914,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-E2510/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA53593 = {S99557, 174, { &P13988,&P5222,&P4626,&P18092 }};	/*Useragent - HTCT9188_TD/1.0 WindowsMobile/6.5 CEOS/5.2 release/5.3 Opera/9.7 WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (WindowsMobile; PPC; Opera Mobi/35267; U; zh-CN; Presto/2.1.1)*/
const Device UA438824 = {S99558, 186, { &P13988,&P5222,&P4626,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTCT9188_TD/1.0 WindowsMobile/6.5 CEOS/5.2 release/5.3 Opera/9.7 WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1; Windows Phone 6.5.3.5)*/
const Device UA54466 = {S99559, 15, { &P14220,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHC140/1.0*/
const Device UA58810 = {S99560, 119, { &P20384,&P22839,&P4223,&P18092 }};	/*Useragent - Zte-tu236_TD/1.0 ThreadX/4.0b Larena/2.40 Release/4.15.2010 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA60811 = {S99561, 46, { &P12578,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-LG300/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA60865 = {S99562, 53, { &P24375,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E330/1.0 UP.Browser/6.2.2.6 (GUI) MMP/1.0*/
const Device UA61821 = {S99563, 8, { &P14334,&P17017,&P17470,&P18092 }};	/*Useragent - SPH-M910*/
const Device UA63006 = {S99564, 188, { &P15364,&P7133,&P4737,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.2.2.51/69/352*/
const Device UA464282 = {S99565, 188, { &P15364,&P7133,&P4737,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.2.2.51/50/352*/
const Device UA464340 = {S99566, 188, { &P15364,&P7133,&P4737,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.2.2.51/27/352*/
const Device UA466657 = {S99567, 188, { &P15364,&P7133,&P4737,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.2.2.51/28/352*/
const Device UA63108 = {S99568, 59, { &P14477,&P22839,&P17470,&P18092 }};	/*Useragent - ZTE-R235/1.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63321 = {S99569, 72, { &P16471,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-F101/1.0 NetFront/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63327 = {S99570, 46, { &P14502,&P17484,&P5001,&P18092 }};	/*Useragent - SCH-U550/1.0 NetFront/3.0.22.2.8 (GUI) MMP/2.0*/
const Device UA66553 = {S99571, 150, { &P11747,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Cricket Eclair 2.1; en-us; ADR6200 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA643155 = {S99572, 150, { &P11747,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Cricket Eclair 2.1; en-us; ADR6200 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA452205 = {S99573, 119, { &P12139,&P5204,&P4777,&P18092 }};	/*Useragent - Nokia5310XpressMusic/2.0 (05.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5310xpressmusic/UC Browser7.7.1.88/69/444*/
const Device UA69428 = {S99574, 36, { &P14983,&P5055,&P4779,&P18092 }};	/*Useragent - NokiaC7-00/UC Browser7.8.0.95/50/355*/
const Device UA80740 = {S99575, 9, { &P17475,&P5768,&P3780,&P18092 }};	/*Useragent - Nokia6300*/
const Device UA81703 = {S99576, 15, { &P13926,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHX510/1.0*/
const Device UA88624 = {S99577, 47, { &P17536,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC3N UP.Browser/6.2.0.15.1.1 (GUI) MMP/2.0*/
const Device UA88780 = {S99578, 14, { &P17903,&P17017,&P17470,&P18092 }};	/*Useragent - NEXIAN NX-G911*/
const Device UA89762 = {S99579, 47, { &P17549,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC3V UP.Browser/6.2.0.15.1.1 (GUI) MMP/2.0*/
const Device UA2370358 = {S99580, 106, { &P12026,&P17485,&P17338,&P18092 }};	/*Useragent - SonyEricssonC510/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonc510/UC Browser7.9.0.102/69/352*/
const Device UA254019 = {S99581, 79, { &P15364,&P5690,&P24207,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) QQBrowser/6.7.10246.201*/
const Device UA646674 = {S99582, 124, { &P15364,&P5690,&P24207,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.7.10246.201*/
const Device UA2303063 = {S99583, 87, { &P15364,&P5690,&P24207,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0) QQBrowser/6.7.10246.201*/
const Device UA421047 = {S99584, 81, { &P12682,&P5448,&P4779,&P18092 }};	/*Useragent - HTC_P3490 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA893565 = {S99585, 73, { &P17595,&P18575,&P17317,&P18092 }};	/*Useragent - Safari/6533.18.5 CFNetwork/454.9.7 Darwin/10.4.1 (i386) (MacBookPro7%2C1)*/
const Device UA1862204 = {S99586, 70, { &P17595,&P18575,&P17317,&P18092 }};	/*Useragent - Safari/6533.18.5 CFNetwork/454.9.7 Darwin/10.4.1 (i386) (MacBook7%2C1)*/
const Device UA446155 = {S99587, 50, { &P18088,&P17017,&P24645,&P18092 }};	/*Useragent - IUC(U;iOS 4.1;Zh-cn;320*480;)/UCWEB7.5.0.66/41/997*/
const Device UA760047 = {S99588, 52, { &P18088,&P17017,&P24645,&P18092 }};	/*Useragent - IUC(U;iOS 4.2.1;Zh-cn;320*480;)/UCWEB7.5.0.66/41/997*/
const Device UA2132261 = {S99589, 52, { &P18088,&P17017,&P24645,&P18092 }};	/*Useragent - IUC(U;iOS 3.1.3;Zh-cn;320*480;)/UCWEB7.5.0.66/39/800*/
const Device UA449609 = {S99590, 69, { &P15364,&P17017,&P20803,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/4.3) KHTML/4.3.5 (like Gecko) SUSE*/
const Device UA475115 = {S99591, 66, { &P16885,&P22835,&P17470,&P18092 }};	/*Useragent - Spice S7000/(X)S006/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA502884 = {S99592, 58, { &P17883,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SN3D UP.Browser/6.2_7.2.7.1.K.1.5.1.120 (GUI) MMP/2.0*/
const Device UA535867 = {S99593, 76, { &P15364,&P7133,&P24188,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:2.1.1) Gecko/X Firefox/4.0.2pre Fennec/4.0.1*/
const Device UA555135 = {S99594, 64, { &P14225,&P18110,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.3;Zh_cn;Desire HD;480*800;)UCWEB7.8.1.96/139/800*/
const Device UA566611 = {S99595, 43, { &P12351,&P17484,&P4777,&P18092 }};	/*Useragent - samsung-gt-s3653w/UC Browser7.7.1.88/69/444*/
const Device UA571620 = {S99596, 44, { &P14831,&P17484,&P17338,&P18092 }};	/*Useragent - samsung-gt-s7233e/UC Browser7.9.0.102/69/352*/
const Device UA2299365 = {S99597, 100, { &P14693,&P5843,&P17338,&P18092 }};	/*Useragent - NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac1-02/UC Browser7.9.0.102/69/352*/
const Device UA573364 = {S99598, 104, { &P12135,&P17483,&P17338,&P18092 }};	/*Useragent - LG-GD510/V100 Teleca/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  lg-gd510/UC Browser7.9.0.102/69/352*/
const Device UA574821 = {S99599, 75, { &P17595,&P17297,&P24686,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:7.0.1) Gecko/X Yoono/1.8.16*/
const Device UA576112 = {S99600, 90, { &P17595,&P17296,&P18368,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:8.0) Gecko/X Thunderbird/8.0 Lightning/1.0*/
const Device UA578662 = {S99601, 148, { &P12280,&P17090,&P20779,&P18092 }};	/*Useragent - MQQBrowser/28 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3*/
const Device UA754216 = {S99602, 148, { &P12280,&P17090,&P20779,&P18092 }};	/*Useragent - MQQBrowser/28 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3*/
const Device UA579372 = {S99603, 44, { &P11948,&P17485,&P17347,&P18092 }};	/*Useragent - sonyericssonk550i/UC Browser8.0.3.107/69/444*/
const Device UA1938228 = {S99604, 44, { &P11948,&P17485,&P17347,&P18092 }};	/*Useragent - sonyericssonk550i/UC Browser8.0.3.107/70/352*/
const Device UA2026198 = {S99605, 171, { &P14186,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.2.139/145/355*/
const Device UA2027138 = {S99606, 171, { &P14186,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.2.139/145/355*/
const Device UA2029876 = {S99607, 171, { &P14186,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA605785 = {S99608, 40, { &P17968,&P17979,&P18393,&P18092 }};	/*Useragent - Dolphin http client/8.0.0(132) (Android)*/
const Device UA615420 = {S99609, 40, { &P17968,&P17979,&P18393,&P18092 }};	/*Useragent - Dolphin http client/8.0.1(133) (Android)*/
const Device UA622019 = {S99610, 45, { &P17968,&P17979,&P18393,&P18092 }};	/*Useragent - Dolphin http client/8.0.0 beta(131) (Android)*/
const Device UA623400 = {S99611, 86, { &P17577,&P5567,&P4536,&P18092 }};	/*Useragent - Mozilla/5.0 Opera/9.5 (KDDI-TS3S; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00*/
const Device UA633500 = {S99612, 113, { &P17475,&P5768,&P17437,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia6300/07.21; Profile/MIDP-2.0 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.4.0.34.8*/
const Device UA635735 = {S99613, 70, { &P17595,&P22328,&P17415,&P18092 }};	/*Useragent - Safari/7534.52.7 CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (iMac12%2C2)*/
const Device UA642198 = {S99614, 75, { &P17595,&P22328,&P17415,&P18092 }};	/*Useragent - Safari/7534.52.7 CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookPro5%2C4)*/
const Device UA643119 = {S99615, 75, { &P17595,&P22328,&P17415,&P18092 }};	/*Useragent - Safari/7534.52.7 CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookAir4%2C2)*/
const Device UA684808 = {S99616, 70, { &P17595,&P22328,&P17415,&P18092 }};	/*Useragent - Safari/7534.52.7 CFNetwork/520.2.5 Darwin/11.2.0 (i386) (MacBook2%2C1)*/
const Device UA2306439 = {S99617, 69, { &P17595,&P22328,&P17415,&P18092 }};	/*Useragent - Safari/7534.52.7 CFNetwork/520.2.5 Darwin/11.2.0 (i386) (MacPro3%2C1)*/
const Device UA655079 = {S99618, 163, { &P15911,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; DROID3 Build/5.5.1_84_D3G-55; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA754453 = {S99619, 92, { &P17595,&P17297,&P19773,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.1.19) Gecko/X SeaMonkey/2.0.14*/
const Device UA771436 = {S99620, 37, { &P16871,&P22784,&P17338,&P18092 }};	/*Useragent - micromaxq5/UC Browser7.9.0.102/69/352*/
const Device UA789343 = {S99621, 102, { &P17595,&P17297,&P22482,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.0.19) Gecko/X Firefox/3.0.19 Flock/2.6.1*/
const Device UA806175 = {S99622, 112, { &P15364,&P6596,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; CrOS i686 0.0.0; en-US) AppleWebKit/533.1 (KHTML, like Gecko) Chrome/5.0.321.0 Safari/533.1*/
const Device UA823393 = {S99623, 54, { &P12449,&P5890,&P20775,&P18092 }};	/*Useragent - MQQBrowser/2.4 (NokiaE5-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA868943 = {S99624, 87, { &P17595,&P17295,&P18209,&P18092 }};	/*Useragent - GoogleEarth/6.0.3.2197(Macintosh;Mac OS X (10.5.8);en;kml:2.2;client:Free;type:default)*/
const Device UA881234 = {S99625, 86, { &P17639,&P5567,&P4536,&P18092 }};	/*Useragent - Mozilla/5.0 Opera/9.5 (KDDI-SH3K; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00*/
const Device UA925844 = {S99626, 95, { &P14632,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; T8788)*/
const Device UA925881 = {S99627, 98, { &P14632,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Surround)*/
const Device UA1842804 = {S99628, 92, { &P15364,&P18342,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; pl-PL; rv:1.9.0.2) Gecko/X Ubuntu/9.25 (jaunty) Firefox/3.5*/
const Device UA1888463 = {S99629, 61, { &P15364,&P18110,&P18196,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:5.0) Gecko/X Thunderbird/5.0*/
const Device UA1937104 = {S99630, 116, { &P11808,&P5204,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia5610d-1/10.00; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA1940144 = {S99631, 84, { &P17595,&P17294,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:9.0) Gecko/X Firefox/9.0 TenFourFox/G3*/
const Device UA2026565 = {S99632, 133, { &P14900,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 3GW100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026622 = {S99633, 133, { &P22495,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; fr-fr; MID7C Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027578 = {S99634, 131, { &P21049,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GEM7012 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2028380 = {S99635, 144, { &P15995,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Evo Shift Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2028910 = {S99636, 133, { &P21482,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MID7108SC Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2032353 = {S99637, 138, { &P18237,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; IdeaTab S2007A-D Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2125927 = {S99638, 119, { &P15364,&P17813,&P19772,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.22) Gecko/X SeaMonkey/1.1.17 (Ubuntu-1.1.17+nobinonly-0ubuntu0.8.04.1)*/
const Device UA2147748 = {S99639, 70, { &P21938,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux; U;2.2.2;zh_cn;Coolpad 5820;320*480;)UCWEB7.6.0.75/139/32521*/
const Device UA2160366 = {S99640, 70, { &P15364,&P5690,&P22267,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.5; Windows NT 5.1; .NET CLR 2.0.40607)*/
const Device UA2165110 = {S99641, 85, { &P14972,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s3370) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2216196 = {S99642, 43, { &P11768,&P17485,&P17347,&P18092 }};	/*Useragent - sonyericssonw395/UC Browser8.0.3.107/69/444*/
const Device UA2216328 = {S99643, 36, { &P12174,&P17484,&P17347,&P18092 }};	/*Useragent - sgh-f480i/UC Browser8.0.3.107/69/444*/
const Device UA2216709 = {S99644, 47, { &P15118,&P5055,&P17347,&P18092 }};	/*Useragent - nokiax7-00/UC Browser8.0.3.107/69/444  3gpp-gba*/
const Device UA2220268 = {S99645, 126, { &P23671,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL55NX720; CC/USA; en) Presto/2.7.61 Version/11.00*/
const Device UA2223512 = {S99646, 93, { &P15364,&P5690,&P17652,&P18092 }};	/*Useragent - MediaGo/2.2.0.223 (Windows NT 5.1) PlayStationStore/4.9.4.14625 PlaybackEngine/1.92.165.06140*/
const Device UA2339541 = {S99647, 93, { &P15364,&P5690,&P17652,&P18092 }};	/*Useragent - MediaGo/2.2.0.223 (Windows NT 5.1) PlayStationStore/4.9.4.14625 PlaybackEngine/1.92.161.06140*/
const Device UA2258643 = {S99648, 96, { &P17595,&P17295,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-US; rv:1.9.1b3pre) Gecko/X Shiretoko/3.1b3pre*/
const Device UA2260278 = {S99649, 54, { &P13552,&P5890,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 (NokiaC6-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA2277259 = {S99650, 35, { &P11945,&P17484,&P4742,&P18092 }};	/*Useragent - gt-s5233s/UC Browser7.5.1.77/69/351*/
const Device UA2289589 = {S99651, 99, { &P14848,&P5843,&P4777,&P18092 }};	/*Useragent - NokiaX3-00/5.0 (11.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax3-00/UC Browser7.7.1.88/69/444*/
const Device UA2293308 = {S99652, 88, { &P17595,&P17297,&P20917,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.7; en-US; rv:1.9.1b2) Gecko/X Firefox/3.1b2*/
const Device UA2300892 = {S99653, 103, { &P13634,&P17483,&P17338,&P18092 }};	/*Useragent - LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  lg-gs500/UC Browser7.9.0.102/69/352*/
const Device UA2312707 = {S99654, 76, { &P21588,&P18110,&P20776,&P18092 }};	/*Useragent - MQQBrowser/25 (Linux; U; 2.3.5; zh-cn; 5860 Build/2.3.025.P1.X.5860;480*800)*/
const Device UA2338020 = {S99655, 139, { &P13685,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-LS670 Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2338112 = {S99656, 86, { &P11996,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5560i) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2338745 = {S99657, 82, { &P17595,&P17296,&P18330,&P18090 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:14.0) Gecko/X Firefox/14.0.1 slurp*/
const Device UA2339316 = {S99658, 153, { &P15047,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; SK17i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2352228 = {S99659, 171, { &P16479,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; SCH-R720 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2366818 = {S99660, 171, { &P18297,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I8530 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2375830 = {S99661, 79, { &P16871,&P22784,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; micromaxq5) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2383526 = {S99662, 175, { &P14913,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SPH-M820-BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2392621 = {S99663, 126, { &P23631,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46CX520; CC/XGA; en) Presto/2.7.61 Version/11.00*/
const Device UA2410789 = {S99664, 85, { &P14972,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; samsung-gt-s3370) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2413172 = {S99665, 85, { &P16366,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s8600) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2420976 = {S99666, 126, { &P13206,&P17485,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; sonyericssonk530i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile*/
const Device UA2440935 = {S99667, 76, { &P14532,&P17483,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; lg-t325) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2443011 = {S99668, 77, { &P23351,&P22784,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; micromax) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2445750 = {S99669, 51, { &P14359,&P5890,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 (Nokia5228;SymbianOS/9.1 Series60/3.0)*/
const Device UA2447550 = {S99670, 39, { &P14708,&P5204,&P23405,&P18092 }};	/*Useragent - MQQBrowser/Mini2.8 (Nokia5130c-2/07.97)*/
const Device UA2454084 = {S99671, 132, { &P17595,&P18207,&P22451,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.0 Chrome/13.0.800.0 Safari/535.1*/
const Device UA2454862 = {S99672, 160, { &P16957,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; XCD35 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2462807 = {S99673, 105, { &P13837,&P5448,&P4578,&P18092 }};	/*Useragent - dopod S900_CMCC/52.1941/WAP2.0 Profile/MID2.0 Configuration/CLDC1.1 (compatible;MSIE 4.01;Windows CE;PPC)*/
const Device UA2463195 = {S99674, 81, { &P16600,&P22784,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; micromaxx600) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2474152 = {S99675, 86, { &P14831,&P17484,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s7233e) UCBrowser8.3.0.154/69/352/UCWEB Mobile*/
const Device UA2481569 = {S99676, 207, { &P18321,&P5629,&P17374,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2482846 = {S99677, 96, { &P15364,&P18110,&P20914,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/534.34 (KHTML, like Gecko) rekonq/1.80 Safari/534.34*/
const Device UA2483080 = {S99678, 131, { &P22138,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N-02E Build/AX) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2483566 = {S99679, 118, { &P13197,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN93) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/352/UCWEB Mobile*/
const Device UA2493459 = {S99680, 47, { &P13324,&P5055,&P17338,&P18092 }};	/*Useragent - nokian8-00/UC Browser7.9.0.102/70/352  3gpp-gba*/
const Device UA2502713 = {S99681, 182, { &P14693,&P5843,&P24672,&P18092 }};	/*Useragent - NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac1-02) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile*/
const Device UA2520220 = {S99682, 81, { &P16600,&P22784,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; micromaxx600) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2520690 = {S99683, 161, { &P14643,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; GT-I8320 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2527280 = {S99684, 192, { &P12605,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID BIONIC Build/6.7.2-223_DBN_M4-23) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (DROID BIONIC, 36, 1.7.1)*/
const Device UA2533414 = {S99685, 141, { &P12788,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Evo 3D CDMA Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2547712 = {S99686, 96, { &P18644,&P5043,&P17338,&P18092 }};	/*Useragent - Nokia112/2.0 (03.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia112/UC Browser7.9.0.102/69/352*/
const Device UA2558241 = {S99687, 141, { &P18224,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Ascend G300 Build/JOP40C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2577716 = {S99688, 150, { &P13644,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2579772 = {S99689, 160, { &P19970,&P17485,&P24675,&P18092 }};	/*Useragent - SonyEricssonG502/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssong502) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2587235 = {S99690, 126, { &P23640,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL32EX427; CC/COL; en) Presto/2.7.61 Version/11.00*/
const Device UA2593981 = {S99691, 90, { &P17595,&P17294,&P18345,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; PPC Mac OS X 10.4; rv:15.0) Gecko/X Firefox/15.0.1 TenFourFox/7450*/
const Device UA2603205 = {S99692, 126, { &P23853,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; Phoenix Build/X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2607484 = {S99693, 73, { &P14651,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;7 Trophy T8686;7.10;8862)*/
const Device UA2607489 = {S99694, 67, { &P14651,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;7 Trophy;7.10;8862)*/
const Device UA2607880 = {S99695, 29, { &P15985,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (GT-P6200 HTJ85B)*/
const Device UA2607983 = {S99696, 29, { &P17323,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (GT-P6800 HTJ85B)*/
const Device UA2608030 = {S99697, 30, { &P17337,&P17979,&P20428,&P18092 }};	/*Useragent - Android OS/12 samsung_GT-P7500*/
const Device UA2627971 = {S99698, 93, { &P17337,&P17979,&P20428,&P18092 }};	/*Useragent - com.june.guessthemovie/1.234 (unknown,Android OS 12,samsung GT-P7500R-GT-P7500R,Scale/1.0000)*/
const Device UA2630893 = {S99699, 58, { &P17337,&P17979,&P20428,&P18092 }};	/*Useragent - com.kakao.talk/Android Version/2.6.2 Model/GT-P7500M MNC/X*/
const Device UA2625731 = {S99700, 99, { &P22037,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Messenger; 1.8.3) (espressowifi; samsung; GT-P3113; 4.1.1/JRO03C)*/
const Device UA2615251 = {S99701, 166, { &P15876,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P5100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2626561 = {S99702, 166, { &P15876,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P5100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2622148 = {S99703, 164, { &P22043,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ph; GT-P5113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P5113, 34, 1.6.0)*/
const Device UA2622473 = {S99704, 164, { &P22043,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P5113, 38, 1.8.1)*/
const Device UA2617334 = {S99705, 96, { &P17323,&P17979,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 Android (13/3.2; 160dpi; 800x1232; samsung; GT-P6800; GT-P6800; smdkc210; en_US)*/
const Device UA2626020 = {S99706, 118, { &P12889,&P18598,&P19802,&P18092 }};	/*Useragent - Logos/2.2.10 (Linux; U; Android 4.2.1; es-US; GT-P7300 Build/cm_p5-userdebug 4.2.1 JOP40D eng.christian.X.X test-keys)*/
const Device UA2630732 = {S99707, 132, { &P12889,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; tr-tr; GT-P7300 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2631949 = {S99708, 131, { &P12889,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7300 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2632150 = {S99709, 60, { &P12889,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P7300 Build/JDQ39)*/
const Device UA2633634 = {S99710, 131, { &P22041,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6810 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2635400 = {S99711, 131, { &P22041,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P6810 Build/JRO03R) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2635010 = {S99712, 131, { &P14908,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P6210 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2642019 = {S99713, 53, { &P15373,&P5890,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8 (Nokia5530;SymbianOS/9.1 Series60/3.0)*/
const Device UA2644097 = {S99714, 157, { &P18214,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2655646 = {S99715, 171, { &P14569,&P5204,&P24673,&P18092 }};	/*Useragent - Nokia7610Supernova/2.0 (06.51) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia7610supernova) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2655855 = {S99716, 99, { &P11924,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssons500i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2697363 = {S99717, 93, { &P20069,&P22784,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; micromaxx78) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2702315 = {S99718, 134, { &P13633,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A953 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2702752 = {S99719, 172, { &P23437,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; iBuddy Connect Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2706494 = {S99720, 164, { &P12896,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P999 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2707103 = {S99721, 135, { &P22762,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; L-01D Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2726358 = {S99722, 163, { &P12871,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2726632 = {S99723, 169, { &P15079,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; MK16i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2729215 = {S99724, 155, { &P14693,&P5843,&P24675,&P18092 }};	/*Useragent - NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac1-02) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2729242 = {S99725, 160, { &P12026,&P17485,&P24675,&P18092 }};	/*Useragent - SonyEricssonC510/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonc510) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2730550 = {S99726, 165, { &P15184,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P920 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2733329 = {S99727, 139, { &P23274,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TAB-PROTAB2-IPS9 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2734652 = {S99728, 143, { &P21111,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IS12M Build/6.7.5-50_MR1-11) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734693 = {S99729, 138, { &P20128,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AT-AS43D2 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734757 = {S99730, 135, { &P21829,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; F-02E Build/V16R46A) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2734895 = {S99731, 211, { &P18113,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; DROID RAZR Build/9.8.2O-72_VZW-16) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, -0400, us*/
const Device UA2735224 = {S99732, 136, { &P22678,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MAXWEST Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2736735 = {S99733, 137, { &P20075,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Xperia S Build/IMM76L) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2741915 = {S99734, 62, { &P15364,&P17188,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 4.0; U; en) Presto/2.2.15 Version/10.10*/
const Device UA2744111 = {S99735, 155, { &P21387,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A360 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2744879 = {S99736, 155, { &P13018,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; MT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2747887 = {S99737, 92, { &P16808,&P22784,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; micromaxq7) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2747960 = {S99738, 222, { &P14912,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung gt-c3222) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile, SAMSUNG-GT-C3222/C3222DDJL3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA2750858 = {S99739, 135, { &P23064,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ThL W1 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2751362 = {S99740, 138, { &P15759,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Vision Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2765811 = {S99741, 138, { &P23314,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Jiayu G3 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2767144 = {S99742, 161, { &P14959,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9103 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2778108 = {S99743, 158, { &P11683,&P17485,&P24673,&P18092 }};	/*Useragent - SonyEricssonC905/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; fa; sonyericssonc905) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2780876 = {S99744, 137, { &P18021,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A770K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2780931 = {S99745, 134, { &P22705,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IQ451 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2786524 = {S99746, 43, { &P15302,&P17485,&P4759,&P18092 }};	/*Useragent - sonyericssonwt13i/UC Browser7.6.1.82/69/351*/
const Device UA2790460 = {S99747, 158, { &P15809,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Rhyme S510b Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2791965 = {S99748, 160, { &P23257,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI T8833 Build/HuaweiT8833) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2794091 = {S99749, 166, { &P12814,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I777 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2794203 = {S99750, 138, { &P21028,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Smartfren Tab 7 Build/JRO03H) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2794753 = {S99751, 90, { &P12856,&P17483,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t300) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2958904 = {S99752, 90, { &P12856,&P17483,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t300) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2804743 = {S99753, 62, { &P15364,&P7127,&P20411,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.0; U; en) Presto/2.2.15 Version/10.20*/
const Device UA2806480 = {S99754, 11, { &P16800,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M5454*/
const Device UA2807994 = {S99755, 42, { &P15655,&P17484,&P4779,&P18092 }};	/*Useragent - samsung-gt-s3850/UC Browser7.8.0.95/69/352*/
const Device UA2808807 = {S99756, 49, { &P15364,&P5890,&P20777,&P18092 }};	/*Useragent - MQQBrowser/2.6 (E66-1;SymbianOS/9.2 Series60/3.1)*/
const Device UA2810805 = {S99757, 146, { &P16292,&P17979,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android-4.0.3; en-us; Galaxy Nexus Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75 Mobile Safari/535.7*/
const Device UA2815520 = {S99758, 60, { &P15364,&P6704,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (Windows 98; U; en) Presto/2.10.289 Version/12.00*/
const Device UA2826054 = {S99759, 146, { &P13914,&P5843,&P24668,&P18092 }};	/*Useragent - Nokia5330-1d/5.0 (06.88) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5330-1d) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2829072 = {S99760, 119, { &P13536,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA6110) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/352/UCWEB Mobile*/
const Device UA2830529 = {S99761, 140, { &P16503,&P5204,&P24668,&P18092 }};	/*Useragent - Nokia7310/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia7310) UCBrowser8.3.1.161/69/355/UCWEB Mobile*/
const Device UA2832090 = {S99762, 158, { &P18382,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; HTC Desire C Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2832734 = {S99763, 168, { &P13746,&P5477,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; GT540 Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2834044 = {S99764, 166, { &P18437,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-N8000 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2835449 = {S99765, 151, { &P21023,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; LG-E405 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2836737 = {S99766, 170, { &P18462,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC Desire X Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2837178 = {S99767, 146, { &P16957,&P5096,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2; en-in; XCD35 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2837204 = {S99768, 118, { &P14438,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE50) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile*/
const Device UA2839496 = {S99769, 186, { &P11808,&P5204,&P24672,&P18092 }};	/*Useragent - Nokia5610d-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5610d-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2840406 = {S99770, 125, { &P12260,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5350) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2840604 = {S99771, 131, { &P15767,&P18432,&P20775,&P18092 }};	/*Useragent - MQQBrowser/2.4 Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A523 Safari/7534.48.3*/
const Device UA2841846 = {S99772, 44, { &P15197,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SAMSUNG-GT-B5310/B5310DDJD2)*/
const Device UA2841850 = {S99773, 64, { &P20067,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; A56; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2842877 = {S99774, 69, { &P20175,&P18110,&P24671,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; MI 1S; 480*854) UCWEB8.5.1.179/145/32504*/
const Device UA2843299 = {S99775, 120, { &P15118,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaX7-00) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/355/UCWEB Mobile*/
const Device UA2843630 = {S99776, 154, { &P17625,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2844061 = {S99777, 157, { &P16292,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9050 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2844154 = {S99778, 117, { &P16024,&P17483,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; lg-t310i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2844379 = {S99779, 147, { &P15364,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; 8020 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2845165 = {S99780, 53, { &P11929,&P5890,&P20784,&P18092 }};	/*Useragent - MQQBrowser/3.3 (Nokia5230;SymbianOS/9.1 Series60/3.0)*/
const Device UA2846243 = {S99781, 142, { &P13960,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 1688-0329-V9 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2846245 = {S99782, 107, { &P15364,&P5690,&P18439,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11 slurp*/
const Device UA2847611 = {S99783, 197, { &P15794,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2847724 = {S99784, 142, { &P20250,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Celkon A 200 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2848682 = {S99785, 122, { &P14834,&P5890,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; Nokia5730S-1) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/28/444/UCWEB Mobile*/
const Device UA2850575 = {S99786, 161, { &P14225,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2850613 = {S99787, 148, { &P21903,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U705T Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2850643 = {S99788, 156, { &P13123,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2850822 = {S99789, 165, { &P18715,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID8048 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2851302 = {S99790, 152, { &P14161,&P5096,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; VM670 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000*/
const Device UA2851311 = {S99791, 169, { &P18324,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; LG-VM696 Build/ZV5.GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2853264 = {S99792, 160, { &P16957,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; XCD35 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2853269 = {S99793, 175, { &P13079,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; HUAWEI-U8850 Build/HuaweiU8850) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2853400 = {S99794, 170, { &P21036,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; Spice Mi-425 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2854654 = {S99795, 142, { &P20491,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Thunder S220 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2855749 = {S99796, 175, { &P17283,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; MB860 Build/4.5.1A-129_OLY-145) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2855836 = {S99797, 142, { &P19851,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax A68 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2858746 = {S99798, 152, { &P18743,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; R805 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2859767 = {S99799, 135, { &P22875,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.6; i9220 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2859970 = {S99800, 145, { &P20516,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Woxter Tablet PC 97IPS Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2860021 = {S99801, 145, { &P22944,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Starmobile FLIRT Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2860072 = {S99802, 156, { &P14887,&P5096,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-CN; GT-I9008L Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2860358 = {S99803, 171, { &P12280,&P17090,&P22589,&P18092 }};	/*Useragent - MQQBrowser/42 Mozilla/5.0 (iPhone 4; U; CPU iPhone OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3 tencent(securitypay)*/
const Device UA2860545 = {S99804, 156, { &P16356,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-N7000 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2860562 = {S99805, 152, { &P18739,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; R801 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2860770 = {S99806, 138, { &P21438,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F120K Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2861290 = {S99807, 158, { &P21381,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-CN; Lenovo A520 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2861626 = {S99808, 130, { &P21480,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PMP3270B Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2861919 = {S99809, 161, { &P13123,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-S5360 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2862264 = {S99810, 159, { &P18223,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LT22i Build/6.1.1.B.1.54) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2862715 = {S99811, 142, { &P21856,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax P600 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2862737 = {S99812, 159, { &P21228,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; Lenovo A750 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2862790 = {S99813, 138, { &P14872,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SPH-D700 Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2863031 = {S99814, 142, { &P20073,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tmn smart a15 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2863297 = {S99815, 157, { &P18280,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; HTC One S Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2863486 = {S99816, 154, { &P12896,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-P999 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2864200 = {S99817, 156, { &P16356,&P18111,&P22281,&P18092 }};	/*Useragent - OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-N7000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2864705 = {S99818, 158, { &P18452,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; LT29i Build/9.1.B.0.411) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2866484 = {S99819, 162, { &P15106,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; HTC Salsa C510e Build/GRI40) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2866944 = {S99820, 161, { &P18321,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.4.1000*/
const Device UA2869377 = {S99821, 159, { &P18447,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT30p Build/7.0.A.3.195) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2869981 = {S99822, 154, { &P12968,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; LT26i Build/6.1.A.2.55) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2879531 = {S99823, 166, { &P20283,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; AZ210A Build/AZ210A_ICS_02.27I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2892763 = {S99824, 80, { &P17595,&P17296,&P22169,&P18090 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:21.0) Gecko/X Firefox/21.0 slurp*/
const Device UA2893419 = {S99825, 78, { &P17595,&P17296,&P22768,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:22.0) Gecko/X Thunderbird/22.0*/
const Device UA2894215 = {S99826, 154, { &P20480,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; JY-G3 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2895652 = {S99827, 151, { &P21334,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A8 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2933936 = {S99828, 145, { &P12082,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Xperia Mini Pro Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2935123 = {S99829, 93, { &P15364,&P22784,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxq6) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2935259 = {S99830, 102, { &P15364,&P5129,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5235) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA2936754 = {S99831, 145, { &P20104,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; 201M Build/9.8.2Q-34_SMJ-102) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2937077 = {S99832, 153, { &P16362,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2937921 = {S99833, 137, { &P22390,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S5310 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937993 = {S99834, 131, { &P20187,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Novo7Fire Build/JDQ39B) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2938073 = {S99835, 137, { &P12945,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A780L Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2938218 = {S99836, 135, { &P22085,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ThL W5 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2939572 = {S99837, 166, { &P23118,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Aqua Star Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2939596 = {S99838, 137, { &P23518,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ZTE V987 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2949186 = {S99839, 69, { &P21401,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux; U;2.3.4;zh_cn;Lenovo A68e;320*480;)UCWEB7.6.0.75/139/32396*/
const Device UA2951839 = {S99840, 161, { &P19971,&P17485,&P24673,&P18092 }};	/*Useragent - SonyEricssonT700/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssont700) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2953917 = {S99841, 94, { &P15364,&P5890,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Symbian; U; S60 V3; en-US; Nokia 6210s) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2956491 = {S99842, 142, { &P23161,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Xtreamer Joyz Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2956769 = {S99843, 158, { &P18507,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I8160 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2957860 = {S99844, 142, { &P20837,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; IRIS_501 Build/LAVAIRIS501) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2958073 = {S99845, 156, { &P23776,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGP321 Build/10.1.1.A.1.253) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2958235 = {S99846, 156, { &P18240,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-P700 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2958774 = {S99847, 168, { &P15438,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Z710a Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2963221 = {S99848, 138, { &P24488,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Discovery Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2978263 = {S99849, 85, { &P15545,&P5448,&P4779,&P18092 }};	/*Useragent - Palm850/v0100 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA2978594 = {S99850, 42, { &P15547,&P17484,&P4759,&P18092 }};	/*Useragent - samsung-gt-c6712/UC Browser7.6.1.82/69/351*/
const Device UA2980692 = {S99851, 35, { &P14711,&P5204,&P21759,&P18092 }};	/*Useragent - iBrowser/Mini2.8 (NokiaC2-01/11.10)*/
const Device UA2981420 = {S99852, 142, { &P21143,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i809 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2981602 = {S99853, 179, { &P13999,&P24691,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; HTC HD2 Build/IMM76I; CyanogenMod-9.0.0 RC0/AOKP/Desire C lib) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2982061 = {S99854, 147, { &P21057,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; A7+ Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982107 = {S99855, 252, { &P14669,&P5843,&P18520,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.5.0.185/83/352*/
const Device UA2982563 = {S99856, 71, { &P18283,&P18111,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.1.1;Zh_cn;GT-I9300;720*1280;)UCWEB7.8.0.95/139/444*/
const Device UA2982891 = {S99857, 33, { &P15685,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SAMSUNG-GT-C3752)*/
const Device UA2983818 = {S99858, 154, { &P21381,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A520 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2984429 = {S99859, 66, { &P13025,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; WT19a; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2986093 = {S99860, 99, { &P14799,&P5477,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT-I5500) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2986167 = {S99861, 95, { &P15865,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; HTC_Status) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2986201 = {S99862, 166, { &P16553,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; SGH-T959V Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2986627 = {S99863, 148, { &P18740,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; U701T Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2987658 = {S99864, 174, { &P13064,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; SAMSUNG-SGH-I727 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2988326 = {S99865, 135, { &P23758,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i9430 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2988331 = {S99866, 159, { &P23974,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Telenor_One_Touch_S Build/IceCreamSandwich) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2989072 = {S99867, 160, { &P21242,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; Lenovo A366t Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2989600 = {S99868, 139, { &P19848,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHW-M440S Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2989848 = {S99869, 138, { &P18722,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A830L Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2990012 = {S99870, 176, { &P12731,&P5629,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (MB855, 42, 1.9.1)*/
const Device UA2991355 = {S99871, 93, { &P20407,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ZTE_V791) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2992691 = {S99872, 88, { &P20518,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; A10) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2992997 = {S99873, 159, { &P18214,&P5629,&P22281,&P18092 }};	/*Useragent - OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S7500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2993894 = {S99874, 152, { &P18314,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M032 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2994832 = {S99875, 159, { &P13042,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Incredible S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2995172 = {S99876, 87, { &P23381,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A5) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2996795 = {S99877, 157, { &P23008,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A91 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2996826 = {S99878, 152, { &P23771,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8190N Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2997081 = {S99879, 153, { &P20934,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.7; 5860E Build/GWK74) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA2997295 = {S99880, 161, { &P12026,&P17485,&P24677,&P18092 }};	/*Useragent - SonyEricssonC510/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonc510) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2997633 = {S99881, 160, { &P15047,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.4; SK17i Build/4.0.2.A.0.62) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57453*/
const Device UA2997704 = {S99882, 158, { &P12972,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2997753 = {S99883, 161, { &P20399,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PantechP8010 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2998177 = {S99884, 134, { &P21229,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; iball Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2998494 = {S99885, 153, { &P12945,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A780 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA2999157 = {S99886, 93, { &P13123,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; GT-S5360) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2999338 = {S99887, 156, { &P23206,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Cloud Z5 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2999668 = {S99888, 124, { &P23814,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U2 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2999741 = {S99889, 137, { &P20597,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F100S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2999800 = {S99890, 98, { &P21882,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Micromax P362) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2999813 = {S99891, 156, { &P13591,&P5096,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-US; ZTE-BLADE Build/FRF91) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2999916 = {S99892, 164, { &P18348,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Velocity 4G Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3000797 = {S99893, 168, { &P13157,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; HTC Sensation XE with Beats Audio Z715e Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3001604 = {S99894, 152, { &P22979,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9128V Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3004416 = {S99895, 152, { &P24538,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ALCATEL ONE TOUCH 8008X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3008589 = {S99896, 88, { &P15364,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A12) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA3009532 = {S99897, 157, { &P24150,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; vivo Xplay Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3015429 = {S99898, 156, { &P21953,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Ctrl_V1 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3015906 = {S99899, 157, { &P18612,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC T329t Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3017160 = {S99900, 162, { &P11660,&P17485,&P24678,&P18092 }};	/*Useragent - SonyEricssonC902/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonc902) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3018857 = {S99901, 93, { &P18679,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-B5330) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3022011 = {S99902, 152, { &P20542,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAP5000DUO Build/PrestigioPAP5000DUO) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3022641 = {S99903, 130, { &P20241,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP5570C Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3023047 = {S99904, 137, { &P21261,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9500 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3024118 = {S99905, 151, { &P24539,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6010 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3041101 = {S99906, 83, { &P12580,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-E2121B/E2121BXXKC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA3043587 = {S99907, 152, { &P18740,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOU701 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3044571 = {S99908, 154, { &P20004,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Karbonn A2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3044604 = {S99909, 90, { &P18223,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; LT22i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3044862 = {S99910, 145, { &P11754,&P17485,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  SonyEricssonP990i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA3045419 = {S99911, 93, { &P16273,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; SGH-T679) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3046063 = {S99912, 157, { &P20206,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; ZTE N880E Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3046118 = {S99913, 133, { &P23322,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ru-ru; TM-7043XD Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3047768 = {S99914, 152, { &P21021,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328t Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA3048452 = {S99915, 161, { &P22621,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax P360 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3048591 = {S99916, 155, { &P20004,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Karbonn A2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA3048695 = {S99917, 87, { &P23354,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A3) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA3049668 = {S99918, 155, { &P18283,&P18598,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9300 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3050226 = {S99919, 150, { &P22659,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Cloudfone Thrill 430d Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3050294 = {S99920, 162, { &P16273,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; SGH-T679 Build/GINGERBREAD) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA3055445 = {S99921, 157, { &P18455,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HTC T528d Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3055747 = {S99922, 163, { &P20262,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; Karbonn_A18 Build/KarbonnA18) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3055940 = {S99923, 156, { &P18464,&P18066,&P23482,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; KFJWA Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.1 Mobile Safari/535.19 Silk-Accelerated=false*/
const Device UA3056499 = {S99924, 87, { &P23359,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A2) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3056507 = {S99925, 169, { &P22903,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A62 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056549 = {S99926, 159, { &P18449,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST27i Build/6.1.1.B.1.10) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056564 = {S99927, 159, { &P21228,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; Lenovo A750 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056758 = {S99928, 99, { &P14031,&P5477,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; GT-I5503) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3056907 = {S99929, 93, { &P12568,&P5096,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; GT-B7510) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3057261 = {S99930, 151, { &P21059,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; A90 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3057372 = {S99931, 88, { &P21261,&P18110,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Adr (Linux; U; 4.2.2; zh-cn; GT-I9500 Build/JDQ39.I9500ZCUBMG1;1080*1920)*/
const Device UA3058073 = {S99932, 134, { &P15389,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; U8800 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3063305 = {S99933, 136, { &P13074,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-P930 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3068865 = {S99934, 182, { &P24547,&P24371,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.4; en-us ; SPHS on Hsdroid Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33896*/
const Device UA3070672 = {S99935, 41, { &P12502,&P17485,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SonyEricssonW20i/R7DA062)*/
const Device UA3070838 = {S99936, 34, { &P15364,&P16110,&P18405,&P18092 }};	/*Useragent - Opera/9.21 (Windows NT 6.2; U; en)*/
const Device UA3071025 = {S99937, 101, { &P15364,&P5129,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia5235) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3071051 = {S99938, 139, { &P21068,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; MI-ONE C1 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3071410 = {S99939, 39, { &P11787,&P5204,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia3120classic/10.00)*/
const Device UA3072332 = {S99940, 92, { &P22642,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Ctrl_V3) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3072337 = {S99941, 148, { &P23071,&P18111,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.1; id-id; T2 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3072612 = {S99942, 159, { &P18296,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC One V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA3073227 = {S99943, 151, { &P22604,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I879 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA3073340 = {S99944, 153, { &P20098,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GN800 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3073572 = {S99945, 153, { &P22607,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-S5303 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA3073966 = {S99946, 160, { &P23017,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Spice Mi-502 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3073997 = {S99947, 148, { &P13141,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-ca; SGH-I727R Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3074379 = {S99948, 156, { &P21129,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i929 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3074856 = {S99949, 146, { &P20005,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-us; Micromax A65 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3075376 = {S99950, 128, { &P23846,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; A12 Build/RheaTK) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3076315 = {S99951, 166, { &P23286,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; TECNO F7 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/352*/
const Device UA3076568 = {S99952, 156, { &P18002,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; IM-A760S Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3077415 = {S99953, 166, { &P15880,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; ME865 Build/6.7.2_GC-180-EDS-20) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3077737 = {S99954, 90, { &P22756,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; C1604) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3077965 = {S99955, 139, { &P11747,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Desire Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3078027 = {S99956, 154, { &P23546,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; AD683G Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3078568 = {S99957, 134, { &P14574,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus S Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3079392 = {S99958, 160, { &P19997,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo P700i Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3079521 = {S99959, 95, { &P15501,&P5477,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; E16i) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3079668 = {S99960, 136, { &P21254,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Polaris Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3079804 = {S99961, 159, { &P20452,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; Lenovo S880 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3080026 = {S99962, 153, { &P16950,&P6437,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.2; Dell Streak 7 Build/HTJ85B) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3080518 = {S99963, 156, { &P13123,&P5629,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3084805 = {S99964, 170, { &P15736,&P17605,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2232/1.0 RTK-E/1.0 MMP/1.0 Release/02.11.2011 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA3085653 = {S99965, 137, { &P20330,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Micromax P350 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3087428 = {S99966, 139, { &P12280,&P19826,&P20779,&P18092 }};	/*Useragent - MQQBrowser/28 Mozilla/5.0 (iPhone; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3*/
const Device UA3087546 = {S99967, 139, { &P21014,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; vivo S11 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3087625 = {S99968, 157, { &P18283,&P18598,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9300 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA3087899 = {S99969, 173, { &P21858,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HUAWEI Y300-0000 Build/HuaweiY300-0000) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3088195 = {S99970, 161, { &P22752,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Micromax P560 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3088393 = {S99971, 155, { &P19920,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-MS870 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3088571 = {S99972, 198, { &P18295,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC One X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.2/552,2.0.2.552,2013-05-22 23:05, +0100, gb*/
const Device UA3088619 = {S99973, 134, { &P15365,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Magic Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3089111 = {S99974, 133, { &P23497,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; P-03E Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3091570 = {S99975, 156, { &P24557,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9128 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3091859 = {S99976, 104, { &P11868,&P5768,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6280) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile TRUSTED/1.0*/
const Device UA3092287 = {S99977, 93, { &P18507,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I8160) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3092326 = {S99978, 142, { &P20181,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0; Swipe_Fablet_F1 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3092388 = {S99979, 92, { &P21737,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; IRIS355) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3092393 = {S99980, 93, { &P14877,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; GT-I9000) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3093045 = {S99981, 156, { &P22596,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SCH-R970 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3093142 = {S99982, 137, { &P17346,&P19826,&P24341,&P18092 }};	/*Useragent - MQQBrowser/44 Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3*/
const Device UA3097905 = {S99983, 118, { &P15364,&P17605,&P4392,&P18092 }};	/*Useragent - SAMSUNG-GT-S3778/1.0 RTK-E/1.0 DF3G/1.0 Release/04.04.2011 Browser/NetFront4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA3098362 = {S99984, 135, { &P22384,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; PTT1026 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3099042 = {S99985, 83, { &P12258,&P17483,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA3099159 = {S99986, 31, { &P12989,&P5843,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (Nokia303/14.60)*/
const Device UA3099484 = {S99987, 157, { &P21155,&P18066,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; Lenovo A800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3099767 = {S99988, 135, { &P15364,&P17146,&P17776,&P18092 }};	/*Useragent - TOUCH87 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3*/
const Device UA3100034 = {S99989, 152, { &P20515,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; A54 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3100075 = {S99990, 159, { &P15364,&P5629,&P20787,&P18092 }};	/*Useragent - MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; JX0189 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3100254 = {S99991, 139, { &P22131,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; PB-S3-77 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3100602 = {S99992, 103, { &P15118,&P5055,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiax7-00) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3100690 = {S99993, 155, { &P15575,&P18111,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I9100G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3100758 = {S99994, 139, { &P23228,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I8258 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3101666 = {S99995, 163, { &P18280,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; HTC One S Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA3101703 = {S99996, 93, { &P15802,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; HTC_Ruby) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3102429 = {S99997, 92, { &P18687,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.3; en-US; LG-P705) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA3102660 = {S99998, 155, { &P12616,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; WT19i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA3102912 = {S99999, 158, { &P20868,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; M040 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA3103620 = {S100000, 155, { &P22916,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-E450 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3103965 = {S100001, 135, { &P20004,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-us; Karbonn A2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3104001 = {S100002, 153, { &P18303,&P5629,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MT27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3104465 = {S100003, 132, { &P13070,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.6; en-us; LG-E739 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3104942 = {S100004, 161, { &P20330,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; Micromax P350 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3105562 = {S100005, 144, { &P23122,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.3; en-us; Karbonn A6 Build/MocorDroid4.0.3) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3106124 = {S100006, 163, { &P15881,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Incredible 2 Build/JDQ39E; CyanogenMod-10.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3106376 = {S100007, 167, { &P20931,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; iBall Slide 3G 7334 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3106646 = {S100008, 89, { &P18721,&P18066,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A90S) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3106818 = {S100009, 178, { &P14635,&P5843,&P22881,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (08.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291*/
const Device UA3107007 = {S100010, 161, { &P18302,&P18066,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; MT25i Build/4.1.B.0.631) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300*/
const Device UA3107331 = {S100011, 107, { &P18088,&P18588,&P17370,&P18089 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/5.0 (compatible; Googlebot/20.2497; U; en) Presto/2.5.25;c2VvLWNyYXdsZXI=*/
const Device UA3107348 = {S100012, 107, { &P18254,&P18588,&P17470,&P18091 }};	/*Useragent - Mozilla/5.0 (Windows;) HealthSTAT Crawler 1.0 For problems contact: crawler@healthline.com;c2VvLWNyYXdsZXI=*/
const Device UA3107424 = {S100013, 167, { &P15364,&P18588,&P17191,&P18089 }};	/*Useragent - Googlebot/2.1 (http://www.googlebot.com/bot.html; MSIE; Windows; Trident/4.0; FunWebProducts; GTB6.5; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.;c2VvLWNyYXdsZXI=*/
const Device UA3107548 = {S100014, 146, { &P18254,&P18110,&P17470,&P18091 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.5; aggregator:Spinn3r (Spinn3r 3.5 / Crawler); http://spinn3r.com/robot) Gecko/X;c2VvLWNyYXdsZXI=*/
const Device UA3107715 = {S100015, 108, { &P15364,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_4010D) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3107797 = {S100016, 173, { &P20169,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; N320 Build/LAVALAVA_N320) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3107917 = {S100017, 160, { &P19863,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3108012 = {S100018, 153, { &P19788,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; E350 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3108025 = {S100019, 93, { &P15364,&P18598,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.2; en-US; Cloud_X3) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3108109 = {S100020, 158, { &P11950,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; X10i Build/3.0.1.G.0.75) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3108118 = {S100021, 134, { &P13941,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; TS7 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3108162 = {S100022, 162, { &P15438,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; HTC Sensation Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3108180 = {S100023, 180, { &P12473,&P5204,&P24055,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311*/
const Device UA3108222 = {S100024, 103, { &P15364,&P17979,&P24717,&P18092 }};	/*Useragent - Instagram 4.1.2 Android (10/2.3.5; 240dpi; 480x800; HTC/sprint; PC36100; supersonic; supersonic; en_US)*/
const Device UA3108260 = {S100025, 171, { &P20002,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 992D Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3108347 = {S100026, 160, { &P21602,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5302 Build/12.0.A.2.245) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3108730 = {S100027, 174, { &P18382,&P18066,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC_Desire_C Build/IML74K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3108763 = {S100028, 135, { &P18283,&P21637,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-I9300 Build/JSS15J) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3108956 = {S100029, 148, { &P24070,&P18598,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; R815T Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3109182 = {S100030, 135, { &P20416,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; F-04E Build/V14R31C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110731 = {S100031, 145, { &P20104,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; 201M Build/7.7.1Q-164_SMJ-165) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3110977 = {S100032, 137, { &P18630,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A800S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3115272 = {S100033, 102, { &P18283,&P18110,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-I9300) U2/1.0.0 UCBrowser/9.3.0.321 Mobile*/
const Device UA3118515 = {S100034, 190, { &P19894,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SCH-R830 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:38, us*/
const Device UA3118593 = {S100035, 153, { &P21597,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Lai 502 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3118774 = {S100036, 143, { &P13018,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; Xperia Neo V Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3118892 = {S100037, 101, { &P14183,&P17484,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; SAMSUNG-SGH-I997) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3118908 = {S100038, 130, { &P20364,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; ELF-II Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3118950 = {S100039, 170, { &P15575,&P18066,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-I9100 Build/IML74K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA3119030 = {S100040, 126, { &P21052,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A9 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3119141 = {S100041, 112, { &P17595,&P22470,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Darwin Power Macintosh; en-US; rv:1.8.0.12) Gecko/X Firefox/1.5.0.12 Fink Community Edition*/
const Device UA3120327 = {S100042, 151, { &P21003,&P20706,&P24673,&P18092 }};	/*Useragent - Nokia310/2.0 (07.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia310) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA3120388 = {S100043, 170, { &P18114,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Transformer Prime TF201 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30; Maxthon (4.0.4.1000);*/
const Device UA3120911 = {S100044, 140, { &P22132,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SOWA_S6600 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3121020 = {S100045, 178, { &P19747,&P5043,&P24672,&P18092 }};	/*Useragent - Nokia206/2.0 (03.58) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia206) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/84/352/UCWEB Mobile*/
const Device UA3121035 = {S100046, 137, { &P21417,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabA2207A-H Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3121077 = {S100047, 33, { &P15364,&P22843,&P21720,&P18092 }};	/*Useragent - iBrowser/2.0,Lava-Discover132/2.0*/
const Device UA3121422 = {S100048, 92, { &P23011,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; iris405) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA3121512 = {S100049, 148, { &P21802,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-tw; GSmart Maya M1 v2 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3123240 = {S100050, 159, { &P12968,&P18111,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LT26i Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA3123307 = {S100051, 129, { &P19907,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; C5155 Build/IML77) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3124292 = {S100052, 128, { &P20518,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; A10 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3124998 = {S100053, 140, { &P18582,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LT28h Build/6.2.B.0.211) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA3125134 = {S100054, 110, { &P15364,&P22468,&P22813,&P18092 }};	/*Useragent - LinkedInBlackBerry/10.1.4 (1) Q5 SQR100-2/10.1.0.4633 (Research In Motion;RIM BlackBerry Device;Q5;SQR100-2;1)*/
const Device UA3125301 = {S100055, 158, { &P14017,&P24162,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; LG-P990 Build/IMM76L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.2/p990*/
const Device UA3125505 = {S100056, 169, { &P14186,&P5096,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5830 Build/FROYO) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3125514 = {S100057, 148, { &P23304,&P18066,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R813T Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3125558 = {S100058, 90, { &P15397,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST18i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3125622 = {S100059, 170, { &P23000,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; TECNO_P3 Build/master) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3125655 = {S100060, 156, { &P24557,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I9128 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3125717 = {S100061, 90, { &P20175,&P18066,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; zh-CN; MI 1S) U2/1.0.0 UCBrowser/9.2.4.329 U2/1.0.0 Mobile*/
const Device UA3125755 = {S100062, 163, { &P15483,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; HTC ChaCha A810e Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3125802 = {S100063, 93, { &P14417,&P5129,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5236) U2/1.0.0 UCBrowser/9.1.0.319 U2/1.0.0 Mobile*/
const Device UA3125816 = {S100064, 90, { &P13018,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; MT11i) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3125826 = {S100065, 169, { &P22864,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A26 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3125844 = {S100066, 88, { &P20158,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A7*) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3125966 = {S100067, 64, { &P15364,&P5690,&P22772,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:24.0) Gecko/X Firefox/24.0 slurp*/
const Device UA3125996 = {S100068, 89, { &P21963,&P18111,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Q800) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3126167 = {S100069, 93, { &P19938,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; SCH-I589) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3126182 = {S100070, 152, { &P18466,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; KFTT Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126217 = {S100071, 142, { &P21304,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; IQ450_Quattro Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126218 = {S100072, 157, { &P15850,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; LG-LU6200 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126400 = {S100073, 137, { &P12841,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Vivid 4G Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126438 = {S100074, 136, { &P24631,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F240 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126464 = {S100075, 142, { &P20507,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; tablet Fnac 7 Build/1.0.3 X-12:51) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126470 = {S100076, 136, { &P19902,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SCH-R530M Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126562 = {S100077, 142, { &P20880,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NEO_N002 Build/NeoNEO_N002) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126608 = {S100078, 135, { &P23889,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PAD703 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126653 = {S100079, 134, { &P18113,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; XT912 Build/JSS15Q) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3126795 = {S100080, 136, { &P22681,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; UMI X1S Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126856 = {S100081, 137, { &P18747,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A840S Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126869 = {S100082, 158, { &P15147,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST15i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126900 = {S100083, 159, { &P21991,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; Aqua Wonder Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126909 = {S100084, 102, { &P22382,&P18110,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0(Linux; U; Opera Mini/7.1.32052/30.3697; en-US; GT-I8262) U2/1.0.0 UCBrowser/9.3.0.321 Mobile*/
const Device UA3126953 = {S100085, 137, { &P20597,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-F100S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127011 = {S100086, 148, { &P18227,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HUAWEI U9510E Build/HuaweiU9510E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127130 = {S100087, 132, { &P20729,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A27 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127322 = {S100088, 138, { &P24511,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SHV-E270L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127352 = {S100089, 137, { &P19865,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A850L Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127445 = {S100090, 142, { &P23541,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE Grand Era Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127471 = {S100091, 165, { &P18645,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SAMSUNG-SGH-I747 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3127478 = {S100092, 163, { &P22906,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo A390_ROW Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127516 = {S100093, 159, { &P15147,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; ST15i Build/4.0.2.A.0.84) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127546 = {S100094, 160, { &P21850,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Spice Mi-435 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127691 = {S100095, 157, { &P21506,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-S7710 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3127749 = {S100096, 138, { &P20511,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Fly_IQ442 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3128129 = {S100097, 142, { &P12280,&P22882,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X; zh-CN) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A465 UCBrowser/9.1.0.287 Mobile*/
const Device UA3128319 = {S100098, 159, { &P18496,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6410LVW Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3129564 = {S100099, 152, { &P19747,&P5043,&P24680,&P18092 }};	/*Useragent - Nokia206/2.0 (03.59) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia206) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3130308 = {S100100, 160, { &P18223,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT22i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3130694 = {S100101, 170, { &P18505,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; MB886 Build/9.8.0Q-97_MB886_FFW-20) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3130697 = {S100102, 172, { &P15877,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT910 Build/6.7.2-180_SPU-19-TA-11.6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3131384 = {S100103, 149, { &P21793,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9205 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3132977 = {S100104, 138, { &P23969,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8190T Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA33635 = {S100105, 55, { &P19968,&P17485,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 SonyEricssonW910iv/R1EA SEMC/NewsReader/1.0*/
const Device UA50184 = {S100106, 62, { &P19968,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW910i/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2981 = {S100107, 12, { &P17897,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU  B832*/
const Device UA101652 = {S100108, 49, { &P17897,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/B832/MIDP2.0/CLDC1.1/Screen-240x320*/
const Device UA4584 = {S100109, 150, { &P15365,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android RCMagicv2.1_v6.0; es-es; HTC Magic Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA5038 = {S100110, 52, { &P13334,&P17483,&P17660,&P18092 }};	/*Useragent - LG-GD900/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA13424 = {S100111, 138, { &P13296,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SonyEricssonX1i/R3AA Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA71766 = {S100112, 81, { &P13296,&P5448,&P4808,&P18092 }};	/*Useragent - X1i Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA19010 = {S100113, 87, { &P14845,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch_HD/2.0; Windows Phone 6.5)*/
const Device UA69514 = {S100114, 89, { &P14845,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch_HD_T8282; Windows Phone 6.5)*/
const Device UA438823 = {S100115, 93, { &P14845,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_Touch_HD_T8282; Windows Phone 6.5.3.5)*/
const Device UA925891 = {S100116, 85, { &P14845,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Touch_HD_T8282; Windows Phone 6.5)*/
const Device UA20115 = {S100117, 68, { &P14233,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-C650/0B.D2.23R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA20398 = {S100118, 86, { &P15741,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-C701A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA24712 = {S100119, 85, { &P15741,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-C701/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA22591 = {S100120, 38, { &P13949,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-G-N290-BASE-P107A2V1.0.0 Obigo/Q03*/
const Device UA66997 = {S100121, 116, { &P13838,&P5776,&P4715,&P18092 }};	/*Useragent - Nokia6600/1.0 (5.53.0) SymbianOS/7.0s Series60/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.4.0.65/27/351*/
const Device UA23518 = {S100122, 41, { &P15611,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP15xQ Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA23985 = {S100123, 123, { &P15644,&P17017,&P4223,&P18092 }};	/*Useragent - CoolPadF600_TD/CMCC ThreadX/1.0 LA2KPH/SW2.0 Release/2010.08.10 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA26626 = {S100124, 107, { &P13612,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-A517/A517UCGG5 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA27965 = {S100125, 39, { &P12195,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GS190 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA61694 = {S100126, 67, { &P12268,&P6712,&P4030,&P18092 }};	/*Useragent - Nokia6202c_CMCC/2.0 (04.51) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA30271 = {S100127, 45, { &P12295,&P6476,&P4902,&P18092 }};	/*Useragent - Samsung-SCHR100 Infraware-Polaris/6.1 MMP/2.0*/
const Device UA30901 = {S100128, 100, { &P12364,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-T636/UEHI1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA33297 = {S100129, 55, { &P12894,&P6476,&P3625,&P18092 }};	/*Useragent - LGE-PLS225/1.0 UP.Browser/6.2.3.7.e.1.101 (GUI) MMP/2.0*/
const Device UA562418 = {S100130, 113, { &P12108,&P5843,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia7230/10.81; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.0.2.26.6*/
const Device UA36876 = {S100131, 147, { &P12216,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Cricket Froyo 2.2; en-us; ADR6300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA53354 = {S100132, 86, { &P13971,&P17662,&P3601,&P18092 }};	/*Useragent - SoftBank/1.0/820SC/SCJ002 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA57111 = {S100133, 42, { &P13971,&P17662,&P3601,&P18092 }};	/*Useragent - Mozilla/4.08 (820SC;SoftBank) NetFront/3.3*/
const Device UA2018790 = {S100134, 135, { &P21562,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; in-id; NX-A892 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA50013 = {S100135, 190, { &P12806,&P24389,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.3 Nokia600/111.010.1009; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.4.1.12 Mobile Safari/533.4 3gpp-gba*/
const Device UA49372 = {S100136, 29, { &P13003,&P17017,&P4924,&P18092 }};	/*Useragent - C5600/001.00 ACS-NetFront/3.2*/
const Device UA49375 = {S100137, 70, { &P14138,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-F188/1.0 ACS-NF/3.4 QTV2.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA60925 = {S100138, 74, { &P14138,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-F188-CSL/1.0 ACS-NF/3.4 QTV2.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA50294 = {S100139, 145, { &P17234,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-tw; Vibo-A688 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA54063 = {S100140, 112, { &P11782,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-S3600/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA55025 = {S100141, 73, { &P13609,&P7063,&P3606,&P18092 }};	/*Useragent - BlackBerry7250/4.1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105*/
const Device UA61120 = {S100142, 91, { &P12636,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SAMSUNG/X01SC*/
const Device UA62256 = {S100143, 43, { &P14391,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GB175/V01 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA63000 = {S100144, 39, { &P14469,&P17486,&P3807,&P18092 }};	/*Useragent - Mot-WX288/1.0 ObigoInternetBrowser/Q03C*/
const Device UA64505 = {S100145, 87, { &P14676,&P5890,&P4615,&P18092 }};	/*Useragent - Nokia3660/1.0 (5.08) SymbianOS/6.1 Series60/1.2 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA64739 = {S100146, 87, { &P14676,&P5890,&P4615,&P18092 }};	/*Useragent - Nokia3660/1.0 (4.57) SymbianOS/6.1 Series60/0.9 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA64755 = {S100147, 62, { &P14702,&P6476,&P3601,&P18092 }};	/*Useragent - Mozilla/4.0 (MobilePhone SCP-8400/US/1.0) NetFront/3.3 MMP/2.0*/
const Device UA67071 = {S100148, 36, { &P12193,&P5129,&P4715,&P18092 }};	/*Useragent - NokiaC5-00/UC Browser7.4.0.65/28/351*/
const Device UA67149 = {S100149, 80, { &P12274,&P5448,&P4715,&P18092 }};	/*Useragent - SAMSUNG-SGH-i900/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65*/
const Device UA67897 = {S100150, 88, { &P11978,&P5448,&P4715,&P18092 }};	/*Useragent - HTC_Touch2_T3333 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65*/
const Device UA68169 = {S100151, 85, { &P13999,&P5448,&P4759,&P18092 }};	/*Useragent - HTC_HD2_T8585 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.6.1.82*/
const Device UA68367 = {S100152, 88, { &P11978,&P5448,&P4759,&P18092 }};	/*Useragent - HTC_Touch2_T3333 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.6.1.82*/
const Device UA68895 = {S100153, 116, { &P13838,&P5776,&P4777,&P18092 }};	/*Useragent - Nokia6600/1.0 (5.53.0) SymbianOS/7.0s Series60/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.7.1.88/27/444*/
const Device UA68559 = {S100154, 15, { &P14933,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHC130/1.0*/
const Device UA80250 = {S100155, 60, { &P13609,&P6208,&P4420,&P18092 }};	/*Useragent - BlackBerry7250/4.0.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA85522 = {S100156, 32, { &P13209,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P900i(c100;TB;W24H11)*/
const Device UA86844 = {S100157, 69, { &P16238,&P17017,&P3534,&P18092 }};	/*Useragent - COCOON/R01 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA126341 = {S100158, 49, { &P17890,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/H777/MIDP2.0/CLDC1.1/Screen-320X240*/
const Device UA250952 = {S100159, 71, { &P17595,&P17297,&P21580,&P18092 }};	/*Useragent - Mac OS X/10.7.2 (11C74); ExchangeWebServices/2.0 (123); Mail/5.1 (1251)*/
const Device UA253903 = {S100160, 15, { &P12399,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonU8i*/
const Device UA359104 = {S100161, 135, { &P16044,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Touchpad Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA396678 = {S100162, 52, { &P15364,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (NokiaNXX;SymbianOS/9.1 Series60/3.0)*/
const Device UA2742410 = {S100163, 91, { &P11639,&P6208,&P4779,&P18092 }};	/*Useragent - BlackBerry9000/4.0.0.248 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/159/444*/
const Device UA451993 = {S100164, 76, { &P17595,&P17296,&P20795,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:2.0.1) Gecko/X Postbox/2.5.3*/
const Device UA2018318 = {S100165, 133, { &P12189,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; U8220 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022003 = {S100166, 139, { &P12189,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; U8220/Pulse Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA492203 = {S100167, 139, { &P12530,&P5448,&P4715,&P18092 }};	/*Useragent - PPC; 240x320; HTC_P3450/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65*/
const Device UA493229 = {S100168, 67, { &P15364,&P7133,&P20418,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 6.1) Opera 7.50  [en]*/
const Device UA501555 = {S100169, 83, { &P15364,&P17350,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 2.0; en-US; rv:1.8.1.6) Gecko/X Firefox/2.0.0.6*/
const Device UA518633 = {S100170, 76, { &P15364,&P17188,&P17202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; WinNT4.0; pl; rv:1.8.1.20) Gecko/X Firefox/2.0.0.20*/
const Device UA560169 = {S100171, 36, { &P12456,&P5890,&P4759,&P18092 }};	/*Useragent - nokian73-1/UC Browser7.6.1.82/69/352*/
const Device UA572549 = {S100172, 44, { &P17595,&P17295,&P21581,&P18092 }};	/*Useragent - MacOutlook/14.13.0.X (Intel Mac OS X 10.5.8)*/
const Device UA572964 = {S100173, 37, { &P15655,&P17484,&P24665,&P18092 }};	/*Useragent - samsung-gt-s3850/UCWEB8.0.3.99/70/350*/
const Device UA574026 = {S100174, 106, { &P11660,&P17485,&P17338,&P18092 }};	/*Useragent - SonyEricssonC902/R3EG Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonc902/UC Browser7.9.0.102/69/352*/
const Device UA574827 = {S100175, 44, { &P17595,&P17297,&P21581,&P18092 }};	/*Useragent - MacOutlook/14.13.0.X (Intel Mac OS X 10.7.2)*/
const Device UA579046 = {S100176, 106, { &P11944,&P17485,&P17347,&P18092 }};	/*Useragent - SonyEricssonT715/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssont715/UC Browser8.0.3.107/69/444*/
const Device UA641362 = {S100177, 97, { &P14781,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; mwp6985)*/
const Device UA706100 = {S100178, 86, { &P15364,&P7133,&P22339,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:9.0.1) Gecko/X Firefox/9.0.1 CometBrowser/9.0.1*/
const Device UA717204 = {S100179, 69, { &P17346,&P17144,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8A293 (iPod; U; CPU OS 4_0 like Mac OS X; en_us)*/
const Device UA2209125 = {S100180, 71, { &P17346,&P17144,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0.8A400 (iPod; U; CPU OS 4_0_2 like Mac OS X; en_us)*/
const Device UA723879 = {S100181, 43, { &P15655,&P17484,&P17338,&P18092 }};	/*Useragent - samsung-gt-s3850/UC Browser7.9.0.102/69/444*/
const Device UA740436 = {S100182, 135, { &P13991,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; ADR6400L Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA745627 = {S100183, 170, { &P15364,&P17296,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; http://device.sprintpcs.com/HTC/APA9292KT/X.rdf; en-us) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16*/
const Device UA792020 = {S100184, 50, { &P15364,&P17017,&P18097,&P18092 }};	/*Useragent - Mozilla/5.0 (SaferSurf rv:9.0) Gecko/X Firefox/9.0*/
const Device UA837030 = {S100185, 92, { &P15364,&P17285,&P21807,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.19pre) Gecko/X Prism zdesktop/7.1.4*/
const Device UA838851 = {S100186, 42, { &P14010,&P17484,&P4779,&P18092 }};	/*Useragent - samsung-gt-s5263/UC Browser7.8.0.95/69/351*/
const Device UA849990 = {S100187, 148, { &P12280,&P17090,&P20784,&P18092 }};	/*Useragent - MQQBrowser/33 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3*/
const Device UA1992217 = {S100188, 148, { &P12280,&P17090,&P20784,&P18092 }};	/*Useragent - MQQBrowser/33 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3*/
const Device UA874164 = {S100189, 116, { &P15364,&P7127,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA925771 = {S100190, 96, { &P14726,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; LG; LG-C900)*/
const Device UA925861 = {S100191, 97, { &P14726,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; LG; LG-C900k)*/
const Device UA925887 = {S100192, 97, { &P14726,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; LG; LG-C900B)*/
const Device UA1864446 = {S100193, 71, { &P15364,&P6704,&P20904,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.7.5) Gecko/X Netscape/8.1.3*/
const Device UA1901576 = {S100194, 98, { &P11699,&P5768,&P17338,&P18092 }};	/*Useragent - Nokia5200/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia5200/UC Browser7.9.0.102/69/352*/
const Device UA1901667 = {S100195, 69, { &P14123,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2.2;Zh_cn;LG-P970;480*800;)UCWEB7.8.0.95/139/351*/
const Device UA1902558 = {S100196, 66, { &P15201,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.4;Zh_cn;HTC Z710e;540*960;)UCWEB7.6.0.75/139/32035*/
const Device UA1904058 = {S100197, 73, { &P15364,&P5690,&P22537,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:5.0) Gecko/X Firefox/5.0 Kylo/1.0.1.76141*/
const Device UA1904524 = {S100198, 76, { &P17595,&P17296,&P18193,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:7.0) Gecko/X Thunderbird/7.0*/
const Device UA1904897 = {S100199, 47, { &P13648,&P5055,&P17338,&P18092 }};	/*Useragent - nokiae7-00/UC Browser7.9.0.102/69/352  3gpp-gba*/
const Device UA1908646 = {S100200, 86, { &P17643,&P5567,&P4536,&P18092 }};	/*Useragent - Mozilla/5.0 Opera/9.5 (KDDI-SN3T; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00*/
const Device UA1919092 = {S100201, 138, { &P21470,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; ru-ru; ZTE Roamer Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1991885 = {S100202, 171, { &P20267,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; P300(Funbook) Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2034275 = {S100203, 169, { &P15147,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; ST15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2026686 = {S100204, 135, { &P22544,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; Vantium Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027802 = {S100205, 155, { &P18700,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-B5512 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028340 = {S100206, 153, { &P14799,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Galaxy5 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029481 = {S100207, 99, { &P13051,&P17979,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/ZTE-TU880_TD/1.0 Linux/2.6.32 Android/2.2 Release/5.25.2011 Browser/AppleWebKit533.1*/
const Device UA2033370 = {S100208, 157, { &P22875,&P5629,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2132393 = {S100209, 75, { &P15364,&P18140,&P4812,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; Linux Mint; en) Presto/2.2.15 Version/10.00*/
const Device UA2141320 = {S100210, 42, { &P12080,&P17484,&P4779,&P18092 }};	/*Useragent - samsung-gt-s3653/UC Browser7.8.0.95/69/444*/
const Device UA2145611 = {S100211, 103, { &P12040,&P17483,&P17338,&P18092 }};	/*Useragent - LG-GW300/V100 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1  lg-gw300/UC Browser7.9.0.102/69/352*/
const Device UA2147186 = {S100212, 79, { &P15364,&P5690,&P22337,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:7.0.1) Gecko/X Firefox/7.0.1 CometBrowser/7.0.1*/
const Device UA2148242 = {S100213, 45, { &P15364,&P17662,&P3976,&P18092 }};	/*Useragent - Mozilla/5.0 (830SH;SoftBank;SNX) NetFront/3.4*/
const Device UA2148378 = {S100214, 74, { &P15364,&P23114,&P20458,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:8.0.1) Gecko/X Firefox/8.0.1 SeaMonkey/2.5*/
const Device UA2160353 = {S100215, 90, { &P11932,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA6210 Navigator)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA2162014 = {S100216, 85, { &P14404,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s5260) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2167766 = {S100217, 152, { &P15575,&P18066,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2209141 = {S100218, 99, { &P17595,&P17296,&P22510,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.0.9) Gecko/X Firefox/3.0.9 Wyzo/3.0.3*/
const Device UA2210506 = {S100219, 66, { &P15364,&P17285,&P17203,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:2.0.1) Gecko/X Firefox/4.0.1 slurp*/
const Device UA2241402 = {S100220, 146, { &P18385,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus 7 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2241404 = {S100221, 157, { &P18385,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Nexus 7 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2242094 = {S100222, 150, { &P18385,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Nexus 7 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2242095 = {S100223, 157, { &P18385,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; Nexus 7 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2258707 = {S100224, 123, { &P15364,&P5690,&P22404,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.0.0.0 Chrome/12.0.742.91 Safari/534.30*/
const Device UA2259088 = {S100225, 127, { &P15364,&P7133,&P18368,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64; rv:8.0) Gecko/X Thunderbird/8.0 OracleBeehiveExtension/1.0.0.4-OracleInternal Lightning/1.0*/
const Device UA2261209 = {S100226, 140, { &P11625,&P5768,&P24668,&P18092 }};	/*Useragent - Nokia5300/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia5300) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2268715 = {S100227, 42, { &P12137,&P17484,&P4715,&P18092 }};	/*Useragent - samsung-gt-s5620/UC Browser7.4.0.65/69/351*/
const Device UA2273407 = {S100228, 42, { &P12080,&P17484,&P4742,&P18092 }};	/*Useragent - samsung-gt-s3653/UC Browser7.5.1.77/69/352*/
const Device UA2286097 = {S100229, 46, { &P14983,&P5055,&P4742,&P18092 }};	/*Useragent - nokiac7-00/UC Browser7.5.1.77/69/351  3gpp-gba*/
const Device UA2299052 = {S100230, 67, { &P15364,&P23098,&P17470,&P18092 }};	/*Useragent - curl/7.7.2 (powerpc-apple-darwin6.0) libcurl 7.7.2 (OpenSSL 0.9.6b)*/
const Device UA2300805 = {S100231, 37, { &P12137,&P17484,&P24665,&P18092 }};	/*Useragent - samsung-gt-s5620/UCWEB8.0.3.99/70/350*/
const Device UA2300999 = {S100232, 114, { &P14714,&P5843,&P17437,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02/05.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.4.0.34.8*/
const Device UA2304614 = {S100233, 115, { &P13107,&P5043,&P17438,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaC2-06/06.51; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.5.0.34.15*/
const Device UA2307024 = {S100234, 35, { &P11689,&P5890,&P17338,&P18092 }};	/*Useragent - nokian70/UC Browser7.9.0.102/69/352*/
const Device UA2392293 = {S100235, 37, { &P11689,&P5890,&P17338,&P18092 }};	/*Useragent - nokian70-1/UC Browser7.9.0.102/70/352*/
const Device UA2307153 = {S100236, 84, { &P11751,&P17483,&P17347,&P18092 }};	/*Useragent - LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  lg-kp500 teleca/UC Browser8.0.3.107/69/444*/
const Device UA2308893 = {S100237, 100, { &P14693,&P5843,&P17347,&P18092 }};	/*Useragent - NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac1-02/UC Browser8.0.3.107/69/444*/
const Device UA2322372 = {S100238, 79, { &P16871,&P22784,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; micromaxq5) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2329038 = {S100239, 83, { &P13979,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; Nokia6220c-1)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA2338482 = {S100240, 148, { &P11953,&P17485,&P24670,&P18092 }};	/*Useragent - SonyEricssonU10i/R7BA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonu10i) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2364870 = {S100241, 153, { &P18714,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V889D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2366752 = {S100242, 80, { &P13856,&P5567,&P17371,&P18092 }};	/*Useragent - Opera/9.80 (BREW; Opera Mini/5.1/27.2239; U; en) Presto/2.8.119 400X240 LG VN270*/
const Device UA2370389 = {S100243, 106, { &P11698,&P17485,&P17338,&P18092 }};	/*Useragent - SonyEricssonW995/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonw995/UC Browser7.9.0.102/69/352*/
const Device UA2373783 = {S100244, 80, { &P13536,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA6110)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA3042557 = {S100245, 116, { &P12856,&P17483,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; lg-t300) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2393517 = {S100246, 148, { &P11953,&P17485,&P24668,&P18092 }};	/*Useragent - SonyEricssonU10i/R7BA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonu10i) UCBrowser8.3.0.154/69/352/UCWEB Mobile*/
const Device UA2400993 = {S100247, 166, { &P24555,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; MTS-SP100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2411454 = {S100248, 130, { &P15767,&P18117,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA2429759 = {S100249, 192, { &P23773,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N8010 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0000, gb*/
const Device UA2438191 = {S100250, 73, { &P15364,&P6729,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win95; de; rv:1.8.0.12) Gecko/X Firefox/1.5.0.12*/
const Device UA2438354 = {S100251, 38, { &P11684,&P5129,&P4742,&P18092 }};	/*Useragent - nokia5800d-1/UC Browser7.5.1.77/69/352*/
const Device UA2444546 = {S100252, 96, { &P18221,&P5843,&P17338,&P18092 }};	/*Useragent - Nokia302/5.0 (14.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia302/UC Browser7.9.0.102/69/352*/
const Device UA2450601 = {S100253, 152, { &P18739,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOR801 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2454997 = {S100254, 120, { &P20069,&P22784,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; micromaxx78) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2463206 = {S100255, 142, { &P20723,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; de-de; AN7G2DTE Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2473145 = {S100256, 89, { &P17595,&P17296,&P22537,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:2.0) Gecko/X Firefox/4.0 Kylo/1.0.1.76141*/
const Device UA2473213 = {S100257, 76, { &P22907,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.2; zh-cn; Lenovo_A390e; 320*480) UCWEB7.9.3.103/139/32590*/
const Device UA2479104 = {S100258, 146, { &P16292,&P18111,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Galaxy Nexus Build/JRO03O) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75 Mobile Safari/535.7*/
const Device UA2495852 = {S100259, 150, { &P12657,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC EVO 3D X515m Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA2496695 = {S100260, 138, { &P12989,&P5843,&P24670,&P18092 }};	/*Useragent - Nokia303/5.0 (13.47) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia303) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2527792 = {S100261, 61, { &P15364,&P17285,&P21168,&P18092 }};	/*Useragent - Netscape 4.8 (Windows Vista): Mozilla/4.8 (Windows NT 6.0; U)*/
const Device UA2540907 = {S100262, 27, { &P15364,&P7133,&P24163,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1)*/
const Device UA2542773 = {S100263, 43, { &P16366,&P17484,&P17347,&P18092 }};	/*Useragent - samsung-gt-s8600/UC Browser8.0.3.107/69/444*/
const Device UA2543861 = {S100264, 154, { &P21395,&P5629,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A300 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2549908 = {S100265, 172, { &P18113,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID RAZR Build/6.7.2-180_DHD-16_M4-31) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2557408 = {S100266, 171, { &P13123,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2572499 = {S100267, 108, { &P14806,&P5448,&P4737,&P18092 }};	/*Useragent - SAMSUNG-SGH-I617/1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 320x240)/UC Browser7.2.2.51*/
const Device UA2572752 = {S100268, 126, { &P15364,&P5690,&P20430,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Google Earth Pro/6.0.2.2074 Safari/532.4*/
const Device UA2573758 = {S100269, 146, { &P11808,&P5204,&P24668,&P18092 }};	/*Useragent - Nokia5610d-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia5610d-1) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2574213 = {S100270, 78, { &P15364,&P17188,&P20460,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 4.0; rv:10.0.2) Gecko/X Firefox/10.0.2 SeaMonkey/2.7.2*/
const Device UA2574727 = {S100271, 89, { &P15364,&P5690,&P4643,&P18092 }};	/*Useragent - User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/3.1; IEMobile/7.0)*/
const Device UA2591720 = {S100272, 119, { &P16808,&P22784,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; micromaxq7) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2596322 = {S100273, 99, { &P14864,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8500) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2599421 = {S100274, 99, { &P14404,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5260) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2607477 = {S100275, 69, { &P14640,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;OMNIA7;7.10;8862)*/
const Device UA2608245 = {S100276, 33, { &P12879,&P17979,&P4459,&P18092 }};	/*Useragent - Pandora/null Android/3.1 GT-P7510*/
const Device UA2619331 = {S100277, 166, { &P15876,&P18066,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P5100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.2.139/145/355*/
const Device UA2630257 = {S100278, 166, { &P15876,&P18066,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P5100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2632127 = {S100279, 170, { &P15876,&P18066,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-P5100 Build/IML74K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2617636 = {S100280, 55, { &P13452,&P5629,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.5023 Android/2.3.3 Device/samsung-GT-P1000*/
const Device UA2627510 = {S100281, 162, { &P13452,&P5629,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.2.2.259*/
const Device UA2628399 = {S100282, 166, { &P17337,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P7500 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2634010 = {S100283, 131, { &P18478,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P6201 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2639909 = {S100284, 63, { &P15364,&P24094,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux mips64; rv:2.0.1) Gecko/X Firefox/4.0.1*/
const Device UA2640598 = {S100285, 140, { &P17475,&P5768,&P24667,&P18092 }};	/*Useragent - Nokia6300/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia6300) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2644360 = {S100286, 157, { &P18513,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S6352 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2644406 = {S100287, 54, { &P12588,&P5055,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1 (NokiaC6-01;SymbianOS/9.1 Series60/3.0)*/
const Device UA2650949 = {S100288, 142, { &P15992,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ZTE-Mimosa X Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2657753 = {S100289, 197, { &P13157,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2657950 = {S100290, 173, { &P20262,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; Karbonn_A18 Build/KarbonnA18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2665198 = {S100291, 168, { &P18471,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; XT907 Build/9.8.1Q_27-2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2667552 = {S100292, 138, { &P20081,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-N7108 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2683492 = {S100293, 280, { &P22114,&P17017,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; jblend) UCBrowser8.3.0.154/69/444/UCWEB Mobile/HS-T39_TD/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0 Config/CLDC-1.1,/HS-T39_TD/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0 Config/CLDC-1.1*/
const Device UA2684027 = {S100294, 70, { &P18708,&P18110,&P20793,&P18092 }};	/*Useragent - JUC (Linux; U; 4.0.4; zh-cn; U30GT-H; 1280*752) UCWEB1.0.1.113/147/800*/
const Device UA2694597 = {S100295, 135, { &P23391,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; eZee'Tab802 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2694784 = {S100296, 67, { &P21273,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; Aqua_S; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2721259 = {S100297, 169, { &P12280,&P17145,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_6 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8E200 Safari/7534.48.3 TeamStream (iPhone3,3 232 1.6.2)*/
const Device UA2722018 = {S100298, 148, { &P12502,&P17485,&P24667,&P18092 }};	/*Useragent - SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; sonyericssonw20i) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2723259 = {S100299, 131, { &P11754,&P17485,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; UCBrowser8.4.0.159/69/352/UCWEB Mobile  SonyEricssonP990i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2723616 = {S100300, 35, { &P18323,&P17484,&P21759,&P18092 }};	/*Useragent - iBrowser/Mini2.8 (SAMSUNG-GT-S5222)*/
const Device UA2726913 = {S100301, 160, { &P19937,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; A73 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2727756 = {S100302, 185, { &P13157,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2730728 = {S100303, 159, { &P12968,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; LT26i Build/6.1.A.2.45) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2735153 = {S100304, 137, { &P18474,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9210T Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2745782 = {S100305, 35, { &P18506,&P5043,&P23405,&P18092 }};	/*Useragent - MQQBrowser/Mini2.8 (Nokia305/03.60)*/
const Device UA2749977 = {S100306, 138, { &P12697,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; shooteru Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2751931 = {S100307, 137, { &P19870,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SCH-L710 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2758317 = {S100308, 52, { &P13163,&P5890,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8 (NokiaN97;SymbianOS/9.1 Series60/3.0)*/
const Device UA2760524 = {S100309, 156, { &P18223,&P18066,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; LT22i Build/6.1.B.0.544) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2774685 = {S100310, 41, { &P12142,&P17485,&P4742,&P18092 }};	/*Useragent - sonyericssonu5i/UC Browser7.5.1.77/69/352*/
const Device UA2774804 = {S100311, 132, { &P13382,&P5448,&P4777,&P18092 }};	/*Useragent - SAMSUNG-SGH-i710/MSIE4.01/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.7.1.88*/
const Device UA2777930 = {S100312, 155, { &P21228,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Lenovo A750 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2794273 = {S100313, 136, { &P18676,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SBM106SH Build/S0020) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2795314 = {S100314, 140, { &P15364,&P22710,&P18589,&P18092 }};	/*Useragent - [FBAN/FB4A;FBAV/2.3;FBBV/X;FBDM/{density=1.5,width=480,height=800};FBLC/en_US;FBCR/Sprint;FBPN/com.facebook.katana;FBDV/PC36100;FBSV/2.3.5;]*/
const Device UA2801321 = {S100315, 42, { &P12080,&P17484,&P4737,&P18092 }};	/*Useragent - samsung-gt-s3653/UC Browser7.2.2.51/69/352*/
const Device UA2801530 = {S100316, 35, { &P15373,&P5129,&P4722,&P18092 }};	/*Useragent - Nokia5530/UC Browser7.0.0.41/50/351*/
const Device UA2801687 = {S100317, 84, { &P15364,&P5690,&P23689,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.6) Gecko/X Palemoon/3.5.6.1*/
const Device UA2803830 = {S100318, 54, { &P17872,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3J UP.Browser/6.2_7.2.7.1.K.2.234 (GUI) MMP/2.0*/
const Device UA2805192 = {S100319, 106, { &P19970,&P17485,&P17338,&P18092 }};	/*Useragent - SonyEricssonG502/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssong502/UC Browser7.9.0.102/69/352*/
const Device UA2810272 = {S100320, 154, { &P13633,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; MotoA953 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2818181 = {S100321, 112, { &P17595,&P17296,&P23675,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) iCab/4.7 Safari/525.27.1*/
const Device UA2824958 = {S100322, 185, { &P13634,&P17483,&P24671,&P18092 }};	/*Useragent - LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-us; lg-gs500) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2826379 = {S100323, 150, { &P20134,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; GT-S5830i Build/ICE CREAM SANDWICH) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2827286 = {S100324, 43, { &P15364,&P18580,&P22246,&P18092 }};	/*Useragent - Flipboard/1.7.2 CFNetwork/609 Darwin/13.0.0*/
const Device UA2828446 = {S100325, 33, { &P18643,&P5043,&P21759,&P18092 }};	/*Useragent - iBrowser/Mini2.8 (Nokia311/03.81)*/
const Device UA2828643 = {S100326, 120, { &P16591,&P22784,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; micromaxq50) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2829949 = {S100327, 54, { &P14033,&P5890,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2 (NokiaC5-03;SymbianOS/9.1 Series60/3.0)*/
const Device UA2831106 = {S100328, 118, { &P13602,&P17484,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; gt-s5233w) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/84/352/UCWEB Mobile*/
const Device UA2832924 = {S100329, 184, { &P12502,&P17485,&P24671,&P18092 }};	/*Useragent - SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; fa; sonyericssonw20i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2835027 = {S100330, 168, { &P15079,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MK16i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2835563 = {S100331, 33, { &P14711,&P5204,&P21724,&P18092 }};	/*Useragent - OneBrowser/3.0 (NokiaC2-01/11.10)*/
const Device UA2837585 = {S100332, 164, { &P15184,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; LG-P920 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2840076 = {S100333, 130, { &P14973,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-V909 Build/HMJ37) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2841776 = {S100334, 132, { &P17595,&P17297,&P19987,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/534.57.7 (KHTML, like Gecko) Version/5.1 Safari/7534.51.22 Sleipnir/3.8.3*/
const Device UA2842946 = {S100335, 149, { &P23926,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GN100 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2843419 = {S100336, 70, { &P23849,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.2; zh-cn; MTS-SP101; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA2843477 = {S100337, 152, { &P18742,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOR807 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2843982 = {S100338, 153, { &P18195,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; YP-G70 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2844194 = {S100339, 140, { &P15759,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; HTC Vision Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2844998 = {S100340, 80, { &P17595,&P17297,&P18429,&P18090 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:17.0) Gecko/X Firefox/17.0 slurp*/
const Device UA2845629 = {S100341, 135, { &P21843,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; EasyPad 971 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2845992 = {S100342, 119, { &P14294,&P5129,&P24673,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia5238) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.7.1.234/50/444/UCWEB Mobile*/
const Device UA2847974 = {S100343, 162, { &P19968,&P17485,&P24675,&P18092 }};	/*Useragent - SonyEricssonW910i/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonw910i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2848554 = {S100344, 170, { &P19779,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A1+ Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2849267 = {S100345, 165, { &P20209,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; MID7036 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2852061 = {S100346, 168, { &P18564,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; HTC One X+ Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2852097 = {S100347, 164, { &P13644,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; PC36100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2852238 = {S100348, 107, { &P15364,&P5690,&P18629,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.57 Safari/537.17 slurp*/
const Device UA2853296 = {S100349, 172, { &P21269,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; Intex Aqua 3.2 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2855290 = {S100350, 127, { &P20888,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; D709 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2855689 = {S100351, 95, { &P12082,&P5477,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.1-update1; en-US; X10a) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2857345 = {S100352, 190, { &P12814,&P5629,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I777 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (SAMSUNG-SGH-I777, 40, 1.8.5)*/
const Device UA2857928 = {S100353, 161, { &P17324,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; GT-I9001 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2858243 = {S100354, 177, { &P20931,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; iBall Slide 3G 7334 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2858493 = {S100355, 171, { &P19938,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SCH-I589 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2858791 = {S100356, 158, { &P15483,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC Chacha A810 Build/X-; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2858797 = {S100357, 135, { &P21909,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SO-03E Build/10.1.E.0.244) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2859941 = {S100358, 138, { &P21640,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-I8245 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2860774 = {S100359, 131, { &P20425,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N-03E Build/AX) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2860845 = {S100360, 135, { &P20343,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZP900H Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2861063 = {S100361, 137, { &P23550,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE T81 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2861327 = {S100362, 134, { &P21006,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; L-05D Build/IMM76I) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2861670 = {S100363, 143, { &P23473,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; RMD-1035 Build/ICS.g12refM1005RU.X) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2862137 = {S100364, 192, { &P18702,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-N8013 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, -0400, us*/
const Device UA2862751 = {S100365, 96, { &P21971,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua_Trendy) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2862819 = {S100366, 137, { &P20290,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A11 Build/KarbonnA11) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2863137 = {S100367, 137, { &P23314,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Jiayu G3 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2863142 = {S100368, 160, { &P23535,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; Lenovo S899t Build/S-2-20) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2863215 = {S100369, 142, { &P21341,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Spice Mi-500 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2863693 = {S100370, 152, { &P20251,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328d Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2863873 = {S100371, 148, { &P21616,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; i-mobile i-style Q3 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2864047 = {S100372, 139, { &P23559,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AMOI N821 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2865514 = {S100373, 138, { &P22866,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AT-AS43D3 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2866278 = {S100374, 138, { &P18021,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A770K Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2866372 = {S100375, 103, { &P15364,&P17979,&P24531,&P18092 }};	/*Useragent - Instagram 3.5.3 Android (10/2.3.5; 240dpi; 480x800; HTC/sprint; PC36100; supersonic; supersonic; en_US)*/
const Device UA2866410 = {S100376, 156, { &P14693,&P5843,&P24677,&P18092 }};	/*Useragent - NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiac1-02) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2866849 = {S100377, 133, { &P23217,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PMP7170B3G Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2867861 = {S100378, 136, { &P18676,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SBM106SH Build/S0020) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2868608 = {S100379, 70, { &P17346,&P19826,&P24531,&P18092 }};	/*Useragent - Instagram 3.5.1 (iPod4,1; iPhone OS 6_1_3; en_US; en) AppleWebKit/420+*/
const Device UA2878200 = {S100380, 92, { &P12845,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-SGH-T528g/T528UDKJ2 Dolfin/1.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA2892749 = {S100381, 133, { &P13633,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A953 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2896967 = {S100382, 163, { &P22582,&P18598,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; SAMSUNG-SGH-I337 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2933032 = {S100383, 31, { &P15364,&P18145,&P17213,&P18092 }};	/*Useragent - Opera/8.51 (FreeBSD 5.1; U; en)*/
const Device UA2937575 = {S100384, 136, { &P21355,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; KM-E100 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2937652 = {S100385, 157, { &P18697,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-I415 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2937841 = {S100386, 131, { &P21841,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; ASTROPLUS Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2937941 = {S100387, 140, { &P15794,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Sensation XL Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2938113 = {S100388, 149, { &P20750,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AN10DG3 Build/IML74K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2950284 = {S100389, 133, { &P23528,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; he-il; MAGPAD9.1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2950700 = {S100390, 141, { &P23920,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Benten A668 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2951226 = {S100391, 157, { &P22875,&P5629,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2952167 = {S100392, 163, { &P20134,&P5629,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.2.2.259*/
const Device UA2952317 = {S100393, 134, { &P21752,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; M030 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2952328 = {S100394, 113, { &P13127,&P6712,&P19812,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia3030/14.60; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/2.3.0.0.48*/
const Device UA2957158 = {S100395, 158, { &P23974,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Telenor_One_Touch_S Build/IceCreamSandwich) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2957576 = {S100396, 134, { &P21493,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IQ444 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2957906 = {S100397, 157, { &P19870,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SCH-L710 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2958345 = {S100398, 137, { &P20235,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Cynus T1 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2958578 = {S100399, 132, { &P20140,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GEM10312BK Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2977466 = {S100400, 82, { &P15364,&P5690,&P20797,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.10) Gecko/X Postbox/1.0.1*/
const Device UA2980715 = {S100401, 140, { &P23576,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; VOTO_W7200A Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2981293 = {S100402, 251, { &P12059,&P5204,&P18520,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.5.0.185/83/352*/
const Device UA2982171 = {S100403, 138, { &P24488,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; it-it; Discovery Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2982260 = {S100404, 182, { &P15786,&P5043,&P24672,&P18092 }};	/*Useragent - NokiaC2-05/2.0 (08.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokiac2-05) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/70/444/UCWEB Mobile*/
const Device UA2982515 = {S100405, 33, { &P13035,&P5043,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (NokiaC2-02/06.96)*/
const Device UA2982658 = {S100406, 36, { &P14358,&P5890,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (NokiaE63-1/0.21.010)*/
const Device UA2983070 = {S100407, 90, { &P15364,&P16110,&P22530,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.9.2.7) Gecko/X Epic/1.1 Firefox/3.6.7*/
const Device UA2983614 = {S100408, 100, { &P21383,&P18110,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Adr (Linux; U; 2.3.5; zh-cn; Lenovo A288t Build/MocorDroid2.3.5.A288t_S161_X;320*480)*/
const Device UA2983626 = {S100409, 157, { &P15649,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2983794 = {S100410, 180, { &P11699,&P5768,&P24672,&P18092 }};	/*Useragent - Nokia5200/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia5200) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile*/
const Device UA2984959 = {S100411, 173, { &P14842,&P5477,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E10i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2985548 = {S100412, 166, { &P21356,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; TECNO N3 Build/master) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2986268 = {S100413, 171, { &P12496,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; SAMSUNG-SGH-I897 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2986326 = {S100414, 162, { &P18304,&P5629,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.2.2.259*/
const Device UA2986422 = {S100415, 147, { &P18737,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; X909 Build/JRO03L) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2986565 = {S100416, 90, { &P15517,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; MT15i) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2988477 = {S100417, 139, { &P15850,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-lu6200 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2989033 = {S100418, 140, { &P22624,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Burst S280 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2989154 = {S100419, 99, { &P15492,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-c3322) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2990385 = {S100420, 106, { &P19857,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; ALCATEL_ONE_TOUCH_903) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA2990701 = {S100421, 158, { &P23974,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Telenor_One_Touch_S Build/IceCreamSandwich) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2992993 = {S100422, 139, { &P21973,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Aqua flash Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2993313 = {S100423, 161, { &P21046,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A89 Build/CelkonCelkon_A89) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2993540 = {S100424, 161, { &P21020,&P18111,&P22281,&P18092 }};	/*Useragent - OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 4.1.2; en-in; Micromax A116 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2995564 = {S100425, 160, { &P19863,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI U9508 Build/HuaweiU9508) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2996028 = {S100426, 170, { &P20005,&P5629,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; Micromax A65 Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.3.2.289*/
const Device UA2996052 = {S100427, 147, { &P20255,&P18066,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R811 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2996112 = {S100428, 100, { &P13206,&P17485,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk530i) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2996201 = {S100429, 93, { &P15364,&P5890,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; NOKIA5500) U2/1.0.0 UCBrowser/8.9.0.277 U2/1.0.0 Mobile*/
const Device UA2996479 = {S100430, 90, { &P19763,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; C6602) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA2997463 = {S100431, 137, { &P18002,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A760S Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2997469 = {S100432, 140, { &P22676,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Celkon A119 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2997473 = {S100433, 136, { &P20396,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE R22 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2998013 = {S100434, 145, { &P21640,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Samsung GT-S9081 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2998338 = {S100435, 88, { &P21057,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; A7+) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2998557 = {S100436, 150, { &P18083,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; V9C Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2998796 = {S100437, 149, { &P18061,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; rk30sdk Build/JRO03H) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2998870 = {S100438, 163, { &P14225,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2999404 = {S100439, 154, { &P14207,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; ADR6350 Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2999798 = {S100440, 89, { &P21472,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; A111) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA2999812 = {S100441, 169, { &P14183,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; SAMSUNG-SGH-I997 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2999981 = {S100442, 130, { &P23222,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; PMP3470B Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2999964 = {S100443, 154, { &P12896,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; LG-P999 Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3000835 = {S100444, 131, { &P20158,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; A7 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3001049 = {S100445, 155, { &P23163,&P18598,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; SCH-I959 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.294 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3002715 = {S100446, 151, { &P22605,&P18111,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I829 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3004486 = {S100447, 171, { &P20002,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ALCATEL ONE TOUCH 992D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3011786 = {S100448, 156, { &P19856,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-LS970 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3013579 = {S100449, 165, { &P18626,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SAMSUNG-SGH-I547 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3017652 = {S100450, 90, { &P13018,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; MT11i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3018232 = {S100451, 157, { &P18469,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IdeaTabS2109A-F Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3018852 = {S100452, 156, { &P15575,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9100 Build/JDQ39E) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3022004 = {S100453, 132, { &P20140,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GEM10312BK Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3022553 = {S100454, 136, { &P18677,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; SBM104SH Build/S0034) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3030109 = {S100455, 132, { &P18280,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; One S Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3031362 = {S100456, 99, { &P15198,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3650) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3034721 = {S100457, 134, { &P20226,&P21637,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; LG-E975 Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3041581 = {S100458, 43, { &P16384,&P17484,&P4779,&P18092 }};	/*Useragent - samsung-gt-s5380k/UC Browser7.8.0.95/69/351*/
const Device UA3042916 = {S100459, 68, { &P12698,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; LG-C660; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA3043510 = {S100460, 169, { &P13889,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-P505 Build/JZO54K; CyanogenMod-10 (Build 6/RC 2)) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3043668 = {S100461, 120, { &P14367,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA6710s) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile*/
const Device UA3044262 = {S100462, 135, { &P21493,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IQ444 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3044566 = {S100463, 152, { &P20292,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC T528t Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA3044717 = {S100464, 158, { &P12336,&P17483,&P24676,&P18092 }};	/*Useragent - LG-CU720/V1.0l Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-cu720) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA3044836 = {S100465, 179, { &P18652,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A87 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA3045054 = {S100466, 170, { &P15880,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MB865 Build/6.7.2_EDEM-18) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3045548 = {S100467, 152, { &P18714,&P5629,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V889D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA3045928 = {S100468, 156, { &P21166,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-CN; ZTE U880E Build/GWK74) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3046352 = {S100469, 157, { &P18714,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; ZTE V889D Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3046403 = {S100470, 174, { &P17149,&P5629,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SCH-I510 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (SCH-I510, 42, 1.9.1)*/
const Device UA3046643 = {S100471, 154, { &P20129,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; MI 2 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA3047586 = {S100472, 135, { &P20740,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AVA757 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3048229 = {S100473, 158, { &P15517,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; MT15i Build/4.1.B.0.587) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3048251 = {S100474, 151, { &P13505,&P24694,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-CN; MB525 Build/MIUI) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3048305 = {S100475, 43, { &P15364,&P18570,&P24428,&P18092 }};	/*Useragent - Dolphin/6.3 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA3048398 = {S100476, 156, { &P20067,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; A56 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3048472 = {S100477, 92, { &P21023,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; LG-E405) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA3048533 = {S100478, 180, { &P14862,&P5204,&P22261,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.0.0.260*/
const Device UA3048566 = {S100479, 163, { &P20262,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; Karbonn_A18 Build/KarbonnA18) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3048999 = {S100480, 94, { &P24555,&P5096,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; MTS-SP100) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3049417 = {S100481, 151, { &P15767,&P18432,&P22783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) 1Password/4.2.1 (like Version/10A523 Mobile/6.0.1 Safari/8536.25)*/
const Device UA3049590 = {S100482, 160, { &P15397,&P5629,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.4; ST18i Build/4.0.2.A.0.62) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.57768*/
const Device UA3049781 = {S100483, 135, { &P18647,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; T-02D Build/V19R50B) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3050095 = {S100484, 148, { &P23304,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R813T Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3050170 = {S100485, 169, { &P19792,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; HTC Desire U dual sim Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3050289 = {S100486, 96, { &P20004,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Karbonn A2+) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA3054418 = {S100487, 169, { &P21475,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax A35 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056377 = {S100488, 152, { &P18737,&P18598,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; X909T Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056714 = {S100489, 157, { &P19845,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ZTE N9120 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3056913 = {S100490, 93, { &P18507,&P5629,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I8160) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3057809 = {S100491, 131, { &P16168,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; NookColor Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3058292 = {S100492, 138, { &P20926,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; AT-AS40SE Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3058357 = {S100493, 162, { &P18385,&P21637,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; Nexus 7 Build/JWR66V) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (Nexus 7, 52, 1.9.9.2)*/
const Device UA3058840 = {S100494, 154, { &P13746,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; LGL35G Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.0.308 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3058929 = {S100495, 196, { &P20705,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTC One Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us*/
const Device UA3058983 = {S100496, 204, { &P22972,&P18598,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SAMSUNG-SGH-I537 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0500, us*/
const Device UA3059113 = {S100497, 190, { &P22228,&P18111,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SM-T210R Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, us*/
const Device UA3070118 = {S100498, 138, { &P18235,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; S2005A-H Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3071341 = {S100499, 161, { &P15364,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Z7H Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3071758 = {S100500, 97, { &P21036,&P5629,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Spice_Mi-425) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3073156 = {S100501, 157, { &P20292,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HTC T528t Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.288 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3073331 = {S100502, 198, { &P21648,&P18598,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9505 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.27/385,1.9.27.385,2013-03-31 15:31, +0000, gb*/
const Device UA3073454 = {S100503, 158, { &P16356,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA3074152 = {S100504, 154, { &P23946,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; I7100 Build/JRO03C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA3074523 = {S100505, 89, { &P21514,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; N350) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA3078505 = {S100506, 100, { &P23010,&P17485,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk790i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3079424 = {S100507, 131, { &P20004,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A2 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.59 Mobile Safari/537.36*/
const Device UA3083331 = {S100508, 135, { &P19861,&P21637,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; SCH-I605 Build/JSS15Q) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.30 Mobile Safari/537.36*/
const Device UA3084983 = {S100509, 61, { &P20032,&P22784,&P18246,&P18092 }};	/*Useragent - MicromaxX291/Q03C MAUI-browser/MIDP-2.0Configuration/CLDC-1.1*/
const Device UA3085290 = {S100510, 95, { &P18506,&P5043,&P4777,&P18092 }};	/*Useragent - Nokia305/2.0 (03.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia305/UC Browser7.7.1.88/69/444*/
const Device UA3085492 = {S100511, 138, { &P21440,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MOMO9 plus III Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA3085558 = {S100512, 150, { &P14967,&P5096,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.1; en-in; LG-P500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3085952 = {S100513, 57, { &P14533,&P7116,&P21731,&P18092 }};	/*Useragent - OneBrowser/for Blackberry3.0.0 (BlackBerry9630/5.0.0.975)*/
const Device UA3086400 = {S100514, 154, { &P14969,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Desire S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA3086499 = {S100515, 148, { &P23304,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; R813T Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA3086550 = {S100516, 172, { &P17625,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-US; ST25i Build/6.0.B.3.184) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA3086523 = {S100517, 70, { &P15767,&P17146,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 (iPad2,1; iPhone OS 5.0.1; en_US; en) AppleWebKit/420+*/
const Device UA3087173 = {S100518, 174, { &P14872,&P5629,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-D700 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (SPH-D700, 42, 1.9.1)*/
const Device UA3087191 = {S100519, 174, { &P18300,&P5629,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T769 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (SGH-T769, 42, 1.9.1)*/
const Device UA3087616 = {S100520, 93, { &P14433,&P5096,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; zh-CN; GT-I9003) U2/1.0.0 UCBrowser/8.8.1.243 U2/1.0.0 Mobile*/
const Device UA3087802 = {S100521, 144, { &P23838,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; StarTrail III Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3087806 = {S100522, 158, { &P18225,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; U9200 Build/HuaweiU9200) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3087962 = {S100523, 135, { &P20462,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PTL21 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3088017 = {S100524, 159, { &P20452,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; Lenovo S880 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3088322 = {S100525, 156, { &P18303,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; MT27i Build/6.1.1.B.1.54) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA3088330 = {S100526, 155, { &P18449,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; ST27i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA3089319 = {S100527, 135, { &P11950,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.3; en-in; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3091784 = {S100528, 160, { &P24688,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Spice Mi-515 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3092186 = {S100529, 181, { &P23174,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GenxDroid7 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3092240 = {S100530, 170, { &P21737,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; IRIS355 Build/IRIS355) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3092349 = {S100531, 127, { &P15198,&P18111,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Genio Build/JRO03H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3092381 = {S100532, 90, { &P13018,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; MT11i) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3092546 = {S100533, 198, { &P19849,&P18066,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-S7560M Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.7/607,2.0.7.607,2013-08-01 08:25, -0400, ca*/
const Device UA3092577 = {S100534, 164, { &P19762,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; HUAWEI MT1-U06 Build/HuaweiMT1-U06) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3092942 = {S100535, 167, { &P20094,&P18066,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; HUAWEI U8825D Build/HuaweiU8825D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3098809 = {S100536, 52, { &P15364,&P17285,&P18215,&P18092 }};	/*Useragent - AppleCoreMedia/1.0.0. (AAS-2.2.2; U; Windows NT 6.0)*/
const Device UA3098888 = {S100537, 174, { &P18673,&P18066,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-S7562 Build/IMM76I; 480*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444*/
const Device UA3099045 = {S100538, 145, { &P24651,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; BE 150 3G Build/ICS.f08refem79v2.X) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3099189 = {S100539, 161, { &P20454,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Coolpad 7019 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3099246 = {S100540, 157, { &P18297,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8530 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3099735 = {S100541, 100, { &P12707,&P5055,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia701) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3099831 = {S100542, 131, { &P20742,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; es-es; TOUCH87 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3099982 = {S100543, 135, { &P20452,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; S8800 Build/JRO03C) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3100002 = {S100544, 169, { &P21181,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Lenovo S560 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA3100033 = {S100545, 155, { &P24504,&P18598,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; zh-cn; Lenovo A830 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA3100215 = {S100546, 162, { &P16356,&P18066,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-N7000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA3100266 = {S100547, 48, { &P15364,&P18580,&P18245,&P18092 }};	/*Useragent - StumbleUpon/2.52 CFNetwork/609.1.4 Darwin/13.0.0*/
const Device UA3100578 = {S100548, 144, { &P18002,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; vi-vn; IM-A760S/770K Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3100659 = {S100549, 131, { &P23897,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; STM805ND Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA3100729 = {S100550, 153, { &P21903,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; U705T Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3100737 = {S100551, 152, { &P18742,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; R807 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3100800 = {S100552, 152, { &P18738,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; R817 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3100934 = {S100553, 156, { &P22611,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-S7572 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3100985 = {S100554, 186, { &P13157,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-CN; HTC Sensation XE with Beats Audio Z715e Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3101874 = {S100555, 163, { &P15932,&P24699,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3110 Build/JDQ39; Carbon-CARBON-JB-EXP-08APR2013) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3101915 = {S100556, 152, { &P21596,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; mobiistar touch LAI 504 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA3101958 = {S100557, 95, { &P22648,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ONIDA_i099) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3102092 = {S100558, 159, { &P21155,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo A800 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3102098 = {S100559, 90, { &P15397,&P5629,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.4; en-US; ST18i) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA3102193 = {S100560, 143, { &P21882,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Micromax P362 Build/JRO03H) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3102266 = {S100561, 144, { &P15602,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; th-th; Airphone_A501 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102381 = {S100562, 137, { &P23924,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; HWA580 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3102581 = {S100563, 159, { &P18321,&P5629,&P22281,&P18092 }};	/*Useragent - OneBrowser/3.8.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3102714 = {S100564, 90, { &P20175,&P18066,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; zh-CN; MI 1S) U2/1.0.0 UCBrowser/9.0.1.275 U2/1.0.0 Mobile*/
const Device UA3103046 = {S100565, 134, { &P17717,&P18066,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-SU760 Build/ICS) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3103103 = {S100566, 136, { &P20494,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Titan Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3103165 = {S100567, 136, { &P23697,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9192 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3103190 = {S100568, 93, { &P13974,&P5096,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5670) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA3103253 = {S100569, 136, { &P21395,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Lenovo A3000-H Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA3103429 = {S100570, 139, { &P23201,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Karbonn S2 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3105270 = {S100571, 133, { &P18641,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-us; SPH-M830 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3105771 = {S100572, 151, { &P13505,&P24694,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-CN; MB525 Build/MIUI) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3105900 = {S100573, 133, { &P20141,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-gb; GT-S5301 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3105908 = {S100574, 152, { &P21752,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; M030 Build/JRO03H) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3106179 = {S100575, 135, { &P18223,&P18111,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.1.2; en-in; LT22i Build/6.2.A.1.100) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3106295 = {S100576, 159, { &P22913,&P18066,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Karbonn A25 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3106526 = {S100577, 155, { &P13974,&P5096,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5670 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3106579 = {S100578, 160, { &P18552,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; XT897 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1/xt897c*/
const Device UA3106831 = {S100579, 156, { &P22610,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S6810 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3106864 = {S100580, 165, { &P22626,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; Iris 349 Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3107137 = {S100581, 90, { &P18449,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.7; en-US; ST27i) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3107208 = {S100582, 159, { &P12505,&P17483,&P24678,&P18092 }};	/*Useragent - LG-GX500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gx500) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3107334 = {S100583, 162, { &P20226,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; LG-E975 Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 CyanogenMod/10.1.2/e975*/
const Device UA3107507 = {S100584, 186, { &P18300,&P22212,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; SAMSUNG-SGH-T769 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA3107642 = {S100585, 164, { &P16336,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; U8800Pro Build/HuaweiU8800Pro) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3107647 = {S100586, 100, { &P18502,&P18148,&P24639,&P18092 }};	/*Useragent - UCWEB/2.0 (Windows; U; wds 8.0; en-US; HTC;  8X by HTC) U2/1.0.0 UCBrowser/3.0.1.302 U2/1.0.0 Mobile*/
const Device UA3107779 = {S100587, 170, { &P22396,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-S5312 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3107812 = {S100588, 138, { &P20244,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; bq Elcano Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3107834 = {S100589, 138, { &P20180,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XOLO_A800 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3108008 = {S100590, 159, { &P22640,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IQ446 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.20.1364.172 Mobile Safari/537.22*/
const Device UA3108055 = {S100591, 159, { &P12505,&P17483,&P24679,&P18092 }};	/*Useragent - LG-GX500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gx500) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3108064 = {S100592, 105, { &P11684,&P5129,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5800d-1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile  3gpp-gba*/
const Device UA3108220 = {S100593, 129, { &P22476,&P18066,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ZTE V70 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3108221 = {S100594, 158, { &P20146,&P18598,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9082 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.3.2.300*/
const Device UA3108246 = {S100595, 151, { &P22604,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; SCH-I879 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3108532 = {S100596, 158, { &P18564,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; HTC One X+ Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3108915 = {S100597, 162, { &P22382,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.20.1364.172 Mobile Safari/537.22*/
const Device UA3111270 = {S100598, 138, { &P22866,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; AT-AS43D3 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3117086 = {S100599, 80, { &P16860,&P5448,&P4779,&P18092 }};	/*Useragent - SAMSUNG-SCH-M715/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA3117439 = {S100600, 76, { &P17595,&P21721,&P18171,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:13.0) Gecko/X Firefox/13.0.1*/
const Device UA3117533 = {S100601, 151, { &P21732,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5.1.12-RT-X.X; zh-cn; K-Touch W650 Build/AliyunOs-2012) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA3117568 = {S100602, 176, { &P17968,&P5629,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; Android Build/GRK39F; 320*480) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/33081*/
const Device UA3118019 = {S100603, 68, { &P21205,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; Andi107; 240*320) UCWEB7.9.0.94/139/444*/
const Device UA3118041 = {S100604, 138, { &P15820,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-SU640 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3118092 = {S100605, 44, { &P15364,&P17484,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SAMSUNG-GT-S5263/S5263DDKA5)*/
const Device UA3118285 = {S100606, 42, { &P13220,&P17485,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (SonyEricssonK770i/R8BC004)*/
const Device UA3118293 = {S100607, 185, { &P15794,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3118345 = {S100608, 165, { &P20884,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A298t Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3118378 = {S100609, 85, { &P15364,&P16110,&P17381,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.2; en-US; rv:1.8.0.12) Gecko/X Firefox/1.5.0.12*/
const Device UA3118835 = {S100610, 149, { &P21657,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HUAWEI C8950D Build/HuaweiC8950D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA3119066 = {S100611, 113, { &P15364,&P7127,&P24195,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; QQBrowser/6.12)*/
const Device UA3119079 = {S100612, 102, { &P13552,&P5129,&P21730,&P18092 }};	/*Useragent - OneBrowser/3.1 (NokiaC6-00/42.0.004/sw_platform=S60;sw_platform_version=5.0;java_build_version=1.4.48)*/
const Device UA3119238 = {S100613, 166, { &P14969,&P18066,&P21771,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Desire S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 XiaoMi/MiuiBrowser/1.0*/
const Device UA3119523 = {S100614, 156, { &P22020,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-I9268 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3119656 = {S100615, 159, { &P21087,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; Lenovo A690 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3120349 = {S100616, 99, { &P21134,&P18111,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; iBall_Andi4.5q) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3120390 = {S100617, 69, { &P22609,&P18111,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.1.2; zh-cn; GT-S6312; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA3120484 = {S100618, 150, { &P19937,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; A73 Build/GRJ90) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3120683 = {S100619, 74, { &P19862,&P18111,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.1.1; zh-cn; Micromax_A110; 480*854) UCWEB7.9.0.94/139/444*/
const Device UA3120829 = {S100620, 132, { &P16168,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; NookColor Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3120972 = {S100621, 173, { &P12973,&P18066,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA3121023 = {S100622, 136, { &P16135,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.0; en-np; XT701 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121140 = {S100623, 168, { &P22646,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; Dream_D1 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/33886*/
const Device UA3121141 = {S100624, 145, { &P15983,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; LG-MS695 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon*/
const Device UA3121188 = {S100625, 96, { &P22997,&P18066,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; Celkon_A107) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3121384 = {S100626, 137, { &P13999,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; NexusHD2 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3121458 = {S100627, 98, { &P22737,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; Micromax A115) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA3121507 = {S100628, 155, { &P22642,&P18066,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; Ctrl_V3 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3121526 = {S100629, 157, { &P15975,&P5629,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; V9A Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.3.2.300*/
const Device UA3121523 = {S100630, 155, { &P24150,&P18598,&P20787,&P18092 }};	/*Useragent - MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; vivo Xplay Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3121595 = {S100631, 153, { &P21809,&P18066,&P22879,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; T8620 Build/HuaweiT8620) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.3 Mobile Safari/533.1*/
const Device UA3121639 = {S100632, 98, { &P21020,&P18598,&P24667,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.2.1; en-US; Micromax_A116) U2/1.0.0 UCBrowser/8.2.0.242 U2/1.0.0 Mobile*/
const Device UA3121728 = {S100633, 128, { &P14903,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; V901 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA3121781 = {S100634, 129, { &P20438,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; N400 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3122099 = {S100635, 129, { &P21472,&P18066,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 4.0.4; en-us; A111 Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3122216 = {S100636, 93, { &P22941,&P18111,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6812) U2/1.0.0 UCBrowser/9.0.2.299 U2/1.0.0 Mobile*/
const Device UA3122237 = {S100637, 152, { &P18739,&P5629,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; R801 Build/GRK39F) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.1.309 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3123916 = {S100638, 91, { &P13824,&P22839,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; zte-f970) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3123926 = {S100639, 144, { &P12871,&P5629,&P23491,&P18092 }};	/*Useragent - OneBrowser/4.2.0/Adr(Linux; U; Android 2.3.5; en-in; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA3124023 = {S100640, 114, { &P15837,&P6909,&P24336,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-05/08.30; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.1.0.0.62*/
const Device UA3124502 = {S100641, 170, { &P23000,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; TECNO_P3 Build/master) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.276 Mobile*/
const Device UA3124493 = {S100642, 90, { &P18223,&P18111,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; LT22i) U2/1.0.0 UCBrowser/9.1.0.297 U2/1.0.0 Mobile*/
const Device UA3124571 = {S100643, 155, { &P19917,&P18111,&P22881,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; LG-P765 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.1.0.297 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3124597 = {S100644, 90, { &P15397,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; ST18i) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3124805 = {S100645, 93, { &P18437,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-N8000) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3124870 = {S100646, 93, { &P14433,&P5629,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; GT-I9003) U2/1.0.0 UCBrowser/8.6.0.276 U2/1.0.0 Mobile*/
const Device UA3124872 = {S100647, 96, { &P14834,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V3; en-US; Nokia5730S-1) U2/1.0.0 UCBrowser/9.0.1.317 U2/1.0.0 Mobile*/
const Device UA3124886 = {S100648, 143, { &P18665,&P18111,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; N861 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA3124927 = {S100649, 156, { &P22748,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-E425 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA3124952 = {S100650, 182, { &P23116,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Aqua.Active Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.6.0.318 Mobile*/
const Device UA3125024 = {S100651, 96, { &P21991,&P18111,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.1; en-US; Aqua_Wonder) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3125029 = {S100652, 134, { &P24042,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; ARNOVA 101 G4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Safari/537.36*/
const Device UA3125042 = {S100653, 93, { &P16356,&P18066,&P24672,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; GT-N7000) U2/1.0.0 UCBrowser/8.6.0.318 U2/1.0.0 Mobile*/
const Device UA3125052 = {S100654, 177, { &P22584,&P24702,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; SGH-I337M Build/JDQ39E) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 LiquidSmooth/Liquid-JB-v2.9/jfltecan*/
const Device UA3125220 = {S100655, 115, { &P12473,&P5204,&P24337,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2730c-1/10.45; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/3.2.0.0.6*/
const Device UA3125227 = {S100656, 173, { &P20192,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3125255 = {S100657, 150, { &P15364,&P18598,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-US; E5 Build/JOP40D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.2.299 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3125284 = {S100658, 162, { &P11953,&P17485,&P24679,&P18092 }};	/*Useragent - SonyEricssonU10i/R7BA Profile/MIDP-2.1 Configuration/CLDC-1.1  UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonu10i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3125306 = {S100659, 139, { &P13644,&P21637,&P23997,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.3; en-us; EVO 4G LTE Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3125330 = {S100660, 168, { &P20192,&P18111,&P24341,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HUAWEI G510-0010 Build/HuaweiG510-0010) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.4 Mobile Safari/533.1*/
const Device UA3125346 = {S100661, 92, { &P15028,&P5096,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; LG-P350) U2/1.0.0 UCBrowser/9.2.0.308 U2/1.0.0 Mobile*/
const Device UA3125400 = {S100662, 93, { &P12972,&P17979,&P24717,&P18092 }};	/*Useragent - Instagram 4.1.2 Android (10/2.3.6; 240dpi; 480x800; samsung; SGH-T989; SGH-T989; qcom; en_US)*/
const Device UA3125494 = {S100663, 161, { &P16292,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-CN; GT-I9050 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3125561 = {S100664, 93, { &P22980,&P18066,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.0.4; en-US; TECNO_D5) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3125576 = {S100665, 160, { &P13914,&P5843,&P24679,&P18092 }};	/*Useragent - Nokia5330-1d/5.0 (06.88) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5330-1d) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3125581 = {S100666, 183, { &P20005,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; Micromax_A65 Build/MocorDroid2.3.5) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3125587 = {S100667, 100, { &P13183,&P17485,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonk810i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3125594 = {S100668, 170, { &P21628,&P18111,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; GT-I8552 Build/JZO54K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3125602 = {S100669, 177, { &P12973,&P18066,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax_A101 Build/MicromaxMicromax_A101) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3125614 = {S100670, 156, { &P24587,&P18111,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; SCH-I869 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.4.329 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3125653 = {S100671, 172, { &P21968,&P18111,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 4030D Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.61541*/
const Device UA3125691 = {S100672, 93, { &P14877,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.3; en-US; GT-I9000) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3125718 = {S100673, 184, { &P19868,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; ALCATEL_ONE_TOUCH_985N Build/GRK39F) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3125758 = {S100674, 95, { &P15364,&P5096,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.2; en-US; Karbonn_A1) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3125783 = {S100675, 154, { &P14207,&P5629,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; ADR6350 Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3125812 = {S100676, 155, { &P13974,&P5096,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-US; GT-S5670 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3125942 = {S100677, 93, { &P18679,&P18111,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-B5330) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3125973 = {S100678, 93, { &P14186,&P5096,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.2.1; en-US; GT-S5830) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3126096 = {S100679, 172, { &P15989,&P18066,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; ST21i Build/11.0.A.4.27) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.7.0.315 Mobile*/
const Device UA3126100 = {S100680, 94, { &P23118,&P5629,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.6; en-US; Aqua Star) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3126135 = {S100681, 155, { &P23697,&P18598,&P24055,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; GT-I9192 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.2.3.324 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126150 = {S100682, 93, { &P17324,&P5629,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 2.3.5; en-US; GT-I9001) U2/1.0.0 UCBrowser/8.7.0.315 U2/1.0.0 Mobile*/
const Device UA3126210 = {S100683, 134, { &P20352,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M-270 Build/IML74K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126236 = {S100684, 138, { &P18465,&P21637,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Amazon Kindle Fire Build/JSS15J) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126251 = {S100685, 93, { &P22941,&P18111,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; en-US; GT-S6812) U2/1.0.0 UCBrowser/9.2.3.324 U2/1.0.0 Mobile*/
const Device UA3126263 = {S100686, 134, { &P24156,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; 202F Build/V17R47F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126339 = {S100687, 167, { &P22968,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-CN; HUAWEI P6-U06 Build/HuaweiP6-U06) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126377 = {S100688, 140, { &P21302,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; IQ4410 Quad Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126395 = {S100689, 137, { &P13999,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NexusHD2 Build/IMM76L) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3126575 = {S100690, 125, { &P24510,&P18066,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; N10 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126615 = {S100691, 152, { &P20438,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; N400 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126618 = {S100692, 159, { &P21094,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; Lenovo A789 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126648 = {S100693, 163, { &P23716,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI A199 Build/HuaweiA199) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126710 = {S100694, 159, { &P16869,&P18598,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-US; Micromax A76 Build/JDQ39) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126719 = {S100695, 161, { &P14969,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; HTC Desire SV Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126759 = {S100696, 151, { &P20518,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A10 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3126885 = {S100697, 129, { &P12889,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7300 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36*/
const Device UA3126931 = {S100698, 161, { &P14186,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; GT-S5830 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127057 = {S100699, 136, { &P20963,&P18111,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SBM200SH Build/S1004) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127070 = {S100700, 144, { &P23242,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; SKY HD9500 Plus Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127081 = {S100701, 159, { &P15047,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; SK17i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127097 = {S100702, 150, { &P21334,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A8 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127111 = {S100703, 160, { &P19779,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-US; A1+ Build/MocorDroid2.3.5) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127141 = {S100704, 155, { &P23248,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-US; Gpad_G2 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127163 = {S100705, 136, { &P18713,&P18598,&P24192,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-P760 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36*/
const Device UA3127192 = {S100706, 157, { &P23680,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; AURUS III Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127238 = {S100707, 159, { &P15397,&P5629,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-US; ST18i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127273 = {S100708, 165, { &P15364,&P18111,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; HUAWEI C8815 Build/HuaweiC8815) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127292 = {S100709, 152, { &P21472,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; A111 Build/IMM76I) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127324 = {S100710, 151, { &P15767,&P22882,&P24468,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 7_0 like Mac OS X; en-US) AppleWebKit/534.35 (KHTML, like Gecko)  Chrome/11.0.696.65 Safari/534.35 Puffin/3.10080IT Mobile*/
const Device UA3127438 = {S100711, 156, { &P14959,&P18066,&P24215,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-I9103 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.3.0.321 U3/0.8.0 Mobile Safari/534.31*/
const Device UA3127567 = {S100712, 46, { &P15364,&P23096,&P18245,&P18091 }};	/*Useragent - Tweetbot/28500 CFNetwork/672.0.2 Darwin/14.0.0*/
const Device UA3127615 = {S100713, 133, { &P22653,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTL22 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3129607 = {S100714, 93, { &P11929,&P5129,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Symbian; U; S60 V5; en-US; Nokia5230) U2/1.0.0 UCBrowser/9.2.0.336 U2/1.0.0 Mobile*/
const Device UA3129906 = {S100715, 159, { &P13018,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MT11i Build/4.1.B.0.587) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA3130826 = {S100716, 132, { &P15767,&P22882,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0_2 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11A501 Safari/7534.48.3 Sleipnir/3.0m*/
const Device UA3131402 = {S100717, 128, { &P24400,&P18598,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SM-T315 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA3132422 = {S100718, 140, { &P17625,&P18066,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST25i Build/6.1.1.B.1.54) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Mobile Safari/537.36*/
const Device UA3132909 = {S100719, 130, { &P22873,&P18111,&P24154,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-N5120 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.82 Safari/537.36*/
const Device UA1071 = {S100720, 130, { &P11888,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E258/1.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzUzNDUyMDExMjkxNzky UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA2358 = {S100721, 21, { &P11953,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonU10i/R7BA*/
const Device UA10223 = {S100722, 68, { &P13479,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHU100/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54659 = {S100723, 53, { &P14254,&P17486,&P3402,&P18092 }};	/*Useragent - MOT-W231/0.0.45 UP.Browser/6.3.0.6.c.23 (GUI) MMP/2.0*/
const Device UA16014 = {S100724, 71, { &P11851,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_P5500 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA16691 = {S100725, 68, { &P13913,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHE870/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA17233 = {S100726, 30, { &P11908,&P17484,&P3430,&P18092 }};	/*Useragent - SAMSUNG-SGH-i908/1.0 Opera 9.5*/
const Device UA20521 = {S100727, 101, { &P13744,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-P850/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA22402 = {S100728, 23, { &P15455,&P19954,&P3816,&P18092 }};	/*Useragent - SEC-SGHP300/1.0 TSS/2.5*/
const Device UA23895 = {S100729, 61, { &P15635,&P6476,&P4977,&P18092 }};	/*Useragent - Mozilla/4.0 (MobilePhone MM-7500/US/1.0) NetFront/3.1 MMP/2.0*/
const Device UA24601 = {S100730, 13, { &P12240,&P17483,&P17660,&P18092 }};	/*Useragent - LG/U8180/v1.0*/
const Device UA26236 = {S100731, 71, { &P13578,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_P3470 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA32156 = {S100732, 128, { &P12470,&P17486,&P3581,&P18092 }};	/*Useragent - MOTO-W760r/Mozilla/4.0 (compatible;MSIE 6.0;Linux W760r)/R63712_U_71.xx.yyI Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony 1.0*/
const Device UA61350 = {S100733, 139, { &P12990,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; HTC-A315c/1.13.1401.0) AndroidWebKit/533.1 (KHTML, Like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369838 = {S100734, 225, { &P12990,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-tw; HTCA315c_TD/1.0 Android/2.2 release/1.1.2011 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022041 = {S100735, 123, { &P12990,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; HTC Bee Build/FRG83) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA49340 = {S100736, 44, { &P14209,&P17484,&P4970,&P18092 }};	/*Useragent - SCH-U430-PARROT/1.0 AU-MIC/2.0 (GUI) MMP/2.0*/
const Device UA54426 = {S100737, 37, { &P14209,&P17484,&P4970,&P18092 }};	/*Useragent - SCH-U430/1.0 AU-MIC/2.0 (GUI) MMP/2.0*/
const Device UA55989 = {S100738, 166, { &P17308,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android FlyX v0.1 Final; en-us; DROIDX Build/VZW) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 480X854 motorola DROIDX*/
const Device UA2024379 = {S100739, 142, { &P17308,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android ICS 1.12.30 B3; en-us; DROIDX Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58490 = {S100740, 66, { &P14215,&P17486,&P17671,&P18092 }};	/*Useragent - MOT-EX122/07.2E.00R  Obligo/WAP2.0 MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62229 = {S100741, 15, { &P14387,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHX520/1.0*/
const Device UA63161 = {S100742, 16, { &P14387,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHX520X/1.0*/
const Device UA62342 = {S100743, 60, { &P13558,&P6478,&P4420,&P18092 }};	/*Useragent - BlackBerry7290/4.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63435 = {S100744, 47, { &P14517,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-VX8610/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA63516 = {S100745, 89, { &P14524,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-NXT901U/MTK Release/11-20-2010 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1*/
const Device UA89700 = {S100746, 88, { &P14524,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-G901/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA64600 = {S100747, 45, { &P14683,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-508A/1.0 ObigoInternetBrowser/Q03C*/
const Device UA67934 = {S100748, 35, { &P15373,&P5129,&P4759,&P18092 }};	/*Useragent - Nokia5530/UC Browser7.6.1.82/50/352*/
const Device UA68177 = {S100749, 36, { &P14748,&P5890,&P4759,&P18092 }};	/*Useragent - NokiaE72-1/UC Browser7.6.1.82/28/351*/
const Device UA68768 = {S100750, 35, { &P14984,&P5129,&P4777,&P18092 }};	/*Useragent - Nokia5250/UC Browser7.7.1.88/50/444*/
const Device UA80229 = {S100751, 60, { &P14125,&P6208,&P4420,&P18092 }};	/*Useragent - BlackBerry7100/4.0.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA80246 = {S100752, 60, { &P12781,&P6208,&P4420,&P18092 }};	/*Useragent - BlackBerry7230/4.0.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87036 = {S100753, 128, { &P16047,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320; HTC_P6300/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87078 = {S100754, 9, { &P17118,&P22835,&P17470,&P18092 }};	/*Useragent - Spice D88*/
const Device UA2128589 = {S100755, 16, { &P17118,&P22835,&P17470,&P18092 }};	/*Useragent - Spice D88/WAP2.0*/
const Device UA87181 = {S100756, 45, { &P17533,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC3H UP.Browser/6.2.0.14.1 (GUI) MMP/2.0*/
const Device UA88276 = {S100757, 45, { &P17556,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-MA34 UP.Browser/6.2.0.13.2 (GUI) MMP/2.0*/
const Device UA126379 = {S100758, 49, { &P17886,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/V388/MIDP2.0/CLDC1.1/Screen-240X320*/
const Device UA253800 = {S100759, 73, { &P15364,&P18145,&P17812,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/3.4; FreeBSD) KHTML/3.4.3 (like Gecko)*/
const Device UA303999 = {S100760, 147, { &P11950,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; X10i Build/FreeX10-beta4_zdzihu) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA323837 = {S100761, 133, { &P11950,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; X10i Build/CMNI15) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA559249 = {S100762, 132, { &P11950,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.3; en-us; X10i Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018264 = {S100763, 147, { &P11950,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; X10i Build/FreeX10-beta3_zdzihu) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2275909 = {S100764, 135, { &P11950,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; X10i Build/CM 6.1.3) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA371226 = {S100765, 134, { &P20554,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; TAB220 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA372277 = {S100766, 120, { &P15364,&P18110,&P22444,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Iron/5.0.381.0 Chrome/5.0.381 Safari/533.4*/
const Device UA414119 = {S100767, 129, { &P18088,&P19954,&P18123,&P18092 }};	/*Useragent - SEC-SGHE900/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/8.01 (J2ME/MIDP; Opera Mini/2.0.4509/1378; nl; U; ssr)*/
const Device UA614025 = {S100768, 86, { &P18095,&P18109,&P18255,&P18092 }};	/*Useragent - Opera/9.80 (Series 60; Opera Mini/7.29482/27.1573; U; en) Presto/2.8.119 Version/11.10*/
const Device UA444225 = {S100769, 30, { &P18088,&P17017,&P4715,&P18092 }};	/*Useragent - j2me/UC Browser7.4.0.65/69/351*/
const Device UA499407 = {S100770, 101, { &P17595,&P17296,&P22511,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.1.6) Gecko/X Firefox/3.5.6 Wyzo/3.5.6.1*/
const Device UA518116 = {S100771, 38, { &P15364,&P17017,&P17602,&P18092 }};	/*Useragent - SH0037D Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA529194 = {S100772, 76, { &P20201,&P5208,&P3499,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Teleca Q7; Brew 3.1.5; U; en) 240X400 LGE-MX700/1.0*/
const Device UA564371 = {S100773, 142, { &P14540,&P5843,&P4732,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (08.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Opera/9.60 (J2ME/MIDP;Opera Mini/4.2.13337Mod.by.Dante./503; U; en)Presto/2.2.0*/
const Device UA571897 = {S100774, 104, { &P12125,&P5204,&P17338,&P18092 }};	/*Useragent - Nokia6500s-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6500s-1/UC Browser7.9.0.102/69/352*/
const Device UA573435 = {S100775, 37, { &P14010,&P17484,&P24665,&P18092 }};	/*Useragent - samsung-gt-s5263/UCWEB8.0.3.99/70/350*/
const Device UA573801 = {S100776, 43, { &P15547,&P17484,&P17338,&P18092 }};	/*Useragent - samsung-gt-c6712/UC Browser7.9.0.102/69/444*/
const Device UA615622 = {S100777, 132, { &P15364,&P16110,&P22428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19 Comodo_Dragon/18.3.0.0*/
const Device UA2165289 = {S100778, 125, { &P15364,&P16110,&P22428,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19 Comodo_Dragon/18.3.0.0*/
const Device UA701392 = {S100779, 136, { &P22240,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; disgo6000 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA715295 = {S100780, 85, { &P18141,&P22218,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 1.51) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2*/
const Device UA792112 = {S100781, 71, { &P17509,&P5567,&P3430,&P18092 }};	/*Useragent - Mozilla/5.0 Opera/9.5 (KDDI-CA3F; BREW; Opera Mobi; U; ja) Presto/2.2.1*/
const Device UA898165 = {S100782, 53, { &P14188,&P5890,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0 (Nokia5233;SymbianOS/9.1 Series60/3.0)*/
const Device UA1868593 = {S100783, 86, { &P17580,&P5567,&P4536,&P18092 }};	/*Useragent - Mozilla/5.0 Opera/9.5 (KDDI-CA3H; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00*/
const Device UA1872185 = {S100784, 142, { &P15364,&P6757,&P21556,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.15; fr-fr; A80HF Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA1890005 = {S100785, 43, { &P17524,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 911SH)*/
const Device UA1905340 = {S100786, 37, { &P13677,&P17484,&P24665,&P18092 }};	/*Useragent - samsung-gt-s5253/UCWEB8.0.3.99/70/350*/
const Device UA1941701 = {S100787, 108, { &P11627,&P17485,&P17347,&P18092 }};	/*Useragent - SonyEricssonK850i/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonk850i/UC Browser8.0.3.107/69/444*/
const Device UA1961577 = {S100788, 87, { &P20369,&P18110,&P17470,&P18092 }};	/*Useragent - QQBrowser/16 (Linux; U; 2.2.2; zh-cn; ZTE ZTE-C N880S Build/N880SV1.0.0B03) Mobile/0070*/
const Device UA1976965 = {S100789, 134, { &P21954,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; 760TPC Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1986682 = {S100790, 130, { &P15364,&P16110,&P22424,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11 Comodo_Dragon/17.5.2.0*/
const Device UA2331180 = {S100791, 123, { &P15364,&P16110,&P22424,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11 Comodo_Dragon/17.5.2.0*/
const Device UA2010490 = {S100792, 168, { &P14183,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I997 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2018256 = {S100793, 131, { &P18047,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; Z71 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023261 = {S100794, 155, { &P21938,&P5096,&P20772,&P18092 }};	/*Useragent - MQQBrowser/21 Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; Coolpad 5820 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023918 = {S100795, 143, { &P21307,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; SH8118U Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2023983 = {S100796, 129, { &P21307,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; SH8118U Build/ERE27) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2026213 = {S100797, 154, { &P18283,&P18066,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-I9300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027014 = {S100798, 169, { &P15397,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; ST18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2029029 = {S100799, 147, { &P13452,&P5096,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2632517 = {S100800, 148, { &P13452,&P5096,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2; en-au; GT-P1000T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2260068 = {S100801, 104, { &P15364,&P18145,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24*/
const Device UA2158214 = {S100802, 146, { &P12513,&P17485,&P24667,&P18092 }};	/*Useragent - SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-in; sonyericssonw20) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2161211 = {S100803, 66, { &P15364,&P17188,&P4809,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 4.0) Opera 5.11 [de]*/
const Device UA2167291 = {S100804, 153, { &P13018,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; MT11i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2222910 = {S100805, 159, { &P14202,&P5477,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E15i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2230807 = {S100806, 168, { &P16362,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-D710 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2260001 = {S100807, 134, { &P15364,&P17838,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.24 (KHTML, like Gecko) Ubuntu/9.10 Chromium/11.0.696.57 Chrome/11.0.696.57 Safari/534.24*/
const Device UA2265955 = {S100808, 88, { &P17595,&P17294,&P20917,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.4; en-US; rv:1.9.1b3) Gecko/X Firefox/3.1b3*/
const Device UA2285705 = {S100809, 99, { &P15364,&P7133,&P22531,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.13) Gecko/X Epic/1.2 Firefox/3.6.10 GTB7.1*/
const Device UA2291165 = {S100810, 105, { &P11953,&P17485,&P4777,&P18092 }};	/*Useragent - SonyEricssonU10i/R7BA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonu10i/UC Browser7.7.1.88/69/444*/
const Device UA2292114 = {S100811, 92, { &P17595,&P17296,&P18196,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:5.0) Gecko/X Thunderbird/5.0 Lightning/1.0b4*/
const Device UA2293450 = {S100812, 76, { &P17595,&P17297,&P20795,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:2.0.1) Gecko/X Postbox/2.5.2*/
const Device UA2294933 = {S100813, 92, { &P17595,&P17295,&P18196,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:5.0) Gecko/X Thunderbird/5.0 Lightning/1.0b5*/
const Device UA2297815 = {S100814, 80, { &P17595,&P17295,&P18194,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:6.0.1) Gecko/X Thunderbird/6.0.1*/
const Device UA2300786 = {S100815, 37, { &P14753,&P5890,&P17338,&P18092 }};	/*Useragent - nokian96-1/UC Browser7.9.0.102/69/352*/
const Device UA2301662 = {S100816, 39, { &P16600,&P22784,&P17338,&P18092 }};	/*Useragent - micromaxx600/UC Browser7.9.0.102/69/352*/
const Device UA2306918 = {S100817, 60, { &P15364,&P6704,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Windows 98; U; en) Presto/2.10.229 Version/11.60*/
const Device UA2318267 = {S100818, 70, { &P20167,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; GT-S5820; 320*480) UCWEB7.9.3.103/139/800*/
const Device UA2318959 = {S100819, 112, { &P18221,&P5843,&P20802,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia302/14.26; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/X S40OviBrowser/1.8.0.50.5*/
const Device UA2332552 = {S100820, 77, { &P13365,&P17483,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; lg-gu220) UCBrowser8.2.1.144/69/405/UCWEB Mobile*/
const Device UA2338620 = {S100821, 141, { &P15364,&P19822,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/12.10 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19*/
const Device UA2442852 = {S100822, 139, { &P15364,&P19822,&P18102,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/12.10 Chromium/18.0.1025.168 Chrome/18.0.1025.168 Safari/535.19*/
const Device UA2339664 = {S100823, 155, { &P21021,&P18066,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328t Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2370560 = {S100824, 43, { &P11768,&P17485,&P17338,&P18092 }};	/*Useragent - sonyericssonw395/UC Browser7.9.0.102/69/352*/
const Device UA2372576 = {S100825, 253, { &P12473,&P5204,&P18524,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/70/352*/
const Device UA2382619 = {S100826, 163, { &P15897,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; XT530 Build/V4.530) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2383420 = {S100827, 254, { &P14708,&P5204,&P18527,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/70/352*/
const Device UA2392196 = {S100828, 84, { &P11751,&P17483,&P17338,&P18092 }};	/*Useragent - LG-KP500 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1  lg-kp500 teleca/UC Browser7.9.0.102/70/352*/
const Device UA2401081 = {S100829, 197, { &P15794,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC Sensation XL with Beats Audio X315e Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2426667 = {S100830, 90, { &P14763,&P5776,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokian72) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2438904 = {S100831, 26, { &P14877,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-I9000/100.20303*/
const Device UA2440039 = {S100832, 103, { &P12258,&P17483,&P17347,&P18092 }};	/*Useragent - LG-GS290/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  lg-gs290/UC Browser8.0.3.107/69/444*/
const Device UA2441685 = {S100833, 77, { &P13365,&P17483,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; lg-gu220) UCBrowser8.3.0.154/69/444/UCWEB Mobile*/
const Device UA2444881 = {S100834, 142, { &P20705,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; HTC ONE 4G+ Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2449414 = {S100835, 137, { &P14207,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ADR6350 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2455091 = {S100836, 180, { &P20333,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Micromax P500(Funbook) Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2456325 = {S100837, 250, { &P18643,&P5043,&P18527,&P18092 }};	/*Useragent - Nokia311/5.0 (03.81) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.6.0.199/69/444*/
const Device UA2475794 = {S100838, 36, { &P12059,&P5204,&P23405,&P18092 }};	/*Useragent - MQQBrowser/Mini2.8 (Nokia2690/10.10)*/
const Device UA2500649 = {S100839, 134, { &P21301,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M031 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2518844 = {S100840, 162, { &P17595,&P18120,&P18727,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/1.6.1.1691 (Phoenix Viewer Release; Firebird skin) Safari/533.3*/
const Device UA2519349 = {S100841, 144, { &P15819,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; USCCADR6285US Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2520909 = {S100842, 65, { &P20265,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.6;Zh_cn;A52;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA2540991 = {S100843, 89, { &P12048,&P5448,&P4737,&P18092 }};	/*Useragent - T-Mobile Dash (compatible; MSIE 4.01; Windows CE; Smartphone; 320x240)/UC Browser7.2.2.51*/
const Device UA2542450 = {S100844, 134, { &P15364,&P5690,&P17810,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1;en-us;LenovoTablet Build/1.0) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2555435 = {S100845, 159, { &P11808,&P5204,&P24673,&P18092 }};	/*Useragent - Nokia5610d-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5610d-1) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2574031 = {S100846, 144, { &P23205,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; cs-cz; GOCLEVER TAB I71 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2607476 = {S100847, 70, { &P15842,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;TITAN X310e;7.10;8862)*/
const Device UA2607542 = {S100848, 143, { &P21812,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GSmart Rio R1 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2607551 = {S100849, 136, { &P21812,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Rio R1 Build/IMM76I) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2608840 = {S100850, 69, { &P12889,&P6437,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android3.2;Zh_cn;GT-P7300;800*1232;)UCWEB7.8.0.95/139/355*/
const Device UA2627966 = {S100851, 118, { &P12889,&P6437,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android3.2;Zh_cn;GT-P7300;800*1232;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA2619058 = {S100852, 168, { &P13452,&P5096,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-P1000R Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1000R, 36, 1.7.1)*/
const Device UA2624442 = {S100853, 166, { &P13452,&P5096,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1000, 39, 1.8.2)*/
const Device UA2625617 = {S100854, 168, { &P13452,&P5096,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-P1000M Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1000M, 39, 1.8.2)*/
const Device UA2614236 = {S100855, 166, { &P15919,&P18066,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.2.139/145/355*/
const Device UA2622353 = {S100856, 166, { &P15919,&P18066,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P3100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2637672 = {S100857, 170, { &P15919,&P18066,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-US; GT-P3100 Build/IML74K) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2619539 = {S100858, 97, { &P15919,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Messenger; 1.8.3) (espressorf; samsung; GT-P3100; 4.0.4/IMM76D)*/
const Device UA2623124 = {S100859, 145, { &P15919,&P18111,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2623641 = {S100860, 147, { &P15919,&P18111,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2636115 = {S100861, 106, { &P22043,&P17979,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.4 Android (16/4.1.1; 160dpi; 800x1232; samsung; GT-P5113; espresso10wifi; espresso10; en_US)*/
const Device UA2629284 = {S100862, 189, { &P12889,&P6546,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.20/286,1.9.20.286,2013-01-25 13:38, +0200, za*/
const Device UA2634960 = {S100863, 131, { &P17323,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P6800 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2663545 = {S100864, 90, { &P17332,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-c310) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2665452 = {S100865, 90, { &P15732,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-s365) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2685053 = {S100866, 29, { &P16362,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (SPH-D710 IMM76I)*/
const Device UA2699849 = {S100867, 151, { &P19973,&P5043,&P24675,&P18092 }};	/*Useragent - Nokia114/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia114) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2720981 = {S100868, 42, { &P12548,&P7472,&P4722,&P18092 }};	/*Useragent - alcatel-ot-807a/UC Browser7.0.0.41/69/4255*/
const Device UA2751907 = {S100869, 91, { &P15518,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5230) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2796050 = {S100870, 96, { &P15518,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-s5230) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2868627 = {S100871, 91, { &P15518,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5230) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2775803 = {S100872, 50, { &P15364,&P5890,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 (Nokia603;SymbianOS/9.1 Series60/3.0)*/
const Device UA2775902 = {S100873, 155, { &P24025,&P5629,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; U8812D Build/HuaweiU8812D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2804070 = {S100874, 36, { &P13554,&P5890,&P4742,&P18092 }};	/*Useragent - nokian79-1/UC Browser7.5.1.77/69/352*/
const Device UA2806658 = {S100875, 42, { &P15655,&P17484,&P4737,&P18092 }};	/*Useragent - samsung-gt-s3850/UC Browser7.2.2.51/69/352*/
const Device UA2807637 = {S100876, 126, { &P23669,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL55HX820; CC/AUS; en) Presto/2.7.61 Version/11.00*/
const Device UA2808034 = {S100877, 36, { &P13552,&P5129,&P4722,&P18092 }};	/*Useragent - NokiaC6-00/UC Browser7.0.0.41/50/351*/
const Device UA2809420 = {S100878, 56, { &P13977,&P5890,&P20777,&P18092 }};	/*Useragent - MQQBrowser/2.6 (NokiaN73(08);SymbianOS/9.1 Series60/3.0)*/
const Device UA2814549 = {S100879, 58, { &P12142,&P5890,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8 (SonyEricssonU5;SymbianOS/9.1 Series60/3.0)*/
const Device UA2815882 = {S100880, 85, { &P14864,&P17484,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s8500) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2815929 = {S100881, 51, { &P14984,&P5890,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 (Nokia5250;SymbianOS/9.1 Series60/3.0)*/
const Device UA2817477 = {S100882, 145, { &P13634,&P17483,&P24670,&P18092 }};	/*Useragent - LG-GS500/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1  Mozilla/5.0 (Java; U; en-in; lg-gs500) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2819609 = {S100883, 87, { &P12707,&P5055,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; nokia701) UCBrowser8.4.0.159/69/352/UCWEB Mobile  3gpp-gba*/
const Device UA2821890 = {S100884, 151, { &P13974,&P5096,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5670 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2823045 = {S100885, 154, { &P18223,&P5629,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT22i Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2825707 = {S100886, 132, { &P21239,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; M9 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2826246 = {S100887, 164, { &P12663,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-E730 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2827950 = {S100888, 156, { &P11950,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xperia X10 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2831759 = {S100889, 26, { &P18702,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-N8013/100.40004*/
const Device UA2833069 = {S100890, 151, { &P12568,&P5096,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; GT-B7510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2835084 = {S100891, 137, { &P12280,&P18432,&P20782,&P18092 }};	/*Useragent - MQQBrowser/31 Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3*/
const Device UA2836704 = {S100892, 164, { &P12980,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-E510 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2837069 = {S100893, 126, { &P14831,&P17484,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s7233e) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/352/UCWEB Mobile*/
const Device UA2837533 = {S100894, 182, { &P15501,&P5477,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; E16i Build/2.1.1.A.0.53; 320*480) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444*/
const Device UA2837746 = {S100895, 141, { &P14919,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; HTC_Pyramid Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2838048 = {S100896, 122, { &P13443,&P5129,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; SamsungI8910) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/50/444/UCWEB Mobile*/
const Device UA2839045 = {S100897, 137, { &P12216,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ADR6300 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2839083 = {S100898, 161, { &P14913,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SPH-M820-BST Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2839278 = {S100899, 151, { &P21229,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; iball Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2839941 = {S100900, 66, { &P15354,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.4; zh-cn; ST17i; 320*480) UCWEB7.9.0.94/139/352*/
const Device UA2842246 = {S100901, 217, { &P22114,&P17017,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; jblend) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/69/444/UCWEB Mobile/HS-T39_TD/1.0 Release/03.03.2011 Threadx/4.0 Mocor/W10 Browser/NF4.0 Profile/MIDP-2.0 Config/CLDC-1.1*/
const Device UA2842298 = {S100902, 68, { &P12889,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 3.1; zh-cn; GT-P7300; 800*1280) UCWEB7.9.0.94/139/444*/
const Device UA2842907 = {S100903, 156, { &P18448,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-in; ST26i Build/11.0.A.3.28) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2843666 = {S100904, 152, { &P14225,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Desire HD Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2844071 = {S100905, 67, { &P21207,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; Andi3n; 320*480) UCWEB7.9.0.94/139/444*/
const Device UA2844149 = {S100906, 68, { &P12280,&P17146,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 (iPhone3,1; iPhone OS 5.0.1; en_US) AppleWebKit/420+*/
const Device UA2844862 = {S100907, 94, { &P16566,&P22784,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; micromaxx266) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2845245 = {S100908, 167, { &P18295,&P18111,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; HTC One X Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2847430 = {S100909, 175, { &P16595,&P5477,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; Micromax A60 Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2847648 = {S100910, 139, { &P18674,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.9; zh-tw; GT-N7005 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2847858 = {S100911, 169, { &P19860,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; Karbonn A9+ Build/IMM76I) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2847894 = {S100912, 164, { &P12706,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P690 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2847975 = {S100913, 168, { &P18451,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT26ii Build/6.1.A.2.45) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2848132 = {S100914, 164, { &P15184,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; LG-P920 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2848944 = {S100915, 159, { &P13914,&P5843,&P24675,&P18092 }};	/*Useragent - Nokia5330-1d/5.0 (06.88) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia5330-1d) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2850550 = {S100916, 157, { &P21068,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE C1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2850698 = {S100917, 172, { &P18009,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTCEVODesign4G Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2850920 = {S100918, 167, { &P18280,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; HTC One S Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2852984 = {S100919, 168, { &P13746,&P5477,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; GT540 Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2853327 = {S100920, 122, { &P17346,&P19826,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B146 Flipboard/1.9.8*/
const Device UA2857087 = {S100921, 146, { &P15555,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; myTouch_4G_Slide Build/IMM76L) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2860578 = {S100922, 138, { &P14433,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9003 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2861087 = {S100923, 153, { &P21598,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Mobiistar Touch Kem 452 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2861170 = {S100924, 200, { &P18295,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC One X Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/387,1.9.28.387,2013-04-03 11:52, +0100, gb*/
const Device UA2861747 = {S100925, 159, { &P15397,&P5629,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-CN; ST18i Build/4.0.2.A.0.62) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2862045 = {S100926, 138, { &P18474,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9210T Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2862255 = {S100927, 142, { &P20250,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Celkon A 200 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2862490 = {S100928, 144, { &P20980,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; A97i Build/CelkonA97i_Celkon) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2862854 = {S100929, 152, { &P18295,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; HTC One X Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2863093 = {S100930, 158, { &P19856,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; LG-LS970 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2863595 = {S100931, 159, { &P19997,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; Lenovo P700 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2863957 = {S100932, 132, { &P23846,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A12 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2864704 = {S100933, 156, { &P20935,&P5629,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-US; ZTE-Z990G Build/GRJ22) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.1.275 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2864723 = {S100934, 136, { &P21804,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; P5-Mini Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2865029 = {S100935, 136, { &P20226,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; LG-E975 Build/JDQ39E) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2865452 = {S100936, 134, { &P23252,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; M6589 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2869059 = {S100937, 158, { &P16273,&P5629,&P22709,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SGH-T679 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 OneBrowser/4.1.2 Mobile Safari/533.1*/
const Device UA2949505 = {S100938, 52, { &P18364,&P5890,&P21746,&P18092 }};	/*Useragent - iBrowser/2.6 (NokiaC5-05;SymbianOS/9.1 Series60/3.0)*/
const Device UA2973383 = {S100939, 99, { &P14985,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8530) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3040693 = {S100940, 36, { &P12449,&P5890,&P4737,&P18092 }};	/*Useragent - NOKIAE5-00/UC Browser7.2.2.51/28/351*/
const Device UA3056544 = {S100941, 100, { &P15875,&P17485,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonck15i) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3098749 = {S100942, 174, { &P12657,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC EVO 3D X515m Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA3124394 = {S100943, 180, { &P14862,&P5204,&P22881,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.1.0.291*/
const Device UA33594 = {S100944, 96, { &P11631,&P5026,&P4948,&P18092 }};	/*Useragent - BlackBerry8110/4.5.0.174 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/303 UP.Browser/5.0.3.3*/
const Device UA3928 = {S100945, 79, { &P13297,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5362 Opera/9.50 (Windows NT 5.1; U; en) V2.31.841.6 (64646)*/
const Device UA5087 = {S100946, 101, { &P13343,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-D510/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA8736 = {S100947, 94, { &P13502,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Touch2_T3335 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA8988 = {S100948, 51, { &P13380,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KG300/V01 Obigo/WAP2.0 LG-KG300/V01 Obigo/WAP2.0*/
const Device UA61764 = {S100949, 89, { &P13380,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KG300j/V01 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1 LG-KG300j/V01 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA9165 = {S100950, 75, { &P13393,&P17483,&P17470,&P18092 }};	/*Useragent - LG-C2500 MIC/WAP2.0 MIDP-2.0/CLDC-1.0 LG-C2500 MIC/WAP2.0 MIDP-2.0/CLDC-1.0*/
const Device UA10238 = {S100951, 132, { &P13482,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-M8910H/1.0 SHP/VPP/R5 Mozilla/5.0 (rv:1.3) NetFront/3.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA53901 = {S100952, 67, { &P14057,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia2332c_CMCC/2.0 (08.20) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA19957 = {S100953, 51, { &P14230,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP235-orange/V10b Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA24654 = {S100954, 39, { &P14230,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP235 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA20026 = {S100955, 67, { &P15022,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-C280/C280BNIE1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA20148 = {S100956, 44, { &P15039,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP202-V10a Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA20149 = {S100957, 44, { &P15039,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP202-V10b Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA24655 = {S100958, 39, { &P15039,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP202 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA20402 = {S100959, 117, { &P15083,&P17605,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C6112C/1.0 RTK-E/1.0 DF/1.0 Release/08.18.2009 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21426 = {S100960, 53, { &P13936,&P17484,&P4901,&P18092 }};	/*Useragent - SAMSUNG-SGH-X460/1.0 UP.Browser/6.1.0.6 (GUI) MMP/1.0*/
const Device UA21817 = {S100961, 46, { &P15347,&P17483,&P3625,&P18092 }};	/*Useragent - LG-LG6300/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA22237 = {S100962, 115, { &P15407,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) ASUS-P527/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61840 = {S100963, 41, { &P11683,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonC905/R1FA SEMC/NewsReader/1.0*/
const Device UA22418 = {S100964, 83, { &P15460,&P17486,&P4203,&P18092 }};	/*Useragent - Motorola-C290-boost Obigo/Q04C1-1.9 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA86287 = {S100965, 77, { &P15460,&P17486,&P4203,&P18092 }};	/*Useragent - Motorola-C290 Obigo/Q04C1-1.9 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22430 = {S100966, 35, { &P15466,&P17017,&P17470,&P18092 }};	/*Useragent - BenQ-C30/1.0/WAP2.0/MIDP2.0/CLDC1.1*/
const Device UA22598 = {S100967, 105, { &P15491,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-A551/1.0 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA23499 = {S100968, 10, { &P12252,&P22839,&P17470,&P18092 }};	/*Useragent - ZTE-G X760*/
const Device UA32760 = {S100969, 10, { &P12760,&P22839,&P17470,&P18092 }};	/*Useragent - ZTE-G i766*/
const Device UA32907 = {S100970, 73, { &P14125,&P7063,&P3606,&P18092 }};	/*Useragent - BlackBerry7100/4.1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100*/
const Device UA32929 = {S100971, 121, { &P12795,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; MDA compact/3.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49797 = {S100972, 130, { &P15232,&P6661,&P4453,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9810; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/6.1.0.50 Mobile Safari/534.11+*/
const Device UA50050 = {S100973, 68, { &P13154,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-R7/1.0 ACS-NF/3.4 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA50426 = {S100974, 118, { &P11669,&P5776,&P4737,&P18092 }};	/*Useragent - Nokia7610/2.0 (7.0642.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.2.2.51/27/401*/
const Device UA51013 = {S100975, 33, { &P14672,&P17483,&P17660,&P18092 }};	/*Useragent - LG-KF900/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA54303 = {S100976, 104, { &P14179,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-A836/A836UXHK5 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA54988 = {S100977, 112, { &P13671,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-M3510/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA56793 = {S100978, 35, { &P13706,&P5129,&P4715,&P18092 }};	/*Useragent - Nokia5235/UC Browser7.4.0.65/50/351*/
const Device UA61282 = {S100979, 97, { &P16670,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Acer_P400 1.023h.00*/
const Device UA61283 = {S100980, 97, { &P16670,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) Acer_P400 1.023f.00*/
const Device UA62247 = {S100981, 140, { &P14389,&P17605,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S5630C_CMCC/1.0 RTK-E/1.0 SHP/VPP/R5 Release/17.08.2008 Browser/NetFront3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA62930 = {S100982, 43, { &P16189,&P6712,&P3402,&P18092 }};	/*Useragent - NOKIA-3806 UP.Browser/6.3.0.8 (GUI) MMP/2.0*/
const Device UA64613 = {S100983, 15, { &P14685,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHC170/1.0*/
const Device UA64804 = {S100984, 98, { &P14707,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-A436/ Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA1851591 = {S100985, 36, { &P13324,&P5055,&P4715,&P18092 }};	/*Useragent - NokiaN8-00/UC Browser7.4.0.65/50/352*/
const Device UA68585 = {S100986, 61, { &P14943,&P17017,&P3402,&P18092 }};	/*Useragent - BenQ-E61/1.00/WAP2.0 UP.Browser/6.3.0.4.c.1.102 (GUI) MMP/2.0*/
const Device UA87158 = {S100987, 45, { &P17530,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC3D UP.Browser/6.2.0.13.2 (GUI) MMP/2.0*/
const Device UA87163 = {S100988, 47, { &P17592,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SN3A UP.Browser/6.2.0.13.1.5 (GUI) MMP/2.0*/
const Device UA351731 = {S100989, 124, { &P15767,&P17090,&P22246,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.7.1*/
const Device UA355516 = {S100990, 124, { &P15767,&P17090,&P22246,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.7.1*/
const Device UA453145 = {S100991, 124, { &P15767,&P17090,&P22246,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.7.2*/
const Device UA582681 = {S100992, 124, { &P15767,&P17090,&P22246,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.7.2*/
const Device UA1947819 = {S100993, 124, { &P15767,&P17090,&P22246,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8H7 Flipboard/1.7.2*/
const Device UA2310422 = {S100994, 124, { &P15767,&P17090,&P22246,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 Flipboard/1.7.2*/
const Device UA529641 = {S100995, 63, { &P11747,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux;U;2.2;Zh_cn;HTC Desire;480*800;)UCWEB7.6.0.75/139/800*/
const Device UA2277659 = {S100996, 63, { &P11747,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux;U;2.2;Zh_cn;HTC Desire;480*800;)UCWEB7.6.0.75/139/999*/
const Device UA562764 = {S100997, 42, { &P14404,&P17484,&P4777,&P18092 }};	/*Useragent - samsung-gt-s5260/UC Browser7.7.1.88/69/444*/
const Device UA573337 = {S100998, 126, { &P15364,&P5690,&P18194,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1; rv:6.0.2) Gecko/X Thunderbird/6.0.2 OracleBeehiveExtension/1.0.0.4-OracleInternal Lightning/1.0b5*/
const Device UA615800 = {S100999, 42, { &P18254,&P17017,&P22250,&P18092 }};	/*Useragent - Flipboard/0.7.17/34,1.8.1,2012-05-01 00:51*/
const Device UA623967 = {S101000, 145, { &P12841,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC PH39100 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA625919 = {S101001, 76, { &P17595,&P18584,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookPro8%2C1)*/
const Device UA657973 = {S101002, 76, { &P17595,&P18584,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookAir4%2C2)*/
const Device UA685284 = {S101003, 71, { &P17595,&P18584,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (iMac12%2C2)*/
const Device UA826600 = {S101004, 73, { &P17595,&P18584,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBook5%2C1)*/
const Device UA872426 = {S101005, 72, { &P17595,&P18584,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacPro3%2C1)*/
const Device UA706824 = {S101006, 106, { &P19970,&P17485,&P17347,&P18092 }};	/*Useragent - SonyEricssonG502/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssong502/UC Browser8.0.3.107/69/444*/
const Device UA713337 = {S101007, 43, { &P15198,&P17484,&P17338,&P18092 }};	/*Useragent - samsung-gt-s3650/UC Browser7.9.0.102/69/352*/
const Device UA2303085 = {S101008, 70, { &P13724,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.1; zh-cn; SCH-i909; 480*800) UCWEB7.9.3.103/139/800*/
const Device UA760149 = {S101009, 53, { &P11929,&P5890,&P20780,&P18092 }};	/*Useragent - MQQBrowser/2.9 (Nokia5230;SymbianOS/9.1 Series60/3.0)*/
const Device UA767147 = {S101010, 44, { &P13206,&P17485,&P17347,&P18092 }};	/*Useragent - sonyericssonk530i/UC Browser8.0.3.107/69/444*/
const Device UA1937731 = {S101011, 44, { &P13206,&P17485,&P17347,&P18092 }};	/*Useragent - sonyericssonk530i/UC Browser8.0.3.107/70/352*/
const Device UA845127 = {S101012, 53, { &P14188,&P5890,&P20780,&P18092 }};	/*Useragent - MQQBrowser/2.9 (Nokia5233;SymbianOS/9.1 Series60/3.0)*/
const Device UA862100 = {S101013, 94, { &P15364,&P22197,&P23696,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8b5) Gecko/X Fedora/1.5-0.5.0.beta2 Firefox/1.4.1*/
const Device UA1842711 = {S101014, 117, { &P15364,&P18346,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 4.10; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.41 Safari/534.7*/
const Device UA897335 = {S101015, 86, { &P18088,&P17017,&P18255,&P18092 }};	/*Useragent - Opera/9.80 (J2ME/MIDP; Opera Mini/7.28870/27.1813; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2159984 = {S101016, 57, { &P18088,&P17017,&P18255,&P18092 }};	/*Useragent - Opera/9.50 (J2ME/MIDP; Opera Mini/7.29952/27.1659; U; en)*/
const Device UA905800 = {S101017, 84, { &P15364,&P16110,&P21170,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:13.0) Gecko/X Firefox/13.0.1 SeaMonkey/2.10.1*/
const Device UA925716 = {S101018, 100, { &P16963,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; DELL; Venue Pro)*/
const Device UA1872518 = {S101019, 17, { &P14831,&P17484,&P17606,&P18092 }};	/*Useragent - samsung-gt-s7233e*/
const Device UA1907394 = {S101020, 152, { &P13094,&P5629,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Lenovo A60 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018844 = {S101021, 146, { &P21612,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT711 Build/RSTCU10160) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2032368 = {S101022, 132, { &P21612,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; XT711 Build/RSTCU10160) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2617418 = {S101023, 131, { &P12889,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; es-es; GT-P7300 Build/HRI83) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2020082 = {S101024, 137, { &P23992,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; zh-cn; N12 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020266 = {S101025, 137, { &P21064,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-us; MA440 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2020497 = {S101026, 139, { &P12634,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HUAWEI-M835 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023955 = {S101027, 153, { &P13077,&P5629,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HW-HUAWEI_C8650/C8650V100R001C92B825; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2032830 = {S101028, 166, { &P13077,&P5629,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8650+ Build/V100R001C92B861) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025686 = {S101029, 157, { &P15575,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026479 = {S101030, 157, { &P15575,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030137 = {S101031, 133, { &P22381,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; SN10T1 Build/IML77) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2775112 = {S101032, 163, { &P14877,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2033497 = {S101033, 150, { &P21332,&P5629,&P20776,&P18092 }};	/*Useragent - MQQBrowser/25 Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-E860 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2134566 = {S101034, 68, { &P17595,&P18576,&P17316,&P18092 }};	/*Useragent - Safari/6531.9 CFNetwork/454.4 Darwin/10.0.0 (i386) (MacBookPro5%2C5)*/
const Device UA2139762 = {S101035, 86, { &P17637,&P5567,&P4536,&P18092 }};	/*Useragent - Mozilla/5.0 Opera/9.5 (KDDI-SN3R; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00*/
const Device UA2222951 = {S101036, 160, { &P15501,&P5477,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.53) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2261310 = {S101037, 37, { &P14454,&P5890,&P17338,&P18092 }};	/*Useragent - nokiae66-1/UC Browser7.9.0.102/69/352*/
const Device UA2267247 = {S101038, 103, { &P15364,&P22204,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en; rv:1.9.1.9) Gecko/X Fedora/2.26.3-9.fc11 Epiphany/2.22 Firefox/3.5*/
const Device UA2282217 = {S101039, 89, { &P15364,&P18110,&P22478,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.14) Gecko/X Firefox/2.0.0.14 Flock/1.1.2*/
const Device UA2287046 = {S101040, 35, { &P16003,&P17484,&P4777,&P18092 }};	/*Useragent - gt-s5233a/UC Browser7.7.1.88/69/352*/
const Device UA2294133 = {S101041, 92, { &P17595,&P17297,&P18196,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:5.0) Gecko/X Thunderbird/5.0 Lightning/1.0b5*/
const Device UA2295909 = {S101042, 126, { &P23663,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46EX720; CC/XGA; en) Presto/2.7.61 Version/11.00*/
const Device UA2298259 = {S101043, 60, { &P15364,&P18145,&P17209,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; FreeBSD amd64; rv:7.0) Gecko/X Firefox/7.0*/
const Device UA2300400 = {S101044, 73, { &P15364,&P17285,&P22537,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; rv:5.0) Gecko/X Firefox/5.0 Kylo/1.0.1.76141*/
const Device UA2300741 = {S101045, 106, { &P11683,&P17485,&P17338,&P18092 }};	/*Useragent - SonyEricssonC905/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonc905/UC Browser7.9.0.102/69/352*/
const Device UA2300974 = {S101046, 43, { &P12273,&P17484,&P17338,&P18092 }};	/*Useragent - samsung-gt-m5650/UC Browser7.9.0.102/69/352*/
const Device UA2302538 = {S101047, 34, { &P14532,&P17483,&P17338,&P18092 }};	/*Useragent - lg-t325/UC Browser7.9.0.102/69/352*/
const Device UA2306662 = {S101048, 116, { &P14569,&P5204,&P17347,&P18092 }};	/*Useragent - Nokia7610Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7610supernova/UC Browser8.0.3.107/69/352*/
const Device UA2309318 = {S101049, 106, { &P11698,&P17485,&P17347,&P18092 }};	/*Useragent - SonyEricssonW995/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonw995/UC Browser8.0.3.107/69/444*/
const Device UA2316352 = {S101050, 79, { &P16072,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN78)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA2324057 = {S101051, 89, { &P15364,&P5690,&P22354,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.2; Win64; x64; rv:9.2) Gecko/X Firefox/9.2-x64 PaleMoon/9.2-x64*/
const Device UA2328468 = {S101052, 45, { &P13800,&P5890,&P24665,&P18092 }};	/*Useragent - NOKIA5630d-1/UC Browser8.0.4.121/28/444/UCWEB*/
const Device UA2347110 = {S101053, 163, { &P13644,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; EVO Build/IML74K; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2423935 = {S101054, 98, { &P12260,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s5350) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2443566 = {S101055, 172, { &P12280,&P17146,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3 UCBrowser8.3.0.140/42/355*/
const Device UA2445532 = {S101056, 145, { &P20265,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-in; A52 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2461749 = {S101057, 98, { &P15197,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-b5310) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2463484 = {S101058, 139, { &P17595,&P17296,&P24196,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.2.0.1563 Safari/535.11*/
const Device UA2520854 = {S101059, 37, { &P14669,&P5843,&P23405,&P18092 }};	/*Useragent - MQQBrowser/Mini2.8 (NokiaX2-00/08.25)*/
const Device UA2586638 = {S101060, 138, { &P21607,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; VS870 4G Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2596329 = {S101061, 87, { &P15518,&P17017,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; s5230) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2610580 = {S101062, 142, { &P13452,&P5096,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; es-us; GT-P1000L Build/FROYO)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2614006 = {S101063, 143, { &P13452,&P5096,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2618807 = {S101064, 141, { &P13452,&P5096,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; fa-fa; GT-P1000 Build/FROYO)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2617072 = {S101065, 164, { &P15876,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P5100, 34, 1.6.0)*/
const Device UA2620035 = {S101066, 164, { &P15876,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P5100, 36, 1.7.1)*/
const Device UA2618253 = {S101067, 86, { &P12889,&P17979,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 Android (16/4.1.1; 160dpi; 800x1232; samsung; GT-P7300; p5; p3; en_US)*/
const Device UA2618674 = {S101068, 90, { &P12889,&P17979,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7300; GT-P7300; p3; en_US)*/
const Device UA2620923 = {S101069, 136, { &P15932,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P3110 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7*/
const Device UA2623005 = {S101070, 136, { &P15932,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; it-it; GT-P3110 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7*/
const Device UA2620655 = {S101071, 93, { &P17323,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Messenger; 1.8.3) (GT-P6800; samsung; GT-P6800; 3.2/HTJ85B)*/
const Device UA2624259 = {S101072, 45, { &P22037,&P17979,&P18062,&P18092 }};	/*Useragent - Windowshop/5.50.1010 (Android/4.0.4/GT-P3113)*/
const Device UA2629428 = {S101073, 90, { &P12879,&P17979,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.1 Android (13/3.2; 160dpi; 800x1232; samsung; GT-P7510; GT-P7510; p3; en_US)*/
const Device UA2627403 = {S101074, 152, { &P15919,&P18066,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2629071 = {S101075, 166, { &P13452,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-P1000 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2635057 = {S101076, 166, { &P13452,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P1000 Build/JRO03R) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2643144 = {S101077, 81, { &P13474,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAX5-01)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA2651656 = {S101078, 157, { &P14433,&P5629,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9003 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2797323 = {S101079, 100, { &P14060,&P17485,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw150i) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2865844 = {S101080, 100, { &P14060,&P17485,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonw150i) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2802276 = {S101081, 36, { &P11689,&P5890,&P4759,&P18092 }};	/*Useragent - nokian70-1/UC Browser7.6.1.82/69/352*/
const Device UA2802726 = {S101082, 94, { &P15364,&P5690,&P23688,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2) Gecko/X Firefox/3.6 Kylo/0.6.1.70394*/
const Device UA2804747 = {S101083, 17, { &P11996,&P17484,&P17606,&P18092 }};	/*Useragent - samsung-gt-s5560i*/
const Device UA2809905 = {S101084, 36, { &P11658,&P5776,&P17347,&P18092 }};	/*Useragent - nokia6680/UC Browser8.0.3.107/69/444*/
const Device UA2810723 = {S101085, 99, { &P17595,&P18207,&P21807,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.8; en-US; rv:1.9.2.16pre) Gecko/X Prism zdesktop/7.1.4*/
const Device UA2814225 = {S101086, 174, { &P15364,&P5096,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-C_N880S/N880SV1.0.0B04; 480*800; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2816505 = {S101087, 86, { &P14060,&P17485,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; sonyericssonw150i) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2817363 = {S101088, 137, { &P13070,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-E739 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2823554 = {S101089, 80, { &P14782,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAE61i)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA2825221 = {S101090, 150, { &P15028,&P5096,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; LG-P350 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2827748 = {S101091, 168, { &P16319,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; U8860 Build/HuaweiU8860) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2829652 = {S101092, 137, { &P22501,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MC7403A Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2832481 = {S101093, 131, { &P17595,&P18207,&P22453,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/535.2 (KHTML, like Gecko) Iron/15.0.900.1 Chrome/15.0.900.2 Safari/535.2*/
const Device UA2832599 = {S101094, 140, { &P15759,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC Vision Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2833846 = {S101095, 253, { &P14708,&P5204,&P18524,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/69/444*/
const Device UA2834109 = {S101096, 169, { &P23919,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; R800i Build/4.0.2.A.0.42) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2834562 = {S101097, 183, { &P18652,&P5629,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Micromax_A87 Build/GINGERBREAD; 480*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444*/
const Device UA2836240 = {S101098, 170, { &P21036,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; Spice Mi-425 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2838296 = {S101099, 119, { &P12487,&P5890,&P24672,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA5700) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.6.0.199/28/444/UCWEB Mobile*/
const Device UA2841936 = {S101100, 53, { &P18088,&P18432,&P24671,&P18092 }};	/*Useragent - IUC(U;iOS 6.0.1;Zh-cn;320*480;)/UCWEB8.5.0.163/42/997*/
const Device UA2842391 = {S101101, 92, { &P19998,&P18110,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Adr (Linux; U; 4.0.3; zh-cn; HUAWEI C8812 Build/C8812V100R001C92B926;480*800)*/
const Device UA2843284 = {S101102, 98, { &P11939,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssons312) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2843387 = {S101103, 162, { &P12731,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; MB855 Build/4.5.1A-1_SUN-254_13) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2843705 = {S101104, 136, { &P18276,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; F-08D Build/V05R11A) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2844174 = {S101105, 144, { &P11836,&P17485,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile  SonyEricssonW960i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2844415 = {S101106, 160, { &P12205,&P17485,&P24673,&P18092 }};	/*Useragent - SonyEricssonC903/R1GA Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; sonyericssonc903) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2844514 = {S101107, 71, { &P16356,&P18110,&P24673,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; GT-N7000; 800*1280) UCWEB8.7.4.225/145/800*/
const Device UA2844926 = {S101108, 153, { &P19859,&P18066,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2847795 = {S101109, 168, { &P15517,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MT15i Build/4.1.B.0.587) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2847791 = {S101110, 71, { &P15364,&P16110,&P18521,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64; rv:18.0) Gecko/X Firefox/18.0 slurp*/
const Device UA2855490 = {S101111, 65, { &P15364,&P18110,&P19981,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64; rv:19.0) Gecko/X Thunderbird/19.0*/
const Device UA2855769 = {S101112, 160, { &P18385,&P18598,&P24406,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; Nexus 7 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 OverDrive Media Console/2.6.4*/
const Device UA2857722 = {S101113, 165, { &P18713,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LG-P769 Build/IMM76L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2858896 = {S101114, 127, { &P20838,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; SHT21 Build/S2150) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2859183 = {S101115, 183, { &P13413,&P5477,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; Orange San Francisco Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2860032 = {S101116, 136, { &P20437,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; FJL21 Build/V31R42A) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2861379 = {S101117, 134, { &P21125,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; L-06D Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2861738 = {S101118, 128, { &P20446,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; F-05E Build/V13R36C) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2861940 = {S101119, 78, { &P17595,&P17296,&P21711,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:20.0) Gecko/X Thunderbird/20.0*/
const Device UA2861978 = {S101120, 152, { &P20542,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; PAP5000DUO Build/PrestigioPAP5000DUO) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2862520 = {S101121, 131, { &P23262,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TAC-70031 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2863218 = {S101122, 95, { &P16566,&P22784,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; micromaxx266) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2863677 = {S101123, 136, { &P18305,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LG-P936 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2864138 = {S101124, 145, { &P13157,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Sensation XE Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2864882 = {S101125, 90, { &P14568,&P17483,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-c105) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2864937 = {S101126, 140, { &P15794,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Sensation XL Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2865530 = {S101127, 139, { &P17346,&P19826,&P21783,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329 Safari/7534.48.3 Sleipnir/2.3.2m*/
const Device UA2865754 = {S101128, 137, { &P16273,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-T679 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2866082 = {S101129, 137, { &P21488,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A860K Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2866395 = {S101130, 136, { &P17968,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; skz_tenderloin Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2981384 = {S101131, 254, { &P14862,&P5204,&P18520,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser/8.5.0.185/83/352*/
const Device UA3068990 = {S101132, 109, { &P15364,&P18148,&P22782,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows Phone 8.0; Trident/7.0; rv:11.0; IEMobile/11.0; ARM; Touch; HTC; HTCPO881; X) like Gecko*/
const Device UA3068995 = {S101133, 117, { &P15364,&P18148,&P22782,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows Phone 8.0; ARM; Trident/7.0; Touch; rv:11.0; IEMobile/11.0; HTC; HTCPO881; Chameleon) like Gecko*/
const Device UA1627 = {S101134, 92, { &P14880,&P5033,&P4948,&P18092 }};	/*Useragent - BlackBerry8320/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100 UP.Browser/5.0.3.3*/
const Device UA2463 = {S101135, 158, { &P11977,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-GW520/V10e Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA21176 = {S101136, 158, { &P11977,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/5.0 (Vodafone/1.0/LG-GW520/V10c Browser/Obigo-Q7.1 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA4823 = {S101137, 107, { &P15081,&P17483,&P3728,&P18092 }};	/*Useragent - LG/KB770/v10c Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA20389 = {S101138, 134, { &P15081,&P17483,&P3728,&P18092 }};	/*Useragent - Mozilla/4.0 (Vodafone/1.0/LG-KB770/v10a Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1)*/
const Device UA27966 = {S101139, 107, { &P15081,&P17483,&P3728,&P18092 }};	/*Useragent - LG-KB770/v10d Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA55136 = {S101140, 83, { &P14288,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK310c/R4EA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA8534 = {S101141, 134, { &P13452,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2627793 = {S101142, 136, { &P13452,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-gb; GT-P1000 Build/ERD62) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2630536 = {S101143, 135, { &P13452,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; pt-br; GT-P1000L Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10139 = {S101144, 139, { &P17730,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-tw; Camangi-FM600 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA12321 = {S101145, 97, { &P11733,&P17485,&P3990,&P18092 }};	/*Useragent - Vodafone/SonyEricssonV800/R1S025 Browser/SEMC-Browser/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA13464 = {S101146, 46, { &P13374,&P5448,&P4626,&P18092 }};	/*Useragent - HTC_Diamond Opera/9.70 (Windows NT 5.1; U; en)*/
const Device UA13543 = {S101147, 98, { &P16177,&P5448,&P17183,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA14220 = {S101148, 115, { &P13796,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) ASUS-P526/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA14334 = {S101149, 73, { &P13451,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-GT-B3410/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA15607 = {S101150, 72, { &P15364,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-Zacubi iTunes/v3.1 MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA82945 = {S101151, 68, { &P15364,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V365/08.C8.37R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA17276 = {S101152, 102, { &P11915,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-D500C/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA64363 = {S101153, 60, { &P11936,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia6810/2.0 (4.24) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA2263998 = {S101154, 149, { &P13640,&P17486,&P20449,&P18092 }};	/*Useragent - MOT-ROKR E2/R564_G_12.00.38P Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola ROKR E2; 781) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.50 [en]*/
const Device UA21912 = {S101155, 98, { &P12014,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-J758/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA22211 = {S101156, 36, { &P15401,&P17017,&P4924,&P18092 }};	/*Useragent - HUAWEI-C6100/001.00 ACS-Netfront/3.2*/
const Device UA22361 = {S101157, 37, { &P15437,&P17017,&P4970,&P18092 }};	/*Useragent - SPH-M540/1.0 AU-MIC/2.0 (GUI) MMP/2.0*/
const Device UA23246 = {S101158, 44, { &P15437,&P17017,&P4970,&P18092 }};	/*Useragent - SPH-M540-parrot/1.0 AU-MIC/2.0 (GUI) MMP/2.0*/
const Device UA61890 = {S101159, 114, { &P12465,&P7205,&P4902,&P18092 }};	/*Useragent - Mozilla/5.0 (SCH-F339/F339DD06;U;REX/3.0;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*320;CTC/2.0) POLARIS/6.15.WEB*/
const Device UA45925 = {S101160, 43, { &P13186,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW850iv/R1ED SEMC/NewsReader/1.0*/
const Device UA50174 = {S101161, 62, { &P13186,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW850i/R1KG Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA25406 = {S101162, 86, { &P12638,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-SGH-i780/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA33090 = {S101163, 97, { &P12849,&P17484,&P3834,&P18092 }};	/*Useragent - SAMSUNG-GT-S7230E/1.0 SHP/VPP/R5 Dolfin/2.0 SMM-MMS/1.2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA33204 = {S101164, 78, { &P13103,&P17486,&P4203,&P18092 }};	/*Useragent - Motorola-K1m-boost Obigo/Q04C1 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49895 = {S101165, 72, { &P13103,&P17486,&P4203,&P18092 }};	/*Useragent - Motorola-K1m Obigo/Q04C1 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA51635 = {S101166, 66, { &P13951,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia7070_CMCC/2.0 (06.82) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA43644 = {S101167, 68, { &P13718,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHJ610/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA43757 = {S101168, 58, { &P13735,&P5768,&P3625,&P18092 }};	/*Useragent - Nokia6275i/2.0 (BL200V0200.nep) UP.Browser/6.2.3.8 MMP/2.0*/
const Device UA43779 = {S101169, 134, { &P13742,&P5448,&P4808,&P18092 }};	/*Useragent - ZTE Bluebelt II/1.0.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA44361 = {S101170, 65, { &P17803,&P17811,&P3807,&P18092 }};	/*Useragent - Lenovo-E268/VN_S105 LMP/LML Release/2010.10.25 Browser/Obigo_Q03c*/
const Device UA54225 = {S101171, 67, { &P17803,&P17811,&P3807,&P18092 }};	/*Useragent - Lenovo-E268/CMCC_S100 LMP/LML Release/2009.10.12 Browser/Obigo_Q03c*/
const Device UA44374 = {S101172, 81, { &P11981,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) i-mate 8502; 240x320*/
const Device UA49435 = {S101173, 155, { &P13024,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i455/UMHA3 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA49943 = {S101174, 101, { &P13113,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E256/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA49972 = {S101175, 87, { &P13126,&P17017,&P4780,&P18092 }};	/*Useragent - Nexian NX-G733/MTK Release/10.01.2009 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1*/
const Device UA50148 = {S101176, 102, { &P11689,&P5776,&P4722,&P18092 }};	/*Useragent - NokiaN70-1/5.0737.3.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.0.0.41/27/352*/
const Device UA51255 = {S101177, 35, { &P11929,&P5129,&P4737,&P18092 }};	/*Useragent - Nokia5230/UC Browser7.2.2.51/50/352*/
const Device UA50734 = {S101178, 118, { &P11782,&P17605,&P3976,&P18092 }};	/*Useragent - SAMSUNG-GT-S3600C/1.0 RTK-E/1.0 DF/1.0 Release/07.01.2008 Browser/NetFront3.4  Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54068 = {S101179, 61, { &P14108,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-C180/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA54430 = {S101180, 65, { &P13879,&P17483,&P17602,&P18092 }};	/*Useragent - LG-T320/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA56124 = {S101181, 36, { &P11929,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia5230/SymbianOS/9.1 Series60/3.0*/
const Device UA60536 = {S101182, 73, { &P11716,&P5448,&P4808,&P18092 }};	/*Useragent - HTC-8100/1.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320)*/
const Device UA61124 = {S101183, 13, { &P12639,&P17483,&P17660,&P18092 }};	/*Useragent - LG/U8130/v1.0*/
const Device UA61763 = {S101184, 82, { &P14321,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) i-mate 6150/ 480x640*/
const Device UA66965 = {S101185, 35, { &P15373,&P5129,&P4715,&P18092 }};	/*Useragent - Nokia5530/UC Browser7.4.0.65/50/351*/
const Device UA67005 = {S101186, 92, { &P14168,&P5448,&P4715,&P18092 }};	/*Useragent - HTC_Touch_Viva_T2223 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65*/
const Device UA67943 = {S101187, 35, { &P13706,&P5129,&P4759,&P18092 }};	/*Useragent - Nokia5235/UC Browser7.6.1.82/50/351*/
const Device UA68223 = {S101188, 34, { &P14454,&P5890,&P4759,&P18092 }};	/*Useragent - NokiaE66/UC Browser7.6.1.82/28/351*/
const Device UA68514 = {S101189, 39, { &P14519,&P5129,&P4777,&P18092 }};	/*Useragent - NokiaN97 mini/UC Browser7.7.1.88/50/444*/
const Device UA68628 = {S101190, 81, { &P14971,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-B5722/B5722DDKA1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA68852 = {S101191, 36, { &P11907,&P5890,&P4777,&P18092 }};	/*Useragent - NokiaE52-1/UC Browser7.7.1.88/28/444*/
const Device UA88591 = {S101192, 54, { &P17855,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-MA35 UP.Browser/6.2_7.2.7.1.K.2.225 (GUI) MMP/2.0*/
const Device UA89366 = {S101193, 47, { &P17541,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC3Q UP.Browser/6.2.0.15.1.1 (GUI) MMP/2.0*/
const Device UA90038 = {S101194, 31, { &P17746,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 F10B(c500;TB;W24H16)*/
const Device UA292128 = {S101195, 36, { &P11905,&P5890,&P4715,&P18092 }};	/*Useragent - NOKIA6120c/UC Browser7.4.0.65/28/352*/
const Device UA213503 = {S101196, 48, { &P15364,&P18570,&P22247,&P18092 }};	/*Useragent - Flipboard/1.5.1 CFNetwork/485.13.9 Darwin/11.0.0*/
const Device UA259541 = {S101197, 143, { &P11860,&P5776,&P18245,&P18091 }};	/*Useragent - Nokia6681/1.0 (2.30.0) Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; baiduspider; +http://www.baidu.com/search/spider.html)*/
const Device UA347091 = {S101198, 105, { &P18467,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (QtEmbedded; U; Linux; en-US) AppleWebKit/532.4 (KHTML, like Gecko) Kobo eReader Safari/532.4*/
const Device UA347092 = {S101199, 103, { &P18467,&P18110,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (QtEmbedded; U; Linux; C -) AppleWebKit/532.4 (KHTML, like Gecko) Kobo eReader Safari/532.4*/
const Device UA359734 = {S101200, 130, { &P15767,&P17146,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3*/
const Device UA360157 = {S101201, 133, { &P22264,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; md-us; M009F Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA446903 = {S101202, 103, { &P15364,&P22187,&P17558,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.13) Gecko/X Fedora/3.5.15-1.fc12 Firefox/3.6.13 GTB7.1*/
const Device UA515266 = {S101203, 142, { &P15364,&P17838,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Ubuntu/9.10 Chromium/10.0.647.0 Chrome/10.0.647.0 Safari/534.16*/
const Device UA570059 = {S101204, 54, { &P12088,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (NokiaX6-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA570165 = {S101205, 64, { &P11684,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (Nokia5800XpressMusic;SymbianOS/9.1 Series60/3.0)*/
const Device UA2260181 = {S101206, 65, { &P11684,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (Nokia5800wXpressMusic;SymbianOS/9.1 Series60/3.0)*/
const Device UA687125 = {S101207, 31, { &P11929,&P5129,&P24668,&P18092 }};	/*Useragent - Nokia5230/UCWEB8.3.0.133/50/999*/
const Device UA754441 = {S101208, 95, { &P12266,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-my511X-orange/1.0/ MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.393 (GUI)*/
const Device UA813691 = {S101209, 138, { &P18219,&P5843,&P24667,&P18092 }};	/*Useragent - Nokia202/2.0 (20.28) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia202) UCBrowser8.2.1.144/70/355/UCWEB Mobile*/
const Device UA814285 = {S101210, 148, { &P12280,&P17090,&P20782,&P18092 }};	/*Useragent - MQQBrowser/31 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3*/
const Device UA2325836 = {S101211, 148, { &P12280,&P17090,&P20782,&P18092 }};	/*Useragent - MQQBrowser/31 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3*/
const Device UA845102 = {S101212, 45, { &P15364,&P18145,&P18397,&P18092 }};	/*Useragent - Opera/6.11 (FreeBSD 4.7-RELEASE i386; U) [en]*/
const Device UA925752 = {S101213, 79, { &P13402,&P5222,&P17183,&P18092 }};	/*Useragent - acer_E101 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5)*/
const Device UA1868088 = {S101214, 116, { &P17595,&P17017,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; X 10_6_6; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.127 Safari/534.16*/
const Device UA1933815 = {S101215, 33, { &P18088,&P17017,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (MAUI WAP Browser)*/
const Device UA2018816 = {S101216, 145, { &P16805,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; GT-I6500U Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019447 = {S101217, 81, { &P15201,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.4;Zh_cn;HTC Sensation Z710e;540*960;)UCWEB7.8.0.95/139/352*/
const Device UA2024762 = {S101218, 168, { &P18283,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-I9300 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033457 = {S101219, 128, { &P22519,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; hu-hu; xPAD-70 Build/ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2167872 = {S101220, 144, { &P15767,&P17145,&P20776,&P18092 }};	/*Useragent - MQQBrowser/2.5.1 Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/7534.48.3*/
const Device UA2223434 = {S101221, 153, { &P15147,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; ST15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2242326 = {S101222, 46, { &P13677,&P17484,&P21759,&P18092 }};	/*Useragent - iBrowser/Mini2.8 (SAMSUNG-GT-S5253/S5253DDKJ2)*/
const Device UA2285815 = {S101223, 56, { &P17595,&P22475,&P21570,&P18092 }};	/*Useragent - Mail936 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac8%2C1)*/
const Device UA2313364 = {S101224, 43, { &P13648,&P5055,&P24665,&P18092 }};	/*Useragent - NokiaE7-00/UC Browser8.0.4.121/50/352/UCWEB*/
const Device UA2333782 = {S101225, 148, { &P12280,&P17090,&P20783,&P18092 }};	/*Useragent - MQQBrowser/32 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3*/
const Device UA2338126 = {S101226, 128, { &P21235,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; T708 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2372034 = {S101227, 251, { &P14711,&P5204,&P18516,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/70/355*/
const Device UA2439428 = {S101228, 44, { &P13179,&P17485,&P17338,&P18092 }};	/*Useragent - sonyericssonw350i/UC Browser7.9.0.102/69/352*/
const Device UA2442743 = {S101229, 250, { &P12059,&P5204,&P18516,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/70/444*/
const Device UA2446279 = {S101230, 161, { &P21347,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI U8661 Build/HuaweiU8661) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2486686 = {S101231, 98, { &P14182,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s5333) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2506107 = {S101232, 98, { &P15198,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s3650) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2520855 = {S101233, 65, { &P20067,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.5;Zh_cn;A56;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA2546454 = {S101234, 163, { &P12871,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Wildfire S A510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2559920 = {S101235, 91, { &P14763,&P5776,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian72) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2609572 = {S101236, 141, { &P13452,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2616959 = {S101237, 136, { &P13452,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; GT-P1000 Build/IMM76L) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7*/
const Device UA2623513 = {S101238, 137, { &P13452,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000T Build/IMM76L) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7*/
const Device UA2610109 = {S101239, 91, { &P17337,&P6546,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.4.4.12415); (Linux; U; Android 3.1; GT-P7500 Build/HONEYCOMB_MR1);*/
const Device UA2619439 = {S101240, 91, { &P17337,&P6546,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.3.8.9816); (Linux; U; Android 3.1; GT-P7500R Build/HONEYCOMB_MR1);*/
const Device UA2623394 = {S101241, 92, { &P17337,&P6546,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 2.0.4.15173); (Linux; U; Android 3.1; GT-P7500M Build/HONEYCOMB_MR1);*/
const Device UA2624131 = {S101242, 92, { &P17337,&P6546,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 3.1; GT-P7500D Build/HONEYCOMB_MR1);*/
const Device UA2615633 = {S101243, 164, { &P17323,&P6437,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6800 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2613545 = {S101244, 95, { &P15985,&P17979,&P22070,&P18092 }};	/*Useragent - Instagram 3.0.5 Android (13/3.2; 160dpi; 600x976; samsung; GT-P6200; GT-P6200; smdkc210; en_GB)*/
const Device UA2615713 = {S101245, 86, { &P15932,&P17979,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.4.5) (espressowifi;samsung;GT-P3110;4.0.3/IML74K)*/
const Device UA2618438 = {S101246, 38, { &P17337,&P17979,&P18062,&P18092 }};	/*Useragent - Android/4.0.4; samsung GT-P7500; 2.3.2*/
const Device UA2632417 = {S101247, 151, { &P17337,&P17979,&P18062,&P18092 }};	/*Useragent - Rdio.Net; Android/2.5; Android/4.0.4; DM=samsung; DN=GT-P7500R; DC=samsung; DF=samsung/GT-P7500R/GT-P7500R:4.0.4/IMM76D/RUXLH2:user/release-keys; DH=p3*/
const Device UA2624780 = {S101248, 164, { &P17337,&P6437,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7500 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2623194 = {S101249, 192, { &P22041,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6810 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0000, gb*/
const Device UA2622519 = {S101250, 71, { &P15919,&P17979,&P20428,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android Jelly Beast 4.2; GT-P3100 Build/JZO54K)*/
const Device UA2623092 = {S101251, 72, { &P15919,&P17979,&P20428,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android Jelly Bean 4.1.1; GT-P3100 Build/JRO03C)*/
const Device UA2626054 = {S101252, 72, { &P15919,&P17979,&P20428,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android Jelly Bean 4.1.2; GT-P3100 Build/JZO54K)*/
const Device UA2627587 = {S101253, 54, { &P15919,&P17979,&P20428,&P18092 }};	/*Useragent - nApps (Android OS 4.0.4; GT-P3100; NaverSearch; 4.3.0)*/
const Device UA2625861 = {S101254, 164, { &P17168,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7310 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7310, 35, 1.7.0)*/
const Device UA2625662 = {S101255, 54, { &P15876,&P17979,&P22080,&P18092 }};	/*Useragent - WhatsApp/2.9.296 Android/4.0.3 Device/samsung-GT-P5100*/
const Device UA2627310 = {S101256, 148, { &P13452,&P5096,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2627862 = {S101257, 148, { &P13452,&P5096,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2628164 = {S101258, 150, { &P13452,&P5096,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2627957 = {S101259, 151, { &P15919,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2627962 = {S101260, 151, { &P15919,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2641339 = {S101261, 145, { &P21070,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; Ultimate10-Android4.0 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2659978 = {S101262, 90, { &P14471,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-s310) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2685192 = {S101263, 83, { &P14834,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; Nokia5730S-1)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA2686630 = {S101264, 136, { &P18295,&P17979,&P18062,&P18092 }};	/*Useragent - One X Linux/2.6.35 Android/4.0.3 Release/06.08.2012 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mobile Safari/533.1*/
const Device UA2722477 = {S101265, 151, { &P12935,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; u8650 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2722547 = {S101266, 158, { &P21068,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE C1 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2775340 = {S101267, 75, { &P20183,&P18110,&P24666,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; MI-ONE Plus; 480*854) UCWEB8.1.1.122/145/32504*/
const Device UA2786847 = {S101268, 153, { &P13889,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; LG-P505 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2810519 = {S101269, 62, { &P15397,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.4;Zh_cn;ST18i;480*854;)UCWEB7.6.0.75/139/32037*/
const Device UA2811319 = {S101270, 74, { &P18382,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.3;Zh_cn;HTC Desire C;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA2813244 = {S101271, 156, { &P12871,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Wildfire S Build/MIUI; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2817372 = {S101272, 154, { &P21112,&P18066,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; OPPOX907 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2819707 = {S101273, 172, { &P15877,&P18149,&P18062,&P18091 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; DROID SPYDER Build/IMM76D; CyanogenMod-BLACK WIDOW ICS) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2821008 = {S101274, 65, { &P12280,&P17145,&P22070,&P18092 }};	/*Useragent - Instagram 3.0.0 (iPhone; iPhone OS 4.2.1; en_US) AppleWebKit/420+*/
const Device UA2824630 = {S101275, 168, { &P11833,&P17485,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile SonyEricssonP1i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2833514 = {S101276, 174, { &P19891,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; SAMSUNG-SGH-I717 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2942989 = {S101277, 91, { &P13734,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5603) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA4579 = {S101278, 100, { &P12558,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Touch_Cruise_T4242 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA21954 = {S101279, 71, { &P12558,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC_P3650)*/
const Device UA21959 = {S101280, 87, { &P12558,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_P3650 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA9296 = {S101281, 68, { &P13404,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V551/08.18.16R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA54635 = {S101282, 69, { &P13404,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V551J/08.18.40R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA10079 = {S101283, 98, { &P15163,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-U808/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA14228 = {S101284, 128, { &P13798,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 640x480; HTC_X7500/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20687 = {S101285, 129, { &P14075,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 640x480; Ameo 16GB/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA33108 = {S101286, 129, { &P14075,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 640x480; Ameo 16GB/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21367 = {S101287, 101, { &P15265,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E640/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA21527 = {S101288, 54, { &P15301,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-Z6m/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0*/
const Device UA87103 = {S101289, 53, { &P15301,&P17486,&P3625,&P18092 }};	/*Useragent - Motorola-Z6m UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0*/
const Device UA22365 = {S101290, 67, { &P15441,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-T309/T309UVEI1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA22431 = {S101291, 12, { &P15467,&P17483,&P17660,&P18092 }};	/*Useragent - LG/U310/v1.0*/
const Device UA22451 = {S101292, 101, { &P15475,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E420/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA22462 = {S101293, 68, { &P15479,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHD800/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22626 = {S101294, 85, { &P15503,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-S621/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA60567 = {S101295, 172, { &P15503,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-S621/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C, Alcatel-OT-S621/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA90190 = {S101296, 86, { &P15503,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-S621A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA22917 = {S101297, 40, { &P15525,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KM710 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA33343 = {S101298, 42, { &P11627,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonK850i/R1FA SEMC/NewsReader/1.0*/
const Device UA23363 = {S101299, 39, { &P15601,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KF240 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA23804 = {S101300, 84, { &P15626,&P17603,&P3534,&P18092 }};	/*Useragent - PANTECH-C630/IUUS10272008 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA525989 = {S101301, 77, { &P15626,&P17603,&P3534,&P18092 }};	/*Useragent - PANTECH-C630/IUUSX Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24275 = {S101302, 68, { &P12094,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-C975/82.21.91I MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64022 = {S101303, 87, { &P11700,&P17485,&P3600,&P18092 }};	/*Useragent - SonyEricssonW810i/R1BC Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA32535 = {S101304, 101, { &P12498,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-L600/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA32915 = {S101305, 54, { &P12791,&P22835,&P17470,&P18092 }};	/*Useragent - Spice_M6400/WapBrowserConfiguration2.0/MIDP2.0/CLDC1.1*/
const Device UA33069 = {S101306, 53, { &P16105,&P17486,&P3402,&P18092 }};	/*Useragent - MOT-W233/2.0.60 UP.Browser/6.3.0.6.c.33 (GUI) MMP/2.0*/
const Device UA34332 = {S101307, 145, { &P13675,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-tw; Vibo-A699 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA35835 = {S101308, 113, { &P11950,&P5062,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android1.6;Zh_cn;X10i;480*854;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355*/
const Device UA2034205 = {S101309, 64, { &P11950,&P5062,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android1.6;Zh_cn;X10i;480*854;)UCWEB7.8.0.95/139/444*/
const Device UA60587 = {S101310, 53, { &P13697,&P17483,&P17660,&P18092 }};	/*Useragent - LG/KU950/v1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA43793 = {S101311, 116, { &P16948,&P5448,&P17183,&P18092 }};	/*Useragent - SAMSUNG-GT-B7320/1.0 MIDP/2.1 CLDC/1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA43852 = {S101312, 133, { &P13754,&P5448,&P4808,&P18092 }};	/*Useragent - ZTE silverbelt/1.0.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA44661 = {S101313, 76, { &P14686,&P5448,&P4578,&P18092 }};	/*Useragent - HTCS621-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 320x240)*/
const Device UA49992 = {S101314, 153, { &P12371,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonSO-01B Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA78119 = {S101315, 141, { &P12371,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SO-01B Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA50514 = {S101316, 140, { &P11870,&P5448,&P4737,&P18092 }};	/*Useragent - HTC-S620/5.2.968/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone)/UC Browser7.2.2.51*/
const Device UA438837 = {S101317, 87, { &P11870,&P5448,&P4737,&P18092 }};	/*Useragent - HTC_S620 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone)/UC Browser7.2.2.51*/
const Device UA54025 = {S101318, 15, { &P14092,&P17017,&P17470,&P18092 }};	/*Useragent - AIKO 70W/WAP2.0*/
const Device UA54186 = {S101319, 57, { &P14145,&P5768,&P3625,&P18092 }};	/*Useragent - Nokia2865/2.0 (FL190V0200.nep) UP.Browser/6.2.3.8 MMP/2.0*/
const Device UA54214 = {S101320, 114, { &P14153,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-P250-ORANGE/P250BVHI2 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA60568 = {S101321, 85, { &P12549,&P22839,&P4223,&P18092 }};	/*Useragent - ZTE-E850(ChinaUnicom)/1.0 NetFront/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61127 = {S101322, 73, { &P12640,&P17017,&P3807,&P18092 }};	/*Useragent - Huawei/1.0/0HuaweiG6610/SW03 Browser/Obigo-Browser/Q03C MMS/Obigo-MMS/1.2*/
const Device UA61375 = {S101323, 91, { &P14395,&P17483,&P3534,&P18092 }};	/*Useragent - LG-GD310_CMCC Release/9.25.2009 Browser/Teleca-Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61466 = {S101324, 71, { &P12724,&P22839,&P4924,&P18092 }};	/*Useragent - ZTE-F156/1.0 ACS-NF/3.2 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64736 = {S101325, 39, { &P14701,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KE500 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA67166 = {S101326, 36, { &P13552,&P5129,&P4715,&P18092 }};	/*Useragent - NokiaC6-00/UC Browser7.4.0.65/50/351*/
const Device UA68068 = {S101327, 36, { &P11907,&P5890,&P4759,&P18092 }};	/*Useragent - NokiaE52-1/UC Browser7.6.1.82/28/352*/
const Device UA566179 = {S101328, 34, { &P14454,&P5890,&P4779,&P18092 }};	/*Useragent - NokiaE66/UC Browser7.8.0.95/28/358*/
const Device UA573842 = {S101329, 120, { &P12139,&P5204,&P17338,&P18092 }};	/*Useragent - Nokia5310XpressMusic/2.0 (08.32) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5310xpressmusic/UC Browser7.9.0.102/69/352*/
const Device UA80358 = {S101330, 130, { &P17488,&P5448,&P4578,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_Mercury/1.20.161.6/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; MIDP-2.0 Configuration/CLDC-1.1; PPC; 240x320)*/
const Device UA80649 = {S101331, 9, { &P13536,&P5890,&P4030,&P18092 }};	/*Useragent - Nokia6110*/
const Device UA87213 = {S101332, 45, { &P17527,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-HI3C UP.Browser/6.2.0.13.2 (GUI) MMP/2.0*/
const Device UA88019 = {S101333, 87, { &P17651,&P17662,&P3601,&P18092 }};	/*Useragent - SoftBank/1.0/812SHs/SHJ001 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA547278 = {S101334, 86, { &P17651,&P17662,&P3601,&P18092 }};	/*Useragent - SoftBank/1.0/812SH/SHJ002 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88014 = {S101335, 86, { &P17526,&P17662,&P3976,&P18092 }};	/*Useragent - SoftBank/1.0/821SH/SHJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88726 = {S101336, 87, { &P16688,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_X960*/
const Device UA245177 = {S101337, 134, { &P12788,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PG86100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA421804 = {S101338, 134, { &P12788,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; PG86100 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA414028 = {S101339, 148, { &P21327,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-gb; i-mobile IE 6010 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA427448 = {S101340, 41, { &P15364,&P18570,&P18245,&P18092 }};	/*Useragent - Maven/1.9 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA458890 = {S101341, 135, { &P21482,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; lt-lt; MID7108 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA473383 = {S101342, 129, { &P15364,&P17285,&P22510,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.1.6) Gecko/X FireDownload/2.0.1 Firefox/3.0.9 Wyzo/3.0.3 FireTorrent/2.0.1*/
const Device UA557613 = {S101343, 60, { &P15364,&P7127,&P18196,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.0; rv:5.0) Gecko/X Thunderbird/5.0*/
const Device UA559758 = {S101344, 46, { &P18254,&P18575,&P17470,&P18092 }};	/*Useragent - MobileRSS/3.4 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA1862517 = {S101345, 52, { &P18254,&P18575,&P17470,&P18092 }};	/*Useragent - FreeRSSReader/1.9.5 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA624960 = {S101346, 44, { &P17595,&P18570,&P18536,&P18092 }};	/*Useragent - Reeder/2.5.4 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA654463 = {S101347, 237, { &P17346,&P17145,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA654634 = {S101348, 136, { &P13054,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Mecha Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA654878 = {S101349, 44, { &P13216,&P17485,&P17347,&P18092 }};	/*Useragent - sonyericssonk750i/UC Browser8.0.3.107/69/444*/
const Device UA1938139 = {S101350, 44, { &P13216,&P17485,&P17347,&P18092 }};	/*Useragent - sonyericssonk750i/UC Browser8.0.3.107/69/352*/
const Device UA692374 = {S101351, 48, { &P17595,&P17779,&P20778,&P18092 }};	/*Useragent - MQQBrowser/27 (iOS; U; CPU like Mac OS X; zh-cn)*/
const Device UA805581 = {S101352, 122, { &P12280,&P17146,&P20777,&P18092 }};	/*Useragent - MQQBrowser/261 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405*/
const Device UA812575 = {S101353, 134, { &P15448,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0; en-us; SGH-T839 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA866157 = {S101354, 65, { &P15364,&P18144,&P18272,&P18092 }};	/*Useragent - Opera/9.80 (X11; OpenBSD i386; U; en) Presto/2.2.15 Version/10.11*/
const Device UA872410 = {S101355, 63, { &P14186,&P18110,&P24661,&P18092 }};	/*Useragent - JUC(Linux;U;2.2.1;Zh_cn;GT-S5830;320*480;)UCWEB7.7.0.85/139/800*/
const Device UA2019713 = {S101356, 68, { &P14090,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2;Zh_cn;GT-I5801;240*400;)UCWEB7.8.0.95/140/444*/
const Device UA1901255 = {S101357, 44, { &P13183,&P17485,&P17338,&P18092 }};	/*Useragent - sonyericssonk810i/UC Browser7.9.0.102/69/352*/
const Device UA1955661 = {S101358, 176, { &P12972,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-T989 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2810837 = {S101359, 171, { &P14031,&P5477,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; GT-I5503 Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2018093 = {S101360, 148, { &P16473,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; ru-ru; GSmart-G1305 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2020678 = {S101361, 144, { &P20894,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.5; en-us; M3 media box board Build/DTV) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023030 = {S101362, 138, { &P21121,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; KIRA_N9000 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024359 = {S101363, 149, { &P20556,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; pl-pl; Android for A701 Board Build/A701) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026539 = {S101364, 135, { &P22985,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-7000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029151 = {S101365, 137, { &P18195,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; YP-G70 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031323 = {S101366, 135, { &P22985,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT-7000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031450 = {S101367, 71, { &P13591,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2.2;Zh_cn;ZTE-BLADE;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA2136816 = {S101368, 138, { &P15364,&P17838,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.6 (KHTML, like Gecko) Ubuntu/9.10 Chromium/7.0.507.0 Chrome/7.0.507.0 Safari/534.6*/
const Device UA2148133 = {S101369, 82, { &P18254,&P7480,&P17191,&P18089 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Googlebot/2.1; +http://www.google.com/bot.html)*/
const Device UA2213632 = {S101370, 126, { &P23651,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL22EX320; CC/GBR; en) Presto/2.7.61 Version/11.00*/
const Device UA2220782 = {S101371, 79, { &P12029,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-in; NOKIAN82)/UC Browser8.4.0.159/28/352/UCWEB Mobile*/
const Device UA2330350 = {S101372, 79, { &P13445,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN85)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2339092 = {S101373, 161, { &P12871,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Wildfire S A510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2339290 = {S101374, 162, { &P13042,&P18066,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HTC Incredible S Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2339931 = {S101375, 153, { &P21435,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ZTE V788D Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2340148 = {S101376, 173, { &P21223,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HW-HUAWEI_C8810 Build/C8810V100R001C92B862) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2370324 = {S101377, 122, { &P15364,&P17483,&P3459,&P18092 }};	/*Useragent - LG-E500/V100 Obigo/Q7.3 MMS/LG-MMS-V1.1/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2376961 = {S101378, 81, { &P13042,&P18110,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Adr (Linux; U; 2.3.5; zh-cn; HTC Incredible S Build/GRJ90;480*800)*/
const Device UA2395777 = {S101379, 148, { &P15028,&P5096,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; LG-P350 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2411645 = {S101380, 138, { &P12280,&P18117,&P20781,&P18092 }};	/*Useragent - MQQBrowser/30 Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA2419571 = {S101381, 146, { &P13644,&P24693,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; PC36100 Build/black bean AOKP) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2440215 = {S101382, 73, { &P22942,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; DOOV_D700; 320*480) UCWEB7.9.3.103/139/32669*/
const Device UA2440439 = {S101383, 11, { &P16778,&P22835,&P17470,&P18092 }};	/*Useragent - Spice M5262*/
const Device UA2446807 = {S101384, 166, { &P20067,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; A56 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2459559 = {S101385, 90, { &P16024,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-t310i) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2613110 = {S101386, 95, { &P13452,&P5629,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.4.5.12736); (Linux; U; Android 2.3.3; GT-P1000 Build/GINGERBREAD_MR1);*/
const Device UA2620530 = {S101387, 96, { &P13452,&P5629,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 2.3.3; GT-P1000R Build/GINGERBREAD_MR1);*/
const Device UA2621954 = {S101388, 96, { &P13452,&P5629,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 2.3.3; GT-P1000M Build/GINGERBREAD_MR1);*/
const Device UA2618701 = {S101389, 98, { &P13452,&P17979,&P22070,&P18092 }};	/*Useragent - Instagram 3.0.5 Android (10/2.3.3; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; gt-p1000; en_GB)*/
const Device UA2618744 = {S101390, 95, { &P13452,&P17979,&P22070,&P18092 }};	/*Useragent - Instagram 3.0.5 Android (8/2.2; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; GT-P1000; en_GB)*/
const Device UA2622070 = {S101391, 148, { &P17337,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622389 = {S101392, 147, { &P17337,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622514 = {S101393, 166, { &P17337,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500R Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7500R, 36, 1.7.1)*/
const Device UA2623209 = {S101394, 164, { &P17337,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7500, 36, 1.7.1)*/
const Device UA2636300 = {S101395, 166, { &P17337,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P7500D Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7500D, 40, 1.8.5)*/
const Device UA2617295 = {S101396, 136, { &P17323,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; GT-P6800 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2623609 = {S101397, 210, { &P12879,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-P7510 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.16/276,1.9.16.276,2012-12-21 16:37, +0000, gb*/
const Device UA2629607 = {S101398, 210, { &P12879,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7510 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, -0500, us*/
const Device UA2631421 = {S101399, 209, { &P12879,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7510 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, -0500, us*/
const Device UA2625015 = {S101400, 54, { &P15919,&P17979,&P22080,&P18092 }};	/*Useragent - WhatsApp/2.9.369 Android/4.0.4 Device/samsung-GT-P3100*/
const Device UA2634986 = {S101401, 154, { &P15985,&P6437,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-CN; GT-P6200 Build/HTJ85B) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2628401 = {S101402, 166, { &P15882,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P5110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2636309 = {S101403, 166, { &P15882,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P5110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2629186 = {S101404, 166, { &P15932,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2632687 = {S101405, 166, { &P15932,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P3110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2632609 = {S101406, 164, { &P17337,&P6437,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7500 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2639222 = {S101407, 30, { &P15364,&P21445,&P4811,&P18092 }};	/*Useragent - Opera/9.00 (Windows ME; U; en)*/
const Device UA2641052 = {S101408, 71, { &P18714,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; ZTE_V889D; 480*800) UCWEB7.9.4.145/139/800*/
const Device UA2759797 = {S101409, 35, { &P18643,&P5043,&P23405,&P18092 }};	/*Useragent - MQQBrowser/Mini2.8 (Nokia311/03.81)*/
const Device UA2778961 = {S101410, 100, { &P12832,&P17485,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonck13i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2813718 = {S101411, 157, { &P16356,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2813870 = {S101412, 102, { &P15364,&P24094,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; CE-HTML/1.0 NETTV/3.0.1; PHILIPS-AVM-2012; en) Presto/2.9.167 Version/11.50*/
const Device UA2818024 = {S101413, 251, { &P18267,&P5043,&P18516,&P18092 }};	/*Useragent - NokiaX2-02/2.0 (11.57) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/70/355*/
const Device UA2827414 = {S101414, 163, { &P14090,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I5801 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2830838 = {S101415, 159, { &P20262,&P18066,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Karbonn_A18 Build/KarbonnA18) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2831174 = {S101416, 171, { &P12972,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2963485 = {S101417, 100, { &P12351,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3653w) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3015459 = {S101418, 154, { &P16503,&P5204,&P24678,&P18092 }};	/*Useragent - Nokia7310/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia7310) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3085451 = {S101419, 67, { &P17346,&P18432,&P22070,&P18092 }};	/*Useragent - Instagram 3.0.1 (iPod touch; iPhone OS 6.0; en_US) AppleWebKit/420+*/
const Device UA50339 = {S101420, 61, { &P11636,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW705/R1GA Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA51243 = {S101421, 61, { &P12205,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonC903/R1GA Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2279 = {S101422, 100, { &P11944,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonT715/R1HA Browser/NetFront Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.5*/
const Device UA4363 = {S101423, 42, { &P11944,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonT715a/R1GA SEMC/NewsReader/1.0*/
const Device UA4833 = {S101424, 92, { &P12210,&P5027,&P4948,&P18092 }};	/*Useragent - BlackBerry8120/4.3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100 UP.Browser/5.0.3.3*/
const Device UA5025 = {S101425, 60, { &P11833,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonP1i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA5341 = {S101426, 73, { &P12251,&P5448,&P22272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) ASUSP550/1.0*/
const Device UA6013 = {S101427, 27, { &P12080,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-S3653/S365MDDIJ1*/
const Device UA61705 = {S101428, 16, { &P12080,&P17484,&P17606,&P18092 }};	/*Useragent - samsung-gt-s3653*/
const Device UA7635 = {S101429, 73, { &P13367,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) O2 Xda Zinc*/
const Device UA9017 = {S101430, 107, { &P13387,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-T209/T209UVFD3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA9019 = {S101431, 108, { &P13387,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-T209R/T209UVFG1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA9875 = {S101432, 86, { &P13398,&P5448,&P17183,&P18092 }};	/*Useragent - SAMSUNG-GT-B7610/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA10251 = {S101433, 58, { &P13486,&P19950,&P3625,&P18092 }};	/*Useragent - SAGEM-myC5-2v/1.0 UP.Browser/6.2.3.3.g.2.107 (GUI) MMP/2.0*/
const Device UA30284 = {S101434, 58, { &P13486,&P19950,&P3625,&P18092 }};	/*Useragent - SAGEM-myC5-2v/1.0 UP.Browser/6.2.3.3.g.2.106 (GUI) MMP/2.0*/
const Device UA83191 = {S101435, 93, { &P13486,&P19950,&P3625,&P18092 }};	/*Useragent - SAGEM-myC5-2/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.6.d.5 (GUI) MMP/1.0*/
const Device UA15370 = {S101436, 74, { &P11812,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; O2 Xda Atom; PPC; 240x320)*/
const Device UA16365 = {S101437, 88, { &P15210,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SAMSUNG-GT-i8000V; Windows Phone 6.5)*/
const Device UA925774 = {S101438, 91, { &P15210,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 SAMSUNG-GT-i8000H/1.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5)*/
const Device UA18812 = {S101439, 59, { &P11937,&P17483,&P3534,&P18092 }};	/*Useragent - LG-TU915 Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA19708 = {S101440, 78, { &P15007,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC i-mate JAMA; 240x320; PPC)*/
const Device UA78793 = {S101441, 116, { &P14218,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-C3300K/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA21378 = {S101442, 39, { &P11988,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KM330 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA21633 = {S101443, 55, { &P13921,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-V3am/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0*/
const Device UA86578 = {S101444, 53, { &P13921,&P17486,&P3625,&P18092 }};	/*Useragent - Motorola_V3c UP.Browser/6.2.3.4.c.1.109 (GUI) MMP/2.0*/
const Device UA21727 = {S101445, 101, { &P15020,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E890/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA21799 = {S101446, 84, { &P15343,&P5208,&P4902,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Polaris 6.15.web; Brew 3.1.5; U; en) LG-LG235/1.0*/
const Device UA55284 = {S101447, 41, { &P14305,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonC901/R1GA SEMC/NewsReader/1.0*/
const Device UA22865 = {S101448, 39, { &P15521,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KF245 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA22940 = {S101449, 46, { &P15536,&P6476,&P3402,&P18092 }};	/*Useragent - UTS-7126M/1.0 UP.Browser/6.3.0.7 (GUI) MMP/2.0*/
const Device UA22952 = {S101450, 78, { &P15542,&P5448,&P4578,&P18092 }};	/*Useragent - HPiPAQrw6800/1.0/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)*/
const Device UA23086 = {S101451, 40, { &P15559,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GB210 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA60387 = {S101452, 164, { &P15567,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaX5-00_TD/CMCC SymbianOS/9.3 Series60/3.2 Release/11.1026.019; Mozilla/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML,like Gecko) Safari/413*/
const Device UA24093 = {S101453, 43, { &P15667,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GB170/V01 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA26132 = {S101454, 98, { &P12151,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-F266/ Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA27975 = {S101455, 99, { &P12203,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-GT-S3310C/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA30149 = {S101456, 99, { &P13660,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-V820L/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA32785 = {S101457, 86, { &P12768,&P17485,&P3499,&P18092 }};	/*Useragent - SonyEricssonW150/R2AD034 TelecaBrowser/Q07C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA33168 = {S101458, 139, { &P12877,&P17605,&P3976,&P18092 }};	/*Useragent - SAMSUNG-GT-C3730C_CMCC/1.0 RTK-E/1.0 SHP/VPP-R5 Release/8.17.2009 Browser/NetFront3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1 SMM-MMS/1.2.0*/
const Device UA43708 = {S101459, 53, { &P13727,&P22839,&P3807,&P18092 }};	/*Useragent - ZTE-U E880/Obigo Q03c/MIDP-2.0 Configuration/CDLC-1.1*/
const Device UA44072 = {S101460, 142, { &P14838,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-SGH-i908E_CMCC/1.0 Release/03.07 Browser/IE6 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA44196 = {S101461, 52, { &P13801,&P17484,&P4902,&P18092 }};	/*Useragent - SCH-M519/ POLARIS/6.15 (GUI; compatible; UP.Browser)*/
const Device UA56653 = {S101462, 125, { &P13830,&P5208,&P4902,&P18092 }};	/*Useragent - Mozilla/5.0 (SCH-W589/W589DK26;U;REX/3.0;BREW/3.1.5;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*320;CTC/2.0) POLARIS/6.15.WEB*/
const Device UA49378 = {S101463, 53, { &P13925,&P17483,&P17660,&P18092 }};	/*Useragent - LG/KU800/v1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA67271 = {S101464, 74, { &P13378,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-J700/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA53935 = {S101465, 86, { &P14070,&P17662,&P3601,&P18092 }};	/*Useragent - SoftBank/1.0/920SC/SCJ002 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA57265 = {S101466, 42, { &P14070,&P17662,&P3601,&P18092 }};	/*Useragent - Mozilla/4.08 (920SC;SoftBank) NetFront/3.3*/
const Device UA55851 = {S101467, 86, { &P14308,&P5448,&P4578,&P18092 }};	/*Useragent - Dopod838Pro/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; Dopod838Pro)*/
const Device UA67642 = {S101468, 36, { &P14748,&P5890,&P4715,&P18092 }};	/*Useragent - NokiaE72-1/UC Browser7.4.0.65/28/355*/
const Device UA59267 = {S101469, 78, { &P16988,&P5448,&P22272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) Toshiba G910_G920*/
const Device UA61037 = {S101470, 47, { &P12614,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-MX8500/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA62289 = {S101471, 66, { &P14395,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GD310/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA63347 = {S101472, 21, { &P14505,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KG130 Obigo/WAP2.0*/
const Device UA63858 = {S101473, 91, { &P15364,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-VF540/1.0/EVN_MK MIDP/2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.1.576 (GUI)*/
const Device UA64959 = {S101474, 103, { &P14708,&P5204,&P4759,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5130c-2/UC Browser7.6.1.82/70/351*/
const Device UA67203 = {S101475, 116, { &P11860,&P5776,&P4715,&P18092 }};	/*Useragent - Nokia6681/2.0 (6.09.00) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.4.0.65/27/351*/
const Device UA67611 = {S101476, 90, { &P14845,&P5448,&P4715,&P18092 }};	/*Useragent - HTC_Touch_HD_T8282 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65*/
const Device UA67811 = {S101477, 30, { &P11929,&P5129,&P24660,&P18092 }};	/*Useragent - Nokia5230/UCWEB7.6.0.75/50/800*/
const Device UA68769 = {S101478, 103, { &P14708,&P5204,&P4777,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5130c-2/UC Browser7.7.1.88/70/351*/
const Device UA68854 = {S101479, 43, { &P20126,&P17485,&P4777,&P18092 }};	/*Useragent - sonyericssonk800i/UC Browser7.7.1.88/69/444*/
const Device UA253933 = {S101480, 43, { &P20126,&P17485,&P4777,&P18092 }};	/*Useragent - sonyericssonk800i/UC Browser7.7.1.88/69/352*/
const Device UA69153 = {S101481, 116, { &P11679,&P5776,&P4779,&P18092 }};	/*Useragent - Nokia6630/1.0 (5.03.08) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/27/351*/
const Device UA69551 = {S101482, 86, { &P14763,&P5776,&P4030,&P18092 }};	/*Useragent - NokiaN72/S024 LMP/2010.06.09 Release/Obigo2.0 Profile/MIDP-2.0 Configuaration/CLDC-1.1*/
const Device UA83588 = {S101483, 21, { &P16240,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonT200/R101*/
const Device UA87038 = {S101484, 35, { &P17762,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SH903iTV(c100;TB;W24H16)*/
const Device UA87206 = {S101485, 47, { &P17862,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3E UP.Browser/6.2.0.13.1.5 (GUI) MMP/2.0*/
const Device UA87224 = {S101486, 45, { &P17642,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3G UP.Browser/6.2.0.13.2 (GUI) MMP/2.0*/
const Device UA100180 = {S101487, 50, { &P17900,&P17895,&P17896,&P18092 }};	/*Useragent - TIANYU-KTOUCH/A7726/MIDP2.0/CLDC1.1/Screen-240x320*/
const Device UA241662 = {S101488, 132, { &P17595,&P17295,&P22451,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.1 (KHTML, like Gecko)  Iron/13.0.800.0 Chrome/13.0.800.0 Safari/535.1*/
const Device UA349027 = {S101489, 32, { &P14748,&P5890,&P24666,&P18092 }};	/*Useragent - NokiaE72-1/UCWEB8.1.0.104/28/800*/
const Device UA427437 = {S101490, 121, { &P12280,&P18117,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.8.1*/
const Device UA427453 = {S101491, 121, { &P12280,&P18117,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Flipboard/1.8.1*/
const Device UA601623 = {S101492, 121, { &P12280,&P18117,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Flipboard/1.8.2*/
const Device UA615695 = {S101493, 123, { &P12280,&P18117,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.8.4*/
const Device UA622719 = {S101494, 121, { &P12280,&P18117,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.8.4*/
const Device UA623015 = {S101495, 121, { &P12280,&P18117,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B179 Flipboard/1.8.4*/
const Device UA806581 = {S101496, 123, { &P12280,&P18117,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.8.3*/
const Device UA825896 = {S101497, 121, { &P12280,&P18117,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.8.3*/
const Device UA448332 = {S101498, 58, { &P15364,&P18110,&P24644,&P18092 }};	/*Useragent - JUC(Linux;U;2.2;Zh_cn;bravo;480*800;)UCWEB7.4.1.61/139/800*/
const Device UA485061 = {S101499, 35, { &P15364,&P17017,&P18398,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; Opera 4.0)*/
const Device UA495818 = {S101500, 82, { &P15207,&P5448,&P4715,&P18092 }};	/*Useragent - HTC_P3400i-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65*/
const Device UA503755 = {S101501, 85, { &P21408,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) WS027SH*/
const Device UA563773 = {S101502, 119, { &P15351,&P5448,&P4779,&P18092 }};	/*Useragent - ASUS-GalaxyMini/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.8.0.95*/
const Device UA575491 = {S101503, 43, { &P17595,&P18120,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 (U; CPU like Mac OS X; zh-cn)*/
const Device UA621894 = {S101504, 46, { &P17968,&P17979,&P24432,&P18092 }};	/*Useragent - Dolphin http client/8.1.0 beta2(135) (Android)*/
const Device UA621942 = {S101505, 40, { &P17968,&P17979,&P24432,&P18092 }};	/*Useragent - Dolphin http client/8.1.0(136) (Android)*/
const Device UA622033 = {S101506, 40, { &P17968,&P17979,&P24432,&P18092 }};	/*Useragent - Dolphin http client/8.1.1(137) (Android)*/
const Device UA622239 = {S101507, 45, { &P17968,&P17979,&P24432,&P18092 }};	/*Useragent - Dolphin http client/8.1.0 beta(133) (Android)*/
const Device UA652368 = {S101508, 37, { &P14540,&P5843,&P23399,&P18092 }};	/*Useragent - MQQBrowser/Mini2.2 (NokiaX2-01/08.63)*/
const Device UA747518 = {S101509, 101, { &P15364,&P7133,&P17652,&P18092 }};	/*Useragent - MediaGo/2.1.0.392 (Windows NT 6.1; WOW64) PlayStationStore/4.7.14.14146 PlaybackEngine/1.88.106.12050*/
const Device UA2167438 = {S101510, 100, { &P15364,&P7133,&P17652,&P18092 }};	/*Useragent - MediaGo/2.2.0.223 (Windows NT 6.1; WOW64) PlayStationStore/4.9.4.14625 PlaybackEngine/1.92.168.06150*/
const Device UA767380 = {S101511, 135, { &P22519,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; xPAD-70 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA769047 = {S101512, 175, { &P16595,&P5477,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; Micromax A60 Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA804648 = {S101513, 123, { &P15364,&P5690,&P24205,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.5.9225.201*/
const Device UA831826 = {S101514, 124, { &P15364,&P17285,&P24207,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.7.10246.201*/
const Device UA862674 = {S101515, 56, { &P15364,&P17017,&P18397,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; UNIX) Opera 6.11 [en]*/
const Device UA899829 = {S101516, 129, { &P15364,&P16110,&P22430,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5 Comodo_Dragon/19.1.0.0*/
const Device UA899838 = {S101517, 122, { &P15364,&P16110,&P22430,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5 Comodo_Dragon/19.1.0.0*/
const Device UA925724 = {S101518, 95, { &P18073,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; T9295)*/
const Device UA1869394 = {S101519, 63, { &P11747,&P18110,&P24645,&P18092 }};	/*Useragent - JUC(Linux;U;2.2;Zh_cn;HTC Desire;480*800;)UCWEB7.5.0.66/139/800*/
const Device UA1874324 = {S101520, 62, { &P14225,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux;U;2.2;Zh_cn;Desire HD;480*800;)UCWEB7.6.0.75/139/800*/
const Device UA1885326 = {S101521, 42, { &P17699,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; SN3O)*/
const Device UA1890180 = {S101522, 69, { &P15364,&P17017,&P17829,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/4.6) KHTML/4.6.0 (like Gecko) SUSE*/
const Device UA1994257 = {S101523, 175, { &P16595,&P5477,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; Micromax A60 Build/ERE27) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA1995764 = {S101524, 34, { &P12059,&P5204,&P21764,&P18092 }};	/*Useragent - iBrowser/Mini2.7 (Nokia2690/10.10)*/
const Device UA1995840 = {S101525, 114, { &P15364,&P17285,&P18528,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.46 Safari/536.5 Sleipnir/3.5.2*/
const Device UA2021456 = {S101526, 128, { &P20497,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; T8 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023616 = {S101527, 150, { &P17951,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Milestone Build/SHOLS_U2_05.26.3) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2520820 = {S101528, 167, { &P18661,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; N860 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2026825 = {S101529, 171, { &P17324,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2027943 = {S101530, 171, { &P17324,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2027503 = {S101531, 150, { &P13591,&P5096,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; ZTE-BLADE Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028403 = {S101532, 142, { &P12496,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I897 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA2028725 = {S101533, 166, { &P24555,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; MTS-SP100 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2032341 = {S101534, 26, { &P17172,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (TF101 IML74K)*/
const Device UA2144390 = {S101535, 66, { &P12964,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux; U;2.3.4;zh_cn;HTC S710d;480*800;)UCWEB7.6.0.75/139/7701*/
const Device UA2148631 = {S101536, 63, { &P15575,&P18110,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.5;Zh_cn;GT-I9100;480*800;)UCWEB7.8.1.96/139/800*/
const Device UA2167638 = {S101537, 83, { &P13443,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; SamsungI8910)/UC Browser8.5.0.183/50/352/UCWEB Mobile*/
const Device UA2208211 = {S101538, 62, { &P15364,&P17188,&P18269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 4.0; U; en) Presto/2.6.30 Version/10.63*/
const Device UA2258284 = {S101539, 124, { &P17595,&P17295,&P17237,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; en-US) AppleWebKit/530.10 (KHTML, like Gecko) Chrome/2.0.181 Safari/530.10*/
const Device UA2319495 = {S101540, 80, { &P11880,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-in; NOKIA5320)/UC Browser8.4.0.159/28/352/UCWEB Mobile*/
const Device UA2337528 = {S101541, 79, { &P15791,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; Nokia603)/UC Browser8.5.0.183/50/352/UCWEB Mobile*/
const Device UA2339055 = {S101542, 171, { &P21988,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us ; PD_Novel Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/405*/
const Device UA2371083 = {S101543, 126, { &P23662,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL32EX721; CC/ESP; en) Presto/2.7.61 Version/11.00*/
const Device UA2588304 = {S101544, 79, { &P11964,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; kau; NokiaE75-1)/UC Browser8.4.0.159/28/352/UCWEB Mobile*/
const Device UA2588238 = {S101545, 75, { &P17595,&P18577,&P18537,&P18092 }};	/*Useragent - Reeder/1010.79.00 CFNetwork/454.12.4 Darwin/10.8.0 (i386) (MacBookPro5%2C5)*/
const Device UA2413040 = {S101546, 83, { &P13800,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA5630d-1)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2447611 = {S101547, 149, { &P23937,&P18066,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Y2 Build/HCL ME Tablet Y2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2494709 = {S101548, 127, { &P21212,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; N90 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2518656 = {S101549, 49, { &P14358,&P5890,&P20776,&P18092 }};	/*Useragent - MQQBrowser/2.5 (E63-1;SymbianOS/9.2 Series60/3.1)*/
const Device UA2543003 = {S101550, 142, { &P14693,&P5843,&P24668,&P18092 }};	/*Useragent - NokiaC1-02/2.0 (03.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac1-02) UCBrowser8.3.0.154/70/444/UCWEB Mobile*/
const Device UA2549234 = {S101551, 169, { &P12280,&P17084,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/5.1 Mobile/8B117 Safari/7534.48.3 UCBrowser8.4.0.159/41/355*/
const Device UA2748427 = {S101552, 97, { &P14851,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; vi; samsung-gt-s5753e) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2609450 = {S101553, 30, { &P12879,&P17979,&P20428,&P18092 }};	/*Useragent - Android OS/12 samsung_GT-P7510*/
const Device UA2609417 = {S101554, 171, { &P13452,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.2.139/145/355*/
const Device UA2633091 = {S101555, 175, { &P13452,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-US; GT-P1000 Build/GINGERBREAD) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2615377 = {S101556, 163, { &P12889,&P6546,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7300 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2614375 = {S101557, 99, { &P15919,&P17979,&P22070,&P18092 }};	/*Useragent - Instagram 3.0.5 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P3100; espressorf; espresso; en_GB)*/
const Device UA2631332 = {S101558, 162, { &P12879,&P6437,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7510, 40, 1.8.5)*/
const Device UA2616265 = {S101559, 166, { &P15882,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P5110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2626938 = {S101560, 166, { &P15882,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P5110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2618968 = {S101561, 38, { &P22043,&P17979,&P18062,&P18092 }};	/*Useragent - Android/4.0.3; samsung GT-P5113; 2.3.1*/
const Device UA2625338 = {S101562, 67, { &P22043,&P17979,&P18062,&P18092 }};	/*Useragent - eHarmony-Android/2.1.7 (GT-P5113; Android OS 4.0.4; en_US; id null)*/
const Device UA2616172 = {S101563, 198, { &P13452,&P18149,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P1000 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:34, es*/
const Device UA2630959 = {S101564, 204, { &P13452,&P18149,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pt-br; GT-P1000L Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/292,1.9.23.292,2013-02-06 08:41, -0300*/
const Device UA2636606 = {S101565, 218, { &P13452,&P18149,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P1000 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/285,1.9.19.285,2013-01-24 11:19, +0800, china, cn*/
const Device UA2633420 = {S101566, 190, { &P14908,&P6437,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P6210 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, -0500, ca*/
const Device UA2619498 = {S101567, 172, { &P13452,&P5629,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1000, 36, 1.7.1)*/
const Device UA2622422 = {S101568, 174, { &P13452,&P5629,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; es-es; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1000, 36, 1.7.1)*/
const Device UA2629000 = {S101569, 176, { &P13452,&P5629,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; pt-br; GT-P1000L Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1000L, 39, 1.8.2)*/
const Device UA2630478 = {S101570, 176, { &P13452,&P5629,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-ca; GT-P1000M Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1000M, 39, 1.8.2)*/
const Device UA2630764 = {S101571, 174, { &P13452,&P5629,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; el-gr; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1000, 39, 1.8.2)*/
const Device UA2635631 = {S101572, 174, { &P13452,&P5629,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1000, 40, 1.8.5)*/
const Device UA2635985 = {S101573, 176, { &P13452,&P5629,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000N Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1000N, 41, 1.9.0)*/
const Device UA2620582 = {S101574, 38, { &P15876,&P17979,&P18062,&P18092 }};	/*Useragent - Android/4.0.3; samsung GT-P5100; 2.3.2*/
const Device UA2623053 = {S101575, 147, { &P22037,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2623829 = {S101576, 132, { &P22041,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P6810 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2630154 = {S101577, 164, { &P17323,&P6437,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6800 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2628913 = {S101578, 166, { &P12879,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P7510 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2630714 = {S101579, 58, { &P17337,&P17979,&P22278,&P18092 }};	/*Useragent - theScore/3.1.1 Android-OS/3.1 (Android; Tablet, GT-P7500D)*/
const Device UA2648621 = {S101580, 157, { &P20452,&P18066,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; Lenovo S880 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2680986 = {S101581, 138, { &P22382,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2685280 = {S101582, 151, { &P13006,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; folio100 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2721924 = {S101583, 168, { &P12496,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I897 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2788041 = {S101584, 152, { &P18740,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOU701 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2804272 = {S101585, 137, { &P14225,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; Desire HD Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2810173 = {S101586, 67, { &P12955,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.7; zh-cn; Blade; 480*800) UCWEB7.9.4.145/139/800*/
const Device UA2811676 = {S101587, 151, { &P14211,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-P100 Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2811772 = {S101588, 67, { &P12848,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.6; zh-cn; MB526; 480*854) UCWEB7.9.4.145/139/800*/
const Device UA2816540 = {S101589, 153, { &P15397,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; ST18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2817149 = {S101590, 26, { &P22043,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-P5113/100.40004*/
const Device UA2817484 = {S101591, 152, { &P18743,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; OPPOR805 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2817918 = {S101592, 162, { &P14225,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2823758 = {S101593, 131, { &P21072,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; DIT1010 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2823764 = {S101594, 46, { &P12137,&P17484,&P21759,&P18092 }};	/*Useragent - iBrowser/Mini2.8 (SAMSUNG-GT-S5620/S5620DDKC1)*/
const Device UA2830147 = {S101595, 140, { &P12280,&P18432,&P20787,&P18092 }};	/*Useragent - MQQBrowser/36 Mozilla/5.0 (iPhone 4S; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3*/
const Device UA2889021 = {S101596, 253, { &P14862,&P5204,&P18516,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.3.0.154/70/355*/
const Device UA2952861 = {S101597, 92, { &P12599,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-lg800g) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2977110 = {S101598, 136, { &P24064,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-S7275 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2977115 = {S101599, 129, { &P24064,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-S7275 Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA3092617 = {S101600, 156, { &P11793,&P5768,&P24679,&P18092 }};	/*Useragent - Nokia3500c/2.0 (05.51) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3500c) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3079629 = {S101601, 160, { &P11808,&P5204,&P24679,&P18092 }};	/*Useragent - Nokia5610d-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5610d-1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA62912 = {S101602, 103, { &P11881,&P17486,&P17671,&P18092 }};	/*Useragent - MOT-EM325/1.0 Release/12.20.2008 Browser/CMCS1.0 Software/1.300 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2480 = {S101603, 128, { &P11980,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320; HTC_P3300/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2902 = {S101604, 92, { &P13241,&P5033,&P4948,&P18092 }};	/*Useragent - BlackBerry8830/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105 UP.Browser/5.0.3.3*/
const Device UA4547 = {S101605, 140, { &P17058,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-SGH-I907/UCJD1 Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA10095 = {S101606, 120, { &P12134,&P17017,&P3976,&P18092 }};	/*Useragent - FarEasTone/1.1/SharpWXT923/SHS001 Mozilla/5.00 (compatible) Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA25572 = {S101607, 119, { &P12134,&P17017,&P3976,&P18092 }};	/*Useragent - FarEasTone/1.0/SharpWXT92/SHS001 Mozilla/4.08 (compatible) Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA13900 = {S101608, 41, { &P19971,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonT700/R3EA SEMC/NewsReader/1.0*/
const Device UA15265 = {S101609, 130, { &P12012,&P17017,&P4893,&P18092 }};	/*Useragent - SIE-EL71/31 UP.Browser/7.1.0.e.24(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.e.24 (GUI) MMP/2.0*/
const Device UA26742 = {S101610, 128, { &P12012,&P17017,&P4893,&P18092 }};	/*Useragent - SIE-EL71/41 UP.Browser/7.1.0.k.6(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.k.6 (GUI) MMP/2.0*/
const Device UA44268 = {S101611, 85, { &P12012,&P17017,&P4893,&P18092 }};	/*Useragent - SIE-EL71/41 UP.Browser/7.1.0.k.6(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54229 = {S101612, 130, { &P12012,&P17017,&P4893,&P18092 }};	/*Useragent - SIE-EL71/12 UP.Browser/7.1.0.e.18(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.e.18 (GUI) MMP/2.0*/
const Device UA54394 = {S101613, 130, { &P12012,&P17017,&P4893,&P18092 }};	/*Useragent - SIE-EL71/22 UP.Browser/7.1.0.e.23(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.e.23 (GUI) MMP/2.0*/
const Device UA18521 = {S101614, 66, { &P12986,&P17017,&P4528,&P18092 }};	/*Useragent - Nexian NX-G505/MTK Release/10.01.2009 Browser/MAUI Profile/No Java*/
const Device UA59154 = {S101615, 162, { &P15150,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaC5-01_TD/CMCC SymbianOS/9.3 Series60/3.2 Release/20.104603; Mozilla/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML,like Gecko) Safari/413*/
const Device UA21042 = {S101616, 83, { &P11969,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonW908c/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA21190 = {S101617, 30, { &P17111,&P17017,&P3807,&P18092 }};	/*Useragent - Indie/Indie_B07A102 Obigo/Q03C*/
const Device UA24402 = {S101618, 33, { &P17111,&P17017,&P3807,&P18092 }};	/*Useragent - Indie/Indie_SF_B07A101 Obigo/Q03C*/
const Device UA21373 = {S101619, 141, { &P14376,&P17486,&P3581,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-AURA/R6639_G_81.20.5FR_A BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony 1.0*/
const Device UA21374 = {S101620, 139, { &P14376,&P17486,&P3581,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-AURA/R6639_G_81.21.06R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony %bv*/
const Device UA62162 = {S101621, 139, { &P14376,&P17486,&P3581,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; OSS/1.0; Chameleon; Linux) MOT-AURA/R6639_G_81.20.75R BER/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony 1.0*/
const Device UA21656 = {S101622, 39, { &P15316,&P6476,&P4970,&P18092 }};	/*Useragent - Samsung-SPHM300 AU-MIC-M300/2.0 MMP/2.0*/
const Device UA88760 = {S101623, 34, { &P15316,&P6476,&P4970,&P18092 }};	/*Useragent - Samsung-SPHM300 AU-MIC/2.0 MMP/2.0*/
const Device UA21734 = {S101624, 77, { &P16253,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) O2 Xda Atom Life*/
const Device UA78600 = {S101625, 74, { &P16253,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) MWG Atom Life*/
const Device UA22257 = {S101626, 20, { &P15413,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-X480/1.0*/
const Device UA82116 = {S101627, 21, { &P15413,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-X480C/1.0*/
const Device UA69547 = {S101628, 81, { &P14336,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK300c/R2BA SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22632 = {S101629, 61, { &P15506,&P19945,&P17470,&P18092 }};	/*Useragent - Panasonic-VS6/1.0/RP1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23584 = {S101630, 47, { &P17560,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-VX8800/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA23665 = {S101631, 43, { &P15617,&P17017,&P3807,&P18092 }};	/*Useragent - Huawei/1.0/G5730/Browser/obigo-browser/q03c*/
const Device UA24336 = {S101632, 70, { &P12097,&P5448,&P22271,&P18092 }};	/*Useragent - HTC_P4350 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA24343 = {S101633, 84, { &P12098,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320;MWg Atom V*/
const Device UA24607 = {S101634, 47, { &P15718,&P17484,&P5001,&P18092 }};	/*Useragent - SCH-U810/1.0 NetFront/3.0.22.1.14 (GUI) MMP/2.0*/
const Device UA25761 = {S101635, 92, { &P12138,&P5027,&P4948,&P18092 }};	/*Useragent - BlackBerry8130/4.3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/126 UP.Browser/5.0.3.3*/
const Device UA63426 = {S101636, 68, { &P13580,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V975/82.31.56I MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA26644 = {S101637, 128, { &P12530,&P5448,&P22272,&P18092 }};	/*Useragent - PPC; 240x320; HTC_P3450/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA28373 = {S101638, 112, { &P12228,&P17017,&P3976,&P18092 }};	/*Useragent - China Unicom/1.1/SH0902C/SHS001 Mozilla/5.00 (compatible) Browser/NF/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA31507 = {S101639, 117, { &P12429,&P17483,&P3534,&P18092 }};	/*Useragent - LG-KF757-Orange/v10d Browser/Obigo-Q05A/3.12MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA32837 = {S101640, 68, { &P17159,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHL320/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA69391 = {S101641, 97, { &P17475,&P5768,&P4779,&P18092 }};	/*Useragent - Nokia6300/2.0 (07.00) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6300/UC Browser7.8.0.95/70/401*/
const Device UA40188 = {S101642, 43, { &P16451,&P17017,&P17470,&P18092 }};	/*Useragent - Huawei/1.0/U1000/B000 Browser/Obigo-Browser*/
const Device UA43778 = {S101643, 61, { &P13741,&P17486,&P17470,&P18092 }};	/*Useragent - MOT-W450/0.1.72.K1/12.24.2008 Browser/UPB6.3 Software/13.119I*/
const Device UA69398 = {S101644, 99, { &P14826,&P5843,&P4779,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (03.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac1-01/UC Browser7.8.0.95/70/401*/
const Device UA42881 = {S101645, 122, { &P20390,&P22839,&P4223,&P18092 }};	/*Useragent - Zte-tu208_TD/1.0 ThreadX/1708ULC Larena/2.00 Release/9.25.2010 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA44200 = {S101646, 65, { &P16704,&P17483,&P3499,&P18092 }};	/*Useragent - LG-TU330/V10a Teleca/Q7.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87445 = {S101647, 94, { &P13093,&P16000,&P20418,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Motorola A1000;708) Opera 7.50 [en]./A1000./1.0*/
const Device UA49937 = {S101648, 41, { &P13112,&P22832,&P17470,&P18092 }};	/*Useragent - Videocon_V1535/WAP2.0 Profile/MT6253/QCIF*/
const Device UA50136 = {S101649, 36, { &P13177,&P17811,&P17810,&P18092 }};	/*Useragent - LENOVO-A310/(2009.04.17) S010/WAP2.0*/
const Device UA50237 = {S101650, 12, { &P14834,&P5890,&P4030,&P18092 }};	/*Useragent - Nokia5730S-1*/
const Device UA50242 = {S101651, 146, { &P24001,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; zh-tw; TaiwanMobileT1 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA50271 = {S101652, 141, { &P13211,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; eee_701 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA53345 = {S101653, 102, { &P13967,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GM630 Browser/Teleca-Q7.1 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA54297 = {S101654, 12, { &P14177,&P17017,&P17470,&P18092 }};	/*Useragent - i-mobileS281*/
const Device UA54592 = {S101655, 56, { &P13477,&P19950,&P17470,&P18092 }};	/*Useragent - SAGEM-my750X/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54994 = {S101656, 70, { &P12071,&P17017,&P17470,&P18092 }};	/*Useragent - W9229ObigolnternetBrowser/QO3C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA56142 = {S101657, 33, { &P14648,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 SO903i(c100;TB;W24H18)*/
const Device UA56166 = {S101658, 78, { &P14266,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-M2710/M2710DDIF4 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA58630 = {S101659, 102, { &P12531,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-J700L/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA62403 = {S101660, 47, { &P14407,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-VX9400/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA62655 = {S101661, 47, { &P14435,&P6476,&P3625,&P18092 }};	/*Useragent - LGE-VX7100/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA63089 = {S101662, 105, { &P12132,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonJ10i2/R7BA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2*/
const Device UA63494 = {S101663, 98, { &P17113,&P19944,&P3625,&P18092 }};	/*Useragent - SHARP-TQ-GX30/1.0 Profile/MIDP-1.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.6.c.1.104 (GUI) MMP/1.0*/
const Device UA84692 = {S101664, 99, { &P17113,&P19944,&P3625,&P18092 }};	/*Useragent - SHARP-TQ-GX30i/1.0 Profile/MIDP-1.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.6.c.1.104 (GUI) MMP/1.0*/
const Device UA67305 = {S101665, 34, { &P13163,&P5129,&P4715,&P18092 }};	/*Useragent - NokiaN97/UC Browser7.4.0.65/50/351*/
const Device UA67095 = {S101666, 116, { &P11658,&P5776,&P4715,&P18092 }};	/*Useragent - Nokia6680/1.0 (5.04.07) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.4.0.65/27/352*/
const Device UA538836 = {S101667, 99, { &P14669,&P5843,&P4759,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (04.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-00/UC Browser7.6.1.82/69/352*/
const Device UA68438 = {S101668, 103, { &P14862,&P5204,&P4777,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2700c-2/UC Browser7.7.1.88/70/352*/
const Device UA68928 = {S101669, 35, { &P11880,&P5890,&P4777,&P18092 }};	/*Useragent - NOKIA5320/UC Browser7.7.1.88/28/444*/
const Device UA69477 = {S101670, 35, { &P13706,&P5129,&P4779,&P18092 }};	/*Useragent - Nokia5235/UC Browser7.8.0.95/50/355*/
const Device UA79879 = {S101671, 89, { &P17595,&P17296,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_6) AppleWebKit (KHTML, like Gecko) Chrome/1*/
const Device UA2317538 = {S101672, 118, { &P17595,&P17296,&P17232,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Chrome/1.0 Safari/533.3*/
const Device UA86795 = {S101673, 42, { &P12280,&P17779,&P18215,&P18092 }};	/*Useragent - Apple iPhone v1.1.4 CoreMedia v1.0.0.4A102*/
const Device UA88545 = {S101674, 40, { &P12280,&P17779,&P18215,&P18092 }};	/*Useragent - Apple iPhone v1.1.5 CoreMedia v1.0.0.4B1*/
const Device UA86870 = {S101675, 87, { &P13838,&P16000,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Nokia 6600/5.53.0; 1657) Opera 8.60 [en]*/
const Device UA88375 = {S101676, 87, { &P13838,&P16000,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Nokia 6600/5.53.0; 9399) Opera 8.65 [en]*/
const Device UA87612 = {S101677, 47, { &P17857,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-PT34 UP.Browser/6.2.0.14.3.1 (GUI) MMP/2.0*/
const Device UA87779 = {S101678, 10, { &P16995,&P22835,&P17470,&P18092 }};	/*Useragent - Spice S580*/
const Device UA89799 = {S101679, 88, { &P17904,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-T780/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA194391 = {S101680, 16, { &P15655,&P17484,&P17606,&P18092 }};	/*Useragent - samsung-gt-s3850*/
const Device UA2304047 = {S101681, 27, { &P15655,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-S3850/S3850ZCKE1*/
const Device UA253939 = {S101682, 43, { &P13206,&P17485,&P4777,&P18092 }};	/*Useragent - sonyericssonk530i/UC Browser7.7.1.88/69/444*/
const Device UA369427 = {S101683, 28, { &P12496,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (SGH-I897 FROYO)*/
const Device UA369455 = {S101684, 180, { &P22214,&P5629,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Spice Mi350n Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; huaqin73_gb Build/ALPS.GB.FDD2.MP.V2.14) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370129 = {S101685, 133, { &P22235,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; PC7251 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA437970 = {S101686, 53, { &P14188,&P5890,&P20777,&P18092 }};	/*Useragent - MQQBrowser/2.6 (Nokia5233;SymbianOS/9.1 Series60/3.0)*/
const Device UA439776 = {S101687, 95, { &P20356,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; ASUS; E600)*/
const Device UA483595 = {S101688, 94, { &P15364,&P17816,&P17260,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.7) Gecko/X Ubuntu/10.04 (lucid) Firefox/3.5.7*/
const Device UA539989 = {S101689, 56, { &P15364,&P23114,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; Warp 4.5; rv:2.0) Gecko/X Firefox/4.0*/
const Device UA553120 = {S101690, 63, { &P11747,&P18110,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;2.2;Zh_cn;HTC Desire;480*800;)UCWEB7.8.1.96/139/800*/
const Device UA653217 = {S101691, 63, { &P11747,&P18110,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;2.2;Zh_cn;HTC Desire;480*800;)UCWEB7.8.1.96/139/999*/
const Device UA559750 = {S101692, 99, { &P12897,&P5843,&P4777,&P18092 }};	/*Useragent - NokiaC2-00/2.0 (03.42) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-00/UC Browser7.7.1.88/69/444*/
const Device UA565680 = {S101693, 83, { &P15364,&P5448,&P15761,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows CE 5.2; en-US; rv:1.9.2a1pre) Gecko/X Fennec/1.0a1*/
const Device UA574055 = {S101694, 42, { &P11880,&P5890,&P24663,&P18092 }};	/*Useragent - NOKIA5320/UC Browser7.9.0.102/28/355/UCWEB*/
const Device UA583795 = {S101695, 32, { &P15364,&P17017,&P24665,&P18092 }};	/*Useragent - UC Browser8.0.4.121/28/351/UCWEB*/
const Device UA756100 = {S101696, 32, { &P15364,&P17017,&P24665,&P18092 }};	/*Useragent - UC Browser8.0.4.121/28/352/UCWEB*/
const Device UA1989910 = {S101697, 32, { &P15364,&P17017,&P24665,&P18092 }};	/*Useragent - UC Browser8.0.4.121/50/352/UCWEB*/
const Device UA597628 = {S101698, 123, { &P15364,&P16110,&P22423,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.4.1.0 Chrome/17.0.963.56 Safari/535.11*/
const Device UA1972710 = {S101699, 130, { &P15364,&P16110,&P22423,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.4.1.0 Chrome/17.0.963.56 Safari/535.11*/
const Device UA599781 = {S101700, 112, { &P15767,&P18117,&P22246,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Flipboard/1.7.2*/
const Device UA2154308 = {S101701, 114, { &P15767,&P18117,&P22246,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Flipboard/1.7.2*/
const Device UA616106 = {S101702, 139, { &P17595,&P17297,&P24196,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.2.0.1563 Safari/535.11*/
const Device UA813304 = {S101703, 139, { &P17595,&P17297,&P24196,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 QQBrowser/1.2.0.1563 Safari/535.11*/
const Device UA616325 = {S101704, 87, { &P21690,&P17017,&P4780,&P18092 }};	/*Useragent - Nexian NX-T910/MTK Release/10.01.2009 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1*/
const Device UA766479 = {S101705, 34, { &P15364,&P5890,&P4722,&P18092 }};	/*Useragent - NOKIAN73/UC Browser7.0.0.41/28/352*/
const Device UA785996 = {S101706, 137, { &P16362,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SPH-D710 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA791311 = {S101707, 150, { &P14923,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; TeamDRH ICS for GTablet Build/IML77) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA881271 = {S101708, 153, { &P13077,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HW-HUAWEI_C8650/C8650V100R001C92B825; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2028517 = {S101709, 166, { &P13077,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8650+ Build/V100R001C92B861) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA899034 = {S101710, 155, { &P13123,&P5629,&P21753,&P18092 }};	/*Useragent - iBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA899181 = {S101711, 46, { &P18254,&P18570,&P18245,&P18091 }};	/*Useragent - Tweetbot/24000 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2339927 = {S101712, 46, { &P18254,&P18570,&P18245,&P18091 }};	/*Useragent - Tweetbot/24203 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA2349418 = {S101713, 46, { &P18254,&P18570,&P18245,&P18091 }};	/*Useragent - Tweetbot/24300 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA925731 = {S101714, 103, { &P13623,&P5222,&P17183,&P18092 }};	/*Useragent - HTC_Snap_S523/V2.03.841.1 (65854) Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5)*/
const Device UA1866350 = {S101715, 94, { &P14949,&P5690,&P17215,&P18092 }};	/*Useragent - Opera/9.50 (Windows NT 5.1; MDA_compact_IV/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1; U; en)*/
const Device UA1899617 = {S101716, 104, { &P15364,&P18144,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD i386) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30*/
const Device UA2141048 = {S101717, 105, { &P15364,&P18144,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; OpenBSD amd64) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30*/
const Device UA1942921 = {S101718, 42, { &P14782,&P5890,&P24665,&P18092 }};	/*Useragent - NOKIAE61i/UC Browser8.0.4.121/28/352/UCWEB*/
const Device UA1946415 = {S101719, 134, { &P16950,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Dell Streak 7 Build/MR1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA1998153 = {S101720, 32, { &P15364,&P17017,&P21912,&P18092 }};	/*Useragent - contype chromeframe/21.0.1180.60*/
const Device UA2194422 = {S101721, 32, { &P15364,&P17017,&P21912,&P18092 }};	/*Useragent - contype chromeframe/21.0.1180.89*/
const Device UA2352045 = {S101722, 32, { &P15364,&P17017,&P21912,&P18092 }};	/*Useragent - contype chromeframe/21.0.1180.83*/
const Device UA2018187 = {S101723, 148, { &P21688,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; th-th; i-mobile 3G 8500 Build/DRD35) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA2024502 = {S101724, 85, { &P17968,&P5062,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android 1.6; Linux; Opera Mobi/ADR-X; U; pl) Presto/2.9.201 Version/11.50*/
const Device UA2025479 = {S101725, 137, { &P16044,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Touchpad Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026897 = {S101726, 157, { &P16103,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; XT720 Build/GWK74; Bravo2.3_V1.0-brianlee) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028032 = {S101727, 149, { &P14186,&P5096,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028173 = {S101728, 138, { &P13633,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MotoA953 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2028978 = {S101729, 155, { &P18304,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-I9070 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030348 = {S101730, 154, { &P12945,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; Lenovo A780 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2340607 = {S101731, 168, { &P12824,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; U8180 Build/HuaweiU8180) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2146295 = {S101732, 68, { &P20144,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2; zh-cn; GT-I5508; 240*320) UCWEB7.9.3.103/140/800*/
const Device UA2199028 = {S101733, 138, { &P19922,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; SGH-I535 Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2203515 = {S101734, 139, { &P19922,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I535 Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2307896 = {S101735, 126, { &P23652,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL46HX729; CC/USA; en) Presto/2.7.61 Version/11.00*/
const Device UA2331290 = {S101736, 54, { &P14033,&P5890,&P20780,&P18092 }};	/*Useragent - MQQBrowser/2.9 (NokiaC5-03;SymbianOS/9.1 Series60/3.0)*/
const Device UA2339683 = {S101737, 155, { &P13452,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2616887 = {S101738, 153, { &P13452,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.3; en-; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2375437 = {S101739, 251, { &P14669,&P5843,&P18524,&P18092 }};	/*Useragent - NokiaX2-00/5.0 (08.35) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/70/352*/
const Device UA2417802 = {S101740, 147, { &P13644,&P24693,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; PC36100 Build/black bean AOKP) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2519009 = {S101741, 139, { &P21877,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; TAB-PROTAB2.4XL Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2546734 = {S101742, 142, { &P22161,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; iberry AUXUS AX03G Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2580417 = {S101743, 91, { &P13365,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gu220) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2591078 = {S101744, 103, { &P14188,&P5129,&P21759,&P18092 }};	/*Useragent - iBrowser/Mini2.8 (Nokia5233/51.1.002/sw_platform=S60;sw_platform_version=5.0;java_build_version=1.4.43)*/
const Device UA2608443 = {S101745, 139, { &P15919,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P3100 Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617252 = {S101746, 147, { &P15919,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Mobile Maxthon*/
const Device UA2624719 = {S101747, 139, { &P15919,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3100 Build/IMM76D)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2619349 = {S101748, 90, { &P12879,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Mail; 1.4.6) (GT-P7510; samsung; GT-P7510; 4.0.4/IMM76D)*/
const Device UA2614334 = {S101749, 91, { &P17337,&P6437,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.4.5.12736); (Linux; U; Android 3.2; GT-P7500 Build/HONEYCOMB_MR2);*/
const Device UA2618705 = {S101750, 92, { &P17337,&P6437,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 3.2; GT-P7500R Build/HONEYCOMB_MR2);*/
const Device UA2616215 = {S101751, 146, { &P15919,&P18066,&P20775,&P18092 }};	/*Useragent - MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2620267 = {S101752, 146, { &P15919,&P18066,&P20775,&P18092 }};	/*Useragent - MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 4.0.3; en-za; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2621148 = {S101753, 175, { &P15876,&P18066,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5100 Build/IML74K; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444*/
const Device UA2625390 = {S101754, 175, { &P15876,&P18066,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5100 Build/IMM76D; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/352*/
const Device UA2620973 = {S101755, 99, { &P15919,&P17979,&P22069,&P18092 }};	/*Useragent - Instagram 1.1.8 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P3100; espressorf; espresso; en_US)*/
const Device UA2619640 = {S101756, 99, { &P15876,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Messenger; 1.8.3) (espresso10rf; samsung; GT-P5100; 4.0.3/IML74K)*/
const Device UA2633596 = {S101757, 30, { &P22043,&P17979,&P20428,&P18092 }};	/*Useragent - Android OS/16 samsung_GT-P5113*/
const Device UA2620431 = {S101758, 144, { &P17337,&P6546,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2625118 = {S101759, 164, { &P14908,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P6210, 39, 1.8.2)*/
const Device UA2621472 = {S101760, 166, { &P15932,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2624064 = {S101761, 166, { &P15932,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P3110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2626563 = {S101762, 192, { &P22052,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7501 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +0100, de*/
const Device UA2624880 = {S101763, 165, { &P14908,&P5096,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2625012 = {S101764, 165, { &P14908,&P5096,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2630525 = {S101765, 192, { &P12889,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P7300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0100, fr*/
const Device UA2627028 = {S101766, 156, { &P13452,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2627043 = {S101767, 156, { &P13452,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2627355 = {S101768, 156, { &P13452,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2630740 = {S101769, 150, { &P13452,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-P1000 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2631115 = {S101770, 155, { &P13452,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-P1000 Build/GFAN_ROM_X) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2630610 = {S101771, 59, { &P12879,&P17979,&P22278,&P18092 }};	/*Useragent - theScore/3.1.0 Android-OS/4.0.4 (Android; Tablet, GT-P7510)*/
const Device UA2630995 = {S101772, 166, { &P22043,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P5113 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2680998 = {S101773, 157, { &P22385,&P18111,&P22261,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I8262D Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/9.0.0.282 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2684730 = {S101774, 151, { &P15438,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2808029 = {S101775, 115, { &P15929,&P22283,&P4223,&P18092 }};	/*Useragent - MOT-EX232/1.0 REXL4/1.0 BrewMP/1.0.3 Release/6.15.2011 Browser/NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2811195 = {S101776, 158, { &P12493,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ru-ru; U8220/U8230 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2815228 = {S101777, 142, { &P15541,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Xperia Play Build/IMM76I) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2815769 = {S101778, 161, { &P17324,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; de-de; GT-I9001 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2816698 = {S101779, 129, { &P21281,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MP959 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2817171 = {S101780, 86, { &P11941,&P17485,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; sonyericssonw580i) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2817402 = {S101781, 173, { &P20370,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; ZTE-C_N760/N760V1.0.0B04; 320*480; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2817685 = {S101782, 158, { &P20537,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; YL-Coolpad_7260 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2824613 = {S101783, 77, { &P12798,&P18110,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Adr (Linux; U; 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.62;480*854)*/
const Device UA2831680 = {S101784, 187, { &P19970,&P17485,&P24671,&P18092 }};	/*Useragent - SonyEricssonG502/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; sonyericssong502) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/83/352/UCWEB Mobile*/
const Device UA2835496 = {S101785, 65, { &P12280,&P17146,&P22070,&P18092 }};	/*Useragent - Instagram 3.0.0 (iPhone; iPhone OS 5.0.1; en_US) AppleWebKit/420+*/
const Device UA2848934 = {S101786, 90, { &P16024,&P17483,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-t310i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA3032064 = {S101787, 93, { &P12456,&P5890,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian73-1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3068893 = {S101788, 125, { &P15767,&P24384,&P24402,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.80 (KHTML, like Gecko) Version/7.1 Mobile/11A4388 Safari/8536.25*/
const Device UA3503 = {S101789, 143, { &P18007,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-tw; CHT8000 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA10341 = {S101790, 84, { &P13496,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Samsung-SPHI325 320x240*/
const Device UA15915 = {S101791, 76, { &P11844,&P17484,&P4924,&P18092 }};	/*Useragent - SAMSUNG-SGH-D528/NetFront 3.2/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA48946 = {S101792, 125, { &P12594,&P5208,&P4902,&P18092 }};	/*Useragent - Mozilla/5.0 (SCH-F839/F839DH11;U;REX/3.0;BREW/3.1.5;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*400;CTC/2.0) POLARIS/6.15.WEB*/
const Device UA53685 = {S101793, 68, { &P14006,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V180/0B.D2.2ER MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA82915 = {S101794, 71, { &P14006,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V180ENS/0B.D1.32R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA19964 = {S101795, 79, { &P15016,&P17486,&P3402,&P18092 }};	/*Useragent - MOT-EM28/0.0.62 UP.Browser/6.3.0.6.c.28 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20408 = {S101796, 77, { &P15085,&P22839,&P17470,&P18092 }};	/*Useragent - ZTE-F951-Orange/1.0 ACS-NF/3.5 QTV5.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49061 = {S101797, 62, { &P13893,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia3100b/1.0 (05.01) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA21354 = {S101798, 100, { &P15564,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Vodafone/1.0/SamsungSGHI620V/I620BUGG3*/
const Device UA87030 = {S101799, 77, { &P15564,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) SEC-SGHI620/1.0*/
const Device UA87451 = {S101800, 100, { &P15564,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Vodafone/1.0/SamsungSGHI620V/I620BUGFA*/
const Device UA21535 = {S101801, 107, { &P15304,&P17483,&P3534,&P18092 }};	/*Useragent - LG-KF700Q Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA22173 = {S101802, 13, { &P15392,&P17483,&P17660,&P18092 }};	/*Useragent - LG/U8380/v1.0*/
const Device UA22204 = {S101803, 23, { &P15398,&P19954,&P3816,&P18092 }};	/*Useragent - SEC-SGHX630/1.0 TSS/2.5*/
const Device UA22595 = {S101804, 23, { &P15490,&P19954,&P3816,&P18092 }};	/*Useragent - SEC-SGHD407/1.0 TSS/2.5*/
const Device UA22612 = {S101805, 82, { &P16177,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_Touch_Pro2_T7373 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA27299 = {S101806, 74, { &P16177,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Sprint T7380*/
const Device UA24369 = {S101807, 37, { &P12654,&P17483,&P17470,&P18092 }};	/*Useragent - LG-KG220 MIC/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA32658 = {S101808, 186, { &P12512,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/5.0 (ZTE-E_N72/N72V1.0.0B02;U;Windows Mobile/6.1;Profile/MIDP-2.0 Configuration/CLDC-1.1;320*240;CTC/2.0) IE/6.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA33276 = {S101809, 101, { &P13056,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-P200/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA49545 = {S101810, 108, { &P13056,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-P200-ORANGE/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA44042 = {S101811, 12, { &P13773,&P17483,&P17660,&P18092 }};	/*Useragent - LG/U370/v1.0*/
const Device UA49428 = {S101812, 101, { &P13022,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-X576/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA50074 = {S101813, 141, { &P17192,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; Archos5 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA54534 = {S101814, 98, { &P13311,&P19944,&P3625,&P18092 }};	/*Useragent - SHARP-TQ-GX25/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.3.3.d.1.109 (GUI) MMP/2.0*/
const Device UA56019 = {S101815, 33, { &P14616,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 N902iS(c100;TB;W24H12)*/
const Device UA60812 = {S101816, 111, { &P15198,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S3650H/1.0 SHP/VPP/R5 NetFront/3.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA81517 = {S101817, 41, { &P12687,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-A950 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA63718 = {S101818, 101, { &P14535,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-X490/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA64539 = {S101819, 81, { &P14680,&P17017,&P3601,&P18092 }};	/*Useragent - New York/R3.0 NF-Browser/3.3 Openwave-MMS/4.5 IA-Java/1.0 Profile/MIDP2.0/CLDC1.1*/
const Device UA67244 = {S101820, 74, { &P11903,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-C3212/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA67251 = {S101821, 85, { &P13999,&P5448,&P4715,&P18092 }};	/*Useragent - HTC_HD2_T8585 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65*/
const Device UA67435 = {S101822, 43, { &P16638,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KF301/V01 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA67437 = {S101823, 142, { &P15699,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; HTC Incredible Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA68605 = {S101824, 139, { &P12530,&P5448,&P4759,&P18092 }};	/*Useragent - PPC; 240x320; HTC_P3450/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.6.1.82*/
const Device UA69243 = {S101825, 116, { &P11860,&P5776,&P4779,&P18092 }};	/*Useragent - Nokia6681/2.0 (6.09.00) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/27/351*/
const Device UA86938 = {S101826, 142, { &P11842,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungSGHM300V/M300BUGG2/Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA318049 = {S101827, 142, { &P11842,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungSGHM300V/M300JBGF3/Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA87664 = {S101828, 86, { &P17791,&P17662,&P3976,&P18092 }};	/*Useragent - SoftBank/1.0/922SH/SHJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87672 = {S101829, 41, { &P17791,&P17662,&P3976,&P18092 }};	/*Useragent - Mozilla/5.0 (922SH;SoftBank) NetFront/3.4*/
const Device UA87777 = {S101830, 37, { &P15245,&P17483,&P3534,&P18092 }};	/*Useragent - LGE-LG280 AU-OBIGO/Q05A1-3.12 MMP/2.0*/
const Device UA88733 = {S101831, 79, { &P12652,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; Neonode N2; 176x220)*/
const Device UA88746 = {S101832, 44, { &P12682,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_P3490 Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA89760 = {S101833, 45, { &P17547,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC3U UP.Browser/6.2.0.15.2 (GUI) MMP/2.0*/
const Device UA157160 = {S101834, 110, { &P15364,&P18145,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; FreeBSD i386; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA2260242 = {S101835, 111, { &P15364,&P18145,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; FreeBSD amd64; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA179050 = {S101836, 124, { &P15364,&P18110,&P22447,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Iron/9.0.600.2 Chrome/9.0.600.2 Safari/534.13*/
const Device UA207014 = {S101837, 80, { &P15364,&P5690,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; Opera Mobi/14316; U; en) Presto/2.7.81 Version/11.00*/
const Device UA662271 = {S101838, 99, { &P15228,&P5043,&P4779,&P18092 }};	/*Useragent - NokiaC2-03/2.0 (06.51) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-03/UC Browser7.8.0.95/70/355*/
const Device UA251626 = {S101839, 123, { &P15364,&P5690,&P24206,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.6.9632.201*/
const Device UA251730 = {S101840, 78, { &P15364,&P5690,&P24206,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) QQBrowser/6.6.9632.201*/
const Device UA465701 = {S101841, 31, { &P18254,&P7480,&P22896,&P18091 }};	/*Useragent - RSSOwl/2.0.5.X (Windows; U; en)*/
const Device UA466749 = {S101842, 82, { &P15364,&P7133,&P22532,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.7) Gecko/X Epic/ Firefox/*/
const Device UA475253 = {S101843, 107, { &P15364,&P17285,&P20805,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) Qt/4.7.0 Safari/533.3*/
const Device UA484741 = {S101844, 89, { &P15364,&P7133,&P21806,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.8pre) Gecko/X Prism zdesktop/2.0*/
const Device UA512811 = {S101845, 97, { &P17595,&P17296,&P21803,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.0.1) Gecko/X Prism (zdesktop/1.0.4)*/
const Device UA557019 = {S101846, 60, { &P13505,&P18110,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;2.2.1;Zh_cn;MB525;480*854;)UCWEB7.8.1.96/139/800*/
const Device UA2139763 = {S101847, 60, { &P13505,&P18110,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;2.2.2;Zh_cn;ME525;480*854;)UCWEB7.8.1.96/139/800*/
const Device UA563610 = {S101848, 103, { &P12125,&P5204,&P4777,&P18092 }};	/*Useragent - Nokia6500s-1/2.0 (06.60) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6500s-1/UC Browser7.7.1.88/69/444*/
const Device UA1953657 = {S101849, 42, { &P12487,&P5890,&P24665,&P18092 }};	/*Useragent - NOKIA5700/UC Browser8.0.4.121/28/444/UCWEB*/
const Device UA616397 = {S101850, 82, { &P17465,&P16110,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.2; Opera Tablet/11648; U; en) Presto/2.7.81 Version/11.00*/
const Device UA616409 = {S101851, 79, { &P18095,&P18094,&P3430,&P18092 }};	/*Useragent - Opera/9.10 (S60; SymbOS; Opera Tablet/9174; U; en-us) Presto/2.6.71 Version/9.5*/
const Device UA616735 = {S101852, 133, { &P18391,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-tw; ASK 105SP Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA651525 = {S101853, 81, { &P17968,&P17979,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android; Linux; Opera Mobi/23731; U; en) Presto/2.9.201 Version/11.50*/
const Device UA754260 = {S101854, 30, { &P14358,&P5890,&P24666,&P18092 }};	/*Useragent - NokiaE63/UCWEB8.1.0.104/28/999*/
const Device UA767295 = {S101855, 39, { &P15795,&P5448,&P20766,&P18092 }};	/*Useragent - NOKIA-Lumia 710/UCWEB2.0.0.149/47/32079*/
const Device UA832269 = {S101856, 144, { &P13054,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; ADR6425LVW Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA845514 = {S101857, 47, { &P15364,&P18570,&P21568,&P18092 }};	/*Useragent - GmailPush/2.0.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA852585 = {S101858, 155, { &P13123,&P5629,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019183 = {S101859, 64, { &P14842,&P5062,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android1.6;Zh_cn;E10i;240*320;)UCWEB7.8.0.95/139/355*/
const Device UA2020509 = {S101860, 133, { &P22563,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; AM7-A1 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021379 = {S101861, 159, { &P13222,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android SDK 8 Froyo (Bionic Build V0.5); en-us; Ally Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021396 = {S101862, 137, { &P21412,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; bq Verne Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022714 = {S101863, 138, { &P21423,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; bq DaVinci Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023148 = {S101864, 146, { &P15364,&P5629,&P20775,&P18092 }};	/*Useragent - MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; H701 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025123 = {S101865, 167, { &P18296,&P18066,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC One V Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2026579 = {S101866, 162, { &P18250,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR MAXX 4G Build/6.5.1-167_DHD-14_M2-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2035262 = {S101867, 159, { &P18250,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR MAXX Build/6.5.1-167_DHD-14_M2-5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2035266 = {S101868, 152, { &P18250,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; DROID RAZR MAXX Build/6.5.1-167_DHD-14_M3-8) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2028823 = {S101869, 154, { &P21945,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; edenTAB ET-701 Build/Gingerbread.GRJ40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2129533 = {S101870, 65, { &P13782,&P17483,&P17602,&P18092 }};	/*Useragent - LG-T310/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA2147417 = {S101871, 44, { &P13206,&P17485,&P17338,&P18092 }};	/*Useragent - sonyericssonk530i/UC Browser7.9.0.102/69/352*/
const Device UA2156414 = {S101872, 153, { &P17346,&P18117,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 TeamStream (iPod4,1 215 1.6.1)*/
const Device UA2158779 = {S101873, 76, { &P17595,&P18120,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Mac OS X; Opera Mobi/35779; U; en) Presto/2.10.254 Version/12.00*/
const Device UA2171625 = {S101874, 35, { &P22987,&P17484,&P21759,&P18092 }};	/*Useragent - iBrowser/Mini2.8 (SAMSUNG GT-C3303)*/
const Device UA2177998 = {S101875, 39, { &P15795,&P5448,&P20763,&P18092 }};	/*Useragent - NOKIA-Lumia 710/UCWEB2.4.0.188/47/32079*/
const Device UA2180921 = {S101876, 74, { &P17595,&P19834,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.1 Darwin/12.1.0 (x86_64) (MacBookAir4%2C2)*/
const Device UA2187348 = {S101877, 74, { &P17595,&P19834,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.1 Darwin/12.1.0 (x86_64) (MacBookPro8%2C2)*/
const Device UA2190156 = {S101878, 69, { &P17595,&P19834,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.1 Darwin/12.1.0 (x86_64) (iMac12%2C2)*/
const Device UA2241306 = {S101879, 202, { &P18385,&P22212,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus 7 Build/JRO03R; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:34, us*/
const Device UA2241810 = {S101880, 202, { &P18385,&P22212,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-sg; Nexus 7 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, sg*/
const Device UA2327491 = {S101881, 33, { &P15803,&P5843,&P21764,&P18092 }};	/*Useragent - iBrowser/Mini2.7 (Nokia200/11.56)*/
const Device UA2337200 = {S101882, 79, { &P12707,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-in; Nokia701)/UC Browser8.4.0.159/50/352/UCWEB Mobile*/
const Device UA2338632 = {S101883, 159, { &P12657,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC EVO 3D X515m Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2348891 = {S101884, 171, { &P12280,&P18432,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 6_0 like Mac OS X; en-us) AppleWebKit/536.26 (KHTML, like Gecko) Version/5.1 Mobile/10A403 Safari/7534.48.3 UCBrowser8.4.0.159/42/355*/
const Device UA2350358 = {S101885, 63, { &P15767,&P18117,&P22070,&P18092 }};	/*Useragent - Instagram 3.0.0 (iPad; iPhone OS 5.1.1; en_US) AppleWebKit/420+*/
const Device UA2375073 = {S101886, 81, { &P14367,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA6710s)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA2445684 = {S101887, 147, { &P12026,&P17485,&P24670,&P18092 }};	/*Useragent - SonyEricssonC510/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; sonyericssonc510) UCBrowser8.4.0.159/70/352/UCWEB Mobile*/
const Device UA2519751 = {S101888, 165, { &P15364,&P17285,&P18727,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/1.6.1.1691 (Phoenix Viewer Release Mesh; Firebird skin) Safari/533.3*/
const Device UA2540563 = {S101889, 167, { &P21203,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; SX-SP715A Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2610664 = {S101890, 147, { &P14908,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6210 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2635476 = {S101891, 161, { &P12879,&P6546,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7510, 42, 1.9.1)*/
const Device UA2618915 = {S101892, 88, { &P22043,&P17979,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.4.6) (espresso10wifi;samsung;GT-P5113;4.0.4/IMM76D)*/
const Device UA2618121 = {S101893, 104, { &P15876,&P17979,&P22069,&P18092 }};	/*Useragent - Instagram 1.1.8 Android (15/4.0.3; 160dpi; 800x1232; samsung; GT-P5100; espresso10rf; espresso10; en_GB)*/
const Device UA2624104 = {S101894, 96, { &P17323,&P17979,&P22069,&P18092 }};	/*Useragent - Instagram 1.1.5 Android (13/3.2; 160dpi; 800x1232; samsung; GT-P6800; GT-P6800; smdkc210; en_GB)*/
const Device UA2623162 = {S101895, 166, { &P17323,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6800 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2624188 = {S101896, 166, { &P15985,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6200 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2627212 = {S101897, 166, { &P15985,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P6200 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2622017 = {S101898, 90, { &P12889,&P17979,&P22073,&P18092 }};	/*Useragent - Instagram 3.3.2 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7300; GT-P7300; p3; en_US)*/
const Device UA2622357 = {S101899, 86, { &P12889,&P17979,&P22073,&P18092 }};	/*Useragent - Instagram 3.3.3 Android (16/4.1.1; 160dpi; 800x1232; samsung; GT-P7300; p5; p3; en_US)*/
const Device UA2631758 = {S101900, 154, { &P17323,&P6437,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-CN; GT-P6800 Build/HTJ85B) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.1.258 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2628647 = {S101901, 166, { &P15876,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P5100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2629671 = {S101902, 165, { &P14908,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2631562 = {S101903, 165, { &P14908,&P5096,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2630737 = {S101904, 161, { &P14908,&P5629,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-P1010 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000*/
const Device UA2634713 = {S101905, 151, { &P15985,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2631060 = {S101906, 57, { &P13452,&P17979,&P22278,&P18092 }};	/*Useragent - theScore/3.1.2 Android-OS/2.2 (Android; Phone, GT-P1000R)*/
const Device UA2730051 = {S101907, 95, { &P12142,&P17485,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; fa; sonyericssonu5i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA3044702 = {S101908, 98, { &P12142,&P17485,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sonyericssonu5i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2737378 = {S101909, 92, { &P16003,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233a) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2869209 = {S101910, 92, { &P16003,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233a) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2820139 = {S101911, 159, { &P13020,&P5096,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HW-HUAWEI_C8500S/C8500SV100R001C92B627SP01; 240*320; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2820685 = {S101912, 35, { &P14673,&P17484,&P21759,&P18092 }};	/*Useragent - iBrowser/Mini2.8 (SAMSUNG-GT-E2652)*/
const Device UA3031114 = {S101913, 99, { &P14985,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8530) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3039675 = {S101914, 213, { &P24283,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 55.8.03.4; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039676 = {S101915, 213, { &P24283,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 57.3.13.5; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039681 = {S101916, 235, { &P24283,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 64.4.08.7; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039688 = {S101917, 213, { &P24283,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 55.8.03.4; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH) ; tr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039690 = {S101918, 235, { &P24283,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 67.2.73.6; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039705 = {S101919, 213, { &P24283,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 57.0.96.2; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH) ; de) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039706 = {S101920, 235, { &P24283,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 69.6.43.6; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039679 = {S101921, 213, { &P24283,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 57.0.96.2; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039733 = {S101922, 235, { &P24283,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 67.2.73.6; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; en) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039735 = {S101923, 213, { &P24283,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 56.2.00.0; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039740 = {S101924, 235, { &P24283,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 82.3.59.9; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039759 = {S101925, 235, { &P24283,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 86.5.68.7; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; en) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039760 = {S101926, 235, { &P24283,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL933; 74.8.30.4; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039682 = {S101927, 253, { &P24290,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.24.5; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039701 = {S101928, 251, { &P24290,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.21.6; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; ) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039702 = {S101929, 251, { &P24290,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.24.5; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; ) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039711 = {S101930, 254, { &P24290,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.27.14; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039716 = {S101931, 253, { &P24290,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.24.5; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; en) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039717 = {S101932, 254, { &P24290,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.27.14; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; ro) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039720 = {S101933, 254, { &P24290,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.27.19; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039724 = {S101934, 254, { &P24290,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.27.13; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039727 = {S101935, 253, { &P24290,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.21.6; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039732 = {S101936, 254, { &P24290,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.27.19; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; de) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039742 = {S101937, 254, { &P24290,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.27.14; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; en) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039710 = {S101938, 253, { &P24290,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.21.6; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; pl) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039755 = {S101939, 254, { &P24290,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL938; 7.0.28.14; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039693 = {S101940, 250, { &P24298,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.24.5; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; ro) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039695 = {S101941, 250, { &P24298,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.24.5; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039696 = {S101942, 248, { &P24298,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.24.5; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; ) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039697 = {S101943, 248, { &P24298,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.21.6; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; ) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039707 = {S101944, 251, { &P24298,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.27.13; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039708 = {S101945, 250, { &P24298,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.24.5; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; en) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039709 = {S101946, 251, { &P24298,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.27.13; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; pl) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039714 = {S101947, 250, { &P24298,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.21.6; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; pl) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039715 = {S101948, 250, { &P24298,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.21.6; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039721 = {S101949, 250, { &P24298,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.24.5; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; pl) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039741 = {S101950, 251, { &P24298,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.27.14; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039744 = {S101951, 251, { &P24298,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.27.19; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039748 = {S101952, 251, { &P24298,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL938; 7.0.28.14; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3080021 = {S101953, 90, { &P15732,&P17483,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-s365) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA51415 = {S101954, 77, { &P13272,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-S5050/S5050XEIF1 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA1117 = {S101955, 61, { &P14716,&P5926,&P4030,&P18092 }};	/*Useragent - Nokia8801/2.0 (03.78) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA505088 = {S101956, 103, { &P14708,&P5204,&P4742,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (07.95) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5130c-2/UC Browser7.5.1.77/69/352*/
const Device UA2439 = {S101957, 70, { &P16054,&P5448,&P4578,&P18092 }};	/*Useragent - HTCP3600-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)*/
const Device UA35924 = {S101958, 71, { &P16054,&P5448,&P4578,&P18092 }};	/*Useragent - HTC_P3600-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)*/
const Device UA3540 = {S101959, 77, { &P17992,&P17017,&P3402,&P18092 }};	/*Useragent - BenQ-S88/1.00/WAP2.0/MIDP2.0/CLDC1.0 UP.Browser/6.3.0.4.c.1.102 (GUI) MMP/2.0*/
const Device UA3927 = {S101960, 122, { &P13297,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5362/V2.31.841.6 (64646) Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA5149 = {S101961, 53, { &P12110,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player htc_wwe / 1.0 / htc_legend / 2.2*/
const Device UA45689 = {S101962, 57, { &P12110,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player vodafone_au / 1.0 / htc_legend / 2.2*/
const Device UA45708 = {S101963, 57, { &P12110,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player vodafone_uk / 1.0 / htc_legend / 2.2*/
const Device UA6311 = {S101964, 115, { &P17595,&P18120,&P17318,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Safari/525.20.1*/
const Device UA10003 = {S101965, 89, { &P13462,&P17484,&P4924,&P18092 }};	/*Useragent - Vodafone/SamsungSGHE860V/E860JBEH4/Browser/NF/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA10004 = {S101966, 89, { &P13462,&P17484,&P4924,&P18092 }};	/*Useragent - Vodafone/SamsungSGHE860V/E860JBFA2/Browser/NF/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA10005 = {S101967, 89, { &P13462,&P17484,&P4924,&P18092 }};	/*Useragent - Vodafone/SamsungSGHE860V/E860AEFA2/Browser/NF/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA86898 = {S101968, 89, { &P13462,&P17484,&P4924,&P18092 }};	/*Useragent - Vodafone/SamsungSGHE860V/E860AEEH6/Browser/NF/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87886 = {S101969, 89, { &P13462,&P17484,&P4924,&P18092 }};	/*Useragent - Vodafone/SamsungSGHE860V/E860BUEH6/Browser/NF/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA10258 = {S101970, 50, { &P13489,&P6476,&P3625,&P18092 }};	/*Useragent - LG7100-parrot/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA40009 = {S101971, 43, { &P13489,&P6476,&P3625,&P18092 }};	/*Useragent - LG7100/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA16532 = {S101972, 71, { &P14966,&P5448,&P22270,&P18092 }};	/*Useragent - Xda_Zest_MSIE/480x640 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA18509 = {S101973, 127, { &P13100,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) SP; 240x320; HTC_S710/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49890 = {S101974, 70, { &P13100,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_S710 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA20423 = {S101975, 119, { &P20395,&P22839,&P4223,&P18092 }};	/*Useragent - Zte-tu232_TD/1.0 ThreadX/4.0b Larena/2.40 Release/4.15.2010 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20898 = {S101976, 83, { &P15142,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) SAMSUNG-SGH-i601/WM534*/
const Device UA21418 = {S101977, 73, { &P15273,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-M3310/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21636 = {S101978, 42, { &P15309,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-LG231 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA21797 = {S101979, 61, { &P15341,&P17017,&P4977,&P18092 }};	/*Useragent - Mozilla/4.0 (MobilePhone PM-8200/US/1.0) NetFront/3.1 MMP/2.0*/
const Device UA21873 = {S101980, 72, { &P12013,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-SGH-I607/UCHA1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA22206 = {S101981, 47, { &P15399,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-VX8100/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA36233 = {S101982, 192, { &P12053,&P17605,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-J708i/1.0 RTK-E/1.0 DF/1.0 Release/12.18.2008 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzUyNDUwMDM4NDQzMTY1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA63142 = {S101983, 197, { &P12053,&P17605,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-J708i_CMCC/1.0 RTK-E/1.0 DF/1.0 Release/12.18.2008 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1/*VPGf5U4AAOpQ8Z/lTADq UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA27291 = {S101984, 122, { &P12166,&P17605,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S3601C_CMCC/1.0 RTK-E/1.0 DF/1.0 Release/01.09.2009 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA27970 = {S101985, 98, { &P12199,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-J706/ Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA30440 = {S101986, 83, { &P12315,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonS500c/R8BE Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA30982 = {S101987, 57, { &P12382,&P5768,&P3625,&P18092 }};	/*Useragent - Nokia6276/2.0 (BL200V0300.nep) UP.Browser/6.2.3.8 MMP/2.0*/
const Device UA32858 = {S101988, 61, { &P12667,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia3555/2.0 (03.46) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA33362 = {S101989, 85, { &P12905,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-G780/MTK Release/01.01.2011 Browser/MAUI Profile/MIDP2.0Configuration/CLDC-1*/
const Device UA33395 = {S101990, 87, { &P12912,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNXG798/MTK Release/10.1.2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA33458 = {S101991, 73, { &P12922,&P6476,&P4970,&P18092 }};	/*Useragent - LG-LX160 AU-MIC-LX160/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA44918 = {S101992, 124, { &P13841,&P5208,&P4521,&P18092 }};	/*Useragent - Mozilla/5.0 (SCH-W799/W799DC01;U;REX/3.0;BREW/3.1.5;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*400;CTC/2.0) POLARIS/6.2.WEB*/
const Device UA43018 = {S101993, 96, { &P17488,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) Vodafone/1.0/HTC_Mercury/2.04.161.2*/
const Device UA58934 = {S101994, 96, { &P17488,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) Vodafone/1.0/HTC_Mercury/2.18.161.4*/
const Device UA44045 = {S101995, 24, { &P17752,&P17017,&P17470,&P18092 }};	/*Useragent - WELLCOM-W189 WAP Browser*/
const Device UA89818 = {S101996, 24, { &P17752,&P17017,&P17470,&P18092 }};	/*Useragent - WELLCOM-W189_WAP_Browser*/
const Device UA44842 = {S101997, 98, { &P17116,&P19944,&P3625,&P18092 }};	/*Useragent - SHARP-TQ-GX15/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.3.2.d.1.110 (GUI) MMP/2.0*/
const Device UA64779 = {S101998, 98, { &P17116,&P19944,&P3625,&P18092 }};	/*Useragent - SHARP-TQ-GX15/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.3.2.d.1.112 (GUI) MMP/2.0*/
const Device UA45336 = {S101999, 13, { &P14432,&P17483,&P17660,&P18092 }};	/*Useragent - LG/KF700/v2.0*/
const Device UA47649 = {S102000, 67, { &P12552,&P17017,&P17470,&P18092 }};	/*Useragent - Fly_E140_India/WAPBrowser2.0Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA60601 = {S102001, 61, { &P12552,&P17017,&P17470,&P18092 }};	/*Useragent - Fly_E140/WAPBrowser2.0Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA48434 = {S102002, 189, { &P12707,&P5055,&P3804,&P18092 }};	/*Useragent - Mozilla/5.0 (Symbian/3; Series60/5.2 Nokia701/111.010.0107; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko) NokiaBrowser/7.4.1.3 Mobile Safari/533.4 3gpp-gba*/
const Device UA48879 = {S102003, 46, { &P12936,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-MX830/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA50169 = {S102004, 75, { &P11797,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-D900i/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA50446 = {S102005, 116, { &P11679,&P5776,&P4737,&P18092 }};	/*Useragent - Nokia6630/1.0 (6.03.40) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.2.2.51/27/352*/
const Device UA51128 = {S102006, 97, { &P14484,&P17484,&P4086,&P18092 }};	/*Useragent - SAMSUNG-GT-S7070C/1.0 SHP/VPP/R5 Dolfin/1.0 SMM-MMS/1.2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA53675 = {S102007, 79, { &P14001,&P17484,&P4924,&P18092 }};	/*Useragent - SAMSUNG-SGH-T419/T419UVHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 NetFront/3.2*/
const Device UA53679 = {S102008, 34, { &P14004,&P17017,&P3807,&P18092 }};	/*Useragent - SHARP SH6017D / Obigo Browser Q03C*/
const Device UA54046 = {S102009, 107, { &P14102,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-ZV50-Vodafone/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA69454 = {S102010, 99, { &P14711,&P5204,&P4777,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-01/UC Browser7.7.1.88/70/444*/
const Device UA57919 = {S102011, 159, { &P20062,&P5129,&P3690,&P18092 }};	/*Useragent - Nokia5802_CMCC/21.0.025.6 (SymbianOS/9.4; U; Series60/5.0 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA60526 = {S102012, 114, { &P15407,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) ASUS-P527/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA60629 = {S102013, 48, { &P15312,&P17017,&P4977,&P18092 }};	/*Useragent - Sanyo-SCP6600CA NetFront/3.1 MMP/2.0 Mozilla/4.0*/
const Device UA61162 = {S102014, 66, { &P12649,&P17483,&P3499,&P18092 }};	/*Useragent - LG-GU295g/V10i Teleca/Q7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA61254 = {S102015, 42, { &P12670,&P17483,&P17660,&P18092 }};	/*Useragent - LG-KP550-Bouygues/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA61426 = {S102016, 53, { &P12716,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-W315/1.0 UP.Browser/6.2.2.6.n.1.101 (GUI) MMP/2.0*/
const Device UA61979 = {S102017, 40, { &P14053,&P17485,&P4203,&P18092 }};	/*Useragent - SonyEricssonZ300i/R3M/TelecaBrowser/4.08*/
const Device UA62619 = {S102018, 84, { &P14430,&P5776,&P3756,&P18092 }};	/*Useragent - Nokia6708/2.0 (V1.0.0) SymbianOS/7.0 UIQ/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA62981 = {S102019, 121, { &P11996,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S5560C/1.0 SHP/R5 Release/02.01.2009 Browser/NetFront3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA63483 = {S102020, 106, { &P17806,&P17811,&P3807,&P18092 }};	/*Useragent - Lenovo-P580/CMCC_S017 BMP/F2 Release/2008.11.18 Browser/TelecaQ03C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA66989 = {S102021, 142, { &P12945,&P17486,&P20418,&P18092 }};	/*Useragent - MOT-A780/R52_G_0D.50.ADP Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola A780; 935) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 7.50 [en]*/
const Device UA67105 = {S102022, 143, { &P12945,&P17486,&P20418,&P18092 }};	/*Useragent - MOT-A780/R52_G_0D.50.AEP Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola A780; 1027) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 7.50 [en]*/
const Device UA82445 = {S102023, 133, { &P12945,&P17486,&P20418,&P18092 }};	/*Useragent - MOT-A780/R52_G_0D.43.A3P Mozilla/4.0 (compatible; MSIE 6.0; Motorola; 850) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 7.50 [zh-tw]*/
const Device UA67314 = {S102024, 80, { &P15210,&P5448,&P4715,&P18092 }};	/*Useragent - SAMSUNG-GT-i8000/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65*/
const Device UA67886 = {S102025, 118, { &P11669,&P5776,&P4759,&P18092 }};	/*Useragent - Nokia7610/2.0 (7.0642.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.6.1.82/27/354*/
const Device UA68562 = {S102026, 87, { &P14935,&P17017,&P3402,&P18092 }};	/*Useragent - BenQ-SIEMENS - E52/1.00/WAP2.0/MIDP2.0/CLDC1.0 UP.Browser/6.3.0.4.c.1.102 (GUI) MMP/2.0*/
const Device UA87191 = {S102027, 47, { &P17523,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-HI3A UP.Browser/6.2.0.13.1.4 (GUI) MMP/2.0*/
const Device UA87907 = {S102028, 116, { &P16048,&P5448,&P22271,&P18092 }};	/*Useragent - PPC; 240x320; HTC_P3350/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA88428 = {S102029, 89, { &P11755,&P16000,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Nokia 6670/6.0522.0; 9399) Opera 8.65 [en]*/
const Device UA195774 = {S102030, 125, { &P18061,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; sdk Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA253951 = {S102031, 40, { &P12142,&P17485,&P4777,&P18092 }};	/*Useragent - SonyEricssonU5/UC Browser7.7.1.88/50/444*/
const Device UA459174 = {S102032, 137, { &P17835,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nook Tablet Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA466596 = {S102033, 36, { &P11905,&P5890,&P4737,&P18092 }};	/*Useragent - NOKIA6120c/UC Browser7.2.2.51/28/351*/
const Device UA533441 = {S102034, 71, { &P17567,&P5567,&P3430,&P18092 }};	/*Useragent - Mozilla/5.0 Opera/9.5 (KDDI-SH3D; BREW; Opera Mobi; U; ja) Presto/2.2.1*/
const Device UA535173 = {S102035, 76, { &P15364,&P17017,&P19772,&P18092 }};	/*Useragent - Mozilla/5.0 (OS/2; U; Warp 4.5; en-US; rv:1.8.1.23) Gecko/X SeaMonkey/1.1.18*/
const Device UA547480 = {S102036, 39, { &P15518,&P17484,&P4777,&P18092 }};	/*Useragent - samsung-s5230/UC Browser7.7.1.88/69/444*/
const Device UA702111 = {S102037, 34, { &P15518,&P17484,&P4777,&P18092 }};	/*Useragent - gt-s5230/UC Browser7.7.1.88/69/444*/
const Device UA558054 = {S102038, 75, { &P12871,&P18110,&P24661,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.3;Zh_cn;HTC Wildfire S A510e;320*480;)UCWEB7.7.0.85/139/800*/
const Device UA561140 = {S102039, 111, { &P11787,&P5204,&P4779,&P18092 }};	/*Useragent - Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia3120classic/UC Browser7.8.0.95/69/352*/
const Device UA603966 = {S102040, 80, { &P15364,&P17285,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/3.0.16)*/
const Device UA616157 = {S102041, 80, { &P15364,&P17285,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; Sleipnir/3.0.17)*/
const Device UA741375 = {S102042, 87, { &P15364,&P17285,&P18351,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; WOW64; Trident/5.0; Sleipnir/3.0.17)*/
const Device UA615543 = {S102043, 138, { &P12280,&P18117,&P18356,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/7534.48.3 Sleipnir/1.9.2m*/
const Device UA626061 = {S102044, 140, { &P12280,&P18117,&P18356,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/1.9.2m*/
const Device UA617361 = {S102045, 81, { &P12280,&P17779,&P17780,&P18092 }};	/*Useragent - Opera/9.80 (iPhone; Opera Mini/6.13548/25.729; U; ru) Presto/2.5.25 Version/10.54*/
const Device UA1899903 = {S102046, 43, { &P12588,&P5055,&P24663,&P18092 }};	/*Useragent - NokiaC6-01/UC Browser7.9.0.102/50/351/UCWEB*/
const Device UA691129 = {S102047, 30, { &P11854,&P5890,&P24666,&P18092 }};	/*Useragent - NokiaE71/UCWEB8.1.0.104/28/999*/
const Device UA744649 = {S102048, 81, { &P15364,&P5690,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; Opera Mobi/23731; U; en) Presto/2.9.201 Version/11.50*/
const Device UA780113 = {S102049, 139, { &P23898,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-uk; Elonex_765ET Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA925719 = {S102050, 103, { &P14630,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; SAMSUNG; SGH-i917R)*/
const Device UA1848935 = {S102051, 34, { &P12004,&P5890,&P4715,&P18092 }};	/*Useragent - NOKIAN95/UC Browser7.4.0.65/28/352*/
const Device UA2207654 = {S102052, 38, { &P12004,&P5890,&P4715,&P18092 }};	/*Useragent - NOKIAN95 8GB/UC Browser7.4.0.65/28/352*/
const Device UA1888901 = {S102053, 71, { &P13042,&P18110,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.3;Zh_cn;HTC Incredible S;480*800;)UCWEB7.8.1.96/139/800*/
const Device UA1925109 = {S102054, 161, { &P13668,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Dream/Sapphire Build/FRG83; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1949474 = {S102055, 142, { &P15767,&P17145,&P20773,&P18092 }};	/*Useragent - MQQBrowser/2.2 Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/7534.48.3*/
const Device UA1960351 = {S102056, 168, { &P13018,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; MT11i Build/4.1.B.0.431) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2028107 = {S102057, 171, { &P16479,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; SCH-R720 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA1976124 = {S102058, 158, { &P20134,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5830i Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1986879 = {S102059, 54, { &P15364,&P17350,&P18107,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Win64 (AMD64); en-US; Firefox*/
const Device UA1993558 = {S102060, 137, { &P16036,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en_us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027165 = {S102061, 135, { &P16036,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en_us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018196 = {S102062, 158, { &P22127,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; zh-cn; 3GW101 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 Lenovo-3GW101/1.0.0*/
const Device UA2018437 = {S102063, 167, { &P16034,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; MOT-XT806/QLCG_M2_02.11.8; 480*854; CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018691 = {S102064, 167, { &P16034,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; MOT-XT806/QLCG_M2_03.10.4; 480*854; CTC/2.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019548 = {S102065, 95, { &P17976,&P5477,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.1-update1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.8.149 Version/11.10*/
const Device UA2020309 = {S102066, 74, { &P16760,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2.1;Zh_cn;Spice Mi-270;240*320;)UCWEB7.8.0.95/140/444*/
const Device UA2021256 = {S102067, 88, { &P17976,&P5629,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Android 2.3.5; Linux; Opera Tablet/ADR-X; U; en) Presto/2.7.81 Version/11.00*/
const Device UA2023016 = {S102068, 129, { &P18103,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; FZ-A1A Build/HTK55) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2023939 = {S102069, 163, { &P14090,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I5801 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2024948 = {S102070, 159, { &P13042,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Incredible S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025941 = {S102071, 159, { &P13042,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Incredible S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029777 = {S102072, 185, { &P13157,&P18066,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC Sensation XE with Beats Audio Z715e Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031976 = {S102073, 168, { &P14872,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SPH-D700 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032208 = {S102074, 149, { &P13452,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2618097 = {S102075, 149, { &P13452,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; fr-fr; GT-P1000 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032399 = {S102076, 134, { &P20602,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; nl-nl; TAB260 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032708 = {S102077, 167, { &P14574,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Nexus S Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032839 = {S102078, 168, { &P12814,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-I777 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2158230 = {S102079, 79, { &P13445,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN85)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA2186464 = {S102080, 71, { &P17595,&P19834,&P18430,&P18092 }};	/*Useragent - Safari/8536.25 CFNetwork/596.1 Darwin/12.1.0 (x86_64) (MacBookAir4%2C2)*/
const Device UA2189374 = {S102081, 71, { &P17595,&P19834,&P18430,&P18092 }};	/*Useragent - Safari/8536.25 CFNetwork/596.1 Darwin/12.1.0 (x86_64) (MacBookPro8%2C1)*/
const Device UA2260798 = {S102082, 138, { &P12280,&P17146,&P18361,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3 Sleipnir/2.0m*/
const Device UA2284074 = {S102083, 125, { &P20148,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG_GT-I7680_TD/1.0 OPhone/2.0 Release/2.1.2010 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 AppleWebKit/528.5+*/
const Device UA2303804 = {S102084, 45, { &P13094,&P18110,&P17810,&P18092 }};	/*Useragent - JUC(Linux; U;2.3.3;zh_cn;Lenovo A60;320*480;)*/
const Device UA2461308 = {S102085, 99, { &P15302,&P17485,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonwt13i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2469165 = {S102086, 99, { &P12351,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s3653w) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2525936 = {S102087, 152, { &P17625,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.7; en-in; ST25i Build/6.0.B.3.184) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2535974 = {S102088, 99, { &P14182,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5333) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2544330 = {S102089, 54, { &P14983,&P5890,&P20780,&P18092 }};	/*Useragent - MQQBrowser/2.9 (NokiaC7-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA2556322 = {S102090, 89, { &P14532,&P17483,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-t325) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2586629 = {S102091, 140, { &P21604,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C2105 Build/15.0.A.1.31) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2586656 = {S102092, 139, { &P21627,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC first Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2590957 = {S102093, 86, { &P13220,&P17485,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; sonyericssonk770i) UCBrowser8.4.0.159/69/352/UCWEB Mobile*/
const Device UA2598672 = {S102094, 90, { &P14532,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t325) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2615423 = {S102095, 180, { &P13452,&P5629,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-P1000 Build/GINGERBREAD; 600*1024) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/352*/
const Device UA2609539 = {S102096, 92, { &P13452,&P17979,&P3508,&P18092 }};	/*Useragent - GetJarSDK/X.00 uk.co.exelentia.wikipedia/50 (X)  Android/2.2 (samsung; GT-P1000T; GT-P1000T)*/
const Device UA2619146 = {S102097, 53, { &P13452,&P17979,&P3508,&P18092 }};	/*Useragent - WhatsApp/2.6.4974 Android/2.2 Device/samsung-GT-P1000*/
const Device UA2609936 = {S102098, 89, { &P15985,&P17979,&P16179,&P18092 }};	/*Useragent - GetJarSDK/X.00 com.nevosoft.mysteryville/2 (X)  Android/3.2 (samsung; GT-P6200; GT-P6200)*/
const Device UA2621190 = {S102099, 61, { &P15985,&P17979,&P16179,&P18092 }};	/*Useragent - com.eurosport.news.android/3.1.1/Android/3.2/samsung-GT-P6200*/
const Device UA2611194 = {S102100, 143, { &P15985,&P6437,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2611639 = {S102101, 95, { &P17337,&P17979,&P16179,&P18092 }};	/*Useragent - GetJarSDK/X.00 com.nevosoft.mysteryville2.free/5 (X)  Android/3.2 (samsung; GT-P7500; GT-P7500)*/
const Device UA2619684 = {S102102, 36, { &P17337,&P17979,&P16179,&P18092 }};	/*Useragent - Android/3.2; samsung GT-P7500; 2.3.2*/
const Device UA2634295 = {S102103, 148, { &P17337,&P17979,&P16179,&P18092 }};	/*Useragent - Rdio.Net; Android/2.5.1; Android/3.2; DM=samsung; DN=GT-P7500R; DC=samsung; DF=samsung/GT-P7500R/GT-P7500R:3.2/HTJ85B/UXKK2:user/release-keys; DH=p3*/
const Device UA2619086 = {S102104, 87, { &P12879,&P18066,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 4.0.4; GT-P7510 Build/Unknown);*/
const Device UA2616305 = {S102105, 145, { &P17337,&P6437,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2634420 = {S102106, 152, { &P17337,&P6437,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2616451 = {S102107, 164, { &P15932,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3110, 34, 1.6.0)*/
const Device UA2623644 = {S102108, 164, { &P15932,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-ph; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3110, 38, 1.8.1)*/
const Device UA2617573 = {S102109, 183, { &P12879,&P18066,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, nl*/
const Device UA2622931 = {S102110, 219, { &P15985,&P18066,&P17373,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/6.5.26969/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2624274 = {S102111, 217, { &P15985,&P18066,&P17373,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/6.5.26969/28.3392; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2620222 = {S102112, 147, { &P15919,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622166 = {S102113, 147, { &P15919,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2620668 = {S102114, 70, { &P15985,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.4;Zh_cn;GT-P6200;600*976;)UCWEB7.8.0.95/139/444*/
const Device UA2637888 = {S102115, 119, { &P15985,&P18066,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android4.0.4;Zh_cn;GT-P6200;600*976;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA2622080 = {S102116, 206, { &P15932,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-P3110 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.11/226,1.9.11.226,2012-12-04 16:21, +0000*/
const Device UA2630408 = {S102117, 210, { &P15932,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; it-it; GT-P3110 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/292,1.9.23.292,2013-02-06 08:41, +0100, it*/
const Device UA2632820 = {S102118, 209, { &P15932,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-P3110 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0100, de*/
const Device UA2633015 = {S102119, 164, { &P15932,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3110, 40, 1.8.5)*/
const Device UA2623464 = {S102120, 210, { &P17337,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; nl-be; GT-P7500 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0100, be*/
const Device UA2633063 = {S102121, 209, { &P17337,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; nl-nl; GT-P7500 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0100, nl*/
const Device UA2631652 = {S102122, 166, { &P17323,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6800 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2660432 = {S102123, 151, { &P19747,&P5043,&P24675,&P18092 }};	/*Useragent - Nokia206/2.0 (03.59) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia206) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2684699 = {S102124, 172, { &P18213,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-S6500D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352*/
const Device UA2719703 = {S102125, 142, { &P22655,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5503 Build/10.1.1.A.1.253) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2759296 = {S102126, 75, { &P19859,&P18110,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Adr (Linux; U; 4.0.3; zh-cn; HTC T328w Build/IML74K;480*800)*/
const Device UA2765906 = {S102127, 154, { &P11667,&P5768,&P24676,&P18092 }};	/*Useragent - Nokia6085/2.0 (06.00) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6085) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2807970 = {S102128, 53, { &P14984,&P5890,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 (Nokia5250;SymbianOS/9.1 Series60/3.0)*/
const Device UA2815138 = {S102129, 81, { &P15201,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.3;Zh_cn;HTC Sensation Z710e;540*960;)UCWEB7.8.0.95/139/352*/
const Device UA2815750 = {S102130, 47, { &P14358,&P5890,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1 (E63;SymbianOS/9.1 Series60/3.0)*/
const Device UA2816256 = {S102131, 153, { &P12798,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.4; en-in; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2817841 = {S102132, 154, { &P18223,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; LT22i Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3032619 = {S102133, 91, { &P14763,&P5776,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian72) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3039678 = {S102134, 213, { &P24258,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 58.0.65.5; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039683 = {S102135, 235, { &P24258,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 67.2.73.6; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039699 = {S102136, 235, { &P24258,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 64.4.08.7; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039700 = {S102137, 213, { &P24258,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 58.0.65.5; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH) ; it) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039712 = {S102138, 235, { &P24258,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 74.8.30.4; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039734 = {S102139, 213, { &P24258,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 58.0.65.5; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH) ; sr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039736 = {S102140, 235, { &P24258,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 82.3.59.9; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039750 = {S102141, 235, { &P24258,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 86.5.68.7; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; de) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039752 = {S102142, 235, { &P24258,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 86.5.68.7; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039754 = {S102143, 213, { &P24258,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 58.0.65.5; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH) ; pt) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039756 = {S102144, 235, { &P24258,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 74.8.30.4; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; sr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039757 = {S102145, 213, { &P24258,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_VL963; 58.7.54.2; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039778 = {S102146, 130, { &P24236,&P18110,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv7l; U; InettvBrowser/2.2(00014A;SonyDTV115;0002;0100) KDL-46EX641; CC/USA; en) Presto/2.10.250 Version/11.60*/
const Device UA63990 = {S102147, 51, { &P13274,&P6712,&P3402,&P18092 }};	/*Useragent - Nokia-1606 UP.Browser/6.3.0.8.c.1.102 (GUI) MMP/2.0*/
const Device UA400 = {S102148, 98, { &P11816,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-L71/AAUG2128AA 08.02.06R/10.21.2005 MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/1.0*/
const Device UA3642 = {S102149, 75, { &P13280,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-W5/08.00.05R MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/1.0*/
const Device UA3936 = {S102150, 83, { &P13300,&P17485,&P3487,&P18092 }};	/*Useragent - SonyEricssonW302c/R1BB Browser/OpenWave/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA5142 = {S102151, 89, { &P13501,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; GIGABYTE-MS800*/
const Device UA30809 = {S102152, 179, { &P12346,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; 2.1-update1; en-us; apanda-A60 Build/ERE27; apanda-A60 Build/DONUT) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA11126 = {S102153, 79, { &P13545,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; O2 Xda Atom Exec; PPC; 240x320)*/
const Device UA14375 = {S102154, 85, { &P13839,&P16000,&P3772,&P18092 }};	/*Useragent - MOTO-Z10/50.00.00 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 335) Opera 8.65 [en]*/
const Device UA17050 = {S102155, 86, { &P11893,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-S700/CMCC_S101 LMP/LML Release/2008.12.24 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA17049 = {S102156, 81, { &P11893,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-S700/S300 LMP/LML Release/2009.06.11 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA20422 = {S102157, 119, { &P20398,&P22839,&P4223,&P18092 }};	/*Useragent - Zte-tu722_TD/1.0 ThreadX/4.0b Larena/2.40 Release/4.15.2010 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21143 = {S102158, 123, { &P13531,&P17605,&P3976,&P18092 }};	/*Useragent - SAMSUNG-GT-B5702C_CMCC/1.0 RTK-E/1.0 DF/1.0 Release/01.14.2009 Browser/NetFront3.4  Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21369 = {S102159, 46, { &P15266,&P7472,&P3625,&P18092 }};	/*Useragent - Alcatel-TH3/1.0 UP.Browser/6.2.ALCATEL MMP/1.0*/
const Device UA21505 = {S102160, 61, { &P15289,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonZ600/R601 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA21798 = {S102161, 49, { &P15342,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-U520 UP.Browser/6.2.3.8.f.1.100 (GUI) MMP/2.0*/
const Device UA22083 = {S102162, 70, { &P17104,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-SGH-i740/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA22401 = {S102163, 85, { &P15454,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-v770/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA22456 = {S102164, 101, { &P15477,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-C425/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA22916 = {S102165, 68, { &P15524,&P19954,&P3976,&P18092 }};	/*Useragent - SEC-SGHF200/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23653 = {S102166, 74, { &P15615,&P17486,&P4203,&P18092 }};	/*Useragent - Motorola-ic902 Obigo/Q04C1 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24012 = {S102167, 97, { &P15653,&P19950,&P3625,&P18092 }};	/*Useragent - SAGEM-myV-55/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.6.d.3.100 (GUI) MMP/1.0*/
const Device UA24483 = {S102168, 46, { &P15693,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-AX300/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA24505 = {S102169, 46, { &P15698,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-AX500/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA24644 = {S102170, 111, { &P15727,&P17483,&P3534,&P18092 }};	/*Useragent - LG-KF701/v10b Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24710 = {S102171, 66, { &P12170,&P6712,&P4030,&P18092 }};	/*Useragent - Nokia5208_CMCC/2.0 (06.20) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA27304 = {S102172, 61, { &P12170,&P6712,&P4030,&P18092 }};	/*Useragent - Nokia5208/2.0 (06.20) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24729 = {S102173, 43, { &P19900,&P17484,&P4203,&P18092 }};	/*Useragent - Samsung-SCHU740 AU-OBIGO/Q04C1-1.22 MMP/2.0*/
const Device UA26722 = {S102174, 88, { &P13624,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-my411C/1.0/ MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.386 (GUI)*/
const Device UA738145 = {S102175, 89, { &P13624,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-my411Cv/1.0/ MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.363 (GUI)*/
const Device UA28161 = {S102176, 97, { &P12226,&P5448,&P17183,&P18092 }};	/*Useragent - HTC_Touch_Pro_T7272 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA31169 = {S102177, 115, { &P12407,&P17605,&P3976,&P18092 }};	/*Useragent - SAMSUNG-S3600H/1.0 RTK-E/1.0 DF/1.0 Release/07.01.2008 Browser/NetFront3.4  Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA31245 = {S102178, 92, { &P11726,&P5810,&P4948,&P18092 }};	/*Useragent - BlackBerry8800/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/179 UP.Browser/5.0.3.3*/
const Device UA32740 = {S102179, 97, { &P16700,&P19950,&P3625,&P18092 }};	/*Useragent - SAGEM-myV-56/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.3.3.g.2.102 (GUI) MMP/2.0*/
const Device UA32983 = {S102180, 44, { &P12815,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KU311/v10d Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA39913 = {S102181, 70, { &P17768,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) DopodD810*/
const Device UA40699 = {S102182, 142, { &P17931,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-T849 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA69948 = {S102183, 99, { &P14711,&P5204,&P4779,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (11.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-01/UC Browser7.8.0.95/70/355*/
const Device UA54031 = {S102184, 83, { &P14096,&P17485,&P3487,&P18092 }};	/*Useragent - SonyEricssonS302c/R1BC Browser/OpenWave/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA43768 = {S102185, 58, { &P13738,&P5690,&P17191,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SC-01B)*/
const Device UA46771 = {S102186, 113, { &P17287,&P5448,&P4808,&P18092 }};	/*Useragent - HPiPAQGlisten/1.00_1.00 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC;320x240*/
const Device UA48028 = {S102187, 37, { &P12610,&P17486,&P3807,&P18092 }};	/*Useragent - MOT-W418G/07.2E.00 Obigo/Q03C MMP/2.0*/
const Device UA49310 = {S102188, 40, { &P16694,&P17483,&P17602,&P18092 }};	/*Useragent - LG-MG280d Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA61280 = {S102189, 40, { &P16694,&P17483,&P17602,&P18092 }};	/*Useragent - LG-MG280c Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA71488 = {S102190, 39, { &P16694,&P17483,&P17602,&P18092 }};	/*Useragent - LG-MG280 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA51976 = {S102191, 115, { &P11903,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-C3212/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA50038 = {S102192, 86, { &P13150,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-G238/MTK Release/22.11.2007 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1*/
const Device UA50307 = {S102193, 34, { &P13977,&P5890,&P4722,&P18092 }};	/*Useragent - NokiaN73/UC Browser7.0.0.41/28/351*/
const Device UA50387 = {S102194, 116, { &P13838,&P5776,&P4737,&P18092 }};	/*Useragent - Nokia6600/1.0 (5.53.0) SymbianOS/7.0s Series60/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.2.2.51/27/352*/
const Device UA53794 = {S102195, 194, { &P14030,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Mozilla/5.0 (HTC-T9199/1.08.706.1;U;Windows Mobile/6.5;Profile/MIDP-2.0 Configuration/CLDC-1.1;480*800;CTC/2.0)MSIE/6.0; Windows Phone 6.5.3.5)*/
const Device UA75533 = {S102196, 195, { &P14030,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Mozilla/5.0 (HTC-T9199/1.18.1401.8;U;Windows Mobile/6.5;Profile/MIDP-2.0 Configuration/CLDC-1.1;480*800;CTC/2.0)MSIE/6.0; Windows Phone 6.5.3.5)*/
const Device UA53934 = {S102197, 118, { &P14069,&P17605,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-F488E/1.0 RTK-E/1.0 DF/1.0 Release/08.03.2007 Browser/NetFront3.4  Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54804 = {S102198, 88, { &P14265,&P19950,&P4893,&P18092 }};	/*Useragent - SAGEM-my501X/1.0/ MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.f.1.160 (GUI)*/
const Device UA55059 = {S102199, 15, { &P14279,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHC210/1.0*/
const Device UA59203 = {S102200, 114, { &P15165,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-P180-ORANGE/P180BVGK7 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA61061 = {S102201, 62, { &P12619,&P17017,&P3866,&P18092 }};	/*Useragent - PG-C300/R03 MIC/1.1.14 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA85342 = {S102202, 62, { &P12619,&P17017,&P3866,&P18092 }};	/*Useragent - PG-C300/R01 MIC/1.1.14 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62618 = {S102203, 96, { &P14429,&P17484,&P3502,&P18092 }};	/*Useragent - SAMSUNG-GT-B7732/1.0 SHP/VPP/R5 Dolfin/1.5 SMM-MMS/1.2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62844 = {S102204, 107, { &P17081,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-P270-ORANGE/6_HJ1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA67079 = {S102205, 36, { &P11907,&P5890,&P4715,&P18092 }};	/*Useragent - NokiaE52-1/UC Browser7.4.0.65/28/351*/
const Device UA67377 = {S102206, 88, { &P12194,&P5448,&P4715,&P18092 }};	/*Useragent - HTC_Touch2_T3320 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65*/
const Device UA67794 = {S102207, 29, { &P14358,&P5890,&P24660,&P18092 }};	/*Useragent - NokiaE63/UCWEB7.6.0.75/28/800*/
const Device UA68003 = {S102208, 34, { &P13554,&P5890,&P4759,&P18092 }};	/*Useragent - NOKIAN79/UC Browser7.6.1.82/28/351*/
const Device UA68808 = {S102209, 43, { &P11941,&P17485,&P4777,&P18092 }};	/*Useragent - sonyericssonw580i/UC Browser7.7.1.88/69/444*/
const Device UA69449 = {S102210, 39, { &P14519,&P5129,&P4779,&P18092 }};	/*Useragent - NokiaN97 mini/UC Browser7.8.0.95/50/355*/
const Device UA81736 = {S102211, 94, { &P14443,&P17484,&P3816,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungSGHX660V/X660BUEK4/Browser/TSS/2.5 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA784964 = {S102212, 94, { &P14443,&P17484,&P3816,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungSGHX660V/X660JBEK2/Browser/TSS/2.5 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA85786 = {S102213, 64, { &P15364,&P5890,&P4924,&P18092 }};	/*Useragent - Mozilla/4.08 (SmartPhone; Symbian OS-Series60/1.03) NetFront/3.2*/
const Device UA87633 = {S102214, 35, { &P17885,&P17017,&P17470,&P18092 }};	/*Useragent - V908/MIDP2.0/CLDC1.1/Screen-240x320*/
const Device UA87722 = {S102215, 37, { &P15688,&P17484,&P4970,&P18092 }};	/*Useragent - SCH-R500/1.0 AU-MIC/2.0 (GUI) MMP/2.0*/
const Device UA88031 = {S102216, 15, { &P11843,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNGSGH-i450*/
const Device UA88489 = {S102217, 31, { &P17754,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 L01A(c100;TB;W24H17)*/
const Device UA88987 = {S102218, 90, { &P18088,&P5448,&P15761,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows CE 5.2; en-US; rv:1.9.2a1pre) Gecko/20090513 Fennec/1.0a1*/
const Device UA90007 = {S102219, 47, { &P17627,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC3Y UP.Browser/6.2.0.17.1.1 (GUI) MMP/2.0*/
const Device UA371348 = {S102220, 58, { &P14642,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; XT532 Build/V4.37B)*/
const Device UA421651 = {S102221, 34, { &P12856,&P17483,&P17338,&P18092 }};	/*Useragent - lg-t300/UC Browser7.9.0.102/69/352*/
const Device UA443887 = {S102222, 89, { &P15364,&P5690,&P21806,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.8pre) Gecko/X Prism zdesktop/2.0*/
const Device UA459255 = {S102223, 37, { &P15988,&P17484,&P23400,&P18092 }};	/*Useragent - MQQBrowser/Mini2.4 (SAMSUNG-GT-C3312)*/
const Device UA459277 = {S102224, 155, { &P15991,&P5629,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA472860 = {S102225, 116, { &P15364,&P17188,&P17243,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 4.0; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3*/
const Device UA474863 = {S102226, 165, { &P15364,&P17285,&P23487,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; chrome://navigator/locale/navigator.properties; rv:1.8.1.21) Gecko/X SecondLife/1.23.5.X (Second Life Release; default skin)*/
const Device UA477577 = {S102227, 82, { &P15364,&P17285,&P22532,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.7) Gecko/X Epic/ Firefox/*/
const Device UA491487 = {S102228, 58, { &P17583,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SA3C UP.Browser/6.2_7.2.7.1.K.1.5.1.120 (GUI) MMP/2.0*/
const Device UA498619 = {S102229, 101, { &P17595,&P17296,&P22536,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.2) Gecko/X Firefox/3.6 Kylo/0.8.4.74873*/
const Device UA2270227 = {S102230, 86, { &P16958,&P17131,&P24644,&P18092 }};	/*Useragent - Palm680/RC1 (iPhone; U; CPU iPhone OS 2_2_1 like Mac OS X; zh-cn)/UCWEB7.4.1.68/41/997*/
const Device UA523440 = {S102231, 43, { &P17521,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 913SH)*/
const Device UA533832 = {S102232, 42, { &P14972,&P17484,&P4759,&P18092 }};	/*Useragent - samsung-gt-s3370/UC Browser7.6.1.82/69/351*/
const Device UA548053 = {S102233, 119, { &P12139,&P5204,&P4759,&P18092 }};	/*Useragent - Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5310xpressmusic/UC Browser7.6.1.82/69/352*/
const Device UA567615 = {S102234, 99, { &P14482,&P5843,&P4777,&P18092 }};	/*Useragent - NokiaC3-01/5.0 (06.05) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac3-01/UC Browser7.7.1.88/69/444*/
const Device UA571479 = {S102235, 62, { &P11905,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (Nokia6120c(B20.01);SymbianOS/9.1 Series60/3.0)*/
const Device UA2214860 = {S102236, 62, { &P11905,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (Nokia6120c(B20.03);SymbianOS/9.1 Series60/3.0)*/
const Device UA581607 = {S102237, 62, { &P15364,&P7133,&P20459,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; rv:9.0.1) Gecko/X SeaMonkey/2.6.1*/
const Device UA590271 = {S102238, 153, { &P15364,&P7133,&P18212,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-GB) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/3.2.8.X (Second Life Release; default skin) Safari/533.3*/
const Device UA597067 = {S102239, 132, { &P16598,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A500 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA601605 = {S102240, 40, { &P17968,&P17979,&P24431,&P18092 }};	/*Useragent - Dolphin http client/7.6.0(127) (Android)*/
const Device UA2025408 = {S102241, 168, { &P11950,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA616383 = {S102242, 83, { &P17465,&P7133,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; Opera Tablet/23731; U; en) Presto/2.9.201 Version/11.50*/
const Device UA616847 = {S102243, 81, { &P12280,&P17779,&P17371,&P18092 }};	/*Useragent - Opera/9.80 (iPhone; Opera Mini/5.19802/25.697; U; ru) Presto/2.5.25 Version/10.54*/
const Device UA857324 = {S102244, 147, { &P15438,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC Sensation Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA894635 = {S102245, 39, { &P15795,&P5448,&P20767,&P18092 }};	/*Useragent - NOKIA-Lumia 710/UCWEB2.1.0.155/47/32079*/
const Device UA1864886 = {S102246, 129, { &P15364,&P5690,&P17247,&P18163 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Chrome/8.0.552.224 Safari/534.10/Nutch-1.2*/
const Device UA1878606 = {S102247, 126, { &P17595,&P17296,&P24725,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/533.21.1 (KHTML, like Gecko) FluidApp/1.0 Safari/533.21.1*/
const Device UA1882423 = {S102248, 43, { &P12615,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 (jig browser web; 1.0.4; 931SC)*/
const Device UA1890834 = {S102249, 71, { &P17568,&P5567,&P3430,&P18092 }};	/*Useragent - Mozilla/5.0 Opera/9.5 (KDDI-SH3F; BREW; Opera Mobi; U; ja) Presto/2.2.1*/
const Device UA1904837 = {S102250, 74, { &P17998,&P5477,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.1-update1;Zh_cn;XCD 28;240*320;)UCWEB7.8.0.95/140/444*/
const Device UA2019167 = {S102251, 137, { &P14832,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; LG-KH5200 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019498 = {S102252, 178, { &P14969,&P5629,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC_Desire_S Build/GRI40; 480*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/800*/
const Device UA2024687 = {S102253, 145, { &P20593,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us; CUBE U9GT 2 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022415 = {S102254, 142, { &P21821,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; TAB-PROTAB2XXL Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024530 = {S102255, 157, { &P13123,&P5629,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024768 = {S102256, 157, { &P22875,&P5629,&P20784,&P18092 }};	/*Useragent - MQQBrowser/3.3/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025494 = {S102257, 157, { &P14186,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028468 = {S102258, 154, { &P13815,&P5096,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2.1; en-in; HTC Wildfire Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029658 = {S102259, 95, { &P17976,&P17979,&P18256,&P18092 }};	/*Useragent - Opera/9.80 (Android android4.0; Linux; Opera Tablet/ADR-X; U; ru) Presto/2.10.254 Version/12.00*/
const Device UA2033614 = {S102260, 170, { &P16760,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; Spice Mi-270 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2033254 = {S102261, 169, { &P18295,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; HTC One X Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034339 = {S102262, 176, { &P13064,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SAMSUNG-SGH-I727 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2177638 = {S102263, 39, { &P15775,&P5448,&P20763,&P18092 }};	/*Useragent - NOKIA-Lumia 800/UCWEB2.4.0.188/47/32079*/
const Device UA2306272 = {S102264, 134, { &P15364,&P5690,&P23732,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko)  QtWeb Internet Browser/3.5 http://www.QtWeb.net*/
const Device UA2372947 = {S102265, 140, { &P12108,&P5843,&P24667,&P18092 }};	/*Useragent - Nokia7230/5.0 (06.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia7230) UCBrowser8.2.1.144/70/355/UCWEB Mobile*/
const Device UA2338455 = {S102266, 151, { &P21388,&P5096,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2.2; en-in; Karbonn_A1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2374153 = {S102267, 79, { &P14107,&P5890,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN76)/UC Browser8.2.0.132/28/355/UCWEB Mobile*/
const Device UA2418803 = {S102268, 149, { &P13526,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; T-Mobile myTouch 4G Build/JRO03L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2545440 = {S102269, 135, { &P18253,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; XT894 Build/IMM76L) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2597485 = {S102270, 99, { &P12260,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5350) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2598111 = {S102271, 98, { &P14972,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s3370) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2599144 = {S102272, 90, { &P14568,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-c105) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2607478 = {S102273, 72, { &P14630,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i917R;7.10;8862)*/
const Device UA2607863 = {S102274, 30, { &P13452,&P17979,&P20428,&P18092 }};	/*Useragent - Android OS/10 samsung_GT-P1000*/
const Device UA2618998 = {S102275, 144, { &P13452,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android HDVS_zz_2.2.1; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2625883 = {S102276, 61, { &P13452,&P17979,&P20428,&P18092 }};	/*Useragent - bitartist.marksix/3.2 (Linux; Android VER:8,samsung,GT-P1000)*/
const Device UA2628409 = {S102277, 52, { &P13452,&P17979,&P20428,&P18092 }};	/*Useragent - com.kakao.talk/Android Version/2.2.2 Model/GT-P1000R*/
const Device UA2613200 = {S102278, 137, { &P17337,&P6437,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; it-it; GT-P7500 Build/HTJ85B)Maxthon AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2624420 = {S102279, 147, { &P13452,&P5096,&P21760,&P18092 }};	/*Useragent - iBrowser/2.8 Mozilla/5.0 (Linux; U; Android 2.2; en-za; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2618916 = {S102280, 96, { &P22043,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Mail; 1.4.6) (espresso10wifi; samsung; GT-P5113; 4.0.4/IMM76D)*/
const Device UA2615944 = {S102281, 165, { &P14908,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2617717 = {S102282, 167, { &P15985,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6200L Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2623106 = {S102283, 166, { &P15985,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6200 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2614382 = {S102284, 57, { &P17337,&P17979,&P22697,&P18092 }};	/*Useragent - TwitterAndroid/1.0.5 (109) GT-P7500/13 (samsung;GT-P7500)*/
const Device UA2633069 = {S102285, 172, { &P15985,&P6437,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444*/
const Device UA2615823 = {S102286, 95, { &P15985,&P17979,&P22071,&P18092 }};	/*Useragent - Instagram 3.1.0 Android (13/3.2; 160dpi; 600x976; samsung; GT-P6200; GT-P6200; smdkc210; en_GB)*/
const Device UA2619239 = {S102287, 90, { &P17337,&P17979,&P22071,&P18092 }};	/*Useragent - Instagram 3.1.0 Android (13/3.2; 160dpi; 800x1232; samsung; GT-P7500; GT-P7500; p3; en_GB)*/
const Device UA2635333 = {S102288, 164, { &P17323,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P6800, 42, 1.9.1)*/
const Device UA2619515 = {S102289, 90, { &P17337,&P17979,&P22068,&P18092 }};	/*Useragent - Instagram 1.0.5 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7500; GT-P7500; p3; en_US)*/
const Device UA2618533 = {S102290, 42, { &P12879,&P17979,&P16179,&P18092 }};	/*Useragent - Layar/7.0.2 Android/3.2 (samsung GT-P7510)*/
const Device UA2635396 = {S102291, 145, { &P12879,&P17979,&P16179,&P18092 }};	/*Useragent - Rdio.Net; Android/2.5.1; Android/3.2; DM=samsung; DN=GT-P7510; DC=samsung; DF=samsung/GT-P7510/GT-P7510:3.2/HTJ85B/VLKL2:user/release-keys; DH=p3*/
const Device UA2619784 = {S102292, 90, { &P12889,&P17979,&P22069,&P18092 }};	/*Useragent - Instagram 1.1.1 Android (12/3.1; 160dpi; 1280x800; samsung; GT-P7300; GT-P7300; p3; en_US)*/
const Device UA2621927 = {S102293, 83, { &P22043,&P17979,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (Android Sportacular; 70-3.0) (Android; GT-P5113; Android OS/4.0.4)*/
const Device UA2630342 = {S102294, 85, { &P22043,&P17979,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (Android Sportacular; 76-3.0.6) (Android; GT-P5113; Android OS/4.1.1)*/
const Device UA2622906 = {S102295, 163, { &P12889,&P6546,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7300 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2622340 = {S102296, 164, { &P12889,&P6437,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7300 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2625286 = {S102297, 106, { &P15882,&P17979,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.0 Android (16/4.1.1; 160dpi; 800x1232; samsung; GT-P5110; espresso10wifi; espresso10; en_GB)*/
const Device UA2626122 = {S102298, 132, { &P15985,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P6200 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2627163 = {S102299, 147, { &P22037,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2637155 = {S102300, 154, { &P22037,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3113 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2628534 = {S102301, 132, { &P14908,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P6210 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2629059 = {S102302, 151, { &P15876,&P18111,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; it-it; GT-P5100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2629597 = {S102303, 164, { &P12889,&P6437,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7300 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2632927 = {S102304, 162, { &P22037,&P18111,&P24406,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 OverDrive Media Console/2.6.4*/
const Device UA2633308 = {S102305, 152, { &P22037,&P18111,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2634138 = {S102306, 131, { &P14905,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7100 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2635897 = {S102307, 151, { &P15876,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P5100 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2638001 = {S102308, 151, { &P15876,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P5100 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2698686 = {S102309, 91, { &P12051,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5600) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2706976 = {S102310, 91, { &P14330,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gx300) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2807355 = {S102311, 126, { &P23656,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40EX725; CC/DEU; en) Presto/2.7.61 Version/11.00*/
const Device UA2817975 = {S102312, 145, { &P21239,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; M9 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2818169 = {S102313, 159, { &P17968,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2944606 = {S102314, 99, { &P15197,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-b5310) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2988381 = {S102315, 152, { &P19974,&P5043,&P24676,&P18092 }};	/*Useragent - Nokia111/2.0 (03.32) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia111) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA3039594 = {S102316, 202, { &P24241,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LM6200-TA; 04.10.08; 0xX;); LG NetCast.TV-2012 0*/
const Device UA3039595 = {S102317, 202, { &P24241,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LM6200-UE; 04.20.18; 0xX;); LG NetCast.TV-2012 0*/
const Device UA3039894 = {S102318, 81, { &P15364,&P24348,&P17203,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-us; rv:2.0) Gecko/X Fedora/16-dev Firefox/4.0*/
const Device UA3068992 = {S102319, 109, { &P21407,&P18148,&P22782,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows Phone 8.0; ARM; Trident/7.0; Touch; rv:11.0; IEMobile/11.0; NOKIA; Lumia 928) like Gecko*/
const Device UA3084097 = {S102320, 162, { &P13123,&P5629,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; GT-S5360 Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084116 = {S102321, 162, { &P13123,&P5629,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-S5360 Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084440 = {S102322, 81, { &P17968,&P17979,&P24501,&P18092 }};	/*Useragent - Opera/9.80 (Android; Opera Mini/18.1/30.3558; U; en) Presto/2.8.119 Version/11.10*/
const Device UA3092788 = {S102323, 90, { &P14471,&P17483,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-s310) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3108958 = {S102324, 154, { &P11699,&P5768,&P24679,&P18092 }};	/*Useragent - Nokia5200/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5200) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3109152 = {S102325, 160, { &P12125,&P5204,&P24679,&P18092 }};	/*Useragent - Nokia6500s-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6500s-1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA88 = {S102326, 57, { &P13490,&P5768,&P3625,&P18092 }};	/*Useragent - Nokia6265/2.0 (HL190V0600.nep) UP.Browser/6.2.3.8 MMP/2.0*/
const Device UA10269 = {S102327, 58, { &P13490,&P5768,&P3625,&P18092 }};	/*Useragent - Nokia6265i/2.0 (HL100V1000.nep) UP.Browser/6.2.3.8 MMP/2.0*/
const Device UA63754 = {S102328, 87, { &P12421,&P17485,&P4203,&P18092 }};	/*Useragent - SonyEricssonT303a/R2DD002 TelecaBrowser/Q04C1-1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA12336 = {S102329, 96, { &P12292,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Touch_3G_T3232 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA15758 = {S102330, 145, { &P15364,&P17486,&P20417,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola ???; 2647) MOT-???/R533_11.12.09Pi_NTA Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [vi]*/
const Device UA66731 = {S102331, 137, { &P15364,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-MOTOKRZRmaxxK3/99.31.04R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12153168) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA2265515 = {S102332, 165, { &P15364,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-E680G/1.0 E680G_G_0D.C5.AEP/04.09.2006 Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola E680G; 1483) Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [zh-cn]*/
const Device UA16360 = {S102333, 77, { &P17621,&P5448,&P3430,&P18092 }};	/*Useragent - SAMSUNG-SPH-M8400/(null)ID4 (compatible; MSIE 6.0; Windows CE; PPC) Opera 9.5*/
const Device UA17370 = {S102334, 60, { &P11916,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia6800/1.0 (3.14) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA41391 = {S102335, 113, { &P12864,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-G508E/G508EZCIG2 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA18170 = {S102336, 67, { &P11812,&P5448,&P22271,&P18092 }};	/*Useragent - Xda_Atmos; 240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA24747 = {S102337, 68, { &P14189,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V557/08.28.40R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20413 = {S102338, 75, { &P15086,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) ASUSM530w/1.0*/
const Device UA20419 = {S102339, 99, { &P17610,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S7120U/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA21260 = {S102340, 39, { &P16529,&P17483,&P3625,&P18092 }};	/*Useragent - LG-CG225 UP.Browser/6.2.3 (GUI) MMP/1.0*/
const Device UA21422 = {S102341, 96, { &P16738,&P19950,&P3625,&P18092 }};	/*Useragent - SAGEM-myX-4/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.6.d.5.100 (GUI) MMP/1.0*/
const Device UA32802 = {S102342, 56, { &P16738,&P19950,&P3625,&P18092 }};	/*Useragent - SAGEM-myX-4/1.2 UP.Browser/6.2.2.6.d.5.100 (GUI) MMP/1.0*/
const Device UA21513 = {S102343, 69, { &P15293,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_MAX Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA88388 = {S102344, 72, { &P15293,&P5448,&P22270,&P18092 }};	/*Useragent - HTC MAX 4G Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA22249 = {S102345, 15, { &P15412,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHX530/1.0*/
const Device UA22932 = {S102346, 117, { &P15532,&P17605,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-G608/1.0 RTK-E/1.0 DF/1.0 Release/06.22.2007 Browser/NetFront3.4  Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23442 = {S102347, 39, { &P15607,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KM500 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA26792 = {S102348, 45, { &P13636,&P17483,&P3625,&P18092 }};	/*Useragent - LG-CG180/V1.0b UP.Browser/6.2.3 (GUI) MMP/1.0*/
const Device UA31166 = {S102349, 138, { &P12405,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; zh-tw; FET-T1 Build/DONUT) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA33150 = {S102350, 46, { &P12647,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-LG830/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA34331 = {S102351, 98, { &P13674,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-F408/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA36176 = {S102352, 97, { &P12059,&P5204,&P4779,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2690/UC Browser7.8.0.95/70/351*/
const Device UA40695 = {S102353, 86, { &P13689,&P17485,&P3600,&P18092 }};	/*Useragent - SonyEricssonD750i/R1N Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63330 = {S102354, 115, { &P12175,&P17484,&P3430,&P18092 }};	/*Useragent - SAMSUNG-S7350/S7350BOIH2 SHP/VPP/R5 Opera/9.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA43767 = {S102355, 84, { &P13738,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) SC-01B*/
const Device UA48137 = {S102356, 40, { &P12635,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GT370 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA48892 = {S102357, 63, { &P15364,&P22839,&P17470,&P18092 }};	/*Useragent - ZTE-C_S180/VE_S180_NBTL25FV1.0.0B15 SMIT-BROWSER/V2.0.0 CTC/1.0*/
const Device UA49173 = {S102358, 148, { &P12945,&P17486,&P20417,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Linux; Motorola A780; 781) MOT-A780/R532L4_G_11.40.0FR Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera 8.00 [es-la]*/
const Device UA49629 = {S102359, 86, { &P17546,&P17017,&P18246,&P18092 }};	/*Useragent - Fly_DS150/MAUIBrowser/Configuration/WAP2.0, Fly_DS150/MAUIBrowser/Configuration/WAP2.0*/
const Device UA89005 = {S102360, 42, { &P17546,&P17017,&P18246,&P18092 }};	/*Useragent - Fly_DS150/MAUIBrowser/Configuration/WAP2.0*/
const Device UA50036 = {S102361, 79, { &P13149,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-T119/T119UVHL2 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA50195 = {S102362, 34, { &P14358,&P5890,&P4722,&P18092 }};	/*Useragent - NokiaE63/UC Browser7.0.0.41/28/352*/
const Device UA50330 = {S102363, 138, { &P14657,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; ko-kr; LU2300 Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA50995 = {S102364, 70, { &P21682,&P17017,&P17470,&P18092 }};	/*Useragent - i-mobile Hitz 2205/2.0 (03.15) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA51312 = {S102365, 21, { &P12502,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW20i/R7DA*/
const Device UA51701 = {S102366, 88, { &P13955,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-my411V/1.0/ MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.317 (GUI)*/
const Device UA63980 = {S102367, 88, { &P13955,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-my411V/1.0/ MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.326 (GUI)*/
const Device UA51853 = {S102368, 53, { &P13361,&P17483,&P17660,&P18092 }};	/*Useragent - LG/KF750/v1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA52814 = {S102369, 34, { &P13163,&P5129,&P4737,&P18092 }};	/*Useragent - NokiaN97/UC Browser7.2.2.51/50/352*/
const Device UA53508 = {S102370, 79, { &P12970,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) ASUS-GalaxyII/1.0*/
const Device UA68564 = {S102371, 52, { &P14048,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-C257/0.1.31 UP.Browser/6.2.3.9.c.6 (GUI) MMP/2.0*/
const Device UA53947 = {S102372, 126, { &P14075,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 640x480; Ameo/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1) Opera 8.65 [de]*/
const Device UA87345 = {S102373, 126, { &P14075,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 640x480; Ameo/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1) Opera 8.65 [en]*/
const Device UA87346 = {S102374, 126, { &P14075,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 640x480; Ameo/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1) Opera 8.65 [en]*/
const Device UA54154 = {S102375, 86, { &P14133,&P17017,&P4901,&P18092 }};	/*Useragent - SIE-MC60/07 Profile/MIDP-1.0 Configuration/CLDC-1.0 UP.Browser/6.1.0.7.3 (GUI) MMP/1.0*/
const Device UA64083 = {S102376, 86, { &P14133,&P17017,&P4901,&P18092 }};	/*Useragent - SIE-MC60/10 Profile/MIDP-1.0 Configuration/CLDC-1.0 UP.Browser/6.1.0.7.3 (GUI) MMP/1.0*/
const Device UA54873 = {S102377, 98, { &P16972,&P19944,&P3625,&P18092 }};	/*Useragent - SHARP-TQ-GX29/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.3.3.h.1.100 (GUI) MMP/2.0*/
const Device UA55135 = {S102378, 37, { &P14287,&P17483,&P17470,&P18092 }};	/*Useragent - LG-C1100 MIC/WAP2.0 MIDP-2.0/CLDC-1.0*/
const Device UA55961 = {S102379, 137, { &P13675,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; Vibo-A699 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA56775 = {S102380, 34, { &P14454,&P5890,&P4715,&P18092 }};	/*Useragent - NokiaE66/UC Browser7.4.0.65/28/351*/
const Device UA60319 = {S102381, 127, { &P15285,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) SP; 320x240; HTC_S630/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61019 = {S102382, 47, { &P12612,&P17017,&P17470,&P18092 }};	/*Useragent - Huawei/1.0/T520/B000 Browser/obigo-browser/q04a*/
const Device UA61404 = {S102383, 37, { &P12712,&P17483,&P17470,&P18092 }};	/*Useragent - LG-KG320 MIC/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA61570 = {S102384, 139, { &P12747,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-tw; MUCHTEL-A2 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA61780 = {S102385, 89, { &P14328,&P17017,&P4901,&P18092 }};	/*Useragent - SIE-A31/07 Profile/MIDP-1.0 Configuration/CLDC-1.0 UP.Browser/6.1.0.7.3.102 (GUI) MMP/1.0*/
const Device UA63413 = {S102386, 89, { &P14328,&P17017,&P4901,&P18092 }};	/*Useragent - SIE-A31/09 Profile/MIDP-1.0 Configuration/CLDC-1.0 UP.Browser/6.1.0.7.3.102 (GUI) MMP/1.0*/
const Device UA87650 = {S102387, 90, { &P14328,&P17017,&P4901,&P18092 }};	/*Useragent - SIE-A31a/03 Profile/MIDP-1.0 Configuration/CLDC-1.0 UP.Browser/6.1.0.7.3.102 (GUI) MMP/1.0*/
const Device UA61966 = {S102388, 46, { &P14355,&P6476,&P3625,&P18092 }};	/*Useragent - CDM8950/T05_0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA63188 = {S102389, 40, { &P13652,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KC780 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA64036 = {S102390, 95, { &P15364,&P19950,&P4893,&P18092 }};	/*Useragent - SAGEM-my501C/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.f.1.160 (GUI)*/
const Device UA64040 = {S102391, 44, { &P13642,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KG800/V10c Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA67085 = {S102392, 34, { &P13554,&P5890,&P4715,&P18092 }};	/*Useragent - NOKIAN79/UC Browser7.4.0.65/28/351*/
const Device UA67799 = {S102393, 138, { &P21017,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.0; nl-nl; TP-07D8 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA68384 = {S102394, 36, { &P14984,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia5250/SymbianOS/9.1 Series60/3.0*/
const Device UA69125 = {S102395, 34, { &P13554,&P5890,&P4779,&P18092 }};	/*Useragent - NOKIAN79/UC Browser7.8.0.95/28/351*/
const Device UA2298028 = {S102396, 35, { &P14984,&P5129,&P4779,&P18092 }};	/*Useragent - Nokia5250/UC Browser7.8.0.95/50/355*/
const Device UA69691 = {S102397, 73, { &P14228,&P5448,&P22272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) ASUSM930/1.0*/
const Device UA71205 = {S102398, 62, { &P17920,&P5448,&P4578,&P18092 }};	/*Useragent - DopodC800/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)*/
const Device UA86680 = {S102399, 88, { &P16548,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-my401C/1.0/ MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.c.1.258 (GUI)*/
const Device UA86740 = {S102400, 47, { &P20208,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-MX8700/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA87230 = {S102401, 45, { &P17705,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-PT33 UP.Browser/6.2.0.13.2 (GUI) MMP/2.0*/
const Device UA87237 = {S102402, 45, { &P17704,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC3G UP.Browser/6.2.0.14.1 (GUI) MMP/2.0*/
const Device UA87348 = {S102403, 123, { &P14075,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 640x480; Ameo/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87368 = {S102404, 79, { &P16231,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) Xda Argon; 240x320*/
const Device UA88381 = {S102405, 88, { &P11679,&P16000,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Nokia 6630/5.03.08; 9399) Opera 8.65 [en]*/
const Device UA89651 = {S102406, 47, { &P17545,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC3T UP.Browser/6.2.0.15.1.1 (GUI) MMP/2.0*/
const Device UA89770 = {S102407, 126, { &P20232,&P17483,&P3459,&P18092 }};	/*Useragent - LG-GU282 Browser/Obigo-Q7.3 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA89808 = {S102408, 70, { &P17375,&P17017,&P17470,&P18092 }};	/*Useragent - BIRD S701_GPRS/1.0 OSE/V1.6.0 ULC/2.0 Release/04.21.2008 Browser/JB4.3*/
const Device UA90176 = {S102409, 61, { &P18284,&P17017,&P3499,&P18092 }};	/*Useragent - KyoceraC4700/Obigo/Q7 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA90263 = {S102410, 45, { &P17538,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC3P UP.Browser/6.2.0.15.2 (GUI) MMP/2.0*/
const Device UA212873 = {S102411, 124, { &P15767,&P17090,&P22247,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.5.1*/
const Device UA216166 = {S102412, 124, { &P15767,&P17090,&P22247,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J3 Flipboard/1.5.1*/
const Device UA331858 = {S102413, 124, { &P15767,&P17090,&P22247,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.5.1*/
const Device UA636092 = {S102414, 124, { &P15767,&P17090,&P22247,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.5.3*/
const Device UA650988 = {S102415, 122, { &P15767,&P17090,&P22247,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.5*/
const Device UA253926 = {S102416, 40, { &P14764,&P17485,&P4777,&P18092 }};	/*Useragent - SonyEricssonU1/UC Browser7.7.1.88/50/444*/
const Device UA255851 = {S102417, 48, { &P15364,&P18570,&P22248,&P18092 }};	/*Useragent - Flipboard/1.6.1 CFNetwork/485.13.9 Darwin/11.0.0*/
const Device UA664837 = {S102418, 43, { &P12080,&P17484,&P17347,&P18092 }};	/*Useragent - samsung-gt-s3653/UC Browser8.0.3.107/70/352*/
const Device UA379593 = {S102419, 41, { &P12061,&P5890,&P24663,&P18092 }};	/*Useragent - NOKIAE65/UC Browser7.9.0.102/28/355/UCWEB*/
const Device UA412450 = {S102420, 121, { &P12280,&P17146,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.8*/
const Device UA601684 = {S102421, 123, { &P12280,&P17146,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.8.2*/
const Device UA604243 = {S102422, 123, { &P12280,&P17146,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.8.4*/
const Device UA739794 = {S102423, 123, { &P12280,&P17146,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A406 Flipboard/1.8.4*/
const Device UA422368 = {S102424, 122, { &P15767,&P17146,&P18357,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/ Sleipnir/1.8.1m*/
const Device UA589923 = {S102425, 120, { &P15767,&P17146,&P18357,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/ Sleipnir/1.8m*/
const Device UA599960 = {S102426, 122, { &P15767,&P17146,&P18357,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/ Sleipnir/1.8.2m*/
const Device UA427436 = {S102427, 40, { &P17968,&P17979,&P24430,&P18092 }};	/*Useragent - Dolphin http client/7.5.0(124) (Android)*/
const Device UA442787 = {S102428, 81, { &P15364,&P7127,&P20901,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; ja-JP; rv:1.4) Gecko/X Netscape/7.1 (ax)*/
const Device UA450811 = {S102429, 71, { &P17595,&P18570,&P17415,&P18092 }};	/*Useragent - Safari/7534.48.3 CFNetwork/520.0.13 Darwin/11.0.0 (i386) (MacBook2%2C1)*/
const Device UA641479 = {S102430, 76, { &P17595,&P18570,&P17415,&P18092 }};	/*Useragent - Safari/7534.48.3 CFNetwork/520.0.13 Darwin/11.0.0 (x86_64) (MacBookPro8%2C2)*/
const Device UA607307 = {S102431, 98, { &P15364,&P16110,&P21905,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/18.0.1025.162)*/
const Device UA507184 = {S102432, 42, { &P15197,&P17484,&P4742,&P18092 }};	/*Useragent - samsung-gt-b5310/UC Browser7.5.1.77/69/351*/
const Device UA528488 = {S102433, 99, { &P16074,&P5768,&P4759,&P18092 }};	/*Useragent - Nokia3110c/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia3110c/UC Browser7.6.1.82/69/352*/
const Device UA531242 = {S102434, 42, { &P14980,&P17484,&P4759,&P18092 }};	/*Useragent - samsung-gt-b7722/UC Browser7.6.1.82/69/351*/
const Device UA532731 = {S102435, 63, { &P11747,&P18110,&P24661,&P18092 }};	/*Useragent - JUC(Linux;U;2.2;Zh_cn;HTC Desire;480*800;)UCWEB7.7.0.85/139/800*/
const Device UA2281114 = {S102436, 63, { &P11747,&P18110,&P24661,&P18092 }};	/*Useragent - JUC(Linux;U;2.2;Zh_cn;HTC Desire;480*800;)UCWEB7.7.0.85/139/999*/
const Device UA535156 = {S102437, 42, { &P14010,&P17484,&P4759,&P18092 }};	/*Useragent - samsung-gt-s5263/UC Browser7.6.1.82/69/351*/
const Device UA554157 = {S102438, 136, { &P11906,&P5448,&P4578,&P18092 }};	/*Useragent - SAMSUNG-GT-I8000U/1.0 Software/WM6.5 Release/01.24.2009 Profile/MIDP-2.1 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)*/
const Device UA554579 = {S102439, 105, { &P19970,&P17485,&P4777,&P18092 }};	/*Useragent - SonyEricssonG502/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssong502/UC Browser7.7.1.88/69/444*/
const Device UA556958 = {S102440, 99, { &P11793,&P5768,&P4777,&P18092 }};	/*Useragent - Nokia3500c/2.0 (06.60) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia3500c/UC Browser7.7.1.88/69/444*/
const Device UA563569 = {S102441, 105, { &P12502,&P17485,&P4777,&P18092 }};	/*Useragent - SonyEricssonW20i/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1  sonyericssonw20i/UC Browser7.7.1.88/69/444*/
const Device UA564726 = {S102442, 35, { &P11945,&P17484,&P4779,&P18092 }};	/*Useragent - gt-s5233s/UC Browser7.8.0.95/69/401*/
const Device UA567453 = {S102443, 49, { &P11854,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (E71-1;SymbianOS/9.1 Series60/3.0)*/
const Device UA572688 = {S102444, 44, { &P14060,&P17485,&P17338,&P18092 }};	/*Useragent - sonyericssonw150i/UC Browser7.9.0.102/69/352*/
const Device UA574756 = {S102445, 39, { &P16620,&P22784,&P17338,&P18092 }};	/*Useragent - micromaxx510/UC Browser7.9.0.102/69/352*/
const Device UA575007 = {S102446, 37, { &P15547,&P17484,&P24665,&P18092 }};	/*Useragent - samsung-gt-c6712/UCWEB8.0.3.99/70/350*/
const Device UA673385 = {S102447, 100, { &P13107,&P5043,&P17338,&P18092 }};	/*Useragent - NokiaC2-06/2.0 (06.96) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-06/UC Browser7.9.0.102/70/355*/
const Device UA587186 = {S102448, 148, { &P14433,&P5096,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA603793 = {S102449, 142, { &P12972,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T989 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA855379 = {S102450, 136, { &P12972,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T989 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA605418 = {S102451, 155, { &P13123,&P5629,&P21760,&P18092 }};	/*Useragent - iBrowser/2.8 Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023607 = {S102452, 165, { &P13974,&P5096,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5670 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA617723 = {S102453, 101, { &P12280,&P18117,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]*/
const Device UA617000 = {S102454, 99, { &P12280,&P18117,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]*/
const Device UA627280 = {S102455, 68, { &P17595,&P18584,&P21570,&P18092 }};	/*Useragent - Mail/1257 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookPro8%2C2)*/
const Device UA1986404 = {S102456, 63, { &P17595,&P18584,&P21570,&P18092 }};	/*Useragent - Mail/1257 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (iMac12%2C2)*/
const Device UA730675 = {S102457, 43, { &P11684,&P5129,&P24666,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UCWEB8.1.0.104/50/999*/
const Device UA1922429 = {S102458, 44, { &P11684,&P5129,&P24666,&P18092 }};	/*Useragent - Nokia5800w XpressMusic/UCWEB8.1.0.104/50/800*/
const Device UA695307 = {S102459, 44, { &P11924,&P17485,&P17338,&P18092 }};	/*Useragent - sonyericssons500i/UC Browser7.9.0.102/69/352*/
const Device UA771223 = {S102460, 44, { &P13216,&P17485,&P17338,&P18092 }};	/*Useragent - sonyericssonk750i/UC Browser7.9.0.102/69/352*/
const Device UA819508 = {S102461, 124, { &P17595,&P17297,&P24726,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) FluidApp Version/1.4 Safari/534.56.5*/
const Device UA1865732 = {S102462, 86, { &P15364,&P17285,&P21925,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0) chromeframe/9.0.597.98*/
const Device UA2277807 = {S102463, 141, { &P15364,&P17285,&P21925,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13 chromeframe/9.0.597.98*/
const Device UA1866606 = {S102464, 142, { &P13902,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungSGHM310V/M310BUHF2 Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA1950441 = {S102465, 163, { &P13964,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I5510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA1982935 = {S102466, 169, { &P14969,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA1996896 = {S102467, 76, { &P17595,&P18579,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacBookAir4%2C2)*/
const Device UA1996966 = {S102468, 76, { &P17595,&P18579,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/596.0.1 Darwin/12.0.0 (x86_64) (MacBookPro8%2C1)*/
const Device UA2399488 = {S102469, 114, { &P15364,&P17285,&P18532,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.75 Safari/537.1 Sleipnir/3.6.2*/
const Device UA2019139 = {S102470, 69, { &P14225,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2;Zh_cn;Desire HD;480*800;)UCWEB7.8.0.95/139/352*/
const Device UA2020375 = {S102471, 64, { &P14202,&P5062,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android1.6;Zh_cn;E15i;320*480;)UCWEB7.8.0.95/139/355*/
const Device UA2021586 = {S102472, 156, { &P14186,&P5629,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022609 = {S102473, 156, { &P17718,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB611 Build/BGN_2.3.5; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023201 = {S102474, 141, { &P22384,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; PTT1026 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023207 = {S102475, 155, { &P14849,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SCH-I800 Build/GRJ22; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023890 = {S102476, 154, { &P15575,&P5629,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024523 = {S102477, 164, { &P14967,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2031801 = {S102478, 164, { &P14967,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; LG-P500 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2024787 = {S102479, 67, { &P12824,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2.2;Zh_cn;U8180;240*320;)UCWEB7.8.0.95/139/444*/
const Device UA2025524 = {S102480, 58, { &P13089,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; U8160 Build/GRK39F)*/
const Device UA2026424 = {S102481, 157, { &P22875,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9220 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026570 = {S102482, 173, { &P14842,&P5477,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; E10i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2029143 = {S102483, 153, { &P13094,&P5629,&P20784,&P18092 }};	/*Useragent - MQQBrowser/3.3/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Lenovo A60 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032203 = {S102484, 150, { &P14997,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; it-it; Liquid Build/GRK39F; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2141462 = {S102485, 132, { &P15364,&P17813,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.32 (KHTML, like Gecko) Ubuntu/8.04 Chromium/13.0.751.0 Chrome/13.0.751.0 Safari/534.32*/
const Device UA2152718 = {S102486, 37, { &P14862,&P5204,&P21764,&P18092 }};	/*Useragent - iBrowser/Mini2.7 (Nokia2700c-2/09.98)*/
const Device UA2201843 = {S102487, 137, { &P12280,&P18432,&P18361,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3 Sleipnir/2.0m*/
const Device UA2301740 = {S102488, 103, { &P15364,&P18146,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; SunOS i86pc) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30*/
const Device UA2302400 = {S102489, 67, { &P19980,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.2; zh-cn; X903; 480*800) UCWEB7.9.3.103/139/7711*/
const Device UA2311197 = {S102490, 52, { &P14748,&P5890,&P21754,&P18092 }};	/*Useragent - iBrowser/2.4 (NokiaE72-1;SymbianOS/9.1 Series60/3.0)*/
const Device UA2316312 = {S102491, 47, { &P14358,&P5890,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8 (E63;SymbianOS/9.1 Series60/3.0)*/
const Device UA2422699 = {S102492, 98, { &P15198,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s3650) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2747450 = {S102493, 96, { &P14851,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; vi; samsung-gt-s5753e) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2429421 = {S102494, 98, { &P14864,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s8500) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2445437 = {S102495, 152, { &P11950,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.3; en-in; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2594084 = {S102496, 138, { &P18648,&P5043,&P24670,&P18092 }};	/*Useragent - Nokia306/2.0 (03.63) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia306) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2607487 = {S102497, 66, { &P14637,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;LG;LG-E900h;7.10;8862)*/
const Device UA2612460 = {S102498, 91, { &P17323,&P6437,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.4.2.11841); (Linux; U; Android 3.2; GT-P6800 Build/HONEYCOMB_MR2);*/
const Device UA2623272 = {S102499, 136, { &P22040,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; fa-fa; GT-P1013 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2627101 = {S102500, 60, { &P22040,&P5096,&P3508,&P18092 }};	/*Useragent - Dalvik/1.2.0 (Linux; U; Android 2.2.1; GT-P1013 Build/FROYO)*/
const Device UA2610272 = {S102501, 147, { &P15919,&P18066,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2612728 = {S102502, 147, { &P15919,&P18066,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2625496 = {S102503, 86, { &P22037,&P17979,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.4.5) (espressowifi;samsung;GT-P3113;4.1.1/JRO03C)*/
const Device UA2619541 = {S102504, 96, { &P17323,&P17979,&P22070,&P18092 }};	/*Useragent - Instagram 3.0.5 Android (13/3.2; 160dpi; 800x1232; samsung; GT-P6800; GT-P6800; smdkc210; en_GB)*/
const Device UA2617629 = {S102505, 99, { &P15919,&P17979,&P22071,&P18092 }};	/*Useragent - Instagram 3.1.0 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P3100; espressorf; espresso; en_GB)*/
const Device UA2618152 = {S102506, 88, { &P12889,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Mail; 1.4.5) (GT-P7300; samsung; GT-P7300; 3.2/HTJ85B)*/
const Device UA2619130 = {S102507, 54, { &P22037,&P17979,&P20428,&P18092 }};	/*Useragent - nApps (Android OS 4.0.4; GT-P3113; NaverSearch; 4.1.0)*/
const Device UA2628411 = {S102508, 72, { &P22037,&P17979,&P20428,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android Jelly Bean 4.1.2; GT-P3113 Build/JZO54K)*/
const Device UA2619257 = {S102509, 168, { &P15919,&P18066,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]*/
const Device UA2630094 = {S102510, 168, { &P15919,&P18066,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; id-id; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]*/
const Device UA2620020 = {S102511, 224, { &P17337,&P6437,&P18525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13 Opera/9.80 (Android; Opera Mini/6.0.25270/28.3126; U; fr) Presto/2.8.119 Version/11.10*/
const Device UA2625936 = {S102512, 217, { &P17337,&P6437,&P18525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Opera/9.80 (Android; Opera Mini/6.0.25270/28.3445; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2620165 = {S102513, 166, { &P22037,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P3113 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355*/
const Device UA2629827 = {S102514, 166, { &P22037,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3113 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2620192 = {S102515, 204, { &P13452,&P22212,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-P1000 Build/JZO54K; CyanogenMod-Sapir) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.10/221,1.9.10.221,2012-11-19 13:37, fr*/
const Device UA2623011 = {S102516, 208, { &P13452,&P22212,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; GT-P1000 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0100, nl*/
const Device UA2622218 = {S102517, 71, { &P17323,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.4;Zh_cn;GT-P6800;800*1232;)UCWEB7.8.0.95/139/444*/
const Device UA2634105 = {S102518, 166, { &P12879,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P7510 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/405*/
const Device UA2623920 = {S102519, 164, { &P17323,&P6437,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6800 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2628282 = {S102520, 52, { &P13452,&P17979,&P22080,&P18092 }};	/*Useragent - WhatsApp/2.9.369 Android/2.2 Device/samsung-GT-P1000*/
const Device UA2626447 = {S102521, 210, { &P15919,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; sv-se; GT-P3100 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +0100, se*/
const Device UA2629611 = {S102522, 210, { &P15919,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; pt-br; GT-P3100 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, -0200, br*/
const Device UA2632653 = {S102523, 209, { &P15919,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3100 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0530, in*/
const Device UA2627202 = {S102524, 147, { &P15919,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2631692 = {S102525, 147, { &P15919,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P3100 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2628151 = {S102526, 151, { &P15919,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2633441 = {S102527, 152, { &P15919,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3100B Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2633809 = {S102528, 151, { &P15919,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P3100 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2631724 = {S102529, 58, { &P22037,&P17979,&P22278,&P18092 }};	/*Useragent - theScore/3.1.3 Android-OS/4.1.1 (Android; Phone, GT-P3113)*/
const Device UA2665255 = {S102530, 102, { &P23534,&P22839,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; zte-p107a20(propus)) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2680960 = {S102531, 152, { &P22385,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2807691 = {S102532, 36, { &P15364,&P7127,&P4809,&P18092 }};	/*Useragent - Opera/5.02 (Windows NT 5.0; U; [en])*/
const Device UA2812733 = {S102533, 35, { &P15988,&P17484,&P21764,&P18092 }};	/*Useragent - iBrowser/Mini2.7 (SAMSUNG-GT-C3312)*/
const Device UA2865767 = {S102534, 90, { &P14532,&P17483,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t325) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2937848 = {S102535, 90, { &P14532,&P17483,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t325) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2945152 = {S102536, 91, { &P14763,&P5776,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian72) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2963053 = {S102537, 99, { &P15198,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3650) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2977061 = {S102538, 133, { &P24084,&P18110,&P19800,&P18092 }};	/*Useragent - Mozilla/5.0 (DTV) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.5 AQUOSBrowser/1.0 (AS00DTV;V;0001;0001) AQUOS-AS/2.0 LC-60LE640X*/
const Device UA2977119 = {S102539, 136, { &P24064,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-S7275 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3025161 = {S102540, 99, { &P15197,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-b5310) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3031107 = {S102541, 99, { &P14182,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5333) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3068862 = {S102542, 75, { &P24547,&P24371,&P24415,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 4.4; SPHS on Hsdroid Build/MocorDroid2.3.5)*/
const Device UA3068870 = {S102543, 151, { &P24547,&P24371,&P24415,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.4; en-us; SPHS on Hsdroid Build/MocorDroid2.3.5) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA3068874 = {S102544, 107, { &P24547,&P24371,&P24415,&P18092 }};	/*Useragent - com.google.android.youtube/4.5.17(Linux; U; Android 4.4; en_AU; SPHS on Hsdroid Build/MocorDroid2.3.5) gzip*/
const Device UA3068984 = {S102545, 108, { &P20273,&P18148,&P22782,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows Phone 8.0; Trident/7.0; rv:11.0; IEMobile/11.0; ARM; Touch; HTC; HTC6990LVW) like Gecko*/
const Device UA91 = {S102546, 71, { &P20254,&P5448,&P3430,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Opera/9.5) HTC-MP6950 BM*/
const Device UA4255 = {S102547, 128, { &P16169,&P5448,&P22271,&P18092 }};	/*Useragent - PPC; 240x320; HTC_P4351/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA5048 = {S102548, 87, { &P13338,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) acer_M900*/
const Device UA6442 = {S102549, 107, { &P12464,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-T809/T809UVEJ9 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA32465 = {S102550, 62, { &P11836,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW960i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA55879 = {S102551, 75, { &P11754,&P17485,&P17470,&P18092 }};	/*Useragent - RSSFeeds/R1A SonyEricssonP990i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA14282 = {S102552, 136, { &P17953,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; de-at; RBM2 Build/CRB17) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA16319 = {S102553, 161, { &P11906,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-GT-I8000U/1.0 Software/WM6.5 Release/01.24.2009 Profile/MIDP-2.1 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA17075 = {S102554, 143, { &P17769,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; zh-cn; dopod A3288 Build/DRC79) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA89693 = {S102555, 144, { &P17769,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; zh-cn; dopod A3288 Build/DRC79) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA17216 = {S102556, 145, { &P11906,&P5448,&P3430,&P18092 }};	/*Useragent - SAMSUNG-GT-I8000U/1.0 Software/WM6.5 Release/01.24.2009 Profile/MIDP-2.1 Configuration/CLDC-1.1 Opera 9.5 (Windows CE; PPC; Opera Mobi; U; zh-cn)*/
const Device UA22310 = {S102557, 42, { &P13240,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW890i/R1FA SEMC/NewsReader/1.0*/
const Device UA21217 = {S102558, 225, { &P15170,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTCA3380_TD/1.0 Android/2.2 release/9.1.2010 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA21632 = {S102559, 87, { &P13705,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-CX70/50 UP.Browser/7.0.2.2.d.3(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23492 = {S102560, 87, { &P13705,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-CX70/56 UP.Browser/7.0.2.2.d.4(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61633 = {S102561, 91, { &P13705,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-CX70/43 UP.Browser/7.0.2.2.d.1.100(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21654 = {S102562, 35, { &P15314,&P6476,&P4381,&P18092 }};	/*Useragent - Samsung-SPHM220 Polaris/5.0 MMP/2.0*/
const Device UA21896 = {S102563, 49, { &P15353,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-V750/1.0 UP.Browser/7.2.6.1.731 (GUI) MMP/2.0*/
const Device UA22217 = {S102564, 46, { &P15364,&P7472,&P3625,&P18092 }};	/*Useragent - Alcatel-TH4/1.0 UP.Browser/6.2.ALCATEL MMP/1.0*/
const Device UA22193 = {S102565, 98, { &P15396,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-P920/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA22613 = {S102566, 80, { &P14504,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonZ500a/R1A SEMC-Browser/4.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63345 = {S102567, 80, { &P14504,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonZ500a/R4A SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22625 = {S102568, 12, { &P15502,&P17483,&P17660,&P18092 }};	/*Useragent - LG/U450/v1.0*/
const Device UA22924 = {S102569, 68, { &P15530,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHI320/1.0 jBrowser/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22931 = {S102570, 23, { &P15531,&P19954,&P3816,&P18092 }};	/*Useragent - SEC-SGHD807/1.0 TSS/2.5*/
const Device UA23739 = {S102571, 56, { &P15623,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-R901_/00.62 UP.Browser/6.2.3.4.c.1.112 (GUI) MMP/2.0*/
const Device UA24775 = {S102572, 110, { &P15757,&P17484,&P4248,&P18092 }};	/*Useragent - SAMSUNG-GT-M8910B/1.0 SHP/VPP/R5 Jasmine/1.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA26171 = {S102573, 50, { &P13573,&P17483,&P3625,&P18092 }};	/*Useragent - LG5500-parrot/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA33093 = {S102574, 43, { &P13573,&P17483,&P3625,&P18092 }};	/*Useragent - LG5500/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA27968 = {S102575, 111, { &P12197,&P17483,&P3534,&P18092 }};	/*Useragent - LG-KF755/v10a Browser/Obigo-Q05A/3.11 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA28108 = {S102576, 70, { &P12220,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-SGH-i600/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA28162 = {S102577, 97, { &P12226,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Touch_Pro_T7272 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA30242 = {S102578, 157, { &P13668,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Ultimate Dream 5.0; en-us; AMS Dream 5.0 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32923 = {S102579, 52, { &P12793,&P17483,&P3625,&P18092 }};	/*Useragent - LG-LG6400/LG6400V00 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA32953 = {S102580, 71, { &P12804,&P22839,&P4924,&P18092 }};	/*Useragent - ZTE-F852/1.0 ACS-NF/3.2 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64349 = {S102581, 60, { &P11895,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia5100/1.0 (5.20) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA88201 = {S102582, 102, { &P12952,&P17486,&P17671,&P18092 }};	/*Useragent - MOT-EM25/1.0 Release/11.01.2008 Browser/CMCS1.0 Software/0.34K Profile/MIDP-2.0 Configuretion/CLDC-1.1*/
const Device UA49397 = {S102583, 45, { &P13010,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-S520/1.0 ObigoInternetBrowser/Q03C*/
const Device UA88740 = {S102584, 46, { &P13010,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-S520A/1.0 ObigoInternetBrowser/Q03C*/
const Device UA49498 = {S102585, 15, { &P14951,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHE310/1.0*/
const Device UA49679 = {S102586, 81, { &P13071,&P17017,&P17470,&P18092 }};	/*Useragent - NexianNX-G755/SPR Release/01.03.2011 Browser/Profile/MIDP2.0 Configuration/CLDC-1*/
const Device UA49824 = {S102587, 105, { &P13295,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonC905c/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3*/
const Device UA50105 = {S102588, 15, { &P13167,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHX210/1.0*/
const Device UA50138 = {S102589, 76, { &P13178,&P17017,&P3402,&P18092 }};	/*Useragent - BenQ-T51/1.0/WAP2.0/MIDP2.0/CLDC1.1 UP.Browser/6.3.0.4.c.1.105 (GUI) MMP/2.0*/
const Device UA83525 = {S102590, 17, { &P11710,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW300i*/
const Device UA50498 = {S102591, 100, { &P14763,&P5776,&P4737,&P18092 }};	/*Useragent - NokiaN72/5.0819.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.2.2.51/27/352*/
const Device UA50415 = {S102592, 36, { &P12088,&P5129,&P4737,&P18092 }};	/*Useragent - NokiaX6-00/UC Browser7.2.2.51/50/352*/
const Device UA51477 = {S102593, 78, { &P13370,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-M3200/M3200JCHJ1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA1844412 = {S102594, 29, { &P14358,&P5890,&P24644,&P18092 }};	/*Useragent - NokiaE63/UCWEB7.4.0.57/28/800*/
const Device UA57801 = {S102595, 81, { &P14113,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK300a/R2AT SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA67889 = {S102596, 60, { &P12513,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA61020 = {S102597, 45, { &P12613,&P17484,&P4223,&P18092 }};	/*Useragent - Samsung-SPHM560 Access-NetFront/3.5.1 MMP/2.0*/
const Device UA61173 = {S102598, 54, { &P24376,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E330N/1.0 UP.Browser/6.2.2.6 (GUI) MMP/1.0*/
const Device UA62139 = {S102599, 75, { &P14374,&P19949,&P4924,&P18092 }};	/*Useragent - NEC-e373/001.00 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1 Qtv/1.00*/
const Device UA84237 = {S102600, 77, { &P14374,&P19949,&P4924,&P18092 }};	/*Useragent - NEC-e373/001.00 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Qtv/1.00*/
const Device UA62490 = {S102601, 47, { &P14414,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-VX8700/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA63787 = {S102602, 124, { &P14541,&P17017,&P4223,&P18092 }};	/*Useragent - CoolPadF608/CMCC ThreadX/1.0 ArenaA2KPH/SW2.0 Release/2009.10.01 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA63915 = {S102603, 84, { &P17798,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-P80/CMCC_S101 LMP/XM Release/2008.08.26 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA64395 = {S102604, 57, { &P16646,&P19950,&P3625,&P18092 }};	/*Useragent - SAGEM-myC5-3/1.0 UP.Browser/6.2.3.3.g.2.107 (GUI) MMP/2.0*/
const Device UA64631 = {S102605, 45, { &P14688,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-S320/1.0 ObigoInternetBrowser/Q03C*/
const Device UA88424 = {S102606, 58, { &P14688,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-S320/1.0 model-orange ObigoInternetBrowser/Q03C*/
const Device UA67884 = {S102607, 30, { &P14188,&P5129,&P24660,&P18092 }};	/*Useragent - Nokia5233/UCWEB7.6.0.75/50/800*/
const Device UA68184 = {S102608, 110, { &P14345,&P16000,&P4759,&P18092 }};	/*Useragent - NokiaN-GageQD/1.0 SymbianOS/6.1 Series60/1.2 Profile/MIDP-1.0 Configuration/CLDC-1.0/UC Browser7.6.1.82/26/401*/
const Device UA68346 = {S102609, 8, { &P11821,&P17483,&P17470,&P18092 }};	/*Useragent - lg-gb270*/
const Device UA69172 = {S102610, 34, { &P13163,&P5129,&P4779,&P18092 }};	/*Useragent - NokiaN97/UC Browser7.8.0.95/50/351*/
const Device UA69512 = {S102611, 30, { &P15463,&P17483,&P4628,&P18092 }};	/*Useragent - LGE-LG600 POLARIS/6.01 MMP/2.0*/
const Device UA69901 = {S102612, 62, { &P17922,&P5448,&P4578,&P18092 }};	/*Useragent - DopodD600/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)*/
const Device UA87416 = {S102613, 121, { &P17922,&P5448,&P4578,&P18092 }};	/*Useragent - dopod D600_CMCC/5.1.422/WAP1.2 Profile/MIDP2.0 Configuration/CLDC1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)*/
const Device UA78253 = {S102614, 148, { &P13452,&P5096,&P20775,&P18092 }};	/*Useragent - MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2622153 = {S102615, 149, { &P13452,&P5096,&P20775,&P18092 }};	/*Useragent - MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.2; th-th; GT-P1000T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA81558 = {S102616, 29, { &P15364,&P19954,&P3816,&P18092 }};	/*Useragent - SEC-SGHD357, TSS/2.5, Rev 1.4*/
const Device UA81713 = {S102617, 23, { &P15364,&P19954,&P3816,&P18092 }};	/*Useragent - SEC-SGHX686/1.0 TSS/2.5*/
const Device UA81555 = {S102618, 28, { &P15364,&P19954,&P3816,&P18092 }};	/*Useragent - SEC-SGHD307, TSS/2.5, REV0.6*/
const Device UA86663 = {S102619, 42, { &P19914,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-S259/ UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA87686 = {S102620, 41, { &P19914,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-S259 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA87473 = {S102621, 43, { &P15570,&P6476,&P4203,&P18092 }};	/*Useragent - Samsung-SPHM610 AU-OBIGO/Q04C1-1.17 MMP/2.0*/
const Device UA87632 = {S102622, 86, { &P17661,&P17662,&P3976,&P18092 }};	/*Useragent - SoftBank/1.0/920SH/SHJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88017 = {S102623, 86, { &P17661,&P17662,&P3976,&P18092 }};	/*Useragent - SoftBank/1.0/921SH/SHJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88013 = {S102624, 86, { &P17657,&P17662,&P3976,&P18092 }};	/*Useragent - SoftBank/1.0/820SH/SHJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88944 = {S102625, 42, { &P17657,&P17662,&P3976,&P18092 }};	/*Useragent - Mozilla/4.08 (820SH;SoftBank) NetFront/3.4*/
const Device UA507002 = {S102626, 54, { &P17874,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-TS3T UP.Browser/6.2_7.2.7.1.K.4.306 (GUI) MMP/2.0*/
const Device UA347738 = {S102627, 114, { &P15767,&P17146,&P22246,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.7.1*/
const Device UA366197 = {S102628, 114, { &P15767,&P17146,&P22246,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Flipboard/1.7.2*/
const Device UA584696 = {S102629, 112, { &P15767,&P17146,&P22246,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A334 Flipboard/1.7.2*/
const Device UA351273 = {S102630, 121, { &P12280,&P17146,&P20778,&P18092 }};	/*Useragent - MQQBrowser/27 Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405*/
const Device UA368805 = {S102631, 158, { &P22393,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; ZTE-C_V9E/FRY_V9EV1.0.0B02; 240*400; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369201 = {S102632, 158, { &P15798,&P5629,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC Explorer A310e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021325 = {S102633, 156, { &P13123,&P5629,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370997 = {S102634, 158, { &P16103,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; fr-ca; XT720 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA419363 = {S102635, 19, { &P17968,&P17979,&P4459,&P18092 }};	/*Useragent - Android/3.1-EAS-1.2*/
const Device UA436853 = {S102636, 127, { &P15767,&P17141,&P22247,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B500 Flipboard/1.5.1*/
const Device UA1917567 = {S102637, 127, { &P15767,&P17141,&P22247,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B500 Flipboard/1.5.3*/
const Device UA459253 = {S102638, 37, { &P15988,&P17484,&P23401,&P18092 }};	/*Useragent - MQQBrowser/Mini2.5 (SAMSUNG-GT-C3312)*/
const Device UA520979 = {S102639, 86, { &P17634,&P5567,&P4536,&P18092 }};	/*Useragent - Mozilla/5.0 Opera/9.5 (KDDI-SH3G; BREW; Opera Mobi; U; ja) Presto/2.4.15 Version/10.00*/
const Device UA531202 = {S102640, 123, { &P17595,&P18120,&P22520,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-us) AppleWebKit/531+ (KHTML, like Gecko) Version/5.0 Safari/531.2+ Midori/0.3*/
const Device UA563026 = {S102641, 152, { &P22392,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-2.0.4; es-es; WORDM902R Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA568720 = {S102642, 36, { &P15118,&P5055,&P4777,&P18092 }};	/*Useragent - NokiaX7-00/UC Browser7.7.1.88/50/444*/
const Device UA569111 = {S102643, 56, { &P13977,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (NokiaN73(08);SymbianOS/9.1 Series60/3.0)*/
const Device UA570033 = {S102644, 56, { &P13977,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (NokiaN73(12);SymbianOS/9.1 Series60/3.0)*/
const Device UA608905 = {S102645, 134, { &P17283,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; MB860 Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA721114 = {S102646, 32, { &P14033,&P5129,&P24666,&P18092 }};	/*Useragent - NokiaC5-03/UCWEB8.1.0.104/50/800*/
const Device UA814612 = {S102647, 149, { &P16356,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-N7000 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA738363 = {S102648, 149, { &P18348,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; HTC Velocity 4G Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA758250 = {S102649, 148, { &P17172,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Transformer TF101 Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA804731 = {S102650, 171, { &P14877,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us ; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA902950 = {S102651, 101, { &P15364,&P22192,&P18211,&P18092 }};	/*Useragent - GoogleEarth/5.2.1.1588(Windows;Microsoft Windows (6.0.6002.2);en-US;kml:2.2;client:Free;type:default)*/
const Device UA1846386 = {S102652, 98, { &P15364,&P22192,&P18211,&P18092 }};	/*Useragent - GoogleEarth/5.2.1.1588(Windows;Microsoft Windows (6.0.6002.2);en;kml:2.2;client:Free;type:default)*/
const Device UA925749 = {S102653, 113, { &P14886,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; USCCHTC-PC93100; USC-US)*/
const Device UA925832 = {S102654, 101, { &P14886,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; 7 Pro T7576)*/
const Device UA1861174 = {S102655, 130, { &P23071,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; zh-cn; T2 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1884681 = {S102656, 63, { &P14877,&P18110,&P24661,&P18092 }};	/*Useragent - JUC(Linux;U;2.2.1;Zh_cn;GT-I9000;480*800;)UCWEB7.7.0.85/139/800*/
const Device UA1886194 = {S102657, 149, { &P22241,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; iView_1010NB Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA1898864 = {S102658, 54, { &P11907,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (NokiaE52-1;SymbianOS/9.1 Series60/3.0)*/
const Device UA1901863 = {S102659, 174, { &P11747,&P5096,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC_Desire Build/FRF91; 480*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/800*/
const Device UA1902690 = {S102660, 53, { &P15373,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (Nokia5530;SymbianOS/9.1 Series60/3.0)*/
const Device UA1907112 = {S102661, 131, { &P13633,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; A953 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1938026 = {S102662, 96, { &P12989,&P5843,&P17338,&P18092 }};	/*Useragent - Nokia303/5.0 (13.44) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia303/UC Browser7.9.0.102/70/355*/
const Device UA1957662 = {S102663, 169, { &P14993,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LT15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA1967999 = {S102664, 160, { &P16957,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; XCD35 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2019782 = {S102665, 132, { &P22348,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; ru-ru; iDx7 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020220 = {S102666, 78, { &P12657,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.4;Zh_cn;HTC EVO 3D X515m;540*960;)UCWEB7.8.0.95/139/355*/
const Device UA2022424 = {S102667, 134, { &P22238,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; pl-pl; TAB_I70 Build/FRF85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023951 = {S102668, 165, { &P18687,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; LG-P705 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2024001 = {S102669, 29, { &P16553,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (SGH-T959V FROYO)*/
const Device UA2026231 = {S102670, 156, { &P16356,&P5629,&P20776,&P18092 }};	/*Useragent - MQQBrowser/25 Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i889 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026467 = {S102671, 164, { &P14123,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P970 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2031837 = {S102672, 160, { &P13505,&P5629,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; ME525+ Build/4.5.3-109_DPP-11) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2032729 = {S102673, 157, { &P15991,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2144791 = {S102674, 78, { &P15364,&P5690,&P24204,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) QQBrowser/6.3.8908.201*/
const Device UA2299443 = {S102675, 123, { &P15364,&P5690,&P24204,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.3.8908.201*/
const Device UA2168117 = {S102676, 81, { &P13648,&P5129,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; NokiaE7-00)/UC Browser8.5.0.183/50/352/UCWEB Mobile*/
const Device UA2219200 = {S102677, 142, { &P13107,&P5043,&P24670,&P18092 }};	/*Useragent - NokiaC2-06/2.0 (07.48) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac2-06) UCBrowser8.4.0.159/70/352/UCWEB Mobile*/
const Device UA2299663 = {S102678, 126, { &P23679,&P24094,&P17217,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; InettvBrowser/2.2 (00014A;SonyDTV115;0002;0100) KDL40NX723; CC/GBR; en) Presto/2.7.61 Version/11.00*/
const Device UA2304770 = {S102679, 132, { &P12280,&P17090,&P20777,&P18092 }};	/*Useragent - MQQBrowser/261 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1*/
const Device UA2309263 = {S102680, 132, { &P12280,&P17090,&P20777,&P18092 }};	/*Useragent - MQQBrowser/261 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2*/
const Device UA2310966 = {S102681, 47, { &P11854,&P5890,&P21754,&P18092 }};	/*Useragent - iBrowser/2.4 (E71-1;SymbianOS/9.1 Series60/3.0)*/
const Device UA2320457 = {S102682, 140, { &P12280,&P17146,&P18356,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3 Sleipnir/1.9.2m*/
const Device UA2336134 = {S102683, 80, { &P12487,&P5890,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIA5700)/UC Browser8.5.0.183/28/352/UCWEB Mobile*/
const Device UA2342787 = {S102684, 35, { &P15228,&P5043,&P21759,&P18092 }};	/*Useragent - iBrowser/Mini2.8 (NokiaC2-03/06.96)*/
const Device UA2360658 = {S102685, 138, { &P18644,&P5043,&P24670,&P18092 }};	/*Useragent - Nokia112/2.0 (03.26) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia112) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2431311 = {S102686, 98, { &P14182,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s5333) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2506960 = {S102687, 98, { &P14985,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s8530) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2595938 = {S102688, 98, { &P12260,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s5350) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2608723 = {S102689, 92, { &P12879,&P6437,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.3.10.10337); (Linux; U; Android 3.2; GT-P7510 Build/HONEYCOMB_MR2);*/
const Device UA2609031 = {S102690, 144, { &P15985,&P6437,&P20775,&P18092 }};	/*Useragent - MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 3.2; vi-vn; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2615073 = {S102691, 164, { &P12879,&P6437,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7510 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2624299 = {S102692, 151, { &P14908,&P5096,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 2.2.1; en-ie; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2618248 = {S102693, 104, { &P15876,&P17979,&P22070,&P18092 }};	/*Useragent - Instagram 3.0.5 Android (15/4.0.3; 160dpi; 800x1232; samsung; GT-P5100; espresso10rf; espresso10; en_GB)*/
const Device UA2616360 = {S102694, 166, { &P17337,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P7500 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2621355 = {S102695, 166, { &P17323,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6800 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/352*/
const Device UA2617006 = {S102696, 197, { &P15876,&P18066,&P17374,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fa-fa; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (X11; Linux zvav; U; fa) Presto/2.8.119 Version/11.10*/
const Device UA2619650 = {S102697, 197, { &P15876,&P18066,&P17374,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2618271 = {S102698, 80, { &P12879,&P17979,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (Android Live Scores; 2.1.3) (Android; GT-P7510; Android OS/3.1)*/
const Device UA2623354 = {S102699, 85, { &P12879,&P17979,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (Android Sportacular; 71-3.0.1) (Android; GT-P7510; Android OS/4.0.4)*/
const Device UA2633110 = {S102700, 83, { &P12879,&P17979,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (Android Sportacular; 77-3.0.7) (Android; GT-P7510; Android OS/3.2)*/
const Device UA2620610 = {S102701, 166, { &P13452,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P1000 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355*/
const Device UA2622122 = {S102702, 166, { &P13452,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P1000 Build/IMM76L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2622818 = {S102703, 136, { &P22043,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2625016 = {S102704, 136, { &P22043,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5113 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7*/
const Device UA2623207 = {S102705, 101, { &P15932,&P17979,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.0 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P3110; espressowifi; espresso; en_GB)*/
const Device UA2633285 = {S102706, 192, { &P20149,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P6211 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0100, de*/
const Device UA2625852 = {S102707, 190, { &P17168,&P6437,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7310 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +1300, nz*/
const Device UA2627750 = {S102708, 152, { &P15876,&P18066,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2637780 = {S102709, 152, { &P15876,&P18066,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2628431 = {S102710, 130, { &P22037,&P17979,&P3508,&P18092 }};	/*Useragent - Orca-Android/2.2.5-release OS/4.1.1 Model/GT-P3113 VersionCode/X Product/Messenger Carrier/null Manufacturer/samsung Brand/samsung*/
const Device UA2628410 = {S102711, 85, { &P22037,&P17979,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (Android Sportacular; 76-3.0.6) (Android; GT-P3113; Android OS/4.1.1)*/
const Device UA2632611 = {S102712, 156, { &P12889,&P6437,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13 Maxthon/4.0.3.3000*/
const Device UA2635890 = {S102713, 149, { &P12889,&P6437,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.3.3000*/
const Device UA2635528 = {S102714, 156, { &P17323,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P6800 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2633992 = {S102715, 163, { &P12889,&P6546,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7300 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2647624 = {S102716, 173, { &P14183,&P5096,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.0.181/145/355*/
const Device UA2986215 = {S102717, 96, { &P12273,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; vi; samsung-gt-m5650) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2737567 = {S102718, 99, { &P14980,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-b7722) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2747895 = {S102719, 155, { &P14482,&P5843,&P24675,&P18092 }};	/*Useragent - NokiaC3-01/5.0 (06.05) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokiac3-01) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2764447 = {S102720, 99, { &P16971,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-sgh-a877) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2794949 = {S102721, 93, { &P11689,&P5890,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian70-1) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2808871 = {S102722, 54, { &P14033,&P5890,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0 (NokiaC5-03;SymbianOS/9.1 Series60/3.0)*/
const Device UA2810725 = {S102723, 131, { &P18632,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; AN10BG3 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2817107 = {S102724, 148, { &P14967,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.3; en-in; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2817724 = {S102725, 149, { &P22103,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HS-U8 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2960131 = {S102726, 91, { &P11996,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5560) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2961885 = {S102727, 99, { &P14182,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5333) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2977064 = {S102728, 86, { &P24087,&P18110,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv6l; U; en) Presto/2.8.115 Version/11.10 AQUOS-AS/1.0 LC-40LE835X*/
const Device UA3084110 = {S102729, 167, { &P14225,&P5629,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3093131 = {S102730, 91, { &P16024,&P17483,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t310i) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA15418 = {S102731, 105, { &P11815,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonU100i/R1BA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.0*/
const Device UA2041 = {S102732, 66, { &P16902,&P17017,&P17470,&P18092 }};	/*Useragent - VertuAscentTi/2.0 (500.10) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA50971 = {S102733, 66, { &P16902,&P17017,&P17470,&P18092 }};	/*Useragent - VertuAscentTi/2.0 (501.01) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA50970 = {S102734, 66, { &P16902,&P17017,&P17470,&P18092 }};	/*Useragent - VertuAscentTi/2.0 (500.03) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA56910 = {S102735, 66, { &P16902,&P17017,&P17470,&P18092 }};	/*Useragent - VertuAscentTi/2.0 (501.02) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA3239 = {S102736, 105, { &P17912,&P17979,&P3525,&P18092 }};	/*Useragent - dopodA6288/1.0 Android/1.5/1.04 release/5.17.2009 Browser/WAP 2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA8897 = {S102737, 79, { &P16676,&P5222,&P17183,&P18092 }};	/*Useragent - acer_E100 Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5)*/
const Device UA9889 = {S102738, 84, { &P14345,&P16000,&P4030,&P18092 }};	/*Useragent - NokiaN-GageQD/1.0 SymbianOS/6.1 Series60/0.9 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA61887 = {S102739, 89, { &P14345,&P16000,&P4030,&P18092 }};	/*Useragent - NokiaN-Gage/1.0 (4.03) SymbianOS/6.1 Series60/0.9 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA10102 = {S102740, 113, { &P13468,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-S3030/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA13659 = {S102741, 15, { &P17827,&P17811,&P17810,&P18092 }};	/*Useragent - LENOVO-i919/66R*/
const Device UA15508 = {S102742, 103, { &P14862,&P5204,&P4715,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.97) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2700c-2/UC Browser7.4.0.65/70/352*/
const Device UA16935 = {S102743, 81, { &P20110,&P5448,&P22271,&P18092 }};	/*Useragent - SAMSUNG-SGH-i608/1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA54598 = {S102744, 68, { &P14243,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-C390/0B.A0.0FR MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA20441 = {S102745, 71, { &P15091,&P22839,&P4924,&P18092 }};	/*Useragent - ZTE-F152/1.0 ACS-NF/3.2 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21511 = {S102746, 30, { &P15292,&P5282,&P3579,&P18092 }};	/*Useragent - ZTE-C79/1.0 SMIT-Browser/2.0.0*/
const Device UA21735 = {S102747, 78, { &P16253,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) O2 Xda Atom Life*/
const Device UA21820 = {S102748, 82, { &P15348,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-TD36t/S100 LMP/LML Release/2010.08.25 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA22169 = {S102749, 37, { &P15391,&P17483,&P3866,&P18092 }};	/*Useragent - LG-KE800 MIC/1.1.14 MIDP-2.0/CLDC-1.1*/
const Device UA22334 = {S102750, 29, { &P15428,&P17017,&P17470,&P18092 }};	/*Useragent - portalmmm/2.0 K550im(c100;TB)*/
const Device UA22413 = {S102751, 39, { &P15459,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KM385 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA22508 = {S102752, 127, { &P12036,&P17017,&P4893,&P18092 }};	/*Useragent - SIE-E71/39 UP.Browser/7.1.0.k.5(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.k.5 (GUI) MMP/2.0*/
const Device UA49815 = {S102753, 127, { &P12036,&P17017,&P4893,&P18092 }};	/*Useragent - SIE-E71/37 UP.Browser/7.1.0.k.4(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.k.4 (GUI) MMP/2.0*/
const Device UA36160 = {S102754, 92, { &P11728,&P5033,&P4948,&P18092 }};	/*Useragent - BlackBerry8820/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100 UP.Browser/5.0.3.3*/
const Device UA22601 = {S102755, 39, { &P15514,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KU730 Obigo/WAP2.0 MIDP 2.0/CLDC 1.1*/
const Device UA22992 = {S102756, 55, { &P15543,&P17484,&P4901,&P18092 }};	/*Useragent - SAMSUNG-SGH-X100/PEARL UP.Browser/6.1.0.6 (GUI) MMP/1.0*/
const Device UA23180 = {S102757, 101, { &P15571,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-X550/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA23354 = {S102758, 41, { &P15598,&P22832,&P17470,&P18092 }};	/*Useragent - Videocon_V1430_V1_0_3_Wap browser ver 2.0*/
const Device UA24068 = {S102759, 78, { &P15659,&P5448,&P4578,&P18092 }};	/*Useragent - HPiPAQhw692x/1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x240)*/
const Device UA24096 = {S102760, 56, { &P15668,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-PARS_/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0*/
const Device UA24324 = {S102761, 11, { &P15680,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHE600*/
const Device UA33020 = {S102762, 12, { &P15680,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHE600C*/
const Device UA24337 = {S102763, 70, { &P12097,&P5448,&P22272,&P18092 }};	/*Useragent - HTC_P4350 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA24630 = {S102764, 37, { &P15725,&P17483,&P3866,&P18092 }};	/*Useragent - LG-ME770 MIC/1.1.14 MIDP-2.0/CLDC-1.1*/
const Device UA33018 = {S102765, 38, { &P15725,&P17483,&P3866,&P18092 }};	/*Useragent - LG-ME770c MIC/1.1.14 MIDP-2.0/CLDC-1.1*/
const Device UA26690 = {S102766, 90, { &P13620,&P16000,&P3772,&P18092 }};	/*Useragent - SonyEricssonM608c/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 513) Opera 8.65 [en]*/
const Device UA55586 = {S102767, 90, { &P13620,&P16000,&P3772,&P18092 }};	/*Useragent - SonyEricssonM608c/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 306) Opera 8.60 [en]*/
const Device UA26701 = {S102768, 51, { &P13621,&P17017,&P17470,&P18092 }};	/*Useragent - MTC535/REF-RU-MTS-P103D2V1.0.1/2.0 Profile/MIDP-2.0*/
const Device UA27978 = {S102769, 100, { &P12204,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-A746/UXHD2 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA28027 = {S102770, 104, { &P12211,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-J638/J638ZTHF2 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA28381 = {S102771, 106, { &P12230,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-G808/1.0 SHP/VPP/R5 NetFront/3.4 Qtv/5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA28434 = {S102772, 106, { &P12242,&P5448,&P4808,&P18092 }};	/*Useragent - garmin-asus-Nuvifone-M20/1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA28435 = {S102773, 118, { &P12242,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) garmin-asus-Nuvifone-M20/1.0 Mozilla/4.0*/
const Device UA30784 = {S102774, 98, { &P20116,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-S366/ Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA33117 = {S102775, 65, { &P12856,&P17483,&P17602,&P18092 }};	/*Useragent - LG-T300/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA33371 = {S102776, 53, { &P14910,&P17483,&P17660,&P18092 }};	/*Useragent - LG/L600V/v1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA33392 = {S102777, 86, { &P12911,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-G272/MTK Release/01.01.2011 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1*/
const Device UA69364 = {S102778, 97, { &P12108,&P5843,&P4779,&P18092 }};	/*Useragent - Nokia7230/5.0 (06.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7230/UC Browser7.8.0.95/70/351*/
const Device UA39911 = {S102779, 80, { &P20272,&P5448,&P22303,&P18092 }};	/*Useragent - HTC-3100/1.2 Mozilla/4.0 (compatible; MSIE 5.5; Windows CE; Smartphone; 240x320)*/
const Device UA39915 = {S102780, 71, { &P17768,&P5448,&P4578,&P18092 }};	/*Useragent - DopodD810/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)*/
const Device UA40105 = {S102781, 132, { &P17283,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; MB860 Build/4.5.91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA44593 = {S102782, 97, { &P12059,&P5204,&P4777,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2690/UC Browser7.7.1.88/70/351*/
const Device UA68518 = {S102783, 97, { &P12108,&P5843,&P4777,&P18092 }};	/*Useragent - Nokia7230/5.0 (06.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7230/UC Browser7.7.1.88/70/351*/
const Device UA45103 = {S102784, 68, { &P16190,&P5448,&P17173,&P18092 }};	/*Useragent - Xda_trion; 240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA45206 = {S102785, 86, { &P12638,&P5448,&P17183,&P18092 }};	/*Useragent - SAMSUNG-SGH-i780/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA45226 = {S102786, 83, { &P13859,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-i325wg/S120 LMP/LML Release/2010.08.16 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA62281 = {S102787, 83, { &P13859,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-i325wg/S118 LMP/LML Release/2010.07.12 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA46697 = {S102788, 44, { &P17916,&P17017,&P17470,&P18092 }};	/*Useragent - dopodF3188 Mozilla/5.0 (like Gecko) Obigo/Q7*/
const Device UA49023 = {S102789, 98, { &P13886,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-A706/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA49453 = {S102790, 136, { &P12937,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; LG-LS700 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA50158 = {S102791, 78, { &P13182,&P17017,&P3402,&P18092 }};	/*Useragent - BenQ-EF71/1.00/WAP2.0/MIDP2.0/CLDC1.0 UP.Browser/6.3.0.7.c.1.105 (GUI) MMP/2.0*/
const Device UA68565 = {S102792, 78, { &P13182,&P17017,&P3402,&P18092 }};	/*Useragent - BenQ-EF71/1.00/WAP2.0/MIDP2.0/CLDC1.0 UP.Browser/6.3.0.7.c.1.103 (GUI) MMP/2.0*/
const Device UA51700 = {S102793, 236, { &P13954,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; zh-cn; dopodA8188_TD/1.0 Android/Dount release/12.3.2009 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/DRD20) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA52035 = {S102794, 74, { &P13399,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-F250/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA53459 = {S102795, 25, { &P13980,&P17017,&P17470,&P18092 }};	/*Useragent - R830/2.0 Profile/MIDP-2.0*/
const Device UA53549 = {S102796, 137, { &P13675,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-tw; Vibo A699 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA63389 = {S102797, 69, { &P12038,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-E1000/80.3F.36. MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54213 = {S102798, 107, { &P14152,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-A227/A227UCHD1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA54281 = {S102799, 146, { &P14166,&P5448,&P17183,&P18092 }};	/*Useragent - SonyEricssonX2a/R3AA Browser/Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA54405 = {S102800, 139, { &P14200,&P17605,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-I6330C_CMCC/1.0 RTK-E/1.0 SHP/VPP-R5 Release/5.25.2009 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1 SMM-MMS/1.2.0*/
const Device UA54868 = {S102801, 76, { &P14268,&P17017,&P3680,&P18092 }};	/*Useragent - WellcomW3112/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA56326 = {S102802, 81, { &P11911,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonZ530i/R6DA Java/SEMC-Java/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA59284 = {S102803, 119, { &P13714,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) 320x320; VZW; UTStar-SMT5800; Window Mobile 6.0 Standard;*/
const Device UA60539 = {S102804, 74, { &P12540,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-U900L/1.0 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA60592 = {S102805, 83, { &P12551,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-P650WG/S100 LMP/LML Release/2010.02.22 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA61018 = {S102806, 70, { &P13421,&P5448,&P22272,&P18092 }};	/*Useragent - HTC_P5530 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA61332 = {S102807, 86, { &P12682,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) MP6950SP*/
const Device UA61335 = {S102808, 131, { &P12684,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; A14 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2023355 = {S102809, 139, { &P12684,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-tw; A14 Build/ERD79) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA61368 = {S102810, 60, { &P20080,&P5043,&P3625,&P18092 }};	/*Useragent - Nokia 2605 UP.Browser/6.2.3.9.j.1.107-20081119 (GUI) MMP/2.0*/
const Device UA61512 = {S102811, 71, { &P12737,&P22839,&P4924,&P18092 }};	/*Useragent - ZTE-F850/1.0 ACS-NF/3.2 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61567 = {S102812, 65, { &P15284,&P17483,&P17470,&P18092 }};	/*Useragent - LG-GT810H Netfront/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62554 = {S102813, 47, { &P16613,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-VX5300/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA2278428 = {S102814, 29, { &P11854,&P5890,&P24660,&P18092 }};	/*Useragent - NokiaE71/UCWEB7.6.0.75/28/800*/
const Device UA67776 = {S102815, 37, { &P13552,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaC6-00/SymbianOS/9.1 Series60/3.0*/
const Device UA67883 = {S102816, 34, { &P12061,&P5890,&P4759,&P18092 }};	/*Useragent - NOKIAE65/UC Browser7.6.1.82/28/352*/
const Device UA68037 = {S102817, 12, { &P15200,&P17484,&P17606,&P18092 }};	/*Useragent - SamsungI8510*/
const Device UA68580 = {S102818, 35, { &P14941,&P17017,&P17470,&P18092 }};	/*Useragent - BenQ-C36/1.0/WAP2.0/MIDP2.0/CLDC1.1*/
const Device UA68989 = {S102819, 30, { &P14188,&P5129,&P24662,&P18092 }};	/*Useragent - Nokia5233/UCWEB7.8.0.87/50/800*/
const Device UA2287472 = {S102820, 31, { &P13552,&P5129,&P24662,&P18092 }};	/*Useragent - NokiaC6-00/UCWEB7.8.0.87/50/999*/
const Device UA1917522 = {S102821, 98, { &P16503,&P5204,&P17338,&P18092 }};	/*Useragent - Nokia7310/2.0 (09.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7310/UC Browser7.9.0.102/70/355*/
const Device UA80279 = {S102822, 60, { &P15864,&P6208,&P4420,&P18092 }};	/*Useragent - BlackBerry7730/4.0.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA81590 = {S102823, 68, { &P17222,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHE790/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87471 = {S102824, 111, { &P17100,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHB510/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA88214 = {S102825, 36, { &P17747,&P17017,&P17470,&P18092 }};	/*Useragent - Mozilla/5.0 (SH906i;FOMA;like Gecko)*/
const Device UA88873 = {S102826, 47, { &P13030,&P5448,&P3430,&P18092 }};	/*Useragent - HTC_P3701 Opera/9.50 (Windows NT 5.1; U; es-ES)*/
const Device UA89156 = {S102827, 93, { &P18490,&P22840,&P4780,&P18092 }};	/*Useragent - MyPhoneQ22Duo/MTK Release/V1 Nov-20-2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA89688 = {S102828, 97, { &P18490,&P22840,&P4780,&P18092 }};	/*Useragent - MyPhoneQ22DUO/MTK Release/V4BSP18 NOV-9-2009 Browser/MAUI Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA90261 = {S102829, 88, { &P13398,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SAMSUNG_B7610/1.0; Windows Phone 6.5)*/
const Device UA350481 = {S102830, 140, { &P15767,&P17090,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Safari/7534.48.3*/
const Device UA360853 = {S102831, 140, { &P15767,&P17090,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Safari/7534.48.3*/
const Device UA371268 = {S102832, 32, { &P15397,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (ST18i 4.0.2.A.0.42)*/
const Device UA373869 = {S102833, 48, { &P15364,&P17017,&P18407,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; ; ) Opera 9.7*/
const Device UA377081 = {S102834, 160, { &P15364,&P7133,&P21927,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729) chromeframe/7.0.517.44*/
const Device UA2266683 = {S102835, 86, { &P15364,&P7133,&P21927,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0) chromeframe/7.0.517.43*/
const Device UA399006 = {S102836, 141, { &P14574,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Nexus S Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75 Mobile Safari/535.7*/
const Device UA724322 = {S102837, 141, { &P14574,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75 Mobile Safari/535.7*/
const Device UA476652 = {S102838, 176, { &P17951,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Itfunz2.1_Eclipse_Terminator_II; zh-cn; Milestone Build/SHOLS_U2_02.36.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018209 = {S102839, 182, { &P17951,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Itfunz2.1_Eclipse_Terminator_II_Final; zh-cn; Milestone Build/SHOLS_U2_02.36.0) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA514541 = {S102840, 49, { &P15364,&P21442,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.01; Windows 2000)*/
const Device UA2324018 = {S102841, 49, { &P15364,&P21442,&P4551,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.00; Windows 2000)*/
const Device UA522038 = {S102842, 117, { &P15364,&P7127,&P17242,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/533.4 (KHTML, like Gecko) Chrome/5.0.375.127 Safari/533.4*/
const Device UA573430 = {S102843, 47, { &P12375,&P5890,&P24663,&P18092 }};	/*Useragent - NOKIANokia N81/UC Browser7.9.0.102/28/355/UCWEB*/
const Device UA616954 = {S102844, 101, { &P12280,&P17146,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]*/
const Device UA618253 = {S102845, 99, { &P12280,&P17146,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_0 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]*/
const Device UA670412 = {S102846, 43, { &P13677,&P17484,&P17347,&P18092 }};	/*Useragent - samsung-gt-s5253/UC Browser8.0.3.107/70/352*/
const Device UA1906586 = {S102847, 76, { &P17595,&P22328,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookPro6%2C2)*/
const Device UA2301399 = {S102848, 76, { &P17595,&P22328,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs/1 CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookAir3%2C2)*/
const Device UA694224 = {S102849, 54, { &P13552,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (NokiaC6-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA705610 = {S102850, 173, { &P12444,&P5477,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us ; U20i Build/2.1.1.A.0.6) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA755370 = {S102851, 165, { &P14433,&P5096,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA805358 = {S102852, 37, { &P14826,&P5843,&P23401,&P18092 }};	/*Useragent - MQQBrowser/Mini2.5 (NokiaC1-01/04.40)*/
const Device UA911661 = {S102853, 97, { &P15364,&P16110,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/20.0.1132.47)*/
const Device UA1988786 = {S102854, 90, { &P15364,&P16110,&P21910,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; chromeframe/20.0.1132.57)*/
const Device UA925843 = {S102855, 95, { &P17669,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; T7575)*/
const Device UA1845573 = {S102856, 86, { &P16958,&P17131,&P24643,&P18092 }};	/*Useragent - Palm680/RC1 (iPhone; U; CPU iPhone OS 2_2_1 like Mac OS X; zh-cn)/UCWEB7.2.0.46/39/997*/
const Device UA1852166 = {S102857, 10, { &P15364,&P17017,&P22484,&P18092 }};	/*Useragent - Opera/9.15*/
const Device UA1883445 = {S102858, 10, { &P15364,&P17017,&P22484,&P18092 }};	/*Useragent - Opera/9.12*/
const Device UA1863607 = {S102859, 119, { &P15364,&P18346,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 4.10; en-US) AppleWebKit/534.13 (KHTML, like Gecko) Chrome/9.0.597.98 Safari/534.13*/
const Device UA1890269 = {S102860, 43, { &P11948,&P17485,&P4777,&P18092 }};	/*Useragent - sonyericssonk550i/UC Browser7.7.1.88/69/444*/
const Device UA1935012 = {S102861, 225, { &P17346,&P17146,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA1938834 = {S102862, 44, { &P13183,&P17485,&P17347,&P18092 }};	/*Useragent - sonyericssonk810i/UC Browser8.0.3.107/69/444*/
const Device UA1954541 = {S102863, 79, { &P15791,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-in; Nokia603)/UC Browser8.4.0.159/50/352/UCWEB Mobile*/
const Device UA1957328 = {S102864, 142, { &P14919,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Pyramid Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1961789 = {S102865, 32, { &P14033,&P5129,&P24668,&P18092 }};	/*Useragent - NokiaC5-03/UCWEB8.3.0.133/50/800*/
const Device UA1981054 = {S102866, 56, { &P23144,&P17017,&P24670,&P18092 }};	/*Useragent - Safari(U;iOS 5.1.1;Zh-cn;320*480;)/UCWEB8.4.1.169/42/997*/
const Device UA1994994 = {S102867, 171, { &P16436,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2023537 = {S102868, 157, { &P14186,&P5629,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2615399 = {S102869, 164, { &P17337,&P6437,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7500 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2023718 = {S102870, 148, { &P13974,&P5096,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5670 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024752 = {S102871, 154, { &P13974,&P5629,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5670 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025076 = {S102872, 164, { &P12706,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P690 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2026336 = {S102873, 165, { &P11747,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2027805 = {S102874, 155, { &P14877,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029223 = {S102875, 153, { &P13094,&P5629,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; Lenovo A60 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029485 = {S102876, 157, { &P21129,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; SCH-i929 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030136 = {S102877, 165, { &P12568,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; GT-B7510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2032734 = {S102878, 154, { &P16356,&P5629,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-N7000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033091 = {S102879, 169, { &P15079,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; MK16i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2168055 = {S102880, 136, { &P17346,&P18117,&P20786,&P18092 }};	/*Useragent - MQQBrowser/35 Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA2242087 = {S102881, 85, { &P18385,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Mail; 1.4.5) (grouper; asus; Nexus 7; 4.1.1/JRO03D)*/
const Device UA2242096 = {S102882, 85, { &P18385,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Mail; 1.4.5) (grouper; asus; Nexus 7; 4.1.2/JZO54K)*/
const Device UA2242100 = {S102883, 90, { &P18385,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Messenger; 1.8.2) (grouper; asus; Nexus 7; 4.1.2/JZO54K)*/
const Device UA2261191 = {S102884, 234, { &P12280,&P17084,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Mobile/8B117,Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C9 Safari/419.3*/
const Device UA2318079 = {S102885, 115, { &P15364,&P7127,&P17237,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/530.1 (KHTML, like Gecko) Chrome/2.0.166.1 Safari/530.1*/
const Device UA2323454 = {S102886, 64, { &P11684,&P5890,&P20777,&P18092 }};	/*Useragent - MQQBrowser/2.6 (Nokia5800XpressMusic;SymbianOS/9.1 Series60/3.0)*/
const Device UA2330127 = {S102887, 130, { &P15364,&P16110,&P22405,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.30 (KHTML, like Gecko) Comodo_Dragon/12.1.0.0 Chrome/12.0.742.91 Safari/534.30*/
const Device UA2374197 = {S102888, 60, { &P15364,&P6704,&P18331,&P18092 }};	/*Useragent - Opera/9.80 (Windows 98; U; en) Presto/2.10.229 Version/11.62*/
const Device UA2422737 = {S102889, 87, { &P23532,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; id; tcl-c616) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2843617 = {S102890, 87, { &P23532,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; id; tcl-c616) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2642810 = {S102891, 86, { &P20126,&P17485,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; sonyericssonk800i) UCBrowser8.2.1.144/70/352/UCWEB Mobile*/
const Device UA2555273 = {S102892, 89, { &P17332,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-c310) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2589513 = {S102893, 154, { &P11787,&P5204,&P24670,&P18092 }};	/*Useragent - Nokia3120classic/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia3120classic) UCBrowser8.4.0.159/70/352/UCWEB Mobile*/
const Device UA2599569 = {S102894, 92, { &P13847,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-lg900g) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2607488 = {S102895, 71, { &P14802,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;SAMSUNG;SGH-i917;7.10;8862)*/
const Device UA2607532 = {S102896, 149, { &P21792,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; GT-I9200 Build/4.5.1A-1_SUN-198_7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2608752 = {S102897, 136, { &P14908,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P6210 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2609500 = {S102898, 130, { &P15985,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2617101 = {S102899, 131, { &P15985,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; pt-br; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2609666 = {S102900, 59, { &P15919,&P17979,&P22697,&P18092 }};	/*Useragent - TwitterAndroid/1.0.5 (109) GT-P3100/15 (samsung;espressorf)*/
const Device UA2609899 = {S102901, 75, { &P17337,&P17979,&P22699,&P18092 }};	/*Useragent - TwitterAndroid/2.0.1 (122) GT-P7500/12 (samsung;GT-P7500;samsung;GT-P7500;)*/
const Device UA2623814 = {S102902, 161, { &P13452,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615596 = {S102903, 29, { &P22052,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (GT-P7501 HTJ85B)*/
const Device UA2616245 = {S102904, 130, { &P22055,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7560 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2616398 = {S102905, 163, { &P22052,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P7501 Build/IMM76L; CyanogenMod-Ganbarou V0.11 GT) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616740 = {S102906, 77, { &P22041,&P18111,&P18112,&P18092 }};	/*Useragent - AndroidDownloadManager/4.1.1 (Linux; U; Android 4.1.1; GT-P6810 Build/JRO03L)*/
const Device UA2621590 = {S102907, 61, { &P22041,&P18111,&P18112,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-P6810 Build/JZO54K)*/
const Device UA2629303 = {S102908, 132, { &P22041,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-tw; GT-P6810 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617568 = {S102909, 139, { &P13452,&P5096,&P20791,&P18092 }};	/*Useragent - MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2618038 = {S102910, 137, { &P13452,&P5096,&P20791,&P18092 }};	/*Useragent - MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2620943 = {S102911, 149, { &P13452,&P5096,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2623177 = {S102912, 151, { &P13452,&P5096,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2618713 = {S102913, 90, { &P12889,&P17979,&P22070,&P18092 }};	/*Useragent - Instagram 3.0.5 Android (13/3.2; 160dpi; 800x1232; samsung; GT-P7300; GT-P7300; p3; en_GB)*/
const Device UA2632499 = {S102914, 168, { &P14908,&P5096,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1010, 40, 1.8.5)*/
const Device UA2620896 = {S102915, 153, { &P17337,&P5629,&P18062,&P18092 }};	/*Useragent - Rdio.Net; Android/2.4.4; Android/4.0.4; DM=samsung; DN=GT-P7500R; DC=samsung; DF=samsung/GT-P7500R/GT-P7500R:4.0.4/IMM76D/RUXLH2:user/release-keys; DH=p3*/
const Device UA2625226 = {S102916, 166, { &P15882,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P5110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2625854 = {S102917, 166, { &P15882,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P5110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2629901 = {S102918, 208, { &P17337,&P22212,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P7500 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0200, za*/
const Device UA2624359 = {S102919, 166, { &P17337,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P7500 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355*/
const Device UA2626512 = {S102920, 167, { &P17337,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P7500M Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2624672 = {S102921, 138, { &P22039,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P2000 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2625792 = {S102922, 145, { &P15932,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2626715 = {S102923, 147, { &P15932,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2628621 = {S102924, 147, { &P22043,&P18111,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2635205 = {S102925, 171, { &P13452,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2635427 = {S102926, 136, { &P13452,&P24697,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P1000 Build/MIUI) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2634827 = {S102927, 209, { &P15876,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-P5100 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0000, gb*/
const Device UA2635254 = {S102928, 210, { &P15876,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; vi-vn; GT-P5100 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.14/271,1.9.14.271,2012-12-21 21:40, +0300, by*/
const Device UA2637843 = {S102929, 148, { &P12879,&P6546,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; fr-fr; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.4.1000*/
const Device UA2729966 = {S102930, 176, { &P11841,&P5204,&P24676,&P18092 }};	/*Useragent - Nokia5220XpressMusic/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5220xpressmusic) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2758419 = {S102931, 170, { &P12280,&P17084,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3 UCBrowser8.3.1.173/41/355*/
const Device UA2960807 = {S102932, 99, { &P14404,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5260) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2977067 = {S102933, 86, { &P24091,&P18110,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv6l; U; en) Presto/2.8.115 Version/11.10 AQUOS-AS/1.0 LC-46LE835X*/
const Device UA3039674 = {S102934, 210, { &P24280,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL933; 55.8.03.4; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039684 = {S102935, 232, { &P24280,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL933; 64.4.08.7; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039722 = {S102936, 232, { &P24280,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL933; 67.2.73.6; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039731 = {S102937, 232, { &P24280,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL933; 64.4.08.7; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; pl) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039737 = {S102938, 232, { &P24280,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL933; 67.8.18.2; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; en) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039738 = {S102939, 232, { &P24280,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL933; 74.8.30.4; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039746 = {S102940, 232, { &P24280,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL933; 72.0.77.6; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039758 = {S102941, 232, { &P24280,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL933; 86.5.68.7; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3052325 = {S102942, 91, { &P14330,&P17483,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gx300) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3059862 = {S102943, 93, { &P11854,&P5890,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae71-1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3068875 = {S102944, 135, { &P12280,&P24384,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_1_2 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B146 Safari/8536.25*/
const Device UA3068876 = {S102945, 135, { &P12280,&P24384,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_1_6 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A403 Safari/8536.25*/
const Device UA3068878 = {S102946, 131, { &P12280,&P24384,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_1 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11D4 Safari/8536.25*/
const Device UA3068879 = {S102947, 133, { &P12280,&P24384,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10B142 Safari/8536.25*/
const Device UA3078973 = {S102948, 178, { &P16074,&P5768,&P24055,&P18092 }};	/*Useragent - Nokia3110c/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; Desktop) AppleWebKit/534.13 (KHTML, like Gecko) UCBrowser/9.2.0.311*/
const Device UA3084043 = {S102949, 163, { &P18507,&P5629,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-I8160P Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084046 = {S102950, 162, { &P18507,&P5629,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-I8160 Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084418 = {S102951, 41, { &P17968,&P17979,&P24496,&P18092 }};	/*Useragent - Dolphin http client/10.2.0(106) (Android)*/
const Device UA3084419 = {S102952, 41, { &P17968,&P17979,&P24496,&P18092 }};	/*Useragent - Dolphin http client/10.2.1(107) (Android)*/
const Device UA3084420 = {S102953, 41, { &P17968,&P17979,&P24496,&P18092 }};	/*Useragent - Dolphin http client/10.2.2(108) (Android)*/
const Device UA3084421 = {S102954, 41, { &P17968,&P17979,&P24496,&P18092 }};	/*Useragent - Dolphin http client/10.2.6(112) (Android)*/
const Device UA3084422 = {S102955, 41, { &P17968,&P17979,&P24496,&P18092 }};	/*Useragent - Dolphin http client/10.2.7(113) (Android)*/
const Device UA3115056 = {S102956, 99, { &P15655,&P17484,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; SAMSUNG-GT-S3850) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA90 = {S102957, 75, { &P20254,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) HTC-MP6950 BM*/
const Device UA15580 = {S102958, 103, { &P14862,&P5204,&P4742,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (09.98) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2700c-2/UC Browser7.5.1.77/70/351*/
const Device UA953 = {S102959, 83, { &P15364,&P17017,&P3680,&P18092 }};	/*Useragent - GIONEE-d002/SW1.0.0/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA33466 = {S102960, 76, { &P11662,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-K1v/08.03.08R MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 EGE/1.0*/
const Device UA45040 = {S102961, 92, { &P11744,&P5033,&P4948,&P18092 }};	/*Useragent - BlackBerry8300/4.2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/179 UP.Browser/5.0.3.3*/
const Device UA27764 = {S102962, 139, { &P18007,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; zh-tw; CHT8000 Build/CRB17) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA5992 = {S102963, 129, { &P12342,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-V1100/98.20.33BR BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 12043064) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA7267 = {S102964, 107, { &P13277,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC P6500; OpVer 27.110.1.612) Opera 8.65 [en]*/
const Device UA51430 = {S102965, 131, { &P13277,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC; 240x320; HTC_P6500/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1) Opera 8.65 [en]*/
const Device UA23563 = {S102966, 65, { &P13440,&P17483,&P4203,&P18092 }};	/*Useragent - LG-CU405/V1.0h Obigo/Q04C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA9496 = {S102967, 56, { &P13419,&P17017,&P3625,&P18092 }};	/*Useragent - KWC-K325/1.1.00 UP.Browser/6.2.3.9.g.1.107 (GUI) MMP/2.0*/
const Device UA84518 = {S102968, 56, { &P13419,&P17017,&P3625,&P18092 }};	/*Useragent - KWC-K325/1.0.04 UP.Browser/6.2.3.9.g.1.107 (GUI) MMP/2.0*/
const Device UA11252 = {S102969, 46, { &P14587,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGHC260/C260MBAGE6 Browser/Rainbow/3.0*/
const Device UA15390 = {S102970, 67, { &P17457,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-v3i/08.B4.38R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA16017 = {S102971, 71, { &P17489,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_P5520 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA20386 = {S102972, 36, { &P15080,&P22839,&P17470,&P18092 }};	/*Useragent - ZTE-A261/P103B9V1.0.0/WAP2.0 Profile*/
const Device UA86819 = {S102973, 68, { &P11992,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V191/0A.52.47R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21031 = {S102974, 54, { &P14136,&P17484,&P4901,&P18092 }};	/*Useragent - SAMSUNG-SGH-E700A/BSI UP.Browser/6.1.0.6 (GUI) MMP/1.0*/
const Device UA57941 = {S102975, 53, { &P14136,&P17484,&P4901,&P18092 }};	/*Useragent - SAMSUNG-SGH-E700/BSI UP.Browser/6.1.0.6 (GUI) MMP/1.0*/
const Device UA21425 = {S102976, 6, { &P15277,&P18588,&P18589,&P18092 }};	/*Useragent - M-6460*/
const Device UA21514 = {S102977, 61, { &P15294,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-C166/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA21515 = {S102978, 23, { &P15295,&P19954,&P3816,&P18092 }};	/*Useragent - SEC-SGHX500/1.0 TSS/2.5*/
const Device UA21721 = {S102979, 68, { &P15323,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V197/0A.63.12R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21736 = {S102980, 39, { &P15328,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KM501 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA21949 = {S102981, 121, { &P12025,&P5448,&P15792,&P18092 }};	/*Useragent - SAMSUNG-SGH-i718/MSIE4.01/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8)*/
const Device UA72550 = {S102982, 104, { &P17179,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-J630/J630JHHD4 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA22610 = {S102983, 106, { &P15496,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-L770S/1.0 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA22840 = {S102984, 13, { &P15516,&P17483,&P17660,&P18092 }};	/*Useragent - LG/U8550/v1.0*/
const Device UA33544 = {S102985, 92, { &P11689,&P16000,&P20449,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Nokia N70/5.0737.3.0.1; 7111) Opera 8.50 [en]*/
const Device UA27997 = {S102986, 98, { &P12208,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-A516/ Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA56112 = {S102987, 62, { &P11815,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonU100i/R7BA Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA32742 = {S102988, 47, { &P12752,&P17017,&P17470,&P18092 }};	/*Useragent - YL-COOLPADS100/06.10.S100 INFRAWARE/2.8 CTC/1.0*/
const Device UA32743 = {S102989, 49, { &P12753,&P17484,&P4203,&P18092 }};	/*Useragent - Samsung-SPHM500 AU-OBIGO/Q04C1-1.17_PRE_1 MMP/2.0*/
const Device UA32762 = {S102990, 73, { &P12761,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) MWg Zinc II*/
const Device UA32965 = {S102991, 99, { &P12809,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-J165L/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA33009 = {S102992, 62, { &P12826,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-B110L/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA33042 = {S102993, 74, { &P15564,&P19954,&P3772,&P18092 }};	/*Useragent - SEC-SGHI620/1.0 Browser/Opera/8.65 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA33129 = {S102994, 13, { &P16697,&P22784,&P17470,&P18092 }};	/*Useragent - MicroMax-X113*/
const Device UA33159 = {S102995, 86, { &P13075,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-V670A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA49715 = {S102996, 85, { &P13075,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-V670/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA33255 = {S102997, 27, { &P15547,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-C6712/C6712ZCKE2*/
const Device UA560263 = {S102998, 16, { &P15547,&P17484,&P17606,&P18092 }};	/*Useragent - samsung-gt-c6712*/
const Device UA33373 = {S102999, 24, { &P12906,&P17017,&P17470,&P18092 }};	/*Useragent - WellcoM W929 WAP Browser*/
const Device UA33454 = {S103000, 35, { &P12921,&P17017,&P17470,&P18092 }};	/*Useragent - BenQ-T60/1.0/WAP2.0/MIDP2.0/CLDC1.1*/
const Device UA56588 = {S103001, 56, { &P13793,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-8720_/00.62 UP.Browser/6.2.3.4.c.1.104 (GUI) MMP/2.0*/
const Device UA44178 = {S103002, 88, { &P13797,&P19950,&P4893,&P18092 }};	/*Useragent - SAGEM-my400V/2.0/ MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.f.1.150 (GUI)*/
const Device UA63230 = {S103003, 88, { &P13797,&P19950,&P4893,&P18092 }};	/*Useragent - SAGEM-my400V/2.0/ MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.f.1.159 (GUI)*/
const Device UA44591 = {S103004, 79, { &P17229,&P17017,&P4893,&P18092 }};	/*Useragent - Bird.M19/LO,901/WAP2.0/MIDP-2.0/CLDC-1.1 Browser/UP.Browser/7.1.0.f.1.144 (GUI)*/
const Device UA54174 = {S103005, 79, { &P17229,&P17017,&P4893,&P18092 }};	/*Useragent - Bird.M19/LU,901/WAP2.0/MIDP-2.0/CLDC-1.1 Browser/UP.Browser/7.1.0.f.1.144 (GUI)*/
const Device UA54334 = {S103006, 79, { &P17229,&P17017,&P4893,&P18092 }};	/*Useragent - Bird.M19/L1,901/WAP2.0/MIDP-2.0/CLDC-1.1 Browser/UP.Browser/7.1.0.f.1.144 (GUI)*/
const Device UA44996 = {S103007, 66, { &P13850,&P17017,&P17470,&P18092 }};	/*Useragent - i-mobileS382/SW10.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA45106 = {S103008, 129, { &P16169,&P5448,&P17173,&P18092 }};	/*Useragent - PPC; 240x320; HTC_P4351/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA49049 = {S103009, 121, { &P13888,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-GT-S6700C/1.0 SHP/R5 Release/02.01.2009 Browser/NetFront3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA49446 = {S103010, 134, { &P13027,&P17605,&P3502,&P18092 }};	/*Useragent - SAMSUNG-GT-S5680_TD/1.0 RTK-E/1.0 SHP/VPP-R5 Release/3.18.2010 Browser/Dolfin1.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 SMM-MMS/1.2.0*/
const Device UA49948 = {S103011, 134, { &P13115,&P17605,&P3976,&P18092 }};	/*Useragent - SAMSUNG-GT-S3710/1.0 RTK-E/1.0 SHP/VPP/R5 Release/07.09.2010 Browser/NetFront3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA50064 = {S103012, 141, { &P17234,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; zh-tw; Vibo-A688 Build/DONUT) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA50197 = {S103013, 63, { &P13194,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHI300/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 WAP/1.0*/
const Device UA54644 = {S103014, 21, { &P14250,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia6510/1.0 (04.12)*/
const Device UA50412 = {S103015, 17, { &P11947,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonK700i*/
const Device UA2125786 = {S103016, 35, { &P15373,&P5129,&P4737,&P18092 }};	/*Useragent - Nokia5530/UC Browser7.2.2.51/50/352*/
const Device UA50986 = {S103017, 156, { &P13920,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i458B/ZTHC1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA56919 = {S103018, 151, { &P13920,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i458B/ Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA62753 = {S103019, 44, { &P13925,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KU800/v10a Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA87302 = {S103020, 53, { &P14520,&P17486,&P3402,&P18092 }};	/*Useragent - MOT-W213/0.0.45 UP.Browser/6.3.0.6.c.17 (GUI) MMP/2.0*/
const Device UA51501 = {S103021, 81, { &P11998,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonZ550i/R6CA Java/SEMC-Java/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA51840 = {S103022, 138, { &P13617,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; MDA_Vario_V/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54133 = {S103023, 45, { &P14126,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-I650/1.0 ObigoInternetBrowser/Q03C*/
const Device UA54283 = {S103024, 98, { &P14168,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_Touch_Viva_T2223 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA54449 = {S103025, 46, { &P17018,&P7472,&P3625,&P18092 }};	/*Useragent - Alcatel-BH4/1.0 UP.Browser/6.2.ALCATEL MMP/1.0*/
const Device UA54568 = {S103026, 40, { &P14134,&P17485,&P4203,&P18092 }};	/*Useragent - SonyEricssonJ220i/R5N TelecaBrowser/4.08*/
const Device UA54781 = {S103027, 98, { &P14259,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-Z360/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA55001 = {S103028, 66, { &P14273,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GD550/V100 Obigo/WAP2.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA55026 = {S103029, 76, { &P14275,&P17017,&P3402,&P18092 }};	/*Useragent - BenQ-T33/1.0/WAP2.0/MIDP2.0/CLDC1.1 UP.Browser/6.3.0.4.c.1.105 (GUI) MMP/2.0*/
const Device UA58815 = {S103030, 61, { &P17706,&P17017,&P17470,&P18092 }};	/*Useragent - SFR114/g8eA/FES-FR-SFR-P103D1V1.0.1/R220/2.0 Profile/MIDP-2.0*/
const Device UA58932 = {S103031, 97, { &P17488,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Vodafone/1.0/HTC_Mercury/3.54.162.4*/
const Device UA60856 = {S103032, 13, { &P12587,&P17017,&P17470,&P18092 }};	/*Useragent - imobileGM5220*/
const Device UA60903 = {S103033, 51, { &P17410,&P6712,&P3402,&P18092 }};	/*Useragent - Nokia-2228 UP.Browser/6.3.0.8.c.1.104 (GUI) MMP/2.0*/
const Device UA62313 = {S103034, 60, { &P14542,&P17017,&P3976,&P18092 }};	/*Useragent - LCTW660 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62347 = {S103035, 101, { &P14400,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E780/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA62566 = {S103036, 64, { &P14423,&P5204,&P3625,&P18092 }};	/*Useragent - Nokia-2705/1.0 UP.Browser/6.2.3.9.j.1.107-20090216 (GUI) MMP/2.0*/
const Device UA63876 = {S103037, 8, { &P13155,&P17486,&P17671,&P18092 }};	/*Useragent - MOT-QA30*/
const Device UA64429 = {S103038, 106, { &P14665,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-M8800C/1.0 SHP/VPP/R5 NetFront/3.5 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA64451 = {S103039, 73, { &P14670,&P17483,&P4970,&P18092 }};	/*Useragent - LG-LX570 AU-MIC-LX570/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64774 = {S103040, 80, { &P14704,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonS710a/R1A SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA67363 = {S103041, 30, { &P11929,&P5129,&P24645,&P18092 }};	/*Useragent - Nokia5230/UCWEB7.5.0.66/50/800*/
const Device UA68697 = {S103042, 103, { &P12473,&P5204,&P4777,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2730c-1/UC Browser7.7.1.88/70/401*/
const Device UA68819 = {S103043, 43, { &P11684,&P5129,&P24662,&P18092 }};	/*Useragent - Nokia5800w XpressMusic/UCWEB7.8.0.87/50/999*/
const Device UA68987 = {S103044, 42, { &P11684,&P5129,&P24662,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UCWEB7.8.0.87/50/800*/
const Device UA68958 = {S103045, 80, { &P11943,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-D980/D980JVIB2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA69069 = {S103046, 141, { &P13100,&P5448,&P4737,&P18092 }};	/*Useragent - SP; 240x320; HTC_S710/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; Smartphone; 240x320)/UC Browser7.2.2.51*/
const Device UA69112 = {S103047, 36, { &P11907,&P5890,&P4779,&P18092 }};	/*Useragent - NokiaE52-1/UC Browser7.8.0.95/28/351*/
const Device UA69143 = {S103048, 36, { &P14367,&P5890,&P4759,&P18092 }};	/*Useragent - NOKIA6710s/UC Browser7.6.1.82/28/352*/
const Device UA69393 = {S103049, 35, { &P15209,&P5768,&P4779,&P18092 }};	/*Useragent - nokia6233/UC Browser7.8.0.95/69/401*/
const Device UA80248 = {S103050, 60, { &P12781,&P6478,&P4420,&P18092 }};	/*Useragent - BlackBerry7230/4.0.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA81098 = {S103051, 8, { &P11689,&P5890,&P3756,&P18092 }};	/*Useragent - NokiaN70*/
const Device UA88514 = {S103052, 10, { &P11689,&P5890,&P3756,&P18092 }};	/*Useragent - NokiaN70-1*/
const Device UA82964 = {S103053, 56, { &P12704,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-MLNBT/00.62 UP.Browser/6.2.3.4.e.1.100 (GUI) MMP/2.0*/
const Device UA86668 = {S103054, 45, { &P20243,&P17483,&P3625,&P18092 }};	/*Useragent - LG-LX140/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA86943 = {S103055, 93, { &P13694,&P17484,&P4924,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungSGHJ600V/J600BUGH1 Browser/NF/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA460139 = {S103056, 93, { &P13694,&P17484,&P4924,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungSGHJ600V/J600DUHC1 Browser/NF/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88650 = {S103057, 43, { &P17749,&P17017,&P17470,&P18092 }};	/*Useragent - WELLCOM/W920/MIDP2.0/CLDC1.1/Screen-240x320*/
const Device UA89360 = {S103058, 44, { &P17700,&P19941,&P20422,&P18092 }};	/*Useragent - KDDI-TS21 UP.Browser/6.0.2.273 (GUI) MMP/1.1*/
const Device UA158298 = {S103059, 90, { &P15364,&P17285,&P17193,&P19831 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0) AddSugarSpiderBot www.idealobserver.com*/
const Device UA251531 = {S103060, 123, { &P15364,&P7133,&P24206,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.6.9632.201*/
const Device UA253981 = {S103061, 44, { &P11948,&P17485,&P17338,&P18092 }};	/*Useragent - sonyericssonk550i/UC Browser7.9.0.102/69/352*/
const Device UA350610 = {S103062, 124, { &P15364,&P17285,&P24208,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.8.10898.201*/
const Device UA358710 = {S103063, 155, { &P13064,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SGH-I727 Build/GRI40; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369727 = {S103064, 109, { &P22119,&P17979,&P3508,&P18092 }};	/*Useragent - HS-T89_TD/1.0 Android/2.2 Release/09.26.2011 Browser/AppleWebKit533.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA427276 = {S103065, 40, { &P17968,&P17979,&P24429,&P18092 }};	/*Useragent - Dolphin http client/7.4.1(122) (Android)*/
const Device UA621865 = {S103066, 45, { &P17968,&P17979,&P24429,&P18092 }};	/*Useragent - Dolphin http client/7.4.0 Beta(119) (Android)*/
const Device UA621952 = {S103067, 40, { &P17968,&P17979,&P24429,&P18092 }};	/*Useragent - Dolphin http client/7.4.0(120) (Android)*/
const Device UA436723 = {S103068, 36, { &P12059,&P5204,&P23399,&P18092 }};	/*Useragent - MQQBrowser/Mini2.2 (Nokia2690/10.10)*/
const Device UA459254 = {S103069, 16, { &P15988,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-C3312*/
const Device UA532358 = {S103070, 138, { &P16168,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0; us-us; LogicPD Zoom2 Build/HONEYCOMB) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA544529 = {S103071, 70, { &P21464,&P22791,&P4808,&P18092 }};	/*Useragent - Cherry Mobile-eclipse2;Mozilla/4.0 (compatible; MSIE 6.0; PPC;240x400)*/
const Device UA589293 = {S103072, 81, { &P17595,&P17297,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X 10.7.3; U; en) Presto/2.8.131 Version/11.11*/
const Device UA617005 = {S103073, 101, { &P12280,&P17145,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]*/
const Device UA617273 = {S103074, 99, { &P12280,&P17090,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]*/
const Device UA618120 = {S103075, 101, { &P12280,&P17090,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]*/
const Device UA618278 = {S103076, 101, { &P12280,&P17090,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]*/
const Device UA622735 = {S103077, 137, { &P15364,&P17820,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Ubuntu/12.04 Chromium/15.0.874.120 Chrome/15.0.874.120 Safari/535.2*/
const Device UA740578 = {S103078, 135, { &P15364,&P17820,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.2 (KHTML, like Gecko) Ubuntu/12.04 Chromium/15.0.874.120 Chrome/15.0.874.120 Safari/535.2*/
const Device UA665416 = {S103079, 72, { &P13051,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2.2; zh-cn; ZTE-T_U880; 480*800) UCWEB7.9.3.103/139/800*/
const Device UA672083 = {S103080, 53, { &P14984,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (Nokia5250;SymbianOS/9.1 Series60/3.0)*/
const Device UA1928814 = {S103081, 35, { &P14519,&P5129,&P24666,&P18092 }};	/*Useragent - NokiaN97 mini/UCWEB8.1.0.104/50/999*/
const Device UA797960 = {S103082, 135, { &P16598,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; A500 Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA833380 = {S103083, 142, { &P15767,&P17145,&P20774,&P18092 }};	/*Useragent - MQQBrowser/2.3 Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/7534.48.3*/
const Device UA838260 = {S103084, 142, { &P16356,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-N7000 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA925761 = {S103085, 78, { &P21408,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Windows Phone 6.5; WS027SH)*/
const Device UA925764 = {S103086, 98, { &P18249,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; Acer; Allegro)*/
const Device UA925783 = {S103087, 108, { &P18249,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; Acer; Allegro; bouygues)*/
const Device UA1935693 = {S103088, 60, { &P13077,&P18110,&P24661,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.3;Zh_cn;C8650;320*480;)UCWEB7.7.0.85/139/800*/
const Device UA1966392 = {S103089, 167, { &P16321,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; X500 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA1984073 = {S103090, 136, { &P18253,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; XT894 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026117 = {S103091, 136, { &P18253,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; XT894 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1995178 = {S103092, 39, { &P15775,&P5448,&P24632,&P18092 }};	/*Useragent - NOKIA-Lumia 800/UCWEB2.3.0.182/47/32079*/
const Device UA1995182 = {S103093, 39, { &P15795,&P5448,&P24632,&P18092 }};	/*Useragent - NOKIA-Lumia 710/UCWEB2.3.0.182/47/32079*/
const Device UA1995829 = {S103094, 251, { &P14826,&P5843,&P18524,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (06.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2) UCBrowser8.4.0.159/70/352*/
const Device UA2018430 = {S103095, 134, { &P14604,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; MB200 Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018861 = {S103096, 128, { &P16950,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; streak7 Build/HMJ19) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2019378 = {S103097, 135, { &P22567,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; VX610A Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020587 = {S103098, 62, { &P15364,&P17979,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.4;Zh_cn;Android;480*800;)UCWEB7.8.1.96/139/800*/
const Device UA2024927 = {S103099, 154, { &P15649,&P5629,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5570 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025003 = {S103100, 72, { &P18295,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.3;Zh_cn;HTC One X;720*1184;)UCWEB7.8.0.95/139/444*/
const Device UA2025099 = {S103101, 159, { &P13042,&P5629,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Incredible S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033233 = {S103102, 159, { &P13042,&P5629,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Incredible S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025171 = {S103103, 132, { &P21270,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; ME722 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025189 = {S103104, 160, { &P20183,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026722 = {S103105, 176, { &P14225,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Desire HD A9191 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2027289 = {S103106, 170, { &P12681,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S 4G Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032396 = {S103107, 170, { &P12681,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S 4G Build/IML74K; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027778 = {S103108, 146, { &P11747,&P5096,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-gb; HTC Desire Build/FRG83G)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2031554 = {S103109, 143, { &P11747,&P5096,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-gb; HTC Desire Build/FRF91)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028500 = {S103110, 163, { &P14799,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I5500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2030563 = {S103111, 168, { &P16356,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-N7000 Build/JRO03H; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033389 = {S103112, 168, { &P16356,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-N7000 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2031322 = {S103113, 69, { &P12706,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.4;Zh_cn;LG-P690;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA2031584 = {S103114, 160, { &P21052,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A9 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2032948 = {S103115, 157, { &P13123,&P5629,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S5360 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2034040 = {S103116, 164, { &P16049,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P698 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2034406 = {S103117, 159, { &P15517,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; MT15i Build/4.0.2.A.0.42; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2159662 = {S103118, 122, { &P15364,&P7133,&P22376,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/1.0.0.8 Chrome/18.0.966.0 Safari/535.12*/
const Device UA2314841 = {S103119, 115, { &P15364,&P7133,&P22376,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.12 (KHTML, like Gecko) Maxthon/1.0.0.8 Chrome/18.0.966.0 Safari/535.12*/
const Device UA2200479 = {S103120, 128, { &P15767,&P18432,&P18361,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 Safari/7534.48.3 Sleipnir/2.0m*/
const Device UA2221868 = {S103121, 85, { &P14764,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-us; SonyEricssonU1)/UC Browser8.4.0.159/50/352/UCWEB Mobile*/
const Device UA2242088 = {S103122, 77, { &P18385,&P17979,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.4.5) (grouper;asus;Nexus 7;4.1.1/JRO03D)*/
const Device UA2242103 = {S103123, 77, { &P18385,&P17979,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.4.6) (grouper;asus;Nexus 7;4.1.2/JZO54K)*/
const Device UA2242097 = {S103124, 77, { &P18385,&P17979,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.4.5) (grouper;asus;Nexus 7;4.1.2/JZO54K)*/
const Device UA2274054 = {S103125, 110, { &P15364,&P18145,&P17245,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; FreeBSD i386; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Chrome/7.0.517.44 Safari/534.7*/
const Device UA2333719 = {S103126, 150, { &P12280,&P17145,&P20784,&P18092 }};	/*Useragent - MQQBrowser/33 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_2_1 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/7534.48.3*/
const Device UA2338446 = {S103127, 158, { &P14090,&P5477,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; GT-I5801 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2441670 = {S103128, 42, { &P11692,&P5890,&P24665,&P18092 }};	/*Useragent - NOKIA3250/UC Browser8.0.4.121/28/444/UCWEB*/
const Device UA2528031 = {S103129, 153, { &P11699,&P5768,&P24673,&P18092 }};	/*Useragent - Nokia5200/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5200) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2586644 = {S103130, 181, { &P21611,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (HUAWEI-M931, 42, 1.9.1)*/
const Device UA2607508 = {S103131, 156, { &P21736,&P5629,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LAVA iris 351 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2613680 = {S103132, 163, { &P17337,&P6546,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7500 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2608921 = {S103133, 163, { &P12889,&P6546,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7300 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2615830 = {S103134, 149, { &P13452,&P5096,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2; in-id; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2610555 = {S103135, 136, { &P17337,&P6546,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37)Maxthon AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2623152 = {S103136, 134, { &P17337,&P6546,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-; GT-P7500 Build/HMJ37)Maxthon AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2621070 = {S103137, 165, { &P14908,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2610966 = {S103138, 164, { &P17323,&P6437,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6800 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2635387 = {S103139, 168, { &P17323,&P6437,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-US; GT-P6800 Build/HTJ85B) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2612164 = {S103140, 130, { &P15919,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2614530 = {S103141, 130, { &P15919,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; id-id; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2612789 = {S103142, 149, { &P13452,&P5096,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2613632 = {S103143, 151, { &P13452,&P5096,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2613020 = {S103144, 75, { &P17323,&P17979,&P22699,&P18092 }};	/*Useragent - TwitterAndroid/2.0.1 (122) GT-P6800/13 (samsung;GT-P6800;samsung;GT-P6800;)*/
const Device UA2618421 = {S103145, 95, { &P15985,&P17979,&P22068,&P18092 }};	/*Useragent - Instagram 1.0.6 Android (13/3.2; 160dpi; 600x976; samsung; GT-P6200; GT-P6200; smdkc210; en_US)*/
const Device UA2617288 = {S103146, 193, { &P14908,&P5629,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-P1010 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.7/210,1.9.7.210,2012-10-25 19:53, gb*/
const Device UA2616175 = {S103147, 179, { &P22043,&P17979,&P3508,&P18092 }};	/*Useragent - Rdio.Net; Android/2.2.10; Android/4.0.4; DM=samsung; DN=GT-P5113; DC=samsung; DF=samsung/espresso10wifibby/espresso10wifi:4.0.4/IMM76D/P5113UEBLH3:user/release-keys; DH=espresso10*/
const Device UA2632091 = {S103148, 166, { &P22037,&P18111,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P3113 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2621796 = {S103149, 161, { &P17323,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; GT-P6800 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2630006 = {S103150, 168, { &P17323,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P6800 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2624737 = {S103151, 145, { &P17323,&P6437,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 3.2; id-id; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2635655 = {S103152, 152, { &P17323,&P6437,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 3.2; id-id; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2620389 = {S103153, 144, { &P17337,&P6546,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2619470 = {S103154, 61, { &P17323,&P17979,&P16179,&P18092 }};	/*Useragent - com.eurosport.news.android/3.1.1/Android/3.2/samsung-GT-P6800*/
const Device UA2619401 = {S103155, 219, { &P17337,&P18066,&P18369,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/7.0.29759/28.3030; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2618236 = {S103156, 104, { &P15876,&P17979,&P22071,&P18092 }};	/*Useragent - Instagram 3.1.0 Android (15/4.0.3; 160dpi; 800x1232; samsung; GT-P5100; espresso10rf; espresso10; en_GB)*/
const Device UA2618389 = {S103157, 187, { &P14908,&P5096,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.6/148,1.9.6.148,2012-09-11 08:01, gb*/
const Device UA2618512 = {S103158, 140, { &P15876,&P18066,&P20791,&P18092 }};	/*Useragent - MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2625511 = {S103159, 152, { &P15876,&P18066,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2618966 = {S103160, 29, { &P18475,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (GT-P7320 MASTER)*/
const Device UA2619672 = {S103161, 161, { &P17337,&P6546,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7500, 36, 1.7.1)*/
const Device UA2622594 = {S103162, 163, { &P17337,&P6546,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500D Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7500D, 36, 1.7.1)*/
const Device UA2635790 = {S103163, 163, { &P17337,&P6546,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-ca; GT-P7500M Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7500M, 41, 1.9.0)*/
const Device UA2619924 = {S103164, 38, { &P17323,&P17979,&P18062,&P18092 }};	/*Useragent - Android/4.0.4; samsung GT-P6800; 2.3.2*/
const Device UA2621734 = {S103165, 166, { &P15932,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P3110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2620618 = {S103166, 166, { &P22043,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P5113 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355*/
const Device UA2622849 = {S103167, 157, { &P13452,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2623056 = {S103168, 157, { &P13452,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2626995 = {S103169, 157, { &P13452,&P5629,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2623217 = {S103170, 162, { &P15985,&P6437,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P6200, 36, 1.7.1)*/
const Device UA2623675 = {S103171, 164, { &P15985,&P6437,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P6200L, 39, 1.8.2)*/
const Device UA2623428 = {S103172, 146, { &P15985,&P6437,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2624887 = {S103173, 145, { &P15985,&P6437,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2625189 = {S103174, 95, { &P14908,&P17979,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.0 Android (8/2.2.1; 240dpi; 600x1024; samsung; GT-P1010; GT-P1010; p1lite; en_US)*/
const Device UA2630477 = {S103175, 97, { &P14908,&P17979,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P6210; GT-P6210; smdk4210; en_US)*/
const Device UA2623943 = {S103176, 189, { &P17168,&P6546,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7310 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0000, gb*/
const Device UA2635527 = {S103177, 203, { &P17168,&P6546,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7310 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13 Flipboard/1.9.19/285,1.9.19.285,2013-01-24 11:21, +0800, china, cn*/
const Device UA2624933 = {S103178, 150, { &P22037,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3113 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2634290 = {S103179, 149, { &P22037,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3113 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2625052 = {S103180, 192, { &P14905,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-P7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0100, be*/
const Device UA2625319 = {S103181, 166, { &P15876,&P18111,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P5100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2631152 = {S103182, 56, { &P14908,&P17979,&P22278,&P18092 }};	/*Useragent - theScore/3.1.0 Android-OS/3.2 (Android; Phone, GT-P6210)*/
const Device UA2631485 = {S103183, 58, { &P14908,&P17979,&P22278,&P18092 }};	/*Useragent - theScore/3.1.1 Android-OS/2.2.1 (Android; Phone, GT-P1010)*/
const Device UA2631248 = {S103184, 59, { &P22043,&P17979,&P22278,&P18092 }};	/*Useragent - theScore/3.1.1 Android-OS/4.1.1 (Android; Tablet, GT-P5113)*/
const Device UA2633694 = {S103185, 156, { &P15876,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; GT-P5100 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2634592 = {S103186, 156, { &P15876,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P5100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2634198 = {S103187, 164, { &P12879,&P6437,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7510 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2636891 = {S103188, 59, { &P12879,&P17979,&P22280,&P18092 }};	/*Useragent - theScore/3.2.1 Android-OS/4.0.4 (Android; Tablet, GT-P7510)*/
const Device UA2635072 = {S103189, 163, { &P12879,&P6546,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7510 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2657281 = {S103190, 90, { &P11996,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; gt-s5560) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2778539 = {S103191, 99, { &P11996,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; samsung-gt-s5560i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2660347 = {S103192, 98, { &P12209,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-ks360 teleca) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2680956 = {S103193, 71, { &P22385,&P18111,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 4.1.2; zh-cn; GT-I8262D; 480*800) UCWEB7.9.4.145/139/800*/
const Device UA2680962 = {S103194, 153, { &P22385,&P18111,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2680991 = {S103195, 141, { &P22385,&P18111,&P20791,&P18092 }};	/*Useragent - MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2721108 = {S103196, 54, { &P13324,&P5890,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0 (NokiaN8-00;SymbianOS/9.1 Series60/3.0)*/
const Device UA2752550 = {S103197, 99, { &P15198,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3650) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2810028 = {S103198, 145, { &P12513,&P17485,&P24668,&P18092 }};	/*Useragent - SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; sonyericssonw20) UCBrowser8.3.0.154/70/444/UCWEB Mobile*/
const Device UA2817753 = {S103199, 77, { &P14969,&P18110,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Adr (Linux; U; 2.3.3; zh-cn; HTC Desire S Build/GRI40;480*800)*/
const Device UA2847833 = {S103200, 159, { &P12125,&P5204,&P24675,&P18092 }};	/*Useragent - Nokia6500s-1/2.0 (10.00) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia6500s-1) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2852410 = {S103201, 99, { &P14768,&P22839,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; zte-p80aiii-f160) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2852553 = {S103202, 90, { &P13782,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t310) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2986854 = {S103203, 90, { &P13782,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; es-LA; lg-t310) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA3008387 = {S103204, 92, { &P11679,&P5776,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6630) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3013063 = {S103205, 88, { &P15518,&P17017,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; s5230) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3013990 = {S103206, 98, { &P12209,&P17483,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-ks360 teleca) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3014833 = {S103207, 90, { &P14532,&P17483,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t325) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3025621 = {S103208, 99, { &P14972,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3370) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3031426 = {S103209, 100, { &P12351,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3653w) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3039646 = {S103210, 200, { &P24326,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 55LM6700-TA; 04.10.10; 0xX;); LG NetCast.TV-2012*/
const Device UA3039652 = {S103211, 207, { &P24357,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+PORTAL_KEY+SCREEN+TUNER; LGE; SP820; 08.00.65; 0xX;); LG NetCast.TV-2012 0*/
const Device UA3040392 = {S103212, 181, { &P15364,&P24349,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (ZmEufAdvOJ6+jpzS29ugr8sbtxJwbNEwTvj+UjrNdn8=) Chromium/26.0.1410.63 Chrome/26.0.1410.63 Safari/535.2*/
const Device UA3040393 = {S103213, 181, { &P15364,&P24349,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (BiJcJ/NnWWNRCJbpE1IDAUARApQI13bBpXjUKIJ3NX0=) Chromium/26.0.1410.63 Chrome/26.0.1410.63 Safari/535.2*/
const Device UA3040394 = {S103214, 181, { &P15364,&P24349,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (2+5mbYbRy06RbV9AqKfDxK1OxQTSP/3ogwQnfKgRpe8=) Chromium/26.0.1410.63 Chrome/26.0.1410.63 Safari/535.2*/
const Device UA3040395 = {S103215, 181, { &P15364,&P24349,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (HRfVdPGb2DXDoQv7j0jiZ7kumjupqYEZuuv+uUjYaXY=) Chromium/26.0.1410.63 Chrome/26.0.1410.63 Safari/535.2*/
const Device UA3040396 = {S103216, 181, { &P15364,&P24349,&P19839,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/18 (XPokjOYxFHsUdRJ3O2EuYjHwfjohEnE9z01PffnbHEI=) Chromium/26.0.1410.63 Chrome/26.0.1410.63 Safari/535.2*/
const Device UA3058783 = {S103217, 91, { &P13365,&P17483,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gu220) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3068985 = {S103218, 121, { &P18502,&P18148,&P22782,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows Phone 8.0; Trident/7.0; rv:11.0; IEMobile/11.0; ARM; Touch; HTC; Windows Phone 8X by HTC) like Gecko*/
const Device UA3068986 = {S103219, 105, { &P18502,&P18148,&P22782,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows Phone 8.0; Trident/7.0; rv:11.0; IEMobile/11.0; ARM; Touch; HTC; PM23300) like Gecko*/
const Device UA3068997 = {S103220, 105, { &P18502,&P18148,&P22782,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows Phone 8.0; ARM; Trident/7.0; Touch; rv:11.0; IEMobile/11.0; HTC; PM23300) like Gecko*/
const Device UA3069000 = {S103221, 121, { &P18502,&P18148,&P22782,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows Phone 8.0; ARM; Trident/7.0; Touch; rv:11.0; IEMobile/11.0; HTC; Windows Phone 8X by HTC) like Gecko*/
const Device UA3108663 = {S103222, 90, { &P17332,&P17483,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-c310) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3110595 = {S103223, 99, { &P12137,&P17484,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; SAMSUNG-GT-S5620) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3119225 = {S103224, 138, { &P19973,&P5043,&P24670,&P18092 }};	/*Useragent - Nokia114/2.0 (03.33) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia114) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA663 = {S103225, 103, { &P14708,&P5204,&P4715,&P18092 }};	/*Useragent - Nokia5130c-2/2.0 (06.93) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5130c-2/UC Browser7.4.0.65/70/351*/
const Device UA3492 = {S103226, 120, { &P12186,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-Z238/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1/*MzUzMjEyMDEwMDQ4ODI5*/
const Device UA4398 = {S103227, 98, { &P13442,&P17484,&P4924,&P18092 }};	/*Useragent - SAMSUNG-SGH-E908/NetFront 3.2/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzUyMDAzMDExNzEyNjE3*/
const Device UA30811 = {S103228, 146, { &P12346,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; apanda-A60 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA33102 = {S103229, 45, { &P14559,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-V177 UP.Browser/6.2.3.9.c.6 (GUI) MMP/2.0*/
const Device UA63934 = {S103230, 52, { &P14559,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-V177/0.1.66 UP.Browser/6.2.3.9.c.9 (GUI) MMP/2.0*/
const Device UA7636 = {S103231, 74, { &P13367,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; O2 Xda Zinc; PPC; 240x320)*/
const Device UA7637 = {S103232, 73, { &P13381,&P17484,&P4924,&P18092 }};	/*Useragent - SAMSUNG-SGH-Z510/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA9086 = {S103233, 119, { &P13370,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-M3200/M3200JCHJ1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA9149 = {S103234, 16, { &P12137,&P17484,&P17606,&P18092 }};	/*Useragent - samsung-gt-s5620*/
const Device UA77744 = {S103235, 27, { &P12137,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-S5620/S5620DDJE1*/
const Device UA10116 = {S103236, 67, { &P13469,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-X496/X496UZEI1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA11176 = {S103237, 48, { &P13713,&P17017,&P17470,&P18092 }};	/*Useragent - FLY-SX210/Profile/MIDP.2.0Configuration/CLDC.1.0*/
const Device UA43631 = {S103238, 98, { &P13713,&P17017,&P17470,&P18092 }};	/*Useragent - FLY-SX210/Profile/MIDP.2.0Configuration/CLDC.1.0, FLY-SX210/Profile/MIDP.2.0Configuration/CLDC.1.0*/
const Device UA11266 = {S103239, 94, { &P12970,&P5448,&P4578,&P18092 }};	/*Useragent - ASUS-GalaxyII/1.0 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320; ASUS-GalaxyII)*/
const Device UA12384 = {S103240, 96, { &P13241,&P5026,&P4948,&P18092 }};	/*Useragent - BlackBerry8830/4.5.0.138 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105 UP.Browser/5.0.3.3*/
const Device UA15159 = {S103241, 77, { &P13528,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-S9402/S9402XEIA2 NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA65152 = {S103242, 92, { &P11631,&P5027,&P4948,&P18092 }};	/*Useragent - BlackBerry8110/4.3.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102 UP.Browser/5.0.3.3*/
const Device UA18827 = {S103243, 17, { &P11940,&P17017,&P17470,&P18092 }};	/*Useragent - W3319 WAP Browser*/
const Device UA20088 = {S103244, 75, { &P11958,&P5448,&P22270,&P18092 }};	/*Useragent - Palm750/v0005 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA20354 = {S103245, 45, { &P15078,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-565A/1.0 ObigoInternetBrowser/Q03C*/
const Device UA21112 = {S103246, 127, { &P17424,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV3xR/86.8A.10R BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 2089) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [en]*/
const Device UA21113 = {S103247, 127, { &P17424,&P5338,&P20417,&P18092 }};	/*Useragent - MOT-RAZRV3xR/86.43.15P BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 2121) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.00 [es]*/
const Device UA21283 = {S103248, 40, { &P15180,&P22835,&P17470,&P18092 }};	/*Useragent - SPICE-C6600/3.00 IAC/V1.0 WESTLAKE/0.1.8*/
const Device UA21446 = {S103249, 127, { &P15284,&P5448,&P4643,&P18092 }};	/*Useragent - LG-GT810H Netfront/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA21534 = {S103250, 124, { &P15303,&P5208,&P4521,&P18092 }};	/*Useragent - Mozilla/5.0 (SCH-W609/W609DJ13;U;REX/3.0;BREW/3.1.5;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*400;CTC/2.0) POLARIS/6.2.WEB*/
const Device UA21733 = {S103251, 77, { &P16253,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) O2 Xda Atom Life*/
const Device UA22239 = {S103252, 89, { &P15409,&P17017,&P3625,&P18092 }};	/*Useragent - Haier-T10C/1.0 iPanel/2.0 WAP2.0 (compatible; UP.Browser/6.2.2.4; UPG1; UP/4.0; Embedded)*/
const Device UA22302 = {S103253, 107, { &P15420,&P17483,&P3728,&P18092 }};	/*Useragent - LG_TB200_CMCC A2000pH/2.32.00 Release/9.14.2009 Browser/Teleca/Q7.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22327 = {S103254, 84, { &P15426,&P17662,&P3976,&P18092 }};	/*Useragent - SoftBank/1.0/920P/PJP23 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22388 = {S103255, 108, { &P15450,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonW902plus/R3DB Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.2*/
const Device UA22627 = {S103256, 98, { &P15504,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-A821/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA22683 = {S103257, 98, { &P15507,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E747/ Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA22728 = {S103258, 97, { &P12059,&P5204,&P4737,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2690/UC Browser7.2.2.51/70/352*/
const Device UA22900 = {S103259, 93, { &P12048,&P5448,&P4808,&P18092 }};	/*Useragent - T-Mobile Dash Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 320x240; Smartphone)*/
const Device UA23003 = {S103260, 53, { &P15546,&P17483,&P17660,&P18092 }};	/*Useragent - LG/KB620/v1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA23170 = {S103261, 56, { &P15568,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-W385m/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0*/
const Device UA23174 = {S103262, 41, { &P19913,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-R400 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA23262 = {S103263, 53, { &P15583,&P17483,&P17660,&P18092 }};	/*Useragent - LG/KT525/v1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA23330 = {S103264, 87, { &P15589,&P22835,&P3807,&P18092 }};	/*Useragent - Spice/1.0/QT-68/Obigo Browser/Q03C Browser/Q03C MMS/1.0/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA32894 = {S103265, 79, { &P15619,&P17486,&P3402,&P18092 }};	/*Useragent - MOT-W403/0.1.27 UP.Browser/6.3.0.6.c.43 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23958 = {S103266, 37, { &P15638,&P17483,&P17470,&P18092 }};	/*Useragent - LG-B2050 MIC/WAP2.0 MIDP-2.0/CLDC-1.0*/
const Device UA24147 = {S103267, 74, { &P15669,&P17486,&P4203,&P18092 }};	/*Useragent - Motorola-ic602 Obigo/Q04C1 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24220 = {S103268, 41, { &P15671,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-A930 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA24381 = {S103269, 21, { &P15683,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonT230/R101*/
const Device UA24668 = {S103270, 90, { &P15364,&P19950,&P3625,&P18092 }};	/*Useragent - SAGEM-my401X/1.0/ MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.g.2.107 (GUI) MMP/2.0*/
const Device UA24713 = {S103271, 43, { &P15742,&P17484,&P4203,&P18092 }};	/*Useragent - Samsung-SCHR610 AU-OBIGO/Q04C1-1.22 MMP/2.0*/
const Device UA26147 = {S103272, 65, { &P12156,&P17483,&P3499,&P18092 }};	/*Useragent - LG-TU750/V10a Teleca/Q7.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA88829 = {S103273, 84, { &P13628,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonZ555a/R10CA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA32792 = {S103274, 61, { &P16637,&P17483,&P17660,&P18092 }};	/*Useragent - LG-KG110 V100 AU/4.10 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA36856 = {S103275, 62, { &P17947,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player us_cellular_wwe / 1.0 / htc_lexikon / 2.2*/
const Device UA36857 = {S103276, 58, { &P17947,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player verizon_wwe / 1.0 / htc_lexikon / 2.2*/
const Device UA37115 = {S103277, 86, { &P15425,&P5448,&P17183,&P18092 }};	/*Useragent - SAMSUNG-GT-B7300/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA46690 = {S103278, 163, { &P16264,&P5129,&P3690,&P18092 }};	/*Useragent - Nokia5232/CMCC (SymbianOS/9.4; Series60/5.0; Release/12.1.091; Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) Safari/525*/
const Device UA40072 = {S103279, 114, { &P16957,&P5096,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.2;Zh_cn;XCD35;480*800;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355*/
const Device UA565320 = {S103280, 65, { &P16957,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2;Zh_cn;XCD35;480*800;)UCWEB7.8.0.95/139/355*/
const Device UA40792 = {S103281, 27, { &P13677,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-S5253/S5253DDKC1*/
const Device UA67772 = {S103282, 16, { &P13677,&P17484,&P17606,&P18092 }};	/*Useragent - samsung-gt-s5253*/
const Device UA43380 = {S103283, 77, { &P14091,&P5448,&P22272,&P18092 }};	/*Useragent - PANTECH-C810/R01 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA45000 = {S103284, 77, { &P14138,&P17017,&P3601,&P18092 }};	/*Useragent - Telkomsel F188/1.0 ACS-NF/3.3 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA46728 = {S103285, 130, { &P13144,&P6661,&P4453,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9860; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/6.1.0.60 Mobile Safari/534.11+*/
const Device UA49346 = {S103286, 40, { &P12997,&P17483,&P17602,&P18092 }};	/*Useragent - LG-MG230d Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA87641 = {S103287, 39, { &P12997,&P17483,&P17602,&P18092 }};	/*Useragent - LG-MG230 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA49922 = {S103288, 73, { &P17831,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-i909/S155 Release/2008.08.20 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA88826 = {S103289, 73, { &P17831,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-i909/S141 Release/2007.11.28 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA50224 = {S103290, 61, { &P14310,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia6108/1.0 (05.30) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA50474 = {S103291, 78, { &P13262,&P17017,&P3402,&P18092 }};	/*Useragent - BenQ-EF61/1.00/WAP2.0/MIDP2.0/CLDC1.0 UP.Browser/6.3.0.4.c.1.102 (GUI) MMP/2.0*/
const Device UA52030 = {S103292, 50, { &P16948,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-B7320/1.0 MIDP/2.1 CLDC/1.1 Mozilla/4.0*/
const Device UA53623 = {S103293, 140, { &P13994,&P17605,&P3976,&P18092 }};	/*Useragent - SAMSUNG-GT-I6320C_CMCC/1.0 RTK-E/1.0 SHP/VPP/R5 Release/17.08.2008 Browser/NetFront3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA54137 = {S103294, 121, { &P14129,&P5890,&P3439,&P18092 }};	/*Useragent - SAMSUNG-GT-I8510T/1.0 SymbianOS/9.3 Series60/3.2 Release/5.30.2008 Browser/OSS3.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA54192 = {S103295, 105, { &P14147,&P17017,&P3976,&P18092 }};	/*Useragent - SHARP/1.0/SH9010C/SHC001 Mozilla/4.08 (compatible) Browser/NF/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54390 = {S103296, 70, { &P14193,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_S743 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA54785 = {S103297, 80, { &P14260,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-X1m/S102 LMP/LML Release/2008.11.21 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA56063 = {S103298, 86, { &P14643,&P17484,&P17606,&P18092 }};	/*Useragent - WidgetManager; SAMSUNG-GT-I8320-Vodafone;AppleWebKit/528.5+(X11;U;Linux;en;i8320BUJC1)*/
const Device UA56665 = {S103299, 116, { &P14381,&P17017,&P3534,&P18092 }};	/*Useragent - Mozilla/5.0 (TY-E379/212810_7350_V0101;U;MTK/08B;Profile/MIDP-2.0 Configuration/CLDC-1.1;240X320;CTC/2.0) Obigo/Q05A*/
const Device UA62192 = {S103300, 116, { &P14381,&P17017,&P3534,&P18092 }};	/*Useragent - Mozilla/5.0 (TY-E379/212210_6480_V0701;U;MTK/08B;Profile/MIDP-2.0 Configuration/CLDC-1.1;240*320;CTC/2.0) Obigo/Q05A*/
const Device UA60553 = {S103301, 65, { &P12545,&P17017,&P3976,&P18092 }};	/*Useragent - T6/1.0 ACS-NF/3.4 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA60747 = {S103302, 125, { &P12574,&P5208,&P4902,&P18092 }};	/*Useragent - Mozilla/5.0 (SCH-F619/F619DG26;U;REX/3.0;BREW/3.1.5;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*320;CTC/2.0) POLARIS/6.15.WEB*/
const Device UA60789 = {S103303, 61, { &P12576,&P19945,&P17470,&P18092 }};	/*Useragent - Panasonic-VS2/1.0/RP1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA60993 = {S103304, 56, { &P12611,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-JACQU/00.62 UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0*/
const Device UA61057 = {S103305, 95, { &P12631,&P7472,&P3534,&P18092 }};	/*Useragent - Alcatel-MISS-SIXTY-MSX-10/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q05A*/
const Device UA61104 = {S103306, 37, { &P12630,&P17483,&P17470,&P18092 }};	/*Useragent - LG-S5200 MIC/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA61635 = {S103307, 39, { &P14311,&P17486,&P3807,&P18092 }};	/*Useragent - Mot-WX265/1.0 ObigoInternetBrowser/Q03C*/
const Device UA63277 = {S103308, 45, { &P14494,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-315M/1.0 ObigoInternetBrowser/Q03C*/
const Device UA67431 = {S103309, 118, { &P11766,&P5776,&P4715,&P18092 }};	/*Useragent - Nokia3230/2.0 (5.0717.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.4.0.65/27/351*/
const Device UA67597 = {S103310, 96, { &P12192,&P5448,&P4715,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.4.0.65*/
const Device UA67823 = {S103311, 42, { &P11684,&P5129,&P24660,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UCWEB7.6.0.75/50/800*/
const Device UA68195 = {S103312, 96, { &P14906,&P5890,&P4030,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.1; U; en-us) AppleWebKit/413 (KHTML, Like Gecko) Safari/413 Nokia/X01NK*/
const Device UA68516 = {S103313, 35, { &P14753,&P5890,&P4777,&P18092 }};	/*Useragent - Nokia N96/UC Browser7.7.1.88/28/444*/
const Device UA2278824 = {S103314, 39, { &P14519,&P5129,&P4759,&P18092 }};	/*Useragent - NokiaN97 mini/UC Browser7.6.1.82/50/351*/
const Device UA68567 = {S103315, 15, { &P14936,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHC250/1.0*/
const Device UA68716 = {S103316, 34, { &P14782,&P5890,&P4777,&P18092 }};	/*Useragent - NOKIAE61/UC Browser7.7.1.88/28/352*/
const Device UA557523 = {S103317, 35, { &P14782,&P5890,&P4777,&P18092 }};	/*Useragent - NOKIAE61i/UC Browser7.7.2.91/28/355*/
const Device UA69044 = {S103318, 43, { &P13183,&P17485,&P4777,&P18092 }};	/*Useragent - sonyericssonk810i/UC Browser7.7.1.88/69/444*/
const Device UA69074 = {S103319, 35, { &P12487,&P5890,&P4777,&P18092 }};	/*Useragent - NOKIA5700/UC Browser7.7.1.88/28/444*/
const Device UA69640 = {S103320, 75, { &P15476,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) /Kyocera-E4000*/
const Device UA70800 = {S103321, 60, { &P13349,&P17979,&P17617,&P18092 }};	/*Useragent - HTC Streaming Player cingular_us / 1.0 / htc_liberty / 2.2.2*/
const Device UA77150 = {S103322, 70, { &P11980,&P5448,&P22272,&P18092 }};	/*Useragent - HTC_P3300 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA86538 = {S103323, 128, { &P11980,&P5448,&P22272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) PPC; 240x320; HTC_P3300/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA81581 = {S103324, 101, { &P20112,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E376/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA86922 = {S103325, 17, { &P12035,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonT280i*/
const Device UA87021 = {S103326, 79, { &P16190,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) Xda_trion; 240x320*/
const Device UA87444 = {S103327, 75, { &P16241,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE;O2 Xda Flame; PPC; 480x640 )*/
const Device UA87503 = {S103328, 101, { &P20101,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-X566/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA87690 = {S103329, 84, { &P17793,&P17662,&P3601,&P18092 }};	/*Useragent - SoftBank/1.0/912T/TJ001 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87769 = {S103330, 84, { &P17789,&P17662,&P3601,&P18092 }};	/*Useragent - SoftBank/1.0/921T/TJ001 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88596 = {S103331, 47, { &P17858,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-PT35 UP.Browser/6.2.0.15.1.1 (GUI) MMP/2.0*/
const Device UA88983 = {S103332, 75, { &P15364,&P17603,&P3534,&P18092 }};	/*Useragent - PANTECH-C510/R01 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA89356 = {S103333, 45, { &P17853,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-KC3K UP.Browser/6.2.0.14.1 (GUI) MMP/2.0*/
const Device UA90224 = {S103334, 55, { &P18088,&P17017,&P21177,&P18092 }};	/*Useragent - Opera/8.01 (J2ME/MIDP; Opera Mini/3.9/1724; en; U; ssr)*/
const Device UA245046 = {S103335, 88, { &P15364,&P17017,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (en-US) AppleWebKit/534.51 (KHTML, like Gecko) Chrome/12.0.742 Safari/534.51*/
const Device UA253786 = {S103336, 9, { &P15364,&P17017,&P17251,&P18092 }};	/*Useragent - Chrome/12*/
const Device UA342185 = {S103337, 123, { &P15364,&P7133,&P24205,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.5.9225.201*/
const Device UA369476 = {S103338, 171, { &P13875,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2;en-us; ZTE-C_R750+/FRY_INT_R750PLUSV1.0.0B02; 320*480; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1)*/
const Device UA370112 = {S103339, 181, { &P16356,&P5629,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-N7000 Build/GINGERBREAD; 800*1280) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/800*/
const Device UA397843 = {S103340, 48, { &P15364,&P18575,&P22246,&P18092 }};	/*Useragent - Flipboard/1.7.2 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA398519 = {S103341, 135, { &P15364,&P17820,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.7 (KHTML, like Gecko) Ubuntu/12.04 Chromium/16.0.912.77 Chrome/16.0.912.77 Safari/535.7*/
const Device UA923739 = {S103342, 133, { &P15364,&P17820,&P17400,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.7 (KHTML, like Gecko) Ubuntu/12.04 Chromium/16.0.912.77 Chrome/16.0.912.77 Safari/535.7*/
const Device UA456157 = {S103343, 83, { &P15767,&P17779,&P18328,&P18092 }};	/*Useragent - Opera/9.80 (iPad; Opera Mini/7.1.32694/27.1407; U; en) Presto/2.8.119 Version/11.10*/
const Device UA458948 = {S103344, 164, { &P18620,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Kyros MID7022 8K Build/GRK39F; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA459156 = {S103345, 70, { &P18314,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; MEIZU_MX; 640*960) UCWEB7.9.3.103/139/800*/
const Device UA459239 = {S103346, 204, { &P15976,&P5055,&P3804,&P18092 }};	/*Useragent - Nokia801T_TD/CMCC (SymbianOS/3; Release/027.005; Series60/5.2 Mozilla/5.0; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/533.4 (KHTML, like Gecko)NokiaBrowser/7.3.1.37 Mobile Safari/533.4 3gpp-gba*/
const Device UA459284 = {S103347, 157, { &P15991,&P5629,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8 Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA460022 = {S103348, 36, { &P11905,&P5890,&P4722,&P18092 }};	/*Useragent - NOKIA6120c/UC Browser7.0.0.41/28/352*/
const Device UA469826 = {S103349, 62, { &P15364,&P17285,&P18374,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; U; en) Presto/2.6.31 Version/10.70*/
const Device UA474066 = {S103350, 73, { &P18182,&P5448,&P24643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB7.2.0.46/31/800*/
const Device UA498230 = {S103351, 139, { &P17595,&P17295,&P22443,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; en-US) AppleWebKit/532.5 (KHTML, like Gecko) Iron/4.0.275.2 Chrome/4.0.275.2 Safari/532.5*/
const Device UA517901 = {S103352, 150, { &P12280,&P17144,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8A400 Safari/6533.18.5*/
const Device UA519936 = {S103353, 37, { &P14983,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaC7-00/SymbianOS/9.1 Series60/3.0*/
const Device UA531573 = {S103354, 30, { &P11929,&P5129,&P24643,&P18092 }};	/*Useragent - Nokia5230/UCWEB7.2.2.51/50/800*/
const Device UA558841 = {S103355, 77, { &P15364,&P18145,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (X11; FreeBSD 8.2-STABLE i386; U; en) Presto/2.9.168 Version/11.50*/
const Device UA569999 = {S103356, 124, { &P16968,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; FujitsuToshibaMobileCommun; IS12T; KDDI)*/
const Device UA577783 = {S103357, 99, { &P12192,&P5448,&P24666,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB8.1.0.104/31/800*/
const Device UA609219 = {S103358, 165, { &P14186,&P5096,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en_us ; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA621845 = {S103359, 48, { &P18254,&P7480,&P24422,&P18091 }};	/*Useragent - Dolphin/1.4 (+http://otsing.delfi.ee/?c=crawler)*/
const Device UA621909 = {S103360, 36, { &P17968,&P17979,&P18551,&P18092 }};	/*Useragent - Dolphin http client/1.0(1) (Android)*/
const Device UA2033322 = {S103361, 40, { &P17968,&P17979,&P18551,&P18092 }};	/*Useragent - Dolphin http client/1.0.6(162) (Android)*/
const Device UA643925 = {S103362, 43, { &P13220,&P17485,&P4777,&P18092 }};	/*Useragent - sonyericssonk770i/UC Browser7.7.1.88/69/444*/
const Device UA1907344 = {S103363, 68, { &P17595,&P22328,&P21570,&P18092 }};	/*Useragent - Mail/1251 CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookPro8%2C2)*/
const Device UA663138 = {S103364, 43, { &P15118,&P5055,&P24663,&P18092 }};	/*Useragent - NokiaX7-00/UC Browser7.9.0.102/50/355/UCWEB*/
const Device UA716053 = {S103365, 131, { &P12496,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; I897 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA738418 = {S103366, 65, { &P17595,&P22475,&P17596,&P18092 }};	/*Useragent - Safari5530.19 CFNetwork/438.14 Darwin/9.8.0 (i386) (MacBook4%2C1)*/
const Device UA838556 = {S103367, 32, { &P13552,&P5129,&P24666,&P18092 }};	/*Useragent - NokiaC6-00/UCWEB8.1.0.104/50/800*/
const Device UA849701 = {S103368, 99, { &P12280,&P17779,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.0001.6400(iPhoneOS (iPhone4S);Mac OS X (5.1.1);en;kml:2.2;client:Free;type:default)*/
const Device UA870547 = {S103369, 96, { &P12280,&P17779,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.0001.6400(iPhoneOS (iPad2);Mac OS X (5.1.1);en;kml:2.2;client:Free;type:default)*/
const Device UA2026460 = {S103370, 139, { &P11747,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; HTC Desire Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2257270 = {S103371, 139, { &P11747,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Desire Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2260383 = {S103372, 37, { &P14826,&P5843,&P23403,&P18092 }};	/*Useragent - MQQBrowser/Mini2.6 (NokiaC1-01/04.40)*/
const Device UA900315 = {S103373, 79, { &P12671,&P5129,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V5; U; en-in; Nokia700)/UC Browser8.4.0.159/50/352/UCWEB Mobile*/
const Device UA902891 = {S103374, 39, { &P15775,&P5448,&P20767,&P18092 }};	/*Useragent - NOKIA-Lumia 800/UCWEB2.1.0.155/47/32079*/
const Device UA912329 = {S103375, 190, { &P16356,&P18066,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-N7000 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, gb*/
const Device UA1951161 = {S103376, 190, { &P16356,&P18066,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-N7000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, sg*/
const Device UA925757 = {S103377, 94, { &P16084,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; LG; GW910)*/
const Device UA1908382 = {S103378, 45, { &P13800,&P5890,&P24663,&P18092 }};	/*Useragent - NOKIA5630d-1/UC Browser7.9.0.102/28/355/UCWEB*/
const Device UA1979724 = {S103379, 37, { &P14540,&P5843,&P23401,&P18092 }};	/*Useragent - MQQBrowser/Mini2.5 (NokiaX2-01/08.63)*/
const Device UA2018178 = {S103380, 140, { &P18010,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; hd-us; A43 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2021428 = {S103381, 66, { &P14202,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.7;Zh_cn;E15i;320*480;)UCWEB7.8.0.95/139/352*/
const Device UA2021481 = {S103382, 156, { &P15575,&P5629,&P20776,&P18092 }};	/*Useragent - MQQBrowser/25 Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021682 = {S103383, 73, { &P17968,&P17979,&P17601,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Linux armv7l; rv:9.0) Gecko/ Firefox/9.0 Fennec/9.0*/
const Device UA2021893 = {S103384, 172, { &P20370,&P5629,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; ZTE-C_N760/N760V1.0.0B03; 320*480; CTC/2.0) AppleWebkit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022089 = {S103385, 136, { &P22388,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.9; en-us; AT&T HD7 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023619 = {S103386, 154, { &P14186,&P5629,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023800 = {S103387, 148, { &P14007,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; pt-pt; Racer Build/GRJ22; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025894 = {S103388, 166, { &P16356,&P18066,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-N7000 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2026110 = {S103389, 157, { &P14186,&P5629,&P20784,&P18092 }};	/*Useragent - MQQBrowser/3.3/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026116 = {S103390, 153, { &P13078,&P5096,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-U V880 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026411 = {S103391, 158, { &P15575,&P5629,&P20784,&P18092 }};	/*Useragent - MQQBrowser/3.3/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; GT-I9100G Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026557 = {S103392, 171, { &P14959,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2026787 = {S103393, 164, { &P12980,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-E510 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2026944 = {S103394, 164, { &P16049,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P698 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2029326 = {S103395, 147, { &P13505,&P24694,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MB525 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029718 = {S103396, 166, { &P13077,&P5629,&P20784,&P18092 }};	/*Useragent - MQQBrowser/3.3/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; HUAWEI C8650+ Build/V100R001C92B861) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029745 = {S103397, 163, { &P13964,&P5096,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I5510 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2031571 = {S103398, 99, { &P13051,&P17979,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/ZTE-TU880_TD/1.0 Linux/2.6.32 Android/2.2 Release/5.25.2011 Browser/AppleWebKit533.1*/
const Device UA2032497 = {S103399, 87, { &P15539,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Mail; 1.4.4) (glacier; HTC; HTC Glacier; 2.3.4/GRJ22)*/
const Device UA2032901 = {S103400, 92, { &P15539,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Messenger; 1.6.0) (glacier; HTC; HTC Glacier; 2.3.4/GRJ22)*/
const Device UA2033509 = {S103401, 160, { &P20183,&P5629,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; MI-ONE Plus Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2033579 = {S103402, 155, { &P18295,&P18066,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC One X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034322 = {S103403, 155, { &P19859,&P18066,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; HTC T328w Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2146225 = {S103404, 74, { &P13349,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2; zh-cn; HTC_Aria_A6380; 320*480) UCWEB7.9.3.103/139/800*/
const Device UA2149435 = {S103405, 131, { &P12280,&P17090,&P20778,&P18092 }};	/*Useragent - MQQBrowser/27 Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2*/
const Device UA2221857 = {S103406, 52, { &P17346,&P17090,&P17470,&P18092 }};	/*Useragent - Facebook 4110.0 (iPod touch; iPhone OS 4.3.5; en_US)*/
const Device UA2242092 = {S103407, 84, { &P18385,&P17979,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (Android Sportacular; 66-2.2.4) (Android; Nexus 7; Android OS/4.1.1)*/
const Device UA2242104 = {S103408, 84, { &P18385,&P17979,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (Android Sportacular; 67-2.2.5) (Android; Nexus 7; Android OS/4.1.2)*/
const Device UA2242266 = {S103409, 80, { &P14058,&P18110,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.1-update1; zh-cn; ZTE-C_N600; 240*320) UCWEB7.9.3.103/140/32160*/
const Device UA2260325 = {S103410, 144, { &P15364,&P17813,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Ubuntu/8.04 Chromium/8.0.552.215 Chrome/8.0.552.215 Safari/534.10*/
const Device UA2316365 = {S103411, 142, { &P14714,&P5843,&P24667,&P18092 }};	/*Useragent - NokiaX3-02/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiax3-02) UCBrowser8.2.1.144/70/355/UCWEB Mobile*/
const Device UA2317432 = {S103412, 35, { &P18267,&P5043,&P21764,&P18092 }};	/*Useragent - iBrowser/Mini2.7 (NokiaX2-02/11.57)*/
const Device UA2319289 = {S103413, 138, { &P15241,&P5843,&P24667,&P18092 }};	/*Useragent - Nokia300/5.0 (07.03) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokia300) UCBrowser8.2.1.144/70/355/UCWEB Mobile*/
const Device UA2339108 = {S103414, 79, { &P15364,&P18145,&P17216,&P18092 }};	/*Useragent - Opera/9.80 (X11; FreeBSD 9.0-RELEASE i386; U; en) Presto/2.10.289 Version/12.00*/
const Device UA2428950 = {S103415, 89, { &P14532,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-t325) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2443259 = {S103416, 169, { &P14993,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LT15i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2467096 = {S103417, 89, { &P14568,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-c105) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2476145 = {S103418, 172, { &P15881,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Incredible 2 Build/JRO03H; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2482999 = {S103419, 89, { &P15732,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-s365) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2506202 = {S103420, 98, { &P14864,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s8500) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2607522 = {S103421, 61, { &P21792,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.9; GT-I9200 Build/IML74K)*/
const Device UA2607526 = {S103422, 144, { &P21792,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-in; GT-I9200 Build/4.1.A.0.562) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2607527 = {S103423, 139, { &P21792,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.9; vi-vn; GT-I9200 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2607524 = {S103424, 199, { &P21792,&P18598,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-I9200 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.25/301,1.9.25.301,2013-02-22 16:02, +0000, pl*/
const Device UA2607535 = {S103425, 198, { &P21792,&P18598,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9200 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.26/377,1.9.26.377,2013-03-16 17:10, +0100, pl*/
const Device UA2607538 = {S103426, 139, { &P21794,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-I9152 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2609020 = {S103427, 90, { &P12879,&P6546,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.3.8.9816); (Linux; U; Android 3.1; GT-P7510 Build/HONEYCOMB_MR1);*/
const Device UA2609307 = {S103428, 164, { &P17323,&P6437,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6800 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2610857 = {S103429, 164, { &P17337,&P6437,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7500 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2613424 = {S103430, 80, { &P14908,&P17979,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.4.5) (GT-P6210;samsung;GT-P6210;3.2/HTJ85B)*/
const Device UA2617509 = {S103431, 81, { &P14908,&P17979,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.4.6) (GT-P1010;samsung;GT-P1010;2.2.1/FROYO)*/
const Device UA2615972 = {S103432, 138, { &P15985,&P6437,&P20791,&P18092 }};	/*Useragent - MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2616279 = {S103433, 139, { &P15985,&P6437,&P20791,&P18092 }};	/*Useragent - MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200L Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2623626 = {S103434, 150, { &P15985,&P6437,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2616654 = {S103435, 96, { &P15882,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Mail; 1.4.5) (espresso10wifi; samsung; GT-P5110; 4.0.4/IMM76D)*/
const Device UA2618326 = {S103436, 70, { &P15919,&P18111,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.1.1;Zh_cn;GT-P3100;600*976;)UCWEB7.8.0.95/139/351*/
const Device UA2633330 = {S103437, 119, { &P15919,&P18111,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android4.1.1;Zh_cn;GT-P3100;600*976;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/444*/
const Device UA2637912 = {S103438, 70, { &P15919,&P18111,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.1.2;Zh_cn;GT-P3100;600*976;)UCWEB7.8.0.95/139/352*/
const Device UA2619008 = {S103439, 82, { &P17337,&P17979,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (Android Sportacular; 69-2.3) (Android; GT-P7500R; Android OS/3.1)*/
const Device UA2620850 = {S103440, 84, { &P17337,&P17979,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (Android Sportacular; 69-2.3) (Android; GT-P7500R; Android OS/4.0.4)*/
const Device UA2634463 = {S103441, 83, { &P17337,&P17979,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (Android Sportacular; 76-3.0.6) (Android; GT-P7500; Android OS/3.2)*/
const Device UA2619817 = {S103442, 87, { &P17337,&P18066,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 4.0.4; GT-P7500 Build/Unknown);*/
const Device UA2621113 = {S103443, 88, { &P17337,&P18066,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 4.0.4; GT-P7500R Build/Unknown);*/
const Device UA2620544 = {S103444, 166, { &P13452,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-P1000 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355*/
const Device UA2624056 = {S103445, 166, { &P13452,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P1000 Build/JRO03R) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2620912 = {S103446, 173, { &P17337,&P6437,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7500 Build/HTJ85B; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/352*/
const Device UA2633124 = {S103447, 164, { &P15919,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3100, 40, 1.8.5)*/
const Device UA2622658 = {S103448, 101, { &P22037,&P17979,&P22073,&P18092 }};	/*Useragent - Instagram 3.3.3 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P3113; espressowifi; espresso; en_CA)*/
const Device UA2623679 = {S103449, 164, { &P17337,&P6437,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-ca; GT-P7500R Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7500R, 34, 1.6.0)*/
const Device UA2624515 = {S103450, 162, { &P17337,&P6437,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; es-us; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7500, 39, 1.8.2)*/
const Device UA2624029 = {S103451, 38, { &P15919,&P17979,&P18062,&P18092 }};	/*Useragent - Android/4.0.4; samsung GT-P3100; 2.3.1*/
const Device UA2624366 = {S103452, 208, { &P15985,&P22212,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P6200 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.16/276,1.9.16.276,2012-12-21 16:37, +0100, be*/
const Device UA2624955 = {S103453, 208, { &P12879,&P22212,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7510 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0200, ch*/
const Device UA2633923 = {S103454, 208, { &P12879,&P22212,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GT-P7510 Build/JRO03R; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0100, fr*/
const Device UA2624768 = {S103455, 166, { &P15882,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P5110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2626722 = {S103456, 52, { &P15985,&P17979,&P22080,&P18092 }};	/*Useragent - WhatsApp/2.9.369 Android/3.2 Device/samsung-GT-P6200*/
const Device UA2632105 = {S103457, 212, { &P22041,&P22212,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P6810 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0800, au*/
const Device UA2628355 = {S103458, 151, { &P22041,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P6810 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2636804 = {S103459, 149, { &P17168,&P6437,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7310 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.4.1000*/
const Device UA2634479 = {S103460, 166, { &P15882,&P18111,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P5110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2632743 = {S103461, 155, { &P14908,&P5096,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000*/
const Device UA2635823 = {S103462, 150, { &P15932,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P3110 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.53463*/
const Device UA2635062 = {S103463, 156, { &P15985,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P6200 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.1.258 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2635266 = {S103464, 170, { &P15919,&P18111,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; GT-P3100 Build/JRO03C) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2637594 = {S103465, 130, { &P15985,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P6200 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.49 Safari/537.31*/
const Device UA2636937 = {S103466, 162, { &P15932,&P18111,&P24406,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 OverDrive Media Console/2.6.4*/
const Device UA2659538 = {S103467, 85, { &P18254,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; vi; opera) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2698661 = {S103468, 97, { &P12209,&P17483,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-ks360 teleca) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2762984 = {S103469, 153, { &P11667,&P5768,&P24673,&P18092 }};	/*Useragent - Nokia6085/2.0 (06.00) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6085) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2763521 = {S103470, 89, { &P17332,&P17483,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-c310) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2764260 = {S103471, 89, { &P13782,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; es-la; lg-t310) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2844846 = {S103472, 89, { &P13782,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-t310) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2951396 = {S103473, 86, { &P13782,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; fa; lg-t310) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2767739 = {S103474, 91, { &P13734,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5603) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2958357 = {S103475, 91, { &P13734,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5603) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2774416 = {S103476, 198, { &P22871,&P18598,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9295 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.28/391,1.9.28.391,2013-04-07 20:11, +0000, pl*/
const Device UA2811742 = {S103477, 73, { &P20183,&P18110,&P24671,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; MI-ONE Plus; 480*854) UCWEB8.5.1.179/145/800*/
const Device UA2819196 = {S103478, 155, { &P16479,&P5629,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; SCH-R720 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2961544 = {S103479, 100, { &P14851,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5753e) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2963144 = {S103480, 90, { &P14568,&P17483,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-c105) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2977093 = {S103481, 136, { &P24061,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-S7270 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA2977122 = {S103482, 76, { &P24065,&P18598,&P19802,&P18092 }};	/*Useragent - AndroidDownloadManager/4.2.2 (Linux; U; Android 4.2.2; GT-S7272 Build/JDQ39)*/
const Device UA2977123 = {S103483, 138, { &P24065,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-S7272 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3008553 = {S103484, 91, { &P11653,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sgh-f480) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3013553 = {S103485, 90, { &P14471,&P17483,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-s310) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3039586 = {S103486, 200, { &P24247,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LM6200-UE; 03.21.12; 0xX;); LG NetCast.TV-2012*/
const Device UA3039589 = {S103487, 209, { &P24254,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux i686) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+PORTAL_KEY+SCREEN+TUNER; LGE; 47LM9600-NC; 09.00.00; 0xX;); LG NetCast.TV-2012*/
const Device UA3039625 = {S103488, 200, { &P24325,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LM6700-UA; 04.31.43; 0xX;); LG NetCast.TV-2012*/
const Device UA3039635 = {S103489, 202, { &P24325,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LM6700-TA; 03.12.04; 0xX;); LG NetCast.TV-2012 0*/
const Device UA3039703 = {S103490, 235, { &P24256,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL963; 64.4.08.7; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039723 = {S103491, 235, { &P24256,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL963; 74.8.30.4; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; de) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039725 = {S103492, 235, { &P24256,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL963; 74.8.30.4; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039743 = {S103493, 235, { &P24256,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL963; 82.3.59.9; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039747 = {S103494, 235, { &P24256,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL963; 64.6.47.6; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; ru) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039749 = {S103495, 235, { &P24256,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL963; 87.7.55.0; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+OFFLINEAPP+WEBSTORAGE) ; ru) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039713 = {S103496, 202, { &P24303,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_L4333; 7.1.33.0.1.1; a5; ) ; ToshibaTP/2.0.0 (+DRM) ; en) AppleWebKit/537.4 (KHTML, like Gecko) TOSHIBA-DTV (DTV_L4333; 7.1.33.0.1.1; 2013A; EU)*/
const Device UA3039728 = {S103497, 206, { &P24303,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_L4333; 7.1.37.16.01.1; a5; ) ; ToshibaTP/2.0.0 (+DRM) ; fr) AppleWebKit/537.4 (KHTML, like Gecko) TOSHIBA-DTV (DTV_L4333; 7.1.37.16.01.1; 2013A; EU)*/
const Device UA3039729 = {S103498, 206, { &P24303,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_L4333; 7.1.45.08.01.1; a5; ) ; ToshibaTP/2.0.0 (+DRM) ; fr) AppleWebKit/537.4 (KHTML, like Gecko) TOSHIBA-DTV (DTV_L4333; 7.1.45.08.01.1; 2013A; EU)*/
const Device UA3039739 = {S103499, 206, { &P24303,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_L4333; 7.1.45.22.01.1; a5; ) ; ToshibaTP/2.0.0 (+DRM) ; fr) AppleWebKit/537.4 (KHTML, like Gecko) TOSHIBA-DTV (DTV_L4333; 7.1.45.22.01.1; 2013A; EU)*/
const Device UA3039745 = {S103500, 206, { &P24303,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_L4333; 7.1.56.12.01.1; a5; ) ; ToshibaTP/2.0.0 (+DRM) ; fr) AppleWebKit/537.4 (KHTML, like Gecko) TOSHIBA-DTV (DTV_L4333; 7.1.56.12.01.1; 2013A; EU)*/
const Device UA3039762 = {S103501, 206, { &P24303,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_L4333; 7.1.68.00.01.1; a5; ) ; ToshibaTP/2.0.0 (+DRM) ; fr) AppleWebKit/537.4 (KHTML, like Gecko) TOSHIBA-DTV (DTV_L4333; 7.1.68.00.01.1; 2013A; EU)*/
const Device UA3059324 = {S103502, 99, { &P14864,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8500) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3059745 = {S103503, 90, { &P14532,&P17483,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-t325) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3068988 = {S103504, 109, { &P18634,&P18148,&P22782,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows Phone 8.0; Trident/7.0; rv:11.0; IEMobile/11.0; ARM; Touch; NOKIA; Lumia 822) like Gecko*/
const Device UA3069039 = {S103505, 171, { &P15364,&P24412,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/23.0.1271.95 Chrome/23.0.1271.95 Safari/537.6+ Ubuntu/13.10 (3.6.1-2ubuntu3) Epiphany/3.6.1*/
const Device UA3069038 = {S103506, 169, { &P15364,&P24412,&P18439,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.6+ (KHTML, like Gecko) Chromium/23.0.1271.95 Chrome/23.0.1271.95 Safari/537.6+ Ubuntu/13.10 (3.6.1-2ubuntu3) Epiphany/3.6.1*/
const Device UA3108441 = {S103507, 92, { &P23343,&P22846,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lemon t99) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3111687 = {S103508, 99, { &P15364,&P17484,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; SAMSUNG-GT-S3653) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3116561 = {S103509, 133, { &P15767,&P17146,&P24411,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Coast/1.0.1.62563 Mobile/9A405 Safari/7534.48.3*/
const Device UA870 = {S103510, 75, { &P13999,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_HD2_T8585 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA32018 = {S103511, 83, { &P13999,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) HTC_HD2_T8585/480x800*/
const Device UA2436 = {S103512, 129, { &P16054,&P5448,&P22272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) PPC; 240x320; HTC_P3600i/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA2796 = {S103513, 250, { &P12059,&P5204,&P4715,&P18092 }};	/*Useragent - Nokia2690/2.0 (09.70) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.4.0.65/70/351*/
const Device UA22722 = {S103514, 97, { &P12059,&P5204,&P4715,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2690/UC Browser7.4.0.65/70/351*/
const Device UA5796 = {S103515, 147, { &P12290,&P17483,&P17660,&P18092 }};	/*Useragent - LG/GD880 Browser/AppleWebKit/531 Widget/LGMW/3.0 MMS/LG-MMS-V1.0/1.2 MediaPlayer/LGPlayer/1.0 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA8033 = {S103516, 130, { &P16381,&P5448,&P22270,&P18092 }};	/*Useragent - PPC; 240x320; HTC_WIZARD/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA8038 = {S103517, 129, { &P16381,&P5448,&P22272,&P18092 }};	/*Useragent - PPC; 240x320; HTC_WIZARD/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA9931 = {S103518, 10, { &P16161,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia 3310*/
const Device UA10063 = {S103519, 91, { &P17330,&P5448,&P22270,&P18092 }};	/*Useragent - HTC-ST7377/2.10.502.4 (93577) Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA13250 = {S103520, 88, { &P15207,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_P3400i-Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA16931 = {S103521, 68, { &P17461,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V550/0E.65.25R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA17111 = {S103522, 139, { &P15366,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-tw; HTC Tattoo Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA17113 = {S103523, 138, { &P15366,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-tw; HTC Tattoo Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA18472 = {S103524, 61, { &P14078,&P19945,&P17470,&P18092 }};	/*Useragent - Panasonic-VS3/1.0/RP1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA19342 = {S103525, 160, { &P11950,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android BlueSparks-V5; fr-fr; SonyEricssonX10i Build/2.1.A.0.435) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA19689 = {S103526, 136, { &P15649,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1.1; en-gb; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA20666 = {S103527, 68, { &P15116,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V620/0E.65.25R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA21029 = {S103528, 54, { &P15153,&P17484,&P4901,&P18092 }};	/*Useragent - SAMSUNG-SGH-X460C/1.0 UP.Browser/6.1.0.6 (GUI) MMP/1.0*/
const Device UA21284 = {S103529, 54, { &P15181,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E630C/1.0 UP.Browser/6.2.2.6 (GUI) MMP/1.0*/
const Device UA61569 = {S103530, 53, { &P15181,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E630/1.0 UP.Browser/6.2.2.6 (GUI) MMP/1.0*/
const Device UA21408 = {S103531, 79, { &P11993,&P5776,&P3756,&P18092 }};	/*Useragent - SEC-SGHD720 SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA55027 = {S103532, 83, { &P11993,&P5776,&P3756,&P18092 }};	/*Useragent - SEC-SGHD720/1.0 SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA21749 = {S103533, 48, { &P15336,&P17017,&P17470,&P18092 }};	/*Useragent - HTIL-V70/1.0 Release/11.05.2008 Browser/Obigo2.0*/
const Device UA21879 = {S103534, 41, { &P14317,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW902/R3EG SEMC/NewsReader/1.0*/
const Device UA22364 = {S103535, 37, { &P15440,&P17483,&P17470,&P18092 }};	/*Useragent - LG-KG120 MIC/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA22375 = {S103536, 39, { &P15444,&P17483,&P17602,&P18092 }};	/*Useragent - LG-GS200 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA22487 = {S103537, 76, { &P15486,&P5208,&P17470,&P18092 }};	/*Useragent - Huawei-U526/1.0 BREW/3.1 Browser/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22951 = {S103538, 92, { &P15171,&P19950,&P3625,&P18092 }};	/*Useragent - SAGEM-myX-7/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.5.d.3 (GUI) MMP/1.0*/
const Device UA23163 = {S103539, 39, { &P15566,&P17483,&P17602,&P18092 }};	/*Useragent - LG-MG295 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA23247 = {S103540, 111, { &P15577,&P17483,&P4848,&P18092 }};	/*Useragent - LG/KE990c/v10a Browser/Obigo-Q05A/3.6 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23564 = {S103541, 52, { &P15612,&P17484,&P4901,&P18092 }};	/*Useragent - SAMSUNG-SGH-X600/K3 UP.Browser/6.1.0.6 (GUI) MMP/1.0*/
const Device UA23971 = {S103542, 48, { &P15641,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHP510/1.0 UP.Browser/6.2.2.6 (GUI) MMP/1.0*/
const Device UA68656 = {S103543, 44, { &P15641,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHP510 UP.Browser/6.2.2.6 (GUI) MMP/1.0*/
const Device UA24334 = {S103544, 71, { &P12097,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_P4350 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA24546 = {S103545, 46, { &P15708,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-AX830/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA24664 = {S103546, 53, { &P15731,&P17483,&P17660,&P18092 }};	/*Useragent - LG/KU450/v10a Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA26451 = {S103547, 38, { &P13595,&P17483,&P4203,&P18092 }};	/*Useragent - LGE-LG800 AU-OBIGO/Q04C1-1.22  MMP/2.0*/
const Device UA27257 = {S103548, 92, { &P11729,&P6173,&P4948,&P18092 }};	/*Useragent - BlackBerry8100/4.2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/120 UP.Browser/5.0.3.3*/
const Device UA27810 = {S103549, 104, { &P12190,&P17017,&P3487,&P18092 }};	/*Useragent - BenQ-E81/48 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.1.341 (GUI) MMP/2.0 Push/PO*/
const Device UA27811 = {S103550, 104, { &P12190,&P17017,&P3487,&P18092 }};	/*Useragent - BenQ-E81/52 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.6.1.421 (GUI) MMP/2.0 Push/PO*/
const Device UA30748 = {S103551, 71, { &P16460,&P22839,&P3976,&P18092 }};	/*Useragent - ZTE-F103/1.0 ACS-NF/3.4 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA31160 = {S103552, 44, { &P12157,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KU970/v1.0 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA35302 = {S103553, 43, { &P11847,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW380i/R10CA SEMC/NewsReader/1.0*/
const Device UA32876 = {S103554, 64, { &P12786,&P17483,&P3534,&P18092 }};	/*Useragent - LG-TU575/V0.9 Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA33056 = {S103555, 98, { &P12838,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-A226/ Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA33110 = {S103556, 72, { &P11840,&P17486,&P4203,&P18092 }};	/*Useragent - Motorola-V9m Obigo/Q04C1 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA33173 = {S103557, 155, { &P15523,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i520/XEGH1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA86683 = {S103558, 156, { &P15523,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 Samsung/SGH-i520V/BUGH1 Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA33218 = {S103559, 9, { &P12886,&P17017,&P17470,&P18092 }};	/*Useragent - TS705/1.0*/
const Device UA33344 = {S103560, 73, { &P12902,&P17017,&P3402,&P18092 }};	/*Useragent - BenQ-S81/1.00/WAP2.0/MIDP2.0/CLDC1.1 UP.Browser/6.3.0.3.c.4 (GUI) MMP/2.0*/
const Device UA33353 = {S103561, 37, { &P12904,&P17483,&P3866,&P18092 }};	/*Useragent - LG-KE820 MIC/1.1.14 MIDP-2.0/CLDC-1.1*/
const Device UA63477 = {S103562, 60, { &P14316,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia3300/1.0 (5.20) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA37284 = {S103563, 145, { &P12338,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; T-Mobile_Espresso Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA40857 = {S103564, 134, { &P13772,&P17979,&P20772,&P18092 }};	/*Useragent - MQQBrowser/21 Mozilla/5.0 (HW-HUAWEI_C8500/C8500V100R001C92B234;U;Android/2.1;240*320;CTC/2.0) AppleWebKit/530.17 Mobile Safari/530.17*/
const Device UA40919 = {S103565, 115, { &P13796,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) ASUS-P526/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA43781 = {S103566, 55, { &P13743,&P17017,&P3625,&P18092 }};	/*Useragent - KWC-K24/1.0.06 UP.Browser/6.2.3.9.g.1.109 (GUI) MMP/2.0*/
const Device UA84514 = {S103567, 55, { &P13743,&P17017,&P3625,&P18092 }};	/*Useragent - KWC-K24/1.0.07 UP.Browser/6.2.3.9.g.1.109 (GUI) MMP/2.0*/
const Device UA44649 = {S103568, 142, { &P16108,&P5448,&P22270,&P18092 }};	/*Useragent - MOT-A3000_CMCC/1.0 Release/09.09.2008 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/WM6.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA44657 = {S103569, 86, { &P12277,&P5448,&P17183,&P18092 }};	/*Useragent - HTC_Snap Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA46103 = {S103570, 54, { &P13294,&P17485,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 SonyEricssonW595v/R3EH SEMC/NewsReader/1.0*/
const Device UA2020522 = {S103571, 180, { &P15575,&P5629,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9100 Build/GINGERBREAD; 480*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/800*/
const Device UA49093 = {S103572, 60, { &P13898,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia6200/1.0 (4.22) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA49287 = {S103573, 13, { &P12976,&P17483,&P17660,&P18092 }};	/*Useragent - LG/KP330/v1.0*/
const Device UA50047 = {S103574, 90, { &P13152,&P19950,&P3625,&P18092 }};	/*Useragent - SAGEM-my301X/1.0/ MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.g.2.106 (GUI) MMP/2.0*/
const Device UA50189 = {S103575, 34, { &P11854,&P5890,&P4722,&P18092 }};	/*Useragent - NokiaE71/UC Browser7.0.0.41/28/352*/
const Device UA67921 = {S103576, 81, { &P11715,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW550i/R4GC Java/SEMC-Java/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA50212 = {S103577, 41, { &P13196,&P19954,&P4948,&P18092 }};	/*Useragent - SEC-SGHC100G/1.0 UP.Browser/5.0.5.1 (GUI)*/
const Device UA50238 = {S103578, 64, { &P11768,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW395/R1BA010 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA50455 = {S103579, 80, { &P12274,&P5448,&P4737,&P18092 }};	/*Useragent - SAMSUNG-SGH-i900/1.0 (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.2.2.51*/
const Device UA50470 = {S103580, 20, { &P14401,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia3510/1.0 (5.02)*/
const Device UA67404 = {S103581, 62, { &P13289,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia8910i/1.0 (05.31) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA51183 = {S103582, 174, { &P12484,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android WildPuzzleROM_2.1 Eclair v1.0.1; en-gb; HTC_Wildfire_A3333 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA51282 = {S103583, 62, { &P13943,&P5204,&P4030,&P18092 }};	/*Useragent - Nokia8800a/2.0 (07.30) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA51961 = {S103584, 77, { &P12220,&P5448,&P22271,&P18092 }};	/*Useragent - SAMSUNG-SGH-i600ORANGE/BVGD2 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA53351 = {S103585, 105, { &P13970,&P17017,&P3976,&P18092 }};	/*Useragent - SHARP/1.1/SH9020C/SHC001 Mozilla/5.00 (compatible) Browser/NF/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53883 = {S103586, 117, { &P14051,&P17605,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S5200C/1.0 RTK-E/1.0 DF/1.0 Release/02.01.2009 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA55206 = {S103587, 78, { &P14298,&P22835,&P3601,&P18092 }};	/*Useragent - Spice S950 NF-Browser/3.3 Openwave-MMS/4.5 IA-Java/1.0 Profile/MIDP2.0/CLDC1.1*/
const Device UA56006 = {S103588, 30, { &P11929,&P5129,&P24644,&P18092 }};	/*Useragent - Nokia5230/UCWEB7.4.0.57/50/800*/
const Device UA59336 = {S103589, 68, { &P14621,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-C380/0B.D1.09R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA56065 = {S103590, 86, { &P14633,&P17484,&P17606,&P18092 }};	/*Useragent - WidgetManager; SAMSUNG-GT-I6410-Vodafone;AppleWebKit/528.5+(X11;U;Linux;en;i6410BUJC1)*/
const Device UA56518 = {S103591, 72, { &P15207,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_P3400i-Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA56778 = {S103592, 36, { &P14983,&P5055,&P4715,&P18092 }};	/*Useragent - NokiaC7-00/UC Browser7.4.0.65/50/351*/
const Device UA56852 = {S103593, 57, { &P14966,&P5690,&P22570,&P18092 }};	/*Useragent - Xda_Zest_Opera/480x640 Opera/8.65 (Windows NT 5.1; U; en)*/
const Device UA58673 = {S103594, 90, { &P14246,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) HPiPAQ610/1.0 (PPC; 240x320)*/
const Device UA59543 = {S103595, 128, { &P12534,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) VDA-GPS/R1 Mozilla/4.0/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA59896 = {S103596, 25, { &P15481,&P17017,&P3601,&P18092 }};	/*Useragent - SGH-F310/1.0 NetFront/3.3*/
const Device UA60939 = {S103597, 37, { &P12597,&P17483,&P17470,&P18092 }};	/*Useragent - LG-F2400 MIC/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA61040 = {S103598, 86, { &P12615,&P17662,&P3976,&P18092 }};	/*Useragent - SoftBank/1.0/931SC/SCJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61119 = {S103599, 48, { &P14398,&P17017,&P3625,&P18092 }};	/*Useragent - SIE-CL75/01 UP.Browser/6.2.3.9.d.1 (GUI) MMP/2.0*/
const Device UA62333 = {S103600, 88, { &P14398,&P17017,&P3625,&P18092 }};	/*Useragent - SIE-CL75/01 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.9.d.1 (GUI) MMP/2.0*/
const Device UA61226 = {S103601, 125, { &P14157,&P5208,&P4902,&P18092 }};	/*Useragent - Mozilla/5.0 (SCH-S239/S239DH25;U;REX/3.0;BREW/3.1.5;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*320;CTC/2.0) POLARIS/6.15.WEB*/
const Device UA61285 = {S103602, 77, { &P12672,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; O2 Xda Stealth; PPC; 240x320)*/
const Device UA61373 = {S103603, 11, { &P12699,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHD410*/
const Device UA61392 = {S103604, 49, { &P12708,&P17485,&P3866,&P18092 }};	/*Useragent - SonyEricssonT250a/R5CA005 TelecaBrowser/1.1.14.20*/
const Device UA62274 = {S103605, 87, { &P14393,&P17017,&P17470,&P18092 }};	/*Useragent - Emegeton Enzo/1.0 TMSS-Browser/1.0.0 Profile/MIDP/2.0-compatible Configuration/CLDC-1.1*/
const Device UA62461 = {S103606, 68, { &P16434,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V361/08.B7.AER MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA62539 = {S103607, 9, { &P11945,&P17484,&P17470,&P18092 }};	/*Useragent - gt-s5233s*/
const Device UA62853 = {S103608, 14, { &P14965,&P17483,&P17660,&P18092 }};	/*Useragent - LG/KF390Q/v1.0*/
const Device UA62999 = {S103609, 80, { &P15364,&P17017,&P4893,&P18092 }};	/*Useragent - Bleu.450x/LV,A01/WAP2.0/MIDP-2.0/CLDC-1.1 Browser/UP.Browser/7.1.0.f.1.160 (GUI)*/
const Device UA63398 = {S103610, 98, { &P14511,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-A801/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA63882 = {S103611, 77, { &P15364,&P17017,&P3600,&P18092 }};	/*Useragent - E71i/MITO838 Browser/SEMC-Browser/4.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA64156 = {S103612, 37, { &P14577,&P17483,&P4203,&P18092 }};	/*Useragent - LGE-LG570 AU-OBIGO/Q04C1-1.27 MMP/2.0*/
const Device UA87831 = {S103613, 38, { &P14577,&P17483,&P4203,&P18092 }};	/*Useragent - LGE-LG570V AU-OBIGO/Q04C1-1.27 MMP/2.0*/
const Device UA64843 = {S103614, 51, { &P14710,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KG200/V01 Obigo/WAP2.0 LG-KG200/V01 Obigo/WAP2.0*/
const Device UA64892 = {S103615, 53, { &P17333,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E800/1.0 UP.Browser/6.2.2.6 (GUI) MMP/1.0*/
const Device UA66986 = {S103616, 74, { &P14741,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-M610/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA67023 = {S103617, 39, { &P14519,&P5129,&P4715,&P18092 }};	/*Useragent - NokiaN97 mini/UC Browser7.4.0.65/50/351*/
const Device UA67537 = {S103618, 135, { &P14835,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-A1600_CMCC/1.0 LinuxOS/2.4.20 Release/6.16.2008 Browser/Opera8.00 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R542_G_11.60.61P*/
const Device UA67549 = {S103619, 130, { &P14835,&P17486,&P20417,&P18092 }};	/*Useragent - MOT-A1600/1.0 LinuxOS/2.4.20 Release/1.28.2008 Browser/Opera8.00 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R542_G_11.62.01R*/
const Device UA67550 = {S103620, 133, { &P14838,&P5448,&P4578,&P18092 }};	/*Useragent - SAMSUNG-SGH-i908E_CMCC/1.0 Release/03.07 Browser/IE6 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 4.01; Windows CE; PPC)*/
const Device UA68561 = {S103621, 41, { &P14934,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-V171 UP.Browser/6.2.2.7 (GUI) MMP/1.0*/
const Device UA68654 = {S103622, 11, { &P14950,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHX120*/
const Device UA69099 = {S103623, 35, { &P11880,&P5890,&P4779,&P18092 }};	/*Useragent - NOKIA5320/UC Browser7.8.0.95/28/351*/
const Device UA69348 = {S103624, 110, { &P14345,&P16000,&P4779,&P18092 }};	/*Useragent - NokiaN-GageQD/1.0 SymbianOS/6.1 Series60/1.2 Profile/MIDP-1.0 Configuration/CLDC-1.0/UC Browser7.8.0.95/26/351*/
const Device UA69438 = {S103625, 36, { &P13648,&P5055,&P4779,&P18092 }};	/*Useragent - NokiaE7-00/UC Browser7.8.0.95/50/355*/
const Device UA69232 = {S103626, 36, { &P12588,&P5055,&P4779,&P18092 }};	/*Useragent - NokiaC6-01/UC Browser7.8.0.95/50/351*/
const Device UA69330 = {S103627, 116, { &P11658,&P5776,&P4779,&P18092 }};	/*Useragent - Nokia6680/1.0 (4.04.07) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/27/352*/
const Device UA69402 = {S103628, 35, { &P12000,&P5768,&P4779,&P18092 }};	/*Useragent - nokia6288/UC Browser7.8.0.95/69/401*/
const Device UA69535 = {S103629, 47, { &P11684,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia5800XpressMusic/SymbianOS/9.1 Series60/3.0*/
const Device UA69603 = {S103630, 141, { &P12379,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; en-ca; LG GW620R Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA72413 = {S103631, 123, { &P14075,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 640x480; Ameo/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA78048 = {S103632, 170, { &P12917,&P17605,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E2330/1.0 RTK-E/1.0 MMP/1.0 Release/12.16.2010 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA85784 = {S103633, 68, { &P15364,&P16000,&P4924,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.5; Symbian OS-Series60) NetFront/3.2*/
const Device UA85785 = {S103634, 68, { &P15364,&P16000,&P4924,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS-Series60) NetFront/3.2*/
const Device UA465412 = {S103635, 89, { &P11766,&P5776,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Nokia 3230/5.0604.0; 9399) Opera 8.65 [en]*/
const Device UA87076 = {S103636, 102, { &P17346,&P17129,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; U; CPU iPhone OS 2_0 like Mac OS X; en-us) AppleWebKit/525.18.1 (KHTML, like Gecko)*/
const Device UA87449 = {S103637, 84, { &P15142,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) SAMSUNG-SGH-i601/WM534*/
const Device UA87465 = {S103638, 71, { &P15364,&P22839,&P4924,&P18092 }};	/*Useragent - ZTE-F158/1.0 ACS-NF/3.2 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87644 = {S103639, 101, { &P23740,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-C426/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA87765 = {S103640, 84, { &P17502,&P17662,&P3601,&P18092 }};	/*Useragent - SoftBank/1.0/810T/TJ001 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87876 = {S103641, 84, { &P17794,&P17662,&P3601,&P18092 }};	/*Useragent - SoftBank/1.0/814T/TJ001 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87877 = {S103642, 84, { &P17496,&P17662,&P3601,&P18092 }};	/*Useragent - SoftBank/1.0/815T/TJ001 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87895 = {S103643, 10, { &P16989,&P22835,&P17470,&P18092 }};	/*Useragent - Spice S710*/
const Device UA88422 = {S103644, 80, { &P14418,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Alltel HTC PPC6800*/
const Device UA88695 = {S103645, 11, { &P16986,&P22835,&P17470,&P18092 }};	/*Useragent - Spice S5330*/
const Device UA88898 = {S103646, 11, { &P16924,&P22835,&P17470,&P18092 }};	/*Useragent - Spice S5420*/
const Device UA89070 = {S103647, 75, { &P20212,&P17483,&P17470,&P18092 }};	/*Useragent - LG-PLS150 AU-MIC-PLS150/1.0 MMP/2.0 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA90311 = {S103648, 78, { &P15364,&P7133,&P18373,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.1; Opera Mobi/7457; U; en) Presto/2.5.28 Version/10.1*/
const Device UA90553 = {S103649, 128, { &P20581,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; ja-jp; L-06C Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA243778 = {S103650, 43, { &P12080,&P17484,&P17338,&P18092 }};	/*Useragent - samsung-gt-s3653/UC Browser7.9.0.102/69/352*/
const Device UA253874 = {S103651, 62, { &P13542,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW830i/R1KG Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA253908 = {S103652, 81, { &P11627,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonK850i/R2AG SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA253935 = {S103653, 43, { &P13179,&P17485,&P4777,&P18092 }};	/*Useragent - sonyericssonw350i/UC Browser7.7.1.88/69/444*/
const Device UA253957 = {S103654, 40, { &P12142,&P17485,&P4779,&P18092 }};	/*Useragent - SonyEricssonU5/UC Browser7.8.0.95/50/351*/
const Device UA437925 = {S103655, 76, { &P17595,&P18584,&P18537,&P18092 }};	/*Useragent - Reeder/1010.59.00 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookAir4%2C2)*/
const Device UA589895 = {S103656, 76, { &P17595,&P18584,&P18537,&P18092 }};	/*Useragent - Reeder/1010.59.00 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (MacBookPro8%2C1)*/
const Device UA728632 = {S103657, 71, { &P17595,&P18584,&P18537,&P18092 }};	/*Useragent - Reeder/1010.69.00 CFNetwork/520.3.2 Darwin/11.3.0 (x86_64) (iMac11%2C1)*/
const Device UA438377 = {S103658, 142, { &P12482,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SGH-T959 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA605709 = {S103659, 142, { &P12482,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; SGH-T959 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA454810 = {S103660, 136, { &P12879,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA821690 = {S103661, 136, { &P12879,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P7510 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2622500 = {S103662, 136, { &P12879,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7*/
const Device UA458452 = {S103663, 124, { &P15767,&P17090,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.8.3*/
const Device UA605074 = {S103664, 124, { &P15767,&P17090,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.8.4*/
const Device UA605481 = {S103665, 124, { &P15767,&P17090,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.8.4*/
const Device UA650734 = {S103666, 124, { &P15767,&P17090,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.8.3*/
const Device UA459250 = {S103667, 37, { &P15988,&P17484,&P23399,&P18092 }};	/*Useragent - MQQBrowser/Mini2.2 (SAMSUNG-GT-C3312)*/
const Device UA459303 = {S103668, 156, { &P15991,&P5629,&P20775,&P18092 }};	/*Useragent - MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA559543 = {S103669, 74, { &P15201,&P18110,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;2.3.3;Zh_cn;HTC Sensation Z710e;540*960;)UCWEB7.8.1.96/139/800*/
const Device UA570932 = {S103670, 53, { &P13706,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (Nokia5235;SymbianOS/9.1 Series60/3.0)*/
const Device UA575046 = {S103671, 42, { &P14753,&P5890,&P24663,&P18092 }};	/*Useragent - Nokia N96/UC Browser7.9.0.102/28/355/UCWEB*/
const Device UA576443 = {S103672, 143, { &P17595,&P17297,&P22492,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_2; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.3.4641 Chrome/7.0.517.450 Safari/534.7*/
const Device UA2166369 = {S103673, 143, { &P17595,&P17297,&P22492,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7_4; en-US) AppleWebKit/534.7 (KHTML, like Gecko) Flock/3.5.3.4641 Chrome/7.0.517.450 Safari/534.7*/
const Device UA580176 = {S103674, 143, { &P17968,&P5096,&P21558,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/1.9.4 Safari/533.1*/
const Device UA589974 = {S103675, 76, { &P17595,&P22328,&P18537,&P18092 }};	/*Useragent - Reeder/1010.59.00 CFNetwork/520.2.5 Darwin/11.2.0 (x86_64) (MacBookPro8%2C2)*/
const Device UA602632 = {S103676, 87, { &P18115,&P5764,&P4860,&P18092 }};	/*Useragent - Opera/9.80 (BlackBerry; Opera Mini/6.24209/27.1366; U; en) Presto/2.8.119 Version/11.10*/
const Device UA619061 = {S103677, 99, { &P12280,&P17084,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]*/
const Device UA622191 = {S103678, 39, { &P17968,&P17979,&P24434,&P18092 }};	/*Useragent - Dolphin http client/8.3.0(65) (Android)*/
const Device UA2220114 = {S103679, 52, { &P17346,&P17146,&P17470,&P18092 }};	/*Useragent - Instagram 2.5.0 (iPod touch; iPhone OS 5.0.1; en_US)*/
const Device UA692099 = {S103680, 32, { &P12088,&P5129,&P24666,&P18092 }};	/*Useragent - NokiaX6-00/UCWEB8.1.0.104/50/999*/
const Device UA2148345 = {S103681, 31, { &P15373,&P5129,&P24666,&P18092 }};	/*Useragent - Nokia5530/UCWEB8.1.0.104/50/800*/
const Device UA733121 = {S103682, 130, { &P17595,&P17297,&P19990,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) Version/5.1 Safari/7534.51.22 Sleipnir/3.4*/
const Device UA697608 = {S103683, 150, { &P15649,&P5096,&P20775,&P18092 }};	/*Useragent - MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA697900 = {S103684, 56, { &P15364,&P17017,&P22643,&P18092 }};	/*Useragent - Mozilla/5.0 (X11) KHTML/4.7.4 (like Gecko) Konqueror/4.7*/
const Device UA708937 = {S103685, 131, { &P15767,&P17146,&P18356,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3 Sleipnir/1.9.2m*/
const Device UA758068 = {S103686, 30, { &P14454,&P5890,&P24666,&P18092 }};	/*Useragent - NokiaE66/UCWEB8.1.0.104/28/800*/
const Device UA765910 = {S103687, 142, { &P15767,&P17145,&P20772,&P18092 }};	/*Useragent - MQQBrowser/2.1 Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/7534.48.3*/
const Device UA776847 = {S103688, 145, { &P16321,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; X500 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA800004 = {S103689, 35, { &P16003,&P17484,&P4779,&P18092 }};	/*Useragent - gt-s5233a/UC Browser7.8.0.95/69/351*/
const Device UA864898 = {S103690, 123, { &P15364,&P7133,&P24204,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.3.8908.201*/
const Device UA896182 = {S103691, 108, { &P17595,&P18120,&P19840,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.5.0 Safari/534.34*/
const Device UA925705 = {S103692, 81, { &P16052,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; XV6975; Windows Phone 6.5.3.5)*/
const Device UA925836 = {S103693, 95, { &P18293,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; ZTE; Tania)*/
const Device UA925863 = {S103694, 87, { &P16968,&P15924,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;FujitsuToshibaMobileCommun;IS12T;7.10;8107)*/
const Device UA1903684 = {S103695, 68, { &P13633,&P17486,&P24663,&P18092 }};	/*Useragent - JUC (Linux; U; 2.2; zh-cn; MotoA953; 480*854) UCWEB7.9.3.103/139/800*/
const Device UA1929146 = {S103696, 32, { &P13324,&P5055,&P24666,&P18092 }};	/*Useragent - NokiaN8-00/UCWEB8.1.0.104/51/800*/
const Device UA1944914 = {S103697, 133, { &P15767,&P17090,&P22581,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-US) AppleWebKit/534.12 (KHTML, like Gecko) Puffin/1.3.4232M Mobile Safari/534.12*/
const Device UA1948328 = {S103698, 148, { &P15649,&P5096,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA1959783 = {S103699, 136, { &P14574,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; Nexus S Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2029631 = {S103700, 136, { &P14574,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Nexus S Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA1991123 = {S103701, 193, { &P15575,&P5629,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, gb*/
const Device UA2032445 = {S103702, 193, { &P15575,&P5629,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.8.4/105,1.8.4.105,2012-06-26 17:19, in*/
const Device UA1995188 = {S103703, 39, { &P18176,&P5448,&P24632,&P18092 }};	/*Useragent - NOKIA-Lumia 900/UCWEB2.3.0.182/47/32079*/
const Device UA2018728 = {S103704, 138, { &P15759,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; HTC Vision Build/ERD62) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018743 = {S103705, 178, { &P14334,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Cricket Eclair 2.1-update1; en-us; SAMSUNG Intercept SPH-M910 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019804 = {S103706, 135, { &P18008,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; es-es; SmartQT10 Build/MID) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022458 = {S103707, 131, { &P23354,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; th-th; A3 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023715 = {S103708, 153, { &P13077,&P5629,&P20780,&P18092 }};	/*Useragent - MQQBrowser/2.9/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HW-HUAWEI_C8650/C8650V100R001C92B825; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2024083 = {S103709, 153, { &P13077,&P5629,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HW-HUAWEI_C8650/C8650V100R001C92B825; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2024138 = {S103710, 141, { &P21862,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0UI; en-us; MAXQ X7 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025077 = {S103711, 171, { &P12972,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; SGH-T989 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2025221 = {S103712, 169, { &P15047,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; SK17i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2025900 = {S103713, 160, { &P16957,&P5096,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; XCD35 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2026314 = {S103714, 157, { &P14959,&P5629,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I9103 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026647 = {S103715, 168, { &P12968,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT26i Build/6.1.A.0.452) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2026705 = {S103716, 161, { &P14334,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SPH-M910 Build/GINGERBREAD; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027521 = {S103717, 155, { &P12798,&P5629,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; LT18i Build/4.0.2.A.0.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028909 = {S103718, 168, { &P12798,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; LT18i Build/4.1.B.0.431) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2032860 = {S103719, 176, { &P13064,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SAMSUNG-SGH-I727 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2034221 = {S103720, 160, { &P20158,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; A7 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2034507 = {S103721, 160, { &P15501,&P5477,&P21760,&P18092 }};	/*Useragent - iBrowser/2.8 Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.53) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2157119 = {S103722, 37, { &P14708,&P5204,&P21764,&P18092 }};	/*Useragent - iBrowser/Mini2.7 (Nokia5130c-2/07.97)*/
const Device UA2163808 = {S103723, 79, { &P16072,&P5890,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (S60V3; U; en-us; NOKIAN78)/UC Browser8.4.0.159/28/352/UCWEB Mobile*/
const Device UA2313675 = {S103724, 37, { &P14540,&P5843,&P23404,&P18092 }};	/*Useragent - MQQBrowser/Mini2.7 (NokiaX2-01/08.63)*/
const Device UA2318193 = {S103725, 99, { &P15364,&P18146,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; SunOS i86pc) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.810.0 Safari/535.1*/
const Device UA2333092 = {S103726, 142, { &P13107,&P5043,&P24667,&P18092 }};	/*Useragent - NokiaC2-06/2.0 (07.48) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac2-06) UCBrowser8.2.1.144/70/355/UCWEB Mobile*/
const Device UA2338006 = {S103727, 142, { &P15364,&P17838,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.10 (KHTML, like Gecko) Ubuntu/9.10 Chromium/8.0.558.0 Chrome/8.0.558.0 Safari/534.10*/
const Device UA2344906 = {S103728, 39, { &P18176,&P5448,&P20763,&P18092 }};	/*Useragent - NOKIA-Lumia 900/UCWEB2.4.0.188/47/32079*/
const Device UA2395960 = {S103729, 147, { &P14799,&P5096,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I5500 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2410536 = {S103730, 118, { &P15364,&P16110,&P19840,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.5.1 (development) Safari/534.34*/
const Device UA2482968 = {S103731, 89, { &P14471,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-s310) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2483508 = {S103732, 175, { &P11841,&P5204,&P24673,&P18092 }};	/*Useragent - Nokia5220XpressMusic/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia5220xpressmusic) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2484383 = {S103733, 99, { &P14060,&P17485,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; sonyericssonw150i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2506776 = {S103734, 90, { &P14763,&P5776,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokian72) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2579922 = {S103735, 89, { &P14568,&P17483,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-c105) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2599339 = {S103736, 99, { &P16366,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8600) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2610501 = {S103737, 137, { &P15985,&P6437,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B)Maxthon AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2610730 = {S103738, 137, { &P17323,&P6437,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6800 Build/HTJ85B)Maxthon AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2617525 = {S103739, 160, { &P17323,&P6437,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-ph; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Mobile Maxthon Mobile Maxthon*/
const Device UA2612938 = {S103740, 129, { &P17337,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2616332 = {S103741, 130, { &P17337,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2631905 = {S103742, 136, { &P17337,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-gb; GT-P7500 Build/ERD62) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2613741 = {S103743, 151, { &P17323,&P6437,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 MobileSafari/534.13*/
const Device UA2614365 = {S103744, 181, { &P12879,&P6437,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.8.4/105,1.8.4.105,2012-06-26 17:19, gb*/
const Device UA2615672 = {S103745, 183, { &P15882,&P18066,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, nl*/
const Device UA2621588 = {S103746, 190, { &P15882,&P18066,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.8.4/102,1.8.4.102,2012-06-23 14:36, china, au*/
const Device UA2616106 = {S103747, 140, { &P15919,&P18066,&P20791,&P18092 }};	/*Useragent - MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2619109 = {S103748, 140, { &P15919,&P18066,&P20791,&P18092 }};	/*Useragent - MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2625510 = {S103749, 152, { &P15919,&P18066,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2627856 = {S103750, 152, { &P15919,&P18066,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2616651 = {S103751, 88, { &P15882,&P17979,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.4.5) (espresso10wifi;samsung;GT-P5110;4.0.4/IMM76D)*/
const Device UA2617536 = {S103752, 101, { &P22037,&P17979,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P3113; espressowifi; espresso; en_US)*/
const Device UA2617726 = {S103753, 145, { &P17323,&P6437,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2617971 = {S103754, 166, { &P15932,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P3110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.0.181/145/352*/
const Device UA2619383 = {S103755, 166, { &P15932,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2636380 = {S103756, 147, { &P17337,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2619127 = {S103757, 147, { &P15882,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2635089 = {S103758, 171, { &P15882,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [UsableNet Lift Mobile]*/
const Device UA2621674 = {S103759, 96, { &P17323,&P17979,&P22071,&P18092 }};	/*Useragent - Instagram 3.1.0 Android (13/3.2; 160dpi; 800x1232; samsung; GT-P6800; GT-P6800; smdkc210; en_GB)*/
const Device UA2619629 = {S103760, 197, { &P15919,&P18066,&P17374,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2622169 = {S103761, 197, { &P15919,&P18066,&P17374,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2630412 = {S103762, 106, { &P22043,&P17979,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 Android (16/4.1.1; 160dpi; 800x1232; samsung; GT-P5113; espresso10wifi; espresso10; en_US)*/
const Device UA2620405 = {S103763, 168, { &P15876,&P18066,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]*/
const Device UA2622643 = {S103764, 168, { &P15876,&P18066,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]*/
const Device UA2620964 = {S103765, 172, { &P12889,&P6546,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7300 Build/HMJ37; 800*1280) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/352*/
const Device UA2621448 = {S103766, 101, { &P15932,&P17979,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 Android (15/4.0.3; 160dpi; 600x976; samsung; GT-P3110; espressowifi; espresso; en_GB)*/
const Device UA2622067 = {S103767, 71, { &P17168,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.4;Zh_cn;GT-P7310;800*1232;)UCWEB7.8.0.95/139/352*/
const Device UA2623402 = {S103768, 166, { &P14908,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P6210 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355*/
const Device UA2625285 = {S103769, 54, { &P17323,&P17979,&P22080,&P18092 }};	/*Useragent - WhatsApp/2.9.369 Android/4.0.4 Device/samsung-GT-P6800*/
const Device UA2625544 = {S103770, 52, { &P12889,&P17979,&P22080,&P18092 }};	/*Useragent - WhatsApp/2.9.369 Android/3.2 Device/samsung-GT-P7300*/
const Device UA2625660 = {S103771, 152, { &P15919,&P18111,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2629832 = {S103772, 140, { &P15919,&P18111,&P20791,&P18092 }};	/*Useragent - MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2626252 = {S103773, 157, { &P13452,&P5629,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2629434 = {S103774, 157, { &P13452,&P5629,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2633640 = {S103775, 145, { &P13452,&P5629,&P20791,&P18092 }};	/*Useragent - MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2634273 = {S103776, 157, { &P13452,&P5629,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2630373 = {S103777, 155, { &P15932,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0(Linux; U; Android 4.0.4; zh-CN; GT-P3110 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.0.212 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2635634 = {S103778, 156, { &P15932,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-CN; GT-P3110 Build/IML74K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2628552 = {S103779, 151, { &P17323,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2628725 = {S103780, 151, { &P15876,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P5100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2629073 = {S103781, 132, { &P22052,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P7501 Build/JOP40D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2634615 = {S103782, 131, { &P22052,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7501 Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2630813 = {S103783, 210, { &P13452,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-P1000 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0000, ie*/
const Device UA2632408 = {S103784, 209, { &P13452,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; GT-P1000 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0100, fr*/
const Device UA2630033 = {S103785, 152, { &P22043,&P18111,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2630745 = {S103786, 130, { &P22041,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P6810 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.87 Safari/537.22*/
const Device UA2633269 = {S103787, 130, { &P22041,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P6810 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22*/
const Device UA2631522 = {S103788, 148, { &P17337,&P6546,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; es-es; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.3.3000*/
const Device UA2634096 = {S103789, 155, { &P17337,&P6546,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13 Maxthon/4.0.3.3000*/
const Device UA2635493 = {S103790, 156, { &P15876,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-P5100 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2633713 = {S103791, 130, { &P19852,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7511 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22*/
const Device UA2634738 = {S103792, 171, { &P14908,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-P1010 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2636641 = {S103793, 162, { &P15882,&P18111,&P24406,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 OverDrive Media Console/2.6.4*/
const Device UA2637142 = {S103794, 166, { &P14908,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P6210 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2637842 = {S103795, 166, { &P14908,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6210 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2654883 = {S103796, 91, { &P12599,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-lg800g) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2730504 = {S103797, 96, { &P14353,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; ru; samsung-gt-s5250) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2665207 = {S103798, 90, { &P22957,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-a350) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2681000 = {S103799, 207, { &P22385,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.19/284,1.9.19.284,2013-01-17 18:10, +0800, china, cn*/
const Device UA2681014 = {S103800, 139, { &P22385,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I8262D Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2702519 = {S103801, 91, { &P13847,&P17483,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; lg-lg900g) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2735187 = {S103802, 99, { &P15197,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-b5310) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2747058 = {S103803, 90, { &P12051,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5600) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2752307 = {S103804, 92, { &P13602,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233w) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2797769 = {S103805, 92, { &P13602,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5233w) U2/1.0.0 UCBrowser/9.0.1.303 U2/1.0.0 Mobile*/
const Device UA2861949 = {S103806, 89, { &P13602,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; vi; gt-s5233w) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2779552 = {S103807, 99, { &P12369,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s7070) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2792285 = {S103808, 91, { &P13310,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gt350) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2801274 = {S103809, 138, { &P22958,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-US780 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2809214 = {S103810, 35, { &P14669,&P5843,&P21764,&P18092 }};	/*Useragent - iBrowser/Mini2.7 (NokiaX2-00/08.25)*/
const Device UA2847306 = {S103811, 87, { &P15364,&P17017,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; opera) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA3044090 = {S103812, 95, { &P15364,&P17017,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; 11a h700tecno) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2960975 = {S103813, 99, { &P12260,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5350) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2977065 = {S103814, 133, { &P24088,&P18110,&P19800,&P18092 }};	/*Useragent - Mozilla/5.0 (DTV) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.5 AQUOSBrowser/1.0 (AS00DTV;V;0001;0001) AQUOS-AS/2.0 LC-46LE840X*/
const Device UA2977118 = {S103815, 136, { &P24064,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-S7275 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Mobile Safari/537.36*/
const Device UA3021020 = {S103816, 91, { &P13365,&P17483,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-gu220) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3030147 = {S103817, 91, { &P11996,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5560) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3039603 = {S103818, 200, { &P24294,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LM7600-TA; 03.10.07; 0xX;); LG NetCast.TV-2012*/
const Device UA3039680 = {S103819, 197, { &P24286,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL838; 55.4.02.0; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039686 = {S103820, 147, { &P24286,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL838; 48.4.72.8; t12; ) ; ToshibaTP/1.1.0 () ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039698 = {S103821, 197, { &P24286,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL838; 55.4.02.0; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM) ; pl) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039718 = {S103822, 197, { &P24286,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL838; 73.0.51.8; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039751 = {S103823, 197, { &P24286,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_RL838; 85.3.88.6; t12; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3068866 = {S103824, 135, { &P19904,&P24371,&P24415,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.4.5; en-us; C5170 Build/IML77) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3068877 = {S103825, 233, { &P12280,&P24384,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_1_6 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10A403 [FBAN/FBIOS;FBAV/5.5;FBBV/X;FBDV/iPhone3,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/7.1.6;FBSS/2; FBCR/;FBID/phone;FBLC/en_US;FBOP/0]*/
const Device UA3068880 = {S103826, 242, { &P12280,&P24384,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_1_4 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B143 [FBAN/FBIOS;FBAV/6.2;FBBV/X;FBDV/iPhone5,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/6.1.4;FBSS/2; FBCR/VinaPhone;FBID/phone;FBLC/en_US;FBOP/1]*/
const Device UA3068881 = {S103827, 234, { &P12280,&P24384,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11D43 [FBAN/FBIOS;FBAV/6.4;FBBV/X;FBDV/iPhone6,1;FBMD/iPhone;FBSN/iPhone OS;FBSV/7.1;FBSS/2; FBCR/AT&T;FBID/phone;FBLC/en_US;FBOP/1]*/
const Device UA3068941 = {S103828, 140, { &P21648,&P21637,&P24394,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; GT-I9505G Build/JWR66V.S014) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1612.0 Mobile Safari/537.36*/
const Device UA3074500 = {S103829, 90, { &P15732,&P17483,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-s365) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3084120 = {S103830, 162, { &P18672,&P5629,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-S5302 Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3114674 = {S103831, 99, { &P15547,&P17484,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; SAMSUNG-GT-C6712) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3126275 = {S103832, 87, { &P15364,&P17017,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; j2me) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA4153 = {S103833, 76, { &P17467,&P5448,&P22270,&P18092 }};	/*Useragent - SAMSUNG-SCH-M490/(null)IC4 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA4562 = {S103834, 92, { &P13316,&P17017,&P3807,&P18092 }};	/*Useragent - Mozilla/4.0(MTS/1.0/MTC547/VF_RU_RU_P107A4V1.2.1/Obigo Q03C/MIDP-2.0 Configuration/CLDC-1.1)*/
const Device UA4737 = {S103835, 145, { &P12173,&P5448,&P4808,&P18092 }};	/*Useragent - LG-CT810/V10x IEMobile/7.11 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA7612 = {S103836, 168, { &P17944,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android SleekSense-Rc2-2.2; en-us; USCCADR6275US Carrier ID 45 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA10096 = {S103837, 145, { &P12530,&P5448,&P4808,&P18092 }};	/*Useragent - PPC; 240x320; HTC_P3450/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA15619 = {S103838, 166, { &P11826,&P5448,&P3430,&P18092 }};	/*Useragent - (Opera) Vodafone/1.0/HPiPAQDataMessenger/1.00.00 Browser/Opera/9.5 Profile/MIDP-2.0 Configuration/CLDC-1.1 Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/15142; U; en)*/
const Device UA16422 = {S103839, 140, { &P14225,&P17295,&P17316,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7; ja-jp; 001HT Build/FRF91) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Safari/530.17*/
const Device UA63402 = {S103840, 68, { &P14158,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V547/08.18.40R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA19720 = {S103841, 162, { &P14195,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia6788i/CMCC SymbianOS/9.3 Series60/3.2 Release/19.013.314.1; Mozilla/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML,like Gecko) Safari/413*/
const Device UA77043 = {S103842, 44, { &P15021,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP233/V10a Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA20645 = {S103843, 146, { &P15109,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 LGKT615/v10A; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA90593 = {S103844, 146, { &P15109,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 LGKT615/v10C; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA20740 = {S103845, 41, { &P15131,&P22832,&P17470,&P18092 }};	/*Useragent - VIDEOCON_V1616/WAP2.0 Profile/MT6253/QVGA*/
const Device UA35888 = {S103846, 130, { &P11961,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-D508/1.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzU2MDk1MDA1MjY5NTE5 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA21137 = {S103847, 118, { &P15163,&P17605,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-U808E/1.0 RTK-E/1.0 DF/1.0 Release/04.23.2008 Browser/NetFront3.4  Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21282 = {S103848, 36, { &P15179,&P17483,&P17470,&P18092 }};	/*Useragent - LG-G262 MIC/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA21432 = {S103849, 226, { &P15280,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTCA3360_TD/1.0 Android/2.2 release/10.1.2010 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA21442 = {S103850, 37, { &P15283,&P17483,&P17470,&P18092 }};	/*Useragent - LG-C1150 MIC/WAP2.0 MIDP-2.0/CLDC-1.0*/
const Device UA21522 = {S103851, 36, { &P15299,&P17483,&P3534,&P18092 }};	/*Useragent - LG10000/1.0 TELECA/Q05A1-3.8 MMP/2.0*/
const Device UA21660 = {S103852, 22, { &P15318,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonT290i/R101*/
const Device UA21739 = {S103853, 48, { &P15330,&P7472,&P4893,&P18092 }};	/*Useragent - Alcatel-OT-S853/1.0 UP.Browser/7.1 (GUI) MMP/2.0*/
const Device UA21752 = {S103854, 37, { &P15339,&P17483,&P17470,&P18092 }};	/*Useragent - LG-B2150 MIC/WAP2.0 MIDP-2.0/CLDC-1.0*/
const Device UA21913 = {S103855, 58, { &P12015,&P17017,&P4222,&P18092 }};	/*Useragent - SANYO-S750/2.130 UP.Browser/7.0.2.1.sn.1.222 (GUI) MMP/2.0*/
const Device UA50991 = {S103856, 58, { &P12015,&P17017,&P4222,&P18092 }};	/*Useragent - SANYO-S750/1.810 UP.Browser/7.0.2.1.sn.1.159 (GUI) MMP/2.0*/
const Device UA22084 = {S103857, 101, { &P15383,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E236/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA22352 = {S103858, 23, { &P20184,&P19954,&P3816,&P18092 }};	/*Useragent - SEC-SGHX506/1.0 TSS/2.5*/
const Device UA24649 = {S103859, 20, { &P15728,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-X300/1.0*/
const Device UA26509 = {S103860, 88, { &P15293,&P5448,&P4808,&P18092 }};	/*Useragent - HTC MAX 4G Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA26879 = {S103861, 90, { &P13649,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHZX20 SHP/CDR-2.2 NetFront/3.2 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA27021 = {S103862, 107, { &P12160,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S8000T/1.0 SHP/R5 NetFront/3.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA28444 = {S103863, 61, { &P12247,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia6650/2.0 (05.40) Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA28483 = {S103864, 175, { &P15377,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) LG-KS20/V10a Browser/IEMobile/7.6 MMS/LG-MMS-WINCE-V1.0/1.2 Java/LGVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA36974 = {S103865, 188, { &P15377,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Vodafone/1.0/LG-KS20/V10b Browser/IEMobile/7.6 MMS/LG-MMS-WINCE-V1.0/1.2 Java/LGVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA28980 = {S103866, 73, { &P12027,&P5448,&P4643,&P18092 }};	/*Useragent - HTC_Polaris Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA87370 = {S103867, 77, { &P12027,&P5448,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) 1.0/HTC_Polaris*/
const Device UA29864 = {S103868, 50, { &P15425,&P17350,&P3430,&P18092 }};	/*Useragent - Opera 9.5(Windows NT;  U; en) SAMSUNG-GT-B7300/1.0*/
const Device UA32868 = {S103869, 29, { &P13923,&P19954,&P3816,&P18092 }};	/*Useragent - SEC-SGHX507, TSS/2.5, Rev 1.4*/
const Device UA51001 = {S103870, 23, { &P13923,&P19954,&P3816,&P18092 }};	/*Useragent - SEC-SGHX507/1.0 TSS/2.5*/
const Device UA33079 = {S103871, 142, { &P12846,&P5208,&P4732,&P18092 }};	/*Useragent - Mozilla/5.0 (HUAWEI/S6510V100R001C1X;U;REX/1.0;Opera Mini/4.2;BREW/3.1.5;Profile/MIDP-2.0 Configuration/CLDC-1.1;320*240;CTC/2.0) NetFront/3.5*/
const Device UA33111 = {S103872, 143, { &P12854,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; pt-br; LG GW620F Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA72922 = {S103873, 68, { &P12870,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V555/08.18.40R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA33391 = {S103874, 39, { &P12910,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP200 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA34322 = {S103875, 115, { &P13671,&P17605,&P3976,&P18092 }};	/*Useragent - SAMSUNG-M3510C/1.0 RTK-E/1.0 DF/1.0 Release/09.01.2008 Browser/NetFront3.4  Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA36966 = {S103876, 67, { &P16969,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-E1210M/E1210MRYIC2 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA41877 = {S103877, 99, { &P12897,&P5843,&P4779,&P18092 }};	/*Useragent - NokiaC2-00/2.0 (03.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-00/UC Browser7.8.0.95/70/401*/
const Device UA54216 = {S103878, 83, { &P14154,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonK818c/R6BC Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA42700 = {S103879, 86, { &P17782,&P17662,&P3601,&P18092 }};	/*Useragent - SoftBank/1.0/910SH/SHJ002 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87849 = {S103880, 42, { &P17782,&P17662,&P3601,&P18092 }};	/*Useragent - Mozilla/4.08 (910SH;SoftBank) NetFront/3.3*/
const Device UA43345 = {S103881, 77, { &P15282,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; HTC-P5310 BM ) Opera 8.65 [en]*/
const Device UA43764 = {S103882, 67, { &P13737,&P17017,&P4924,&P18092 }};	/*Useragent - F230/1.0 ACS-NF/3.2 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA43853 = {S103883, 138, { &P13755,&P17486,&P3581,&P18092 }};	/*Useragent - MOT-MOTOROKR-U9/Mozilla/4.0 (compatible;MSIE 6.0;Linux MOTOROKR U9)/R6632_G_81.xx.yyI Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony 1.0*/
const Device UA48378 = {S103884, 143, { &P22233,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; ME861 Build/OLYKC_U4_1.15.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA49136 = {S103885, 47, { &P16617,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-VX9800/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA49314 = {S103886, 97, { &P12983,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-C416 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA49373 = {S103887, 27, { &P14010,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-S5263/S5263DDKE1*/
const Device UA68283 = {S103888, 16, { &P14010,&P17484,&P17606,&P18092 }};	/*Useragent - samsung-gt-s5263*/
const Device UA49434 = {S103889, 61, { &P13023,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-C161/1.0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA49562 = {S103890, 119, { &P13057,&P5043,&P4030,&P18092 }};	/*Useragent - NokiaC3-03/5.0 (p) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 AppleWebKit/420+ (KHTML, like Gecko) Safari/420+*/
const Device UA49718 = {S103891, 75, { &P13076,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry8600/5.0.0.564 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/1*/
const Device UA49893 = {S103892, 125, { &P13101,&P5208,&P4902,&P18092 }};	/*Useragent - Mozilla/5.0 (SCH-S569/S569DF09;U;REX/3.0;BREW/3.1.5;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*320;CTC/2.0) POLARIS/6.15.WEB*/
const Device UA50044 = {S103893, 43, { &P13151,&P17483,&P3625,&P18092 }};	/*Useragent - LG8600/1.1 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA87461 = {S103894, 43, { &P13151,&P17483,&P3625,&P18092 }};	/*Useragent - LG8600/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA53603 = {S103895, 35, { &P13992,&P17017,&P4924,&P18092 }};	/*Useragent - HUAWEI-M318/001.00 ACS-NetFront/3.2*/
const Device UA53824 = {S103896, 69, { &P14040,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHI321N/1.0 jBrowser/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54101 = {S103897, 81, { &P14120,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-S710/S100 LMP/LML Release/2010.05.22 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA54209 = {S103898, 102, { &P14150,&P17485,&P4223,&P18092 }};	/*Useragent - SonyEricssonX5/R1DA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.0*/
const Device UA54484 = {S103899, 74, { &P14228,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) ASUSM930/1.0*/
const Device UA54596 = {S103900, 15, { &P15437,&P17484,&P17606,&P18092 }};	/*Useragent - samsungsph-m540*/
const Device UA54943 = {S103901, 104, { &P14743,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonJ20i/R7BA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2*/
const Device UA56507 = {S103902, 37, { &P14427,&P17017,&P4970,&P18092 }};	/*Useragent - SPH-M800/1.0 AU-MIC/2.0 (GUI) MMP/2.0*/
const Device UA83438 = {S103903, 17, { &P16182,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonK600i*/
const Device UA59320 = {S103904, 78, { &P16253,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) O2 Xda Atom Life*/
const Device UA61092 = {S103905, 120, { &P12626,&P5208,&P4902,&P18092 }};	/*Useragent - Mozilla/5.0 (SCH-W709/1.0;U;REX/3.0;BREW/3.1.5;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*400;CTC/2.0) POLARIS/6.15.WEB*/
const Device UA61169 = {S103906, 51, { &P12652,&P17017,&P3692,&P18092 }};	/*Useragent - WinWAP-X/3.0 (Neonode N2; Smartphone; WCE; 176x220)*/
const Device UA61253 = {S103907, 143, { &P15382,&P5448,&P22272,&P18092 }};	/*Useragent - SAMSUNG-SGH-i728_CMCC_U1/1.0 Release/03.07 Browser/IE6 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA61299 = {S103908, 91, { &P12676,&P19941,&P3976,&P18092 }};	/*Useragent - KDDI-AM01 UP.Browser/ KJ/ Mozilla/4.0 (REX; U; en-us; Sanyo; SCP-6750/US; NetFront/3.4/AMB)*/
const Device UA61339 = {S103909, 44, { &P12686,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KP230-V10a Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA61356 = {S103910, 71, { &P12693,&P22839,&P4924,&P18092 }};	/*Useragent - ZTE-F260/1.0 ACS-NF/3.2 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61556 = {S103911, 28, { &P12745,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHX800/1.0 NetFront/3.2*/
const Device UA62553 = {S103912, 47, { &P16481,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-VX5200/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA62686 = {S103913, 40, { &P14440,&P17485,&P4203,&P18092 }};	/*Useragent - SonyEricssonJ210i/R2L/TelecaBrowser/4.08*/
const Device UA63541 = {S103914, 90, { &P14526,&P17017,&P4901,&P18092 }};	/*Useragent - SIE-AF51/12 Profile/MIDP-1.0 Configuration/CLDC-1.0 UP.Browser/6.1.0.7.3.102 (GUI) MMP/1.0*/
const Device UA64184 = {S103915, 69, { &P14580,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V1050/83.38.49I MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA67062 = {S103916, 29, { &P11854,&P5890,&P24644,&P18092 }};	/*Useragent - NokiaE71/UCWEB7.4.0.57/28/800*/
const Device UA67104 = {S103917, 95, { &P14778,&P16000,&P20418,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Motorola M1000;817c) Opera 7.50 [en]./M1000./1.0*/
const Device UA67211 = {S103918, 155, { &P14784,&P17979,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android FroydVillain 1.6 (2.2.1); en-gb; HTC Hero Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67240 = {S103919, 118, { &P11755,&P5776,&P4715,&P18092 }};	/*Useragent - Nokia6670/2.0 (6.0522.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.4.0.65/27/351*/
const Device UA67836 = {S103920, 31, { &P14748,&P5890,&P24660,&P18092 }};	/*Useragent - NokiaE72-1/UCWEB7.6.0.75/28/800*/
const Device UA67793 = {S103921, 31, { &P12088,&P5129,&P24660,&P18092 }};	/*Useragent - NokiaX6-00/UCWEB7.6.0.75/50/800*/
const Device UA67829 = {S103922, 30, { &P15373,&P5129,&P24660,&P18092 }};	/*Useragent - Nokia5530/UCWEB7.6.0.75/50/800*/
const Device UA68107 = {S103923, 116, { &P13838,&P5776,&P4759,&P18092 }};	/*Useragent - Nokia6600/1.0 (5.53.0) SymbianOS/7.0s Series60/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.6.1.82/27/351*/
const Device UA68326 = {S103924, 11, { &P14927,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHX430*/
const Device UA68715 = {S103925, 29, { &P14358,&P5890,&P24662,&P18092 }};	/*Useragent - NokiaE63/UCWEB7.8.0.87/28/999*/
const Device UA68844 = {S103926, 29, { &P11854,&P5890,&P24662,&P18092 }};	/*Useragent - NokiaE71/UCWEB7.8.0.87/28/999*/
const Device UA69157 = {S103927, 43, { &P13216,&P17485,&P4777,&P18092 }};	/*Useragent - sonyericssonk750i/UC Browser7.7.1.88/69/444*/
const Device UA69227 = {S103928, 116, { &P13838,&P5776,&P4779,&P18092 }};	/*Useragent - Nokia6600/1.0 (5.53.0) SymbianOS/7.0s Series60/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.8.0.95/27/351*/
const Device UA69467 = {S103929, 35, { &P14782,&P5890,&P4779,&P18092 }};	/*Useragent - NOKIAE61i/UC Browser7.8.0.95/28/355*/
const Device UA69872 = {S103930, 77, { &P16055,&P5448,&P4578,&P18092 }};	/*Useragent - HTC_S310-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 176x220)*/
const Device UA73524 = {S103931, 38, { &P15249,&P17483,&P3866,&P18092 }};	/*Useragent - LG-ME850d MIC/1.1.14 MIDP-2.0/CLDC-1.1*/
const Device UA76139 = {S103932, 83, { &P12695,&P17979,&P3508,&P18092 }};	/*Useragent - MOT-ME860/1.0 Mozilla/5.0 Android/2.2.2 Release/6.30.2011 Browser/AppleWebKit/533.1*/
const Device UA79794 = {S103933, 156, { &P17324,&P5629,&P20776,&P18092 }};	/*Useragent - MQQBrowser/25 Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9001 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA82259 = {S103934, 45, { &P17344,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-C630/1.0 ObigoInternetBrowser/Q03C*/
const Device UA83929 = {S103935, 37, { &P16661,&P17483,&P17470,&P18092 }};	/*Useragent - LG-G1800 MIC/WAP2.0 MIDP-2.0/CLDC-1.0*/
const Device UA84453 = {S103936, 85, { &P12567,&P5776,&P3756,&P18092 }};	/*Useragent - Panasonic-X800/1.0 SymbianOS/7.0 Series60/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA84485 = {S103937, 85, { &P12567,&P5776,&P3756,&P18092 }};	/*Useragent - Panasonic-X701/1.0 SymbianOS/7.0 Series60/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA85499 = {S103938, 32, { &P19997,&P17658,&P18392,&P18092 }};	/*Useragent - DoCoMo/2.0 P700i(c100;TB;W30H15)*/
const Device UA85581 = {S103939, 28, { &P17284,&P17017,&P17470,&P18092 }};	/*Useragent - portalmmm/2.0 Z320i(c100;TB)*/
const Device UA85670 = {S103940, 27, { &P17244,&P17017,&P17470,&P18092 }};	/*Useragent - portalmmm/2.0 S401i(c10;TB)*/
const Device UA85835 = {S103941, 86, { &P15436,&P16000,&P20448,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Symbian OS; SonyEricsson P900; 306) Opera 6.30 [en]*/
const Device UA86472 = {S103942, 82, { &P14806,&P5448,&P17173,&P18092 }};	/*Useragent - SAMSUNG-SGH-I617/1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA86876 = {S103943, 90, { &P14782,&P16000,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; Nokia E61/0633.09.04; 9730) Opera 8.65 [en]*/
const Device UA86990 = {S103944, 82, { &P17166,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) ASUS1210/MR2_V3.5.0A*/
const Device UA87359 = {S103945, 105, { &P16492,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; MDA III/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1)*/
const Device UA87637 = {S103946, 86, { &P17521,&P17662,&P3976,&P18092 }};	/*Useragent - SoftBank/1.0/913SH/SHJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87833 = {S103947, 74, { &P14594,&P6476,&P17660,&P18092 }};	/*Useragent - LG-LX260 EMBIDER-LX260/2.0 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87887 = {S103948, 84, { &P17663,&P17662,&P3976,&P18092 }};	/*Useragent - SoftBank/1.0/821P/PJP10 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88015 = {S103949, 86, { &P17529,&P17662,&P3976,&P18092 }};	/*Useragent - SoftBank/1.0/822SH/SHJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88911 = {S103950, 138, { &P13200,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; ja-jp; HT-03A Build/CDB72) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA90351 = {S103951, 102, { &P16962,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-M130L/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA126430 = {S103952, 121, { &P15364,&P5690,&P18245,&P19831 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) Speedy Spider (http://www.entireweb.com/about/search_tech/speedy_spider/)*/
const Device UA253994 = {S103953, 47, { &P12142,&P17485,&P24663,&P18092 }};	/*Useragent - SonyEricssonU5/UC Browser7.9.0.102/50/354/UCWEB*/
const Device UA361777 = {S103954, 122, { &P15364,&P17285,&P24200,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1 QQBrowser/6.9.11153.201*/
const Device UA359856 = {S103955, 155, { &P16362,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; SPH-D710 Build/GRJ22; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368722 = {S103956, 67, { &P15354,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.4;Zh_cn;ST17i;320*480;)UCWEB7.8.0.95/139/352*/
const Device UA370461 = {S103957, 67, { &P14255,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2.1;Zh_cn;Ideos;240*320;)UCWEB7.8.0.95/139/352*/
const Device UA370516 = {S103958, 73, { &P17713,&P17979,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.3.8) (lgl55c;LGE;LGL55C;2.3.4/GRJ22)*/
const Device UA371265 = {S103959, 138, { &P15341,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; cs-cz; PMP3084BA Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA380771 = {S103960, 123, { &P15364,&P17285,&P22419,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Comodo_Dragon/17.0.3.0 Chrome/17.0.963.38 Safari/535.11*/
const Device UA403016 = {S103961, 140, { &P20557,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.2; en-gb; XOOM 2 ME Build/1.6.0_229-MZ606) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1*/
const Device UA444188 = {S103962, 30, { &P14188,&P5129,&P24644,&P18092 }};	/*Useragent - Nokia5233/UCWEB7.4.0.57/50/800*/
const Device UA456232 = {S103963, 43, { &P15767,&P18117,&P24449,&P18092 }};	/*Useragent - Dolphin HD 4.1 (iPad; iPhone OS 5.1; en_US)*/
const Device UA622003 = {S103964, 45, { &P15767,&P18117,&P24449,&P18092 }};	/*Useragent - Dolphin HD 4.1 (iPad; iPhone OS 5.1.1; en_US)*/
const Device UA456404 = {S103965, 143, { &P18295,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; sv-se; HTC One X Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA525271 = {S103966, 35, { &P14753,&P5890,&P4759,&P18092 }};	/*Useragent - Nokia N96/UC Browser7.6.1.82/28/351*/
const Device UA534313 = {S103967, 35, { &P14984,&P5129,&P4759,&P18092 }};	/*Useragent - Nokia5250/UC Browser7.6.1.82/50/352*/
const Device UA569754 = {S103968, 75, { &P17595,&P17297,&P21573,&P18092 }};	/*Useragent - Mac OS X/10.7.1 (11B2118); ExchangeWebServices/2.0 (123); Mail/5.0 (1244.2)*/
const Device UA876086 = {S103969, 73, { &P17595,&P17297,&P21573,&P18092 }};	/*Useragent - Mac OS X/10.7.1 (11B26); ExchangeWebServices/2.0 (123); Mail/5.0 (1244.2)*/
const Device UA599634 = {S103970, 48, { &P15364,&P18575,&P22244,&P18092 }};	/*Useragent - Flipboard/1.8.1 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA616586 = {S103971, 161, { &P15541,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; Xperia Play Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2027390 = {S103972, 157, { &P15541,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xperia Play Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA617039 = {S103973, 105, { &P17346,&P18117,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]*/
const Device UA617606 = {S103974, 103, { &P17346,&P18117,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 5_1 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]*/
const Device UA637453 = {S103975, 85, { &P18141,&P22219,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 1.52) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2*/
const Device UA832664 = {S103976, 143, { &P13991,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; ADR6400L Build/GRJ22)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA858044 = {S103977, 68, { &P15364,&P5690,&P22570,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 5.1; zh-cn) Opera 8.65*/
const Device UA898057 = {S103978, 97, { &P15364,&P16110,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; chromeframe/19.0.1084.52)*/
const Device UA899300 = {S103979, 90, { &P15364,&P16110,&P21908,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; chromeframe/19.0.1084.52)*/
const Device UA1957385 = {S103980, 46, { &P12280,&P17084,&P17470,&P18092 }};	/*Useragent - Facebook 4110.0 (iPhone; iPhone OS 4.1; en_US)*/
const Device UA924431 = {S103981, 27, { &P15364,&P17350,&P18405,&P18092 }};	/*Useragent - Opera/9.26 (Windows; U; pl)*/
const Device UA925762 = {S103982, 124, { &P16968,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; FujitsuToshibaMobileCommun; IS12T; KDDI)*/
const Device UA925819 = {S103983, 98, { &P14697,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Schubert)*/
const Device UA1858432 = {S103984, 83, { &P18182,&P5448,&P18373,&P18092 }};	/*Useragent - Opera/9.80 (Windows Mobile; WCE; Opera Mobi/7457; U; en) Presto/2.5.28 Version/10.1*/
const Device UA1874873 = {S103985, 87, { &P17595,&P23099,&P17470,&P18092 }};	/*Useragent - RealPlayer%20Downloader/1569 CFNetwork/454.11.12 Darwin/10.7.0 (i386) (MacBookPro7%2C1)*/
const Device UA1884166 = {S103986, 40, { &P12375,&P5890,&P4777,&P18092 }};	/*Useragent - NOKIANokia N81/UC Browser7.7.1.88/28/444*/
const Device UA1925092 = {S103987, 91, { &P12181,&P6173,&P4779,&P18092 }};	/*Useragent - BlackBerry9530/4.2.0.128 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/161/352*/
const Device UA1932864 = {S103988, 32, { &P12193,&P5129,&P24666,&P18092 }};	/*Useragent - NokiaC5-00/UCWEB8.1.0.104/28/800*/
const Device UA1970328 = {S103989, 32, { &P12193,&P5129,&P24667,&P18092 }};	/*Useragent - NokiaC5-00/UCWEB8.2.0.116/28/800*/
const Device UA1950251 = {S103990, 196, { &P17262,&P5629,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Kindle Fire Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.8.4/111,1.8.4.111,2012-07-09 19:03, us*/
const Device UA1962110 = {S103991, 164, { &P12706,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; LG-P690 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA1972463 = {S103992, 53, { &P11929,&P5890,&P20777,&P18092 }};	/*Useragent - MQQBrowser/2.6 (Nokia5230;SymbianOS/9.1 Series60/3.0)*/
const Device UA1975201 = {S103993, 167, { &P18295,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; HTC One X Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2018330 = {S103994, 156, { &P21410,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1-1.0.0; es-es; bq Verne Plus Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2019871 = {S103995, 137, { &P20288,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; HTC_P515E Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2020692 = {S103996, 150, { &P14186,&P5096,&P20775,&P18092 }};	/*Useragent - MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021327 = {S103997, 139, { &P21862,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3; en-us; MAXQ X7 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2021736 = {S103998, 153, { &P13077,&P5629,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HW-HUAWEI_C8650/C8650V100R001C92B825; 320*480; CTC/2.0) AppleWebKit/533.1 Mobile Safari/533.1*/
const Device UA2021820 = {S103999, 148, { &P15897,&P5629,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT531 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022980 = {S104000, 79, { &P14868,&P5062,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android1.6;Zh_cn;Android Dev Phone 1;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA2025170 = {S104001, 157, { &P16436,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-I8150 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2025251 = {S104002, 101, { &P13157,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.4;Zh_cn;HTC Sensation XE with Beats Audio Z715e;540*960;)UCWEB7.8.0.95/139/352*/
const Device UA2025617 = {S104003, 81, { &P14225,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Mail; 1.4.2) (ace; HTC; Desire HD; 2.3.3/GRI40)*/
const Device UA2025923 = {S104004, 155, { &P14186,&P5629,&P21760,&P18092 }};	/*Useragent - iBrowser/2.8 Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026212 = {S104005, 153, { &P13078,&P5096,&P20784,&P18092 }};	/*Useragent - MQQBrowser/3.3/Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ZTE-U V880 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026393 = {S104006, 170, { &P21036,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; Spice Mi-425 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2026609 = {S104007, 173, { &P14183,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; SAMSUNG-SGH-I997 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2026886 = {S104008, 138, { &P17835,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; NookTablet Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2032184 = {S104009, 168, { &P18223,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; LT22i Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2032663 = {S104010, 168, { &P17968,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; huaqin73_gb Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2032771 = {S104011, 168, { &P18303,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; MT27i Build/6.0.B.1.564) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2033465 = {S104012, 167, { &P14574,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Nexus S Build/IMM76L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2162262 = {S104013, 136, { &P17346,&P18117,&P20784,&P18092 }};	/*Useragent - MQQBrowser/33 Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3*/
const Device UA2241157 = {S104014, 162, { &P18385,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Nexus 7 Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (Nexus 7, 34, 1.6.0)*/
const Device UA2241896 = {S104015, 162, { &P18385,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Nexus 7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (Nexus 7, 34, 1.6.0)*/
const Device UA2241453 = {S104016, 141, { &P18385,&P17979,&P3508,&P18092 }};	/*Useragent - Rdio.Net; Android/2.2.8; Android/4.1.1; DM=asus; DN=Nexus 7; DC=google; DF=google/nakasi/grouper:4.1.1/JRO03D/X:user/release-keys; DH=grouper*/
const Device UA2241460 = {S104017, 142, { &P18385,&P17979,&P3508,&P18092 }};	/*Useragent - Rdio.Net; Android/2.2.10; Android/4.1.2; DM=asus; DN=Nexus 7; DC=google; DF=google/nakasi/grouper:4.1.2/JZO54K/X:user/release-keys; DH=grouper*/
const Device UA2241984 = {S104018, 180, { &P18385,&P18111,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1; en-us; Nexus 7 Build/JRN84D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, us*/
const Device UA2313198 = {S104019, 79, { &P15364,&P18145,&P3403,&P18092 }};	/*Useragent - Opera/9.80 (X11; FreeBSD 9.0-RELEASE i386; U; en) Presto/2.10.229 Version/11.61*/
const Device UA2317415 = {S104020, 47, { &P11854,&P5890,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8 (E71;SymbianOS/9.1 Series60/3.0)*/
const Device UA2432873 = {S104021, 90, { &P14330,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-gx300) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2462422 = {S104022, 139, { &P19777,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-I9105 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2462426 = {S104023, 138, { &P19777,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9105 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2505294 = {S104024, 91, { &P12165,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; gt-s5230w) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2530075 = {S104025, 99, { &P14851,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s5753e) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2555487 = {S104026, 89, { &P22957,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-a350) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2586643 = {S104027, 165, { &P21611,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2607491 = {S104028, 64, { &P18073,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;T9295;7.10;8862)*/
const Device UA2607509 = {S104029, 161, { &P21736,&P5629,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LAVA iris 351 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 UCBrowser/2.2.2.259*/
const Device UA2607511 = {S104030, 170, { &P21736,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; LAVA iris 351 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2607636 = {S104031, 71, { &P14908,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2.1;Zh_cn;GT-P1010;600*1024;)UCWEB7.8.0.95/139/355*/
const Device UA2611205 = {S104032, 156, { &P13452,&P5629,&P20775,&P18092 }};	/*Useragent - MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2607801 = {S104033, 149, { &P13452,&P5096,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7/Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2608234 = {S104034, 157, { &P13452,&P5629,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2609508 = {S104035, 151, { &P13452,&P5096,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2611229 = {S104036, 149, { &P13452,&P5096,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2610284 = {S104037, 137, { &P22038,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7503 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2620945 = {S104038, 29, { &P22041,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (GT-P6810 IMM76D)*/
const Device UA2611086 = {S104039, 91, { &P14908,&P6437,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.3.9.10087); (Linux; U; Android 3.2; GT-P6210 Build/HONEYCOMB_MR2);*/
const Device UA2611410 = {S104040, 80, { &P17168,&P17979,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.4.5) (GT-P7310;samsung;GT-P7310;3.2/HTJ85B)*/
const Device UA2611636 = {S104041, 139, { &P15876,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5100 Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617535 = {S104042, 139, { &P15876,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P5100 Build/IMM76D)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2613525 = {S104043, 145, { &P15985,&P6437,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2617092 = {S104044, 146, { &P15985,&P6437,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2613633 = {S104045, 157, { &P13452,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2622104 = {S104046, 157, { &P13452,&P5629,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2614716 = {S104047, 138, { &P17323,&P6437,&P20791,&P18092 }};	/*Useragent - MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2622524 = {S104048, 150, { &P17323,&P6437,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2615579 = {S104049, 80, { &P18478,&P17979,&P18334,&P18092 }};	/*Useragent - YahooMobileMail/1.0 (Android Mail; 1.4.5) (GT-P6201;samsung;GT-P6201;3.2/HTJ85B)*/
const Device UA2616338 = {S104050, 145, { &P17337,&P6437,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 3.2; hu-hu; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2616815 = {S104051, 164, { &P12889,&P6437,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7300 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/405*/
const Device UA2617277 = {S104052, 165, { &P12889,&P6437,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7300B Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2617206 = {S104053, 97, { &P14908,&P17979,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P6210; GT-P6210; smdk4210; en_US)*/
const Device UA2617235 = {S104054, 166, { &P13452,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P1000 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2619087 = {S104055, 166, { &P13452,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P1000 Build/IMM76L) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2617374 = {S104056, 36, { &P17168,&P17979,&P16179,&P18092 }};	/*Useragent - Android/3.2; samsung GT-P7310; 2.3.1*/
const Device UA2617442 = {S104057, 147, { &P15985,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617511 = {S104058, 98, { &P13452,&P17979,&P22071,&P18092 }};	/*Useragent - Instagram 3.1.0 Android (10/2.3.3; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; gt-p1000; en_GB)*/
const Device UA2622445 = {S104059, 95, { &P13452,&P17979,&P22071,&P18092 }};	/*Useragent - Instagram 3.1.0 Android (8/2.2; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; GT-P1000; en_GB)*/
const Device UA2617562 = {S104060, 166, { &P22037,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P3113 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2617575 = {S104061, 57, { &P15985,&P17979,&P22697,&P18092 }};	/*Useragent - TwitterAndroid/1.0.5 (109) GT-P6200/13 (samsung;GT-P6200)*/
const Device UA2617934 = {S104062, 77, { &P13452,&P17979,&P22699,&P18092 }};	/*Useragent - TwitterAndroid/2.0.1 (122) GT-P1000N/8 (samsung;GT-P1000N;samsung;GT-P1000N;)*/
const Device UA2618849 = {S104063, 90, { &P12879,&P17979,&P22071,&P18092 }};	/*Useragent - Instagram 3.1.0 Android (12/3.1; 160dpi; 1280x800; samsung; GT-P7510; GT-P7510; p3; en_US)*/
const Device UA2618955 = {S104064, 116, { &P22037,&P17979,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.0001.8239(Android;Android (GT-P3113-espressowifi-user-4.0.4);en-US;kml:2.2;client:Free;type:default)*/
const Device UA2619049 = {S104065, 75, { &P15985,&P17979,&P22699,&P18092 }};	/*Useragent - TwitterAndroid/2.0.1 (122) GT-P6200/13 (samsung;GT-P6200;samsung;GT-P6200;)*/
const Device UA2619374 = {S104066, 161, { &P17168,&P6546,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7310 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7310, 36, 1.7.1)*/
const Device UA2619797 = {S104067, 147, { &P15919,&P18066,&P21729,&P18092 }};	/*Useragent - OneBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-za; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2619971 = {S104068, 85, { &P15876,&P17979,&P22699,&P18092 }};	/*Useragent - TwitterAndroid/2.0.1 (122) GT-P5100/15 (samsung;espresso10rf;samsung;espresso10rfxx;)*/
const Device UA2620053 = {S104069, 31, { &P14908,&P17979,&P20428,&P18092 }};	/*Useragent - Android OS/3.2 samsung_GT-P6210*/
const Device UA2620430 = {S104070, 145, { &P12889,&P6437,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2628555 = {S104071, 152, { &P12889,&P6437,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2620932 = {S104072, 90, { &P12879,&P17979,&P22070,&P18092 }};	/*Useragent - Instagram 3.0.5 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7510; GT-P7510; p3; en_US)*/
const Device UA2621026 = {S104073, 147, { &P17323,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2621473 = {S104074, 96, { &P17323,&P17979,&P22068,&P18092 }};	/*Useragent - Instagram 1.0.5 Android (13/3.2; 160dpi; 800x1232; samsung; GT-P6800; GT-P6800; smdkc210; en_GB)*/
const Device UA2621587 = {S104075, 152, { &P12889,&P6437,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2633167 = {S104076, 145, { &P12889,&P6437,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2624178 = {S104077, 162, { &P17168,&P6437,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7310 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7310, 38, 1.8.1)*/
const Device UA2623100 = {S104078, 207, { &P12879,&P18149,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0100, si*/
const Device UA2623777 = {S104079, 149, { &P12879,&P5629,&P18062,&P18092 }};	/*Useragent - Rdio.Net; Android/2.4.2; Android/4.0.4; DM=samsung; DN=GT-P7510; DC=samsung; DF=samsung/GT-P7510/GT-P7510:4.0.4/IMM76D/UELPL:user/release-keys; DH=p3*/
const Device UA2624102 = {S104080, 164, { &P17168,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; GT-P7310 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7310, 38, 1.8.1)*/
const Device UA2634266 = {S104081, 164, { &P17168,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7310 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7310, 40, 1.8.5)*/
const Device UA2624360 = {S104082, 145, { &P15919,&P18111,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2625741 = {S104083, 101, { &P15932,&P17979,&P22073,&P18092 }};	/*Useragent - Instagram 3.3.2 Android (15/4.0.3; 160dpi; 600x976; samsung; GT-P3110; espressowifi; espresso; en_US)*/
const Device UA2625759 = {S104084, 168, { &P15919,&P18111,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]*/
const Device UA2631105 = {S104085, 169, { &P15919,&P18111,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P3100B Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]*/
const Device UA2626004 = {S104086, 166, { &P15876,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P5100 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2626352 = {S104087, 166, { &P22037,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P3113 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2626991 = {S104088, 150, { &P15985,&P6437,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 UCBrowser/2.2.2.259*/
const Device UA2630077 = {S104089, 151, { &P15985,&P6437,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200L Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 UCBrowser/2.2.2.259*/
const Device UA2627023 = {S104090, 147, { &P22043,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2627034 = {S104091, 151, { &P17323,&P6437,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 MobileSafari/534.13*/
const Device UA2627924 = {S104092, 152, { &P17323,&P6437,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2627064 = {S104093, 208, { &P14908,&P22212,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P6210 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +0000, gb*/
const Device UA2627745 = {S104094, 151, { &P15876,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5100 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2628468 = {S104095, 151, { &P15876,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5100 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2627796 = {S104096, 172, { &P12889,&P6437,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7300 Build/HTJ85B; 320*493) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444*/
const Device UA2628044 = {S104097, 166, { &P22043,&P18111,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P5113 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2628899 = {S104098, 164, { &P15876,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P5100, 39, 1.8.2)*/
const Device UA2636184 = {S104099, 156, { &P15932,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-P3110 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2635124 = {S104100, 162, { &P17323,&P6437,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P6800, 39, 1.8.2)*/
const Device UA2631220 = {S104101, 81, { &P22043,&P17979,&P4459,&P18092 }};	/*Useragent - Evernote Android/AndroidMarket_45x_243.14540 (en_CA); Android/4.1.1; GT-P5113/16;*/
const Device UA2631778 = {S104102, 152, { &P12879,&P18066,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2631803 = {S104103, 149, { &P22052,&P6437,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-P7501 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.3.3000*/
const Device UA2632204 = {S104104, 152, { &P15932,&P18111,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2632978 = {S104105, 166, { &P17323,&P18066,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6800 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2633164 = {S104106, 170, { &P17323,&P18066,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-P6800 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2633156 = {S104107, 151, { &P14905,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2636957 = {S104108, 156, { &P12879,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P7510 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2634235 = {S104109, 156, { &P17337,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P7500 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2636802 = {S104110, 156, { &P17168,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P7310 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2634724 = {S104111, 59, { &P22043,&P17979,&P22280,&P18092 }};	/*Useragent - theScore/3.2.1 Android-OS/4.1.1 (Android; Tablet, GT-P5113)*/
const Device UA2634806 = {S104112, 156, { &P15882,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-P5110 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2635665 = {S104113, 156, { &P15882,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-P5110 Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2634859 = {S104114, 57, { &P13452,&P17979,&P22280,&P18092 }};	/*Useragent - theScore/3.2.0 Android-OS/2.2 (Android; Phone, GT-P1000R)*/
const Device UA2634931 = {S104115, 151, { &P17337,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2662080 = {S104116, 100, { &P14831,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s7233e) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2748423 = {S104117, 90, { &P14422,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-p520) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2801270 = {S104118, 137, { &P22958,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-US780 Build/JZO54K) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2817434 = {S104119, 40, { &P16463,&P17484,&P24632,&P18092 }};	/*Useragent - SAMSUNG-GT-I8350/UCWEB2.3.0.182/47/32079*/
const Device UA2847785 = {S104120, 97, { &P12142,&P17485,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; zh-CN; sonyericssonu5i) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2847896 = {S104121, 98, { &P16366,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s8600) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2853553 = {S104122, 99, { &P12540,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-sgh-u900) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2868123 = {S104123, 99, { &P14864,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8500) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2891332 = {S104124, 90, { &P15364,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-a230) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2987835 = {S104125, 90, { &P15364,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-c300) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA3100501 = {S104126, 90, { &P15364,&P17483,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; Pt-BR; lg-c300) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2944204 = {S104127, 156, { &P11793,&P5768,&P24678,&P18092 }};	/*Useragent - Nokia3500c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3500c) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2953142 = {S104128, 92, { &P14413,&P17486,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mot-ex115) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2977056 = {S104129, 133, { &P24080,&P18110,&P19800,&P18092 }};	/*Useragent - Mozilla/5.0 (DTV) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.5 AQUOSBrowser/1.0 (AS00DTV;V;0001;0001) AQUOS-AS/2.0 LC-60LE840X*/
const Device UA2977060 = {S104130, 86, { &P24083,&P18110,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv6l; U; en) Presto/2.8.115 Version/11.10 AQUOS-AS/1.0 LC-70LE735X*/
const Device UA2977080 = {S104131, 163, { &P24048,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly s Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2977104 = {S104132, 156, { &P24061,&P18598,&P24178,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-S7270 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.63 Mobile Safari/537.36 OPR/15.0.1162.60140*/
const Device UA2977106 = {S104133, 138, { &P24064,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-S7275 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3003156 = {S104134, 90, { &P17332,&P17483,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-c310) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3039584 = {S104135, 202, { &P24249,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 55LM6200-UE; 04.02.85; 0xX;); LG NetCast.TV-2012 0*/
const Device UA3039602 = {S104136, 200, { &P24292,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 55LM7600-TA; 04.10.08; 0xX;); LG NetCast.TV-2012*/
const Device UA3039605 = {S104137, 198, { &P15364,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+SCREEN+TUNER; LGE; GLOBAL-PLAT4; 03.17.00; 0xX;); LG NetCast.TV-2012*/
const Device UA3039633 = {S104138, 203, { &P15364,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; DLOBAL-PLAT4; 04.34.05; 0xX;); LG NetCast.TV-2012 0*/
const Device UA3048429 = {S104139, 99, { &P16366,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8600) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA3058566 = {S104140, 98, { &P12209,&P17483,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lg-ks360 teleca) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3058760 = {S104141, 99, { &P12260,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5350) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3068892 = {S104142, 122, { &P15767,&P24384,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11D4 Safari/8536.25*/
const Device UA3068895 = {S104143, 123, { &P15767,&P24384,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11D8a Safari/8536.25*/
const Device UA3068896 = {S104144, 125, { &P15767,&P24384,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.49.1 (KHTML, like Gecko) Version/6.0 Mobile/11D17 Safari/8536.25*/
const Device UA3068897 = {S104145, 123, { &P15767,&P24384,&P18431,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Version/6.0 Mobile/11D10 Safari/8536.25*/
const Device UA3068905 = {S104146, 127, { &P12280,&P24384,&P24402,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU OS 7_1 like Mac OS X) AppleWebKit/537.80 (KHTML, like Gecko) Version/7.1 Mobile/11A4388 Safari/8536.25*/
const Device UA3084064 = {S104147, 156, { &P24061,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-S7270 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084112 = {S104148, 164, { &P22998,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; TECNO_N7 Build/TECNOTECNO_N7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3118320 = {S104149, 96, { &P12303,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; vi; samsung-gt-b3410w) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA128 = {S104150, 97, { &P17475,&P5768,&P4715,&P18092 }};	/*Useragent - Nokia6300/2.0 (05.00) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6300/UC Browser7.4.0.65/69/352*/
const Device UA906 = {S104151, 77, { &P20111,&P5448,&P3430,&P18092 }};	/*Useragent - SAMSUNG-SPH-M7350/(null)ID4 (compatible; MSIE 6.0; Windows CE; PPC) Opera 9.5*/
const Device UA65310 = {S104152, 115, { &P12056,&P5204,&P4715,&P18092 }};	/*Useragent - Nokia7210Supernova/2.0 (06.51) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7210supernova/UC Browser7.4.0.65/70/351*/
const Device UA2456 = {S104153, 127, { &P11972,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-F338/1.0 SHP/VPP/R5 NetFront/3.4 Qtv5.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1/*MzU5MDQyMDEwMzkwNjcx*/
const Device UA2513 = {S104154, 46, { &P17006,&P22835,&P17470,&P18092 }};	/*Useragent - SPICE-S600/WAP Browser, SPICE-S600/WAP Browser*/
const Device UA61747 = {S104155, 22, { &P17006,&P22835,&P17470,&P18092 }};	/*Useragent - SPICE-S600/WAP Browser*/
const Device UA22662 = {S104156, 97, { &P12059,&P5204,&P4742,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2690/UC Browser7.5.1.77/70/351*/
const Device UA4546 = {S104157, 140, { &P17058,&P5448,&P17183,&P18092 }};	/*Useragent - SAMSUNG-SGH-I907/UCJD1 Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA5824 = {S104158, 160, { &P12144,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android SenseHero 2.1; en-us; A6277 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 Sprint APA6277KT*/
const Device UA5951 = {S104159, 121, { &P14222,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) 320x240; VZW; UTStar-XV6175.1; Windows Mobile 6.5 Standard;*/
const Device UA9159 = {S104160, 122, { &P13392,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-C3510/C3510JAIL1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA8844 = {S104161, 90, { &P17984,&P17017,&P3625,&P18092 }};	/*Useragent - Haier-1200P/1.0 iPanel/2.0 WAP2.0 (compatible; UP.Browser/6.2.2.4; UPG1; UP/4.0; Embedded)*/
const Device UA9486 = {S104162, 122, { &P12301,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-S3100/S3100XXIH4 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA9965 = {S104163, 96, { &P17799,&P17811,&P3504,&P18092 }};	/*Useragent - LENOVO-P709/S100 NXP5210/DV7 Release/4.25.2007 Obigo/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA12443 = {S104164, 93, { &P13374,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_TOUCH_DIAMOND2/2.0; Windows Phone 6.5)*/
const Device UA13658 = {S104165, 58, { &P13507,&P17017,&P17470,&P18092 }};	/*Useragent - TCL-e300/w05.44.08/WAP-2.0/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA54166 = {S104166, 108, { &P14137,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-Z720M-Vodafone/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA14421 = {S104167, 68, { &P11803,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V195/0A.63.12R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA14534 = {S104168, 62, { &P11784,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW950i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA16223 = {S104169, 55, { &P13999,&P5448,&P4536,&P18092 }};	/*Useragent - HTC_Touch_HD2_T8585 Opera/10.00 (Windows NT 5.1; U; en)*/
const Device UA17447 = {S104170, 68, { &P16882,&P17017,&P17470,&P18092 }};	/*Useragent - VertuAscentTi-C/2.0 (501.02) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA35729 = {S104171, 68, { &P16882,&P17017,&P17470,&P18092 }};	/*Useragent - VertuAscentTi-C/2.0 (501.01) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA18731 = {S104172, 32, { &P11923,&P19954,&P3816,&P18092 }};	/*Useragent - SEC-SGHP207/a/, TSS/2.5, Rev1.1h*/
const Device UA35779 = {S104173, 29, { &P11923,&P19954,&P3816,&P18092 }};	/*Useragent - SEC-SGHP207, TSS/2.5, Rev1.1e*/
const Device UA19951 = {S104174, 236, { &P15011,&P5062,&P3441,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.6; zh-cn; dopodA6388_TD/1.0 Android/Dount release/2.26.2010 Browser/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Build/DRD20) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA20139 = {S104175, 84, { &P17510,&P17662,&P3976,&P18092 }};	/*Useragent - SoftBank/1.0/820P/PJP10 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20275 = {S104176, 106, { &P14246,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) HPiPAQ610/1.0 (PPC; 240x320)*/
const Device UA54618 = {S104177, 65, { &P14246,&P5448,&P4808,&P18092 }};	/*Useragent - HPiPAQ610/1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC)*/
const Device UA20326 = {S104178, 105, { &P15069,&P17017,&P3976,&P18092 }};	/*Useragent - SHARP/1.1/SH1810C/SHC001 Mozilla/5.00 (compatible) Browser/NF/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20997 = {S104179, 145, { &P16054,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 240x320; HTC_P3600/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21174 = {S104180, 48, { &P11976,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-E218/CMCC_S101 LMP/LML Release/2009.05.20*/
const Device UA21292 = {S104181, 74, { &P15185,&P17484,&P4901,&P18092 }};	/*Useragent - SAMSUNG-SGH-X608/1.0*MzUyNjIzMDAwMDA1MDEw UP.Browser/6.1.0.6 (GUI) MMP/1.0*/
const Device UA21301 = {S104182, 164, { &P15188,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P929 Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA59231 = {S104183, 164, { &P15188,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; LG-P929 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA21438 = {S104184, 74, { &P15282,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) HTC-P5310 BM*/
const Device UA21499 = {S104185, 96, { &P15286,&P17017,&P17470,&P18092 }};	/*Useragent - Fly-E300/Profile/MIDP.2.0Configuration/CLDC.1.0, Fly-E300/Profile/MIDP.2.0Configuration/CLDC.1.0*/
const Device UA21659 = {S104186, 92, { &P15317,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-Mandarina-Duck/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA21667 = {S104187, 40, { &P15320,&P17017,&P17470,&P18092 }};	/*Useragent - YuLong-Coolpad2938/1.0 iPanel/2.0 WAP2.0*/
const Device UA21738 = {S104188, 53, { &P13103,&P17486,&P3625,&P18092 }};	/*Useragent - Motorola-K1m UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0*/
const Device UA21891 = {S104189, 47, { &P15352,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-RD9100/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA22276 = {S104190, 37, { &P15418,&P17483,&P17470,&P18092 }};	/*Useragent - LG-C3300 MIC/WAP2.0 MIDP-2.0/CLDC-1.0*/
const Device UA22376 = {S104191, 85, { &P15445,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-C707/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA22420 = {S104192, 70, { &P15462,&P5448,&P22270,&P18092 }};	/*Useragent - Xda_Venn-Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA22471 = {S104193, 87, { &P14026,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-CX75/13 UP.Browser/7.0.2.2.d.7(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53882 = {S104194, 87, { &P14026,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-CX75/25 UP.Browser/7.0.2.2.d.7(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53975 = {S104195, 87, { &P14026,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-CX75/10 UP.Browser/7.0.2.2.d.5(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22687 = {S104196, 37, { &P15509,&P17483,&P4203,&P18092 }};	/*Useragent - LGE-LG385 AU-OBIGO/Q04C1-1.27 MMP/2.0*/
const Device UA23006 = {S104197, 71, { &P15548,&P22839,&P4924,&P18092 }};	/*Useragent - ZTE-F153/1.0 ACS-NF/3.2 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23100 = {S104198, 127, { &P15563,&P6476,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-L288/1.0 SHP/VPP1.0 Release/12.15.2006 Browser/R5.NetFront3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1 SMM-MMS/1.2.0*/
const Device UA23195 = {S104199, 101, { &P15574,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E480/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA24573 = {S104200, 46, { &P12445,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-LG380/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA31977 = {S104201, 42, { &P12445,&P17483,&P3625,&P18092 }};	/*Useragent - LG380/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA24616 = {S104202, 86, { &P15721,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-G933/MTK Release/01.01.2010 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1*/
const Device UA28380 = {S104203, 90, { &P12229,&P17662,&P3716,&P18092 }};	/*Useragent - SoftBank/1.0/705P/PJP10 Browser/Teleca-Browser/3.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA28438 = {S104204, 86, { &P12244,&P17662,&P3601,&P18092 }};	/*Useragent - SoftBank/1.0/708SC/SCJ001 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA28484 = {S104205, 74, { &P12251,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) ASUSP550/1.0*/
const Device UA30103 = {S104206, 115, { &P12286,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) ASUS-P835/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA30267 = {S104207, 121, { &P17771,&P5448,&P22270,&P18092 }};	/*Useragent - dopod T3238_CMCC/5.2.20257/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA30804 = {S104208, 121, { &P12344,&P5890,&P3439,&P18092 }};	/*Useragent - SAMSUNG-SGH-L878E/1.0 SymbianOS/9.3 Series60/3.2 Release/6.25.2008 Browser/OSS3.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA32009 = {S104209, 42, { &P11858,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonZ710i/R1JC SEMC/NewsReader/1.0*/
const Device UA32781 = {S104210, 111, { &P12767,&P19954,&P3625,&P18092 }};	/*Useragent - SEC-SGHC500/1.0 Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA32970 = {S104211, 39, { &P12812,&P17483,&P3625,&P18092 }};	/*Useragent - LG-MG220 UP.Browser/6.2.3 (GUI) MMP/1.0*/
const Device UA33000 = {S104212, 85, { &P12823,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-Crystal/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA33138 = {S104213, 84, { &P17795,&P17662,&P3976,&P18092 }};	/*Useragent - SoftBank/1.0/001P/PJP10 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA33334 = {S104214, 39, { &P12900,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KE590 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA34480 = {S104215, 42, { &P11877,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonZ610i/R1JG SEMC/NewsReader/1.0*/
const Device UA35435 = {S104216, 76, { &P16860,&P5448,&P3430,&P18092 }};	/*Useragent - SAMSUNG-SCH-M715/(null)ID4 (compatible; MSIE 6.0; Windows CE; PPC) Opera 9.5*/
const Device UA36092 = {S104217, 172, { &P15809,&P5448,&P22271,&P18092 }};	/*Useragent - HTC_P3650 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) O2MTouchC Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6;HTC_P5500; tmn PortalBliss 1.0)*/
const Device UA36480 = {S104218, 116, { &P12080,&P17484,&P3636,&P18092 }};	/*Useragent - SAMSUNG-GT-S3653/S3653JVIH2 SHP/VPP/R5 Jasmine/0.8 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA36907 = {S104219, 164, { &P11747,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Special; en-gb; Desire_A8181 Build/Jaguaralani Gamer's Edition) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA75299 = {S104220, 150, { &P11747,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android PDAMobiz 2.2 II; en-us; Desire_A8181 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA37290 = {S104221, 41, { &P15742,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-R610 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA38353 = {S104222, 86, { &P17524,&P17662,&P3601,&P18092 }};	/*Useragent - SoftBank/1.0/911SH/SHJ002 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA39351 = {S104223, 175, { &P13505,&P5096,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; MB525 Build/3.4.2-117; 480*854) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.1.100/139/800*/
const Device UA39407 = {S104224, 175, { &P13505,&P5096,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; MB525 Build/3.4.2-177; 480*854) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.1.100/139/800*/
const Device UA41179 = {S104225, 90, { &P13894,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-B6520/1.0 Mozilla/4.0 Release/8.11.2010 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA43534 = {S104226, 140, { &P12096,&P5448,&P22270,&P18092 }};	/*Useragent - PPC; 240x320; HTC_TyTN/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Dopod838Pro*/
const Device UA44372 = {S104227, 70, { &P17921,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) DopodM700*/
const Device UA44373 = {S104228, 71, { &P17921,&P5448,&P4578,&P18092 }};	/*Useragent - DopodM700/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)*/
const Device UA44999 = {S104229, 16, { &P14182,&P17484,&P17606,&P18092 }};	/*Useragent - samsung-gt-s5333*/
const Device UA47993 = {S104230, 134, { &P12595,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T759 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA48888 = {S104231, 84, { &P17788,&P17662,&P3601,&P18092 }};	/*Useragent - SoftBank/1.0/911T/TJ002 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49138 = {S104232, 76, { &P13908,&P17017,&P3680,&P18092 }};	/*Useragent - WellcomW3110/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49196 = {S104233, 44, { &P12951,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-216/1.0 ObigoInternetBrowser/Q03C*/
const Device UA49228 = {S104234, 104, { &P12965,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-L811/L811JVHL1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA49276 = {S104235, 53, { &P12913,&P17483,&P17660,&P18092 }};	/*Useragent - LGE/U8210/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49294 = {S104236, 86, { &P12979,&P17662,&P3601,&P18092 }};	/*Useragent - SoftBank/1.0/805SC/SCJ001 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49324 = {S104237, 86, { &P12988,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-C700A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA50483 = {S104238, 118, { &P11669,&P5776,&P4722,&P18092 }};	/*Useragent - Nokia7610/2.0 (7.0642.0) SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0/UC Browser7.0.0.41/27/352*/
const Device UA51634 = {S104239, 161, { &P13950,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia6788/CMCC SymbianOS/9.3 Series60/3.2 Release/11.018.314.1; Mozilla/5.0 Profile/MIDP-2.1 Configuration/CLDC-1.1 AppleWebKit/413 (KHTML,like Gecko) Safari/413*/
const Device UA53350 = {S104240, 102, { &P13969,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) LG-MS20 IEMobile/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA53468 = {S104241, 39, { &P13984,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KF305 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA53493 = {S104242, 118, { &P17919,&P5448,&P22272,&P18092 }};	/*Useragent - dopod P660_CMCC/5.2.1941/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA53553 = {S104243, 63, { &P13844,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonZ555i/R10CA Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53713 = {S104244, 189, { &P13988,&P5448,&P4626,&P18092 }};	/*Useragent - HTCT9188_TD/1.0 WindowsMobile/6.5 CEOS/5.2 release/5.3 Opera/9.7 WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA54167 = {S104245, 85, { &P14138,&P22839,&P3601,&P18092 }};	/*Useragent - ZTE-F188(Smart Sandbox)/1.0 ACS-NF/3.3 QTV2.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54411 = {S104246, 113, { &P14204,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-L400-Vodafone/L400BUGJ2 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA54456 = {S104247, 44, { &P14216,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KU310/v10d Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA55031 = {S104248, 72, { &P14276,&P17017,&P3680,&P18092 }};	/*Useragent - AKAI4411/R2AE SEMC-Browser/4.0.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA55981 = {S104249, 77, { &P12450,&P5448,&P4737,&P18092 }};	/*Useragent - SAMSUNG-SCH-i220a (compatible; MSIE 4.01; Windows CE; PPC)/UC Browser7.2.2.51*/
const Device UA56004 = {S104250, 70, { &P14614,&P5926,&P3625,&P18092 }};	/*Useragent - Nokia6255/1.0 (N190V0700.nep) UP.Browser/6.2.3.2.h.1.104 (GUI) MMP/2.0*/
const Device UA56105 = {S104251, 16, { &P15197,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-B5310*/
const Device UA56109 = {S104252, 59, { &P13230,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-A-A6/00.02 UP.Browser/7.2.7.2.531 (GUI) MMP/2.0 Push/PO*/
const Device UA56765 = {S104253, 21, { &P14719,&P6712,&P4030,&P18092 }};	/*Useragent - Nokia6310/1.0 (05.01)*/
const Device UA59476 = {S104254, 82, { &P17801,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-TD30t/S100 LMP/LML Release/2009.10.15 Profile/MIDP2.1 Configuration/CLDC1.1*/
const Device UA60542 = {S104255, 137, { &P12541,&P17605,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-B7702_TD/1.0 RTK-E/1.0 SHP/VPP-R5 Release/03.18.2010 Browser/NetFront3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1 SMM-MMS/1.2.0*/
const Device UA60602 = {S104256, 80, { &P12553,&P17484,&P4381,&P18092 }};	/*Useragent - SAMSUNG-SGH-L168-CMCC/Polaris 5.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA60719 = {S104257, 61, { &P12567,&P19945,&P17470,&P18092 }};	/*Useragent - Panasonic-MX7/1.0/RP1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA84451 = {S104258, 59, { &P12567,&P19945,&P17470,&P18092 }};	/*Useragent - Panasonic-X400P/RP1 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA84490 = {S104259, 61, { &P12567,&P19945,&P17470,&P18092 }};	/*Useragent - Panasonic-SA7/1.0/RP1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA60940 = {S104260, 81, { &P12598,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; O2 Graphite; Smartphone; 240x320)*/
const Device UA60957 = {S104261, 98, { &P12603,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-A811/1.0 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA60986 = {S104262, 44, { &P12609,&P17486,&P4381,&P18092 }};	/*Useragent - MOT-A1890/GE00.08.02P INFRAWARE/5.01 CTC/1.0*/
const Device UA61070 = {S104263, 74, { &P14390,&P17484,&P3816,&P18092 }};	/*Useragent - SAMSUNG-SGH-T609/T609UVFC8 Profile/MIDP-2.0 Configuration/CLDC-1.1 TSS/2.5*/
const Device UA61109 = {S104264, 87, { &P12631,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-MissSixty/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA61126 = {S104265, 122, { &P14970,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-E3210/E3210DDKB6 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.102 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA61252 = {S104266, 114, { &P12668,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-P220-ORANGE/P220BVGK1 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA61361 = {S104267, 98, { &P12694,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-T336/ Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA61766 = {S104268, 104, { &P14322,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-C5110-ORANGE/BVIF4 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA62416 = {S104269, 135, { &P15364,&P17486,&P3581,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible;MSIE 6.0;Linux MOTOZINE ZN5)MOTOZINE ZN5/R6637_G_81.03.0AR Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony 1.0*/
const Device UA63909 = {S104270, 147, { &P14556,&P17605,&P3502,&P18092 }};	/*Useragent - SAMSUNG-GT-C5530_TD/1.0 RTK-E/1.0 SHP/VPP-R5 Release/03.18.2010 Browser/Dolfin1.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 Nextreaming SMM-MMS/1.2.0*/
const Device UA65022 = {S104271, 188, { &P15364,&P7133,&P4742,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.5.1.77/70/351*/
const Device UA67285 = {S104272, 36, { &P14748,&P5890,&P4722,&P18092 }};	/*Useragent - NokiaE72-1/UC Browser7.0.0.41/28/352*/
const Device UA67790 = {S104273, 36, { &P14033,&P5129,&P4715,&P18092 }};	/*Useragent - NokiaC5-03/UC Browser7.4.0.65/50/355*/
const Device UA67561 = {S104274, 29, { &P13977,&P5890,&P24660,&P18092 }};	/*Useragent - NokiaN73/UCWEB7.6.0.75/28/401*/
const Device UA67636 = {S104275, 37, { &P12088,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaX6-00/SymbianOS/9.1 Series60/3.0*/
const Device UA67990 = {S104276, 106, { &P13308,&P17485,&P4759,&P18092 }};	/*Useragent - SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1 sonyericssonj105i/UC Browser7.6.1.82/70/351*/
const Device UA68570 = {S104277, 15, { &P14937,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHC230/1.0*/
const Device UA69026 = {S104278, 31, { &P14748,&P5890,&P24662,&P18092 }};	/*Useragent - NokiaE72-1/UCWEB7.8.0.87/28/800*/
const Device UA68731 = {S104279, 45, { &P11932,&P5890,&P4777,&P18092 }};	/*Useragent - NOKIA6210 Navigator/UC Browser7.7.1.88/28/352*/
const Device UA68779 = {S104280, 30, { &P15373,&P5129,&P24662,&P18092 }};	/*Useragent - Nokia5530/UCWEB7.8.0.87/50/999*/
const Device UA69204 = {S104281, 38, { &P13979,&P5890,&P4779,&P18092 }};	/*Useragent - Nokia6220c-1/UC Browser7.8.0.95/28/352*/
const Device UA69246 = {S104282, 43, { &P20126,&P17485,&P4779,&P18092 }};	/*Useragent - sonyericssonk800i/UC Browser7.8.0.95/70/352*/
const Device UA69560 = {S104283, 61, { &P15226,&P17483,&P17602,&P18092 }};	/*Useragent - LG-MU500 Obigo/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA69590 = {S104284, 27, { &P15026,&P17483,&P17602,&P18092 }};	/*Useragent - LG-CP150/V10n Teleca/WAP2.0*/
const Device UA69770 = {S104285, 113, { &P14682,&P5843,&P4779,&P18092 }};	/*Useragent - Nokia6303iclassic/5.0 (10.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303iclassic/UC Browser7.8.0.95/70/351*/
const Device UA71388 = {S104286, 112, { &P16946,&P5448,&P22270,&P18092 }};	/*Useragent - Palm Treo Pro/ ver 1.01/ Windows CE;IE Mobile 7.11 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA76022 = {S104287, 188, { &P13505,&P5477,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; MB525 Build/JRDNEM_U3_2.21.0; 480*854) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/800*/
const Device UA76084 = {S104288, 156, { &P15575,&P5629,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-I9100 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76576 = {S104289, 96, { &P11744,&P5026,&P4948,&P18092 }};	/*Useragent - BlackBerry8300/4.5.0.182 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102 UP.Browser/5.0.3.3*/
const Device UA80193 = {S104290, 20, { &P15896,&P5765,&P3955,&P18092 }};	/*Useragent - BlackBerry5810/3.6.0*/
const Device UA81548 = {S104291, 20, { &P20110,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-C406/1.0*/
const Device UA82120 = {S104292, 20, { &P20159,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-X486/1.0*/
const Device UA82270 = {S104293, 36, { &P15515,&P7472,&P4948,&P18092 }};	/*Useragent - Alcatel-BG3/1.0 UP.Browser/5.0.3.1.2*/
const Device UA83472 = {S104294, 80, { &P12604,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonS700c/R3B SEMC-Browser/4.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA83585 = {S104295, 83, { &P17439,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonT123i/R1DA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA83668 = {S104296, 17, { &P12924,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonV600i*/
const Device UA83733 = {S104297, 21, { &P16191,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonZ200/R101*/
const Device UA83826 = {S104298, 39, { &P16497,&P17483,&P3625,&P18092 }};	/*Useragent - LG-C2000 UP.Browser/6.2.3 (GUI) MMP/1.0*/
const Device UA83872 = {S104299, 8, { &P14232,&P17483,&P17660,&P18092 }};	/*Useragent - LG-KE970*/
const Device UA84070 = {S104300, 37, { &P16641,&P17483,&P17470,&P18092 }};	/*Useragent - LG-S5000 MIC/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA84650 = {S104301, 97, { &P17130,&P19944,&P4901,&P18092 }};	/*Useragent - SHARP-TQ-GX10i/1.1 Profile/MIDP-1.0 Configuration/CLDC-1.0 UP.Browser/6.1.0.6.1.d.2 (GUI) MMP/1.0*/
const Device UA85340 = {S104302, 70, { &P16505,&P17017,&P3534,&P18092 }};	/*Useragent - PG-C150/R01 Browser/Obigo/Q05A Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA86942 = {S104303, 93, { &P12154,&P17484,&P4924,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungSGHE570V/E570BUGA3 Browser/NF/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA86978 = {S104304, 67, { &P16259,&P5448,&P4578,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Xda cosmo; 320x240)*/
const Device UA87034 = {S104305, 66, { &P16227,&P5448,&P22271,&P18092 }};	/*Useragent - Xda_nova; 240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA87141 = {S104306, 47, { &P17499,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-CA37 UP.Browser/6.2.0.12.1.3 (GUI) MMP/2.0*/
const Device UA87375 = {S104307, 129, { &P16054,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) PPC; 240x320; HTC_P3600i/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87418 = {S104308, 119, { &P13837,&P5448,&P22270,&P18092 }};	/*Useragent - dopod S900_CMCC/5.2.1941/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA87556 = {S104309, 39, { &P15440,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KG120 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA87620 = {S104310, 126, { &P17126,&P22857,&P17470,&P18092 }};	/*Useragent - BIRD.F501/1.00 Nucleus RTOS/V1.11.19 MTK6227/05C_V24 Release/11.21.2006 Browser/Teleca Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA87734 = {S104311, 127, { &P17061,&P22857,&P17470,&P18092 }};	/*Useragent - Spice S650/KAA736_0.90.626 Nucleus/1.0 MTK/6223 Release/06.26.2007 Browser/Teleca_obigo Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA87747 = {S104312, 86, { &P17506,&P17662,&P3976,&P18092 }};	/*Useragent - SoftBank/1.0/816SH/SHJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87839 = {S104313, 102, { &P14105,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) Vodafone/1.0/HTC_VPACompactIV/4.14.162.2*/
const Device UA87898 = {S104314, 23, { &P17800,&P17811,&P17810,&P18092 }};	/*Useragent - LENOVO-V800 LENOVO-V800*/
const Device UA88433 = {S104315, 79, { &P14495,&P16000,&P19841,&P18092 }};	/*Useragent - Mozilla/4.1 (compatible; MSIE 5.0; Symbian OS; Nokia 3650;452) Opera 6.20  [en]*/
const Device UA88451 = {S104316, 121, { &P17772,&P5448,&P22270,&P18092 }};	/*Useragent - dopod T2222_CMCC/5.2.19578/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA88603 = {S104317, 79, { &P16612,&P17483,&P3625,&P18092 }};	/*Useragent - LG-KG810 UP.Browser/6.2.3 (GUI) MMP/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88902 = {S104318, 24, { &P17753,&P17017,&P17470,&P18092 }};	/*Useragent - WELLCOM_W819 WAP Browser*/
const Device UA89090 = {S104319, 92, { &P16626,&P17603,&P4203,&P18092 }};	/*Useragent - PANTECH-EUROPA-U4000-orange/1.0 Obigo/Q04C MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA90595 = {S104320, 127, { &P12765,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; A100 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA90782 = {S104321, 152, { &P12697,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; es-es; HTC/Shooter_U/1.01.161.1 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA213502 = {S104322, 48, { &P15364,&P18575,&P22247,&P18092 }};	/*Useragent - Flipboard/1.5.1 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA253774 = {S104323, 8, { &P15364,&P17017,&P4813,&P18092 }};	/*Useragent - Chrome/0*/
const Device UA440270 = {S104324, 94, { &P15364,&P17017,&P4813,&P18092 }};	/*Useragent - Mozilla/5.0 (SaferSurf) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.2.149.30 Safari/525.13*/
const Device UA253823 = {S104325, 43, { &P15364,&P17485,&P4722,&P18092 }};	/*Useragent - sonyericssonw810i/UC Browser7.0.0.41/69/352*/
const Device UA253964 = {S104326, 43, { &P13183,&P17485,&P4779,&P18092 }};	/*Useragent - sonyericssonk810i/UC Browser7.8.0.95/70/351*/
const Device UA257586 = {S104327, 81, { &P15364,&P17285,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; Opera Mobi/23731; U; en) Presto/2.9.201 Version/11.50*/
const Device UA322672 = {S104328, 80, { &P15364,&P17285,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; Opera Mobi/14316; U; en) Presto/2.7.81 Version/11.00*/
const Device UA429367 = {S104329, 91, { &P13144,&P7116,&P4779,&P18092 }};	/*Useragent - BlackBerry9860/5.0.0.496 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/161/350*/
const Device UA429430 = {S104330, 90, { &P11639,&P7149,&P4779,&P18092 }};	/*Useragent - BlackBerry9000/5.2.0.93 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.8.0.95/159/351*/
const Device UA438848 = {S104331, 150, { &P14030,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.9; zh-tw; HTC T9199 For Vodafone Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA440999 = {S104332, 120, { &P15767,&P18117,&P18357,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B176 Safari/ Sleipnir/1.8.3m*/
const Device UA447261 = {S104333, 16, { &P16971,&P17484,&P17606,&P18092 }};	/*Useragent - samsung-sgh-a877*/
const Device UA458575 = {S104334, 143, { &P17968,&P5629,&P21559,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/2.2.0 Safari/533.1*/
const Device UA2026051 = {S104335, 143, { &P17968,&P5629,&P21559,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/2.2.0 Safari/533.1*/
const Device UA466614 = {S104336, 29, { &P14358,&P5890,&P17349,&P18092 }};	/*Useragent - NokiaE63/UCWEB7.3.1.56/28/999*/
const Device UA515226 = {S104337, 29, { &P13163,&P5129,&P24660,&P18092 }};	/*Useragent - NokiaN97/UCWEB7.6.0.75/50/999*/
const Device UA546520 = {S104338, 63, { &P12110,&P18110,&P24660,&P18092 }};	/*Useragent - JUC(Linux;U;2.2;Zh_cn;HTC Legend;320*480;)UCWEB7.6.0.75/139/800*/
const Device UA560291 = {S104339, 29, { &P14454,&P5890,&P24662,&P18092 }};	/*Useragent - NokiaE66/UCWEB7.8.0.87/28/800*/
const Device UA566088 = {S104340, 9, { &P12599,&P17483,&P17660,&P18092 }};	/*Useragent - lg-lg800g*/
const Device UA574712 = {S104341, 42, { &P13536,&P5890,&P24663,&P18092 }};	/*Useragent - NOKIA6110/UC Browser7.9.0.102/28/355/UCWEB*/
const Device UA616518 = {S104342, 160, { &P14993,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; Xperia Arc Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616582 = {S104343, 160, { &P14993,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xperia Arc Build/IMM76D; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616590 = {S104344, 141, { &P14993,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; Xperia Arc Build/MIUIv4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616596 = {S104345, 141, { &P14993,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xperia Arc Build/MIUIv4) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA3039582 = {S104346, 190, { &P24225,&P18110,&P24300,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; U; Linux 35230; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.1.4(+mouse+3D+SCREEN+TUNER; LGE; 42LW5700-TA; 04.03.10; 0xX;); LG NetCast.TV-2011*/
const Device UA618246 = {S104347, 101, { &P12280,&P17137,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]*/
const Device UA618268 = {S104348, 99, { &P15767,&P18117,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]*/
const Device UA656498 = {S104349, 43, { &P17595,&P18575,&P17470,&P18092 }};	/*Useragent - Reeder/1.4 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA664754 = {S104350, 104, { &P18254,&P17658,&P18392,&P18168 }};	/*Useragent - DoCoMo/2.0 P900i(c100;TB;W24H11)(compatible; ichiro/mobile goo;+http://help.goo.ne.jp/door/crawler.html)*/
const Device UA788461 = {S104351, 35, { &P15803,&P5843,&P23401,&P18092 }};	/*Useragent - MQQBrowser/Mini2.5 (Nokia200/10.58)*/
const Device UA842221 = {S104352, 118, { &P15364,&P7127,&P17317,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1*/
const Device UA857481 = {S104353, 143, { &P17968,&P5096,&P21557,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/2.1.0 Safari/533.1*/
const Device UA879566 = {S104354, 145, { &P17937,&P5096,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; MOTWX435KT Build/FROYO)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA896993 = {S104355, 143, { &P18280,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC One S Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA907421 = {S104356, 36, { &P18176,&P5448,&P4702,&P18092 }};	/*Useragent - NOKIA Lumia 900/2175.1601.8773.12141*/
const Device UA1956848 = {S104357, 190, { &P15575,&P18066,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-I9100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, in*/
const Device UA925859 = {S104358, 98, { &P20276,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Ultimate)*/
const Device UA1894548 = {S104359, 40, { &P12375,&P5890,&P4779,&P18092 }};	/*Useragent - NOKIANokia N81/UC Browser7.8.0.95/28/355*/
const Device UA1904378 = {S104360, 52, { &P12061,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (NokiaE65;SymbianOS/9.1 Series60/3.0)*/
const Device UA1927479 = {S104361, 32, { &P12449,&P5890,&P24666,&P18092 }};	/*Useragent - NOKIAE5-00/UCWEB8.1.0.104/28/800*/
const Device UA1946229 = {S104362, 123, { &P15364,&P17285,&P24201,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1 QQBrowser/6.10.11509.201*/
const Device UA1956970 = {S104363, 96, { &P15364,&P17017,&P18275,&P18092 }};	/*Useragent - Mozilla/5.0 (SaferSurf) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.21 Safari/536.11*/
const Device UA1973082 = {S104364, 86, { &P13220,&P17485,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; sonyericssonk770i) UCBrowser8.3.0.154/69/352/UCWEB Mobile*/
const Device UA1980264 = {S104365, 136, { &P17926,&P6437,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2.1; en-us; AT100 Build/HTK55D)Maxthon AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA1987900 = {S104366, 142, { &P15767,&P17145,&P20775,&P18092 }};	/*Useragent - MQQBrowser/2.4 Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Safari/7534.48.3*/
const Device UA1992027 = {S104367, 37, { &P14540,&P5843,&P23403,&P18092 }};	/*Useragent - MQQBrowser/Mini2.6 (NokiaX2-01/08.63)*/
const Device UA1997446 = {S104368, 93, { &P15364,&P17017,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (SaferSurf) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1165.0 Safari/537.1*/
const Device UA1998539 = {S104369, 130, { &P17595,&P17297,&P18532,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/534.56.5 (KHTML, like Gecko) Version/5.1 Safari/7534.51.22 Sleipnir/3.6*/
const Device UA2007459 = {S104370, 73, { &P16329,&P22221,&P17652,&P18092 }};	/*Useragent - Mozilla/5.0 (PLAYSTATION 3 4.23) AppleWebKit/531.22.8 (KHTML, like Gecko)*/
const Device UA2019703 = {S104371, 182, { &P13042,&P5629,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC_Incredible_S Build/GRI40; 480*800) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/800*/
const Device UA2020440 = {S104372, 73, { &P17968,&P17979,&P17600,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; Linux armv7l; rv:8.0) Gecko/ Firefox/8.0 Fennec/8.0*/
const Device UA2022137 = {S104373, 155, { &P17283,&P24694,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; MB860 Build/MIUI-##VERSION##-OLYMPUS-EN) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022497 = {S104374, 128, { &P16033,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; MT810 Build/ECLAIR) UC AppleWebKit/530+ (KHTML, like Gecko) Mobile Safari/530*/
const Device UA2023075 = {S104375, 134, { &P22561,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; nl-nl; TAB411 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024205 = {S104376, 163, { &P15897,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; XT530 Build/V4.530) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2024386 = {S104377, 159, { &P15501,&P5477,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.53) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2024462 = {S104378, 101, { &P15794,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.5;Zh_cn;HTC Sensation XL with Beats Audio X315e;480*800;)UCWEB7.8.0.95/139/351*/
const Device UA2024888 = {S104379, 165, { &P14123,&P5096,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P970 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2025451 = {S104380, 167, { &P12968,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; LT26i Build/6.0.A.3.62) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2025594 = {S104381, 157, { &P14186,&P5629,&P20780,&P18092 }};	/*Useragent - MQQBrowser/2.9/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-S5830 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027253 = {S104382, 159, { &P13042,&P5629,&P20784,&P18092 }};	/*Useragent - MQQBrowser/3.3/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; HTC Incredible S Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027288 = {S104383, 165, { &P14123,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; LG-P970 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2028424 = {S104384, 165, { &P15060,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; GT-S5660 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2029265 = {S104385, 157, { &P12814,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I777 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029615 = {S104386, 143, { &P17968,&P18066,&P21557,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/2.1.0 Safari/533.1*/
const Device UA2030719 = {S104387, 166, { &P13591,&P5096,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; ZTE-BLADE Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2031808 = {S104388, 169, { &P14969,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2033571 = {S104389, 186, { &P13644,&P5629,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; PC36100 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, us*/
const Device UA2217656 = {S104390, 35, { &P14519,&P5129,&P24668,&P18092 }};	/*Useragent - NokiaN97 mini/UCWEB8.3.0.133/50/999*/
const Device UA2241486 = {S104391, 110, { &P18385,&P17979,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.0001.8177(Android;Android (Nexus 7-grouper-user-4.1.2);ja-JP;kml:2.2;client:Free;type:default)*/
const Device UA2242126 = {S104392, 146, { &P18385,&P18111,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; Nexus 7 Build/JRO03D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2242176 = {S104393, 144, { &P18385,&P18111,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1; zh-cn; Nexus 7 Build/JRN84D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2242389 = {S104394, 60, { &P15364,&P5690,&P17191,&P18163 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)/Nutch-1.0*/
const Device UA2275947 = {S104395, 29, { &P14454,&P5890,&P24660,&P18092 }};	/*Useragent - NokiaE66/UCWEB7.6.0.75/28/999*/
const Device UA2319406 = {S104396, 37, { &P14826,&P5843,&P23404,&P18092 }};	/*Useragent - MQQBrowser/Mini2.7 (NokiaC1-01/04.40)*/
const Device UA2334789 = {S104397, 132, { &P12280,&P17084,&P17138,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; 6532.9) AppleWebKit/en_US (KHTML, like Gecko) Mobile/8C148 Safari/6533.18.5*/
const Device UA2428401 = {S104398, 90, { &P13365,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-gu220) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2457721 = {S104399, 98, { &P16366,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s8600) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2458061 = {S104400, 90, { &P11653,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; sgh-f480) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2462444 = {S104401, 31, { &P19748,&P5043,&P21724,&P18092 }};	/*Useragent - OneBrowser/3.0 (Nokia205/03.18)*/
const Device UA2483440 = {S104402, 99, { &P16373,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s7250d) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2540543 = {S104403, 34, { &P21002,&P17484,&P21724,&P18092 }};	/*Useragent - OneBrowser/3.0 (SAMSUNG-GT-C3312R)*/
const Device UA2833723 = {S104404, 142, { &P14482,&P5843,&P24670,&P18092 }};	/*Useragent - NokiaC3-01/5.0 (05.65) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-in; nokiac3-01) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2578849 = {S104405, 98, { &P12273,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-m5650) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2586583 = {S104406, 141, { &P21602,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5302 Build/12.0.A.1.222) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2586655 = {S104407, 151, { &P21628,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8552 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2607482 = {S104408, 64, { &P17669,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;T7575;7.10;8862)*/
const Device UA2607479 = {S104409, 66, { &P14726,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;LG;LG-C900B;7.10;8862)*/
const Device UA2607528 = {S104410, 138, { &P21792,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-I9200 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2607536 = {S104411, 138, { &P21792,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.9; GT-I9200 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2607595 = {S104412, 129, { &P14905,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7100 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2631330 = {S104413, 148, { &P13452,&P5096,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2; zh-tw; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2608389 = {S104414, 154, { &P13452,&P5629,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2627061 = {S104415, 154, { &P13452,&P5629,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.4; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2608487 = {S104416, 142, { &P15985,&P6437,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2609302 = {S104417, 157, { &P13452,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2635680 = {S104418, 157, { &P13452,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2610567 = {S104419, 151, { &P13452,&P5096,&P20784,&P18092 }};	/*Useragent - MQQBrowser/3.3/Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2611112 = {S104420, 149, { &P13452,&P5096,&P20784,&P18092 }};	/*Useragent - MQQBrowser/3.3/Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2613922 = {S104421, 150, { &P13452,&P5096,&P20784,&P18092 }};	/*Useragent - MQQBrowser/3.3/Mozilla/5.0 (Linux; U; Android 2.2; en-au; GT-P1000T Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2610853 = {S104422, 166, { &P17323,&P6437,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [operamini] [UsableNet Lift Mobile]*/
const Device UA2613603 = {S104423, 173, { &P17337,&P6546,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37; 800*1280) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.4.145/139/800*/
const Device UA2613853 = {S104424, 30, { &P17323,&P17979,&P20428,&P18092 }};	/*Useragent - Android OS/13 samsung_GT-P6800*/
const Device UA2626558 = {S104425, 132, { &P17323,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/1.0 (Linux; Java; android; Android SDK 15; samsung-GT-P6800; Mobile Safari;BOCHK 4.3.4) NetUtil/2.0 HTTPCore/1.0 SS/800x1232*/
const Device UA2614026 = {S104426, 147, { &P15919,&P18066,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616203 = {S104427, 147, { &P15919,&P18066,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.3; ar-ae; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616488 = {S104428, 147, { &P17323,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616608 = {S104429, 146, { &P15919,&P18066,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 4.0.3; ar-ae; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2624252 = {S104430, 146, { &P15919,&P18066,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617212 = {S104431, 36, { &P14908,&P17979,&P18062,&P18092 }};	/*Useragent - Pandora/1.7.1 Android/4.0.4 GT-P6210*/
const Device UA2617410 = {S104432, 166, { &P15985,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6200 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2617666 = {S104433, 140, { &P22037,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0; en-gb; GT-P3113 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2617759 = {S104434, 140, { &P15932,&P18066,&P20791,&P18092 }};	/*Useragent - MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2620713 = {S104435, 152, { &P15932,&P18066,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2627975 = {S104436, 55, { &P15919,&P17979,&P22076,&P18092 }};	/*Useragent - WhatsApp/2.7.8402 Android/4.0.3 Device/samsung-GT-P3100*/
const Device UA2618416 = {S104437, 94, { &P15932,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Mail; 1.4.6) (espressowifi; samsung; GT-P3110; 4.0.4/IMM76D)*/
const Device UA2618458 = {S104438, 171, { &P14908,&P5629,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-P1010 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2618531 = {S104439, 55, { &P13452,&P5629,&P22076,&P18092 }};	/*Useragent - WhatsApp/2.7.4705 Android/2.3.3 Device/samsung-GT-P1000*/
const Device UA2618558 = {S104440, 57, { &P17323,&P17979,&P22697,&P18092 }};	/*Useragent - TwitterAndroid/1.0.5 (109) GT-P6800/15 (samsung;GT-P6800)*/
const Device UA2618673 = {S104441, 188, { &P13452,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.7/210,1.9.7.210,2012-10-25 19:53, it*/
const Device UA2618780 = {S104442, 90, { &P14908,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Mail; 1.4.6) (GT-P6210; samsung; GT-P6210; 4.0.4/IMM76D)*/
const Device UA2618972 = {S104443, 90, { &P12879,&P17979,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 Android (13/3.2; 160dpi; 800x1232; samsung; GT-P7510; GT-P7510; p3; en_US)*/
const Device UA2618991 = {S104444, 33, { &P15882,&P17979,&P20428,&P18092 }};	/*Useragent - Android OS/4.0.3 samsung_GT-P5110*/
const Device UA2619573 = {S104445, 110, { &P12879,&P17979,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.0000.7386(Android;Android (GT-P7510-GT-P7510-user-3.2);nl-NL;kml:2.2;client:Free;type:default)*/
const Device UA2619698 = {S104446, 147, { &P15882,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2619877 = {S104447, 147, { &P15882,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2619882 = {S104448, 104, { &P15876,&P17979,&P22068,&P18092 }};	/*Useragent - Instagram 1.0.6 Android (15/4.0.3; 160dpi; 800x1232; samsung; GT-P5100; espresso10rf; espresso10; en_GB)*/
const Device UA2620721 = {S104449, 174, { &P15985,&P18066,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6200 Build/IMM76D; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444*/
const Device UA2620946 = {S104450, 30, { &P12889,&P17979,&P20428,&P18092 }};	/*Useragent - Android OS/12 samsung_GT-P7300*/
const Device UA2621191 = {S104451, 140, { &P15985,&P18066,&P20791,&P18092 }};	/*Useragent - MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6200 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2626115 = {S104452, 152, { &P15985,&P18066,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6200 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2622403 = {S104453, 171, { &P14908,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us ; GT-P1010 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2629179 = {S104454, 190, { &P18478,&P6437,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6201 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.22/288,1.9.22.288,2013-01-31 13:09, +0000, gb*/
const Device UA2622377 = {S104455, 90, { &P12879,&P17979,&P22069,&P18092 }};	/*Useragent - Instagram 1.1.8 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7510; GT-P7510; p3; en_US)*/
const Device UA2622427 = {S104456, 87, { &P17323,&P18066,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.4.5.12736); (Linux; U; Android 4.0.4; GT-P6800 Build/Unknown);*/
const Device UA2622581 = {S104457, 164, { &P12879,&P6437,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7510 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2622666 = {S104458, 145, { &P22043,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2623955 = {S104459, 147, { &P22043,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622732 = {S104460, 191, { &P18475,&P6437,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-au; GT-P7320T Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.16/276,1.9.16.276,2012-12-21 16:37, +1100, au*/
const Device UA2625575 = {S104461, 190, { &P18475,&P6437,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7320 Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +0800, sg*/
const Device UA2623025 = {S104462, 95, { &P17931,&P17979,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.0 Android (8/2.2; 240dpi; 600x1024; samsung; SGH-T849; SGH-T849; GT-P1000; en_US)*/
const Device UA2623045 = {S104463, 174, { &P17323,&P6437,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/999*/
const Device UA2623285 = {S104464, 208, { &P15932,&P22212,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P3110 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.16/276,1.9.16.276,2012-12-21 16:37, +0000, gb*/
const Device UA2623381 = {S104465, 192, { &P17337,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-tw; GT-P7500 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0800, tw*/
const Device UA2629001 = {S104466, 192, { &P17337,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-sg; GT-P7500 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.20/286,1.9.20.286,2013-01-25 13:38, +0800, sg*/
const Device UA2623891 = {S104467, 208, { &P12889,&P22212,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7300 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.16/276,1.9.16.276,2012-12-21 16:37, +0000, gh*/
const Device UA2623951 = {S104468, 164, { &P22037,&P18598,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3113 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3113, 38, 1.8.1)*/
const Device UA2635045 = {S104469, 171, { &P22037,&P18598,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3113 Build/JOP40G) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (GT-P3113, 40, 1.8.5)*/
const Device UA2624012 = {S104470, 161, { &P12889,&P6546,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7300, 39, 1.8.2)*/
const Device UA2624297 = {S104471, 166, { &P12889,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P7300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2628541 = {S104472, 147, { &P13452,&P5096,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 2.2; fa-fa; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2625763 = {S104473, 168, { &P15882,&P18066,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]*/
const Device UA2633215 = {S104474, 168, { &P15882,&P18066,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; de-de; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]*/
const Device UA2626042 = {S104475, 116, { &P15876,&P17979,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.0002.8421(Android;Android (GT-P5100-espresso10rf-user-4.0.3);it-IT;kml:2.2;client:Free;type:default)*/
const Device UA2626467 = {S104476, 162, { &P14908,&P6437,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6210 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P6210, 39, 1.8.2)*/
const Device UA2626752 = {S104477, 153, { &P14908,&P6437,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0(Linux; U; Android 3.2; zh-CN; GT-P6210 Build/HTJ85B) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.0.212 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2634471 = {S104478, 29, { &P18478,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (GT-P6201 HTJ85B)*/
const Device UA2627107 = {S104479, 118, { &P22043,&P17979,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.0001.8177(Android;Android (GT-P5113-espresso10wifi-user-4.1.1);en-CA;kml:2.2;client:Free;type:default)*/
const Device UA2635730 = {S104480, 156, { &P22042,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P3108 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.1.258 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2627431 = {S104481, 154, { &P15882,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2627723 = {S104482, 147, { &P15882,&P18111,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2627441 = {S104483, 164, { &P14905,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7100, 39, 1.8.2)*/
const Device UA2627540 = {S104484, 162, { &P12889,&P6437,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7300, 39, 1.8.2)*/
const Device UA2628417 = {S104485, 81, { &P22037,&P17979,&P4459,&P18092 }};	/*Useragent - Evernote Android/AndroidMarket_45x_243.14540 (en_CA); Android/4.1.1; GT-P3113/16;*/
const Device UA2628677 = {S104486, 149, { &P17323,&P6437,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2628807 = {S104487, 45, { &P13452,&P5096,&P22287,&P18092 }};	/*Useragent - TuneIn Radio/5.2 (Android 8; GT-P1000M; Java)*/
const Device UA2631328 = {S104488, 44, { &P13452,&P5096,&P22287,&P18092 }};	/*Useragent - TuneIn Radio/5.2 (Android 8; GT-P1000; Java)*/
const Device UA2636294 = {S104489, 118, { &P15882,&P17979,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.0003.8527(Android;Android (GT-P5110-espresso10wifi-user-4.1.1);en-GB;kml:2.2;client:Free;type:default)*/
const Device UA2629390 = {S104490, 164, { &P15985,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P6200, 39, 1.8.2)*/
const Device UA2629412 = {S104491, 157, { &P13452,&P18066,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2631230 = {S104492, 159, { &P13452,&P18066,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P1000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2629522 = {S104493, 151, { &P15932,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2636307 = {S104494, 151, { &P15932,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P3110 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2629707 = {S104495, 144, { &P12879,&P6546,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2637994 = {S104496, 151, { &P12879,&P6546,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2630546 = {S104497, 155, { &P22041,&P6437,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6810 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 MobileSafari/534.13 Maxthon/4.0.3.3000*/
const Device UA2630723 = {S104498, 166, { &P13452,&P18149,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P1000 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2631159 = {S104499, 166, { &P12889,&P6437,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ru-ru; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [operamini] [UsableNet Lift Mobile]*/
const Device UA2631201 = {S104500, 192, { &P19852,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P7511 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0100, de*/
const Device UA2631425 = {S104501, 182, { &P15932,&P22211,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3110 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3110, 40, 1.8.5)*/
const Device UA2632654 = {S104502, 181, { &P15932,&P22211,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3110 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3110, 40, 1.8.5)*/
const Device UA2631515 = {S104503, 56, { &P15985,&P17979,&P22278,&P18092 }};	/*Useragent - theScore/3.1.0 Android-OS/3.2 (Android; Phone, GT-P6200)*/
const Device UA2631840 = {S104504, 149, { &P17323,&P6437,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ru-ru; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.3.3000*/
const Device UA2632278 = {S104505, 151, { &P15882,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P5110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2635802 = {S104506, 151, { &P15882,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P5110 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2632808 = {S104507, 166, { &P15985,&P6437,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [operamini] [UsableNet Lift Mobile]*/
const Device UA2632874 = {S104508, 136, { &P15919,&P18111,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; ru-ru; GT-P3100 Build/JZO54K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7*/
const Device UA2633047 = {S104509, 210, { &P17323,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; da-dk; GT-P6800 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0700, dk*/
const Device UA2635787 = {S104510, 209, { &P17323,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-P6800 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.27/381,1.9.27.381,2013-03-21 11:39, +0000, gb*/
const Device UA2635080 = {S104511, 130, { &P22052,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7501 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Safari/537.22*/
const Device UA2633158 = {S104512, 157, { &P17168,&P6437,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7310 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13 UCBrowser/2.2.2.259*/
const Device UA2633861 = {S104513, 151, { &P14908,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2633915 = {S104514, 154, { &P12889,&P6437,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-CN; GT-P7300 Build/HTJ85B) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2634385 = {S104515, 162, { &P12879,&P18066,&P24406,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 OverDrive Media Console/2.6.4*/
const Device UA2634875 = {S104516, 58, { &P22037,&P17979,&P22280,&P18092 }};	/*Useragent - theScore/3.2.1 Android-OS/4.0.4 (Android; Phone, GT-P3113)*/
const Device UA2637405 = {S104517, 152, { &P17337,&P18066,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2636794 = {S104518, 151, { &P15882,&P18066,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; GT-P5110 Build/IML74K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.53463*/
const Device UA2637849 = {S104519, 152, { &P15882,&P18111,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2681005 = {S104520, 155, { &P22385,&P18111,&P20787,&P18092 }};	/*Useragent - MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-I8262D Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2681006 = {S104521, 88, { &P22385,&P18111,&P20787,&P18092 }};	/*Useragent - MQQBrowser/3.6/Adr (Linux; U; 4.1.2; zh-cn; GT-I8262D Build/JZO54K.I8262DZCAMA9;480*800)*/
const Device UA2681015 = {S104522, 88, { &P22385,&P18111,&P20787,&P18092 }};	/*Useragent - MQQBrowser/3.6/Adr (Linux; U; 4.1.2; zh-cn; GT-I8262D Build/JZO54K.I8262DZCAMB2;480*800)*/
const Device UA2719716 = {S104523, 143, { &P22654,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; C5502 Build/10.1.1.A.1.226) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2741789 = {S104524, 139, { &P22715,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; OUYA Console Build/JZO54K-OUYA) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Safari/537.31*/
const Device UA2749913 = {S104525, 92, { &P23343,&P22846,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; lemon t99) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2751414 = {S104526, 99, { &P14404,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5260) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2762786 = {S104527, 98, { &P16971,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-sgh-a877) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2774402 = {S104528, 196, { &P22871,&P18598,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-I9295 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.1/549,2.0.1.549,2013-05-15 23:10, +0200, pl*/
const Device UA2776407 = {S104529, 85, { &P14864,&P17484,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; samsung-gt-s8500) UCBrowser8.2.1.144/70/352/UCWEB Mobile*/
const Device UA2867543 = {S104530, 88, { &P15518,&P17017,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; s5230) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2951730 = {S104531, 99, { &P12849,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s7230e) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2964905 = {S104532, 92, { &P11658,&P5776,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6680) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2977057 = {S104533, 86, { &P24051,&P18110,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv6l; U; en) Presto/2.8.115 Version/11.10 AQUOS-AS/1.0 LC-40LE835M*/
const Device UA2977063 = {S104534, 86, { &P24086,&P18110,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv6l; U; en) Presto/2.8.115 Version/11.10 AQUOS-AS/1.0 LC-70LE735M*/
const Device UA2977066 = {S104535, 86, { &P24089,&P18110,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv6l; U; en) Presto/2.8.115 Version/11.10 AQUOS-AS/1.0 LC-60LE835X*/
const Device UA2977320 = {S104536, 76, { &P15364,&P7133,&P24186,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:26.0) Gecko/X Firefox/26.0 slurp*/
const Device UA3011782 = {S104537, 99, { &P14353,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5250) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3017063 = {S104538, 99, { &P14864,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8500) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3039562 = {S104539, 223, { &P15364,&P18110,&P24302,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LA740S-ZB; 03.20.04; 0xX;); LG NetCast.TV-2013 /03.20.04 (LG, 47LA740S-ZB, wireless)*/
const Device UA3039569 = {S104540, 225, { &P15364,&P18110,&P24302,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; GLOBAL-PLAT4; 03.20.00; 0xX;); LG NetCast.TV-2013 /03.20.00 (LG, GLOBAL-PLAT4, wireless)*/
const Device UA3039565 = {S104541, 223, { &P24230,&P18110,&P24302,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LA6210-TD; 03.10.61; 0xX;); LG NetCast.TV-2013 /03.10.61 (LG, 47LA6210-TD, wireless)*/
const Device UA3039568 = {S104542, 223, { &P24235,&P18110,&P24302,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LA8600-NA; 03.20.23; 0xX;); LG NetCast.TV-2013 /03.20.23 (LG, 47LA8600-NA, wireless)*/
const Device UA3039572 = {S104543, 223, { &P24239,&P18110,&P24302,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LA6200-TA; 03.20.09; 0xX;); LG NetCast.TV-2013 /03.20.09 (LG, 42LA6200-TA, wireless)*/
const Device UA3039647 = {S104544, 200, { &P24250,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 65LM6200-TA; 04.00.28; 0xX;); LG NetCast.TV-2012*/
const Device UA3039592 = {S104545, 200, { &P24257,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 55LM6400-UA; 04.12.23; 0xX;); LG NetCast.TV-2012*/
const Device UA3039596 = {S104546, 200, { &P24285,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LM660S-ZA; 04.10.23; 0xX;); LG NetCast.TV-2012*/
const Device UA3039600 = {S104547, 197, { &P24289,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+SCREEN+TUNER; LGE; 42LS5700-TB; 03.09.38; 0xX;); LG NetCast.TV-2012*/
const Device UA3039623 = {S104548, 200, { &P24324,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LM6700-DA; 04.10.11; 0xX;); LG NetCast.TV-2012*/
const Device UA3039638 = {S104549, 200, { &P24324,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LM6700-TA; 04.10.10; 0xX;); LG NetCast.TV-2012*/
const Device UA3039685 = {S104550, 253, { &P24296,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_WL968; 7.0.27.3; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039689 = {S104551, 253, { &P24296,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_WL968; 7.0.27.8; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; pl) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039726 = {S104552, 254, { &P24296,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_WL968; 7.0.27.13; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3058966 = {S104553, 93, { &P11905,&P5890,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6120c) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3068918 = {S104554, 136, { &P18471,&P18111,&P24394,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-78) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1612.0 Mobile Safari/537.36*/
const Device UA3068931 = {S104555, 127, { &P18639,&P21637,&P24394,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; Nexus 10 Build/JWR66Y) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1612.0 Safari/537.36*/
const Device UA3068981 = {S104556, 155, { &P18635,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; Nexus 4 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.55 Mobile Safari/537.36 OPR/16.0.1212.62222*/
const Device UA3084052 = {S104557, 154, { &P18635,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2; Nexus 4 Build/JOP40C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63417*/
const Device UA3084056 = {S104558, 160, { &P21411,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Q-Smart S20 Build/IMM76I) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084065 = {S104559, 152, { &P23162,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; B15 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084066 = {S104560, 162, { &P15649,&P5629,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-S5570 Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084098 = {S104561, 160, { &P18195,&P5629,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.5; YP-G70 Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084099 = {S104562, 157, { &P18002,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; IM-A760S Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084427 = {S104563, 88, { &P18095,&P18109,&P24500,&P18092 }};	/*Useragent - Opera/9.80 (Series 60; Opera Mini/17.3.2013/28.3692; U; en) Presto/2.8.119 Version/11.10*/
const Device UA727 = {S104564, 70, { &P17489,&P5448,&P22271,&P18092 }};	/*Useragent - HTC_P5520 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA2657 = {S104565, 74, { &P12018,&P5448,&P17173,&P18092 }};	/*Useragent - HTC-8500/1.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA2667 = {S104566, 144, { &P12025,&P5448,&P22271,&P18092 }};	/*Useragent - SAMSUNG-SGH-i718plus_CMCC/1.0 Release/03.07 Browser/IE6 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA3189 = {S104567, 156, { &P14784,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android ChocolateEclair 1.0.1; en-gb; HTC Hero Build/CE1.0.1) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA3541 = {S104568, 121, { &P17148,&P17017,&P17470,&P18092 }};	/*Useragent - BIRD_D730/V1.0.0/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 BIRD_D730/V1.0.0/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA88523 = {S104569, 60, { &P17148,&P17017,&P17470,&P18092 }};	/*Useragent - BIRD_D730/V1.0.0/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA3640 = {S104570, 130, { &P13279,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-P858/1.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzU3ODk0MDAwMDgyNzMw UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA8035 = {S104571, 130, { &P16381,&P5448,&P17173,&P18092 }};	/*Useragent - PPC; 240x320; HTC_WIZARD/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA8903 = {S104572, 98, { &P13362,&P17484,&P4924,&P18092 }};	/*Useragent - SAMSUNG-SGH-D838/NetFront 3.2/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzUyNjY2MDEwMTk0NTU1*/
const Device UA8981 = {S104573, 89, { &P13367,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) O2 Xda Zinc*/
const Device UA13799 = {S104574, 103, { &P12473,&P5204,&P4737,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2730c-1/UC Browser7.2.2.51/70/352*/
const Device UA14224 = {S104575, 88, { &P17923,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8; DopodU1000) Opera 8.65 [en]*/
const Device UA14552 = {S104576, 139, { &P13514,&P17486,&P17671,&P18092 }};	/*Useragent - MOT-MOTOVE66_CMCC/1.0 LinuxOS/2.6.10 Release/11.28.2008 Browser/OSS1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R6743_G_74.31.00RPL*/
const Device UA15529 = {S104577, 103, { &P14862,&P5204,&P4737,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (07.80) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2700c-2/UC Browser7.2.2.51/70/352*/
const Device UA15868 = {S104578, 71, { &P13534,&P17017,&P3601,&P18092 }};	/*Useragent - BenQ-E53/1.0 ACS-NF/3.3 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA15965 = {S104579, 37, { &P13539,&P17017,&P17470,&P18092 }};	/*Useragent - Fly_DS185/WAP2.0Configuration/Browser*/
const Device UA16303 = {S104580, 86, { &P11870,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_S620 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA16680 = {S104581, 58, { &P13909,&P17484,&P4924,&P18092 }};	/*Useragent - SAMSUNG-SGH-F308/NetFront 3.2/WAP2.0/*MzU0MjExMDEwMjkxMzM5*/
const Device UA16754 = {S104582, 83, { &P13931,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonK858c/R1DA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA16854 = {S104583, 98, { &P13945,&P17484,&P4924,&P18092 }};	/*Useragent - SAMSUNG-SGH-D808/NetFront 3.2/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzU3ODEyMDA3MjI5MjEx*/
const Device UA18184 = {S104584, 76, { &P14978,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerry9650x/5.0.0.549 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/1*/
const Device UA18815 = {S104585, 141, { &P11938,&P17486,&P17671,&P18092 }};	/*Useragent - MOT-MOTOROKRE8_CMCC/1.0 LinuxOS/2.6.10 Release/11.16.2007 Browser/OSS1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/R6718_G_71.10.13R_C*/
const Device UA19193 = {S104586, 120, { &P11951,&P17484,&P3601,&P18092 }};	/*Useragent - SAMSUNG-SGH-Z368/1.0 SHP/VPP/R5 NetFront/3.3 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1/*MzU0MDQ1MDEwMDk0NjY5*/
const Device UA20258 = {S104587, 118, { &P15057,&P17605,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-U708E/1.0 RTK-E/1.0 DF/1.0 Release/02.05.2008 Browser/NetFront3.4  Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20322 = {S104588, 43, { &P15067,&P17483,&P3625,&P18092 }};	/*Useragent - LG8550/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA20333 = {S104589, 104, { &P15071,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonT818/R1FA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.4.3*/
const Device UA20589 = {S104590, 107, { &P20125,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-P240/P240XAHB4 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA20654 = {S104591, 63, { &P15112,&P6712,&P3402,&P18092 }};	/*Useragent - NOKIA-7208/1.0 UP.Browser/6.3.0.7.g.1.100-20Mar09 (GUI) MMP/2.0*/
const Device UA21211 = {S104592, 147, { &P17064,&P7227,&P4222,&P18092 }};	/*Useragent - Vodafone/Sharp902SH/SHG001/1.0 Browser/UP.Browser/7.0.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Ext-J-Profile/JSCL-1.2.2 Ext-V-Profile/VSCL-2.0.0*/
const Device UA86895 = {S104593, 147, { &P17064,&P7227,&P4222,&P18092 }};	/*Useragent - Vodafone/Sharp902SH/SHG002/1.0 Browser/UP.Browser/7.0.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Ext-J-Profile/JSCL-1.2.2 Ext-V-Profile/VSCL-2.0.0*/
const Device UA21276 = {S104594, 135, { &P15177,&P5448,&P22371,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.9) PPC; 240x320; i-mate K-JAM PPC/1.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21452 = {S104595, 70, { &P15285,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_S630 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA21626 = {S104596, 94, { &P15305,&P19944,&P3625,&P18092 }};	/*Useragent - SHARP-TQ-GX27/1.0 Profile/MIDP-1.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.4.163 (GUI) MMP/1.0*/
const Device UA21717 = {S104597, 101, { &P15322,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E378/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA21903 = {S104598, 92, { &P15355,&P19945,&P4901,&P18092 }};	/*Useragent - Panasonic-SC3/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.1.0.7.8 (GUI) MMP/1.0*/
const Device UA22219 = {S104599, 55, { &P15403,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-VU30/00.62 UP.Browser/6.2.3.4.c.1.124 (GUI) MMP/2.0*/
const Device UA22426 = {S104600, 83, { &P15364,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonMODEL/R1JG Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22516 = {S104601, 120, { &P12700,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-E848/1.0 Release/2.21.2007 Browser/NetFront3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzU2NTQzMDE3ODM4Njgw*/
const Device UA22619 = {S104602, 61, { &P15499,&P19945,&P17470,&P18092 }};	/*Useragent - Panasonic-VS7/1.0/RP1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22685 = {S104603, 72, { &P15508,&P5704,&P4836,&P18092 }};	/*Useragent - MOT-E680/R51_G_0F.38.A4R MIB/2.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22861 = {S104604, 57, { &P15519,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-280/1.0 model-orange ObigoInternetBrowser/Q03C*/
const Device UA90174 = {S104605, 44, { &P15519,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-280/1.0 ObigoInternetBrowser/Q03C*/
const Device UA22945 = {S104606, 102, { &P15540,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) LG-MS25 IEMobile/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA22994 = {S104607, 86, { &P15544,&P17017,&P4780,&P18092 }};	/*Useragent - NexianNX-G988/MTK Release/10.01.2009 Browser/MAUI Profile/MIDP2.0 Configuration/CLDC-1*/
const Device UA23089 = {S104608, 47, { &P17909,&P17017,&P17470,&P18092 }};	/*Useragent - Huawei/1.0/T566/B000 Browser/obigo-browser/q04a*/
const Device UA23326 = {S104609, 42, { &P15586,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-M569/ UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA23809 = {S104610, 53, { &P15627,&P17483,&P17660,&P18092 }};	/*Useragent - LGE/U8150/1.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA23883 = {S104611, 78, { &P15633,&P17486,&P4203,&P18092 }};	/*Useragent - Motorola-ic502 Obigo/Q04C1-1.9 MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA24086 = {S104612, 45, { &P15664,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-C550/1.0 ObigoInternetBrowser/Q03C*/
const Device UA24311 = {S104613, 108, { &P15678,&P17484,&P3834,&P18092 }};	/*Useragent - SAMSUNG-GT-S6888/1.0 SHP/VPP/R5 Dolfin/2.0 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA24581 = {S104614, 37, { &P16590,&P17483,&P17470,&P18092 }};	/*Useragent - LG-B2100 MIC/WAP2.0 MIDP-2.0/CLDC-1.0*/
const Device UA26044 = {S104615, 54, { &P13371,&P17485,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 SonyEricssonC902v/R3EE SEMC/NewsReader/1.0*/
const Device UA28095 = {S104616, 55, { &P12217,&P22839,&P17470,&P18092 }};	/*Useragent - ZTE F860/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA29751 = {S104617, 60, { &P12269,&P17017,&P17470,&P18092 }};	/*Useragent - PICASSO-ZF908/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA31074 = {S104618, 99, { &P14635,&P5843,&P4742,&P18092 }};	/*Useragent - NokiaC3-00/5.0 (07.20) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac3-00/UC Browser7.5.1.77/70/401*/
const Device UA31099 = {S104619, 120, { &P12397,&P17017,&P3976,&P18092 }};	/*Useragent - FarEasTone/1.1/SharpWXT930/SHS001 Mozilla/5.00 (compatible) Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA32488 = {S104620, 104, { &P17166,&P5448,&P3772,&P18092 }};	/*Useragent - ASUS1210/V4.2.4 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 240x320; ASUS1210) Opera 8.65*/
const Device UA32506 = {S104621, 121, { &P17913,&P5448,&P22270,&P18092 }};	/*Useragent - dopod T8288_CMCC/5.2.19965/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA32752 = {S104622, 12, { &P12756,&P17017,&P17470,&P18092 }};	/*Useragent - WellcoM_W930*/
const Device UA32938 = {S104623, 37, { &P12799,&P17483,&P17470,&P18092 }};	/*Useragent - LG-M4410 MIC/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA32977 = {S104624, 39, { &P16674,&P17483,&P3625,&P18092 }};	/*Useragent - LG-KX500/1.0 UP.Browser/6.2.3.8 CTC/1.0*/
const Device UA32984 = {S104625, 128, { &P13617,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) PPC; 240x320; MDA Vario/1.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA32999 = {S104626, 117, { &P12822,&P17605,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-F118/1.0 RTK-E/1.0 DF/1.0 Release/06.22.2007 Browser/NetFront3.4  Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA33136 = {S104627, 37, { &P12862,&P17483,&P17470,&P18092 }};	/*Useragent - LG-U8290 MIC/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA33194 = {S104628, 38, { &P12881,&P17483,&P3866,&P18092 }};	/*Useragent - LG-ME970c MIC/1.1.14 MIDP-2.0/CLDC-1.1*/
const Device UA34045 = {S104629, 119, { &P12139,&P5204,&P4779,&P18092 }};	/*Useragent - Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5310xpressmusic/UC Browser7.8.0.95/70/351*/
const Device UA35150 = {S104630, 72, { &P11812,&P5448,&P22272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) O2 Xda Atom*/
const Device UA37696 = {S104631, 104, { &P12490,&P17485,&P4223,&P18092 }};	/*Useragent - SonyEricssonK700/R7DA Browser/NetFront/3.5 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.2*/
const Device UA41314 = {S104632, 191, { &P13948,&P17605,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E768/1.0 RTK-E/1.0 DF/1.0 Release/11.27.2006 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzU1Njc1MDEwMDg4NDI1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA44019 = {S104633, 135, { &P15944,&P5448,&P4808,&P18092 }};	/*Useragent - Debussy/WAP1.2 Profile/MIDP2.0 Configuration/CLDC1.0 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; Smartphone; 176x220; HPiPAQ510/1.0)*/
const Device UA44073 = {S104634, 92, { &P14838,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-i908E_CMCC/1.0 Release/03.07 Browser/IE6 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA44658 = {S104635, 70, { &P12277,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_Snap Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA44927 = {S104636, 99, { &P13846,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-S7520U/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA46506 = {S104637, 36, { &P15425,&P17484,&P3430,&P18092 }};	/*Useragent - SAMSUNG-GT-B7300C_CMCC/1.0 Opera 9.5*/
const Device UA47612 = {S104638, 138, { &P17281,&P6545,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; SCH-I905 4G Build/HONEYCOMB) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA47639 = {S104639, 74, { &P14459,&P5810,&P3417,&P18092 }};	/*Useragent - BlackBerry7130e/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/105*/
const Device UA48895 = {S104640, 61, { &P12941,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonP908/R101 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA48947 = {S104641, 84, { &P12730,&P17662,&P3976,&P18092 }};	/*Useragent - SoftBank/1.0/840P/PJP10 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49218 = {S104642, 195, { &P12960,&P5448,&P4808,&P18092 }};	/*Useragent - SAMSUNG-GT-I8180C_CMCC/1.0 WindowsCE/5.0 WindowsMobile/6.5 Release/09.04.2009 Browser/IE6 Profile/MIDP-2.1 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA49236 = {S104643, 37, { &P14270,&P17484,&P4970,&P18092 }};	/*Useragent - SCH-U540/1.0 AU-MIC/2.0 (GUI) MMP/2.0*/
const Device UA49454 = {S104644, 122, { &P13030,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_P3701/1.37.621.2 Mozilla/4.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA87566 = {S104645, 71, { &P13030,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_P3702 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA49522 = {S104646, 71, { &P13047,&P17017,&P3601,&P18092 }};	/*Useragent - SUN F188/1.0 ACS-NF/3.3 QTV5.02 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49607 = {S104647, 58, { &P13061,&P5768,&P3625,&P18092 }};	/*Useragent - Nokia6165i/2.0 (AZ100V1800.nep) UP.Browser/6.2.3.8 MMP/2.0*/
const Device UA49616 = {S104648, 113, { &P13063,&P7205,&P4521,&P18092 }};	/*Useragent - Mozilla/5.0 (SCH-W289/W289EC01;U;REX/3.0;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*320;CTC/2.0) POLARIS/6.2.WEB*/
const Device UA49801 = {S104649, 97, { &P13084,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-Mandarina-Duck-MOON/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA49891 = {S104650, 48, { &P13663,&P17017,&P4977,&P18092 }};	/*Useragent - Sanyo-SCP7000CA NetFront/3.1 MMP/2.0 Mozilla/4.0*/
const Device UA50016 = {S104651, 47, { &P13140,&P17017,&P17470,&P18092 }};	/*Useragent - Fly-B400/Profile/MIDP.2.0Configuration/CLDC.1.0*/
const Device UA50186 = {S104652, 59, { &P13191,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E635/E635UVEF3 UP.Browser/6.2.2.6 (GUI) MMP/1.0*/
const Device UA50270 = {S104653, 80, { &P12076,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-A177/A177UCIC3 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA51094 = {S104654, 24, { &P13934,&P17017,&P17470,&P18092 }};	/*Useragent - WELLCOM-W981_WAP_Browser*/
const Device UA51254 = {S104655, 81, { &P12016,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW900i/R5BC Java/SEMC-Java/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA51339 = {S104656, 122, { &P14669,&P5843,&P17436,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX2-00/04.21; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.3.23.12*/
const Device UA51979 = {S104657, 86, { &P13961,&P17662,&P3601,&P18092 }};	/*Useragent - SoftBank/1.0/709SC/SCJ001 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA52777 = {S104658, 56, { &P14599,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-GATW2/00.62 UP.Browser/6.2.3.4.c.1.114 (GUI) MMP/2.0*/
const Device UA53467 = {S104659, 37, { &P13983,&P17017,&P17470,&P18092 }};	/*Useragent - Fly_DS100/WAP2.0Configuration/Browser*/
const Device UA53726 = {S104660, 107, { &P14015,&P17017,&P17470,&P18092 }};	/*Useragent - ASUS-J208/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ASUS-J208/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA53772 = {S104661, 70, { &P14020,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_S730 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA53946 = {S104662, 77, { &P14074,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Sprint Treo850e*/
const Device UA54029 = {S104663, 42, { &P14095,&P17017,&P4970,&P18092 }};	/*Useragent - FLY-SL600/BSI AU.Browser/2.0 QO3C1 MMP/1.0*/
const Device UA54138 = {S104664, 121, { &P14130,&P5890,&P3439,&P18092 }};	/*Useragent - SAMSUNG-GT-I8510H/1.0 SymbianOS/9.3 Series60/3.2 Release/5.30.2008 Browser/OSS3.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA54161 = {S104665, 101, { &P14136,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E700/BSI Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA54219 = {S104666, 33, { &P14155,&P17017,&P17470,&P18092 }};	/*Useragent - SHARP SH7110C / Obigo Browser 2.0*/
const Device UA55092 = {S104667, 75, { &P14284,&P17483,&P17470,&P18092 }};	/*Useragent - LG-C2600 MIC/WAP2.0 MIDP-2.0/CLDC-1.0 LG-C2600 MIC/WAP2.0 MIDP-2.0/CLDC-1.0*/
const Device UA55980 = {S104668, 56, { &P14609,&P5926,&P3625,&P18092 }};	/*Useragent - Nokia6235/1.0 (S190V0200.nep) UP.Browser/6.2.3.2 MMP/2.0*/
const Device UA56026 = {S104669, 62, { &P14618,&P6712,&P3402,&P18092 }};	/*Useragent - Nokia2608/2.0 UP.Browser/6.3.0.7.g.1.100-27Oct08 (GUI) MMP/2.0*/
const Device UA56199 = {S104670, 191, { &P13679,&P17605,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-D988/1.0 RTK-E/1.0 DF/1.0 Release/08.17.2007 Browser/Openwave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzU0NDAxMDIxNTQ2Njg2 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA56331 = {S104671, 81, { &P15551,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonZ558i/R4FA Java/SEMC-Java/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA56339 = {S104672, 35, { &P11880,&P5890,&P4715,&P18092 }};	/*Useragent - NOKIA5320/UC Browser7.4.0.65/28/352*/
const Device UA56595 = {S104673, 71, { &P17923,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) DopodU1000*/
const Device UA57582 = {S104674, 148, { &P15438,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; vi-vn; HTC_Sensation_S710e Build/FRG83D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA59268 = {S104675, 85, { &P16988,&P5448,&P3772,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC) Opera 8.65 [en] Toshiba G910_G920*/
const Device UA60528 = {S104676, 124, { &P12537,&P5208,&P4521,&P18092 }};	/*Useragent - Mozilla/5.0 (SCH-W319/W319DG29;U;REX/3.0;BREW/3.1.5;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*320;CTC/2.0) POLARIS/6.2.WEB*/
const Device UA60663 = {S104677, 112, { &P12563,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-S8300C/1.0 SHP/VPP/R5 NetFront/3.5 NexPlayer/2.9.1 SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA60631 = {S104678, 45, { &P12556,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-S218/1.0 ObigoInternetBrowser/Q03C*/
const Device UA60731 = {S104679, 45, { &P12569,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-383A/1.0 ObigoInternetBrowser/Q03C*/
const Device UA60942 = {S104680, 127, { &P15259,&P6661,&P4453,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 9900; en) AppleWebKit/534.11+ (KHTML, like Gecko) Version/6.1.0.73 Mobile Safari/534.11+*/
const Device UA60970 = {S104681, 68, { &P15552,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V635/08.48.24R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA61073 = {S104682, 41, { &P12623,&P17483,&P17470,&P18092 }};	/*Useragent - LG-U8500-V10 MIC/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA61078 = {S104683, 49, { &P15753,&P17486,&P3487,&P18092 }};	/*Useragent - MOT-VE20/1.0 UP.Browser/7.2.6.1.731 (GUI) MMP/2.0*/
const Device UA61305 = {S104684, 97, { &P12678,&P17484,&P4924,&P18092 }};	/*Useragent - SamsungSGH-ZV30/1.0 SHP/VPP/R5 NetFront/3.2 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA61379 = {S104685, 81, { &P12703,&P5448,&P4578,&P18092 }};	/*Useragent - HTC-2125/1.2 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 176x220)*/
const Device UA61481 = {S104686, 57, { &P12729,&P5926,&P3625,&P18092 }};	/*Useragent - Nokia2855/2.0 (VR100V0600.nep) UP.Browser/6.2.3.8 MMP/2.0*/
const Device UA61520 = {S104687, 23, { &P12741,&P19954,&P3816,&P18092 }};	/*Useragent - SEC-SGHP710/1.0 TSS/2.5*/
const Device UA61585 = {S104688, 84, { &P12748,&P17662,&P3976,&P18092 }};	/*Useragent - SoftBank/1.0/002P/PJP10 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61648 = {S104689, 102, { &P15364,&P24739,&P17470,&P18092 }};	/*Useragent - Karbonn-K400/0.1 NXP/1.0 Release/08.10.2009 Browser/mBrowser4.0 Profile/MIDP-1.0 Configration/CLDC-1.0*/
const Device UA61839 = {S104690, 153, { &P17557,&P17017,&P4222,&P18092 }};	/*Useragent - SharpWXT91/SHS001/1.0 Browser/UP.Browser/7.0.2.1a.f.1.101 (GUI) Profile/MIDP-2.0 Configuration/CLDC-1.1 Ext-J-Profile/JSCL-1.2.2 Ext-V-Profile/VSCL-2.0.0*/
const Device UA63644 = {S104691, 77, { &P14530,&P17017,&P17470,&P18092 }};	/*Useragent - Fly-HT2_i08 ObigoInternetBrowser/QO3C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA65323 = {S104692, 115, { &P12056,&P5204,&P4759,&P18092 }};	/*Useragent - Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7210supernova/UC Browser7.6.1.82/70/351*/
const Device UA66652 = {S104693, 87, { &P13374,&P5448,&P17183,&P18092 }};	/*Useragent - HTC_P3700 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA66786 = {S104694, 103, { &P12473,&P5204,&P4742,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2730c-1/UC Browser7.5.1.77/70/351*/
const Device UA66787 = {S104695, 253, { &P12473,&P5204,&P4742,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (10.45) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2;.NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)/UC Browser7.5.1.77/70/352*/
const Device UA67043 = {S104696, 35, { &P14782,&P5890,&P4715,&P18092 }};	/*Useragent - NOKIAE61i/UC Browser7.4.0.65/28/352*/
const Device UA67215 = {S104697, 106, { &P13308,&P17485,&P4715,&P18092 }};	/*Useragent - SonyEricssonJ105i/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1 sonyericssonj105i/UC Browser7.4.0.65/70/351*/
const Device UA67318 = {S104698, 135, { &P14812,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-tw; Vibo-A600 Build/Froyo) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA67323 = {S104699, 39, { &P14519,&P5890,&P3439,&P18092 }};	/*Useragent - NokiaN97mini/SymbianOS/9.1 Series60/3.0*/
const Device UA67517 = {S104700, 95, { &P14763,&P5776,&P24660,&P18092 }};	/*Useragent - NokiaN72/5.0819.4.0.1 Series60/2.8 Profile/MIDP-2.0 Configuration/CLDC-1.1/UCWEB7.6.0.75/27/401*/
const Device UA67545 = {S104701, 31, { &P13552,&P5129,&P24660,&P18092 }};	/*Useragent - NokiaC6-00/UCWEB7.6.0.75/50/401*/
const Device UA67714 = {S104702, 86, { &P14863,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-P612/2009.08.12 P612_S112_090812/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA67908 = {S104703, 60, { &P14880,&P7048,&P3412,&P18092 }};	/*Useragent - BlackBerry8320/4.3.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA68032 = {S104704, 87, { &P14901,&P17017,&P3402,&P18092 }};	/*Useragent - BenQ-SIEMENS - C31/1.00/WAP2.0/MIDP2.0/CLDC1.0 UP.Browser/6.3.0.4.c.1.102 (GUI) MMP/2.0*/
const Device UA68083 = {S104705, 116, { &P11679,&P5776,&P4759,&P18092 }};	/*Useragent - Nokia6630/1.0 (5.03.08) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.6.1.82/27/351*/
const Device UA68374 = {S104706, 43, { &P20126,&P17485,&P4759,&P18092 }};	/*Useragent - sonyericssonk800i/UC Browser7.6.1.82/69/352*/
const Device UA68537 = {S104707, 115, { &P12056,&P5204,&P4777,&P18092 }};	/*Useragent - Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7210supernova/UC Browser7.7.1.88/70/351*/
const Device UA68572 = {S104708, 29, { &P14748,&P5890,&P24661,&P18092 }};	/*Useragent - NokiaE72/UCWEB7.7.0.81/28/999*/
const Device UA68588 = {S104709, 41, { &P14944,&P17486,&P3625,&P18092 }};	/*Useragent - MOT-C155 UP.Browser/6.2.2.7 (GUI) MMP/1.0*/
const Device UA68638 = {S104710, 11, { &P12834,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHE710*/
const Device UA68673 = {S104711, 29, { &P14954,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHE315-E315UVDH1-TSS1.01*/
const Device UA81615 = {S104712, 29, { &P14954,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHE315-E315UVEC3-TSS1.01*/
const Device UA68941 = {S104713, 36, { &P12236,&P5890,&P4777,&P18092 }};	/*Useragent - NOKIA6700s/UC Browser7.7.1.88/28/444*/
const Device UA68994 = {S104714, 34, { &P14519,&P5129,&P24662,&P18092 }};	/*Useragent - NokiaN97 mini/UCWEB7.8.0.87/50/800*/
const Device UA69061 = {S104715, 7, { &P13782,&P17483,&P17470,&P18092 }};	/*Useragent - lg-t310*/
const Device UA69641 = {S104716, 97, { &P12108,&P5843,&P4759,&P18092 }};	/*Useragent - Nokia7230/5.0 (06.90) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7230/UC Browser7.6.1.82/70/351*/
const Device UA69650 = {S104717, 99, { &P14711,&P5204,&P4759,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (10.50) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-01/UC Browser7.6.1.82/70/351*/
const Device UA69651 = {S104718, 99, { &P14540,&P5843,&P4737,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (07.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-01/UC Browser7.2.2.51/70/351*/
const Device UA70539 = {S104719, 67, { &P16190,&P5448,&P22271,&P18092 }};	/*Useragent - Xda_trion; 240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6)*/
const Device UA71968 = {S104720, 148, { &P13452,&P5096,&P20776,&P18092 }};	/*Useragent - MQQBrowser/25 Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72529 = {S104721, 65, { &P15302,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonWT13i/R3AD013 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA74244 = {S104722, 49, { &P15253,&P7114,&P3625,&P18092 }};	/*Useragent - SCH-A650 UP.Browser/6.2.2.3.d.2.100 (GUI) MMP/2.0*/
const Device UA74997 = {S104723, 65, { &P15875,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonCK15i/R3AC020 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA75108 = {S104724, 160, { &P12444,&P5477,&P20775,&P18092 }};	/*Useragent - MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; U20i Build/2.1.1.A.0.6) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA80130 = {S104725, 85, { &P17673,&P5448,&P4578,&P18092 }};	/*Useragent - AUDIOVOX-SMT5600/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 176x220)*/
const Device UA80131 = {S104726, 89, { &P17673,&P5448,&P4578,&P18092 }};	/*Useragent - AUDIOVOX-SMT5600/1.2 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 176x220)*/
const Device UA80946 = {S104727, 80, { &P16222,&P5890,&P4030,&P18092 }};	/*Useragent - Nokia7650/1.0 SymbianOS/6.1 Series60/0.9 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA81099 = {S104728, 9, { &P11689,&P16000,&P4030,&P18092 }};	/*Useragent - Nokia N70*/
const Device UA81630 = {S104729, 20, { &P17476,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-E356/1.0*/
const Device UA81841 = {S104730, 79, { &P17312,&P5776,&P3756,&P18092 }};	/*Useragent - SEC-SGHD730 SymbianOS/7.0s Series60/2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA81875 = {S104731, 23, { &P17289,&P19954,&P3816,&P18092 }};	/*Useragent - SEC-SGHE620/1.0 TSS/2.5*/
const Device UA83397 = {S104732, 17, { &P14234,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonK300i*/
const Device UA83410 = {S104733, 17, { &P15537,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonK500i*/
const Device UA83476 = {S104734, 17, { &P13587,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonK750c*/
const Device UA83539 = {S104735, 17, { &P13373,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonZ310i*/
const Device UA83938 = {S104736, 39, { &P20223,&P17483,&P3625,&P18092 }};	/*Useragent - LG-G4015 UP.Browser/6.2.2 (GUI) MMP/1.0*/
const Device UA84052 = {S104737, 37, { &P20217,&P17483,&P17470,&P18092 }};	/*Useragent - LG-MG800 MIC/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA84134 = {S104738, 48, { &P12623,&P17483,&P17660,&P18092 }};	/*Useragent - LG-U8500 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA84765 = {S104739, 46, { &P17846,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-CA23 UP.Browser/6.2.0.5.136 (GUI) MMP/2.0*/
const Device UA85300 = {S104740, 51, { &P16493,&P17017,&P17470,&P18092 }};	/*Useragent - PG-1410/R01 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA85578 = {S104741, 28, { &P17230,&P17017,&P17470,&P18092 }};	/*Useragent - portalmmm/2.0 Z650i(c100;TB)*/
const Device UA85663 = {S104742, 27, { &P17298,&P17017,&P17470,&P18092 }};	/*Useragent - portalmmm/2.0 S400i(c10;TB)*/
const Device UA85724 = {S104743, 27, { &P17268,&P17017,&P17470,&P18092 }};	/*Useragent - portalmmm/2.0 S501i(c20;TB)*/
const Device UA85294 = {S104744, 52, { &P16486,&P17017,&P17470,&P18092 }};	/*Useragent - PG-1310V/R01 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA86485 = {S104745, 96, { &P16381,&P5448,&P4578,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_wizard/2.21.3.102/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)*/
const Device UA87028 = {S104746, 77, { &P11696,&P5448,&P22371,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.9) MOT-Q9/01.02.23R*/
const Device UA87300 = {S104747, 148, { &P12025,&P5448,&P4578,&P18092 }};	/*Useragent - SAMSUNG-SGH-i718plus_CMCC/1.0 Release/03.07 Browser/IE6 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)*/
const Device UA87382 = {S104748, 5, { &P16262,&P18588,&P18589,&P18092 }};	/*Useragent - O2Jet*/
const Device UA87700 = {S104749, 31, { &P19921,&P17484,&P17470,&P18092 }};	/*Useragent - SCH-W339/ Embider/5.10.TI (GUI)*/
const Device UA87709 = {S104750, 84, { &P17785,&P17662,&P3976,&P18092 }};	/*Useragent - SoftBank/1.0/810P/PJP10 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87776 = {S104751, 103, { &P17990,&P17017,&P17470,&P18092 }};	/*Useragent - KONKA_M526_CMCC/KCEM526.CMCC.2.01.704 MTK/6225 Release/7.4.2007 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87888 = {S104752, 84, { &P17664,&P17662,&P3976,&P18092 }};	/*Useragent - SoftBank/1.0/822P/PJP10 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88012 = {S104753, 86, { &P17655,&P17662,&P3976,&P18092 }};	/*Useragent - SoftBank/1.0/815SH/SHJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88018 = {S104754, 86, { &P17655,&P17662,&P3976,&P18092 }};	/*Useragent - SoftBank/1.0/814SH/SHJ001 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA88261 = {S104755, 45, { &P19923,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-A645/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA88052 = {S104756, 73, { &P14308,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) Dopod838Pro*/
const Device UA88625 = {S104757, 10, { &P17068,&P22835,&P17470,&P18092 }};	/*Useragent - Spice S820*/
const Device UA88689 = {S104758, 40, { &P16391,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KC560 Teleca/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA88581 = {S104759, 12, { &P17751,&P17017,&P17470,&P18092 }};	/*Useragent - WELLCOM_W398*/
const Device UA88946 = {S104760, 34, { &P20113,&P17484,&P4970,&P18092 }};	/*Useragent - Samsung-SPHA860 AU-MIC/2.0 MMP/2.0*/
const Device UA89071 = {S104761, 142, { &P13199,&P17484,&P3625,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungSGHB520V/B520BUHI1 Browser/Openwave/6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA90354 = {S104762, 102, { &P20315,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-F265L/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA90383 = {S104763, 136, { &P13163,&P5129,&P20804,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.4; U; Series60/5.0; en-GB; Nokia N97 RM-505) AppleWebKit/532.4 (KHTML, like Gecko) Qt/4.6.3 Mobile Safari/532.4*/
const Device UA251556 = {S104764, 123, { &P15364,&P17285,&P24206,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0) AppleWebKit/534.36 (KHTML, like Gecko) Chrome/12.0.742.53 Safari/534.36 QQBrowser/6.6.9632.201*/
const Device UA253828 = {S104765, 61, { &P11713,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonG705/R1BA Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA253899 = {S104766, 65, { &P13344,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW100i/R1CA003 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA253909 = {S104767, 40, { &P12142,&P17485,&P4759,&P18092 }};	/*Useragent - SonyEricssonU5/UC Browser7.6.1.82/50/352*/
const Device UA294826 = {S104768, 78, { &P15364,&P5690,&P18373,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; Opera Mobi/7457; U; en) Presto/2.5.28 Version/10.1*/
const Device UA332115 = {S104769, 126, { &P15767,&P17145,&P22247,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Flipboard/1.5.1*/
const Device UA347104 = {S104770, 138, { &P18467,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-us; Kobo Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA368799 = {S104771, 150, { &P15649,&P5096,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-S5570 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA368881 = {S104772, 142, { &P14225,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Desire HD Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA369071 = {S104773, 95, { &P15575,&P5629,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.3.9.10087); (Linux; U; Android 2.3.4; GT-I9100 Build/GINGERBREAD_MR1);*/
const Device UA370432 = {S104774, 95, { &P15575,&P5629,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.3.8.9816); (Linux; U; Android 2.3.3; GT-I9100M Build/GINGERBREAD_MR1);*/
const Device UA369159 = {S104775, 146, { &P14877,&P5096,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I9000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370408 = {S104776, 149, { &P15404,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; ko-kr; MB861 Build/4.5.91; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA399165 = {S104777, 101, { &P20079,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 719)*/
const Device UA438805 = {S104778, 73, { &P12243,&P5448,&P4578,&P18092 }};	/*Useragent - HTC_Snap_S521 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone)*/
const Device UA438838 = {S104779, 80, { &P11734,&P5222,&P17183,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; HTC_P4550; Windows Phone 6.5)*/
const Device UA459159 = {S104780, 69, { &P18314,&P18110,&P24662,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; MEIZU_MX; 640*960) UCWEB7.8.0.95/139/355*/
const Device UA459197 = {S104781, 37, { &P18323,&P17484,&P23401,&P18092 }};	/*Useragent - MQQBrowser/Mini2.5 (SAMSUNG-GT-S5222)*/
const Device UA459275 = {S104782, 157, { &P15991,&P5629,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA459311 = {S104783, 154, { &P15991,&P5629,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.6; en-au; GT-S6102 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA521193 = {S104784, 34, { &P14519,&P5129,&P24660,&P18092 }};	/*Useragent - NokiaN97 mini/UCWEB7.6.0.75/50/800*/
const Device UA535958 = {S104785, 75, { &P17465,&P18110,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Linux i686; Opera Tablet/86; U; en) Presto/2.7.81 Version/11.00*/
const Device UA616423 = {S104786, 85, { &P17465,&P18110,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (And. 2.2.1; Linux; Opera Tablet/ADR-X; U; en) Presto/2.7.81 Version/11.00*/
const Device UA551773 = {S104787, 31, { &P14033,&P5129,&P24662,&P18092 }};	/*Useragent - NokiaC5-03/UCWEB7.8.0.87/50/800*/
const Device UA587785 = {S104788, 35, { &P14826,&P5843,&P21757,&P18092 }};	/*Useragent - iBrowser/Mini2.4 (NokiaC1-01/04.40)*/
const Device UA590231 = {S104789, 45, { &P15767,&P17146,&P24447,&P18092 }};	/*Useragent - Dolphin HD 3.5 (iPad; iPhone OS 5.0.1; en_US)*/
const Device UA618245 = {S104790, 105, { &P17346,&P17145,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 4_2_1 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]*/
const Device UA618381 = {S104791, 105, { &P17346,&P17146,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod touch; U; CPU iPhone OS 5_0_1 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]*/
const Device UA619803 = {S104792, 82, { &P12280,&P17779,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) [operamini] [UsableNet Lift Mobile]*/
const Device UA621905 = {S104793, 44, { &P15364,&P18575,&P18542,&P18092 }};	/*Useragent - Dolphin/4.1 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA622329 = {S104794, 45, { &P15767,&P17145,&P24449,&P18092 }};	/*Useragent - Dolphin HD 4.1 (iPad; iPhone OS 4.2.1; en_US)*/
const Device UA642957 = {S104795, 157, { &P12788,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; HTC Evo 3D Build/GRJ22; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA648435 = {S104796, 44, { &P15364,&P18575,&P18157,&P18092 }};	/*Useragent - Puffin/3337 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA653883 = {S104797, 126, { &P15767,&P17145,&P22246,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C148 Flipboard/1.7.2*/
const Device UA707408 = {S104798, 30, { &P13163,&P5129,&P24666,&P18092 }};	/*Useragent - NokiaN97/UCWEB8.1.0.104/50/999*/
const Device UA721116 = {S104799, 161, { &P18113,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; DROID RAZR Build/6.5.1-167_DHD-14_M3-8)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA753979 = {S104800, 32, { &P12088,&P5129,&P24668,&P18092 }};	/*Useragent - NokiaX6-00/UCWEB8.3.0.133/50/800*/
const Device UA859187 = {S104801, 142, { &P14877,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-I9000 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA899089 = {S104802, 36, { &P23144,&P17779,&P23078,&P18092 }};	/*Useragent - Instacast/2.0.1 (like iTunes/10.1.2)*/
const Device UA912231 = {S104803, 191, { &P14969,&P5629,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, gb*/
const Device UA925754 = {S104804, 111, { &P18086,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; Microsoft; XDeviceEmulator)*/
const Device UA925763 = {S104805, 100, { &P14634,&P15924,&P4702,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; SAMSUNG; Taylor)*/
const Device UA925856 = {S104806, 97, { &P18036,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; MWP6885)*/
const Device UA925893 = {S104807, 121, { &P15364,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; ZTE; Windows Phone - Internet 7; SFR)*/
const Device UA1896857 = {S104808, 160, { &P15364,&P7133,&P18726,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.3 (KHTML, like Gecko) SecondLife/1.5.2.1185 (Phoenix Viewer Release; Firebird skin) Safari/533.3*/
const Device UA1904776 = {S104809, 36, { &P13847,&P17483,&P17338,&P18092 }};	/*Useragent - lg-lg900g/UC Browser7.9.0.102/69/352*/
const Device UA1908344 = {S104810, 31, { &P14984,&P5129,&P24666,&P18092 }};	/*Useragent - Nokia5250/UCWEB8.1.0.104/50/999*/
const Device UA1942039 = {S104811, 48, { &P12399,&P17485,&P24665,&P18092 }};	/*Useragent - SonyEricssonU8i/UC Browser8.0.4.121/50/352/UCWEB*/
const Device UA1952832 = {S104812, 39, { &P18176,&P5448,&P24630,&P18092 }};	/*Useragent - NOKIA-Lumia 900/UCWEB2.2.0.157/47/32079*/
const Device UA1959578 = {S104813, 136, { &P17346,&P18117,&P18361,&P18092 }};	/*Useragent - Mozilla/5.0 (iPod; CPU iPhone OS 5_1_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9B206 Safari/7534.48.3 Sleipnir/2.0m*/
const Device UA1989146 = {S104814, 168, { &P12616,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; WT19i Build/4.1.B.0.431) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA1990599 = {S104815, 131, { &P17595,&P17297,&P18352,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.53.11 (KHTML, like Gecko) Version/5.1 Safari/7534.51.22 Sleipnir/3.3*/
const Device UA1991141 = {S104816, 191, { &P18280,&P18066,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC One S Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, gb*/
const Device UA2018278 = {S104817, 133, { &P13242,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.2; en-us; A853 Build/GRH78C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2018632 = {S104818, 144, { &P21410,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.0.0; es-es; bq Verne Plus Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2018925 = {S104819, 145, { &P12082,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; X10 TripNMiUI-IRIS Build/NI17) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2019028 = {S104820, 76, { &P13215,&P5477,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.1-update1;Zh_cn;GT-I5700;320*480;)UCWEB7.8.0.95/139/355*/
const Device UA2019615 = {S104821, 70, { &P12110,&P5096,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.2;Zh_cn;HTC Legend;320*480;)UCWEB7.8.0.95/139/355*/
const Device UA2020392 = {S104822, 161, { &P15501,&P5477,&P20775,&P18092 }};	/*Useragent - MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.1-update1; en-in; E16i Build/2.1.1.A.0.53) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA2021227 = {S104823, 134, { &P16044,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Touchpad Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2022090 = {S104824, 58, { &P21988,&P5477,&P4861,&P18092 }};	/*Useragent - Dalvik/1.1.0 (Linux; U; Android 2.1; PD_Novel Build/ERD79)*/
const Device UA2022093 = {S104825, 70, { &P16479,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.4;Zh_cn;SCH-R720;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA2022984 = {S104826, 149, { &P17968,&P5477,&P21558,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en_gb) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/1.9.4 Safari/533.1*/
const Device UA2024500 = {S104827, 131, { &P12482,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.1; en-us; T959 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2024873 = {S104828, 144, { &P16553,&P5096,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; SGH-T959V Build/FROYO)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026849 = {S104829, 173, { &P23744,&P5629,&P15760,&P18092 }};	/*Useragent - 2.3 SAMSUNG-SGH-I807D---Mozilla/5.0 (Linux; U; Android 2.3; xx-xx; SAMSUNG-SGH-I807D Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2026967 = {S104830, 149, { &P14433,&P5096,&P21760,&P18092 }};	/*Useragent - iBrowser/2.8 Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-I9003 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028344 = {S104831, 146, { &P14967,&P5096,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2; en-in; LG-P500 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028407 = {S104832, 141, { &P14161,&P5096,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; VM670 Build/FRG83G)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2028877 = {S104833, 168, { &P21997,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; Spice Mi280 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2029594 = {S104834, 153, { &P21399,&P5629,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; Lenovo P70 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2029993 = {S104835, 146, { &P15575,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android RGUI 2.3.23; en-us; GT-I9100 Build/RGUI MIUI) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2030979 = {S104836, 141, { &P12788,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; PG86100 Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77 Mobile Safari/535.7*/
const Device UA2031255 = {S104837, 159, { &P12657,&P5629,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; HTC EVO 3D X515m Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2162550 = {S104838, 39, { &P15775,&P5448,&P20766,&P18092 }};	/*Useragent - NOKIA-Lumia 800/UCWEB2.0.0.149/47/32079*/
const Device UA2171185 = {S104839, 126, { &P17595,&P18207,&P18532,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8) AppleWebKit/536.25 (KHTML, like Gecko) Version/5.1 Safari/7534.51.22 Sleipnir/3.6*/
const Device UA2208333 = {S104840, 43, { &P17595,&P18575,&P18536,&P18092 }};	/*Useragent - Reeder/2.3 CFNetwork/485.12.7 Darwin/10.4.0*/
const Device UA2213790 = {S104841, 56, { &P14519,&P5890,&P20771,&P18092 }};	/*Useragent - MQQBrowser/2.0 (NokiaN97mini;SymbianOS/9.1 Series60/3.0)*/
const Device UA2241410 = {S104842, 99, { &P18385,&P17979,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 Android (16/4.1.2; 213dpi; 800x1205; asus/google; Nexus 7; grouper; grouper; en_US)*/
const Device UA2241429 = {S104843, 63, { &P18385,&P17979,&P20428,&P18092 }};	/*Useragent - TransperaSDK v3.3.0.262 : google : Nexus 7 : Android OS : 4.1.1*/
const Device UA2241470 = {S104844, 133, { &P18385,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us; Nexus 7 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2308515 = {S104845, 44, { &P11924,&P17485,&P17347,&P18092 }};	/*Useragent - sonyericssons500i/UC Browser8.0.3.107/70/352*/
const Device UA2308699 = {S104846, 83, { &P17976,&P17979,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Android; Linux; Opera Tablet/23731; U; en) Presto/2.9.201 Version/11.50*/
const Device UA2322620 = {S104847, 40, { &P16463,&P17484,&P20767,&P18092 }};	/*Useragent - SAMSUNG-GT-I8350/UCWEB2.1.0.155/47/32079*/
const Device UA2428304 = {S104848, 99, { &P14831,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s7233e) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2458357 = {S104849, 91, { &P11679,&P5776,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia6630) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2458664 = {S104850, 87, { &P12516,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; s8003) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2540523 = {S104851, 17, { &P21000,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-S5222R*/
const Device UA2540524 = {S104852, 34, { &P21000,&P17484,&P21724,&P18092 }};	/*Useragent - OneBrowser/3.0 (SAMSUNG-GT-S5222R)*/
const Device UA2540551 = {S104853, 73, { &P20249,&P17979,&P4459,&P18092 }};	/*Useragent - Evernote Android/AndroidMarket_43x_58 (en_CA); Android/3.1; HTC-P715a/12;*/
const Device UA2540564 = {S104854, 33, { &P20249,&P17979,&P4459,&P18092 }};	/*Useragent - Android/3.1; HTC HTC-P715a; 2.3.0*/
const Device UA2540535 = {S104855, 52, { &P21001,&P17484,&P18122,&P18092 }};	/*Useragent - SAMSUNG-GT-S3802 [operamini] [UsableNet Lift Mobile]*/
const Device UA2540622 = {S104856, 137, { &P21203,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; es-es; SX-SP715A Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7*/
const Device UA2586586 = {S104857, 233, { &P18696,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; BOHP P5 Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 baiduboxapp/042_1.7.3_diordna_008_084/spla_01_6.3.2_5P+PHOB/Xa/544F1608FBFB8335CXA5342AA69%7CX/1*/
const Device UA2586589 = {S104858, 244, { &P18696,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GN205 Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) FlyFlow/2.4 Version/4.0 Mobile Safari/533.1 baidubrowser/042_1.8.4.2_diordna_008_084/EENOIG_01_4.3.2_502NG/Xj/8DDC5302B7B201F86B1C73BXFF%7CX/1*/
const Device UA2588364 = {S104859, 86, { &P13220,&P17485,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; sonyericssonk770i) UCBrowser8.2.1.144/69/444/UCWEB Mobile*/
const Device UA2595954 = {S104860, 155, { &P11793,&P5768,&P24675,&P18092 }};	/*Useragent - Nokia3500c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia3500c) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2598004 = {S104861, 88, { &P15518,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; s5230) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2607475 = {S104862, 64, { &P14632,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;HTC;T8788;7.10;8862)*/
const Device UA2607533 = {S104863, 130, { &P21792,&P6437,&P16179,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-I9200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2607540 = {S104864, 138, { &P21794,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9152 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2607647 = {S104865, 148, { &P13452,&P5096,&P20772,&P18092 }};	/*Useragent - MQQBrowser/21 Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2607985 = {S104866, 181, { &P22041,&P6437,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6810 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, au*/
const Device UA2608203 = {S104867, 157, { &P14905,&P24691,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7100 Build/RS_Supremacy_AOKP v0.2 - IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2608996 = {S104868, 69, { &P12879,&P6437,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android3.2;Zh_cn;GT-P7510;800*1232;)UCWEB7.8.0.95/139/351*/
const Device UA2609422 = {S104869, 143, { &P15985,&P6437,&P21760,&P18092 }};	/*Useragent - iBrowser/2.8 Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2609941 = {S104870, 151, { &P15985,&P6437,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 3.2; zh-tw; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 MobileSafari/534.13*/
const Device UA2610257 = {S104871, 164, { &P12879,&P6437,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7510 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2610504 = {S104872, 145, { &P15919,&P18066,&P21753,&P18092 }};	/*Useragent - iBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2611005 = {S104873, 145, { &P15919,&P18066,&P21753,&P18092 }};	/*Useragent - iBrowser/3.0/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2610788 = {S104874, 137, { &P12879,&P6437,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ru-ru; GT-P7510 Build/HTJ85B)Maxthon AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2612351 = {S104875, 142, { &P17337,&P6546,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2612495 = {S104876, 91, { &P17168,&P6437,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.4.5.12736); (Linux; U; Android 3.2; GT-P7310 Build/HONEYCOMB_MR2);*/
const Device UA2612731 = {S104877, 157, { &P13452,&P5629,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8 Mozilla/5.0 (Linux; U; Android 2.3.3; en-gb; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2613194 = {S104878, 147, { &P13452,&P5096,&P21753,&P18092 }};	/*Useragent - iBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2613602 = {S104879, 143, { &P12889,&P6437,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2613956 = {S104880, 144, { &P15919,&P18066,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2614015 = {S104881, 151, { &P14908,&P5096,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2618829 = {S104882, 151, { &P14908,&P5096,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2615265 = {S104883, 166, { &P22043,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P5113 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2615523 = {S104884, 71, { &P17337,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.4;Zh_cn;GT-P7500;800*1232;)UCWEB7.8.0.95/139/444*/
const Device UA2615805 = {S104885, 142, { &P12889,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; GT-P7300 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2615904 = {S104886, 163, { &P17168,&P6546,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7310 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2615955 = {S104887, 166, { &P14908,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P6210 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2616126 = {S104888, 166, { &P15932,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/352*/
const Device UA2616283 = {S104889, 199, { &P17168,&P22212,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7310 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, se*/
const Device UA2636896 = {S104890, 208, { &P17168,&P22212,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; nl-nl; GT-P7310 Build/JRO03H; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0100, nl*/
const Device UA2616341 = {S104891, 152, { &P15876,&P18066,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2630457 = {S104892, 145, { &P15876,&P18066,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616401 = {S104893, 87, { &P15985,&P18066,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.4.5.12736); (Linux; U; Android 4.0.4; GT-P6200 Build/Unknown);*/
const Device UA2616712 = {S104894, 147, { &P12879,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616800 = {S104895, 66, { &P17337,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.4; GT-P7500 Build/GINGERBREAD)*/
const Device UA2633866 = {S104896, 137, { &P17337,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; de-de; GT-P7500 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2616884 = {S104897, 30, { &P17168,&P17979,&P20428,&P18092 }};	/*Useragent - Android OS/12 samsung_GT-P7310*/
const Device UA2617124 = {S104898, 147, { &P15985,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; id-id; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617198 = {S104899, 45, { &P15876,&P18066,&P22292,&P18092 }};	/*Useragent - TuneIn Radio/6.6 (Android 15; GT-P5100; Java)*/
const Device UA2617284 = {S104900, 29, { &P19852,&P17979,&P21556,&P18092 }};	/*Useragent - Android/1.0 (GT-P7511 HTJ85B)*/
const Device UA2617392 = {S104901, 140, { &P22042,&P18066,&P20791,&P18092 }};	/*Useragent - MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2628152 = {S104902, 152, { &P22042,&P18066,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2618113 = {S104903, 164, { &P22041,&P6437,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6810 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2618197 = {S104904, 147, { &P17323,&P18066,&P20787,&P18092 }};	/*Useragent - MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2618264 = {S104905, 87, { &P17168,&P18066,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 4.0.4; GT-P7310 Build/Unknown);*/
const Device UA2618712 = {S104906, 90, { &P12889,&P17979,&P22068,&P18092 }};	/*Useragent - Instagram 1.0.6 Android (12/3.1; 160dpi; 1280x800; samsung; GT-P7300; GT-P7300; p3; en_GB)*/
const Device UA2619491 = {S104907, 140, { &P17323,&P18066,&P20791,&P18092 }};	/*Useragent - MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2619570 = {S104908, 166, { &P12879,&P6437,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [operamini] [UsableNet Lift Mobile]*/
const Device UA2619772 = {S104909, 131, { &P17337,&P6437,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.2; GT-P7500 Build/unknown) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2619790 = {S104910, 95, { &P13452,&P17979,&P22068,&P18092 }};	/*Useragent - Instagram 1.0.6 Android (8/2.2; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; GT-P1000; en_GB)*/
const Device UA2620627 = {S104911, 99, { &P13452,&P17979,&P22068,&P18092 }};	/*Useragent - Instagram 1.0.6 Android (10/2.3.3; 240dpi; 600x1024; samsung; GT-P1000; GT-P1000; gt-p1000n; en_GB)*/
const Device UA2620026 = {S104912, 53, { &P12889,&P17979,&P22076,&P18092 }};	/*Useragent - WhatsApp/2.7.8604 Android/3.1 Device/samsung-GT-P7300*/
const Device UA2620308 = {S104913, 172, { &P12879,&P6546,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7510 Build/HMJ37; 800*1280) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/352*/
const Device UA2620350 = {S104914, 53, { &P13452,&P17979,&P22076,&P18092 }};	/*Useragent - WhatsApp/2.7.5752 Android/2.2 Device/samsung-GT-P1000*/
const Device UA2620395 = {S104915, 89, { &P13452,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Mail; 1.4.6) (GT-P1000; samsung; GT-P1000; 2.2.1/FROYO)*/
const Device UA2620642 = {S104916, 95, { &P13452,&P17979,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Messenger; 1.8.3) (GT-P1000; samsung; GT-P1000; 4.0.4/IMM76L)*/
const Device UA2620484 = {S104917, 102, { &P22037,&P17979,&P22070,&P18092 }};	/*Useragent - Instagram 3.0.5 Android (15/4.0.4; 160dpi; 1024x552; samsung; GT-P3113; espressowifi; espresso; en_CA)*/
const Device UA2620521 = {S104918, 53, { &P15985,&P17979,&P22076,&P18092 }};	/*Useragent - WhatsApp/2.7.8863 Android/3.2 Device/samsung-GT-P6200*/
const Device UA2620730 = {S104919, 145, { &P22043,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2624286 = {S104920, 147, { &P22043,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2620819 = {S104921, 185, { &P12879,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P7510 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.10/220,1.9.10.220,2008-02-29 09:33, vn*/
const Device UA2621047 = {S104922, 99, { &P15919,&P17979,&P22068,&P18092 }};	/*Useragent - Instagram 1.0.6 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P3100; espressorf; espresso; en_GB)*/
const Device UA2621428 = {S104923, 166, { &P17323,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6800 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2621432 = {S104924, 164, { &P22041,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6810 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P6810, 36, 1.7.1)*/
const Device UA2621510 = {S104925, 183, { &P13452,&P22212,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P1000 Build/JZO54K; CyanogenMod-Sapir) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P1000, 36, 1.7.1)*/
const Device UA2621612 = {S104926, 182, { &P18475,&P6437,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-au; GT-P7320T Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, au*/
const Device UA2621647 = {S104927, 181, { &P17323,&P6437,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, id*/
const Device UA2621672 = {S104928, 30, { &P15985,&P17979,&P20428,&P18092 }};	/*Useragent - Android OS/13 samsung_GT-P6200*/
const Device UA2621950 = {S104929, 90, { &P12879,&P17979,&P22073,&P18092 }};	/*Useragent - Instagram 3.3.1 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7510; GT-P7510; p3; en_US)*/
const Device UA2622001 = {S104930, 44, { &P13452,&P5096,&P22293,&P18092 }};	/*Useragent - TuneIn Radio/6.7 (Android 8; GT-P1000; Java)*/
const Device UA2622105 = {S104931, 150, { &P17337,&P6437,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7500 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2622314 = {S104932, 106, { &P22043,&P17979,&P22073,&P18092 }};	/*Useragent - Instagram 3.3.3 Android (15/4.0.4; 160dpi; 800x1232; samsung; GT-P5113; espresso10wifi; espresso10; en_CA)*/
const Device UA2622532 = {S104933, 61, { &P12889,&P17979,&P4459,&P18092 }};	/*Useragent - Evernote Android/master_90 (ko_KR); Android/3.1; GT-P7300/12;*/
const Device UA2622748 = {S104934, 192, { &P15876,&P18598,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P5100 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.14/271,1.9.14.271,2012-12-20 13:41, +0300, by*/
const Device UA2622807 = {S104935, 183, { &P13452,&P18149,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P1000, 36, 1.7.1)*/
const Device UA2622855 = {S104936, 147, { &P15985,&P18066,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622856 = {S104937, 147, { &P22037,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2622956 = {S104938, 164, { &P22052,&P6437,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7501 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2623070 = {S104939, 36, { &P12889,&P17979,&P16179,&P18092 }};	/*Useragent - Android/3.2; samsung GT-P7300; 2.3.2*/
const Device UA2623189 = {S104940, 166, { &P17337,&P6437,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 [operamini] [UsableNet Lift Mobile]*/
const Device UA2623322 = {S104941, 69, { &P22052,&P6437,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android3.2;Zh_cn;GT-P7501;800*1232;)UCWEB7.8.0.95/139/352*/
const Device UA2623323 = {S104942, 118, { &P22052,&P6437,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android3.2;Zh_cn;GT-P7501;800*1232;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/352*/
const Device UA2623333 = {S104943, 219, { &P15919,&P18066,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fa-fa; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; fa) Presto/2.8.119 Version/11.10*/
const Device UA2623419 = {S104944, 164, { &P14908,&P6437,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6210 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355*/
const Device UA2623642 = {S104945, 162, { &P13452,&P18149,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; GT-P1000 Build/IMM76I; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2623750 = {S104946, 219, { &P15876,&P18066,&P17371,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; fa-fa; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (Android; Opera Mini/5.1.21927/28.3392; U; fa) Presto/2.8.119 Version/11.10*/
const Device UA2623963 = {S104947, 45, { &P17337,&P6437,&P22292,&P18092 }};	/*Useragent - TuneIn Radio/6.6 (Android 13; GT-P7500; Java)*/
const Device UA2624434 = {S104948, 46, { &P17337,&P6437,&P22292,&P18092 }};	/*Useragent - TuneIn Radio/6.6 (Android 13; GT-P7500R; Java)*/
const Device UA2624224 = {S104949, 55, { &P14905,&P17979,&P22077,&P18092 }};	/*Useragent - WhatsApp/2.8.5610 Android/3.0.1 Device/samsung-GT-P7100*/
const Device UA2624922 = {S104950, 174, { &P15932,&P18066,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3110 Build/IMM76D; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444*/
const Device UA2625217 = {S104951, 144, { &P17337,&P6546,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2625284 = {S104952, 175, { &P22042,&P18066,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build/IMM76D; 600*976) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.4.145/139/800*/
const Device UA2625379 = {S104953, 145, { &P17323,&P6437,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8 Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2626853 = {S104954, 42, { &P17337,&P17979,&P3525,&P18092 }};	/*Useragent - Orca-Android/1.5.005 OS/3.1 Model/GT-P7500*/
const Device UA2627365 = {S104955, 70, { &P15919,&P18598,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.2.1;Zh_cn;GT-P3100;600*976;)UCWEB7.8.0.95/139/444*/
const Device UA2628157 = {S104956, 151, { &P15985,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6200 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2628462 = {S104957, 151, { &P22042,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2629031 = {S104958, 169, { &P17337,&P22211,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-gb; GT-P7500 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2637978 = {S104959, 168, { &P17337,&P22211,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-ca; GT-P7500 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2629090 = {S104960, 149, { &P13452,&P24694,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-P1000 Build/MIUI) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2629435 = {S104961, 152, { &P13452,&P24694,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-P1000 Build/MIUI-XJ) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2629484 = {S104962, 55, { &P13452,&P5629,&P22080,&P18092 }};	/*Useragent - WhatsApp/2.9.1587 Android/2.3.3 Device/samsung-GT-P1000*/
const Device UA2629504 = {S104963, 144, { &P17337,&P6546,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8 Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2629619 = {S104964, 149, { &P12879,&P6437,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; es-es; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.3.3000*/
const Device UA2629678 = {S104965, 152, { &P15932,&P18066,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2629756 = {S104966, 156, { &P15932,&P18066,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P3110 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2629790 = {S104967, 137, { &P12889,&P6437,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ru-ru; GT-P7300 Build/HTJ85B)Maxthon AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2629920 = {S104968, 90, { &P17168,&P17979,&P22074,&P18092 }};	/*Useragent - Instagram 3.4.2 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7310; GT-P7310; p3; en_US)*/
const Device UA2630029 = {S104969, 79, { &P14908,&P17979,&P4459,&P18092 }};	/*Useragent - Evernote Android/AndroidMarket_45x_243.14540 (en_US); Android/3.2; GT-P6210/13;*/
const Device UA2630138 = {S104970, 149, { &P17337,&P6546,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 UCBrowser/2.2.2.259*/
const Device UA2631830 = {S104971, 156, { &P17337,&P6546,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13 UCBrowser/2.2.2.259*/
const Device UA2630149 = {S104972, 168, { &P15932,&P18111,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; sk-sk; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]*/
const Device UA2630241 = {S104973, 149, { &P15985,&P6437,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2633760 = {S104974, 150, { &P15985,&P6437,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200L Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2630405 = {S104975, 71, { &P15876,&P18111,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.1.1;Zh_cn;GT-P5100;800*1232;)UCWEB7.8.0.95/139/351*/
const Device UA2630406 = {S104976, 120, { &P15876,&P18111,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android4.1.1;Zh_cn;GT-P5100;800*1232;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/351*/
const Device UA2630469 = {S104977, 151, { &P15882,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5110 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2633847 = {S104978, 151, { &P15882,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2630606 = {S104979, 192, { &P15985,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-P6200 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0100, pl*/
const Device UA2631207 = {S104980, 175, { &P15985,&P17979,&P3553,&P18092 }};	/*Useragent - Android_CellLocation_2.0.X_GgPoOkKfHjObFgFeIpBlDcAgCfKpDkAaInDiObLoHcLeKdLfBcDpIjFcIbNgHaBfNiCoEeIdHcGmFgHjDdOeNbMiLmNiAkKl@GO%E5%A4%A9%E6%B0%94EX_3.52;X;X;samsung;GT-P6200;13*/
const Device UA2631222 = {S104981, 71, { &P13452,&P18111,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.1.1;Zh_cn;GT-P1000;600*1024;)UCWEB7.8.0.95/139/444*/
const Device UA2631525 = {S104982, 58, { &P22037,&P17979,&P22279,&P18092 }};	/*Useragent - theScore/3.0.3 Android-OS/4.1.1 (Android; Phone, GT-P3113)*/
const Device UA2631620 = {S104983, 190, { &P20149,&P6437,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-P6211 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0100, de*/
const Device UA2631954 = {S104984, 138, { &P15919,&P17979,&P3508,&P18092 }};	/*Useragent - Orca-Android/2.2.4-release OS/4.0.4 Model/GT-P3100 VersionCode/97447 Product/Messenger Carrier/MY_MAXIS Manufacturer/samsung Brand/samsung*/
const Device UA2632015 = {S104985, 149, { &P14908,&P6437,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P6210 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.3.3000*/
const Device UA2632280 = {S104986, 161, { &P13452,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; zh-CN; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2633328 = {S104987, 161, { &P13452,&P5629,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; zh-CN; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2632312 = {S104988, 163, { &P12879,&P6545,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; GT-P7510 Build/HRI83) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7510, 39, 1.8.2)*/
const Device UA2632314 = {S104989, 149, { &P17337,&P6437,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fa-fa; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.3.3000*/
const Device UA2632744 = {S104990, 116, { &P15932,&P17979,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.0001.8177(Android;Android (GT-P3110-espressowifi-user-4.1.1);en-GB;kml:2.2;client:Free;type:default)*/
const Device UA2633217 = {S104991, 169, { &P15932,&P22211,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3110 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2634969 = {S104992, 168, { &P15932,&P22211,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3110 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2633227 = {S104993, 163, { &P13452,&P22212,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P1000 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2633313 = {S104994, 154, { &P12879,&P6437,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-CN; GT-P7510 Build/HTJ85B) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.1.258 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2633348 = {S104995, 130, { &P22041,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P6810 Build/JRO03R) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.28 Safari/537.31*/
const Device UA2633552 = {S104996, 153, { &P17337,&P6546,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-CN; GT-P7500 Build/HMJ37) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.1.258 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2633575 = {S104997, 152, { &P15882,&P18066,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2634214 = {S104998, 147, { &P22043,&P18111,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2634225 = {S104999, 163, { &P12879,&P6546,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7510 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.0.181/145/352*/
const Device UA2634285 = {S105000, 156, { &P15882,&P18066,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P5110 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.1.258 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2634300 = {S105001, 154, { &P18478,&P6437,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-CN; GT-P6201 Build/HTJ85B) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2634651 = {S105002, 150, { &P15919,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P3100 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2634781 = {S105003, 150, { &P14908,&P5096,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2637799 = {S105004, 150, { &P14908,&P5096,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2634883 = {S105005, 56, { &P14908,&P17979,&P22280,&P18092 }};	/*Useragent - theScore/3.2.1 Android-OS/3.2 (Android; Phone, GT-P6210)*/
const Device UA2637691 = {S105006, 58, { &P14908,&P17979,&P22280,&P18092 }};	/*Useragent - theScore/3.2.2 Android-OS/2.2.1 (Android; Phone, GT-P1010)*/
const Device UA2634925 = {S105007, 174, { &P14908,&P5096,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2.1; en-us; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 [UsableNet Lift Mobile]*/
const Device UA2635198 = {S105008, 190, { &P22041,&P6437,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; fr-fr; GT-P6810 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0100, fr*/
const Device UA2635249 = {S105009, 147, { &P15876,&P18111,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.1; id-id; GT-P5100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2635293 = {S105010, 166, { &P22037,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; GT-P3113 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2635323 = {S105011, 134, { &P15876,&P17979,&P3508,&P18092 }};	/*Useragent - Orca-Android/2.2.5-release OS/4.1.1 Model/GT-P5100 VersionCode/X Product/Messenger Carrier/MY_MAXIS Manufacturer/samsung Brand/samsung*/
const Device UA2635403 = {S105012, 136, { &P22037,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-gb; GT-P3113 Build/ERD62) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2635761 = {S105013, 147, { &P15919,&P18066,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2637366 = {S105014, 147, { &P15919,&P18066,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2636373 = {S105015, 225, { &P15932,&P18149,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3110 Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.27/383,1.9.27.383,2013-03-25 21:34, +0800, china, cn*/
const Device UA2636906 = {S105016, 162, { &P22043,&P18111,&P24406,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 OverDrive Media Console/2.6.4*/
const Device UA2636919 = {S105017, 151, { &P17168,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7310 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2637020 = {S105018, 147, { &P15932,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; sv-se; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2637808 = {S105019, 147, { &P15932,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2637081 = {S105020, 149, { &P12879,&P6437,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7510 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2637251 = {S105021, 166, { &P13452,&P18598,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; GT-P1000 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/444*/
const Device UA2637730 = {S105022, 170, { &P15985,&P18066,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-P6200 Build/IMM76D) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2637805 = {S105023, 168, { &P19852,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P7511 Build/JDQ39; CyanogenMod-Ganbarou V3.04 GT-P7511) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2637869 = {S105024, 169, { &P19852,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; de-at; GT-P7511 Build/JOP40G; CyanogenMod-Ganbarou V3.02 GT-P7511) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2637977 = {S105025, 151, { &P22043,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-ca; GT-P5113 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2651117 = {S105026, 117, { &P16024,&P17483,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; lg-t310i) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2658162 = {S105027, 98, { &P14404,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s5260) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2658210 = {S105028, 153, { &P11699,&P5768,&P24675,&P18092 }};	/*Useragent - Nokia5200/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia5200) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2702291 = {S105029, 88, { &P12516,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; s8003) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2702865 = {S105030, 92, { &P12165,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5230w) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2719661 = {S105031, 141, { &P22655,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C5503 Build/10.3.A.0.175) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.169 Mobile Safari/537.22*/
const Device UA2719734 = {S105032, 139, { &P22656,&P18111,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SOL22 Build/10.2.F.3.15) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2751571 = {S105033, 99, { &P14182,&P17484,&P24677,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5333) U2/1.0.0 UCBrowser/9.0.0.260 U2/1.0.0 Mobile*/
const Device UA2763086 = {S105034, 90, { &P13310,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-gt350) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2763201 = {S105035, 91, { &P14413,&P17486,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; mot-ex115) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2792088 = {S105036, 100, { &P12849,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s7230e) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2834458 = {S105037, 76, { &P12856,&P17483,&P24670,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-in; lg-t300) UCBrowser8.4.0.159/69/444/UCWEB Mobile*/
const Device UA2844598 = {S105038, 98, { &P12369,&P17484,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; samsung-gt-s7070) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2851226 = {S105039, 88, { &P23411,&P17017,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; vi; q-mobile) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2951413 = {S105040, 98, { &P14768,&P22839,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; zte-p80aiii-f160) U2/1.0.0 UCBrowser/8.7.0.218 U2/1.0.0 Mobile*/
const Device UA2962976 = {S105041, 99, { &P14972,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s3370) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA2977058 = {S105042, 86, { &P24081,&P18110,&P17365,&P18092 }};	/*Useragent - Opera/9.80 (Linux armv6l; U; en) Presto/2.8.115 Version/11.10 AQUOS-AS/1.0 LC-52LE835X*/
const Device UA3006104 = {S105043, 88, { &P12516,&P17017,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; s8003) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3010974 = {S105044, 92, { &P12165,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5230w) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3015150 = {S105045, 99, { &P16366,&P17484,&P24678,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8600) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3035661 = {S105046, 100, { &P14851,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s5753e) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3039570 = {S105047, 223, { &P24237,&P18110,&P24302,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 50LA6210-TD; 03.20.09; 0xX;); LG NetCast.TV-2013 /03.20.09 (LG, 50LA6210-TD, wireless)*/
const Device UA3039574 = {S105048, 223, { &P24243,&P18110,&P24302,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 55LA6210-TD; 03.20.15; 0xX;); LG NetCast.TV-2013 /03.20.15 (LG, 55LA6210-TD, wireless)*/
const Device UA3039578 = {S105049, 223, { &P24244,&P18110,&P24302,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 55LA8600-TA; 03.05.56; 0xX;); LG NetCast.TV-2013 /03.05.56 (LG, 55LA8600-TA, wireless)*/
const Device UA3039604 = {S105050, 202, { &P24306,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 50PM4710-TD; 04.00.09; 0xX;); LG NetCast.TV-2012 0*/
const Device UA3039606 = {S105051, 200, { &P24284,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 55LM660S-ZA; 03.03.31; 0xX;); LG NetCast.TV-2012*/
const Device UA3039607 = {S105052, 184, { &P24310,&P18110,&P24300,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; U; Linux 35230; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.1.4(+3D+SCREEN+TUNER; LGE; 47LW5700-TA; 04.01.19; 0xX;); LG NetCast.TV-2011*/
const Device UA3039612 = {S105053, 190, { &P24220,&P24094,&P24299,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; U; Linux mips; en) AppleWebKit/531.2+ (KHTML, like Gecko, Safari/531.2+); LG Browser/4.0.9(+mouse+3D+SCREEN+TUNER; LGE;42LE7500-ZA; 03.05.04; 0xX;); LG NetCast.TV-2011*/
const Device UA3039619 = {S105054, 202, { &P24318,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LM640S-ZA; 04.01.73; 0xX;); LG NetCast.TV-2012 0*/
const Device UA3039620 = {S105055, 200, { &P24320,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 55LM620S-ZE; 04.20.23; 0xX;); LG NetCast.TV-2012*/
const Device UA3039622 = {S105056, 200, { &P24322,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LM6600-TA; 04.10.09; 0xX;); LG NetCast.TV-2012*/
const Device UA3039630 = {S105057, 200, { &P24295,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LM7600-DA; 04.00.15; 0xX;); LG NetCast.TV-2012*/
const Device UA3039632 = {S105058, 200, { &P24356,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LM6410-TB; 04.20.11; 0xX;); LG NetCast.TV-2012*/
const Device UA3039644 = {S105059, 200, { &P24319,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LM640S-ZA; 04.40.23; 0xX;); LG NetCast.TV-2012*/
const Device UA3039645 = {S105060, 200, { &P24362,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 84LM9600-TA; 04.20.74; 0xX;); LG NetCast.TV-2012*/
const Device UA3039657 = {S105061, 197, { &P24287,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+SCREEN+TUNER; LGE; 32LS570T-ZB; 04.10.23; 0xX;); LG NetCast.TV-2012*/
const Device UA3039769 = {S105062, 160, { &P24245,&P24094,&P17366,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; Opera TV Store/4234; U; es-ES) Presto/2.9.167 Version/11.50 Model/Sony-KDL-46HX755 SonyCEBrowser/1.0 (KDL-46HX755; CTV/PKG1.306BRB; CHL)*/
const Device UA3040307 = {S105063, 181, { &P21065,&P24348,&P18366,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Fedora/16 (sTgbfDapDmqYrcInk12ne+0od4IFAmY5UFkEXIq+Vlg=) Chromium/21.0.1180.81 Chrome/21.0.1180.81 Safari/535.2*/
const Device UA3040574 = {S105064, 118, { &P17595,&P18120,&P24153,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X) AppleWebKit/907.0 (KHTML, like Gecko) Chrome/33.0.1.0.8321.0.15.0 Safari/907.0*/
const Device UA3058534 = {S105065, 99, { &P16366,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s8600) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3058961 = {S105066, 92, { &P12165,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5230w) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3068907 = {S105067, 136, { &P20705,&P21637,&P24394,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.3; HTC One Build/JWR66Y.H1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1616.0 Mobile Safari/537.36*/
const Device UA3068929 = {S105068, 135, { &P20705,&P18111,&P24394,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; HTC One Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1612.0 Mobile Safari/537.36*/
const Device UA3068937 = {S105069, 135, { &P21648,&P18598,&P24394,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9505 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1612.0 Mobile Safari/537.36*/
const Device UA3069040 = {S105070, 169, { &P15364,&P24412,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.32 (KHTML, like Gecko) Chromium/25.0.1349.2 Chrome/25.0.1349.2 Safari/537.32 Ubuntu/13.10 (3.6.1-2ubuntu3) Epiphany/3.6.1*/
const Device UA3069041 = {S105071, 174, { &P15364,&P24412,&P19811,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.32 (KHTML, like Gecko) Chromium/25.0.1349.2 Chrome/25.0.1349.2 Safari/537.32 Ubuntu/13.10 (3.9.2-0ubuntu1~saucy1) Epiphany/3.9.2*/
const Device UA3080332 = {S105072, 91, { &P12004,&P5890,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokian95) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3084047 = {S105073, 160, { &P17625,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; ST25i Build/6.1.1.B.1.89) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084051 = {S105074, 160, { &P21141,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; Lenovo P770 Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084067 = {S105075, 159, { &P15364,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; MOBHIG0080 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084117 = {S105076, 145, { &P15364,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Q80 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084072 = {S105077, 172, { &P22962,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; ALCATEL ONE TOUCH 6030A Build/JRO03C) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084076 = {S105078, 156, { &P20013,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; LG-F180 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084096 = {S105079, 162, { &P15991,&P5629,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; GT-S6102 Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3097334 = {S105080, 58, { &P15364,&P18110,&P24522,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686; rv:6.0) Gecko/X Firefox/29.0*/
const Device UA3097336 = {S105081, 85, { &P15364,&P18110,&P24522,&P18092 }};	/*Useragent - Mozilla/5.066 (X11; I; Linux x86-64+Win64; cs; rv:1.9.1.14) Gecko/X Firefox/29.0.0.99*/
const Device UA3097337 = {S105082, 83, { &P18141,&P24527,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 3.00) AppleWebKit/536.26 (KHTML, like Gecko) Silk/3.2*/
const Device UA3108442 = {S105083, 92, { &P11766,&P5776,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3230) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3116553 = {S105084, 136, { &P18471,&P18111,&P24588,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT907 Build/9.8.1Q-78) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/32.0.1651.2 Mobile Safari/537.36*/
const Device UA3116558 = {S105085, 145, { &P12280,&P22882,&P24462,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Coast/1.1.0.00000 Mobile/11A4449d Safari/7534.48.3*/
const Device UA3116559 = {S105086, 143, { &P12280,&P22882,&P24462,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Coast/1.1.0.64172 Mobile/11A465 Safari/7534.48.3*/
const Device UA3125258 = {S105087, 93, { &P12061,&P5890,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae65-1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3126328 = {S105088, 99, { &P14010,&P17484,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; SAMSUNG-GT-S5263) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3126734 = {S105089, 92, { &P11945,&P17484,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; GT-S5233S) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA215 = {S105090, 97, { &P17475,&P5768,&P4742,&P18092 }};	/*Useragent - Nokia6300/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6300/UC Browser7.5.1.77/69/401*/
const Device UA378 = {S105091, 89, { &P11812,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) O2 Xda Atom*/
const Device UA480 = {S105092, 103, { &P14862,&P5204,&P4722,&P18092 }};	/*Useragent - Nokia2700c-2/2.0 (07.15) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2700c-2/UC Browser7.0.0.41/70/352*/
const Device UA765 = {S105093, 125, { &P14708,&P5204,&P4617,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40/3.0FP1; Nokia5130c-2/07.96; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/0.8.3*/
const Device UA1284 = {S105094, 99, { &P16074,&P5768,&P4715,&P18092 }};	/*Useragent - Nokia3110c/2.0 (07.21) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia3110c/UC Browser7.4.0.65/70/351*/
const Device UA2892 = {S105095, 152, { &P16108,&P17486,&P3430,&P18092 }};	/*Useragent - MOT-A3000_CMCC/1.0 Release/09.09.2008 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/WM6.1 Opera/9.5 (Microsoft Windows; PPC; Opera Mobi/16747; U; en)*/
const Device UA4360 = {S105096, 113, { &P14682,&P5843,&P4759,&P18092 }};	/*Useragent - Nokia6303iclassic/5.0 (07.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303iclassic/UC Browser7.6.1.82/70/351*/
const Device UA4766 = {S105097, 97, { &P12101,&P5026,&P4948,&P18092 }};	/*Useragent - BlackBerry8703e/4.5.0.148 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/104 UP.Browser/5.0.3.3*/
const Device UA4811 = {S105098, 86, { &P12192,&P5448,&P22270,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA6575 = {S105099, 119, { &P12139,&P5204,&P4715,&P18092 }};	/*Useragent - Nokia5310XpressMusic/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia5310xpressmusic/UC Browser7.4.0.65/70/351*/
const Device UA7729 = {S105100, 154, { &P13406,&P17605,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-CC03/1.0 RTK-E/1.0 SYSOL/1.0 Release/09.01.2007 Browser/OpenWave6.2.3.3.c.1.101/*MzU5MDk5MDEwNjgyNzU0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA7792 = {S105101, 73, { &P15364,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-SLVR(RED)/08.C4.35R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA8278 = {S105102, 143, { &P14784,&P17979,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android RCMix_v2.0; en-gb; HTC_HERO Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA8545 = {S105103, 191, { &P13454,&P17605,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-F258/1.0 RTK-E/1.0 DF/1.0 Release/07.04.2007 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzUyNTkxMDIwODE1MzMw UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA9286 = {S105104, 16, { &P14864,&P17484,&P17606,&P18092 }};	/*Useragent - samsung-gt-s8500*/
const Device UA9990 = {S105105, 90, { &P13459,&P16000,&P3772,&P18092 }};	/*Useragent - SonyEricssonW958c/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 306) Opera 8.60 [en]*/
const Device UA11234 = {S105106, 98, { &P13563,&P17484,&P4924,&P18092 }};	/*Useragent - SAMSUNG-SGH-E878/NetFront 3.2/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzU5MTU2MDA4MDQwMzY5*/
const Device UA12179 = {S105107, 111, { &P14557,&P5843,&P4742,&P18092 }};	/*Useragent - Nokia6303classic/2.0 (09.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303classic/UC Browser7.5.1.77/70/351*/
const Device UA12181 = {S105108, 111, { &P14557,&P5843,&P4715,&P18092 }};	/*Useragent - Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303classic/UC Browser7.4.0.65/70/351*/
const Device UA12706 = {S105109, 74, { &P16392,&P5448,&P22270,&P18092 }};	/*Useragent - HTC-8900/1.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA13066 = {S105110, 192, { &P13761,&P17605,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-D908i/1.0 RTK-E/1.0 DF/1.0 Release/11.27.2006 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzU1NjU0MDEwMTkyNDEy UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA13674 = {S105111, 67, { &P11763,&P5338,&P4836,&P18092 }};	/*Useragent - MOT-K3/99.41.04R MIB/BER2.2 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA13795 = {S105112, 103, { &P12473,&P5204,&P4715,&P18092 }};	/*Useragent - Nokia2730c-1/2.0 (09.41) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2730c-1/UC Browser7.4.0.65/70/351*/
const Device UA14361 = {S105113, 7, { &P14545,&P17483,&P17470,&P18092 }};	/*Useragent - lg-x330*/
const Device UA15369 = {S105114, 73, { &P11812,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) O2 Xda Atom*/
const Device UA15371 = {S105115, 76, { &P15998,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) HP iPAQ rw6828*/
const Device UA16192 = {S105116, 94, { &P13545,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) O2 Xda Atom Exec*/
const Device UA16250 = {S105117, 35, { &P11868,&P5768,&P4742,&P18092 }};	/*Useragent - nokia6280/UC Browser7.5.1.77/69/401*/
const Device UA16325 = {S105118, 90, { &P16225,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) 480x800 SAMSUNG SCH-i920 PPC*/
const Device UA16366 = {S105119, 51, { &P15210,&P17484,&P3430,&P18092 }};	/*Useragent - SAMSUNG_GT_i8000 Opera/9.50 (Windows NT 5.1; U; en)*/
const Device UA16900 = {S105120, 99, { &P14540,&P5843,&P4715,&P18092 }};	/*Useragent - NokiaX2-01/5.0 (07.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax2-01/UC Browser7.4.0.65/70/351*/
const Device UA16911 = {S105121, 99, { &P14711,&P5204,&P4737,&P18092 }};	/*Useragent - NokiaC2-01/5.0 (10.50) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac2-01/UC Browser7.2.2.51/70/352*/
const Device UA17232 = {S105122, 86, { &P11908,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-i908/1.0 Release/03.07 Browser/IE6 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA17371 = {S105123, 52, { &P11917,&P17017,&P17470,&P18092 }};	/*Useragent - Q-Mobile-F220/Profile/MIDP.2.0Configuration/CLDC.1.0*/
const Device UA18514 = {S105124, 103, { &P14089,&P17484,&P4924,&P18092 }};	/*Useragent - SAMSUNG-SGH-E578-CMCC/NetFront 3.2/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzUyOTk3MDExOTY1MDEw*/
const Device UA18746 = {S105125, 24, { &P11928,&P17017,&P17470,&P18092 }};	/*Useragent - WELLCOM-W388_WAP_Browser*/
const Device UA19349 = {S105126, 191, { &P14163,&P17605,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-F278/1.0 RTK-E/1.0 DF/1.0 Release/07.04.2007 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzU0NDIwMDIwMTYxNTk0 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA19950 = {S105127, 77, { &P15010,&P17484,&P4841,&P18092 }};	/*Useragent - SCH-W239/1.0 POLARIS/5.30.WAP CTC/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA20234 = {S105128, 79, { &P15050,&P5448,&P22270,&P18092 }};	/*Useragent - MOT-Q9W/01.11.21I Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA20250 = {S105129, 83, { &P15055,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW888c/R8BA Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA20442 = {S105130, 111, { &P15092,&P17483,&P3534,&P18092 }};	/*Useragent - LG/KF690/v10a Browser/Obigo-Q05A/3.12 MMS/LG-MMS-V1.0/1.2 Java/ASVM/1.1 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA20499 = {S105131, 115, { &P11902,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E898/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0 Untrusted/1.0*/
const Device UA20630 = {S105132, 58, { &P12384,&P5768,&P3625,&P18092 }};	/*Useragent - Nokia2865i/2.0 (FL190V0200.nep) UP.Browser/6.2.3.8 MMP/2.0*/
const Device UA20986 = {S105133, 77, { &P11966,&P5448,&P4578,&P18092 }};	/*Useragent - HTC-S411-Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 240x320)*/
const Device UA20987 = {S105134, 78, { &P17918,&P5448,&P4578,&P18092 }};	/*Useragent - DopodS301/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; Smartphone; 240x320)*/
const Device UA21010 = {S105135, 115, { &P17773,&P5448,&P22270,&P18092 }};	/*Useragent - dopod S700/5.2.19538/WAP2.0 Profile/MIDP2.0 Configuration/CLDC1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA21142 = {S105136, 84, { &P17518,&P17662,&P3976,&P18092 }};	/*Useragent - SoftBank/1.0/930P/PJP22 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21370 = {S105137, 53, { &P15267,&P17017,&P4901,&P18092 }};	/*Useragent - Panasonic-A210/1.0 UP.Browser/6.1.0.7.8 (GUI) MMP/1.0*/
const Device UA21750 = {S105138, 124, { &P15337,&P17017,&P17470,&P18092 }};	/*Useragent - FLY-SX390/2.0 (03.15) Profile/MIDP-2.0 Configuration/CLDC-1.1, FLY-SX390/2.0 (03.15) Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA21764 = {S105139, 35, { &P12000,&P5768,&P4737,&P18092 }};	/*Useragent - nokia6288/UC Browser7.2.2.51/69/401*/
const Device UA21950 = {S105140, 145, { &P12025,&P5448,&P17173,&P18092 }};	/*Useragent - SAMSUNG-SGH-i718plus_CMCC/1.0 Release/03.07 Browser/IE6 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA22078 = {S105141, 38, { &P15380,&P17483,&P3866,&P18092 }};	/*Useragent - LG-ME770d MIC/1.1.14 MIDP-2.0/CLDC-1.1*/
const Device UA22227 = {S105142, 99, { &P14219,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-C5510U/1.0 SHP/VPP/R5 NetFront/3.5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA22245 = {S105143, 79, { &P15411,&P17483,&P3625,&P18092 }};	/*Useragent - LG-F1200 UP.Browser/6.2.3 (GUI) MMP/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA22332 = {S105144, 46, { &P15427,&P17662,&P3601,&P18092 }};	/*Useragent - SoftBank/1.0/830SC/SCJ001 Browser/NetFront/3.3*/
const Device UA22387 = {S105145, 53, { &P15449,&P17484,&P4901,&P18092 }};	/*Useragent - SAMSUNG-SGH-X600A/K3 UP.Browser/6.1.0.6 (GUI) MMP/1.0*/
const Device UA22600 = {S105146, 30, { &P15493,&P7336,&P17470,&P18092 }};	/*Useragent - Philips292 / Obigo Browser 2.0*/
const Device UA22629 = {S105147, 71, { &P15505,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) DopodD818c*/
const Device UA22642 = {S105148, 79, { &P17176,&P17017,&P4893,&P18092 }};	/*Useragent - Bird.M29/LV,A01/WAP2.0/MIDP-2.0/CLDC-1.1 Browser/UP.Browser/7.1.0.f.1.160 (GUI)*/
const Device UA22923 = {S105149, 37, { &P15529,&P17483,&P17470,&P18092 }};	/*Useragent - LG-C3400 MIC/WAP2.0 MIDP-2.0/CLDC-1.0*/
const Device UA23016 = {S105150, 79, { &P14463,&P17485,&P3680,&P18092 }};	/*Useragent - SonyEricssonT610/R2L SEMC-Browser/4.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23172 = {S105151, 103, { &P15569,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonZ710c/R1EF Release/Sep-08-2006 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA23191 = {S105152, 85, { &P15573,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-C717/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA23274 = {S105153, 105, { &P13290,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonC702a/R3EA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.3.3*/
const Device UA23353 = {S105154, 23, { &P15597,&P19954,&P3816,&P18092 }};	/*Useragent - SEC-SGHD347/1.0 TSS/2.5*/
const Device UA23901 = {S105155, 48, { &P15637,&P7472,&P4893,&P18092 }};	/*Useragent - Alcatel-OT-C552/1.0 UP.Browser/7.1 (GUI) MMP/2.0*/
const Device UA24216 = {S105156, 146, { &P15670,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.2; U; Series60/3.1 LG KS10/v10A; Profile/MIDP-2.0 Configuration/CLDC-1.1 ) AppleWebKit/413 (KHTML, like Gecko) Safari/413*/
const Device UA24338 = {S105157, 87, { &P12097,&P5448,&P4808,&P18092 }};	/*Useragent - HTC_P4350 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA24354 = {S105158, 94, { &P17124,&P19944,&P3625,&P18092 }};	/*Useragent - SHARP-TQ-GX20/1.0 Profile/MIDP-1.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.2.107 (GUI) MMP/1.0*/
const Device UA24353 = {S105159, 93, { &P12101,&P5810,&P4948,&P18092 }};	/*Useragent - BlackBerry8703e/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/126 UP.Browser/5.0.3.3*/
const Device UA24609 = {S105160, 77, { &P15719,&P19948,&P17470,&P18092 }};	/*Useragent - PHILIPS9A9W ObigoInternetBrowser/QO3C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA25979 = {S105161, 191, { &P12145,&P17605,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-J708/1.0 RTK-E/1.0 DF/1.0 Release/02.05.2008 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzUzMzg1MDIwNDgzMzMz UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA26452 = {S105162, 189, { &P13596,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/5.0 (dopod-T5399/1.21.706.12;U;Windows Mobile/6.5;Profile/MIDP-2.0 Configuration/CLDC-1.1;480*640;CTC/2.0) MSIE/6.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA26884 = {S105163, 9, { &P13602,&P17484,&P17470,&P18092 }};	/*Useragent - gt-s5233w*/
const Device UA26965 = {S105164, 70, { &P13656,&P17017,&P17470,&P18092 }};	/*Useragent - W891 ObigoInternetBrowser/QO3C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA27474 = {S105165, 105, { &P11679,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Series60/3.0 Nokia6630/4.06.0 Profile/MIDP-2.0 Configuration/CLDC-1.1)*/
const Device UA27715 = {S105166, 72, { &P18028,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-MileStone/00.00.00 MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA27880 = {S105167, 135, { &P13921,&P17486,&P3581,&P18092 }};	/*Useragent - MOT-ZN5/FSRM319_81_00_7FR_V3 Mozilla/4.0 (compatible; OSS/1.0; Linux MOTOZINE ZN5) Profile/MIDP-2.0 Configuration/CLDC-1.1 Symphony 1.0*/
const Device UA27973 = {S105168, 104, { &P12201,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) PPC; 480x800; GSmart-S1200*/
const Device UA28408 = {S105169, 88, { &P12238,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) PPC; 240x320; GSmart-MS802*/
const Device UA28482 = {S105170, 191, { &P15377,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) LG-KS20/V10a Browser/IEMobile/7.6 MMS/LG-MMS-WINCE-V1.0/1.2 Java/LGVM/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA29237 = {S105171, 22, { &P11761,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW995a/R1EA*/
const Device UA29539 = {S105172, 13, { &P12264,&P17017,&P17470,&P18092 }};	/*Useragent - wellcom w3331*/
const Device UA30089 = {S105173, 92, { &P11886,&P5810,&P4948,&P18092 }};	/*Useragent - BlackBerry8700/4.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/120 UP.Browser/5.0.3.3*/
const Device UA30262 = {S105174, 90, { &P12291,&P19950,&P3625,&P18092 }};	/*Useragent - SAGEM-my302X/1.0/ MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.g.2.106 (GUI) MMP/2.0*/
const Device UA30334 = {S105175, 101, { &P12300,&P5448,&P17183,&P18092 }};	/*Useragent - HTC-P4600/%ROM_VERSION% Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA30858 = {S105176, 61, { &P17802,&P17811,&P3807,&P18092 }};	/*Useragent - Lenovo-E228/CMCC BMP/F2 Release/2008.09.10 Browser/TelecaQ03C*/
const Device UA30898 = {S105177, 46, { &P12363,&P17017,&P3487,&P18092 }};	/*Useragent - Utec-M699 UP.Browser/7.2.6.1.591 (GUI) MMP/2.0*/
const Device UA31190 = {S105178, 88, { &P14864,&P5267,&P17606,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8500/1.0; U; Bada/1.0; vi-vn) AppleWebKit/533.1 (KHTML*/
const Device UA31454 = {S105179, 92, { &P15998,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) HP iPAQ rw6828*/
const Device UA32203 = {S105180, 150, { &P12482,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Master Rom 1.1.4; en-us; SGH-T959 Build/ECLAIR) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA32239 = {S105181, 98, { &P12483,&P17484,&P4924,&P18092 }};	/*Useragent - SAMSUNG-SGH-U608/1.0 Release/1.24.2007 Browser/NetFront3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA32719 = {S105182, 83, { &P12517,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonK111i/R1EA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA32754 = {S105183, 93, { &P12757,&P19949,&P3625,&P18092 }};	/*Useragent - NEC-e238/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.3.2.g.1.106 (GUI) MMP/2.0*/
const Device UA32801 = {S105184, 125, { &P12771,&P5890,&P3439,&P18092 }};	/*Useragent - Samsung-SGH-i408_CMCC/1.0 SymbianOS/9.2 Series60/3.1 Release/5.25.2007 Browser/OSS3.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA32895 = {S105185, 39, { &P12789,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KG280 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA32902 = {S105186, 130, { &P12023,&P17484,&P4223,&P18092 }};	/*Useragent - SAMSUNG-GT-M8910U/1.0 SHP/VPP/R5 Release/26.08.2009 NetFront/3.5 Nextreaming SMM-MMS/1.2.0 profile/MIDP-2.1 configuration/CLDC-1.1*/
const Device UA32954 = {S105187, 83, { &P12805,&P17017,&P3807,&P18092 }};	/*Useragent - CLARO-X930/X931_V1_Z8_ES_A1EF101 Obigo/Q03C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA32993 = {S105188, 80, { &P12819,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-P82/S103 LMP/LML Release/2009.05.19 Profile/MIDP2.0 Configuration/CLDC1.1*/
const Device UA33010 = {S105189, 237, { &P12827,&P17605,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-L708E_CMCC/1.0 RTK-E/1.0 DF/1.0 Release/08.17.2007 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzU1Mjc5MDIzODMzMjIy UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA33064 = {S105190, 37, { &P12842,&P17483,&P17470,&P18092 }};	/*Useragent - LG-KG245/MIC/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA33114 = {S105191, 39, { &P12855,&P6476,&P4970,&P18092 }};	/*Useragent - Samsung-SPHA640 AU-MIC-A640/2.0 MMP/2.0*/
const Device UA33397 = {S105192, 44, { &P12913,&P17483,&P17602,&P18092 }};	/*Useragent - LG-U8210/V100 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA33445 = {S105193, 11, { &P12920,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHE810*/
const Device UA33504 = {S105194, 97, { &P17475,&P5768,&P4777,&P18092 }};	/*Useragent - Nokia6300/2.0 (07.00) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6300/UC Browser7.7.1.88/70/444*/
const Device UA33510 = {S105195, 123, { &P14708,&P5204,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia5130c-2/07.95; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.2.26.6*/
const Device UA33884 = {S105196, 92, { &P11886,&P7063,&P4948,&P18092 }};	/*Useragent - BlackBerry8700/4.1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100 UP.Browser/5.0.3.3*/
const Device UA34149 = {S105197, 163, { &P13218,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; en-us; A555 Build/1.16.2) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17 320X480 motorola A555*/
const Device UA34232 = {S105198, 42, { &P11708,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonK618i/R1KG SEMC/NewsReader/1.0*/
const Device UA34955 = {S105199, 121, { &P14848,&P5843,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-00/08.50; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.2.26.6*/
const Device UA35205 = {S105200, 123, { &P14862,&P5204,&P17435,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia2700c-2/09.95; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.2.26.6*/
const Device UA35278 = {S105201, 132, { &P11840,&P17486,&P3772,&P18092 }};	/*Useragent - MOT-MOTOV9/A0.05.20R_03 BER2.2 Mozilla/4.0 (compatible; MSIE 6.0; 13003348) Profile/MIDP-2.0 Configuration/CLDC-1.1  Opera 8.60 [zh]*/
const Device UA35438 = {S105202, 95, { &P11906,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-I8000U/1.0 Software/WM6.5 Release/01.24.2009 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA35936 = {S105203, 129, { &P11981,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; PPC i-mate 8502; 240x320; PPC i-mate 8502/ 240x320; PPC i-mate 8502; 240x320; PPC)*/
const Device UA36750 = {S105204, 43, { &P13550,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonV630iv/R1CE SEMC/NewsReader/1.0*/
const Device UA36977 = {S105205, 21, { &P12255,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonZ520i/R3F*/
const Device UA37318 = {S105206, 62, { &P14258,&P17486,&P17470,&P18092 }};	/*Useragent - MOT-W375/1.0 Release/03.30.2007 Browser/UPB6.3 Software/0.0.80*/
const Device UA37678 = {S105207, 152, { &P12460,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android VENROM_DL25_RC3_2.2.1; en-us; SCH-R880 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA37837 = {S105208, 125, { &P12496,&P5096,&P24662,&P18092 }};	/*Useragent - Mozilla/5.0(Linux;U;Android2.2;Zh_cn;SAMSUNG-SGH-I897;480*800;)AppleWebKit/528.5+ (KHTML) Version/3.1.2/UCWEB7.8.0.95/139/355*/
const Device UA38228 = {S105209, 170, { &P20518,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1-update1; zh-cn; Asus A10 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17; A10-V5.2.4-user-20101118*/
const Device UA38638 = {S105210, 91, { &P15526,&P5448,&P4808,&P18092 }};	/*Useragent - HD_mini_T5555 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA38760 = {S105211, 96, { &P11886,&P5026,&P4948,&P18092 }};	/*Useragent - BlackBerry8700/4.5.0.124 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100 UP.Browser/5.0.3.3*/
const Device UA39762 = {S105212, 51, { &P15364,&P7133,&P18407,&P18092 }};	/*Useragent - Opera/9.70 (Windows NT 6.1; U; en-gb) Presto/2.2.15*/
const Device UA39838 = {S105213, 74, { &P11958,&P5448,&P22272,&P18092 }};	/*Useragent - Palm750/v0101 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA39910 = {S105214, 73, { &P20272,&P5448,&P15792,&P18092 }};	/*Useragent - HTC-3100/1.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8)*/
const Device UA40244 = {S105215, 182, { &P14967,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Custom 2.3.3 by Snegovik; ru-ru; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MMS/LG-Android-MMS-V1.0/1.2*/
const Device UA40414 = {S105216, 154, { &P21270,&P5096,&P20772,&P18092 }};	/*Useragent - MQQBrowser/21 Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; ME722 Build/MLS2GC_2.6.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41150 = {S105217, 99, { &P14826,&P5843,&P4715,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (04.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac1-01/UC Browser7.4.0.65/70/351*/
const Device UA41542 = {S105218, 68, { &P16439,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V186/08.30.10R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA41577 = {S105219, 136, { &P14017,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1.2; de-de; LG-P990 Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA41890 = {S105220, 74, { &P14101,&P5448,&P22272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) ASUS-P525/1.0*/
const Device UA42148 = {S105221, 145, { &P12082,&P17979,&P20428,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android X.666 v1.1; en-us; X10a Build/2.1.A.0.435) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA42721 = {S105222, 73, { &P15051,&P17484,&P3816,&P18092 }};	/*Useragent - SAMSUNG-SGH-X668/TSS 2.5.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA43386 = {S105223, 73, { &P16224,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) o2 Xda comet*/
const Device UA43533 = {S105224, 139, { &P12096,&P5448,&P22271,&P18092 }};	/*Useragent - PPC; 240x320; HTC_TyTN/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Dopod838Pro*/
const Device UA44074 = {S105225, 42, { &P13778,&P17484,&P4841,&P18092 }};	/*Useragent - SCH-M609/M609CH01 POLARIS/5.30.WAP CTC/1.0*/
const Device UA44340 = {S105226, 125, { &P13801,&P5208,&P4902,&P18092 }};	/*Useragent - Mozilla/5.0 (SCH-M519/M519CJ26;U;REX/3.0;BREW/3.1.5;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*320;CTC/2.0) POLARIS/6.15.WEB*/
const Device UA44599 = {S105227, 97, { &P12059,&P5204,&P4722,&P18092 }};	/*Useragent - Nokia2690/2.0 (10.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia2690/UC Browser7.0.0.41/70/352*/
const Device UA44926 = {S105228, 75, { &P13845,&P17017,&P17470,&P18092 }};	/*Useragent - Fly_DS210 ObigoInternetBrowser/QO3C Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA45332 = {S105229, 22, { &P12130,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonK608i/R2BA*/
const Device UA45387 = {S105230, 113, { &P16672,&P17483,&P17470,&P18092 }};	/*Useragent - LG-KG77_CMCC/08.03.14 Infineon/SGold2 Release/29.12.2006 Browser/MIC1.1.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA45493 = {S105231, 145, { &P12173,&P5448,&P17183,&P18092 }};	/*Useragent - LG-CT810/V10x IEMobile/7.11 Profile/MIDP-2.0 Configuration/CLDC-1.1 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA45652 = {S105232, 153, { &P11950,&P5629,&P20775,&P18092 }};	/*Useragent - MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA46700 = {S105233, 91, { &P12300,&P5448,&P4808,&P18092 }};	/*Useragent - HTC-P4600/1.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0)*/
const Device UA48138 = {S105234, 12, { &P12640,&P17017,&P17470,&P18092 }};	/*Useragent - HUAWEI G6610*/
const Device UA48655 = {S105235, 79, { &P18115,&P7116,&P3507,&P18092 }};	/*Useragent - BlackBerryUnknown/5.0.0.526 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/11*/
const Device UA49125 = {S105236, 189, { &P13904,&P17605,&P3625,&P18092 }};	/*Useragent - SAMSUNG-E2210C/1.0 RTK-E/1.0 DF/1.0 Release/14.01.2009 Browser/OpenWave6.2.3.3.c.1.101 Profile/MIDP-2.0 Configuration/CLDC-1.1/*MzUzNDY2MDMxMTQ0ODgw UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA49179 = {S105237, 86, { &P12946,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-OT-V570A/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA49188 = {S105238, 52, { &P12950,&P17484,&P4901,&P18092 }};	/*Useragent - SAMSUNG-SGH-E100/T2 UP.Browser/6.1.0.6 (GUI) MMP/1.0*/
const Device UA49213 = {S105239, 46, { &P14272,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-U410T/1.0 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA49329 = {S105240, 39, { &P12992,&P17483,&P3625,&P18092 }};	/*Useragent - LG-KX266/1.0 UP.Browser/6.2.3.8 CTC/1.0*/
const Device UA49361 = {S105241, 102, { &P13001,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-S5320/S5320XXIK1 SHP/VPP/R5 NetFront/3.4 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA49359 = {S105242, 121, { &P13000,&P5890,&P3439,&P18092 }};	/*Useragent - SAMSUNG-SGH-G818E/1.0 SymbianOS/9.2 Series60/3.1 Release/1.15.2008 Browser/OSS3.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA49412 = {S105243, 85, { &P13017,&P7472,&P4203,&P18092 }};	/*Useragent - Alcatel-OT-S920/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q04A*/
const Device UA49447 = {S105244, 42, { &P13028,&P17484,&P4841,&P18092 }};	/*Useragent - SCH-F299/F299DE04 POLARIS/5.30.WAP CTC/1.0*/
const Device UA49510 = {S105245, 41, { &P13043,&P17483,&P3625,&P18092 }};	/*Useragent - LG-MX100 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA49803 = {S105246, 132, { &P13086,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; en-; Geeksphone-One) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA49804 = {S105247, 125, { &P13087,&P5208,&P4902,&P18092 }};	/*Useragent - Mozilla/5.0 (SCH-S559/S559DI05;U;REX/3.0;BREW/3.1.5;Profile/MIDP-2.1 Configuration/CLDC-1.1;240*320;CTC/2.0) POLARIS/6.15.WEB*/
const Device UA49825 = {S105248, 52, { &P13091,&P19950,&P3625,&P18092 }};	/*Useragent - SAGEM-myX-8/1.0 UP.Browser/6.2.3.3.g.1 (GUI) MMP/2.0*/
const Device UA49894 = {S105249, 85, { &P13102,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-Z310/1.0 SHP/VPP/R5 SMM-MMS/1.2.0 profile/MIDP-2.0 configuration/CLDC-1.1*/
const Device UA49950 = {S105250, 42, { &P11672,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonZ770i/R1FA SEMC/NewsReader/1.0*/
const Device UA49982 = {S105251, 108, { &P13129,&P17483,&P3728,&P18092 }};	/*Useragent - LG-GD300s_CMCC A2000pH/2.30.10 Release/20.4.2009 Browser/Teleca/Q7.1 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA49999 = {S105252, 46, { &P13133,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-MX300/1.0 UP.Browser/6.2.3.9 (GUI) MMP/2.0*/
const Device UA50009 = {S105253, 82, { &P13137,&P17485,&P3601,&P18092 }};	/*Useragent - SonyEricssonW600c/R7B Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA50005 = {S105254, 112, { &P13134,&P5448,&P22270,&P18092 }};	/*Useragent - ZTE-W N61/1.0.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA50145 = {S105255, 110, { &P14345,&P16000,&P4722,&P18092 }};	/*Useragent - NokiaN-GageQD/1.0 SymbianOS/6.1 Series60/1.2 Profile/MIDP-1.0 Configuration/CLDC-1.0/UC Browser7.0.0.41/26/352*/
const Device UA50171 = {S105256, 13, { &P14124,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-S3500*/
const Device UA50258 = {S105257, 92, { &P13207,&P17811,&P17810,&P18092 }};	/*Useragent - Lenovo-P609/2008.12.16 R002_QCY_LANG_C_V0.1.0/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA50313 = {S105258, 74, { &P13227,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-D820/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA50482 = {S105259, 16, { &P11939,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonS312*/
const Device UA50496 = {S105260, 35, { &P11880,&P5890,&P4737,&P18092 }};	/*Useragent - NOKIA5320/UC Browser7.2.2.51/28/352*/
const Device UA50798 = {S105261, 54, { &P12423,&P17485,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 SonyEricssonW715v/R1GA SEMC/NewsReader/1.0*/
const Device UA50975 = {S105262, 129, { &P13864,&P17017,&P4893,&P18092 }};	/*Useragent - SIE-S75/40 UP.Browser/7.1.0.e.20(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.1.0.e.20 (GUI) MMP/2.0*/
const Device UA51235 = {S105263, 74, { &P11685,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-D900/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA51514 = {S105264, 74, { &P11633,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-G600/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA51588 = {S105265, 42, { &P13317,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonTM717/R1FC SEMC/NewsReader/1.0*/
const Device UA51894 = {S105266, 73, { &P13367,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) O2 Xda Zinc*/
const Device UA52385 = {S105267, 75, { &P11919,&P17485,&P17470,&P18092 }};	/*Useragent - RSSFeeds/R1A SonyEricssonM600i/R100 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA52435 = {S105268, 151, { &P12311,&P17979,&P4755,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Ultimate EB28 v3.0.1; en-us; SPH-M900 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA52811 = {S105269, 78, { &P14603,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-B2100/B2100JBID1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Untrusted/1.0*/
const Device UA53289 = {S105270, 78, { &P13545,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) O2 Xda Atom Exec*/
const Device UA53475 = {S105271, 24, { &P13985,&P17017,&P17470,&P18092 }};	/*Useragent - WELLCOM-W980 WAP Browser*/
const Device UA53680 = {S105272, 104, { &P17404,&P17017,&P3625,&P18092 }};	/*Useragent - Bird.S689/K03,MN/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.6.d.3.100 (GUI) MMP/1.0*/
const Device UA53951 = {S105273, 37, { &P14077,&P17483,&P3866,&P18092 }};	/*Useragent - LG-KE858 MIC/1.1.14 MIDP-2.0/CLDC-1.1*/
const Device UA54070 = {S105274, 47, { &P14109,&P17017,&P17470,&P18092 }};	/*Useragent - Fly-V130/Profile/MIDP.2.0Configuration/CLDC.1.0*/
const Device UA54107 = {S105275, 76, { &P14122,&P17484,&P4924,&P18092 }};	/*Useragent - SAMSUNG-SGH-E778/NetFront 3.2/WAP2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54199 = {S105276, 119, { &P14149,&P17605,&P3976,&P18092 }};	/*Useragent - SAMSUNG-C3310C_CMCC/1.0 RTK-E/1.0 DF/1.0 Release/11.05.2008 Browser/NetFront3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA54391 = {S105277, 142, { &P14194,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-GT-S3030C/1.0 Release/8.25.2008 Browser/Openwave6.2.3 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA54410 = {S105278, 118, { &P14203,&P17605,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-U908E/1.0 RTK-E/1.0 DF/1.0 Release/08.03.2007 Browser/NetFront3.4  Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA55055 = {S105279, 35, { &P14278,&P17017,&P3807,&P18092 }};	/*Useragent - ONDA OTV1000-P107A2V1.0.0 Obigo/Q03*/
const Device UA55601 = {S105280, 47, { &P13457,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonGeneric A1/R8BF SEMC/NewsReader/1.0*/
const Device UA55693 = {S105281, 55, { &P23012,&P17485,&P17470,&P18092 }};	/*Useragent - Mozilla/4.0 SonyEricssonW890iv/R1FA SEMC/NewsReader/1.0*/
const Device UA55984 = {S105282, 116, { &P11658,&P5776,&P4737,&P18092 }};	/*Useragent - Nokia6680/1.0 (5.04.07) SymbianOS/8.0 Series60/2.6 Profile/MIDP-2.0 Configuration/CLDC-1.1/UC Browser7.2.2.51/27/401*/
const Device UA56015 = {S105283, 57, { &P12307,&P5926,&P3625,&P18092 }};	/*Useragent - Nokia6235i/1.0 (S190V0200.nep) UP.Browser/6.2.3.2 MMP/2.0*/
const Device UA56093 = {S105284, 15, { &P14638,&P19954,&P17470,&P18092 }};	/*Useragent - SEC-SGHE610/1.0*/
const Device UA56658 = {S105285, 91, { &P18088,&P5448,&P4643,&P18092 }};	/*Useragent - DPD-S900C/5U-DIYPDA???? OPERA/9.5 CTC/1.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA56808 = {S105286, 73, { &P13386,&P17483,&P17660,&P18092 }};	/*Useragent - LG-GM730/v10a WM/6.1 IEMobile/6.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA56872 = {S105287, 99, { &P14826,&P5843,&P4737,&P18092 }};	/*Useragent - NokiaC1-01/2.0 (04.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiac1-01/UC Browser7.2.2.51/70/352*/
const Device UA57685 = {S105288, 164, { &P15592,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux;U; Android 2.1-update1; zh-cn; ZTE-V9e Build/V9eV1.0.0B02;800*480;CTC/2.0) AppleWebkit/530.17(KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA57754 = {S105289, 81, { &P12711,&P5448,&P22270,&P18092 }};	/*Useragent - Motorola_ES405B/191 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11)*/
const Device UA58033 = {S105290, 173, { &P11950,&P7305,&P3553,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android Eclair BitSnacker.com v2.5; sv-se; SonyEricssonX10i Build/2.1.A.0.435) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17*/
const Device UA58263 = {S105291, 136, { &P14186,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1.1; en-us; GT-S5830 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA58935 = {S105292, 151, { &P15124,&P17605,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-C278/1.0 RTK-E/1.0 DF/1.0 Release/17.08.2007 Browser/OpenWave6.2.3.3.c.1.101/*MzU0MDQ1MDMwMTk4MjUw UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA60537 = {S105293, 39, { &P12539,&P17483,&P3625,&P18092 }};	/*Useragent - LG-MG185 UP.Browser/6.2.3 (GUI) MMP/1.0*/
const Device UA60555 = {S105294, 39, { &P12546,&P17483,&P3625,&P18092 }};	/*Useragent - LG-CG300 UP.Browser/6.2.3 (GUI) MMP/1.0*/
const Device UA60743 = {S105295, 22, { &P14440,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonJ210i/R101*/
const Device UA60744 = {S105296, 45, { &P12529,&P17484,&P4841,&P18092 }};	/*Useragent - SAMSUNG-SCH-F309/1.0 POLARIS/5.30.WAP CTC/1.0*/
const Device UA60844 = {S105297, 53, { &P11840,&P17486,&P3625,&P18092 }};	/*Useragent - Motorola-V9m UP.Browser/6.2.3.4.c.1.123 (GUI) MMP/2.0*/
const Device UA60853 = {S105298, 54, { &P12585,&P19950,&P4901,&P18092 }};	/*Useragent - SAGEM-myX-6/1.0 UP.Browser/6.1.0.6.1.c.4 (GUI) MMP/1.0*/
const Device UA60855 = {S105299, 99, { &P12586,&P5448,&P4808,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.0) i-mate 810-F/ 320x240*/
const Device UA60880 = {S105300, 85, { &P12591,&P7472,&P3807,&P18092 }};	/*Useragent - Alcatel-ELLE-N3/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 ObigoInternetBrowser/Q03C*/
const Device UA60971 = {S105301, 48, { &P12606,&P17017,&P4203,&P18092 }};	/*Useragent - Huawei/1.0/H11HW/B000 Browser/Obigo-Browser/Q04A*/
const Device UA61076 = {S105302, 68, { &P12624,&P5521,&P4836,&P18092 }};	/*Useragent - MOT-V545/0E.66.04R MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA61128 = {S105303, 51, { &P12641,&P5890,&P3625,&P18092 }};	/*Useragent - Nokia 6206 UP.Browser/6.2.3.9.j.1.106 (GUI) MMP/2.0*/
const Device UA61142 = {S105304, 39, { &P12642,&P17483,&P17602,&P18092 }};	/*Useragent - LG-KG296 Obigo/WAP2.0 MIDP-2.0/CLDC-1.1*/
const Device UA61170 = {S105305, 68, { &P12653,&P19954,&P4924,&P18092 }};	/*Useragent - SEC-SGHX810/1.0 NetFront/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61194 = {S105306, 85, { &P12658,&P17484,&P4841,&P18092 }};	/*Useragent - SAMSUNG-SCH-W699/1.0 POLARIS/5.30.WAP CTC/1.0 Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA61229 = {S105307, 51, { &P12661,&P17017,&P17470,&P18092 }};	/*Useragent - PG-1400/R01 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61352 = {S105308, 32, { &P12691,&P17484,&P17470,&P18092 }};	/*Useragent - SCH-W629 Infraware/5.10.CU (GUI)*/
const Device UA61364 = {S105309, 51, { &P12696,&P17017,&P17470,&P18092 }};	/*Useragent - PG-8000/R01 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA61355 = {S105310, 46, { &P12692,&P17484,&P5001,&P18092 }};	/*Useragent - SCH-U620/1.0 NetFront/3.0.22.2.3 (GUI) MMP/2.0*/
const Device UA61447 = {S105311, 37, { &P12721,&P17483,&P17470,&P18092 }};	/*Useragent - LG-F2300 MIC/WAP2.0 MIDP-2.0/CLDC-1.0*/
const Device UA63072 = {S105312, 51, { &P16204,&P6712,&P3625,&P18092 }};	/*Useragent - Nokia 7205 UP.Browser/6.2.3.9.j.1.106 (GUI) MMP/2.0*/
const Device UA63394 = {S105313, 86, { &P14509,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-M6V/50 UP.Browser/7.0.2.2.d.3(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA65024 = {S105314, 99, { &P16074,&P5768,&P4722,&P18092 }};	/*Useragent - Nokia3110c/2.0 (06.02) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia3110c/UC Browser7.0.0.41/70/352*/
const Device UA65638 = {S105315, 93, { &P14172,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-my700x/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 Browser/UP.Browser/7.2.7.1.300 (GUI)*/
const Device UA67096 = {S105316, 42, { &P11684,&P5129,&P24645,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UCWEB7.5.0.66/50/999*/
const Device UA67277 = {S105317, 35, { &P12487,&P5890,&P4715,&P18092 }};	/*Useragent - NOKIA5700/UC Browser7.4.0.65/28/401*/
const Device UA67330 = {S105318, 29, { &P13977,&P5890,&P24645,&P18092 }};	/*Useragent - NokiaN73/UCWEB7.5.0.66/28/999*/
const Device UA67376 = {S105319, 21, { &P14809,&P6909,&P3780,&P18092 }};	/*Useragent - Nokia8910/1.0 (04.02)*/
const Device UA67448 = {S105320, 35, { &P13536,&P5890,&P4715,&P18092 }};	/*Useragent - NOKIA6110/UC Browser7.4.0.65/28/352*/
const Device UA67527 = {S105321, 31, { &P12193,&P5129,&P24660,&P18092 }};	/*Useragent - NokiaC5-00/UCWEB7.6.0.75/28/401*/
const Device UA67554 = {S105322, 16, { &P14980,&P17484,&P17606,&P18092 }};	/*Useragent - samsung-gt-b7722*/
const Device UA67608 = {S105323, 37, { &P15261,&P17483,&P3866,&P18092 }};	/*Useragent - LG-KG90N MIC/1.1.14 MIDP-2.0/CLDC-1.1*/
const Device UA67691 = {S105324, 7, { &P14858,&P17483,&P17470,&P18092 }};	/*Useragent - lg-c100*/
const Device UA67690 = {S105325, 35, { &P14984,&P5129,&P4715,&P18092 }};	/*Useragent - Nokia5250/UC Browser7.4.0.65/50/355*/
const Device UA67780 = {S105326, 36, { &P14294,&P5890,&P3439,&P18092 }};	/*Useragent - Nokia5238/SymbianOS/9.1 Series60/3.0*/
const Device UA67798 = {S105327, 88, { &P14985,&P5520,&P17606,&P18092 }};	/*Useragent - Mozilla/5.0 (SAMSUNG; SAMSUNG-GT-S8530/1.0; U; Bada/1.2; vi-vn) AppleWebKit/533.1 (KHTML*/
const Device UA67861 = {S105328, 74, { &P14877,&P17979,&P22699,&P18092 }};	/*Useragent - TwitterAndroid/2.0.1 (122) GT-I9000/8 (samsung;GT-I9000;samsung;GT-I9000;)*/
const Device UA67890 = {S105329, 8, { &P13365,&P17483,&P17470,&P18092 }};	/*Useragent - lg-gu220*/
const Device UA67915 = {S105330, 17, { &P14881,&P17486,&P17470,&P18092 }};	/*Useragent - Moto-Morrison/1.0*/
const Device UA67973 = {S105331, 31, { &P11880,&P5890,&P24660,&P18092 }};	/*Useragent - Nokia 5320/UCWEB7.6.0.75/28/800*/
const Device UA68035 = {S105332, 16, { &P14972,&P17484,&P17606,&P18092 }};	/*Useragent - samsung-gt-s3370*/
const Device UA68048 = {S105333, 35, { &P14782,&P5890,&P4759,&P18092 }};	/*Useragent - NOKIAE61i/UC Browser7.6.1.82/28/352*/
const Device UA68086 = {S105334, 35, { &P12487,&P5890,&P4759,&P18092 }};	/*Useragent - NOKIA5700/UC Browser7.6.1.82/28/352*/
const Device UA68578 = {S105335, 51, { &P13771,&P17484,&P17606,&P18092 }};	/*Useragent - SamsungE708 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA68637 = {S105336, 47, { &P14945,&P17484,&P4948,&P18092 }};	/*Useragent - SAMSUNG-SGH-S500/SHARK UP.Browser/5.0.5.1 (GUI)*/
const Device UA68722 = {S105337, 31, { &P11907,&P5890,&P24662,&P18092 }};	/*Useragent - NokiaE52-1/UCWEB7.8.0.87/28/999*/
const Device UA68793 = {S105338, 92, { &P14845,&P5448,&P24662,&P18092 }};	/*Useragent - HTC_Touch_HD_T8282 Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC)/UCWEB7.8.0.87/31/999*/
const Device UA68846 = {S105339, 36, { &P16653,&P17483,&P3866,&P18092 }};	/*Useragent - LG-KG70 MIC/1.1.14 MIDP-2.0/CLDC-1.1*/
const Device UA68905 = {S105340, 42, { &P13677,&P17484,&P4777,&P18092 }};	/*Useragent - samsung-gt-s5253/UC Browser7.7.1.88/69/444*/
const Device UA69155 = {S105341, 40, { &P14764,&P17485,&P4779,&P18092 }};	/*Useragent - SonyEricssonU1/UC Browser7.8.0.95/50/351*/
const Device UA69225 = {S105342, 35, { &P14753,&P5890,&P4779,&P18092 }};	/*Useragent - Nokia N96/UC Browser7.8.0.95/28/352*/
const Device UA69262 = {S105343, 111, { &P14557,&P5843,&P4779,&P18092 }};	/*Useragent - Nokia6303classic/2.0 (10.12) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303classic/UC Browser7.8.0.95/70/351*/
const Device UA69506 = {S105344, 102, { &P12513,&P17485,&P4777,&P18092 }};	/*Useragent - SonyEricssonW20/R7DA Profile/MIDP-2.1 Configuration/CLDC-1.1 sonyericssonw20/UC Browser7.7.1.88/70/444*/
const Device UA69597 = {S105345, 113, { &P14682,&P5843,&P4737,&P18092 }};	/*Useragent - Nokia6303iclassic/5.0 (07.10) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia6303iclassic/UC Browser7.2.2.51/70/351*/
const Device UA69656 = {S105346, 35, { &P11945,&P17484,&P4715,&P18092 }};	/*Useragent - gt-s5233s/UC Browser7.4.0.65/70/351*/
const Device UA69848 = {S105347, 115, { &P12056,&P5204,&P4779,&P18092 }};	/*Useragent - Nokia7210Supernova/2.0 (07.23) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokia7210supernova/UC Browser7.8.0.95/70/355*/
const Device UA71376 = {S105348, 80, { &P15562,&P17484,&P3976,&P18092 }};	/*Useragent - SAMSUNG-SGH-F218/1.0 Release/7.25.2007 Browser/NetFront3.4/*MzU4MjIwMDEwMTYwOTAz*/
const Device UA72210 = {S105349, 142, { &P13787,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; SGH-I987 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA72375 = {S105350, 102, { &P12192,&P5448,&P17183,&P18092 }};	/*Useragent - HTC_Touch_Diamond2_T5353 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 8.12; MSIEMobile 6.5)*/
const Device UA73148 = {S105351, 153, { &P11950,&P5629,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.3; zh-cn; X10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA73218 = {S105352, 153, { &P11702,&P6757,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 1.5; zh-cn; ME600 Build/CUPCAKE) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1*/
const Device UA73630 = {S105353, 164, { &P12685,&P17017,&P4222,&P18092 }};	/*Useragent - Smartone/SharpSX813/SHS001/1.0 Browser/UP.Browser/7.0.2.1a.1.c.1.103 (GUI) Profile/MIDP-2.0 Configuration/CLDC-1.1 Ext-J-Profile/JSCL-1.2.2 Ext-V-Profile/VSCL-2.0.0*/
const Device UA75326 = {S105354, 139, { &P12681,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us; Nexus S 4G Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA76262 = {S105355, 28, { &P14464,&P17017,&P17470,&P18092 }};	/*Useragent - HPiPAQhw6900/1.0/Mozilla/4.0*/
const Device UA77882 = {S105356, 182, { &P13633,&P5096,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; MotoA953 Build/MILS2_U6_2.2.16; 480*854) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/800*/
const Device UA80128 = {S105357, 16, { &P17673,&P17017,&P17470,&P18092 }};	/*Useragent - AUDIOVOX-SMT5600*/
const Device UA80273 = {S105358, 60, { &P14606,&P6208,&P4420,&P18092 }};	/*Useragent - BlackBerry7520/4.0.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA80259 = {S105359, 60, { &P11665,&P6208,&P4420,&P18092 }};	/*Useragent - BlackBerry7280/4.0.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA80567 = {S105360, 80, { &P11769,&P16000,&P4615,&P18092 }};	/*Useragent - Nokia3600/1.0 SymbianOS/6.1 Series60/0.9 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA80954 = {S105361, 64, { &P14564,&P5204,&P17470,&P18092 }};	/*Useragent - Nokia7900/2.0 (nd3.30_p) Profile/MIDP-2.1 Configuration/CLDC-1.1*/
const Device UA81091 = {S105362, 10, { &P12061,&P5890,&P4030,&P18092 }};	/*Useragent - NokiaE65-1*/
const Device UA81425 = {S105363, 87, { &P15036,&P17017,&P4222,&P18092 }};	/*Useragent - SIE-SP65/47 UP.Browser/7.0.2.2.d.1(GUI) MMP/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA81499 = {S105364, 41, { &P19947,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-A850 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA81546 = {S105365, 20, { &P20338,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-C400/1.0*/
const Device UA81675 = {S105366, 41, { &P19919,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-W579 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA81723 = {S105367, 20, { &P20092,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-X636/1.0*/
const Device UA81747 = {S105368, 41, { &P19926,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-N330 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA81905 = {S105369, 101, { &P17313,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E750/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.c.1.101 (GUI) MMP/2.0*/
const Device UA81918 = {S105370, 53, { &P20334,&P17484,&P3625,&P18092 }};	/*Useragent - SAMSUNG-SGH-E820/1.0 UP.Browser/6.2.2.6 (GUI) MMP/1.0*/
const Device UA82118 = {S105371, 20, { &P20302,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-X481/1.0*/
const Device UA82626 = {S105372, 8, { &P12094,&P17486,&P17671,&P18092 }};	/*Useragent - MOT-C975*/
const Device UA82680 = {S105373, 48, { &P17689,&P17486,&P4901,&P18092 }};	/*Useragent - Motorola-E365 UP.Browser/6.1.0.7.3 (GUI) MMP/1.0*/
const Device UA83133 = {S105374, 47, { &P16507,&P19950,&P4948,&P18092 }};	/*Useragent - SAGEM-my202C/Orange1.0 UP.Browser/5.0.5.6 (GUI)*/
const Device UA83139 = {S105375, 90, { &P16511,&P19950,&P3625,&P18092 }};	/*Useragent - SAGEM-my300X/1.0/ MIDP-2.0 Configuration/CLDC-1.1 UP.Browser/6.2.3.3.g.2.106 (GUI) MMP/2.0*/
const Device UA83273 = {S105376, 99, { &P14139,&P19950,&P3625,&P18092 }};	/*Useragent - SAGEM-myX-5-2T/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.2.6.d.4.102 (GUI) MMP/1.0*/
const Device UA83205 = {S105377, 55, { &P13924,&P19950,&P17470,&P18092 }};	/*Useragent - SAGEM-myW-7/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA81520 = {S105378, 41, { &P19925,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-A970 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA83698 = {S105379, 17, { &P13547,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW700c*/
const Device UA83818 = {S105380, 39, { &P16484,&P17483,&P3625,&P18092 }};	/*Useragent - LG-C1300 UP.Browser/6.2.2 (GUI) MMP/1.0*/
const Device UA83831 = {S105381, 56, { &P16508,&P17483,&P17660,&P18092 }};	/*Useragent - LG-C3100 AU/4.10 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA83925 = {S105382, 56, { &P16201,&P17483,&P17660,&P18092 }};	/*Useragent - LG-G1600 AU/4.10 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA83927 = {S105383, 56, { &P16664,&P17483,&P17660,&P18092 }};	/*Useragent - LG-G1610 AU/4.10 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA84180 = {S105384, 56, { &P16826,&P17483,&P17660,&P18092 }};	/*Useragent - LG-G7100 AU/4.10 Profile/MIDP-1.0 Configuration/CLDC-1.0*/
const Device UA84258 = {S105385, 74, { &P18088,&P19949,&P4924,&P18092 }};	/*Useragent - NEC-e353/001.00 ACS-NF/3.2 Profile/MIDP-2.0 Configuration/CLDC-1.1 Qtv/1.0*/
const Device UA84385 = {S105386, 35, { &P16863,&P19948,&P17470,&P18092 }};	/*Useragent - PHILIPS859 ObigoInternetBrowser/2.0*/
const Device UA84946 = {S105387, 73, { &P12022,&P17017,&P3402,&P18092 }};	/*Useragent - BenQ-S80/1.00/WAP2.0/MIDP2.0/CLDC1.1 UP.Browser/6.3.0.1.119 (GUI) MMP/2.0*/
const Device UA85269 = {S105388, 12, { &P17001,&P17603,&P17470,&P18092 }};	/*Useragent - PANTECH G300*/
const Device UA85272 = {S105389, 7, { &P16997,&P18588,&P18589,&P18092 }};	/*Useragent - PT-G700*/
const Device UA84255 = {S105390, 89, { &P18088,&P19949,&P3625,&P18092 }};	/*Useragent - NEC-e121/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.0 UP.Browser/6.2.3.9.d.1 (GUI) MMP/2.0*/
const Device UA85322 = {S105391, 51, { &P16692,&P17017,&P17470,&P18092 }};	/*Useragent - PG-3500/R01 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA85339 = {S105392, 62, { &P16504,&P17017,&P3866,&P18092 }};	/*Useragent - PG-C120/R01 MIC/1.1.14 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA85200 = {S105393, 57, { &P16079,&P17017,&P4901,&P18092 }};	/*Useragent - O2-X2i/6.1.0.7 UP.Browser/6.1.0.7.8.c.1.103 (GUI) MMP/1.0*/
const Device UA85606 = {S105394, 28, { &P16793,&P17017,&P17470,&P18092 }};	/*Useragent - portalmmm/2.0 SG344i(c10;TB)*/
const Device UA85701 = {S105395, 27, { &P17325,&P17017,&P17470,&P18092 }};	/*Useragent - portalmmm/2.0 S410i(c10;TB)*/
const Device UA85838 = {S105396, 86, { &P15436,&P16000,&P20447,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Symbian OS; SonyEricsson P900; 316) Opera 6.31 [en]*/
const Device UA85843 = {S105397, 86, { &P12550,&P16000,&P20447,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 5.0; Symbian OS; SonyEricsson P910; 318) Opera 6.31 [en]*/
const Device UA86228 = {S105398, 16, { &P15523,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-I520*/
const Device UA86230 = {S105399, 16, { &P17478,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-SGH-I750*/
const Device UA86340 = {S105400, 42, { &P14003,&P17017,&P4970,&P18092 }};	/*Useragent - FLY-2040i/BSI AU.Browser/2.0 QO3C1 MMP/1.0*/
const Device UA86376 = {S105401, 63, { &P15364,&P17188,&P3601,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 4.0) NetFront/3.3*/
const Device UA86493 = {S105402, 98, { &P13926,&P17484,&P4030,&P18092 }};	/*Useragent - Vodafone/1.0/SamsungSGHX510V/X510BUFK3 Browser/Nokia/3.0.1 Profile/MIDP-2.0 Configuration/CLDC-1.0*/
const Device UA86508 = {S105403, 95, { &P16489,&P5448,&P17173,&P18092 }};	/*Useragent - T600; V3.02.E8 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) PPC; 480x640; GIGABYTE-g-Smart*/
const Device UA86512 = {S105404, 74, { &P16392,&P5448,&P17173,&P18092 }};	/*Useragent - HTC-8900/1.2 Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA86542 = {S105405, 73, { &P11734,&P5448,&P17173,&P18092 }};	/*Useragent - HTC_TyTN_II Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA86650 = {S105406, 67, { &P16227,&P5448,&P17173,&P18092 }};	/*Useragent - Xda_nova; 240x320 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12)*/
const Device UA86669 = {S105407, 41, { &P19927,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-R510 UP.Browser/6.2.3.8 (GUI) MMP/2.0*/
const Device UA86737 = {S105408, 122, { &P17595,&P18120,&P17153,&P18092 }};	/*Useragent - Mozilla/5.0 (itouch; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1A538a Safari/419.3*/
const Device UA86813 = {S105409, 119, { &P15364,&P22740,&P18098,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux armv6l; U) Opera 8.5 [en_US] Maemo browser 0.7.6 RX-34_2007SE_2.2006.51-6*/
const Device UA86842 = {S105410, 90, { &P17426,&P16000,&P3772,&P18092 }};	/*Useragent - SonyEricssonP700i/R100 Mozilla/4.0 (compatible; MSIE 6.0; Symbian OS; 513) Opera 8.65 [en]*/
const Device UA87096 = {S105411, 76, { &P13783,&P5035,&P3418,&P18092 }};	/*Useragent - BlackBerry9100/4.6.0.31 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/100*/
const Device UA87113 = {S105412, 36, { &P12647,&P17483,&P3534,&P18092 }};	/*Useragent - LGE-LG830 AU-OBIGO/Q05A1-3.8 MMP/2.0*/
const Device UA87309 = {S105413, 138, { &P16346,&P5448,&P22272,&P18092 }};	/*Useragent - MOT-Q8_CMCC/1.0 Release/01.10.2007 Profile/MIDP-2.0 Configuration/CLDC-1.1 Software/WM6.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA87161 = {S105414, 47, { &P17498,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-CA35 UP.Browser/6.2.0.11.1.2 (GUI) MMP/2.0*/
const Device UA87347 = {S105415, 117, { &P12598,&P5448,&P22271,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.6) Xda Graphite/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87365 = {S105416, 68, { &P11870,&P5448,&P15792,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.8) HTCS620*/
const Device UA87405 = {S105417, 134, { &P14105,&P5448,&P4578,&P18092 }};	/*Useragent - Vodafone/1.0/HTC_VPACompactIV/5.4.172.1/Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; MIDP-2.0 Configuration/CLDC-1.1; PPC; 240x320)*/
const Device UA87428 = {S105418, 39, { &P16590,&P17483,&P17602,&P18092 }};	/*Useragent - LG-B2100 Obigo/WAP2.0 MIDP-2.0/CLDC-1.0*/
const Device UA87432 = {S105419, 46, { &P16852,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-LG200/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA87472 = {S105420, 43, { &P12683,&P17484,&P4203,&P18092 }};	/*Useragent - Samsung-SPHM620 AU-OBIGO/Q04C1-1.22 MMP/2.0*/
const Device UA87549 = {S105421, 70, { &P16483,&P5448,&P22272,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7) HTC/X03HT*/
const Device UA87557 = {S105422, 119, { &P17688,&P5448,&P22270,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 7.11) 320x240; VZW; Motorola-A4500; Windows Mobile 6.1 Standard*/
const Device UA87766 = {S105423, 86, { &P17792,&P17662,&P3601,&P18092 }};	/*Useragent - SoftBank/1.0/706SC/SCJ001 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA87822 = {S105424, 46, { &P20218,&P17483,&P3625,&P18092 }};	/*Useragent - LGE-MX240/1.0 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA87865 = {S105425, 140, { &P20336,&P5448,&P22272,&P18092 }};	/*Useragent - SAMSUNG-SGH-i688_CMCC/1.0 Release/03.07 Browser/IE6 Profile/MIDP-2.0 Configuration/CLDC-1.1 (compatible; MSIE 6.0; Windows CE; IEMobile 7.7)*/
const Device UA87952 = {S105426, 47, { &P17864,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-SA39 UP.Browser/6.2.0.12.1.4 (GUI) MMP/2.0*/
const Device UA87982 = {S105427, 47, { &P17495,&P19941,&P3625,&P18092 }};	/*Useragent - KDDI-CA34 UP.Browser/6.2.0.10.3.3 (GUI) MMP/2.0*/
const Device UA88170 = {S105428, 77, { &P16070,&P5448,&P4578,&P18092 }};	/*Useragent - Qtek8020 (Mozilla\4.0 compatible; MSIE 4.01; Windows CE; Smartphone; 176x220)*/
const Device UA88067 = {S105429, 41, { &P17464,&P17484,&P3625,&P18092 }};	/*Useragent - SCH-A915 UP.Browser/6.2.3.2 (GUI) MMP/2.0*/
const Device UA88446 = {S105430, 73, { &P20271,&P5448,&P17173,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; IEMobile 6.12) HTC-SMT5800*/
const Device UA88758 = {S105431, 39, { &P12855,&P6476,&P3866,&P18092 }};	/*Useragent - Samsung-SPHA640 AU-MIC/1.1.12.0 MMP/2.0*/
const Device UA88787 = {S105432, 122, { &P16961,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; GT-I7500) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA88782 = {S105433, 105, { &P17412,&P17485,&P3976,&P18092 }};	/*Useragent - SonyEricssonA195i/R1AA Browser/NetFront/3.4 Profile/MIDP-2.1 Configuration/CLDC-1.1 JavaPlatform/JP-8.5.0*/
const Device UA88899 = {S105434, 92, { &P14171,&P19950,&P3487,&P18092 }};	/*Useragent - SAGEM-my721X/1.0/EAN_ET MIDP-2.0 Configuration/CLDC-1.L Browser/UP.Browser/7.2.7.1.352 (GUI)*/
const Device UA88943 = {S105435, 163, { &P14733,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; pl-pl; Era G1 Build/CRC1) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2 (AdMob-ANDROID-20090728)*/
const Device UA88963 = {S105436, 54, { &P16507,&P19950,&P4901,&P18092 }};	/*Useragent - SAGEM-my202/1.0 UP.Browser/6.1.0.6.1.c.5 (GUI) MMP/1.0*/
const Device UA89002 = {S105437, 10, { &P16912,&P22835,&P17470,&P18092 }};	/*Useragent - Spice S705*/
const Device UA89696 = {S105438, 146, { &P17912,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; zh-cn; dopod A6288 Build/CUPCAKE) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA89752 = {S105439, 140, { &P16138,&P6757,&P3525,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 1.5; zh-cn; MT710 Build/CUPCAKE) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2*/
const Device UA90560 = {S105440, 121, { &P14714,&P5843,&P17431,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; NokiaX3-02/05.65; Profile/MIDP-2.1 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.0.9.10*/
const Device UA90572 = {S105441, 122, { &P13798,&P5448,&P3430,&P18092 }};	/*Useragent - Opera/9.5 (Microsoft Windows; PPC; 640x480; HTC_X7500/1.0 Profile/MIDP-2.0 Configuration/CLDC-1.1; Opera Mobi/2745; U; en)*/
const Device UA135944 = {S105442, 8, { &P23545,&P17017,&P18246,&P18092 }};	/*Useragent - mauie800*/
const Device UA158256 = {S105443, 16, { &P15198,&P17484,&P17606,&P18092 }};	/*Useragent - samsung-gt-s3650*/
const Device UA253793 = {S105444, 107, { &P17595,&P22472,&P17825,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; Konqueror/3.5; Darwin 8.10.0; X11; Power Macintosh; en_US)KHTML/3.5.6 (like Gecko)*/
const Device UA253890 = {S105445, 62, { &P11762,&P17485,&P17470,&P18092 }};	/*Useragent - SonyEricssonW710i/R1JC Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA253941 = {S105446, 104, { &P12026,&P17485,&P4777,&P18092 }};	/*Useragent - SonyEricssonC510/R1HA Profile/MIDP-2.1 Configuration/CLDC-1.1 sonyericssonc510/UC Browser7.7.1.88/70/351*/
const Device UA253949 = {S105447, 43, { &P13206,&P17485,&P4779,&P18092 }};	/*Useragent - sonyericssonk530i/UC Browser7.8.0.95/70/351*/
const Device UA253987 = {S105448, 107, { &P19968,&P17485,&P17338,&P18092 }};	/*Useragent - SonyEricssonW910i/R1FA Profile/MIDP-2.1 Configuration/CLDC-1.1 sonyericssonw910i/UC Browser7.9.0.102/70/352*/
const Device UA257876 = {S105449, 124, { &P15767,&P17090,&P22248,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1 Flipboard/1.6.1*/
const Device UA258004 = {S105450, 192, { &P16877,&P5890,&P3439,&P18092 }};	/*Useragent - Mozilla/5.0 (SymbianOS/9.3; Series60/3.2 VertuConstellationQuest/051.719.2; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/525 (KHTML, like Gecko) Version/3.0 BrowserNG/7.2.6.2 3gpp-gba*/
const Device UA366205 = {S105451, 50, { &P18254,&P17017,&P18245,&P18169 }};	/*Useragent - Mozilla/5.0 (compatible; en-US; RGAnalytics/1.0; )*/
const Device UA369122 = {S105452, 151, { &P11747,&P5096,&P20780,&P18092 }};	/*Useragent - MQQBrowser/2.9/Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369190 = {S105453, 141, { &P13503,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; HTC Desire Z Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA369796 = {S105454, 155, { &P15354,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; sv-se; ST17i Build/4.0.2.A.0.42; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369813 = {S105455, 149, { &P21473,&P5629,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 2.3.4; zh-cn; XT319 Build/V4.37B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369830 = {S105456, 146, { &P14090,&P5096,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2; en-gb; GT-I5801 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA369881 = {S105457, 58, { &P14011,&P5629,&P15760,&P18092 }};	/*Useragent - Dalvik/1.4.0 (Linux; U; Android 2.3.7; U8500 Build/GRK39F)*/
const Device UA370052 = {S105458, 95, { &P15575,&P5629,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Mail; 1.3.9) (GT-I9100; samsung; GT-I9100; 2.3.4/GINGERBREAD)*/
const Device UA370123 = {S105459, 155, { &P14969,&P5629,&P20780,&P18092 }};	/*Useragent - MQQBrowser/2.9/Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370145 = {S105460, 154, { &P21340,&P5096,&P20776,&P18092 }};	/*Useragent - MQQBrowser/25 Mozilla/5.0 (Linux; U; Android 2.2.2; zh-cn; HUAWEI T8300 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370187 = {S105461, 133, { &P13384,&P5629,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-gb; U8120 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA370255 = {S105462, 46, { &P13141,&P17979,&P22287,&P18092 }};	/*Useragent - TuneIn Radio/5.2 (Android 10; SGH-I727R; Java)*/
const Device UA370430 = {S105463, 143, { &P17968,&P18066,&P21558,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en_us) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Ninesky-android-mobile/1.9.1 Safari/533.1*/
const Device UA370672 = {S105464, 64, { &P16376,&P5096,&P3508,&P18092 }};	/*Useragent - Logos/0.9.3 (Linux; U; Android 2.2.1; en_US; PRS-T1 Build/FRG83)*/
const Device UA371078 = {S105465, 156, { &P15201,&P5629,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; HTC Sensation 4G Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA390172 = {S105466, 16, { &P14985,&P17484,&P17606,&P18092 }};	/*Useragent - samsung-gt-s8530*/
const Device UA428993 = {S105467, 129, { &P15216,&P5156,&P3519,&P18092 }};	/*Useragent - Mozilla/5.0 (BlackBerry; U; BlackBerry 8520; en-US) AppleWebKit/534.1+ (KHTML, like Gecko) Version/6.0.0.141 Mobile Safari/534.1+*/
const Device UA452823 = {S105468, 80, { &P17595,&P23097,&P17470,&P18092 }};	/*Useragent - WebProcess/7534.48.3 CFNetwork/520.0.13 Darwin/11.1.0 (x86_64) (MacBookPro8%2C2)*/
const Device UA458861 = {S105469, 167, { &P17956,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Kyros MID7016 GB 8K Build/GRK39F; CyanogenMod-7.1.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA459100 = {S105470, 69, { &P21225,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.3;Zh_cn;MID7010;480*800;)UCWEB7.8.0.95/139/444*/
const Device UA459146 = {S105471, 70, { &P18314,&P18110,&P24667,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; MEIZU MX; 640*960) UCWEB8.2.3.141/145/999*/
const Device UA459153 = {S105472, 70, { &P18314,&P18110,&P24668,&P18092 }};	/*Useragent - JUC (Linux; U; 2.3.5; zh-cn; MEIZU MX; 640*960) UCWEB8.3.2.156/145/800*/
const Device UA459248 = {S105473, 35, { &P15988,&P17484,&P21757,&P18092 }};	/*Useragent - iBrowser/Mini2.4 (SAMSUNG-GT-C3312)*/
const Device UA474991 = {S105474, 42, { &P11684,&P5129,&P24644,&P18092 }};	/*Useragent - Nokia5800 XpressMusic/UCWEB7.4.0.57/50/999*/
const Device UA494202 = {S105475, 34, { &P14519,&P5129,&P24645,&P18092 }};	/*Useragent - NokiaN97 mini/UCWEB7.5.0.66/50/999*/
const Device UA500415 = {S105476, 29, { &P14454,&P5890,&P24645,&P18092 }};	/*Useragent - NokiaE66/UCWEB7.5.0.66/28/800*/
const Device UA549949 = {S105477, 29, { &P13163,&P5129,&P24662,&P18092 }};	/*Useragent - NokiaN97/UCWEB7.8.0.87/50/999*/
const Device UA564878 = {S105478, 99, { &P14848,&P5843,&P4779,&P18092 }};	/*Useragent - NokiaX3-00/5.0 (08.40) Profile/MIDP-2.1 Configuration/CLDC-1.1 nokiax3-00/UC Browser7.8.0.95/70/351*/
const Device UA572788 = {S105479, 43, { &P14864,&P17484,&P17338,&P18092 }};	/*Useragent - samsung-gt-s8500/UC Browser7.9.0.102/69/352*/
const Device UA582508 = {S105480, 162, { &P15201,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Sensation Z710e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Mobile Maxthon*/
const Device UA590385 = {S105481, 97, { &P15364,&P18110,&P21779,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux i686) AppleWebKit/534.34 (KHTML, like Gecko) QupZilla/1.1.8 Safari/534.34*/
const Device UA606813 = {S105482, 45, { &P15364,&P18570,&P18538,&P18092 }};	/*Useragent - Dolphin/4.0.1 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA606902 = {S105483, 33, { &P15364,&P17017,&P21905,&P18092 }};	/*Useragent - contype chromeframe/18.0.1025.162*/
const Device UA607209 = {S105484, 44, { &P12280,&P18117,&P18538,&P18092 }};	/*Useragent - Dolphin 4.0.1 (iPhone; iPhone OS 5.1; en_US)*/
const Device UA616389 = {S105485, 83, { &P17465,&P5690,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 5.1; Opera Tablet/23731; U; en) Presto/2.9.201 Version/11.50*/
const Device UA616391 = {S105486, 83, { &P17465,&P17285,&P18070,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; Opera Tablet/23731; U; en) Presto/2.9.201 Version/11.50*/
const Device UA616406 = {S105487, 82, { &P17465,&P17285,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Windows NT 6.0; Opera Tablet/11648; U; en) Presto/2.7.81 Version/11.00*/
const Device UA616425 = {S105488, 103, { &P12280,&P17779,&P18167,&P18092 }};	/*Useragent - Mozilla/5.0 (CrOS; iPhone) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1071.0 Mobile Safari/536.3*/
const Device UA616561 = {S105489, 139, { &P15397,&P24697,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Xperia Ray Build/MIUI) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA616614 = {S105490, 190, { &P15364,&P18110,&P24300,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; U; Linux i686; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.1.4(+mouse+3D+SCREEN+TUNER; LGE; GLOBAL-PLAT3; 00.03.01; 0xX;); LG NetCast.TV-2011*/
const Device UA3039583 = {S105491, 182, { &P15364,&P18110,&P24300,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; U; Linux 35230; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.1.4(+SCREEN+TUNER; LGE; GLOBAL-PLAT3; 04.02.28; 0xX;); LG NetCast.TV-2011*/
const Device UA618788 = {S105492, 80, { &P12280,&P17779,&P17368,&P18092 }};	/*Useragent - Mozilla Compatible/5.0 (iPhone/iPad; Mxit Mobi/5.9/1.8.5.168; U;) Opera Mini/3.1*/
const Device UA619499 = {S105493, 99, { &P15767,&P17090,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) [operamini] [UsableNet Lift Mobile]*/
const Device UA621669 = {S105494, 85, { &P12280,&P17779,&P18255,&P18092 }};	/*Useragent - Opera/9.80 (iPhone; Opera Mini/7.2.37083/27.1667; U; es) Presto/2.8.119 Version/11.10*/
const Device UA621886 = {S105495, 43, { &P15364,&P18570,&P18542,&P18092 }};	/*Useragent - Dolphin/4.1 CFNetwork/548.0.4 Darwin/11.0.0*/
const Device UA621969 = {S105496, 44, { &P15364,&P18570,&P18540,&P18092 }};	/*Useragent - Dolphin/3.5 CFNetwork/485.13.9 Darwin/11.0.0*/
const Device UA621973 = {S105497, 50, { &P17346,&P17146,&P18540,&P18092 }};	/*Useragent - Dolphin 3.5.1 (iPod touch; iPhone OS 5.0.1; en_US)*/
const Device UA622177 = {S105498, 46, { &P12280,&P17146,&P18538,&P18092 }};	/*Useragent - Dolphin 4.0.1 (iPhone; iPhone OS 5.0.1; en_US)*/
const Device UA622183 = {S105499, 46, { &P15364,&P18570,&P18551,&P18092 }};	/*Useragent - Dolphin/1.0.1 CFNetwork/485.13.9 Darwin/11.0.0*/
const Device UA622023 = {S105500, 45, { &P15767,&P17090,&P24448,&P18092 }};	/*Useragent - Dolphin HD 4.0 (iPad; iPhone OS 4.3.3; en_US)*/
const Device UA622356 = {S105501, 43, { &P15364,&P18570,&P18544,&P18092 }};	/*Useragent - Dolphin/5.0 CFNetwork/548.1.4 Darwin/11.0.0*/
const Device UA622403 = {S105502, 46, { &P12280,&P17090,&P18538,&P18092 }};	/*Useragent - Dolphin 4.0.1 (iPhone; iPhone OS 4.3.5; fi_FI)*/
const Device UA622413 = {S105503, 63, { &P15767,&P17146,&P24451,&P18092 }};	/*Useragent - Dolphin HD 5.0 (iPad; iPhone OS 5.0.1; en_US@calendar=buddhist)*/
const Device UA622355 = {S105504, 46, { &P12280,&P17090,&P18540,&P18092 }};	/*Useragent - Dolphin 3.5.1 (iPhone; iPhone OS 4.3.3; en_US)*/
const Device UA622481 = {S105505, 45, { &P15767,&P17146,&P24452,&P18092 }};	/*Useragent - Dolphin HD 5.1 (iPad; iPhone OS 5.0.1; zh_TW)*/
const Device UA622449 = {S105506, 46, { &P12280,&P18117,&P18540,&P18092 }};	/*Useragent - Dolphin 3.5.1 (iPhone; iPhone OS 5.1.1; en_US)*/
const Device UA692765 = {S105507, 17, { &P16384,&P17484,&P17606,&P18092 }};	/*Useragent - samsung-gt-s5380k*/
const Device UA858300 = {S105508, 157, { &P19891,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; SAMSUNG-SGH-I717 Build/GINGERBREAD)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA876256 = {S105509, 85, { &P18141,&P22216,&P23485,&P18092 }};	/*Useragent - Mozilla/5.0 (PlayStation Vita 1.65) AppleWebKit/531.22.8 (KHTML, like Gecko) Silk/3.2*/
const Device UA899664 = {S105510, 52, { &P17346,&P17084,&P17470,&P18092 }};	/*Useragent - Knott's 1.0 rv:54 (iPod touch; iPhone OS 4.1; en_US)*/
const Device UA903631 = {S105511, 36, { &P15775,&P5448,&P4702,&P18092 }};	/*Useragent - NOKIA Lumia 800/1750.0805.8773.12220*/
const Device UA905237 = {S105512, 157, { &P12605,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; DROID BIONIC Build/5.5.1_84_DBN-74)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA912469 = {S105513, 191, { &P18295,&P18066,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC One X Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, gb*/
const Device UA925713 = {S105514, 98, { &P20276,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; HTC; Ultimate)*/
const Device UA925770 = {S105515, 95, { &P14904,&P15924,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.5; Trident/3.1; IEMobile/7.0; HTC; Mazaa)*/
const Device UA925907 = {S105516, 108, { &P18562,&P18148,&P18263,&P18092 }};	/*Useragent - Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; P4301)*/
const Device UA1881493 = {S105517, 76, { &P15364,&P18140,&P17374,&P18092 }};	/*Useragent - Opera/9.80 (X11; Linux i686; U; Linux Mint; en) Presto/2.8.131 Version/11.11*/
const Device UA1895774 = {S105518, 76, { &P17595,&P23097,&P17415,&P18092 }};	/*Useragent - Safari/7534.48.3 CFNetwork/520.0.13 Darwin/11.1.0 (x86_64) (MacBookPro3%2C1)*/
const Device UA1927254 = {S105519, 32, { &P11880,&P5890,&P24666,&P18092 }};	/*Useragent - Nokia 5320/UCWEB8.1.0.104/28/800*/
const Device UA1934628 = {S105520, 133, { &P12280,&P17090,&P22246,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8J2 Flipboard/1.7.1*/
const Device UA1962419 = {S105521, 168, { &P15397,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; ST18i Build/4.1.B.0.431) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA1977898 = {S105522, 32, { &P15364,&P17017,&P21910,&P18092 }};	/*Useragent - contype chromeframe/20.0.1132.57*/
const Device UA1986102 = {S105523, 131, { &P12280,&P17146,&P18357,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/ Sleipnir/1.8.3m*/
const Device UA2003826 = {S105524, 130, { &P17595,&P17296,&P18532,&P18092 }};	/*Useragent - Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1 Safari/7534.51.22 Sleipnir/3.6*/
const Device UA2019501 = {S105525, 64, { &P12082,&P5062,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android1.6;Zh_cn;X10a;480*854;)UCWEB7.8.0.95/139/355*/
const Device UA2021541 = {S105526, 150, { &P11747,&P5096,&P20777,&P18092 }};	/*Useragent - MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; HTC Desire Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2023447 = {S105527, 70, { &P14959,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.5;Zh_cn;GT-I9103;480*800;)UCWEB7.8.0.95/139/351*/
const Device UA2023809 = {S105528, 163, { &P14090,&P5096,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2; en-us ; GT-I5801 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2024548 = {S105529, 146, { &P16349,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; Sony Tablet S Build/TISU0077)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2024971 = {S105530, 164, { &P14967,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.3; en_us ; LG-P500 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2025321 = {S105531, 168, { &P14877,&P24689,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; nl-nl; GT-I9000 Build/IMM76I; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2026497 = {S105532, 191, { &P18296,&P18066,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; HTC One V Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, gb*/
const Device UA2026875 = {S105533, 153, { &P15880,&P5629,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; MB865 Build/5.5.1-175_EDMR1.25)Maxthon AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2027178 = {S105534, 168, { &P15539,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en-us ; HTC Glacier Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2027953 = {S105535, 169, { &P13815,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; HTC Wildfire Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2030590 = {S105536, 169, { &P13251,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.2; en-us ; Dell Streak Build/FRG83G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2030610 = {S105537, 191, { &P14969,&P5629,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-gb; HTC Desire S Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.9.6/135,1.9.6.135,2012-08-19 13:37, gb*/
const Device UA2031672 = {S105538, 158, { &P15995,&P18149,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; HTC Evo Shift Build/GRI40; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2032064 = {S105539, 166, { &P16553,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; SGH-T959V Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2032383 = {S105540, 140, { &P15699,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Incredible Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2032491 = {S105541, 168, { &P12814,&P24690,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; SGH-I777 Build/JRO03L; CyanogenMod-PARANOIDANDROID) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2033304 = {S105542, 169, { &P16552,&P5096,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; en-us ; Micromax_A70 Build/FRG83) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2034650 = {S105543, 97, { &P18562,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; NOKIA; P4301)*/
const Device UA2043763 = {S105544, 98, { &P18249,&P6830,&P4643,&P18092 }};	/*Useragent - Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0; Acer; Allegro)*/
const Device UA2044193 = {S105545, 172, { &P18465,&P22212,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; vi-vn; Amazon Kindle Fire Build/JRO03L; CyanogenMod-10)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2044224 = {S105546, 200, { &P18465,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Amazon Kindle Fire Build/JRO03L) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.6/170,1.9.6.170,2012-09-28 11:34, in*/
const Device UA2192110 = {S105547, 112, { &P15364,&P7133,&P18366,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1 slurp*/
const Device UA2241317 = {S105548, 150, { &P18385,&P18111,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; Ainovo Flame (Nexus 7) Build/JRO03D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2241335 = {S105549, 118, { &P18385,&P24693,&P18112,&P18092 }};	/*Useragent - Logos/2.0.9 (Linux; U; Android 4.1.1; en-GB; Nexus 7 Build/aokp_grouper-userdebug 4.1.1 JRO03H eng.root.X.X test-keys)*/
const Device UA2241355 = {S105550, 70, { &P18385,&P18111,&P24673,&P18092 }};	/*Useragent - JUC (Linux; U; 4.1.1; zh-cn; Nexus 7; 800*1205) UCWEB8.7.0.187/145/999*/
const Device UA2241447 = {S105551, 129, { &P18385,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-us; Nexus 7 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2242115 = {S105552, 131, { &P18385,&P18111,&P18106,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; Nexus 7 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/58.0.1025.166  Safari/535.19*/
const Device UA2242157 = {S105553, 139, { &P18385,&P18111,&P20791,&P18092 }};	/*Useragent - MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; Nexus 7 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2242159 = {S105554, 124, { &P18385,&P17979,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android; Nexus 7 Build/JRO03D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Safari/535.19*/
const Device UA2276817 = {S105555, 65, { &P17595,&P22475,&P21790,&P18092 }};	/*Useragent - 1PasswordThumbs1 CFNetwork/438.16 Darwin/9.8.0 (i386) (iMac7%2C1)*/
const Device UA2297939 = {S105556, 91, { &P17595,&P18120,&P17269,&P18092 }};	/*Useragent - Opera/9.80 (Macintosh; Intel Mac OS X; Opera Mobi/14316; U; en) Presto/2.7.81 Version/11.00*/
const Device UA2300295 = {S105557, 36, { &P11679,&P5776,&P17338,&P18092 }};	/*Useragent - nokia6630/UC Browser7.9.0.102/69/352*/
const Device UA2306206 = {S105558, 32, { &P12588,&P5055,&P24666,&P18092 }};	/*Useragent - NokiaC6-01/UCWEB8.1.0.104/51/800*/
const Device UA2314384 = {S105559, 49, { &P14358,&P5890,&P20775,&P18092 }};	/*Useragent - MQQBrowser/2.4 (E63-1;SymbianOS/9.1 Series60/3.0)*/
const Device UA2316605 = {S105560, 97, { &P15364,&P7133,&P18202,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.34 (KHTML, like Gecko) PhantomJS/1.4.1 Safari/534.34*/
const Device UA2451836 = {S105561, 117, { &P13365,&P17483,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; lg-gu220) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/352/UCWEB Mobile*/
const Device UA2458453 = {S105562, 87, { &P15518,&P17017,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; s5230) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2462443 = {S105563, 138, { &P19748,&P5043,&P24668,&P18092 }};	/*Useragent - Nokia205/2.0 (03.18) Profile/MIDP-2.1 Configuration/CLDC-1.1 Mozilla/5.0 (Java; U; en-us; nokia205) UCBrowser8.3.1.161/70/465/UCWEB Mobile*/
const Device UA2502243 = {S105564, 118, { &P13847,&P17483,&P24671,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; lg-lg900g) AppleWebKit/530.13 (KHTML, like Gecko) UCBrowser/8.5.0.185/82/405/UCWEB Mobile*/
const Device UA2505228 = {S105565, 91, { &P11766,&P5776,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; nokia3230) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2506200 = {S105566, 99, { &P14831,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; samsung-gt-s7233e) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2530760 = {S105567, 141, { &P20947,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; HTCEVOV4G Build/unknown) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2540485 = {S105568, 138, { &P21714,&P18111,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; ZTE V988 Build/JZO54K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2540498 = {S105569, 146, { &P21264,&P18111,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; PadFone Infinity Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2540518 = {S105570, 33, { &P20999,&P17484,&P17606,&P18092 }};	/*Useragent - SAMSUNG-GT-S5292 SAMSUNG-GT-S5292*/
const Device UA2540534 = {S105571, 105, { &P21001,&P17484,&P18369,&P18092 }};	/*Useragent - SAMSUNG-GT-S3802 Opera/9.80 (J2ME/MIDP; Opera Mini/7.0.32562/28.3692; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2540550 = {S105572, 28, { &P20249,&P17979,&P20428,&P18092 }};	/*Useragent - Android OS/3.1 HTC_HTC-P715a*/
const Device UA2540573 = {S105573, 140, { &P21203,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; SX-SP715A Build/IML74K)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2540582 = {S105574, 159, { &P20249,&P5629,&P4459,&P18092 }};	/*Useragent - Rdio.Net; Android/2.4.9; Android/3.1; DM=HTC; DN=HTC-P715a; DC=rogers_wwe; DF=rogers_wwe/htc_puccinilte/puccinilte:3.1/HMJ15/X.4:user/release-keys; DH=verdilte*/
const Device UA2540597 = {S105575, 166, { &P21203,&P5629,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.4; en_us ; SX-SP715A Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2540606 = {S105576, 190, { &P20249,&P6546,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-ca; HTC-P715a Build/HMJ15) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.16/276,1.9.16.276,2012-12-21 16:37, -0500, ca*/
const Device UA2540628 = {S105577, 167, { &P21203,&P18066,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us ; SX-SP715A Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355*/
const Device UA2579234 = {S105578, 101, { &P23534,&P22839,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; zte-p107a20(propus)) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2580476 = {S105579, 92, { &P11658,&P5776,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6680) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2586649 = {S105580, 158, { &P21611,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; HUAWEI-M931 Build/HuaweiM931) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2586661 = {S105581, 83, { &P21629,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Adr (Linux; U; 4.1.2; zh-cn; Lenovo S920 Build/S820_S101_X;720*1280)*/
const Device UA2597086 = {S105582, 153, { &P11625,&P5768,&P24675,&P18092 }};	/*Useragent - Nokia5300/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia5300) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2607481 = {S105583, 65, { &P20203,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;LG;LG-E906;7.10;8862)*/
const Device UA2607486 = {S105584, 64, { &P18293,&P21697,&P18418,&P18092 }};	/*Useragent - Windows Phone Search (Windows Phone OS 7.10;ZTE;Tania;7.10;8862)*/
const Device UA2607512 = {S105585, 170, { &P21736,&P5629,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; LAVA iris 351 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/405*/
const Device UA2607513 = {S105586, 154, { &P21736,&P5629,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; LAVA iris 351 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2607514 = {S105587, 170, { &P21736,&P5629,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.5; en-us ; LAVA iris 351 Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/405*/
const Device UA2607515 = {S105588, 75, { &P21736,&P5629,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android2.3.5;Zh_cn;LAVA iris 351;320*480;)UCWEB7.8.0.95/139/444*/
const Device UA2607531 = {S105589, 150, { &P21792,&P5629,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-tw; GT-I9200 Build/GWK74) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2607534 = {S105590, 171, { &P21792,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-I9200 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/352*/
const Device UA2607539 = {S105591, 199, { &P21794,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-gb; GT-I9152 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +0100, pl*/
const Device UA2607553 = {S105592, 138, { &P21802,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; Maya M1 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2607935 = {S105593, 149, { &P13452,&P5096,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8/Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2608172 = {S105594, 183, { &P15919,&P18066,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, id*/
const Device UA2609464 = {S105595, 163, { &P17337,&P6546,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-us ; GT-P7500 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.2.139/145/355*/
const Device UA2610456 = {S105596, 164, { &P12879,&P6437,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7510 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.2.3.162/145/355*/
const Device UA2611320 = {S105597, 45, { &P13452,&P17979,&P22286,&P18092 }};	/*Useragent - TuneIn Radio/5.1 (Android 10; GT-P1000; Java)*/
const Device UA2611389 = {S105598, 134, { &P14908,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; es-us; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2611455 = {S105599, 164, { &P12889,&P6437,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7300 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2611752 = {S105600, 154, { &P17323,&P6546,&P4459,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 MobileSafari/534.13 DolphinHDCN/7.0.0*/
const Device UA2611782 = {S105601, 137, { &P14908,&P6437,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6210 Build/HTJ85B)Maxthon AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2611936 = {S105602, 183, { &P22041,&P18066,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; fr-fr; GT-P6810 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, fr*/
const Device UA2611975 = {S105603, 164, { &P22052,&P6437,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7501 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/355*/
const Device UA2612535 = {S105604, 151, { &P17337,&P6546,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2612877 = {S105605, 147, { &P15882,&P18066,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2614057 = {S105606, 69, { &P17168,&P6437,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android3.2;Zh_cn;GT-P7310;800*1232;)UCWEB7.8.0.95/139/352*/
const Device UA2614097 = {S105607, 91, { &P12889,&P6546,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.4.5.12736); (Linux; U; Android 3.1; GT-P7300 Build/HONEYCOMB_MR1);*/
const Device UA2614541 = {S105608, 130, { &P15876,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; pt-br; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615039 = {S105609, 147, { &P12879,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615149 = {S105610, 45, { &P13452,&P17979,&P22285,&P18092 }};	/*Useragent - TuneIn Radio/4.1 (Android 10; GT-P1000; Java)*/
const Device UA2615232 = {S105611, 101, { &P15932,&P17979,&P22070,&P18092 }};	/*Useragent - Instagram 3.0.5 Android (15/4.0.3; 160dpi; 600x976; samsung; GT-P3110; espressowifi; espresso; en_GB)*/
const Device UA2615321 = {S105612, 110, { &P17337,&P17979,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.0000.7386(Android;Android (GT-P7500-GT-P7500-user-3.1);it-IT;kml:2.2;client:Free;type:default)*/
const Device UA2615613 = {S105613, 136, { &P15985,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; vi-vn; GT-P6200 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.77  Safari/535.7*/
const Device UA2615624 = {S105614, 145, { &P15985,&P6437,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2615717 = {S105615, 140, { &P22041,&P18066,&P20791,&P18092 }};	/*Useragent - MQQBrowser/4.0/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GT-P6810 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1*/
const Device UA2615748 = {S105616, 145, { &P15985,&P18066,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2615880 = {S105617, 174, { &P12879,&P6437,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7510 Build/HTJ85B; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.4.145/139/800*/
const Device UA2616087 = {S105618, 203, { &P17323,&P22212,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; pt-br; GT-P6800 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.6/175,1.9.6.175,2012-10-11 17:42, br*/
const Device UA2616201 = {S105619, 132, { &P22042,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P3108 Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2616189 = {S105620, 181, { &P17337,&P6437,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ar-ae; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, sa*/
const Device UA2616275 = {S105621, 142, { &P12889,&P6546,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2616314 = {S105622, 145, { &P17337,&P18066,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616407 = {S105623, 135, { &P15876,&P17979,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 10.2.1.5; GT-P5100 Build/IML74K) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2616426 = {S105624, 145, { &P17337,&P6437,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; ms-my; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2616467 = {S105625, 45, { &P15876,&P18066,&P22291,&P18092 }};	/*Useragent - TuneIn Radio/6.5 (Android 15; GT-P5100; Java)*/
const Device UA2616475 = {S105626, 166, { &P17168,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P7310 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2616576 = {S105627, 183, { &P17168,&P18111,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; cs-cz; GT-P7310 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, cz*/
const Device UA2616593 = {S105628, 145, { &P14908,&P6437,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6210 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2616614 = {S105629, 91, { &P12889,&P6437,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.4.5.12736); (Linux; U; Android 3.2; GT-P7300 Build/HONEYCOMB_MR2);*/
const Device UA2616631 = {S105630, 112, { &P12879,&P17979,&P18208,&P18092 }};	/*Useragent - GoogleEarth/6.2.0001.6363(Android;Android (GT-P7510-GT-P7510-user-4.0.4);en-US;kml:2.2;client:Free;type:default)*/
const Device UA2616639 = {S105631, 144, { &P17337,&P6546,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2616645 = {S105632, 136, { &P22041,&P18066,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6810 Build/IMM76D) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7*/
const Device UA2616750 = {S105633, 183, { &P14905,&P18066,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P7100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.8.4/104,1.8.4.104,2012-06-25 20:49, de*/
const Device UA2616793 = {S105634, 147, { &P15919,&P18066,&P20779,&P18092 }};	/*Useragent - MQQBrowser/2.8 Mozilla/5.0 (Linux; U; Android 4.0.3; ar-ae; GT-P3100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2616863 = {S105635, 284, { &P12879,&P18066,&P20444,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2a1pre) Gecko/X Firefox/3.8a1pre AppEngine CirruxCache CoralWebPrxMozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30,gzip(gfe) (via docs.google.com/viewer)*/
const Device UA2617019 = {S105636, 156, { &P17337,&P18698,&P15760,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; vi-vn; GT-P7500 Build/GRK39F; CyanogenMod-7.2.0) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2617045 = {S105637, 164, { &P17168,&P6437,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7310 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2617083 = {S105638, 95, { &P17931,&P17979,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 Android (8/2.2; 240dpi; 600x1024; samsung; SGH-T849; SGH-T849; GT-P1000; en_US)*/
const Device UA2617214 = {S105639, 147, { &P15876,&P18066,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617387 = {S105640, 139, { &P22037,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P3113 Build/IMM76D)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2617661 = {S105641, 181, { &P12889,&P6437,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, zm*/
const Device UA2617865 = {S105642, 151, { &P14908,&P5096,&P20781,&P18092 }};	/*Useragent - MQQBrowser/3.0/Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2617908 = {S105643, 129, { &P12889,&P5096,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P7300 Build/FROYO) UC AppleWebKit/534.31 (KHTML, like Gecko) Mobile Safari/534.31*/
const Device UA2617976 = {S105644, 151, { &P14908,&P5096,&P20787,&P18092 }};	/*Useragent - MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2618173 = {S105645, 148, { &P13452,&P5096,&P20771,&P18092 }};	/*Useragent - MQQBrowser/20 Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2618183 = {S105646, 157, { &P13452,&P5629,&P20778,&P18092 }};	/*Useragent - MQQBrowser/2.7 Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2618204 = {S105647, 90, { &P17168,&P17979,&P22072,&P18092 }};	/*Useragent - Instagram 3.2.0 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7310; GT-P7310; p3; en_US)*/
const Device UA2618275 = {S105648, 147, { &P15882,&P18066,&P20787,&P18092 }};	/*Useragent - MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2618530 = {S105649, 125, { &P15919,&P17979,&P3553,&P18092 }};	/*Useragent - Android_CellLocation_2.0.X_KjDdHjNlNlNfLpObCmOoGkJcIjMbDcIi@%E4%B8%96%E7%BA%AA%E4%BD%B3%E7%BC%98_3.11;X;X;samsung;GT-P3100;15*/
const Device UA2618546 = {S105650, 139, { &P15932,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; ru-ru; GT-P3110 Build/IMM76D)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2618740 = {S105651, 144, { &P12889,&P6546,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2618810 = {S105652, 166, { &P12879,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P7510 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2618881 = {S105653, 46, { &P17337,&P6546,&P22290,&P18092 }};	/*Useragent - TuneIn Radio/6.3 (Android 12; GT-P7500M; Java)*/
const Device UA2618888 = {S105654, 154, { &P17337,&P17979,&P3508,&P18092 }};	/*Useragent - Rdio.Net; Android/2.2.11; Android/4.0.4; DM=samsung; DN=GT-P7500R; DC=samsung; DF=samsung/GT-P7500R/GT-P7500R:4.0.4/IMM76D/RUXLH2:user/release-keys; DH=p3*/
const Device UA2618892 = {S105655, 149, { &P13452,&P5096,&P20787,&P18092 }};	/*Useragent - MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 2.2; zh-cn; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2618947 = {S105656, 145, { &P18478,&P6437,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6201 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2619010 = {S105657, 147, { &P12879,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7510 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2619046 = {S105658, 151, { &P17337,&P6546,&P20787,&P18092 }};	/*Useragent - MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2619287 = {S105659, 129, { &P12889,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-gb; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2619309 = {S105660, 95, { &P14908,&P5629,&P18154,&P18092 }};	/*Useragent - YahooMobileMessenger/1.0 (Android Mail; 1.4.6) (GT-P1010; samsung; GT-P1010; 2.3.6/GINGERBREAD)*/
const Device UA2619393 = {S105661, 147, { &P15919,&P18066,&P20787,&P18092 }};	/*Useragent - MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2619397 = {S105662, 148, { &P22052,&P22212,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7501 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2619477 = {S105663, 146, { &P15919,&P18066,&P20776,&P18092 }};	/*Useragent - MQQBrowser/25 Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2619641 = {S105664, 71, { &P15882,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.3;Zh_cn;GT-P5110;800*1232;)UCWEB7.8.0.95/139/444*/
const Device UA2619660 = {S105665, 45, { &P22043,&P18066,&P22293,&P18092 }};	/*Useragent - TuneIn Radio/6.7 (Android 15; GT-P5113; Java)*/
const Device UA2619737 = {S105666, 150, { &P22230,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P3200 Build/Smartfren.androtab.v25.2) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2619744 = {S105667, 183, { &P15932,&P18066,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-05 07:52, gb*/
const Device UA2619892 = {S105668, 46, { &P17337,&P18066,&P22292,&P18092 }};	/*Useragent - TuneIn Radio/6.6 (Android 15; GT-P7500R; Java)*/
const Device UA2619946 = {S105669, 166, { &P13452,&P18149,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P1000 Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2620045 = {S105670, 166, { &P22041,&P18066,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6810 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2620176 = {S105671, 145, { &P15919,&P18066,&P21760,&P18092 }};	/*Useragent - iBrowser/2.8 Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P3100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2620378 = {S105672, 45, { &P12879,&P18066,&P22293,&P18092 }};	/*Useragent - TuneIn Radio/6.7 (Android 15; GT-P7510; Java)*/
const Device UA2620547 = {S105673, 177, { &P15985,&P6437,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-gb; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57*/
const Device UA2620601 = {S105674, 50, { &P13452,&P17979,&P22299,&P18092 }};	/*Useragent - TuneIn Radio Pro/6.6 (Android 10; GT-P1000M; Java)*/
const Device UA2620725 = {S105675, 145, { &P12879,&P6437,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2620783 = {S105676, 148, { &P17337,&P17979,&P4861,&P18092 }};	/*Useragent - Rdio.Net; Android/2.1.4; Android/3.2; DM=samsung; DN=GT-P7500R; DC=samsung; DF=samsung/GT-P7500R/GT-P7500R:3.2/HTJ85B/UXKK2:user/release-keys; DH=p3*/
const Device UA2620873 = {S105677, 45, { &P22037,&P18066,&P22293,&P18092 }};	/*Useragent - TuneIn Radio/6.7 (Android 15; GT-P3113; Java)*/
const Device UA2620992 = {S105678, 118, { &P12879,&P24693,&P18112,&P18092 }};	/*Useragent - Logos/2.2.7 (Linux; U; Android 4.1.2; en-NZ; GT-P7510 Build/aokp_p4wifi-userdebug 4.1.2 JZO54K eng.trip.X.X test-keys)*/
const Device UA2621002 = {S105679, 106, { &P22043,&P17979,&P22071,&P18092 }};	/*Useragent - Instagram 3.1.0 Android (15/4.0.4; 160dpi; 800x1232; samsung; GT-P5113; espresso10wifi; espresso10; en_CA)*/
const Device UA2621157 = {S105680, 151, { &P14908,&P5096,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2.1; en-au; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2621403 = {S105681, 163, { &P13452,&P5629,&P3508,&P18092 }};	/*Useragent - Rdio.Net; Android/2.4.2; Android/2.2; DM=samsung; DN=GT-P1000M; DC=samsung; DF=samsung/GT-P1000M/GT-P1000M/GT-P1000M:2.2/FROYO/UGJJ7:user/release-keys; DH=GT-P1000*/
const Device UA2621420 = {S105682, 87, { &P22041,&P18066,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 4.0.4; GT-P6810 Build/Unknown);*/
const Device UA2621563 = {S105683, 49, { &P12879,&P18066,&P22298,&P18092 }};	/*Useragent - TuneIn Radio Pro/6.3 (Android 15; GT-P7510; Java)*/
const Device UA2621677 = {S105684, 164, { &P18478,&P6437,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6201 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.2.240/145/355*/
const Device UA2621817 = {S105685, 49, { &P22037,&P18066,&P22300,&P18092 }};	/*Useragent - TuneIn Radio Pro/6.7 (Android 15; GT-P3113; Java)*/
const Device UA2621917 = {S105686, 71, { &P12879,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.4;Zh_cn;GT-P7510;800*1232;)UCWEB7.8.0.95/139/351*/
const Device UA2621987 = {S105687, 132, { &P14908,&P17979,&P3508,&P18092 }};	/*Useragent - Orca-Android/2.2.2-release OS/3.2 Model/GT-P6210 VersionCode/88371 Product/Messenger Carrier/null Manufacturer/samsung Brand/samsung*/
const Device UA2621993 = {S105688, 90, { &P17168,&P17979,&P22073,&P18092 }};	/*Useragent - Instagram 3.3.2 Android (12/3.1; 160dpi; 800x1280; samsung; GT-P7310; GT-P7310; p3; en_GB)*/
const Device UA2622289 = {S105689, 45, { &P22037,&P18066,&P22292,&P18092 }};	/*Useragent - TuneIn Radio/6.6 (Android 15; GT-P3113; Java)*/
const Device UA2622383 = {S105690, 46, { &P17337,&P6546,&P22287,&P18092 }};	/*Useragent - TuneIn Radio/5.2 (Android 12; GT-P7500R; Java)*/
const Device UA2622485 = {S105691, 164, { &P22041,&P6437,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P6810 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.1.235/145/355*/
const Device UA2622558 = {S105692, 71, { &P12889,&P18066,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android4.0.4;Zh_cn;GT-P7300;800*1232;)UCWEB7.8.0.95/139/355*/
const Device UA2622604 = {S105693, 45, { &P12879,&P6437,&P22293,&P18092 }};	/*Useragent - TuneIn Radio/6.7 (Android 13; GT-P7510; Java)*/
const Device UA2622631 = {S105694, 101, { &P17323,&P17979,&P4861,&P18092 }};	/*Useragent - Orca-Android/2.1.2-release OS/3.2 Model/GT-P6800 VersionCode/78596 Product/Messenger Carrier/MY_MAXIS*/
const Device UA2622695 = {S105695, 197, { &P15985,&P18066,&P17374,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Opera/9.80 (X11; Linux zvav; U; en) Presto/2.8.119 Version/11.10*/
const Device UA2622703 = {S105696, 144, { &P17323,&P6437,&P20775,&P18092 }};	/*Useragent - MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 3.2; fa-fa; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2622716 = {S105697, 101, { &P22037,&P17979,&P22071,&P18092 }};	/*Useragent - Instagram 3.1.0 Android (15/4.0.4; 160dpi; 600x976; samsung; GT-P3113; espressowifi; espresso; en_CA)*/
const Device UA2622914 = {S105698, 45, { &P22043,&P18066,&P22290,&P18092 }};	/*Useragent - TuneIn Radio/6.3 (Android 15; GT-P5113; Java)*/
const Device UA2623016 = {S105699, 151, { &P17168,&P6546,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7310 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13*/
const Device UA2623035 = {S105700, 180, { &P22037,&P22212,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3113 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P3113, 38, 1.8.1)*/
const Device UA2623068 = {S105701, 168, { &P15932,&P18066,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-au; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]*/
const Device UA2623156 = {S105702, 183, { &P14908,&P18066,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6210 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.8.4/111,1.8.4.111,2012-07-09 19:03, us*/
const Device UA2623233 = {S105703, 145, { &P22041,&P6437,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 3.2; en-au; GT-P6810 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2623274 = {S105704, 136, { &P17168,&P5477,&P4861,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.1; en-gb; GT-P7310 Build/ERD62) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2623342 = {S105705, 175, { &P17337,&P18066,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7500 Build/IMM76D; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/355*/
const Device UA2623471 = {S105706, 49, { &P22043,&P18066,&P22299,&P18092 }};	/*Useragent - TuneIn Radio Pro/6.6 (Android 15; GT-P5113; Java)*/
const Device UA2623477 = {S105707, 178, { &P22043,&P5629,&P18062,&P18092 }};	/*Useragent - Rdio.Net; Android/2.4.4; Android/4.0.4; DM=samsung; DN=GT-P5113; DC=samsung; DF=samsung/espresso10wifibby/espresso10wifi:4.0.4/IMM76D/P5113UEBLH3:user/release-keys; DH=espresso10*/
const Device UA2624069 = {S105708, 166, { &P15932,&P18111,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us ; GT-P3110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/355*/
const Device UA2624081 = {S105709, 136, { &P15932,&P18111,&P18162,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GT-P3110 Build/JRO03C) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75  Safari/535.7*/
const Device UA2624089 = {S105710, 150, { &P22043,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P5113 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2624241 = {S105711, 61, { &P22039,&P18066,&P18062,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.0.4; GT-P2000 Build/IMM76D)*/
const Device UA2624304 = {S105712, 45, { &P22037,&P18066,&P22291,&P18092 }};	/*Useragent - TuneIn Radio/6.5 (Android 15; GT-P3113; Java)*/
const Device UA2624450 = {S105713, 163, { &P13452,&P22212,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P1000 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2624464 = {S105714, 78, { &P17168,&P17979,&P3553,&P18092 }};	/*Useragent - Android_CellLocation_2.0.com.qihoo360.mobilesafe;null;null;samsung;GT-P7310;15*/
const Device UA2624725 = {S105715, 75, { &P22041,&P17979,&P18062,&P18092 }};	/*Useragent - Evernote Android/AndroidMarket_442_189 (en_US); Android/4.0.4; GT-P6810/15;*/
const Device UA2624751 = {S105716, 94, { &P18475,&P17979,&P22073,&P18092 }};	/*Useragent - Instagram 3.3.0 Android (15/4.0.4; 160dpi; 800x1232; samsung; GT-P7320; GT-P7320; qcom; en_US)*/
const Device UA2624826 = {S105717, 147, { &P15882,&P18111,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2624837 = {S105718, 214, { &P15876,&P18149,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P5100 Build/IMM76L; CyanogenMod-9) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0000, gb*/
const Device UA2624987 = {S105719, 142, { &P17337,&P6546,&P21753,&P18092 }};	/*Useragent - iBrowser/3.0/Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2625088 = {S105720, 142, { &P17337,&P6546,&P21750,&P18092 }};	/*Useragent - iBrowser/2.7 Mozilla/5.0 (Linux; U; Android 3.1; en-gb; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2625110 = {S105721, 223, { &P22052,&P18149,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P7501 Build/IMM76L; CyanogenMod-Ganbarou V1.00 GT) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.19/283,1.9.19.283,2013-01-11 20:13, +0100, de*/
const Device UA2625336 = {S105722, 67, { &P22037,&P17979,&P4861,&P18092 }};	/*Useragent - eHarmony-Android/2.1.8 (GT-P3113; Android OS 4.1.1; en_CA; id null)*/
const Device UA2625441 = {S105723, 147, { &P22043,&P18066,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P5113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2625468 = {S105724, 208, { &P15882,&P22212,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; nl-nl; GT-P5110 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.17/277,1.9.17.277,2012-12-26 18:15, +0100, nl*/
const Device UA2625868 = {S105725, 147, { &P15932,&P18066,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2625878 = {S105726, 147, { &P22042,&P18066,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2626344 = {S105727, 45, { &P22037,&P18111,&P22293,&P18092 }};	/*Useragent - TuneIn Radio/6.7 (Android 16; GT-P3113; Java)*/
const Device UA2626601 = {S105728, 49, { &P22037,&P18111,&P22301,&P18092 }};	/*Useragent - TuneIn Radio Pro/6.8 (Android 16; GT-P3113; Java)*/
const Device UA2626668 = {S105729, 45, { &P22043,&P18111,&P22293,&P18092 }};	/*Useragent - TuneIn Radio/6.7 (Android 16; GT-P5113; Java)*/
const Device UA2626754 = {S105730, 152, { &P17337,&P18066,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7500 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2626756 = {S105731, 150, { &P12879,&P6437,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-tw; GT-P7510 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2626836 = {S105732, 167, { &P13452,&P24694,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; en-us ; GT-P1000 Build/MIUI-XJ) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2626875 = {S105733, 150, { &P12889,&P6437,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-tw; GT-P7300 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2626934 = {S105734, 147, { &P15985,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2627029 = {S105735, 152, { &P13452,&P18066,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/GINGERBREAD) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2627098 = {S105736, 168, { &P17337,&P18066,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P7500 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]*/
const Device UA2627123 = {S105737, 130, { &P15932,&P17979,&P3508,&P18092 }};	/*Useragent - Orca-Android/2.2.5-release OS/4.1.1 Model/GT-P3110 VersionCode/X Product/Messenger Carrier/null Manufacturer/samsung Brand/samsung*/
const Device UA2627137 = {S105738, 45, { &P12879,&P6437,&P22289,&P18092 }};	/*Useragent - TuneIn Radio/6.1 (Android 13; GT-P7510; Java)*/
const Device UA2627145 = {S105739, 46, { &P17337,&P6437,&P22290,&P18092 }};	/*Useragent - TuneIn Radio/6.3 (Android 13; GT-P7500R; Java)*/
const Device UA2627344 = {S105740, 169, { &P22052,&P22211,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7501 Build/JOP40D; CyanogenMod-Ganbarou V3.00 GT-P7501) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2627464 = {S105741, 145, { &P17323,&P6437,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2627624 = {S105742, 166, { &P17337,&P18066,&P18516,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P7500 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.3.0.143/145/444*/
const Device UA2627702 = {S105743, 166, { &P15919,&P18598,&P18520,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; GT-P3100 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.5.3.246/145/355*/
const Device UA2627720 = {S105744, 193, { &P14908,&P5629,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; es-es; GT-P1010 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, es*/
const Device UA2627930 = {S105745, 170, { &P15876,&P18111,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-US; GT-P5100 Build/JRO03C) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2627948 = {S105746, 145, { &P15985,&P6437,&P20784,&P18092 }};	/*Useragent - MQQBrowser/3.3/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2628032 = {S105747, 147, { &P17337,&P5629,&P4459,&P18092 }};	/*Useragent - Rdio.Net; Android/2.4.9; Android/3.1; DM=samsung; DN=GT-P7500R; DC=samsung; DF=samsung/GT-P7500R/GT-P7500R:3.1/HMJ37/UXKH4:user/release-keys; DH=p3*/
const Device UA2628109 = {S105748, 136, { &P12889,&P17979,&P3508,&P18092 }};	/*Useragent - Orca-Android/2.2.1-release OS/3.1 Model/GT-P7300 VersionCode/87403 Product/Messenger Carrier/MY_MAXIS Manufacturer/samsung Brand/samsung*/
const Device UA2628122 = {S105749, 58, { &P20171,&P17979,&P20428,&P18092 }};	/*Useragent - com.kakao.talk/Android Version/2.5.0 Model/GT-P1000R MNC/X*/
const Device UA2628127 = {S105750, 152, { &P15882,&P18111,&P20791,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P5110 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile MQQBrowser/4.0 Safari/533.1*/
const Device UA2628505 = {S105751, 180, { &P17337,&P22212,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7500 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7500, 39, 1.8.2)*/
const Device UA2628781 = {S105752, 144, { &P12889,&P6546,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2628804 = {S105753, 174, { &P14908,&P5096,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-cn; GT-P1010 Build/FROYO; 600*1024) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/352*/
const Device UA2628903 = {S105754, 152, { &P22042,&P18066,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2628950 = {S105755, 87, { &P13452,&P18066,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.4.6.13904); (Linux; U; Android 4.0.4; GT-P1000 Build/Unknown);*/
const Device UA2628954 = {S105756, 156, { &P17323,&P18066,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P6800 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2628990 = {S105757, 211, { &P17168,&P18149,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-tw; GT-P7310 Build/IMM76I; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.20/286,1.9.20.286,2013-01-25 13:38, +0800, tw*/
const Device UA2629047 = {S105758, 147, { &P22042,&P18066,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2629216 = {S105759, 147, { &P15876,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5100 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2629227 = {S105760, 151, { &P17337,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7500 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2629331 = {S105761, 157, { &P13452,&P24694,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; GT-P1000 Build/MIUI-XJ) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Maxthon/4.0.3.3000*/
const Device UA2629440 = {S105762, 145, { &P18478,&P6437,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6201 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2629566 = {S105763, 49, { &P12879,&P18066,&P22297,&P18092 }};	/*Useragent - TuneIn Radio Pro/6.1 (Android 15; GT-P7510; Java)*/
const Device UA2629628 = {S105764, 166, { &P17337,&P18598,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; GT-P7500 Build/JOP40G) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2629749 = {S105765, 187, { &P13452,&P18149,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-au; GT-P1000 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]*/
const Device UA2629753 = {S105766, 165, { &P15932,&P22211,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P3110 Build/JOP40G; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2629763 = {S105767, 151, { &P15932,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; zh-cn; GT-P3110 Build/IML74K) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2629778 = {S105768, 49, { &P12879,&P18066,&P22294,&P18092 }};	/*Useragent - TuneIn Radio Pro/5.2 (Android 15; GT-P7510; Java)*/
const Device UA2629812 = {S105769, 210, { &P12889,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; pl-pl; GT-P7300 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0100, pl*/
const Device UA2629821 = {S105770, 156, { &P15919,&P18066,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P3100 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2629843 = {S105771, 139, { &P15985,&P18066,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; uk-ua; GT-P6200 Build/IMM76D)Maxthon AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2629863 = {S105772, 166, { &P12879,&P5629,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.1; en-us ; GT-P7510 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2629869 = {S105773, 166, { &P15876,&P18598,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us ; GT-P5100 Build/JOP40D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/352*/
const Device UA2630021 = {S105774, 166, { &P13452,&P18111,&P18524,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us ; GT-P1000 Build/JZO54K) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.4.1.204/145/444*/
const Device UA2630061 = {S105775, 63, { &P15932,&P17979,&P18062,&P18092 }};	/*Useragent - com.eurosport.news.android/3.1.3/Android/4.0.3/samsung-GT-P3110*/
const Device UA2630167 = {S105776, 147, { &P15882,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2630250 = {S105777, 149, { &P17337,&P5629,&P16179,&P18092 }};	/*Useragent - Rdio.Net; Android/2.4.10; Android/3.2; DM=samsung; DN=GT-P7500R; DC=samsung; DF=samsung/GT-P7500R/GT-P7500R:3.2/HTJ85B/UXKK2:user/release-keys; DH=p3*/
const Device UA2630316 = {S105778, 106, { &P22043,&P17979,&P22070,&P18092 }};	/*Useragent - Instagram 3.0.5 Android (15/4.0.4; 160dpi; 800x1232; samsung; GT-P5113; espresso10wifi; espresso10; en_CA)*/
const Device UA2630322 = {S105779, 145, { &P12889,&P6437,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 3.2; fa-fa; GT-P7300 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2630596 = {S105780, 180, { &P22043,&P22212,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P5113 Build/JRO03L; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P5113, 40, 1.8.5)*/
const Device UA2630642 = {S105781, 151, { &P22037,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3113 Build/JRO03C) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2630708 = {S105782, 147, { &P17323,&P18066,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2630751 = {S105783, 149, { &P20149,&P6437,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6211 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2630800 = {S105784, 151, { &P17323,&P6437,&P20785,&P18092 }};	/*Useragent - MQQBrowser/3.4/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 MobileSafari/534.13*/
const Device UA2630868 = {S105785, 144, { &P12879,&P6546,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.1; en-us; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2631029 = {S105786, 208, { &P22037,&P22212,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3113 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, -0600, us*/
const Device UA2631161 = {S105787, 69, { &P12879,&P6546,&P24662,&P18092 }};	/*Useragent - JUC(Linux;U;Android3.1;Zh_cn;GT-P7510;1280*800;)UCWEB7.8.0.95/139/351*/
const Device UA2631179 = {S105788, 114, { &P15919,&P17979,&P20429,&P18092 }};	/*Useragent - GoogleEarth/7.0.0002.8421(Android;Android (GT-P3100-espressorf-user-4.0.3);it-IT;kml:2.2;client:Free;type:default)*/
const Device UA2631415 = {S105789, 148, { &P12889,&P6546,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; es-es; GT-P7300 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Maxthon/4.0.3.3000*/
const Device UA2631422 = {S105790, 147, { &P15932,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2631511 = {S105791, 169, { &P14905,&P22211,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; ru-ru; GT-P7100 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2631614 = {S105792, 192, { &P13452,&P18598,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P1000 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.23/294,1.9.23.294,2013-02-07 14:38, +0700, th*/
const Device UA2631672 = {S105793, 151, { &P13452,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-es; GT-P1000 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2631727 = {S105794, 149, { &P12889,&P6437,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7300 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2631767 = {S105795, 183, { &P15882,&P18111,&P22244,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; fr-fr; GT-P5110 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.8.4/101,1.8.4.101,2012-06-22 01:57, fr*/
const Device UA2632026 = {S105796, 90, { &P12879,&P17979,&P22068,&P18092 }};	/*Useragent - Instagram 1.0.6 Android (13/3.2; 160dpi; 800x1232; samsung; GT-P7510; GT-P7510; p3; en_US)*/
const Device UA2632131 = {S105797, 152, { &P15985,&P18066,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ph; GT-P6200 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2632251 = {S105798, 145, { &P17168,&P6437,&P20783,&P18092 }};	/*Useragent - MQQBrowser/3.2/Mozilla/5.0 (Linux; U; Android 3.2; ru-ru; GT-P7310 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2632279 = {S105799, 190, { &P22052,&P6437,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-P7501 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.20/286,1.9.20.286,2013-01-25 13:38, +0100, de*/
const Device UA2632317 = {S105800, 151, { &P12879,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7510 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2632328 = {S105801, 59, { &P17323,&P17979,&P22278,&P18092 }};	/*Useragent - theScore/3.1.2 Android-OS/4.0.4 (Android; Tablet, GT-P6800)*/
const Device UA2632351 = {S105802, 148, { &P12879,&P6546,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7510 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2632363 = {S105803, 167, { &P12879,&P6546,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; en-US; GT-P7510 Build/HMJ37) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2632385 = {S105804, 169, { &P15882,&P22211,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-P5110 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2632387 = {S105805, 152, { &P22037,&P18066,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-ca; GT-P3113 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2632396 = {S105806, 169, { &P17323,&P22211,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; ar-eg; GT-P6800 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2632454 = {S105807, 150, { &P17337,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7500 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2632455 = {S105808, 210, { &P14908,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; ru-ru; GT-P6210 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0000, ru*/
const Device UA2632541 = {S105809, 182, { &P12879,&P22211,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-us; GT-P7510 Build/JOP40D; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7510, 40, 1.8.5)*/
const Device UA2632650 = {S105810, 151, { &P22043,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-us; GT-P5113 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2632676 = {S105811, 168, { &P17168,&P22211,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-nz; GT-P7310 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2632693 = {S105812, 175, { &P15932,&P18111,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3110 Build/JRO03C; 1024*552) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444*/
const Device UA2632811 = {S105813, 147, { &P22042,&P18066,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P3108 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2632830 = {S105814, 85, { &P14908,&P5096,&P21567,&P18092 }};	/*Useragent - YahooMobile/1.0 (mail; 1.3.9.10087); (Linux; U; Android 2.2.1; GT-P1010 Build/FROYO);*/
const Device UA2632964 = {S105815, 156, { &P12879,&P6546,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13 UCBrowser/2.2.2.259*/
const Device UA2633025 = {S105816, 153, { &P17337,&P6546,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-CN; GT-P7500 Build/HMJ37) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2633080 = {S105817, 167, { &P15919,&P22212,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P3100 Build/JZO54K; CyanogenMod-10.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2633205 = {S105818, 175, { &P15882,&P18066,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P5110 Build/IMM76D; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.0.94/139/444*/
const Device UA2633579 = {S105819, 44, { &P13452,&P5096,&P22284,&P18092 }};	/*Useragent - TuneIn Radio/4.0 (Android 8; GT-P1000; Java)*/
const Device UA2633617 = {S105820, 174, { &P14908,&P5629,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.6; en-gb; GT-P1010 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1TeamStream (GT-P1010, 40, 1.8.5)*/
const Device UA2633718 = {S105821, 151, { &P18478,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; de-de; GT-P6201 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.3.3000*/
const Device UA2633936 = {S105822, 176, { &P17323,&P18066,&P24663,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D; 800*1232) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1/UCWEB7.9.3.103/139/800*/
const Device UA2633963 = {S105823, 164, { &P18475,&P6437,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-au; GT-P7320T Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13TeamStream (GT-P7320T, 40, 1.8.5)*/
const Device UA2633966 = {S105824, 156, { &P15882,&P18066,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-CN; GT-P5110 Build/IMM76D) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2634043 = {S105825, 171, { &P13452,&P18149,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IMM76L; CyanogenMod-9.0.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.1.232*/
const Device UA2634045 = {S105826, 146, { &P13452,&P5096,&P21746,&P18092 }};	/*Useragent - iBrowser/26 Mozilla/5.0 (Linux; U; Android 2.2; fa-fa; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2634107 = {S105827, 151, { &P22043,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P5113 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2634249 = {S105828, 150, { &P12889,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-P7300 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2634275 = {S105829, 163, { &P13452,&P22212,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-P1000 Build/JZO54K; CyanogenMod-10) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2634321 = {S105830, 152, { &P17323,&P18066,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2634334 = {S105831, 151, { &P17168,&P18066,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7310 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2634349 = {S105832, 148, { &P17337,&P6546,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2634368 = {S105833, 149, { &P17337,&P6437,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P7500 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2634448 = {S105834, 150, { &P17337,&P6546,&P20786,&P18092 }};	/*Useragent - MQQBrowser/3.5/Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7500 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 MobileSafari/534.13*/
const Device UA2634505 = {S105835, 138, { &P22039,&P18066,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P2000 Build/IMM76D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Mobile Safari/537.22*/
const Device UA2634533 = {S105836, 160, { &P12879,&P6437,&P24406,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P7510 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 OverDrive Media Console/2.6.4*/
const Device UA2634609 = {S105837, 143, { &P17337,&P6437,&P21760,&P18092 }};	/*Useragent - iBrowser/2.8 Mozilla/5.0 (Linux; U; Android 3.2; fa-fa; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2634857 = {S105838, 58, { &P17337,&P17979,&P22280,&P18092 }};	/*Useragent - theScore/3.2.1 Android-OS/3.1 (Android; Tablet, GT-P7500M)*/
const Device UA2634872 = {S105839, 160, { &P14908,&P6437,&P24406,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us; GT-P6210 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 OverDrive Media Console/2.6.4*/
const Device UA2634888 = {S105840, 165, { &P15932,&P18598,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-us ; GT-P3110 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2634916 = {S105841, 156, { &P15932,&P18111,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-P3110 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2634952 = {S105842, 147, { &P22037,&P18111,&P20788,&P18092 }};	/*Useragent - MQQBrowser/3.7/Mozilla/5.0 (Linux; U; Android 4.1.1; zh-cn; GT-P3113 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2635117 = {S105843, 146, { &P15876,&P18066,&P20775,&P18092 }};	/*Useragent - MQQBrowser/24 Mozilla/5.0 (Linux; U; Android 4.0.3; ar-ae; GT-P5100 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2635127 = {S105844, 49, { &P22037,&P18111,&P22299,&P18092 }};	/*Useragent - TuneIn Radio Pro/6.6 (Android 16; GT-P3113; Java)*/
const Device UA2635239 = {S105845, 166, { &P22041,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P6810 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2635303 = {S105846, 166, { &P12889,&P18066,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us ; GT-P7300 Build/IMM76D) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2635471 = {S105847, 164, { &P12879,&P18111,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; GT-P7510 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7510, 40, 1.8.5)*/
const Device UA2635516 = {S105848, 61, { &P22041,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.1; GT-P6810 Build/JOP40D)*/
const Device UA2635574 = {S105849, 228, { &P22052,&P22211,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; en-gb; GT-P7501 Build/JDQ39; CyanogenMod-Ganbarou V3.03 GT-P7501) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +0100, de*/
const Device UA2635584 = {S105850, 150, { &P17337,&P6437,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; ru-ru; GT-P7500 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 UCBrowser/2.2.2.259*/
const Device UA2635596 = {S105851, 147, { &P15876,&P18111,&P20782,&P18092 }};	/*Useragent - MQQBrowser/3.1/Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; GT-P5100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2635607 = {S105852, 151, { &P17323,&P6437,&P20780,&P18092 }};	/*Useragent - MQQBrowser/2.9/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 MobileSafari/534.13*/
const Device UA2635615 = {S105853, 151, { &P15876,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.1; GT-P5100 Build/JRO03C) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.53005*/
const Device UA2635735 = {S105854, 153, { &P17168,&P6546,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-CN; GT-P7310 Build/HMJ37) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2635912 = {S105855, 45, { &P22037,&P18111,&P22292,&P18092 }};	/*Useragent - TuneIn Radio/6.6 (Android 16; GT-P3113; Java)*/
const Device UA2636076 = {S105856, 151, { &P13452,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P1000 Build/JOP40D) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2636086 = {S105857, 153, { &P12879,&P6546,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-CN; GT-P7510 Build/HMJ37) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2636091 = {S105858, 149, { &P14908,&P6437,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6210 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2636093 = {S105859, 192, { &P18478,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P6201 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.27/381,1.9.27.381,2013-03-21 11:39, +0100, de*/
const Device UA2636145 = {S105860, 148, { &P17168,&P6546,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7310 Build/HMJ37) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2636208 = {S105861, 45, { &P22037,&P18111,&P22289,&P18092 }};	/*Useragent - TuneIn Radio/6.1 (Android 16; GT-P3113; Java)*/
const Device UA2636317 = {S105862, 145, { &P15985,&P6437,&P20787,&P18092 }};	/*Useragent - MQQBrowser/3.6/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13*/
const Device UA2636344 = {S105863, 156, { &P17323,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-P6800 Build/JRO03L) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2636363 = {S105864, 145, { &P12879,&P17979,&P3508,&P18092 }};	/*Useragent - Mozilla/5.0 (Android; (OS 4.0.4) (IM 28,684MB) (EM 28,684MB) (Build IMM76D.VLLH3) (Model GT-P7510)  (Brand samsung)) WW_Mobile_Android/2.2.15.509*/
const Device UA2636376 = {S105865, 150, { &P15919,&P18598,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-P3100 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2636428 = {S105866, 150, { &P15932,&P18598,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-cn; GT-P3110 Build/JDQ39) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2636469 = {S105867, 164, { &P17168,&P6437,&P18527,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; en-us ; GT-P7310 Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1/UCBrowser/8.6.1.262/145/355*/
const Device UA2636478 = {S105868, 151, { &P12879,&P18111,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-P7510 Build/JZO54K) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.53005*/
const Device UA2636503 = {S105869, 164, { &P15882,&P22211,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P5110 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2636543 = {S105870, 156, { &P22043,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-P5113 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2636570 = {S105871, 169, { &P15985,&P18066,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; es-us; GT-P6200L Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]*/
const Device UA2636719 = {S105872, 164, { &P12889,&P18066,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; pl-pl; GT-P7300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30TeamStream (GT-P7300, 42, 1.9.1)*/
const Device UA2636732 = {S105873, 149, { &P14908,&P5096,&P21755,&P18092 }};	/*Useragent - iBrowser/3.1/Mozilla/5.0 (Linux; U; Android 2.2.1; en-gb; GT-P1010 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1*/
const Device UA2636746 = {S105874, 168, { &P15876,&P18111,&P18122,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; ro-ro; GT-P5100 Build/JRO03C) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 [operamini] [UsableNet Lift Mobile]*/
const Device UA2636845 = {S105875, 155, { &P14908,&P5096,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.2.1; zh-CN; GT-P1010 Build/FROYO) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.2.274 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2636879 = {S105876, 153, { &P13452,&P24694,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 2.3.7; es-es; GT-P1000 Build/MIUI-XJ) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 Mobile Maxthon*/
const Device UA2636918 = {S105877, 155, { &P17168,&P6546,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.1; zh-cn; GT-P7310 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.13 Maxthon/4.0.4.1000*/
const Device UA2636925 = {S105878, 60, { &P14905,&P18598,&P19802,&P18092 }};	/*Useragent - Dalvik/1.6.0 (Linux; U; Android 4.2.2; GT-P7100 Build/JDQ39)*/
const Device UA2636998 = {S105879, 130, { &P22041,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P6810 Build/JOP40D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Safari/537.31*/
const Device UA2637003 = {S105880, 166, { &P13452,&P18149,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P1000 Build/IMM76L; CyanogenMod-9.1.0) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2637004 = {S105881, 192, { &P17323,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; en-us; GT-P6800 Build/JRO03H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.27/381,1.9.27.381,2013-03-21 11:39, +0800, hk*/
const Device UA2637023 = {S105882, 156, { &P22037,&P18111,&P19807,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-P3113 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.8.3.278 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2637084 = {S105883, 150, { &P17323,&P6437,&P20769,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6800 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 UCBrowser/2.3.1.257*/
const Device UA2637105 = {S105884, 149, { &P17323,&P6437,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.2; GT-P6800 Build/HTJ85B) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.52315*/
const Device UA2637179 = {S105885, 199, { &P18475,&P18066,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; zh-cn; GT-P7320 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.27/383,1.9.27.383,2013-03-25 21:34, +0800, china, cn*/
const Device UA2637300 = {S105886, 164, { &P15919,&P22211,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.2.2; en-us; GT-P3100 Build/JDQ39; CyanogenMod-10.1) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2637320 = {S105887, 162, { &P22041,&P18066,&P24406,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GT-P6810 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 OverDrive Media Console/2.6.4*/
const Device UA2637365 = {S105888, 136, { &P15985,&P17979,&P3508,&P18092 }};	/*Useragent - Orca-Android/2.2.3-release OS/3.2 Model/GT-P6200 VersionCode/94668 Product/Messenger Carrier/MY_MAXIS Manufacturer/samsung Brand/samsung*/
const Device UA2637444 = {S105889, 132, { &P14908,&P18598,&P19802,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.1; en-ca; GT-P6210 Build/JOP40D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2637447 = {S105890, 156, { &P15876,&P18111,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.1; zh-CN; GT-P5100 Build/JRO03C) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2637451 = {S105891, 170, { &P13452,&P18066,&P18326,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-US; GT-P1000 Build/IMM76L) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1 UCBrowser/8.2.0.242 Mobile*/
const Device UA2637592 = {S105892, 147, { &P17323,&P18066,&P21728,&P18092 }};	/*Useragent - OneBrowser/3.5/Mozilla/5.0 (Linux; U; Android 4.0.4; en-gb; GT-P6800 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30*/
const Device UA2637614 = {S105893, 186, { &P19852,&P6437,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 3.2; de-de; GT-P7511 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13 Flipboard/1.9.27/381,1.9.27.381,2013-03-21 11:39, +0100*/
const Device UA2637641 = {S105894, 130, { &P14908,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; GT-P6210 Build/JOP40G) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.40 Safari/537.31*/
const Device UA2637729 = {S105895, 192, { &P13452,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-au; GT-P1000 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Flipboard/1.9.26/365,1.9.26.365,2013-02-27 19:42, +1100, au*/
const Device UA2637775 = {S105896, 148, { &P12889,&P6546,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.1; GT-P7300 Build/HMJ37) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.53463*/
const Device UA2637868 = {S105897, 162, { &P15882,&P18066,&P24406,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.3; en-gb; GT-P5110 Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 OverDrive Media Console/2.6.4*/
const Device UA2637879 = {S105898, 151, { &P12889,&P18066,&P22377,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; it-it; GT-P7300 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30 Maxthon/4.0.4.1000*/
const Device UA2637903 = {S105899, 149, { &P12889,&P6437,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 3.2; GT-P7300 Build/HTJ85B) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.123 Safari/537.22 OPR/14.0.1025.53005*/
const Device UA2638021 = {S105900, 151, { &P15985,&P6437,&P20780,&P18092 }};	/*Useragent - MQQBrowser/2.9/Mozilla/5.0 (Linux; U; Android 3.2; zh-cn; GT-P6200 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 MobileSafari/534.13*/
const Device UA2662307 = {S105901, 92, { &P11679,&P5776,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6630) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2676267 = {S105902, 142, { &P19862,&P18066,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; Micromax A110 Build/IMM76D) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2680950 = {S105903, 157, { &P22385,&P18111,&P18529,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-CN; GT-I8262D Build/JZO54K) AppleWebKit/534.31 (KHTML, like Gecko) UCBrowser/8.7.5.267 U3/0.8.0 Mobile Safari/534.31*/
const Device UA2681012 = {S105904, 94, { &P22385,&P18111,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0 (Linux; U; Adr 4.1.2; zh-CN; GT-I8262D) U2/1.0.0 UCBrowser/8.8.1.258 U2/1.0.0 Mobile*/
const Device UA2681016 = {S105905, 199, { &P22382,&P18111,&P22243,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; fr-fr; GT-I8262 Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/1.9.27/381,1.9.27.381,2013-03-21 11:39, +0000, sn*/
const Device UA2697831 = {S105906, 89, { &P14422,&P17483,&P24673,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-us; lg-p520) U2/1.0.0 UCBrowser/8.7.1.234 U2/1.0.0 Mobile*/
const Device UA2719588 = {S105907, 140, { &P22655,&P18598,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C5503 Build/10.3.A.0.317) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.68 Mobile Safari/537.36*/
const Device UA2719702 = {S105908, 141, { &P22655,&P18598,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C5503 Build/10.3.A.0.232) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19*/
const Device UA2719711 = {S105909, 156, { &P22654,&P18111,&P21419,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; C5502 Build/10.1.1.A.1.164) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.1 Mobile Safari/533.1*/
const Device UA2719719 = {S105910, 156, { &P22654,&P18111,&P22589,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; C5502 Build/10.1.1.A.1.273) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/4.2 Mobile Safari/533.1*/
const Device UA2748168 = {S105911, 164, { &P19862,&P18066,&P20768,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; Micromax A110 Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 UCBrowser/2.2.2.259*/
const Device UA2778995 = {S105912, 150, { &P23526,&P18066,&P18062,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.0.4; en-us; GFIVE Fararee (A78) Build/IMM76D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2779737 = {S105913, 92, { &P11766,&P5776,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia3230) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2784671 = {S105914, 131, { &P22920,&P18111,&P22234,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; WL-101GQC Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36*/
const Device UA2845324 = {S105915, 139, { &P15802,&P18111,&P18112,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.1.2; en-us; Amaze 4G Build/JZO54K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30*/
const Device UA2848410 = {S105916, 129, { &P23316,&P18066,&P18338,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; NC10+ Build/IMM76D) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166  Safari/535.19*/
const Device UA2849094 = {S105917, 91, { &P12165,&P17484,&P24675,&P18092 }};	/*Useragent - UCWEB/2.0(Java; U; MIDP-2.0; en-US; gt-s5230w) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA2877950 = {S105918, 140, { &P23213,&P18598,&P21169,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; C6806 Build/14.1.B.1.277) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31*/
const Device UA2952372 = {S105919, 92, { &P12174,&P17484,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sgh-f480i) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA2977062 = {S105920, 133, { &P24085,&P18110,&P19800,&P18092 }};	/*Useragent - Mozilla/5.0 (DTV) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.5 AQUOSBrowser/1.0 (AS00DTV;V;0001;0001) AQUOS-AS/2.0 LC-52LE840X*/
const Device UA2977070 = {S105921, 133, { &P24090,&P18110,&P19800,&P18092 }};	/*Useragent - Mozilla/5.0 (DTV) AppleWebKit/531.2+ (KHTML, like Gecko) Espial/6.0.5 AQUOSBrowser/1.0 (AS00DTV;V;0001;0001) AQUOS-AS/2.0 LC-46LX840H*/
const Device UA2977073 = {S105922, 184, { &P24048,&P18598,&P22571,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-tw; HTC Butterfly s Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30TeamStream (HTC Butterfly s, 50, 1.9.9)*/
const Device UA2977079 = {S105923, 163, { &P24048,&P18598,&P24173,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTC Butterfly s Build/JDQ39) AppleWebKit/537.31 (KHTML, like Gecko) Chrome/26.0.1410.58 Mobile Safari/537.31 OPR/14.0.1074.58201*/
const Device UA2977086 = {S105924, 153, { &P24048,&P18598,&P22770,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; zh-tw; HTC Butterfly s Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Maxthon*/
const Device UA2977092 = {S105925, 160, { &P24061,&P18598,&P19809,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-S7270 Build/JDQ39) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 YaBrowser/1.1.1364.172 Mobile Safari/537.22*/
const Device UA2977107 = {S105926, 196, { &P24064,&P18598,&P22245,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; U; Android 4.2.2; fr-fr; GT-S7275 Build/JDQ39) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30 Flipboard/2.0.2/552,2.0.2.552,2013-05-22 23:05, +0200, fr*/
const Device UA2977296 = {S105927, 76, { &P15364,&P17285,&P24186,&P18090 }};	/*Useragent - Mozilla/5.0 (Windows NT 6.0; Win64; x64; rv:26.0) Gecko/X Firefox/26.0 slurp*/
const Device UA2986345 = {S105928, 91, { &P15364,&P22839,&P24676,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; zte nova) U2/1.0.0 UCBrowser/8.9.0.251 U2/1.0.0 Mobile*/
const Device UA3013001 = {S105929, 154, { &P11625,&P5768,&P24678,&P18092 }};	/*Useragent - Nokia5300/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5300) U2/1.0.0 UCBrowser/9.1.0.291 U2/1.0.0 Mobile*/
const Device UA3024026 = {S105930, 144, { &P23698,&P18598,&P24343,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SAMSUNG-SGH-I527 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1500.94 Mobile Safari/537.36*/
const Device UA3034282 = {S105931, 100, { &P14831,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; samsung-gt-s7233e) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3039556 = {S105932, 223, { &P24216,&P18110,&P24302,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LA690S-ZB; 03.10.74; 0xX;); LG NetCast.TV-2013 /03.10.74 (LG, 42LA690S-ZB, wireless)*/
const Device UA3039557 = {S105933, 223, { &P24281,&P18110,&P24302,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LA740S-ZB; 03.10.81; 0xX;); LG NetCast.TV-2013 /03.10.81 (LG, 42LA740S-ZB, wireless)*/
const Device UA3039558 = {S105934, 185, { &P24282,&P18110,&P24302,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 32LN570V-ZA; 02.06.01; 0xX;); LG NetCast.TV-2013*/
const Device UA3039559 = {S105935, 223, { &P24315,&P18110,&P24302,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LA660S-ZA; 03.10.74; 0xX;); LG NetCast.TV-2013 /03.10.74 (LG, 47LA660S-ZA, wireless)*/
const Device UA3039560 = {S105936, 223, { &P24227,&P18110,&P24302,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 55LA6200-UA; 03.07.24; 0xX;); LG NetCast.TV-2013 /03.07.24 (LG, 55LA6200-UA, wireless)*/
const Device UA3039561 = {S105937, 223, { &P24317,&P18110,&P24302,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LA667S-ZB; 03.10.81; 0xX;); LG NetCast.TV-2013 /03.10.81 (LG, 42LA667S-ZB, wireless)*/
const Device UA3039563 = {S105938, 220, { &P24307,&P18110,&P24302,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LA790V-ZA; 03.05.02; 0xX;); LG NetCast.TV-2013 /03.05.02 (LG, 47LA790V-ZA, wired)*/
const Device UA3039564 = {S105939, 223, { &P24229,&P18110,&P24302,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LA860V-ZA; 03.05.49; 0xX;); LG NetCast.TV-2013 /03.05.49 (LG, 42LA860V-ZA, wireless)*/
const Device UA3039567 = {S105940, 223, { &P24233,&P18110,&P24302,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LA6210-TD; 03.10.34; 0xX;); LG NetCast.TV-2013 /03.10.34 (LG, 42LA6210-TD, wireless)*/
const Device UA3039571 = {S105941, 220, { &P24238,&P18110,&P24302,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+SCREEN+TUNER; LGE; 32LN571B-TE; 03.10.65; 0xX;); LG NetCast.TV-2013 /03.10.65 (LG, 32LN571B-TE, wireless)*/
const Device UA3039579 = {S105942, 220, { &P24246,&P18110,&P24302,&P18092 }};	/*Useragent - Mozilla/5.0 (Unknown; Linux armv7l) AppleWebKit/537.1+ (KHTML, like Gecko) Safari/537.1+ LG Browser/6.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LA6200-TA; 03.20.09; 0xX;); LG NetCast.TV-2013 /03.20.09 (LG, 47LA6200-TA, wired)*/
const Device UA3039580 = {S105943, 198, { &P24251,&P18110,&P24300,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; U; Linux 35230; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.1.4(+mouse+PORTAL_KEY+SCREEN+TUNER; LGE; 37LV375S-ZC; 12.00.00; 0xX;); LG NetCast.TV-2012*/
const Device UA3039588 = {S105944, 202, { &P24253,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LM960V-ZB; 04.10.26; 0xX;); LG NetCast.TV-2012 0*/
const Device UA3039591 = {S105945, 197, { &P24255,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+SCREEN+TUNER; LGE; 55LS5700-TB; 04.00.28; 0xX;); LG NetCast.TV-2012*/
const Device UA3039597 = {S105946, 187, { &P24312,&P18110,&P24300,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; U; Linux 35230; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.1.4(+mouse+SCREEN+TUNER; LGE; 55LV3730-TD; 04.03.10; 0xX;); LG NetCast.TV-2011*/
const Device UA3039598 = {S105947, 199, { &P24288,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+SCREEN+TUNER; LGE; 42LS570T-ZB; 04.10.23; 0xX;); LG NetCast.TV-2012 0*/
const Device UA3039608 = {S105948, 198, { &P24311,&P18110,&P24300,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; U; Linux 35230; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.1.4(+mouse+PORTAL_KEY+SCREEN+TUNER; LGE; 37LV5500-ZC; 06.01.04; 0xX;); LG NetCast.TV-2011*/
const Device UA3039626 = {S105949, 200, { &P24328,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 42LM6710-TB; 04.00.29; 0xX;); LG NetCast.TV-2012*/
const Device UA3039634 = {S105950, 202, { &P24360,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 55LM9600-TA; 04.20.12; 0xX;); LG NetCast.TV-2012 0*/
const Device UA3039639 = {S105951, 202, { &P24358,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LM6410-TB; 04.20.11; 0xX;); LG NetCast.TV-2012 0*/
const Device UA3039648 = {S105952, 200, { &P24359,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 32LM620T-ZE; 04.40.23; 0xX;); LG NetCast.TV-2012*/
const Device UA3039656 = {S105953, 197, { &P24291,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+SCREEN+TUNER; LGE; 47LS5700-TB; 04.30.11; 0xX;); LG NetCast.TV-2012*/
const Device UA3039659 = {S105954, 189, { &P24364,&P18110,&P24300,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; U; Linux 35230; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.1.4(+mouse+3D+SCREEN+TUNER; LGE; 50PZ950-TA; 05.01.01; 0xX;); LG NetCast.TV-2011*/
const Device UA3039663 = {S105955, 202, { &P24333,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 72LM9500-TA; 04.30.11; 0xX;); LG NetCast.TV-2012 0*/
const Device UA3039670 = {S105956, 202, { &P24332,&P18110,&P24301,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; Linux armv7l) AppleWebKit/534.26+ (KHTML, like Gecko) Version/5.0 Safari/534.26+ LG Browser/5.00.00(+mouse+3D+SCREEN+TUNER; LGE; 47LM669S-ZC; 04.40.27; 0xX;); LG NetCast.TV-2012 0*/
const Device UA3039687 = {S105957, 253, { &P24293,&P24094,&P24304,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux mipsel; U; HbbTV/1.1.1 (; TOSHIBA; DTV_TL968; 7.0.27.3; a5; ) ; ToshibaTP/1.3.0 (+VIDEO_MP4+VIDEO_X_MS_ASF+AUDIO_MPEG+AUDIO_MP4+DRM+3D+NATIVELAUNCH+WEBSTORAGE+OFFLINEAPP+HAS_CMD_HTTP_SERVER) ; fr) AppleWebKit/534.1 (KHTML, like Gecko)*/
const Device UA3039764 = {S105958, 158, { &P24226,&P24094,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; Opera TV Store/4502; U; fr) Presto/2.10.250 Version/11.60 Model/Sony-KDL-55HX855 SonyCEBrowser/1.0 (KDL-55HX855; CTV/PKG1.700EUA; FRA)*/
const Device UA3039771 = {S105959, 158, { &P24234,&P24094,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; Opera TV Store/4510; U; de) Presto/2.10.250 Version/11.60 Model/Sony-KDL-46HX758 SonyCEBrowser/1.0 (KDL-46HX758; CTV/PKG1.910EUA; DEU)*/
const Device UA3039770 = {S105960, 158, { &P24248,&P24094,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; Opera TV Store/4510; U; el) Presto/2.10.250 Version/11.60 Model/Sony-KDL-46HX850 SonyCEBrowser/1.0 (KDL-46HX850; CTV/PKG1.910EUA; GRC)*/
const Device UA3039785 = {S105961, 134, { &P24252,&P24094,&P18104,&P18092 }};	/*Useragent - Opera/9.80 (Linux mips; U; Model/Sony-KDL-40HX75A SonyCEBrowser/1.0 (KDL-40HX75A; CTV/1.0; TWN; en); en) Presto/2.10.250 Version/11.60*/
const Device UA3040391 = {S105962, 126, { &P15364,&P24349,&P17415,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.13+ (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2 Fedora/18 Epiphany/3.6.1*/
const Device UA3044425 = {S105963, 153, { &P11667,&P5768,&P24675,&P18092 }};	/*Useragent - Nokia6085/2.0 (06.00) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0(Java; U; MIDP-2.0; en-US; nokia6085) U2/1.0.0 UCBrowser/8.8.1.252 U2/1.0.0 Mobile*/
const Device UA3059009 = {S105964, 92, { &P11658,&P5776,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6680) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3068891 = {S105965, 147, { &P12280,&P24384,&P4201,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; U; CPU iPhone OS 7_1_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/9B206 Safari/6531.22.7*/
const Device UA3068894 = {S105966, 115, { &P15767,&P24384,&P22701,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.46 (KHTML, like Gecko) Mobile/11D11 Twitter for iPhone*/
const Device UA3068899 = {S105967, 98, { &P15767,&P24384,&P17596,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_1 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/11D41*/
const Device UA3068916 = {S105968, 144, { &P18472,&P18111,&P24394,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; DROID RAZR HD Build/9.8.1Q-79) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1612.0 Mobile Safari/537.36*/
const Device UA3068928 = {S105969, 135, { &P21261,&P18598,&P24394,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; GT-I9500 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1622.0 Mobile Safari/537.36*/
const Device UA3068932 = {S105970, 135, { &P22594,&P18598,&P24394,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SGH-M919 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1612.0 Mobile Safari/537.36*/
const Device UA3068939 = {S105971, 140, { &P16292,&P18598,&P24394,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Galaxy Nexus Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1612.0 Mobile Safari/537.36*/
const Device UA3068977 = {S105972, 129, { &P15767,&P18432,&P24398,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/31.0.1610.0 Mobile/12E55 Safari/8536.25*/
const Device UA3068978 = {S105973, 129, { &P15767,&P19826,&P24398,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 6_1 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) CriOS/31.0.1610.0 Mobile/12E55 Safari/8536.25*/
const Device UA3068979 = {S105974, 131, { &P15767,&P22882,&P24398,&P18092 }};	/*Useragent - Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/31.0.1617.0 Mobile/12E55 Safari/8536.25*/
const Device UA3068980 = {S105975, 145, { &P12280,&P22882,&P24398,&P18092 }};	/*Useragent - Mozilla/5.0 (iPhone; CPU iPhone OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) CriOS/31.0.1617.610 Mobile/11A4449d Safari/8536.25*/
const Device UA3079207 = {S105976, 93, { &P14454,&P5890,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokiae66-1) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3079496 = {S105977, 154, { &P11625,&P5768,&P24679,&P18092 }};	/*Useragent - Nokia5300/2.0 (07.20) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia5300) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3084057 = {S105978, 165, { &P16263,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.3; MediaPad 7 Lite Build/HuaweiMediaPad) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084058 = {S105979, 162, { &P14186,&P5629,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.4; GT-S5830 Build/GINGERBREAD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084062 = {S105980, 162, { &P21020,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; Micromax A116 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084074 = {S105981, 153, { &P22653,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; HTL22 Build/JDQ39) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084049 = {S105982, 146, { &P19991,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; A511 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084082 = {S105983, 159, { &P19939,&P5629,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.6; XT303 Build/07.17.04R_S) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084088 = {S105984, 160, { &P20124,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.1; ZTE Grand X Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084089 = {S105985, 158, { &P22851,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; XT919 Build/2_290_2008) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084095 = {S105986, 156, { &P12698,&P5629,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 2.3.4; LG-C660h Build/GRJ22) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084100 = {S105987, 171, { &P14993,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; LT15i Build/Xperia Ultimate HDT 4.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084073 = {S105988, 151, { &P17337,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; GT-P7500M Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084102 = {S105989, 151, { &P21334,&P18066,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.0.4; A8 Build/IMM76D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084104 = {S105990, 157, { &P19865,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; IM-A850L Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3084109 = {S105991, 157, { &P18645,&P18598,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.2.2; SPH-L710 Build/JDQ39E) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63537*/
const Device UA3092782 = {S105992, 91, { &P12051,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; gt-s5600) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3097330 = {S105993, 61, { &P15364,&P18110,&P24521,&P18092 }};	/*Useragent - Mozilla/5.0 (X11; Linux x86_32; rv:28.0) Gecko/X Firefox/28.0*/
const Device UA3097331 = {S105994, 61, { &P15364,&P17350,&P24521,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; AltWindows; ru) Gecko/X Firefox/28.0*/
const Device UA3097333 = {S105995, 157, { &P18283,&P18111,&P24523,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; GT-I9300 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.30 Mobile Safari/537.36 OPR/17.0.1254.63751*/
const Device UA3108412 = {S105996, 91, { &P11653,&P17484,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; sgh-f480) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3111478 = {S105997, 99, { &P13677,&P17484,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; SAMSUNG-GT-S5253) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3125313 = {S105998, 92, { &P11679,&P5776,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; nokia6630) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3125459 = {S105999, 92, { &P13088,&P17486,&P24679,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; mot-ex223) U2/1.0.0 UCBrowser/9.2.0.311 U2/1.0.0 Mobile*/
const Device UA3126729 = {S106000, 100, { &P16384,&P17484,&P24680,&P18092 }};	/*Useragent - UCWEB/2.0 (Java; U; MIDP-2.0; en-US; SAMSUNG-GT-S5380K) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3128420 = {S106001, 156, { &P16074,&P5768,&P24680,&P18092 }};	/*Useragent - Nokia3110c/2.0 (07.30) Profile/MIDP-2.0 Configuration/CLDC-1.1 UCWEB/2.0 (Java; U; MIDP-2.0; en-US; Nokia3110c) U2/1.0.0 UCBrowser/9.3.0.326 U2/1.0.0 Mobile*/
const Device UA3130693 = {S106002, 157, { &P12972,&P18111,&P24399,&P18092 }};	/*Useragent - Mozilla/5.0 (Linux; Android 4.1.2; SGH-T989 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.72 Mobile Safari/537.36 OPR/16.0.1212.63780*/
const Device UA138 = {S106003, 97, { &P17475,&P5768,&P4737,&P18092 }};	/*Useragent - Nokia6300/2.0 (05.00) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6300/UC Browser7.2.2.51/69/401*/
const Device UA24870 = {S106004, 121, { &P17475,&P5768,&P17436,&P18092 }};	/*Useragent - Mozilla/5.0 (Series40; Nokia6300/07.00; Profile/MIDP-2.0 Configuration/CLDC-1.1) Gecko/20100401 S40OviBrowser/1.0.3.23.12*/
const Device UA27996 = {S106005, 75, { &P12207,&P17017,&P3601,&P18092 }};	/*Useragent - 880SH/1.103 Browser/VF-NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA33502 = {S106006, 97, { &P17475,&P5768,&P4722,&P18092 }};	/*Useragent - Nokia6300/2.0 (05.50) Profile/MIDP-2.0 Configuration/CLDC-1.1 nokia6300/UC Browser7.0.0.41/69/352*/
const Device UA51489 = {S106007, 83, { &P13373,&P17485,&P18589,&P18092 }};	/*Useragent -  SonyEricssonZ310iv/R1DB Java/SEMC-Java/2.0 Profile/MIDP-2.0 Configuration/CLDC-1.1*/
const Device UA79857 = {S106008, 14, { &P15364,&P17017,&P17213,&P18092 }};	/*Useragent - Opera/8 Presto*/
const Device UA79861 = {S106009, 14, { &P15364,&P17017,&P4811,&P18092 }};	/*Useragent - Opera/9 Presto*/
const Device UA79925 = {S106010, 90, { &P17595,&P17291,&P17249,&P18092 }};	/*Useragent - Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_1) AppleWebKit (KHTML, like Gecko) Chrome/10*/
const Device UA79916 = {S106011, 89, { &P17595,&P17291,&P17248,&P18092 }};	/*Useragent - Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_1) AppleWebKit (KHTML, like Gecko) Chrome/9*/
const Device UA79934 = {S106012, 90, { &P17595,&P17291,&P17250,&P18092 }};	/*Useragent - Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_1) AppleWebKit (KHTML, like Gecko) Chrome/11*/
const Device UA79943 = {S106013, 90, { &P17595,&P17291,&P17251,&P18092 }};	/*Useragent - Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_1) AppleWebKit (KHTML, like Gecko) Chrome/12*/
const Device UA79952 = {S106014, 90, { &P17595,&P17291,&P17252,&P18092 }};	/*Useragent - Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_1) AppleWebKit (KHTML, like Gecko) Chrome/13*/
const Device UA79961 = {S106015, 90, { &P17595,&P17291,&P17253,&P18092 }};	/*Useragent - Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_1) AppleWebKit (KHTML, like Gecko) Chrome/14*/
const Device UA79970 = {S106016, 90, { &P17595,&P17291,&P17254,&P18092 }};	/*Useragent - Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_1) AppleWebKit (KHTML, like Gecko) Chrome/15*/
const Device UA80021 = {S106017, 90, { &P15364,&P17285,&P17314,&P18092 }};	/*Useragent - Mozilla/5.0 (Windows; U; Windows NT 6.0;) AppleWebkit (KHTML, like Gecko) Version/2 Safari*/
const Device UA79906 = {S106018, 89, { &P17595,&P17291,&P17247,&P18092 }};	/*Useragent - Mozilla/5.0 (Mactintosh; U; Intel Mac OS X 10_1) AppleWebKit (KHTML, like Gecko) Chrome/8*/
const Device UA253777 = {S106019, 8, { &P15364,&P17017,&P17240,&P18092 }};	/*Useragent - Chrome/3*/
const Device UA253776 = {S106020, 8, { &P15364,&P17017,&P17237,&P18092 }};	/*Useragent - Chrome/2*/
const Device UA253780 = {S106021, 8, { &P15364,&P17017,&P17243,&P18092 }};	/*Useragent - Chrome/6*/
const Device UA253781 = {S106022, 8, { &P15364,&P17017,&P17245,&P18092 }};	/*Useragent - Chrome/7*/
const Device UA253787 = {S106023, 9, { &P15364,&P17017,&P17252,&P18092 }};	/*Useragent - Chrome/13*/
const Device UA253788 = {S106024, 9, { &P15364,&P17017,&P17253,&P18092 }};	/*Useragent - Chrome/14*/
const Device UA253789 = {S106025, 9, { &P15364,&P17017,&P17254,&P18092 }};	/*Useragent - Chrome/15*/
const Device UA364283 = {S106026, 128, { &P11758,&P6909,&P3780,&P18089 }};	/*Useragent - Nokia6820/2.0 (4.83) Profile/MIDP-1.0 Configuration/CLDC-1.0 (compatible; Googlebot-Mobile/2.1; +http://www.google.com/bot.html)*/
const Device UA616607 = {S106027, 181, { &P24223,&P18110,&P24300,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; U; Linux 35230; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.1.4(+SCREEN+TUNER; LGE; 47LV3730-TD; 04.01.09; 0xX;); LG NetCast.TV-2011*/
const Device UA616609 = {S106028, 187, { &P24219,&P18110,&P24300,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; U; Linux 35230; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.1.4(+mouse+SCREEN+TUNER; LGE; 42LV3730-TD; 04.03.10; 0xX;); LG NetCast.TV-2011*/
const Device UA616613 = {S106029, 198, { &P24224,&P24094,&P24299,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; U; Linux mips; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.0.10(+mouse+PORTAL_KEY+SCREEN+TUNER; LGE; 42LE5500-ZA; 03.21.01; 0xX;); LG NetCast.TV-2010*/
const Device UA616616 = {S106030, 201, { &P24222,&P18110,&P24300,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; U; Linux 35230; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.1.4(+mouse+3D+PORTAL_KEY+SCREEN+TUNER; LGE; 42LW5500-ZE; 05.11.08; 0xX;); LG NetCast.TV-2011*/
const Device UA616615 = {S106031, 205, { &P24220,&P24094,&P24297,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; U; Linux mips; en) AppleWebKit/528.5+ (KHTML, like Gecko, Safari/528.5+); LG Browser/3.0.13(+mouse+3D+SCREEN+TUNER; LGE; 42LE7500-ZA; 03.05.04; 0xX;); LG Netcast.Media-2011 (avkb/pcx*/
const Device UA616610 = {S106032, 180, { &P24221,&P24094,&P24299,&P18092 }};	/*Useragent - Mozilla/5.0 (DirectFB; U; Linux mips; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.0.10(+SCREEN+TUNER; LGE; 42LD650-ZC; 03.20.21; 0xX;); LG NetCast.TV-2010*/
const Device UA622126 = {S106033, 29, { &P12280,&P17779,&P18540,&P18092 }};	/*Useragent - Dolphin 3.5.1 (iPhone; iPhone*/
const Device UA622215 = {S106034, 43, { &P15364,&P18570,&P18539,&P18092 }};	/*Useragent - Dolphin/3.0 CFNetwork/548.0.3 Darwin/11.0.0*/
const Device UA622497 = {S106035, 74, { &P18425,&P17017,&P24668,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; vi; fpt-b730) UCBrowser8.3.0.154/69/352/UCWEB Mobile*/
const Device UA622516 = {S106036, 35, { &P18425,&P17017,&P17338,&P18092 }};	/*Useragent - fpt-b730/UC Browser7.9.0.102/69/352*/
const Device UA622518 = {S106037, 77, { &P18425,&P17017,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; en-us; fpt-b730) UCBrowser8.2.0.132/69/452/UCWEB Mobile*/
const Device UA622512 = {S106038, 74, { &P18425,&P17017,&P24667,&P18092 }};	/*Useragent - Mozilla/5.0 (Java; U; vi; fpt-b730) UCBrowser8.2.1.144/69/352/UCWEB Mobile*/
const Device UA622526 = {S106039, 34, { &P18425,&P17017,&P4779,&P18092 }};	/*Useragent - fpt-b730/UC Browser7.8.0.95/69/401*/
const Device UA2242179 = {S106040, 53, { &P21032,&P18615,&P18628,&P18092 }};	/*Useragent - Mozilla/5.0 (Mobile; rv:14.0) Gecko/14.0 Firefox/14.0*/


regex_t R0;	/*Regex - (?i)bot|spider|crawler|slurp*/
regex_t R1;	/*Regex - (?i)bot|spider|crawler|slurp|[^\s]+(?=bot|spider|crawler|slurp)[^\s]+*/
regex_t R2;	/*Regex - .+*/
regex_t R3;	/*Regex - W3C_Validator*/
regex_t R4;	/*Regex - ^Opera/*/
regex_t R5;	/*Regex - Windows NT 6\.0*/
regex_t R6;	/*Regex - Windows NT 6\.1*/
regex_t R7;	/*Regex - Opera Mini|Opera Mobi*/
regex_t R8;	/*Regex - MIDP|CLDC*/
regex_t R9;	/*Regex - (?<=Opera/)[\d+.]+*/
regex_t R10;	/*Regex - (?<=Version/)\d+\.\d+*/
regex_t R11;	/*Regex - (?<=\()[^;]+*/
regex_t R12;	/*Regex - (?<=Opera/\d\.\d \()[^;\)]+*/
regex_t R13;	/*Regex - Android( |)2\.2\.1*/
regex_t R14;	/*Regex - (?<=;)([^;\)]+)(?=Build/)*/
regex_t R15;	/*Regex - (?<=\w\w(_|-)\w\w;)[^;\)]+*/
regex_t R16;	/*Regex - (?<=;\sAndroid)[\d\s][^;\)]+*/
regex_t R17;	/*Regex - (?<=UCWEB)\d+\.\d+|(?<=UCWEB/)\d+\.\d+*/
regex_t R18;	/*Regex - (?<=UC Browser)\d+\.\d+*/
regex_t R19;	/*Regex - (?<=UP\.Browser)[\d\.]+*/
regex_t R20;	/*Regex - (?<=Opera/)\d+\.\d+|(?<=Opera )\d+\.\d+*/
regex_t R21;	/*Regex - (?<=Opera Mini/)\d+\.\d+*/
regex_t R22;	/*Regex - (?<=Opera Mobi/)\d+\.\d+*/
regex_t R23;	/*Regex - (?<=Ninesky-android-mobile/)\d+\.\d+*/
regex_t R24;	/*Regex - (?<=CrMo/)\d+*/
regex_t R25;	/*Regex - (?<=Chrome/)\d+*/
regex_t R26;	/*Regex - (?<=AppleWebKit/)[\d\.\+]+*/
regex_t R27;	/*Regex - (?<=Mobile Safari/)[\d\.]+*/
regex_t R28;	/*Regex - (?<=Flipboard/)\d+\.\d+*/
regex_t R29;	/*Regex - MQQBrowser/(\d+\.\d|\d+ )*/
regex_t R30;	/*Regex - Maxthon*/
regex_t R31;	/*Regex - UCBrowser/\d+\.\d*/
regex_t R32;	/*Regex - iBrowser/\d+\.\d*/
regex_t R33;	/*Regex - TeamStream*/
regex_t R34;	/*Regex - Maxthon/\d+\.\d*/
regex_t R35;	/*Regex - Ninesky/\d+\.\d*/
regex_t R36;	/*Regex - OPR/\d+\.\d+*/
regex_t R37;	/*Regex - Android( |)2\.2\.2*/
regex_t R38;	/*Regex - MQQBrowser/\d+\.\d*/
regex_t R39;	/*Regex - Android( |)2\.2\.3*/
regex_t R40;	/*Regex - Android( |)2\.3\.1*/
regex_t R41;	/*Regex - Android( |)2\.3\.2*/
regex_t R42;	/*Regex - Android( |)2\.3\.3*/
regex_t R43;	/*Regex - (?<=UCBrowser)\d+\.\d+*/
regex_t R44;	/*Regex - (?<=UCBrowser/)\d+\.\d+*/
regex_t R45;	/*Regex - ^(One|i)Browser/(\d+|\d+\.\d)*/
regex_t R46;	/*Regex - Mobile Maxthon*/
regex_t R47;	/*Regex - \)Maxthon|Maxthon$*/
regex_t R48;	/*Regex - Ninesky/\d\.\d*/
regex_t R49;	/*Regex - ^YahooMobile/\d+\.\d*/
regex_t R50;	/*Regex - Android( |)2\.3\.4*/
regex_t R51;	/*Regex - \)Maxthon*/
regex_t R52;	/*Regex - Cyanogen*/
regex_t R53;	/*Regex - Maxthon$*/
regex_t R54;	/*Regex - (One|i)Browser/\d\.\d*/
regex_t R55;	/*Regex - Android( |)2\.3\.5*/
regex_t R56;	/*Regex - OneBrowser/\d+\.\d*/
regex_t R57;	/*Regex - Maxthon(/\d+\.\d+|$)*/
regex_t R58;	/*Regex - UCBrowser/\d+\.\d+*/
regex_t R59;	/*Regex - ^iBrowser/(\d+\.\d|\d+ )*/
regex_t R60;	/*Regex - Android( |)2\.3\.6*/
regex_t R61;	/*Regex - OneBrowser/\d\.\d*/
regex_t R62;	/*Regex - Android( |)2\.3\.7*/
regex_t R63;	/*Regex - (?<=CyanogenMod-)\d*/
regex_t R64;	/*Regex - (?<=CyanogenMod)\d*/
regex_t R65;	/*Regex - CyanogenMod*/
regex_t R66;	/*Regex - CM7*/
regex_t R67;	/*Regex - MQQBrowser/\d\.\d*/
regex_t R68;	/*Regex - Chrome/[0-9][^\d]*/
regex_t R69;	/*Regex - Android*/
regex_t R70;	/*Regex - (?<=Chrome/)\d+\.\d+*/
regex_t R71;	/*Regex - (?<=\(|;|,)[^\);]+*/
regex_t R72;	/*Regex - (?<=RockMelt/)\d+\.\d+*/
regex_t R73;	/*Regex - Ubuntu/\d+\.\d+*/
regex_t R74;	/*Regex - Sleipnir/\d+\.\d+*/
regex_t R75;	/*Regex - Nutch-\d+\.\d+*/
regex_t R76;	/*Regex - Comodo_Dragon/\d+\.\d+*/
regex_t R77;	/*Regex - Iron/\d+\.\d*/
regex_t R78;	/*Regex - GoogleTV*/
regex_t R79;	/*Regex - chromeframe/\d+\.\d*/
regex_t R80;	/*Regex - Flock/\d+\.\d*/
regex_t R81;	/*Regex - Chrome/1[0-9]*/
regex_t R82;	/*Regex - (?<=^).+(?=Mozilla)*/
regex_t R83;	/*Regex - Puffin/\d+\.\d*/
regex_t R84;	/*Regex - (?<=QQBrowser/)\d+\.\d+*/
regex_t R85;	/*Regex - AvantBrowser/Tri-Core*/
regex_t R86;	/*Regex - (?<=Midori/)\d\.\d*/
regex_t R87;	/*Regex - Chrome/2[0-9]*/
regex_t R88;	/*Regex - Ubuntu Chromium*/
regex_t R89;	/*Regex - Chrome/3[0-9]*/
regex_t R90;	/*Regex - Linux Mint*/
regex_t R91;	/*Regex - Ubuntu;*/
regex_t R92;	/*Regex - ASUS\)*/
regex_t R93;	/*Regex - Mac OS X \d+(\.|_)\d*/
regex_t R94;	/*Regex - Windows NT \d\.\d*/
regex_t R95;	/*Regex - (?<=KDL)[^;]+*/
regex_t R96;	/*Regex - SonyEricsson*/
regex_t R97;	/*Regex - (?<=SonyEricsson)[^/]+*/
regex_t R98;	/*Regex - (?<=NetFront/)[\d\.]+*/
regex_t R99;	/*Regex - (?<=Opera )[^\s]+*/
regex_t R100;	/*Regex - (?<=SEMC-Browser/)[^\s]+*/
regex_t R101;	/*Regex - (?<=TelecaBrowser/)[^\s]+*/
regex_t R102;	/*Regex - (?<=SymbianOS/)[^;]+ */
regex_t R103;	/*Regex - (?<=Series60/)[^\s]+*/
regex_t R104;	/*Regex - (?<=U(C|P)\.Browser/)\d+\.\d+*/
regex_t R105;	/*Regex - (?<=UCBrowser)(|/)\d+\.\d+*/
regex_t R106;	/*Regex - UCWEB/\d\.\d*/
regex_t R107;	/*Regex - Silk-Accelerated*/
regex_t R108;	/*Regex - (?<=Silk/)\d+\.\d+*/
regex_t R109;	/*Regex - (?<=Accelerated=)(true|false)*/
regex_t R110;	/*Regex - Android \d|Android\d*/
regex_t R111;	/*Regex - GT-\w\d+*/
regex_t R112;	/*Regex - 2\.2/FROYO*/
regex_t R113;	/*Regex - theScore/\d\.\d*/
regex_t R114;	/*Regex - ^Instagram \d\.\d*/
regex_t R115;	/*Regex - ^WhatsApp/2\.7*/
regex_t R116;	/*Regex - (FROYO|GINGERBREAD)\)$*/
regex_t R117;	/*Regex - Android 1|Android1*/
regex_t R118;	/*Regex - Android 2\.0|Android2\.0*/
regex_t R119;	/*Regex - Android 2\.1|Android2\.1*/
regex_t R120;	/*Regex - Android 2\.2|Android2\.2*/
regex_t R121;	/*Regex - Maxthon/\d+\.\d|Maxthon*/
regex_t R122;	/*Regex - ^iBrowser/\d+*/
regex_t R123;	/*Regex - Android 2\.3|Android2\.3*/
regex_t R124;	/*Regex - Android 2\.4|Android2\.4*/
regex_t R125;	/*Regex - Android 3|Android3*/
regex_t R126;	/*Regex - Flipboard/\d+\.\d*/
regex_t R127;	/*Regex - \[operamini\]*/
regex_t R128;	/*Regex - ^(One|i)Browser/(\d+\.\d|\d+ )*/
regex_t R129;	/*Regex - (?<=OverDrive Media Console/)\d\.\d*/
regex_t R130;	/*Regex - Android( |)4\.0*/
regex_t R131;	/*Regex - Android( |)4\.0\.\d+*/
regex_t R132;	/*Regex - Chrome/\d+*/
regex_t R133;	/*Regex - Android 4\.0\.1|Android4\.0\.1*/
regex_t R134;	/*Regex - (?<=^).+(?=Linux)*/
regex_t R135;	/*Regex - Android 4\.0\.2|Android4\.0\.2*/
regex_t R136;	/*Regex - Android 4\.0\.3|Android4\.0\.3*/
regex_t R137;	/*Regex - Maxthon(/| \()\d+\.\d*/
regex_t R138;	/*Regex - ^(One|i)Browser/\d+\.\d*/
regex_t R139;	/*Regex - Android 4\.0\.4|Android4\.0\.4*/
regex_t R140;	/*Regex - (One|i)Browser/(\d+\.\d|\d+ )*/
regex_t R141;	/*Regex - Maxthon \(\d\.\d*/
regex_t R142;	/*Regex - PARANOIDANDROID*/
regex_t R143;	/*Regex - Android 4\.0\.(5|6|8|9)*/
regex_t R144;	/*Regex - Android 4\.1|Android4\.1*/
regex_t R145;	/*Regex - Android( |)4\.1\.\d+*/
regex_t R146;	/*Regex - Android 4\.1\.1|Android4\.1\.1*/
regex_t R147;	/*Regex - Maxthon$|\)Maxthon*/
regex_t R148;	/*Regex - ^OneBrowser/\d+\.\d*/
regex_t R149;	/*Regex - MiuiBrowser/\d\.\d*/
regex_t R150;	/*Regex - Android 4\.1\.2|Android4\.1\.2*/
regex_t R151;	/*Regex - (?<=^OneBrowser/)\d\.\d*/
regex_t R152;	/*Regex - Android 4\.2|Android4\.2*/
regex_t R153;	/*Regex - Android( |)4\.2\.\d+*/
regex_t R154;	/*Regex - Android 4\.2\.1|Android4\.2\.1*/
regex_t R155;	/*Regex - Maxthon(/\d+\.\d|$)*/
regex_t R156;	/*Regex - Android 4\.2\.2|Android4\.2\.2*/
regex_t R157;	/*Regex - Maxthon \(\d+\.\d*/
regex_t R158;	/*Regex - CARBON-JB*/
regex_t R159;	/*Regex - Liqiud-JB*/
regex_t R160;	/*Regex - \((iPhone|iPad|iPod)*/
regex_t R161;	/*Regex - Flipboard*/
regex_t R162;	/*Regex - Nokia|nokia*/
regex_t R163;	/*Regex - webOS|hpwOS*/
regex_t R164;	/*Regex - (?<=\()iPhone|iPad|iPod*/
regex_t R165;	/*Regex - (?<=Version/)[\d\.]+*/
regex_t R166;	/*Regex - (?<=OS )[\d(_|\.)\d]+*/
regex_t R167;	/*Regex - (?<=Sleipnir/)[\d\.]+*/
regex_t R168;	/*Regex - (?<=AppleWebKit/)[\d\.]+*/
regex_t R169;	/*Regex - (?<=Mobile/)[\d\w]+*/
regex_t R170;	/*Regex - (?<=Safari/)[\d\.]+*/
regex_t R171;	/*Regex - (?<=Dolphin )[\d\.]+*/
regex_t R172;	/*Regex - (?<=Dolphin HD )[\d\.]+*/
regex_t R173;	/*Regex - Twitter*/
regex_t R174;	/*Regex - Miu Ptt*/
regex_t R175;	/*Regex - Incredimail/\d+\.\d*/
regex_t R176;	/*Regex - PhantomJS/\d+\.\d*/
regex_t R177;	/*Regex - UP\.Browser/(\d+\.\d|\d+ )*/
regex_t R178;	/*Regex - BOLT*/
regex_t R179;	/*Regex - mac*/
regex_t R180;	/*Regex - win*/
regex_t R181;	/*Regex - (?<=BOLT/)[\d\.]+*/
regex_t R182;	/*Regex - (F|f)ire(F|f)o(X|x)|(I|i)ceweasel|BonEcho*/
regex_t R183;	/*Regex - \(Macintosh;|\(Win|\(X1\d;|WOW64*/
regex_t R184;	/*Regex - Fennec|Linux Mint|Thunderbird*/
regex_t R185;	/*Regex - (?<=X11;)\w+*/
regex_t R186;	/*Regex - (?<=Firefox)/\d+\.\d+*/
regex_t R187;	/*Regex - (?<=Iceweasel)/\d+\.\d+*/
regex_t R188;	/*Regex - (?<=Mozilla/)\d\.\d*/
regex_t R189;	/*Regex - (?<=Mozilla/\d\.\d \()[^;\)]+*/
regex_t R190;	/*Regex - Windows NT \d\.\d(?=;)|Mac OS X [^\s]+(?=;)|Ubuntu|X11|SunOS|OpenBSD|FreeBSD*/
regex_t R191;	/*Regex - Fedora/\d+\.\d+*/
regex_t R192;	/*Regex - Nutch*/
regex_t R193;	/*Regex - Debian-\d\.\d*/
regex_t R194;	/*Regex - \w(U|u)buntu/\d+\.\d+*/
regex_t R195;	/*Regex - (?<=\((U|u)buntu-)[^\)]+*/
regex_t R196;	/*Regex - (?<=Firefox)[^\d]+*/
regex_t R197;	/*Regex - (?<=Win)[^;]+*/
regex_t R198;	/*Regex - Fedora/*/
regex_t R199;	/*Regex - Pale(M|m)oon/\d+\.\d*/
regex_t R200;	/*Regex - (E|e)pic/\d+\.\d|Epic/*/
regex_t R201;	/*Regex - Comet(Bird|Browser)/\d+\.\d*/
regex_t R202;	/*Regex - Kylo/\d+\.\d*/
regex_t R203;	/*Regex - Wyzo/\d+\.\d*/
regex_t R204;	/*Regex - Camino/\d+\.\d*/
regex_t R205;	/*Regex - Minefield/\d+\.\d*/
regex_t R206;	/*Regex - Navigator/\d+\.\d*/
regex_t R207;	/*Regex - IceDragon/\d+\.\d*/
regex_t R208;	/*Regex - Opera \d+\.\d\d$*/
regex_t R209;	/*Regex - lolifox/\d\.\d*/
regex_t R210;	/*Regex - Avant TriCore*/
regex_t R211;	/*Regex - Midori/(|\")\d+(|\")\.(|\")\d(|\")*/
regex_t R212;	/*Regex - SeaMonkey/\d\.\d*/
regex_t R213;	/*Regex - firefox/\d\.\d*/
regex_t R214;	/*Regex - BonEcho/\d\.\d*/
regex_t R215;	/*Regex - Maemo Browser \d\.\d*/
regex_t R216;	/*Regex - Linux mips*/
regex_t R217;	/*Regex - Firefox/*/
regex_t R218;	/*Regex - Fennec/|\(Mobile; rv*/
regex_t R219;	/*Regex - (?<=Mozilla/)[\d\.]+*/
regex_t R220;	/*Regex - (?<=Fennec/)[\d\.]+*/
regex_t R221;	/*Regex - Fennec*/
regex_t R222;	/*Regex - (?<=\()[^\)]+*/
regex_t R223;	/*Regex - (?<=Gecko/)[\d]+*/
regex_t R224;	/*Regex - (?<=Firefox/)[\d\.]+*/
regex_t R225;	/*Regex - MSIE*/
regex_t R226;	/*Regex - compatible*/
regex_t R227;	/*Regex - Windows XP*/
regex_t R228;	/*Regex - Windows NT*/
regex_t R229;	/*Regex - Windows ME*/
regex_t R230;	/*Regex - Win32*/
regex_t R231;	/*Regex - Windows [\d\.]+*/
regex_t R232;	/*Regex - Windows NT [\d\.]+*/
regex_t R233;	/*Regex - Mac_PowerPC*/
regex_t R234;	/*Regex - PPC Mac OS*/
regex_t R235;	/*Regex - Mobile*/
regex_t R236;	/*Regex - Windows CE*/
regex_t R237;	/*Regex - Phone*/
regex_t R238;	/*Regex - UC(\.| ||)Browser*/
regex_t R239;	/*Regex - Opera*/
regex_t R240;	/*Regex - (?i)LG-|LG_|SC-|Toshiba|XV\d+|Nokia|LGE|PalmOS|POLARIS|PalmSource|Nutch|NetFront|Samsung|HTC*/
regex_t R241;	/*Regex - (?<=MSIE\s)[\d\.]+*/
regex_t R242;	/*Regex - (?<=Outlook )[\d\.]+*/
regex_t R243;	/*Regex - (?<=Windows)[\s\w\d\.]+*/
regex_t R244;	/*Regex - Windows NT \d\.\d(?=;)|Mac OS X [^\s]+(?=;)|Ubuntu|X11*/
regex_t R245;	/*Regex - chromeframe*/
regex_t R246;	/*Regex - Media Center PC*/
regex_t R247;	/*Regex - Linux*/
regex_t R248;	/*Regex - Zune*/
regex_t R249;	/*Regex - ARM*/
regex_t R250;	/*Regex - Touch*/
regex_t R251;	/*Regex - WOW64;*/
regex_t R252;	/*Regex - (?<=heritrix/)[\d\.]+*/
regex_t R253;	/*Regex - Maxthon|MAXTHON*/
regex_t R254;	/*Regex - SlimBrowser/\d+\.\d*/
regex_t R255;	/*Regex - Avant(| )Browser*/
regex_t R256;	/*Regex - Xbox*/
regex_t R257;	/*Regex - Midori/\d+\.\d*/
regex_t R258;	/*Regex - (?<=Maxthon)(/| )\d+\.\d*/
regex_t R259;	/*Regex - (?<=MAXTHON) \d\.\d*/
regex_t R260;	/*Regex - Win(95|32|98|NT4\.0|64|3\.\d)*/
regex_t R261;	/*Regex - PPC*/
regex_t R262;	/*Regex - Tablet PC*/
regex_t R263;	/*Regex - SV1*/
regex_t R264;	/*Regex - (?<=UCWEB)\d\.\d*/
regex_t R265;	/*Regex - Nokia*/
regex_t R266;	/*Regex - Series40*/
regex_t R267;	/*Regex - (?<=Nokia)[^/]+*/
regex_t R268;	/*Regex - (?<=Series40)[^;]+*/
regex_t R269;	/*Regex - (?<=S40OviBrowser/)\d+\.\d+\.\d+*/
regex_t R270;	/*Regex - iBrowser/Mini\d+\.\d*/
regex_t R271;	/*Regex - Series60*/
regex_t R272;	/*Regex - (?<=Series60/)[\w\d\.]+*/
regex_t R273;	/*Regex - MQQBrowser/Mini\d\.\d*/
regex_t R274;	/*Regex - ^Mozilla*/
regex_t R275;	/*Regex - Safari*/
regex_t R276;	/*Regex - Windows|Macintosh|X11*/
regex_t R277;	/*Regex - Chrome/*/
regex_t R278;	/*Regex - (?<=Safari/)\d+\.\d+*/
regex_t R279;	/*Regex - Windows NT \d\.\d(?=;)|Mac OS X [^\s]+(?=;)|Ubuntu( |(/|)\d+\.\d+)|X11*/
regex_t R280;	/*Regex - (?<=Version/)\d\.\d*/
regex_t R281;	/*Regex - (?<=Mac OS X 10_\d_\d;)[^;\)]+*/
regex_t R282;	/*Regex - Mac OS X 10_\d*/
regex_t R283;	/*Regex - (?<=PhantomJS/)\d\.\d*/
regex_t R284;	/*Regex - SecondLife/\d+\.\d*/
regex_t R285;	/*Regex - Google Earth/\d+\.\d*/
regex_t R286;	/*Regex - rekonq(/\d+\.\d| )*/
regex_t R287;	/*Regex - Skyfire/\d+\.\d*/
regex_t R288;	/*Regex - Sleipnir/\d+\.\d*/
regex_t R289;	/*Regex - QupZilla/\d+\.\d*/
regex_t R290;	/*Regex - konqueror/\d+\.\d*/
regex_t R291;	/*Regex - Google Earth Pro/\d+\.\d*/
regex_t R292;	/*Regex - OmniWeb/.+\.\d+*/
regex_t R293;	/*Regex - Qt/\d\.\d*/
regex_t R294;	/*Regex - kded/\d\.\d*/
regex_t R295;	/*Regex - Midori/\d\.\d*/
regex_t R296;	/*Regex - iCab/\d\.\d*/
regex_t R297;	/*Regex - (?<=Fluid/)\d\.\d*/
regex_t R298;	/*Regex - (?<=Iron/)\d\.\d*/
regex_t R299;	/*Regex - (?<=Stainless/)\d\.\d*/
regex_t R300;	/*Regex - Skyfire/*/
regex_t R301;	/*Regex - (?<=Skyfire/)[\d\.]+*/
regex_t R302;	/*Regex - Windows Phone OS*/
regex_t R303;	/*Regex - ^WordPress/*/
regex_t R304;	/*Regex - (?<=http://)\w+*/
regex_t R305;	/*Regex - Thunderbird*/
regex_t R306;	/*Regex - \(Macintosh;|\(Windows|\(X11;*/
regex_t R307;	/*Regex - Firefox|Iceweasel|Fennec|Linux Mint|Ubuntu*/
regex_t R308;	/*Regex - (?<=Thunderbird/)\d+\.\d+*/
regex_t R309;	/*Regex - (?<=(Fedora)/)\d+\.\d+*/
regex_t R310;	/*Regex - Darwin*/
regex_t R311;	/*Regex - CFNetwork*/
regex_t R312;	/*Regex - iPhone*/
regex_t R313;	/*Regex - ^[^/]+*/
regex_t R314;	/*Regex - (i)?Mac([^\d]+)?*/
regex_t R315;	/*Regex - (?<=Darwin/)\d+\.\d+*/
regex_t R316;	/*Regex - ^Dolphin/\d+\.\d*/
regex_t R317;	/*Regex - Android 4\.3|Android4\.3*/
regex_t R318;	/*Regex - Android 4\.4|Android4\.4*/
regex_t R319;	/*Regex - (?i)alcatel*/
regex_t R320;	/*Regex - (?<=Alcatel-)[^/\s]+*/
regex_t R321;	/*Regex - ^Amoi*/
regex_t R322;	/*Regex - ^Apple*/
regex_t R323;	/*Regex - CoreMedia*/
regex_t R324;	/*Regex - ^AUDIOVOX*/
regex_t R325;	/*Regex - BenQ*/
regex_t R326;	/*Regex - BENQ*/
regex_t R327;	/*Regex - ^BIRD*/
regex_t R328;	/*Regex - Black[B|b]erry*/
regex_t R329;	/*Regex - ^Mozilla/*/
regex_t R330;	/*Regex - BlackBerry*/
regex_t R331;	/*Regex - Version/\d+*/
regex_t R332;	/*Regex - (?<=Black[B|b]erry)[^/]+*/
regex_t R333;	/*Regex - (?<=Black[B|b]erry\w{4}/)\d+\.\d+\.\d+|(?<=Black[B|b]erry\w{5}/)\d+\.\d+\.\d+*/
regex_t R334;	/*Regex - (?<=UP.Browser/)\d+\.\d+*/
regex_t R335;	/*Regex - UC Browser\d+\.\d+*/
regex_t R336;	/*Regex - (?<=BlackBerry)[^;]+*/
regex_t R337;	/*Regex - (?<=Version/)\d+\.\d+\.\d+*/
regex_t R338;	/*Regex - ^DoCoMo*/
regex_t R339;	/*Regex - (?<=DoCoMo/)\d+\.\d+*/
regex_t R340;	/*Regex - (?<=DoCoMo/\d\.\d[\s|/])[\w\d]+*/
regex_t R341;	/*Regex - ^Fly|^FLY*/
regex_t R342;	/*Regex - ^HD2|HTC*/
regex_t R343;	/*Regex - (?<=\sHTC[\s|_])[^;\s]+*/
regex_t R344;	/*Regex - (?i)Huawei*/
regex_t R345;	/*Regex - ^KDDI*/
regex_t R346;	/*Regex - KDDI-\w+*/
regex_t R347;	/*Regex - ^kyocera|^QC-|^KWC-*/
regex_t R348;	/*Regex - ^LCT*/
regex_t R349;	/*Regex - ^Lenovo*/
regex_t R350;	/*Regex - (?<=^Lenovo[-_])[\w\d]+*/
regex_t R351;	/*Regex - ^LG/*/
regex_t R352;	/*Regex - (?<=^LG/)[\w\d]+*/
regex_t R353;	/*Regex - (?<=^LG(-|/))[^\s/]+*/
regex_t R354;	/*Regex - (?<=NovarraVision)\d\.\d*/
regex_t R355;	/*Regex - (?<=Browser/Teleca-)[^\s]+*/
regex_t R356;	/*Regex - (?<=UCBrowser)\d\.\d*/
regex_t R357;	/*Regex - (?<=Browser/Obigo-)[^/]+/[\d\.]+*/
regex_t R358;	/*Regex - (?<=Opera Mini/)[^\);]+*/
regex_t R359;	/*Regex - ^LG-*/
regex_t R360;	/*Regex - (?<=^LG-)[\w\d]+*/
regex_t R361;	/*Regex - UC( |)Browser(|/)\d+\.\d*/
regex_t R362;	/*Regex - ^LGE-*/
regex_t R363;	/*Regex - (?<=^LGE-)[\w\d]+*/
regex_t R364;	/*Regex - (?<=^LGE-)[^\s/]+*/
regex_t R365;	/*Regex - ^MOT-*/
regex_t R366;	/*Regex - (?<=^MOT-)[^/]+*/
regex_t R367;	/*Regex - (?<=UP\.Browser/)[\d\.]+*/
regex_t R368;	/*Regex - \([^\)]+\)*/
regex_t R369;	/*Regex - ^NEC|KGT*/
regex_t R370;	/*Regex - ^Nexian|^NEXIAN*/
regex_t R371;	/*Regex - (?i)nokia*/
regex_t R372;	/*Regex - Series(40|60)*/
regex_t R373;	/*Regex - (?i)(?<=Nokia)[^/\)]+*/
regex_t R374;	/*Regex - (?<=S40OviBrowser/)\d+\.\d+.\d+*/
regex_t R375;	/*Regex - ^iBrowser/Mini\d+\.\d+*/
regex_t R376;	/*Regex - ^OneBrowser/\d+\.\d+*/
regex_t R377;	/*Regex - ^MQQBrowser/Mini\d+\.\d+*/
regex_t R378;	/*Regex - Palm*/
regex_t R379;	/*Regex - Palm(S|s)ource*/
regex_t R380;	/*Regex - ^Panasonic*/
regex_t R381;	/*Regex - ^Pantech|^PT-|^PANTECH|^PG-*/
regex_t R382;	/*Regex - ^Philips|^PHILIPS*/
regex_t R383;	/*Regex - ^Qtek*/
regex_t R384;	/*Regex - iPhone|iPod|iPad|Nokia|BlackBerry|Android|Skyfire*/
regex_t R385;	/*Regex - (?<=Mozilla/\d\.\d \()[^)]+*/
regex_t R386;	/*Regex - Phantomjs/\d\.\d*/
regex_t R387;	/*Regex - LG Browser/\d\.\d*/
regex_t R388;	/*Regex - (?<=LGE;)[^;]+*/
regex_t R389;	/*Regex - ^Sagem|^SAGEM*/
regex_t R390;	/*Regex - (?i)SAMSUNG|SEC-|SCH-*/
regex_t R391;	/*Regex - Series60/*/
regex_t R392;	/*Regex - Samsung/*/
regex_t R393;	/*Regex - SAMSUNG-GT-*/
regex_t R394;	/*Regex - (?i)(?<=SAMSUNG)-?[^/]+*/
regex_t R395;	/*Regex - (?i)(?<=SEC|SCH)-?[^/]+*/
regex_t R396;	/*Regex - (?<=Bada/)\d+\.\d+*/
regex_t R397;	/*Regex - (?<=Dolfin/)\d+\.\d+*/
regex_t R398;	/*Regex - (?<=Jasmine/)\d+\.\d+*/
regex_t R399;	/*Regex - UC(\.|)Browser/\d+\.\d+*/
regex_t R400;	/*Regex - (?<=Windows)\w+;*/
regex_t R401;	/*Regex - Android \d\.\d*/
regex_t R402;	/*Regex - UCWEB\d\.\d*/
regex_t R403;	/*Regex - (?<=SAMSUNG-GT-)[^/-]+*/
regex_t R404;	/*Regex - (?<=UC\.Browser/)\d+\.\d+*/
regex_t R405;	/*Regex - (?<=MSIEMobile )\d+\.\d+*/
regex_t R406;	/*Regex - (?<=Samsung/)[^/]+*/
regex_t R407;	/*Regex - (?<=Series60/)[\d\.]+*/
regex_t R408;	/*Regex - (?<=SymbianOS/)[\d\.]+*/
regex_t R409;	/*Regex - ^Sanyo|^SANYO*/
regex_t R410;	/*Regex - Sanyo*/
regex_t R411;	/*Regex - SCP*/
regex_t R412;	/*Regex - ^Sharp|^SHARP*/
regex_t R413;	/*Regex - ^SIE-*/
regex_t R414;	/*Regex - ^SoftBank|^Vodafone|^J-PHONE*/
regex_t R415;	/*Regex - (SoftBank|Vodafone|J-PHONE)\/\d\.\d\/\w+*/
regex_t R416;	/*Regex - Spice*/
regex_t R417;	/*Regex - ^TIANYU-KTOUCH*/
regex_t R418;	/*Regex - (?<=TIANYU-KTOUCH/)[^/]+*/
regex_t R419;	/*Regex - Toshiba*/
regex_t R420;	/*Regex - ^Vodafone*/
regex_t R421;	/*Regex - webOS*/
regex_t R422;	/*Regex - HTC*/
regex_t R423;	/*Regex - (?<=\(|;|\))[^;\)]+*/
regex_t R424;	/*Regex - ^ZTE*/
regex_t R425;	/*Regex - (?i)Webkit*/
regex_t R426;	/*Regex - Chrome|Safari|Android|Nokia|BlackBerry*/
regex_t R427;	/*Regex - (?<=\(KHTML, like Gecko\))[^\(\)]+*/
regex_t R428;	/*Regex - Gecko*/
regex_t R429;	/*Regex - Fennec|Linux Mint|Firefox|IceWeasel|Thunderbird|WebKit|BonEcho*/
regex_t R430;	/*Regex - Windows NT \d\.\d(?=;)|Mac OS X [^\s]+(?=;)|Ubuntu|X11|Linux i686*/
regex_t R431;	/*Regex - Win\d(\d|x)*/
regex_t R432;	/*Regex - Fedora*/
regex_t R433;	/*Regex - (?<=StumbleUpon/)\d\.\d*/
regex_t R434;	/*Regex - Postbox/\d+\.\d*/
regex_t R435;	/*Regex - Minefield/\d\.\d*/
regex_t R436;	/*Regex - Netscape(\d|)/\d\.\d*/
regex_t R437;	/*Regex - lolifox/\d+\.\d*/
regex_t R438;	/*Regex - Icedove/\d+\.\d*/
regex_t R439;	/*Regex - zdesktop/\d+\.\d*/
regex_t R440;	/*Regex - Abrowser/\d+\.\d*/
regex_t R441;	/*Regex - Mnenhy/\d+\.\d*/
regex_t R442;	/*Regex - Konqueror/\d+\.\d*/
regex_t R443;	/*Regex - Ubuntu/\d+\.\d*/
regex_t R444;	/*Regex - Palemoon/\d+\.\d*/
regex_t R445;	/*Regex - SeaMonkey(/\d\.\d+|$)*/
regex_t R446;	/*Regex - GranParadiso/\d\.\d*/
regex_t R447;	/*Regex - K9Browser/\d+\.\d*/
regex_t R448;	/*Regex - iTunes*/
regex_t R449;	/*Regex - iPod/\d\.\d*/
regex_t R450;	/*Regex - (?<=Opera/)[\d+.]+|(?<=Opera )[\d\.]+*/
regex_t R451;	/*Regex - (?<=Presto/)\d+\.\d+*/
regex_t R452;	/*Regex - Opera Mini*/
regex_t R453;	/*Regex - (?<=Opera Mini/att/)\d(/|\.)\d*/
regex_t R454;	/*Regex - ^[^;]+*/
regex_t R455;	/*Regex - (?<=Opera Mini/)\d(/|\.)\d*/
regex_t R456;	/*Regex - (?<= \d\d\dX\d\d\d).+*/
regex_t R457;	/*Regex - (?<=Windows NT )\d\.\d*/
regex_t R458;	/*Regex - Opera Mini//\d+*/
regex_t R459;	/*Regex - Version/\d+\.\d+*/
regex_t R460;	/*Regex - Opera Mobi*/
regex_t R461;	/*Regex - ^portalmmm*/
regex_t R462;	/*Regex - Outlook*/
regex_t R463;	/*Regex - (?<=Microsoft Outlook )[\d\.]+*/
regex_t R464;	/*Regex - (?<=Outlook 20\d\d)[\d\.]+*/
regex_t R465;	/*Regex - Profile/MIDP-\d\.\d Configuration/CLDC-\d\.\d*/
regex_t R466;	/*Regex - .+(?=Profile/MIDP-\d\.\d Configuration/CLDC-\d\.\d)*/
regex_t R467;	/*Regex - WAP|MIDP|CLDC|Teleca-|Screen-|mobile*/
regex_t R468;	/*Regex - ^iOS/.+dataaccessd*/
regex_t R469;	/*Regex - ^Podcasts*/
regex_t R470;	/*Regex - ^Tiny Tiny RSS*/
regex_t R471;	/*Regex - ^Amazon CloudFront*/
regex_t R472;	/*Regex - ^Wada\.vn*/
regex_t R473;	/*Regex - ^FeedDemon*/
regex_t R474;	/*Regex - ^python-requests*/
regex_t R475;	/*Regex - ^RSSInclude*/
regex_t R476;	/*Regex - ^curl*/
regex_t R477;	/*Regex - ^Pulp/*/
regex_t R478;	/*Regex - Win98*/
regex_t R479;	/*Regex - AppEngine*/
regex_t R480;	/*Regex - ^[^\(]+*/
regex_t R481;	/*Regex - darwin*/
regex_t R482;	/*Regex - ^(I|J)UC*/
regex_t R483;	/*Regex - ^(J|I)UC*/
regex_t R484;	/*Regex - UCWEB\d+\.\d*/
regex_t R485;	/*Regex - (?<=(Z|z)h(-|_)cn;)[^;]+*/
regex_t R486;	/*Regex - Linux;\sU;\s\d\.\d*/
regex_t R487;	/*Regex - iOS \d\.\d*/
regex_t R488;	/*Regex - ^server-bag*/
regex_t R489;	/*Regex - iPhone OS,\d\.\d*/
regex_t R490;	/*Regex - ,iPhone|,iPod|,iPad*/
regex_t R491;	/*Regex - ^-1*/
regex_t R492;	/*Regex - ^MicroMax|^Micromax*/
regex_t R493;	/*Regex - ^Micromax.+$*/
regex_t R494;	/*Regex - jig browser*/
regex_t R495;	/*Regex - 1.0.4; .+\)$*/
regex_t R496;	/*Regex - Konqueror*/
regex_t R497;	/*Regex - Konqueror/\d\.\d*/
regex_t R498;	/*Regex - Ubuntu*/
regex_t R499;	/*Regex - heritrix*/
regex_t R500;	/*Regex - heritrix/\d\.\d*/
regex_t R501;	/*Regex - Java/\d\.\d*/
regex_t R502;	/*Regex - (?<=Java/)\d\.\d*/
regex_t R503;	/*Regex - Windows .+\)*/
regex_t R504;	/*Regex - Max OS X \d+\.\d*/
regex_t R505;	/*Regex - UCWEB*/
regex_t R506;	/*Regex - (?<=UCBrowser)(|/)\d\.\d*/
regex_t R507;	/*Regex - (?<=\w\w-\w\w;)[^\)]+*/
regex_t R508;	/*Regex - ^MacOutlook*/
regex_t R509;	/*Regex - (?<=MacOutlook/)[\d\.]+*/
regex_t R510;	/*Regex - (?<=Mac OS X )[\d\.]+*/
regex_t R511;	/*Regex - \w+*/


// Initialises the properties provided.
void initSpecificProperties(char *properties) {
	int index = 0;
	char *src, *dest, *start;
	_propertiesCount = 0;

	// Count the number of properties included in the string
	// provided.
	src = properties - 1;
	do {
		src++;
		if (*src == ',' || *src == '|' || *src == '\0') {
			_propertiesCount++;
		}
	} while (*src != '\0');

	// Copy the string so that we're not dependent on the calling
	// method retaining the original string.
	_allProperties = (char*)malloc(src - properties + 1);
	_properties = (char**)malloc((src - properties + 1) * sizeof(char**));

	// Convert all the | or , to NULL to make property detection easier
	// when properties are returned later.
	dest = _allProperties - 1;
	src = properties - 1;
	start = _allProperties;
	do {
		dest++;
		src++;
		if (*src == ',' || *src == '|' || *src == '\0') {
			*dest = '\0';
			*(_properties + index) = start;
			index++;
			start = dest + 1;
		}
		else
			*dest = *src;
	} while (*src != '\0');
}

// Passed a | or , seperated list of properties. Initialises the properties
// and the regexes used in detection.
int init(char *properties) {

	// If properties are provided initialise them.
	if (properties != NULL && strlen(properties) > 0)
		initSpecificProperties(properties);
	if (fiftyone_degrees_regcomp(&R0, S106041, REG_EXTENDED|REG_NOSUB) != 0) return -2;	/*Regex - (?i)bot|spider|crawler|slurp*/
	if (fiftyone_degrees_regcomp(&R1, S106042, REG_EXTENDED) != 0) return -3;	/*Regex - (?i)bot|spider|crawler|slurp|[^\s]+(?=bot|spider|crawler|slurp)[^\s]+*/
	if (fiftyone_degrees_regcomp(&R2, S106043, REG_EXTENDED) != 0) return -4;	/*Regex - .+*/
	if (fiftyone_degrees_regcomp(&R3, S106044, REG_EXTENDED|REG_NOSUB) != 0) return -5;	/*Regex - W3C_Validator*/
	if (fiftyone_degrees_regcomp(&R4, S106045, REG_EXTENDED|REG_NOSUB) != 0) return -6;	/*Regex - ^Opera/*/
	if (fiftyone_degrees_regcomp(&R5, S106046, REG_EXTENDED|REG_NOSUB) != 0) return -7;	/*Regex - Windows NT 6\.0*/
	if (fiftyone_degrees_regcomp(&R6, S106047, REG_EXTENDED|REG_NOSUB) != 0) return -8;	/*Regex - Windows NT 6\.1*/
	if (fiftyone_degrees_regcomp(&R7, S106048, REG_EXTENDED|REG_NOSUB) != 0) return -9;	/*Regex - Opera Mini|Opera Mobi*/
	if (fiftyone_degrees_regcomp(&R8, S106049, REG_EXTENDED|REG_NOSUB) != 0) return -10;	/*Regex - MIDP|CLDC*/
	if (fiftyone_degrees_regcomp(&R9, S106050, REG_EXTENDED) != 0) return -11;	/*Regex - (?<=Opera/)[\d+.]+*/
	if (fiftyone_degrees_regcomp(&R10, S106051, REG_EXTENDED) != 0) return -12;	/*Regex - (?<=Version/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R11, S106052, REG_EXTENDED) != 0) return -13;	/*Regex - (?<=\()[^;]+*/
	if (fiftyone_degrees_regcomp(&R12, S106053, REG_EXTENDED) != 0) return -14;	/*Regex - (?<=Opera/\d\.\d \()[^;\)]+*/
	if (fiftyone_degrees_regcomp(&R13, S106054, REG_EXTENDED|REG_NOSUB) != 0) return -15;	/*Regex - Android( |)2\.2\.1*/
	if (fiftyone_degrees_regcomp(&R14, S106055, REG_EXTENDED) != 0) return -16;	/*Regex - (?<=;)([^;\)]+)(?=Build/)*/
	if (fiftyone_degrees_regcomp(&R15, S106056, REG_EXTENDED) != 0) return -17;	/*Regex - (?<=\w\w(_|-)\w\w;)[^;\)]+*/
	if (fiftyone_degrees_regcomp(&R16, S106057, REG_EXTENDED) != 0) return -18;	/*Regex - (?<=;\sAndroid)[\d\s][^;\)]+*/
	if (fiftyone_degrees_regcomp(&R17, S106058, REG_EXTENDED) != 0) return -19;	/*Regex - (?<=UCWEB)\d+\.\d+|(?<=UCWEB/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R18, S106059, REG_EXTENDED) != 0) return -20;	/*Regex - (?<=UC Browser)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R19, S106060, REG_EXTENDED) != 0) return -21;	/*Regex - (?<=UP\.Browser)[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R20, S106061, REG_EXTENDED) != 0) return -22;	/*Regex - (?<=Opera/)\d+\.\d+|(?<=Opera )\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R21, S106062, REG_EXTENDED) != 0) return -23;	/*Regex - (?<=Opera Mini/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R22, S106063, REG_EXTENDED) != 0) return -24;	/*Regex - (?<=Opera Mobi/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R23, S106064, REG_EXTENDED) != 0) return -25;	/*Regex - (?<=Ninesky-android-mobile/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R24, S106065, REG_EXTENDED) != 0) return -26;	/*Regex - (?<=CrMo/)\d+*/
	if (fiftyone_degrees_regcomp(&R25, S106066, REG_EXTENDED) != 0) return -27;	/*Regex - (?<=Chrome/)\d+*/
	if (fiftyone_degrees_regcomp(&R26, S106067, REG_EXTENDED) != 0) return -28;	/*Regex - (?<=AppleWebKit/)[\d\.\+]+*/
	if (fiftyone_degrees_regcomp(&R27, S106068, REG_EXTENDED) != 0) return -29;	/*Regex - (?<=Mobile Safari/)[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R28, S106069, REG_EXTENDED) != 0) return -30;	/*Regex - (?<=Flipboard/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R29, S106070, REG_EXTENDED) != 0) return -31;	/*Regex - MQQBrowser/(\d+\.\d|\d+ )*/
	if (fiftyone_degrees_regcomp(&R30, S106071, REG_EXTENDED) != 0) return -32;	/*Regex - Maxthon*/
	if (fiftyone_degrees_regcomp(&R31, S106072, REG_EXTENDED) != 0) return -33;	/*Regex - UCBrowser/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R32, S106073, REG_EXTENDED) != 0) return -34;	/*Regex - iBrowser/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R33, S106074, REG_EXTENDED) != 0) return -35;	/*Regex - TeamStream*/
	if (fiftyone_degrees_regcomp(&R34, S106075, REG_EXTENDED) != 0) return -36;	/*Regex - Maxthon/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R35, S106076, REG_EXTENDED) != 0) return -37;	/*Regex - Ninesky/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R36, S106077, REG_EXTENDED) != 0) return -38;	/*Regex - OPR/\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R37, S106078, REG_EXTENDED|REG_NOSUB) != 0) return -39;	/*Regex - Android( |)2\.2\.2*/
	if (fiftyone_degrees_regcomp(&R38, S106079, REG_EXTENDED) != 0) return -40;	/*Regex - MQQBrowser/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R39, S106080, REG_EXTENDED|REG_NOSUB) != 0) return -41;	/*Regex - Android( |)2\.2\.3*/
	if (fiftyone_degrees_regcomp(&R40, S106081, REG_EXTENDED|REG_NOSUB) != 0) return -42;	/*Regex - Android( |)2\.3\.1*/
	if (fiftyone_degrees_regcomp(&R41, S106082, REG_EXTENDED|REG_NOSUB) != 0) return -43;	/*Regex - Android( |)2\.3\.2*/
	if (fiftyone_degrees_regcomp(&R42, S106083, REG_EXTENDED|REG_NOSUB) != 0) return -44;	/*Regex - Android( |)2\.3\.3*/
	if (fiftyone_degrees_regcomp(&R43, S106084, REG_EXTENDED) != 0) return -45;	/*Regex - (?<=UCBrowser)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R44, S106085, REG_EXTENDED) != 0) return -46;	/*Regex - (?<=UCBrowser/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R45, S106086, REG_EXTENDED) != 0) return -47;	/*Regex - ^(One|i)Browser/(\d+|\d+\.\d)*/
	if (fiftyone_degrees_regcomp(&R46, S106087, REG_EXTENDED) != 0) return -48;	/*Regex - Mobile Maxthon*/
	if (fiftyone_degrees_regcomp(&R47, S106088, REG_EXTENDED) != 0) return -49;	/*Regex - \)Maxthon|Maxthon$*/
	if (fiftyone_degrees_regcomp(&R48, S106089, REG_EXTENDED) != 0) return -50;	/*Regex - Ninesky/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R49, S106090, REG_EXTENDED) != 0) return -51;	/*Regex - ^YahooMobile/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R50, S106091, REG_EXTENDED|REG_NOSUB) != 0) return -52;	/*Regex - Android( |)2\.3\.4*/
	if (fiftyone_degrees_regcomp(&R51, S106092, REG_EXTENDED) != 0) return -53;	/*Regex - \)Maxthon*/
	if (fiftyone_degrees_regcomp(&R52, S106093, REG_EXTENDED) != 0) return -54;	/*Regex - Cyanogen*/
	if (fiftyone_degrees_regcomp(&R53, S106094, REG_EXTENDED) != 0) return -55;	/*Regex - Maxthon$*/
	if (fiftyone_degrees_regcomp(&R54, S106095, REG_EXTENDED) != 0) return -56;	/*Regex - (One|i)Browser/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R55, S106096, REG_EXTENDED|REG_NOSUB) != 0) return -57;	/*Regex - Android( |)2\.3\.5*/
	if (fiftyone_degrees_regcomp(&R56, S106097, REG_EXTENDED) != 0) return -58;	/*Regex - OneBrowser/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R57, S106098, REG_EXTENDED) != 0) return -59;	/*Regex - Maxthon(/\d+\.\d+|$)*/
	if (fiftyone_degrees_regcomp(&R58, S106099, REG_EXTENDED) != 0) return -60;	/*Regex - UCBrowser/\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R59, S106100, REG_EXTENDED) != 0) return -61;	/*Regex - ^iBrowser/(\d+\.\d|\d+ )*/
	if (fiftyone_degrees_regcomp(&R60, S106101, REG_EXTENDED|REG_NOSUB) != 0) return -62;	/*Regex - Android( |)2\.3\.6*/
	if (fiftyone_degrees_regcomp(&R61, S106102, REG_EXTENDED) != 0) return -63;	/*Regex - OneBrowser/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R62, S106103, REG_EXTENDED|REG_NOSUB) != 0) return -64;	/*Regex - Android( |)2\.3\.7*/
	if (fiftyone_degrees_regcomp(&R63, S106104, REG_EXTENDED) != 0) return -65;	/*Regex - (?<=CyanogenMod-)\d*/
	if (fiftyone_degrees_regcomp(&R64, S106105, REG_EXTENDED) != 0) return -66;	/*Regex - (?<=CyanogenMod)\d*/
	if (fiftyone_degrees_regcomp(&R65, S106106, REG_EXTENDED) != 0) return -67;	/*Regex - CyanogenMod*/
	if (fiftyone_degrees_regcomp(&R66, S106107, REG_EXTENDED) != 0) return -68;	/*Regex - CM7*/
	if (fiftyone_degrees_regcomp(&R67, S106108, REG_EXTENDED) != 0) return -69;	/*Regex - MQQBrowser/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R68, S106109, REG_EXTENDED|REG_NOSUB) != 0) return -70;	/*Regex - Chrome/[0-9][^\d]*/
	if (fiftyone_degrees_regcomp(&R69, S50458, REG_EXTENDED|REG_NOSUB) != 0) return -71;	/*Regex - Android*/
	if (fiftyone_degrees_regcomp(&R70, S106110, REG_EXTENDED) != 0) return -72;	/*Regex - (?<=Chrome/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R71, S106111, REG_EXTENDED) != 0) return -73;	/*Regex - (?<=\(|;|,)[^\);]+*/
	if (fiftyone_degrees_regcomp(&R72, S106112, REG_EXTENDED) != 0) return -74;	/*Regex - (?<=RockMelt/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R73, S106113, REG_EXTENDED) != 0) return -75;	/*Regex - Ubuntu/\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R74, S106114, REG_EXTENDED) != 0) return -76;	/*Regex - Sleipnir/\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R75, S106115, REG_EXTENDED) != 0) return -77;	/*Regex - Nutch-\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R76, S106116, REG_EXTENDED) != 0) return -78;	/*Regex - Comodo_Dragon/\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R77, S106117, REG_EXTENDED) != 0) return -79;	/*Regex - Iron/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R78, S106118, REG_EXTENDED) != 0) return -80;	/*Regex - GoogleTV*/
	if (fiftyone_degrees_regcomp(&R79, S106119, REG_EXTENDED) != 0) return -81;	/*Regex - chromeframe/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R80, S106120, REG_EXTENDED) != 0) return -82;	/*Regex - Flock/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R81, S106121, REG_EXTENDED|REG_NOSUB) != 0) return -83;	/*Regex - Chrome/1[0-9]*/
	if (fiftyone_degrees_regcomp(&R82, S106122, REG_EXTENDED) != 0) return -84;	/*Regex - (?<=^).+(?=Mozilla)*/
	if (fiftyone_degrees_regcomp(&R83, S106123, REG_EXTENDED) != 0) return -85;	/*Regex - Puffin/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R84, S106124, REG_EXTENDED) != 0) return -86;	/*Regex - (?<=QQBrowser/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R85, S106125, REG_EXTENDED) != 0) return -87;	/*Regex - AvantBrowser/Tri-Core*/
	if (fiftyone_degrees_regcomp(&R86, S106126, REG_EXTENDED) != 0) return -88;	/*Regex - (?<=Midori/)\d\.\d*/
	if (fiftyone_degrees_regcomp(&R87, S106127, REG_EXTENDED|REG_NOSUB) != 0) return -89;	/*Regex - Chrome/2[0-9]*/
	if (fiftyone_degrees_regcomp(&R88, S106128, REG_EXTENDED) != 0) return -90;	/*Regex - Ubuntu Chromium*/
	if (fiftyone_degrees_regcomp(&R89, S106129, REG_EXTENDED|REG_NOSUB) != 0) return -91;	/*Regex - Chrome/3[0-9]*/
	if (fiftyone_degrees_regcomp(&R90, S106130, REG_EXTENDED) != 0) return -92;	/*Regex - Linux Mint*/
	if (fiftyone_degrees_regcomp(&R91, S106131, REG_EXTENDED) != 0) return -93;	/*Regex - Ubuntu;*/
	if (fiftyone_degrees_regcomp(&R92, S106132, REG_EXTENDED) != 0) return -94;	/*Regex - ASUS\)*/
	if (fiftyone_degrees_regcomp(&R93, S106133, REG_EXTENDED) != 0) return -95;	/*Regex - Mac OS X \d+(\.|_)\d*/
	if (fiftyone_degrees_regcomp(&R94, S106134, REG_EXTENDED) != 0) return -96;	/*Regex - Windows NT \d\.\d*/
	if (fiftyone_degrees_regcomp(&R95, S106135, REG_EXTENDED) != 0) return -97;	/*Regex - (?<=KDL)[^;]+*/
	if (fiftyone_degrees_regcomp(&R96, S106136, REG_EXTENDED|REG_NOSUB) != 0) return -98;	/*Regex - SonyEricsson*/
	if (fiftyone_degrees_regcomp(&R97, S106137, REG_EXTENDED) != 0) return -99;	/*Regex - (?<=SonyEricsson)[^/]+*/
	if (fiftyone_degrees_regcomp(&R98, S106138, REG_EXTENDED) != 0) return -100;	/*Regex - (?<=NetFront/)[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R99, S106139, REG_EXTENDED) != 0) return -101;	/*Regex - (?<=Opera )[^\s]+*/
	if (fiftyone_degrees_regcomp(&R100, S106140, REG_EXTENDED) != 0) return -102;	/*Regex - (?<=SEMC-Browser/)[^\s]+*/
	if (fiftyone_degrees_regcomp(&R101, S106141, REG_EXTENDED) != 0) return -103;	/*Regex - (?<=TelecaBrowser/)[^\s]+*/
	if (fiftyone_degrees_regcomp(&R102, S106142, REG_EXTENDED) != 0) return -104;	/*Regex - (?<=SymbianOS/)[^;]+ */
	if (fiftyone_degrees_regcomp(&R103, S106143, REG_EXTENDED) != 0) return -105;	/*Regex - (?<=Series60/)[^\s]+*/
	if (fiftyone_degrees_regcomp(&R104, S106144, REG_EXTENDED) != 0) return -106;	/*Regex - (?<=U(C|P)\.Browser/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R105, S106145, REG_EXTENDED) != 0) return -107;	/*Regex - (?<=UCBrowser)(|/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R106, S106146, REG_EXTENDED) != 0) return -108;	/*Regex - UCWEB/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R107, S106147, REG_EXTENDED|REG_NOSUB) != 0) return -109;	/*Regex - Silk-Accelerated*/
	if (fiftyone_degrees_regcomp(&R108, S106148, REG_EXTENDED) != 0) return -110;	/*Regex - (?<=Silk/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R109, S106149, REG_EXTENDED) != 0) return -111;	/*Regex - (?<=Accelerated=)(true|false)*/
	if (fiftyone_degrees_regcomp(&R110, S106150, REG_EXTENDED|REG_NOSUB) != 0) return -112;	/*Regex - Android \d|Android\d*/
	if (fiftyone_degrees_regcomp(&R111, S106151, REG_EXTENDED) != 0) return -113;	/*Regex - GT-\w\d+*/
	if (fiftyone_degrees_regcomp(&R112, S106152, REG_EXTENDED) != 0) return -114;	/*Regex - 2\.2/FROYO*/
	if (fiftyone_degrees_regcomp(&R113, S106153, REG_EXTENDED) != 0) return -115;	/*Regex - theScore/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R114, S106154, REG_EXTENDED) != 0) return -116;	/*Regex - ^Instagram \d\.\d*/
	if (fiftyone_degrees_regcomp(&R115, S106155, REG_EXTENDED) != 0) return -117;	/*Regex - ^WhatsApp/2\.7*/
	if (fiftyone_degrees_regcomp(&R116, S106156, REG_EXTENDED) != 0) return -118;	/*Regex - (FROYO|GINGERBREAD)\)$*/
	if (fiftyone_degrees_regcomp(&R117, S106157, REG_EXTENDED|REG_NOSUB) != 0) return -119;	/*Regex - Android 1|Android1*/
	if (fiftyone_degrees_regcomp(&R118, S106158, REG_EXTENDED|REG_NOSUB) != 0) return -120;	/*Regex - Android 2\.0|Android2\.0*/
	if (fiftyone_degrees_regcomp(&R119, S106159, REG_EXTENDED|REG_NOSUB) != 0) return -121;	/*Regex - Android 2\.1|Android2\.1*/
	if (fiftyone_degrees_regcomp(&R120, S106160, REG_EXTENDED|REG_NOSUB) != 0) return -122;	/*Regex - Android 2\.2|Android2\.2*/
	if (fiftyone_degrees_regcomp(&R121, S106161, REG_EXTENDED) != 0) return -123;	/*Regex - Maxthon/\d+\.\d|Maxthon*/
	if (fiftyone_degrees_regcomp(&R122, S106162, REG_EXTENDED) != 0) return -124;	/*Regex - ^iBrowser/\d+*/
	if (fiftyone_degrees_regcomp(&R123, S106163, REG_EXTENDED|REG_NOSUB) != 0) return -125;	/*Regex - Android 2\.3|Android2\.3*/
	if (fiftyone_degrees_regcomp(&R124, S106164, REG_EXTENDED|REG_NOSUB) != 0) return -126;	/*Regex - Android 2\.4|Android2\.4*/
	if (fiftyone_degrees_regcomp(&R125, S106165, REG_EXTENDED|REG_NOSUB) != 0) return -127;	/*Regex - Android 3|Android3*/
	if (fiftyone_degrees_regcomp(&R126, S106166, REG_EXTENDED) != 0) return -128;	/*Regex - Flipboard/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R127, S106167, REG_EXTENDED) != 0) return -129;	/*Regex - \[operamini\]*/
	if (fiftyone_degrees_regcomp(&R128, S106168, REG_EXTENDED) != 0) return -130;	/*Regex - ^(One|i)Browser/(\d+\.\d|\d+ )*/
	if (fiftyone_degrees_regcomp(&R129, S106169, REG_EXTENDED) != 0) return -131;	/*Regex - (?<=OverDrive Media Console/)\d\.\d*/
	if (fiftyone_degrees_regcomp(&R130, S106170, REG_EXTENDED|REG_NOSUB) != 0) return -132;	/*Regex - Android( |)4\.0*/
	if (fiftyone_degrees_regcomp(&R131, S106171, REG_EXTENDED|REG_NOSUB) != 0) return -133;	/*Regex - Android( |)4\.0\.\d+*/
	if (fiftyone_degrees_regcomp(&R132, S106172, REG_EXTENDED) != 0) return -134;	/*Regex - Chrome/\d+*/
	if (fiftyone_degrees_regcomp(&R133, S106173, REG_EXTENDED|REG_NOSUB) != 0) return -135;	/*Regex - Android 4\.0\.1|Android4\.0\.1*/
	if (fiftyone_degrees_regcomp(&R134, S106174, REG_EXTENDED) != 0) return -136;	/*Regex - (?<=^).+(?=Linux)*/
	if (fiftyone_degrees_regcomp(&R135, S106175, REG_EXTENDED|REG_NOSUB) != 0) return -137;	/*Regex - Android 4\.0\.2|Android4\.0\.2*/
	if (fiftyone_degrees_regcomp(&R136, S106176, REG_EXTENDED|REG_NOSUB) != 0) return -138;	/*Regex - Android 4\.0\.3|Android4\.0\.3*/
	if (fiftyone_degrees_regcomp(&R137, S106177, REG_EXTENDED) != 0) return -139;	/*Regex - Maxthon(/| \()\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R138, S106178, REG_EXTENDED) != 0) return -140;	/*Regex - ^(One|i)Browser/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R139, S106179, REG_EXTENDED|REG_NOSUB) != 0) return -141;	/*Regex - Android 4\.0\.4|Android4\.0\.4*/
	if (fiftyone_degrees_regcomp(&R140, S106180, REG_EXTENDED) != 0) return -142;	/*Regex - (One|i)Browser/(\d+\.\d|\d+ )*/
	if (fiftyone_degrees_regcomp(&R141, S106181, REG_EXTENDED) != 0) return -143;	/*Regex - Maxthon \(\d\.\d*/
	if (fiftyone_degrees_regcomp(&R142, S106182, REG_EXTENDED) != 0) return -144;	/*Regex - PARANOIDANDROID*/
	if (fiftyone_degrees_regcomp(&R143, S106183, REG_EXTENDED|REG_NOSUB) != 0) return -145;	/*Regex - Android 4\.0\.(5|6|8|9)*/
	if (fiftyone_degrees_regcomp(&R144, S106184, REG_EXTENDED|REG_NOSUB) != 0) return -146;	/*Regex - Android 4\.1|Android4\.1*/
	if (fiftyone_degrees_regcomp(&R145, S106185, REG_EXTENDED|REG_NOSUB) != 0) return -147;	/*Regex - Android( |)4\.1\.\d+*/
	if (fiftyone_degrees_regcomp(&R146, S106186, REG_EXTENDED|REG_NOSUB) != 0) return -148;	/*Regex - Android 4\.1\.1|Android4\.1\.1*/
	if (fiftyone_degrees_regcomp(&R147, S106187, REG_EXTENDED) != 0) return -149;	/*Regex - Maxthon$|\)Maxthon*/
	if (fiftyone_degrees_regcomp(&R148, S106188, REG_EXTENDED) != 0) return -150;	/*Regex - ^OneBrowser/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R149, S106189, REG_EXTENDED) != 0) return -151;	/*Regex - MiuiBrowser/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R150, S106190, REG_EXTENDED|REG_NOSUB) != 0) return -152;	/*Regex - Android 4\.1\.2|Android4\.1\.2*/
	if (fiftyone_degrees_regcomp(&R151, S106191, REG_EXTENDED) != 0) return -153;	/*Regex - (?<=^OneBrowser/)\d\.\d*/
	if (fiftyone_degrees_regcomp(&R152, S106192, REG_EXTENDED|REG_NOSUB) != 0) return -154;	/*Regex - Android 4\.2|Android4\.2*/
	if (fiftyone_degrees_regcomp(&R153, S106193, REG_EXTENDED|REG_NOSUB) != 0) return -155;	/*Regex - Android( |)4\.2\.\d+*/
	if (fiftyone_degrees_regcomp(&R154, S106194, REG_EXTENDED|REG_NOSUB) != 0) return -156;	/*Regex - Android 4\.2\.1|Android4\.2\.1*/
	if (fiftyone_degrees_regcomp(&R155, S106195, REG_EXTENDED) != 0) return -157;	/*Regex - Maxthon(/\d+\.\d|$)*/
	if (fiftyone_degrees_regcomp(&R156, S106196, REG_EXTENDED|REG_NOSUB) != 0) return -158;	/*Regex - Android 4\.2\.2|Android4\.2\.2*/
	if (fiftyone_degrees_regcomp(&R157, S106197, REG_EXTENDED) != 0) return -159;	/*Regex - Maxthon \(\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R158, S106198, REG_EXTENDED) != 0) return -160;	/*Regex - CARBON-JB*/
	if (fiftyone_degrees_regcomp(&R159, S106199, REG_EXTENDED) != 0) return -161;	/*Regex - Liqiud-JB*/
	if (fiftyone_degrees_regcomp(&R160, S106200, REG_EXTENDED|REG_NOSUB) != 0) return -162;	/*Regex - \((iPhone|iPad|iPod)*/
	if (fiftyone_degrees_regcomp(&R161, S106201, REG_EXTENDED|REG_NOSUB) != 0) return -163;	/*Regex - Flipboard*/
	if (fiftyone_degrees_regcomp(&R162, S106202, REG_EXTENDED|REG_NOSUB) != 0) return -164;	/*Regex - Nokia|nokia*/
	if (fiftyone_degrees_regcomp(&R163, S106203, REG_EXTENDED|REG_NOSUB) != 0) return -165;	/*Regex - webOS|hpwOS*/
	if (fiftyone_degrees_regcomp(&R164, S106204, REG_EXTENDED) != 0) return -166;	/*Regex - (?<=\()iPhone|iPad|iPod*/
	if (fiftyone_degrees_regcomp(&R165, S106205, REG_EXTENDED) != 0) return -167;	/*Regex - (?<=Version/)[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R166, S106206, REG_EXTENDED) != 0) return -168;	/*Regex - (?<=OS )[\d(_|\.)\d]+*/
	if (fiftyone_degrees_regcomp(&R167, S106207, REG_EXTENDED) != 0) return -169;	/*Regex - (?<=Sleipnir/)[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R168, S106208, REG_EXTENDED) != 0) return -170;	/*Regex - (?<=AppleWebKit/)[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R169, S106209, REG_EXTENDED) != 0) return -171;	/*Regex - (?<=Mobile/)[\d\w]+*/
	if (fiftyone_degrees_regcomp(&R170, S106210, REG_EXTENDED) != 0) return -172;	/*Regex - (?<=Safari/)[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R171, S106211, REG_EXTENDED) != 0) return -173;	/*Regex - (?<=Dolphin )[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R172, S106212, REG_EXTENDED) != 0) return -174;	/*Regex - (?<=Dolphin HD )[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R173, S106213, REG_EXTENDED) != 0) return -175;	/*Regex - Twitter*/
	if (fiftyone_degrees_regcomp(&R174, S106214, REG_EXTENDED) != 0) return -176;	/*Regex - Miu Ptt*/
	if (fiftyone_degrees_regcomp(&R175, S106215, REG_EXTENDED) != 0) return -177;	/*Regex - Incredimail/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R176, S106216, REG_EXTENDED) != 0) return -178;	/*Regex - PhantomJS/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R177, S106217, REG_EXTENDED) != 0) return -179;	/*Regex - UP\.Browser/(\d+\.\d|\d+ )*/
	if (fiftyone_degrees_regcomp(&R178, S106218, REG_EXTENDED|REG_NOSUB) != 0) return -180;	/*Regex - BOLT*/
	if (fiftyone_degrees_regcomp(&R179, S106219, REG_EXTENDED|REG_NOSUB) != 0) return -181;	/*Regex - mac*/
	if (fiftyone_degrees_regcomp(&R180, S106220, REG_EXTENDED|REG_NOSUB) != 0) return -182;	/*Regex - win*/
	if (fiftyone_degrees_regcomp(&R181, S106221, REG_EXTENDED) != 0) return -183;	/*Regex - (?<=BOLT/)[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R182, S106222, REG_EXTENDED|REG_NOSUB) != 0) return -184;	/*Regex - (F|f)ire(F|f)o(X|x)|(I|i)ceweasel|BonEcho*/
	if (fiftyone_degrees_regcomp(&R183, S106223, REG_EXTENDED|REG_NOSUB) != 0) return -185;	/*Regex - \(Macintosh;|\(Win|\(X1\d;|WOW64*/
	if (fiftyone_degrees_regcomp(&R184, S106224, REG_EXTENDED|REG_NOSUB) != 0) return -186;	/*Regex - Fennec|Linux Mint|Thunderbird*/
	if (fiftyone_degrees_regcomp(&R185, S106225, REG_EXTENDED) != 0) return -187;	/*Regex - (?<=X11;)\w+*/
	if (fiftyone_degrees_regcomp(&R186, S106226, REG_EXTENDED) != 0) return -188;	/*Regex - (?<=Firefox)/\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R187, S106227, REG_EXTENDED) != 0) return -189;	/*Regex - (?<=Iceweasel)/\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R188, S106228, REG_EXTENDED) != 0) return -190;	/*Regex - (?<=Mozilla/)\d\.\d*/
	if (fiftyone_degrees_regcomp(&R189, S106229, REG_EXTENDED) != 0) return -191;	/*Regex - (?<=Mozilla/\d\.\d \()[^;\)]+*/
	if (fiftyone_degrees_regcomp(&R190, S106230, REG_EXTENDED) != 0) return -192;	/*Regex - Windows NT \d\.\d(?=;)|Mac OS X [^\s]+(?=;)|Ubuntu|X11|SunOS|OpenBSD|FreeBSD*/
	if (fiftyone_degrees_regcomp(&R191, S106231, REG_EXTENDED) != 0) return -193;	/*Regex - Fedora/\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R192, S106232, REG_EXTENDED) != 0) return -194;	/*Regex - Nutch*/
	if (fiftyone_degrees_regcomp(&R193, S106233, REG_EXTENDED) != 0) return -195;	/*Regex - Debian-\d\.\d*/
	if (fiftyone_degrees_regcomp(&R194, S106234, REG_EXTENDED) != 0) return -196;	/*Regex - \w(U|u)buntu/\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R195, S106235, REG_EXTENDED) != 0) return -197;	/*Regex - (?<=\((U|u)buntu-)[^\)]+*/
	if (fiftyone_degrees_regcomp(&R196, S106236, REG_EXTENDED) != 0) return -198;	/*Regex - (?<=Firefox)[^\d]+*/
	if (fiftyone_degrees_regcomp(&R197, S106237, REG_EXTENDED) != 0) return -199;	/*Regex - (?<=Win)[^;]+*/
	if (fiftyone_degrees_regcomp(&R198, S106238, REG_EXTENDED) != 0) return -200;	/*Regex - Fedora/*/
	if (fiftyone_degrees_regcomp(&R199, S106239, REG_EXTENDED) != 0) return -201;	/*Regex - Pale(M|m)oon/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R200, S106240, REG_EXTENDED) != 0) return -202;	/*Regex - (E|e)pic/\d+\.\d|Epic/*/
	if (fiftyone_degrees_regcomp(&R201, S106241, REG_EXTENDED) != 0) return -203;	/*Regex - Comet(Bird|Browser)/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R202, S106242, REG_EXTENDED) != 0) return -204;	/*Regex - Kylo/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R203, S106243, REG_EXTENDED) != 0) return -205;	/*Regex - Wyzo/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R204, S106244, REG_EXTENDED) != 0) return -206;	/*Regex - Camino/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R205, S106245, REG_EXTENDED) != 0) return -207;	/*Regex - Minefield/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R206, S106246, REG_EXTENDED) != 0) return -208;	/*Regex - Navigator/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R207, S106247, REG_EXTENDED) != 0) return -209;	/*Regex - IceDragon/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R208, S106248, REG_EXTENDED) != 0) return -210;	/*Regex - Opera \d+\.\d\d$*/
	if (fiftyone_degrees_regcomp(&R209, S106249, REG_EXTENDED) != 0) return -211;	/*Regex - lolifox/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R210, S106250, REG_EXTENDED) != 0) return -212;	/*Regex - Avant TriCore*/
	if (fiftyone_degrees_regcomp(&R211, S106251, REG_EXTENDED) != 0) return -213;	/*Regex - Midori/(|\")\d+(|\")\.(|\")\d(|\")*/
	if (fiftyone_degrees_regcomp(&R212, S106252, REG_EXTENDED) != 0) return -214;	/*Regex - SeaMonkey/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R213, S106253, REG_EXTENDED) != 0) return -215;	/*Regex - firefox/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R214, S106254, REG_EXTENDED) != 0) return -216;	/*Regex - BonEcho/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R215, S106255, REG_EXTENDED) != 0) return -217;	/*Regex - Maemo Browser \d\.\d*/
	if (fiftyone_degrees_regcomp(&R216, S106256, REG_EXTENDED) != 0) return -218;	/*Regex - Linux mips*/
	if (fiftyone_degrees_regcomp(&R217, S106257, REG_EXTENDED|REG_NOSUB) != 0) return -219;	/*Regex - Firefox/*/
	if (fiftyone_degrees_regcomp(&R218, S106258, REG_EXTENDED|REG_NOSUB) != 0) return -220;	/*Regex - Fennec/|\(Mobile; rv*/
	if (fiftyone_degrees_regcomp(&R219, S106259, REG_EXTENDED) != 0) return -221;	/*Regex - (?<=Mozilla/)[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R220, S106260, REG_EXTENDED) != 0) return -222;	/*Regex - (?<=Fennec/)[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R221, S106261, REG_EXTENDED) != 0) return -223;	/*Regex - Fennec*/
	if (fiftyone_degrees_regcomp(&R222, S106262, REG_EXTENDED) != 0) return -224;	/*Regex - (?<=\()[^\)]+*/
	if (fiftyone_degrees_regcomp(&R223, S106263, REG_EXTENDED) != 0) return -225;	/*Regex - (?<=Gecko/)[\d]+*/
	if (fiftyone_degrees_regcomp(&R224, S106264, REG_EXTENDED) != 0) return -226;	/*Regex - (?<=Firefox/)[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R225, S106265, REG_EXTENDED|REG_NOSUB) != 0) return -227;	/*Regex - MSIE*/
	if (fiftyone_degrees_regcomp(&R226, S106266, REG_EXTENDED|REG_NOSUB) != 0) return -228;	/*Regex - compatible*/
	if (fiftyone_degrees_regcomp(&R227, S106267, REG_EXTENDED|REG_NOSUB) != 0) return -229;	/*Regex - Windows XP*/
	if (fiftyone_degrees_regcomp(&R228, S106268, REG_EXTENDED|REG_NOSUB) != 0) return -230;	/*Regex - Windows NT*/
	if (fiftyone_degrees_regcomp(&R229, S106269, REG_EXTENDED|REG_NOSUB) != 0) return -231;	/*Regex - Windows ME*/
	if (fiftyone_degrees_regcomp(&R230, S106270, REG_EXTENDED|REG_NOSUB) != 0) return -232;	/*Regex - Win32*/
	if (fiftyone_degrees_regcomp(&R231, S106271, REG_EXTENDED|REG_NOSUB) != 0) return -233;	/*Regex - Windows [\d\.]+*/
	if (fiftyone_degrees_regcomp(&R232, S106272, REG_EXTENDED|REG_NOSUB) != 0) return -234;	/*Regex - Windows NT [\d\.]+*/
	if (fiftyone_degrees_regcomp(&R233, S106273, REG_EXTENDED|REG_NOSUB) != 0) return -235;	/*Regex - Mac_PowerPC*/
	if (fiftyone_degrees_regcomp(&R234, S106274, REG_EXTENDED|REG_NOSUB) != 0) return -236;	/*Regex - PPC Mac OS*/
	if (fiftyone_degrees_regcomp(&R235, S106275, REG_EXTENDED|REG_NOSUB) != 0) return -237;	/*Regex - Mobile*/
	if (fiftyone_degrees_regcomp(&R236, S106276, REG_EXTENDED|REG_NOSUB) != 0) return -238;	/*Regex - Windows CE*/
	if (fiftyone_degrees_regcomp(&R237, S106277, REG_EXTENDED|REG_NOSUB) != 0) return -239;	/*Regex - Phone*/
	if (fiftyone_degrees_regcomp(&R238, S106278, REG_EXTENDED|REG_NOSUB) != 0) return -240;	/*Regex - UC(\.| ||)Browser*/
	if (fiftyone_degrees_regcomp(&R239, S106279, REG_EXTENDED|REG_NOSUB) != 0) return -241;	/*Regex - Opera*/
	if (fiftyone_degrees_regcomp(&R240, S106280, REG_EXTENDED|REG_NOSUB) != 0) return -242;	/*Regex - (?i)LG-|LG_|SC-|Toshiba|XV\d+|Nokia|LGE|PalmOS|POLARIS|PalmSource|Nutch|NetFront|Samsung|HTC*/
	if (fiftyone_degrees_regcomp(&R241, S106281, REG_EXTENDED) != 0) return -243;	/*Regex - (?<=MSIE\s)[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R242, S106282, REG_EXTENDED) != 0) return -244;	/*Regex - (?<=Outlook )[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R243, S106283, REG_EXTENDED) != 0) return -245;	/*Regex - (?<=Windows)[\s\w\d\.]+*/
	if (fiftyone_degrees_regcomp(&R244, S106284, REG_EXTENDED) != 0) return -246;	/*Regex - Windows NT \d\.\d(?=;)|Mac OS X [^\s]+(?=;)|Ubuntu|X11*/
	if (fiftyone_degrees_regcomp(&R245, S106285, REG_EXTENDED) != 0) return -247;	/*Regex - chromeframe*/
	if (fiftyone_degrees_regcomp(&R246, S106286, REG_EXTENDED) != 0) return -248;	/*Regex - Media Center PC*/
	if (fiftyone_degrees_regcomp(&R247, S39426, REG_EXTENDED) != 0) return -249;	/*Regex - Linux*/
	if (fiftyone_degrees_regcomp(&R248, S106287, REG_EXTENDED) != 0) return -250;	/*Regex - Zune*/
	if (fiftyone_degrees_regcomp(&R249, S106288, REG_EXTENDED) != 0) return -251;	/*Regex - ARM*/
	if (fiftyone_degrees_regcomp(&R250, S106289, REG_EXTENDED) != 0) return -252;	/*Regex - Touch*/
	if (fiftyone_degrees_regcomp(&R251, S106290, REG_EXTENDED) != 0) return -253;	/*Regex - WOW64;*/
	if (fiftyone_degrees_regcomp(&R252, S106291, REG_EXTENDED) != 0) return -254;	/*Regex - (?<=heritrix/)[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R253, S106292, REG_EXTENDED) != 0) return -255;	/*Regex - Maxthon|MAXTHON*/
	if (fiftyone_degrees_regcomp(&R254, S106293, REG_EXTENDED) != 0) return -256;	/*Regex - SlimBrowser/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R255, S106294, REG_EXTENDED) != 0) return -257;	/*Regex - Avant(| )Browser*/
	if (fiftyone_degrees_regcomp(&R256, S106295, REG_EXTENDED) != 0) return -258;	/*Regex - Xbox*/
	if (fiftyone_degrees_regcomp(&R257, S106296, REG_EXTENDED) != 0) return -259;	/*Regex - Midori/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R258, S106297, REG_EXTENDED) != 0) return -260;	/*Regex - (?<=Maxthon)(/| )\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R259, S106298, REG_EXTENDED) != 0) return -261;	/*Regex - (?<=MAXTHON) \d\.\d*/
	if (fiftyone_degrees_regcomp(&R260, S106299, REG_EXTENDED) != 0) return -262;	/*Regex - Win(95|32|98|NT4\.0|64|3\.\d)*/
	if (fiftyone_degrees_regcomp(&R261, S106300, REG_EXTENDED) != 0) return -263;	/*Regex - PPC*/
	if (fiftyone_degrees_regcomp(&R262, S106301, REG_EXTENDED) != 0) return -264;	/*Regex - Tablet PC*/
	if (fiftyone_degrees_regcomp(&R263, S106302, REG_EXTENDED) != 0) return -265;	/*Regex - SV1*/
	if (fiftyone_degrees_regcomp(&R264, S106303, REG_EXTENDED) != 0) return -266;	/*Regex - (?<=UCWEB)\d\.\d*/
	if (fiftyone_degrees_regcomp(&R265, S106304, REG_EXTENDED|REG_NOSUB) != 0) return -267;	/*Regex - Nokia*/
	if (fiftyone_degrees_regcomp(&R266, S106305, REG_EXTENDED|REG_NOSUB) != 0) return -268;	/*Regex - Series40*/
	if (fiftyone_degrees_regcomp(&R267, S106306, REG_EXTENDED) != 0) return -269;	/*Regex - (?<=Nokia)[^/]+*/
	if (fiftyone_degrees_regcomp(&R268, S106307, REG_EXTENDED) != 0) return -270;	/*Regex - (?<=Series40)[^;]+*/
	if (fiftyone_degrees_regcomp(&R269, S106308, REG_EXTENDED) != 0) return -271;	/*Regex - (?<=S40OviBrowser/)\d+\.\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R270, S106309, REG_EXTENDED) != 0) return -272;	/*Regex - iBrowser/Mini\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R271, S106310, REG_EXTENDED|REG_NOSUB) != 0) return -273;	/*Regex - Series60*/
	if (fiftyone_degrees_regcomp(&R272, S106311, REG_EXTENDED) != 0) return -274;	/*Regex - (?<=Series60/)[\w\d\.]+*/
	if (fiftyone_degrees_regcomp(&R273, S106312, REG_EXTENDED) != 0) return -275;	/*Regex - MQQBrowser/Mini\d\.\d*/
	if (fiftyone_degrees_regcomp(&R274, S106313, REG_EXTENDED|REG_NOSUB) != 0) return -276;	/*Regex - ^Mozilla*/
	if (fiftyone_degrees_regcomp(&R275, S106314, REG_EXTENDED|REG_NOSUB) != 0) return -277;	/*Regex - Safari*/
	if (fiftyone_degrees_regcomp(&R276, S106315, REG_EXTENDED|REG_NOSUB) != 0) return -278;	/*Regex - Windows|Macintosh|X11*/
	if (fiftyone_degrees_regcomp(&R277, S106316, REG_EXTENDED|REG_NOSUB) != 0) return -279;	/*Regex - Chrome/*/
	if (fiftyone_degrees_regcomp(&R278, S106317, REG_EXTENDED) != 0) return -280;	/*Regex - (?<=Safari/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R279, S106318, REG_EXTENDED) != 0) return -281;	/*Regex - Windows NT \d\.\d(?=;)|Mac OS X [^\s]+(?=;)|Ubuntu( |(/|)\d+\.\d+)|X11*/
	if (fiftyone_degrees_regcomp(&R280, S106319, REG_EXTENDED) != 0) return -282;	/*Regex - (?<=Version/)\d\.\d*/
	if (fiftyone_degrees_regcomp(&R281, S106320, REG_EXTENDED) != 0) return -283;	/*Regex - (?<=Mac OS X 10_\d_\d;)[^;\)]+*/
	if (fiftyone_degrees_regcomp(&R282, S106321, REG_EXTENDED) != 0) return -284;	/*Regex - Mac OS X 10_\d*/
	if (fiftyone_degrees_regcomp(&R283, S106322, REG_EXTENDED) != 0) return -285;	/*Regex - (?<=PhantomJS/)\d\.\d*/
	if (fiftyone_degrees_regcomp(&R284, S106323, REG_EXTENDED) != 0) return -286;	/*Regex - SecondLife/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R285, S106324, REG_EXTENDED) != 0) return -287;	/*Regex - Google Earth/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R286, S106325, REG_EXTENDED) != 0) return -288;	/*Regex - rekonq(/\d+\.\d| )*/
	if (fiftyone_degrees_regcomp(&R287, S106326, REG_EXTENDED) != 0) return -289;	/*Regex - Skyfire/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R288, S106327, REG_EXTENDED) != 0) return -290;	/*Regex - Sleipnir/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R289, S106328, REG_EXTENDED) != 0) return -291;	/*Regex - QupZilla/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R290, S106329, REG_EXTENDED) != 0) return -292;	/*Regex - konqueror/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R291, S106330, REG_EXTENDED) != 0) return -293;	/*Regex - Google Earth Pro/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R292, S106331, REG_EXTENDED) != 0) return -294;	/*Regex - OmniWeb/.+\.\d+*/
	if (fiftyone_degrees_regcomp(&R293, S106332, REG_EXTENDED) != 0) return -295;	/*Regex - Qt/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R294, S106333, REG_EXTENDED) != 0) return -296;	/*Regex - kded/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R295, S106334, REG_EXTENDED) != 0) return -297;	/*Regex - Midori/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R296, S106335, REG_EXTENDED) != 0) return -298;	/*Regex - iCab/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R297, S106336, REG_EXTENDED) != 0) return -299;	/*Regex - (?<=Fluid/)\d\.\d*/
	if (fiftyone_degrees_regcomp(&R298, S106337, REG_EXTENDED) != 0) return -300;	/*Regex - (?<=Iron/)\d\.\d*/
	if (fiftyone_degrees_regcomp(&R299, S106338, REG_EXTENDED) != 0) return -301;	/*Regex - (?<=Stainless/)\d\.\d*/
	if (fiftyone_degrees_regcomp(&R300, S106339, REG_EXTENDED|REG_NOSUB) != 0) return -302;	/*Regex - Skyfire/*/
	if (fiftyone_degrees_regcomp(&R301, S106340, REG_EXTENDED) != 0) return -303;	/*Regex - (?<=Skyfire/)[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R302, S106341, REG_EXTENDED|REG_NOSUB) != 0) return -304;	/*Regex - Windows Phone OS*/
	if (fiftyone_degrees_regcomp(&R303, S106342, REG_EXTENDED|REG_NOSUB) != 0) return -305;	/*Regex - ^WordPress/*/
	if (fiftyone_degrees_regcomp(&R304, S106343, REG_EXTENDED) != 0) return -306;	/*Regex - (?<=http://)\w+*/
	if (fiftyone_degrees_regcomp(&R305, S106344, REG_EXTENDED|REG_NOSUB) != 0) return -307;	/*Regex - Thunderbird*/
	if (fiftyone_degrees_regcomp(&R306, S106345, REG_EXTENDED|REG_NOSUB) != 0) return -308;	/*Regex - \(Macintosh;|\(Windows|\(X11;*/
	if (fiftyone_degrees_regcomp(&R307, S106346, REG_EXTENDED|REG_NOSUB) != 0) return -309;	/*Regex - Firefox|Iceweasel|Fennec|Linux Mint|Ubuntu*/
	if (fiftyone_degrees_regcomp(&R308, S106347, REG_EXTENDED) != 0) return -310;	/*Regex - (?<=Thunderbird/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R309, S106348, REG_EXTENDED) != 0) return -311;	/*Regex - (?<=(Fedora)/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R310, S106349, REG_EXTENDED|REG_NOSUB) != 0) return -312;	/*Regex - Darwin*/
	if (fiftyone_degrees_regcomp(&R311, S106350, REG_EXTENDED|REG_NOSUB) != 0) return -313;	/*Regex - CFNetwork*/
	if (fiftyone_degrees_regcomp(&R312, S64617, REG_EXTENDED|REG_NOSUB) != 0) return -314;	/*Regex - iPhone*/
	if (fiftyone_degrees_regcomp(&R313, S106351, REG_EXTENDED) != 0) return -315;	/*Regex - ^[^/]+*/
	if (fiftyone_degrees_regcomp(&R314, S106352, REG_EXTENDED) != 0) return -316;	/*Regex - (i)?Mac([^\d]+)?*/
	if (fiftyone_degrees_regcomp(&R315, S106353, REG_EXTENDED) != 0) return -317;	/*Regex - (?<=Darwin/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R316, S106354, REG_EXTENDED) != 0) return -318;	/*Regex - ^Dolphin/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R317, S106355, REG_EXTENDED|REG_NOSUB) != 0) return -319;	/*Regex - Android 4\.3|Android4\.3*/
	if (fiftyone_degrees_regcomp(&R318, S106356, REG_EXTENDED|REG_NOSUB) != 0) return -320;	/*Regex - Android 4\.4|Android4\.4*/
	if (fiftyone_degrees_regcomp(&R319, S106357, REG_EXTENDED|REG_NOSUB) != 0) return -321;	/*Regex - (?i)alcatel*/
	if (fiftyone_degrees_regcomp(&R320, S106358, REG_EXTENDED) != 0) return -322;	/*Regex - (?<=Alcatel-)[^/\s]+*/
	if (fiftyone_degrees_regcomp(&R321, S106359, REG_EXTENDED|REG_NOSUB) != 0) return -323;	/*Regex - ^Amoi*/
	if (fiftyone_degrees_regcomp(&R322, S106360, REG_EXTENDED|REG_NOSUB) != 0) return -324;	/*Regex - ^Apple*/
	if (fiftyone_degrees_regcomp(&R323, S106361, REG_EXTENDED|REG_NOSUB) != 0) return -325;	/*Regex - CoreMedia*/
	if (fiftyone_degrees_regcomp(&R324, S106362, REG_EXTENDED|REG_NOSUB) != 0) return -326;	/*Regex - ^AUDIOVOX*/
	if (fiftyone_degrees_regcomp(&R325, S106363, REG_EXTENDED|REG_NOSUB) != 0) return -327;	/*Regex - BenQ*/
	if (fiftyone_degrees_regcomp(&R326, S106364, REG_EXTENDED|REG_NOSUB) != 0) return -328;	/*Regex - BENQ*/
	if (fiftyone_degrees_regcomp(&R327, S106365, REG_EXTENDED|REG_NOSUB) != 0) return -329;	/*Regex - ^BIRD*/
	if (fiftyone_degrees_regcomp(&R328, S106366, REG_EXTENDED|REG_NOSUB) != 0) return -330;	/*Regex - Black[B|b]erry*/
	if (fiftyone_degrees_regcomp(&R329, S106367, REG_EXTENDED|REG_NOSUB) != 0) return -331;	/*Regex - ^Mozilla/*/
	if (fiftyone_degrees_regcomp(&R330, S106368, REG_EXTENDED|REG_NOSUB) != 0) return -332;	/*Regex - BlackBerry*/
	if (fiftyone_degrees_regcomp(&R331, S106369, REG_EXTENDED|REG_NOSUB) != 0) return -333;	/*Regex - Version/\d+*/
	if (fiftyone_degrees_regcomp(&R332, S106370, REG_EXTENDED) != 0) return -334;	/*Regex - (?<=Black[B|b]erry)[^/]+*/
	if (fiftyone_degrees_regcomp(&R333, S106371, REG_EXTENDED) != 0) return -335;	/*Regex - (?<=Black[B|b]erry\w{4}/)\d+\.\d+\.\d+|(?<=Black[B|b]erry\w{5}/)\d+\.\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R334, S106372, REG_EXTENDED) != 0) return -336;	/*Regex - (?<=UP.Browser/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R335, S106373, REG_EXTENDED) != 0) return -337;	/*Regex - UC Browser\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R336, S106374, REG_EXTENDED) != 0) return -338;	/*Regex - (?<=BlackBerry)[^;]+*/
	if (fiftyone_degrees_regcomp(&R337, S106375, REG_EXTENDED) != 0) return -339;	/*Regex - (?<=Version/)\d+\.\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R338, S106376, REG_EXTENDED|REG_NOSUB) != 0) return -340;	/*Regex - ^DoCoMo*/
	if (fiftyone_degrees_regcomp(&R339, S106377, REG_EXTENDED) != 0) return -341;	/*Regex - (?<=DoCoMo/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R340, S106378, REG_EXTENDED) != 0) return -342;	/*Regex - (?<=DoCoMo/\d\.\d[\s|/])[\w\d]+*/
	if (fiftyone_degrees_regcomp(&R341, S106379, REG_EXTENDED|REG_NOSUB) != 0) return -343;	/*Regex - ^Fly|^FLY*/
	if (fiftyone_degrees_regcomp(&R342, S106380, REG_EXTENDED|REG_NOSUB) != 0) return -344;	/*Regex - ^HD2|HTC*/
	if (fiftyone_degrees_regcomp(&R343, S106381, REG_EXTENDED) != 0) return -345;	/*Regex - (?<=\sHTC[\s|_])[^;\s]+*/
	if (fiftyone_degrees_regcomp(&R344, S106382, REG_EXTENDED|REG_NOSUB) != 0) return -346;	/*Regex - (?i)Huawei*/
	if (fiftyone_degrees_regcomp(&R345, S106383, REG_EXTENDED|REG_NOSUB) != 0) return -347;	/*Regex - ^KDDI*/
	if (fiftyone_degrees_regcomp(&R346, S106384, REG_EXTENDED) != 0) return -348;	/*Regex - KDDI-\w+*/
	if (fiftyone_degrees_regcomp(&R347, S106385, REG_EXTENDED|REG_NOSUB) != 0) return -349;	/*Regex - ^kyocera|^QC-|^KWC-*/
	if (fiftyone_degrees_regcomp(&R348, S106386, REG_EXTENDED|REG_NOSUB) != 0) return -350;	/*Regex - ^LCT*/
	if (fiftyone_degrees_regcomp(&R349, S106387, REG_EXTENDED|REG_NOSUB) != 0) return -351;	/*Regex - ^Lenovo*/
	if (fiftyone_degrees_regcomp(&R350, S106388, REG_EXTENDED) != 0) return -352;	/*Regex - (?<=^Lenovo[-_])[\w\d]+*/
	if (fiftyone_degrees_regcomp(&R351, S106389, REG_EXTENDED|REG_NOSUB) != 0) return -353;	/*Regex - ^LG/*/
	if (fiftyone_degrees_regcomp(&R352, S106390, REG_EXTENDED) != 0) return -354;	/*Regex - (?<=^LG/)[\w\d]+*/
	if (fiftyone_degrees_regcomp(&R353, S106391, REG_EXTENDED) != 0) return -355;	/*Regex - (?<=^LG(-|/))[^\s/]+*/
	if (fiftyone_degrees_regcomp(&R354, S106392, REG_EXTENDED) != 0) return -356;	/*Regex - (?<=NovarraVision)\d\.\d*/
	if (fiftyone_degrees_regcomp(&R355, S106393, REG_EXTENDED) != 0) return -357;	/*Regex - (?<=Browser/Teleca-)[^\s]+*/
	if (fiftyone_degrees_regcomp(&R356, S106394, REG_EXTENDED) != 0) return -358;	/*Regex - (?<=UCBrowser)\d\.\d*/
	if (fiftyone_degrees_regcomp(&R357, S106395, REG_EXTENDED) != 0) return -359;	/*Regex - (?<=Browser/Obigo-)[^/]+/[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R358, S106396, REG_EXTENDED) != 0) return -360;	/*Regex - (?<=Opera Mini/)[^\);]+*/
	if (fiftyone_degrees_regcomp(&R359, S106397, REG_EXTENDED|REG_NOSUB) != 0) return -361;	/*Regex - ^LG-*/
	if (fiftyone_degrees_regcomp(&R360, S106398, REG_EXTENDED) != 0) return -362;	/*Regex - (?<=^LG-)[\w\d]+*/
	if (fiftyone_degrees_regcomp(&R361, S106399, REG_EXTENDED) != 0) return -363;	/*Regex - UC( |)Browser(|/)\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R362, S106400, REG_EXTENDED|REG_NOSUB) != 0) return -364;	/*Regex - ^LGE-*/
	if (fiftyone_degrees_regcomp(&R363, S106401, REG_EXTENDED) != 0) return -365;	/*Regex - (?<=^LGE-)[\w\d]+*/
	if (fiftyone_degrees_regcomp(&R364, S106402, REG_EXTENDED) != 0) return -366;	/*Regex - (?<=^LGE-)[^\s/]+*/
	if (fiftyone_degrees_regcomp(&R365, S106403, REG_EXTENDED|REG_NOSUB) != 0) return -367;	/*Regex - ^MOT-*/
	if (fiftyone_degrees_regcomp(&R366, S106404, REG_EXTENDED) != 0) return -368;	/*Regex - (?<=^MOT-)[^/]+*/
	if (fiftyone_degrees_regcomp(&R367, S106405, REG_EXTENDED) != 0) return -369;	/*Regex - (?<=UP\.Browser/)[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R368, S106406, REG_EXTENDED) != 0) return -370;	/*Regex - \([^\)]+\)*/
	if (fiftyone_degrees_regcomp(&R369, S106407, REG_EXTENDED|REG_NOSUB) != 0) return -371;	/*Regex - ^NEC|KGT*/
	if (fiftyone_degrees_regcomp(&R370, S106408, REG_EXTENDED|REG_NOSUB) != 0) return -372;	/*Regex - ^Nexian|^NEXIAN*/
	if (fiftyone_degrees_regcomp(&R371, S106409, REG_EXTENDED|REG_NOSUB) != 0) return -373;	/*Regex - (?i)nokia*/
	if (fiftyone_degrees_regcomp(&R372, S106410, REG_EXTENDED|REG_NOSUB) != 0) return -374;	/*Regex - Series(40|60)*/
	if (fiftyone_degrees_regcomp(&R373, S106411, REG_EXTENDED) != 0) return -375;	/*Regex - (?i)(?<=Nokia)[^/\)]+*/
	if (fiftyone_degrees_regcomp(&R374, S106412, REG_EXTENDED) != 0) return -376;	/*Regex - (?<=S40OviBrowser/)\d+\.\d+.\d+*/
	if (fiftyone_degrees_regcomp(&R375, S106413, REG_EXTENDED) != 0) return -377;	/*Regex - ^iBrowser/Mini\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R376, S106414, REG_EXTENDED) != 0) return -378;	/*Regex - ^OneBrowser/\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R377, S106415, REG_EXTENDED) != 0) return -379;	/*Regex - ^MQQBrowser/Mini\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R378, S106416, REG_EXTENDED|REG_NOSUB) != 0) return -380;	/*Regex - Palm*/
	if (fiftyone_degrees_regcomp(&R379, S106417, REG_EXTENDED|REG_NOSUB) != 0) return -381;	/*Regex - Palm(S|s)ource*/
	if (fiftyone_degrees_regcomp(&R380, S106418, REG_EXTENDED|REG_NOSUB) != 0) return -382;	/*Regex - ^Panasonic*/
	if (fiftyone_degrees_regcomp(&R381, S106419, REG_EXTENDED|REG_NOSUB) != 0) return -383;	/*Regex - ^Pantech|^PT-|^PANTECH|^PG-*/
	if (fiftyone_degrees_regcomp(&R382, S106420, REG_EXTENDED|REG_NOSUB) != 0) return -384;	/*Regex - ^Philips|^PHILIPS*/
	if (fiftyone_degrees_regcomp(&R383, S106421, REG_EXTENDED|REG_NOSUB) != 0) return -385;	/*Regex - ^Qtek*/
	if (fiftyone_degrees_regcomp(&R384, S106422, REG_EXTENDED|REG_NOSUB) != 0) return -386;	/*Regex - iPhone|iPod|iPad|Nokia|BlackBerry|Android|Skyfire*/
	if (fiftyone_degrees_regcomp(&R385, S106423, REG_EXTENDED) != 0) return -387;	/*Regex - (?<=Mozilla/\d\.\d \()[^)]+*/
	if (fiftyone_degrees_regcomp(&R386, S106424, REG_EXTENDED) != 0) return -388;	/*Regex - Phantomjs/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R387, S106425, REG_EXTENDED) != 0) return -389;	/*Regex - LG Browser/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R388, S106426, REG_EXTENDED) != 0) return -390;	/*Regex - (?<=LGE;)[^;]+*/
	if (fiftyone_degrees_regcomp(&R389, S106427, REG_EXTENDED|REG_NOSUB) != 0) return -391;	/*Regex - ^Sagem|^SAGEM*/
	if (fiftyone_degrees_regcomp(&R390, S106428, REG_EXTENDED|REG_NOSUB) != 0) return -392;	/*Regex - (?i)SAMSUNG|SEC-|SCH-*/
	if (fiftyone_degrees_regcomp(&R391, S106429, REG_EXTENDED|REG_NOSUB) != 0) return -393;	/*Regex - Series60/*/
	if (fiftyone_degrees_regcomp(&R392, S106430, REG_EXTENDED|REG_NOSUB) != 0) return -394;	/*Regex - Samsung/*/
	if (fiftyone_degrees_regcomp(&R393, S106431, REG_EXTENDED|REG_NOSUB) != 0) return -395;	/*Regex - SAMSUNG-GT-*/
	if (fiftyone_degrees_regcomp(&R394, S106432, REG_EXTENDED) != 0) return -396;	/*Regex - (?i)(?<=SAMSUNG)-?[^/]+*/
	if (fiftyone_degrees_regcomp(&R395, S106433, REG_EXTENDED) != 0) return -397;	/*Regex - (?i)(?<=SEC|SCH)-?[^/]+*/
	if (fiftyone_degrees_regcomp(&R396, S106434, REG_EXTENDED) != 0) return -398;	/*Regex - (?<=Bada/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R397, S106435, REG_EXTENDED) != 0) return -399;	/*Regex - (?<=Dolfin/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R398, S106436, REG_EXTENDED) != 0) return -400;	/*Regex - (?<=Jasmine/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R399, S106437, REG_EXTENDED) != 0) return -401;	/*Regex - UC(\.|)Browser/\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R400, S106438, REG_EXTENDED) != 0) return -402;	/*Regex - (?<=Windows)\w+;*/
	if (fiftyone_degrees_regcomp(&R401, S106439, REG_EXTENDED) != 0) return -403;	/*Regex - Android \d\.\d*/
	if (fiftyone_degrees_regcomp(&R402, S106440, REG_EXTENDED) != 0) return -404;	/*Regex - UCWEB\d\.\d*/
	if (fiftyone_degrees_regcomp(&R403, S106441, REG_EXTENDED) != 0) return -405;	/*Regex - (?<=SAMSUNG-GT-)[^/-]+*/
	if (fiftyone_degrees_regcomp(&R404, S106442, REG_EXTENDED) != 0) return -406;	/*Regex - (?<=UC\.Browser/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R405, S106443, REG_EXTENDED) != 0) return -407;	/*Regex - (?<=MSIEMobile )\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R406, S106444, REG_EXTENDED) != 0) return -408;	/*Regex - (?<=Samsung/)[^/]+*/
	if (fiftyone_degrees_regcomp(&R407, S106445, REG_EXTENDED) != 0) return -409;	/*Regex - (?<=Series60/)[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R408, S106446, REG_EXTENDED) != 0) return -410;	/*Regex - (?<=SymbianOS/)[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R409, S106447, REG_EXTENDED|REG_NOSUB) != 0) return -411;	/*Regex - ^Sanyo|^SANYO*/
	if (fiftyone_degrees_regcomp(&R410, S106448, REG_EXTENDED|REG_NOSUB) != 0) return -412;	/*Regex - Sanyo*/
	if (fiftyone_degrees_regcomp(&R411, S106449, REG_EXTENDED|REG_NOSUB) != 0) return -413;	/*Regex - SCP*/
	if (fiftyone_degrees_regcomp(&R412, S106450, REG_EXTENDED|REG_NOSUB) != 0) return -414;	/*Regex - ^Sharp|^SHARP*/
	if (fiftyone_degrees_regcomp(&R413, S106451, REG_EXTENDED|REG_NOSUB) != 0) return -415;	/*Regex - ^SIE-*/
	if (fiftyone_degrees_regcomp(&R414, S106452, REG_EXTENDED|REG_NOSUB) != 0) return -416;	/*Regex - ^SoftBank|^Vodafone|^J-PHONE*/
	if (fiftyone_degrees_regcomp(&R415, S106453, REG_EXTENDED) != 0) return -417;	/*Regex - (SoftBank|Vodafone|J-PHONE)\/\d\.\d\/\w+*/
	if (fiftyone_degrees_regcomp(&R416, S106454, REG_EXTENDED|REG_NOSUB) != 0) return -418;	/*Regex - Spice*/
	if (fiftyone_degrees_regcomp(&R417, S106455, REG_EXTENDED|REG_NOSUB) != 0) return -419;	/*Regex - ^TIANYU-KTOUCH*/
	if (fiftyone_degrees_regcomp(&R418, S106456, REG_EXTENDED) != 0) return -420;	/*Regex - (?<=TIANYU-KTOUCH/)[^/]+*/
	if (fiftyone_degrees_regcomp(&R419, S39438, REG_EXTENDED|REG_NOSUB) != 0) return -421;	/*Regex - Toshiba*/
	if (fiftyone_degrees_regcomp(&R420, S106457, REG_EXTENDED|REG_NOSUB) != 0) return -422;	/*Regex - ^Vodafone*/
	if (fiftyone_degrees_regcomp(&R421, S106458, REG_EXTENDED|REG_NOSUB) != 0) return -423;	/*Regex - webOS*/
	if (fiftyone_degrees_regcomp(&R422, S106459, REG_EXTENDED|REG_NOSUB) != 0) return -424;	/*Regex - HTC*/
	if (fiftyone_degrees_regcomp(&R423, S106460, REG_EXTENDED) != 0) return -425;	/*Regex - (?<=\(|;|\))[^;\)]+*/
	if (fiftyone_degrees_regcomp(&R424, S106461, REG_EXTENDED|REG_NOSUB) != 0) return -426;	/*Regex - ^ZTE*/
	if (fiftyone_degrees_regcomp(&R425, S106462, REG_EXTENDED|REG_NOSUB) != 0) return -427;	/*Regex - (?i)Webkit*/
	if (fiftyone_degrees_regcomp(&R426, S106463, REG_EXTENDED|REG_NOSUB) != 0) return -428;	/*Regex - Chrome|Safari|Android|Nokia|BlackBerry*/
	if (fiftyone_degrees_regcomp(&R427, S106464, REG_EXTENDED) != 0) return -429;	/*Regex - (?<=\(KHTML, like Gecko\))[^\(\)]+*/
	if (fiftyone_degrees_regcomp(&R428, S76, REG_EXTENDED|REG_NOSUB) != 0) return -430;	/*Regex - Gecko*/
	if (fiftyone_degrees_regcomp(&R429, S106465, REG_EXTENDED|REG_NOSUB) != 0) return -431;	/*Regex - Fennec|Linux Mint|Firefox|IceWeasel|Thunderbird|WebKit|BonEcho*/
	if (fiftyone_degrees_regcomp(&R430, S106466, REG_EXTENDED) != 0) return -432;	/*Regex - Windows NT \d\.\d(?=;)|Mac OS X [^\s]+(?=;)|Ubuntu|X11|Linux i686*/
	if (fiftyone_degrees_regcomp(&R431, S106467, REG_EXTENDED) != 0) return -433;	/*Regex - Win\d(\d|x)*/
	if (fiftyone_degrees_regcomp(&R432, S106468, REG_EXTENDED) != 0) return -434;	/*Regex - Fedora*/
	if (fiftyone_degrees_regcomp(&R433, S106469, REG_EXTENDED) != 0) return -435;	/*Regex - (?<=StumbleUpon/)\d\.\d*/
	if (fiftyone_degrees_regcomp(&R434, S106470, REG_EXTENDED) != 0) return -436;	/*Regex - Postbox/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R435, S106471, REG_EXTENDED) != 0) return -437;	/*Regex - Minefield/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R436, S106472, REG_EXTENDED) != 0) return -438;	/*Regex - Netscape(\d|)/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R437, S106473, REG_EXTENDED) != 0) return -439;	/*Regex - lolifox/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R438, S106474, REG_EXTENDED) != 0) return -440;	/*Regex - Icedove/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R439, S106475, REG_EXTENDED) != 0) return -441;	/*Regex - zdesktop/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R440, S106476, REG_EXTENDED) != 0) return -442;	/*Regex - Abrowser/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R441, S106477, REG_EXTENDED) != 0) return -443;	/*Regex - Mnenhy/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R442, S106478, REG_EXTENDED) != 0) return -444;	/*Regex - Konqueror/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R443, S106479, REG_EXTENDED) != 0) return -445;	/*Regex - Ubuntu/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R444, S106480, REG_EXTENDED) != 0) return -446;	/*Regex - Palemoon/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R445, S106481, REG_EXTENDED) != 0) return -447;	/*Regex - SeaMonkey(/\d\.\d+|$)*/
	if (fiftyone_degrees_regcomp(&R446, S106482, REG_EXTENDED) != 0) return -448;	/*Regex - GranParadiso/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R447, S106483, REG_EXTENDED) != 0) return -449;	/*Regex - K9Browser/\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R448, S106484, REG_EXTENDED|REG_NOSUB) != 0) return -450;	/*Regex - iTunes*/
	if (fiftyone_degrees_regcomp(&R449, S106485, REG_EXTENDED) != 0) return -451;	/*Regex - iPod/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R450, S106486, REG_EXTENDED) != 0) return -452;	/*Regex - (?<=Opera/)[\d+.]+|(?<=Opera )[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R451, S106487, REG_EXTENDED) != 0) return -453;	/*Regex - (?<=Presto/)\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R452, S106488, REG_EXTENDED|REG_NOSUB) != 0) return -454;	/*Regex - Opera Mini*/
	if (fiftyone_degrees_regcomp(&R453, S106489, REG_EXTENDED) != 0) return -455;	/*Regex - (?<=Opera Mini/att/)\d(/|\.)\d*/
	if (fiftyone_degrees_regcomp(&R454, S106490, REG_EXTENDED) != 0) return -456;	/*Regex - ^[^;]+*/
	if (fiftyone_degrees_regcomp(&R455, S106491, REG_EXTENDED) != 0) return -457;	/*Regex - (?<=Opera Mini/)\d(/|\.)\d*/
	if (fiftyone_degrees_regcomp(&R456, S106492, REG_EXTENDED) != 0) return -458;	/*Regex - (?<= \d\d\dX\d\d\d).+*/
	if (fiftyone_degrees_regcomp(&R457, S106493, REG_EXTENDED) != 0) return -459;	/*Regex - (?<=Windows NT )\d\.\d*/
	if (fiftyone_degrees_regcomp(&R458, S106494, REG_EXTENDED) != 0) return -460;	/*Regex - Opera Mini//\d+*/
	if (fiftyone_degrees_regcomp(&R459, S106495, REG_EXTENDED) != 0) return -461;	/*Regex - Version/\d+\.\d+*/
	if (fiftyone_degrees_regcomp(&R460, S106496, REG_EXTENDED|REG_NOSUB) != 0) return -462;	/*Regex - Opera Mobi*/
	if (fiftyone_degrees_regcomp(&R461, S106497, REG_EXTENDED|REG_NOSUB) != 0) return -463;	/*Regex - ^portalmmm*/
	if (fiftyone_degrees_regcomp(&R462, S106498, REG_EXTENDED|REG_NOSUB) != 0) return -464;	/*Regex - Outlook*/
	if (fiftyone_degrees_regcomp(&R463, S106499, REG_EXTENDED) != 0) return -465;	/*Regex - (?<=Microsoft Outlook )[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R464, S106500, REG_EXTENDED) != 0) return -466;	/*Regex - (?<=Outlook 20\d\d)[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R465, S106501, REG_EXTENDED|REG_NOSUB) != 0) return -467;	/*Regex - Profile/MIDP-\d\.\d Configuration/CLDC-\d\.\d*/
	if (fiftyone_degrees_regcomp(&R466, S106502, REG_EXTENDED) != 0) return -468;	/*Regex - .+(?=Profile/MIDP-\d\.\d Configuration/CLDC-\d\.\d)*/
	if (fiftyone_degrees_regcomp(&R467, S106503, REG_EXTENDED|REG_NOSUB) != 0) return -469;	/*Regex - WAP|MIDP|CLDC|Teleca-|Screen-|mobile*/
	if (fiftyone_degrees_regcomp(&R468, S106504, REG_EXTENDED|REG_NOSUB) != 0) return -470;	/*Regex - ^iOS/.+dataaccessd*/
	if (fiftyone_degrees_regcomp(&R469, S106505, REG_EXTENDED|REG_NOSUB) != 0) return -471;	/*Regex - ^Podcasts*/
	if (fiftyone_degrees_regcomp(&R470, S106506, REG_EXTENDED|REG_NOSUB) != 0) return -472;	/*Regex - ^Tiny Tiny RSS*/
	if (fiftyone_degrees_regcomp(&R471, S106507, REG_EXTENDED|REG_NOSUB) != 0) return -473;	/*Regex - ^Amazon CloudFront*/
	if (fiftyone_degrees_regcomp(&R472, S106508, REG_EXTENDED|REG_NOSUB) != 0) return -474;	/*Regex - ^Wada\.vn*/
	if (fiftyone_degrees_regcomp(&R473, S106509, REG_EXTENDED|REG_NOSUB) != 0) return -475;	/*Regex - ^FeedDemon*/
	if (fiftyone_degrees_regcomp(&R474, S106510, REG_EXTENDED|REG_NOSUB) != 0) return -476;	/*Regex - ^python-requests*/
	if (fiftyone_degrees_regcomp(&R475, S106511, REG_EXTENDED|REG_NOSUB) != 0) return -477;	/*Regex - ^RSSInclude*/
	if (fiftyone_degrees_regcomp(&R476, S106512, REG_EXTENDED|REG_NOSUB) != 0) return -478;	/*Regex - ^curl*/
	if (fiftyone_degrees_regcomp(&R477, S106513, REG_EXTENDED|REG_NOSUB) != 0) return -479;	/*Regex - ^Pulp/*/
	if (fiftyone_degrees_regcomp(&R478, S106514, REG_EXTENDED|REG_NOSUB) != 0) return -480;	/*Regex - Win98*/
	if (fiftyone_degrees_regcomp(&R479, S106515, REG_EXTENDED|REG_NOSUB) != 0) return -481;	/*Regex - AppEngine*/
	if (fiftyone_degrees_regcomp(&R480, S106516, REG_EXTENDED) != 0) return -482;	/*Regex - ^[^\(]+*/
	if (fiftyone_degrees_regcomp(&R481, S106517, REG_EXTENDED) != 0) return -483;	/*Regex - darwin*/
	if (fiftyone_degrees_regcomp(&R482, S106518, REG_EXTENDED|REG_NOSUB) != 0) return -484;	/*Regex - ^(I|J)UC*/
	if (fiftyone_degrees_regcomp(&R483, S106519, REG_EXTENDED) != 0) return -485;	/*Regex - ^(J|I)UC*/
	if (fiftyone_degrees_regcomp(&R484, S106520, REG_EXTENDED) != 0) return -486;	/*Regex - UCWEB\d+\.\d*/
	if (fiftyone_degrees_regcomp(&R485, S106521, REG_EXTENDED) != 0) return -487;	/*Regex - (?<=(Z|z)h(-|_)cn;)[^;]+*/
	if (fiftyone_degrees_regcomp(&R486, S106522, REG_EXTENDED) != 0) return -488;	/*Regex - Linux;\sU;\s\d\.\d*/
	if (fiftyone_degrees_regcomp(&R487, S106523, REG_EXTENDED) != 0) return -489;	/*Regex - iOS \d\.\d*/
	if (fiftyone_degrees_regcomp(&R488, S106524, REG_EXTENDED|REG_NOSUB) != 0) return -490;	/*Regex - ^server-bag*/
	if (fiftyone_degrees_regcomp(&R489, S106525, REG_EXTENDED) != 0) return -491;	/*Regex - iPhone OS,\d\.\d*/
	if (fiftyone_degrees_regcomp(&R490, S106526, REG_EXTENDED) != 0) return -492;	/*Regex - ,iPhone|,iPod|,iPad*/
	if (fiftyone_degrees_regcomp(&R491, S106527, REG_EXTENDED|REG_NOSUB) != 0) return -493;	/*Regex - ^-1*/
	if (fiftyone_degrees_regcomp(&R492, S106528, REG_EXTENDED|REG_NOSUB) != 0) return -494;	/*Regex - ^MicroMax|^Micromax*/
	if (fiftyone_degrees_regcomp(&R493, S106529, REG_EXTENDED) != 0) return -495;	/*Regex - ^Micromax.+$*/
	if (fiftyone_degrees_regcomp(&R494, S106530, REG_EXTENDED|REG_NOSUB) != 0) return -496;	/*Regex - jig browser*/
	if (fiftyone_degrees_regcomp(&R495, S106531, REG_EXTENDED) != 0) return -497;	/*Regex - 1.0.4; .+\)$*/
	if (fiftyone_degrees_regcomp(&R496, S106532, REG_EXTENDED|REG_NOSUB) != 0) return -498;	/*Regex - Konqueror*/
	if (fiftyone_degrees_regcomp(&R497, S106533, REG_EXTENDED) != 0) return -499;	/*Regex - Konqueror/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R498, S106534, REG_EXTENDED) != 0) return -500;	/*Regex - Ubuntu*/
	if (fiftyone_degrees_regcomp(&R499, S106535, REG_EXTENDED|REG_NOSUB) != 0) return -501;	/*Regex - heritrix*/
	if (fiftyone_degrees_regcomp(&R500, S106536, REG_EXTENDED) != 0) return -502;	/*Regex - heritrix/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R501, S106537, REG_EXTENDED|REG_NOSUB) != 0) return -503;	/*Regex - Java/\d\.\d*/
	if (fiftyone_degrees_regcomp(&R502, S106538, REG_EXTENDED) != 0) return -504;	/*Regex - (?<=Java/)\d\.\d*/
	if (fiftyone_degrees_regcomp(&R503, S106539, REG_EXTENDED) != 0) return -505;	/*Regex - Windows .+\)*/
	if (fiftyone_degrees_regcomp(&R504, S106540, REG_EXTENDED) != 0) return -506;	/*Regex - Max OS X \d+\.\d*/
	if (fiftyone_degrees_regcomp(&R505, S106541, REG_EXTENDED|REG_NOSUB) != 0) return -507;	/*Regex - UCWEB*/
	if (fiftyone_degrees_regcomp(&R506, S106542, REG_EXTENDED) != 0) return -508;	/*Regex - (?<=UCBrowser)(|/)\d\.\d*/
	if (fiftyone_degrees_regcomp(&R507, S106543, REG_EXTENDED) != 0) return -509;	/*Regex - (?<=\w\w-\w\w;)[^\)]+*/
	if (fiftyone_degrees_regcomp(&R508, S106544, REG_EXTENDED|REG_NOSUB) != 0) return -510;	/*Regex - ^MacOutlook*/
	if (fiftyone_degrees_regcomp(&R509, S106545, REG_EXTENDED) != 0) return -511;	/*Regex - (?<=MacOutlook/)[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R510, S106546, REG_EXTENDED) != 0) return -512;	/*Regex - (?<=Mac OS X )[\d\.]+*/
	if (fiftyone_degrees_regcomp(&R511, S106547, REG_EXTENDED|REG_NOSUB) != 0) return -513;	/*Regex - \w+*/
	return 0;
}


// Frees resources.
void destroy(void) {

	// Free the memory used by the properties.
	if (_propertiesCount > 0) {
		free(_properties);
		free(_allProperties);
	}

	fiftyone_degrees_regfree(&R0);	/*Regex - (?i)bot|spider|crawler|slurp*/
	fiftyone_degrees_regfree(&R1);	/*Regex - (?i)bot|spider|crawler|slurp|[^\s]+(?=bot|spider|crawler|slurp)[^\s]+*/
	fiftyone_degrees_regfree(&R2);	/*Regex - .+*/
	fiftyone_degrees_regfree(&R3);	/*Regex - W3C_Validator*/
	fiftyone_degrees_regfree(&R4);	/*Regex - ^Opera/*/
	fiftyone_degrees_regfree(&R5);	/*Regex - Windows NT 6\.0*/
	fiftyone_degrees_regfree(&R6);	/*Regex - Windows NT 6\.1*/
	fiftyone_degrees_regfree(&R7);	/*Regex - Opera Mini|Opera Mobi*/
	fiftyone_degrees_regfree(&R8);	/*Regex - MIDP|CLDC*/
	fiftyone_degrees_regfree(&R9);	/*Regex - (?<=Opera/)[\d+.]+*/
	fiftyone_degrees_regfree(&R10);	/*Regex - (?<=Version/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R11);	/*Regex - (?<=\()[^;]+*/
	fiftyone_degrees_regfree(&R12);	/*Regex - (?<=Opera/\d\.\d \()[^;\)]+*/
	fiftyone_degrees_regfree(&R13);	/*Regex - Android( |)2\.2\.1*/
	fiftyone_degrees_regfree(&R14);	/*Regex - (?<=;)([^;\)]+)(?=Build/)*/
	fiftyone_degrees_regfree(&R15);	/*Regex - (?<=\w\w(_|-)\w\w;)[^;\)]+*/
	fiftyone_degrees_regfree(&R16);	/*Regex - (?<=;\sAndroid)[\d\s][^;\)]+*/
	fiftyone_degrees_regfree(&R17);	/*Regex - (?<=UCWEB)\d+\.\d+|(?<=UCWEB/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R18);	/*Regex - (?<=UC Browser)\d+\.\d+*/
	fiftyone_degrees_regfree(&R19);	/*Regex - (?<=UP\.Browser)[\d\.]+*/
	fiftyone_degrees_regfree(&R20);	/*Regex - (?<=Opera/)\d+\.\d+|(?<=Opera )\d+\.\d+*/
	fiftyone_degrees_regfree(&R21);	/*Regex - (?<=Opera Mini/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R22);	/*Regex - (?<=Opera Mobi/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R23);	/*Regex - (?<=Ninesky-android-mobile/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R24);	/*Regex - (?<=CrMo/)\d+*/
	fiftyone_degrees_regfree(&R25);	/*Regex - (?<=Chrome/)\d+*/
	fiftyone_degrees_regfree(&R26);	/*Regex - (?<=AppleWebKit/)[\d\.\+]+*/
	fiftyone_degrees_regfree(&R27);	/*Regex - (?<=Mobile Safari/)[\d\.]+*/
	fiftyone_degrees_regfree(&R28);	/*Regex - (?<=Flipboard/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R29);	/*Regex - MQQBrowser/(\d+\.\d|\d+ )*/
	fiftyone_degrees_regfree(&R30);	/*Regex - Maxthon*/
	fiftyone_degrees_regfree(&R31);	/*Regex - UCBrowser/\d+\.\d*/
	fiftyone_degrees_regfree(&R32);	/*Regex - iBrowser/\d+\.\d*/
	fiftyone_degrees_regfree(&R33);	/*Regex - TeamStream*/
	fiftyone_degrees_regfree(&R34);	/*Regex - Maxthon/\d+\.\d*/
	fiftyone_degrees_regfree(&R35);	/*Regex - Ninesky/\d+\.\d*/
	fiftyone_degrees_regfree(&R36);	/*Regex - OPR/\d+\.\d+*/
	fiftyone_degrees_regfree(&R37);	/*Regex - Android( |)2\.2\.2*/
	fiftyone_degrees_regfree(&R38);	/*Regex - MQQBrowser/\d+\.\d*/
	fiftyone_degrees_regfree(&R39);	/*Regex - Android( |)2\.2\.3*/
	fiftyone_degrees_regfree(&R40);	/*Regex - Android( |)2\.3\.1*/
	fiftyone_degrees_regfree(&R41);	/*Regex - Android( |)2\.3\.2*/
	fiftyone_degrees_regfree(&R42);	/*Regex - Android( |)2\.3\.3*/
	fiftyone_degrees_regfree(&R43);	/*Regex - (?<=UCBrowser)\d+\.\d+*/
	fiftyone_degrees_regfree(&R44);	/*Regex - (?<=UCBrowser/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R45);	/*Regex - ^(One|i)Browser/(\d+|\d+\.\d)*/
	fiftyone_degrees_regfree(&R46);	/*Regex - Mobile Maxthon*/
	fiftyone_degrees_regfree(&R47);	/*Regex - \)Maxthon|Maxthon$*/
	fiftyone_degrees_regfree(&R48);	/*Regex - Ninesky/\d\.\d*/
	fiftyone_degrees_regfree(&R49);	/*Regex - ^YahooMobile/\d+\.\d*/
	fiftyone_degrees_regfree(&R50);	/*Regex - Android( |)2\.3\.4*/
	fiftyone_degrees_regfree(&R51);	/*Regex - \)Maxthon*/
	fiftyone_degrees_regfree(&R52);	/*Regex - Cyanogen*/
	fiftyone_degrees_regfree(&R53);	/*Regex - Maxthon$*/
	fiftyone_degrees_regfree(&R54);	/*Regex - (One|i)Browser/\d\.\d*/
	fiftyone_degrees_regfree(&R55);	/*Regex - Android( |)2\.3\.5*/
	fiftyone_degrees_regfree(&R56);	/*Regex - OneBrowser/\d+\.\d*/
	fiftyone_degrees_regfree(&R57);	/*Regex - Maxthon(/\d+\.\d+|$)*/
	fiftyone_degrees_regfree(&R58);	/*Regex - UCBrowser/\d+\.\d+*/
	fiftyone_degrees_regfree(&R59);	/*Regex - ^iBrowser/(\d+\.\d|\d+ )*/
	fiftyone_degrees_regfree(&R60);	/*Regex - Android( |)2\.3\.6*/
	fiftyone_degrees_regfree(&R61);	/*Regex - OneBrowser/\d\.\d*/
	fiftyone_degrees_regfree(&R62);	/*Regex - Android( |)2\.3\.7*/
	fiftyone_degrees_regfree(&R63);	/*Regex - (?<=CyanogenMod-)\d*/
	fiftyone_degrees_regfree(&R64);	/*Regex - (?<=CyanogenMod)\d*/
	fiftyone_degrees_regfree(&R65);	/*Regex - CyanogenMod*/
	fiftyone_degrees_regfree(&R66);	/*Regex - CM7*/
	fiftyone_degrees_regfree(&R67);	/*Regex - MQQBrowser/\d\.\d*/
	fiftyone_degrees_regfree(&R68);	/*Regex - Chrome/[0-9][^\d]*/
	fiftyone_degrees_regfree(&R69);	/*Regex - Android*/
	fiftyone_degrees_regfree(&R70);	/*Regex - (?<=Chrome/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R71);	/*Regex - (?<=\(|;|,)[^\);]+*/
	fiftyone_degrees_regfree(&R72);	/*Regex - (?<=RockMelt/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R73);	/*Regex - Ubuntu/\d+\.\d+*/
	fiftyone_degrees_regfree(&R74);	/*Regex - Sleipnir/\d+\.\d+*/
	fiftyone_degrees_regfree(&R75);	/*Regex - Nutch-\d+\.\d+*/
	fiftyone_degrees_regfree(&R76);	/*Regex - Comodo_Dragon/\d+\.\d+*/
	fiftyone_degrees_regfree(&R77);	/*Regex - Iron/\d+\.\d*/
	fiftyone_degrees_regfree(&R78);	/*Regex - GoogleTV*/
	fiftyone_degrees_regfree(&R79);	/*Regex - chromeframe/\d+\.\d*/
	fiftyone_degrees_regfree(&R80);	/*Regex - Flock/\d+\.\d*/
	fiftyone_degrees_regfree(&R81);	/*Regex - Chrome/1[0-9]*/
	fiftyone_degrees_regfree(&R82);	/*Regex - (?<=^).+(?=Mozilla)*/
	fiftyone_degrees_regfree(&R83);	/*Regex - Puffin/\d+\.\d*/
	fiftyone_degrees_regfree(&R84);	/*Regex - (?<=QQBrowser/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R85);	/*Regex - AvantBrowser/Tri-Core*/
	fiftyone_degrees_regfree(&R86);	/*Regex - (?<=Midori/)\d\.\d*/
	fiftyone_degrees_regfree(&R87);	/*Regex - Chrome/2[0-9]*/
	fiftyone_degrees_regfree(&R88);	/*Regex - Ubuntu Chromium*/
	fiftyone_degrees_regfree(&R89);	/*Regex - Chrome/3[0-9]*/
	fiftyone_degrees_regfree(&R90);	/*Regex - Linux Mint*/
	fiftyone_degrees_regfree(&R91);	/*Regex - Ubuntu;*/
	fiftyone_degrees_regfree(&R92);	/*Regex - ASUS\)*/
	fiftyone_degrees_regfree(&R93);	/*Regex - Mac OS X \d+(\.|_)\d*/
	fiftyone_degrees_regfree(&R94);	/*Regex - Windows NT \d\.\d*/
	fiftyone_degrees_regfree(&R95);	/*Regex - (?<=KDL)[^;]+*/
	fiftyone_degrees_regfree(&R96);	/*Regex - SonyEricsson*/
	fiftyone_degrees_regfree(&R97);	/*Regex - (?<=SonyEricsson)[^/]+*/
	fiftyone_degrees_regfree(&R98);	/*Regex - (?<=NetFront/)[\d\.]+*/
	fiftyone_degrees_regfree(&R99);	/*Regex - (?<=Opera )[^\s]+*/
	fiftyone_degrees_regfree(&R100);	/*Regex - (?<=SEMC-Browser/)[^\s]+*/
	fiftyone_degrees_regfree(&R101);	/*Regex - (?<=TelecaBrowser/)[^\s]+*/
	fiftyone_degrees_regfree(&R102);	/*Regex - (?<=SymbianOS/)[^;]+ */
	fiftyone_degrees_regfree(&R103);	/*Regex - (?<=Series60/)[^\s]+*/
	fiftyone_degrees_regfree(&R104);	/*Regex - (?<=U(C|P)\.Browser/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R105);	/*Regex - (?<=UCBrowser)(|/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R106);	/*Regex - UCWEB/\d\.\d*/
	fiftyone_degrees_regfree(&R107);	/*Regex - Silk-Accelerated*/
	fiftyone_degrees_regfree(&R108);	/*Regex - (?<=Silk/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R109);	/*Regex - (?<=Accelerated=)(true|false)*/
	fiftyone_degrees_regfree(&R110);	/*Regex - Android \d|Android\d*/
	fiftyone_degrees_regfree(&R111);	/*Regex - GT-\w\d+*/
	fiftyone_degrees_regfree(&R112);	/*Regex - 2\.2/FROYO*/
	fiftyone_degrees_regfree(&R113);	/*Regex - theScore/\d\.\d*/
	fiftyone_degrees_regfree(&R114);	/*Regex - ^Instagram \d\.\d*/
	fiftyone_degrees_regfree(&R115);	/*Regex - ^WhatsApp/2\.7*/
	fiftyone_degrees_regfree(&R116);	/*Regex - (FROYO|GINGERBREAD)\)$*/
	fiftyone_degrees_regfree(&R117);	/*Regex - Android 1|Android1*/
	fiftyone_degrees_regfree(&R118);	/*Regex - Android 2\.0|Android2\.0*/
	fiftyone_degrees_regfree(&R119);	/*Regex - Android 2\.1|Android2\.1*/
	fiftyone_degrees_regfree(&R120);	/*Regex - Android 2\.2|Android2\.2*/
	fiftyone_degrees_regfree(&R121);	/*Regex - Maxthon/\d+\.\d|Maxthon*/
	fiftyone_degrees_regfree(&R122);	/*Regex - ^iBrowser/\d+*/
	fiftyone_degrees_regfree(&R123);	/*Regex - Android 2\.3|Android2\.3*/
	fiftyone_degrees_regfree(&R124);	/*Regex - Android 2\.4|Android2\.4*/
	fiftyone_degrees_regfree(&R125);	/*Regex - Android 3|Android3*/
	fiftyone_degrees_regfree(&R126);	/*Regex - Flipboard/\d+\.\d*/
	fiftyone_degrees_regfree(&R127);	/*Regex - \[operamini\]*/
	fiftyone_degrees_regfree(&R128);	/*Regex - ^(One|i)Browser/(\d+\.\d|\d+ )*/
	fiftyone_degrees_regfree(&R129);	/*Regex - (?<=OverDrive Media Console/)\d\.\d*/
	fiftyone_degrees_regfree(&R130);	/*Regex - Android( |)4\.0*/
	fiftyone_degrees_regfree(&R131);	/*Regex - Android( |)4\.0\.\d+*/
	fiftyone_degrees_regfree(&R132);	/*Regex - Chrome/\d+*/
	fiftyone_degrees_regfree(&R133);	/*Regex - Android 4\.0\.1|Android4\.0\.1*/
	fiftyone_degrees_regfree(&R134);	/*Regex - (?<=^).+(?=Linux)*/
	fiftyone_degrees_regfree(&R135);	/*Regex - Android 4\.0\.2|Android4\.0\.2*/
	fiftyone_degrees_regfree(&R136);	/*Regex - Android 4\.0\.3|Android4\.0\.3*/
	fiftyone_degrees_regfree(&R137);	/*Regex - Maxthon(/| \()\d+\.\d*/
	fiftyone_degrees_regfree(&R138);	/*Regex - ^(One|i)Browser/\d+\.\d*/
	fiftyone_degrees_regfree(&R139);	/*Regex - Android 4\.0\.4|Android4\.0\.4*/
	fiftyone_degrees_regfree(&R140);	/*Regex - (One|i)Browser/(\d+\.\d|\d+ )*/
	fiftyone_degrees_regfree(&R141);	/*Regex - Maxthon \(\d\.\d*/
	fiftyone_degrees_regfree(&R142);	/*Regex - PARANOIDANDROID*/
	fiftyone_degrees_regfree(&R143);	/*Regex - Android 4\.0\.(5|6|8|9)*/
	fiftyone_degrees_regfree(&R144);	/*Regex - Android 4\.1|Android4\.1*/
	fiftyone_degrees_regfree(&R145);	/*Regex - Android( |)4\.1\.\d+*/
	fiftyone_degrees_regfree(&R146);	/*Regex - Android 4\.1\.1|Android4\.1\.1*/
	fiftyone_degrees_regfree(&R147);	/*Regex - Maxthon$|\)Maxthon*/
	fiftyone_degrees_regfree(&R148);	/*Regex - ^OneBrowser/\d+\.\d*/
	fiftyone_degrees_regfree(&R149);	/*Regex - MiuiBrowser/\d\.\d*/
	fiftyone_degrees_regfree(&R150);	/*Regex - Android 4\.1\.2|Android4\.1\.2*/
	fiftyone_degrees_regfree(&R151);	/*Regex - (?<=^OneBrowser/)\d\.\d*/
	fiftyone_degrees_regfree(&R152);	/*Regex - Android 4\.2|Android4\.2*/
	fiftyone_degrees_regfree(&R153);	/*Regex - Android( |)4\.2\.\d+*/
	fiftyone_degrees_regfree(&R154);	/*Regex - Android 4\.2\.1|Android4\.2\.1*/
	fiftyone_degrees_regfree(&R155);	/*Regex - Maxthon(/\d+\.\d|$)*/
	fiftyone_degrees_regfree(&R156);	/*Regex - Android 4\.2\.2|Android4\.2\.2*/
	fiftyone_degrees_regfree(&R157);	/*Regex - Maxthon \(\d+\.\d*/
	fiftyone_degrees_regfree(&R158);	/*Regex - CARBON-JB*/
	fiftyone_degrees_regfree(&R159);	/*Regex - Liqiud-JB*/
	fiftyone_degrees_regfree(&R160);	/*Regex - \((iPhone|iPad|iPod)*/
	fiftyone_degrees_regfree(&R161);	/*Regex - Flipboard*/
	fiftyone_degrees_regfree(&R162);	/*Regex - Nokia|nokia*/
	fiftyone_degrees_regfree(&R163);	/*Regex - webOS|hpwOS*/
	fiftyone_degrees_regfree(&R164);	/*Regex - (?<=\()iPhone|iPad|iPod*/
	fiftyone_degrees_regfree(&R165);	/*Regex - (?<=Version/)[\d\.]+*/
	fiftyone_degrees_regfree(&R166);	/*Regex - (?<=OS )[\d(_|\.)\d]+*/
	fiftyone_degrees_regfree(&R167);	/*Regex - (?<=Sleipnir/)[\d\.]+*/
	fiftyone_degrees_regfree(&R168);	/*Regex - (?<=AppleWebKit/)[\d\.]+*/
	fiftyone_degrees_regfree(&R169);	/*Regex - (?<=Mobile/)[\d\w]+*/
	fiftyone_degrees_regfree(&R170);	/*Regex - (?<=Safari/)[\d\.]+*/
	fiftyone_degrees_regfree(&R171);	/*Regex - (?<=Dolphin )[\d\.]+*/
	fiftyone_degrees_regfree(&R172);	/*Regex - (?<=Dolphin HD )[\d\.]+*/
	fiftyone_degrees_regfree(&R173);	/*Regex - Twitter*/
	fiftyone_degrees_regfree(&R174);	/*Regex - Miu Ptt*/
	fiftyone_degrees_regfree(&R175);	/*Regex - Incredimail/\d+\.\d*/
	fiftyone_degrees_regfree(&R176);	/*Regex - PhantomJS/\d+\.\d*/
	fiftyone_degrees_regfree(&R177);	/*Regex - UP\.Browser/(\d+\.\d|\d+ )*/
	fiftyone_degrees_regfree(&R178);	/*Regex - BOLT*/
	fiftyone_degrees_regfree(&R179);	/*Regex - mac*/
	fiftyone_degrees_regfree(&R180);	/*Regex - win*/
	fiftyone_degrees_regfree(&R181);	/*Regex - (?<=BOLT/)[\d\.]+*/
	fiftyone_degrees_regfree(&R182);	/*Regex - (F|f)ire(F|f)o(X|x)|(I|i)ceweasel|BonEcho*/
	fiftyone_degrees_regfree(&R183);	/*Regex - \(Macintosh;|\(Win|\(X1\d;|WOW64*/
	fiftyone_degrees_regfree(&R184);	/*Regex - Fennec|Linux Mint|Thunderbird*/
	fiftyone_degrees_regfree(&R185);	/*Regex - (?<=X11;)\w+*/
	fiftyone_degrees_regfree(&R186);	/*Regex - (?<=Firefox)/\d+\.\d+*/
	fiftyone_degrees_regfree(&R187);	/*Regex - (?<=Iceweasel)/\d+\.\d+*/
	fiftyone_degrees_regfree(&R188);	/*Regex - (?<=Mozilla/)\d\.\d*/
	fiftyone_degrees_regfree(&R189);	/*Regex - (?<=Mozilla/\d\.\d \()[^;\)]+*/
	fiftyone_degrees_regfree(&R190);	/*Regex - Windows NT \d\.\d(?=;)|Mac OS X [^\s]+(?=;)|Ubuntu|X11|SunOS|OpenBSD|FreeBSD*/
	fiftyone_degrees_regfree(&R191);	/*Regex - Fedora/\d+\.\d+*/
	fiftyone_degrees_regfree(&R192);	/*Regex - Nutch*/
	fiftyone_degrees_regfree(&R193);	/*Regex - Debian-\d\.\d*/
	fiftyone_degrees_regfree(&R194);	/*Regex - \w(U|u)buntu/\d+\.\d+*/
	fiftyone_degrees_regfree(&R195);	/*Regex - (?<=\((U|u)buntu-)[^\)]+*/
	fiftyone_degrees_regfree(&R196);	/*Regex - (?<=Firefox)[^\d]+*/
	fiftyone_degrees_regfree(&R197);	/*Regex - (?<=Win)[^;]+*/
	fiftyone_degrees_regfree(&R198);	/*Regex - Fedora/*/
	fiftyone_degrees_regfree(&R199);	/*Regex - Pale(M|m)oon/\d+\.\d*/
	fiftyone_degrees_regfree(&R200);	/*Regex - (E|e)pic/\d+\.\d|Epic/*/
	fiftyone_degrees_regfree(&R201);	/*Regex - Comet(Bird|Browser)/\d+\.\d*/
	fiftyone_degrees_regfree(&R202);	/*Regex - Kylo/\d+\.\d*/
	fiftyone_degrees_regfree(&R203);	/*Regex - Wyzo/\d+\.\d*/
	fiftyone_degrees_regfree(&R204);	/*Regex - Camino/\d+\.\d*/
	fiftyone_degrees_regfree(&R205);	/*Regex - Minefield/\d+\.\d*/
	fiftyone_degrees_regfree(&R206);	/*Regex - Navigator/\d+\.\d*/
	fiftyone_degrees_regfree(&R207);	/*Regex - IceDragon/\d+\.\d*/
	fiftyone_degrees_regfree(&R208);	/*Regex - Opera \d+\.\d\d$*/
	fiftyone_degrees_regfree(&R209);	/*Regex - lolifox/\d\.\d*/
	fiftyone_degrees_regfree(&R210);	/*Regex - Avant TriCore*/
	fiftyone_degrees_regfree(&R211);	/*Regex - Midori/(|\")\d+(|\")\.(|\")\d(|\")*/
	fiftyone_degrees_regfree(&R212);	/*Regex - SeaMonkey/\d\.\d*/
	fiftyone_degrees_regfree(&R213);	/*Regex - firefox/\d\.\d*/
	fiftyone_degrees_regfree(&R214);	/*Regex - BonEcho/\d\.\d*/
	fiftyone_degrees_regfree(&R215);	/*Regex - Maemo Browser \d\.\d*/
	fiftyone_degrees_regfree(&R216);	/*Regex - Linux mips*/
	fiftyone_degrees_regfree(&R217);	/*Regex - Firefox/*/
	fiftyone_degrees_regfree(&R218);	/*Regex - Fennec/|\(Mobile; rv*/
	fiftyone_degrees_regfree(&R219);	/*Regex - (?<=Mozilla/)[\d\.]+*/
	fiftyone_degrees_regfree(&R220);	/*Regex - (?<=Fennec/)[\d\.]+*/
	fiftyone_degrees_regfree(&R221);	/*Regex - Fennec*/
	fiftyone_degrees_regfree(&R222);	/*Regex - (?<=\()[^\)]+*/
	fiftyone_degrees_regfree(&R223);	/*Regex - (?<=Gecko/)[\d]+*/
	fiftyone_degrees_regfree(&R224);	/*Regex - (?<=Firefox/)[\d\.]+*/
	fiftyone_degrees_regfree(&R225);	/*Regex - MSIE*/
	fiftyone_degrees_regfree(&R226);	/*Regex - compatible*/
	fiftyone_degrees_regfree(&R227);	/*Regex - Windows XP*/
	fiftyone_degrees_regfree(&R228);	/*Regex - Windows NT*/
	fiftyone_degrees_regfree(&R229);	/*Regex - Windows ME*/
	fiftyone_degrees_regfree(&R230);	/*Regex - Win32*/
	fiftyone_degrees_regfree(&R231);	/*Regex - Windows [\d\.]+*/
	fiftyone_degrees_regfree(&R232);	/*Regex - Windows NT [\d\.]+*/
	fiftyone_degrees_regfree(&R233);	/*Regex - Mac_PowerPC*/
	fiftyone_degrees_regfree(&R234);	/*Regex - PPC Mac OS*/
	fiftyone_degrees_regfree(&R235);	/*Regex - Mobile*/
	fiftyone_degrees_regfree(&R236);	/*Regex - Windows CE*/
	fiftyone_degrees_regfree(&R237);	/*Regex - Phone*/
	fiftyone_degrees_regfree(&R238);	/*Regex - UC(\.| ||)Browser*/
	fiftyone_degrees_regfree(&R239);	/*Regex - Opera*/
	fiftyone_degrees_regfree(&R240);	/*Regex - (?i)LG-|LG_|SC-|Toshiba|XV\d+|Nokia|LGE|PalmOS|POLARIS|PalmSource|Nutch|NetFront|Samsung|HTC*/
	fiftyone_degrees_regfree(&R241);	/*Regex - (?<=MSIE\s)[\d\.]+*/
	fiftyone_degrees_regfree(&R242);	/*Regex - (?<=Outlook )[\d\.]+*/
	fiftyone_degrees_regfree(&R243);	/*Regex - (?<=Windows)[\s\w\d\.]+*/
	fiftyone_degrees_regfree(&R244);	/*Regex - Windows NT \d\.\d(?=;)|Mac OS X [^\s]+(?=;)|Ubuntu|X11*/
	fiftyone_degrees_regfree(&R245);	/*Regex - chromeframe*/
	fiftyone_degrees_regfree(&R246);	/*Regex - Media Center PC*/
	fiftyone_degrees_regfree(&R247);	/*Regex - Linux*/
	fiftyone_degrees_regfree(&R248);	/*Regex - Zune*/
	fiftyone_degrees_regfree(&R249);	/*Regex - ARM*/
	fiftyone_degrees_regfree(&R250);	/*Regex - Touch*/
	fiftyone_degrees_regfree(&R251);	/*Regex - WOW64;*/
	fiftyone_degrees_regfree(&R252);	/*Regex - (?<=heritrix/)[\d\.]+*/
	fiftyone_degrees_regfree(&R253);	/*Regex - Maxthon|MAXTHON*/
	fiftyone_degrees_regfree(&R254);	/*Regex - SlimBrowser/\d+\.\d*/
	fiftyone_degrees_regfree(&R255);	/*Regex - Avant(| )Browser*/
	fiftyone_degrees_regfree(&R256);	/*Regex - Xbox*/
	fiftyone_degrees_regfree(&R257);	/*Regex - Midori/\d+\.\d*/
	fiftyone_degrees_regfree(&R258);	/*Regex - (?<=Maxthon)(/| )\d+\.\d*/
	fiftyone_degrees_regfree(&R259);	/*Regex - (?<=MAXTHON) \d\.\d*/
	fiftyone_degrees_regfree(&R260);	/*Regex - Win(95|32|98|NT4\.0|64|3\.\d)*/
	fiftyone_degrees_regfree(&R261);	/*Regex - PPC*/
	fiftyone_degrees_regfree(&R262);	/*Regex - Tablet PC*/
	fiftyone_degrees_regfree(&R263);	/*Regex - SV1*/
	fiftyone_degrees_regfree(&R264);	/*Regex - (?<=UCWEB)\d\.\d*/
	fiftyone_degrees_regfree(&R265);	/*Regex - Nokia*/
	fiftyone_degrees_regfree(&R266);	/*Regex - Series40*/
	fiftyone_degrees_regfree(&R267);	/*Regex - (?<=Nokia)[^/]+*/
	fiftyone_degrees_regfree(&R268);	/*Regex - (?<=Series40)[^;]+*/
	fiftyone_degrees_regfree(&R269);	/*Regex - (?<=S40OviBrowser/)\d+\.\d+\.\d+*/
	fiftyone_degrees_regfree(&R270);	/*Regex - iBrowser/Mini\d+\.\d*/
	fiftyone_degrees_regfree(&R271);	/*Regex - Series60*/
	fiftyone_degrees_regfree(&R272);	/*Regex - (?<=Series60/)[\w\d\.]+*/
	fiftyone_degrees_regfree(&R273);	/*Regex - MQQBrowser/Mini\d\.\d*/
	fiftyone_degrees_regfree(&R274);	/*Regex - ^Mozilla*/
	fiftyone_degrees_regfree(&R275);	/*Regex - Safari*/
	fiftyone_degrees_regfree(&R276);	/*Regex - Windows|Macintosh|X11*/
	fiftyone_degrees_regfree(&R277);	/*Regex - Chrome/*/
	fiftyone_degrees_regfree(&R278);	/*Regex - (?<=Safari/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R279);	/*Regex - Windows NT \d\.\d(?=;)|Mac OS X [^\s]+(?=;)|Ubuntu( |(/|)\d+\.\d+)|X11*/
	fiftyone_degrees_regfree(&R280);	/*Regex - (?<=Version/)\d\.\d*/
	fiftyone_degrees_regfree(&R281);	/*Regex - (?<=Mac OS X 10_\d_\d;)[^;\)]+*/
	fiftyone_degrees_regfree(&R282);	/*Regex - Mac OS X 10_\d*/
	fiftyone_degrees_regfree(&R283);	/*Regex - (?<=PhantomJS/)\d\.\d*/
	fiftyone_degrees_regfree(&R284);	/*Regex - SecondLife/\d+\.\d*/
	fiftyone_degrees_regfree(&R285);	/*Regex - Google Earth/\d+\.\d*/
	fiftyone_degrees_regfree(&R286);	/*Regex - rekonq(/\d+\.\d| )*/
	fiftyone_degrees_regfree(&R287);	/*Regex - Skyfire/\d+\.\d*/
	fiftyone_degrees_regfree(&R288);	/*Regex - Sleipnir/\d+\.\d*/
	fiftyone_degrees_regfree(&R289);	/*Regex - QupZilla/\d+\.\d*/
	fiftyone_degrees_regfree(&R290);	/*Regex - konqueror/\d+\.\d*/
	fiftyone_degrees_regfree(&R291);	/*Regex - Google Earth Pro/\d+\.\d*/
	fiftyone_degrees_regfree(&R292);	/*Regex - OmniWeb/.+\.\d+*/
	fiftyone_degrees_regfree(&R293);	/*Regex - Qt/\d\.\d*/
	fiftyone_degrees_regfree(&R294);	/*Regex - kded/\d\.\d*/
	fiftyone_degrees_regfree(&R295);	/*Regex - Midori/\d\.\d*/
	fiftyone_degrees_regfree(&R296);	/*Regex - iCab/\d\.\d*/
	fiftyone_degrees_regfree(&R297);	/*Regex - (?<=Fluid/)\d\.\d*/
	fiftyone_degrees_regfree(&R298);	/*Regex - (?<=Iron/)\d\.\d*/
	fiftyone_degrees_regfree(&R299);	/*Regex - (?<=Stainless/)\d\.\d*/
	fiftyone_degrees_regfree(&R300);	/*Regex - Skyfire/*/
	fiftyone_degrees_regfree(&R301);	/*Regex - (?<=Skyfire/)[\d\.]+*/
	fiftyone_degrees_regfree(&R302);	/*Regex - Windows Phone OS*/
	fiftyone_degrees_regfree(&R303);	/*Regex - ^WordPress/*/
	fiftyone_degrees_regfree(&R304);	/*Regex - (?<=http://)\w+*/
	fiftyone_degrees_regfree(&R305);	/*Regex - Thunderbird*/
	fiftyone_degrees_regfree(&R306);	/*Regex - \(Macintosh;|\(Windows|\(X11;*/
	fiftyone_degrees_regfree(&R307);	/*Regex - Firefox|Iceweasel|Fennec|Linux Mint|Ubuntu*/
	fiftyone_degrees_regfree(&R308);	/*Regex - (?<=Thunderbird/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R309);	/*Regex - (?<=(Fedora)/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R310);	/*Regex - Darwin*/
	fiftyone_degrees_regfree(&R311);	/*Regex - CFNetwork*/
	fiftyone_degrees_regfree(&R312);	/*Regex - iPhone*/
	fiftyone_degrees_regfree(&R313);	/*Regex - ^[^/]+*/
	fiftyone_degrees_regfree(&R314);	/*Regex - (i)?Mac([^\d]+)?*/
	fiftyone_degrees_regfree(&R315);	/*Regex - (?<=Darwin/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R316);	/*Regex - ^Dolphin/\d+\.\d*/
	fiftyone_degrees_regfree(&R317);	/*Regex - Android 4\.3|Android4\.3*/
	fiftyone_degrees_regfree(&R318);	/*Regex - Android 4\.4|Android4\.4*/
	fiftyone_degrees_regfree(&R319);	/*Regex - (?i)alcatel*/
	fiftyone_degrees_regfree(&R320);	/*Regex - (?<=Alcatel-)[^/\s]+*/
	fiftyone_degrees_regfree(&R321);	/*Regex - ^Amoi*/
	fiftyone_degrees_regfree(&R322);	/*Regex - ^Apple*/
	fiftyone_degrees_regfree(&R323);	/*Regex - CoreMedia*/
	fiftyone_degrees_regfree(&R324);	/*Regex - ^AUDIOVOX*/
	fiftyone_degrees_regfree(&R325);	/*Regex - BenQ*/
	fiftyone_degrees_regfree(&R326);	/*Regex - BENQ*/
	fiftyone_degrees_regfree(&R327);	/*Regex - ^BIRD*/
	fiftyone_degrees_regfree(&R328);	/*Regex - Black[B|b]erry*/
	fiftyone_degrees_regfree(&R329);	/*Regex - ^Mozilla/*/
	fiftyone_degrees_regfree(&R330);	/*Regex - BlackBerry*/
	fiftyone_degrees_regfree(&R331);	/*Regex - Version/\d+*/
	fiftyone_degrees_regfree(&R332);	/*Regex - (?<=Black[B|b]erry)[^/]+*/
	fiftyone_degrees_regfree(&R333);	/*Regex - (?<=Black[B|b]erry\w{4}/)\d+\.\d+\.\d+|(?<=Black[B|b]erry\w{5}/)\d+\.\d+\.\d+*/
	fiftyone_degrees_regfree(&R334);	/*Regex - (?<=UP.Browser/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R335);	/*Regex - UC Browser\d+\.\d+*/
	fiftyone_degrees_regfree(&R336);	/*Regex - (?<=BlackBerry)[^;]+*/
	fiftyone_degrees_regfree(&R337);	/*Regex - (?<=Version/)\d+\.\d+\.\d+*/
	fiftyone_degrees_regfree(&R338);	/*Regex - ^DoCoMo*/
	fiftyone_degrees_regfree(&R339);	/*Regex - (?<=DoCoMo/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R340);	/*Regex - (?<=DoCoMo/\d\.\d[\s|/])[\w\d]+*/
	fiftyone_degrees_regfree(&R341);	/*Regex - ^Fly|^FLY*/
	fiftyone_degrees_regfree(&R342);	/*Regex - ^HD2|HTC*/
	fiftyone_degrees_regfree(&R343);	/*Regex - (?<=\sHTC[\s|_])[^;\s]+*/
	fiftyone_degrees_regfree(&R344);	/*Regex - (?i)Huawei*/
	fiftyone_degrees_regfree(&R345);	/*Regex - ^KDDI*/
	fiftyone_degrees_regfree(&R346);	/*Regex - KDDI-\w+*/
	fiftyone_degrees_regfree(&R347);	/*Regex - ^kyocera|^QC-|^KWC-*/
	fiftyone_degrees_regfree(&R348);	/*Regex - ^LCT*/
	fiftyone_degrees_regfree(&R349);	/*Regex - ^Lenovo*/
	fiftyone_degrees_regfree(&R350);	/*Regex - (?<=^Lenovo[-_])[\w\d]+*/
	fiftyone_degrees_regfree(&R351);	/*Regex - ^LG/*/
	fiftyone_degrees_regfree(&R352);	/*Regex - (?<=^LG/)[\w\d]+*/
	fiftyone_degrees_regfree(&R353);	/*Regex - (?<=^LG(-|/))[^\s/]+*/
	fiftyone_degrees_regfree(&R354);	/*Regex - (?<=NovarraVision)\d\.\d*/
	fiftyone_degrees_regfree(&R355);	/*Regex - (?<=Browser/Teleca-)[^\s]+*/
	fiftyone_degrees_regfree(&R356);	/*Regex - (?<=UCBrowser)\d\.\d*/
	fiftyone_degrees_regfree(&R357);	/*Regex - (?<=Browser/Obigo-)[^/]+/[\d\.]+*/
	fiftyone_degrees_regfree(&R358);	/*Regex - (?<=Opera Mini/)[^\);]+*/
	fiftyone_degrees_regfree(&R359);	/*Regex - ^LG-*/
	fiftyone_degrees_regfree(&R360);	/*Regex - (?<=^LG-)[\w\d]+*/
	fiftyone_degrees_regfree(&R361);	/*Regex - UC( |)Browser(|/)\d+\.\d*/
	fiftyone_degrees_regfree(&R362);	/*Regex - ^LGE-*/
	fiftyone_degrees_regfree(&R363);	/*Regex - (?<=^LGE-)[\w\d]+*/
	fiftyone_degrees_regfree(&R364);	/*Regex - (?<=^LGE-)[^\s/]+*/
	fiftyone_degrees_regfree(&R365);	/*Regex - ^MOT-*/
	fiftyone_degrees_regfree(&R366);	/*Regex - (?<=^MOT-)[^/]+*/
	fiftyone_degrees_regfree(&R367);	/*Regex - (?<=UP\.Browser/)[\d\.]+*/
	fiftyone_degrees_regfree(&R368);	/*Regex - \([^\)]+\)*/
	fiftyone_degrees_regfree(&R369);	/*Regex - ^NEC|KGT*/
	fiftyone_degrees_regfree(&R370);	/*Regex - ^Nexian|^NEXIAN*/
	fiftyone_degrees_regfree(&R371);	/*Regex - (?i)nokia*/
	fiftyone_degrees_regfree(&R372);	/*Regex - Series(40|60)*/
	fiftyone_degrees_regfree(&R373);	/*Regex - (?i)(?<=Nokia)[^/\)]+*/
	fiftyone_degrees_regfree(&R374);	/*Regex - (?<=S40OviBrowser/)\d+\.\d+.\d+*/
	fiftyone_degrees_regfree(&R375);	/*Regex - ^iBrowser/Mini\d+\.\d+*/
	fiftyone_degrees_regfree(&R376);	/*Regex - ^OneBrowser/\d+\.\d+*/
	fiftyone_degrees_regfree(&R377);	/*Regex - ^MQQBrowser/Mini\d+\.\d+*/
	fiftyone_degrees_regfree(&R378);	/*Regex - Palm*/
	fiftyone_degrees_regfree(&R379);	/*Regex - Palm(S|s)ource*/
	fiftyone_degrees_regfree(&R380);	/*Regex - ^Panasonic*/
	fiftyone_degrees_regfree(&R381);	/*Regex - ^Pantech|^PT-|^PANTECH|^PG-*/
	fiftyone_degrees_regfree(&R382);	/*Regex - ^Philips|^PHILIPS*/
	fiftyone_degrees_regfree(&R383);	/*Regex - ^Qtek*/
	fiftyone_degrees_regfree(&R384);	/*Regex - iPhone|iPod|iPad|Nokia|BlackBerry|Android|Skyfire*/
	fiftyone_degrees_regfree(&R385);	/*Regex - (?<=Mozilla/\d\.\d \()[^)]+*/
	fiftyone_degrees_regfree(&R386);	/*Regex - Phantomjs/\d\.\d*/
	fiftyone_degrees_regfree(&R387);	/*Regex - LG Browser/\d\.\d*/
	fiftyone_degrees_regfree(&R388);	/*Regex - (?<=LGE;)[^;]+*/
	fiftyone_degrees_regfree(&R389);	/*Regex - ^Sagem|^SAGEM*/
	fiftyone_degrees_regfree(&R390);	/*Regex - (?i)SAMSUNG|SEC-|SCH-*/
	fiftyone_degrees_regfree(&R391);	/*Regex - Series60/*/
	fiftyone_degrees_regfree(&R392);	/*Regex - Samsung/*/
	fiftyone_degrees_regfree(&R393);	/*Regex - SAMSUNG-GT-*/
	fiftyone_degrees_regfree(&R394);	/*Regex - (?i)(?<=SAMSUNG)-?[^/]+*/
	fiftyone_degrees_regfree(&R395);	/*Regex - (?i)(?<=SEC|SCH)-?[^/]+*/
	fiftyone_degrees_regfree(&R396);	/*Regex - (?<=Bada/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R397);	/*Regex - (?<=Dolfin/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R398);	/*Regex - (?<=Jasmine/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R399);	/*Regex - UC(\.|)Browser/\d+\.\d+*/
	fiftyone_degrees_regfree(&R400);	/*Regex - (?<=Windows)\w+;*/
	fiftyone_degrees_regfree(&R401);	/*Regex - Android \d\.\d*/
	fiftyone_degrees_regfree(&R402);	/*Regex - UCWEB\d\.\d*/
	fiftyone_degrees_regfree(&R403);	/*Regex - (?<=SAMSUNG-GT-)[^/-]+*/
	fiftyone_degrees_regfree(&R404);	/*Regex - (?<=UC\.Browser/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R405);	/*Regex - (?<=MSIEMobile )\d+\.\d+*/
	fiftyone_degrees_regfree(&R406);	/*Regex - (?<=Samsung/)[^/]+*/
	fiftyone_degrees_regfree(&R407);	/*Regex - (?<=Series60/)[\d\.]+*/
	fiftyone_degrees_regfree(&R408);	/*Regex - (?<=SymbianOS/)[\d\.]+*/
	fiftyone_degrees_regfree(&R409);	/*Regex - ^Sanyo|^SANYO*/
	fiftyone_degrees_regfree(&R410);	/*Regex - Sanyo*/
	fiftyone_degrees_regfree(&R411);	/*Regex - SCP*/
	fiftyone_degrees_regfree(&R412);	/*Regex - ^Sharp|^SHARP*/
	fiftyone_degrees_regfree(&R413);	/*Regex - ^SIE-*/
	fiftyone_degrees_regfree(&R414);	/*Regex - ^SoftBank|^Vodafone|^J-PHONE*/
	fiftyone_degrees_regfree(&R415);	/*Regex - (SoftBank|Vodafone|J-PHONE)\/\d\.\d\/\w+*/
	fiftyone_degrees_regfree(&R416);	/*Regex - Spice*/
	fiftyone_degrees_regfree(&R417);	/*Regex - ^TIANYU-KTOUCH*/
	fiftyone_degrees_regfree(&R418);	/*Regex - (?<=TIANYU-KTOUCH/)[^/]+*/
	fiftyone_degrees_regfree(&R419);	/*Regex - Toshiba*/
	fiftyone_degrees_regfree(&R420);	/*Regex - ^Vodafone*/
	fiftyone_degrees_regfree(&R421);	/*Regex - webOS*/
	fiftyone_degrees_regfree(&R422);	/*Regex - HTC*/
	fiftyone_degrees_regfree(&R423);	/*Regex - (?<=\(|;|\))[^;\)]+*/
	fiftyone_degrees_regfree(&R424);	/*Regex - ^ZTE*/
	fiftyone_degrees_regfree(&R425);	/*Regex - (?i)Webkit*/
	fiftyone_degrees_regfree(&R426);	/*Regex - Chrome|Safari|Android|Nokia|BlackBerry*/
	fiftyone_degrees_regfree(&R427);	/*Regex - (?<=\(KHTML, like Gecko\))[^\(\)]+*/
	fiftyone_degrees_regfree(&R428);	/*Regex - Gecko*/
	fiftyone_degrees_regfree(&R429);	/*Regex - Fennec|Linux Mint|Firefox|IceWeasel|Thunderbird|WebKit|BonEcho*/
	fiftyone_degrees_regfree(&R430);	/*Regex - Windows NT \d\.\d(?=;)|Mac OS X [^\s]+(?=;)|Ubuntu|X11|Linux i686*/
	fiftyone_degrees_regfree(&R431);	/*Regex - Win\d(\d|x)*/
	fiftyone_degrees_regfree(&R432);	/*Regex - Fedora*/
	fiftyone_degrees_regfree(&R433);	/*Regex - (?<=StumbleUpon/)\d\.\d*/
	fiftyone_degrees_regfree(&R434);	/*Regex - Postbox/\d+\.\d*/
	fiftyone_degrees_regfree(&R435);	/*Regex - Minefield/\d\.\d*/
	fiftyone_degrees_regfree(&R436);	/*Regex - Netscape(\d|)/\d\.\d*/
	fiftyone_degrees_regfree(&R437);	/*Regex - lolifox/\d+\.\d*/
	fiftyone_degrees_regfree(&R438);	/*Regex - Icedove/\d+\.\d*/
	fiftyone_degrees_regfree(&R439);	/*Regex - zdesktop/\d+\.\d*/
	fiftyone_degrees_regfree(&R440);	/*Regex - Abrowser/\d+\.\d*/
	fiftyone_degrees_regfree(&R441);	/*Regex - Mnenhy/\d+\.\d*/
	fiftyone_degrees_regfree(&R442);	/*Regex - Konqueror/\d+\.\d*/
	fiftyone_degrees_regfree(&R443);	/*Regex - Ubuntu/\d+\.\d*/
	fiftyone_degrees_regfree(&R444);	/*Regex - Palemoon/\d+\.\d*/
	fiftyone_degrees_regfree(&R445);	/*Regex - SeaMonkey(/\d\.\d+|$)*/
	fiftyone_degrees_regfree(&R446);	/*Regex - GranParadiso/\d\.\d*/
	fiftyone_degrees_regfree(&R447);	/*Regex - K9Browser/\d+\.\d*/
	fiftyone_degrees_regfree(&R448);	/*Regex - iTunes*/
	fiftyone_degrees_regfree(&R449);	/*Regex - iPod/\d\.\d*/
	fiftyone_degrees_regfree(&R450);	/*Regex - (?<=Opera/)[\d+.]+|(?<=Opera )[\d\.]+*/
	fiftyone_degrees_regfree(&R451);	/*Regex - (?<=Presto/)\d+\.\d+*/
	fiftyone_degrees_regfree(&R452);	/*Regex - Opera Mini*/
	fiftyone_degrees_regfree(&R453);	/*Regex - (?<=Opera Mini/att/)\d(/|\.)\d*/
	fiftyone_degrees_regfree(&R454);	/*Regex - ^[^;]+*/
	fiftyone_degrees_regfree(&R455);	/*Regex - (?<=Opera Mini/)\d(/|\.)\d*/
	fiftyone_degrees_regfree(&R456);	/*Regex - (?<= \d\d\dX\d\d\d).+*/
	fiftyone_degrees_regfree(&R457);	/*Regex - (?<=Windows NT )\d\.\d*/
	fiftyone_degrees_regfree(&R458);	/*Regex - Opera Mini//\d+*/
	fiftyone_degrees_regfree(&R459);	/*Regex - Version/\d+\.\d+*/
	fiftyone_degrees_regfree(&R460);	/*Regex - Opera Mobi*/
	fiftyone_degrees_regfree(&R461);	/*Regex - ^portalmmm*/
	fiftyone_degrees_regfree(&R462);	/*Regex - Outlook*/
	fiftyone_degrees_regfree(&R463);	/*Regex - (?<=Microsoft Outlook )[\d\.]+*/
	fiftyone_degrees_regfree(&R464);	/*Regex - (?<=Outlook 20\d\d)[\d\.]+*/
	fiftyone_degrees_regfree(&R465);	/*Regex - Profile/MIDP-\d\.\d Configuration/CLDC-\d\.\d*/
	fiftyone_degrees_regfree(&R466);	/*Regex - .+(?=Profile/MIDP-\d\.\d Configuration/CLDC-\d\.\d)*/
	fiftyone_degrees_regfree(&R467);	/*Regex - WAP|MIDP|CLDC|Teleca-|Screen-|mobile*/
	fiftyone_degrees_regfree(&R468);	/*Regex - ^iOS/.+dataaccessd*/
	fiftyone_degrees_regfree(&R469);	/*Regex - ^Podcasts*/
	fiftyone_degrees_regfree(&R470);	/*Regex - ^Tiny Tiny RSS*/
	fiftyone_degrees_regfree(&R471);	/*Regex - ^Amazon CloudFront*/
	fiftyone_degrees_regfree(&R472);	/*Regex - ^Wada\.vn*/
	fiftyone_degrees_regfree(&R473);	/*Regex - ^FeedDemon*/
	fiftyone_degrees_regfree(&R474);	/*Regex - ^python-requests*/
	fiftyone_degrees_regfree(&R475);	/*Regex - ^RSSInclude*/
	fiftyone_degrees_regfree(&R476);	/*Regex - ^curl*/
	fiftyone_degrees_regfree(&R477);	/*Regex - ^Pulp/*/
	fiftyone_degrees_regfree(&R478);	/*Regex - Win98*/
	fiftyone_degrees_regfree(&R479);	/*Regex - AppEngine*/
	fiftyone_degrees_regfree(&R480);	/*Regex - ^[^\(]+*/
	fiftyone_degrees_regfree(&R481);	/*Regex - darwin*/
	fiftyone_degrees_regfree(&R482);	/*Regex - ^(I|J)UC*/
	fiftyone_degrees_regfree(&R483);	/*Regex - ^(J|I)UC*/
	fiftyone_degrees_regfree(&R484);	/*Regex - UCWEB\d+\.\d*/
	fiftyone_degrees_regfree(&R485);	/*Regex - (?<=(Z|z)h(-|_)cn;)[^;]+*/
	fiftyone_degrees_regfree(&R486);	/*Regex - Linux;\sU;\s\d\.\d*/
	fiftyone_degrees_regfree(&R487);	/*Regex - iOS \d\.\d*/
	fiftyone_degrees_regfree(&R488);	/*Regex - ^server-bag*/
	fiftyone_degrees_regfree(&R489);	/*Regex - iPhone OS,\d\.\d*/
	fiftyone_degrees_regfree(&R490);	/*Regex - ,iPhone|,iPod|,iPad*/
	fiftyone_degrees_regfree(&R491);	/*Regex - ^-1*/
	fiftyone_degrees_regfree(&R492);	/*Regex - ^MicroMax|^Micromax*/
	fiftyone_degrees_regfree(&R493);	/*Regex - ^Micromax.+$*/
	fiftyone_degrees_regfree(&R494);	/*Regex - jig browser*/
	fiftyone_degrees_regfree(&R495);	/*Regex - 1.0.4; .+\)$*/
	fiftyone_degrees_regfree(&R496);	/*Regex - Konqueror*/
	fiftyone_degrees_regfree(&R497);	/*Regex - Konqueror/\d\.\d*/
	fiftyone_degrees_regfree(&R498);	/*Regex - Ubuntu*/
	fiftyone_degrees_regfree(&R499);	/*Regex - heritrix*/
	fiftyone_degrees_regfree(&R500);	/*Regex - heritrix/\d\.\d*/
	fiftyone_degrees_regfree(&R501);	/*Regex - Java/\d\.\d*/
	fiftyone_degrees_regfree(&R502);	/*Regex - (?<=Java/)\d\.\d*/
	fiftyone_degrees_regfree(&R503);	/*Regex - Windows .+\)*/
	fiftyone_degrees_regfree(&R504);	/*Regex - Max OS X \d+\.\d*/
	fiftyone_degrees_regfree(&R505);	/*Regex - UCWEB*/
	fiftyone_degrees_regfree(&R506);	/*Regex - (?<=UCBrowser)(|/)\d\.\d*/
	fiftyone_degrees_regfree(&R507);	/*Regex - (?<=\w\w-\w\w;)[^\)]+*/
	fiftyone_degrees_regfree(&R508);	/*Regex - ^MacOutlook*/
	fiftyone_degrees_regfree(&R509);	/*Regex - (?<=MacOutlook/)[\d\.]+*/
	fiftyone_degrees_regfree(&R510);	/*Regex - (?<=Mac OS X )[\d\.]+*/
	fiftyone_degrees_regfree(&R511);	/*Regex - \w+*/
}


const Segment SGI0[] = {{'\0', '\0'}, {NULL}};
const Segment SGI1[] = {{&S2910[6], 4}, {NULL}};
const Segment SGI2[] = {{&S2910[32], 3}, {NULL}};
const Segment SGI3[] = {{&S2911[6], 4}, {NULL}};
const Segment SGI4[] = {{&S2911[32], 3}, {NULL}};
const Segment SGI5[] = {{&S2911[81], 5}, {NULL}};
const Segment SGI6[] = {{&S4598[57], 10}, {NULL}};
const Segment SGI7[] = {{&S4598[57], 27}, {NULL}};
const Segment SGI8[] = {{&S4598[30], 19}, {NULL}};
const Segment SGI9[] = {{&S4598[132], 3}, {NULL}};
const Segment SGI10[] = {{&S4598[98], 5}, {NULL}};
const Segment SGI11[] = {{&S4598[58], 8}, {NULL}};
const Segment SGI12[] = {{&S4600[13], 8}, {NULL}};
const Segment SGI13[] = {{&S4600[23], 12}, {NULL}};
const Segment SGI14[] = {{&S4611[8], 8}, {NULL}};
const Segment SGI15[] = {{&S4658[57], 10}, {NULL}};
const Segment SGI16[] = {{&S4658[57], 27}, {NULL}};
const Segment SGI17[] = {{&S4658[30], 19}, {NULL}};
const Segment SGI18[] = {{&S4658[132], 3}, {NULL}};
const Segment SGI19[] = {{&S4658[98], 5}, {NULL}};
const Segment SGI20[] = {{&S4658[58], 8}, {NULL}};
const Segment SGI21[] = {{&S4669[62], 10}, {NULL}};
const Segment SGI22[] = {{&S4669[62], 27}, {NULL}};
const Segment SGI23[] = {{&S4669[30], 24}, {NULL}};
const Segment SGI24[] = {{&S4669[137], 3}, {NULL}};
const Segment SGI25[] = {{&S4669[103], 5}, {NULL}};
const Segment SGI26[] = {{&S4669[63], 8}, {NULL}};
const Segment SGI27[] = {{&S8833[6], 4}, {NULL}};
const Segment SGI28[] = {{&S8833[32], 3}, {NULL}};
const Segment SGI29[] = {{&S8833[81], 5}, {NULL}};
const Segment SGI30[] = {{&S11344[6], 4}, {NULL}};
const Segment SGI31[] = {{&S11344[32], 3}, {NULL}};
const Segment SGI32[] = {{&S11344[81], 5}, {NULL}};
const Segment SGI33[] = {{&S11345[6], 4}, {NULL}};
const Segment SGI34[] = {{&S11345[32], 3}, {NULL}};
const Segment SGI35[] = {{&S11345[81], 5}, {NULL}};
const Segment SGI36[] = {{&S11346[6], 4}, {NULL}};
const Segment SGI37[] = {{&S11346[32], 3}, {NULL}};
const Segment SGI38[] = {{&S11557[44], 18}, {NULL}};
const Segment SGI39[] = {{&S11557[44], 29}, {NULL}};
const Segment SGI40[] = {{&S11557[30], 6}, {NULL}};
const Segment SGI41[] = {{&S11557[121], 3}, {NULL}};
const Segment SGI42[] = {{&S11557[87], 5}, {NULL}};
const Segment SGI43[] = {{&S11559[44], 12}, {NULL}};
const Segment SGI44[] = {{&S11559[44], 24}, {NULL}};
const Segment SGI45[] = {{&S11559[30], 6}, {NULL}};
const Segment SGI46[] = {{&S11559[116], 3}, {NULL}};
const Segment SGI47[] = {{&S11559[82], 5}, {NULL}};
const Segment SGI48[] = {{&S11555[42], 14}, {NULL}};
const Segment SGI49[] = {{&S11555[42], 26}, {NULL}};
const Segment SGI50[] = {{&S11555[30], 4}, {NULL}};
const Segment SGI51[] = {{&S11555[116], 3}, {NULL}};
const Segment SGI52[] = {{&S11555[82], 5}, {NULL}};
const Segment SGI53[] = {{&S11556[50], 14}, {NULL}};
const Segment SGI54[] = {{&S11556[50], 25}, {NULL}};
const Segment SGI55[] = {{&S11556[30], 12}, {NULL}};
const Segment SGI56[] = {{&S11556[123], 3}, {NULL}};
const Segment SGI57[] = {{&S11556[89], 5}, {NULL}};
const Segment SGI58[] = {{&S11558[42], 23}, {NULL}};
const Segment SGI59[] = {{&S11558[42], 34}, {NULL}};
const Segment SGI60[] = {{&S11558[30], 4}, {NULL}};
const Segment SGI61[] = {{&S11558[124], 3}, {NULL}};
const Segment SGI62[] = {{&S11558[90], 5}, {NULL}};
const Segment SGI63[] = {{&S11560[42], 12}, {NULL}};
const Segment SGI64[] = {{&S11560[42], 23}, {NULL}};
const Segment SGI65[] = {{&S11560[30], 4}, {NULL}};
const Segment SGI66[] = {{&S11560[113], 3}, {NULL}};
const Segment SGI67[] = {{&S11560[79], 5}, {NULL}};
const Segment SGI68[] = {{&S11561[42], 14}, {NULL}};
const Segment SGI69[] = {{&S11561[42], 25}, {NULL}};
const Segment SGI70[] = {{&S11561[30], 4}, {NULL}};
const Segment SGI71[] = {{&S11561[115], 3}, {NULL}};
const Segment SGI72[] = {{&S11561[81], 5}, {NULL}};
const Segment SGI73[] = {{&S11562[35], 14}, {NULL}};
const Segment SGI74[] = {{&S11562[30], 4}, {NULL}};
const Segment SGI75[] = {{&S11562[108], 3}, {NULL}};
const Segment SGI76[] = {{&S11562[74], 5}, {NULL}};
const Segment SGI77[] = {{&S11575[44], 30}, {NULL}};
const Segment SGI78[] = {{&S11575[44], 42}, {NULL}};
const Segment SGI79[] = {{&S11575[30], 6}, {NULL}};
const Segment SGI80[] = {{&S11563[42], 22}, {NULL}};
const Segment SGI81[] = {{&S11563[42], 33}, {NULL}};
const Segment SGI82[] = {{&S11563[30], 4}, {NULL}};
const Segment SGI83[] = {{&S11575[134], 3}, {NULL}};
const Segment SGI84[] = {{&S11563[123], 3}, {NULL}};
const Segment SGI85[] = {{&S11575[100], 5}, {NULL}};
const Segment SGI86[] = {{&S11563[89], 5}, {NULL}};
const Segment SGI87[] = {{&S11564[42], 101}, {NULL}};
const Segment SGI88[] = {{&S11564[42], 112}, {NULL}};
const Segment SGI89[] = {{&S11564[30], 4}, {NULL}};
const Segment SGI90[] = {{&S11564[202], 3}, {NULL}};
const Segment SGI91[] = {{&S11564[168], 5}, {NULL}};
const Segment SGI92[] = {{&S11565[42], 23}, {NULL}};
const Segment SGI93[] = {{&S11565[42], 34}, {NULL}};
const Segment SGI94[] = {{&S11565[30], 4}, {NULL}};
const Segment SGI95[] = {{&S11565[124], 3}, {NULL}};
const Segment SGI96[] = {{&S11580[44], 18}, {NULL}};
const Segment SGI97[] = {{&S11565[90], 5}, {NULL}};
const Segment SGI98[] = {{&S11580[44], 30}, {NULL}};
const Segment SGI99[] = {{&S11580[30], 6}, {NULL}};
const Segment SGI100[] = {{&S11580[122], 3}, {NULL}};
const Segment SGI101[] = {{&S11580[88], 5}, {NULL}};
const Segment SGI102[] = {{&S11566[42], 23}, {NULL}};
const Segment SGI103[] = {{&S11566[42], 34}, {NULL}};
const Segment SGI104[] = {{&S11566[30], 4}, {NULL}};
const Segment SGI105[] = {{&S11566[124], 3}, {NULL}};
const Segment SGI106[] = {{&S11566[90], 5}, {NULL}};
const Segment SGI107[] = {{&S11567[42], 23}, {NULL}};
const Segment SGI108[] = {{&S11567[42], 34}, {NULL}};
const Segment SGI109[] = {{&S11567[30], 4}, {NULL}};
const Segment SGI110[] = {{&S11567[124], 3}, {NULL}};
const Segment SGI111[] = {{&S11567[90], 5}, {NULL}};
const Segment SGI112[] = {{&S11568[42], 11}, {NULL}};
const Segment SGI113[] = {{&S11568[42], 22}, {NULL}};
const Segment SGI114[] = {{&S11585[44], 12}, {NULL}};
const Segment SGI115[] = {{&S11568[30], 4}, {NULL}};
const Segment SGI116[] = {{&S11585[44], 23}, {NULL}};
const Segment SGI117[] = {{&S11585[30], 6}, {NULL}};
const Segment SGI118[] = {{&S11568[112], 3}, {NULL}};
const Segment SGI119[] = {{&S11585[115], 3}, {NULL}};
const Segment SGI120[] = {{&S11568[78], 5}, {NULL}};
const Segment SGI121[] = {{&S11585[81], 5}, {NULL}};
const Segment SGI122[] = {{&S11569[42], 23}, {NULL}};
const Segment SGI123[] = {{&S11569[42], 34}, {NULL}};
const Segment SGI124[] = {{&S11586[44], 17}, {NULL}};
const Segment SGI125[] = {{&S11569[30], 4}, {NULL}};
const Segment SGI126[] = {{&S11586[44], 28}, {NULL}};
const Segment SGI127[] = {{&S11586[30], 6}, {NULL}};
const Segment SGI128[] = {{&S11569[124], 3}, {NULL}};
const Segment SGI129[] = {{&S11586[120], 3}, {NULL}};
const Segment SGI130[] = {{&S11569[90], 5}, {NULL}};
const Segment SGI131[] = {{&S11586[86], 5}, {NULL}};
const Segment SGI132[] = {{&S11570[42], 12}, {NULL}};
const Segment SGI133[] = {{&S11570[42], 23}, {NULL}};
const Segment SGI134[] = {{&S11587[44], 12}, {NULL}};
const Segment SGI135[] = {{&S11587[44], 23}, {NULL}};
const Segment SGI136[] = {{&S11570[30], 4}, {NULL}};
const Segment SGI137[] = {{&S11587[30], 6}, {NULL}};
const Segment SGI138[] = {{&S11570[113], 3}, {NULL}};
const Segment SGI139[] = {{&S11587[115], 3}, {NULL}};
const Segment SGI140[] = {{&S11570[79], 5}, {NULL}};
const Segment SGI141[] = {{&S11587[81], 5}, {NULL}};
const Segment SGI142[] = {{&S11571[42], 7}, {NULL}};
const Segment SGI143[] = {{&S11588[44], 11}, {NULL}};
const Segment SGI144[] = {{&S11571[42], 18}, {NULL}};
const Segment SGI145[] = {{&S11571[30], 4}, {NULL}};
const Segment SGI146[] = {{&S11588[44], 22}, {NULL}};
const Segment SGI147[] = {{&S11588[30], 6}, {NULL}};
const Segment SGI148[] = {{&S11571[108], 3}, {NULL}};
const Segment SGI149[] = {{&S11588[114], 3}, {NULL}};
const Segment SGI150[] = {{&S11571[74], 5}, {NULL}};
const Segment SGI151[] = {{&S11588[80], 5}, {NULL}};
const Segment SGI152[] = {{&S11572[42], 10}, {NULL}};
const Segment SGI153[] = {{&S11572[42], 21}, {NULL}};
const Segment SGI154[] = {{&S11572[30], 4}, {NULL}};
const Segment SGI155[] = {{&S11572[111], 3}, {NULL}};
const Segment SGI156[] = {{&S11572[77], 5}, {NULL}};
const Segment SGI157[] = {{&S11573[42], 30}, {NULL}};
const Segment SGI158[] = {{&S11573[42], 41}, {NULL}};
const Segment SGI159[] = {{&S11573[30], 4}, {NULL}};
const Segment SGI160[] = {{&S11573[131], 3}, {NULL}};
const Segment SGI161[] = {{&S11573[97], 5}, {NULL}};
const Segment SGI162[] = {{&S11574[42], 30}, {NULL}};
const Segment SGI163[] = {{&S11574[42], 41}, {NULL}};
const Segment SGI164[] = {{&S11574[30], 4}, {NULL}};
const Segment SGI165[] = {{&S11574[131], 3}, {NULL}};
const Segment SGI166[] = {{&S11574[97], 5}, {NULL}};
const Segment SGI167[] = {{&S11576[42], 30}, {NULL}};
const Segment SGI168[] = {{&S11576[42], 41}, {NULL}};
const Segment SGI169[] = {{&S11576[30], 4}, {NULL}};
const Segment SGI170[] = {{&S11576[131], 3}, {NULL}};
const Segment SGI171[] = {{&S11576[97], 5}, {NULL}};
const Segment SGI172[] = {{&S11601[38], 12}, {NULL}};
const Segment SGI173[] = {{&S11601[31], 6}, {NULL}};
const Segment SGI174[] = {{&S11579[42], 13}, {NULL}};
const Segment SGI175[] = {{&S11579[42], 24}, {NULL}};
const Segment SGI176[] = {{&S11579[30], 4}, {NULL}};
const Segment SGI177[] = {{&S11579[114], 3}, {NULL}};
const Segment SGI178[] = {{&S11579[80], 5}, {NULL}};
const Segment SGI179[] = {{&S11604[44], 12}, {NULL}};
const Segment SGI180[] = {{&S11604[44], 22}, {NULL}};
const Segment SGI181[] = {{&S11604[30], 6}, {NULL}};
const Segment SGI182[] = {{&S11604[114], 3}, {NULL}};
const Segment SGI183[] = {{&S11581[42], 23}, {NULL}};
const Segment SGI184[] = {{&S11581[42], 34}, {NULL}};
const Segment SGI185[] = {{&S11604[80], 5}, {NULL}};
const Segment SGI186[] = {{&S11581[30], 4}, {NULL}};
const Segment SGI187[] = {{&S11581[124], 3}, {NULL}};
const Segment SGI188[] = {{&S11581[90], 5}, {NULL}};
const Segment SGI189[] = {{&S11582[42], 18}, {NULL}};
const Segment SGI190[] = {{&S11582[42], 29}, {NULL}};
const Segment SGI191[] = {{&S11582[30], 4}, {NULL}};
const Segment SGI192[] = {{&S11582[119], 3}, {NULL}};
const Segment SGI193[] = {{&S11582[85], 5}, {NULL}};
const Segment SGI194[] = {{&S11583[42], 12}, {NULL}};
const Segment SGI195[] = {{&S11583[42], 23}, {NULL}};
const Segment SGI196[] = {{&S11583[30], 4}, {NULL}};
const Segment SGI197[] = {{&S11583[115], 3}, {NULL}};
const Segment SGI198[] = {{&S11583[79], 7}, {NULL}};
const Segment SGI199[] = {{&S11583[135], 8}, {NULL}};
const Segment SGI200[] = {{&S11584[36], 12}, {NULL}};
const Segment SGI201[] = {{&S11584[31], 4}, {NULL}};
const Segment SGI202[] = {{&S11593[40], 12}, {NULL}};
const Segment SGI203[] = {{&S11593[30], 4}, {NULL}};
const Segment SGI204[] = {{&S11593[111], 3}, {NULL}};
const Segment SGI205[] = {{&S11593[77], 5}, {NULL}};
const Segment SGI206[] = {{&S11594[50], 12}, {NULL}};
const Segment SGI207[] = {{&S11594[50], 23}, {NULL}};
const Segment SGI208[] = {{&S11594[30], 12}, {NULL}};
const Segment SGI209[] = {{&S11594[121], 3}, {NULL}};
const Segment SGI210[] = {{&S11594[87], 5}, {NULL}};
const Segment SGI211[] = {{&S11595[42], 12}, {NULL}};
const Segment SGI212[] = {{&S11595[42], 23}, {NULL}};
const Segment SGI213[] = {{&S11595[30], 4}, {NULL}};
const Segment SGI214[] = {{&S11595[82], 4}, {NULL}};
const Segment SGI215[] = {{&S11595[82], 3}, {NULL}};
const Segment SGI216[] = {{&S11596[42], 14}, {NULL}};
const Segment SGI217[] = {{&S11596[42], 25}, {NULL}};
const Segment SGI218[] = {{&S11596[30], 4}, {NULL}};
const Segment SGI219[] = {{&S11596[115], 3}, {NULL}};
const Segment SGI220[] = {{&S11596[133], 5}, {NULL}};
const Segment SGI221[] = {{&S11602[42], 12}, {NULL}};
const Segment SGI222[] = {{&S11602[42], 23}, {NULL}};
const Segment SGI223[] = {{&S11602[30], 4}, {NULL}};
const Segment SGI224[] = {{&S11602[82], 6}, {NULL}};
const Segment SGI225[] = {{&S11603[36], 7}, {NULL}};
const Segment SGI226[] = {{&S11603[31], 4}, {NULL}};
const Segment SGI227[] = {{&S11605[42], 17}, {NULL}};
const Segment SGI228[] = {{&S11605[42], 28}, {NULL}};
const Segment SGI229[] = {{&S11605[30], 4}, {NULL}};
const Segment SGI230[] = {{&S11605[118], 3}, {NULL}};
const Segment SGI231[] = {{&S11605[84], 5}, {NULL}};
const Segment SGI232[] = {{&S11609[32], 11}, {NULL}};
const Segment SGI233[] = {{&S14906[74], 10}, {NULL}};
const Segment SGI234[] = {{&S14906[74], 27}, {NULL}};
const Segment SGI235[] = {{&S14906[30], 36}, {NULL}};
const Segment SGI236[] = {{&S14906[149], 3}, {NULL}};
const Segment SGI237[] = {{&S14906[115], 5}, {NULL}};
const Segment SGI238[] = {{&S14906[75], 8}, {NULL}};
const Segment SGI239[] = {{&S14908[58], 10}, {NULL}};
const Segment SGI240[] = {{&S14908[58], 27}, {NULL}};
const Segment SGI241[] = {{&S14908[30], 20}, {NULL}};
const Segment SGI242[] = {{&S14908[133], 3}, {NULL}};
const Segment SGI243[] = {{&S14908[99], 5}, {NULL}};
const Segment SGI244[] = {{&S14908[59], 8}, {NULL}};
const Segment SGI245[] = {{&S14909[56], 10}, {NULL}};
const Segment SGI246[] = {{&S14909[56], 27}, {NULL}};
const Segment SGI247[] = {{&S14909[30], 18}, {NULL}};
const Segment SGI248[] = {{&S14909[131], 3}, {NULL}};
const Segment SGI249[] = {{&S14909[97], 5}, {NULL}};
const Segment SGI250[] = {{&S14909[57], 8}, {NULL}};
const Segment SGI251[] = {{&S14910[74], 10}, {NULL}};
const Segment SGI252[] = {{&S14910[74], 27}, {NULL}};
const Segment SGI253[] = {{&S14910[30], 36}, {NULL}};
const Segment SGI254[] = {{&S14910[149], 3}, {NULL}};
const Segment SGI255[] = {{&S14910[115], 5}, {NULL}};
const Segment SGI256[] = {{&S14910[75], 8}, {NULL}};
const Segment SGI257[] = {{&S14912[74], 10}, {NULL}};
const Segment SGI258[] = {{&S14912[74], 27}, {NULL}};
const Segment SGI259[] = {{&S14912[30], 36}, {NULL}};
const Segment SGI260[] = {{&S14912[149], 3}, {NULL}};
const Segment SGI261[] = {{&S14912[115], 5}, {NULL}};
const Segment SGI262[] = {{&S14912[75], 8}, {NULL}};
const Segment SGI263[] = {{&S14913[74], 10}, {NULL}};
const Segment SGI264[] = {{&S14913[74], 27}, {NULL}};
const Segment SGI265[] = {{&S14913[30], 36}, {NULL}};
const Segment SGI266[] = {{&S14913[149], 3}, {NULL}};
const Segment SGI267[] = {{&S14913[115], 5}, {NULL}};
const Segment SGI268[] = {{&S14913[75], 8}, {NULL}};
const Segment SGI269[] = {{&S14916[58], 10}, {NULL}};
const Segment SGI270[] = {{&S14916[58], 27}, {NULL}};
const Segment SGI271[] = {{&S14916[30], 20}, {NULL}};
const Segment SGI272[] = {{&S14916[133], 3}, {NULL}};
const Segment SGI273[] = {{&S14916[99], 5}, {NULL}};
const Segment SGI274[] = {{&S14916[59], 8}, {NULL}};
const Segment SGI275[] = {{&S14941[74], 10}, {NULL}};
const Segment SGI276[] = {{&S14941[74], 27}, {NULL}};
const Segment SGI277[] = {{&S14941[30], 36}, {NULL}};
const Segment SGI278[] = {{&S14941[149], 3}, {NULL}};
const Segment SGI279[] = {{&S14941[115], 5}, {NULL}};
const Segment SGI280[] = {{&S14941[75], 8}, {NULL}};
const Segment SGI281[] = {{&S16389[75], 5}, {NULL}};
const Segment SGI282[] = {{&S16579[96], 5}, {NULL}};
const Segment SGI283[] = {{&S16681[109], 3}, {NULL}};
const Segment SGI284[] = {{&S16681[95], 5}, {NULL}};
const Segment SGI285[] = {{&S20352[56], 7}, {NULL}};
const Segment SGI286[] = {{&S20352[30], 25}, {NULL}};
const Segment SGI287[] = {{&S20352[128], 3}, {NULL}};
const Segment SGI288[] = {{&S20352[94], 5}, {NULL}};
const Segment SGI289[] = {{&S22947[53], 8}, {&S22947[53], 8}, {&S22947[53], 8}, {&S22947[53], 8}, {&S22947[53], 8}, {NULL}};
const Segment SGI290[] = {{&S22947[117], 9}, {NULL}};
const Segment SGI291[] = {{&S22691[50], 6}, {NULL}};
const Segment SGI292[] = {{&S22691[50], 24}, {NULL}};
const Segment SGI293[] = {{&S22691[30], 12}, {NULL}};
const Segment SGI294[] = {{&S22691[123], 3}, {NULL}};
const Segment SGI295[] = {{&S22691[88], 6}, {NULL}};
const Segment SGI296[] = {{&S22696[64], 6}, {NULL}};
const Segment SGI297[] = {{&S22696[64], 23}, {NULL}};
const Segment SGI298[] = {{&S22696[30], 26}, {NULL}};
const Segment SGI299[] = {{&S22696[136], 3}, {NULL}};
const Segment SGI300[] = {{&S22696[101], 6}, {NULL}};
const Segment SGI301[] = {{&S22697[45], 4}, {NULL}};
const Segment SGI302[] = {{&S22697[94], 3}, {NULL}};
const Segment SGI303[] = {{&S22697[71], 6}, {NULL}};
const Segment SGI304[] = {{&S22698[50], 6}, {NULL}};
const Segment SGI305[] = {{&S22698[50], 24}, {NULL}};
const Segment SGI306[] = {{&S22698[30], 12}, {NULL}};
const Segment SGI307[] = {{&S22698[123], 3}, {NULL}};
const Segment SGI308[] = {{&S22698[88], 6}, {NULL}};
const Segment SGI309[] = {{&S22700[50], 6}, {NULL}};
const Segment SGI310[] = {{&S22700[50], 24}, {NULL}};
const Segment SGI311[] = {{&S22700[30], 12}, {NULL}};
const Segment SGI312[] = {{&S22700[123], 3}, {NULL}};
const Segment SGI313[] = {{&S22700[88], 6}, {NULL}};
const Segment SGI314[] = {{&S22701[50], 6}, {NULL}};
const Segment SGI315[] = {{&S22701[50], 24}, {NULL}};
const Segment SGI316[] = {{&S22701[30], 12}, {NULL}};
const Segment SGI317[] = {{&S22701[123], 3}, {NULL}};
const Segment SGI318[] = {{&S22701[88], 6}, {NULL}};
const Segment SGI319[] = {{&S22702[50], 6}, {NULL}};
const Segment SGI320[] = {{&S22702[50], 23}, {NULL}};
const Segment SGI321[] = {{&S22702[30], 12}, {NULL}};
const Segment SGI322[] = {{&S22702[122], 3}, {NULL}};
const Segment SGI323[] = {{&S22702[87], 6}, {NULL}};
const Segment SGI324[] = {{&S22703[50], 6}, {NULL}};
const Segment SGI325[] = {{&S22703[50], 23}, {NULL}};
const Segment SGI326[] = {{&S22703[30], 12}, {NULL}};
const Segment SGI327[] = {{&S22703[122], 3}, {NULL}};
const Segment SGI328[] = {{&S22703[87], 6}, {NULL}};
const Segment SGI329[] = {{&S22704[48], 6}, {NULL}};
const Segment SGI330[] = {{&S22704[30], 12}, {NULL}};
const Segment SGI331[] = {{&S22704[120], 3}, {NULL}};
const Segment SGI332[] = {{&S22704[85], 6}, {NULL}};
const Segment SGI333[] = {{&S22705[50], 6}, {NULL}};
const Segment SGI334[] = {{&S22705[50], 24}, {NULL}};
const Segment SGI335[] = {{&S22705[30], 12}, {NULL}};
const Segment SGI336[] = {{&S22705[123], 3}, {NULL}};
const Segment SGI337[] = {{&S22705[88], 6}, {NULL}};
const Segment SGI338[] = {{&S22707[46], 5}, {NULL}};
const Segment SGI339[] = {{&S22707[26], 12}, {NULL}};
const Segment SGI340[] = {{&S22707[88], 3}, {NULL}};
const Segment SGI341[] = {{&S22707[65], 6}, {NULL}};
const Segment SGI342[] = {{&S22708[50], 6}, {NULL}};
const Segment SGI343[] = {{&S22708[50], 24}, {NULL}};
const Segment SGI344[] = {{&S22708[30], 12}, {NULL}};
const Segment SGI345[] = {{&S22708[123], 3}, {NULL}};
const Segment SGI346[] = {{&S22708[88], 6}, {NULL}};
const Segment SGI347[] = {{&S22709[44], 6}, {NULL}};
const Segment SGI348[] = {{&S22709[31], 12}, {NULL}};
const Segment SGI349[] = {{&S22710[50], 6}, {NULL}};
const Segment SGI350[] = {{&S22710[50], 23}, {NULL}};
const Segment SGI351[] = {{&S22710[30], 12}, {NULL}};
const Segment SGI352[] = {{&S22710[122], 3}, {NULL}};
const Segment SGI353[] = {{&S22710[87], 6}, {NULL}};
const Segment SGI354[] = {{&S22711[50], 6}, {NULL}};
const Segment SGI355[] = {{&S22711[50], 23}, {NULL}};
const Segment SGI356[] = {{&S22711[30], 12}, {NULL}};
const Segment SGI357[] = {{&S22711[90], 4}, {NULL}};
const Segment SGI358[] = {{&S22711[90], 3}, {NULL}};
const Segment SGI359[] = {{&S22713[44], 6}, {NULL}};
const Segment SGI360[] = {{&S22713[31], 12}, {NULL}};
const Segment SGI361[] = {{&S22714[50], 6}, {NULL}};
const Segment SGI362[] = {{&S22714[50], 24}, {NULL}};
const Segment SGI363[] = {{&S22714[30], 12}, {NULL}};
const Segment SGI364[] = {{&S22714[91], 4}, {NULL}};
const Segment SGI365[] = {{&S22714[91], 3}, {NULL}};
const Segment SGI366[] = {{&S22715[50], 6}, {NULL}};
const Segment SGI367[] = {{&S22715[50], 24}, {NULL}};
const Segment SGI368[] = {{&S22715[30], 12}, {NULL}};
const Segment SGI369[] = {{&S22715[91], 4}, {NULL}};
const Segment SGI370[] = {{&S22715[91], 3}, {NULL}};
const Segment SGI371[] = {{&S22716[50], 6}, {NULL}};
const Segment SGI372[] = {{&S22716[50], 23}, {NULL}};
const Segment SGI373[] = {{&S22716[30], 12}, {NULL}};
const Segment SGI374[] = {{&S22716[90], 4}, {NULL}};
const Segment SGI375[] = {{&S22716[90], 3}, {NULL}};
const Segment SGI376[] = {{&S22717[50], 6}, {NULL}};
const Segment SGI377[] = {{&S22717[50], 23}, {NULL}};
const Segment SGI378[] = {{&S22717[30], 12}, {NULL}};
const Segment SGI379[] = {{&S22717[90], 6}, {NULL}};
const Segment SGI380[] = {{&S22718[50], 6}, {NULL}};
const Segment SGI381[] = {{&S22718[50], 24}, {NULL}};
const Segment SGI382[] = {{&S22718[30], 12}, {NULL}};
const Segment SGI383[] = {{&S22718[91], 6}, {NULL}};
const Segment SGI384[] = {{&S22719[50], 6}, {NULL}};
const Segment SGI385[] = {{&S22719[50], 23}, {NULL}};
const Segment SGI386[] = {{&S22719[30], 12}, {NULL}};
const Segment SGI387[] = {{&S22719[122], 3}, {NULL}};
const Segment SGI388[] = {{&S22719[87], 6}, {NULL}};
const Segment SGI389[] = {{&S23184[56], 9}, {NULL}};
const Segment SGI390[] = {{&S23184[56], 20}, {NULL}};
const Segment SGI391[] = {{&S23184[30], 18}, {NULL}};
const Segment SGI392[] = {{&S23184[137], 3}, {NULL}};
const Segment SGI393[] = {{&S23184[103], 5}, {NULL}};
const Segment SGI394[] = {{&S23528[6], 4}, {NULL}};
const Segment SGI395[] = {{&S23528[32], 3}, {NULL}};
const Segment SGI396[] = {{&S23528[81], 5}, {NULL}};
const Segment SGI397[] = {{&S23529[6], 4}, {NULL}};
const Segment SGI398[] = {{&S23529[32], 3}, {NULL}};
const Segment SGI399[] = {{&S23529[81], 5}, {NULL}};
const Segment SGI400[] = {{&S22786[44], 10}, {NULL}};
const Segment SGI401[] = {{&S22786[44], 21}, {NULL}};
const Segment SGI402[] = {{&S22786[30], 6}, {NULL}};
const Segment SGI403[] = {{&S22786[113], 3}, {NULL}};
const Segment SGI404[] = {{&S22786[79], 5}, {NULL}};
const Segment SGI405[] = {{&S22787[42], 10}, {NULL}};
const Segment SGI406[] = {{&S22787[30], 6}, {NULL}};
const Segment SGI407[] = {{&S22787[111], 3}, {NULL}};
const Segment SGI408[] = {{&S22787[77], 5}, {NULL}};
const Segment SGI409[] = {{&S22790[44], 10}, {NULL}};
const Segment SGI410[] = {{&S22790[44], 29}, {NULL}};
const Segment SGI411[] = {{&S22790[30], 6}, {NULL}};
const Segment SGI412[] = {{&S22790[121], 3}, {NULL}};
const Segment SGI413[] = {{&S22790[87], 5}, {NULL}};
const Segment SGI414[] = {{&S22792[44], 10}, {NULL}};
const Segment SGI415[] = {{&S22792[44], 21}, {NULL}};
const Segment SGI416[] = {{&S22792[30], 6}, {NULL}};
const Segment SGI417[] = {{&S22792[114], 3}, {NULL}};
const Segment SGI418[] = {{&S22792[79], 6}, {NULL}};
const Segment SGI419[] = {{&S22795[44], 10}, {NULL}};
const Segment SGI420[] = {{&S22795[44], 21}, {NULL}};
const Segment SGI421[] = {{&S22795[30], 6}, {NULL}};
const Segment SGI422[] = {{&S22795[113], 3}, {NULL}};
const Segment SGI423[] = {{&S22795[79], 5}, {NULL}};
const Segment SGI424[] = {{&S22799[38], 10}, {NULL}};
const Segment SGI425[] = {{&S22799[31], 6}, {NULL}};
const Segment SGI426[] = {{&S22800[44], 11}, {NULL}};
const Segment SGI427[] = {{&S22800[44], 22}, {NULL}};
const Segment SGI428[] = {{&S22800[30], 6}, {NULL}};
const Segment SGI429[] = {{&S22800[114], 3}, {NULL}};
const Segment SGI430[] = {{&S22800[80], 5}, {NULL}};
const Segment SGI431[] = {{&S22803[44], 10}, {NULL}};
const Segment SGI432[] = {{&S22803[44], 21}, {NULL}};
const Segment SGI433[] = {{&S22803[30], 6}, {NULL}};
const Segment SGI434[] = {{&S22803[82], 4}, {NULL}};
const Segment SGI435[] = {{&S22803[82], 3}, {NULL}};
const Segment SGI436[] = {{&S22806[44], 10}, {NULL}};
const Segment SGI437[] = {{&S22806[44], 21}, {NULL}};
const Segment SGI438[] = {{&S22806[30], 6}, {NULL}};
const Segment SGI439[] = {{&S22806[82], 6}, {NULL}};
const Segment SGI440[] = {{&S22862[44], 10}, {NULL}};
const Segment SGI441[] = {{&S22862[44], 21}, {NULL}};
const Segment SGI442[] = {{&S22862[30], 6}, {NULL}};
const Segment SGI443[] = {{&S22862[113], 3}, {NULL}};
const Segment SGI444[] = {{&S22862[79], 5}, {NULL}};
const Segment SGI445[] = {{&S22863[44], 11}, {NULL}};
const Segment SGI446[] = {{&S22863[44], 22}, {NULL}};
const Segment SGI447[] = {{&S22863[30], 6}, {NULL}};
const Segment SGI448[] = {{&S22863[114], 3}, {NULL}};
const Segment SGI449[] = {{&S22863[80], 5}, {NULL}};
const Segment SGI450[] = {{&S22864[44], 11}, {NULL}};
const Segment SGI451[] = {{&S22864[44], 22}, {NULL}};
const Segment SGI452[] = {{&S22864[30], 6}, {NULL}};
const Segment SGI453[] = {{&S22864[114], 3}, {NULL}};
const Segment SGI454[] = {{&S22864[80], 5}, {NULL}};
const Segment SGI455[] = {{&S22879[38], 10}, {NULL}};
const Segment SGI456[] = {{&S22879[31], 6}, {NULL}};
const Segment SGI457[] = {{&S22883[44], 10}, {NULL}};
const Segment SGI458[] = {{&S22883[44], 21}, {NULL}};
const Segment SGI459[] = {{&S22883[30], 6}, {NULL}};
const Segment SGI460[] = {{&S22883[82], 4}, {NULL}};
const Segment SGI461[] = {{&S22883[82], 3}, {NULL}};
const Segment SGI462[] = {{&S22887[38], 11}, {NULL}};
const Segment SGI463[] = {{&S22887[31], 6}, {NULL}};
const Segment SGI464[] = {{&S22889[44], 10}, {NULL}};
const Segment SGI465[] = {{&S22889[44], 21}, {NULL}};
const Segment SGI466[] = {{&S22889[30], 6}, {NULL}};
const Segment SGI467[] = {{&S22889[82], 6}, {NULL}};
const Segment SGI468[] = {{&S22894[38], 11}, {NULL}};
const Segment SGI469[] = {{&S22894[31], 6}, {NULL}};
const Segment SGI470[] = {{&S22908[44], 21}, {NULL}};
const Segment SGI471[] = {{&S22908[30], 6}, {NULL}};
const Segment SGI472[] = {{&S22912[44], 10}, {NULL}};
const Segment SGI473[] = {{&S22912[44], 21}, {NULL}};
const Segment SGI474[] = {{&S22912[30], 6}, {NULL}};
const Segment SGI475[] = {{&S22912[124], 3}, {NULL}};
const Segment SGI476[] = {{&S22912[89], 6}, {NULL}};
const Segment SGI477[] = {{&S22912[144], 8}, {NULL}};
const Segment SGI478[] = {{&S22917[44], 11}, {NULL}};
const Segment SGI479[] = {{&S22917[44], 22}, {NULL}};
const Segment SGI480[] = {{&S22917[30], 6}, {NULL}};
const Segment SGI481[] = {{&S22917[116], 3}, {NULL}};
const Segment SGI482[] = {{&S22917[80], 7}, {NULL}};
const Segment SGI483[] = {{&S22917[136], 8}, {NULL}};
const Segment SGI484[] = {{&S22923[42], 11}, {NULL}};
const Segment SGI485[] = {{&S22923[30], 6}, {NULL}};
const Segment SGI486[] = {{&S22923[112], 3}, {NULL}};
const Segment SGI487[] = {{&S22923[78], 5}, {NULL}};
const Segment SGI488[] = {{&S22930[45], 11}, {NULL}};
const Segment SGI489[] = {{&S22930[45], 28}, {NULL}};
const Segment SGI490[] = {{&S22930[31], 6}, {NULL}};
const Segment SGI491[] = {{&S22937[42], 10}, {NULL}};
const Segment SGI492[] = {{&S22937[30], 6}, {NULL}};
const Segment SGI493[] = {{&S22937[111], 3}, {NULL}};
const Segment SGI494[] = {{&S22937[77], 5}, {NULL}};
const Segment SGI495[] = {{&S22941[44], 10}, {NULL}};
const Segment SGI496[] = {{&S22941[30], 6}, {NULL}};
const Segment SGI497[] = {{&S22950[44], 18}, {NULL}};
const Segment SGI498[] = {{&S22950[44], 29}, {NULL}};
const Segment SGI499[] = {{&S22950[30], 6}, {NULL}};
const Segment SGI500[] = {{&S22950[121], 3}, {NULL}};
const Segment SGI501[] = {{&S22950[87], 5}, {NULL}};
const Segment SGI502[] = {{&S22958[44], 10}, {NULL}};
const Segment SGI503[] = {{&S22958[44], 21}, {NULL}};
const Segment SGI504[] = {{&S22958[30], 6}, {NULL}};
const Segment SGI505[] = {{&S22958[115], 3}, {NULL}};
const Segment SGI506[] = {{&S22958[79], 7}, {NULL}};
const Segment SGI507[] = {{&S22958[135], 8}, {NULL}};
const Segment SGI508[] = {{&S22965[46], 10}, {NULL}};
const Segment SGI509[] = {{&S22965[46], 27}, {NULL}};
const Segment SGI510[] = {{&S22965[32], 6}, {NULL}};
const Segment SGI511[] = {{&S22969[44], 10}, {NULL}};
const Segment SGI512[] = {{&S22969[44], 21}, {NULL}};
const Segment SGI513[] = {{&S22969[30], 6}, {NULL}};
const Segment SGI514[] = {{&S22969[126], 3}, {NULL}};
const Segment SGI515[] = {{&S22969[79], 6}, {NULL}};
const Segment SGI516[] = {{&S22987[39], 9}, {NULL}};
const Segment SGI517[] = {{&S22987[94], 3}, {NULL}};
const Segment SGI518[] = {{&S22987[71], 6}, {NULL}};
const Segment SGI519[] = {{&S22784[42], 21}, {NULL}};
const Segment SGI520[] = {{&S22784[42], 32}, {NULL}};
const Segment SGI521[] = {{&S22784[30], 4}, {NULL}};
const Segment SGI522[] = {{&S22784[122], 3}, {NULL}};
const Segment SGI523[] = {{&S22784[88], 5}, {NULL}};
const Segment SGI524[] = {{&S22785[42], 10}, {NULL}};
const Segment SGI525[] = {{&S22785[42], 21}, {NULL}};
const Segment SGI526[] = {{&S22785[30], 4}, {NULL}};
const Segment SGI527[] = {{&S22785[112], 3}, {NULL}};
const Segment SGI528[] = {{&S22785[77], 6}, {NULL}};
const Segment SGI529[] = {{&S22788[42], 18}, {NULL}};
const Segment SGI530[] = {{&S22788[42], 29}, {NULL}};
const Segment SGI531[] = {{&S22788[30], 4}, {NULL}};
const Segment SGI532[] = {{&S22788[119], 3}, {NULL}};
const Segment SGI533[] = {{&S22788[85], 5}, {NULL}};
const Segment SGI534[] = {{&S22791[42], 11}, {NULL}};
const Segment SGI535[] = {{&S22791[42], 22}, {NULL}};
const Segment SGI536[] = {{&S22791[30], 4}, {NULL}};
const Segment SGI537[] = {{&S22791[112], 3}, {NULL}};
const Segment SGI538[] = {{&S22791[78], 5}, {NULL}};
const Segment SGI539[] = {{&S22793[40], 10}, {NULL}};
const Segment SGI540[] = {{&S22793[30], 4}, {NULL}};
const Segment SGI541[] = {{&S22793[109], 3}, {NULL}};
const Segment SGI542[] = {{&S22793[75], 5}, {NULL}};
const Segment SGI543[] = {{&S22794[42], 10}, {NULL}};
const Segment SGI544[] = {{&S22794[42], 21}, {NULL}};
const Segment SGI545[] = {{&S22794[30], 4}, {NULL}};
const Segment SGI546[] = {{&S22794[111], 3}, {NULL}};
const Segment SGI547[] = {{&S22794[77], 5}, {NULL}};
const Segment SGI548[] = {{&S22796[42], 10}, {NULL}};
const Segment SGI549[] = {{&S22796[42], 21}, {NULL}};
const Segment SGI550[] = {{&S22796[30], 4}, {NULL}};
const Segment SGI551[] = {{&S22796[111], 3}, {NULL}};
const Segment SGI552[] = {{&S22796[77], 5}, {NULL}};
const Segment SGI553[] = {{&S22797[36], 10}, {NULL}};
const Segment SGI554[] = {{&S22797[31], 4}, {NULL}};
const Segment SGI555[] = {{&S22798[42], 11}, {NULL}};
const Segment SGI556[] = {{&S22798[42], 22}, {NULL}};
const Segment SGI557[] = {{&S22798[30], 4}, {NULL}};
const Segment SGI558[] = {{&S22798[112], 3}, {NULL}};
const Segment SGI559[] = {{&S22798[78], 5}, {NULL}};
const Segment SGI560[] = {{&S22801[42], 10}, {NULL}};
const Segment SGI561[] = {{&S22801[42], 22}, {NULL}};
const Segment SGI562[] = {{&S22801[30], 4}, {NULL}};
const Segment SGI563[] = {{&S22801[112], 3}, {NULL}};
const Segment SGI564[] = {{&S22801[78], 5}, {NULL}};
const Segment SGI565[] = {{&S22802[36], 11}, {NULL}};
const Segment SGI566[] = {{&S22802[31], 4}, {NULL}};
const Segment SGI567[] = {{&S22804[42], 10}, {NULL}};
const Segment SGI568[] = {{&S22804[42], 21}, {NULL}};
const Segment SGI569[] = {{&S22804[30], 4}, {NULL}};
const Segment SGI570[] = {{&S22804[80], 4}, {NULL}};
const Segment SGI571[] = {{&S22804[80], 3}, {NULL}};
const Segment SGI572[] = {{&S22805[36], 11}, {NULL}};
const Segment SGI573[] = {{&S22805[31], 4}, {NULL}};
const Segment SGI574[] = {{&S22807[42], 10}, {NULL}};
const Segment SGI575[] = {{&S22807[42], 21}, {NULL}};
const Segment SGI576[] = {{&S22807[30], 4}, {NULL}};
const Segment SGI577[] = {{&S22807[80], 6}, {NULL}};
const Segment SGI578[] = {{&S22865[37], 8}, {NULL}};
const Segment SGI579[] = {{&S22865[91], 3}, {NULL}};
const Segment SGI580[] = {{&S22865[68], 6}, {NULL}};
const Segment SGI581[] = {{&S22866[42], 11}, {NULL}};
const Segment SGI582[] = {{&S22866[42], 22}, {NULL}};
const Segment SGI583[] = {{&S22866[30], 4}, {NULL}};
const Segment SGI584[] = {{&S22866[113], 3}, {NULL}};
const Segment SGI585[] = {{&S22866[78], 6}, {NULL}};
const Segment SGI586[] = {{&S22867[42], 11}, {NULL}};
const Segment SGI587[] = {{&S22867[42], 22}, {NULL}};
const Segment SGI588[] = {{&S22867[30], 4}, {NULL}};
const Segment SGI589[] = {{&S22867[112], 3}, {NULL}};
const Segment SGI590[] = {{&S22867[78], 5}, {NULL}};
const Segment SGI591[] = {{&S22868[42], 11}, {NULL}};
const Segment SGI592[] = {{&S22868[42], 22}, {NULL}};
const Segment SGI593[] = {{&S22868[30], 4}, {NULL}};
const Segment SGI594[] = {{&S22868[112], 3}, {NULL}};
const Segment SGI595[] = {{&S22868[78], 5}, {NULL}};
const Segment SGI596[] = {{&S22869[42], 11}, {NULL}};
const Segment SGI597[] = {{&S22869[42], 22}, {NULL}};
const Segment SGI598[] = {{&S22869[30], 4}, {NULL}};
const Segment SGI599[] = {{&S22869[112], 3}, {NULL}};
const Segment SGI600[] = {{&S22869[78], 5}, {NULL}};
const Segment SGI601[] = {{&S22870[42], 10}, {NULL}};
const Segment SGI602[] = {{&S22870[42], 21}, {NULL}};
const Segment SGI603[] = {{&S22870[30], 4}, {NULL}};
const Segment SGI604[] = {{&S22870[111], 3}, {NULL}};
const Segment SGI605[] = {{&S22870[77], 5}, {NULL}};
const Segment SGI606[] = {{&S22871[42], 11}, {NULL}};
const Segment SGI607[] = {{&S22871[42], 22}, {NULL}};
const Segment SGI608[] = {{&S22871[30], 4}, {NULL}};
const Segment SGI609[] = {{&S22871[112], 3}, {NULL}};
const Segment SGI610[] = {{&S22871[78], 5}, {NULL}};
const Segment SGI611[] = {{&S22872[42], 11}, {NULL}};
const Segment SGI612[] = {{&S22872[42], 22}, {NULL}};
const Segment SGI613[] = {{&S22872[30], 4}, {NULL}};
const Segment SGI614[] = {{&S22872[112], 3}, {NULL}};
const Segment SGI615[] = {{&S22872[78], 5}, {NULL}};
const Segment SGI616[] = {{&S22873[42], 17}, {NULL}};
const Segment SGI617[] = {{&S22873[42], 28}, {NULL}};
const Segment SGI618[] = {{&S22873[30], 4}, {NULL}};
const Segment SGI619[] = {{&S22873[118], 3}, {NULL}};
const Segment SGI620[] = {{&S22873[84], 5}, {NULL}};
const Segment SGI621[] = {{&S22874[37], 8}, {NULL}};
const Segment SGI622[] = {{&S22874[37], 19}, {NULL}};
const Segment SGI623[] = {{&S22874[99], 3}, {NULL}};
const Segment SGI624[] = {{&S22874[69], 5}, {NULL}};
const Segment SGI625[] = {{&S22875[36], 10}, {NULL}};
const Segment SGI626[] = {{&S22875[31], 4}, {NULL}};
const Segment SGI627[] = {{&S22876[36], 11}, {NULL}};
const Segment SGI628[] = {{&S22876[31], 4}, {NULL}};
const Segment SGI629[] = {{&S22877[41], 10}, {NULL}};
const Segment SGI630[] = {{&S22877[41], 21}, {NULL}};
const Segment SGI631[] = {{&S22877[110], 3}, {NULL}};
const Segment SGI632[] = {{&S22877[76], 5}, {NULL}};
const Segment SGI633[] = {{&S22878[42], 10}, {NULL}};
const Segment SGI634[] = {{&S22878[42], 27}, {NULL}};
const Segment SGI635[] = {{&S22878[30], 4}, {NULL}};
const Segment SGI636[] = {{&S22880[36], 11}, {NULL}};
const Segment SGI637[] = {{&S22880[31], 4}, {NULL}};
const Segment SGI638[] = {{&S22881[36], 11}, {NULL}};
const Segment SGI639[] = {{&S22881[31], 4}, {NULL}};
const Segment SGI640[] = {{&S22882[36], 11}, {NULL}};
const Segment SGI641[] = {{&S22882[31], 4}, {NULL}};
const Segment SGI642[] = {{&S22884[40], 10}, {NULL}};
const Segment SGI643[] = {{&S22884[30], 4}, {NULL}};
const Segment SGI644[] = {{&S22884[109], 3}, {NULL}};
const Segment SGI645[] = {{&S22884[75], 5}, {NULL}};
const Segment SGI646[] = {{&S22885[36], 11}, {NULL}};
const Segment SGI647[] = {{&S22885[31], 4}, {NULL}};
const Segment SGI648[] = {{&S22886[42], 10}, {NULL}};
const Segment SGI649[] = {{&S22886[42], 21}, {NULL}};
const Segment SGI650[] = {{&S22886[30], 4}, {NULL}};
const Segment SGI651[] = {{&S22886[80], 4}, {NULL}};
const Segment SGI652[] = {{&S22886[80], 3}, {NULL}};
const Segment SGI653[] = {{&S22888[42], 10}, {NULL}};
const Segment SGI654[] = {{&S22888[42], 21}, {NULL}};
const Segment SGI655[] = {{&S22888[30], 4}, {NULL}};
const Segment SGI656[] = {{&S22888[80], 6}, {NULL}};
const Segment SGI657[] = {{&S22890[40], 25}, {NULL}};
const Segment SGI658[] = {{&S22890[30], 4}, {NULL}};
const Segment SGI659[] = {{&S22890[125], 3}, {NULL}};
const Segment SGI660[] = {{&S22890[91], 5}, {NULL}};
const Segment SGI661[] = {{&S22891[42], 10}, {NULL}};
const Segment SGI662[] = {{&S22891[42], 21}, {NULL}};
const Segment SGI663[] = {{&S22891[30], 4}, {NULL}};
const Segment SGI664[] = {{&S22891[112], 3}, {NULL}};
const Segment SGI665[] = {{&S22891[77], 6}, {NULL}};
const Segment SGI666[] = {{&S24475[13], 8}, {NULL}};
const Segment SGI667[] = {{&S24475[22], 12}, {NULL}};
const Segment SGI668[] = {{&S22892[45], 9}, {NULL}};
const Segment SGI669[] = {{&S22892[30], 7}, {NULL}};
const Segment SGI670[] = {{&S22892[104], 3}, {NULL}};
const Segment SGI671[] = {{&S22892[68], 7}, {NULL}};
const Segment SGI672[] = {{&S22892[124], 8}, {NULL}};
const Segment SGI673[] = {{&S22893[107], 3}, {NULL}};
const Segment SGI674[] = {{&S22893[73], 5}, {NULL}};
const Segment SGI675[] = {{&S22895[40], 11}, {NULL}};
const Segment SGI676[] = {{&S22895[30], 4}, {NULL}};
const Segment SGI677[] = {{&S22895[110], 3}, {NULL}};
const Segment SGI678[] = {{&S22895[76], 5}, {NULL}};
const Segment SGI679[] = {{&S22896[42], 25}, {NULL}};
const Segment SGI680[] = {{&S22896[42], 36}, {NULL}};
const Segment SGI681[] = {{&S22896[30], 4}, {NULL}};
const Segment SGI682[] = {{&S22896[126], 3}, {NULL}};
const Segment SGI683[] = {{&S22896[92], 5}, {NULL}};
const Segment SGI684[] = {{&S22897[41], 10}, {NULL}};
const Segment SGI685[] = {{&S22897[41], 21}, {NULL}};
const Segment SGI686[] = {{&S22897[30], 3}, {NULL}};
const Segment SGI687[] = {{&S22897[110], 3}, {NULL}};
const Segment SGI688[] = {{&S22897[76], 5}, {NULL}};
const Segment SGI689[] = {{&S22898[42], 10}, {NULL}};
const Segment SGI690[] = {{&S22898[42], 27}, {NULL}};
const Segment SGI691[] = {{&S22898[30], 4}, {NULL}};
const Segment SGI692[] = {{&S22900[42], 10}, {NULL}};
const Segment SGI693[] = {{&S22900[42], 21}, {NULL}};
const Segment SGI694[] = {{&S22900[30], 4}, {NULL}};
const Segment SGI695[] = {{&S22900[112], 3}, {NULL}};
const Segment SGI696[] = {{&S22900[77], 6}, {NULL}};
const Segment SGI697[] = {{&S22901[40], 11}, {NULL}};
const Segment SGI698[] = {{&S22901[30], 4}, {NULL}};
const Segment SGI699[] = {{&S22901[110], 3}, {NULL}};
const Segment SGI700[] = {{&S22901[76], 5}, {NULL}};
const Segment SGI701[] = {{&S22902[42], 10}, {NULL}};
const Segment SGI702[] = {{&S22902[42], 27}, {NULL}};
const Segment SGI703[] = {{&S22902[30], 4}, {NULL}};
const Segment SGI704[] = {{&S22903[42], 10}, {NULL}};
const Segment SGI705[] = {{&S22903[42], 21}, {NULL}};
const Segment SGI706[] = {{&S22903[30], 4}, {NULL}};
const Segment SGI707[] = {{&S22903[122], 3}, {NULL}};
const Segment SGI708[] = {{&S22903[87], 6}, {NULL}};
const Segment SGI709[] = {{&S22903[142], 8}, {NULL}};
const Segment SGI710[] = {{&S22904[42], 10}, {NULL}};
const Segment SGI711[] = {{&S22904[42], 19}, {NULL}};
const Segment SGI712[] = {{&S22904[30], 4}, {NULL}};
const Segment SGI713[] = {{&S22904[109], 3}, {NULL}};
const Segment SGI714[] = {{&S22904[75], 5}, {NULL}};
const Segment SGI715[] = {{&S22905[42], 10}, {NULL}};
const Segment SGI716[] = {{&S22905[42], 29}, {NULL}};
const Segment SGI717[] = {{&S22905[30], 4}, {NULL}};
const Segment SGI718[] = {{&S22905[119], 3}, {NULL}};
const Segment SGI719[] = {{&S22905[85], 5}, {NULL}};
const Segment SGI720[] = {{&S22907[42], 21}, {NULL}};
const Segment SGI721[] = {{&S22907[30], 4}, {NULL}};
const Segment SGI722[] = {{&S22909[42], 11}, {NULL}};
const Segment SGI723[] = {{&S22909[42], 22}, {NULL}};
const Segment SGI724[] = {{&S22909[30], 4}, {NULL}};
const Segment SGI725[] = {{&S22909[113], 3}, {NULL}};
const Segment SGI726[] = {{&S22909[78], 6}, {NULL}};
const Segment SGI727[] = {{&S22910[42], 18}, {NULL}};
const Segment SGI728[] = {{&S22910[42], 29}, {NULL}};
const Segment SGI729[] = {{&S22910[30], 4}, {NULL}};
const Segment SGI730[] = {{&S22910[119], 3}, {NULL}};
const Segment SGI731[] = {{&S22910[85], 5}, {NULL}};
const Segment SGI732[] = {{&S22911[42], 9}, {NULL}};
const Segment SGI733[] = {{&S22911[30], 4}, {NULL}};
const Segment SGI734[] = {{&S22913[42], 10}, {NULL}};
const Segment SGI735[] = {{&S22913[42], 21}, {&S22913[209], 7}, {NULL}};
const Segment SGI736[] = {{&S22913[30], 4}, {NULL}};
const Segment SGI737[] = {{&S22913[111], 3}, {NULL}};
const Segment SGI738[] = {{&S22913[77], 5}, {NULL}};
const Segment SGI739[] = {{&S22914[42], 10}, {&S22914[42], 10}, {NULL}};
const Segment SGI740[] = {{&S22914[42], 21}, {&S22914[42], 21}, {&S22914[287], 16}, {NULL}};
const Segment SGI741[] = {{&S22914[30], 4}, {&S22914[30], 4}, {NULL}};
const Segment SGI742[] = {{&S24517[50], 10}, {NULL}};
const Segment SGI743[] = {{&S24517[50], 27}, {NULL}};
const Segment SGI744[] = {{&S24517[30], 12}, {NULL}};
const Segment SGI745[] = {{&S24517[125], 3}, {NULL}};
const Segment SGI746[] = {{&S24517[91], 5}, {NULL}};
const Segment SGI747[] = {{&S24517[51], 8}, {NULL}};
const Segment SGI748[] = {{&S22914[77], 2}, {&S22914[77], 2}, {NULL}};
const Segment SGI749[] = {{&S22916[42], 10}, {NULL}};
const Segment SGI750[] = {{&S22916[42], 21}, {NULL}};
const Segment SGI751[] = {{&S22916[30], 4}, {NULL}};
const Segment SGI752[] = {{&S22916[111], 3}, {NULL}};
const Segment SGI753[] = {{&S22916[77], 5}, {NULL}};
const Segment SGI754[] = {{&S22918[42], 10}, {NULL}};
const Segment SGI755[] = {{&S22918[42], 21}, {&S22918[241], 35}, {NULL}};
const Segment SGI756[] = {{&S22918[30], 4}, {NULL}};
const Segment SGI757[] = {{&S22918[111], 3}, {NULL}};
const Segment SGI758[] = {{&S22918[77], 5}, {NULL}};
const Segment SGI759[] = {{&S22919[42], 10}, {NULL}};
const Segment SGI760[] = {{&S22919[42], 27}, {NULL}};
const Segment SGI761[] = {{&S22919[30], 4}, {NULL}};
const Segment SGI762[] = {{&S22920[42], 11}, {NULL}};
const Segment SGI763[] = {{&S22920[42], 22}, {NULL}};
const Segment SGI764[] = {{&S22920[30], 4}, {NULL}};
const Segment SGI765[] = {{&S22920[81], 6}, {NULL}};
const Segment SGI766[] = {{&S22921[43], 11}, {NULL}};
const Segment SGI767[] = {{&S22921[43], 28}, {NULL}};
const Segment SGI768[] = {{&S22921[31], 4}, {NULL}};
const Segment SGI769[] = {{&S22922[42], 10}, {NULL}};
const Segment SGI770[] = {{&S22922[42], 27}, {NULL}};
const Segment SGI771[] = {{&S22922[30], 4}, {NULL}};
const Segment SGI772[] = {{&S22924[43], 11}, {NULL}};
const Segment SGI773[] = {{&S22924[43], 28}, {NULL}};
const Segment SGI774[] = {{&S22924[31], 4}, {NULL}};
const Segment SGI775[] = {{&S24531[55], 8}, {&S24531[55], 8}, {&S24531[55], 8}, {NULL}};
const Segment SGI776[] = {{&S22925[43], 11}, {NULL}};
const Segment SGI777[] = {{&S22925[43], 28}, {NULL}};
const Segment SGI778[] = {{&S22925[31], 4}, {NULL}};
const Segment SGI779[] = {{&S22926[43], 11}, {NULL}};
const Segment SGI780[] = {{&S22926[43], 28}, {NULL}};
const Segment SGI781[] = {{&S22926[31], 4}, {NULL}};
const Segment SGI782[] = {{&S22927[56], 10}, {NULL}};
const Segment SGI783[] = {{&S22927[56], 27}, {NULL}};
const Segment SGI784[] = {{&S22927[44], 4}, {NULL}};
const Segment SGI785[] = {{&S22928[42], 10}, {NULL}};
const Segment SGI786[] = {{&S22928[42], 27}, {NULL}};
const Segment SGI787[] = {{&S22928[30], 4}, {NULL}};
const Segment SGI788[] = {{&S22929[42], 10}, {NULL}};
const Segment SGI789[] = {{&S22929[42], 21}, {NULL}};
const Segment SGI790[] = {{&S22929[30], 4}, {NULL}};
const Segment SGI791[] = {{&S22929[111], 3}, {NULL}};
const Segment SGI792[] = {{&S22929[77], 5}, {NULL}};
const Segment SGI793[] = {{&S22931[43], 11}, {NULL}};
const Segment SGI794[] = {{&S22931[43], 28}, {NULL}};
const Segment SGI795[] = {{&S22931[31], 4}, {NULL}};
const Segment SGI796[] = {{&S22932[43], 11}, {NULL}};
const Segment SGI797[] = {{&S22932[43], 28}, {NULL}};
const Segment SGI798[] = {{&S22932[31], 4}, {NULL}};
const Segment SGI799[] = {{&S22933[43], 11}, {NULL}};
const Segment SGI800[] = {{&S22933[43], 28}, {NULL}};
const Segment SGI801[] = {{&S22933[31], 4}, {NULL}};
const Segment SGI802[] = {{&S22934[40], 11}, {NULL}};
const Segment SGI803[] = {{&S22934[30], 4}, {NULL}};
const Segment SGI804[] = {{&S22934[110], 3}, {NULL}};
const Segment SGI805[] = {{&S22934[76], 5}, {NULL}};
const Segment SGI806[] = {{&S22935[42], 11}, {NULL}};
const Segment SGI807[] = {{&S22935[42], 20}, {NULL}};
const Segment SGI808[] = {{&S22935[30], 4}, {NULL}};
const Segment SGI809[] = {{&S22935[110], 3}, {NULL}};
const Segment SGI810[] = {{&S22935[76], 5}, {NULL}};
const Segment SGI811[] = {{&S22936[40], 11}, {NULL}};
const Segment SGI812[] = {{&S22936[30], 4}, {NULL}};
const Segment SGI813[] = {{&S22936[110], 3}, {NULL}};
const Segment SGI814[] = {{&S22936[76], 5}, {NULL}};
const Segment SGI815[] = {{&S22938[42], 11}, {NULL}};
const Segment SGI816[] = {{&S22938[42], 28}, {NULL}};
const Segment SGI817[] = {{&S22938[30], 4}, {NULL}};
const Segment SGI818[] = {{&S22939[50], 11}, {NULL}};
const Segment SGI819[] = {{&S22939[50], 22}, {NULL}};
const Segment SGI820[] = {{&S22939[38], 4}, {NULL}};
const Segment SGI821[] = {{&S22940[42], 11}, {NULL}};
const Segment SGI822[] = {{&S22940[42], 20}, {NULL}};
const Segment SGI823[] = {{&S22940[30], 4}, {NULL}};
const Segment SGI824[] = {{&S22940[110], 3}, {NULL}};
const Segment SGI825[] = {{&S22940[76], 5}, {NULL}};
const Segment SGI826[] = {{&S22942[42], 11}, {NULL}};
const Segment SGI827[] = {{&S22942[42], 22}, {NULL}};
const Segment SGI828[] = {{&S22942[30], 4}, {NULL}};
const Segment SGI829[] = {{&S22942[114], 3}, {NULL}};
const Segment SGI830[] = {{&S22942[78], 7}, {NULL}};
const Segment SGI831[] = {{&S22942[134], 8}, {NULL}};
const Segment SGI832[] = {{&S24558[23], 8}, {NULL}};
const Segment SGI833[] = {{&S22943[62], 11}, {NULL}};
const Segment SGI834[] = {{&S22943[62], 22}, {NULL}};
const Segment SGI835[] = {{&S22943[50], 4}, {NULL}};
const Segment SGI836[] = {{&S22944[42], 10}, {NULL}};
const Segment SGI837[] = {{&S22944[42], 21}, {NULL}};
const Segment SGI838[] = {{&S22944[30], 4}, {NULL}};
const Segment SGI839[] = {{&S22945[42], 10}, {NULL}};
const Segment SGI840[] = {{&S22945[42], 27}, {NULL}};
const Segment SGI841[] = {{&S22945[30], 4}, {NULL}};
const Segment SGI842[] = {{&S22948[30], 1}, {NULL}};
const Segment SGI843[] = {{&S22948[61], 10}, {NULL}};
const Segment SGI844[] = {{&S22948[49], 4}, {NULL}};
const Segment SGI845[] = {{&S22949[99], 29}, {NULL}};
const Segment SGI846[] = {{&S22951[42], 25}, {NULL}};
const Segment SGI847[] = {{&S22951[42], 37}, {NULL}};
const Segment SGI848[] = {{&S22951[30], 4}, {NULL}};
const Segment SGI849[] = {{&S22952[42], 19}, {NULL}};
const Segment SGI850[] = {{&S22952[42], 30}, {NULL}};
const Segment SGI851[] = {{&S22952[30], 4}, {NULL}};
const Segment SGI852[] = {{&S22952[120], 3}, {NULL}};
const Segment SGI853[] = {{&S22952[86], 5}, {NULL}};
const Segment SGI854[] = {{&S22953[42], 10}, {NULL}};
const Segment SGI855[] = {{&S22953[42], 21}, {&S22953[271], 7}, {NULL}};
const Segment SGI856[] = {{&S22953[30], 4}, {NULL}};
const Segment SGI857[] = {{&S22953[111], 3}, {NULL}};
const Segment SGI858[] = {{&S22953[77], 5}, {NULL}};
const Segment SGI859[] = {{&S22955[42], 10}, {NULL}};
const Segment SGI860[] = {{&S22955[42], 21}, {&S22955[209], 13}, {NULL}};
const Segment SGI861[] = {{&S22955[30], 4}, {NULL}};
const Segment SGI862[] = {{&S22955[111], 3}, {NULL}};
const Segment SGI863[] = {{&S22955[77], 5}, {NULL}};
const Segment SGI864[] = {{&S22956[42], 10}, {NULL}};
const Segment SGI865[] = {{&S22956[42], 21}, {NULL}};
const Segment SGI866[] = {{&S22956[30], 4}, {NULL}};
const Segment SGI867[] = {{&S22956[113], 3}, {NULL}};
const Segment SGI868[] = {{&S22956[77], 7}, {NULL}};
const Segment SGI869[] = {{&S22956[133], 8}, {NULL}};
const Segment SGI870[] = {{&S22957[42], 19}, {NULL}};
const Segment SGI871[] = {{&S22957[42], 30}, {NULL}};
const Segment SGI872[] = {{&S22957[30], 4}, {NULL}};
const Segment SGI873[] = {{&S22957[120], 3}, {NULL}};
const Segment SGI874[] = {{&S22957[86], 5}, {NULL}};
const Segment SGI875[] = {{&S22959[42], 9}, {NULL}};
const Segment SGI876[] = {{&S22959[42], 20}, {NULL}};
const Segment SGI877[] = {{&S22959[30], 4}, {NULL}};
const Segment SGI878[] = {{&S22959[110], 3}, {NULL}};
const Segment SGI879[] = {{&S22959[76], 5}, {NULL}};
const Segment SGI880[] = {{&S22960[44], 10}, {NULL}};
const Segment SGI881[] = {{&S22960[44], 27}, {NULL}};
const Segment SGI882[] = {{&S22960[32], 4}, {NULL}};
const Segment SGI883[] = {{&S22961[58], 10}, {NULL}};
const Segment SGI884[] = {{&S22961[58], 27}, {NULL}};
const Segment SGI885[] = {{&S22961[46], 4}, {NULL}};
const Segment SGI886[] = {{&S22962[42], 10}, {NULL}};
const Segment SGI887[] = {{&S22962[30], 4}, {NULL}};
const Segment SGI888[] = {{&S22964[42], 10}, {NULL}};
const Segment SGI889[] = {{&S22964[42], 21}, {NULL}};
const Segment SGI890[] = {{&S22964[30], 4}, {NULL}};
const Segment SGI891[] = {{&S22964[111], 3}, {&S22964[111], 3}, {NULL}};
const Segment SGI892[] = {{&S22964[77], 5}, {NULL}};
const Segment SGI893[] = {{&S22967[42], 19}, {NULL}};
const Segment SGI894[] = {{&S22967[42], 30}, {NULL}};
const Segment SGI895[] = {{&S22967[30], 4}, {NULL}};
const Segment SGI896[] = {{&S22967[120], 3}, {NULL}};
const Segment SGI897[] = {{&S22967[86], 5}, {NULL}};
const Segment SGI898[] = {{&S22968[37], 4}, {NULL}};
const Segment SGI899[] = {{&S22970[42], 11}, {NULL}};
const Segment SGI900[] = {{&S22970[42], 22}, {NULL}};
const Segment SGI901[] = {{&S22970[30], 4}, {NULL}};
const Segment SGI902[] = {{&S22970[113], 3}, {NULL}};
const Segment SGI903[] = {{&S22970[78], 6}, {NULL}};
const Segment SGI904[] = {{&S22971[42], 10}, {NULL}};
const Segment SGI905[] = {{&S22971[30], 4}, {NULL}};
const Segment SGI906[] = {{&S22972[42], 10}, {NULL}};
const Segment SGI907[] = {{&S22972[42], 21}, {NULL}};
const Segment SGI908[] = {{&S22972[30], 4}, {NULL}};
const Segment SGI909[] = {{&S22972[111], 3}, {NULL}};
const Segment SGI910[] = {{&S22972[77], 5}, {NULL}};
const Segment SGI911[] = {{&S22972[77], 4}, {NULL}};
const Segment SGI912[] = {{&S22973[42], 10}, {NULL}};
const Segment SGI913[] = {{&S22973[42], 21}, {NULL}};
const Segment SGI914[] = {{&S22973[30], 4}, {NULL}};
const Segment SGI915[] = {{&S22973[111], 3}, {NULL}};
const Segment SGI916[] = {{&S22973[77], 5}, {NULL}};
const Segment SGI917[] = {{&S22973[129], 9}, {NULL}};
const Segment SGI918[] = {{&S22974[50], 11}, {NULL}};
const Segment SGI919[] = {{&S22974[50], 22}, {NULL}};
const Segment SGI920[] = {{&S22974[38], 4}, {NULL}};
const Segment SGI921[] = {{&S22975[42], 10}, {NULL}};
const Segment SGI922[] = {{&S22975[42], 21}, {NULL}};
const Segment SGI923[] = {{&S22975[30], 4}, {NULL}};
const Segment SGI924[] = {{&S22975[77], 5}, {NULL}};
const Segment SGI925[] = {{&S22976[42], 10}, {NULL}};
const Segment SGI926[] = {{&S22976[42], 21}, {NULL}};
const Segment SGI927[] = {{&S22976[30], 4}, {NULL}};
const Segment SGI928[] = {{&S22976[111], 3}, {NULL}};
const Segment SGI929[] = {{&S22976[77], 5}, {NULL}};
const Segment SGI930[] = {{&S22976[129], 7}, {NULL}};
const Segment SGI931[] = {{&S22979[42], 11}, {NULL}};
const Segment SGI932[] = {{&S22979[42], 22}, {NULL}};
const Segment SGI933[] = {{&S22979[30], 4}, {NULL}};
const Segment SGI934[] = {{&S22979[114], 3}, {NULL}};
const Segment SGI935[] = {{&S22979[78], 7}, {NULL}};
const Segment SGI936[] = {{&S22979[134], 8}, {NULL}};
const Segment SGI937[] = {{&S22980[42], 11}, {NULL}};
const Segment SGI938[] = {{&S22980[42], 20}, {NULL}};
const Segment SGI939[] = {{&S22980[30], 4}, {NULL}};
const Segment SGI940[] = {{&S22980[110], 3}, {NULL}};
const Segment SGI941[] = {{&S22980[76], 5}, {NULL}};
const Segment SGI942[] = {{&S22982[59], 10}, {NULL}};
const Segment SGI943[] = {{&S22982[59], 21}, {NULL}};
const Segment SGI944[] = {{&S22982[47], 4}, {NULL}};
const Segment SGI945[] = {{&S22982[11], 3}, {NULL}};
const Segment SGI946[] = {{&S22982[94], 5}, {NULL}};
const Segment SGI947[] = {{&S22982[0], 14}, {NULL}};
const Segment SGI948[] = {{&S22983[42], 11}, {NULL}};
const Segment SGI949[] = {{&S22983[42], 22}, {NULL}};
const Segment SGI950[] = {{&S22983[30], 4}, {NULL}};
const Segment SGI951[] = {{&S22983[114], 3}, {NULL}};
const Segment SGI952[] = {{&S22983[78], 7}, {NULL}};
const Segment SGI953[] = {{&S22983[134], 8}, {NULL}};
const Segment SGI954[] = {{&S22984[42], 9}, {NULL}};
const Segment SGI955[] = {{&S22984[42], 20}, {NULL}};
const Segment SGI956[] = {{&S22984[30], 4}, {NULL}};
const Segment SGI957[] = {{&S22984[110], 3}, {NULL}};
const Segment SGI958[] = {{&S22984[76], 5}, {NULL}};
const Segment SGI959[] = {{&S22986[42], 10}, {NULL}};
const Segment SGI960[] = {{&S22986[42], 21}, {NULL}};
const Segment SGI961[] = {{&S22986[30], 4}, {NULL}};
const Segment SGI962[] = {{&S22986[111], 3}, {NULL}};
const Segment SGI963[] = {{&S22986[73], 5}, {NULL}};
const Segment SGI964[] = {{&S22988[40], 11}, {NULL}};
const Segment SGI965[] = {{&S22988[30], 4}, {NULL}};
const Segment SGI966[] = {{&S22988[112], 3}, {NULL}};
const Segment SGI967[] = {{&S22988[76], 7}, {NULL}};
const Segment SGI968[] = {{&S22988[132], 8}, {NULL}};
const Segment SGI969[] = {{&S22989[42], 25}, {NULL}};
const Segment SGI970[] = {{&S22989[42], 37}, {NULL}};
const Segment SGI971[] = {{&S22989[30], 4}, {NULL}};
const Segment SGI972[] = {{&S22989[127], 3}, {NULL}};
const Segment SGI973[] = {{&S22989[93], 5}, {NULL}};
const Segment SGI974[] = {{&S22990[36], 16}, {NULL}};
const Segment SGI975[] = {{&S22990[25], 4}, {NULL}};
const Segment SGI976[] = {{&S25554[6], 4}, {NULL}};
const Segment SGI977[] = {{&S25554[32], 3}, {NULL}};
const Segment SGI978[] = {{&S25554[81], 5}, {NULL}};
const Segment SGI979[] = {{&S25555[6], 4}, {NULL}};
const Segment SGI980[] = {{&S25555[32], 3}, {NULL}};
const Segment SGI981[] = {{&S25555[81], 5}, {NULL}};
const Segment SGI982[] = {{&S24848[44], 9}, {NULL}};
const Segment SGI983[] = {{&S24848[44], 20}, {NULL}};
const Segment SGI984[] = {{&S24848[30], 6}, {NULL}};
const Segment SGI985[] = {{&S24848[78], 6}, {NULL}};
const Segment SGI986[] = {{&S24848[134], 6}, {NULL}};
const Segment SGI987[] = {{&S24851[44], 14}, {NULL}};
const Segment SGI988[] = {{&S24851[44], 25}, {NULL}};
const Segment SGI989[] = {{&S24851[30], 6}, {NULL}};
const Segment SGI990[] = {{&S24851[117], 3}, {NULL}};
const Segment SGI991[] = {{&S24851[83], 5}, {NULL}};
const Segment SGI992[] = {{&S24852[44], 14}, {NULL}};
const Segment SGI993[] = {{&S24852[44], 25}, {NULL}};
const Segment SGI994[] = {{&S24852[30], 6}, {NULL}};
const Segment SGI995[] = {{&S24852[117], 3}, {NULL}};
const Segment SGI996[] = {{&S24852[83], 5}, {NULL}};
const Segment SGI997[] = {{&S24853[44], 10}, {NULL}};
const Segment SGI998[] = {{&S24853[44], 21}, {NULL}};
const Segment SGI999[] = {{&S24853[30], 6}, {NULL}};
const Segment SGI1000[] = {{&S24853[113], 3}, {NULL}};
const Segment SGI1001[] = {{&S24853[79], 5}, {NULL}};
const Segment SGI1002[] = {{&S24854[42], 10}, {NULL}};
const Segment SGI1003[] = {{&S24854[30], 6}, {NULL}};
const Segment SGI1004[] = {{&S24854[111], 3}, {NULL}};
const Segment SGI1005[] = {{&S24854[77], 5}, {NULL}};
const Segment SGI1006[] = {{&S24855[44], 17}, {NULL}};
const Segment SGI1007[] = {{&S24855[44], 28}, {NULL}};
const Segment SGI1008[] = {{&S24855[30], 6}, {NULL}};
const Segment SGI1009[] = {{&S24855[120], 3}, {NULL}};
const Segment SGI1010[] = {{&S24855[86], 5}, {NULL}};
const Segment SGI1011[] = {{&S24861[44], 9}, {NULL}};
const Segment SGI1012[] = {{&S24861[44], 20}, {NULL}};
const Segment SGI1013[] = {{&S24861[30], 6}, {NULL}};
const Segment SGI1014[] = {{&S24861[113], 3}, {NULL}};
const Segment SGI1015[] = {{&S24861[78], 6}, {NULL}};
const Segment SGI1016[] = {{&S24861[133], 8}, {NULL}};
const Segment SGI1017[] = {{&S24863[42], 9}, {NULL}};
const Segment SGI1018[] = {{&S24863[30], 6}, {NULL}};
const Segment SGI1019[] = {{&S24863[110], 3}, {NULL}};
const Segment SGI1020[] = {{&S24863[76], 5}, {NULL}};
const Segment SGI1021[] = {{&S24868[42], 17}, {NULL}};
const Segment SGI1022[] = {{&S24868[30], 6}, {NULL}};
const Segment SGI1023[] = {{&S24868[118], 3}, {NULL}};
const Segment SGI1024[] = {{&S24868[84], 5}, {NULL}};
const Segment SGI1025[] = {{&S24874[44], 9}, {NULL}};
const Segment SGI1026[] = {{&S24874[44], 20}, {NULL}};
const Segment SGI1027[] = {{&S24874[30], 6}, {NULL}};
const Segment SGI1028[] = {{&S24874[112], 3}, {NULL}};
const Segment SGI1029[] = {{&S24874[78], 5}, {NULL}};
const Segment SGI1030[] = {{&S24876[44], 10}, {NULL}};
const Segment SGI1031[] = {{&S24876[44], 21}, {NULL}};
const Segment SGI1032[] = {{&S24876[30], 6}, {NULL}};
const Segment SGI1033[] = {{&S24876[115], 3}, {NULL}};
const Segment SGI1034[] = {{&S24876[79], 7}, {NULL}};
const Segment SGI1035[] = {{&S24876[135], 8}, {NULL}};
const Segment SGI1036[] = {{&S24878[38], 9}, {NULL}};
const Segment SGI1037[] = {{&S24878[31], 6}, {NULL}};
const Segment SGI1038[] = {{&S24881[38], 10}, {NULL}};
const Segment SGI1039[] = {{&S24881[31], 6}, {NULL}};
const Segment SGI1040[] = {{&S24883[44], 9}, {NULL}};
const Segment SGI1041[] = {{&S24883[44], 20}, {NULL}};
const Segment SGI1042[] = {{&S24883[30], 6}, {NULL}};
const Segment SGI1043[] = {{&S24883[81], 4}, {NULL}};
const Segment SGI1044[] = {{&S24883[81], 3}, {NULL}};
const Segment SGI1045[] = {{&S24857[42], 14}, {NULL}};
const Segment SGI1046[] = {{&S24857[42], 25}, {NULL}};
const Segment SGI1047[] = {{&S24857[30], 4}, {NULL}};
const Segment SGI1048[] = {{&S24857[115], 3}, {NULL}};
const Segment SGI1049[] = {{&S24857[81], 5}, {NULL}};
const Segment SGI1050[] = {{&S24858[42], 9}, {NULL}};
const Segment SGI1051[] = {{&S24858[42], 20}, {NULL}};
const Segment SGI1052[] = {{&S24858[30], 4}, {NULL}};
const Segment SGI1053[] = {{&S24858[111], 3}, {NULL}};
const Segment SGI1054[] = {{&S24858[76], 6}, {NULL}};
const Segment SGI1055[] = {{&S24858[131], 8}, {NULL}};
const Segment SGI1056[] = {{&S24859[42], 18}, {NULL}};
const Segment SGI1057[] = {{&S24859[42], 29}, {NULL}};
const Segment SGI1058[] = {{&S24859[30], 4}, {NULL}};
const Segment SGI1059[] = {{&S24859[119], 3}, {NULL}};
const Segment SGI1060[] = {{&S24859[85], 5}, {NULL}};
const Segment SGI1061[] = {{&S24860[42], 14}, {NULL}};
const Segment SGI1062[] = {{&S24860[42], 25}, {NULL}};
const Segment SGI1063[] = {{&S24860[30], 4}, {NULL}};
const Segment SGI1064[] = {{&S24860[115], 3}, {NULL}};
const Segment SGI1065[] = {{&S24860[81], 5}, {NULL}};
const Segment SGI1066[] = {{&S24862[37], 7}, {NULL}};
const Segment SGI1067[] = {{&S24862[89], 3}, {NULL}};
const Segment SGI1068[] = {{&S24862[66], 6}, {NULL}};
const Segment SGI1069[] = {{&S24869[42], 17}, {NULL}};
const Segment SGI1070[] = {{&S24869[42], 28}, {NULL}};
const Segment SGI1071[] = {{&S24869[30], 4}, {NULL}};
const Segment SGI1072[] = {{&S24869[118], 3}, {NULL}};
const Segment SGI1073[] = {{&S24869[84], 5}, {NULL}};
const Segment SGI1074[] = {{&S24870[40], 10}, {NULL}};
const Segment SGI1075[] = {{&S24870[30], 4}, {NULL}};
const Segment SGI1076[] = {{&S24870[109], 3}, {NULL}};
const Segment SGI1077[] = {{&S24870[75], 5}, {NULL}};
const Segment SGI1078[] = {{&S24871[42], 10}, {NULL}};
const Segment SGI1079[] = {{&S24871[42], 21}, {NULL}};
const Segment SGI1080[] = {{&S24871[30], 4}, {NULL}};
const Segment SGI1081[] = {{&S24871[111], 3}, {NULL}};
const Segment SGI1082[] = {{&S24871[77], 5}, {NULL}};
const Segment SGI1083[] = {{&S24872[42], 21}, {NULL}};
const Segment SGI1084[] = {{&S24872[42], 32}, {NULL}};
const Segment SGI1085[] = {{&S24872[30], 4}, {NULL}};
const Segment SGI1086[] = {{&S24872[122], 3}, {NULL}};
const Segment SGI1087[] = {{&S24872[88], 5}, {NULL}};
const Segment SGI1088[] = {{&S24875[42], 9}, {NULL}};
const Segment SGI1089[] = {{&S24875[42], 20}, {NULL}};
const Segment SGI1090[] = {{&S24875[30], 4}, {NULL}};
const Segment SGI1091[] = {{&S24875[110], 3}, {NULL}};
const Segment SGI1092[] = {{&S24875[76], 5}, {NULL}};
const Segment SGI1093[] = {{&S24877[36], 9}, {NULL}};
const Segment SGI1094[] = {{&S24877[31], 4}, {NULL}};
const Segment SGI1095[] = {{&S24880[36], 10}, {NULL}};
const Segment SGI1096[] = {{&S24880[31], 4}, {NULL}};
const Segment SGI1097[] = {{&S26910[49], 8}, {NULL}};
const Segment SGI1098[] = {{&S26910[49], 25}, {NULL}};
const Segment SGI1099[] = {{&S26910[30], 11}, {NULL}};
const Segment SGI1100[] = {{&S24884[41], 9}, {NULL}};
const Segment SGI1101[] = {{&S26910[122], 3}, {NULL}};
const Segment SGI1102[] = {{&S24884[41], 20}, {NULL}};
const Segment SGI1103[] = {{&S26910[88], 5}, {NULL}};
const Segment SGI1104[] = {{&S24884[29], 4}, {NULL}};
const Segment SGI1105[] = {{&S24884[109], 3}, {NULL}};
const Segment SGI1106[] = {{&S24884[75], 5}, {NULL}};
const Segment SGI1107[] = {{&S24884[75], 4}, {NULL}};
const Segment SGI1108[] = {{&S27158[72], 10}, {NULL}};
const Segment SGI1109[] = {{&S27158[72], 27}, {NULL}};
const Segment SGI1110[] = {{&S27158[30], 34}, {NULL}};
const Segment SGI1111[] = {{&S27158[147], 3}, {NULL}};
const Segment SGI1112[] = {{&S27158[113], 5}, {NULL}};
const Segment SGI1113[] = {{&S27232[22], 12}, {NULL}};
const Segment SGI1114[] = {{&S26112[44], 11}, {NULL}};
const Segment SGI1115[] = {{&S26112[44], 22}, {NULL}};
const Segment SGI1116[] = {{&S26112[30], 6}, {NULL}};
const Segment SGI1117[] = {{&S26112[114], 3}, {NULL}};
const Segment SGI1118[] = {{&S26112[80], 5}, {NULL}};
const Segment SGI1119[] = {{&S26113[44], 28}, {NULL}};
const Segment SGI1120[] = {{&S26113[44], 39}, {NULL}};
const Segment SGI1121[] = {{&S26113[30], 6}, {NULL}};
const Segment SGI1122[] = {{&S26113[131], 3}, {NULL}};
const Segment SGI1123[] = {{&S26113[97], 5}, {NULL}};
const Segment SGI1124[] = {{&S26114[44], 28}, {NULL}};
const Segment SGI1125[] = {{&S26114[44], 39}, {NULL}};
const Segment SGI1126[] = {{&S26114[30], 6}, {NULL}};
const Segment SGI1127[] = {{&S26114[131], 3}, {NULL}};
const Segment SGI1128[] = {{&S26114[97], 5}, {NULL}};
const Segment SGI1129[] = {{&S26115[44], 28}, {NULL}};
const Segment SGI1130[] = {{&S26115[44], 39}, {NULL}};
const Segment SGI1131[] = {{&S26115[30], 6}, {NULL}};
const Segment SGI1132[] = {{&S26115[131], 3}, {NULL}};
const Segment SGI1133[] = {{&S26115[97], 5}, {NULL}};
const Segment SGI1134[] = {{&S26117[44], 28}, {NULL}};
const Segment SGI1135[] = {{&S26117[44], 39}, {NULL}};
const Segment SGI1136[] = {{&S26117[30], 6}, {NULL}};
const Segment SGI1137[] = {{&S26117[131], 3}, {NULL}};
const Segment SGI1138[] = {{&S26117[97], 5}, {NULL}};
const Segment SGI1139[] = {{&S26118[44], 11}, {NULL}};
const Segment SGI1140[] = {{&S26118[44], 22}, {NULL}};
const Segment SGI1141[] = {{&S26118[30], 6}, {NULL}};
const Segment SGI1142[] = {{&S26118[114], 3}, {NULL}};
const Segment SGI1143[] = {{&S26118[80], 5}, {NULL}};
const Segment SGI1144[] = {{&S26119[44], 10}, {NULL}};
const Segment SGI1145[] = {{&S26119[44], 21}, {NULL}};
const Segment SGI1146[] = {{&S26119[30], 6}, {NULL}};
const Segment SGI1147[] = {{&S26119[113], 3}, {NULL}};
const Segment SGI1148[] = {{&S26119[79], 5}, {NULL}};
const Segment SGI1149[] = {{&S26120[38], 10}, {NULL}};
const Segment SGI1150[] = {{&S26120[31], 6}, {NULL}};
const Segment SGI1151[] = {{&S26121[44], 10}, {NULL}};
const Segment SGI1152[] = {{&S26121[44], 21}, {NULL}};
const Segment SGI1153[] = {{&S26121[30], 6}, {NULL}};
const Segment SGI1154[] = {{&S26121[82], 4}, {NULL}};
const Segment SGI1155[] = {{&S26121[82], 3}, {NULL}};
const Segment SGI1156[] = {{&S26122[38], 11}, {NULL}};
const Segment SGI1157[] = {{&S26122[31], 6}, {NULL}};
const Segment SGI1158[] = {{&S26123[38], 11}, {NULL}};
const Segment SGI1159[] = {{&S26123[31], 6}, {NULL}};
const Segment SGI1160[] = {{&S26124[39], 8}, {NULL}};
const Segment SGI1161[] = {{&S26124[92], 3}, {NULL}};
const Segment SGI1162[] = {{&S26124[69], 6}, {NULL}};
const Segment SGI1163[] = {{&S26125[42], 10}, {NULL}};
const Segment SGI1164[] = {{&S26125[30], 6}, {NULL}};
const Segment SGI1165[] = {{&S26125[111], 3}, {NULL}};
const Segment SGI1166[] = {{&S26125[77], 5}, {NULL}};
const Segment SGI1167[] = {{&S26127[44], 10}, {NULL}};
const Segment SGI1168[] = {{&S26127[44], 21}, {NULL}};
const Segment SGI1169[] = {{&S26127[30], 6}, {NULL}};
const Segment SGI1170[] = {{&S26127[82], 6}, {NULL}};
const Segment SGI1171[] = {{&S27786[56], 12}, {NULL}};
const Segment SGI1172[] = {{&S27786[56], 38}, {NULL}};
const Segment SGI1173[] = {{&S27786[30], 18}, {NULL}};
const Segment SGI1174[] = {{&S27786[160], 3}, {NULL}};
const Segment SGI1175[] = {{&S27786[126], 5}, {NULL}};
const Segment SGI1176[] = {{&S27326[44], 7}, {NULL}};
const Segment SGI1177[] = {{&S27326[44], 28}, {NULL}};
const Segment SGI1178[] = {{&S27326[30], 6}, {NULL}};
const Segment SGI1179[] = {{&S27326[120], 3}, {NULL}};
const Segment SGI1180[] = {{&S27326[86], 5}, {NULL}};
const Segment SGI1181[] = {{&S27327[44], 7}, {NULL}};
const Segment SGI1182[] = {{&S27327[44], 24}, {NULL}};
const Segment SGI1183[] = {{&S27327[30], 6}, {NULL}};
const Segment SGI1184[] = {{&S27327[116], 3}, {NULL}};
const Segment SGI1185[] = {{&S27327[82], 5}, {NULL}};
const Segment SGI1186[] = {{&S27328[44], 7}, {NULL}};
const Segment SGI1187[] = {{&S27328[44], 28}, {NULL}};
const Segment SGI1188[] = {{&S27328[30], 6}, {NULL}};
const Segment SGI1189[] = {{&S27328[120], 3}, {NULL}};
const Segment SGI1190[] = {{&S27328[86], 5}, {NULL}};
const Segment SGI1191[] = {{&S27329[44], 7}, {NULL}};
const Segment SGI1192[] = {{&S27329[44], 28}, {NULL}};
const Segment SGI1193[] = {{&S27329[30], 6}, {NULL}};
const Segment SGI1194[] = {{&S27329[120], 3}, {NULL}};
const Segment SGI1195[] = {{&S27329[86], 5}, {NULL}};
const Segment SGI1196[] = {{&S27331[43], 7}, {NULL}};
const Segment SGI1197[] = {{&S27331[43], 28}, {NULL}};
const Segment SGI1198[] = {{&S27331[119], 3}, {NULL}};
const Segment SGI1199[] = {{&S27331[85], 5}, {NULL}};
const Segment SGI1200[] = {{&S27332[44], 7}, {NULL}};
const Segment SGI1201[] = {{&S27332[44], 21}, {NULL}};
const Segment SGI1202[] = {{&S27332[30], 6}, {NULL}};
const Segment SGI1203[] = {{&S27332[113], 3}, {NULL}};
const Segment SGI1204[] = {{&S27332[79], 5}, {NULL}};
const Segment SGI1205[] = {{&S27333[44], 7}, {NULL}};
const Segment SGI1206[] = {{&S27333[44], 24}, {NULL}};
const Segment SGI1207[] = {{&S27333[30], 6}, {NULL}};
const Segment SGI1208[] = {{&S27333[116], 3}, {NULL}};
const Segment SGI1209[] = {{&S27333[82], 5}, {NULL}};
const Segment SGI1210[] = {{&S27334[44], 7}, {NULL}};
const Segment SGI1211[] = {{&S27334[44], 22}, {NULL}};
const Segment SGI1212[] = {{&S27334[30], 6}, {NULL}};
const Segment SGI1213[] = {{&S27334[114], 3}, {NULL}};
const Segment SGI1214[] = {{&S27334[80], 5}, {NULL}};
const Segment SGI1215[] = {{&S27335[44], 7}, {NULL}};
const Segment SGI1216[] = {{&S27335[44], 21}, {NULL}};
const Segment SGI1217[] = {{&S27335[30], 6}, {NULL}};
const Segment SGI1218[] = {{&S27335[113], 3}, {NULL}};
const Segment SGI1219[] = {{&S27335[79], 5}, {NULL}};
const Segment SGI1220[] = {{&S27336[42], 7}, {NULL}};
const Segment SGI1221[] = {{&S27336[30], 6}, {NULL}};
const Segment SGI1222[] = {{&S27336[112], 3}, {NULL}};
const Segment SGI1223[] = {{&S27336[78], 5}, {NULL}};
const Segment SGI1224[] = {{&S27337[44], 7}, {NULL}};
const Segment SGI1225[] = {{&S27337[44], 22}, {NULL}};
const Segment SGI1226[] = {{&S27337[30], 6}, {NULL}};
const Segment SGI1227[] = {{&S27337[114], 3}, {NULL}};
const Segment SGI1228[] = {{&S27337[80], 5}, {NULL}};
const Segment SGI1229[] = {{&S27338[44], 7}, {NULL}};
const Segment SGI1230[] = {{&S27338[44], 22}, {NULL}};
const Segment SGI1231[] = {{&S27338[30], 6}, {NULL}};
const Segment SGI1232[] = {{&S27338[114], 3}, {NULL}};
const Segment SGI1233[] = {{&S27338[80], 5}, {NULL}};
const Segment SGI1234[] = {{&S27340[94], 7}, {NULL}};
const Segment SGI1235[] = {{&S27340[94], 33}, {NULL}};
const Segment SGI1236[] = {{&S27340[80], 6}, {NULL}};
const Segment SGI1237[] = {{&S27340[175], 3}, {NULL}};
const Segment SGI1238[] = {{&S27340[141], 5}, {NULL}};
const Segment SGI1239[] = {{&S27343[101], 7}, {NULL}};
const Segment SGI1240[] = {{&S27343[101], 29}, {NULL}};
const Segment SGI1241[] = {{&S27343[87], 6}, {NULL}};
const Segment SGI1242[] = {{&S27343[178], 3}, {NULL}};
const Segment SGI1243[] = {{&S27343[144], 5}, {NULL}};
const Segment SGI1244[] = {{&S27344[97], 7}, {NULL}};
const Segment SGI1245[] = {{&S27344[97], 33}, {NULL}};
const Segment SGI1246[] = {{&S27344[83], 6}, {NULL}};
const Segment SGI1247[] = {{&S27344[178], 3}, {NULL}};
const Segment SGI1248[] = {{&S27344[144], 5}, {NULL}};
const Segment SGI1249[] = {{&S27345[44], 7}, {NULL}};
const Segment SGI1250[] = {{&S27345[44], 28}, {NULL}};
const Segment SGI1251[] = {{&S27345[30], 6}, {NULL}};
const Segment SGI1252[] = {{&S27345[120], 3}, {NULL}};
const Segment SGI1253[] = {{&S27345[86], 5}, {NULL}};
const Segment SGI1254[] = {{&S27346[94], 7}, {NULL}};
const Segment SGI1255[] = {{&S27346[94], 33}, {NULL}};
const Segment SGI1256[] = {{&S27346[80], 6}, {NULL}};
const Segment SGI1257[] = {{&S27346[175], 3}, {NULL}};
const Segment SGI1258[] = {{&S27346[141], 5}, {NULL}};
const Segment SGI1259[] = {{&S27349[44], 6}, {NULL}};
const Segment SGI1260[] = {{&S27349[44], 27}, {NULL}};
const Segment SGI1261[] = {{&S27349[30], 6}, {NULL}};
const Segment SGI1262[] = {{&S27349[119], 3}, {NULL}};
const Segment SGI1263[] = {{&S27349[85], 5}, {NULL}};
const Segment SGI1264[] = {{&S27351[43], 7}, {NULL}};
const Segment SGI1265[] = {{&S27351[43], 28}, {NULL}};
const Segment SGI1266[] = {{&S27351[30], 5}, {NULL}};
const Segment SGI1267[] = {{&S27351[119], 3}, {NULL}};
const Segment SGI1268[] = {{&S27351[85], 5}, {NULL}};
const Segment SGI1269[] = {{&S27353[44], 7}, {NULL}};
const Segment SGI1270[] = {{&S27353[44], 28}, {NULL}};
const Segment SGI1271[] = {{&S27353[30], 6}, {NULL}};
const Segment SGI1272[] = {{&S27353[120], 3}, {NULL}};
const Segment SGI1273[] = {{&S27353[86], 5}, {NULL}};
const Segment SGI1274[] = {{&S27354[44], 6}, {NULL}};
const Segment SGI1275[] = {{&S27354[44], 27}, {NULL}};
const Segment SGI1276[] = {{&S27354[30], 6}, {NULL}};
const Segment SGI1277[] = {{&S27354[119], 3}, {NULL}};
const Segment SGI1278[] = {{&S27354[85], 5}, {NULL}};
const Segment SGI1279[] = {{&S27360[94], 7}, {NULL}};
const Segment SGI1280[] = {{&S27360[94], 30}, {NULL}};
const Segment SGI1281[] = {{&S27360[80], 6}, {NULL}};
const Segment SGI1282[] = {{&S27360[172], 3}, {NULL}};
const Segment SGI1283[] = {{&S27360[138], 5}, {NULL}};
const Segment SGI1284[] = {{&S27361[44], 7}, {NULL}};
const Segment SGI1285[] = {{&S27361[44], 20}, {NULL}};
const Segment SGI1286[] = {{&S27361[30], 6}, {NULL}};
const Segment SGI1287[] = {{&S27361[112], 3}, {NULL}};
const Segment SGI1288[] = {{&S27361[78], 5}, {NULL}};
const Segment SGI1289[] = {{&S27364[44], 7}, {NULL}};
const Segment SGI1290[] = {{&S27364[44], 22}, {NULL}};
const Segment SGI1291[] = {{&S27364[30], 6}, {NULL}};
const Segment SGI1292[] = {{&S27364[114], 3}, {NULL}};
const Segment SGI1293[] = {{&S27364[80], 5}, {NULL}};
const Segment SGI1294[] = {{&S27370[38], 7}, {NULL}};
const Segment SGI1295[] = {{&S27370[31], 6}, {NULL}};
const Segment SGI1296[] = {{&S27373[44], 7}, {NULL}};
const Segment SGI1297[] = {{&S27373[44], 22}, {NULL}};
const Segment SGI1298[] = {{&S27373[30], 6}, {NULL}};
const Segment SGI1299[] = {{&S27373[83], 4}, {NULL}};
const Segment SGI1300[] = {{&S27373[83], 3}, {NULL}};
const Segment SGI1301[] = {{&S27381[44], 7}, {NULL}};
const Segment SGI1302[] = {{&S27381[44], 24}, {NULL}};
const Segment SGI1303[] = {{&S27381[30], 6}, {NULL}};
const Segment SGI1304[] = {{&S27381[85], 4}, {NULL}};
const Segment SGI1305[] = {{&S27381[85], 3}, {NULL}};
const Segment SGI1306[] = {{&S27386[44], 7}, {NULL}};
const Segment SGI1307[] = {{&S27386[44], 24}, {NULL}};
const Segment SGI1308[] = {{&S27386[30], 6}, {NULL}};
const Segment SGI1309[] = {{&S27386[85], 4}, {NULL}};
const Segment SGI1310[] = {{&S27386[85], 3}, {NULL}};
const Segment SGI1311[] = {{&S27387[44], 7}, {NULL}};
const Segment SGI1312[] = {{&S27387[44], 20}, {NULL}};
const Segment SGI1313[] = {{&S27387[30], 6}, {NULL}};
const Segment SGI1314[] = {{&S27387[81], 4}, {NULL}};
const Segment SGI1315[] = {{&S27387[81], 3}, {NULL}};
const Segment SGI1316[] = {{&S27390[44], 7}, {NULL}};
const Segment SGI1317[] = {{&S27390[44], 22}, {NULL}};
const Segment SGI1318[] = {{&S27390[30], 6}, {NULL}};
const Segment SGI1319[] = {{&S27390[83], 6}, {NULL}};
const Segment SGI1320[] = {{&S27391[44], 7}, {NULL}};
const Segment SGI1321[] = {{&S27391[44], 24}, {NULL}};
const Segment SGI1322[] = {{&S27391[30], 6}, {NULL}};
const Segment SGI1323[] = {{&S27391[85], 6}, {NULL}};
const Segment SGI1324[] = {{&S27438[44], 17}, {NULL}};
const Segment SGI1325[] = {{&S27438[44], 28}, {NULL}};
const Segment SGI1326[] = {{&S27438[30], 6}, {NULL}};
const Segment SGI1327[] = {{&S27438[120], 3}, {NULL}};
const Segment SGI1328[] = {{&S27438[86], 5}, {NULL}};
const Segment SGI1329[] = {{&S27439[44], 11}, {NULL}};
const Segment SGI1330[] = {{&S27439[44], 22}, {NULL}};
const Segment SGI1331[] = {{&S27439[30], 6}, {NULL}};
const Segment SGI1332[] = {{&S27439[114], 3}, {NULL}};
const Segment SGI1333[] = {{&S27439[80], 5}, {NULL}};
const Segment SGI1334[] = {{&S27440[44], 10}, {NULL}};
const Segment SGI1335[] = {{&S27440[44], 21}, {NULL}};
const Segment SGI1336[] = {{&S27440[30], 6}, {NULL}};
const Segment SGI1337[] = {{&S27440[113], 3}, {NULL}};
const Segment SGI1338[] = {{&S27440[79], 5}, {NULL}};
const Segment SGI1339[] = {{&S27441[44], 11}, {NULL}};
const Segment SGI1340[] = {{&S27441[44], 22}, {NULL}};
const Segment SGI1341[] = {{&S27441[30], 6}, {NULL}};
const Segment SGI1342[] = {{&S27441[114], 3}, {NULL}};
const Segment SGI1343[] = {{&S27441[80], 5}, {NULL}};
const Segment SGI1344[] = {{&S27444[44], 11}, {NULL}};
const Segment SGI1345[] = {{&S27444[44], 22}, {NULL}};
const Segment SGI1346[] = {{&S27444[30], 6}, {NULL}};
const Segment SGI1347[] = {{&S27444[114], 3}, {NULL}};
const Segment SGI1348[] = {{&S27444[80], 5}, {NULL}};
const Segment SGI1349[] = {{&S27445[108], 3}, {NULL}};
const Segment SGI1350[] = {{&S27445[75], 5}, {NULL}};
const Segment SGI1351[] = {{&S27446[38], 10}, {NULL}};
const Segment SGI1352[] = {{&S27446[31], 6}, {NULL}};
const Segment SGI1353[] = {{&S27447[38], 11}, {NULL}};
const Segment SGI1354[] = {{&S27447[31], 6}, {NULL}};
const Segment SGI1355[] = {{&S27449[44], 10}, {NULL}};
const Segment SGI1356[] = {{&S27449[44], 21}, {NULL}};
const Segment SGI1357[] = {{&S27449[30], 6}, {NULL}};
const Segment SGI1358[] = {{&S27449[82], 4}, {NULL}};
const Segment SGI1359[] = {{&S27449[82], 3}, {NULL}};
const Segment SGI1360[] = {{&S27450[38], 11}, {NULL}};
const Segment SGI1361[] = {{&S27450[31], 6}, {NULL}};
const Segment SGI1362[] = {{&S27451[44], 10}, {NULL}};
const Segment SGI1363[] = {{&S27451[44], 21}, {NULL}};
const Segment SGI1364[] = {{&S27451[30], 6}, {NULL}};
const Segment SGI1365[] = {{&S27451[82], 6}, {NULL}};
const Segment SGI1366[] = {{&S27452[42], 10}, {NULL}};
const Segment SGI1367[] = {{&S27452[30], 6}, {NULL}};
const Segment SGI1368[] = {{&S27452[111], 3}, {NULL}};
const Segment SGI1369[] = {{&S27452[77], 5}, {NULL}};
const Segment SGI1370[] = {{&S27453[38], 11}, {NULL}};
const Segment SGI1371[] = {{&S27453[31], 6}, {NULL}};
const Segment SGI1372[] = {{&S28683[22], 12}, {NULL}};
const Segment SGI1373[] = {{&S28778[22], 12}, {NULL}};
const Segment SGI1374[] = {{&S27379[42], 7}, {NULL}};
const Segment SGI1375[] = {{&S27379[42], 18}, {NULL}};
const Segment SGI1376[] = {{&S27379[30], 4}, {NULL}};
const Segment SGI1377[] = {{&S27379[108], 3}, {NULL}};
const Segment SGI1378[] = {{&S27379[74], 5}, {NULL}};
const Segment SGI1379[] = {{&S27454[42], 10}, {NULL}};
const Segment SGI1380[] = {{&S27454[42], 21}, {NULL}};
const Segment SGI1381[] = {{&S27454[30], 4}, {NULL}};
const Segment SGI1382[] = {{&S27454[111], 3}, {NULL}};
const Segment SGI1383[] = {{&S27454[77], 5}, {NULL}};
const Segment SGI1384[] = {{&S29702[22], 12}, {NULL}};
const Segment SGI1385[] = {{&S29960[6], 4}, {NULL}};
const Segment SGI1386[] = {{&S29960[32], 3}, {NULL}};
const Segment SGI1387[] = {{&S29960[81], 5}, {NULL}};
const Segment SGI1388[] = {{&S29971[22], 12}, {NULL}};
const Segment SGI1389[] = {{&S29035[44], 11}, {NULL}};
const Segment SGI1390[] = {{&S29035[44], 22}, {NULL}};
const Segment SGI1391[] = {{&S29035[30], 6}, {NULL}};
const Segment SGI1392[] = {{&S29035[114], 3}, {NULL}};
const Segment SGI1393[] = {{&S29035[80], 5}, {NULL}};
const Segment SGI1394[] = {{&S29036[44], 9}, {NULL}};
const Segment SGI1395[] = {{&S29036[44], 20}, {NULL}};
const Segment SGI1396[] = {{&S29036[30], 6}, {NULL}};
const Segment SGI1397[] = {{&S29036[112], 3}, {NULL}};
const Segment SGI1398[] = {{&S29036[78], 5}, {NULL}};
const Segment SGI1399[] = {{&S29039[44], 5}, {NULL}};
const Segment SGI1400[] = {{&S29039[44], 16}, {NULL}};
const Segment SGI1401[] = {{&S29039[30], 6}, {NULL}};
const Segment SGI1402[] = {{&S29039[108], 3}, {NULL}};
const Segment SGI1403[] = {{&S29039[74], 5}, {NULL}};
const Segment SGI1404[] = {{&S29040[44], 9}, {NULL}};
const Segment SGI1405[] = {{&S29040[44], 20}, {NULL}};
const Segment SGI1406[] = {{&S29040[30], 6}, {NULL}};
const Segment SGI1407[] = {{&S29040[112], 3}, {NULL}};
const Segment SGI1408[] = {{&S29040[78], 5}, {NULL}};
const Segment SGI1409[] = {{&S29042[44], 5}, {NULL}};
const Segment SGI1410[] = {{&S29042[44], 16}, {NULL}};
const Segment SGI1411[] = {{&S29042[30], 6}, {NULL}};
const Segment SGI1412[] = {{&S29042[108], 3}, {NULL}};
const Segment SGI1413[] = {{&S29042[74], 5}, {NULL}};
const Segment SGI1414[] = {{&S29046[44], 19}, {NULL}};
const Segment SGI1415[] = {{&S29046[44], 30}, {NULL}};
const Segment SGI1416[] = {{&S29046[30], 6}, {NULL}};
const Segment SGI1417[] = {{&S29046[122], 3}, {NULL}};
const Segment SGI1418[] = {{&S29046[88], 5}, {NULL}};
const Segment SGI1419[] = {{&S29047[44], 14}, {NULL}};
const Segment SGI1420[] = {{&S29047[44], 26}, {NULL}};
const Segment SGI1421[] = {{&S29047[30], 6}, {NULL}};
const Segment SGI1422[] = {{&S29047[118], 3}, {NULL}};
const Segment SGI1423[] = {{&S29047[84], 5}, {NULL}};
const Segment SGI1424[] = {{&S29059[44], 8}, {NULL}};
const Segment SGI1425[] = {{&S29059[44], 20}, {NULL}};
const Segment SGI1426[] = {{&S29059[30], 6}, {NULL}};
const Segment SGI1427[] = {{&S29059[112], 3}, {NULL}};
const Segment SGI1428[] = {{&S29059[78], 5}, {NULL}};
const Segment SGI1429[] = {{&S29066[44], 7}, {NULL}};
const Segment SGI1430[] = {{&S29066[44], 18}, {NULL}};
const Segment SGI1431[] = {{&S29066[30], 6}, {NULL}};
const Segment SGI1432[] = {{&S29066[110], 3}, {NULL}};
const Segment SGI1433[] = {{&S29066[76], 5}, {NULL}};
const Segment SGI1434[] = {{&S29071[44], 17}, {NULL}};
const Segment SGI1435[] = {{&S29071[44], 26}, {NULL}};
const Segment SGI1436[] = {{&S29071[30], 6}, {NULL}};
const Segment SGI1437[] = {{&S29071[118], 3}, {NULL}};
const Segment SGI1438[] = {{&S29071[84], 5}, {NULL}};
const Segment SGI1439[] = {{&S29072[61], 6}, {NULL}};
const Segment SGI1440[] = {{&S29072[61], 17}, {NULL}};
const Segment SGI1441[] = {{&S29072[47], 6}, {NULL}};
const Segment SGI1442[] = {{&S29072[126], 3}, {NULL}};
const Segment SGI1443[] = {{&S29072[92], 5}, {NULL}};
const Segment SGI1444[] = {{&S29074[44], 8}, {NULL}};
const Segment SGI1445[] = {{&S29074[44], 19}, {NULL}};
const Segment SGI1446[] = {{&S29074[30], 6}, {NULL}};
const Segment SGI1447[] = {{&S29074[111], 3}, {NULL}};
const Segment SGI1448[] = {{&S29074[77], 5}, {NULL}};
const Segment SGI1449[] = {{&S29077[44], 9}, {NULL}};
const Segment SGI1450[] = {{&S29077[44], 20}, {NULL}};
const Segment SGI1451[] = {{&S29077[30], 6}, {NULL}};
const Segment SGI1452[] = {{&S29077[112], 3}, {NULL}};
const Segment SGI1453[] = {{&S29077[78], 5}, {NULL}};
const Segment SGI1454[] = {{&S29078[44], 9}, {NULL}};
const Segment SGI1455[] = {{&S29078[44], 20}, {NULL}};
const Segment SGI1456[] = {{&S29078[30], 6}, {NULL}};
const Segment SGI1457[] = {{&S29078[112], 3}, {NULL}};
const Segment SGI1458[] = {{&S29078[78], 5}, {NULL}};
const Segment SGI1459[] = {{&S29080[44], 12}, {NULL}};
const Segment SGI1460[] = {{&S29080[44], 23}, {NULL}};
const Segment SGI1461[] = {{&S29080[30], 6}, {NULL}};
const Segment SGI1462[] = {{&S29080[115], 3}, {NULL}};
const Segment SGI1463[] = {{&S29080[81], 5}, {NULL}};
const Segment SGI1464[] = {{&S29081[44], 4}, {NULL}};
const Segment SGI1465[] = {{&S29081[44], 15}, {NULL}};
const Segment SGI1466[] = {{&S29081[30], 6}, {NULL}};
const Segment SGI1467[] = {{&S29081[107], 3}, {NULL}};
const Segment SGI1468[] = {{&S29081[73], 5}, {NULL}};
const Segment SGI1469[] = {{&S29082[44], 13}, {NULL}};
const Segment SGI1470[] = {{&S29082[44], 25}, {NULL}};
const Segment SGI1471[] = {{&S29082[30], 6}, {NULL}};
const Segment SGI1472[] = {{&S29082[117], 3}, {NULL}};
const Segment SGI1473[] = {{&S29082[83], 5}, {NULL}};
const Segment SGI1474[] = {{&S29083[44], 11}, {NULL}};
const Segment SGI1475[] = {{&S29083[44], 22}, {NULL}};
const Segment SGI1476[] = {{&S29083[30], 6}, {NULL}};
const Segment SGI1477[] = {{&S29083[114], 3}, {NULL}};
const Segment SGI1478[] = {{&S29083[80], 5}, {NULL}};
const Segment SGI1479[] = {{&S29084[44], 19}, {NULL}};
const Segment SGI1480[] = {{&S29084[44], 31}, {NULL}};
const Segment SGI1481[] = {{&S29084[30], 6}, {NULL}};
const Segment SGI1482[] = {{&S29084[123], 3}, {NULL}};
const Segment SGI1483[] = {{&S29084[89], 5}, {NULL}};
const Segment SGI1484[] = {{&S29086[44], 12}, {NULL}};
const Segment SGI1485[] = {{&S29086[44], 23}, {NULL}};
const Segment SGI1486[] = {{&S29086[30], 6}, {NULL}};
const Segment SGI1487[] = {{&S29086[115], 3}, {NULL}};
const Segment SGI1488[] = {{&S29086[81], 5}, {NULL}};
const Segment SGI1489[] = {{&S29088[44], 7}, {NULL}};
const Segment SGI1490[] = {{&S29088[44], 19}, {NULL}};
const Segment SGI1491[] = {{&S29088[30], 6}, {NULL}};
const Segment SGI1492[] = {{&S29088[111], 3}, {NULL}};
const Segment SGI1493[] = {{&S29088[77], 5}, {NULL}};
const Segment SGI1494[] = {{&S29090[44], 11}, {NULL}};
const Segment SGI1495[] = {{&S29090[44], 22}, {NULL}};
const Segment SGI1496[] = {{&S29090[30], 6}, {NULL}};
const Segment SGI1497[] = {{&S29090[114], 3}, {NULL}};
const Segment SGI1498[] = {{&S29090[80], 5}, {NULL}};
const Segment SGI1499[] = {{&S29094[44], 6}, {NULL}};
const Segment SGI1500[] = {{&S29094[44], 17}, {NULL}};
const Segment SGI1501[] = {{&S29094[30], 6}, {NULL}};
const Segment SGI1502[] = {{&S29094[109], 3}, {NULL}};
const Segment SGI1503[] = {{&S29094[75], 5}, {NULL}};
const Segment SGI1504[] = {{&S29095[44], 12}, {NULL}};
const Segment SGI1505[] = {{&S29095[44], 34}, {NULL}};
const Segment SGI1506[] = {{&S29095[30], 6}, {NULL}};
const Segment SGI1507[] = {{&S29095[70], 3}, {NULL}};
const Segment SGI1508[] = {{&S29095[92], 5}, {NULL}};
const Segment SGI1509[] = {{&S29096[44], 8}, {NULL}};
const Segment SGI1510[] = {{&S29096[44], 20}, {NULL}};
const Segment SGI1511[] = {{&S29096[30], 6}, {NULL}};
const Segment SGI1512[] = {{&S29096[112], 3}, {NULL}};
const Segment SGI1513[] = {{&S29096[78], 5}, {NULL}};
const Segment SGI1514[] = {{&S29097[44], 7}, {NULL}};
const Segment SGI1515[] = {{&S29097[44], 19}, {NULL}};
const Segment SGI1516[] = {{&S29097[30], 6}, {NULL}};
const Segment SGI1517[] = {{&S29097[111], 3}, {NULL}};
const Segment SGI1518[] = {{&S29097[77], 5}, {NULL}};
const Segment SGI1519[] = {{&S29098[44], 27}, {NULL}};
const Segment SGI1520[] = {{&S29098[44], 39}, {NULL}};
const Segment SGI1521[] = {{&S29098[30], 6}, {NULL}};
const Segment SGI1522[] = {{&S29098[131], 3}, {NULL}};
const Segment SGI1523[] = {{&S29098[97], 5}, {NULL}};
const Segment SGI1524[] = {{&S29100[44], 14}, {NULL}};
const Segment SGI1525[] = {{&S29100[44], 25}, {NULL}};
const Segment SGI1526[] = {{&S29100[30], 6}, {NULL}};
const Segment SGI1527[] = {{&S29100[117], 3}, {NULL}};
const Segment SGI1528[] = {{&S29100[83], 5}, {NULL}};
const Segment SGI1529[] = {{&S29106[44], 6}, {NULL}};
const Segment SGI1530[] = {{&S29106[44], 17}, {NULL}};
const Segment SGI1531[] = {{&S29106[30], 6}, {NULL}};
const Segment SGI1532[] = {{&S29106[109], 3}, {NULL}};
const Segment SGI1533[] = {{&S29106[75], 5}, {NULL}};
const Segment SGI1534[] = {{&S29107[44], 6}, {NULL}};
const Segment SGI1535[] = {{&S29107[44], 18}, {NULL}};
const Segment SGI1536[] = {{&S29107[30], 6}, {NULL}};
const Segment SGI1537[] = {{&S29107[110], 3}, {NULL}};
const Segment SGI1538[] = {{&S29107[76], 5}, {NULL}};
const Segment SGI1539[] = {{&S29111[44], 21}, {NULL}};
const Segment SGI1540[] = {{&S29111[30], 6}, {NULL}};
const Segment SGI1541[] = {{&S29111[130], 3}, {NULL}};
const Segment SGI1542[] = {{&S29111[95], 6}, {NULL}};
const Segment SGI1543[] = {{&S29113[44], 11}, {NULL}};
const Segment SGI1544[] = {{&S29113[44], 22}, {NULL}};
const Segment SGI1545[] = {{&S29113[30], 6}, {NULL}};
const Segment SGI1546[] = {{&S29113[114], 3}, {NULL}};
const Segment SGI1547[] = {{&S29113[80], 5}, {NULL}};
const Segment SGI1548[] = {{&S29114[44], 9}, {NULL}};
const Segment SGI1549[] = {{&S29114[44], 21}, {NULL}};
const Segment SGI1550[] = {{&S29114[30], 6}, {NULL}};
const Segment SGI1551[] = {{&S29114[113], 3}, {NULL}};
const Segment SGI1552[] = {{&S29114[79], 5}, {NULL}};
const Segment SGI1553[] = {{&S29115[44], 15}, {NULL}};
const Segment SGI1554[] = {{&S29115[44], 24}, {NULL}};
const Segment SGI1555[] = {{&S29115[30], 6}, {NULL}};
const Segment SGI1556[] = {{&S29115[116], 3}, {NULL}};
const Segment SGI1557[] = {{&S29115[82], 5}, {NULL}};
const Segment SGI1558[] = {{&S29119[44], 15}, {NULL}};
const Segment SGI1559[] = {{&S29119[44], 27}, {NULL}};
const Segment SGI1560[] = {{&S29119[30], 6}, {NULL}};
const Segment SGI1561[] = {{&S29119[119], 3}, {NULL}};
const Segment SGI1562[] = {{&S29119[85], 5}, {NULL}};
const Segment SGI1563[] = {{&S29121[44], 4}, {NULL}};
const Segment SGI1564[] = {{&S29121[44], 16}, {NULL}};
const Segment SGI1565[] = {{&S29121[30], 6}, {NULL}};
const Segment SGI1566[] = {{&S29121[108], 3}, {NULL}};
const Segment SGI1567[] = {{&S29121[74], 5}, {NULL}};
const Segment SGI1568[] = {{&S29122[44], 9}, {NULL}};
const Segment SGI1569[] = {{&S29122[44], 20}, {NULL}};
const Segment SGI1570[] = {{&S29122[30], 6}, {NULL}};
const Segment SGI1571[] = {{&S29122[112], 3}, {NULL}};
const Segment SGI1572[] = {{&S29122[78], 5}, {NULL}};
const Segment SGI1573[] = {{&S29124[44], 8}, {NULL}};
const Segment SGI1574[] = {{&S29124[44], 20}, {NULL}};
const Segment SGI1575[] = {{&S29124[30], 6}, {NULL}};
const Segment SGI1576[] = {{&S29124[112], 3}, {NULL}};
const Segment SGI1577[] = {{&S29124[78], 5}, {NULL}};
const Segment SGI1578[] = {{&S29126[44], 10}, {NULL}};
const Segment SGI1579[] = {{&S29126[44], 22}, {NULL}};
const Segment SGI1580[] = {{&S29126[30], 6}, {NULL}};
const Segment SGI1581[] = {{&S29126[114], 3}, {NULL}};
const Segment SGI1582[] = {{&S29126[80], 5}, {NULL}};
const Segment SGI1583[] = {{&S29127[44], 12}, {NULL}};
const Segment SGI1584[] = {{&S29127[44], 21}, {NULL}};
const Segment SGI1585[] = {{&S29127[30], 6}, {NULL}};
const Segment SGI1586[] = {{&S29127[113], 3}, {NULL}};
const Segment SGI1587[] = {{&S29127[79], 5}, {NULL}};
const Segment SGI1588[] = {{&S29130[44], 8}, {NULL}};
const Segment SGI1589[] = {{&S29130[44], 20}, {NULL}};
const Segment SGI1590[] = {{&S29130[30], 6}, {NULL}};
const Segment SGI1591[] = {{&S29130[112], 3}, {NULL}};
const Segment SGI1592[] = {{&S29130[78], 5}, {NULL}};
const Segment SGI1593[] = {{&S29132[44], 10}, {NULL}};
const Segment SGI1594[] = {{&S29132[44], 22}, {NULL}};
const Segment SGI1595[] = {{&S29132[30], 6}, {NULL}};
const Segment SGI1596[] = {{&S29132[114], 3}, {NULL}};
const Segment SGI1597[] = {{&S29132[80], 5}, {NULL}};
const Segment SGI1598[] = {{&S29141[44], 11}, {NULL}};
const Segment SGI1599[] = {{&S29141[44], 23}, {NULL}};
const Segment SGI1600[] = {{&S29141[30], 6}, {NULL}};
const Segment SGI1601[] = {{&S29141[115], 3}, {NULL}};
const Segment SGI1602[] = {{&S29141[81], 5}, {NULL}};
const Segment SGI1603[] = {{&S29147[44], 14}, {NULL}};
const Segment SGI1604[] = {{&S29147[44], 26}, {NULL}};
const Segment SGI1605[] = {{&S29147[30], 6}, {NULL}};
const Segment SGI1606[] = {{&S29147[118], 3}, {NULL}};
const Segment SGI1607[] = {{&S29147[84], 5}, {NULL}};
const Segment SGI1608[] = {{&S29148[44], 10}, {NULL}};
const Segment SGI1609[] = {{&S29148[44], 21}, {NULL}};
const Segment SGI1610[] = {{&S29148[30], 6}, {NULL}};
const Segment SGI1611[] = {{&S29148[113], 3}, {NULL}};
const Segment SGI1612[] = {{&S29148[79], 5}, {NULL}};
const Segment SGI1613[] = {{&S29150[44], 6}, {NULL}};
const Segment SGI1614[] = {{&S29150[44], 18}, {NULL}};
const Segment SGI1615[] = {{&S29150[30], 6}, {NULL}};
const Segment SGI1616[] = {{&S29150[110], 3}, {NULL}};
const Segment SGI1617[] = {{&S29150[76], 5}, {NULL}};
const Segment SGI1618[] = {{&S29151[44], 21}, {NULL}};
const Segment SGI1619[] = {{&S29151[30], 6}, {NULL}};
const Segment SGI1620[] = {{&S29151[130], 3}, {NULL}};
const Segment SGI1621[] = {{&S29151[95], 6}, {NULL}};
const Segment SGI1622[] = {{&S29152[44], 5}, {NULL}};
const Segment SGI1623[] = {{&S29152[44], 16}, {NULL}};
const Segment SGI1624[] = {{&S29152[30], 6}, {NULL}};
const Segment SGI1625[] = {{&S29152[108], 3}, {NULL}};
const Segment SGI1626[] = {{&S29152[74], 5}, {NULL}};
const Segment SGI1627[] = {{&S29154[44], 7}, {NULL}};
const Segment SGI1628[] = {{&S29154[44], 40}, {NULL}};
const Segment SGI1629[] = {{&S29154[30], 6}, {NULL}};
const Segment SGI1630[] = {{&S29154[132], 3}, {NULL}};
const Segment SGI1631[] = {{&S29154[98], 5}, {NULL}};
const Segment SGI1632[] = {{&S29156[44], 7}, {NULL}};
const Segment SGI1633[] = {{&S29156[44], 19}, {NULL}};
const Segment SGI1634[] = {{&S29156[30], 6}, {NULL}};
const Segment SGI1635[] = {{&S29156[111], 3}, {NULL}};
const Segment SGI1636[] = {{&S29156[77], 5}, {NULL}};
const Segment SGI1637[] = {{&S29157[44], 11}, {NULL}};
const Segment SGI1638[] = {{&S29157[44], 23}, {NULL}};
const Segment SGI1639[] = {{&S29157[30], 6}, {NULL}};
const Segment SGI1640[] = {{&S29157[115], 3}, {NULL}};
const Segment SGI1641[] = {{&S29157[81], 5}, {NULL}};
const Segment SGI1642[] = {{&S29160[37], 22}, {NULL}};
const Segment SGI1643[] = {{&S29160[30], 6}, {NULL}};
const Segment SGI1644[] = {{&S29160[128], 3}, {NULL}};
const Segment SGI1645[] = {{&S29160[93], 6}, {NULL}};
const Segment SGI1646[] = {{&S29160[146], 6}, {NULL}};
const Segment SGI1647[] = {{&S29161[37], 23}, {NULL}};
const Segment SGI1648[] = {{&S29161[30], 6}, {NULL}};
const Segment SGI1649[] = {{&S29161[129], 3}, {NULL}};
const Segment SGI1650[] = {{&S29161[94], 6}, {NULL}};
const Segment SGI1651[] = {{&S29161[147], 6}, {NULL}};
const Segment SGI1652[] = {{&S29162[44], 8}, {NULL}};
const Segment SGI1653[] = {{&S29162[44], 19}, {NULL}};
const Segment SGI1654[] = {{&S29162[30], 6}, {NULL}};
const Segment SGI1655[] = {{&S29162[111], 3}, {NULL}};
const Segment SGI1656[] = {{&S29162[77], 5}, {NULL}};
const Segment SGI1657[] = {{&S29163[44], 7}, {NULL}};
const Segment SGI1658[] = {{&S29163[44], 19}, {NULL}};
const Segment SGI1659[] = {{&S29163[30], 6}, {NULL}};
const Segment SGI1660[] = {{&S29163[111], 3}, {NULL}};
const Segment SGI1661[] = {{&S29163[77], 5}, {NULL}};
const Segment SGI1662[] = {{&S29165[43], 32}, {NULL}};
const Segment SGI1663[] = {{&S29167[44], 12}, {NULL}};
const Segment SGI1664[] = {{&S29167[44], 34}, {NULL}};
const Segment SGI1665[] = {{&S29167[30], 6}, {NULL}};
const Segment SGI1666[] = {{&S29167[70], 3}, {NULL}};
const Segment SGI1667[] = {{&S29167[92], 5}, {NULL}};
const Segment SGI1668[] = {{&S29170[44], 24}, {NULL}};
const Segment SGI1669[] = {{&S29170[44], 35}, {NULL}};
const Segment SGI1670[] = {{&S29170[30], 6}, {NULL}};
const Segment SGI1671[] = {{&S29170[128], 3}, {NULL}};
const Segment SGI1672[] = {{&S29170[93], 6}, {NULL}};
const Segment SGI1673[] = {{&S29172[44], 8}, {NULL}};
const Segment SGI1674[] = {{&S29172[44], 20}, {NULL}};
const Segment SGI1675[] = {{&S29172[30], 6}, {NULL}};
const Segment SGI1676[] = {{&S29172[112], 3}, {NULL}};
const Segment SGI1677[] = {{&S29172[78], 5}, {NULL}};
const Segment SGI1678[] = {{&S29173[44], 8}, {NULL}};
const Segment SGI1679[] = {{&S29173[44], 19}, {NULL}};
const Segment SGI1680[] = {{&S29173[30], 6}, {NULL}};
const Segment SGI1681[] = {{&S29173[111], 3}, {NULL}};
const Segment SGI1682[] = {{&S29173[77], 5}, {NULL}};
const Segment SGI1683[] = {{&S29178[44], 13}, {NULL}};
const Segment SGI1684[] = {{&S29178[44], 25}, {NULL}};
const Segment SGI1685[] = {{&S29178[30], 6}, {NULL}};
const Segment SGI1686[] = {{&S29178[117], 3}, {NULL}};
const Segment SGI1687[] = {{&S29178[83], 5}, {NULL}};
const Segment SGI1688[] = {{&S29179[44], 6}, {NULL}};
const Segment SGI1689[] = {{&S29179[44], 18}, {NULL}};
const Segment SGI1690[] = {{&S29179[30], 6}, {NULL}};
const Segment SGI1691[] = {{&S29179[110], 3}, {NULL}};
const Segment SGI1692[] = {{&S29179[76], 5}, {NULL}};
const Segment SGI1693[] = {{&S29180[44], 9}, {NULL}};
const Segment SGI1694[] = {{&S29180[44], 20}, {NULL}};
const Segment SGI1695[] = {{&S29180[30], 6}, {NULL}};
const Segment SGI1696[] = {{&S29180[112], 3}, {NULL}};
const Segment SGI1697[] = {{&S29180[78], 5}, {NULL}};
const Segment SGI1698[] = {{&S29181[44], 5}, {NULL}};
const Segment SGI1699[] = {{&S29181[44], 16}, {NULL}};
const Segment SGI1700[] = {{&S29181[30], 6}, {NULL}};
const Segment SGI1701[] = {{&S29181[108], 3}, {NULL}};
const Segment SGI1702[] = {{&S29181[74], 5}, {NULL}};
const Segment SGI1703[] = {{&S29186[44], 7}, {NULL}};
const Segment SGI1704[] = {{&S29186[44], 19}, {NULL}};
const Segment SGI1705[] = {{&S29186[30], 6}, {NULL}};
const Segment SGI1706[] = {{&S29186[111], 3}, {NULL}};
const Segment SGI1707[] = {{&S29186[77], 5}, {NULL}};
const Segment SGI1708[] = {{&S29187[44], 11}, {NULL}};
const Segment SGI1709[] = {{&S29187[44], 22}, {NULL}};
const Segment SGI1710[] = {{&S29187[30], 6}, {NULL}};
const Segment SGI1711[] = {{&S29187[114], 3}, {NULL}};
const Segment SGI1712[] = {{&S29187[80], 5}, {NULL}};
const Segment SGI1713[] = {{&S29192[44], 4}, {NULL}};
const Segment SGI1714[] = {{&S29192[44], 16}, {NULL}};
const Segment SGI1715[] = {{&S29192[30], 6}, {NULL}};
const Segment SGI1716[] = {{&S29192[108], 3}, {NULL}};
const Segment SGI1717[] = {{&S29192[74], 5}, {NULL}};
const Segment SGI1718[] = {{&S29193[44], 7}, {NULL}};
const Segment SGI1719[] = {{&S29193[44], 19}, {NULL}};
const Segment SGI1720[] = {{&S29193[30], 6}, {NULL}};
const Segment SGI1721[] = {{&S29193[111], 3}, {NULL}};
const Segment SGI1722[] = {{&S29193[77], 5}, {NULL}};
const Segment SGI1723[] = {{&S29202[44], 7}, {NULL}};
const Segment SGI1724[] = {{&S29202[44], 18}, {NULL}};
const Segment SGI1725[] = {{&S29202[30], 6}, {NULL}};
const Segment SGI1726[] = {{&S29202[110], 3}, {NULL}};
const Segment SGI1727[] = {{&S29202[76], 5}, {NULL}};
const Segment SGI1728[] = {{&S29206[44], 9}, {NULL}};
const Segment SGI1729[] = {{&S29206[44], 21}, {NULL}};
const Segment SGI1730[] = {{&S29206[30], 6}, {NULL}};
const Segment SGI1731[] = {{&S29206[113], 3}, {NULL}};
const Segment SGI1732[] = {{&S29206[79], 5}, {NULL}};
const Segment SGI1733[] = {{&S29207[44], 12}, {NULL}};
const Segment SGI1734[] = {{&S29207[44], 32}, {NULL}};
const Segment SGI1735[] = {{&S29207[30], 6}, {NULL}};
const Segment SGI1736[] = {{&S29207[124], 3}, {NULL}};
const Segment SGI1737[] = {{&S29207[90], 5}, {NULL}};
const Segment SGI1738[] = {{&S29209[44], 12}, {NULL}};
const Segment SGI1739[] = {{&S29209[44], 34}, {NULL}};
const Segment SGI1740[] = {{&S29209[30], 6}, {NULL}};
const Segment SGI1741[] = {{&S29209[70], 3}, {NULL}};
const Segment SGI1742[] = {{&S29209[92], 5}, {NULL}};
const Segment SGI1743[] = {{&S29219[44], 21}, {NULL}};
const Segment SGI1744[] = {{&S29219[44], 33}, {NULL}};
const Segment SGI1745[] = {{&S29219[30], 6}, {NULL}};
const Segment SGI1746[] = {{&S29219[125], 3}, {NULL}};
const Segment SGI1747[] = {{&S29219[91], 5}, {NULL}};
const Segment SGI1748[] = {{&S29221[44], 7}, {NULL}};
const Segment SGI1749[] = {{&S29221[44], 18}, {NULL}};
const Segment SGI1750[] = {{&S29221[30], 6}, {NULL}};
const Segment SGI1751[] = {{&S29221[110], 3}, {NULL}};
const Segment SGI1752[] = {{&S29221[76], 5}, {NULL}};
const Segment SGI1753[] = {{&S29223[44], 23}, {NULL}};
const Segment SGI1754[] = {{&S29223[44], 34}, {NULL}};
const Segment SGI1755[] = {{&S29223[30], 6}, {NULL}};
const Segment SGI1756[] = {{&S29223[126], 3}, {NULL}};
const Segment SGI1757[] = {{&S29223[92], 5}, {NULL}};
const Segment SGI1758[] = {{&S29224[44], 7}, {NULL}};
const Segment SGI1759[] = {{&S29224[44], 30}, {NULL}};
const Segment SGI1760[] = {{&S29224[30], 6}, {NULL}};
const Segment SGI1761[] = {{&S29224[122], 3}, {NULL}};
const Segment SGI1762[] = {{&S29224[88], 5}, {NULL}};
const Segment SGI1763[] = {{&S29227[44], 7}, {NULL}};
const Segment SGI1764[] = {{&S29227[44], 18}, {NULL}};
const Segment SGI1765[] = {{&S29227[30], 6}, {NULL}};
const Segment SGI1766[] = {{&S29227[110], 3}, {NULL}};
const Segment SGI1767[] = {{&S29227[76], 5}, {NULL}};
const Segment SGI1768[] = {{&S29228[44], 8}, {NULL}};
const Segment SGI1769[] = {{&S29228[44], 19}, {NULL}};
const Segment SGI1770[] = {{&S29228[30], 6}, {NULL}};
const Segment SGI1771[] = {{&S29228[111], 3}, {NULL}};
const Segment SGI1772[] = {{&S29228[77], 5}, {NULL}};
const Segment SGI1773[] = {{&S29229[44], 17}, {NULL}};
const Segment SGI1774[] = {{&S29229[44], 28}, {NULL}};
const Segment SGI1775[] = {{&S29229[30], 6}, {NULL}};
const Segment SGI1776[] = {{&S29229[120], 3}, {NULL}};
const Segment SGI1777[] = {{&S29229[86], 5}, {NULL}};
const Segment SGI1778[] = {{&S29238[44], 10}, {NULL}};
const Segment SGI1779[] = {{&S29238[44], 22}, {NULL}};
const Segment SGI1780[] = {{&S29238[30], 6}, {NULL}};
const Segment SGI1781[] = {{&S29238[114], 3}, {NULL}};
const Segment SGI1782[] = {{&S29238[80], 5}, {NULL}};
const Segment SGI1783[] = {{&S29239[44], 10}, {NULL}};
const Segment SGI1784[] = {{&S29239[44], 22}, {NULL}};
const Segment SGI1785[] = {{&S29239[30], 6}, {NULL}};
const Segment SGI1786[] = {{&S29239[114], 3}, {NULL}};
const Segment SGI1787[] = {{&S29239[80], 5}, {NULL}};
const Segment SGI1788[] = {{&S29241[44], 7}, {NULL}};
const Segment SGI1789[] = {{&S29241[44], 18}, {NULL}};
const Segment SGI1790[] = {{&S29241[30], 6}, {NULL}};
const Segment SGI1791[] = {{&S29241[110], 3}, {NULL}};
const Segment SGI1792[] = {{&S29241[76], 5}, {NULL}};
const Segment SGI1793[] = {{&S29242[44], 8}, {NULL}};
const Segment SGI1794[] = {{&S29242[44], 20}, {NULL}};
const Segment SGI1795[] = {{&S29242[30], 6}, {NULL}};
const Segment SGI1796[] = {{&S29242[112], 3}, {NULL}};
const Segment SGI1797[] = {{&S29242[78], 5}, {NULL}};
const Segment SGI1798[] = {{&S29243[44], 10}, {NULL}};
const Segment SGI1799[] = {{&S29243[44], 19}, {NULL}};
const Segment SGI1800[] = {{&S29243[30], 6}, {NULL}};
const Segment SGI1801[] = {{&S29243[111], 3}, {NULL}};
const Segment SGI1802[] = {{&S29243[77], 5}, {NULL}};
const Segment SGI1803[] = {{&S29246[44], 12}, {NULL}};
const Segment SGI1804[] = {{&S29246[44], 24}, {NULL}};
const Segment SGI1805[] = {{&S29246[30], 6}, {NULL}};
const Segment SGI1806[] = {{&S29246[116], 3}, {NULL}};
const Segment SGI1807[] = {{&S29246[82], 5}, {NULL}};
const Segment SGI1808[] = {{&S29247[44], 10}, {NULL}};
const Segment SGI1809[] = {{&S29247[44], 22}, {NULL}};
const Segment SGI1810[] = {{&S29247[30], 6}, {NULL}};
const Segment SGI1811[] = {{&S29247[114], 3}, {NULL}};
const Segment SGI1812[] = {{&S29247[80], 5}, {NULL}};
const Segment SGI1813[] = {{&S29250[44], 10}, {NULL}};
const Segment SGI1814[] = {{&S29250[44], 21}, {NULL}};
const Segment SGI1815[] = {{&S29250[30], 6}, {NULL}};
const Segment SGI1816[] = {{&S29250[113], 3}, {NULL}};
const Segment SGI1817[] = {{&S29250[79], 5}, {NULL}};
const Segment SGI1818[] = {{&S29253[44], 6}, {NULL}};
const Segment SGI1819[] = {{&S29253[44], 18}, {NULL}};
const Segment SGI1820[] = {{&S29253[30], 6}, {NULL}};
const Segment SGI1821[] = {{&S29253[110], 3}, {NULL}};
const Segment SGI1822[] = {{&S29253[76], 5}, {NULL}};
const Segment SGI1823[] = {{&S29254[44], 9}, {NULL}};
const Segment SGI1824[] = {{&S29254[44], 28}, {NULL}};
const Segment SGI1825[] = {{&S29254[30], 6}, {NULL}};
const Segment SGI1826[] = {{&S29254[120], 3}, {NULL}};
const Segment SGI1827[] = {{&S29254[86], 5}, {NULL}};
const Segment SGI1828[] = {{&S29257[44], 15}, {NULL}};
const Segment SGI1829[] = {{&S29257[44], 26}, {NULL}};
const Segment SGI1830[] = {{&S29257[30], 6}, {NULL}};
const Segment SGI1831[] = {{&S29257[118], 3}, {NULL}};
const Segment SGI1832[] = {{&S29257[84], 5}, {NULL}};
const Segment SGI1833[] = {{&S29258[44], 5}, {NULL}};
const Segment SGI1834[] = {{&S29258[44], 16}, {NULL}};
const Segment SGI1835[] = {{&S29258[30], 6}, {NULL}};
const Segment SGI1836[] = {{&S29258[108], 3}, {NULL}};
const Segment SGI1837[] = {{&S29258[74], 5}, {NULL}};
const Segment SGI1838[] = {{&S29260[44], 8}, {NULL}};
const Segment SGI1839[] = {{&S29260[44], 20}, {NULL}};
const Segment SGI1840[] = {{&S29260[30], 6}, {NULL}};
const Segment SGI1841[] = {{&S29260[112], 3}, {NULL}};
const Segment SGI1842[] = {{&S29260[78], 5}, {NULL}};
const Segment SGI1843[] = {{&S29262[44], 9}, {NULL}};
const Segment SGI1844[] = {{&S29262[44], 20}, {NULL}};
const Segment SGI1845[] = {{&S29262[30], 6}, {NULL}};
const Segment SGI1846[] = {{&S29262[112], 3}, {NULL}};
const Segment SGI1847[] = {{&S29262[78], 5}, {NULL}};
const Segment SGI1848[] = {{&S29263[44], 6}, {NULL}};
const Segment SGI1849[] = {{&S29263[44], 39}, {NULL}};
const Segment SGI1850[] = {{&S29263[30], 6}, {NULL}};
const Segment SGI1851[] = {{&S29263[131], 3}, {NULL}};
const Segment SGI1852[] = {{&S29263[97], 5}, {NULL}};
const Segment SGI1853[] = {{&S29264[44], 7}, {NULL}};
const Segment SGI1854[] = {{&S29264[44], 30}, {NULL}};
const Segment SGI1855[] = {{&S29264[30], 6}, {NULL}};
const Segment SGI1856[] = {{&S29264[91], 4}, {NULL}};
const Segment SGI1857[] = {{&S29264[91], 3}, {NULL}};
const Segment SGI1858[] = {{&S29265[44], 8}, {NULL}};
const Segment SGI1859[] = {{&S29265[44], 19}, {NULL}};
const Segment SGI1860[] = {{&S29265[30], 6}, {NULL}};
const Segment SGI1861[] = {{&S29265[111], 3}, {NULL}};
const Segment SGI1862[] = {{&S29265[77], 5}, {NULL}};
const Segment SGI1863[] = {{&S29266[44], 6}, {NULL}};
const Segment SGI1864[] = {{&S29266[44], 18}, {NULL}};
const Segment SGI1865[] = {{&S29266[30], 6}, {NULL}};
const Segment SGI1866[] = {{&S29266[79], 4}, {NULL}};
const Segment SGI1867[] = {{&S29266[79], 3}, {NULL}};
const Segment SGI1868[] = {{&S29269[44], 7}, {NULL}};
const Segment SGI1869[] = {{&S29269[44], 40}, {NULL}};
const Segment SGI1870[] = {{&S29269[30], 6}, {NULL}};
const Segment SGI1871[] = {{&S29269[132], 3}, {NULL}};
const Segment SGI1872[] = {{&S29269[98], 5}, {NULL}};
const Segment SGI1873[] = {{&S29270[44], 7}, {NULL}};
const Segment SGI1874[] = {{&S29270[44], 19}, {NULL}};
const Segment SGI1875[] = {{&S29270[30], 6}, {NULL}};
const Segment SGI1876[] = {{&S29270[111], 3}, {NULL}};
const Segment SGI1877[] = {{&S29270[77], 5}, {NULL}};
const Segment SGI1878[] = {{&S29272[44], 8}, {NULL}};
const Segment SGI1879[] = {{&S29272[44], 20}, {NULL}};
const Segment SGI1880[] = {{&S29272[30], 6}, {NULL}};
const Segment SGI1881[] = {{&S29272[112], 3}, {NULL}};
const Segment SGI1882[] = {{&S29272[78], 5}, {NULL}};
const Segment SGI1883[] = {{&S29275[44], 14}, {NULL}};
const Segment SGI1884[] = {{&S29275[44], 26}, {NULL}};
const Segment SGI1885[] = {{&S29275[30], 6}, {NULL}};
const Segment SGI1886[] = {{&S29275[118], 3}, {NULL}};
const Segment SGI1887[] = {{&S29275[84], 5}, {NULL}};
const Segment SGI1888[] = {{&S29277[44], 7}, {NULL}};
const Segment SGI1889[] = {{&S29277[44], 19}, {NULL}};
const Segment SGI1890[] = {{&S29277[30], 6}, {NULL}};
const Segment SGI1891[] = {{&S29277[111], 3}, {NULL}};
const Segment SGI1892[] = {{&S29277[77], 5}, {NULL}};
const Segment SGI1893[] = {{&S29278[44], 5}, {NULL}};
const Segment SGI1894[] = {{&S29278[44], 16}, {NULL}};
const Segment SGI1895[] = {{&S29278[30], 6}, {NULL}};
const Segment SGI1896[] = {{&S29278[108], 3}, {NULL}};
const Segment SGI1897[] = {{&S29278[74], 5}, {NULL}};
const Segment SGI1898[] = {{&S29279[44], 8}, {NULL}};
const Segment SGI1899[] = {{&S29279[44], 20}, {NULL}};
const Segment SGI1900[] = {{&S29279[30], 6}, {NULL}};
const Segment SGI1901[] = {{&S29279[112], 3}, {NULL}};
const Segment SGI1902[] = {{&S29279[78], 5}, {NULL}};
const Segment SGI1903[] = {{&S29282[44], 8}, {NULL}};
const Segment SGI1904[] = {{&S29282[44], 20}, {NULL}};
const Segment SGI1905[] = {{&S29282[30], 6}, {NULL}};
const Segment SGI1906[] = {{&S29282[112], 3}, {NULL}};
const Segment SGI1907[] = {{&S29282[78], 5}, {NULL}};
const Segment SGI1908[] = {{&S29287[44], 10}, {NULL}};
const Segment SGI1909[] = {{&S29287[44], 22}, {NULL}};
const Segment SGI1910[] = {{&S29287[30], 6}, {NULL}};
const Segment SGI1911[] = {{&S29287[114], 3}, {NULL}};
const Segment SGI1912[] = {{&S29287[80], 5}, {NULL}};
const Segment SGI1913[] = {{&S29288[44], 12}, {NULL}};
const Segment SGI1914[] = {{&S29288[44], 24}, {NULL}};
const Segment SGI1915[] = {{&S29288[30], 6}, {NULL}};
const Segment SGI1916[] = {{&S29288[116], 3}, {NULL}};
const Segment SGI1917[] = {{&S29288[82], 5}, {NULL}};
const Segment SGI1918[] = {{&S29291[44], 11}, {NULL}};
const Segment SGI1919[] = {{&S29291[44], 23}, {NULL}};
const Segment SGI1920[] = {{&S29291[30], 6}, {NULL}};
const Segment SGI1921[] = {{&S29291[115], 3}, {NULL}};
const Segment SGI1922[] = {{&S29291[81], 5}, {NULL}};
const Segment SGI1923[] = {{&S29294[44], 10}, {NULL}};
const Segment SGI1924[] = {{&S29294[44], 22}, {NULL}};
const Segment SGI1925[] = {{&S29294[30], 6}, {NULL}};
const Segment SGI1926[] = {{&S29294[114], 3}, {NULL}};
const Segment SGI1927[] = {{&S29294[80], 5}, {NULL}};
const Segment SGI1928[] = {{&S29299[44], 17}, {NULL}};
const Segment SGI1929[] = {{&S29299[44], 28}, {NULL}};
const Segment SGI1930[] = {{&S29299[30], 6}, {NULL}};
const Segment SGI1931[] = {{&S29299[89], 4}, {NULL}};
const Segment SGI1932[] = {{&S29299[89], 3}, {NULL}};
const Segment SGI1933[] = {{&S29304[44], 8}, {NULL}};
const Segment SGI1934[] = {{&S29304[44], 20}, {NULL}};
const Segment SGI1935[] = {{&S29304[30], 6}, {NULL}};
const Segment SGI1936[] = {{&S29304[112], 3}, {NULL}};
const Segment SGI1937[] = {{&S29304[78], 5}, {NULL}};
const Segment SGI1938[] = {{&S29307[44], 8}, {NULL}};
const Segment SGI1939[] = {{&S29307[44], 19}, {NULL}};
const Segment SGI1940[] = {{&S29307[30], 6}, {NULL}};
const Segment SGI1941[] = {{&S29307[80], 4}, {NULL}};
const Segment SGI1942[] = {{&S29307[80], 3}, {NULL}};
const Segment SGI1943[] = {{&S29308[44], 9}, {NULL}};
const Segment SGI1944[] = {{&S29308[44], 21}, {NULL}};
const Segment SGI1945[] = {{&S29308[30], 6}, {NULL}};
const Segment SGI1946[] = {{&S29308[113], 3}, {NULL}};
const Segment SGI1947[] = {{&S29308[79], 5}, {NULL}};
const Segment SGI1948[] = {{&S29309[44], 8}, {NULL}};
const Segment SGI1949[] = {{&S29309[44], 20}, {NULL}};
const Segment SGI1950[] = {{&S29309[30], 6}, {NULL}};
const Segment SGI1951[] = {{&S29309[112], 3}, {NULL}};
const Segment SGI1952[] = {{&S29309[78], 5}, {NULL}};
const Segment SGI1953[] = {{&S29312[44], 11}, {NULL}};
const Segment SGI1954[] = {{&S29312[44], 23}, {NULL}};
const Segment SGI1955[] = {{&S29312[30], 6}, {NULL}};
const Segment SGI1956[] = {{&S29312[115], 3}, {NULL}};
const Segment SGI1957[] = {{&S29312[81], 5}, {NULL}};
const Segment SGI1958[] = {{&S29316[44], 7}, {NULL}};
const Segment SGI1959[] = {{&S29316[44], 18}, {NULL}};
const Segment SGI1960[] = {{&S29316[30], 6}, {NULL}};
const Segment SGI1961[] = {{&S29316[110], 3}, {NULL}};
const Segment SGI1962[] = {{&S29316[76], 5}, {NULL}};
const Segment SGI1963[] = {{&S29318[44], 11}, {NULL}};
const Segment SGI1964[] = {{&S29318[44], 23}, {NULL}};
const Segment SGI1965[] = {{&S29318[30], 6}, {NULL}};
const Segment SGI1966[] = {{&S29318[115], 3}, {NULL}};
const Segment SGI1967[] = {{&S29318[81], 5}, {NULL}};
const Segment SGI1968[] = {{&S29319[44], 28}, {NULL}};
const Segment SGI1969[] = {{&S29319[44], 40}, {NULL}};
const Segment SGI1970[] = {{&S29319[30], 6}, {NULL}};
const Segment SGI1971[] = {{&S29319[132], 3}, {NULL}};
const Segment SGI1972[] = {{&S29319[98], 5}, {NULL}};
const Segment SGI1973[] = {{&S29320[37], 24}, {NULL}};
const Segment SGI1974[] = {{&S29320[30], 6}, {NULL}};
const Segment SGI1975[] = {{&S29320[130], 3}, {NULL}};
const Segment SGI1976[] = {{&S29320[95], 6}, {NULL}};
const Segment SGI1977[] = {{&S29320[148], 6}, {NULL}};
const Segment SGI1978[] = {{&S29322[44], 7}, {NULL}};
const Segment SGI1979[] = {{&S30456[22], 12}, {NULL}};
const Segment SGI1980[] = {{&S29322[44], 19}, {NULL}};
const Segment SGI1981[] = {{&S29322[30], 6}, {NULL}};
const Segment SGI1982[] = {{&S29322[111], 3}, {NULL}};
const Segment SGI1983[] = {{&S29322[77], 5}, {NULL}};
const Segment SGI1984[] = {{&S29332[44], 7}, {NULL}};
const Segment SGI1985[] = {{&S29332[44], 19}, {NULL}};
const Segment SGI1986[] = {{&S29332[30], 6}, {NULL}};
const Segment SGI1987[] = {{&S29332[111], 3}, {NULL}};
const Segment SGI1988[] = {{&S29332[77], 5}, {NULL}};
const Segment SGI1989[] = {{&S29334[44], 8}, {NULL}};
const Segment SGI1990[] = {{&S29334[44], 19}, {NULL}};
const Segment SGI1991[] = {{&S29334[30], 6}, {NULL}};
const Segment SGI1992[] = {{&S29334[80], 6}, {NULL}};
const Segment SGI1993[] = {{&S29344[11], 1}, {NULL}};
const Segment SGI1994[] = {{&S29344[44], 10}, {NULL}};
const Segment SGI1995[] = {{&S29344[30], 6}, {NULL}};
const Segment SGI1996[] = {{&S29344[115], 3}, {NULL}};
const Segment SGI1997[] = {{&S29344[81], 5}, {NULL}};
const Segment SGI1998[] = {{&S29031[42], 14}, {NULL}};
const Segment SGI1999[] = {{&S29031[42], 25}, {NULL}};
const Segment SGI2000[] = {{&S29031[30], 4}, {NULL}};
const Segment SGI2001[] = {{&S29031[115], 3}, {NULL}};
const Segment SGI2002[] = {{&S29031[81], 5}, {NULL}};
const Segment SGI2003[] = {{&S29347[44], 31}, {NULL}};
const Segment SGI2004[] = {{&S29347[30], 6}, {NULL}};
const Segment SGI2005[] = {{&S29032[42], 28}, {NULL}};
const Segment SGI2006[] = {{&S29032[30], 4}, {NULL}};
const Segment SGI2007[] = {{&S29032[136], 3}, {NULL}};
const Segment SGI2008[] = {{&S29032[102], 5}, {NULL}};
const Segment SGI2009[] = {{&S29033[42], 5}, {NULL}};
const Segment SGI2010[] = {{&S29033[42], 16}, {NULL}};
const Segment SGI2011[] = {{&S29033[30], 4}, {NULL}};
const Segment SGI2012[] = {{&S29033[106], 3}, {NULL}};
const Segment SGI2013[] = {{&S29033[72], 5}, {NULL}};
const Segment SGI2014[] = {{&S29355[44], 6}, {NULL}};
const Segment SGI2015[] = {{&S29037[42], 29}, {NULL}};
const Segment SGI2016[] = {{&S29355[44], 18}, {NULL}};
const Segment SGI2017[] = {{&S29037[30], 4}, {NULL}};
const Segment SGI2018[] = {{&S29355[30], 6}, {NULL}};
const Segment SGI2019[] = {{&S29355[110], 3}, {NULL}};
const Segment SGI2020[] = {{&S29037[137], 3}, {NULL}};
const Segment SGI2021[] = {{&S29355[76], 5}, {NULL}};
const Segment SGI2022[] = {{&S29037[103], 5}, {NULL}};
const Segment SGI2023[] = {{&S29356[44], 9}, {NULL}};
const Segment SGI2024[] = {{&S29356[44], 21}, {NULL}};
const Segment SGI2025[] = {{&S29356[30], 6}, {NULL}};
const Segment SGI2026[] = {{&S29356[113], 3}, {NULL}};
const Segment SGI2027[] = {{&S29356[79], 5}, {NULL}};
const Segment SGI2028[] = {{&S29041[42], 12}, {NULL}};
const Segment SGI2029[] = {{&S29041[42], 23}, {NULL}};
const Segment SGI2030[] = {{&S29041[30], 4}, {NULL}};
const Segment SGI2031[] = {{&S29041[113], 3}, {NULL}};
const Segment SGI2032[] = {{&S29041[79], 5}, {NULL}};
const Segment SGI2033[] = {{&S29043[42], 42}, {NULL}};
const Segment SGI2034[] = {{&S29043[42], 53}, {NULL}};
const Segment SGI2035[] = {{&S29043[30], 4}, {NULL}};
const Segment SGI2036[] = {{&S29043[143], 3}, {NULL}};
const Segment SGI2037[] = {{&S29362[44], 7}, {NULL}};
const Segment SGI2038[] = {{&S29043[109], 5}, {NULL}};
const Segment SGI2039[] = {{&S29362[44], 19}, {NULL}};
const Segment SGI2040[] = {{&S29362[30], 6}, {NULL}};
const Segment SGI2041[] = {{&S29362[111], 3}, {NULL}};
const Segment SGI2042[] = {{&S29362[77], 5}, {NULL}};
const Segment SGI2043[] = {{&S29045[42], 21}, {NULL}};
const Segment SGI2044[] = {{&S29045[42], 46}, {NULL}};
const Segment SGI2045[] = {{&S29045[30], 4}, {NULL}};
const Segment SGI2046[] = {{&S29045[136], 3}, {NULL}};
const Segment SGI2047[] = {{&S29045[102], 5}, {NULL}};
const Segment SGI2048[] = {{&S29365[44], 7}, {NULL}};
const Segment SGI2049[] = {{&S29365[44], 18}, {NULL}};
const Segment SGI2050[] = {{&S29365[30], 6}, {NULL}};
const Segment SGI2051[] = {{&S29365[110], 3}, {NULL}};
const Segment SGI2052[] = {{&S29365[76], 5}, {NULL}};
const Segment SGI2053[] = {{&S29366[44], 8}, {NULL}};
const Segment SGI2054[] = {{&S29366[44], 20}, {NULL}};
const Segment SGI2055[] = {{&S29366[30], 6}, {NULL}};
const Segment SGI2056[] = {{&S29366[112], 3}, {NULL}};
const Segment SGI2057[] = {{&S29366[78], 5}, {NULL}};
const Segment SGI2058[] = {{&S29048[42], 10}, {NULL}};
const Segment SGI2059[] = {{&S29048[42], 21}, {NULL}};
const Segment SGI2060[] = {{&S29048[30], 4}, {NULL}};
const Segment SGI2061[] = {{&S29048[111], 3}, {NULL}};
const Segment SGI2062[] = {{&S29048[77], 5}, {NULL}};
const Segment SGI2063[] = {{&S29368[44], 8}, {NULL}};
const Segment SGI2064[] = {{&S29368[44], 20}, {NULL}};
const Segment SGI2065[] = {{&S29368[30], 6}, {NULL}};
const Segment SGI2066[] = {{&S29049[42], 24}, {NULL}};
const Segment SGI2067[] = {{&S29049[42], 35}, {NULL}};
const Segment SGI2068[] = {{&S29049[30], 4}, {NULL}};
const Segment SGI2069[] = {{&S29368[112], 3}, {NULL}};
const Segment SGI2070[] = {{&S29368[78], 5}, {NULL}};
const Segment SGI2071[] = {{&S29049[125], 3}, {NULL}};
const Segment SGI2072[] = {{&S29049[91], 5}, {NULL}};
const Segment SGI2073[] = {{&S29050[42], 8}, {NULL}};
const Segment SGI2074[] = {{&S29050[42], 19}, {NULL}};
const Segment SGI2075[] = {{&S29050[30], 4}, {NULL}};
const Segment SGI2076[] = {{&S29050[109], 3}, {NULL}};
const Segment SGI2077[] = {{&S29370[44], 12}, {NULL}};
const Segment SGI2078[] = {{&S29050[75], 5}, {NULL}};
const Segment SGI2079[] = {{&S29370[44], 24}, {NULL}};
const Segment SGI2080[] = {{&S29370[30], 6}, {NULL}};
const Segment SGI2081[] = {{&S29370[116], 3}, {NULL}};
const Segment SGI2082[] = {{&S29370[82], 5}, {NULL}};
const Segment SGI2083[] = {{&S29051[42], 7}, {NULL}};
const Segment SGI2084[] = {{&S29051[42], 19}, {NULL}};
const Segment SGI2085[] = {{&S29051[30], 4}, {NULL}};
const Segment SGI2086[] = {{&S29051[109], 3}, {NULL}};
const Segment SGI2087[] = {{&S29051[75], 5}, {NULL}};
const Segment SGI2088[] = {{&S29053[42], 13}, {NULL}};
const Segment SGI2089[] = {{&S29053[30], 4}, {NULL}};
const Segment SGI2090[] = {{&S29053[116], 3}, {NULL}};
const Segment SGI2091[] = {{&S29053[82], 5}, {NULL}};
const Segment SGI2092[] = {{&S29054[42], 9}, {NULL}};
const Segment SGI2093[] = {{&S29054[42], 21}, {NULL}};
const Segment SGI2094[] = {{&S29054[30], 4}, {NULL}};
const Segment SGI2095[] = {{&S29054[111], 3}, {NULL}};
const Segment SGI2096[] = {{&S29054[77], 5}, {NULL}};
const Segment SGI2097[] = {{&S29055[42], 6}, {NULL}};
const Segment SGI2098[] = {{&S29055[42], 18}, {NULL}};
const Segment SGI2099[] = {{&S29055[30], 4}, {NULL}};
const Segment SGI2100[] = {{&S29378[44], 7}, {NULL}};
const Segment SGI2101[] = {{&S29055[108], 3}, {NULL}};
const Segment SGI2102[] = {{&S29378[44], 19}, {NULL}};
const Segment SGI2103[] = {{&S29378[30], 6}, {NULL}};
const Segment SGI2104[] = {{&S29055[74], 5}, {NULL}};
const Segment SGI2105[] = {{&S29378[111], 3}, {NULL}};
const Segment SGI2106[] = {{&S29378[77], 5}, {NULL}};
const Segment SGI2107[] = {{&S29379[44], 7}, {NULL}};
const Segment SGI2108[] = {{&S29379[44], 18}, {NULL}};
const Segment SGI2109[] = {{&S29379[30], 6}, {NULL}};
const Segment SGI2110[] = {{&S29379[110], 3}, {NULL}};
const Segment SGI2111[] = {{&S29379[76], 5}, {NULL}};
const Segment SGI2112[] = {{&S29060[42], 10}, {NULL}};
const Segment SGI2113[] = {{&S29060[42], 21}, {NULL}};
const Segment SGI2114[] = {{&S29060[30], 4}, {NULL}};
const Segment SGI2115[] = {{&S29060[111], 3}, {NULL}};
const Segment SGI2116[] = {{&S29060[77], 5}, {NULL}};
const Segment SGI2117[] = {{&S29384[44], 14}, {NULL}};
const Segment SGI2118[] = {{&S29384[44], 26}, {NULL}};
const Segment SGI2119[] = {{&S29384[30], 6}, {NULL}};
const Segment SGI2120[] = {{&S29384[118], 3}, {NULL}};
const Segment SGI2121[] = {{&S29384[84], 5}, {NULL}};
const Segment SGI2122[] = {{&S29385[42], 7}, {NULL}};
const Segment SGI2123[] = {{&S29385[30], 6}, {NULL}};
const Segment SGI2124[] = {{&S29063[42], 12}, {NULL}};
const Segment SGI2125[] = {{&S29063[42], 23}, {NULL}};
const Segment SGI2126[] = {{&S29385[109], 3}, {NULL}};
const Segment SGI2127[] = {{&S29063[30], 4}, {NULL}};
const Segment SGI2128[] = {{&S29385[75], 5}, {NULL}};
const Segment SGI2129[] = {{&S29063[113], 3}, {NULL}};
const Segment SGI2130[] = {{&S29063[79], 5}, {NULL}};
const Segment SGI2131[] = {{&S29386[44], 6}, {NULL}};
const Segment SGI2132[] = {{&S29386[44], 18}, {NULL}};
const Segment SGI2133[] = {{&S29386[30], 6}, {NULL}};
const Segment SGI2134[] = {{&S29386[110], 3}, {NULL}};
const Segment SGI2135[] = {{&S29386[76], 5}, {NULL}};
const Segment SGI2136[] = {{&S29065[42], 14}, {NULL}};
const Segment SGI2137[] = {{&S29065[42], 25}, {NULL}};
const Segment SGI2138[] = {{&S29065[30], 4}, {NULL}};
const Segment SGI2139[] = {{&S29065[115], 3}, {NULL}};
const Segment SGI2140[] = {{&S29065[81], 5}, {NULL}};
const Segment SGI2141[] = {{&S29390[44], 17}, {NULL}};
const Segment SGI2142[] = {{&S29390[44], 28}, {NULL}};
const Segment SGI2143[] = {{&S29390[30], 6}, {NULL}};
const Segment SGI2144[] = {{&S29067[42], 26}, {NULL}};
const Segment SGI2145[] = {{&S29067[42], 38}, {NULL}};
const Segment SGI2146[] = {{&S29067[30], 4}, {NULL}};
const Segment SGI2147[] = {{&S29390[89], 6}, {NULL}};
const Segment SGI2148[] = {{&S29067[128], 3}, {NULL}};
const Segment SGI2149[] = {{&S29067[94], 5}, {NULL}};
const Segment SGI2150[] = {{&S29391[44], 7}, {NULL}};
const Segment SGI2151[] = {{&S29391[44], 19}, {NULL}};
const Segment SGI2152[] = {{&S29391[30], 6}, {NULL}};
const Segment SGI2153[] = {{&S29391[111], 3}, {NULL}};
const Segment SGI2154[] = {{&S29068[42], 7}, {NULL}};
const Segment SGI2155[] = {{&S29391[77], 5}, {NULL}};
const Segment SGI2156[] = {{&S29068[42], 19}, {NULL}};
const Segment SGI2157[] = {{&S29068[30], 4}, {NULL}};
const Segment SGI2158[] = {{&S29068[109], 3}, {NULL}};
const Segment SGI2159[] = {{&S29068[75], 5}, {NULL}};
const Segment SGI2160[] = {{&S29069[42], 16}, {NULL}};
const Segment SGI2161[] = {{&S29069[42], 27}, {NULL}};
const Segment SGI2162[] = {{&S29393[44], 11}, {NULL}};
const Segment SGI2163[] = {{&S29069[30], 4}, {NULL}};
const Segment SGI2164[] = {{&S29393[44], 23}, {NULL}};
const Segment SGI2165[] = {{&S29393[30], 6}, {NULL}};
const Segment SGI2166[] = {{&S29069[117], 3}, {NULL}};
const Segment SGI2167[] = {{&S29393[115], 3}, {NULL}};
const Segment SGI2168[] = {{&S29069[83], 5}, {NULL}};
const Segment SGI2169[] = {{&S29393[81], 5}, {NULL}};
const Segment SGI2170[] = {{&S29070[42], 6}, {NULL}};
const Segment SGI2171[] = {{&S29070[42], 17}, {NULL}};
const Segment SGI2172[] = {{&S29070[30], 4}, {NULL}};
const Segment SGI2173[] = {{&S29070[107], 3}, {NULL}};
const Segment SGI2174[] = {{&S29070[73], 5}, {NULL}};
const Segment SGI2175[] = {{&S29073[42], 13}, {NULL}};
const Segment SGI2176[] = {{&S29073[42], 24}, {NULL}};
const Segment SGI2177[] = {{&S29073[30], 4}, {NULL}};
const Segment SGI2178[] = {{&S29073[114], 3}, {NULL}};
const Segment SGI2179[] = {{&S29073[80], 5}, {NULL}};
const Segment SGI2180[] = {{&S29404[44], 7}, {NULL}};
const Segment SGI2181[] = {{&S29404[44], 19}, {NULL}};
const Segment SGI2182[] = {{&S29404[30], 6}, {NULL}};
const Segment SGI2183[] = {{&S29404[111], 3}, {NULL}};
const Segment SGI2184[] = {{&S29404[77], 5}, {NULL}};
const Segment SGI2185[] = {{&S29079[42], 9}, {NULL}};
const Segment SGI2186[] = {{&S29079[42], 21}, {NULL}};
const Segment SGI2187[] = {{&S29405[44], 11}, {NULL}};
const Segment SGI2188[] = {{&S29079[30], 4}, {NULL}};
const Segment SGI2189[] = {{&S29405[44], 23}, {NULL}};
const Segment SGI2190[] = {{&S29405[30], 6}, {NULL}};
const Segment SGI2191[] = {{&S29079[111], 3}, {NULL}};
const Segment SGI2192[] = {{&S29405[115], 3}, {NULL}};
const Segment SGI2193[] = {{&S29079[77], 5}, {NULL}};
const Segment SGI2194[] = {{&S29405[81], 5}, {NULL}};
const Segment SGI2195[] = {{&S29085[42], 9}, {NULL}};
const Segment SGI2196[] = {{&S29085[42], 20}, {NULL}};
const Segment SGI2197[] = {{&S29085[30], 4}, {NULL}};
const Segment SGI2198[] = {{&S29085[110], 3}, {NULL}};
const Segment SGI2199[] = {{&S29085[76], 5}, {NULL}};
const Segment SGI2200[] = {{&S29087[42], 6}, {NULL}};
const Segment SGI2201[] = {{&S29087[42], 17}, {NULL}};
const Segment SGI2202[] = {{&S29087[30], 4}, {NULL}};
const Segment SGI2203[] = {{&S29087[107], 3}, {NULL}};
const Segment SGI2204[] = {{&S29087[73], 5}, {NULL}};
const Segment SGI2205[] = {{&S29089[42], 7}, {NULL}};
const Segment SGI2206[] = {{&S29089[42], 18}, {NULL}};
const Segment SGI2207[] = {{&S29089[30], 4}, {NULL}};
const Segment SGI2208[] = {{&S29089[108], 3}, {NULL}};
const Segment SGI2209[] = {{&S29089[74], 5}, {NULL}};
const Segment SGI2210[] = {{&S29419[44], 10}, {NULL}};
const Segment SGI2211[] = {{&S29419[44], 22}, {NULL}};
const Segment SGI2212[] = {{&S29419[30], 6}, {NULL}};
const Segment SGI2213[] = {{&S29419[114], 3}, {NULL}};
const Segment SGI2214[] = {{&S29419[80], 5}, {NULL}};
const Segment SGI2215[] = {{&S29091[50], 13}, {NULL}};
const Segment SGI2216[] = {{&S29091[50], 25}, {NULL}};
const Segment SGI2217[] = {{&S29091[30], 12}, {NULL}};
const Segment SGI2218[] = {{&S29420[44], 5}, {NULL}};
const Segment SGI2219[] = {{&S29420[44], 17}, {NULL}};
const Segment SGI2220[] = {{&S29420[30], 6}, {NULL}};
const Segment SGI2221[] = {{&S29091[124], 3}, {NULL}};
const Segment SGI2222[] = {{&S29091[89], 6}, {NULL}};
const Segment SGI2223[] = {{&S29420[109], 3}, {NULL}};
const Segment SGI2224[] = {{&S29420[75], 5}, {NULL}};
const Segment SGI2225[] = {{&S29092[42], 10}, {NULL}};
const Segment SGI2226[] = {{&S29092[42], 22}, {NULL}};
const Segment SGI2227[] = {{&S29092[30], 4}, {NULL}};
const Segment SGI2228[] = {{&S29092[113], 3}, {NULL}};
const Segment SGI2229[] = {{&S29092[78], 6}, {NULL}};
const Segment SGI2230[] = {{&S29422[44], 6}, {NULL}};
const Segment SGI2231[] = {{&S29422[44], 25}, {NULL}};
const Segment SGI2232[] = {{&S29422[30], 6}, {NULL}};
const Segment SGI2233[] = {{&S29422[117], 3}, {NULL}};
const Segment SGI2234[] = {{&S29422[83], 5}, {NULL}};
const Segment SGI2235[] = {{&S29093[42], 12}, {NULL}};
const Segment SGI2236[] = {{&S29093[42], 23}, {NULL}};
const Segment SGI2237[] = {{&S29093[30], 4}, {NULL}};
const Segment SGI2238[] = {{&S29093[114], 3}, {NULL}};
const Segment SGI2239[] = {{&S29093[79], 6}, {NULL}};
const Segment SGI2240[] = {{&S29425[44], 5}, {NULL}};
const Segment SGI2241[] = {{&S29425[44], 17}, {NULL}};
const Segment SGI2242[] = {{&S29425[30], 6}, {NULL}};
const Segment SGI2243[] = {{&S29425[109], 3}, {NULL}};
const Segment SGI2244[] = {{&S29425[75], 5}, {NULL}};
const Segment SGI2245[] = {{&S29426[44], 14}, {NULL}};
const Segment SGI2246[] = {{&S29426[44], 26}, {NULL}};
const Segment SGI2247[] = {{&S29426[30], 6}, {NULL}};
const Segment SGI2248[] = {{&S29426[118], 3}, {NULL}};
const Segment SGI2249[] = {{&S29426[84], 5}, {NULL}};
const Segment SGI2250[] = {{&S29099[42], 17}, {NULL}};
const Segment SGI2251[] = {{&S29099[42], 29}, {NULL}};
const Segment SGI2252[] = {{&S29099[30], 4}, {NULL}};
const Segment SGI2253[] = {{&S29099[119], 3}, {NULL}};
const Segment SGI2254[] = {{&S29099[85], 5}, {NULL}};
const Segment SGI2255[] = {{&S29429[44], 10}, {NULL}};
const Segment SGI2256[] = {{&S29429[44], 22}, {NULL}};
const Segment SGI2257[] = {{&S29429[30], 6}, {NULL}};
const Segment SGI2258[] = {{&S29429[114], 3}, {NULL}};
const Segment SGI2259[] = {{&S29429[80], 5}, {NULL}};
const Segment SGI2260[] = {{&S29430[44], 11}, {NULL}};
const Segment SGI2261[] = {{&S29430[44], 22}, {NULL}};
const Segment SGI2262[] = {{&S29430[30], 6}, {NULL}};
const Segment SGI2263[] = {{&S29430[114], 3}, {NULL}};
const Segment SGI2264[] = {{&S29430[80], 5}, {NULL}};
const Segment SGI2265[] = {{&S29105[42], 16}, {NULL}};
const Segment SGI2266[] = {{&S29105[42], 27}, {NULL}};
const Segment SGI2267[] = {{&S29105[30], 4}, {NULL}};
const Segment SGI2268[] = {{&S29105[117], 3}, {NULL}};
const Segment SGI2269[] = {{&S29105[83], 5}, {NULL}};
const Segment SGI2270[] = {{&S29434[44], 10}, {NULL}};
const Segment SGI2271[] = {{&S29434[44], 22}, {NULL}};
const Segment SGI2272[] = {{&S29434[30], 6}, {NULL}};
const Segment SGI2273[] = {{&S29434[114], 3}, {NULL}};
const Segment SGI2274[] = {{&S29434[80], 5}, {NULL}};
const Segment SGI2275[] = {{&S29435[44], 4}, {NULL}};
const Segment SGI2276[] = {{&S29435[44], 15}, {NULL}};
const Segment SGI2277[] = {{&S29435[30], 6}, {NULL}};
const Segment SGI2278[] = {{&S29435[107], 3}, {NULL}};
const Segment SGI2279[] = {{&S29435[73], 5}, {NULL}};
const Segment SGI2280[] = {{&S29109[42], 9}, {NULL}};
const Segment SGI2281[] = {{&S29109[42], 21}, {NULL}};
const Segment SGI2282[] = {{&S29109[30], 4}, {NULL}};
const Segment SGI2283[] = {{&S29109[111], 3}, {NULL}};
const Segment SGI2284[] = {{&S29109[77], 5}, {NULL}};
const Segment SGI2285[] = {{&S29437[38], 10}, {NULL}};
const Segment SGI2286[] = {{&S29437[31], 6}, {NULL}};
const Segment SGI2287[] = {{&S29110[42], 6}, {NULL}};
const Segment SGI2288[] = {{&S29110[42], 18}, {NULL}};
const Segment SGI2289[] = {{&S29110[30], 4}, {NULL}};
const Segment SGI2290[] = {{&S29110[108], 3}, {NULL}};
const Segment SGI2291[] = {{&S29110[74], 5}, {NULL}};
const Segment SGI2292[] = {{&S29442[44], 22}, {NULL}};
const Segment SGI2293[] = {{&S29442[44], 34}, {NULL}};
const Segment SGI2294[] = {{&S29442[30], 6}, {NULL}};
const Segment SGI2295[] = {{&S29442[126], 3}, {NULL}};
const Segment SGI2296[] = {{&S29442[92], 5}, {NULL}};
const Segment SGI2297[] = {{&S29444[44], 9}, {NULL}};
const Segment SGI2298[] = {{&S29444[44], 21}, {NULL}};
const Segment SGI2299[] = {{&S29444[30], 6}, {NULL}};
const Segment SGI2300[] = {{&S29444[113], 3}, {NULL}};
const Segment SGI2301[] = {{&S29116[42], 22}, {NULL}};
const Segment SGI2302[] = {{&S29116[42], 33}, {NULL}};
const Segment SGI2303[] = {{&S29444[79], 5}, {NULL}};
const Segment SGI2304[] = {{&S29116[30], 4}, {NULL}};
const Segment SGI2305[] = {{&S29116[123], 3}, {NULL}};
const Segment SGI2306[] = {{&S29116[89], 5}, {NULL}};
const Segment SGI2307[] = {{&S29445[44], 14}, {NULL}};
const Segment SGI2308[] = {{&S29445[44], 32}, {NULL}};
const Segment SGI2309[] = {{&S29445[30], 6}, {NULL}};
const Segment SGI2310[] = {{&S29117[42], 4}, {NULL}};
const Segment SGI2311[] = {{&S29445[124], 3}, {NULL}};
const Segment SGI2312[] = {{&S29117[42], 16}, {NULL}};
const Segment SGI2313[] = {{&S29117[30], 4}, {NULL}};
const Segment SGI2314[] = {{&S29445[90], 5}, {NULL}};
const Segment SGI2315[] = {{&S29117[106], 3}, {NULL}};
const Segment SGI2316[] = {{&S29117[72], 5}, {NULL}};
const Segment SGI2317[] = {{&S29120[42], 8}, {NULL}};
const Segment SGI2318[] = {{&S29120[42], 28}, {NULL}};
const Segment SGI2319[] = {{&S29120[30], 4}, {NULL}};
const Segment SGI2320[] = {{&S29120[118], 3}, {NULL}};
const Segment SGI2321[] = {{&S29120[84], 5}, {NULL}};
const Segment SGI2322[] = {{&S29125[42], 10}, {NULL}};
const Segment SGI2323[] = {{&S29125[42], 22}, {NULL}};
const Segment SGI2324[] = {{&S29125[30], 4}, {NULL}};
const Segment SGI2325[] = {{&S29125[112], 3}, {NULL}};
const Segment SGI2326[] = {{&S29125[78], 5}, {NULL}};
const Segment SGI2327[] = {{&S29128[42], 9}, {NULL}};
const Segment SGI2328[] = {{&S29128[42], 21}, {NULL}};
const Segment SGI2329[] = {{&S29128[30], 4}, {NULL}};
const Segment SGI2330[] = {{&S29128[111], 3}, {NULL}};
const Segment SGI2331[] = {{&S29128[77], 5}, {NULL}};
const Segment SGI2332[] = {{&S29129[42], 32}, {NULL}};
const Segment SGI2333[] = {{&S29129[30], 4}, {NULL}};
const Segment SGI2334[] = {{&S29134[42], 7}, {NULL}};
const Segment SGI2335[] = {{&S29134[42], 19}, {NULL}};
const Segment SGI2336[] = {{&S29134[30], 4}, {NULL}};
const Segment SGI2337[] = {{&S29134[109], 3}, {NULL}};
const Segment SGI2338[] = {{&S29134[75], 5}, {NULL}};
const Segment SGI2339[] = {{&S29135[42], 9}, {NULL}};
const Segment SGI2340[] = {{&S29135[42], 20}, {NULL}};
const Segment SGI2341[] = {{&S29135[30], 4}, {NULL}};
const Segment SGI2342[] = {{&S29135[110], 3}, {NULL}};
const Segment SGI2343[] = {{&S29135[76], 5}, {NULL}};
const Segment SGI2344[] = {{&S29137[42], 8}, {NULL}};
const Segment SGI2345[] = {{&S29137[42], 19}, {NULL}};
const Segment SGI2346[] = {{&S29137[30], 4}, {NULL}};
const Segment SGI2347[] = {{&S29137[109], 3}, {NULL}};
const Segment SGI2348[] = {{&S29137[75], 5}, {NULL}};
const Segment SGI2349[] = {{&S29138[42], 11}, {NULL}};
const Segment SGI2350[] = {{&S29138[42], 22}, {NULL}};
const Segment SGI2351[] = {{&S29138[30], 4}, {NULL}};
const Segment SGI2352[] = {{&S29138[112], 3}, {NULL}};
const Segment SGI2353[] = {{&S29138[78], 5}, {NULL}};
const Segment SGI2354[] = {{&S29139[42], 19}, {NULL}};
const Segment SGI2355[] = {{&S29139[42], 30}, {NULL}};
const Segment SGI2356[] = {{&S29139[30], 4}, {NULL}};
const Segment SGI2357[] = {{&S29139[120], 3}, {NULL}};
const Segment SGI2358[] = {{&S29139[86], 5}, {NULL}};
const Segment SGI2359[] = {{&S29140[42], 10}, {NULL}};
const Segment SGI2360[] = {{&S29140[42], 21}, {NULL}};
const Segment SGI2361[] = {{&S29140[30], 4}, {NULL}};
const Segment SGI2362[] = {{&S29140[111], 3}, {NULL}};
const Segment SGI2363[] = {{&S29140[77], 5}, {NULL}};
const Segment SGI2364[] = {{&S29144[42], 18}, {NULL}};
const Segment SGI2365[] = {{&S29144[42], 29}, {NULL}};
const Segment SGI2366[] = {{&S29144[30], 4}, {NULL}};
const Segment SGI2367[] = {{&S29144[119], 3}, {NULL}};
const Segment SGI2368[] = {{&S29144[85], 5}, {NULL}};
const Segment SGI2369[] = {{&S29145[42], 26}, {NULL}};
const Segment SGI2370[] = {{&S29145[42], 39}, {NULL}};
const Segment SGI2371[] = {{&S29145[30], 4}, {NULL}};
const Segment SGI2372[] = {{&S29145[77], 3}, {NULL}};
const Segment SGI2373[] = {{&S29145[95], 5}, {NULL}};
const Segment SGI2374[] = {{&S29146[42], 7}, {NULL}};
const Segment SGI2375[] = {{&S29146[42], 49}, {NULL}};
const Segment SGI2376[] = {{&S29146[30], 4}, {NULL}};
const Segment SGI2377[] = {{&S29146[139], 3}, {NULL}};
const Segment SGI2378[] = {{&S29146[105], 5}, {NULL}};
const Segment SGI2379[] = {{&S29159[42], 10}, {NULL}};
const Segment SGI2380[] = {{&S29159[42], 22}, {NULL}};
const Segment SGI2381[] = {{&S29159[30], 4}, {NULL}};
const Segment SGI2382[] = {{&S29159[112], 3}, {NULL}};
const Segment SGI2383[] = {{&S29159[78], 5}, {NULL}};
const Segment SGI2384[] = {{&S29166[42], 9}, {NULL}};
const Segment SGI2385[] = {{&S29166[42], 21}, {NULL}};
const Segment SGI2386[] = {{&S29166[30], 4}, {NULL}};
const Segment SGI2387[] = {{&S29166[111], 3}, {NULL}};
const Segment SGI2388[] = {{&S29166[77], 5}, {NULL}};
const Segment SGI2389[] = {{&S29168[42], 16}, {NULL}};
const Segment SGI2390[] = {{&S29168[42], 27}, {NULL}};
const Segment SGI2391[] = {{&S29168[30], 4}, {NULL}};
const Segment SGI2392[] = {{&S29168[117], 3}, {NULL}};
const Segment SGI2393[] = {{&S29168[83], 5}, {NULL}};
const Segment SGI2394[] = {{&S29175[42], 8}, {NULL}};
const Segment SGI2395[] = {{&S29175[42], 25}, {NULL}};
const Segment SGI2396[] = {{&S29175[30], 4}, {NULL}};
const Segment SGI2397[] = {{&S29175[115], 3}, {NULL}};
const Segment SGI2398[] = {{&S29175[81], 5}, {NULL}};
const Segment SGI2399[] = {{&S29176[42], 32}, {NULL}};
const Segment SGI2400[] = {{&S29176[30], 4}, {NULL}};
const Segment SGI2401[] = {{&S29182[40], 14}, {NULL}};
const Segment SGI2402[] = {{&S29182[30], 4}, {NULL}};
const Segment SGI2403[] = {{&S29182[113], 3}, {NULL}};
const Segment SGI2404[] = {{&S29182[79], 5}, {NULL}};
const Segment SGI2405[] = {{&S29183[42], 9}, {NULL}};
const Segment SGI2406[] = {{&S29183[42], 21}, {NULL}};
const Segment SGI2407[] = {{&S29183[30], 4}, {NULL}};
const Segment SGI2408[] = {{&S29183[112], 3}, {NULL}};
const Segment SGI2409[] = {{&S29183[77], 6}, {NULL}};
const Segment SGI2410[] = {{&S29184[42], 7}, {NULL}};
const Segment SGI2411[] = {{&S29184[42], 19}, {NULL}};
const Segment SGI2412[] = {{&S29184[30], 4}, {NULL}};
const Segment SGI2413[] = {{&S29184[109], 3}, {NULL}};
const Segment SGI2414[] = {{&S29184[75], 5}, {NULL}};
const Segment SGI2415[] = {{&S29185[42], 7}, {NULL}};
const Segment SGI2416[] = {{&S29185[42], 18}, {NULL}};
const Segment SGI2417[] = {{&S29185[30], 4}, {NULL}};
const Segment SGI2418[] = {{&S29185[108], 3}, {NULL}};
const Segment SGI2419[] = {{&S29185[74], 5}, {NULL}};
const Segment SGI2420[] = {{&S29190[42], 12}, {NULL}};
const Segment SGI2421[] = {{&S29190[42], 23}, {NULL}};
const Segment SGI2422[] = {{&S29190[30], 4}, {NULL}};
const Segment SGI2423[] = {{&S29190[113], 3}, {NULL}};
const Segment SGI2424[] = {{&S29190[79], 5}, {NULL}};
const Segment SGI2425[] = {{&S29195[42], 7}, {NULL}};
const Segment SGI2426[] = {{&S29195[42], 18}, {NULL}};
const Segment SGI2427[] = {{&S29195[30], 4}, {NULL}};
const Segment SGI2428[] = {{&S29195[108], 3}, {NULL}};
const Segment SGI2429[] = {{&S29195[74], 5}, {NULL}};
const Segment SGI2430[] = {{&S29196[42], 7}, {NULL}};
const Segment SGI2431[] = {{&S29196[42], 19}, {NULL}};
const Segment SGI2432[] = {{&S29196[30], 4}, {NULL}};
const Segment SGI2433[] = {{&S29196[109], 3}, {NULL}};
const Segment SGI2434[] = {{&S29196[75], 5}, {NULL}};
const Segment SGI2435[] = {{&S29198[42], 8}, {NULL}};
const Segment SGI2436[] = {{&S29198[42], 20}, {NULL}};
const Segment SGI2437[] = {{&S29198[30], 4}, {NULL}};
const Segment SGI2438[] = {{&S29198[110], 3}, {NULL}};
const Segment SGI2439[] = {{&S29198[76], 5}, {NULL}};
const Segment SGI2440[] = {{&S29215[42], 26}, {NULL}};
const Segment SGI2441[] = {{&S29215[42], 39}, {NULL}};
const Segment SGI2442[] = {{&S29215[30], 4}, {NULL}};
const Segment SGI2443[] = {{&S29215[129], 3}, {NULL}};
const Segment SGI2444[] = {{&S29215[95], 5}, {NULL}};
const Segment SGI2445[] = {{&S29216[42], 7}, {NULL}};
const Segment SGI2446[] = {{&S29216[42], 18}, {NULL}};
const Segment SGI2447[] = {{&S29216[30], 4}, {NULL}};
const Segment SGI2448[] = {{&S29216[108], 3}, {NULL}};
const Segment SGI2449[] = {{&S29216[74], 5}, {NULL}};
const Segment SGI2450[] = {{&S29220[42], 17}, {NULL}};
const Segment SGI2451[] = {{&S29220[42], 28}, {NULL}};
const Segment SGI2452[] = {{&S29220[30], 4}, {NULL}};
const Segment SGI2453[] = {{&S29220[118], 3}, {NULL}};
const Segment SGI2454[] = {{&S29220[84], 5}, {NULL}};
const Segment SGI2455[] = {{&S29222[42], 10}, {NULL}};
const Segment SGI2456[] = {{&S29222[42], 21}, {NULL}};
const Segment SGI2457[] = {{&S29222[30], 4}, {NULL}};
const Segment SGI2458[] = {{&S29222[111], 3}, {NULL}};
const Segment SGI2459[] = {{&S29222[77], 5}, {NULL}};
const Segment SGI2460[] = {{&S29225[42], 15}, {NULL}};
const Segment SGI2461[] = {{&S29225[42], 26}, {NULL}};
const Segment SGI2462[] = {{&S29225[30], 4}, {NULL}};
const Segment SGI2463[] = {{&S29225[116], 3}, {NULL}};
const Segment SGI2464[] = {{&S29225[82], 5}, {NULL}};
const Segment SGI2465[] = {{&S29226[36], 11}, {NULL}};
const Segment SGI2466[] = {{&S29226[31], 4}, {NULL}};
const Segment SGI2467[] = {{&S29230[42], 13}, {NULL}};
const Segment SGI2468[] = {{&S29230[42], 24}, {NULL}};
const Segment SGI2469[] = {{&S29230[30], 4}, {NULL}};
const Segment SGI2470[] = {{&S29230[114], 3}, {NULL}};
const Segment SGI2471[] = {{&S29230[80], 5}, {NULL}};
const Segment SGI2472[] = {{&S29232[42], 9}, {NULL}};
const Segment SGI2473[] = {{&S29232[42], 20}, {NULL}};
const Segment SGI2474[] = {{&S29232[30], 4}, {NULL}};
const Segment SGI2475[] = {{&S29232[110], 3}, {NULL}};
const Segment SGI2476[] = {{&S29232[76], 5}, {NULL}};
const Segment SGI2477[] = {{&S29233[36], 9}, {NULL}};
const Segment SGI2478[] = {{&S29233[31], 4}, {NULL}};
const Segment SGI2479[] = {{&S29234[42], 13}, {NULL}};
const Segment SGI2480[] = {{&S29234[42], 24}, {NULL}};
const Segment SGI2481[] = {{&S29234[30], 4}, {NULL}};
const Segment SGI2482[] = {{&S29234[114], 3}, {NULL}};
const Segment SGI2483[] = {{&S29234[80], 5}, {NULL}};
const Segment SGI2484[] = {{&S29235[42], 13}, {NULL}};
const Segment SGI2485[] = {{&S29235[42], 24}, {NULL}};
const Segment SGI2486[] = {{&S29235[30], 4}, {NULL}};
const Segment SGI2487[] = {{&S29235[114], 3}, {NULL}};
const Segment SGI2488[] = {{&S29235[80], 5}, {NULL}};
const Segment SGI2489[] = {{&S29236[42], 12}, {NULL}};
const Segment SGI2490[] = {{&S29236[42], 23}, {NULL}};
const Segment SGI2491[] = {{&S29236[30], 4}, {&S29236[50], 15}, {NULL}};
const Segment SGI2492[] = {{&S29236[113], 3}, {NULL}};
const Segment SGI2493[] = {{&S29236[79], 5}, {NULL}};
const Segment SGI2494[] = {{&S29237[42], 8}, {NULL}};
const Segment SGI2495[] = {{&S29237[42], 20}, {NULL}};
const Segment SGI2496[] = {{&S29237[30], 4}, {NULL}};
const Segment SGI2497[] = {{&S29237[110], 3}, {NULL}};
const Segment SGI2498[] = {{&S29237[76], 5}, {NULL}};
const Segment SGI2499[] = {{&S29244[42], 7}, {NULL}};
const Segment SGI2500[] = {{&S29244[42], 18}, {NULL}};
const Segment SGI2501[] = {{&S29244[30], 4}, {NULL}};
const Segment SGI2502[] = {{&S29244[74], 5}, {NULL}};
const Segment SGI2503[] = {{&S29251[36], 4}, {NULL}};
const Segment SGI2504[] = {{&S29251[31], 4}, {NULL}};
const Segment SGI2505[] = {{&S29255[42], 32}, {NULL}};
const Segment SGI2506[] = {{&S29255[30], 4}, {NULL}};
const Segment SGI2507[] = {{&S29256[42], 9}, {NULL}};
const Segment SGI2508[] = {{&S29256[42], 20}, {NULL}};
const Segment SGI2509[] = {{&S29256[30], 4}, {NULL}};
const Segment SGI2510[] = {{&S29256[110], 3}, {NULL}};
const Segment SGI2511[] = {{&S29256[76], 5}, {NULL}};
const Segment SGI2512[] = {{&S29259[50], 4}, {NULL}};
const Segment SGI2513[] = {{&S29259[103], 5}, {NULL}};
const Segment SGI2514[] = {{&S29268[42], 7}, {NULL}};
const Segment SGI2515[] = {{&S29268[42], 19}, {NULL}};
const Segment SGI2516[] = {{&S29268[30], 4}, {NULL}};
const Segment SGI2517[] = {{&S29268[109], 3}, {NULL}};
const Segment SGI2518[] = {{&S29268[75], 5}, {NULL}};
const Segment SGI2519[] = {{&S29274[42], 29}, {NULL}};
const Segment SGI2520[] = {{&S29274[30], 4}, {NULL}};
const Segment SGI2521[] = {{&S29274[137], 3}, {NULL}};
const Segment SGI2522[] = {{&S29274[103], 5}, {NULL}};
const Segment SGI2523[] = {{&S29284[42], 8}, {NULL}};
const Segment SGI2524[] = {{&S29284[42], 28}, {NULL}};
const Segment SGI2525[] = {{&S29284[30], 4}, {NULL}};
const Segment SGI2526[] = {{&S29284[118], 3}, {NULL}};
const Segment SGI2527[] = {{&S29284[84], 5}, {NULL}};
const Segment SGI2528[] = {{&S29286[42], 10}, {NULL}};
const Segment SGI2529[] = {{&S29286[42], 21}, {NULL}};
const Segment SGI2530[] = {{&S29286[30], 4}, {NULL}};
const Segment SGI2531[] = {{&S29286[80], 4}, {NULL}};
const Segment SGI2532[] = {{&S29286[80], 3}, {NULL}};
const Segment SGI2533[] = {{&S29296[42], 8}, {NULL}};
const Segment SGI2534[] = {{&S29296[42], 20}, {NULL}};
const Segment SGI2535[] = {{&S29296[30], 4}, {NULL}};
const Segment SGI2536[] = {{&S29296[110], 3}, {NULL}};
const Segment SGI2537[] = {{&S29296[76], 5}, {NULL}};
const Segment SGI2538[] = {{&S29298[42], 8}, {NULL}};
const Segment SGI2539[] = {{&S29298[42], 20}, {NULL}};
const Segment SGI2540[] = {{&S29298[30], 4}, {NULL}};
const Segment SGI2541[] = {{&S29298[110], 3}, {NULL}};
const Segment SGI2542[] = {{&S29298[76], 5}, {NULL}};
const Segment SGI2543[] = {{&S29300[49], 8}, {NULL}};
const Segment SGI2544[] = {{&S29300[49], 45}, {NULL}};
const Segment SGI2545[] = {{&S29300[30], 11}, {NULL}};
const Segment SGI2546[] = {{&S29300[142], 3}, {NULL}};
const Segment SGI2547[] = {{&S29300[108], 5}, {NULL}};
const Segment SGI2548[] = {{&S29301[42], 11}, {NULL}};
const Segment SGI2549[] = {{&S29301[42], 22}, {NULL}};
const Segment SGI2550[] = {{&S29301[30], 4}, {NULL}};
const Segment SGI2551[] = {{&S29301[112], 3}, {NULL}};
const Segment SGI2552[] = {{&S29301[78], 5}, {NULL}};
const Segment SGI2553[] = {{&S29328[42], 32}, {NULL}};
const Segment SGI2554[] = {{&S29328[30], 4}, {NULL}};
const Segment SGI2555[] = {{&S29333[42], 6}, {NULL}};
const Segment SGI2556[] = {{&S29333[42], 17}, {NULL}};
const Segment SGI2557[] = {{&S29333[30], 4}, {NULL}};
const Segment SGI2558[] = {{&S29333[107], 3}, {NULL}};
const Segment SGI2559[] = {{&S29333[73], 5}, {NULL}};
const Segment SGI2560[] = {{&S29335[42], 12}, {NULL}};
const Segment SGI2561[] = {{&S29335[42], 24}, {NULL}};
const Segment SGI2562[] = {{&S29335[30], 4}, {NULL}};
const Segment SGI2563[] = {{&S29335[114], 3}, {NULL}};
const Segment SGI2564[] = {{&S29335[80], 5}, {NULL}};
const Segment SGI2565[] = {{&S29337[42], 12}, {NULL}};
const Segment SGI2566[] = {{&S29337[42], 23}, {NULL}};
const Segment SGI2567[] = {{&S29337[30], 4}, {NULL}};
const Segment SGI2568[] = {{&S29337[113], 3}, {NULL}};
const Segment SGI2569[] = {{&S29337[79], 5}, {NULL}};
const Segment SGI2570[] = {{&S29338[42], 9}, {NULL}};
const Segment SGI2571[] = {{&S29338[42], 20}, {NULL}};
const Segment SGI2572[] = {{&S29338[30], 4}, {NULL}};
const Segment SGI2573[] = {{&S29338[110], 3}, {NULL}};
const Segment SGI2574[] = {{&S29338[76], 5}, {NULL}};
const Segment SGI2575[] = {{&S29339[42], 8}, {NULL}};
const Segment SGI2576[] = {{&S29339[42], 28}, {NULL}};
const Segment SGI2577[] = {{&S29339[30], 4}, {NULL}};
const Segment SGI2578[] = {{&S29339[118], 3}, {NULL}};
const Segment SGI2579[] = {{&S29339[84], 5}, {NULL}};
const Segment SGI2580[] = {{&S29343[42], 26}, {NULL}};
const Segment SGI2581[] = {{&S29343[42], 39}, {NULL}};
const Segment SGI2582[] = {{&S29343[30], 4}, {NULL}};
const Segment SGI2583[] = {{&S29343[129], 3}, {NULL}};
const Segment SGI2584[] = {{&S29343[95], 5}, {NULL}};
const Segment SGI2585[] = {{&S29345[42], 9}, {NULL}};
const Segment SGI2586[] = {{&S29345[42], 21}, {NULL}};
const Segment SGI2587[] = {{&S29345[30], 4}, {NULL}};
const Segment SGI2588[] = {{&S29345[111], 3}, {NULL}};
const Segment SGI2589[] = {{&S29345[77], 5}, {NULL}};
const Segment SGI2590[] = {{&S29350[42], 15}, {NULL}};
const Segment SGI2591[] = {{&S29350[42], 26}, {NULL}};
const Segment SGI2592[] = {{&S29350[30], 4}, {NULL}};
const Segment SGI2593[] = {{&S29350[117], 3}, {NULL}};
const Segment SGI2594[] = {{&S29350[82], 6}, {NULL}};
const Segment SGI2595[] = {{&S29352[42], 7}, {NULL}};
const Segment SGI2596[] = {{&S29352[42], 49}, {NULL}};
const Segment SGI2597[] = {{&S29352[30], 4}, {NULL}};
const Segment SGI2598[] = {{&S29352[139], 3}, {NULL}};
const Segment SGI2599[] = {{&S29352[105], 5}, {NULL}};
const Segment SGI2600[] = {{&S29353[42], 10}, {NULL}};
const Segment SGI2601[] = {{&S29353[42], 22}, {NULL}};
const Segment SGI2602[] = {{&S29353[30], 4}, {NULL}};
const Segment SGI2603[] = {{&S29353[112], 3}, {NULL}};
const Segment SGI2604[] = {{&S29353[78], 5}, {NULL}};
const Segment SGI2605[] = {{&S29354[42], 7}, {NULL}};
const Segment SGI2606[] = {{&S29354[42], 19}, {NULL}};
const Segment SGI2607[] = {{&S29354[30], 4}, {NULL}};
const Segment SGI2608[] = {{&S29354[109], 3}, {NULL}};
const Segment SGI2609[] = {{&S29354[75], 5}, {NULL}};
const Segment SGI2610[] = {{&S29361[42], 12}, {NULL}};
const Segment SGI2611[] = {{&S29361[42], 23}, {NULL}};
const Segment SGI2612[] = {{&S29361[30], 4}, {NULL}};
const Segment SGI2613[] = {{&S29361[113], 3}, {NULL}};
const Segment SGI2614[] = {{&S29361[79], 5}, {NULL}};
const Segment SGI2615[] = {{&S29364[42], 10}, {NULL}};
const Segment SGI2616[] = {{&S29364[42], 22}, {NULL}};
const Segment SGI2617[] = {{&S29364[30], 4}, {NULL}};
const Segment SGI2618[] = {{&S29364[112], 3}, {NULL}};
const Segment SGI2619[] = {{&S29364[78], 5}, {NULL}};
const Segment SGI2620[] = {{&S29369[42], 12}, {NULL}};
const Segment SGI2621[] = {{&S29369[42], 23}, {NULL}};
const Segment SGI2622[] = {{&S29369[30], 4}, {NULL}};
const Segment SGI2623[] = {{&S29369[113], 3}, {NULL}};
const Segment SGI2624[] = {{&S29369[79], 5}, {NULL}};
const Segment SGI2625[] = {{&S29375[99], 4}, {NULL}};
const Segment SGI2626[] = {{&S29375[151], 3}, {NULL}};
const Segment SGI2627[] = {{&S29375[117], 5}, {NULL}};
const Segment SGI2628[] = {{&S29380[44], 9}, {NULL}};
const Segment SGI2629[] = {{&S29380[44], 20}, {NULL}};
const Segment SGI2630[] = {{&S29380[30], 6}, {NULL}};
const Segment SGI2631[] = {{&S29380[112], 3}, {NULL}};
const Segment SGI2632[] = {{&S29380[78], 5}, {NULL}};
const Segment SGI2633[] = {{&S29388[42], 9}, {NULL}};
const Segment SGI2634[] = {{&S29388[42], 20}, {NULL}};
const Segment SGI2635[] = {{&S29388[30], 4}, {NULL}};
const Segment SGI2636[] = {{&S29388[110], 3}, {NULL}};
const Segment SGI2637[] = {{&S29388[76], 5}, {NULL}};
const Segment SGI2638[] = {{&S29395[42], 6}, {NULL}};
const Segment SGI2639[] = {{&S29395[42], 18}, {NULL}};
const Segment SGI2640[] = {{&S29395[30], 4}, {NULL}};
const Segment SGI2641[] = {{&S29395[108], 3}, {NULL}};
const Segment SGI2642[] = {{&S29395[74], 5}, {NULL}};
const Segment SGI2643[] = {{&S29402[42], 12}, {NULL}};
const Segment SGI2644[] = {{&S29402[42], 23}, {NULL}};
const Segment SGI2645[] = {{&S29402[30], 4}, {NULL}};
const Segment SGI2646[] = {{&S29402[113], 3}, {NULL}};
const Segment SGI2647[] = {{&S29402[79], 5}, {NULL}};
const Segment SGI2648[] = {{&S29412[42], 11}, {NULL}};
const Segment SGI2649[] = {{&S29412[42], 22}, {NULL}};
const Segment SGI2650[] = {{&S29412[30], 4}, {NULL}};
const Segment SGI2651[] = {{&S29412[112], 3}, {NULL}};
const Segment SGI2652[] = {{&S29412[78], 5}, {NULL}};
const Segment SGI2653[] = {{&S29414[42], 9}, {NULL}};
const Segment SGI2654[] = {{&S29414[42], 20}, {NULL}};
const Segment SGI2655[] = {{&S29414[30], 4}, {NULL}};
const Segment SGI2656[] = {{&S29414[110], 3}, {NULL}};
const Segment SGI2657[] = {{&S29414[76], 5}, {NULL}};
const Segment SGI2658[] = {{&S29416[31], 4}, {NULL}};
const Segment SGI2659[] = {{&S29424[42], 8}, {NULL}};
const Segment SGI2660[] = {{&S29424[42], 20}, {NULL}};
const Segment SGI2661[] = {{&S29424[30], 4}, {NULL}};
const Segment SGI2662[] = {{&S29424[110], 3}, {NULL}};
const Segment SGI2663[] = {{&S29424[76], 5}, {NULL}};
const Segment SGI2664[] = {{&S29433[42], 14}, {NULL}};
const Segment SGI2665[] = {{&S29433[42], 26}, {NULL}};
const Segment SGI2666[] = {{&S29433[30], 4}, {NULL}};
const Segment SGI2667[] = {{&S29433[116], 3}, {NULL}};
const Segment SGI2668[] = {{&S29433[82], 5}, {NULL}};
const Segment SGI2669[] = {{&S29438[42], 6}, {NULL}};
const Segment SGI2670[] = {{&S29438[42], 17}, {NULL}};
const Segment SGI2671[] = {{&S29438[30], 4}, {NULL}};
const Segment SGI2672[] = {{&S29438[107], 3}, {NULL}};
const Segment SGI2673[] = {{&S29438[73], 5}, {NULL}};
const Segment SGI2674[] = {{&S32000[50], 6}, {NULL}};
const Segment SGI2675[] = {{&S32000[50], 24}, {NULL}};
const Segment SGI2676[] = {{&S32000[30], 12}, {NULL}};
const Segment SGI2677[] = {{&S32000[123], 3}, {NULL}};
const Segment SGI2678[] = {{&S32000[88], 6}, {NULL}};
const Segment SGI2679[] = {{&S32001[50], 6}, {NULL}};
const Segment SGI2680[] = {{&S32001[50], 23}, {NULL}};
const Segment SGI2681[] = {{&S32001[30], 12}, {NULL}};
const Segment SGI2682[] = {{&S32001[122], 3}, {NULL}};
const Segment SGI2683[] = {{&S32001[87], 6}, {NULL}};
const Segment SGI2684[] = {{&S32003[50], 6}, {NULL}};
const Segment SGI2685[] = {{&S32003[50], 23}, {NULL}};
const Segment SGI2686[] = {{&S32003[30], 12}, {NULL}};
const Segment SGI2687[] = {{&S32003[122], 3}, {NULL}};
const Segment SGI2688[] = {{&S32003[87], 6}, {NULL}};
const Segment SGI2689[] = {{&S32004[50], 6}, {NULL}};
const Segment SGI2690[] = {{&S32004[50], 23}, {NULL}};
const Segment SGI2691[] = {{&S32004[30], 12}, {NULL}};
const Segment SGI2692[] = {{&S32004[122], 3}, {NULL}};
const Segment SGI2693[] = {{&S32004[87], 6}, {NULL}};
const Segment SGI2694[] = {{&S32005[50], 6}, {NULL}};
const Segment SGI2695[] = {{&S32005[50], 24}, {NULL}};
const Segment SGI2696[] = {{&S32005[30], 12}, {NULL}};
const Segment SGI2697[] = {{&S32005[123], 3}, {NULL}};
const Segment SGI2698[] = {{&S32005[88], 6}, {NULL}};
const Segment SGI2699[] = {{&S32006[50], 6}, {NULL}};
const Segment SGI2700[] = {{&S32006[50], 23}, {NULL}};
const Segment SGI2701[] = {{&S32006[30], 12}, {NULL}};
const Segment SGI2702[] = {{&S32006[122], 3}, {NULL}};
const Segment SGI2703[] = {{&S32006[87], 6}, {NULL}};
const Segment SGI2704[] = {{&S32007[50], 6}, {NULL}};
const Segment SGI2705[] = {{&S32007[50], 24}, {NULL}};
const Segment SGI2706[] = {{&S32007[30], 12}, {NULL}};
const Segment SGI2707[] = {{&S32007[123], 3}, {NULL}};
const Segment SGI2708[] = {{&S32007[88], 6}, {NULL}};
const Segment SGI2709[] = {{&S32009[50], 6}, {NULL}};
const Segment SGI2710[] = {{&S32009[50], 24}, {NULL}};
const Segment SGI2711[] = {{&S32009[30], 12}, {NULL}};
const Segment SGI2712[] = {{&S32009[123], 3}, {NULL}};
const Segment SGI2713[] = {{&S32009[88], 6}, {NULL}};
const Segment SGI2714[] = {{&S32010[44], 6}, {NULL}};
const Segment SGI2715[] = {{&S32010[31], 12}, {NULL}};
const Segment SGI2716[] = {{&S32011[50], 6}, {NULL}};
const Segment SGI2717[] = {{&S32011[50], 23}, {NULL}};
const Segment SGI2718[] = {{&S32011[30], 12}, {NULL}};
const Segment SGI2719[] = {{&S32011[90], 4}, {NULL}};
const Segment SGI2720[] = {{&S32011[90], 3}, {NULL}};
const Segment SGI2721[] = {{&S32012[50], 6}, {NULL}};
const Segment SGI2722[] = {{&S32012[50], 24}, {NULL}};
const Segment SGI2723[] = {{&S32012[30], 12}, {NULL}};
const Segment SGI2724[] = {{&S32012[91], 4}, {NULL}};
const Segment SGI2725[] = {{&S32012[91], 3}, {NULL}};
const Segment SGI2726[] = {{&S31703[56], 20}, {NULL}};
const Segment SGI2727[] = {{&S31703[56], 32}, {NULL}};
const Segment SGI2728[] = {{&S31703[30], 18}, {NULL}};
const Segment SGI2729[] = {{&S31703[136], 3}, {NULL}};
const Segment SGI2730[] = {{&S31703[102], 5}, {NULL}};
const Segment SGI2731[] = {{&S31704[42], 20}, {NULL}};
const Segment SGI2732[] = {{&S31704[30], 6}, {NULL}};
const Segment SGI2733[] = {{&S31704[122], 3}, {NULL}};
const Segment SGI2734[] = {{&S31704[88], 5}, {NULL}};
const Segment SGI2735[] = {{&S31705[44], 20}, {NULL}};
const Segment SGI2736[] = {{&S31705[44], 32}, {NULL}};
const Segment SGI2737[] = {{&S31705[30], 6}, {NULL}};
const Segment SGI2738[] = {{&S31705[124], 3}, {NULL}};
const Segment SGI2739[] = {{&S31705[90], 5}, {NULL}};
const Segment SGI2740[] = {{&S32115[44], 9}, {NULL}};
const Segment SGI2741[] = {{&S32115[44], 20}, {NULL}};
const Segment SGI2742[] = {{&S32115[30], 6}, {NULL}};
const Segment SGI2743[] = {{&S32115[112], 3}, {NULL}};
const Segment SGI2744[] = {{&S32115[78], 5}, {NULL}};
const Segment SGI2745[] = {{&S32116[44], 16}, {NULL}};
const Segment SGI2746[] = {{&S32116[44], 27}, {NULL}};
const Segment SGI2747[] = {{&S32116[30], 6}, {NULL}};
const Segment SGI2748[] = {{&S32116[119], 3}, {NULL}};
const Segment SGI2749[] = {{&S32116[85], 5}, {NULL}};
const Segment SGI2750[] = {{&S32122[44], 14}, {NULL}};
const Segment SGI2751[] = {{&S32122[30], 6}, {NULL}};
const Segment SGI2752[] = {{&S32122[106], 3}, {NULL}};
const Segment SGI2753[] = {{&S32122[72], 5}, {NULL}};
const Segment SGI2754[] = {{&S32134[38], 9}, {NULL}};
const Segment SGI2755[] = {{&S32134[31], 6}, {NULL}};
const Segment SGI2756[] = {{&S32135[44], 9}, {NULL}};
const Segment SGI2757[] = {{&S32135[44], 20}, {NULL}};
const Segment SGI2758[] = {{&S32135[30], 6}, {NULL}};
const Segment SGI2759[] = {{&S32135[81], 4}, {NULL}};
const Segment SGI2760[] = {{&S32135[81], 3}, {NULL}};
const Segment SGI2761[] = {{&S32324[22], 9}, {NULL}};
const Segment SGI2762[] = {{&S32324[44], 21}, {NULL}};
const Segment SGI2763[] = {{&S32324[30], 6}, {&S32324[52], 13}, {NULL}};
const Segment SGI2764[] = {{&S32324[113], 3}, {NULL}};
const Segment SGI2765[] = {{&S32324[79], 5}, {NULL}};
const Segment SGI2766[] = {{&S32328[44], 14}, {NULL}};
const Segment SGI2767[] = {{&S32328[44], 26}, {NULL}};
const Segment SGI2768[] = {{&S32328[30], 6}, {NULL}};
const Segment SGI2769[] = {{&S32328[118], 3}, {NULL}};
const Segment SGI2770[] = {{&S32328[84], 5}, {NULL}};
const Segment SGI2771[] = {{&S32329[44], 10}, {NULL}};
const Segment SGI2772[] = {{&S32329[44], 21}, {NULL}};
const Segment SGI2773[] = {{&S32329[30], 6}, {NULL}};
const Segment SGI2774[] = {{&S32329[113], 3}, {NULL}};
const Segment SGI2775[] = {{&S32329[79], 5}, {NULL}};
const Segment SGI2776[] = {{&S32339[23], 9}, {NULL}};
const Segment SGI2777[] = {{&S32339[31], 6}, {&S32339[46], 13}, {NULL}};
const Segment SGI2778[] = {{&S32346[44], 12}, {NULL}};
const Segment SGI2779[] = {{&S32346[44], 24}, {NULL}};
const Segment SGI2780[] = {{&S32346[30], 6}, {NULL}};
const Segment SGI2781[] = {{&S32346[116], 3}, {NULL}};
const Segment SGI2782[] = {{&S32346[82], 5}, {NULL}};
const Segment SGI2783[] = {{&S32350[43], 21}, {NULL}};
const Segment SGI2784[] = {{&S32350[111], 3}, {NULL}};
const Segment SGI2785[] = {{&S32350[78], 5}, {NULL}};
const Segment SGI2786[] = {{&S32352[44], 5}, {NULL}};
const Segment SGI2787[] = {{&S32352[44], 17}, {NULL}};
const Segment SGI2788[] = {{&S32352[30], 6}, {NULL}};
const Segment SGI2789[] = {{&S32352[109], 3}, {NULL}};
const Segment SGI2790[] = {{&S32352[75], 5}, {NULL}};
const Segment SGI2791[] = {{&S32358[44], 7}, {NULL}};
const Segment SGI2792[] = {{&S32358[44], 19}, {NULL}};
const Segment SGI2793[] = {{&S32358[30], 6}, {NULL}};
const Segment SGI2794[] = {{&S32358[111], 3}, {NULL}};
const Segment SGI2795[] = {{&S32358[77], 5}, {NULL}};
const Segment SGI2796[] = {{&S32363[44], 8}, {NULL}};
const Segment SGI2797[] = {{&S32363[44], 20}, {NULL}};
const Segment SGI2798[] = {{&S32363[30], 6}, {NULL}};
const Segment SGI2799[] = {{&S32363[112], 3}, {NULL}};
const Segment SGI2800[] = {{&S32363[78], 5}, {NULL}};
const Segment SGI2801[] = {{&S32368[44], 13}, {NULL}};
const Segment SGI2802[] = {{&S32368[44], 24}, {NULL}};
const Segment SGI2803[] = {{&S32368[30], 6}, {NULL}};
const Segment SGI2804[] = {{&S32368[116], 3}, {NULL}};
const Segment SGI2805[] = {{&S32368[82], 5}, {NULL}};
const Segment SGI2806[] = {{&S32369[38], 14}, {NULL}};
const Segment SGI2807[] = {{&S32369[31], 6}, {NULL}};
const Segment SGI2808[] = {{&S32371[30], 6}, {NULL}};
const Segment SGI2809[] = {{&S32371[91], 3}, {NULL}};
const Segment SGI2810[] = {{&S32371[57], 5}, {NULL}};
const Segment SGI2811[] = {{&S32373[44], 9}, {NULL}};
const Segment SGI2812[] = {{&S32373[44], 21}, {NULL}};
const Segment SGI2813[] = {{&S32373[30], 6}, {NULL}};
const Segment SGI2814[] = {{&S32373[113], 3}, {NULL}};
const Segment SGI2815[] = {{&S32373[79], 5}, {NULL}};
const Segment SGI2816[] = {{&S32382[44], 12}, {NULL}};
const Segment SGI2817[] = {{&S32382[44], 24}, {NULL}};
const Segment SGI2818[] = {{&S32382[30], 6}, {NULL}};
const Segment SGI2819[] = {{&S32382[116], 3}, {NULL}};
const Segment SGI2820[] = {{&S32382[82], 5}, {NULL}};
const Segment SGI2821[] = {{&S32388[44], 8}, {NULL}};
const Segment SGI2822[] = {{&S32388[44], 20}, {NULL}};
const Segment SGI2823[] = {{&S32388[30], 6}, {NULL}};
const Segment SGI2824[] = {{&S32388[112], 3}, {NULL}};
const Segment SGI2825[] = {{&S32388[78], 5}, {NULL}};
const Segment SGI2826[] = {{&S32392[44], 7}, {NULL}};
const Segment SGI2827[] = {{&S32392[44], 18}, {NULL}};
const Segment SGI2828[] = {{&S32392[30], 6}, {NULL}};
const Segment SGI2829[] = {{&S32392[110], 3}, {NULL}};
const Segment SGI2830[] = {{&S32392[76], 5}, {NULL}};
const Segment SGI2831[] = {{&S32399[44], 8}, {NULL}};
const Segment SGI2832[] = {{&S32399[44], 19}, {NULL}};
const Segment SGI2833[] = {{&S32399[30], 6}, {NULL}};
const Segment SGI2834[] = {{&S32399[111], 3}, {NULL}};
const Segment SGI2835[] = {{&S32399[77], 5}, {NULL}};
const Segment SGI2836[] = {{&S32400[30], 6}, {NULL}};
const Segment SGI2837[] = {{&S32400[104], 3}, {NULL}};
const Segment SGI2838[] = {{&S32400[57], 6}, {NULL}};
const Segment SGI2839[] = {{&S32403[44], 9}, {NULL}};
const Segment SGI2840[] = {{&S32403[44], 20}, {NULL}};
const Segment SGI2841[] = {{&S32403[30], 6}, {NULL}};
const Segment SGI2842[] = {{&S32403[112], 3}, {NULL}};
const Segment SGI2843[] = {{&S32403[78], 5}, {NULL}};
const Segment SGI2844[] = {{&S33214[48], 6}, {NULL}};
const Segment SGI2845[] = {{&S33214[30], 12}, {NULL}};
const Segment SGI2846[] = {{&S33214[120], 3}, {NULL}};
const Segment SGI2847[] = {{&S33214[85], 6}, {NULL}};
const Segment SGI2848[] = {{&S33215[50], 6}, {NULL}};
const Segment SGI2849[] = {{&S33215[50], 24}, {NULL}};
const Segment SGI2850[] = {{&S33215[30], 12}, {NULL}};
const Segment SGI2851[] = {{&S33215[123], 3}, {NULL}};
const Segment SGI2852[] = {{&S33215[88], 6}, {NULL}};
const Segment SGI2853[] = {{&S33216[86], 6}, {NULL}};
const Segment SGI2854[] = {{&S33216[86], 24}, {NULL}};
const Segment SGI2855[] = {{&S33216[30], 48}, {NULL}};
const Segment SGI2856[] = {{&S33216[159], 3}, {NULL}};
const Segment SGI2857[] = {{&S33216[124], 6}, {NULL}};
const Segment SGI2858[] = {{&S33219[46], 5}, {NULL}};
const Segment SGI2859[] = {{&S33219[26], 12}, {NULL}};
const Segment SGI2860[] = {{&S33219[88], 3}, {NULL}};
const Segment SGI2861[] = {{&S33219[65], 6}, {NULL}};
const Segment SGI2862[] = {{&S33220[50], 6}, {NULL}};
const Segment SGI2863[] = {{&S33220[50], 23}, {NULL}};
const Segment SGI2864[] = {{&S33220[30], 12}, {NULL}};
const Segment SGI2865[] = {{&S33220[122], 3}, {NULL}};
const Segment SGI2866[] = {{&S33220[87], 6}, {NULL}};
const Segment SGI2867[] = {{&S33221[53], 6}, {NULL}};
const Segment SGI2868[] = {{&S33221[53], 24}, {NULL}};
const Segment SGI2869[] = {{&S33221[30], 15}, {NULL}};
const Segment SGI2870[] = {{&S33221[126], 3}, {NULL}};
const Segment SGI2871[] = {{&S33221[91], 6}, {NULL}};
const Segment SGI2872[] = {{&S33222[50], 6}, {NULL}};
const Segment SGI2873[] = {{&S33222[50], 24}, {NULL}};
const Segment SGI2874[] = {{&S33222[30], 12}, {NULL}};
const Segment SGI2875[] = {{&S33222[123], 3}, {NULL}};
const Segment SGI2876[] = {{&S33222[88], 6}, {NULL}};
const Segment SGI2877[] = {{&S33223[45], 4}, {NULL}};
const Segment SGI2878[] = {{&S33223[94], 3}, {NULL}};
const Segment SGI2879[] = {{&S33223[71], 6}, {NULL}};
const Segment SGI2880[] = {{&S33224[50], 6}, {NULL}};
const Segment SGI2881[] = {{&S33224[50], 21}, {NULL}};
const Segment SGI2882[] = {{&S33224[30], 12}, {NULL}};
const Segment SGI2883[] = {{&S33224[120], 3}, {NULL}};
const Segment SGI2884[] = {{&S33224[85], 6}, {NULL}};
const Segment SGI2885[] = {{&S33225[50], 6}, {NULL}};
const Segment SGI2886[] = {{&S33225[50], 23}, {NULL}};
const Segment SGI2887[] = {{&S33225[30], 12}, {NULL}};
const Segment SGI2888[] = {{&S33225[122], 3}, {NULL}};
const Segment SGI2889[] = {{&S33225[87], 6}, {NULL}};
const Segment SGI2890[] = {{&S33226[44], 6}, {NULL}};
const Segment SGI2891[] = {{&S33226[31], 12}, {NULL}};
const Segment SGI2892[] = {{&S33228[50], 6}, {NULL}};
const Segment SGI2893[] = {{&S33228[50], 23}, {NULL}};
const Segment SGI2894[] = {{&S33228[30], 12}, {NULL}};
const Segment SGI2895[] = {{&S33228[90], 4}, {NULL}};
const Segment SGI2896[] = {{&S33228[90], 3}, {NULL}};
const Segment SGI2897[] = {{&S33229[50], 6}, {NULL}};
const Segment SGI2898[] = {{&S33229[50], 23}, {NULL}};
const Segment SGI2899[] = {{&S33229[30], 12}, {NULL}};
const Segment SGI2900[] = {{&S33229[122], 3}, {NULL}};
const Segment SGI2901[] = {{&S33229[87], 6}, {NULL}};
const Segment SGI2902[] = {{&S33230[50], 6}, {NULL}};
const Segment SGI2903[] = {{&S33230[50], 23}, {NULL}};
const Segment SGI2904[] = {{&S33230[30], 12}, {NULL}};
const Segment SGI2905[] = {{&S33230[90], 4}, {NULL}};
const Segment SGI2906[] = {{&S33230[90], 3}, {NULL}};
const Segment SGI2907[] = {{&S33231[50], 6}, {NULL}};
const Segment SGI2908[] = {{&S33231[50], 24}, {NULL}};
const Segment SGI2909[] = {{&S33231[30], 12}, {NULL}};
const Segment SGI2910[] = {{&S33231[123], 3}, {NULL}};
const Segment SGI2911[] = {{&S33231[88], 6}, {NULL}};
const Segment SGI2912[] = {{&S33232[50], 6}, {NULL}};
const Segment SGI2913[] = {{&S33232[50], 23}, {NULL}};
const Segment SGI2914[] = {{&S33232[30], 12}, {NULL}};
const Segment SGI2915[] = {{&S33232[90], 6}, {NULL}};
const Segment SGI2916[] = {{&S32318[35], 7}, {NULL}};
const Segment SGI2917[] = {{&S32318[30], 4}, {NULL}};
const Segment SGI2918[] = {{&S32318[101], 3}, {NULL}};
const Segment SGI2919[] = {{&S32318[67], 5}, {NULL}};
const Segment SGI2920[] = {{&S32322[42], 11}, {NULL}};
const Segment SGI2921[] = {{&S32322[42], 23}, {NULL}};
const Segment SGI2922[] = {{&S32322[30], 4}, {NULL}};
const Segment SGI2923[] = {{&S32322[113], 3}, {NULL}};
const Segment SGI2924[] = {{&S32322[79], 5}, {NULL}};
const Segment SGI2925[] = {{&S32323[30], 4}, {NULL}};
const Segment SGI2926[] = {{&S32323[89], 3}, {NULL}};
const Segment SGI2927[] = {{&S32323[55], 5}, {NULL}};
const Segment SGI2928[] = {{&S32325[42], 9}, {NULL}};
const Segment SGI2929[] = {{&S32325[42], 20}, {NULL}};
const Segment SGI2930[] = {{&S32325[30], 4}, {NULL}};
const Segment SGI2931[] = {{&S32325[110], 3}, {NULL}};
const Segment SGI2932[] = {{&S32325[76], 5}, {NULL}};
const Segment SGI2933[] = {{&S32326[42], 20}, {NULL}};
const Segment SGI2934[] = {{&S32326[30], 4}, {NULL}};
const Segment SGI2935[] = {{&S32326[122], 3}, {NULL}};
const Segment SGI2936[] = {{&S32326[88], 5}, {NULL}};
const Segment SGI2937[] = {{&S32327[30], 4}, {NULL}};
const Segment SGI2938[] = {{&S32327[82], 3}, {NULL}};
const Segment SGI2939[] = {{&S32327[48], 5}, {NULL}};
const Segment SGI2940[] = {{&S32330[42], 9}, {NULL}};
const Segment SGI2941[] = {{&S32330[42], 21}, {NULL}};
const Segment SGI2942[] = {{&S32330[30], 4}, {NULL}};
const Segment SGI2943[] = {{&S32330[111], 3}, {NULL}};
const Segment SGI2944[] = {{&S32330[77], 5}, {NULL}};
const Segment SGI2945[] = {{&S32331[42], 20}, {NULL}};
const Segment SGI2946[] = {{&S32331[30], 4}, {NULL}};
const Segment SGI2947[] = {{&S32331[122], 3}, {NULL}};
const Segment SGI2948[] = {{&S32331[88], 5}, {NULL}};
const Segment SGI2949[] = {{&S32332[42], 20}, {NULL}};
const Segment SGI2950[] = {{&S32332[30], 4}, {NULL}};
const Segment SGI2951[] = {{&S32332[123], 3}, {NULL}};
const Segment SGI2952[] = {{&S32332[88], 6}, {NULL}};
const Segment SGI2953[] = {{&S32333[42], 5}, {NULL}};
const Segment SGI2954[] = {{&S32333[42], 17}, {NULL}};
const Segment SGI2955[] = {{&S32333[30], 4}, {NULL}};
const Segment SGI2956[] = {{&S32333[107], 3}, {NULL}};
const Segment SGI2957[] = {{&S32333[73], 5}, {NULL}};
const Segment SGI2958[] = {{&S32334[42], 20}, {NULL}};
const Segment SGI2959[] = {{&S32334[30], 4}, {NULL}};
const Segment SGI2960[] = {{&S32334[127], 3}, {NULL}};
const Segment SGI2961[] = {{&S32335[42], 13}, {NULL}};
const Segment SGI2962[] = {{&S32335[42], 24}, {NULL}};
const Segment SGI2963[] = {{&S32335[30], 4}, {NULL}};
const Segment SGI2964[] = {{&S32335[114], 3}, {NULL}};
const Segment SGI2965[] = {{&S32335[80], 5}, {NULL}};
const Segment SGI2966[] = {{&S32337[42], 9}, {NULL}};
const Segment SGI2967[] = {{&S32337[42], 28}, {NULL}};
const Segment SGI2968[] = {{&S32337[30], 4}, {NULL}};
const Segment SGI2969[] = {{&S32337[118], 3}, {NULL}};
const Segment SGI2970[] = {{&S32337[84], 5}, {NULL}};
const Segment SGI2971[] = {{&S32342[42], 14}, {NULL}};
const Segment SGI2972[] = {{&S32342[42], 25}, {NULL}};
const Segment SGI2973[] = {{&S32342[30], 4}, {NULL}};
const Segment SGI2974[] = {{&S32342[115], 3}, {NULL}};
const Segment SGI2975[] = {{&S32342[81], 5}, {NULL}};
const Segment SGI2976[] = {{&S32343[36], 9}, {NULL}};
const Segment SGI2977[] = {{&S32343[31], 4}, {NULL}};
const Segment SGI2978[] = {{&S32347[30], 4}, {NULL}};
const Segment SGI2979[] = {{&S32347[124], 3}, {NULL}};
const Segment SGI2980[] = {{&S32348[42], 5}, {NULL}};
const Segment SGI2981[] = {{&S32348[42], 17}, {NULL}};
const Segment SGI2982[] = {{&S32348[30], 4}, {NULL}};
const Segment SGI2983[] = {{&S32348[107], 3}, {NULL}};
const Segment SGI2984[] = {{&S32348[73], 5}, {NULL}};
const Segment SGI2985[] = {{&S32357[8], 3}, {NULL}};
const Segment SGI2986[] = {{&S32357[73], 8}, {NULL}};
const Segment SGI2987[] = {{&S32360[36], 9}, {NULL}};
const Segment SGI2988[] = {{&S32360[31], 4}, {NULL}};
const Segment SGI2989[] = {{&S33356[48], 10}, {NULL}};
const Segment SGI2990[] = {{&S33356[30], 12}, {NULL}};
const Segment SGI2991[] = {{&S33356[119], 3}, {NULL}};
const Segment SGI2992[] = {{&S33356[84], 6}, {NULL}};
const Segment SGI2993[] = {{&S32365[42], 9}, {NULL}};
const Segment SGI2994[] = {{&S32365[42], 20}, {NULL}};
const Segment SGI2995[] = {{&S32365[30], 4}, {NULL}};
const Segment SGI2996[] = {{&S32365[79], 4}, {NULL}};
const Segment SGI2997[] = {{&S33357[42], 10}, {NULL}};
const Segment SGI2998[] = {{&S32365[79], 3}, {NULL}};
const Segment SGI2999[] = {{&S33357[42], 21}, {NULL}};
const Segment SGI3000[] = {{&S33357[30], 4}, {NULL}};
const Segment SGI3001[] = {{&S33357[111], 3}, {NULL}};
const Segment SGI3002[] = {{&S33357[77], 5}, {NULL}};
const Segment SGI3003[] = {{&S32366[50], 5}, {NULL}};
const Segment SGI3004[] = {{&S32366[50], 16}, {NULL}};
const Segment SGI3005[] = {{&S32366[30], 12}, {NULL}};
const Segment SGI3006[] = {{&S32366[115], 3}, {NULL}};
const Segment SGI3007[] = {{&S32366[80], 6}, {NULL}};
const Segment SGI3008[] = {{&S33358[50], 11}, {NULL}};
const Segment SGI3009[] = {{&S33358[50], 23}, {NULL}};
const Segment SGI3010[] = {{&S33358[30], 12}, {NULL}};
const Segment SGI3011[] = {{&S33358[122], 3}, {NULL}};
const Segment SGI3012[] = {{&S33358[87], 6}, {NULL}};
const Segment SGI3013[] = {{&S32367[36], 13}, {NULL}};
const Segment SGI3014[] = {{&S32367[31], 4}, {NULL}};
const Segment SGI3015[] = {{&S33359[50], 11}, {NULL}};
const Segment SGI3016[] = {{&S33359[50], 23}, {NULL}};
const Segment SGI3017[] = {{&S33359[30], 12}, {NULL}};
const Segment SGI3018[] = {{&S33359[122], 3}, {NULL}};
const Segment SGI3019[] = {{&S33359[87], 6}, {NULL}};
const Segment SGI3020[] = {{&S33360[50], 10}, {NULL}};
const Segment SGI3021[] = {{&S33360[50], 22}, {NULL}};
const Segment SGI3022[] = {{&S33360[30], 12}, {NULL}};
const Segment SGI3023[] = {{&S33360[121], 3}, {NULL}};
const Segment SGI3024[] = {{&S33360[86], 6}, {NULL}};
const Segment SGI3025[] = {{&S33361[50], 11}, {NULL}};
const Segment SGI3026[] = {{&S33361[50], 23}, {NULL}};
const Segment SGI3027[] = {{&S33361[30], 12}, {NULL}};
const Segment SGI3028[] = {{&S33361[122], 3}, {NULL}};
const Segment SGI3029[] = {{&S33361[87], 6}, {NULL}};
const Segment SGI3030[] = {{&S33361[87], 5}, {NULL}};
const Segment SGI3031[] = {{&S33362[44], 10}, {NULL}};
const Segment SGI3032[] = {{&S33362[31], 12}, {NULL}};
const Segment SGI3033[] = {{&S32374[42], 51}, {NULL}};
const Segment SGI3034[] = {{&S32374[30], 4}, {&S32374[50], 43}, {NULL}};
const Segment SGI3035[] = {{&S32374[155], 3}, {NULL}};
const Segment SGI3036[] = {{&S32374[121], 5}, {NULL}};
const Segment SGI3037[] = {{&S33363[44], 11}, {NULL}};
const Segment SGI3038[] = {{&S33363[31], 12}, {NULL}};
const Segment SGI3039[] = {{&S33364[50], 10}, {NULL}};
const Segment SGI3040[] = {{&S33364[50], 22}, {NULL}};
const Segment SGI3041[] = {{&S33364[30], 12}, {NULL}};
const Segment SGI3042[] = {{&S33364[89], 4}, {NULL}};
const Segment SGI3043[] = {{&S33364[89], 3}, {NULL}};
const Segment SGI3044[] = {{&S33365[50], 8}, {NULL}};
const Segment SGI3045[] = {{&S32377[42], 5}, {NULL}};
const Segment SGI3046[] = {{&S33365[50], 20}, {NULL}};
const Segment SGI3047[] = {{&S32377[42], 17}, {NULL}};
const Segment SGI3048[] = {{&S33365[30], 12}, {NULL}};
const Segment SGI3049[] = {{&S32377[30], 4}, {NULL}};
const Segment SGI3050[] = {{&S32377[108], 3}, {NULL}};
const Segment SGI3051[] = {{&S33365[119], 3}, {NULL}};
const Segment SGI3052[] = {{&S32377[73], 6}, {NULL}};
const Segment SGI3053[] = {{&S33365[84], 6}, {NULL}};
const Segment SGI3054[] = {{&S33366[57], 10}, {NULL}};
const Segment SGI3055[] = {{&S33366[57], 22}, {NULL}};
const Segment SGI3056[] = {{&S33366[45], 4}, {NULL}};
const Segment SGI3057[] = {{&S33366[129], 3}, {NULL}};
const Segment SGI3058[] = {{&S33366[93], 7}, {NULL}};
const Segment SGI3059[] = {{&S33366[149], 8}, {NULL}};
const Segment SGI3060[] = {{&S33367[44], 10}, {NULL}};
const Segment SGI3061[] = {{&S33367[44], 27}, {NULL}};
const Segment SGI3062[] = {{&S33367[30], 6}, {NULL}};
const Segment SGI3063[] = {{&S33367[119], 3}, {NULL}};
const Segment SGI3064[] = {{&S33367[85], 5}, {NULL}};
const Segment SGI3065[] = {{&S32385[40], 13}, {NULL}};
const Segment SGI3066[] = {{&S32385[30], 4}, {NULL}};
const Segment SGI3067[] = {{&S32385[112], 3}, {NULL}};
const Segment SGI3068[] = {{&S32385[78], 5}, {NULL}};
const Segment SGI3069[] = {{&S32387[11], 1}, {NULL}};
const Segment SGI3070[] = {{&S32387[42], 12}, {NULL}};
const Segment SGI3071[] = {{&S32387[30], 4}, {NULL}};
const Segment SGI3072[] = {{&S32387[102], 3}, {NULL}};
const Segment SGI3073[] = {{&S32387[68], 5}, {NULL}};
const Segment SGI3074[] = {{&S32391[42], 7}, {NULL}};
const Segment SGI3075[] = {{&S32391[42], 57}, {NULL}};
const Segment SGI3076[] = {{&S32391[30], 4}, {NULL}};
const Segment SGI3077[] = {{&S32391[147], 3}, {NULL}};
const Segment SGI3078[] = {{&S32391[113], 5}, {NULL}};
const Segment SGI3079[] = {{&S32393[42], 11}, {NULL}};
const Segment SGI3080[] = {{&S32393[42], 22}, {NULL}};
const Segment SGI3081[] = {{&S32393[30], 4}, {NULL}};
const Segment SGI3082[] = {{&S32393[112], 3}, {NULL}};
const Segment SGI3083[] = {{&S32393[78], 5}, {NULL}};
const Segment SGI3084[] = {{&S32394[42], 14}, {NULL}};
const Segment SGI3085[] = {{&S32394[42], 25}, {NULL}};
const Segment SGI3086[] = {{&S32394[30], 4}, {NULL}};
const Segment SGI3087[] = {{&S32394[115], 3}, {NULL}};
const Segment SGI3088[] = {{&S32394[81], 5}, {NULL}};
const Segment SGI3089[] = {{&S32397[42], 9}, {NULL}};
const Segment SGI3090[] = {{&S32397[42], 20}, {NULL}};
const Segment SGI3091[] = {{&S32397[30], 4}, {NULL}};
const Segment SGI3092[] = {{&S32397[110], 3}, {NULL}};
const Segment SGI3093[] = {{&S32397[76], 5}, {NULL}};
const Segment SGI3094[] = {{&S32401[42], 13}, {NULL}};
const Segment SGI3095[] = {{&S32401[42], 24}, {NULL}};
const Segment SGI3096[] = {{&S32401[30], 4}, {NULL}};
const Segment SGI3097[] = {{&S32401[114], 3}, {NULL}};
const Segment SGI3098[] = {{&S32401[80], 5}, {NULL}};
const Segment SGI3099[] = {{&S32468[41], 9}, {NULL}};
const Segment SGI3100[] = {{&S32468[41], 20}, {NULL}};
const Segment SGI3101[] = {{&S32468[30], 3}, {NULL}};
const Segment SGI3102[] = {{&S32468[109], 3}, {NULL}};
const Segment SGI3103[] = {{&S32468[75], 5}, {NULL}};
const Segment SGI3104[] = {{&S32469[42], 8}, {NULL}};
const Segment SGI3105[] = {{&S32469[42], 19}, {NULL}};
const Segment SGI3106[] = {{&S32469[30], 4}, {NULL}};
const Segment SGI3107[] = {{&S32469[109], 3}, {NULL}};
const Segment SGI3108[] = {{&S32469[75], 5}, {NULL}};
const Segment SGI3109[] = {{&S32470[42], 8}, {NULL}};
const Segment SGI3110[] = {{&S32470[42], 19}, {NULL}};
const Segment SGI3111[] = {{&S32470[30], 4}, {NULL}};
const Segment SGI3112[] = {{&S32470[109], 3}, {NULL}};
const Segment SGI3113[] = {{&S32470[75], 5}, {NULL}};
const Segment SGI3114[] = {{&S32471[42], 9}, {NULL}};
const Segment SGI3115[] = {{&S32471[42], 20}, {NULL}};
const Segment SGI3116[] = {{&S32471[30], 4}, {NULL}};
const Segment SGI3117[] = {{&S32471[112], 3}, {NULL}};
const Segment SGI3118[] = {{&S32471[76], 7}, {NULL}};
const Segment SGI3119[] = {{&S32471[132], 8}, {NULL}};
const Segment SGI3120[] = {{&S32472[42], 9}, {NULL}};
const Segment SGI3121[] = {{&S32472[42], 20}, {NULL}};
const Segment SGI3122[] = {{&S32472[30], 4}, {NULL}};
const Segment SGI3123[] = {{&S32472[110], 3}, {NULL}};
const Segment SGI3124[] = {{&S32472[76], 5}, {NULL}};
const Segment SGI3125[] = {{&S32473[41], 9}, {NULL}};
const Segment SGI3126[] = {{&S32473[41], 20}, {NULL}};
const Segment SGI3127[] = {{&S32473[109], 3}, {NULL}};
const Segment SGI3128[] = {{&S32473[75], 5}, {NULL}};
const Segment SGI3129[] = {{&S32476[40], 9}, {NULL}};
const Segment SGI3130[] = {{&S32476[30], 4}, {NULL}};
const Segment SGI3131[] = {{&S32476[108], 3}, {NULL}};
const Segment SGI3132[] = {{&S32476[74], 5}, {NULL}};
const Segment SGI3133[] = {{&S32481[42], 9}, {NULL}};
const Segment SGI3134[] = {{&S32481[42], 20}, {NULL}};
const Segment SGI3135[] = {{&S32481[30], 4}, {NULL}};
const Segment SGI3136[] = {{&S32481[110], 3}, {NULL}};
const Segment SGI3137[] = {{&S32481[76], 5}, {NULL}};
const Segment SGI3138[] = {{&S32482[36], 9}, {NULL}};
const Segment SGI3139[] = {{&S32482[31], 4}, {NULL}};
const Segment SGI3140[] = {{&S32805[44], 10}, {NULL}};
const Segment SGI3141[] = {{&S32805[44], 21}, {NULL}};
const Segment SGI3142[] = {{&S32805[30], 6}, {NULL}};
const Segment SGI3143[] = {{&S32805[113], 3}, {NULL}};
const Segment SGI3144[] = {{&S32805[79], 5}, {NULL}};
const Segment SGI3145[] = {{&S32810[44], 7}, {NULL}};
const Segment SGI3146[] = {{&S32810[44], 18}, {NULL}};
const Segment SGI3147[] = {{&S32810[30], 6}, {NULL}};
const Segment SGI3148[] = {{&S32810[110], 3}, {NULL}};
const Segment SGI3149[] = {{&S32810[76], 5}, {NULL}};
const Segment SGI3150[] = {{&S32811[44], 7}, {NULL}};
const Segment SGI3151[] = {{&S32811[44], 24}, {NULL}};
const Segment SGI3152[] = {{&S32811[30], 6}, {NULL}};
const Segment SGI3153[] = {{&S32811[116], 3}, {NULL}};
const Segment SGI3154[] = {{&S32811[82], 5}, {NULL}};
const Segment SGI3155[] = {{&S32812[44], 10}, {NULL}};
const Segment SGI3156[] = {{&S32812[44], 27}, {NULL}};
const Segment SGI3157[] = {{&S32812[30], 6}, {NULL}};
const Segment SGI3158[] = {{&S32812[119], 3}, {NULL}};
const Segment SGI3159[] = {{&S32812[85], 5}, {NULL}};
const Segment SGI3160[] = {{&S32814[38], 7}, {NULL}};
const Segment SGI3161[] = {{&S32814[31], 6}, {NULL}};
const Segment SGI3162[] = {{&S32815[38], 10}, {NULL}};
const Segment SGI3163[] = {{&S32815[31], 6}, {NULL}};
const Segment SGI3164[] = {{&S32908[44], 14}, {NULL}};
const Segment SGI3165[] = {{&S32908[44], 25}, {NULL}};
const Segment SGI3166[] = {{&S32908[30], 6}, {NULL}};
const Segment SGI3167[] = {{&S32908[117], 3}, {NULL}};
const Segment SGI3168[] = {{&S32908[83], 5}, {NULL}};
const Segment SGI3169[] = {{&S32909[44], 14}, {NULL}};
const Segment SGI3170[] = {{&S32909[44], 25}, {NULL}};
const Segment SGI3171[] = {{&S32909[30], 6}, {NULL}};
const Segment SGI3172[] = {{&S32909[117], 3}, {NULL}};
const Segment SGI3173[] = {{&S32909[83], 5}, {NULL}};
const Segment SGI3174[] = {{&S32911[44], 14}, {NULL}};
const Segment SGI3175[] = {{&S32911[44], 25}, {NULL}};
const Segment SGI3176[] = {{&S32911[30], 6}, {NULL}};
const Segment SGI3177[] = {{&S32911[117], 3}, {NULL}};
const Segment SGI3178[] = {{&S32911[83], 5}, {NULL}};
const Segment SGI3179[] = {{&S32912[43], 14}, {NULL}};
const Segment SGI3180[] = {{&S32912[43], 25}, {NULL}};
const Segment SGI3181[] = {{&S32912[116], 3}, {NULL}};
const Segment SGI3182[] = {{&S32912[82], 5}, {NULL}};
const Segment SGI3183[] = {{&S32916[44], 9}, {NULL}};
const Segment SGI3184[] = {{&S32916[44], 20}, {NULL}};
const Segment SGI3185[] = {{&S32916[30], 6}, {NULL}};
const Segment SGI3186[] = {{&S32916[112], 3}, {NULL}};
const Segment SGI3187[] = {{&S32916[78], 5}, {NULL}};
const Segment SGI3188[] = {{&S32918[44], 9}, {NULL}};
const Segment SGI3189[] = {{&S32918[44], 20}, {NULL}};
const Segment SGI3190[] = {{&S32918[30], 6}, {NULL}};
const Segment SGI3191[] = {{&S32920[44], 14}, {NULL}};
const Segment SGI3192[] = {{&S32920[44], 25}, {NULL}};
const Segment SGI3193[] = {{&S32920[30], 6}, {NULL}};
const Segment SGI3194[] = {{&S32920[117], 3}, {NULL}};
const Segment SGI3195[] = {{&S32920[83], 5}, {NULL}};
const Segment SGI3196[] = {{&S32922[38], 9}, {NULL}};
const Segment SGI3197[] = {{&S32922[31], 6}, {NULL}};
const Segment SGI3198[] = {{&S32924[44], 10}, {NULL}};
const Segment SGI3199[] = {{&S32924[44], 21}, {NULL}};
const Segment SGI3200[] = {{&S32924[30], 6}, {NULL}};
const Segment SGI3201[] = {{&S32924[113], 3}, {NULL}};
const Segment SGI3202[] = {{&S32924[79], 5}, {NULL}};
const Segment SGI3203[] = {{&S32968[44], 25}, {NULL}};
const Segment SGI3204[] = {{&S32968[44], 37}, {NULL}};
const Segment SGI3205[] = {{&S32968[30], 6}, {NULL}};
const Segment SGI3206[] = {{&S32968[129], 3}, {NULL}};
const Segment SGI3207[] = {{&S32968[95], 5}, {NULL}};
const Segment SGI3208[] = {{&S32969[44], 25}, {NULL}};
const Segment SGI3209[] = {{&S32969[44], 37}, {NULL}};
const Segment SGI3210[] = {{&S32969[30], 6}, {NULL}};
const Segment SGI3211[] = {{&S32969[129], 3}, {NULL}};
const Segment SGI3212[] = {{&S32969[95], 5}, {NULL}};
const Segment SGI3213[] = {{&S32970[44], 25}, {NULL}};
const Segment SGI3214[] = {{&S32970[44], 37}, {NULL}};
const Segment SGI3215[] = {{&S32970[30], 6}, {NULL}};
const Segment SGI3216[] = {{&S32970[129], 3}, {NULL}};
const Segment SGI3217[] = {{&S32970[95], 5}, {NULL}};
const Segment SGI3218[] = {{&S32972[44], 14}, {NULL}};
const Segment SGI3219[] = {{&S32972[44], 26}, {NULL}};
const Segment SGI3220[] = {{&S32972[30], 6}, {NULL}};
const Segment SGI3221[] = {{&S32972[118], 3}, {NULL}};
const Segment SGI3222[] = {{&S32972[84], 5}, {NULL}};
const Segment SGI3223[] = {{&S32973[44], 25}, {NULL}};
const Segment SGI3224[] = {{&S32973[44], 37}, {NULL}};
const Segment SGI3225[] = {{&S32973[30], 6}, {NULL}};
const Segment SGI3226[] = {{&S32973[129], 3}, {NULL}};
const Segment SGI3227[] = {{&S32973[95], 5}, {NULL}};
const Segment SGI3228[] = {{&S32974[44], 25}, {NULL}};
const Segment SGI3229[] = {{&S32974[44], 37}, {NULL}};
const Segment SGI3230[] = {{&S32974[30], 6}, {NULL}};
const Segment SGI3231[] = {{&S32974[129], 3}, {NULL}};
const Segment SGI3232[] = {{&S32974[95], 5}, {NULL}};
const Segment SGI3233[] = {{&S32975[44], 25}, {NULL}};
const Segment SGI3234[] = {{&S32975[44], 37}, {NULL}};
const Segment SGI3235[] = {{&S32975[30], 6}, {NULL}};
const Segment SGI3236[] = {{&S32975[129], 3}, {NULL}};
const Segment SGI3237[] = {{&S32975[95], 5}, {NULL}};
const Segment SGI3238[] = {{&S32976[44], 25}, {NULL}};
const Segment SGI3239[] = {{&S32976[44], 37}, {NULL}};
const Segment SGI3240[] = {{&S32976[30], 6}, {NULL}};
const Segment SGI3241[] = {{&S32976[129], 3}, {NULL}};
const Segment SGI3242[] = {{&S32976[95], 5}, {NULL}};
const Segment SGI3243[] = {{&S32977[44], 25}, {NULL}};
const Segment SGI3244[] = {{&S32977[44], 37}, {NULL}};
const Segment SGI3245[] = {{&S32977[30], 6}, {NULL}};
const Segment SGI3246[] = {{&S32977[129], 3}, {NULL}};
const Segment SGI3247[] = {{&S32977[95], 5}, {NULL}};
const Segment SGI3248[] = {{&S32978[44], 25}, {NULL}};
const Segment SGI3249[] = {{&S32978[44], 37}, {NULL}};
const Segment SGI3250[] = {{&S32978[30], 6}, {NULL}};
const Segment SGI3251[] = {{&S32978[129], 3}, {NULL}};
const Segment SGI3252[] = {{&S32978[95], 5}, {NULL}};
const Segment SGI3253[] = {{&S32979[44], 25}, {NULL}};
const Segment SGI3254[] = {{&S32979[44], 37}, {NULL}};
const Segment SGI3255[] = {{&S32979[30], 6}, {NULL}};
const Segment SGI3256[] = {{&S32979[129], 3}, {NULL}};
const Segment SGI3257[] = {{&S32979[95], 5}, {NULL}};
const Segment SGI3258[] = {{&S32980[44], 25}, {NULL}};
const Segment SGI3259[] = {{&S32980[44], 37}, {NULL}};
const Segment SGI3260[] = {{&S32980[30], 6}, {NULL}};
const Segment SGI3261[] = {{&S32980[129], 3}, {NULL}};
const Segment SGI3262[] = {{&S32980[95], 5}, {NULL}};
const Segment SGI3263[] = {{&S32981[44], 25}, {NULL}};
const Segment SGI3264[] = {{&S32981[44], 37}, {NULL}};
const Segment SGI3265[] = {{&S32981[30], 6}, {NULL}};
const Segment SGI3266[] = {{&S32981[129], 3}, {NULL}};
const Segment SGI3267[] = {{&S32981[95], 5}, {NULL}};
const Segment SGI3268[] = {{&S32982[44], 25}, {NULL}};
const Segment SGI3269[] = {{&S32982[44], 37}, {NULL}};
const Segment SGI3270[] = {{&S32982[30], 6}, {NULL}};
const Segment SGI3271[] = {{&S32982[129], 3}, {NULL}};
const Segment SGI3272[] = {{&S32982[95], 5}, {NULL}};
const Segment SGI3273[] = {{&S32983[44], 25}, {NULL}};
const Segment SGI3274[] = {{&S32983[44], 37}, {NULL}};
const Segment SGI3275[] = {{&S32983[30], 6}, {NULL}};
const Segment SGI3276[] = {{&S32983[129], 3}, {NULL}};
const Segment SGI3277[] = {{&S32983[95], 5}, {NULL}};
const Segment SGI3278[] = {{&S32984[44], 24}, {NULL}};
const Segment SGI3279[] = {{&S32984[44], 36}, {NULL}};
const Segment SGI3280[] = {{&S32984[30], 6}, {NULL}};
const Segment SGI3281[] = {{&S32984[128], 3}, {NULL}};
const Segment SGI3282[] = {{&S32984[94], 5}, {NULL}};
const Segment SGI3283[] = {{&S32985[44], 25}, {NULL}};
const Segment SGI3284[] = {{&S32985[44], 37}, {NULL}};
const Segment SGI3285[] = {{&S32985[30], 6}, {NULL}};
const Segment SGI3286[] = {{&S32985[129], 3}, {NULL}};
const Segment SGI3287[] = {{&S32985[95], 5}, {NULL}};
const Segment SGI3288[] = {{&S32986[42], 14}, {NULL}};
const Segment SGI3289[] = {{&S32986[30], 6}, {NULL}};
const Segment SGI3290[] = {{&S32986[116], 3}, {NULL}};
const Segment SGI3291[] = {{&S32986[82], 5}, {NULL}};
const Segment SGI3292[] = {{&S32987[44], 25}, {NULL}};
const Segment SGI3293[] = {{&S32987[44], 37}, {NULL}};
const Segment SGI3294[] = {{&S32987[30], 6}, {NULL}};
const Segment SGI3295[] = {{&S32987[129], 3}, {NULL}};
const Segment SGI3296[] = {{&S32987[95], 5}, {NULL}};
const Segment SGI3297[] = {{&S32988[44], 14}, {NULL}};
const Segment SGI3298[] = {{&S32988[44], 25}, {NULL}};
const Segment SGI3299[] = {{&S32988[30], 6}, {NULL}};
const Segment SGI3300[] = {{&S32988[117], 3}, {NULL}};
const Segment SGI3301[] = {{&S32988[83], 5}, {NULL}};
const Segment SGI3302[] = {{&S32989[57], 14}, {NULL}};
const Segment SGI3303[] = {{&S32989[57], 26}, {NULL}};
const Segment SGI3304[] = {{&S32989[30], 19}, {NULL}};
const Segment SGI3305[] = {{&S32989[131], 3}, {NULL}};
const Segment SGI3306[] = {{&S32989[97], 5}, {NULL}};
const Segment SGI3307[] = {{&S32990[38], 14}, {NULL}};
const Segment SGI3308[] = {{&S32990[31], 6}, {NULL}};
const Segment SGI3309[] = {{&S32991[44], 14}, {NULL}};
const Segment SGI3310[] = {{&S32991[44], 26}, {NULL}};
const Segment SGI3311[] = {{&S32991[30], 6}, {NULL}};
const Segment SGI3312[] = {{&S32991[87], 4}, {NULL}};
const Segment SGI3313[] = {{&S32991[87], 3}, {NULL}};
const Segment SGI3314[] = {{&S32992[44], 25}, {NULL}};
const Segment SGI3315[] = {{&S32992[44], 37}, {NULL}};
const Segment SGI3316[] = {{&S32992[30], 6}, {NULL}};
const Segment SGI3317[] = {{&S32992[129], 3}, {NULL}};
const Segment SGI3318[] = {{&S32992[95], 5}, {NULL}};
const Segment SGI3319[] = {{&S32994[56], 14}, {NULL}};
const Segment SGI3320[] = {{&S32994[56], 26}, {NULL}};
const Segment SGI3321[] = {{&S32994[30], 18}, {NULL}};
const Segment SGI3322[] = {{&S32994[130], 3}, {NULL}};
const Segment SGI3323[] = {{&S32994[96], 5}, {NULL}};
const Segment SGI3324[] = {{&S32996[44], 14}, {NULL}};
const Segment SGI3325[] = {{&S32996[44], 26}, {NULL}};
const Segment SGI3326[] = {{&S32996[30], 6}, {NULL}};
const Segment SGI3327[] = {{&S32996[87], 6}, {NULL}};
const Segment SGI3328[] = {{&S32997[44], 25}, {NULL}};
const Segment SGI3329[] = {{&S32997[44], 37}, {NULL}};
const Segment SGI3330[] = {{&S32997[30], 6}, {NULL}};
const Segment SGI3331[] = {{&S32997[129], 3}, {NULL}};
const Segment SGI3332[] = {{&S32997[95], 5}, {NULL}};
const Segment SGI3333[] = {{&S32998[44], 25}, {NULL}};
const Segment SGI3334[] = {{&S32998[44], 37}, {NULL}};
const Segment SGI3335[] = {{&S32998[30], 6}, {NULL}};
const Segment SGI3336[] = {{&S32998[129], 3}, {NULL}};
const Segment SGI3337[] = {{&S32998[95], 5}, {NULL}};
const Segment SGI3338[] = {{&S32999[44], 25}, {NULL}};
const Segment SGI3339[] = {{&S32999[44], 37}, {NULL}};
const Segment SGI3340[] = {{&S32999[30], 6}, {NULL}};
const Segment SGI3341[] = {{&S32999[129], 3}, {NULL}};
const Segment SGI3342[] = {{&S32999[95], 5}, {NULL}};
const Segment SGI3343[] = {{&S33001[44], 25}, {NULL}};
const Segment SGI3344[] = {{&S33001[44], 37}, {NULL}};
const Segment SGI3345[] = {{&S33001[30], 6}, {NULL}};
const Segment SGI3346[] = {{&S33001[129], 3}, {NULL}};
const Segment SGI3347[] = {{&S33001[95], 5}, {NULL}};
const Segment SGI3348[] = {{&S32910[42], 14}, {NULL}};
const Segment SGI3349[] = {{&S32910[42], 25}, {NULL}};
const Segment SGI3350[] = {{&S32910[30], 4}, {NULL}};
const Segment SGI3351[] = {{&S32910[115], 3}, {NULL}};
const Segment SGI3352[] = {{&S32910[81], 5}, {NULL}};
const Segment SGI3353[] = {{&S34550[47], 19}, {NULL}};
const Segment SGI3354[] = {{&S34550[47], 36}, {NULL}};
const Segment SGI3355[] = {{&S34550[30], 9}, {NULL}};
const Segment SGI3356[] = {{&S34550[36], 3}, {NULL}};
const Segment SGI3357[] = {{&S34550[97], 5}, {NULL}};
const Segment SGI3358[] = {{&S32971[55], 20}, {NULL}};
const Segment SGI3359[] = {{&S32971[43], 4}, {NULL}};
const Segment SGI3360[] = {{&S32971[124], 3}, {NULL}};
const Segment SGI3361[] = {{&S32971[89], 6}, {NULL}};
const Segment SGI3362[] = {{&S32993[42], 14}, {NULL}};
const Segment SGI3363[] = {{&S32993[42], 25}, {NULL}};
const Segment SGI3364[] = {{&S32993[30], 4}, {NULL}};
const Segment SGI3365[] = {{&S32993[115], 3}, {NULL}};
const Segment SGI3366[] = {{&S32993[81], 5}, {NULL}};
const Segment SGI3367[] = {{&S32995[42], 14}, {NULL}};
const Segment SGI3368[] = {{&S32995[42], 25}, {NULL}};
const Segment SGI3369[] = {{&S32995[30], 4}, {NULL}};
const Segment SGI3370[] = {{&S32995[84], 4}, {NULL}};
const Segment SGI3371[] = {{&S32995[84], 3}, {NULL}};
const Segment SGI3372[] = {{&S33000[42], 14}, {NULL}};
const Segment SGI3373[] = {{&S33000[42], 25}, {NULL}};
const Segment SGI3374[] = {{&S33000[30], 4}, {NULL}};
const Segment SGI3375[] = {{&S33000[84], 6}, {NULL}};
const Segment SGI3376[] = {{&S33682[44], 7}, {NULL}};
const Segment SGI3377[] = {{&S33682[44], 18}, {NULL}};
const Segment SGI3378[] = {{&S33682[30], 6}, {NULL}};
const Segment SGI3379[] = {{&S33682[110], 3}, {NULL}};
const Segment SGI3380[] = {{&S33682[76], 5}, {NULL}};
const Segment SGI3381[] = {{&S33683[44], 7}, {NULL}};
const Segment SGI3382[] = {{&S33683[44], 37}, {NULL}};
const Segment SGI3383[] = {{&S33683[30], 6}, {NULL}};
const Segment SGI3384[] = {{&S33683[129], 3}, {NULL}};
const Segment SGI3385[] = {{&S33683[95], 5}, {NULL}};
const Segment SGI3386[] = {{&S33685[42], 7}, {NULL}};
const Segment SGI3387[] = {{&S33685[30], 6}, {NULL}};
const Segment SGI3388[] = {{&S33685[108], 3}, {NULL}};
const Segment SGI3389[] = {{&S33685[74], 5}, {NULL}};
const Segment SGI3390[] = {{&S33686[42], 7}, {NULL}};
const Segment SGI3391[] = {{&S33686[30], 6}, {NULL}};
const Segment SGI3392[] = {{&S33686[110], 3}, {NULL}};
const Segment SGI3393[] = {{&S33686[74], 7}, {NULL}};
const Segment SGI3394[] = {{&S33686[130], 8}, {NULL}};
const Segment SGI3395[] = {{&S33687[38], 7}, {NULL}};
const Segment SGI3396[] = {{&S33687[31], 6}, {NULL}};
const Segment SGI3397[] = {{&S33690[44], 7}, {NULL}};
const Segment SGI3398[] = {{&S33690[44], 18}, {NULL}};
const Segment SGI3399[] = {{&S33690[30], 6}, {NULL}};
const Segment SGI3400[] = {{&S33690[112], 3}, {NULL}};
const Segment SGI3401[] = {{&S33690[76], 7}, {NULL}};
const Segment SGI3402[] = {{&S33690[132], 8}, {NULL}};
const Segment SGI3403[] = {{&S33695[44], 7}, {NULL}};
const Segment SGI3404[] = {{&S33695[44], 20}, {NULL}};
const Segment SGI3405[] = {{&S33695[30], 6}, {NULL}};
const Segment SGI3406[] = {{&S33695[112], 3}, {NULL}};
const Segment SGI3407[] = {{&S33695[78], 5}, {NULL}};
const Segment SGI3408[] = {{&S33700[63], 10}, {NULL}};
const Segment SGI3409[] = {{&S33700[63], 21}, {NULL}};
const Segment SGI3410[] = {{&S33700[30], 25}, {NULL}};
const Segment SGI3411[] = {{&S33700[132], 3}, {NULL}};
const Segment SGI3412[] = {{&S33700[98], 5}, {NULL}};
const Segment SGI3413[] = {{&S33702[44], 10}, {NULL}};
const Segment SGI3414[] = {{&S33702[44], 21}, {NULL}};
const Segment SGI3415[] = {{&S33702[30], 6}, {NULL}};
const Segment SGI3416[] = {{&S33702[113], 3}, {NULL}};
const Segment SGI3417[] = {{&S33702[79], 5}, {NULL}};
const Segment SGI3418[] = {{&S33703[72], 10}, {NULL}};
const Segment SGI3419[] = {{&S33703[72], 21}, {NULL}};
const Segment SGI3420[] = {{&S33703[30], 34}, {NULL}};
const Segment SGI3421[] = {{&S33703[141], 3}, {NULL}};
const Segment SGI3422[] = {{&S33703[107], 5}, {NULL}};
const Segment SGI3423[] = {{&S33706[62], 10}, {NULL}};
const Segment SGI3424[] = {{&S33706[62], 21}, {NULL}};
const Segment SGI3425[] = {{&S33706[30], 24}, {NULL}};
const Segment SGI3426[] = {{&S33706[131], 3}, {NULL}};
const Segment SGI3427[] = {{&S33706[97], 5}, {NULL}};
const Segment SGI3428[] = {{&S33707[44], 10}, {NULL}};
const Segment SGI3429[] = {{&S33707[44], 21}, {NULL}};
const Segment SGI3430[] = {{&S33707[30], 6}, {NULL}};
const Segment SGI3431[] = {{&S33707[115], 3}, {NULL}};
const Segment SGI3432[] = {{&S33707[79], 7}, {NULL}};
const Segment SGI3433[] = {{&S33707[135], 8}, {NULL}};
const Segment SGI3434[] = {{&S33708[38], 10}, {NULL}};
const Segment SGI3435[] = {{&S33708[31], 6}, {NULL}};
const Segment SGI3436[] = {{&S33711[42], 10}, {NULL}};
const Segment SGI3437[] = {{&S33711[30], 6}, {NULL}};
const Segment SGI3438[] = {{&S33711[111], 3}, {NULL}};
const Segment SGI3439[] = {{&S33711[77], 5}, {NULL}};
const Segment SGI3440[] = {{&S33689[42], 10}, {NULL}};
const Segment SGI3441[] = {{&S33689[42], 21}, {NULL}};
const Segment SGI3442[] = {{&S33689[30], 4}, {NULL}};
const Segment SGI3443[] = {{&S33689[111], 3}, {NULL}};
const Segment SGI3444[] = {{&S33689[77], 5}, {NULL}};
const Segment SGI3445[] = {{&S33691[42], 7}, {NULL}};
const Segment SGI3446[] = {{&S33691[42], 18}, {NULL}};
const Segment SGI3447[] = {{&S33691[30], 4}, {NULL}};
const Segment SGI3448[] = {{&S33691[109], 3}, {NULL}};
const Segment SGI3449[] = {{&S33691[74], 6}, {NULL}};
const Segment SGI3450[] = {{&S33701[42], 10}, {NULL}};
const Segment SGI3451[] = {{&S33701[42], 21}, {NULL}};
const Segment SGI3452[] = {{&S33701[30], 4}, {NULL}};
const Segment SGI3453[] = {{&S33701[111], 3}, {NULL}};
const Segment SGI3454[] = {{&S33701[77], 5}, {NULL}};
const Segment SGI3455[] = {{&S33705[40], 10}, {NULL}};
const Segment SGI3456[] = {{&S33705[30], 4}, {NULL}};
const Segment SGI3457[] = {{&S33705[109], 3}, {NULL}};
const Segment SGI3458[] = {{&S33705[75], 5}, {NULL}};
const Segment SGI3459[] = {{&S33709[36], 10}, {NULL}};
const Segment SGI3460[] = {{&S33709[31], 4}, {NULL}};
const Segment SGI3461[] = {{&S35572[64], 10}, {NULL}};
const Segment SGI3462[] = {{&S35572[64], 27}, {NULL}};
const Segment SGI3463[] = {{&S35572[30], 26}, {NULL}};
const Segment SGI3464[] = {{&S35572[139], 3}, {NULL}};
const Segment SGI3465[] = {{&S35572[105], 5}, {NULL}};
const Segment SGI3466[] = {{&S35574[54], 10}, {NULL}};
const Segment SGI3467[] = {{&S35574[54], 27}, {NULL}};
const Segment SGI3468[] = {{&S35574[30], 16}, {NULL}};
const Segment SGI3469[] = {{&S35574[129], 3}, {NULL}};
const Segment SGI3470[] = {{&S35574[95], 5}, {NULL}};
const Segment SGI3471[] = {{&S35576[62], 10}, {NULL}};
const Segment SGI3472[] = {{&S35576[62], 27}, {NULL}};
const Segment SGI3473[] = {{&S35576[30], 24}, {NULL}};
const Segment SGI3474[] = {{&S35576[137], 3}, {NULL}};
const Segment SGI3475[] = {{&S35576[103], 5}, {NULL}};
const Segment SGI3476[] = {{&S35577[56], 10}, {NULL}};
const Segment SGI3477[] = {{&S35577[56], 27}, {NULL}};
const Segment SGI3478[] = {{&S35577[30], 18}, {NULL}};
const Segment SGI3479[] = {{&S35577[131], 3}, {NULL}};
const Segment SGI3480[] = {{&S35577[97], 5}, {NULL}};
const Segment SGI3481[] = {{&S35583[61], 10}, {NULL}};
const Segment SGI3482[] = {{&S35583[61], 27}, {NULL}};
const Segment SGI3483[] = {{&S35583[30], 23}, {NULL}};
const Segment SGI3484[] = {{&S35583[136], 3}, {NULL}};
const Segment SGI3485[] = {{&S35583[102], 5}, {NULL}};
const Segment SGI3486[] = {{&S35584[71], 10}, {NULL}};
const Segment SGI3487[] = {{&S35584[71], 27}, {NULL}};
const Segment SGI3488[] = {{&S35584[30], 33}, {NULL}};
const Segment SGI3489[] = {{&S35584[146], 3}, {NULL}};
const Segment SGI3490[] = {{&S35584[112], 5}, {NULL}};
const Segment SGI3491[] = {{&S35586[61], 10}, {NULL}};
const Segment SGI3492[] = {{&S35586[61], 27}, {NULL}};
const Segment SGI3493[] = {{&S35586[30], 23}, {NULL}};
const Segment SGI3494[] = {{&S35586[136], 3}, {NULL}};
const Segment SGI3495[] = {{&S35586[102], 5}, {NULL}};
const Segment SGI3496[] = {{&S35015[50], 6}, {NULL}};
const Segment SGI3497[] = {{&S35015[50], 23}, {NULL}};
const Segment SGI3498[] = {{&S35015[30], 12}, {NULL}};
const Segment SGI3499[] = {{&S35015[122], 3}, {NULL}};
const Segment SGI3500[] = {{&S35015[87], 6}, {NULL}};
const Segment SGI3501[] = {{&S35016[50], 6}, {NULL}};
const Segment SGI3502[] = {{&S35016[50], 23}, {NULL}};
const Segment SGI3503[] = {{&S35016[30], 12}, {NULL}};
const Segment SGI3504[] = {{&S35016[122], 3}, {NULL}};
const Segment SGI3505[] = {{&S35016[87], 6}, {NULL}};
const Segment SGI3506[] = {{&S35017[50], 6}, {NULL}};
const Segment SGI3507[] = {{&S35017[50], 23}, {NULL}};
const Segment SGI3508[] = {{&S35017[30], 12}, {NULL}};
const Segment SGI3509[] = {{&S35017[122], 3}, {NULL}};
const Segment SGI3510[] = {{&S35017[87], 6}, {NULL}};
const Segment SGI3511[] = {{&S35018[45], 4}, {NULL}};
const Segment SGI3512[] = {{&S35018[94], 3}, {NULL}};
const Segment SGI3513[] = {{&S35018[71], 6}, {NULL}};
const Segment SGI3514[] = {{&S35019[64], 6}, {NULL}};
const Segment SGI3515[] = {{&S35019[64], 23}, {NULL}};
const Segment SGI3516[] = {{&S35019[30], 26}, {NULL}};
const Segment SGI3517[] = {{&S35019[136], 3}, {NULL}};
const Segment SGI3518[] = {{&S35019[101], 6}, {NULL}};
const Segment SGI3519[] = {{&S35020[46], 5}, {NULL}};
const Segment SGI3520[] = {{&S35020[26], 12}, {NULL}};
const Segment SGI3521[] = {{&S35020[88], 3}, {NULL}};
const Segment SGI3522[] = {{&S35020[65], 6}, {NULL}};
const Segment SGI3523[] = {{&S35021[50], 6}, {NULL}};
const Segment SGI3524[] = {{&S35021[50], 23}, {NULL}};
const Segment SGI3525[] = {{&S35021[30], 12}, {NULL}};
const Segment SGI3526[] = {{&S35021[122], 3}, {NULL}};
const Segment SGI3527[] = {{&S35021[87], 6}, {NULL}};
const Segment SGI3528[] = {{&S35022[50], 6}, {NULL}};
const Segment SGI3529[] = {{&S35022[50], 24}, {NULL}};
const Segment SGI3530[] = {{&S35022[30], 12}, {NULL}};
const Segment SGI3531[] = {{&S35022[123], 3}, {NULL}};
const Segment SGI3532[] = {{&S35022[88], 6}, {NULL}};
const Segment SGI3533[] = {{&S35024[50], 6}, {NULL}};
const Segment SGI3534[] = {{&S35024[50], 24}, {NULL}};
const Segment SGI3535[] = {{&S35024[30], 12}, {NULL}};
const Segment SGI3536[] = {{&S35024[123], 3}, {NULL}};
const Segment SGI3537[] = {{&S35024[88], 6}, {NULL}};
const Segment SGI3538[] = {{&S35025[44], 6}, {NULL}};
const Segment SGI3539[] = {{&S35025[31], 12}, {NULL}};
const Segment SGI3540[] = {{&S35026[50], 6}, {NULL}};
const Segment SGI3541[] = {{&S35026[50], 24}, {NULL}};
const Segment SGI3542[] = {{&S35026[30], 12}, {NULL}};
const Segment SGI3543[] = {{&S35026[123], 3}, {NULL}};
const Segment SGI3544[] = {{&S35026[88], 6}, {NULL}};
const Segment SGI3545[] = {{&S35027[50], 6}, {NULL}};
const Segment SGI3546[] = {{&S35027[50], 23}, {NULL}};
const Segment SGI3547[] = {{&S35027[30], 12}, {NULL}};
const Segment SGI3548[] = {{&S35027[90], 4}, {NULL}};
const Segment SGI3549[] = {{&S35027[90], 3}, {NULL}};
const Segment SGI3550[] = {{&S35028[50], 6}, {NULL}};
const Segment SGI3551[] = {{&S35028[50], 23}, {NULL}};
const Segment SGI3552[] = {{&S35028[30], 12}, {NULL}};
const Segment SGI3553[] = {{&S35028[90], 4}, {NULL}};
const Segment SGI3554[] = {{&S35028[90], 3}, {NULL}};
const Segment SGI3555[] = {{&S36018[6], 4}, {NULL}};
const Segment SGI3556[] = {{&S36018[32], 4}, {NULL}};
const Segment SGI3557[] = {{&S36018[81], 5}, {NULL}};
const Segment SGI3558[] = {{&S35141[44], 9}, {NULL}};
const Segment SGI3559[] = {{&S35141[44], 20}, {NULL}};
const Segment SGI3560[] = {{&S35141[30], 6}, {NULL}};
const Segment SGI3561[] = {{&S35141[112], 3}, {NULL}};
const Segment SGI3562[] = {{&S35141[78], 5}, {NULL}};
const Segment SGI3563[] = {{&S36825[6], 4}, {NULL}};
const Segment SGI3564[] = {{&S36825[32], 4}, {NULL}};
const Segment SGI3565[] = {{&S36825[81], 5}, {NULL}};
const Segment SGI3566[] = {{&S35419[44], 18}, {NULL}};
const Segment SGI3567[] = {{&S35419[44], 35}, {NULL}};
const Segment SGI3568[] = {{&S35419[30], 6}, {NULL}};
const Segment SGI3569[] = {{&S35419[127], 3}, {NULL}};
const Segment SGI3570[] = {{&S35419[93], 5}, {NULL}};
const Segment SGI3571[] = {{&S35420[48], 10}, {NULL}};
const Segment SGI3572[] = {{&S35420[48], 21}, {NULL}};
const Segment SGI3573[] = {{&S35420[30], 10}, {NULL}};
const Segment SGI3574[] = {{&S35420[117], 3}, {NULL}};
const Segment SGI3575[] = {{&S35420[83], 5}, {NULL}};
const Segment SGI3576[] = {{&S35421[48], 10}, {NULL}};
const Segment SGI3577[] = {{&S35421[48], 21}, {NULL}};
const Segment SGI3578[] = {{&S35421[30], 10}, {NULL}};
const Segment SGI3579[] = {{&S35421[117], 3}, {NULL}};
const Segment SGI3580[] = {{&S35421[83], 5}, {NULL}};
const Segment SGI3581[] = {{&S35423[44], 18}, {NULL}};
const Segment SGI3582[] = {{&S35423[44], 29}, {NULL}};
const Segment SGI3583[] = {{&S35423[30], 6}, {NULL}};
const Segment SGI3584[] = {{&S35423[121], 3}, {NULL}};
const Segment SGI3585[] = {{&S35423[87], 5}, {NULL}};
const Segment SGI3586[] = {{&S35424[44], 10}, {NULL}};
const Segment SGI3587[] = {{&S35424[44], 21}, {NULL}};
const Segment SGI3588[] = {{&S35424[30], 6}, {NULL}};
const Segment SGI3589[] = {{&S35424[113], 3}, {NULL}};
const Segment SGI3590[] = {{&S35424[79], 5}, {NULL}};
const Segment SGI3591[] = {{&S35433[48], 18}, {NULL}};
const Segment SGI3592[] = {{&S35433[48], 29}, {NULL}};
const Segment SGI3593[] = {{&S35433[30], 10}, {NULL}};
const Segment SGI3594[] = {{&S35433[125], 3}, {NULL}};
const Segment SGI3595[] = {{&S35433[91], 5}, {NULL}};
const Segment SGI3596[] = {{&S35440[38], 18}, {NULL}};
const Segment SGI3597[] = {{&S35440[31], 6}, {NULL}};
const Segment SGI3598[] = {{&S35441[48], 18}, {NULL}};
const Segment SGI3599[] = {{&S35441[48], 29}, {NULL}};
const Segment SGI3600[] = {{&S35441[30], 10}, {NULL}};
const Segment SGI3601[] = {{&S35441[125], 3}, {NULL}};
const Segment SGI3602[] = {{&S35441[91], 5}, {NULL}};
const Segment SGI3603[] = {{&S35442[44], 10}, {NULL}};
const Segment SGI3604[] = {{&S35442[44], 21}, {NULL}};
const Segment SGI3605[] = {{&S35442[30], 6}, {NULL}};
const Segment SGI3606[] = {{&S35442[114], 3}, {NULL}};
const Segment SGI3607[] = {{&S35442[79], 6}, {NULL}};
const Segment SGI3608[] = {{&S35415[42], 28}, {NULL}};
const Segment SGI3609[] = {{&S35415[42], 39}, {NULL}};
const Segment SGI3610[] = {{&S35415[30], 4}, {NULL}};
const Segment SGI3611[] = {{&S35415[129], 3}, {NULL}};
const Segment SGI3612[] = {{&S35415[95], 5}, {NULL}};
const Segment SGI3613[] = {{&S35416[42], 10}, {NULL}};
const Segment SGI3614[] = {{&S35416[42], 21}, {NULL}};
const Segment SGI3615[] = {{&S35416[30], 4}, {NULL}};
const Segment SGI3616[] = {{&S35416[111], 3}, {NULL}};
const Segment SGI3617[] = {{&S35416[77], 5}, {NULL}};
const Segment SGI3618[] = {{&S35417[42], 28}, {NULL}};
const Segment SGI3619[] = {{&S35417[42], 39}, {NULL}};
const Segment SGI3620[] = {{&S35417[30], 4}, {NULL}};
const Segment SGI3621[] = {{&S35417[129], 3}, {NULL}};
const Segment SGI3622[] = {{&S35417[95], 5}, {NULL}};
const Segment SGI3623[] = {{&S35418[42], 28}, {NULL}};
const Segment SGI3624[] = {{&S35418[42], 39}, {NULL}};
const Segment SGI3625[] = {{&S35418[30], 4}, {NULL}};
const Segment SGI3626[] = {{&S35418[129], 3}, {NULL}};
const Segment SGI3627[] = {{&S35418[95], 5}, {NULL}};
const Segment SGI3628[] = {{&S35422[42], 18}, {NULL}};
const Segment SGI3629[] = {{&S35422[42], 29}, {NULL}};
const Segment SGI3630[] = {{&S35422[30], 4}, {NULL}};
const Segment SGI3631[] = {{&S35422[119], 3}, {NULL}};
const Segment SGI3632[] = {{&S35422[85], 5}, {NULL}};
const Segment SGI3633[] = {{&S35425[42], 28}, {NULL}};
const Segment SGI3634[] = {{&S35425[42], 39}, {NULL}};
const Segment SGI3635[] = {{&S35425[30], 4}, {NULL}};
const Segment SGI3636[] = {{&S35425[129], 3}, {NULL}};
const Segment SGI3637[] = {{&S35425[95], 5}, {NULL}};
const Segment SGI3638[] = {{&S35426[42], 28}, {NULL}};
const Segment SGI3639[] = {{&S35426[42], 39}, {NULL}};
const Segment SGI3640[] = {{&S35426[30], 4}, {NULL}};
const Segment SGI3641[] = {{&S35426[129], 3}, {NULL}};
const Segment SGI3642[] = {{&S35426[95], 5}, {NULL}};
const Segment SGI3643[] = {{&S35427[42], 28}, {NULL}};
const Segment SGI3644[] = {{&S35427[42], 39}, {NULL}};
const Segment SGI3645[] = {{&S35427[30], 4}, {NULL}};
const Segment SGI3646[] = {{&S35427[129], 3}, {NULL}};
const Segment SGI3647[] = {{&S35427[95], 5}, {NULL}};
const Segment SGI3648[] = {{&S35428[41], 28}, {NULL}};
const Segment SGI3649[] = {{&S35428[41], 39}, {NULL}};
const Segment SGI3650[] = {{&S35428[128], 3}, {NULL}};
const Segment SGI3651[] = {{&S35428[94], 5}, {NULL}};
const Segment SGI3652[] = {{&S35429[42], 27}, {NULL}};
const Segment SGI3653[] = {{&S35429[42], 38}, {NULL}};
const Segment SGI3654[] = {{&S35429[30], 4}, {NULL}};
const Segment SGI3655[] = {{&S35429[128], 3}, {NULL}};
const Segment SGI3656[] = {{&S35429[94], 5}, {NULL}};
const Segment SGI3657[] = {{&S35430[41], 28}, {NULL}};
const Segment SGI3658[] = {{&S35430[41], 39}, {NULL}};
const Segment SGI3659[] = {{&S35430[30], 3}, {NULL}};
const Segment SGI3660[] = {{&S35430[128], 3}, {NULL}};
const Segment SGI3661[] = {{&S35430[94], 5}, {NULL}};
const Segment SGI3662[] = {{&S35431[42], 27}, {NULL}};
const Segment SGI3663[] = {{&S35431[42], 38}, {NULL}};
const Segment SGI3664[] = {{&S35431[30], 4}, {NULL}};
const Segment SGI3665[] = {{&S35431[128], 3}, {NULL}};
const Segment SGI3666[] = {{&S35431[94], 5}, {NULL}};
const Segment SGI3667[] = {{&S35432[40], 28}, {NULL}};
const Segment SGI3668[] = {{&S35432[30], 4}, {NULL}};
const Segment SGI3669[] = {{&S35432[127], 3}, {NULL}};
const Segment SGI3670[] = {{&S35432[93], 5}, {NULL}};
const Segment SGI3671[] = {{&S35434[42], 18}, {NULL}};
const Segment SGI3672[] = {{&S35434[42], 35}, {NULL}};
const Segment SGI3673[] = {{&S35434[30], 4}, {NULL}};
const Segment SGI3674[] = {{&S35435[42], 28}, {NULL}};
const Segment SGI3675[] = {{&S35435[42], 39}, {NULL}};
const Segment SGI3676[] = {{&S35435[30], 4}, {NULL}};
const Segment SGI3677[] = {{&S35435[129], 3}, {NULL}};
const Segment SGI3678[] = {{&S35435[95], 5}, {NULL}};
const Segment SGI3679[] = {{&S35436[42], 17}, {NULL}};
const Segment SGI3680[] = {{&S35436[42], 28}, {NULL}};
const Segment SGI3681[] = {{&S35436[30], 4}, {NULL}};
const Segment SGI3682[] = {{&S35436[118], 3}, {NULL}};
const Segment SGI3683[] = {{&S35436[84], 5}, {NULL}};
const Segment SGI3684[] = {{&S35437[36], 18}, {NULL}};
const Segment SGI3685[] = {{&S35437[31], 4}, {NULL}};
const Segment SGI3686[] = {{&S35438[42], 10}, {NULL}};
const Segment SGI3687[] = {{&S35438[42], 21}, {NULL}};
const Segment SGI3688[] = {{&S35438[30], 4}, {NULL}};
const Segment SGI3689[] = {{&S35438[111], 3}, {NULL}};
const Segment SGI3690[] = {{&S35438[77], 5}, {NULL}};
const Segment SGI3691[] = {{&S35439[42], 18}, {NULL}};
const Segment SGI3692[] = {{&S35439[42], 29}, {NULL}};
const Segment SGI3693[] = {{&S35439[30], 4}, {NULL}};
const Segment SGI3694[] = {{&S35439[121], 3}, {NULL}};
const Segment SGI3695[] = {{&S35439[85], 7}, {NULL}};
const Segment SGI3696[] = {{&S35439[141], 8}, {NULL}};
const Segment SGI3697[] = {{&S35443[40], 18}, {NULL}};
const Segment SGI3698[] = {{&S35443[30], 4}, {NULL}};
const Segment SGI3699[] = {{&S35443[117], 3}, {NULL}};
const Segment SGI3700[] = {{&S35443[83], 5}, {NULL}};
const Segment SGI3701[] = {{&S37320[23], 12}, {NULL}};
const Segment SGI3702[] = {{&S35759[44], 27}, {NULL}};
const Segment SGI3703[] = {{&S35759[44], 39}, {NULL}};
const Segment SGI3704[] = {{&S35759[30], 6}, {NULL}};
const Segment SGI3705[] = {{&S35759[131], 3}, {NULL}};
const Segment SGI3706[] = {{&S35759[97], 5}, {NULL}};
const Segment SGI3707[] = {{&S35760[44], 101}, {NULL}};
const Segment SGI3708[] = {{&S35760[44], 113}, {NULL}};
const Segment SGI3709[] = {{&S35760[30], 6}, {NULL}};
const Segment SGI3710[] = {{&S35760[205], 3}, {NULL}};
const Segment SGI3711[] = {{&S35760[171], 5}, {NULL}};
const Segment SGI3712[] = {{&S35762[44], 24}, {NULL}};
const Segment SGI3713[] = {{&S35762[44], 36}, {NULL}};
const Segment SGI3714[] = {{&S35762[30], 6}, {NULL}};
const Segment SGI3715[] = {{&S35762[128], 3}, {NULL}};
const Segment SGI3716[] = {{&S35762[94], 5}, {NULL}};
const Segment SGI3717[] = {{&S35763[44], 20}, {NULL}};
const Segment SGI3718[] = {{&S35763[44], 31}, {NULL}};
const Segment SGI3719[] = {{&S35763[30], 6}, {NULL}};
const Segment SGI3720[] = {{&S35763[123], 3}, {NULL}};
const Segment SGI3721[] = {{&S35763[89], 5}, {NULL}};
const Segment SGI3722[] = {{&S35764[44], 26}, {NULL}};
const Segment SGI3723[] = {{&S35764[44], 38}, {NULL}};
const Segment SGI3724[] = {{&S35764[30], 6}, {NULL}};
const Segment SGI3725[] = {{&S35764[130], 3}, {NULL}};
const Segment SGI3726[] = {{&S35764[96], 5}, {NULL}};
const Segment SGI3727[] = {{&S35766[44], 26}, {NULL}};
const Segment SGI3728[] = {{&S35766[44], 38}, {NULL}};
const Segment SGI3729[] = {{&S35766[30], 6}, {NULL}};
const Segment SGI3730[] = {{&S35766[130], 3}, {NULL}};
const Segment SGI3731[] = {{&S35766[96], 5}, {NULL}};
const Segment SGI3732[] = {{&S35767[44], 15}, {NULL}};
const Segment SGI3733[] = {{&S35767[44], 27}, {NULL}};
const Segment SGI3734[] = {{&S35767[30], 6}, {NULL}};
const Segment SGI3735[] = {{&S35767[119], 3}, {NULL}};
const Segment SGI3736[] = {{&S35767[85], 5}, {NULL}};
const Segment SGI3737[] = {{&S35768[44], 26}, {NULL}};
const Segment SGI3738[] = {{&S35768[44], 38}, {NULL}};
const Segment SGI3739[] = {{&S35768[30], 6}, {NULL}};
const Segment SGI3740[] = {{&S35768[130], 3}, {NULL}};
const Segment SGI3741[] = {{&S35768[96], 5}, {NULL}};
const Segment SGI3742[] = {{&S35769[44], 26}, {NULL}};
const Segment SGI3743[] = {{&S35769[44], 38}, {NULL}};
const Segment SGI3744[] = {{&S35769[30], 6}, {NULL}};
const Segment SGI3745[] = {{&S35769[130], 3}, {NULL}};
const Segment SGI3746[] = {{&S35769[96], 5}, {NULL}};
const Segment SGI3747[] = {{&S35770[44], 26}, {NULL}};
const Segment SGI3748[] = {{&S35770[44], 38}, {NULL}};
const Segment SGI3749[] = {{&S35770[30], 6}, {NULL}};
const Segment SGI3750[] = {{&S35770[130], 3}, {NULL}};
const Segment SGI3751[] = {{&S35770[96], 5}, {NULL}};
const Segment SGI3752[] = {{&S35774[44], 101}, {NULL}};
const Segment SGI3753[] = {{&S35774[44], 112}, {NULL}};
const Segment SGI3754[] = {{&S35774[30], 6}, {NULL}};
const Segment SGI3755[] = {{&S35774[204], 3}, {NULL}};
const Segment SGI3756[] = {{&S35774[170], 5}, {NULL}};
const Segment SGI3757[] = {{&S35775[44], 14}, {NULL}};
const Segment SGI3758[] = {{&S35775[44], 26}, {NULL}};
const Segment SGI3759[] = {{&S35775[30], 6}, {NULL}};
const Segment SGI3760[] = {{&S35775[118], 3}, {NULL}};
const Segment SGI3761[] = {{&S35775[84], 5}, {NULL}};
const Segment SGI3762[] = {{&S35776[44], 10}, {NULL}};
const Segment SGI3763[] = {{&S35776[44], 22}, {NULL}};
const Segment SGI3764[] = {{&S35776[30], 6}, {NULL}};
const Segment SGI3765[] = {{&S35776[114], 3}, {NULL}};
const Segment SGI3766[] = {{&S35776[80], 5}, {NULL}};
const Segment SGI3767[] = {{&S35780[44], 11}, {NULL}};
const Segment SGI3768[] = {{&S35780[44], 23}, {NULL}};
const Segment SGI3769[] = {{&S35780[30], 6}, {NULL}};
const Segment SGI3770[] = {{&S35780[115], 3}, {NULL}};
const Segment SGI3771[] = {{&S35780[81], 5}, {NULL}};
const Segment SGI3772[] = {{&S35781[44], 20}, {NULL}};
const Segment SGI3773[] = {{&S35781[44], 32}, {NULL}};
const Segment SGI3774[] = {{&S35781[30], 6}, {NULL}};
const Segment SGI3775[] = {{&S35781[124], 3}, {NULL}};
const Segment SGI3776[] = {{&S35781[90], 5}, {NULL}};
const Segment SGI3777[] = {{&S35783[44], 20}, {NULL}};
const Segment SGI3778[] = {{&S35783[44], 32}, {NULL}};
const Segment SGI3779[] = {{&S35783[30], 6}, {NULL}};
const Segment SGI3780[] = {{&S35783[124], 3}, {NULL}};
const Segment SGI3781[] = {{&S35783[90], 5}, {NULL}};
const Segment SGI3782[] = {{&S35785[38], 11}, {NULL}};
const Segment SGI3783[] = {{&S35785[31], 6}, {NULL}};
const Segment SGI3784[] = {{&S35787[44], 11}, {NULL}};
const Segment SGI3785[] = {{&S35787[44], 23}, {NULL}};
const Segment SGI3786[] = {{&S35787[30], 6}, {NULL}};
const Segment SGI3787[] = {{&S35787[84], 4}, {NULL}};
const Segment SGI3788[] = {{&S35787[84], 3}, {NULL}};
const Segment SGI3789[] = {{&S35789[44], 11}, {NULL}};
const Segment SGI3790[] = {{&S35789[44], 23}, {NULL}};
const Segment SGI3791[] = {{&S35789[30], 6}, {NULL}};
const Segment SGI3792[] = {{&S35789[117], 3}, {NULL}};
const Segment SGI3793[] = {{&S35789[81], 7}, {NULL}};
const Segment SGI3794[] = {{&S35789[137], 8}, {NULL}};
const Segment SGI3795[] = {{&S35791[44], 11}, {NULL}};
const Segment SGI3796[] = {{&S35791[44], 23}, {NULL}};
const Segment SGI3797[] = {{&S35791[30], 6}, {NULL}};
const Segment SGI3798[] = {{&S35791[84], 6}, {NULL}};
const Segment SGI3799[] = {{&S35831[44], 11}, {NULL}};
const Segment SGI3800[] = {{&S35831[44], 22}, {NULL}};
const Segment SGI3801[] = {{&S35831[30], 6}, {NULL}};
const Segment SGI3802[] = {{&S35831[114], 3}, {NULL}};
const Segment SGI3803[] = {{&S35831[80], 5}, {NULL}};
const Segment SGI3804[] = {{&S35838[38], 11}, {NULL}};
const Segment SGI3805[] = {{&S35838[31], 6}, {NULL}};
const Segment SGI3806[] = {{&S35867[44], 11}, {NULL}};
const Segment SGI3807[] = {{&S35867[44], 22}, {NULL}};
const Segment SGI3808[] = {{&S35867[30], 6}, {NULL}};
const Segment SGI3809[] = {{&S35867[114], 3}, {NULL}};
const Segment SGI3810[] = {{&S35867[80], 5}, {NULL}};
const Segment SGI3811[] = {{&S35868[44], 10}, {NULL}};
const Segment SGI3812[] = {{&S35868[44], 21}, {NULL}};
const Segment SGI3813[] = {{&S35868[30], 6}, {NULL}};
const Segment SGI3814[] = {{&S35868[113], 3}, {NULL}};
const Segment SGI3815[] = {{&S35868[79], 5}, {NULL}};
const Segment SGI3816[] = {{&S35869[38], 10}, {NULL}};
const Segment SGI3817[] = {{&S35869[31], 6}, {NULL}};
const Segment SGI3818[] = {{&S35870[44], 10}, {NULL}};
const Segment SGI3819[] = {{&S35870[44], 21}, {NULL}};
const Segment SGI3820[] = {{&S35870[30], 6}, {NULL}};
const Segment SGI3821[] = {{&S35870[82], 4}, {NULL}};
const Segment SGI3822[] = {{&S35870[82], 3}, {NULL}};
const Segment SGI3823[] = {{&S35871[38], 11}, {NULL}};
const Segment SGI3824[] = {{&S35871[31], 6}, {NULL}};
const Segment SGI3825[] = {{&S35872[44], 10}, {NULL}};
const Segment SGI3826[] = {{&S35872[44], 21}, {NULL}};
const Segment SGI3827[] = {{&S35872[30], 6}, {NULL}};
const Segment SGI3828[] = {{&S35872[82], 6}, {NULL}};
const Segment SGI3829[] = {{&S35873[42], 10}, {NULL}};
const Segment SGI3830[] = {{&S35873[30], 6}, {NULL}};
const Segment SGI3831[] = {{&S35873[111], 3}, {NULL}};
const Segment SGI3832[] = {{&S35873[77], 5}, {NULL}};
const Segment SGI3833[] = {{&S35761[40], 20}, {NULL}};
const Segment SGI3834[] = {{&S35761[30], 4}, {NULL}};
const Segment SGI3835[] = {{&S35761[119], 3}, {NULL}};
const Segment SGI3836[] = {{&S35761[85], 5}, {NULL}};
const Segment SGI3837[] = {{&S35765[42], 26}, {NULL}};
const Segment SGI3838[] = {{&S35765[42], 37}, {NULL}};
const Segment SGI3839[] = {{&S35765[30], 4}, {NULL}};
const Segment SGI3840[] = {{&S35765[127], 3}, {NULL}};
const Segment SGI3841[] = {{&S35765[93], 5}, {NULL}};
const Segment SGI3842[] = {{&S35771[42], 26}, {NULL}};
const Segment SGI3843[] = {{&S35771[42], 37}, {NULL}};
const Segment SGI3844[] = {{&S35771[30], 4}, {NULL}};
const Segment SGI3845[] = {{&S35771[127], 3}, {NULL}};
const Segment SGI3846[] = {{&S35771[93], 5}, {NULL}};
const Segment SGI3847[] = {{&S35772[42], 26}, {NULL}};
const Segment SGI3848[] = {{&S35772[42], 37}, {NULL}};
const Segment SGI3849[] = {{&S35772[30], 4}, {NULL}};
const Segment SGI3850[] = {{&S35772[127], 3}, {NULL}};
const Segment SGI3851[] = {{&S35772[93], 5}, {NULL}};
const Segment SGI3852[] = {{&S35773[42], 24}, {NULL}};
const Segment SGI3853[] = {{&S35773[42], 35}, {NULL}};
const Segment SGI3854[] = {{&S35773[30], 4}, {NULL}};
const Segment SGI3855[] = {{&S35773[125], 3}, {NULL}};
const Segment SGI3856[] = {{&S35773[91], 5}, {NULL}};
const Segment SGI3857[] = {{&S35777[42], 14}, {NULL}};
const Segment SGI3858[] = {{&S35777[42], 25}, {NULL}};
const Segment SGI3859[] = {{&S35777[30], 4}, {NULL}};
const Segment SGI3860[] = {{&S35777[115], 3}, {NULL}};
const Segment SGI3861[] = {{&S35777[81], 5}, {NULL}};
const Segment SGI3862[] = {{&S35778[42], 11}, {NULL}};
const Segment SGI3863[] = {{&S35778[42], 22}, {NULL}};
const Segment SGI3864[] = {{&S35778[30], 4}, {NULL}};
const Segment SGI3865[] = {{&S35778[112], 3}, {NULL}};
const Segment SGI3866[] = {{&S35778[78], 5}, {NULL}};
const Segment SGI3867[] = {{&S35779[42], 7}, {NULL}};
const Segment SGI3868[] = {{&S35779[42], 18}, {NULL}};
const Segment SGI3869[] = {{&S35779[30], 4}, {NULL}};
const Segment SGI3870[] = {{&S35779[108], 3}, {NULL}};
const Segment SGI3871[] = {{&S35779[74], 5}, {NULL}};
const Segment SGI3872[] = {{&S35782[42], 20}, {NULL}};
const Segment SGI3873[] = {{&S35782[42], 31}, {NULL}};
const Segment SGI3874[] = {{&S35782[30], 4}, {NULL}};
const Segment SGI3875[] = {{&S35782[121], 3}, {NULL}};
const Segment SGI3876[] = {{&S35782[87], 5}, {NULL}};
const Segment SGI3877[] = {{&S35784[36], 11}, {NULL}};
const Segment SGI3878[] = {{&S35784[31], 4}, {NULL}};
const Segment SGI3879[] = {{&S35786[42], 11}, {NULL}};
const Segment SGI3880[] = {{&S35786[42], 22}, {NULL}};
const Segment SGI3881[] = {{&S35786[30], 4}, {NULL}};
const Segment SGI3882[] = {{&S35786[81], 4}, {NULL}};
const Segment SGI3883[] = {{&S35786[81], 3}, {NULL}};
const Segment SGI3884[] = {{&S35788[36], 7}, {NULL}};
const Segment SGI3885[] = {{&S35788[31], 4}, {NULL}};
const Segment SGI3886[] = {{&S35790[42], 11}, {NULL}};
const Segment SGI3887[] = {{&S35790[42], 22}, {NULL}};
const Segment SGI3888[] = {{&S35790[30], 4}, {NULL}};
const Segment SGI3889[] = {{&S35790[81], 6}, {NULL}};
const Segment SGI3890[] = {{&S35830[42], 11}, {NULL}};
const Segment SGI3891[] = {{&S35830[42], 22}, {NULL}};
const Segment SGI3892[] = {{&S35830[30], 4}, {NULL}};
const Segment SGI3893[] = {{&S35830[112], 3}, {NULL}};
const Segment SGI3894[] = {{&S35830[78], 5}, {NULL}};
const Segment SGI3895[] = {{&S35832[42], 11}, {NULL}};
const Segment SGI3896[] = {{&S35832[42], 22}, {NULL}};
const Segment SGI3897[] = {{&S35832[30], 4}, {NULL}};
const Segment SGI3898[] = {{&S35832[112], 3}, {NULL}};
const Segment SGI3899[] = {{&S35832[78], 5}, {NULL}};
const Segment SGI3900[] = {{&S35833[42], 10}, {NULL}};
const Segment SGI3901[] = {{&S35833[42], 21}, {NULL}};
const Segment SGI3902[] = {{&S35833[30], 4}, {NULL}};
const Segment SGI3903[] = {{&S35833[111], 3}, {NULL}};
const Segment SGI3904[] = {{&S35833[77], 5}, {NULL}};
const Segment SGI3905[] = {{&S35835[36], 10}, {NULL}};
const Segment SGI3906[] = {{&S35835[31], 4}, {NULL}};
const Segment SGI3907[] = {{&S35836[36], 11}, {NULL}};
const Segment SGI3908[] = {{&S35836[31], 4}, {NULL}};
const Segment SGI3909[] = {{&S35837[42], 10}, {NULL}};
const Segment SGI3910[] = {{&S35837[42], 21}, {NULL}};
const Segment SGI3911[] = {{&S35837[30], 4}, {NULL}};
const Segment SGI3912[] = {{&S35837[112], 3}, {NULL}};
const Segment SGI3913[] = {{&S35837[77], 6}, {NULL}};
const Segment SGI3914[] = {{&S35839[36], 11}, {NULL}};
const Segment SGI3915[] = {{&S35839[31], 4}, {NULL}};
const Segment SGI3916[] = {{&S35841[42], 11}, {NULL}};
const Segment SGI3917[] = {{&S35841[42], 22}, {NULL}};
const Segment SGI3918[] = {{&S35841[30], 4}, {NULL}};
const Segment SGI3919[] = {{&S35841[112], 3}, {NULL}};
const Segment SGI3920[] = {{&S35841[78], 5}, {NULL}};
const Segment SGI3921[] = {{&S35842[41], 9}, {NULL}};
const Segment SGI3922[] = {{&S35842[41], 21}, {NULL}};
const Segment SGI3923[] = {{&S35842[30], 4}, {NULL}};
const Segment SGI3924[] = {{&S35842[111], 3}, {NULL}};
const Segment SGI3925[] = {{&S35842[76], 5}, {NULL}};
const Segment SGI3926[] = {{&S35844[42], 10}, {NULL}};
const Segment SGI3927[] = {{&S35844[42], 21}, {NULL}};
const Segment SGI3928[] = {{&S35844[30], 4}, {NULL}};
const Segment SGI3929[] = {{&S35844[111], 3}, {NULL}};
const Segment SGI3930[] = {{&S35844[77], 5}, {NULL}};
const Segment SGI3931[] = {{&S35847[40], 10}, {NULL}};
const Segment SGI3932[] = {{&S35847[30], 4}, {NULL}};
const Segment SGI3933[] = {{&S35847[109], 3}, {NULL}};
const Segment SGI3934[] = {{&S35847[75], 5}, {NULL}};
const Segment SGI3935[] = {{&S36153[44], 10}, {NULL}};
const Segment SGI3936[] = {{&S36153[44], 21}, {NULL}};
const Segment SGI3937[] = {{&S36153[30], 6}, {NULL}};
const Segment SGI3938[] = {{&S36153[113], 3}, {NULL}};
const Segment SGI3939[] = {{&S36153[79], 5}, {NULL}};
const Segment SGI3940[] = {{&S36154[38], 10}, {NULL}};
const Segment SGI3941[] = {{&S36154[31], 6}, {NULL}};
const Segment SGI3942[] = {{&S36157[44], 10}, {NULL}};
const Segment SGI3943[] = {{&S36157[44], 21}, {NULL}};
const Segment SGI3944[] = {{&S36157[30], 6}, {NULL}};
const Segment SGI3945[] = {{&S36157[115], 3}, {NULL}};
const Segment SGI3946[] = {{&S36157[79], 7}, {NULL}};
const Segment SGI3947[] = {{&S36157[135], 8}, {NULL}};
const Segment SGI3948[] = {{&S36159[42], 10}, {NULL}};
const Segment SGI3949[] = {{&S36159[30], 6}, {NULL}};
const Segment SGI3950[] = {{&S36159[111], 3}, {NULL}};
const Segment SGI3951[] = {{&S36159[77], 5}, {NULL}};
const Segment SGI3952[] = {{&S36160[43], 10}, {NULL}};
const Segment SGI3953[] = {{&S36160[43], 21}, {NULL}};
const Segment SGI3954[] = {{&S36160[30], 5}, {NULL}};
const Segment SGI3955[] = {{&S36160[112], 3}, {NULL}};
const Segment SGI3956[] = {{&S36160[78], 5}, {NULL}};
const Segment SGI3957[] = {{&S36161[44], 9}, {NULL}};
const Segment SGI3958[] = {{&S36161[44], 20}, {NULL}};
const Segment SGI3959[] = {{&S36161[30], 6}, {NULL}};
const Segment SGI3960[] = {{&S36161[112], 3}, {NULL}};
const Segment SGI3961[] = {{&S36161[78], 5}, {NULL}};
const Segment SGI3962[] = {{&S36162[44], 9}, {NULL}};
const Segment SGI3963[] = {{&S36162[44], 20}, {NULL}};
const Segment SGI3964[] = {{&S36162[30], 6}, {NULL}};
const Segment SGI3965[] = {{&S36162[112], 3}, {NULL}};
const Segment SGI3966[] = {{&S36162[78], 5}, {NULL}};
const Segment SGI3967[] = {{&S36163[43], 10}, {NULL}};
const Segment SGI3968[] = {{&S36163[43], 21}, {NULL}};
const Segment SGI3969[] = {{&S36163[112], 3}, {NULL}};
const Segment SGI3970[] = {{&S36163[78], 5}, {NULL}};
const Segment SGI3971[] = {{&S36832[50], 6}, {NULL}};
const Segment SGI3972[] = {{&S36832[50], 27}, {NULL}};
const Segment SGI3973[] = {{&S36832[30], 12}, {NULL}};
const Segment SGI3974[] = {{&S36832[126], 3}, {NULL}};
const Segment SGI3975[] = {{&S36832[91], 6}, {NULL}};
const Segment SGI3976[] = {{&S36833[49], 7}, {NULL}};
const Segment SGI3977[] = {{&S36833[49], 28}, {NULL}};
const Segment SGI3978[] = {{&S36833[126], 3}, {NULL}};
const Segment SGI3979[] = {{&S36833[91], 6}, {NULL}};
const Segment SGI3980[] = {{&S36834[50], 7}, {NULL}};
const Segment SGI3981[] = {{&S36834[50], 28}, {NULL}};
const Segment SGI3982[] = {{&S36834[30], 12}, {NULL}};
const Segment SGI3983[] = {{&S36834[127], 3}, {NULL}};
const Segment SGI3984[] = {{&S36834[92], 6}, {NULL}};
const Segment SGI3985[] = {{&S36835[49], 7}, {NULL}};
const Segment SGI3986[] = {{&S36835[49], 28}, {NULL}};
const Segment SGI3987[] = {{&S36835[30], 11}, {NULL}};
const Segment SGI3988[] = {{&S36835[126], 3}, {NULL}};
const Segment SGI3989[] = {{&S36835[91], 6}, {NULL}};
const Segment SGI3990[] = {{&S36836[50], 7}, {NULL}};
const Segment SGI3991[] = {{&S36836[50], 29}, {NULL}};
const Segment SGI3992[] = {{&S36836[30], 12}, {NULL}};
const Segment SGI3993[] = {{&S36836[128], 3}, {NULL}};
const Segment SGI3994[] = {{&S36836[93], 6}, {NULL}};
const Segment SGI3995[] = {{&S36837[50], 7}, {NULL}};
const Segment SGI3996[] = {{&S36837[50], 28}, {NULL}};
const Segment SGI3997[] = {{&S36837[30], 12}, {NULL}};
const Segment SGI3998[] = {{&S36837[127], 3}, {NULL}};
const Segment SGI3999[] = {{&S36837[92], 6}, {NULL}};
const Segment SGI4000[] = {{&S36838[50], 7}, {NULL}};
const Segment SGI4001[] = {{&S36838[50], 28}, {NULL}};
const Segment SGI4002[] = {{&S36838[30], 12}, {NULL}};
const Segment SGI4003[] = {{&S36838[127], 3}, {NULL}};
const Segment SGI4004[] = {{&S36838[92], 6}, {NULL}};
const Segment SGI4005[] = {{&S36839[50], 6}, {NULL}};
const Segment SGI4006[] = {{&S36839[50], 27}, {NULL}};
const Segment SGI4007[] = {{&S36839[30], 12}, {NULL}};
const Segment SGI4008[] = {{&S36839[126], 3}, {NULL}};
const Segment SGI4009[] = {{&S36839[91], 6}, {NULL}};
const Segment SGI4010[] = {{&S36840[50], 7}, {NULL}};
const Segment SGI4011[] = {{&S36840[50], 29}, {NULL}};
const Segment SGI4012[] = {{&S36840[30], 12}, {NULL}};
const Segment SGI4013[] = {{&S36840[128], 3}, {NULL}};
const Segment SGI4014[] = {{&S36840[93], 6}, {NULL}};
const Segment SGI4015[] = {{&S36841[50], 7}, {NULL}};
const Segment SGI4016[] = {{&S36841[50], 29}, {NULL}};
const Segment SGI4017[] = {{&S36841[30], 12}, {NULL}};
const Segment SGI4018[] = {{&S36841[127], 3}, {NULL}};
const Segment SGI4019[] = {{&S36841[93], 5}, {NULL}};
const Segment SGI4020[] = {{&S36842[96], 7}, {NULL}};
const Segment SGI4021[] = {{&S36842[96], 28}, {NULL}};
const Segment SGI4022[] = {{&S36842[76], 12}, {NULL}};
const Segment SGI4023[] = {{&S36842[173], 3}, {NULL}};
const Segment SGI4024[] = {{&S36842[138], 6}, {NULL}};
const Segment SGI4025[] = {{&S36843[48], 7}, {NULL}};
const Segment SGI4026[] = {{&S36843[30], 12}, {NULL}};
const Segment SGI4027[] = {{&S36843[126], 3}, {NULL}};
const Segment SGI4028[] = {{&S36843[91], 6}, {NULL}};
const Segment SGI4029[] = {{&S36844[50], 7}, {NULL}};
const Segment SGI4030[] = {{&S36844[50], 29}, {NULL}};
const Segment SGI4031[] = {{&S36844[30], 12}, {NULL}};
const Segment SGI4032[] = {{&S36844[128], 3}, {NULL}};
const Segment SGI4033[] = {{&S36844[93], 6}, {NULL}};
const Segment SGI4034[] = {{&S36845[50], 7}, {NULL}};
const Segment SGI4035[] = {{&S36845[50], 29}, {NULL}};
const Segment SGI4036[] = {{&S36845[30], 12}, {NULL}};
const Segment SGI4037[] = {{&S36845[128], 3}, {NULL}};
const Segment SGI4038[] = {{&S36845[93], 6}, {NULL}};
const Segment SGI4039[] = {{&S36846[50], 7}, {NULL}};
const Segment SGI4040[] = {{&S36846[50], 29}, {NULL}};
const Segment SGI4041[] = {{&S36846[30], 12}, {NULL}};
const Segment SGI4042[] = {{&S36846[76], 3}, {NULL}};
const Segment SGI4043[] = {{&S36846[93], 6}, {NULL}};
const Segment SGI4044[] = {{&S36847[50], 7}, {NULL}};
const Segment SGI4045[] = {{&S36847[50], 28}, {NULL}};
const Segment SGI4046[] = {{&S36847[30], 12}, {NULL}};
const Segment SGI4047[] = {{&S36847[75], 3}, {NULL}};
const Segment SGI4048[] = {{&S36847[92], 6}, {NULL}};
const Segment SGI4049[] = {{&S36848[50], 7}, {NULL}};
const Segment SGI4050[] = {{&S36314[44], 7}, {NULL}};
const Segment SGI4051[] = {{&S36314[44], 18}, {NULL}};
const Segment SGI4052[] = {{&S36314[30], 6}, {NULL}};
const Segment SGI4053[] = {{&S36314[110], 3}, {NULL}};
const Segment SGI4054[] = {{&S36314[76], 5}, {NULL}};
const Segment SGI4055[] = {{&S36315[44], 7}, {NULL}};
const Segment SGI4056[] = {{&S36315[44], 19}, {NULL}};
const Segment SGI4057[] = {{&S36315[30], 6}, {NULL}};
const Segment SGI4058[] = {{&S36315[111], 3}, {NULL}};
const Segment SGI4059[] = {{&S36315[77], 5}, {NULL}};
const Segment SGI4060[] = {{&S36316[44], 7}, {NULL}};
const Segment SGI4061[] = {{&S36316[44], 19}, {NULL}};
const Segment SGI4062[] = {{&S36316[30], 6}, {NULL}};
const Segment SGI4063[] = {{&S36316[111], 3}, {NULL}};
const Segment SGI4064[] = {{&S36316[77], 5}, {NULL}};
const Segment SGI4065[] = {{&S36320[38], 7}, {NULL}};
const Segment SGI4066[] = {{&S36320[31], 6}, {NULL}};
const Segment SGI4067[] = {{&S36321[44], 7}, {NULL}};
const Segment SGI4068[] = {{&S36321[44], 19}, {NULL}};
const Segment SGI4069[] = {{&S36321[30], 6}, {NULL}};
const Segment SGI4070[] = {{&S36321[111], 3}, {NULL}};
const Segment SGI4071[] = {{&S36321[77], 5}, {NULL}};
const Segment SGI4072[] = {{&S36848[50], 29}, {NULL}};
const Segment SGI4073[] = {{&S36848[30], 12}, {NULL}};
const Segment SGI4074[] = {{&S36848[128], 3}, {NULL}};
const Segment SGI4075[] = {{&S36848[93], 6}, {NULL}};
const Segment SGI4076[] = {{&S36849[105], 7}, {NULL}};
const Segment SGI4077[] = {{&S36849[105], 32}, {NULL}};
const Segment SGI4078[] = {{&S36849[85], 12}, {NULL}};
const Segment SGI4079[] = {{&S36849[186], 3}, {NULL}};
const Segment SGI4080[] = {{&S36849[151], 6}, {NULL}};
const Segment SGI4081[] = {{&S36850[101], 7}, {NULL}};
const Segment SGI4082[] = {{&S36330[43], 20}, {NULL}};
const Segment SGI4083[] = {{&S36850[101], 28}, {NULL}};
const Segment SGI4084[] = {{&S36850[81], 12}, {NULL}};
const Segment SGI4085[] = {{&S36330[110], 3}, {NULL}};
const Segment SGI4086[] = {{&S36330[77], 5}, {NULL}};
const Segment SGI4087[] = {{&S36850[178], 3}, {NULL}};
const Segment SGI4088[] = {{&S36850[143], 6}, {NULL}};
const Segment SGI4089[] = {{&S36851[50], 7}, {NULL}};
const Segment SGI4090[] = {{&S36851[50], 29}, {NULL}};
const Segment SGI4091[] = {{&S36851[30], 12}, {NULL}};
const Segment SGI4092[] = {{&S36851[76], 3}, {NULL}};
const Segment SGI4093[] = {{&S36851[93], 6}, {NULL}};
const Segment SGI4094[] = {{&S36852[50], 7}, {NULL}};
const Segment SGI4095[] = {{&S36852[50], 29}, {NULL}};
const Segment SGI4096[] = {{&S36852[30], 12}, {NULL}};
const Segment SGI4097[] = {{&S36852[128], 3}, {NULL}};
const Segment SGI4098[] = {{&S36852[93], 6}, {NULL}};
const Segment SGI4099[] = {{&S36853[50], 7}, {NULL}};
const Segment SGI4100[] = {{&S36853[50], 29}, {NULL}};
const Segment SGI4101[] = {{&S36853[30], 12}, {NULL}};
const Segment SGI4102[] = {{&S36853[128], 3}, {NULL}};
const Segment SGI4103[] = {{&S36853[93], 6}, {NULL}};
const Segment SGI4104[] = {{&S36854[50], 7}, {NULL}};
const Segment SGI4105[] = {{&S36854[50], 28}, {NULL}};
const Segment SGI4106[] = {{&S36854[30], 12}, {NULL}};
const Segment SGI4107[] = {{&S36854[127], 3}, {NULL}};
const Segment SGI4108[] = {{&S36854[92], 6}, {NULL}};
const Segment SGI4109[] = {{&S36855[50], 7}, {NULL}};
const Segment SGI4110[] = {{&S36855[50], 28}, {NULL}};
const Segment SGI4111[] = {{&S36855[30], 12}, {NULL}};
const Segment SGI4112[] = {{&S36855[127], 3}, {NULL}};
const Segment SGI4113[] = {{&S36855[92], 6}, {NULL}};
const Segment SGI4114[] = {{&S36856[50], 7}, {NULL}};
const Segment SGI4115[] = {{&S36856[50], 29}, {NULL}};
const Segment SGI4116[] = {{&S36856[30], 12}, {NULL}};
const Segment SGI4117[] = {{&S36856[128], 3}, {NULL}};
const Segment SGI4118[] = {{&S36856[93], 6}, {NULL}};
const Segment SGI4119[] = {{&S36857[50], 7}, {NULL}};
const Segment SGI4120[] = {{&S36857[50], 28}, {NULL}};
const Segment SGI4121[] = {{&S36857[30], 12}, {NULL}};
const Segment SGI4122[] = {{&S36857[127], 3}, {NULL}};
const Segment SGI4123[] = {{&S36857[92], 6}, {NULL}};
const Segment SGI4124[] = {{&S36858[50], 7}, {NULL}};
const Segment SGI4125[] = {{&S36858[50], 28}, {NULL}};
const Segment SGI4126[] = {{&S36858[30], 12}, {NULL}};
const Segment SGI4127[] = {{&S36858[127], 3}, {NULL}};
const Segment SGI4128[] = {{&S36858[92], 6}, {NULL}};
const Segment SGI4129[] = {{&S36859[105], 7}, {NULL}};
const Segment SGI4130[] = {{&S36859[105], 32}, {NULL}};
const Segment SGI4131[] = {{&S36859[85], 12}, {NULL}};
const Segment SGI4132[] = {{&S36859[186], 3}, {NULL}};
const Segment SGI4133[] = {{&S36859[151], 6}, {NULL}};
const Segment SGI4134[] = {{&S36860[50], 7}, {NULL}};
const Segment SGI4135[] = {{&S36860[50], 29}, {NULL}};
const Segment SGI4136[] = {{&S36860[30], 12}, {NULL}};
const Segment SGI4137[] = {{&S36860[128], 3}, {NULL}};
const Segment SGI4138[] = {{&S36860[93], 6}, {NULL}};
const Segment SGI4139[] = {{&S36861[50], 7}, {NULL}};
const Segment SGI4140[] = {{&S36861[50], 29}, {NULL}};
const Segment SGI4141[] = {{&S36861[30], 12}, {NULL}};
const Segment SGI4142[] = {{&S36861[128], 3}, {NULL}};
const Segment SGI4143[] = {{&S36861[93], 6}, {NULL}};
const Segment SGI4144[] = {{&S36862[50], 7}, {NULL}};
const Segment SGI4145[] = {{&S36862[50], 29}, {NULL}};
const Segment SGI4146[] = {{&S36862[30], 12}, {NULL}};
const Segment SGI4147[] = {{&S36862[128], 3}, {NULL}};
const Segment SGI4148[] = {{&S36862[93], 6}, {NULL}};
const Segment SGI4149[] = {{&S36863[50], 7}, {NULL}};
const Segment SGI4150[] = {{&S36863[50], 29}, {NULL}};
const Segment SGI4151[] = {{&S36863[30], 12}, {NULL}};
const Segment SGI4152[] = {{&S36863[128], 3}, {NULL}};
const Segment SGI4153[] = {{&S36863[93], 6}, {NULL}};
const Segment SGI4154[] = {{&S36864[50], 7}, {NULL}};
const Segment SGI4155[] = {{&S36864[50], 28}, {NULL}};
const Segment SGI4156[] = {{&S36864[30], 12}, {NULL}};
const Segment SGI4157[] = {{&S36864[127], 3}, {NULL}};
const Segment SGI4158[] = {{&S36864[92], 6}, {NULL}};
const Segment SGI4159[] = {{&S36865[44], 7}, {NULL}};
const Segment SGI4160[] = {{&S36865[31], 12}, {NULL}};
const Segment SGI4161[] = {{&S36866[50], 7}, {NULL}};
const Segment SGI4162[] = {{&S36866[50], 29}, {NULL}};
const Segment SGI4163[] = {{&S36866[30], 12}, {NULL}};
const Segment SGI4164[] = {{&S36866[96], 4}, {NULL}};
const Segment SGI4165[] = {{&S36866[96], 3}, {NULL}};
const Segment SGI4166[] = {{&S36390[44], 7}, {NULL}};
const Segment SGI4167[] = {{&S36390[44], 28}, {NULL}};
const Segment SGI4168[] = {{&S36390[30], 6}, {NULL}};
const Segment SGI4169[] = {{&S36390[120], 3}, {NULL}};
const Segment SGI4170[] = {{&S36390[86], 5}, {NULL}};
const Segment SGI4171[] = {{&S36391[44], 7}, {NULL}};
const Segment SGI4172[] = {{&S36391[44], 58}, {NULL}};
const Segment SGI4173[] = {{&S36391[30], 6}, {NULL}};
const Segment SGI4174[] = {{&S36391[150], 3}, {NULL}};
const Segment SGI4175[] = {{&S36391[116], 5}, {NULL}};
const Segment SGI4176[] = {{&S36396[44], 7}, {NULL}};
const Segment SGI4177[] = {{&S36396[44], 63}, {NULL}};
const Segment SGI4178[] = {{&S36396[30], 6}, {NULL}};
const Segment SGI4179[] = {{&S36396[70], 3}, {NULL}};
const Segment SGI4180[] = {{&S36396[121], 5}, {NULL}};
const Segment SGI4181[] = {{&S36397[44], 7}, {NULL}};
const Segment SGI4182[] = {{&S36397[44], 67}, {NULL}};
const Segment SGI4183[] = {{&S36397[30], 6}, {NULL}};
const Segment SGI4184[] = {{&S36397[159], 3}, {NULL}};
const Segment SGI4185[] = {{&S36397[125], 5}, {NULL}};
const Segment SGI4186[] = {{&S36398[44], 7}, {NULL}};
const Segment SGI4187[] = {{&S36398[44], 27}, {NULL}};
const Segment SGI4188[] = {{&S36398[30], 6}, {NULL}};
const Segment SGI4189[] = {{&S36398[119], 3}, {NULL}};
const Segment SGI4190[] = {{&S36398[85], 5}, {NULL}};
const Segment SGI4191[] = {{&S36401[44], 7}, {NULL}};
const Segment SGI4192[] = {{&S36401[44], 58}, {NULL}};
const Segment SGI4193[] = {{&S36401[30], 6}, {NULL}};
const Segment SGI4194[] = {{&S36401[150], 3}, {NULL}};
const Segment SGI4195[] = {{&S36401[116], 5}, {NULL}};
const Segment SGI4196[] = {{&S36402[44], 7}, {NULL}};
const Segment SGI4197[] = {{&S36402[44], 58}, {NULL}};
const Segment SGI4198[] = {{&S36402[30], 6}, {NULL}};
const Segment SGI4199[] = {{&S36402[150], 3}, {NULL}};
const Segment SGI4200[] = {{&S36402[116], 5}, {NULL}};
const Segment SGI4201[] = {{&S36403[44], 7}, {NULL}};
const Segment SGI4202[] = {{&S36403[44], 58}, {NULL}};
const Segment SGI4203[] = {{&S36403[30], 6}, {NULL}};
const Segment SGI4204[] = {{&S36403[150], 3}, {NULL}};
const Segment SGI4205[] = {{&S36403[116], 5}, {NULL}};
const Segment SGI4206[] = {{&S36404[44], 7}, {NULL}};
const Segment SGI4207[] = {{&S36404[44], 27}, {NULL}};
const Segment SGI4208[] = {{&S36404[30], 6}, {NULL}};
const Segment SGI4209[] = {{&S36404[119], 3}, {NULL}};
const Segment SGI4210[] = {{&S36404[85], 5}, {NULL}};
const Segment SGI4211[] = {{&S36405[44], 7}, {NULL}};
const Segment SGI4212[] = {{&S36405[44], 60}, {NULL}};
const Segment SGI4213[] = {{&S36405[30], 6}, {NULL}};
const Segment SGI4214[] = {{&S36405[70], 3}, {NULL}};
const Segment SGI4215[] = {{&S36405[118], 5}, {NULL}};
const Segment SGI4216[] = {{&S36406[44], 7}, {NULL}};
const Segment SGI4217[] = {{&S36406[44], 58}, {NULL}};
const Segment SGI4218[] = {{&S36406[30], 6}, {NULL}};
const Segment SGI4219[] = {{&S36406[150], 3}, {NULL}};
const Segment SGI4220[] = {{&S36406[116], 5}, {NULL}};
const Segment SGI4221[] = {{&S36407[44], 7}, {NULL}};
const Segment SGI4222[] = {{&S36407[44], 57}, {NULL}};
const Segment SGI4223[] = {{&S36407[30], 6}, {NULL}};
const Segment SGI4224[] = {{&S36407[149], 3}, {NULL}};
const Segment SGI4225[] = {{&S36407[115], 5}, {NULL}};
const Segment SGI4226[] = {{&S36408[44], 7}, {NULL}};
const Segment SGI4227[] = {{&S36408[44], 27}, {NULL}};
const Segment SGI4228[] = {{&S36408[30], 6}, {NULL}};
const Segment SGI4229[] = {{&S36408[119], 3}, {NULL}};
const Segment SGI4230[] = {{&S36408[85], 5}, {NULL}};
const Segment SGI4231[] = {{&S36409[44], 7}, {NULL}};
const Segment SGI4232[] = {{&S36409[44], 61}, {NULL}};
const Segment SGI4233[] = {{&S36409[30], 6}, {NULL}};
const Segment SGI4234[] = {{&S36409[70], 3}, {NULL}};
const Segment SGI4235[] = {{&S36409[119], 5}, {NULL}};
const Segment SGI4236[] = {{&S36412[44], 7}, {NULL}};
const Segment SGI4237[] = {{&S36412[44], 47}, {NULL}};
const Segment SGI4238[] = {{&S36412[30], 6}, {NULL}};
const Segment SGI4239[] = {{&S36412[139], 3}, {NULL}};
const Segment SGI4240[] = {{&S36412[105], 5}, {NULL}};
const Segment SGI4241[] = {{&S36413[44], 7}, {NULL}};
const Segment SGI4242[] = {{&S36413[44], 58}, {NULL}};
const Segment SGI4243[] = {{&S36413[30], 6}, {NULL}};
const Segment SGI4244[] = {{&S36413[150], 3}, {NULL}};
const Segment SGI4245[] = {{&S36413[116], 5}, {NULL}};
const Segment SGI4246[] = {{&S36414[44], 7}, {NULL}};
const Segment SGI4247[] = {{&S36414[44], 67}, {NULL}};
const Segment SGI4248[] = {{&S36414[30], 6}, {NULL}};
const Segment SGI4249[] = {{&S36414[159], 3}, {NULL}};
const Segment SGI4250[] = {{&S36414[125], 5}, {NULL}};
const Segment SGI4251[] = {{&S36437[44], 7}, {NULL}};
const Segment SGI4252[] = {{&S36437[44], 67}, {NULL}};
const Segment SGI4253[] = {{&S36437[30], 6}, {NULL}};
const Segment SGI4254[] = {{&S36437[159], 3}, {NULL}};
const Segment SGI4255[] = {{&S36437[125], 5}, {NULL}};
const Segment SGI4256[] = {{&S36443[55], 7}, {NULL}};
const Segment SGI4257[] = {{&S36443[55], 32}, {NULL}};
const Segment SGI4258[] = {{&S36443[41], 6}, {NULL}};
const Segment SGI4259[] = {{&S36443[135], 3}, {NULL}};
const Segment SGI4260[] = {{&S36443[101], 5}, {NULL}};
const Segment SGI4261[] = {{&S36449[44], 7}, {NULL}};
const Segment SGI4262[] = {{&S36449[44], 58}, {NULL}};
const Segment SGI4263[] = {{&S36449[30], 6}, {NULL}};
const Segment SGI4264[] = {{&S36449[150], 3}, {NULL}};
const Segment SGI4265[] = {{&S36449[116], 5}, {NULL}};
const Segment SGI4266[] = {{&S36466[44], 7}, {NULL}};
const Segment SGI4267[] = {{&S36466[44], 62}, {NULL}};
const Segment SGI4268[] = {{&S36466[30], 6}, {NULL}};
const Segment SGI4269[] = {{&S36466[70], 3}, {NULL}};
const Segment SGI4270[] = {{&S36466[120], 5}, {NULL}};
const Segment SGI4271[] = {{&S36469[38], 7}, {NULL}};
const Segment SGI4272[] = {{&S36469[31], 6}, {NULL}};
const Segment SGI4273[] = {{&S36306[42], 12}, {NULL}};
const Segment SGI4274[] = {{&S36306[42], 24}, {NULL}};
const Segment SGI4275[] = {{&S36306[30], 4}, {NULL}};
const Segment SGI4276[] = {{&S36306[114], 3}, {NULL}};
const Segment SGI4277[] = {{&S36306[80], 5}, {NULL}};
const Segment SGI4278[] = {{&S36307[42], 7}, {NULL}};
const Segment SGI4279[] = {{&S36307[42], 18}, {NULL}};
const Segment SGI4280[] = {{&S36307[30], 4}, {NULL}};
const Segment SGI4281[] = {{&S36307[108], 3}, {NULL}};
const Segment SGI4282[] = {{&S36307[74], 5}, {NULL}};
const Segment SGI4283[] = {{&S36308[42], 7}, {NULL}};
const Segment SGI4284[] = {{&S36308[42], 18}, {NULL}};
const Segment SGI4285[] = {{&S36308[30], 4}, {NULL}};
const Segment SGI4286[] = {{&S36308[108], 3}, {NULL}};
const Segment SGI4287[] = {{&S36308[74], 5}, {NULL}};
const Segment SGI4288[] = {{&S36309[42], 7}, {NULL}};
const Segment SGI4289[] = {{&S36309[42], 19}, {NULL}};
const Segment SGI4290[] = {{&S36309[30], 4}, {NULL}};
const Segment SGI4291[] = {{&S36309[109], 3}, {NULL}};
const Segment SGI4292[] = {{&S36309[75], 5}, {NULL}};
const Segment SGI4293[] = {{&S36310[42], 7}, {NULL}};
const Segment SGI4294[] = {{&S36310[42], 19}, {NULL}};
const Segment SGI4295[] = {{&S36310[30], 4}, {NULL}};
const Segment SGI4296[] = {{&S36310[109], 3}, {NULL}};
const Segment SGI4297[] = {{&S36310[75], 5}, {NULL}};
const Segment SGI4298[] = {{&S36311[42], 7}, {NULL}};
const Segment SGI4299[] = {{&S36311[42], 18}, {NULL}};
const Segment SGI4300[] = {{&S36311[30], 4}, {NULL}};
const Segment SGI4301[] = {{&S36311[108], 3}, {NULL}};
const Segment SGI4302[] = {{&S36311[74], 5}, {NULL}};
const Segment SGI4303[] = {{&S36312[42], 7}, {NULL}};
const Segment SGI4304[] = {{&S36312[42], 18}, {NULL}};
const Segment SGI4305[] = {{&S36312[30], 4}, {NULL}};
const Segment SGI4306[] = {{&S36312[108], 3}, {NULL}};
const Segment SGI4307[] = {{&S36312[74], 5}, {NULL}};
const Segment SGI4308[] = {{&S36313[42], 7}, {NULL}};
const Segment SGI4309[] = {{&S36313[42], 19}, {NULL}};
const Segment SGI4310[] = {{&S36313[30], 4}, {NULL}};
const Segment SGI4311[] = {{&S36313[109], 3}, {NULL}};
const Segment SGI4312[] = {{&S36313[75], 5}, {NULL}};
const Segment SGI4313[] = {{&S36319[42], 7}, {NULL}};
const Segment SGI4314[] = {{&S36319[42], 19}, {NULL}};
const Segment SGI4315[] = {{&S36319[30], 4}, {NULL}};
const Segment SGI4316[] = {{&S36319[109], 3}, {NULL}};
const Segment SGI4317[] = {{&S36319[75], 5}, {NULL}};
const Segment SGI4318[] = {{&S36325[42], 7}, {NULL}};
const Segment SGI4319[] = {{&S36325[42], 18}, {NULL}};
const Segment SGI4320[] = {{&S36325[30], 4}, {NULL}};
const Segment SGI4321[] = {{&S36325[108], 3}, {NULL}};
const Segment SGI4322[] = {{&S36325[74], 5}, {NULL}};
const Segment SGI4323[] = {{&S36331[36], 7}, {NULL}};
const Segment SGI4324[] = {{&S36331[31], 4}, {NULL}};
const Segment SGI4325[] = {{&S36332[42], 7}, {NULL}};
const Segment SGI4326[] = {{&S36332[42], 18}, {NULL}};
const Segment SGI4327[] = {{&S36332[30], 4}, {NULL}};
const Segment SGI4328[] = {{&S36332[108], 3}, {NULL}};
const Segment SGI4329[] = {{&S36332[74], 5}, {NULL}};
const Segment SGI4330[] = {{&S36333[42], 7}, {NULL}};
const Segment SGI4331[] = {{&S36333[42], 19}, {NULL}};
const Segment SGI4332[] = {{&S36333[30], 4}, {NULL}};
const Segment SGI4333[] = {{&S36333[109], 3}, {NULL}};
const Segment SGI4334[] = {{&S36333[75], 5}, {NULL}};
const Segment SGI4335[] = {{&S36619[44], 18}, {NULL}};
const Segment SGI4336[] = {{&S36619[44], 29}, {NULL}};
const Segment SGI4337[] = {{&S36619[30], 6}, {NULL}};
const Segment SGI4338[] = {{&S36619[121], 3}, {NULL}};
const Segment SGI4339[] = {{&S36619[87], 5}, {NULL}};
const Segment SGI4340[] = {{&S36620[44], 18}, {NULL}};
const Segment SGI4341[] = {{&S36620[44], 29}, {NULL}};
const Segment SGI4342[] = {{&S36620[30], 6}, {NULL}};
const Segment SGI4343[] = {{&S36620[121], 3}, {NULL}};
const Segment SGI4344[] = {{&S36620[87], 5}, {NULL}};
const Segment SGI4345[] = {{&S36621[44], 17}, {NULL}};
const Segment SGI4346[] = {{&S36621[44], 28}, {NULL}};
const Segment SGI4347[] = {{&S36621[30], 6}, {NULL}};
const Segment SGI4348[] = {{&S36621[120], 3}, {NULL}};
const Segment SGI4349[] = {{&S36621[86], 5}, {NULL}};
const Segment SGI4350[] = {{&S36622[44], 18}, {NULL}};
const Segment SGI4351[] = {{&S36622[44], 35}, {NULL}};
const Segment SGI4352[] = {{&S36622[30], 6}, {NULL}};
const Segment SGI4353[] = {{&S36623[44], 18}, {NULL}};
const Segment SGI4354[] = {{&S36623[44], 29}, {NULL}};
const Segment SGI4355[] = {{&S36623[30], 6}, {NULL}};
const Segment SGI4356[] = {{&S36623[122], 3}, {NULL}};
const Segment SGI4357[] = {{&S36623[87], 6}, {NULL}};
const Segment SGI4358[] = {{&S36624[43], 18}, {NULL}};
const Segment SGI4359[] = {{&S36624[43], 29}, {NULL}};
const Segment SGI4360[] = {{&S36624[120], 3}, {NULL}};
const Segment SGI4361[] = {{&S36624[86], 5}, {NULL}};
const Segment SGI4362[] = {{&S36625[38], 18}, {NULL}};
const Segment SGI4363[] = {{&S36625[31], 6}, {NULL}};
const Segment SGI4364[] = {{&S36626[44], 18}, {NULL}};
const Segment SGI4365[] = {{&S36626[44], 35}, {NULL}};
const Segment SGI4366[] = {{&S36626[30], 6}, {NULL}};
const Segment SGI4367[] = {{&S36627[44], 18}, {NULL}};
const Segment SGI4368[] = {{&S36627[44], 29}, {NULL}};
const Segment SGI4369[] = {{&S36627[30], 6}, {NULL}};
const Segment SGI4370[] = {{&S36627[90], 4}, {NULL}};
const Segment SGI4371[] = {{&S36627[90], 3}, {NULL}};
const Segment SGI4372[] = {{&S36628[42], 18}, {NULL}};
const Segment SGI4373[] = {{&S36628[30], 6}, {NULL}};
const Segment SGI4374[] = {{&S36628[119], 3}, {NULL}};
const Segment SGI4375[] = {{&S36628[85], 5}, {NULL}};
const Segment SGI4376[] = {{&S36629[44], 17}, {NULL}};
const Segment SGI4377[] = {{&S36629[44], 28}, {NULL}};
const Segment SGI4378[] = {{&S36629[30], 6}, {NULL}};
const Segment SGI4379[] = {{&S36629[120], 3}, {NULL}};
const Segment SGI4380[] = {{&S36629[86], 5}, {NULL}};
const Segment SGI4381[] = {{&S36630[44], 18}, {NULL}};
const Segment SGI4382[] = {{&S36630[44], 29}, {NULL}};
const Segment SGI4383[] = {{&S36630[30], 6}, {NULL}};
const Segment SGI4384[] = {{&S36630[123], 3}, {NULL}};
const Segment SGI4385[] = {{&S36630[87], 7}, {NULL}};
const Segment SGI4386[] = {{&S36630[143], 8}, {NULL}};
const Segment SGI4387[] = {{&S36631[43], 18}, {NULL}};
const Segment SGI4388[] = {{&S36631[43], 29}, {NULL}};
const Segment SGI4389[] = {{&S36631[30], 5}, {NULL}};
const Segment SGI4390[] = {{&S36631[120], 3}, {NULL}};
const Segment SGI4391[] = {{&S36631[86], 5}, {NULL}};
const Segment SGI4392[] = {{&S36473[42], 7}, {NULL}};
const Segment SGI4393[] = {{&S36473[42], 18}, {NULL}};
const Segment SGI4394[] = {{&S36473[30], 4}, {NULL}};
const Segment SGI4395[] = {{&S36473[108], 3}, {NULL}};
const Segment SGI4396[] = {{&S36473[74], 5}, {NULL}};
const Segment SGI4397[] = {{&S36873[44], 11}, {NULL}};
const Segment SGI4398[] = {{&S36873[44], 22}, {NULL}};
const Segment SGI4399[] = {{&S36873[30], 6}, {NULL}};
const Segment SGI4400[] = {{&S36873[114], 3}, {NULL}};
const Segment SGI4401[] = {{&S36873[80], 5}, {NULL}};
const Segment SGI4402[] = {{&S36874[44], 9}, {NULL}};
const Segment SGI4403[] = {{&S36874[44], 20}, {NULL}};
const Segment SGI4404[] = {{&S36874[30], 6}, {NULL}};
const Segment SGI4405[] = {{&S36874[112], 3}, {NULL}};
const Segment SGI4406[] = {{&S36874[78], 5}, {NULL}};
const Segment SGI4407[] = {{&S36881[44], 10}, {NULL}};
const Segment SGI4408[] = {{&S36881[44], 21}, {NULL}};
const Segment SGI4409[] = {{&S36881[30], 6}, {NULL}};
const Segment SGI4410[] = {{&S36881[113], 3}, {NULL}};
const Segment SGI4411[] = {{&S36881[79], 5}, {NULL}};
const Segment SGI4412[] = {{&S36887[38], 11}, {NULL}};
const Segment SGI4413[] = {{&S36887[31], 6}, {NULL}};
const Segment SGI4414[] = {{&S36888[38], 10}, {NULL}};
const Segment SGI4415[] = {{&S36888[31], 6}, {NULL}};
const Segment SGI4416[] = {{&S36890[44], 10}, {NULL}};
const Segment SGI4417[] = {{&S36890[44], 21}, {NULL}};
const Segment SGI4418[] = {{&S36890[30], 6}, {NULL}};
const Segment SGI4419[] = {{&S36890[114], 3}, {NULL}};
const Segment SGI4420[] = {{&S36890[79], 6}, {NULL}};
const Segment SGI4421[] = {{&S36901[44], 8}, {NULL}};
const Segment SGI4422[] = {{&S36901[44], 19}, {NULL}};
const Segment SGI4423[] = {{&S36901[30], 6}, {NULL}};
const Segment SGI4424[] = {{&S36901[113], 3}, {NULL}};
const Segment SGI4425[] = {{&S36901[77], 7}, {NULL}};
const Segment SGI4426[] = {{&S36901[133], 8}, {NULL}};
const Segment SGI4427[] = {{&S36903[44], 8}, {NULL}};
const Segment SGI4428[] = {{&S36903[44], 19}, {NULL}};
const Segment SGI4429[] = {{&S36903[30], 6}, {NULL}};
const Segment SGI4430[] = {{&S36903[111], 3}, {NULL}};
const Segment SGI4431[] = {{&S36903[77], 5}, {NULL}};
const Segment SGI4432[] = {{&S36907[38], 8}, {NULL}};
const Segment SGI4433[] = {{&S36907[31], 6}, {NULL}};
const Segment SGI4434[] = {{&S37518[50], 10}, {NULL}};
const Segment SGI4435[] = {{&S37518[50], 22}, {NULL}};
const Segment SGI4436[] = {{&S37518[30], 12}, {NULL}};
const Segment SGI4437[] = {{&S37518[120], 3}, {NULL}};
const Segment SGI4438[] = {{&S37518[86], 5}, {NULL}};
const Segment SGI4439[] = {{&S37519[49], 10}, {NULL}};
const Segment SGI4440[] = {{&S37519[49], 22}, {NULL}};
const Segment SGI4441[] = {{&S37519[120], 3}, {NULL}};
const Segment SGI4442[] = {{&S37519[85], 6}, {NULL}};
const Segment SGI4443[] = {{&S37520[50], 10}, {NULL}};
const Segment SGI4444[] = {{&S37520[50], 22}, {NULL}};
const Segment SGI4445[] = {{&S37520[30], 12}, {NULL}};
const Segment SGI4446[] = {{&S37520[121], 3}, {NULL}};
const Segment SGI4447[] = {{&S37520[86], 6}, {NULL}};
const Segment SGI4448[] = {{&S37521[50], 9}, {NULL}};
const Segment SGI4449[] = {{&S37521[50], 21}, {NULL}};
const Segment SGI4450[] = {{&S37521[30], 12}, {NULL}};
const Segment SGI4451[] = {{&S37521[120], 3}, {NULL}};
const Segment SGI4452[] = {{&S37521[85], 6}, {NULL}};
const Segment SGI4453[] = {{&S37522[50], 9}, {NULL}};
const Segment SGI4454[] = {{&S37522[50], 21}, {NULL}};
const Segment SGI4455[] = {{&S37522[30], 12}, {NULL}};
const Segment SGI4456[] = {{&S37522[120], 3}, {NULL}};
const Segment SGI4457[] = {{&S37522[85], 6}, {NULL}};
const Segment SGI4458[] = {{&S37523[49], 10}, {NULL}};
const Segment SGI4459[] = {{&S37523[49], 22}, {NULL}};
const Segment SGI4460[] = {{&S37523[30], 11}, {NULL}};
const Segment SGI4461[] = {{&S37523[120], 3}, {NULL}};
const Segment SGI4462[] = {{&S37523[85], 6}, {NULL}};
const Segment SGI4463[] = {{&S37524[50], 10}, {NULL}};
const Segment SGI4464[] = {{&S37524[50], 22}, {NULL}};
const Segment SGI4465[] = {{&S37524[30], 12}, {NULL}};
const Segment SGI4466[] = {{&S37524[121], 3}, {NULL}};
const Segment SGI4467[] = {{&S37524[86], 6}, {NULL}};
const Segment SGI4468[] = {{&S37525[48], 10}, {NULL}};
const Segment SGI4469[] = {{&S37525[30], 12}, {NULL}};
const Segment SGI4470[] = {{&S37525[119], 3}, {NULL}};
const Segment SGI4471[] = {{&S37525[84], 6}, {NULL}};
const Segment SGI4472[] = {{&S37526[50], 10}, {NULL}};
const Segment SGI4473[] = {{&S37526[50], 22}, {NULL}};
const Segment SGI4474[] = {{&S37526[30], 12}, {NULL}};
const Segment SGI4475[] = {{&S37526[122], 3}, {NULL}};
const Segment SGI4476[] = {{&S37526[86], 7}, {NULL}};
const Segment SGI4477[] = {{&S37526[142], 8}, {NULL}};
const Segment SGI4478[] = {{&S37527[44], 10}, {NULL}};
const Segment SGI4479[] = {{&S37527[31], 12}, {NULL}};
const Segment SGI4480[] = {{&S37528[50], 11}, {NULL}};
const Segment SGI4481[] = {{&S37528[50], 23}, {NULL}};
const Segment SGI4482[] = {{&S37528[30], 12}, {NULL}};
const Segment SGI4483[] = {{&S37528[122], 3}, {NULL}};
const Segment SGI4484[] = {{&S37528[87], 6}, {NULL}};
const Segment SGI4485[] = {{&S37529[44], 11}, {NULL}};
const Segment SGI4486[] = {{&S37529[31], 12}, {NULL}};
const Segment SGI4487[] = {{&S36867[41], 10}, {NULL}};
const Segment SGI4488[] = {{&S36867[41], 21}, {NULL}};
const Segment SGI4489[] = {{&S36867[30], 3}, {NULL}};
const Segment SGI4490[] = {{&S36867[110], 3}, {NULL}};
const Segment SGI4491[] = {{&S36867[76], 5}, {NULL}};
const Segment SGI4492[] = {{&S36868[42], 10}, {NULL}};
const Segment SGI4493[] = {{&S36868[42], 21}, {NULL}};
const Segment SGI4494[] = {{&S36868[30], 4}, {NULL}};
const Segment SGI4495[] = {{&S36868[111], 3}, {NULL}};
const Segment SGI4496[] = {{&S36868[77], 5}, {NULL}};
const Segment SGI4497[] = {{&S36869[59], 10}, {NULL}};
const Segment SGI4498[] = {{&S36869[59], 21}, {NULL}};
const Segment SGI4499[] = {{&S36869[30], 21}, {NULL}};
const Segment SGI4500[] = {{&S36869[128], 3}, {NULL}};
const Segment SGI4501[] = {{&S36869[94], 5}, {NULL}};
const Segment SGI4502[] = {{&S36870[40], 10}, {NULL}};
const Segment SGI4503[] = {{&S36870[30], 4}, {NULL}};
const Segment SGI4504[] = {{&S36870[109], 3}, {NULL}};
const Segment SGI4505[] = {{&S36870[75], 5}, {NULL}};
const Segment SGI4506[] = {{&S36871[42], 10}, {NULL}};
const Segment SGI4507[] = {{&S36871[42], 21}, {NULL}};
const Segment SGI4508[] = {{&S36871[30], 4}, {NULL}};
const Segment SGI4509[] = {{&S36871[112], 3}, {NULL}};
const Segment SGI4510[] = {{&S36871[77], 6}, {NULL}};
const Segment SGI4511[] = {{&S36872[42], 10}, {NULL}};
const Segment SGI4512[] = {{&S36872[42], 24}, {NULL}};
const Segment SGI4513[] = {{&S36872[30], 4}, {NULL}};
const Segment SGI4514[] = {{&S36872[114], 3}, {NULL}};
const Segment SGI4515[] = {{&S36872[80], 5}, {NULL}};
const Segment SGI4516[] = {{&S36875[42], 10}, {NULL}};
const Segment SGI4517[] = {{&S36875[42], 21}, {NULL}};
const Segment SGI4518[] = {{&S36875[30], 4}, {NULL}};
const Segment SGI4519[] = {{&S36875[112], 3}, {NULL}};
const Segment SGI4520[] = {{&S36875[77], 6}, {NULL}};
const Segment SGI4521[] = {{&S36876[50], 10}, {NULL}};
const Segment SGI4522[] = {{&S36876[50], 21}, {NULL}};
const Segment SGI4523[] = {{&S36876[30], 12}, {NULL}};
const Segment SGI4524[] = {{&S36876[119], 3}, {NULL}};
const Segment SGI4525[] = {{&S36876[85], 5}, {NULL}};
const Segment SGI4526[] = {{&S36877[42], 9}, {NULL}};
const Segment SGI4527[] = {{&S36877[42], 20}, {NULL}};
const Segment SGI4528[] = {{&S36877[30], 4}, {NULL}};
const Segment SGI4529[] = {{&S36877[110], 3}, {NULL}};
const Segment SGI4530[] = {{&S36877[76], 5}, {NULL}};
const Segment SGI4531[] = {{&S36878[42], 10}, {NULL}};
const Segment SGI4532[] = {{&S36878[42], 26}, {NULL}};
const Segment SGI4533[] = {{&S36878[30], 4}, {NULL}};
const Segment SGI4534[] = {{&S36878[116], 3}, {NULL}};
const Segment SGI4535[] = {{&S36878[82], 5}, {NULL}};
const Segment SGI4536[] = {{&S36879[41], 10}, {NULL}};
const Segment SGI4537[] = {{&S36879[41], 21}, {NULL}};
const Segment SGI4538[] = {{&S36879[110], 3}, {NULL}};
const Segment SGI4539[] = {{&S36879[76], 5}, {NULL}};
const Segment SGI4540[] = {{&S36880[42], 9}, {NULL}};
const Segment SGI4541[] = {{&S36880[42], 20}, {NULL}};
const Segment SGI4542[] = {{&S36880[30], 4}, {NULL}};
const Segment SGI4543[] = {{&S36880[110], 3}, {NULL}};
const Segment SGI4544[] = {{&S36880[76], 5}, {NULL}};
const Segment SGI4545[] = {{&S36882[42], 10}, {NULL}};
const Segment SGI4546[] = {{&S36882[42], 21}, {NULL}};
const Segment SGI4547[] = {{&S36882[30], 4}, {NULL}};
const Segment SGI4548[] = {{&S36882[111], 3}, {NULL}};
const Segment SGI4549[] = {{&S36882[77], 5}, {NULL}};
const Segment SGI4550[] = {{&S36883[42], 11}, {NULL}};
const Segment SGI4551[] = {{&S36883[42], 22}, {NULL}};
const Segment SGI4552[] = {{&S36883[30], 4}, {NULL}};
const Segment SGI4553[] = {{&S36883[112], 3}, {NULL}};
const Segment SGI4554[] = {{&S36883[78], 5}, {NULL}};
const Segment SGI4555[] = {{&S36884[36], 10}, {NULL}};
const Segment SGI4556[] = {{&S36884[31], 4}, {NULL}};
const Segment SGI4557[] = {{&S36885[42], 10}, {NULL}};
const Segment SGI4558[] = {{&S36885[42], 21}, {NULL}};
const Segment SGI4559[] = {{&S36885[30], 4}, {NULL}};
const Segment SGI4560[] = {{&S36885[111], 3}, {NULL}};
const Segment SGI4561[] = {{&S36885[77], 5}, {NULL}};
const Segment SGI4562[] = {{&S36886[36], 11}, {NULL}};
const Segment SGI4563[] = {{&S36886[31], 4}, {NULL}};
const Segment SGI4564[] = {{&S36889[42], 10}, {NULL}};
const Segment SGI4565[] = {{&S36889[42], 21}, {NULL}};
const Segment SGI4566[] = {{&S36889[30], 4}, {NULL}};
const Segment SGI4567[] = {{&S36889[80], 4}, {NULL}};
const Segment SGI4568[] = {{&S36889[80], 3}, {NULL}};
const Segment SGI4569[] = {{&S36900[42], 8}, {NULL}};
const Segment SGI4570[] = {{&S36900[42], 19}, {NULL}};
const Segment SGI4571[] = {{&S36900[30], 4}, {NULL}};
const Segment SGI4572[] = {{&S36900[111], 3}, {NULL}};
const Segment SGI4573[] = {{&S36900[75], 7}, {NULL}};
const Segment SGI4574[] = {{&S36900[131], 8}, {NULL}};
const Segment SGI4575[] = {{&S37008[43], 11}, {NULL}};
const Segment SGI4576[] = {{&S37008[43], 22}, {NULL}};
const Segment SGI4577[] = {{&S37008[30], 5}, {NULL}};
const Segment SGI4578[] = {{&S37008[113], 3}, {NULL}};
const Segment SGI4579[] = {{&S36902[42], 8}, {NULL}};
const Segment SGI4580[] = {{&S37008[79], 5}, {NULL}};
const Segment SGI4581[] = {{&S36902[42], 19}, {NULL}};
const Segment SGI4582[] = {{&S36902[30], 4}, {NULL}};
const Segment SGI4583[] = {{&S36902[109], 3}, {NULL}};
const Segment SGI4584[] = {{&S36902[75], 5}, {NULL}};
const Segment SGI4585[] = {{&S37009[44], 11}, {NULL}};
const Segment SGI4586[] = {{&S37009[44], 22}, {NULL}};
const Segment SGI4587[] = {{&S37009[30], 6}, {NULL}};
const Segment SGI4588[] = {{&S37009[114], 3}, {NULL}};
const Segment SGI4589[] = {{&S37009[80], 5}, {NULL}};
const Segment SGI4590[] = {{&S37010[44], 10}, {NULL}};
const Segment SGI4591[] = {{&S37010[44], 21}, {NULL}};
const Segment SGI4592[] = {{&S37010[30], 6}, {NULL}};
const Segment SGI4593[] = {{&S37010[113], 3}, {NULL}};
const Segment SGI4594[] = {{&S37010[79], 5}, {NULL}};
const Segment SGI4595[] = {{&S36906[36], 8}, {NULL}};
const Segment SGI4596[] = {{&S36906[31], 4}, {NULL}};
const Segment SGI4597[] = {{&S37011[43], 11}, {NULL}};
const Segment SGI4598[] = {{&S37011[43], 22}, {NULL}};
const Segment SGI4599[] = {{&S37011[113], 3}, {NULL}};
const Segment SGI4600[] = {{&S37011[79], 5}, {NULL}};
const Segment SGI4601[] = {{&S37012[44], 10}, {NULL}};
const Segment SGI4602[] = {{&S37012[44], 21}, {NULL}};
const Segment SGI4603[] = {{&S37012[30], 6}, {NULL}};
const Segment SGI4604[] = {{&S37012[113], 3}, {NULL}};
const Segment SGI4605[] = {{&S37012[79], 5}, {NULL}};
const Segment SGI4606[] = {{&S37014[42], 11}, {NULL}};
const Segment SGI4607[] = {{&S37014[30], 6}, {NULL}};
const Segment SGI4608[] = {{&S37014[112], 3}, {NULL}};
const Segment SGI4609[] = {{&S37014[78], 5}, {NULL}};
const Segment SGI4610[] = {{&S37015[44], 11}, {NULL}};
const Segment SGI4611[] = {{&S37015[44], 22}, {NULL}};
const Segment SGI4612[] = {{&S37015[30], 6}, {NULL}};
const Segment SGI4613[] = {{&S37015[114], 3}, {NULL}};
const Segment SGI4614[] = {{&S37015[80], 5}, {NULL}};
const Segment SGI4615[] = {{&S36915[42], 11}, {NULL}};
const Segment SGI4616[] = {{&S36915[42], 22}, {NULL}};
const Segment SGI4617[] = {{&S36915[30], 4}, {NULL}};
const Segment SGI4618[] = {{&S36915[112], 3}, {NULL}};
const Segment SGI4619[] = {{&S36915[78], 5}, {NULL}};
const Segment SGI4620[] = {{&S37016[44], 11}, {NULL}};
const Segment SGI4621[] = {{&S37016[44], 22}, {NULL}};
const Segment SGI4622[] = {{&S37016[30], 6}, {NULL}};
const Segment SGI4623[] = {{&S37016[114], 3}, {&S37016[156], 3}, {NULL}};
const Segment SGI4624[] = {{&S37016[80], 5}, {NULL}};
const Segment SGI4625[] = {{&S37017[38], 11}, {NULL}};
const Segment SGI4626[] = {{&S37017[31], 6}, {NULL}};
const Segment SGI4627[] = {{&S37018[44], 11}, {NULL}};
const Segment SGI4628[] = {{&S37018[44], 29}, {NULL}};
const Segment SGI4629[] = {{&S37018[30], 6}, {NULL}};
const Segment SGI4630[] = {{&S37019[44], 11}, {NULL}};
const Segment SGI4631[] = {{&S37019[44], 22}, {NULL}};
const Segment SGI4632[] = {{&S37019[30], 6}, {NULL}};
const Segment SGI4633[] = {{&S37019[116], 3}, {NULL}};
const Segment SGI4634[] = {{&S37019[80], 7}, {NULL}};
const Segment SGI4635[] = {{&S37019[136], 8}, {NULL}};
const Segment SGI4636[] = {{&S37013[42], 11}, {NULL}};
const Segment SGI4637[] = {{&S37013[42], 22}, {NULL}};
const Segment SGI4638[] = {{&S37013[30], 4}, {NULL}};
const Segment SGI4639[] = {{&S37013[112], 3}, {NULL}};
const Segment SGI4640[] = {{&S37013[78], 5}, {NULL}};
const Segment SGI4641[] = {{&S38747[50], 10}, {NULL}};
const Segment SGI4642[] = {{&S38747[50], 27}, {NULL}};
const Segment SGI4643[] = {{&S38747[30], 12}, {NULL}};
const Segment SGI4644[] = {{&S38747[125], 3}, {NULL}};
const Segment SGI4645[] = {{&S38747[91], 5}, {NULL}};
const Segment SGI4646[] = {{&S37279[44], 8}, {NULL}};
const Segment SGI4647[] = {{&S37279[44], 19}, {NULL}};
const Segment SGI4648[] = {{&S37279[30], 6}, {NULL}};
const Segment SGI4649[] = {{&S37279[111], 3}, {NULL}};
const Segment SGI4650[] = {{&S37279[77], 5}, {NULL}};
const Segment SGI4651[] = {{&S37265[42], 23}, {NULL}};
const Segment SGI4652[] = {{&S37265[42], 34}, {NULL}};
const Segment SGI4653[] = {{&S37265[30], 4}, {NULL}};
const Segment SGI4654[] = {{&S37265[124], 3}, {NULL}};
const Segment SGI4655[] = {{&S37265[90], 5}, {NULL}};
const Segment SGI4656[] = {{&S37266[42], 23}, {NULL}};
const Segment SGI4657[] = {{&S37266[42], 34}, {NULL}};
const Segment SGI4658[] = {{&S37266[30], 4}, {NULL}};
const Segment SGI4659[] = {{&S37266[124], 3}, {NULL}};
const Segment SGI4660[] = {{&S37266[90], 5}, {NULL}};
const Segment SGI4661[] = {{&S37267[40], 12}, {NULL}};
const Segment SGI4662[] = {{&S37267[30], 4}, {NULL}};
const Segment SGI4663[] = {{&S37267[111], 3}, {NULL}};
const Segment SGI4664[] = {{&S37267[77], 5}, {NULL}};
const Segment SGI4665[] = {{&S37268[42], 12}, {NULL}};
const Segment SGI4666[] = {{&S37268[42], 23}, {NULL}};
const Segment SGI4667[] = {{&S37268[30], 4}, {NULL}};
const Segment SGI4668[] = {{&S37268[113], 3}, {NULL}};
const Segment SGI4669[] = {{&S37268[79], 5}, {NULL}};
const Segment SGI4670[] = {{&S37269[42], 23}, {NULL}};
const Segment SGI4671[] = {{&S37269[42], 34}, {NULL}};
const Segment SGI4672[] = {{&S37269[30], 4}, {NULL}};
const Segment SGI4673[] = {{&S37269[124], 3}, {NULL}};
const Segment SGI4674[] = {{&S37269[90], 5}, {NULL}};
const Segment SGI4675[] = {{&S37270[42], 23}, {NULL}};
const Segment SGI4676[] = {{&S37270[42], 34}, {NULL}};
const Segment SGI4677[] = {{&S37270[30], 4}, {NULL}};
const Segment SGI4678[] = {{&S37270[124], 3}, {NULL}};
const Segment SGI4679[] = {{&S37270[90], 5}, {NULL}};
const Segment SGI4680[] = {{&S37271[42], 23}, {NULL}};
const Segment SGI4681[] = {{&S37271[42], 34}, {NULL}};
const Segment SGI4682[] = {{&S37271[30], 4}, {NULL}};
const Segment SGI4683[] = {{&S37271[124], 3}, {NULL}};
const Segment SGI4684[] = {{&S37271[90], 5}, {NULL}};
const Segment SGI4685[] = {{&S37272[42], 23}, {NULL}};
const Segment SGI4686[] = {{&S37272[42], 34}, {NULL}};
const Segment SGI4687[] = {{&S37272[30], 4}, {NULL}};
const Segment SGI4688[] = {{&S37272[124], 3}, {NULL}};
const Segment SGI4689[] = {{&S37272[90], 5}, {NULL}};
const Segment SGI4690[] = {{&S37273[42], 23}, {NULL}};
const Segment SGI4691[] = {{&S37273[42], 34}, {NULL}};
const Segment SGI4692[] = {{&S37273[30], 4}, {NULL}};
const Segment SGI4693[] = {{&S37273[124], 3}, {NULL}};
const Segment SGI4694[] = {{&S37273[90], 5}, {NULL}};
const Segment SGI4695[] = {{&S37274[42], 23}, {NULL}};
const Segment SGI4696[] = {{&S37274[42], 34}, {NULL}};
const Segment SGI4697[] = {{&S37274[30], 4}, {NULL}};
const Segment SGI4698[] = {{&S37274[124], 3}, {NULL}};
const Segment SGI4699[] = {{&S37274[90], 5}, {NULL}};
const Segment SGI4700[] = {{&S37275[36], 12}, {NULL}};
const Segment SGI4701[] = {{&S37275[31], 4}, {NULL}};
const Segment SGI4702[] = {{&S37276[42], 12}, {NULL}};
const Segment SGI4703[] = {{&S37276[42], 23}, {NULL}};
const Segment SGI4704[] = {{&S37276[30], 4}, {NULL}};
const Segment SGI4705[] = {{&S37276[82], 4}, {NULL}};
const Segment SGI4706[] = {{&S37276[82], 3}, {NULL}};
const Segment SGI4707[] = {{&S37277[42], 23}, {NULL}};
const Segment SGI4708[] = {{&S37277[42], 34}, {NULL}};
const Segment SGI4709[] = {{&S37277[30], 4}, {NULL}};
const Segment SGI4710[] = {{&S37277[124], 3}, {NULL}};
const Segment SGI4711[] = {{&S37277[90], 5}, {NULL}};
const Segment SGI4712[] = {{&S37278[42], 8}, {NULL}};
const Segment SGI4713[] = {{&S37278[42], 19}, {NULL}};
const Segment SGI4714[] = {{&S37278[30], 4}, {NULL}};
const Segment SGI4715[] = {{&S37278[109], 3}, {NULL}};
const Segment SGI4716[] = {{&S37278[75], 5}, {NULL}};
const Segment SGI4717[] = {{&S37280[42], 12}, {NULL}};
const Segment SGI4718[] = {{&S37280[42], 23}, {NULL}};
const Segment SGI4719[] = {{&S37280[30], 4}, {NULL}};
const Segment SGI4720[] = {{&S37280[82], 6}, {NULL}};
const Segment SGI4721[] = {{&S37556[44], 10}, {NULL}};
const Segment SGI4722[] = {{&S37556[44], 21}, {NULL}};
const Segment SGI4723[] = {{&S37556[30], 6}, {NULL}};
const Segment SGI4724[] = {{&S37556[113], 3}, {NULL}};
const Segment SGI4725[] = {{&S37556[79], 5}, {NULL}};
const Segment SGI4726[] = {{&S37557[38], 10}, {NULL}};
const Segment SGI4727[] = {{&S37557[31], 6}, {NULL}};
const Segment SGI4728[] = {{&S37559[42], 10}, {NULL}};
const Segment SGI4729[] = {{&S37559[30], 6}, {NULL}};
const Segment SGI4730[] = {{&S37559[111], 3}, {NULL}};
const Segment SGI4731[] = {{&S37559[77], 5}, {NULL}};
const Segment SGI4732[] = {{&S37560[44], 10}, {NULL}};
const Segment SGI4733[] = {{&S37560[44], 21}, {NULL}};
const Segment SGI4734[] = {{&S37560[30], 6}, {NULL}};
const Segment SGI4735[] = {{&S37560[82], 4}, {NULL}};
const Segment SGI4736[] = {{&S37560[82], 3}, {NULL}};
const Segment SGI4737[] = {{&S37563[44], 10}, {NULL}};
const Segment SGI4738[] = {{&S37563[44], 21}, {NULL}};
const Segment SGI4739[] = {{&S37563[30], 6}, {NULL}};
const Segment SGI4740[] = {{&S37563[82], 6}, {NULL}};
const Segment SGI4741[] = {{&S37565[45], 10}, {NULL}};
const Segment SGI4742[] = {{&S37565[45], 27}, {NULL}};
const Segment SGI4743[] = {{&S37565[31], 6}, {NULL}};
const Segment SGI4744[] = {{&S37566[44], 10}, {NULL}};
const Segment SGI4745[] = {{&S37566[44], 27}, {NULL}};
const Segment SGI4746[] = {{&S37566[30], 6}, {NULL}};
const Segment SGI4747[] = {{&S37567[44], 9}, {NULL}};
const Segment SGI4748[] = {{&S37567[30], 6}, {NULL}};
const Segment SGI4749[] = {{&S37568[44], 10}, {NULL}};
const Segment SGI4750[] = {{&S37568[44], 27}, {NULL}};
const Segment SGI4751[] = {{&S37568[30], 6}, {NULL}};
const Segment SGI4752[] = {{&S37569[45], 10}, {NULL}};
const Segment SGI4753[] = {{&S37569[45], 27}, {NULL}};
const Segment SGI4754[] = {{&S37569[31], 6}, {NULL}};
const Segment SGI4755[] = {{&S37570[44], 10}, {NULL}};
const Segment SGI4756[] = {{&S37570[44], 27}, {NULL}};
const Segment SGI4757[] = {{&S37570[30], 6}, {NULL}};
const Segment SGI4758[] = {{&S37572[46], 10}, {NULL}};
const Segment SGI4759[] = {{&S37572[46], 27}, {NULL}};
const Segment SGI4760[] = {{&S37572[32], 6}, {NULL}};
const Segment SGI4761[] = {{&S37573[44], 10}, {NULL}};
const Segment SGI4762[] = {{&S37573[44], 19}, {NULL}};
const Segment SGI4763[] = {{&S37573[30], 6}, {NULL}};
const Segment SGI4764[] = {{&S37573[111], 3}, {NULL}};
const Segment SGI4765[] = {{&S37573[77], 5}, {NULL}};
const Segment SGI4766[] = {{&S37574[44], 10}, {NULL}};
const Segment SGI4767[] = {{&S37574[44], 21}, {NULL}};
const Segment SGI4768[] = {{&S37574[30], 6}, {NULL}};
const Segment SGI4769[] = {{&S37574[126], 3}, {NULL}};
const Segment SGI4770[] = {{&S37574[79], 6}, {NULL}};
const Segment SGI4771[] = {{&S37575[59], 10}, {NULL}};
const Segment SGI4772[] = {{&S37575[59], 27}, {NULL}};
const Segment SGI4773[] = {{&S37575[45], 6}, {NULL}};
const Segment SGI4774[] = {{&S37576[42], 10}, {NULL}};
const Segment SGI4775[] = {{&S37576[42], 21}, {NULL}};
const Segment SGI4776[] = {{&S37576[30], 4}, {NULL}};
const Segment SGI4777[] = {{&S37576[111], 3}, {NULL}};
const Segment SGI4778[] = {{&S37576[77], 5}, {NULL}};
const Segment SGI4779[] = {{&S38340[50], 13}, {NULL}};
const Segment SGI4780[] = {{&S38340[50], 24}, {NULL}};
const Segment SGI4781[] = {{&S38340[30], 12}, {NULL}};
const Segment SGI4782[] = {{&S38340[124], 3}, {NULL}};
const Segment SGI4783[] = {{&S38340[88], 7}, {NULL}};
const Segment SGI4784[] = {{&S38340[144], 8}, {NULL}};
const Segment SGI4785[] = {{&S38341[50], 13}, {NULL}};
const Segment SGI4786[] = {{&S38341[50], 24}, {NULL}};
const Segment SGI4787[] = {{&S38341[30], 12}, {NULL}};
const Segment SGI4788[] = {{&S38341[123], 3}, {NULL}};
const Segment SGI4789[] = {{&S38341[88], 6}, {NULL}};
const Segment SGI4790[] = {{&S38342[50], 6}, {NULL}};
const Segment SGI4791[] = {{&S38342[50], 17}, {NULL}};
const Segment SGI4792[] = {{&S38342[30], 12}, {NULL}};
const Segment SGI4793[] = {{&S38342[116], 3}, {&S38342[159], 3}, {NULL}};
const Segment SGI4794[] = {{&S38342[81], 6}, {NULL}};
const Segment SGI4795[] = {{&S38344[48], 6}, {NULL}};
const Segment SGI4796[] = {{&S38344[30], 12}, {NULL}};
const Segment SGI4797[] = {{&S38344[114], 3}, {NULL}};
const Segment SGI4798[] = {{&S38344[79], 6}, {NULL}};
const Segment SGI4799[] = {{&S38346[50], 6}, {NULL}};
const Segment SGI4800[] = {{&S38346[50], 17}, {NULL}};
const Segment SGI4801[] = {{&S38346[30], 12}, {NULL}};
const Segment SGI4802[] = {{&S38346[116], 3}, {NULL}};
const Segment SGI4803[] = {{&S38346[81], 6}, {NULL}};
const Segment SGI4804[] = {{&S38347[50], 6}, {NULL}};
const Segment SGI4805[] = {{&S38347[50], 17}, {NULL}};
const Segment SGI4806[] = {{&S38347[30], 12}, {NULL}};
const Segment SGI4807[] = {{&S38347[116], 3}, {&S38347[31], 3}, {NULL}};
const Segment SGI4808[] = {{&S38347[81], 6}, {NULL}};
const Segment SGI4809[] = {{&S38348[44], 6}, {NULL}};
const Segment SGI4810[] = {{&S38348[31], 12}, {NULL}};
const Segment SGI4811[] = {{&S38349[44], 13}, {NULL}};
const Segment SGI4812[] = {{&S38349[31], 12}, {NULL}};
const Segment SGI4813[] = {{&S38350[50], 6}, {NULL}};
const Segment SGI4814[] = {{&S38350[50], 17}, {NULL}};
const Segment SGI4815[] = {{&S38350[30], 12}, {NULL}};
const Segment SGI4816[] = {{&S38350[117], 3}, {NULL}};
const Segment SGI4817[] = {{&S38350[81], 7}, {NULL}};
const Segment SGI4818[] = {{&S38350[137], 8}, {NULL}};
const Segment SGI4819[] = {{&S38351[42], 9}, {NULL}};
const Segment SGI4820[] = {{&S38351[42], 20}, {NULL}};
const Segment SGI4821[] = {{&S38351[30], 4}, {NULL}};
const Segment SGI4822[] = {{&S38351[111], 3}, {NULL}};
const Segment SGI4823[] = {{&S38351[76], 6}, {NULL}};
const Segment SGI4824[] = {{&S38352[42], 9}, {NULL}};
const Segment SGI4825[] = {{&S38352[42], 20}, {NULL}};
const Segment SGI4826[] = {{&S38352[30], 4}, {NULL}};
const Segment SGI4827[] = {{&S38352[111], 3}, {NULL}};
const Segment SGI4828[] = {{&S38352[76], 6}, {NULL}};
const Segment SGI4829[] = {{&S38353[42], 9}, {NULL}};
const Segment SGI4830[] = {{&S38353[42], 28}, {NULL}};
const Segment SGI4831[] = {{&S38353[30], 4}, {NULL}};
const Segment SGI4832[] = {{&S38353[119], 3}, {NULL}};
const Segment SGI4833[] = {{&S38353[84], 6}, {NULL}};
const Segment SGI4834[] = {{&S38354[50], 14}, {NULL}};
const Segment SGI4835[] = {{&S38354[50], 25}, {NULL}};
const Segment SGI4836[] = {{&S38354[30], 12}, {NULL}};
const Segment SGI4837[] = {{&S38354[124], 3}, {NULL}};
const Segment SGI4838[] = {{&S38354[89], 6}, {NULL}};
const Segment SGI4839[] = {{&S38355[50], 18}, {NULL}};
const Segment SGI4840[] = {{&S38355[50], 29}, {NULL}};
const Segment SGI4841[] = {{&S38355[30], 12}, {NULL}};
const Segment SGI4842[] = {{&S38355[128], 3}, {NULL}};
const Segment SGI4843[] = {{&S38355[93], 6}, {NULL}};
const Segment SGI4844[] = {{&S38356[50], 14}, {NULL}};
const Segment SGI4845[] = {{&S38356[50], 25}, {NULL}};
const Segment SGI4846[] = {{&S38356[30], 12}, {NULL}};
const Segment SGI4847[] = {{&S38356[124], 3}, {NULL}};
const Segment SGI4848[] = {{&S38356[89], 6}, {NULL}};
const Segment SGI4849[] = {{&S38357[42], 8}, {NULL}};
const Segment SGI4850[] = {{&S38357[42], 19}, {NULL}};
const Segment SGI4851[] = {{&S38357[30], 4}, {NULL}};
const Segment SGI4852[] = {{&S38357[110], 3}, {NULL}};
const Segment SGI4853[] = {{&S38357[75], 6}, {NULL}};
const Segment SGI4854[] = {{&S38358[50], 23}, {NULL}};
const Segment SGI4855[] = {{&S38358[50], 34}, {NULL}};
const Segment SGI4856[] = {{&S38358[30], 12}, {NULL}};
const Segment SGI4857[] = {{&S38358[133], 3}, {NULL}};
const Segment SGI4858[] = {{&S38358[98], 6}, {NULL}};
const Segment SGI4859[] = {{&S38359[50], 23}, {NULL}};
const Segment SGI4860[] = {{&S38359[50], 34}, {NULL}};
const Segment SGI4861[] = {{&S38359[30], 12}, {NULL}};
const Segment SGI4862[] = {{&S38359[133], 3}, {NULL}};
const Segment SGI4863[] = {{&S38359[98], 6}, {NULL}};
const Segment SGI4864[] = {{&S38360[50], 12}, {NULL}};
const Segment SGI4865[] = {{&S38360[50], 23}, {NULL}};
const Segment SGI4866[] = {{&S38360[30], 12}, {NULL}};
const Segment SGI4867[] = {{&S38360[122], 3}, {NULL}};
const Segment SGI4868[] = {{&S38360[87], 6}, {NULL}};
const Segment SGI4869[] = {{&S38361[50], 23}, {NULL}};
const Segment SGI4870[] = {{&S38361[50], 34}, {NULL}};
const Segment SGI4871[] = {{&S38361[30], 12}, {NULL}};
const Segment SGI4872[] = {{&S38361[133], 3}, {NULL}};
const Segment SGI4873[] = {{&S38361[98], 6}, {NULL}};
const Segment SGI4874[] = {{&S38362[50], 11}, {NULL}};
const Segment SGI4875[] = {{&S38362[50], 22}, {NULL}};
const Segment SGI4876[] = {{&S38362[30], 12}, {NULL}};
const Segment SGI4877[] = {{&S38362[121], 3}, {NULL}};
const Segment SGI4878[] = {{&S38362[86], 6}, {NULL}};
const Segment SGI4879[] = {{&S38363[50], 23}, {NULL}};
const Segment SGI4880[] = {{&S38363[50], 34}, {NULL}};
const Segment SGI4881[] = {{&S38363[30], 12}, {NULL}};
const Segment SGI4882[] = {{&S38363[133], 3}, {NULL}};
const Segment SGI4883[] = {{&S38363[98], 6}, {NULL}};
const Segment SGI4884[] = {{&S38364[50], 12}, {NULL}};
const Segment SGI4885[] = {{&S38364[50], 23}, {NULL}};
const Segment SGI4886[] = {{&S38364[30], 12}, {NULL}};
const Segment SGI4887[] = {{&S38364[122], 3}, {NULL}};
const Segment SGI4888[] = {{&S38364[87], 6}, {NULL}};
const Segment SGI4889[] = {{&S38365[50], 10}, {NULL}};
const Segment SGI4890[] = {{&S38365[50], 21}, {NULL}};
const Segment SGI4891[] = {{&S38365[30], 12}, {NULL}};
const Segment SGI4892[] = {{&S38365[120], 3}, {NULL}};
const Segment SGI4893[] = {{&S38365[85], 6}, {NULL}};
const Segment SGI4894[] = {{&S38366[50], 30}, {NULL}};
const Segment SGI4895[] = {{&S38366[50], 41}, {NULL}};
const Segment SGI4896[] = {{&S38366[30], 12}, {NULL}};
const Segment SGI4897[] = {{&S38366[140], 3}, {NULL}};
const Segment SGI4898[] = {{&S38366[105], 6}, {NULL}};
const Segment SGI4899[] = {{&S38367[50], 31}, {NULL}};
const Segment SGI4900[] = {{&S38367[50], 42}, {NULL}};
const Segment SGI4901[] = {{&S38367[30], 12}, {NULL}};
const Segment SGI4902[] = {{&S38367[141], 3}, {NULL}};
const Segment SGI4903[] = {{&S38367[106], 6}, {NULL}};
const Segment SGI4904[] = {{&S38368[50], 30}, {NULL}};
const Segment SGI4905[] = {{&S38368[50], 41}, {NULL}};
const Segment SGI4906[] = {{&S38368[30], 12}, {NULL}};
const Segment SGI4907[] = {{&S38368[140], 3}, {NULL}};
const Segment SGI4908[] = {{&S38368[105], 6}, {NULL}};
const Segment SGI4909[] = {{&S38369[50], 31}, {NULL}};
const Segment SGI4910[] = {{&S38369[50], 42}, {NULL}};
const Segment SGI4911[] = {{&S38369[30], 12}, {NULL}};
const Segment SGI4912[] = {{&S38369[141], 3}, {NULL}};
const Segment SGI4913[] = {{&S38369[106], 6}, {NULL}};
const Segment SGI4914[] = {{&S38370[50], 23}, {NULL}};
const Segment SGI4915[] = {{&S38370[50], 34}, {NULL}};
const Segment SGI4916[] = {{&S38370[30], 12}, {NULL}};
const Segment SGI4917[] = {{&S38370[133], 3}, {NULL}};
const Segment SGI4918[] = {{&S38370[98], 6}, {NULL}};
const Segment SGI4919[] = {{&S38371[42], 11}, {NULL}};
const Segment SGI4920[] = {{&S38371[42], 22}, {NULL}};
const Segment SGI4921[] = {{&S38371[30], 4}, {NULL}};
const Segment SGI4922[] = {{&S38371[113], 3}, {NULL}};
const Segment SGI4923[] = {{&S38371[78], 6}, {NULL}};
const Segment SGI4924[] = {{&S38372[50], 13}, {NULL}};
const Segment SGI4925[] = {{&S38372[50], 24}, {NULL}};
const Segment SGI4926[] = {{&S38372[30], 12}, {NULL}};
const Segment SGI4927[] = {{&S38372[123], 3}, {NULL}};
const Segment SGI4928[] = {{&S38372[88], 6}, {NULL}};
const Segment SGI4929[] = {{&S38373[50], 22}, {NULL}};
const Segment SGI4930[] = {{&S38373[50], 33}, {NULL}};
const Segment SGI4931[] = {{&S38373[30], 12}, {NULL}};
const Segment SGI4932[] = {{&S38373[132], 3}, {NULL}};
const Segment SGI4933[] = {{&S38373[97], 6}, {NULL}};
const Segment SGI4934[] = {{&S38374[42], 8}, {NULL}};
const Segment SGI4935[] = {{&S38374[42], 19}, {NULL}};
const Segment SGI4936[] = {{&S38374[30], 4}, {NULL}};
const Segment SGI4937[] = {{&S38374[110], 3}, {NULL}};
const Segment SGI4938[] = {{&S38374[75], 6}, {NULL}};
const Segment SGI4939[] = {{&S38375[41], 9}, {NULL}};
const Segment SGI4940[] = {{&S38375[41], 20}, {NULL}};
const Segment SGI4941[] = {{&S38375[110], 3}, {NULL}};
const Segment SGI4942[] = {{&S38375[75], 6}, {NULL}};
const Segment SGI4943[] = {{&S38376[40], 9}, {NULL}};
const Segment SGI4944[] = {{&S38376[30], 4}, {NULL}};
const Segment SGI4945[] = {{&S38376[109], 3}, {NULL}};
const Segment SGI4946[] = {{&S38376[74], 6}, {NULL}};
const Segment SGI4947[] = {{&S38377[41], 9}, {NULL}};
const Segment SGI4948[] = {{&S38377[41], 20}, {NULL}};
const Segment SGI4949[] = {{&S38377[30], 3}, {NULL}};
const Segment SGI4950[] = {{&S38377[110], 3}, {NULL}};
const Segment SGI4951[] = {{&S38377[75], 6}, {NULL}};
const Segment SGI4952[] = {{&S38378[44], 12}, {NULL}};
const Segment SGI4953[] = {{&S38378[31], 12}, {NULL}};
const Segment SGI4954[] = {{&S38379[42], 12}, {NULL}};
const Segment SGI4955[] = {{&S38379[42], 23}, {NULL}};
const Segment SGI4956[] = {{&S38379[30], 4}, {NULL}};
const Segment SGI4957[] = {{&S38379[114], 3}, {NULL}};
const Segment SGI4958[] = {{&S38379[79], 6}, {NULL}};
const Segment SGI4959[] = {{&S38380[50], 23}, {NULL}};
const Segment SGI4960[] = {{&S38380[50], 34}, {NULL}};
const Segment SGI4961[] = {{&S38380[30], 12}, {NULL}};
const Segment SGI4962[] = {{&S38380[133], 3}, {NULL}};
const Segment SGI4963[] = {{&S38380[98], 6}, {NULL}};
const Segment SGI4964[] = {{&S38380[98], 4}, {NULL}};
const Segment SGI4965[] = {{&S38508[44], 15}, {NULL}};
const Segment SGI4966[] = {{&S38508[44], 26}, {NULL}};
const Segment SGI4967[] = {{&S38508[30], 6}, {NULL}};
const Segment SGI4968[] = {{&S38508[118], 3}, {NULL}};
const Segment SGI4969[] = {{&S38508[84], 5}, {NULL}};
const Segment SGI4970[] = {{&S38509[44], 18}, {NULL}};
const Segment SGI4971[] = {{&S38509[30], 6}, {NULL}};
const Segment SGI4972[] = {{&S38509[123], 3}, {NULL}};
const Segment SGI4973[] = {{&S38509[89], 5}, {NULL}};
const Segment SGI4974[] = {{&S38511[44], 14}, {NULL}};
const Segment SGI4975[] = {{&S38511[44], 25}, {NULL}};
const Segment SGI4976[] = {{&S38511[30], 6}, {NULL}};
const Segment SGI4977[] = {{&S38511[118], 3}, {NULL}};
const Segment SGI4978[] = {{&S38511[84], 5}, {NULL}};
const Segment SGI4979[] = {{&S38512[38], 9}, {NULL}};
const Segment SGI4980[] = {{&S38512[31], 6}, {NULL}};
const Segment SGI4981[] = {{&S38513[44], 14}, {NULL}};
const Segment SGI4982[] = {{&S38513[44], 31}, {NULL}};
const Segment SGI4983[] = {{&S38513[30], 6}, {NULL}};
const Segment SGI4984[] = {{&S38513[124], 3}, {NULL}};
const Segment SGI4985[] = {{&S38513[90], 5}, {NULL}};
const Segment SGI4986[] = {{&S38514[38], 11}, {NULL}};
const Segment SGI4987[] = {{&S38514[31], 6}, {NULL}};
const Segment SGI4988[] = {{&S38515[44], 13}, {NULL}};
const Segment SGI4989[] = {{&S38515[44], 30}, {NULL}};
const Segment SGI4990[] = {{&S38515[30], 6}, {NULL}};
const Segment SGI4991[] = {{&S38515[123], 3}, {NULL}};
const Segment SGI4992[] = {{&S38515[89], 5}, {NULL}};
const Segment SGI4993[] = {{&S38516[43], 14}, {NULL}};
const Segment SGI4994[] = {{&S38516[43], 31}, {NULL}};
const Segment SGI4995[] = {{&S38516[30], 5}, {NULL}};
const Segment SGI4996[] = {{&S38516[123], 3}, {NULL}};
const Segment SGI4997[] = {{&S38516[89], 5}, {NULL}};
const Segment SGI4998[] = {{&S38517[44], 9}, {NULL}};
const Segment SGI4999[] = {{&S38517[44], 21}, {NULL}};
const Segment SGI5000[] = {{&S38517[30], 6}, {NULL}};
const Segment SGI5001[] = {{&S38517[115], 3}, {NULL}};
const Segment SGI5002[] = {{&S38517[79], 7}, {NULL}};
const Segment SGI5003[] = {{&S38517[135], 8}, {NULL}};
const Segment SGI5004[] = {{&S38518[44], 14}, {NULL}};
const Segment SGI5005[] = {{&S38518[44], 31}, {NULL}};
const Segment SGI5006[] = {{&S38518[30], 6}, {NULL}};
const Segment SGI5007[] = {{&S38518[124], 3}, {NULL}};
const Segment SGI5008[] = {{&S38519[44], 14}, {NULL}};
const Segment SGI5009[] = {{&S38519[44], 25}, {NULL}};
const Segment SGI5010[] = {{&S38519[30], 6}, {NULL}};
const Segment SGI5011[] = {{&S38519[118], 3}, {NULL}};
const Segment SGI5012[] = {{&S38520[44], 9}, {NULL}};
const Segment SGI5013[] = {{&S38520[44], 21}, {NULL}};
const Segment SGI5014[] = {{&S38520[30], 6}, {NULL}};
const Segment SGI5015[] = {{&S38521[44], 14}, {NULL}};
const Segment SGI5016[] = {{&S38521[44], 30}, {NULL}};
const Segment SGI5017[] = {{&S38521[30], 6}, {NULL}};
const Segment SGI5018[] = {{&S38521[123], 3}, {NULL}};
const Segment SGI5019[] = {{&S38521[89], 5}, {NULL}};
const Segment SGI5020[] = {{&S38522[44], 14}, {NULL}};
const Segment SGI5021[] = {{&S38522[44], 31}, {NULL}};
const Segment SGI5022[] = {{&S38522[30], 6}, {NULL}};
const Segment SGI5023[] = {{&S38522[123], 3}, {NULL}};
const Segment SGI5024[] = {{&S38522[89], 5}, {NULL}};
const Segment SGI5025[] = {{&S38510[42], 11}, {NULL}};
const Segment SGI5026[] = {{&S38510[42], 22}, {NULL}};
const Segment SGI5027[] = {{&S38510[30], 4}, {NULL}};
const Segment SGI5028[] = {{&S38510[112], 3}, {NULL}};
const Segment SGI5029[] = {{&S38510[78], 5}, {NULL}};
const Segment SGI5030[] = {{&S38681[44], 6}, {NULL}};
const Segment SGI5031[] = {{&S38681[44], 18}, {NULL}};
const Segment SGI5032[] = {{&S38681[30], 6}, {NULL}};
const Segment SGI5033[] = {{&S38681[110], 3}, {NULL}};
const Segment SGI5034[] = {{&S38681[76], 5}, {NULL}};
const Segment SGI5035[] = {{&S38682[38], 6}, {NULL}};
const Segment SGI5036[] = {{&S38682[31], 6}, {NULL}};
const Segment SGI5037[] = {{&S38686[43], 6}, {NULL}};
const Segment SGI5038[] = {{&S38686[43], 18}, {NULL}};
const Segment SGI5039[] = {{&S38686[109], 3}, {NULL}};
const Segment SGI5040[] = {{&S38686[75], 5}, {NULL}};
const Segment SGI5041[] = {{&S38688[44], 5}, {NULL}};
const Segment SGI5042[] = {{&S38688[44], 17}, {NULL}};
const Segment SGI5043[] = {{&S38688[30], 6}, {NULL}};
const Segment SGI5044[] = {{&S38688[109], 3}, {NULL}};
const Segment SGI5045[] = {{&S38688[75], 5}, {NULL}};
const Segment SGI5046[] = {{&S38689[44], 6}, {NULL}};
const Segment SGI5047[] = {{&S38689[44], 18}, {NULL}};
const Segment SGI5048[] = {{&S38689[30], 6}, {NULL}};
const Segment SGI5049[] = {{&S38689[110], 3}, {NULL}};
const Segment SGI5050[] = {{&S38689[76], 5}, {NULL}};
const Segment SGI5051[] = {{&S38694[44], 5}, {NULL}};
const Segment SGI5052[] = {{&S38694[44], 17}, {NULL}};
const Segment SGI5053[] = {{&S38694[30], 6}, {NULL}};
const Segment SGI5054[] = {{&S38694[109], 3}, {NULL}};
const Segment SGI5055[] = {{&S38694[75], 5}, {NULL}};
const Segment SGI5056[] = {{&S38695[43], 6}, {NULL}};
const Segment SGI5057[] = {{&S38695[43], 18}, {NULL}};
const Segment SGI5058[] = {{&S38695[30], 5}, {NULL}};
const Segment SGI5059[] = {{&S38695[109], 3}, {NULL}};
const Segment SGI5060[] = {{&S38695[75], 5}, {NULL}};
const Segment SGI5061[] = {{&S40273[8], 3}, {NULL}};
const Segment SGI5062[] = {{&S40273[170], 6}, {NULL}};
const Segment SGI5063[] = {{&S38849[44], 14}, {NULL}};
const Segment SGI5064[] = {{&S38849[44], 42}, {NULL}};
const Segment SGI5065[] = {{&S38849[30], 6}, {NULL}};
const Segment SGI5066[] = {{&S38849[134], 3}, {NULL}};
const Segment SGI5067[] = {{&S38849[100], 5}, {NULL}};
const Segment SGI5068[] = {{&S38850[44], 14}, {NULL}};
const Segment SGI5069[] = {{&S38850[44], 38}, {NULL}};
const Segment SGI5070[] = {{&S38850[30], 6}, {NULL}};
const Segment SGI5071[] = {{&S38850[130], 3}, {NULL}};
const Segment SGI5072[] = {{&S38850[96], 5}, {NULL}};
const Segment SGI5073[] = {{&S38851[44], 14}, {NULL}};
const Segment SGI5074[] = {{&S38851[44], 38}, {NULL}};
const Segment SGI5075[] = {{&S38851[30], 6}, {NULL}};
const Segment SGI5076[] = {{&S38851[130], 3}, {NULL}};
const Segment SGI5077[] = {{&S38851[96], 5}, {NULL}};
const Segment SGI5078[] = {{&S38852[44], 14}, {NULL}};
const Segment SGI5079[] = {{&S38852[44], 38}, {NULL}};
const Segment SGI5080[] = {{&S38852[30], 6}, {NULL}};
const Segment SGI5081[] = {{&S38852[130], 3}, {NULL}};
const Segment SGI5082[] = {{&S38852[96], 5}, {NULL}};
const Segment SGI5083[] = {{&S38853[44], 14}, {NULL}};
const Segment SGI5084[] = {{&S38853[44], 38}, {NULL}};
const Segment SGI5085[] = {{&S38853[30], 6}, {NULL}};
const Segment SGI5086[] = {{&S38853[130], 3}, {NULL}};
const Segment SGI5087[] = {{&S38853[96], 5}, {NULL}};
const Segment SGI5088[] = {{&S38858[38], 14}, {NULL}};
const Segment SGI5089[] = {{&S38858[31], 6}, {NULL}};
const Segment SGI5090[] = {{&S38952[44], 7}, {NULL}};
const Segment SGI5091[] = {{&S38952[44], 18}, {NULL}};
const Segment SGI5092[] = {{&S38952[30], 6}, {NULL}};
const Segment SGI5093[] = {{&S38952[110], 3}, {NULL}};
const Segment SGI5094[] = {{&S38952[76], 5}, {NULL}};
const Segment SGI5095[] = {{&S38955[38], 7}, {NULL}};
const Segment SGI5096[] = {{&S38955[31], 6}, {NULL}};
const Segment SGI5097[] = {{&S38957[44], 7}, {NULL}};
const Segment SGI5098[] = {{&S38957[44], 18}, {NULL}};
const Segment SGI5099[] = {{&S38957[30], 6}, {NULL}};
const Segment SGI5100[] = {{&S38957[79], 4}, {NULL}};
const Segment SGI5101[] = {{&S38957[79], 3}, {NULL}};
const Segment SGI5102[] = {{&S41769[42], 6}, {NULL}};
const Segment SGI5103[] = {{&S41769[42], 22}, {NULL}};
const Segment SGI5104[] = {{&S41769[30], 4}, {NULL}};
const Segment SGI5105[] = {{&S41769[113], 3}, {NULL}};
const Segment SGI5106[] = {{&S41769[78], 6}, {NULL}};
const Segment SGI5107[] = {{&S41769[133], 8}, {NULL}};
const Segment SGI5108[] = {{&S41772[42], 6}, {NULL}};
const Segment SGI5109[] = {{&S41772[42], 22}, {NULL}};
const Segment SGI5110[] = {{&S41772[30], 4}, {NULL}};
const Segment SGI5111[] = {{&S41772[113], 3}, {NULL}};
const Segment SGI5112[] = {{&S41772[78], 6}, {NULL}};
const Segment SGI5113[] = {{&S41772[133], 8}, {NULL}};
const Segment SGI5114[] = {{&S41773[42], 6}, {NULL}};
const Segment SGI5115[] = {{&S41773[42], 22}, {NULL}};
const Segment SGI5116[] = {{&S41773[30], 4}, {NULL}};
const Segment SGI5117[] = {{&S41773[113], 3}, {NULL}};
const Segment SGI5118[] = {{&S41773[78], 6}, {NULL}};
const Segment SGI5119[] = {{&S41773[133], 8}, {NULL}};
const Segment SGI5120[] = {{&S41774[42], 6}, {NULL}};
const Segment SGI5121[] = {{&S41774[42], 23}, {NULL}};
const Segment SGI5122[] = {{&S41774[30], 4}, {NULL}};
const Segment SGI5123[] = {{&S41774[114], 3}, {NULL}};
const Segment SGI5124[] = {{&S41774[79], 6}, {NULL}};
const Segment SGI5125[] = {{&S41774[134], 8}, {NULL}};
const Segment SGI5126[] = {{&S41775[42], 6}, {NULL}};
const Segment SGI5127[] = {{&S41775[42], 21}, {NULL}};
const Segment SGI5128[] = {{&S41775[30], 4}, {NULL}};
const Segment SGI5129[] = {{&S41775[112], 3}, {NULL}};
const Segment SGI5130[] = {{&S41775[77], 6}, {NULL}};
const Segment SGI5131[] = {{&S41775[132], 8}, {NULL}};
const Segment SGI5132[] = {{&S41776[42], 6}, {NULL}};
const Segment SGI5133[] = {{&S41776[42], 22}, {NULL}};
const Segment SGI5134[] = {{&S41776[30], 4}, {NULL}};
const Segment SGI5135[] = {{&S41776[113], 3}, {NULL}};
const Segment SGI5136[] = {{&S41776[78], 6}, {NULL}};
const Segment SGI5137[] = {{&S41776[133], 8}, {NULL}};
const Segment SGI5138[] = {{&S41779[42], 6}, {NULL}};
const Segment SGI5139[] = {{&S41779[42], 22}, {NULL}};
const Segment SGI5140[] = {{&S41779[30], 4}, {NULL}};
const Segment SGI5141[] = {{&S41779[113], 3}, {NULL}};
const Segment SGI5142[] = {{&S41779[78], 6}, {NULL}};
const Segment SGI5143[] = {{&S41779[133], 8}, {NULL}};
const Segment SGI5144[] = {{&S38950[42], 7}, {NULL}};
const Segment SGI5145[] = {{&S38950[42], 18}, {NULL}};
const Segment SGI5146[] = {{&S38950[30], 4}, {NULL}};
const Segment SGI5147[] = {{&S38950[108], 3}, {NULL}};
const Segment SGI5148[] = {{&S38950[74], 5}, {NULL}};
const Segment SGI5149[] = {{&S38951[42], 7}, {NULL}};
const Segment SGI5150[] = {{&S38951[42], 21}, {NULL}};
const Segment SGI5151[] = {{&S38951[30], 4}, {NULL}};
const Segment SGI5152[] = {{&S38951[111], 3}, {NULL}};
const Segment SGI5153[] = {{&S38951[77], 5}, {NULL}};
const Segment SGI5154[] = {{&S38953[42], 7}, {NULL}};
const Segment SGI5155[] = {{&S38953[42], 21}, {NULL}};
const Segment SGI5156[] = {{&S38953[30], 4}, {NULL}};
const Segment SGI5157[] = {{&S38953[111], 3}, {NULL}};
const Segment SGI5158[] = {{&S38953[77], 5}, {NULL}};
const Segment SGI5159[] = {{&S38954[36], 7}, {NULL}};
const Segment SGI5160[] = {{&S38954[31], 4}, {NULL}};
const Segment SGI5161[] = {{&S39238[44], 11}, {NULL}};
const Segment SGI5162[] = {{&S39238[44], 22}, {NULL}};
const Segment SGI5163[] = {{&S39238[30], 6}, {NULL}};
const Segment SGI5164[] = {{&S39238[114], 3}, {NULL}};
const Segment SGI5165[] = {{&S39238[80], 5}, {NULL}};
const Segment SGI5166[] = {{&S39239[44], 10}, {NULL}};
const Segment SGI5167[] = {{&S39239[44], 21}, {NULL}};
const Segment SGI5168[] = {{&S39239[30], 6}, {NULL}};
const Segment SGI5169[] = {{&S39239[113], 3}, {NULL}};
const Segment SGI5170[] = {{&S39239[79], 5}, {NULL}};
const Segment SGI5171[] = {{&S39240[39], 8}, {NULL}};
const Segment SGI5172[] = {{&S39240[92], 3}, {NULL}};
const Segment SGI5173[] = {{&S39240[69], 6}, {NULL}};
const Segment SGI5174[] = {{&S39241[38], 10}, {NULL}};
const Segment SGI5175[] = {{&S39241[31], 6}, {NULL}};
const Segment SGI5176[] = {{&S39242[44], 10}, {NULL}};
const Segment SGI5177[] = {{&S39242[44], 21}, {NULL}};
const Segment SGI5178[] = {{&S39242[30], 6}, {NULL}};
const Segment SGI5179[] = {{&S39242[82], 4}, {NULL}};
const Segment SGI5180[] = {{&S39242[82], 3}, {NULL}};
const Segment SGI5181[] = {{&S39243[44], 10}, {NULL}};
const Segment SGI5182[] = {{&S39243[44], 21}, {NULL}};
const Segment SGI5183[] = {{&S39243[30], 6}, {NULL}};
const Segment SGI5184[] = {{&S39243[82], 6}, {NULL}};
const Segment SGI5185[] = {{&S39244[42], 10}, {NULL}};
const Segment SGI5186[] = {{&S39244[30], 6}, {NULL}};
const Segment SGI5187[] = {{&S39244[111], 3}, {NULL}};
const Segment SGI5188[] = {{&S39244[77], 5}, {NULL}};
const Segment SGI5189[] = {{&S39715[48], 28}, {NULL}};
const Segment SGI5190[] = {{&S39715[30], 12}, {NULL}};
const Segment SGI5191[] = {{&S39715[137], 3}, {NULL}};
const Segment SGI5192[] = {{&S39715[102], 6}, {NULL}};
const Segment SGI5193[] = {{&S39716[50], 28}, {NULL}};
const Segment SGI5194[] = {{&S39716[50], 40}, {NULL}};
const Segment SGI5195[] = {{&S39716[30], 12}, {NULL}};
const Segment SGI5196[] = {{&S39716[139], 3}, {NULL}};
const Segment SGI5197[] = {{&S39716[104], 6}, {NULL}};
const Segment SGI5198[] = {{&S39717[50], 28}, {NULL}};
const Segment SGI5199[] = {{&S39717[50], 40}, {NULL}};
const Segment SGI5200[] = {{&S39717[30], 12}, {NULL}};
const Segment SGI5201[] = {{&S39717[139], 3}, {NULL}};
const Segment SGI5202[] = {{&S39717[104], 6}, {NULL}};
const Segment SGI5203[] = {{&S39718[48], 28}, {NULL}};
const Segment SGI5204[] = {{&S39718[30], 12}, {NULL}};
const Segment SGI5205[] = {{&S39718[137], 3}, {NULL}};
const Segment SGI5206[] = {{&S39718[102], 6}, {NULL}};
const Segment SGI5207[] = {{&S39719[50], 28}, {NULL}};
const Segment SGI5208[] = {{&S39719[50], 40}, {NULL}};
const Segment SGI5209[] = {{&S39719[30], 12}, {NULL}};
const Segment SGI5210[] = {{&S39719[139], 3}, {NULL}};
const Segment SGI5211[] = {{&S39719[104], 6}, {NULL}};
const Segment SGI5212[] = {{&S39720[50], 18}, {NULL}};
const Segment SGI5213[] = {{&S39720[50], 30}, {NULL}};
const Segment SGI5214[] = {{&S39720[30], 12}, {NULL}};
const Segment SGI5215[] = {{&S39720[129], 3}, {NULL}};
const Segment SGI5216[] = {{&S39720[94], 6}, {NULL}};
const Segment SGI5217[] = {{&S39721[50], 28}, {NULL}};
const Segment SGI5218[] = {{&S39721[50], 40}, {NULL}};
const Segment SGI5219[] = {{&S39721[30], 12}, {NULL}};
const Segment SGI5220[] = {{&S39721[139], 3}, {NULL}};
const Segment SGI5221[] = {{&S39721[104], 6}, {NULL}};
const Segment SGI5222[] = {{&S39722[42], 28}, {NULL}};
const Segment SGI5223[] = {{&S39722[42], 40}, {NULL}};
const Segment SGI5224[] = {{&S39722[30], 4}, {NULL}};
const Segment SGI5225[] = {{&S39722[131], 3}, {NULL}};
const Segment SGI5226[] = {{&S39722[96], 6}, {NULL}};
const Segment SGI5227[] = {{&S39723[50], 28}, {NULL}};
const Segment SGI5228[] = {{&S39723[50], 40}, {NULL}};
const Segment SGI5229[] = {{&S39723[30], 12}, {NULL}};
const Segment SGI5230[] = {{&S39723[139], 3}, {NULL}};
const Segment SGI5231[] = {{&S39723[104], 6}, {NULL}};
const Segment SGI5232[] = {{&S39724[50], 28}, {NULL}};
const Segment SGI5233[] = {{&S39724[50], 40}, {NULL}};
const Segment SGI5234[] = {{&S39724[30], 12}, {NULL}};
const Segment SGI5235[] = {{&S39724[139], 3}, {NULL}};
const Segment SGI5236[] = {{&S39724[104], 6}, {NULL}};
const Segment SGI5237[] = {{&S39725[50], 28}, {NULL}};
const Segment SGI5238[] = {{&S39725[50], 40}, {NULL}};
const Segment SGI5239[] = {{&S39725[30], 12}, {NULL}};
const Segment SGI5240[] = {{&S39725[139], 3}, {NULL}};
const Segment SGI5241[] = {{&S39725[104], 6}, {NULL}};
const Segment SGI5242[] = {{&S39726[49], 28}, {NULL}};
const Segment SGI5243[] = {{&S39726[49], 40}, {NULL}};
const Segment SGI5244[] = {{&S39726[138], 3}, {NULL}};
const Segment SGI5245[] = {{&S39726[103], 6}, {NULL}};
const Segment SGI5246[] = {{&S39727[50], 28}, {NULL}};
const Segment SGI5247[] = {{&S39727[50], 40}, {NULL}};
const Segment SGI5248[] = {{&S39727[30], 12}, {NULL}};
const Segment SGI5249[] = {{&S39727[139], 3}, {NULL}};
const Segment SGI5250[] = {{&S39727[104], 6}, {NULL}};
const Segment SGI5251[] = {{&S39728[49], 28}, {NULL}};
const Segment SGI5252[] = {{&S39728[49], 40}, {NULL}};
const Segment SGI5253[] = {{&S39728[30], 11}, {NULL}};
const Segment SGI5254[] = {{&S39728[138], 3}, {NULL}};
const Segment SGI5255[] = {{&S39728[103], 6}, {NULL}};
const Segment SGI5256[] = {{&S39729[50], 18}, {NULL}};
const Segment SGI5257[] = {{&S39729[50], 30}, {NULL}};
const Segment SGI5258[] = {{&S39729[30], 12}, {NULL}};
const Segment SGI5259[] = {{&S39730[50], 18}, {NULL}};
const Segment SGI5260[] = {{&S39730[50], 30}, {NULL}};
const Segment SGI5261[] = {{&S39730[30], 12}, {NULL}};
const Segment SGI5262[] = {{&S39730[130], 3}, {NULL}};
const Segment SGI5263[] = {{&S39730[94], 7}, {NULL}};
const Segment SGI5264[] = {{&S39730[150], 8}, {NULL}};
const Segment SGI5265[] = {{&S39731[44], 18}, {NULL}};
const Segment SGI5266[] = {{&S39731[31], 12}, {NULL}};
const Segment SGI5267[] = {{&S42360[42], 9}, {NULL}};
const Segment SGI5268[] = {{&S42360[42], 20}, {NULL}};
const Segment SGI5269[] = {{&S42360[30], 4}, {NULL}};
const Segment SGI5270[] = {{&S42360[111], 3}, {NULL}};
const Segment SGI5271[] = {{&S42360[76], 6}, {NULL}};
const Segment SGI5272[] = {{&S42360[131], 8}, {NULL}};
const Segment SGI5273[] = {{&S42361[42], 13}, {NULL}};
const Segment SGI5274[] = {{&S42361[42], 24}, {NULL}};
const Segment SGI5275[] = {{&S42361[30], 4}, {NULL}};
const Segment SGI5276[] = {{&S42361[115], 3}, {NULL}};
const Segment SGI5277[] = {{&S42361[80], 6}, {NULL}};
const Segment SGI5278[] = {{&S42361[135], 8}, {NULL}};
const Segment SGI5279[] = {{&S42362[42], 7}, {NULL}};
const Segment SGI5280[] = {{&S42362[42], 18}, {NULL}};
const Segment SGI5281[] = {{&S42362[30], 4}, {NULL}};
const Segment SGI5282[] = {{&S42362[109], 3}, {NULL}};
const Segment SGI5283[] = {{&S42362[74], 6}, {NULL}};
const Segment SGI5284[] = {{&S42362[129], 8}, {NULL}};
const Segment SGI5285[] = {{&S40233[50], 10}, {NULL}};
const Segment SGI5286[] = {{&S40233[50], 21}, {NULL}};
const Segment SGI5287[] = {{&S40233[30], 12}, {NULL}};
const Segment SGI5288[] = {{&S41270[41], 8}, {NULL}};
const Segment SGI5289[] = {{&S40233[120], 3}, {NULL}};
const Segment SGI5290[] = {{&S40233[85], 6}, {NULL}};
const Segment SGI5291[] = {{&S40234[50], 11}, {NULL}};
const Segment SGI5292[] = {{&S40234[50], 22}, {NULL}};
const Segment SGI5293[] = {{&S40234[30], 12}, {NULL}};
const Segment SGI5294[] = {{&S40234[121], 3}, {NULL}};
const Segment SGI5295[] = {{&S40234[86], 6}, {NULL}};
const Segment SGI5296[] = {{&S40235[50], 11}, {NULL}};
const Segment SGI5297[] = {{&S40235[50], 22}, {NULL}};
const Segment SGI5298[] = {{&S40235[30], 12}, {NULL}};
const Segment SGI5299[] = {{&S40235[121], 3}, {NULL}};
const Segment SGI5300[] = {{&S40235[86], 6}, {NULL}};
const Segment SGI5301[] = {{&S40236[50], 11}, {NULL}};
const Segment SGI5302[] = {{&S40236[50], 23}, {NULL}};
const Segment SGI5303[] = {{&S40236[30], 12}, {NULL}};
const Segment SGI5304[] = {{&S40236[122], 3}, {NULL}};
const Segment SGI5305[] = {{&S40236[87], 6}, {NULL}};
const Segment SGI5306[] = {{&S40237[48], 10}, {NULL}};
const Segment SGI5307[] = {{&S40237[30], 12}, {NULL}};
const Segment SGI5308[] = {{&S40237[118], 3}, {NULL}};
const Segment SGI5309[] = {{&S40237[83], 6}, {NULL}};
const Segment SGI5310[] = {{&S40238[44], 10}, {NULL}};
const Segment SGI5311[] = {{&S40238[31], 12}, {NULL}};
const Segment SGI5312[] = {{&S40239[44], 11}, {NULL}};
const Segment SGI5313[] = {{&S40239[31], 12}, {NULL}};
const Segment SGI5314[] = {{&S40240[44], 11}, {NULL}};
const Segment SGI5315[] = {{&S40240[31], 12}, {NULL}};
const Segment SGI5316[] = {{&S40241[44], 11}, {NULL}};
const Segment SGI5317[] = {{&S40241[31], 12}, {NULL}};
const Segment SGI5318[] = {{&S42551[42], 6}, {NULL}};
const Segment SGI5319[] = {{&S42551[42], 22}, {NULL}};
const Segment SGI5320[] = {{&S42551[30], 4}, {NULL}};
const Segment SGI5321[] = {{&S42551[113], 3}, {NULL}};
const Segment SGI5322[] = {{&S42551[78], 6}, {NULL}};
const Segment SGI5323[] = {{&S42551[133], 8}, {NULL}};
const Segment SGI5324[] = {{&S42552[42], 6}, {NULL}};
const Segment SGI5325[] = {{&S42552[42], 22}, {NULL}};
const Segment SGI5326[] = {{&S42552[30], 4}, {NULL}};
const Segment SGI5327[] = {{&S42552[113], 3}, {NULL}};
const Segment SGI5328[] = {{&S42552[78], 6}, {NULL}};
const Segment SGI5329[] = {{&S42552[133], 8}, {NULL}};
const Segment SGI5330[] = {{&S42555[42], 6}, {NULL}};
const Segment SGI5331[] = {{&S42555[42], 22}, {NULL}};
const Segment SGI5332[] = {{&S42555[30], 4}, {NULL}};
const Segment SGI5333[] = {{&S42555[113], 3}, {NULL}};
const Segment SGI5334[] = {{&S42555[78], 6}, {NULL}};
const Segment SGI5335[] = {{&S42555[133], 8}, {NULL}};
const Segment SGI5336[] = {{&S42558[42], 6}, {NULL}};
const Segment SGI5337[] = {{&S42558[42], 22}, {NULL}};
const Segment SGI5338[] = {{&S42558[30], 4}, {NULL}};
const Segment SGI5339[] = {{&S42558[113], 3}, {NULL}};
const Segment SGI5340[] = {{&S42558[78], 6}, {NULL}};
const Segment SGI5341[] = {{&S42558[133], 8}, {NULL}};
const Segment SGI5342[] = {{&S42559[42], 6}, {NULL}};
const Segment SGI5343[] = {{&S42559[42], 22}, {NULL}};
const Segment SGI5344[] = {{&S42559[30], 4}, {NULL}};
const Segment SGI5345[] = {{&S42559[113], 3}, {NULL}};
const Segment SGI5346[] = {{&S42559[78], 6}, {NULL}};
const Segment SGI5347[] = {{&S42559[133], 8}, {NULL}};
const Segment SGI5348[] = {{&S39971[44], 11}, {NULL}};
const Segment SGI5349[] = {{&S39971[44], 22}, {NULL}};
const Segment SGI5350[] = {{&S39971[30], 6}, {NULL}};
const Segment SGI5351[] = {{&S39971[114], 3}, {NULL}};
const Segment SGI5352[] = {{&S39971[80], 5}, {NULL}};
const Segment SGI5353[] = {{&S39974[44], 10}, {NULL}};
const Segment SGI5354[] = {{&S39974[44], 22}, {NULL}};
const Segment SGI5355[] = {{&S39974[30], 6}, {NULL}};
const Segment SGI5356[] = {{&S39974[114], 3}, {NULL}};
const Segment SGI5357[] = {{&S39974[80], 5}, {NULL}};
const Segment SGI5358[] = {{&S39979[44], 11}, {NULL}};
const Segment SGI5359[] = {{&S39979[44], 23}, {NULL}};
const Segment SGI5360[] = {{&S39979[30], 6}, {NULL}};
const Segment SGI5361[] = {{&S39979[115], 3}, {NULL}};
const Segment SGI5362[] = {{&S39979[81], 5}, {NULL}};
const Segment SGI5363[] = {{&S39980[38], 11}, {NULL}};
const Segment SGI5364[] = {{&S39980[31], 6}, {NULL}};
const Segment SGI5365[] = {{&S39981[44], 11}, {NULL}};
const Segment SGI5366[] = {{&S39981[44], 21}, {NULL}};
const Segment SGI5367[] = {{&S39981[30], 6}, {NULL}};
const Segment SGI5368[] = {{&S39981[113], 3}, {NULL}};
const Segment SGI5369[] = {{&S39981[79], 5}, {NULL}};
const Segment SGI5370[] = {{&S39984[44], 11}, {NULL}};
const Segment SGI5371[] = {{&S39984[44], 40}, {NULL}};
const Segment SGI5372[] = {{&S39984[30], 6}, {NULL}};
const Segment SGI5373[] = {{&S39984[132], 3}, {NULL}};
const Segment SGI5374[] = {{&S39984[98], 5}, {NULL}};
const Segment SGI5375[] = {{&S39987[41], 11}, {NULL}};
const Segment SGI5376[] = {{&S39987[30], 6}, {NULL}};
const Segment SGI5377[] = {{&S39987[111], 3}, {NULL}};
const Segment SGI5378[] = {{&S39987[77], 5}, {NULL}};
const Segment SGI5379[] = {{&S39991[40], 21}, {NULL}};
const Segment SGI5380[] = {{&S39991[107], 3}, {NULL}};
const Segment SGI5381[] = {{&S39991[75], 5}, {NULL}};
const Segment SGI5382[] = {{&S39991[125], 5}, {NULL}};
const Segment SGI5383[] = {{&S39993[44], 11}, {NULL}};
const Segment SGI5384[] = {{&S39993[44], 22}, {NULL}};
const Segment SGI5385[] = {{&S39993[30], 6}, {NULL}};
const Segment SGI5386[] = {{&S39993[113], 3}, {NULL}};
const Segment SGI5387[] = {{&S39993[80], 5}, {NULL}};
const Segment SGI5388[] = {{&S39973[42], 11}, {NULL}};
const Segment SGI5389[] = {{&S39973[42], 22}, {NULL}};
const Segment SGI5390[] = {{&S39973[30], 4}, {NULL}};
const Segment SGI5391[] = {{&S39973[112], 3}, {NULL}};
const Segment SGI5392[] = {{&S39973[78], 5}, {NULL}};
const Segment SGI5393[] = {{&S39975[42], 10}, {NULL}};
const Segment SGI5394[] = {{&S39975[42], 21}, {NULL}};
const Segment SGI5395[] = {{&S39975[30], 4}, {NULL}};
const Segment SGI5396[] = {{&S39975[111], 3}, {NULL}};
const Segment SGI5397[] = {{&S39975[77], 5}, {NULL}};
const Segment SGI5398[] = {{&S39976[42], 11}, {NULL}};
const Segment SGI5399[] = {{&S39976[42], 22}, {NULL}};
const Segment SGI5400[] = {{&S39976[30], 4}, {NULL}};
const Segment SGI5401[] = {{&S39976[112], 3}, {NULL}};
const Segment SGI5402[] = {{&S39976[78], 5}, {NULL}};
const Segment SGI5403[] = {{&S39977[42], 11}, {NULL}};
const Segment SGI5404[] = {{&S39977[42], 22}, {NULL}};
const Segment SGI5405[] = {{&S39977[30], 4}, {NULL}};
const Segment SGI5406[] = {{&S39977[113], 3}, {NULL}};
const Segment SGI5407[] = {{&S39977[78], 6}, {NULL}};
const Segment SGI5408[] = {{&S39978[42], 11}, {NULL}};
const Segment SGI5409[] = {{&S39978[42], 22}, {NULL}};
const Segment SGI5410[] = {{&S39978[30], 4}, {NULL}};
const Segment SGI5411[] = {{&S39978[112], 3}, {NULL}};
const Segment SGI5412[] = {{&S39978[78], 5}, {NULL}};
const Segment SGI5413[] = {{&S39982[42], 11}, {NULL}};
const Segment SGI5414[] = {{&S39982[42], 23}, {NULL}};
const Segment SGI5415[] = {{&S39982[30], 4}, {NULL}};
const Segment SGI5416[] = {{&S39982[113], 3}, {NULL}};
const Segment SGI5417[] = {{&S39982[79], 5}, {NULL}};
const Segment SGI5418[] = {{&S39983[42], 11}, {NULL}};
const Segment SGI5419[] = {{&S39983[42], 22}, {NULL}};
const Segment SGI5420[] = {{&S39983[30], 4}, {NULL}};
const Segment SGI5421[] = {{&S39983[112], 3}, {NULL}};
const Segment SGI5422[] = {{&S39983[78], 5}, {NULL}};
const Segment SGI5423[] = {{&S39986[60], 11}, {NULL}};
const Segment SGI5424[] = {{&S39986[60], 22}, {NULL}};
const Segment SGI5425[] = {{&S39986[48], 4}, {NULL}};
const Segment SGI5426[] = {{&S39986[131], 3}, {NULL}};
const Segment SGI5427[] = {{&S39986[96], 6}, {NULL}};
const Segment SGI5428[] = {{&S39986[151], 8}, {NULL}};
const Segment SGI5429[] = {{&S39988[36], 11}, {NULL}};
const Segment SGI5430[] = {{&S39988[31], 4}, {NULL}};
const Segment SGI5431[] = {{&S39989[42], 11}, {NULL}};
const Segment SGI5432[] = {{&S39989[42], 22}, {NULL}};
const Segment SGI5433[] = {{&S39989[30], 4}, {NULL}};
const Segment SGI5434[] = {{&S39989[112], 3}, {NULL}};
const Segment SGI5435[] = {{&S39989[78], 5}, {NULL}};
const Segment SGI5436[] = {{&S39990[42], 11}, {NULL}};
const Segment SGI5437[] = {{&S39990[42], 22}, {NULL}};
const Segment SGI5438[] = {{&S39990[30], 4}, {NULL}};
const Segment SGI5439[] = {{&S39990[112], 3}, {NULL}};
const Segment SGI5440[] = {{&S39990[78], 5}, {NULL}};
const Segment SGI5441[] = {{&S39992[42], 11}, {NULL}};
const Segment SGI5442[] = {{&S39992[42], 22}, {NULL}};
const Segment SGI5443[] = {{&S39992[30], 4}, {NULL}};
const Segment SGI5444[] = {{&S39992[112], 3}, {NULL}};
const Segment SGI5445[] = {{&S39992[78], 5}, {NULL}};
const Segment SGI5446[] = {{&S39992[130], 6}, {NULL}};
const Segment SGI5447[] = {{&S39994[42], 11}, {NULL}};
const Segment SGI5448[] = {{&S39994[42], 23}, {NULL}};
const Segment SGI5449[] = {{&S39994[30], 4}, {NULL}};
const Segment SGI5450[] = {{&S39994[112], 3}, {NULL}};
const Segment SGI5451[] = {{&S39994[79], 5}, {NULL}};
const Segment SGI5452[] = {{&S41547[70], 5}, {NULL}};
const Segment SGI5453[] = {{&S40112[44], 10}, {NULL}};
const Segment SGI5454[] = {{&S40112[44], 21}, {NULL}};
const Segment SGI5455[] = {{&S40112[30], 6}, {NULL}};
const Segment SGI5456[] = {{&S40112[113], 3}, {NULL}};
const Segment SGI5457[] = {{&S40112[79], 5}, {NULL}};
const Segment SGI5458[] = {{&S40113[44], 9}, {NULL}};
const Segment SGI5459[] = {{&S40113[44], 20}, {NULL}};
const Segment SGI5460[] = {{&S40113[30], 6}, {NULL}};
const Segment SGI5461[] = {{&S40113[112], 3}, {NULL}};
const Segment SGI5462[] = {{&S41561[84], 6}, {NULL}};
const Segment SGI5463[] = {{&S40113[78], 5}, {NULL}};
const Segment SGI5464[] = {{&S40114[44], 10}, {NULL}};
const Segment SGI5465[] = {{&S40114[44], 21}, {NULL}};
const Segment SGI5466[] = {{&S40114[30], 6}, {NULL}};
const Segment SGI5467[] = {{&S40114[113], 3}, {NULL}};
const Segment SGI5468[] = {{&S40114[79], 5}, {NULL}};
const Segment SGI5469[] = {{&S40115[43], 10}, {NULL}};
const Segment SGI5470[] = {{&S40115[43], 21}, {NULL}};
const Segment SGI5471[] = {{&S40115[112], 3}, {NULL}};
const Segment SGI5472[] = {{&S40115[78], 5}, {NULL}};
const Segment SGI5473[] = {{&S40116[43], 10}, {NULL}};
const Segment SGI5474[] = {{&S40116[43], 21}, {NULL}};
const Segment SGI5475[] = {{&S40116[30], 5}, {NULL}};
const Segment SGI5476[] = {{&S40116[112], 3}, {NULL}};
const Segment SGI5477[] = {{&S40116[78], 5}, {NULL}};
const Segment SGI5478[] = {{&S40117[44], 10}, {NULL}};
const Segment SGI5479[] = {{&S40117[44], 21}, {NULL}};
const Segment SGI5480[] = {{&S40117[30], 6}, {NULL}};
const Segment SGI5481[] = {{&S40117[115], 3}, {NULL}};
const Segment SGI5482[] = {{&S40117[79], 7}, {NULL}};
const Segment SGI5483[] = {{&S40117[135], 8}, {NULL}};
const Segment SGI5484[] = {{&S40118[38], 10}, {NULL}};
const Segment SGI5485[] = {{&S40118[31], 6}, {NULL}};
const Segment SGI5486[] = {{&S41569[5], 8}, {NULL}};
const Segment SGI5487[] = {{&S40154[44], 11}, {NULL}};
const Segment SGI5488[] = {{&S40154[44], 22}, {NULL}};
const Segment SGI5489[] = {{&S40154[30], 6}, {NULL}};
const Segment SGI5490[] = {{&S40154[114], 3}, {NULL}};
const Segment SGI5491[] = {{&S40154[80], 5}, {NULL}};
const Segment SGI5492[] = {{&S40155[39], 8}, {NULL}};
const Segment SGI5493[] = {{&S40155[92], 3}, {NULL}};
const Segment SGI5494[] = {{&S40155[69], 6}, {NULL}};
const Segment SGI5495[] = {{&S40156[44], 11}, {NULL}};
const Segment SGI5496[] = {{&S40156[44], 22}, {NULL}};
const Segment SGI5497[] = {{&S40156[30], 6}, {NULL}};
const Segment SGI5498[] = {{&S40156[114], 3}, {NULL}};
const Segment SGI5499[] = {{&S40156[80], 5}, {NULL}};
const Segment SGI5500[] = {{&S40157[44], 10}, {NULL}};
const Segment SGI5501[] = {{&S40157[44], 21}, {NULL}};
const Segment SGI5502[] = {{&S40157[30], 6}, {NULL}};
const Segment SGI5503[] = {{&S40157[113], 3}, {NULL}};
const Segment SGI5504[] = {{&S40157[79], 5}, {NULL}};
const Segment SGI5505[] = {{&S40158[44], 10}, {NULL}};
const Segment SGI5506[] = {{&S40158[44], 21}, {NULL}};
const Segment SGI5507[] = {{&S40158[30], 6}, {NULL}};
const Segment SGI5508[] = {{&S40158[115], 3}, {NULL}};
const Segment SGI5509[] = {{&S40158[79], 7}, {NULL}};
const Segment SGI5510[] = {{&S40158[135], 8}, {NULL}};
const Segment SGI5511[] = {{&S40159[38], 10}, {NULL}};
const Segment SGI5512[] = {{&S40159[31], 6}, {NULL}};
const Segment SGI5513[] = {{&S40160[44], 10}, {NULL}};
const Segment SGI5514[] = {{&S40160[44], 21}, {NULL}};
const Segment SGI5515[] = {{&S40160[30], 6}, {NULL}};
const Segment SGI5516[] = {{&S40160[82], 4}, {NULL}};
const Segment SGI5517[] = {{&S40160[82], 3}, {NULL}};
const Segment SGI5518[] = {{&S40161[38], 11}, {NULL}};
const Segment SGI5519[] = {{&S40161[31], 6}, {NULL}};
const Segment SGI5520[] = {{&S40162[42], 10}, {NULL}};
const Segment SGI5521[] = {{&S40162[30], 6}, {NULL}};
const Segment SGI5522[] = {{&S40162[111], 3}, {NULL}};
const Segment SGI5523[] = {{&S40162[77], 5}, {NULL}};
const Segment SGI5524[] = {{&S40163[44], 10}, {NULL}};
const Segment SGI5525[] = {{&S40163[44], 21}, {NULL}};
const Segment SGI5526[] = {{&S40163[30], 6}, {NULL}};
const Segment SGI5527[] = {{&S40163[82], 6}, {NULL}};
const Segment SGI5528[] = {{&S40184[44], 13}, {NULL}};
const Segment SGI5529[] = {{&S40184[44], 24}, {NULL}};
const Segment SGI5530[] = {{&S40184[30], 6}, {NULL}};
const Segment SGI5531[] = {{&S40184[116], 3}, {NULL}};
const Segment SGI5532[] = {{&S40184[82], 5}, {NULL}};
const Segment SGI5533[] = {{&S40189[44], 7}, {NULL}};
const Segment SGI5534[] = {{&S40189[44], 18}, {NULL}};
const Segment SGI5535[] = {{&S40189[30], 6}, {NULL}};
const Segment SGI5536[] = {{&S40189[110], 3}, {NULL}};
const Segment SGI5537[] = {{&S40189[76], 5}, {NULL}};
const Segment SGI5538[] = {{&S40190[38], 13}, {NULL}};
const Segment SGI5539[] = {{&S40190[31], 6}, {NULL}};
const Segment SGI5540[] = {{&S40195[42], 10}, {NULL}};
const Segment SGI5541[] = {{&S40195[30], 6}, {NULL}};
const Segment SGI5542[] = {{&S40195[111], 3}, {NULL}};
const Segment SGI5543[] = {{&S40195[77], 5}, {NULL}};
const Segment SGI5544[] = {{&S40198[44], 10}, {NULL}};
const Segment SGI5545[] = {{&S40198[44], 21}, {NULL}};
const Segment SGI5546[] = {{&S40198[30], 6}, {NULL}};
const Segment SGI5547[] = {{&S40198[113], 3}, {NULL}};
const Segment SGI5548[] = {{&S40198[79], 5}, {NULL}};
const Segment SGI5549[] = {{&S42825[42], 6}, {NULL}};
const Segment SGI5550[] = {{&S42825[42], 21}, {NULL}};
const Segment SGI5551[] = {{&S42825[30], 4}, {NULL}};
const Segment SGI5552[] = {{&S42825[112], 3}, {NULL}};
const Segment SGI5553[] = {{&S42825[77], 6}, {NULL}};
const Segment SGI5554[] = {{&S42825[132], 8}, {NULL}};
const Segment SGI5555[] = {{&S42826[42], 6}, {NULL}};
const Segment SGI5556[] = {{&S42826[42], 22}, {NULL}};
const Segment SGI5557[] = {{&S42826[30], 4}, {NULL}};
const Segment SGI5558[] = {{&S42826[113], 3}, {NULL}};
const Segment SGI5559[] = {{&S42826[78], 6}, {NULL}};
const Segment SGI5560[] = {{&S40201[38], 10}, {NULL}};
const Segment SGI5561[] = {{&S42826[133], 8}, {NULL}};
const Segment SGI5562[] = {{&S40201[31], 6}, {NULL}};
const Segment SGI5563[] = {{&S42828[42], 6}, {NULL}};
const Segment SGI5564[] = {{&S42828[42], 23}, {NULL}};
const Segment SGI5565[] = {{&S42828[30], 4}, {NULL}};
const Segment SGI5566[] = {{&S42828[114], 3}, {NULL}};
const Segment SGI5567[] = {{&S42828[79], 6}, {NULL}};
const Segment SGI5568[] = {{&S42828[134], 8}, {NULL}};
const Segment SGI5569[] = {{&S42829[42], 6}, {NULL}};
const Segment SGI5570[] = {{&S42829[42], 22}, {NULL}};
const Segment SGI5571[] = {{&S42829[30], 4}, {NULL}};
const Segment SGI5572[] = {{&S42829[113], 3}, {NULL}};
const Segment SGI5573[] = {{&S42829[78], 6}, {NULL}};
const Segment SGI5574[] = {{&S42829[133], 8}, {NULL}};
const Segment SGI5575[] = {{&S40582[50], 6}, {NULL}};
const Segment SGI5576[] = {{&S40582[50], 23}, {NULL}};
const Segment SGI5577[] = {{&S40582[30], 12}, {NULL}};
const Segment SGI5578[] = {{&S40582[122], 3}, {NULL}};
const Segment SGI5579[] = {{&S40582[87], 6}, {NULL}};
const Segment SGI5580[] = {{&S40583[50], 6}, {NULL}};
const Segment SGI5581[] = {{&S40583[50], 23}, {NULL}};
const Segment SGI5582[] = {{&S40583[30], 12}, {NULL}};
const Segment SGI5583[] = {{&S40583[122], 3}, {NULL}};
const Segment SGI5584[] = {{&S40583[87], 6}, {NULL}};
const Segment SGI5585[] = {{&S40585[50], 6}, {NULL}};
const Segment SGI5586[] = {{&S40585[50], 24}, {NULL}};
const Segment SGI5587[] = {{&S40585[30], 12}, {NULL}};
const Segment SGI5588[] = {{&S40585[123], 3}, {NULL}};
const Segment SGI5589[] = {{&S40585[88], 6}, {NULL}};
const Segment SGI5590[] = {{&S40586[46], 5}, {NULL}};
const Segment SGI5591[] = {{&S40586[26], 12}, {NULL}};
const Segment SGI5592[] = {{&S40586[88], 3}, {NULL}};
const Segment SGI5593[] = {{&S40586[65], 6}, {NULL}};
const Segment SGI5594[] = {{&S40587[50], 6}, {NULL}};
const Segment SGI5595[] = {{&S40587[50], 24}, {NULL}};
const Segment SGI5596[] = {{&S40587[30], 12}, {NULL}};
const Segment SGI5597[] = {{&S40587[123], 3}, {NULL}};
const Segment SGI5598[] = {{&S40587[88], 6}, {NULL}};
const Segment SGI5599[] = {{&S40588[50], 6}, {NULL}};
const Segment SGI5600[] = {{&S40588[50], 23}, {NULL}};
const Segment SGI5601[] = {{&S40588[30], 12}, {NULL}};
const Segment SGI5602[] = {{&S40588[122], 3}, {NULL}};
const Segment SGI5603[] = {{&S40588[87], 6}, {NULL}};
const Segment SGI5604[] = {{&S40589[50], 6}, {NULL}};
const Segment SGI5605[] = {{&S40589[50], 23}, {NULL}};
const Segment SGI5606[] = {{&S40589[30], 12}, {NULL}};
const Segment SGI5607[] = {{&S40589[122], 3}, {NULL}};
const Segment SGI5608[] = {{&S40589[87], 6}, {NULL}};
const Segment SGI5609[] = {{&S40590[44], 6}, {NULL}};
const Segment SGI5610[] = {{&S40590[31], 12}, {NULL}};
const Segment SGI5611[] = {{&S40180[42], 7}, {NULL}};
const Segment SGI5612[] = {{&S40180[42], 18}, {NULL}};
const Segment SGI5613[] = {{&S40180[30], 4}, {NULL}};
const Segment SGI5614[] = {{&S40180[108], 3}, {NULL}};
const Segment SGI5615[] = {{&S40180[74], 5}, {NULL}};
const Segment SGI5616[] = {{&S40181[42], 13}, {NULL}};
const Segment SGI5617[] = {{&S40181[42], 24}, {NULL}};
const Segment SGI5618[] = {{&S40181[30], 4}, {NULL}};
const Segment SGI5619[] = {{&S40181[114], 3}, {NULL}};
const Segment SGI5620[] = {{&S40181[80], 5}, {NULL}};
const Segment SGI5621[] = {{&S40182[36], 13}, {NULL}};
const Segment SGI5622[] = {{&S40182[31], 4}, {NULL}};
const Segment SGI5623[] = {{&S40183[40], 13}, {NULL}};
const Segment SGI5624[] = {{&S40183[30], 4}, {NULL}};
const Segment SGI5625[] = {{&S40183[112], 3}, {NULL}};
const Segment SGI5626[] = {{&S40183[78], 5}, {NULL}};
const Segment SGI5627[] = {{&S40193[42], 13}, {NULL}};
const Segment SGI5628[] = {{&S40193[42], 24}, {NULL}};
const Segment SGI5629[] = {{&S40193[30], 4}, {NULL}};
const Segment SGI5630[] = {{&S40193[83], 4}, {NULL}};
const Segment SGI5631[] = {{&S40193[83], 3}, {NULL}};
const Segment SGI5632[] = {{&S40325[44], 5}, {NULL}};
const Segment SGI5633[] = {{&S40325[44], 16}, {NULL}};
const Segment SGI5634[] = {{&S40325[30], 6}, {NULL}};
const Segment SGI5635[] = {{&S40325[108], 3}, {NULL}};
const Segment SGI5636[] = {{&S40325[74], 5}, {NULL}};
const Segment SGI5637[] = {{&S40326[38], 5}, {NULL}};
const Segment SGI5638[] = {{&S40326[31], 6}, {NULL}};
const Segment SGI5639[] = {{&S40324[42], 5}, {NULL}};
const Segment SGI5640[] = {{&S40324[42], 16}, {NULL}};
const Segment SGI5641[] = {{&S40324[30], 4}, {NULL}};
const Segment SGI5642[] = {{&S40324[106], 3}, {NULL}};
const Segment SGI5643[] = {{&S40324[72], 5}, {NULL}};
const Segment SGI5644[] = {{&S40327[36], 5}, {NULL}};
const Segment SGI5645[] = {{&S40327[31], 4}, {NULL}};
const Segment SGI5646[] = {{&S40328[42], 5}, {NULL}};
const Segment SGI5647[] = {{&S40328[42], 16}, {NULL}};
const Segment SGI5648[] = {{&S40328[30], 4}, {NULL}};
const Segment SGI5649[] = {{&S40328[106], 3}, {NULL}};
const Segment SGI5650[] = {{&S40328[72], 5}, {NULL}};
const Segment SGI5651[] = {{&S40877[50], 10}, {NULL}};
const Segment SGI5652[] = {{&S40877[50], 22}, {NULL}};
const Segment SGI5653[] = {{&S40877[30], 12}, {NULL}};
const Segment SGI5654[] = {{&S40877[122], 3}, {NULL}};
const Segment SGI5655[] = {{&S40877[86], 7}, {NULL}};
const Segment SGI5656[] = {{&S40877[142], 8}, {NULL}};
const Segment SGI5657[] = {{&S40878[50], 10}, {NULL}};
const Segment SGI5658[] = {{&S40878[50], 22}, {NULL}};
const Segment SGI5659[] = {{&S40878[30], 12}, {NULL}};
const Segment SGI5660[] = {{&S40878[121], 3}, {NULL}};
const Segment SGI5661[] = {{&S40878[86], 6}, {NULL}};
const Segment SGI5662[] = {{&S40879[44], 10}, {NULL}};
const Segment SGI5663[] = {{&S40879[31], 12}, {NULL}};
const Segment SGI5664[] = {{&S40880[48], 10}, {NULL}};
const Segment SGI5665[] = {{&S40880[30], 12}, {NULL}};
const Segment SGI5666[] = {{&S40880[119], 3}, {NULL}};
const Segment SGI5667[] = {{&S40880[84], 6}, {NULL}};
const Segment SGI5668[] = {{&S40881[49], 10}, {NULL}};
const Segment SGI5669[] = {{&S40881[49], 22}, {NULL}};
const Segment SGI5670[] = {{&S40881[120], 3}, {NULL}};
const Segment SGI5671[] = {{&S40881[85], 6}, {NULL}};
const Segment SGI5672[] = {{&S40882[49], 10}, {NULL}};
const Segment SGI5673[] = {{&S40882[49], 22}, {NULL}};
const Segment SGI5674[] = {{&S40882[30], 11}, {NULL}};
const Segment SGI5675[] = {{&S40882[120], 3}, {NULL}};
const Segment SGI5676[] = {{&S40882[85], 6}, {NULL}};
const Segment SGI5677[] = {{&S40883[50], 9}, {NULL}};
const Segment SGI5678[] = {{&S40883[50], 21}, {NULL}};
const Segment SGI5679[] = {{&S40883[30], 12}, {NULL}};
const Segment SGI5680[] = {{&S40883[120], 3}, {NULL}};
const Segment SGI5681[] = {{&S40883[85], 6}, {NULL}};
const Segment SGI5682[] = {{&S40884[50], 9}, {NULL}};
const Segment SGI5683[] = {{&S40884[50], 21}, {NULL}};
const Segment SGI5684[] = {{&S40884[30], 12}, {NULL}};
const Segment SGI5685[] = {{&S40884[120], 3}, {NULL}};
const Segment SGI5686[] = {{&S40884[85], 6}, {NULL}};
const Segment SGI5687[] = {{&S40885[50], 10}, {NULL}};
const Segment SGI5688[] = {{&S40885[50], 22}, {NULL}};
const Segment SGI5689[] = {{&S40885[30], 12}, {NULL}};
const Segment SGI5690[] = {{&S40885[121], 3}, {NULL}};
const Segment SGI5691[] = {{&S40885[86], 6}, {NULL}};
const Segment SGI5692[] = {{&S42012[6], 4}, {NULL}};
const Segment SGI5693[] = {{&S42012[32], 3}, {NULL}};
const Segment SGI5694[] = {{&S42012[81], 5}, {NULL}};
const Segment SGI5695[] = {{&S42013[6], 4}, {NULL}};
const Segment SGI5696[] = {{&S42013[32], 3}, {NULL}};
const Segment SGI5697[] = {{&S40727[11], 1}, {NULL}};
const Segment SGI5698[] = {{&S40727[44], 8}, {NULL}};
const Segment SGI5699[] = {{&S40727[30], 6}, {NULL}};
const Segment SGI5700[] = {{&S40727[113], 3}, {NULL}};
const Segment SGI5701[] = {{&S40727[79], 5}, {NULL}};
const Segment SGI5702[] = {{&S42264[56], 10}, {NULL}};
const Segment SGI5703[] = {{&S42264[56], 27}, {NULL}};
const Segment SGI5704[] = {{&S42264[30], 18}, {NULL}};
const Segment SGI5705[] = {{&S42264[131], 3}, {NULL}};
const Segment SGI5706[] = {{&S42264[97], 5}, {NULL}};
const Segment SGI5707[] = {{&S40733[42], 9}, {NULL}};
const Segment SGI5708[] = {{&S40733[42], 20}, {NULL}};
const Segment SGI5709[] = {{&S40733[30], 4}, {NULL}};
const Segment SGI5710[] = {{&S40733[110], 3}, {NULL}};
const Segment SGI5711[] = {{&S40733[76], 5}, {NULL}};
const Segment SGI5712[] = {{&S40957[44], 10}, {NULL}};
const Segment SGI5713[] = {{&S40957[44], 21}, {NULL}};
const Segment SGI5714[] = {{&S40957[30], 6}, {NULL}};
const Segment SGI5715[] = {{&S40957[113], 3}, {NULL}};
const Segment SGI5716[] = {{&S40957[79], 5}, {NULL}};
const Segment SGI5717[] = {{&S40959[38], 10}, {NULL}};
const Segment SGI5718[] = {{&S40959[31], 6}, {NULL}};
const Segment SGI5719[] = {{&S42522[6], 4}, {NULL}};
const Segment SGI5720[] = {{&S42522[32], 7}, {NULL}};
const Segment SGI5721[] = {{&S42522[82], 5}, {NULL}};
const Segment SGI5722[] = {{&S42522[88], 13}, {NULL}};
const Segment SGI5723[] = {{&S41079[44], 10}, {NULL}};
const Segment SGI5724[] = {{&S41079[44], 21}, {NULL}};
const Segment SGI5725[] = {{&S41079[30], 6}, {NULL}};
const Segment SGI5726[] = {{&S41079[113], 3}, {NULL}};
const Segment SGI5727[] = {{&S41079[79], 5}, {NULL}};
const Segment SGI5728[] = {{&S41080[38], 10}, {NULL}};
const Segment SGI5729[] = {{&S41080[31], 6}, {NULL}};
const Segment SGI5730[] = {{&S41083[44], 10}, {NULL}};
const Segment SGI5731[] = {{&S41083[44], 21}, {NULL}};
const Segment SGI5732[] = {{&S41083[30], 6}, {NULL}};
const Segment SGI5733[] = {{&S42645[6], 4}, {NULL}};
const Segment SGI5734[] = {{&S42645[32], 7}, {NULL}};
const Segment SGI5735[] = {{&S42645[82], 5}, {NULL}};
const Segment SGI5736[] = {{&S42645[88], 13}, {NULL}};
const Segment SGI5737[] = {{&S42686[37], 7}, {NULL}};
const Segment SGI5738[] = {{&S42686[30], 6}, {NULL}};
const Segment SGI5739[] = {{&S42686[105], 3}, {NULL}};
const Segment SGI5740[] = {{&S42686[70], 6}, {NULL}};
const Segment SGI5741[] = {{&S41171[44], 18}, {NULL}};
const Segment SGI5742[] = {{&S41171[44], 30}, {NULL}};
const Segment SGI5743[] = {{&S41171[30], 6}, {NULL}};
const Segment SGI5744[] = {{&S41171[122], 3}, {NULL}};
const Segment SGI5745[] = {{&S41171[88], 5}, {NULL}};
const Segment SGI5746[] = {{&S41173[44], 18}, {NULL}};
const Segment SGI5747[] = {{&S41173[44], 29}, {NULL}};
const Segment SGI5748[] = {{&S41173[30], 6}, {NULL}};
const Segment SGI5749[] = {{&S41173[121], 3}, {NULL}};
const Segment SGI5750[] = {{&S41173[87], 5}, {NULL}};
const Segment SGI5751[] = {{&S41174[44], 9}, {NULL}};
const Segment SGI5752[] = {{&S41174[44], 20}, {NULL}};
const Segment SGI5753[] = {{&S41174[30], 6}, {NULL}};
const Segment SGI5754[] = {{&S41174[112], 3}, {NULL}};
const Segment SGI5755[] = {{&S41174[78], 5}, {NULL}};
const Segment SGI5756[] = {{&S41178[44], 9}, {NULL}};
const Segment SGI5757[] = {{&S41178[44], 20}, {NULL}};
const Segment SGI5758[] = {{&S41178[30], 6}, {NULL}};
const Segment SGI5759[] = {{&S41178[112], 3}, {NULL}};
const Segment SGI5760[] = {{&S41178[78], 5}, {NULL}};
const Segment SGI5761[] = {{&S41179[44], 9}, {NULL}};
const Segment SGI5762[] = {{&S41179[44], 19}, {NULL}};
const Segment SGI5763[] = {{&S41179[30], 6}, {NULL}};
const Segment SGI5764[] = {{&S41179[111], 3}, {NULL}};
const Segment SGI5765[] = {{&S41179[77], 5}, {NULL}};
const Segment SGI5766[] = {{&S42708[40], 7}, {NULL}};
const Segment SGI5767[] = {{&S42708[30], 9}, {NULL}};
const Segment SGI5768[] = {{&S42708[108], 3}, {NULL}};
const Segment SGI5769[] = {{&S42708[73], 6}, {NULL}};
const Segment SGI5770[] = {{&S42709[40], 7}, {NULL}};
const Segment SGI5771[] = {{&S42709[30], 9}, {NULL}};
const Segment SGI5772[] = {{&S42709[108], 3}, {NULL}};
const Segment SGI5773[] = {{&S42709[73], 6}, {NULL}};
const Segment SGI5774[] = {{&S41169[42], 18}, {NULL}};
const Segment SGI5775[] = {{&S41169[42], 46}, {NULL}};
const Segment SGI5776[] = {{&S41169[30], 4}, {NULL}};
const Segment SGI5777[] = {{&S41169[136], 3}, {NULL}};
const Segment SGI5778[] = {{&S41169[102], 5}, {NULL}};
const Segment SGI5779[] = {{&S41170[42], 19}, {NULL}};
const Segment SGI5780[] = {{&S41170[42], 30}, {NULL}};
const Segment SGI5781[] = {{&S41170[30], 4}, {NULL}};
const Segment SGI5782[] = {{&S41170[120], 3}, {NULL}};
const Segment SGI5783[] = {{&S41170[86], 5}, {NULL}};
const Segment SGI5784[] = {{&S41172[40], 18}, {NULL}};
const Segment SGI5785[] = {{&S41172[30], 4}, {NULL}};
const Segment SGI5786[] = {{&S41172[117], 3}, {NULL}};
const Segment SGI5787[] = {{&S41172[83], 5}, {NULL}};
const Segment SGI5788[] = {{&S41175[42], 9}, {NULL}};
const Segment SGI5789[] = {{&S41175[42], 20}, {NULL}};
const Segment SGI5790[] = {{&S41175[30], 4}, {NULL}};
const Segment SGI5791[] = {{&S41175[110], 3}, {NULL}};
const Segment SGI5792[] = {{&S41175[76], 5}, {NULL}};
const Segment SGI5793[] = {{&S41176[42], 18}, {NULL}};
const Segment SGI5794[] = {{&S41176[42], 29}, {NULL}};
const Segment SGI5795[] = {{&S41176[30], 4}, {NULL}};
const Segment SGI5796[] = {{&S41176[119], 3}, {NULL}};
const Segment SGI5797[] = {{&S41176[85], 5}, {NULL}};
const Segment SGI5798[] = {{&S41177[36], 9}, {NULL}};
const Segment SGI5799[] = {{&S41177[31], 4}, {NULL}};
const Segment SGI5800[] = {{&S41180[42], 9}, {NULL}};
const Segment SGI5801[] = {{&S41180[42], 19}, {NULL}};
const Segment SGI5802[] = {{&S41180[30], 4}, {NULL}};
const Segment SGI5803[] = {{&S41180[109], 3}, {NULL}};
const Segment SGI5804[] = {{&S41180[75], 5}, {NULL}};
const Segment SGI5805[] = {{&S41181[42], 9}, {NULL}};
const Segment SGI5806[] = {{&S41181[42], 20}, {NULL}};
const Segment SGI5807[] = {{&S41181[30], 4}, {NULL}};
const Segment SGI5808[] = {{&S41181[79], 4}, {NULL}};
const Segment SGI5809[] = {{&S41181[79], 3}, {NULL}};
const Segment SGI5810[] = {{&S41182[42], 9}, {NULL}};
const Segment SGI5811[] = {{&S41182[42], 20}, {NULL}};
const Segment SGI5812[] = {{&S41182[30], 4}, {NULL}};
const Segment SGI5813[] = {{&S41182[79], 6}, {NULL}};
const Segment SGI5814[] = {{&S42684[66], 5}, {NULL}};
const Segment SGI5815[] = {{&S42684[66], 17}, {NULL}};
const Segment SGI5816[] = {{&S42684[30], 28}, {NULL}};
const Segment SGI5817[] = {{&S42684[132], 3}, {NULL}};
const Segment SGI5818[] = {{&S42684[97], 6}, {NULL}};
const Segment SGI5819[] = {{&S42685[45], 7}, {NULL}};
const Segment SGI5820[] = {{&S42685[45], 19}, {NULL}};
const Segment SGI5821[] = {{&S42685[30], 7}, {NULL}};
const Segment SGI5822[] = {{&S42685[113], 3}, {NULL}};
const Segment SGI5823[] = {{&S42685[78], 6}, {NULL}};
const Segment SGI5824[] = {{&S42687[65], 5}, {NULL}};
const Segment SGI5825[] = {{&S42687[65], 17}, {NULL}};
const Segment SGI5826[] = {{&S42687[30], 27}, {NULL}};
const Segment SGI5827[] = {{&S42687[131], 3}, {NULL}};
const Segment SGI5828[] = {{&S42687[96], 6}, {NULL}};
const Segment SGI5829[] = {{&S42688[65], 5}, {NULL}};
const Segment SGI5830[] = {{&S42688[65], 17}, {NULL}};
const Segment SGI5831[] = {{&S42688[30], 27}, {NULL}};
const Segment SGI5832[] = {{&S42688[131], 3}, {NULL}};
const Segment SGI5833[] = {{&S42688[96], 6}, {NULL}};
const Segment SGI5834[] = {{&S42689[38], 5}, {NULL}};
const Segment SGI5835[] = {{&S42689[30], 7}, {NULL}};
const Segment SGI5836[] = {{&S42689[104], 3}, {NULL}};
const Segment SGI5837[] = {{&S42689[69], 6}, {NULL}};
const Segment SGI5838[] = {{&S42690[38], 7}, {NULL}};
const Segment SGI5839[] = {{&S42690[30], 7}, {NULL}};
const Segment SGI5840[] = {{&S42690[106], 3}, {NULL}};
const Segment SGI5841[] = {{&S42690[71], 6}, {NULL}};
const Segment SGI5842[] = {{&S42691[38], 7}, {NULL}};
const Segment SGI5843[] = {{&S42691[30], 7}, {NULL}};
const Segment SGI5844[] = {{&S42691[106], 3}, {NULL}};
const Segment SGI5845[] = {{&S42691[71], 6}, {NULL}};
const Segment SGI5846[] = {{&S42692[64], 7}, {NULL}};
const Segment SGI5847[] = {{&S42692[30], 33}, {NULL}};
const Segment SGI5848[] = {{&S42692[132], 3}, {NULL}};
const Segment SGI5849[] = {{&S42692[97], 6}, {NULL}};
const Segment SGI5850[] = {{&S42693[64], 7}, {NULL}};
const Segment SGI5851[] = {{&S42693[30], 33}, {NULL}};
const Segment SGI5852[] = {{&S42693[132], 3}, {NULL}};
const Segment SGI5853[] = {{&S42693[97], 6}, {NULL}};
const Segment SGI5854[] = {{&S42694[38], 7}, {NULL}};
const Segment SGI5855[] = {{&S42694[30], 7}, {NULL}};
const Segment SGI5856[] = {{&S42694[106], 3}, {NULL}};
const Segment SGI5857[] = {{&S42694[71], 6}, {NULL}};
const Segment SGI5858[] = {{&S42695[67], 5}, {NULL}};
const Segment SGI5859[] = {{&S42695[67], 17}, {NULL}};
const Segment SGI5860[] = {{&S42695[30], 29}, {NULL}};
const Segment SGI5861[] = {{&S42695[133], 3}, {NULL}};
const Segment SGI5862[] = {{&S42695[98], 6}, {NULL}};
const Segment SGI5863[] = {{&S42696[45], 2}, {NULL}};
const Segment SGI5864[] = {{&S42696[45], 22}, {NULL}};
const Segment SGI5865[] = {{&S42696[30], 7}, {NULL}};
const Segment SGI5866[] = {{&S42696[116], 3}, {NULL}};
const Segment SGI5867[] = {{&S42696[81], 6}, {NULL}};
const Segment SGI5868[] = {{&S42697[38], 7}, {NULL}};
const Segment SGI5869[] = {{&S42697[30], 7}, {NULL}};
const Segment SGI5870[] = {{&S42697[106], 3}, {NULL}};
const Segment SGI5871[] = {{&S42697[71], 6}, {NULL}};
const Segment SGI5872[] = {{&S42698[61], 7}, {NULL}};
const Segment SGI5873[] = {{&S42698[30], 30}, {NULL}};
const Segment SGI5874[] = {{&S42698[129], 3}, {NULL}};
const Segment SGI5875[] = {{&S42698[94], 6}, {NULL}};
const Segment SGI5876[] = {{&S42699[38], 7}, {NULL}};
const Segment SGI5877[] = {{&S42699[30], 7}, {NULL}};
const Segment SGI5878[] = {{&S42699[106], 3}, {NULL}};
const Segment SGI5879[] = {{&S42699[71], 6}, {NULL}};
const Segment SGI5880[] = {{&S42700[38], 7}, {NULL}};
const Segment SGI5881[] = {{&S42700[30], 7}, {NULL}};
const Segment SGI5882[] = {{&S42700[106], 3}, {NULL}};
const Segment SGI5883[] = {{&S42700[71], 6}, {NULL}};
const Segment SGI5884[] = {{&S42701[45], 2}, {NULL}};
const Segment SGI5885[] = {{&S42701[45], 20}, {NULL}};
const Segment SGI5886[] = {{&S42701[30], 7}, {NULL}};
const Segment SGI5887[] = {{&S42701[114], 3}, {NULL}};
const Segment SGI5888[] = {{&S42701[79], 6}, {NULL}};
const Segment SGI5889[] = {{&S42702[38], 7}, {NULL}};
const Segment SGI5890[] = {{&S42702[30], 7}, {NULL}};
const Segment SGI5891[] = {{&S42702[106], 3}, {NULL}};
const Segment SGI5892[] = {{&S42702[71], 6}, {NULL}};
const Segment SGI5893[] = {{&S42703[38], 7}, {NULL}};
const Segment SGI5894[] = {{&S42703[30], 7}, {NULL}};
const Segment SGI5895[] = {{&S42703[106], 3}, {NULL}};
const Segment SGI5896[] = {{&S42703[71], 6}, {NULL}};
const Segment SGI5897[] = {{&S42704[38], 7}, {NULL}};
const Segment SGI5898[] = {{&S42704[30], 7}, {NULL}};
const Segment SGI5899[] = {{&S42704[106], 3}, {NULL}};
const Segment SGI5900[] = {{&S42704[71], 6}, {NULL}};
const Segment SGI5901[] = {{&S42705[38], 7}, {NULL}};
const Segment SGI5902[] = {{&S42705[30], 7}, {NULL}};
const Segment SGI5903[] = {{&S42705[106], 3}, {NULL}};
const Segment SGI5904[] = {{&S42705[71], 6}, {NULL}};
const Segment SGI5905[] = {{&S42706[38], 7}, {NULL}};
const Segment SGI5906[] = {{&S42706[30], 7}, {NULL}};
const Segment SGI5907[] = {{&S42706[106], 3}, {NULL}};
const Segment SGI5908[] = {{&S42706[71], 6}, {NULL}};
const Segment SGI5909[] = {{&S41493[44], 10}, {NULL}};
const Segment SGI5910[] = {{&S41493[44], 21}, {NULL}};
const Segment SGI5911[] = {{&S41493[30], 6}, {NULL}};
const Segment SGI5912[] = {{&S41493[113], 3}, {NULL}};
const Segment SGI5913[] = {{&S41493[79], 5}, {NULL}};
const Segment SGI5914[] = {{&S42023[52], 10}, {NULL}};
const Segment SGI5915[] = {{&S42023[52], 21}, {NULL}};
const Segment SGI5916[] = {{&S42023[30], 14}, {NULL}};
const Segment SGI5917[] = {{&S42023[122], 3}, {NULL}};
const Segment SGI5918[] = {{&S42023[87], 6}, {NULL}};
const Segment SGI5919[] = {{&S42024[42], 13}, {NULL}};
const Segment SGI5920[] = {{&S42024[42], 24}, {NULL}};
const Segment SGI5921[] = {{&S42024[30], 4}, {NULL}};
const Segment SGI5922[] = {{&S42024[115], 3}, {NULL}};
const Segment SGI5923[] = {{&S42024[80], 6}, {NULL}};
const Segment SGI5924[] = {{&S42025[42], 10}, {NULL}};
const Segment SGI5925[] = {{&S42025[42], 21}, {NULL}};
const Segment SGI5926[] = {{&S42025[30], 4}, {NULL}};
const Segment SGI5927[] = {{&S42025[112], 3}, {NULL}};
const Segment SGI5928[] = {{&S42025[77], 6}, {NULL}};
const Segment SGI5929[] = {{&S42026[67], 10}, {NULL}};
const Segment SGI5930[] = {{&S42026[67], 21}, {NULL}};
const Segment SGI5931[] = {{&S42026[30], 28}, {NULL}};
const Segment SGI5932[] = {{&S42026[137], 3}, {NULL}};
const Segment SGI5933[] = {{&S42026[102], 6}, {NULL}};
const Segment SGI5934[] = {{&S42027[50], 17}, {NULL}};
const Segment SGI5935[] = {{&S42027[50], 28}, {NULL}};
const Segment SGI5936[] = {{&S42027[30], 12}, {NULL}};
const Segment SGI5937[] = {{&S42027[127], 3}, {NULL}};
const Segment SGI5938[] = {{&S42027[92], 6}, {NULL}};
const Segment SGI5939[] = {{&S42028[50], 10}, {NULL}};
const Segment SGI5940[] = {{&S42028[50], 21}, {NULL}};
const Segment SGI5941[] = {{&S42028[30], 12}, {NULL}};
const Segment SGI5942[] = {{&S42028[120], 3}, {NULL}};
const Segment SGI5943[] = {{&S42028[85], 6}, {NULL}};
const Segment SGI5944[] = {{&S42029[48], 10}, {NULL}};
const Segment SGI5945[] = {{&S42029[30], 12}, {NULL}};
const Segment SGI5946[] = {{&S42029[118], 3}, {NULL}};
const Segment SGI5947[] = {{&S42029[83], 6}, {NULL}};
const Segment SGI5948[] = {{&S42030[42], 10}, {NULL}};
const Segment SGI5949[] = {{&S42030[42], 21}, {NULL}};
const Segment SGI5950[] = {{&S42030[30], 4}, {NULL}};
const Segment SGI5951[] = {{&S42030[112], 3}, {NULL}};
const Segment SGI5952[] = {{&S42030[77], 6}, {NULL}};
const Segment SGI5953[] = {{&S42031[50], 10}, {NULL}};
const Segment SGI5954[] = {{&S42031[50], 21}, {NULL}};
const Segment SGI5955[] = {{&S42031[30], 12}, {NULL}};
const Segment SGI5956[] = {{&S42031[120], 3}, {NULL}};
const Segment SGI5957[] = {{&S42031[85], 6}, {NULL}};
const Segment SGI5958[] = {{&S42032[50], 10}, {NULL}};
const Segment SGI5959[] = {{&S42032[50], 21}, {NULL}};
const Segment SGI5960[] = {{&S42032[30], 12}, {NULL}};
const Segment SGI5961[] = {{&S42032[120], 3}, {NULL}};
const Segment SGI5962[] = {{&S42032[85], 6}, {NULL}};
const Segment SGI5963[] = {{&S42033[55], 10}, {NULL}};
const Segment SGI5964[] = {{&S42033[55], 21}, {NULL}};
const Segment SGI5965[] = {{&S42033[30], 17}, {NULL}};
const Segment SGI5966[] = {{&S42033[125], 3}, {NULL}};
const Segment SGI5967[] = {{&S42033[90], 6}, {NULL}};
const Segment SGI5968[] = {{&S42034[50], 10}, {NULL}};
const Segment SGI5969[] = {{&S42034[50], 21}, {NULL}};
const Segment SGI5970[] = {{&S42034[30], 12}, {NULL}};
const Segment SGI5971[] = {{&S42034[88], 4}, {NULL}};
const Segment SGI5972[] = {{&S42034[88], 3}, {NULL}};
const Segment SGI5973[] = {{&S42035[42], 10}, {NULL}};
const Segment SGI5974[] = {{&S42035[42], 21}, {NULL}};
const Segment SGI5975[] = {{&S42035[30], 4}, {NULL}};
const Segment SGI5976[] = {{&S42035[112], 3}, {NULL}};
const Segment SGI5977[] = {{&S42035[77], 6}, {NULL}};
const Segment SGI5978[] = {{&S41899[44], 8}, {NULL}};
const Segment SGI5979[] = {{&S41899[44], 17}, {NULL}};
const Segment SGI5980[] = {{&S41899[30], 6}, {NULL}};
const Segment SGI5981[] = {{&S41899[109], 3}, {NULL}};
const Segment SGI5982[] = {{&S41899[75], 5}, {NULL}};
const Segment SGI5983[] = {{&S41900[44], 7}, {NULL}};
const Segment SGI5984[] = {{&S41900[44], 16}, {NULL}};
const Segment SGI5985[] = {{&S41900[30], 6}, {NULL}};
const Segment SGI5986[] = {{&S41900[108], 3}, {NULL}};
const Segment SGI5987[] = {{&S41900[74], 5}, {NULL}};
const Segment SGI5988[] = {{&S41901[43], 8}, {NULL}};
const Segment SGI5989[] = {{&S41901[43], 17}, {NULL}};
const Segment SGI5990[] = {{&S41901[30], 5}, {NULL}};
const Segment SGI5991[] = {{&S41901[108], 3}, {NULL}};
const Segment SGI5992[] = {{&S41901[74], 5}, {NULL}};
const Segment SGI5993[] = {{&S41902[44], 7}, {NULL}};
const Segment SGI5994[] = {{&S41902[44], 16}, {NULL}};
const Segment SGI5995[] = {{&S41902[30], 6}, {NULL}};
const Segment SGI5996[] = {{&S41902[108], 3}, {NULL}};
const Segment SGI5997[] = {{&S41902[74], 5}, {NULL}};
const Segment SGI5998[] = {{&S41903[44], 8}, {NULL}};
const Segment SGI5999[] = {{&S41903[44], 17}, {NULL}};
const Segment SGI6000[] = {{&S41903[30], 6}, {NULL}};
const Segment SGI6001[] = {{&S41903[109], 3}, {NULL}};
const Segment SGI6002[] = {{&S41903[75], 5}, {NULL}};
const Segment SGI6003[] = {{&S41903[127], 8}, {NULL}};
const Segment SGI6004[] = {{&S41904[43], 8}, {NULL}};
const Segment SGI6005[] = {{&S41904[43], 17}, {NULL}};
const Segment SGI6006[] = {{&S41904[108], 3}, {NULL}};
const Segment SGI6007[] = {{&S41904[74], 5}, {NULL}};
const Segment SGI6008[] = {{&S41905[44], 8}, {NULL}};
const Segment SGI6009[] = {{&S41905[44], 17}, {NULL}};
const Segment SGI6010[] = {{&S41905[30], 6}, {NULL}};
const Segment SGI6011[] = {{&S41905[109], 3}, {NULL}};
const Segment SGI6012[] = {{&S41905[75], 5}, {NULL}};
const Segment SGI6013[] = {{&S41907[38], 8}, {NULL}};
const Segment SGI6014[] = {{&S41907[31], 6}, {NULL}};
const Segment SGI6015[] = {{&S43462[6], 4}, {NULL}};
const Segment SGI6016[] = {{&S43462[32], 7}, {NULL}};
const Segment SGI6017[] = {{&S43462[82], 5}, {NULL}};
const Segment SGI6018[] = {{&S43462[88], 13}, {NULL}};
const Segment SGI6019[] = {{&S44803[41], 7}, {NULL}};
const Segment SGI6020[] = {{&S44803[41], 20}, {NULL}};
const Segment SGI6021[] = {{&S44803[110], 3}, {NULL}};
const Segment SGI6022[] = {{&S44803[75], 6}, {NULL}};
const Segment SGI6023[] = {{&S44803[130], 8}, {NULL}};
const Segment SGI6024[] = {{&S44804[42], 7}, {NULL}};
const Segment SGI6025[] = {{&S44804[42], 20}, {NULL}};
const Segment SGI6026[] = {{&S44804[30], 4}, {NULL}};
const Segment SGI6027[] = {{&S44804[111], 3}, {NULL}};
const Segment SGI6028[] = {{&S44804[76], 6}, {NULL}};
const Segment SGI6029[] = {{&S44805[41], 7}, {NULL}};
const Segment SGI6030[] = {{&S44805[41], 20}, {NULL}};
const Segment SGI6031[] = {{&S44805[30], 3}, {NULL}};
const Segment SGI6032[] = {{&S44805[110], 3}, {NULL}};
const Segment SGI6033[] = {{&S44805[75], 6}, {NULL}};
const Segment SGI6034[] = {{&S44805[130], 8}, {NULL}};
const Segment SGI6035[] = {{&S44806[42], 6}, {NULL}};
const Segment SGI6036[] = {{&S44806[42], 19}, {NULL}};
const Segment SGI6037[] = {{&S44806[30], 4}, {NULL}};
const Segment SGI6038[] = {{&S44806[110], 3}, {NULL}};
const Segment SGI6039[] = {{&S44806[75], 6}, {NULL}};
const Segment SGI6040[] = {{&S44806[130], 8}, {NULL}};
const Segment SGI6041[] = {{&S44807[42], 6}, {NULL}};
const Segment SGI6042[] = {{&S44807[42], 19}, {NULL}};
const Segment SGI6043[] = {{&S44807[30], 4}, {NULL}};
const Segment SGI6044[] = {{&S44807[110], 3}, {NULL}};
const Segment SGI6045[] = {{&S44807[75], 6}, {NULL}};
const Segment SGI6046[] = {{&S44807[130], 8}, {NULL}};
const Segment SGI6047[] = {{&S44808[40], 7}, {NULL}};
const Segment SGI6048[] = {{&S44808[30], 4}, {NULL}};
const Segment SGI6049[] = {{&S44808[109], 3}, {NULL}};
const Segment SGI6050[] = {{&S44808[74], 6}, {NULL}};
const Segment SGI6051[] = {{&S44808[129], 8}, {NULL}};
const Segment SGI6052[] = {{&S44809[42], 7}, {NULL}};
const Segment SGI6053[] = {{&S44809[42], 20}, {NULL}};
const Segment SGI6054[] = {{&S44809[30], 4}, {NULL}};
const Segment SGI6055[] = {{&S44809[111], 3}, {NULL}};
const Segment SGI6056[] = {{&S44809[76], 6}, {NULL}};
const Segment SGI6057[] = {{&S44809[131], 8}, {NULL}};
const Segment SGI6058[] = {{&S44810[42], 7}, {NULL}};
const Segment SGI6059[] = {{&S44810[42], 20}, {NULL}};
const Segment SGI6060[] = {{&S44810[30], 4}, {NULL}};
const Segment SGI6061[] = {{&S44810[112], 3}, {NULL}};
const Segment SGI6062[] = {{&S44810[76], 7}, {NULL}};
const Segment SGI6063[] = {{&S44810[132], 8}, {NULL}};
const Segment SGI6064[] = {{&S41898[42], 8}, {NULL}};
const Segment SGI6065[] = {{&S41898[42], 17}, {NULL}};
const Segment SGI6066[] = {{&S41898[30], 4}, {NULL}};
const Segment SGI6067[] = {{&S41898[107], 3}, {NULL}};
const Segment SGI6068[] = {{&S41898[73], 5}, {NULL}};
const Segment SGI6069[] = {{&S41906[42], 8}, {NULL}};
const Segment SGI6070[] = {{&S41906[42], 20}, {NULL}};
const Segment SGI6071[] = {{&S41906[30], 4}, {NULL}};
const Segment SGI6072[] = {{&S41906[110], 3}, {NULL}};
const Segment SGI6073[] = {{&S41906[76], 5}, {NULL}};
const Segment SGI6074[] = {{&S41908[36], 8}, {NULL}};
const Segment SGI6075[] = {{&S41908[31], 4}, {NULL}};
const Segment SGI6076[] = {{&S41933[42], 9}, {NULL}};
const Segment SGI6077[] = {{&S41933[42], 20}, {NULL}};
const Segment SGI6078[] = {{&S41933[30], 4}, {NULL}};
const Segment SGI6079[] = {{&S41933[110], 3}, {NULL}};
const Segment SGI6080[] = {{&S41933[76], 5}, {NULL}};
const Segment SGI6081[] = {{&S41943[42], 9}, {NULL}};
const Segment SGI6082[] = {{&S41943[42], 20}, {NULL}};
const Segment SGI6083[] = {{&S41943[30], 4}, {NULL}};
const Segment SGI6084[] = {{&S41943[110], 3}, {NULL}};
const Segment SGI6085[] = {{&S41943[76], 5}, {NULL}};
const Segment SGI6086[] = {{&S41944[42], 29}, {NULL}};
const Segment SGI6087[] = {{&S41944[42], 40}, {NULL}};
const Segment SGI6088[] = {{&S41944[30], 4}, {NULL}};
const Segment SGI6089[] = {{&S41944[130], 3}, {NULL}};
const Segment SGI6090[] = {{&S41944[96], 5}, {NULL}};
const Segment SGI6091[] = {{&S41945[42], 11}, {NULL}};
const Segment SGI6092[] = {{&S41945[42], 22}, {NULL}};
const Segment SGI6093[] = {{&S41945[30], 4}, {NULL}};
const Segment SGI6094[] = {{&S41945[112], 3}, {NULL}};
const Segment SGI6095[] = {{&S41945[78], 5}, {NULL}};
const Segment SGI6096[] = {{&S41946[41], 11}, {NULL}};
const Segment SGI6097[] = {{&S41946[41], 22}, {NULL}};
const Segment SGI6098[] = {{&S41946[111], 3}, {NULL}};
const Segment SGI6099[] = {{&S41946[77], 5}, {NULL}};
const Segment SGI6100[] = {{&S41947[36], 9}, {NULL}};
const Segment SGI6101[] = {{&S41947[31], 4}, {NULL}};
const Segment SGI6102[] = {{&S41948[42], 9}, {NULL}};
const Segment SGI6103[] = {{&S41948[42], 20}, {NULL}};
const Segment SGI6104[] = {{&S41948[30], 4}, {NULL}};
const Segment SGI6105[] = {{&S41948[112], 3}, {NULL}};
const Segment SGI6106[] = {{&S41948[76], 7}, {NULL}};
const Segment SGI6107[] = {{&S41948[132], 8}, {NULL}};
const Segment SGI6108[] = {{&S41949[42], 11}, {NULL}};
const Segment SGI6109[] = {{&S41949[42], 22}, {NULL}};
const Segment SGI6110[] = {{&S41949[30], 4}, {NULL}};
const Segment SGI6111[] = {{&S41949[112], 3}, {NULL}};
const Segment SGI6112[] = {{&S41949[78], 5}, {NULL}};
const Segment SGI6113[] = {{&S42455[50], 10}, {NULL}};
const Segment SGI6114[] = {{&S42455[50], 22}, {NULL}};
const Segment SGI6115[] = {{&S42455[30], 12}, {NULL}};
const Segment SGI6116[] = {{&S42455[121], 3}, {NULL}};
const Segment SGI6117[] = {{&S42455[86], 6}, {NULL}};
const Segment SGI6118[] = {{&S42456[44], 10}, {NULL}};
const Segment SGI6119[] = {{&S42456[31], 12}, {NULL}};
const Segment SGI6120[] = {{&S42457[57], 10}, {NULL}};
const Segment SGI6121[] = {{&S42457[57], 22}, {NULL}};
const Segment SGI6122[] = {{&S42457[45], 4}, {NULL}};
const Segment SGI6123[] = {{&S42457[128], 3}, {NULL}};
const Segment SGI6124[] = {{&S42457[93], 6}, {NULL}};
const Segment SGI6125[] = {{&S42217[44], 21}, {NULL}};
const Segment SGI6126[] = {{&S42217[44], 32}, {NULL}};
const Segment SGI6127[] = {{&S42217[30], 6}, {NULL}};
const Segment SGI6128[] = {{&S42217[124], 3}, {NULL}};
const Segment SGI6129[] = {{&S42217[90], 5}, {NULL}};
const Segment SGI6130[] = {{&S42218[44], 21}, {NULL}};
const Segment SGI6131[] = {{&S42218[44], 33}, {NULL}};
const Segment SGI6132[] = {{&S42218[30], 6}, {NULL}};
const Segment SGI6133[] = {{&S42218[125], 3}, {NULL}};
const Segment SGI6134[] = {{&S42218[91], 5}, {NULL}};
const Segment SGI6135[] = {{&S42219[44], 21}, {NULL}};
const Segment SGI6136[] = {{&S42219[44], 33}, {NULL}};
const Segment SGI6137[] = {{&S42219[30], 6}, {NULL}};
const Segment SGI6138[] = {{&S42219[125], 3}, {NULL}};
const Segment SGI6139[] = {{&S42219[91], 5}, {NULL}};
const Segment SGI6140[] = {{&S42220[43], 21}, {NULL}};
const Segment SGI6141[] = {{&S42220[43], 33}, {NULL}};
const Segment SGI6142[] = {{&S42220[124], 3}, {NULL}};
const Segment SGI6143[] = {{&S42220[90], 5}, {NULL}};
const Segment SGI6144[] = {{&S42221[42], 21}, {NULL}};
const Segment SGI6145[] = {{&S42221[30], 6}, {NULL}};
const Segment SGI6146[] = {{&S42221[123], 3}, {NULL}};
const Segment SGI6147[] = {{&S42221[89], 5}, {NULL}};
const Segment SGI6148[] = {{&S42222[44], 20}, {NULL}};
const Segment SGI6149[] = {{&S42222[44], 32}, {NULL}};
const Segment SGI6150[] = {{&S42222[30], 6}, {NULL}};
const Segment SGI6151[] = {{&S42222[124], 3}, {NULL}};
const Segment SGI6152[] = {{&S42222[90], 5}, {NULL}};
const Segment SGI6153[] = {{&S42223[43], 21}, {NULL}};
const Segment SGI6154[] = {{&S42223[43], 33}, {NULL}};
const Segment SGI6155[] = {{&S42223[30], 5}, {NULL}};
const Segment SGI6156[] = {{&S42223[124], 3}, {NULL}};
const Segment SGI6157[] = {{&S42223[90], 5}, {NULL}};
const Segment SGI6158[] = {{&S42224[44], 20}, {NULL}};
const Segment SGI6159[] = {{&S42224[44], 32}, {NULL}};
const Segment SGI6160[] = {{&S42224[30], 6}, {NULL}};
const Segment SGI6161[] = {{&S42224[124], 3}, {NULL}};
const Segment SGI6162[] = {{&S42224[90], 5}, {NULL}};
const Segment SGI6163[] = {{&S42225[44], 20}, {NULL}};
const Segment SGI6164[] = {{&S42225[44], 32}, {NULL}};
const Segment SGI6165[] = {{&S42225[30], 6}, {NULL}};
const Segment SGI6166[] = {{&S42225[124], 3}, {NULL}};
const Segment SGI6167[] = {{&S42225[90], 5}, {NULL}};
const Segment SGI6168[] = {{&S42226[44], 21}, {NULL}};
const Segment SGI6169[] = {{&S42226[44], 33}, {NULL}};
const Segment SGI6170[] = {{&S42226[30], 6}, {NULL}};
const Segment SGI6171[] = {{&S42226[125], 3}, {NULL}};
const Segment SGI6172[] = {{&S42226[91], 5}, {NULL}};
const Segment SGI6173[] = {{&S42227[44], 20}, {NULL}};
const Segment SGI6174[] = {{&S42227[44], 32}, {NULL}};
const Segment SGI6175[] = {{&S42227[30], 6}, {NULL}};
const Segment SGI6176[] = {{&S42227[124], 3}, {NULL}};
const Segment SGI6177[] = {{&S42227[90], 5}, {NULL}};
const Segment SGI6178[] = {{&S42228[44], 21}, {NULL}};
const Segment SGI6179[] = {{&S42228[44], 33}, {NULL}};
const Segment SGI6180[] = {{&S42228[30], 6}, {NULL}};
const Segment SGI6181[] = {{&S42228[127], 3}, {NULL}};
const Segment SGI6182[] = {{&S42228[91], 7}, {NULL}};
const Segment SGI6183[] = {{&S42228[147], 8}, {NULL}};
const Segment SGI6184[] = {{&S42229[38], 21}, {NULL}};
const Segment SGI6185[] = {{&S42229[31], 6}, {NULL}};
const Segment SGI6186[] = {{&S42232[44], 21}, {NULL}};
const Segment SGI6187[] = {{&S42232[44], 36}, {NULL}};
const Segment SGI6188[] = {{&S42232[30], 6}, {NULL}};
const Segment SGI6189[] = {{&S42232[128], 3}, {NULL}};
const Segment SGI6190[] = {{&S42232[94], 5}, {NULL}};
const Segment SGI6191[] = {{&S42446[44], 12}, {NULL}};
const Segment SGI6192[] = {{&S42446[44], 24}, {NULL}};
const Segment SGI6193[] = {{&S42446[30], 6}, {NULL}};
const Segment SGI6194[] = {{&S42446[116], 3}, {NULL}};
const Segment SGI6195[] = {{&S42446[82], 5}, {NULL}};
const Segment SGI6196[] = {{&S42447[44], 14}, {NULL}};
const Segment SGI6197[] = {{&S42447[44], 26}, {NULL}};
const Segment SGI6198[] = {{&S42447[30], 6}, {NULL}};
const Segment SGI6199[] = {{&S42447[118], 3}, {NULL}};
const Segment SGI6200[] = {{&S42447[84], 5}, {NULL}};
const Segment SGI6201[] = {{&S42448[44], 14}, {NULL}};
const Segment SGI6202[] = {{&S42448[44], 26}, {NULL}};
const Segment SGI6203[] = {{&S42448[30], 6}, {NULL}};
const Segment SGI6204[] = {{&S42448[118], 3}, {NULL}};
const Segment SGI6205[] = {{&S42448[84], 5}, {NULL}};
const Segment SGI6206[] = {{&S42449[44], 15}, {NULL}};
const Segment SGI6207[] = {{&S42449[44], 26}, {NULL}};
const Segment SGI6208[] = {{&S42449[30], 6}, {NULL}};
const Segment SGI6209[] = {{&S42449[118], 3}, {NULL}};
const Segment SGI6210[] = {{&S42449[84], 5}, {NULL}};
const Segment SGI6211[] = {{&S42450[44], 15}, {NULL}};
const Segment SGI6212[] = {{&S42450[44], 27}, {NULL}};
const Segment SGI6213[] = {{&S42450[30], 6}, {NULL}};
const Segment SGI6214[] = {{&S42450[119], 3}, {NULL}};
const Segment SGI6215[] = {{&S42450[85], 5}, {NULL}};
const Segment SGI6216[] = {{&S42451[43], 15}, {NULL}};
const Segment SGI6217[] = {{&S42451[43], 27}, {NULL}};
const Segment SGI6218[] = {{&S42451[118], 3}, {NULL}};
const Segment SGI6219[] = {{&S42451[84], 5}, {NULL}};
const Segment SGI6220[] = {{&S42452[44], 15}, {NULL}};
const Segment SGI6221[] = {{&S42452[44], 27}, {NULL}};
const Segment SGI6222[] = {{&S42452[30], 6}, {NULL}};
const Segment SGI6223[] = {{&S42452[119], 3}, {NULL}};
const Segment SGI6224[] = {{&S42452[85], 5}, {NULL}};
const Segment SGI6225[] = {{&S42453[44], 12}, {NULL}};
const Segment SGI6226[] = {{&S42453[44], 24}, {NULL}};
const Segment SGI6227[] = {{&S42453[30], 6}, {NULL}};
const Segment SGI6228[] = {{&S42453[116], 3}, {NULL}};
const Segment SGI6229[] = {{&S42453[82], 5}, {NULL}};
const Segment SGI6230[] = {{&S42454[38], 12}, {NULL}};
const Segment SGI6231[] = {{&S42454[31], 6}, {NULL}};
const Segment SGI6232[] = {{&S42772[50], 11}, {NULL}};
const Segment SGI6233[] = {{&S42772[50], 33}, {NULL}};
const Segment SGI6234[] = {{&S42772[30], 12}, {NULL}};
const Segment SGI6235[] = {{&S42772[132], 3}, {NULL}};
const Segment SGI6236[] = {{&S42772[97], 6}, {NULL}};
const Segment SGI6237[] = {{&S42773[50], 11}, {NULL}};
const Segment SGI6238[] = {{&S42773[50], 34}, {NULL}};
const Segment SGI6239[] = {{&S42773[30], 12}, {NULL}};
const Segment SGI6240[] = {{&S42773[133], 3}, {NULL}};
const Segment SGI6241[] = {{&S42773[98], 6}, {NULL}};
const Segment SGI6242[] = {{&S42774[50], 7}, {NULL}};
const Segment SGI6243[] = {{&S42774[50], 30}, {NULL}};
const Segment SGI6244[] = {{&S42774[30], 12}, {NULL}};
const Segment SGI6245[] = {{&S42774[129], 3}, {NULL}};
const Segment SGI6246[] = {{&S42774[94], 6}, {NULL}};
const Segment SGI6247[] = {{&S42775[50], 11}, {NULL}};
const Segment SGI6248[] = {{&S42775[50], 34}, {NULL}};
const Segment SGI6249[] = {{&S42775[30], 12}, {NULL}};
const Segment SGI6250[] = {{&S42775[133], 3}, {NULL}};
const Segment SGI6251[] = {{&S42775[98], 6}, {NULL}};
const Segment SGI6252[] = {{&S42776[50], 11}, {NULL}};
const Segment SGI6253[] = {{&S42776[50], 33}, {NULL}};
const Segment SGI6254[] = {{&S42776[30], 12}, {NULL}};
const Segment SGI6255[] = {{&S42776[132], 3}, {NULL}};
const Segment SGI6256[] = {{&S42776[97], 6}, {NULL}};
const Segment SGI6257[] = {{&S42777[50], 7}, {NULL}};
const Segment SGI6258[] = {{&S42777[50], 29}, {NULL}};
const Segment SGI6259[] = {{&S42777[30], 12}, {NULL}};
const Segment SGI6260[] = {{&S42777[128], 3}, {NULL}};
const Segment SGI6261[] = {{&S42777[93], 6}, {NULL}};
const Segment SGI6262[] = {{&S42778[50], 7}, {NULL}};
const Segment SGI6263[] = {{&S42778[50], 29}, {NULL}};
const Segment SGI6264[] = {{&S42778[30], 12}, {NULL}};
const Segment SGI6265[] = {{&S42778[128], 3}, {NULL}};
const Segment SGI6266[] = {{&S42778[93], 6}, {NULL}};
const Segment SGI6267[] = {{&S42779[50], 7}, {NULL}};
const Segment SGI6268[] = {{&S42779[50], 29}, {NULL}};
const Segment SGI6269[] = {{&S42779[30], 12}, {NULL}};
const Segment SGI6270[] = {{&S42779[128], 3}, {NULL}};
const Segment SGI6271[] = {{&S42779[93], 6}, {NULL}};
const Segment SGI6272[] = {{&S42780[101], 7}, {NULL}};
const Segment SGI6273[] = {{&S42780[101], 31}, {NULL}};
const Segment SGI6274[] = {{&S42780[81], 12}, {NULL}};
const Segment SGI6275[] = {{&S42780[181], 3}, {NULL}};
const Segment SGI6276[] = {{&S42780[146], 6}, {NULL}};
const Segment SGI6277[] = {{&S42781[50], 11}, {NULL}};
const Segment SGI6278[] = {{&S42781[50], 33}, {NULL}};
const Segment SGI6279[] = {{&S42781[30], 12}, {NULL}};
const Segment SGI6280[] = {{&S42781[132], 3}, {NULL}};
const Segment SGI6281[] = {{&S42781[97], 6}, {NULL}};
const Segment SGI6282[] = {{&S42782[50], 7}, {NULL}};
const Segment SGI6283[] = {{&S42782[50], 59}, {NULL}};
const Segment SGI6284[] = {{&S42782[30], 12}, {NULL}};
const Segment SGI6285[] = {{&S42782[158], 3}, {NULL}};
const Segment SGI6286[] = {{&S42782[123], 6}, {NULL}};
const Segment SGI6287[] = {{&S42783[50], 7}, {NULL}};
const Segment SGI6288[] = {{&S42783[50], 29}, {NULL}};
const Segment SGI6289[] = {{&S42783[30], 12}, {NULL}};
const Segment SGI6290[] = {{&S42783[128], 3}, {NULL}};
const Segment SGI6291[] = {{&S42783[93], 6}, {NULL}};
const Segment SGI6292[] = {{&S42784[50], 11}, {NULL}};
const Segment SGI6293[] = {{&S42784[50], 34}, {NULL}};
const Segment SGI6294[] = {{&S42784[30], 12}, {NULL}};
const Segment SGI6295[] = {{&S42784[133], 3}, {NULL}};
const Segment SGI6296[] = {{&S42784[98], 6}, {NULL}};
const Segment SGI6297[] = {{&S42785[101], 7}, {NULL}};
const Segment SGI6298[] = {{&S42785[101], 29}, {NULL}};
const Segment SGI6299[] = {{&S42785[81], 12}, {NULL}};
const Segment SGI6300[] = {{&S42785[179], 3}, {NULL}};
const Segment SGI6301[] = {{&S42785[144], 6}, {NULL}};
const Segment SGI6302[] = {{&S42786[50], 7}, {NULL}};
const Segment SGI6303[] = {{&S42786[50], 29}, {NULL}};
const Segment SGI6304[] = {{&S42786[30], 12}, {NULL}};
const Segment SGI6305[] = {{&S42786[128], 3}, {NULL}};
const Segment SGI6306[] = {{&S42786[93], 6}, {NULL}};
const Segment SGI6307[] = {{&S42787[107], 7}, {NULL}};
const Segment SGI6308[] = {{&S42787[107], 35}, {NULL}};
const Segment SGI6309[] = {{&S42787[87], 12}, {NULL}};
const Segment SGI6310[] = {{&S42787[191], 3}, {NULL}};
const Segment SGI6311[] = {{&S42787[156], 6}, {NULL}};
const Segment SGI6312[] = {{&S42788[50], 7}, {NULL}};
const Segment SGI6313[] = {{&S42788[50], 28}, {NULL}};
const Segment SGI6314[] = {{&S42788[30], 12}, {NULL}};
const Segment SGI6315[] = {{&S42788[127], 3}, {NULL}};
const Segment SGI6316[] = {{&S42788[92], 6}, {NULL}};
const Segment SGI6317[] = {{&S42789[50], 7}, {NULL}};
const Segment SGI6318[] = {{&S42789[50], 29}, {NULL}};
const Segment SGI6319[] = {{&S42789[30], 12}, {NULL}};
const Segment SGI6320[] = {{&S42789[128], 3}, {NULL}};
const Segment SGI6321[] = {{&S42789[93], 6}, {NULL}};
const Segment SGI6322[] = {{&S42790[50], 7}, {NULL}};
const Segment SGI6323[] = {{&S42790[50], 29}, {NULL}};
const Segment SGI6324[] = {{&S42790[30], 12}, {NULL}};
const Segment SGI6325[] = {{&S42790[128], 3}, {NULL}};
const Segment SGI6326[] = {{&S42790[93], 6}, {NULL}};
const Segment SGI6327[] = {{&S42791[50], 11}, {NULL}};
const Segment SGI6328[] = {{&S42791[50], 22}, {NULL}};
const Segment SGI6329[] = {{&S42791[30], 12}, {NULL}};
const Segment SGI6330[] = {{&S42791[121], 3}, {NULL}};
const Segment SGI6331[] = {{&S42791[86], 6}, {NULL}};
const Segment SGI6332[] = {{&S42792[99], 7}, {NULL}};
const Segment SGI6333[] = {{&S42792[99], 31}, {NULL}};
const Segment SGI6334[] = {{&S42792[79], 12}, {NULL}};
const Segment SGI6335[] = {{&S42792[179], 3}, {NULL}};
const Segment SGI6336[] = {{&S42792[144], 6}, {NULL}};
const Segment SGI6337[] = {{&S42793[50], 7}, {NULL}};
const Segment SGI6338[] = {{&S42793[50], 29}, {NULL}};
const Segment SGI6339[] = {{&S42793[30], 12}, {NULL}};
const Segment SGI6340[] = {{&S42793[128], 3}, {NULL}};
const Segment SGI6341[] = {{&S42793[93], 6}, {NULL}};
const Segment SGI6342[] = {{&S42794[50], 11}, {NULL}};
const Segment SGI6343[] = {{&S42794[50], 33}, {NULL}};
const Segment SGI6344[] = {{&S42794[30], 12}, {NULL}};
const Segment SGI6345[] = {{&S42794[132], 3}, {NULL}};
const Segment SGI6346[] = {{&S42794[97], 6}, {NULL}};
const Segment SGI6347[] = {{&S42795[98], 7}, {NULL}};
const Segment SGI6348[] = {{&S42795[98], 31}, {NULL}};
const Segment SGI6349[] = {{&S42795[78], 12}, {NULL}};
const Segment SGI6350[] = {{&S42795[178], 3}, {NULL}};
const Segment SGI6351[] = {{&S42795[143], 6}, {NULL}};
const Segment SGI6352[] = {{&S42796[50], 7}, {NULL}};
const Segment SGI6353[] = {{&S42796[50], 62}, {NULL}};
const Segment SGI6354[] = {{&S42796[30], 12}, {NULL}};
const Segment SGI6355[] = {{&S42796[161], 3}, {NULL}};
const Segment SGI6356[] = {{&S42796[126], 6}, {NULL}};
const Segment SGI6357[] = {{&S42797[50], 7}, {NULL}};
const Segment SGI6358[] = {{&S42797[50], 58}, {NULL}};
const Segment SGI6359[] = {{&S42797[30], 12}, {NULL}};
const Segment SGI6360[] = {{&S42797[157], 3}, {NULL}};
const Segment SGI6361[] = {{&S42797[122], 6}, {NULL}};
const Segment SGI6362[] = {{&S42798[50], 7}, {NULL}};
const Segment SGI6363[] = {{&S42798[50], 28}, {NULL}};
const Segment SGI6364[] = {{&S42798[30], 12}, {NULL}};
const Segment SGI6365[] = {{&S42798[127], 3}, {NULL}};
const Segment SGI6366[] = {{&S42798[92], 6}, {NULL}};
const Segment SGI6367[] = {{&S42799[50], 7}, {NULL}};
const Segment SGI6368[] = {{&S42799[50], 30}, {NULL}};
const Segment SGI6369[] = {{&S42799[30], 12}, {NULL}};
const Segment SGI6370[] = {{&S42799[129], 3}, {NULL}};
const Segment SGI6371[] = {{&S42799[94], 6}, {NULL}};
const Segment SGI6372[] = {{&S42800[50], 7}, {NULL}};
const Segment SGI6373[] = {{&S42800[50], 59}, {NULL}};
const Segment SGI6374[] = {{&S42800[30], 12}, {NULL}};
const Segment SGI6375[] = {{&S42800[158], 3}, {NULL}};
const Segment SGI6376[] = {{&S42800[123], 6}, {NULL}};
const Segment SGI6377[] = {{&S42801[50], 7}, {NULL}};
const Segment SGI6378[] = {{&S42801[50], 30}, {NULL}};
const Segment SGI6379[] = {{&S42801[30], 12}, {NULL}};
const Segment SGI6380[] = {{&S42801[129], 3}, {NULL}};
const Segment SGI6381[] = {{&S42801[94], 6}, {NULL}};
const Segment SGI6382[] = {{&S42802[50], 7}, {NULL}};
const Segment SGI6383[] = {{&S42802[50], 18}, {NULL}};
const Segment SGI6384[] = {{&S42802[30], 12}, {NULL}};
const Segment SGI6385[] = {{&S42802[118], 3}, {NULL}};
const Segment SGI6386[] = {{&S42802[82], 6}, {NULL}};
const Segment SGI6387[] = {{&S42802[138], 8}, {NULL}};
const Segment SGI6388[] = {{&S42803[98], 7}, {NULL}};
const Segment SGI6389[] = {{&S42803[98], 33}, {NULL}};
const Segment SGI6390[] = {{&S42803[78], 12}, {NULL}};
const Segment SGI6391[] = {{&S42803[180], 3}, {NULL}};
const Segment SGI6392[] = {{&S42803[145], 6}, {NULL}};
const Segment SGI6393[] = {{&S45651[41], 7}, {NULL}};
const Segment SGI6394[] = {{&S45651[41], 20}, {NULL}};
const Segment SGI6395[] = {{&S45651[110], 3}, {NULL}};
const Segment SGI6396[] = {{&S45651[75], 6}, {NULL}};
const Segment SGI6397[] = {{&S45651[130], 8}, {NULL}};
const Segment SGI6398[] = {{&S45652[42], 6}, {NULL}};
const Segment SGI6399[] = {{&S45652[42], 19}, {NULL}};
const Segment SGI6400[] = {{&S45652[30], 4}, {NULL}};
const Segment SGI6401[] = {{&S45652[110], 3}, {NULL}};
const Segment SGI6402[] = {{&S45652[75], 6}, {NULL}};
const Segment SGI6403[] = {{&S45652[130], 8}, {NULL}};
const Segment SGI6404[] = {{&S45653[42], 12}, {NULL}};
const Segment SGI6405[] = {{&S45653[42], 25}, {NULL}};
const Segment SGI6406[] = {{&S45653[30], 4}, {NULL}};
const Segment SGI6407[] = {{&S45653[116], 3}, {NULL}};
const Segment SGI6408[] = {{&S45653[81], 6}, {NULL}};
const Segment SGI6409[] = {{&S45653[136], 8}, {NULL}};
const Segment SGI6410[] = {{&S45654[42], 6}, {NULL}};
const Segment SGI6411[] = {{&S45654[42], 19}, {NULL}};
const Segment SGI6412[] = {{&S45654[30], 4}, {NULL}};
const Segment SGI6413[] = {{&S45654[110], 3}, {NULL}};
const Segment SGI6414[] = {{&S45654[75], 6}, {NULL}};
const Segment SGI6415[] = {{&S45654[130], 8}, {NULL}};
const Segment SGI6416[] = {{&S45655[41], 7}, {NULL}};
const Segment SGI6417[] = {{&S45655[41], 20}, {NULL}};
const Segment SGI6418[] = {{&S45655[30], 3}, {NULL}};
const Segment SGI6419[] = {{&S45655[110], 3}, {NULL}};
const Segment SGI6420[] = {{&S45655[75], 6}, {NULL}};
const Segment SGI6421[] = {{&S45655[130], 8}, {NULL}};
const Segment SGI6422[] = {{&S45656[40], 7}, {NULL}};
const Segment SGI6423[] = {{&S45656[30], 4}, {NULL}};
const Segment SGI6424[] = {{&S45656[109], 3}, {NULL}};
const Segment SGI6425[] = {{&S45656[74], 6}, {NULL}};
const Segment SGI6426[] = {{&S45656[129], 8}, {NULL}};
const Segment SGI6427[] = {{&S45657[42], 7}, {NULL}};
const Segment SGI6428[] = {{&S45657[42], 20}, {NULL}};
const Segment SGI6429[] = {{&S45657[30], 4}, {NULL}};
const Segment SGI6430[] = {{&S45657[111], 3}, {NULL}};
const Segment SGI6431[] = {{&S45657[76], 6}, {NULL}};
const Segment SGI6432[] = {{&S45657[131], 8}, {NULL}};
const Segment SGI6433[] = {{&S45658[42], 7}, {NULL}};
const Segment SGI6434[] = {{&S45658[42], 20}, {NULL}};
const Segment SGI6435[] = {{&S45658[30], 4}, {NULL}};
const Segment SGI6436[] = {{&S45658[112], 3}, {NULL}};
const Segment SGI6437[] = {{&S45658[76], 7}, {NULL}};
const Segment SGI6438[] = {{&S45658[132], 8}, {NULL}};
const Segment SGI6439[] = {{&S45659[42], 7}, {NULL}};
const Segment SGI6440[] = {{&S45659[42], 20}, {NULL}};
const Segment SGI6441[] = {{&S45659[30], 4}, {NULL}};
const Segment SGI6442[] = {{&S45659[111], 3}, {NULL}};
const Segment SGI6443[] = {{&S45659[76], 6}, {NULL}};
const Segment SGI6444[] = {{&S42843[48], 10}, {NULL}};
const Segment SGI6445[] = {{&S42843[30], 12}, {NULL}};
const Segment SGI6446[] = {{&S42843[118], 3}, {NULL}};
const Segment SGI6447[] = {{&S42843[83], 6}, {NULL}};
const Segment SGI6448[] = {{&S42844[65], 7}, {NULL}};
const Segment SGI6449[] = {{&S42844[65], 18}, {NULL}};
const Segment SGI6450[] = {{&S42844[30], 27}, {NULL}};
const Segment SGI6451[] = {{&S42844[132], 3}, {NULL}};
const Segment SGI6452[] = {{&S42844[97], 6}, {NULL}};
const Segment SGI6453[] = {{&S42844[152], 8}, {NULL}};
const Segment SGI6454[] = {{&S42845[65], 8}, {NULL}};
const Segment SGI6455[] = {{&S42845[65], 19}, {NULL}};
const Segment SGI6456[] = {{&S42845[45], 12}, {NULL}};
const Segment SGI6457[] = {{&S42845[133], 3}, {NULL}};
const Segment SGI6458[] = {{&S42845[98], 6}, {NULL}};
const Segment SGI6459[] = {{&S42846[65], 7}, {NULL}};
const Segment SGI6460[] = {{&S42846[65], 18}, {NULL}};
const Segment SGI6461[] = {{&S42846[30], 27}, {NULL}};
const Segment SGI6462[] = {{&S42846[132], 3}, {NULL}};
const Segment SGI6463[] = {{&S42846[97], 6}, {NULL}};
const Segment SGI6464[] = {{&S42847[51], 7}, {NULL}};
const Segment SGI6465[] = {{&S42847[51], 18}, {NULL}};
const Segment SGI6466[] = {{&S42847[30], 13}, {NULL}};
const Segment SGI6467[] = {{&S42847[118], 3}, {NULL}};
const Segment SGI6468[] = {{&S42847[83], 6}, {NULL}};
const Segment SGI6469[] = {{&S42847[138], 8}, {NULL}};
const Segment SGI6470[] = {{&S42848[50], 11}, {NULL}};
const Segment SGI6471[] = {{&S42848[50], 22}, {NULL}};
const Segment SGI6472[] = {{&S42848[30], 12}, {NULL}};
const Segment SGI6473[] = {{&S42848[121], 3}, {NULL}};
const Segment SGI6474[] = {{&S42848[86], 6}, {NULL}};
const Segment SGI6475[] = {{&S42849[50], 7}, {NULL}};
const Segment SGI6476[] = {{&S42849[50], 18}, {NULL}};
const Segment SGI6477[] = {{&S42849[30], 12}, {NULL}};
const Segment SGI6478[] = {{&S42849[117], 3}, {NULL}};
const Segment SGI6479[] = {{&S42849[82], 6}, {NULL}};
const Segment SGI6480[] = {{&S42850[50], 7}, {NULL}};
const Segment SGI6481[] = {{&S42850[50], 18}, {NULL}};
const Segment SGI6482[] = {{&S42850[30], 12}, {NULL}};
const Segment SGI6483[] = {{&S42850[117], 3}, {NULL}};
const Segment SGI6484[] = {{&S42850[82], 6}, {NULL}};
const Segment SGI6485[] = {{&S42850[137], 8}, {NULL}};
const Segment SGI6486[] = {{&S42851[50], 15}, {NULL}};
const Segment SGI6487[] = {{&S42851[50], 26}, {NULL}};
const Segment SGI6488[] = {{&S42851[30], 12}, {NULL}};
const Segment SGI6489[] = {{&S45686[42], 7}, {NULL}};
const Segment SGI6490[] = {{&S45686[42], 20}, {NULL}};
const Segment SGI6491[] = {{&S42851[125], 3}, {NULL}};
const Segment SGI6492[] = {{&S45686[30], 4}, {NULL}};
const Segment SGI6493[] = {{&S42851[90], 6}, {NULL}};
const Segment SGI6494[] = {{&S45686[111], 3}, {NULL}};
const Segment SGI6495[] = {{&S45686[76], 6}, {NULL}};
const Segment SGI6496[] = {{&S45686[131], 8}, {NULL}};
const Segment SGI6497[] = {{&S42852[50], 7}, {NULL}};
const Segment SGI6498[] = {{&S42852[50], 18}, {NULL}};
const Segment SGI6499[] = {{&S42852[30], 12}, {NULL}};
const Segment SGI6500[] = {{&S45687[42], 7}, {NULL}};
const Segment SGI6501[] = {{&S45687[42], 63}, {NULL}};
const Segment SGI6502[] = {{&S45687[30], 4}, {NULL}};
const Segment SGI6503[] = {{&S42852[82], 6}, {NULL}};
const Segment SGI6504[] = {{&S42852[138], 6}, {NULL}};
const Segment SGI6505[] = {{&S45687[71], 3}, {NULL}};
const Segment SGI6506[] = {{&S45687[119], 6}, {NULL}};
const Segment SGI6507[] = {{&S45687[174], 8}, {NULL}};
const Segment SGI6508[] = {{&S42853[50], 10}, {NULL}};
const Segment SGI6509[] = {{&S42853[50], 21}, {NULL}};
const Segment SGI6510[] = {{&S42853[30], 12}, {NULL}};
const Segment SGI6511[] = {{&S45688[42], 7}, {NULL}};
const Segment SGI6512[] = {{&S45688[42], 63}, {NULL}};
const Segment SGI6513[] = {{&S42853[120], 3}, {NULL}};
const Segment SGI6514[] = {{&S45688[30], 4}, {NULL}};
const Segment SGI6515[] = {{&S42853[85], 6}, {NULL}};
const Segment SGI6516[] = {{&S45688[71], 3}, {NULL}};
const Segment SGI6517[] = {{&S45688[119], 6}, {NULL}};
const Segment SGI6518[] = {{&S45688[174], 8}, {NULL}};
const Segment SGI6519[] = {{&S42854[48], 7}, {NULL}};
const Segment SGI6520[] = {{&S42854[30], 12}, {NULL}};
const Segment SGI6521[] = {{&S42854[115], 3}, {NULL}};
const Segment SGI6522[] = {{&S45689[42], 7}, {NULL}};
const Segment SGI6523[] = {{&S42854[80], 6}, {NULL}};
const Segment SGI6524[] = {{&S45689[42], 61}, {NULL}};
const Segment SGI6525[] = {{&S45689[30], 4}, {NULL}};
const Segment SGI6526[] = {{&S45689[152], 3}, {NULL}};
const Segment SGI6527[] = {{&S45689[117], 6}, {NULL}};
const Segment SGI6528[] = {{&S45689[172], 8}, {NULL}};
const Segment SGI6529[] = {{&S42855[44], 7}, {NULL}};
const Segment SGI6530[] = {{&S42855[31], 12}, {NULL}};
const Segment SGI6531[] = {{&S45690[42], 7}, {NULL}};
const Segment SGI6532[] = {{&S45690[42], 61}, {NULL}};
const Segment SGI6533[] = {{&S45690[30], 4}, {NULL}};
const Segment SGI6534[] = {{&S42856[50], 7}, {NULL}};
const Segment SGI6535[] = {{&S45690[152], 3}, {NULL}};
const Segment SGI6536[] = {{&S42856[50], 18}, {NULL}};
const Segment SGI6537[] = {{&S42856[30], 12}, {NULL}};
const Segment SGI6538[] = {{&S45690[117], 6}, {NULL}};
const Segment SGI6539[] = {{&S45690[172], 8}, {NULL}};
const Segment SGI6540[] = {{&S42856[85], 4}, {NULL}};
const Segment SGI6541[] = {{&S42856[85], 3}, {NULL}};
const Segment SGI6542[] = {{&S45691[42], 7}, {NULL}};
const Segment SGI6543[] = {{&S45691[42], 59}, {NULL}};
const Segment SGI6544[] = {{&S45691[30], 4}, {NULL}};
const Segment SGI6545[] = {{&S42857[50], 7}, {NULL}};
const Segment SGI6546[] = {{&S42857[50], 18}, {NULL}};
const Segment SGI6547[] = {{&S42857[30], 12}, {NULL}};
const Segment SGI6548[] = {{&S45691[150], 3}, {NULL}};
const Segment SGI6549[] = {{&S45691[115], 6}, {NULL}};
const Segment SGI6550[] = {{&S42857[82], 6}, {NULL}};
const Segment SGI6551[] = {{&S45691[170], 8}, {NULL}};
const Segment SGI6552[] = {{&S42857[137], 8}, {NULL}};
const Segment SGI6553[] = {{&S45692[42], 7}, {NULL}};
const Segment SGI6554[] = {{&S45692[42], 59}, {NULL}};
const Segment SGI6555[] = {{&S45692[30], 4}, {NULL}};
const Segment SGI6556[] = {{&S45692[150], 3}, {NULL}};
const Segment SGI6557[] = {{&S45692[115], 6}, {NULL}};
const Segment SGI6558[] = {{&S45692[170], 8}, {NULL}};
const Segment SGI6559[] = {{&S45693[42], 7}, {NULL}};
const Segment SGI6560[] = {{&S45693[42], 61}, {NULL}};
const Segment SGI6561[] = {{&S45693[30], 4}, {NULL}};
const Segment SGI6562[] = {{&S45693[152], 3}, {NULL}};
const Segment SGI6563[] = {{&S45693[117], 6}, {NULL}};
const Segment SGI6564[] = {{&S45693[172], 8}, {NULL}};
const Segment SGI6565[] = {{&S45694[42], 7}, {NULL}};
const Segment SGI6566[] = {{&S45694[42], 20}, {NULL}};
const Segment SGI6567[] = {{&S45694[30], 4}, {NULL}};
const Segment SGI6568[] = {{&S45694[111], 3}, {NULL}};
const Segment SGI6569[] = {{&S45694[76], 6}, {NULL}};
const Segment SGI6570[] = {{&S45694[131], 8}, {NULL}};
const Segment SGI6571[] = {{&S42874[50], 6}, {NULL}};
const Segment SGI6572[] = {{&S42874[50], 24}, {NULL}};
const Segment SGI6573[] = {{&S42874[30], 12}, {NULL}};
const Segment SGI6574[] = {{&S42874[123], 3}, {NULL}};
const Segment SGI6575[] = {{&S42874[88], 6}, {NULL}};
const Segment SGI6576[] = {{&S42875[45], 4}, {NULL}};
const Segment SGI6577[] = {{&S42875[94], 3}, {NULL}};
const Segment SGI6578[] = {{&S42875[71], 6}, {NULL}};
const Segment SGI6579[] = {{&S42877[50], 6}, {NULL}};
const Segment SGI6580[] = {{&S42877[50], 24}, {NULL}};
const Segment SGI6581[] = {{&S42877[30], 12}, {NULL}};
const Segment SGI6582[] = {{&S42877[123], 3}, {NULL}};
const Segment SGI6583[] = {{&S42877[88], 6}, {NULL}};
const Segment SGI6584[] = {{&S42878[50], 6}, {NULL}};
const Segment SGI6585[] = {{&S42878[50], 24}, {NULL}};
const Segment SGI6586[] = {{&S42878[30], 12}, {NULL}};
const Segment SGI6587[] = {{&S42878[124], 3}, {NULL}};
const Segment SGI6588[] = {{&S42878[88], 7}, {NULL}};
const Segment SGI6589[] = {{&S42878[144], 8}, {NULL}};
const Segment SGI6590[] = {{&S42879[44], 6}, {NULL}};
const Segment SGI6591[] = {{&S42879[31], 12}, {NULL}};
const Segment SGI6592[] = {{&S42880[50], 6}, {NULL}};
const Segment SGI6593[] = {{&S42880[50], 24}, {NULL}};
const Segment SGI6594[] = {{&S42880[30], 12}, {NULL}};
const Segment SGI6595[] = {{&S42880[91], 4}, {NULL}};
const Segment SGI6596[] = {{&S42880[91], 3}, {NULL}};
const Segment SGI6597[] = {{&S42881[50], 6}, {NULL}};
const Segment SGI6598[] = {{&S42881[50], 24}, {NULL}};
const Segment SGI6599[] = {{&S42881[30], 12}, {NULL}};
const Segment SGI6600[] = {{&S42881[91], 4}, {NULL}};
const Segment SGI6601[] = {{&S42881[91], 3}, {NULL}};
const Segment SGI6602[] = {{&S43254[44], 7}, {NULL}};
const Segment SGI6603[] = {{&S43254[44], 22}, {NULL}};
const Segment SGI6604[] = {{&S43254[30], 6}, {NULL}};
const Segment SGI6605[] = {{&S43254[114], 3}, {NULL}};
const Segment SGI6606[] = {{&S43254[80], 5}, {NULL}};
const Segment SGI6607[] = {{&S43376[42], 11}, {NULL}};
const Segment SGI6608[] = {{&S43376[42], 22}, {NULL}};
const Segment SGI6609[] = {{&S43376[30], 4}, {NULL}};
const Segment SGI6610[] = {{&S43376[112], 3}, {NULL}};
const Segment SGI6611[] = {{&S43376[78], 5}, {NULL}};
const Segment SGI6612[] = {{&S43377[42], 11}, {NULL}};
const Segment SGI6613[] = {{&S43377[42], 22}, {NULL}};
const Segment SGI6614[] = {{&S43377[30], 4}, {NULL}};
const Segment SGI6615[] = {{&S43377[112], 3}, {NULL}};
const Segment SGI6616[] = {{&S43377[78], 5}, {NULL}};
const Segment SGI6617[] = {{&S43378[42], 10}, {NULL}};
const Segment SGI6618[] = {{&S43378[42], 21}, {NULL}};
const Segment SGI6619[] = {{&S43378[30], 4}, {NULL}};
const Segment SGI6620[] = {{&S43378[111], 3}, {NULL}};
const Segment SGI6621[] = {{&S43378[77], 5}, {NULL}};
const Segment SGI6622[] = {{&S43379[40], 11}, {NULL}};
const Segment SGI6623[] = {{&S43379[30], 4}, {NULL}};
const Segment SGI6624[] = {{&S43379[110], 3}, {NULL}};
const Segment SGI6625[] = {{&S43379[76], 5}, {NULL}};
const Segment SGI6626[] = {{&S43382[42], 10}, {NULL}};
const Segment SGI6627[] = {{&S43382[42], 21}, {NULL}};
const Segment SGI6628[] = {{&S43382[30], 4}, {NULL}};
const Segment SGI6629[] = {{&S43382[111], 3}, {NULL}};
const Segment SGI6630[] = {{&S43382[77], 5}, {NULL}};
const Segment SGI6631[] = {{&S43383[36], 10}, {NULL}};
const Segment SGI6632[] = {{&S43383[31], 4}, {NULL}};
const Segment SGI6633[] = {{&S43384[42], 10}, {NULL}};
const Segment SGI6634[] = {{&S43384[42], 21}, {NULL}};
const Segment SGI6635[] = {{&S43384[30], 4}, {NULL}};
const Segment SGI6636[] = {{&S43384[80], 4}, {NULL}};
const Segment SGI6637[] = {{&S43384[80], 3}, {NULL}};
const Segment SGI6638[] = {{&S43386[40], 10}, {NULL}};
const Segment SGI6639[] = {{&S43386[30], 4}, {NULL}};
const Segment SGI6640[] = {{&S43386[109], 3}, {NULL}};
const Segment SGI6641[] = {{&S43386[75], 5}, {NULL}};
const Segment SGI6642[] = {{&S43380[44], 11}, {NULL}};
const Segment SGI6643[] = {{&S43380[44], 22}, {NULL}};
const Segment SGI6644[] = {{&S43380[30], 6}, {NULL}};
const Segment SGI6645[] = {{&S43380[114], 3}, {NULL}};
const Segment SGI6646[] = {{&S43380[80], 5}, {NULL}};
const Segment SGI6647[] = {{&S43381[44], 11}, {NULL}};
const Segment SGI6648[] = {{&S43381[44], 22}, {NULL}};
const Segment SGI6649[] = {{&S43381[30], 6}, {NULL}};
const Segment SGI6650[] = {{&S43381[114], 3}, {NULL}};
const Segment SGI6651[] = {{&S43381[80], 5}, {NULL}};
const Segment SGI6652[] = {{&S43385[38], 11}, {NULL}};
const Segment SGI6653[] = {{&S43385[31], 6}, {NULL}};
const Segment SGI6654[] = {{&S45005[50], 10}, {NULL}};
const Segment SGI6655[] = {{&S45005[50], 21}, {NULL}};
const Segment SGI6656[] = {{&S45005[30], 12}, {NULL}};
const Segment SGI6657[] = {{&S45005[119], 3}, {NULL}};
const Segment SGI6658[] = {{&S45005[85], 5}, {NULL}};
const Segment SGI6659[] = {{&S43593[42], 15}, {NULL}};
const Segment SGI6660[] = {{&S43593[42], 26}, {NULL}};
const Segment SGI6661[] = {{&S43593[30], 4}, {NULL}};
const Segment SGI6662[] = {{&S43593[116], 3}, {NULL}};
const Segment SGI6663[] = {{&S43593[82], 5}, {NULL}};
const Segment SGI6664[] = {{&S46263[11], 1}, {NULL}};
const Segment SGI6665[] = {{&S46263[41], 9}, {NULL}};
const Segment SGI6666[] = {{&S46263[112], 3}, {NULL}};
const Segment SGI6667[] = {{&S46263[77], 6}, {NULL}};
const Segment SGI6668[] = {{&S46263[132], 8}, {NULL}};
const Segment SGI6669[] = {{&S46264[42], 7}, {NULL}};
const Segment SGI6670[] = {{&S46264[42], 18}, {NULL}};
const Segment SGI6671[] = {{&S46264[30], 4}, {NULL}};
const Segment SGI6672[] = {{&S46264[109], 3}, {NULL}};
const Segment SGI6673[] = {{&S46264[74], 6}, {NULL}};
const Segment SGI6674[] = {{&S46264[129], 8}, {NULL}};
const Segment SGI6675[] = {{&S46265[11], 1}, {NULL}};
const Segment SGI6676[] = {{&S46265[42], 14}, {NULL}};
const Segment SGI6677[] = {{&S46265[30], 4}, {NULL}};
const Segment SGI6678[] = {{&S46265[118], 3}, {NULL}};
const Segment SGI6679[] = {{&S46265[83], 6}, {NULL}};
const Segment SGI6680[] = {{&S46265[138], 8}, {NULL}};
const Segment SGI6681[] = {{&S46266[11], 1}, {NULL}};
const Segment SGI6682[] = {{&S46266[42], 10}, {NULL}};
const Segment SGI6683[] = {{&S46266[30], 4}, {NULL}};
const Segment SGI6684[] = {{&S46266[114], 3}, {NULL}};
const Segment SGI6685[] = {{&S46266[79], 6}, {NULL}};
const Segment SGI6686[] = {{&S46266[134], 8}, {NULL}};
const Segment SGI6687[] = {{&S46267[14], 1}, {NULL}};
const Segment SGI6688[] = {{&S46267[57], 7}, {NULL}};
const Segment SGI6689[] = {{&S46267[45], 4}, {NULL}};
const Segment SGI6690[] = {{&S46267[126], 3}, {NULL}};
const Segment SGI6691[] = {{&S46267[91], 6}, {NULL}};
const Segment SGI6692[] = {{&S46267[146], 8}, {NULL}};
const Segment SGI6693[] = {{&S46268[11], 1}, {NULL}};
const Segment SGI6694[] = {{&S46268[41], 9}, {NULL}};
const Segment SGI6695[] = {{&S46268[30], 3}, {NULL}};
const Segment SGI6696[] = {{&S46268[112], 3}, {NULL}};
const Segment SGI6697[] = {{&S46268[77], 6}, {NULL}};
const Segment SGI6698[] = {{&S46268[132], 8}, {NULL}};
const Segment SGI6699[] = {{&S46269[8], 1}, {NULL}};
const Segment SGI6700[] = {{&S46269[51], 6}, {NULL}};
const Segment SGI6701[] = {{&S46269[39], 4}, {NULL}};
const Segment SGI6702[] = {{&S46269[119], 3}, {NULL}};
const Segment SGI6703[] = {{&S46269[84], 6}, {NULL}};
const Segment SGI6704[] = {{&S46269[139], 8}, {NULL}};
const Segment SGI6705[] = {{&S46270[11], 1}, {NULL}};
const Segment SGI6706[] = {{&S46270[42], 10}, {NULL}};
const Segment SGI6707[] = {{&S46270[30], 4}, {NULL}};
const Segment SGI6708[] = {{&S46270[114], 3}, {NULL}};
const Segment SGI6709[] = {{&S46270[79], 6}, {NULL}};
const Segment SGI6710[] = {{&S46270[134], 8}, {NULL}};
const Segment SGI6711[] = {{&S46271[11], 1}, {NULL}};
const Segment SGI6712[] = {{&S46271[42], 9}, {NULL}};
const Segment SGI6713[] = {{&S46271[30], 4}, {NULL}};
const Segment SGI6714[] = {{&S46271[113], 3}, {NULL}};
const Segment SGI6715[] = {{&S46271[78], 6}, {NULL}};
const Segment SGI6716[] = {{&S46271[133], 8}, {NULL}};
const Segment SGI6717[] = {{&S46272[11], 1}, {NULL}};
const Segment SGI6718[] = {{&S46272[42], 10}, {NULL}};
const Segment SGI6719[] = {{&S46272[30], 4}, {NULL}};
const Segment SGI6720[] = {{&S46272[114], 3}, {NULL}};
const Segment SGI6721[] = {{&S46272[79], 6}, {NULL}};
const Segment SGI6722[] = {{&S46272[134], 8}, {NULL}};
const Segment SGI6723[] = {{&S43586[44], 13}, {NULL}};
const Segment SGI6724[] = {{&S43586[44], 24}, {NULL}};
const Segment SGI6725[] = {{&S43586[30], 6}, {NULL}};
const Segment SGI6726[] = {{&S43586[116], 3}, {NULL}};
const Segment SGI6727[] = {{&S43586[82], 5}, {NULL}};
const Segment SGI6728[] = {{&S43589[44], 6}, {NULL}};
const Segment SGI6729[] = {{&S43589[44], 17}, {NULL}};
const Segment SGI6730[] = {{&S43589[30], 6}, {NULL}};
const Segment SGI6731[] = {{&S43589[109], 3}, {NULL}};
const Segment SGI6732[] = {{&S43589[75], 5}, {NULL}};
const Segment SGI6733[] = {{&S43592[38], 13}, {NULL}};
const Segment SGI6734[] = {{&S43592[31], 6}, {NULL}};
const Segment SGI6735[] = {{&S43811[44], 14}, {NULL}};
const Segment SGI6736[] = {{&S43811[44], 25}, {NULL}};
const Segment SGI6737[] = {{&S43811[30], 6}, {NULL}};
const Segment SGI6738[] = {{&S43811[117], 3}, {NULL}};
const Segment SGI6739[] = {{&S43811[83], 5}, {NULL}};
const Segment SGI6740[] = {{&S43812[44], 27}, {NULL}};
const Segment SGI6741[] = {{&S43812[44], 38}, {NULL}};
const Segment SGI6742[] = {{&S43812[30], 6}, {NULL}};
const Segment SGI6743[] = {{&S43812[130], 3}, {NULL}};
const Segment SGI6744[] = {{&S43812[96], 5}, {NULL}};
const Segment SGI6745[] = {{&S43813[38], 14}, {NULL}};
const Segment SGI6746[] = {{&S43813[31], 6}, {NULL}};
const Segment SGI6747[] = {{&S43763[50], 7}, {NULL}};
const Segment SGI6748[] = {{&S43763[50], 29}, {NULL}};
const Segment SGI6749[] = {{&S43763[30], 12}, {NULL}};
const Segment SGI6750[] = {{&S43763[128], 3}, {NULL}};
const Segment SGI6751[] = {{&S43763[93], 6}, {NULL}};
const Segment SGI6752[] = {{&S43764[49], 7}, {NULL}};
const Segment SGI6753[] = {{&S43764[49], 29}, {NULL}};
const Segment SGI6754[] = {{&S43764[127], 3}, {NULL}};
const Segment SGI6755[] = {{&S43764[92], 6}, {NULL}};
const Segment SGI6756[] = {{&S43765[50], 6}, {NULL}};
const Segment SGI6757[] = {{&S43765[50], 28}, {NULL}};
const Segment SGI6758[] = {{&S43765[30], 12}, {NULL}};
const Segment SGI6759[] = {{&S43765[127], 3}, {NULL}};
const Segment SGI6760[] = {{&S43765[92], 6}, {NULL}};
const Segment SGI6761[] = {{&S43766[50], 6}, {NULL}};
const Segment SGI6762[] = {{&S43766[50], 28}, {NULL}};
const Segment SGI6763[] = {{&S43766[30], 12}, {NULL}};
const Segment SGI6764[] = {{&S43766[127], 3}, {NULL}};
const Segment SGI6765[] = {{&S43766[92], 6}, {NULL}};
const Segment SGI6766[] = {{&S43767[49], 7}, {NULL}};
const Segment SGI6767[] = {{&S43767[49], 29}, {NULL}};
const Segment SGI6768[] = {{&S43767[127], 3}, {NULL}};
const Segment SGI6769[] = {{&S43767[92], 6}, {NULL}};
const Segment SGI6770[] = {{&S43768[49], 7}, {NULL}};
const Segment SGI6771[] = {{&S43768[49], 29}, {NULL}};
const Segment SGI6772[] = {{&S43768[30], 11}, {NULL}};
const Segment SGI6773[] = {{&S43768[127], 3}, {NULL}};
const Segment SGI6774[] = {{&S43768[92], 6}, {NULL}};
const Segment SGI6775[] = {{&S43769[50], 7}, {NULL}};
const Segment SGI6776[] = {{&S43769[50], 22}, {NULL}};
const Segment SGI6777[] = {{&S43769[30], 12}, {NULL}};
const Segment SGI6778[] = {{&S43769[121], 3}, {NULL}};
const Segment SGI6779[] = {{&S43769[86], 6}, {NULL}};
const Segment SGI6780[] = {{&S43770[50], 7}, {NULL}};
const Segment SGI6781[] = {{&S43770[50], 29}, {NULL}};
const Segment SGI6782[] = {{&S43770[30], 12}, {NULL}};
const Segment SGI6783[] = {{&S43770[128], 3}, {NULL}};
const Segment SGI6784[] = {{&S43770[93], 6}, {NULL}};
const Segment SGI6785[] = {{&S43771[50], 7}, {NULL}};
const Segment SGI6786[] = {{&S43771[50], 29}, {NULL}};
const Segment SGI6787[] = {{&S43771[30], 12}, {NULL}};
const Segment SGI6788[] = {{&S43771[128], 3}, {NULL}};
const Segment SGI6789[] = {{&S43771[93], 6}, {NULL}};
const Segment SGI6790[] = {{&S43772[50], 7}, {NULL}};
const Segment SGI6791[] = {{&S43772[50], 22}, {NULL}};
const Segment SGI6792[] = {{&S43772[30], 12}, {NULL}};
const Segment SGI6793[] = {{&S43772[121], 3}, {NULL}};
const Segment SGI6794[] = {{&S43772[86], 6}, {NULL}};
const Segment SGI6795[] = {{&S43773[50], 7}, {NULL}};
const Segment SGI6796[] = {{&S43773[50], 29}, {NULL}};
const Segment SGI6797[] = {{&S43773[30], 12}, {NULL}};
const Segment SGI6798[] = {{&S43773[128], 3}, {NULL}};
const Segment SGI6799[] = {{&S43773[93], 6}, {NULL}};
const Segment SGI6800[] = {{&S43774[50], 7}, {NULL}};
const Segment SGI6801[] = {{&S43774[50], 29}, {NULL}};
const Segment SGI6802[] = {{&S43774[30], 12}, {NULL}};
const Segment SGI6803[] = {{&S43774[128], 3}, {NULL}};
const Segment SGI6804[] = {{&S43774[93], 6}, {NULL}};
const Segment SGI6805[] = {{&S43775[49], 7}, {NULL}};
const Segment SGI6806[] = {{&S43775[49], 29}, {NULL}};
const Segment SGI6807[] = {{&S43775[30], 11}, {NULL}};
const Segment SGI6808[] = {{&S43775[127], 3}, {NULL}};
const Segment SGI6809[] = {{&S43775[92], 6}, {NULL}};
const Segment SGI6810[] = {{&S43776[50], 7}, {NULL}};
const Segment SGI6811[] = {{&S43776[50], 29}, {NULL}};
const Segment SGI6812[] = {{&S43776[30], 12}, {NULL}};
const Segment SGI6813[] = {{&S43776[128], 3}, {NULL}};
const Segment SGI6814[] = {{&S43776[93], 6}, {NULL}};
const Segment SGI6815[] = {{&S43777[99], 7}, {NULL}};
const Segment SGI6816[] = {{&S43777[99], 29}, {NULL}};
const Segment SGI6817[] = {{&S43777[79], 12}, {NULL}};
const Segment SGI6818[] = {{&S43777[177], 3}, {NULL}};
const Segment SGI6819[] = {{&S43777[142], 6}, {NULL}};
const Segment SGI6820[] = {{&S43778[50], 7}, {NULL}};
const Segment SGI6821[] = {{&S43778[50], 29}, {NULL}};
const Segment SGI6822[] = {{&S43778[30], 12}, {NULL}};
const Segment SGI6823[] = {{&S43778[128], 3}, {NULL}};
const Segment SGI6824[] = {{&S43778[93], 6}, {NULL}};
const Segment SGI6825[] = {{&S43779[57], 7}, {NULL}};
const Segment SGI6826[] = {{&S43779[57], 29}, {NULL}};
const Segment SGI6827[] = {{&S43779[30], 19}, {NULL}};
const Segment SGI6828[] = {{&S43779[135], 3}, {NULL}};
const Segment SGI6829[] = {{&S43779[100], 6}, {NULL}};
const Segment SGI6830[] = {{&S43780[107], 7}, {NULL}};
const Segment SGI6831[] = {{&S43780[107], 35}, {NULL}};
const Segment SGI6832[] = {{&S43780[87], 12}, {NULL}};
const Segment SGI6833[] = {{&S43780[191], 3}, {NULL}};
const Segment SGI6834[] = {{&S43780[156], 6}, {NULL}};
const Segment SGI6835[] = {{&S43781[50], 6}, {NULL}};
const Segment SGI6836[] = {{&S43781[50], 28}, {NULL}};
const Segment SGI6837[] = {{&S43781[30], 12}, {NULL}};
const Segment SGI6838[] = {{&S43781[127], 3}, {NULL}};
const Segment SGI6839[] = {{&S43781[92], 6}, {NULL}};
const Segment SGI6840[] = {{&S43782[50], 6}, {NULL}};
const Segment SGI6841[] = {{&S43782[50], 28}, {NULL}};
const Segment SGI6842[] = {{&S43782[30], 12}, {NULL}};
const Segment SGI6843[] = {{&S43782[127], 3}, {NULL}};
const Segment SGI6844[] = {{&S43782[92], 6}, {NULL}};
const Segment SGI6845[] = {{&S43783[48], 7}, {NULL}};
const Segment SGI6846[] = {{&S43783[30], 12}, {NULL}};
const Segment SGI6847[] = {{&S43783[126], 3}, {NULL}};
const Segment SGI6848[] = {{&S43783[91], 6}, {NULL}};
const Segment SGI6849[] = {{&S43784[103], 7}, {NULL}};
const Segment SGI6850[] = {{&S43784[103], 31}, {NULL}};
const Segment SGI6851[] = {{&S43784[83], 12}, {NULL}};
const Segment SGI6852[] = {{&S43784[183], 3}, {NULL}};
const Segment SGI6853[] = {{&S43784[148], 6}, {NULL}};
const Segment SGI6854[] = {{&S43785[99], 7}, {NULL}};
const Segment SGI6855[] = {{&S43785[99], 35}, {NULL}};
const Segment SGI6856[] = {{&S43785[79], 12}, {NULL}};
const Segment SGI6857[] = {{&S43785[183], 3}, {NULL}};
const Segment SGI6858[] = {{&S43785[148], 6}, {NULL}};
const Segment SGI6859[] = {{&S43786[105], 7}, {NULL}};
const Segment SGI6860[] = {{&S43786[105], 33}, {NULL}};
const Segment SGI6861[] = {{&S43786[85], 12}, {NULL}};
const Segment SGI6862[] = {{&S43786[187], 3}, {NULL}};
const Segment SGI6863[] = {{&S43786[152], 6}, {NULL}};
const Segment SGI6864[] = {{&S43787[101], 7}, {NULL}};
const Segment SGI6865[] = {{&S43787[101], 31}, {NULL}};
const Segment SGI6866[] = {{&S43787[81], 12}, {NULL}};
const Segment SGI6867[] = {{&S43787[181], 3}, {NULL}};
const Segment SGI6868[] = {{&S43787[146], 6}, {NULL}};
const Segment SGI6869[] = {{&S43788[103], 7}, {NULL}};
const Segment SGI6870[] = {{&S43788[103], 35}, {NULL}};
const Segment SGI6871[] = {{&S43788[83], 12}, {NULL}};
const Segment SGI6872[] = {{&S43788[187], 3}, {NULL}};
const Segment SGI6873[] = {{&S43788[152], 6}, {NULL}};
const Segment SGI6874[] = {{&S43789[50], 7}, {NULL}};
const Segment SGI6875[] = {{&S43789[50], 29}, {NULL}};
const Segment SGI6876[] = {{&S43789[30], 12}, {NULL}};
const Segment SGI6877[] = {{&S43789[128], 3}, {NULL}};
const Segment SGI6878[] = {{&S43789[93], 6}, {NULL}};
const Segment SGI6879[] = {{&S43790[44], 7}, {NULL}};
const Segment SGI6880[] = {{&S43790[31], 12}, {NULL}};
const Segment SGI6881[] = {{&S43791[50], 7}, {NULL}};
const Segment SGI6882[] = {{&S43791[50], 29}, {NULL}};
const Segment SGI6883[] = {{&S43791[30], 12}, {NULL}};
const Segment SGI6884[] = {{&S43791[96], 4}, {NULL}};
const Segment SGI6885[] = {{&S43791[96], 3}, {NULL}};
const Segment SGI6886[] = {{&S44080[42], 7}, {NULL}};
const Segment SGI6887[] = {{&S44080[42], 18}, {NULL}};
const Segment SGI6888[] = {{&S44080[30], 4}, {NULL}};
const Segment SGI6889[] = {{&S44080[108], 3}, {NULL}};
const Segment SGI6890[] = {{&S44080[74], 5}, {NULL}};
const Segment SGI6891[] = {{&S44082[42], 7}, {NULL}};
const Segment SGI6892[] = {{&S44082[42], 20}, {NULL}};
const Segment SGI6893[] = {{&S44082[30], 4}, {NULL}};
const Segment SGI6894[] = {{&S44082[110], 3}, {NULL}};
const Segment SGI6895[] = {{&S44082[76], 5}, {NULL}};
const Segment SGI6896[] = {{&S44074[44], 7}, {NULL}};
const Segment SGI6897[] = {{&S44074[44], 58}, {NULL}};
const Segment SGI6898[] = {{&S44074[30], 6}, {NULL}};
const Segment SGI6899[] = {{&S44074[73], 3}, {NULL}};
const Segment SGI6900[] = {{&S44074[116], 5}, {NULL}};
const Segment SGI6901[] = {{&S44075[44], 7}, {NULL}};
const Segment SGI6902[] = {{&S44075[44], 70}, {NULL}};
const Segment SGI6903[] = {{&S44075[30], 6}, {NULL}};
const Segment SGI6904[] = {{&S44075[162], 3}, {NULL}};
const Segment SGI6905[] = {{&S44075[128], 5}, {NULL}};
const Segment SGI6906[] = {{&S44076[44], 7}, {NULL}};
const Segment SGI6907[] = {{&S44076[44], 59}, {NULL}};
const Segment SGI6908[] = {{&S44076[30], 6}, {NULL}};
const Segment SGI6909[] = {{&S44076[151], 3}, {NULL}};
const Segment SGI6910[] = {{&S44076[117], 5}, {NULL}};
const Segment SGI6911[] = {{&S44077[44], 7}, {NULL}};
const Segment SGI6912[] = {{&S44077[44], 32}, {NULL}};
const Segment SGI6913[] = {{&S44077[30], 6}, {NULL}};
const Segment SGI6914[] = {{&S44077[124], 3}, {NULL}};
const Segment SGI6915[] = {{&S44077[90], 5}, {NULL}};
const Segment SGI6916[] = {{&S44078[44], 7}, {NULL}};
const Segment SGI6917[] = {{&S44078[44], 60}, {NULL}};
const Segment SGI6918[] = {{&S44078[30], 6}, {NULL}};
const Segment SGI6919[] = {{&S44078[152], 3}, {NULL}};
const Segment SGI6920[] = {{&S44078[118], 5}, {NULL}};
const Segment SGI6921[] = {{&S44081[44], 7}, {NULL}};
const Segment SGI6922[] = {{&S44081[44], 59}, {NULL}};
const Segment SGI6923[] = {{&S44081[30], 6}, {NULL}};
const Segment SGI6924[] = {{&S44081[151], 3}, {NULL}};
const Segment SGI6925[] = {{&S44081[117], 5}, {NULL}};
const Segment SGI6926[] = {{&S44083[44], 7}, {NULL}};
const Segment SGI6927[] = {{&S44083[44], 59}, {NULL}};
const Segment SGI6928[] = {{&S44083[30], 6}, {NULL}};
const Segment SGI6929[] = {{&S44083[151], 3}, {NULL}};
const Segment SGI6930[] = {{&S44083[117], 5}, {NULL}};
const Segment SGI6931[] = {{&S44086[38], 7}, {NULL}};
const Segment SGI6932[] = {{&S44086[31], 6}, {NULL}};
const Segment SGI6933[] = {{&S44010[50], 7}, {NULL}};
const Segment SGI6934[] = {{&S44010[50], 16}, {NULL}};
const Segment SGI6935[] = {{&S44010[30], 12}, {NULL}};
const Segment SGI6936[] = {{&S44010[115], 3}, {NULL}};
const Segment SGI6937[] = {{&S44010[80], 6}, {NULL}};
const Segment SGI6938[] = {{&S44011[44], 7}, {NULL}};
const Segment SGI6939[] = {{&S44011[31], 12}, {NULL}};
const Segment SGI6940[] = {{&S44012[50], 7}, {NULL}};
const Segment SGI6941[] = {{&S44012[50], 16}, {NULL}};
const Segment SGI6942[] = {{&S44012[30], 12}, {NULL}};
const Segment SGI6943[] = {{&S44012[115], 3}, {NULL}};
const Segment SGI6944[] = {{&S44012[80], 6}, {NULL}};
const Segment SGI6945[] = {{&S44013[50], 7}, {NULL}};
const Segment SGI6946[] = {{&S44013[50], 16}, {NULL}};
const Segment SGI6947[] = {{&S44013[30], 12}, {NULL}};
const Segment SGI6948[] = {{&S44013[115], 3}, {NULL}};
const Segment SGI6949[] = {{&S44013[80], 6}, {NULL}};
const Segment SGI6950[] = {{&S44013[133], 9}, {NULL}};
const Segment SGI6951[] = {{&S44352[42], 8}, {NULL}};
const Segment SGI6952[] = {{&S44352[42], 19}, {NULL}};
const Segment SGI6953[] = {{&S44352[30], 4}, {NULL}};
const Segment SGI6954[] = {{&S44352[109], 3}, {NULL}};
const Segment SGI6955[] = {{&S44352[75], 5}, {NULL}};
const Segment SGI6956[] = {{&S44353[42], 8}, {NULL}};
const Segment SGI6957[] = {{&S44353[42], 19}, {NULL}};
const Segment SGI6958[] = {{&S44353[30], 4}, {NULL}};
const Segment SGI6959[] = {{&S44353[109], 3}, {NULL}};
const Segment SGI6960[] = {{&S44353[75], 5}, {NULL}};
const Segment SGI6961[] = {{&S44354[41], 9}, {NULL}};
const Segment SGI6962[] = {{&S44354[41], 20}, {NULL}};
const Segment SGI6963[] = {{&S44354[109], 3}, {NULL}};
const Segment SGI6964[] = {{&S44354[75], 5}, {NULL}};
const Segment SGI6965[] = {{&S44355[41], 9}, {NULL}};
const Segment SGI6966[] = {{&S44355[41], 20}, {NULL}};
const Segment SGI6967[] = {{&S44355[30], 3}, {NULL}};
const Segment SGI6968[] = {{&S44355[109], 3}, {NULL}};
const Segment SGI6969[] = {{&S44355[75], 5}, {NULL}};
const Segment SGI6970[] = {{&S44356[42], 9}, {NULL}};
const Segment SGI6971[] = {{&S44356[42], 20}, {NULL}};
const Segment SGI6972[] = {{&S44356[30], 4}, {NULL}};
const Segment SGI6973[] = {{&S44356[110], 3}, {NULL}};
const Segment SGI6974[] = {{&S44356[76], 5}, {NULL}};
const Segment SGI6975[] = {{&S44357[42], 9}, {NULL}};
const Segment SGI6976[] = {{&S44357[42], 20}, {NULL}};
const Segment SGI6977[] = {{&S44357[30], 4}, {NULL}};
const Segment SGI6978[] = {{&S44357[110], 3}, {NULL}};
const Segment SGI6979[] = {{&S44357[76], 5}, {NULL}};
const Segment SGI6980[] = {{&S44360[42], 9}, {NULL}};
const Segment SGI6981[] = {{&S44360[42], 20}, {NULL}};
const Segment SGI6982[] = {{&S44360[30], 4}, {NULL}};
const Segment SGI6983[] = {{&S44360[112], 3}, {NULL}};
const Segment SGI6984[] = {{&S44360[76], 7}, {NULL}};
const Segment SGI6985[] = {{&S44360[132], 8}, {NULL}};
const Segment SGI6986[] = {{&S44361[36], 9}, {NULL}};
const Segment SGI6987[] = {{&S44361[31], 4}, {NULL}};
const Segment SGI6988[] = {{&S44362[42], 9}, {NULL}};
const Segment SGI6989[] = {{&S44362[42], 20}, {NULL}};
const Segment SGI6990[] = {{&S44362[30], 4}, {NULL}};
const Segment SGI6991[] = {{&S44362[79], 4}, {NULL}};
const Segment SGI6992[] = {{&S44362[79], 3}, {NULL}};
const Segment SGI6993[] = {{&S44358[44], 9}, {NULL}};
const Segment SGI6994[] = {{&S44358[44], 20}, {NULL}};
const Segment SGI6995[] = {{&S44358[30], 6}, {NULL}};
const Segment SGI6996[] = {{&S44358[112], 3}, {NULL}};
const Segment SGI6997[] = {{&S44358[78], 5}, {NULL}};
const Segment SGI6998[] = {{&S44359[44], 9}, {NULL}};
const Segment SGI6999[] = {{&S44359[44], 20}, {NULL}};
const Segment SGI7000[] = {{&S44359[30], 6}, {NULL}};
const Segment SGI7001[] = {{&S44359[112], 3}, {NULL}};
const Segment SGI7002[] = {{&S44359[78], 5}, {NULL}};
const Segment SGI7003[] = {{&S44533[42], 11}, {NULL}};
const Segment SGI7004[] = {{&S44533[42], 32}, {NULL}};
const Segment SGI7005[] = {{&S44533[30], 4}, {NULL}};
const Segment SGI7006[] = {{&S44533[122], 3}, {NULL}};
const Segment SGI7007[] = {{&S44533[88], 5}, {NULL}};
const Segment SGI7008[] = {{&S44534[42], 11}, {NULL}};
const Segment SGI7009[] = {{&S44534[42], 33}, {NULL}};
const Segment SGI7010[] = {{&S44534[30], 4}, {NULL}};
const Segment SGI7011[] = {{&S44534[123], 3}, {NULL}};
const Segment SGI7012[] = {{&S44534[89], 5}, {NULL}};
const Segment SGI7013[] = {{&S44535[42], 11}, {NULL}};
const Segment SGI7014[] = {{&S44535[42], 22}, {NULL}};
const Segment SGI7015[] = {{&S44535[30], 4}, {NULL}};
const Segment SGI7016[] = {{&S44535[112], 3}, {NULL}};
const Segment SGI7017[] = {{&S44535[78], 5}, {NULL}};
const Segment SGI7018[] = {{&S44536[36], 11}, {NULL}};
const Segment SGI7019[] = {{&S44536[31], 4}, {NULL}};
const Segment SGI7020[] = {{&S44561[42], 10}, {NULL}};
const Segment SGI7021[] = {{&S44561[42], 21}, {NULL}};
const Segment SGI7022[] = {{&S44561[30], 4}, {NULL}};
const Segment SGI7023[] = {{&S44561[111], 3}, {NULL}};
const Segment SGI7024[] = {{&S44561[77], 5}, {NULL}};
const Segment SGI7025[] = {{&S44563[42], 10}, {NULL}};
const Segment SGI7026[] = {{&S44563[42], 21}, {NULL}};
const Segment SGI7027[] = {{&S44563[30], 4}, {NULL}};
const Segment SGI7028[] = {{&S44563[113], 3}, {NULL}};
const Segment SGI7029[] = {{&S44563[77], 7}, {NULL}};
const Segment SGI7030[] = {{&S44563[133], 8}, {NULL}};
const Segment SGI7031[] = {{&S44564[36], 10}, {NULL}};
const Segment SGI7032[] = {{&S44564[31], 4}, {NULL}};
const Segment SGI7033[] = {{&S44607[42], 10}, {NULL}};
const Segment SGI7034[] = {{&S44607[42], 21}, {NULL}};
const Segment SGI7035[] = {{&S44607[30], 4}, {NULL}};
const Segment SGI7036[] = {{&S44607[111], 3}, {NULL}};
const Segment SGI7037[] = {{&S44607[77], 5}, {NULL}};
const Segment SGI7038[] = {{&S44608[42], 10}, {NULL}};
const Segment SGI7039[] = {{&S44608[42], 21}, {NULL}};
const Segment SGI7040[] = {{&S44608[30], 4}, {NULL}};
const Segment SGI7041[] = {{&S44608[112], 3}, {NULL}};
const Segment SGI7042[] = {{&S44608[77], 6}, {NULL}};
const Segment SGI7043[] = {{&S44609[36], 10}, {NULL}};
const Segment SGI7044[] = {{&S44609[31], 4}, {NULL}};
const Segment SGI7045[] = {{&S44610[42], 11}, {NULL}};
const Segment SGI7046[] = {{&S44610[42], 22}, {NULL}};
const Segment SGI7047[] = {{&S44610[30], 4}, {NULL}};
const Segment SGI7048[] = {{&S44610[112], 3}, {NULL}};
const Segment SGI7049[] = {{&S44610[78], 5}, {NULL}};
const Segment SGI7050[] = {{&S44611[36], 11}, {NULL}};
const Segment SGI7051[] = {{&S44611[31], 4}, {NULL}};
const Segment SGI7052[] = {{&S44673[42], 10}, {NULL}};
const Segment SGI7053[] = {{&S44673[42], 31}, {NULL}};
const Segment SGI7054[] = {{&S44673[30], 4}, {NULL}};
const Segment SGI7055[] = {{&S44673[121], 3}, {NULL}};
const Segment SGI7056[] = {{&S44673[87], 5}, {NULL}};
const Segment SGI7057[] = {{&S44674[42], 10}, {NULL}};
const Segment SGI7058[] = {{&S44674[42], 31}, {NULL}};
const Segment SGI7059[] = {{&S44674[30], 4}, {NULL}};
const Segment SGI7060[] = {{&S44674[121], 3}, {NULL}};
const Segment SGI7061[] = {{&S44674[87], 5}, {NULL}};
const Segment SGI7062[] = {{&S44675[42], 10}, {NULL}};
const Segment SGI7063[] = {{&S44675[42], 31}, {NULL}};
const Segment SGI7064[] = {{&S44675[30], 4}, {NULL}};
const Segment SGI7065[] = {{&S44675[121], 3}, {NULL}};
const Segment SGI7066[] = {{&S44675[87], 5}, {NULL}};
const Segment SGI7067[] = {{&S44676[42], 10}, {NULL}};
const Segment SGI7068[] = {{&S44676[42], 31}, {NULL}};
const Segment SGI7069[] = {{&S44676[30], 4}, {NULL}};
const Segment SGI7070[] = {{&S44676[121], 3}, {NULL}};
const Segment SGI7071[] = {{&S44676[87], 5}, {NULL}};
const Segment SGI7072[] = {{&S44678[42], 10}, {NULL}};
const Segment SGI7073[] = {{&S44678[42], 20}, {NULL}};
const Segment SGI7074[] = {{&S44678[30], 4}, {NULL}};
const Segment SGI7075[] = {{&S44678[110], 3}, {NULL}};
const Segment SGI7076[] = {{&S44678[76], 5}, {NULL}};
const Segment SGI7077[] = {{&S44679[88], 6}, {NULL}};
const Segment SGI7078[] = {{&S44679[88], 26}, {NULL}};
const Segment SGI7079[] = {{&S44679[76], 4}, {NULL}};
const Segment SGI7080[] = {{&S44679[162], 3}, {NULL}};
const Segment SGI7081[] = {{&S44679[128], 5}, {NULL}};
const Segment SGI7082[] = {{&S44680[42], 10}, {NULL}};
const Segment SGI7083[] = {{&S44680[42], 31}, {NULL}};
const Segment SGI7084[] = {{&S44680[30], 4}, {NULL}};
const Segment SGI7085[] = {{&S44680[121], 3}, {NULL}};
const Segment SGI7086[] = {{&S44680[87], 5}, {NULL}};
const Segment SGI7087[] = {{&S44681[42], 10}, {NULL}};
const Segment SGI7088[] = {{&S44681[42], 30}, {NULL}};
const Segment SGI7089[] = {{&S44681[30], 4}, {NULL}};
const Segment SGI7090[] = {{&S44681[120], 3}, {NULL}};
const Segment SGI7091[] = {{&S44681[86], 5}, {NULL}};
const Segment SGI7092[] = {{&S44682[42], 10}, {NULL}};
const Segment SGI7093[] = {{&S44682[42], 26}, {NULL}};
const Segment SGI7094[] = {{&S44682[30], 4}, {NULL}};
const Segment SGI7095[] = {{&S44682[116], 3}, {NULL}};
const Segment SGI7096[] = {{&S44682[82], 5}, {NULL}};
const Segment SGI7097[] = {{&S44683[89], 6}, {NULL}};
const Segment SGI7098[] = {{&S44683[89], 27}, {NULL}};
const Segment SGI7099[] = {{&S44683[77], 4}, {NULL}};
const Segment SGI7100[] = {{&S44683[164], 3}, {NULL}};
const Segment SGI7101[] = {{&S44683[130], 5}, {NULL}};
const Segment SGI7102[] = {{&S44684[42], 10}, {NULL}};
const Segment SGI7103[] = {{&S44684[42], 20}, {NULL}};
const Segment SGI7104[] = {{&S44684[30], 4}, {NULL}};
const Segment SGI7105[] = {{&S44684[110], 3}, {NULL}};
const Segment SGI7106[] = {{&S44684[76], 5}, {NULL}};
const Segment SGI7107[] = {{&S44687[42], 10}, {NULL}};
const Segment SGI7108[] = {{&S44687[42], 31}, {NULL}};
const Segment SGI7109[] = {{&S44687[30], 4}, {NULL}};
const Segment SGI7110[] = {{&S44687[121], 3}, {NULL}};
const Segment SGI7111[] = {{&S44687[87], 5}, {NULL}};
const Segment SGI7112[] = {{&S44689[42], 10}, {NULL}};
const Segment SGI7113[] = {{&S44689[42], 21}, {NULL}};
const Segment SGI7114[] = {{&S44689[30], 4}, {NULL}};
const Segment SGI7115[] = {{&S44689[111], 3}, {NULL}};
const Segment SGI7116[] = {{&S44689[77], 5}, {NULL}};
const Segment SGI7117[] = {{&S44698[42], 10}, {NULL}};
const Segment SGI7118[] = {{&S44698[42], 31}, {NULL}};
const Segment SGI7119[] = {{&S44698[30], 4}, {NULL}};
const Segment SGI7120[] = {{&S44698[121], 3}, {NULL}};
const Segment SGI7121[] = {{&S44698[87], 5}, {NULL}};
const Segment SGI7122[] = {{&S44699[42], 10}, {NULL}};
const Segment SGI7123[] = {{&S44699[42], 20}, {NULL}};
const Segment SGI7124[] = {{&S44699[30], 4}, {NULL}};
const Segment SGI7125[] = {{&S44699[110], 3}, {NULL}};
const Segment SGI7126[] = {{&S44699[76], 5}, {NULL}};
const Segment SGI7127[] = {{&S44700[36], 6}, {NULL}};
const Segment SGI7128[] = {{&S44700[31], 4}, {NULL}};
const Segment SGI7129[] = {{&S44702[36], 10}, {NULL}};
const Segment SGI7130[] = {{&S44702[31], 4}, {NULL}};
const Segment SGI7131[] = {{&S44704[42], 10}, {NULL}};
const Segment SGI7132[] = {{&S44704[42], 31}, {NULL}};
const Segment SGI7133[] = {{&S44704[30], 4}, {NULL}};
const Segment SGI7134[] = {{&S44704[90], 4}, {NULL}};
const Segment SGI7135[] = {{&S44704[90], 3}, {NULL}};
const Segment SGI7136[] = {{&S44705[42], 10}, {NULL}};
const Segment SGI7137[] = {{&S44705[42], 31}, {NULL}};
const Segment SGI7138[] = {{&S44705[30], 4}, {NULL}};
const Segment SGI7139[] = {{&S44705[90], 6}, {NULL}};
const Segment SGI7140[] = {{&S44768[42], 16}, {NULL}};
const Segment SGI7141[] = {{&S44768[42], 27}, {NULL}};
const Segment SGI7142[] = {{&S44768[30], 4}, {NULL}};
const Segment SGI7143[] = {{&S44768[117], 3}, {NULL}};
const Segment SGI7144[] = {{&S44768[83], 5}, {NULL}};
const Segment SGI7145[] = {{&S44770[42], 16}, {NULL}};
const Segment SGI7146[] = {{&S44770[42], 27}, {NULL}};
const Segment SGI7147[] = {{&S44770[30], 4}, {NULL}};
const Segment SGI7148[] = {{&S44770[117], 3}, {NULL}};
const Segment SGI7149[] = {{&S44770[83], 5}, {NULL}};
const Segment SGI7150[] = {{&S44771[42], 18}, {NULL}};
const Segment SGI7151[] = {{&S44771[42], 29}, {NULL}};
const Segment SGI7152[] = {{&S44771[30], 4}, {NULL}};
const Segment SGI7153[] = {{&S44771[119], 3}, {NULL}};
const Segment SGI7154[] = {{&S44771[85], 5}, {NULL}};
const Segment SGI7155[] = {{&S44772[42], 18}, {NULL}};
const Segment SGI7156[] = {{&S44772[42], 29}, {NULL}};
const Segment SGI7157[] = {{&S44772[30], 4}, {NULL}};
const Segment SGI7158[] = {{&S44772[119], 3}, {NULL}};
const Segment SGI7159[] = {{&S44772[85], 5}, {NULL}};
const Segment SGI7160[] = {{&S44777[36], 16}, {NULL}};
const Segment SGI7161[] = {{&S44777[31], 4}, {NULL}};
const Segment SGI7162[] = {{&S44778[36], 18}, {NULL}};
const Segment SGI7163[] = {{&S44778[31], 4}, {NULL}};
const Segment SGI7164[] = {{&S44779[42], 16}, {NULL}};
const Segment SGI7165[] = {{&S44779[42], 27}, {NULL}};
const Segment SGI7166[] = {{&S44779[30], 4}, {NULL}};
const Segment SGI7167[] = {{&S44779[86], 4}, {NULL}};
const Segment SGI7168[] = {{&S44779[86], 3}, {NULL}};
const Segment SGI7169[] = {{&S44780[42], 16}, {NULL}};
const Segment SGI7170[] = {{&S44780[42], 27}, {NULL}};
const Segment SGI7171[] = {{&S44780[30], 4}, {NULL}};
const Segment SGI7172[] = {{&S44780[86], 6}, {NULL}};
const Segment SGI7173[] = {{&S44677[97], 6}, {NULL}};
const Segment SGI7174[] = {{&S44677[97], 27}, {NULL}};
const Segment SGI7175[] = {{&S44677[83], 6}, {NULL}};
const Segment SGI7176[] = {{&S44677[172], 3}, {NULL}};
const Segment SGI7177[] = {{&S44677[138], 5}, {NULL}};
const Segment SGI7178[] = {{&S44686[88], 6}, {NULL}};
const Segment SGI7179[] = {{&S44686[88], 27}, {NULL}};
const Segment SGI7180[] = {{&S44686[74], 6}, {NULL}};
const Segment SGI7181[] = {{&S44686[163], 3}, {NULL}};
const Segment SGI7182[] = {{&S44686[129], 5}, {NULL}};
const Segment SGI7183[] = {{&S44690[90], 6}, {NULL}};
const Segment SGI7184[] = {{&S44690[90], 27}, {NULL}};
const Segment SGI7185[] = {{&S44690[76], 6}, {NULL}};
const Segment SGI7186[] = {{&S44690[165], 3}, {NULL}};
const Segment SGI7187[] = {{&S44690[131], 5}, {NULL}};
const Segment SGI7188[] = {{&S44691[92], 6}, {NULL}};
const Segment SGI7189[] = {{&S44691[92], 29}, {NULL}};
const Segment SGI7190[] = {{&S44691[78], 6}, {NULL}};
const Segment SGI7191[] = {{&S44691[169], 3}, {NULL}};
const Segment SGI7192[] = {{&S44691[135], 5}, {NULL}};
const Segment SGI7193[] = {{&S44693[97], 6}, {NULL}};
const Segment SGI7194[] = {{&S44693[97], 27}, {NULL}};
const Segment SGI7195[] = {{&S44693[83], 6}, {NULL}};
const Segment SGI7196[] = {{&S44693[172], 3}, {NULL}};
const Segment SGI7197[] = {{&S44693[138], 5}, {NULL}};
const Segment SGI7198[] = {{&S44695[93], 6}, {NULL}};
const Segment SGI7199[] = {{&S44695[93], 27}, {NULL}};
const Segment SGI7200[] = {{&S44695[79], 6}, {NULL}};
const Segment SGI7201[] = {{&S44695[168], 3}, {NULL}};
const Segment SGI7202[] = {{&S44695[134], 5}, {NULL}};
const Segment SGI7203[] = {{&S44773[44], 7}, {NULL}};
const Segment SGI7204[] = {{&S44773[44], 18}, {NULL}};
const Segment SGI7205[] = {{&S44773[30], 6}, {NULL}};
const Segment SGI7206[] = {{&S44773[110], 3}, {NULL}};
const Segment SGI7207[] = {{&S44773[76], 5}, {NULL}};
const Segment SGI7208[] = {{&S45014[42], 19}, {NULL}};
const Segment SGI7209[] = {{&S45014[42], 30}, {NULL}};
const Segment SGI7210[] = {{&S45014[30], 4}, {NULL}};
const Segment SGI7211[] = {{&S45014[120], 3}, {NULL}};
const Segment SGI7212[] = {{&S45014[86], 5}, {NULL}};
const Segment SGI7213[] = {{&S45015[42], 25}, {NULL}};
const Segment SGI7214[] = {{&S45015[42], 36}, {NULL}};
const Segment SGI7215[] = {{&S45015[30], 4}, {NULL}};
const Segment SGI7216[] = {{&S45015[126], 3}, {NULL}};
const Segment SGI7217[] = {{&S45015[92], 5}, {NULL}};
const Segment SGI7218[] = {{&S45019[42], 25}, {NULL}};
const Segment SGI7219[] = {{&S45019[42], 36}, {NULL}};
const Segment SGI7220[] = {{&S45019[30], 4}, {NULL}};
const Segment SGI7221[] = {{&S45019[126], 3}, {NULL}};
const Segment SGI7222[] = {{&S45019[92], 5}, {NULL}};
const Segment SGI7223[] = {{&S45022[42], 11}, {NULL}};
const Segment SGI7224[] = {{&S45022[42], 22}, {NULL}};
const Segment SGI7225[] = {{&S45022[30], 4}, {NULL}};
const Segment SGI7226[] = {{&S45022[112], 3}, {NULL}};
const Segment SGI7227[] = {{&S45022[78], 5}, {NULL}};
const Segment SGI7228[] = {{&S44933[44], 15}, {NULL}};
const Segment SGI7229[] = {{&S44933[44], 26}, {NULL}};
const Segment SGI7230[] = {{&S44933[30], 6}, {NULL}};
const Segment SGI7231[] = {{&S44933[118], 3}, {NULL}};
const Segment SGI7232[] = {{&S44933[84], 5}, {NULL}};
const Segment SGI7233[] = {{&S47765[42], 13}, {NULL}};
const Segment SGI7234[] = {{&S47765[42], 24}, {NULL}};
const Segment SGI7235[] = {{&S47765[30], 4}, {NULL}};
const Segment SGI7236[] = {{&S47765[115], 3}, {NULL}};
const Segment SGI7237[] = {{&S47765[80], 6}, {NULL}};
const Segment SGI7238[] = {{&S47765[135], 8}, {NULL}};
const Segment SGI7239[] = {{&S47766[42], 12}, {NULL}};
const Segment SGI7240[] = {{&S47766[42], 23}, {NULL}};
const Segment SGI7241[] = {{&S47766[30], 4}, {NULL}};
const Segment SGI7242[] = {{&S47766[114], 3}, {NULL}};
const Segment SGI7243[] = {{&S47766[79], 6}, {NULL}};
const Segment SGI7244[] = {{&S47766[134], 8}, {NULL}};
const Segment SGI7245[] = {{&S45007[44], 26}, {NULL}};
const Segment SGI7246[] = {{&S45007[44], 38}, {NULL}};
const Segment SGI7247[] = {{&S45007[30], 6}, {NULL}};
const Segment SGI7248[] = {{&S45007[130], 3}, {NULL}};
const Segment SGI7249[] = {{&S47767[42], 13}, {NULL}};
const Segment SGI7250[] = {{&S47767[42], 24}, {NULL}};
const Segment SGI7251[] = {{&S47767[30], 4}, {NULL}};
const Segment SGI7252[] = {{&S45007[96], 5}, {NULL}};
const Segment SGI7253[] = {{&S47767[115], 3}, {NULL}};
const Segment SGI7254[] = {{&S47767[80], 6}, {NULL}};
const Segment SGI7255[] = {{&S45008[44], 25}, {NULL}};
const Segment SGI7256[] = {{&S45008[44], 37}, {NULL}};
const Segment SGI7257[] = {{&S47768[40], 13}, {NULL}};
const Segment SGI7258[] = {{&S45008[30], 6}, {NULL}};
const Segment SGI7259[] = {{&S47768[30], 4}, {NULL}};
const Segment SGI7260[] = {{&S45008[129], 3}, {NULL}};
const Segment SGI7261[] = {{&S47768[113], 3}, {NULL}};
const Segment SGI7262[] = {{&S45008[95], 5}, {NULL}};
const Segment SGI7263[] = {{&S47768[78], 6}, {NULL}};
const Segment SGI7264[] = {{&S47768[133], 8}, {NULL}};
const Segment SGI7265[] = {{&S47769[42], 13}, {NULL}};
const Segment SGI7266[] = {{&S47769[42], 26}, {NULL}};
const Segment SGI7267[] = {{&S47769[30], 4}, {NULL}};
const Segment SGI7268[] = {{&S45009[44], 25}, {NULL}};
const Segment SGI7269[] = {{&S45009[44], 37}, {NULL}};
const Segment SGI7270[] = {{&S45009[30], 6}, {NULL}};
const Segment SGI7271[] = {{&S47769[117], 3}, {NULL}};
const Segment SGI7272[] = {{&S47769[82], 6}, {NULL}};
const Segment SGI7273[] = {{&S45009[129], 3}, {NULL}};
const Segment SGI7274[] = {{&S47769[137], 8}, {NULL}};
const Segment SGI7275[] = {{&S45009[95], 5}, {NULL}};
const Segment SGI7276[] = {{&S47770[41], 13}, {NULL}};
const Segment SGI7277[] = {{&S47770[41], 24}, {NULL}};
const Segment SGI7278[] = {{&S47770[30], 3}, {NULL}};
const Segment SGI7279[] = {{&S47770[114], 3}, {NULL}};
const Segment SGI7280[] = {{&S45010[44], 25}, {NULL}};
const Segment SGI7281[] = {{&S45010[44], 37}, {NULL}};
const Segment SGI7282[] = {{&S47770[79], 6}, {NULL}};
const Segment SGI7283[] = {{&S45010[30], 6}, {NULL}};
const Segment SGI7284[] = {{&S47770[134], 8}, {NULL}};
const Segment SGI7285[] = {{&S45010[129], 3}, {NULL}};
const Segment SGI7286[] = {{&S45010[95], 5}, {NULL}};
const Segment SGI7287[] = {{&S47771[42], 13}, {NULL}};
const Segment SGI7288[] = {{&S47771[42], 24}, {NULL}};
const Segment SGI7289[] = {{&S47771[30], 4}, {NULL}};
const Segment SGI7290[] = {{&S47771[115], 3}, {NULL}};
const Segment SGI7291[] = {{&S45011[44], 23}, {NULL}};
const Segment SGI7292[] = {{&S47771[80], 6}, {NULL}};
const Segment SGI7293[] = {{&S45011[44], 35}, {NULL}};
const Segment SGI7294[] = {{&S47771[135], 8}, {NULL}};
const Segment SGI7295[] = {{&S45011[30], 6}, {NULL}};
const Segment SGI7296[] = {{&S45011[127], 3}, {NULL}};
const Segment SGI7297[] = {{&S45011[93], 5}, {NULL}};
const Segment SGI7298[] = {{&S47772[42], 13}, {NULL}};
const Segment SGI7299[] = {{&S47772[42], 24}, {NULL}};
const Segment SGI7300[] = {{&S47772[30], 4}, {NULL}};
const Segment SGI7301[] = {{&S47772[115], 3}, {NULL}};
const Segment SGI7302[] = {{&S47772[80], 6}, {NULL}};
const Segment SGI7303[] = {{&S47772[135], 8}, {NULL}};
const Segment SGI7304[] = {{&S45012[44], 25}, {NULL}};
const Segment SGI7305[] = {{&S45012[44], 37}, {NULL}};
const Segment SGI7306[] = {{&S45012[30], 6}, {NULL}};
const Segment SGI7307[] = {{&S45012[129], 3}, {NULL}};
const Segment SGI7308[] = {{&S47773[41], 13}, {NULL}};
const Segment SGI7309[] = {{&S45012[95], 5}, {NULL}};
const Segment SGI7310[] = {{&S47773[41], 24}, {NULL}};
const Segment SGI7311[] = {{&S47773[114], 3}, {NULL}};
const Segment SGI7312[] = {{&S47773[79], 6}, {NULL}};
const Segment SGI7313[] = {{&S47773[134], 8}, {NULL}};
const Segment SGI7314[] = {{&S45013[44], 11}, {NULL}};
const Segment SGI7315[] = {{&S45013[44], 23}, {NULL}};
const Segment SGI7316[] = {{&S45013[30], 6}, {NULL}};
const Segment SGI7317[] = {{&S47774[42], 13}, {NULL}};
const Segment SGI7318[] = {{&S47774[42], 24}, {NULL}};
const Segment SGI7319[] = {{&S47774[30], 4}, {NULL}};
const Segment SGI7320[] = {{&S45013[115], 3}, {NULL}};
const Segment SGI7321[] = {{&S45013[81], 5}, {NULL}};
const Segment SGI7322[] = {{&S47774[115], 3}, {NULL}};
const Segment SGI7323[] = {{&S47774[80], 5}, {NULL}};
const Segment SGI7324[] = {{&S47774[135], 8}, {NULL}};
const Segment SGI7325[] = {{&S47775[42], 13}, {NULL}};
const Segment SGI7326[] = {{&S47775[42], 24}, {NULL}};
const Segment SGI7327[] = {{&S47775[30], 4}, {NULL}};
const Segment SGI7328[] = {{&S47775[115], 3}, {NULL}};
const Segment SGI7329[] = {{&S47775[80], 6}, {NULL}};
const Segment SGI7330[] = {{&S47776[42], 13}, {NULL}};
const Segment SGI7331[] = {{&S47776[42], 24}, {NULL}};
const Segment SGI7332[] = {{&S45016[44], 25}, {NULL}};
const Segment SGI7333[] = {{&S47776[30], 4}, {NULL}};
const Segment SGI7334[] = {{&S45016[44], 37}, {NULL}};
const Segment SGI7335[] = {{&S45016[30], 6}, {NULL}};
const Segment SGI7336[] = {{&S47776[115], 3}, {NULL}};
const Segment SGI7337[] = {{&S47776[80], 6}, {NULL}};
const Segment SGI7338[] = {{&S45016[129], 3}, {NULL}};
const Segment SGI7339[] = {{&S47776[135], 8}, {NULL}};
const Segment SGI7340[] = {{&S45016[95], 5}, {NULL}};
const Segment SGI7341[] = {{&S45017[44], 25}, {NULL}};
const Segment SGI7342[] = {{&S45017[44], 37}, {NULL}};
const Segment SGI7343[] = {{&S45017[30], 6}, {NULL}};
const Segment SGI7344[] = {{&S45017[129], 3}, {NULL}};
const Segment SGI7345[] = {{&S45017[95], 5}, {NULL}};
const Segment SGI7346[] = {{&S45018[44], 25}, {NULL}};
const Segment SGI7347[] = {{&S45018[44], 37}, {NULL}};
const Segment SGI7348[] = {{&S45018[30], 6}, {NULL}};
const Segment SGI7349[] = {{&S45018[129], 3}, {NULL}};
const Segment SGI7350[] = {{&S45018[95], 5}, {NULL}};
const Segment SGI7351[] = {{&S45020[44], 19}, {NULL}};
const Segment SGI7352[] = {{&S45020[44], 30}, {NULL}};
const Segment SGI7353[] = {{&S45020[30], 6}, {NULL}};
const Segment SGI7354[] = {{&S45020[122], 3}, {NULL}};
const Segment SGI7355[] = {{&S45020[88], 5}, {NULL}};
const Segment SGI7356[] = {{&S45021[44], 23}, {NULL}};
const Segment SGI7357[] = {{&S45021[44], 35}, {NULL}};
const Segment SGI7358[] = {{&S45021[30], 6}, {NULL}};
const Segment SGI7359[] = {{&S45021[127], 3}, {NULL}};
const Segment SGI7360[] = {{&S45021[93], 5}, {NULL}};
const Segment SGI7361[] = {{&S45023[44], 19}, {NULL}};
const Segment SGI7362[] = {{&S45023[44], 31}, {NULL}};
const Segment SGI7363[] = {{&S45023[30], 6}, {NULL}};
const Segment SGI7364[] = {{&S45023[123], 3}, {NULL}};
const Segment SGI7365[] = {{&S45023[89], 5}, {NULL}};
const Segment SGI7366[] = {{&S45024[44], 25}, {NULL}};
const Segment SGI7367[] = {{&S45024[44], 37}, {NULL}};
const Segment SGI7368[] = {{&S45024[30], 6}, {NULL}};
const Segment SGI7369[] = {{&S45024[129], 3}, {NULL}};
const Segment SGI7370[] = {{&S45024[95], 5}, {NULL}};
const Segment SGI7371[] = {{&S45441[42], 19}, {NULL}};
const Segment SGI7372[] = {{&S45441[42], 30}, {NULL}};
const Segment SGI7373[] = {{&S45441[30], 4}, {NULL}};
const Segment SGI7374[] = {{&S45441[120], 3}, {NULL}};
const Segment SGI7375[] = {{&S45441[86], 5}, {NULL}};
const Segment SGI7376[] = {{&S45489[41], 11}, {NULL}};
const Segment SGI7377[] = {{&S45489[41], 22}, {NULL}};
const Segment SGI7378[] = {{&S45489[30], 3}, {NULL}};
const Segment SGI7379[] = {{&S45489[111], 3}, {NULL}};
const Segment SGI7380[] = {{&S45489[77], 5}, {NULL}};
const Segment SGI7381[] = {{&S45494[41], 11}, {NULL}};
const Segment SGI7382[] = {{&S45494[41], 22}, {NULL}};
const Segment SGI7383[] = {{&S45494[111], 3}, {NULL}};
const Segment SGI7384[] = {{&S45494[77], 5}, {NULL}};
const Segment SGI7385[] = {{&S45497[42], 10}, {NULL}};
const Segment SGI7386[] = {{&S45497[42], 21}, {NULL}};
const Segment SGI7387[] = {{&S45497[30], 4}, {NULL}};
const Segment SGI7388[] = {{&S45497[111], 3}, {NULL}};
const Segment SGI7389[] = {{&S45497[77], 5}, {NULL}};
const Segment SGI7390[] = {{&S45498[42], 11}, {NULL}};
const Segment SGI7391[] = {{&S45498[42], 22}, {NULL}};
const Segment SGI7392[] = {{&S45498[30], 4}, {NULL}};
const Segment SGI7393[] = {{&S45498[112], 3}, {NULL}};
const Segment SGI7394[] = {{&S45498[78], 5}, {NULL}};
const Segment SGI7395[] = {{&S48053[42], 17}, {NULL}};
const Segment SGI7396[] = {{&S48053[42], 30}, {NULL}};
const Segment SGI7397[] = {{&S48053[30], 4}, {NULL}};
const Segment SGI7398[] = {{&S48053[121], 3}, {NULL}};
const Segment SGI7399[] = {{&S48053[86], 6}, {NULL}};
const Segment SGI7400[] = {{&S48053[141], 8}, {NULL}};
const Segment SGI7401[] = {{&S48054[42], 10}, {NULL}};
const Segment SGI7402[] = {{&S48054[42], 23}, {NULL}};
const Segment SGI7403[] = {{&S48054[30], 4}, {NULL}};
const Segment SGI7404[] = {{&S48054[114], 3}, {NULL}};
const Segment SGI7405[] = {{&S48054[79], 6}, {NULL}};
const Segment SGI7406[] = {{&S48054[134], 8}, {NULL}};
const Segment SGI7407[] = {{&S48055[42], 10}, {NULL}};
const Segment SGI7408[] = {{&S48055[42], 23}, {NULL}};
const Segment SGI7409[] = {{&S48055[30], 4}, {NULL}};
const Segment SGI7410[] = {{&S48055[114], 3}, {NULL}};
const Segment SGI7411[] = {{&S48055[79], 5}, {NULL}};
const Segment SGI7412[] = {{&S48055[134], 8}, {NULL}};
const Segment SGI7413[] = {{&S45192[50], 7}, {NULL}};
const Segment SGI7414[] = {{&S45192[50], 26}, {NULL}};
const Segment SGI7415[] = {{&S45192[30], 12}, {NULL}};
const Segment SGI7416[] = {{&S45192[125], 3}, {NULL}};
const Segment SGI7417[] = {{&S45192[90], 6}, {NULL}};
const Segment SGI7418[] = {{&S45193[50], 12}, {NULL}};
const Segment SGI7419[] = {{&S45193[50], 23}, {NULL}};
const Segment SGI7420[] = {{&S45193[30], 12}, {NULL}};
const Segment SGI7421[] = {{&S45193[122], 3}, {NULL}};
const Segment SGI7422[] = {{&S45193[87], 6}, {NULL}};
const Segment SGI7423[] = {{&S45194[50], 4}, {NULL}};
const Segment SGI7424[] = {{&S45194[50], 15}, {NULL}};
const Segment SGI7425[] = {{&S45194[30], 12}, {NULL}};
const Segment SGI7426[] = {{&S45194[114], 3}, {NULL}};
const Segment SGI7427[] = {{&S45194[79], 6}, {NULL}};
const Segment SGI7428[] = {{&S45195[50], 7}, {NULL}};
const Segment SGI7429[] = {{&S45195[50], 18}, {NULL}};
const Segment SGI7430[] = {{&S45195[30], 12}, {NULL}};
const Segment SGI7431[] = {{&S45195[117], 3}, {NULL}};
const Segment SGI7432[] = {{&S45195[82], 6}, {NULL}};
const Segment SGI7433[] = {{&S45196[50], 7}, {NULL}};
const Segment SGI7434[] = {{&S45196[50], 18}, {NULL}};
const Segment SGI7435[] = {{&S45196[30], 12}, {NULL}};
const Segment SGI7436[] = {{&S45196[117], 3}, {NULL}};
const Segment SGI7437[] = {{&S45196[82], 6}, {NULL}};
const Segment SGI7438[] = {{&S45197[50], 14}, {NULL}};
const Segment SGI7439[] = {{&S45197[50], 25}, {NULL}};
const Segment SGI7440[] = {{&S45197[30], 12}, {NULL}};
const Segment SGI7441[] = {{&S45197[124], 3}, {NULL}};
const Segment SGI7442[] = {{&S45197[89], 6}, {NULL}};
const Segment SGI7443[] = {{&S45198[50], 9}, {NULL}};
const Segment SGI7444[] = {{&S45198[50], 20}, {NULL}};
const Segment SGI7445[] = {{&S45198[30], 12}, {NULL}};
const Segment SGI7446[] = {{&S45198[119], 3}, {NULL}};
const Segment SGI7447[] = {{&S45198[84], 6}, {NULL}};
const Segment SGI7448[] = {{&S45199[50], 22}, {NULL}};
const Segment SGI7449[] = {{&S45199[50], 33}, {NULL}};
const Segment SGI7450[] = {{&S45199[30], 12}, {NULL}};
const Segment SGI7451[] = {{&S45199[132], 3}, {NULL}};
const Segment SGI7452[] = {{&S45199[97], 6}, {NULL}};
const Segment SGI7453[] = {{&S45200[56], 8}, {NULL}};
const Segment SGI7454[] = {{&S45200[56], 20}, {NULL}};
const Segment SGI7455[] = {{&S45200[30], 18}, {NULL}};
const Segment SGI7456[] = {{&S45200[125], 3}, {NULL}};
const Segment SGI7457[] = {{&S45200[90], 6}, {NULL}};
const Segment SGI7458[] = {{&S45201[50], 7}, {NULL}};
const Segment SGI7459[] = {{&S45201[50], 18}, {NULL}};
const Segment SGI7460[] = {{&S45201[30], 12}, {NULL}};
const Segment SGI7461[] = {{&S45201[117], 3}, {NULL}};
const Segment SGI7462[] = {{&S45201[82], 6}, {NULL}};
const Segment SGI7463[] = {{&S45202[50], 5}, {NULL}};
const Segment SGI7464[] = {{&S45202[50], 17}, {NULL}};
const Segment SGI7465[] = {{&S45202[30], 12}, {NULL}};
const Segment SGI7466[] = {{&S45202[116], 3}, {NULL}};
const Segment SGI7467[] = {{&S45202[81], 6}, {NULL}};
const Segment SGI7468[] = {{&S45203[50], 5}, {NULL}};
const Segment SGI7469[] = {{&S45203[50], 17}, {NULL}};
const Segment SGI7470[] = {{&S45203[30], 12}, {NULL}};
const Segment SGI7471[] = {{&S45203[116], 3}, {NULL}};
const Segment SGI7472[] = {{&S45203[81], 6}, {NULL}};
const Segment SGI7473[] = {{&S45204[50], 51}, {NULL}};
const Segment SGI7474[] = {{&S45204[30], 12}, {&S45204[58], 43}, {NULL}};
const Segment SGI7475[] = {{&S45204[164], 3}, {NULL}};
const Segment SGI7476[] = {{&S45575[42], 10}, {NULL}};
const Segment SGI7477[] = {{&S45575[42], 21}, {NULL}};
const Segment SGI7478[] = {{&S45575[30], 4}, {NULL}};
const Segment SGI7479[] = {{&S45204[129], 6}, {NULL}};
const Segment SGI7480[] = {{&S45575[111], 3}, {NULL}};
const Segment SGI7481[] = {{&S45575[77], 5}, {NULL}};
const Segment SGI7482[] = {{&S45205[42], 6}, {NULL}};
const Segment SGI7483[] = {{&S45205[42], 17}, {NULL}};
const Segment SGI7484[] = {{&S45205[30], 4}, {NULL}};
const Segment SGI7485[] = {{&S45205[108], 3}, {NULL}};
const Segment SGI7486[] = {{&S45205[73], 6}, {NULL}};
const Segment SGI7487[] = {{&S45577[36], 10}, {NULL}};
const Segment SGI7488[] = {{&S45577[31], 4}, {NULL}};
const Segment SGI7489[] = {{&S45206[50], 9}, {NULL}};
const Segment SGI7490[] = {{&S45206[50], 21}, {NULL}};
const Segment SGI7491[] = {{&S45206[30], 12}, {NULL}};
const Segment SGI7492[] = {{&S45206[120], 3}, {NULL}};
const Segment SGI7493[] = {{&S45206[85], 6}, {NULL}};
const Segment SGI7494[] = {{&S45207[50], 7}, {NULL}};
const Segment SGI7495[] = {{&S45207[50], 18}, {NULL}};
const Segment SGI7496[] = {{&S45207[30], 12}, {NULL}};
const Segment SGI7497[] = {{&S45207[117], 3}, {NULL}};
const Segment SGI7498[] = {{&S45207[82], 6}, {NULL}};
const Segment SGI7499[] = {{&S45208[56], 8}, {NULL}};
const Segment SGI7500[] = {{&S45208[56], 20}, {NULL}};
const Segment SGI7501[] = {{&S45208[30], 18}, {NULL}};
const Segment SGI7502[] = {{&S45208[125], 3}, {NULL}};
const Segment SGI7503[] = {{&S45208[90], 6}, {NULL}};
const Segment SGI7504[] = {{&S45209[44], 6}, {NULL}};
const Segment SGI7505[] = {{&S45209[31], 12}, {NULL}};
const Segment SGI7506[] = {{&S45210[50], 6}, {NULL}};
const Segment SGI7507[] = {{&S45210[50], 18}, {NULL}};
const Segment SGI7508[] = {{&S45210[30], 12}, {NULL}};
const Segment SGI7509[] = {{&S45210[117], 3}, {NULL}};
const Segment SGI7510[] = {{&S45210[82], 6}, {NULL}};
const Segment SGI7511[] = {{&S45211[56], 10}, {NULL}};
const Segment SGI7512[] = {{&S45211[56], 22}, {NULL}};
const Segment SGI7513[] = {{&S45211[30], 18}, {NULL}};
const Segment SGI7514[] = {{&S45211[127], 3}, {NULL}};
const Segment SGI7515[] = {{&S45211[92], 6}, {NULL}};
const Segment SGI7516[] = {{&S45212[42], 6}, {NULL}};
const Segment SGI7517[] = {{&S45212[42], 17}, {NULL}};
const Segment SGI7518[] = {{&S45212[30], 4}, {NULL}};
const Segment SGI7519[] = {{&S45212[76], 4}, {NULL}};
const Segment SGI7520[] = {{&S45212[76], 3}, {NULL}};
const Segment SGI7521[] = {{&S45213[50], 5}, {NULL}};
const Segment SGI7522[] = {{&S45213[50], 17}, {NULL}};
const Segment SGI7523[] = {{&S45213[30], 12}, {NULL}};
const Segment SGI7524[] = {{&S45213[116], 3}, {NULL}};
const Segment SGI7525[] = {{&S45213[81], 6}, {NULL}};
const Segment SGI7526[] = {{&S45214[50], 5}, {NULL}};
const Segment SGI7527[] = {{&S45214[50], 16}, {NULL}};
const Segment SGI7528[] = {{&S45214[30], 12}, {NULL}};
const Segment SGI7529[] = {{&S45214[115], 3}, {NULL}};
const Segment SGI7530[] = {{&S45214[80], 6}, {NULL}};
const Segment SGI7531[] = {{&S45215[56], 9}, {NULL}};
const Segment SGI7532[] = {{&S45215[56], 21}, {NULL}};
const Segment SGI7533[] = {{&S45215[30], 18}, {NULL}};
const Segment SGI7534[] = {{&S45215[126], 3}, {NULL}};
const Segment SGI7535[] = {{&S45215[91], 6}, {NULL}};
const Segment SGI7536[] = {{&S45216[50], 18}, {NULL}};
const Segment SGI7537[] = {{&S45216[50], 30}, {NULL}};
const Segment SGI7538[] = {{&S45216[30], 12}, {NULL}};
const Segment SGI7539[] = {{&S45216[129], 3}, {NULL}};
const Segment SGI7540[] = {{&S45216[94], 6}, {NULL}};
const Segment SGI7541[] = {{&S45217[56], 8}, {NULL}};
const Segment SGI7542[] = {{&S45217[56], 20}, {NULL}};
const Segment SGI7543[] = {{&S45217[30], 18}, {NULL}};
const Segment SGI7544[] = {{&S45217[125], 3}, {NULL}};
const Segment SGI7545[] = {{&S45217[90], 6}, {NULL}};
const Segment SGI7546[] = {{&S45218[52], 6}, {NULL}};
const Segment SGI7547[] = {{&S45218[52], 18}, {NULL}};
const Segment SGI7548[] = {{&S45218[30], 14}, {NULL}};
const Segment SGI7549[] = {{&S45218[119], 3}, {NULL}};
const Segment SGI7550[] = {{&S45218[84], 6}, {NULL}};
const Segment SGI7551[] = {{&S45219[56], 8}, {NULL}};
const Segment SGI7552[] = {{&S45219[56], 20}, {NULL}};
const Segment SGI7553[] = {{&S45219[30], 18}, {NULL}};
const Segment SGI7554[] = {{&S45219[125], 3}, {NULL}};
const Segment SGI7555[] = {{&S45219[90], 6}, {NULL}};
const Segment SGI7556[] = {{&S45220[48], 13}, {NULL}};
const Segment SGI7557[] = {{&S45220[30], 12}, {NULL}};
const Segment SGI7558[] = {{&S45220[121], 3}, {NULL}};
const Segment SGI7559[] = {{&S45220[86], 6}, {NULL}};
const Segment SGI7560[] = {{&S45222[50], 16}, {NULL}};
const Segment SGI7561[] = {{&S45222[50], 28}, {NULL}};
const Segment SGI7562[] = {{&S45222[30], 12}, {NULL}};
const Segment SGI7563[] = {{&S45222[127], 3}, {NULL}};
const Segment SGI7564[] = {{&S45222[92], 6}, {NULL}};
const Segment SGI7565[] = {{&S45610[42], 7}, {NULL}};
const Segment SGI7566[] = {{&S45610[42], 18}, {NULL}};
const Segment SGI7567[] = {{&S45610[30], 4}, {NULL}};
const Segment SGI7568[] = {{&S45610[108], 3}, {NULL}};
const Segment SGI7569[] = {{&S45610[74], 5}, {NULL}};
const Segment SGI7570[] = {{&S45223[50], 10}, {NULL}};
const Segment SGI7571[] = {{&S45223[50], 21}, {NULL}};
const Segment SGI7572[] = {{&S45223[30], 12}, {NULL}};
const Segment SGI7573[] = {{&S45223[120], 3}, {NULL}};
const Segment SGI7574[] = {{&S45223[85], 6}, {NULL}};
const Segment SGI7575[] = {{&S45611[36], 7}, {NULL}};
const Segment SGI7576[] = {{&S45611[31], 4}, {NULL}};
const Segment SGI7577[] = {{&S45224[56], 8}, {NULL}};
const Segment SGI7578[] = {{&S45224[56], 20}, {NULL}};
const Segment SGI7579[] = {{&S45224[30], 18}, {NULL}};
const Segment SGI7580[] = {{&S45612[41], 7}, {NULL}};
const Segment SGI7581[] = {{&S45612[41], 18}, {NULL}};
const Segment SGI7582[] = {{&S45224[125], 3}, {NULL}};
const Segment SGI7583[] = {{&S45224[90], 6}, {NULL}};
const Segment SGI7584[] = {{&S45612[107], 3}, {NULL}};
const Segment SGI7585[] = {{&S45612[73], 5}, {NULL}};
const Segment SGI7586[] = {{&S45225[56], 8}, {NULL}};
const Segment SGI7587[] = {{&S45225[56], 20}, {NULL}};
const Segment SGI7588[] = {{&S45613[42], 6}, {NULL}};
const Segment SGI7589[] = {{&S45225[30], 18}, {NULL}};
const Segment SGI7590[] = {{&S45613[42], 17}, {NULL}};
const Segment SGI7591[] = {{&S45613[30], 4}, {NULL}};
const Segment SGI7592[] = {{&S45225[125], 3}, {NULL}};
const Segment SGI7593[] = {{&S45613[107], 3}, {NULL}};
const Segment SGI7594[] = {{&S45225[90], 6}, {NULL}};
const Segment SGI7595[] = {{&S45613[73], 5}, {NULL}};
const Segment SGI7596[] = {{&S45226[56], 14}, {NULL}};
const Segment SGI7597[] = {{&S45614[42], 7}, {NULL}};
const Segment SGI7598[] = {{&S45226[56], 26}, {NULL}};
const Segment SGI7599[] = {{&S45614[42], 18}, {NULL}};
const Segment SGI7600[] = {{&S45614[30], 4}, {NULL}};
const Segment SGI7601[] = {{&S45226[30], 18}, {NULL}};
const Segment SGI7602[] = {{&S45614[108], 3}, {NULL}};
const Segment SGI7603[] = {{&S45226[131], 3}, {NULL}};
const Segment SGI7604[] = {{&S45614[74], 5}, {NULL}};
const Segment SGI7605[] = {{&S45226[96], 6}, {NULL}};
const Segment SGI7606[] = {{&S45615[41], 7}, {NULL}};
const Segment SGI7607[] = {{&S45615[41], 18}, {NULL}};
const Segment SGI7608[] = {{&S45227[56], 8}, {NULL}};
const Segment SGI7609[] = {{&S45615[30], 3}, {NULL}};
const Segment SGI7610[] = {{&S45227[56], 20}, {NULL}};
const Segment SGI7611[] = {{&S45227[30], 18}, {NULL}};
const Segment SGI7612[] = {{&S45615[107], 3}, {NULL}};
const Segment SGI7613[] = {{&S45227[125], 3}, {NULL}};
const Segment SGI7614[] = {{&S45615[73], 5}, {NULL}};
const Segment SGI7615[] = {{&S45227[90], 6}, {NULL}};
const Segment SGI7616[] = {{&S45228[56], 9}, {NULL}};
const Segment SGI7617[] = {{&S45228[56], 21}, {NULL}};
const Segment SGI7618[] = {{&S45228[30], 18}, {NULL}};
const Segment SGI7619[] = {{&S45228[126], 3}, {NULL}};
const Segment SGI7620[] = {{&S45228[91], 6}, {NULL}};
const Segment SGI7621[] = {{&S45229[42], 32}, {NULL}};
const Segment SGI7622[] = {{&S45229[30], 4}, {NULL}};
const Segment SGI7623[] = {{&S45230[56], 8}, {NULL}};
const Segment SGI7624[] = {{&S45230[56], 20}, {NULL}};
const Segment SGI7625[] = {{&S45230[30], 18}, {NULL}};
const Segment SGI7626[] = {{&S45230[125], 3}, {NULL}};
const Segment SGI7627[] = {{&S45230[90], 6}, {NULL}};
const Segment SGI7628[] = {{&S45231[50], 7}, {NULL}};
const Segment SGI7629[] = {{&S45231[50], 18}, {NULL}};
const Segment SGI7630[] = {{&S45231[30], 12}, {NULL}};
const Segment SGI7631[] = {{&S45231[117], 3}, {NULL}};
const Segment SGI7632[] = {{&S45231[82], 6}, {NULL}};
const Segment SGI7633[] = {{&S45232[56], 7}, {NULL}};
const Segment SGI7634[] = {{&S45232[56], 19}, {NULL}};
const Segment SGI7635[] = {{&S45232[30], 18}, {NULL}};
const Segment SGI7636[] = {{&S45232[124], 3}, {NULL}};
const Segment SGI7637[] = {{&S45232[89], 6}, {NULL}};
const Segment SGI7638[] = {{&S45233[50], 9}, {NULL}};
const Segment SGI7639[] = {{&S45233[50], 21}, {NULL}};
const Segment SGI7640[] = {{&S45233[30], 12}, {NULL}};
const Segment SGI7641[] = {{&S45233[88], 4}, {NULL}};
const Segment SGI7642[] = {{&S45233[88], 3}, {NULL}};
const Segment SGI7643[] = {{&S45234[50], 13}, {NULL}};
const Segment SGI7644[] = {{&S45234[50], 24}, {NULL}};
const Segment SGI7645[] = {{&S45234[30], 12}, {NULL}};
const Segment SGI7646[] = {{&S45234[123], 3}, {NULL}};
const Segment SGI7647[] = {{&S45234[88], 6}, {NULL}};
const Segment SGI7648[] = {{&S45235[56], 7}, {NULL}};
const Segment SGI7649[] = {{&S45235[56], 19}, {NULL}};
const Segment SGI7650[] = {{&S45235[30], 18}, {NULL}};
const Segment SGI7651[] = {{&S45235[124], 3}, {NULL}};
const Segment SGI7652[] = {{&S45235[89], 6}, {NULL}};
const Segment SGI7653[] = {{&S45236[50], 7}, {NULL}};
const Segment SGI7654[] = {{&S45236[50], 19}, {NULL}};
const Segment SGI7655[] = {{&S45236[30], 12}, {NULL}};
const Segment SGI7656[] = {{&S45236[118], 3}, {NULL}};
const Segment SGI7657[] = {{&S45236[83], 6}, {NULL}};
const Segment SGI7658[] = {{&S45237[56], 6}, {NULL}};
const Segment SGI7659[] = {{&S45237[56], 18}, {NULL}};
const Segment SGI7660[] = {{&S45237[30], 18}, {NULL}};
const Segment SGI7661[] = {{&S45237[123], 3}, {NULL}};
const Segment SGI7662[] = {{&S45237[88], 6}, {NULL}};
const Segment SGI7663[] = {{&S45238[50], 18}, {NULL}};
const Segment SGI7664[] = {{&S45238[50], 29}, {NULL}};
const Segment SGI7665[] = {{&S45238[30], 12}, {NULL}};
const Segment SGI7666[] = {{&S45238[128], 3}, {NULL}};
const Segment SGI7667[] = {{&S45238[93], 6}, {NULL}};
const Segment SGI7668[] = {{&S45239[56], 8}, {NULL}};
const Segment SGI7669[] = {{&S45239[56], 20}, {NULL}};
const Segment SGI7670[] = {{&S45239[30], 18}, {NULL}};
const Segment SGI7671[] = {{&S45239[125], 3}, {NULL}};
const Segment SGI7672[] = {{&S45239[90], 6}, {NULL}};
const Segment SGI7673[] = {{&S45240[50], 4}, {NULL}};
const Segment SGI7674[] = {{&S45240[50], 18}, {NULL}};
const Segment SGI7675[] = {{&S45240[30], 12}, {NULL}};
const Segment SGI7676[] = {{&S45240[117], 3}, {NULL}};
const Segment SGI7677[] = {{&S45240[82], 6}, {NULL}};
const Segment SGI7678[] = {{&S45241[50], 9}, {NULL}};
const Segment SGI7679[] = {{&S45241[50], 28}, {NULL}};
const Segment SGI7680[] = {{&S45241[30], 12}, {NULL}};
const Segment SGI7681[] = {{&S45241[127], 3}, {NULL}};
const Segment SGI7682[] = {{&S45241[92], 6}, {NULL}};
const Segment SGI7683[] = {{&S45242[56], 9}, {NULL}};
const Segment SGI7684[] = {{&S45242[56], 21}, {NULL}};
const Segment SGI7685[] = {{&S45242[30], 18}, {NULL}};
const Segment SGI7686[] = {{&S45242[126], 3}, {NULL}};
const Segment SGI7687[] = {{&S45242[91], 6}, {NULL}};
const Segment SGI7688[] = {{&S45243[50], 9}, {NULL}};
const Segment SGI7689[] = {{&S45243[50], 21}, {NULL}};
const Segment SGI7690[] = {{&S45243[30], 12}, {NULL}};
const Segment SGI7691[] = {{&S45243[120], 3}, {NULL}};
const Segment SGI7692[] = {{&S45243[85], 6}, {NULL}};
const Segment SGI7693[] = {{&S45244[58], 8}, {NULL}};
const Segment SGI7694[] = {{&S45244[58], 20}, {NULL}};
const Segment SGI7695[] = {{&S45244[30], 20}, {NULL}};
const Segment SGI7696[] = {{&S45244[127], 3}, {NULL}};
const Segment SGI7697[] = {{&S45244[92], 6}, {NULL}};
const Segment SGI7698[] = {{&S45245[56], 8}, {NULL}};
const Segment SGI7699[] = {{&S45245[56], 20}, {NULL}};
const Segment SGI7700[] = {{&S45245[30], 18}, {NULL}};
const Segment SGI7701[] = {{&S45245[125], 3}, {NULL}};
const Segment SGI7702[] = {{&S45245[90], 6}, {NULL}};
const Segment SGI7703[] = {{&S45246[90], 3}, {NULL}};
const Segment SGI7704[] = {{&S45246[58], 6}, {NULL}};
const Segment SGI7705[] = {{&S45247[57], 10}, {NULL}};
const Segment SGI7706[] = {{&S45247[57], 22}, {NULL}};
const Segment SGI7707[] = {{&S45247[30], 19}, {NULL}};
const Segment SGI7708[] = {{&S45247[128], 3}, {NULL}};
const Segment SGI7709[] = {{&S45247[93], 6}, {NULL}};
const Segment SGI7710[] = {{&S45248[56], 8}, {NULL}};
const Segment SGI7711[] = {{&S45248[56], 20}, {NULL}};
const Segment SGI7712[] = {{&S45248[30], 18}, {NULL}};
const Segment SGI7713[] = {{&S45248[125], 3}, {NULL}};
const Segment SGI7714[] = {{&S45248[90], 6}, {NULL}};
const Segment SGI7715[] = {{&S45249[56], 7}, {NULL}};
const Segment SGI7716[] = {{&S45249[56], 19}, {NULL}};
const Segment SGI7717[] = {{&S45249[30], 18}, {NULL}};
const Segment SGI7718[] = {{&S45249[124], 3}, {NULL}};
const Segment SGI7719[] = {{&S45249[89], 6}, {NULL}};
const Segment SGI7720[] = {{&S45250[57], 10}, {NULL}};
const Segment SGI7721[] = {{&S45250[57], 22}, {NULL}};
const Segment SGI7722[] = {{&S45250[30], 19}, {NULL}};
const Segment SGI7723[] = {{&S45250[128], 3}, {NULL}};
const Segment SGI7724[] = {{&S45250[93], 6}, {NULL}};
const Segment SGI7725[] = {{&S45251[56], 9}, {NULL}};
const Segment SGI7726[] = {{&S45251[56], 21}, {NULL}};
const Segment SGI7727[] = {{&S45251[30], 18}, {NULL}};
const Segment SGI7728[] = {{&S45251[126], 3}, {NULL}};
const Segment SGI7729[] = {{&S45251[91], 6}, {NULL}};
const Segment SGI7730[] = {{&S45252[56], 9}, {NULL}};
const Segment SGI7731[] = {{&S45252[56], 21}, {NULL}};
const Segment SGI7732[] = {{&S45252[30], 18}, {NULL}};
const Segment SGI7733[] = {{&S45252[126], 3}, {NULL}};
const Segment SGI7734[] = {{&S45252[91], 6}, {NULL}};
const Segment SGI7735[] = {{&S45253[42], 23}, {NULL}};
const Segment SGI7736[] = {{&S45253[30], 4}, {NULL}};
const Segment SGI7737[] = {{&S45253[132], 3}, {NULL}};
const Segment SGI7738[] = {{&S45253[97], 6}, {NULL}};
const Segment SGI7739[] = {{&S45254[56], 8}, {NULL}};
const Segment SGI7740[] = {{&S45254[56], 20}, {NULL}};
const Segment SGI7741[] = {{&S45254[30], 18}, {NULL}};
const Segment SGI7742[] = {{&S45254[125], 3}, {NULL}};
const Segment SGI7743[] = {{&S45254[90], 6}, {NULL}};
const Segment SGI7744[] = {{&S45255[50], 11}, {NULL}};
const Segment SGI7745[] = {{&S45255[50], 22}, {NULL}};
const Segment SGI7746[] = {{&S45255[30], 12}, {NULL}};
const Segment SGI7747[] = {{&S45255[121], 3}, {NULL}};
const Segment SGI7748[] = {{&S45255[86], 6}, {NULL}};
const Segment SGI7749[] = {{&S45256[50], 15}, {NULL}};
const Segment SGI7750[] = {{&S45256[50], 27}, {NULL}};
const Segment SGI7751[] = {{&S45256[30], 12}, {NULL}};
const Segment SGI7752[] = {{&S45256[126], 3}, {NULL}};
const Segment SGI7753[] = {{&S45256[91], 6}, {NULL}};
const Segment SGI7754[] = {{&S45257[50], 10}, {NULL}};
const Segment SGI7755[] = {{&S45257[50], 21}, {NULL}};
const Segment SGI7756[] = {{&S45257[30], 12}, {NULL}};
const Segment SGI7757[] = {{&S45257[120], 3}, {NULL}};
const Segment SGI7758[] = {{&S45257[85], 6}, {NULL}};
const Segment SGI7759[] = {{&S45258[56], 9}, {NULL}};
const Segment SGI7760[] = {{&S45258[56], 21}, {NULL}};
const Segment SGI7761[] = {{&S45258[30], 18}, {NULL}};
const Segment SGI7762[] = {{&S45258[126], 3}, {NULL}};
const Segment SGI7763[] = {{&S45258[91], 6}, {NULL}};
const Segment SGI7764[] = {{&S45259[56], 8}, {NULL}};
const Segment SGI7765[] = {{&S45259[56], 20}, {NULL}};
const Segment SGI7766[] = {{&S45259[30], 18}, {NULL}};
const Segment SGI7767[] = {{&S45259[125], 3}, {NULL}};
const Segment SGI7768[] = {{&S45259[90], 6}, {NULL}};
const Segment SGI7769[] = {{&S45260[42], 32}, {NULL}};
const Segment SGI7770[] = {{&S45260[30], 4}, {NULL}};
const Segment SGI7771[] = {{&S45261[50], 7}, {NULL}};
const Segment SGI7772[] = {{&S45261[50], 18}, {NULL}};
const Segment SGI7773[] = {{&S45261[30], 12}, {NULL}};
const Segment SGI7774[] = {{&S45261[117], 3}, {NULL}};
const Segment SGI7775[] = {{&S45261[82], 6}, {NULL}};
const Segment SGI7776[] = {{&S45262[42], 6}, {NULL}};
const Segment SGI7777[] = {{&S45262[42], 17}, {NULL}};
const Segment SGI7778[] = {{&S45262[30], 4}, {NULL}};
const Segment SGI7779[] = {{&S45262[76], 4}, {NULL}};
const Segment SGI7780[] = {{&S45262[76], 3}, {NULL}};
const Segment SGI7781[] = {{&S45263[56], 8}, {NULL}};
const Segment SGI7782[] = {{&S45263[56], 20}, {NULL}};
const Segment SGI7783[] = {{&S45263[30], 18}, {NULL}};
const Segment SGI7784[] = {{&S45263[125], 3}, {NULL}};
const Segment SGI7785[] = {{&S45263[90], 6}, {NULL}};
const Segment SGI7786[] = {{&S45264[56], 10}, {NULL}};
const Segment SGI7787[] = {{&S45264[56], 22}, {NULL}};
const Segment SGI7788[] = {{&S45264[30], 18}, {NULL}};
const Segment SGI7789[] = {{&S45264[127], 3}, {NULL}};
const Segment SGI7790[] = {{&S45264[92], 6}, {NULL}};
const Segment SGI7791[] = {{&S45265[56], 12}, {NULL}};
const Segment SGI7792[] = {{&S45265[56], 24}, {NULL}};
const Segment SGI7793[] = {{&S45265[30], 18}, {NULL}};
const Segment SGI7794[] = {{&S45265[129], 3}, {NULL}};
const Segment SGI7795[] = {{&S45265[94], 6}, {NULL}};
const Segment SGI7796[] = {{&S45266[56], 16}, {NULL}};
const Segment SGI7797[] = {{&S45266[56], 28}, {NULL}};
const Segment SGI7798[] = {{&S45266[30], 18}, {NULL}};
const Segment SGI7799[] = {{&S45266[133], 3}, {NULL}};
const Segment SGI7800[] = {{&S45266[98], 6}, {NULL}};
const Segment SGI7801[] = {{&S45267[56], 11}, {NULL}};
const Segment SGI7802[] = {{&S45267[56], 23}, {NULL}};
const Segment SGI7803[] = {{&S45267[30], 18}, {NULL}};
const Segment SGI7804[] = {{&S45267[128], 3}, {NULL}};
const Segment SGI7805[] = {{&S45267[93], 6}, {NULL}};
const Segment SGI7806[] = {{&S45268[56], 10}, {NULL}};
const Segment SGI7807[] = {{&S45268[56], 22}, {NULL}};
const Segment SGI7808[] = {{&S45268[30], 18}, {NULL}};
const Segment SGI7809[] = {{&S45268[127], 3}, {NULL}};
const Segment SGI7810[] = {{&S45268[92], 6}, {NULL}};
const Segment SGI7811[] = {{&S45269[42], 18}, {NULL}};
const Segment SGI7812[] = {{&S45269[30], 4}, {NULL}};
const Segment SGI7813[] = {{&S45269[108], 3}, {NULL}};
const Segment SGI7814[] = {{&S45269[74], 6}, {NULL}};
const Segment SGI7815[] = {{&S45270[50], 4}, {NULL}};
const Segment SGI7816[] = {{&S45270[50], 16}, {NULL}};
const Segment SGI7817[] = {{&S45270[30], 12}, {NULL}};
const Segment SGI7818[] = {{&S45270[115], 3}, {NULL}};
const Segment SGI7819[] = {{&S45270[80], 6}, {NULL}};
const Segment SGI7820[] = {{&S45271[56], 9}, {NULL}};
const Segment SGI7821[] = {{&S45271[56], 21}, {NULL}};
const Segment SGI7822[] = {{&S45271[30], 18}, {NULL}};
const Segment SGI7823[] = {{&S45271[126], 3}, {NULL}};
const Segment SGI7824[] = {{&S45271[91], 6}, {NULL}};
const Segment SGI7825[] = {{&S45272[42], 32}, {NULL}};
const Segment SGI7826[] = {{&S45272[30], 4}, {NULL}};
const Segment SGI7827[] = {{&S45273[50], 7}, {NULL}};
const Segment SGI7828[] = {{&S45273[50], 19}, {NULL}};
const Segment SGI7829[] = {{&S45273[30], 12}, {NULL}};
const Segment SGI7830[] = {{&S45273[118], 3}, {NULL}};
const Segment SGI7831[] = {{&S45273[83], 6}, {NULL}};
const Segment SGI7832[] = {{&S45274[52], 6}, {NULL}};
const Segment SGI7833[] = {{&S45274[52], 18}, {NULL}};
const Segment SGI7834[] = {{&S45274[30], 14}, {NULL}};
const Segment SGI7835[] = {{&S45274[119], 3}, {NULL}};
const Segment SGI7836[] = {{&S45274[84], 6}, {NULL}};
const Segment SGI7837[] = {{&S45276[56], 8}, {NULL}};
const Segment SGI7838[] = {{&S45276[56], 20}, {NULL}};
const Segment SGI7839[] = {{&S45276[30], 18}, {NULL}};
const Segment SGI7840[] = {{&S45276[125], 3}, {NULL}};
const Segment SGI7841[] = {{&S45276[90], 6}, {NULL}};
const Segment SGI7842[] = {{&S45277[42], 18}, {NULL}};
const Segment SGI7843[] = {{&S45277[42], 28}, {NULL}};
const Segment SGI7844[] = {{&S45277[30], 4}, {NULL}};
const Segment SGI7845[] = {{&S45277[119], 3}, {NULL}};
const Segment SGI7846[] = {{&S45277[84], 6}, {NULL}};
const Segment SGI7847[] = {{&S45278[56], 11}, {NULL}};
const Segment SGI7848[] = {{&S45278[56], 23}, {NULL}};
const Segment SGI7849[] = {{&S45278[30], 18}, {NULL}};
const Segment SGI7850[] = {{&S45278[128], 3}, {NULL}};
const Segment SGI7851[] = {{&S45278[93], 6}, {NULL}};
const Segment SGI7852[] = {{&S45279[67], 43}, {NULL}};
const Segment SGI7853[] = {{&S45279[67], 55}, {NULL}};
const Segment SGI7854[] = {{&S45279[30], 29}, {NULL}};
const Segment SGI7855[] = {{&S45279[171], 3}, {NULL}};
const Segment SGI7856[] = {{&S45279[136], 6}, {NULL}};
const Segment SGI7857[] = {{&S45280[50], 13}, {NULL}};
const Segment SGI7858[] = {{&S45280[50], 24}, {NULL}};
const Segment SGI7859[] = {{&S45280[30], 12}, {NULL}};
const Segment SGI7860[] = {{&S45280[123], 3}, {NULL}};
const Segment SGI7861[] = {{&S45280[88], 6}, {NULL}};
const Segment SGI7862[] = {{&S45281[56], 10}, {NULL}};
const Segment SGI7863[] = {{&S45281[56], 22}, {NULL}};
const Segment SGI7864[] = {{&S45281[30], 18}, {NULL}};
const Segment SGI7865[] = {{&S45281[127], 3}, {NULL}};
const Segment SGI7866[] = {{&S45281[92], 6}, {NULL}};
const Segment SGI7867[] = {{&S45282[48], 31}, {NULL}};
const Segment SGI7868[] = {{&S45282[145], 3}, {NULL}};
const Segment SGI7869[] = {{&S45282[110], 6}, {NULL}};
const Segment SGI7870[] = {{&S45283[50], 6}, {NULL}};
const Segment SGI7871[] = {{&S45283[50], 25}, {NULL}};
const Segment SGI7872[] = {{&S45283[30], 12}, {NULL}};
const Segment SGI7873[] = {{&S45283[124], 3}, {NULL}};
const Segment SGI7874[] = {{&S45283[89], 6}, {NULL}};
const Segment SGI7875[] = {{&S45284[42], 28}, {NULL}};
const Segment SGI7876[] = {{&S45284[30], 4}, {NULL}};
const Segment SGI7877[] = {{&S45284[137], 3}, {NULL}};
const Segment SGI7878[] = {{&S45284[102], 6}, {NULL}};
const Segment SGI7879[] = {{&S45285[50], 12}, {NULL}};
const Segment SGI7880[] = {{&S45285[50], 32}, {NULL}};
const Segment SGI7881[] = {{&S45285[30], 12}, {NULL}};
const Segment SGI7882[] = {{&S45285[131], 3}, {NULL}};
const Segment SGI7883[] = {{&S45285[96], 6}, {NULL}};
const Segment SGI7884[] = {{&S45286[56], 8}, {NULL}};
const Segment SGI7885[] = {{&S45286[56], 20}, {NULL}};
const Segment SGI7886[] = {{&S45286[30], 18}, {NULL}};
const Segment SGI7887[] = {{&S45286[125], 3}, {NULL}};
const Segment SGI7888[] = {{&S45286[90], 6}, {NULL}};
const Segment SGI7889[] = {{&S45287[39], 22}, {NULL}};
const Segment SGI7890[] = {{&S45287[125], 3}, {NULL}};
const Segment SGI7891[] = {{&S45287[91], 6}, {NULL}};
const Segment SGI7892[] = {{&S45288[56], 8}, {NULL}};
const Segment SGI7893[] = {{&S45288[56], 20}, {NULL}};
const Segment SGI7894[] = {{&S45288[30], 18}, {NULL}};
const Segment SGI7895[] = {{&S45288[125], 3}, {NULL}};
const Segment SGI7896[] = {{&S45288[90], 6}, {NULL}};
const Segment SGI7897[] = {{&S45289[56], 11}, {NULL}};
const Segment SGI7898[] = {{&S45289[56], 23}, {NULL}};
const Segment SGI7899[] = {{&S45289[30], 18}, {NULL}};
const Segment SGI7900[] = {{&S45289[128], 3}, {NULL}};
const Segment SGI7901[] = {{&S45289[93], 6}, {NULL}};
const Segment SGI7902[] = {{&S45290[56], 11}, {NULL}};
const Segment SGI7903[] = {{&S45290[56], 23}, {NULL}};
const Segment SGI7904[] = {{&S45290[30], 18}, {NULL}};
const Segment SGI7905[] = {{&S45290[128], 3}, {NULL}};
const Segment SGI7906[] = {{&S45290[93], 6}, {NULL}};
const Segment SGI7907[] = {{&S45291[56], 8}, {NULL}};
const Segment SGI7908[] = {{&S45291[56], 20}, {NULL}};
const Segment SGI7909[] = {{&S45291[30], 18}, {NULL}};
const Segment SGI7910[] = {{&S45291[125], 3}, {NULL}};
const Segment SGI7911[] = {{&S45291[90], 6}, {NULL}};
const Segment SGI7912[] = {{&S45292[42], 27}, {NULL}};
const Segment SGI7913[] = {{&S45292[30], 4}, {NULL}};
const Segment SGI7914[] = {{&S45292[136], 3}, {NULL}};
const Segment SGI7915[] = {{&S45292[101], 6}, {NULL}};
const Segment SGI7916[] = {{&S45293[56], 5}, {NULL}};
const Segment SGI7917[] = {{&S45293[56], 17}, {NULL}};
const Segment SGI7918[] = {{&S45293[30], 18}, {NULL}};
const Segment SGI7919[] = {{&S45293[122], 3}, {NULL}};
const Segment SGI7920[] = {{&S45293[87], 6}, {NULL}};
const Segment SGI7921[] = {{&S45294[78], 9}, {NULL}};
const Segment SGI7922[] = {{&S45294[78], 21}, {NULL}};
const Segment SGI7923[] = {{&S45294[30], 40}, {NULL}};
const Segment SGI7924[] = {{&S45294[148], 3}, {NULL}};
const Segment SGI7925[] = {{&S45294[113], 6}, {NULL}};
const Segment SGI7926[] = {{&S45295[56], 9}, {NULL}};
const Segment SGI7927[] = {{&S45295[56], 21}, {NULL}};
const Segment SGI7928[] = {{&S45295[30], 18}, {NULL}};
const Segment SGI7929[] = {{&S45295[126], 3}, {NULL}};
const Segment SGI7930[] = {{&S45295[91], 6}, {NULL}};
const Segment SGI7931[] = {{&S45296[42], 30}, {NULL}};
const Segment SGI7932[] = {{&S45296[30], 4}, {NULL}};
const Segment SGI7933[] = {{&S45296[139], 3}, {NULL}};
const Segment SGI7934[] = {{&S45296[104], 6}, {NULL}};
const Segment SGI7935[] = {{&S45297[56], 8}, {NULL}};
const Segment SGI7936[] = {{&S45297[56], 20}, {NULL}};
const Segment SGI7937[] = {{&S45297[30], 18}, {NULL}};
const Segment SGI7938[] = {{&S45297[125], 3}, {NULL}};
const Segment SGI7939[] = {{&S45297[90], 6}, {NULL}};
const Segment SGI7940[] = {{&S45298[50], 7}, {NULL}};
const Segment SGI7941[] = {{&S45298[50], 26}, {NULL}};
const Segment SGI7942[] = {{&S45298[30], 12}, {NULL}};
const Segment SGI7943[] = {{&S45298[93], 4}, {NULL}};
const Segment SGI7944[] = {{&S45298[93], 3}, {NULL}};
const Segment SGI7945[] = {{&S45299[57], 11}, {NULL}};
const Segment SGI7946[] = {{&S45299[57], 23}, {NULL}};
const Segment SGI7947[] = {{&S45299[30], 19}, {NULL}};
const Segment SGI7948[] = {{&S45299[129], 3}, {NULL}};
const Segment SGI7949[] = {{&S45299[94], 6}, {NULL}};
const Segment SGI7950[] = {{&S45300[56], 9}, {NULL}};
const Segment SGI7951[] = {{&S45300[56], 21}, {NULL}};
const Segment SGI7952[] = {{&S45300[30], 18}, {NULL}};
const Segment SGI7953[] = {{&S45300[126], 3}, {NULL}};
const Segment SGI7954[] = {{&S45300[91], 6}, {NULL}};
const Segment SGI7955[] = {{&S45301[56], 7}, {NULL}};
const Segment SGI7956[] = {{&S45301[56], 19}, {NULL}};
const Segment SGI7957[] = {{&S45301[30], 18}, {NULL}};
const Segment SGI7958[] = {{&S45301[124], 3}, {NULL}};
const Segment SGI7959[] = {{&S45301[89], 6}, {NULL}};
const Segment SGI7960[] = {{&S45302[56], 9}, {NULL}};
const Segment SGI7961[] = {{&S45302[56], 21}, {NULL}};
const Segment SGI7962[] = {{&S45302[30], 18}, {NULL}};
const Segment SGI7963[] = {{&S45302[126], 3}, {NULL}};
const Segment SGI7964[] = {{&S45302[91], 6}, {NULL}};
const Segment SGI7965[] = {{&S45303[56], 11}, {NULL}};
const Segment SGI7966[] = {{&S45303[56], 23}, {NULL}};
const Segment SGI7967[] = {{&S45303[30], 18}, {NULL}};
const Segment SGI7968[] = {{&S45303[128], 3}, {NULL}};
const Segment SGI7969[] = {{&S45303[93], 6}, {NULL}};
const Segment SGI7970[] = {{&S45304[50], 13}, {NULL}};
const Segment SGI7971[] = {{&S45304[50], 25}, {NULL}};
const Segment SGI7972[] = {{&S45304[30], 12}, {NULL}};
const Segment SGI7973[] = {{&S45304[124], 3}, {NULL}};
const Segment SGI7974[] = {{&S45304[89], 6}, {NULL}};
const Segment SGI7975[] = {{&S45442[44], 19}, {NULL}};
const Segment SGI7976[] = {{&S45442[44], 31}, {NULL}};
const Segment SGI7977[] = {{&S45442[30], 6}, {NULL}};
const Segment SGI7978[] = {{&S45305[42], 23}, {NULL}};
const Segment SGI7979[] = {{&S45305[30], 4}, {NULL}};
const Segment SGI7980[] = {{&S45442[123], 3}, {NULL}};
const Segment SGI7981[] = {{&S45442[89], 5}, {NULL}};
const Segment SGI7982[] = {{&S45305[132], 3}, {NULL}};
const Segment SGI7983[] = {{&S45305[97], 6}, {NULL}};
const Segment SGI7984[] = {{&S45443[44], 18}, {NULL}};
const Segment SGI7985[] = {{&S45443[44], 30}, {NULL}};
const Segment SGI7986[] = {{&S45443[30], 6}, {NULL}};
const Segment SGI7987[] = {{&S45306[50], 16}, {NULL}};
const Segment SGI7988[] = {{&S45306[50], 28}, {NULL}};
const Segment SGI7989[] = {{&S45306[30], 12}, {NULL}};
const Segment SGI7990[] = {{&S45443[122], 3}, {NULL}};
const Segment SGI7991[] = {{&S45443[88], 5}, {NULL}};
const Segment SGI7992[] = {{&S45306[127], 3}, {NULL}};
const Segment SGI7993[] = {{&S45306[92], 6}, {NULL}};
const Segment SGI7994[] = {{&S45444[44], 19}, {NULL}};
const Segment SGI7995[] = {{&S45444[44], 31}, {NULL}};
const Segment SGI7996[] = {{&S45444[30], 6}, {NULL}};
const Segment SGI7997[] = {{&S45307[57], 9}, {NULL}};
const Segment SGI7998[] = {{&S45307[57], 21}, {NULL}};
const Segment SGI7999[] = {{&S45307[30], 19}, {NULL}};
const Segment SGI8000[] = {{&S45444[123], 3}, {NULL}};
const Segment SGI8001[] = {{&S45444[89], 5}, {NULL}};
const Segment SGI8002[] = {{&S45307[127], 3}, {NULL}};
const Segment SGI8003[] = {{&S45307[92], 6}, {NULL}};
const Segment SGI8004[] = {{&S45445[43], 19}, {NULL}};
const Segment SGI8005[] = {{&S45445[43], 31}, {NULL}};
const Segment SGI8006[] = {{&S45308[56], 5}, {NULL}};
const Segment SGI8007[] = {{&S45308[56], 17}, {NULL}};
const Segment SGI8008[] = {{&S45308[30], 18}, {NULL}};
const Segment SGI8009[] = {{&S45445[122], 3}, {NULL}};
const Segment SGI8010[] = {{&S45445[88], 5}, {NULL}};
const Segment SGI8011[] = {{&S45308[122], 3}, {NULL}};
const Segment SGI8012[] = {{&S45308[87], 6}, {NULL}};
const Segment SGI8013[] = {{&S45446[43], 19}, {NULL}};
const Segment SGI8014[] = {{&S45446[43], 31}, {NULL}};
const Segment SGI8015[] = {{&S45309[50], 8}, {NULL}};
const Segment SGI8016[] = {{&S45446[30], 5}, {NULL}};
const Segment SGI8017[] = {{&S45309[50], 20}, {NULL}};
const Segment SGI8018[] = {{&S45309[30], 12}, {NULL}};
const Segment SGI8019[] = {{&S45446[122], 3}, {NULL}};
const Segment SGI8020[] = {{&S45446[88], 5}, {NULL}};
const Segment SGI8021[] = {{&S45309[87], 4}, {NULL}};
const Segment SGI8022[] = {{&S45309[87], 3}, {NULL}};
const Segment SGI8023[] = {{&S45310[50], 12}, {NULL}};
const Segment SGI8024[] = {{&S45447[44], 18}, {NULL}};
const Segment SGI8025[] = {{&S45447[44], 30}, {NULL}};
const Segment SGI8026[] = {{&S45310[50], 23}, {NULL}};
const Segment SGI8027[] = {{&S45447[30], 6}, {NULL}};
const Segment SGI8028[] = {{&S45310[30], 12}, {NULL}};
const Segment SGI8029[] = {{&S45447[122], 3}, {NULL}};
const Segment SGI8030[] = {{&S45310[122], 3}, {NULL}};
const Segment SGI8031[] = {{&S45447[88], 5}, {NULL}};
const Segment SGI8032[] = {{&S45310[87], 6}, {NULL}};
const Segment SGI8033[] = {{&S45311[56], 9}, {NULL}};
const Segment SGI8034[] = {{&S45311[56], 21}, {NULL}};
const Segment SGI8035[] = {{&S45311[30], 18}, {NULL}};
const Segment SGI8036[] = {{&S45311[126], 3}, {NULL}};
const Segment SGI8037[] = {{&S45311[91], 6}, {NULL}};
const Segment SGI8038[] = {{&S45312[56], 9}, {NULL}};
const Segment SGI8039[] = {{&S45312[56], 21}, {NULL}};
const Segment SGI8040[] = {{&S45312[30], 18}, {NULL}};
const Segment SGI8041[] = {{&S45312[126], 3}, {NULL}};
const Segment SGI8042[] = {{&S45312[91], 6}, {NULL}};
const Segment SGI8043[] = {{&S45313[56], 8}, {NULL}};
const Segment SGI8044[] = {{&S45313[56], 20}, {NULL}};
const Segment SGI8045[] = {{&S45313[30], 18}, {NULL}};
const Segment SGI8046[] = {{&S45313[125], 3}, {NULL}};
const Segment SGI8047[] = {{&S45313[90], 6}, {NULL}};
const Segment SGI8048[] = {{&S45314[56], 7}, {NULL}};
const Segment SGI8049[] = {{&S45314[56], 19}, {NULL}};
const Segment SGI8050[] = {{&S45314[30], 18}, {NULL}};
const Segment SGI8051[] = {{&S45314[124], 3}, {NULL}};
const Segment SGI8052[] = {{&S45314[89], 6}, {NULL}};
const Segment SGI8053[] = {{&S45315[77], 3}, {NULL}};
const Segment SGI8054[] = {{&S45315[45], 3}, {NULL}};
const Segment SGI8055[] = {{&S45315[95], 5}, {NULL}};
const Segment SGI8056[] = {{&S45316[56], 8}, {NULL}};
const Segment SGI8057[] = {{&S45316[56], 20}, {NULL}};
const Segment SGI8058[] = {{&S45316[30], 18}, {NULL}};
const Segment SGI8059[] = {{&S45316[125], 3}, {NULL}};
const Segment SGI8060[] = {{&S45316[90], 6}, {NULL}};
const Segment SGI8061[] = {{&S45317[56], 9}, {NULL}};
const Segment SGI8062[] = {{&S45317[56], 21}, {NULL}};
const Segment SGI8063[] = {{&S45317[30], 18}, {NULL}};
const Segment SGI8064[] = {{&S45317[126], 3}, {NULL}};
const Segment SGI8065[] = {{&S45317[91], 6}, {NULL}};
const Segment SGI8066[] = {{&S45318[56], 8}, {NULL}};
const Segment SGI8067[] = {{&S45318[56], 20}, {NULL}};
const Segment SGI8068[] = {{&S45318[30], 18}, {NULL}};
const Segment SGI8069[] = {{&S45318[125], 3}, {NULL}};
const Segment SGI8070[] = {{&S45318[90], 6}, {NULL}};
const Segment SGI8071[] = {{&S45319[50], 18}, {NULL}};
const Segment SGI8072[] = {{&S45319[30], 12}, {NULL}};
const Segment SGI8073[] = {{&S45319[129], 3}, {NULL}};
const Segment SGI8074[] = {{&S45319[95], 6}, {NULL}};
const Segment SGI8075[] = {{&S45320[42], 4}, {NULL}};
const Segment SGI8076[] = {{&S45320[42], 15}, {NULL}};
const Segment SGI8077[] = {{&S45320[30], 4}, {NULL}};
const Segment SGI8078[] = {{&S45320[105], 3}, {NULL}};
const Segment SGI8079[] = {{&S45320[71], 5}, {NULL}};
const Segment SGI8080[] = {{&S45321[50], 5}, {NULL}};
const Segment SGI8081[] = {{&S45321[50], 20}, {NULL}};
const Segment SGI8082[] = {{&S45321[30], 12}, {NULL}};
const Segment SGI8083[] = {{&S45321[119], 3}, {NULL}};
const Segment SGI8084[] = {{&S45321[84], 6}, {NULL}};
const Segment SGI8085[] = {{&S45490[44], 10}, {NULL}};
const Segment SGI8086[] = {{&S45490[44], 21}, {NULL}};
const Segment SGI8087[] = {{&S45490[30], 6}, {NULL}};
const Segment SGI8088[] = {{&S45490[113], 3}, {NULL}};
const Segment SGI8089[] = {{&S45490[79], 5}, {NULL}};
const Segment SGI8090[] = {{&S45491[44], 11}, {NULL}};
const Segment SGI8091[] = {{&S45491[44], 23}, {NULL}};
const Segment SGI8092[] = {{&S45491[30], 6}, {NULL}};
const Segment SGI8093[] = {{&S45491[115], 3}, {NULL}};
const Segment SGI8094[] = {{&S45491[81], 5}, {NULL}};
const Segment SGI8095[] = {{&S45492[44], 11}, {NULL}};
const Segment SGI8096[] = {{&S45492[44], 22}, {NULL}};
const Segment SGI8097[] = {{&S45492[30], 6}, {NULL}};
const Segment SGI8098[] = {{&S45492[114], 3}, {NULL}};
const Segment SGI8099[] = {{&S45492[80], 5}, {NULL}};
const Segment SGI8100[] = {{&S45493[44], 10}, {NULL}};
const Segment SGI8101[] = {{&S45493[44], 21}, {NULL}};
const Segment SGI8102[] = {{&S45493[30], 6}, {NULL}};
const Segment SGI8103[] = {{&S45493[113], 3}, {NULL}};
const Segment SGI8104[] = {{&S45493[79], 5}, {NULL}};
const Segment SGI8105[] = {{&S45495[43], 11}, {NULL}};
const Segment SGI8106[] = {{&S45495[43], 22}, {NULL}};
const Segment SGI8107[] = {{&S45495[113], 3}, {NULL}};
const Segment SGI8108[] = {{&S45495[79], 5}, {NULL}};
const Segment SGI8109[] = {{&S45496[43], 11}, {NULL}};
const Segment SGI8110[] = {{&S45496[43], 22}, {NULL}};
const Segment SGI8111[] = {{&S45496[30], 5}, {NULL}};
const Segment SGI8112[] = {{&S45496[113], 3}, {NULL}};
const Segment SGI8113[] = {{&S45496[79], 5}, {NULL}};
const Segment SGI8114[] = {{&S45499[44], 11}, {NULL}};
const Segment SGI8115[] = {{&S45499[44], 22}, {NULL}};
const Segment SGI8116[] = {{&S45499[30], 6}, {NULL}};
const Segment SGI8117[] = {{&S45499[114], 3}, {NULL}};
const Segment SGI8118[] = {{&S45499[80], 5}, {NULL}};
const Segment SGI8119[] = {{&S45503[44], 10}, {NULL}};
const Segment SGI8120[] = {{&S45503[44], 21}, {NULL}};
const Segment SGI8121[] = {{&S45503[30], 6}, {NULL}};
const Segment SGI8122[] = {{&S45503[113], 3}, {NULL}};
const Segment SGI8123[] = {{&S45503[79], 5}, {NULL}};
const Segment SGI8124[] = {{&S45872[42], 7}, {NULL}};
const Segment SGI8125[] = {{&S45872[42], 23}, {NULL}};
const Segment SGI8126[] = {{&S45872[30], 4}, {NULL}};
const Segment SGI8127[] = {{&S45872[113], 3}, {NULL}};
const Segment SGI8128[] = {{&S45872[79], 5}, {NULL}};
const Segment SGI8129[] = {{&S45873[41], 7}, {NULL}};
const Segment SGI8130[] = {{&S45873[41], 23}, {NULL}};
const Segment SGI8131[] = {{&S45873[30], 3}, {NULL}};
const Segment SGI8132[] = {{&S45873[112], 3}, {NULL}};
const Segment SGI8133[] = {{&S45873[78], 5}, {NULL}};
const Segment SGI8134[] = {{&S45874[42], 7}, {NULL}};
const Segment SGI8135[] = {{&S45874[42], 18}, {NULL}};
const Segment SGI8136[] = {{&S45874[30], 4}, {NULL}};
const Segment SGI8137[] = {{&S45874[108], 3}, {NULL}};
const Segment SGI8138[] = {{&S45874[74], 5}, {NULL}};
const Segment SGI8139[] = {{&S45875[42], 6}, {NULL}};
const Segment SGI8140[] = {{&S45875[42], 22}, {NULL}};
const Segment SGI8141[] = {{&S45875[30], 4}, {NULL}};
const Segment SGI8142[] = {{&S45875[112], 3}, {NULL}};
const Segment SGI8143[] = {{&S45875[78], 5}, {NULL}};
const Segment SGI8144[] = {{&S45877[42], 7}, {NULL}};
const Segment SGI8145[] = {{&S45877[42], 17}, {NULL}};
const Segment SGI8146[] = {{&S45877[30], 4}, {NULL}};
const Segment SGI8147[] = {{&S45877[107], 3}, {NULL}};
const Segment SGI8148[] = {{&S45877[73], 5}, {NULL}};
const Segment SGI8149[] = {{&S45878[42], 7}, {NULL}};
const Segment SGI8150[] = {{&S45878[42], 23}, {NULL}};
const Segment SGI8151[] = {{&S45878[30], 4}, {NULL}};
const Segment SGI8152[] = {{&S45878[113], 3}, {NULL}};
const Segment SGI8153[] = {{&S45878[79], 5}, {NULL}};
const Segment SGI8154[] = {{&S45880[42], 7}, {NULL}};
const Segment SGI8155[] = {{&S45880[42], 19}, {NULL}};
const Segment SGI8156[] = {{&S45880[30], 4}, {NULL}};
const Segment SGI8157[] = {{&S45880[109], 3}, {NULL}};
const Segment SGI8158[] = {{&S45880[75], 5}, {NULL}};
const Segment SGI8159[] = {{&S45881[42], 7}, {NULL}};
const Segment SGI8160[] = {{&S45881[42], 23}, {NULL}};
const Segment SGI8161[] = {{&S45881[30], 4}, {NULL}};
const Segment SGI8162[] = {{&S45881[113], 3}, {NULL}};
const Segment SGI8163[] = {{&S45881[79], 5}, {NULL}};
const Segment SGI8164[] = {{&S45882[42], 7}, {NULL}};
const Segment SGI8165[] = {{&S45882[42], 19}, {NULL}};
const Segment SGI8166[] = {{&S45882[30], 4}, {NULL}};
const Segment SGI8167[] = {{&S45882[109], 3}, {NULL}};
const Segment SGI8168[] = {{&S45882[75], 5}, {NULL}};
const Segment SGI8169[] = {{&S45884[42], 7}, {NULL}};
const Segment SGI8170[] = {{&S45884[42], 23}, {NULL}};
const Segment SGI8171[] = {{&S45884[30], 4}, {NULL}};
const Segment SGI8172[] = {{&S45884[113], 3}, {NULL}};
const Segment SGI8173[] = {{&S45884[79], 5}, {NULL}};
const Segment SGI8174[] = {{&S45886[42], 7}, {NULL}};
const Segment SGI8175[] = {{&S45886[42], 18}, {NULL}};
const Segment SGI8176[] = {{&S45886[30], 4}, {NULL}};
const Segment SGI8177[] = {{&S45886[108], 3}, {NULL}};
const Segment SGI8178[] = {{&S45886[74], 5}, {NULL}};
const Segment SGI8179[] = {{&S45887[42], 7}, {NULL}};
const Segment SGI8180[] = {{&S45887[42], 17}, {NULL}};
const Segment SGI8181[] = {{&S45887[30], 4}, {NULL}};
const Segment SGI8182[] = {{&S45887[107], 3}, {NULL}};
const Segment SGI8183[] = {{&S45887[73], 5}, {NULL}};
const Segment SGI8184[] = {{&S45889[42], 7}, {NULL}};
const Segment SGI8185[] = {{&S45889[42], 23}, {NULL}};
const Segment SGI8186[] = {{&S45889[30], 4}, {NULL}};
const Segment SGI8187[] = {{&S45889[113], 3}, {NULL}};
const Segment SGI8188[] = {{&S45889[79], 5}, {NULL}};
const Segment SGI8189[] = {{&S45891[42], 7}, {NULL}};
const Segment SGI8190[] = {{&S45891[42], 17}, {NULL}};
const Segment SGI8191[] = {{&S45891[30], 4}, {NULL}};
const Segment SGI8192[] = {{&S45891[107], 3}, {NULL}};
const Segment SGI8193[] = {{&S45891[73], 5}, {NULL}};
const Segment SGI8194[] = {{&S45896[42], 7}, {NULL}};
const Segment SGI8195[] = {{&S45896[42], 23}, {NULL}};
const Segment SGI8196[] = {{&S45896[30], 4}, {NULL}};
const Segment SGI8197[] = {{&S45896[113], 3}, {NULL}};
const Segment SGI8198[] = {{&S45896[79], 5}, {NULL}};
const Segment SGI8199[] = {{&S45897[42], 7}, {NULL}};
const Segment SGI8200[] = {{&S45897[42], 17}, {NULL}};
const Segment SGI8201[] = {{&S45897[30], 4}, {NULL}};
const Segment SGI8202[] = {{&S45897[107], 3}, {NULL}};
const Segment SGI8203[] = {{&S45897[73], 5}, {NULL}};
const Segment SGI8204[] = {{&S45900[36], 7}, {NULL}};
const Segment SGI8205[] = {{&S45900[31], 4}, {NULL}};
const Segment SGI8206[] = {{&S45572[44], 12}, {NULL}};
const Segment SGI8207[] = {{&S45572[44], 23}, {NULL}};
const Segment SGI8208[] = {{&S45572[30], 6}, {NULL}};
const Segment SGI8209[] = {{&S45572[115], 3}, {NULL}};
const Segment SGI8210[] = {{&S45572[81], 5}, {NULL}};
const Segment SGI8211[] = {{&S45574[44], 10}, {NULL}};
const Segment SGI8212[] = {{&S45574[44], 21}, {NULL}};
const Segment SGI8213[] = {{&S45574[30], 6}, {NULL}};
const Segment SGI8214[] = {{&S45574[113], 3}, {NULL}};
const Segment SGI8215[] = {{&S45574[79], 5}, {NULL}};
const Segment SGI8216[] = {{&S45576[42], 10}, {NULL}};
const Segment SGI8217[] = {{&S45576[30], 6}, {NULL}};
const Segment SGI8218[] = {{&S45576[113], 3}, {NULL}};
const Segment SGI8219[] = {{&S45576[77], 7}, {NULL}};
const Segment SGI8220[] = {{&S45576[133], 8}, {NULL}};
const Segment SGI8221[] = {{&S51543[0], 84}, {NULL}};
const Segment SGI8222[] = {{&S51544[0], 84}, {NULL}};
const Segment SGI8223[] = {{&S46245[11], 1}, {NULL}};
const Segment SGI8224[] = {{&S46245[42], 13}, {NULL}};
const Segment SGI8225[] = {{&S46245[30], 4}, {NULL}};
const Segment SGI8226[] = {{&S46245[116], 3}, {NULL}};
const Segment SGI8227[] = {{&S46245[82], 5}, {NULL}};
const Segment SGI8228[] = {{&S46247[42], 9}, {NULL}};
const Segment SGI8229[] = {{&S46247[42], 20}, {NULL}};
const Segment SGI8230[] = {{&S46247[30], 4}, {NULL}};
const Segment SGI8231[] = {{&S46247[110], 3}, {NULL}};
const Segment SGI8232[] = {{&S46247[76], 5}, {NULL}};
const Segment SGI8233[] = {{&S46248[42], 10}, {NULL}};
const Segment SGI8234[] = {{&S46248[42], 21}, {NULL}};
const Segment SGI8235[] = {{&S46248[30], 4}, {NULL}};
const Segment SGI8236[] = {{&S46248[111], 3}, {NULL}};
const Segment SGI8237[] = {{&S46248[77], 5}, {NULL}};
const Segment SGI8238[] = {{&S46250[36], 10}, {NULL}};
const Segment SGI8239[] = {{&S46250[31], 4}, {NULL}};
const Segment SGI8240[] = {{&S46251[36], 9}, {NULL}};
const Segment SGI8241[] = {{&S46251[31], 4}, {NULL}};
const Segment SGI8242[] = {{&S45979[50], 17}, {NULL}};
const Segment SGI8243[] = {{&S45979[50], 37}, {NULL}};
const Segment SGI8244[] = {{&S45979[30], 12}, {NULL}};
const Segment SGI8245[] = {{&S45979[136], 3}, {NULL}};
const Segment SGI8246[] = {{&S45979[101], 6}, {NULL}};
const Segment SGI8247[] = {{&S45980[50], 17}, {NULL}};
const Segment SGI8248[] = {{&S45980[50], 44}, {NULL}};
const Segment SGI8249[] = {{&S45980[30], 12}, {NULL}};
const Segment SGI8250[] = {{&S45980[143], 3}, {NULL}};
const Segment SGI8251[] = {{&S45980[108], 6}, {NULL}};
const Segment SGI8252[] = {{&S45981[50], 17}, {NULL}};
const Segment SGI8253[] = {{&S45981[50], 37}, {NULL}};
const Segment SGI8254[] = {{&S45981[30], 12}, {NULL}};
const Segment SGI8255[] = {{&S45981[136], 3}, {NULL}};
const Segment SGI8256[] = {{&S45981[101], 6}, {NULL}};
const Segment SGI8257[] = {{&S45982[50], 17}, {NULL}};
const Segment SGI8258[] = {{&S45982[50], 37}, {NULL}};
const Segment SGI8259[] = {{&S45982[30], 12}, {NULL}};
const Segment SGI8260[] = {{&S45982[136], 3}, {NULL}};
const Segment SGI8261[] = {{&S45982[101], 6}, {NULL}};
const Segment SGI8262[] = {{&S45983[50], 16}, {NULL}};
const Segment SGI8263[] = {{&S45983[50], 36}, {NULL}};
const Segment SGI8264[] = {{&S45983[30], 12}, {NULL}};
const Segment SGI8265[] = {{&S45983[135], 3}, {NULL}};
const Segment SGI8266[] = {{&S45983[100], 6}, {NULL}};
const Segment SGI8267[] = {{&S45984[50], 16}, {NULL}};
const Segment SGI8268[] = {{&S45984[50], 36}, {NULL}};
const Segment SGI8269[] = {{&S45984[30], 12}, {NULL}};
const Segment SGI8270[] = {{&S45984[135], 3}, {NULL}};
const Segment SGI8271[] = {{&S45984[100], 6}, {NULL}};
const Segment SGI8272[] = {{&S45985[50], 17}, {NULL}};
const Segment SGI8273[] = {{&S45985[50], 37}, {NULL}};
const Segment SGI8274[] = {{&S45985[30], 12}, {NULL}};
const Segment SGI8275[] = {{&S45985[136], 3}, {NULL}};
const Segment SGI8276[] = {{&S45985[101], 6}, {NULL}};
const Segment SGI8277[] = {{&S45986[50], 17}, {NULL}};
const Segment SGI8278[] = {{&S45986[50], 37}, {NULL}};
const Segment SGI8279[] = {{&S45986[30], 12}, {NULL}};
const Segment SGI8280[] = {{&S45986[136], 3}, {NULL}};
const Segment SGI8281[] = {{&S45986[101], 6}, {NULL}};
const Segment SGI8282[] = {{&S45987[65], 17}, {NULL}};
const Segment SGI8283[] = {{&S45987[65], 37}, {NULL}};
const Segment SGI8284[] = {{&S45987[30], 27}, {NULL}};
const Segment SGI8285[] = {{&S45987[151], 3}, {NULL}};
const Segment SGI8286[] = {{&S45987[116], 6}, {NULL}};
const Segment SGI8287[] = {{&S45988[50], 17}, {NULL}};
const Segment SGI8288[] = {{&S45988[50], 38}, {NULL}};
const Segment SGI8289[] = {{&S45988[30], 12}, {NULL}};
const Segment SGI8290[] = {{&S45988[137], 3}, {NULL}};
const Segment SGI8291[] = {{&S45988[102], 6}, {NULL}};
const Segment SGI8292[] = {{&S45989[50], 17}, {NULL}};
const Segment SGI8293[] = {{&S45989[50], 37}, {NULL}};
const Segment SGI8294[] = {{&S45989[30], 12}, {NULL}};
const Segment SGI8295[] = {{&S45989[136], 3}, {NULL}};
const Segment SGI8296[] = {{&S45989[101], 6}, {NULL}};
const Segment SGI8297[] = {{&S45990[50], 17}, {NULL}};
const Segment SGI8298[] = {{&S45990[50], 37}, {NULL}};
const Segment SGI8299[] = {{&S45990[30], 12}, {NULL}};
const Segment SGI8300[] = {{&S45990[136], 3}, {NULL}};
const Segment SGI8301[] = {{&S45990[101], 6}, {NULL}};
const Segment SGI8302[] = {{&S45991[50], 17}, {NULL}};
const Segment SGI8303[] = {{&S45991[50], 37}, {NULL}};
const Segment SGI8304[] = {{&S45991[30], 12}, {NULL}};
const Segment SGI8305[] = {{&S45991[136], 3}, {NULL}};
const Segment SGI8306[] = {{&S45991[101], 6}, {NULL}};
const Segment SGI8307[] = {{&S45992[50], 17}, {NULL}};
const Segment SGI8308[] = {{&S45992[50], 37}, {NULL}};
const Segment SGI8309[] = {{&S45992[30], 12}, {NULL}};
const Segment SGI8310[] = {{&S45992[136], 3}, {NULL}};
const Segment SGI8311[] = {{&S45992[101], 6}, {NULL}};
const Segment SGI8312[] = {{&S45993[50], 17}, {NULL}};
const Segment SGI8313[] = {{&S45993[50], 37}, {NULL}};
const Segment SGI8314[] = {{&S45993[30], 12}, {NULL}};
const Segment SGI8315[] = {{&S45993[136], 3}, {NULL}};
const Segment SGI8316[] = {{&S45993[101], 6}, {NULL}};
const Segment SGI8317[] = {{&S45994[50], 17}, {NULL}};
const Segment SGI8318[] = {{&S45994[50], 37}, {NULL}};
const Segment SGI8319[] = {{&S45994[30], 12}, {NULL}};
const Segment SGI8320[] = {{&S45994[136], 3}, {NULL}};
const Segment SGI8321[] = {{&S45994[101], 6}, {NULL}};
const Segment SGI8322[] = {{&S45995[50], 7}, {NULL}};
const Segment SGI8323[] = {{&S45995[50], 30}, {NULL}};
const Segment SGI8324[] = {{&S45995[30], 12}, {NULL}};
const Segment SGI8325[] = {{&S45995[129], 3}, {NULL}};
const Segment SGI8326[] = {{&S45995[94], 6}, {NULL}};
const Segment SGI8327[] = {{&S45996[50], 7}, {NULL}};
const Segment SGI8328[] = {{&S45996[50], 40}, {NULL}};
const Segment SGI8329[] = {{&S45996[30], 12}, {NULL}};
const Segment SGI8330[] = {{&S45996[139], 3}, {NULL}};
const Segment SGI8331[] = {{&S45996[104], 6}, {NULL}};
const Segment SGI8332[] = {{&S45997[50], 7}, {NULL}};
const Segment SGI8333[] = {{&S45997[50], 31}, {NULL}};
const Segment SGI8334[] = {{&S45997[30], 12}, {NULL}};
const Segment SGI8335[] = {{&S45997[130], 3}, {NULL}};
const Segment SGI8336[] = {{&S45997[95], 6}, {NULL}};
const Segment SGI8337[] = {{&S45998[50], 7}, {NULL}};
const Segment SGI8338[] = {{&S45998[50], 30}, {NULL}};
const Segment SGI8339[] = {{&S45998[30], 12}, {NULL}};
const Segment SGI8340[] = {{&S45998[129], 3}, {NULL}};
const Segment SGI8341[] = {{&S45998[94], 6}, {NULL}};
const Segment SGI8342[] = {{&S45999[44], 17}, {NULL}};
const Segment SGI8343[] = {{&S45999[31], 12}, {NULL}};
const Segment SGI8344[] = {{&S46000[50], 7}, {NULL}};
const Segment SGI8345[] = {{&S46000[50], 30}, {NULL}};
const Segment SGI8346[] = {{&S46000[30], 12}, {NULL}};
const Segment SGI8347[] = {{&S46000[129], 3}, {NULL}};
const Segment SGI8348[] = {{&S46000[94], 6}, {NULL}};
const Segment SGI8349[] = {{&S46001[50], 7}, {NULL}};
const Segment SGI8350[] = {{&S46001[50], 30}, {NULL}};
const Segment SGI8351[] = {{&S46001[30], 12}, {NULL}};
const Segment SGI8352[] = {{&S46001[129], 3}, {NULL}};
const Segment SGI8353[] = {{&S46001[94], 6}, {NULL}};
const Segment SGI8354[] = {{&S46002[50], 7}, {NULL}};
const Segment SGI8355[] = {{&S46002[50], 30}, {NULL}};
const Segment SGI8356[] = {{&S46002[30], 12}, {NULL}};
const Segment SGI8357[] = {{&S46002[129], 3}, {NULL}};
const Segment SGI8358[] = {{&S46002[94], 6}, {NULL}};
const Segment SGI8359[] = {{&S45871[44], 7}, {NULL}};
const Segment SGI8360[] = {{&S45871[44], 23}, {NULL}};
const Segment SGI8361[] = {{&S45871[30], 6}, {NULL}};
const Segment SGI8362[] = {{&S45871[115], 3}, {NULL}};
const Segment SGI8363[] = {{&S45871[81], 5}, {NULL}};
const Segment SGI8364[] = {{&S45876[44], 7}, {NULL}};
const Segment SGI8365[] = {{&S45876[44], 22}, {NULL}};
const Segment SGI8366[] = {{&S45876[30], 6}, {NULL}};
const Segment SGI8367[] = {{&S45876[114], 3}, {NULL}};
const Segment SGI8368[] = {{&S45876[80], 5}, {NULL}};
const Segment SGI8369[] = {{&S45879[44], 7}, {NULL}};
const Segment SGI8370[] = {{&S45879[44], 23}, {NULL}};
const Segment SGI8371[] = {{&S45879[30], 6}, {NULL}};
const Segment SGI8372[] = {{&S45879[115], 3}, {NULL}};
const Segment SGI8373[] = {{&S45879[81], 5}, {NULL}};
const Segment SGI8374[] = {{&S45888[44], 7}, {NULL}};
const Segment SGI8375[] = {{&S45888[44], 23}, {NULL}};
const Segment SGI8376[] = {{&S45888[30], 6}, {NULL}};
const Segment SGI8377[] = {{&S45888[115], 3}, {NULL}};
const Segment SGI8378[] = {{&S45888[81], 5}, {NULL}};
const Segment SGI8379[] = {{&S45894[44], 7}, {NULL}};
const Segment SGI8380[] = {{&S45894[44], 23}, {NULL}};
const Segment SGI8381[] = {{&S45894[30], 6}, {NULL}};
const Segment SGI8382[] = {{&S45894[115], 3}, {NULL}};
const Segment SGI8383[] = {{&S45894[81], 5}, {NULL}};
const Segment SGI8384[] = {{&S45898[44], 7}, {NULL}};
const Segment SGI8385[] = {{&S45898[44], 23}, {NULL}};
const Segment SGI8386[] = {{&S45898[30], 6}, {NULL}};
const Segment SGI8387[] = {{&S45898[115], 3}, {NULL}};
const Segment SGI8388[] = {{&S45898[81], 5}, {NULL}};
const Segment SGI8389[] = {{&S46381[50], 8}, {NULL}};
const Segment SGI8390[] = {{&S46381[50], 17}, {NULL}};
const Segment SGI8391[] = {{&S46381[30], 12}, {NULL}};
const Segment SGI8392[] = {{&S46381[117], 3}, {NULL}};
const Segment SGI8393[] = {{&S46381[81], 6}, {NULL}};
const Segment SGI8394[] = {{&S46381[137], 8}, {NULL}};
const Segment SGI8395[] = {{&S46382[50], 8}, {NULL}};
const Segment SGI8396[] = {{&S46382[50], 17}, {NULL}};
const Segment SGI8397[] = {{&S46382[30], 12}, {NULL}};
const Segment SGI8398[] = {{&S46382[116], 3}, {NULL}};
const Segment SGI8399[] = {{&S46382[81], 6}, {NULL}};
const Segment SGI8400[] = {{&S46246[44], 9}, {NULL}};
const Segment SGI8401[] = {{&S46246[44], 20}, {NULL}};
const Segment SGI8402[] = {{&S46246[30], 6}, {NULL}};
const Segment SGI8403[] = {{&S46246[112], 3}, {NULL}};
const Segment SGI8404[] = {{&S46246[78], 5}, {NULL}};
const Segment SGI8405[] = {{&S46808[42], 11}, {NULL}};
const Segment SGI8406[] = {{&S46808[42], 27}, {NULL}};
const Segment SGI8407[] = {{&S46808[30], 4}, {NULL}};
const Segment SGI8408[] = {{&S46808[117], 3}, {NULL}};
const Segment SGI8409[] = {{&S46808[83], 5}, {NULL}};
const Segment SGI8410[] = {{&S46249[11], 1}, {NULL}};
const Segment SGI8411[] = {{&S46249[44], 13}, {NULL}};
const Segment SGI8412[] = {{&S46249[30], 6}, {NULL}};
const Segment SGI8413[] = {{&S46249[118], 3}, {NULL}};
const Segment SGI8414[] = {{&S46249[84], 5}, {NULL}};
const Segment SGI8415[] = {{&S46813[42], 11}, {NULL}};
const Segment SGI8416[] = {{&S46813[42], 21}, {NULL}};
const Segment SGI8417[] = {{&S46813[30], 4}, {NULL}};
const Segment SGI8418[] = {{&S46813[113], 3}, {NULL}};
const Segment SGI8419[] = {{&S46813[77], 7}, {NULL}};
const Segment SGI8420[] = {{&S46813[133], 8}, {NULL}};
const Segment SGI8421[] = {{&S46814[42], 11}, {NULL}};
const Segment SGI8422[] = {{&S46814[42], 26}, {NULL}};
const Segment SGI8423[] = {{&S46814[30], 4}, {NULL}};
const Segment SGI8424[] = {{&S46814[116], 3}, {NULL}};
const Segment SGI8425[] = {{&S46814[82], 5}, {NULL}};
const Segment SGI8426[] = {{&S47652[56], 6}, {NULL}};
const Segment SGI8427[] = {{&S47652[56], 25}, {NULL}};
const Segment SGI8428[] = {{&S47652[30], 18}, {NULL}};
const Segment SGI8429[] = {{&S47652[145], 3}, {NULL}};
const Segment SGI8430[] = {{&S47652[111], 5}, {NULL}};
const Segment SGI8431[] = {{&S49785[42], 21}, {NULL}};
const Segment SGI8432[] = {{&S49785[42], 32}, {NULL}};
const Segment SGI8433[] = {{&S49785[30], 4}, {NULL}};
const Segment SGI8434[] = {{&S49785[123], 3}, {NULL}};
const Segment SGI8435[] = {{&S49785[88], 6}, {NULL}};
const Segment SGI8436[] = {{&S49785[143], 8}, {NULL}};
const Segment SGI8437[] = {{&S49786[42], 21}, {NULL}};
const Segment SGI8438[] = {{&S49786[42], 32}, {NULL}};
const Segment SGI8439[] = {{&S49786[30], 4}, {NULL}};
const Segment SGI8440[] = {{&S49786[123], 3}, {NULL}};
const Segment SGI8441[] = {{&S49786[88], 6}, {NULL}};
const Segment SGI8442[] = {{&S49786[143], 8}, {NULL}};
const Segment SGI8443[] = {{&S49787[42], 21}, {NULL}};
const Segment SGI8444[] = {{&S49787[42], 32}, {NULL}};
const Segment SGI8445[] = {{&S49787[30], 4}, {NULL}};
const Segment SGI8446[] = {{&S49787[123], 3}, {NULL}};
const Segment SGI8447[] = {{&S49787[88], 6}, {NULL}};
const Segment SGI8448[] = {{&S49787[143], 8}, {NULL}};
const Segment SGI8449[] = {{&S49788[42], 21}, {NULL}};
const Segment SGI8450[] = {{&S49788[42], 32}, {NULL}};
const Segment SGI8451[] = {{&S49788[30], 4}, {NULL}};
const Segment SGI8452[] = {{&S49788[123], 3}, {NULL}};
const Segment SGI8453[] = {{&S49788[88], 6}, {NULL}};
const Segment SGI8454[] = {{&S49788[143], 8}, {NULL}};
const Segment SGI8455[] = {{&S49789[42], 22}, {NULL}};
const Segment SGI8456[] = {{&S49789[42], 35}, {NULL}};
const Segment SGI8457[] = {{&S49789[30], 4}, {NULL}};
const Segment SGI8458[] = {{&S49789[126], 3}, {NULL}};
const Segment SGI8459[] = {{&S49789[91], 6}, {NULL}};
const Segment SGI8460[] = {{&S49789[146], 8}, {NULL}};
const Segment SGI8461[] = {{&S49790[42], 21}, {NULL}};
const Segment SGI8462[] = {{&S49790[42], 32}, {NULL}};
const Segment SGI8463[] = {{&S49790[30], 4}, {NULL}};
const Segment SGI8464[] = {{&S49790[123], 3}, {NULL}};
const Segment SGI8465[] = {{&S49790[88], 6}, {NULL}};
const Segment SGI8466[] = {{&S49790[143], 8}, {NULL}};
const Segment SGI8467[] = {{&S49791[42], 21}, {NULL}};
const Segment SGI8468[] = {{&S49791[42], 39}, {NULL}};
const Segment SGI8469[] = {{&S49791[30], 4}, {NULL}};
const Segment SGI8470[] = {{&S49791[130], 3}, {NULL}};
const Segment SGI8471[] = {{&S49791[95], 6}, {NULL}};
const Segment SGI8472[] = {{&S49791[150], 8}, {NULL}};
const Segment SGI8473[] = {{&S47054[42], 8}, {NULL}};
const Segment SGI8474[] = {{&S47054[42], 17}, {NULL}};
const Segment SGI8475[] = {{&S47054[30], 4}, {NULL}};
const Segment SGI8476[] = {{&S47054[107], 3}, {NULL}};
const Segment SGI8477[] = {{&S47054[73], 5}, {NULL}};
const Segment SGI8478[] = {{&S47055[42], 8}, {NULL}};
const Segment SGI8479[] = {{&S47055[42], 17}, {NULL}};
const Segment SGI8480[] = {{&S47055[30], 4}, {NULL}};
const Segment SGI8481[] = {{&S47055[107], 3}, {NULL}};
const Segment SGI8482[] = {{&S47055[73], 5}, {NULL}};
const Segment SGI8483[] = {{&S47056[41], 8}, {NULL}};
const Segment SGI8484[] = {{&S47056[41], 17}, {NULL}};
const Segment SGI8485[] = {{&S47056[106], 3}, {NULL}};
const Segment SGI8486[] = {{&S47056[72], 5}, {NULL}};
const Segment SGI8487[] = {{&S47057[41], 8}, {NULL}};
const Segment SGI8488[] = {{&S47057[41], 17}, {NULL}};
const Segment SGI8489[] = {{&S47057[30], 3}, {NULL}};
const Segment SGI8490[] = {{&S47057[106], 3}, {NULL}};
const Segment SGI8491[] = {{&S47057[72], 5}, {NULL}};
const Segment SGI8492[] = {{&S47058[42], 7}, {NULL}};
const Segment SGI8493[] = {{&S47058[42], 16}, {NULL}};
const Segment SGI8494[] = {{&S47058[30], 4}, {NULL}};
const Segment SGI8495[] = {{&S47058[106], 3}, {NULL}};
const Segment SGI8496[] = {{&S47058[72], 5}, {NULL}};
const Segment SGI8497[] = {{&S47059[42], 8}, {NULL}};
const Segment SGI8498[] = {{&S47059[42], 17}, {NULL}};
const Segment SGI8499[] = {{&S47059[30], 4}, {NULL}};
const Segment SGI8500[] = {{&S47059[109], 3}, {NULL}};
const Segment SGI8501[] = {{&S47059[73], 7}, {NULL}};
const Segment SGI8502[] = {{&S47059[129], 8}, {NULL}};
const Segment SGI8503[] = {{&S47061[36], 8}, {NULL}};
const Segment SGI8504[] = {{&S47061[31], 4}, {NULL}};
const Segment SGI8505[] = {{&S47062[42], 10}, {NULL}};
const Segment SGI8506[] = {{&S47062[42], 19}, {NULL}};
const Segment SGI8507[] = {{&S47062[30], 4}, {NULL}};
const Segment SGI8508[] = {{&S47062[109], 3}, {NULL}};
const Segment SGI8509[] = {{&S47062[75], 5}, {NULL}};
const Segment SGI8510[] = {{&S47063[42], 8}, {NULL}};
const Segment SGI8511[] = {{&S47063[42], 19}, {NULL}};
const Segment SGI8512[] = {{&S47063[30], 4}, {NULL}};
const Segment SGI8513[] = {{&S47063[109], 3}, {NULL}};
const Segment SGI8514[] = {{&S47063[75], 5}, {NULL}};
const Segment SGI8515[] = {{&S46678[44], 7}, {NULL}};
const Segment SGI8516[] = {{&S46678[44], 18}, {NULL}};
const Segment SGI8517[] = {{&S46678[30], 6}, {NULL}};
const Segment SGI8518[] = {{&S46678[110], 3}, {NULL}};
const Segment SGI8519[] = {{&S46678[76], 5}, {NULL}};
const Segment SGI8520[] = {{&S46679[44], 20}, {NULL}};
const Segment SGI8521[] = {{&S46679[44], 31}, {NULL}};
const Segment SGI8522[] = {{&S46679[30], 6}, {NULL}};
const Segment SGI8523[] = {{&S46679[123], 3}, {NULL}};
const Segment SGI8524[] = {{&S46679[89], 5}, {NULL}};
const Segment SGI8525[] = {{&S46681[44], 13}, {NULL}};
const Segment SGI8526[] = {{&S46681[44], 30}, {NULL}};
const Segment SGI8527[] = {{&S46681[30], 6}, {NULL}};
const Segment SGI8528[] = {{&S46681[122], 3}, {NULL}};
const Segment SGI8529[] = {{&S46681[88], 5}, {NULL}};
const Segment SGI8530[] = {{&S46682[44], 7}, {NULL}};
const Segment SGI8531[] = {{&S46682[44], 24}, {NULL}};
const Segment SGI8532[] = {{&S46682[30], 6}, {NULL}};
const Segment SGI8533[] = {{&S46682[116], 3}, {NULL}};
const Segment SGI8534[] = {{&S46682[82], 5}, {NULL}};
const Segment SGI8535[] = {{&S46684[38], 7}, {NULL}};
const Segment SGI8536[] = {{&S46684[31], 6}, {NULL}};
const Segment SGI8537[] = {{&S46841[50], 4}, {NULL}};
const Segment SGI8538[] = {{&S46841[50], 15}, {NULL}};
const Segment SGI8539[] = {{&S46841[30], 12}, {NULL}};
const Segment SGI8540[] = {{&S46841[114], 3}, {NULL}};
const Segment SGI8541[] = {{&S46841[79], 6}, {NULL}};
const Segment SGI8542[] = {{&S46842[50], 11}, {NULL}};
const Segment SGI8543[] = {{&S46842[50], 22}, {NULL}};
const Segment SGI8544[] = {{&S46842[30], 12}, {NULL}};
const Segment SGI8545[] = {{&S46842[121], 3}, {NULL}};
const Segment SGI8546[] = {{&S46842[86], 6}, {NULL}};
const Segment SGI8547[] = {{&S46843[50], 10}, {NULL}};
const Segment SGI8548[] = {{&S46843[50], 21}, {NULL}};
const Segment SGI8549[] = {{&S46811[44], 7}, {NULL}};
const Segment SGI8550[] = {{&S46843[30], 12}, {NULL}};
const Segment SGI8551[] = {{&S46811[44], 26}, {NULL}};
const Segment SGI8552[] = {{&S46811[30], 6}, {NULL}};
const Segment SGI8553[] = {{&S46843[120], 3}, {NULL}};
const Segment SGI8554[] = {{&S46811[118], 3}, {NULL}};
const Segment SGI8555[] = {{&S46843[85], 6}, {NULL}};
const Segment SGI8556[] = {{&S46811[84], 5}, {NULL}};
const Segment SGI8557[] = {{&S46844[50], 17}, {NULL}};
const Segment SGI8558[] = {{&S46844[50], 28}, {NULL}};
const Segment SGI8559[] = {{&S46844[30], 12}, {NULL}};
const Segment SGI8560[] = {{&S46844[127], 3}, {NULL}};
const Segment SGI8561[] = {{&S46844[92], 6}, {NULL}};
const Segment SGI8562[] = {{&S46845[50], 4}, {NULL}};
const Segment SGI8563[] = {{&S46845[50], 14}, {NULL}};
const Segment SGI8564[] = {{&S46845[30], 12}, {NULL}};
const Segment SGI8565[] = {{&S46845[112], 3}, {NULL}};
const Segment SGI8566[] = {{&S46845[78], 5}, {NULL}};
const Segment SGI8567[] = {{&S46815[44], 11}, {NULL}};
const Segment SGI8568[] = {{&S46815[44], 27}, {NULL}};
const Segment SGI8569[] = {{&S46815[30], 6}, {NULL}};
const Segment SGI8570[] = {{&S46815[119], 3}, {NULL}};
const Segment SGI8571[] = {{&S46815[85], 5}, {NULL}};
const Segment SGI8572[] = {{&S46846[50], 4}, {NULL}};
const Segment SGI8573[] = {{&S46846[50], 14}, {NULL}};
const Segment SGI8574[] = {{&S46846[30], 12}, {NULL}};
const Segment SGI8575[] = {{&S46846[112], 3}, {NULL}};
const Segment SGI8576[] = {{&S46846[78], 5}, {NULL}};
const Segment SGI8577[] = {{&S46816[44], 11}, {NULL}};
const Segment SGI8578[] = {{&S46816[44], 31}, {NULL}};
const Segment SGI8579[] = {{&S46816[30], 6}, {NULL}};
const Segment SGI8580[] = {{&S46816[123], 3}, {NULL}};
const Segment SGI8581[] = {{&S46816[89], 5}, {NULL}};
const Segment SGI8582[] = {{&S46847[50], 4}, {NULL}};
const Segment SGI8583[] = {{&S46847[50], 14}, {NULL}};
const Segment SGI8584[] = {{&S46847[30], 12}, {NULL}};
const Segment SGI8585[] = {{&S46847[112], 3}, {NULL}};
const Segment SGI8586[] = {{&S46847[78], 5}, {NULL}};
const Segment SGI8587[] = {{&S46817[38], 11}, {NULL}};
const Segment SGI8588[] = {{&S46817[31], 6}, {NULL}};
const Segment SGI8589[] = {{&S46848[44], 10}, {NULL}};
const Segment SGI8590[] = {{&S46848[31], 12}, {NULL}};
const Segment SGI8591[] = {{&S46818[44], 11}, {NULL}};
const Segment SGI8592[] = {{&S46818[44], 31}, {NULL}};
const Segment SGI8593[] = {{&S46818[30], 6}, {NULL}};
const Segment SGI8594[] = {{&S46818[123], 3}, {NULL}};
const Segment SGI8595[] = {{&S46849[50], 4}, {NULL}};
const Segment SGI8596[] = {{&S46849[50], 14}, {NULL}};
const Segment SGI8597[] = {{&S46818[89], 5}, {NULL}};
const Segment SGI8598[] = {{&S46849[30], 12}, {NULL}};
const Segment SGI8599[] = {{&S46849[112], 3}, {NULL}};
const Segment SGI8600[] = {{&S46849[78], 5}, {NULL}};
const Segment SGI8601[] = {{&S46819[44], 11}, {NULL}};
const Segment SGI8602[] = {{&S46819[44], 31}, {NULL}};
const Segment SGI8603[] = {{&S46819[30], 6}, {NULL}};
const Segment SGI8604[] = {{&S46850[44], 4}, {NULL}};
const Segment SGI8605[] = {{&S46850[31], 12}, {NULL}};
const Segment SGI8606[] = {{&S46819[123], 3}, {NULL}};
const Segment SGI8607[] = {{&S46819[89], 5}, {NULL}};
const Segment SGI8608[] = {{&S46851[50], 4}, {NULL}};
const Segment SGI8609[] = {{&S46851[50], 14}, {NULL}};
const Segment SGI8610[] = {{&S46851[30], 12}, {NULL}};
const Segment SGI8611[] = {{&S46851[112], 3}, {NULL}};
const Segment SGI8612[] = {{&S46851[78], 5}, {NULL}};
const Segment SGI8613[] = {{&S46852[50], 4}, {NULL}};
const Segment SGI8614[] = {{&S46852[50], 14}, {NULL}};
const Segment SGI8615[] = {{&S46852[30], 12}, {NULL}};
const Segment SGI8616[] = {{&S46852[112], 3}, {NULL}};
const Segment SGI8617[] = {{&S46852[78], 5}, {NULL}};
const Segment SGI8618[] = {{&S46823[44], 11}, {NULL}};
const Segment SGI8619[] = {{&S46823[44], 25}, {NULL}};
const Segment SGI8620[] = {{&S46823[30], 6}, {NULL}};
const Segment SGI8621[] = {{&S46823[117], 3}, {NULL}};
const Segment SGI8622[] = {{&S46823[83], 5}, {NULL}};
const Segment SGI8623[] = {{&S46853[50], 4}, {NULL}};
const Segment SGI8624[] = {{&S46853[50], 14}, {NULL}};
const Segment SGI8625[] = {{&S46853[30], 12}, {NULL}};
const Segment SGI8626[] = {{&S46853[112], 3}, {NULL}};
const Segment SGI8627[] = {{&S46853[78], 5}, {NULL}};
const Segment SGI8628[] = {{&S47060[44], 8}, {NULL}};
const Segment SGI8629[] = {{&S47060[44], 17}, {NULL}};
const Segment SGI8630[] = {{&S47060[30], 6}, {NULL}};
const Segment SGI8631[] = {{&S47060[109], 3}, {NULL}};
const Segment SGI8632[] = {{&S47060[75], 5}, {NULL}};
const Segment SGI8633[] = {{&S47130[44], 13}, {NULL}};
const Segment SGI8634[] = {{&S47130[44], 25}, {NULL}};
const Segment SGI8635[] = {{&S47130[30], 6}, {NULL}};
const Segment SGI8636[] = {{&S47130[117], 3}, {NULL}};
const Segment SGI8637[] = {{&S47130[83], 5}, {NULL}};
const Segment SGI8638[] = {{&S47131[44], 13}, {NULL}};
const Segment SGI8639[] = {{&S47131[44], 23}, {NULL}};
const Segment SGI8640[] = {{&S47131[30], 6}, {NULL}};
const Segment SGI8641[] = {{&S47131[115], 3}, {NULL}};
const Segment SGI8642[] = {{&S47131[81], 5}, {NULL}};
const Segment SGI8643[] = {{&S47135[44], 13}, {NULL}};
const Segment SGI8644[] = {{&S47135[44], 23}, {NULL}};
const Segment SGI8645[] = {{&S47135[30], 6}, {NULL}};
const Segment SGI8646[] = {{&S47135[115], 3}, {NULL}};
const Segment SGI8647[] = {{&S47135[81], 5}, {NULL}};
const Segment SGI8648[] = {{&S47529[42], 9}, {NULL}};
const Segment SGI8649[] = {{&S47529[42], 20}, {NULL}};
const Segment SGI8650[] = {{&S47529[30], 4}, {NULL}};
const Segment SGI8651[] = {{&S47529[110], 3}, {NULL}};
const Segment SGI8652[] = {{&S47529[76], 5}, {NULL}};
const Segment SGI8653[] = {{&S47530[36], 9}, {NULL}};
const Segment SGI8654[] = {{&S47530[31], 4}, {NULL}};
const Segment SGI8655[] = {{&S47532[41], 10}, {NULL}};
const Segment SGI8656[] = {{&S47532[41], 21}, {NULL}};
const Segment SGI8657[] = {{&S47532[30], 3}, {NULL}};
const Segment SGI8658[] = {{&S47532[110], 3}, {NULL}};
const Segment SGI8659[] = {{&S47532[76], 5}, {NULL}};
const Segment SGI8660[] = {{&S47533[42], 10}, {NULL}};
const Segment SGI8661[] = {{&S47533[42], 21}, {NULL}};
const Segment SGI8662[] = {{&S47533[30], 4}, {NULL}};
const Segment SGI8663[] = {{&S47533[111], 3}, {NULL}};
const Segment SGI8664[] = {{&S47533[77], 5}, {NULL}};
const Segment SGI8665[] = {{&S47534[42], 9}, {NULL}};
const Segment SGI8666[] = {{&S47534[42], 20}, {NULL}};
const Segment SGI8667[] = {{&S47534[30], 4}, {NULL}};
const Segment SGI8668[] = {{&S47534[110], 3}, {NULL}};
const Segment SGI8669[] = {{&S47534[76], 5}, {NULL}};
const Segment SGI8670[] = {{&S47535[41], 10}, {NULL}};
const Segment SGI8671[] = {{&S47535[41], 21}, {NULL}};
const Segment SGI8672[] = {{&S47535[110], 3}, {NULL}};
const Segment SGI8673[] = {{&S47535[76], 5}, {NULL}};
const Segment SGI8674[] = {{&S47536[42], 10}, {NULL}};
const Segment SGI8675[] = {{&S47536[42], 21}, {NULL}};
const Segment SGI8676[] = {{&S47536[30], 4}, {NULL}};
const Segment SGI8677[] = {{&S47536[111], 3}, {NULL}};
const Segment SGI8678[] = {{&S47536[77], 5}, {NULL}};
const Segment SGI8679[] = {{&S47537[42], 9}, {NULL}};
const Segment SGI8680[] = {{&S47537[42], 20}, {NULL}};
const Segment SGI8681[] = {{&S47537[30], 4}, {NULL}};
const Segment SGI8682[] = {{&S47537[110], 3}, {NULL}};
const Segment SGI8683[] = {{&S47537[76], 5}, {NULL}};
const Segment SGI8684[] = {{&S47538[42], 10}, {NULL}};
const Segment SGI8685[] = {{&S47538[42], 35}, {NULL}};
const Segment SGI8686[] = {{&S47538[30], 4}, {NULL}};
const Segment SGI8687[] = {{&S47538[125], 3}, {NULL}};
const Segment SGI8688[] = {{&S47538[91], 5}, {NULL}};
const Segment SGI8689[] = {{&S47539[42], 10}, {NULL}};
const Segment SGI8690[] = {{&S47539[42], 27}, {NULL}};
const Segment SGI8691[] = {{&S47539[30], 4}, {NULL}};
const Segment SGI8692[] = {{&S47540[36], 10}, {NULL}};
const Segment SGI8693[] = {{&S47540[31], 4}, {NULL}};
const Segment SGI8694[] = {{&S47547[42], 10}, {NULL}};
const Segment SGI8695[] = {{&S47547[42], 21}, {NULL}};
const Segment SGI8696[] = {{&S47547[30], 4}, {NULL}};
const Segment SGI8697[] = {{&S47547[111], 3}, {NULL}};
const Segment SGI8698[] = {{&S47547[77], 5}, {NULL}};
const Segment SGI8699[] = {{&S47106[50], 11}, {NULL}};
const Segment SGI8700[] = {{&S47106[50], 33}, {NULL}};
const Segment SGI8701[] = {{&S47106[30], 12}, {NULL}};
const Segment SGI8702[] = {{&S47106[132], 3}, {NULL}};
const Segment SGI8703[] = {{&S47106[97], 6}, {NULL}};
const Segment SGI8704[] = {{&S47107[50], 11}, {NULL}};
const Segment SGI8705[] = {{&S47107[50], 33}, {NULL}};
const Segment SGI8706[] = {{&S47107[30], 12}, {NULL}};
const Segment SGI8707[] = {{&S47107[80], 3}, {NULL}};
const Segment SGI8708[] = {{&S47107[97], 6}, {NULL}};
const Segment SGI8709[] = {{&S47108[50], 11}, {NULL}};
const Segment SGI8710[] = {{&S47108[50], 33}, {NULL}};
const Segment SGI8711[] = {{&S47108[30], 12}, {NULL}};
const Segment SGI8712[] = {{&S47108[132], 3}, {NULL}};
const Segment SGI8713[] = {{&S47108[97], 6}, {NULL}};
const Segment SGI8714[] = {{&S47109[50], 11}, {NULL}};
const Segment SGI8715[] = {{&S47109[50], 33}, {NULL}};
const Segment SGI8716[] = {{&S47109[30], 12}, {NULL}};
const Segment SGI8717[] = {{&S47109[132], 3}, {NULL}};
const Segment SGI8718[] = {{&S47109[97], 6}, {NULL}};
const Segment SGI8719[] = {{&S47110[50], 11}, {NULL}};
const Segment SGI8720[] = {{&S47110[50], 33}, {NULL}};
const Segment SGI8721[] = {{&S47110[30], 12}, {NULL}};
const Segment SGI8722[] = {{&S47110[132], 3}, {NULL}};
const Segment SGI8723[] = {{&S47110[97], 6}, {NULL}};
const Segment SGI8724[] = {{&S47111[50], 11}, {NULL}};
const Segment SGI8725[] = {{&S47111[50], 33}, {NULL}};
const Segment SGI8726[] = {{&S47111[30], 12}, {NULL}};
const Segment SGI8727[] = {{&S47111[132], 3}, {NULL}};
const Segment SGI8728[] = {{&S47111[97], 6}, {NULL}};
const Segment SGI8729[] = {{&S47112[50], 11}, {NULL}};
const Segment SGI8730[] = {{&S47112[50], 33}, {NULL}};
const Segment SGI8731[] = {{&S47112[30], 12}, {NULL}};
const Segment SGI8732[] = {{&S47112[132], 3}, {NULL}};
const Segment SGI8733[] = {{&S47112[97], 6}, {NULL}};
const Segment SGI8734[] = {{&S47113[50], 11}, {NULL}};
const Segment SGI8735[] = {{&S47113[50], 33}, {NULL}};
const Segment SGI8736[] = {{&S47113[30], 12}, {NULL}};
const Segment SGI8737[] = {{&S47113[132], 3}, {NULL}};
const Segment SGI8738[] = {{&S47113[97], 6}, {NULL}};
const Segment SGI8739[] = {{&S47114[50], 11}, {NULL}};
const Segment SGI8740[] = {{&S47114[50], 25}, {NULL}};
const Segment SGI8741[] = {{&S47114[30], 12}, {NULL}};
const Segment SGI8742[] = {{&S47114[124], 3}, {NULL}};
const Segment SGI8743[] = {{&S47114[89], 6}, {NULL}};
const Segment SGI8744[] = {{&S47115[50], 11}, {NULL}};
const Segment SGI8745[] = {{&S47115[50], 33}, {NULL}};
const Segment SGI8746[] = {{&S47115[30], 12}, {NULL}};
const Segment SGI8747[] = {{&S47115[132], 3}, {NULL}};
const Segment SGI8748[] = {{&S47115[97], 6}, {NULL}};
const Segment SGI8749[] = {{&S47116[50], 11}, {NULL}};
const Segment SGI8750[] = {{&S47116[50], 33}, {NULL}};
const Segment SGI8751[] = {{&S47116[30], 12}, {NULL}};
const Segment SGI8752[] = {{&S47116[132], 3}, {NULL}};
const Segment SGI8753[] = {{&S47116[97], 6}, {NULL}};
const Segment SGI8754[] = {{&S47117[50], 11}, {NULL}};
const Segment SGI8755[] = {{&S47117[50], 33}, {NULL}};
const Segment SGI8756[] = {{&S47117[30], 12}, {NULL}};
const Segment SGI8757[] = {{&S47117[132], 3}, {NULL}};
const Segment SGI8758[] = {{&S47117[97], 6}, {NULL}};
const Segment SGI8759[] = {{&S47118[50], 11}, {NULL}};
const Segment SGI8760[] = {{&S47118[50], 33}, {NULL}};
const Segment SGI8761[] = {{&S47118[30], 12}, {NULL}};
const Segment SGI8762[] = {{&S47118[132], 3}, {NULL}};
const Segment SGI8763[] = {{&S47118[97], 6}, {NULL}};
const Segment SGI8764[] = {{&S47119[50], 11}, {NULL}};
const Segment SGI8765[] = {{&S47119[50], 33}, {NULL}};
const Segment SGI8766[] = {{&S47119[30], 12}, {NULL}};
const Segment SGI8767[] = {{&S47119[132], 3}, {NULL}};
const Segment SGI8768[] = {{&S47119[97], 6}, {NULL}};
const Segment SGI8769[] = {{&S47120[44], 11}, {NULL}};
const Segment SGI8770[] = {{&S47120[31], 12}, {NULL}};
const Segment SGI8771[] = {{&S47121[50], 11}, {NULL}};
const Segment SGI8772[] = {{&S47121[50], 33}, {NULL}};
const Segment SGI8773[] = {{&S47121[30], 12}, {NULL}};
const Segment SGI8774[] = {{&S47121[132], 3}, {NULL}};
const Segment SGI8775[] = {{&S47121[97], 6}, {NULL}};
const Segment SGI8776[] = {{&S47122[50], 11}, {NULL}};
const Segment SGI8777[] = {{&S47122[50], 33}, {NULL}};
const Segment SGI8778[] = {{&S47122[30], 12}, {NULL}};
const Segment SGI8779[] = {{&S47122[132], 3}, {NULL}};
const Segment SGI8780[] = {{&S47122[97], 6}, {NULL}};
const Segment SGI8781[] = {{&S47123[50], 11}, {NULL}};
const Segment SGI8782[] = {{&S47123[50], 33}, {NULL}};
const Segment SGI8783[] = {{&S47123[30], 12}, {NULL}};
const Segment SGI8784[] = {{&S47123[132], 3}, {NULL}};
const Segment SGI8785[] = {{&S47123[97], 6}, {NULL}};
const Segment SGI8786[] = {{&S47124[42], 11}, {NULL}};
const Segment SGI8787[] = {{&S47124[42], 33}, {NULL}};
const Segment SGI8788[] = {{&S47124[30], 4}, {NULL}};
const Segment SGI8789[] = {{&S47124[124], 3}, {NULL}};
const Segment SGI8790[] = {{&S47124[89], 6}, {NULL}};
const Segment SGI8791[] = {{&S47125[50], 11}, {NULL}};
const Segment SGI8792[] = {{&S47125[50], 33}, {NULL}};
const Segment SGI8793[] = {{&S47125[30], 12}, {NULL}};
const Segment SGI8794[] = {{&S47125[77], 3}, {NULL}};
const Segment SGI8795[] = {{&S47125[97], 6}, {NULL}};
const Segment SGI8796[] = {{&S47126[42], 11}, {NULL}};
const Segment SGI8797[] = {{&S47126[42], 33}, {NULL}};
const Segment SGI8798[] = {{&S47126[30], 4}, {NULL}};
const Segment SGI8799[] = {{&S47126[92], 4}, {NULL}};
const Segment SGI8800[] = {{&S47126[92], 3}, {NULL}};
const Segment SGI8801[] = {{&S47204[50], 10}, {NULL}};
const Segment SGI8802[] = {{&S47204[50], 22}, {NULL}};
const Segment SGI8803[] = {{&S47204[30], 12}, {NULL}};
const Segment SGI8804[] = {{&S47204[121], 3}, {NULL}};
const Segment SGI8805[] = {{&S47204[86], 6}, {NULL}};
const Segment SGI8806[] = {{&S47205[50], 10}, {NULL}};
const Segment SGI8807[] = {{&S47205[50], 22}, {NULL}};
const Segment SGI8808[] = {{&S47205[30], 12}, {NULL}};
const Segment SGI8809[] = {{&S47205[121], 3}, {NULL}};
const Segment SGI8810[] = {{&S47205[86], 6}, {NULL}};
const Segment SGI8811[] = {{&S47205[141], 8}, {NULL}};
const Segment SGI8812[] = {{&S47206[50], 11}, {NULL}};
const Segment SGI8813[] = {{&S47206[50], 23}, {NULL}};
const Segment SGI8814[] = {{&S47206[30], 12}, {NULL}};
const Segment SGI8815[] = {{&S47206[122], 3}, {NULL}};
const Segment SGI8816[] = {{&S47206[87], 6}, {NULL}};
const Segment SGI8817[] = {{&S47207[50], 11}, {NULL}};
const Segment SGI8818[] = {{&S47207[50], 23}, {NULL}};
const Segment SGI8819[] = {{&S47207[30], 12}, {NULL}};
const Segment SGI8820[] = {{&S47207[122], 3}, {NULL}};
const Segment SGI8821[] = {{&S47207[87], 6}, {NULL}};
const Segment SGI8822[] = {{&S47208[42], 10}, {NULL}};
const Segment SGI8823[] = {{&S47208[42], 22}, {NULL}};
const Segment SGI8824[] = {{&S47208[30], 4}, {NULL}};
const Segment SGI8825[] = {{&S47208[113], 3}, {NULL}};
const Segment SGI8826[] = {{&S47208[78], 6}, {NULL}};
const Segment SGI8827[] = {{&S47209[44], 10}, {NULL}};
const Segment SGI8828[] = {{&S47209[31], 12}, {NULL}};
const Segment SGI8829[] = {{&S47210[50], 11}, {NULL}};
const Segment SGI8830[] = {{&S47210[50], 23}, {NULL}};
const Segment SGI8831[] = {{&S47210[30], 12}, {NULL}};
const Segment SGI8832[] = {{&S47210[122], 3}, {NULL}};
const Segment SGI8833[] = {{&S47210[87], 6}, {NULL}};
const Segment SGI8834[] = {{&S47210[142], 8}, {NULL}};
const Segment SGI8835[] = {{&S47211[42], 10}, {NULL}};
const Segment SGI8836[] = {{&S47211[42], 22}, {NULL}};
const Segment SGI8837[] = {{&S47211[30], 4}, {NULL}};
const Segment SGI8838[] = {{&S47211[113], 3}, {NULL}};
const Segment SGI8839[] = {{&S47211[78], 6}, {NULL}};
const Segment SGI8840[] = {{&S47211[133], 8}, {NULL}};
const Segment SGI8841[] = {{&S47212[50], 10}, {NULL}};
const Segment SGI8842[] = {{&S47212[50], 22}, {NULL}};
const Segment SGI8843[] = {{&S47212[30], 12}, {NULL}};
const Segment SGI8844[] = {{&S47212[89], 4}, {NULL}};
const Segment SGI8845[] = {{&S47212[89], 3}, {NULL}};
const Segment SGI8846[] = {{&S48445[8], 3}, {NULL}};
const Segment SGI8847[] = {{&S47407[49], 19}, {NULL}};
const Segment SGI8848[] = {{&S47407[49], 30}, {NULL}};
const Segment SGI8849[] = {{&S47407[30], 11}, {NULL}};
const Segment SGI8850[] = {{&S47407[128], 3}, {NULL}};
const Segment SGI8851[] = {{&S47407[93], 6}, {NULL}};
const Segment SGI8852[] = {{&S47408[50], 18}, {NULL}};
const Segment SGI8853[] = {{&S47408[50], 29}, {NULL}};
const Segment SGI8854[] = {{&S47408[30], 12}, {NULL}};
const Segment SGI8855[] = {{&S47408[128], 3}, {NULL}};
const Segment SGI8856[] = {{&S47408[93], 6}, {NULL}};
const Segment SGI8857[] = {{&S47409[49], 19}, {NULL}};
const Segment SGI8858[] = {{&S47409[49], 30}, {NULL}};
const Segment SGI8859[] = {{&S47409[128], 3}, {NULL}};
const Segment SGI8860[] = {{&S47409[93], 6}, {NULL}};
const Segment SGI8861[] = {{&S47410[50], 19}, {NULL}};
const Segment SGI8862[] = {{&S47410[50], 30}, {NULL}};
const Segment SGI8863[] = {{&S47410[30], 12}, {NULL}};
const Segment SGI8864[] = {{&S47410[129], 3}, {NULL}};
const Segment SGI8865[] = {{&S47410[94], 6}, {NULL}};
const Segment SGI8866[] = {{&S47411[50], 18}, {NULL}};
const Segment SGI8867[] = {{&S47411[50], 29}, {NULL}};
const Segment SGI8868[] = {{&S47411[30], 12}, {NULL}};
const Segment SGI8869[] = {{&S47411[128], 3}, {NULL}};
const Segment SGI8870[] = {{&S47411[93], 6}, {NULL}};
const Segment SGI8871[] = {{&S47412[50], 19}, {NULL}};
const Segment SGI8872[] = {{&S47412[50], 30}, {NULL}};
const Segment SGI8873[] = {{&S47412[30], 12}, {NULL}};
const Segment SGI8874[] = {{&S47412[129], 3}, {NULL}};
const Segment SGI8875[] = {{&S47412[94], 6}, {NULL}};
const Segment SGI8876[] = {{&S47413[42], 14}, {NULL}};
const Segment SGI8877[] = {{&S47413[42], 26}, {NULL}};
const Segment SGI8878[] = {{&S47413[30], 4}, {NULL}};
const Segment SGI8879[] = {{&S47413[118], 3}, {NULL}};
const Segment SGI8880[] = {{&S47413[82], 7}, {NULL}};
const Segment SGI8881[] = {{&S47413[138], 8}, {NULL}};
const Segment SGI8882[] = {{&S47541[44], 12}, {NULL}};
const Segment SGI8883[] = {{&S47541[44], 24}, {NULL}};
const Segment SGI8884[] = {{&S47541[30], 6}, {NULL}};
const Segment SGI8885[] = {{&S47541[116], 3}, {NULL}};
const Segment SGI8886[] = {{&S47541[82], 5}, {NULL}};
const Segment SGI8887[] = {{&S47542[44], 12}, {NULL}};
const Segment SGI8888[] = {{&S47542[44], 24}, {NULL}};
const Segment SGI8889[] = {{&S47542[30], 6}, {NULL}};
const Segment SGI8890[] = {{&S47542[116], 3}, {NULL}};
const Segment SGI8891[] = {{&S47542[82], 5}, {NULL}};
const Segment SGI8892[] = {{&S47543[44], 13}, {NULL}};
const Segment SGI8893[] = {{&S47543[44], 25}, {NULL}};
const Segment SGI8894[] = {{&S47543[30], 6}, {NULL}};
const Segment SGI8895[] = {{&S47543[117], 3}, {NULL}};
const Segment SGI8896[] = {{&S47543[83], 5}, {NULL}};
const Segment SGI8897[] = {{&S47544[44], 10}, {NULL}};
const Segment SGI8898[] = {{&S47544[44], 22}, {NULL}};
const Segment SGI8899[] = {{&S47544[30], 6}, {NULL}};
const Segment SGI8900[] = {{&S47544[114], 3}, {NULL}};
const Segment SGI8901[] = {{&S47544[80], 5}, {NULL}};
const Segment SGI8902[] = {{&S47545[38], 10}, {NULL}};
const Segment SGI8903[] = {{&S47545[31], 6}, {NULL}};
const Segment SGI8904[] = {{&S47595[44], 10}, {NULL}};
const Segment SGI8905[] = {{&S47595[44], 21}, {NULL}};
const Segment SGI8906[] = {{&S47595[30], 6}, {NULL}};
const Segment SGI8907[] = {{&S47595[113], 3}, {NULL}};
const Segment SGI8908[] = {{&S47595[79], 5}, {NULL}};
const Segment SGI8909[] = {{&S47597[38], 10}, {NULL}};
const Segment SGI8910[] = {{&S47597[31], 6}, {NULL}};
const Segment SGI8911[] = {{&S50846[42], 9}, {NULL}};
const Segment SGI8912[] = {{&S50846[42], 20}, {NULL}};
const Segment SGI8913[] = {{&S50846[30], 4}, {NULL}};
const Segment SGI8914[] = {{&S50846[111], 3}, {NULL}};
const Segment SGI8915[] = {{&S50846[76], 6}, {NULL}};
const Segment SGI8916[] = {{&S50846[131], 8}, {NULL}};
const Segment SGI8917[] = {{&S50847[42], 9}, {NULL}};
const Segment SGI8918[] = {{&S50847[42], 20}, {NULL}};
const Segment SGI8919[] = {{&S50847[30], 4}, {NULL}};
const Segment SGI8920[] = {{&S50847[111], 3}, {NULL}};
const Segment SGI8921[] = {{&S50847[76], 5}, {NULL}};
const Segment SGI8922[] = {{&S50847[131], 8}, {NULL}};
const Segment SGI8923[] = {{&S47674[44], 10}, {NULL}};
const Segment SGI8924[] = {{&S47674[44], 21}, {NULL}};
const Segment SGI8925[] = {{&S47674[30], 6}, {NULL}};
const Segment SGI8926[] = {{&S47674[113], 3}, {NULL}};
const Segment SGI8927[] = {{&S47674[79], 5}, {NULL}};
const Segment SGI8928[] = {{&S47675[38], 10}, {NULL}};
const Segment SGI8929[] = {{&S47675[31], 6}, {NULL}};
const Segment SGI8930[] = {{&S47930[42], 15}, {NULL}};
const Segment SGI8931[] = {{&S47930[42], 26}, {NULL}};
const Segment SGI8932[] = {{&S47930[30], 4}, {NULL}};
const Segment SGI8933[] = {{&S47930[116], 3}, {NULL}};
const Segment SGI8934[] = {{&S47930[82], 5}, {NULL}};
const Segment SGI8935[] = {{&S47836[44], 6}, {NULL}};
const Segment SGI8936[] = {{&S47836[44], 20}, {NULL}};
const Segment SGI8937[] = {{&S47836[30], 6}, {NULL}};
const Segment SGI8938[] = {{&S47836[112], 3}, {NULL}};
const Segment SGI8939[] = {{&S47836[78], 5}, {NULL}};
const Segment SGI8940[] = {{&S47837[44], 6}, {NULL}};
const Segment SGI8941[] = {{&S47837[44], 20}, {NULL}};
const Segment SGI8942[] = {{&S47837[30], 6}, {NULL}};
const Segment SGI8943[] = {{&S47837[112], 3}, {NULL}};
const Segment SGI8944[] = {{&S47837[78], 5}, {NULL}};
const Segment SGI8945[] = {{&S47838[38], 6}, {NULL}};
const Segment SGI8946[] = {{&S47838[31], 6}, {NULL}};
const Segment SGI8947[] = {{&S47705[50], 20}, {NULL}};
const Segment SGI8948[] = {{&S47839[44], 6}, {NULL}};
const Segment SGI8949[] = {{&S47705[50], 31}, {NULL}};
const Segment SGI8950[] = {{&S47839[44], 20}, {NULL}};
const Segment SGI8951[] = {{&S47705[30], 12}, {NULL}};
const Segment SGI8952[] = {{&S47839[30], 6}, {NULL}};
const Segment SGI8953[] = {{&S47839[112], 3}, {NULL}};
const Segment SGI8954[] = {{&S47705[130], 3}, {NULL}};
const Segment SGI8955[] = {{&S47839[78], 5}, {NULL}};
const Segment SGI8956[] = {{&S47705[95], 6}, {NULL}};
const Segment SGI8957[] = {{&S47706[50], 20}, {NULL}};
const Segment SGI8958[] = {{&S47706[50], 31}, {NULL}};
const Segment SGI8959[] = {{&S47706[30], 12}, {NULL}};
const Segment SGI8960[] = {{&S47706[130], 3}, {NULL}};
const Segment SGI8961[] = {{&S47706[95], 6}, {NULL}};
const Segment SGI8962[] = {{&S48311[50], 6}, {NULL}};
const Segment SGI8963[] = {{&S48311[50], 24}, {NULL}};
const Segment SGI8964[] = {{&S48311[30], 12}, {NULL}};
const Segment SGI8965[] = {{&S48311[123], 3}, {NULL}};
const Segment SGI8966[] = {{&S48311[88], 6}, {NULL}};
const Segment SGI8967[] = {{&S48312[50], 6}, {NULL}};
const Segment SGI8968[] = {{&S48312[50], 23}, {NULL}};
const Segment SGI8969[] = {{&S48312[30], 12}, {NULL}};
const Segment SGI8970[] = {{&S48312[122], 3}, {NULL}};
const Segment SGI8971[] = {{&S48312[87], 6}, {NULL}};
const Segment SGI8972[] = {{&S48313[50], 6}, {NULL}};
const Segment SGI8973[] = {{&S48313[50], 24}, {NULL}};
const Segment SGI8974[] = {{&S48313[30], 12}, {NULL}};
const Segment SGI8975[] = {{&S48313[123], 3}, {NULL}};
const Segment SGI8976[] = {{&S48313[88], 6}, {NULL}};
const Segment SGI8977[] = {{&S48314[50], 6}, {NULL}};
const Segment SGI8978[] = {{&S48314[50], 23}, {NULL}};
const Segment SGI8979[] = {{&S48314[30], 12}, {NULL}};
const Segment SGI8980[] = {{&S48314[122], 3}, {NULL}};
const Segment SGI8981[] = {{&S48314[87], 6}, {NULL}};
const Segment SGI8982[] = {{&S48315[44], 6}, {NULL}};
const Segment SGI8983[] = {{&S48315[31], 12}, {NULL}};
const Segment SGI8984[] = {{&S48317[50], 10}, {NULL}};
const Segment SGI8985[] = {{&S48317[50], 21}, {NULL}};
const Segment SGI8986[] = {{&S48317[30], 12}, {NULL}};
const Segment SGI8987[] = {{&S48317[120], 3}, {NULL}};
const Segment SGI8988[] = {{&S48317[85], 6}, {NULL}};
const Segment SGI8989[] = {{&S48318[50], 11}, {NULL}};
const Segment SGI8990[] = {{&S48318[50], 22}, {NULL}};
const Segment SGI8991[] = {{&S48318[30], 12}, {NULL}};
const Segment SGI8992[] = {{&S48318[121], 3}, {NULL}};
const Segment SGI8993[] = {{&S48318[86], 6}, {NULL}};
const Segment SGI8994[] = {{&S48319[44], 10}, {NULL}};
const Segment SGI8995[] = {{&S48319[31], 12}, {NULL}};
const Segment SGI8996[] = {{&S48320[50], 10}, {NULL}};
const Segment SGI8997[] = {{&S48320[50], 21}, {NULL}};
const Segment SGI8998[] = {{&S48320[30], 12}, {NULL}};
const Segment SGI8999[] = {{&S48320[88], 4}, {NULL}};
const Segment SGI9000[] = {{&S48320[88], 3}, {NULL}};
const Segment SGI9001[] = {{&S48321[44], 11}, {NULL}};
const Segment SGI9002[] = {{&S48321[31], 12}, {NULL}};
const Segment SGI9003[] = {{&S48322[48], 10}, {NULL}};
const Segment SGI9004[] = {{&S48322[30], 12}, {NULL}};
const Segment SGI9005[] = {{&S48322[118], 3}, {NULL}};
const Segment SGI9006[] = {{&S48322[83], 6}, {NULL}};
const Segment SGI9007[] = {{&S48368[50], 6}, {NULL}};
const Segment SGI9008[] = {{&S48368[50], 28}, {NULL}};
const Segment SGI9009[] = {{&S48368[30], 12}, {NULL}};
const Segment SGI9010[] = {{&S48368[127], 3}, {NULL}};
const Segment SGI9011[] = {{&S48368[92], 6}, {NULL}};
const Segment SGI9012[] = {{&S48369[50], 6}, {NULL}};
const Segment SGI9013[] = {{&S48369[50], 28}, {NULL}};
const Segment SGI9014[] = {{&S48369[30], 12}, {NULL}};
const Segment SGI9015[] = {{&S48369[127], 3}, {NULL}};
const Segment SGI9016[] = {{&S48369[92], 6}, {NULL}};
const Segment SGI9017[] = {{&S48370[50], 6}, {NULL}};
const Segment SGI9018[] = {{&S48370[50], 28}, {NULL}};
const Segment SGI9019[] = {{&S48370[30], 12}, {NULL}};
const Segment SGI9020[] = {{&S48370[127], 3}, {NULL}};
const Segment SGI9021[] = {{&S48370[92], 6}, {NULL}};
const Segment SGI9022[] = {{&S48371[50], 6}, {NULL}};
const Segment SGI9023[] = {{&S48371[50], 28}, {NULL}};
const Segment SGI9024[] = {{&S48371[30], 12}, {NULL}};
const Segment SGI9025[] = {{&S48371[127], 3}, {NULL}};
const Segment SGI9026[] = {{&S48371[92], 6}, {NULL}};
const Segment SGI9027[] = {{&S48372[50], 6}, {NULL}};
const Segment SGI9028[] = {{&S48372[50], 28}, {NULL}};
const Segment SGI9029[] = {{&S48372[30], 12}, {NULL}};
const Segment SGI9030[] = {{&S48372[127], 3}, {NULL}};
const Segment SGI9031[] = {{&S48372[92], 6}, {NULL}};
const Segment SGI9032[] = {{&S48373[50], 6}, {NULL}};
const Segment SGI9033[] = {{&S48373[50], 28}, {NULL}};
const Segment SGI9034[] = {{&S48373[30], 12}, {NULL}};
const Segment SGI9035[] = {{&S48373[127], 3}, {NULL}};
const Segment SGI9036[] = {{&S48373[92], 6}, {NULL}};
const Segment SGI9037[] = {{&S48374[44], 6}, {NULL}};
const Segment SGI9038[] = {{&S48374[31], 12}, {NULL}};
const Segment SGI9039[] = {{&S48375[50], 6}, {NULL}};
const Segment SGI9040[] = {{&S48375[50], 28}, {NULL}};
const Segment SGI9041[] = {{&S48375[30], 12}, {NULL}};
const Segment SGI9042[] = {{&S48375[128], 3}, {NULL}};
const Segment SGI9043[] = {{&S48375[92], 7}, {NULL}};
const Segment SGI9044[] = {{&S48375[148], 8}, {NULL}};
const Segment SGI9045[] = {{&S48664[44], 11}, {NULL}};
const Segment SGI9046[] = {{&S48664[44], 33}, {NULL}};
const Segment SGI9047[] = {{&S48664[30], 6}, {NULL}};
const Segment SGI9048[] = {{&S48664[125], 3}, {NULL}};
const Segment SGI9049[] = {{&S48664[91], 5}, {NULL}};
const Segment SGI9050[] = {{&S48665[44], 11}, {NULL}};
const Segment SGI9051[] = {{&S48665[44], 23}, {NULL}};
const Segment SGI9052[] = {{&S48665[30], 6}, {NULL}};
const Segment SGI9053[] = {{&S48665[115], 3}, {NULL}};
const Segment SGI9054[] = {{&S48665[81], 5}, {NULL}};
const Segment SGI9055[] = {{&S48666[44], 11}, {NULL}};
const Segment SGI9056[] = {{&S48666[44], 27}, {NULL}};
const Segment SGI9057[] = {{&S48666[30], 6}, {NULL}};
const Segment SGI9058[] = {{&S48666[119], 3}, {NULL}};
const Segment SGI9059[] = {{&S48666[85], 5}, {NULL}};
const Segment SGI9060[] = {{&S48668[44], 11}, {NULL}};
const Segment SGI9061[] = {{&S48668[44], 33}, {NULL}};
const Segment SGI9062[] = {{&S48668[30], 6}, {NULL}};
const Segment SGI9063[] = {{&S48668[125], 3}, {NULL}};
const Segment SGI9064[] = {{&S48668[91], 5}, {NULL}};
const Segment SGI9065[] = {{&S48669[44], 11}, {NULL}};
const Segment SGI9066[] = {{&S48669[44], 33}, {NULL}};
const Segment SGI9067[] = {{&S48669[30], 6}, {NULL}};
const Segment SGI9068[] = {{&S48669[125], 3}, {NULL}};
const Segment SGI9069[] = {{&S48669[91], 5}, {NULL}};
const Segment SGI9070[] = {{&S48670[44], 11}, {NULL}};
const Segment SGI9071[] = {{&S48670[44], 33}, {NULL}};
const Segment SGI9072[] = {{&S48670[30], 6}, {NULL}};
const Segment SGI9073[] = {{&S48670[125], 3}, {NULL}};
const Segment SGI9074[] = {{&S48670[91], 5}, {NULL}};
const Segment SGI9075[] = {{&S48671[44], 11}, {NULL}};
const Segment SGI9076[] = {{&S48671[44], 33}, {NULL}};
const Segment SGI9077[] = {{&S48671[30], 6}, {NULL}};
const Segment SGI9078[] = {{&S48671[125], 3}, {NULL}};
const Segment SGI9079[] = {{&S48671[91], 5}, {NULL}};
const Segment SGI9080[] = {{&S48672[44], 11}, {NULL}};
const Segment SGI9081[] = {{&S48672[44], 33}, {NULL}};
const Segment SGI9082[] = {{&S48672[30], 6}, {NULL}};
const Segment SGI9083[] = {{&S48672[125], 3}, {NULL}};
const Segment SGI9084[] = {{&S48672[91], 5}, {NULL}};
const Segment SGI9085[] = {{&S48673[44], 11}, {NULL}};
const Segment SGI9086[] = {{&S48673[44], 33}, {NULL}};
const Segment SGI9087[] = {{&S48673[30], 6}, {NULL}};
const Segment SGI9088[] = {{&S48673[125], 3}, {NULL}};
const Segment SGI9089[] = {{&S48673[91], 5}, {NULL}};
const Segment SGI9090[] = {{&S48674[44], 11}, {NULL}};
const Segment SGI9091[] = {{&S48674[44], 33}, {NULL}};
const Segment SGI9092[] = {{&S48674[30], 6}, {NULL}};
const Segment SGI9093[] = {{&S48674[125], 3}, {NULL}};
const Segment SGI9094[] = {{&S48674[91], 5}, {NULL}};
const Segment SGI9095[] = {{&S48675[44], 11}, {NULL}};
const Segment SGI9096[] = {{&S48675[44], 34}, {NULL}};
const Segment SGI9097[] = {{&S48675[30], 6}, {NULL}};
const Segment SGI9098[] = {{&S48675[126], 3}, {NULL}};
const Segment SGI9099[] = {{&S48675[92], 5}, {NULL}};
const Segment SGI9100[] = {{&S48677[44], 11}, {NULL}};
const Segment SGI9101[] = {{&S48677[44], 21}, {NULL}};
const Segment SGI9102[] = {{&S48677[30], 6}, {NULL}};
const Segment SGI9103[] = {{&S48677[113], 3}, {NULL}};
const Segment SGI9104[] = {{&S48677[79], 5}, {NULL}};
const Segment SGI9105[] = {{&S48679[44], 11}, {NULL}};
const Segment SGI9106[] = {{&S48679[44], 33}, {NULL}};
const Segment SGI9107[] = {{&S48679[30], 6}, {NULL}};
const Segment SGI9108[] = {{&S48679[125], 3}, {NULL}};
const Segment SGI9109[] = {{&S48679[91], 5}, {NULL}};
const Segment SGI9110[] = {{&S48680[38], 11}, {NULL}};
const Segment SGI9111[] = {{&S48680[31], 6}, {NULL}};
const Segment SGI9112[] = {{&S48681[44], 11}, {NULL}};
const Segment SGI9113[] = {{&S48681[44], 27}, {NULL}};
const Segment SGI9114[] = {{&S48681[30], 6}, {NULL}};
const Segment SGI9115[] = {{&S48681[88], 4}, {NULL}};
const Segment SGI9116[] = {{&S48681[88], 3}, {NULL}};
const Segment SGI9117[] = {{&S48770[43], 27}, {NULL}};
const Segment SGI9118[] = {{&S48770[43], 39}, {NULL}};
const Segment SGI9119[] = {{&S48770[30], 5}, {NULL}};
const Segment SGI9120[] = {{&S48770[130], 3}, {NULL}};
const Segment SGI9121[] = {{&S48770[96], 5}, {NULL}};
const Segment SGI9122[] = {{&S48676[42], 11}, {NULL}};
const Segment SGI9123[] = {{&S48676[42], 22}, {NULL}};
const Segment SGI9124[] = {{&S48676[30], 4}, {NULL}};
const Segment SGI9125[] = {{&S48771[44], 27}, {NULL}};
const Segment SGI9126[] = {{&S48771[44], 39}, {NULL}};
const Segment SGI9127[] = {{&S48771[30], 6}, {NULL}};
const Segment SGI9128[] = {{&S48676[112], 3}, {NULL}};
const Segment SGI9129[] = {{&S48676[78], 5}, {NULL}};
const Segment SGI9130[] = {{&S48771[131], 3}, {NULL}};
const Segment SGI9131[] = {{&S48771[97], 5}, {NULL}};
const Segment SGI9132[] = {{&S48772[44], 26}, {NULL}};
const Segment SGI9133[] = {{&S48772[44], 38}, {NULL}};
const Segment SGI9134[] = {{&S48772[30], 6}, {NULL}};
const Segment SGI9135[] = {{&S48772[130], 3}, {NULL}};
const Segment SGI9136[] = {{&S48772[96], 5}, {NULL}};
const Segment SGI9137[] = {{&S48774[44], 27}, {NULL}};
const Segment SGI9138[] = {{&S48774[44], 38}, {NULL}};
const Segment SGI9139[] = {{&S48774[30], 6}, {NULL}};
const Segment SGI9140[] = {{&S48774[130], 3}, {NULL}};
const Segment SGI9141[] = {{&S48774[96], 5}, {NULL}};
const Segment SGI9142[] = {{&S48776[38], 27}, {NULL}};
const Segment SGI9143[] = {{&S48776[31], 6}, {NULL}};
const Segment SGI9144[] = {{&S48777[44], 27}, {NULL}};
const Segment SGI9145[] = {{&S48777[44], 39}, {NULL}};
const Segment SGI9146[] = {{&S48777[30], 6}, {NULL}};
const Segment SGI9147[] = {{&S48777[131], 3}, {NULL}};
const Segment SGI9148[] = {{&S48777[97], 5}, {NULL}};
const Segment SGI9149[] = {{&S48778[44], 26}, {NULL}};
const Segment SGI9150[] = {{&S48778[44], 38}, {NULL}};
const Segment SGI9151[] = {{&S48778[30], 6}, {NULL}};
const Segment SGI9152[] = {{&S48778[130], 3}, {NULL}};
const Segment SGI9153[] = {{&S48778[96], 5}, {NULL}};
const Segment SGI9154[] = {{&S48779[43], 27}, {NULL}};
const Segment SGI9155[] = {{&S48779[43], 39}, {NULL}};
const Segment SGI9156[] = {{&S48779[130], 3}, {NULL}};
const Segment SGI9157[] = {{&S48779[96], 5}, {NULL}};
const Segment SGI9158[] = {{&S48780[44], 26}, {NULL}};
const Segment SGI9159[] = {{&S48780[44], 38}, {NULL}};
const Segment SGI9160[] = {{&S48780[30], 6}, {NULL}};
const Segment SGI9161[] = {{&S48780[130], 3}, {NULL}};
const Segment SGI9162[] = {{&S48780[96], 5}, {NULL}};
const Segment SGI9163[] = {{&S48781[44], 26}, {NULL}};
const Segment SGI9164[] = {{&S48781[44], 38}, {NULL}};
const Segment SGI9165[] = {{&S48781[30], 6}, {NULL}};
const Segment SGI9166[] = {{&S48781[130], 3}, {NULL}};
const Segment SGI9167[] = {{&S48781[96], 5}, {NULL}};
const Segment SGI9168[] = {{&S48782[44], 26}, {NULL}};
const Segment SGI9169[] = {{&S48782[44], 38}, {NULL}};
const Segment SGI9170[] = {{&S48782[30], 6}, {NULL}};
const Segment SGI9171[] = {{&S48782[130], 3}, {NULL}};
const Segment SGI9172[] = {{&S48782[96], 5}, {NULL}};
const Segment SGI9173[] = {{&S48560[50], 10}, {NULL}};
const Segment SGI9174[] = {{&S48560[50], 22}, {NULL}};
const Segment SGI9175[] = {{&S48560[30], 12}, {NULL}};
const Segment SGI9176[] = {{&S48560[121], 3}, {NULL}};
const Segment SGI9177[] = {{&S48560[86], 6}, {NULL}};
const Segment SGI9178[] = {{&S48561[44], 10}, {NULL}};
const Segment SGI9179[] = {{&S48561[31], 12}, {NULL}};
const Segment SGI9180[] = {{&S48607[50], 10}, {NULL}};
const Segment SGI9181[] = {{&S48607[50], 22}, {NULL}};
const Segment SGI9182[] = {{&S48607[30], 12}, {NULL}};
const Segment SGI9183[] = {{&S48607[121], 3}, {NULL}};
const Segment SGI9184[] = {{&S48607[86], 6}, {NULL}};
const Segment SGI9185[] = {{&S48608[44], 10}, {NULL}};
const Segment SGI9186[] = {{&S48608[31], 12}, {NULL}};
const Segment SGI9187[] = {{&S48609[50], 10}, {NULL}};
const Segment SGI9188[] = {{&S48609[50], 22}, {NULL}};
const Segment SGI9189[] = {{&S48609[30], 12}, {NULL}};
const Segment SGI9190[] = {{&S48609[121], 3}, {NULL}};
const Segment SGI9191[] = {{&S48609[86], 6}, {NULL}};
const Segment SGI9192[] = {{&S48773[42], 27}, {NULL}};
const Segment SGI9193[] = {{&S48773[42], 38}, {NULL}};
const Segment SGI9194[] = {{&S48773[30], 4}, {NULL}};
const Segment SGI9195[] = {{&S48773[128], 3}, {NULL}};
const Segment SGI9196[] = {{&S48773[94], 5}, {NULL}};
const Segment SGI9197[] = {{&S48610[48], 10}, {NULL}};
const Segment SGI9198[] = {{&S48610[30], 12}, {NULL}};
const Segment SGI9199[] = {{&S48610[119], 3}, {NULL}};
const Segment SGI9200[] = {{&S48610[84], 6}, {NULL}};
const Segment SGI9201[] = {{&S48775[42], 27}, {NULL}};
const Segment SGI9202[] = {{&S48775[42], 38}, {NULL}};
const Segment SGI9203[] = {{&S48611[50], 9}, {NULL}};
const Segment SGI9204[] = {{&S48775[30], 4}, {NULL}};
const Segment SGI9205[] = {{&S48611[50], 21}, {NULL}};
const Segment SGI9206[] = {{&S48611[30], 12}, {NULL}};
const Segment SGI9207[] = {{&S48775[128], 3}, {NULL}};
const Segment SGI9208[] = {{&S48775[94], 5}, {NULL}};
const Segment SGI9209[] = {{&S48611[120], 3}, {NULL}};
const Segment SGI9210[] = {{&S48611[85], 6}, {NULL}};
const Segment SGI9211[] = {{&S48612[50], 9}, {NULL}};
const Segment SGI9212[] = {{&S48612[50], 21}, {NULL}};
const Segment SGI9213[] = {{&S48612[30], 12}, {NULL}};
const Segment SGI9214[] = {{&S48612[120], 3}, {NULL}};
const Segment SGI9215[] = {{&S48612[85], 6}, {NULL}};
const Segment SGI9216[] = {{&S48613[49], 10}, {NULL}};
const Segment SGI9217[] = {{&S48613[49], 22}, {NULL}};
const Segment SGI9218[] = {{&S48613[120], 3}, {NULL}};
const Segment SGI9219[] = {{&S48613[85], 6}, {NULL}};
const Segment SGI9220[] = {{&S49085[44], 11}, {NULL}};
const Segment SGI9221[] = {{&S49085[44], 22}, {NULL}};
const Segment SGI9222[] = {{&S49085[30], 6}, {NULL}};
const Segment SGI9223[] = {{&S49085[114], 3}, {NULL}};
const Segment SGI9224[] = {{&S49085[80], 5}, {NULL}};
const Segment SGI9225[] = {{&S49087[38], 11}, {NULL}};
const Segment SGI9226[] = {{&S49087[31], 6}, {NULL}};
const Segment SGI9227[] = {{&S48685[50], 15}, {NULL}};
const Segment SGI9228[] = {{&S48685[50], 26}, {NULL}};
const Segment SGI9229[] = {{&S48685[30], 12}, {NULL}};
const Segment SGI9230[] = {{&S48685[125], 3}, {NULL}};
const Segment SGI9231[] = {{&S48685[90], 6}, {NULL}};
const Segment SGI9232[] = {{&S48686[50], 13}, {NULL}};
const Segment SGI9233[] = {{&S48686[50], 24}, {NULL}};
const Segment SGI9234[] = {{&S48686[30], 12}, {NULL}};
const Segment SGI9235[] = {{&S48686[123], 3}, {NULL}};
const Segment SGI9236[] = {{&S48686[88], 6}, {NULL}};
const Segment SGI9237[] = {{&S48687[50], 14}, {NULL}};
const Segment SGI9238[] = {{&S48687[50], 25}, {NULL}};
const Segment SGI9239[] = {{&S48687[30], 12}, {NULL}};
const Segment SGI9240[] = {{&S48687[124], 3}, {NULL}};
const Segment SGI9241[] = {{&S48687[89], 6}, {NULL}};
const Segment SGI9242[] = {{&S48688[44], 13}, {NULL}};
const Segment SGI9243[] = {{&S48688[31], 12}, {NULL}};
const Segment SGI9244[] = {{&S49269[44], 7}, {NULL}};
const Segment SGI9245[] = {{&S49269[44], 18}, {NULL}};
const Segment SGI9246[] = {{&S49269[30], 6}, {NULL}};
const Segment SGI9247[] = {{&S49269[110], 3}, {NULL}};
const Segment SGI9248[] = {{&S49269[76], 5}, {NULL}};
const Segment SGI9249[] = {{&S49270[44], 7}, {NULL}};
const Segment SGI9250[] = {{&S49270[44], 16}, {NULL}};
const Segment SGI9251[] = {{&S49270[30], 6}, {NULL}};
const Segment SGI9252[] = {{&S49270[108], 3}, {NULL}};
const Segment SGI9253[] = {{&S49270[74], 5}, {NULL}};
const Segment SGI9254[] = {{&S49272[44], 7}, {NULL}};
const Segment SGI9255[] = {{&S49272[44], 18}, {NULL}};
const Segment SGI9256[] = {{&S49272[30], 6}, {NULL}};
const Segment SGI9257[] = {{&S49272[112], 3}, {NULL}};
const Segment SGI9258[] = {{&S49272[76], 7}, {NULL}};
const Segment SGI9259[] = {{&S49272[132], 8}, {NULL}};
const Segment SGI9260[] = {{&S49274[38], 7}, {NULL}};
const Segment SGI9261[] = {{&S49274[31], 6}, {NULL}};
const Segment SGI9262[] = {{&S49319[44], 11}, {NULL}};
const Segment SGI9263[] = {{&S49319[44], 22}, {NULL}};
const Segment SGI9264[] = {{&S49319[30], 6}, {NULL}};
const Segment SGI9265[] = {{&S49319[114], 3}, {NULL}};
const Segment SGI9266[] = {{&S49319[80], 5}, {NULL}};
const Segment SGI9267[] = {{&S49322[38], 11}, {NULL}};
const Segment SGI9268[] = {{&S49322[31], 6}, {NULL}};
const Segment SGI9269[] = {{&S48994[42], 29}, {NULL}};
const Segment SGI9270[] = {{&S48994[42], 40}, {NULL}};
const Segment SGI9271[] = {{&S48994[30], 4}, {NULL}};
const Segment SGI9272[] = {{&S48994[130], 3}, {NULL}};
const Segment SGI9273[] = {{&S48994[96], 5}, {NULL}};
const Segment SGI9274[] = {{&S50239[77], 5}, {NULL}};
const Segment SGI9275[] = {{&S50319[53], 9}, {NULL}};
const Segment SGI9276[] = {{&S50319[53], 20}, {NULL}};
const Segment SGI9277[] = {{&S50319[121], 3}, {NULL}};
const Segment SGI9278[] = {{&S50319[87], 5}, {NULL}};
const Segment SGI9279[] = {{&S50321[46], 9}, {NULL}};
const Segment SGI9280[] = {{&S50321[46], 21}, {NULL}};
const Segment SGI9281[] = {{&S50321[30], 8}, {NULL}};
const Segment SGI9282[] = {{&S50321[115], 3}, {NULL}};
const Segment SGI9283[] = {{&S50321[81], 5}, {NULL}};
const Segment SGI9284[] = {{&S52660[9], 1}, {NULL}};
const Segment SGI9285[] = {{&S52660[52], 17}, {NULL}};
const Segment SGI9286[] = {{&S52660[40], 4}, {NULL}};
const Segment SGI9287[] = {{&S52660[118], 3}, {NULL}};
const Segment SGI9288[] = {{&S52660[83], 6}, {NULL}};
const Segment SGI9289[] = {{&S52660[138], 8}, {NULL}};
const Segment SGI9290[] = {{&S52661[42], 14}, {NULL}};
const Segment SGI9291[] = {{&S52661[42], 27}, {NULL}};
const Segment SGI9292[] = {{&S52661[30], 4}, {NULL}};
const Segment SGI9293[] = {{&S52661[118], 3}, {NULL}};
const Segment SGI9294[] = {{&S52661[83], 6}, {NULL}};
const Segment SGI9295[] = {{&S52661[138], 8}, {NULL}};
const Segment SGI9296[] = {{&S52662[42], 14}, {NULL}};
const Segment SGI9297[] = {{&S52662[42], 27}, {NULL}};
const Segment SGI9298[] = {{&S52662[30], 4}, {NULL}};
const Segment SGI9299[] = {{&S52662[118], 3}, {NULL}};
const Segment SGI9300[] = {{&S52662[83], 6}, {NULL}};
const Segment SGI9301[] = {{&S52662[138], 8}, {NULL}};
const Segment SGI9302[] = {{&S52663[42], 11}, {NULL}};
const Segment SGI9303[] = {{&S52663[42], 24}, {NULL}};
const Segment SGI9304[] = {{&S52663[30], 4}, {NULL}};
const Segment SGI9305[] = {{&S52663[115], 3}, {NULL}};
const Segment SGI9306[] = {{&S52663[80], 6}, {NULL}};
const Segment SGI9307[] = {{&S52663[135], 8}, {NULL}};
const Segment SGI9308[] = {{&S52664[42], 11}, {NULL}};
const Segment SGI9309[] = {{&S52664[42], 24}, {NULL}};
const Segment SGI9310[] = {{&S52664[30], 4}, {NULL}};
const Segment SGI9311[] = {{&S52664[115], 3}, {NULL}};
const Segment SGI9312[] = {{&S52664[80], 6}, {NULL}};
const Segment SGI9313[] = {{&S52664[135], 8}, {NULL}};
const Segment SGI9314[] = {{&S52665[9], 1}, {NULL}};
const Segment SGI9315[] = {{&S52665[52], 14}, {NULL}};
const Segment SGI9316[] = {{&S52665[40], 4}, {NULL}};
const Segment SGI9317[] = {{&S52665[115], 3}, {NULL}};
const Segment SGI9318[] = {{&S52665[80], 6}, {NULL}};
const Segment SGI9319[] = {{&S52665[135], 8}, {NULL}};
const Segment SGI9320[] = {{&S52666[42], 11}, {NULL}};
const Segment SGI9321[] = {{&S52666[42], 21}, {NULL}};
const Segment SGI9322[] = {{&S52666[30], 4}, {NULL}};
const Segment SGI9323[] = {{&S52666[112], 3}, {NULL}};
const Segment SGI9324[] = {{&S52666[77], 6}, {NULL}};
const Segment SGI9325[] = {{&S52666[132], 8}, {NULL}};
const Segment SGI9326[] = {{&S52667[42], 11}, {NULL}};
const Segment SGI9327[] = {{&S52667[42], 27}, {NULL}};
const Segment SGI9328[] = {{&S52667[30], 4}, {NULL}};
const Segment SGI9329[] = {{&S49084[42], 11}, {NULL}};
const Segment SGI9330[] = {{&S52667[118], 3}, {NULL}};
const Segment SGI9331[] = {{&S49084[42], 22}, {NULL}};
const Segment SGI9332[] = {{&S49084[30], 4}, {NULL}};
const Segment SGI9333[] = {{&S52667[83], 6}, {NULL}};
const Segment SGI9334[] = {{&S52667[138], 8}, {NULL}};
const Segment SGI9335[] = {{&S49084[112], 3}, {NULL}};
const Segment SGI9336[] = {{&S49084[78], 5}, {NULL}};
const Segment SGI9337[] = {{&S52668[40], 11}, {NULL}};
const Segment SGI9338[] = {{&S52668[30], 4}, {NULL}};
const Segment SGI9339[] = {{&S52668[113], 3}, {NULL}};
const Segment SGI9340[] = {{&S52668[78], 6}, {NULL}};
const Segment SGI9341[] = {{&S52668[133], 8}, {NULL}};
const Segment SGI9342[] = {{&S49086[36], 11}, {NULL}};
const Segment SGI9343[] = {{&S49086[31], 4}, {NULL}};
const Segment SGI9344[] = {{&S52669[42], 9}, {NULL}};
const Segment SGI9345[] = {{&S52669[30], 4}, {NULL}};
const Segment SGI9346[] = {{&S52669[111], 3}, {NULL}};
const Segment SGI9347[] = {{&S52669[76], 6}, {NULL}};
const Segment SGI9348[] = {{&S52669[131], 8}, {NULL}};
const Segment SGI9349[] = {{&S52670[42], 11}, {NULL}};
const Segment SGI9350[] = {{&S52670[42], 22}, {NULL}};
const Segment SGI9351[] = {{&S52670[30], 4}, {NULL}};
const Segment SGI9352[] = {{&S52670[113], 3}, {NULL}};
const Segment SGI9353[] = {{&S52670[78], 6}, {NULL}};
const Segment SGI9354[] = {{&S52670[133], 8}, {NULL}};
const Segment SGI9355[] = {{&S52671[42], 11}, {NULL}};
const Segment SGI9356[] = {{&S52671[42], 22}, {NULL}};
const Segment SGI9357[] = {{&S52671[30], 4}, {NULL}};
const Segment SGI9358[] = {{&S52671[113], 3}, {NULL}};
const Segment SGI9359[] = {{&S52671[78], 6}, {NULL}};
const Segment SGI9360[] = {{&S52671[133], 8}, {NULL}};
const Segment SGI9361[] = {{&S52672[42], 11}, {NULL}};
const Segment SGI9362[] = {{&S52672[42], 22}, {NULL}};
const Segment SGI9363[] = {{&S52672[30], 4}, {NULL}};
const Segment SGI9364[] = {{&S52672[113], 3}, {NULL}};
const Segment SGI9365[] = {{&S52672[78], 6}, {NULL}};
const Segment SGI9366[] = {{&S52672[133], 8}, {NULL}};
const Segment SGI9367[] = {{&S52673[42], 11}, {NULL}};
const Segment SGI9368[] = {{&S52673[42], 26}, {NULL}};
const Segment SGI9369[] = {{&S52673[30], 4}, {NULL}};
const Segment SGI9370[] = {{&S52673[117], 3}, {NULL}};
const Segment SGI9371[] = {{&S52673[82], 6}, {NULL}};
const Segment SGI9372[] = {{&S52673[137], 8}, {NULL}};
const Segment SGI9373[] = {{&S52674[85], 11}, {NULL}};
const Segment SGI9374[] = {{&S52674[42], 19}, {&S52674[85], 22}, {NULL}};
const Segment SGI9375[] = {{&S52674[30], 4}, {&S52674[30], 4}, {NULL}};
const Segment SGI9376[] = {{&S52674[156], 3}, {&S52674[156], 3}, {NULL}};
const Segment SGI9377[] = {{&S52674[121], 6}, {&S52674[121], 6}, {NULL}};
const Segment SGI9378[] = {{&S52674[176], 8}, {&S52674[176], 8}, {NULL}};
const Segment SGI9379[] = {{&S52675[42], 11}, {NULL}};
const Segment SGI9380[] = {{&S52675[42], 26}, {NULL}};
const Segment SGI9381[] = {{&S52675[30], 4}, {NULL}};
const Segment SGI9382[] = {{&S52675[117], 3}, {NULL}};
const Segment SGI9383[] = {{&S52675[82], 5}, {NULL}};
const Segment SGI9384[] = {{&S52675[137], 8}, {NULL}};
const Segment SGI9385[] = {{&S52706[42], 11}, {NULL}};
const Segment SGI9386[] = {{&S52706[42], 24}, {NULL}};
const Segment SGI9387[] = {{&S52706[30], 4}, {NULL}};
const Segment SGI9388[] = {{&S52706[115], 3}, {NULL}};
const Segment SGI9389[] = {{&S52706[80], 6}, {NULL}};
const Segment SGI9390[] = {{&S52706[135], 8}, {NULL}};
const Segment SGI9391[] = {{&S52707[42], 10}, {NULL}};
const Segment SGI9392[] = {{&S52707[42], 23}, {NULL}};
const Segment SGI9393[] = {{&S52707[30], 4}, {NULL}};
const Segment SGI9394[] = {{&S52707[114], 3}, {NULL}};
const Segment SGI9395[] = {{&S52707[79], 6}, {NULL}};
const Segment SGI9396[] = {{&S52707[134], 8}, {NULL}};
const Segment SGI9397[] = {{&S52708[42], 29}, {NULL}};
const Segment SGI9398[] = {{&S52708[42], 42}, {NULL}};
const Segment SGI9399[] = {{&S52708[30], 4}, {NULL}};
const Segment SGI9400[] = {{&S52708[133], 3}, {NULL}};
const Segment SGI9401[] = {{&S52708[98], 6}, {NULL}};
const Segment SGI9402[] = {{&S52708[153], 8}, {NULL}};
const Segment SGI9403[] = {{&S52709[42], 11}, {NULL}};
const Segment SGI9404[] = {{&S52709[42], 24}, {NULL}};
const Segment SGI9405[] = {{&S52709[30], 4}, {NULL}};
const Segment SGI9406[] = {{&S52709[115], 3}, {NULL}};
const Segment SGI9407[] = {{&S52709[80], 6}, {NULL}};
const Segment SGI9408[] = {{&S52709[135], 8}, {NULL}};
const Segment SGI9409[] = {{&S52710[40], 10}, {NULL}};
const Segment SGI9410[] = {{&S52710[30], 4}, {NULL}};
const Segment SGI9411[] = {{&S52710[112], 3}, {NULL}};
const Segment SGI9412[] = {{&S52710[77], 6}, {NULL}};
const Segment SGI9413[] = {{&S52710[132], 8}, {NULL}};
const Segment SGI9414[] = {{&S52711[40], 11}, {NULL}};
const Segment SGI9415[] = {{&S52711[30], 4}, {NULL}};
const Segment SGI9416[] = {{&S52711[113], 3}, {NULL}};
const Segment SGI9417[] = {{&S52711[78], 6}, {NULL}};
const Segment SGI9418[] = {{&S52711[133], 8}, {NULL}};
const Segment SGI9419[] = {{&S52712[42], 10}, {NULL}};
const Segment SGI9420[] = {{&S52712[42], 23}, {NULL}};
const Segment SGI9421[] = {{&S52712[30], 4}, {NULL}};
const Segment SGI9422[] = {{&S52712[114], 3}, {NULL}};
const Segment SGI9423[] = {{&S52712[79], 5}, {NULL}};
const Segment SGI9424[] = {{&S52712[134], 8}, {NULL}};
const Segment SGI9425[] = {{&S49271[42], 7}, {NULL}};
const Segment SGI9426[] = {{&S49271[42], 18}, {NULL}};
const Segment SGI9427[] = {{&S49271[30], 4}, {NULL}};
const Segment SGI9428[] = {{&S49271[108], 3}, {NULL}};
const Segment SGI9429[] = {{&S49271[74], 5}, {NULL}};
const Segment SGI9430[] = {{&S49317[42], 10}, {NULL}};
const Segment SGI9431[] = {{&S49317[42], 21}, {NULL}};
const Segment SGI9432[] = {{&S49317[30], 4}, {NULL}};
const Segment SGI9433[] = {{&S49317[111], 3}, {NULL}};
const Segment SGI9434[] = {{&S49317[77], 5}, {NULL}};
const Segment SGI9435[] = {{&S49318[42], 11}, {NULL}};
const Segment SGI9436[] = {{&S49318[42], 22}, {NULL}};
const Segment SGI9437[] = {{&S49318[30], 4}, {NULL}};
const Segment SGI9438[] = {{&S49318[112], 3}, {NULL}};
const Segment SGI9439[] = {{&S49318[78], 5}, {NULL}};
const Segment SGI9440[] = {{&S49320[36], 11}, {NULL}};
const Segment SGI9441[] = {{&S49320[31], 4}, {NULL}};
const Segment SGI9442[] = {{&S49321[36], 10}, {NULL}};
const Segment SGI9443[] = {{&S49321[31], 4}, {NULL}};
const Segment SGI9444[] = {{&S49364[42], 18}, {NULL}};
const Segment SGI9445[] = {{&S49364[42], 29}, {NULL}};
const Segment SGI9446[] = {{&S49364[30], 4}, {NULL}};
const Segment SGI9447[] = {{&S49364[119], 3}, {NULL}};
const Segment SGI9448[] = {{&S49364[85], 5}, {NULL}};
const Segment SGI9449[] = {{&S49365[40], 18}, {NULL}};
const Segment SGI9450[] = {{&S49365[30], 4}, {NULL}};
const Segment SGI9451[] = {{&S49365[117], 3}, {NULL}};
const Segment SGI9452[] = {{&S49365[83], 5}, {NULL}};
const Segment SGI9453[] = {{&S49366[36], 18}, {NULL}};
const Segment SGI9454[] = {{&S49366[31], 4}, {NULL}};
const Segment SGI9455[] = {{&S49352[50], 17}, {NULL}};
const Segment SGI9456[] = {{&S49352[50], 28}, {NULL}};
const Segment SGI9457[] = {{&S49352[30], 12}, {NULL}};
const Segment SGI9458[] = {{&S49352[127], 3}, {NULL}};
const Segment SGI9459[] = {{&S49352[92], 6}, {NULL}};
const Segment SGI9460[] = {{&S49353[44], 17}, {NULL}};
const Segment SGI9461[] = {{&S49353[31], 12}, {NULL}};
const Segment SGI9462[] = {{&S49354[48], 17}, {NULL}};
const Segment SGI9463[] = {{&S49354[30], 12}, {NULL}};
const Segment SGI9464[] = {{&S49354[125], 3}, {NULL}};
const Segment SGI9465[] = {{&S49354[90], 6}, {NULL}};
const Segment SGI9466[] = {{&S53126[42], 12}, {NULL}};
const Segment SGI9467[] = {{&S53126[42], 23}, {NULL}};
const Segment SGI9468[] = {{&S53126[30], 4}, {NULL}};
const Segment SGI9469[] = {{&S53126[114], 3}, {NULL}};
const Segment SGI9470[] = {{&S53126[79], 6}, {NULL}};
const Segment SGI9471[] = {{&S53127[42], 12}, {NULL}};
const Segment SGI9472[] = {{&S53127[42], 23}, {NULL}};
const Segment SGI9473[] = {{&S53127[30], 4}, {NULL}};
const Segment SGI9474[] = {{&S53127[114], 3}, {NULL}};
const Segment SGI9475[] = {{&S53127[79], 6}, {NULL}};
const Segment SGI9476[] = {{&S53127[134], 8}, {NULL}};
const Segment SGI9477[] = {{&S53128[42], 11}, {NULL}};
const Segment SGI9478[] = {{&S53128[42], 22}, {NULL}};
const Segment SGI9479[] = {{&S53128[30], 4}, {NULL}};
const Segment SGI9480[] = {{&S53128[113], 3}, {NULL}};
const Segment SGI9481[] = {{&S53128[78], 6}, {NULL}};
const Segment SGI9482[] = {{&S53128[133], 8}, {NULL}};
const Segment SGI9483[] = {{&S53129[41], 12}, {NULL}};
const Segment SGI9484[] = {{&S53129[41], 23}, {NULL}};
const Segment SGI9485[] = {{&S53129[113], 3}, {NULL}};
const Segment SGI9486[] = {{&S53129[78], 6}, {NULL}};
const Segment SGI9487[] = {{&S53129[133], 8}, {NULL}};
const Segment SGI9488[] = {{&S53130[41], 12}, {NULL}};
const Segment SGI9489[] = {{&S53130[41], 23}, {NULL}};
const Segment SGI9490[] = {{&S53130[30], 3}, {NULL}};
const Segment SGI9491[] = {{&S53130[113], 3}, {NULL}};
const Segment SGI9492[] = {{&S53130[78], 6}, {NULL}};
const Segment SGI9493[] = {{&S53130[133], 8}, {NULL}};
const Segment SGI9494[] = {{&S53131[42], 11}, {NULL}};
const Segment SGI9495[] = {{&S53131[42], 22}, {NULL}};
const Segment SGI9496[] = {{&S53131[30], 4}, {NULL}};
const Segment SGI9497[] = {{&S53131[113], 3}, {NULL}};
const Segment SGI9498[] = {{&S53131[78], 6}, {NULL}};
const Segment SGI9499[] = {{&S53131[133], 8}, {NULL}};
const Segment SGI9500[] = {{&S53132[42], 14}, {NULL}};
const Segment SGI9501[] = {{&S53132[42], 25}, {NULL}};
const Segment SGI9502[] = {{&S53132[30], 4}, {NULL}};
const Segment SGI9503[] = {{&S53132[116], 3}, {NULL}};
const Segment SGI9504[] = {{&S53132[81], 6}, {NULL}};
const Segment SGI9505[] = {{&S53132[136], 8}, {NULL}};
const Segment SGI9506[] = {{&S51898[42], 7}, {NULL}};
const Segment SGI9507[] = {{&S51898[42], 18}, {NULL}};
const Segment SGI9508[] = {{&S51898[30], 4}, {NULL}};
const Segment SGI9509[] = {{&S51898[109], 3}, {NULL}};
const Segment SGI9510[] = {{&S51898[74], 6}, {NULL}};
const Segment SGI9511[] = {{&S51899[44], 7}, {NULL}};
const Segment SGI9512[] = {{&S51899[44], 18}, {NULL}};
const Segment SGI9513[] = {{&S51899[30], 6}, {NULL}};
const Segment SGI9514[] = {{&S51899[111], 3}, {NULL}};
const Segment SGI9515[] = {{&S51899[76], 6}, {NULL}};
const Segment SGI9516[] = {{&S51900[42], 7}, {NULL}};
const Segment SGI9517[] = {{&S51900[42], 18}, {NULL}};
const Segment SGI9518[] = {{&S51900[30], 4}, {NULL}};
const Segment SGI9519[] = {{&S51900[109], 3}, {NULL}};
const Segment SGI9520[] = {{&S51900[74], 6}, {NULL}};
const Segment SGI9521[] = {{&S51901[42], 7}, {NULL}};
const Segment SGI9522[] = {{&S51901[42], 18}, {NULL}};
const Segment SGI9523[] = {{&S51901[30], 4}, {NULL}};
const Segment SGI9524[] = {{&S51901[110], 3}, {NULL}};
const Segment SGI9525[] = {{&S51901[74], 7}, {NULL}};
const Segment SGI9526[] = {{&S51901[130], 8}, {NULL}};
const Segment SGI9527[] = {{&S51903[42], 7}, {NULL}};
const Segment SGI9528[] = {{&S51903[42], 18}, {NULL}};
const Segment SGI9529[] = {{&S51903[30], 4}, {NULL}};
const Segment SGI9530[] = {{&S51903[109], 3}, {NULL}};
const Segment SGI9531[] = {{&S51903[74], 6}, {NULL}};
const Segment SGI9532[] = {{&S49494[50], 12}, {NULL}};
const Segment SGI9533[] = {{&S49494[50], 23}, {NULL}};
const Segment SGI9534[] = {{&S49494[30], 12}, {NULL}};
const Segment SGI9535[] = {{&S49494[122], 3}, {NULL}};
const Segment SGI9536[] = {{&S49494[87], 6}, {NULL}};
const Segment SGI9537[] = {{&S49495[50], 12}, {NULL}};
const Segment SGI9538[] = {{&S49495[50], 23}, {NULL}};
const Segment SGI9539[] = {{&S49495[30], 12}, {NULL}};
const Segment SGI9540[] = {{&S49495[122], 3}, {NULL}};
const Segment SGI9541[] = {{&S49495[87], 6}, {NULL}};
const Segment SGI9542[] = {{&S49496[50], 11}, {NULL}};
const Segment SGI9543[] = {{&S49496[50], 22}, {NULL}};
const Segment SGI9544[] = {{&S49496[30], 12}, {NULL}};
const Segment SGI9545[] = {{&S49496[121], 3}, {NULL}};
const Segment SGI9546[] = {{&S49496[86], 6}, {NULL}};
const Segment SGI9547[] = {{&S49497[44], 12}, {NULL}};
const Segment SGI9548[] = {{&S49497[31], 12}, {NULL}};
const Segment SGI9549[] = {{&S49498[49], 12}, {NULL}};
const Segment SGI9550[] = {{&S49498[49], 23}, {NULL}};
const Segment SGI9551[] = {{&S49498[121], 3}, {NULL}};
const Segment SGI9552[] = {{&S49498[86], 6}, {NULL}};
const Segment SGI9553[] = {{&S49499[49], 12}, {NULL}};
const Segment SGI9554[] = {{&S49499[49], 23}, {NULL}};
const Segment SGI9555[] = {{&S49499[30], 11}, {NULL}};
const Segment SGI9556[] = {{&S49499[121], 3}, {NULL}};
const Segment SGI9557[] = {{&S49499[86], 6}, {NULL}};
const Segment SGI9558[] = {{&S49500[50], 11}, {NULL}};
const Segment SGI9559[] = {{&S49500[50], 22}, {NULL}};
const Segment SGI9560[] = {{&S49500[30], 12}, {NULL}};
const Segment SGI9561[] = {{&S49500[121], 3}, {NULL}};
const Segment SGI9562[] = {{&S49500[86], 6}, {NULL}};
const Segment SGI9563[] = {{&S50019[48], 7}, {NULL}};
const Segment SGI9564[] = {{&S50019[30], 12}, {NULL}};
const Segment SGI9565[] = {{&S50019[115], 3}, {NULL}};
const Segment SGI9566[] = {{&S50019[80], 6}, {NULL}};
const Segment SGI9567[] = {{&S50020[60], 7}, {NULL}};
const Segment SGI9568[] = {{&S50020[60], 18}, {NULL}};
const Segment SGI9569[] = {{&S50020[30], 21}, {NULL}};
const Segment SGI9570[] = {{&S50020[127], 3}, {NULL}};
const Segment SGI9571[] = {{&S50020[92], 6}, {NULL}};
const Segment SGI9572[] = {{&S50021[48], 7}, {NULL}};
const Segment SGI9573[] = {{&S50021[48], 18}, {NULL}};
const Segment SGI9574[] = {{&S50021[30], 10}, {NULL}};
const Segment SGI9575[] = {{&S50021[115], 3}, {NULL}};
const Segment SGI9576[] = {{&S50021[80], 6}, {NULL}};
const Segment SGI9577[] = {{&S50022[50], 7}, {NULL}};
const Segment SGI9578[] = {{&S50022[50], 18}, {NULL}};
const Segment SGI9579[] = {{&S50022[30], 12}, {NULL}};
const Segment SGI9580[] = {{&S50022[117], 3}, {NULL}};
const Segment SGI9581[] = {{&S50022[82], 6}, {NULL}};
const Segment SGI9582[] = {{&S53973[42], 24}, {NULL}};
const Segment SGI9583[] = {{&S53973[42], 35}, {NULL}};
const Segment SGI9584[] = {{&S53973[30], 4}, {NULL}};
const Segment SGI9585[] = {{&S53973[126], 3}, {NULL}};
const Segment SGI9586[] = {{&S53973[91], 6}, {NULL}};
const Segment SGI9587[] = {{&S53973[146], 8}, {NULL}};
const Segment SGI9588[] = {{&S53974[42], 24}, {NULL}};
const Segment SGI9589[] = {{&S53974[42], 35}, {NULL}};
const Segment SGI9590[] = {{&S53974[30], 4}, {NULL}};
const Segment SGI9591[] = {{&S53974[126], 3}, {NULL}};
const Segment SGI9592[] = {{&S53974[91], 6}, {NULL}};
const Segment SGI9593[] = {{&S53974[146], 8}, {NULL}};
const Segment SGI9594[] = {{&S53975[42], 24}, {NULL}};
const Segment SGI9595[] = {{&S53975[42], 35}, {NULL}};
const Segment SGI9596[] = {{&S53975[30], 4}, {NULL}};
const Segment SGI9597[] = {{&S53975[126], 3}, {NULL}};
const Segment SGI9598[] = {{&S53975[91], 6}, {NULL}};
const Segment SGI9599[] = {{&S53975[146], 8}, {NULL}};
const Segment SGI9600[] = {{&S53976[42], 23}, {NULL}};
const Segment SGI9601[] = {{&S53976[42], 34}, {NULL}};
const Segment SGI9602[] = {{&S53976[30], 4}, {NULL}};
const Segment SGI9603[] = {{&S53976[125], 3}, {NULL}};
const Segment SGI9604[] = {{&S53976[90], 6}, {NULL}};
const Segment SGI9605[] = {{&S53976[145], 8}, {NULL}};
const Segment SGI9606[] = {{&S53977[42], 24}, {NULL}};
const Segment SGI9607[] = {{&S53977[42], 35}, {NULL}};
const Segment SGI9608[] = {{&S53977[30], 4}, {NULL}};
const Segment SGI9609[] = {{&S53977[126], 3}, {NULL}};
const Segment SGI9610[] = {{&S53977[91], 6}, {NULL}};
const Segment SGI9611[] = {{&S53977[146], 8}, {NULL}};
const Segment SGI9612[] = {{&S53978[42], 24}, {NULL}};
const Segment SGI9613[] = {{&S53978[42], 35}, {NULL}};
const Segment SGI9614[] = {{&S53978[30], 4}, {NULL}};
const Segment SGI9615[] = {{&S53978[126], 3}, {NULL}};
const Segment SGI9616[] = {{&S53978[91], 6}, {NULL}};
const Segment SGI9617[] = {{&S53978[146], 8}, {NULL}};
const Segment SGI9618[] = {{&S53979[42], 23}, {NULL}};
const Segment SGI9619[] = {{&S53979[42], 34}, {NULL}};
const Segment SGI9620[] = {{&S53979[30], 4}, {NULL}};
const Segment SGI9621[] = {{&S53979[125], 3}, {NULL}};
const Segment SGI9622[] = {{&S53979[90], 6}, {NULL}};
const Segment SGI9623[] = {{&S53979[145], 8}, {NULL}};
const Segment SGI9624[] = {{&S53980[42], 23}, {NULL}};
const Segment SGI9625[] = {{&S53980[42], 34}, {NULL}};
const Segment SGI9626[] = {{&S53980[30], 4}, {NULL}};
const Segment SGI9627[] = {{&S53980[125], 3}, {NULL}};
const Segment SGI9628[] = {{&S53980[90], 6}, {NULL}};
const Segment SGI9629[] = {{&S53980[145], 8}, {NULL}};
const Segment SGI9630[] = {{&S53981[42], 18}, {NULL}};
const Segment SGI9631[] = {{&S53981[42], 29}, {NULL}};
const Segment SGI9632[] = {{&S53981[30], 4}, {NULL}};
const Segment SGI9633[] = {{&S53981[120], 3}, {NULL}};
const Segment SGI9634[] = {{&S53981[85], 6}, {NULL}};
const Segment SGI9635[] = {{&S53981[140], 8}, {NULL}};
const Segment SGI9636[] = {{&S53982[42], 19}, {NULL}};
const Segment SGI9637[] = {{&S53982[42], 30}, {NULL}};
const Segment SGI9638[] = {{&S53982[30], 4}, {NULL}};
const Segment SGI9639[] = {{&S53982[121], 3}, {NULL}};
const Segment SGI9640[] = {{&S53982[86], 6}, {NULL}};
const Segment SGI9641[] = {{&S53982[141], 8}, {NULL}};
const Segment SGI9642[] = {{&S53983[42], 12}, {NULL}};
const Segment SGI9643[] = {{&S53983[42], 23}, {NULL}};
const Segment SGI9644[] = {{&S53983[30], 4}, {NULL}};
const Segment SGI9645[] = {{&S53983[114], 3}, {NULL}};
const Segment SGI9646[] = {{&S53983[79], 6}, {NULL}};
const Segment SGI9647[] = {{&S53983[134], 8}, {NULL}};
const Segment SGI9648[] = {{&S53984[42], 18}, {NULL}};
const Segment SGI9649[] = {{&S53984[42], 29}, {NULL}};
const Segment SGI9650[] = {{&S53984[30], 4}, {NULL}};
const Segment SGI9651[] = {{&S53984[120], 3}, {NULL}};
const Segment SGI9652[] = {{&S53984[85], 5}, {NULL}};
const Segment SGI9653[] = {{&S53984[140], 8}, {NULL}};
const Segment SGI9654[] = {{&S50693[44], 15}, {NULL}};
const Segment SGI9655[] = {{&S50693[44], 27}, {NULL}};
const Segment SGI9656[] = {{&S50693[30], 6}, {NULL}};
const Segment SGI9657[] = {{&S50693[119], 3}, {NULL}};
const Segment SGI9658[] = {{&S50693[85], 5}, {NULL}};
const Segment SGI9659[] = {{&S50698[44], 15}, {NULL}};
const Segment SGI9660[] = {{&S50698[44], 36}, {NULL}};
const Segment SGI9661[] = {{&S50698[30], 6}, {NULL}};
const Segment SGI9662[] = {{&S50201[50], 6}, {NULL}};
const Segment SGI9663[] = {{&S50201[50], 23}, {NULL}};
const Segment SGI9664[] = {{&S50698[128], 3}, {NULL}};
const Segment SGI9665[] = {{&S50201[30], 12}, {NULL}};
const Segment SGI9666[] = {{&S50698[94], 5}, {NULL}};
const Segment SGI9667[] = {{&S50201[122], 3}, {NULL}};
const Segment SGI9668[] = {{&S50201[87], 6}, {NULL}};
const Segment SGI9669[] = {{&S50203[50], 6}, {NULL}};
const Segment SGI9670[] = {{&S50203[50], 24}, {NULL}};
const Segment SGI9671[] = {{&S50203[30], 12}, {NULL}};
const Segment SGI9672[] = {{&S50203[123], 3}, {NULL}};
const Segment SGI9673[] = {{&S50203[88], 6}, {NULL}};
const Segment SGI9674[] = {{&S50204[50], 6}, {NULL}};
const Segment SGI9675[] = {{&S50204[50], 24}, {NULL}};
const Segment SGI9676[] = {{&S50204[30], 12}, {NULL}};
const Segment SGI9677[] = {{&S50204[123], 3}, {NULL}};
const Segment SGI9678[] = {{&S50204[88], 6}, {NULL}};
const Segment SGI9679[] = {{&S50205[50], 6}, {NULL}};
const Segment SGI9680[] = {{&S50205[50], 23}, {NULL}};
const Segment SGI9681[] = {{&S50205[30], 12}, {NULL}};
const Segment SGI9682[] = {{&S50205[122], 3}, {NULL}};
const Segment SGI9683[] = {{&S50205[87], 6}, {NULL}};
const Segment SGI9684[] = {{&S50206[44], 6}, {NULL}};
const Segment SGI9685[] = {{&S50206[31], 12}, {NULL}};
const Segment SGI9686[] = {{&S50945[44], 17}, {NULL}};
const Segment SGI9687[] = {{&S50945[44], 28}, {NULL}};
const Segment SGI9688[] = {{&S50945[30], 6}, {NULL}};
const Segment SGI9689[] = {{&S50945[120], 3}, {NULL}};
const Segment SGI9690[] = {{&S50945[86], 5}, {NULL}};
const Segment SGI9691[] = {{&S50946[44], 25}, {NULL}};
const Segment SGI9692[] = {{&S50946[44], 36}, {NULL}};
const Segment SGI9693[] = {{&S50946[30], 6}, {NULL}};
const Segment SGI9694[] = {{&S50946[128], 3}, {NULL}};
const Segment SGI9695[] = {{&S50946[94], 5}, {NULL}};
const Segment SGI9696[] = {{&S50948[38], 17}, {NULL}};
const Segment SGI9697[] = {{&S50948[31], 6}, {NULL}};
const Segment SGI9698[] = {{&S50423[50], 17}, {NULL}};
const Segment SGI9699[] = {{&S50423[50], 28}, {NULL}};
const Segment SGI9700[] = {{&S50423[30], 12}, {NULL}};
const Segment SGI9701[] = {{&S50423[127], 3}, {NULL}};
const Segment SGI9702[] = {{&S50423[92], 6}, {NULL}};
const Segment SGI9703[] = {{&S50424[48], 17}, {NULL}};
const Segment SGI9704[] = {{&S50424[30], 12}, {NULL}};
const Segment SGI9705[] = {{&S50424[125], 3}, {NULL}};
const Segment SGI9706[] = {{&S50424[90], 6}, {NULL}};
const Segment SGI9707[] = {{&S50425[44], 17}, {NULL}};
const Segment SGI9708[] = {{&S50425[31], 12}, {NULL}};
const Segment SGI9709[] = {{&S51107[44], 4}, {NULL}};
const Segment SGI9710[] = {{&S51107[44], 15}, {NULL}};
const Segment SGI9711[] = {{&S51107[30], 6}, {NULL}};
const Segment SGI9712[] = {{&S51107[107], 3}, {NULL}};
const Segment SGI9713[] = {{&S51107[73], 5}, {NULL}};
const Segment SGI9714[] = {{&S51108[44], 10}, {NULL}};
const Segment SGI9715[] = {{&S51108[44], 21}, {NULL}};
const Segment SGI9716[] = {{&S51108[30], 6}, {NULL}};
const Segment SGI9717[] = {{&S51108[113], 3}, {NULL}};
const Segment SGI9718[] = {{&S51108[79], 5}, {NULL}};
const Segment SGI9719[] = {{&S51902[45], 7}, {NULL}};
const Segment SGI9720[] = {{&S51902[30], 7}, {NULL}};
const Segment SGI9721[] = {{&S51902[102], 3}, {NULL}};
const Segment SGI9722[] = {{&S51902[66], 7}, {NULL}};
const Segment SGI9723[] = {{&S51902[122], 8}, {NULL}};
const Segment SGI9724[] = {{&S50689[42], 15}, {NULL}};
const Segment SGI9725[] = {{&S50689[42], 25}, {NULL}};
const Segment SGI9726[] = {{&S50689[30], 4}, {NULL}};
const Segment SGI9727[] = {{&S50689[117], 3}, {NULL}};
const Segment SGI9728[] = {{&S50689[81], 6}, {NULL}};
const Segment SGI9729[] = {{&S50689[137], 8}, {NULL}};
const Segment SGI9730[] = {{&S50690[42], 15}, {NULL}};
const Segment SGI9731[] = {{&S50690[42], 25}, {NULL}};
const Segment SGI9732[] = {{&S50690[30], 4}, {NULL}};
const Segment SGI9733[] = {{&S50690[115], 3}, {NULL}};
const Segment SGI9734[] = {{&S50690[81], 5}, {NULL}};
const Segment SGI9735[] = {{&S50691[36], 15}, {NULL}};
const Segment SGI9736[] = {{&S50691[31], 4}, {NULL}};
const Segment SGI9737[] = {{&S50692[41], 15}, {NULL}};
const Segment SGI9738[] = {{&S50692[41], 25}, {NULL}};
const Segment SGI9739[] = {{&S50692[30], 3}, {NULL}};
const Segment SGI9740[] = {{&S50692[114], 3}, {NULL}};
const Segment SGI9741[] = {{&S50692[80], 5}, {NULL}};
const Segment SGI9742[] = {{&S50694[42], 14}, {NULL}};
const Segment SGI9743[] = {{&S50694[42], 24}, {NULL}};
const Segment SGI9744[] = {{&S50694[30], 4}, {NULL}};
const Segment SGI9745[] = {{&S50694[114], 3}, {NULL}};
const Segment SGI9746[] = {{&S50694[80], 5}, {NULL}};
const Segment SGI9747[] = {{&S50695[42], 14}, {NULL}};
const Segment SGI9748[] = {{&S50695[42], 24}, {NULL}};
const Segment SGI9749[] = {{&S50695[30], 4}, {NULL}};
const Segment SGI9750[] = {{&S50695[114], 3}, {NULL}};
const Segment SGI9751[] = {{&S50695[80], 5}, {NULL}};
const Segment SGI9752[] = {{&S50696[42], 14}, {NULL}};
const Segment SGI9753[] = {{&S50696[42], 24}, {NULL}};
const Segment SGI9754[] = {{&S50696[30], 4}, {NULL}};
const Segment SGI9755[] = {{&S50696[114], 3}, {NULL}};
const Segment SGI9756[] = {{&S50696[80], 5}, {NULL}};
const Segment SGI9757[] = {{&S50697[42], 15}, {NULL}};
const Segment SGI9758[] = {{&S50697[42], 25}, {NULL}};
const Segment SGI9759[] = {{&S50697[30], 4}, {NULL}};
const Segment SGI9760[] = {{&S50697[115], 3}, {NULL}};
const Segment SGI9761[] = {{&S50697[81], 5}, {NULL}};
const Segment SGI9762[] = {{&S52023[6], 4}, {NULL}};
const Segment SGI9763[] = {{&S52023[32], 3}, {NULL}};
const Segment SGI9764[] = {{&S52023[81], 5}, {NULL}};
const Segment SGI9765[] = {{&S51375[44], 13}, {NULL}};
const Segment SGI9766[] = {{&S51375[44], 24}, {NULL}};
const Segment SGI9767[] = {{&S51375[30], 6}, {NULL}};
const Segment SGI9768[] = {{&S51375[116], 3}, {NULL}};
const Segment SGI9769[] = {{&S51375[82], 5}, {NULL}};
const Segment SGI9770[] = {{&S51377[44], 9}, {NULL}};
const Segment SGI9771[] = {{&S51377[44], 20}, {NULL}};
const Segment SGI9772[] = {{&S51377[30], 6}, {NULL}};
const Segment SGI9773[] = {{&S51377[112], 3}, {NULL}};
const Segment SGI9774[] = {{&S51377[78], 5}, {NULL}};
const Segment SGI9775[] = {{&S51058[50], 11}, {NULL}};
const Segment SGI9776[] = {{&S51058[50], 23}, {NULL}};
const Segment SGI9777[] = {{&S51058[30], 12}, {NULL}};
const Segment SGI9778[] = {{&S51058[122], 3}, {NULL}};
const Segment SGI9779[] = {{&S51058[87], 6}, {NULL}};
const Segment SGI9780[] = {{&S51059[44], 11}, {NULL}};
const Segment SGI9781[] = {{&S51059[31], 12}, {NULL}};
const Segment SGI9782[] = {{&S51087[50], 7}, {NULL}};
const Segment SGI9783[] = {{&S51087[50], 29}, {NULL}};
const Segment SGI9784[] = {{&S51087[30], 12}, {NULL}};
const Segment SGI9785[] = {{&S51087[128], 3}, {NULL}};
const Segment SGI9786[] = {{&S51087[93], 6}, {NULL}};
const Segment SGI9787[] = {{&S51088[61], 7}, {NULL}};
const Segment SGI9788[] = {{&S51088[61], 29}, {NULL}};
const Segment SGI9789[] = {{&S51088[41], 12}, {NULL}};
const Segment SGI9790[] = {{&S51088[139], 3}, {NULL}};
const Segment SGI9791[] = {{&S51088[104], 6}, {NULL}};
const Segment SGI9792[] = {{&S51089[61], 7}, {NULL}};
const Segment SGI9793[] = {{&S51089[61], 29}, {NULL}};
const Segment SGI9794[] = {{&S51089[41], 12}, {NULL}};
const Segment SGI9795[] = {{&S51089[139], 3}, {NULL}};
const Segment SGI9796[] = {{&S51089[104], 6}, {NULL}};
const Segment SGI9797[] = {{&S51090[61], 7}, {NULL}};
const Segment SGI9798[] = {{&S51090[61], 29}, {NULL}};
const Segment SGI9799[] = {{&S51090[41], 12}, {NULL}};
const Segment SGI9800[] = {{&S51090[139], 3}, {NULL}};
const Segment SGI9801[] = {{&S51090[104], 6}, {NULL}};
const Segment SGI9802[] = {{&S51091[50], 7}, {NULL}};
const Segment SGI9803[] = {{&S51091[50], 32}, {NULL}};
const Segment SGI9804[] = {{&S51091[30], 12}, {NULL}};
const Segment SGI9805[] = {{&S51091[131], 3}, {NULL}};
const Segment SGI9806[] = {{&S51091[96], 6}, {NULL}};
const Segment SGI9807[] = {{&S51092[61], 7}, {NULL}};
const Segment SGI9808[] = {{&S51092[61], 29}, {NULL}};
const Segment SGI9809[] = {{&S51092[41], 12}, {NULL}};
const Segment SGI9810[] = {{&S51092[139], 3}, {NULL}};
const Segment SGI9811[] = {{&S51092[104], 6}, {NULL}};
const Segment SGI9812[] = {{&S51093[61], 7}, {NULL}};
const Segment SGI9813[] = {{&S51093[61], 29}, {NULL}};
const Segment SGI9814[] = {{&S51093[41], 12}, {NULL}};
const Segment SGI9815[] = {{&S51093[139], 3}, {NULL}};
const Segment SGI9816[] = {{&S51093[104], 6}, {NULL}};
const Segment SGI9817[] = {{&S51094[61], 7}, {NULL}};
const Segment SGI9818[] = {{&S51094[61], 29}, {NULL}};
const Segment SGI9819[] = {{&S51094[41], 12}, {NULL}};
const Segment SGI9820[] = {{&S51094[139], 3}, {NULL}};
const Segment SGI9821[] = {{&S51094[104], 6}, {NULL}};
const Segment SGI9822[] = {{&S51095[61], 7}, {NULL}};
const Segment SGI9823[] = {{&S51095[61], 29}, {NULL}};
const Segment SGI9824[] = {{&S51095[41], 12}, {NULL}};
const Segment SGI9825[] = {{&S51095[139], 3}, {NULL}};
const Segment SGI9826[] = {{&S51095[104], 6}, {NULL}};
const Segment SGI9827[] = {{&S51096[61], 7}, {NULL}};
const Segment SGI9828[] = {{&S51096[61], 29}, {NULL}};
const Segment SGI9829[] = {{&S51096[41], 12}, {NULL}};
const Segment SGI9830[] = {{&S51096[139], 3}, {NULL}};
const Segment SGI9831[] = {{&S51096[104], 6}, {NULL}};
const Segment SGI9832[] = {{&S51097[61], 7}, {NULL}};
const Segment SGI9833[] = {{&S51097[61], 29}, {NULL}};
const Segment SGI9834[] = {{&S51097[41], 12}, {NULL}};
const Segment SGI9835[] = {{&S51097[139], 3}, {NULL}};
const Segment SGI9836[] = {{&S51097[104], 6}, {NULL}};
const Segment SGI9837[] = {{&S51098[61], 7}, {NULL}};
const Segment SGI9838[] = {{&S51098[61], 29}, {NULL}};
const Segment SGI9839[] = {{&S51098[41], 12}, {NULL}};
const Segment SGI9840[] = {{&S51098[139], 3}, {NULL}};
const Segment SGI9841[] = {{&S51098[104], 6}, {NULL}};
const Segment SGI9842[] = {{&S51099[61], 7}, {NULL}};
const Segment SGI9843[] = {{&S51099[61], 33}, {NULL}};
const Segment SGI9844[] = {{&S51099[41], 12}, {NULL}};
const Segment SGI9845[] = {{&S51099[143], 3}, {NULL}};
const Segment SGI9846[] = {{&S51099[108], 6}, {NULL}};
const Segment SGI9847[] = {{&S51100[61], 7}, {NULL}};
const Segment SGI9848[] = {{&S51100[61], 29}, {NULL}};
const Segment SGI9849[] = {{&S51100[41], 12}, {NULL}};
const Segment SGI9850[] = {{&S51100[139], 3}, {NULL}};
const Segment SGI9851[] = {{&S51100[104], 6}, {NULL}};
const Segment SGI9852[] = {{&S51101[50], 7}, {NULL}};
const Segment SGI9853[] = {{&S51101[50], 29}, {NULL}};
const Segment SGI9854[] = {{&S51101[30], 12}, {NULL}};
const Segment SGI9855[] = {{&S51101[76], 3}, {NULL}};
const Segment SGI9856[] = {{&S51101[93], 6}, {NULL}};
const Segment SGI9857[] = {{&S51102[44], 7}, {NULL}};
const Segment SGI9858[] = {{&S51102[31], 12}, {NULL}};
const Segment SGI9859[] = {{&S51103[50], 7}, {NULL}};
const Segment SGI9860[] = {{&S51103[50], 32}, {NULL}};
const Segment SGI9861[] = {{&S51103[30], 12}, {NULL}};
const Segment SGI9862[] = {{&S51103[131], 3}, {NULL}};
const Segment SGI9863[] = {{&S51103[96], 6}, {NULL}};
const Segment SGI9864[] = {{&S51104[50], 7}, {NULL}};
const Segment SGI9865[] = {{&S51104[50], 29}, {NULL}};
const Segment SGI9866[] = {{&S51104[30], 12}, {NULL}};
const Segment SGI9867[] = {{&S51104[96], 4}, {NULL}};
const Segment SGI9868[] = {{&S51104[96], 3}, {NULL}};
const Segment SGI9869[] = {{&S51206[50], 10}, {NULL}};
const Segment SGI9870[] = {{&S51206[50], 22}, {NULL}};
const Segment SGI9871[] = {{&S51206[30], 12}, {NULL}};
const Segment SGI9872[] = {{&S51206[121], 3}, {NULL}};
const Segment SGI9873[] = {{&S51206[86], 6}, {NULL}};
const Segment SGI9874[] = {{&S51207[44], 10}, {NULL}};
const Segment SGI9875[] = {{&S51207[31], 12}, {NULL}};
const Segment SGI9876[] = {{&S51208[50], 10}, {NULL}};
const Segment SGI9877[] = {{&S51208[50], 22}, {NULL}};
const Segment SGI9878[] = {{&S51208[30], 12}, {NULL}};
const Segment SGI9879[] = {{&S51208[89], 4}, {NULL}};
const Segment SGI9880[] = {{&S51208[89], 3}, {NULL}};
const Segment SGI9881[] = {{&S51209[48], 10}, {NULL}};
const Segment SGI9882[] = {{&S51209[30], 12}, {NULL}};
const Segment SGI9883[] = {{&S51209[119], 3}, {NULL}};
const Segment SGI9884[] = {{&S51209[84], 6}, {NULL}};
const Segment SGI9885[] = {{&S51780[44], 11}, {NULL}};
const Segment SGI9886[] = {{&S51780[44], 22}, {NULL}};
const Segment SGI9887[] = {{&S51780[30], 6}, {NULL}};
const Segment SGI9888[] = {{&S51780[114], 3}, {NULL}};
const Segment SGI9889[] = {{&S51780[80], 5}, {NULL}};
const Segment SGI9890[] = {{&S51781[38], 11}, {NULL}};
const Segment SGI9891[] = {{&S51781[31], 6}, {NULL}};
const Segment SGI9892[] = {{&S51210[42], 12}, {NULL}};
const Segment SGI9893[] = {{&S51210[42], 23}, {NULL}};
const Segment SGI9894[] = {{&S51210[30], 4}, {NULL}};
const Segment SGI9895[] = {{&S51210[113], 3}, {NULL}};
const Segment SGI9896[] = {{&S51210[79], 5}, {NULL}};
const Segment SGI9897[] = {{&S51212[36], 12}, {NULL}};
const Segment SGI9898[] = {{&S51212[31], 4}, {NULL}};
const Segment SGI9899[] = {{&S52349[6], 4}, {NULL}};
const Segment SGI9900[] = {{&S52349[32], 4}, {NULL}};
const Segment SGI9901[] = {{&S52349[81], 5}, {NULL}};
const Segment SGI9902[] = {{&S51372[42], 13}, {NULL}};
const Segment SGI9903[] = {{&S51372[42], 24}, {NULL}};
const Segment SGI9904[] = {{&S51372[30], 4}, {NULL}};
const Segment SGI9905[] = {{&S51372[114], 3}, {NULL}};
const Segment SGI9906[] = {{&S51372[80], 5}, {NULL}};
const Segment SGI9907[] = {{&S51373[42], 15}, {NULL}};
const Segment SGI9908[] = {{&S51373[42], 26}, {NULL}};
const Segment SGI9909[] = {{&S51373[30], 4}, {NULL}};
const Segment SGI9910[] = {{&S51373[116], 3}, {NULL}};
const Segment SGI9911[] = {{&S51373[82], 5}, {NULL}};
const Segment SGI9912[] = {{&S51376[42], 9}, {NULL}};
const Segment SGI9913[] = {{&S51376[42], 20}, {NULL}};
const Segment SGI9914[] = {{&S51376[30], 4}, {NULL}};
const Segment SGI9915[] = {{&S51376[110], 3}, {NULL}};
const Segment SGI9916[] = {{&S51376[76], 5}, {NULL}};
const Segment SGI9917[] = {{&S51378[36], 13}, {NULL}};
const Segment SGI9918[] = {{&S51378[31], 4}, {NULL}};
const Segment SGI9919[] = {{&S51416[50], 15}, {NULL}};
const Segment SGI9920[] = {{&S51416[50], 26}, {NULL}};
const Segment SGI9921[] = {{&S51416[30], 12}, {NULL}};
const Segment SGI9922[] = {{&S51416[125], 3}, {NULL}};
const Segment SGI9923[] = {{&S51416[90], 6}, {NULL}};
const Segment SGI9924[] = {{&S51417[50], 23}, {NULL}};
const Segment SGI9925[] = {{&S51417[50], 34}, {NULL}};
const Segment SGI9926[] = {{&S51417[30], 12}, {NULL}};
const Segment SGI9927[] = {{&S51417[133], 3}, {NULL}};
const Segment SGI9928[] = {{&S51417[98], 6}, {NULL}};
const Segment SGI9929[] = {{&S51418[50], 12}, {NULL}};
const Segment SGI9930[] = {{&S51418[50], 23}, {NULL}};
const Segment SGI9931[] = {{&S51418[30], 12}, {NULL}};
const Segment SGI9932[] = {{&S51418[122], 3}, {NULL}};
const Segment SGI9933[] = {{&S51418[87], 6}, {NULL}};
const Segment SGI9934[] = {{&S51419[50], 22}, {NULL}};
const Segment SGI9935[] = {{&S51419[50], 33}, {NULL}};
const Segment SGI9936[] = {{&S51419[30], 12}, {NULL}};
const Segment SGI9937[] = {{&S51419[132], 3}, {NULL}};
const Segment SGI9938[] = {{&S51419[97], 6}, {NULL}};
const Segment SGI9939[] = {{&S51420[44], 22}, {NULL}};
const Segment SGI9940[] = {{&S51420[31], 12}, {NULL}};
const Segment SGI9941[] = {{&S51489[50], 10}, {NULL}};
const Segment SGI9942[] = {{&S51489[50], 22}, {NULL}};
const Segment SGI9943[] = {{&S51489[30], 12}, {NULL}};
const Segment SGI9944[] = {{&S51489[121], 3}, {NULL}};
const Segment SGI9945[] = {{&S51489[86], 6}, {NULL}};
const Segment SGI9946[] = {{&S51490[50], 10}, {NULL}};
const Segment SGI9947[] = {{&S51490[50], 22}, {NULL}};
const Segment SGI9948[] = {{&S51490[30], 12}, {NULL}};
const Segment SGI9949[] = {{&S51490[122], 3}, {NULL}};
const Segment SGI9950[] = {{&S51490[86], 7}, {NULL}};
const Segment SGI9951[] = {{&S51490[142], 8}, {NULL}};
const Segment SGI9952[] = {{&S51491[44], 10}, {NULL}};
const Segment SGI9953[] = {{&S51491[31], 12}, {NULL}};
const Segment SGI9954[] = {{&S51667[42], 11}, {NULL}};
const Segment SGI9955[] = {{&S51667[42], 22}, {NULL}};
const Segment SGI9956[] = {{&S51667[30], 4}, {NULL}};
const Segment SGI9957[] = {{&S51667[113], 3}, {NULL}};
const Segment SGI9958[] = {{&S51667[78], 6}, {NULL}};
const Segment SGI9959[] = {{&S51668[50], 11}, {NULL}};
const Segment SGI9960[] = {{&S51668[50], 22}, {NULL}};
const Segment SGI9961[] = {{&S51668[30], 12}, {NULL}};
const Segment SGI9962[] = {{&S51668[121], 3}, {NULL}};
const Segment SGI9963[] = {{&S51668[86], 6}, {NULL}};
const Segment SGI9964[] = {{&S51669[50], 11}, {NULL}};
const Segment SGI9965[] = {{&S51669[50], 23}, {NULL}};
const Segment SGI9966[] = {{&S51669[30], 12}, {NULL}};
const Segment SGI9967[] = {{&S51669[122], 3}, {NULL}};
const Segment SGI9968[] = {{&S51669[87], 6}, {NULL}};
const Segment SGI9969[] = {{&S51670[42], 11}, {NULL}};
const Segment SGI9970[] = {{&S51670[42], 22}, {NULL}};
const Segment SGI9971[] = {{&S51670[30], 4}, {NULL}};
const Segment SGI9972[] = {{&S51670[113], 3}, {NULL}};
const Segment SGI9973[] = {{&S51670[78], 6}, {NULL}};
const Segment SGI9974[] = {{&S51671[42], 11}, {NULL}};
const Segment SGI9975[] = {{&S51671[42], 22}, {NULL}};
const Segment SGI9976[] = {{&S51671[30], 4}, {NULL}};
const Segment SGI9977[] = {{&S51671[114], 3}, {NULL}};
const Segment SGI9978[] = {{&S51671[78], 7}, {NULL}};
const Segment SGI9979[] = {{&S51671[134], 8}, {NULL}};
const Segment SGI9980[] = {{&S51672[42], 11}, {NULL}};
const Segment SGI9981[] = {{&S51672[42], 22}, {NULL}};
const Segment SGI9982[] = {{&S51672[30], 4}, {NULL}};
const Segment SGI9983[] = {{&S51672[114], 3}, {NULL}};
const Segment SGI9984[] = {{&S51672[78], 6}, {NULL}};
const Segment SGI9985[] = {{&S51672[134], 8}, {NULL}};
const Segment SGI9986[] = {{&S51673[42], 11}, {NULL}};
const Segment SGI9987[] = {{&S51673[42], 22}, {NULL}};
const Segment SGI9988[] = {{&S51673[30], 4}, {NULL}};
const Segment SGI9989[] = {{&S51673[113], 3}, {NULL}};
const Segment SGI9990[] = {{&S51673[78], 6}, {NULL}};
const Segment SGI9991[] = {{&S51674[50], 11}, {NULL}};
const Segment SGI9992[] = {{&S51674[50], 22}, {NULL}};
const Segment SGI9993[] = {{&S51674[30], 12}, {NULL}};
const Segment SGI9994[] = {{&S51674[120], 3}, {NULL}};
const Segment SGI9995[] = {{&S51674[86], 5}, {NULL}};
const Segment SGI9996[] = {{&S51675[50], 11}, {NULL}};
const Segment SGI9997[] = {{&S51675[50], 22}, {NULL}};
const Segment SGI9998[] = {{&S51675[30], 12}, {NULL}};
const Segment SGI9999[] = {{&S51675[121], 3}, {NULL}};
const Segment SGI10000[] = {{&S51675[86], 6}, {NULL}};
const Segment SGI10001[] = {{&S51676[42], 11}, {NULL}};
const Segment SGI10002[] = {{&S51676[42], 22}, {NULL}};
const Segment SGI10003[] = {{&S51676[30], 4}, {NULL}};
const Segment SGI10004[] = {{&S51676[112], 3}, {NULL}};
const Segment SGI10005[] = {{&S51676[78], 6}, {NULL}};
const Segment SGI10006[] = {{&S51677[50], 11}, {NULL}};
const Segment SGI10007[] = {{&S51677[50], 23}, {NULL}};
const Segment SGI10008[] = {{&S51677[30], 12}, {NULL}};
const Segment SGI10009[] = {{&S51677[121], 3}, {NULL}};
const Segment SGI10010[] = {{&S51677[87], 6}, {NULL}};
const Segment SGI10011[] = {{&S51678[35], 11}, {NULL}};
const Segment SGI10012[] = {{&S51678[30], 4}, {NULL}};
const Segment SGI10013[] = {{&S51678[106], 3}, {NULL}};
const Segment SGI10014[] = {{&S51678[71], 6}, {NULL}};
const Segment SGI10015[] = {{&S51693[50], 7}, {NULL}};
const Segment SGI10016[] = {{&S51693[50], 18}, {NULL}};
const Segment SGI10017[] = {{&S51693[30], 12}, {NULL}};
const Segment SGI10018[] = {{&S51693[117], 3}, {NULL}};
const Segment SGI10019[] = {{&S51693[82], 6}, {NULL}};
const Segment SGI10020[] = {{&S51694[50], 7}, {NULL}};
const Segment SGI10021[] = {{&S51694[50], 18}, {NULL}};
const Segment SGI10022[] = {{&S51694[30], 12}, {NULL}};
const Segment SGI10023[] = {{&S51694[118], 3}, {NULL}};
const Segment SGI10024[] = {{&S51694[82], 7}, {NULL}};
const Segment SGI10025[] = {{&S51694[138], 8}, {NULL}};
const Segment SGI10026[] = {{&S51695[50], 7}, {NULL}};
const Segment SGI10027[] = {{&S51695[50], 18}, {NULL}};
const Segment SGI10028[] = {{&S51695[30], 12}, {NULL}};
const Segment SGI10029[] = {{&S51695[117], 3}, {NULL}};
const Segment SGI10030[] = {{&S51695[82], 6}, {NULL}};
const Segment SGI10031[] = {{&S51696[44], 7}, {NULL}};
const Segment SGI10032[] = {{&S51696[31], 12}, {NULL}};
const Segment SGI10033[] = {{&S51697[44], 7}, {NULL}};
const Segment SGI10034[] = {{&S51697[31], 12}, {NULL}};
const Segment SGI10035[] = {{&S51774[42], 13}, {NULL}};
const Segment SGI10036[] = {{&S51774[42], 25}, {NULL}};
const Segment SGI10037[] = {{&S51774[30], 4}, {NULL}};
const Segment SGI10038[] = {{&S51774[115], 3}, {NULL}};
const Segment SGI10039[] = {{&S51774[81], 5}, {NULL}};
const Segment SGI10040[] = {{&S51775[42], 8}, {NULL}};
const Segment SGI10041[] = {{&S51775[42], 20}, {NULL}};
const Segment SGI10042[] = {{&S51775[30], 4}, {NULL}};
const Segment SGI10043[] = {{&S51775[110], 3}, {NULL}};
const Segment SGI10044[] = {{&S51775[76], 5}, {NULL}};
const Segment SGI10045[] = {{&S51776[40], 8}, {NULL}};
const Segment SGI10046[] = {{&S51776[30], 4}, {NULL}};
const Segment SGI10047[] = {{&S51776[108], 3}, {NULL}};
const Segment SGI10048[] = {{&S51776[74], 5}, {NULL}};
const Segment SGI10049[] = {{&S51777[36], 8}, {NULL}};
const Segment SGI10050[] = {{&S51777[31], 4}, {NULL}};
const Segment SGI10051[] = {{&S52494[44], 8}, {NULL}};
const Segment SGI10052[] = {{&S52494[44], 19}, {NULL}};
const Segment SGI10053[] = {{&S52494[30], 6}, {NULL}};
const Segment SGI10054[] = {{&S52494[111], 3}, {NULL}};
const Segment SGI10055[] = {{&S52494[77], 5}, {NULL}};
const Segment SGI10056[] = {{&S52495[38], 8}, {NULL}};
const Segment SGI10057[] = {{&S52495[31], 6}, {NULL}};
const Segment SGI10058[] = {{&S52496[44], 7}, {NULL}};
const Segment SGI10059[] = {{&S52496[44], 19}, {NULL}};
const Segment SGI10060[] = {{&S52496[30], 6}, {NULL}};
const Segment SGI10061[] = {{&S52496[111], 3}, {NULL}};
const Segment SGI10062[] = {{&S52496[77], 5}, {NULL}};
const Segment SGI10063[] = {{&S52497[44], 7}, {NULL}};
const Segment SGI10064[] = {{&S52497[44], 19}, {NULL}};
const Segment SGI10065[] = {{&S52497[30], 6}, {NULL}};
const Segment SGI10066[] = {{&S52497[111], 3}, {NULL}};
const Segment SGI10067[] = {{&S52497[77], 5}, {NULL}};
const Segment SGI10068[] = {{&S51779[42], 11}, {NULL}};
const Segment SGI10069[] = {{&S51779[42], 22}, {NULL}};
const Segment SGI10070[] = {{&S51779[30], 4}, {NULL}};
const Segment SGI10071[] = {{&S51779[112], 3}, {NULL}};
const Segment SGI10072[] = {{&S51779[78], 5}, {NULL}};
const Segment SGI10073[] = {{&S51782[36], 11}, {NULL}};
const Segment SGI10074[] = {{&S51782[31], 4}, {NULL}};
const Segment SGI10075[] = {{&S51783[42], 11}, {NULL}};
const Segment SGI10076[] = {{&S51783[42], 22}, {NULL}};
const Segment SGI10077[] = {{&S51783[30], 4}, {NULL}};
const Segment SGI10078[] = {{&S51783[113], 3}, {NULL}};
const Segment SGI10079[] = {{&S51783[78], 6}, {NULL}};
const Segment SGI10080[] = {{&S51784[42], 11}, {NULL}};
const Segment SGI10081[] = {{&S51784[42], 22}, {NULL}};
const Segment SGI10082[] = {{&S51784[30], 4}, {NULL}};
const Segment SGI10083[] = {{&S51784[81], 6}, {NULL}};
const Segment SGI10084[] = {{&S52062[42], 12}, {NULL}};
const Segment SGI10085[] = {{&S52062[42], 23}, {NULL}};
const Segment SGI10086[] = {{&S52062[30], 4}, {NULL}};
const Segment SGI10087[] = {{&S52062[114], 3}, {NULL}};
const Segment SGI10088[] = {{&S52062[79], 6}, {NULL}};
const Segment SGI10089[] = {{&S52063[50], 23}, {NULL}};
const Segment SGI10090[] = {{&S52063[50], 34}, {NULL}};
const Segment SGI10091[] = {{&S52063[30], 12}, {NULL}};
const Segment SGI10092[] = {{&S52063[133], 3}, {NULL}};
const Segment SGI10093[] = {{&S52063[98], 6}, {NULL}};
const Segment SGI10094[] = {{&S52064[42], 23}, {NULL}};
const Segment SGI10095[] = {{&S52064[42], 34}, {NULL}};
const Segment SGI10096[] = {{&S52064[30], 4}, {NULL}};
const Segment SGI10097[] = {{&S52064[125], 3}, {NULL}};
const Segment SGI10098[] = {{&S52064[90], 6}, {NULL}};
const Segment SGI10099[] = {{&S52065[42], 23}, {NULL}};
const Segment SGI10100[] = {{&S52065[42], 34}, {NULL}};
const Segment SGI10101[] = {{&S52065[30], 4}, {NULL}};
const Segment SGI10102[] = {{&S52065[125], 3}, {NULL}};
const Segment SGI10103[] = {{&S52065[90], 6}, {NULL}};
const Segment SGI10104[] = {{&S52066[42], 23}, {NULL}};
const Segment SGI10105[] = {{&S52066[42], 34}, {NULL}};
const Segment SGI10106[] = {{&S52066[30], 4}, {NULL}};
const Segment SGI10107[] = {{&S52066[125], 3}, {NULL}};
const Segment SGI10108[] = {{&S52066[90], 6}, {NULL}};
const Segment SGI10109[] = {{&S52067[42], 23}, {NULL}};
const Segment SGI10110[] = {{&S52067[42], 34}, {NULL}};
const Segment SGI10111[] = {{&S52067[30], 4}, {NULL}};
const Segment SGI10112[] = {{&S52067[125], 3}, {NULL}};
const Segment SGI10113[] = {{&S52067[90], 6}, {NULL}};
const Segment SGI10114[] = {{&S52068[42], 23}, {NULL}};
const Segment SGI10115[] = {{&S52068[42], 34}, {NULL}};
const Segment SGI10116[] = {{&S52068[30], 4}, {NULL}};
const Segment SGI10117[] = {{&S52068[125], 3}, {NULL}};
const Segment SGI10118[] = {{&S52068[90], 6}, {NULL}};
const Segment SGI10119[] = {{&S52069[50], 23}, {NULL}};
const Segment SGI10120[] = {{&S52069[50], 34}, {NULL}};
const Segment SGI10121[] = {{&S52069[30], 12}, {NULL}};
const Segment SGI10122[] = {{&S52069[133], 3}, {NULL}};
const Segment SGI10123[] = {{&S52069[98], 6}, {NULL}};
const Segment SGI10124[] = {{&S52070[50], 23}, {NULL}};
const Segment SGI10125[] = {{&S52070[50], 34}, {NULL}};
const Segment SGI10126[] = {{&S52070[30], 12}, {NULL}};
const Segment SGI10127[] = {{&S52070[133], 3}, {NULL}};
const Segment SGI10128[] = {{&S52070[98], 6}, {NULL}};
const Segment SGI10129[] = {{&S52071[50], 23}, {NULL}};
const Segment SGI10130[] = {{&S52071[50], 34}, {NULL}};
const Segment SGI10131[] = {{&S52071[30], 12}, {NULL}};
const Segment SGI10132[] = {{&S52071[133], 3}, {NULL}};
const Segment SGI10133[] = {{&S52071[98], 6}, {NULL}};
const Segment SGI10134[] = {{&S52072[42], 23}, {NULL}};
const Segment SGI10135[] = {{&S52072[42], 34}, {NULL}};
const Segment SGI10136[] = {{&S52072[30], 4}, {NULL}};
const Segment SGI10137[] = {{&S52072[125], 3}, {NULL}};
const Segment SGI10138[] = {{&S52072[90], 6}, {NULL}};
const Segment SGI10139[] = {{&S52073[50], 12}, {NULL}};
const Segment SGI10140[] = {{&S52073[50], 23}, {NULL}};
const Segment SGI10141[] = {{&S52073[30], 12}, {NULL}};
const Segment SGI10142[] = {{&S52073[122], 3}, {NULL}};
const Segment SGI10143[] = {{&S52073[87], 6}, {NULL}};
const Segment SGI10144[] = {{&S52074[50], 23}, {NULL}};
const Segment SGI10145[] = {{&S52074[50], 34}, {NULL}};
const Segment SGI10146[] = {{&S52074[30], 12}, {NULL}};
const Segment SGI10147[] = {{&S52074[133], 3}, {NULL}};
const Segment SGI10148[] = {{&S52074[98], 6}, {NULL}};
const Segment SGI10149[] = {{&S52075[49], 12}, {NULL}};
const Segment SGI10150[] = {{&S52075[49], 23}, {NULL}};
const Segment SGI10151[] = {{&S52075[30], 11}, {NULL}};
const Segment SGI10152[] = {{&S52075[121], 3}, {NULL}};
const Segment SGI10153[] = {{&S52075[86], 6}, {NULL}};
const Segment SGI10154[] = {{&S52076[50], 23}, {NULL}};
const Segment SGI10155[] = {{&S52076[50], 34}, {NULL}};
const Segment SGI10156[] = {{&S52076[30], 12}, {NULL}};
const Segment SGI10157[] = {{&S52076[133], 3}, {NULL}};
const Segment SGI10158[] = {{&S52076[98], 6}, {NULL}};
const Segment SGI10159[] = {{&S52077[50], 23}, {NULL}};
const Segment SGI10160[] = {{&S52077[50], 34}, {NULL}};
const Segment SGI10161[] = {{&S52077[30], 12}, {NULL}};
const Segment SGI10162[] = {{&S52077[133], 3}, {NULL}};
const Segment SGI10163[] = {{&S52077[98], 6}, {NULL}};
const Segment SGI10164[] = {{&S52078[42], 12}, {NULL}};
const Segment SGI10165[] = {{&S52078[42], 23}, {NULL}};
const Segment SGI10166[] = {{&S52078[30], 4}, {NULL}};
const Segment SGI10167[] = {{&S52078[114], 3}, {NULL}};
const Segment SGI10168[] = {{&S52078[79], 6}, {NULL}};
const Segment SGI10169[] = {{&S52079[50], 23}, {NULL}};
const Segment SGI10170[] = {{&S52079[50], 34}, {NULL}};
const Segment SGI10171[] = {{&S52079[30], 12}, {NULL}};
const Segment SGI10172[] = {{&S52079[133], 3}, {NULL}};
const Segment SGI10173[] = {{&S52079[98], 6}, {NULL}};
const Segment SGI10174[] = {{&S52080[42], 12}, {NULL}};
const Segment SGI10175[] = {{&S52080[42], 25}, {NULL}};
const Segment SGI10176[] = {{&S52080[30], 4}, {NULL}};
const Segment SGI10177[] = {{&S52080[116], 3}, {NULL}};
const Segment SGI10178[] = {{&S52080[81], 6}, {NULL}};
const Segment SGI10179[] = {{&S52081[44], 12}, {NULL}};
const Segment SGI10180[] = {{&S52081[31], 12}, {NULL}};
const Segment SGI10181[] = {{&S52082[36], 12}, {NULL}};
const Segment SGI10182[] = {{&S52082[31], 4}, {NULL}};
const Segment SGI10183[] = {{&S52083[42], 12}, {NULL}};
const Segment SGI10184[] = {{&S52083[42], 23}, {NULL}};
const Segment SGI10185[] = {{&S52083[30], 4}, {NULL}};
const Segment SGI10186[] = {{&S52083[82], 4}, {NULL}};
const Segment SGI10187[] = {{&S52083[82], 3}, {NULL}};
const Segment SGI10188[] = {{&S52002[42], 18}, {NULL}};
const Segment SGI10189[] = {{&S52002[42], 29}, {NULL}};
const Segment SGI10190[] = {{&S52002[30], 4}, {NULL}};
const Segment SGI10191[] = {{&S52002[119], 3}, {NULL}};
const Segment SGI10192[] = {{&S52002[85], 5}, {NULL}};
const Segment SGI10193[] = {{&S52807[44], 6}, {NULL}};
const Segment SGI10194[] = {{&S52807[44], 17}, {NULL}};
const Segment SGI10195[] = {{&S52807[30], 6}, {NULL}};
const Segment SGI10196[] = {{&S52807[109], 3}, {NULL}};
const Segment SGI10197[] = {{&S52807[75], 5}, {NULL}};
const Segment SGI10198[] = {{&S52808[44], 6}, {NULL}};
const Segment SGI10199[] = {{&S52808[44], 17}, {NULL}};
const Segment SGI10200[] = {{&S52808[30], 6}, {NULL}};
const Segment SGI10201[] = {{&S52808[109], 3}, {NULL}};
const Segment SGI10202[] = {{&S52808[75], 5}, {NULL}};
const Segment SGI10203[] = {{&S52809[44], 6}, {NULL}};
const Segment SGI10204[] = {{&S52809[44], 17}, {NULL}};
const Segment SGI10205[] = {{&S52809[30], 6}, {NULL}};
const Segment SGI10206[] = {{&S52809[109], 3}, {NULL}};
const Segment SGI10207[] = {{&S52809[75], 5}, {NULL}};
const Segment SGI10208[] = {{&S52810[44], 6}, {NULL}};
const Segment SGI10209[] = {{&S52810[44], 17}, {NULL}};
const Segment SGI10210[] = {{&S52810[30], 6}, {NULL}};
const Segment SGI10211[] = {{&S52810[109], 3}, {NULL}};
const Segment SGI10212[] = {{&S52810[75], 5}, {NULL}};
const Segment SGI10213[] = {{&S52811[44], 6}, {NULL}};
const Segment SGI10214[] = {{&S52811[44], 17}, {NULL}};
const Segment SGI10215[] = {{&S52811[30], 6}, {NULL}};
const Segment SGI10216[] = {{&S52811[109], 3}, {NULL}};
const Segment SGI10217[] = {{&S52811[75], 5}, {NULL}};
const Segment SGI10218[] = {{&S52812[44], 6}, {NULL}};
const Segment SGI10219[] = {{&S52812[44], 17}, {NULL}};
const Segment SGI10220[] = {{&S52812[30], 6}, {NULL}};
const Segment SGI10221[] = {{&S52812[109], 3}, {NULL}};
const Segment SGI10222[] = {{&S52812[75], 5}, {NULL}};
const Segment SGI10223[] = {{&S52813[38], 6}, {NULL}};
const Segment SGI10224[] = {{&S52813[31], 6}, {NULL}};
const Segment SGI10225[] = {{&S55858[42], 9}, {NULL}};
const Segment SGI10226[] = {{&S55858[42], 20}, {NULL}};
const Segment SGI10227[] = {{&S55858[30], 4}, {NULL}};
const Segment SGI10228[] = {{&S55858[111], 3}, {NULL}};
const Segment SGI10229[] = {{&S55858[76], 6}, {NULL}};
const Segment SGI10230[] = {{&S55858[131], 8}, {NULL}};
const Segment SGI10231[] = {{&S55859[42], 11}, {NULL}};
const Segment SGI10232[] = {{&S55859[42], 22}, {NULL}};
const Segment SGI10233[] = {{&S55859[30], 4}, {NULL}};
const Segment SGI10234[] = {{&S55859[113], 3}, {NULL}};
const Segment SGI10235[] = {{&S55859[78], 6}, {NULL}};
const Segment SGI10236[] = {{&S55859[133], 8}, {NULL}};
const Segment SGI10237[] = {{&S52452[50], 14}, {NULL}};
const Segment SGI10238[] = {{&S52452[50], 24}, {NULL}};
const Segment SGI10239[] = {{&S52452[30], 12}, {NULL}};
const Segment SGI10240[] = {{&S52452[123], 3}, {NULL}};
const Segment SGI10241[] = {{&S52452[88], 6}, {NULL}};
const Segment SGI10242[] = {{&S52453[50], 14}, {NULL}};
const Segment SGI10243[] = {{&S52453[50], 24}, {NULL}};
const Segment SGI10244[] = {{&S52453[30], 12}, {NULL}};
const Segment SGI10245[] = {{&S52453[123], 3}, {NULL}};
const Segment SGI10246[] = {{&S52453[88], 6}, {NULL}};
const Segment SGI10247[] = {{&S52454[50], 14}, {NULL}};
const Segment SGI10248[] = {{&S52454[50], 24}, {NULL}};
const Segment SGI10249[] = {{&S52454[30], 12}, {NULL}};
const Segment SGI10250[] = {{&S52454[123], 3}, {NULL}};
const Segment SGI10251[] = {{&S52454[88], 6}, {NULL}};
const Segment SGI10252[] = {{&S52455[50], 14}, {NULL}};
const Segment SGI10253[] = {{&S52455[50], 24}, {NULL}};
const Segment SGI10254[] = {{&S52455[30], 12}, {NULL}};
const Segment SGI10255[] = {{&S52455[123], 3}, {NULL}};
const Segment SGI10256[] = {{&S52455[88], 6}, {NULL}};
const Segment SGI10257[] = {{&S52456[50], 14}, {NULL}};
const Segment SGI10258[] = {{&S52456[50], 25}, {NULL}};
const Segment SGI10259[] = {{&S52456[30], 12}, {NULL}};
const Segment SGI10260[] = {{&S52456[124], 3}, {NULL}};
const Segment SGI10261[] = {{&S52456[89], 6}, {NULL}};
const Segment SGI10262[] = {{&S52457[50], 14}, {NULL}};
const Segment SGI10263[] = {{&S52457[50], 24}, {NULL}};
const Segment SGI10264[] = {{&S52457[30], 12}, {NULL}};
const Segment SGI10265[] = {{&S52457[123], 3}, {NULL}};
const Segment SGI10266[] = {{&S52457[88], 6}, {NULL}};
const Segment SGI10267[] = {{&S52458[50], 14}, {NULL}};
const Segment SGI10268[] = {{&S52458[50], 24}, {NULL}};
const Segment SGI10269[] = {{&S52458[30], 12}, {NULL}};
const Segment SGI10270[] = {{&S52458[123], 3}, {NULL}};
const Segment SGI10271[] = {{&S52458[88], 6}, {NULL}};
const Segment SGI10272[] = {{&S52459[50], 14}, {NULL}};
const Segment SGI10273[] = {{&S52459[50], 24}, {NULL}};
const Segment SGI10274[] = {{&S52459[30], 12}, {NULL}};
const Segment SGI10275[] = {{&S52459[123], 3}, {NULL}};
const Segment SGI10276[] = {{&S52459[88], 6}, {NULL}};
const Segment SGI10277[] = {{&S52460[50], 7}, {NULL}};
const Segment SGI10278[] = {{&S52460[50], 18}, {NULL}};
const Segment SGI10279[] = {{&S52460[30], 12}, {NULL}};
const Segment SGI10280[] = {{&S52460[117], 3}, {NULL}};
const Segment SGI10281[] = {{&S52460[82], 6}, {NULL}};
const Segment SGI10282[] = {{&S52461[42], 14}, {NULL}};
const Segment SGI10283[] = {{&S52461[42], 25}, {NULL}};
const Segment SGI10284[] = {{&S52461[30], 4}, {NULL}};
const Segment SGI10285[] = {{&S52461[116], 3}, {NULL}};
const Segment SGI10286[] = {{&S52461[81], 6}, {NULL}};
const Segment SGI10287[] = {{&S52462[50], 14}, {NULL}};
const Segment SGI10288[] = {{&S52462[50], 24}, {NULL}};
const Segment SGI10289[] = {{&S52462[30], 12}, {NULL}};
const Segment SGI10290[] = {{&S52462[123], 3}, {NULL}};
const Segment SGI10291[] = {{&S52462[88], 6}, {NULL}};
const Segment SGI10292[] = {{&S52463[50], 7}, {NULL}};
const Segment SGI10293[] = {{&S52463[50], 18}, {NULL}};
const Segment SGI10294[] = {{&S52463[30], 12}, {NULL}};
const Segment SGI10295[] = {{&S52463[117], 3}, {NULL}};
const Segment SGI10296[] = {{&S52463[82], 6}, {NULL}};
const Segment SGI10297[] = {{&S52464[50], 14}, {NULL}};
const Segment SGI10298[] = {{&S52464[50], 24}, {NULL}};
const Segment SGI10299[] = {{&S52464[30], 12}, {NULL}};
const Segment SGI10300[] = {{&S52464[123], 3}, {NULL}};
const Segment SGI10301[] = {{&S52464[88], 6}, {NULL}};
const Segment SGI10302[] = {{&S52465[44], 14}, {NULL}};
const Segment SGI10303[] = {{&S52465[31], 12}, {NULL}};
const Segment SGI10304[] = {{&S56003[44], 8}, {NULL}};
const Segment SGI10305[] = {{&S56003[44], 20}, {NULL}};
const Segment SGI10306[] = {{&S56003[30], 6}, {NULL}};
const Segment SGI10307[] = {{&S56003[113], 3}, {NULL}};
const Segment SGI10308[] = {{&S56003[78], 6}, {NULL}};
const Segment SGI10309[] = {{&S56004[44], 8}, {NULL}};
const Segment SGI10310[] = {{&S56004[44], 20}, {NULL}};
const Segment SGI10311[] = {{&S56004[30], 6}, {NULL}};
const Segment SGI10312[] = {{&S56004[113], 3}, {NULL}};
const Segment SGI10313[] = {{&S56004[78], 6}, {NULL}};
const Segment SGI10314[] = {{&S56005[44], 8}, {NULL}};
const Segment SGI10315[] = {{&S56005[44], 20}, {NULL}};
const Segment SGI10316[] = {{&S56005[30], 6}, {NULL}};
const Segment SGI10317[] = {{&S56005[113], 3}, {NULL}};
const Segment SGI10318[] = {{&S56005[78], 6}, {NULL}};
const Segment SGI10319[] = {{&S56006[44], 11}, {NULL}};
const Segment SGI10320[] = {{&S56006[44], 23}, {NULL}};
const Segment SGI10321[] = {{&S56006[30], 6}, {NULL}};
const Segment SGI10322[] = {{&S56006[116], 3}, {NULL}};
const Segment SGI10323[] = {{&S56006[81], 6}, {NULL}};
const Segment SGI10324[] = {{&S56007[44], 11}, {NULL}};
const Segment SGI10325[] = {{&S56007[44], 23}, {NULL}};
const Segment SGI10326[] = {{&S56007[30], 6}, {NULL}};
const Segment SGI10327[] = {{&S56007[116], 3}, {NULL}};
const Segment SGI10328[] = {{&S56007[81], 6}, {NULL}};
const Segment SGI10329[] = {{&S56008[44], 11}, {NULL}};
const Segment SGI10330[] = {{&S56008[44], 23}, {NULL}};
const Segment SGI10331[] = {{&S56008[30], 6}, {NULL}};
const Segment SGI10332[] = {{&S56008[116], 3}, {NULL}};
const Segment SGI10333[] = {{&S56008[81], 6}, {NULL}};
const Segment SGI10334[] = {{&S53503[44], 11}, {NULL}};
const Segment SGI10335[] = {{&S53503[44], 23}, {NULL}};
const Segment SGI10336[] = {{&S53503[30], 6}, {NULL}};
const Segment SGI10337[] = {{&S53503[115], 3}, {NULL}};
const Segment SGI10338[] = {{&S53503[81], 5}, {NULL}};
const Segment SGI10339[] = {{&S53507[44], 5}, {NULL}};
const Segment SGI10340[] = {{&S53507[44], 16}, {NULL}};
const Segment SGI10341[] = {{&S53507[30], 6}, {NULL}};
const Segment SGI10342[] = {{&S53507[108], 3}, {NULL}};
const Segment SGI10343[] = {{&S53507[74], 5}, {NULL}};
const Segment SGI10344[] = {{&S54121[6], 4}, {NULL}};
const Segment SGI10345[] = {{&S54121[32], 3}, {NULL}};
const Segment SGI10346[] = {{&S54121[81], 5}, {NULL}};
const Segment SGI10347[] = {{&S53743[44], 9}, {NULL}};
const Segment SGI10348[] = {{&S53743[44], 21}, {NULL}};
const Segment SGI10349[] = {{&S53743[30], 6}, {NULL}};
const Segment SGI10350[] = {{&S53743[113], 3}, {NULL}};
const Segment SGI10351[] = {{&S53743[79], 5}, {NULL}};
const Segment SGI10352[] = {{&S53744[44], 12}, {NULL}};
const Segment SGI10353[] = {{&S53744[44], 24}, {NULL}};
const Segment SGI10354[] = {{&S53744[30], 6}, {NULL}};
const Segment SGI10355[] = {{&S53744[116], 3}, {NULL}};
const Segment SGI10356[] = {{&S53744[82], 5}, {NULL}};
const Segment SGI10357[] = {{&S53745[44], 12}, {NULL}};
const Segment SGI10358[] = {{&S53745[44], 24}, {NULL}};
const Segment SGI10359[] = {{&S53745[30], 6}, {NULL}};
const Segment SGI10360[] = {{&S53745[116], 3}, {NULL}};
const Segment SGI10361[] = {{&S53745[82], 5}, {NULL}};
const Segment SGI10362[] = {{&S53746[44], 11}, {NULL}};
const Segment SGI10363[] = {{&S53746[44], 23}, {NULL}};
const Segment SGI10364[] = {{&S53746[30], 6}, {NULL}};
const Segment SGI10365[] = {{&S53746[115], 3}, {NULL}};
const Segment SGI10366[] = {{&S53746[81], 5}, {NULL}};
const Segment SGI10367[] = {{&S53747[42], 12}, {NULL}};
const Segment SGI10368[] = {{&S53747[30], 6}, {NULL}};
const Segment SGI10369[] = {{&S53747[114], 3}, {NULL}};
const Segment SGI10370[] = {{&S53747[80], 5}, {NULL}};
const Segment SGI10371[] = {{&S53748[43], 12}, {NULL}};
const Segment SGI10372[] = {{&S53748[43], 24}, {NULL}};
const Segment SGI10373[] = {{&S52916[42], 12}, {NULL}};
const Segment SGI10374[] = {{&S52916[42], 24}, {NULL}};
const Segment SGI10375[] = {{&S52916[30], 4}, {NULL}};
const Segment SGI10376[] = {{&S53748[115], 3}, {NULL}};
const Segment SGI10377[] = {{&S53748[81], 5}, {NULL}};
const Segment SGI10378[] = {{&S52916[114], 3}, {NULL}};
const Segment SGI10379[] = {{&S52916[80], 5}, {NULL}};
const Segment SGI10380[] = {{&S53749[43], 12}, {NULL}};
const Segment SGI10381[] = {{&S53749[43], 24}, {NULL}};
const Segment SGI10382[] = {{&S53749[30], 5}, {NULL}};
const Segment SGI10383[] = {{&S52917[42], 10}, {NULL}};
const Segment SGI10384[] = {{&S52917[42], 21}, {NULL}};
const Segment SGI10385[] = {{&S53749[115], 3}, {NULL}};
const Segment SGI10386[] = {{&S52917[30], 4}, {NULL}};
const Segment SGI10387[] = {{&S53749[81], 5}, {NULL}};
const Segment SGI10388[] = {{&S52917[111], 3}, {NULL}};
const Segment SGI10389[] = {{&S52917[77], 5}, {NULL}};
const Segment SGI10390[] = {{&S53750[44], 11}, {NULL}};
const Segment SGI10391[] = {{&S53750[44], 23}, {NULL}};
const Segment SGI10392[] = {{&S52918[36], 10}, {NULL}};
const Segment SGI10393[] = {{&S53750[30], 6}, {NULL}};
const Segment SGI10394[] = {{&S52918[31], 4}, {NULL}};
const Segment SGI10395[] = {{&S53750[115], 3}, {NULL}};
const Segment SGI10396[] = {{&S53750[81], 5}, {NULL}};
const Segment SGI10397[] = {{&S53751[38], 9}, {NULL}};
const Segment SGI10398[] = {{&S53751[31], 6}, {NULL}};
const Segment SGI10399[] = {{&S53779[44], 8}, {NULL}};
const Segment SGI10400[] = {{&S53779[44], 19}, {NULL}};
const Segment SGI10401[] = {{&S53779[30], 6}, {NULL}};
const Segment SGI10402[] = {{&S53779[111], 3}, {NULL}};
const Segment SGI10403[] = {{&S53779[77], 5}, {NULL}};
const Segment SGI10404[] = {{&S53780[38], 8}, {NULL}};
const Segment SGI10405[] = {{&S53780[31], 6}, {NULL}};
const Segment SGI10406[] = {{&S53797[44], 10}, {NULL}};
const Segment SGI10407[] = {{&S53797[44], 21}, {NULL}};
const Segment SGI10408[] = {{&S53797[30], 6}, {NULL}};
const Segment SGI10409[] = {{&S53797[113], 3}, {NULL}};
const Segment SGI10410[] = {{&S53797[79], 5}, {NULL}};
const Segment SGI10411[] = {{&S53798[38], 10}, {NULL}};
const Segment SGI10412[] = {{&S53798[31], 6}, {NULL}};
const Segment SGI10413[] = {{&S54470[6], 4}, {NULL}};
const Segment SGI10414[] = {{&S54470[32], 7}, {NULL}};
const Segment SGI10415[] = {{&S54470[79], 5}, {NULL}};
const Segment SGI10416[] = {{&S54471[6], 4}, {NULL}};
const Segment SGI10417[] = {{&S54471[32], 7}, {NULL}};
const Segment SGI10418[] = {{&S54471[79], 5}, {NULL}};
const Segment SGI10419[] = {{&S54472[6], 4}, {NULL}};
const Segment SGI10420[] = {{&S54472[32], 7}, {NULL}};
const Segment SGI10421[] = {{&S54472[77], 5}, {NULL}};
const Segment SGI10422[] = {{&S54082[44], 11}, {NULL}};
const Segment SGI10423[] = {{&S54082[44], 22}, {NULL}};
const Segment SGI10424[] = {{&S54082[30], 6}, {NULL}};
const Segment SGI10425[] = {{&S54082[114], 3}, {NULL}};
const Segment SGI10426[] = {{&S54082[80], 5}, {NULL}};
const Segment SGI10427[] = {{&S54083[44], 11}, {NULL}};
const Segment SGI10428[] = {{&S54083[44], 22}, {NULL}};
const Segment SGI10429[] = {{&S54083[30], 6}, {NULL}};
const Segment SGI10430[] = {{&S54083[114], 3}, {NULL}};
const Segment SGI10431[] = {{&S54083[80], 5}, {NULL}};
const Segment SGI10432[] = {{&S54084[44], 11}, {NULL}};
const Segment SGI10433[] = {{&S54084[44], 22}, {NULL}};
const Segment SGI10434[] = {{&S54084[30], 6}, {NULL}};
const Segment SGI10435[] = {{&S54084[114], 3}, {NULL}};
const Segment SGI10436[] = {{&S54084[80], 5}, {NULL}};
const Segment SGI10437[] = {{&S54085[44], 11}, {NULL}};
const Segment SGI10438[] = {{&S54085[44], 22}, {NULL}};
const Segment SGI10439[] = {{&S54085[30], 6}, {NULL}};
const Segment SGI10440[] = {{&S54085[114], 3}, {NULL}};
const Segment SGI10441[] = {{&S54085[80], 5}, {NULL}};
const Segment SGI10442[] = {{&S54088[38], 11}, {NULL}};
const Segment SGI10443[] = {{&S54088[31], 6}, {NULL}};
const Segment SGI10444[] = {{&S53279[50], 7}, {NULL}};
const Segment SGI10445[] = {{&S53279[50], 29}, {NULL}};
const Segment SGI10446[] = {{&S53279[30], 12}, {NULL}};
const Segment SGI10447[] = {{&S53279[128], 3}, {NULL}};
const Segment SGI10448[] = {{&S53279[93], 6}, {NULL}};
const Segment SGI10449[] = {{&S53280[50], 7}, {NULL}};
const Segment SGI10450[] = {{&S53280[50], 59}, {NULL}};
const Segment SGI10451[] = {{&S53280[30], 12}, {NULL}};
const Segment SGI10452[] = {{&S53280[158], 3}, {NULL}};
const Segment SGI10453[] = {{&S53280[123], 6}, {NULL}};
const Segment SGI10454[] = {{&S53281[44], 7}, {NULL}};
const Segment SGI10455[] = {{&S53281[31], 12}, {NULL}};
const Segment SGI10456[] = {{&S53500[42], 5}, {NULL}};
const Segment SGI10457[] = {{&S53500[42], 16}, {NULL}};
const Segment SGI10458[] = {{&S53500[30], 4}, {NULL}};
const Segment SGI10459[] = {{&S53500[106], 3}, {NULL}};
const Segment SGI10460[] = {{&S53500[72], 5}, {NULL}};
const Segment SGI10461[] = {{&S53501[42], 12}, {NULL}};
const Segment SGI10462[] = {{&S53501[42], 23}, {NULL}};
const Segment SGI10463[] = {{&S53501[30], 4}, {NULL}};
const Segment SGI10464[] = {{&S53501[113], 3}, {NULL}};
const Segment SGI10465[] = {{&S53501[79], 5}, {NULL}};
const Segment SGI10466[] = {{&S53502[36], 12}, {NULL}};
const Segment SGI10467[] = {{&S53502[31], 4}, {NULL}};
const Segment SGI10468[] = {{&S53504[36], 5}, {NULL}};
const Segment SGI10469[] = {{&S53504[31], 4}, {NULL}};
const Segment SGI10470[] = {{&S53505[42], 5}, {NULL}};
const Segment SGI10471[] = {{&S53505[42], 16}, {NULL}};
const Segment SGI10472[] = {{&S53505[30], 4}, {NULL}};
const Segment SGI10473[] = {{&S53505[106], 3}, {NULL}};
const Segment SGI10474[] = {{&S53505[72], 5}, {NULL}};
const Segment SGI10475[] = {{&S53506[42], 11}, {NULL}};
const Segment SGI10476[] = {{&S53506[42], 22}, {NULL}};
const Segment SGI10477[] = {{&S53506[30], 4}, {NULL}};
const Segment SGI10478[] = {{&S53506[113], 3}, {NULL}};
const Segment SGI10479[] = {{&S53506[78], 6}, {NULL}};
const Segment SGI10480[] = {{&S53508[41], 5}, {NULL}};
const Segment SGI10481[] = {{&S53508[41], 16}, {NULL}};
const Segment SGI10482[] = {{&S53508[29], 4}, {NULL}};
const Segment SGI10483[] = {{&S53508[115], 3}, {NULL}};
const Segment SGI10484[] = {{&S53508[71], 5}, {NULL}};
const Segment SGI10485[] = {{&S56139[42], 14}, {NULL}};
const Segment SGI10486[] = {{&S56139[30], 4}, {NULL}};
const Segment SGI10487[] = {{&S56139[105], 3}, {NULL}};
const Segment SGI10488[] = {{&S56139[70], 6}, {NULL}};
const Segment SGI10489[] = {{&S56140[42], 15}, {NULL}};
const Segment SGI10490[] = {{&S56140[30], 4}, {NULL}};
const Segment SGI10491[] = {{&S56140[106], 3}, {NULL}};
const Segment SGI10492[] = {{&S56140[71], 6}, {NULL}};
const Segment SGI10493[] = {{&S59599[8], 7}, {NULL}};
const Segment SGI10494[] = {{&S59599[0], 45}, {NULL}};
const Segment SGI10495[] = {{&S59600[8], 7}, {NULL}};
const Segment SGI10496[] = {{&S59600[0], 85}, {NULL}};
const Segment SGI10497[] = {{&S59601[8], 6}, {NULL}};
const Segment SGI10498[] = {{&S59601[0], 91}, {NULL}};
const Segment SGI10499[] = {{&S59602[8], 6}, {NULL}};
const Segment SGI10500[] = {{&S59602[0], 84}, {NULL}};
const Segment SGI10501[] = {{&S59603[8], 6}, {NULL}};
const Segment SGI10502[] = {{&S59603[0], 97}, {NULL}};
const Segment SGI10503[] = {{&S57146[30], 4}, {NULL}};
const Segment SGI10504[] = {{&S57146[90], 3}, {NULL}};
const Segment SGI10505[] = {{&S57146[55], 6}, {NULL}};
const Segment SGI10506[] = {{&S57146[110], 8}, {NULL}};
const Segment SGI10507[] = {{&S57147[35], 7}, {NULL}};
const Segment SGI10508[] = {{&S57147[30], 4}, {NULL}};
const Segment SGI10509[] = {{&S57147[104], 3}, {NULL}};
const Segment SGI10510[] = {{&S57147[69], 6}, {NULL}};
const Segment SGI10511[] = {{&S57147[124], 8}, {NULL}};
const Segment SGI10512[] = {{&S57148[35], 7}, {NULL}};
const Segment SGI10513[] = {{&S57148[30], 4}, {NULL}};
const Segment SGI10514[] = {{&S57148[104], 3}, {NULL}};
const Segment SGI10515[] = {{&S57148[69], 6}, {NULL}};
const Segment SGI10516[] = {{&S57148[124], 8}, {NULL}};
const Segment SGI10517[] = {{&S57149[42], 7}, {NULL}};
const Segment SGI10518[] = {{&S57149[42], 20}, {NULL}};
const Segment SGI10519[] = {{&S57149[30], 4}, {NULL}};
const Segment SGI10520[] = {{&S57149[111], 3}, {NULL}};
const Segment SGI10521[] = {{&S57149[76], 6}, {NULL}};
const Segment SGI10522[] = {{&S57149[131], 8}, {NULL}};
const Segment SGI10523[] = {{&S53778[42], 8}, {NULL}};
const Segment SGI10524[] = {{&S53778[42], 19}, {NULL}};
const Segment SGI10525[] = {{&S53778[30], 4}, {NULL}};
const Segment SGI10526[] = {{&S53778[109], 3}, {NULL}};
const Segment SGI10527[] = {{&S53778[75], 5}, {NULL}};
const Segment SGI10528[] = {{&S53781[36], 8}, {NULL}};
const Segment SGI10529[] = {{&S53781[31], 4}, {NULL}};
const Segment SGI10530[] = {{&S53782[42], 8}, {NULL}};
const Segment SGI10531[] = {{&S53782[42], 19}, {NULL}};
const Segment SGI10532[] = {{&S53782[30], 4}, {NULL}};
const Segment SGI10533[] = {{&S53782[109], 3}, {NULL}};
const Segment SGI10534[] = {{&S53782[75], 5}, {NULL}};
const Segment SGI10535[] = {{&S53796[42], 10}, {NULL}};
const Segment SGI10536[] = {{&S53796[42], 21}, {NULL}};
const Segment SGI10537[] = {{&S53796[30], 4}, {NULL}};
const Segment SGI10538[] = {{&S53796[111], 3}, {NULL}};
const Segment SGI10539[] = {{&S53796[77], 5}, {NULL}};
const Segment SGI10540[] = {{&S57347[42], 8}, {NULL}};
const Segment SGI10541[] = {{&S53823[50], 29}, {NULL}};
const Segment SGI10542[] = {{&S57347[30], 4}, {NULL}};
const Segment SGI10543[] = {{&S53823[50], 40}, {NULL}};
const Segment SGI10544[] = {{&S53823[30], 12}, {NULL}};
const Segment SGI10545[] = {{&S57347[100], 3}, {NULL}};
const Segment SGI10546[] = {{&S57347[64], 7}, {NULL}};
const Segment SGI10547[] = {{&S53823[139], 3}, {NULL}};
const Segment SGI10548[] = {{&S57347[120], 8}, {NULL}};
const Segment SGI10549[] = {{&S53823[104], 6}, {NULL}};
const Segment SGI10550[] = {{&S55131[35], 10}, {NULL}};
const Segment SGI10551[] = {{&S55131[66], 6}, {NULL}};
const Segment SGI10552[] = {{&S55131[36], 8}, {NULL}};
const Segment SGI10553[] = {{&S57489[42], 7}, {NULL}};
const Segment SGI10554[] = {{&S57489[42], 20}, {NULL}};
const Segment SGI10555[] = {{&S57489[30], 4}, {NULL}};
const Segment SGI10556[] = {{&S57489[111], 3}, {NULL}};
const Segment SGI10557[] = {{&S57489[76], 6}, {NULL}};
const Segment SGI10558[] = {{&S57489[131], 8}, {NULL}};
const Segment SGI10559[] = {{&S57490[42], 7}, {NULL}};
const Segment SGI10560[] = {{&S57490[42], 20}, {NULL}};
const Segment SGI10561[] = {{&S57490[30], 4}, {NULL}};
const Segment SGI10562[] = {{&S57490[111], 3}, {NULL}};
const Segment SGI10563[] = {{&S57490[76], 5}, {NULL}};
const Segment SGI10564[] = {{&S57490[131], 8}, {NULL}};
const Segment SGI10565[] = {{&S54080[42], 11}, {NULL}};
const Segment SGI10566[] = {{&S54080[42], 22}, {NULL}};
const Segment SGI10567[] = {{&S54080[30], 4}, {NULL}};
const Segment SGI10568[] = {{&S54080[112], 3}, {NULL}};
const Segment SGI10569[] = {{&S54080[78], 5}, {NULL}};
const Segment SGI10570[] = {{&S54081[42], 11}, {NULL}};
const Segment SGI10571[] = {{&S54081[42], 22}, {NULL}};
const Segment SGI10572[] = {{&S54081[30], 4}, {NULL}};
const Segment SGI10573[] = {{&S54081[112], 3}, {NULL}};
const Segment SGI10574[] = {{&S54081[78], 5}, {NULL}};
const Segment SGI10575[] = {{&S54086[36], 11}, {NULL}};
const Segment SGI10576[] = {{&S54086[31], 4}, {NULL}};
const Segment SGI10577[] = {{&S54087[36], 11}, {NULL}};
const Segment SGI10578[] = {{&S54087[31], 4}, {NULL}};
const Segment SGI10579[] = {{&S57582[42], 9}, {NULL}};
const Segment SGI10580[] = {{&S57582[42], 22}, {NULL}};
const Segment SGI10581[] = {{&S57582[30], 4}, {NULL}};
const Segment SGI10582[] = {{&S57582[113], 3}, {NULL}};
const Segment SGI10583[] = {{&S57582[78], 6}, {NULL}};
const Segment SGI10584[] = {{&S57582[133], 8}, {NULL}};
const Segment SGI10585[] = {{&S55393[5], 8}, {NULL}};
const Segment SGI10586[] = {{&S54931[44], 8}, {NULL}};
const Segment SGI10587[] = {{&S54931[44], 19}, {NULL}};
const Segment SGI10588[] = {{&S54931[30], 6}, {NULL}};
const Segment SGI10589[] = {{&S54931[111], 3}, {NULL}};
const Segment SGI10590[] = {{&S54931[77], 5}, {NULL}};
const Segment SGI10591[] = {{&S54935[44], 7}, {NULL}};
const Segment SGI10592[] = {{&S54935[44], 18}, {NULL}};
const Segment SGI10593[] = {{&S54935[30], 6}, {NULL}};
const Segment SGI10594[] = {{&S54935[110], 3}, {NULL}};
const Segment SGI10595[] = {{&S54935[76], 5}, {NULL}};
const Segment SGI10596[] = {{&S54609[42], 7}, {NULL}};
const Segment SGI10597[] = {{&S54609[42], 18}, {NULL}};
const Segment SGI10598[] = {{&S54609[30], 4}, {NULL}};
const Segment SGI10599[] = {{&S54609[108], 3}, {NULL}};
const Segment SGI10600[] = {{&S54609[74], 5}, {NULL}};
const Segment SGI10601[] = {{&S54611[36], 7}, {NULL}};
const Segment SGI10602[] = {{&S54611[31], 4}, {NULL}};
const Segment SGI10603[] = {{&S55085[44], 10}, {NULL}};
const Segment SGI10604[] = {{&S55085[44], 32}, {NULL}};
const Segment SGI10605[] = {{&S55085[30], 6}, {NULL}};
const Segment SGI10606[] = {{&S55085[124], 3}, {NULL}};
const Segment SGI10607[] = {{&S55085[90], 5}, {NULL}};
const Segment SGI10608[] = {{&S55208[44], 7}, {NULL}};
const Segment SGI10609[] = {{&S55208[44], 18}, {NULL}};
const Segment SGI10610[] = {{&S55208[30], 6}, {NULL}};
const Segment SGI10611[] = {{&S55208[110], 3}, {NULL}};
const Segment SGI10612[] = {{&S55208[76], 5}, {NULL}};
const Segment SGI10613[] = {{&S55209[38], 7}, {NULL}};
const Segment SGI10614[] = {{&S55209[31], 6}, {NULL}};
const Segment SGI10615[] = {{&S55276[44], 11}, {NULL}};
const Segment SGI10616[] = {{&S55276[44], 22}, {NULL}};
const Segment SGI10617[] = {{&S55276[30], 6}, {NULL}};
const Segment SGI10618[] = {{&S55276[114], 3}, {NULL}};
const Segment SGI10619[] = {{&S55276[80], 5}, {NULL}};
const Segment SGI10620[] = {{&S55277[44], 11}, {NULL}};
const Segment SGI10621[] = {{&S55277[44], 23}, {NULL}};
const Segment SGI10622[] = {{&S55277[30], 6}, {NULL}};
const Segment SGI10623[] = {{&S55277[115], 3}, {NULL}};
const Segment SGI10624[] = {{&S55277[81], 5}, {NULL}};
const Segment SGI10625[] = {{&S55278[38], 11}, {NULL}};
const Segment SGI10626[] = {{&S55278[31], 6}, {NULL}};
const Segment SGI10627[] = {{&S55279[44], 11}, {NULL}};
const Segment SGI10628[] = {{&S55279[44], 22}, {NULL}};
const Segment SGI10629[] = {{&S55279[30], 6}, {NULL}};
const Segment SGI10630[] = {{&S55279[83], 4}, {NULL}};
const Segment SGI10631[] = {{&S55279[83], 3}, {NULL}};
const Segment SGI10632[] = {{&S55971[86], 5}, {NULL}};
const Segment SGI10633[] = {{&S55367[44], 9}, {NULL}};
const Segment SGI10634[] = {{&S55367[44], 20}, {NULL}};
const Segment SGI10635[] = {{&S55367[30], 6}, {NULL}};
const Segment SGI10636[] = {{&S55367[112], 3}, {NULL}};
const Segment SGI10637[] = {{&S55367[78], 5}, {NULL}};
const Segment SGI10638[] = {{&S55434[44], 19}, {NULL}};
const Segment SGI10639[] = {{&S55434[44], 30}, {NULL}};
const Segment SGI10640[] = {{&S55434[30], 6}, {NULL}};
const Segment SGI10641[] = {{&S55434[122], 3}, {NULL}};
const Segment SGI10642[] = {{&S55434[88], 5}, {NULL}};
const Segment SGI10643[] = {{&S55438[38], 19}, {NULL}};
const Segment SGI10644[] = {{&S55438[31], 6}, {NULL}};
const Segment SGI10645[] = {{&S55439[44], 14}, {NULL}};
const Segment SGI10646[] = {{&S55439[44], 25}, {NULL}};
const Segment SGI10647[] = {{&S55439[30], 6}, {NULL}};
const Segment SGI10648[] = {{&S55439[117], 3}, {NULL}};
const Segment SGI10649[] = {{&S55439[83], 5}, {NULL}};
const Segment SGI10650[] = {{&S58499[42], 6}, {NULL}};
const Segment SGI10651[] = {{&S58499[42], 23}, {NULL}};
const Segment SGI10652[] = {{&S58499[30], 4}, {NULL}};
const Segment SGI10653[] = {{&S58499[114], 3}, {NULL}};
const Segment SGI10654[] = {{&S58499[79], 6}, {NULL}};
const Segment SGI10655[] = {{&S58499[134], 8}, {NULL}};
const Segment SGI10656[] = {{&S58500[42], 6}, {NULL}};
const Segment SGI10657[] = {{&S58500[42], 23}, {NULL}};
const Segment SGI10658[] = {{&S58500[30], 4}, {NULL}};
const Segment SGI10659[] = {{&S58500[114], 3}, {NULL}};
const Segment SGI10660[] = {{&S58500[79], 6}, {NULL}};
const Segment SGI10661[] = {{&S58500[134], 8}, {NULL}};
const Segment SGI10662[] = {{&S55219[42], 10}, {NULL}};
const Segment SGI10663[] = {{&S55219[30], 4}, {NULL}};
const Segment SGI10664[] = {{&S55219[100], 3}, {NULL}};
const Segment SGI10665[] = {{&S55219[66], 5}, {NULL}};
const Segment SGI10666[] = {{&S58502[42], 6}, {NULL}};
const Segment SGI10667[] = {{&S58502[42], 21}, {NULL}};
const Segment SGI10668[] = {{&S58502[30], 4}, {NULL}};
const Segment SGI10669[] = {{&S55220[42], 10}, {NULL}};
const Segment SGI10670[] = {{&S55220[30], 4}, {NULL}};
const Segment SGI10671[] = {{&S58502[112], 3}, {NULL}};
const Segment SGI10672[] = {{&S55220[100], 3}, {NULL}};
const Segment SGI10673[] = {{&S58502[77], 6}, {NULL}};
const Segment SGI10674[] = {{&S58502[132], 8}, {NULL}};
const Segment SGI10675[] = {{&S55220[66], 5}, {NULL}};
const Segment SGI10676[] = {{&S58503[42], 6}, {NULL}};
const Segment SGI10677[] = {{&S58503[42], 22}, {NULL}};
const Segment SGI10678[] = {{&S58503[30], 4}, {NULL}};
const Segment SGI10679[] = {{&S58503[113], 3}, {NULL}};
const Segment SGI10680[] = {{&S58503[78], 6}, {NULL}};
const Segment SGI10681[] = {{&S58503[133], 8}, {NULL}};
const Segment SGI10682[] = {{&S58504[42], 6}, {NULL}};
const Segment SGI10683[] = {{&S58504[42], 21}, {NULL}};
const Segment SGI10684[] = {{&S58504[30], 4}, {NULL}};
const Segment SGI10685[] = {{&S58504[112], 3}, {NULL}};
const Segment SGI10686[] = {{&S58504[77], 6}, {NULL}};
const Segment SGI10687[] = {{&S58504[132], 8}, {NULL}};
const Segment SGI10688[] = {{&S58505[42], 6}, {NULL}};
const Segment SGI10689[] = {{&S58505[42], 23}, {NULL}};
const Segment SGI10690[] = {{&S58505[30], 4}, {NULL}};
const Segment SGI10691[] = {{&S58505[114], 3}, {NULL}};
const Segment SGI10692[] = {{&S58505[79], 6}, {NULL}};
const Segment SGI10693[] = {{&S58505[134], 8}, {NULL}};
const Segment SGI10694[] = {{&S55262[42], 14}, {NULL}};
const Segment SGI10695[] = {{&S55593[48], 6}, {NULL}};
const Segment SGI10696[] = {{&S55262[42], 25}, {NULL}};
const Segment SGI10697[] = {{&S55593[48], 17}, {NULL}};
const Segment SGI10698[] = {{&S55262[30], 4}, {NULL}};
const Segment SGI10699[] = {{&S55593[30], 10}, {NULL}};
const Segment SGI10700[] = {{&S55262[115], 3}, {NULL}};
const Segment SGI10701[] = {{&S55593[113], 3}, {NULL}};
const Segment SGI10702[] = {{&S55262[81], 5}, {NULL}};
const Segment SGI10703[] = {{&S55593[79], 5}, {NULL}};
const Segment SGI10704[] = {{&S55263[36], 14}, {NULL}};
const Segment SGI10705[] = {{&S55263[31], 4}, {NULL}};
const Segment SGI10706[] = {{&S55594[44], 6}, {NULL}};
const Segment SGI10707[] = {{&S55594[44], 17}, {NULL}};
const Segment SGI10708[] = {{&S55594[30], 6}, {NULL}};
const Segment SGI10709[] = {{&S55594[109], 3}, {NULL}};
const Segment SGI10710[] = {{&S55594[75], 5}, {NULL}};
const Segment SGI10711[] = {{&S55595[48], 6}, {NULL}};
const Segment SGI10712[] = {{&S55595[48], 17}, {NULL}};
const Segment SGI10713[] = {{&S55595[30], 10}, {NULL}};
const Segment SGI10714[] = {{&S55595[113], 3}, {NULL}};
const Segment SGI10715[] = {{&S55595[79], 5}, {NULL}};
const Segment SGI10716[] = {{&S55596[38], 6}, {NULL}};
const Segment SGI10717[] = {{&S55596[31], 6}, {NULL}};
const Segment SGI10718[] = {{&S55275[42], 11}, {NULL}};
const Segment SGI10719[] = {{&S55275[42], 22}, {NULL}};
const Segment SGI10720[] = {{&S55275[30], 4}, {NULL}};
const Segment SGI10721[] = {{&S55275[112], 3}, {NULL}};
const Segment SGI10722[] = {{&S55275[78], 5}, {NULL}};
const Segment SGI10723[] = {{&S58597[42], 8}, {NULL}};
const Segment SGI10724[] = {{&S58597[42], 21}, {NULL}};
const Segment SGI10725[] = {{&S58597[30], 4}, {NULL}};
const Segment SGI10726[] = {{&S58597[112], 3}, {NULL}};
const Segment SGI10727[] = {{&S58597[77], 6}, {NULL}};
const Segment SGI10728[] = {{&S58597[132], 8}, {NULL}};
const Segment SGI10729[] = {{&S58598[42], 8}, {NULL}};
const Segment SGI10730[] = {{&S58598[42], 21}, {NULL}};
const Segment SGI10731[] = {{&S58598[30], 4}, {NULL}};
const Segment SGI10732[] = {{&S58598[113], 3}, {NULL}};
const Segment SGI10733[] = {{&S58598[77], 7}, {NULL}};
const Segment SGI10734[] = {{&S58598[133], 8}, {NULL}};
const Segment SGI10735[] = {{&S58599[40], 8}, {NULL}};
const Segment SGI10736[] = {{&S58599[30], 4}, {NULL}};
const Segment SGI10737[] = {{&S58599[110], 3}, {NULL}};
const Segment SGI10738[] = {{&S58599[75], 6}, {NULL}};
const Segment SGI10739[] = {{&S58599[130], 8}, {NULL}};
const Segment SGI10740[] = {{&S58600[42], 8}, {NULL}};
const Segment SGI10741[] = {{&S58600[42], 21}, {NULL}};
const Segment SGI10742[] = {{&S58600[30], 4}, {NULL}};
const Segment SGI10743[] = {{&S58600[112], 3}, {NULL}};
const Segment SGI10744[] = {{&S58600[77], 5}, {NULL}};
const Segment SGI10745[] = {{&S58600[132], 8}, {NULL}};
const Segment SGI10746[] = {{&S55437[42], 14}, {NULL}};
const Segment SGI10747[] = {{&S55437[42], 26}, {NULL}};
const Segment SGI10748[] = {{&S55437[30], 4}, {NULL}};
const Segment SGI10749[] = {{&S55437[116], 3}, {NULL}};
const Segment SGI10750[] = {{&S55437[82], 5}, {NULL}};
const Segment SGI10751[] = {{&S55440[36], 14}, {NULL}};
const Segment SGI10752[] = {{&S55440[31], 4}, {NULL}};
const Segment SGI10753[] = {{&S54937[50], 7}, {NULL}};
const Segment SGI10754[] = {{&S54937[50], 48}, {NULL}};
const Segment SGI10755[] = {{&S54937[30], 12}, {NULL}};
const Segment SGI10756[] = {{&S54937[147], 3}, {NULL}};
const Segment SGI10757[] = {{&S54937[112], 6}, {NULL}};
const Segment SGI10758[] = {{&S54938[50], 7}, {NULL}};
const Segment SGI10759[] = {{&S54938[50], 18}, {NULL}};
const Segment SGI10760[] = {{&S54938[30], 12}, {NULL}};
const Segment SGI10761[] = {{&S54938[117], 3}, {NULL}};
const Segment SGI10762[] = {{&S54938[82], 6}, {NULL}};
const Segment SGI10763[] = {{&S54939[50], 7}, {NULL}};
const Segment SGI10764[] = {{&S54939[50], 60}, {NULL}};
const Segment SGI10765[] = {{&S54939[30], 12}, {NULL}};
const Segment SGI10766[] = {{&S54939[159], 3}, {NULL}};
const Segment SGI10767[] = {{&S54939[124], 6}, {NULL}};
const Segment SGI10768[] = {{&S54940[50], 7}, {NULL}};
const Segment SGI10769[] = {{&S54940[50], 18}, {NULL}};
const Segment SGI10770[] = {{&S54940[30], 12}, {NULL}};
const Segment SGI10771[] = {{&S54940[117], 3}, {NULL}};
const Segment SGI10772[] = {{&S54940[82], 6}, {NULL}};
const Segment SGI10773[] = {{&S54941[50], 7}, {NULL}};
const Segment SGI10774[] = {{&S54941[50], 60}, {NULL}};
const Segment SGI10775[] = {{&S54941[30], 12}, {NULL}};
const Segment SGI10776[] = {{&S54941[159], 3}, {NULL}};
const Segment SGI10777[] = {{&S54941[124], 6}, {NULL}};
const Segment SGI10778[] = {{&S54942[50], 7}, {NULL}};
const Segment SGI10779[] = {{&S54942[50], 19}, {NULL}};
const Segment SGI10780[] = {{&S54942[30], 12}, {NULL}};
const Segment SGI10781[] = {{&S54942[118], 3}, {NULL}};
const Segment SGI10782[] = {{&S54942[83], 6}, {NULL}};
const Segment SGI10783[] = {{&S54943[44], 7}, {NULL}};
const Segment SGI10784[] = {{&S54943[31], 12}, {NULL}};
const Segment SGI10785[] = {{&S58896[44], 8}, {NULL}};
const Segment SGI10786[] = {{&S58896[44], 20}, {NULL}};
const Segment SGI10787[] = {{&S58896[30], 6}, {NULL}};
const Segment SGI10788[] = {{&S58896[113], 3}, {NULL}};
const Segment SGI10789[] = {{&S58896[78], 6}, {NULL}};
const Segment SGI10790[] = {{&S58897[44], 8}, {NULL}};
const Segment SGI10791[] = {{&S58897[44], 20}, {NULL}};
const Segment SGI10792[] = {{&S58897[30], 6}, {NULL}};
const Segment SGI10793[] = {{&S58897[113], 3}, {NULL}};
const Segment SGI10794[] = {{&S58897[78], 6}, {NULL}};
const Segment SGI10795[] = {{&S58898[44], 9}, {NULL}};
const Segment SGI10796[] = {{&S58898[44], 21}, {NULL}};
const Segment SGI10797[] = {{&S58898[30], 6}, {NULL}};
const Segment SGI10798[] = {{&S58898[114], 3}, {NULL}};
const Segment SGI10799[] = {{&S58898[79], 6}, {NULL}};
const Segment SGI10800[] = {{&S58903[44], 27}, {NULL}};
const Segment SGI10801[] = {{&S58903[44], 39}, {NULL}};
const Segment SGI10802[] = {{&S58903[30], 6}, {NULL}};
const Segment SGI10803[] = {{&S58903[132], 3}, {NULL}};
const Segment SGI10804[] = {{&S58903[97], 6}, {NULL}};
const Segment SGI10805[] = {{&S58909[44], 8}, {NULL}};
const Segment SGI10806[] = {{&S58909[44], 20}, {NULL}};
const Segment SGI10807[] = {{&S58909[30], 6}, {NULL}};
const Segment SGI10808[] = {{&S58909[113], 3}, {NULL}};
const Segment SGI10809[] = {{&S58909[78], 6}, {NULL}};
const Segment SGI10810[] = {{&S58911[44], 10}, {NULL}};
const Segment SGI10811[] = {{&S58911[44], 22}, {NULL}};
const Segment SGI10812[] = {{&S58911[30], 6}, {NULL}};
const Segment SGI10813[] = {{&S58911[115], 3}, {NULL}};
const Segment SGI10814[] = {{&S58911[80], 6}, {NULL}};
const Segment SGI10815[] = {{&S58913[44], 6}, {NULL}};
const Segment SGI10816[] = {{&S58913[44], 18}, {NULL}};
const Segment SGI10817[] = {{&S58913[30], 6}, {NULL}};
const Segment SGI10818[] = {{&S58913[111], 3}, {NULL}};
const Segment SGI10819[] = {{&S58913[76], 6}, {NULL}};
const Segment SGI10820[] = {{&S58914[44], 7}, {NULL}};
const Segment SGI10821[] = {{&S58914[44], 19}, {NULL}};
const Segment SGI10822[] = {{&S58914[30], 6}, {NULL}};
const Segment SGI10823[] = {{&S58914[112], 3}, {NULL}};
const Segment SGI10824[] = {{&S58914[77], 6}, {NULL}};
const Segment SGI10825[] = {{&S58915[44], 8}, {NULL}};
const Segment SGI10826[] = {{&S58915[44], 20}, {NULL}};
const Segment SGI10827[] = {{&S58915[30], 6}, {NULL}};
const Segment SGI10828[] = {{&S58915[113], 3}, {NULL}};
const Segment SGI10829[] = {{&S58915[78], 6}, {NULL}};
const Segment SGI10830[] = {{&S58918[44], 11}, {NULL}};
const Segment SGI10831[] = {{&S58918[44], 23}, {NULL}};
const Segment SGI10832[] = {{&S58918[30], 6}, {NULL}};
const Segment SGI10833[] = {{&S58918[116], 3}, {NULL}};
const Segment SGI10834[] = {{&S58918[81], 6}, {NULL}};
const Segment SGI10835[] = {{&S58928[44], 9}, {NULL}};
const Segment SGI10836[] = {{&S58928[44], 21}, {NULL}};
const Segment SGI10837[] = {{&S58928[30], 6}, {NULL}};
const Segment SGI10838[] = {{&S58928[114], 3}, {NULL}};
const Segment SGI10839[] = {{&S58928[79], 6}, {NULL}};
const Segment SGI10840[] = {{&S58929[44], 7}, {NULL}};
const Segment SGI10841[] = {{&S58929[44], 19}, {NULL}};
const Segment SGI10842[] = {{&S58929[30], 6}, {NULL}};
const Segment SGI10843[] = {{&S58929[112], 3}, {NULL}};
const Segment SGI10844[] = {{&S58929[77], 6}, {NULL}};
const Segment SGI10845[] = {{&S55757[42], 9}, {NULL}};
const Segment SGI10846[] = {{&S55757[42], 20}, {NULL}};
const Segment SGI10847[] = {{&S55757[30], 4}, {NULL}};
const Segment SGI10848[] = {{&S55757[110], 3}, {NULL}};
const Segment SGI10849[] = {{&S55757[76], 5}, {NULL}};
const Segment SGI10850[] = {{&S55758[36], 9}, {NULL}};
const Segment SGI10851[] = {{&S55758[31], 4}, {NULL}};
const Segment SGI10852[] = {{&S55759[42], 9}, {NULL}};
const Segment SGI10853[] = {{&S55759[42], 40}, {NULL}};
const Segment SGI10854[] = {{&S55759[30], 4}, {NULL}};
const Segment SGI10855[] = {{&S55760[42], 9}, {NULL}};
const Segment SGI10856[] = {{&S55760[42], 40}, {NULL}};
const Segment SGI10857[] = {{&S55760[30], 4}, {NULL}};
const Segment SGI10858[] = {{&S59103[42], 10}, {NULL}};
const Segment SGI10859[] = {{&S59103[42], 23}, {NULL}};
const Segment SGI10860[] = {{&S59103[30], 4}, {NULL}};
const Segment SGI10861[] = {{&S59103[114], 3}, {NULL}};
const Segment SGI10862[] = {{&S59103[79], 6}, {NULL}};
const Segment SGI10863[] = {{&S59103[134], 8}, {NULL}};
const Segment SGI10864[] = {{&S59104[42], 10}, {NULL}};
const Segment SGI10865[] = {{&S59104[42], 23}, {NULL}};
const Segment SGI10866[] = {{&S59104[30], 4}, {NULL}};
const Segment SGI10867[] = {{&S59104[114], 3}, {NULL}};
const Segment SGI10868[] = {{&S59104[79], 5}, {NULL}};
const Segment SGI10869[] = {{&S59104[134], 8}, {NULL}};
const Segment SGI10870[] = {{&S59105[42], 9}, {NULL}};
const Segment SGI10871[] = {{&S59105[30], 4}, {NULL}};
const Segment SGI10872[] = {{&S59105[101], 3}, {NULL}};
const Segment SGI10873[] = {{&S59105[65], 7}, {NULL}};
const Segment SGI10874[] = {{&S59105[121], 8}, {NULL}};
const Segment SGI10875[] = {{&S56306[42], 10}, {NULL}};
const Segment SGI10876[] = {{&S56306[30], 6}, {NULL}};
const Segment SGI10877[] = {{&S56306[111], 3}, {NULL}};
const Segment SGI10878[] = {{&S56306[77], 5}, {NULL}};
const Segment SGI10879[] = {{&S56307[44], 10}, {NULL}};
const Segment SGI10880[] = {{&S56307[44], 21}, {NULL}};
const Segment SGI10881[] = {{&S56307[30], 6}, {NULL}};
const Segment SGI10882[] = {{&S56307[113], 3}, {NULL}};
const Segment SGI10883[] = {{&S56307[79], 5}, {NULL}};
const Segment SGI10884[] = {{&S56308[38], 10}, {NULL}};
const Segment SGI10885[] = {{&S56308[31], 6}, {NULL}};
const Segment SGI10886[] = {{&S55554[50], 5}, {NULL}};
const Segment SGI10887[] = {{&S55554[50], 17}, {NULL}};
const Segment SGI10888[] = {{&S55554[30], 12}, {NULL}};
const Segment SGI10889[] = {{&S55554[116], 3}, {NULL}};
const Segment SGI10890[] = {{&S55554[81], 6}, {NULL}};
const Segment SGI10891[] = {{&S55555[56], 11}, {NULL}};
const Segment SGI10892[] = {{&S55555[56], 23}, {NULL}};
const Segment SGI10893[] = {{&S55555[30], 18}, {NULL}};
const Segment SGI10894[] = {{&S55555[128], 3}, {NULL}};
const Segment SGI10895[] = {{&S55555[93], 6}, {NULL}};
const Segment SGI10896[] = {{&S55556[42], 5}, {NULL}};
const Segment SGI10897[] = {{&S55556[42], 16}, {NULL}};
const Segment SGI10898[] = {{&S55556[30], 4}, {NULL}};
const Segment SGI10899[] = {{&S55556[107], 3}, {NULL}};
const Segment SGI10900[] = {{&S55556[72], 6}, {NULL}};
const Segment SGI10901[] = {{&S55557[42], 12}, {NULL}};
const Segment SGI10902[] = {{&S55557[42], 23}, {NULL}};
const Segment SGI10903[] = {{&S55557[30], 4}, {NULL}};
const Segment SGI10904[] = {{&S55557[114], 3}, {NULL}};
const Segment SGI10905[] = {{&S55557[79], 6}, {NULL}};
const Segment SGI10906[] = {{&S55558[44], 5}, {NULL}};
const Segment SGI10907[] = {{&S55558[31], 12}, {NULL}};
const Segment SGI10908[] = {{&S55559[36], 12}, {NULL}};
const Segment SGI10909[] = {{&S55559[31], 4}, {NULL}};
const Segment SGI10910[] = {{&S55610[50], 8}, {NULL}};
const Segment SGI10911[] = {{&S55610[50], 25}, {NULL}};
const Segment SGI10912[] = {{&S55610[30], 12}, {NULL}};
const Segment SGI10913[] = {{&S55610[124], 3}, {NULL}};
const Segment SGI10914[] = {{&S55610[89], 6}, {NULL}};
const Segment SGI10915[] = {{&S55611[50], 8}, {NULL}};
const Segment SGI10916[] = {{&S55611[50], 26}, {NULL}};
const Segment SGI10917[] = {{&S55611[30], 12}, {NULL}};
const Segment SGI10918[] = {{&S55611[125], 3}, {NULL}};
const Segment SGI10919[] = {{&S55611[90], 6}, {NULL}};
const Segment SGI10920[] = {{&S55612[50], 8}, {NULL}};
const Segment SGI10921[] = {{&S55612[50], 26}, {NULL}};
const Segment SGI10922[] = {{&S55612[30], 12}, {NULL}};
const Segment SGI10923[] = {{&S55612[125], 3}, {NULL}};
const Segment SGI10924[] = {{&S55612[90], 6}, {NULL}};
const Segment SGI10925[] = {{&S55613[50], 8}, {NULL}};
const Segment SGI10926[] = {{&S55613[50], 26}, {NULL}};
const Segment SGI10927[] = {{&S55613[30], 12}, {NULL}};
const Segment SGI10928[] = {{&S55613[125], 3}, {NULL}};
const Segment SGI10929[] = {{&S55613[90], 6}, {NULL}};
const Segment SGI10930[] = {{&S55614[42], 8}, {NULL}};
const Segment SGI10931[] = {{&S55614[42], 20}, {NULL}};
const Segment SGI10932[] = {{&S55614[30], 4}, {NULL}};
const Segment SGI10933[] = {{&S55614[111], 3}, {NULL}};
const Segment SGI10934[] = {{&S55614[76], 6}, {NULL}};
const Segment SGI10935[] = {{&S55616[50], 8}, {NULL}};
const Segment SGI10936[] = {{&S55616[50], 26}, {NULL}};
const Segment SGI10937[] = {{&S55616[30], 12}, {NULL}};
const Segment SGI10938[] = {{&S55616[125], 3}, {NULL}};
const Segment SGI10939[] = {{&S55616[90], 6}, {NULL}};
const Segment SGI10940[] = {{&S55617[44], 8}, {NULL}};
const Segment SGI10941[] = {{&S55617[31], 12}, {NULL}};
const Segment SGI10942[] = {{&S55761[50], 18}, {NULL}};
const Segment SGI10943[] = {{&S55761[50], 30}, {NULL}};
const Segment SGI10944[] = {{&S55761[30], 12}, {NULL}};
const Segment SGI10945[] = {{&S55761[129], 3}, {NULL}};
const Segment SGI10946[] = {{&S55761[94], 6}, {NULL}};
const Segment SGI10947[] = {{&S55762[48], 18}, {NULL}};
const Segment SGI10948[] = {{&S55762[30], 12}, {NULL}};
const Segment SGI10949[] = {{&S55762[127], 3}, {NULL}};
const Segment SGI10950[] = {{&S55762[92], 6}, {NULL}};
const Segment SGI10951[] = {{&S55763[44], 18}, {NULL}};
const Segment SGI10952[] = {{&S55763[31], 12}, {NULL}};
const Segment SGI10953[] = {{&S59484[42], 7}, {NULL}};
const Segment SGI10954[] = {{&S59484[30], 4}, {NULL}};
const Segment SGI10955[] = {{&S59484[98], 3}, {NULL}};
const Segment SGI10956[] = {{&S59484[63], 6}, {NULL}};
const Segment SGI10957[] = {{&S59484[118], 8}, {NULL}};
const Segment SGI10958[] = {{&S59485[134], 17}, {NULL}};
const Segment SGI10959[] = {{&S59485[30], 4}, {NULL}};
const Segment SGI10960[] = {{&S59485[98], 3}, {NULL}};
const Segment SGI10961[] = {{&S59485[63], 6}, {NULL}};
const Segment SGI10962[] = {{&S59485[118], 8}, {NULL}};
const Segment SGI10963[] = {{&S59486[42], 14}, {NULL}};
const Segment SGI10964[] = {{&S59486[30], 4}, {NULL}};
const Segment SGI10965[] = {{&S59486[105], 3}, {NULL}};
const Segment SGI10966[] = {{&S59486[70], 6}, {NULL}};
const Segment SGI10967[] = {{&S59486[125], 8}, {NULL}};
const Segment SGI10968[] = {{&S59487[42], 11}, {NULL}};
const Segment SGI10969[] = {{&S59487[42], 24}, {NULL}};
const Segment SGI10970[] = {{&S59487[30], 4}, {NULL}};
const Segment SGI10971[] = {{&S59487[115], 3}, {NULL}};
const Segment SGI10972[] = {{&S59487[80], 6}, {NULL}};
const Segment SGI10973[] = {{&S59487[135], 8}, {NULL}};
const Segment SGI10974[] = {{&S59488[30], 4}, {NULL}};
const Segment SGI10975[] = {{&S59488[91], 3}, {NULL}};
const Segment SGI10976[] = {{&S59488[56], 6}, {NULL}};
const Segment SGI10977[] = {{&S59488[111], 8}, {NULL}};
const Segment SGI10978[] = {{&S56034[55], 10}, {NULL}};
const Segment SGI10979[] = {{&S56034[55], 21}, {NULL}};
const Segment SGI10980[] = {{&S56034[43], 4}, {NULL}};
const Segment SGI10981[] = {{&S56034[126], 3}, {NULL}};
const Segment SGI10982[] = {{&S56034[91], 6}, {NULL}};
const Segment SGI10983[] = {{&S56035[50], 25}, {NULL}};
const Segment SGI10984[] = {{&S56035[50], 36}, {NULL}};
const Segment SGI10985[] = {{&S56035[30], 12}, {NULL}};
const Segment SGI10986[] = {{&S56035[135], 3}, {NULL}};
const Segment SGI10987[] = {{&S56035[100], 6}, {NULL}};
const Segment SGI10988[] = {{&S56036[50], 25}, {NULL}};
const Segment SGI10989[] = {{&S56036[50], 36}, {NULL}};
const Segment SGI10990[] = {{&S56036[30], 12}, {NULL}};
const Segment SGI10991[] = {{&S56036[135], 3}, {NULL}};
const Segment SGI10992[] = {{&S56036[100], 6}, {NULL}};
const Segment SGI10993[] = {{&S56037[50], 25}, {NULL}};
const Segment SGI10994[] = {{&S56037[50], 36}, {NULL}};
const Segment SGI10995[] = {{&S56037[30], 12}, {NULL}};
const Segment SGI10996[] = {{&S56037[135], 3}, {NULL}};
const Segment SGI10997[] = {{&S56037[100], 6}, {NULL}};
const Segment SGI10998[] = {{&S56038[50], 24}, {NULL}};
const Segment SGI10999[] = {{&S56038[50], 35}, {NULL}};
const Segment SGI11000[] = {{&S56038[30], 12}, {NULL}};
const Segment SGI11001[] = {{&S56038[134], 3}, {NULL}};
const Segment SGI11002[] = {{&S56038[99], 6}, {NULL}};
const Segment SGI11003[] = {{&S56039[50], 25}, {NULL}};
const Segment SGI11004[] = {{&S56039[50], 36}, {NULL}};
const Segment SGI11005[] = {{&S56039[30], 12}, {NULL}};
const Segment SGI11006[] = {{&S56039[135], 3}, {NULL}};
const Segment SGI11007[] = {{&S56039[100], 6}, {NULL}};
const Segment SGI11008[] = {{&S56040[50], 25}, {NULL}};
const Segment SGI11009[] = {{&S56040[50], 36}, {NULL}};
const Segment SGI11010[] = {{&S56040[30], 12}, {NULL}};
const Segment SGI11011[] = {{&S56040[135], 3}, {NULL}};
const Segment SGI11012[] = {{&S56040[100], 6}, {NULL}};
const Segment SGI11013[] = {{&S56041[50], 25}, {NULL}};
const Segment SGI11014[] = {{&S56041[50], 36}, {NULL}};
const Segment SGI11015[] = {{&S56041[30], 12}, {NULL}};
const Segment SGI11016[] = {{&S56041[135], 3}, {NULL}};
const Segment SGI11017[] = {{&S56041[100], 6}, {NULL}};
const Segment SGI11018[] = {{&S56042[50], 25}, {NULL}};
const Segment SGI11019[] = {{&S56042[50], 36}, {NULL}};
const Segment SGI11020[] = {{&S56042[30], 12}, {NULL}};
const Segment SGI11021[] = {{&S56042[135], 3}, {NULL}};
const Segment SGI11022[] = {{&S56042[100], 6}, {NULL}};
const Segment SGI11023[] = {{&S56043[50], 25}, {NULL}};
const Segment SGI11024[] = {{&S56043[50], 36}, {NULL}};
const Segment SGI11025[] = {{&S56043[30], 12}, {NULL}};
const Segment SGI11026[] = {{&S56043[135], 3}, {NULL}};
const Segment SGI11027[] = {{&S56043[100], 6}, {NULL}};
const Segment SGI11028[] = {{&S56044[50], 25}, {NULL}};
const Segment SGI11029[] = {{&S56044[50], 36}, {NULL}};
const Segment SGI11030[] = {{&S56044[30], 12}, {NULL}};
const Segment SGI11031[] = {{&S56044[135], 3}, {NULL}};
const Segment SGI11032[] = {{&S56044[100], 6}, {NULL}};
const Segment SGI11033[] = {{&S56045[50], 25}, {NULL}};
const Segment SGI11034[] = {{&S56045[50], 36}, {NULL}};
const Segment SGI11035[] = {{&S56045[30], 12}, {NULL}};
const Segment SGI11036[] = {{&S56045[135], 3}, {NULL}};
const Segment SGI11037[] = {{&S56045[100], 6}, {NULL}};
const Segment SGI11038[] = {{&S56844[44], 18}, {NULL}};
const Segment SGI11039[] = {{&S56844[44], 30}, {NULL}};
const Segment SGI11040[] = {{&S56844[30], 6}, {NULL}};
const Segment SGI11041[] = {{&S56844[122], 3}, {NULL}};
const Segment SGI11042[] = {{&S56046[50], 25}, {NULL}};
const Segment SGI11043[] = {{&S56046[50], 36}, {NULL}};
const Segment SGI11044[] = {{&S56844[88], 5}, {NULL}};
const Segment SGI11045[] = {{&S56046[30], 12}, {NULL}};
const Segment SGI11046[] = {{&S56046[135], 3}, {NULL}};
const Segment SGI11047[] = {{&S56046[100], 6}, {NULL}};
const Segment SGI11048[] = {{&S56845[44], 23}, {NULL}};
const Segment SGI11049[] = {{&S56845[44], 35}, {NULL}};
const Segment SGI11050[] = {{&S56845[30], 6}, {NULL}};
const Segment SGI11051[] = {{&S56047[50], 21}, {NULL}};
const Segment SGI11052[] = {{&S56845[127], 3}, {NULL}};
const Segment SGI11053[] = {{&S56047[50], 32}, {NULL}};
const Segment SGI11054[] = {{&S56047[30], 12}, {NULL}};
const Segment SGI11055[] = {{&S56845[93], 5}, {NULL}};
const Segment SGI11056[] = {{&S56047[131], 3}, {NULL}};
const Segment SGI11057[] = {{&S56047[96], 6}, {NULL}};
const Segment SGI11058[] = {{&S56048[50], 14}, {NULL}};
const Segment SGI11059[] = {{&S56048[50], 25}, {NULL}};
const Segment SGI11060[] = {{&S56048[30], 12}, {NULL}};
const Segment SGI11061[] = {{&S56847[38], 18}, {NULL}};
const Segment SGI11062[] = {{&S56048[124], 3}, {NULL}};
const Segment SGI11063[] = {{&S56847[31], 6}, {NULL}};
const Segment SGI11064[] = {{&S56048[89], 6}, {NULL}};
const Segment SGI11065[] = {{&S56049[44], 14}, {NULL}};
const Segment SGI11066[] = {{&S56049[31], 12}, {NULL}};
const Segment SGI11067[] = {{&S56848[44], 18}, {NULL}};
const Segment SGI11068[] = {{&S56848[44], 30}, {NULL}};
const Segment SGI11069[] = {{&S56848[30], 6}, {NULL}};
const Segment SGI11070[] = {{&S56848[91], 4}, {NULL}};
const Segment SGI11071[] = {{&S56848[91], 3}, {NULL}};
const Segment SGI11072[] = {{&S56050[50], 14}, {NULL}};
const Segment SGI11073[] = {{&S56050[50], 25}, {NULL}};
const Segment SGI11074[] = {{&S56050[30], 12}, {NULL}};
const Segment SGI11075[] = {{&S56849[44], 18}, {NULL}};
const Segment SGI11076[] = {{&S56050[92], 4}, {NULL}};
const Segment SGI11077[] = {{&S56849[44], 30}, {NULL}};
const Segment SGI11078[] = {{&S56050[92], 3}, {NULL}};
const Segment SGI11079[] = {{&S56849[30], 6}, {NULL}};
const Segment SGI11080[] = {{&S56849[91], 6}, {NULL}};
const Segment SGI11081[] = {{&S56182[50], 4}, {NULL}};
const Segment SGI11082[] = {{&S56182[50], 15}, {NULL}};
const Segment SGI11083[] = {{&S56182[30], 12}, {NULL}};
const Segment SGI11084[] = {{&S56182[114], 3}, {NULL}};
const Segment SGI11085[] = {{&S56182[79], 6}, {NULL}};
const Segment SGI11086[] = {{&S56183[44], 4}, {NULL}};
const Segment SGI11087[] = {{&S56183[31], 12}, {NULL}};
const Segment SGI11088[] = {{&S57028[6], 4}, {NULL}};
const Segment SGI11089[] = {{&S57028[83], 5}, {NULL}};
const Segment SGI11090[] = {{&S59834[42], 10}, {NULL}};
const Segment SGI11091[] = {{&S59834[42], 25}, {NULL}};
const Segment SGI11092[] = {{&S59834[30], 4}, {NULL}};
const Segment SGI11093[] = {{&S59834[118], 3}, {NULL}};
const Segment SGI11094[] = {{&S59834[81], 8}, {NULL}};
const Segment SGI11095[] = {{&S59834[138], 8}, {NULL}};
const Segment SGI11096[] = {{&S56251[50], 7}, {NULL}};
const Segment SGI11097[] = {{&S56251[50], 25}, {NULL}};
const Segment SGI11098[] = {{&S56251[30], 12}, {NULL}};
const Segment SGI11099[] = {{&S56251[124], 3}, {NULL}};
const Segment SGI11100[] = {{&S56251[89], 6}, {NULL}};
const Segment SGI11101[] = {{&S56252[48], 7}, {NULL}};
const Segment SGI11102[] = {{&S56252[30], 12}, {NULL}};
const Segment SGI11103[] = {{&S56252[115], 3}, {NULL}};
const Segment SGI11104[] = {{&S56252[80], 6}, {NULL}};
const Segment SGI11105[] = {{&S56253[50], 6}, {NULL}};
const Segment SGI11106[] = {{&S56253[50], 17}, {NULL}};
const Segment SGI11107[] = {{&S56253[30], 12}, {NULL}};
const Segment SGI11108[] = {{&S56253[116], 3}, {NULL}};
const Segment SGI11109[] = {{&S56253[81], 6}, {NULL}};
const Segment SGI11110[] = {{&S56254[49], 7}, {NULL}};
const Segment SGI11111[] = {{&S56254[49], 18}, {NULL}};
const Segment SGI11112[] = {{&S56254[30], 11}, {NULL}};
const Segment SGI11113[] = {{&S56254[116], 3}, {NULL}};
const Segment SGI11114[] = {{&S56254[81], 6}, {NULL}};
const Segment SGI11115[] = {{&S56255[50], 7}, {NULL}};
const Segment SGI11116[] = {{&S56255[50], 18}, {NULL}};
const Segment SGI11117[] = {{&S56255[30], 12}, {NULL}};
const Segment SGI11118[] = {{&S56255[117], 3}, {NULL}};
const Segment SGI11119[] = {{&S56255[82], 6}, {NULL}};
const Segment SGI11120[] = {{&S56256[50], 6}, {NULL}};
const Segment SGI11121[] = {{&S56256[50], 17}, {NULL}};
const Segment SGI11122[] = {{&S56256[30], 12}, {NULL}};
const Segment SGI11123[] = {{&S56256[116], 3}, {NULL}};
const Segment SGI11124[] = {{&S56256[81], 6}, {NULL}};
const Segment SGI11125[] = {{&S56257[50], 7}, {NULL}};
const Segment SGI11126[] = {{&S56257[50], 18}, {NULL}};
const Segment SGI11127[] = {{&S56257[30], 12}, {NULL}};
const Segment SGI11128[] = {{&S56257[117], 3}, {NULL}};
const Segment SGI11129[] = {{&S56257[82], 6}, {NULL}};
const Segment SGI11130[] = {{&S56258[42], 7}, {NULL}};
const Segment SGI11131[] = {{&S56258[42], 18}, {NULL}};
const Segment SGI11132[] = {{&S56258[30], 4}, {NULL}};
const Segment SGI11133[] = {{&S56258[109], 3}, {NULL}};
const Segment SGI11134[] = {{&S56258[74], 6}, {NULL}};
const Segment SGI11135[] = {{&S56259[49], 7}, {NULL}};
const Segment SGI11136[] = {{&S56259[49], 18}, {NULL}};
const Segment SGI11137[] = {{&S56259[116], 3}, {NULL}};
const Segment SGI11138[] = {{&S56259[81], 6}, {NULL}};
const Segment SGI11139[] = {{&S56260[44], 7}, {NULL}};
const Segment SGI11140[] = {{&S56260[44], 18}, {NULL}};
const Segment SGI11141[] = {{&S56260[30], 6}, {NULL}};
const Segment SGI11142[] = {{&S56260[111], 3}, {NULL}};
const Segment SGI11143[] = {{&S56260[76], 6}, {NULL}};
const Segment SGI11144[] = {{&S56261[50], 7}, {NULL}};
const Segment SGI11145[] = {{&S56261[50], 22}, {NULL}};
const Segment SGI11146[] = {{&S56261[30], 12}, {NULL}};
const Segment SGI11147[] = {{&S56261[121], 3}, {NULL}};
const Segment SGI11148[] = {{&S56261[86], 6}, {NULL}};
const Segment SGI11149[] = {{&S56262[44], 7}, {NULL}};
const Segment SGI11150[] = {{&S56262[31], 12}, {NULL}};
const Segment SGI11151[] = {{&S59917[42], 6}, {NULL}};
const Segment SGI11152[] = {{&S59917[42], 23}, {NULL}};
const Segment SGI11153[] = {{&S59917[30], 4}, {NULL}};
const Segment SGI11154[] = {{&S59917[114], 3}, {NULL}};
const Segment SGI11155[] = {{&S59917[79], 6}, {NULL}};
const Segment SGI11156[] = {{&S59917[134], 8}, {NULL}};
const Segment SGI11157[] = {{&S59919[42], 6}, {NULL}};
const Segment SGI11158[] = {{&S59919[42], 21}, {NULL}};
const Segment SGI11159[] = {{&S59919[30], 4}, {NULL}};
const Segment SGI11160[] = {{&S59919[112], 3}, {NULL}};
const Segment SGI11161[] = {{&S59919[77], 6}, {NULL}};
const Segment SGI11162[] = {{&S59919[132], 8}, {NULL}};
const Segment SGI11163[] = {{&S59920[42], 6}, {NULL}};
const Segment SGI11164[] = {{&S59920[42], 23}, {NULL}};
const Segment SGI11165[] = {{&S59920[30], 4}, {NULL}};
const Segment SGI11166[] = {{&S59920[114], 3}, {NULL}};
const Segment SGI11167[] = {{&S59920[79], 6}, {NULL}};
const Segment SGI11168[] = {{&S59920[134], 8}, {NULL}};
const Segment SGI11169[] = {{&S59921[42], 6}, {NULL}};
const Segment SGI11170[] = {{&S59921[42], 21}, {NULL}};
const Segment SGI11171[] = {{&S59921[30], 4}, {NULL}};
const Segment SGI11172[] = {{&S59921[112], 3}, {NULL}};
const Segment SGI11173[] = {{&S59921[77], 6}, {NULL}};
const Segment SGI11174[] = {{&S59921[132], 8}, {NULL}};
const Segment SGI11175[] = {{&S59922[42], 6}, {NULL}};
const Segment SGI11176[] = {{&S59922[42], 22}, {NULL}};
const Segment SGI11177[] = {{&S59922[30], 4}, {NULL}};
const Segment SGI11178[] = {{&S59922[113], 3}, {NULL}};
const Segment SGI11179[] = {{&S59922[78], 6}, {NULL}};
const Segment SGI11180[] = {{&S59922[133], 8}, {NULL}};
const Segment SGI11181[] = {{&S59923[42], 6}, {NULL}};
const Segment SGI11182[] = {{&S59923[42], 22}, {NULL}};
const Segment SGI11183[] = {{&S59923[30], 4}, {NULL}};
const Segment SGI11184[] = {{&S59923[113], 3}, {NULL}};
const Segment SGI11185[] = {{&S59923[78], 6}, {NULL}};
const Segment SGI11186[] = {{&S59923[133], 8}, {NULL}};
const Segment SGI11187[] = {{&S59932[42], 5}, {NULL}};
const Segment SGI11188[] = {{&S59932[42], 16}, {NULL}};
const Segment SGI11189[] = {{&S59932[30], 4}, {NULL}};
const Segment SGI11190[] = {{&S59932[107], 3}, {NULL}};
const Segment SGI11191[] = {{&S59932[72], 6}, {NULL}};
const Segment SGI11192[] = {{&S59932[127], 8}, {NULL}};
const Segment SGI11193[] = {{&S59933[42], 5}, {NULL}};
const Segment SGI11194[] = {{&S59933[42], 16}, {NULL}};
const Segment SGI11195[] = {{&S59933[30], 4}, {NULL}};
const Segment SGI11196[] = {{&S58895[44], 22}, {NULL}};
const Segment SGI11197[] = {{&S58895[30], 6}, {NULL}};
const Segment SGI11198[] = {{&S58895[128], 3}, {NULL}};
const Segment SGI11199[] = {{&S58895[93], 6}, {NULL}};
const Segment SGI11200[] = {{&S58899[30], 4}, {NULL}};
const Segment SGI11201[] = {{&S58899[90], 3}, {NULL}};
const Segment SGI11202[] = {{&S58899[56], 5}, {NULL}};
const Segment SGI11203[] = {{&S56846[42], 23}, {NULL}};
const Segment SGI11204[] = {{&S56846[42], 35}, {NULL}};
const Segment SGI11205[] = {{&S56846[30], 4}, {NULL}};
const Segment SGI11206[] = {{&S56846[125], 3}, {NULL}};
const Segment SGI11207[] = {{&S56846[91], 5}, {NULL}};
const Segment SGI11208[] = {{&S57228[44], 11}, {NULL}};
const Segment SGI11209[] = {{&S57228[44], 22}, {NULL}};
const Segment SGI11210[] = {{&S57228[30], 6}, {NULL}};
const Segment SGI11211[] = {{&S57228[114], 3}, {NULL}};
const Segment SGI11212[] = {{&S57228[80], 5}, {NULL}};
const Segment SGI11213[] = {{&S57229[38], 11}, {NULL}};
const Segment SGI11214[] = {{&S57229[31], 6}, {NULL}};
const Segment SGI11215[] = {{&S57250[44], 21}, {NULL}};
const Segment SGI11216[] = {{&S57250[44], 33}, {NULL}};
const Segment SGI11217[] = {{&S57250[30], 6}, {&S57250[52], 25}, {NULL}};
const Segment SGI11218[] = {{&S57250[125], 3}, {NULL}};
const Segment SGI11219[] = {{&S57250[91], 5}, {NULL}};
const Segment SGI11220[] = {{&S57251[44], 9}, {NULL}};
const Segment SGI11221[] = {{&S57251[44], 21}, {NULL}};
const Segment SGI11222[] = {{&S57251[30], 6}, {NULL}};
const Segment SGI11223[] = {{&S57251[113], 3}, {NULL}};
const Segment SGI11224[] = {{&S57251[79], 5}, {NULL}};
const Segment SGI11225[] = {{&S57252[44], 9}, {NULL}};
const Segment SGI11226[] = {{&S57252[44], 20}, {NULL}};
const Segment SGI11227[] = {{&S57252[30], 6}, {NULL}};
const Segment SGI11228[] = {{&S57252[112], 3}, {NULL}};
const Segment SGI11229[] = {{&S57252[78], 5}, {NULL}};
const Segment SGI11230[] = {{&S57253[44], 9}, {NULL}};
const Segment SGI11231[] = {{&S57253[44], 21}, {NULL}};
const Segment SGI11232[] = {{&S57253[30], 6}, {NULL}};
const Segment SGI11233[] = {{&S57253[113], 3}, {NULL}};
const Segment SGI11234[] = {{&S57253[79], 5}, {NULL}};
const Segment SGI11235[] = {{&S57254[44], 11}, {NULL}};
const Segment SGI11236[] = {{&S57254[44], 22}, {NULL}};
const Segment SGI11237[] = {{&S57254[30], 6}, {NULL}};
const Segment SGI11238[] = {{&S57254[114], 3}, {NULL}};
const Segment SGI11239[] = {{&S57254[80], 5}, {NULL}};
const Segment SGI11240[] = {{&S57255[44], 11}, {NULL}};
const Segment SGI11241[] = {{&S57255[44], 22}, {NULL}};
const Segment SGI11242[] = {{&S57255[30], 6}, {NULL}};
const Segment SGI11243[] = {{&S57255[114], 3}, {NULL}};
const Segment SGI11244[] = {{&S57255[80], 5}, {NULL}};
const Segment SGI11245[] = {{&S57257[44], 15}, {NULL}};
const Segment SGI11246[] = {{&S57257[44], 27}, {NULL}};
const Segment SGI11247[] = {{&S57257[30], 6}, {NULL}};
const Segment SGI11248[] = {{&S57257[119], 3}, {NULL}};
const Segment SGI11249[] = {{&S57257[85], 5}, {NULL}};
const Segment SGI11250[] = {{&S57260[44], 9}, {NULL}};
const Segment SGI11251[] = {{&S57260[44], 20}, {NULL}};
const Segment SGI11252[] = {{&S57260[30], 6}, {NULL}};
const Segment SGI11253[] = {{&S57260[112], 3}, {NULL}};
const Segment SGI11254[] = {{&S57260[78], 5}, {NULL}};
const Segment SGI11255[] = {{&S57262[44], 9}, {NULL}};
const Segment SGI11256[] = {{&S57262[44], 19}, {NULL}};
const Segment SGI11257[] = {{&S57262[30], 6}, {NULL}};
const Segment SGI11258[] = {{&S57262[111], 3}, {NULL}};
const Segment SGI11259[] = {{&S57262[77], 5}, {NULL}};
const Segment SGI11260[] = {{&S57263[38], 9}, {NULL}};
const Segment SGI11261[] = {{&S57263[31], 6}, {NULL}};
const Segment SGI11262[] = {{&S56970[42], 9}, {NULL}};
const Segment SGI11263[] = {{&S56970[42], 20}, {NULL}};
const Segment SGI11264[] = {{&S56970[30], 4}, {NULL}};
const Segment SGI11265[] = {{&S56970[110], 3}, {NULL}};
const Segment SGI11266[] = {{&S56970[76], 5}, {NULL}};
const Segment SGI11267[] = {{&S56971[42], 13}, {NULL}};
const Segment SGI11268[] = {{&S56971[42], 24}, {NULL}};
const Segment SGI11269[] = {{&S56971[30], 4}, {NULL}};
const Segment SGI11270[] = {{&S56971[114], 3}, {NULL}};
const Segment SGI11271[] = {{&S56971[80], 5}, {NULL}};
const Segment SGI11272[] = {{&S56972[36], 9}, {NULL}};
const Segment SGI11273[] = {{&S56972[31], 4}, {NULL}};
const Segment SGI11274[] = {{&S56973[42], 9}, {NULL}};
const Segment SGI11275[] = {{&S56973[42], 40}, {NULL}};
const Segment SGI11276[] = {{&S56973[30], 4}, {NULL}};
const Segment SGI11277[] = {{&S57026[6], 4}, {NULL}};
const Segment SGI11278[] = {{&S57026[84], 5}, {NULL}};
const Segment SGI11279[] = {{&S56730[50], 9}, {NULL}};
const Segment SGI11280[] = {{&S56730[50], 21}, {NULL}};
const Segment SGI11281[] = {{&S56730[30], 12}, {NULL}};
const Segment SGI11282[] = {{&S56730[120], 3}, {NULL}};
const Segment SGI11283[] = {{&S56730[85], 6}, {NULL}};
const Segment SGI11284[] = {{&S56731[50], 10}, {NULL}};
const Segment SGI11285[] = {{&S56731[50], 22}, {NULL}};
const Segment SGI11286[] = {{&S56731[30], 12}, {NULL}};
const Segment SGI11287[] = {{&S56731[121], 3}, {NULL}};
const Segment SGI11288[] = {{&S56731[86], 6}, {NULL}};
const Segment SGI11289[] = {{&S56732[44], 9}, {NULL}};
const Segment SGI11290[] = {{&S56732[31], 12}, {NULL}};
const Segment SGI11291[] = {{&S56733[44], 10}, {NULL}};
const Segment SGI11292[] = {{&S56733[31], 12}, {NULL}};
const Segment SGI11293[] = {{&S56734[50], 10}, {NULL}};
const Segment SGI11294[] = {{&S56734[50], 21}, {NULL}};
const Segment SGI11295[] = {{&S56734[30], 12}, {NULL}};
const Segment SGI11296[] = {{&S56734[120], 3}, {NULL}};
const Segment SGI11297[] = {{&S56734[85], 6}, {NULL}};
const Segment SGI11298[] = {{&S56735[50], 9}, {NULL}};
const Segment SGI11299[] = {{&S56735[50], 68}, {NULL}};
const Segment SGI11300[] = {{&S56735[30], 12}, {NULL}};
const Segment SGI11301[] = {{&S56739[42], 28}, {NULL}};
const Segment SGI11302[] = {{&S56739[30], 4}, {NULL}};
const Segment SGI11303[] = {{&S56739[119], 3}, {NULL}};
const Segment SGI11304[] = {{&S56739[84], 6}, {NULL}};
const Segment SGI11305[] = {{&S60422[42], 11}, {NULL}};
const Segment SGI11306[] = {{&S60422[42], 22}, {NULL}};
const Segment SGI11307[] = {{&S60422[30], 4}, {NULL}};
const Segment SGI11308[] = {{&S60422[113], 3}, {NULL}};
const Segment SGI11309[] = {{&S60422[78], 6}, {NULL}};
const Segment SGI11310[] = {{&S60422[133], 8}, {NULL}};
const Segment SGI11311[] = {{&S60423[42], 11}, {NULL}};
const Segment SGI11312[] = {{&S60423[42], 22}, {NULL}};
const Segment SGI11313[] = {{&S60423[30], 4}, {NULL}};
const Segment SGI11314[] = {{&S60423[113], 3}, {NULL}};
const Segment SGI11315[] = {{&S60423[78], 6}, {NULL}};
const Segment SGI11316[] = {{&S60423[133], 8}, {NULL}};
const Segment SGI11317[] = {{&S60424[42], 11}, {NULL}};
const Segment SGI11318[] = {{&S60424[42], 22}, {NULL}};
const Segment SGI11319[] = {{&S60424[30], 4}, {NULL}};
const Segment SGI11320[] = {{&S60424[113], 3}, {NULL}};
const Segment SGI11321[] = {{&S60424[78], 6}, {NULL}};
const Segment SGI11322[] = {{&S60424[133], 8}, {NULL}};
const Segment SGI11323[] = {{&S57224[48], 10}, {NULL}};
const Segment SGI11324[] = {{&S57224[48], 27}, {NULL}};
const Segment SGI11325[] = {{&S57224[36], 4}, {NULL}};
const Segment SGI11326[] = {{&S57225[42], 10}, {NULL}};
const Segment SGI11327[] = {{&S57225[42], 21}, {NULL}};
const Segment SGI11328[] = {{&S57225[30], 4}, {NULL}};
const Segment SGI11329[] = {{&S57225[111], 3}, {NULL}};
const Segment SGI11330[] = {{&S57225[77], 5}, {NULL}};
const Segment SGI11331[] = {{&S57226[36], 10}, {NULL}};
const Segment SGI11332[] = {{&S57226[31], 4}, {NULL}};
const Segment SGI11333[] = {{&S57227[42], 10}, {NULL}};
const Segment SGI11334[] = {{&S57227[42], 21}, {NULL}};
const Segment SGI11335[] = {{&S57227[30], 4}, {NULL}};
const Segment SGI11336[] = {{&S57227[111], 3}, {NULL}};
const Segment SGI11337[] = {{&S57227[77], 5}, {NULL}};
const Segment SGI11338[] = {{&S57249[40], 9}, {NULL}};
const Segment SGI11339[] = {{&S57249[30], 4}, {NULL}};
const Segment SGI11340[] = {{&S57249[108], 3}, {NULL}};
const Segment SGI11341[] = {{&S57249[74], 5}, {NULL}};
const Segment SGI11342[] = {{&S57256[42], 11}, {NULL}};
const Segment SGI11343[] = {{&S57256[42], 22}, {NULL}};
const Segment SGI11344[] = {{&S57256[30], 4}, {NULL}};
const Segment SGI11345[] = {{&S57256[112], 3}, {NULL}};
const Segment SGI11346[] = {{&S57256[78], 5}, {NULL}};
const Segment SGI11347[] = {{&S57258[42], 15}, {NULL}};
const Segment SGI11348[] = {{&S57258[42], 26}, {NULL}};
const Segment SGI11349[] = {{&S57258[30], 4}, {NULL}};
const Segment SGI11350[] = {{&S57258[116], 3}, {NULL}};
const Segment SGI11351[] = {{&S57258[82], 5}, {NULL}};
const Segment SGI11352[] = {{&S57259[42], 9}, {NULL}};
const Segment SGI11353[] = {{&S57259[42], 20}, {NULL}};
const Segment SGI11354[] = {{&S57259[30], 4}, {NULL}};
const Segment SGI11355[] = {{&S57259[110], 3}, {NULL}};
const Segment SGI11356[] = {{&S57259[76], 5}, {NULL}};
const Segment SGI11357[] = {{&S57261[36], 9}, {NULL}};
const Segment SGI11358[] = {{&S57261[31], 4}, {NULL}};
const Segment SGI11359[] = {{&S57264[42], 9}, {NULL}};
const Segment SGI11360[] = {{&S57264[42], 20}, {NULL}};
const Segment SGI11361[] = {{&S57264[30], 4}, {NULL}};
const Segment SGI11362[] = {{&S57264[79], 4}, {NULL}};
const Segment SGI11363[] = {{&S57264[79], 3}, {NULL}};
const Segment SGI11364[] = {{&S60603[42], 9}, {NULL}};
const Segment SGI11365[] = {{&S60603[42], 20}, {NULL}};
const Segment SGI11366[] = {{&S60603[30], 4}, {NULL}};
const Segment SGI11367[] = {{&S60603[111], 3}, {NULL}};
const Segment SGI11368[] = {{&S60603[76], 6}, {NULL}};
const Segment SGI11369[] = {{&S60603[131], 8}, {NULL}};
const Segment SGI11370[] = {{&S57824[44], 5}, {NULL}};
const Segment SGI11371[] = {{&S57824[44], 16}, {NULL}};
const Segment SGI11372[] = {{&S57824[30], 6}, {NULL}};
const Segment SGI11373[] = {{&S57824[108], 3}, {NULL}};
const Segment SGI11374[] = {{&S57824[74], 5}, {NULL}};
const Segment SGI11375[] = {{&S57825[38], 5}, {NULL}};
const Segment SGI11376[] = {{&S57825[31], 6}, {NULL}};
const Segment SGI11377[] = {{&S57611[42], 10}, {NULL}};
const Segment SGI11378[] = {{&S57611[42], 21}, {NULL}};
const Segment SGI11379[] = {{&S57611[30], 4}, {NULL}};
const Segment SGI11380[] = {{&S57611[111], 3}, {NULL}};
const Segment SGI11381[] = {{&S57611[77], 5}, {NULL}};
const Segment SGI11382[] = {{&S57612[36], 10}, {NULL}};
const Segment SGI11383[] = {{&S57612[31], 4}, {NULL}};
const Segment SGI11384[] = {{&S57613[42], 10}, {NULL}};
const Segment SGI11385[] = {{&S57613[42], 21}, {NULL}};
const Segment SGI11386[] = {{&S57613[30], 4}, {NULL}};
const Segment SGI11387[] = {{&S57613[80], 4}, {NULL}};
const Segment SGI11388[] = {{&S57613[80], 3}, {NULL}};
const Segment SGI11389[] = {{&S57237[50], 9}, {NULL}};
const Segment SGI11390[] = {{&S57237[50], 21}, {NULL}};
const Segment SGI11391[] = {{&S57237[30], 12}, {NULL}};
const Segment SGI11392[] = {{&S57237[120], 3}, {NULL}};
const Segment SGI11393[] = {{&S57237[85], 6}, {NULL}};
const Segment SGI11394[] = {{&S57238[50], 13}, {NULL}};
const Segment SGI11395[] = {{&S57238[50], 25}, {NULL}};
const Segment SGI11396[] = {{&S57238[30], 12}, {NULL}};
const Segment SGI11397[] = {{&S57238[124], 3}, {NULL}};
const Segment SGI11398[] = {{&S57238[89], 6}, {NULL}};
const Segment SGI11399[] = {{&S57239[50], 8}, {NULL}};
const Segment SGI11400[] = {{&S57239[50], 20}, {NULL}};
const Segment SGI11401[] = {{&S57239[30], 12}, {NULL}};
const Segment SGI11402[] = {{&S57239[119], 3}, {NULL}};
const Segment SGI11403[] = {{&S57239[84], 6}, {NULL}};
const Segment SGI11404[] = {{&S57240[44], 8}, {NULL}};
const Segment SGI11405[] = {{&S57240[31], 12}, {NULL}};
const Segment SGI11406[] = {{&S58900[66], 8}, {NULL}};
const Segment SGI11407[] = {{&S58900[66], 20}, {NULL}};
const Segment SGI11408[] = {{&S58900[30], 28}, {NULL}};
const Segment SGI11409[] = {{&S58900[135], 3}, {NULL}};
const Segment SGI11410[] = {{&S58900[100], 6}, {NULL}};
const Segment SGI11411[] = {{&S58901[66], 8}, {NULL}};
const Segment SGI11412[] = {{&S58901[66], 20}, {NULL}};
const Segment SGI11413[] = {{&S58901[30], 28}, {NULL}};
const Segment SGI11414[] = {{&S58901[135], 3}, {NULL}};
const Segment SGI11415[] = {{&S58901[100], 6}, {NULL}};
const Segment SGI11416[] = {{&S58902[66], 8}, {NULL}};
const Segment SGI11417[] = {{&S58902[66], 20}, {NULL}};
const Segment SGI11418[] = {{&S58902[30], 28}, {NULL}};
const Segment SGI11419[] = {{&S58902[135], 3}, {NULL}};
const Segment SGI11420[] = {{&S58902[100], 6}, {NULL}};
const Segment SGI11421[] = {{&S58904[64], 8}, {NULL}};
const Segment SGI11422[] = {{&S58904[64], 20}, {NULL}};
const Segment SGI11423[] = {{&S58904[30], 26}, {NULL}};
const Segment SGI11424[] = {{&S58904[133], 3}, {NULL}};
const Segment SGI11425[] = {{&S58904[98], 6}, {NULL}};
const Segment SGI11426[] = {{&S58905[66], 8}, {NULL}};
const Segment SGI11427[] = {{&S58905[66], 20}, {NULL}};
const Segment SGI11428[] = {{&S58905[30], 28}, {NULL}};
const Segment SGI11429[] = {{&S58905[135], 3}, {NULL}};
const Segment SGI11430[] = {{&S58905[100], 6}, {NULL}};
const Segment SGI11431[] = {{&S58906[66], 8}, {NULL}};
const Segment SGI11432[] = {{&S58906[66], 20}, {NULL}};
const Segment SGI11433[] = {{&S58906[30], 28}, {NULL}};
const Segment SGI11434[] = {{&S58906[135], 3}, {NULL}};
const Segment SGI11435[] = {{&S58906[100], 6}, {NULL}};
const Segment SGI11436[] = {{&S58907[64], 7}, {NULL}};
const Segment SGI11437[] = {{&S58907[64], 19}, {NULL}};
const Segment SGI11438[] = {{&S58907[30], 26}, {NULL}};
const Segment SGI11439[] = {{&S58907[132], 3}, {NULL}};
const Segment SGI11440[] = {{&S58907[97], 6}, {NULL}};
const Segment SGI11441[] = {{&S58908[64], 9}, {NULL}};
const Segment SGI11442[] = {{&S58908[64], 21}, {NULL}};
const Segment SGI11443[] = {{&S58908[30], 26}, {NULL}};
const Segment SGI11444[] = {{&S58908[134], 3}, {NULL}};
const Segment SGI11445[] = {{&S58908[99], 6}, {NULL}};
const Segment SGI11446[] = {{&S58910[45], 12}, {NULL}};
const Segment SGI11447[] = {{&S58910[45], 24}, {NULL}};
const Segment SGI11448[] = {{&S58910[30], 7}, {NULL}};
const Segment SGI11449[] = {{&S58910[118], 3}, {NULL}};
const Segment SGI11450[] = {{&S58910[83], 6}, {NULL}};
const Segment SGI11451[] = {{&S58912[57], 7}, {NULL}};
const Segment SGI11452[] = {{&S58912[57], 19}, {NULL}};
const Segment SGI11453[] = {{&S58912[30], 19}, {NULL}};
const Segment SGI11454[] = {{&S58912[125], 3}, {NULL}};
const Segment SGI11455[] = {{&S58912[90], 6}, {NULL}};
const Segment SGI11456[] = {{&S58916[66], 8}, {NULL}};
const Segment SGI11457[] = {{&S58916[66], 20}, {NULL}};
const Segment SGI11458[] = {{&S58916[30], 28}, {NULL}};
const Segment SGI11459[] = {{&S58916[135], 3}, {NULL}};
const Segment SGI11460[] = {{&S58916[100], 6}, {NULL}};
const Segment SGI11461[] = {{&S58917[66], 8}, {NULL}};
const Segment SGI11462[] = {{&S58917[66], 20}, {NULL}};
const Segment SGI11463[] = {{&S58917[30], 28}, {NULL}};
const Segment SGI11464[] = {{&S58917[135], 3}, {NULL}};
const Segment SGI11465[] = {{&S58917[100], 6}, {NULL}};
const Segment SGI11466[] = {{&S58919[23], 16}, {NULL}};
const Segment SGI11467[] = {{&S58919[31], 20}, {NULL}};
const Segment SGI11468[] = {{&S58920[56], 6}, {NULL}};
const Segment SGI11469[] = {{&S58920[56], 18}, {NULL}};
const Segment SGI11470[] = {{&S58920[30], 18}, {NULL}};
const Segment SGI11471[] = {{&S58920[123], 3}, {NULL}};
const Segment SGI11472[] = {{&S58920[88], 6}, {NULL}};
const Segment SGI11473[] = {{&S58921[64], 8}, {NULL}};
const Segment SGI11474[] = {{&S58921[64], 20}, {NULL}};
const Segment SGI11475[] = {{&S58921[30], 26}, {NULL}};
const Segment SGI11476[] = {{&S58921[133], 3}, {NULL}};
const Segment SGI11477[] = {{&S58921[98], 6}, {NULL}};
const Segment SGI11478[] = {{&S58043[42], 10}, {NULL}};
const Segment SGI11479[] = {{&S58043[42], 25}, {NULL}};
const Segment SGI11480[] = {{&S58043[30], 4}, {NULL}};
const Segment SGI11481[] = {{&S58043[115], 3}, {NULL}};
const Segment SGI11482[] = {{&S58043[81], 5}, {NULL}};
const Segment SGI11483[] = {{&S58922[45], 9}, {NULL}};
const Segment SGI11484[] = {{&S58922[45], 21}, {NULL}};
const Segment SGI11485[] = {{&S58922[30], 7}, {NULL}};
const Segment SGI11486[] = {{&S58922[115], 3}, {NULL}};
const Segment SGI11487[] = {{&S58922[80], 6}, {NULL}};
const Segment SGI11488[] = {{&S58923[64], 14}, {NULL}};
const Segment SGI11489[] = {{&S58923[64], 26}, {NULL}};
const Segment SGI11490[] = {{&S58923[30], 26}, {NULL}};
const Segment SGI11491[] = {{&S58923[139], 3}, {NULL}};
const Segment SGI11492[] = {{&S58923[104], 6}, {NULL}};
const Segment SGI11493[] = {{&S58924[64], 6}, {NULL}};
const Segment SGI11494[] = {{&S58924[64], 18}, {NULL}};
const Segment SGI11495[] = {{&S58924[30], 26}, {NULL}};
const Segment SGI11496[] = {{&S58924[131], 3}, {NULL}};
const Segment SGI11497[] = {{&S58924[96], 6}, {NULL}};
const Segment SGI11498[] = {{&S58925[65], 13}, {NULL}};
const Segment SGI11499[] = {{&S58925[65], 25}, {NULL}};
const Segment SGI11500[] = {{&S58925[30], 27}, {NULL}};
const Segment SGI11501[] = {{&S58925[139], 3}, {NULL}};
const Segment SGI11502[] = {{&S58925[104], 6}, {NULL}};
const Segment SGI11503[] = {{&S58926[64], 6}, {NULL}};
const Segment SGI11504[] = {{&S58926[64], 18}, {NULL}};
const Segment SGI11505[] = {{&S58926[30], 26}, {NULL}};
const Segment SGI11506[] = {{&S58926[131], 3}, {NULL}};
const Segment SGI11507[] = {{&S58926[96], 6}, {NULL}};
const Segment SGI11508[] = {{&S58927[45], 8}, {NULL}};
const Segment SGI11509[] = {{&S58927[45], 20}, {NULL}};
const Segment SGI11510[] = {{&S58927[30], 7}, {NULL}};
const Segment SGI11511[] = {{&S58927[114], 3}, {NULL}};
const Segment SGI11512[] = {{&S58927[79], 6}, {NULL}};
const Segment SGI11513[] = {{&S60159[30], 4}, {NULL}};
const Segment SGI11514[] = {{&S60159[90], 3}, {NULL}};
const Segment SGI11515[] = {{&S60159[56], 5}, {NULL}};
const Segment SGI11516[] = {{&S58110[42], 9}, {NULL}};
const Segment SGI11517[] = {{&S58110[42], 20}, {NULL}};
const Segment SGI11518[] = {{&S58110[30], 4}, {NULL}};
const Segment SGI11519[] = {{&S58110[110], 3}, {NULL}};
const Segment SGI11520[] = {{&S58110[76], 5}, {NULL}};
const Segment SGI11521[] = {{&S58111[36], 9}, {NULL}};
const Segment SGI11522[] = {{&S58111[31], 4}, {NULL}};
const Segment SGI11523[] = {{&S58112[42], 9}, {NULL}};
const Segment SGI11524[] = {{&S58112[42], 40}, {NULL}};
const Segment SGI11525[] = {{&S58112[30], 4}, {NULL}};
const Segment SGI11526[] = {{&S57573[50], 7}, {NULL}};
const Segment SGI11527[] = {{&S57573[50], 16}, {NULL}};
const Segment SGI11528[] = {{&S57573[30], 12}, {NULL}};
const Segment SGI11529[] = {{&S57573[115], 3}, {NULL}};
const Segment SGI11530[] = {{&S57573[80], 6}, {NULL}};
const Segment SGI11531[] = {{&S59106[105], 3}, {NULL}};
const Segment SGI11532[] = {{&S59106[68], 8}, {NULL}};
const Segment SGI11533[] = {{&S59107[43], 8}, {NULL}};
const Segment SGI11534[] = {{&S59107[43], 39}, {NULL}};
const Segment SGI11535[] = {{&S59107[30], 5}, {NULL}};
const Segment SGI11536[] = {{&S59107[130], 3}, {NULL}};
const Segment SGI11537[] = {{&S59107[96], 5}, {NULL}};
const Segment SGI11538[] = {{&S59108[50], 16}, {NULL}};
const Segment SGI11539[] = {{&S59108[50], 27}, {NULL}};
const Segment SGI11540[] = {{&S59108[30], 12}, {NULL}};
const Segment SGI11541[] = {{&S59108[126], 3}, {NULL}};
const Segment SGI11542[] = {{&S59108[91], 6}, {NULL}};
const Segment SGI11543[] = {{&S59108[146], 8}, {NULL}};
const Segment SGI11544[] = {{&S59109[45], 10}, {NULL}};
const Segment SGI11545[] = {{&S59109[45], 21}, {NULL}};
const Segment SGI11546[] = {{&S59109[30], 7}, {NULL}};
const Segment SGI11547[] = {{&S59109[114], 3}, {NULL}};
const Segment SGI11548[] = {{&S59109[80], 5}, {NULL}};
const Segment SGI11549[] = {{&S59110[47], 9}, {NULL}};
const Segment SGI11550[] = {{&S59110[47], 21}, {NULL}};
const Segment SGI11551[] = {{&S59110[30], 9}, {NULL}};
const Segment SGI11552[] = {{&S59110[116], 3}, {NULL}};
const Segment SGI11553[] = {{&S59110[82], 5}, {NULL}};
const Segment SGI11554[] = {{&S59111[37], 8}, {NULL}};
const Segment SGI11555[] = {{&S59111[31], 5}, {NULL}};
const Segment SGI11556[] = {{&S59112[46], 10}, {NULL}};
const Segment SGI11557[] = {{&S59112[46], 22}, {NULL}};
const Segment SGI11558[] = {{&S59112[30], 8}, {NULL}};
const Segment SGI11559[] = {{&S59112[116], 3}, {NULL}};
const Segment SGI11560[] = {{&S59112[82], 5}, {NULL}};
const Segment SGI11561[] = {{&S59115[46], 5}, {NULL}};
const Segment SGI11562[] = {{&S59115[46], 17}, {NULL}};
const Segment SGI11563[] = {{&S59115[30], 8}, {NULL}};
const Segment SGI11564[] = {{&S59115[111], 3}, {NULL}};
const Segment SGI11565[] = {{&S59115[77], 5}, {NULL}};
const Segment SGI11566[] = {{&S59116[48], 11}, {NULL}};
const Segment SGI11567[] = {{&S59116[48], 23}, {NULL}};
const Segment SGI11568[] = {{&S59116[30], 10}, {NULL}};
const Segment SGI11569[] = {{&S59116[37], 3}, {NULL}};
const Segment SGI11570[] = {{&S59116[85], 5}, {NULL}};
const Segment SGI11571[] = {{&S59117[46], 12}, {NULL}};
const Segment SGI11572[] = {{&S59117[46], 24}, {NULL}};
const Segment SGI11573[] = {{&S59117[30], 8}, {NULL}};
const Segment SGI11574[] = {{&S59117[118], 3}, {NULL}};
const Segment SGI11575[] = {{&S59117[84], 5}, {NULL}};
const Segment SGI11576[] = {{&S59118[46], 11}, {NULL}};
const Segment SGI11577[] = {{&S59118[46], 23}, {NULL}};
const Segment SGI11578[] = {{&S59118[30], 8}, {NULL}};
const Segment SGI11579[] = {{&S59118[117], 3}, {NULL}};
const Segment SGI11580[] = {{&S59118[83], 5}, {NULL}};
const Segment SGI11581[] = {{&S58251[42], 13}, {NULL}};
const Segment SGI11582[] = {{&S58251[42], 24}, {NULL}};
const Segment SGI11583[] = {{&S58251[30], 4}, {NULL}};
const Segment SGI11584[] = {{&S58251[114], 3}, {NULL}};
const Segment SGI11585[] = {{&S59119[46], 10}, {NULL}};
const Segment SGI11586[] = {{&S59119[46], 22}, {NULL}};
const Segment SGI11587[] = {{&S58251[80], 5}, {NULL}};
const Segment SGI11588[] = {{&S59119[30], 8}, {NULL}};
const Segment SGI11589[] = {{&S59119[116], 3}, {NULL}};
const Segment SGI11590[] = {{&S59119[82], 5}, {NULL}};
const Segment SGI11591[] = {{&S59120[46], 8}, {NULL}};
const Segment SGI11592[] = {{&S59120[46], 20}, {NULL}};
const Segment SGI11593[] = {{&S59120[30], 8}, {NULL}};
const Segment SGI11594[] = {{&S59120[114], 3}, {NULL}};
const Segment SGI11595[] = {{&S59120[80], 5}, {NULL}};
const Segment SGI11596[] = {{&S59123[46], 11}, {NULL}};
const Segment SGI11597[] = {{&S59123[46], 23}, {NULL}};
const Segment SGI11598[] = {{&S59123[30], 8}, {NULL}};
const Segment SGI11599[] = {{&S59123[117], 3}, {NULL}};
const Segment SGI11600[] = {{&S59123[83], 5}, {NULL}};
const Segment SGI11601[] = {{&S59124[46], 13}, {NULL}};
const Segment SGI11602[] = {{&S59124[46], 25}, {NULL}};
const Segment SGI11603[] = {{&S59124[30], 8}, {NULL}};
const Segment SGI11604[] = {{&S61422[44], 7}, {NULL}};
const Segment SGI11605[] = {{&S61422[44], 19}, {NULL}};
const Segment SGI11606[] = {{&S61422[30], 6}, {NULL}};
const Segment SGI11607[] = {{&S61422[112], 3}, {NULL}};
const Segment SGI11608[] = {{&S61422[77], 6}, {NULL}};
const Segment SGI11609[] = {{&S61423[45], 7}, {NULL}};
const Segment SGI11610[] = {{&S61423[45], 19}, {NULL}};
const Segment SGI11611[] = {{&S61423[30], 7}, {NULL}};
const Segment SGI11612[] = {{&S61423[113], 3}, {NULL}};
const Segment SGI11613[] = {{&S61423[78], 6}, {NULL}};
const Segment SGI11614[] = {{&S61425[44], 7}, {NULL}};
const Segment SGI11615[] = {{&S61425[44], 19}, {NULL}};
const Segment SGI11616[] = {{&S61425[30], 6}, {NULL}};
const Segment SGI11617[] = {{&S61425[112], 3}, {NULL}};
const Segment SGI11618[] = {{&S61425[77], 6}, {NULL}};
const Segment SGI11619[] = {{&S58830[44], 12}, {NULL}};
const Segment SGI11620[] = {{&S58830[44], 23}, {NULL}};
const Segment SGI11621[] = {{&S58830[30], 6}, {NULL}};
const Segment SGI11622[] = {{&S58830[115], 3}, {NULL}};
const Segment SGI11623[] = {{&S58830[81], 5}, {NULL}};
const Segment SGI11624[] = {{&S58831[44], 13}, {NULL}};
const Segment SGI11625[] = {{&S58831[44], 24}, {NULL}};
const Segment SGI11626[] = {{&S58831[30], 6}, {NULL}};
const Segment SGI11627[] = {{&S58831[116], 3}, {NULL}};
const Segment SGI11628[] = {{&S58831[82], 5}, {NULL}};
const Segment SGI11629[] = {{&S58832[43], 13}, {NULL}};
const Segment SGI11630[] = {{&S58832[43], 24}, {NULL}};
const Segment SGI11631[] = {{&S58832[30], 5}, {NULL}};
const Segment SGI11632[] = {{&S58832[115], 3}, {NULL}};
const Segment SGI11633[] = {{&S58832[81], 5}, {NULL}};
const Segment SGI11634[] = {{&S58833[44], 13}, {NULL}};
const Segment SGI11635[] = {{&S58833[44], 24}, {NULL}};
const Segment SGI11636[] = {{&S58833[30], 6}, {NULL}};
const Segment SGI11637[] = {{&S58833[116], 3}, {NULL}};
const Segment SGI11638[] = {{&S58833[82], 5}, {NULL}};
const Segment SGI11639[] = {{&S58835[44], 13}, {NULL}};
const Segment SGI11640[] = {{&S58835[44], 25}, {NULL}};
const Segment SGI11641[] = {{&S58835[30], 6}, {NULL}};
const Segment SGI11642[] = {{&S58835[117], 3}, {NULL}};
const Segment SGI11643[] = {{&S58835[83], 5}, {NULL}};
const Segment SGI11644[] = {{&S58836[44], 13}, {NULL}};
const Segment SGI11645[] = {{&S58836[44], 24}, {NULL}};
const Segment SGI11646[] = {{&S58836[30], 6}, {NULL}};
const Segment SGI11647[] = {{&S58836[118], 3}, {NULL}};
const Segment SGI11648[] = {{&S58836[82], 7}, {NULL}};
const Segment SGI11649[] = {{&S58836[138], 8}, {NULL}};
const Segment SGI11650[] = {{&S58837[38], 13}, {NULL}};
const Segment SGI11651[] = {{&S58837[31], 6}, {NULL}};
const Segment SGI11652[] = {{&S59124[119], 3}, {NULL}};
const Segment SGI11653[] = {{&S59124[85], 5}, {NULL}};
const Segment SGI11654[] = {{&S59125[50], 13}, {NULL}};
const Segment SGI11655[] = {{&S59125[30], 12}, {NULL}};
const Segment SGI11656[] = {{&S59125[39], 3}, {NULL}};
const Segment SGI11657[] = {{&S59125[91], 5}, {NULL}};
const Segment SGI11658[] = {{&S59126[55], 8}, {NULL}};
const Segment SGI11659[] = {{&S59126[55], 19}, {NULL}};
const Segment SGI11660[] = {{&S59126[30], 17}, {NULL}};
const Segment SGI11661[] = {{&S59126[122], 3}, {NULL}};
const Segment SGI11662[] = {{&S59126[88], 5}, {NULL}};
const Segment SGI11663[] = {{&S59127[48], 15}, {NULL}};
const Segment SGI11664[] = {{&S59127[30], 10}, {NULL}};
const Segment SGI11665[] = {{&S59127[37], 3}, {NULL}};
const Segment SGI11666[] = {{&S59127[100], 5}, {NULL}};
const Segment SGI11667[] = {{&S59129[54], 10}, {NULL}};
const Segment SGI11668[] = {{&S59129[30], 18}, {NULL}};
const Segment SGI11669[] = {{&S59129[123], 3}, {NULL}};
const Segment SGI11670[] = {{&S59129[89], 5}, {NULL}};
const Segment SGI11671[] = {{&S59131[48], 20}, {NULL}};
const Segment SGI11672[] = {{&S59131[30], 10}, {NULL}};
const Segment SGI11673[] = {{&S59131[37], 3}, {NULL}};
const Segment SGI11674[] = {{&S59131[110], 5}, {NULL}};
const Segment SGI11675[] = {{&S59132[53], 6}, {NULL}};
const Segment SGI11676[] = {{&S59132[53], 27}, {NULL}};
const Segment SGI11677[] = {{&S59132[30], 15}, {NULL}};
const Segment SGI11678[] = {{&S59132[40], 3}, {NULL}};
const Segment SGI11679[] = {{&S59132[94], 5}, {NULL}};
const Segment SGI11680[] = {{&S58954[52], 9}, {NULL}};
const Segment SGI11681[] = {{&S58954[52], 20}, {NULL}};
const Segment SGI11682[] = {{&S58954[38], 6}, {NULL}};
const Segment SGI11683[] = {{&S58954[120], 3}, {NULL}};
const Segment SGI11684[] = {{&S58954[86], 5}, {NULL}};
const Segment SGI11685[] = {{&S58956[41], 9}, {NULL}};
const Segment SGI11686[] = {{&S58956[30], 6}, {NULL}};
const Segment SGI11687[] = {{&S58956[109], 3}, {NULL}};
const Segment SGI11688[] = {{&S58956[75], 5}, {NULL}};
const Segment SGI11689[] = {{&S58106[42], 10}, {NULL}};
const Segment SGI11690[] = {{&S58106[42], 22}, {NULL}};
const Segment SGI11691[] = {{&S58106[30], 4}, {NULL}};
const Segment SGI11692[] = {{&S58106[114], 3}, {NULL}};
const Segment SGI11693[] = {{&S58106[78], 7}, {NULL}};
const Segment SGI11694[] = {{&S58106[78], 6}, {NULL}};
const Segment SGI11695[] = {{&S58107[50], 10}, {NULL}};
const Segment SGI11696[] = {{&S58107[50], 21}, {NULL}};
const Segment SGI11697[] = {{&S58107[30], 12}, {NULL}};
const Segment SGI11698[] = {{&S58107[120], 3}, {NULL}};
const Segment SGI11699[] = {{&S58107[85], 6}, {NULL}};
const Segment SGI11700[] = {{&S58108[44], 10}, {NULL}};
const Segment SGI11701[] = {{&S58108[31], 12}, {NULL}};
const Segment SGI11702[] = {{&S59180[44], 11}, {NULL}};
const Segment SGI11703[] = {{&S59180[44], 22}, {NULL}};
const Segment SGI11704[] = {{&S59180[30], 6}, {NULL}};
const Segment SGI11705[] = {{&S59180[114], 3}, {NULL}};
const Segment SGI11706[] = {{&S59180[80], 5}, {NULL}};
const Segment SGI11707[] = {{&S59181[44], 11}, {NULL}};
const Segment SGI11708[] = {{&S59181[44], 22}, {NULL}};
const Segment SGI11709[] = {{&S59181[30], 6}, {NULL}};
const Segment SGI11710[] = {{&S59181[114], 3}, {NULL}};
const Segment SGI11711[] = {{&S59181[80], 5}, {NULL}};
const Segment SGI11712[] = {{&S59183[38], 11}, {NULL}};
const Segment SGI11713[] = {{&S59183[31], 6}, {NULL}};
const Segment SGI11714[] = {{&S59185[38], 11}, {NULL}};
const Segment SGI11715[] = {{&S59185[31], 6}, {NULL}};
const Segment SGI11716[] = {{&S59244[44], 7}, {NULL}};
const Segment SGI11717[] = {{&S59244[44], 60}, {NULL}};
const Segment SGI11718[] = {{&S59244[30], 6}, {NULL}};
const Segment SGI11719[] = {{&S59244[152], 3}, {NULL}};
const Segment SGI11720[] = {{&S59244[118], 5}, {NULL}};
const Segment SGI11721[] = {{&S58834[42], 13}, {NULL}};
const Segment SGI11722[] = {{&S58834[42], 24}, {NULL}};
const Segment SGI11723[] = {{&S58834[30], 4}, {NULL}};
const Segment SGI11724[] = {{&S58834[114], 3}, {NULL}};
const Segment SGI11725[] = {{&S58834[80], 5}, {NULL}};
const Segment SGI11726[] = {{&S58838[36], 13}, {NULL}};
const Segment SGI11727[] = {{&S58838[31], 4}, {NULL}};
const Segment SGI11728[] = {{&S59469[44], 23}, {NULL}};
const Segment SGI11729[] = {{&S59469[44], 35}, {NULL}};
const Segment SGI11730[] = {{&S59469[30], 6}, {NULL}};
const Segment SGI11731[] = {{&S59469[127], 3}, {NULL}};
const Segment SGI11732[] = {{&S58957[42], 7}, {NULL}};
const Segment SGI11733[] = {{&S58957[42], 18}, {NULL}};
const Segment SGI11734[] = {{&S59469[93], 5}, {NULL}};
const Segment SGI11735[] = {{&S58957[30], 4}, {NULL}};
const Segment SGI11736[] = {{&S58957[107], 3}, {NULL}};
const Segment SGI11737[] = {{&S58957[74], 5}, {NULL}};
const Segment SGI11738[] = {{&S59470[44], 23}, {NULL}};
const Segment SGI11739[] = {{&S59470[44], 35}, {NULL}};
const Segment SGI11740[] = {{&S59470[30], 6}, {NULL}};
const Segment SGI11741[] = {{&S59470[127], 3}, {NULL}};
const Segment SGI11742[] = {{&S59470[93], 5}, {NULL}};
const Segment SGI11743[] = {{&S59471[44], 23}, {NULL}};
const Segment SGI11744[] = {{&S59471[44], 35}, {NULL}};
const Segment SGI11745[] = {{&S59471[30], 6}, {NULL}};
const Segment SGI11746[] = {{&S59471[127], 3}, {NULL}};
const Segment SGI11747[] = {{&S59471[93], 5}, {NULL}};
const Segment SGI11748[] = {{&S59697[38], 7}, {NULL}};
const Segment SGI11749[] = {{&S59697[30], 7}, {NULL}};
const Segment SGI11750[] = {{&S59697[126], 3}, {NULL}};
const Segment SGI11751[] = {{&S59697[91], 6}, {NULL}};
const Segment SGI11752[] = {{&S59698[38], 5}, {NULL}};
const Segment SGI11753[] = {{&S59698[30], 7}, {NULL}};
const Segment SGI11754[] = {{&S59698[124], 3}, {NULL}};
const Segment SGI11755[] = {{&S59698[89], 6}, {NULL}};
const Segment SGI11756[] = {{&S59699[38], 7}, {NULL}};
const Segment SGI11757[] = {{&S59699[30], 7}, {NULL}};
const Segment SGI11758[] = {{&S59699[126], 3}, {NULL}};
const Segment SGI11759[] = {{&S59699[91], 6}, {NULL}};
const Segment SGI11760[] = {{&S59700[38], 7}, {NULL}};
const Segment SGI11761[] = {{&S59700[30], 7}, {NULL}};
const Segment SGI11762[] = {{&S59700[126], 3}, {NULL}};
const Segment SGI11763[] = {{&S59700[91], 6}, {NULL}};
const Segment SGI11764[] = {{&S59701[38], 7}, {NULL}};
const Segment SGI11765[] = {{&S59701[30], 7}, {NULL}};
const Segment SGI11766[] = {{&S59701[126], 3}, {NULL}};
const Segment SGI11767[] = {{&S59701[91], 6}, {NULL}};
const Segment SGI11768[] = {{&S59702[38], 7}, {NULL}};
const Segment SGI11769[] = {{&S59702[30], 7}, {NULL}};
const Segment SGI11770[] = {{&S59702[126], 3}, {NULL}};
const Segment SGI11771[] = {{&S59702[91], 6}, {NULL}};
const Segment SGI11772[] = {{&S59703[38], 7}, {NULL}};
const Segment SGI11773[] = {{&S59703[30], 7}, {NULL}};
const Segment SGI11774[] = {{&S59703[126], 3}, {NULL}};
const Segment SGI11775[] = {{&S59703[91], 6}, {NULL}};
const Segment SGI11776[] = {{&S59704[38], 7}, {NULL}};
const Segment SGI11777[] = {{&S59704[30], 7}, {NULL}};
const Segment SGI11778[] = {{&S59704[126], 3}, {NULL}};
const Segment SGI11779[] = {{&S59704[91], 6}, {NULL}};
const Segment SGI11780[] = {{&S59705[38], 7}, {NULL}};
const Segment SGI11781[] = {{&S59705[30], 7}, {NULL}};
const Segment SGI11782[] = {{&S59705[126], 3}, {NULL}};
const Segment SGI11783[] = {{&S59705[91], 6}, {NULL}};
const Segment SGI11784[] = {{&S59706[38], 7}, {NULL}};
const Segment SGI11785[] = {{&S59706[30], 7}, {NULL}};
const Segment SGI11786[] = {{&S59706[126], 3}, {NULL}};
const Segment SGI11787[] = {{&S59706[91], 6}, {NULL}};
const Segment SGI11788[] = {{&S59707[38], 7}, {NULL}};
const Segment SGI11789[] = {{&S59707[30], 7}, {NULL}};
const Segment SGI11790[] = {{&S59707[126], 3}, {NULL}};
const Segment SGI11791[] = {{&S59707[91], 6}, {NULL}};
const Segment SGI11792[] = {{&S59708[38], 7}, {NULL}};
const Segment SGI11793[] = {{&S59708[30], 7}, {NULL}};
const Segment SGI11794[] = {{&S59708[126], 3}, {NULL}};
const Segment SGI11795[] = {{&S59708[91], 6}, {NULL}};
const Segment SGI11796[] = {{&S59709[23], 16}, {NULL}};
const Segment SGI11797[] = {{&S59709[31], 40}, {NULL}};
const Segment SGI11798[] = {{&S59710[38], 7}, {NULL}};
const Segment SGI11799[] = {{&S59710[30], 7}, {NULL}};
const Segment SGI11800[] = {{&S59710[126], 3}, {NULL}};
const Segment SGI11801[] = {{&S59710[91], 6}, {NULL}};
const Segment SGI11802[] = {{&S64825[0], 44}, {NULL}};
const Segment SGI11803[] = {{&S59781[3], 8}, {NULL}};
const Segment SGI11804[] = {{&S59783[71], 5}, {NULL}};
const Segment SGI11805[] = {{&S59182[42], 11}, {NULL}};
const Segment SGI11806[] = {{&S59604[44], 13}, {NULL}};
const Segment SGI11807[] = {{&S59182[42], 22}, {NULL}};
const Segment SGI11808[] = {{&S59604[44], 24}, {NULL}};
const Segment SGI11809[] = {{&S59182[30], 4}, {NULL}};
const Segment SGI11810[] = {{&S59604[30], 6}, {NULL}};
const Segment SGI11811[] = {{&S59182[112], 3}, {NULL}};
const Segment SGI11812[] = {{&S59604[116], 3}, {NULL}};
const Segment SGI11813[] = {{&S59182[78], 5}, {NULL}};
const Segment SGI11814[] = {{&S59604[82], 5}, {NULL}};
const Segment SGI11815[] = {{&S59184[42], 11}, {NULL}};
const Segment SGI11816[] = {{&S59184[42], 22}, {NULL}};
const Segment SGI11817[] = {{&S59184[30], 4}, {NULL}};
const Segment SGI11818[] = {{&S59184[112], 3}, {NULL}};
const Segment SGI11819[] = {{&S59184[78], 5}, {NULL}};
const Segment SGI11820[] = {{&S58683[42], 9}, {NULL}};
const Segment SGI11821[] = {{&S58683[42], 20}, {NULL}};
const Segment SGI11822[] = {{&S58683[30], 4}, {NULL}};
const Segment SGI11823[] = {{&S58683[111], 3}, {NULL}};
const Segment SGI11824[] = {{&S58683[76], 6}, {NULL}};
const Segment SGI11825[] = {{&S58684[42], 18}, {NULL}};
const Segment SGI11826[] = {{&S58684[42], 29}, {NULL}};
const Segment SGI11827[] = {{&S58684[30], 4}, {NULL}};
const Segment SGI11828[] = {{&S58684[120], 3}, {NULL}};
const Segment SGI11829[] = {{&S58684[85], 6}, {NULL}};
const Segment SGI11830[] = {{&S58685[50], 9}, {NULL}};
const Segment SGI11831[] = {{&S58685[50], 20}, {NULL}};
const Segment SGI11832[] = {{&S58685[30], 12}, {NULL}};
const Segment SGI11833[] = {{&S58685[119], 3}, {NULL}};
const Segment SGI11834[] = {{&S58685[84], 6}, {NULL}};
const Segment SGI11835[] = {{&S58686[50], 18}, {NULL}};
const Segment SGI11836[] = {{&S58686[50], 29}, {NULL}};
const Segment SGI11837[] = {{&S58686[30], 12}, {NULL}};
const Segment SGI11838[] = {{&S58686[128], 3}, {NULL}};
const Segment SGI11839[] = {{&S58686[93], 6}, {NULL}};
const Segment SGI11840[] = {{&S58687[50], 18}, {NULL}};
const Segment SGI11841[] = {{&S58687[50], 29}, {NULL}};
const Segment SGI11842[] = {{&S58687[30], 12}, {NULL}};
const Segment SGI11843[] = {{&S58687[93], 6}, {NULL}};
const Segment SGI11844[] = {{&S58688[42], 9}, {NULL}};
const Segment SGI11845[] = {{&S58688[42], 20}, {NULL}};
const Segment SGI11846[] = {{&S58688[30], 4}, {NULL}};
const Segment SGI11847[] = {{&S58688[112], 3}, {NULL}};
const Segment SGI11848[] = {{&S58688[76], 7}, {NULL}};
const Segment SGI11849[] = {{&S58688[132], 8}, {NULL}};
const Segment SGI11850[] = {{&S58689[42], 9}, {NULL}};
const Segment SGI11851[] = {{&S58689[42], 20}, {NULL}};
const Segment SGI11852[] = {{&S58689[30], 4}, {NULL}};
const Segment SGI11853[] = {{&S58689[111], 3}, {NULL}};
const Segment SGI11854[] = {{&S58689[76], 6}, {NULL}};
const Segment SGI11855[] = {{&S59605[44], 10}, {NULL}};
const Segment SGI11856[] = {{&S59605[44], 21}, {NULL}};
const Segment SGI11857[] = {{&S59605[30], 6}, {NULL}};
const Segment SGI11858[] = {{&S59605[113], 3}, {NULL}};
const Segment SGI11859[] = {{&S59605[79], 5}, {NULL}};
const Segment SGI11860[] = {{&S59606[38], 10}, {NULL}};
const Segment SGI11861[] = {{&S59606[31], 6}, {NULL}};
const Segment SGI11862[] = {{&S59858[44], 10}, {NULL}};
const Segment SGI11863[] = {{&S59858[44], 21}, {NULL}};
const Segment SGI11864[] = {{&S59858[30], 6}, {NULL}};
const Segment SGI11865[] = {{&S59858[113], 3}, {NULL}};
const Segment SGI11866[] = {{&S59858[79], 5}, {NULL}};
const Segment SGI11867[] = {{&S59859[44], 10}, {NULL}};
const Segment SGI11868[] = {{&S59859[44], 21}, {NULL}};
const Segment SGI11869[] = {{&S59859[30], 6}, {NULL}};
const Segment SGI11870[] = {{&S59859[113], 3}, {NULL}};
const Segment SGI11871[] = {{&S59859[79], 5}, {NULL}};
const Segment SGI11872[] = {{&S59860[38], 10}, {NULL}};
const Segment SGI11873[] = {{&S59860[31], 6}, {NULL}};
const Segment SGI11874[] = {{&S59862[44], 10}, {NULL}};
const Segment SGI11875[] = {{&S59862[44], 21}, {NULL}};
const Segment SGI11876[] = {{&S59862[30], 6}, {NULL}};
const Segment SGI11877[] = {{&S59862[82], 4}, {NULL}};
const Segment SGI11878[] = {{&S59862[82], 3}, {NULL}};
const Segment SGI11879[] = {{&S59863[44], 10}, {NULL}};
const Segment SGI11880[] = {{&S59863[44], 21}, {NULL}};
const Segment SGI11881[] = {{&S59863[30], 6}, {NULL}};
const Segment SGI11882[] = {{&S59863[82], 6}, {NULL}};
const Segment SGI11883[] = {{&S59931[37], 16}, {NULL}};
const Segment SGI11884[] = {{&S59931[30], 6}, {NULL}};
const Segment SGI11885[] = {{&S59931[116], 3}, {NULL}};
const Segment SGI11886[] = {{&S59931[81], 6}, {NULL}};
const Segment SGI11887[] = {{&S59931[134], 6}, {NULL}};
const Segment SGI11888[] = {{&S59010[56], 8}, {NULL}};
const Segment SGI11889[] = {{&S59010[56], 20}, {NULL}};
const Segment SGI11890[] = {{&S59010[30], 18}, {NULL}};
const Segment SGI11891[] = {{&S59010[125], 3}, {NULL}};
const Segment SGI11892[] = {{&S59010[90], 6}, {NULL}};
const Segment SGI11893[] = {{&S59011[56], 8}, {NULL}};
const Segment SGI11894[] = {{&S59011[56], 20}, {NULL}};
const Segment SGI11895[] = {{&S59011[30], 18}, {NULL}};
const Segment SGI11896[] = {{&S59011[125], 3}, {NULL}};
const Segment SGI11897[] = {{&S59011[90], 6}, {NULL}};
const Segment SGI11898[] = {{&S59012[56], 8}, {NULL}};
const Segment SGI11899[] = {{&S59012[56], 20}, {NULL}};
const Segment SGI11900[] = {{&S59012[30], 18}, {NULL}};
const Segment SGI11901[] = {{&S59012[125], 3}, {NULL}};
const Segment SGI11902[] = {{&S59012[90], 6}, {NULL}};
const Segment SGI11903[] = {{&S59013[57], 8}, {NULL}};
const Segment SGI11904[] = {{&S59013[57], 20}, {NULL}};
const Segment SGI11905[] = {{&S59013[30], 19}, {NULL}};
const Segment SGI11906[] = {{&S59013[126], 3}, {NULL}};
const Segment SGI11907[] = {{&S59013[91], 6}, {NULL}};
const Segment SGI11908[] = {{&S59186[50], 9}, {NULL}};
const Segment SGI11909[] = {{&S59186[50], 20}, {NULL}};
const Segment SGI11910[] = {{&S59186[30], 12}, {NULL}};
const Segment SGI11911[] = {{&S59186[119], 3}, {NULL}};
const Segment SGI11912[] = {{&S59186[84], 6}, {NULL}};
const Segment SGI11913[] = {{&S59205[46], 9}, {NULL}};
const Segment SGI11914[] = {{&S59205[26], 12}, {NULL}};
const Segment SGI11915[] = {{&S59205[92], 3}, {NULL}};
const Segment SGI11916[] = {{&S59205[69], 6}, {NULL}};
const Segment SGI11917[] = {{&S59206[50], 13}, {NULL}};
const Segment SGI11918[] = {{&S59206[50], 24}, {NULL}};
const Segment SGI11919[] = {{&S59206[30], 12}, {NULL}};
const Segment SGI11920[] = {{&S59206[123], 3}, {NULL}};
const Segment SGI11921[] = {{&S59206[88], 6}, {NULL}};
const Segment SGI11922[] = {{&S59207[46], 6}, {NULL}};
const Segment SGI11923[] = {{&S59207[26], 12}, {NULL}};
const Segment SGI11924[] = {{&S59207[89], 3}, {NULL}};
const Segment SGI11925[] = {{&S59207[66], 6}, {NULL}};
const Segment SGI11926[] = {{&S60155[44], 10}, {NULL}};
const Segment SGI11927[] = {{&S60155[44], 21}, {NULL}};
const Segment SGI11928[] = {{&S60155[30], 6}, {NULL}};
const Segment SGI11929[] = {{&S60155[113], 3}, {NULL}};
const Segment SGI11930[] = {{&S59208[30], 12}, {NULL}};
const Segment SGI11931[] = {{&S60155[79], 5}, {NULL}};
const Segment SGI11932[] = {{&S59208[97], 3}, {NULL}};
const Segment SGI11933[] = {{&S59208[63], 5}, {NULL}};
const Segment SGI11934[] = {{&S59208[108], 11}, {NULL}};
const Segment SGI11935[] = {{&S59209[50], 6}, {NULL}};
const Segment SGI11936[] = {{&S59209[30], 12}, {NULL}};
const Segment SGI11937[] = {{&S59209[126], 3}, {NULL}};
const Segment SGI11938[] = {{&S59209[91], 6}, {NULL}};
const Segment SGI11939[] = {{&S59210[50], 51}, {NULL}};
const Segment SGI11940[] = {{&S59210[30], 12}, {&S59210[58], 43}, {NULL}};
const Segment SGI11941[] = {{&S59210[164], 3}, {NULL}};
const Segment SGI11942[] = {{&S59210[129], 6}, {NULL}};
const Segment SGI11943[] = {{&S59211[50], 35}, {NULL}};
const Segment SGI11944[] = {{&S59211[50], 47}, {NULL}};
const Segment SGI11945[] = {{&S59211[30], 12}, {&S59211[58], 39}, {NULL}};
const Segment SGI11946[] = {{&S59211[146], 3}, {NULL}};
const Segment SGI11947[] = {{&S59211[111], 6}, {NULL}};
const Segment SGI11948[] = {{&S59212[50], 48}, {NULL}};
const Segment SGI11949[] = {{&S59212[30], 12}, {&S59212[58], 40}, {NULL}};
const Segment SGI11950[] = {{&S59212[161], 3}, {NULL}};
const Segment SGI11951[] = {{&S59212[126], 6}, {NULL}};
const Segment SGI11952[] = {{&S59213[50], 63}, {NULL}};
const Segment SGI11953[] = {{&S59213[30], 12}, {&S59213[58], 55}, {NULL}};
const Segment SGI11954[] = {{&S59213[176], 3}, {NULL}};
const Segment SGI11955[] = {{&S59213[141], 6}, {NULL}};
const Segment SGI11956[] = {{&S59214[50], 44}, {NULL}};
const Segment SGI11957[] = {{&S59214[30], 12}, {&S59214[58], 36}, {NULL}};
const Segment SGI11958[] = {{&S59214[157], 3}, {NULL}};
const Segment SGI11959[] = {{&S59214[122], 6}, {NULL}};
const Segment SGI11960[] = {{&S59216[50], 5}, {NULL}};
const Segment SGI11961[] = {{&S59216[50], 17}, {NULL}};
const Segment SGI11962[] = {{&S59216[30], 12}, {NULL}};
const Segment SGI11963[] = {{&S59216[116], 3}, {NULL}};
const Segment SGI11964[] = {{&S59216[81], 6}, {NULL}};
const Segment SGI11965[] = {{&S59217[50], 13}, {NULL}};
const Segment SGI11966[] = {{&S59217[50], 25}, {NULL}};
const Segment SGI11967[] = {{&S59217[30], 12}, {&S59217[58], 17}, {NULL}};
const Segment SGI11968[] = {{&S59217[124], 3}, {NULL}};
const Segment SGI11969[] = {{&S59217[89], 6}, {NULL}};
const Segment SGI11970[] = {{&S59218[37], 7}, {NULL}};
const Segment SGI11971[] = {{&S59218[30], 6}, {NULL}};
const Segment SGI11972[] = {{&S59218[105], 3}, {NULL}};
const Segment SGI11973[] = {{&S59218[70], 6}, {NULL}};
const Segment SGI11974[] = {{&S59219[37], 7}, {NULL}};
const Segment SGI11975[] = {{&S59219[30], 6}, {NULL}};
const Segment SGI11976[] = {{&S59219[105], 3}, {NULL}};
const Segment SGI11977[] = {{&S59219[70], 6}, {NULL}};
const Segment SGI11978[] = {{&S59220[50], 9}, {NULL}};
const Segment SGI11979[] = {{&S59220[50], 21}, {NULL}};
const Segment SGI11980[] = {{&S59220[30], 12}, {NULL}};
const Segment SGI11981[] = {{&S59220[88], 6}, {NULL}};
const Segment SGI11982[] = {{&S59221[56], 5}, {NULL}};
const Segment SGI11983[] = {{&S59221[56], 17}, {NULL}};
const Segment SGI11984[] = {{&S59221[30], 18}, {NULL}};
const Segment SGI11985[] = {{&S59221[122], 3}, {NULL}};
const Segment SGI11986[] = {{&S59221[87], 6}, {NULL}};
const Segment SGI11987[] = {{&S59222[50], 9}, {NULL}};
const Segment SGI11988[] = {{&S59222[50], 21}, {NULL}};
const Segment SGI11989[] = {{&S59222[30], 12}, {NULL}};
const Segment SGI11990[] = {{&S59222[120], 3}, {NULL}};
const Segment SGI11991[] = {{&S59222[85], 6}, {NULL}};
const Segment SGI11992[] = {{&S59223[30], 4}, {NULL}};
const Segment SGI11993[] = {{&S59224[43], 7}, {NULL}};
const Segment SGI11994[] = {{&S59224[30], 12}, {NULL}};
const Segment SGI11995[] = {{&S59224[110], 3}, {NULL}};
const Segment SGI11996[] = {{&S59224[75], 6}, {NULL}};
const Segment SGI11997[] = {{&S59225[50], 15}, {NULL}};
const Segment SGI11998[] = {{&S59225[50], 26}, {NULL}};
const Segment SGI11999[] = {{&S59225[30], 12}, {NULL}};
const Segment SGI12000[] = {{&S59225[125], 3}, {NULL}};
const Segment SGI12001[] = {{&S59225[90], 6}, {NULL}};
const Segment SGI12002[] = {{&S59226[56], 10}, {NULL}};
const Segment SGI12003[] = {{&S59226[56], 22}, {NULL}};
const Segment SGI12004[] = {{&S59226[30], 18}, {NULL}};
const Segment SGI12005[] = {{&S59226[127], 3}, {NULL}};
const Segment SGI12006[] = {{&S59226[92], 6}, {NULL}};
const Segment SGI12007[] = {{&S59451[50], 11}, {NULL}};
const Segment SGI12008[] = {{&S59451[50], 23}, {NULL}};
const Segment SGI12009[] = {{&S59451[30], 12}, {NULL}};
const Segment SGI12010[] = {{&S59451[122], 3}, {NULL}};
const Segment SGI12011[] = {{&S59451[87], 6}, {NULL}};
const Segment SGI12012[] = {{&S59452[44], 11}, {NULL}};
const Segment SGI12013[] = {{&S59452[31], 12}, {NULL}};
const Segment SGI12014[] = {{&S65443[8], 6}, {NULL}};
const Segment SGI12015[] = {{&S65443[0], 84}, {NULL}};
const Segment SGI12016[] = {{&S65444[8], 7}, {NULL}};
const Segment SGI12017[] = {{&S65444[0], 85}, {NULL}};
const Segment SGI12018[] = {{&S65445[8], 7}, {&S65445[8], 7}, {NULL}};
const Segment SGI12019[] = {{&S65445[0], 172}, {NULL}};
const Segment SGI12020[] = {{&S65446[8], 6}, {&S65446[8], 6}, {NULL}};
const Segment SGI12021[] = {{&S65446[0], 170}, {NULL}};
const Segment SGI12022[] = {{&S65447[8], 6}, {NULL}};
const Segment SGI12023[] = {{&S65447[0], 97}, {NULL}};
const Segment SGI12024[] = {{&S59968[42], 4}, {NULL}};
const Segment SGI12025[] = {{&S59968[42], 15}, {NULL}};
const Segment SGI12026[] = {{&S59968[30], 4}, {NULL}};
const Segment SGI12027[] = {{&S59968[105], 3}, {NULL}};
const Segment SGI12028[] = {{&S59968[71], 5}, {NULL}};
const Segment SGI12029[] = {{&S59970[36], 4}, {NULL}};
const Segment SGI12030[] = {{&S59970[31], 4}, {NULL}};
const Segment SGI12031[] = {{&S60704[6], 4}, {NULL}};
const Segment SGI12032[] = {{&S60704[32], 3}, {NULL}};
const Segment SGI12033[] = {{&S60704[81], 5}, {NULL}};
const Segment SGI12034[] = {{&S63158[42], 9}, {NULL}};
const Segment SGI12035[] = {{&S63158[42], 22}, {NULL}};
const Segment SGI12036[] = {{&S63158[30], 4}, {NULL}};
const Segment SGI12037[] = {{&S63158[113], 3}, {NULL}};
const Segment SGI12038[] = {{&S63158[78], 6}, {NULL}};
const Segment SGI12039[] = {{&S63158[133], 8}, {NULL}};
const Segment SGI12040[] = {{&S63159[42], 8}, {NULL}};
const Segment SGI12041[] = {{&S63159[42], 21}, {NULL}};
const Segment SGI12042[] = {{&S63159[30], 4}, {NULL}};
const Segment SGI12043[] = {{&S63159[112], 3}, {NULL}};
const Segment SGI12044[] = {{&S63159[77], 6}, {NULL}};
const Segment SGI12045[] = {{&S63159[132], 8}, {NULL}};
const Segment SGI12046[] = {{&S60908[71], 5}, {NULL}};
const Segment SGI12047[] = {{&S60909[8], 8}, {NULL}};
const Segment SGI12048[] = {{&S59807[50], 7}, {NULL}};
const Segment SGI12049[] = {{&S59807[50], 59}, {NULL}};
const Segment SGI12050[] = {{&S59807[30], 12}, {NULL}};
const Segment SGI12051[] = {{&S59807[158], 3}, {NULL}};
const Segment SGI12052[] = {{&S59807[123], 6}, {NULL}};
const Segment SGI12053[] = {{&S59808[50], 7}, {NULL}};
const Segment SGI12054[] = {{&S59808[50], 59}, {NULL}};
const Segment SGI12055[] = {{&S59808[30], 12}, {NULL}};
const Segment SGI12056[] = {{&S59808[158], 3}, {NULL}};
const Segment SGI12057[] = {{&S59808[123], 6}, {NULL}};
const Segment SGI12058[] = {{&S59809[50], 7}, {NULL}};
const Segment SGI12059[] = {{&S59809[50], 59}, {NULL}};
const Segment SGI12060[] = {{&S59809[30], 12}, {NULL}};
const Segment SGI12061[] = {{&S59809[158], 3}, {NULL}};
const Segment SGI12062[] = {{&S59809[123], 6}, {NULL}};
const Segment SGI12063[] = {{&S59810[50], 7}, {NULL}};
const Segment SGI12064[] = {{&S59810[50], 27}, {NULL}};
const Segment SGI12065[] = {{&S59810[30], 12}, {NULL}};
const Segment SGI12066[] = {{&S59810[126], 3}, {NULL}};
const Segment SGI12067[] = {{&S59810[91], 6}, {NULL}};
const Segment SGI12068[] = {{&S59811[50], 7}, {NULL}};
const Segment SGI12069[] = {{&S59811[50], 59}, {NULL}};
const Segment SGI12070[] = {{&S59811[30], 12}, {NULL}};
const Segment SGI12071[] = {{&S59811[158], 3}, {NULL}};
const Segment SGI12072[] = {{&S59811[123], 6}, {NULL}};
const Segment SGI12073[] = {{&S59812[50], 7}, {NULL}};
const Segment SGI12074[] = {{&S59812[50], 59}, {NULL}};
const Segment SGI12075[] = {{&S59812[30], 12}, {NULL}};
const Segment SGI12076[] = {{&S59812[158], 3}, {NULL}};
const Segment SGI12077[] = {{&S59812[123], 6}, {NULL}};
const Segment SGI12078[] = {{&S59813[50], 7}, {NULL}};
const Segment SGI12079[] = {{&S59813[50], 28}, {NULL}};
const Segment SGI12080[] = {{&S59813[30], 12}, {NULL}};
const Segment SGI12081[] = {{&S59813[128], 3}, {NULL}};
const Segment SGI12082[] = {{&S59813[92], 7}, {NULL}};
const Segment SGI12083[] = {{&S59813[148], 8}, {NULL}};
const Segment SGI12084[] = {{&S59814[50], 7}, {NULL}};
const Segment SGI12085[] = {{&S59814[50], 27}, {NULL}};
const Segment SGI12086[] = {{&S59814[30], 12}, {NULL}};
const Segment SGI12087[] = {{&S59814[126], 3}, {NULL}};
const Segment SGI12088[] = {{&S59814[91], 6}, {NULL}};
const Segment SGI12089[] = {{&S61073[44], 9}, {NULL}};
const Segment SGI12090[] = {{&S61073[44], 20}, {NULL}};
const Segment SGI12091[] = {{&S61073[30], 6}, {NULL}};
const Segment SGI12092[] = {{&S61073[112], 3}, {NULL}};
const Segment SGI12093[] = {{&S61073[78], 5}, {NULL}};
const Segment SGI12094[] = {{&S61075[38], 9}, {NULL}};
const Segment SGI12095[] = {{&S61075[31], 6}, {NULL}};
const Segment SGI12096[] = {{&S61077[44], 9}, {NULL}};
const Segment SGI12097[] = {{&S61077[44], 20}, {NULL}};
const Segment SGI12098[] = {{&S61077[30], 6}, {NULL}};
const Segment SGI12099[] = {{&S61077[112], 3}, {NULL}};
const Segment SGI12100[] = {{&S61077[78], 5}, {NULL}};
const Segment SGI12101[] = {{&S66029[8], 7}, {NULL}};
const Segment SGI12102[] = {{&S66029[0], 85}, {NULL}};
const Segment SGI12103[] = {{&S66030[8], 7}, {NULL}};
const Segment SGI12104[] = {{&S66030[0], 85}, {NULL}};
const Segment SGI12105[] = {{&S66031[8], 6}, {NULL}};
const Segment SGI12106[] = {{&S66031[0], 84}, {NULL}};
const Segment SGI12107[] = {{&S63804[44], 6}, {NULL}};
const Segment SGI12108[] = {{&S63804[44], 18}, {NULL}};
const Segment SGI12109[] = {{&S63804[30], 6}, {NULL}};
const Segment SGI12110[] = {{&S63804[111], 3}, {NULL}};
const Segment SGI12111[] = {{&S63804[76], 6}, {NULL}};
const Segment SGI12112[] = {{&S63805[44], 6}, {NULL}};
const Segment SGI12113[] = {{&S63805[44], 18}, {NULL}};
const Segment SGI12114[] = {{&S63805[30], 6}, {NULL}};
const Segment SGI12115[] = {{&S63805[111], 3}, {NULL}};
const Segment SGI12116[] = {{&S63805[76], 6}, {NULL}};
const Segment SGI12117[] = {{&S63806[44], 6}, {NULL}};
const Segment SGI12118[] = {{&S63806[44], 18}, {NULL}};
const Segment SGI12119[] = {{&S63806[30], 6}, {NULL}};
const Segment SGI12120[] = {{&S63806[111], 3}, {NULL}};
const Segment SGI12121[] = {{&S63806[76], 6}, {NULL}};
const Segment SGI12122[] = {{&S63807[44], 6}, {NULL}};
const Segment SGI12123[] = {{&S63807[44], 31}, {NULL}};
const Segment SGI12124[] = {{&S63807[30], 6}, {NULL}};
const Segment SGI12125[] = {{&S63807[124], 3}, {NULL}};
const Segment SGI12126[] = {{&S63807[89], 6}, {NULL}};
const Segment SGI12127[] = {{&S60238[42], 7}, {NULL}};
const Segment SGI12128[] = {{&S60238[42], 19}, {NULL}};
const Segment SGI12129[] = {{&S60238[30], 4}, {NULL}};
const Segment SGI12130[] = {{&S60238[109], 3}, {NULL}};
const Segment SGI12131[] = {{&S60238[75], 5}, {NULL}};
const Segment SGI12132[] = {{&S60239[50], 7}, {NULL}};
const Segment SGI12133[] = {{&S60239[50], 18}, {NULL}};
const Segment SGI12134[] = {{&S60239[30], 12}, {NULL}};
const Segment SGI12135[] = {{&S60239[117], 3}, {NULL}};
const Segment SGI12136[] = {{&S60239[82], 6}, {NULL}};
const Segment SGI12137[] = {{&S60240[42], 7}, {NULL}};
const Segment SGI12138[] = {{&S60240[42], 18}, {NULL}};
const Segment SGI12139[] = {{&S60240[30], 4}, {NULL}};
const Segment SGI12140[] = {{&S60240[109], 3}, {NULL}};
const Segment SGI12141[] = {{&S60240[74], 6}, {NULL}};
const Segment SGI12142[] = {{&S60241[42], 7}, {NULL}};
const Segment SGI12143[] = {{&S60241[42], 18}, {NULL}};
const Segment SGI12144[] = {{&S60241[30], 4}, {NULL}};
const Segment SGI12145[] = {{&S60241[109], 3}, {NULL}};
const Segment SGI12146[] = {{&S60241[74], 6}, {NULL}};
const Segment SGI12147[] = {{&S60242[51], 7}, {NULL}};
const Segment SGI12148[] = {{&S60242[51], 18}, {NULL}};
const Segment SGI12149[] = {{&S60242[30], 12}, {NULL}};
const Segment SGI12150[] = {{&S60242[118], 3}, {NULL}};
const Segment SGI12151[] = {{&S60242[83], 6}, {NULL}};
const Segment SGI12152[] = {{&S60243[50], 7}, {NULL}};
const Segment SGI12153[] = {{&S60243[50], 18}, {NULL}};
const Segment SGI12154[] = {{&S60243[30], 12}, {NULL}};
const Segment SGI12155[] = {{&S60243[117], 3}, {NULL}};
const Segment SGI12156[] = {{&S60243[82], 6}, {NULL}};
const Segment SGI12157[] = {{&S60244[42], 7}, {NULL}};
const Segment SGI12158[] = {{&S60244[42], 18}, {NULL}};
const Segment SGI12159[] = {{&S60244[30], 4}, {NULL}};
const Segment SGI12160[] = {{&S60244[109], 3}, {NULL}};
const Segment SGI12161[] = {{&S60244[74], 6}, {NULL}};
const Segment SGI12162[] = {{&S60245[50], 11}, {NULL}};
const Segment SGI12163[] = {{&S60245[50], 22}, {NULL}};
const Segment SGI12164[] = {{&S60245[30], 12}, {NULL}};
const Segment SGI12165[] = {{&S60245[121], 3}, {NULL}};
const Segment SGI12166[] = {{&S60245[86], 6}, {NULL}};
const Segment SGI12167[] = {{&S61309[44], 6}, {NULL}};
const Segment SGI12168[] = {{&S61309[44], 17}, {NULL}};
const Segment SGI12169[] = {{&S61309[30], 6}, {NULL}};
const Segment SGI12170[] = {{&S61309[109], 3}, {NULL}};
const Segment SGI12171[] = {{&S61309[75], 5}, {NULL}};
const Segment SGI12172[] = {{&S61310[38], 6}, {NULL}};
const Segment SGI12173[] = {{&S61310[31], 6}, {NULL}};
const Segment SGI12174[] = {{&S60493[50], 16}, {NULL}};
const Segment SGI12175[] = {{&S60493[50], 25}, {NULL}};
const Segment SGI12176[] = {{&S60493[30], 12}, {NULL}};
const Segment SGI12177[] = {{&S60493[124], 3}, {NULL}};
const Segment SGI12178[] = {{&S60493[89], 6}, {NULL}};
const Segment SGI12179[] = {{&S60494[50], 16}, {NULL}};
const Segment SGI12180[] = {{&S60494[50], 27}, {NULL}};
const Segment SGI12181[] = {{&S60494[30], 12}, {NULL}};
const Segment SGI12182[] = {{&S60494[126], 3}, {NULL}};
const Segment SGI12183[] = {{&S60494[91], 6}, {NULL}};
const Segment SGI12184[] = {{&S60495[50], 17}, {NULL}};
const Segment SGI12185[] = {{&S60495[50], 28}, {NULL}};
const Segment SGI12186[] = {{&S60495[30], 12}, {NULL}};
const Segment SGI12187[] = {{&S60495[127], 3}, {NULL}};
const Segment SGI12188[] = {{&S60495[92], 6}, {NULL}};
const Segment SGI12189[] = {{&S61506[44], 7}, {NULL}};
const Segment SGI12190[] = {{&S61506[44], 29}, {NULL}};
const Segment SGI12191[] = {{&S61506[30], 6}, {NULL}};
const Segment SGI12192[] = {{&S61506[121], 3}, {NULL}};
const Segment SGI12193[] = {{&S61506[87], 5}, {NULL}};
const Segment SGI12194[] = {{&S61507[44], 7}, {NULL}};
const Segment SGI12195[] = {{&S61507[44], 30}, {NULL}};
const Segment SGI12196[] = {{&S61507[30], 6}, {NULL}};
const Segment SGI12197[] = {{&S61507[122], 3}, {NULL}};
const Segment SGI12198[] = {{&S61507[88], 5}, {NULL}};
const Segment SGI12199[] = {{&S61508[44], 17}, {NULL}};
const Segment SGI12200[] = {{&S61508[44], 33}, {NULL}};
const Segment SGI12201[] = {{&S61508[30], 6}, {NULL}};
const Segment SGI12202[] = {{&S61508[125], 3}, {NULL}};
const Segment SGI12203[] = {{&S61508[91], 5}, {NULL}};
const Segment SGI12204[] = {{&S61509[38], 7}, {NULL}};
const Segment SGI12205[] = {{&S61509[31], 6}, {NULL}};
const Segment SGI12206[] = {{&S60726[50], 7}, {NULL}};
const Segment SGI12207[] = {{&S60726[50], 18}, {NULL}};
const Segment SGI12208[] = {{&S60726[30], 12}, {NULL}};
const Segment SGI12209[] = {{&S60726[117], 3}, {NULL}};
const Segment SGI12210[] = {{&S60726[82], 6}, {NULL}};
const Segment SGI12211[] = {{&S60727[50], 15}, {NULL}};
const Segment SGI12212[] = {{&S60727[50], 26}, {NULL}};
const Segment SGI12213[] = {{&S60727[30], 12}, {NULL}};
const Segment SGI12214[] = {{&S60727[125], 3}, {NULL}};
const Segment SGI12215[] = {{&S60727[90], 6}, {NULL}};
const Segment SGI12216[] = {{&S60728[50], 7}, {NULL}};
const Segment SGI12217[] = {{&S60728[50], 18}, {NULL}};
const Segment SGI12218[] = {{&S60728[30], 12}, {NULL}};
const Segment SGI12219[] = {{&S60728[117], 3}, {NULL}};
const Segment SGI12220[] = {{&S60728[82], 6}, {NULL}};
const Segment SGI12221[] = {{&S60729[44], 7}, {NULL}};
const Segment SGI12222[] = {{&S60729[31], 12}, {NULL}};
const Segment SGI12223[] = {{&S61637[44], 10}, {NULL}};
const Segment SGI12224[] = {{&S61637[44], 21}, {NULL}};
const Segment SGI12225[] = {{&S61637[30], 6}, {NULL}};
const Segment SGI12226[] = {{&S61637[113], 3}, {NULL}};
const Segment SGI12227[] = {{&S61637[79], 5}, {NULL}};
const Segment SGI12228[] = {{&S61638[44], 10}, {NULL}};
const Segment SGI12229[] = {{&S61638[44], 21}, {NULL}};
const Segment SGI12230[] = {{&S61638[30], 6}, {NULL}};
const Segment SGI12231[] = {{&S61638[82], 4}, {NULL}};
const Segment SGI12232[] = {{&S61638[82], 3}, {NULL}};
const Segment SGI12233[] = {{&S61640[44], 10}, {NULL}};
const Segment SGI12234[] = {{&S61640[44], 21}, {NULL}};
const Segment SGI12235[] = {{&S61640[30], 6}, {NULL}};
const Segment SGI12236[] = {{&S61640[113], 3}, {NULL}};
const Segment SGI12237[] = {{&S61640[79], 5}, {NULL}};
const Segment SGI12238[] = {{&S61642[44], 10}, {NULL}};
const Segment SGI12239[] = {{&S61642[44], 21}, {NULL}};
const Segment SGI12240[] = {{&S61642[30], 6}, {NULL}};
const Segment SGI12241[] = {{&S61642[82], 6}, {NULL}};
const Segment SGI12242[] = {{&S61645[44], 10}, {NULL}};
const Segment SGI12243[] = {{&S61645[44], 21}, {NULL}};
const Segment SGI12244[] = {{&S61645[30], 6}, {NULL}};
const Segment SGI12245[] = {{&S61645[82], 6}, {NULL}};
const Segment SGI12246[] = {{&S64435[42], 15}, {NULL}};
const Segment SGI12247[] = {{&S64435[30], 4}, {NULL}};
const Segment SGI12248[] = {{&S64435[106], 3}, {NULL}};
const Segment SGI12249[] = {{&S64435[71], 6}, {NULL}};
const Segment SGI12250[] = {{&S64435[126], 8}, {NULL}};
const Segment SGI12251[] = {{&S61078[42], 9}, {NULL}};
const Segment SGI12252[] = {{&S61078[42], 20}, {NULL}};
const Segment SGI12253[] = {{&S61078[30], 4}, {NULL}};
const Segment SGI12254[] = {{&S61078[110], 3}, {NULL}};
const Segment SGI12255[] = {{&S61078[76], 5}, {NULL}};
const Segment SGI12256[] = {{&S61989[8], 8}, {NULL}};
const Segment SGI12257[] = {{&S64783[40], 8}, {NULL}};
const Segment SGI12258[] = {{&S64783[30], 4}, {NULL}};
const Segment SGI12259[] = {{&S64783[110], 3}, {NULL}};
const Segment SGI12260[] = {{&S64783[74], 7}, {NULL}};
const Segment SGI12261[] = {{&S64783[130], 8}, {NULL}};
const Segment SGI12262[] = {{&S64784[42], 9}, {NULL}};
const Segment SGI12263[] = {{&S64784[42], 22}, {NULL}};
const Segment SGI12264[] = {{&S64784[30], 4}, {NULL}};
const Segment SGI12265[] = {{&S64784[113], 3}, {NULL}};
const Segment SGI12266[] = {{&S64784[78], 6}, {NULL}};
const Segment SGI12267[] = {{&S64784[133], 8}, {NULL}};
const Segment SGI12268[] = {{&S64785[42], 8}, {NULL}};
const Segment SGI12269[] = {{&S64785[42], 21}, {NULL}};
const Segment SGI12270[] = {{&S64785[30], 4}, {NULL}};
const Segment SGI12271[] = {{&S64785[112], 3}, {NULL}};
const Segment SGI12272[] = {{&S64785[77], 6}, {NULL}};
const Segment SGI12273[] = {{&S64785[132], 8}, {NULL}};
const Segment SGI12274[] = {{&S64786[42], 8}, {NULL}};
const Segment SGI12275[] = {{&S64786[42], 21}, {NULL}};
const Segment SGI12276[] = {{&S64786[30], 4}, {NULL}};
const Segment SGI12277[] = {{&S64786[112], 3}, {NULL}};
const Segment SGI12278[] = {{&S64786[77], 6}, {NULL}};
const Segment SGI12279[] = {{&S64786[132], 8}, {NULL}};
const Segment SGI12280[] = {{&S64787[42], 5}, {NULL}};
const Segment SGI12281[] = {{&S64787[30], 4}, {NULL}};
const Segment SGI12282[] = {{&S64787[97], 3}, {NULL}};
const Segment SGI12283[] = {{&S64787[61], 7}, {NULL}};
const Segment SGI12284[] = {{&S64787[117], 8}, {NULL}};
const Segment SGI12285[] = {{&S64788[42], 8}, {NULL}};
const Segment SGI12286[] = {{&S64788[42], 21}, {NULL}};
const Segment SGI12287[] = {{&S64788[30], 4}, {NULL}};
const Segment SGI12288[] = {{&S64788[112], 3}, {NULL}};
const Segment SGI12289[] = {{&S64788[77], 6}, {NULL}};
const Segment SGI12290[] = {{&S64788[132], 8}, {NULL}};
const Segment SGI12291[] = {{&S64789[42], 13}, {NULL}};
const Segment SGI12292[] = {{&S64789[42], 24}, {NULL}};
const Segment SGI12293[] = {{&S64789[30], 4}, {NULL}};
const Segment SGI12294[] = {{&S64789[115], 3}, {NULL}};
const Segment SGI12295[] = {{&S64789[80], 6}, {NULL}};
const Segment SGI12296[] = {{&S64789[135], 8}, {NULL}};
const Segment SGI12297[] = {{&S64790[42], 12}, {NULL}};
const Segment SGI12298[] = {{&S64790[42], 25}, {NULL}};
const Segment SGI12299[] = {{&S64790[30], 4}, {NULL}};
const Segment SGI12300[] = {{&S64790[116], 3}, {NULL}};
const Segment SGI12301[] = {{&S64790[81], 6}, {NULL}};
const Segment SGI12302[] = {{&S64790[136], 8}, {NULL}};
const Segment SGI12303[] = {{&S64791[42], 21}, {NULL}};
const Segment SGI12304[] = {{&S64791[42], 34}, {NULL}};
const Segment SGI12305[] = {{&S64791[30], 4}, {NULL}};
const Segment SGI12306[] = {{&S64791[125], 3}, {NULL}};
const Segment SGI12307[] = {{&S64791[90], 6}, {NULL}};
const Segment SGI12308[] = {{&S64791[145], 8}, {NULL}};
const Segment SGI12309[] = {{&S64792[42], 10}, {NULL}};
const Segment SGI12310[] = {{&S64792[42], 23}, {NULL}};
const Segment SGI12311[] = {{&S64792[30], 4}, {NULL}};
const Segment SGI12312[] = {{&S64792[114], 3}, {NULL}};
const Segment SGI12313[] = {{&S64792[79], 6}, {NULL}};
const Segment SGI12314[] = {{&S64792[134], 8}, {NULL}};
const Segment SGI12315[] = {{&S64793[42], 7}, {NULL}};
const Segment SGI12316[] = {{&S64793[42], 20}, {NULL}};
const Segment SGI12317[] = {{&S64793[30], 4}, {NULL}};
const Segment SGI12318[] = {{&S64793[111], 3}, {NULL}};
const Segment SGI12319[] = {{&S64793[76], 6}, {NULL}};
const Segment SGI12320[] = {{&S64793[131], 8}, {NULL}};
const Segment SGI12321[] = {{&S64794[42], 10}, {NULL}};
const Segment SGI12322[] = {{&S64794[42], 23}, {NULL}};
const Segment SGI12323[] = {{&S64794[30], 4}, {NULL}};
const Segment SGI12324[] = {{&S64794[114], 3}, {NULL}};
const Segment SGI12325[] = {{&S64794[79], 6}, {NULL}};
const Segment SGI12326[] = {{&S64794[134], 8}, {NULL}};
const Segment SGI12327[] = {{&S64795[47], 4}, {NULL}};
const Segment SGI12328[] = {{&S64795[99], 6}, {NULL}};
const Segment SGI12329[] = {{&S64796[40], 10}, {NULL}};
const Segment SGI12330[] = {{&S64796[30], 4}, {NULL}};
const Segment SGI12331[] = {{&S64796[112], 3}, {NULL}};
const Segment SGI12332[] = {{&S64796[77], 6}, {NULL}};
const Segment SGI12333[] = {{&S64796[132], 8}, {NULL}};
const Segment SGI12334[] = {{&S64797[46], 7}, {NULL}};
const Segment SGI12335[] = {{&S64797[46], 20}, {NULL}};
const Segment SGI12336[] = {{&S64797[30], 8}, {NULL}};
const Segment SGI12337[] = {{&S64797[115], 3}, {NULL}};
const Segment SGI12338[] = {{&S64797[80], 6}, {NULL}};
const Segment SGI12339[] = {{&S64797[135], 8}, {NULL}};
const Segment SGI12340[] = {{&S64798[54], 14}, {NULL}};
const Segment SGI12341[] = {{&S64798[54], 33}, {NULL}};
const Segment SGI12342[] = {{&S64798[30], 16}, {NULL}};
const Segment SGI12343[] = {{&S64798[104], 6}, {NULL}};
const Segment SGI12344[] = {{&S64799[42], 13}, {NULL}};
const Segment SGI12345[] = {{&S64799[42], 24}, {NULL}};
const Segment SGI12346[] = {{&S64799[30], 4}, {NULL}};
const Segment SGI12347[] = {{&S64799[115], 3}, {NULL}};
const Segment SGI12348[] = {{&S64799[80], 5}, {NULL}};
const Segment SGI12349[] = {{&S64799[135], 8}, {NULL}};
const Segment SGI12350[] = {{&S62392[6], 4}, {NULL}};
const Segment SGI12351[] = {{&S62392[32], 6}, {NULL}};
const Segment SGI12352[] = {{&S62392[78], 5}, {NULL}};
const Segment SGI12353[] = {{&S62393[6], 4}, {NULL}};
const Segment SGI12354[] = {{&S62393[72], 5}, {NULL}};
const Segment SGI12355[] = {{&S62394[6], 4}, {NULL}};
const Segment SGI12356[] = {{&S62394[32], 7}, {NULL}};
const Segment SGI12357[] = {{&S62394[79], 5}, {NULL}};
const Segment SGI12358[] = {{&S67410[8], 7}, {NULL}};
const Segment SGI12359[] = {{&S67410[0], 45}, {NULL}};
const Segment SGI12360[] = {{&S67411[8], 7}, {NULL}};
const Segment SGI12361[] = {{&S67411[0], 45}, {NULL}};
const Segment SGI12362[] = {{&S67412[8], 6}, {NULL}};
const Segment SGI12363[] = {{&S67412[0], 59}, {NULL}};
const Segment SGI12364[] = {{&S67413[8], 6}, {NULL}};
const Segment SGI12365[] = {{&S67413[0], 57}, {NULL}};
const Segment SGI12366[] = {{&S67414[8], 6}, {NULL}};
const Segment SGI12367[] = {{&S67414[0], 44}, {NULL}};
const Segment SGI12368[] = {{&S65188[45], 7}, {NULL}};
const Segment SGI12369[] = {{&S65188[45], 20}, {NULL}};
const Segment SGI12370[] = {{&S65188[30], 7}, {NULL}};
const Segment SGI12371[] = {{&S65188[114], 3}, {NULL}};
const Segment SGI12372[] = {{&S65188[79], 6}, {NULL}};
const Segment SGI12373[] = {{&S65188[134], 8}, {NULL}};
const Segment SGI12374[] = {{&S67535[0], 90}, {NULL}};
const Segment SGI12375[] = {{&S62535[86], 5}, {NULL}};
const Segment SGI12376[] = {{&S65281[42], 21}, {NULL}};
const Segment SGI12377[] = {{&S65281[42], 32}, {NULL}};
const Segment SGI12378[] = {{&S65281[30], 4}, {&S65281[50], 24}, {NULL}};
const Segment SGI12379[] = {{&S65281[124], 3}, {NULL}};
const Segment SGI12380[] = {{&S65281[88], 7}, {NULL}};
const Segment SGI12381[] = {{&S65281[144], 8}, {NULL}};
const Segment SGI12382[] = {{&S65282[42], 21}, {NULL}};
const Segment SGI12383[] = {{&S65282[42], 32}, {NULL}};
const Segment SGI12384[] = {{&S65282[30], 4}, {&S65282[50], 24}, {NULL}};
const Segment SGI12385[] = {{&S65282[123], 3}, {NULL}};
const Segment SGI12386[] = {{&S65282[88], 6}, {NULL}};
const Segment SGI12387[] = {{&S65282[143], 8}, {NULL}};
const Segment SGI12388[] = {{&S65306[40], 17}, {NULL}};
const Segment SGI12389[] = {{&S65306[30], 4}, {NULL}};
const Segment SGI12390[] = {{&S65306[117], 3}, {NULL}};
const Segment SGI12391[] = {{&S65306[82], 6}, {NULL}};
const Segment SGI12392[] = {{&S65306[137], 8}, {NULL}};
const Segment SGI12393[] = {{&S65307[42], 17}, {NULL}};
const Segment SGI12394[] = {{&S65307[42], 28}, {NULL}};
const Segment SGI12395[] = {{&S65307[30], 4}, {NULL}};
const Segment SGI12396[] = {{&S65307[119], 3}, {NULL}};
const Segment SGI12397[] = {{&S65307[84], 6}, {NULL}};
const Segment SGI12398[] = {{&S65307[139], 8}, {NULL}};
const Segment SGI12399[] = {{&S61529[57], 8}, {NULL}};
const Segment SGI12400[] = {{&S61529[57], 20}, {NULL}};
const Segment SGI12401[] = {{&S61529[30], 19}, {NULL}};
const Segment SGI12402[] = {{&S61529[126], 3}, {NULL}};
const Segment SGI12403[] = {{&S61529[91], 6}, {NULL}};
const Segment SGI12404[] = {{&S61530[57], 8}, {NULL}};
const Segment SGI12405[] = {{&S61530[57], 20}, {NULL}};
const Segment SGI12406[] = {{&S61530[30], 19}, {NULL}};
const Segment SGI12407[] = {{&S61530[126], 3}, {NULL}};
const Segment SGI12408[] = {{&S61530[91], 6}, {NULL}};
const Segment SGI12409[] = {{&S61531[57], 8}, {NULL}};
const Segment SGI12410[] = {{&S61531[57], 20}, {NULL}};
const Segment SGI12411[] = {{&S61531[30], 19}, {NULL}};
const Segment SGI12412[] = {{&S61531[126], 3}, {NULL}};
const Segment SGI12413[] = {{&S61531[91], 6}, {NULL}};
const Segment SGI12414[] = {{&S61540[11], 1}, {NULL}};
const Segment SGI12415[] = {{&S61540[50], 8}, {NULL}};
const Segment SGI12416[] = {{&S61540[30], 12}, {NULL}};
const Segment SGI12417[] = {{&S61540[120], 3}, {NULL}};
const Segment SGI12418[] = {{&S61540[86], 6}, {NULL}};
const Segment SGI12419[] = {{&S61541[50], 10}, {NULL}};
const Segment SGI12420[] = {{&S61541[50], 21}, {NULL}};
const Segment SGI12421[] = {{&S61541[30], 12}, {NULL}};
const Segment SGI12422[] = {{&S61541[120], 3}, {NULL}};
const Segment SGI12423[] = {{&S61541[85], 6}, {NULL}};
const Segment SGI12424[] = {{&S61542[11], 1}, {NULL}};
const Segment SGI12425[] = {{&S61542[50], 9}, {NULL}};
const Segment SGI12426[] = {{&S61542[30], 12}, {NULL}};
const Segment SGI12427[] = {{&S61542[122], 3}, {NULL}};
const Segment SGI12428[] = {{&S61542[87], 6}, {NULL}};
const Segment SGI12429[] = {{&S61543[44], 10}, {NULL}};
const Segment SGI12430[] = {{&S61543[31], 12}, {NULL}};
const Segment SGI12431[] = {{&S62713[8], 8}, {NULL}};
const Segment SGI12432[] = {{&S62454[44], 13}, {NULL}};
const Segment SGI12433[] = {{&S62454[44], 24}, {NULL}};
const Segment SGI12434[] = {{&S62454[30], 6}, {NULL}};
const Segment SGI12435[] = {{&S62454[116], 3}, {NULL}};
const Segment SGI12436[] = {{&S62454[82], 5}, {NULL}};
const Segment SGI12437[] = {{&S61805[50], 14}, {NULL}};
const Segment SGI12438[] = {{&S61805[50], 25}, {NULL}};
const Segment SGI12439[] = {{&S61805[30], 12}, {NULL}};
const Segment SGI12440[] = {{&S61805[124], 3}, {NULL}};
const Segment SGI12441[] = {{&S61805[89], 6}, {NULL}};
const Segment SGI12442[] = {{&S61806[50], 16}, {NULL}};
const Segment SGI12443[] = {{&S61806[50], 27}, {NULL}};
const Segment SGI12444[] = {{&S61806[30], 12}, {NULL}};
const Segment SGI12445[] = {{&S61806[126], 3}, {NULL}};
const Segment SGI12446[] = {{&S61806[91], 6}, {NULL}};
const Segment SGI12447[] = {{&S61807[50], 16}, {NULL}};
const Segment SGI12448[] = {{&S61807[50], 27}, {NULL}};
const Segment SGI12449[] = {{&S61807[30], 12}, {NULL}};
const Segment SGI12450[] = {{&S61807[126], 3}, {NULL}};
const Segment SGI12451[] = {{&S61807[91], 6}, {NULL}};
const Segment SGI12452[] = {{&S61808[44], 16}, {NULL}};
const Segment SGI12453[] = {{&S61808[31], 12}, {NULL}};
const Segment SGI12454[] = {{&S61809[50], 16}, {NULL}};
const Segment SGI12455[] = {{&S61809[50], 27}, {NULL}};
const Segment SGI12456[] = {{&S61809[30], 12}, {NULL}};
const Segment SGI12457[] = {{&S61809[94], 4}, {NULL}};
const Segment SGI12458[] = {{&S61809[94], 3}, {NULL}};
const Segment SGI12459[] = {{&S61824[50], 10}, {NULL}};
const Segment SGI12460[] = {{&S61824[50], 22}, {NULL}};
const Segment SGI12461[] = {{&S61824[30], 12}, {NULL}};
const Segment SGI12462[] = {{&S61824[121], 3}, {NULL}};
const Segment SGI12463[] = {{&S61824[86], 6}, {NULL}};
const Segment SGI12464[] = {{&S61825[44], 10}, {NULL}};
const Segment SGI12465[] = {{&S61825[31], 12}, {NULL}};
const Segment SGI12466[] = {{&S61879[48], 10}, {NULL}};
const Segment SGI12467[] = {{&S61879[30], 12}, {NULL}};
const Segment SGI12468[] = {{&S61879[119], 3}, {NULL}};
const Segment SGI12469[] = {{&S61879[84], 6}, {NULL}};
const Segment SGI12470[] = {{&S61880[50], 10}, {NULL}};
const Segment SGI12471[] = {{&S61880[50], 22}, {NULL}};
const Segment SGI12472[] = {{&S61880[30], 12}, {NULL}};
const Segment SGI12473[] = {{&S61880[121], 3}, {NULL}};
const Segment SGI12474[] = {{&S61880[86], 6}, {NULL}};
const Segment SGI12475[] = {{&S61881[44], 10}, {NULL}};
const Segment SGI12476[] = {{&S61881[31], 12}, {NULL}};
const Segment SGI12477[] = {{&S61883[50], 7}, {NULL}};
const Segment SGI12478[] = {{&S61883[50], 18}, {NULL}};
const Segment SGI12479[] = {{&S61883[30], 12}, {NULL}};
const Segment SGI12480[] = {{&S61883[117], 3}, {NULL}};
const Segment SGI12481[] = {{&S61883[82], 6}, {NULL}};
const Segment SGI12482[] = {{&S61884[48], 7}, {NULL}};
const Segment SGI12483[] = {{&S61884[30], 12}, {NULL}};
const Segment SGI12484[] = {{&S61884[115], 3}, {NULL}};
const Segment SGI12485[] = {{&S61884[80], 6}, {NULL}};
const Segment SGI12486[] = {{&S61885[44], 7}, {NULL}};
const Segment SGI12487[] = {{&S61885[31], 12}, {NULL}};
const Segment SGI12488[] = {{&S62801[37], 17}, {NULL}};
const Segment SGI12489[] = {{&S62801[30], 6}, {NULL}};
const Segment SGI12490[] = {{&S62801[123], 3}, {NULL}};
const Segment SGI12491[] = {{&S62801[88], 6}, {NULL}};
const Segment SGI12492[] = {{&S62801[141], 6}, {NULL}};
const Segment SGI12493[] = {{&S62055[42], 6}, {NULL}};
const Segment SGI12494[] = {{&S62055[42], 18}, {NULL}};
const Segment SGI12495[] = {{&S62055[30], 4}, {NULL}};
const Segment SGI12496[] = {{&S62055[110], 3}, {NULL}};
const Segment SGI12497[] = {{&S62055[74], 7}, {NULL}};
const Segment SGI12498[] = {{&S62055[74], 6}, {NULL}};
const Segment SGI12499[] = {{&S62056[50], 6}, {NULL}};
const Segment SGI12500[] = {{&S62056[50], 17}, {NULL}};
const Segment SGI12501[] = {{&S62056[30], 12}, {NULL}};
const Segment SGI12502[] = {{&S62056[116], 3}, {NULL}};
const Segment SGI12503[] = {{&S62056[81], 6}, {NULL}};
const Segment SGI12504[] = {{&S62057[44], 6}, {NULL}};
const Segment SGI12505[] = {{&S62057[31], 12}, {NULL}};
const Segment SGI12506[] = {{&S62158[51], 11}, {NULL}};
const Segment SGI12507[] = {{&S62158[51], 23}, {NULL}};
const Segment SGI12508[] = {{&S62158[30], 13}, {NULL}};
const Segment SGI12509[] = {{&S62158[123], 3}, {NULL}};
const Segment SGI12510[] = {{&S62158[88], 6}, {NULL}};
const Segment SGI12511[] = {{&S62159[50], 11}, {NULL}};
const Segment SGI12512[] = {{&S62159[50], 23}, {NULL}};
const Segment SGI12513[] = {{&S62159[30], 12}, {NULL}};
const Segment SGI12514[] = {{&S62159[122], 3}, {NULL}};
const Segment SGI12515[] = {{&S62159[87], 6}, {NULL}};
const Segment SGI12516[] = {{&S62160[44], 11}, {NULL}};
const Segment SGI12517[] = {{&S62160[31], 12}, {NULL}};
const Segment SGI12518[] = {{&S62360[42], 6}, {NULL}};
const Segment SGI12519[] = {{&S62360[42], 17}, {NULL}};
const Segment SGI12520[] = {{&S62360[30], 4}, {NULL}};
const Segment SGI12521[] = {{&S62360[107], 3}, {NULL}};
const Segment SGI12522[] = {{&S62360[73], 5}, {NULL}};
const Segment SGI12523[] = {{&S62449[42], 12}, {NULL}};
const Segment SGI12524[] = {{&S62449[42], 23}, {NULL}};
const Segment SGI12525[] = {{&S62449[30], 4}, {NULL}};
const Segment SGI12526[] = {{&S62449[112], 3}, {NULL}};
const Segment SGI12527[] = {{&S62449[79], 5}, {NULL}};
const Segment SGI12528[] = {{&S62450[42], 12}, {NULL}};
const Segment SGI12529[] = {{&S62450[42], 23}, {NULL}};
const Segment SGI12530[] = {{&S62450[30], 4}, {NULL}};
const Segment SGI12531[] = {{&S62450[112], 3}, {NULL}};
const Segment SGI12532[] = {{&S62450[79], 5}, {NULL}};
const Segment SGI12533[] = {{&S62451[41], 13}, {NULL}};
const Segment SGI12534[] = {{&S62451[41], 24}, {NULL}};
const Segment SGI12535[] = {{&S62451[112], 3}, {NULL}};
const Segment SGI12536[] = {{&S62451[79], 5}, {NULL}};
const Segment SGI12537[] = {{&S62452[42], 13}, {NULL}};
const Segment SGI12538[] = {{&S62452[42], 24}, {NULL}};
const Segment SGI12539[] = {{&S62452[30], 4}, {NULL}};
const Segment SGI12540[] = {{&S62452[113], 3}, {NULL}};
const Segment SGI12541[] = {{&S62452[80], 5}, {NULL}};
const Segment SGI12542[] = {{&S62453[36], 13}, {NULL}};
const Segment SGI12543[] = {{&S62453[31], 4}, {NULL}};
const Segment SGI12544[] = {{&S63372[6], 4}, {NULL}};
const Segment SGI12545[] = {{&S63372[32], 3}, {NULL}};
const Segment SGI12546[] = {{&S63372[81], 5}, {NULL}};
const Segment SGI12547[] = {{&S62580[42], 8}, {NULL}};
const Segment SGI12548[] = {{&S62580[42], 19}, {NULL}};
const Segment SGI12549[] = {{&S62580[30], 4}, {NULL}};
const Segment SGI12550[] = {{&S62580[109], 3}, {NULL}};
const Segment SGI12551[] = {{&S62580[75], 5}, {NULL}};
const Segment SGI12552[] = {{&S62581[36], 8}, {NULL}};
const Segment SGI12553[] = {{&S62581[31], 4}, {NULL}};
const Segment SGI12554[] = {{&S63192[44], 12}, {NULL}};
const Segment SGI12555[] = {{&S63192[44], 24}, {NULL}};
const Segment SGI12556[] = {{&S63192[30], 6}, {NULL}};
const Segment SGI12557[] = {{&S63192[116], 3}, {NULL}};
const Segment SGI12558[] = {{&S63192[82], 5}, {NULL}};
const Segment SGI12559[] = {{&S63194[44], 12}, {NULL}};
const Segment SGI12560[] = {{&S63194[44], 23}, {NULL}};
const Segment SGI12561[] = {{&S63194[30], 6}, {NULL}};
const Segment SGI12562[] = {{&S63194[115], 3}, {NULL}};
const Segment SGI12563[] = {{&S63194[81], 5}, {NULL}};
const Segment SGI12564[] = {{&S63195[44], 12}, {NULL}};
const Segment SGI12565[] = {{&S63195[44], 23}, {NULL}};
const Segment SGI12566[] = {{&S63195[30], 6}, {NULL}};
const Segment SGI12567[] = {{&S63195[115], 3}, {NULL}};
const Segment SGI12568[] = {{&S63195[81], 5}, {NULL}};
const Segment SGI12569[] = {{&S63197[44], 12}, {NULL}};
const Segment SGI12570[] = {{&S63197[44], 23}, {NULL}};
const Segment SGI12571[] = {{&S63197[30], 6}, {NULL}};
const Segment SGI12572[] = {{&S63197[115], 3}, {NULL}};
const Segment SGI12573[] = {{&S63197[81], 5}, {NULL}};
const Segment SGI12574[] = {{&S63199[38], 12}, {NULL}};
const Segment SGI12575[] = {{&S63199[31], 6}, {NULL}};
const Segment SGI12576[] = {{&S63200[44], 12}, {NULL}};
const Segment SGI12577[] = {{&S63200[44], 22}, {NULL}};
const Segment SGI12578[] = {{&S63200[30], 6}, {NULL}};
const Segment SGI12579[] = {{&S63200[114], 3}, {NULL}};
const Segment SGI12580[] = {{&S63200[80], 5}, {NULL}};
const Segment SGI12581[] = {{&S68780[0], 108}, {NULL}};
const Segment SGI12582[] = {{&S68782[0], 108}, {NULL}};
const Segment SGI12583[] = {{&S63299[44], 6}, {NULL}};
const Segment SGI12584[] = {{&S63299[44], 17}, {NULL}};
const Segment SGI12585[] = {{&S63299[30], 6}, {NULL}};
const Segment SGI12586[] = {{&S63299[109], 3}, {NULL}};
const Segment SGI12587[] = {{&S63299[75], 5}, {NULL}};
const Segment SGI12588[] = {{&S63300[44], 6}, {NULL}};
const Segment SGI12589[] = {{&S63300[44], 17}, {NULL}};
const Segment SGI12590[] = {{&S63300[30], 6}, {NULL}};
const Segment SGI12591[] = {{&S63300[109], 3}, {NULL}};
const Segment SGI12592[] = {{&S63300[75], 5}, {NULL}};
const Segment SGI12593[] = {{&S63301[44], 6}, {NULL}};
const Segment SGI12594[] = {{&S63301[44], 17}, {NULL}};
const Segment SGI12595[] = {{&S63301[30], 6}, {NULL}};
const Segment SGI12596[] = {{&S63301[109], 3}, {NULL}};
const Segment SGI12597[] = {{&S63301[75], 5}, {NULL}};
const Segment SGI12598[] = {{&S63302[44], 6}, {NULL}};
const Segment SGI12599[] = {{&S63302[44], 17}, {NULL}};
const Segment SGI12600[] = {{&S63302[30], 6}, {NULL}};
const Segment SGI12601[] = {{&S63302[109], 3}, {NULL}};
const Segment SGI12602[] = {{&S63302[75], 5}, {NULL}};
const Segment SGI12603[] = {{&S63303[42], 6}, {NULL}};
const Segment SGI12604[] = {{&S63303[30], 6}, {NULL}};
const Segment SGI12605[] = {{&S63303[107], 3}, {NULL}};
const Segment SGI12606[] = {{&S63303[73], 5}, {NULL}};
const Segment SGI12607[] = {{&S63304[38], 6}, {NULL}};
const Segment SGI12608[] = {{&S63304[31], 6}, {NULL}};
const Segment SGI12609[] = {{&S63409[44], 10}, {NULL}};
const Segment SGI12610[] = {{&S63409[44], 21}, {NULL}};
const Segment SGI12611[] = {{&S63409[30], 6}, {NULL}};
const Segment SGI12612[] = {{&S63409[113], 3}, {NULL}};
const Segment SGI12613[] = {{&S63409[79], 5}, {NULL}};
const Segment SGI12614[] = {{&S63410[44], 10}, {NULL}};
const Segment SGI12615[] = {{&S63410[44], 21}, {NULL}};
const Segment SGI12616[] = {{&S63410[30], 6}, {NULL}};
const Segment SGI12617[] = {{&S63410[115], 3}, {NULL}};
const Segment SGI12618[] = {{&S63410[79], 7}, {NULL}};
const Segment SGI12619[] = {{&S63410[135], 8}, {NULL}};
const Segment SGI12620[] = {{&S63411[38], 10}, {NULL}};
const Segment SGI12621[] = {{&S63411[31], 6}, {NULL}};
const Segment SGI12622[] = {{&S62851[50], 23}, {NULL}};
const Segment SGI12623[] = {{&S62851[50], 34}, {NULL}};
const Segment SGI12624[] = {{&S62851[30], 12}, {NULL}};
const Segment SGI12625[] = {{&S62851[133], 3}, {NULL}};
const Segment SGI12626[] = {{&S62851[98], 6}, {NULL}};
const Segment SGI12627[] = {{&S62852[50], 23}, {NULL}};
const Segment SGI12628[] = {{&S62852[50], 34}, {NULL}};
const Segment SGI12629[] = {{&S62852[30], 12}, {NULL}};
const Segment SGI12630[] = {{&S62852[133], 3}, {NULL}};
const Segment SGI12631[] = {{&S62852[98], 6}, {NULL}};
const Segment SGI12632[] = {{&S62853[44], 23}, {NULL}};
const Segment SGI12633[] = {{&S62853[31], 12}, {NULL}};
const Segment SGI12634[] = {{&S62854[50], 6}, {NULL}};
const Segment SGI12635[] = {{&S62854[50], 17}, {NULL}};
const Segment SGI12636[] = {{&S62854[30], 12}, {NULL}};
const Segment SGI12637[] = {{&S62854[116], 3}, {NULL}};
const Segment SGI12638[] = {{&S62854[81], 6}, {NULL}};
const Segment SGI12639[] = {{&S62855[50], 6}, {NULL}};
const Segment SGI12640[] = {{&S62855[50], 17}, {NULL}};
const Segment SGI12641[] = {{&S62855[30], 12}, {NULL}};
const Segment SGI12642[] = {{&S62855[117], 3}, {NULL}};
const Segment SGI12643[] = {{&S62855[81], 7}, {NULL}};
const Segment SGI12644[] = {{&S62855[137], 8}, {NULL}};
const Segment SGI12645[] = {{&S62915[50], 6}, {NULL}};
const Segment SGI12646[] = {{&S62915[50], 17}, {NULL}};
const Segment SGI12647[] = {{&S62915[30], 12}, {NULL}};
const Segment SGI12648[] = {{&S62915[116], 3}, {NULL}};
const Segment SGI12649[] = {{&S62915[81], 6}, {NULL}};
const Segment SGI12650[] = {{&S62916[50], 6}, {NULL}};
const Segment SGI12651[] = {{&S62916[50], 17}, {NULL}};
const Segment SGI12652[] = {{&S62916[30], 12}, {NULL}};
const Segment SGI12653[] = {{&S62916[116], 3}, {NULL}};
const Segment SGI12654[] = {{&S62916[81], 6}, {NULL}};
const Segment SGI12655[] = {{&S62917[50], 6}, {NULL}};
const Segment SGI12656[] = {{&S62917[50], 17}, {NULL}};
const Segment SGI12657[] = {{&S62917[30], 12}, {NULL}};
const Segment SGI12658[] = {{&S62917[116], 3}, {NULL}};
const Segment SGI12659[] = {{&S62917[81], 6}, {NULL}};
const Segment SGI12660[] = {{&S62918[50], 6}, {NULL}};
const Segment SGI12661[] = {{&S62918[50], 17}, {NULL}};
const Segment SGI12662[] = {{&S62918[30], 12}, {NULL}};
const Segment SGI12663[] = {{&S62918[116], 3}, {NULL}};
const Segment SGI12664[] = {{&S62918[81], 6}, {NULL}};
const Segment SGI12665[] = {{&S62919[50], 6}, {NULL}};
const Segment SGI12666[] = {{&S62919[50], 17}, {NULL}};
const Segment SGI12667[] = {{&S62919[30], 12}, {NULL}};
const Segment SGI12668[] = {{&S62919[116], 3}, {NULL}};
const Segment SGI12669[] = {{&S62919[81], 6}, {NULL}};
const Segment SGI12670[] = {{&S62920[50], 6}, {NULL}};
const Segment SGI12671[] = {{&S62920[50], 17}, {NULL}};
const Segment SGI12672[] = {{&S62920[30], 12}, {NULL}};
const Segment SGI12673[] = {{&S62920[116], 3}, {NULL}};
const Segment SGI12674[] = {{&S62920[81], 6}, {NULL}};
const Segment SGI12675[] = {{&S62921[44], 6}, {NULL}};
const Segment SGI12676[] = {{&S62921[31], 12}, {NULL}};
const Segment SGI12677[] = {{&S65458[42], 16}, {NULL}};
const Segment SGI12678[] = {{&S65458[30], 4}, {NULL}};
const Segment SGI12679[] = {{&S65458[107], 3}, {NULL}};
const Segment SGI12680[] = {{&S65458[72], 6}, {NULL}};
const Segment SGI12681[] = {{&S63191[42], 12}, {NULL}};
const Segment SGI12682[] = {{&S63191[42], 23}, {NULL}};
const Segment SGI12683[] = {{&S63191[30], 4}, {NULL}};
const Segment SGI12684[] = {{&S63191[113], 3}, {NULL}};
const Segment SGI12685[] = {{&S63191[79], 5}, {NULL}};
const Segment SGI12686[] = {{&S63193[42], 12}, {NULL}};
const Segment SGI12687[] = {{&S63193[42], 23}, {NULL}};
const Segment SGI12688[] = {{&S63193[30], 4}, {NULL}};
const Segment SGI12689[] = {{&S63193[113], 3}, {NULL}};
const Segment SGI12690[] = {{&S63193[79], 5}, {NULL}};
const Segment SGI12691[] = {{&S63198[36], 12}, {NULL}};
const Segment SGI12692[] = {{&S63198[31], 4}, {NULL}};
const Segment SGI12693[] = {{&S64379[44], 15}, {NULL}};
const Segment SGI12694[] = {{&S64379[44], 26}, {NULL}};
const Segment SGI12695[] = {{&S64379[30], 6}, {NULL}};
const Segment SGI12696[] = {{&S64379[118], 3}, {NULL}};
const Segment SGI12697[] = {{&S64379[84], 6}, {NULL}};
const Segment SGI12698[] = {{&S64379[138], 8}, {NULL}};
const Segment SGI12699[] = {{&S64415[6], 4}, {NULL}};
const Segment SGI12700[] = {{&S64415[75], 4}, {NULL}};
const Segment SGI12701[] = {{&S64416[87], 3}, {NULL}};
const Segment SGI12702[] = {{&S63255[50], 11}, {NULL}};
const Segment SGI12703[] = {{&S64416[6], 4}, {NULL}};
const Segment SGI12704[] = {{&S63255[50], 22}, {NULL}};
const Segment SGI12705[] = {{&S64416[114], 4}, {NULL}};
const Segment SGI12706[] = {{&S63255[30], 12}, {NULL}};
const Segment SGI12707[] = {{&S63255[121], 3}, {NULL}};
const Segment SGI12708[] = {{&S63255[86], 6}, {NULL}};
const Segment SGI12709[] = {{&S63256[42], 11}, {NULL}};
const Segment SGI12710[] = {{&S63256[42], 22}, {NULL}};
const Segment SGI12711[] = {{&S63256[30], 4}, {NULL}};
const Segment SGI12712[] = {{&S63256[112], 3}, {NULL}};
const Segment SGI12713[] = {{&S63256[78], 5}, {NULL}};
const Segment SGI12714[] = {{&S63257[44], 11}, {NULL}};
const Segment SGI12715[] = {{&S63257[31], 12}, {NULL}};
const Segment SGI12716[] = {{&S63258[50], 12}, {NULL}};
const Segment SGI12717[] = {{&S63258[50], 23}, {NULL}};
const Segment SGI12718[] = {{&S63258[30], 12}, {NULL}};
const Segment SGI12719[] = {{&S63258[122], 3}, {NULL}};
const Segment SGI12720[] = {{&S63258[87], 6}, {NULL}};
const Segment SGI12721[] = {{&S63259[50], 7}, {NULL}};
const Segment SGI12722[] = {{&S63259[50], 18}, {NULL}};
const Segment SGI12723[] = {{&S63259[30], 12}, {NULL}};
const Segment SGI12724[] = {{&S63259[117], 3}, {NULL}};
const Segment SGI12725[] = {{&S63259[82], 6}, {NULL}};
const Segment SGI12726[] = {{&S63260[50], 20}, {NULL}};
const Segment SGI12727[] = {{&S63260[50], 31}, {NULL}};
const Segment SGI12728[] = {{&S63260[30], 12}, {NULL}};
const Segment SGI12729[] = {{&S63260[130], 3}, {NULL}};
const Segment SGI12730[] = {{&S63260[95], 6}, {NULL}};
const Segment SGI12731[] = {{&S63261[44], 7}, {NULL}};
const Segment SGI12732[] = {{&S63261[31], 12}, {NULL}};
const Segment SGI12733[] = {{&S63262[50], 7}, {NULL}};
const Segment SGI12734[] = {{&S63262[50], 18}, {NULL}};
const Segment SGI12735[] = {{&S63262[30], 12}, {NULL}};
const Segment SGI12736[] = {{&S63262[85], 4}, {NULL}};
const Segment SGI12737[] = {{&S63262[85], 3}, {NULL}};
const Segment SGI12738[] = {{&S63408[42], 10}, {NULL}};
const Segment SGI12739[] = {{&S63408[42], 21}, {NULL}};
const Segment SGI12740[] = {{&S63408[30], 4}, {NULL}};
const Segment SGI12741[] = {{&S63408[111], 3}, {NULL}};
const Segment SGI12742[] = {{&S63408[77], 5}, {NULL}};
const Segment SGI12743[] = {{&S63412[36], 10}, {NULL}};
const Segment SGI12744[] = {{&S63412[31], 4}, {NULL}};
const Segment SGI12745[] = {{&S64073[6], 4}, {NULL}};
const Segment SGI12746[] = {{&S64073[88], 5}, {NULL}};
const Segment SGI12747[] = {{&S67108[44], 8}, {NULL}};
const Segment SGI12748[] = {{&S67108[44], 20}, {NULL}};
const Segment SGI12749[] = {{&S67108[30], 6}, {NULL}};
const Segment SGI12750[] = {{&S67108[113], 3}, {NULL}};
const Segment SGI12751[] = {{&S67108[78], 6}, {NULL}};
const Segment SGI12752[] = {{&S69358[8], 6}, {NULL}};
const Segment SGI12753[] = {{&S69358[0], 91}, {NULL}};
const Segment SGI12754[] = {{&S69359[8], 6}, {&S69359[8], 6}, {NULL}};
const Segment SGI12755[] = {{&S69359[0], 170}, {NULL}};
const Segment SGI12756[] = {{&S69360[8], 7}, {NULL}};
const Segment SGI12757[] = {{&S69360[0], 85}, {NULL}};
const Segment SGI12758[] = {{&S69361[8], 6}, {NULL}};
const Segment SGI12759[] = {{&S69361[0], 84}, {NULL}};
const Segment SGI12760[] = {{&S63609[42], 9}, {NULL}};
const Segment SGI12761[] = {{&S63609[42], 20}, {NULL}};
const Segment SGI12762[] = {{&S63609[30], 4}, {NULL}};
const Segment SGI12763[] = {{&S63609[110], 3}, {NULL}};
const Segment SGI12764[] = {{&S63609[76], 5}, {NULL}};
const Segment SGI12765[] = {{&S63610[42], 9}, {NULL}};
const Segment SGI12766[] = {{&S63610[42], 20}, {NULL}};
const Segment SGI12767[] = {{&S63610[30], 4}, {NULL}};
const Segment SGI12768[] = {{&S63610[112], 3}, {NULL}};
const Segment SGI12769[] = {{&S63610[76], 7}, {NULL}};
const Segment SGI12770[] = {{&S63610[132], 8}, {NULL}};
const Segment SGI12771[] = {{&S63611[36], 9}, {NULL}};
const Segment SGI12772[] = {{&S63611[31], 4}, {NULL}};
const Segment SGI12773[] = {{&S67297[42], 6}, {NULL}};
const Segment SGI12774[] = {{&S67297[30], 4}, {NULL}};
const Segment SGI12775[] = {{&S67298[42], 6}, {NULL}};
const Segment SGI12776[] = {{&S67298[30], 4}, {NULL}};
const Segment SGI12777[] = {{&S67298[98], 3}, {NULL}};
const Segment SGI12778[] = {{&S67298[62], 7}, {NULL}};
const Segment SGI12779[] = {{&S67298[118], 8}, {NULL}};
const Segment SGI12780[] = {{&S67299[42], 10}, {NULL}};
const Segment SGI12781[] = {{&S67299[30], 4}, {NULL}};
const Segment SGI12782[] = {{&S67299[102], 3}, {NULL}};
const Segment SGI12783[] = {{&S67299[66], 7}, {NULL}};
const Segment SGI12784[] = {{&S67299[122], 8}, {NULL}};
const Segment SGI12785[] = {{&S63938[42], 16}, {NULL}};
const Segment SGI12786[] = {{&S63938[42], 27}, {NULL}};
const Segment SGI12787[] = {{&S63938[30], 4}, {NULL}};
const Segment SGI12788[] = {{&S63938[117], 3}, {NULL}};
const Segment SGI12789[] = {{&S63938[83], 5}, {NULL}};
const Segment SGI12790[] = {{&S63820[50], 5}, {NULL}};
const Segment SGI12791[] = {{&S63820[50], 16}, {NULL}};
const Segment SGI12792[] = {{&S63820[30], 12}, {NULL}};
const Segment SGI12793[] = {{&S63820[115], 3}, {NULL}};
const Segment SGI12794[] = {{&S63820[80], 6}, {NULL}};
const Segment SGI12795[] = {{&S63821[50], 10}, {NULL}};
const Segment SGI12796[] = {{&S63821[50], 21}, {NULL}};
const Segment SGI12797[] = {{&S63821[30], 12}, {NULL}};
const Segment SGI12798[] = {{&S63821[120], 3}, {NULL}};
const Segment SGI12799[] = {{&S63821[85], 6}, {NULL}};
const Segment SGI12800[] = {{&S63822[50], 7}, {NULL}};
const Segment SGI12801[] = {{&S63822[50], 18}, {NULL}};
const Segment SGI12802[] = {{&S63822[30], 12}, {NULL}};
const Segment SGI12803[] = {{&S63822[117], 3}, {NULL}};
const Segment SGI12804[] = {{&S63822[82], 6}, {NULL}};
const Segment SGI12805[] = {{&S63823[44], 7}, {NULL}};
const Segment SGI12806[] = {{&S63823[31], 12}, {NULL}};
const Segment SGI12807[] = {{&S64009[50], 7}, {NULL}};
const Segment SGI12808[] = {{&S64009[50], 18}, {NULL}};
const Segment SGI12809[] = {{&S64009[30], 12}, {NULL}};
const Segment SGI12810[] = {{&S64009[117], 3}, {NULL}};
const Segment SGI12811[] = {{&S64009[82], 6}, {NULL}};
const Segment SGI12812[] = {{&S64010[44], 7}, {NULL}};
const Segment SGI12813[] = {{&S64010[31], 12}, {NULL}};
const Segment SGI12814[] = {{&S64011[50], 7}, {NULL}};
const Segment SGI12815[] = {{&S64011[50], 24}, {NULL}};
const Segment SGI12816[] = {{&S64011[30], 12}, {NULL}};
const Segment SGI12817[] = {{&S64011[123], 3}, {NULL}};
const Segment SGI12818[] = {{&S64011[88], 6}, {NULL}};
const Segment SGI12819[] = {{&S64755[44], 23}, {NULL}};
const Segment SGI12820[] = {{&S64755[44], 32}, {NULL}};
const Segment SGI12821[] = {{&S64755[30], 6}, {NULL}};
const Segment SGI12822[] = {{&S64755[124], 3}, {NULL}};
const Segment SGI12823[] = {{&S64755[90], 5}, {NULL}};
const Segment SGI12824[] = {{&S64292[50], 10}, {NULL}};
const Segment SGI12825[] = {{&S64292[50], 22}, {NULL}};
const Segment SGI12826[] = {{&S64292[30], 12}, {NULL}};
const Segment SGI12827[] = {{&S64292[121], 3}, {NULL}};
const Segment SGI12828[] = {{&S64292[86], 6}, {NULL}};
const Segment SGI12829[] = {{&S64919[44], 4}, {NULL}};
const Segment SGI12830[] = {{&S64919[44], 15}, {NULL}};
const Segment SGI12831[] = {{&S64919[30], 6}, {NULL}};
const Segment SGI12832[] = {{&S64919[107], 3}, {NULL}};
const Segment SGI12833[] = {{&S64919[73], 5}, {NULL}};
const Segment SGI12834[] = {{&S64920[44], 4}, {NULL}};
const Segment SGI12835[] = {{&S64920[44], 16}, {NULL}};
const Segment SGI12836[] = {{&S64920[30], 6}, {NULL}};
const Segment SGI12837[] = {{&S64920[108], 3}, {NULL}};
const Segment SGI12838[] = {{&S64920[74], 5}, {NULL}};
const Segment SGI12839[] = {{&S64921[44], 4}, {NULL}};
const Segment SGI12840[] = {{&S64921[44], 13}, {NULL}};
const Segment SGI12841[] = {{&S64921[30], 6}, {NULL}};
const Segment SGI12842[] = {{&S64921[105], 3}, {NULL}};
const Segment SGI12843[] = {{&S64921[71], 5}, {NULL}};
const Segment SGI12844[] = {{&S64922[44], 4}, {NULL}};
const Segment SGI12845[] = {{&S64922[44], 19}, {NULL}};
const Segment SGI12846[] = {{&S64922[30], 6}, {NULL}};
const Segment SGI12847[] = {{&S64922[111], 3}, {NULL}};
const Segment SGI12848[] = {{&S64922[77], 5}, {NULL}};
const Segment SGI12849[] = {{&S64923[44], 4}, {NULL}};
const Segment SGI12850[] = {{&S64923[44], 15}, {NULL}};
const Segment SGI12851[] = {{&S64923[30], 6}, {NULL}};
const Segment SGI12852[] = {{&S64923[107], 3}, {NULL}};
const Segment SGI12853[] = {{&S64923[73], 5}, {NULL}};
const Segment SGI12854[] = {{&S64926[44], 4}, {NULL}};
const Segment SGI12855[] = {{&S64926[44], 12}, {NULL}};
const Segment SGI12856[] = {{&S64926[30], 6}, {NULL}};
const Segment SGI12857[] = {{&S64926[104], 3}, {NULL}};
const Segment SGI12858[] = {{&S64926[70], 5}, {NULL}};
const Segment SGI12859[] = {{&S64935[44], 10}, {NULL}};
const Segment SGI12860[] = {{&S64935[44], 21}, {NULL}};
const Segment SGI12861[] = {{&S64935[30], 6}, {NULL}};
const Segment SGI12862[] = {{&S64935[143], 4}, {NULL}};
const Segment SGI12863[] = {{&S64935[169], 3}, {NULL}};
const Segment SGI12864[] = {{&S64935[113], 3}, {&S64935[218], 5}, {NULL}};
const Segment SGI12865[] = {{&S64935[79], 5}, {NULL}};
const Segment SGI12866[] = {{&S64391[42], 6}, {NULL}};
const Segment SGI12867[] = {{&S64391[42], 17}, {NULL}};
const Segment SGI12868[] = {{&S64391[30], 4}, {NULL}};
const Segment SGI12869[] = {{&S64391[108], 3}, {NULL}};
const Segment SGI12870[] = {{&S64391[73], 6}, {NULL}};
const Segment SGI12871[] = {{&S64392[42], 18}, {NULL}};
const Segment SGI12872[] = {{&S64392[42], 29}, {NULL}};
const Segment SGI12873[] = {{&S64392[30], 4}, {NULL}};
const Segment SGI12874[] = {{&S64392[120], 3}, {NULL}};
const Segment SGI12875[] = {{&S64392[85], 6}, {NULL}};
const Segment SGI12876[] = {{&S64393[40], 6}, {NULL}};
const Segment SGI12877[] = {{&S64393[30], 4}, {NULL}};
const Segment SGI12878[] = {{&S64393[106], 3}, {NULL}};
const Segment SGI12879[] = {{&S64393[71], 6}, {NULL}};
const Segment SGI12880[] = {{&S64394[42], 6}, {NULL}};
const Segment SGI12881[] = {{&S64394[42], 17}, {NULL}};
const Segment SGI12882[] = {{&S64394[30], 4}, {NULL}};
const Segment SGI12883[] = {{&S64394[109], 3}, {NULL}};
const Segment SGI12884[] = {{&S64394[73], 7}, {NULL}};
const Segment SGI12885[] = {{&S64394[129], 8}, {NULL}};
const Segment SGI12886[] = {{&S64395[36], 6}, {NULL}};
const Segment SGI12887[] = {{&S64395[31], 4}, {NULL}};
const Segment SGI12888[] = {{&S64456[50], 12}, {NULL}};
const Segment SGI12889[] = {{&S64456[50], 23}, {NULL}};
const Segment SGI12890[] = {{&S64456[30], 12}, {NULL}};
const Segment SGI12891[] = {{&S64456[122], 3}, {NULL}};
const Segment SGI12892[] = {{&S64456[87], 6}, {NULL}};
const Segment SGI12893[] = {{&S64457[50], 11}, {NULL}};
const Segment SGI12894[] = {{&S64457[50], 22}, {NULL}};
const Segment SGI12895[] = {{&S64457[30], 12}, {NULL}};
const Segment SGI12896[] = {{&S64457[121], 3}, {NULL}};
const Segment SGI12897[] = {{&S64457[86], 6}, {NULL}};
const Segment SGI12898[] = {{&S64458[44], 11}, {NULL}};
const Segment SGI12899[] = {{&S64458[31], 12}, {NULL}};
const Segment SGI12900[] = {{&S65298[6], 4}, {NULL}};
const Segment SGI12901[] = {{&S65298[82], 5}, {NULL}};
const Segment SGI12902[] = {{&S65312[44], 12}, {NULL}};
const Segment SGI12903[] = {{&S65312[44], 34}, {NULL}};
const Segment SGI12904[] = {{&S65312[30], 6}, {NULL}};
const Segment SGI12905[] = {{&S65312[70], 3}, {NULL}};
const Segment SGI12906[] = {{&S65312[92], 5}, {NULL}};
const Segment SGI12907[] = {{&S65313[44], 12}, {NULL}};
const Segment SGI12908[] = {{&S65313[44], 34}, {NULL}};
const Segment SGI12909[] = {{&S65313[30], 6}, {NULL}};
const Segment SGI12910[] = {{&S65313[70], 3}, {NULL}};
const Segment SGI12911[] = {{&S65313[92], 5}, {NULL}};
const Segment SGI12912[] = {{&S65314[44], 12}, {NULL}};
const Segment SGI12913[] = {{&S65314[44], 34}, {NULL}};
const Segment SGI12914[] = {{&S65314[30], 6}, {NULL}};
const Segment SGI12915[] = {{&S65314[70], 3}, {NULL}};
const Segment SGI12916[] = {{&S65314[92], 5}, {NULL}};
const Segment SGI12917[] = {{&S64914[44], 4}, {NULL}};
const Segment SGI12918[] = {{&S64914[44], 25}, {NULL}};
const Segment SGI12919[] = {{&S64914[30], 6}, {NULL}};
const Segment SGI12920[] = {{&S64914[117], 3}, {NULL}};
const Segment SGI12921[] = {{&S64914[83], 5}, {NULL}};
const Segment SGI12922[] = {{&S64915[44], 4}, {NULL}};
const Segment SGI12923[] = {{&S64915[44], 20}, {NULL}};
const Segment SGI12924[] = {{&S64915[30], 6}, {NULL}};
const Segment SGI12925[] = {{&S64915[112], 3}, {NULL}};
const Segment SGI12926[] = {{&S64915[78], 5}, {NULL}};
const Segment SGI12927[] = {{&S64916[44], 11}, {NULL}};
const Segment SGI12928[] = {{&S64916[44], 25}, {NULL}};
const Segment SGI12929[] = {{&S64916[30], 6}, {NULL}};
const Segment SGI12930[] = {{&S64916[117], 3}, {NULL}};
const Segment SGI12931[] = {{&S64916[83], 5}, {NULL}};
const Segment SGI12932[] = {{&S64917[44], 4}, {NULL}};
const Segment SGI12933[] = {{&S64917[44], 17}, {NULL}};
const Segment SGI12934[] = {{&S64917[30], 6}, {NULL}};
const Segment SGI12935[] = {{&S64917[109], 3}, {NULL}};
const Segment SGI12936[] = {{&S64917[75], 5}, {NULL}};
const Segment SGI12937[] = {{&S64918[44], 4}, {NULL}};
const Segment SGI12938[] = {{&S64918[44], 24}, {NULL}};
const Segment SGI12939[] = {{&S64918[30], 6}, {NULL}};
const Segment SGI12940[] = {{&S64918[116], 3}, {NULL}};
const Segment SGI12941[] = {{&S64918[82], 5}, {NULL}};
const Segment SGI12942[] = {{&S64924[44], 4}, {NULL}};
const Segment SGI12943[] = {{&S64924[44], 15}, {NULL}};
const Segment SGI12944[] = {{&S64924[30], 6}, {NULL}};
const Segment SGI12945[] = {{&S64924[107], 3}, {NULL}};
const Segment SGI12946[] = {{&S64924[73], 5}, {NULL}};
const Segment SGI12947[] = {{&S64925[44], 11}, {NULL}};
const Segment SGI12948[] = {{&S64925[44], 22}, {NULL}};
const Segment SGI12949[] = {{&S64925[30], 6}, {NULL}};
const Segment SGI12950[] = {{&S64925[114], 3}, {NULL}};
const Segment SGI12951[] = {{&S64925[80], 5}, {NULL}};
const Segment SGI12952[] = {{&S65449[44], 10}, {NULL}};
const Segment SGI12953[] = {{&S65449[44], 21}, {NULL}};
const Segment SGI12954[] = {{&S65449[30], 6}, {NULL}};
const Segment SGI12955[] = {{&S65449[113], 3}, {NULL}};
const Segment SGI12956[] = {{&S65449[79], 5}, {NULL}};
const Segment SGI12957[] = {{&S65450[44], 10}, {NULL}};
const Segment SGI12958[] = {{&S65450[44], 21}, {NULL}};
const Segment SGI12959[] = {{&S65450[30], 6}, {NULL}};
const Segment SGI12960[] = {{&S65450[113], 3}, {NULL}};
const Segment SGI12961[] = {{&S65450[79], 5}, {NULL}};
const Segment SGI12962[] = {{&S65452[38], 10}, {NULL}};
const Segment SGI12963[] = {{&S65452[31], 6}, {NULL}};
const Segment SGI12964[] = {{&S65453[42], 10}, {NULL}};
const Segment SGI12965[] = {{&S65453[30], 6}, {NULL}};
const Segment SGI12966[] = {{&S65453[111], 3}, {NULL}};
const Segment SGI12967[] = {{&S65453[77], 5}, {NULL}};
const Segment SGI12968[] = {{&S65455[38], 10}, {NULL}};
const Segment SGI12969[] = {{&S65455[31], 6}, {NULL}};
const Segment SGI12970[] = {{&S65981[79], 6}, {NULL}};
const Segment SGI12971[] = {{&S65982[8], 8}, {NULL}};
const Segment SGI12972[] = {{&S65983[5], 8}, {NULL}};
const Segment SGI12973[] = {{&S66205[6], 4}, {NULL}};
const Segment SGI12974[] = {{&S66205[32], 7}, {NULL}};
const Segment SGI12975[] = {{&S66205[79], 5}, {NULL}};
const Segment SGI12976[] = {{&S66206[6], 4}, {NULL}};
const Segment SGI12977[] = {{&S66206[32], 7}, {NULL}};
const Segment SGI12978[] = {{&S66206[79], 5}, {NULL}};
const Segment SGI12979[] = {{&S66207[6], 4}, {NULL}};
const Segment SGI12980[] = {{&S66207[32], 7}, {NULL}};
const Segment SGI12981[] = {{&S66207[79], 5}, {NULL}};
const Segment SGI12982[] = {{&S66208[6], 4}, {NULL}};
const Segment SGI12983[] = {{&S66208[32], 7}, {NULL}};
const Segment SGI12984[] = {{&S66208[82], 5}, {NULL}};
const Segment SGI12985[] = {{&S68846[42], 8}, {NULL}};
const Segment SGI12986[] = {{&S68846[42], 19}, {NULL}};
const Segment SGI12987[] = {{&S68846[30], 4}, {NULL}};
const Segment SGI12988[] = {{&S68846[110], 3}, {NULL}};
const Segment SGI12989[] = {{&S68846[75], 6}, {NULL}};
const Segment SGI12990[] = {{&S68846[130], 8}, {NULL}};
const Segment SGI12991[] = {{&S68847[42], 8}, {NULL}};
const Segment SGI12992[] = {{&S68847[42], 19}, {NULL}};
const Segment SGI12993[] = {{&S68847[30], 4}, {NULL}};
const Segment SGI12994[] = {{&S68847[110], 3}, {NULL}};
const Segment SGI12995[] = {{&S68847[130], 8}, {NULL}};
const Segment SGI12996[] = {{&S65775[44], 10}, {NULL}};
const Segment SGI12997[] = {{&S65775[44], 21}, {NULL}};
const Segment SGI12998[] = {{&S65775[30], 6}, {NULL}};
const Segment SGI12999[] = {{&S65775[113], 3}, {NULL}};
const Segment SGI13000[] = {{&S65775[79], 5}, {NULL}};
const Segment SGI13001[] = {{&S65778[38], 10}, {NULL}};
const Segment SGI13002[] = {{&S65778[31], 6}, {NULL}};
const Segment SGI13003[] = {{&S65210[42], 7}, {NULL}};
const Segment SGI13004[] = {{&S65210[42], 18}, {NULL}};
const Segment SGI13005[] = {{&S65210[30], 4}, {NULL}};
const Segment SGI13006[] = {{&S65210[108], 3}, {NULL}};
const Segment SGI13007[] = {{&S65210[74], 5}, {NULL}};
const Segment SGI13008[] = {{&S65211[42], 7}, {NULL}};
const Segment SGI13009[] = {{&S65211[42], 18}, {NULL}};
const Segment SGI13010[] = {{&S65211[30], 4}, {NULL}};
const Segment SGI13011[] = {{&S65211[77], 4}, {NULL}};
const Segment SGI13012[] = {{&S65211[77], 3}, {NULL}};
const Segment SGI13013[] = {{&S65212[36], 7}, {NULL}};
const Segment SGI13014[] = {{&S65212[31], 4}, {NULL}};
const Segment SGI13015[] = {{&S65968[44], 14}, {NULL}};
const Segment SGI13016[] = {{&S65968[44], 25}, {NULL}};
const Segment SGI13017[] = {{&S65968[30], 6}, {NULL}};
const Segment SGI13018[] = {{&S65968[117], 3}, {NULL}};
const Segment SGI13019[] = {{&S65968[83], 5}, {NULL}};
const Segment SGI13020[] = {{&S65969[42], 14}, {NULL}};
const Segment SGI13021[] = {{&S65969[30], 6}, {NULL}};
const Segment SGI13022[] = {{&S65969[115], 3}, {NULL}};
const Segment SGI13023[] = {{&S65969[81], 5}, {NULL}};
const Segment SGI13024[] = {{&S65970[38], 14}, {NULL}};
const Segment SGI13025[] = {{&S65970[31], 6}, {NULL}};
const Segment SGI13026[] = {{&S65204[50], 9}, {NULL}};
const Segment SGI13027[] = {{&S65204[50], 20}, {NULL}};
const Segment SGI13028[] = {{&S65204[30], 12}, {NULL}};
const Segment SGI13029[] = {{&S65204[119], 3}, {NULL}};
const Segment SGI13030[] = {{&S65204[84], 6}, {NULL}};
const Segment SGI13031[] = {{&S66009[44], 11}, {NULL}};
const Segment SGI13032[] = {{&S66009[44], 22}, {NULL}};
const Segment SGI13033[] = {{&S66009[30], 6}, {NULL}};
const Segment SGI13034[] = {{&S66009[114], 3}, {NULL}};
const Segment SGI13035[] = {{&S66009[80], 5}, {NULL}};
const Segment SGI13036[] = {{&S66010[38], 11}, {NULL}};
const Segment SGI13037[] = {{&S66010[31], 6}, {NULL}};
const Segment SGI13038[] = {{&S65297[6], 4}, {NULL}};
const Segment SGI13039[] = {{&S65297[83], 5}, {NULL}};
const Segment SGI13040[] = {{&S65310[42], 13}, {NULL}};
const Segment SGI13041[] = {{&S65310[42], 24}, {NULL}};
const Segment SGI13042[] = {{&S65310[30], 4}, {NULL}};
const Segment SGI13043[] = {{&S65310[114], 3}, {NULL}};
const Segment SGI13044[] = {{&S65310[80], 5}, {NULL}};
const Segment SGI13045[] = {{&S65311[42], 10}, {NULL}};
const Segment SGI13046[] = {{&S65311[42], 21}, {NULL}};
const Segment SGI13047[] = {{&S65311[30], 4}, {NULL}};
const Segment SGI13048[] = {{&S65311[111], 3}, {NULL}};
const Segment SGI13049[] = {{&S65311[77], 5}, {NULL}};
const Segment SGI13050[] = {{&S69086[42], 9}, {NULL}};
const Segment SGI13051[] = {{&S69086[42], 22}, {NULL}};
const Segment SGI13052[] = {{&S69086[30], 4}, {NULL}};
const Segment SGI13053[] = {{&S69086[113], 3}, {NULL}};
const Segment SGI13054[] = {{&S69086[78], 6}, {NULL}};
const Segment SGI13055[] = {{&S69086[133], 8}, {NULL}};
const Segment SGI13056[] = {{&S65448[42], 10}, {NULL}};
const Segment SGI13057[] = {{&S65448[42], 21}, {NULL}};
const Segment SGI13058[] = {{&S65448[30], 4}, {NULL}};
const Segment SGI13059[] = {{&S65448[111], 3}, {NULL}};
const Segment SGI13060[] = {{&S65448[77], 5}, {NULL}};
const Segment SGI13061[] = {{&S65451[42], 10}, {NULL}};
const Segment SGI13062[] = {{&S65451[42], 21}, {NULL}};
const Segment SGI13063[] = {{&S65451[30], 4}, {NULL}};
const Segment SGI13064[] = {{&S65451[111], 3}, {NULL}};
const Segment SGI13065[] = {{&S65451[77], 5}, {NULL}};
const Segment SGI13066[] = {{&S65454[36], 10}, {NULL}};
const Segment SGI13067[] = {{&S65454[31], 4}, {NULL}};
const Segment SGI13068[] = {{&S65456[40], 10}, {NULL}};
const Segment SGI13069[] = {{&S65456[30], 4}, {NULL}};
const Segment SGI13070[] = {{&S65456[109], 3}, {NULL}};
const Segment SGI13071[] = {{&S65456[75], 5}, {NULL}};
const Segment SGI13072[] = {{&S66230[44], 10}, {NULL}};
const Segment SGI13073[] = {{&S66230[44], 21}, {NULL}};
const Segment SGI13074[] = {{&S66230[30], 6}, {NULL}};
const Segment SGI13075[] = {{&S66230[113], 3}, {NULL}};
const Segment SGI13076[] = {{&S66230[79], 5}, {NULL}};
const Segment SGI13077[] = {{&S65517[56], 9}, {NULL}};
const Segment SGI13078[] = {{&S65517[56], 21}, {NULL}};
const Segment SGI13079[] = {{&S65517[30], 18}, {NULL}};
const Segment SGI13080[] = {{&S65517[126], 3}, {NULL}};
const Segment SGI13081[] = {{&S65517[91], 6}, {NULL}};
const Segment SGI13082[] = {{&S65518[56], 9}, {NULL}};
const Segment SGI13083[] = {{&S65518[56], 21}, {NULL}};
const Segment SGI13084[] = {{&S65518[30], 18}, {NULL}};
const Segment SGI13085[] = {{&S65518[126], 3}, {NULL}};
const Segment SGI13086[] = {{&S65518[91], 6}, {NULL}};
const Segment SGI13087[] = {{&S69286[42], 13}, {NULL}};
const Segment SGI13088[] = {{&S69286[42], 24}, {NULL}};
const Segment SGI13089[] = {{&S69286[30], 4}, {NULL}};
const Segment SGI13090[] = {{&S69286[115], 3}, {NULL}};
const Segment SGI13091[] = {{&S69286[80], 6}, {NULL}};
const Segment SGI13092[] = {{&S69286[135], 8}, {NULL}};
const Segment SGI13093[] = {{&S69287[42], 13}, {NULL}};
const Segment SGI13094[] = {{&S69287[42], 24}, {NULL}};
const Segment SGI13095[] = {{&S69287[30], 4}, {NULL}};
const Segment SGI13096[] = {{&S69287[115], 3}, {NULL}};
const Segment SGI13097[] = {{&S69287[80], 6}, {NULL}};
const Segment SGI13098[] = {{&S69287[135], 8}, {NULL}};
const Segment SGI13099[] = {{&S69288[42], 13}, {NULL}};
const Segment SGI13100[] = {{&S69288[42], 23}, {NULL}};
const Segment SGI13101[] = {{&S69288[30], 4}, {NULL}};
const Segment SGI13102[] = {{&S69288[114], 3}, {NULL}};
const Segment SGI13103[] = {{&S69288[79], 6}, {NULL}};
const Segment SGI13104[] = {{&S69288[134], 8}, {NULL}};
const Segment SGI13105[] = {{&S69289[42], 13}, {NULL}};
const Segment SGI13106[] = {{&S69289[42], 24}, {NULL}};
const Segment SGI13107[] = {{&S69289[30], 4}, {NULL}};
const Segment SGI13108[] = {{&S69289[115], 3}, {NULL}};
const Segment SGI13109[] = {{&S69289[80], 6}, {NULL}};
const Segment SGI13110[] = {{&S69289[135], 8}, {NULL}};
const Segment SGI13111[] = {{&S69290[40], 13}, {NULL}};
const Segment SGI13112[] = {{&S69290[30], 4}, {NULL}};
const Segment SGI13113[] = {{&S69290[113], 3}, {NULL}};
const Segment SGI13114[] = {{&S69290[78], 6}, {NULL}};
const Segment SGI13115[] = {{&S69290[133], 8}, {NULL}};
const Segment SGI13116[] = {{&S69291[42], 13}, {NULL}};
const Segment SGI13117[] = {{&S69291[42], 24}, {NULL}};
const Segment SGI13118[] = {{&S69291[30], 4}, {NULL}};
const Segment SGI13119[] = {{&S69291[115], 3}, {NULL}};
const Segment SGI13120[] = {{&S69291[80], 5}, {NULL}};
const Segment SGI13121[] = {{&S65610[50], 27}, {NULL}};
const Segment SGI13122[] = {{&S65610[50], 38}, {NULL}};
const Segment SGI13123[] = {{&S65610[30], 12}, {NULL}};
const Segment SGI13124[] = {{&S65610[137], 3}, {NULL}};
const Segment SGI13125[] = {{&S65610[102], 6}, {NULL}};
const Segment SGI13126[] = {{&S65611[50], 27}, {NULL}};
const Segment SGI13127[] = {{&S65611[50], 38}, {NULL}};
const Segment SGI13128[] = {{&S65611[30], 12}, {NULL}};
const Segment SGI13129[] = {{&S65611[138], 3}, {NULL}};
const Segment SGI13130[] = {{&S65611[102], 7}, {NULL}};
const Segment SGI13131[] = {{&S65611[158], 8}, {NULL}};
const Segment SGI13132[] = {{&S65612[44], 27}, {NULL}};
const Segment SGI13133[] = {{&S65612[31], 12}, {NULL}};
const Segment SGI13134[] = {{&S65555[42], 6}, {NULL}};
const Segment SGI13135[] = {{&S65555[42], 17}, {NULL}};
const Segment SGI13136[] = {{&S65555[30], 4}, {NULL}};
const Segment SGI13137[] = {{&S65555[107], 3}, {NULL}};
const Segment SGI13138[] = {{&S65555[73], 5}, {NULL}};
const Segment SGI13139[] = {{&S66951[125], 3}, {NULL}};
const Segment SGI13140[] = {{&S66951[90], 6}, {NULL}};
const Segment SGI13141[] = {{&S69397[44], 8}, {NULL}};
const Segment SGI13142[] = {{&S69397[44], 20}, {NULL}};
const Segment SGI13143[] = {{&S69397[30], 6}, {NULL}};
const Segment SGI13144[] = {{&S69397[113], 3}, {NULL}};
const Segment SGI13145[] = {{&S69397[78], 6}, {NULL}};
const Segment SGI13146[] = {{&S69398[44], 8}, {NULL}};
const Segment SGI13147[] = {{&S69398[44], 20}, {NULL}};
const Segment SGI13148[] = {{&S69398[30], 6}, {NULL}};
const Segment SGI13149[] = {{&S69398[113], 3}, {NULL}};
const Segment SGI13150[] = {{&S69398[78], 6}, {NULL}};
const Segment SGI13151[] = {{&S68407[42], 11}, {NULL}};
const Segment SGI13152[] = {{&S68407[42], 33}, {NULL}};
const Segment SGI13153[] = {{&S68407[30], 4}, {NULL}};
const Segment SGI13154[] = {{&S68407[124], 3}, {NULL}};
const Segment SGI13155[] = {{&S68407[89], 6}, {NULL}};
const Segment SGI13156[] = {{&S68408[44], 11}, {NULL}};
const Segment SGI13157[] = {{&S68408[44], 33}, {NULL}};
const Segment SGI13158[] = {{&S68408[30], 6}, {NULL}};
const Segment SGI13159[] = {{&S68408[74], 3}, {NULL}};
const Segment SGI13160[] = {{&S68408[91], 6}, {NULL}};
const Segment SGI13161[] = {{&S68409[44], 11}, {NULL}};
const Segment SGI13162[] = {{&S68409[44], 33}, {NULL}};
const Segment SGI13163[] = {{&S68409[30], 6}, {NULL}};
const Segment SGI13164[] = {{&S68409[126], 3}, {NULL}};
const Segment SGI13165[] = {{&S68409[91], 6}, {NULL}};
const Segment SGI13166[] = {{&S68410[42], 11}, {NULL}};
const Segment SGI13167[] = {{&S68410[42], 34}, {NULL}};
const Segment SGI13168[] = {{&S68410[30], 4}, {NULL}};
const Segment SGI13169[] = {{&S68410[125], 3}, {NULL}};
const Segment SGI13170[] = {{&S68410[90], 6}, {NULL}};
const Segment SGI13171[] = {{&S65827[50], 19}, {NULL}};
const Segment SGI13172[] = {{&S65827[50], 30}, {NULL}};
const Segment SGI13173[] = {{&S65827[30], 12}, {NULL}};
const Segment SGI13174[] = {{&S65827[129], 3}, {NULL}};
const Segment SGI13175[] = {{&S65827[94], 6}, {NULL}};
const Segment SGI13176[] = {{&S65705[42], 16}, {NULL}};
const Segment SGI13177[] = {{&S65705[30], 4}, {NULL}};
const Segment SGI13178[] = {{&S65705[106], 3}, {NULL}};
const Segment SGI13179[] = {{&S65705[72], 5}, {NULL}};
const Segment SGI13180[] = {{&S65959[50], 14}, {NULL}};
const Segment SGI13181[] = {{&S65959[50], 25}, {NULL}};
const Segment SGI13182[] = {{&S65959[30], 12}, {NULL}};
const Segment SGI13183[] = {{&S65959[124], 3}, {NULL}};
const Segment SGI13184[] = {{&S65959[89], 6}, {NULL}};
const Segment SGI13185[] = {{&S67130[36], 7}, {NULL}};
const Segment SGI13186[] = {{&S67130[30], 5}, {NULL}};
const Segment SGI13187[] = {{&S67130[107], 3}, {NULL}};
const Segment SGI13188[] = {{&S67130[73], 5}, {NULL}};
const Segment SGI13189[] = {{&S65960[50], 14}, {NULL}};
const Segment SGI13190[] = {{&S65960[50], 25}, {NULL}};
const Segment SGI13191[] = {{&S65960[30], 12}, {NULL}};
const Segment SGI13192[] = {{&S65960[92], 4}, {NULL}};
const Segment SGI13193[] = {{&S65960[92], 3}, {NULL}};
const Segment SGI13194[] = {{&S67131[36], 7}, {NULL}};
const Segment SGI13195[] = {{&S67131[30], 5}, {NULL}};
const Segment SGI13196[] = {{&S67131[107], 3}, {NULL}};
const Segment SGI13197[] = {{&S67131[73], 5}, {NULL}};
const Segment SGI13198[] = {{&S65961[44], 14}, {NULL}};
const Segment SGI13199[] = {{&S65961[31], 12}, {NULL}};
const Segment SGI13200[] = {{&S67132[36], 7}, {NULL}};
const Segment SGI13201[] = {{&S67132[30], 5}, {NULL}};
const Segment SGI13202[] = {{&S67132[107], 3}, {NULL}};
const Segment SGI13203[] = {{&S67132[73], 5}, {NULL}};
const Segment SGI13204[] = {{&S67133[36], 7}, {NULL}};
const Segment SGI13205[] = {{&S67133[30], 5}, {NULL}};
const Segment SGI13206[] = {{&S67133[107], 3}, {NULL}};
const Segment SGI13207[] = {{&S67133[73], 5}, {NULL}};
const Segment SGI13208[] = {{&S67134[36], 7}, {NULL}};
const Segment SGI13209[] = {{&S67134[30], 5}, {NULL}};
const Segment SGI13210[] = {{&S67134[107], 3}, {NULL}};
const Segment SGI13211[] = {{&S67134[73], 5}, {NULL}};
const Segment SGI13212[] = {{&S67135[36], 7}, {NULL}};
const Segment SGI13213[] = {{&S67135[30], 5}, {NULL}};
const Segment SGI13214[] = {{&S67135[107], 3}, {NULL}};
const Segment SGI13215[] = {{&S67135[73], 5}, {NULL}};
const Segment SGI13216[] = {{&S67136[36], 7}, {NULL}};
const Segment SGI13217[] = {{&S67136[30], 5}, {NULL}};
const Segment SGI13218[] = {{&S67136[107], 3}, {NULL}};
const Segment SGI13219[] = {{&S67136[73], 5}, {NULL}};
const Segment SGI13220[] = {{&S67137[36], 7}, {NULL}};
const Segment SGI13221[] = {{&S67137[30], 5}, {NULL}};
const Segment SGI13222[] = {{&S67137[107], 3}, {NULL}};
const Segment SGI13223[] = {{&S67137[73], 5}, {NULL}};
const Segment SGI13224[] = {{&S67138[36], 7}, {NULL}};
const Segment SGI13225[] = {{&S67138[30], 5}, {NULL}};
const Segment SGI13226[] = {{&S67138[107], 3}, {NULL}};
const Segment SGI13227[] = {{&S67138[73], 5}, {NULL}};
const Segment SGI13228[] = {{&S67139[36], 7}, {NULL}};
const Segment SGI13229[] = {{&S67139[30], 5}, {NULL}};
const Segment SGI13230[] = {{&S67139[107], 3}, {NULL}};
const Segment SGI13231[] = {{&S67139[73], 5}, {NULL}};
const Segment SGI13232[] = {{&S66696[44], 6}, {NULL}};
const Segment SGI13233[] = {{&S66696[44], 17}, {NULL}};
const Segment SGI13234[] = {{&S66696[30], 6}, {NULL}};
const Segment SGI13235[] = {{&S66696[109], 3}, {NULL}};
const Segment SGI13236[] = {{&S66696[75], 5}, {NULL}};
const Segment SGI13237[] = {{&S66044[42], 12}, {NULL}};
const Segment SGI13238[] = {{&S66044[42], 23}, {NULL}};
const Segment SGI13239[] = {{&S66044[30], 4}, {NULL}};
const Segment SGI13240[] = {{&S66044[113], 3}, {NULL}};
const Segment SGI13241[] = {{&S66044[79], 5}, {NULL}};
const Segment SGI13242[] = {{&S66047[42], 12}, {NULL}};
const Segment SGI13243[] = {{&S66047[42], 23}, {NULL}};
const Segment SGI13244[] = {{&S66047[30], 4}, {NULL}};
const Segment SGI13245[] = {{&S66047[82], 4}, {NULL}};
const Segment SGI13246[] = {{&S66047[82], 3}, {NULL}};
const Segment SGI13247[] = {{&S66262[42], 14}, {NULL}};
const Segment SGI13248[] = {{&S66262[42], 25}, {NULL}};
const Segment SGI13249[] = {{&S66262[30], 4}, {NULL}};
const Segment SGI13250[] = {{&S66262[117], 3}, {NULL}};
const Segment SGI13251[] = {{&S66262[82], 6}, {NULL}};
const Segment SGI13252[] = {{&S66263[42], 14}, {NULL}};
const Segment SGI13253[] = {{&S66263[42], 25}, {NULL}};
const Segment SGI13254[] = {{&S66263[30], 4}, {NULL}};
const Segment SGI13255[] = {{&S66263[116], 3}, {NULL}};
const Segment SGI13256[] = {{&S66263[81], 6}, {NULL}};
const Segment SGI13257[] = {{&S66264[42], 14}, {NULL}};
const Segment SGI13258[] = {{&S66264[42], 25}, {NULL}};
const Segment SGI13259[] = {{&S66264[30], 4}, {NULL}};
const Segment SGI13260[] = {{&S66264[116], 3}, {NULL}};
const Segment SGI13261[] = {{&S66264[81], 6}, {NULL}};
const Segment SGI13262[] = {{&S66266[42], 16}, {NULL}};
const Segment SGI13263[] = {{&S66266[42], 27}, {NULL}};
const Segment SGI13264[] = {{&S66266[30], 4}, {NULL}};
const Segment SGI13265[] = {{&S66266[118], 3}, {NULL}};
const Segment SGI13266[] = {{&S66266[83], 6}, {NULL}};
const Segment SGI13267[] = {{&S66267[50], 9}, {NULL}};
const Segment SGI13268[] = {{&S66267[50], 20}, {NULL}};
const Segment SGI13269[] = {{&S66267[30], 12}, {NULL}};
const Segment SGI13270[] = {{&S66267[119], 3}, {NULL}};
const Segment SGI13271[] = {{&S66267[84], 6}, {NULL}};
const Segment SGI13272[] = {{&S66268[50], 9}, {NULL}};
const Segment SGI13273[] = {{&S66268[50], 20}, {NULL}};
const Segment SGI13274[] = {{&S66268[30], 12}, {NULL}};
const Segment SGI13275[] = {{&S66268[120], 3}, {NULL}};
const Segment SGI13276[] = {{&S66268[84], 7}, {NULL}};
const Segment SGI13277[] = {{&S66268[140], 8}, {NULL}};
const Segment SGI13278[] = {{&S66269[44], 9}, {NULL}};
const Segment SGI13279[] = {{&S66269[31], 12}, {NULL}};
const Segment SGI13280[] = {{&S66270[50], 9}, {NULL}};
const Segment SGI13281[] = {{&S66270[50], 20}, {NULL}};
const Segment SGI13282[] = {{&S66270[30], 12}, {NULL}};
const Segment SGI13283[] = {{&S66270[87], 4}, {NULL}};
const Segment SGI13284[] = {{&S66270[87], 3}, {NULL}};
const Segment SGI13285[] = {{&S66229[42], 10}, {NULL}};
const Segment SGI13286[] = {{&S66229[42], 21}, {NULL}};
const Segment SGI13287[] = {{&S66229[30], 4}, {NULL}};
const Segment SGI13288[] = {{&S66229[111], 3}, {NULL}};
const Segment SGI13289[] = {{&S66229[77], 5}, {NULL}};
const Segment SGI13290[] = {{&S66232[36], 10}, {NULL}};
const Segment SGI13291[] = {{&S66232[31], 4}, {NULL}};
const Segment SGI13292[] = {{&S72854[8], 7}, {NULL}};
const Segment SGI13293[] = {{&S72854[0], 45}, {NULL}};
const Segment SGI13294[] = {{&S72855[8], 7}, {NULL}};
const Segment SGI13295[] = {{&S72855[0], 85}, {NULL}};
const Segment SGI13296[] = {{&S72856[8], 6}, {&S72856[8], 6}, {NULL}};
const Segment SGI13297[] = {{&S72856[0], 170}, {NULL}};
const Segment SGI13298[] = {{&S72857[8], 6}, {NULL}};
const Segment SGI13299[] = {{&S72857[0], 84}, {NULL}};
const Segment SGI13300[] = {{&S72858[8], 7}, {NULL}};
const Segment SGI13301[] = {{&S72858[0], 85}, {NULL}};
const Segment SGI13302[] = {{&S66852[45], 4}, {NULL}};
const Segment SGI13303[] = {{&S66852[105], 3}, {NULL}};
const Segment SGI13304[] = {{&S66852[94], 3}, {NULL}};
const Segment SGI13305[] = {{&S66852[71], 6}, {NULL}};
const Segment SGI13306[] = {{&S66853[37], 4}, {NULL}};
const Segment SGI13307[] = {{&S66853[56], 3}, {NULL}};
const Segment SGI13308[] = {{&S67366[44], 10}, {NULL}};
const Segment SGI13309[] = {{&S67366[44], 21}, {NULL}};
const Segment SGI13310[] = {{&S67366[30], 6}, {NULL}};
const Segment SGI13311[] = {{&S67366[113], 3}, {NULL}};
const Segment SGI13312[] = {{&S67366[79], 5}, {NULL}};
const Segment SGI13313[] = {{&S67367[44], 10}, {NULL}};
const Segment SGI13314[] = {{&S67367[44], 21}, {NULL}};
const Segment SGI13315[] = {{&S67367[30], 6}, {NULL}};
const Segment SGI13316[] = {{&S67367[113], 3}, {NULL}};
const Segment SGI13317[] = {{&S67367[79], 5}, {NULL}};
const Segment SGI13318[] = {{&S67368[44], 10}, {NULL}};
const Segment SGI13319[] = {{&S67368[44], 21}, {NULL}};
const Segment SGI13320[] = {{&S67368[30], 6}, {NULL}};
const Segment SGI13321[] = {{&S67368[113], 3}, {NULL}};
const Segment SGI13322[] = {{&S67368[79], 5}, {NULL}};
const Segment SGI13323[] = {{&S67369[44], 10}, {NULL}};
const Segment SGI13324[] = {{&S67369[44], 21}, {NULL}};
const Segment SGI13325[] = {{&S67369[30], 6}, {NULL}};
const Segment SGI13326[] = {{&S67369[113], 3}, {NULL}};
const Segment SGI13327[] = {{&S67369[79], 5}, {NULL}};
const Segment SGI13328[] = {{&S67370[44], 10}, {NULL}};
const Segment SGI13329[] = {{&S67370[44], 21}, {NULL}};
const Segment SGI13330[] = {{&S67370[30], 6}, {NULL}};
const Segment SGI13331[] = {{&S67370[113], 3}, {NULL}};
const Segment SGI13332[] = {{&S67370[79], 5}, {NULL}};
const Segment SGI13333[] = {{&S67371[44], 10}, {NULL}};
const Segment SGI13334[] = {{&S67371[44], 21}, {NULL}};
const Segment SGI13335[] = {{&S67371[30], 6}, {NULL}};
const Segment SGI13336[] = {{&S67371[113], 3}, {NULL}};
const Segment SGI13337[] = {{&S67371[79], 5}, {NULL}};
const Segment SGI13338[] = {{&S67373[44], 10}, {NULL}};
const Segment SGI13339[] = {{&S67373[44], 21}, {NULL}};
const Segment SGI13340[] = {{&S67373[30], 6}, {NULL}};
const Segment SGI13341[] = {{&S67373[113], 3}, {NULL}};
const Segment SGI13342[] = {{&S67373[79], 5}, {NULL}};
const Segment SGI13343[] = {{&S67374[38], 10}, {NULL}};
const Segment SGI13344[] = {{&S67374[31], 6}, {NULL}};
const Segment SGI13345[] = {{&S66744[42], 12}, {NULL}};
const Segment SGI13346[] = {{&S66744[42], 24}, {NULL}};
const Segment SGI13347[] = {{&S66744[30], 4}, {NULL}};
const Segment SGI13348[] = {{&S66744[114], 3}, {NULL}};
const Segment SGI13349[] = {{&S66744[80], 5}, {NULL}};
const Segment SGI13350[] = {{&S66745[36], 12}, {NULL}};
const Segment SGI13351[] = {{&S66745[31], 4}, {NULL}};
const Segment SGI13352[] = {{&S67109[50], 11}, {NULL}};
const Segment SGI13353[] = {{&S67109[50], 23}, {NULL}};
const Segment SGI13354[] = {{&S67109[30], 12}, {NULL}};
const Segment SGI13355[] = {{&S67109[122], 3}, {NULL}};
const Segment SGI13356[] = {{&S67109[87], 6}, {NULL}};
const Segment SGI13357[] = {{&S67592[44], 10}, {NULL}};
const Segment SGI13358[] = {{&S67110[44], 11}, {NULL}};
const Segment SGI13359[] = {{&S67592[44], 22}, {NULL}};
const Segment SGI13360[] = {{&S67110[31], 12}, {NULL}};
const Segment SGI13361[] = {{&S67592[30], 6}, {NULL}};
const Segment SGI13362[] = {{&S67592[114], 3}, {NULL}};
const Segment SGI13363[] = {{&S67592[80], 5}, {NULL}};
const Segment SGI13364[] = {{&S67593[44], 10}, {NULL}};
const Segment SGI13365[] = {{&S67593[44], 22}, {NULL}};
const Segment SGI13366[] = {{&S67593[30], 6}, {NULL}};
const Segment SGI13367[] = {{&S67593[114], 3}, {NULL}};
const Segment SGI13368[] = {{&S67593[80], 5}, {NULL}};
const Segment SGI13369[] = {{&S67594[44], 10}, {NULL}};
const Segment SGI13370[] = {{&S67594[44], 22}, {NULL}};
const Segment SGI13371[] = {{&S67594[30], 6}, {NULL}};
const Segment SGI13372[] = {{&S67594[114], 3}, {NULL}};
const Segment SGI13373[] = {{&S67594[80], 5}, {NULL}};
const Segment SGI13374[] = {{&S67124[50], 7}, {NULL}};
const Segment SGI13375[] = {{&S67124[50], 18}, {NULL}};
const Segment SGI13376[] = {{&S67124[30], 12}, {NULL}};
const Segment SGI13377[] = {{&S67124[117], 3}, {NULL}};
const Segment SGI13378[] = {{&S67124[82], 6}, {NULL}};
const Segment SGI13379[] = {{&S67125[50], 13}, {NULL}};
const Segment SGI13380[] = {{&S67125[50], 24}, {NULL}};
const Segment SGI13381[] = {{&S67125[30], 12}, {NULL}};
const Segment SGI13382[] = {{&S67125[123], 3}, {NULL}};
const Segment SGI13383[] = {{&S67125[88], 6}, {NULL}};
const Segment SGI13384[] = {{&S67126[44], 7}, {NULL}};
const Segment SGI13385[] = {{&S67126[31], 12}, {NULL}};
const Segment SGI13386[] = {{&S70600[42], 11}, {NULL}};
const Segment SGI13387[] = {{&S70600[42], 24}, {NULL}};
const Segment SGI13388[] = {{&S70600[30], 4}, {NULL}};
const Segment SGI13389[] = {{&S70600[115], 3}, {NULL}};
const Segment SGI13390[] = {{&S70600[80], 6}, {NULL}};
const Segment SGI13391[] = {{&S70600[135], 8}, {NULL}};
const Segment SGI13392[] = {{&S70601[9], 1}, {NULL}};
const Segment SGI13393[] = {{&S70601[52], 17}, {NULL}};
const Segment SGI13394[] = {{&S70601[40], 4}, {NULL}};
const Segment SGI13395[] = {{&S70601[118], 3}, {NULL}};
const Segment SGI13396[] = {{&S70601[83], 6}, {NULL}};
const Segment SGI13397[] = {{&S70601[138], 8}, {NULL}};
const Segment SGI13398[] = {{&S70602[9], 1}, {NULL}};
const Segment SGI13399[] = {{&S70602[52], 14}, {NULL}};
const Segment SGI13400[] = {{&S70602[40], 4}, {NULL}};
const Segment SGI13401[] = {{&S70602[115], 3}, {NULL}};
const Segment SGI13402[] = {{&S70602[80], 6}, {NULL}};
const Segment SGI13403[] = {{&S70602[135], 8}, {NULL}};
const Segment SGI13404[] = {{&S70603[42], 11}, {NULL}};
const Segment SGI13405[] = {{&S70603[42], 24}, {NULL}};
const Segment SGI13406[] = {{&S70603[30], 4}, {NULL}};
const Segment SGI13407[] = {{&S70603[115], 3}, {NULL}};
const Segment SGI13408[] = {{&S70603[80], 6}, {NULL}};
const Segment SGI13409[] = {{&S70603[135], 8}, {NULL}};
const Segment SGI13410[] = {{&S70604[42], 11}, {NULL}};
const Segment SGI13411[] = {{&S70604[42], 22}, {NULL}};
const Segment SGI13412[] = {{&S70604[30], 4}, {NULL}};
const Segment SGI13413[] = {{&S70604[113], 3}, {NULL}};
const Segment SGI13414[] = {{&S70604[78], 6}, {NULL}};
const Segment SGI13415[] = {{&S70604[133], 8}, {NULL}};
const Segment SGI13416[] = {{&S70605[42], 11}, {NULL}};
const Segment SGI13417[] = {{&S70605[42], 24}, {NULL}};
const Segment SGI13418[] = {{&S70605[30], 4}, {NULL}};
const Segment SGI13419[] = {{&S70605[115], 3}, {NULL}};
const Segment SGI13420[] = {{&S70605[80], 6}, {NULL}};
const Segment SGI13421[] = {{&S70605[135], 8}, {NULL}};
const Segment SGI13422[] = {{&S70606[9], 1}, {NULL}};
const Segment SGI13423[] = {{&S70606[52], 14}, {NULL}};
const Segment SGI13424[] = {{&S70606[40], 4}, {NULL}};
const Segment SGI13425[] = {{&S70606[115], 3}, {NULL}};
const Segment SGI13426[] = {{&S70606[80], 5}, {NULL}};
const Segment SGI13427[] = {{&S70606[135], 8}, {NULL}};
const Segment SGI13428[] = {{&S67711[44], 11}, {NULL}};
const Segment SGI13429[] = {{&S67711[44], 22}, {NULL}};
const Segment SGI13430[] = {{&S67711[30], 6}, {NULL}};
const Segment SGI13431[] = {{&S67711[114], 3}, {NULL}};
const Segment SGI13432[] = {{&S67711[80], 5}, {NULL}};
const Segment SGI13433[] = {{&S67033[42], 10}, {NULL}};
const Segment SGI13434[] = {{&S67033[42], 19}, {NULL}};
const Segment SGI13435[] = {{&S67033[30], 4}, {NULL}};
const Segment SGI13436[] = {{&S67033[109], 3}, {NULL}};
const Segment SGI13437[] = {{&S67033[75], 5}, {NULL}};
const Segment SGI13438[] = {{&S67034[36], 10}, {NULL}};
const Segment SGI13439[] = {{&S67034[31], 4}, {NULL}};
const Segment SGI13440[] = {{&S67205[50], 23}, {NULL}};
const Segment SGI13441[] = {{&S67205[50], 34}, {NULL}};
const Segment SGI13442[] = {{&S67205[30], 12}, {NULL}};
const Segment SGI13443[] = {{&S67205[133], 3}, {NULL}};
const Segment SGI13444[] = {{&S67205[98], 6}, {NULL}};
const Segment SGI13445[] = {{&S67206[50], 23}, {NULL}};
const Segment SGI13446[] = {{&S67206[50], 34}, {NULL}};
const Segment SGI13447[] = {{&S67206[30], 12}, {NULL}};
const Segment SGI13448[] = {{&S67206[133], 3}, {NULL}};
const Segment SGI13449[] = {{&S67206[98], 6}, {NULL}};
const Segment SGI13450[] = {{&S68378[8], 8}, {NULL}};
const Segment SGI13451[] = {{&S67763[44], 9}, {NULL}};
const Segment SGI13452[] = {{&S67763[44], 21}, {NULL}};
const Segment SGI13453[] = {{&S67763[30], 6}, {NULL}};
const Segment SGI13454[] = {{&S68389[6], 4}, {NULL}};
const Segment SGI13455[] = {{&S68389[32], 3}, {NULL}};
const Segment SGI13456[] = {{&S67763[113], 3}, {NULL}};
const Segment SGI13457[] = {{&S68389[81], 5}, {NULL}};
const Segment SGI13458[] = {{&S67763[79], 5}, {NULL}};
const Segment SGI13459[] = {{&S67764[44], 6}, {NULL}};
const Segment SGI13460[] = {{&S67764[44], 18}, {NULL}};
const Segment SGI13461[] = {{&S67764[30], 6}, {NULL}};
const Segment SGI13462[] = {{&S67764[110], 3}, {NULL}};
const Segment SGI13463[] = {{&S67764[76], 5}, {NULL}};
const Segment SGI13464[] = {{&S67765[44], 19}, {NULL}};
const Segment SGI13465[] = {{&S67765[44], 28}, {NULL}};
const Segment SGI13466[] = {{&S67765[30], 6}, {NULL}};
const Segment SGI13467[] = {{&S67765[120], 3}, {NULL}};
const Segment SGI13468[] = {{&S67765[86], 5}, {NULL}};
const Segment SGI13469[] = {{&S67961[31], 8}, {NULL}};
const Segment SGI13470[] = {{&S67961[54], 3}, {NULL}};
const Segment SGI13471[] = {{&S67962[39], 8}, {NULL}};
const Segment SGI13472[] = {{&S67962[103], 3}, {NULL}};
const Segment SGI13473[] = {{&S67962[92], 3}, {NULL}};
const Segment SGI13474[] = {{&S67962[69], 6}, {NULL}};
const Segment SGI13475[] = {{&S68657[46], 5}, {NULL}};
const Segment SGI13476[] = {{&S70835[39], 4}, {NULL}};
const Segment SGI13477[] = {{&S70835[92], 3}, {NULL}};
const Segment SGI13478[] = {{&S70835[57], 6}, {NULL}};
const Segment SGI13479[] = {{&S70835[112], 8}, {NULL}};
const Segment SGI13480[] = {{&S70836[40], 10}, {NULL}};
const Segment SGI13481[] = {{&S70836[30], 4}, {NULL}};
const Segment SGI13482[] = {{&S70836[112], 3}, {NULL}};
const Segment SGI13483[] = {{&S70836[77], 6}, {NULL}};
const Segment SGI13484[] = {{&S70836[132], 8}, {NULL}};
const Segment SGI13485[] = {{&S70837[42], 10}, {NULL}};
const Segment SGI13486[] = {{&S70837[42], 23}, {NULL}};
const Segment SGI13487[] = {{&S70837[30], 4}, {NULL}};
const Segment SGI13488[] = {{&S70837[114], 3}, {NULL}};
const Segment SGI13489[] = {{&S70837[79], 6}, {NULL}};
const Segment SGI13490[] = {{&S70837[134], 8}, {NULL}};
const Segment SGI13491[] = {{&S67365[42], 10}, {NULL}};
const Segment SGI13492[] = {{&S67365[42], 21}, {NULL}};
const Segment SGI13493[] = {{&S67365[30], 4}, {NULL}};
const Segment SGI13494[] = {{&S67365[111], 3}, {NULL}};
const Segment SGI13495[] = {{&S67365[77], 5}, {NULL}};
const Segment SGI13496[] = {{&S67372[42], 10}, {NULL}};
const Segment SGI13497[] = {{&S67372[42], 21}, {NULL}};
const Segment SGI13498[] = {{&S67372[30], 4}, {NULL}};
const Segment SGI13499[] = {{&S67372[111], 3}, {NULL}};
const Segment SGI13500[] = {{&S67372[77], 5}, {NULL}};
const Segment SGI13501[] = {{&S73630[8], 6}, {&S73630[8], 6}, {NULL}};
const Segment SGI13502[] = {{&S73630[0], 90}, {NULL}};
const Segment SGI13503[] = {{&S73631[8], 7}, {NULL}};
const Segment SGI13504[] = {{&S73631[0], 45}, {NULL}};
const Segment SGI13505[] = {{&S73632[8], 6}, {NULL}};
const Segment SGI13506[] = {{&S73632[0], 44}, {NULL}};
const Segment SGI13507[] = {{&S73633[8], 7}, {NULL}};
const Segment SGI13508[] = {{&S73633[0], 45}, {NULL}};
const Segment SGI13509[] = {{&S68222[44], 6}, {NULL}};
const Segment SGI13510[] = {{&S68222[44], 17}, {NULL}};
const Segment SGI13511[] = {{&S68222[30], 6}, {NULL}};
const Segment SGI13512[] = {{&S68222[109], 3}, {NULL}};
const Segment SGI13513[] = {{&S68222[75], 5}, {NULL}};
const Segment SGI13514[] = {{&S68223[38], 6}, {NULL}};
const Segment SGI13515[] = {{&S68223[31], 6}, {NULL}};
const Segment SGI13516[] = {{&S68877[81], 6}, {NULL}};
const Segment SGI13517[] = {{&S67766[6], 4}, {NULL}};
const Segment SGI13518[] = {{&S67766[97], 5}, {NULL}};
const Segment SGI13519[] = {{&S67831[50], 6}, {NULL}};
const Segment SGI13520[] = {{&S67831[50], 27}, {NULL}};
const Segment SGI13521[] = {{&S67831[30], 12}, {NULL}};
const Segment SGI13522[] = {{&S67831[126], 3}, {NULL}};
const Segment SGI13523[] = {{&S67831[91], 6}, {NULL}};
const Segment SGI13524[] = {{&S67832[50], 6}, {NULL}};
const Segment SGI13525[] = {{&S67832[50], 27}, {NULL}};
const Segment SGI13526[] = {{&S67832[30], 12}, {NULL}};
const Segment SGI13527[] = {{&S67832[126], 3}, {NULL}};
const Segment SGI13528[] = {{&S67832[91], 6}, {NULL}};
const Segment SGI13529[] = {{&S67833[50], 6}, {NULL}};
const Segment SGI13530[] = {{&S67833[50], 28}, {NULL}};
const Segment SGI13531[] = {{&S67833[30], 12}, {NULL}};
const Segment SGI13532[] = {{&S67833[127], 3}, {NULL}};
const Segment SGI13533[] = {{&S67833[92], 6}, {NULL}};
const Segment SGI13534[] = {{&S67834[50], 6}, {NULL}};
const Segment SGI13535[] = {{&S67834[50], 27}, {NULL}};
const Segment SGI13536[] = {{&S67834[30], 12}, {NULL}};
const Segment SGI13537[] = {{&S67834[126], 3}, {NULL}};
const Segment SGI13538[] = {{&S67834[91], 6}, {NULL}};
const Segment SGI13539[] = {{&S67835[44], 6}, {NULL}};
const Segment SGI13540[] = {{&S67835[31], 12}, {NULL}};
const Segment SGI13541[] = {{&S67836[50], 6}, {NULL}};
const Segment SGI13542[] = {{&S67836[50], 28}, {NULL}};
const Segment SGI13543[] = {{&S67836[30], 12}, {NULL}};
const Segment SGI13544[] = {{&S67836[75], 3}, {NULL}};
const Segment SGI13545[] = {{&S67836[92], 6}, {NULL}};
const Segment SGI13546[] = {{&S71090[42], 13}, {NULL}};
const Segment SGI13547[] = {{&S71090[42], 24}, {NULL}};
const Segment SGI13548[] = {{&S71090[30], 4}, {NULL}};
const Segment SGI13549[] = {{&S71090[115], 3}, {NULL}};
const Segment SGI13550[] = {{&S71090[80], 6}, {NULL}};
const Segment SGI13551[] = {{&S71090[135], 8}, {NULL}};
const Segment SGI13552[] = {{&S71091[42], 13}, {NULL}};
const Segment SGI13553[] = {{&S71091[42], 62}, {NULL}};
const Segment SGI13554[] = {{&S71091[30], 4}, {NULL}};
const Segment SGI13555[] = {{&S71091[114], 3}, {NULL}};
const Segment SGI13556[] = {{&S71091[79], 6}, {NULL}};
const Segment SGI13557[] = {{&S69002[117], 5}, {NULL}};
const Segment SGI13558[] = {{&S67710[42], 11}, {NULL}};
const Segment SGI13559[] = {{&S67710[42], 22}, {NULL}};
const Segment SGI13560[] = {{&S67710[30], 4}, {NULL}};
const Segment SGI13561[] = {{&S67710[112], 3}, {NULL}};
const Segment SGI13562[] = {{&S67710[78], 5}, {NULL}};
const Segment SGI13563[] = {{&S67713[36], 11}, {NULL}};
const Segment SGI13564[] = {{&S67713[31], 4}, {NULL}};
const Segment SGI13565[] = {{&S67762[42], 6}, {NULL}};
const Segment SGI13566[] = {{&S67762[42], 17}, {NULL}};
const Segment SGI13567[] = {{&S67762[30], 4}, {NULL}};
const Segment SGI13568[] = {{&S67762[107], 3}, {NULL}};
const Segment SGI13569[] = {{&S67762[73], 5}, {NULL}};
const Segment SGI13570[] = {{&S68712[44], 10}, {NULL}};
const Segment SGI13571[] = {{&S68712[44], 21}, {NULL}};
const Segment SGI13572[] = {{&S68712[30], 6}, {NULL}};
const Segment SGI13573[] = {{&S68712[113], 3}, {NULL}};
const Segment SGI13574[] = {{&S68712[79], 5}, {NULL}};
const Segment SGI13575[] = {{&S68713[38], 10}, {NULL}};
const Segment SGI13576[] = {{&S68713[31], 6}, {NULL}};
const Segment SGI13577[] = {{&S71512[42], 13}, {NULL}};
const Segment SGI13578[] = {{&S71512[42], 26}, {NULL}};
const Segment SGI13579[] = {{&S71512[30], 4}, {NULL}};
const Segment SGI13580[] = {{&S71512[117], 3}, {NULL}};
const Segment SGI13581[] = {{&S71512[82], 6}, {NULL}};
const Segment SGI13582[] = {{&S71512[137], 8}, {NULL}};
const Segment SGI13583[] = {{&S69019[44], 10}, {NULL}};
const Segment SGI13584[] = {{&S69019[44], 21}, {NULL}};
const Segment SGI13585[] = {{&S69019[30], 6}, {NULL}};
const Segment SGI13586[] = {{&S69019[113], 3}, {NULL}};
const Segment SGI13587[] = {{&S69019[79], 5}, {NULL}};
const Segment SGI13588[] = {{&S69020[38], 10}, {NULL}};
const Segment SGI13589[] = {{&S69020[31], 6}, {NULL}};
const Segment SGI13590[] = {{&S69021[44], 10}, {NULL}};
const Segment SGI13591[] = {{&S69021[44], 22}, {NULL}};
const Segment SGI13592[] = {{&S69021[30], 6}, {NULL}};
const Segment SGI13593[] = {{&S69021[114], 3}, {NULL}};
const Segment SGI13594[] = {{&S69021[80], 5}, {NULL}};
const Segment SGI13595[] = {{&S68263[6], 4}, {NULL}};
const Segment SGI13596[] = {{&S68263[89], 5}, {NULL}};
const Segment SGI13597[] = {{&S68243[42], 12}, {NULL}};
const Segment SGI13598[] = {{&S68243[42], 23}, {NULL}};
const Segment SGI13599[] = {{&S68243[30], 4}, {NULL}};
const Segment SGI13600[] = {{&S68243[113], 3}, {NULL}};
const Segment SGI13601[] = {{&S68243[79], 5}, {NULL}};
const Segment SGI13602[] = {{&S69082[44], 11}, {NULL}};
const Segment SGI13603[] = {{&S69082[44], 22}, {NULL}};
const Segment SGI13604[] = {{&S69082[30], 6}, {NULL}};
const Segment SGI13605[] = {{&S69082[114], 3}, {NULL}};
const Segment SGI13606[] = {{&S69082[80], 5}, {NULL}};
const Segment SGI13607[] = {{&S68246[42], 22}, {NULL}};
const Segment SGI13608[] = {{&S68246[42], 33}, {NULL}};
const Segment SGI13609[] = {{&S68246[30], 4}, {NULL}};
const Segment SGI13610[] = {{&S68246[123], 3}, {NULL}};
const Segment SGI13611[] = {{&S68246[89], 5}, {NULL}};
const Segment SGI13612[] = {{&S71702[42], 19}, {NULL}};
const Segment SGI13613[] = {{&S71702[42], 32}, {NULL}};
const Segment SGI13614[] = {{&S71702[30], 4}, {NULL}};
const Segment SGI13615[] = {{&S71702[123], 3}, {NULL}};
const Segment SGI13616[] = {{&S71702[88], 6}, {NULL}};
const Segment SGI13617[] = {{&S71702[143], 8}, {NULL}};
const Segment SGI13618[] = {{&S71703[42], 19}, {NULL}};
const Segment SGI13619[] = {{&S71703[42], 32}, {NULL}};
const Segment SGI13620[] = {{&S71703[30], 4}, {NULL}};
const Segment SGI13621[] = {{&S71703[123], 3}, {NULL}};
const Segment SGI13622[] = {{&S71703[88], 6}, {NULL}};
const Segment SGI13623[] = {{&S71703[143], 8}, {NULL}};
const Segment SGI13624[] = {{&S71704[42], 19}, {NULL}};
const Segment SGI13625[] = {{&S71704[42], 32}, {NULL}};
const Segment SGI13626[] = {{&S71704[30], 4}, {NULL}};
const Segment SGI13627[] = {{&S71704[124], 3}, {NULL}};
const Segment SGI13628[] = {{&S71704[88], 7}, {NULL}};
const Segment SGI13629[] = {{&S71704[144], 8}, {NULL}};
const Segment SGI13630[] = {{&S71705[42], 19}, {NULL}};
const Segment SGI13631[] = {{&S71705[42], 32}, {NULL}};
const Segment SGI13632[] = {{&S71705[30], 4}, {NULL}};
const Segment SGI13633[] = {{&S71705[123], 3}, {NULL}};
const Segment SGI13634[] = {{&S71705[88], 5}, {NULL}};
const Segment SGI13635[] = {{&S71705[143], 8}, {NULL}};
const Segment SGI13636[] = {{&S69186[44], 9}, {NULL}};
const Segment SGI13637[] = {{&S69186[44], 20}, {NULL}};
const Segment SGI13638[] = {{&S69186[30], 6}, {NULL}};
const Segment SGI13639[] = {{&S69186[112], 3}, {NULL}};
const Segment SGI13640[] = {{&S69186[78], 5}, {NULL}};
const Segment SGI13641[] = {{&S71828[44], 11}, {NULL}};
const Segment SGI13642[] = {{&S71828[44], 23}, {NULL}};
const Segment SGI13643[] = {{&S71828[30], 6}, {NULL}};
const Segment SGI13644[] = {{&S71828[116], 3}, {NULL}};
const Segment SGI13645[] = {{&S71828[81], 6}, {NULL}};
const Segment SGI13646[] = {{&S68411[42], 9}, {NULL}};
const Segment SGI13647[] = {{&S68411[42], 20}, {NULL}};
const Segment SGI13648[] = {{&S68411[30], 4}, {NULL}};
const Segment SGI13649[] = {{&S68411[110], 3}, {NULL}};
const Segment SGI13650[] = {{&S68411[76], 5}, {NULL}};
const Segment SGI13651[] = {{&S68413[36], 9}, {NULL}};
const Segment SGI13652[] = {{&S68413[31], 4}, {NULL}};
const Segment SGI13653[] = {{&S69808[84], 5}, {NULL}};
const Segment SGI13654[] = {{&S68406[50], 17}, {NULL}};
const Segment SGI13655[] = {{&S68406[50], 28}, {NULL}};
const Segment SGI13656[] = {{&S68406[30], 12}, {NULL}};
const Segment SGI13657[] = {{&S68406[127], 3}, {NULL}};
const Segment SGI13658[] = {{&S68406[92], 6}, {NULL}};
const Segment SGI13659[] = {{&S68520[41], 24}, {NULL}};
const Segment SGI13660[] = {{&S68520[30], 4}, {NULL}};
const Segment SGI13661[] = {{&S68520[113], 3}, {NULL}};
const Segment SGI13662[] = {{&S68520[79], 5}, {NULL}};
const Segment SGI13663[] = {{&S71972[42], 13}, {NULL}};
const Segment SGI13664[] = {{&S71972[42], 24}, {NULL}};
const Segment SGI13665[] = {{&S71972[30], 4}, {NULL}};
const Segment SGI13666[] = {{&S71972[115], 3}, {NULL}};
const Segment SGI13667[] = {{&S71972[80], 6}, {NULL}};
const Segment SGI13668[] = {{&S71972[135], 8}, {NULL}};
const Segment SGI13669[] = {{&S71973[59], 6}, {NULL}};
const Segment SGI13670[] = {{&S71973[59], 17}, {NULL}};
const Segment SGI13671[] = {{&S71973[47], 4}, {NULL}};
const Segment SGI13672[] = {{&S71973[125], 3}, {NULL}};
const Segment SGI13673[] = {{&S71973[90], 6}, {NULL}};
const Segment SGI13674[] = {{&S71973[145], 8}, {NULL}};
const Segment SGI13675[] = {{&S71974[11], 1}, {NULL}};
const Segment SGI13676[] = {{&S71974[42], 10}, {NULL}};
const Segment SGI13677[] = {{&S71974[30], 4}, {NULL}};
const Segment SGI13678[] = {{&S71974[114], 3}, {NULL}};
const Segment SGI13679[] = {{&S71974[79], 6}, {NULL}};
const Segment SGI13680[] = {{&S71974[134], 8}, {NULL}};
const Segment SGI13681[] = {{&S71975[42], 13}, {NULL}};
const Segment SGI13682[] = {{&S71975[42], 24}, {NULL}};
const Segment SGI13683[] = {{&S71975[30], 4}, {NULL}};
const Segment SGI13684[] = {{&S71975[115], 3}, {NULL}};
const Segment SGI13685[] = {{&S71975[80], 6}, {NULL}};
const Segment SGI13686[] = {{&S71975[135], 8}, {NULL}};
const Segment SGI13687[] = {{&S71976[42], 21}, {NULL}};
const Segment SGI13688[] = {{&S71976[30], 4}, {NULL}};
const Segment SGI13689[] = {{&S71976[128], 3}, {NULL}};
const Segment SGI13690[] = {{&S71976[93], 6}, {NULL}};
const Segment SGI13691[] = {{&S71976[148], 8}, {NULL}};
const Segment SGI13692[] = {{&S71977[42], 15}, {NULL}};
const Segment SGI13693[] = {{&S71977[42], 26}, {NULL}};
const Segment SGI13694[] = {{&S71977[30], 4}, {NULL}};
const Segment SGI13695[] = {{&S71977[117], 3}, {NULL}};
const Segment SGI13696[] = {{&S71977[82], 6}, {NULL}};
const Segment SGI13697[] = {{&S71977[137], 8}, {NULL}};
const Segment SGI13698[] = {{&S68631[50], 11}, {NULL}};
const Segment SGI13699[] = {{&S68631[50], 22}, {NULL}};
const Segment SGI13700[] = {{&S68631[30], 12}, {NULL}};
const Segment SGI13701[] = {{&S68631[121], 3}, {NULL}};
const Segment SGI13702[] = {{&S68631[86], 6}, {NULL}};
const Segment SGI13703[] = {{&S68632[50], 11}, {NULL}};
const Segment SGI13704[] = {{&S68632[50], 23}, {NULL}};
const Segment SGI13705[] = {{&S68632[30], 12}, {NULL}};
const Segment SGI13706[] = {{&S68632[122], 3}, {NULL}};
const Segment SGI13707[] = {{&S68632[87], 6}, {NULL}};
const Segment SGI13708[] = {{&S68633[50], 11}, {NULL}};
const Segment SGI13709[] = {{&S68633[50], 22}, {NULL}};
const Segment SGI13710[] = {{&S68633[30], 12}, {NULL}};
const Segment SGI13711[] = {{&S68633[121], 3}, {NULL}};
const Segment SGI13712[] = {{&S68633[86], 6}, {NULL}};
const Segment SGI13713[] = {{&S68634[50], 11}, {NULL}};
const Segment SGI13714[] = {{&S68634[50], 24}, {NULL}};
const Segment SGI13715[] = {{&S68634[30], 12}, {NULL}};
const Segment SGI13716[] = {{&S68634[123], 3}, {NULL}};
const Segment SGI13717[] = {{&S68634[88], 6}, {NULL}};
const Segment SGI13718[] = {{&S68635[44], 11}, {NULL}};
const Segment SGI13719[] = {{&S68635[31], 12}, {NULL}};
const Segment SGI13720[] = {{&S68636[50], 11}, {NULL}};
const Segment SGI13721[] = {{&S68636[50], 23}, {NULL}};
const Segment SGI13722[] = {{&S68636[30], 12}, {NULL}};
const Segment SGI13723[] = {{&S68636[90], 4}, {NULL}};
const Segment SGI13724[] = {{&S68636[90], 3}, {NULL}};
const Segment SGI13725[] = {{&S69573[44], 15}, {NULL}};
const Segment SGI13726[] = {{&S69573[44], 26}, {NULL}};
const Segment SGI13727[] = {{&S69573[30], 6}, {NULL}};
const Segment SGI13728[] = {{&S69573[118], 3}, {NULL}};
const Segment SGI13729[] = {{&S69573[84], 5}, {NULL}};
const Segment SGI13730[] = {{&S69693[44], 9}, {NULL}};
const Segment SGI13731[] = {{&S69693[44], 20}, {NULL}};
const Segment SGI13732[] = {{&S69693[30], 6}, {NULL}};
const Segment SGI13733[] = {{&S69693[112], 3}, {NULL}};
const Segment SGI13734[] = {{&S69693[78], 5}, {NULL}};
const Segment SGI13735[] = {{&S68897[42], 11}, {NULL}};
const Segment SGI13736[] = {{&S68897[42], 22}, {NULL}};
const Segment SGI13737[] = {{&S68897[30], 4}, {NULL}};
const Segment SGI13738[] = {{&S68897[112], 3}, {NULL}};
const Segment SGI13739[] = {{&S68897[78], 5}, {NULL}};
const Segment SGI13740[] = {{&S68898[42], 4}, {NULL}};
const Segment SGI13741[] = {{&S68898[42], 15}, {NULL}};
const Segment SGI13742[] = {{&S68898[30], 4}, {NULL}};
const Segment SGI13743[] = {{&S68898[105], 3}, {NULL}};
const Segment SGI13744[] = {{&S68898[71], 5}, {NULL}};
const Segment SGI13745[] = {{&S68899[42], 23}, {NULL}};
const Segment SGI13746[] = {{&S68899[42], 34}, {NULL}};
const Segment SGI13747[] = {{&S68899[30], 4}, {&S68899[50], 26}, {NULL}};
const Segment SGI13748[] = {{&S68899[124], 3}, {NULL}};
const Segment SGI13749[] = {{&S68899[90], 5}, {NULL}};
const Segment SGI13750[] = {{&S68864[50], 15}, {NULL}};
const Segment SGI13751[] = {{&S68864[50], 26}, {NULL}};
const Segment SGI13752[] = {{&S68864[30], 12}, {NULL}};
const Segment SGI13753[] = {{&S68864[125], 3}, {NULL}};
const Segment SGI13754[] = {{&S68864[90], 6}, {NULL}};
const Segment SGI13755[] = {{&S68865[50], 8}, {NULL}};
const Segment SGI13756[] = {{&S68865[50], 19}, {NULL}};
const Segment SGI13757[] = {{&S68865[30], 12}, {NULL}};
const Segment SGI13758[] = {{&S68865[118], 3}, {NULL}};
const Segment SGI13759[] = {{&S68865[83], 6}, {NULL}};
const Segment SGI13760[] = {{&S68866[44], 15}, {NULL}};
const Segment SGI13761[] = {{&S68866[31], 12}, {NULL}};
const Segment SGI13762[] = {{&S69018[42], 10}, {NULL}};
const Segment SGI13763[] = {{&S69018[42], 21}, {NULL}};
const Segment SGI13764[] = {{&S69018[30], 4}, {NULL}};
const Segment SGI13765[] = {{&S69018[111], 3}, {NULL}};
const Segment SGI13766[] = {{&S69018[77], 5}, {NULL}};
const Segment SGI13767[] = {{&S69853[44], 10}, {NULL}};
const Segment SGI13768[] = {{&S69853[44], 21}, {NULL}};
const Segment SGI13769[] = {{&S69853[30], 6}, {NULL}};
const Segment SGI13770[] = {{&S69853[113], 3}, {NULL}};
const Segment SGI13771[] = {{&S69853[79], 5}, {NULL}};
const Segment SGI13772[] = {{&S69854[38], 10}, {NULL}};
const Segment SGI13773[] = {{&S69854[31], 6}, {NULL}};
const Segment SGI13774[] = {{&S69878[44], 10}, {NULL}};
const Segment SGI13775[] = {{&S69878[44], 21}, {NULL}};
const Segment SGI13776[] = {{&S69878[30], 6}, {NULL}};
const Segment SGI13777[] = {{&S69878[113], 3}, {NULL}};
const Segment SGI13778[] = {{&S69878[79], 5}, {NULL}};
const Segment SGI13779[] = {{&S69879[38], 10}, {NULL}};
const Segment SGI13780[] = {{&S69879[31], 6}, {NULL}};
const Segment SGI13781[] = {{&S69880[42], 10}, {NULL}};
const Segment SGI13782[] = {{&S69880[30], 6}, {NULL}};
const Segment SGI13783[] = {{&S69880[111], 3}, {NULL}};
const Segment SGI13784[] = {{&S69880[77], 5}, {NULL}};
const Segment SGI13785[] = {{&S71827[42], 5}, {NULL}};
const Segment SGI13786[] = {{&S71827[42], 17}, {NULL}};
const Segment SGI13787[] = {{&S71827[30], 4}, {NULL}};
const Segment SGI13788[] = {{&S71827[108], 3}, {NULL}};
const Segment SGI13789[] = {{&S71827[73], 6}, {NULL}};
const Segment SGI13790[] = {{&S69081[42], 11}, {NULL}};
const Segment SGI13791[] = {{&S69081[42], 22}, {NULL}};
const Segment SGI13792[] = {{&S69081[30], 4}, {NULL}};
const Segment SGI13793[] = {{&S69081[112], 3}, {NULL}};
const Segment SGI13794[] = {{&S69081[78], 5}, {NULL}};
const Segment SGI13795[] = {{&S69084[36], 11}, {NULL}};
const Segment SGI13796[] = {{&S69084[31], 4}, {NULL}};
const Segment SGI13797[] = {{&S69187[42], 9}, {NULL}};
const Segment SGI13798[] = {{&S69187[42], 20}, {NULL}};
const Segment SGI13799[] = {{&S69187[30], 4}, {NULL}};
const Segment SGI13800[] = {{&S69187[110], 3}, {NULL}};
const Segment SGI13801[] = {{&S69187[76], 5}, {NULL}};
const Segment SGI13802[] = {{&S69189[42], 9}, {NULL}};
const Segment SGI13803[] = {{&S69189[42], 20}, {NULL}};
const Segment SGI13804[] = {{&S69189[30], 4}, {NULL}};
const Segment SGI13805[] = {{&S69189[79], 4}, {NULL}};
const Segment SGI13806[] = {{&S69189[79], 3}, {NULL}};
const Segment SGI13807[] = {{&S69190[42], 9}, {NULL}};
const Segment SGI13808[] = {{&S69190[42], 20}, {NULL}};
const Segment SGI13809[] = {{&S69190[30], 4}, {NULL}};
const Segment SGI13810[] = {{&S69190[79], 6}, {NULL}};
const Segment SGI13811[] = {{&S70066[44], 11}, {NULL}};
const Segment SGI13812[] = {{&S70066[44], 22}, {NULL}};
const Segment SGI13813[] = {{&S70066[30], 6}, {NULL}};
const Segment SGI13814[] = {{&S70066[114], 3}, {NULL}};
const Segment SGI13815[] = {{&S70066[80], 5}, {NULL}};
const Segment SGI13816[] = {{&S70067[38], 11}, {NULL}};
const Segment SGI13817[] = {{&S70067[31], 6}, {NULL}};
const Segment SGI13818[] = {{&S70561[8], 8}, {NULL}};
const Segment SGI13819[] = {{&S70680[82], 6}, {NULL}};
const Segment SGI13820[] = {{&S69310[50], 12}, {NULL}};
const Segment SGI13821[] = {{&S69310[50], 23}, {NULL}};
const Segment SGI13822[] = {{&S69310[30], 12}, {NULL}};
const Segment SGI13823[] = {{&S69310[122], 3}, {NULL}};
const Segment SGI13824[] = {{&S69310[87], 6}, {NULL}};
const Segment SGI13825[] = {{&S69311[44], 12}, {NULL}};
const Segment SGI13826[] = {{&S69311[31], 12}, {NULL}};
const Segment SGI13827[] = {{&S70259[44], 10}, {NULL}};
const Segment SGI13828[] = {{&S70259[44], 21}, {NULL}};
const Segment SGI13829[] = {{&S70259[30], 6}, {NULL}};
const Segment SGI13830[] = {{&S70259[113], 3}, {NULL}};
const Segment SGI13831[] = {{&S70259[79], 5}, {NULL}};
const Segment SGI13832[] = {{&S70261[38], 10}, {NULL}};
const Segment SGI13833[] = {{&S70261[31], 6}, {NULL}};
const Segment SGI13834[] = {{&S70263[42], 10}, {NULL}};
const Segment SGI13835[] = {{&S70263[30], 6}, {NULL}};
const Segment SGI13836[] = {{&S70263[111], 3}, {NULL}};
const Segment SGI13837[] = {{&S70263[77], 5}, {NULL}};
const Segment SGI13838[] = {{&S69572[42], 15}, {NULL}};
const Segment SGI13839[] = {{&S69572[42], 26}, {NULL}};
const Segment SGI13840[] = {{&S69572[30], 4}, {NULL}};
const Segment SGI13841[] = {{&S69572[116], 3}, {NULL}};
const Segment SGI13842[] = {{&S69572[82], 5}, {NULL}};
const Segment SGI13843[] = {{&S69575[36], 15}, {NULL}};
const Segment SGI13844[] = {{&S69575[31], 4}, {NULL}};
const Segment SGI13845[] = {{&S69632[42], 9}, {NULL}};
const Segment SGI13846[] = {{&S69632[42], 20}, {NULL}};
const Segment SGI13847[] = {{&S69632[30], 4}, {NULL}};
const Segment SGI13848[] = {{&S69632[110], 3}, {NULL}};
const Segment SGI13849[] = {{&S69632[76], 5}, {NULL}};
const Segment SGI13850[] = {{&S71030[8], 8}, {NULL}};
const Segment SGI13851[] = {{&S72990[42], 13}, {NULL}};
const Segment SGI13852[] = {{&S72990[42], 26}, {NULL}};
const Segment SGI13853[] = {{&S72990[30], 4}, {NULL}};
const Segment SGI13854[] = {{&S72990[97], 3}, {NULL}};
const Segment SGI13855[] = {{&S72990[82], 6}, {NULL}};
const Segment SGI13856[] = {{&S70655[44], 6}, {NULL}};
const Segment SGI13857[] = {{&S70655[44], 17}, {NULL}};
const Segment SGI13858[] = {{&S70655[30], 6}, {NULL}};
const Segment SGI13859[] = {{&S70655[109], 3}, {NULL}};
const Segment SGI13860[] = {{&S70655[75], 5}, {NULL}};
const Segment SGI13861[] = {{&S70656[42], 6}, {NULL}};
const Segment SGI13862[] = {{&S70656[30], 6}, {NULL}};
const Segment SGI13863[] = {{&S70656[107], 3}, {NULL}};
const Segment SGI13864[] = {{&S70656[73], 5}, {NULL}};
const Segment SGI13865[] = {{&S70657[38], 6}, {NULL}};
const Segment SGI13866[] = {{&S70657[31], 6}, {NULL}};
const Segment SGI13867[] = {{&S75822[8], 3}, {NULL}};
const Segment SGI13868[] = {{&S75822[0], 50}, {NULL}};
const Segment SGI13869[] = {{&S73176[42], 8}, {NULL}};
const Segment SGI13870[] = {{&S73176[42], 19}, {NULL}};
const Segment SGI13871[] = {{&S73176[30], 4}, {NULL}};
const Segment SGI13872[] = {{&S73176[110], 3}, {NULL}};
const Segment SGI13873[] = {{&S73176[75], 6}, {NULL}};
const Segment SGI13874[] = {{&S73176[130], 8}, {NULL}};
const Segment SGI13875[] = {{&S73177[42], 8}, {NULL}};
const Segment SGI13876[] = {{&S73177[42], 19}, {NULL}};
const Segment SGI13877[] = {{&S73177[30], 4}, {NULL}};
const Segment SGI13878[] = {{&S73177[111], 3}, {NULL}};
const Segment SGI13879[] = {{&S73177[75], 7}, {NULL}};
const Segment SGI13880[] = {{&S73177[131], 8}, {NULL}};
const Segment SGI13881[] = {{&S70782[44], 10}, {NULL}};
const Segment SGI13882[] = {{&S70782[44], 21}, {NULL}};
const Segment SGI13883[] = {{&S70782[30], 6}, {NULL}};
const Segment SGI13884[] = {{&S70782[113], 3}, {NULL}};
const Segment SGI13885[] = {{&S70782[79], 5}, {NULL}};
const Segment SGI13886[] = {{&S70785[44], 10}, {NULL}};
const Segment SGI13887[] = {{&S70785[44], 21}, {NULL}};
const Segment SGI13888[] = {{&S70785[30], 6}, {NULL}};
const Segment SGI13889[] = {{&S70785[82], 4}, {NULL}};
const Segment SGI13890[] = {{&S70785[82], 3}, {NULL}};
const Segment SGI13891[] = {{&S70065[42], 11}, {NULL}};
const Segment SGI13892[] = {{&S70065[42], 22}, {NULL}};
const Segment SGI13893[] = {{&S70065[30], 4}, {NULL}};
const Segment SGI13894[] = {{&S70065[112], 3}, {NULL}};
const Segment SGI13895[] = {{&S70065[78], 5}, {NULL}};
const Segment SGI13896[] = {{&S76312[8], 6}, {NULL}};
const Segment SGI13897[] = {{&S76312[0], 92}, {NULL}};
const Segment SGI13898[] = {{&S76313[8], 7}, {NULL}};
const Segment SGI13899[] = {{&S76313[0], 85}, {NULL}};
const Segment SGI13900[] = {{&S76314[8], 7}, {NULL}};
const Segment SGI13901[] = {{&S76314[0], 85}, {NULL}};
const Segment SGI13902[] = {{&S76315[8], 6}, {NULL}};
const Segment SGI13903[] = {{&S76315[0], 84}, {NULL}};
const Segment SGI13904[] = {{&S71829[56], 11}, {NULL}};
const Segment SGI13905[] = {{&S71829[56], 23}, {NULL}};
const Segment SGI13906[] = {{&S71829[30], 18}, {NULL}};
const Segment SGI13907[] = {{&S71829[128], 3}, {NULL}};
const Segment SGI13908[] = {{&S71829[93], 6}, {NULL}};
const Segment SGI13909[] = {{&S70116[50], 8}, {NULL}};
const Segment SGI13910[] = {{&S70116[50], 19}, {NULL}};
const Segment SGI13911[] = {{&S70116[30], 12}, {NULL}};
const Segment SGI13912[] = {{&S70116[118], 3}, {NULL}};
const Segment SGI13913[] = {{&S70116[83], 6}, {NULL}};
const Segment SGI13914[] = {{&S70117[50], 8}, {NULL}};
const Segment SGI13915[] = {{&S70117[50], 19}, {NULL}};
const Segment SGI13916[] = {{&S70117[30], 12}, {NULL}};
const Segment SGI13917[] = {{&S70117[118], 3}, {NULL}};
const Segment SGI13918[] = {{&S70117[83], 6}, {NULL}};
const Segment SGI13919[] = {{&S70118[50], 8}, {NULL}};
const Segment SGI13920[] = {{&S70118[50], 19}, {NULL}};
const Segment SGI13921[] = {{&S70118[30], 12}, {NULL}};
const Segment SGI13922[] = {{&S70118[118], 3}, {NULL}};
const Segment SGI13923[] = {{&S70118[83], 6}, {NULL}};
const Segment SGI13924[] = {{&S70119[44], 8}, {NULL}};
const Segment SGI13925[] = {{&S70119[31], 12}, {NULL}};
const Segment SGI13926[] = {{&S76599[8], 6}, {NULL}};
const Segment SGI13927[] = {{&S76599[0], 84}, {NULL}};
const Segment SGI13928[] = {{&S76600[8], 7}, {NULL}};
const Segment SGI13929[] = {{&S76600[0], 85}, {NULL}};
const Segment SGI13930[] = {{&S73934[42], 8}, {NULL}};
const Segment SGI13931[] = {{&S73934[42], 19}, {NULL}};
const Segment SGI13932[] = {{&S73934[30], 4}, {NULL}};
const Segment SGI13933[] = {{&S73934[110], 3}, {NULL}};
const Segment SGI13934[] = {{&S73934[75], 6}, {NULL}};
const Segment SGI13935[] = {{&S73934[130], 8}, {NULL}};
const Segment SGI13936[] = {{&S73935[60], 8}, {NULL}};
const Segment SGI13937[] = {{&S73935[60], 19}, {NULL}};
const Segment SGI13938[] = {{&S73935[30], 22}, {NULL}};
const Segment SGI13939[] = {{&S73935[128], 3}, {NULL}};
const Segment SGI13940[] = {{&S73935[93], 6}, {NULL}};
const Segment SGI13941[] = {{&S73935[148], 8}, {NULL}};
const Segment SGI13942[] = {{&S73936[60], 8}, {NULL}};
const Segment SGI13943[] = {{&S73936[60], 19}, {NULL}};
const Segment SGI13944[] = {{&S73936[30], 22}, {NULL}};
const Segment SGI13945[] = {{&S73936[128], 3}, {NULL}};
const Segment SGI13946[] = {{&S73936[93], 6}, {NULL}};
const Segment SGI13947[] = {{&S73936[148], 8}, {NULL}};
const Segment SGI13948[] = {{&S73937[42], 21}, {NULL}};
const Segment SGI13949[] = {{&S73937[42], 32}, {NULL}};
const Segment SGI13950[] = {{&S73937[30], 4}, {NULL}};
const Segment SGI13951[] = {{&S73937[123], 3}, {NULL}};
const Segment SGI13952[] = {{&S73937[88], 6}, {NULL}};
const Segment SGI13953[] = {{&S73937[143], 8}, {NULL}};
const Segment SGI13954[] = {{&S73938[42], 21}, {NULL}};
const Segment SGI13955[] = {{&S73938[42], 32}, {NULL}};
const Segment SGI13956[] = {{&S73938[30], 4}, {NULL}};
const Segment SGI13957[] = {{&S73938[123], 3}, {NULL}};
const Segment SGI13958[] = {{&S73938[88], 6}, {NULL}};
const Segment SGI13959[] = {{&S73938[143], 8}, {NULL}};
const Segment SGI13960[] = {{&S73939[40], 21}, {NULL}};
const Segment SGI13961[] = {{&S73939[30], 4}, {NULL}};
const Segment SGI13962[] = {{&S73939[121], 3}, {NULL}};
const Segment SGI13963[] = {{&S73939[86], 6}, {NULL}};
const Segment SGI13964[] = {{&S73939[141], 8}, {NULL}};
const Segment SGI13965[] = {{&S74051[40], 5}, {NULL}};
const Segment SGI13966[] = {{&S74051[30], 4}, {NULL}};
const Segment SGI13967[] = {{&S74051[107], 3}, {NULL}};
const Segment SGI13968[] = {{&S74051[127], 8}, {NULL}};
const Segment SGI13969[] = {{&S74052[42], 5}, {NULL}};
const Segment SGI13970[] = {{&S74052[42], 18}, {NULL}};
const Segment SGI13971[] = {{&S74052[30], 4}, {NULL}};
const Segment SGI13972[] = {{&S74052[109], 3}, {NULL}};
const Segment SGI13973[] = {{&S74052[74], 6}, {NULL}};
const Segment SGI13974[] = {{&S74052[129], 8}, {NULL}};
const Segment SGI13975[] = {{&S74053[42], 5}, {NULL}};
const Segment SGI13976[] = {{&S74053[42], 18}, {NULL}};
const Segment SGI13977[] = {{&S74053[30], 4}, {NULL}};
const Segment SGI13978[] = {{&S74053[109], 3}, {NULL}};
const Segment SGI13979[] = {{&S74053[129], 8}, {NULL}};
const Segment SGI13980[] = {{&S74054[42], 12}, {NULL}};
const Segment SGI13981[] = {{&S74054[42], 25}, {NULL}};
const Segment SGI13982[] = {{&S74054[30], 4}, {NULL}};
const Segment SGI13983[] = {{&S74054[116], 3}, {NULL}};
const Segment SGI13984[] = {{&S74054[81], 6}, {NULL}};
const Segment SGI13985[] = {{&S74054[136], 8}, {NULL}};
const Segment SGI13986[] = {{&S71556[44], 7}, {NULL}};
const Segment SGI13987[] = {{&S71556[44], 18}, {NULL}};
const Segment SGI13988[] = {{&S71556[30], 6}, {NULL}};
const Segment SGI13989[] = {{&S71556[110], 3}, {NULL}};
const Segment SGI13990[] = {{&S71556[76], 5}, {NULL}};
const Segment SGI13991[] = {{&S76984[0], 107}, {NULL}};
const Segment SGI13992[] = {{&S70779[50], 7}, {NULL}};
const Segment SGI13993[] = {{&S70779[50], 19}, {NULL}};
const Segment SGI13994[] = {{&S70779[30], 12}, {NULL}};
const Segment SGI13995[] = {{&S70779[118], 3}, {NULL}};
const Segment SGI13996[] = {{&S70779[83], 6}, {NULL}};
const Segment SGI13997[] = {{&S70780[44], 7}, {NULL}};
const Segment SGI13998[] = {{&S70780[31], 12}, {NULL}};
const Segment SGI13999[] = {{&S70788[50], 11}, {NULL}};
const Segment SGI14000[] = {{&S70788[50], 22}, {NULL}};
const Segment SGI14001[] = {{&S70788[30], 12}, {NULL}};
const Segment SGI14002[] = {{&S70788[121], 3}, {NULL}};
const Segment SGI14003[] = {{&S70788[86], 6}, {NULL}};
const Segment SGI14004[] = {{&S71756[44], 13}, {NULL}};
const Segment SGI14005[] = {{&S71756[44], 24}, {NULL}};
const Segment SGI14006[] = {{&S71756[30], 6}, {NULL}};
const Segment SGI14007[] = {{&S71756[116], 3}, {NULL}};
const Segment SGI14008[] = {{&S71756[82], 5}, {NULL}};
const Segment SGI14009[] = {{&S71757[44], 6}, {NULL}};
const Segment SGI14010[] = {{&S71757[44], 18}, {NULL}};
const Segment SGI14011[] = {{&S71757[30], 6}, {NULL}};
const Segment SGI14012[] = {{&S71757[110], 3}, {NULL}};
const Segment SGI14013[] = {{&S71757[76], 5}, {NULL}};
const Segment SGI14014[] = {{&S71787[44], 6}, {NULL}};
const Segment SGI14015[] = {{&S71787[44], 27}, {NULL}};
const Segment SGI14016[] = {{&S71787[30], 6}, {NULL}};
const Segment SGI14017[] = {{&S71787[68], 3}, {NULL}};
const Segment SGI14018[] = {{&S71787[85], 5}, {NULL}};
const Segment SGI14019[] = {{&S71788[44], 6}, {NULL}};
const Segment SGI14020[] = {{&S71788[44], 27}, {NULL}};
const Segment SGI14021[] = {{&S71788[30], 6}, {NULL}};
const Segment SGI14022[] = {{&S71788[119], 3}, {NULL}};
const Segment SGI14023[] = {{&S71788[85], 5}, {NULL}};
const Segment SGI14024[] = {{&S71789[44], 6}, {NULL}};
const Segment SGI14025[] = {{&S71789[44], 27}, {NULL}};
const Segment SGI14026[] = {{&S71789[30], 6}, {NULL}};
const Segment SGI14027[] = {{&S71789[119], 3}, {NULL}};
const Segment SGI14028[] = {{&S71789[85], 5}, {NULL}};
const Segment SGI14029[] = {{&S71790[44], 6}, {NULL}};
const Segment SGI14030[] = {{&S71790[44], 28}, {NULL}};
const Segment SGI14031[] = {{&S71790[30], 6}, {NULL}};
const Segment SGI14032[] = {{&S71790[120], 3}, {NULL}};
const Segment SGI14033[] = {{&S71790[86], 5}, {NULL}};
const Segment SGI14034[] = {{&S71792[44], 6}, {NULL}};
const Segment SGI14035[] = {{&S71792[44], 17}, {NULL}};
const Segment SGI14036[] = {{&S71792[30], 6}, {NULL}};
const Segment SGI14037[] = {{&S71792[109], 3}, {NULL}};
const Segment SGI14038[] = {{&S71792[75], 5}, {NULL}};
const Segment SGI14039[] = {{&S71793[44], 6}, {NULL}};
const Segment SGI14040[] = {{&S71793[44], 27}, {NULL}};
const Segment SGI14041[] = {{&S71793[30], 6}, {NULL}};
const Segment SGI14042[] = {{&S71793[119], 3}, {NULL}};
const Segment SGI14043[] = {{&S71793[85], 5}, {NULL}};
const Segment SGI14044[] = {{&S71794[38], 6}, {NULL}};
const Segment SGI14045[] = {{&S71794[31], 6}, {NULL}};
const Segment SGI14046[] = {{&S71830[44], 8}, {NULL}};
const Segment SGI14047[] = {{&S71830[44], 19}, {NULL}};
const Segment SGI14048[] = {{&S71830[30], 6}, {NULL}};
const Segment SGI14049[] = {{&S71830[111], 3}, {NULL}};
const Segment SGI14050[] = {{&S71830[77], 5}, {NULL}};
const Segment SGI14051[] = {{&S71198[42], 11}, {NULL}};
const Segment SGI14052[] = {{&S71198[42], 22}, {NULL}};
const Segment SGI14053[] = {{&S71198[30], 4}, {NULL}};
const Segment SGI14054[] = {{&S71198[112], 3}, {NULL}};
const Segment SGI14055[] = {{&S71198[78], 5}, {NULL}};
const Segment SGI14056[] = {{&S74617[42], 17}, {NULL}};
const Segment SGI14057[] = {{&S74617[42], 28}, {NULL}};
const Segment SGI14058[] = {{&S74617[30], 4}, {NULL}};
const Segment SGI14059[] = {{&S74617[119], 3}, {NULL}};
const Segment SGI14060[] = {{&S74617[84], 6}, {NULL}};
const Segment SGI14061[] = {{&S74617[139], 8}, {NULL}};
const Segment SGI14062[] = {{&S72935[82], 6}, {NULL}};
const Segment SGI14063[] = {{&S72081[44], 10}, {NULL}};
const Segment SGI14064[] = {{&S72081[44], 21}, {NULL}};
const Segment SGI14065[] = {{&S72081[30], 6}, {NULL}};
const Segment SGI14066[] = {{&S72081[113], 3}, {NULL}};
const Segment SGI14067[] = {{&S72081[79], 5}, {NULL}};
const Segment SGI14068[] = {{&S72082[44], 10}, {NULL}};
const Segment SGI14069[] = {{&S72082[44], 21}, {NULL}};
const Segment SGI14070[] = {{&S72082[30], 6}, {NULL}};
const Segment SGI14071[] = {{&S72082[113], 3}, {NULL}};
const Segment SGI14072[] = {{&S72082[79], 5}, {NULL}};
const Segment SGI14073[] = {{&S72083[44], 10}, {NULL}};
const Segment SGI14074[] = {{&S72083[44], 21}, {NULL}};
const Segment SGI14075[] = {{&S72083[30], 6}, {NULL}};
const Segment SGI14076[] = {{&S72083[113], 3}, {NULL}};
const Segment SGI14077[] = {{&S72083[79], 5}, {NULL}};
const Segment SGI14078[] = {{&S72084[44], 10}, {NULL}};
const Segment SGI14079[] = {{&S72084[44], 21}, {NULL}};
const Segment SGI14080[] = {{&S72084[30], 6}, {NULL}};
const Segment SGI14081[] = {{&S72084[113], 3}, {NULL}};
const Segment SGI14082[] = {{&S72084[79], 5}, {NULL}};
const Segment SGI14083[] = {{&S72085[44], 10}, {NULL}};
const Segment SGI14084[] = {{&S72085[44], 21}, {NULL}};
const Segment SGI14085[] = {{&S72085[30], 6}, {NULL}};
const Segment SGI14086[] = {{&S72085[113], 3}, {NULL}};
const Segment SGI14087[] = {{&S72085[79], 5}, {NULL}};
const Segment SGI14088[] = {{&S72086[38], 10}, {NULL}};
const Segment SGI14089[] = {{&S72086[31], 6}, {NULL}};
const Segment SGI14090[] = {{&S71164[50], 9}, {NULL}};
const Segment SGI14091[] = {{&S71164[50], 28}, {NULL}};
const Segment SGI14092[] = {{&S71164[30], 12}, {NULL}};
const Segment SGI14093[] = {{&S71164[127], 3}, {NULL}};
const Segment SGI14094[] = {{&S71164[92], 6}, {NULL}};
const Segment SGI14095[] = {{&S71165[50], 14}, {NULL}};
const Segment SGI14096[] = {{&S71165[50], 33}, {NULL}};
const Segment SGI14097[] = {{&S71165[30], 12}, {NULL}};
const Segment SGI14098[] = {{&S71165[132], 3}, {NULL}};
const Segment SGI14099[] = {{&S71165[97], 6}, {NULL}};
const Segment SGI14100[] = {{&S71166[50], 14}, {NULL}};
const Segment SGI14101[] = {{&S71166[50], 33}, {NULL}};
const Segment SGI14102[] = {{&S71166[30], 12}, {NULL}};
const Segment SGI14103[] = {{&S71166[132], 3}, {NULL}};
const Segment SGI14104[] = {{&S71166[97], 6}, {NULL}};
const Segment SGI14105[] = {{&S71167[50], 14}, {NULL}};
const Segment SGI14106[] = {{&S71167[50], 33}, {NULL}};
const Segment SGI14107[] = {{&S71167[30], 12}, {NULL}};
const Segment SGI14108[] = {{&S71167[132], 3}, {NULL}};
const Segment SGI14109[] = {{&S71167[97], 6}, {NULL}};
const Segment SGI14110[] = {{&S71168[50], 14}, {NULL}};
const Segment SGI14111[] = {{&S71168[50], 33}, {NULL}};
const Segment SGI14112[] = {{&S71168[30], 12}, {NULL}};
const Segment SGI14113[] = {{&S71168[132], 3}, {NULL}};
const Segment SGI14114[] = {{&S71168[97], 6}, {NULL}};
const Segment SGI14115[] = {{&S71169[50], 14}, {NULL}};
const Segment SGI14116[] = {{&S71169[50], 33}, {NULL}};
const Segment SGI14117[] = {{&S71169[30], 12}, {NULL}};
const Segment SGI14118[] = {{&S71169[132], 3}, {NULL}};
const Segment SGI14119[] = {{&S71169[97], 6}, {NULL}};
const Segment SGI14120[] = {{&S71170[50], 14}, {NULL}};
const Segment SGI14121[] = {{&S71170[50], 33}, {NULL}};
const Segment SGI14122[] = {{&S71170[30], 12}, {NULL}};
const Segment SGI14123[] = {{&S71170[132], 3}, {NULL}};
const Segment SGI14124[] = {{&S71170[97], 6}, {NULL}};
const Segment SGI14125[] = {{&S71552[42], 10}, {NULL}};
const Segment SGI14126[] = {{&S71552[42], 21}, {NULL}};
const Segment SGI14127[] = {{&S71552[30], 4}, {NULL}};
const Segment SGI14128[] = {{&S71552[110], 3}, {NULL}};
const Segment SGI14129[] = {{&S71552[77], 5}, {NULL}};
const Segment SGI14130[] = {{&S72211[44], 11}, {NULL}};
const Segment SGI14131[] = {{&S72211[44], 22}, {NULL}};
const Segment SGI14132[] = {{&S72211[30], 6}, {NULL}};
const Segment SGI14133[] = {{&S72211[114], 3}, {NULL}};
const Segment SGI14134[] = {{&S72211[80], 5}, {NULL}};
const Segment SGI14135[] = {{&S72212[38], 11}, {NULL}};
const Segment SGI14136[] = {{&S72212[31], 6}, {NULL}};
const Segment SGI14137[] = {{&S71573[42], 9}, {NULL}};
const Segment SGI14138[] = {{&S71573[42], 21}, {NULL}};
const Segment SGI14139[] = {{&S71573[30], 4}, {NULL}};
const Segment SGI14140[] = {{&S71573[111], 3}, {NULL}};
const Segment SGI14141[] = {{&S71573[77], 5}, {NULL}};
const Segment SGI14142[] = {{&S71574[36], 9}, {NULL}};
const Segment SGI14143[] = {{&S71574[31], 4}, {NULL}};
const Segment SGI14144[] = {{&S74985[6], 4}, {NULL}};
const Segment SGI14145[] = {{&S74985[81], 5}, {NULL}};
const Segment SGI14146[] = {{&S72477[45], 10}, {NULL}};
const Segment SGI14147[] = {{&S72477[30], 6}, {NULL}};
const Segment SGI14148[] = {{&S72477[114], 3}, {NULL}};
const Segment SGI14149[] = {{&S72477[80], 5}, {NULL}};
const Segment SGI14150[] = {{&S72477[138], 13}, {NULL}};
const Segment SGI14151[] = {{&S75168[42], 21}, {NULL}};
const Segment SGI14152[] = {{&S75168[42], 36}, {NULL}};
const Segment SGI14153[] = {{&S75168[30], 4}, {&S75168[50], 28}, {NULL}};
const Segment SGI14154[] = {{&S75168[127], 3}, {NULL}};
const Segment SGI14155[] = {{&S75168[92], 6}, {NULL}};
const Segment SGI14156[] = {{&S75168[147], 8}, {NULL}};
const Segment SGI14157[] = {{&S75169[42], 21}, {NULL}};
const Segment SGI14158[] = {{&S75169[42], 32}, {NULL}};
const Segment SGI14159[] = {{&S75169[30], 4}, {&S75169[50], 24}, {NULL}};
const Segment SGI14160[] = {{&S75169[123], 3}, {NULL}};
const Segment SGI14161[] = {{&S75169[88], 6}, {NULL}};
const Segment SGI14162[] = {{&S75169[143], 8}, {NULL}};
const Segment SGI14163[] = {{&S75170[42], 21}, {NULL}};
const Segment SGI14164[] = {{&S75170[42], 32}, {NULL}};
const Segment SGI14165[] = {{&S75170[30], 4}, {&S75170[50], 24}, {NULL}};
const Segment SGI14166[] = {{&S75170[123], 3}, {NULL}};
const Segment SGI14167[] = {{&S75170[88], 6}, {NULL}};
const Segment SGI14168[] = {{&S75170[143], 8}, {NULL}};
const Segment SGI14169[] = {{&S75171[42], 21}, {NULL}};
const Segment SGI14170[] = {{&S75171[42], 31}, {NULL}};
const Segment SGI14171[] = {{&S75171[30], 4}, {&S75171[50], 23}, {NULL}};
const Segment SGI14172[] = {{&S75171[122], 3}, {NULL}};
const Segment SGI14173[] = {{&S75171[87], 6}, {NULL}};
const Segment SGI14174[] = {{&S75171[142], 8}, {NULL}};
const Segment SGI14175[] = {{&S72567[6], 4}, {NULL}};
const Segment SGI14176[] = {{&S72567[82], 5}, {NULL}};
const Segment SGI14177[] = {{&S71554[50], 8}, {NULL}};
const Segment SGI14178[] = {{&S71554[50], 20}, {NULL}};
const Segment SGI14179[] = {{&S71554[30], 12}, {NULL}};
const Segment SGI14180[] = {{&S71554[119], 3}, {NULL}};
const Segment SGI14181[] = {{&S71554[84], 6}, {NULL}};
const Segment SGI14182[] = {{&S71555[44], 8}, {NULL}};
const Segment SGI14183[] = {{&S71555[31], 12}, {NULL}};
const Segment SGI14184[] = {{&S75317[42], 6}, {NULL}};
const Segment SGI14185[] = {{&S75317[42], 19}, {NULL}};
const Segment SGI14186[] = {{&S75317[30], 4}, {NULL}};
const Segment SGI14187[] = {{&S75317[110], 3}, {NULL}};
const Segment SGI14188[] = {{&S75317[75], 6}, {NULL}};
const Segment SGI14189[] = {{&S75317[130], 8}, {NULL}};
const Segment SGI14190[] = {{&S73442[82], 6}, {NULL}};
const Segment SGI14191[] = {{&S72767[44], 17}, {NULL}};
const Segment SGI14192[] = {{&S72767[44], 28}, {NULL}};
const Segment SGI14193[] = {{&S72767[30], 6}, {NULL}};
const Segment SGI14194[] = {{&S72767[120], 3}, {NULL}};
const Segment SGI14195[] = {{&S72767[86], 5}, {NULL}};
const Segment SGI14196[] = {{&S72769[44], 11}, {NULL}};
const Segment SGI14197[] = {{&S72769[44], 22}, {NULL}};
const Segment SGI14198[] = {{&S72769[30], 6}, {NULL}};
const Segment SGI14199[] = {{&S72769[114], 3}, {NULL}};
const Segment SGI14200[] = {{&S72769[80], 5}, {NULL}};
const Segment SGI14201[] = {{&S77864[8], 6}, {&S77864[8], 6}, {NULL}};
const Segment SGI14202[] = {{&S77864[0], 90}, {NULL}};
const Segment SGI14203[] = {{&S77865[8], 7}, {NULL}};
const Segment SGI14204[] = {{&S77865[0], 45}, {NULL}};
const Segment SGI14205[] = {{&S77866[8], 6}, {NULL}};
const Segment SGI14206[] = {{&S77866[0], 44}, {NULL}};
const Segment SGI14207[] = {{&S72962[6], 4}, {NULL}};
const Segment SGI14208[] = {{&S72962[76], 5}, {NULL}};
const Segment SGI14209[] = {{&S72968[45], 10}, {NULL}};
const Segment SGI14210[] = {{&S72968[30], 6}, {NULL}};
const Segment SGI14211[] = {{&S72968[114], 3}, {NULL}};
const Segment SGI14212[] = {{&S72968[80], 5}, {NULL}};
const Segment SGI14213[] = {{&S72968[138], 13}, {NULL}};
const Segment SGI14214[] = {{&S72969[44], 10}, {NULL}};
const Segment SGI14215[] = {{&S72969[44], 21}, {NULL}};
const Segment SGI14216[] = {{&S72969[30], 6}, {NULL}};
const Segment SGI14217[] = {{&S72969[114], 3}, {NULL}};
const Segment SGI14218[] = {{&S72969[79], 6}, {NULL}};
const Segment SGI14219[] = {{&S72969[134], 8}, {NULL}};
const Segment SGI14220[] = {{&S72969[143], 13}, {NULL}};
const Segment SGI14221[] = {{&S73543[110], 3}, {NULL}};
const Segment SGI14222[] = {{&S73543[96], 5}, {NULL}};
const Segment SGI14223[] = {{&S72117[42], 13}, {NULL}};
const Segment SGI14224[] = {{&S72117[42], 25}, {NULL}};
const Segment SGI14225[] = {{&S72117[30], 4}, {NULL}};
const Segment SGI14226[] = {{&S72117[115], 3}, {NULL}};
const Segment SGI14227[] = {{&S72117[81], 5}, {NULL}};
const Segment SGI14228[] = {{&S72118[42], 8}, {NULL}};
const Segment SGI14229[] = {{&S72118[42], 20}, {NULL}};
const Segment SGI14230[] = {{&S72118[30], 4}, {NULL}};
const Segment SGI14231[] = {{&S72118[110], 3}, {NULL}};
const Segment SGI14232[] = {{&S72118[76], 5}, {NULL}};
const Segment SGI14233[] = {{&S72119[42], 9}, {NULL}};
const Segment SGI14234[] = {{&S72119[42], 21}, {NULL}};
const Segment SGI14235[] = {{&S72119[30], 4}, {NULL}};
const Segment SGI14236[] = {{&S72119[111], 3}, {NULL}};
const Segment SGI14237[] = {{&S72119[77], 5}, {NULL}};
const Segment SGI14238[] = {{&S72120[36], 13}, {NULL}};
const Segment SGI14239[] = {{&S72120[31], 4}, {NULL}};
const Segment SGI14240[] = {{&S73148[44], 26}, {NULL}};
const Segment SGI14241[] = {{&S73148[30], 6}, {NULL}};
const Segment SGI14242[] = {{&S73148[136], 3}, {NULL}};
const Segment SGI14243[] = {{&S73148[102], 5}, {NULL}};
const Segment SGI14244[] = {{&S72210[42], 11}, {NULL}};
const Segment SGI14245[] = {{&S72210[42], 22}, {NULL}};
const Segment SGI14246[] = {{&S72210[30], 4}, {NULL}};
const Segment SGI14247[] = {{&S72210[112], 3}, {NULL}};
const Segment SGI14248[] = {{&S72210[78], 5}, {NULL}};
const Segment SGI14249[] = {{&S71997[50], 6}, {NULL}};
const Segment SGI14250[] = {{&S71997[50], 17}, {NULL}};
const Segment SGI14251[] = {{&S71997[30], 12}, {NULL}};
const Segment SGI14252[] = {{&S71997[116], 3}, {NULL}};
const Segment SGI14253[] = {{&S71997[81], 6}, {NULL}};
const Segment SGI14254[] = {{&S71998[44], 6}, {NULL}};
const Segment SGI14255[] = {{&S71998[31], 12}, {NULL}};
const Segment SGI14256[] = {{&S72023[50], 5}, {NULL}};
const Segment SGI14257[] = {{&S72023[50], 16}, {NULL}};
const Segment SGI14258[] = {{&S72023[30], 12}, {NULL}};
const Segment SGI14259[] = {{&S72023[115], 3}, {NULL}};
const Segment SGI14260[] = {{&S72023[80], 6}, {NULL}};
const Segment SGI14261[] = {{&S72024[50], 5}, {NULL}};
const Segment SGI14262[] = {{&S72024[50], 16}, {NULL}};
const Segment SGI14263[] = {{&S72024[30], 12}, {NULL}};
const Segment SGI14264[] = {{&S72024[115], 3}, {NULL}};
const Segment SGI14265[] = {{&S72024[80], 6}, {NULL}};
const Segment SGI14266[] = {{&S73195[44], 6}, {NULL}};
const Segment SGI14267[] = {{&S73195[44], 18}, {NULL}};
const Segment SGI14268[] = {{&S73195[30], 6}, {NULL}};
const Segment SGI14269[] = {{&S73195[110], 3}, {NULL}};
const Segment SGI14270[] = {{&S73195[76], 5}, {NULL}};
const Segment SGI14271[] = {{&S73196[44], 7}, {NULL}};
const Segment SGI14272[] = {{&S73196[44], 19}, {NULL}};
const Segment SGI14273[] = {{&S73196[30], 6}, {NULL}};
const Segment SGI14274[] = {{&S73196[111], 3}, {NULL}};
const Segment SGI14275[] = {{&S73196[77], 5}, {NULL}};
const Segment SGI14276[] = {{&S78114[8], 7}, {NULL}};
const Segment SGI14277[] = {{&S78114[0], 85}, {NULL}};
const Segment SGI14278[] = {{&S78115[8], 6}, {NULL}};
const Segment SGI14279[] = {{&S78115[0], 84}, {NULL}};
const Segment SGI14280[] = {{&S78116[8], 6}, {NULL}};
const Segment SGI14281[] = {{&S78116[0], 92}, {NULL}};
const Segment SGI14282[] = {{&S78117[8], 6}, {NULL}};
const Segment SGI14283[] = {{&S78117[0], 92}, {NULL}};
const Segment SGI14284[] = {{&S78118[8], 7}, {NULL}};
const Segment SGI14285[] = {{&S78118[0], 85}, {NULL}};
const Segment SGI14286[] = {{&S72208[50], 7}, {NULL}};
const Segment SGI14287[] = {{&S72208[50], 18}, {NULL}};
const Segment SGI14288[] = {{&S72208[30], 12}, {NULL}};
const Segment SGI14289[] = {{&S72208[117], 3}, {NULL}};
const Segment SGI14290[] = {{&S72208[82], 6}, {NULL}};
const Segment SGI14291[] = {{&S72209[50], 12}, {NULL}};
const Segment SGI14292[] = {{&S72209[50], 23}, {NULL}};
const Segment SGI14293[] = {{&S72209[30], 12}, {NULL}};
const Segment SGI14294[] = {{&S72209[122], 3}, {NULL}};
const Segment SGI14295[] = {{&S72209[87], 6}, {NULL}};
const Segment SGI14296[] = {{&S72447[42], 11}, {NULL}};
const Segment SGI14297[] = {{&S72447[42], 22}, {NULL}};
const Segment SGI14298[] = {{&S72447[30], 4}, {NULL}};
const Segment SGI14299[] = {{&S72447[112], 3}, {NULL}};
const Segment SGI14300[] = {{&S72447[78], 5}, {NULL}};
const Segment SGI14301[] = {{&S72448[42], 7}, {NULL}};
const Segment SGI14302[] = {{&S72448[42], 18}, {NULL}};
const Segment SGI14303[] = {{&S72448[30], 4}, {NULL}};
const Segment SGI14304[] = {{&S72448[108], 3}, {NULL}};
const Segment SGI14305[] = {{&S72448[74], 5}, {NULL}};
const Segment SGI14306[] = {{&S72449[36], 11}, {NULL}};
const Segment SGI14307[] = {{&S72449[31], 4}, {NULL}};
const Segment SGI14308[] = {{&S73524[44], 10}, {NULL}};
const Segment SGI14309[] = {{&S73524[44], 21}, {NULL}};
const Segment SGI14310[] = {{&S73524[30], 6}, {NULL}};
const Segment SGI14311[] = {{&S73524[113], 3}, {NULL}};
const Segment SGI14312[] = {{&S73524[79], 5}, {NULL}};
const Segment SGI14313[] = {{&S72568[6], 4}, {NULL}};
const Segment SGI14314[] = {{&S72568[80], 5}, {NULL}};
const Segment SGI14315[] = {{&S72444[42], 10}, {NULL}};
const Segment SGI14316[] = {{&S72444[42], 22}, {NULL}};
const Segment SGI14317[] = {{&S72444[30], 4}, {NULL}};
const Segment SGI14318[] = {{&S72444[113], 3}, {NULL}};
const Segment SGI14319[] = {{&S72444[78], 6}, {NULL}};
const Segment SGI14320[] = {{&S72445[50], 10}, {NULL}};
const Segment SGI14321[] = {{&S72445[50], 22}, {NULL}};
const Segment SGI14322[] = {{&S72445[30], 12}, {NULL}};
const Segment SGI14323[] = {{&S72445[121], 3}, {NULL}};
const Segment SGI14324[] = {{&S72445[86], 6}, {NULL}};
const Segment SGI14325[] = {{&S72768[42], 17}, {NULL}};
const Segment SGI14326[] = {{&S72768[42], 28}, {NULL}};
const Segment SGI14327[] = {{&S72768[30], 4}, {NULL}};
const Segment SGI14328[] = {{&S72768[118], 3}, {NULL}};
const Segment SGI14329[] = {{&S72768[84], 5}, {NULL}};
const Segment SGI14330[] = {{&S72770[42], 11}, {NULL}};
const Segment SGI14331[] = {{&S72770[42], 23}, {NULL}};
const Segment SGI14332[] = {{&S72770[30], 4}, {NULL}};
const Segment SGI14333[] = {{&S72770[113], 3}, {NULL}};
const Segment SGI14334[] = {{&S72770[79], 5}, {NULL}};
const Segment SGI14335[] = {{&S76030[42], 18}, {NULL}};
const Segment SGI14336[] = {{&S76030[42], 29}, {NULL}};
const Segment SGI14337[] = {{&S76030[30], 4}, {NULL}};
const Segment SGI14338[] = {{&S76030[120], 3}, {NULL}};
const Segment SGI14339[] = {{&S76030[85], 6}, {NULL}};
const Segment SGI14340[] = {{&S76030[140], 8}, {NULL}};
const Segment SGI14341[] = {{&S72538[51], 6}, {NULL}};
const Segment SGI14342[] = {{&S72538[30], 12}, {NULL}};
const Segment SGI14343[] = {{&S72538[122], 3}, {NULL}};
const Segment SGI14344[] = {{&S72538[88], 5}, {NULL}};
const Segment SGI14345[] = {{&S72538[146], 13}, {NULL}};
const Segment SGI14346[] = {{&S72539[51], 6}, {NULL}};
const Segment SGI14347[] = {{&S72539[30], 12}, {NULL}};
const Segment SGI14348[] = {{&S72539[122], 3}, {NULL}};
const Segment SGI14349[] = {{&S72539[88], 5}, {NULL}};
const Segment SGI14350[] = {{&S72539[146], 13}, {NULL}};
const Segment SGI14351[] = {{&S74347[77], 5}, {NULL}};
const Segment SGI14352[] = {{&S73784[6], 4}, {NULL}};
const Segment SGI14353[] = {{&S73784[87], 5}, {NULL}};
const Segment SGI14354[] = {{&S72645[50], 9}, {NULL}};
const Segment SGI14355[] = {{&S72645[50], 20}, {NULL}};
const Segment SGI14356[] = {{&S72645[30], 12}, {NULL}};
const Segment SGI14357[] = {{&S72645[119], 3}, {NULL}};
const Segment SGI14358[] = {{&S72645[84], 6}, {NULL}};
const Segment SGI14359[] = {{&S72646[44], 9}, {NULL}};
const Segment SGI14360[] = {{&S72646[31], 12}, {NULL}};
const Segment SGI14361[] = {{&S72684[50], 7}, {NULL}};
const Segment SGI14362[] = {{&S72684[50], 18}, {NULL}};
const Segment SGI14363[] = {{&S72684[30], 12}, {NULL}};
const Segment SGI14364[] = {{&S72684[117], 3}, {NULL}};
const Segment SGI14365[] = {{&S72684[82], 6}, {NULL}};
const Segment SGI14366[] = {{&S72685[44], 7}, {NULL}};
const Segment SGI14367[] = {{&S72685[31], 12}, {NULL}};
const Segment SGI14368[] = {{&S74473[8], 8}, {NULL}};
const Segment SGI14369[] = {{&S72942[42], 8}, {NULL}};
const Segment SGI14370[] = {{&S72942[42], 19}, {NULL}};
const Segment SGI14371[] = {{&S72942[30], 4}, {NULL}};
const Segment SGI14372[] = {{&S72942[109], 3}, {NULL}};
const Segment SGI14373[] = {{&S72942[75], 5}, {NULL}};
const Segment SGI14374[] = {{&S72945[42], 27}, {NULL}};
const Segment SGI14375[] = {{&S72945[42], 38}, {NULL}};
const Segment SGI14376[] = {{&S72945[30], 4}, {&S72945[50], 30}, {NULL}};
const Segment SGI14377[] = {{&S72945[128], 3}, {NULL}};
const Segment SGI14378[] = {{&S72945[94], 5}, {NULL}};
const Segment SGI14379[] = {{&S72963[6], 4}, {NULL}};
const Segment SGI14380[] = {{&S72963[74], 5}, {NULL}};
const Segment SGI14381[] = {{&S74556[52], 10}, {NULL}};
const Segment SGI14382[] = {{&S74556[52], 21}, {NULL}};
const Segment SGI14383[] = {{&S74556[30], 14}, {NULL}};
const Segment SGI14384[] = {{&S74556[122], 3}, {NULL}};
const Segment SGI14385[] = {{&S74556[87], 6}, {NULL}};
const Segment SGI14386[] = {{&S74557[53], 10}, {NULL}};
const Segment SGI14387[] = {{&S74557[53], 21}, {NULL}};
const Segment SGI14388[] = {{&S74557[30], 15}, {NULL}};
const Segment SGI14389[] = {{&S74557[123], 3}, {NULL}};
const Segment SGI14390[] = {{&S74557[88], 6}, {NULL}};
const Segment SGI14391[] = {{&S74558[49], 10}, {NULL}};
const Segment SGI14392[] = {{&S74558[49], 21}, {NULL}};
const Segment SGI14393[] = {{&S74558[30], 11}, {NULL}};
const Segment SGI14394[] = {{&S74558[119], 3}, {NULL}};
const Segment SGI14395[] = {{&S74558[84], 6}, {NULL}};
const Segment SGI14396[] = {{&S74559[54], 10}, {NULL}};
const Segment SGI14397[] = {{&S74559[54], 21}, {NULL}};
const Segment SGI14398[] = {{&S74559[30], 16}, {NULL}};
const Segment SGI14399[] = {{&S74559[124], 3}, {NULL}};
const Segment SGI14400[] = {{&S74559[89], 6}, {NULL}};
const Segment SGI14401[] = {{&S74560[54], 10}, {NULL}};
const Segment SGI14402[] = {{&S74560[54], 21}, {NULL}};
const Segment SGI14403[] = {{&S74560[30], 16}, {NULL}};
const Segment SGI14404[] = {{&S74560[124], 3}, {NULL}};
const Segment SGI14405[] = {{&S74560[89], 6}, {NULL}};
const Segment SGI14406[] = {{&S74561[54], 10}, {NULL}};
const Segment SGI14407[] = {{&S74561[54], 21}, {NULL}};
const Segment SGI14408[] = {{&S74561[30], 16}, {NULL}};
const Segment SGI14409[] = {{&S74561[124], 3}, {NULL}};
const Segment SGI14410[] = {{&S74561[89], 6}, {NULL}};
const Segment SGI14411[] = {{&S74562[48], 10}, {NULL}};
const Segment SGI14412[] = {{&S74562[31], 16}, {NULL}};
const Segment SGI14413[] = {{&S74563[54], 10}, {NULL}};
const Segment SGI14414[] = {{&S74563[54], 21}, {NULL}};
const Segment SGI14415[] = {{&S74563[30], 16}, {NULL}};
const Segment SGI14416[] = {{&S74563[92], 4}, {NULL}};
const Segment SGI14417[] = {{&S74563[92], 3}, {NULL}};
const Segment SGI14418[] = {{&S76478[40], 7}, {NULL}};
const Segment SGI14419[] = {{&S76478[30], 4}, {NULL}};
const Segment SGI14420[] = {{&S76478[107], 3}, {NULL}};
const Segment SGI14421[] = {{&S76478[72], 6}, {NULL}};
const Segment SGI14422[] = {{&S76478[127], 8}, {NULL}};
const Segment SGI14423[] = {{&S76479[42], 7}, {NULL}};
const Segment SGI14424[] = {{&S76479[42], 18}, {NULL}};
const Segment SGI14425[] = {{&S76479[30], 4}, {NULL}};
const Segment SGI14426[] = {{&S76479[109], 3}, {NULL}};
const Segment SGI14427[] = {{&S76479[74], 6}, {NULL}};
const Segment SGI14428[] = {{&S76479[129], 8}, {NULL}};
const Segment SGI14429[] = {{&S74109[44], 10}, {NULL}};
const Segment SGI14430[] = {{&S74109[44], 22}, {NULL}};
const Segment SGI14431[] = {{&S74109[30], 6}, {NULL}};
const Segment SGI14432[] = {{&S74109[114], 3}, {NULL}};
const Segment SGI14433[] = {{&S74109[80], 5}, {NULL}};
const Segment SGI14434[] = {{&S79181[8], 6}, {NULL}};
const Segment SGI14435[] = {{&S79181[0], 97}, {NULL}};
const Segment SGI14436[] = {{&S79182[8], 7}, {NULL}};
const Segment SGI14437[] = {{&S79182[0], 85}, {NULL}};
const Segment SGI14438[] = {{&S79183[8], 6}, {NULL}};
const Segment SGI14439[] = {{&S79183[0], 84}, {NULL}};
const Segment SGI14440[] = {{&S74194[44], 11}, {NULL}};
const Segment SGI14441[] = {{&S74194[44], 22}, {NULL}};
const Segment SGI14442[] = {{&S74194[30], 6}, {NULL}};
const Segment SGI14443[] = {{&S74194[114], 3}, {NULL}};
const Segment SGI14444[] = {{&S74194[80], 5}, {NULL}};
const Segment SGI14445[] = {{&S74195[38], 11}, {NULL}};
const Segment SGI14446[] = {{&S74195[31], 6}, {NULL}};
const Segment SGI14447[] = {{&S76601[42], 7}, {NULL}};
const Segment SGI14448[] = {{&S76601[42], 20}, {NULL}};
const Segment SGI14449[] = {{&S76601[30], 4}, {NULL}};
const Segment SGI14450[] = {{&S76601[111], 3}, {NULL}};
const Segment SGI14451[] = {{&S76601[76], 6}, {NULL}};
const Segment SGI14452[] = {{&S76601[131], 8}, {NULL}};
const Segment SGI14453[] = {{&S79282[0], 83}, {NULL}};
const Segment SGI14454[] = {{&S79283[0], 83}, {NULL}};
const Segment SGI14455[] = {{&S79284[0], 83}, {NULL}};
const Segment SGI14456[] = {{&S73140[50], 7}, {NULL}};
const Segment SGI14457[] = {{&S73140[50], 18}, {NULL}};
const Segment SGI14458[] = {{&S73140[30], 12}, {NULL}};
const Segment SGI14459[] = {{&S73140[117], 3}, {NULL}};
const Segment SGI14460[] = {{&S73140[82], 6}, {NULL}};
const Segment SGI14461[] = {{&S74267[44], 14}, {NULL}};
const Segment SGI14462[] = {{&S74267[44], 25}, {NULL}};
const Segment SGI14463[] = {{&S74267[30], 6}, {NULL}};
const Segment SGI14464[] = {{&S74267[117], 3}, {NULL}};
const Segment SGI14465[] = {{&S74267[83], 5}, {NULL}};
const Segment SGI14466[] = {{&S74268[38], 14}, {NULL}};
const Segment SGI14467[] = {{&S74268[31], 6}, {NULL}};
const Segment SGI14468[] = {{&S74269[42], 14}, {NULL}};
const Segment SGI14469[] = {{&S74269[30], 6}, {NULL}};
const Segment SGI14470[] = {{&S74269[115], 3}, {NULL}};
const Segment SGI14471[] = {{&S74269[81], 5}, {NULL}};
const Segment SGI14472[] = {{&S73504[42], 17}, {NULL}};
const Segment SGI14473[] = {{&S73504[42], 28}, {NULL}};
const Segment SGI14474[] = {{&S73504[30], 4}, {NULL}};
const Segment SGI14475[] = {{&S73504[118], 3}, {NULL}};
const Segment SGI14476[] = {{&S73504[84], 5}, {NULL}};
const Segment SGI14477[] = {{&S76862[42], 6}, {NULL}};
const Segment SGI14478[] = {{&S76862[42], 17}, {NULL}};
const Segment SGI14479[] = {{&S76862[30], 4}, {NULL}};
const Segment SGI14480[] = {{&S76862[108], 3}, {NULL}};
const Segment SGI14481[] = {{&S76862[73], 6}, {NULL}};
const Segment SGI14482[] = {{&S76862[128], 8}, {NULL}};
const Segment SGI14483[] = {{&S76863[42], 6}, {NULL}};
const Segment SGI14484[] = {{&S76863[42], 17}, {NULL}};
const Segment SGI14485[] = {{&S76863[30], 4}, {NULL}};
const Segment SGI14486[] = {{&S76863[108], 3}, {NULL}};
const Segment SGI14487[] = {{&S76863[73], 6}, {NULL}};
const Segment SGI14488[] = {{&S76863[128], 8}, {NULL}};
const Segment SGI14489[] = {{&S76864[42], 6}, {NULL}};
const Segment SGI14490[] = {{&S76864[42], 17}, {NULL}};
const Segment SGI14491[] = {{&S76864[30], 4}, {NULL}};
const Segment SGI14492[] = {{&S76864[108], 3}, {NULL}};
const Segment SGI14493[] = {{&S76864[73], 6}, {NULL}};
const Segment SGI14494[] = {{&S76864[128], 8}, {NULL}};
const Segment SGI14495[] = {{&S76865[42], 6}, {NULL}};
const Segment SGI14496[] = {{&S76865[42], 17}, {NULL}};
const Segment SGI14497[] = {{&S76865[30], 4}, {NULL}};
const Segment SGI14498[] = {{&S76865[108], 3}, {NULL}};
const Segment SGI14499[] = {{&S76865[73], 6}, {NULL}};
const Segment SGI14500[] = {{&S76865[128], 8}, {NULL}};
const Segment SGI14501[] = {{&S76866[42], 6}, {NULL}};
const Segment SGI14502[] = {{&S76866[42], 17}, {NULL}};
const Segment SGI14503[] = {{&S76866[30], 4}, {NULL}};
const Segment SGI14504[] = {{&S76866[108], 3}, {NULL}};
const Segment SGI14505[] = {{&S76866[73], 6}, {NULL}};
const Segment SGI14506[] = {{&S76866[128], 8}, {NULL}};
const Segment SGI14507[] = {{&S76867[42], 6}, {NULL}};
const Segment SGI14508[] = {{&S76867[42], 17}, {NULL}};
const Segment SGI14509[] = {{&S76867[30], 4}, {NULL}};
const Segment SGI14510[] = {{&S76867[107], 3}, {NULL}};
const Segment SGI14511[] = {{&S76867[73], 5}, {NULL}};
const Segment SGI14512[] = {{&S76867[127], 8}, {NULL}};
const Segment SGI14513[] = {{&S76868[42], 6}, {NULL}};
const Segment SGI14514[] = {{&S76868[42], 17}, {NULL}};
const Segment SGI14515[] = {{&S76868[30], 4}, {NULL}};
const Segment SGI14516[] = {{&S73525[42], 10}, {NULL}};
const Segment SGI14517[] = {{&S73525[42], 21}, {NULL}};
const Segment SGI14518[] = {{&S76868[108], 3}, {NULL}};
const Segment SGI14519[] = {{&S73525[30], 4}, {NULL}};
const Segment SGI14520[] = {{&S76868[73], 6}, {NULL}};
const Segment SGI14521[] = {{&S76868[128], 8}, {NULL}};
const Segment SGI14522[] = {{&S73525[80], 4}, {NULL}};
const Segment SGI14523[] = {{&S73525[80], 3}, {NULL}};
const Segment SGI14524[] = {{&S76869[42], 6}, {NULL}};
const Segment SGI14525[] = {{&S76869[42], 17}, {NULL}};
const Segment SGI14526[] = {{&S76869[30], 4}, {NULL}};
const Segment SGI14527[] = {{&S73526[42], 10}, {NULL}};
const Segment SGI14528[] = {{&S73526[42], 21}, {NULL}};
const Segment SGI14529[] = {{&S76869[108], 3}, {NULL}};
const Segment SGI14530[] = {{&S73526[30], 4}, {NULL}};
const Segment SGI14531[] = {{&S76869[73], 6}, {NULL}};
const Segment SGI14532[] = {{&S76869[128], 8}, {NULL}};
const Segment SGI14533[] = {{&S73526[111], 3}, {NULL}};
const Segment SGI14534[] = {{&S73526[77], 5}, {NULL}};
const Segment SGI14535[] = {{&S76870[42], 6}, {NULL}};
const Segment SGI14536[] = {{&S76870[42], 17}, {NULL}};
const Segment SGI14537[] = {{&S76870[30], 4}, {NULL}};
const Segment SGI14538[] = {{&S73527[36], 10}, {NULL}};
const Segment SGI14539[] = {{&S73527[31], 4}, {NULL}};
const Segment SGI14540[] = {{&S76870[108], 3}, {NULL}};
const Segment SGI14541[] = {{&S76870[73], 6}, {NULL}};
const Segment SGI14542[] = {{&S76870[128], 8}, {NULL}};
const Segment SGI14543[] = {{&S73528[42], 10}, {NULL}};
const Segment SGI14544[] = {{&S73528[42], 21}, {NULL}};
const Segment SGI14545[] = {{&S73528[30], 4}, {NULL}};
const Segment SGI14546[] = {{&S73528[80], 6}, {NULL}};
const Segment SGI14547[] = {{&S73396[6], 4}, {NULL}};
const Segment SGI14548[] = {{&S73396[96], 5}, {NULL}};
const Segment SGI14549[] = {{&S74604[45], 10}, {NULL}};
const Segment SGI14550[] = {{&S74604[30], 6}, {NULL}};
const Segment SGI14551[] = {{&S74604[114], 3}, {NULL}};
const Segment SGI14552[] = {{&S74604[80], 5}, {NULL}};
const Segment SGI14553[] = {{&S74604[138], 13}, {NULL}};
const Segment SGI14554[] = {{&S74621[44], 9}, {NULL}};
const Segment SGI14555[] = {{&S74621[44], 20}, {NULL}};
const Segment SGI14556[] = {{&S74621[30], 6}, {NULL}};
const Segment SGI14557[] = {{&S74621[112], 3}, {NULL}};
const Segment SGI14558[] = {{&S74621[78], 5}, {NULL}};
const Segment SGI14559[] = {{&S74622[38], 9}, {NULL}};
const Segment SGI14560[] = {{&S74622[31], 6}, {NULL}};
const Segment SGI14561[] = {{&S73884[97], 5}, {NULL}};
const Segment SGI14562[] = {{&S79852[0], 107}, {NULL}};
const Segment SGI14563[] = {{&S74720[44], 11}, {NULL}};
const Segment SGI14564[] = {{&S74720[44], 22}, {NULL}};
const Segment SGI14565[] = {{&S74720[30], 6}, {NULL}};
const Segment SGI14566[] = {{&S74720[114], 3}, {NULL}};
const Segment SGI14567[] = {{&S74720[80], 5}, {NULL}};
const Segment SGI14568[] = {{&S77368[42], 15}, {NULL}};
const Segment SGI14569[] = {{&S77368[42], 26}, {NULL}};
const Segment SGI14570[] = {{&S77368[30], 4}, {NULL}};
const Segment SGI14571[] = {{&S77368[117], 3}, {NULL}};
const Segment SGI14572[] = {{&S77368[82], 6}, {NULL}};
const Segment SGI14573[] = {{&S77368[137], 8}, {NULL}};
const Segment SGI14574[] = {{&S77369[42], 8}, {NULL}};
const Segment SGI14575[] = {{&S77369[42], 19}, {NULL}};
const Segment SGI14576[] = {{&S77369[30], 4}, {NULL}};
const Segment SGI14577[] = {{&S77369[110], 3}, {NULL}};
const Segment SGI14578[] = {{&S77369[75], 6}, {NULL}};
const Segment SGI14579[] = {{&S77369[130], 8}, {NULL}};
const Segment SGI14580[] = {{&S75182[8], 8}, {NULL}};
const Segment SGI14581[] = {{&S75244[82], 6}, {NULL}};
const Segment SGI14582[] = {{&S75519[8], 8}, {NULL}};
const Segment SGI14583[] = {{&S78008[42], 7}, {NULL}};
const Segment SGI14584[] = {{&S78008[42], 18}, {NULL}};
const Segment SGI14585[] = {{&S78008[30], 4}, {NULL}};
const Segment SGI14586[] = {{&S78008[109], 3}, {NULL}};
const Segment SGI14587[] = {{&S78008[74], 6}, {NULL}};
const Segment SGI14588[] = {{&S78008[129], 8}, {NULL}};
const Segment SGI14589[] = {{&S75714[8], 8}, {NULL}};
const Segment SGI14590[] = {{&S74721[42], 11}, {NULL}};
const Segment SGI14591[] = {{&S74721[42], 22}, {NULL}};
const Segment SGI14592[] = {{&S74721[30], 4}, {NULL}};
const Segment SGI14593[] = {{&S74721[112], 3}, {NULL}};
const Segment SGI14594[] = {{&S74721[78], 5}, {NULL}};
const Segment SGI14595[] = {{&S75509[44], 6}, {NULL}};
const Segment SGI14596[] = {{&S75509[44], 18}, {NULL}};
const Segment SGI14597[] = {{&S75509[30], 6}, {NULL}};
const Segment SGI14598[] = {{&S75509[110], 3}, {NULL}};
const Segment SGI14599[] = {{&S75509[76], 5}, {NULL}};
const Segment SGI14600[] = {{&S75567[44], 11}, {NULL}};
const Segment SGI14601[] = {{&S75567[44], 22}, {NULL}};
const Segment SGI14602[] = {{&S75567[30], 6}, {NULL}};
const Segment SGI14603[] = {{&S75567[114], 3}, {NULL}};
const Segment SGI14604[] = {{&S75567[80], 5}, {NULL}};
const Segment SGI14605[] = {{&S75735[44], 14}, {NULL}};
const Segment SGI14606[] = {{&S75735[44], 25}, {NULL}};
const Segment SGI14607[] = {{&S75735[30], 6}, {NULL}};
const Segment SGI14608[] = {{&S75735[117], 3}, {NULL}};
const Segment SGI14609[] = {{&S75735[83], 5}, {NULL}};
const Segment SGI14610[] = {{&S75736[42], 14}, {NULL}};
const Segment SGI14611[] = {{&S75736[30], 6}, {NULL}};
const Segment SGI14612[] = {{&S75736[115], 3}, {NULL}};
const Segment SGI14613[] = {{&S75736[81], 5}, {NULL}};
const Segment SGI14614[] = {{&S75737[38], 14}, {NULL}};
const Segment SGI14615[] = {{&S75737[31], 6}, {NULL}};
const Segment SGI14616[] = {{&S74588[42], 16}, {NULL}};
const Segment SGI14617[] = {{&S74588[30], 4}, {NULL}};
const Segment SGI14618[] = {{&S74588[107], 3}, {NULL}};
const Segment SGI14619[] = {{&S74588[72], 6}, {NULL}};
const Segment SGI14620[] = {{&S74636[51], 6}, {NULL}};
const Segment SGI14621[] = {{&S74636[30], 12}, {NULL}};
const Segment SGI14622[] = {{&S74636[122], 3}, {NULL}};
const Segment SGI14623[] = {{&S74636[88], 5}, {NULL}};
const Segment SGI14624[] = {{&S74636[146], 13}, {NULL}};
const Segment SGI14625[] = {{&S74637[50], 6}, {NULL}};
const Segment SGI14626[] = {{&S74637[50], 23}, {NULL}};
const Segment SGI14627[] = {{&S74637[30], 12}, {NULL}};
const Segment SGI14628[] = {{&S74637[122], 3}, {NULL}};
const Segment SGI14629[] = {{&S74637[87], 6}, {NULL}};
const Segment SGI14630[] = {{&S74637[142], 8}, {NULL}};
const Segment SGI14631[] = {{&S74637[151], 13}, {NULL}};
const Segment SGI14632[] = {{&S75042[42], 14}, {NULL}};
const Segment SGI14633[] = {{&S75042[42], 25}, {NULL}};
const Segment SGI14634[] = {{&S75042[30], 4}, {NULL}};
const Segment SGI14635[] = {{&S75042[115], 3}, {NULL}};
const Segment SGI14636[] = {{&S75042[81], 5}, {NULL}};
const Segment SGI14637[] = {{&S75965[31], 8}, {NULL}};
const Segment SGI14638[] = {{&S75965[54], 3}, {NULL}};
const Segment SGI14639[] = {{&S75966[39], 8}, {NULL}};
const Segment SGI14640[] = {{&S75966[103], 3}, {NULL}};
const Segment SGI14641[] = {{&S75966[92], 3}, {NULL}};
const Segment SGI14642[] = {{&S75966[69], 6}, {NULL}};
const Segment SGI14643[] = {{&S81309[8], 6}, {NULL}};
const Segment SGI14644[] = {{&S81309[0], 84}, {NULL}};
const Segment SGI14645[] = {{&S81310[8], 7}, {NULL}};
const Segment SGI14646[] = {{&S81310[0], 85}, {NULL}};
const Segment SGI14647[] = {{&S81311[8], 6}, {&S81311[8], 6}, {NULL}};
const Segment SGI14648[] = {{&S81311[0], 170}, {NULL}};
const Segment SGI14649[] = {{&S81312[8], 6}, {NULL}};
const Segment SGI14650[] = {{&S81312[0], 97}, {NULL}};
const Segment SGI14651[] = {{&S76184[44], 7}, {NULL}};
const Segment SGI14652[] = {{&S76184[44], 18}, {NULL}};
const Segment SGI14653[] = {{&S76184[30], 6}, {NULL}};
const Segment SGI14654[] = {{&S76184[110], 3}, {NULL}};
const Segment SGI14655[] = {{&S76184[76], 5}, {NULL}};
const Segment SGI14656[] = {{&S75277[42], 27}, {NULL}};
const Segment SGI14657[] = {{&S75277[42], 38}, {NULL}};
const Segment SGI14658[] = {{&S75277[30], 4}, {&S75277[50], 30}, {NULL}};
const Segment SGI14659[] = {{&S75277[128], 3}, {NULL}};
const Segment SGI14660[] = {{&S75277[94], 5}, {NULL}};
const Segment SGI14661[] = {{&S76300[44], 10}, {NULL}};
const Segment SGI14662[] = {{&S76300[44], 21}, {NULL}};
const Segment SGI14663[] = {{&S76300[30], 6}, {NULL}};
const Segment SGI14664[] = {{&S76300[79], 6}, {NULL}};
const Segment SGI14665[] = {{&S76300[106], 13}, {NULL}};
const Segment SGI14666[] = {{&S76662[45], 8}, {NULL}};
const Segment SGI14667[] = {{&S76832[82], 6}, {NULL}};
const Segment SGI14668[] = {{&S76876[8], 8}, {NULL}};
const Segment SGI14669[] = {{&S79018[65], 8}, {NULL}};
const Segment SGI14670[] = {{&S79018[65], 19}, {NULL}};
const Segment SGI14671[] = {{&S79018[53], 4}, {NULL}};
const Segment SGI14672[] = {{&S79018[133], 3}, {NULL}};
const Segment SGI14673[] = {{&S79018[98], 6}, {NULL}};
const Segment SGI14674[] = {{&S79018[153], 8}, {NULL}};
const Segment SGI14675[] = {{&S75586[97], 5}, {NULL}};
const Segment SGI14676[] = {{&S75443[50], 16}, {NULL}};
const Segment SGI14677[] = {{&S75443[50], 27}, {NULL}};
const Segment SGI14678[] = {{&S75443[30], 12}, {NULL}};
const Segment SGI14679[] = {{&S75443[126], 3}, {NULL}};
const Segment SGI14680[] = {{&S75443[91], 6}, {NULL}};
const Segment SGI14681[] = {{&S75444[50], 16}, {NULL}};
const Segment SGI14682[] = {{&S75444[50], 27}, {NULL}};
const Segment SGI14683[] = {{&S75444[30], 12}, {NULL}};
const Segment SGI14684[] = {{&S75444[126], 3}, {NULL}};
const Segment SGI14685[] = {{&S75444[91], 6}, {NULL}};
const Segment SGI14686[] = {{&S79147[6], 4}, {NULL}};
const Segment SGI14687[] = {{&S79147[74], 5}, {NULL}};
const Segment SGI14688[] = {{&S75470[50], 8}, {NULL}};
const Segment SGI14689[] = {{&S75470[50], 31}, {NULL}};
const Segment SGI14690[] = {{&S75470[30], 12}, {NULL}};
const Segment SGI14691[] = {{&S75470[130], 3}, {NULL}};
const Segment SGI14692[] = {{&S75470[95], 6}, {NULL}};
const Segment SGI14693[] = {{&S75471[50], 8}, {NULL}};
const Segment SGI14694[] = {{&S75471[50], 31}, {NULL}};
const Segment SGI14695[] = {{&S75471[30], 12}, {NULL}};
const Segment SGI14696[] = {{&S75471[130], 3}, {NULL}};
const Segment SGI14697[] = {{&S75471[95], 6}, {NULL}};
const Segment SGI14698[] = {{&S75472[50], 7}, {NULL}};
const Segment SGI14699[] = {{&S75472[50], 28}, {NULL}};
const Segment SGI14700[] = {{&S75472[30], 12}, {NULL}};
const Segment SGI14701[] = {{&S75472[95], 4}, {NULL}};
const Segment SGI14702[] = {{&S75472[95], 3}, {NULL}};
const Segment SGI14703[] = {{&S75473[50], 7}, {NULL}};
const Segment SGI14704[] = {{&S75473[50], 28}, {NULL}};
const Segment SGI14705[] = {{&S75473[30], 12}, {NULL}};
const Segment SGI14706[] = {{&S75473[127], 3}, {NULL}};
const Segment SGI14707[] = {{&S75473[92], 6}, {NULL}};
const Segment SGI14708[] = {{&S75474[50], 7}, {NULL}};
const Segment SGI14709[] = {{&S75474[50], 28}, {NULL}};
const Segment SGI14710[] = {{&S75474[30], 12}, {NULL}};
const Segment SGI14711[] = {{&S75474[95], 4}, {NULL}};
const Segment SGI14712[] = {{&S75474[95], 3}, {NULL}};
const Segment SGI14713[] = {{&S75475[50], 7}, {NULL}};
const Segment SGI14714[] = {{&S75475[50], 28}, {NULL}};
const Segment SGI14715[] = {{&S75475[30], 12}, {NULL}};
const Segment SGI14716[] = {{&S75475[127], 3}, {NULL}};
const Segment SGI14717[] = {{&S75475[92], 6}, {NULL}};
const Segment SGI14718[] = {{&S75476[50], 7}, {NULL}};
const Segment SGI14719[] = {{&S75476[50], 28}, {NULL}};
const Segment SGI14720[] = {{&S75476[30], 12}, {NULL}};
const Segment SGI14721[] = {{&S75476[127], 3}, {NULL}};
const Segment SGI14722[] = {{&S75476[92], 6}, {NULL}};
const Segment SGI14723[] = {{&S75477[44], 8}, {NULL}};
const Segment SGI14724[] = {{&S75477[31], 12}, {NULL}};
const Segment SGI14725[] = {{&S79180[42], 6}, {NULL}};
const Segment SGI14726[] = {{&S79180[42], 17}, {NULL}};
const Segment SGI14727[] = {{&S79180[30], 4}, {NULL}};
const Segment SGI14728[] = {{&S79180[108], 3}, {NULL}};
const Segment SGI14729[] = {{&S79180[73], 6}, {NULL}};
const Segment SGI14730[] = {{&S79180[128], 8}, {NULL}};
const Segment SGI14731[] = {{&S75478[50], 7}, {NULL}};
const Segment SGI14732[] = {{&S75478[50], 28}, {NULL}};
const Segment SGI14733[] = {{&S75478[30], 12}, {NULL}};
const Segment SGI14734[] = {{&S75478[95], 4}, {NULL}};
const Segment SGI14735[] = {{&S75478[95], 3}, {NULL}};
const Segment SGI14736[] = {{&S75479[50], 7}, {NULL}};
const Segment SGI14737[] = {{&S75479[50], 28}, {NULL}};
const Segment SGI14738[] = {{&S75479[30], 12}, {NULL}};
const Segment SGI14739[] = {{&S75479[127], 3}, {NULL}};
const Segment SGI14740[] = {{&S75479[92], 6}, {NULL}};
const Segment SGI14741[] = {{&S75480[50], 7}, {NULL}};
const Segment SGI14742[] = {{&S75480[50], 28}, {NULL}};
const Segment SGI14743[] = {{&S75480[30], 12}, {NULL}};
const Segment SGI14744[] = {{&S75480[95], 6}, {NULL}};
const Segment SGI14745[] = {{&S77131[93], 6}, {NULL}};
const Segment SGI14746[] = {{&S79301[42], 6}, {NULL}};
const Segment SGI14747[] = {{&S79301[42], 19}, {NULL}};
const Segment SGI14748[] = {{&S79301[30], 4}, {NULL}};
const Segment SGI14749[] = {{&S79301[111], 3}, {NULL}};
const Segment SGI14750[] = {{&S79301[75], 6}, {NULL}};
const Segment SGI14751[] = {{&S79301[131], 8}, {NULL}};
const Segment SGI14752[] = {{&S75619[56], 9}, {NULL}};
const Segment SGI14753[] = {{&S75619[56], 21}, {NULL}};
const Segment SGI14754[] = {{&S75619[30], 18}, {NULL}};
const Segment SGI14755[] = {{&S75619[126], 3}, {NULL}};
const Segment SGI14756[] = {{&S75619[91], 6}, {NULL}};
const Segment SGI14757[] = {{&S76804[44], 8}, {NULL}};
const Segment SGI14758[] = {{&S76804[44], 19}, {NULL}};
const Segment SGI14759[] = {{&S76804[30], 6}, {NULL}};
const Segment SGI14760[] = {{&S76804[111], 3}, {NULL}};
const Segment SGI14761[] = {{&S76804[77], 5}, {NULL}};
const Segment SGI14762[] = {{&S76805[44], 7}, {NULL}};
const Segment SGI14763[] = {{&S76805[44], 18}, {NULL}};
const Segment SGI14764[] = {{&S76805[30], 6}, {NULL}};
const Segment SGI14765[] = {{&S76805[110], 3}, {NULL}};
const Segment SGI14766[] = {{&S76805[76], 5}, {NULL}};
const Segment SGI14767[] = {{&S79364[33], 4}, {NULL}};
const Segment SGI14768[] = {{&S79364[82], 3}, {NULL}};
const Segment SGI14769[] = {{&S75823[42], 5}, {NULL}};
const Segment SGI14770[] = {{&S79364[67], 6}, {NULL}};
const Segment SGI14771[] = {{&S75823[42], 16}, {NULL}};
const Segment SGI14772[] = {{&S75823[30], 4}, {NULL}};
const Segment SGI14773[] = {{&S75823[106], 3}, {NULL}};
const Segment SGI14774[] = {{&S75823[72], 5}, {NULL}};
const Segment SGI14775[] = {{&S75689[45], 4}, {NULL}};
const Segment SGI14776[] = {{&S75689[105], 3}, {NULL}};
const Segment SGI14777[] = {{&S75689[94], 3}, {NULL}};
const Segment SGI14778[] = {{&S75689[71], 6}, {NULL}};
const Segment SGI14779[] = {{&S75690[37], 4}, {NULL}};
const Segment SGI14780[] = {{&S75690[56], 3}, {NULL}};
const Segment SGI14781[] = {{&S77349[81], 5}, {NULL}};
const Segment SGI14782[] = {{&S76872[44], 10}, {NULL}};
const Segment SGI14783[] = {{&S76872[44], 19}, {NULL}};
const Segment SGI14784[] = {{&S76872[30], 6}, {NULL}};
const Segment SGI14785[] = {{&S76872[111], 3}, {NULL}};
const Segment SGI14786[] = {{&S76872[77], 5}, {NULL}};
const Segment SGI14787[] = {{&S76873[38], 10}, {NULL}};
const Segment SGI14788[] = {{&S76873[31], 6}, {NULL}};
const Segment SGI14789[] = {{&S77430[83], 6}, {NULL}};
const Segment SGI14790[] = {{&S75921[50], 6}, {NULL}};
const Segment SGI14791[] = {{&S75921[50], 17}, {NULL}};
const Segment SGI14792[] = {{&S75921[30], 12}, {NULL}};
const Segment SGI14793[] = {{&S75921[116], 3}, {NULL}};
const Segment SGI14794[] = {{&S75921[81], 6}, {NULL}};
const Segment SGI14795[] = {{&S75922[44], 6}, {NULL}};
const Segment SGI14796[] = {{&S75922[31], 12}, {NULL}};
const Segment SGI14797[] = {{&S76064[50], 6}, {NULL}};
const Segment SGI14798[] = {{&S76064[50], 18}, {NULL}};
const Segment SGI14799[] = {{&S76064[30], 12}, {NULL}};
const Segment SGI14800[] = {{&S76064[117], 3}, {NULL}};
const Segment SGI14801[] = {{&S76064[82], 6}, {NULL}};
const Segment SGI14802[] = {{&S76065[50], 6}, {NULL}};
const Segment SGI14803[] = {{&S76065[50], 18}, {NULL}};
const Segment SGI14804[] = {{&S76065[30], 12}, {NULL}};
const Segment SGI14805[] = {{&S76065[117], 3}, {NULL}};
const Segment SGI14806[] = {{&S76065[82], 6}, {NULL}};
const Segment SGI14807[] = {{&S76066[50], 6}, {NULL}};
const Segment SGI14808[] = {{&S76066[50], 18}, {NULL}};
const Segment SGI14809[] = {{&S76066[30], 12}, {NULL}};
const Segment SGI14810[] = {{&S76066[117], 3}, {NULL}};
const Segment SGI14811[] = {{&S76066[82], 6}, {NULL}};
const Segment SGI14812[] = {{&S76264[42], 6}, {NULL}};
const Segment SGI14813[] = {{&S76264[42], 17}, {NULL}};
const Segment SGI14814[] = {{&S76264[30], 4}, {NULL}};
const Segment SGI14815[] = {{&S76264[107], 3}, {NULL}};
const Segment SGI14816[] = {{&S76264[73], 5}, {NULL}};
const Segment SGI14817[] = {{&S76266[42], 12}, {NULL}};
const Segment SGI14818[] = {{&S76266[42], 23}, {NULL}};
const Segment SGI14819[] = {{&S76266[30], 4}, {NULL}};
const Segment SGI14820[] = {{&S76266[113], 3}, {NULL}};
const Segment SGI14821[] = {{&S76266[79], 5}, {NULL}};
const Segment SGI14822[] = {{&S76110[50], 15}, {NULL}};
const Segment SGI14823[] = {{&S76110[50], 26}, {NULL}};
const Segment SGI14824[] = {{&S76110[30], 12}, {NULL}};
const Segment SGI14825[] = {{&S76110[125], 3}, {NULL}};
const Segment SGI14826[] = {{&S76110[90], 6}, {NULL}};
const Segment SGI14827[] = {{&S76267[42], 6}, {NULL}};
const Segment SGI14828[] = {{&S76267[42], 17}, {NULL}};
const Segment SGI14829[] = {{&S76267[30], 4}, {NULL}};
const Segment SGI14830[] = {{&S76267[108], 3}, {NULL}};
const Segment SGI14831[] = {{&S76267[73], 6}, {NULL}};
const Segment SGI14832[] = {{&S76268[42], 25}, {NULL}};
const Segment SGI14833[] = {{&S76268[30], 4}, {NULL}};
const Segment SGI14834[] = {{&S76268[133], 3}, {NULL}};
const Segment SGI14835[] = {{&S76268[99], 5}, {NULL}};
const Segment SGI14836[] = {{&S76291[99], 4}, {NULL}};
const Segment SGI14837[] = {{&S76291[151], 3}, {NULL}};
const Segment SGI14838[] = {{&S76291[117], 5}, {NULL}};
const Segment SGI14839[] = {{&S76422[42], 8}, {NULL}};
const Segment SGI14840[] = {{&S76422[42], 20}, {NULL}};
const Segment SGI14841[] = {{&S76422[30], 4}, {NULL}};
const Segment SGI14842[] = {{&S76422[110], 3}, {NULL}};
const Segment SGI14843[] = {{&S76422[76], 5}, {NULL}};
const Segment SGI14844[] = {{&S76423[36], 8}, {NULL}};
const Segment SGI14845[] = {{&S76423[31], 4}, {NULL}};
const Segment SGI14846[] = {{&S76430[42], 11}, {NULL}};
const Segment SGI14847[] = {{&S76430[42], 22}, {NULL}};
const Segment SGI14848[] = {{&S76430[30], 4}, {NULL}};
const Segment SGI14849[] = {{&S76430[112], 3}, {NULL}};
const Segment SGI14850[] = {{&S76430[78], 5}, {NULL}};
const Segment SGI14851[] = {{&S76259[50], 7}, {NULL}};
const Segment SGI14852[] = {{&S76259[50], 18}, {NULL}};
const Segment SGI14853[] = {{&S76259[30], 12}, {NULL}};
const Segment SGI14854[] = {{&S76259[117], 3}, {NULL}};
const Segment SGI14855[] = {{&S76259[82], 6}, {NULL}};
const Segment SGI14856[] = {{&S76260[42], 7}, {NULL}};
const Segment SGI14857[] = {{&S76260[42], 18}, {NULL}};
const Segment SGI14858[] = {{&S76260[30], 4}, {NULL}};
const Segment SGI14859[] = {{&S76260[109], 3}, {NULL}};
const Segment SGI14860[] = {{&S76260[74], 6}, {NULL}};
const Segment SGI14861[] = {{&S76261[44], 7}, {NULL}};
const Segment SGI14862[] = {{&S76261[31], 12}, {NULL}};
const Segment SGI14863[] = {{&S77587[6], 4}, {NULL}};
const Segment SGI14864[] = {{&S77587[84], 5}, {NULL}};
const Segment SGI14865[] = {{&S77589[44], 9}, {NULL}};
const Segment SGI14866[] = {{&S77589[44], 21}, {NULL}};
const Segment SGI14867[] = {{&S77589[30], 6}, {NULL}};
const Segment SGI14868[] = {{&S77589[113], 3}, {NULL}};
const Segment SGI14869[] = {{&S77589[79], 5}, {NULL}};
const Segment SGI14870[] = {{&S76431[50], 14}, {NULL}};
const Segment SGI14871[] = {{&S76431[50], 25}, {NULL}};
const Segment SGI14872[] = {{&S76431[30], 12}, {NULL}};
const Segment SGI14873[] = {{&S76431[124], 3}, {NULL}};
const Segment SGI14874[] = {{&S76431[89], 6}, {NULL}};
const Segment SGI14875[] = {{&S76432[44], 14}, {NULL}};
const Segment SGI14876[] = {{&S76432[31], 12}, {NULL}};
const Segment SGI14877[] = {{&S76688[42], 7}, {NULL}};
const Segment SGI14878[] = {{&S76688[42], 18}, {NULL}};
const Segment SGI14879[] = {{&S76688[30], 4}, {NULL}};
const Segment SGI14880[] = {{&S76688[108], 3}, {NULL}};
const Segment SGI14881[] = {{&S76688[74], 5}, {NULL}};
const Segment SGI14882[] = {{&S77630[44], 9}, {NULL}};
const Segment SGI14883[] = {{&S77630[44], 21}, {NULL}};
const Segment SGI14884[] = {{&S77630[30], 6}, {NULL}};
const Segment SGI14885[] = {{&S77630[113], 3}, {NULL}};
const Segment SGI14886[] = {{&S77630[79], 5}, {NULL}};
const Segment SGI14887[] = {{&S76684[50], 6}, {NULL}};
const Segment SGI14888[] = {{&S76684[50], 18}, {NULL}};
const Segment SGI14889[] = {{&S76684[30], 12}, {NULL}};
const Segment SGI14890[] = {{&S76684[117], 3}, {NULL}};
const Segment SGI14891[] = {{&S76684[82], 6}, {NULL}};
const Segment SGI14892[] = {{&S76685[50], 6}, {NULL}};
const Segment SGI14893[] = {{&S76685[50], 18}, {NULL}};
const Segment SGI14894[] = {{&S76685[30], 12}, {NULL}};
const Segment SGI14895[] = {{&S76686[44], 6}, {NULL}};
const Segment SGI14896[] = {{&S76686[31], 12}, {NULL}};
const Segment SGI14897[] = {{&S77003[42], 14}, {NULL}};
const Segment SGI14898[] = {{&S77003[42], 25}, {NULL}};
const Segment SGI14899[] = {{&S77003[30], 4}, {NULL}};
const Segment SGI14900[] = {{&S77003[115], 3}, {NULL}};
const Segment SGI14901[] = {{&S77003[81], 5}, {NULL}};
const Segment SGI14902[] = {{&S77004[36], 14}, {NULL}};
const Segment SGI14903[] = {{&S77004[31], 4}, {NULL}};
const Segment SGI14904[] = {{&S83190[0], 107}, {NULL}};
const Segment SGI14905[] = {{&S83191[0], 107}, {NULL}};
const Segment SGI14906[] = {{&S80399[42], 14}, {NULL}};
const Segment SGI14907[] = {{&S80399[42], 25}, {NULL}};
const Segment SGI14908[] = {{&S80399[30], 4}, {NULL}};
const Segment SGI14909[] = {{&S80399[116], 3}, {NULL}};
const Segment SGI14910[] = {{&S80399[81], 6}, {NULL}};
const Segment SGI14911[] = {{&S80399[136], 8}, {NULL}};
const Segment SGI14912[] = {{&S80400[42], 11}, {NULL}};
const Segment SGI14913[] = {{&S80400[42], 22}, {NULL}};
const Segment SGI14914[] = {{&S80400[30], 4}, {NULL}};
const Segment SGI14915[] = {{&S80400[113], 3}, {NULL}};
const Segment SGI14916[] = {{&S80400[78], 6}, {NULL}};
const Segment SGI14917[] = {{&S80400[133], 8}, {NULL}};
const Segment SGI14918[] = {{&S80401[42], 11}, {NULL}};
const Segment SGI14919[] = {{&S80401[42], 22}, {NULL}};
const Segment SGI14920[] = {{&S80401[30], 4}, {NULL}};
const Segment SGI14921[] = {{&S80401[113], 3}, {NULL}};
const Segment SGI14922[] = {{&S80401[78], 6}, {NULL}};
const Segment SGI14923[] = {{&S80401[133], 8}, {NULL}};
const Segment SGI14924[] = {{&S77065[50], 8}, {NULL}};
const Segment SGI14925[] = {{&S77065[50], 19}, {NULL}};
const Segment SGI14926[] = {{&S77065[30], 12}, {NULL}};
const Segment SGI14927[] = {{&S77065[118], 3}, {NULL}};
const Segment SGI14928[] = {{&S77065[83], 6}, {NULL}};
const Segment SGI14929[] = {{&S77066[50], 8}, {NULL}};
const Segment SGI14930[] = {{&S77066[50], 19}, {NULL}};
const Segment SGI14931[] = {{&S77066[30], 12}, {NULL}};
const Segment SGI14932[] = {{&S77066[86], 4}, {NULL}};
const Segment SGI14933[] = {{&S77066[86], 3}, {NULL}};
const Segment SGI14934[] = {{&S78595[84], 6}, {NULL}};
const Segment SGI14935[] = {{&S78617[121], 3}, {NULL}};
const Segment SGI14936[] = {{&S78617[86], 6}, {NULL}};
const Segment SGI14937[] = {{&S77282[51], 6}, {NULL}};
const Segment SGI14938[] = {{&S77282[30], 12}, {NULL}};
const Segment SGI14939[] = {{&S77282[123], 3}, {NULL}};
const Segment SGI14940[] = {{&S77282[89], 5}, {NULL}};
const Segment SGI14941[] = {{&S77282[147], 13}, {NULL}};
const Segment SGI14942[] = {{&S78218[44], 10}, {NULL}};
const Segment SGI14943[] = {{&S78218[44], 21}, {NULL}};
const Segment SGI14944[] = {{&S78218[30], 6}, {NULL}};
const Segment SGI14945[] = {{&S78218[113], 3}, {NULL}};
const Segment SGI14946[] = {{&S78218[79], 5}, {NULL}};
const Segment SGI14947[] = {{&S77586[6], 4}, {NULL}};
const Segment SGI14948[] = {{&S77586[79], 5}, {NULL}};
const Segment SGI14949[] = {{&S78425[44], 14}, {NULL}};
const Segment SGI14950[] = {{&S78425[44], 25}, {NULL}};
const Segment SGI14951[] = {{&S78425[30], 6}, {NULL}};
const Segment SGI14952[] = {{&S78425[117], 3}, {NULL}};
const Segment SGI14953[] = {{&S78425[83], 5}, {NULL}};
const Segment SGI14954[] = {{&S78447[45], 11}, {NULL}};
const Segment SGI14955[] = {{&S78447[30], 6}, {NULL}};
const Segment SGI14956[] = {{&S78447[115], 3}, {NULL}};
const Segment SGI14957[] = {{&S78447[81], 5}, {NULL}};
const Segment SGI14958[] = {{&S78447[139], 13}, {NULL}};
const Segment SGI14959[] = {{&S78488[67], 8}, {NULL}};
const Segment SGI14960[] = {{&S78488[67], 19}, {NULL}};
const Segment SGI14961[] = {{&S78488[53], 6}, {NULL}};
const Segment SGI14962[] = {{&S78488[135], 3}, {NULL}};
const Segment SGI14963[] = {{&S78488[100], 6}, {NULL}};
const Segment SGI14964[] = {{&S78488[155], 8}, {NULL}};
const Segment SGI14965[] = {{&S77958[42], 8}, {NULL}};
const Segment SGI14966[] = {{&S77958[42], 19}, {NULL}};
const Segment SGI14967[] = {{&S77958[30], 4}, {NULL}};
const Segment SGI14968[] = {{&S77958[109], 3}, {NULL}};
const Segment SGI14969[] = {{&S77958[75], 5}, {NULL}};
const Segment SGI14970[] = {{&S78681[30], 6}, {NULL}};
const Segment SGI14971[] = {{&S78681[92], 2}, {NULL}};
const Segment SGI14972[] = {{&S78681[57], 6}, {NULL}};
const Segment SGI14973[] = {{&S77796[45], 4}, {NULL}};
const Segment SGI14974[] = {{&S77796[105], 3}, {NULL}};
const Segment SGI14975[] = {{&S77796[94], 3}, {NULL}};
const Segment SGI14976[] = {{&S77796[71], 6}, {NULL}};
const Segment SGI14977[] = {{&S77797[37], 4}, {NULL}};
const Segment SGI14978[] = {{&S77797[56], 3}, {NULL}};
const Segment SGI14979[] = {{&S78764[44], 31}, {NULL}};
const Segment SGI14980[] = {{&S78764[30], 6}, {NULL}};
const Segment SGI14981[] = {{&S78764[141], 3}, {NULL}};
const Segment SGI14982[] = {{&S78764[107], 5}, {NULL}};
const Segment SGI14983[] = {{&S78765[44], 31}, {NULL}};
const Segment SGI14984[] = {{&S78765[30], 6}, {NULL}};
const Segment SGI14985[] = {{&S78765[141], 3}, {NULL}};
const Segment SGI14986[] = {{&S78765[107], 5}, {NULL}};
const Segment SGI14987[] = {{&S78766[44], 29}, {NULL}};
const Segment SGI14988[] = {{&S78766[30], 6}, {NULL}};
const Segment SGI14989[] = {{&S78766[139], 3}, {NULL}};
const Segment SGI14990[] = {{&S78766[105], 5}, {NULL}};
const Segment SGI14991[] = {{&S78768[44], 8}, {NULL}};
const Segment SGI14992[] = {{&S78768[44], 33}, {NULL}};
const Segment SGI14993[] = {{&S78768[30], 6}, {NULL}};
const Segment SGI14994[] = {{&S78768[125], 3}, {NULL}};
const Segment SGI14995[] = {{&S78768[91], 5}, {NULL}};
const Segment SGI14996[] = {{&S78769[44], 8}, {NULL}};
const Segment SGI14997[] = {{&S78769[44], 33}, {NULL}};
const Segment SGI14998[] = {{&S78769[30], 6}, {NULL}};
const Segment SGI14999[] = {{&S77908[50], 6}, {NULL}};
const Segment SGI15000[] = {{&S77908[50], 24}, {NULL}};
const Segment SGI15001[] = {{&S77908[30], 12}, {NULL}};
const Segment SGI15002[] = {{&S77908[88], 6}, {NULL}};
const Segment SGI15003[] = {{&S77908[115], 13}, {NULL}};
const Segment SGI15004[] = {{&S77909[50], 6}, {NULL}};
const Segment SGI15005[] = {{&S77909[50], 23}, {NULL}};
const Segment SGI15006[] = {{&S77909[30], 12}, {NULL}};
const Segment SGI15007[] = {{&S77909[87], 6}, {NULL}};
const Segment SGI15008[] = {{&S77909[114], 13}, {NULL}};
const Segment SGI15009[] = {{&S78769[125], 3}, {NULL}};
const Segment SGI15010[] = {{&S78769[91], 5}, {NULL}};
const Segment SGI15011[] = {{&S78770[44], 8}, {NULL}};
const Segment SGI15012[] = {{&S78770[44], 33}, {NULL}};
const Segment SGI15013[] = {{&S78770[30], 6}, {NULL}};
const Segment SGI15014[] = {{&S78770[94], 4}, {NULL}};
const Segment SGI15015[] = {{&S78770[94], 3}, {NULL}};
const Segment SGI15016[] = {{&S78446[43], 10}, {NULL}};
const Segment SGI15017[] = {{&S78446[30], 4}, {NULL}};
const Segment SGI15018[] = {{&S78446[112], 3}, {NULL}};
const Segment SGI15019[] = {{&S78446[78], 5}, {NULL}};
const Segment SGI15020[] = {{&S78446[136], 13}, {NULL}};
const Segment SGI15021[] = {{&S78448[43], 11}, {NULL}};
const Segment SGI15022[] = {{&S78448[30], 4}, {NULL}};
const Segment SGI15023[] = {{&S78448[113], 3}, {NULL}};
const Segment SGI15024[] = {{&S78448[79], 5}, {NULL}};
const Segment SGI15025[] = {{&S78448[137], 13}, {NULL}};
const Segment SGI15026[] = {{&S78449[43], 11}, {NULL}};
const Segment SGI15027[] = {{&S78449[30], 4}, {NULL}};
const Segment SGI15028[] = {{&S78449[113], 3}, {NULL}};
const Segment SGI15029[] = {{&S78449[79], 5}, {NULL}};
const Segment SGI15030[] = {{&S78449[137], 13}, {NULL}};
const Segment SGI15031[] = {{&S78450[43], 11}, {NULL}};
const Segment SGI15032[] = {{&S78450[30], 4}, {NULL}};
const Segment SGI15033[] = {{&S78450[113], 3}, {NULL}};
const Segment SGI15034[] = {{&S78450[79], 5}, {NULL}};
const Segment SGI15035[] = {{&S78450[137], 13}, {NULL}};
const Segment SGI15036[] = {{&S78162[50], 10}, {NULL}};
const Segment SGI15037[] = {{&S78162[50], 22}, {NULL}};
const Segment SGI15038[] = {{&S78162[30], 12}, {NULL}};
const Segment SGI15039[] = {{&S78162[121], 3}, {NULL}};
const Segment SGI15040[] = {{&S78162[86], 6}, {NULL}};
const Segment SGI15041[] = {{&S78163[50], 10}, {NULL}};
const Segment SGI15042[] = {{&S78163[50], 22}, {NULL}};
const Segment SGI15043[] = {{&S78163[30], 12}, {NULL}};
const Segment SGI15044[] = {{&S78163[121], 3}, {NULL}};
const Segment SGI15045[] = {{&S78163[86], 6}, {NULL}};
const Segment SGI15046[] = {{&S78164[50], 10}, {NULL}};
const Segment SGI15047[] = {{&S78164[50], 22}, {NULL}};
const Segment SGI15048[] = {{&S78164[30], 12}, {NULL}};
const Segment SGI15049[] = {{&S78164[89], 4}, {NULL}};
const Segment SGI15050[] = {{&S78164[89], 3}, {NULL}};
const Segment SGI15051[] = {{&S78165[50], 10}, {NULL}};
const Segment SGI15052[] = {{&S78165[50], 22}, {NULL}};
const Segment SGI15053[] = {{&S78165[30], 12}, {NULL}};
const Segment SGI15054[] = {{&S78165[89], 6}, {NULL}};
const Segment SGI15055[] = {{&S79679[30], 8}, {NULL}};
const Segment SGI15056[] = {{&S79021[44], 28}, {NULL}};
const Segment SGI15057[] = {{&S79021[30], 6}, {NULL}};
const Segment SGI15058[] = {{&S79021[138], 3}, {NULL}};
const Segment SGI15059[] = {{&S79021[104], 5}, {NULL}};
const Segment SGI15060[] = {{&S79022[44], 28}, {NULL}};
const Segment SGI15061[] = {{&S79022[30], 6}, {NULL}};
const Segment SGI15062[] = {{&S79022[138], 3}, {NULL}};
const Segment SGI15063[] = {{&S79022[104], 5}, {NULL}};
const Segment SGI15064[] = {{&S78294[51], 6}, {NULL}};
const Segment SGI15065[] = {{&S78294[30], 12}, {NULL}};
const Segment SGI15066[] = {{&S78294[122], 3}, {NULL}};
const Segment SGI15067[] = {{&S78294[88], 5}, {NULL}};
const Segment SGI15068[] = {{&S78294[146], 13}, {NULL}};
const Segment SGI15069[] = {{&S79356[44], 7}, {NULL}};
const Segment SGI15070[] = {{&S79356[44], 26}, {NULL}};
const Segment SGI15071[] = {{&S79356[30], 6}, {NULL}};
const Segment SGI15072[] = {{&S79356[67], 3}, {NULL}};
const Segment SGI15073[] = {{&S79356[84], 5}, {NULL}};
const Segment SGI15074[] = {{&S79357[44], 7}, {NULL}};
const Segment SGI15075[] = {{&S79357[44], 26}, {NULL}};
const Segment SGI15076[] = {{&S79357[30], 6}, {NULL}};
const Segment SGI15077[] = {{&S79357[118], 3}, {NULL}};
const Segment SGI15078[] = {{&S79357[84], 5}, {NULL}};
const Segment SGI15079[] = {{&S79361[44], 7}, {NULL}};
const Segment SGI15080[] = {{&S79361[44], 26}, {NULL}};
const Segment SGI15081[] = {{&S79361[30], 6}, {NULL}};
const Segment SGI15082[] = {{&S79361[87], 4}, {NULL}};
const Segment SGI15083[] = {{&S79361[87], 3}, {NULL}};
const Segment SGI15084[] = {{&S85340[0], 15}, {NULL}};
const Segment SGI15085[] = {{&S79586[44], 5}, {NULL}};
const Segment SGI15086[] = {{&S79586[44], 17}, {NULL}};
const Segment SGI15087[] = {{&S79586[30], 6}, {NULL}};
const Segment SGI15088[] = {{&S79586[109], 3}, {NULL}};
const Segment SGI15089[] = {{&S79586[75], 5}, {NULL}};
const Segment SGI15090[] = {{&S79704[45], 11}, {NULL}};
const Segment SGI15091[] = {{&S79704[30], 6}, {NULL}};
const Segment SGI15092[] = {{&S79704[115], 3}, {NULL}};
const Segment SGI15093[] = {{&S79704[81], 5}, {NULL}};
const Segment SGI15094[] = {{&S79704[139], 13}, {NULL}};
const Segment SGI15095[] = {{&S80387[8], 8}, {NULL}};
const Segment SGI15096[] = {{&S79073[50], 7}, {NULL}};
const Segment SGI15097[] = {{&S79073[50], 18}, {NULL}};
const Segment SGI15098[] = {{&S79073[30], 12}, {NULL}};
const Segment SGI15099[] = {{&S79073[117], 3}, {NULL}};
const Segment SGI15100[] = {{&S79073[82], 6}, {NULL}};
const Segment SGI15101[] = {{&S79074[50], 7}, {NULL}};
const Segment SGI15102[] = {{&S79074[50], 26}, {NULL}};
const Segment SGI15103[] = {{&S79074[30], 12}, {NULL}};
const Segment SGI15104[] = {{&S79074[125], 3}, {NULL}};
const Segment SGI15105[] = {{&S79074[90], 6}, {NULL}};
const Segment SGI15106[] = {{&S79075[50], 7}, {NULL}};
const Segment SGI15107[] = {{&S79075[50], 29}, {NULL}};
const Segment SGI15108[] = {{&S79075[30], 12}, {NULL}};
const Segment SGI15109[] = {{&S79075[76], 3}, {NULL}};
const Segment SGI15110[] = {{&S79075[93], 6}, {NULL}};
const Segment SGI15111[] = {{&S79094[51], 6}, {NULL}};
const Segment SGI15112[] = {{&S79094[30], 12}, {NULL}};
const Segment SGI15113[] = {{&S79094[122], 3}, {NULL}};
const Segment SGI15114[] = {{&S79094[88], 5}, {NULL}};
const Segment SGI15115[] = {{&S79094[146], 13}, {NULL}};
const Segment SGI15116[] = {{&S79703[43], 10}, {NULL}};
const Segment SGI15117[] = {{&S79703[30], 4}, {NULL}};
const Segment SGI15118[] = {{&S79703[112], 3}, {NULL}};
const Segment SGI15119[] = {{&S79703[78], 5}, {NULL}};
const Segment SGI15120[] = {{&S79703[136], 13}, {NULL}};
const Segment SGI15121[] = {{&S79705[43], 11}, {NULL}};
const Segment SGI15122[] = {{&S79705[30], 4}, {NULL}};
const Segment SGI15123[] = {{&S79705[113], 3}, {NULL}};
const Segment SGI15124[] = {{&S79705[79], 5}, {NULL}};
const Segment SGI15125[] = {{&S79705[137], 13}, {NULL}};
const Segment SGI15126[] = {{&S79706[43], 11}, {NULL}};
const Segment SGI15127[] = {{&S79706[30], 4}, {NULL}};
const Segment SGI15128[] = {{&S79706[113], 3}, {NULL}};
const Segment SGI15129[] = {{&S79706[79], 5}, {NULL}};
const Segment SGI15130[] = {{&S79706[137], 13}, {NULL}};
const Segment SGI15131[] = {{&S79268[50], 8}, {NULL}};
const Segment SGI15132[] = {{&S79268[50], 20}, {NULL}};
const Segment SGI15133[] = {{&S79268[30], 12}, {NULL}};
const Segment SGI15134[] = {{&S79268[119], 3}, {NULL}};
const Segment SGI15135[] = {{&S79268[84], 6}, {NULL}};
const Segment SGI15136[] = {{&S79365[50], 8}, {NULL}};
const Segment SGI15137[] = {{&S79365[50], 19}, {NULL}};
const Segment SGI15138[] = {{&S79365[30], 12}, {NULL}};
const Segment SGI15139[] = {{&S79365[118], 3}, {NULL}};
const Segment SGI15140[] = {{&S79365[83], 6}, {NULL}};
const Segment SGI15141[] = {{&S80755[8], 8}, {NULL}};
const Segment SGI15142[] = {{&S79891[42], 10}, {NULL}};
const Segment SGI15143[] = {{&S79891[42], 21}, {NULL}};
const Segment SGI15144[] = {{&S79891[30], 4}, {NULL}};
const Segment SGI15145[] = {{&S79891[111], 3}, {NULL}};
const Segment SGI15146[] = {{&S79891[77], 5}, {NULL}};
const Segment SGI15147[] = {{&S79473[50], 13}, {NULL}};
const Segment SGI15148[] = {{&S79473[50], 24}, {NULL}};
const Segment SGI15149[] = {{&S79473[30], 12}, {NULL}};
const Segment SGI15150[] = {{&S79473[123], 3}, {NULL}};
const Segment SGI15151[] = {{&S79473[88], 6}, {NULL}};
const Segment SGI15152[] = {{&S79474[50], 17}, {NULL}};
const Segment SGI15153[] = {{&S79474[50], 28}, {NULL}};
const Segment SGI15154[] = {{&S79474[30], 12}, {NULL}};
const Segment SGI15155[] = {{&S79474[127], 3}, {NULL}};
const Segment SGI15156[] = {{&S79474[92], 6}, {NULL}};
const Segment SGI15157[] = {{&S80272[44], 8}, {NULL}};
const Segment SGI15158[] = {{&S80272[44], 19}, {NULL}};
const Segment SGI15159[] = {{&S80272[30], 6}, {NULL}};
const Segment SGI15160[] = {{&S80272[111], 3}, {NULL}};
const Segment SGI15161[] = {{&S80272[77], 5}, {NULL}};
const Segment SGI15162[] = {{&S80275[31], 12}, {NULL}};
const Segment SGI15163[] = {{&S80275[58], 3}, {NULL}};
const Segment SGI15164[] = {{&S80276[39], 12}, {NULL}};
const Segment SGI15165[] = {{&S80276[107], 3}, {NULL}};
const Segment SGI15166[] = {{&S80276[96], 3}, {NULL}};
const Segment SGI15167[] = {{&S80276[73], 6}, {NULL}};
const Segment SGI15168[] = {{&S80351[45], 14}, {NULL}};
const Segment SGI15169[] = {{&S80351[30], 6}, {NULL}};
const Segment SGI15170[] = {{&S80351[119], 3}, {NULL}};
const Segment SGI15171[] = {{&S80351[85], 5}, {NULL}};
const Segment SGI15172[] = {{&S80351[143], 13}, {NULL}};
const Segment SGI15173[] = {{&S82952[40], 7}, {NULL}};
const Segment SGI15174[] = {{&S82952[30], 4}, {NULL}};
const Segment SGI15175[] = {{&S82952[107], 3}, {NULL}};
const Segment SGI15176[] = {{&S82952[72], 6}, {NULL}};
const Segment SGI15177[] = {{&S82952[127], 8}, {NULL}};
const Segment SGI15178[] = {{&S82953[42], 7}, {NULL}};
const Segment SGI15179[] = {{&S82953[42], 18}, {NULL}};
const Segment SGI15180[] = {{&S82953[30], 4}, {NULL}};
const Segment SGI15181[] = {{&S82953[109], 3}, {NULL}};
const Segment SGI15182[] = {{&S82953[74], 6}, {NULL}};
const Segment SGI15183[] = {{&S82953[129], 8}, {NULL}};
const Segment SGI15184[] = {{&S80398[44], 11}, {NULL}};
const Segment SGI15185[] = {{&S80398[44], 23}, {NULL}};
const Segment SGI15186[] = {{&S80398[30], 6}, {NULL}};
const Segment SGI15187[] = {{&S80398[115], 3}, {NULL}};
const Segment SGI15188[] = {{&S80398[81], 5}, {NULL}};
const Segment SGI15189[] = {{&S86006[0], 107}, {NULL}};
const Segment SGI15190[] = {{&S86007[0], 107}, {NULL}};
const Segment SGI15191[] = {{&S80428[45], 10}, {NULL}};
const Segment SGI15192[] = {{&S80428[30], 6}, {NULL}};
const Segment SGI15193[] = {{&S80428[114], 3}, {NULL}};
const Segment SGI15194[] = {{&S80428[80], 5}, {NULL}};
const Segment SGI15195[] = {{&S80428[138], 13}, {NULL}};
const Segment SGI15196[] = {{&S79546[51], 6}, {NULL}};
const Segment SGI15197[] = {{&S79546[30], 12}, {NULL}};
const Segment SGI15198[] = {{&S79546[123], 3}, {NULL}};
const Segment SGI15199[] = {{&S79546[89], 5}, {NULL}};
const Segment SGI15200[] = {{&S79546[147], 13}, {NULL}};
const Segment SGI15201[] = {{&S79547[50], 6}, {NULL}};
const Segment SGI15202[] = {{&S79547[50], 24}, {NULL}};
const Segment SGI15203[] = {{&S79547[30], 12}, {NULL}};
const Segment SGI15204[] = {{&S79547[123], 3}, {NULL}};
const Segment SGI15205[] = {{&S79547[88], 6}, {NULL}};
const Segment SGI15206[] = {{&S79547[143], 8}, {NULL}};
const Segment SGI15207[] = {{&S79547[152], 13}, {NULL}};
const Segment SGI15208[] = {{&S79579[45], 4}, {NULL}};
const Segment SGI15209[] = {{&S79579[105], 3}, {NULL}};
const Segment SGI15210[] = {{&S79579[94], 3}, {NULL}};
const Segment SGI15211[] = {{&S79579[71], 6}, {NULL}};
const Segment SGI15212[] = {{&S79580[37], 4}, {NULL}};
const Segment SGI15213[] = {{&S79580[56], 3}, {NULL}};
const Segment SGI15214[] = {{&S79582[50], 7}, {NULL}};
const Segment SGI15215[] = {{&S79582[50], 25}, {NULL}};
const Segment SGI15216[] = {{&S79582[30], 12}, {NULL}};
const Segment SGI15217[] = {{&S79582[124], 3}, {NULL}};
const Segment SGI15218[] = {{&S79582[89], 6}, {NULL}};
const Segment SGI15219[] = {{&S79583[50], 7}, {NULL}};
const Segment SGI15220[] = {{&S79583[50], 25}, {NULL}};
const Segment SGI15221[] = {{&S79583[30], 12}, {NULL}};
const Segment SGI15222[] = {{&S79583[124], 3}, {NULL}};
const Segment SGI15223[] = {{&S79583[89], 6}, {NULL}};
const Segment SGI15224[] = {{&S79584[50], 7}, {NULL}};
const Segment SGI15225[] = {{&S79584[50], 25}, {NULL}};
const Segment SGI15226[] = {{&S79584[30], 12}, {NULL}};
const Segment SGI15227[] = {{&S79584[124], 3}, {NULL}};
const Segment SGI15228[] = {{&S79584[89], 6}, {NULL}};
const Segment SGI15229[] = {{&S79585[50], 7}, {NULL}};
const Segment SGI15230[] = {{&S79585[50], 25}, {NULL}};
const Segment SGI15231[] = {{&S79585[30], 12}, {NULL}};
const Segment SGI15232[] = {{&S79585[92], 4}, {NULL}};
const Segment SGI15233[] = {{&S79585[92], 3}, {NULL}};
const Segment SGI15234[] = {{&S81171[74], 5}, {NULL}};
const Segment SGI15235[] = {{&S81205[8], 8}, {NULL}};
const Segment SGI15236[] = {{&S80427[43], 10}, {NULL}};
const Segment SGI15237[] = {{&S80427[30], 4}, {NULL}};
const Segment SGI15238[] = {{&S80427[112], 3}, {NULL}};
const Segment SGI15239[] = {{&S80427[78], 5}, {NULL}};
const Segment SGI15240[] = {{&S80427[136], 13}, {NULL}};
const Segment SGI15241[] = {{&S80429[43], 11}, {NULL}};
const Segment SGI15242[] = {{&S80429[30], 4}, {NULL}};
const Segment SGI15243[] = {{&S80429[113], 3}, {NULL}};
const Segment SGI15244[] = {{&S80429[79], 5}, {NULL}};
const Segment SGI15245[] = {{&S80429[137], 13}, {NULL}};
const Segment SGI15246[] = {{&S80754[45], 10}, {NULL}};
const Segment SGI15247[] = {{&S80754[30], 6}, {NULL}};
const Segment SGI15248[] = {{&S80754[114], 3}, {NULL}};
const Segment SGI15249[] = {{&S80754[80], 5}, {NULL}};
const Segment SGI15250[] = {{&S80754[138], 13}, {NULL}};
const Segment SGI15251[] = {{&S80826[45], 10}, {NULL}};
const Segment SGI15252[] = {{&S80826[30], 6}, {NULL}};
const Segment SGI15253[] = {{&S80826[114], 3}, {NULL}};
const Segment SGI15254[] = {{&S80826[80], 5}, {NULL}};
const Segment SGI15255[] = {{&S80826[138], 13}, {NULL}};
const Segment SGI15256[] = {{&S79879[50], 14}, {NULL}};
const Segment SGI15257[] = {{&S79879[50], 25}, {NULL}};
const Segment SGI15258[] = {{&S79879[30], 12}, {NULL}};
const Segment SGI15259[] = {{&S79879[124], 3}, {NULL}};
const Segment SGI15260[] = {{&S79879[89], 6}, {NULL}};
const Segment SGI15261[] = {{&S79880[44], 14}, {NULL}};
const Segment SGI15262[] = {{&S79880[31], 12}, {NULL}};
const Segment SGI15263[] = {{&S79881[48], 14}, {NULL}};
const Segment SGI15264[] = {{&S79881[30], 12}, {NULL}};
const Segment SGI15265[] = {{&S79881[122], 3}, {NULL}};
const Segment SGI15266[] = {{&S79881[87], 6}, {NULL}};
const Segment SGI15267[] = {{&S80513[42], 7}, {NULL}};
const Segment SGI15268[] = {{&S80513[42], 18}, {NULL}};
const Segment SGI15269[] = {{&S80513[30], 4}, {NULL}};
const Segment SGI15270[] = {{&S80513[108], 3}, {NULL}};
const Segment SGI15271[] = {{&S80513[74], 5}, {NULL}};
const Segment SGI15272[] = {{&S80514[42], 7}, {NULL}};
const Segment SGI15273[] = {{&S80514[42], 18}, {NULL}};
const Segment SGI15274[] = {{&S80514[30], 4}, {NULL}};
const Segment SGI15275[] = {{&S80514[108], 3}, {NULL}};
const Segment SGI15276[] = {{&S80514[74], 5}, {NULL}};
const Segment SGI15277[] = {{&S81541[32], 57}, {NULL}};
const Segment SGI15278[] = {{&S80654[37], 8}, {NULL}};
const Segment SGI15279[] = {{&S80654[102], 3}, {NULL}};
const Segment SGI15280[] = {{&S80654[91], 3}, {NULL}};
const Segment SGI15281[] = {{&S80654[68], 6}, {NULL}};
const Segment SGI15282[] = {{&S80655[29], 8}, {NULL}};
const Segment SGI15283[] = {{&S80655[53], 3}, {NULL}};
const Segment SGI15284[] = {{&S80656[37], 9}, {NULL}};
const Segment SGI15285[] = {{&S80656[103], 3}, {NULL}};
const Segment SGI15286[] = {{&S80656[92], 3}, {NULL}};
const Segment SGI15287[] = {{&S80656[69], 6}, {NULL}};
const Segment SGI15288[] = {{&S80657[42], 6}, {NULL}};
const Segment SGI15289[] = {{&S80657[42], 17}, {NULL}};
const Segment SGI15290[] = {{&S80657[30], 4}, {NULL}};
const Segment SGI15291[] = {{&S80657[107], 3}, {NULL}};
const Segment SGI15292[] = {{&S80657[73], 5}, {NULL}};
const Segment SGI15293[] = {{&S81165[44], 8}, {NULL}};
const Segment SGI15294[] = {{&S81165[44], 19}, {NULL}};
const Segment SGI15295[] = {{&S81165[30], 6}, {NULL}};
const Segment SGI15296[] = {{&S81165[111], 3}, {NULL}};
const Segment SGI15297[] = {{&S81165[77], 5}, {NULL}};
const Segment SGI15298[] = {{&S81760[43], 8}, {&S81760[43], 8}, {NULL}};
const Segment SGI15299[] = {{&S80128[50], 9}, {NULL}};
const Segment SGI15300[] = {{&S80128[50], 21}, {NULL}};
const Segment SGI15301[] = {{&S80128[30], 12}, {NULL}};
const Segment SGI15302[] = {{&S80128[120], 3}, {NULL}};
const Segment SGI15303[] = {{&S80128[85], 6}, {NULL}};
const Segment SGI15304[] = {{&S82045[6], 4}, {NULL}};
const Segment SGI15305[] = {{&S82045[32], 3}, {NULL}};
const Segment SGI15306[] = {{&S82045[81], 5}, {NULL}};
const Segment SGI15307[] = {{&S82059[74], 5}, {NULL}};
const Segment SGI15308[] = {{&S81482[6], 4}, {NULL}};
const Segment SGI15309[] = {{&S81482[78], 5}, {NULL}};
const Segment SGI15310[] = {{&S82162[49], 7}, {NULL}};
const Segment SGI15311[] = {{&S82162[49], 24}, {NULL}};
const Segment SGI15312[] = {{&S82162[30], 11}, {NULL}};
const Segment SGI15313[] = {{&S82162[122], 3}, {NULL}};
const Segment SGI15314[] = {{&S82162[87], 6}, {NULL}};
const Segment SGI15315[] = {{&S81063[42], 14}, {NULL}};
const Segment SGI15316[] = {{&S81063[42], 25}, {NULL}};
const Segment SGI15317[] = {{&S81063[30], 4}, {NULL}};
const Segment SGI15318[] = {{&S81063[115], 3}, {NULL}};
const Segment SGI15319[] = {{&S81063[81], 5}, {NULL}};
const Segment SGI15320[] = {{&S80516[56], 13}, {NULL}};
const Segment SGI15321[] = {{&S80516[56], 25}, {NULL}};
const Segment SGI15322[] = {{&S80516[30], 18}, {NULL}};
const Segment SGI15323[] = {{&S80516[130], 3}, {NULL}};
const Segment SGI15324[] = {{&S80516[95], 6}, {NULL}};
const Segment SGI15325[] = {{&S80517[56], 13}, {NULL}};
const Segment SGI15326[] = {{&S80517[56], 25}, {NULL}};
const Segment SGI15327[] = {{&S80517[30], 18}, {NULL}};
const Segment SGI15328[] = {{&S80517[130], 3}, {NULL}};
const Segment SGI15329[] = {{&S80517[95], 6}, {NULL}};
const Segment SGI15330[] = {{&S80518[56], 13}, {NULL}};
const Segment SGI15331[] = {{&S80518[56], 25}, {NULL}};
const Segment SGI15332[] = {{&S80518[30], 18}, {NULL}};
const Segment SGI15333[] = {{&S80518[130], 3}, {NULL}};
const Segment SGI15334[] = {{&S80518[95], 6}, {NULL}};
const Segment SGI15335[] = {{&S80519[56], 9}, {NULL}};
const Segment SGI15336[] = {{&S80519[56], 21}, {NULL}};
const Segment SGI15337[] = {{&S80519[30], 18}, {NULL}};
const Segment SGI15338[] = {{&S80519[126], 3}, {NULL}};
const Segment SGI15339[] = {{&S80519[91], 6}, {NULL}};
const Segment SGI15340[] = {{&S87388[0], 107}, {NULL}};
const Segment SGI15341[] = {{&S81729[39], 8}, {NULL}};
const Segment SGI15342[] = {{&S81729[103], 3}, {NULL}};
const Segment SGI15343[] = {{&S81729[92], 3}, {NULL}};
const Segment SGI15344[] = {{&S81729[69], 6}, {NULL}};
const Segment SGI15345[] = {{&S81730[31], 8}, {NULL}};
const Segment SGI15346[] = {{&S81730[54], 3}, {NULL}};
const Segment SGI15347[] = {{&S81731[39], 8}, {NULL}};
const Segment SGI15348[] = {{&S81731[103], 3}, {NULL}};
const Segment SGI15349[] = {{&S81731[92], 3}, {NULL}};
const Segment SGI15350[] = {{&S81731[69], 6}, {NULL}};
const Segment SGI15351[] = {{&S81732[31], 8}, {NULL}};
const Segment SGI15352[] = {{&S81732[54], 3}, {NULL}};
const Segment SGI15353[] = {{&S81742[44], 4}, {NULL}};
const Segment SGI15354[] = {{&S81742[44], 16}, {NULL}};
const Segment SGI15355[] = {{&S81742[30], 6}, {NULL}};
const Segment SGI15356[] = {{&S81742[108], 3}, {NULL}};
const Segment SGI15357[] = {{&S81742[74], 5}, {NULL}};
const Segment SGI15358[] = {{&S83914[44], 6}, {NULL}};
const Segment SGI15359[] = {{&S83914[44], 27}, {NULL}};
const Segment SGI15360[] = {{&S83914[30], 6}, {NULL}};
const Segment SGI15361[] = {{&S83914[120], 3}, {NULL}};
const Segment SGI15362[] = {{&S83914[85], 6}, {NULL}};
const Segment SGI15363[] = {{&S83915[42], 6}, {NULL}};
const Segment SGI15364[] = {{&S83915[42], 27}, {NULL}};
const Segment SGI15365[] = {{&S83915[30], 4}, {NULL}};
const Segment SGI15366[] = {{&S83915[118], 3}, {NULL}};
const Segment SGI15367[] = {{&S83915[83], 6}, {NULL}};
const Segment SGI15368[] = {{&S83916[42], 6}, {NULL}};
const Segment SGI15369[] = {{&S83916[42], 27}, {NULL}};
const Segment SGI15370[] = {{&S83916[30], 4}, {NULL}};
const Segment SGI15371[] = {{&S83916[118], 3}, {NULL}};
const Segment SGI15372[] = {{&S83916[83], 6}, {NULL}};
const Segment SGI15373[] = {{&S83917[44], 6}, {NULL}};
const Segment SGI15374[] = {{&S83917[44], 27}, {NULL}};
const Segment SGI15375[] = {{&S83917[30], 6}, {NULL}};
const Segment SGI15376[] = {{&S83917[120], 3}, {NULL}};
const Segment SGI15377[] = {{&S83917[85], 6}, {NULL}};
const Segment SGI15378[] = {{&S81861[44], 7}, {NULL}};
const Segment SGI15379[] = {{&S81861[44], 18}, {NULL}};
const Segment SGI15380[] = {{&S81861[30], 6}, {NULL}};
const Segment SGI15381[] = {{&S81861[110], 3}, {NULL}};
const Segment SGI15382[] = {{&S81861[76], 5}, {NULL}};
const Segment SGI15383[] = {{&S81862[44], 7}, {NULL}};
const Segment SGI15384[] = {{&S81862[44], 18}, {NULL}};
const Segment SGI15385[] = {{&S81862[30], 6}, {NULL}};
const Segment SGI15386[] = {{&S81862[79], 4}, {NULL}};
const Segment SGI15387[] = {{&S81862[79], 3}, {NULL}};
const Segment SGI15388[] = {{&S80794[50], 9}, {NULL}};
const Segment SGI15389[] = {{&S80794[50], 20}, {NULL}};
const Segment SGI15390[] = {{&S80794[30], 12}, {NULL}};
const Segment SGI15391[] = {{&S80794[119], 3}, {NULL}};
const Segment SGI15392[] = {{&S80794[84], 6}, {NULL}};
const Segment SGI15393[] = {{&S80795[50], 9}, {NULL}};
const Segment SGI15394[] = {{&S80795[50], 20}, {NULL}};
const Segment SGI15395[] = {{&S80795[30], 12}, {NULL}};
const Segment SGI15396[] = {{&S80795[120], 3}, {NULL}};
const Segment SGI15397[] = {{&S80795[84], 7}, {NULL}};
const Segment SGI15398[] = {{&S80795[140], 8}, {NULL}};
const Segment SGI15399[] = {{&S80796[44], 9}, {NULL}};
const Segment SGI15400[] = {{&S80796[31], 12}, {NULL}};
const Segment SGI15401[] = {{&S80944[6], 4}, {NULL}};
const Segment SGI15402[] = {{&S80944[88], 5}, {NULL}};
const Segment SGI15403[] = {{&S81523[42], 13}, {NULL}};
const Segment SGI15404[] = {{&S81523[42], 24}, {NULL}};
const Segment SGI15405[] = {{&S81523[30], 4}, {NULL}};
const Segment SGI15406[] = {{&S81523[114], 3}, {NULL}};
const Segment SGI15407[] = {{&S81523[80], 5}, {NULL}};
const Segment SGI15408[] = {{&S81524[42], 11}, {NULL}};
const Segment SGI15409[] = {{&S81524[42], 23}, {NULL}};
const Segment SGI15410[] = {{&S81524[30], 4}, {NULL}};
const Segment SGI15411[] = {{&S81524[113], 3}, {NULL}};
const Segment SGI15412[] = {{&S81524[79], 5}, {NULL}};
const Segment SGI15413[] = {{&S81525[36], 11}, {NULL}};
const Segment SGI15414[] = {{&S81525[31], 4}, {NULL}};
const Segment SGI15415[] = {{&S84576[42], 6}, {NULL}};
const Segment SGI15416[] = {{&S84576[42], 17}, {NULL}};
const Segment SGI15417[] = {{&S84576[30], 4}, {NULL}};
const Segment SGI15418[] = {{&S84576[108], 3}, {NULL}};
const Segment SGI15419[] = {{&S84576[73], 6}, {NULL}};
const Segment SGI15420[] = {{&S84576[128], 8}, {NULL}};
const Segment SGI15421[] = {{&S81673[39], 3}, {NULL}};
const Segment SGI15422[] = {{&S81673[39], 15}, {NULL}};
const Segment SGI15423[] = {{&S81673[101], 2}, {NULL}};
const Segment SGI15424[] = {{&S81673[68], 6}, {NULL}};
const Segment SGI15425[] = {{&S82264[44], 9}, {NULL}};
const Segment SGI15426[] = {{&S82264[44], 20}, {NULL}};
const Segment SGI15427[] = {{&S82264[30], 6}, {NULL}};
const Segment SGI15428[] = {{&S82264[112], 3}, {NULL}};
const Segment SGI15429[] = {{&S82264[78], 5}, {NULL}};
const Segment SGI15430[] = {{&S82265[44], 9}, {NULL}};
const Segment SGI15431[] = {{&S82265[44], 20}, {NULL}};
const Segment SGI15432[] = {{&S82265[30], 6}, {NULL}};
const Segment SGI15433[] = {{&S82265[112], 3}, {NULL}};
const Segment SGI15434[] = {{&S82265[78], 5}, {NULL}};
const Segment SGI15435[] = {{&S82266[44], 9}, {NULL}};
const Segment SGI15436[] = {{&S82266[44], 20}, {NULL}};
const Segment SGI15437[] = {{&S82266[30], 6}, {NULL}};
const Segment SGI15438[] = {{&S82266[112], 3}, {NULL}};
const Segment SGI15439[] = {{&S82266[78], 5}, {NULL}};
const Segment SGI15440[] = {{&S82267[44], 9}, {NULL}};
const Segment SGI15441[] = {{&S82267[44], 20}, {NULL}};
const Segment SGI15442[] = {{&S82267[30], 6}, {NULL}};
const Segment SGI15443[] = {{&S82267[112], 3}, {NULL}};
const Segment SGI15444[] = {{&S82267[78], 5}, {NULL}};
const Segment SGI15445[] = {{&S82268[44], 9}, {NULL}};
const Segment SGI15446[] = {{&S82268[44], 20}, {NULL}};
const Segment SGI15447[] = {{&S82268[30], 6}, {NULL}};
const Segment SGI15448[] = {{&S82268[112], 3}, {NULL}};
const Segment SGI15449[] = {{&S82268[78], 5}, {NULL}};
const Segment SGI15450[] = {{&S81237[50], 8}, {NULL}};
const Segment SGI15451[] = {{&S81237[50], 19}, {NULL}};
const Segment SGI15452[] = {{&S81237[30], 12}, {NULL}};
const Segment SGI15453[] = {{&S81237[118], 3}, {NULL}};
const Segment SGI15454[] = {{&S81237[83], 6}, {NULL}};
const Segment SGI15455[] = {{&S87962[0], 83}, {NULL}};
const Segment SGI15456[] = {{&S87963[0], 83}, {NULL}};
const Segment SGI15457[] = {{&S82322[45], 10}, {NULL}};
const Segment SGI15458[] = {{&S82322[30], 6}, {NULL}};
const Segment SGI15459[] = {{&S82322[114], 3}, {NULL}};
const Segment SGI15460[] = {{&S82322[80], 5}, {NULL}};
const Segment SGI15461[] = {{&S82322[138], 13}, {NULL}};
const Segment SGI15462[] = {{&S82323[44], 10}, {NULL}};
const Segment SGI15463[] = {{&S82323[44], 21}, {NULL}};
const Segment SGI15464[] = {{&S82323[30], 6}, {NULL}};
const Segment SGI15465[] = {{&S82323[114], 3}, {NULL}};
const Segment SGI15466[] = {{&S82323[79], 6}, {NULL}};
const Segment SGI15467[] = {{&S82323[134], 8}, {NULL}};
const Segment SGI15468[] = {{&S82323[143], 13}, {NULL}};
const Segment SGI15469[] = {{&S82363[44], 11}, {NULL}};
const Segment SGI15470[] = {{&S82363[44], 24}, {NULL}};
const Segment SGI15471[] = {{&S82363[30], 6}, {NULL}};
const Segment SGI15472[] = {{&S82363[116], 3}, {NULL}};
const Segment SGI15473[] = {{&S82363[82], 5}, {NULL}};
const Segment SGI15474[] = {{&S82365[44], 6}, {NULL}};
const Segment SGI15475[] = {{&S82365[44], 17}, {NULL}};
const Segment SGI15476[] = {{&S82365[30], 6}, {NULL}};
const Segment SGI15477[] = {{&S82365[109], 3}, {NULL}};
const Segment SGI15478[] = {{&S82365[75], 5}, {NULL}};
const Segment SGI15479[] = {{&S82366[44], 20}, {NULL}};
const Segment SGI15480[] = {{&S82366[44], 31}, {NULL}};
const Segment SGI15481[] = {{&S82366[30], 6}, {NULL}};
const Segment SGI15482[] = {{&S82366[123], 3}, {NULL}};
const Segment SGI15483[] = {{&S82366[89], 5}, {NULL}};
const Segment SGI15484[] = {{&S82367[44], 6}, {NULL}};
const Segment SGI15485[] = {{&S82367[44], 17}, {NULL}};
const Segment SGI15486[] = {{&S82367[30], 6}, {NULL}};
const Segment SGI15487[] = {{&S82367[109], 3}, {NULL}};
const Segment SGI15488[] = {{&S82367[75], 5}, {NULL}};
const Segment SGI15489[] = {{&S84844[42], 15}, {NULL}};
const Segment SGI15490[] = {{&S84844[42], 26}, {NULL}};
const Segment SGI15491[] = {{&S84844[30], 4}, {NULL}};
const Segment SGI15492[] = {{&S84844[117], 3}, {NULL}};
const Segment SGI15493[] = {{&S84844[82], 6}, {NULL}};
const Segment SGI15494[] = {{&S84844[137], 8}, {NULL}};
const Segment SGI15495[] = {{&S81942[40], 10}, {NULL}};
const Segment SGI15496[] = {{&S81942[30], 4}, {NULL}};
const Segment SGI15497[] = {{&S81942[109], 3}, {NULL}};
const Segment SGI15498[] = {{&S81942[75], 5}, {NULL}};
const Segment SGI15499[] = {{&S81943[42], 10}, {NULL}};
const Segment SGI15500[] = {{&S81943[42], 21}, {NULL}};
const Segment SGI15501[] = {{&S81943[30], 4}, {NULL}};
const Segment SGI15502[] = {{&S81943[111], 3}, {NULL}};
const Segment SGI15503[] = {{&S81943[77], 5}, {NULL}};
const Segment SGI15504[] = {{&S84948[42], 11}, {NULL}};
const Segment SGI15505[] = {{&S84948[42], 22}, {NULL}};
const Segment SGI15506[] = {{&S84948[30], 4}, {NULL}};
const Segment SGI15507[] = {{&S84948[113], 3}, {NULL}};
const Segment SGI15508[] = {{&S84948[78], 6}, {NULL}};
const Segment SGI15509[] = {{&S84948[133], 8}, {NULL}};
const Segment SGI15510[] = {{&S85041[6], 4}, {NULL}};
const Segment SGI15511[] = {{&S85041[79], 5}, {NULL}};
const Segment SGI15512[] = {{&S81955[42], 5}, {NULL}};
const Segment SGI15513[] = {{&S81955[42], 16}, {NULL}};
const Segment SGI15514[] = {{&S81955[30], 4}, {NULL}};
const Segment SGI15515[] = {{&S81955[106], 3}, {NULL}};
const Segment SGI15516[] = {{&S81955[72], 5}, {NULL}};
const Segment SGI15517[] = {{&S81955[117], 7}, {NULL}};
const Segment SGI15518[] = {{&S82630[45], 10}, {NULL}};
const Segment SGI15519[] = {{&S82630[30], 6}, {NULL}};
const Segment SGI15520[] = {{&S82630[114], 3}, {NULL}};
const Segment SGI15521[] = {{&S82630[80], 5}, {NULL}};
const Segment SGI15522[] = {{&S82630[138], 13}, {NULL}};
const Segment SGI15523[] = {{&S82631[44], 10}, {NULL}};
const Segment SGI15524[] = {{&S82631[44], 21}, {NULL}};
const Segment SGI15525[] = {{&S82631[30], 6}, {NULL}};
const Segment SGI15526[] = {{&S82631[114], 3}, {NULL}};
const Segment SGI15527[] = {{&S82631[79], 6}, {NULL}};
const Segment SGI15528[] = {{&S82631[134], 8}, {NULL}};
const Segment SGI15529[] = {{&S82631[143], 13}, {NULL}};
const Segment SGI15530[] = {{&S82040[43], 4}, {NULL}};
const Segment SGI15531[] = {{&S82040[96], 5}, {NULL}};
const Segment SGI15532[] = {{&S82679[44], 15}, {NULL}};
const Segment SGI15533[] = {{&S82679[44], 27}, {NULL}};
const Segment SGI15534[] = {{&S82679[30], 6}, {NULL}};
const Segment SGI15535[] = {{&S82679[119], 3}, {NULL}};
const Segment SGI15536[] = {{&S82679[85], 5}, {NULL}};
const Segment SGI15537[] = {{&S82680[44], 18}, {NULL}};
const Segment SGI15538[] = {{&S82680[44], 30}, {NULL}};
const Segment SGI15539[] = {{&S82680[30], 6}, {NULL}};
const Segment SGI15540[] = {{&S82680[122], 3}, {NULL}};
const Segment SGI15541[] = {{&S82680[88], 5}, {NULL}};
const Segment SGI15542[] = {{&S81521[50], 11}, {NULL}};
const Segment SGI15543[] = {{&S81521[50], 23}, {NULL}};
const Segment SGI15544[] = {{&S81521[30], 12}, {NULL}};
const Segment SGI15545[] = {{&S81521[122], 3}, {NULL}};
const Segment SGI15546[] = {{&S81521[87], 6}, {NULL}};
const Segment SGI15547[] = {{&S81522[44], 11}, {NULL}};
const Segment SGI15548[] = {{&S81522[31], 12}, {NULL}};
const Segment SGI15549[] = {{&S82118[42], 4}, {NULL}};
const Segment SGI15550[] = {{&S82118[42], 20}, {NULL}};
const Segment SGI15551[] = {{&S82118[30], 4}, {NULL}};
const Segment SGI15552[] = {{&S82118[110], 3}, {NULL}};
const Segment SGI15553[] = {{&S82118[76], 5}, {NULL}};
const Segment SGI15554[] = {{&S82119[42], 4}, {NULL}};
const Segment SGI15555[] = {{&S82119[42], 20}, {NULL}};
const Segment SGI15556[] = {{&S82119[30], 4}, {NULL}};
const Segment SGI15557[] = {{&S82119[110], 3}, {NULL}};
const Segment SGI15558[] = {{&S82119[76], 5}, {NULL}};
const Segment SGI15559[] = {{&S82120[42], 4}, {NULL}};
const Segment SGI15560[] = {{&S82120[42], 19}, {NULL}};
const Segment SGI15561[] = {{&S82120[30], 4}, {NULL}};
const Segment SGI15562[] = {{&S82120[109], 3}, {NULL}};
const Segment SGI15563[] = {{&S82120[75], 5}, {NULL}};
const Segment SGI15564[] = {{&S82121[42], 4}, {NULL}};
const Segment SGI15565[] = {{&S82121[42], 20}, {NULL}};
const Segment SGI15566[] = {{&S82121[30], 4}, {NULL}};
const Segment SGI15567[] = {{&S82121[110], 3}, {NULL}};
const Segment SGI15568[] = {{&S82121[76], 5}, {NULL}};
const Segment SGI15569[] = {{&S82122[42], 4}, {NULL}};
const Segment SGI15570[] = {{&S82122[42], 20}, {NULL}};
const Segment SGI15571[] = {{&S82122[30], 4}, {NULL}};
const Segment SGI15572[] = {{&S82122[110], 3}, {NULL}};
const Segment SGI15573[] = {{&S82122[76], 5}, {NULL}};
const Segment SGI15574[] = {{&S81578[50], 7}, {NULL}};
const Segment SGI15575[] = {{&S81578[50], 18}, {NULL}};
const Segment SGI15576[] = {{&S81578[30], 12}, {NULL}};
const Segment SGI15577[] = {{&S81578[117], 3}, {NULL}};
const Segment SGI15578[] = {{&S81578[82], 6}, {NULL}};
const Segment SGI15579[] = {{&S82123[42], 4}, {NULL}};
const Segment SGI15580[] = {{&S82123[42], 20}, {NULL}};
const Segment SGI15581[] = {{&S82123[30], 4}, {NULL}};
const Segment SGI15582[] = {{&S82123[110], 3}, {NULL}};
const Segment SGI15583[] = {{&S82123[76], 5}, {NULL}};
const Segment SGI15584[] = {{&S81600[98], 3}, {NULL}};
const Segment SGI15585[] = {{&S81600[66], 6}, {NULL}};
const Segment SGI15586[] = {{&S82199[42], 17}, {NULL}};
const Segment SGI15587[] = {{&S82199[42], 29}, {NULL}};
const Segment SGI15588[] = {{&S82199[30], 4}, {NULL}};
const Segment SGI15589[] = {{&S82199[119], 3}, {NULL}};
const Segment SGI15590[] = {{&S82199[85], 5}, {NULL}};
const Segment SGI15591[] = {{&S82760[59], 10}, {NULL}};
const Segment SGI15592[] = {{&S82760[59], 21}, {NULL}};
const Segment SGI15593[] = {{&S82760[45], 6}, {NULL}};
const Segment SGI15594[] = {{&S82760[128], 3}, {NULL}};
const Segment SGI15595[] = {{&S82760[94], 5}, {NULL}};
const Segment SGI15596[] = {{&S82836[44], 7}, {NULL}};
const Segment SGI15597[] = {{&S82836[44], 19}, {NULL}};
const Segment SGI15598[] = {{&S82836[30], 6}, {NULL}};
const Segment SGI15599[] = {{&S82836[111], 3}, {NULL}};
const Segment SGI15600[] = {{&S82836[77], 5}, {NULL}};
const Segment SGI15601[] = {{&S82860[44], 10}, {NULL}};
const Segment SGI15602[] = {{&S82860[44], 21}, {NULL}};
const Segment SGI15603[] = {{&S82860[30], 6}, {NULL}};
const Segment SGI15604[] = {{&S82860[79], 6}, {NULL}};
const Segment SGI15605[] = {{&S82860[106], 13}, {NULL}};
const Segment SGI15606[] = {{&S85441[43], 6}, {NULL}};
const Segment SGI15607[] = {{&S85441[43], 17}, {NULL}};
const Segment SGI15608[] = {{&S85441[30], 5}, {NULL}};
const Segment SGI15609[] = {{&S85441[109], 3}, {NULL}};
const Segment SGI15610[] = {{&S85441[74], 6}, {NULL}};
const Segment SGI15611[] = {{&S85441[129], 8}, {NULL}};
const Segment SGI15612[] = {{&S85442[42], 6}, {NULL}};
const Segment SGI15613[] = {{&S85442[30], 4}, {NULL}};
const Segment SGI15614[] = {{&S85442[99], 3}, {NULL}};
const Segment SGI15615[] = {{&S85442[62], 8}, {NULL}};
const Segment SGI15616[] = {{&S85442[119], 8}, {NULL}};
const Segment SGI15617[] = {{&S85443[42], 6}, {NULL}};
const Segment SGI15618[] = {{&S85443[30], 4}, {NULL}};
const Segment SGI15619[] = {{&S85443[98], 3}, {NULL}};
const Segment SGI15620[] = {{&S85443[62], 6}, {NULL}};
const Segment SGI15621[] = {{&S85443[118], 8}, {NULL}};
const Segment SGI15622[] = {{&S82926[44], 14}, {NULL}};
const Segment SGI15623[] = {{&S82926[44], 26}, {NULL}};
const Segment SGI15624[] = {{&S82926[30], 6}, {NULL}};
const Segment SGI15625[] = {{&S82926[84], 6}, {NULL}};
const Segment SGI15626[] = {{&S82926[111], 13}, {NULL}};
const Segment SGI15627[] = {{&S83422[84], 6}, {NULL}};
const Segment SGI15628[] = {{&S82364[42], 11}, {NULL}};
const Segment SGI15629[] = {{&S82364[42], 24}, {NULL}};
const Segment SGI15630[] = {{&S82364[30], 4}, {NULL}};
const Segment SGI15631[] = {{&S82364[114], 3}, {NULL}};
const Segment SGI15632[] = {{&S82364[80], 5}, {NULL}};
const Segment SGI15633[] = {{&S82511[56], 8}, {&S82511[76], 15}, {NULL}};
const Segment SGI15634[] = {{&S82511[56], 19}, {NULL}};
const Segment SGI15635[] = {{&S82511[30], 4}, {NULL}};
const Segment SGI15636[] = {{&S82511[151], 3}, {NULL}};
const Segment SGI15637[] = {{&S82511[116], 6}, {NULL}};
const Segment SGI15638[] = {{&S82512[56], 11}, {&S82512[79], 15}, {NULL}};
const Segment SGI15639[] = {{&S82512[56], 22}, {NULL}};
const Segment SGI15640[] = {{&S82512[30], 4}, {NULL}};
const Segment SGI15641[] = {{&S82512[154], 3}, {NULL}};
const Segment SGI15642[] = {{&S82512[119], 6}, {NULL}};
const Segment SGI15643[] = {{&S82513[56], 11}, {&S82513[79], 15}, {NULL}};
const Segment SGI15644[] = {{&S82513[56], 22}, {NULL}};
const Segment SGI15645[] = {{&S82513[30], 4}, {NULL}};
const Segment SGI15646[] = {{&S82513[154], 3}, {NULL}};
const Segment SGI15647[] = {{&S82513[119], 6}, {NULL}};
const Segment SGI15648[] = {{&S82514[61], 10}, {&S82514[83], 15}, {NULL}};
const Segment SGI15649[] = {{&S82514[61], 21}, {NULL}};
const Segment SGI15650[] = {{&S82514[30], 4}, {NULL}};
const Segment SGI15651[] = {{&S82514[158], 3}, {NULL}};
const Segment SGI15652[] = {{&S82514[123], 6}, {NULL}};
const Segment SGI15653[] = {{&S82023[42], 15}, {NULL}};
const Segment SGI15654[] = {{&S82023[42], 26}, {NULL}};
const Segment SGI15655[] = {{&S82023[30], 4}, {NULL}};
const Segment SGI15656[] = {{&S82023[117], 3}, {NULL}};
const Segment SGI15657[] = {{&S82023[82], 6}, {NULL}};
const Segment SGI15658[] = {{&S82024[42], 11}, {NULL}};
const Segment SGI15659[] = {{&S82024[42], 22}, {NULL}};
const Segment SGI15660[] = {{&S82024[30], 4}, {NULL}};
const Segment SGI15661[] = {{&S82024[113], 3}, {NULL}};
const Segment SGI15662[] = {{&S82024[78], 6}, {NULL}};
const Segment SGI15663[] = {{&S85898[42], 12}, {NULL}};
const Segment SGI15664[] = {{&S85898[42], 23}, {NULL}};
const Segment SGI15665[] = {{&S85898[30], 4}, {NULL}};
const Segment SGI15666[] = {{&S85898[114], 3}, {NULL}};
const Segment SGI15667[] = {{&S85898[79], 6}, {NULL}};
const Segment SGI15668[] = {{&S85898[134], 8}, {NULL}};
const Segment SGI15669[] = {{&S85899[42], 5}, {NULL}};
const Segment SGI15670[] = {{&S85899[42], 16}, {NULL}};
const Segment SGI15671[] = {{&S85899[30], 4}, {NULL}};
const Segment SGI15672[] = {{&S85899[72], 6}, {NULL}};
const Segment SGI15673[] = {{&S85899[126], 8}, {NULL}};
const Segment SGI15674[] = {{&S85900[42], 5}, {NULL}};
const Segment SGI15675[] = {{&S85900[42], 16}, {NULL}};
const Segment SGI15676[] = {{&S85900[30], 4}, {NULL}};
const Segment SGI15677[] = {{&S85900[107], 3}, {NULL}};
const Segment SGI15678[] = {{&S85900[72], 6}, {NULL}};
const Segment SGI15679[] = {{&S85900[127], 8}, {NULL}};
const Segment SGI15680[] = {{&S82257[50], 7}, {NULL}};
const Segment SGI15681[] = {{&S82257[50], 18}, {NULL}};
const Segment SGI15682[] = {{&S82257[30], 12}, {NULL}};
const Segment SGI15683[] = {{&S82257[117], 3}, {NULL}};
const Segment SGI15684[] = {{&S82257[82], 6}, {NULL}};
const Segment SGI15685[] = {{&S82258[50], 6}, {NULL}};
const Segment SGI15686[] = {{&S82258[50], 17}, {NULL}};
const Segment SGI15687[] = {{&S82258[30], 12}, {NULL}};
const Segment SGI15688[] = {{&S82258[116], 3}, {NULL}};
const Segment SGI15689[] = {{&S82258[81], 6}, {NULL}};
const Segment SGI15690[] = {{&S82259[50], 6}, {NULL}};
const Segment SGI15691[] = {{&S82259[50], 17}, {NULL}};
const Segment SGI15692[] = {{&S82259[30], 12}, {NULL}};
const Segment SGI15693[] = {{&S82259[116], 3}, {NULL}};
const Segment SGI15694[] = {{&S82259[81], 6}, {NULL}};
const Segment SGI15695[] = {{&S82835[42], 9}, {NULL}};
const Segment SGI15696[] = {{&S82835[42], 20}, {NULL}};
const Segment SGI15697[] = {{&S82835[30], 4}, {NULL}};
const Segment SGI15698[] = {{&S82835[110], 3}, {NULL}};
const Segment SGI15699[] = {{&S82835[76], 5}, {NULL}};
const Segment SGI15700[] = {{&S82361[50], 11}, {NULL}};
const Segment SGI15701[] = {{&S82361[50], 23}, {NULL}};
const Segment SGI15702[] = {{&S82361[30], 12}, {NULL}};
const Segment SGI15703[] = {{&S82361[122], 3}, {NULL}};
const Segment SGI15704[] = {{&S82361[87], 6}, {NULL}};
const Segment SGI15705[] = {{&S82362[44], 11}, {NULL}};
const Segment SGI15706[] = {{&S82362[31], 12}, {NULL}};
const Segment SGI15707[] = {{&S83593[45], 10}, {NULL}};
const Segment SGI15708[] = {{&S83593[30], 6}, {NULL}};
const Segment SGI15709[] = {{&S83593[114], 3}, {NULL}};
const Segment SGI15710[] = {{&S83593[80], 5}, {NULL}};
const Segment SGI15711[] = {{&S83593[138], 13}, {NULL}};
const Segment SGI15712[] = {{&S83018[42], 10}, {NULL}};
const Segment SGI15713[] = {{&S83018[42], 21}, {NULL}};
const Segment SGI15714[] = {{&S83018[30], 4}, {NULL}};
const Segment SGI15715[] = {{&S83018[77], 6}, {NULL}};
const Segment SGI15716[] = {{&S83018[104], 13}, {NULL}};
const Segment SGI15717[] = {{&S83741[44], 15}, {NULL}};
const Segment SGI15718[] = {{&S83741[44], 26}, {NULL}};
const Segment SGI15719[] = {{&S83741[30], 6}, {NULL}};
const Segment SGI15720[] = {{&S83741[119], 3}, {NULL}};
const Segment SGI15721[] = {{&S83741[84], 6}, {NULL}};
const Segment SGI15722[] = {{&S83742[44], 8}, {NULL}};
const Segment SGI15723[] = {{&S83742[44], 19}, {NULL}};
const Segment SGI15724[] = {{&S83742[30], 6}, {NULL}};
const Segment SGI15725[] = {{&S83742[80], 4}, {NULL}};
const Segment SGI15726[] = {{&S83742[80], 3}, {NULL}};
const Segment SGI15727[] = {{&S83743[44], 8}, {NULL}};
const Segment SGI15728[] = {{&S83743[44], 19}, {NULL}};
const Segment SGI15729[] = {{&S83743[30], 6}, {NULL}};
const Segment SGI15730[] = {{&S83743[80], 6}, {NULL}};
const Segment SGI15731[] = {{&S83772[44], 5}, {NULL}};
const Segment SGI15732[] = {{&S83772[44], 16}, {NULL}};
const Segment SGI15733[] = {{&S83772[30], 6}, {NULL}};
const Segment SGI15734[] = {{&S83772[108], 3}, {NULL}};
const Segment SGI15735[] = {{&S83772[74], 5}, {NULL}};
const Segment SGI15736[] = {{&S89388[0], 15}, {NULL}};
const Segment SGI15737[] = {{&S83908[54], 10}, {NULL}};
const Segment SGI15738[] = {{&S83908[54], 21}, {NULL}};
const Segment SGI15739[] = {{&S83908[30], 16}, {NULL}};
const Segment SGI15740[] = {{&S83908[123], 3}, {NULL}};
const Segment SGI15741[] = {{&S83908[89], 5}, {NULL}};
const Segment SGI15742[] = {{&S83909[52], 10}, {NULL}};
const Segment SGI15743[] = {{&S83909[45], 6}, {&S83909[52], 21}, {NULL}};
const Segment SGI15744[] = {{&S83909[30], 14}, {NULL}};
const Segment SGI15745[] = {{&S83909[121], 3}, {NULL}};
const Segment SGI15746[] = {{&S83909[87], 5}, {NULL}};
const Segment SGI15747[] = {{&S83910[73], 10}, {NULL}};
const Segment SGI15748[] = {{&S83910[73], 21}, {NULL}};
const Segment SGI15749[] = {{&S83910[30], 34}, {NULL}};
const Segment SGI15750[] = {{&S83910[142], 3}, {NULL}};
const Segment SGI15751[] = {{&S83910[108], 5}, {NULL}};
const Segment SGI15752[] = {{&S83911[75], 10}, {NULL}};
const Segment SGI15753[] = {{&S83911[75], 21}, {NULL}};
const Segment SGI15754[] = {{&S83911[30], 36}, {NULL}};
const Segment SGI15755[] = {{&S83911[144], 3}, {NULL}};
const Segment SGI15756[] = {{&S83911[110], 5}, {NULL}};
const Segment SGI15757[] = {{&S83912[71], 10}, {NULL}};
const Segment SGI15758[] = {{&S83912[71], 21}, {NULL}};
const Segment SGI15759[] = {{&S83912[30], 32}, {NULL}};
const Segment SGI15760[] = {{&S83912[140], 3}, {NULL}};
const Segment SGI15761[] = {{&S83912[106], 5}, {NULL}};
const Segment SGI15762[] = {{&S83913[48], 10}, {NULL}};
const Segment SGI15763[] = {{&S83913[48], 21}, {NULL}};
const Segment SGI15764[] = {{&S83913[30], 10}, {NULL}};
const Segment SGI15765[] = {{&S83913[117], 3}, {NULL}};
const Segment SGI15766[] = {{&S83913[83], 5}, {NULL}};
const Segment SGI15767[] = {{&S83856[45], 14}, {NULL}};
const Segment SGI15768[] = {{&S83856[30], 6}, {NULL}};
const Segment SGI15769[] = {{&S83856[119], 3}, {NULL}};
const Segment SGI15770[] = {{&S83856[85], 5}, {NULL}};
const Segment SGI15771[] = {{&S83856[143], 13}, {NULL}};
const Segment SGI15772[] = {{&S83857[44], 14}, {NULL}};
const Segment SGI15773[] = {{&S83857[44], 26}, {NULL}};
const Segment SGI15774[] = {{&S83857[30], 6}, {NULL}};
const Segment SGI15775[] = {{&S83857[119], 3}, {NULL}};
const Segment SGI15776[] = {{&S83857[84], 6}, {NULL}};
const Segment SGI15777[] = {{&S83857[139], 8}, {NULL}};
const Segment SGI15778[] = {{&S83857[148], 13}, {NULL}};
const Segment SGI15779[] = {{&S82620[51], 6}, {NULL}};
const Segment SGI15780[] = {{&S82620[30], 12}, {NULL}};
const Segment SGI15781[] = {{&S82620[123], 3}, {NULL}};
const Segment SGI15782[] = {{&S82620[89], 5}, {NULL}};
const Segment SGI15783[] = {{&S82620[147], 13}, {NULL}};
const Segment SGI15784[] = {{&S86421[6], 4}, {NULL}};
const Segment SGI15785[] = {{&S86421[83], 5}, {NULL}};
const Segment SGI15786[] = {{&S83928[6], 4}, {NULL}};
const Segment SGI15787[] = {{&S83928[87], 5}, {NULL}};
const Segment SGI15788[] = {{&S83935[45], 10}, {NULL}};
const Segment SGI15789[] = {{&S83935[30], 6}, {NULL}};
const Segment SGI15790[] = {{&S83935[114], 3}, {NULL}};
const Segment SGI15791[] = {{&S83935[80], 5}, {NULL}};
const Segment SGI15792[] = {{&S83935[138], 13}, {NULL}};
const Segment SGI15793[] = {{&S82675[50], 13}, {NULL}};
const Segment SGI15794[] = {{&S82675[50], 24}, {NULL}};
const Segment SGI15795[] = {{&S82675[30], 12}, {NULL}};
const Segment SGI15796[] = {{&S82675[123], 3}, {NULL}};
const Segment SGI15797[] = {{&S82675[88], 6}, {NULL}};
const Segment SGI15798[] = {{&S89557[8], 6}, {NULL}};
const Segment SGI15799[] = {{&S89557[0], 84}, {NULL}};
const Segment SGI15800[] = {{&S82689[6], 4}, {NULL}};
const Segment SGI15801[] = {{&S82689[82], 5}, {NULL}};
const Segment SGI15802[] = {{&S89558[8], 7}, {NULL}};
const Segment SGI15803[] = {{&S89558[0], 85}, {NULL}};
const Segment SGI15804[] = {{&S83315[43], 9}, {NULL}};
const Segment SGI15805[] = {{&S83315[30], 4}, {NULL}};
const Segment SGI15806[] = {{&S83315[111], 3}, {NULL}};
const Segment SGI15807[] = {{&S83315[77], 5}, {NULL}};
const Segment SGI15808[] = {{&S83315[135], 13}, {NULL}};
const Segment SGI15809[] = {{&S84129[62], 8}, {&S84129[62], 8}, {NULL}};
const Segment SGI15810[] = {{&S84129[0], 13}, {NULL}};
const Segment SGI15811[] = {{&S84130[62], 8}, {NULL}};
const Segment SGI15812[] = {{&S84130[0], 13}, {NULL}};
const Segment SGI15813[] = {{&S84024[44], 10}, {NULL}};
const Segment SGI15814[] = {{&S84024[44], 21}, {NULL}};
const Segment SGI15815[] = {{&S84024[30], 6}, {NULL}};
const Segment SGI15816[] = {{&S84024[79], 6}, {NULL}};
const Segment SGI15817[] = {{&S84024[106], 13}, {NULL}};
const Segment SGI15818[] = {{&S84134[80], 6}, {NULL}};
const Segment SGI15819[] = {{&S83391[42], 10}, {NULL}};
const Segment SGI15820[] = {{&S83391[42], 21}, {NULL}};
const Segment SGI15821[] = {{&S83391[30], 4}, {NULL}};
const Segment SGI15822[] = {{&S83391[111], 3}, {NULL}};
const Segment SGI15823[] = {{&S83391[77], 5}, {NULL}};
const Segment SGI15824[] = {{&S84214[77], 6}, {NULL}};
const Segment SGI15825[] = {{&S83572[42], 12}, {NULL}};
const Segment SGI15826[] = {{&S83572[42], 23}, {NULL}};
const Segment SGI15827[] = {{&S83572[30], 4}, {NULL}};
const Segment SGI15828[] = {{&S83572[113], 3}, {NULL}};
const Segment SGI15829[] = {{&S83572[79], 5}, {NULL}};
const Segment SGI15830[] = {{&S83573[30], 4}, {NULL}};
const Segment SGI15831[] = {{&S83573[99], 3}, {NULL}};
const Segment SGI15832[] = {{&S83573[65], 5}, {NULL}};
const Segment SGI15833[] = {{&S83574[40], 12}, {NULL}};
const Segment SGI15834[] = {{&S83574[30], 4}, {NULL}};
const Segment SGI15835[] = {{&S83574[111], 3}, {NULL}};
const Segment SGI15836[] = {{&S83574[77], 5}, {NULL}};
const Segment SGI15837[] = {{&S83575[42], 11}, {NULL}};
const Segment SGI15838[] = {{&S83575[30], 4}, {NULL}};
const Segment SGI15839[] = {{&S83575[101], 3}, {NULL}};
const Segment SGI15840[] = {{&S83575[67], 5}, {NULL}};
const Segment SGI15841[] = {{&S84343[73], 10}, {NULL}};
const Segment SGI15842[] = {{&S84343[73], 22}, {NULL}};
const Segment SGI15843[] = {{&S84343[30], 35}, {NULL}};
const Segment SGI15844[] = {{&S84343[144], 3}, {NULL}};
const Segment SGI15845[] = {{&S84343[109], 6}, {NULL}};
const Segment SGI15846[] = {{&S84344[63], 10}, {NULL}};
const Segment SGI15847[] = {{&S84344[63], 22}, {NULL}};
const Segment SGI15848[] = {{&S84344[30], 25}, {NULL}};
const Segment SGI15849[] = {{&S84344[134], 3}, {NULL}};
const Segment SGI15850[] = {{&S84344[99], 6}, {NULL}};
const Segment SGI15851[] = {{&S83579[42], 8}, {NULL}};
const Segment SGI15852[] = {{&S83579[42], 19}, {NULL}};
const Segment SGI15853[] = {{&S83579[30], 4}, {NULL}};
const Segment SGI15854[] = {{&S83579[109], 3}, {NULL}};
const Segment SGI15855[] = {{&S83579[75], 5}, {NULL}};
const Segment SGI15856[] = {{&S84345[54], 10}, {NULL}};
const Segment SGI15857[] = {{&S84345[54], 22}, {NULL}};
const Segment SGI15858[] = {{&S84345[30], 16}, {NULL}};
const Segment SGI15859[] = {{&S84345[125], 3}, {NULL}};
const Segment SGI15860[] = {{&S84345[90], 6}, {NULL}};
const Segment SGI15861[] = {{&S84346[63], 10}, {NULL}};
const Segment SGI15862[] = {{&S84346[63], 22}, {NULL}};
const Segment SGI15863[] = {{&S84346[30], 24}, {NULL}};
const Segment SGI15864[] = {{&S84346[134], 3}, {NULL}};
const Segment SGI15865[] = {{&S84346[99], 6}, {NULL}};
const Segment SGI15866[] = {{&S84347[62], 10}, {NULL}};
const Segment SGI15867[] = {{&S84347[62], 22}, {NULL}};
const Segment SGI15868[] = {{&S84347[30], 24}, {NULL}};
const Segment SGI15869[] = {{&S84347[133], 3}, {NULL}};
const Segment SGI15870[] = {{&S84347[98], 6}, {NULL}};
const Segment SGI15871[] = {{&S84348[62], 10}, {NULL}};
const Segment SGI15872[] = {{&S84348[62], 22}, {NULL}};
const Segment SGI15873[] = {{&S84348[30], 24}, {NULL}};
const Segment SGI15874[] = {{&S84348[133], 3}, {NULL}};
const Segment SGI15875[] = {{&S84348[98], 6}, {NULL}};
const Segment SGI15876[] = {{&S84349[62], 10}, {NULL}};
const Segment SGI15877[] = {{&S84349[62], 22}, {NULL}};
const Segment SGI15878[] = {{&S84349[30], 24}, {NULL}};
const Segment SGI15879[] = {{&S84349[133], 3}, {NULL}};
const Segment SGI15880[] = {{&S84349[98], 6}, {NULL}};
const Segment SGI15881[] = {{&S84350[62], 10}, {NULL}};
const Segment SGI15882[] = {{&S84350[62], 22}, {NULL}};
const Segment SGI15883[] = {{&S84350[30], 24}, {NULL}};
const Segment SGI15884[] = {{&S84350[133], 3}, {NULL}};
const Segment SGI15885[] = {{&S84350[98], 6}, {NULL}};
const Segment SGI15886[] = {{&S84351[62], 10}, {NULL}};
const Segment SGI15887[] = {{&S84351[62], 22}, {NULL}};
const Segment SGI15888[] = {{&S84351[30], 24}, {NULL}};
const Segment SGI15889[] = {{&S84351[133], 3}, {NULL}};
const Segment SGI15890[] = {{&S84351[98], 6}, {NULL}};
const Segment SGI15891[] = {{&S84352[73], 10}, {NULL}};
const Segment SGI15892[] = {{&S84352[73], 22}, {NULL}};
const Segment SGI15893[] = {{&S89857[8], 7}, {NULL}};
const Segment SGI15894[] = {{&S89857[0], 45}, {NULL}};
const Segment SGI15895[] = {{&S84352[30], 35}, {NULL}};
const Segment SGI15896[] = {{&S84352[144], 3}, {NULL}};
const Segment SGI15897[] = {{&S84352[109], 6}, {NULL}};
const Segment SGI15898[] = {{&S84353[74], 10}, {NULL}};
const Segment SGI15899[] = {{&S84353[74], 22}, {NULL}};
const Segment SGI15900[] = {{&S84353[30], 36}, {NULL}};
const Segment SGI15901[] = {{&S84353[145], 3}, {NULL}};
const Segment SGI15902[] = {{&S84353[110], 6}, {NULL}};
const Segment SGI15903[] = {{&S84354[68], 10}, {NULL}};
const Segment SGI15904[] = {{&S84354[68], 22}, {NULL}};
const Segment SGI15905[] = {{&S84354[30], 29}, {NULL}};
const Segment SGI15906[] = {{&S84354[139], 3}, {NULL}};
const Segment SGI15907[] = {{&S84354[104], 6}, {NULL}};
const Segment SGI15908[] = {{&S84355[62], 10}, {NULL}};
const Segment SGI15909[] = {{&S84355[62], 22}, {NULL}};
const Segment SGI15910[] = {{&S84355[30], 24}, {NULL}};
const Segment SGI15911[] = {{&S84355[133], 3}, {NULL}};
const Segment SGI15912[] = {{&S84355[98], 6}, {NULL}};
const Segment SGI15913[] = {{&S84356[57], 10}, {NULL}};
const Segment SGI15914[] = {{&S84356[57], 22}, {NULL}};
const Segment SGI15915[] = {{&S84356[30], 19}, {NULL}};
const Segment SGI15916[] = {{&S84356[128], 3}, {NULL}};
const Segment SGI15917[] = {{&S84356[93], 6}, {NULL}};
const Segment SGI15918[] = {{&S84357[59], 10}, {NULL}};
const Segment SGI15919[] = {{&S84357[59], 22}, {NULL}};
const Segment SGI15920[] = {{&S84357[30], 21}, {NULL}};
const Segment SGI15921[] = {{&S84357[130], 3}, {NULL}};
const Segment SGI15922[] = {{&S84357[95], 6}, {NULL}};
const Segment SGI15923[] = {{&S84361[95], 3}, {NULL}};
const Segment SGI15924[] = {{&S84361[81], 5}, {NULL}};
const Segment SGI15925[] = {{&S84420[8], 8}, {NULL}};
const Segment SGI15926[] = {{&S83750[42], 6}, {NULL}};
const Segment SGI15927[] = {{&S83750[42], 22}, {NULL}};
const Segment SGI15928[] = {{&S83750[30], 4}, {NULL}};
const Segment SGI15929[] = {{&S83750[112], 3}, {NULL}};
const Segment SGI15930[] = {{&S83750[78], 5}, {NULL}};
const Segment SGI15931[] = {{&S83751[42], 6}, {NULL}};
const Segment SGI15932[] = {{&S83751[42], 16}, {NULL}};
const Segment SGI15933[] = {{&S83751[30], 4}, {NULL}};
const Segment SGI15934[] = {{&S83751[106], 3}, {NULL}};
const Segment SGI15935[] = {{&S83751[72], 5}, {NULL}};
const Segment SGI15936[] = {{&S84412[45], 14}, {NULL}};
const Segment SGI15937[] = {{&S83752[42], 6}, {NULL}};
const Segment SGI15938[] = {{&S83752[42], 16}, {NULL}};
const Segment SGI15939[] = {{&S84412[30], 6}, {NULL}};
const Segment SGI15940[] = {{&S83752[30], 4}, {NULL}};
const Segment SGI15941[] = {{&S83752[106], 3}, {NULL}};
const Segment SGI15942[] = {{&S84412[119], 3}, {NULL}};
const Segment SGI15943[] = {{&S83752[72], 5}, {NULL}};
const Segment SGI15944[] = {{&S84412[85], 5}, {NULL}};
const Segment SGI15945[] = {{&S84412[143], 13}, {NULL}};
const Segment SGI15946[] = {{&S83753[42], 6}, {NULL}};
const Segment SGI15947[] = {{&S83753[42], 39}, {NULL}};
const Segment SGI15948[] = {{&S83753[30], 4}, {NULL}};
const Segment SGI15949[] = {{&S83753[129], 3}, {NULL}};
const Segment SGI15950[] = {{&S83753[95], 5}, {NULL}};
const Segment SGI15951[] = {{&S84536[8], 8}, {NULL}};
const Segment SGI15952[] = {{&S83840[43], 9}, {NULL}};
const Segment SGI15953[] = {{&S83840[30], 4}, {NULL}};
const Segment SGI15954[] = {{&S83840[111], 3}, {NULL}};
const Segment SGI15955[] = {{&S83840[77], 5}, {NULL}};
const Segment SGI15956[] = {{&S83840[135], 13}, {NULL}};
const Segment SGI15957[] = {{&S83181[50], 6}, {NULL}};
const Segment SGI15958[] = {{&S83181[50], 24}, {NULL}};
const Segment SGI15959[] = {{&S83181[30], 12}, {NULL}};
const Segment SGI15960[] = {{&S83181[88], 6}, {NULL}};
const Segment SGI15961[] = {{&S83181[115], 13}, {NULL}};
const Segment SGI15962[] = {{&S83205[50], 4}, {NULL}};
const Segment SGI15963[] = {{&S83205[101], 6}, {NULL}};
const Segment SGI15964[] = {{&S84696[44], 7}, {NULL}};
const Segment SGI15965[] = {{&S84696[44], 19}, {NULL}};
const Segment SGI15966[] = {{&S84696[30], 6}, {NULL}};
const Segment SGI15967[] = {{&S84696[111], 3}, {NULL}};
const Segment SGI15968[] = {{&S84696[77], 5}, {NULL}};
const Segment SGI15969[] = {{&S84730[43], 8}, {&S84730[43], 8}, {NULL}};
const Segment SGI15970[] = {{&S84730[69], 9}, {NULL}};
const Segment SGI15971[] = {{&S84730[73], 6}, {NULL}};
const Segment SGI15972[] = {{&S84853[44], 8}, {NULL}};
const Segment SGI15973[] = {{&S84853[44], 20}, {NULL}};
const Segment SGI15974[] = {{&S84853[30], 6}, {NULL}};
const Segment SGI15975[] = {{&S84853[112], 3}, {NULL}};
const Segment SGI15976[] = {{&S84853[78], 5}, {NULL}};
const Segment SGI15977[] = {{&S84263[43], 4}, {NULL}};
const Segment SGI15978[] = {{&S84263[96], 5}, {NULL}};
const Segment SGI15979[] = {{&S83488[45], 4}, {NULL}};
const Segment SGI15980[] = {{&S83488[105], 3}, {NULL}};
const Segment SGI15981[] = {{&S83488[94], 3}, {NULL}};
const Segment SGI15982[] = {{&S83488[71], 6}, {NULL}};
const Segment SGI15983[] = {{&S83489[37], 4}, {NULL}};
const Segment SGI15984[] = {{&S83489[56], 3}, {NULL}};
const Segment SGI15985[] = {{&S85052[44], 15}, {NULL}};
const Segment SGI15986[] = {{&S85052[44], 26}, {NULL}};
const Segment SGI15987[] = {{&S85052[30], 6}, {NULL}};
const Segment SGI15988[] = {{&S85052[119], 3}, {NULL}};
const Segment SGI15989[] = {{&S85052[84], 6}, {NULL}};
const Segment SGI15990[] = {{&S85053[44], 8}, {NULL}};
const Segment SGI15991[] = {{&S85053[44], 19}, {NULL}};
const Segment SGI15992[] = {{&S85053[30], 6}, {NULL}};
const Segment SGI15993[] = {{&S85053[80], 4}, {NULL}};
const Segment SGI15994[] = {{&S85053[80], 3}, {NULL}};
const Segment SGI15995[] = {{&S85054[44], 8}, {NULL}};
const Segment SGI15996[] = {{&S85054[44], 19}, {NULL}};
const Segment SGI15997[] = {{&S85054[30], 6}, {NULL}};
const Segment SGI15998[] = {{&S85054[80], 6}, {NULL}};
const Segment SGI15999[] = {{&S85088[45], 10}, {NULL}};
const Segment SGI16000[] = {{&S85088[30], 6}, {NULL}};
const Segment SGI16001[] = {{&S85088[114], 3}, {NULL}};
const Segment SGI16002[] = {{&S85088[80], 5}, {NULL}};
const Segment SGI16003[] = {{&S85088[138], 13}, {NULL}};
const Segment SGI16004[] = {{&S87182[39], 4}, {NULL}};
const Segment SGI16005[] = {{&S87182[92], 3}, {NULL}};
const Segment SGI16006[] = {{&S87182[57], 6}, {NULL}};
const Segment SGI16007[] = {{&S87182[112], 8}, {NULL}};
const Segment SGI16008[] = {{&S85361[45], 10}, {NULL}};
const Segment SGI16009[] = {{&S85361[30], 6}, {NULL}};
const Segment SGI16010[] = {{&S85361[114], 3}, {NULL}};
const Segment SGI16011[] = {{&S85361[80], 5}, {NULL}};
const Segment SGI16012[] = {{&S85361[138], 13}, {NULL}};
const Segment SGI16013[] = {{&S84071[50], 8}, {NULL}};
const Segment SGI16014[] = {{&S84071[50], 19}, {NULL}};
const Segment SGI16015[] = {{&S84071[30], 12}, {NULL}};
const Segment SGI16016[] = {{&S84071[118], 3}, {NULL}};
const Segment SGI16017[] = {{&S84071[83], 6}, {NULL}};
const Segment SGI16018[] = {{&S84072[57], 10}, {NULL}};
const Segment SGI16019[] = {{&S84072[57], 22}, {NULL}};
const Segment SGI16020[] = {{&S84072[30], 19}, {NULL}};
const Segment SGI16021[] = {{&S84072[128], 3}, {NULL}};
const Segment SGI16022[] = {{&S84072[93], 6}, {NULL}};
const Segment SGI16023[] = {{&S85581[45], 10}, {NULL}};
const Segment SGI16024[] = {{&S85581[30], 6}, {NULL}};
const Segment SGI16025[] = {{&S85581[114], 3}, {NULL}};
const Segment SGI16026[] = {{&S85581[80], 5}, {NULL}};
const Segment SGI16027[] = {{&S85581[138], 13}, {NULL}};
const Segment SGI16028[] = {{&S85648[44], 7}, {NULL}};
const Segment SGI16029[] = {{&S85648[44], 21}, {NULL}};
const Segment SGI16030[] = {{&S85648[30], 6}, {NULL}};
const Segment SGI16031[] = {{&S85648[113], 3}, {NULL}};
const Segment SGI16032[] = {{&S85648[79], 5}, {NULL}};
const Segment SGI16033[] = {{&S85649[38], 7}, {NULL}};
const Segment SGI16034[] = {{&S85649[31], 6}, {NULL}};
const Segment SGI16035[] = {{&S84261[50], 16}, {NULL}};
const Segment SGI16036[] = {{&S84261[50], 27}, {NULL}};
const Segment SGI16037[] = {{&S84261[30], 12}, {NULL}};
const Segment SGI16038[] = {{&S84261[126], 3}, {NULL}};
const Segment SGI16039[] = {{&S84261[91], 6}, {NULL}};
const Segment SGI16040[] = {{&S85839[44], 11}, {NULL}};
const Segment SGI16041[] = {{&S85839[44], 22}, {NULL}};
const Segment SGI16042[] = {{&S85839[30], 6}, {NULL}};
const Segment SGI16043[] = {{&S85839[114], 3}, {NULL}};
const Segment SGI16044[] = {{&S85839[80], 5}, {NULL}};
const Segment SGI16045[] = {{&S85839[148], 3}, {NULL}};
const Segment SGI16046[] = {{&S85842[44], 11}, {NULL}};
const Segment SGI16047[] = {{&S85842[44], 22}, {NULL}};
const Segment SGI16048[] = {{&S85842[30], 6}, {NULL}};
const Segment SGI16049[] = {{&S85842[114], 3}, {NULL}};
const Segment SGI16050[] = {{&S85842[80], 5}, {NULL}};
const Segment SGI16051[] = {{&S85842[148], 3}, {NULL}};
const Segment SGI16052[] = {{&S85844[44], 10}, {NULL}};
const Segment SGI16053[] = {{&S85844[44], 21}, {NULL}};
const Segment SGI16054[] = {{&S85844[30], 6}, {NULL}};
const Segment SGI16055[] = {{&S85844[113], 3}, {NULL}};
const Segment SGI16056[] = {{&S85844[79], 5}, {NULL}};
const Segment SGI16057[] = {{&S85844[147], 3}, {NULL}};
const Segment SGI16058[] = {{&S91475[8], 6}, {NULL}};
const Segment SGI16059[] = {{&S91475[0], 59}, {NULL}};
const Segment SGI16060[] = {{&S91476[8], 6}, {NULL}};
const Segment SGI16061[] = {{&S91476[0], 59}, {NULL}};
const Segment SGI16062[] = {{&S91477[8], 6}, {NULL}};
const Segment SGI16063[] = {{&S91477[0], 44}, {NULL}};
const Segment SGI16064[] = {{&S88205[54], 14}, {NULL}};
const Segment SGI16065[] = {{&S88205[54], 33}, {NULL}};
const Segment SGI16066[] = {{&S88205[30], 16}, {NULL}};
const Segment SGI16067[] = {{&S88205[104], 6}, {NULL}};
const Segment SGI16068[] = {{&S88206[54], 14}, {NULL}};
const Segment SGI16069[] = {{&S88206[54], 33}, {NULL}};
const Segment SGI16070[] = {{&S88206[30], 16}, {NULL}};
const Segment SGI16071[] = {{&S88206[104], 6}, {NULL}};
const Segment SGI16072[] = {{&S86218[44], 10}, {NULL}};
const Segment SGI16073[] = {{&S86218[44], 21}, {NULL}};
const Segment SGI16074[] = {{&S86218[30], 6}, {NULL}};
const Segment SGI16075[] = {{&S86218[113], 3}, {NULL}};
const Segment SGI16076[] = {{&S86218[79], 5}, {NULL}};
const Segment SGI16077[] = {{&S86223[31], 7}, {NULL}};
const Segment SGI16078[] = {{&S86223[53], 3}, {NULL}};
const Segment SGI16079[] = {{&S86318[44], 14}, {NULL}};
const Segment SGI16080[] = {{&S86318[44], 26}, {NULL}};
const Segment SGI16081[] = {{&S86318[30], 6}, {NULL}};
const Segment SGI16082[] = {{&S86318[118], 3}, {NULL}};
const Segment SGI16083[] = {{&S86318[84], 5}, {NULL}};
const Segment SGI16084[] = {{&S86319[38], 14}, {NULL}};
const Segment SGI16085[] = {{&S86319[31], 6}, {NULL}};
const Segment SGI16086[] = {{&S86320[44], 14}, {NULL}};
const Segment SGI16087[] = {{&S86320[44], 26}, {NULL}};
const Segment SGI16088[] = {{&S86320[30], 6}, {NULL}};
const Segment SGI16089[] = {{&S86320[87], 4}, {NULL}};
const Segment SGI16090[] = {{&S86320[87], 3}, {NULL}};
const Segment SGI16091[] = {{&S91816[0], 107}, {NULL}};
const Segment SGI16092[] = {{&S91817[0], 107}, {NULL}};
const Segment SGI16093[] = {{&S85910[64], 8}, {NULL}};
const Segment SGI16094[] = {{&S85910[64], 20}, {NULL}};
const Segment SGI16095[] = {{&S85910[30], 26}, {NULL}};
const Segment SGI16096[] = {{&S85910[133], 3}, {NULL}};
const Segment SGI16097[] = {{&S85910[98], 6}, {NULL}};
const Segment SGI16098[] = {{&S85911[55], 8}, {NULL}};
const Segment SGI16099[] = {{&S85911[55], 20}, {NULL}};
const Segment SGI16100[] = {{&S85911[30], 17}, {NULL}};
const Segment SGI16101[] = {{&S85911[124], 3}, {NULL}};
const Segment SGI16102[] = {{&S85911[89], 6}, {NULL}};
const Segment SGI16103[] = {{&S85319[42], 10}, {NULL}};
const Segment SGI16104[] = {{&S85319[42], 21}, {NULL}};
const Segment SGI16105[] = {{&S85319[30], 4}, {NULL}};
const Segment SGI16106[] = {{&S85319[77], 6}, {NULL}};
const Segment SGI16107[] = {{&S85319[104], 13}, {NULL}};
const Segment SGI16108[] = {{&S91860[0], 108}, {NULL}};
const Segment SGI16109[] = {{&S85337[42], 11}, {NULL}};
const Segment SGI16110[] = {{&S85337[42], 22}, {NULL}};
const Segment SGI16111[] = {{&S85337[30], 4}, {NULL}};
const Segment SGI16112[] = {{&S85337[112], 3}, {NULL}};
const Segment SGI16113[] = {{&S85337[78], 5}, {NULL}};
const Segment SGI16114[] = {{&S84944[50], 7}, {NULL}};
const Segment SGI16115[] = {{&S84944[50], 18}, {NULL}};
const Segment SGI16116[] = {{&S84944[30], 12}, {NULL}};
const Segment SGI16117[] = {{&S84944[117], 3}, {NULL}};
const Segment SGI16118[] = {{&S84944[82], 6}, {NULL}};
const Segment SGI16119[] = {{&S84945[50], 11}, {NULL}};
const Segment SGI16120[] = {{&S84945[50], 22}, {NULL}};
const Segment SGI16121[] = {{&S84945[30], 12}, {NULL}};
const Segment SGI16122[] = {{&S84945[121], 3}, {NULL}};
const Segment SGI16123[] = {{&S84945[86], 6}, {NULL}};
const Segment SGI16124[] = {{&S84946[50], 20}, {NULL}};
const Segment SGI16125[] = {{&S84946[50], 31}, {NULL}};
const Segment SGI16126[] = {{&S84946[30], 12}, {NULL}};
const Segment SGI16127[] = {{&S84946[130], 3}, {NULL}};
const Segment SGI16128[] = {{&S84946[95], 6}, {NULL}};
const Segment SGI16129[] = {{&S85986[79], 6}, {NULL}};
const Segment SGI16130[] = {{&S85433[42], 9}, {NULL}};
const Segment SGI16131[] = {{&S85433[42], 21}, {NULL}};
const Segment SGI16132[] = {{&S85433[30], 4}, {NULL}};
const Segment SGI16133[] = {{&S85433[111], 3}, {NULL}};
const Segment SGI16134[] = {{&S85433[77], 5}, {NULL}};
const Segment SGI16135[] = {{&S86042[8], 8}, {NULL}};
const Segment SGI16136[] = {{&S84970[51], 6}, {NULL}};
const Segment SGI16137[] = {{&S84970[30], 12}, {NULL}};
const Segment SGI16138[] = {{&S84970[123], 3}, {NULL}};
const Segment SGI16139[] = {{&S84970[89], 5}, {NULL}};
const Segment SGI16140[] = {{&S84970[147], 13}, {NULL}};
const Segment SGI16141[] = {{&S84972[57], 10}, {NULL}};
const Segment SGI16142[] = {{&S84972[57], 35}, {NULL}};
const Segment SGI16143[] = {{&S84972[30], 19}, {NULL}};
const Segment SGI16144[] = {{&S84972[141], 3}, {NULL}};
const Segment SGI16145[] = {{&S84972[106], 6}, {NULL}};
const Segment SGI16146[] = {{&S84973[57], 10}, {NULL}};
const Segment SGI16147[] = {{&S84973[57], 22}, {NULL}};
const Segment SGI16148[] = {{&S84973[30], 19}, {NULL}};
const Segment SGI16149[] = {{&S84973[128], 3}, {NULL}};
const Segment SGI16150[] = {{&S84973[93], 6}, {NULL}};
const Segment SGI16151[] = {{&S92043[0], 107}, {NULL}};
const Segment SGI16152[] = {{&S86339[59], 7}, {NULL}};
const Segment SGI16153[] = {{&S86339[59], 19}, {NULL}};
const Segment SGI16154[] = {{&S86339[45], 6}, {NULL}};
const Segment SGI16155[] = {{&S86339[126], 3}, {NULL}};
const Segment SGI16156[] = {{&S86339[92], 5}, {NULL}};
const Segment SGI16157[] = {{&S86885[56], 10}, {NULL}};
const Segment SGI16158[] = {{&S86885[56], 21}, {NULL}};
const Segment SGI16159[] = {{&S86885[30], 18}, {NULL}};
const Segment SGI16160[] = {{&S86885[125], 3}, {NULL}};
const Segment SGI16161[] = {{&S86885[91], 5}, {NULL}};
const Segment SGI16162[] = {{&S86886[58], 10}, {NULL}};
const Segment SGI16163[] = {{&S86886[58], 21}, {NULL}};
const Segment SGI16164[] = {{&S86886[30], 20}, {NULL}};
const Segment SGI16165[] = {{&S86886[127], 3}, {NULL}};
const Segment SGI16166[] = {{&S86886[93], 5}, {NULL}};
const Segment SGI16167[] = {{&S86945[45], 10}, {NULL}};
const Segment SGI16168[] = {{&S86945[30], 6}, {NULL}};
const Segment SGI16169[] = {{&S86945[114], 3}, {NULL}};
const Segment SGI16170[] = {{&S86945[80], 5}, {NULL}};
const Segment SGI16171[] = {{&S86945[138], 13}, {NULL}};
const Segment SGI16172[] = {{&S92256[0], 107}, {NULL}};
const Segment SGI16173[] = {{&S92257[0], 107}, {NULL}};
const Segment SGI16174[] = {{&S86475[53], 8}, {&S86475[53], 8}, {NULL}};
const Segment SGI16175[] = {{&S86476[66], 8}, {NULL}};
const Segment SGI16176[] = {{&S85798[42], 15}, {NULL}};
const Segment SGI16177[] = {{&S85798[42], 26}, {NULL}};
const Segment SGI16178[] = {{&S85798[30], 4}, {NULL}};
const Segment SGI16179[] = {{&S85798[116], 3}, {NULL}};
const Segment SGI16180[] = {{&S85798[82], 5}, {NULL}};
const Segment SGI16181[] = {{&S86500[8], 8}, {NULL}};
const Segment SGI16182[] = {{&S86535[55], 10}, {NULL}};
const Segment SGI16183[] = {{&S86535[55], 21}, {NULL}};
const Segment SGI16184[] = {{&S86535[30], 17}, {NULL}};
const Segment SGI16185[] = {{&S86535[124], 3}, {NULL}};
const Segment SGI16186[] = {{&S86535[90], 5}, {NULL}};
const Segment SGI16187[] = {{&S86536[53], 10}, {NULL}};
const Segment SGI16188[] = {{&S86536[53], 21}, {NULL}};
const Segment SGI16189[] = {{&S86536[30], 15}, {NULL}};
const Segment SGI16190[] = {{&S86536[122], 3}, {NULL}};
const Segment SGI16191[] = {{&S86536[88], 5}, {NULL}};
const Segment SGI16192[] = {{&S86537[53], 10}, {NULL}};
const Segment SGI16193[] = {{&S86537[53], 21}, {NULL}};
const Segment SGI16194[] = {{&S86537[30], 15}, {NULL}};
const Segment SGI16195[] = {{&S86537[122], 3}, {NULL}};
const Segment SGI16196[] = {{&S86537[88], 5}, {NULL}};
const Segment SGI16197[] = {{&S86538[58], 10}, {NULL}};
const Segment SGI16198[] = {{&S87098[45], 10}, {NULL}};
const Segment SGI16199[] = {{&S86538[58], 21}, {NULL}};
const Segment SGI16200[] = {{&S86538[30], 20}, {NULL}};
const Segment SGI16201[] = {{&S87098[30], 6}, {NULL}};
const Segment SGI16202[] = {{&S86538[127], 3}, {NULL}};
const Segment SGI16203[] = {{&S86538[93], 5}, {NULL}};
const Segment SGI16204[] = {{&S87098[114], 3}, {NULL}};
const Segment SGI16205[] = {{&S87098[80], 5}, {NULL}};
const Segment SGI16206[] = {{&S87098[138], 13}, {NULL}};
const Segment SGI16207[] = {{&S85838[42], 11}, {NULL}};
const Segment SGI16208[] = {{&S86539[52], 10}, {NULL}};
const Segment SGI16209[] = {{&S85838[42], 22}, {NULL}};
const Segment SGI16210[] = {{&S85838[30], 4}, {NULL}};
const Segment SGI16211[] = {{&S86539[45], 6}, {&S86539[52], 21}, {NULL}};
const Segment SGI16212[] = {{&S86539[30], 14}, {NULL}};
const Segment SGI16213[] = {{&S86539[121], 3}, {NULL}};
const Segment SGI16214[] = {{&S85838[112], 3}, {NULL}};
const Segment SGI16215[] = {{&S86539[87], 5}, {NULL}};
const Segment SGI16216[] = {{&S85838[78], 5}, {NULL}};
const Segment SGI16217[] = {{&S85838[146], 3}, {NULL}};
const Segment SGI16218[] = {{&S86540[52], 10}, {NULL}};
const Segment SGI16219[] = {{&S86540[52], 21}, {NULL}};
const Segment SGI16220[] = {{&S86540[30], 14}, {NULL}};
const Segment SGI16221[] = {{&S86540[121], 3}, {NULL}};
const Segment SGI16222[] = {{&S86540[87], 5}, {NULL}};
const Segment SGI16223[] = {{&S85840[42], 10}, {NULL}};
const Segment SGI16224[] = {{&S85840[42], 21}, {NULL}};
const Segment SGI16225[] = {{&S85840[30], 4}, {NULL}};
const Segment SGI16226[] = {{&S86541[66], 10}, {NULL}};
const Segment SGI16227[] = {{&S86541[66], 21}, {NULL}};
const Segment SGI16228[] = {{&S86541[30], 28}, {NULL}};
const Segment SGI16229[] = {{&S85840[111], 3}, {NULL}};
const Segment SGI16230[] = {{&S86541[135], 3}, {NULL}};
const Segment SGI16231[] = {{&S85840[77], 5}, {NULL}};
const Segment SGI16232[] = {{&S86541[101], 5}, {NULL}};
const Segment SGI16233[] = {{&S85840[145], 3}, {NULL}};
const Segment SGI16234[] = {{&S86542[68], 10}, {NULL}};
const Segment SGI16235[] = {{&S86542[68], 21}, {NULL}};
const Segment SGI16236[] = {{&S85841[42], 11}, {NULL}};
const Segment SGI16237[] = {{&S86542[30], 30}, {NULL}};
const Segment SGI16238[] = {{&S85841[42], 22}, {NULL}};
const Segment SGI16239[] = {{&S85841[30], 4}, {NULL}};
const Segment SGI16240[] = {{&S86542[137], 3}, {NULL}};
const Segment SGI16241[] = {{&S86542[103], 5}, {NULL}};
const Segment SGI16242[] = {{&S85841[112], 3}, {NULL}};
const Segment SGI16243[] = {{&S85841[78], 5}, {NULL}};
const Segment SGI16244[] = {{&S85841[146], 3}, {NULL}};
const Segment SGI16245[] = {{&S86543[91], 10}, {NULL}};
const Segment SGI16246[] = {{&S86543[91], 21}, {NULL}};
const Segment SGI16247[] = {{&S86543[30], 53}, {NULL}};
const Segment SGI16248[] = {{&S86543[160], 3}, {NULL}};
const Segment SGI16249[] = {{&S86543[126], 5}, {NULL}};
const Segment SGI16250[] = {{&S85843[42], 11}, {NULL}};
const Segment SGI16251[] = {{&S85843[42], 22}, {NULL}};
const Segment SGI16252[] = {{&S85843[30], 4}, {NULL}};
const Segment SGI16253[] = {{&S86544[50], 10}, {NULL}};
const Segment SGI16254[] = {{&S86544[50], 21}, {NULL}};
const Segment SGI16255[] = {{&S85843[112], 3}, {NULL}};
const Segment SGI16256[] = {{&S86544[30], 12}, {NULL}};
const Segment SGI16257[] = {{&S85843[78], 5}, {NULL}};
const Segment SGI16258[] = {{&S86544[119], 3}, {NULL}};
const Segment SGI16259[] = {{&S86544[85], 5}, {NULL}};
const Segment SGI16260[] = {{&S85843[146], 3}, {NULL}};
const Segment SGI16261[] = {{&S85428[50], 10}, {NULL}};
const Segment SGI16262[] = {{&S85428[50], 22}, {NULL}};
const Segment SGI16263[] = {{&S85428[30], 12}, {NULL}};
const Segment SGI16264[] = {{&S85428[121], 3}, {NULL}};
const Segment SGI16265[] = {{&S85428[86], 6}, {NULL}};
const Segment SGI16266[] = {{&S87236[44], 10}, {NULL}};
const Segment SGI16267[] = {{&S87236[44], 21}, {NULL}};
const Segment SGI16268[] = {{&S87236[30], 6}, {NULL}};
const Segment SGI16269[] = {{&S87236[79], 6}, {NULL}};
const Segment SGI16270[] = {{&S87236[106], 13}, {NULL}};
const Segment SGI16271[] = {{&S87285[44], 9}, {NULL}};
const Segment SGI16272[] = {{&S87285[44], 20}, {NULL}};
const Segment SGI16273[] = {{&S87285[30], 6}, {NULL}};
const Segment SGI16274[] = {{&S87285[112], 3}, {NULL}};
const Segment SGI16275[] = {{&S87285[78], 5}, {NULL}};
const Segment SGI16276[] = {{&S87286[44], 9}, {NULL}};
const Segment SGI16277[] = {{&S87286[44], 20}, {NULL}};
const Segment SGI16278[] = {{&S87286[30], 6}, {NULL}};
const Segment SGI16279[] = {{&S87286[112], 3}, {NULL}};
const Segment SGI16280[] = {{&S87286[78], 5}, {NULL}};
const Segment SGI16281[] = {{&S92610[0], 107}, {NULL}};
const Segment SGI16282[] = {{&S87406[44], 9}, {NULL}};
const Segment SGI16283[] = {{&S87406[44], 20}, {NULL}};
const Segment SGI16284[] = {{&S87406[30], 6}, {NULL}};
const Segment SGI16285[] = {{&S87406[112], 3}, {NULL}};
const Segment SGI16286[] = {{&S87406[78], 5}, {NULL}};
const Segment SGI16287[] = {{&S85654[48], 7}, {NULL}};
const Segment SGI16288[] = {{&S85654[30], 12}, {NULL}};
const Segment SGI16289[] = {{&S85654[125], 3}, {NULL}};
const Segment SGI16290[] = {{&S85654[90], 6}, {NULL}};
const Segment SGI16291[] = {{&S85655[50], 7}, {NULL}};
const Segment SGI16292[] = {{&S85655[50], 28}, {NULL}};
const Segment SGI16293[] = {{&S85655[30], 12}, {NULL}};
const Segment SGI16294[] = {{&S85655[127], 3}, {NULL}};
const Segment SGI16295[] = {{&S85655[92], 6}, {NULL}};
const Segment SGI16296[] = {{&S89563[42], 16}, {NULL}};
const Segment SGI16297[] = {{&S89563[42], 27}, {NULL}};
const Segment SGI16298[] = {{&S89563[30], 4}, {NULL}};
const Segment SGI16299[] = {{&S89563[118], 3}, {NULL}};
const Segment SGI16300[] = {{&S89563[83], 6}, {NULL}};
const Segment SGI16301[] = {{&S89563[138], 8}, {NULL}};
const Segment SGI16302[] = {{&S86220[37], 7}, {NULL}};
const Segment SGI16303[] = {{&S86220[100], 3}, {NULL}};
const Segment SGI16304[] = {{&S86220[89], 3}, {NULL}};
const Segment SGI16305[] = {{&S86220[66], 6}, {NULL}};
const Segment SGI16306[] = {{&S86222[29], 7}, {NULL}};
const Segment SGI16307[] = {{&S86222[51], 3}, {NULL}};
const Segment SGI16308[] = {{&S87118[43], 8}, {&S87118[43], 8}, {NULL}};
const Segment SGI16309[] = {{&S87118[75], 12}, {NULL}};
const Segment SGI16310[] = {{&S87546[31], 8}, {NULL}};
const Segment SGI16311[] = {{&S87546[54], 3}, {NULL}};
const Segment SGI16312[] = {{&S87228[44], 8}, {&S87228[44], 8}, {NULL}};
const Segment SGI16313[] = {{&S93057[8], 6}, {NULL}};
const Segment SGI16314[] = {{&S93057[0], 44}, {NULL}};
const Segment SGI16315[] = {{&S87906[44], 14}, {NULL}};
const Segment SGI16316[] = {{&S87906[44], 25}, {NULL}};
const Segment SGI16317[] = {{&S87906[30], 6}, {NULL}};
const Segment SGI16318[] = {{&S87906[117], 3}, {NULL}};
const Segment SGI16319[] = {{&S87906[83], 5}, {NULL}};
const Segment SGI16320[] = {{&S87907[44], 14}, {NULL}};
const Segment SGI16321[] = {{&S87907[44], 25}, {NULL}};
const Segment SGI16322[] = {{&S87907[30], 6}, {NULL}};
const Segment SGI16323[] = {{&S87907[117], 3}, {NULL}};
const Segment SGI16324[] = {{&S87907[83], 5}, {NULL}};
const Segment SGI16325[] = {{&S86022[42], 41}, {NULL}};
const Segment SGI16326[] = {{&S86022[30], 4}, {NULL}};
const Segment SGI16327[] = {{&S86023[42], 36}, {NULL}};
const Segment SGI16328[] = {{&S86023[30], 4}, {NULL}};
const Segment SGI16329[] = {{&S86024[42], 6}, {NULL}};
const Segment SGI16330[] = {{&S86024[42], 17}, {NULL}};
const Segment SGI16331[] = {{&S86024[30], 4}, {NULL}};
const Segment SGI16332[] = {{&S86024[76], 4}, {NULL}};
const Segment SGI16333[] = {{&S86024[44], 3}, {NULL}};
const Segment SGI16334[] = {{&S86025[42], 6}, {NULL}};
const Segment SGI16335[] = {{&S86025[42], 17}, {NULL}};
const Segment SGI16336[] = {{&S86025[30], 4}, {NULL}};
const Segment SGI16337[] = {{&S86025[108], 3}, {NULL}};
const Segment SGI16338[] = {{&S86025[73], 6}, {NULL}};
const Segment SGI16339[] = {{&S87591[80], 6}, {NULL}};
const Segment SGI16340[] = {{&S93261[8], 6}, {NULL}};
const Segment SGI16341[] = {{&S93261[0], 44}, {NULL}};
const Segment SGI16342[] = {{&S88067[45], 10}, {NULL}};
const Segment SGI16343[] = {{&S88067[30], 6}, {NULL}};
const Segment SGI16344[] = {{&S88067[114], 3}, {NULL}};
const Segment SGI16345[] = {{&S88067[80], 5}, {NULL}};
const Segment SGI16346[] = {{&S88067[138], 13}, {NULL}};
const Segment SGI16347[] = {{&S87713[47], 8}, {NULL}};
const Segment SGI16348[] = {{&S93421[0], 108}, {NULL}};
const Segment SGI16349[] = {{&S86944[43], 10}, {NULL}};
const Segment SGI16350[] = {{&S86944[30], 4}, {NULL}};
const Segment SGI16351[] = {{&S86944[112], 3}, {NULL}};
const Segment SGI16352[] = {{&S86944[78], 5}, {NULL}};
const Segment SGI16353[] = {{&S86944[136], 13}, {NULL}};
const Segment SGI16354[] = {{&S93444[0], 107}, {NULL}};
const Segment SGI16355[] = {{&S93445[0], 107}, {NULL}};
const Segment SGI16356[] = {{&S88178[44], 15}, {NULL}};
const Segment SGI16357[] = {{&S88178[44], 26}, {NULL}};
const Segment SGI16358[] = {{&S88178[30], 6}, {NULL}};
const Segment SGI16359[] = {{&S88178[118], 3}, {NULL}};
const Segment SGI16360[] = {{&S88178[84], 5}, {NULL}};
const Segment SGI16361[] = {{&S88182[39], 8}, {NULL}};
const Segment SGI16362[] = {{&S88182[103], 3}, {NULL}};
const Segment SGI16363[] = {{&S88182[92], 3}, {NULL}};
const Segment SGI16364[] = {{&S88182[69], 6}, {NULL}};
const Segment SGI16365[] = {{&S88183[31], 8}, {NULL}};
const Segment SGI16366[] = {{&S88183[54], 3}, {NULL}};
const Segment SGI16367[] = {{&S88189[44], 9}, {NULL}};
const Segment SGI16368[] = {{&S88189[44], 20}, {NULL}};
const Segment SGI16369[] = {{&S88189[30], 6}, {NULL}};
const Segment SGI16370[] = {{&S88189[112], 3}, {NULL}};
const Segment SGI16371[] = {{&S88189[78], 5}, {NULL}};
const Segment SGI16372[] = {{&S88190[38], 9}, {NULL}};
const Segment SGI16373[] = {{&S88190[31], 6}, {NULL}};
const Segment SGI16374[] = {{&S88229[45], 9}, {NULL}};
const Segment SGI16375[] = {{&S88229[30], 6}, {NULL}};
const Segment SGI16376[] = {{&S88229[113], 3}, {NULL}};
const Segment SGI16377[] = {{&S88229[79], 5}, {NULL}};
const Segment SGI16378[] = {{&S88229[137], 13}, {NULL}};
const Segment SGI16379[] = {{&S90369[42], 8}, {NULL}};
const Segment SGI16380[] = {{&S90369[42], 19}, {NULL}};
const Segment SGI16381[] = {{&S90369[30], 4}, {NULL}};
const Segment SGI16382[] = {{&S90369[110], 3}, {NULL}};
const Segment SGI16383[] = {{&S90369[75], 6}, {NULL}};
const Segment SGI16384[] = {{&S90369[130], 8}, {NULL}};
const Segment SGI16385[] = {{&S90370[42], 8}, {NULL}};
const Segment SGI16386[] = {{&S90370[42], 19}, {NULL}};
const Segment SGI16387[] = {{&S90370[30], 4}, {NULL}};
const Segment SGI16388[] = {{&S90370[110], 3}, {NULL}};
const Segment SGI16389[] = {{&S90370[75], 6}, {NULL}};
const Segment SGI16390[] = {{&S90370[130], 8}, {NULL}};
const Segment SGI16391[] = {{&S90371[42], 8}, {NULL}};
const Segment SGI16392[] = {{&S90371[42], 19}, {NULL}};
const Segment SGI16393[] = {{&S90371[30], 4}, {NULL}};
const Segment SGI16394[] = {{&S90371[110], 3}, {NULL}};
const Segment SGI16395[] = {{&S90371[75], 6}, {NULL}};
const Segment SGI16396[] = {{&S90371[130], 8}, {NULL}};
const Segment SGI16397[] = {{&S90372[42], 8}, {NULL}};
const Segment SGI16398[] = {{&S90372[42], 19}, {NULL}};
const Segment SGI16399[] = {{&S90372[30], 4}, {NULL}};
const Segment SGI16400[] = {{&S90372[110], 3}, {NULL}};
const Segment SGI16401[] = {{&S90372[75], 6}, {NULL}};
const Segment SGI16402[] = {{&S90372[130], 8}, {NULL}};
const Segment SGI16403[] = {{&S86550[50], 14}, {NULL}};
const Segment SGI16404[] = {{&S86550[50], 25}, {NULL}};
const Segment SGI16405[] = {{&S86550[30], 12}, {NULL}};
const Segment SGI16406[] = {{&S86550[124], 3}, {NULL}};
const Segment SGI16407[] = {{&S86550[89], 6}, {NULL}};
const Segment SGI16408[] = {{&S86551[44], 14}, {NULL}};
const Segment SGI16409[] = {{&S86551[31], 12}, {NULL}};
const Segment SGI16410[] = {{&S87046[42], 10}, {NULL}};
const Segment SGI16411[] = {{&S87046[42], 22}, {NULL}};
const Segment SGI16412[] = {{&S87046[30], 4}, {NULL}};
const Segment SGI16413[] = {{&S87046[112], 3}, {NULL}};
const Segment SGI16414[] = {{&S87046[78], 5}, {NULL}};
const Segment SGI16415[] = {{&S87047[36], 10}, {NULL}};
const Segment SGI16416[] = {{&S87047[31], 4}, {NULL}};
const Segment SGI16417[] = {{&S87063[45], 4}, {NULL}};
const Segment SGI16418[] = {{&S87063[98], 6}, {NULL}};
const Segment SGI16419[] = {{&S93739[8], 6}, {NULL}};
const Segment SGI16420[] = {{&S93739[0], 44}, {NULL}};
const Segment SGI16421[] = {{&S93740[8], 7}, {NULL}};
const Segment SGI16422[] = {{&S93740[0], 45}, {NULL}};
const Segment SGI16423[] = {{&S87968[80], 6}, {NULL}};
const Segment SGI16424[] = {{&S87998[101], 5}, {NULL}};
const Segment SGI16425[] = {{&S87132[42], 10}, {NULL}};
const Segment SGI16426[] = {{&S87132[42], 21}, {NULL}};
const Segment SGI16427[] = {{&S87132[30], 4}, {NULL}};
const Segment SGI16428[] = {{&S87132[77], 6}, {NULL}};
const Segment SGI16429[] = {{&S87132[104], 13}, {NULL}};
const Segment SGI16430[] = {{&S87188[42], 6}, {NULL}};
const Segment SGI16431[] = {{&S87188[42], 17}, {NULL}};
const Segment SGI16432[] = {{&S87188[30], 4}, {NULL}};
const Segment SGI16433[] = {{&S87188[107], 3}, {NULL}};
const Segment SGI16434[] = {{&S87188[73], 5}, {NULL}};
const Segment SGI16435[] = {{&S90748[37], 5}, {NULL}};
const Segment SGI16436[] = {{&S90748[98], 3}, {NULL}};
const Segment SGI16437[] = {{&S90748[87], 3}, {NULL}};
const Segment SGI16438[] = {{&S90748[64], 6}, {NULL}};
const Segment SGI16439[] = {{&S90749[29], 5}, {NULL}};
const Segment SGI16440[] = {{&S88219[47], 8}, {NULL}};
const Segment SGI16441[] = {{&S87287[42], 9}, {NULL}};
const Segment SGI16442[] = {{&S87287[42], 20}, {NULL}};
const Segment SGI16443[] = {{&S87287[30], 4}, {NULL}};
const Segment SGI16444[] = {{&S87287[110], 3}, {NULL}};
const Segment SGI16445[] = {{&S87287[76], 5}, {NULL}};
const Segment SGI16446[] = {{&S87290[42], 9}, {NULL}};
const Segment SGI16447[] = {{&S87290[42], 20}, {NULL}};
const Segment SGI16448[] = {{&S87290[30], 4}, {NULL}};
const Segment SGI16449[] = {{&S87290[110], 3}, {NULL}};
const Segment SGI16450[] = {{&S87290[76], 5}, {NULL}};
const Segment SGI16451[] = {{&S90749[49], 3}, {NULL}};
const Segment SGI16452[] = {{&S88803[44], 14}, {NULL}};
const Segment SGI16453[] = {{&S88803[44], 26}, {NULL}};
const Segment SGI16454[] = {{&S88803[30], 6}, {NULL}};
const Segment SGI16455[] = {{&S88803[84], 6}, {NULL}};
const Segment SGI16456[] = {{&S88803[111], 13}, {NULL}};
const Segment SGI16457[] = {{&S88842[6], 4}, {NULL}};
const Segment SGI16458[] = {{&S88842[83], 5}, {NULL}};
const Segment SGI16459[] = {{&S90944[42], 8}, {NULL}};
const Segment SGI16460[] = {{&S90944[42], 19}, {NULL}};
const Segment SGI16461[] = {{&S90944[30], 4}, {NULL}};
const Segment SGI16462[] = {{&S90944[110], 3}, {NULL}};
const Segment SGI16463[] = {{&S90944[75], 6}, {NULL}};
const Segment SGI16464[] = {{&S90944[130], 8}, {NULL}};
const Segment SGI16465[] = {{&S88457[82], 6}, {NULL}};
const Segment SGI16466[] = {{&S88467[75], 6}, {NULL}};
const Segment SGI16467[] = {{&S94306[0], 107}, {NULL}};
const Segment SGI16468[] = {{&S88586[46], 10}, {NULL}};
const Segment SGI16469[] = {{&S88586[46], 21}, {NULL}};
const Segment SGI16470[] = {{&S88586[30], 8}, {NULL}};
const Segment SGI16471[] = {{&S88586[115], 3}, {NULL}};
const Segment SGI16472[] = {{&S88586[81], 5}, {NULL}};
const Segment SGI16473[] = {{&S88603[50], 10}, {NULL}};
const Segment SGI16474[] = {{&S88603[50], 32}, {NULL}};
const Segment SGI16475[] = {{&S88603[30], 12}, {NULL}};
const Segment SGI16476[] = {{&S88603[130], 3}, {NULL}};
const Segment SGI16477[] = {{&S88603[96], 5}, {NULL}};
const Segment SGI16478[] = {{&S88603[51], 8}, {NULL}};
const Segment SGI16479[] = {{&S88604[42], 10}, {NULL}};
const Segment SGI16480[] = {{&S88604[42], 22}, {NULL}};
const Segment SGI16481[] = {{&S88604[30], 4}, {NULL}};
const Segment SGI16482[] = {{&S88604[136], 3}, {NULL}};
const Segment SGI16483[] = {{&S88604[102], 5}, {NULL}};
const Segment SGI16484[] = {{&S88604[43], 8}, {NULL}};
const Segment SGI16485[] = {{&S87523[42], 14}, {NULL}};
const Segment SGI16486[] = {{&S87523[42], 25}, {NULL}};
const Segment SGI16487[] = {{&S87523[30], 4}, {NULL}};
const Segment SGI16488[] = {{&S87523[115], 3}, {NULL}};
const Segment SGI16489[] = {{&S87523[81], 5}, {NULL}};
const Segment SGI16490[] = {{&S88989[44], 13}, {NULL}};
const Segment SGI16491[] = {{&S88989[44], 25}, {NULL}};
const Segment SGI16492[] = {{&S88989[30], 6}, {NULL}};
const Segment SGI16493[] = {{&S88989[117], 3}, {NULL}};
const Segment SGI16494[] = {{&S88989[83], 5}, {NULL}};
const Segment SGI16495[] = {{&S86802[65], 6}, {NULL}};
const Segment SGI16496[] = {{&S86802[65], 24}, {NULL}};
const Segment SGI16497[] = {{&S86802[45], 12}, {NULL}};
const Segment SGI16498[] = {{&S86802[138], 3}, {NULL}};
const Segment SGI16499[] = {{&S86802[103], 6}, {NULL}};
const Segment SGI16500[] = {{&S87545[37], 8}, {NULL}};
const Segment SGI16501[] = {{&S87545[101], 3}, {NULL}};
const Segment SGI16502[] = {{&S87545[90], 3}, {NULL}};
const Segment SGI16503[] = {{&S87545[67], 6}, {NULL}};
const Segment SGI16504[] = {{&S86899[50], 9}, {NULL}};
const Segment SGI16505[] = {{&S86899[50], 20}, {NULL}};
const Segment SGI16506[] = {{&S86899[30], 12}, {NULL}};
const Segment SGI16507[] = {{&S86899[119], 3}, {NULL}};
const Segment SGI16508[] = {{&S86899[84], 6}, {NULL}};
const Segment SGI16509[] = {{&S87547[29], 8}, {NULL}};
const Segment SGI16510[] = {{&S87547[52], 3}, {NULL}};
const Segment SGI16511[] = {{&S94576[0], 107}, {NULL}};
const Segment SGI16512[] = {{&S86924[51], 6}, {NULL}};
const Segment SGI16513[] = {{&S86924[30], 12}, {NULL}};
const Segment SGI16514[] = {{&S86924[122], 3}, {NULL}};
const Segment SGI16515[] = {{&S86924[88], 5}, {NULL}};
const Segment SGI16516[] = {{&S86924[146], 13}, {NULL}};
const Segment SGI16517[] = {{&S86925[50], 6}, {NULL}};
const Segment SGI16518[] = {{&S86925[50], 23}, {NULL}};
const Segment SGI16519[] = {{&S86925[30], 12}, {NULL}};
const Segment SGI16520[] = {{&S86925[122], 3}, {NULL}};
const Segment SGI16521[] = {{&S86925[87], 6}, {NULL}};
const Segment SGI16522[] = {{&S86925[142], 8}, {NULL}};
const Segment SGI16523[] = {{&S86925[151], 13}, {NULL}};
const Segment SGI16524[] = {{&S87593[43], 12}, {NULL}};
const Segment SGI16525[] = {{&S87593[30], 4}, {NULL}};
const Segment SGI16526[] = {{&S87593[114], 3}, {NULL}};
const Segment SGI16527[] = {{&S87593[80], 5}, {NULL}};
const Segment SGI16528[] = {{&S87593[138], 13}, {NULL}};
const Segment SGI16529[] = {{&S94630[8], 7}, {NULL}};
const Segment SGI16530[] = {{&S94630[0], 85}, {NULL}};
const Segment SGI16531[] = {{&S94631[8], 16}, {NULL}};
const Segment SGI16532[] = {{&S94631[0], 94}, {NULL}};
const Segment SGI16533[] = {{&S94632[8], 8}, {NULL}};
const Segment SGI16534[] = {{&S94632[0], 46}, {NULL}};
const Segment SGI16535[] = {{&S94633[8], 8}, {NULL}};
const Segment SGI16536[] = {{&S94633[0], 86}, {NULL}};
const Segment SGI16537[] = {{&S94634[8], 7}, {NULL}};
const Segment SGI16538[] = {{&S94634[0], 85}, {NULL}};
const Segment SGI16539[] = {{&S94635[8], 7}, {NULL}};
const Segment SGI16540[] = {{&S94635[0], 45}, {NULL}};
const Segment SGI16541[] = {{&S87658[42], 24}, {NULL}};
const Segment SGI16542[] = {{&S87658[42], 35}, {NULL}};
const Segment SGI16543[] = {{&S87658[30], 4}, {NULL}};
const Segment SGI16544[] = {{&S87658[125], 3}, {NULL}};
const Segment SGI16545[] = {{&S87658[91], 5}, {NULL}};
const Segment SGI16546[] = {{&S88889[62], 8}, {NULL}};
const Segment SGI16547[] = {{&S89027[62], 8}, {NULL}};
const Segment SGI16548[] = {{&S89027[0], 13}, {NULL}};
const Segment SGI16549[] = {{&S89437[44], 4}, {NULL}};
const Segment SGI16550[] = {{&S89437[44], 15}, {NULL}};
const Segment SGI16551[] = {{&S89437[30], 6}, {NULL}};
const Segment SGI16552[] = {{&S89437[107], 3}, {NULL}};
const Segment SGI16553[] = {{&S89437[73], 5}, {NULL}};
const Segment SGI16554[] = {{&S89438[44], 4}, {NULL}};
const Segment SGI16555[] = {{&S89438[44], 15}, {NULL}};
const Segment SGI16556[] = {{&S89438[30], 6}, {NULL}};
const Segment SGI16557[] = {{&S89438[76], 4}, {NULL}};
const Segment SGI16558[] = {{&S89438[76], 3}, {NULL}};
const Segment SGI16559[] = {{&S89439[44], 4}, {NULL}};
const Segment SGI16560[] = {{&S89439[44], 15}, {NULL}};
const Segment SGI16561[] = {{&S89439[30], 6}, {NULL}};
const Segment SGI16562[] = {{&S89439[76], 6}, {NULL}};
const Segment SGI16563[] = {{&S87922[42], 9}, {NULL}};
const Segment SGI16564[] = {{&S87922[42], 21}, {NULL}};
const Segment SGI16565[] = {{&S87922[30], 4}, {NULL}};
const Segment SGI16566[] = {{&S87922[111], 3}, {NULL}};
const Segment SGI16567[] = {{&S87922[77], 5}, {NULL}};
const Segment SGI16568[] = {{&S89183[80], 6}, {NULL}};
const Segment SGI16569[] = {{&S89519[44], 10}, {NULL}};
const Segment SGI16570[] = {{&S89519[44], 21}, {NULL}};
const Segment SGI16571[] = {{&S89519[30], 6}, {NULL}};
const Segment SGI16572[] = {{&S89519[79], 6}, {NULL}};
const Segment SGI16573[] = {{&S89519[106], 13}, {NULL}};
const Segment SGI16574[] = {{&S89538[44], 10}, {NULL}};
const Segment SGI16575[] = {{&S89538[44], 21}, {NULL}};
const Segment SGI16576[] = {{&S89538[30], 6}, {NULL}};
const Segment SGI16577[] = {{&S89538[79], 6}, {NULL}};
const Segment SGI16578[] = {{&S89538[106], 13}, {NULL}};
const Segment SGI16579[] = {{&S89297[48], 11}, {NULL}};
const Segment SGI16580[] = {{&S89297[48], 36}, {NULL}};
const Segment SGI16581[] = {{&S89297[30], 10}, {NULL}};
const Segment SGI16582[] = {{&S89297[132], 3}, {NULL}};
const Segment SGI16583[] = {{&S89297[98], 5}, {NULL}};
const Segment SGI16584[] = {{&S87418[50], 27}, {NULL}};
const Segment SGI16585[] = {{&S87418[50], 38}, {NULL}};
const Segment SGI16586[] = {{&S87418[30], 12}, {NULL}};
const Segment SGI16587[] = {{&S87418[137], 3}, {NULL}};
const Segment SGI16588[] = {{&S87418[102], 6}, {NULL}};
const Segment SGI16589[] = {{&S89470[43], 8}, {&S89470[43], 8}, {NULL}};
const Segment SGI16590[] = {{&S89471[57], 8}, {NULL}};
const Segment SGI16591[] = {{&S89569[45], 14}, {NULL}};
const Segment SGI16592[] = {{&S89569[30], 7}, {NULL}};
const Segment SGI16593[] = {{&S89569[108], 3}, {NULL}};
const Segment SGI16594[] = {{&S89569[73], 6}, {NULL}};
const Segment SGI16595[] = {{&S88228[43], 9}, {NULL}};
const Segment SGI16596[] = {{&S88228[30], 4}, {NULL}};
const Segment SGI16597[] = {{&S88228[111], 3}, {NULL}};
const Segment SGI16598[] = {{&S88228[77], 5}, {NULL}};
const Segment SGI16599[] = {{&S88228[135], 13}, {NULL}};
const Segment SGI16600[] = {{&S88284[42], 101}, {NULL}};
const Segment SGI16601[] = {{&S88284[42], 112}, {NULL}};
const Segment SGI16602[] = {{&S88284[30], 4}, {NULL}};
const Segment SGI16603[] = {{&S88284[202], 3}, {NULL}};
const Segment SGI16604[] = {{&S88284[168], 5}, {NULL}};
const Segment SGI16605[] = {{&S88287[42], 10}, {NULL}};
const Segment SGI16606[] = {{&S88287[42], 21}, {NULL}};
const Segment SGI16607[] = {{&S88287[30], 4}, {NULL}};
const Segment SGI16608[] = {{&S88287[111], 3}, {NULL}};
const Segment SGI16609[] = {{&S88287[77], 5}, {NULL}};
const Segment SGI16610[] = {{&S89611[62], 8}, {&S89611[62], 8}, {NULL}};
const Segment SGI16611[] = {{&S89611[0], 13}, {NULL}};
const Segment SGI16612[] = {{&S89612[62], 8}, {NULL}};
const Segment SGI16613[] = {{&S89612[0], 13}, {NULL}};
const Segment SGI16614[] = {{&S89873[44], 7}, {NULL}};
const Segment SGI16615[] = {{&S89873[44], 19}, {NULL}};
const Segment SGI16616[] = {{&S89873[30], 6}, {NULL}};
const Segment SGI16617[] = {{&S89873[111], 3}, {NULL}};
const Segment SGI16618[] = {{&S89873[77], 5}, {NULL}};
const Segment SGI16619[] = {{&S89769[60], 8}, {&S89769[60], 8}, {NULL}};
const Segment SGI16620[] = {{&S89769[0], 13}, {NULL}};
const Segment SGI16621[] = {{&S88475[43], 10}, {NULL}};
const Segment SGI16622[] = {{&S88475[30], 4}, {NULL}};
const Segment SGI16623[] = {{&S88475[112], 3}, {NULL}};
const Segment SGI16624[] = {{&S88475[78], 5}, {NULL}};
const Segment SGI16625[] = {{&S88475[136], 13}, {NULL}};
const Segment SGI16626[] = {{&S90031[44], 9}, {NULL}};
const Segment SGI16627[] = {{&S90031[44], 20}, {NULL}};
const Segment SGI16628[] = {{&S90031[30], 6}, {NULL}};
const Segment SGI16629[] = {{&S90031[112], 3}, {NULL}};
const Segment SGI16630[] = {{&S90031[78], 5}, {NULL}};
const Segment SGI16631[] = {{&S90032[44], 9}, {NULL}};
const Segment SGI16632[] = {{&S90032[44], 21}, {NULL}};
const Segment SGI16633[] = {{&S90032[30], 6}, {NULL}};
const Segment SGI16634[] = {{&S90032[113], 3}, {NULL}};
const Segment SGI16635[] = {{&S90032[79], 5}, {NULL}};
const Segment SGI16636[] = {{&S90149[44], 10}, {NULL}};
const Segment SGI16637[] = {{&S90149[44], 21}, {NULL}};
const Segment SGI16638[] = {{&S90149[30], 6}, {NULL}};
const Segment SGI16639[] = {{&S90149[79], 6}, {NULL}};
const Segment SGI16640[] = {{&S90149[106], 13}, {NULL}};
const Segment SGI16641[] = {{&S89927[59], 8}, {&S89927[59], 8}, {NULL}};
const Segment SGI16642[] = {{&S89927[0], 13}, {NULL}};
const Segment SGI16643[] = {{&S90081[47], 8}, {NULL}};
const Segment SGI16644[] = {{&S95951[8], 6}, {NULL}};
const Segment SGI16645[] = {{&S95951[0], 47}, {NULL}};
const Segment SGI16646[] = {{&S95952[8], 7}, {NULL}};
const Segment SGI16647[] = {{&S95952[0], 45}, {NULL}};
const Segment SGI16648[] = {{&S95953[8], 6}, {NULL}};
const Segment SGI16649[] = {{&S95953[0], 57}, {NULL}};
const Segment SGI16650[] = {{&S95954[8], 6}, {NULL}};
const Segment SGI16651[] = {{&S95954[0], 44}, {NULL}};
const Segment SGI16652[] = {{&S90411[44], 10}, {NULL}};
const Segment SGI16653[] = {{&S90411[44], 21}, {NULL}};
const Segment SGI16654[] = {{&S90411[30], 6}, {NULL}};
const Segment SGI16655[] = {{&S90411[143], 4}, {NULL}};
const Segment SGI16656[] = {{&S90411[169], 3}, {NULL}};
const Segment SGI16657[] = {{&S90411[113], 3}, {&S90411[218], 5}, {NULL}};
const Segment SGI16658[] = {{&S90411[79], 5}, {NULL}};
const Segment SGI16659[] = {{&S90256[60], 8}, {&S90256[60], 8}, {NULL}};
const Segment SGI16660[] = {{&S90256[0], 13}, {NULL}};
const Segment SGI16661[] = {{&S90257[59], 8}, {&S90257[59], 8}, {&S90257[59], 8}, {NULL}};
const Segment SGI16662[] = {{&S90257[0], 13}, {NULL}};
const Segment SGI16663[] = {{&S88862[57], 10}, {NULL}};
const Segment SGI16664[] = {{&S88862[57], 21}, {NULL}};
const Segment SGI16665[] = {{&S88862[45], 4}, {NULL}};
const Segment SGI16666[] = {{&S88862[126], 3}, {NULL}};
const Segment SGI16667[] = {{&S90258[62], 8}, {&S90258[62], 8}, {NULL}};
const Segment SGI16668[] = {{&S88862[92], 5}, {NULL}};
const Segment SGI16669[] = {{&S90258[0], 13}, {NULL}};
const Segment SGI16670[] = {{&S88862[0], 14}, {NULL}};
const Segment SGI16671[] = {{&S90259[61], 8}, {NULL}};
const Segment SGI16672[] = {{&S90259[0], 13}, {NULL}};
const Segment SGI16673[] = {{&S88973[42], 11}, {NULL}};
const Segment SGI16674[] = {{&S88973[42], 21}, {NULL}};
const Segment SGI16675[] = {{&S88973[30], 4}, {NULL}};
const Segment SGI16676[] = {{&S88973[111], 3}, {NULL}};
const Segment SGI16677[] = {{&S88973[77], 5}, {NULL}};
const Segment SGI16678[] = {{&S90563[45], 10}, {NULL}};
const Segment SGI16679[] = {{&S90563[30], 6}, {NULL}};
const Segment SGI16680[] = {{&S90563[114], 3}, {NULL}};
const Segment SGI16681[] = {{&S90563[80], 5}, {NULL}};
const Segment SGI16682[] = {{&S90563[138], 13}, {NULL}};
const Segment SGI16683[] = {{&S90564[44], 10}, {NULL}};
const Segment SGI16684[] = {{&S90564[44], 21}, {NULL}};
const Segment SGI16685[] = {{&S90564[30], 6}, {NULL}};
const Segment SGI16686[] = {{&S90564[114], 3}, {NULL}};
const Segment SGI16687[] = {{&S90564[79], 6}, {NULL}};
const Segment SGI16688[] = {{&S90564[134], 8}, {NULL}};
const Segment SGI16689[] = {{&S90564[143], 13}, {NULL}};
const Segment SGI16690[] = {{&S88185[50], 14}, {NULL}};
const Segment SGI16691[] = {{&S88185[50], 25}, {NULL}};
const Segment SGI16692[] = {{&S88185[30], 12}, {NULL}};
const Segment SGI16693[] = {{&S88185[124], 3}, {NULL}};
const Segment SGI16694[] = {{&S88185[89], 6}, {NULL}};
const Segment SGI16695[] = {{&S88187[50], 7}, {NULL}};
const Segment SGI16696[] = {{&S88187[50], 21}, {NULL}};
const Segment SGI16697[] = {{&S88187[30], 12}, {NULL}};
const Segment SGI16698[] = {{&S88187[88], 4}, {NULL}};
const Segment SGI16699[] = {{&S88187[88], 3}, {NULL}};
const Segment SGI16700[] = {{&S88188[50], 7}, {NULL}};
const Segment SGI16701[] = {{&S88188[50], 21}, {NULL}};
const Segment SGI16702[] = {{&S88188[30], 12}, {NULL}};
const Segment SGI16703[] = {{&S88188[120], 3}, {NULL}};
const Segment SGI16704[] = {{&S88188[85], 6}, {NULL}};
const Segment SGI16705[] = {{&S90753[44], 7}, {NULL}};
const Segment SGI16706[] = {{&S90753[44], 18}, {NULL}};
const Segment SGI16707[] = {{&S90753[30], 6}, {NULL}};
const Segment SGI16708[] = {{&S90753[110], 3}, {NULL}};
const Segment SGI16709[] = {{&S90753[76], 5}, {NULL}};
const Segment SGI16710[] = {{&S96301[8], 7}, {NULL}};
const Segment SGI16711[] = {{&S96301[0], 58}, {NULL}};
const Segment SGI16712[] = {{&S96302[8], 7}, {NULL}};
const Segment SGI16713[] = {{&S96302[0], 45}, {NULL}};
const Segment SGI16714[] = {{&S90777[44], 11}, {NULL}};
const Segment SGI16715[] = {{&S90777[44], 23}, {NULL}};
const Segment SGI16716[] = {{&S90777[30], 6}, {NULL}};
const Segment SGI16717[] = {{&S90777[115], 3}, {NULL}};
const Segment SGI16718[] = {{&S90777[81], 5}, {NULL}};
const Segment SGI16719[] = {{&S90603[78], 6}, {NULL}};
const Segment SGI16720[] = {{&S90825[59], 10}, {NULL}};
const Segment SGI16721[] = {{&S90825[59], 21}, {NULL}};
const Segment SGI16722[] = {{&S90825[45], 6}, {NULL}};
const Segment SGI16723[] = {{&S90825[128], 3}, {NULL}};
const Segment SGI16724[] = {{&S90825[94], 5}, {NULL}};
const Segment SGI16725[] = {{&S90825[0], 14}, {NULL}};
const Segment SGI16726[] = {{&S90888[44], 14}, {NULL}};
const Segment SGI16727[] = {{&S90888[44], 26}, {NULL}};
const Segment SGI16728[] = {{&S90888[30], 6}, {NULL}};
const Segment SGI16729[] = {{&S90888[84], 6}, {NULL}};
const Segment SGI16730[] = {{&S90888[111], 13}, {NULL}};
const Segment SGI16731[] = {{&S89251[42], 15}, {NULL}};
const Segment SGI16732[] = {{&S89251[42], 26}, {NULL}};
const Segment SGI16733[] = {{&S89251[30], 4}, {NULL}};
const Segment SGI16734[] = {{&S89251[116], 3}, {NULL}};
const Segment SGI16735[] = {{&S89251[82], 5}, {NULL}};
const Segment SGI16736[] = {{&S92694[44], 7}, {NULL}};
const Segment SGI16737[] = {{&S92694[44], 19}, {NULL}};
const Segment SGI16738[] = {{&S92694[30], 6}, {NULL}};
const Segment SGI16739[] = {{&S92694[112], 3}, {NULL}};
const Segment SGI16740[] = {{&S92694[77], 6}, {NULL}};
const Segment SGI16741[] = {{&S89294[40], 10}, {NULL}};
const Segment SGI16742[] = {{&S89294[30], 4}, {NULL}};
const Segment SGI16743[] = {{&S89294[139], 4}, {NULL}};
const Segment SGI16744[] = {{&S89294[165], 3}, {NULL}};
const Segment SGI16745[] = {{&S89294[109], 3}, {&S89294[214], 5}, {NULL}};
const Segment SGI16746[] = {{&S89294[75], 5}, {NULL}};
const Segment SGI16747[] = {{&S89295[42], 10}, {NULL}};
const Segment SGI16748[] = {{&S89295[42], 21}, {NULL}};
const Segment SGI16749[] = {{&S89295[30], 4}, {NULL}};
const Segment SGI16750[] = {{&S89295[141], 4}, {NULL}};
const Segment SGI16751[] = {{&S89295[167], 3}, {NULL}};
const Segment SGI16752[] = {{&S89295[111], 3}, {&S89295[216], 5}, {NULL}};
const Segment SGI16753[] = {{&S89295[77], 5}, {NULL}};
const Segment SGI16754[] = {{&S89296[36], 10}, {NULL}};
const Segment SGI16755[] = {{&S89296[31], 4}, {NULL}};
const Segment SGI16756[] = {{&S89296[65], 4}, {NULL}};
const Segment SGI16757[] = {{&S89296[91], 3}, {NULL}};
const Segment SGI16758[] = {{&S89296[140], 5}, {NULL}};
const Segment SGI16759[] = {{&S90960[45], 10}, {NULL}};
const Segment SGI16760[] = {{&S90960[30], 6}, {NULL}};
const Segment SGI16761[] = {{&S90960[114], 3}, {NULL}};
const Segment SGI16762[] = {{&S90960[80], 5}, {NULL}};
const Segment SGI16763[] = {{&S90960[138], 13}, {NULL}};
const Segment SGI16764[] = {{&S88418[50], 16}, {NULL}};
const Segment SGI16765[] = {{&S88418[50], 27}, {NULL}};
const Segment SGI16766[] = {{&S88418[30], 12}, {NULL}};
const Segment SGI16767[] = {{&S88418[126], 3}, {NULL}};
const Segment SGI16768[] = {{&S88418[91], 6}, {NULL}};
const Segment SGI16769[] = {{&S88419[44], 16}, {NULL}};
const Segment SGI16770[] = {{&S88419[31], 12}, {NULL}};
const Segment SGI16771[] = {{&S89346[42], 10}, {NULL}};
const Segment SGI16772[] = {{&S89346[42], 21}, {NULL}};
const Segment SGI16773[] = {{&S89346[30], 4}, {NULL}};
const Segment SGI16774[] = {{&S89346[77], 6}, {NULL}};
const Segment SGI16775[] = {{&S89346[104], 13}, {NULL}};
const Segment SGI16776[] = {{&S88705[37], 12}, {NULL}};
const Segment SGI16777[] = {{&S88705[64], 3}, {NULL}};
const Segment SGI16778[] = {{&S90834[13], 8}, {NULL}};
const Segment SGI16779[] = {{&S91284[44], 7}, {NULL}};
const Segment SGI16780[] = {{&S91284[44], 18}, {NULL}};
const Segment SGI16781[] = {{&S91284[30], 6}, {NULL}};
const Segment SGI16782[] = {{&S91284[110], 3}, {NULL}};
const Segment SGI16783[] = {{&S91284[76], 5}, {NULL}};
const Segment SGI16784[] = {{&S91306[44], 4}, {NULL}};
const Segment SGI16785[] = {{&S91306[44], 16}, {NULL}};
const Segment SGI16786[] = {{&S91306[30], 6}, {NULL}};
const Segment SGI16787[] = {{&S91306[108], 3}, {NULL}};
const Segment SGI16788[] = {{&S91306[74], 5}, {NULL}};
const Segment SGI16789[] = {{&S90899[86], 5}, {NULL}};
const Segment SGI16790[] = {{&S90899[112], 13}, {NULL}};
const Segment SGI16791[] = {{&S91337[59], 10}, {NULL}};
const Segment SGI16792[] = {{&S91337[59], 21}, {NULL}};
const Segment SGI16793[] = {{&S91337[45], 6}, {NULL}};
const Segment SGI16794[] = {{&S91337[128], 3}, {NULL}};
const Segment SGI16795[] = {{&S91337[94], 5}, {NULL}};
const Segment SGI16796[] = {{&S90962[60], 9}, {NULL}};
const Segment SGI16797[] = {{&S90962[60], 20}, {NULL}};
const Segment SGI16798[] = {{&S90962[30], 22}, {NULL}};
const Segment SGI16799[] = {{&S90962[128], 3}, {NULL}};
const Segment SGI16800[] = {{&S90962[94], 5}, {NULL}};
const Segment SGI16801[] = {{&S88954[50], 6}, {NULL}};
const Segment SGI16802[] = {{&S88954[50], 17}, {NULL}};
const Segment SGI16803[] = {{&S88954[30], 12}, {NULL}};
const Segment SGI16804[] = {{&S88954[116], 3}, {NULL}};
const Segment SGI16805[] = {{&S88954[81], 6}, {NULL}};
const Segment SGI16806[] = {{&S90993[59], 8}, {&S90993[59], 8}, {NULL}};
const Segment SGI16807[] = {{&S90993[0], 13}, {NULL}};
const Segment SGI16808[] = {{&S93526[42], 11}, {NULL}};
const Segment SGI16809[] = {{&S93526[42], 23}, {NULL}};
const Segment SGI16810[] = {{&S93526[30], 4}, {NULL}};
const Segment SGI16811[] = {{&S93526[114], 3}, {NULL}};
const Segment SGI16812[] = {{&S93526[79], 6}, {NULL}};
const Segment SGI16813[] = {{&S93526[134], 8}, {NULL}};
const Segment SGI16814[] = {{&S93527[42], 12}, {NULL}};
const Segment SGI16815[] = {{&S93527[42], 23}, {NULL}};
const Segment SGI16816[] = {{&S93527[30], 4}, {NULL}};
const Segment SGI16817[] = {{&S93527[114], 3}, {NULL}};
const Segment SGI16818[] = {{&S93527[79], 6}, {NULL}};
const Segment SGI16819[] = {{&S93527[134], 8}, {NULL}};
const Segment SGI16820[] = {{&S89234[50], 8}, {NULL}};
const Segment SGI16821[] = {{&S89234[50], 19}, {NULL}};
const Segment SGI16822[] = {{&S89234[30], 12}, {NULL}};
const Segment SGI16823[] = {{&S89234[118], 3}, {NULL}};
const Segment SGI16824[] = {{&S89234[83], 6}, {NULL}};
const Segment SGI16825[] = {{&S89235[50], 7}, {NULL}};
const Segment SGI16826[] = {{&S89235[50], 18}, {NULL}};
const Segment SGI16827[] = {{&S89235[30], 12}, {NULL}};
const Segment SGI16828[] = {{&S89235[117], 3}, {NULL}};
const Segment SGI16829[] = {{&S89235[82], 6}, {NULL}};
const Segment SGI16830[] = {{&S90030[42], 9}, {NULL}};
const Segment SGI16831[] = {{&S90030[42], 20}, {NULL}};
const Segment SGI16832[] = {{&S90030[30], 4}, {NULL}};
const Segment SGI16833[] = {{&S90030[110], 3}, {NULL}};
const Segment SGI16834[] = {{&S90030[76], 5}, {NULL}};
const Segment SGI16835[] = {{&S90035[29], 8}, {NULL}};
const Segment SGI16836[] = {{&S90035[52], 3}, {NULL}};
const Segment SGI16837[] = {{&S91098[70], 10}, {NULL}};
const Segment SGI16838[] = {{&S91098[70], 21}, {NULL}};
const Segment SGI16839[] = {{&S91098[30], 32}, {NULL}};
const Segment SGI16840[] = {{&S91098[139], 3}, {NULL}};
const Segment SGI16841[] = {{&S91098[105], 5}, {NULL}};
const Segment SGI16842[] = {{&S91099[59], 10}, {NULL}};
const Segment SGI16843[] = {{&S91099[59], 21}, {NULL}};
const Segment SGI16844[] = {{&S91099[30], 21}, {NULL}};
const Segment SGI16845[] = {{&S91099[128], 3}, {NULL}};
const Segment SGI16846[] = {{&S91099[94], 5}, {NULL}};
const Segment SGI16847[] = {{&S91100[66], 10}, {NULL}};
const Segment SGI16848[] = {{&S91100[66], 21}, {NULL}};
const Segment SGI16849[] = {{&S91100[30], 28}, {NULL}};
const Segment SGI16850[] = {{&S91100[135], 3}, {NULL}};
const Segment SGI16851[] = {{&S91100[101], 5}, {NULL}};
const Segment SGI16852[] = {{&S91101[77], 10}, {NULL}};
const Segment SGI16853[] = {{&S91101[77], 21}, {NULL}};
const Segment SGI16854[] = {{&S91101[30], 39}, {NULL}};
const Segment SGI16855[] = {{&S91101[146], 3}, {NULL}};
const Segment SGI16856[] = {{&S91101[112], 5}, {NULL}};
const Segment SGI16857[] = {{&S91102[59], 10}, {NULL}};
const Segment SGI16858[] = {{&S91102[59], 21}, {NULL}};
const Segment SGI16859[] = {{&S91102[30], 21}, {NULL}};
const Segment SGI16860[] = {{&S91102[46], 3}, {NULL}};
const Segment SGI16861[] = {{&S91102[94], 5}, {NULL}};
const Segment SGI16862[] = {{&S91103[59], 10}, {NULL}};
const Segment SGI16863[] = {{&S91103[59], 21}, {NULL}};
const Segment SGI16864[] = {{&S91103[30], 21}, {NULL}};
const Segment SGI16865[] = {{&S91103[128], 3}, {NULL}};
const Segment SGI16866[] = {{&S91103[94], 5}, {NULL}};
const Segment SGI16867[] = {{&S89421[50], 10}, {NULL}};
const Segment SGI16868[] = {{&S89421[50], 21}, {NULL}};
const Segment SGI16869[] = {{&S89421[30], 12}, {NULL}};
const Segment SGI16870[] = {{&S89421[120], 3}, {NULL}};
const Segment SGI16871[] = {{&S89421[85], 6}, {NULL}};
const Segment SGI16872[] = {{&S89422[50], 10}, {NULL}};
const Segment SGI16873[] = {{&S89422[50], 21}, {NULL}};
const Segment SGI16874[] = {{&S89422[30], 12}, {NULL}};
const Segment SGI16875[] = {{&S89422[88], 4}, {NULL}};
const Segment SGI16876[] = {{&S89422[88], 3}, {NULL}};
const Segment SGI16877[] = {{&S89423[50], 10}, {NULL}};
const Segment SGI16878[] = {{&S89423[50], 21}, {NULL}};
const Segment SGI16879[] = {{&S89423[30], 12}, {NULL}};
const Segment SGI16880[] = {{&S89423[88], 6}, {NULL}};
const Segment SGI16881[] = {{&S90191[42], 12}, {NULL}};
const Segment SGI16882[] = {{&S90191[42], 24}, {NULL}};
const Segment SGI16883[] = {{&S90191[30], 4}, {NULL}};
const Segment SGI16884[] = {{&S90191[114], 3}, {NULL}};
const Segment SGI16885[] = {{&S90191[80], 5}, {NULL}};
const Segment SGI16886[] = {{&S89492[50], 6}, {NULL}};
const Segment SGI16887[] = {{&S89492[50], 23}, {NULL}};
const Segment SGI16888[] = {{&S89492[30], 12}, {NULL}};
const Segment SGI16889[] = {{&S89492[87], 6}, {NULL}};
const Segment SGI16890[] = {{&S89492[114], 13}, {NULL}};
const Segment SGI16891[] = {{&S91104[54], 7}, {NULL}};
const Segment SGI16892[] = {{&S91104[30], 23}, {NULL}};
const Segment SGI16893[] = {{&S91104[120], 3}, {NULL}};
const Segment SGI16894[] = {{&S91104[86], 5}, {NULL}};
const Segment SGI16895[] = {{&S91105[62], 10}, {NULL}};
const Segment SGI16896[] = {{&S91105[62], 21}, {NULL}};
const Segment SGI16897[] = {{&S91105[30], 24}, {NULL}};
const Segment SGI16898[] = {{&S91105[131], 3}, {NULL}};
const Segment SGI16899[] = {{&S91105[97], 5}, {NULL}};
const Segment SGI16900[] = {{&S93797[42], 13}, {NULL}};
const Segment SGI16901[] = {{&S93797[42], 24}, {NULL}};
const Segment SGI16902[] = {{&S93797[30], 4}, {NULL}};
const Segment SGI16903[] = {{&S93797[115], 3}, {NULL}};
const Segment SGI16904[] = {{&S93797[80], 6}, {NULL}};
const Segment SGI16905[] = {{&S93797[135], 8}, {NULL}};
const Segment SGI16906[] = {{&S91187[45], 8}, {NULL}};
const Segment SGI16907[] = {{&S92143[44], 10}, {NULL}};
const Segment SGI16908[] = {{&S92143[44], 21}, {NULL}};
const Segment SGI16909[] = {{&S92143[30], 6}, {NULL}};
const Segment SGI16910[] = {{&S92143[143], 4}, {NULL}};
const Segment SGI16911[] = {{&S92143[169], 3}, {NULL}};
const Segment SGI16912[] = {{&S92143[113], 3}, {&S92143[218], 5}, {NULL}};
const Segment SGI16913[] = {{&S92143[79], 5}, {NULL}};
const Segment SGI16914[] = {{&S89693[50], 7}, {NULL}};
const Segment SGI16915[] = {{&S89693[50], 18}, {NULL}};
const Segment SGI16916[] = {{&S89693[30], 12}, {NULL}};
const Segment SGI16917[] = {{&S89693[117], 3}, {NULL}};
const Segment SGI16918[] = {{&S89693[82], 6}, {NULL}};
const Segment SGI16919[] = {{&S92332[57], 10}, {NULL}};
const Segment SGI16920[] = {{&S92332[57], 21}, {NULL}};
const Segment SGI16921[] = {{&S92332[43], 6}, {NULL}};
const Segment SGI16922[] = {{&S92332[126], 3}, {NULL}};
const Segment SGI16923[] = {{&S92332[92], 5}, {NULL}};
const Segment SGI16924[] = {{&S92332[0], 12}, {NULL}};
const Segment SGI16925[] = {{&S89772[51], 10}, {NULL}};
const Segment SGI16926[] = {{&S89772[30], 12}, {NULL}};
const Segment SGI16927[] = {{&S89772[120], 3}, {NULL}};
const Segment SGI16928[] = {{&S89772[86], 5}, {NULL}};
const Segment SGI16929[] = {{&S89772[144], 13}, {NULL}};
const Segment SGI16930[] = {{&S89773[50], 10}, {NULL}};
const Segment SGI16931[] = {{&S89773[50], 21}, {NULL}};
const Segment SGI16932[] = {{&S89773[30], 12}, {NULL}};
const Segment SGI16933[] = {{&S89773[120], 3}, {NULL}};
const Segment SGI16934[] = {{&S89773[85], 6}, {NULL}};
const Segment SGI16935[] = {{&S89773[140], 8}, {NULL}};
const Segment SGI16936[] = {{&S89773[149], 13}, {NULL}};
const Segment SGI16937[] = {{&S89855[50], 11}, {NULL}};
const Segment SGI16938[] = {{&S89855[50], 23}, {NULL}};
const Segment SGI16939[] = {{&S89855[30], 12}, {NULL}};
const Segment SGI16940[] = {{&S89855[122], 3}, {NULL}};
const Segment SGI16941[] = {{&S89855[87], 6}, {NULL}};
const Segment SGI16942[] = {{&S89856[50], 11}, {NULL}};
const Segment SGI16943[] = {{&S89856[50], 24}, {NULL}};
const Segment SGI16944[] = {{&S89856[30], 12}, {NULL}};
const Segment SGI16945[] = {{&S89856[123], 3}, {NULL}};
const Segment SGI16946[] = {{&S89856[88], 6}, {NULL}};
const Segment SGI16947[] = {{&S89863[50], 9}, {NULL}};
const Segment SGI16948[] = {{&S89863[50], 20}, {NULL}};
const Segment SGI16949[] = {{&S89863[30], 12}, {NULL}};
const Segment SGI16950[] = {{&S89863[119], 3}, {NULL}};
const Segment SGI16951[] = {{&S89863[84], 6}, {NULL}};
const Segment SGI16952[] = {{&S89864[37], 8}, {NULL}};
const Segment SGI16953[] = {{&S89864[60], 3}, {NULL}};
const Segment SGI16954[] = {{&S91395[58], 10}, {NULL}};
const Segment SGI16955[] = {{&S91395[58], 35}, {NULL}};
const Segment SGI16956[] = {{&S91395[30], 20}, {NULL}};
const Segment SGI16957[] = {{&S91395[141], 3}, {NULL}};
const Segment SGI16958[] = {{&S91395[107], 5}, {NULL}};
const Segment SGI16959[] = {{&S91396[53], 10}, {NULL}};
const Segment SGI16960[] = {{&S91396[53], 35}, {NULL}};
const Segment SGI16961[] = {{&S91396[30], 15}, {NULL}};
const Segment SGI16962[] = {{&S91396[136], 3}, {NULL}};
const Segment SGI16963[] = {{&S91396[102], 5}, {NULL}};
const Segment SGI16964[] = {{&S91456[66], 10}, {NULL}};
const Segment SGI16965[] = {{&S91456[66], 21}, {NULL}};
const Segment SGI16966[] = {{&S91456[30], 28}, {NULL}};
const Segment SGI16967[] = {{&S91456[135], 3}, {NULL}};
const Segment SGI16968[] = {{&S91456[101], 5}, {NULL}};
const Segment SGI16969[] = {{&S92564[45], 10}, {NULL}};
const Segment SGI16970[] = {{&S92564[30], 6}, {NULL}};
const Segment SGI16971[] = {{&S92564[114], 3}, {NULL}};
const Segment SGI16972[] = {{&S92564[80], 5}, {NULL}};
const Segment SGI16973[] = {{&S92564[138], 13}, {NULL}};
const Segment SGI16974[] = {{&S90055[51], 14}, {NULL}};
const Segment SGI16975[] = {{&S90055[30], 12}, {NULL}};
const Segment SGI16976[] = {{&S90055[124], 3}, {NULL}};
const Segment SGI16977[] = {{&S90055[90], 5}, {NULL}};
const Segment SGI16978[] = {{&S90055[148], 13}, {NULL}};
const Segment SGI16979[] = {{&S91457[54], 10}, {NULL}};
const Segment SGI16980[] = {{&S91457[54], 21}, {NULL}};
const Segment SGI16981[] = {{&S91457[30], 16}, {NULL}};
const Segment SGI16982[] = {{&S91457[123], 3}, {NULL}};
const Segment SGI16983[] = {{&S91457[89], 5}, {NULL}};
const Segment SGI16984[] = {{&S91458[55], 10}, {NULL}};
const Segment SGI16985[] = {{&S91458[55], 21}, {NULL}};
const Segment SGI16986[] = {{&S91458[30], 17}, {NULL}};
const Segment SGI16987[] = {{&S91458[124], 3}, {NULL}};
const Segment SGI16988[] = {{&S91458[90], 5}, {NULL}};
const Segment SGI16989[] = {{&S91459[63], 10}, {NULL}};
const Segment SGI16990[] = {{&S91459[63], 21}, {NULL}};
const Segment SGI16991[] = {{&S91459[30], 25}, {NULL}};
const Segment SGI16992[] = {{&S91459[132], 3}, {NULL}};
const Segment SGI16993[] = {{&S91459[98], 5}, {NULL}};
const Segment SGI16994[] = {{&S91460[53], 10}, {NULL}};
const Segment SGI16995[] = {{&S91460[53], 21}, {NULL}};
const Segment SGI16996[] = {{&S91460[30], 15}, {NULL}};
const Segment SGI16997[] = {{&S91460[122], 3}, {NULL}};
const Segment SGI16998[] = {{&S91460[88], 5}, {NULL}};
const Segment SGI16999[] = {{&S91461[59], 10}, {NULL}};
const Segment SGI17000[] = {{&S91461[59], 21}, {NULL}};
const Segment SGI17001[] = {{&S91461[30], 21}, {NULL}};
const Segment SGI17002[] = {{&S91461[128], 3}, {NULL}};
const Segment SGI17003[] = {{&S91461[94], 5}, {NULL}};
const Segment SGI17004[] = {{&S91462[61], 10}, {NULL}};
const Segment SGI17005[] = {{&S91462[61], 21}, {NULL}};
const Segment SGI17006[] = {{&S91462[30], 23}, {NULL}};
const Segment SGI17007[] = {{&S91462[130], 3}, {NULL}};
const Segment SGI17008[] = {{&S91462[96], 5}, {NULL}};
const Segment SGI17009[] = {{&S91463[53], 10}, {NULL}};
const Segment SGI17010[] = {{&S91463[53], 21}, {NULL}};
const Segment SGI17011[] = {{&S91463[30], 15}, {NULL}};
const Segment SGI17012[] = {{&S91463[122], 3}, {NULL}};
const Segment SGI17013[] = {{&S91463[88], 5}, {NULL}};
const Segment SGI17014[] = {{&S91464[61], 10}, {NULL}};
const Segment SGI17015[] = {{&S91464[61], 21}, {NULL}};
const Segment SGI17016[] = {{&S91464[30], 23}, {NULL}};
const Segment SGI17017[] = {{&S91464[130], 3}, {NULL}};
const Segment SGI17018[] = {{&S91464[96], 5}, {NULL}};
const Segment SGI17019[] = {{&S91465[96], 10}, {NULL}};
const Segment SGI17020[] = {{&S91465[96], 21}, {NULL}};
const Segment SGI17021[] = {{&S91465[30], 58}, {NULL}};
const Segment SGI17022[] = {{&S91465[42], 3}, {NULL}};
const Segment SGI17023[] = {{&S91465[131], 5}, {NULL}};
const Segment SGI17024[] = {{&S91466[55], 10}, {NULL}};
const Segment SGI17025[] = {{&S91466[55], 21}, {NULL}};
const Segment SGI17026[] = {{&S91466[30], 17}, {NULL}};
const Segment SGI17027[] = {{&S91466[124], 3}, {NULL}};
const Segment SGI17028[] = {{&S91466[90], 5}, {NULL}};
const Segment SGI17029[] = {{&S91467[59], 10}, {NULL}};
const Segment SGI17030[] = {{&S91467[59], 21}, {NULL}};
const Segment SGI17031[] = {{&S91467[30], 21}, {NULL}};
const Segment SGI17032[] = {{&S91467[128], 3}, {NULL}};
const Segment SGI17033[] = {{&S91467[94], 5}, {NULL}};
const Segment SGI17034[] = {{&S91468[52], 10}, {NULL}};
const Segment SGI17035[] = {{&S91468[52], 21}, {NULL}};
const Segment SGI17036[] = {{&S91468[30], 14}, {NULL}};
const Segment SGI17037[] = {{&S91468[121], 3}, {NULL}};
const Segment SGI17038[] = {{&S91468[87], 5}, {NULL}};
const Segment SGI17039[] = {{&S91469[54], 10}, {NULL}};
const Segment SGI17040[] = {{&S91469[54], 21}, {NULL}};
const Segment SGI17041[] = {{&S91469[30], 16}, {NULL}};
const Segment SGI17042[] = {{&S91469[123], 3}, {NULL}};
const Segment SGI17043[] = {{&S91469[89], 5}, {NULL}};
const Segment SGI17044[] = {{&S91500[45], 15}, {NULL}};
const Segment SGI17045[] = {{&S91500[45], 27}, {NULL}};
const Segment SGI17046[] = {{&S91500[30], 7}, {NULL}};
const Segment SGI17047[] = {{&S91500[121], 3}, {NULL}};
const Segment SGI17048[] = {{&S91500[86], 6}, {NULL}};
const Segment SGI17049[] = {{&S91501[45], 15}, {NULL}};
const Segment SGI17050[] = {{&S91501[45], 27}, {NULL}};
const Segment SGI17051[] = {{&S91501[30], 7}, {NULL}};
const Segment SGI17052[] = {{&S91501[121], 3}, {NULL}};
const Segment SGI17053[] = {{&S91501[86], 6}, {NULL}};
const Segment SGI17054[] = {{&S91501[141], 8}, {NULL}};
const Segment SGI17055[] = {{&S90804[43], 10}, {NULL}};
const Segment SGI17056[] = {{&S90804[30], 4}, {NULL}};
const Segment SGI17057[] = {{&S90804[112], 3}, {NULL}};
const Segment SGI17058[] = {{&S90804[78], 5}, {NULL}};
const Segment SGI17059[] = {{&S90804[136], 13}, {NULL}};
const Segment SGI17060[] = {{&S90821[57], 10}, {NULL}};
const Segment SGI17061[] = {{&S90821[57], 21}, {NULL}};
const Segment SGI17062[] = {{&S90821[45], 4}, {NULL}};
const Segment SGI17063[] = {{&S90821[126], 3}, {NULL}};
const Segment SGI17064[] = {{&S90821[92], 5}, {NULL}};
const Segment SGI17065[] = {{&S90821[0], 14}, {NULL}};
const Segment SGI17066[] = {{&S90822[57], 11}, {NULL}};
const Segment SGI17067[] = {{&S90822[57], 22}, {NULL}};
const Segment SGI17068[] = {{&S90822[45], 4}, {NULL}};
const Segment SGI17069[] = {{&S90822[127], 3}, {NULL}};
const Segment SGI17070[] = {{&S90822[93], 5}, {NULL}};
const Segment SGI17071[] = {{&S90822[0], 14}, {NULL}};
const Segment SGI17072[] = {{&S90823[57], 11}, {NULL}};
const Segment SGI17073[] = {{&S90823[57], 22}, {NULL}};
const Segment SGI17074[] = {{&S90823[45], 4}, {NULL}};
const Segment SGI17075[] = {{&S90823[127], 3}, {NULL}};
const Segment SGI17076[] = {{&S90823[93], 5}, {NULL}};
const Segment SGI17077[] = {{&S90823[0], 14}, {NULL}};
const Segment SGI17078[] = {{&S90824[55], 10}, {NULL}};
const Segment SGI17079[] = {{&S90824[45], 4}, {NULL}};
const Segment SGI17080[] = {{&S90824[124], 3}, {NULL}};
const Segment SGI17081[] = {{&S90824[90], 5}, {NULL}};
const Segment SGI17082[] = {{&S90824[0], 14}, {NULL}};
const Segment SGI17083[] = {{&S90826[57], 11}, {NULL}};
const Segment SGI17084[] = {{&S90826[57], 22}, {NULL}};
const Segment SGI17085[] = {{&S90826[45], 4}, {NULL}};
const Segment SGI17086[] = {{&S90826[127], 3}, {NULL}};
const Segment SGI17087[] = {{&S90826[93], 5}, {NULL}};
const Segment SGI17088[] = {{&S90826[0], 14}, {NULL}};
const Segment SGI17089[] = {{&S94412[42], 16}, {NULL}};
const Segment SGI17090[] = {{&S94412[42], 27}, {NULL}};
const Segment SGI17091[] = {{&S94412[30], 4}, {NULL}};
const Segment SGI17092[] = {{&S94412[118], 3}, {NULL}};
const Segment SGI17093[] = {{&S94412[83], 6}, {NULL}};
const Segment SGI17094[] = {{&S94412[138], 8}, {NULL}};
const Segment SGI17095[] = {{&S90968[43], 12}, {NULL}};
const Segment SGI17096[] = {{&S90968[30], 4}, {NULL}};
const Segment SGI17097[] = {{&S90968[114], 3}, {NULL}};
const Segment SGI17098[] = {{&S90968[80], 5}, {NULL}};
const Segment SGI17099[] = {{&S90968[138], 13}, {NULL}};
const Segment SGI17100[] = {{&S98043[8], 7}, {NULL}};
const Segment SGI17101[] = {{&S98043[0], 45}, {NULL}};
const Segment SGI17102[] = {{&S98044[8], 6}, {NULL}};
const Segment SGI17103[] = {{&S98044[0], 44}, {NULL}};
const Segment SGI17104[] = {{&S98168[0], 124}, {NULL}};
const Segment SGI17105[] = {{&S94655[42], 12}, {NULL}};
const Segment SGI17106[] = {{&S94655[42], 23}, {NULL}};
const Segment SGI17107[] = {{&S94655[30], 4}, {NULL}};
const Segment SGI17108[] = {{&S94655[114], 3}, {NULL}};
const Segment SGI17109[] = {{&S94655[79], 6}, {NULL}};
const Segment SGI17110[] = {{&S94655[134], 8}, {NULL}};
const Segment SGI17111[] = {{&S90549[50], 27}, {NULL}};
const Segment SGI17112[] = {{&S90549[30], 12}, {NULL}};
const Segment SGI17113[] = {{&S90549[144], 3}, {NULL}};
const Segment SGI17114[] = {{&S90549[109], 6}, {NULL}};
const Segment SGI17115[] = {{&S90550[50], 27}, {NULL}};
const Segment SGI17116[] = {{&S90550[30], 12}, {NULL}};
const Segment SGI17117[] = {{&S90550[144], 3}, {NULL}};
const Segment SGI17118[] = {{&S90550[109], 6}, {NULL}};
const Segment SGI17119[] = {{&S93081[45], 10}, {NULL}};
const Segment SGI17120[] = {{&S93081[30], 6}, {NULL}};
const Segment SGI17121[] = {{&S90551[50], 27}, {NULL}};
const Segment SGI17122[] = {{&S90551[30], 12}, {NULL}};
const Segment SGI17123[] = {{&S93081[114], 3}, {NULL}};
const Segment SGI17124[] = {{&S93081[80], 5}, {NULL}};
const Segment SGI17125[] = {{&S90551[144], 3}, {NULL}};
const Segment SGI17126[] = {{&S93081[138], 13}, {NULL}};
const Segment SGI17127[] = {{&S90551[109], 6}, {NULL}};
const Segment SGI17128[] = {{&S98268[0], 106}, {NULL}};
const Segment SGI17129[] = {{&S94694[44], 9}, {NULL}};
const Segment SGI17130[] = {{&S94694[44], 21}, {NULL}};
const Segment SGI17131[] = {{&S94694[30], 6}, {NULL}};
const Segment SGI17132[] = {{&S94694[114], 3}, {NULL}};
const Segment SGI17133[] = {{&S94694[79], 6}, {NULL}};
const Segment SGI17134[] = {{&S91305[42], 12}, {NULL}};
const Segment SGI17135[] = {{&S91305[42], 23}, {NULL}};
const Segment SGI17136[] = {{&S91305[30], 4}, {NULL}};
const Segment SGI17137[] = {{&S91305[113], 3}, {NULL}};
const Segment SGI17138[] = {{&S91305[79], 5}, {NULL}};
const Segment SGI17139[] = {{&S91508[42], 9}, {NULL}};
const Segment SGI17140[] = {{&S91508[42], 21}, {NULL}};
const Segment SGI17141[] = {{&S91508[30], 4}, {NULL}};
const Segment SGI17142[] = {{&S91508[111], 3}, {NULL}};
const Segment SGI17143[] = {{&S91508[77], 5}, {NULL}};
const Segment SGI17144[] = {{&S92298[6], 4}, {NULL}};
const Segment SGI17145[] = {{&S92298[72], 5}, {NULL}};
const Segment SGI17146[] = {{&S90805[51], 10}, {NULL}};
const Segment SGI17147[] = {{&S90805[30], 12}, {NULL}};
const Segment SGI17148[] = {{&S90805[120], 3}, {NULL}};
const Segment SGI17149[] = {{&S90805[86], 5}, {NULL}};
const Segment SGI17150[] = {{&S90805[144], 13}, {NULL}};
const Segment SGI17151[] = {{&S92360[43], 8}, {&S92360[43], 8}, {NULL}};
const Segment SGI17152[] = {{&S92361[64], 8}, {NULL}};
const Segment SGI17153[] = {{&S93528[44], 18}, {NULL}};
const Segment SGI17154[] = {{&S93528[44], 29}, {NULL}};
const Segment SGI17155[] = {{&S93528[30], 6}, {NULL}};
const Segment SGI17156[] = {{&S93528[121], 3}, {NULL}};
const Segment SGI17157[] = {{&S93528[87], 5}, {NULL}};
const Segment SGI17158[] = {{&S92666[95], 3}, {NULL}};
const Segment SGI17159[] = {{&S92666[81], 5}, {NULL}};
const Segment SGI17160[] = {{&S91097[50], 4}, {NULL}};
const Segment SGI17161[] = {{&S91097[50], 15}, {NULL}};
const Segment SGI17162[] = {{&S91097[30], 12}, {NULL}};
const Segment SGI17163[] = {{&S91097[114], 3}, {NULL}};
const Segment SGI17164[] = {{&S91097[79], 6}, {NULL}};
const Segment SGI17165[] = {{&S95143[42], 10}, {NULL}};
const Segment SGI17166[] = {{&S95143[42], 23}, {NULL}};
const Segment SGI17167[] = {{&S95143[30], 4}, {NULL}};
const Segment SGI17168[] = {{&S95143[114], 3}, {NULL}};
const Segment SGI17169[] = {{&S95143[79], 6}, {NULL}};
const Segment SGI17170[] = {{&S95143[134], 8}, {NULL}};
const Segment SGI17171[] = {{&S95193[44], 11}, {NULL}};
const Segment SGI17172[] = {{&S95193[44], 23}, {NULL}};
const Segment SGI17173[] = {{&S95193[30], 6}, {NULL}};
const Segment SGI17174[] = {{&S95193[116], 3}, {NULL}};
const Segment SGI17175[] = {{&S95193[81], 6}, {NULL}};
const Segment SGI17176[] = {{&S91157[51], 6}, {NULL}};
const Segment SGI17177[] = {{&S91157[30], 12}, {NULL}};
const Segment SGI17178[] = {{&S91157[122], 3}, {NULL}};
const Segment SGI17179[] = {{&S91157[88], 5}, {NULL}};
const Segment SGI17180[] = {{&S91157[146], 13}, {NULL}};
const Segment SGI17181[] = {{&S92919[13], 8}, {NULL}};
const Segment SGI17182[] = {{&S93844[59], 10}, {NULL}};
const Segment SGI17183[] = {{&S93844[59], 21}, {NULL}};
const Segment SGI17184[] = {{&S93844[45], 6}, {NULL}};
const Segment SGI17185[] = {{&S93844[128], 3}, {NULL}};
const Segment SGI17186[] = {{&S93844[94], 5}, {NULL}};
const Segment SGI17187[] = {{&S93844[0], 14}, {NULL}};
const Segment SGI17188[] = {{&S93846[71], 10}, {NULL}};
const Segment SGI17189[] = {{&S93846[71], 21}, {NULL}};
const Segment SGI17190[] = {{&S93846[45], 18}, {NULL}};
const Segment SGI17191[] = {{&S93846[140], 3}, {NULL}};
const Segment SGI17192[] = {{&S93846[106], 5}, {NULL}};
const Segment SGI17193[] = {{&S93846[0], 14}, {NULL}};
const Segment SGI17194[] = {{&S93847[55], 10}, {NULL}};
const Segment SGI17195[] = {{&S93847[55], 27}, {NULL}};
const Segment SGI17196[] = {{&S93847[0], 14}, {NULL}};
const Segment SGI17197[] = {{&S92237[42], 12}, {NULL}};
const Segment SGI17198[] = {{&S92237[42], 23}, {NULL}};
const Segment SGI17199[] = {{&S92237[30], 4}, {NULL}};
const Segment SGI17200[] = {{&S92237[79], 6}, {NULL}};
const Segment SGI17201[] = {{&S92237[106], 13}, {NULL}};
const Segment SGI17202[] = {{&S93947[44], 10}, {NULL}};
const Segment SGI17203[] = {{&S93947[44], 21}, {NULL}};
const Segment SGI17204[] = {{&S93947[30], 6}, {NULL}};
const Segment SGI17205[] = {{&S93947[114], 3}, {NULL}};
const Segment SGI17206[] = {{&S93947[79], 6}, {NULL}};
const Segment SGI17207[] = {{&S93947[134], 8}, {NULL}};
const Segment SGI17208[] = {{&S93947[143], 13}, {NULL}};
const Segment SGI17209[] = {{&S91219[50], 6}, {NULL}};
const Segment SGI17210[] = {{&S91219[50], 23}, {NULL}};
const Segment SGI17211[] = {{&S91219[30], 12}, {NULL}};
const Segment SGI17212[] = {{&S91219[87], 6}, {NULL}};
const Segment SGI17213[] = {{&S91219[114], 13}, {NULL}};
const Segment SGI17214[] = {{&S92330[42], 14}, {NULL}};
const Segment SGI17215[] = {{&S92330[42], 25}, {NULL}};
const Segment SGI17216[] = {{&S92330[30], 4}, {NULL}};
const Segment SGI17217[] = {{&S93128[45], 8}, {NULL}};
const Segment SGI17218[] = {{&S92330[115], 3}, {NULL}};
const Segment SGI17219[] = {{&S92330[81], 5}, {NULL}};
const Segment SGI17220[] = {{&S93129[62], 8}, {NULL}};
const Segment SGI17221[] = {{&S93129[0], 13}, {NULL}};
const Segment SGI17222[] = {{&S93130[60], 8}, {&S93130[60], 8}, {NULL}};
const Segment SGI17223[] = {{&S93130[0], 13}, {NULL}};
const Segment SGI17224[] = {{&S93986[45], 14}, {NULL}};
const Segment SGI17225[] = {{&S93986[30], 6}, {NULL}};
const Segment SGI17226[] = {{&S93986[119], 3}, {NULL}};
const Segment SGI17227[] = {{&S93986[85], 5}, {NULL}};
const Segment SGI17228[] = {{&S93986[143], 13}, {NULL}};
const Segment SGI17229[] = {{&S93987[44], 14}, {NULL}};
const Segment SGI17230[] = {{&S93987[44], 26}, {NULL}};
const Segment SGI17231[] = {{&S93987[30], 6}, {NULL}};
const Segment SGI17232[] = {{&S93987[119], 3}, {NULL}};
const Segment SGI17233[] = {{&S93987[84], 6}, {NULL}};
const Segment SGI17234[] = {{&S93987[139], 8}, {NULL}};
const Segment SGI17235[] = {{&S93987[148], 13}, {NULL}};
const Segment SGI17236[] = {{&S99492[0], 107}, {NULL}};
const Segment SGI17237[] = {{&S91307[50], 20}, {NULL}};
const Segment SGI17238[] = {{&S91307[50], 31}, {NULL}};
const Segment SGI17239[] = {{&S91307[30], 12}, {NULL}};
const Segment SGI17240[] = {{&S91307[130], 3}, {NULL}};
const Segment SGI17241[] = {{&S91307[95], 6}, {NULL}};
const Segment SGI17242[] = {{&S91308[50], 20}, {NULL}};
const Segment SGI17243[] = {{&S91308[50], 31}, {NULL}};
const Segment SGI17244[] = {{&S91308[30], 12}, {NULL}};
const Segment SGI17245[] = {{&S91308[130], 3}, {NULL}};
const Segment SGI17246[] = {{&S91308[95], 6}, {NULL}};
const Segment SGI17247[] = {{&S95763[42], 10}, {NULL}};
const Segment SGI17248[] = {{&S95763[42], 23}, {NULL}};
const Segment SGI17249[] = {{&S95763[30], 4}, {NULL}};
const Segment SGI17250[] = {{&S95763[114], 3}, {NULL}};
const Segment SGI17251[] = {{&S95763[79], 6}, {NULL}};
const Segment SGI17252[] = {{&S95763[134], 8}, {NULL}};
const Segment SGI17253[] = {{&S92507[43], 10}, {NULL}};
const Segment SGI17254[] = {{&S92507[30], 4}, {NULL}};
const Segment SGI17255[] = {{&S92507[112], 3}, {NULL}};
const Segment SGI17256[] = {{&S92507[78], 5}, {NULL}};
const Segment SGI17257[] = {{&S92507[136], 13}, {NULL}};
const Segment SGI17258[] = {{&S94146[44], 10}, {NULL}};
const Segment SGI17259[] = {{&S94146[44], 21}, {NULL}};
const Segment SGI17260[] = {{&S94146[30], 6}, {NULL}};
const Segment SGI17261[] = {{&S94146[79], 6}, {NULL}};
const Segment SGI17262[] = {{&S94146[106], 13}, {NULL}};
const Segment SGI17263[] = {{&S94162[52], 10}, {NULL}};
const Segment SGI17264[] = {{&S94162[52], 21}, {NULL}};
const Segment SGI17265[] = {{&S94162[38], 6}, {NULL}};
const Segment SGI17266[] = {{&S94162[87], 5}, {NULL}};
const Segment SGI17267[] = {{&S93358[53], 8}, {&S93358[53], 8}, {NULL}};
const Segment SGI17268[] = {{&S93358[82], 9}, {NULL}};
const Segment SGI17269[] = {{&S93358[86], 6}, {NULL}};
const Segment SGI17270[] = {{&S93359[60], 8}, {&S93359[60], 8}, {NULL}};
const Segment SGI17271[] = {{&S93359[0], 13}, {NULL}};
const Segment SGI17272[] = {{&S91516[51], 6}, {NULL}};
const Segment SGI17273[] = {{&S91516[30], 12}, {NULL}};
const Segment SGI17274[] = {{&S91516[122], 3}, {NULL}};
const Segment SGI17275[] = {{&S91516[88], 5}, {NULL}};
const Segment SGI17276[] = {{&S91516[146], 13}, {NULL}};
const Segment SGI17277[] = {{&S94207[44], 13}, {NULL}};
const Segment SGI17278[] = {{&S94207[44], 24}, {NULL}};
const Segment SGI17279[] = {{&S94207[30], 6}, {NULL}};
const Segment SGI17280[] = {{&S94207[116], 3}, {NULL}};
const Segment SGI17281[] = {{&S94207[82], 5}, {NULL}};
const Segment SGI17282[] = {{&S91593[50], 14}, {NULL}};
const Segment SGI17283[] = {{&S91593[50], 25}, {NULL}};
const Segment SGI17284[] = {{&S91593[30], 12}, {NULL}};
const Segment SGI17285[] = {{&S91593[89], 6}, {NULL}};
const Segment SGI17286[] = {{&S91593[116], 13}, {NULL}};
const Segment SGI17287[] = {{&S91601[50], 6}, {NULL}};
const Segment SGI17288[] = {{&S91601[50], 23}, {NULL}};
const Segment SGI17289[] = {{&S91601[30], 12}, {NULL}};
const Segment SGI17290[] = {{&S91601[87], 6}, {NULL}};
const Segment SGI17291[] = {{&S91601[114], 13}, {NULL}};
const Segment SGI17292[] = {{&S94244[59], 14}, {NULL}};
const Segment SGI17293[] = {{&S94244[59], 26}, {NULL}};
const Segment SGI17294[] = {{&S94244[45], 6}, {NULL}};
const Segment SGI17295[] = {{&S94244[133], 3}, {NULL}};
const Segment SGI17296[] = {{&S94244[99], 5}, {NULL}};
const Segment SGI17297[] = {{&S94312[44], 18}, {NULL}};
const Segment SGI17298[] = {{&S94312[44], 29}, {NULL}};
const Segment SGI17299[] = {{&S94312[30], 6}, {NULL}};
const Segment SGI17300[] = {{&S94312[121], 3}, {NULL}};
const Segment SGI17301[] = {{&S94312[87], 5}, {NULL}};
const Segment SGI17302[] = {{&S94312[144], 10}, {NULL}};
const Segment SGI17303[] = {{&S91703[51], 6}, {NULL}};
const Segment SGI17304[] = {{&S91703[30], 12}, {NULL}};
const Segment SGI17305[] = {{&S91703[123], 3}, {NULL}};
const Segment SGI17306[] = {{&S91703[89], 5}, {NULL}};
const Segment SGI17307[] = {{&S91703[147], 13}, {NULL}};
const Segment SGI17308[] = {{&S91704[50], 6}, {NULL}};
const Segment SGI17309[] = {{&S91704[50], 24}, {NULL}};
const Segment SGI17310[] = {{&S91704[30], 12}, {NULL}};
const Segment SGI17311[] = {{&S91704[123], 3}, {NULL}};
const Segment SGI17312[] = {{&S91704[88], 6}, {NULL}};
const Segment SGI17313[] = {{&S91704[143], 8}, {NULL}};
const Segment SGI17314[] = {{&S91704[152], 13}, {NULL}};
const Segment SGI17315[] = {{&S91757[50], 6}, {NULL}};
const Segment SGI17316[] = {{&S91757[50], 23}, {NULL}};
const Segment SGI17317[] = {{&S100015[0], 108}, {NULL}};
const Segment SGI17318[] = {{&S91757[30], 12}, {NULL}};
const Segment SGI17319[] = {{&S91757[165], 3}, {NULL}};
const Segment SGI17320[] = {{&S91757[131], 3}, {NULL}};
const Segment SGI17321[] = {{&S91757[96], 6}, {NULL}};
const Segment SGI17322[] = {{&S91757[151], 8}, {NULL}};
const Segment SGI17323[] = {{&S95983[44], 11}, {NULL}};
const Segment SGI17324[] = {{&S95983[44], 23}, {NULL}};
const Segment SGI17325[] = {{&S95983[30], 6}, {NULL}};
const Segment SGI17326[] = {{&S95983[116], 3}, {NULL}};
const Segment SGI17327[] = {{&S95983[81], 6}, {NULL}};
const Segment SGI17328[] = {{&S94419[44], 6}, {NULL}};
const Segment SGI17329[] = {{&S94419[44], 25}, {NULL}};
const Segment SGI17330[] = {{&S94419[30], 6}, {NULL}};
const Segment SGI17331[] = {{&S94419[117], 3}, {NULL}};
const Segment SGI17332[] = {{&S94419[83], 5}, {NULL}};
const Segment SGI17333[] = {{&S94420[44], 6}, {NULL}};
const Segment SGI17334[] = {{&S94420[44], 29}, {NULL}};
const Segment SGI17335[] = {{&S94420[30], 6}, {NULL}};
const Segment SGI17336[] = {{&S94420[121], 3}, {NULL}};
const Segment SGI17337[] = {{&S94420[87], 5}, {NULL}};
const Segment SGI17338[] = {{&S94424[44], 7}, {NULL}};
const Segment SGI17339[] = {{&S94424[44], 27}, {NULL}};
const Segment SGI17340[] = {{&S94424[30], 6}, {NULL}};
const Segment SGI17341[] = {{&S94424[119], 3}, {NULL}};
const Segment SGI17342[] = {{&S94424[85], 5}, {NULL}};
const Segment SGI17343[] = {{&S94425[44], 7}, {NULL}};
const Segment SGI17344[] = {{&S94425[44], 27}, {NULL}};
const Segment SGI17345[] = {{&S94425[30], 6}, {NULL}};
const Segment SGI17346[] = {{&S94425[88], 4}, {NULL}};
const Segment SGI17347[] = {{&S94425[88], 3}, {NULL}};
const Segment SGI17348[] = {{&S94441[44], 14}, {NULL}};
const Segment SGI17349[] = {{&S94441[44], 26}, {NULL}};
const Segment SGI17350[] = {{&S94441[30], 6}, {NULL}};
const Segment SGI17351[] = {{&S94441[118], 3}, {NULL}};
const Segment SGI17352[] = {{&S94441[84], 5}, {NULL}};
const Segment SGI17353[] = {{&S94467[59], 10}, {NULL}};
const Segment SGI17354[] = {{&S94467[59], 21}, {NULL}};
const Segment SGI17355[] = {{&S94467[45], 6}, {NULL}};
const Segment SGI17356[] = {{&S94467[128], 3}, {NULL}};
const Segment SGI17357[] = {{&S94467[94], 5}, {NULL}};
const Segment SGI17358[] = {{&S91849[50], 6}, {NULL}};
const Segment SGI17359[] = {{&S91849[50], 23}, {NULL}};
const Segment SGI17360[] = {{&S91849[30], 12}, {NULL}};
const Segment SGI17361[] = {{&S91849[87], 6}, {NULL}};
const Segment SGI17362[] = {{&S91849[114], 13}, {NULL}};
const Segment SGI17363[] = {{&S100118[8], 8}, {NULL}};
const Segment SGI17364[] = {{&S100118[0], 86}, {NULL}};
const Segment SGI17365[] = {{&S100119[8], 7}, {NULL}};
const Segment SGI17366[] = {{&S100119[0], 85}, {NULL}};
const Segment SGI17367[] = {{&S93080[43], 10}, {NULL}};
const Segment SGI17368[] = {{&S93080[30], 4}, {NULL}};
const Segment SGI17369[] = {{&S93080[112], 3}, {NULL}};
const Segment SGI17370[] = {{&S93080[78], 5}, {NULL}};
const Segment SGI17371[] = {{&S93080[136], 13}, {NULL}};
const Segment SGI17372[] = {{&S93082[42], 10}, {NULL}};
const Segment SGI17373[] = {{&S93082[42], 21}, {NULL}};
const Segment SGI17374[] = {{&S93082[30], 4}, {NULL}};
const Segment SGI17375[] = {{&S93082[112], 3}, {NULL}};
const Segment SGI17376[] = {{&S93082[77], 6}, {NULL}};
const Segment SGI17377[] = {{&S93082[132], 8}, {NULL}};
const Segment SGI17378[] = {{&S93082[141], 13}, {NULL}};
const Segment SGI17379[] = {{&S94719[59], 10}, {NULL}};
const Segment SGI17380[] = {{&S94719[59], 21}, {NULL}};
const Segment SGI17381[] = {{&S94719[45], 6}, {NULL}};
const Segment SGI17382[] = {{&S94719[128], 3}, {NULL}};
const Segment SGI17383[] = {{&S94719[94], 5}, {NULL}};
const Segment SGI17384[] = {{&S100420[0], 106}, {NULL}};
const Segment SGI17385[] = {{&S94061[48], 17}, {NULL}};
const Segment SGI17386[] = {{&S94061[30], 10}, {NULL}};
const Segment SGI17387[] = {{&S94061[37], 3}, {NULL}};
const Segment SGI17388[] = {{&S94061[113], 5}, {NULL}};
const Segment SGI17389[] = {{&S94068[61], 8}, {NULL}};
const Segment SGI17390[] = {{&S94068[0], 13}, {NULL}};
const Segment SGI17391[] = {{&S92277[50], 7}, {NULL}};
const Segment SGI17392[] = {{&S92277[50], 18}, {NULL}};
const Segment SGI17393[] = {{&S92277[30], 12}, {NULL}};
const Segment SGI17394[] = {{&S92277[117], 3}, {NULL}};
const Segment SGI17395[] = {{&S92277[82], 6}, {NULL}};
const Segment SGI17396[] = {{&S93443[42], 12}, {NULL}};
const Segment SGI17397[] = {{&S93443[42], 23}, {NULL}};
const Segment SGI17398[] = {{&S93443[30], 4}, {NULL}};
const Segment SGI17399[] = {{&S93443[79], 6}, {NULL}};
const Segment SGI17400[] = {{&S93443[106], 13}, {NULL}};
const Segment SGI17401[] = {{&S93453[42], 9}, {NULL}};
const Segment SGI17402[] = {{&S93453[42], 20}, {NULL}};
const Segment SGI17403[] = {{&S93453[30], 4}, {NULL}};
const Segment SGI17404[] = {{&S93453[76], 6}, {NULL}};
const Segment SGI17405[] = {{&S93453[103], 13}, {NULL}};
const Segment SGI17406[] = {{&S94924[45], 10}, {NULL}};
const Segment SGI17407[] = {{&S94924[30], 6}, {NULL}};
const Segment SGI17408[] = {{&S94924[114], 3}, {NULL}};
const Segment SGI17409[] = {{&S94924[80], 5}, {NULL}};
const Segment SGI17410[] = {{&S94924[138], 13}, {NULL}};
const Segment SGI17411[] = {{&S94186[50], 10}, {NULL}};
const Segment SGI17412[] = {{&S94186[50], 22}, {NULL}};
const Segment SGI17413[] = {{&S94186[30], 12}, {NULL}};
const Segment SGI17414[] = {{&S94186[121], 3}, {NULL}};
const Segment SGI17415[] = {{&S94186[86], 6}, {NULL}};
const Segment SGI17416[] = {{&S94187[72], 10}, {NULL}};
const Segment SGI17417[] = {{&S94187[72], 22}, {NULL}};
const Segment SGI17418[] = {{&S94187[30], 34}, {NULL}};
const Segment SGI17419[] = {{&S94187[143], 3}, {NULL}};
const Segment SGI17420[] = {{&S94187[108], 6}, {NULL}};
const Segment SGI17421[] = {{&S94188[62], 10}, {NULL}};
const Segment SGI17422[] = {{&S94188[62], 22}, {NULL}};
const Segment SGI17423[] = {{&S94188[30], 24}, {NULL}};
const Segment SGI17424[] = {{&S94188[133], 3}, {NULL}};
const Segment SGI17425[] = {{&S94188[98], 6}, {NULL}};
const Segment SGI17426[] = {{&S94189[57], 10}, {NULL}};
const Segment SGI17427[] = {{&S94189[57], 22}, {NULL}};
const Segment SGI17428[] = {{&S94189[30], 19}, {NULL}};
const Segment SGI17429[] = {{&S94189[128], 3}, {NULL}};
const Segment SGI17430[] = {{&S94189[93], 6}, {NULL}};
const Segment SGI17431[] = {{&S94190[72], 10}, {NULL}};
const Segment SGI17432[] = {{&S94190[72], 22}, {NULL}};
const Segment SGI17433[] = {{&S94190[30], 34}, {NULL}};
const Segment SGI17434[] = {{&S94190[143], 3}, {NULL}};
const Segment SGI17435[] = {{&S94190[108], 6}, {NULL}};
const Segment SGI17436[] = {{&S94191[56], 10}, {NULL}};
const Segment SGI17437[] = {{&S94191[56], 22}, {NULL}};
const Segment SGI17438[] = {{&S94191[30], 18}, {NULL}};
const Segment SGI17439[] = {{&S94191[127], 3}, {NULL}};
const Segment SGI17440[] = {{&S94191[92], 6}, {NULL}};
const Segment SGI17441[] = {{&S94192[71], 10}, {NULL}};
const Segment SGI17442[] = {{&S94192[71], 22}, {NULL}};
const Segment SGI17443[] = {{&S94192[30], 33}, {NULL}};
const Segment SGI17444[] = {{&S94192[142], 3}, {NULL}};
const Segment SGI17445[] = {{&S94192[107], 6}, {NULL}};
const Segment SGI17446[] = {{&S94193[72], 10}, {NULL}};
const Segment SGI17447[] = {{&S94193[72], 22}, {NULL}};
const Segment SGI17448[] = {{&S94193[30], 34}, {NULL}};
const Segment SGI17449[] = {{&S94193[143], 3}, {NULL}};
const Segment SGI17450[] = {{&S94193[108], 6}, {NULL}};
const Segment SGI17451[] = {{&S100747[8], 7}, {NULL}};
const Segment SGI17452[] = {{&S100747[0], 45}, {NULL}};
const Segment SGI17453[] = {{&S93609[43], 9}, {NULL}};
const Segment SGI17454[] = {{&S93609[30], 4}, {NULL}};
const Segment SGI17455[] = {{&S93609[111], 3}, {NULL}};
const Segment SGI17456[] = {{&S93609[77], 5}, {NULL}};
const Segment SGI17457[] = {{&S93609[135], 13}, {NULL}};
const Segment SGI17458[] = {{&S100868[0], 42}, {NULL}};
const Segment SGI17459[] = {{&S92544[51], 6}, {NULL}};
const Segment SGI17460[] = {{&S92544[30], 12}, {NULL}};
const Segment SGI17461[] = {{&S92544[122], 3}, {NULL}};
const Segment SGI17462[] = {{&S92544[88], 5}, {NULL}};
const Segment SGI17463[] = {{&S92544[146], 13}, {NULL}};
const Segment SGI17464[] = {{&S94249[8], 8}, {NULL}};
const Segment SGI17465[] = {{&S95196[44], 10}, {NULL}};
const Segment SGI17466[] = {{&S95196[44], 21}, {NULL}};
const Segment SGI17467[] = {{&S95196[30], 6}, {NULL}};
const Segment SGI17468[] = {{&S93747[42], 36}, {NULL}};
const Segment SGI17469[] = {{&S93747[30], 4}, {NULL}};
const Segment SGI17470[] = {{&S95196[113], 3}, {NULL}};
const Segment SGI17471[] = {{&S95196[79], 5}, {NULL}};
const Segment SGI17472[] = {{&S95196[137], 7}, {NULL}};
const Segment SGI17473[] = {{&S93747[144], 3}, {NULL}};
const Segment SGI17474[] = {{&S93747[110], 5}, {NULL}};
const Segment SGI17475[] = {{&S93748[42], 13}, {NULL}};
const Segment SGI17476[] = {{&S93748[42], 24}, {NULL}};
const Segment SGI17477[] = {{&S93748[30], 4}, {NULL}};
const Segment SGI17478[] = {{&S93748[83], 4}, {NULL}};
const Segment SGI17479[] = {{&S93748[83], 3}, {NULL}};
const Segment SGI17480[] = {{&S95221[44], 10}, {NULL}};
const Segment SGI17481[] = {{&S95221[44], 21}, {NULL}};
const Segment SGI17482[] = {{&S95221[30], 6}, {NULL}};
const Segment SGI17483[] = {{&S95221[158], 3}, {NULL}};
const Segment SGI17484[] = {{&S95221[124], 3}, {NULL}};
const Segment SGI17485[] = {{&S95221[89], 6}, {NULL}};
const Segment SGI17486[] = {{&S95221[144], 8}, {NULL}};
const Segment SGI17487[] = {{&S95246[44], 18}, {NULL}};
const Segment SGI17488[] = {{&S95246[44], 29}, {NULL}};
const Segment SGI17489[] = {{&S95246[30], 6}, {NULL}};
const Segment SGI17490[] = {{&S95246[121], 3}, {NULL}};
const Segment SGI17491[] = {{&S95246[87], 5}, {NULL}};
const Segment SGI17492[] = {{&S95246[145], 7}, {NULL}};
const Segment SGI17493[] = {{&S95246[145], 11}, {NULL}};
const Segment SGI17494[] = {{&S94417[11], 1}, {NULL}};
const Segment SGI17495[] = {{&S94417[48], 11}, {NULL}};
const Segment SGI17496[] = {{&S94417[30], 10}, {NULL}};
const Segment SGI17497[] = {{&S94417[108], 3}, {NULL}};
const Segment SGI17498[] = {{&S94417[73], 6}, {NULL}};
const Segment SGI17499[] = {{&S92709[65], 14}, {NULL}};
const Segment SGI17500[] = {{&S92709[65], 25}, {NULL}};
const Segment SGI17501[] = {{&S92709[45], 12}, {NULL}};
const Segment SGI17502[] = {{&S92709[139], 3}, {NULL}};
const Segment SGI17503[] = {{&S92709[104], 6}, {NULL}};
const Segment SGI17504[] = {{&S94468[8], 8}, {NULL}};
const Segment SGI17505[] = {{&S95424[44], 18}, {NULL}};
const Segment SGI17506[] = {{&S95424[44], 29}, {NULL}};
const Segment SGI17507[] = {{&S95424[30], 6}, {NULL}};
const Segment SGI17508[] = {{&S95424[121], 3}, {NULL}};
const Segment SGI17509[] = {{&S95424[87], 5}, {NULL}};
const Segment SGI17510[] = {{&S95424[145], 7}, {NULL}};
const Segment SGI17511[] = {{&S93843[57], 10}, {NULL}};
const Segment SGI17512[] = {{&S93843[57], 21}, {NULL}};
const Segment SGI17513[] = {{&S93843[45], 4}, {NULL}};
const Segment SGI17514[] = {{&S93843[126], 3}, {NULL}};
const Segment SGI17515[] = {{&S93843[92], 5}, {NULL}};
const Segment SGI17516[] = {{&S93843[0], 14}, {NULL}};
const Segment SGI17517[] = {{&S93845[57], 10}, {NULL}};
const Segment SGI17518[] = {{&S93845[57], 21}, {NULL}};
const Segment SGI17519[] = {{&S93845[45], 4}, {NULL}};
const Segment SGI17520[] = {{&S93845[127], 3}, {NULL}};
const Segment SGI17521[] = {{&S93845[92], 6}, {NULL}};
const Segment SGI17522[] = {{&S93845[0], 14}, {NULL}};
const Segment SGI17523[] = {{&S101294[8], 7}, {NULL}};
const Segment SGI17524[] = {{&S101294[0], 85}, {NULL}};
const Segment SGI17525[] = {{&S101295[8], 7}, {&S101295[8], 7}, {NULL}};
const Segment SGI17526[] = {{&S101295[0], 172}, {NULL}};
const Segment SGI17527[] = {{&S101296[8], 8}, {NULL}};
const Segment SGI17528[] = {{&S101296[0], 86}, {NULL}};
const Segment SGI17529[] = {{&S92899[51], 10}, {NULL}};
const Segment SGI17530[] = {{&S92899[30], 12}, {NULL}};
const Segment SGI17531[] = {{&S92899[121], 3}, {NULL}};
const Segment SGI17532[] = {{&S92899[87], 5}, {NULL}};
const Segment SGI17533[] = {{&S92899[145], 13}, {NULL}};
const Segment SGI17534[] = {{&S92975[50], 10}, {NULL}};
const Segment SGI17535[] = {{&S92975[50], 21}, {NULL}};
const Segment SGI17536[] = {{&S92975[30], 12}, {NULL}};
const Segment SGI17537[] = {{&S92975[85], 6}, {NULL}};
const Segment SGI17538[] = {{&S92975[112], 13}, {NULL}};
const Segment SGI17539[] = {{&S93058[50], 7}, {NULL}};
const Segment SGI17540[] = {{&S93058[50], 18}, {NULL}};
const Segment SGI17541[] = {{&S93058[30], 12}, {NULL}};
const Segment SGI17542[] = {{&S93058[117], 3}, {NULL}};
const Segment SGI17543[] = {{&S93058[82], 6}, {NULL}};
const Segment SGI17544[] = {{&S95657[44], 10}, {NULL}};
const Segment SGI17545[] = {{&S95657[44], 21}, {NULL}};
const Segment SGI17546[] = {{&S95657[30], 6}, {NULL}};
const Segment SGI17547[] = {{&S95657[79], 6}, {NULL}};
const Segment SGI17548[] = {{&S95657[106], 13}, {NULL}};
const Segment SGI17549[] = {{&S94738[53], 8}, {&S94738[53], 8}, {NULL}};
const Segment SGI17550[] = {{&S95690[44], 16}, {NULL}};
const Segment SGI17551[] = {{&S95690[44], 27}, {NULL}};
const Segment SGI17552[] = {{&S95690[30], 6}, {NULL}};
const Segment SGI17553[] = {{&S95690[120], 3}, {NULL}};
const Segment SGI17554[] = {{&S95690[85], 6}, {NULL}};
const Segment SGI17555[] = {{&S95691[44], 16}, {NULL}};
const Segment SGI17556[] = {{&S95691[44], 27}, {NULL}};
const Segment SGI17557[] = {{&S95691[30], 6}, {NULL}};
const Segment SGI17558[] = {{&S95691[119], 3}, {NULL}};
const Segment SGI17559[] = {{&S95691[85], 5}, {NULL}};
const Segment SGI17560[] = {{&S95707[44], 11}, {NULL}};
const Segment SGI17561[] = {{&S95707[44], 22}, {NULL}};
const Segment SGI17562[] = {{&S95707[30], 6}, {NULL}};
const Segment SGI17563[] = {{&S95707[115], 3}, {NULL}};
const Segment SGI17564[] = {{&S95707[80], 6}, {NULL}};
const Segment SGI17565[] = {{&S95707[135], 8}, {NULL}};
const Segment SGI17566[] = {{&S94796[8], 8}, {NULL}};
const Segment SGI17567[] = {{&S95790[45], 10}, {NULL}};
const Segment SGI17568[] = {{&S95790[30], 6}, {NULL}};
const Segment SGI17569[] = {{&S95790[114], 3}, {NULL}};
const Segment SGI17570[] = {{&S95790[80], 5}, {NULL}};
const Segment SGI17571[] = {{&S95790[138], 13}, {NULL}};
const Segment SGI17572[] = {{&S94976[44], 8}, {NULL}};
const Segment SGI17573[] = {{&S95055[8], 8}, {NULL}};
const Segment SGI17574[] = {{&S96009[44], 7}, {NULL}};
const Segment SGI17575[] = {{&S96009[44], 18}, {NULL}};
const Segment SGI17576[] = {{&S96009[30], 6}, {NULL}};
const Segment SGI17577[] = {{&S96009[110], 3}, {NULL}};
const Segment SGI17578[] = {{&S96009[76], 5}, {NULL}};
const Segment SGI17579[] = {{&S96013[45], 14}, {NULL}};
const Segment SGI17580[] = {{&S96013[30], 6}, {NULL}};
const Segment SGI17581[] = {{&S96013[119], 3}, {NULL}};
const Segment SGI17582[] = {{&S96013[85], 5}, {NULL}};
const Segment SGI17583[] = {{&S96013[143], 13}, {NULL}};
const Segment SGI17584[] = {{&S96041[59], 10}, {NULL}};
const Segment SGI17585[] = {{&S96041[59], 21}, {NULL}};
const Segment SGI17586[] = {{&S96041[45], 6}, {NULL}};
const Segment SGI17587[] = {{&S96041[128], 3}, {NULL}};
const Segment SGI17588[] = {{&S96041[94], 5}, {NULL}};
const Segment SGI17589[] = {{&S93500[47], 24}, {NULL}};
const Segment SGI17590[] = {{&S96041[0], 14}, {NULL}};
const Segment SGI17591[] = {{&S93500[135], 3}, {NULL}};
const Segment SGI17592[] = {{&S93500[101], 6}, {NULL}};
const Segment SGI17593[] = {{&S93501[47], 24}, {NULL}};
const Segment SGI17594[] = {{&S93501[135], 3}, {NULL}};
const Segment SGI17595[] = {{&S93501[101], 6}, {NULL}};
const Segment SGI17596[] = {{&S93502[47], 24}, {NULL}};
const Segment SGI17597[] = {{&S93502[135], 3}, {NULL}};
const Segment SGI17598[] = {{&S93502[101], 6}, {NULL}};
const Segment SGI17599[] = {{&S93503[47], 28}, {NULL}};
const Segment SGI17600[] = {{&S93503[139], 3}, {NULL}};
const Segment SGI17601[] = {{&S93503[105], 6}, {NULL}};
const Segment SGI17602[] = {{&S94421[42], 10}, {NULL}};
const Segment SGI17603[] = {{&S94421[42], 21}, {NULL}};
const Segment SGI17604[] = {{&S94421[30], 4}, {NULL}};
const Segment SGI17605[] = {{&S94421[80], 4}, {NULL}};
const Segment SGI17606[] = {{&S94421[80], 3}, {NULL}};
const Segment SGI17607[] = {{&S94422[42], 4}, {NULL}};
const Segment SGI17608[] = {{&S94422[95], 5}, {NULL}};
const Segment SGI17609[] = {{&S93581[51], 6}, {NULL}};
const Segment SGI17610[] = {{&S93581[30], 12}, {NULL}};
const Segment SGI17611[] = {{&S93581[122], 3}, {NULL}};
const Segment SGI17612[] = {{&S93581[88], 5}, {NULL}};
const Segment SGI17613[] = {{&S93581[146], 13}, {NULL}};
const Segment SGI17614[] = {{&S93619[51], 6}, {NULL}};
const Segment SGI17615[] = {{&S93619[30], 12}, {NULL}};
const Segment SGI17616[] = {{&S93619[122], 3}, {NULL}};
const Segment SGI17617[] = {{&S93619[88], 5}, {NULL}};
const Segment SGI17618[] = {{&S93619[146], 13}, {NULL}};
const Segment SGI17619[] = {{&S102159[8], 3}, {NULL}};
const Segment SGI17620[] = {{&S102159[0], 46}, {NULL}};
const Segment SGI17621[] = {{&S95296[101], 5}, {NULL}};
const Segment SGI17622[] = {{&S93720[47], 30}, {NULL}};
const Segment SGI17623[] = {{&S93720[141], 3}, {NULL}};
const Segment SGI17624[] = {{&S93720[107], 6}, {NULL}};
const Segment SGI17625[] = {{&S102163[8], 7}, {NULL}};
const Segment SGI17626[] = {{&S102163[0], 85}, {NULL}};
const Segment SGI17627[] = {{&S93721[50], 36}, {NULL}};
const Segment SGI17628[] = {{&S93721[30], 12}, {NULL}};
const Segment SGI17629[] = {{&S93721[153], 3}, {NULL}};
const Segment SGI17630[] = {{&S93721[118], 6}, {NULL}};
const Segment SGI17631[] = {{&S93722[47], 30}, {NULL}};
const Segment SGI17632[] = {{&S93722[141], 3}, {NULL}};
const Segment SGI17633[] = {{&S93722[107], 6}, {NULL}};
const Segment SGI17634[] = {{&S93723[50], 12}, {NULL}};
const Segment SGI17635[] = {{&S93723[50], 23}, {NULL}};
const Segment SGI17636[] = {{&S93723[30], 12}, {NULL}};
const Segment SGI17637[] = {{&S93723[90], 4}, {NULL}};
const Segment SGI17638[] = {{&S93723[90], 3}, {NULL}};
const Segment SGI17639[] = {{&S96325[44], 8}, {NULL}};
const Segment SGI17640[] = {{&S96325[44], 20}, {NULL}};
const Segment SGI17641[] = {{&S96325[30], 6}, {NULL}};
const Segment SGI17642[] = {{&S96325[112], 3}, {NULL}};
const Segment SGI17643[] = {{&S96325[78], 5}, {NULL}};
const Segment SGI17644[] = {{&S93785[50], 7}, {NULL}};
const Segment SGI17645[] = {{&S93785[50], 18}, {NULL}};
const Segment SGI17646[] = {{&S93785[30], 12}, {NULL}};
const Segment SGI17647[] = {{&S93785[85], 4}, {NULL}};
const Segment SGI17648[] = {{&S93785[85], 3}, {NULL}};
const Segment SGI17649[] = {{&S93786[50], 7}, {NULL}};
const Segment SGI17650[] = {{&S93786[50], 18}, {NULL}};
const Segment SGI17651[] = {{&S93786[30], 12}, {NULL}};
const Segment SGI17652[] = {{&S93786[85], 6}, {NULL}};
const Segment SGI17653[] = {{&S93787[50], 18}, {NULL}};
const Segment SGI17654[] = {{&S93787[30], 12}, {NULL}};
const Segment SGI17655[] = {{&S95362[60], 10}, {NULL}};
const Segment SGI17656[] = {{&S95362[60], 22}, {NULL}};
const Segment SGI17657[] = {{&S95362[30], 22}, {NULL}};
const Segment SGI17658[] = {{&S95362[131], 3}, {NULL}};
const Segment SGI17659[] = {{&S95362[96], 6}, {NULL}};
const Segment SGI17660[] = {{&S95363[69], 10}, {NULL}};
const Segment SGI17661[] = {{&S95363[69], 22}, {NULL}};
const Segment SGI17662[] = {{&S95363[30], 31}, {NULL}};
const Segment SGI17663[] = {{&S95363[140], 3}, {NULL}};
const Segment SGI17664[] = {{&S95363[105], 6}, {NULL}};
const Segment SGI17665[] = {{&S95370[111], 3}, {NULL}};
const Segment SGI17666[] = {{&S95370[76], 6}, {NULL}};
const Segment SGI17667[] = {{&S95464[74], 20}, {NULL}};
const Segment SGI17668[] = {{&S95464[74], 31}, {NULL}};
const Segment SGI17669[] = {{&S95464[30], 16}, {NULL}};
const Segment SGI17670[] = {{&S95464[153], 3}, {NULL}};
const Segment SGI17671[] = {{&S95464[119], 5}, {NULL}};
const Segment SGI17672[] = {{&S94921[29], 7}, {NULL}};
const Segment SGI17673[] = {{&S94921[51], 3}, {NULL}};
const Segment SGI17674[] = {{&S94928[42], 25}, {NULL}};
const Segment SGI17675[] = {{&S94928[30], 4}, {NULL}};
const Segment SGI17676[] = {{&S94928[133], 3}, {NULL}};
const Segment SGI17677[] = {{&S94928[99], 5}, {NULL}};
const Segment SGI17678[] = {{&S102564[8], 3}, {NULL}};
const Segment SGI17679[] = {{&S102564[0], 46}, {NULL}};
const Segment SGI17680[] = {{&S102583[8], 7}, {NULL}};
const Segment SGI17681[] = {{&S102583[0], 45}, {NULL}};
const Segment SGI17682[] = {{&S102584[8], 8}, {NULL}};
const Segment SGI17683[] = {{&S102584[0], 46}, {NULL}};
const Segment SGI17684[] = {{&S102605[8], 7}, {NULL}};
const Segment SGI17685[] = {{&S102605[0], 45}, {NULL}};
const Segment SGI17686[] = {{&S98075[42], 16}, {NULL}};
const Segment SGI17687[] = {{&S98075[42], 27}, {NULL}};
const Segment SGI17688[] = {{&S98075[30], 4}, {NULL}};
const Segment SGI17689[] = {{&S102606[8], 7}, {NULL}};
const Segment SGI17690[] = {{&S102606[0], 58}, {NULL}};
const Segment SGI17691[] = {{&S98075[118], 3}, {NULL}};
const Segment SGI17692[] = {{&S98075[83], 6}, {NULL}};
const Segment SGI17693[] = {{&S98075[138], 8}, {NULL}};
const Segment SGI17694[] = {{&S98090[42], 8}, {NULL}};
const Segment SGI17695[] = {{&S98090[42], 19}, {NULL}};
const Segment SGI17696[] = {{&S98090[30], 4}, {NULL}};
const Segment SGI17697[] = {{&S98090[110], 3}, {NULL}};
const Segment SGI17698[] = {{&S98090[75], 6}, {NULL}};
const Segment SGI17699[] = {{&S98090[130], 8}, {NULL}};
const Segment SGI17700[] = {{&S98091[42], 14}, {NULL}};
const Segment SGI17701[] = {{&S98091[42], 25}, {NULL}};
const Segment SGI17702[] = {{&S98091[30], 4}, {NULL}};
const Segment SGI17703[] = {{&S98091[116], 3}, {NULL}};
const Segment SGI17704[] = {{&S98091[81], 6}, {NULL}};
const Segment SGI17705[] = {{&S98091[136], 8}, {NULL}};
const Segment SGI17706[] = {{&S98092[42], 15}, {NULL}};
const Segment SGI17707[] = {{&S98092[42], 26}, {NULL}};
const Segment SGI17708[] = {{&S98092[30], 4}, {NULL}};
const Segment SGI17709[] = {{&S98092[117], 3}, {NULL}};
const Segment SGI17710[] = {{&S98092[82], 6}, {NULL}};
const Segment SGI17711[] = {{&S98092[137], 8}, {NULL}};
const Segment SGI17712[] = {{&S94199[45], 5}, {NULL}};
const Segment SGI17713[] = {{&S94199[106], 3}, {NULL}};
const Segment SGI17714[] = {{&S94199[95], 3}, {NULL}};
const Segment SGI17715[] = {{&S94199[72], 6}, {NULL}};
const Segment SGI17716[] = {{&S94200[37], 5}, {NULL}};
const Segment SGI17717[] = {{&S95732[95], 3}, {NULL}};
const Segment SGI17718[] = {{&S94200[57], 3}, {NULL}};
const Segment SGI17719[] = {{&S95732[58], 5}, {NULL}};
const Segment SGI17720[] = {{&S95222[42], 10}, {NULL}};
const Segment SGI17721[] = {{&S95222[42], 21}, {NULL}};
const Segment SGI17722[] = {{&S95222[30], 4}, {NULL}};
const Segment SGI17723[] = {{&S95222[156], 3}, {NULL}};
const Segment SGI17724[] = {{&S95222[122], 3}, {NULL}};
const Segment SGI17725[] = {{&S95222[87], 6}, {NULL}};
const Segment SGI17726[] = {{&S95222[142], 8}, {NULL}};
const Segment SGI17727[] = {{&S95228[43], 10}, {NULL}};
const Segment SGI17728[] = {{&S95228[30], 4}, {NULL}};
const Segment SGI17729[] = {{&S95228[112], 3}, {NULL}};
const Segment SGI17730[] = {{&S95228[78], 5}, {NULL}};
const Segment SGI17731[] = {{&S95228[136], 13}, {NULL}};
const Segment SGI17732[] = {{&S94272[51], 6}, {NULL}};
const Segment SGI17733[] = {{&S94272[30], 12}, {NULL}};
const Segment SGI17734[] = {{&S94272[122], 3}, {NULL}};
const Segment SGI17735[] = {{&S94272[88], 5}, {NULL}};
const Segment SGI17736[] = {{&S94272[146], 13}, {NULL}};
const Segment SGI17737[] = {{&S102995[8], 8}, {NULL}};
const Segment SGI17738[] = {{&S102995[0], 86}, {NULL}};
const Segment SGI17739[] = {{&S102996[8], 7}, {NULL}};
const Segment SGI17740[] = {{&S102996[0], 85}, {NULL}};
const Segment SGI17741[] = {{&S103023[8], 7}, {NULL}};
const Segment SGI17742[] = {{&S103023[0], 45}, {NULL}};
const Segment SGI17743[] = {{&S103025[8], 3}, {NULL}};
const Segment SGI17744[] = {{&S103025[0], 46}, {NULL}};
const Segment SGI17745[] = {{&S95397[29], 10}, {NULL}};
const Segment SGI17746[] = {{&S95397[54], 3}, {NULL}};
const Segment SGI17747[] = {{&S95977[45], 15}, {NULL}};
const Segment SGI17748[] = {{&S95977[45], 27}, {NULL}};
const Segment SGI17749[] = {{&S95977[30], 7}, {NULL}};
const Segment SGI17750[] = {{&S95977[121], 3}, {NULL}};
const Segment SGI17751[] = {{&S95977[86], 6}, {NULL}};
const Segment SGI17752[] = {{&S95978[45], 15}, {NULL}};
const Segment SGI17753[] = {{&S95978[45], 27}, {NULL}};
const Segment SGI17754[] = {{&S95978[30], 7}, {NULL}};
const Segment SGI17755[] = {{&S95978[121], 3}, {NULL}};
const Segment SGI17756[] = {{&S95978[86], 6}, {NULL}};
const Segment SGI17757[] = {{&S95978[141], 8}, {NULL}};
const Segment SGI17758[] = {{&S103245[8], 7}, {NULL}};
const Segment SGI17759[] = {{&S103245[0], 45}, {NULL}};
const Segment SGI17760[] = {{&S103305[8], 17}, {NULL}};
const Segment SGI17761[] = {{&S103305[0], 95}, {NULL}};
const Segment SGI17762[] = {{&S103308[8], 7}, {NULL}};
const Segment SGI17763[] = {{&S103308[0], 45}, {NULL}};
const Segment SGI17764[] = {{&S95775[57], 10}, {NULL}};
const Segment SGI17765[] = {{&S95775[57], 21}, {NULL}};
const Segment SGI17766[] = {{&S95775[45], 4}, {NULL}};
const Segment SGI17767[] = {{&S95775[126], 3}, {NULL}};
const Segment SGI17768[] = {{&S95775[92], 5}, {NULL}};
const Segment SGI17769[] = {{&S95775[0], 14}, {NULL}};
const Segment SGI17770[] = {{&S96388[43], 8}, {&S96388[43], 8}, {NULL}};
const Segment SGI17771[] = {{&S96390[59], 8}, {&S96390[59], 8}, {NULL}};
const Segment SGI17772[] = {{&S96390[0], 13}, {NULL}};
const Segment SGI17773[] = {{&S96014[42], 7}, {NULL}};
const Segment SGI17774[] = {{&S96014[42], 19}, {NULL}};
const Segment SGI17775[] = {{&S96014[30], 4}, {NULL}};
const Segment SGI17776[] = {{&S96014[109], 3}, {NULL}};
const Segment SGI17777[] = {{&S96014[75], 5}, {NULL}};
const Segment SGI17778[] = {{&S94923[51], 6}, {NULL}};
const Segment SGI17779[] = {{&S94923[30], 12}, {NULL}};
const Segment SGI17780[] = {{&S94923[122], 3}, {NULL}};
const Segment SGI17781[] = {{&S94923[88], 5}, {NULL}};
const Segment SGI17782[] = {{&S94923[146], 13}, {NULL}};
const Segment SGI17783[] = {{&S94948[65], 6}, {NULL}};
const Segment SGI17784[] = {{&S94948[65], 23}, {NULL}};
const Segment SGI17785[] = {{&S94948[45], 12}, {NULL}};
const Segment SGI17786[] = {{&S94948[137], 3}, {NULL}};
const Segment SGI17787[] = {{&S94948[102], 6}, {NULL}};
const Segment SGI17788[] = {{&S97754[44], 10}, {NULL}};
const Segment SGI17789[] = {{&S97754[44], 21}, {NULL}};
const Segment SGI17790[] = {{&S97754[30], 6}, {NULL}};
const Segment SGI17791[] = {{&S97754[79], 6}, {NULL}};
const Segment SGI17792[] = {{&S97754[106], 13}, {NULL}};
const Segment SGI17793[] = {{&S96135[42], 7}, {NULL}};
const Segment SGI17794[] = {{&S96135[42], 18}, {NULL}};
const Segment SGI17795[] = {{&S96135[30], 4}, {NULL}};
const Segment SGI17796[] = {{&S96135[74], 6}, {NULL}};
const Segment SGI17797[] = {{&S96135[101], 13}, {NULL}};
const Segment SGI17798[] = {{&S96569[0], 13}, {NULL}};
const Segment SGI17799[] = {{&S97798[44], 10}, {NULL}};
const Segment SGI17800[] = {{&S97798[44], 21}, {NULL}};
const Segment SGI17801[] = {{&S97798[30], 6}, {NULL}};
const Segment SGI17802[] = {{&S97798[79], 6}, {NULL}};
const Segment SGI17803[] = {{&S97798[106], 13}, {NULL}};
const Segment SGI17804[] = {{&S99063[42], 11}, {NULL}};
const Segment SGI17805[] = {{&S99063[42], 22}, {NULL}};
const Segment SGI17806[] = {{&S99063[30], 4}, {NULL}};
const Segment SGI17807[] = {{&S99063[113], 3}, {NULL}};
const Segment SGI17808[] = {{&S99063[78], 6}, {NULL}};
const Segment SGI17809[] = {{&S99063[133], 8}, {NULL}};
const Segment SGI17810[] = {{&S95072[50], 6}, {NULL}};
const Segment SGI17811[] = {{&S95072[50], 23}, {NULL}};
const Segment SGI17812[] = {{&S95072[30], 12}, {NULL}};
const Segment SGI17813[] = {{&S95072[87], 6}, {NULL}};
const Segment SGI17814[] = {{&S95072[114], 13}, {NULL}};
const Segment SGI17815[] = {{&S103853[8], 7}, {NULL}};
const Segment SGI17816[] = {{&S103853[0], 48}, {NULL}};
const Segment SGI17817[] = {{&S97922[44], 18}, {NULL}};
const Segment SGI17818[] = {{&S97922[44], 29}, {NULL}};
const Segment SGI17819[] = {{&S97922[30], 6}, {NULL}};
const Segment SGI17820[] = {{&S97922[87], 6}, {NULL}};
const Segment SGI17821[] = {{&S97922[114], 13}, {NULL}};
const Segment SGI17822[] = {{&S96686[13], 8}, {NULL}};
const Segment SGI17823[] = {{&S96688[53], 8}, {&S96688[53], 8}, {NULL}};
const Segment SGI17824[] = {{&S96688[88], 12}, {NULL}};
const Segment SGI17825[] = {{&S96691[60], 8}, {&S96691[60], 8}, {NULL}};
const Segment SGI17826[] = {{&S96691[0], 13}, {NULL}};
const Segment SGI17827[] = {{&S95167[50], 11}, {NULL}};
const Segment SGI17828[] = {{&S95167[50], 22}, {NULL}};
const Segment SGI17829[] = {{&S95167[30], 12}, {NULL}};
const Segment SGI17830[] = {{&S95167[121], 3}, {NULL}};
const Segment SGI17831[] = {{&S95167[86], 6}, {NULL}};
const Segment SGI17832[] = {{&S103934[8], 7}, {NULL}};
const Segment SGI17833[] = {{&S103934[0], 45}, {NULL}};
const Segment SGI17834[] = {{&S96341[42], 18}, {NULL}};
const Segment SGI17835[] = {{&S96341[42], 29}, {NULL}};
const Segment SGI17836[] = {{&S96341[30], 4}, {NULL}};
const Segment SGI17837[] = {{&S96341[119], 3}, {NULL}};
const Segment SGI17838[] = {{&S96341[85], 5}, {NULL}};
const Segment SGI17839[] = {{&S98123[57], 10}, {NULL}};
const Segment SGI17840[] = {{&S98123[57], 21}, {NULL}};
const Segment SGI17841[] = {{&S98123[43], 6}, {NULL}};
const Segment SGI17842[] = {{&S98123[126], 3}, {NULL}};
const Segment SGI17843[] = {{&S98123[92], 5}, {NULL}};
const Segment SGI17844[] = {{&S98123[0], 12}, {NULL}};
const Segment SGI17845[] = {{&S95324[50], 6}, {NULL}};
const Segment SGI17846[] = {{&S95324[50], 24}, {NULL}};
const Segment SGI17847[] = {{&S95324[30], 12}, {NULL}};
const Segment SGI17848[] = {{&S95324[88], 6}, {NULL}};
const Segment SGI17849[] = {{&S95324[115], 13}, {NULL}};
const Segment SGI17850[] = {{&S95367[50], 7}, {NULL}};
const Segment SGI17851[] = {{&S95367[50], 18}, {NULL}};
const Segment SGI17852[] = {{&S95367[30], 12}, {NULL}};
const Segment SGI17853[] = {{&S95367[117], 3}, {NULL}};
const Segment SGI17854[] = {{&S95367[82], 6}, {NULL}};
const Segment SGI17855[] = {{&S95368[50], 7}, {NULL}};
const Segment SGI17856[] = {{&S95368[50], 18}, {NULL}};
const Segment SGI17857[] = {{&S95368[30], 12}, {NULL}};
const Segment SGI17858[] = {{&S95368[85], 4}, {NULL}};
const Segment SGI17859[] = {{&S95368[85], 3}, {NULL}};
const Segment SGI17860[] = {{&S98203[45], 18}, {NULL}};
const Segment SGI17861[] = {{&S98203[30], 6}, {NULL}};
const Segment SGI17862[] = {{&S98203[122], 3}, {NULL}};
const Segment SGI17863[] = {{&S98203[88], 5}, {NULL}};
const Segment SGI17864[] = {{&S98203[146], 13}, {NULL}};
const Segment SGI17865[] = {{&S104186[8], 14}, {NULL}};
const Segment SGI17866[] = {{&S104186[0], 92}, {NULL}};
const Segment SGI17867[] = {{&S104191[8], 7}, {NULL}};
const Segment SGI17868[] = {{&S104191[0], 85}, {NULL}};
const Segment SGI17869[] = {{&S104212[8], 7}, {NULL}};
const Segment SGI17870[] = {{&S104212[0], 85}, {NULL}};
const Segment SGI17871[] = {{&S96495[42], 10}, {NULL}};
const Segment SGI17872[] = {{&S96495[42], 21}, {NULL}};
const Segment SGI17873[] = {{&S96495[30], 4}, {NULL}};
const Segment SGI17874[] = {{&S96495[77], 6}, {NULL}};
const Segment SGI17875[] = {{&S96495[104], 13}, {NULL}};
const Segment SGI17876[] = {{&S99525[42], 7}, {NULL}};
const Segment SGI17877[] = {{&S99525[42], 20}, {NULL}};
const Segment SGI17878[] = {{&S99525[30], 4}, {NULL}};
const Segment SGI17879[] = {{&S99525[111], 3}, {NULL}};
const Segment SGI17880[] = {{&S99525[76], 6}, {NULL}};
const Segment SGI17881[] = {{&S99525[131], 8}, {NULL}};
const Segment SGI17882[] = {{&S104233[8], 6}, {NULL}};
const Segment SGI17883[] = {{&S104233[0], 44}, {NULL}};
const Segment SGI17884[] = {{&S104237[8], 8}, {NULL}};
const Segment SGI17885[] = {{&S104237[0], 86}, {NULL}};
const Segment SGI17886[] = {{&S104264[8], 9}, {NULL}};
const Segment SGI17887[] = {{&S104264[0], 87}, {NULL}};
const Segment SGI17888[] = {{&S104293[8], 3}, {NULL}};
const Segment SGI17889[] = {{&S104293[0], 36}, {NULL}};
const Segment SGI17890[] = {{&S98406[44], 18}, {NULL}};
const Segment SGI17891[] = {{&S98406[44], 29}, {NULL}};
const Segment SGI17892[] = {{&S98406[30], 6}, {NULL}};
const Segment SGI17893[] = {{&S98406[87], 6}, {NULL}};
const Segment SGI17894[] = {{&S98406[114], 13}, {NULL}};
const Segment SGI17895[] = {{&S97035[13], 8}, {NULL}};
const Segment SGI17896[] = {{&S97035[22], 6}, {NULL}};
const Segment SGI17897[] = {{&S97036[13], 8}, {NULL}};
const Segment SGI17898[] = {{&S97043[23], 8}, {NULL}};
const Segment SGI17899[] = {{&S97044[53], 8}, {&S97044[53], 8}, {&S97044[53], 8}, {NULL}};
const Segment SGI17900[] = {{&S97045[59], 8}, {&S97045[59], 8}, {NULL}};
const Segment SGI17901[] = {{&S97045[0], 13}, {NULL}};
const Segment SGI17902[] = {{&S97046[62], 8}, {&S97046[62], 8}, {NULL}};
const Segment SGI17903[] = {{&S97046[0], 13}, {NULL}};
const Segment SGI17904[] = {{&S98592[49], 15}, {NULL}};
const Segment SGI17905[] = {{&S98592[49], 26}, {NULL}};
const Segment SGI17906[] = {{&S98592[30], 11}, {NULL}};
const Segment SGI17907[] = {{&S98592[123], 3}, {NULL}};
const Segment SGI17908[] = {{&S98592[89], 5}, {NULL}};
const Segment SGI17909[] = {{&S98593[44], 15}, {NULL}};
const Segment SGI17910[] = {{&S98593[44], 26}, {NULL}};
const Segment SGI17911[] = {{&S98593[30], 6}, {NULL}};
const Segment SGI17912[] = {{&S98593[118], 3}, {NULL}};
const Segment SGI17913[] = {{&S98593[84], 5}, {NULL}};
const Segment SGI17914[] = {{&S104604[8], 6}, {NULL}};
const Segment SGI17915[] = {{&S104604[0], 57}, {NULL}};
const Segment SGI17916[] = {{&S104605[8], 6}, {NULL}};
const Segment SGI17917[] = {{&S104605[0], 44}, {NULL}};
const Segment SGI17918[] = {{&S104612[8], 7}, {NULL}};
const Segment SGI17919[] = {{&S104612[0], 45}, {NULL}};
const Segment SGI17920[] = {{&S104649[8], 19}, {NULL}};
const Segment SGI17921[] = {{&S104649[0], 97}, {NULL}};
const Segment SGI17922[] = {{&S104678[8], 7}, {NULL}};
const Segment SGI17923[] = {{&S104678[0], 45}, {NULL}};
const Segment SGI17924[] = {{&S104679[8], 7}, {NULL}};
const Segment SGI17925[] = {{&S104679[0], 45}, {NULL}};
const Segment SGI17926[] = {{&S98787[44], 9}, {NULL}};
const Segment SGI17927[] = {{&S98787[44], 20}, {NULL}};
const Segment SGI17928[] = {{&S98787[30], 6}, {NULL}};
const Segment SGI17929[] = {{&S98787[78], 6}, {NULL}};
const Segment SGI17930[] = {{&S98787[105], 13}, {NULL}};
const Segment SGI17931[] = {{&S97047[42], 10}, {NULL}};
const Segment SGI17932[] = {{&S97047[42], 21}, {NULL}};
const Segment SGI17933[] = {{&S97047[30], 4}, {NULL}};
const Segment SGI17934[] = {{&S97047[111], 3}, {NULL}};
const Segment SGI17935[] = {{&S97047[77], 5}, {NULL}};
const Segment SGI17936[] = {{&S97047[135], 11}, {NULL}};
const Segment SGI17937[] = {{&S97080[42], 12}, {NULL}};
const Segment SGI17938[] = {{&S97080[42], 23}, {NULL}};
const Segment SGI17939[] = {{&S97080[30], 4}, {NULL}};
const Segment SGI17940[] = {{&S97080[79], 6}, {NULL}};
const Segment SGI17941[] = {{&S97080[106], 13}, {NULL}};
const Segment SGI17942[] = {{&S97356[21], 8}, {NULL}};
const Segment SGI17943[] = {{&S96311[50], 8}, {NULL}};
const Segment SGI17944[] = {{&S96311[50], 19}, {NULL}};
const Segment SGI17945[] = {{&S96311[30], 12}, {NULL}};
const Segment SGI17946[] = {{&S96311[118], 3}, {NULL}};
const Segment SGI17947[] = {{&S96311[83], 6}, {NULL}};
const Segment SGI17948[] = {{&S99104[57], 10}, {NULL}};
const Segment SGI17949[] = {{&S99104[57], 21}, {NULL}};
const Segment SGI17950[] = {{&S99104[43], 6}, {NULL}};
const Segment SGI17951[] = {{&S99104[126], 3}, {NULL}};
const Segment SGI17952[] = {{&S99104[92], 5}, {NULL}};
const Segment SGI17953[] = {{&S99104[0], 12}, {NULL}};
const Segment SGI17954[] = {{&S105152[8], 7}, {NULL}};
const Segment SGI17955[] = {{&S105152[0], 85}, {NULL}};
const Segment SGI17956[] = {{&S105155[8], 7}, {NULL}};
const Segment SGI17957[] = {{&S105155[0], 48}, {NULL}};
const Segment SGI17958[] = {{&S97346[42], 9}, {NULL}};
const Segment SGI17959[] = {{&S97346[42], 20}, {NULL}};
const Segment SGI17960[] = {{&S97346[30], 4}, {NULL}};
const Segment SGI17961[] = {{&S97346[110], 3}, {NULL}};
const Segment SGI17962[] = {{&S97346[76], 5}, {NULL}};
const Segment SGI17963[] = {{&S99173[31], 4}, {NULL}};
const Segment SGI17964[] = {{&S99173[50], 3}, {NULL}};
const Segment SGI17965[] = {{&S105237[8], 8}, {NULL}};
const Segment SGI17966[] = {{&S105237[0], 86}, {NULL}};
const Segment SGI17967[] = {{&S105243[8], 7}, {NULL}};
const Segment SGI17968[] = {{&S105243[0], 85}, {NULL}};
const Segment SGI17969[] = {{&S99210[44], 10}, {NULL}};
const Segment SGI17970[] = {{&S99210[44], 21}, {NULL}};
const Segment SGI17971[] = {{&S99210[30], 6}, {NULL}};
const Segment SGI17972[] = {{&S99210[157], 3}, {NULL}};
const Segment SGI17973[] = {{&S99210[123], 3}, {NULL}};
const Segment SGI17974[] = {{&S99210[88], 6}, {NULL}};
const Segment SGI17975[] = {{&S99210[143], 8}, {NULL}};
const Segment SGI17976[] = {{&S105300[8], 7}, {NULL}};
const Segment SGI17977[] = {{&S105300[0], 85}, {NULL}};
const Segment SGI17978[] = {{&S100601[54], 14}, {NULL}};
const Segment SGI17979[] = {{&S100601[54], 33}, {NULL}};
const Segment SGI17980[] = {{&S100601[30], 16}, {NULL}};
const Segment SGI17981[] = {{&S100601[104], 6}, {NULL}};
const Segment SGI17982[] = {{&S99338[44], 27}, {NULL}};
const Segment SGI17983[] = {{&S99338[44], 39}, {NULL}};
const Segment SGI17984[] = {{&S99338[30], 6}, {NULL}};
const Segment SGI17985[] = {{&S99338[97], 6}, {NULL}};
const Segment SGI17986[] = {{&S99338[124], 13}, {NULL}};
const Segment SGI17987[] = {{&S99340[44], 10}, {NULL}};
const Segment SGI17988[] = {{&S99340[44], 21}, {NULL}};
const Segment SGI17989[] = {{&S99340[30], 6}, {NULL}};
const Segment SGI17990[] = {{&S99340[79], 6}, {NULL}};
const Segment SGI17991[] = {{&S99340[106], 13}, {NULL}};
const Segment SGI17992[] = {{&S97748[43], 8}, {&S97748[43], 8}, {NULL}};
const Segment SGI17993[] = {{&S97748[77], 6}, {NULL}};
const Segment SGI17994[] = {{&S97749[43], 8}, {&S97749[43], 8}, {NULL}};
const Segment SGI17995[] = {{&S97599[42], 10}, {NULL}};
const Segment SGI17996[] = {{&S97599[42], 21}, {NULL}};
const Segment SGI17997[] = {{&S97599[30], 4}, {NULL}};
const Segment SGI17998[] = {{&S97599[77], 6}, {NULL}};
const Segment SGI17999[] = {{&S97599[104], 13}, {NULL}};
const Segment SGI18000[] = {{&S97750[45], 8}, {NULL}};
const Segment SGI18001[] = {{&S97753[61], 8}, {NULL}};
const Segment SGI18002[] = {{&S97753[0], 13}, {NULL}};
const Segment SGI18003[] = {{&S100784[45], 7}, {NULL}};
const Segment SGI18004[] = {{&S100784[45], 20}, {NULL}};
const Segment SGI18005[] = {{&S100784[30], 7}, {NULL}};
const Segment SGI18006[] = {{&S100784[114], 3}, {NULL}};
const Segment SGI18007[] = {{&S100784[79], 6}, {NULL}};
const Segment SGI18008[] = {{&S100784[134], 8}, {NULL}};
const Segment SGI18009[] = {{&S96479[50], 6}, {NULL}};
const Segment SGI18010[] = {{&S96479[50], 24}, {NULL}};
const Segment SGI18011[] = {{&S96479[30], 12}, {NULL}};
const Segment SGI18012[] = {{&S96479[121], 3}, {NULL}};
const Segment SGI18013[] = {{&S96479[88], 5}, {NULL}};
const Segment SGI18014[] = {{&S99482[44], 14}, {NULL}};
const Segment SGI18015[] = {{&S99482[44], 26}, {NULL}};
const Segment SGI18016[] = {{&S99482[30], 6}, {NULL}};
const Segment SGI18017[] = {{&S99482[119], 3}, {NULL}};
const Segment SGI18018[] = {{&S99482[84], 6}, {NULL}};
const Segment SGI18019[] = {{&S99482[139], 8}, {NULL}};
const Segment SGI18020[] = {{&S99482[148], 13}, {NULL}};
const Segment SGI18021[] = {{&S96646[51], 10}, {NULL}};
const Segment SGI18022[] = {{&S96646[30], 12}, {NULL}};
const Segment SGI18023[] = {{&S96646[121], 3}, {NULL}};
const Segment SGI18024[] = {{&S96646[87], 5}, {NULL}};
const Segment SGI18025[] = {{&S96646[145], 13}, {NULL}};
const Segment SGI18026[] = {{&S101025[42], 7}, {NULL}};
const Segment SGI18027[] = {{&S101025[42], 18}, {NULL}};
const Segment SGI18028[] = {{&S101025[30], 4}, {NULL}};
const Segment SGI18029[] = {{&S101025[109], 3}, {NULL}};
const Segment SGI18030[] = {{&S101025[74], 6}, {NULL}};
const Segment SGI18031[] = {{&S101025[129], 8}, {NULL}};
const Segment SGI18032[] = {{&S96720[51], 10}, {NULL}};
const Segment SGI18033[] = {{&S96720[30], 12}, {NULL}};
const Segment SGI18034[] = {{&S96720[121], 3}, {NULL}};
const Segment SGI18035[] = {{&S96720[87], 5}, {NULL}};
const Segment SGI18036[] = {{&S96720[145], 13}, {NULL}};
const Segment SGI18037[] = {{&S97755[42], 10}, {NULL}};
const Segment SGI18038[] = {{&S97755[42], 21}, {NULL}};
const Segment SGI18039[] = {{&S97755[30], 4}, {NULL}};
const Segment SGI18040[] = {{&S97755[77], 6}, {NULL}};
const Segment SGI18041[] = {{&S97755[104], 13}, {NULL}};
const Segment SGI18042[] = {{&S98172[45], 8}, {NULL}};
const Segment SGI18043[] = {{&S98172[0], 12}, {NULL}};
const Segment SGI18044[] = {{&S98175[59], 8}, {&S98175[59], 8}, {&S98175[59], 8}, {NULL}};
const Segment SGI18045[] = {{&S98175[0], 13}, {NULL}};
const Segment SGI18046[] = {{&S98176[62], 8}, {&S98176[62], 8}, {NULL}};
const Segment SGI18047[] = {{&S98176[0], 13}, {NULL}};
const Segment SGI18048[] = {{&S98177[61], 8}, {NULL}};
const Segment SGI18049[] = {{&S98177[0], 13}, {NULL}};
const Segment SGI18050[] = {{&S97936[42], 11}, {NULL}};
const Segment SGI18051[] = {{&S97936[42], 22}, {NULL}};
const Segment SGI18052[] = {{&S97936[30], 4}, {NULL}};
const Segment SGI18053[] = {{&S97936[112], 3}, {NULL}};
const Segment SGI18054[] = {{&S97936[78], 5}, {NULL}};
const Segment SGI18055[] = {{&S98249[71], 5}, {NULL}};
const Segment SGI18056[] = {{&S99790[44], 7}, {NULL}};
const Segment SGI18057[] = {{&S99790[44], 18}, {NULL}};
const Segment SGI18058[] = {{&S99790[30], 6}, {NULL}};
const Segment SGI18059[] = {{&S99790[110], 3}, {NULL}};
const Segment SGI18060[] = {{&S99790[76], 5}, {NULL}};
const Segment SGI18061[] = {{&S99790[134], 7}, {NULL}};
const Segment SGI18062[] = {{&S99790[134], 11}, {NULL}};
const Segment SGI18063[] = {{&S96990[51], 10}, {NULL}};
const Segment SGI18064[] = {{&S96990[30], 12}, {NULL}};
const Segment SGI18065[] = {{&S96990[120], 3}, {NULL}};
const Segment SGI18066[] = {{&S96990[86], 5}, {NULL}};
const Segment SGI18067[] = {{&S96990[144], 13}, {NULL}};
const Segment SGI18068[] = {{&S96991[50], 10}, {NULL}};
const Segment SGI18069[] = {{&S96991[50], 21}, {NULL}};
const Segment SGI18070[] = {{&S96991[30], 12}, {NULL}};
const Segment SGI18071[] = {{&S96991[120], 3}, {NULL}};
const Segment SGI18072[] = {{&S96991[85], 6}, {NULL}};
const Segment SGI18073[] = {{&S96991[140], 8}, {NULL}};
const Segment SGI18074[] = {{&S96991[149], 13}, {NULL}};
const Segment SGI18075[] = {{&S98333[82], 5}, {NULL}};
const Segment SGI18076[] = {{&S99862[45], 11}, {NULL}};
const Segment SGI18077[] = {{&S99862[30], 6}, {NULL}};
const Segment SGI18078[] = {{&S99862[115], 3}, {NULL}};
const Segment SGI18079[] = {{&S99862[81], 5}, {NULL}};
const Segment SGI18080[] = {{&S99862[139], 13}, {NULL}};
const Segment SGI18081[] = {{&S97019[51], 6}, {NULL}};
const Segment SGI18082[] = {{&S97019[30], 12}, {NULL}};
const Segment SGI18083[] = {{&S97019[122], 3}, {NULL}};
const Segment SGI18084[] = {{&S97019[88], 5}, {NULL}};
const Segment SGI18085[] = {{&S97019[146], 13}, {NULL}};
const Segment SGI18086[] = {{&S97028[65], 6}, {NULL}};
const Segment SGI18087[] = {{&S97028[65], 23}, {NULL}};
const Segment SGI18088[] = {{&S97028[45], 12}, {NULL}};
const Segment SGI18089[] = {{&S97028[137], 3}, {NULL}};
const Segment SGI18090[] = {{&S97028[102], 6}, {NULL}};
const Segment SGI18091[] = {{&S101308[37], 4}, {NULL}};
const Segment SGI18092[] = {{&S101308[97], 3}, {NULL}};
const Segment SGI18093[] = {{&S101308[86], 3}, {NULL}};
const Segment SGI18094[] = {{&S101308[63], 6}, {NULL}};
const Segment SGI18095[] = {{&S101309[29], 4}, {NULL}};
const Segment SGI18096[] = {{&S101309[48], 3}, {NULL}};
const Segment SGI18097[] = {{&S101339[42], 18}, {NULL}};
const Segment SGI18098[] = {{&S101339[42], 29}, {NULL}};
const Segment SGI18099[] = {{&S101339[30], 4}, {NULL}};
const Segment SGI18100[] = {{&S101339[120], 3}, {NULL}};
const Segment SGI18101[] = {{&S101339[85], 6}, {NULL}};
const Segment SGI18102[] = {{&S101339[140], 8}, {NULL}};
const Segment SGI18103[] = {{&S99984[44], 9}, {NULL}};
const Segment SGI18104[] = {{&S99984[44], 20}, {NULL}};
const Segment SGI18105[] = {{&S99984[30], 6}, {NULL}};
const Segment SGI18106[] = {{&S99984[112], 3}, {NULL}};
const Segment SGI18107[] = {{&S99984[78], 5}, {NULL}};
const Segment SGI18108[] = {{&S98211[43], 10}, {NULL}};
const Segment SGI18109[] = {{&S98211[30], 4}, {NULL}};
const Segment SGI18110[] = {{&S98211[112], 3}, {NULL}};
const Segment SGI18111[] = {{&S98211[78], 5}, {NULL}};
const Segment SGI18112[] = {{&S98211[136], 13}, {NULL}};
const Segment SGI18113[] = {{&S97241[50], 8}, {NULL}};
const Segment SGI18114[] = {{&S97241[50], 19}, {NULL}};
const Segment SGI18115[] = {{&S97241[30], 12}, {NULL}};
const Segment SGI18116[] = {{&S97241[118], 3}, {NULL}};
const Segment SGI18117[] = {{&S100056[44], 10}, {NULL}};
const Segment SGI18118[] = {{&S97241[83], 6}, {NULL}};
const Segment SGI18119[] = {{&S100056[44], 21}, {NULL}};
const Segment SGI18120[] = {{&S100056[30], 6}, {NULL}};
const Segment SGI18121[] = {{&S100056[114], 3}, {NULL}};
const Segment SGI18122[] = {{&S100056[79], 6}, {NULL}};
const Segment SGI18123[] = {{&S100056[134], 8}, {NULL}};
const Segment SGI18124[] = {{&S100056[143], 13}, {NULL}};
const Segment SGI18125[] = {{&S97242[50], 11}, {NULL}};
const Segment SGI18126[] = {{&S97242[50], 22}, {NULL}};
const Segment SGI18127[] = {{&S97242[30], 12}, {NULL}};
const Segment SGI18128[] = {{&S97242[121], 3}, {NULL}};
const Segment SGI18129[] = {{&S97242[86], 6}, {NULL}};
const Segment SGI18130[] = {{&S97302[51], 6}, {NULL}};
const Segment SGI18131[] = {{&S97302[30], 12}, {NULL}};
const Segment SGI18132[] = {{&S97302[122], 3}, {NULL}};
const Segment SGI18133[] = {{&S97302[88], 5}, {NULL}};
const Segment SGI18134[] = {{&S97302[146], 13}, {NULL}};
const Segment SGI18135[] = {{&S98388[42], 10}, {NULL}};
const Segment SGI18136[] = {{&S98388[42], 21}, {NULL}};
const Segment SGI18137[] = {{&S98388[30], 4}, {NULL}};
const Segment SGI18138[] = {{&S98388[77], 6}, {NULL}};
const Segment SGI18139[] = {{&S98388[104], 13}, {NULL}};
const Segment SGI18140[] = {{&S98591[46], 7}, {NULL}};
const Segment SGI18141[] = {{&S98591[46], 20}, {NULL}};
const Segment SGI18142[] = {{&S98591[30], 8}, {NULL}};
const Segment SGI18143[] = {{&S98591[115], 3}, {NULL}};
const Segment SGI18144[] = {{&S98591[80], 6}, {NULL}};
const Segment SGI18145[] = {{&S98591[135], 8}, {NULL}};
const Segment SGI18146[] = {{&S98640[59], 8}, {&S98640[59], 8}, {&S98640[59], 8}, {NULL}};
const Segment SGI18147[] = {{&S98640[0], 13}, {NULL}};
const Segment SGI18148[] = {{&S98641[62], 8}, {&S98641[62], 8}, {NULL}};
const Segment SGI18149[] = {{&S98641[0], 13}, {NULL}};
const Segment SGI18150[] = {{&S100134[44], 9}, {NULL}};
const Segment SGI18151[] = {{&S100134[44], 20}, {NULL}};
const Segment SGI18152[] = {{&S100134[30], 6}, {NULL}};
const Segment SGI18153[] = {{&S100134[112], 3}, {NULL}};
const Segment SGI18154[] = {{&S100134[78], 5}, {NULL}};
const Segment SGI18155[] = {{&S98642[62], 8}, {NULL}};
const Segment SGI18156[] = {{&S98642[0], 13}, {NULL}};
const Segment SGI18157[] = {{&S100165[44], 7}, {NULL}};
const Segment SGI18158[] = {{&S100165[44], 18}, {NULL}};
const Segment SGI18159[] = {{&S100165[30], 6}, {NULL}};
const Segment SGI18160[] = {{&S100165[110], 3}, {NULL}};
const Segment SGI18161[] = {{&S100165[76], 5}, {NULL}};
const Segment SGI18162[] = {{&S100166[44], 13}, {NULL}};
const Segment SGI18163[] = {{&S100166[44], 24}, {NULL}};
const Segment SGI18164[] = {{&S100166[30], 6}, {NULL}};
const Segment SGI18165[] = {{&S100166[116], 3}, {NULL}};
const Segment SGI18166[] = {{&S100166[82], 5}, {NULL}};
const Segment SGI18167[] = {{&S98524[42], 10}, {NULL}};
const Segment SGI18168[] = {{&S98524[42], 21}, {NULL}};
const Segment SGI18169[] = {{&S98524[30], 4}, {NULL}};
const Segment SGI18170[] = {{&S98524[111], 3}, {NULL}};
const Segment SGI18171[] = {{&S98524[77], 5}, {NULL}};
const Segment SGI18172[] = {{&S98525[42], 14}, {NULL}};
const Segment SGI18173[] = {{&S98525[42], 25}, {NULL}};
const Segment SGI18174[] = {{&S98525[30], 4}, {NULL}};
const Segment SGI18175[] = {{&S98525[115], 3}, {NULL}};
const Segment SGI18176[] = {{&S98525[81], 5}, {NULL}};
const Segment SGI18177[] = {{&S98589[42], 10}, {NULL}};
const Segment SGI18178[] = {{&S98589[42], 21}, {NULL}};
const Segment SGI18179[] = {{&S98589[30], 4}, {NULL}};
const Segment SGI18180[] = {{&S98589[111], 3}, {NULL}};
const Segment SGI18181[] = {{&S98589[77], 5}, {NULL}};
const Segment SGI18182[] = {{&S101651[42], 16}, {NULL}};
const Segment SGI18183[] = {{&S101651[42], 27}, {NULL}};
const Segment SGI18184[] = {{&S101651[30], 4}, {NULL}};
const Segment SGI18185[] = {{&S101651[118], 3}, {NULL}};
const Segment SGI18186[] = {{&S101651[83], 6}, {NULL}};
const Segment SGI18187[] = {{&S101651[138], 8}, {NULL}};
const Segment SGI18188[] = {{&S101652[42], 9}, {NULL}};
const Segment SGI18189[] = {{&S101652[42], 22}, {NULL}};
const Segment SGI18190[] = {{&S101652[30], 4}, {NULL}};
const Segment SGI18191[] = {{&S101652[113], 3}, {NULL}};
const Segment SGI18192[] = {{&S101652[78], 6}, {NULL}};
const Segment SGI18193[] = {{&S101652[133], 8}, {NULL}};
const Segment SGI18194[] = {{&S98681[57], 12}, {NULL}};
const Segment SGI18195[] = {{&S98681[57], 23}, {NULL}};
const Segment SGI18196[] = {{&S98681[45], 4}, {NULL}};
const Segment SGI18197[] = {{&S98681[128], 3}, {NULL}};
const Segment SGI18198[] = {{&S98681[94], 5}, {NULL}};
const Segment SGI18199[] = {{&S98681[0], 14}, {NULL}};
const Segment SGI18200[] = {{&S101723[42], 18}, {NULL}};
const Segment SGI18201[] = {{&S101723[42], 29}, {NULL}};
const Segment SGI18202[] = {{&S101723[30], 4}, {NULL}};
const Segment SGI18203[] = {{&S101723[120], 3}, {NULL}};
const Segment SGI18204[] = {{&S101723[85], 6}, {NULL}};
const Segment SGI18205[] = {{&S101723[140], 8}, {NULL}};
const Segment SGI18206[] = {{&S101724[6], 4}, {NULL}};
const Segment SGI18207[] = {{&S101724[80], 5}, {NULL}};
const Segment SGI18208[] = {{&S97579[50], 6}, {NULL}};
const Segment SGI18209[] = {{&S97579[50], 24}, {NULL}};
const Segment SGI18210[] = {{&S97579[30], 12}, {NULL}};
const Segment SGI18211[] = {{&S97579[123], 3}, {NULL}};
const Segment SGI18212[] = {{&S97579[88], 6}, {NULL}};
const Segment SGI18213[] = {{&S97579[143], 8}, {NULL}};
const Segment SGI18214[] = {{&S97579[152], 13}, {NULL}};
const Segment SGI18215[] = {{&S101813[42], 9}, {NULL}};
const Segment SGI18216[] = {{&S101813[42], 22}, {NULL}};
const Segment SGI18217[] = {{&S101813[30], 4}, {NULL}};
const Segment SGI18218[] = {{&S101813[113], 3}, {NULL}};
const Segment SGI18219[] = {{&S101813[78], 6}, {NULL}};
const Segment SGI18220[] = {{&S101813[133], 8}, {NULL}};
const Segment SGI18221[] = {{&S100512[59], 9}, {NULL}};
const Segment SGI18222[] = {{&S100512[59], 20}, {NULL}};
const Segment SGI18223[] = {{&S100512[45], 6}, {NULL}};
const Segment SGI18224[] = {{&S100512[127], 3}, {NULL}};
const Segment SGI18225[] = {{&S100512[93], 5}, {NULL}};
const Segment SGI18226[] = {{&S99011[69], 14}, {NULL}};
const Segment SGI18227[] = {{&S99011[69], 25}, {NULL}};
const Segment SGI18228[] = {{&S99011[30], 31}, {NULL}};
const Segment SGI18229[] = {{&S99011[142], 3}, {NULL}};
const Segment SGI18230[] = {{&S99011[108], 5}, {NULL}};
const Segment SGI18231[] = {{&S99012[70], 14}, {NULL}};
const Segment SGI18232[] = {{&S99012[70], 25}, {NULL}};
const Segment SGI18233[] = {{&S99012[30], 32}, {NULL}};
const Segment SGI18234[] = {{&S99012[143], 3}, {NULL}};
const Segment SGI18235[] = {{&S99012[109], 5}, {NULL}};
const Segment SGI18236[] = {{&S99013[70], 14}, {NULL}};
const Segment SGI18237[] = {{&S99013[70], 25}, {NULL}};
const Segment SGI18238[] = {{&S99013[30], 32}, {NULL}};
const Segment SGI18239[] = {{&S99013[143], 3}, {NULL}};
const Segment SGI18240[] = {{&S99013[109], 5}, {NULL}};
const Segment SGI18241[] = {{&S99014[51], 14}, {NULL}};
const Segment SGI18242[] = {{&S99014[51], 26}, {NULL}};
const Segment SGI18243[] = {{&S99014[30], 13}, {NULL}};
const Segment SGI18244[] = {{&S99014[125], 3}, {NULL}};
const Segment SGI18245[] = {{&S99014[91], 5}, {NULL}};
const Segment SGI18246[] = {{&S98874[42], 10}, {NULL}};
const Segment SGI18247[] = {{&S98874[42], 21}, {NULL}};
const Segment SGI18248[] = {{&S98874[30], 4}, {NULL}};
const Segment SGI18249[] = {{&S98874[77], 6}, {NULL}};
const Segment SGI18250[] = {{&S98874[104], 13}, {NULL}};
const Segment SGI18251[] = {{&S99032[51], 20}, {NULL}};
const Segment SGI18252[] = {{&S99032[51], 32}, {NULL}};
const Segment SGI18253[] = {{&S99032[30], 13}, {NULL}};
const Segment SGI18254[] = {{&S99032[131], 3}, {NULL}};
const Segment SGI18255[] = {{&S99032[97], 5}, {NULL}};
const Segment SGI18256[] = {{&S99033[80], 20}, {NULL}};
const Segment SGI18257[] = {{&S99033[80], 31}, {NULL}};
const Segment SGI18258[] = {{&S99033[30], 42}, {NULL}};
const Segment SGI18259[] = {{&S99033[160], 3}, {NULL}};
const Segment SGI18260[] = {{&S99033[125], 6}, {NULL}};
const Segment SGI18261[] = {{&S99034[67], 20}, {NULL}};
const Segment SGI18262[] = {{&S99034[67], 31}, {NULL}};
const Segment SGI18263[] = {{&S99034[30], 29}, {NULL}};
const Segment SGI18264[] = {{&S99034[147], 3}, {NULL}};
const Segment SGI18265[] = {{&S99034[112], 6}, {NULL}};
const Segment SGI18266[] = {{&S100577[44], 10}, {NULL}};
const Segment SGI18267[] = {{&S100577[44], 21}, {NULL}};
const Segment SGI18268[] = {{&S100577[30], 6}, {NULL}};
const Segment SGI18269[] = {{&S100577[79], 6}, {NULL}};
const Segment SGI18270[] = {{&S100577[106], 13}, {NULL}};
const Segment SGI18271[] = {{&S97770[51], 10}, {NULL}};
const Segment SGI18272[] = {{&S97770[30], 12}, {NULL}};
const Segment SGI18273[] = {{&S97770[121], 3}, {NULL}};
const Segment SGI18274[] = {{&S97770[87], 5}, {NULL}};
const Segment SGI18275[] = {{&S97770[145], 13}, {NULL}};
const Segment SGI18276[] = {{&S98933[50], 10}, {NULL}};
const Segment SGI18277[] = {{&S98933[50], 21}, {NULL}};
const Segment SGI18278[] = {{&S98933[38], 4}, {NULL}};
const Segment SGI18279[] = {{&S98933[85], 5}, {NULL}};
const Segment SGI18280[] = {{&S98933[0], 14}, {NULL}};
const Segment SGI18281[] = {{&S101859[29], 4}, {NULL}};
const Segment SGI18282[] = {{&S101859[48], 3}, {NULL}};
const Segment SGI18283[] = {{&S99141[13], 8}, {NULL}};
const Segment SGI18284[] = {{&S99147[13], 8}, {NULL}};
const Segment SGI18285[] = {{&S99147[22], 6}, {NULL}};
const Segment SGI18286[] = {{&S99148[13], 8}, {NULL}};
const Segment SGI18287[] = {{&S99154[62], 8}, {NULL}};
const Segment SGI18288[] = {{&S99154[0], 13}, {NULL}};
const Segment SGI18289[] = {{&S99155[59], 8}, {&S99155[59], 8}, {&S99155[59], 8}, {NULL}};
const Segment SGI18290[] = {{&S99155[0], 13}, {NULL}};
const Segment SGI18291[] = {{&S99156[62], 8}, {&S99156[62], 8}, {NULL}};
const Segment SGI18292[] = {{&S99156[0], 13}, {NULL}};
const Segment SGI18293[] = {{&S100676[44], 10}, {NULL}};
const Segment SGI18294[] = {{&S100676[44], 21}, {NULL}};
const Segment SGI18295[] = {{&S100676[30], 6}, {NULL}};
const Segment SGI18296[] = {{&S100676[79], 6}, {NULL}};
const Segment SGI18297[] = {{&S100676[106], 13}, {NULL}};
const Segment SGI18298[] = {{&S97891[50], 10}, {NULL}};
const Segment SGI18299[] = {{&S97891[50], 22}, {NULL}};
const Segment SGI18300[] = {{&S97891[30], 12}, {NULL}};
const Segment SGI18301[] = {{&S97891[86], 6}, {NULL}};
const Segment SGI18302[] = {{&S97891[113], 13}, {NULL}};
const Segment SGI18303[] = {{&S100733[44], 22}, {NULL}};
const Segment SGI18304[] = {{&S100733[30], 6}, {NULL}};
const Segment SGI18305[] = {{&S100733[116], 3}, {NULL}};
const Segment SGI18306[] = {{&S100733[82], 5}, {NULL}};
const Segment SGI18307[] = {{&S100735[44], 9}, {NULL}};
const Segment SGI18308[] = {{&S100735[44], 20}, {NULL}};
const Segment SGI18309[] = {{&S100735[30], 6}, {NULL}};
const Segment SGI18310[] = {{&S100735[81], 4}, {NULL}};
const Segment SGI18311[] = {{&S100735[81], 3}, {NULL}};
const Segment SGI18312[] = {{&S99265[80], 5}, {NULL}};
const Segment SGI18313[] = {{&S100760[44], 6}, {NULL}};
const Segment SGI18314[] = {{&S100760[44], 32}, {NULL}};
const Segment SGI18315[] = {{&S100760[30], 6}, {NULL}};
const Segment SGI18316[] = {{&S100760[124], 3}, {NULL}};
const Segment SGI18317[] = {{&S100760[90], 5}, {NULL}};
const Segment SGI18318[] = {{&S99149[60], 10}, {NULL}};
const Segment SGI18319[] = {{&S99149[55], 4}, {NULL}};
const Segment SGI18320[] = {{&S100761[44], 6}, {NULL}};
const Segment SGI18321[] = {{&S100761[44], 18}, {NULL}};
const Segment SGI18322[] = {{&S100761[30], 6}, {NULL}};
const Segment SGI18323[] = {{&S100761[110], 3}, {NULL}};
const Segment SGI18324[] = {{&S100761[76], 5}, {NULL}};
const Segment SGI18325[] = {{&S99150[60], 11}, {NULL}};
const Segment SGI18326[] = {{&S99150[55], 4}, {NULL}};
const Segment SGI18327[] = {{&S100763[44], 6}, {NULL}};
const Segment SGI18328[] = {{&S99151[60], 11}, {NULL}};
const Segment SGI18329[] = {{&S100763[44], 32}, {NULL}};
const Segment SGI18330[] = {{&S100763[30], 6}, {NULL}};
const Segment SGI18331[] = {{&S99151[55], 4}, {NULL}};
const Segment SGI18332[] = {{&S100763[124], 3}, {NULL}};
const Segment SGI18333[] = {{&S100763[90], 5}, {NULL}};
const Segment SGI18334[] = {{&S100764[44], 6}, {NULL}};
const Segment SGI18335[] = {{&S100764[44], 20}, {NULL}};
const Segment SGI18336[] = {{&S100764[30], 6}, {NULL}};
const Segment SGI18337[] = {{&S100764[112], 3}, {NULL}};
const Segment SGI18338[] = {{&S100764[78], 5}, {NULL}};
const Segment SGI18339[] = {{&S102062[42], 8}, {NULL}};
const Segment SGI18340[] = {{&S102062[42], 19}, {NULL}};
const Segment SGI18341[] = {{&S102062[30], 4}, {NULL}};
const Segment SGI18342[] = {{&S102062[110], 3}, {NULL}};
const Segment SGI18343[] = {{&S102062[75], 6}, {NULL}};
const Segment SGI18344[] = {{&S102062[130], 8}, {NULL}};
const Segment SGI18345[] = {{&S99177[43], 10}, {NULL}};
const Segment SGI18346[] = {{&S99177[30], 4}, {NULL}};
const Segment SGI18347[] = {{&S99177[112], 3}, {NULL}};
const Segment SGI18348[] = {{&S99177[78], 5}, {NULL}};
const Segment SGI18349[] = {{&S99177[136], 13}, {NULL}};
const Segment SGI18350[] = {{&S100793[44], 5}, {NULL}};
const Segment SGI18351[] = {{&S100793[44], 16}, {NULL}};
const Segment SGI18352[] = {{&S100793[30], 6}, {NULL}};
const Segment SGI18353[] = {{&S100793[108], 3}, {NULL}};
const Segment SGI18354[] = {{&S100793[74], 5}, {NULL}};
const Segment SGI18355[] = {{&S97992[50], 6}, {NULL}};
const Segment SGI18356[] = {{&S97992[50], 23}, {NULL}};
const Segment SGI18357[] = {{&S97992[30], 12}, {NULL}};
const Segment SGI18358[] = {{&S97992[122], 3}, {NULL}};
const Segment SGI18359[] = {{&S97992[87], 6}, {NULL}};
const Segment SGI18360[] = {{&S97992[142], 8}, {NULL}};
const Segment SGI18361[] = {{&S97992[151], 13}, {NULL}};
const Segment SGI18362[] = {{&S100884[59], 10}, {NULL}};
const Segment SGI18363[] = {{&S100884[59], 21}, {NULL}};
const Segment SGI18364[] = {{&S100884[45], 6}, {NULL}};
const Segment SGI18365[] = {{&S100884[128], 3}, {NULL}};
const Segment SGI18366[] = {{&S100884[94], 5}, {NULL}};
const Segment SGI18367[] = {{&S100884[0], 14}, {NULL}};
const Segment SGI18368[] = {{&S102152[55], 12}, {&S102152[55], 12}, {NULL}};
const Segment SGI18369[] = {{&S102152[55], 23}, {NULL}};
const Segment SGI18370[] = {{&S102152[30], 4}, {NULL}};
const Segment SGI18371[] = {{&S102152[151], 3}, {NULL}};
const Segment SGI18372[] = {{&S102152[116], 6}, {NULL}};
const Segment SGI18373[] = {{&S102152[171], 8}, {NULL}};
const Segment SGI18374[] = {{&S99571[57], 9}, {NULL}};
const Segment SGI18375[] = {{&S99571[57], 20}, {NULL}};
const Segment SGI18376[] = {{&S99571[30], 19}, {NULL}};
const Segment SGI18377[] = {{&S99571[126], 3}, {NULL}};
const Segment SGI18378[] = {{&S99571[91], 6}, {NULL}};
const Segment SGI18379[] = {{&S99572[57], 9}, {NULL}};
const Segment SGI18380[] = {{&S99572[57], 20}, {NULL}};
const Segment SGI18381[] = {{&S99572[30], 19}, {NULL}};
const Segment SGI18382[] = {{&S99572[126], 3}, {NULL}};
const Segment SGI18383[] = {{&S99572[91], 6}, {NULL}};
const Segment SGI18384[] = {{&S101063[44], 10}, {NULL}};
const Segment SGI18385[] = {{&S101063[44], 21}, {NULL}};
const Segment SGI18386[] = {{&S101063[30], 6}, {NULL}};
const Segment SGI18387[] = {{&S101063[120], 3}, {NULL}};
const Segment SGI18388[] = {{&S101063[86], 5}, {NULL}};
const Segment SGI18389[] = {{&S101063[66], 7}, {NULL}};
const Segment SGI18390[] = {{&S102349[42], 8}, {NULL}};
const Segment SGI18391[] = {{&S102349[42], 19}, {NULL}};
const Segment SGI18392[] = {{&S102349[30], 4}, {NULL}};
const Segment SGI18393[] = {{&S102349[110], 3}, {NULL}};
const Segment SGI18394[] = {{&S102349[75], 6}, {NULL}};
const Segment SGI18395[] = {{&S102349[130], 8}, {NULL}};
const Segment SGI18396[] = {{&S99695[13], 8}, {NULL}};
const Segment SGI18397[] = {{&S99696[13], 8}, {NULL}};
const Segment SGI18398[] = {{&S102363[42], 8}, {NULL}};
const Segment SGI18399[] = {{&S102363[42], 19}, {NULL}};
const Segment SGI18400[] = {{&S102363[30], 4}, {NULL}};
const Segment SGI18401[] = {{&S102363[110], 3}, {NULL}};
const Segment SGI18402[] = {{&S102363[75], 6}, {NULL}};
const Segment SGI18403[] = {{&S102363[130], 8}, {NULL}};
const Segment SGI18404[] = {{&S99697[22], 8}, {NULL}};
const Segment SGI18405[] = {{&S99698[60], 8}, {&S99698[60], 8}, {NULL}};
const Segment SGI18406[] = {{&S99699[31], 3}, {NULL}};
const Segment SGI18407[] = {{&S99699[43], 8}, {NULL}};
const Segment SGI18408[] = {{&S99700[76], 8}, {NULL}};
const Segment SGI18409[] = {{&S99705[60], 8}, {&S99705[60], 8}, {NULL}};
const Segment SGI18410[] = {{&S99705[0], 13}, {NULL}};
const Segment SGI18411[] = {{&S102393[44], 9}, {NULL}};
const Segment SGI18412[] = {{&S102393[44], 21}, {NULL}};
const Segment SGI18413[] = {{&S102393[30], 6}, {NULL}};
const Segment SGI18414[] = {{&S102393[114], 3}, {NULL}};
const Segment SGI18415[] = {{&S102393[79], 6}, {NULL}};
const Segment SGI18416[] = {{&S99397[42], 7}, {NULL}};
const Segment SGI18417[] = {{&S99397[42], 18}, {NULL}};
const Segment SGI18418[] = {{&S99397[30], 4}, {NULL}};
const Segment SGI18419[] = {{&S99397[74], 6}, {NULL}};
const Segment SGI18420[] = {{&S99397[101], 13}, {NULL}};
const Segment SGI18421[] = {{&S101258[44], 10}, {NULL}};
const Segment SGI18422[] = {{&S101258[44], 21}, {NULL}};
const Segment SGI18423[] = {{&S101258[30], 6}, {NULL}};
const Segment SGI18424[] = {{&S101258[112], 3}, {NULL}};
const Segment SGI18425[] = {{&S101258[79], 5}, {NULL}};
const Segment SGI18426[] = {{&S101258[116], 14}, {NULL}};
const Segment SGI18427[] = {{&S102471[29], 4}, {NULL}};
const Segment SGI18428[] = {{&S102471[48], 3}, {NULL}};
const Segment SGI18429[] = {{&S99757[44], 14}, {NULL}};
const Segment SGI18430[] = {{&S99757[44], 26}, {NULL}};
const Segment SGI18431[] = {{&S99757[84], 5}, {NULL}};
const Segment SGI18432[] = {{&S102552[42], 6}, {NULL}};
const Segment SGI18433[] = {{&S102552[42], 17}, {NULL}};
const Segment SGI18434[] = {{&S102552[30], 4}, {NULL}};
const Segment SGI18435[] = {{&S102552[108], 3}, {NULL}};
const Segment SGI18436[] = {{&S102552[73], 6}, {NULL}};
const Segment SGI18437[] = {{&S102552[128], 8}, {NULL}};
const Segment SGI18438[] = {{&S102554[42], 13}, {NULL}};
const Segment SGI18439[] = {{&S102554[42], 24}, {NULL}};
const Segment SGI18440[] = {{&S102554[30], 4}, {NULL}};
const Segment SGI18441[] = {{&S102554[115], 3}, {NULL}};
const Segment SGI18442[] = {{&S102554[80], 6}, {NULL}};
const Segment SGI18443[] = {{&S102554[135], 8}, {NULL}};
const Segment SGI18444[] = {{&S102555[42], 13}, {NULL}};
const Segment SGI18445[] = {{&S102555[42], 24}, {NULL}};
const Segment SGI18446[] = {{&S102555[30], 4}, {NULL}};
const Segment SGI18447[] = {{&S102555[116], 3}, {NULL}};
const Segment SGI18448[] = {{&S102555[80], 7}, {NULL}};
const Segment SGI18449[] = {{&S102555[136], 8}, {NULL}};
const Segment SGI18450[] = {{&S98631[65], 10}, {NULL}};
const Segment SGI18451[] = {{&S98631[65], 21}, {NULL}};
const Segment SGI18452[] = {{&S98631[45], 12}, {NULL}};
const Segment SGI18453[] = {{&S98631[135], 3}, {NULL}};
const Segment SGI18454[] = {{&S98631[100], 6}, {NULL}};
const Segment SGI18455[] = {{&S101511[44], 9}, {NULL}};
const Segment SGI18456[] = {{&S101511[44], 20}, {NULL}};
const Segment SGI18457[] = {{&S101511[30], 6}, {NULL}};
const Segment SGI18458[] = {{&S101511[112], 3}, {NULL}};
const Segment SGI18459[] = {{&S101511[78], 5}, {NULL}};
const Segment SGI18460[] = {{&S98699[51], 6}, {NULL}};
const Segment SGI18461[] = {{&S98699[30], 12}, {NULL}};
const Segment SGI18462[] = {{&S98699[122], 3}, {NULL}};
const Segment SGI18463[] = {{&S98699[88], 5}, {NULL}};
const Segment SGI18464[] = {{&S98699[146], 13}, {NULL}};
const Segment SGI18465[] = {{&S98707[65], 6}, {NULL}};
const Segment SGI18466[] = {{&S98707[65], 23}, {NULL}};
const Segment SGI18467[] = {{&S98707[45], 12}, {NULL}};
const Segment SGI18468[] = {{&S98707[137], 3}, {NULL}};
const Segment SGI18469[] = {{&S98707[102], 6}, {NULL}};
const Segment SGI18470[] = {{&S98718[51], 14}, {NULL}};
const Segment SGI18471[] = {{&S98718[30], 12}, {NULL}};
const Segment SGI18472[] = {{&S98718[124], 3}, {NULL}};
const Segment SGI18473[] = {{&S98718[90], 5}, {NULL}};
const Segment SGI18474[] = {{&S98718[148], 13}, {NULL}};
const Segment SGI18475[] = {{&S102793[42], 106}, {NULL}};
const Segment SGI18476[] = {{&S102793[42], 117}, {NULL}};
const Segment SGI18477[] = {{&S102793[30], 4}, {NULL}};
const Segment SGI18478[] = {{&S102793[208], 3}, {NULL}};
const Segment SGI18479[] = {{&S102793[173], 6}, {NULL}};
const Segment SGI18480[] = {{&S102793[228], 8}, {NULL}};
const Segment SGI18481[] = {{&S99672[43], 7}, {NULL}};
const Segment SGI18482[] = {{&S99672[30], 4}, {NULL}};
const Segment SGI18483[] = {{&S99672[109], 3}, {NULL}};
const Segment SGI18484[] = {{&S99672[75], 5}, {NULL}};
const Segment SGI18485[] = {{&S99672[133], 13}, {NULL}};
const Segment SGI18486[] = {{&S101727[57], 10}, {NULL}};
const Segment SGI18487[] = {{&S101727[57], 21}, {NULL}};
const Segment SGI18488[] = {{&S101727[43], 6}, {NULL}};
const Segment SGI18489[] = {{&S101727[126], 3}, {NULL}};
const Segment SGI18490[] = {{&S101727[92], 5}, {NULL}};
const Segment SGI18491[] = {{&S101727[0], 12}, {NULL}};
const Segment SGI18492[] = {{&S101764[45], 10}, {NULL}};
const Segment SGI18493[] = {{&S101764[30], 6}, {NULL}};
const Segment SGI18494[] = {{&S101764[114], 3}, {NULL}};
const Segment SGI18495[] = {{&S101764[80], 5}, {NULL}};
const Segment SGI18496[] = {{&S101764[138], 13}, {NULL}};
const Segment SGI18497[] = {{&S99767[57], 7}, {NULL}};
const Segment SGI18498[] = {{&S99767[57], 18}, {NULL}};
const Segment SGI18499[] = {{&S99767[45], 4}, {NULL}};
const Segment SGI18500[] = {{&S99767[123], 3}, {NULL}};
const Segment SGI18501[] = {{&S99767[89], 5}, {NULL}};
const Segment SGI18502[] = {{&S99767[0], 14}, {NULL}};
const Segment SGI18503[] = {{&S100024[0], 13}, {NULL}};
const Segment SGI18504[] = {{&S99792[43], 7}, {NULL}};
const Segment SGI18505[] = {{&S99792[30], 4}, {NULL}};
const Segment SGI18506[] = {{&S99792[109], 3}, {NULL}};
const Segment SGI18507[] = {{&S99792[75], 5}, {NULL}};
const Segment SGI18508[] = {{&S99792[133], 13}, {NULL}};
const Segment SGI18509[] = {{&S101823[44], 16}, {NULL}};
const Segment SGI18510[] = {{&S101823[44], 27}, {NULL}};
const Segment SGI18511[] = {{&S101823[30], 6}, {NULL}};
const Segment SGI18512[] = {{&S101823[119], 3}, {NULL}};
const Segment SGI18513[] = {{&S101823[85], 5}, {NULL}};
const Segment SGI18514[] = {{&S102962[42], 9}, {NULL}};
const Segment SGI18515[] = {{&S102962[42], 20}, {NULL}};
const Segment SGI18516[] = {{&S102962[30], 4}, {NULL}};
const Segment SGI18517[] = {{&S102962[111], 3}, {NULL}};
const Segment SGI18518[] = {{&S102962[76], 6}, {NULL}};
const Segment SGI18519[] = {{&S102962[131], 8}, {NULL}};
const Segment SGI18520[] = {{&S101862[44], 10}, {NULL}};
const Segment SGI18521[] = {{&S101862[44], 22}, {NULL}};
const Segment SGI18522[] = {{&S101862[30], 6}, {NULL}};
const Segment SGI18523[] = {{&S101862[114], 3}, {NULL}};
const Segment SGI18524[] = {{&S101862[80], 5}, {NULL}};
const Segment SGI18525[] = {{&S100109[55], 11}, {NULL}};
const Segment SGI18526[] = {{&S100109[55], 22}, {NULL}};
const Segment SGI18527[] = {{&S100109[30], 17}, {NULL}};
const Segment SGI18528[] = {{&S100109[126], 3}, {NULL}};
const Segment SGI18529[] = {{&S100109[91], 6}, {NULL}};
const Segment SGI18530[] = {{&S103012[42], 11}, {NULL}};
const Segment SGI18531[] = {{&S103012[42], 22}, {NULL}};
const Segment SGI18532[] = {{&S103012[30], 4}, {NULL}};
const Segment SGI18533[] = {{&S103012[113], 3}, {NULL}};
const Segment SGI18534[] = {{&S103012[78], 6}, {NULL}};
const Segment SGI18535[] = {{&S103012[133], 8}, {NULL}};
const Segment SGI18536[] = {{&S100131[56], 9}, {NULL}};
const Segment SGI18537[] = {{&S100131[56], 20}, {NULL}};
const Segment SGI18538[] = {{&S100131[30], 18}, {NULL}};
const Segment SGI18539[] = {{&S100131[124], 3}, {NULL}};
const Segment SGI18540[] = {{&S100131[90], 5}, {NULL}};
const Segment SGI18541[] = {{&S101902[45], 10}, {NULL}};
const Segment SGI18542[] = {{&S101902[30], 6}, {NULL}};
const Segment SGI18543[] = {{&S101902[114], 3}, {NULL}};
const Segment SGI18544[] = {{&S101902[80], 5}, {NULL}};
const Segment SGI18545[] = {{&S101902[138], 13}, {NULL}};
const Segment SGI18546[] = {{&S99002[42], 12}, {NULL}};
const Segment SGI18547[] = {{&S99002[42], 22}, {NULL}};
const Segment SGI18548[] = {{&S99002[30], 4}, {NULL}};
const Segment SGI18549[] = {{&S99002[113], 3}, {NULL}};
const Segment SGI18550[] = {{&S99002[78], 6}, {NULL}};
const Segment SGI18551[] = {{&S99003[42], 12}, {NULL}};
const Segment SGI18552[] = {{&S99003[42], 24}, {NULL}};
const Segment SGI18553[] = {{&S99003[30], 4}, {NULL}};
const Segment SGI18554[] = {{&S99003[115], 3}, {NULL}};
const Segment SGI18555[] = {{&S99003[80], 6}, {NULL}};
const Segment SGI18556[] = {{&S99004[42], 12}, {NULL}};
const Segment SGI18557[] = {{&S99004[42], 23}, {NULL}};
const Segment SGI18558[] = {{&S99004[30], 4}, {NULL}};
const Segment SGI18559[] = {{&S99004[114], 3}, {NULL}};
const Segment SGI18560[] = {{&S99004[79], 6}, {NULL}};
const Segment SGI18561[] = {{&S99005[42], 12}, {NULL}};
const Segment SGI18562[] = {{&S99005[42], 23}, {NULL}};
const Segment SGI18563[] = {{&S99005[30], 4}, {NULL}};
const Segment SGI18564[] = {{&S99005[114], 3}, {NULL}};
const Segment SGI18565[] = {{&S99005[79], 6}, {NULL}};
const Segment SGI18566[] = {{&S99006[50], 12}, {NULL}};
const Segment SGI18567[] = {{&S99006[50], 23}, {NULL}};
const Segment SGI18568[] = {{&S99006[30], 12}, {NULL}};
const Segment SGI18569[] = {{&S99006[122], 3}, {NULL}};
const Segment SGI18570[] = {{&S99006[87], 6}, {NULL}};
const Segment SGI18571[] = {{&S100276[25], 8}, {NULL}};
const Segment SGI18572[] = {{&S99022[50], 28}, {NULL}};
const Segment SGI18573[] = {{&S99022[50], 39}, {NULL}};
const Segment SGI18574[] = {{&S99022[30], 12}, {NULL}};
const Segment SGI18575[] = {{&S99022[138], 3}, {NULL}};
const Segment SGI18576[] = {{&S99022[103], 6}, {NULL}};
const Segment SGI18577[] = {{&S100280[47], 8}, {NULL}};
const Segment SGI18578[] = {{&S99068[37], 8}, {NULL}};
const Segment SGI18579[] = {{&S102066[31], 12}, {NULL}};
const Segment SGI18580[] = {{&S99068[60], 3}, {NULL}};
const Segment SGI18581[] = {{&S102066[58], 3}, {NULL}};
const Segment SGI18582[] = {{&S100375[0], 13}, {NULL}};
const Segment SGI18583[] = {{&S99133[65], 6}, {NULL}};
const Segment SGI18584[] = {{&S99133[65], 24}, {NULL}};
const Segment SGI18585[] = {{&S99133[45], 12}, {NULL}};
const Segment SGI18586[] = {{&S99133[138], 3}, {NULL}};
const Segment SGI18587[] = {{&S99133[103], 6}, {NULL}};
const Segment SGI18588[] = {{&S102258[57], 14}, {NULL}};
const Segment SGI18589[] = {{&S102258[57], 26}, {NULL}};
const Segment SGI18590[] = {{&S102258[43], 6}, {NULL}};
const Segment SGI18591[] = {{&S102258[131], 3}, {NULL}};
const Segment SGI18592[] = {{&S102258[97], 5}, {NULL}};
const Segment SGI18593[] = {{&S102258[0], 12}, {NULL}};
const Segment SGI18594[] = {{&S99213[65], 10}, {NULL}};
const Segment SGI18595[] = {{&S99213[65], 22}, {NULL}};
const Segment SGI18596[] = {{&S99213[45], 12}, {NULL}};
const Segment SGI18597[] = {{&S99213[136], 3}, {NULL}};
const Segment SGI18598[] = {{&S99213[101], 6}, {NULL}};
const Segment SGI18599[] = {{&S102260[45], 14}, {NULL}};
const Segment SGI18600[] = {{&S102260[30], 6}, {NULL}};
const Segment SGI18601[] = {{&S102260[119], 3}, {NULL}};
const Segment SGI18602[] = {{&S102260[85], 5}, {NULL}};
const Segment SGI18603[] = {{&S102260[143], 13}, {NULL}};
const Segment SGI18604[] = {{&S102281[45], 10}, {NULL}};
const Segment SGI18605[] = {{&S102281[30], 6}, {NULL}};
const Segment SGI18606[] = {{&S102281[114], 3}, {NULL}};
const Segment SGI18607[] = {{&S102281[80], 5}, {NULL}};
const Segment SGI18608[] = {{&S102281[138], 13}, {NULL}};
const Segment SGI18609[] = {{&S100662[0], 13}, {NULL}};
const Segment SGI18610[] = {{&S103630[42], 11}, {NULL}};
const Segment SGI18611[] = {{&S103630[42], 22}, {NULL}};
const Segment SGI18612[] = {{&S103630[30], 4}, {NULL}};
const Segment SGI18613[] = {{&S103630[113], 3}, {NULL}};
const Segment SGI18614[] = {{&S103630[78], 6}, {NULL}};
const Segment SGI18615[] = {{&S103630[133], 8}, {NULL}};
const Segment SGI18616[] = {{&S100413[43], 18}, {NULL}};
const Segment SGI18617[] = {{&S100413[30], 4}, {NULL}};
const Segment SGI18618[] = {{&S100413[120], 3}, {NULL}};
const Segment SGI18619[] = {{&S100413[86], 5}, {NULL}};
const Segment SGI18620[] = {{&S100413[144], 13}, {NULL}};
const Segment SGI18621[] = {{&S100738[54], 8}, {NULL}};
const Segment SGI18622[] = {{&S100738[54], 17}, {NULL}};
const Segment SGI18623[] = {{&S100738[30], 16}, {NULL}};
const Segment SGI18624[] = {{&S100738[119], 3}, {NULL}};
const Segment SGI18625[] = {{&S100738[85], 5}, {NULL}};
const Segment SGI18626[] = {{&S100739[53], 8}, {NULL}};
const Segment SGI18627[] = {{&S100739[53], 18}, {NULL}};
const Segment SGI18628[] = {{&S100739[30], 15}, {NULL}};
const Segment SGI18629[] = {{&S100739[119], 3}, {NULL}};
const Segment SGI18630[] = {{&S100739[85], 5}, {NULL}};
const Segment SGI18631[] = {{&S100858[74], 8}, {NULL}};
const Segment SGI18632[] = {{&S100861[62], 8}, {NULL}};
const Segment SGI18633[] = {{&S100861[0], 13}, {NULL}};
const Segment SGI18634[] = {{&S103872[42], 11}, {NULL}};
const Segment SGI18635[] = {{&S103872[42], 24}, {NULL}};
const Segment SGI18636[] = {{&S103872[30], 4}, {NULL}};
const Segment SGI18637[] = {{&S103872[115], 3}, {NULL}};
const Segment SGI18638[] = {{&S103872[80], 6}, {NULL}};
const Segment SGI18639[] = {{&S103872[135], 8}, {NULL}};
const Segment SGI18640[] = {{&S102448[56], 10}, {NULL}};
const Segment SGI18641[] = {{&S102448[56], 21}, {NULL}};
const Segment SGI18642[] = {{&S102448[42], 6}, {NULL}};
const Segment SGI18643[] = {{&S102448[125], 3}, {NULL}};
const Segment SGI18644[] = {{&S102448[91], 5}, {NULL}};
const Segment SGI18645[] = {{&S102452[45], 10}, {NULL}};
const Segment SGI18646[] = {{&S102452[30], 6}, {NULL}};
const Segment SGI18647[] = {{&S102452[114], 3}, {NULL}};
const Segment SGI18648[] = {{&S102452[80], 5}, {NULL}};
const Segment SGI18649[] = {{&S102452[138], 13}, {NULL}};
const Segment SGI18650[] = {{&S102478[45], 9}, {NULL}};
const Segment SGI18651[] = {{&S102478[30], 6}, {NULL}};
const Segment SGI18652[] = {{&S102478[113], 3}, {NULL}};
const Segment SGI18653[] = {{&S102478[79], 5}, {NULL}};
const Segment SGI18654[] = {{&S102478[137], 13}, {NULL}};
const Segment SGI18655[] = {{&S103950[42], 8}, {NULL}};
const Segment SGI18656[] = {{&S103950[42], 19}, {NULL}};
const Segment SGI18657[] = {{&S103950[30], 4}, {NULL}};
const Segment SGI18658[] = {{&S103950[110], 3}, {NULL}};
const Segment SGI18659[] = {{&S103950[75], 6}, {NULL}};
const Segment SGI18660[] = {{&S103950[130], 8}, {NULL}};
const Segment SGI18661[] = {{&S102499[44], 10}, {NULL}};
const Segment SGI18662[] = {{&S102499[44], 21}, {NULL}};
const Segment SGI18663[] = {{&S102499[30], 6}, {NULL}};
const Segment SGI18664[] = {{&S102499[113], 3}, {NULL}};
const Segment SGI18665[] = {{&S102499[79], 5}, {NULL}};
const Segment SGI18666[] = {{&S102500[38], 10}, {NULL}};
const Segment SGI18667[] = {{&S102500[31], 6}, {NULL}};
const Segment SGI18668[] = {{&S100734[42], 101}, {NULL}};
const Segment SGI18669[] = {{&S100734[42], 112}, {NULL}};
const Segment SGI18670[] = {{&S100734[30], 4}, {NULL}};
const Segment SGI18671[] = {{&S100734[202], 3}, {NULL}};
const Segment SGI18672[] = {{&S100734[168], 5}, {NULL}};
const Segment SGI18673[] = {{&S100779[42], 11}, {NULL}};
const Segment SGI18674[] = {{&S100779[42], 23}, {NULL}};
const Segment SGI18675[] = {{&S100779[30], 4}, {NULL}};
const Segment SGI18676[] = {{&S100779[113], 3}, {NULL}};
const Segment SGI18677[] = {{&S100779[79], 5}, {NULL}};
const Segment SGI18678[] = {{&S101067[62], 8}, {NULL}};
const Segment SGI18679[] = {{&S101067[0], 13}, {NULL}};
const Segment SGI18680[] = {{&S104000[29], 19}, {NULL}};
const Segment SGI18681[] = {{&S104000[63], 3}, {NULL}};
const Segment SGI18682[] = {{&S101068[60], 8}, {&S101068[60], 8}, {NULL}};
const Segment SGI18683[] = {{&S101068[0], 13}, {NULL}};
const Segment SGI18684[] = {{&S101071[53], 8}, {&S101071[53], 8}, {NULL}};
const Segment SGI18685[] = {{&S101072[36], 8}, {NULL}};
const Segment SGI18686[] = {{&S101073[60], 8}, {&S101073[60], 8}, {NULL}};
const Segment SGI18687[] = {{&S101073[0], 13}, {NULL}};
const Segment SGI18688[] = {{&S99763[51], 7}, {NULL}};
const Segment SGI18689[] = {{&S99763[30], 12}, {NULL}};
const Segment SGI18690[] = {{&S99763[117], 3}, {NULL}};
const Segment SGI18691[] = {{&S99763[83], 5}, {NULL}};
const Segment SGI18692[] = {{&S99763[141], 13}, {NULL}};
const Segment SGI18693[] = {{&S102665[44], 6}, {NULL}};
const Segment SGI18694[] = {{&S102665[44], 17}, {NULL}};
const Segment SGI18695[] = {{&S102665[30], 6}, {NULL}};
const Segment SGI18696[] = {{&S102665[109], 3}, {NULL}};
const Segment SGI18697[] = {{&S102665[75], 5}, {NULL}};
const Segment SGI18698[] = {{&S102692[59], 10}, {NULL}};
const Segment SGI18699[] = {{&S102692[59], 21}, {NULL}};
const Segment SGI18700[] = {{&S102692[45], 6}, {NULL}};
const Segment SGI18701[] = {{&S102692[128], 3}, {NULL}};
const Segment SGI18702[] = {{&S102692[94], 5}, {NULL}};
const Segment SGI18703[] = {{&S102692[0], 14}, {NULL}};
const Segment SGI18704[] = {{&S102716[45], 18}, {NULL}};
const Segment SGI18705[] = {{&S102716[30], 6}, {NULL}};
const Segment SGI18706[] = {{&S102716[122], 3}, {NULL}};
const Segment SGI18707[] = {{&S102716[88], 5}, {NULL}};
const Segment SGI18708[] = {{&S102716[146], 13}, {NULL}};
const Segment SGI18709[] = {{&S100799[55], 10}, {NULL}};
const Segment SGI18710[] = {{&S100799[55], 21}, {NULL}};
const Segment SGI18711[] = {{&S100799[43], 4}, {NULL}};
const Segment SGI18712[] = {{&S100799[124], 3}, {NULL}};
const Segment SGI18713[] = {{&S100799[90], 5}, {NULL}};
const Segment SGI18714[] = {{&S100799[0], 10}, {NULL}};
const Segment SGI18715[] = {{&S100800[55], 11}, {NULL}};
const Segment SGI18716[] = {{&S100800[55], 22}, {NULL}};
const Segment SGI18717[] = {{&S100800[43], 4}, {NULL}};
const Segment SGI18718[] = {{&S100800[125], 3}, {NULL}};
const Segment SGI18719[] = {{&S100800[91], 5}, {NULL}};
const Segment SGI18720[] = {{&S100800[0], 10}, {NULL}};
const Segment SGI18721[] = {{&S101244[59], 8}, {&S101244[59], 8}, {NULL}};
const Segment SGI18722[] = {{&S101244[0], 13}, {NULL}};
const Segment SGI18723[] = {{&S101245[64], 8}, {NULL}};
const Segment SGI18724[] = {{&S101246[23], 8}, {NULL}};
const Segment SGI18725[] = {{&S101247[53], 8}, {&S101247[53], 8}, {&S101247[53], 8}, {NULL}};
const Segment SGI18726[] = {{&S101250[48], 10}, {NULL}};
const Segment SGI18727[] = {{&S101250[31], 16}, {NULL}};
const Segment SGI18728[] = {{&S101250[49], 8}, {NULL}};
const Segment SGI18729[] = {{&S101251[49], 10}, {NULL}};
const Segment SGI18730[] = {{&S101251[31], 17}, {NULL}};
const Segment SGI18731[] = {{&S101251[50], 8}, {NULL}};
const Segment SGI18732[] = {{&S101252[49], 10}, {NULL}};
const Segment SGI18733[] = {{&S101252[31], 17}, {NULL}};
const Segment SGI18734[] = {{&S101252[50], 8}, {NULL}};
const Segment SGI18735[] = {{&S101253[25], 8}, {NULL}};
const Segment SGI18736[] = {{&S104174[42], 106}, {NULL}};
const Segment SGI18737[] = {{&S104174[42], 117}, {NULL}};
const Segment SGI18738[] = {{&S104174[30], 4}, {NULL}};
const Segment SGI18739[] = {{&S104174[208], 3}, {NULL}};
const Segment SGI18740[] = {{&S104174[173], 6}, {NULL}};
const Segment SGI18741[] = {{&S104174[228], 8}, {NULL}};
const Segment SGI18742[] = {{&S101255[46], 8}, {NULL}};
const Segment SGI18743[] = {{&S102851[45], 10}, {NULL}};
const Segment SGI18744[] = {{&S102851[30], 6}, {NULL}};
const Segment SGI18745[] = {{&S102851[114], 3}, {NULL}};
const Segment SGI18746[] = {{&S101264[71], 5}, {NULL}};
const Segment SGI18747[] = {{&S102851[80], 5}, {NULL}};
const Segment SGI18748[] = {{&S102851[138], 13}, {NULL}};
const Segment SGI18749[] = {{&S102870[56], 10}, {NULL}};
const Segment SGI18750[] = {{&S102870[56], 21}, {NULL}};
const Segment SGI18751[] = {{&S102870[42], 6}, {NULL}};
const Segment SGI18752[] = {{&S102870[125], 3}, {NULL}};
const Segment SGI18753[] = {{&S102870[91], 5}, {NULL}};
const Segment SGI18754[] = {{&S102909[59], 10}, {NULL}};
const Segment SGI18755[] = {{&S102909[59], 21}, {NULL}};
const Segment SGI18756[] = {{&S102909[45], 6}, {NULL}};
const Segment SGI18757[] = {{&S102909[94], 5}, {NULL}};
const Segment SGI18758[] = {{&S102909[0], 14}, {NULL}};
const Segment SGI18759[] = {{&S102912[44], 10}, {NULL}};
const Segment SGI18760[] = {{&S102912[44], 21}, {NULL}};
const Segment SGI18761[] = {{&S102912[30], 6}, {NULL}};
const Segment SGI18762[] = {{&S102912[113], 3}, {NULL}};
const Segment SGI18763[] = {{&S102912[79], 5}, {NULL}};
const Segment SGI18764[] = {{&S102912[124], 14}, {NULL}};
const Segment SGI18765[] = {{&S102914[44], 10}, {NULL}};
const Segment SGI18766[] = {{&S102914[44], 21}, {NULL}};
const Segment SGI18767[] = {{&S102914[30], 6}, {NULL}};
const Segment SGI18768[] = {{&S102914[113], 3}, {NULL}};
const Segment SGI18769[] = {{&S102914[79], 5}, {NULL}};
const Segment SGI18770[] = {{&S102914[136], 10}, {NULL}};
const Segment SGI18771[] = {{&S100852[42], 11}, {NULL}};
const Segment SGI18772[] = {{&S100852[42], 22}, {NULL}};
const Segment SGI18773[] = {{&S100852[30], 4}, {NULL}};
const Segment SGI18774[] = {{&S100852[112], 3}, {NULL}};
const Segment SGI18775[] = {{&S100852[78], 5}, {NULL}};
const Segment SGI18776[] = {{&S100852[135], 10}, {NULL}};
const Segment SGI18777[] = {{&S100853[42], 10}, {NULL}};
const Segment SGI18778[] = {{&S100853[42], 21}, {NULL}};
const Segment SGI18779[] = {{&S100853[30], 4}, {NULL}};
const Segment SGI18780[] = {{&S100853[111], 3}, {NULL}};
const Segment SGI18781[] = {{&S100853[77], 5}, {NULL}};
const Segment SGI18782[] = {{&S100853[134], 10}, {NULL}};
const Segment SGI18783[] = {{&S100854[42], 11}, {NULL}};
const Segment SGI18784[] = {{&S100854[42], 22}, {NULL}};
const Segment SGI18785[] = {{&S100854[30], 4}, {NULL}};
const Segment SGI18786[] = {{&S100854[112], 3}, {NULL}};
const Segment SGI18787[] = {{&S100854[78], 5}, {NULL}};
const Segment SGI18788[] = {{&S100854[135], 10}, {NULL}};
const Segment SGI18789[] = {{&S101389[62], 8}, {&S101389[62], 8}, {NULL}};
const Segment SGI18790[] = {{&S101389[0], 13}, {NULL}};
const Segment SGI18791[] = {{&S101390[59], 8}, {&S101390[59], 8}, {&S101390[59], 8}, {NULL}};
const Segment SGI18792[] = {{&S101390[0], 13}, {NULL}};
const Segment SGI18793[] = {{&S101400[46], 8}, {NULL}};
const Segment SGI18794[] = {{&S101032[43], 10}, {NULL}};
const Segment SGI18795[] = {{&S101032[30], 4}, {NULL}};
const Segment SGI18796[] = {{&S101032[112], 3}, {NULL}};
const Segment SGI18797[] = {{&S101032[78], 5}, {NULL}};
const Segment SGI18798[] = {{&S101032[136], 13}, {NULL}};
const Segment SGI18799[] = {{&S100139[50], 11}, {NULL}};
const Segment SGI18800[] = {{&S100139[50], 22}, {NULL}};
const Segment SGI18801[] = {{&S100139[30], 12}, {NULL}};
const Segment SGI18802[] = {{&S100139[121], 3}, {NULL}};
const Segment SGI18803[] = {{&S100139[86], 6}, {NULL}};
const Segment SGI18804[] = {{&S101062[42], 11}, {NULL}};
const Segment SGI18805[] = {{&S101062[42], 22}, {NULL}};
const Segment SGI18806[] = {{&S101062[30], 4}, {NULL}};
const Segment SGI18807[] = {{&S101553[22], 8}, {NULL}};
const Segment SGI18808[] = {{&S101062[119], 3}, {NULL}};
const Segment SGI18809[] = {{&S101062[85], 5}, {NULL}};
const Segment SGI18810[] = {{&S101062[65], 7}, {NULL}};
const Segment SGI18811[] = {{&S101064[42], 10}, {NULL}};
const Segment SGI18812[] = {{&S101064[42], 21}, {NULL}};
const Segment SGI18813[] = {{&S101064[30], 4}, {NULL}};
const Segment SGI18814[] = {{&S101064[118], 3}, {NULL}};
const Segment SGI18815[] = {{&S101064[84], 5}, {NULL}};
const Segment SGI18816[] = {{&S101064[64], 7}, {NULL}};
const Segment SGI18817[] = {{&S101557[61], 8}, {NULL}};
const Segment SGI18818[] = {{&S101557[0], 13}, {NULL}};
const Segment SGI18819[] = {{&S101561[23], 8}, {NULL}};
const Segment SGI18820[] = {{&S101562[58], 8}, {NULL}};
const Segment SGI18821[] = {{&S101562[41], 9}, {NULL}};
const Segment SGI18822[] = {{&S101562[24], 8}, {NULL}};
const Segment SGI18823[] = {{&S101574[23], 8}, {NULL}};
const Segment SGI18824[] = {{&S103095[44], 7}, {NULL}};
const Segment SGI18825[] = {{&S103095[44], 19}, {NULL}};
const Segment SGI18826[] = {{&S103095[30], 6}, {NULL}};
const Segment SGI18827[] = {{&S103095[111], 3}, {NULL}};
const Segment SGI18828[] = {{&S103095[77], 5}, {NULL}};
const Segment SGI18829[] = {{&S101579[48], 8}, {NULL}};
const Segment SGI18830[] = {{&S101579[0], 12}, {NULL}};
const Segment SGI18831[] = {{&S103097[44], 8}, {NULL}};
const Segment SGI18832[] = {{&S103097[44], 20}, {NULL}};
const Segment SGI18833[] = {{&S103097[30], 6}, {NULL}};
const Segment SGI18834[] = {{&S103097[112], 3}, {NULL}};
const Segment SGI18835[] = {{&S103097[78], 5}, {NULL}};
const Segment SGI18836[] = {{&S103137[45], 10}, {NULL}};
const Segment SGI18837[] = {{&S103137[30], 6}, {NULL}};
const Segment SGI18838[] = {{&S103137[114], 3}, {NULL}};
const Segment SGI18839[] = {{&S103137[80], 5}, {NULL}};
const Segment SGI18840[] = {{&S103137[138], 13}, {NULL}};
const Segment SGI18841[] = {{&S101144[42], 15}, {NULL}};
const Segment SGI18842[] = {{&S101144[42], 26}, {NULL}};
const Segment SGI18843[] = {{&S101144[30], 4}, {NULL}};
const Segment SGI18844[] = {{&S101144[116], 3}, {NULL}};
const Segment SGI18845[] = {{&S101144[82], 5}, {NULL}};
const Segment SGI18846[] = {{&S103143[59], 10}, {NULL}};
const Segment SGI18847[] = {{&S103143[59], 21}, {NULL}};
const Segment SGI18848[] = {{&S103143[45], 6}, {NULL}};
const Segment SGI18849[] = {{&S103143[128], 3}, {NULL}};
const Segment SGI18850[] = {{&S103143[94], 5}, {NULL}};
const Segment SGI18851[] = {{&S103143[0], 14}, {NULL}};
const Segment SGI18852[] = {{&S103157[44], 10}, {NULL}};
const Segment SGI18853[] = {{&S103157[44], 21}, {NULL}};
const Segment SGI18854[] = {{&S103157[30], 6}, {NULL}};
const Segment SGI18855[] = {{&S103157[113], 3}, {NULL}};
const Segment SGI18856[] = {{&S103157[79], 5}, {NULL}};
const Segment SGI18857[] = {{&S103157[147], 3}, {NULL}};
const Segment SGI18858[] = {{&S101683[22], 6}, {NULL}};
const Segment SGI18859[] = {{&S101748[48], 8}, {&S101748[48], 8}, {NULL}};
const Segment SGI18860[] = {{&S101755[61], 8}, {NULL}};
const Segment SGI18861[] = {{&S101755[0], 13}, {NULL}};
const Segment SGI18862[] = {{&S101756[76], 8}, {NULL}};
const Segment SGI18863[] = {{&S101757[22], 8}, {NULL}};
const Segment SGI18864[] = {{&S101771[50], 8}, {NULL}};
const Segment SGI18865[] = {{&S101771[0], 12}, {NULL}};
const Segment SGI18866[] = {{&S101256[42], 10}, {NULL}};
const Segment SGI18867[] = {{&S101256[42], 21}, {NULL}};
const Segment SGI18868[] = {{&S101256[30], 4}, {NULL}};
const Segment SGI18869[] = {{&S101256[110], 3}, {NULL}};
const Segment SGI18870[] = {{&S101256[77], 5}, {NULL}};
const Segment SGI18871[] = {{&S101256[114], 14}, {NULL}};
const Segment SGI18872[] = {{&S101257[42], 10}, {NULL}};
const Segment SGI18873[] = {{&S101257[42], 21}, {NULL}};
const Segment SGI18874[] = {{&S101257[30], 4}, {NULL}};
const Segment SGI18875[] = {{&S101257[110], 3}, {NULL}};
const Segment SGI18876[] = {{&S101257[77], 5}, {NULL}};
const Segment SGI18877[] = {{&S101257[114], 14}, {NULL}};
const Segment SGI18878[] = {{&S104770[42], 6}, {NULL}};
const Segment SGI18879[] = {{&S104770[42], 19}, {NULL}};
const Segment SGI18880[] = {{&S104770[30], 4}, {NULL}};
const Segment SGI18881[] = {{&S104770[110], 3}, {NULL}};
const Segment SGI18882[] = {{&S104770[75], 6}, {NULL}};
const Segment SGI18883[] = {{&S104770[130], 8}, {NULL}};
const Segment SGI18884[] = {{&S103358[45], 10}, {NULL}};
const Segment SGI18885[] = {{&S103358[30], 6}, {NULL}};
const Segment SGI18886[] = {{&S103358[114], 3}, {NULL}};
const Segment SGI18887[] = {{&S103358[80], 5}, {NULL}};
const Segment SGI18888[] = {{&S103358[138], 13}, {NULL}};
const Segment SGI18889[] = {{&S101853[6], 4}, {NULL}};
const Segment SGI18890[] = {{&S101853[76], 5}, {NULL}};
const Segment SGI18891[] = {{&S101861[70], 6}, {NULL}};
const Segment SGI18892[] = {{&S101861[70], 18}, {NULL}};
const Segment SGI18893[] = {{&S101861[30], 31}, {NULL}};
const Segment SGI18894[] = {{&S101861[136], 3}, {NULL}};
const Segment SGI18895[] = {{&S101861[102], 5}, {NULL}};
const Segment SGI18896[] = {{&S104818[44], 15}, {NULL}};
const Segment SGI18897[] = {{&S104818[44], 27}, {NULL}};
const Segment SGI18898[] = {{&S104818[30], 6}, {NULL}};
const Segment SGI18899[] = {{&S104818[120], 3}, {NULL}};
const Segment SGI18900[] = {{&S104818[85], 6}, {NULL}};
const Segment SGI18901[] = {{&S101892[66], 8}, {NULL}};
const Segment SGI18902[] = {{&S101893[62], 8}, {NULL}};
const Segment SGI18903[] = {{&S101893[0], 13}, {NULL}};
const Segment SGI18904[] = {{&S101894[60], 8}, {&S101894[60], 8}, {NULL}};
const Segment SGI18905[] = {{&S101894[0], 13}, {NULL}};
const Segment SGI18906[] = {{&S101898[60], 8}, {&S101898[60], 8}, {NULL}};
const Segment SGI18907[] = {{&S101898[0], 13}, {NULL}};
const Segment SGI18908[] = {{&S101899[62], 8}, {NULL}};
const Segment SGI18909[] = {{&S101899[0], 13}, {NULL}};
const Segment SGI18910[] = {{&S101906[47], 8}, {NULL}};
const Segment SGI18911[] = {{&S101906[0], 12}, {NULL}};
const Segment SGI18912[] = {{&S100411[51], 6}, {NULL}};
const Segment SGI18913[] = {{&S100411[30], 12}, {NULL}};
const Segment SGI18914[] = {{&S100411[122], 3}, {NULL}};
const Segment SGI18915[] = {{&S100411[88], 5}, {NULL}};
const Segment SGI18916[] = {{&S100411[146], 13}, {NULL}};
const Segment SGI18917[] = {{&S101356[29], 8}, {NULL}};
const Segment SGI18918[] = {{&S101356[52], 3}, {NULL}};
const Segment SGI18919[] = {{&S101361[44], 20}, {NULL}};
const Segment SGI18920[] = {{&S101361[44], 29}, {NULL}};
const Segment SGI18921[] = {{&S101361[30], 6}, {NULL}};
const Segment SGI18922[] = {{&S101361[121], 3}, {NULL}};
const Segment SGI18923[] = {{&S101361[87], 5}, {NULL}};
const Segment SGI18924[] = {{&S101414[43], 10}, {NULL}};
const Segment SGI18925[] = {{&S101414[30], 4}, {NULL}};
const Segment SGI18926[] = {{&S101414[112], 3}, {NULL}};
const Segment SGI18927[] = {{&S101414[78], 5}, {NULL}};
const Segment SGI18928[] = {{&S101414[136], 13}, {NULL}};
const Segment SGI18929[] = {{&S101526[42], 4}, {NULL}};
const Segment SGI18930[] = {{&S101526[42], 15}, {NULL}};
const Segment SGI18931[] = {{&S101526[30], 4}, {NULL}};
const Segment SGI18932[] = {{&S101526[105], 3}, {NULL}};
const Segment SGI18933[] = {{&S101526[71], 5}, {NULL}};
const Segment SGI18934[] = {{&S103461[44], 10}, {NULL}};
const Segment SGI18935[] = {{&S103461[44], 21}, {NULL}};
const Segment SGI18936[] = {{&S103461[30], 6}, {NULL}};
const Segment SGI18937[] = {{&S103461[113], 3}, {NULL}};
const Segment SGI18938[] = {{&S103461[79], 5}, {NULL}};
const Segment SGI18939[] = {{&S103461[137], 7}, {NULL}};
const Segment SGI18940[] = {{&S103461[137], 11}, {NULL}};
const Segment SGI18941[] = {{&S102096[71], 8}, {&S102096[71], 8}, {NULL}};
const Segment SGI18942[] = {{&S102097[45], 8}, {NULL}};
const Segment SGI18943[] = {{&S102098[70], 8}, {&S102098[70], 8}, {NULL}};
const Segment SGI18944[] = {{&S102099[53], 8}, {NULL}};
const Segment SGI18945[] = {{&S102101[76], 8}, {&S102101[76], 8}, {NULL}};
const Segment SGI18946[] = {{&S102102[21], 8}, {NULL}};
const Segment SGI18947[] = {{&S102103[53], 8}, {&S102103[53], 8}, {&S102103[53], 8}, {NULL}};
const Segment SGI18948[] = {{&S103522[44], 12}, {NULL}};
const Segment SGI18949[] = {{&S103522[44], 24}, {NULL}};
const Segment SGI18950[] = {{&S103522[30], 6}, {NULL}};
const Segment SGI18951[] = {{&S103522[116], 3}, {NULL}};
const Segment SGI18952[] = {{&S103522[82], 5}, {NULL}};
const Segment SGI18953[] = {{&S103523[44], 12}, {NULL}};
const Segment SGI18954[] = {{&S103523[44], 23}, {NULL}};
const Segment SGI18955[] = {{&S103523[30], 6}, {NULL}};
const Segment SGI18956[] = {{&S103523[115], 3}, {NULL}};
const Segment SGI18957[] = {{&S103523[81], 5}, {NULL}};
const Segment SGI18958[] = {{&S101685[42], 8}, {NULL}};
const Segment SGI18959[] = {{&S101685[42], 20}, {NULL}};
const Segment SGI18960[] = {{&S101685[30], 4}, {NULL}};
const Segment SGI18961[] = {{&S101685[110], 3}, {NULL}};
const Segment SGI18962[] = {{&S101685[76], 5}, {NULL}};
const Segment SGI18963[] = {{&S102274[22], 8}, {NULL}};
const Segment SGI18964[] = {{&S102275[52], 10}, {NULL}};
const Segment SGI18965[] = {{&S102275[52], 21}, {NULL}};
const Segment SGI18966[] = {{&S102275[30], 14}, {NULL}};
const Segment SGI18967[] = {{&S102275[121], 3}, {NULL}};
const Segment SGI18968[] = {{&S102275[87], 5}, {NULL}};
const Segment SGI18969[] = {{&S102275[53], 8}, {NULL}};
const Segment SGI18970[] = {{&S102276[37], 23}, {NULL}};
const Segment SGI18971[] = {{&S102276[52], 8}, {NULL}};
const Segment SGI18972[] = {{&S102277[31], 3}, {NULL}};
const Segment SGI18973[] = {{&S102277[43], 8}, {NULL}};
const Segment SGI18974[] = {{&S102280[73], 8}, {NULL}};
const Segment SGI18975[] = {{&S102284[27], 8}, {&S102284[27], 8}, {NULL}};
const Segment SGI18976[] = {{&S102286[59], 8}, {&S102286[59], 8}, {NULL}};
const Segment SGI18977[] = {{&S102286[0], 13}, {NULL}};
const Segment SGI18978[] = {{&S102287[60], 8}, {&S102287[60], 8}, {NULL}};
const Segment SGI18979[] = {{&S102287[0], 13}, {NULL}};
const Segment SGI18980[] = {{&S105246[30], 4}, {NULL}};
const Segment SGI18981[] = {{&S105246[104], 3}, {NULL}};
const Segment SGI18982[] = {{&S102289[60], 8}, {&S102289[60], 8}, {NULL}};
const Segment SGI18983[] = {{&S102289[0], 13}, {NULL}};
const Segment SGI18984[] = {{&S105246[69], 6}, {NULL}};
const Segment SGI18985[] = {{&S105246[124], 8}, {NULL}};
const Segment SGI18986[] = {{&S102290[33], 8}, {NULL}};
const Segment SGI18987[] = {{&S102291[53], 8}, {&S102291[53], 8}, {&S102291[53], 8}, {NULL}};
const Segment SGI18988[] = {{&S102292[60], 8}, {&S102292[60], 8}, {NULL}};
const Segment SGI18989[] = {{&S102292[0], 13}, {NULL}};
const Segment SGI18990[] = {{&S102293[73], 9}, {NULL}};
const Segment SGI18991[] = {{&S102293[56], 8}, {NULL}};
const Segment SGI18992[] = {{&S102294[75], 9}, {NULL}};
const Segment SGI18993[] = {{&S102294[58], 8}, {NULL}};
const Segment SGI18994[] = {{&S102297[62], 8}, {NULL}};
const Segment SGI18995[] = {{&S102297[0], 13}, {NULL}};
const Segment SGI18996[] = {{&S102322[6], 4}, {NULL}};
const Segment SGI18997[] = {{&S102322[32], 4}, {NULL}};
const Segment SGI18998[] = {{&S102322[76], 5}, {NULL}};
const Segment SGI18999[] = {{&S105352[56], 7}, {NULL}};
const Segment SGI19000[] = {{&S105352[56], 20}, {NULL}};
const Segment SGI19001[] = {{&S105352[44], 4}, {NULL}};
const Segment SGI19002[] = {{&S105352[125], 3}, {NULL}};
const Segment SGI19003[] = {{&S105352[90], 6}, {NULL}};
const Segment SGI19004[] = {{&S105352[145], 8}, {NULL}};
const Segment SGI19005[] = {{&S103683[58], 10}, {NULL}};
const Segment SGI19006[] = {{&S103683[58], 21}, {NULL}};
const Segment SGI19007[] = {{&S103683[44], 6}, {NULL}};
const Segment SGI19008[] = {{&S103683[127], 3}, {NULL}};
const Segment SGI19009[] = {{&S103683[93], 5}, {NULL}};
const Segment SGI19010[] = {{&S103683[0], 14}, {NULL}};
const Segment SGI19011[] = {{&S103698[56], 10}, {NULL}};
const Segment SGI19012[] = {{&S103698[56], 21}, {NULL}};
const Segment SGI19013[] = {{&S103698[42], 6}, {NULL}};
const Segment SGI19014[] = {{&S103698[125], 3}, {NULL}};
const Segment SGI19015[] = {{&S103698[91], 5}, {NULL}};
const Segment SGI19016[] = {{&S103706[44], 11}, {NULL}};
const Segment SGI19017[] = {{&S103706[44], 20}, {NULL}};
const Segment SGI19018[] = {{&S103706[30], 6}, {NULL}};
const Segment SGI19019[] = {{&S103706[112], 3}, {NULL}};
const Segment SGI19020[] = {{&S103706[78], 5}, {NULL}};
const Segment SGI19021[] = {{&S105432[30], 4}, {NULL}};
const Segment SGI19022[] = {{&S105432[94], 3}, {NULL}};
const Segment SGI19023[] = {{&S105432[58], 7}, {NULL}};
const Segment SGI19024[] = {{&S105432[114], 8}, {NULL}};
const Segment SGI19025[] = {{&S100795[50], 9}, {NULL}};
const Segment SGI19026[] = {{&S100795[50], 20}, {NULL}};
const Segment SGI19027[] = {{&S100795[30], 12}, {NULL}};
const Segment SGI19028[] = {{&S100795[119], 3}, {NULL}};
const Segment SGI19029[] = {{&S100795[84], 6}, {NULL}};
const Segment SGI19030[] = {{&S100796[50], 9}, {NULL}};
const Segment SGI19031[] = {{&S100796[50], 20}, {NULL}};
const Segment SGI19032[] = {{&S105435[42], 8}, {NULL}};
const Segment SGI19033[] = {{&S100796[30], 12}, {NULL}};
const Segment SGI19034[] = {{&S105435[42], 18}, {NULL}};
const Segment SGI19035[] = {{&S105435[30], 4}, {NULL}};
const Segment SGI19036[] = {{&S100796[87], 4}, {NULL}};
const Segment SGI19037[] = {{&S100796[87], 3}, {NULL}};
const Segment SGI19038[] = {{&S105435[110], 3}, {NULL}};
const Segment SGI19039[] = {{&S105435[74], 7}, {NULL}};
const Segment SGI19040[] = {{&S105435[130], 8}, {NULL}};
const Segment SGI19041[] = {{&S105438[42], 13}, {NULL}};
const Segment SGI19042[] = {{&S105438[42], 26}, {NULL}};
const Segment SGI19043[] = {{&S105438[30], 4}, {NULL}};
const Segment SGI19044[] = {{&S105438[118], 3}, {NULL}};
const Segment SGI19045[] = {{&S105438[82], 7}, {NULL}};
const Segment SGI19046[] = {{&S105438[138], 8}, {NULL}};
const Segment SGI19047[] = {{&S105439[42], 7}, {NULL}};
const Segment SGI19048[] = {{&S105439[42], 20}, {NULL}};
const Segment SGI19049[] = {{&S105439[30], 4}, {NULL}};
const Segment SGI19050[] = {{&S105439[112], 3}, {NULL}};
const Segment SGI19051[] = {{&S105439[76], 7}, {NULL}};
const Segment SGI19052[] = {{&S105439[132], 8}, {NULL}};
const Segment SGI19053[] = {{&S100805[63], 6}, {NULL}};
const Segment SGI19054[] = {{&S100805[63], 23}, {NULL}};
const Segment SGI19055[] = {{&S100805[43], 12}, {NULL}};
const Segment SGI19056[] = {{&S100805[135], 3}, {NULL}};
const Segment SGI19057[] = {{&S100805[100], 6}, {NULL}};
const Segment SGI19058[] = {{&S102503[64], 8}, {NULL}};
const Segment SGI19059[] = {{&S102504[60], 8}, {&S102504[60], 8}, {NULL}};
const Segment SGI19060[] = {{&S102504[0], 13}, {NULL}};
const Segment SGI19061[] = {{&S102505[61], 8}, {NULL}};
const Segment SGI19062[] = {{&S102505[0], 13}, {NULL}};
const Segment SGI19063[] = {{&S102506[48], 8}, {&S102506[48], 8}, {NULL}};
const Segment SGI19064[] = {{&S102507[25], 8}, {NULL}};
const Segment SGI19065[] = {{&S102508[49], 10}, {NULL}};
const Segment SGI19066[] = {{&S102508[31], 17}, {NULL}};
const Segment SGI19067[] = {{&S102508[50], 8}, {NULL}};
const Segment SGI19068[] = {{&S102520[44], 8}, {NULL}};
const Segment SGI19069[] = {{&S101860[42], 8}, {NULL}};
const Segment SGI19070[] = {{&S101860[42], 20}, {NULL}};
const Segment SGI19071[] = {{&S101860[30], 4}, {NULL}};
const Segment SGI19072[] = {{&S101860[110], 3}, {NULL}};
const Segment SGI19073[] = {{&S101860[76], 5}, {NULL}};
const Segment SGI19074[] = {{&S102529[49], 8}, {NULL}};
const Segment SGI19075[] = {{&S102529[0], 12}, {NULL}};
const Segment SGI19076[] = {{&S105525[29], 4}, {NULL}};
const Segment SGI19077[] = {{&S105525[48], 3}, {NULL}};
const Segment SGI19078[] = {{&S100894[50], 6}, {NULL}};
const Segment SGI19079[] = {{&S100894[50], 24}, {NULL}};
const Segment SGI19080[] = {{&S100894[30], 12}, {NULL}};
const Segment SGI19081[] = {{&S100894[166], 3}, {NULL}};
const Segment SGI19082[] = {{&S100894[132], 3}, {NULL}};
const Segment SGI19083[] = {{&S100894[97], 6}, {NULL}};
const Segment SGI19084[] = {{&S100894[152], 8}, {NULL}};
const Segment SGI19085[] = {{&S102578[57], 15}, {NULL}};
const Segment SGI19086[] = {{&S102578[57], 27}, {NULL}};
const Segment SGI19087[] = {{&S102578[30], 19}, {NULL}};
const Segment SGI19088[] = {{&S102578[133], 3}, {NULL}};
const Segment SGI19089[] = {{&S102578[98], 6}, {NULL}};
const Segment SGI19090[] = {{&S100909[51], 14}, {NULL}};
const Segment SGI19091[] = {{&S100909[30], 12}, {NULL}};
const Segment SGI19092[] = {{&S100909[124], 3}, {NULL}};
const Segment SGI19093[] = {{&S100909[90], 5}, {NULL}};
const Segment SGI19094[] = {{&S100909[148], 13}, {NULL}};
const Segment SGI19095[] = {{&S100919[51], 7}, {NULL}};
const Segment SGI19096[] = {{&S100919[30], 12}, {NULL}};
const Segment SGI19097[] = {{&S100919[117], 3}, {NULL}};
const Segment SGI19098[] = {{&S100919[83], 5}, {NULL}};
const Segment SGI19099[] = {{&S100919[141], 13}, {NULL}};
const Segment SGI19100[] = {{&S102669[23], 6}, {NULL}};
const Segment SGI19101[] = {{&S103957[31], 5}, {NULL}};
const Segment SGI19102[] = {{&S103957[51], 3}, {NULL}};
const Segment SGI19103[] = {{&S103959[44], 11}, {NULL}};
const Segment SGI19104[] = {{&S103959[44], 23}, {NULL}};
const Segment SGI19105[] = {{&S103959[30], 6}, {NULL}};
const Segment SGI19106[] = {{&S103959[115], 3}, {NULL}};
const Segment SGI19107[] = {{&S103959[81], 5}, {NULL}};
const Segment SGI19108[] = {{&S102693[62], 8}, {NULL}};
const Segment SGI19109[] = {{&S102693[0], 13}, {NULL}};
const Segment SGI19110[] = {{&S102698[72], 7}, {NULL}};
const Segment SGI19111[] = {{&S102698[55], 8}, {NULL}};
const Segment SGI19112[] = {{&S102699[75], 9}, {NULL}};
const Segment SGI19113[] = {{&S102032[42], 13}, {NULL}};
const Segment SGI19114[] = {{&S102699[58], 8}, {NULL}};
const Segment SGI19115[] = {{&S102032[42], 24}, {NULL}};
const Segment SGI19116[] = {{&S102032[30], 4}, {NULL}};
const Segment SGI19117[] = {{&S102032[114], 3}, {NULL}};
const Segment SGI19118[] = {{&S102032[80], 5}, {NULL}};
const Segment SGI19119[] = {{&S102700[75], 7}, {NULL}};
const Segment SGI19120[] = {{&S102700[58], 8}, {NULL}};
const Segment SGI19121[] = {{&S102705[61], 8}, {NULL}};
const Segment SGI19122[] = {{&S102705[0], 13}, {NULL}};
const Segment SGI19123[] = {{&S102049[42], 14}, {NULL}};
const Segment SGI19124[] = {{&S102049[42], 26}, {NULL}};
const Segment SGI19125[] = {{&S102049[30], 4}, {NULL}};
const Segment SGI19126[] = {{&S102049[116], 3}, {NULL}};
const Segment SGI19127[] = {{&S102049[82], 5}, {NULL}};
const Segment SGI19128[] = {{&S102710[42], 8}, {NULL}};
const Segment SGI19129[] = {{&S102711[75], 9}, {NULL}};
const Segment SGI19130[] = {{&S102711[58], 8}, {NULL}};
const Segment SGI19131[] = {{&S101021[50], 7}, {NULL}};
const Segment SGI19132[] = {{&S101021[50], 23}, {NULL}};
const Segment SGI19133[] = {{&S101021[30], 12}, {NULL}};
const Segment SGI19134[] = {{&S101021[122], 3}, {NULL}};
const Segment SGI19135[] = {{&S101021[87], 6}, {NULL}};
const Segment SGI19136[] = {{&S101022[50], 7}, {NULL}};
const Segment SGI19137[] = {{&S103996[58], 10}, {NULL}};
const Segment SGI19138[] = {{&S101022[50], 23}, {NULL}};
const Segment SGI19139[] = {{&S103996[58], 21}, {NULL}};
const Segment SGI19140[] = {{&S101022[30], 12}, {NULL}};
const Segment SGI19141[] = {{&S103996[44], 6}, {NULL}};
const Segment SGI19142[] = {{&S103996[127], 3}, {NULL}};
const Segment SGI19143[] = {{&S101022[90], 4}, {NULL}};
const Segment SGI19144[] = {{&S101022[90], 3}, {NULL}};
const Segment SGI19145[] = {{&S103996[93], 5}, {NULL}};
const Segment SGI19146[] = {{&S103996[0], 14}, {NULL}};
const Segment SGI19147[] = {{&S102069[43], 10}, {NULL}};
const Segment SGI19148[] = {{&S102069[30], 4}, {NULL}};
const Segment SGI19149[] = {{&S104007[45], 18}, {NULL}};
const Segment SGI19150[] = {{&S104007[30], 6}, {NULL}};
const Segment SGI19151[] = {{&S102069[112], 3}, {NULL}};
const Segment SGI19152[] = {{&S102069[78], 5}, {NULL}};
const Segment SGI19153[] = {{&S104007[122], 3}, {NULL}};
const Segment SGI19154[] = {{&S102069[136], 13}, {NULL}};
const Segment SGI19155[] = {{&S104007[88], 5}, {NULL}};
const Segment SGI19156[] = {{&S104007[146], 13}, {NULL}};
const Segment SGI19157[] = {{&S101036[63], 6}, {NULL}};
const Segment SGI19158[] = {{&S101036[63], 24}, {NULL}};
const Segment SGI19159[] = {{&S101036[43], 12}, {NULL}};
const Segment SGI19160[] = {{&S101036[136], 3}, {NULL}};
const Segment SGI19161[] = {{&S101036[101], 6}, {NULL}};
const Segment SGI19162[] = {{&S104031[31], 8}, {NULL}};
const Segment SGI19163[] = {{&S104031[55], 3}, {NULL}};
const Segment SGI19164[] = {{&S104035[59], 10}, {NULL}};
const Segment SGI19165[] = {{&S104035[59], 21}, {NULL}};
const Segment SGI19166[] = {{&S104035[45], 6}, {NULL}};
const Segment SGI19167[] = {{&S104035[128], 3}, {NULL}};
const Segment SGI19168[] = {{&S104035[94], 5}, {NULL}};
const Segment SGI19169[] = {{&S104035[0], 14}, {NULL}};
const Segment SGI19170[] = {{&S105623[37], 10}, {NULL}};
const Segment SGI19171[] = {{&S105623[27], 9}, {NULL}};
const Segment SGI19172[] = {{&S105623[107], 2}, {NULL}};
const Segment SGI19173[] = {{&S105623[73], 6}, {NULL}};
const Segment SGI19174[] = {{&S101115[51], 22}, {NULL}};
const Segment SGI19175[] = {{&S101115[30], 12}, {NULL}};
const Segment SGI19176[] = {{&S101115[132], 3}, {NULL}};
const Segment SGI19177[] = {{&S101115[98], 5}, {NULL}};
const Segment SGI19178[] = {{&S101115[156], 13}, {NULL}};
const Segment SGI19179[] = {{&S102838[70], 11}, {NULL}};
const Segment SGI19180[] = {{&S102838[63], 6}, {&S102838[70], 33}, {NULL}};
const Segment SGI19181[] = {{&S102838[30], 32}, {NULL}};
const Segment SGI19182[] = {{&S102838[152], 3}, {NULL}};
const Segment SGI19183[] = {{&S102838[117], 6}, {NULL}};
const Segment SGI19184[] = {{&S102839[76], 11}, {NULL}};
const Segment SGI19185[] = {{&S102839[76], 33}, {NULL}};
const Segment SGI19186[] = {{&S102839[30], 38}, {NULL}};
const Segment SGI19187[] = {{&S102839[158], 3}, {NULL}};
const Segment SGI19188[] = {{&S102839[123], 6}, {NULL}};
const Segment SGI19189[] = {{&S101141[42], 10}, {NULL}};
const Segment SGI19190[] = {{&S101141[42], 21}, {NULL}};
const Segment SGI19191[] = {{&S101141[30], 4}, {NULL}};
const Segment SGI19192[] = {{&S101141[111], 3}, {NULL}};
const Segment SGI19193[] = {{&S101141[77], 5}, {NULL}};
const Segment SGI19194[] = {{&S101142[42], 10}, {NULL}};
const Segment SGI19195[] = {{&S101142[42], 21}, {NULL}};
const Segment SGI19196[] = {{&S101142[30], 4}, {NULL}};
const Segment SGI19197[] = {{&S101142[112], 3}, {NULL}};
const Segment SGI19198[] = {{&S101142[77], 6}, {NULL}};
const Segment SGI19199[] = {{&S101143[42], 11}, {NULL}};
const Segment SGI19200[] = {{&S101143[42], 22}, {NULL}};
const Segment SGI19201[] = {{&S101143[30], 4}, {NULL}};
const Segment SGI19202[] = {{&S101143[112], 3}, {NULL}};
const Segment SGI19203[] = {{&S101143[78], 5}, {NULL}};
const Segment SGI19204[] = {{&S102900[27], 8}, {NULL}};
const Segment SGI19205[] = {{&S102901[27], 8}, {&S102901[27], 8}, {&S102901[27], 8}, {NULL}};
const Segment SGI19206[] = {{&S102903[13], 8}, {NULL}};
const Segment SGI19207[] = {{&S102913[60], 8}, {&S102913[60], 8}, {NULL}};
const Segment SGI19208[] = {{&S102913[0], 13}, {NULL}};
const Segment SGI19209[] = {{&S102915[55], 8}, {&S102915[55], 8}, {&S102915[55], 8}, {NULL}};
const Segment SGI19210[] = {{&S101216[50], 11}, {NULL}};
const Segment SGI19211[] = {{&S101216[50], 22}, {NULL}};
const Segment SGI19212[] = {{&S101216[30], 12}, {NULL}};
const Segment SGI19213[] = {{&S101216[121], 3}, {NULL}};
const Segment SGI19214[] = {{&S101216[86], 6}, {NULL}};
const Segment SGI19215[] = {{&S102279[55], 10}, {NULL}};
const Segment SGI19216[] = {{&S102279[55], 21}, {NULL}};
const Segment SGI19217[] = {{&S102279[43], 4}, {NULL}};
const Segment SGI19218[] = {{&S102279[124], 3}, {NULL}};
const Segment SGI19219[] = {{&S102279[90], 5}, {NULL}};
const Segment SGI19220[] = {{&S102279[0], 10}, {NULL}};
const Segment SGI19221[] = {{&S104223[44], 7}, {NULL}};
const Segment SGI19222[] = {{&S104223[44], 22}, {NULL}};
const Segment SGI19223[] = {{&S104223[30], 6}, {NULL}};
const Segment SGI19224[] = {{&S104223[158], 3}, {NULL}};
const Segment SGI19225[] = {{&S104223[124], 3}, {NULL}};
const Segment SGI19226[] = {{&S104223[89], 6}, {NULL}};
const Segment SGI19227[] = {{&S104223[144], 8}, {NULL}};
const Segment SGI19228[] = {{&S101307[50], 11}, {NULL}};
const Segment SGI19229[] = {{&S101307[50], 22}, {NULL}};
const Segment SGI19230[] = {{&S101307[30], 12}, {NULL}};
const Segment SGI19231[] = {{&S101307[121], 3}, {NULL}};
const Segment SGI19232[] = {{&S101307[86], 6}, {NULL}};
const Segment SGI19233[] = {{&S101359[51], 10}, {NULL}};
const Segment SGI19234[] = {{&S101359[30], 12}, {NULL}};
const Segment SGI19235[] = {{&S101359[120], 3}, {NULL}};
const Segment SGI19236[] = {{&S101359[86], 5}, {NULL}};
const Segment SGI19237[] = {{&S101359[144], 13}, {NULL}};
const Segment SGI19238[] = {{&S101360[50], 14}, {NULL}};
const Segment SGI19239[] = {{&S101360[50], 25}, {NULL}};
const Segment SGI19240[] = {{&S101360[30], 12}, {NULL}};
const Segment SGI19241[] = {{&S101360[124], 3}, {NULL}};
const Segment SGI19242[] = {{&S101360[89], 6}, {NULL}};
const Segment SGI19243[] = {{&S103098[24], 7}, {NULL}};
const Segment SGI19244[] = {{&S103098[46], 3}, {NULL}};
const Segment SGI19245[] = {{&S102465[43], 10}, {NULL}};
const Segment SGI19246[] = {{&S102465[30], 4}, {NULL}};
const Segment SGI19247[] = {{&S102465[112], 3}, {NULL}};
const Segment SGI19248[] = {{&S102465[78], 5}, {NULL}};
const Segment SGI19249[] = {{&S102465[136], 13}, {NULL}};
const Segment SGI19250[] = {{&S102470[29], 9}, {NULL}};
const Segment SGI19251[] = {{&S102470[53], 3}, {NULL}};
const Segment SGI19252[] = {{&S103144[27], 8}, {&S103144[27], 8}, {&S103144[27], 8}, {NULL}};
const Segment SGI19253[] = {{&S103145[59], 8}, {&S103145[59], 8}, {NULL}};
const Segment SGI19254[] = {{&S103145[0], 13}, {NULL}};
const Segment SGI19255[] = {{&S103147[56], 8}, {NULL}};
const Segment SGI19256[] = {{&S103154[53], 8}, {NULL}};
const Segment SGI19257[] = {{&S103156[62], 8}, {NULL}};
const Segment SGI19258[] = {{&S103156[0], 13}, {NULL}};
const Segment SGI19259[] = {{&S103160[13], 8}, {NULL}};
const Segment SGI19260[] = {{&S103164[23], 8}, {NULL}};
const Segment SGI19261[] = {{&S103174[61], 8}, {&S103174[61], 8}, {NULL}};
const Segment SGI19262[] = {{&S103174[0], 13}, {NULL}};
const Segment SGI19263[] = {{&S103175[61], 8}, {&S103175[61], 8}, {NULL}};
const Segment SGI19264[] = {{&S103175[0], 13}, {NULL}};
const Segment SGI19265[] = {{&S103182[47], 8}, {NULL}};
const Segment SGI19266[] = {{&S103182[0], 12}, {NULL}};
const Segment SGI19267[] = {{&S103183[49], 8}, {NULL}};
const Segment SGI19268[] = {{&S103183[0], 12}, {NULL}};
const Segment SGI19269[] = {{&S103184[50], 8}, {NULL}};
const Segment SGI19270[] = {{&S103184[0], 12}, {NULL}};
const Segment SGI19271[] = {{&S103188[50], 8}, {NULL}};
const Segment SGI19272[] = {{&S103188[0], 12}, {NULL}};
const Segment SGI19273[] = {{&S101512[51], 14}, {NULL}};
const Segment SGI19274[] = {{&S101512[30], 12}, {NULL}};
const Segment SGI19275[] = {{&S101512[124], 3}, {NULL}};
const Segment SGI19276[] = {{&S101512[90], 5}, {NULL}};
const Segment SGI19277[] = {{&S101512[148], 13}, {NULL}};
const Segment SGI19278[] = {{&S101523[51], 14}, {NULL}};
const Segment SGI19279[] = {{&S101523[30], 12}, {NULL}};
const Segment SGI19280[] = {{&S101523[124], 3}, {NULL}};
const Segment SGI19281[] = {{&S101523[90], 5}, {NULL}};
const Segment SGI19282[] = {{&S101523[148], 13}, {NULL}};
const Segment SGI19283[] = {{&S104353[30], 6}, {NULL}};
const Segment SGI19284[] = {{&S104353[91], 3}, {NULL}};
const Segment SGI19285[] = {{&S104353[33], 3}, {NULL}};
const Segment SGI19286[] = {{&S104353[57], 5}, {NULL}};
const Segment SGI19287[] = {{&S102558[42], 101}, {NULL}};
const Segment SGI19288[] = {{&S102558[42], 112}, {NULL}};
const Segment SGI19289[] = {{&S102558[30], 4}, {NULL}};
const Segment SGI19290[] = {{&S102558[202], 3}, {NULL}};
const Segment SGI19291[] = {{&S102558[168], 5}, {NULL}};
const Segment SGI19292[] = {{&S101585[42], 11}, {NULL}};
const Segment SGI19293[] = {{&S101585[42], 22}, {NULL}};
const Segment SGI19294[] = {{&S101585[30], 4}, {NULL}};
const Segment SGI19295[] = {{&S101585[113], 3}, {NULL}};
const Segment SGI19296[] = {{&S101585[78], 6}, {NULL}};
const Segment SGI19297[] = {{&S102614[56], 10}, {NULL}};
const Segment SGI19298[] = {{&S102614[56], 21}, {NULL}};
const Segment SGI19299[] = {{&S102614[44], 4}, {NULL}};
const Segment SGI19300[] = {{&S102614[125], 3}, {NULL}};
const Segment SGI19301[] = {{&S102614[91], 5}, {NULL}};
const Segment SGI19302[] = {{&S102615[56], 11}, {NULL}};
const Segment SGI19303[] = {{&S102615[56], 22}, {NULL}};
const Segment SGI19304[] = {{&S102615[44], 4}, {NULL}};
const Segment SGI19305[] = {{&S102615[126], 3}, {NULL}};
const Segment SGI19306[] = {{&S102615[92], 5}, {NULL}};
const Segment SGI19307[] = {{&S102631[42], 27}, {NULL}};
const Segment SGI19308[] = {{&S102631[30], 4}, {NULL}};
const Segment SGI19309[] = {{&S102631[135], 3}, {NULL}};
const Segment SGI19310[] = {{&S102631[101], 5}, {NULL}};
const Segment SGI19311[] = {{&S102659[42], 12}, {NULL}};
const Segment SGI19312[] = {{&S102659[42], 23}, {NULL}};
const Segment SGI19313[] = {{&S102659[30], 4}, {NULL}};
const Segment SGI19314[] = {{&S102659[157], 3}, {NULL}};
const Segment SGI19315[] = {{&S102659[123], 3}, {NULL}};
const Segment SGI19316[] = {{&S102659[88], 6}, {NULL}};
const Segment SGI19317[] = {{&S102659[143], 8}, {NULL}};
const Segment SGI19318[] = {{&S102664[43], 7}, {NULL}};
const Segment SGI19319[] = {{&S102664[30], 4}, {NULL}};
const Segment SGI19320[] = {{&S102664[109], 3}, {NULL}};
const Segment SGI19321[] = {{&S102664[75], 5}, {NULL}};
const Segment SGI19322[] = {{&S102664[133], 13}, {NULL}};
const Segment SGI19323[] = {{&S102667[42], 9}, {NULL}};
const Segment SGI19324[] = {{&S102667[42], 21}, {NULL}};
const Segment SGI19325[] = {{&S102667[30], 4}, {NULL}};
const Segment SGI19326[] = {{&S102667[111], 3}, {NULL}};
const Segment SGI19327[] = {{&S102667[77], 5}, {NULL}};
const Segment SGI19328[] = {{&S103407[74], 9}, {NULL}};
const Segment SGI19329[] = {{&S103408[74], 9}, {NULL}};
const Segment SGI19330[] = {{&S103430[43], 8}, {&S103430[43], 8}, {NULL}};
const Segment SGI19331[] = {{&S104384[45], 10}, {NULL}};
const Segment SGI19332[] = {{&S104384[30], 6}, {NULL}};
const Segment SGI19333[] = {{&S103431[43], 8}, {&S103431[43], 8}, {NULL}};
const Segment SGI19334[] = {{&S103431[75], 6}, {NULL}};
const Segment SGI19335[] = {{&S104384[114], 3}, {NULL}};
const Segment SGI19336[] = {{&S104384[80], 5}, {NULL}};
const Segment SGI19337[] = {{&S104384[138], 13}, {NULL}};
const Segment SGI19338[] = {{&S103435[73], 8}, {NULL}};
const Segment SGI19339[] = {{&S103439[74], 7}, {NULL}};
const Segment SGI19340[] = {{&S103439[56], 8}, {NULL}};
const Segment SGI19341[] = {{&S103440[74], 9}, {NULL}};
const Segment SGI19342[] = {{&S103440[56], 8}, {NULL}};
const Segment SGI19343[] = {{&S103441[75], 7}, {NULL}};
const Segment SGI19344[] = {{&S103441[58], 8}, {NULL}};
const Segment SGI19345[] = {{&S103448[61], 8}, {NULL}};
const Segment SGI19346[] = {{&S103448[0], 13}, {NULL}};
const Segment SGI19347[] = {{&S101789[50], 9}, {NULL}};
const Segment SGI19348[] = {{&S101789[50], 20}, {NULL}};
const Segment SGI19349[] = {{&S101789[30], 12}, {NULL}};
const Segment SGI19350[] = {{&S101789[119], 3}, {NULL}};
const Segment SGI19351[] = {{&S101789[84], 6}, {NULL}};
const Segment SGI19352[] = {{&S103451[23], 8}, {NULL}};
const Segment SGI19353[] = {{&S104419[59], 10}, {NULL}};
const Segment SGI19354[] = {{&S104419[59], 21}, {NULL}};
const Segment SGI19355[] = {{&S104419[45], 6}, {NULL}};
const Segment SGI19356[] = {{&S104419[128], 3}, {NULL}};
const Segment SGI19357[] = {{&S104419[94], 5}, {NULL}};
const Segment SGI19358[] = {{&S104419[0], 14}, {NULL}};
const Segment SGI19359[] = {{&S103456[44], 8}, {NULL}};
const Segment SGI19360[] = {{&S103564[107], 6}, {NULL}};
const Segment SGI19361[] = {{&S103582[70], 20}, {NULL}};
const Segment SGI19362[] = {{&S103582[70], 31}, {NULL}};
const Segment SGI19363[] = {{&S103582[30], 32}, {NULL}};
const Segment SGI19364[] = {{&S103582[150], 3}, {NULL}};
const Segment SGI19365[] = {{&S103582[115], 6}, {NULL}};
const Segment SGI19366[] = {{&S104674[44], 21}, {NULL}};
const Segment SGI19367[] = {{&S104674[44], 33}, {NULL}};
const Segment SGI19368[] = {{&S104674[30], 6}, {NULL}};
const Segment SGI19369[] = {{&S104674[125], 3}, {NULL}};
const Segment SGI19370[] = {{&S104674[91], 5}, {NULL}};
const Segment SGI19371[] = {{&S102063[50], 26}, {NULL}};
const Segment SGI19372[] = {{&S102063[30], 12}, {NULL}};
const Segment SGI19373[] = {{&S102063[143], 3}, {NULL}};
const Segment SGI19374[] = {{&S102063[108], 6}, {NULL}};
const Segment SGI19375[] = {{&S102064[50], 26}, {NULL}};
const Segment SGI19376[] = {{&S102064[30], 12}, {NULL}};
const Segment SGI19377[] = {{&S102064[143], 3}, {NULL}};
const Segment SGI19378[] = {{&S102064[108], 6}, {NULL}};
const Segment SGI19379[] = {{&S102873[43], 12}, {NULL}};
const Segment SGI19380[] = {{&S102873[30], 4}, {NULL}};
const Segment SGI19381[] = {{&S102873[114], 3}, {NULL}};
const Segment SGI19382[] = {{&S102873[80], 5}, {NULL}};
const Segment SGI19383[] = {{&S102873[138], 13}, {NULL}};
const Segment SGI19384[] = {{&S102910[57], 10}, {NULL}};
const Segment SGI19385[] = {{&S102910[57], 21}, {NULL}};
const Segment SGI19386[] = {{&S102910[45], 4}, {NULL}};
const Segment SGI19387[] = {{&S102910[92], 5}, {NULL}};
const Segment SGI19388[] = {{&S102910[0], 14}, {NULL}};
const Segment SGI19389[] = {{&S102911[42], 10}, {NULL}};
const Segment SGI19390[] = {{&S102911[42], 21}, {NULL}};
const Segment SGI19391[] = {{&S102911[30], 4}, {NULL}};
const Segment SGI19392[] = {{&S102911[111], 3}, {NULL}};
const Segment SGI19393[] = {{&S102911[77], 5}, {NULL}};
const Segment SGI19394[] = {{&S102911[122], 14}, {NULL}};
const Segment SGI19395[] = {{&S104771[58], 10}, {NULL}};
const Segment SGI19396[] = {{&S104771[58], 21}, {NULL}};
const Segment SGI19397[] = {{&S104771[44], 6}, {NULL}};
const Segment SGI19398[] = {{&S104771[127], 3}, {NULL}};
const Segment SGI19399[] = {{&S104771[93], 5}, {NULL}};
const Segment SGI19400[] = {{&S104771[0], 14}, {NULL}};
const Segment SGI19401[] = {{&S103705[65], 28}, {NULL}};
const Segment SGI19402[] = {{&S103705[65], 40}, {NULL}};
const Segment SGI19403[] = {{&S103705[30], 27}, {NULL}};
const Segment SGI19404[] = {{&S103705[154], 3}, {NULL}};
const Segment SGI19405[] = {{&S103705[119], 6}, {NULL}};
const Segment SGI19406[] = {{&S104819[44], 20}, {NULL}};
const Segment SGI19407[] = {{&S104819[44], 30}, {NULL}};
const Segment SGI19408[] = {{&S104819[30], 6}, {NULL}};
const Segment SGI19409[] = {{&S104819[122], 3}, {NULL}};
const Segment SGI19410[] = {{&S104819[88], 5}, {NULL}};
const Segment SGI19411[] = {{&S103751[66], 8}, {NULL}};
const Segment SGI19412[] = {{&S103752[61], 8}, {NULL}};
const Segment SGI19413[] = {{&S103752[0], 13}, {NULL}};
const Segment SGI19414[] = {{&S104828[44], 11}, {NULL}};
const Segment SGI19415[] = {{&S104828[44], 22}, {NULL}};
const Segment SGI19416[] = {{&S104828[30], 6}, {NULL}};
const Segment SGI19417[] = {{&S104828[121], 3}, {NULL}};
const Segment SGI19418[] = {{&S104828[87], 5}, {NULL}};
const Segment SGI19419[] = {{&S104828[67], 7}, {NULL}};
const Segment SGI19420[] = {{&S104830[57], 10}, {NULL}};
const Segment SGI19421[] = {{&S104830[57], 21}, {NULL}};
const Segment SGI19422[] = {{&S104830[43], 6}, {NULL}};
const Segment SGI19423[] = {{&S104830[126], 3}, {NULL}};
const Segment SGI19424[] = {{&S104830[92], 5}, {NULL}};
const Segment SGI19425[] = {{&S104830[0], 12}, {NULL}};
const Segment SGI19426[] = {{&S103759[60], 8}, {&S103759[60], 8}, {NULL}};
const Segment SGI19427[] = {{&S103759[0], 13}, {NULL}};
const Segment SGI19428[] = {{&S103762[62], 8}, {NULL}};
const Segment SGI19429[] = {{&S103762[0], 13}, {NULL}};
const Segment SGI19430[] = {{&S103766[61], 8}, {NULL}};
const Segment SGI19431[] = {{&S103766[0], 13}, {NULL}};
const Segment SGI19432[] = {{&S103769[46], 8}, {NULL}};
const Segment SGI19433[] = {{&S103770[44], 8}, {NULL}};
const Segment SGI19434[] = {{&S104881[59], 10}, {NULL}};
const Segment SGI19435[] = {{&S104881[59], 21}, {NULL}};
const Segment SGI19436[] = {{&S104881[45], 6}, {NULL}};
const Segment SGI19437[] = {{&S104881[128], 3}, {NULL}};
const Segment SGI19438[] = {{&S104881[94], 5}, {NULL}};
const Segment SGI19439[] = {{&S104881[0], 14}, {NULL}};
const Segment SGI19440[] = {{&S103109[42], 12}, {NULL}};
const Segment SGI19441[] = {{&S103109[42], 23}, {NULL}};
const Segment SGI19442[] = {{&S103109[30], 4}, {NULL}};
const Segment SGI19443[] = {{&S103109[120], 3}, {NULL}};
const Segment SGI19444[] = {{&S103109[86], 5}, {NULL}};
const Segment SGI19445[] = {{&S103109[66], 7}, {NULL}};
const Segment SGI19446[] = {{&S103110[43], 10}, {NULL}};
const Segment SGI19447[] = {{&S103110[30], 4}, {NULL}};
const Segment SGI19448[] = {{&S103110[112], 3}, {NULL}};
const Segment SGI19449[] = {{&S103110[78], 5}, {NULL}};
const Segment SGI19450[] = {{&S103110[136], 13}, {NULL}};
const Segment SGI19451[] = {{&S103134[57], 10}, {NULL}};
const Segment SGI19452[] = {{&S103134[57], 21}, {NULL}};
const Segment SGI19453[] = {{&S103134[45], 4}, {NULL}};
const Segment SGI19454[] = {{&S103134[126], 3}, {NULL}};
const Segment SGI19455[] = {{&S103134[92], 5}, {NULL}};
const Segment SGI19456[] = {{&S103134[0], 14}, {NULL}};
const Segment SGI19457[] = {{&S103836[57], 29}, {NULL}};
const Segment SGI19458[] = {{&S103836[57], 40}, {NULL}};
const Segment SGI19459[] = {{&S103836[30], 19}, {NULL}};
const Segment SGI19460[] = {{&S103836[145], 3}, {NULL}};
const Segment SGI19461[] = {{&S103836[111], 5}, {NULL}};
const Segment SGI19462[] = {{&S103142[57], 10}, {NULL}};
const Segment SGI19463[] = {{&S103142[57], 21}, {NULL}};
const Segment SGI19464[] = {{&S103142[45], 4}, {NULL}};
const Segment SGI19465[] = {{&S103142[126], 3}, {NULL}};
const Segment SGI19466[] = {{&S103142[92], 5}, {NULL}};
const Segment SGI19467[] = {{&S103142[0], 14}, {NULL}};
const Segment SGI19468[] = {{&S105003[44], 10}, {NULL}};
const Segment SGI19469[] = {{&S105003[44], 21}, {NULL}};
const Segment SGI19470[] = {{&S105003[30], 6}, {NULL}};
const Segment SGI19471[] = {{&S105003[112], 3}, {NULL}};
const Segment SGI19472[] = {{&S105003[79], 5}, {NULL}};
const Segment SGI19473[] = {{&S105003[116], 14}, {NULL}};
const Segment SGI19474[] = {{&S105007[58], 10}, {NULL}};
const Segment SGI19475[] = {{&S105007[58], 21}, {NULL}};
const Segment SGI19476[] = {{&S105007[44], 6}, {NULL}};
const Segment SGI19477[] = {{&S105007[127], 3}, {NULL}};
const Segment SGI19478[] = {{&S105007[93], 5}, {NULL}};
const Segment SGI19479[] = {{&S105007[0], 14}, {NULL}};
const Segment SGI19480[] = {{&S102250[37], 6}, {NULL}};
const Segment SGI19481[] = {{&S102250[58], 3}, {NULL}};
const Segment SGI19482[] = {{&S103918[63], 10}, {NULL}};
const Segment SGI19483[] = {{&S103918[63], 21}, {NULL}};
const Segment SGI19484[] = {{&S103918[30], 24}, {NULL}};
const Segment SGI19485[] = {{&S103918[132], 3}, {NULL}};
const Segment SGI19486[] = {{&S103918[98], 5}, {NULL}};
const Segment SGI19487[] = {{&S103932[78], 5}, {NULL}};
const Segment SGI19488[] = {{&S103279[37], 5}, {NULL}};
const Segment SGI19489[] = {{&S103279[98], 3}, {NULL}};
const Segment SGI19490[] = {{&S103279[87], 3}, {NULL}};
const Segment SGI19491[] = {{&S103279[64], 6}, {NULL}};
const Segment SGI19492[] = {{&S103280[29], 5}, {NULL}};
const Segment SGI19493[] = {{&S103280[49], 3}, {NULL}};
const Segment SGI19494[] = {{&S104038[13], 8}, {NULL}};
const Segment SGI19495[] = {{&S104040[43], 8}, {&S104040[43], 8}, {NULL}};
const Segment SGI19496[] = {{&S104049[43], 8}, {&S104049[43], 8}, {NULL}};
const Segment SGI19497[] = {{&S104053[61], 8}, {&S104053[61], 8}, {NULL}};
const Segment SGI19498[] = {{&S104053[0], 13}, {NULL}};
const Segment SGI19499[] = {{&S104056[21], 8}, {NULL}};
const Segment SGI19500[] = {{&S104058[62], 8}, {&S104058[62], 8}, {NULL}};
const Segment SGI19501[] = {{&S104058[0], 13}, {NULL}};
const Segment SGI19502[] = {{&S104059[59], 8}, {&S104059[59], 8}, {&S104059[59], 8}, {NULL}};
const Segment SGI19503[] = {{&S104059[0], 13}, {NULL}};
const Segment SGI19504[] = {{&S104061[27], 8}, {&S104061[27], 8}, {NULL}};
const Segment SGI19505[] = {{&S104062[27], 8}, {&S104062[27], 8}, {&S104062[27], 8}, {NULL}};
const Segment SGI19506[] = {{&S104063[60], 8}, {&S104063[60], 8}, {NULL}};
const Segment SGI19507[] = {{&S104063[0], 13}, {NULL}};
const Segment SGI19508[] = {{&S104064[83], 7}, {NULL}};
const Segment SGI19509[] = {{&S104064[43], 8}, {NULL}};
const Segment SGI19510[] = {{&S104065[27], 8}, {&S104065[27], 8}, {&S104065[27], 8}, {NULL}};
const Segment SGI19511[] = {{&S104068[27], 8}, {NULL}};
const Segment SGI19512[] = {{&S104069[23], 8}, {NULL}};
const Segment SGI19513[] = {{&S104072[60], 8}, {&S104072[60], 8}, {NULL}};
const Segment SGI19514[] = {{&S104072[0], 13}, {NULL}};
const Segment SGI19515[] = {{&S104074[60], 8}, {&S104074[60], 8}, {NULL}};
const Segment SGI19516[] = {{&S104074[0], 13}, {NULL}};
const Segment SGI19517[] = {{&S104079[55], 8}, {&S104079[55], 8}, {&S104079[55], 8}, {NULL}};
const Segment SGI19518[] = {{&S104083[61], 8}, {NULL}};
const Segment SGI19519[] = {{&S104083[0], 13}, {NULL}};
const Segment SGI19520[] = {{&S102482[51], 6}, {NULL}};
const Segment SGI19521[] = {{&S102482[30], 12}, {NULL}};
const Segment SGI19522[] = {{&S102482[122], 3}, {NULL}};
const Segment SGI19523[] = {{&S102482[88], 5}, {NULL}};
const Segment SGI19524[] = {{&S102482[146], 13}, {NULL}};
const Segment SGI19525[] = {{&S103397[43], 10}, {NULL}};
const Segment SGI19526[] = {{&S103397[30], 4}, {NULL}};
const Segment SGI19527[] = {{&S103397[112], 3}, {NULL}};
const Segment SGI19528[] = {{&S103397[78], 5}, {NULL}};
const Segment SGI19529[] = {{&S103397[136], 13}, {NULL}};
const Segment SGI19530[] = {{&S104101[69], 8}, {NULL}};
const Segment SGI19531[] = {{&S104111[50], 8}, {NULL}};
const Segment SGI19532[] = {{&S104111[0], 12}, {NULL}};
const Segment SGI19533[] = {{&S104114[47], 8}, {NULL}};
const Segment SGI19534[] = {{&S104114[0], 12}, {NULL}};
const Segment SGI19535[] = {{&S104158[52], 7}, {NULL}};
const Segment SGI19536[] = {{&S104158[52], 18}, {NULL}};
const Segment SGI19537[] = {{&S104158[30], 14}, {NULL}};
const Segment SGI19538[] = {{&S104158[119], 3}, {NULL}};
const Segment SGI19539[] = {{&S104158[84], 6}, {NULL}};
const Segment SGI19540[] = {{&S104219[46], 14}, {NULL}};
const Segment SGI19541[] = {{&S104219[46], 47}, {NULL}};
const Segment SGI19542[] = {{&S104219[30], 8}, {NULL}};
const Segment SGI19543[] = {{&S105464[44], 8}, {NULL}};
const Segment SGI19544[] = {{&S105464[44], 19}, {NULL}};
const Segment SGI19545[] = {{&S105464[30], 6}, {NULL}};
const Segment SGI19546[] = {{&S102641[56], 11}, {NULL}};
const Segment SGI19547[] = {{&S102641[56], 23}, {NULL}};
const Segment SGI19548[] = {{&S102641[30], 18}, {NULL}};
const Segment SGI19549[] = {{&S102641[128], 3}, {NULL}};
const Segment SGI19550[] = {{&S102641[93], 6}, {NULL}};
const Segment SGI19551[] = {{&S102655[42], 4}, {NULL}};
const Segment SGI19552[] = {{&S102655[42], 15}, {NULL}};
const Segment SGI19553[] = {{&S102655[30], 4}, {NULL}};
const Segment SGI19554[] = {{&S102655[106], 3}, {NULL}};
const Segment SGI19555[] = {{&S102655[71], 6}, {NULL}};
const Segment SGI19556[] = {{&S102657[50], 14}, {NULL}};
const Segment SGI19557[] = {{&S102657[50], 26}, {NULL}};
const Segment SGI19558[] = {{&S102657[30], 12}, {NULL}};
const Segment SGI19559[] = {{&S102657[125], 3}, {NULL}};
const Segment SGI19560[] = {{&S102657[90], 6}, {NULL}};
const Segment SGI19561[] = {{&S105539[45], 11}, {NULL}};
const Segment SGI19562[] = {{&S105539[30], 6}, {NULL}};
const Segment SGI19563[] = {{&S105539[115], 3}, {NULL}};
const Segment SGI19564[] = {{&S105539[81], 5}, {NULL}};
const Segment SGI19565[] = {{&S105539[139], 13}, {NULL}};
const Segment SGI19566[] = {{&S105542[45], 14}, {NULL}};
const Segment SGI19567[] = {{&S105542[30], 6}, {NULL}};
const Segment SGI19568[] = {{&S105542[118], 3}, {NULL}};
const Segment SGI19569[] = {{&S105542[84], 5}, {NULL}};
const Segment SGI19570[] = {{&S105542[142], 13}, {NULL}};
const Segment SGI19571[] = {{&S104219[141], 3}, {NULL}};
const Segment SGI19572[] = {{&S104219[107], 5}, {NULL}};
const Segment SGI19573[] = {{&S104220[54], 14}, {NULL}};
const Segment SGI19574[] = {{&S104220[54], 25}, {NULL}};
const Segment SGI19575[] = {{&S104220[30], 16}, {NULL}};
const Segment SGI19576[] = {{&S104220[127], 3}, {NULL}};
const Segment SGI19577[] = {{&S104220[93], 5}, {NULL}};
const Segment SGI19578[] = {{&S102808[42], 5}, {NULL}};
const Segment SGI19579[] = {{&S102808[42], 16}, {NULL}};
const Segment SGI19580[] = {{&S102808[30], 4}, {NULL}};
const Segment SGI19581[] = {{&S102808[107], 3}, {NULL}};
const Segment SGI19582[] = {{&S102808[72], 6}, {NULL}};
const Segment SGI19583[] = {{&S102809[50], 5}, {NULL}};
const Segment SGI19584[] = {{&S102809[50], 16}, {NULL}};
const Segment SGI19585[] = {{&S102809[30], 12}, {NULL}};
const Segment SGI19586[] = {{&S102809[115], 3}, {NULL}};
const Segment SGI19587[] = {{&S102809[80], 6}, {NULL}};
const Segment SGI19588[] = {{&S103713[43], 7}, {NULL}};
const Segment SGI19589[] = {{&S103713[30], 4}, {NULL}};
const Segment SGI19590[] = {{&S103713[109], 3}, {NULL}};
const Segment SGI19591[] = {{&S103713[75], 5}, {NULL}};
const Segment SGI19592[] = {{&S103713[133], 13}, {NULL}};
const Segment SGI19593[] = {{&S103729[55], 10}, {NULL}};
const Segment SGI19594[] = {{&S103729[55], 21}, {NULL}};
const Segment SGI19595[] = {{&S103729[43], 4}, {NULL}};
const Segment SGI19596[] = {{&S103729[124], 3}, {NULL}};
const Segment SGI19597[] = {{&S103729[90], 5}, {NULL}};
const Segment SGI19598[] = {{&S103729[0], 10}, {NULL}};
const Segment SGI19599[] = {{&S102850[51], 6}, {NULL}};
const Segment SGI19600[] = {{&S102850[30], 12}, {NULL}};
const Segment SGI19601[] = {{&S102850[122], 3}, {NULL}};
const Segment SGI19602[] = {{&S102850[88], 5}, {NULL}};
const Segment SGI19603[] = {{&S102850[146], 13}, {NULL}};
const Segment SGI19604[] = {{&S104391[77], 7}, {NULL}};
const Segment SGI19605[] = {{&S104424[22], 8}, {NULL}};
const Segment SGI19606[] = {{&S104425[42], 7}, {NULL}};
const Segment SGI19607[] = {{&S104425[59], 8}, {NULL}};
const Segment SGI19608[] = {{&S102898[42], 10}, {NULL}};
const Segment SGI19609[] = {{&S105642[59], 10}, {NULL}};
const Segment SGI19610[] = {{&S102898[42], 22}, {NULL}};
const Segment SGI19611[] = {{&S105642[59], 21}, {NULL}};
const Segment SGI19612[] = {{&S102898[30], 4}, {NULL}};
const Segment SGI19613[] = {{&S105642[45], 6}, {NULL}};
const Segment SGI19614[] = {{&S102898[113], 3}, {NULL}};
const Segment SGI19615[] = {{&S105642[128], 3}, {NULL}};
const Segment SGI19616[] = {{&S102898[78], 6}, {NULL}};
const Segment SGI19617[] = {{&S105642[94], 5}, {NULL}};
const Segment SGI19618[] = {{&S105642[0], 14}, {NULL}};
const Segment SGI19619[] = {{&S102899[42], 11}, {NULL}};
const Segment SGI19620[] = {{&S102899[42], 23}, {NULL}};
const Segment SGI19621[] = {{&S102899[30], 4}, {NULL}};
const Segment SGI19622[] = {{&S105643[44], 10}, {NULL}};
const Segment SGI19623[] = {{&S105643[44], 21}, {NULL}};
const Segment SGI19624[] = {{&S105643[30], 6}, {NULL}};
const Segment SGI19625[] = {{&S102899[114], 3}, {NULL}};
const Segment SGI19626[] = {{&S102899[79], 6}, {NULL}};
const Segment SGI19627[] = {{&S105643[82], 6}, {NULL}};
const Segment SGI19628[] = {{&S104431[28], 8}, {NULL}};
const Segment SGI19629[] = {{&S105644[59], 10}, {NULL}};
const Segment SGI19630[] = {{&S105644[59], 21}, {NULL}};
const Segment SGI19631[] = {{&S105644[45], 6}, {NULL}};
const Segment SGI19632[] = {{&S105644[128], 3}, {NULL}};
const Segment SGI19633[] = {{&S105644[94], 5}, {NULL}};
const Segment SGI19634[] = {{&S105644[0], 14}, {NULL}};
const Segment SGI19635[] = {{&S104436[47], 8}, {NULL}};
const Segment SGI19636[] = {{&S104436[0], 12}, {NULL}};
const Segment SGI19637[] = {{&S104437[71], 8}, {NULL}};
const Segment SGI19638[] = {{&S104439[47], 8}, {NULL}};
const Segment SGI19639[] = {{&S104439[0], 12}, {NULL}};
const Segment SGI19640[] = {{&S104440[27], 8}, {&S104440[27], 8}, {NULL}};
const Segment SGI19641[] = {{&S104442[48], 8}, {&S104442[48], 8}, {NULL}};
const Segment SGI19642[] = {{&S104443[60], 8}, {&S104443[60], 8}, {NULL}};
const Segment SGI19643[] = {{&S104443[0], 13}, {NULL}};
const Segment SGI19644[] = {{&S104444[25], 8}, {NULL}};
const Segment SGI19645[] = {{&S104445[77], 7}, {NULL}};
const Segment SGI19646[] = {{&S104445[43], 8}, {&S104445[43], 8}, {NULL}};
const Segment SGI19647[] = {{&S104448[62], 8}, {NULL}};
const Segment SGI19648[] = {{&S104448[0], 13}, {NULL}};
const Segment SGI19649[] = {{&S104450[22], 8}, {NULL}};
const Segment SGI19650[] = {{&S104455[60], 8}, {&S104455[60], 8}, {NULL}};
const Segment SGI19651[] = {{&S104455[0], 13}, {NULL}};
const Segment SGI19652[] = {{&S104462[79], 8}, {NULL}};
const Segment SGI19653[] = {{&S104462[0], 13}, {NULL}};
const Segment SGI19654[] = {{&S105680[59], 10}, {NULL}};
const Segment SGI19655[] = {{&S105680[59], 21}, {NULL}};
const Segment SGI19656[] = {{&S105680[45], 6}, {NULL}};
const Segment SGI19657[] = {{&S105680[128], 3}, {NULL}};
const Segment SGI19658[] = {{&S105680[94], 5}, {NULL}};
const Segment SGI19659[] = {{&S105680[0], 14}, {NULL}};
const Segment SGI19660[] = {{&S104475[83], 7}, {NULL}};
const Segment SGI19661[] = {{&S104475[43], 8}, {NULL}};
const Segment SGI19662[] = {{&S103849[42], 102}, {NULL}};
const Segment SGI19663[] = {{&S103849[42], 113}, {NULL}};
const Segment SGI19664[] = {{&S103849[30], 4}, {NULL}};
const Segment SGI19665[] = {{&S103849[203], 3}, {NULL}};
const Segment SGI19666[] = {{&S103849[169], 5}, {NULL}};
const Segment SGI19667[] = {{&S104478[13], 8}, {NULL}};
const Segment SGI19668[] = {{&S104479[85], 7}, {NULL}};
const Segment SGI19669[] = {{&S104479[43], 8}, {NULL}};
const Segment SGI19670[] = {{&S104485[69], 8}, {NULL}};
const Segment SGI19671[] = {{&S104489[85], 7}, {NULL}};
const Segment SGI19672[] = {{&S104489[43], 8}, {NULL}};
const Segment SGI19673[] = {{&S104503[47], 8}, {NULL}};
const Segment SGI19674[] = {{&S104503[0], 12}, {NULL}};
const Segment SGI19675[] = {{&S104516[49], 8}, {NULL}};
const Segment SGI19676[] = {{&S104516[0], 12}, {NULL}};
const Segment SGI19677[] = {{&S105753[44], 10}, {NULL}};
const Segment SGI19678[] = {{&S105753[44], 21}, {NULL}};
const Segment SGI19679[] = {{&S105753[30], 6}, {NULL}};
const Segment SGI19680[] = {{&S105753[158], 3}, {NULL}};
const Segment SGI19681[] = {{&S105753[124], 3}, {NULL}};
const Segment SGI19682[] = {{&S105753[89], 6}, {NULL}};
const Segment SGI19683[] = {{&S105753[144], 8}, {NULL}};
const Segment SGI19684[] = {{&S104567[60], 10}, {NULL}};
const Segment SGI19685[] = {{&S104567[60], 23}, {NULL}};
const Segment SGI19686[] = {{&S104567[30], 22}, {NULL}};
const Segment SGI19687[] = {{&S104567[132], 3}, {NULL}};
const Segment SGI19688[] = {{&S104567[97], 6}, {NULL}};
const Segment SGI19689[] = {{&S105814[62], 10}, {NULL}};
const Segment SGI19690[] = {{&S105814[55], 6}, {NULL}};
const Segment SGI19691[] = {{&S103127[63], 10}, {NULL}};
const Segment SGI19692[] = {{&S103127[63], 22}, {NULL}};
const Segment SGI19693[] = {{&S103127[43], 12}, {NULL}};
const Segment SGI19694[] = {{&S103127[134], 3}, {NULL}};
const Segment SGI19695[] = {{&S103127[99], 6}, {NULL}};
const Segment SGI19696[] = {{&S104033[57], 10}, {NULL}};
const Segment SGI19697[] = {{&S104033[57], 21}, {NULL}};
const Segment SGI19698[] = {{&S104033[45], 4}, {NULL}};
const Segment SGI19699[] = {{&S104033[126], 3}, {NULL}};
const Segment SGI19700[] = {{&S104033[92], 5}, {NULL}};
const Segment SGI19701[] = {{&S104033[0], 14}, {NULL}};
const Segment SGI19702[] = {{&S103140[42], 10}, {NULL}};
const Segment SGI19703[] = {{&S103140[42], 22}, {NULL}};
const Segment SGI19704[] = {{&S103140[30], 4}, {NULL}};
const Segment SGI19705[] = {{&S103140[113], 3}, {NULL}};
const Segment SGI19706[] = {{&S103140[78], 6}, {NULL}};
const Segment SGI19707[] = {{&S103141[42], 10}, {NULL}};
const Segment SGI19708[] = {{&S104036[57], 10}, {NULL}};
const Segment SGI19709[] = {{&S103141[42], 22}, {NULL}};
const Segment SGI19710[] = {{&S104036[57], 21}, {NULL}};
const Segment SGI19711[] = {{&S103141[30], 4}, {NULL}};
const Segment SGI19712[] = {{&S104036[45], 4}, {NULL}};
const Segment SGI19713[] = {{&S103141[113], 3}, {NULL}};
const Segment SGI19714[] = {{&S104036[126], 3}, {NULL}};
const Segment SGI19715[] = {{&S103141[78], 6}, {NULL}};
const Segment SGI19716[] = {{&S104036[92], 5}, {NULL}};
const Segment SGI19717[] = {{&S104036[0], 14}, {NULL}};
const Segment SGI19718[] = {{&S105873[57], 10}, {NULL}};
const Segment SGI19719[] = {{&S105873[57], 21}, {NULL}};
const Segment SGI19720[] = {{&S105873[43], 6}, {NULL}};
const Segment SGI19721[] = {{&S105873[126], 3}, {NULL}};
const Segment SGI19722[] = {{&S105873[92], 5}, {NULL}};
const Segment SGI19723[] = {{&S105873[0], 12}, {NULL}};
const Segment SGI19724[] = {{&S105875[44], 10}, {NULL}};
const Segment SGI19725[] = {{&S105875[44], 21}, {NULL}};
const Segment SGI19726[] = {{&S105875[30], 6}, {NULL}};
const Segment SGI19727[] = {{&S105875[79], 6}, {NULL}};
const Segment SGI19728[] = {{&S105875[106], 13}, {NULL}};
const Segment SGI19729[] = {{&S103228[50], 12}, {NULL}};
const Segment SGI19730[] = {{&S103228[50], 23}, {NULL}};
const Segment SGI19731[] = {{&S103228[30], 12}, {NULL}};
const Segment SGI19732[] = {{&S103228[122], 3}, {NULL}};
const Segment SGI19733[] = {{&S103228[87], 6}, {NULL}};
const Segment SGI19734[] = {{&S104835[50], 10}, {NULL}};
const Segment SGI19735[] = {{&S104835[50], 25}, {NULL}};
const Segment SGI19736[] = {{&S104835[30], 12}, {NULL}};
const Segment SGI19737[] = {{&S104835[123], 3}, {NULL}};
const Segment SGI19738[] = {{&S104835[89], 5}, {NULL}};
const Segment SGI19739[] = {{&S104835[51], 8}, {NULL}};
const Segment SGI19740[] = {{&S104842[0], 13}, {NULL}};
const Segment SGI19741[] = {{&S104897[22], 8}, {NULL}};
const Segment SGI19742[] = {{&S104900[13], 8}, {NULL}};
const Segment SGI19743[] = {{&S104906[60], 8}, {&S104906[60], 8}, {NULL}};
const Segment SGI19744[] = {{&S104906[0], 13}, {NULL}};
const Segment SGI19745[] = {{&S104910[59], 8}, {&S104910[59], 8}, {&S104910[59], 8}, {NULL}};
const Segment SGI19746[] = {{&S104910[0], 13}, {NULL}};
const Segment SGI19747[] = {{&S104911[62], 8}, {&S104911[62], 8}, {NULL}};
const Segment SGI19748[] = {{&S104911[0], 13}, {NULL}};
const Segment SGI19749[] = {{&S104912[45], 8}, {NULL}};
const Segment SGI19750[] = {{&S104912[0], 12}, {NULL}};
const Segment SGI19751[] = {{&S104914[45], 8}, {NULL}};
const Segment SGI19752[] = {{&S104914[0], 12}, {NULL}};
const Segment SGI19753[] = {{&S104915[48], 8}, {&S104915[48], 8}, {NULL}};
const Segment SGI19754[] = {{&S104915[83], 6}, {NULL}};
const Segment SGI19755[] = {{&S103380[50], 5}, {NULL}};
const Segment SGI19756[] = {{&S103380[50], 17}, {NULL}};
const Segment SGI19757[] = {{&S103380[30], 12}, {NULL}};
const Segment SGI19758[] = {{&S103380[116], 3}, {NULL}};
const Segment SGI19759[] = {{&S103380[81], 6}, {NULL}};
const Segment SGI19760[] = {{&S104916[53], 8}, {&S104916[53], 8}, {NULL}};
const Segment SGI19761[] = {{&S104917[62], 8}, {NULL}};
const Segment SGI19762[] = {{&S104917[0], 13}, {NULL}};
const Segment SGI19763[] = {{&S104918[45], 8}, {NULL}};
const Segment SGI19764[] = {{&S104918[0], 12}, {NULL}};
const Segment SGI19765[] = {{&S104922[61], 8}, {NULL}};
const Segment SGI19766[] = {{&S104922[0], 13}, {NULL}};
const Segment SGI19767[] = {{&S104928[22], 8}, {NULL}};
const Segment SGI19768[] = {{&S104929[60], 8}, {&S104929[60], 8}, {NULL}};
const Segment SGI19769[] = {{&S104929[0], 13}, {NULL}};
const Segment SGI19770[] = {{&S104932[62], 8}, {NULL}};
const Segment SGI19771[] = {{&S104932[0], 13}, {NULL}};
const Segment SGI19772[] = {{&S104933[49], 8}, {NULL}};
const Segment SGI19773[] = {{&S104939[21], 8}, {NULL}};
const Segment SGI19774[] = {{&S104949[47], 8}, {NULL}};
const Segment SGI19775[] = {{&S104954[34], 8}, {NULL}};
const Segment SGI19776[] = {{&S104962[47], 8}, {NULL}};
const Segment SGI19777[] = {{&S104968[60], 8}, {&S104968[60], 8}, {NULL}};
const Segment SGI19778[] = {{&S104968[0], 13}, {NULL}};
const Segment SGI19779[] = {{&S104969[67], 8}, {NULL}};
const Segment SGI19780[] = {{&S104980[164], 8}, {NULL}};
const Segment SGI19781[] = {{&S104982[49], 8}, {NULL}};
const Segment SGI19782[] = {{&S104982[0], 12}, {NULL}};
const Segment SGI19783[] = {{&S104984[42], 8}, {NULL}};
const Segment SGI19784[] = {{&S104990[83], 7}, {NULL}};
const Segment SGI19785[] = {{&S104990[43], 8}, {NULL}};
const Segment SGI19786[] = {{&S105005[47], 8}, {NULL}};
const Segment SGI19787[] = {{&S105005[0], 12}, {NULL}};
const Segment SGI19788[] = {{&S105006[49], 8}, {NULL}};
const Segment SGI19789[] = {{&S105006[0], 12}, {NULL}};
const Segment SGI19790[] = {{&S105011[42], 8}, {NULL}};
const Segment SGI19791[] = {{&S103526[44], 10}, {NULL}};
const Segment SGI19792[] = {{&S103526[44], 21}, {NULL}};
const Segment SGI19793[] = {{&S103526[30], 6}, {NULL}};
const Segment SGI19794[] = {{&S103526[113], 3}, {NULL}};
const Segment SGI19795[] = {{&S103526[79], 5}, {NULL}};
const Segment SGI19796[] = {{&S104230[42], 10}, {NULL}};
const Segment SGI19797[] = {{&S104230[42], 21}, {NULL}};
const Segment SGI19798[] = {{&S104230[30], 4}, {NULL}};
const Segment SGI19799[] = {{&S104230[111], 3}, {NULL}};
const Segment SGI19800[] = {{&S104230[77], 5}, {NULL}};
const Segment SGI19801[] = {{&S105102[49], 10}, {NULL}};
const Segment SGI19802[] = {{&S105102[49], 21}, {NULL}};
const Segment SGI19803[] = {{&S105102[30], 11}, {NULL}};
const Segment SGI19804[] = {{&S105102[119], 3}, {NULL}};
const Segment SGI19805[] = {{&S105102[84], 6}, {NULL}};
const Segment SGI19806[] = {{&S105180[55], 10}, {NULL}};
const Segment SGI19807[] = {{&S105180[55], 22}, {NULL}};
const Segment SGI19808[] = {{&S105180[30], 17}, {NULL}};
const Segment SGI19809[] = {{&S105180[126], 3}, {NULL}};
const Segment SGI19810[] = {{&S105180[91], 6}, {NULL}};
const Segment SGI19811[] = {{&S105207[60], 10}, {NULL}};
const Segment SGI19812[] = {{&S105207[60], 21}, {NULL}};
const Segment SGI19813[] = {{&S105207[30], 22}, {NULL}};
const Segment SGI19814[] = {{&S105207[129], 3}, {NULL}};
const Segment SGI19815[] = {{&S105207[95], 5}, {NULL}};
const Segment SGI19816[] = {{&S105215[63], 9}, {NULL}};
const Segment SGI19817[] = {{&S105215[63], 20}, {NULL}};
const Segment SGI19818[] = {{&S105215[30], 25}, {NULL}};
const Segment SGI19819[] = {{&S105215[131], 3}, {NULL}};
const Segment SGI19820[] = {{&S105215[97], 5}, {NULL}};
const Segment SGI19821[] = {{&S105221[49], 6}, {NULL}};
const Segment SGI19822[] = {{&S105221[49], 23}, {NULL}};
const Segment SGI19823[] = {{&S105221[30], 11}, {NULL}};
const Segment SGI19824[] = {{&S105221[121], 3}, {NULL}};
const Segment SGI19825[] = {{&S105221[86], 6}, {NULL}};
const Segment SGI19826[] = {{&S104413[56], 10}, {NULL}};
const Segment SGI19827[] = {{&S104413[56], 21}, {NULL}};
const Segment SGI19828[] = {{&S104413[44], 4}, {NULL}};
const Segment SGI19829[] = {{&S104413[125], 3}, {NULL}};
const Segment SGI19830[] = {{&S104413[91], 5}, {NULL}};
const Segment SGI19831[] = {{&S104420[57], 10}, {NULL}};
const Segment SGI19832[] = {{&S104420[57], 21}, {NULL}};
const Segment SGI19833[] = {{&S104420[45], 4}, {NULL}};
const Segment SGI19834[] = {{&S104420[126], 3}, {NULL}};
const Segment SGI19835[] = {{&S104420[92], 5}, {NULL}};
const Segment SGI19836[] = {{&S104420[0], 14}, {NULL}};
const Segment SGI19837[] = {{&S104421[57], 11}, {NULL}};
const Segment SGI19838[] = {{&S104421[57], 22}, {NULL}};
const Segment SGI19839[] = {{&S104421[45], 4}, {NULL}};
const Segment SGI19840[] = {{&S104421[127], 3}, {NULL}};
const Segment SGI19841[] = {{&S104421[93], 5}, {NULL}};
const Segment SGI19842[] = {{&S105268[59], 10}, {NULL}};
const Segment SGI19843[] = {{&S105268[59], 21}, {NULL}};
const Segment SGI19844[] = {{&S104421[0], 14}, {NULL}};
const Segment SGI19845[] = {{&S105268[30], 21}, {NULL}};
const Segment SGI19846[] = {{&S105268[128], 3}, {NULL}};
const Segment SGI19847[] = {{&S105268[94], 5}, {NULL}};
const Segment SGI19848[] = {{&S103704[42], 12}, {NULL}};
const Segment SGI19849[] = {{&S103704[42], 23}, {NULL}};
const Segment SGI19850[] = {{&S103704[30], 4}, {NULL}};
const Segment SGI19851[] = {{&S103704[114], 3}, {NULL}};
const Segment SGI19852[] = {{&S103704[79], 6}, {NULL}};
const Segment SGI19853[] = {{&S104472[55], 10}, {NULL}};
const Segment SGI19854[] = {{&S104472[55], 21}, {NULL}};
const Segment SGI19855[] = {{&S104472[43], 4}, {NULL}};
const Segment SGI19856[] = {{&S104472[124], 3}, {NULL}};
const Segment SGI19857[] = {{&S104472[90], 5}, {NULL}};
const Segment SGI19858[] = {{&S104472[0], 10}, {NULL}};
const Segment SGI19859[] = {{&S103721[63], 6}, {NULL}};
const Segment SGI19860[] = {{&S103721[63], 24}, {NULL}};
const Segment SGI19861[] = {{&S103721[43], 12}, {NULL}};
const Segment SGI19862[] = {{&S103721[136], 3}, {NULL}};
const Segment SGI19863[] = {{&S103721[101], 6}, {NULL}};
const Segment SGI19864[] = {{&S103740[42], 10}, {NULL}};
const Segment SGI19865[] = {{&S103740[42], 21}, {NULL}};
const Segment SGI19866[] = {{&S103740[30], 4}, {NULL}};
const Segment SGI19867[] = {{&S103740[112], 3}, {NULL}};
const Segment SGI19868[] = {{&S103740[77], 6}, {NULL}};
const Segment SGI19869[] = {{&S103741[42], 10}, {NULL}};
const Segment SGI19870[] = {{&S103741[42], 22}, {NULL}};
const Segment SGI19871[] = {{&S103741[30], 4}, {NULL}};
const Segment SGI19872[] = {{&S103741[113], 3}, {NULL}};
const Segment SGI19873[] = {{&S103741[78], 6}, {NULL}};
const Segment SGI19874[] = {{&S103742[42], 10}, {NULL}};
const Segment SGI19875[] = {{&S103742[42], 21}, {NULL}};
const Segment SGI19876[] = {{&S103742[30], 4}, {NULL}};
const Segment SGI19877[] = {{&S103742[112], 3}, {NULL}};
const Segment SGI19878[] = {{&S103742[77], 6}, {NULL}};
const Segment SGI19879[] = {{&S105328[27], 8}, {&S105328[27], 8}, {&S105328[27], 8}, {NULL}};
const Segment SGI19880[] = {{&S105458[48], 8}, {&S105458[48], 8}, {NULL}};
const Segment SGI19881[] = {{&S105458[83], 12}, {NULL}};
const Segment SGI19882[] = {{&S104698[42], 11}, {NULL}};
const Segment SGI19883[] = {{&S104698[42], 22}, {NULL}};
const Segment SGI19884[] = {{&S104698[30], 4}, {NULL}};
const Segment SGI19885[] = {{&S104698[112], 3}, {NULL}};
const Segment SGI19886[] = {{&S104698[78], 5}, {NULL}};
const Segment SGI19887[] = {{&S104720[56], 10}, {NULL}};
const Segment SGI19888[] = {{&S104720[56], 21}, {NULL}};
const Segment SGI19889[] = {{&S104720[44], 4}, {NULL}};
const Segment SGI19890[] = {{&S104720[125], 3}, {NULL}};
const Segment SGI19891[] = {{&S104720[91], 5}, {NULL}};
const Segment SGI19892[] = {{&S104775[54], 10}, {NULL}};
const Segment SGI19893[] = {{&S104775[54], 21}, {NULL}};
const Segment SGI19894[] = {{&S104775[42], 4}, {NULL}};
const Segment SGI19895[] = {{&S104775[123], 3}, {NULL}};
const Segment SGI19896[] = {{&S104775[89], 5}, {NULL}};
const Segment SGI19897[] = {{&S104775[0], 11}, {NULL}};
const Segment SGI19898[] = {{&S105554[28], 9}, {NULL}};
const Segment SGI19899[] = {{&S105554[63], 6}, {NULL}};
const Segment SGI19900[] = {{&S103994[56], 15}, {NULL}};
const Segment SGI19901[] = {{&S103994[56], 27}, {NULL}};
const Segment SGI19902[] = {{&S103994[30], 18}, {NULL}};
const Segment SGI19903[] = {{&S103994[132], 3}, {NULL}};
const Segment SGI19904[] = {{&S103994[97], 6}, {NULL}};
const Segment SGI19905[] = {{&S104821[29], 10}, {NULL}};
const Segment SGI19906[] = {{&S104821[54], 3}, {NULL}};
const Segment SGI19907[] = {{&S104823[42], 10}, {NULL}};
const Segment SGI19908[] = {{&S104823[42], 21}, {NULL}};
const Segment SGI19909[] = {{&S104823[30], 4}, {NULL}};
const Segment SGI19910[] = {{&S104823[111], 3}, {NULL}};
const Segment SGI19911[] = {{&S104823[77], 5}, {NULL}};
const Segment SGI19912[] = {{&S104831[55], 9}, {NULL}};
const Segment SGI19913[] = {{&S104831[55], 20}, {NULL}};
const Segment SGI19914[] = {{&S104831[43], 4}, {NULL}};
const Segment SGI19915[] = {{&S104831[123], 3}, {NULL}};
const Segment SGI19916[] = {{&S104831[89], 5}, {NULL}};
const Segment SGI19917[] = {{&S104831[0], 10}, {NULL}};
const Segment SGI19918[] = {{&S104844[42], 9}, {NULL}};
const Segment SGI19919[] = {{&S104844[42], 20}, {NULL}};
const Segment SGI19920[] = {{&S104844[30], 4}, {NULL}};
const Segment SGI19921[] = {{&S104844[110], 3}, {NULL}};
const Segment SGI19922[] = {{&S104844[76], 5}, {NULL}};
const Segment SGI19923[] = {{&S105611[61], 8}, {NULL}};
const Segment SGI19924[] = {{&S105611[0], 13}, {NULL}};
const Segment SGI19925[] = {{&S105612[77], 7}, {NULL}};
const Segment SGI19926[] = {{&S105612[43], 8}, {&S105612[43], 8}, {NULL}};
const Segment SGI19927[] = {{&S104865[56], 10}, {NULL}};
const Segment SGI19928[] = {{&S104865[56], 21}, {NULL}};
const Segment SGI19929[] = {{&S104865[44], 4}, {NULL}};
const Segment SGI19930[] = {{&S104865[125], 3}, {NULL}};
const Segment SGI19931[] = {{&S104865[91], 5}, {NULL}};
const Segment SGI19932[] = {{&S105630[79], 7}, {NULL}};
const Segment SGI19933[] = {{&S105630[43], 8}, {&S105630[43], 8}, {NULL}};
const Segment SGI19934[] = {{&S104878[55], 10}, {NULL}};
const Segment SGI19935[] = {{&S104878[55], 21}, {NULL}};
const Segment SGI19936[] = {{&S104878[43], 4}, {NULL}};
const Segment SGI19937[] = {{&S104878[124], 3}, {NULL}};
const Segment SGI19938[] = {{&S104878[90], 5}, {NULL}};
const Segment SGI19939[] = {{&S104878[0], 10}, {NULL}};
const Segment SGI19940[] = {{&S105638[79], 8}, {NULL}};
const Segment SGI19941[] = {{&S105638[0], 13}, {NULL}};
const Segment SGI19942[] = {{&S105647[60], 8}, {&S105647[60], 8}, {NULL}};
const Segment SGI19943[] = {{&S105647[0], 13}, {NULL}};
const Segment SGI19944[] = {{&S105649[114], 8}, {NULL}};
const Segment SGI19945[] = {{&S105654[56], 8}, {&S105654[56], 8}, {&S105654[56], 8}, {NULL}};
const Segment SGI19946[] = {{&S105660[48], 8}, {&S105660[48], 8}, {NULL}};
const Segment SGI19947[] = {{&S105660[83], 12}, {NULL}};
const Segment SGI19948[] = {{&S105676[53], 8}, {&S105676[53], 8}, {&S105676[53], 8}, {NULL}};
const Segment SGI19949[] = {{&S105679[62], 8}, {NULL}};
const Segment SGI19950[] = {{&S105679[0], 13}, {NULL}};
const Segment SGI19951[] = {{&S105681[53], 8}, {&S105681[53], 8}, {&S105681[53], 8}, {&S105681[53], 8}, {&S105681[53], 8}, {NULL}};
const Segment SGI19952[] = {{&S105681[117], 9}, {NULL}};
const Segment SGI19953[] = {{&S105687[40], 8}, {NULL}};
const Segment SGI19954[] = {{&S105688[60], 8}, {&S105688[60], 8}, {NULL}};
const Segment SGI19955[] = {{&S105688[0], 13}, {NULL}};
const Segment SGI19956[] = {{&S105694[40], 8}, {NULL}};
const Segment SGI19957[] = {{&S105697[61], 8}, {NULL}};
const Segment SGI19958[] = {{&S105697[0], 13}, {NULL}};
const Segment SGI19959[] = {{&S105707[55], 8}, {NULL}};
const Segment SGI19960[] = {{&S105714[67], 8}, {NULL}};
const Segment SGI19961[] = {{&S105715[63], 8}, {NULL}};
const Segment SGI19962[] = {{&S105716[62], 8}, {&S105716[62], 8}, {NULL}};
const Segment SGI19963[] = {{&S105716[0], 13}, {NULL}};
const Segment SGI19964[] = {{&S105722[58], 8}, {NULL}};
const Segment SGI19965[] = {{&S105722[41], 9}, {NULL}};
const Segment SGI19966[] = {{&S105722[24], 8}, {NULL}};
const Segment SGI19967[] = {{&S105737[42], 8}, {NULL}};
const Segment SGI19968[] = {{&S105747[53], 8}, {&S105747[53], 8}, {&S105747[53], 8}, {NULL}};
const Segment SGI19969[] = {{&S105748[40], 8}, {NULL}};
const Segment SGI19970[] = {{&S105749[31], 3}, {NULL}};
const Segment SGI19971[] = {{&S105749[43], 8}, {NULL}};
const Segment SGI19972[] = {{&S105775[55], 8}, {NULL}};
const Segment SGI19973[] = {{&S105777[54], 8}, {&S105777[54], 8}, {&S105777[54], 8}, {NULL}};
const Segment SGI19974[] = {{&S105778[62], 8}, {NULL}};
const Segment SGI19975[] = {{&S105778[0], 13}, {NULL}};
const Segment SGI19976[] = {{&S105788[81], 7}, {NULL}};
const Segment SGI19977[] = {{&S105788[43], 8}, {NULL}};
const Segment SGI19978[] = {{&S105796[60], 8}, {&S105796[60], 8}, {NULL}};
const Segment SGI19979[] = {{&S105796[0], 13}, {NULL}};
const Segment SGI19980[] = {{&S105801[50], 8}, {NULL}};
const Segment SGI19981[] = {{&S105801[0], 12}, {NULL}};
const Segment SGI19982[] = {{&S104287[50], 7}, {NULL}};
const Segment SGI19983[] = {{&S104287[50], 29}, {NULL}};
const Segment SGI19984[] = {{&S104287[30], 12}, {NULL}};
const Segment SGI19985[] = {{&S104287[171], 3}, {NULL}};
const Segment SGI19986[] = {{&S104287[137], 3}, {NULL}};
const Segment SGI19987[] = {{&S104287[102], 6}, {NULL}};
const Segment SGI19988[] = {{&S104287[157], 8}, {NULL}};
const Segment SGI19989[] = {{&S105838[48], 8}, {NULL}};
const Segment SGI19990[] = {{&S105838[0], 12}, {NULL}};
const Segment SGI19991[] = {{&S105864[89], 8}, {NULL}};
const Segment SGI19992[] = {{&S105888[40], 8}, {NULL}};
const Segment SGI19993[] = {{&S104374[50], 7}, {NULL}};
const Segment SGI19994[] = {{&S104374[50], 19}, {NULL}};
const Segment SGI19995[] = {{&S104374[30], 12}, {NULL}};
const Segment SGI19996[] = {{&S104374[86], 4}, {NULL}};
const Segment SGI19997[] = {{&S104374[86], 3}, {NULL}};
const Segment SGI19998[] = {{&S104377[62], 6}, {NULL}};
const Segment SGI19999[] = {{&S104377[62], 24}, {NULL}};
const Segment SGI20000[] = {{&S104377[42], 12}, {NULL}};
const Segment SGI20001[] = {{&S104377[135], 3}, {NULL}};
const Segment SGI20002[] = {{&S104377[100], 6}, {NULL}};
const Segment SGI20003[] = {{&S105208[37], 16}, {NULL}};
const Segment SGI20004[] = {{&S105208[109], 3}, {NULL}};
const Segment SGI20005[] = {{&S105208[98], 3}, {NULL}};
const Segment SGI20006[] = {{&S105208[75], 6}, {NULL}};
const Segment SGI20007[] = {{&S105356[42], 10}, {NULL}};
const Segment SGI20008[] = {{&S105356[42], 31}, {NULL}};
const Segment SGI20009[] = {{&S105356[30], 4}, {NULL}};
const Segment SGI20010[] = {{&S105356[165], 3}, {NULL}};
const Segment SGI20011[] = {{&S105356[131], 3}, {NULL}};
const Segment SGI20012[] = {{&S105356[96], 6}, {NULL}};
const Segment SGI20013[] = {{&S105356[151], 8}, {NULL}};
const Segment SGI20014[] = {{&S105452[57], 12}, {NULL}};
const Segment SGI20015[] = {{&S105452[57], 23}, {NULL}};
const Segment SGI20016[] = {{&S105452[45], 4}, {NULL}};
const Segment SGI20017[] = {{&S105452[128], 3}, {NULL}};
const Segment SGI20018[] = {{&S105452[94], 5}, {NULL}};
const Segment SGI20019[] = {{&S105452[0], 14}, {NULL}};
const Segment SGI20020[] = {{&S105456[54], 10}, {NULL}};
const Segment SGI20021[] = {{&S105456[54], 21}, {NULL}};
const Segment SGI20022[] = {{&S105456[42], 4}, {NULL}};
const Segment SGI20023[] = {{&S105456[123], 3}, {NULL}};
const Segment SGI20024[] = {{&S105456[89], 5}, {NULL}};
const Segment SGI20025[] = {{&S105456[0], 11}, {NULL}};
const Segment SGI20026[] = {{&S104724[64], 6}, {NULL}};
const Segment SGI20027[] = {{&S104724[64], 23}, {NULL}};
const Segment SGI20028[] = {{&S104724[44], 12}, {NULL}};
const Segment SGI20029[] = {{&S104724[136], 3}, {NULL}};
const Segment SGI20030[] = {{&S104724[101], 6}, {NULL}};
const Segment SGI20031[] = {{&S105526[56], 12}, {NULL}};
const Segment SGI20032[] = {{&S105526[56], 23}, {NULL}};
const Segment SGI20033[] = {{&S105526[44], 4}, {NULL}};
const Segment SGI20034[] = {{&S105526[127], 3}, {NULL}};
const Segment SGI20035[] = {{&S105526[93], 5}, {NULL}};
const Segment SGI20036[] = {{&S105528[43], 10}, {NULL}};
const Segment SGI20037[] = {{&S105528[30], 4}, {NULL}};
const Segment SGI20038[] = {{&S105528[112], 3}, {NULL}};
const Segment SGI20039[] = {{&S105528[78], 5}, {NULL}};
const Segment SGI20040[] = {{&S105528[136], 13}, {NULL}};
const Segment SGI20041[] = {{&S104820[37], 8}, {NULL}};
const Segment SGI20042[] = {{&S104820[60], 3}, {NULL}};
const Segment SGI20043[] = {{&S105593[57], 10}, {NULL}};
const Segment SGI20044[] = {{&S105593[57], 21}, {NULL}};
const Segment SGI20045[] = {{&S105593[45], 4}, {NULL}};
const Segment SGI20046[] = {{&S105593[126], 3}, {NULL}};
const Segment SGI20047[] = {{&S105593[92], 5}, {NULL}};
const Segment SGI20048[] = {{&S105593[0], 14}, {NULL}};
const Segment SGI20049[] = {{&S104822[64], 6}, {NULL}};
const Segment SGI20050[] = {{&S104822[64], 24}, {NULL}};
const Segment SGI20051[] = {{&S104822[44], 12}, {NULL}};
const Segment SGI20052[] = {{&S104822[137], 3}, {NULL}};
const Segment SGI20053[] = {{&S104822[102], 6}, {NULL}};
const Segment SGI20054[] = {{&S104824[36], 10}, {NULL}};
const Segment SGI20055[] = {{&S104824[31], 4}, {NULL}};
const Segment SGI20056[] = {{&S104826[30], 12}, {NULL}};
const Segment SGI20057[] = {{&S104826[97], 3}, {NULL}};
const Segment SGI20058[] = {{&S104826[131], 3}, {NULL}};
const Segment SGI20059[] = {{&S104826[63], 5}, {NULL}};
const Segment SGI20060[] = {{&S105645[56], 10}, {NULL}};
const Segment SGI20061[] = {{&S105645[56], 21}, {NULL}};
const Segment SGI20062[] = {{&S105645[44], 4}, {NULL}};
const Segment SGI20063[] = {{&S105645[125], 3}, {NULL}};
const Segment SGI20064[] = {{&S105645[91], 5}, {NULL}};
const Segment SGI20065[] = {{&S105655[57], 10}, {NULL}};
const Segment SGI20066[] = {{&S105655[57], 21}, {NULL}};
const Segment SGI20067[] = {{&S105655[45], 4}, {NULL}};
const Segment SGI20068[] = {{&S105655[126], 3}, {NULL}};
const Segment SGI20069[] = {{&S105655[92], 5}, {NULL}};
const Segment SGI20070[] = {{&S105655[0], 14}, {NULL}};
const Segment SGI20071[] = {{&S105826[54], 10}, {NULL}};
const Segment SGI20072[] = {{&S105826[54], 21}, {NULL}};
const Segment SGI20073[] = {{&S105826[42], 4}, {NULL}};
const Segment SGI20074[] = {{&S105826[123], 3}, {NULL}};
const Segment SGI20075[] = {{&S105826[89], 5}, {NULL}};
const Segment SGI20076[] = {{&S105826[0], 11}, {NULL}};
const Segment SGI20077[] = {{&S105012[42], 10}, {NULL}};
const Segment SGI20078[] = {{&S105012[42], 21}, {NULL}};
const Segment SGI20079[] = {{&S105012[30], 4}, {NULL}};
const Segment SGI20080[] = {{&S105012[112], 3}, {NULL}};
const Segment SGI20081[] = {{&S105012[77], 6}, {NULL}};
const Segment SGI20082[] = {{&S105197[50], 6}, {NULL}};
const Segment SGI20083[] = {{&S105197[50], 18}, {NULL}};
const Segment SGI20084[] = {{&S105197[30], 12}, {NULL}};
const Segment SGI20085[] = {{&S105197[117], 3}, {NULL}};
const Segment SGI20086[] = {{&S105197[82], 6}, {NULL}};
const Segment SGI20087[] = {{&S105209[50], 10}, {NULL}};
const Segment SGI20088[] = {{&S105209[50], 21}, {NULL}};
const Segment SGI20089[] = {{&S105209[30], 12}, {NULL}};
const Segment SGI20090[] = {{&S105209[120], 3}, {NULL}};
const Segment SGI20091[] = {{&S105209[85], 6}, {NULL}};
const Segment SGI20092[] = {{&S105219[44], 9}, {NULL}};
const Segment SGI20093[] = {{&S105219[44], 21}, {NULL}};
const Segment SGI20094[] = {{&S105219[30], 6}, {NULL}};
const Segment SGI20095[] = {{&S105219[113], 3}, {NULL}};
const Segment SGI20096[] = {{&S105219[79], 5}, {NULL}};
const Segment SGI20097[] = {{&S105288[49], 9}, {NULL}};
const Segment SGI20098[] = {{&S105288[49], 27}, {NULL}};
const Segment SGI20099[] = {{&S105288[29], 12}, {NULL}};
const Segment SGI20100[] = {{&S105288[140], 3}, {NULL}};
const Segment SGI20101[] = {{&S105288[106], 6}, {NULL}};
const Segment SGI20102[] = {{&S105291[44], 10}, {NULL}};
const Segment SGI20103[] = {{&S105291[44], 21}, {NULL}};
const Segment SGI20104[] = {{&S105291[30], 6}, {NULL}};
const Segment SGI20105[] = {{&S105291[113], 3}, {NULL}};
const Segment SGI20106[] = {{&S105291[79], 5}, {NULL}};
const Segment SGI20107[] = {{&S105551[42], 9}, {NULL}};
const Segment SGI20108[] = {{&S105551[42], 21}, {NULL}};
const Segment SGI20109[] = {{&S105551[30], 4}, {NULL}};
const Segment SGI20110[] = {{&S105551[112], 3}, {NULL}};
const Segment SGI20111[] = {{&S105551[77], 6}, {NULL}};
const Segment SGI20112[] = {{&S105598[42], 10}, {NULL}};
const Segment SGI20113[] = {{&S105598[42], 21}, {NULL}};
const Segment SGI20114[] = {{&S105598[30], 4}, {NULL}};
const Segment SGI20115[] = {{&S105598[111], 3}, {NULL}};
const Segment SGI20116[] = {{&S105598[77], 5}, {NULL}};
const Segment SGI20117[] = {{&S105608[42], 10}, {NULL}};
const Segment SGI20118[] = {{&S105608[42], 22}, {NULL}};
const Segment SGI20119[] = {{&S105608[30], 4}, {NULL}};
const Segment SGI20120[] = {{&S105608[113], 3}, {NULL}};
const Segment SGI20121[] = {{&S105608[78], 6}, {NULL}};
const Segment SGI20122[] = {{&S105659[42], 10}, {NULL}};
const Segment SGI20123[] = {{&S105659[42], 21}, {NULL}};
const Segment SGI20124[] = {{&S105659[30], 4}, {NULL}};
const Segment SGI20125[] = {{&S105659[112], 3}, {NULL}};
const Segment SGI20126[] = {{&S105659[77], 6}, {NULL}};
const Segment SGI20127[] = {{&S105704[42], 10}, {NULL}};
const Segment SGI20128[] = {{&S105704[42], 21}, {NULL}};
const Segment SGI20129[] = {{&S105704[30], 4}, {NULL}};
const Segment SGI20130[] = {{&S105704[112], 3}, {NULL}};
const Segment SGI20131[] = {{&S105704[77], 6}, {NULL}};
const Segment SGI20132[] = {{&S4653[42], 10}, {NULL}};
const Segment SGI20133[] = {{&S4653[42], 21}, {NULL}};
const Segment SGI20134[] = {{&S4653[30], 4}, {NULL}};
const Segment SGI20135[] = {{&S4653[111], 3}, {NULL}};
const Segment SGI20136[] = {{&S4653[77], 5}, {NULL}};
const Segment SGI20137[] = {{&S4694[44], 10}, {NULL}};
const Segment SGI20138[] = {{&S4694[44], 27}, {NULL}};
const Segment SGI20139[] = {{&S4694[30], 6}, {NULL}};
const Segment SGI20140[] = {{&S4694[119], 3}, {NULL}};
const Segment SGI20141[] = {{&S4694[85], 5}, {NULL}};
const Segment SGI20142[] = {{&S4587[44], 10}, {NULL}};
const Segment SGI20143[] = {{&S4587[44], 21}, {NULL}};
const Segment SGI20144[] = {{&S4587[30], 6}, {NULL}};
const Segment SGI20145[] = {{&S4587[113], 3}, {NULL}};
const Segment SGI20146[] = {{&S4587[79], 5}, {NULL}};
const Segment SGI20147[] = {{&S4683[44], 10}, {NULL}};
const Segment SGI20148[] = {{&S4683[44], 27}, {NULL}};
const Segment SGI20149[] = {{&S4683[30], 6}, {NULL}};
const Segment SGI20150[] = {{&S4683[119], 3}, {NULL}};
const Segment SGI20151[] = {{&S4683[85], 5}, {NULL}};
const Segment SGI20152[] = {{&S11577[44], 14}, {NULL}};
const Segment SGI20153[] = {{&S11577[44], 26}, {NULL}};
const Segment SGI20154[] = {{&S11577[30], 6}, {NULL}};
const Segment SGI20155[] = {{&S11577[118], 3}, {NULL}};
const Segment SGI20156[] = {{&S11577[84], 5}, {NULL}};
const Segment SGI20157[] = {{&S11578[44], 12}, {NULL}};
const Segment SGI20158[] = {{&S11578[44], 24}, {NULL}};
const Segment SGI20159[] = {{&S11578[30], 6}, {NULL}};
const Segment SGI20160[] = {{&S11578[116], 3}, {NULL}};
const Segment SGI20161[] = {{&S11578[82], 5}, {NULL}};
const Segment SGI20162[] = {{&S11589[44], 23}, {NULL}};
const Segment SGI20163[] = {{&S11589[44], 35}, {NULL}};
const Segment SGI20164[] = {{&S11589[30], 6}, {NULL}};
const Segment SGI20165[] = {{&S11589[127], 3}, {NULL}};
const Segment SGI20166[] = {{&S11589[93], 5}, {NULL}};
const Segment SGI20167[] = {{&S11590[44], 23}, {NULL}};
const Segment SGI20168[] = {{&S11590[44], 35}, {NULL}};
const Segment SGI20169[] = {{&S11590[30], 6}, {NULL}};
const Segment SGI20170[] = {{&S11590[127], 3}, {NULL}};
const Segment SGI20171[] = {{&S11590[93], 5}, {NULL}};
const Segment SGI20172[] = {{&S11591[38], 12}, {NULL}};
const Segment SGI20173[] = {{&S11591[31], 6}, {NULL}};
const Segment SGI20174[] = {{&S11592[44], 18}, {NULL}};
const Segment SGI20175[] = {{&S11592[44], 30}, {NULL}};
const Segment SGI20176[] = {{&S11592[30], 6}, {NULL}};
const Segment SGI20177[] = {{&S11592[122], 3}, {NULL}};
const Segment SGI20178[] = {{&S11592[88], 5}, {NULL}};
const Segment SGI20179[] = {{&S11597[44], 23}, {NULL}};
const Segment SGI20180[] = {{&S11597[44], 35}, {NULL}};
const Segment SGI20181[] = {{&S11597[30], 6}, {NULL}};
const Segment SGI20182[] = {{&S11597[127], 3}, {NULL}};
const Segment SGI20183[] = {{&S11597[93], 5}, {NULL}};
const Segment SGI20184[] = {{&S11598[44], 11}, {NULL}};
const Segment SGI20185[] = {{&S11598[44], 23}, {NULL}};
const Segment SGI20186[] = {{&S11598[30], 6}, {NULL}};
const Segment SGI20187[] = {{&S11598[115], 3}, {NULL}};
const Segment SGI20188[] = {{&S11598[81], 5}, {NULL}};
const Segment SGI20189[] = {{&S11599[44], 12}, {NULL}};
const Segment SGI20190[] = {{&S11599[44], 24}, {NULL}};
const Segment SGI20191[] = {{&S11599[30], 6}, {NULL}};
const Segment SGI20192[] = {{&S11599[116], 3}, {NULL}};
const Segment SGI20193[] = {{&S11599[82], 5}, {NULL}};
const Segment SGI20194[] = {{&S11600[44], 23}, {NULL}};
const Segment SGI20195[] = {{&S11600[44], 35}, {NULL}};
const Segment SGI20196[] = {{&S11600[30], 6}, {NULL}};
const Segment SGI20197[] = {{&S11600[127], 3}, {NULL}};
const Segment SGI20198[] = {{&S11600[93], 5}, {NULL}};
const Segment SGI20199[] = {{&S11606[42], 12}, {NULL}};
const Segment SGI20200[] = {{&S11606[30], 6}, {NULL}};
const Segment SGI20201[] = {{&S11606[114], 3}, {NULL}};
const Segment SGI20202[] = {{&S11606[80], 5}, {NULL}};
const Segment SGI20203[] = {{&S11607[44], 12}, {NULL}};
const Segment SGI20204[] = {{&S11607[44], 24}, {NULL}};
const Segment SGI20205[] = {{&S11607[30], 6}, {NULL}};
const Segment SGI20206[] = {{&S11607[85], 4}, {NULL}};
const Segment SGI20207[] = {{&S11607[85], 3}, {NULL}};
const Segment SGI20208[] = {{&S11608[44], 23}, {NULL}};
const Segment SGI20209[] = {{&S11608[44], 35}, {NULL}};
const Segment SGI20210[] = {{&S11608[30], 6}, {NULL}};
const Segment SGI20211[] = {{&S11608[127], 3}, {NULL}};
const Segment SGI20212[] = {{&S11608[93], 5}, {NULL}};
const Segment SGI20213[] = {{&S4558[42], 10}, {NULL}};
const Segment SGI20214[] = {{&S4558[30], 6}, {NULL}};
const Segment SGI20215[] = {{&S4558[117], 3}, {NULL}};
const Segment SGI20216[] = {{&S4558[83], 5}, {NULL}};
const Segment SGI20217[] = {{&S4559[44], 11}, {NULL}};
const Segment SGI20218[] = {{&S4559[44], 28}, {NULL}};
const Segment SGI20219[] = {{&S4559[30], 6}, {NULL}};
const Segment SGI20220[] = {{&S4559[120], 3}, {NULL}};
const Segment SGI20221[] = {{&S4559[86], 5}, {NULL}};
const Segment SGI20222[] = {{&S4560[44], 11}, {NULL}};
const Segment SGI20223[] = {{&S4560[44], 28}, {NULL}};
const Segment SGI20224[] = {{&S4560[30], 6}, {NULL}};
const Segment SGI20225[] = {{&S4560[120], 3}, {NULL}};
const Segment SGI20226[] = {{&S4560[86], 5}, {NULL}};
const Segment SGI20227[] = {{&S4561[44], 10}, {NULL}};
const Segment SGI20228[] = {{&S4561[44], 27}, {NULL}};
const Segment SGI20229[] = {{&S4561[30], 6}, {NULL}};
const Segment SGI20230[] = {{&S4561[119], 3}, {NULL}};
const Segment SGI20231[] = {{&S4561[85], 5}, {NULL}};
const Segment SGI20232[] = {{&S4563[44], 29}, {NULL}};
const Segment SGI20233[] = {{&S4563[44], 46}, {NULL}};
const Segment SGI20234[] = {{&S4563[30], 6}, {NULL}};
const Segment SGI20235[] = {{&S4563[138], 3}, {NULL}};
const Segment SGI20236[] = {{&S4563[104], 5}, {NULL}};
const Segment SGI20237[] = {{&S4564[44], 10}, {NULL}};
const Segment SGI20238[] = {{&S4564[44], 35}, {NULL}};
const Segment SGI20239[] = {{&S4564[30], 6}, {NULL}};
const Segment SGI20240[] = {{&S4564[127], 3}, {NULL}};
const Segment SGI20241[] = {{&S4564[93], 5}, {NULL}};
const Segment SGI20242[] = {{&S4565[44], 29}, {NULL}};
const Segment SGI20243[] = {{&S4565[44], 46}, {NULL}};
const Segment SGI20244[] = {{&S4565[30], 6}, {NULL}};
const Segment SGI20245[] = {{&S4565[138], 3}, {NULL}};
const Segment SGI20246[] = {{&S4565[104], 5}, {NULL}};
const Segment SGI20247[] = {{&S4566[44], 28}, {NULL}};
const Segment SGI20248[] = {{&S4566[44], 45}, {NULL}};
const Segment SGI20249[] = {{&S4566[30], 6}, {NULL}};
const Segment SGI20250[] = {{&S4566[137], 3}, {NULL}};
const Segment SGI20251[] = {{&S4566[103], 5}, {NULL}};
const Segment SGI20252[] = {{&S4567[44], 28}, {NULL}};
const Segment SGI20253[] = {{&S4567[44], 45}, {NULL}};
const Segment SGI20254[] = {{&S4567[30], 6}, {NULL}};
const Segment SGI20255[] = {{&S4567[137], 3}, {NULL}};
const Segment SGI20256[] = {{&S4567[103], 5}, {NULL}};
const Segment SGI20257[] = {{&S4568[44], 28}, {NULL}};
const Segment SGI20258[] = {{&S4568[44], 45}, {NULL}};
const Segment SGI20259[] = {{&S4568[30], 6}, {NULL}};
const Segment SGI20260[] = {{&S4568[137], 3}, {NULL}};
const Segment SGI20261[] = {{&S4568[103], 5}, {NULL}};
const Segment SGI20262[] = {{&S4569[43], 10}, {NULL}};
const Segment SGI20263[] = {{&S4569[43], 27}, {NULL}};
const Segment SGI20264[] = {{&S4569[30], 5}, {NULL}};
const Segment SGI20265[] = {{&S4569[118], 3}, {NULL}};
const Segment SGI20266[] = {{&S4569[84], 5}, {NULL}};
const Segment SGI20267[] = {{&S4570[43], 10}, {NULL}};
const Segment SGI20268[] = {{&S4570[43], 27}, {NULL}};
const Segment SGI20269[] = {{&S4570[118], 3}, {NULL}};
const Segment SGI20270[] = {{&S4570[84], 5}, {NULL}};
const Segment SGI20271[] = {{&S4571[44], 9}, {NULL}};
const Segment SGI20272[] = {{&S4571[44], 26}, {NULL}};
const Segment SGI20273[] = {{&S4571[30], 6}, {NULL}};
const Segment SGI20274[] = {{&S4571[118], 3}, {NULL}};
const Segment SGI20275[] = {{&S4571[84], 5}, {NULL}};
const Segment SGI20276[] = {{&S4572[44], 28}, {NULL}};
const Segment SGI20277[] = {{&S4572[44], 45}, {NULL}};
const Segment SGI20278[] = {{&S4572[30], 6}, {NULL}};
const Segment SGI20279[] = {{&S4572[137], 3}, {NULL}};
const Segment SGI20280[] = {{&S4572[103], 5}, {NULL}};
const Segment SGI20281[] = {{&S4574[44], 29}, {NULL}};
const Segment SGI20282[] = {{&S4574[44], 46}, {NULL}};
const Segment SGI20283[] = {{&S4574[30], 6}, {NULL}};
const Segment SGI20284[] = {{&S4574[138], 3}, {NULL}};
const Segment SGI20285[] = {{&S4574[104], 5}, {NULL}};
const Segment SGI20286[] = {{&S4575[44], 29}, {NULL}};
const Segment SGI20287[] = {{&S4575[44], 46}, {NULL}};
const Segment SGI20288[] = {{&S4575[30], 6}, {NULL}};
const Segment SGI20289[] = {{&S4575[138], 3}, {NULL}};
const Segment SGI20290[] = {{&S4575[104], 5}, {NULL}};
const Segment SGI20291[] = {{&S4576[44], 29}, {NULL}};
const Segment SGI20292[] = {{&S4576[44], 46}, {NULL}};
const Segment SGI20293[] = {{&S4576[30], 6}, {NULL}};
const Segment SGI20294[] = {{&S4576[138], 3}, {NULL}};
const Segment SGI20295[] = {{&S4576[104], 5}, {NULL}};
const Segment SGI20296[] = {{&S4577[44], 28}, {NULL}};
const Segment SGI20297[] = {{&S4577[44], 45}, {NULL}};
const Segment SGI20298[] = {{&S4577[30], 6}, {NULL}};
const Segment SGI20299[] = {{&S4577[137], 3}, {NULL}};
const Segment SGI20300[] = {{&S4577[103], 5}, {NULL}};
const Segment SGI20301[] = {{&S4578[44], 29}, {NULL}};
const Segment SGI20302[] = {{&S4578[44], 46}, {NULL}};
const Segment SGI20303[] = {{&S4578[30], 6}, {NULL}};
const Segment SGI20304[] = {{&S4578[138], 3}, {NULL}};
const Segment SGI20305[] = {{&S4578[104], 5}, {NULL}};
const Segment SGI20306[] = {{&S4579[42], 11}, {NULL}};
const Segment SGI20307[] = {{&S4579[30], 6}, {NULL}};
const Segment SGI20308[] = {{&S4579[118], 3}, {NULL}};
const Segment SGI20309[] = {{&S4579[84], 5}, {NULL}};
const Segment SGI20310[] = {{&S4582[44], 28}, {NULL}};
const Segment SGI20311[] = {{&S4582[44], 45}, {NULL}};
const Segment SGI20312[] = {{&S4582[30], 6}, {NULL}};
const Segment SGI20313[] = {{&S4582[137], 3}, {NULL}};
const Segment SGI20314[] = {{&S4582[103], 5}, {NULL}};
const Segment SGI20315[] = {{&S4584[44], 28}, {NULL}};
const Segment SGI20316[] = {{&S4584[44], 45}, {NULL}};
const Segment SGI20317[] = {{&S4584[30], 6}, {NULL}};
const Segment SGI20318[] = {{&S4584[137], 3}, {NULL}};
const Segment SGI20319[] = {{&S4584[103], 5}, {NULL}};
const Segment SGI20320[] = {{&S4588[38], 10}, {NULL}};
const Segment SGI20321[] = {{&S4588[31], 6}, {NULL}};
const Segment SGI20322[] = {{&S4592[38], 11}, {NULL}};
const Segment SGI20323[] = {{&S4592[31], 6}, {NULL}};
const Segment SGI20324[] = {{&S4602[44], 10}, {NULL}};
const Segment SGI20325[] = {{&S4602[44], 27}, {NULL}};
const Segment SGI20326[] = {{&S4602[30], 6}, {NULL}};
const Segment SGI20327[] = {{&S4602[129], 3}, {NULL}};
const Segment SGI20328[] = {{&S4602[94], 6}, {NULL}};
const Segment SGI20329[] = {{&S4602[149], 8}, {NULL}};
const Segment SGI20330[] = {{&S4604[38], 11}, {NULL}};
const Segment SGI20331[] = {{&S4604[31], 6}, {NULL}};
const Segment SGI20332[] = {{&S4626[44], 10}, {NULL}};
const Segment SGI20333[] = {{&S4626[44], 27}, {NULL}};
const Segment SGI20334[] = {{&S4626[30], 6}, {NULL}};
const Segment SGI20335[] = {{&S4626[88], 4}, {NULL}};
const Segment SGI20336[] = {{&S4626[88], 3}, {NULL}};
const Segment SGI20337[] = {{&S4642[44], 10}, {NULL}};
const Segment SGI20338[] = {{&S4642[44], 27}, {NULL}};
const Segment SGI20339[] = {{&S4642[30], 6}, {NULL}};
const Segment SGI20340[] = {{&S4642[119], 3}, {NULL}};
const Segment SGI20341[] = {{&S4642[85], 5}, {NULL}};
const Segment SGI20342[] = {{&S4642[137], 9}, {NULL}};
const Segment SGI20343[] = {{&S4648[44], 10}, {NULL}};
const Segment SGI20344[] = {{&S4648[44], 27}, {NULL}};
const Segment SGI20345[] = {{&S4648[30], 6}, {NULL}};
const Segment SGI20346[] = {{&S4648[120], 3}, {NULL}};
const Segment SGI20347[] = {{&S4648[85], 6}, {NULL}};
const Segment SGI20348[] = {{&S4650[42], 11}, {NULL}};
const Segment SGI20349[] = {{&S4650[30], 6}, {NULL}};
const Segment SGI20350[] = {{&S4650[118], 3}, {NULL}};
const Segment SGI20351[] = {{&S4650[84], 5}, {NULL}};
const Segment SGI20352[] = {{&S4656[44], 10}, {NULL}};
const Segment SGI20353[] = {{&S4656[44], 27}, {NULL}};
const Segment SGI20354[] = {{&S4656[30], 6}, {NULL}};
const Segment SGI20355[] = {{&S4656[88], 6}, {NULL}};
const Segment SGI20356[] = {{&S4659[42], 29}, {NULL}};
const Segment SGI20357[] = {{&S4659[30], 6}, {NULL}};
const Segment SGI20358[] = {{&S4659[136], 3}, {NULL}};
const Segment SGI20359[] = {{&S4659[102], 5}, {NULL}};
const Segment SGI20360[] = {{&S4666[44], 11}, {NULL}};
const Segment SGI20361[] = {{&S4666[44], 28}, {NULL}};
const Segment SGI20362[] = {{&S4666[30], 6}, {NULL}};
const Segment SGI20363[] = {{&S4666[89], 4}, {NULL}};
const Segment SGI20364[] = {{&S4666[89], 3}, {NULL}};
const Segment SGI20365[] = {{&S4672[44], 11}, {NULL}};
const Segment SGI20366[] = {{&S4672[44], 28}, {NULL}};
const Segment SGI20367[] = {{&S4672[30], 6}, {NULL}};
const Segment SGI20368[] = {{&S4672[121], 3}, {NULL}};
const Segment SGI20369[] = {{&S4672[86], 6}, {NULL}};
const Segment SGI20370[] = {{&S4672[86], 5}, {NULL}};
const Segment SGI20371[] = {{&S4693[44], 10}, {NULL}};
const Segment SGI20372[] = {{&S4693[44], 27}, {NULL}};
const Segment SGI20373[] = {{&S4693[30], 6}, {NULL}};
const Segment SGI20374[] = {{&S4693[85], 6}, {NULL}};
const Segment SGI20375[] = {{&S4562[44], 11}, {NULL}};
const Segment SGI20376[] = {{&S4562[44], 28}, {NULL}};
const Segment SGI20377[] = {{&S4562[30], 6}, {NULL}};
const Segment SGI20378[] = {{&S4562[120], 3}, {NULL}};
const Segment SGI20379[] = {{&S4562[86], 5}, {NULL}};
const Segment SGI20380[] = {{&S4573[44], 10}, {NULL}};
const Segment SGI20381[] = {{&S4573[44], 27}, {NULL}};
const Segment SGI20382[] = {{&S4573[30], 6}, {NULL}};
const Segment SGI20383[] = {{&S4573[119], 3}, {NULL}};
const Segment SGI20384[] = {{&S4573[85], 5}, {NULL}};
const Segment SGI20385[] = {{&S4581[44], 11}, {NULL}};
const Segment SGI20386[] = {{&S4581[44], 28}, {NULL}};
const Segment SGI20387[] = {{&S4581[30], 6}, {NULL}};
const Segment SGI20388[] = {{&S4581[120], 3}, {NULL}};
const Segment SGI20389[] = {{&S4581[86], 5}, {NULL}};
const Segment SGI20390[] = {{&S4591[38], 10}, {NULL}};
const Segment SGI20391[] = {{&S4591[31], 6}, {NULL}};
const Segment SGI20392[] = {{&S4599[44], 10}, {NULL}};
const Segment SGI20393[] = {{&S4599[44], 33}, {NULL}};
const Segment SGI20394[] = {{&S4599[30], 6}, {NULL}};
const Segment SGI20395[] = {{&S4620[38], 11}, {NULL}};
const Segment SGI20396[] = {{&S4620[31], 6}, {NULL}};
const Segment SGI20397[] = {{&S4621[44], 10}, {NULL}};
const Segment SGI20398[] = {{&S4621[44], 27}, {NULL}};
const Segment SGI20399[] = {{&S4621[30], 6}, {NULL}};
const Segment SGI20400[] = {{&S4621[88], 4}, {NULL}};
const Segment SGI20401[] = {{&S4621[88], 3}, {NULL}};
const Segment SGI20402[] = {{&S4625[44], 10}, {NULL}};
const Segment SGI20403[] = {{&S4625[44], 27}, {NULL}};
const Segment SGI20404[] = {{&S4625[30], 6}, {NULL}};
const Segment SGI20405[] = {{&S4625[119], 3}, {NULL}};
const Segment SGI20406[] = {{&S4625[85], 5}, {NULL}};
const Segment SGI20407[] = {{&S4628[44], 10}, {NULL}};
const Segment SGI20408[] = {{&S4628[44], 27}, {NULL}};
const Segment SGI20409[] = {{&S4628[30], 6}, {NULL}};
const Segment SGI20410[] = {{&S4628[88], 4}, {NULL}};
const Segment SGI20411[] = {{&S4628[88], 3}, {NULL}};
const Segment SGI20412[] = {{&S4635[44], 11}, {NULL}};
const Segment SGI20413[] = {{&S4635[30], 6}, {NULL}};
const Segment SGI20414[] = {{&S4635[103], 3}, {NULL}};
const Segment SGI20415[] = {{&S4635[69], 5}, {NULL}};
const Segment SGI20416[] = {{&S4652[44], 10}, {NULL}};
const Segment SGI20417[] = {{&S4652[44], 27}, {NULL}};
const Segment SGI20418[] = {{&S4652[30], 6}, {NULL}};
const Segment SGI20419[] = {{&S4652[88], 6}, {NULL}};
const Segment SGI20420[] = {{&S4661[44], 11}, {NULL}};
const Segment SGI20421[] = {{&S4661[44], 28}, {NULL}};
const Segment SGI20422[] = {{&S4661[30], 6}, {NULL}};
const Segment SGI20423[] = {{&S4661[89], 4}, {NULL}};
const Segment SGI20424[] = {{&S4661[89], 3}, {NULL}};
const Segment SGI20425[] = {{&S4665[42], 10}, {NULL}};
const Segment SGI20426[] = {{&S4665[30], 6}, {NULL}};
const Segment SGI20427[] = {{&S4665[117], 3}, {NULL}};
const Segment SGI20428[] = {{&S4665[83], 5}, {NULL}};
const Segment SGI20429[] = {{&S4673[44], 11}, {NULL}};
const Segment SGI20430[] = {{&S4673[44], 28}, {NULL}};
const Segment SGI20431[] = {{&S4673[30], 6}, {NULL}};
const Segment SGI20432[] = {{&S4673[89], 6}, {NULL}};
const Segment SGI20433[] = {{&S4676[44], 10}, {NULL}};
const Segment SGI20434[] = {{&S4676[44], 21}, {NULL}};
const Segment SGI20435[] = {{&S4676[30], 6}, {NULL}};
const Segment SGI20436[] = {{&S4676[113], 3}, {NULL}};
const Segment SGI20437[] = {{&S4676[79], 5}, {NULL}};
const Segment SGI20438[] = {{&S4679[44], 10}, {NULL}};
const Segment SGI20439[] = {{&S4679[44], 27}, {NULL}};
const Segment SGI20440[] = {{&S4679[30], 6}, {NULL}};
const Segment SGI20441[] = {{&S4679[88], 6}, {NULL}};
const Segment SGI20442[] = {{&S4690[42], 11}, {NULL}};
const Segment SGI20443[] = {{&S4690[30], 6}, {NULL}};
const Segment SGI20444[] = {{&S4690[118], 3}, {NULL}};
const Segment SGI20445[] = {{&S4690[84], 5}, {NULL}};
const Segment SGI20446[] = {{&S4692[44], 10}, {NULL}};
const Segment SGI20447[] = {{&S4692[44], 21}, {NULL}};
const Segment SGI20448[] = {{&S4692[30], 6}, {NULL}};
const Segment SGI20449[] = {{&S4692[113], 3}, {NULL}};
const Segment SGI20450[] = {{&S4692[79], 5}, {NULL}};
const Segment SGI20451[] = {{&S4695[44], 10}, {NULL}};
const Segment SGI20452[] = {{&S4695[44], 27}, {NULL}};
const Segment SGI20453[] = {{&S4695[30], 6}, {NULL}};
const Segment SGI20454[] = {{&S4695[85], 6}, {NULL}};
const Segment SGI20455[] = {{&S4580[44], 10}, {NULL}};
const Segment SGI20456[] = {{&S4580[44], 27}, {NULL}};
const Segment SGI20457[] = {{&S4580[30], 6}, {NULL}};
const Segment SGI20458[] = {{&S4580[119], 3}, {NULL}};
const Segment SGI20459[] = {{&S4580[85], 5}, {NULL}};
const Segment SGI20460[] = {{&S4583[44], 11}, {NULL}};
const Segment SGI20461[] = {{&S4583[44], 28}, {NULL}};
const Segment SGI20462[] = {{&S4583[30], 6}, {NULL}};
const Segment SGI20463[] = {{&S4583[120], 3}, {NULL}};
const Segment SGI20464[] = {{&S4583[86], 5}, {NULL}};
const Segment SGI20465[] = {{&S4585[44], 30}, {NULL}};
const Segment SGI20466[] = {{&S4585[44], 47}, {NULL}};
const Segment SGI20467[] = {{&S4585[30], 6}, {NULL}};
const Segment SGI20468[] = {{&S4585[139], 3}, {NULL}};
const Segment SGI20469[] = {{&S4585[105], 5}, {NULL}};
const Segment SGI20470[] = {{&S4586[44], 10}, {NULL}};
const Segment SGI20471[] = {{&S4586[44], 21}, {NULL}};
const Segment SGI20472[] = {{&S4586[30], 6}, {NULL}};
const Segment SGI20473[] = {{&S4586[113], 3}, {NULL}};
const Segment SGI20474[] = {{&S4586[79], 5}, {NULL}};
const Segment SGI20475[] = {{&S4593[38], 10}, {NULL}};
const Segment SGI20476[] = {{&S4593[31], 6}, {NULL}};
const Segment SGI20477[] = {{&S4595[44], 29}, {NULL}};
const Segment SGI20478[] = {{&S4595[44], 46}, {NULL}};
const Segment SGI20479[] = {{&S4595[30], 6}, {NULL}};
const Segment SGI20480[] = {{&S4595[138], 3}, {NULL}};
const Segment SGI20481[] = {{&S4595[104], 5}, {NULL}};
const Segment SGI20482[] = {{&S4596[44], 11}, {NULL}};
const Segment SGI20483[] = {{&S4596[44], 28}, {NULL}};
const Segment SGI20484[] = {{&S4596[30], 6}, {NULL}};
const Segment SGI20485[] = {{&S4596[120], 3}, {NULL}};
const Segment SGI20486[] = {{&S4596[86], 5}, {NULL}};
const Segment SGI20487[] = {{&S4601[44], 11}, {NULL}};
const Segment SGI20488[] = {{&S4601[44], 28}, {NULL}};
const Segment SGI20489[] = {{&S4601[30], 6}, {NULL}};
const Segment SGI20490[] = {{&S4601[89], 4}, {NULL}};
const Segment SGI20491[] = {{&S4601[89], 3}, {NULL}};
const Segment SGI20492[] = {{&S4612[44], 28}, {NULL}};
const Segment SGI20493[] = {{&S4612[44], 45}, {NULL}};
const Segment SGI20494[] = {{&S4612[30], 6}, {NULL}};
const Segment SGI20495[] = {{&S4612[137], 3}, {NULL}};
const Segment SGI20496[] = {{&S4612[103], 5}, {NULL}};
const Segment SGI20497[] = {{&S4614[44], 29}, {NULL}};
const Segment SGI20498[] = {{&S4614[44], 46}, {NULL}};
const Segment SGI20499[] = {{&S4614[30], 6}, {NULL}};
const Segment SGI20500[] = {{&S4614[138], 3}, {NULL}};
const Segment SGI20501[] = {{&S4614[104], 5}, {NULL}};
const Segment SGI20502[] = {{&S4619[44], 11}, {NULL}};
const Segment SGI20503[] = {{&S4619[44], 28}, {NULL}};
const Segment SGI20504[] = {{&S4619[30], 6}, {NULL}};
const Segment SGI20505[] = {{&S4619[89], 4}, {NULL}};
const Segment SGI20506[] = {{&S4619[89], 3}, {NULL}};
const Segment SGI20507[] = {{&S4627[44], 10}, {NULL}};
const Segment SGI20508[] = {{&S4627[44], 27}, {NULL}};
const Segment SGI20509[] = {{&S4627[30], 6}, {NULL}};
const Segment SGI20510[] = {{&S4627[88], 4}, {NULL}};
const Segment SGI20511[] = {{&S4627[88], 3}, {NULL}};
const Segment SGI20512[] = {{&S4630[38], 11}, {NULL}};
const Segment SGI20513[] = {{&S4630[31], 6}, {NULL}};
const Segment SGI20514[] = {{&S4632[44], 10}, {NULL}};
const Segment SGI20515[] = {{&S4632[44], 27}, {NULL}};
const Segment SGI20516[] = {{&S4632[30], 6}, {NULL}};
const Segment SGI20517[] = {{&S4632[88], 4}, {NULL}};
const Segment SGI20518[] = {{&S4632[88], 3}, {NULL}};
const Segment SGI20519[] = {{&S4636[44], 10}, {NULL}};
const Segment SGI20520[] = {{&S4636[44], 27}, {NULL}};
const Segment SGI20521[] = {{&S4636[30], 6}, {NULL}};
const Segment SGI20522[] = {{&S4636[119], 3}, {NULL}};
const Segment SGI20523[] = {{&S4636[85], 5}, {NULL}};
const Segment SGI20524[] = {{&S4643[44], 10}, {NULL}};
const Segment SGI20525[] = {{&S4643[44], 27}, {NULL}};
const Segment SGI20526[] = {{&S4643[30], 6}, {NULL}};
const Segment SGI20527[] = {{&S4643[88], 6}, {NULL}};
const Segment SGI20528[] = {{&S4646[42], 10}, {NULL}};
const Segment SGI20529[] = {{&S4646[30], 6}, {NULL}};
const Segment SGI20530[] = {{&S4646[117], 3}, {NULL}};
const Segment SGI20531[] = {{&S4646[83], 5}, {NULL}};
const Segment SGI20532[] = {{&S4657[44], 11}, {NULL}};
const Segment SGI20533[] = {{&S4657[44], 28}, {NULL}};
const Segment SGI20534[] = {{&S4657[30], 6}, {NULL}};
const Segment SGI20535[] = {{&S4657[89], 6}, {NULL}};
const Segment SGI20536[] = {{&S4667[38], 11}, {NULL}};
const Segment SGI20537[] = {{&S4667[31], 6}, {NULL}};
const Segment SGI20538[] = {{&S4674[44], 10}, {NULL}};
const Segment SGI20539[] = {{&S4674[44], 27}, {NULL}};
const Segment SGI20540[] = {{&S4674[30], 6}, {NULL}};
const Segment SGI20541[] = {{&S4674[88], 6}, {NULL}};
const Segment SGI20542[] = {{&S4677[44], 11}, {NULL}};
const Segment SGI20543[] = {{&S4677[44], 28}, {NULL}};
const Segment SGI20544[] = {{&S4677[30], 6}, {NULL}};
const Segment SGI20545[] = {{&S4677[89], 6}, {NULL}};
const Segment SGI20546[] = {{&S4678[44], 10}, {NULL}};
const Segment SGI20547[] = {{&S4678[44], 27}, {NULL}};
const Segment SGI20548[] = {{&S4678[30], 6}, {NULL}};
const Segment SGI20549[] = {{&S4678[85], 6}, {NULL}};
const Segment SGI20550[] = {{&S4689[44], 27}, {NULL}};
const Segment SGI20551[] = {{&S4689[30], 6}, {NULL}};
const Segment SGI20552[] = {{&S4691[42], 11}, {NULL}};
const Segment SGI20553[] = {{&S4691[30], 6}, {NULL}};
const Segment SGI20554[] = {{&S4691[118], 3}, {NULL}};
const Segment SGI20555[] = {{&S4691[84], 5}, {NULL}};
const Segment SGI20556[] = {{&S4697[44], 10}, {NULL}};
const Segment SGI20557[] = {{&S4697[44], 21}, {NULL}};
const Segment SGI20558[] = {{&S4697[30], 6}, {NULL}};
const Segment SGI20559[] = {{&S4697[113], 3}, {NULL}};
const Segment SGI20560[] = {{&S4697[79], 5}, {NULL}};
const Segment SGI20561[] = {{&S10355[44], 11}, {NULL}};
const Segment SGI20562[] = {{&S10355[44], 23}, {NULL}};
const Segment SGI20563[] = {{&S10355[30], 6}, {NULL}};
const Segment SGI20564[] = {{&S10355[115], 3}, {NULL}};
const Segment SGI20565[] = {{&S10355[81], 5}, {NULL}};
const Segment SGI20566[] = {{&S10357[44], 20}, {NULL}};
const Segment SGI20567[] = {{&S10357[44], 32}, {NULL}};
const Segment SGI20568[] = {{&S10357[30], 6}, {NULL}};
const Segment SGI20569[] = {{&S10357[124], 3}, {NULL}};
const Segment SGI20570[] = {{&S10357[90], 5}, {NULL}};
const Segment SGI20571[] = {{&S10411[44], 11}, {NULL}};
const Segment SGI20572[] = {{&S10411[44], 23}, {NULL}};
const Segment SGI20573[] = {{&S10411[30], 6}, {NULL}};
const Segment SGI20574[] = {{&S10411[115], 3}, {NULL}};
const Segment SGI20575[] = {{&S10411[81], 5}, {NULL}};
const Segment SGI20576[] = {{&S13041[44], 10}, {NULL}};
const Segment SGI20577[] = {{&S13041[44], 21}, {NULL}};
const Segment SGI20578[] = {{&S13041[30], 6}, {NULL}};
const Segment SGI20579[] = {{&S13041[136], 3}, {NULL}};
const Segment SGI20580[] = {{&S13041[102], 5}, {NULL}};
const Segment SGI20581[] = {{&S9520[44], 10}, {NULL}};
const Segment SGI20582[] = {{&S9520[44], 27}, {NULL}};
const Segment SGI20583[] = {{&S9520[30], 6}, {NULL}};
const Segment SGI20584[] = {{&S9520[119], 3}, {NULL}};
const Segment SGI20585[] = {{&S9520[85], 5}, {NULL}};
const Segment SGI20586[] = {{&S9521[44], 29}, {NULL}};
const Segment SGI20587[] = {{&S9521[44], 46}, {NULL}};
const Segment SGI20588[] = {{&S9521[30], 6}, {NULL}};
const Segment SGI20589[] = {{&S9521[138], 3}, {NULL}};
const Segment SGI20590[] = {{&S9521[104], 5}, {NULL}};
const Segment SGI20591[] = {{&S9522[44], 29}, {NULL}};
const Segment SGI20592[] = {{&S9522[44], 46}, {NULL}};
const Segment SGI20593[] = {{&S9522[30], 6}, {NULL}};
const Segment SGI20594[] = {{&S9522[138], 3}, {NULL}};
const Segment SGI20595[] = {{&S9522[104], 5}, {NULL}};
const Segment SGI20596[] = {{&S9523[44], 29}, {NULL}};
const Segment SGI20597[] = {{&S9523[44], 46}, {NULL}};
const Segment SGI20598[] = {{&S9523[30], 6}, {NULL}};
const Segment SGI20599[] = {{&S9523[138], 3}, {NULL}};
const Segment SGI20600[] = {{&S9523[104], 5}, {NULL}};
const Segment SGI20601[] = {{&S9527[44], 29}, {NULL}};
const Segment SGI20602[] = {{&S9527[44], 46}, {NULL}};
const Segment SGI20603[] = {{&S9527[30], 6}, {NULL}};
const Segment SGI20604[] = {{&S9527[138], 3}, {NULL}};
const Segment SGI20605[] = {{&S9527[104], 5}, {NULL}};
const Segment SGI20606[] = {{&S9532[44], 30}, {NULL}};
const Segment SGI20607[] = {{&S9532[44], 47}, {NULL}};
const Segment SGI20608[] = {{&S9532[30], 6}, {NULL}};
const Segment SGI20609[] = {{&S9532[139], 3}, {NULL}};
const Segment SGI20610[] = {{&S9532[105], 5}, {NULL}};
const Segment SGI20611[] = {{&S9534[38], 10}, {NULL}};
const Segment SGI20612[] = {{&S9534[31], 6}, {NULL}};
const Segment SGI20613[] = {{&S9556[44], 10}, {NULL}};
const Segment SGI20614[] = {{&S9556[44], 27}, {NULL}};
const Segment SGI20615[] = {{&S9556[30], 6}, {NULL}};
const Segment SGI20616[] = {{&S9556[88], 4}, {NULL}};
const Segment SGI20617[] = {{&S9556[88], 3}, {NULL}};
const Segment SGI20618[] = {{&S9570[44], 10}, {NULL}};
const Segment SGI20619[] = {{&S9570[44], 27}, {NULL}};
const Segment SGI20620[] = {{&S9570[30], 6}, {NULL}};
const Segment SGI20621[] = {{&S9570[120], 3}, {NULL}};
const Segment SGI20622[] = {{&S9570[85], 6}, {NULL}};
const Segment SGI20623[] = {{&S9584[42], 10}, {NULL}};
const Segment SGI20624[] = {{&S9584[30], 6}, {NULL}};
const Segment SGI20625[] = {{&S9584[117], 3}, {NULL}};
const Segment SGI20626[] = {{&S9584[83], 5}, {NULL}};
const Segment SGI20627[] = {{&S9524[44], 28}, {NULL}};
const Segment SGI20628[] = {{&S9524[44], 45}, {NULL}};
const Segment SGI20629[] = {{&S9524[30], 6}, {NULL}};
const Segment SGI20630[] = {{&S9524[137], 3}, {NULL}};
const Segment SGI20631[] = {{&S9524[103], 5}, {NULL}};
const Segment SGI20632[] = {{&S9525[44], 11}, {NULL}};
const Segment SGI20633[] = {{&S9525[44], 28}, {NULL}};
const Segment SGI20634[] = {{&S9525[30], 6}, {NULL}};
const Segment SGI20635[] = {{&S9525[120], 3}, {NULL}};
const Segment SGI20636[] = {{&S9525[86], 5}, {NULL}};
const Segment SGI20637[] = {{&S9526[44], 11}, {NULL}};
const Segment SGI20638[] = {{&S9526[44], 28}, {NULL}};
const Segment SGI20639[] = {{&S9526[30], 6}, {NULL}};
const Segment SGI20640[] = {{&S9526[120], 3}, {NULL}};
const Segment SGI20641[] = {{&S9526[86], 5}, {NULL}};
const Segment SGI20642[] = {{&S9528[44], 10}, {NULL}};
const Segment SGI20643[] = {{&S9528[44], 27}, {NULL}};
const Segment SGI20644[] = {{&S9528[30], 6}, {NULL}};
const Segment SGI20645[] = {{&S9528[119], 3}, {NULL}};
const Segment SGI20646[] = {{&S9528[85], 5}, {NULL}};
const Segment SGI20647[] = {{&S9529[44], 28}, {NULL}};
const Segment SGI20648[] = {{&S9529[44], 45}, {NULL}};
const Segment SGI20649[] = {{&S9529[30], 6}, {NULL}};
const Segment SGI20650[] = {{&S9529[137], 3}, {NULL}};
const Segment SGI20651[] = {{&S9529[103], 5}, {NULL}};
const Segment SGI20652[] = {{&S9530[44], 18}, {NULL}};
const Segment SGI20653[] = {{&S9530[44], 35}, {NULL}};
const Segment SGI20654[] = {{&S9530[30], 6}, {NULL}};
const Segment SGI20655[] = {{&S9530[127], 3}, {NULL}};
const Segment SGI20656[] = {{&S9530[93], 5}, {NULL}};
const Segment SGI20657[] = {{&S9533[44], 28}, {NULL}};
const Segment SGI20658[] = {{&S9533[44], 45}, {NULL}};
const Segment SGI20659[] = {{&S9533[30], 6}, {NULL}};
const Segment SGI20660[] = {{&S9533[137], 3}, {NULL}};
const Segment SGI20661[] = {{&S9533[103], 5}, {NULL}};
const Segment SGI20662[] = {{&S9535[38], 10}, {NULL}};
const Segment SGI20663[] = {{&S9535[31], 6}, {NULL}};
const Segment SGI20664[] = {{&S9536[44], 11}, {NULL}};
const Segment SGI20665[] = {{&S9536[44], 28}, {NULL}};
const Segment SGI20666[] = {{&S9536[30], 6}, {NULL}};
const Segment SGI20667[] = {{&S9536[120], 3}, {NULL}};
const Segment SGI20668[] = {{&S9536[86], 5}, {NULL}};
const Segment SGI20669[] = {{&S9843[44], 21}, {NULL}};
const Segment SGI20670[] = {{&S9843[44], 32}, {NULL}};
const Segment SGI20671[] = {{&S9843[30], 6}, {NULL}};
const Segment SGI20672[] = {{&S9843[124], 3}, {NULL}};
const Segment SGI20673[] = {{&S9843[90], 5}, {NULL}};
const Segment SGI20674[] = {{&S9542[44], 28}, {NULL}};
const Segment SGI20675[] = {{&S9844[38], 1}, {NULL}};
const Segment SGI20676[] = {{&S9542[44], 45}, {NULL}};
const Segment SGI20677[] = {{&S9542[30], 6}, {NULL}};
const Segment SGI20678[] = {{&S9844[62], 13}, {NULL}};
const Segment SGI20679[] = {{&S9844[49], 6}, {NULL}};
const Segment SGI20680[] = {{&S9542[137], 3}, {NULL}};
const Segment SGI20681[] = {{&S9844[123], 3}, {NULL}};
const Segment SGI20682[] = {{&S9542[103], 5}, {NULL}};
const Segment SGI20683[] = {{&S9844[89], 5}, {NULL}};
const Segment SGI20684[] = {{&S9845[44], 26}, {NULL}};
const Segment SGI20685[] = {{&S9845[44], 37}, {NULL}};
const Segment SGI20686[] = {{&S9845[30], 6}, {NULL}};
const Segment SGI20687[] = {{&S9845[129], 3}, {NULL}};
const Segment SGI20688[] = {{&S9845[95], 5}, {NULL}};
const Segment SGI20689[] = {{&S9545[44], 11}, {NULL}};
const Segment SGI20690[] = {{&S9545[44], 28}, {NULL}};
const Segment SGI20691[] = {{&S9545[30], 6}, {NULL}};
const Segment SGI20692[] = {{&S9545[120], 3}, {NULL}};
const Segment SGI20693[] = {{&S9846[44], 16}, {NULL}};
const Segment SGI20694[] = {{&S9545[86], 5}, {NULL}};
const Segment SGI20695[] = {{&S9846[44], 27}, {NULL}};
const Segment SGI20696[] = {{&S9846[30], 6}, {NULL}};
const Segment SGI20697[] = {{&S9846[119], 3}, {NULL}};
const Segment SGI20698[] = {{&S9846[85], 5}, {NULL}};
const Segment SGI20699[] = {{&S9847[44], 26}, {NULL}};
const Segment SGI20700[] = {{&S9847[44], 37}, {NULL}};
const Segment SGI20701[] = {{&S9847[30], 6}, {NULL}};
const Segment SGI20702[] = {{&S9847[129], 3}, {NULL}};
const Segment SGI20703[] = {{&S9847[95], 5}, {NULL}};
const Segment SGI20704[] = {{&S9549[38], 11}, {NULL}};
const Segment SGI20705[] = {{&S9549[31], 6}, {NULL}};
const Segment SGI20706[] = {{&S9848[43], 16}, {NULL}};
const Segment SGI20707[] = {{&S9848[43], 27}, {NULL}};
const Segment SGI20708[] = {{&S9848[30], 5}, {NULL}};
const Segment SGI20709[] = {{&S9848[118], 3}, {NULL}};
const Segment SGI20710[] = {{&S9848[84], 5}, {NULL}};
const Segment SGI20711[] = {{&S9551[38], 11}, {NULL}};
const Segment SGI20712[] = {{&S9849[43], 16}, {NULL}};
const Segment SGI20713[] = {{&S9551[31], 6}, {NULL}};
const Segment SGI20714[] = {{&S9849[43], 27}, {NULL}};
const Segment SGI20715[] = {{&S9849[118], 3}, {NULL}};
const Segment SGI20716[] = {{&S9849[84], 5}, {NULL}};
const Segment SGI20717[] = {{&S9850[44], 16}, {NULL}};
const Segment SGI20718[] = {{&S9850[44], 27}, {NULL}};
const Segment SGI20719[] = {{&S9850[30], 6}, {NULL}};
const Segment SGI20720[] = {{&S9850[119], 3}, {NULL}};
const Segment SGI20721[] = {{&S9850[85], 5}, {NULL}};
const Segment SGI20722[] = {{&S9851[44], 15}, {NULL}};
const Segment SGI20723[] = {{&S9555[44], 10}, {NULL}};
const Segment SGI20724[] = {{&S9851[44], 26}, {NULL}};
const Segment SGI20725[] = {{&S9555[44], 27}, {NULL}};
const Segment SGI20726[] = {{&S9851[30], 6}, {NULL}};
const Segment SGI20727[] = {{&S9555[30], 6}, {NULL}};
const Segment SGI20728[] = {{&S9851[118], 3}, {NULL}};
const Segment SGI20729[] = {{&S9851[84], 5}, {NULL}};
const Segment SGI20730[] = {{&S9555[88], 4}, {NULL}};
const Segment SGI20731[] = {{&S9555[88], 3}, {NULL}};
const Segment SGI20732[] = {{&S9852[44], 16}, {NULL}};
const Segment SGI20733[] = {{&S9852[44], 27}, {NULL}};
const Segment SGI20734[] = {{&S9852[30], 6}, {NULL}};
const Segment SGI20735[] = {{&S9852[119], 3}, {NULL}};
const Segment SGI20736[] = {{&S9852[85], 5}, {NULL}};
const Segment SGI20737[] = {{&S9853[44], 15}, {NULL}};
const Segment SGI20738[] = {{&S9853[44], 26}, {NULL}};
const Segment SGI20739[] = {{&S9853[30], 6}, {NULL}};
const Segment SGI20740[] = {{&S9853[118], 3}, {NULL}};
const Segment SGI20741[] = {{&S9853[84], 5}, {NULL}};
const Segment SGI20742[] = {{&S9854[44], 22}, {NULL}};
const Segment SGI20743[] = {{&S9854[44], 33}, {NULL}};
const Segment SGI20744[] = {{&S9854[30], 6}, {NULL}};
const Segment SGI20745[] = {{&S9854[125], 3}, {NULL}};
const Segment SGI20746[] = {{&S9854[91], 5}, {NULL}};
const Segment SGI20747[] = {{&S9855[44], 26}, {NULL}};
const Segment SGI20748[] = {{&S9855[44], 37}, {NULL}};
const Segment SGI20749[] = {{&S9855[30], 6}, {NULL}};
const Segment SGI20750[] = {{&S9855[129], 3}, {NULL}};
const Segment SGI20751[] = {{&S9855[95], 5}, {NULL}};
const Segment SGI20752[] = {{&S9856[44], 12}, {NULL}};
const Segment SGI20753[] = {{&S9856[44], 23}, {NULL}};
const Segment SGI20754[] = {{&S9856[30], 6}, {NULL}};
const Segment SGI20755[] = {{&S9856[115], 3}, {NULL}};
const Segment SGI20756[] = {{&S9856[81], 5}, {NULL}};
const Segment SGI20757[] = {{&S9857[44], 26}, {NULL}};
const Segment SGI20758[] = {{&S9857[44], 37}, {NULL}};
const Segment SGI20759[] = {{&S9857[30], 6}, {NULL}};
const Segment SGI20760[] = {{&S9857[129], 3}, {NULL}};
const Segment SGI20761[] = {{&S9857[95], 5}, {NULL}};
const Segment SGI20762[] = {{&S9858[44], 25}, {NULL}};
const Segment SGI20763[] = {{&S9858[44], 36}, {NULL}};
const Segment SGI20764[] = {{&S9858[30], 6}, {NULL}};
const Segment SGI20765[] = {{&S9858[128], 3}, {NULL}};
const Segment SGI20766[] = {{&S9858[94], 5}, {NULL}};
const Segment SGI20767[] = {{&S9859[44], 26}, {NULL}};
const Segment SGI20768[] = {{&S9859[44], 37}, {NULL}};
const Segment SGI20769[] = {{&S9859[30], 6}, {NULL}};
const Segment SGI20770[] = {{&S9859[129], 3}, {NULL}};
const Segment SGI20771[] = {{&S9859[95], 5}, {NULL}};
const Segment SGI20772[] = {{&S9562[44], 10}, {NULL}};
const Segment SGI20773[] = {{&S9562[44], 27}, {NULL}};
const Segment SGI20774[] = {{&S9562[30], 6}, {NULL}};
const Segment SGI20775[] = {{&S9562[88], 6}, {NULL}};
const Segment SGI20776[] = {{&S9860[44], 101}, {NULL}};
const Segment SGI20777[] = {{&S9860[44], 112}, {NULL}};
const Segment SGI20778[] = {{&S9860[30], 6}, {NULL}};
const Segment SGI20779[] = {{&S9860[204], 3}, {NULL}};
const Segment SGI20780[] = {{&S9860[170], 5}, {NULL}};
const Segment SGI20781[] = {{&S9565[38], 11}, {NULL}};
const Segment SGI20782[] = {{&S9565[31], 6}, {NULL}};
const Segment SGI20783[] = {{&S9861[44], 15}, {NULL}};
const Segment SGI20784[] = {{&S9861[44], 26}, {NULL}};
const Segment SGI20785[] = {{&S9861[30], 6}, {NULL}};
const Segment SGI20786[] = {{&S9861[118], 3}, {NULL}};
const Segment SGI20787[] = {{&S9861[84], 5}, {NULL}};
const Segment SGI20788[] = {{&S9567[38], 11}, {NULL}};
const Segment SGI20789[] = {{&S9567[31], 6}, {NULL}};
const Segment SGI20790[] = {{&S9862[44], 26}, {NULL}};
const Segment SGI20791[] = {{&S9862[44], 37}, {NULL}};
const Segment SGI20792[] = {{&S9862[30], 6}, {NULL}};
const Segment SGI20793[] = {{&S9862[129], 3}, {NULL}};
const Segment SGI20794[] = {{&S9862[95], 5}, {NULL}};
const Segment SGI20795[] = {{&S9863[44], 21}, {NULL}};
const Segment SGI20796[] = {{&S9863[44], 32}, {NULL}};
const Segment SGI20797[] = {{&S9863[30], 6}, {NULL}};
const Segment SGI20798[] = {{&S9863[124], 3}, {NULL}};
const Segment SGI20799[] = {{&S9863[90], 5}, {NULL}};
const Segment SGI20800[] = {{&S9864[44], 26}, {NULL}};
const Segment SGI20801[] = {{&S9864[44], 37}, {NULL}};
const Segment SGI20802[] = {{&S9864[30], 6}, {NULL}};
const Segment SGI20803[] = {{&S9864[129], 3}, {NULL}};
const Segment SGI20804[] = {{&S9864[95], 5}, {NULL}};
const Segment SGI20805[] = {{&S9865[38], 22}, {NULL}};
const Segment SGI20806[] = {{&S9865[31], 6}, {NULL}};
const Segment SGI20807[] = {{&S9873[44], 22}, {NULL}};
const Segment SGI20808[] = {{&S9873[44], 33}, {NULL}};
const Segment SGI20809[] = {{&S9873[30], 6}, {NULL}};
const Segment SGI20810[] = {{&S9874[44], 16}, {NULL}};
const Segment SGI20811[] = {{&S9874[44], 27}, {NULL}};
const Segment SGI20812[] = {{&S9874[30], 6}, {NULL}};
const Segment SGI20813[] = {{&S9874[119], 3}, {NULL}};
const Segment SGI20814[] = {{&S9874[85], 5}, {NULL}};
const Segment SGI20815[] = {{&S9587[42], 10}, {NULL}};
const Segment SGI20816[] = {{&S9587[30], 6}, {NULL}};
const Segment SGI20817[] = {{&S9587[117], 3}, {NULL}};
const Segment SGI20818[] = {{&S9587[83], 5}, {NULL}};
const Segment SGI20819[] = {{&S9879[38], 16}, {NULL}};
const Segment SGI20820[] = {{&S9879[31], 6}, {NULL}};
const Segment SGI20821[] = {{&S9593[44], 27}, {NULL}};
const Segment SGI20822[] = {{&S9593[30], 6}, {NULL}};
const Segment SGI20823[] = {{&S9887[44], 11}, {NULL}};
const Segment SGI20824[] = {{&S9887[44], 22}, {NULL}};
const Segment SGI20825[] = {{&S9887[30], 6}, {NULL}};
const Segment SGI20826[] = {{&S9887[83], 4}, {NULL}};
const Segment SGI20827[] = {{&S9887[83], 3}, {NULL}};
const Segment SGI20828[] = {{&S9598[42], 11}, {NULL}};
const Segment SGI20829[] = {{&S9598[30], 6}, {NULL}};
const Segment SGI20830[] = {{&S9598[118], 3}, {NULL}};
const Segment SGI20831[] = {{&S9598[84], 5}, {NULL}};
const Segment SGI20832[] = {{&S9889[44], 22}, {NULL}};
const Segment SGI20833[] = {{&S9889[44], 33}, {NULL}};
const Segment SGI20834[] = {{&S9889[30], 6}, {NULL}};
const Segment SGI20835[] = {{&S9889[94], 4}, {NULL}};
const Segment SGI20836[] = {{&S9889[94], 3}, {NULL}};
const Segment SGI20837[] = {{&S9604[42], 11}, {NULL}};
const Segment SGI20838[] = {{&S9892[44], 11}, {NULL}};
const Segment SGI20839[] = {{&S9892[44], 22}, {NULL}};
const Segment SGI20840[] = {{&S9604[30], 6}, {NULL}};
const Segment SGI20841[] = {{&S9892[30], 6}, {NULL}};
const Segment SGI20842[] = {{&S9604[118], 3}, {NULL}};
const Segment SGI20843[] = {{&S9892[114], 3}, {NULL}};
const Segment SGI20844[] = {{&S9604[84], 5}, {NULL}};
const Segment SGI20845[] = {{&S9892[80], 5}, {NULL}};
const Segment SGI20846[] = {{&S9910[44], 22}, {NULL}};
const Segment SGI20847[] = {{&S9910[44], 33}, {NULL}};
const Segment SGI20848[] = {{&S9910[30], 6}, {NULL}};
const Segment SGI20849[] = {{&S9910[94], 6}, {NULL}};
const Segment SGI20850[] = {{&S9914[38], 11}, {NULL}};
const Segment SGI20851[] = {{&S9914[31], 6}, {NULL}};
const Segment SGI20852[] = {{&S9915[44], 16}, {NULL}};
const Segment SGI20853[] = {{&S9915[44], 27}, {NULL}};
const Segment SGI20854[] = {{&S9915[30], 6}, {NULL}};
const Segment SGI20855[] = {{&S9915[88], 4}, {NULL}};
const Segment SGI20856[] = {{&S9915[88], 3}, {NULL}};
const Segment SGI20857[] = {{&S9920[44], 11}, {NULL}};
const Segment SGI20858[] = {{&S9920[44], 22}, {NULL}};
const Segment SGI20859[] = {{&S9920[30], 6}, {NULL}};
const Segment SGI20860[] = {{&S9920[83], 6}, {NULL}};
const Segment SGI20861[] = {{&S9926[44], 33}, {NULL}};
const Segment SGI20862[] = {{&S9926[30], 6}, {NULL}};
const Segment SGI20863[] = {{&S9866[44], 15}, {NULL}};
const Segment SGI20864[] = {{&S9866[44], 26}, {NULL}};
const Segment SGI20865[] = {{&S9866[30], 6}, {NULL}};
const Segment SGI20866[] = {{&S9866[118], 3}, {NULL}};
const Segment SGI20867[] = {{&S9866[84], 5}, {NULL}};
const Segment SGI20868[] = {{&S9867[44], 11}, {NULL}};
const Segment SGI20869[] = {{&S9867[44], 22}, {NULL}};
const Segment SGI20870[] = {{&S9867[30], 6}, {NULL}};
const Segment SGI20871[] = {{&S9867[114], 3}, {NULL}};
const Segment SGI20872[] = {{&S9867[80], 5}, {NULL}};
const Segment SGI20873[] = {{&S9885[44], 21}, {NULL}};
const Segment SGI20874[] = {{&S9885[44], 32}, {NULL}};
const Segment SGI20875[] = {{&S9885[30], 6}, {NULL}};
const Segment SGI20876[] = {{&S9885[124], 3}, {NULL}};
const Segment SGI20877[] = {{&S9885[90], 5}, {NULL}};
const Segment SGI20878[] = {{&S9531[44], 10}, {NULL}};
const Segment SGI20879[] = {{&S9531[44], 27}, {NULL}};
const Segment SGI20880[] = {{&S9531[30], 6}, {NULL}};
const Segment SGI20881[] = {{&S9531[119], 3}, {NULL}};
const Segment SGI20882[] = {{&S9531[85], 5}, {NULL}};
const Segment SGI20883[] = {{&S9888[38], 11}, {NULL}};
const Segment SGI20884[] = {{&S9888[31], 6}, {NULL}};
const Segment SGI20885[] = {{&S9537[44], 29}, {NULL}};
const Segment SGI20886[] = {{&S9537[44], 46}, {NULL}};
const Segment SGI20887[] = {{&S9537[30], 6}, {NULL}};
const Segment SGI20888[] = {{&S9537[138], 3}, {NULL}};
const Segment SGI20889[] = {{&S9537[104], 5}, {NULL}};
const Segment SGI20890[] = {{&S9538[44], 29}, {NULL}};
const Segment SGI20891[] = {{&S9538[44], 46}, {NULL}};
const Segment SGI20892[] = {{&S9538[30], 6}, {NULL}};
const Segment SGI20893[] = {{&S9538[138], 3}, {NULL}};
const Segment SGI20894[] = {{&S9538[104], 5}, {NULL}};
const Segment SGI20895[] = {{&S9540[44], 11}, {NULL}};
const Segment SGI20896[] = {{&S9540[44], 28}, {NULL}};
const Segment SGI20897[] = {{&S9540[30], 6}, {NULL}};
const Segment SGI20898[] = {{&S9540[120], 3}, {NULL}};
const Segment SGI20899[] = {{&S9540[86], 5}, {NULL}};
const Segment SGI20900[] = {{&S9544[38], 10}, {NULL}};
const Segment SGI20901[] = {{&S9544[31], 6}, {NULL}};
const Segment SGI20902[] = {{&S9906[44], 22}, {NULL}};
const Segment SGI20903[] = {{&S9906[44], 33}, {NULL}};
const Segment SGI20904[] = {{&S9906[30], 6}, {NULL}};
const Segment SGI20905[] = {{&S9906[125], 3}, {NULL}};
const Segment SGI20906[] = {{&S9906[91], 5}, {NULL}};
const Segment SGI20907[] = {{&S9907[38], 15}, {NULL}};
const Segment SGI20908[] = {{&S9907[31], 6}, {NULL}};
const Segment SGI20909[] = {{&S9552[44], 11}, {NULL}};
const Segment SGI20910[] = {{&S9552[44], 28}, {NULL}};
const Segment SGI20911[] = {{&S9552[30], 6}, {NULL}};
const Segment SGI20912[] = {{&S9552[120], 3}, {NULL}};
const Segment SGI20913[] = {{&S9552[86], 5}, {NULL}};
const Segment SGI20914[] = {{&S9569[44], 10}, {NULL}};
const Segment SGI20915[] = {{&S9569[44], 21}, {NULL}};
const Segment SGI20916[] = {{&S9569[30], 6}, {NULL}};
const Segment SGI20917[] = {{&S9569[113], 3}, {NULL}};
const Segment SGI20918[] = {{&S9569[79], 5}, {NULL}};
const Segment SGI20919[] = {{&S9574[44], 10}, {NULL}};
const Segment SGI20920[] = {{&S9574[44], 21}, {NULL}};
const Segment SGI20921[] = {{&S9574[30], 6}, {NULL}};
const Segment SGI20922[] = {{&S9574[113], 3}, {NULL}};
const Segment SGI20923[] = {{&S9574[79], 5}, {NULL}};
const Segment SGI20924[] = {{&S9575[38], 11}, {NULL}};
const Segment SGI20925[] = {{&S9575[31], 6}, {NULL}};
const Segment SGI20926[] = {{&S9582[42], 10}, {NULL}};
const Segment SGI20927[] = {{&S9582[30], 6}, {NULL}};
const Segment SGI20928[] = {{&S9582[117], 3}, {NULL}};
const Segment SGI20929[] = {{&S9582[83], 5}, {NULL}};
const Segment SGI20930[] = {{&S10354[44], 101}, {NULL}};
const Segment SGI20931[] = {{&S10354[44], 112}, {NULL}};
const Segment SGI20932[] = {{&S10354[30], 6}, {NULL}};
const Segment SGI20933[] = {{&S10354[204], 3}, {NULL}};
const Segment SGI20934[] = {{&S10354[170], 5}, {NULL}};
const Segment SGI20935[] = {{&S10356[44], 20}, {NULL}};
const Segment SGI20936[] = {{&S10356[44], 32}, {NULL}};
const Segment SGI20937[] = {{&S10356[30], 6}, {NULL}};
const Segment SGI20938[] = {{&S10356[124], 3}, {NULL}};
const Segment SGI20939[] = {{&S10356[90], 5}, {NULL}};
const Segment SGI20940[] = {{&S10358[44], 24}, {NULL}};
const Segment SGI20941[] = {{&S10358[44], 35}, {NULL}};
const Segment SGI20942[] = {{&S10358[30], 6}, {NULL}};
const Segment SGI20943[] = {{&S10358[127], 3}, {NULL}};
const Segment SGI20944[] = {{&S10358[93], 5}, {NULL}};
const Segment SGI20945[] = {{&S10359[44], 11}, {NULL}};
const Segment SGI20946[] = {{&S10359[44], 22}, {NULL}};
const Segment SGI20947[] = {{&S10359[30], 6}, {NULL}};
const Segment SGI20948[] = {{&S10359[114], 3}, {NULL}};
const Segment SGI20949[] = {{&S10359[80], 5}, {NULL}};
const Segment SGI20950[] = {{&S10360[44], 20}, {NULL}};
const Segment SGI20951[] = {{&S10360[44], 31}, {NULL}};
const Segment SGI20952[] = {{&S10360[30], 6}, {NULL}};
const Segment SGI20953[] = {{&S10360[123], 3}, {NULL}};
const Segment SGI20954[] = {{&S10360[89], 5}, {NULL}};
const Segment SGI20955[] = {{&S10361[44], 27}, {NULL}};
const Segment SGI20956[] = {{&S10361[44], 38}, {NULL}};
const Segment SGI20957[] = {{&S10361[30], 6}, {NULL}};
const Segment SGI20958[] = {{&S10361[130], 3}, {NULL}};
const Segment SGI20959[] = {{&S10361[96], 5}, {NULL}};
const Segment SGI20960[] = {{&S10362[44], 26}, {NULL}};
const Segment SGI20961[] = {{&S10362[44], 37}, {NULL}};
const Segment SGI20962[] = {{&S10362[30], 6}, {NULL}};
const Segment SGI20963[] = {{&S10362[129], 3}, {NULL}};
const Segment SGI20964[] = {{&S10362[95], 5}, {NULL}};
const Segment SGI20965[] = {{&S10365[42], 7}, {NULL}};
const Segment SGI20966[] = {{&S10365[30], 6}, {NULL}};
const Segment SGI20967[] = {{&S10365[108], 3}, {NULL}};
const Segment SGI20968[] = {{&S10365[74], 5}, {NULL}};
const Segment SGI20969[] = {{&S10368[44], 26}, {NULL}};
const Segment SGI20970[] = {{&S10368[44], 37}, {NULL}};
const Segment SGI20971[] = {{&S10368[30], 6}, {NULL}};
const Segment SGI20972[] = {{&S10368[129], 3}, {NULL}};
const Segment SGI20973[] = {{&S10368[95], 5}, {NULL}};
const Segment SGI20974[] = {{&S10369[44], 15}, {NULL}};
const Segment SGI20975[] = {{&S10369[44], 26}, {NULL}};
const Segment SGI20976[] = {{&S10369[30], 6}, {NULL}};
const Segment SGI20977[] = {{&S10369[118], 3}, {NULL}};
const Segment SGI20978[] = {{&S10369[84], 5}, {NULL}};
const Segment SGI20979[] = {{&S10370[44], 27}, {NULL}};
const Segment SGI20980[] = {{&S10370[44], 38}, {NULL}};
const Segment SGI20981[] = {{&S10370[30], 6}, {NULL}};
const Segment SGI20982[] = {{&S10370[130], 3}, {NULL}};
const Segment SGI20983[] = {{&S10370[96], 5}, {NULL}};
const Segment SGI20984[] = {{&S10371[44], 24}, {NULL}};
const Segment SGI20985[] = {{&S10371[44], 35}, {NULL}};
const Segment SGI20986[] = {{&S10371[30], 6}, {NULL}};
const Segment SGI20987[] = {{&S10371[127], 3}, {NULL}};
const Segment SGI20988[] = {{&S10371[93], 5}, {NULL}};
const Segment SGI20989[] = {{&S10372[44], 26}, {NULL}};
const Segment SGI20990[] = {{&S10372[44], 37}, {NULL}};
const Segment SGI20991[] = {{&S10372[30], 6}, {NULL}};
const Segment SGI20992[] = {{&S10372[129], 3}, {NULL}};
const Segment SGI20993[] = {{&S10372[95], 5}, {NULL}};
const Segment SGI20994[] = {{&S10373[44], 14}, {NULL}};
const Segment SGI20995[] = {{&S10373[44], 25}, {NULL}};
const Segment SGI20996[] = {{&S10373[30], 6}, {NULL}};
const Segment SGI20997[] = {{&S10373[117], 3}, {NULL}};
const Segment SGI20998[] = {{&S10373[83], 5}, {NULL}};
const Segment SGI20999[] = {{&S9868[44], 21}, {NULL}};
const Segment SGI21000[] = {{&S9868[44], 32}, {NULL}};
const Segment SGI21001[] = {{&S9868[30], 6}, {NULL}};
const Segment SGI21002[] = {{&S10374[44], 7}, {NULL}};
const Segment SGI21003[] = {{&S10374[44], 18}, {NULL}};
const Segment SGI21004[] = {{&S9868[124], 3}, {NULL}};
const Segment SGI21005[] = {{&S10374[30], 6}, {NULL}};
const Segment SGI21006[] = {{&S9868[90], 5}, {NULL}};
const Segment SGI21007[] = {{&S10374[110], 3}, {NULL}};
const Segment SGI21008[] = {{&S10374[76], 5}, {NULL}};
const Segment SGI21009[] = {{&S9869[44], 22}, {NULL}};
const Segment SGI21010[] = {{&S9869[44], 33}, {NULL}};
const Segment SGI21011[] = {{&S9869[30], 6}, {NULL}};
const Segment SGI21012[] = {{&S10375[44], 20}, {NULL}};
const Segment SGI21013[] = {{&S9869[125], 3}, {NULL}};
const Segment SGI21014[] = {{&S10375[44], 31}, {NULL}};
const Segment SGI21015[] = {{&S10375[30], 6}, {NULL}};
const Segment SGI21016[] = {{&S9869[91], 5}, {NULL}};
const Segment SGI21017[] = {{&S10375[123], 3}, {NULL}};
const Segment SGI21018[] = {{&S10375[89], 5}, {NULL}};
const Segment SGI21019[] = {{&S9870[44], 26}, {NULL}};
const Segment SGI21020[] = {{&S9870[44], 37}, {NULL}};
const Segment SGI21021[] = {{&S9870[30], 6}, {NULL}};
const Segment SGI21022[] = {{&S9870[129], 3}, {NULL}};
const Segment SGI21023[] = {{&S9870[95], 5}, {NULL}};
const Segment SGI21024[] = {{&S10376[44], 26}, {NULL}};
const Segment SGI21025[] = {{&S10376[44], 37}, {NULL}};
const Segment SGI21026[] = {{&S10376[30], 6}, {NULL}};
const Segment SGI21027[] = {{&S10376[129], 3}, {NULL}};
const Segment SGI21028[] = {{&S10376[95], 5}, {NULL}};
const Segment SGI21029[] = {{&S9871[44], 16}, {NULL}};
const Segment SGI21030[] = {{&S9871[44], 27}, {NULL}};
const Segment SGI21031[] = {{&S9871[30], 6}, {NULL}};
const Segment SGI21032[] = {{&S9871[119], 3}, {NULL}};
const Segment SGI21033[] = {{&S9871[85], 5}, {NULL}};
const Segment SGI21034[] = {{&S9872[44], 16}, {NULL}};
const Segment SGI21035[] = {{&S9872[44], 27}, {NULL}};
const Segment SGI21036[] = {{&S9872[30], 6}, {NULL}};
const Segment SGI21037[] = {{&S9872[119], 3}, {NULL}};
const Segment SGI21038[] = {{&S9872[85], 5}, {NULL}};
const Segment SGI21039[] = {{&S10381[44], 26}, {NULL}};
const Segment SGI21040[] = {{&S10381[44], 37}, {NULL}};
const Segment SGI21041[] = {{&S10381[30], 6}, {NULL}};
const Segment SGI21042[] = {{&S10381[129], 3}, {NULL}};
const Segment SGI21043[] = {{&S10381[95], 5}, {NULL}};
const Segment SGI21044[] = {{&S9875[44], 16}, {NULL}};
const Segment SGI21045[] = {{&S9875[44], 27}, {NULL}};
const Segment SGI21046[] = {{&S9875[30], 6}, {NULL}};
const Segment SGI21047[] = {{&S9875[119], 3}, {NULL}};
const Segment SGI21048[] = {{&S9875[85], 5}, {NULL}};
const Segment SGI21049[] = {{&S10382[42], 14}, {NULL}};
const Segment SGI21050[] = {{&S10382[30], 6}, {NULL}};
const Segment SGI21051[] = {{&S10382[115], 3}, {NULL}};
const Segment SGI21052[] = {{&S10382[81], 5}, {NULL}};
const Segment SGI21053[] = {{&S9876[44], 22}, {NULL}};
const Segment SGI21054[] = {{&S9876[44], 33}, {NULL}};
const Segment SGI21055[] = {{&S9876[30], 6}, {NULL}};
const Segment SGI21056[] = {{&S9876[94], 4}, {NULL}};
const Segment SGI21057[] = {{&S9876[94], 3}, {NULL}};
const Segment SGI21058[] = {{&S10383[42], 20}, {NULL}};
const Segment SGI21059[] = {{&S10383[30], 6}, {NULL}};
const Segment SGI21060[] = {{&S14925[44], 10}, {NULL}};
const Segment SGI21061[] = {{&S10383[121], 3}, {NULL}};
const Segment SGI21062[] = {{&S14925[44], 21}, {NULL}};
const Segment SGI21063[] = {{&S14925[30], 6}, {NULL}};
const Segment SGI21064[] = {{&S10383[87], 5}, {NULL}};
const Segment SGI21065[] = {{&S9877[44], 26}, {NULL}};
const Segment SGI21066[] = {{&S9877[44], 37}, {NULL}};
const Segment SGI21067[] = {{&S9877[30], 6}, {NULL}};
const Segment SGI21068[] = {{&S14925[113], 3}, {NULL}};
const Segment SGI21069[] = {{&S14925[79], 5}, {NULL}};
const Segment SGI21070[] = {{&S9877[129], 3}, {NULL}};
const Segment SGI21071[] = {{&S9877[95], 5}, {NULL}};
const Segment SGI21072[] = {{&S10384[44], 11}, {NULL}};
const Segment SGI21073[] = {{&S10384[44], 23}, {NULL}};
const Segment SGI21074[] = {{&S10384[30], 6}, {NULL}};
const Segment SGI21075[] = {{&S10384[115], 3}, {NULL}};
const Segment SGI21076[] = {{&S10384[81], 5}, {NULL}};
const Segment SGI21077[] = {{&S9878[44], 25}, {NULL}};
const Segment SGI21078[] = {{&S9878[44], 36}, {NULL}};
const Segment SGI21079[] = {{&S9878[30], 6}, {NULL}};
const Segment SGI21080[] = {{&S9878[128], 3}, {NULL}};
const Segment SGI21081[] = {{&S9878[94], 5}, {NULL}};
const Segment SGI21082[] = {{&S10385[38], 11}, {NULL}};
const Segment SGI21083[] = {{&S10385[31], 6}, {NULL}};
const Segment SGI21084[] = {{&S9880[44], 26}, {NULL}};
const Segment SGI21085[] = {{&S9880[44], 37}, {NULL}};
const Segment SGI21086[] = {{&S9880[30], 6}, {NULL}};
const Segment SGI21087[] = {{&S9880[129], 3}, {NULL}};
const Segment SGI21088[] = {{&S9880[95], 5}, {NULL}};
const Segment SGI21089[] = {{&S9881[43], 16}, {NULL}};
const Segment SGI21090[] = {{&S9881[43], 27}, {NULL}};
const Segment SGI21091[] = {{&S9881[30], 5}, {NULL}};
const Segment SGI21092[] = {{&S9881[118], 3}, {NULL}};
const Segment SGI21093[] = {{&S9881[84], 5}, {NULL}};
const Segment SGI21094[] = {{&S9882[38], 22}, {NULL}};
const Segment SGI21095[] = {{&S9882[31], 6}, {NULL}};
const Segment SGI21096[] = {{&S9883[44], 15}, {NULL}};
const Segment SGI21097[] = {{&S9883[44], 26}, {NULL}};
const Segment SGI21098[] = {{&S9883[30], 6}, {NULL}};
const Segment SGI21099[] = {{&S10392[44], 11}, {NULL}};
const Segment SGI21100[] = {{&S10392[44], 22}, {NULL}};
const Segment SGI21101[] = {{&S9883[118], 3}, {NULL}};
const Segment SGI21102[] = {{&S10392[30], 6}, {NULL}};
const Segment SGI21103[] = {{&S9883[84], 5}, {NULL}};
const Segment SGI21104[] = {{&S9884[38], 16}, {NULL}};
const Segment SGI21105[] = {{&S9884[31], 6}, {NULL}};
const Segment SGI21106[] = {{&S10395[42], 11}, {NULL}};
const Segment SGI21107[] = {{&S10395[30], 6}, {NULL}};
const Segment SGI21108[] = {{&S9886[44], 26}, {NULL}};
const Segment SGI21109[] = {{&S9886[44], 37}, {NULL}};
const Segment SGI21110[] = {{&S10395[112], 3}, {NULL}};
const Segment SGI21111[] = {{&S9886[30], 6}, {NULL}};
const Segment SGI21112[] = {{&S10395[78], 5}, {NULL}};
const Segment SGI21113[] = {{&S9886[129], 3}, {NULL}};
const Segment SGI21114[] = {{&S9886[95], 5}, {NULL}};
const Segment SGI21115[] = {{&S10396[38], 7}, {NULL}};
const Segment SGI21116[] = {{&S10396[31], 6}, {NULL}};
const Segment SGI21117[] = {{&S10397[44], 11}, {NULL}};
const Segment SGI21118[] = {{&S10397[44], 22}, {NULL}};
const Segment SGI21119[] = {{&S10397[30], 6}, {NULL}};
const Segment SGI21120[] = {{&S10397[83], 4}, {NULL}};
const Segment SGI21121[] = {{&S10397[83], 3}, {NULL}};
const Segment SGI21122[] = {{&S9890[44], 26}, {NULL}};
const Segment SGI21123[] = {{&S9890[44], 37}, {NULL}};
const Segment SGI21124[] = {{&S9890[30], 6}, {NULL}};
const Segment SGI21125[] = {{&S9890[129], 3}, {NULL}};
const Segment SGI21126[] = {{&S9890[95], 5}, {NULL}};
const Segment SGI21127[] = {{&S9891[62], 12}, {NULL}};
const Segment SGI21128[] = {{&S9891[49], 6}, {NULL}};
const Segment SGI21129[] = {{&S9891[122], 3}, {NULL}};
const Segment SGI21130[] = {{&S9891[88], 5}, {NULL}};
const Segment SGI21131[] = {{&S9893[44], 15}, {NULL}};
const Segment SGI21132[] = {{&S9893[44], 26}, {NULL}};
const Segment SGI21133[] = {{&S9893[30], 6}, {NULL}};
const Segment SGI21134[] = {{&S9893[118], 3}, {NULL}};
const Segment SGI21135[] = {{&S9893[84], 5}, {NULL}};
const Segment SGI21136[] = {{&S9894[44], 26}, {NULL}};
const Segment SGI21137[] = {{&S9894[44], 37}, {NULL}};
const Segment SGI21138[] = {{&S9894[30], 6}, {NULL}};
const Segment SGI21139[] = {{&S9894[129], 3}, {NULL}};
const Segment SGI21140[] = {{&S9894[95], 5}, {NULL}};
const Segment SGI21141[] = {{&S10406[44], 11}, {NULL}};
const Segment SGI21142[] = {{&S10406[44], 22}, {NULL}};
const Segment SGI21143[] = {{&S10406[30], 6}, {NULL}};
const Segment SGI21144[] = {{&S10406[116], 3}, {NULL}};
const Segment SGI21145[] = {{&S10406[80], 7}, {NULL}};
const Segment SGI21146[] = {{&S10406[136], 8}, {NULL}};
const Segment SGI21147[] = {{&S9895[44], 26}, {NULL}};
const Segment SGI21148[] = {{&S9895[44], 37}, {NULL}};
const Segment SGI21149[] = {{&S9895[30], 6}, {NULL}};
const Segment SGI21150[] = {{&S9895[129], 3}, {NULL}};
const Segment SGI21151[] = {{&S9895[95], 5}, {NULL}};
const Segment SGI21152[] = {{&S9896[43], 16}, {NULL}};
const Segment SGI21153[] = {{&S9896[43], 27}, {NULL}};
const Segment SGI21154[] = {{&S9896[118], 3}, {NULL}};
const Segment SGI21155[] = {{&S9896[84], 5}, {NULL}};
const Segment SGI21156[] = {{&S10409[44], 11}, {NULL}};
const Segment SGI21157[] = {{&S10409[44], 22}, {NULL}};
const Segment SGI21158[] = {{&S10409[30], 6}, {NULL}};
const Segment SGI21159[] = {{&S10409[83], 6}, {NULL}};
const Segment SGI21160[] = {{&S9897[44], 15}, {NULL}};
const Segment SGI21161[] = {{&S9897[44], 26}, {NULL}};
const Segment SGI21162[] = {{&S9897[30], 6}, {NULL}};
const Segment SGI21163[] = {{&S9897[118], 3}, {NULL}};
const Segment SGI21164[] = {{&S9897[84], 5}, {NULL}};
const Segment SGI21165[] = {{&S9898[44], 26}, {NULL}};
const Segment SGI21166[] = {{&S9898[44], 37}, {NULL}};
const Segment SGI21167[] = {{&S9898[30], 6}, {NULL}};
const Segment SGI21168[] = {{&S9898[129], 3}, {NULL}};
const Segment SGI21169[] = {{&S9898[95], 5}, {NULL}};
const Segment SGI21170[] = {{&S9899[44], 101}, {NULL}};
const Segment SGI21171[] = {{&S9899[44], 112}, {NULL}};
const Segment SGI21172[] = {{&S9899[30], 6}, {NULL}};
const Segment SGI21173[] = {{&S9899[204], 3}, {NULL}};
const Segment SGI21174[] = {{&S9899[170], 5}, {NULL}};
const Segment SGI21175[] = {{&S9900[44], 11}, {NULL}};
const Segment SGI21176[] = {{&S9900[44], 22}, {NULL}};
const Segment SGI21177[] = {{&S9900[30], 6}, {NULL}};
const Segment SGI21178[] = {{&S9900[83], 4}, {NULL}};
const Segment SGI21179[] = {{&S9900[83], 3}, {NULL}};
const Segment SGI21180[] = {{&S10421[44], 11}, {NULL}};
const Segment SGI21181[] = {{&S10421[44], 22}, {NULL}};
const Segment SGI21182[] = {{&S10421[30], 6}, {NULL}};
const Segment SGI21183[] = {{&S10421[80], 6}, {NULL}};
const Segment SGI21184[] = {{&S10429[44], 15}, {NULL}};
const Segment SGI21185[] = {{&S10429[44], 26}, {NULL}};
const Segment SGI21186[] = {{&S10429[30], 6}, {NULL}};
const Segment SGI21187[] = {{&S10429[118], 3}, {NULL}};
const Segment SGI21188[] = {{&S10429[84], 5}, {NULL}};
const Segment SGI21189[] = {{&S14940[42], 10}, {NULL}};
const Segment SGI21190[] = {{&S14940[42], 21}, {NULL}};
const Segment SGI21191[] = {{&S14940[30], 4}, {NULL}};
const Segment SGI21192[] = {{&S14940[111], 3}, {NULL}};
const Segment SGI21193[] = {{&S14940[77], 5}, {NULL}};
const Segment SGI21194[] = {{&S10366[44], 20}, {NULL}};
const Segment SGI21195[] = {{&S10366[44], 31}, {NULL}};
const Segment SGI21196[] = {{&S10366[30], 6}, {NULL}};
const Segment SGI21197[] = {{&S10366[123], 3}, {NULL}};
const Segment SGI21198[] = {{&S10366[89], 5}, {NULL}};
const Segment SGI21199[] = {{&S10367[44], 20}, {NULL}};
const Segment SGI21200[] = {{&S10367[44], 31}, {NULL}};
const Segment SGI21201[] = {{&S10367[30], 6}, {NULL}};
const Segment SGI21202[] = {{&S10367[123], 3}, {NULL}};
const Segment SGI21203[] = {{&S10367[89], 5}, {NULL}};
const Segment SGI21204[] = {{&S10378[44], 15}, {NULL}};
const Segment SGI21205[] = {{&S10378[44], 26}, {NULL}};
const Segment SGI21206[] = {{&S10378[30], 6}, {NULL}};
const Segment SGI21207[] = {{&S10378[118], 3}, {NULL}};
const Segment SGI21208[] = {{&S10378[84], 5}, {NULL}};
const Segment SGI21209[] = {{&S10380[44], 11}, {NULL}};
const Segment SGI21210[] = {{&S10380[44], 22}, {NULL}};
const Segment SGI21211[] = {{&S10380[30], 6}, {NULL}};
const Segment SGI21212[] = {{&S10380[114], 3}, {NULL}};
const Segment SGI21213[] = {{&S10380[80], 5}, {NULL}};
const Segment SGI21214[] = {{&S10386[44], 11}, {NULL}};
const Segment SGI21215[] = {{&S10386[44], 22}, {NULL}};
const Segment SGI21216[] = {{&S10386[30], 6}, {NULL}};
const Segment SGI21217[] = {{&S10386[114], 3}, {NULL}};
const Segment SGI21218[] = {{&S10386[80], 5}, {NULL}};
const Segment SGI21219[] = {{&S9901[44], 26}, {NULL}};
const Segment SGI21220[] = {{&S9901[44], 37}, {NULL}};
const Segment SGI21221[] = {{&S9901[30], 6}, {NULL}};
const Segment SGI21222[] = {{&S9901[129], 3}, {NULL}};
const Segment SGI21223[] = {{&S9901[95], 5}, {NULL}};
const Segment SGI21224[] = {{&S15016[42], 24}, {NULL}};
const Segment SGI21225[] = {{&S15016[42], 36}, {NULL}};
const Segment SGI21226[] = {{&S15016[30], 4}, {NULL}};
const Segment SGI21227[] = {{&S9902[44], 26}, {NULL}};
const Segment SGI21228[] = {{&S9902[44], 37}, {NULL}};
const Segment SGI21229[] = {{&S9902[30], 6}, {NULL}};
const Segment SGI21230[] = {{&S15016[126], 3}, {NULL}};
const Segment SGI21231[] = {{&S15016[92], 5}, {NULL}};
const Segment SGI21232[] = {{&S10412[38], 11}, {NULL}};
const Segment SGI21233[] = {{&S9902[129], 3}, {NULL}};
const Segment SGI21234[] = {{&S10412[31], 6}, {NULL}};
const Segment SGI21235[] = {{&S9902[95], 5}, {NULL}};
const Segment SGI21236[] = {{&S9903[44], 26}, {NULL}};
const Segment SGI21237[] = {{&S9903[44], 37}, {NULL}};
const Segment SGI21238[] = {{&S9903[30], 6}, {NULL}};
const Segment SGI21239[] = {{&S9903[129], 3}, {NULL}};
const Segment SGI21240[] = {{&S9903[95], 5}, {NULL}};
const Segment SGI21241[] = {{&S9904[44], 26}, {NULL}};
const Segment SGI21242[] = {{&S9904[44], 37}, {NULL}};
const Segment SGI21243[] = {{&S9904[30], 6}, {NULL}};
const Segment SGI21244[] = {{&S9904[129], 3}, {NULL}};
const Segment SGI21245[] = {{&S9904[95], 5}, {NULL}};
const Segment SGI21246[] = {{&S9905[44], 26}, {NULL}};
const Segment SGI21247[] = {{&S9905[44], 37}, {NULL}};
const Segment SGI21248[] = {{&S9905[30], 6}, {NULL}};
const Segment SGI21249[] = {{&S9905[129], 3}, {NULL}};
const Segment SGI21250[] = {{&S9905[95], 5}, {NULL}};
const Segment SGI21251[] = {{&S9908[44], 26}, {NULL}};
const Segment SGI21252[] = {{&S9908[44], 37}, {NULL}};
const Segment SGI21253[] = {{&S9908[30], 6}, {NULL}};
const Segment SGI21254[] = {{&S9908[129], 3}, {NULL}};
const Segment SGI21255[] = {{&S9908[95], 5}, {NULL}};
const Segment SGI21256[] = {{&S9909[44], 26}, {NULL}};
const Segment SGI21257[] = {{&S9909[44], 37}, {NULL}};
const Segment SGI21258[] = {{&S9909[30], 6}, {NULL}};
const Segment SGI21259[] = {{&S9909[129], 3}, {NULL}};
const Segment SGI21260[] = {{&S9909[95], 5}, {NULL}};
const Segment SGI21261[] = {{&S9911[44], 11}, {NULL}};
const Segment SGI21262[] = {{&S9911[44], 22}, {NULL}};
const Segment SGI21263[] = {{&S9911[30], 6}, {NULL}};
const Segment SGI21264[] = {{&S9911[83], 6}, {NULL}};
const Segment SGI21265[] = {{&S9912[44], 26}, {NULL}};
const Segment SGI21266[] = {{&S9912[44], 37}, {NULL}};
const Segment SGI21267[] = {{&S9912[30], 6}, {NULL}};
const Segment SGI21268[] = {{&S9912[129], 3}, {NULL}};
const Segment SGI21269[] = {{&S9912[95], 5}, {NULL}};
const Segment SGI21270[] = {{&S9913[44], 11}, {NULL}};
const Segment SGI21271[] = {{&S9913[44], 22}, {NULL}};
const Segment SGI21272[] = {{&S9913[30], 6}, {NULL}};
const Segment SGI21273[] = {{&S9913[114], 3}, {NULL}};
const Segment SGI21274[] = {{&S9913[80], 5}, {NULL}};
const Segment SGI21275[] = {{&S9916[44], 15}, {NULL}};
const Segment SGI21276[] = {{&S9916[44], 26}, {NULL}};
const Segment SGI21277[] = {{&S9916[30], 6}, {NULL}};
const Segment SGI21278[] = {{&S9916[118], 3}, {NULL}};
const Segment SGI21279[] = {{&S9916[84], 5}, {NULL}};
const Segment SGI21280[] = {{&S9917[44], 15}, {NULL}};
const Segment SGI21281[] = {{&S9917[44], 26}, {NULL}};
const Segment SGI21282[] = {{&S9917[30], 6}, {NULL}};
const Segment SGI21283[] = {{&S9917[118], 3}, {NULL}};
const Segment SGI21284[] = {{&S9917[84], 5}, {NULL}};
const Segment SGI21285[] = {{&S9918[44], 16}, {NULL}};
const Segment SGI21286[] = {{&S9918[44], 27}, {NULL}};
const Segment SGI21287[] = {{&S9918[30], 6}, {NULL}};
const Segment SGI21288[] = {{&S9918[119], 3}, {NULL}};
const Segment SGI21289[] = {{&S9918[85], 5}, {NULL}};
const Segment SGI21290[] = {{&S9919[44], 22}, {NULL}};
const Segment SGI21291[] = {{&S9919[44], 33}, {NULL}};
const Segment SGI21292[] = {{&S9919[30], 6}, {NULL}};
const Segment SGI21293[] = {{&S9919[94], 6}, {NULL}};
const Segment SGI21294[] = {{&S9921[44], 15}, {NULL}};
const Segment SGI21295[] = {{&S9921[44], 26}, {NULL}};
const Segment SGI21296[] = {{&S9921[30], 6}, {NULL}};
const Segment SGI21297[] = {{&S9921[118], 3}, {NULL}};
const Segment SGI21298[] = {{&S9921[84], 5}, {NULL}};
const Segment SGI21299[] = {{&S9922[44], 15}, {NULL}};
const Segment SGI21300[] = {{&S9922[44], 26}, {NULL}};
const Segment SGI21301[] = {{&S9922[30], 6}, {NULL}};
const Segment SGI21302[] = {{&S9922[118], 3}, {NULL}};
const Segment SGI21303[] = {{&S9922[84], 5}, {NULL}};
const Segment SGI21304[] = {{&S9924[44], 26}, {NULL}};
const Segment SGI21305[] = {{&S9924[44], 37}, {NULL}};
const Segment SGI21306[] = {{&S9924[30], 6}, {NULL}};
const Segment SGI21307[] = {{&S9924[129], 3}, {NULL}};
const Segment SGI21308[] = {{&S9924[95], 5}, {NULL}};
const Segment SGI21309[] = {{&S9925[43], 16}, {NULL}};
const Segment SGI21310[] = {{&S9925[43], 27}, {NULL}};
const Segment SGI21311[] = {{&S9925[30], 5}, {NULL}};
const Segment SGI21312[] = {{&S9925[118], 3}, {NULL}};
const Segment SGI21313[] = {{&S9925[84], 5}, {NULL}};
const Segment SGI21314[] = {{&S10363[44], 20}, {NULL}};
const Segment SGI21315[] = {{&S10363[44], 31}, {NULL}};
const Segment SGI21316[] = {{&S10363[30], 6}, {NULL}};
const Segment SGI21317[] = {{&S10363[123], 3}, {NULL}};
const Segment SGI21318[] = {{&S10363[89], 5}, {NULL}};
const Segment SGI21319[] = {{&S10364[44], 15}, {NULL}};
const Segment SGI21320[] = {{&S10364[44], 26}, {NULL}};
const Segment SGI21321[] = {{&S10364[30], 6}, {NULL}};
const Segment SGI21322[] = {{&S10364[118], 3}, {NULL}};
const Segment SGI21323[] = {{&S10364[84], 5}, {NULL}};
const Segment SGI21324[] = {{&S10377[44], 21}, {NULL}};
const Segment SGI21325[] = {{&S10377[44], 32}, {NULL}};
const Segment SGI21326[] = {{&S10377[30], 6}, {NULL}};
const Segment SGI21327[] = {{&S10377[124], 3}, {NULL}};
const Segment SGI21328[] = {{&S10377[90], 5}, {NULL}};
const Segment SGI21329[] = {{&S10379[44], 11}, {NULL}};
const Segment SGI21330[] = {{&S10379[44], 22}, {NULL}};
const Segment SGI21331[] = {{&S10379[30], 6}, {NULL}};
const Segment SGI21332[] = {{&S10379[114], 3}, {NULL}};
const Segment SGI21333[] = {{&S10379[80], 5}, {NULL}};
const Segment SGI21334[] = {{&S10387[44], 11}, {NULL}};
const Segment SGI21335[] = {{&S10387[44], 22}, {NULL}};
const Segment SGI21336[] = {{&S10387[30], 6}, {NULL}};
const Segment SGI21337[] = {{&S10387[114], 3}, {NULL}};
const Segment SGI21338[] = {{&S10387[80], 5}, {NULL}};
const Segment SGI21339[] = {{&S10389[44], 26}, {NULL}};
const Segment SGI21340[] = {{&S10389[44], 37}, {NULL}};
const Segment SGI21341[] = {{&S10389[30], 6}, {NULL}};
const Segment SGI21342[] = {{&S10389[129], 3}, {NULL}};
const Segment SGI21343[] = {{&S10389[95], 5}, {NULL}};
const Segment SGI21344[] = {{&S10391[44], 26}, {NULL}};
const Segment SGI21345[] = {{&S10391[44], 37}, {NULL}};
const Segment SGI21346[] = {{&S10391[30], 6}, {NULL}};
const Segment SGI21347[] = {{&S10391[129], 3}, {NULL}};
const Segment SGI21348[] = {{&S10391[95], 5}, {NULL}};
const Segment SGI21349[] = {{&S10393[44], 20}, {NULL}};
const Segment SGI21350[] = {{&S10393[44], 31}, {NULL}};
const Segment SGI21351[] = {{&S10393[30], 6}, {NULL}};
const Segment SGI21352[] = {{&S10393[123], 3}, {NULL}};
const Segment SGI21353[] = {{&S10393[89], 5}, {NULL}};
const Segment SGI21354[] = {{&S10394[38], 15}, {NULL}};
const Segment SGI21355[] = {{&S10394[31], 6}, {NULL}};
const Segment SGI21356[] = {{&S10399[38], 11}, {NULL}};
const Segment SGI21357[] = {{&S10399[31], 6}, {NULL}};
const Segment SGI21358[] = {{&S10400[38], 21}, {NULL}};
const Segment SGI21359[] = {{&S10400[31], 6}, {NULL}};
const Segment SGI21360[] = {{&S10401[44], 20}, {NULL}};
const Segment SGI21361[] = {{&S10401[44], 31}, {NULL}};
const Segment SGI21362[] = {{&S10401[30], 6}, {NULL}};
const Segment SGI21363[] = {{&S10401[123], 3}, {NULL}};
const Segment SGI21364[] = {{&S10401[89], 5}, {NULL}};
const Segment SGI21365[] = {{&S10402[44], 24}, {NULL}};
const Segment SGI21366[] = {{&S10402[44], 35}, {NULL}};
const Segment SGI21367[] = {{&S10402[30], 6}, {NULL}};
const Segment SGI21368[] = {{&S10402[127], 3}, {NULL}};
const Segment SGI21369[] = {{&S10402[93], 5}, {NULL}};
const Segment SGI21370[] = {{&S15963[44], 4}, {NULL}};
const Segment SGI21371[] = {{&S15963[44], 21}, {NULL}};
const Segment SGI21372[] = {{&S15963[30], 6}, {NULL}};
const Segment SGI21373[] = {{&S15963[113], 3}, {NULL}};
const Segment SGI21374[] = {{&S15963[79], 5}, {NULL}};
const Segment SGI21375[] = {{&S15974[44], 12}, {NULL}};
const Segment SGI21376[] = {{&S15974[44], 29}, {NULL}};
const Segment SGI21377[] = {{&S15974[30], 6}, {NULL}};
const Segment SGI21378[] = {{&S15974[121], 3}, {NULL}};
const Segment SGI21379[] = {{&S15974[87], 5}, {NULL}};
const Segment SGI21380[] = {{&S15981[38], 6}, {NULL}};
const Segment SGI21381[] = {{&S15981[31], 6}, {NULL}};
const Segment SGI21382[] = {{&S15983[44], 11}, {NULL}};
const Segment SGI21383[] = {{&S15983[44], 28}, {NULL}};
const Segment SGI21384[] = {{&S15983[30], 6}, {NULL}};
const Segment SGI21385[] = {{&S15983[120], 3}, {NULL}};
const Segment SGI21386[] = {{&S15983[86], 5}, {NULL}};
const Segment SGI21387[] = {{&S15986[44], 9}, {NULL}};
const Segment SGI21388[] = {{&S15986[44], 26}, {NULL}};
const Segment SGI21389[] = {{&S15986[30], 6}, {NULL}};
const Segment SGI21390[] = {{&S15986[118], 3}, {NULL}};
const Segment SGI21391[] = {{&S15986[84], 5}, {NULL}};
const Segment SGI21392[] = {{&S15987[44], 15}, {NULL}};
const Segment SGI21393[] = {{&S15987[44], 32}, {NULL}};
const Segment SGI21394[] = {{&S15987[30], 6}, {NULL}};
const Segment SGI21395[] = {{&S15987[124], 3}, {NULL}};
const Segment SGI21396[] = {{&S15987[90], 5}, {NULL}};
const Segment SGI21397[] = {{&S10403[44], 14}, {NULL}};
const Segment SGI21398[] = {{&S10403[44], 25}, {NULL}};
const Segment SGI21399[] = {{&S10403[30], 6}, {NULL}};
const Segment SGI21400[] = {{&S10403[117], 3}, {NULL}};
const Segment SGI21401[] = {{&S10403[83], 5}, {NULL}};
const Segment SGI21402[] = {{&S15989[44], 12}, {NULL}};
const Segment SGI21403[] = {{&S15989[44], 29}, {NULL}};
const Segment SGI21404[] = {{&S15989[30], 6}, {NULL}};
const Segment SGI21405[] = {{&S15989[121], 3}, {NULL}};
const Segment SGI21406[] = {{&S15989[87], 5}, {NULL}};
const Segment SGI21407[] = {{&S10404[44], 26}, {NULL}};
const Segment SGI21408[] = {{&S10404[44], 37}, {NULL}};
const Segment SGI21409[] = {{&S10404[30], 6}, {NULL}};
const Segment SGI21410[] = {{&S10404[129], 3}, {NULL}};
const Segment SGI21411[] = {{&S10404[95], 5}, {NULL}};
const Segment SGI21412[] = {{&S10405[44], 21}, {NULL}};
const Segment SGI21413[] = {{&S10405[44], 32}, {NULL}};
const Segment SGI21414[] = {{&S10405[30], 6}, {NULL}};
const Segment SGI21415[] = {{&S10405[93], 4}, {NULL}};
const Segment SGI21416[] = {{&S10405[93], 3}, {NULL}};
const Segment SGI21417[] = {{&S15994[44], 9}, {NULL}};
const Segment SGI21418[] = {{&S15994[44], 26}, {NULL}};
const Segment SGI21419[] = {{&S15994[30], 6}, {NULL}};
const Segment SGI21420[] = {{&S10407[44], 32}, {NULL}};
const Segment SGI21421[] = {{&S10407[44], 43}, {NULL}};
const Segment SGI21422[] = {{&S10407[30], 6}, {NULL}};
const Segment SGI21423[] = {{&S15994[118], 3}, {NULL}};
const Segment SGI21424[] = {{&S15994[84], 5}, {NULL}};
const Segment SGI21425[] = {{&S10407[135], 3}, {NULL}};
const Segment SGI21426[] = {{&S10407[101], 5}, {NULL}};
const Segment SGI21427[] = {{&S10408[44], 11}, {NULL}};
const Segment SGI21428[] = {{&S10408[44], 22}, {NULL}};
const Segment SGI21429[] = {{&S10408[30], 6}, {NULL}};
const Segment SGI21430[] = {{&S10408[83], 4}, {NULL}};
const Segment SGI21431[] = {{&S10408[83], 3}, {NULL}};
const Segment SGI21432[] = {{&S15996[44], 14}, {NULL}};
const Segment SGI21433[] = {{&S15996[44], 25}, {NULL}};
const Segment SGI21434[] = {{&S15996[30], 6}, {NULL}};
const Segment SGI21435[] = {{&S15996[117], 3}, {NULL}};
const Segment SGI21436[] = {{&S15996[83], 5}, {NULL}};
const Segment SGI21437[] = {{&S10410[44], 21}, {NULL}};
const Segment SGI21438[] = {{&S10410[44], 32}, {NULL}};
const Segment SGI21439[] = {{&S10410[30], 6}, {NULL}};
const Segment SGI21440[] = {{&S10410[93], 6}, {NULL}};
const Segment SGI21441[] = {{&S15999[44], 11}, {NULL}};
const Segment SGI21442[] = {{&S15999[44], 28}, {NULL}};
const Segment SGI21443[] = {{&S15999[30], 6}, {NULL}};
const Segment SGI21444[] = {{&S15999[120], 3}, {NULL}};
const Segment SGI21445[] = {{&S15999[86], 5}, {NULL}};
const Segment SGI21446[] = {{&S16000[44], 8}, {NULL}};
const Segment SGI21447[] = {{&S10413[44], 21}, {NULL}};
const Segment SGI21448[] = {{&S16000[44], 19}, {NULL}};
const Segment SGI21449[] = {{&S16000[30], 6}, {NULL}};
const Segment SGI21450[] = {{&S10413[44], 37}, {NULL}};
const Segment SGI21451[] = {{&S10413[30], 6}, {NULL}};
const Segment SGI21452[] = {{&S16000[111], 3}, {NULL}};
const Segment SGI21453[] = {{&S10413[129], 3}, {NULL}};
const Segment SGI21454[] = {{&S16000[77], 5}, {NULL}};
const Segment SGI21455[] = {{&S10413[95], 5}, {NULL}};
const Segment SGI21456[] = {{&S16002[44], 11}, {NULL}};
const Segment SGI21457[] = {{&S16002[44], 28}, {NULL}};
const Segment SGI21458[] = {{&S16002[30], 6}, {NULL}};
const Segment SGI21459[] = {{&S16002[120], 3}, {NULL}};
const Segment SGI21460[] = {{&S16002[86], 5}, {NULL}};
const Segment SGI21461[] = {{&S10416[42], 20}, {NULL}};
const Segment SGI21462[] = {{&S10416[30], 6}, {NULL}};
const Segment SGI21463[] = {{&S10416[121], 3}, {NULL}};
const Segment SGI21464[] = {{&S10416[87], 5}, {NULL}};
const Segment SGI21465[] = {{&S16003[44], 11}, {NULL}};
const Segment SGI21466[] = {{&S16003[44], 28}, {NULL}};
const Segment SGI21467[] = {{&S16003[30], 6}, {NULL}};
const Segment SGI21468[] = {{&S16003[120], 3}, {NULL}};
const Segment SGI21469[] = {{&S16003[86], 5}, {NULL}};
const Segment SGI21470[] = {{&S10417[44], 11}, {NULL}};
const Segment SGI21471[] = {{&S10417[44], 22}, {NULL}};
const Segment SGI21472[] = {{&S10417[30], 6}, {NULL}};
const Segment SGI21473[] = {{&S10417[83], 6}, {NULL}};
const Segment SGI21474[] = {{&S16004[44], 8}, {NULL}};
const Segment SGI21475[] = {{&S16004[44], 19}, {NULL}};
const Segment SGI21476[] = {{&S16004[30], 6}, {NULL}};
const Segment SGI21477[] = {{&S16004[111], 3}, {NULL}};
const Segment SGI21478[] = {{&S16004[77], 5}, {NULL}};
const Segment SGI21479[] = {{&S10418[44], 15}, {NULL}};
const Segment SGI21480[] = {{&S10418[44], 26}, {NULL}};
const Segment SGI21481[] = {{&S10418[30], 6}, {NULL}};
const Segment SGI21482[] = {{&S10418[87], 6}, {NULL}};
const Segment SGI21483[] = {{&S16006[44], 10}, {NULL}};
const Segment SGI21484[] = {{&S16006[44], 27}, {NULL}};
const Segment SGI21485[] = {{&S16006[30], 6}, {NULL}};
const Segment SGI21486[] = {{&S16006[119], 3}, {NULL}};
const Segment SGI21487[] = {{&S16006[85], 5}, {NULL}};
const Segment SGI21488[] = {{&S16009[44], 11}, {NULL}};
const Segment SGI21489[] = {{&S16009[44], 28}, {NULL}};
const Segment SGI21490[] = {{&S16009[30], 6}, {NULL}};
const Segment SGI21491[] = {{&S16009[120], 3}, {NULL}};
const Segment SGI21492[] = {{&S16009[86], 5}, {NULL}};
const Segment SGI21493[] = {{&S16010[44], 10}, {NULL}};
const Segment SGI21494[] = {{&S16010[44], 27}, {NULL}};
const Segment SGI21495[] = {{&S16010[30], 6}, {NULL}};
const Segment SGI21496[] = {{&S16010[119], 3}, {NULL}};
const Segment SGI21497[] = {{&S16010[85], 5}, {NULL}};
const Segment SGI21498[] = {{&S16012[44], 8}, {NULL}};
const Segment SGI21499[] = {{&S16012[44], 25}, {NULL}};
const Segment SGI21500[] = {{&S16012[30], 6}, {NULL}};
const Segment SGI21501[] = {{&S16012[117], 3}, {NULL}};
const Segment SGI21502[] = {{&S16012[83], 5}, {NULL}};
const Segment SGI21503[] = {{&S16021[44], 10}, {NULL}};
const Segment SGI21504[] = {{&S16021[44], 27}, {NULL}};
const Segment SGI21505[] = {{&S16021[30], 6}, {NULL}};
const Segment SGI21506[] = {{&S16021[119], 3}, {NULL}};
const Segment SGI21507[] = {{&S16021[85], 5}, {NULL}};
const Segment SGI21508[] = {{&S16025[44], 11}, {NULL}};
const Segment SGI21509[] = {{&S16025[44], 28}, {NULL}};
const Segment SGI21510[] = {{&S16025[30], 6}, {NULL}};
const Segment SGI21511[] = {{&S16025[120], 3}, {NULL}};
const Segment SGI21512[] = {{&S16025[86], 5}, {NULL}};
const Segment SGI21513[] = {{&S16027[44], 12}, {NULL}};
const Segment SGI21514[] = {{&S16027[44], 29}, {NULL}};
const Segment SGI21515[] = {{&S16027[30], 6}, {NULL}};
const Segment SGI21516[] = {{&S16027[121], 3}, {NULL}};
const Segment SGI21517[] = {{&S16027[87], 5}, {NULL}};
const Segment SGI21518[] = {{&S16031[44], 10}, {NULL}};
const Segment SGI21519[] = {{&S16031[44], 27}, {NULL}};
const Segment SGI21520[] = {{&S16031[30], 6}, {NULL}};
const Segment SGI21521[] = {{&S16031[119], 3}, {NULL}};
const Segment SGI21522[] = {{&S16031[85], 5}, {NULL}};
const Segment SGI21523[] = {{&S16032[44], 12}, {NULL}};
const Segment SGI21524[] = {{&S16032[44], 29}, {NULL}};
const Segment SGI21525[] = {{&S16032[30], 6}, {NULL}};
const Segment SGI21526[] = {{&S16032[121], 3}, {NULL}};
const Segment SGI21527[] = {{&S16032[87], 5}, {NULL}};
const Segment SGI21528[] = {{&S16034[44], 13}, {NULL}};
const Segment SGI21529[] = {{&S16034[44], 30}, {NULL}};
const Segment SGI21530[] = {{&S16034[30], 6}, {NULL}};
const Segment SGI21531[] = {{&S16034[122], 3}, {NULL}};
const Segment SGI21532[] = {{&S16034[88], 5}, {NULL}};
const Segment SGI21533[] = {{&S16035[44], 12}, {NULL}};
const Segment SGI21534[] = {{&S16035[44], 29}, {NULL}};
const Segment SGI21535[] = {{&S16035[30], 6}, {NULL}};
const Segment SGI21536[] = {{&S16035[121], 3}, {NULL}};
const Segment SGI21537[] = {{&S16035[87], 5}, {NULL}};
const Segment SGI21538[] = {{&S16038[44], 10}, {NULL}};
const Segment SGI21539[] = {{&S16038[44], 27}, {NULL}};
const Segment SGI21540[] = {{&S16038[30], 6}, {NULL}};
const Segment SGI21541[] = {{&S16038[119], 3}, {NULL}};
const Segment SGI21542[] = {{&S16038[85], 5}, {NULL}};
const Segment SGI21543[] = {{&S16040[44], 13}, {NULL}};
const Segment SGI21544[] = {{&S16040[44], 30}, {NULL}};
const Segment SGI21545[] = {{&S16040[30], 6}, {NULL}};
const Segment SGI21546[] = {{&S16040[122], 3}, {NULL}};
const Segment SGI21547[] = {{&S16040[88], 5}, {NULL}};
const Segment SGI21548[] = {{&S16041[44], 11}, {NULL}};
const Segment SGI21549[] = {{&S16041[44], 28}, {NULL}};
const Segment SGI21550[] = {{&S16041[30], 6}, {NULL}};
const Segment SGI21551[] = {{&S16041[120], 3}, {NULL}};
const Segment SGI21552[] = {{&S16041[86], 5}, {NULL}};
const Segment SGI21553[] = {{&S16045[44], 11}, {NULL}};
const Segment SGI21554[] = {{&S16045[44], 28}, {NULL}};
const Segment SGI21555[] = {{&S16045[30], 6}, {NULL}};
const Segment SGI21556[] = {{&S16045[120], 3}, {NULL}};
const Segment SGI21557[] = {{&S16045[86], 5}, {NULL}};
const Segment SGI21558[] = {{&S16048[44], 10}, {NULL}};
const Segment SGI21559[] = {{&S16048[44], 27}, {NULL}};
const Segment SGI21560[] = {{&S16048[30], 6}, {NULL}};
const Segment SGI21561[] = {{&S16048[119], 3}, {NULL}};
const Segment SGI21562[] = {{&S16048[85], 5}, {NULL}};
const Segment SGI21563[] = {{&S16049[44], 12}, {NULL}};
const Segment SGI21564[] = {{&S16049[44], 29}, {NULL}};
const Segment SGI21565[] = {{&S16049[30], 6}, {NULL}};
const Segment SGI21566[] = {{&S16049[121], 3}, {NULL}};
const Segment SGI21567[] = {{&S16049[87], 5}, {NULL}};
const Segment SGI21568[] = {{&S16050[44], 5}, {NULL}};
const Segment SGI21569[] = {{&S16050[44], 16}, {NULL}};
const Segment SGI21570[] = {{&S16050[30], 6}, {NULL}};
const Segment SGI21571[] = {{&S16050[108], 3}, {NULL}};
const Segment SGI21572[] = {{&S16050[74], 5}, {NULL}};
const Segment SGI21573[] = {{&S16052[44], 12}, {NULL}};
const Segment SGI21574[] = {{&S16052[44], 29}, {NULL}};
const Segment SGI21575[] = {{&S16052[30], 6}, {NULL}};
const Segment SGI21576[] = {{&S16052[121], 3}, {NULL}};
const Segment SGI21577[] = {{&S16052[87], 5}, {NULL}};
const Segment SGI21578[] = {{&S16063[44], 12}, {NULL}};
const Segment SGI21579[] = {{&S16063[44], 29}, {NULL}};
const Segment SGI21580[] = {{&S16063[30], 6}, {NULL}};
const Segment SGI21581[] = {{&S16063[121], 3}, {NULL}};
const Segment SGI21582[] = {{&S16063[87], 5}, {NULL}};
const Segment SGI21583[] = {{&S16067[44], 12}, {NULL}};
const Segment SGI21584[] = {{&S16067[44], 29}, {NULL}};
const Segment SGI21585[] = {{&S16067[30], 6}, {NULL}};
const Segment SGI21586[] = {{&S16067[121], 3}, {NULL}};
const Segment SGI21587[] = {{&S16067[87], 5}, {NULL}};
const Segment SGI21588[] = {{&S16070[44], 10}, {NULL}};
const Segment SGI21589[] = {{&S16070[44], 27}, {NULL}};
const Segment SGI21590[] = {{&S16070[30], 6}, {NULL}};
const Segment SGI21591[] = {{&S16070[119], 3}, {NULL}};
const Segment SGI21592[] = {{&S16070[85], 5}, {NULL}};
const Segment SGI21593[] = {{&S16082[44], 9}, {NULL}};
const Segment SGI21594[] = {{&S16082[44], 22}, {NULL}};
const Segment SGI21595[] = {{&S16082[30], 6}, {NULL}};
const Segment SGI21596[] = {{&S16082[114], 3}, {NULL}};
const Segment SGI21597[] = {{&S16082[80], 5}, {NULL}};
const Segment SGI21598[] = {{&S16084[44], 16}, {NULL}};
const Segment SGI21599[] = {{&S16084[44], 27}, {NULL}};
const Segment SGI21600[] = {{&S16084[30], 6}, {NULL}};
const Segment SGI21601[] = {{&S16084[119], 3}, {NULL}};
const Segment SGI21602[] = {{&S16084[85], 5}, {NULL}};
const Segment SGI21603[] = {{&S16088[44], 11}, {NULL}};
const Segment SGI21604[] = {{&S16088[44], 28}, {NULL}};
const Segment SGI21605[] = {{&S16088[30], 6}, {NULL}};
const Segment SGI21606[] = {{&S16088[120], 3}, {NULL}};
const Segment SGI21607[] = {{&S16088[86], 5}, {NULL}};
const Segment SGI21608[] = {{&S16090[44], 8}, {NULL}};
const Segment SGI21609[] = {{&S16090[44], 25}, {NULL}};
const Segment SGI21610[] = {{&S16090[30], 6}, {NULL}};
const Segment SGI21611[] = {{&S16090[117], 3}, {NULL}};
const Segment SGI21612[] = {{&S16090[83], 5}, {NULL}};
const Segment SGI21613[] = {{&S16091[44], 10}, {NULL}};
const Segment SGI21614[] = {{&S16091[44], 27}, {NULL}};
const Segment SGI21615[] = {{&S16091[30], 6}, {NULL}};
const Segment SGI21616[] = {{&S16091[119], 3}, {NULL}};
const Segment SGI21617[] = {{&S16091[85], 5}, {NULL}};
const Segment SGI21618[] = {{&S16092[44], 9}, {NULL}};
const Segment SGI21619[] = {{&S16092[44], 22}, {NULL}};
const Segment SGI21620[] = {{&S16092[30], 6}, {NULL}};
const Segment SGI21621[] = {{&S16092[114], 3}, {NULL}};
const Segment SGI21622[] = {{&S16092[80], 5}, {NULL}};
const Segment SGI21623[] = {{&S16094[44], 11}, {NULL}};
const Segment SGI21624[] = {{&S16094[44], 28}, {NULL}};
const Segment SGI21625[] = {{&S16094[30], 6}, {NULL}};
const Segment SGI21626[] = {{&S16094[120], 3}, {NULL}};
const Segment SGI21627[] = {{&S16094[86], 5}, {NULL}};
const Segment SGI21628[] = {{&S16102[44], 10}, {NULL}};
const Segment SGI21629[] = {{&S16102[44], 27}, {NULL}};
const Segment SGI21630[] = {{&S16102[30], 6}, {NULL}};
const Segment SGI21631[] = {{&S16102[119], 3}, {NULL}};
const Segment SGI21632[] = {{&S16102[85], 5}, {NULL}};
const Segment SGI21633[] = {{&S16108[44], 12}, {NULL}};
const Segment SGI21634[] = {{&S16108[44], 29}, {NULL}};
const Segment SGI21635[] = {{&S16108[30], 6}, {NULL}};
const Segment SGI21636[] = {{&S16108[121], 3}, {NULL}};
const Segment SGI21637[] = {{&S16108[87], 5}, {NULL}};
const Segment SGI21638[] = {{&S16110[44], 7}, {NULL}};
const Segment SGI21639[] = {{&S16110[44], 24}, {NULL}};
const Segment SGI21640[] = {{&S16110[30], 6}, {NULL}};
const Segment SGI21641[] = {{&S16110[116], 3}, {NULL}};
const Segment SGI21642[] = {{&S16110[82], 5}, {NULL}};
const Segment SGI21643[] = {{&S16115[44], 11}, {NULL}};
const Segment SGI21644[] = {{&S16115[44], 28}, {NULL}};
const Segment SGI21645[] = {{&S16115[30], 6}, {NULL}};
const Segment SGI21646[] = {{&S16115[120], 3}, {NULL}};
const Segment SGI21647[] = {{&S16115[86], 5}, {NULL}};
const Segment SGI21648[] = {{&S16119[44], 11}, {NULL}};
const Segment SGI21649[] = {{&S16119[44], 28}, {NULL}};
const Segment SGI21650[] = {{&S16119[30], 6}, {NULL}};
const Segment SGI21651[] = {{&S16119[120], 3}, {NULL}};
const Segment SGI21652[] = {{&S16119[86], 5}, {NULL}};
const Segment SGI21653[] = {{&S16134[44], 11}, {NULL}};
const Segment SGI21654[] = {{&S16134[44], 28}, {NULL}};
const Segment SGI21655[] = {{&S16134[30], 6}, {NULL}};
const Segment SGI21656[] = {{&S16134[120], 3}, {NULL}};
const Segment SGI21657[] = {{&S16134[86], 5}, {NULL}};
const Segment SGI21658[] = {{&S16136[44], 12}, {NULL}};
const Segment SGI21659[] = {{&S16136[44], 29}, {NULL}};
const Segment SGI21660[] = {{&S16136[30], 6}, {NULL}};
const Segment SGI21661[] = {{&S16136[121], 3}, {NULL}};
const Segment SGI21662[] = {{&S16136[87], 5}, {NULL}};
const Segment SGI21663[] = {{&S16139[44], 8}, {NULL}};
const Segment SGI21664[] = {{&S16139[44], 25}, {NULL}};
const Segment SGI21665[] = {{&S16139[30], 6}, {NULL}};
const Segment SGI21666[] = {{&S16139[117], 3}, {NULL}};
const Segment SGI21667[] = {{&S16139[83], 5}, {NULL}};
const Segment SGI21668[] = {{&S16142[44], 8}, {NULL}};
const Segment SGI21669[] = {{&S16142[44], 25}, {NULL}};
const Segment SGI21670[] = {{&S16142[30], 6}, {NULL}};
const Segment SGI21671[] = {{&S16142[117], 3}, {NULL}};
const Segment SGI21672[] = {{&S16142[83], 5}, {NULL}};
const Segment SGI21673[] = {{&S16146[44], 7}, {NULL}};
const Segment SGI21674[] = {{&S16146[44], 24}, {NULL}};
const Segment SGI21675[] = {{&S16146[30], 6}, {NULL}};
const Segment SGI21676[] = {{&S16146[116], 3}, {NULL}};
const Segment SGI21677[] = {{&S16146[82], 5}, {NULL}};
const Segment SGI21678[] = {{&S16155[44], 9}, {NULL}};
const Segment SGI21679[] = {{&S16155[44], 26}, {NULL}};
const Segment SGI21680[] = {{&S16155[30], 6}, {NULL}};
const Segment SGI21681[] = {{&S16155[118], 3}, {NULL}};
const Segment SGI21682[] = {{&S16155[84], 5}, {NULL}};
const Segment SGI21683[] = {{&S16163[44], 16}, {NULL}};
const Segment SGI21684[] = {{&S16163[44], 27}, {NULL}};
const Segment SGI21685[] = {{&S16163[30], 6}, {NULL}};
const Segment SGI21686[] = {{&S16163[119], 3}, {NULL}};
const Segment SGI21687[] = {{&S16163[85], 5}, {NULL}};
const Segment SGI21688[] = {{&S16165[44], 4}, {NULL}};
const Segment SGI21689[] = {{&S16165[44], 21}, {NULL}};
const Segment SGI21690[] = {{&S16165[30], 6}, {NULL}};
const Segment SGI21691[] = {{&S16165[113], 3}, {NULL}};
const Segment SGI21692[] = {{&S16165[79], 5}, {NULL}};
const Segment SGI21693[] = {{&S16171[44], 9}, {NULL}};
const Segment SGI21694[] = {{&S16171[44], 26}, {NULL}};
const Segment SGI21695[] = {{&S16171[30], 6}, {NULL}};
const Segment SGI21696[] = {{&S16171[118], 3}, {NULL}};
const Segment SGI21697[] = {{&S16171[84], 5}, {NULL}};
const Segment SGI21698[] = {{&S16175[44], 10}, {NULL}};
const Segment SGI21699[] = {{&S16175[44], 27}, {NULL}};
const Segment SGI21700[] = {{&S16175[30], 6}, {NULL}};
const Segment SGI21701[] = {{&S16175[119], 3}, {NULL}};
const Segment SGI21702[] = {{&S16175[85], 5}, {NULL}};
const Segment SGI21703[] = {{&S16180[44], 8}, {NULL}};
const Segment SGI21704[] = {{&S16180[44], 25}, {NULL}};
const Segment SGI21705[] = {{&S16180[30], 6}, {NULL}};
const Segment SGI21706[] = {{&S16180[117], 3}, {NULL}};
const Segment SGI21707[] = {{&S16180[83], 5}, {NULL}};
const Segment SGI21708[] = {{&S16181[44], 6}, {NULL}};
const Segment SGI21709[] = {{&S16181[44], 23}, {NULL}};
const Segment SGI21710[] = {{&S16181[30], 6}, {NULL}};
const Segment SGI21711[] = {{&S16181[115], 3}, {NULL}};
const Segment SGI21712[] = {{&S16181[81], 5}, {NULL}};
const Segment SGI21713[] = {{&S16185[44], 11}, {NULL}};
const Segment SGI21714[] = {{&S16185[44], 28}, {NULL}};
const Segment SGI21715[] = {{&S16185[30], 6}, {NULL}};
const Segment SGI21716[] = {{&S16185[120], 3}, {NULL}};
const Segment SGI21717[] = {{&S16185[86], 5}, {NULL}};
const Segment SGI21718[] = {{&S16188[44], 12}, {NULL}};
const Segment SGI21719[] = {{&S16188[44], 29}, {NULL}};
const Segment SGI21720[] = {{&S16188[30], 6}, {NULL}};
const Segment SGI21721[] = {{&S16188[121], 3}, {NULL}};
const Segment SGI21722[] = {{&S16188[87], 5}, {NULL}};
const Segment SGI21723[] = {{&S16194[44], 8}, {NULL}};
const Segment SGI21724[] = {{&S16194[44], 25}, {NULL}};
const Segment SGI21725[] = {{&S16194[30], 6}, {NULL}};
const Segment SGI21726[] = {{&S16194[86], 4}, {NULL}};
const Segment SGI21727[] = {{&S16194[86], 3}, {NULL}};
const Segment SGI21728[] = {{&S16196[44], 12}, {NULL}};
const Segment SGI21729[] = {{&S16196[44], 29}, {NULL}};
const Segment SGI21730[] = {{&S16196[30], 6}, {NULL}};
const Segment SGI21731[] = {{&S16196[121], 3}, {NULL}};
const Segment SGI21732[] = {{&S16196[87], 5}, {NULL}};
const Segment SGI21733[] = {{&S16197[44], 11}, {NULL}};
const Segment SGI21734[] = {{&S16197[44], 28}, {NULL}};
const Segment SGI21735[] = {{&S16197[30], 6}, {NULL}};
const Segment SGI21736[] = {{&S16197[120], 3}, {NULL}};
const Segment SGI21737[] = {{&S16197[86], 5}, {NULL}};
const Segment SGI21738[] = {{&S16200[44], 10}, {NULL}};
const Segment SGI21739[] = {{&S16200[44], 27}, {NULL}};
const Segment SGI21740[] = {{&S16200[30], 6}, {NULL}};
const Segment SGI21741[] = {{&S16200[119], 3}, {NULL}};
const Segment SGI21742[] = {{&S16200[85], 5}, {NULL}};
const Segment SGI21743[] = {{&S16206[44], 8}, {NULL}};
const Segment SGI21744[] = {{&S16206[44], 25}, {NULL}};
const Segment SGI21745[] = {{&S16206[30], 6}, {NULL}};
const Segment SGI21746[] = {{&S16206[117], 3}, {NULL}};
const Segment SGI21747[] = {{&S16206[83], 5}, {NULL}};
const Segment SGI21748[] = {{&S16211[44], 12}, {NULL}};
const Segment SGI21749[] = {{&S16211[44], 29}, {NULL}};
const Segment SGI21750[] = {{&S16211[30], 6}, {NULL}};
const Segment SGI21751[] = {{&S16211[121], 3}, {NULL}};
const Segment SGI21752[] = {{&S16211[87], 5}, {NULL}};
const Segment SGI21753[] = {{&S16214[38], 26}, {NULL}};
const Segment SGI21754[] = {{&S16214[31], 6}, {NULL}};
const Segment SGI21755[] = {{&S10420[44], 26}, {NULL}};
const Segment SGI21756[] = {{&S10420[44], 37}, {NULL}};
const Segment SGI21757[] = {{&S10420[30], 6}, {NULL}};
const Segment SGI21758[] = {{&S10420[129], 3}, {NULL}};
const Segment SGI21759[] = {{&S10420[95], 5}, {NULL}};
const Segment SGI21760[] = {{&S10422[44], 32}, {NULL}};
const Segment SGI21761[] = {{&S10422[30], 6}, {NULL}};
const Segment SGI21762[] = {{&S10423[44], 11}, {NULL}};
const Segment SGI21763[] = {{&S10423[44], 22}, {NULL}};
const Segment SGI21764[] = {{&S10423[30], 6}, {NULL}};
const Segment SGI21765[] = {{&S10423[80], 6}, {NULL}};
const Segment SGI21766[] = {{&S10424[44], 26}, {NULL}};
const Segment SGI21767[] = {{&S10424[44], 37}, {NULL}};
const Segment SGI21768[] = {{&S10424[30], 6}, {NULL}};
const Segment SGI21769[] = {{&S10424[129], 3}, {NULL}};
const Segment SGI21770[] = {{&S10424[95], 5}, {NULL}};
const Segment SGI21771[] = {{&S10425[44], 21}, {NULL}};
const Segment SGI21772[] = {{&S10425[44], 32}, {NULL}};
const Segment SGI21773[] = {{&S10425[30], 6}, {NULL}};
const Segment SGI21774[] = {{&S10425[90], 6}, {NULL}};
const Segment SGI21775[] = {{&S10426[42], 21}, {NULL}};
const Segment SGI21776[] = {{&S10426[30], 6}, {NULL}};
const Segment SGI21777[] = {{&S10426[122], 3}, {NULL}};
const Segment SGI21778[] = {{&S10426[88], 5}, {NULL}};
const Segment SGI21779[] = {{&S10432[44], 15}, {NULL}};
const Segment SGI21780[] = {{&S10432[44], 26}, {NULL}};
const Segment SGI21781[] = {{&S10432[30], 6}, {NULL}};
const Segment SGI21782[] = {{&S10432[118], 3}, {NULL}};
const Segment SGI21783[] = {{&S10432[84], 5}, {NULL}};
const Segment SGI21784[] = {{&S16248[44], 11}, {NULL}};
const Segment SGI21785[] = {{&S16248[44], 28}, {NULL}};
const Segment SGI21786[] = {{&S16248[30], 6}, {NULL}};
const Segment SGI21787[] = {{&S16248[120], 3}, {NULL}};
const Segment SGI21788[] = {{&S16248[86], 5}, {NULL}};
const Segment SGI21789[] = {{&S16251[44], 5}, {NULL}};
const Segment SGI21790[] = {{&S16251[44], 22}, {NULL}};
const Segment SGI21791[] = {{&S16251[30], 6}, {NULL}};
const Segment SGI21792[] = {{&S16251[114], 3}, {NULL}};
const Segment SGI21793[] = {{&S16251[80], 5}, {NULL}};
const Segment SGI21794[] = {{&S16257[44], 8}, {NULL}};
const Segment SGI21795[] = {{&S16257[44], 25}, {NULL}};
const Segment SGI21796[] = {{&S16257[30], 6}, {NULL}};
const Segment SGI21797[] = {{&S16257[117], 3}, {NULL}};
const Segment SGI21798[] = {{&S16257[83], 5}, {NULL}};
const Segment SGI21799[] = {{&S16271[44], 11}, {NULL}};
const Segment SGI21800[] = {{&S16271[44], 28}, {NULL}};
const Segment SGI21801[] = {{&S16271[30], 6}, {NULL}};
const Segment SGI21802[] = {{&S16271[120], 3}, {NULL}};
const Segment SGI21803[] = {{&S16271[86], 5}, {NULL}};
const Segment SGI21804[] = {{&S16273[44], 10}, {NULL}};
const Segment SGI21805[] = {{&S16273[44], 27}, {NULL}};
const Segment SGI21806[] = {{&S16273[30], 6}, {NULL}};
const Segment SGI21807[] = {{&S16273[119], 3}, {NULL}};
const Segment SGI21808[] = {{&S16273[85], 5}, {NULL}};
const Segment SGI21809[] = {{&S16280[44], 12}, {NULL}};
const Segment SGI21810[] = {{&S16280[44], 29}, {NULL}};
const Segment SGI21811[] = {{&S16280[30], 6}, {NULL}};
const Segment SGI21812[] = {{&S16280[121], 3}, {NULL}};
const Segment SGI21813[] = {{&S16280[87], 5}, {NULL}};
const Segment SGI21814[] = {{&S16282[44], 10}, {NULL}};
const Segment SGI21815[] = {{&S16282[44], 27}, {NULL}};
const Segment SGI21816[] = {{&S16282[30], 6}, {NULL}};
const Segment SGI21817[] = {{&S16282[119], 3}, {NULL}};
const Segment SGI21818[] = {{&S16282[85], 5}, {NULL}};
const Segment SGI21819[] = {{&S16283[44], 8}, {NULL}};
const Segment SGI21820[] = {{&S16283[44], 25}, {NULL}};
const Segment SGI21821[] = {{&S16283[30], 6}, {NULL}};
const Segment SGI21822[] = {{&S16283[117], 3}, {NULL}};
const Segment SGI21823[] = {{&S16283[83], 5}, {NULL}};
const Segment SGI21824[] = {{&S16290[44], 10}, {NULL}};
const Segment SGI21825[] = {{&S16290[44], 27}, {NULL}};
const Segment SGI21826[] = {{&S16290[30], 6}, {NULL}};
const Segment SGI21827[] = {{&S16290[119], 3}, {NULL}};
const Segment SGI21828[] = {{&S16290[85], 5}, {NULL}};
const Segment SGI21829[] = {{&S16295[44], 5}, {NULL}};
const Segment SGI21830[] = {{&S16295[44], 22}, {NULL}};
const Segment SGI21831[] = {{&S16295[30], 6}, {NULL}};
const Segment SGI21832[] = {{&S16295[114], 3}, {NULL}};
const Segment SGI21833[] = {{&S16295[80], 5}, {NULL}};
const Segment SGI21834[] = {{&S16311[44], 6}, {NULL}};
const Segment SGI21835[] = {{&S16311[44], 23}, {NULL}};
const Segment SGI21836[] = {{&S16311[30], 6}, {NULL}};
const Segment SGI21837[] = {{&S16311[115], 3}, {NULL}};
const Segment SGI21838[] = {{&S16311[81], 5}, {NULL}};
const Segment SGI21839[] = {{&S16325[44], 6}, {NULL}};
const Segment SGI21840[] = {{&S16325[44], 23}, {NULL}};
const Segment SGI21841[] = {{&S16325[30], 6}, {NULL}};
const Segment SGI21842[] = {{&S16325[115], 3}, {NULL}};
const Segment SGI21843[] = {{&S16325[81], 5}, {NULL}};
const Segment SGI21844[] = {{&S16330[44], 11}, {NULL}};
const Segment SGI21845[] = {{&S16330[44], 28}, {NULL}};
const Segment SGI21846[] = {{&S16330[30], 6}, {NULL}};
const Segment SGI21847[] = {{&S16330[120], 3}, {NULL}};
const Segment SGI21848[] = {{&S16330[86], 5}, {NULL}};
const Segment SGI21849[] = {{&S16332[44], 9}, {NULL}};
const Segment SGI21850[] = {{&S16332[44], 26}, {NULL}};
const Segment SGI21851[] = {{&S16332[30], 6}, {NULL}};
const Segment SGI21852[] = {{&S16332[118], 3}, {NULL}};
const Segment SGI21853[] = {{&S16332[84], 5}, {NULL}};
const Segment SGI21854[] = {{&S16334[44], 8}, {NULL}};
const Segment SGI21855[] = {{&S16334[44], 25}, {NULL}};
const Segment SGI21856[] = {{&S16334[30], 6}, {NULL}};
const Segment SGI21857[] = {{&S16334[117], 3}, {NULL}};
const Segment SGI21858[] = {{&S16334[83], 5}, {NULL}};
const Segment SGI21859[] = {{&S16350[44], 10}, {NULL}};
const Segment SGI21860[] = {{&S16350[44], 27}, {NULL}};
const Segment SGI21861[] = {{&S16350[30], 6}, {NULL}};
const Segment SGI21862[] = {{&S16350[119], 3}, {NULL}};
const Segment SGI21863[] = {{&S16350[85], 5}, {NULL}};
const Segment SGI21864[] = {{&S16353[44], 14}, {NULL}};
const Segment SGI21865[] = {{&S16353[44], 31}, {NULL}};
const Segment SGI21866[] = {{&S16353[30], 6}, {NULL}};
const Segment SGI21867[] = {{&S16353[123], 3}, {NULL}};
const Segment SGI21868[] = {{&S16353[89], 5}, {NULL}};
const Segment SGI21869[] = {{&S16355[44], 12}, {NULL}};
const Segment SGI21870[] = {{&S16355[44], 29}, {NULL}};
const Segment SGI21871[] = {{&S16355[30], 6}, {NULL}};
const Segment SGI21872[] = {{&S16355[121], 3}, {NULL}};
const Segment SGI21873[] = {{&S16355[87], 5}, {NULL}};
const Segment SGI21874[] = {{&S16358[44], 8}, {NULL}};
const Segment SGI21875[] = {{&S16358[44], 25}, {NULL}};
const Segment SGI21876[] = {{&S16358[30], 6}, {NULL}};
const Segment SGI21877[] = {{&S16358[117], 3}, {NULL}};
const Segment SGI21878[] = {{&S16358[83], 5}, {NULL}};
const Segment SGI21879[] = {{&S16369[44], 11}, {NULL}};
const Segment SGI21880[] = {{&S16369[44], 28}, {NULL}};
const Segment SGI21881[] = {{&S16369[30], 6}, {NULL}};
const Segment SGI21882[] = {{&S16369[120], 3}, {NULL}};
const Segment SGI21883[] = {{&S16369[86], 5}, {NULL}};
const Segment SGI21884[] = {{&S16373[44], 13}, {NULL}};
const Segment SGI21885[] = {{&S16373[44], 30}, {NULL}};
const Segment SGI21886[] = {{&S16373[30], 6}, {NULL}};
const Segment SGI21887[] = {{&S16373[122], 3}, {NULL}};
const Segment SGI21888[] = {{&S16373[88], 5}, {NULL}};
const Segment SGI21889[] = {{&S16397[44], 5}, {NULL}};
const Segment SGI21890[] = {{&S16397[44], 22}, {NULL}};
const Segment SGI21891[] = {{&S16397[30], 6}, {NULL}};
const Segment SGI21892[] = {{&S16397[114], 3}, {NULL}};
const Segment SGI21893[] = {{&S16397[80], 5}, {NULL}};
const Segment SGI21894[] = {{&S15984[42], 14}, {NULL}};
const Segment SGI21895[] = {{&S15984[42], 25}, {NULL}};
const Segment SGI21896[] = {{&S15984[30], 4}, {NULL}};
const Segment SGI21897[] = {{&S15984[115], 3}, {NULL}};
const Segment SGI21898[] = {{&S15984[81], 5}, {NULL}};
const Segment SGI21899[] = {{&S15995[42], 9}, {NULL}};
const Segment SGI21900[] = {{&S15995[42], 26}, {NULL}};
const Segment SGI21901[] = {{&S15995[30], 4}, {NULL}};
const Segment SGI21902[] = {{&S15995[116], 3}, {NULL}};
const Segment SGI21903[] = {{&S15995[82], 5}, {NULL}};
const Segment SGI21904[] = {{&S16425[44], 9}, {NULL}};
const Segment SGI21905[] = {{&S16425[44], 26}, {NULL}};
const Segment SGI21906[] = {{&S16425[30], 6}, {NULL}};
const Segment SGI21907[] = {{&S16425[118], 3}, {NULL}};
const Segment SGI21908[] = {{&S16425[84], 5}, {NULL}};
const Segment SGI21909[] = {{&S16426[44], 9}, {NULL}};
const Segment SGI21910[] = {{&S16426[44], 26}, {NULL}};
const Segment SGI21911[] = {{&S16426[30], 6}, {NULL}};
const Segment SGI21912[] = {{&S16426[118], 3}, {NULL}};
const Segment SGI21913[] = {{&S16426[84], 5}, {NULL}};
const Segment SGI21914[] = {{&S16024[42], 13}, {NULL}};
const Segment SGI21915[] = {{&S16024[42], 30}, {NULL}};
const Segment SGI21916[] = {{&S16024[30], 4}, {NULL}};
const Segment SGI21917[] = {{&S16024[120], 3}, {NULL}};
const Segment SGI21918[] = {{&S16024[86], 5}, {NULL}};
const Segment SGI21919[] = {{&S16463[44], 10}, {NULL}};
const Segment SGI21920[] = {{&S16463[44], 27}, {NULL}};
const Segment SGI21921[] = {{&S16463[30], 6}, {NULL}};
const Segment SGI21922[] = {{&S16463[119], 3}, {NULL}};
const Segment SGI21923[] = {{&S16463[85], 5}, {NULL}};
const Segment SGI21924[] = {{&S16072[42], 16}, {NULL}};
const Segment SGI21925[] = {{&S16072[42], 27}, {NULL}};
const Segment SGI21926[] = {{&S16072[30], 4}, {NULL}};
const Segment SGI21927[] = {{&S16072[117], 3}, {NULL}};
const Segment SGI21928[] = {{&S16072[83], 5}, {NULL}};
const Segment SGI21929[] = {{&S16502[44], 11}, {NULL}};
const Segment SGI21930[] = {{&S16502[44], 28}, {NULL}};
const Segment SGI21931[] = {{&S16502[30], 6}, {NULL}};
const Segment SGI21932[] = {{&S16502[120], 3}, {NULL}};
const Segment SGI21933[] = {{&S16502[86], 5}, {NULL}};
const Segment SGI21934[] = {{&S16543[44], 9}, {NULL}};
const Segment SGI21935[] = {{&S16543[44], 26}, {NULL}};
const Segment SGI21936[] = {{&S16543[30], 6}, {NULL}};
const Segment SGI21937[] = {{&S16543[118], 3}, {NULL}};
const Segment SGI21938[] = {{&S16543[84], 5}, {NULL}};
const Segment SGI21939[] = {{&S16594[44], 8}, {NULL}};
const Segment SGI21940[] = {{&S16594[44], 25}, {NULL}};
const Segment SGI21941[] = {{&S16594[30], 6}, {NULL}};
const Segment SGI21942[] = {{&S16594[117], 3}, {NULL}};
const Segment SGI21943[] = {{&S16594[83], 5}, {NULL}};
const Segment SGI21944[] = {{&S16599[44], 8}, {NULL}};
const Segment SGI21945[] = {{&S16599[44], 25}, {NULL}};
const Segment SGI21946[] = {{&S16599[30], 6}, {NULL}};
const Segment SGI21947[] = {{&S16599[117], 3}, {NULL}};
const Segment SGI21948[] = {{&S16599[83], 5}, {NULL}};
const Segment SGI21949[] = {{&S16632[44], 8}, {NULL}};
const Segment SGI21950[] = {{&S16632[44], 25}, {NULL}};
const Segment SGI21951[] = {{&S16632[30], 6}, {NULL}};
const Segment SGI21952[] = {{&S16632[117], 3}, {NULL}};
const Segment SGI21953[] = {{&S16632[83], 5}, {NULL}};
const Segment SGI21954[] = {{&S16658[44], 11}, {NULL}};
const Segment SGI21955[] = {{&S16658[44], 22}, {NULL}};
const Segment SGI21956[] = {{&S16658[30], 6}, {NULL}};
const Segment SGI21957[] = {{&S16658[114], 3}, {NULL}};
const Segment SGI21958[] = {{&S16658[80], 5}, {NULL}};
const Segment SGI21959[] = {{&S16675[44], 13}, {NULL}};
const Segment SGI21960[] = {{&S16675[44], 30}, {NULL}};
const Segment SGI21961[] = {{&S16675[30], 6}, {NULL}};
const Segment SGI21962[] = {{&S16675[121], 3}, {NULL}};
const Segment SGI21963[] = {{&S16675[88], 5}, {NULL}};
const Segment SGI21964[] = {{&S16690[44], 7}, {NULL}};
const Segment SGI21965[] = {{&S16690[44], 24}, {NULL}};
const Segment SGI21966[] = {{&S16690[30], 6}, {NULL}};
const Segment SGI21967[] = {{&S16690[116], 3}, {NULL}};
const Segment SGI21968[] = {{&S16690[82], 5}, {NULL}};
const Segment SGI21969[] = {{&S16691[44], 9}, {NULL}};
const Segment SGI21970[] = {{&S16691[44], 26}, {NULL}};
const Segment SGI21971[] = {{&S16691[30], 6}, {NULL}};
const Segment SGI21972[] = {{&S16691[118], 3}, {NULL}};
const Segment SGI21973[] = {{&S16691[84], 5}, {NULL}};
const Segment SGI21974[] = {{&S16118[42], 10}, {NULL}};
const Segment SGI21975[] = {{&S16118[42], 21}, {NULL}};
const Segment SGI21976[] = {{&S16118[30], 4}, {NULL}};
const Segment SGI21977[] = {{&S16118[111], 3}, {NULL}};
const Segment SGI21978[] = {{&S16118[77], 5}, {NULL}};
const Segment SGI21979[] = {{&S16183[42], 9}, {NULL}};
const Segment SGI21980[] = {{&S16183[42], 26}, {NULL}};
const Segment SGI21981[] = {{&S16183[30], 4}, {NULL}};
const Segment SGI21982[] = {{&S16183[116], 3}, {NULL}};
const Segment SGI21983[] = {{&S16183[82], 5}, {NULL}};
const Segment SGI21984[] = {{&S10819[44], 10}, {NULL}};
const Segment SGI21985[] = {{&S10819[44], 27}, {NULL}};
const Segment SGI21986[] = {{&S10819[30], 6}, {NULL}};
const Segment SGI21987[] = {{&S10819[119], 3}, {NULL}};
const Segment SGI21988[] = {{&S10819[85], 5}, {NULL}};
const Segment SGI21989[] = {{&S10823[38], 10}, {NULL}};
const Segment SGI21990[] = {{&S10823[31], 6}, {NULL}};
const Segment SGI21991[] = {{&S10824[44], 11}, {NULL}};
const Segment SGI21992[] = {{&S10824[44], 28}, {NULL}};
const Segment SGI21993[] = {{&S10824[30], 6}, {NULL}};
const Segment SGI21994[] = {{&S10824[120], 3}, {NULL}};
const Segment SGI21995[] = {{&S10824[86], 5}, {NULL}};
const Segment SGI21996[] = {{&S10831[44], 10}, {NULL}};
const Segment SGI21997[] = {{&S10831[44], 27}, {NULL}};
const Segment SGI21998[] = {{&S10831[30], 6}, {NULL}};
const Segment SGI21999[] = {{&S10831[88], 4}, {NULL}};
const Segment SGI22000[] = {{&S10831[88], 3}, {NULL}};
const Segment SGI22001[] = {{&S16804[44], 10}, {NULL}};
const Segment SGI22002[] = {{&S16804[44], 27}, {NULL}};
const Segment SGI22003[] = {{&S10833[44], 11}, {NULL}};
const Segment SGI22004[] = {{&S16804[30], 6}, {NULL}};
const Segment SGI22005[] = {{&S10833[44], 28}, {NULL}};
const Segment SGI22006[] = {{&S10833[30], 6}, {NULL}};
const Segment SGI22007[] = {{&S16804[119], 3}, {NULL}};
const Segment SGI22008[] = {{&S10833[120], 3}, {NULL}};
const Segment SGI22009[] = {{&S16804[85], 5}, {NULL}};
const Segment SGI22010[] = {{&S10833[86], 5}, {NULL}};
const Segment SGI22011[] = {{&S10834[44], 10}, {NULL}};
const Segment SGI22012[] = {{&S10834[44], 27}, {NULL}};
const Segment SGI22013[] = {{&S10834[30], 6}, {NULL}};
const Segment SGI22014[] = {{&S10834[119], 3}, {NULL}};
const Segment SGI22015[] = {{&S10834[85], 5}, {NULL}};
const Segment SGI22016[] = {{&S16217[42], 14}, {NULL}};
const Segment SGI22017[] = {{&S16217[42], 25}, {NULL}};
const Segment SGI22018[] = {{&S16217[30], 4}, {NULL}};
const Segment SGI22019[] = {{&S16217[115], 3}, {NULL}};
const Segment SGI22020[] = {{&S16217[81], 5}, {NULL}};
const Segment SGI22021[] = {{&S10837[38], 11}, {NULL}};
const Segment SGI22022[] = {{&S10837[31], 6}, {NULL}};
const Segment SGI22023[] = {{&S16227[42], 9}, {NULL}};
const Segment SGI22024[] = {{&S16227[42], 21}, {NULL}};
const Segment SGI22025[] = {{&S16227[30], 4}, {NULL}};
const Segment SGI22026[] = {{&S16227[111], 3}, {NULL}};
const Segment SGI22027[] = {{&S16227[77], 5}, {NULL}};
const Segment SGI22028[] = {{&S10846[44], 10}, {NULL}};
const Segment SGI22029[] = {{&S10846[44], 27}, {NULL}};
const Segment SGI22030[] = {{&S10846[30], 6}, {NULL}};
const Segment SGI22031[] = {{&S10846[88], 4}, {NULL}};
const Segment SGI22032[] = {{&S10846[88], 3}, {NULL}};
const Segment SGI22033[] = {{&S10847[42], 10}, {NULL}};
const Segment SGI22034[] = {{&S10847[30], 6}, {NULL}};
const Segment SGI22035[] = {{&S10847[117], 3}, {NULL}};
const Segment SGI22036[] = {{&S10847[83], 5}, {NULL}};
const Segment SGI22037[] = {{&S16231[42], 14}, {NULL}};
const Segment SGI22038[] = {{&S16231[42], 25}, {NULL}};
const Segment SGI22039[] = {{&S16231[30], 4}, {NULL}};
const Segment SGI22040[] = {{&S16231[115], 3}, {NULL}};
const Segment SGI22041[] = {{&S16231[81], 5}, {NULL}};
const Segment SGI22042[] = {{&S10848[44], 11}, {NULL}};
const Segment SGI22043[] = {{&S10848[44], 28}, {NULL}};
const Segment SGI22044[] = {{&S10848[30], 6}, {NULL}};
const Segment SGI22045[] = {{&S10848[89], 4}, {NULL}};
const Segment SGI22046[] = {{&S10848[89], 3}, {NULL}};
const Segment SGI22047[] = {{&S10850[38], 11}, {NULL}};
const Segment SGI22048[] = {{&S10850[31], 6}, {NULL}};
const Segment SGI22049[] = {{&S16262[42], 14}, {NULL}};
const Segment SGI22050[] = {{&S16262[42], 25}, {NULL}};
const Segment SGI22051[] = {{&S16262[30], 4}, {NULL}};
const Segment SGI22052[] = {{&S16262[115], 3}, {NULL}};
const Segment SGI22053[] = {{&S16262[81], 5}, {NULL}};
const Segment SGI22054[] = {{&S16292[42], 8}, {NULL}};
const Segment SGI22055[] = {{&S16292[42], 19}, {NULL}};
const Segment SGI22056[] = {{&S16292[30], 4}, {NULL}};
const Segment SGI22057[] = {{&S16292[109], 3}, {NULL}};
const Segment SGI22058[] = {{&S16292[75], 5}, {NULL}};
const Segment SGI22059[] = {{&S16872[44], 11}, {NULL}};
const Segment SGI22060[] = {{&S16872[44], 28}, {NULL}};
const Segment SGI22061[] = {{&S16872[30], 6}, {NULL}};
const Segment SGI22062[] = {{&S16872[120], 3}, {NULL}};
const Segment SGI22063[] = {{&S16872[86], 5}, {NULL}};
const Segment SGI22064[] = {{&S16874[44], 13}, {NULL}};
const Segment SGI22065[] = {{&S16874[44], 30}, {NULL}};
const Segment SGI22066[] = {{&S16874[30], 6}, {NULL}};
const Segment SGI22067[] = {{&S16874[122], 3}, {NULL}};
const Segment SGI22068[] = {{&S16874[88], 5}, {NULL}};
const Segment SGI22069[] = {{&S16881[44], 10}, {NULL}};
const Segment SGI22070[] = {{&S16881[44], 27}, {NULL}};
const Segment SGI22071[] = {{&S16881[30], 6}, {NULL}};
const Segment SGI22072[] = {{&S16881[119], 3}, {NULL}};
const Segment SGI22073[] = {{&S16881[85], 5}, {NULL}};
const Segment SGI22074[] = {{&S16335[42], 11}, {NULL}};
const Segment SGI22075[] = {{&S16335[42], 28}, {NULL}};
const Segment SGI22076[] = {{&S16335[30], 4}, {NULL}};
const Segment SGI22077[] = {{&S16898[44], 5}, {NULL}};
const Segment SGI22078[] = {{&S16898[44], 22}, {NULL}};
const Segment SGI22079[] = {{&S16898[30], 6}, {NULL}};
const Segment SGI22080[] = {{&S16335[118], 3}, {NULL}};
const Segment SGI22081[] = {{&S16898[114], 3}, {NULL}};
const Segment SGI22082[] = {{&S16335[84], 5}, {NULL}};
const Segment SGI22083[] = {{&S16898[80], 5}, {NULL}};
const Segment SGI22084[] = {{&S16340[42], 11}, {NULL}};
const Segment SGI22085[] = {{&S16340[42], 22}, {NULL}};
const Segment SGI22086[] = {{&S16340[30], 4}, {NULL}};
const Segment SGI22087[] = {{&S16340[112], 3}, {NULL}};
const Segment SGI22088[] = {{&S16340[78], 5}, {NULL}};
const Segment SGI22089[] = {{&S16381[42], 14}, {NULL}};
const Segment SGI22090[] = {{&S16381[42], 25}, {NULL}};
const Segment SGI22091[] = {{&S16381[30], 4}, {NULL}};
const Segment SGI22092[] = {{&S16381[115], 3}, {NULL}};
const Segment SGI22093[] = {{&S16381[81], 5}, {NULL}};
const Segment SGI22094[] = {{&S16964[44], 9}, {NULL}};
const Segment SGI22095[] = {{&S16964[44], 26}, {NULL}};
const Segment SGI22096[] = {{&S16964[30], 6}, {NULL}};
const Segment SGI22097[] = {{&S16964[118], 3}, {NULL}};
const Segment SGI22098[] = {{&S16964[84], 5}, {NULL}};
const Segment SGI22099[] = {{&S16398[42], 13}, {NULL}};
const Segment SGI22100[] = {{&S16398[42], 24}, {NULL}};
const Segment SGI22101[] = {{&S16398[30], 4}, {NULL}};
const Segment SGI22102[] = {{&S16398[114], 3}, {NULL}};
const Segment SGI22103[] = {{&S16398[80], 5}, {NULL}};
const Segment SGI22104[] = {{&S16412[44], 4}, {NULL}};
const Segment SGI22105[] = {{&S16412[131], 3}, {NULL}};
const Segment SGI22106[] = {{&S16412[97], 5}, {NULL}};
const Segment SGI22107[] = {{&S16439[44], 8}, {NULL}};
const Segment SGI22108[] = {{&S16439[44], 29}, {NULL}};
const Segment SGI22109[] = {{&S16439[30], 6}, {NULL}};
const Segment SGI22110[] = {{&S16439[121], 3}, {NULL}};
const Segment SGI22111[] = {{&S16439[87], 5}, {NULL}};
const Segment SGI22112[] = {{&S16464[42], 101}, {NULL}};
const Segment SGI22113[] = {{&S16464[42], 112}, {NULL}};
const Segment SGI22114[] = {{&S16464[30], 4}, {NULL}};
const Segment SGI22115[] = {{&S16464[202], 3}, {NULL}};
const Segment SGI22116[] = {{&S16464[168], 5}, {NULL}};
const Segment SGI22117[] = {{&S16465[44], 6}, {NULL}};
const Segment SGI22118[] = {{&S16465[44], 17}, {NULL}};
const Segment SGI22119[] = {{&S16465[30], 6}, {NULL}};
const Segment SGI22120[] = {{&S16465[109], 3}, {NULL}};
const Segment SGI22121[] = {{&S16465[75], 5}, {NULL}};
const Segment SGI22122[] = {{&S17055[44], 11}, {NULL}};
const Segment SGI22123[] = {{&S17055[44], 28}, {NULL}};
const Segment SGI22124[] = {{&S17055[30], 6}, {NULL}};
const Segment SGI22125[] = {{&S16482[42], 14}, {NULL}};
const Segment SGI22126[] = {{&S16482[42], 25}, {NULL}};
const Segment SGI22127[] = {{&S17055[120], 3}, {NULL}};
const Segment SGI22128[] = {{&S16482[30], 4}, {NULL}};
const Segment SGI22129[] = {{&S17055[86], 5}, {NULL}};
const Segment SGI22130[] = {{&S16482[115], 3}, {NULL}};
const Segment SGI22131[] = {{&S16482[81], 5}, {NULL}};
const Segment SGI22132[] = {{&S16495[44], 8}, {NULL}};
const Segment SGI22133[] = {{&S16495[44], 29}, {NULL}};
const Segment SGI22134[] = {{&S16495[30], 6}, {NULL}};
const Segment SGI22135[] = {{&S16495[121], 3}, {NULL}};
const Segment SGI22136[] = {{&S16495[87], 5}, {NULL}};
const Segment SGI22137[] = {{&S16499[42], 16}, {NULL}};
const Segment SGI22138[] = {{&S16499[42], 27}, {NULL}};
const Segment SGI22139[] = {{&S16499[30], 4}, {NULL}};
const Segment SGI22140[] = {{&S16499[117], 3}, {NULL}};
const Segment SGI22141[] = {{&S16499[83], 5}, {NULL}};
const Segment SGI22142[] = {{&S16501[42], 17}, {NULL}};
const Segment SGI22143[] = {{&S16501[42], 29}, {NULL}};
const Segment SGI22144[] = {{&S16501[30], 4}, {NULL}};
const Segment SGI22145[] = {{&S16501[119], 3}, {NULL}};
const Segment SGI22146[] = {{&S16501[85], 5}, {NULL}};
const Segment SGI22147[] = {{&S17090[44], 10}, {NULL}};
const Segment SGI22148[] = {{&S17090[44], 27}, {NULL}};
const Segment SGI22149[] = {{&S17090[30], 6}, {NULL}};
const Segment SGI22150[] = {{&S17090[119], 3}, {NULL}};
const Segment SGI22151[] = {{&S17090[85], 5}, {NULL}};
const Segment SGI22152[] = {{&S16541[62], 28}, {NULL}};
const Segment SGI22153[] = {{&S16541[62], 39}, {NULL}};
const Segment SGI22154[] = {{&S16541[30], 24}, {&S16541[70], 31}, {NULL}};
const Segment SGI22155[] = {{&S16541[149], 3}, {NULL}};
const Segment SGI22156[] = {{&S16541[115], 5}, {NULL}};
const Segment SGI22157[] = {{&S16578[42], 14}, {NULL}};
const Segment SGI22158[] = {{&S16578[42], 25}, {NULL}};
const Segment SGI22159[] = {{&S16578[30], 4}, {NULL}};
const Segment SGI22160[] = {{&S16578[115], 3}, {NULL}};
const Segment SGI22161[] = {{&S16578[81], 5}, {NULL}};
const Segment SGI22162[] = {{&S16619[30], 4}, {NULL}};
const Segment SGI22163[] = {{&S16619[56], 4}, {NULL}};
const Segment SGI22164[] = {{&S16622[42], 5}, {NULL}};
const Segment SGI22165[] = {{&S16622[42], 22}, {NULL}};
const Segment SGI22166[] = {{&S16622[30], 4}, {NULL}};
const Segment SGI22167[] = {{&S16622[112], 3}, {NULL}};
const Segment SGI22168[] = {{&S16622[78], 5}, {NULL}};
const Segment SGI22169[] = {{&S16701[62], 28}, {NULL}};
const Segment SGI22170[] = {{&S16701[62], 39}, {NULL}};
const Segment SGI22171[] = {{&S16701[30], 24}, {&S16701[70], 31}, {NULL}};
const Segment SGI22172[] = {{&S16701[149], 3}, {NULL}};
const Segment SGI22173[] = {{&S16701[115], 5}, {NULL}};
const Segment SGI22174[] = {{&S17235[44], 6}, {NULL}};
const Segment SGI22175[] = {{&S17235[44], 23}, {NULL}};
const Segment SGI22176[] = {{&S17235[30], 6}, {NULL}};
const Segment SGI22177[] = {{&S17235[115], 3}, {NULL}};
const Segment SGI22178[] = {{&S17235[81], 5}, {NULL}};
const Segment SGI22179[] = {{&S16789[47], 4}, {NULL}};
const Segment SGI22180[] = {{&S16789[101], 6}, {NULL}};
const Segment SGI22181[] = {{&S16845[42], 14}, {NULL}};
const Segment SGI22182[] = {{&S16845[42], 25}, {NULL}};
const Segment SGI22183[] = {{&S16845[30], 4}, {NULL}};
const Segment SGI22184[] = {{&S16845[115], 3}, {NULL}};
const Segment SGI22185[] = {{&S16845[81], 5}, {NULL}};
const Segment SGI22186[] = {{&S16860[44], 10}, {NULL}};
const Segment SGI22187[] = {{&S16860[44], 22}, {NULL}};
const Segment SGI22188[] = {{&S16860[30], 6}, {NULL}};
const Segment SGI22189[] = {{&S16860[114], 3}, {NULL}};
const Segment SGI22190[] = {{&S16860[80], 5}, {NULL}};
const Segment SGI22191[] = {{&S16878[41], 12}, {NULL}};
const Segment SGI22192[] = {{&S16878[41], 23}, {NULL}};
const Segment SGI22193[] = {{&S16878[30], 4}, {NULL}};
const Segment SGI22194[] = {{&S16878[112], 3}, {NULL}};
const Segment SGI22195[] = {{&S16878[78], 5}, {NULL}};
const Segment SGI22196[] = {{&S16885[56], 14}, {NULL}};
const Segment SGI22197[] = {{&S16885[56], 33}, {NULL}};
const Segment SGI22198[] = {{&S16885[30], 18}, {NULL}};
const Segment SGI22199[] = {{&S16885[137], 3}, {NULL}};
const Segment SGI22200[] = {{&S16885[103], 5}, {NULL}};
const Segment SGI22201[] = {{&S17297[44], 12}, {NULL}};
const Segment SGI22202[] = {{&S17297[44], 29}, {NULL}};
const Segment SGI22203[] = {{&S17297[30], 6}, {NULL}};
const Segment SGI22204[] = {{&S17297[121], 3}, {NULL}};
const Segment SGI22205[] = {{&S17297[87], 5}, {NULL}};
const Segment SGI22206[] = {{&S16922[44], 10}, {NULL}};
const Segment SGI22207[] = {{&S16922[44], 27}, {NULL}};
const Segment SGI22208[] = {{&S16922[30], 6}, {NULL}};
const Segment SGI22209[] = {{&S16922[119], 3}, {NULL}};
const Segment SGI22210[] = {{&S16922[85], 5}, {NULL}};
const Segment SGI22211[] = {{&S16936[54], 14}, {NULL}};
const Segment SGI22212[] = {{&S16936[54], 33}, {NULL}};
const Segment SGI22213[] = {{&S16936[158], 3}, {NULL}};
const Segment SGI22214[] = {{&S16936[124], 5}, {NULL}};
const Segment SGI22215[] = {{&S16988[44], 86}, {NULL}};
const Segment SGI22216[] = {{&S16988[44], 107}, {NULL}};
const Segment SGI22217[] = {{&S16988[30], 6}, {NULL}};
const Segment SGI22218[] = {{&S16988[198], 3}, {NULL}};
const Segment SGI22219[] = {{&S16988[124], 5}, {NULL}};
const Segment SGI22220[] = {{&S14918[44], 10}, {NULL}};
const Segment SGI22221[] = {{&S14918[44], 27}, {NULL}};
const Segment SGI22222[] = {{&S14918[30], 6}, {NULL}};
const Segment SGI22223[] = {{&S14918[119], 3}, {NULL}};
const Segment SGI22224[] = {{&S14918[85], 5}, {NULL}};
const Segment SGI22225[] = {{&S17037[42], 16}, {NULL}};
const Segment SGI22226[] = {{&S17037[42], 27}, {NULL}};
const Segment SGI22227[] = {{&S17037[30], 4}, {NULL}};
const Segment SGI22228[] = {{&S17037[117], 3}, {NULL}};
const Segment SGI22229[] = {{&S17037[83], 5}, {NULL}};
const Segment SGI22230[] = {{&S17082[42], 14}, {NULL}};
const Segment SGI22231[] = {{&S17082[42], 25}, {NULL}};
const Segment SGI22232[] = {{&S17082[30], 4}, {NULL}};
const Segment SGI22233[] = {{&S17082[115], 3}, {NULL}};
const Segment SGI22234[] = {{&S17082[81], 5}, {NULL}};
const Segment SGI22235[] = {{&S15022[44], 19}, {NULL}};
const Segment SGI22236[] = {{&S15022[44], 31}, {NULL}};
const Segment SGI22237[] = {{&S15022[30], 6}, {NULL}};
const Segment SGI22238[] = {{&S15022[123], 3}, {NULL}};
const Segment SGI22239[] = {{&S15022[89], 5}, {NULL}};
const Segment SGI22240[] = {{&S13008[44], 10}, {NULL}};
const Segment SGI22241[] = {{&S13008[44], 27}, {NULL}};
const Segment SGI22242[] = {{&S13008[30], 6}, {NULL}};
const Segment SGI22243[] = {{&S13008[119], 3}, {NULL}};
const Segment SGI22244[] = {{&S13008[85], 5}, {NULL}};
const Segment SGI22245[] = {{&S13034[38], 10}, {NULL}};
const Segment SGI22246[] = {{&S13034[31], 6}, {NULL}};
const Segment SGI22247[] = {{&S17193[44], 86}, {NULL}};
const Segment SGI22248[] = {{&S17193[44], 107}, {NULL}};
const Segment SGI22249[] = {{&S17193[30], 6}, {NULL}};
const Segment SGI22250[] = {{&S17193[198], 3}, {NULL}};
const Segment SGI22251[] = {{&S17193[124], 5}, {NULL}};
const Segment SGI22252[] = {{&S17248[44], 9}, {NULL}};
const Segment SGI22253[] = {{&S17248[44], 21}, {NULL}};
const Segment SGI22254[] = {{&S17248[30], 6}, {NULL}};
const Segment SGI22255[] = {{&S17248[113], 3}, {NULL}};
const Segment SGI22256[] = {{&S17248[79], 5}, {NULL}};
const Segment SGI22257[] = {{&S17271[44], 9}, {NULL}};
const Segment SGI22258[] = {{&S17271[44], 21}, {NULL}};
const Segment SGI22259[] = {{&S17271[30], 6}, {NULL}};
const Segment SGI22260[] = {{&S17271[113], 3}, {NULL}};
const Segment SGI22261[] = {{&S17271[79], 5}, {NULL}};
const Segment SGI22262[] = {{&S17283[30], 4}, {NULL}};
const Segment SGI22263[] = {{&S17283[97], 3}, {NULL}};
const Segment SGI22264[] = {{&S17283[62], 6}, {NULL}};
const Segment SGI22265[] = {{&S17283[117], 8}, {NULL}};
const Segment SGI22266[] = {{&S13007[44], 10}, {NULL}};
const Segment SGI22267[] = {{&S13007[44], 27}, {NULL}};
const Segment SGI22268[] = {{&S13007[30], 6}, {NULL}};
const Segment SGI22269[] = {{&S13007[119], 3}, {NULL}};
const Segment SGI22270[] = {{&S13007[85], 5}, {NULL}};
const Segment SGI22271[] = {{&S13009[44], 11}, {NULL}};
const Segment SGI22272[] = {{&S13009[44], 28}, {NULL}};
const Segment SGI22273[] = {{&S13009[30], 6}, {NULL}};
const Segment SGI22274[] = {{&S13009[120], 3}, {NULL}};
const Segment SGI22275[] = {{&S13009[86], 5}, {NULL}};
const Segment SGI22276[] = {{&S13014[38], 10}, {NULL}};
const Segment SGI22277[] = {{&S13014[31], 6}, {NULL}};
const Segment SGI22278[] = {{&S13017[44], 11}, {NULL}};
const Segment SGI22279[] = {{&S13017[44], 28}, {NULL}};
const Segment SGI22280[] = {{&S13017[30], 6}, {NULL}};
const Segment SGI22281[] = {{&S13017[120], 3}, {NULL}};
const Segment SGI22282[] = {{&S13017[86], 5}, {NULL}};
const Segment SGI22283[] = {{&S13018[44], 28}, {NULL}};
const Segment SGI22284[] = {{&S13018[44], 45}, {NULL}};
const Segment SGI22285[] = {{&S13018[30], 6}, {NULL}};
const Segment SGI22286[] = {{&S13018[137], 3}, {NULL}};
const Segment SGI22287[] = {{&S13018[103], 5}, {NULL}};
const Segment SGI22288[] = {{&S13028[44], 10}, {NULL}};
const Segment SGI22289[] = {{&S13028[44], 27}, {NULL}};
const Segment SGI22290[] = {{&S13028[30], 6}, {NULL}};
const Segment SGI22291[] = {{&S13028[88], 4}, {NULL}};
const Segment SGI22292[] = {{&S13028[88], 3}, {NULL}};
const Segment SGI22293[] = {{&S13038[44], 10}, {NULL}};
const Segment SGI22294[] = {{&S13038[44], 21}, {NULL}};
const Segment SGI22295[] = {{&S13038[30], 6}, {NULL}};
const Segment SGI22296[] = {{&S13038[113], 3}, {NULL}};
const Segment SGI22297[] = {{&S13038[79], 5}, {NULL}};
const Segment SGI22298[] = {{&S13039[38], 11}, {NULL}};
const Segment SGI22299[] = {{&S13039[31], 6}, {NULL}};
const Segment SGI22300[] = {{&S13042[44], 10}, {NULL}};
const Segment SGI22301[] = {{&S13042[44], 27}, {NULL}};
const Segment SGI22302[] = {{&S13042[30], 6}, {NULL}};
const Segment SGI22303[] = {{&S13042[88], 6}, {NULL}};
const Segment SGI22304[] = {{&S13044[42], 10}, {NULL}};
const Segment SGI22305[] = {{&S13044[30], 6}, {NULL}};
const Segment SGI22306[] = {{&S13044[117], 3}, {NULL}};
const Segment SGI22307[] = {{&S13044[83], 5}, {NULL}};
const Segment SGI22308[] = {{&S16141[23], 15}, {NULL}};
const Segment SGI22309[] = {{&S16141[31], 24}, {NULL}};
const Segment SGI22310[] = {{&S13010[44], 11}, {NULL}};
const Segment SGI22311[] = {{&S13010[44], 28}, {NULL}};
const Segment SGI22312[] = {{&S13010[30], 6}, {NULL}};
const Segment SGI22313[] = {{&S13010[120], 3}, {NULL}};
const Segment SGI22314[] = {{&S13010[86], 5}, {NULL}};
const Segment SGI22315[] = {{&S13015[44], 11}, {NULL}};
const Segment SGI22316[] = {{&S13015[44], 28}, {NULL}};
const Segment SGI22317[] = {{&S13015[30], 6}, {NULL}};
const Segment SGI22318[] = {{&S13015[120], 3}, {NULL}};
const Segment SGI22319[] = {{&S13015[86], 5}, {NULL}};
const Segment SGI22320[] = {{&S13020[44], 29}, {NULL}};
const Segment SGI22321[] = {{&S13020[44], 46}, {NULL}};
const Segment SGI22322[] = {{&S13020[30], 6}, {NULL}};
const Segment SGI22323[] = {{&S13020[138], 3}, {NULL}};
const Segment SGI22324[] = {{&S13020[104], 5}, {NULL}};
const Segment SGI22325[] = {{&S13021[38], 10}, {NULL}};
const Segment SGI22326[] = {{&S13021[31], 6}, {NULL}};
const Segment SGI22327[] = {{&S13024[44], 10}, {NULL}};
const Segment SGI22328[] = {{&S13024[44], 27}, {NULL}};
const Segment SGI22329[] = {{&S13024[30], 6}, {NULL}};
const Segment SGI22330[] = {{&S13024[119], 3}, {NULL}};
const Segment SGI22331[] = {{&S13024[85], 5}, {NULL}};
const Segment SGI22332[] = {{&S13029[44], 11}, {NULL}};
const Segment SGI22333[] = {{&S13029[44], 28}, {NULL}};
const Segment SGI22334[] = {{&S13029[30], 6}, {NULL}};
const Segment SGI22335[] = {{&S13029[120], 3}, {NULL}};
const Segment SGI22336[] = {{&S13029[86], 5}, {NULL}};
const Segment SGI22337[] = {{&S13030[44], 29}, {NULL}};
const Segment SGI22338[] = {{&S13030[44], 46}, {NULL}};
const Segment SGI22339[] = {{&S13030[30], 6}, {NULL}};
const Segment SGI22340[] = {{&S13030[138], 3}, {NULL}};
const Segment SGI22341[] = {{&S13030[104], 5}, {NULL}};
const Segment SGI22342[] = {{&S13032[44], 10}, {NULL}};
const Segment SGI22343[] = {{&S13032[44], 21}, {NULL}};
const Segment SGI22344[] = {{&S13032[30], 6}, {NULL}};
const Segment SGI22345[] = {{&S13032[113], 3}, {NULL}};
const Segment SGI22346[] = {{&S13032[79], 5}, {NULL}};
const Segment SGI22347[] = {{&S13035[38], 11}, {NULL}};
const Segment SGI22348[] = {{&S13035[31], 6}, {NULL}};
const Segment SGI22349[] = {{&S13050[42], 10}, {NULL}};
const Segment SGI22350[] = {{&S13050[30], 6}, {NULL}};
const Segment SGI22351[] = {{&S13050[117], 3}, {NULL}};
const Segment SGI22352[] = {{&S13050[83], 5}, {NULL}};
const Segment SGI22353[] = {{&S19002[44], 19}, {NULL}};
const Segment SGI22354[] = {{&S19002[44], 35}, {NULL}};
const Segment SGI22355[] = {{&S19002[30], 6}, {NULL}};
const Segment SGI22356[] = {{&S19002[127], 3}, {NULL}};
const Segment SGI22357[] = {{&S19002[93], 5}, {NULL}};
const Segment SGI22358[] = {{&S19004[44], 19}, {NULL}};
const Segment SGI22359[] = {{&S19004[44], 36}, {NULL}};
const Segment SGI22360[] = {{&S19004[30], 6}, {NULL}};
const Segment SGI22361[] = {{&S19004[128], 3}, {NULL}};
const Segment SGI22362[] = {{&S19004[94], 5}, {NULL}};
const Segment SGI22363[] = {{&S14529[44], 29}, {NULL}};
const Segment SGI22364[] = {{&S14529[44], 46}, {NULL}};
const Segment SGI22365[] = {{&S14529[30], 6}, {NULL}};
const Segment SGI22366[] = {{&S14529[138], 3}, {NULL}};
const Segment SGI22367[] = {{&S14529[104], 5}, {NULL}};
const Segment SGI22368[] = {{&S14532[44], 10}, {NULL}};
const Segment SGI22369[] = {{&S14532[44], 27}, {NULL}};
const Segment SGI22370[] = {{&S14532[30], 6}, {NULL}};
const Segment SGI22371[] = {{&S14532[119], 3}, {NULL}};
const Segment SGI22372[] = {{&S14532[85], 5}, {NULL}};
const Segment SGI22373[] = {{&S14536[44], 29}, {NULL}};
const Segment SGI22374[] = {{&S14536[44], 46}, {NULL}};
const Segment SGI22375[] = {{&S14536[30], 6}, {NULL}};
const Segment SGI22376[] = {{&S14536[138], 3}, {NULL}};
const Segment SGI22377[] = {{&S14536[104], 5}, {NULL}};
const Segment SGI22378[] = {{&S14537[38], 10}, {NULL}};
const Segment SGI22379[] = {{&S14537[31], 6}, {NULL}};
const Segment SGI22380[] = {{&S19050[44], 20}, {NULL}};
const Segment SGI22381[] = {{&S19050[44], 37}, {NULL}};
const Segment SGI22382[] = {{&S19050[30], 6}, {NULL}};
const Segment SGI22383[] = {{&S19050[129], 3}, {NULL}};
const Segment SGI22384[] = {{&S19050[95], 5}, {NULL}};
const Segment SGI22385[] = {{&S14546[44], 10}, {NULL}};
const Segment SGI22386[] = {{&S14546[44], 27}, {NULL}};
const Segment SGI22387[] = {{&S14546[30], 6}, {NULL}};
const Segment SGI22388[] = {{&S14546[88], 4}, {NULL}};
const Segment SGI22389[] = {{&S14546[88], 3}, {NULL}};
const Segment SGI22390[] = {{&S14562[44], 10}, {NULL}};
const Segment SGI22391[] = {{&S14562[44], 27}, {NULL}};
const Segment SGI22392[] = {{&S14562[30], 6}, {NULL}};
const Segment SGI22393[] = {{&S14562[88], 6}, {NULL}};
const Segment SGI22394[] = {{&S14567[42], 10}, {NULL}};
const Segment SGI22395[] = {{&S14567[30], 6}, {NULL}};
const Segment SGI22396[] = {{&S14567[117], 3}, {NULL}};
const Segment SGI22397[] = {{&S14567[83], 5}, {NULL}};
const Segment SGI22398[] = {{&S14904[44], 10}, {NULL}};
const Segment SGI22399[] = {{&S14904[44], 27}, {NULL}};
const Segment SGI22400[] = {{&S14904[30], 6}, {NULL}};
const Segment SGI22401[] = {{&S14904[119], 3}, {NULL}};
const Segment SGI22402[] = {{&S14904[85], 5}, {NULL}};
const Segment SGI22403[] = {{&S14905[42], 10}, {NULL}};
const Segment SGI22404[] = {{&S14905[30], 6}, {NULL}};
const Segment SGI22405[] = {{&S14905[117], 3}, {NULL}};
const Segment SGI22406[] = {{&S14905[83], 5}, {NULL}};
const Segment SGI22407[] = {{&S14907[44], 11}, {NULL}};
const Segment SGI22408[] = {{&S14907[44], 28}, {NULL}};
const Segment SGI22409[] = {{&S14907[30], 6}, {NULL}};
const Segment SGI22410[] = {{&S14907[120], 3}, {NULL}};
const Segment SGI22411[] = {{&S14907[86], 5}, {NULL}};
const Segment SGI22412[] = {{&S14911[44], 28}, {NULL}};
const Segment SGI22413[] = {{&S14911[44], 45}, {NULL}};
const Segment SGI22414[] = {{&S14911[30], 6}, {NULL}};
const Segment SGI22415[] = {{&S14911[137], 3}, {NULL}};
const Segment SGI22416[] = {{&S14911[103], 5}, {NULL}};
const Segment SGI22417[] = {{&S14914[44], 28}, {NULL}};
const Segment SGI22418[] = {{&S14914[44], 45}, {NULL}};
const Segment SGI22419[] = {{&S14914[30], 6}, {NULL}};
const Segment SGI22420[] = {{&S14914[137], 3}, {NULL}};
const Segment SGI22421[] = {{&S14914[103], 5}, {NULL}};
const Segment SGI22422[] = {{&S14915[44], 11}, {NULL}};
const Segment SGI22423[] = {{&S14915[44], 28}, {NULL}};
const Segment SGI22424[] = {{&S14915[30], 6}, {NULL}};
const Segment SGI22425[] = {{&S14915[120], 3}, {NULL}};
const Segment SGI22426[] = {{&S14915[86], 5}, {NULL}};
const Segment SGI22427[] = {{&S14917[44], 10}, {NULL}};
const Segment SGI22428[] = {{&S14917[44], 27}, {NULL}};
const Segment SGI22429[] = {{&S14917[30], 6}, {NULL}};
const Segment SGI22430[] = {{&S14917[120], 3}, {NULL}};
const Segment SGI22431[] = {{&S14917[85], 6}, {NULL}};
const Segment SGI22432[] = {{&S14919[38], 10}, {NULL}};
const Segment SGI22433[] = {{&S14919[31], 6}, {NULL}};
const Segment SGI22434[] = {{&S14929[38], 11}, {NULL}};
const Segment SGI22435[] = {{&S14929[31], 6}, {NULL}};
const Segment SGI22436[] = {{&S14934[38], 11}, {NULL}};
const Segment SGI22437[] = {{&S14934[31], 6}, {NULL}};
const Segment SGI22438[] = {{&S14939[43], 9}, {NULL}};
const Segment SGI22439[] = {{&S14939[43], 26}, {NULL}};
const Segment SGI22440[] = {{&S14939[30], 6}, {NULL}};
const Segment SGI22441[] = {{&S14939[119], 3}, {NULL}};
const Segment SGI22442[] = {{&S14939[83], 7}, {NULL}};
const Segment SGI22443[] = {{&S14939[139], 8}, {NULL}};
const Segment SGI22444[] = {{&S14530[44], 10}, {NULL}};
const Segment SGI22445[] = {{&S14530[44], 27}, {NULL}};
const Segment SGI22446[] = {{&S14530[30], 6}, {NULL}};
const Segment SGI22447[] = {{&S14530[119], 3}, {NULL}};
const Segment SGI22448[] = {{&S14530[85], 5}, {NULL}};
const Segment SGI22449[] = {{&S14531[44], 28}, {NULL}};
const Segment SGI22450[] = {{&S14531[44], 45}, {NULL}};
const Segment SGI22451[] = {{&S14531[30], 6}, {NULL}};
const Segment SGI22452[] = {{&S14531[137], 3}, {NULL}};
const Segment SGI22453[] = {{&S14531[103], 5}, {NULL}};
const Segment SGI22454[] = {{&S14943[44], 10}, {NULL}};
const Segment SGI22455[] = {{&S14943[44], 27}, {NULL}};
const Segment SGI22456[] = {{&S14943[30], 6}, {NULL}};
const Segment SGI22457[] = {{&S14943[88], 4}, {NULL}};
const Segment SGI22458[] = {{&S14943[88], 3}, {NULL}};
const Segment SGI22459[] = {{&S14533[44], 28}, {NULL}};
const Segment SGI22460[] = {{&S14533[44], 45}, {NULL}};
const Segment SGI22461[] = {{&S14533[30], 6}, {NULL}};
const Segment SGI22462[] = {{&S14533[137], 3}, {NULL}};
const Segment SGI22463[] = {{&S14533[103], 5}, {NULL}};
const Segment SGI22464[] = {{&S14534[44], 28}, {NULL}};
const Segment SGI22465[] = {{&S14534[44], 45}, {NULL}};
const Segment SGI22466[] = {{&S14534[30], 6}, {NULL}};
const Segment SGI22467[] = {{&S14534[137], 3}, {NULL}};
const Segment SGI22468[] = {{&S14534[103], 5}, {NULL}};
const Segment SGI22469[] = {{&S14951[44], 10}, {NULL}};
const Segment SGI22470[] = {{&S14951[44], 27}, {NULL}};
const Segment SGI22471[] = {{&S14951[30], 6}, {NULL}};
const Segment SGI22472[] = {{&S14951[88], 6}, {NULL}};
const Segment SGI22473[] = {{&S14542[38], 10}, {NULL}};
const Segment SGI22474[] = {{&S14542[31], 6}, {NULL}};
const Segment SGI22475[] = {{&S14961[42], 11}, {NULL}};
const Segment SGI22476[] = {{&S14961[30], 6}, {NULL}};
const Segment SGI22477[] = {{&S14961[118], 3}, {NULL}};
const Segment SGI22478[] = {{&S14961[84], 5}, {NULL}};
const Segment SGI22479[] = {{&S14558[44], 10}, {NULL}};
const Segment SGI22480[] = {{&S14558[44], 27}, {NULL}};
const Segment SGI22481[] = {{&S14558[30], 6}, {NULL}};
const Segment SGI22482[] = {{&S14558[88], 4}, {NULL}};
const Segment SGI22483[] = {{&S14558[88], 3}, {NULL}};
const Segment SGI22484[] = {{&S14568[42], 10}, {NULL}};
const Segment SGI22485[] = {{&S14568[30], 6}, {NULL}};
const Segment SGI22486[] = {{&S14568[117], 3}, {NULL}};
const Segment SGI22487[] = {{&S14568[83], 5}, {NULL}};
const Segment SGI22488[] = {{&S15000[44], 24}, {NULL}};
const Segment SGI22489[] = {{&S15000[44], 35}, {NULL}};
const Segment SGI22490[] = {{&S15000[30], 6}, {NULL}};
const Segment SGI22491[] = {{&S15000[127], 3}, {NULL}};
const Segment SGI22492[] = {{&S15000[93], 5}, {NULL}};
const Segment SGI22493[] = {{&S15001[44], 24}, {NULL}};
const Segment SGI22494[] = {{&S15001[44], 35}, {NULL}};
const Segment SGI22495[] = {{&S15001[30], 6}, {NULL}};
const Segment SGI22496[] = {{&S15001[127], 3}, {NULL}};
const Segment SGI22497[] = {{&S15001[93], 5}, {NULL}};
const Segment SGI22498[] = {{&S15002[44], 23}, {NULL}};
const Segment SGI22499[] = {{&S15002[44], 34}, {NULL}};
const Segment SGI22500[] = {{&S15002[30], 6}, {NULL}};
const Segment SGI22501[] = {{&S15002[126], 3}, {NULL}};
const Segment SGI22502[] = {{&S15002[92], 5}, {NULL}};
const Segment SGI22503[] = {{&S15003[44], 14}, {NULL}};
const Segment SGI22504[] = {{&S15003[44], 25}, {NULL}};
const Segment SGI22505[] = {{&S15003[30], 6}, {NULL}};
const Segment SGI22506[] = {{&S15003[117], 3}, {NULL}};
const Segment SGI22507[] = {{&S15003[83], 5}, {NULL}};
const Segment SGI22508[] = {{&S15004[44], 13}, {NULL}};
const Segment SGI22509[] = {{&S15004[44], 24}, {NULL}};
const Segment SGI22510[] = {{&S15004[30], 6}, {NULL}};
const Segment SGI22511[] = {{&S15004[116], 3}, {NULL}};
const Segment SGI22512[] = {{&S15004[82], 5}, {NULL}};
const Segment SGI22513[] = {{&S15005[44], 19}, {NULL}};
const Segment SGI22514[] = {{&S15005[44], 30}, {NULL}};
const Segment SGI22515[] = {{&S15005[30], 6}, {NULL}};
const Segment SGI22516[] = {{&S15005[122], 3}, {NULL}};
const Segment SGI22517[] = {{&S15005[88], 5}, {NULL}};
const Segment SGI22518[] = {{&S15006[36], 1}, {NULL}};
const Segment SGI22519[] = {{&S15006[60], 12}, {NULL}};
const Segment SGI22520[] = {{&S15006[47], 6}, {NULL}};
const Segment SGI22521[] = {{&S15006[120], 3}, {NULL}};
const Segment SGI22522[] = {{&S15006[86], 5}, {NULL}};
const Segment SGI22523[] = {{&S15007[44], 24}, {NULL}};
const Segment SGI22524[] = {{&S15007[44], 35}, {NULL}};
const Segment SGI22525[] = {{&S15007[30], 6}, {NULL}};
const Segment SGI22526[] = {{&S15007[127], 3}, {NULL}};
const Segment SGI22527[] = {{&S15007[93], 5}, {NULL}};
const Segment SGI22528[] = {{&S15009[42], 19}, {NULL}};
const Segment SGI22529[] = {{&S15009[30], 6}, {NULL}};
const Segment SGI22530[] = {{&S15009[120], 3}, {NULL}};
const Segment SGI22531[] = {{&S15009[86], 5}, {NULL}};
const Segment SGI22532[] = {{&S15011[44], 24}, {NULL}};
const Segment SGI22533[] = {{&S15011[44], 35}, {NULL}};
const Segment SGI22534[] = {{&S15011[30], 6}, {NULL}};
const Segment SGI22535[] = {{&S15011[127], 3}, {NULL}};
const Segment SGI22536[] = {{&S15011[93], 5}, {NULL}};
const Segment SGI22537[] = {{&S15012[44], 24}, {NULL}};
const Segment SGI22538[] = {{&S15012[44], 35}, {NULL}};
const Segment SGI22539[] = {{&S15012[30], 6}, {NULL}};
const Segment SGI22540[] = {{&S15012[127], 3}, {NULL}};
const Segment SGI22541[] = {{&S15012[93], 5}, {NULL}};
const Segment SGI22542[] = {{&S15013[44], 24}, {NULL}};
const Segment SGI22543[] = {{&S15013[44], 35}, {NULL}};
const Segment SGI22544[] = {{&S15013[30], 6}, {NULL}};
const Segment SGI22545[] = {{&S15013[127], 3}, {NULL}};
const Segment SGI22546[] = {{&S15013[93], 5}, {NULL}};
const Segment SGI22547[] = {{&S15014[44], 24}, {NULL}};
const Segment SGI22548[] = {{&S15014[44], 35}, {NULL}};
const Segment SGI22549[] = {{&S15014[30], 6}, {NULL}};
const Segment SGI22550[] = {{&S15014[127], 3}, {NULL}};
const Segment SGI22551[] = {{&S15014[93], 5}, {NULL}};
const Segment SGI22552[] = {{&S15015[44], 19}, {NULL}};
const Segment SGI22553[] = {{&S15015[44], 30}, {NULL}};
const Segment SGI22554[] = {{&S15015[30], 6}, {NULL}};
const Segment SGI22555[] = {{&S15015[122], 3}, {NULL}};
const Segment SGI22556[] = {{&S15015[88], 5}, {NULL}};
const Segment SGI22557[] = {{&S15023[44], 24}, {NULL}};
const Segment SGI22558[] = {{&S15023[44], 35}, {NULL}};
const Segment SGI22559[] = {{&S15023[30], 6}, {NULL}};
const Segment SGI22560[] = {{&S15023[127], 3}, {NULL}};
const Segment SGI22561[] = {{&S15023[93], 5}, {NULL}};
const Segment SGI22562[] = {{&S15024[38], 14}, {NULL}};
const Segment SGI22563[] = {{&S15024[31], 6}, {NULL}};
const Segment SGI22564[] = {{&S15027[44], 14}, {NULL}};
const Segment SGI22565[] = {{&S15027[44], 31}, {NULL}};
const Segment SGI22566[] = {{&S15027[30], 6}, {NULL}};
const Segment SGI22567[] = {{&S15027[124], 3}, {NULL}};
const Segment SGI22568[] = {{&S15027[89], 6}, {NULL}};
const Segment SGI22569[] = {{&S15036[44], 14}, {NULL}};
const Segment SGI22570[] = {{&S15036[44], 25}, {NULL}};
const Segment SGI22571[] = {{&S15036[30], 6}, {NULL}};
const Segment SGI22572[] = {{&S15036[86], 4}, {NULL}};
const Segment SGI22573[] = {{&S15036[86], 3}, {NULL}};
const Segment SGI22574[] = {{&S15041[44], 14}, {NULL}};
const Segment SGI22575[] = {{&S15041[44], 25}, {NULL}};
const Segment SGI22576[] = {{&S15041[30], 6}, {NULL}};
const Segment SGI22577[] = {{&S15041[86], 6}, {NULL}};
const Segment SGI22578[] = {{&S15047[44], 101}, {NULL}};
const Segment SGI22579[] = {{&S15047[44], 112}, {NULL}};
const Segment SGI22580[] = {{&S15047[30], 6}, {NULL}};
const Segment SGI22581[] = {{&S15047[204], 3}, {NULL}};
const Segment SGI22582[] = {{&S15047[170], 5}, {NULL}};
const Segment SGI22583[] = {{&S15048[44], 19}, {NULL}};
const Segment SGI22584[] = {{&S15048[44], 30}, {NULL}};
const Segment SGI22585[] = {{&S15048[30], 6}, {NULL}};
const Segment SGI22586[] = {{&S15048[122], 3}, {NULL}};
const Segment SGI22587[] = {{&S15048[88], 5}, {NULL}};
const Segment SGI22588[] = {{&S15050[44], 25}, {NULL}};
const Segment SGI22589[] = {{&S15050[30], 6}, {NULL}};
const Segment SGI22590[] = {{&S24849[42], 9}, {NULL}};
const Segment SGI22591[] = {{&S24849[30], 6}, {NULL}};
const Segment SGI22592[] = {{&S24849[110], 3}, {NULL}};
const Segment SGI22593[] = {{&S24849[76], 5}, {NULL}};
const Segment SGI22594[] = {{&S24850[44], 14}, {NULL}};
const Segment SGI22595[] = {{&S24850[44], 25}, {NULL}};
const Segment SGI22596[] = {{&S24850[30], 6}, {NULL}};
const Segment SGI22597[] = {{&S24850[117], 3}, {NULL}};
const Segment SGI22598[] = {{&S24850[83], 5}, {NULL}};
const Segment SGI22599[] = {{&S24856[44], 9}, {NULL}};
const Segment SGI22600[] = {{&S24856[44], 20}, {NULL}};
const Segment SGI22601[] = {{&S24856[30], 6}, {NULL}};
const Segment SGI22602[] = {{&S24856[112], 3}, {NULL}};
const Segment SGI22603[] = {{&S24856[78], 5}, {NULL}};
const Segment SGI22604[] = {{&S24864[44], 19}, {NULL}};
const Segment SGI22605[] = {{&S24864[44], 30}, {NULL}};
const Segment SGI22606[] = {{&S24864[30], 6}, {NULL}};
const Segment SGI22607[] = {{&S24864[122], 3}, {NULL}};
const Segment SGI22608[] = {{&S24864[88], 5}, {NULL}};
const Segment SGI22609[] = {{&S24865[39], 7}, {NULL}};
const Segment SGI22610[] = {{&S24865[91], 3}, {NULL}};
const Segment SGI22611[] = {{&S24865[68], 6}, {NULL}};
const Segment SGI22612[] = {{&S24866[44], 9}, {NULL}};
const Segment SGI22613[] = {{&S24866[44], 20}, {NULL}};
const Segment SGI22614[] = {{&S24866[30], 6}, {NULL}};
const Segment SGI22615[] = {{&S24866[112], 3}, {NULL}};
const Segment SGI22616[] = {{&S24866[78], 5}, {NULL}};
const Segment SGI22617[] = {{&S24867[44], 10}, {NULL}};
const Segment SGI22618[] = {{&S24867[44], 21}, {NULL}};
const Segment SGI22619[] = {{&S24867[30], 6}, {NULL}};
const Segment SGI22620[] = {{&S24867[113], 3}, {NULL}};
const Segment SGI22621[] = {{&S24867[79], 5}, {NULL}};
const Segment SGI22622[] = {{&S14920[44], 10}, {NULL}};
const Segment SGI22623[] = {{&S14920[44], 27}, {NULL}};
const Segment SGI22624[] = {{&S14920[30], 6}, {NULL}};
const Segment SGI22625[] = {{&S14920[119], 3}, {NULL}};
const Segment SGI22626[] = {{&S14920[85], 5}, {NULL}};
const Segment SGI22627[] = {{&S14921[38], 10}, {NULL}};
const Segment SGI22628[] = {{&S14921[31], 6}, {NULL}};
const Segment SGI22629[] = {{&S24873[44], 10}, {NULL}};
const Segment SGI22630[] = {{&S24873[44], 22}, {NULL}};
const Segment SGI22631[] = {{&S24873[30], 6}, {NULL}};
const Segment SGI22632[] = {{&S24873[114], 3}, {NULL}};
const Segment SGI22633[] = {{&S24873[80], 5}, {NULL}};
const Segment SGI22634[] = {{&S24879[38], 9}, {NULL}};
const Segment SGI22635[] = {{&S24879[31], 6}, {NULL}};
const Segment SGI22636[] = {{&S14932[44], 10}, {NULL}};
const Segment SGI22637[] = {{&S14932[44], 21}, {NULL}};
const Segment SGI22638[] = {{&S14932[30], 6}, {NULL}};
const Segment SGI22639[] = {{&S14932[113], 3}, {NULL}};
const Segment SGI22640[] = {{&S14932[79], 5}, {NULL}};
const Segment SGI22641[] = {{&S24882[44], 9}, {NULL}};
const Segment SGI22642[] = {{&S24882[44], 20}, {NULL}};
const Segment SGI22643[] = {{&S24882[30], 6}, {NULL}};
const Segment SGI22644[] = {{&S24882[81], 4}, {NULL}};
const Segment SGI22645[] = {{&S24882[81], 3}, {NULL}};
const Segment SGI22646[] = {{&S14936[44], 10}, {NULL}};
const Segment SGI22647[] = {{&S14936[44], 27}, {NULL}};
const Segment SGI22648[] = {{&S14936[30], 6}, {NULL}};
const Segment SGI22649[] = {{&S14936[88], 4}, {NULL}};
const Segment SGI22650[] = {{&S14936[88], 3}, {NULL}};
const Segment SGI22651[] = {{&S14958[44], 10}, {NULL}};
const Segment SGI22652[] = {{&S14958[44], 27}, {NULL}};
const Segment SGI22653[] = {{&S14958[30], 6}, {NULL}};
const Segment SGI22654[] = {{&S14958[88], 6}, {NULL}};
const Segment SGI22655[] = {{&S14964[42], 10}, {NULL}};
const Segment SGI22656[] = {{&S14964[30], 6}, {NULL}};
const Segment SGI22657[] = {{&S14964[117], 3}, {NULL}};
const Segment SGI22658[] = {{&S14964[83], 5}, {NULL}};
const Segment SGI22659[] = {{&S15018[44], 14}, {NULL}};
const Segment SGI22660[] = {{&S15018[44], 25}, {NULL}};
const Segment SGI22661[] = {{&S15018[30], 6}, {NULL}};
const Segment SGI22662[] = {{&S15018[117], 3}, {NULL}};
const Segment SGI22663[] = {{&S15018[83], 5}, {NULL}};
const Segment SGI22664[] = {{&S15019[44], 20}, {NULL}};
const Segment SGI22665[] = {{&S15019[44], 31}, {NULL}};
const Segment SGI22666[] = {{&S15019[30], 6}, {NULL}};
const Segment SGI22667[] = {{&S15019[123], 3}, {NULL}};
const Segment SGI22668[] = {{&S15019[89], 5}, {NULL}};
const Segment SGI22669[] = {{&S15038[44], 19}, {NULL}};
const Segment SGI22670[] = {{&S15038[44], 30}, {NULL}};
const Segment SGI22671[] = {{&S15038[30], 6}, {NULL}};
const Segment SGI22672[] = {{&S15038[122], 3}, {NULL}};
const Segment SGI22673[] = {{&S15038[88], 5}, {NULL}};
const Segment SGI22674[] = {{&S14535[44], 10}, {NULL}};
const Segment SGI22675[] = {{&S14535[44], 27}, {NULL}};
const Segment SGI22676[] = {{&S14535[30], 6}, {NULL}};
const Segment SGI22677[] = {{&S14535[119], 3}, {NULL}};
const Segment SGI22678[] = {{&S14535[85], 5}, {NULL}};
const Segment SGI22679[] = {{&S14538[38], 10}, {NULL}};
const Segment SGI22680[] = {{&S14538[31], 6}, {NULL}};
const Segment SGI22681[] = {{&S20359[44], 48}, {NULL}};
const Segment SGI22682[] = {{&S20359[44], 59}, {NULL}};
const Segment SGI22683[] = {{&S20359[30], 6}, {&S20359[52], 51}, {NULL}};
const Segment SGI22684[] = {{&S20359[151], 3}, {NULL}};
const Segment SGI22685[] = {{&S14544[44], 29}, {NULL}};
const Segment SGI22686[] = {{&S14544[44], 46}, {NULL}};
const Segment SGI22687[] = {{&S14544[30], 6}, {NULL}};
const Segment SGI22688[] = {{&S20359[117], 5}, {NULL}};
const Segment SGI22689[] = {{&S14544[138], 3}, {NULL}};
const Segment SGI22690[] = {{&S14544[104], 5}, {NULL}};
const Segment SGI22691[] = {{&S14564[42], 10}, {NULL}};
const Segment SGI22692[] = {{&S14564[30], 6}, {NULL}};
const Segment SGI22693[] = {{&S14564[117], 3}, {NULL}};
const Segment SGI22694[] = {{&S14564[83], 5}, {NULL}};
const Segment SGI22695[] = {{&S14571[44], 10}, {NULL}};
const Segment SGI22696[] = {{&S14571[44], 27}, {NULL}};
const Segment SGI22697[] = {{&S14571[30], 6}, {NULL}};
const Segment SGI22698[] = {{&S14571[88], 4}, {NULL}};
const Segment SGI22699[] = {{&S14571[88], 3}, {NULL}};
const Segment SGI22700[] = {{&S20432[37], 7}, {NULL}};
const Segment SGI22701[] = {{&S20432[30], 6}, {NULL}};
const Segment SGI22702[] = {{&S20432[109], 3}, {NULL}};
const Segment SGI22703[] = {{&S20432[75], 5}, {NULL}};
const Segment SGI22704[] = {{&S20434[44], 45}, {NULL}};
const Segment SGI22705[] = {{&S20434[44], 56}, {NULL}};
const Segment SGI22706[] = {{&S20434[30], 6}, {&S20434[52], 48}, {NULL}};
const Segment SGI22707[] = {{&S20434[148], 3}, {NULL}};
const Segment SGI22708[] = {{&S20434[114], 5}, {NULL}};
const Segment SGI22709[] = {{&S20436[38], 48}, {NULL}};
const Segment SGI22710[] = {{&S20436[31], 6}, {&S20436[46], 51}, {NULL}};
const Segment SGI22711[] = {{&S20454[44], 5}, {NULL}};
const Segment SGI22712[] = {{&S20454[44], 22}, {NULL}};
const Segment SGI22713[] = {{&S20454[30], 6}, {NULL}};
const Segment SGI22714[] = {{&S20454[114], 3}, {NULL}};
const Segment SGI22715[] = {{&S20454[80], 5}, {NULL}};
const Segment SGI22716[] = {{&S15959[44], 13}, {NULL}};
const Segment SGI22717[] = {{&S15959[44], 24}, {NULL}};
const Segment SGI22718[] = {{&S15959[30], 6}, {NULL}};
const Segment SGI22719[] = {{&S15959[116], 3}, {NULL}};
const Segment SGI22720[] = {{&S15959[82], 5}, {NULL}};
const Segment SGI22721[] = {{&S15960[44], 11}, {NULL}};
const Segment SGI22722[] = {{&S15960[44], 32}, {NULL}};
const Segment SGI22723[] = {{&S15960[30], 6}, {NULL}};
const Segment SGI22724[] = {{&S15960[124], 3}, {NULL}};
const Segment SGI22725[] = {{&S15960[90], 5}, {NULL}};
const Segment SGI22726[] = {{&S20480[22], 9}, {NULL}};
const Segment SGI22727[] = {{&S20480[44], 21}, {NULL}};
const Segment SGI22728[] = {{&S20480[30], 6}, {&S20480[52], 13}, {NULL}};
const Segment SGI22729[] = {{&S20480[113], 3}, {NULL}};
const Segment SGI22730[] = {{&S20480[79], 5}, {NULL}};
const Segment SGI22731[] = {{&S15968[44], 11}, {NULL}};
const Segment SGI22732[] = {{&S15968[44], 29}, {NULL}};
const Segment SGI22733[] = {{&S15968[30], 6}, {NULL}};
const Segment SGI22734[] = {{&S15968[121], 3}, {NULL}};
const Segment SGI22735[] = {{&S15968[87], 5}, {NULL}};
const Segment SGI22736[] = {{&S15558[44], 19}, {NULL}};
const Segment SGI22737[] = {{&S15558[44], 38}, {NULL}};
const Segment SGI22738[] = {{&S15558[30], 6}, {NULL}};
const Segment SGI22739[] = {{&S15558[69], 3}, {NULL}};
const Segment SGI22740[] = {{&S15558[96], 5}, {NULL}};
const Segment SGI22741[] = {{&S15559[44], 19}, {NULL}};
const Segment SGI22742[] = {{&S15559[44], 37}, {NULL}};
const Segment SGI22743[] = {{&S15559[30], 6}, {NULL}};
const Segment SGI22744[] = {{&S15559[69], 3}, {NULL}};
const Segment SGI22745[] = {{&S15559[95], 5}, {NULL}};
const Segment SGI22746[] = {{&S15982[44], 17}, {NULL}};
const Segment SGI22747[] = {{&S15982[44], 28}, {NULL}};
const Segment SGI22748[] = {{&S15982[30], 6}, {NULL}};
const Segment SGI22749[] = {{&S15982[120], 3}, {NULL}};
const Segment SGI22750[] = {{&S15560[44], 19}, {NULL}};
const Segment SGI22751[] = {{&S15560[44], 38}, {NULL}};
const Segment SGI22752[] = {{&S15982[86], 5}, {NULL}};
const Segment SGI22753[] = {{&S15560[30], 6}, {NULL}};
const Segment SGI22754[] = {{&S15560[69], 3}, {NULL}};
const Segment SGI22755[] = {{&S15560[96], 5}, {NULL}};
const Segment SGI22756[] = {{&S15561[44], 19}, {NULL}};
const Segment SGI22757[] = {{&S15561[44], 38}, {NULL}};
const Segment SGI22758[] = {{&S15561[30], 6}, {NULL}};
const Segment SGI22759[] = {{&S15561[69], 3}, {NULL}};
const Segment SGI22760[] = {{&S15561[96], 5}, {NULL}};
const Segment SGI22761[] = {{&S15985[44], 9}, {NULL}};
const Segment SGI22762[] = {{&S15985[44], 26}, {NULL}};
const Segment SGI22763[] = {{&S15985[30], 6}, {NULL}};
const Segment SGI22764[] = {{&S15985[118], 3}, {NULL}};
const Segment SGI22765[] = {{&S15985[84], 5}, {NULL}};
const Segment SGI22766[] = {{&S15562[44], 19}, {NULL}};
const Segment SGI22767[] = {{&S15562[44], 38}, {NULL}};
const Segment SGI22768[] = {{&S15562[30], 6}, {NULL}};
const Segment SGI22769[] = {{&S15562[69], 3}, {NULL}};
const Segment SGI22770[] = {{&S15562[96], 5}, {NULL}};
const Segment SGI22771[] = {{&S15563[44], 19}, {NULL}};
const Segment SGI22772[] = {{&S15563[44], 38}, {NULL}};
const Segment SGI22773[] = {{&S15563[30], 6}, {NULL}};
const Segment SGI22774[] = {{&S15563[69], 3}, {NULL}};
const Segment SGI22775[] = {{&S15563[96], 5}, {NULL}};
const Segment SGI22776[] = {{&S15564[44], 7}, {NULL}};
const Segment SGI22777[] = {{&S15564[44], 26}, {NULL}};
const Segment SGI22778[] = {{&S15564[30], 6}, {NULL}};
const Segment SGI22779[] = {{&S15564[57], 3}, {NULL}};
const Segment SGI22780[] = {{&S15564[84], 5}, {NULL}};
const Segment SGI22781[] = {{&S15992[44], 11}, {NULL}};
const Segment SGI22782[] = {{&S15992[44], 22}, {NULL}};
const Segment SGI22783[] = {{&S15992[30], 6}, {NULL}};
const Segment SGI22784[] = {{&S15992[114], 3}, {NULL}};
const Segment SGI22785[] = {{&S15565[44], 19}, {NULL}};
const Segment SGI22786[] = {{&S15565[44], 37}, {NULL}};
const Segment SGI22787[] = {{&S15992[80], 5}, {NULL}};
const Segment SGI22788[] = {{&S15565[30], 6}, {NULL}};
const Segment SGI22789[] = {{&S15565[69], 3}, {NULL}};
const Segment SGI22790[] = {{&S15565[95], 5}, {NULL}};
const Segment SGI22791[] = {{&S15993[44], 18}, {NULL}};
const Segment SGI22792[] = {{&S15993[44], 29}, {NULL}};
const Segment SGI22793[] = {{&S15993[30], 6}, {NULL}};
const Segment SGI22794[] = {{&S15993[121], 3}, {NULL}};
const Segment SGI22795[] = {{&S15993[87], 5}, {NULL}};
const Segment SGI22796[] = {{&S15566[44], 20}, {NULL}};
const Segment SGI22797[] = {{&S15566[44], 39}, {NULL}};
const Segment SGI22798[] = {{&S15566[30], 6}, {NULL}};
const Segment SGI22799[] = {{&S15566[70], 3}, {NULL}};
const Segment SGI22800[] = {{&S20512[44], 12}, {NULL}};
const Segment SGI22801[] = {{&S20512[44], 29}, {NULL}};
const Segment SGI22802[] = {{&S15566[97], 5}, {NULL}};
const Segment SGI22803[] = {{&S20512[30], 6}, {NULL}};
const Segment SGI22804[] = {{&S20512[121], 3}, {NULL}};
const Segment SGI22805[] = {{&S20512[87], 5}, {NULL}};
const Segment SGI22806[] = {{&S15567[44], 19}, {NULL}};
const Segment SGI22807[] = {{&S15567[44], 38}, {NULL}};
const Segment SGI22808[] = {{&S15567[30], 6}, {NULL}};
const Segment SGI22809[] = {{&S15567[69], 3}, {NULL}};
const Segment SGI22810[] = {{&S15567[96], 5}, {NULL}};
const Segment SGI22811[] = {{&S15568[44], 7}, {NULL}};
const Segment SGI22812[] = {{&S15568[44], 26}, {NULL}};
const Segment SGI22813[] = {{&S15568[30], 6}, {NULL}};
const Segment SGI22814[] = {{&S15568[57], 3}, {NULL}};
const Segment SGI22815[] = {{&S15998[44], 14}, {NULL}};
const Segment SGI22816[] = {{&S15568[84], 5}, {NULL}};
const Segment SGI22817[] = {{&S15998[44], 25}, {NULL}};
const Segment SGI22818[] = {{&S15998[30], 6}, {NULL}};
const Segment SGI22819[] = {{&S15998[117], 3}, {NULL}};
const Segment SGI22820[] = {{&S15998[83], 5}, {NULL}};
const Segment SGI22821[] = {{&S15569[44], 21}, {NULL}};
const Segment SGI22822[] = {{&S15569[44], 40}, {NULL}};
const Segment SGI22823[] = {{&S15569[30], 6}, {NULL}};
const Segment SGI22824[] = {{&S15569[71], 3}, {NULL}};
const Segment SGI22825[] = {{&S15569[98], 5}, {NULL}};
const Segment SGI22826[] = {{&S15570[44], 19}, {NULL}};
const Segment SGI22827[] = {{&S15570[44], 38}, {NULL}};
const Segment SGI22828[] = {{&S16001[44], 12}, {NULL}};
const Segment SGI22829[] = {{&S15570[30], 6}, {NULL}};
const Segment SGI22830[] = {{&S16001[44], 23}, {NULL}};
const Segment SGI22831[] = {{&S16001[30], 6}, {NULL}};
const Segment SGI22832[] = {{&S15570[69], 3}, {NULL}};
const Segment SGI22833[] = {{&S16001[115], 3}, {NULL}};
const Segment SGI22834[] = {{&S15570[96], 5}, {NULL}};
const Segment SGI22835[] = {{&S16001[81], 5}, {NULL}};
const Segment SGI22836[] = {{&S15571[44], 20}, {NULL}};
const Segment SGI22837[] = {{&S15571[44], 38}, {NULL}};
const Segment SGI22838[] = {{&S15571[30], 6}, {NULL}};
const Segment SGI22839[] = {{&S15571[70], 3}, {NULL}};
const Segment SGI22840[] = {{&S15571[96], 5}, {NULL}};
const Segment SGI22841[] = {{&S15572[44], 21}, {NULL}};
const Segment SGI22842[] = {{&S15572[44], 39}, {NULL}};
const Segment SGI22843[] = {{&S15572[30], 6}, {NULL}};
const Segment SGI22844[] = {{&S15572[71], 3}, {NULL}};
const Segment SGI22845[] = {{&S15572[97], 5}, {NULL}};
const Segment SGI22846[] = {{&S15573[38], 7}, {NULL}};
const Segment SGI22847[] = {{&S15573[31], 6}, {NULL}};
const Segment SGI22848[] = {{&S16008[44], 8}, {NULL}};
const Segment SGI22849[] = {{&S16008[44], 19}, {NULL}};
const Segment SGI22850[] = {{&S16008[30], 6}, {NULL}};
const Segment SGI22851[] = {{&S16008[111], 3}, {NULL}};
const Segment SGI22852[] = {{&S15574[44], 19}, {NULL}};
const Segment SGI22853[] = {{&S16008[77], 5}, {NULL}};
const Segment SGI22854[] = {{&S15574[44], 37}, {NULL}};
const Segment SGI22855[] = {{&S15574[30], 6}, {NULL}};
const Segment SGI22856[] = {{&S15574[69], 3}, {NULL}};
const Segment SGI22857[] = {{&S15574[95], 5}, {NULL}};
const Segment SGI22858[] = {{&S15575[44], 19}, {NULL}};
const Segment SGI22859[] = {{&S15575[44], 37}, {NULL}};
const Segment SGI22860[] = {{&S15575[30], 6}, {NULL}};
const Segment SGI22861[] = {{&S15575[69], 3}, {NULL}};
const Segment SGI22862[] = {{&S15575[95], 5}, {NULL}};
const Segment SGI22863[] = {{&S16011[38], 37}, {NULL}};
const Segment SGI22864[] = {{&S16011[31], 6}, {NULL}};
const Segment SGI22865[] = {{&S15576[44], 21}, {NULL}};
const Segment SGI22866[] = {{&S15576[44], 39}, {NULL}};
const Segment SGI22867[] = {{&S15576[30], 6}, {NULL}};
const Segment SGI22868[] = {{&S15576[71], 3}, {NULL}};
const Segment SGI22869[] = {{&S15576[97], 5}, {NULL}};
const Segment SGI22870[] = {{&S15577[44], 19}, {NULL}};
const Segment SGI22871[] = {{&S15577[44], 37}, {NULL}};
const Segment SGI22872[] = {{&S15577[30], 6}, {NULL}};
const Segment SGI22873[] = {{&S16014[44], 12}, {NULL}};
const Segment SGI22874[] = {{&S16014[44], 23}, {NULL}};
const Segment SGI22875[] = {{&S16014[30], 6}, {NULL}};
const Segment SGI22876[] = {{&S15577[69], 3}, {NULL}};
const Segment SGI22877[] = {{&S15577[95], 5}, {NULL}};
const Segment SGI22878[] = {{&S16014[115], 3}, {NULL}};
const Segment SGI22879[] = {{&S16014[81], 5}, {NULL}};
const Segment SGI22880[] = {{&S15578[44], 7}, {NULL}};
const Segment SGI22881[] = {{&S15578[44], 25}, {NULL}};
const Segment SGI22882[] = {{&S15578[30], 6}, {NULL}};
const Segment SGI22883[] = {{&S15578[57], 3}, {NULL}};
const Segment SGI22884[] = {{&S15578[83], 5}, {NULL}};
const Segment SGI22885[] = {{&S15579[44], 19}, {NULL}};
const Segment SGI22886[] = {{&S15579[44], 36}, {NULL}};
const Segment SGI22887[] = {{&S15579[30], 6}, {NULL}};
const Segment SGI22888[] = {{&S15579[69], 3}, {NULL}};
const Segment SGI22889[] = {{&S15579[94], 5}, {NULL}};
const Segment SGI22890[] = {{&S16019[44], 10}, {NULL}};
const Segment SGI22891[] = {{&S16019[44], 21}, {NULL}};
const Segment SGI22892[] = {{&S16019[30], 6}, {NULL}};
const Segment SGI22893[] = {{&S16019[113], 3}, {NULL}};
const Segment SGI22894[] = {{&S15580[44], 19}, {NULL}};
const Segment SGI22895[] = {{&S16019[79], 5}, {NULL}};
const Segment SGI22896[] = {{&S15580[44], 37}, {NULL}};
const Segment SGI22897[] = {{&S15580[30], 6}, {NULL}};
const Segment SGI22898[] = {{&S15580[69], 3}, {NULL}};
const Segment SGI22899[] = {{&S15580[95], 5}, {NULL}};
const Segment SGI22900[] = {{&S15581[44], 20}, {NULL}};
const Segment SGI22901[] = {{&S15581[44], 38}, {NULL}};
const Segment SGI22902[] = {{&S15581[30], 6}, {NULL}};
const Segment SGI22903[] = {{&S15581[70], 3}, {NULL}};
const Segment SGI22904[] = {{&S15581[96], 5}, {NULL}};
const Segment SGI22905[] = {{&S16022[44], 37}, {NULL}};
const Segment SGI22906[] = {{&S16022[44], 48}, {NULL}};
const Segment SGI22907[] = {{&S16022[30], 6}, {NULL}};
const Segment SGI22908[] = {{&S16022[140], 3}, {NULL}};
const Segment SGI22909[] = {{&S16022[106], 5}, {NULL}};
const Segment SGI22910[] = {{&S15582[44], 7}, {NULL}};
const Segment SGI22911[] = {{&S15582[44], 25}, {NULL}};
const Segment SGI22912[] = {{&S15582[30], 6}, {NULL}};
const Segment SGI22913[] = {{&S15582[57], 3}, {NULL}};
const Segment SGI22914[] = {{&S15582[83], 5}, {NULL}};
const Segment SGI22915[] = {{&S15583[44], 7}, {NULL}};
const Segment SGI22916[] = {{&S15583[44], 25}, {NULL}};
const Segment SGI22917[] = {{&S15583[30], 6}, {NULL}};
const Segment SGI22918[] = {{&S15583[86], 4}, {NULL}};
const Segment SGI22919[] = {{&S15583[86], 3}, {NULL}};
const Segment SGI22920[] = {{&S15584[44], 7}, {NULL}};
const Segment SGI22921[] = {{&S15584[44], 25}, {NULL}};
const Segment SGI22922[] = {{&S15584[30], 6}, {NULL}};
const Segment SGI22923[] = {{&S15584[86], 4}, {NULL}};
const Segment SGI22924[] = {{&S15584[86], 3}, {NULL}};
const Segment SGI22925[] = {{&S16028[44], 8}, {NULL}};
const Segment SGI22926[] = {{&S16028[44], 19}, {NULL}};
const Segment SGI22927[] = {{&S16028[30], 6}, {NULL}};
const Segment SGI22928[] = {{&S15585[44], 19}, {NULL}};
const Segment SGI22929[] = {{&S15585[44], 36}, {NULL}};
const Segment SGI22930[] = {{&S15585[30], 6}, {NULL}};
const Segment SGI22931[] = {{&S16028[111], 3}, {NULL}};
const Segment SGI22932[] = {{&S16028[77], 5}, {NULL}};
const Segment SGI22933[] = {{&S15585[69], 3}, {NULL}};
const Segment SGI22934[] = {{&S15585[94], 5}, {NULL}};
const Segment SGI22935[] = {{&S15586[44], 7}, {NULL}};
const Segment SGI22936[] = {{&S15586[44], 25}, {NULL}};
const Segment SGI22937[] = {{&S15586[30], 6}, {NULL}};
const Segment SGI22938[] = {{&S16030[44], 15}, {NULL}};
const Segment SGI22939[] = {{&S16030[44], 26}, {NULL}};
const Segment SGI22940[] = {{&S16030[30], 6}, {NULL}};
const Segment SGI22941[] = {{&S15586[86], 4}, {NULL}};
const Segment SGI22942[] = {{&S15586[86], 3}, {NULL}};
const Segment SGI22943[] = {{&S16030[118], 3}, {NULL}};
const Segment SGI22944[] = {{&S16030[84], 5}, {NULL}};
const Segment SGI22945[] = {{&S15587[44], 7}, {NULL}};
const Segment SGI22946[] = {{&S15587[44], 25}, {NULL}};
const Segment SGI22947[] = {{&S15587[30], 6}, {NULL}};
const Segment SGI22948[] = {{&S15587[57], 3}, {NULL}};
const Segment SGI22949[] = {{&S15587[83], 5}, {NULL}};
const Segment SGI22950[] = {{&S15588[44], 7}, {NULL}};
const Segment SGI22951[] = {{&S15588[44], 24}, {NULL}};
const Segment SGI22952[] = {{&S15588[30], 6}, {NULL}};
const Segment SGI22953[] = {{&S15588[57], 3}, {NULL}};
const Segment SGI22954[] = {{&S15588[82], 5}, {NULL}};
const Segment SGI22955[] = {{&S15589[44], 19}, {NULL}};
const Segment SGI22956[] = {{&S15589[44], 37}, {NULL}};
const Segment SGI22957[] = {{&S15589[30], 6}, {NULL}};
const Segment SGI22958[] = {{&S15589[69], 3}, {NULL}};
const Segment SGI22959[] = {{&S15589[95], 5}, {NULL}};
const Segment SGI22960[] = {{&S16036[44], 8}, {NULL}};
const Segment SGI22961[] = {{&S16036[44], 19}, {NULL}};
const Segment SGI22962[] = {{&S16036[30], 6}, {NULL}};
const Segment SGI22963[] = {{&S16036[111], 3}, {NULL}};
const Segment SGI22964[] = {{&S16036[77], 5}, {NULL}};
const Segment SGI22965[] = {{&S15590[44], 19}, {NULL}};
const Segment SGI22966[] = {{&S15590[44], 36}, {NULL}};
const Segment SGI22967[] = {{&S15590[30], 6}, {NULL}};
const Segment SGI22968[] = {{&S15590[69], 3}, {NULL}};
const Segment SGI22969[] = {{&S16037[44], 10}, {NULL}};
const Segment SGI22970[] = {{&S16037[44], 21}, {NULL}};
const Segment SGI22971[] = {{&S15590[94], 5}, {NULL}};
const Segment SGI22972[] = {{&S16037[30], 6}, {NULL}};
const Segment SGI22973[] = {{&S16037[113], 3}, {NULL}};
const Segment SGI22974[] = {{&S16037[79], 5}, {NULL}};
const Segment SGI22975[] = {{&S15591[44], 19}, {NULL}};
const Segment SGI22976[] = {{&S15591[44], 38}, {NULL}};
const Segment SGI22977[] = {{&S15591[30], 6}, {NULL}};
const Segment SGI22978[] = {{&S15591[69], 3}, {NULL}};
const Segment SGI22979[] = {{&S15591[96], 5}, {NULL}};
const Segment SGI22980[] = {{&S15592[44], 7}, {NULL}};
const Segment SGI22981[] = {{&S15592[44], 25}, {NULL}};
const Segment SGI22982[] = {{&S15592[30], 6}, {NULL}};
const Segment SGI22983[] = {{&S15592[57], 3}, {NULL}};
const Segment SGI22984[] = {{&S15592[83], 5}, {NULL}};
const Segment SGI22985[] = {{&S16042[44], 16}, {NULL}};
const Segment SGI22986[] = {{&S15593[44], 7}, {NULL}};
const Segment SGI22987[] = {{&S16042[44], 27}, {NULL}};
const Segment SGI22988[] = {{&S15593[44], 25}, {NULL}};
const Segment SGI22989[] = {{&S16042[30], 6}, {NULL}};
const Segment SGI22990[] = {{&S15593[30], 6}, {NULL}};
const Segment SGI22991[] = {{&S16042[119], 3}, {NULL}};
const Segment SGI22992[] = {{&S15593[57], 3}, {NULL}};
const Segment SGI22993[] = {{&S15593[83], 5}, {NULL}};
const Segment SGI22994[] = {{&S15594[44], 7}, {NULL}};
const Segment SGI22995[] = {{&S15594[44], 26}, {NULL}};
const Segment SGI22996[] = {{&S15594[30], 6}, {NULL}};
const Segment SGI22997[] = {{&S15594[87], 4}, {NULL}};
const Segment SGI22998[] = {{&S15594[87], 3}, {NULL}};
const Segment SGI22999[] = {{&S15595[44], 7}, {NULL}};
const Segment SGI23000[] = {{&S15595[44], 25}, {NULL}};
const Segment SGI23001[] = {{&S15595[30], 6}, {NULL}};
const Segment SGI23002[] = {{&S15595[86], 6}, {NULL}};
const Segment SGI23003[] = {{&S15596[44], 7}, {NULL}};
const Segment SGI23004[] = {{&S15596[44], 25}, {NULL}};
const Segment SGI23005[] = {{&S15596[30], 6}, {NULL}};
const Segment SGI23006[] = {{&S15596[57], 3}, {NULL}};
const Segment SGI23007[] = {{&S15596[83], 5}, {NULL}};
const Segment SGI23008[] = {{&S15597[38], 7}, {NULL}};
const Segment SGI23009[] = {{&S15597[31], 6}, {NULL}};
const Segment SGI23010[] = {{&S15598[44], 7}, {NULL}};
const Segment SGI23011[] = {{&S15598[44], 26}, {NULL}};
const Segment SGI23012[] = {{&S15598[30], 6}, {NULL}};
const Segment SGI23013[] = {{&S15598[57], 3}, {NULL}};
const Segment SGI23014[] = {{&S15598[84], 5}, {NULL}};
const Segment SGI23015[] = {{&S15599[42], 19}, {NULL}};
const Segment SGI23016[] = {{&S15599[30], 6}, {NULL}};
const Segment SGI23017[] = {{&S15599[67], 3}, {NULL}};
const Segment SGI23018[] = {{&S15599[93], 5}, {NULL}};
const Segment SGI23019[] = {{&S15600[42], 19}, {NULL}};
const Segment SGI23020[] = {{&S15600[30], 6}, {NULL}};
const Segment SGI23021[] = {{&S15600[67], 3}, {NULL}};
const Segment SGI23022[] = {{&S15600[93], 5}, {NULL}};
const Segment SGI23023[] = {{&S15601[44], 7}, {NULL}};
const Segment SGI23024[] = {{&S15601[44], 24}, {NULL}};
const Segment SGI23025[] = {{&S15601[30], 6}, {NULL}};
const Segment SGI23026[] = {{&S15601[57], 3}, {NULL}};
const Segment SGI23027[] = {{&S15601[82], 5}, {NULL}};
const Segment SGI23028[] = {{&S15602[44], 7}, {NULL}};
const Segment SGI23029[] = {{&S15602[44], 25}, {NULL}};
const Segment SGI23030[] = {{&S15602[30], 6}, {NULL}};
const Segment SGI23031[] = {{&S15602[86], 6}, {NULL}};
const Segment SGI23032[] = {{&S15603[44], 7}, {NULL}};
const Segment SGI23033[] = {{&S15603[44], 25}, {NULL}};
const Segment SGI23034[] = {{&S15603[30], 6}, {NULL}};
const Segment SGI23035[] = {{&S15603[86], 6}, {NULL}};
const Segment SGI23036[] = {{&S15604[44], 7}, {NULL}};
const Segment SGI23037[] = {{&S15604[44], 25}, {NULL}};
const Segment SGI23038[] = {{&S15604[30], 6}, {NULL}};
const Segment SGI23039[] = {{&S15604[86], 6}, {NULL}};
const Segment SGI23040[] = {{&S15605[44], 20}, {NULL}};
const Segment SGI23041[] = {{&S15605[44], 38}, {NULL}};
const Segment SGI23042[] = {{&S15605[30], 6}, {NULL}};
const Segment SGI23043[] = {{&S15605[70], 3}, {NULL}};
const Segment SGI23044[] = {{&S15605[96], 5}, {NULL}};
const Segment SGI23045[] = {{&S15606[44], 7}, {NULL}};
const Segment SGI23046[] = {{&S15606[44], 26}, {NULL}};
const Segment SGI23047[] = {{&S15606[30], 6}, {NULL}};
const Segment SGI23048[] = {{&S15606[87], 6}, {NULL}};
const Segment SGI23049[] = {{&S20602[44], 5}, {NULL}};
const Segment SGI23050[] = {{&S20602[44], 22}, {NULL}};
const Segment SGI23051[] = {{&S20602[30], 6}, {NULL}};
const Segment SGI23052[] = {{&S20602[114], 3}, {NULL}};
const Segment SGI23053[] = {{&S20602[80], 5}, {NULL}};
const Segment SGI23054[] = {{&S15608[44], 25}, {NULL}};
const Segment SGI23055[] = {{&S15608[30], 6}, {NULL}};
const Segment SGI23056[] = {{&S16042[85], 5}, {NULL}};
const Segment SGI23057[] = {{&S14922[44], 10}, {NULL}};
const Segment SGI23058[] = {{&S14922[44], 26}, {NULL}};
const Segment SGI23059[] = {{&S14922[30], 6}, {NULL}};
const Segment SGI23060[] = {{&S14922[118], 3}, {NULL}};
const Segment SGI23061[] = {{&S14922[84], 5}, {NULL}};
const Segment SGI23062[] = {{&S14923[44], 10}, {NULL}};
const Segment SGI23063[] = {{&S14923[44], 27}, {NULL}};
const Segment SGI23064[] = {{&S14923[30], 6}, {NULL}};
const Segment SGI23065[] = {{&S14923[119], 3}, {NULL}};
const Segment SGI23066[] = {{&S14923[85], 5}, {NULL}};
const Segment SGI23067[] = {{&S16056[44], 7}, {NULL}};
const Segment SGI23068[] = {{&S16056[44], 18}, {NULL}};
const Segment SGI23069[] = {{&S16056[30], 6}, {NULL}};
const Segment SGI23070[] = {{&S14926[38], 10}, {NULL}};
const Segment SGI23071[] = {{&S14926[31], 6}, {NULL}};
const Segment SGI23072[] = {{&S16056[110], 3}, {NULL}};
const Segment SGI23073[] = {{&S16056[76], 5}, {NULL}};
const Segment SGI23074[] = {{&S14930[44], 28}, {NULL}};
const Segment SGI23075[] = {{&S14930[44], 45}, {NULL}};
const Segment SGI23076[] = {{&S14930[30], 6}, {NULL}};
const Segment SGI23077[] = {{&S16060[44], 14}, {NULL}};
const Segment SGI23078[] = {{&S16060[44], 25}, {NULL}};
const Segment SGI23079[] = {{&S16060[30], 6}, {NULL}};
const Segment SGI23080[] = {{&S14930[137], 3}, {NULL}};
const Segment SGI23081[] = {{&S14930[103], 5}, {NULL}};
const Segment SGI23082[] = {{&S16060[117], 3}, {NULL}};
const Segment SGI23083[] = {{&S16060[83], 5}, {NULL}};
const Segment SGI23084[] = {{&S16071[44], 4}, {NULL}};
const Segment SGI23085[] = {{&S16071[44], 26}, {NULL}};
const Segment SGI23086[] = {{&S16071[30], 6}, {NULL}};
const Segment SGI23087[] = {{&S16071[118], 3}, {NULL}};
const Segment SGI23088[] = {{&S16071[84], 5}, {NULL}};
const Segment SGI23089[] = {{&S14942[44], 10}, {NULL}};
const Segment SGI23090[] = {{&S14942[44], 20}, {NULL}};
const Segment SGI23091[] = {{&S14942[30], 6}, {NULL}};
const Segment SGI23092[] = {{&S14942[112], 3}, {NULL}};
const Segment SGI23093[] = {{&S14942[78], 5}, {NULL}};
const Segment SGI23094[] = {{&S14944[44], 10}, {NULL}};
const Segment SGI23095[] = {{&S14944[44], 27}, {NULL}};
const Segment SGI23096[] = {{&S14944[30], 6}, {NULL}};
const Segment SGI23097[] = {{&S14944[88], 4}, {NULL}};
const Segment SGI23098[] = {{&S14944[88], 3}, {NULL}};
const Segment SGI23099[] = {{&S14945[44], 10}, {NULL}};
const Segment SGI23100[] = {{&S14945[44], 23}, {NULL}};
const Segment SGI23101[] = {{&S14945[30], 6}, {NULL}};
const Segment SGI23102[] = {{&S14945[115], 3}, {NULL}};
const Segment SGI23103[] = {{&S14945[81], 5}, {NULL}};
const Segment SGI23104[] = {{&S16077[44], 9}, {NULL}};
const Segment SGI23105[] = {{&S16077[44], 20}, {NULL}};
const Segment SGI23106[] = {{&S16077[30], 6}, {NULL}};
const Segment SGI23107[] = {{&S16077[112], 3}, {NULL}};
const Segment SGI23108[] = {{&S16077[78], 5}, {NULL}};
const Segment SGI23109[] = {{&S14947[44], 10}, {NULL}};
const Segment SGI23110[] = {{&S14947[44], 27}, {NULL}};
const Segment SGI23111[] = {{&S14947[30], 6}, {NULL}};
const Segment SGI23112[] = {{&S14947[120], 3}, {NULL}};
const Segment SGI23113[] = {{&S14947[85], 6}, {NULL}};
const Segment SGI23114[] = {{&S16080[44], 6}, {NULL}};
const Segment SGI23115[] = {{&S16080[44], 23}, {NULL}};
const Segment SGI23116[] = {{&S16080[30], 6}, {NULL}};
const Segment SGI23117[] = {{&S16080[115], 3}, {NULL}};
const Segment SGI23118[] = {{&S16080[81], 5}, {NULL}};
const Segment SGI23119[] = {{&S16085[44], 21}, {NULL}};
const Segment SGI23120[] = {{&S16085[44], 38}, {NULL}};
const Segment SGI23121[] = {{&S16085[30], 6}, {NULL}};
const Segment SGI23122[] = {{&S16085[130], 3}, {NULL}};
const Segment SGI23123[] = {{&S16085[96], 5}, {NULL}};
const Segment SGI23124[] = {{&S16086[44], 10}, {NULL}};
const Segment SGI23125[] = {{&S16086[44], 27}, {NULL}};
const Segment SGI23126[] = {{&S16086[30], 6}, {NULL}};
const Segment SGI23127[] = {{&S16086[119], 3}, {NULL}};
const Segment SGI23128[] = {{&S14957[42], 10}, {NULL}};
const Segment SGI23129[] = {{&S16086[85], 5}, {NULL}};
const Segment SGI23130[] = {{&S14957[30], 6}, {NULL}};
const Segment SGI23131[] = {{&S14957[117], 3}, {NULL}};
const Segment SGI23132[] = {{&S14957[83], 5}, {NULL}};
const Segment SGI23133[] = {{&S16087[44], 27}, {NULL}};
const Segment SGI23134[] = {{&S16087[44], 37}, {NULL}};
const Segment SGI23135[] = {{&S16087[30], 6}, {&S16087[52], 29}, {NULL}};
const Segment SGI23136[] = {{&S16087[129], 3}, {NULL}};
const Segment SGI23137[] = {{&S16087[95], 5}, {NULL}};
const Segment SGI23138[] = {{&S14959[44], 10}, {NULL}};
const Segment SGI23139[] = {{&S14959[44], 27}, {NULL}};
const Segment SGI23140[] = {{&S14959[30], 6}, {NULL}};
const Segment SGI23141[] = {{&S14959[88], 6}, {NULL}};
const Segment SGI23142[] = {{&S14962[44], 10}, {NULL}};
const Segment SGI23143[] = {{&S14962[44], 21}, {NULL}};
const Segment SGI23144[] = {{&S14962[30], 6}, {NULL}};
const Segment SGI23145[] = {{&S14962[113], 3}, {NULL}};
const Segment SGI23146[] = {{&S14962[79], 5}, {NULL}};
const Segment SGI23147[] = {{&S16096[44], 17}, {NULL}};
const Segment SGI23148[] = {{&S16096[44], 28}, {NULL}};
const Segment SGI23149[] = {{&S16096[30], 6}, {NULL}};
const Segment SGI23150[] = {{&S16096[120], 3}, {NULL}};
const Segment SGI23151[] = {{&S16096[86], 5}, {NULL}};
const Segment SGI23152[] = {{&S16103[44], 8}, {NULL}};
const Segment SGI23153[] = {{&S16103[44], 19}, {NULL}};
const Segment SGI23154[] = {{&S16103[30], 6}, {NULL}};
const Segment SGI23155[] = {{&S16103[111], 3}, {NULL}};
const Segment SGI23156[] = {{&S16103[77], 5}, {NULL}};
const Segment SGI23157[] = {{&S16109[44], 13}, {NULL}};
const Segment SGI23158[] = {{&S16109[44], 24}, {NULL}};
const Segment SGI23159[] = {{&S16109[30], 6}, {NULL}};
const Segment SGI23160[] = {{&S16109[116], 3}, {NULL}};
const Segment SGI23161[] = {{&S16109[82], 5}, {NULL}};
const Segment SGI23162[] = {{&S16111[44], 10}, {NULL}};
const Segment SGI23163[] = {{&S16111[44], 27}, {NULL}};
const Segment SGI23164[] = {{&S16111[30], 6}, {NULL}};
const Segment SGI23165[] = {{&S16111[119], 3}, {NULL}};
const Segment SGI23166[] = {{&S16111[85], 5}, {NULL}};
const Segment SGI23167[] = {{&S16114[44], 12}, {NULL}};
const Segment SGI23168[] = {{&S16114[44], 29}, {NULL}};
const Segment SGI23169[] = {{&S16114[30], 6}, {NULL}};
const Segment SGI23170[] = {{&S16114[121], 3}, {NULL}};
const Segment SGI23171[] = {{&S16114[87], 5}, {NULL}};
const Segment SGI23172[] = {{&S20364[30], 4}, {NULL}};
const Segment SGI23173[] = {{&S20364[55], 4}, {NULL}};
const Segment SGI23174[] = {{&S15008[44], 19}, {NULL}};
const Segment SGI23175[] = {{&S15008[44], 30}, {NULL}};
const Segment SGI23176[] = {{&S15008[30], 6}, {NULL}};
const Segment SGI23177[] = {{&S15008[122], 3}, {NULL}};
const Segment SGI23178[] = {{&S15008[88], 5}, {NULL}};
const Segment SGI23179[] = {{&S15010[44], 19}, {NULL}};
const Segment SGI23180[] = {{&S15010[44], 30}, {NULL}};
const Segment SGI23181[] = {{&S15010[30], 6}, {NULL}};
const Segment SGI23182[] = {{&S15010[122], 3}, {NULL}};
const Segment SGI23183[] = {{&S15010[88], 5}, {NULL}};
const Segment SGI23184[] = {{&S20369[42], 8}, {NULL}};
const Segment SGI23185[] = {{&S20369[30], 4}, {NULL}};
const Segment SGI23186[] = {{&S20369[100], 3}, {NULL}};
const Segment SGI23187[] = {{&S20369[64], 7}, {NULL}};
const Segment SGI23188[] = {{&S20369[120], 8}, {NULL}};
const Segment SGI23189[] = {{&S16143[44], 6}, {NULL}};
const Segment SGI23190[] = {{&S16143[44], 17}, {NULL}};
const Segment SGI23191[] = {{&S16143[30], 6}, {NULL}};
const Segment SGI23192[] = {{&S16143[109], 3}, {NULL}};
const Segment SGI23193[] = {{&S16143[75], 5}, {NULL}};
const Segment SGI23194[] = {{&S16144[38], 18}, {NULL}};
const Segment SGI23195[] = {{&S16144[31], 6}, {NULL}};
const Segment SGI23196[] = {{&S15017[44], 14}, {NULL}};
const Segment SGI23197[] = {{&S15017[44], 25}, {NULL}};
const Segment SGI23198[] = {{&S15017[30], 6}, {NULL}};
const Segment SGI23199[] = {{&S15017[117], 3}, {NULL}};
const Segment SGI23200[] = {{&S15017[83], 5}, {NULL}};
const Segment SGI23201[] = {{&S16145[44], 5}, {NULL}};
const Segment SGI23202[] = {{&S16145[44], 16}, {NULL}};
const Segment SGI23203[] = {{&S16145[30], 6}, {NULL}};
const Segment SGI23204[] = {{&S16145[108], 3}, {NULL}};
const Segment SGI23205[] = {{&S16145[74], 5}, {NULL}};
const Segment SGI23206[] = {{&S15020[44], 20}, {NULL}};
const Segment SGI23207[] = {{&S20379[42], 5}, {NULL}};
const Segment SGI23208[] = {{&S15020[44], 31}, {NULL}};
const Segment SGI23209[] = {{&S20379[42], 16}, {NULL}};
const Segment SGI23210[] = {{&S15020[30], 6}, {NULL}};
const Segment SGI23211[] = {{&S20379[30], 4}, {NULL}};
const Segment SGI23212[] = {{&S15020[123], 3}, {NULL}};
const Segment SGI23213[] = {{&S20379[106], 3}, {NULL}};
const Segment SGI23214[] = {{&S20379[72], 5}, {NULL}};
const Segment SGI23215[] = {{&S15020[89], 5}, {NULL}};
const Segment SGI23216[] = {{&S15021[44], 23}, {NULL}};
const Segment SGI23217[] = {{&S15021[44], 34}, {NULL}};
const Segment SGI23218[] = {{&S15021[30], 6}, {NULL}};
const Segment SGI23219[] = {{&S15021[126], 3}, {NULL}};
const Segment SGI23220[] = {{&S15021[92], 5}, {NULL}};
const Segment SGI23221[] = {{&S16153[44], 6}, {NULL}};
const Segment SGI23222[] = {{&S16153[44], 17}, {NULL}};
const Segment SGI23223[] = {{&S16153[30], 6}, {NULL}};
const Segment SGI23224[] = {{&S16153[109], 3}, {NULL}};
const Segment SGI23225[] = {{&S15025[44], 24}, {NULL}};
const Segment SGI23226[] = {{&S16153[75], 5}, {NULL}};
const Segment SGI23227[] = {{&S15025[44], 35}, {NULL}};
const Segment SGI23228[] = {{&S15025[30], 6}, {NULL}};
const Segment SGI23229[] = {{&S15025[127], 3}, {NULL}};
const Segment SGI23230[] = {{&S15025[93], 5}, {NULL}};
const Segment SGI23231[] = {{&S16154[44], 7}, {NULL}};
const Segment SGI23232[] = {{&S16154[44], 18}, {NULL}};
const Segment SGI23233[] = {{&S16154[30], 6}, {NULL}};
const Segment SGI23234[] = {{&S16154[110], 3}, {NULL}};
const Segment SGI23235[] = {{&S15026[44], 13}, {NULL}};
const Segment SGI23236[] = {{&S15026[44], 24}, {NULL}};
const Segment SGI23237[] = {{&S16154[76], 5}, {NULL}};
const Segment SGI23238[] = {{&S15026[30], 6}, {NULL}};
const Segment SGI23239[] = {{&S15026[116], 3}, {NULL}};
const Segment SGI23240[] = {{&S15026[82], 5}, {NULL}};
const Segment SGI23241[] = {{&S16156[44], 9}, {NULL}};
const Segment SGI23242[] = {{&S16156[44], 20}, {NULL}};
const Segment SGI23243[] = {{&S16156[30], 6}, {NULL}};
const Segment SGI23244[] = {{&S15028[44], 24}, {NULL}};
const Segment SGI23245[] = {{&S15028[44], 35}, {NULL}};
const Segment SGI23246[] = {{&S16156[112], 3}, {NULL}};
const Segment SGI23247[] = {{&S15028[30], 6}, {NULL}};
const Segment SGI23248[] = {{&S16156[78], 5}, {NULL}};
const Segment SGI23249[] = {{&S15028[127], 3}, {NULL}};
const Segment SGI23250[] = {{&S15028[93], 5}, {NULL}};
const Segment SGI23251[] = {{&S15029[38], 14}, {NULL}};
const Segment SGI23252[] = {{&S15029[31], 6}, {NULL}};
const Segment SGI23253[] = {{&S15030[44], 23}, {NULL}};
const Segment SGI23254[] = {{&S15030[44], 34}, {NULL}};
const Segment SGI23255[] = {{&S15030[30], 6}, {NULL}};
const Segment SGI23256[] = {{&S16159[44], 9}, {NULL}};
const Segment SGI23257[] = {{&S16159[44], 20}, {NULL}};
const Segment SGI23258[] = {{&S16159[30], 6}, {NULL}};
const Segment SGI23259[] = {{&S15030[126], 3}, {NULL}};
const Segment SGI23260[] = {{&S15030[92], 5}, {NULL}};
const Segment SGI23261[] = {{&S16159[112], 3}, {NULL}};
const Segment SGI23262[] = {{&S16159[78], 5}, {NULL}};
const Segment SGI23263[] = {{&S15031[44], 24}, {NULL}};
const Segment SGI23264[] = {{&S15031[44], 35}, {NULL}};
const Segment SGI23265[] = {{&S15031[30], 6}, {NULL}};
const Segment SGI23266[] = {{&S16160[44], 7}, {NULL}};
const Segment SGI23267[] = {{&S16160[44], 18}, {NULL}};
const Segment SGI23268[] = {{&S16160[30], 6}, {NULL}};
const Segment SGI23269[] = {{&S15031[127], 3}, {NULL}};
const Segment SGI23270[] = {{&S15031[93], 5}, {NULL}};
const Segment SGI23271[] = {{&S16160[110], 3}, {NULL}};
const Segment SGI23272[] = {{&S16160[76], 5}, {NULL}};
const Segment SGI23273[] = {{&S15033[44], 19}, {NULL}};
const Segment SGI23274[] = {{&S15033[44], 30}, {NULL}};
const Segment SGI23275[] = {{&S15033[30], 6}, {NULL}};
const Segment SGI23276[] = {{&S15033[122], 3}, {NULL}};
const Segment SGI23277[] = {{&S15033[88], 5}, {NULL}};
const Segment SGI23278[] = {{&S15034[44], 14}, {NULL}};
const Segment SGI23279[] = {{&S15034[44], 25}, {NULL}};
const Segment SGI23280[] = {{&S15034[30], 6}, {NULL}};
const Segment SGI23281[] = {{&S15034[86], 4}, {NULL}};
const Segment SGI23282[] = {{&S15034[86], 3}, {NULL}};
const Segment SGI23283[] = {{&S15035[44], 24}, {NULL}};
const Segment SGI23284[] = {{&S15035[44], 35}, {NULL}};
const Segment SGI23285[] = {{&S15035[30], 6}, {NULL}};
const Segment SGI23286[] = {{&S15035[127], 3}, {NULL}};
const Segment SGI23287[] = {{&S15035[93], 5}, {NULL}};
const Segment SGI23288[] = {{&S16168[44], 5}, {NULL}};
const Segment SGI23289[] = {{&S16168[44], 16}, {NULL}};
const Segment SGI23290[] = {{&S16168[30], 6}, {NULL}};
const Segment SGI23291[] = {{&S16168[108], 3}, {NULL}};
const Segment SGI23292[] = {{&S16168[74], 5}, {NULL}};
const Segment SGI23293[] = {{&S15037[44], 20}, {NULL}};
const Segment SGI23294[] = {{&S15037[44], 31}, {NULL}};
const Segment SGI23295[] = {{&S15037[30], 6}, {NULL}};
const Segment SGI23296[] = {{&S15037[123], 3}, {NULL}};
const Segment SGI23297[] = {{&S15037[89], 5}, {NULL}};
const Segment SGI23298[] = {{&S15039[44], 16}, {NULL}};
const Segment SGI23299[] = {{&S15039[44], 27}, {NULL}};
const Segment SGI23300[] = {{&S15039[30], 6}, {NULL}};
const Segment SGI23301[] = {{&S15039[119], 3}, {NULL}};
const Segment SGI23302[] = {{&S15039[85], 5}, {NULL}};
const Segment SGI23303[] = {{&S15040[44], 101}, {NULL}};
const Segment SGI23304[] = {{&S15040[44], 112}, {NULL}};
const Segment SGI23305[] = {{&S15040[30], 6}, {NULL}};
const Segment SGI23306[] = {{&S15040[204], 3}, {NULL}};
const Segment SGI23307[] = {{&S15040[170], 5}, {NULL}};
const Segment SGI23308[] = {{&S15042[44], 24}, {NULL}};
const Segment SGI23309[] = {{&S15042[44], 35}, {NULL}};
const Segment SGI23310[] = {{&S15042[30], 6}, {NULL}};
const Segment SGI23311[] = {{&S16177[44], 14}, {NULL}};
const Segment SGI23312[] = {{&S16177[44], 31}, {NULL}};
const Segment SGI23313[] = {{&S16177[30], 6}, {NULL}};
const Segment SGI23314[] = {{&S15042[127], 3}, {NULL}};
const Segment SGI23315[] = {{&S15042[93], 5}, {NULL}};
const Segment SGI23316[] = {{&S16177[123], 3}, {NULL}};
const Segment SGI23317[] = {{&S16177[89], 5}, {NULL}};
const Segment SGI23318[] = {{&S15043[44], 14}, {NULL}};
const Segment SGI23319[] = {{&S15043[44], 25}, {NULL}};
const Segment SGI23320[] = {{&S15043[30], 6}, {NULL}};
const Segment SGI23321[] = {{&S16178[44], 11}, {NULL}};
const Segment SGI23322[] = {{&S16178[44], 22}, {NULL}};
const Segment SGI23323[] = {{&S15043[86], 6}, {NULL}};
const Segment SGI23324[] = {{&S16178[30], 6}, {NULL}};
const Segment SGI23325[] = {{&S16178[114], 3}, {NULL}};
const Segment SGI23326[] = {{&S16178[80], 5}, {NULL}};
const Segment SGI23327[] = {{&S15044[42], 19}, {NULL}};
const Segment SGI23328[] = {{&S15044[30], 6}, {NULL}};
const Segment SGI23329[] = {{&S15044[120], 3}, {NULL}};
const Segment SGI23330[] = {{&S15044[86], 5}, {NULL}};
const Segment SGI23331[] = {{&S16179[44], 6}, {NULL}};
const Segment SGI23332[] = {{&S16179[44], 17}, {NULL}};
const Segment SGI23333[] = {{&S16179[30], 6}, {NULL}};
const Segment SGI23334[] = {{&S16179[109], 3}, {NULL}};
const Segment SGI23335[] = {{&S16179[75], 5}, {NULL}};
const Segment SGI23336[] = {{&S20423[36], 5}, {NULL}};
const Segment SGI23337[] = {{&S20423[31], 4}, {NULL}};
const Segment SGI23338[] = {{&S15049[44], 24}, {NULL}};
const Segment SGI23339[] = {{&S15049[44], 35}, {NULL}};
const Segment SGI23340[] = {{&S15049[30], 6}, {NULL}};
const Segment SGI23341[] = {{&S15049[127], 3}, {NULL}};
const Segment SGI23342[] = {{&S15049[93], 5}, {NULL}};
const Segment SGI23343[] = {{&S16184[44], 22}, {NULL}};
const Segment SGI23344[] = {{&S16184[44], 33}, {NULL}};
const Segment SGI23345[] = {{&S16184[30], 6}, {NULL}};
const Segment SGI23346[] = {{&S16184[125], 3}, {NULL}};
const Segment SGI23347[] = {{&S16184[91], 5}, {NULL}};
const Segment SGI23348[] = {{&S16189[44], 6}, {NULL}};
const Segment SGI23349[] = {{&S16189[44], 17}, {NULL}};
const Segment SGI23350[] = {{&S16189[30], 6}, {NULL}};
const Segment SGI23351[] = {{&S16189[109], 3}, {NULL}};
const Segment SGI23352[] = {{&S16189[75], 5}, {NULL}};
const Segment SGI23353[] = {{&S20437[22], 9}, {NULL}};
const Segment SGI23354[] = {{&S20437[42], 21}, {NULL}};
const Segment SGI23355[] = {{&S20437[30], 4}, {&S20437[50], 13}, {NULL}};
const Segment SGI23356[] = {{&S20437[111], 3}, {NULL}};
const Segment SGI23357[] = {{&S20437[77], 5}, {NULL}};
const Segment SGI23358[] = {{&S16190[44], 12}, {NULL}};
const Segment SGI23359[] = {{&S16190[44], 23}, {NULL}};
const Segment SGI23360[] = {{&S16190[30], 6}, {NULL}};
const Segment SGI23361[] = {{&S16190[115], 3}, {NULL}};
const Segment SGI23362[] = {{&S16190[81], 5}, {NULL}};
const Segment SGI23363[] = {{&S15051[44], 25}, {NULL}};
const Segment SGI23364[] = {{&S15051[30], 6}, {NULL}};
const Segment SGI23365[] = {{&S15052[42], 14}, {NULL}};
const Segment SGI23366[] = {{&S15052[30], 6}, {NULL}};
const Segment SGI23367[] = {{&S15052[115], 3}, {NULL}};
const Segment SGI23368[] = {{&S15052[81], 5}, {NULL}};
const Segment SGI23369[] = {{&S20463[42], 9}, {NULL}};
const Segment SGI23370[] = {{&S20463[42], 26}, {NULL}};
const Segment SGI23371[] = {{&S20463[30], 4}, {NULL}};
const Segment SGI23372[] = {{&S20463[116], 3}, {NULL}};
const Segment SGI23373[] = {{&S20463[82], 5}, {NULL}};
const Segment SGI23374[] = {{&S20504[41], 12}, {NULL}};
const Segment SGI23375[] = {{&S20504[41], 23}, {NULL}};
const Segment SGI23376[] = {{&S20504[30], 4}, {NULL}};
const Segment SGI23377[] = {{&S20504[112], 3}, {NULL}};
const Segment SGI23378[] = {{&S20504[78], 5}, {NULL}};
const Segment SGI23379[] = {{&S16201[38], 8}, {NULL}};
const Segment SGI23380[] = {{&S16201[31], 6}, {NULL}};
const Segment SGI23381[] = {{&S16202[44], 9}, {NULL}};
const Segment SGI23382[] = {{&S16202[44], 20}, {NULL}};
const Segment SGI23383[] = {{&S16202[30], 6}, {NULL}};
const Segment SGI23384[] = {{&S16202[112], 3}, {NULL}};
const Segment SGI23385[] = {{&S16202[78], 5}, {NULL}};
const Segment SGI23386[] = {{&S16205[38], 10}, {NULL}};
const Segment SGI23387[] = {{&S16205[31], 6}, {NULL}};
const Segment SGI23388[] = {{&S16207[44], 12}, {NULL}};
const Segment SGI23389[] = {{&S16207[44], 23}, {NULL}};
const Segment SGI23390[] = {{&S16207[30], 6}, {NULL}};
const Segment SGI23391[] = {{&S16207[115], 3}, {NULL}};
const Segment SGI23392[] = {{&S16207[81], 5}, {NULL}};
const Segment SGI23393[] = {{&S16212[44], 4}, {NULL}};
const Segment SGI23394[] = {{&S16212[44], 15}, {NULL}};
const Segment SGI23395[] = {{&S16212[30], 6}, {NULL}};
const Segment SGI23396[] = {{&S16212[107], 3}, {NULL}};
const Segment SGI23397[] = {{&S16212[73], 5}, {NULL}};
const Segment SGI23398[] = {{&S16216[44], 11}, {NULL}};
const Segment SGI23399[] = {{&S16216[44], 22}, {NULL}};
const Segment SGI23400[] = {{&S16216[30], 6}, {NULL}};
const Segment SGI23401[] = {{&S16216[114], 3}, {NULL}};
const Segment SGI23402[] = {{&S16216[80], 5}, {NULL}};
const Segment SGI23403[] = {{&S16241[44], 10}, {NULL}};
const Segment SGI23404[] = {{&S16241[44], 27}, {NULL}};
const Segment SGI23405[] = {{&S16241[30], 6}, {NULL}};
const Segment SGI23406[] = {{&S16241[119], 3}, {NULL}};
const Segment SGI23407[] = {{&S16241[85], 5}, {NULL}};
const Segment SGI23408[] = {{&S16266[44], 23}, {NULL}};
const Segment SGI23409[] = {{&S16266[44], 34}, {NULL}};
const Segment SGI23410[] = {{&S16266[30], 6}, {NULL}};
const Segment SGI23411[] = {{&S16266[126], 3}, {NULL}};
const Segment SGI23412[] = {{&S16266[92], 5}, {NULL}};
const Segment SGI23413[] = {{&S16267[44], 13}, {NULL}};
const Segment SGI23414[] = {{&S16267[44], 24}, {NULL}};
const Segment SGI23415[] = {{&S16267[30], 6}, {NULL}};
const Segment SGI23416[] = {{&S16267[116], 3}, {NULL}};
const Segment SGI23417[] = {{&S16267[82], 5}, {NULL}};
const Segment SGI23418[] = {{&S16269[44], 6}, {NULL}};
const Segment SGI23419[] = {{&S16269[44], 17}, {NULL}};
const Segment SGI23420[] = {{&S16269[30], 6}, {NULL}};
const Segment SGI23421[] = {{&S16269[109], 3}, {NULL}};
const Segment SGI23422[] = {{&S16269[75], 5}, {NULL}};
const Segment SGI23423[] = {{&S16270[44], 8}, {NULL}};
const Segment SGI23424[] = {{&S16270[44], 19}, {NULL}};
const Segment SGI23425[] = {{&S16270[30], 6}, {NULL}};
const Segment SGI23426[] = {{&S16270[111], 3}, {NULL}};
const Segment SGI23427[] = {{&S16270[77], 5}, {NULL}};
const Segment SGI23428[] = {{&S20610[44], 8}, {NULL}};
const Segment SGI23429[] = {{&S20610[44], 29}, {NULL}};
const Segment SGI23430[] = {{&S20610[30], 6}, {NULL}};
const Segment SGI23431[] = {{&S20610[121], 3}, {NULL}};
const Segment SGI23432[] = {{&S20610[87], 5}, {NULL}};
const Segment SGI23433[] = {{&S15961[44], 9}, {NULL}};
const Segment SGI23434[] = {{&S15961[44], 20}, {NULL}};
const Segment SGI23435[] = {{&S15961[30], 6}, {NULL}};
const Segment SGI23436[] = {{&S15961[112], 3}, {NULL}};
const Segment SGI23437[] = {{&S15961[78], 5}, {NULL}};
const Segment SGI23438[] = {{&S15962[44], 11}, {NULL}};
const Segment SGI23439[] = {{&S15962[44], 22}, {NULL}};
const Segment SGI23440[] = {{&S15962[30], 6}, {NULL}};
const Segment SGI23441[] = {{&S15962[114], 3}, {NULL}};
const Segment SGI23442[] = {{&S15962[80], 5}, {NULL}};
const Segment SGI23443[] = {{&S26116[44], 11}, {NULL}};
const Segment SGI23444[] = {{&S26116[44], 22}, {NULL}};
const Segment SGI23445[] = {{&S26116[30], 6}, {NULL}};
const Segment SGI23446[] = {{&S26116[114], 3}, {NULL}};
const Segment SGI23447[] = {{&S26116[80], 5}, {NULL}};
const Segment SGI23448[] = {{&S15964[44], 23}, {NULL}};
const Segment SGI23449[] = {{&S15964[44], 40}, {NULL}};
const Segment SGI23450[] = {{&S15964[30], 6}, {NULL}};
const Segment SGI23451[] = {{&S15964[132], 3}, {NULL}};
const Segment SGI23452[] = {{&S15964[98], 5}, {NULL}};
const Segment SGI23453[] = {{&S15966[44], 14}, {NULL}};
const Segment SGI23454[] = {{&S15966[44], 25}, {NULL}};
const Segment SGI23455[] = {{&S15966[30], 6}, {NULL}};
const Segment SGI23456[] = {{&S15966[117], 3}, {NULL}};
const Segment SGI23457[] = {{&S15966[83], 5}, {NULL}};
const Segment SGI23458[] = {{&S15967[44], 11}, {NULL}};
const Segment SGI23459[] = {{&S15967[44], 22}, {NULL}};
const Segment SGI23460[] = {{&S15967[30], 6}, {NULL}};
const Segment SGI23461[] = {{&S15967[114], 3}, {NULL}};
const Segment SGI23462[] = {{&S15967[80], 5}, {NULL}};
const Segment SGI23463[] = {{&S16304[44], 4}, {NULL}};
const Segment SGI23464[] = {{&S16304[44], 15}, {NULL}};
const Segment SGI23465[] = {{&S16304[30], 6}, {NULL}};
const Segment SGI23466[] = {{&S16304[107], 3}, {NULL}};
const Segment SGI23467[] = {{&S16304[73], 5}, {NULL}};
const Segment SGI23468[] = {{&S26126[44], 11}, {NULL}};
const Segment SGI23469[] = {{&S26126[44], 22}, {NULL}};
const Segment SGI23470[] = {{&S26126[30], 6}, {NULL}};
const Segment SGI23471[] = {{&S26126[114], 3}, {NULL}};
const Segment SGI23472[] = {{&S26126[80], 5}, {NULL}};
const Segment SGI23473[] = {{&S15972[44], 8}, {NULL}};
const Segment SGI23474[] = {{&S15972[44], 19}, {NULL}};
const Segment SGI23475[] = {{&S15972[30], 6}, {NULL}};
const Segment SGI23476[] = {{&S15972[111], 3}, {NULL}};
const Segment SGI23477[] = {{&S15972[77], 5}, {NULL}};
const Segment SGI23478[] = {{&S16307[44], 6}, {NULL}};
const Segment SGI23479[] = {{&S16307[44], 17}, {NULL}};
const Segment SGI23480[] = {{&S16307[30], 6}, {NULL}};
const Segment SGI23481[] = {{&S16307[109], 3}, {NULL}};
const Segment SGI23482[] = {{&S16307[75], 5}, {NULL}};
const Segment SGI23483[] = {{&S15976[44], 22}, {NULL}};
const Segment SGI23484[] = {{&S15976[30], 6}, {NULL}};
const Segment SGI23485[] = {{&S15976[116], 3}, {NULL}};
const Segment SGI23486[] = {{&S15976[82], 5}, {NULL}};
const Segment SGI23487[] = {{&S15977[44], 5}, {NULL}};
const Segment SGI23488[] = {{&S15977[44], 16}, {NULL}};
const Segment SGI23489[] = {{&S15977[30], 6}, {NULL}};
const Segment SGI23490[] = {{&S15977[77], 4}, {NULL}};
const Segment SGI23491[] = {{&S15977[77], 3}, {NULL}};
const Segment SGI23492[] = {{&S15978[44], 4}, {NULL}};
const Segment SGI23493[] = {{&S15978[44], 15}, {NULL}};
const Segment SGI23494[] = {{&S15978[30], 6}, {NULL}};
const Segment SGI23495[] = {{&S15978[107], 3}, {NULL}};
const Segment SGI23496[] = {{&S15978[73], 5}, {NULL}};
const Segment SGI23497[] = {{&S15979[38], 8}, {NULL}};
const Segment SGI23498[] = {{&S15979[31], 6}, {NULL}};
const Segment SGI23499[] = {{&S16324[44], 7}, {NULL}};
const Segment SGI23500[] = {{&S16324[44], 18}, {NULL}};
const Segment SGI23501[] = {{&S16324[30], 6}, {NULL}};
const Segment SGI23502[] = {{&S16324[110], 3}, {NULL}};
const Segment SGI23503[] = {{&S16324[76], 5}, {NULL}};
const Segment SGI23504[] = {{&S15988[11], 1}, {NULL}};
const Segment SGI23505[] = {{&S15988[44], 10}, {NULL}};
const Segment SGI23506[] = {{&S15988[30], 6}, {NULL}};
const Segment SGI23507[] = {{&S15988[131], 3}, {NULL}};
const Segment SGI23508[] = {{&S15988[97], 5}, {NULL}};
const Segment SGI23509[] = {{&S15990[44], 12}, {NULL}};
const Segment SGI23510[] = {{&S15990[44], 23}, {NULL}};
const Segment SGI23511[] = {{&S15990[30], 6}, {NULL}};
const Segment SGI23512[] = {{&S15990[115], 3}, {NULL}};
const Segment SGI23513[] = {{&S15990[81], 5}, {NULL}};
const Segment SGI23514[] = {{&S15991[44], 23}, {NULL}};
const Segment SGI23515[] = {{&S15991[44], 40}, {NULL}};
const Segment SGI23516[] = {{&S15991[30], 6}, {NULL}};
const Segment SGI23517[] = {{&S15991[132], 3}, {NULL}};
const Segment SGI23518[] = {{&S15991[98], 5}, {NULL}};
const Segment SGI23519[] = {{&S16336[44], 12}, {NULL}};
const Segment SGI23520[] = {{&S16336[44], 23}, {NULL}};
const Segment SGI23521[] = {{&S16336[30], 6}, {NULL}};
const Segment SGI23522[] = {{&S16336[115], 3}, {NULL}};
const Segment SGI23523[] = {{&S16336[81], 5}, {NULL}};
const Segment SGI23524[] = {{&S15997[44], 13}, {NULL}};
const Segment SGI23525[] = {{&S15997[44], 24}, {NULL}};
const Segment SGI23526[] = {{&S15997[30], 6}, {NULL}};
const Segment SGI23527[] = {{&S15997[116], 3}, {NULL}};
const Segment SGI23528[] = {{&S15997[82], 5}, {NULL}};
const Segment SGI23529[] = {{&S16341[44], 25}, {NULL}};
const Segment SGI23530[] = {{&S16341[44], 36}, {NULL}};
const Segment SGI23531[] = {{&S16341[30], 6}, {NULL}};
const Segment SGI23532[] = {{&S16341[128], 3}, {NULL}};
const Segment SGI23533[] = {{&S16341[94], 5}, {NULL}};
const Segment SGI23534[] = {{&S16007[44], 13}, {NULL}};
const Segment SGI23535[] = {{&S16007[44], 24}, {NULL}};
const Segment SGI23536[] = {{&S16007[30], 6}, {NULL}};
const Segment SGI23537[] = {{&S16007[116], 3}, {NULL}};
const Segment SGI23538[] = {{&S16007[82], 5}, {NULL}};
const Segment SGI23539[] = {{&S16354[44], 6}, {NULL}};
const Segment SGI23540[] = {{&S16354[44], 17}, {NULL}};
const Segment SGI23541[] = {{&S16354[30], 6}, {NULL}};
const Segment SGI23542[] = {{&S16354[109], 3}, {NULL}};
const Segment SGI23543[] = {{&S16354[75], 5}, {NULL}};
const Segment SGI23544[] = {{&S16015[44], 8}, {NULL}};
const Segment SGI23545[] = {{&S16015[44], 19}, {NULL}};
const Segment SGI23546[] = {{&S16015[30], 6}, {NULL}};
const Segment SGI23547[] = {{&S16015[111], 3}, {NULL}};
const Segment SGI23548[] = {{&S16015[77], 5}, {NULL}};
const Segment SGI23549[] = {{&S16016[44], 6}, {NULL}};
const Segment SGI23550[] = {{&S16016[44], 17}, {NULL}};
const Segment SGI23551[] = {{&S16016[30], 6}, {NULL}};
const Segment SGI23552[] = {{&S16016[109], 3}, {NULL}};
const Segment SGI23553[] = {{&S16016[75], 5}, {NULL}};
const Segment SGI23554[] = {{&S16018[44], 13}, {NULL}};
const Segment SGI23555[] = {{&S16360[44], 16}, {NULL}};
const Segment SGI23556[] = {{&S16018[30], 6}, {NULL}};
const Segment SGI23557[] = {{&S16360[44], 27}, {NULL}};
const Segment SGI23558[] = {{&S16360[30], 6}, {NULL}};
const Segment SGI23559[] = {{&S16018[105], 3}, {NULL}};
const Segment SGI23560[] = {{&S16018[71], 5}, {NULL}};
const Segment SGI23561[] = {{&S16360[119], 3}, {NULL}};
const Segment SGI23562[] = {{&S16360[85], 5}, {NULL}};
const Segment SGI23563[] = {{&S16365[44], 14}, {NULL}};
const Segment SGI23564[] = {{&S16365[44], 25}, {NULL}};
const Segment SGI23565[] = {{&S16365[30], 6}, {NULL}};
const Segment SGI23566[] = {{&S16365[117], 3}, {NULL}};
const Segment SGI23567[] = {{&S16365[83], 5}, {NULL}};
const Segment SGI23568[] = {{&S16026[44], 7}, {NULL}};
const Segment SGI23569[] = {{&S16026[44], 18}, {NULL}};
const Segment SGI23570[] = {{&S16026[30], 6}, {NULL}};
const Segment SGI23571[] = {{&S16026[110], 3}, {NULL}};
const Segment SGI23572[] = {{&S16026[76], 5}, {NULL}};
const Segment SGI23573[] = {{&S16029[44], 8}, {NULL}};
const Segment SGI23574[] = {{&S16029[44], 19}, {NULL}};
const Segment SGI23575[] = {{&S16029[30], 6}, {NULL}};
const Segment SGI23576[] = {{&S16029[111], 3}, {NULL}};
const Segment SGI23577[] = {{&S16029[77], 5}, {NULL}};
const Segment SGI23578[] = {{&S16374[44], 14}, {NULL}};
const Segment SGI23579[] = {{&S16374[44], 25}, {NULL}};
const Segment SGI23580[] = {{&S16374[30], 6}, {NULL}};
const Segment SGI23581[] = {{&S16374[117], 3}, {NULL}};
const Segment SGI23582[] = {{&S16374[83], 5}, {NULL}};
const Segment SGI23583[] = {{&S16388[44], 7}, {NULL}};
const Segment SGI23584[] = {{&S16388[44], 18}, {NULL}};
const Segment SGI23585[] = {{&S16388[30], 6}, {NULL}};
const Segment SGI23586[] = {{&S16388[110], 3}, {NULL}};
const Segment SGI23587[] = {{&S16388[76], 5}, {NULL}};
const Segment SGI23588[] = {{&S16409[44], 8}, {NULL}};
const Segment SGI23589[] = {{&S16409[44], 19}, {NULL}};
const Segment SGI23590[] = {{&S16409[30], 6}, {NULL}};
const Segment SGI23591[] = {{&S16409[111], 3}, {NULL}};
const Segment SGI23592[] = {{&S16409[77], 5}, {NULL}};
const Segment SGI23593[] = {{&S16033[44], 15}, {NULL}};
const Segment SGI23594[] = {{&S16033[44], 26}, {NULL}};
const Segment SGI23595[] = {{&S16033[30], 6}, {NULL}};
const Segment SGI23596[] = {{&S16033[118], 3}, {NULL}};
const Segment SGI23597[] = {{&S16033[84], 5}, {NULL}};
const Segment SGI23598[] = {{&S16044[44], 12}, {NULL}};
const Segment SGI23599[] = {{&S16044[44], 23}, {NULL}};
const Segment SGI23600[] = {{&S16044[30], 6}, {NULL}};
const Segment SGI23601[] = {{&S16044[115], 3}, {NULL}};
const Segment SGI23602[] = {{&S16044[81], 5}, {NULL}};
const Segment SGI23603[] = {{&S16047[44], 6}, {NULL}};
const Segment SGI23604[] = {{&S16047[44], 17}, {NULL}};
const Segment SGI23605[] = {{&S16047[30], 6}, {NULL}};
const Segment SGI23606[] = {{&S16047[109], 3}, {NULL}};
const Segment SGI23607[] = {{&S16047[75], 5}, {NULL}};
const Segment SGI23608[] = {{&S16051[44], 9}, {NULL}};
const Segment SGI23609[] = {{&S16051[44], 20}, {NULL}};
const Segment SGI23610[] = {{&S16051[30], 6}, {NULL}};
const Segment SGI23611[] = {{&S16051[112], 3}, {NULL}};
const Segment SGI23612[] = {{&S16051[78], 5}, {NULL}};
const Segment SGI23613[] = {{&S16053[44], 7}, {NULL}};
const Segment SGI23614[] = {{&S16053[44], 18}, {NULL}};
const Segment SGI23615[] = {{&S16053[30], 6}, {NULL}};
const Segment SGI23616[] = {{&S16053[110], 3}, {NULL}};
const Segment SGI23617[] = {{&S16053[76], 5}, {NULL}};
const Segment SGI23618[] = {{&S16064[44], 16}, {NULL}};
const Segment SGI23619[] = {{&S16064[30], 6}, {NULL}};
const Segment SGI23620[] = {{&S16064[108], 3}, {NULL}};
const Segment SGI23621[] = {{&S16064[74], 5}, {NULL}};
const Segment SGI23622[] = {{&S16455[44], 6}, {NULL}};
const Segment SGI23623[] = {{&S16455[44], 23}, {NULL}};
const Segment SGI23624[] = {{&S16455[30], 6}, {NULL}};
const Segment SGI23625[] = {{&S16455[115], 3}, {NULL}};
const Segment SGI23626[] = {{&S16455[81], 5}, {NULL}};
const Segment SGI23627[] = {{&S16068[44], 9}, {NULL}};
const Segment SGI23628[] = {{&S16068[44], 26}, {NULL}};
const Segment SGI23629[] = {{&S16068[30], 6}, {NULL}};
const Segment SGI23630[] = {{&S16068[118], 3}, {NULL}};
const Segment SGI23631[] = {{&S16068[84], 5}, {NULL}};
const Segment SGI23632[] = {{&S16069[38], 13}, {NULL}};
const Segment SGI23633[] = {{&S16069[31], 6}, {NULL}};
const Segment SGI23634[] = {{&S16074[44], 4}, {NULL}};
const Segment SGI23635[] = {{&S16074[44], 15}, {NULL}};
const Segment SGI23636[] = {{&S16074[30], 6}, {NULL}};
const Segment SGI23637[] = {{&S16074[107], 3}, {NULL}};
const Segment SGI23638[] = {{&S16074[73], 5}, {NULL}};
const Segment SGI23639[] = {{&S16075[44], 10}, {NULL}};
const Segment SGI23640[] = {{&S16075[44], 21}, {NULL}};
const Segment SGI23641[] = {{&S16075[30], 6}, {NULL}};
const Segment SGI23642[] = {{&S16075[113], 3}, {NULL}};
const Segment SGI23643[] = {{&S16075[79], 5}, {NULL}};
const Segment SGI23644[] = {{&S16089[44], 7}, {NULL}};
const Segment SGI23645[] = {{&S16089[44], 24}, {NULL}};
const Segment SGI23646[] = {{&S16089[30], 6}, {NULL}};
const Segment SGI23647[] = {{&S16089[116], 3}, {NULL}};
const Segment SGI23648[] = {{&S16089[82], 5}, {NULL}};
const Segment SGI23649[] = {{&S16093[44], 4}, {NULL}};
const Segment SGI23650[] = {{&S16093[44], 15}, {NULL}};
const Segment SGI23651[] = {{&S16093[30], 6}, {NULL}};
const Segment SGI23652[] = {{&S16093[107], 3}, {NULL}};
const Segment SGI23653[] = {{&S16093[73], 5}, {NULL}};
const Segment SGI23654[] = {{&S16095[44], 9}, {NULL}};
const Segment SGI23655[] = {{&S16095[44], 26}, {NULL}};
const Segment SGI23656[] = {{&S16095[30], 6}, {NULL}};
const Segment SGI23657[] = {{&S16095[118], 3}, {NULL}};
const Segment SGI23658[] = {{&S16095[84], 5}, {NULL}};
const Segment SGI23659[] = {{&S16097[44], 10}, {NULL}};
const Segment SGI23660[] = {{&S16097[44], 21}, {NULL}};
const Segment SGI23661[] = {{&S16097[30], 6}, {NULL}};
const Segment SGI23662[] = {{&S16097[113], 3}, {NULL}};
const Segment SGI23663[] = {{&S16097[79], 5}, {NULL}};
const Segment SGI23664[] = {{&S16100[44], 24}, {NULL}};
const Segment SGI23665[] = {{&S16100[44], 35}, {NULL}};
const Segment SGI23666[] = {{&S16100[30], 6}, {NULL}};
const Segment SGI23667[] = {{&S16100[127], 3}, {NULL}};
const Segment SGI23668[] = {{&S16100[93], 5}, {NULL}};
const Segment SGI23669[] = {{&S16104[44], 6}, {NULL}};
const Segment SGI23670[] = {{&S16104[44], 17}, {NULL}};
const Segment SGI23671[] = {{&S16104[30], 6}, {NULL}};
const Segment SGI23672[] = {{&S16104[109], 3}, {NULL}};
const Segment SGI23673[] = {{&S16104[75], 5}, {NULL}};
const Segment SGI23674[] = {{&S16117[44], 9}, {NULL}};
const Segment SGI23675[] = {{&S16117[44], 20}, {NULL}};
const Segment SGI23676[] = {{&S16117[30], 6}, {NULL}};
const Segment SGI23677[] = {{&S16117[112], 3}, {NULL}};
const Segment SGI23678[] = {{&S16117[78], 5}, {NULL}};
const Segment SGI23679[] = {{&S16121[44], 9}, {NULL}};
const Segment SGI23680[] = {{&S16121[44], 20}, {NULL}};
const Segment SGI23681[] = {{&S16121[30], 6}, {NULL}};
const Segment SGI23682[] = {{&S16121[112], 3}, {NULL}};
const Segment SGI23683[] = {{&S16121[78], 5}, {NULL}};
const Segment SGI23684[] = {{&S16122[44], 14}, {NULL}};
const Segment SGI23685[] = {{&S16122[44], 31}, {NULL}};
const Segment SGI23686[] = {{&S16122[30], 6}, {NULL}};
const Segment SGI23687[] = {{&S16122[123], 3}, {NULL}};
const Segment SGI23688[] = {{&S16122[89], 5}, {NULL}};
const Segment SGI23689[] = {{&S16532[44], 8}, {NULL}};
const Segment SGI23690[] = {{&S16532[44], 25}, {NULL}};
const Segment SGI23691[] = {{&S16532[30], 6}, {NULL}};
const Segment SGI23692[] = {{&S16126[44], 13}, {NULL}};
const Segment SGI23693[] = {{&S16532[117], 3}, {NULL}};
const Segment SGI23694[] = {{&S16126[44], 24}, {NULL}};
const Segment SGI23695[] = {{&S16126[30], 6}, {NULL}};
const Segment SGI23696[] = {{&S16532[83], 5}, {NULL}};
const Segment SGI23697[] = {{&S16126[116], 3}, {NULL}};
const Segment SGI23698[] = {{&S16126[82], 5}, {NULL}};
const Segment SGI23699[] = {{&S16128[44], 11}, {NULL}};
const Segment SGI23700[] = {{&S16128[44], 28}, {NULL}};
const Segment SGI23701[] = {{&S16128[30], 6}, {NULL}};
const Segment SGI23702[] = {{&S16128[120], 3}, {NULL}};
const Segment SGI23703[] = {{&S16128[86], 5}, {NULL}};
const Segment SGI23704[] = {{&S16129[44], 20}, {NULL}};
const Segment SGI23705[] = {{&S16129[44], 37}, {NULL}};
const Segment SGI23706[] = {{&S16129[30], 6}, {NULL}};
const Segment SGI23707[] = {{&S16537[44], 7}, {NULL}};
const Segment SGI23708[] = {{&S16537[44], 18}, {NULL}};
const Segment SGI23709[] = {{&S16537[30], 6}, {NULL}};
const Segment SGI23710[] = {{&S16129[129], 3}, {NULL}};
const Segment SGI23711[] = {{&S16537[110], 3}, {NULL}};
const Segment SGI23712[] = {{&S16129[95], 5}, {NULL}};
const Segment SGI23713[] = {{&S16537[76], 5}, {NULL}};
const Segment SGI23714[] = {{&S16131[44], 21}, {NULL}};
const Segment SGI23715[] = {{&S16131[44], 32}, {NULL}};
const Segment SGI23716[] = {{&S16131[30], 6}, {NULL}};
const Segment SGI23717[] = {{&S16131[124], 3}, {NULL}};
const Segment SGI23718[] = {{&S16131[90], 5}, {NULL}};
const Segment SGI23719[] = {{&S16132[44], 16}, {NULL}};
const Segment SGI23720[] = {{&S16132[44], 27}, {NULL}};
const Segment SGI23721[] = {{&S16132[30], 6}, {NULL}};
const Segment SGI23722[] = {{&S16132[119], 3}, {NULL}};
const Segment SGI23723[] = {{&S16132[85], 5}, {NULL}};
const Segment SGI23724[] = {{&S16542[44], 9}, {NULL}};
const Segment SGI23725[] = {{&S16542[44], 20}, {NULL}};
const Segment SGI23726[] = {{&S16542[30], 6}, {NULL}};
const Segment SGI23727[] = {{&S16542[112], 3}, {NULL}};
const Segment SGI23728[] = {{&S16542[78], 5}, {NULL}};
const Segment SGI23729[] = {{&S16133[44], 6}, {NULL}};
const Segment SGI23730[] = {{&S16133[44], 24}, {NULL}};
const Segment SGI23731[] = {{&S16133[30], 6}, {NULL}};
const Segment SGI23732[] = {{&S16133[116], 3}, {NULL}};
const Segment SGI23733[] = {{&S16133[82], 5}, {NULL}};
const Segment SGI23734[] = {{&S16147[44], 15}, {NULL}};
const Segment SGI23735[] = {{&S16147[44], 26}, {NULL}};
const Segment SGI23736[] = {{&S16147[30], 6}, {NULL}};
const Segment SGI23737[] = {{&S16147[118], 3}, {NULL}};
const Segment SGI23738[] = {{&S16147[84], 5}, {NULL}};
const Segment SGI23739[] = {{&S16148[44], 8}, {NULL}};
const Segment SGI23740[] = {{&S16148[44], 19}, {NULL}};
const Segment SGI23741[] = {{&S16148[30], 6}, {NULL}};
const Segment SGI23742[] = {{&S16148[111], 3}, {NULL}};
const Segment SGI23743[] = {{&S16148[77], 5}, {NULL}};
const Segment SGI23744[] = {{&S16150[44], 16}, {NULL}};
const Segment SGI23745[] = {{&S16150[44], 33}, {NULL}};
const Segment SGI23746[] = {{&S16150[30], 6}, {NULL}};
const Segment SGI23747[] = {{&S16150[126], 3}, {NULL}};
const Segment SGI23748[] = {{&S16150[91], 6}, {NULL}};
const Segment SGI23749[] = {{&S16151[44], 14}, {NULL}};
const Segment SGI23750[] = {{&S16151[44], 25}, {NULL}};
const Segment SGI23751[] = {{&S16151[30], 6}, {NULL}};
const Segment SGI23752[] = {{&S16151[117], 3}, {NULL}};
const Segment SGI23753[] = {{&S16151[83], 5}, {NULL}};
const Segment SGI23754[] = {{&S16158[44], 22}, {NULL}};
const Segment SGI23755[] = {{&S16158[44], 39}, {NULL}};
const Segment SGI23756[] = {{&S16158[30], 6}, {NULL}};
const Segment SGI23757[] = {{&S16158[131], 3}, {NULL}};
const Segment SGI23758[] = {{&S16158[97], 5}, {NULL}};
const Segment SGI23759[] = {{&S16161[44], 7}, {NULL}};
const Segment SGI23760[] = {{&S16161[44], 28}, {NULL}};
const Segment SGI23761[] = {{&S16161[30], 6}, {NULL}};
const Segment SGI23762[] = {{&S16161[120], 3}, {NULL}};
const Segment SGI23763[] = {{&S16161[86], 5}, {NULL}};
const Segment SGI23764[] = {{&S16162[44], 7}, {NULL}};
const Segment SGI23765[] = {{&S16162[44], 26}, {NULL}};
const Segment SGI23766[] = {{&S16162[30], 6}, {NULL}};
const Segment SGI23767[] = {{&S16162[118], 3}, {NULL}};
const Segment SGI23768[] = {{&S16162[84], 5}, {NULL}};
const Segment SGI23769[] = {{&S16164[38], 15}, {NULL}};
const Segment SGI23770[] = {{&S16164[31], 6}, {NULL}};
const Segment SGI23771[] = {{&S16169[44], 5}, {NULL}};
const Segment SGI23772[] = {{&S16169[44], 16}, {NULL}};
const Segment SGI23773[] = {{&S16169[30], 6}, {NULL}};
const Segment SGI23774[] = {{&S16169[108], 3}, {NULL}};
const Segment SGI23775[] = {{&S16169[74], 5}, {NULL}};
const Segment SGI23776[] = {{&S16170[44], 15}, {NULL}};
const Segment SGI23777[] = {{&S16170[44], 32}, {NULL}};
const Segment SGI23778[] = {{&S16170[30], 6}, {NULL}};
const Segment SGI23779[] = {{&S16170[124], 3}, {NULL}};
const Segment SGI23780[] = {{&S16170[90], 5}, {NULL}};
const Segment SGI23781[] = {{&S16172[44], 16}, {NULL}};
const Segment SGI23782[] = {{&S16172[44], 33}, {NULL}};
const Segment SGI23783[] = {{&S16172[30], 6}, {NULL}};
const Segment SGI23784[] = {{&S16172[126], 3}, {NULL}};
const Segment SGI23785[] = {{&S16172[91], 6}, {NULL}};
const Segment SGI23786[] = {{&S16173[44], 24}, {NULL}};
const Segment SGI23787[] = {{&S16173[44], 35}, {NULL}};
const Segment SGI23788[] = {{&S16173[30], 6}, {NULL}};
const Segment SGI23789[] = {{&S16173[127], 3}, {NULL}};
const Segment SGI23790[] = {{&S16173[93], 5}, {NULL}};
const Segment SGI23791[] = {{&S16174[44], 4}, {NULL}};
const Segment SGI23792[] = {{&S16174[44], 15}, {NULL}};
const Segment SGI23793[] = {{&S16174[30], 6}, {NULL}};
const Segment SGI23794[] = {{&S16174[107], 3}, {NULL}};
const Segment SGI23795[] = {{&S16174[73], 5}, {NULL}};
const Segment SGI23796[] = {{&S16176[44], 7}, {NULL}};
const Segment SGI23797[] = {{&S16176[44], 18}, {NULL}};
const Segment SGI23798[] = {{&S16176[30], 6}, {NULL}};
const Segment SGI23799[] = {{&S16176[110], 3}, {NULL}};
const Segment SGI23800[] = {{&S16176[76], 5}, {NULL}};
const Segment SGI23801[] = {{&S16603[44], 7}, {NULL}};
const Segment SGI23802[] = {{&S16603[44], 18}, {NULL}};
const Segment SGI23803[] = {{&S16603[30], 6}, {NULL}};
const Segment SGI23804[] = {{&S16603[110], 3}, {NULL}};
const Segment SGI23805[] = {{&S16603[76], 5}, {NULL}};
const Segment SGI23806[] = {{&S16182[44], 9}, {NULL}};
const Segment SGI23807[] = {{&S16182[44], 20}, {NULL}};
const Segment SGI23808[] = {{&S16182[30], 6}, {NULL}};
const Segment SGI23809[] = {{&S16182[112], 3}, {NULL}};
const Segment SGI23810[] = {{&S16182[78], 5}, {NULL}};
const Segment SGI23811[] = {{&S16187[44], 10}, {NULL}};
const Segment SGI23812[] = {{&S16187[30], 6}, {NULL}};
const Segment SGI23813[] = {{&S16187[104], 3}, {NULL}};
const Segment SGI23814[] = {{&S16187[70], 5}, {NULL}};
const Segment SGI23815[] = {{&S16191[44], 7}, {NULL}};
const Segment SGI23816[] = {{&S16191[44], 18}, {NULL}};
const Segment SGI23817[] = {{&S16191[30], 6}, {NULL}};
const Segment SGI23818[] = {{&S16191[79], 4}, {NULL}};
const Segment SGI23819[] = {{&S16191[79], 3}, {NULL}};
const Segment SGI23820[] = {{&S16192[44], 12}, {NULL}};
const Segment SGI23821[] = {{&S16192[44], 68}, {NULL}};
const Segment SGI23822[] = {{&S16192[30], 6}, {NULL}};
const Segment SGI23823[] = {{&S16192[161], 3}, {NULL}};
const Segment SGI23824[] = {{&S16192[127], 5}, {NULL}};
const Segment SGI23825[] = {{&S16195[44], 7}, {NULL}};
const Segment SGI23826[] = {{&S16195[44], 28}, {NULL}};
const Segment SGI23827[] = {{&S16195[30], 6}, {NULL}};
const Segment SGI23828[] = {{&S16195[89], 4}, {NULL}};
const Segment SGI23829[] = {{&S16195[89], 3}, {NULL}};
const Segment SGI23830[] = {{&S16198[44], 6}, {NULL}};
const Segment SGI23831[] = {{&S16198[44], 17}, {NULL}};
const Segment SGI23832[] = {{&S16198[30], 6}, {NULL}};
const Segment SGI23833[] = {{&S16198[109], 3}, {NULL}};
const Segment SGI23834[] = {{&S16198[75], 5}, {NULL}};
const Segment SGI23835[] = {{&S16208[44], 9}, {NULL}};
const Segment SGI23836[] = {{&S16208[44], 20}, {NULL}};
const Segment SGI23837[] = {{&S16208[30], 6}, {NULL}};
const Segment SGI23838[] = {{&S16208[112], 3}, {NULL}};
const Segment SGI23839[] = {{&S16208[78], 5}, {NULL}};
const Segment SGI23840[] = {{&S16209[44], 7}, {NULL}};
const Segment SGI23841[] = {{&S16209[44], 22}, {NULL}};
const Segment SGI23842[] = {{&S16209[30], 6}, {NULL}};
const Segment SGI23843[] = {{&S16209[114], 3}, {NULL}};
const Segment SGI23844[] = {{&S16209[80], 5}, {NULL}};
const Segment SGI23845[] = {{&S16213[38], 6}, {NULL}};
const Segment SGI23846[] = {{&S16213[31], 6}, {NULL}};
const Segment SGI23847[] = {{&S16218[44], 13}, {NULL}};
const Segment SGI23848[] = {{&S16218[44], 24}, {NULL}};
const Segment SGI23849[] = {{&S16218[30], 6}, {NULL}};
const Segment SGI23850[] = {{&S16218[116], 3}, {NULL}};
const Segment SGI23851[] = {{&S16218[82], 5}, {NULL}};
const Segment SGI23852[] = {{&S16219[44], 10}, {NULL}};
const Segment SGI23853[] = {{&S16219[44], 21}, {NULL}};
const Segment SGI23854[] = {{&S16219[30], 6}, {NULL}};
const Segment SGI23855[] = {{&S16219[113], 3}, {NULL}};
const Segment SGI23856[] = {{&S16219[79], 5}, {NULL}};
const Segment SGI23857[] = {{&S16220[44], 16}, {NULL}};
const Segment SGI23858[] = {{&S16220[44], 27}, {NULL}};
const Segment SGI23859[] = {{&S16220[30], 6}, {NULL}};
const Segment SGI23860[] = {{&S16220[119], 3}, {NULL}};
const Segment SGI23861[] = {{&S16220[85], 5}, {NULL}};
const Segment SGI23862[] = {{&S16224[44], 7}, {NULL}};
const Segment SGI23863[] = {{&S16224[44], 24}, {NULL}};
const Segment SGI23864[] = {{&S16224[30], 6}, {NULL}};
const Segment SGI23865[] = {{&S16224[116], 3}, {NULL}};
const Segment SGI23866[] = {{&S16224[82], 5}, {NULL}};
const Segment SGI23867[] = {{&S16226[44], 11}, {NULL}};
const Segment SGI23868[] = {{&S16226[44], 27}, {NULL}};
const Segment SGI23869[] = {{&S16226[30], 6}, {NULL}};
const Segment SGI23870[] = {{&S16226[119], 3}, {NULL}};
const Segment SGI23871[] = {{&S16226[85], 5}, {NULL}};
const Segment SGI23872[] = {{&S16228[44], 7}, {NULL}};
const Segment SGI23873[] = {{&S16228[44], 18}, {NULL}};
const Segment SGI23874[] = {{&S16228[30], 6}, {NULL}};
const Segment SGI23875[] = {{&S16228[110], 3}, {NULL}};
const Segment SGI23876[] = {{&S16228[76], 5}, {NULL}};
const Segment SGI23877[] = {{&S16230[44], 5}, {NULL}};
const Segment SGI23878[] = {{&S16230[44], 16}, {NULL}};
const Segment SGI23879[] = {{&S16230[30], 6}, {NULL}};
const Segment SGI23880[] = {{&S16230[108], 3}, {NULL}};
const Segment SGI23881[] = {{&S16230[74], 5}, {NULL}};
const Segment SGI23882[] = {{&S16235[44], 10}, {NULL}};
const Segment SGI23883[] = {{&S16235[44], 42}, {NULL}};
const Segment SGI23884[] = {{&S16235[30], 6}, {NULL}};
const Segment SGI23885[] = {{&S16235[134], 3}, {NULL}};
const Segment SGI23886[] = {{&S16235[100], 5}, {NULL}};
const Segment SGI23887[] = {{&S16237[44], 21}, {NULL}};
const Segment SGI23888[] = {{&S16237[44], 32}, {NULL}};
const Segment SGI23889[] = {{&S16237[30], 6}, {NULL}};
const Segment SGI23890[] = {{&S16237[124], 3}, {NULL}};
const Segment SGI23891[] = {{&S16237[90], 5}, {NULL}};
const Segment SGI23892[] = {{&S16240[44], 13}, {NULL}};
const Segment SGI23893[] = {{&S16240[44], 24}, {NULL}};
const Segment SGI23894[] = {{&S16240[30], 6}, {NULL}};
const Segment SGI23895[] = {{&S16240[116], 3}, {NULL}};
const Segment SGI23896[] = {{&S16240[82], 5}, {NULL}};
const Segment SGI23897[] = {{&S16242[44], 10}, {NULL}};
const Segment SGI23898[] = {{&S16242[30], 6}, {NULL}};
const Segment SGI23899[] = {{&S16242[115], 3}, {NULL}};
const Segment SGI23900[] = {{&S16242[81], 5}, {NULL}};
const Segment SGI23901[] = {{&S16246[38], 6}, {NULL}};
const Segment SGI23902[] = {{&S16246[31], 6}, {NULL}};
const Segment SGI23903[] = {{&S16247[44], 5}, {NULL}};
const Segment SGI23904[] = {{&S16247[44], 16}, {NULL}};
const Segment SGI23905[] = {{&S16247[30], 6}, {NULL}};
const Segment SGI23906[] = {{&S16247[108], 3}, {NULL}};
const Segment SGI23907[] = {{&S16247[74], 5}, {NULL}};
const Segment SGI23908[] = {{&S16250[44], 24}, {NULL}};
const Segment SGI23909[] = {{&S16250[44], 41}, {NULL}};
const Segment SGI23910[] = {{&S16250[30], 6}, {NULL}};
const Segment SGI23911[] = {{&S16250[134], 3}, {NULL}};
const Segment SGI23912[] = {{&S16250[99], 6}, {NULL}};
const Segment SGI23913[] = {{&S16252[44], 8}, {NULL}};
const Segment SGI23914[] = {{&S16252[44], 19}, {NULL}};
const Segment SGI23915[] = {{&S16252[30], 6}, {NULL}};
const Segment SGI23916[] = {{&S16252[111], 3}, {NULL}};
const Segment SGI23917[] = {{&S16252[77], 5}, {NULL}};
const Segment SGI23918[] = {{&S16254[44], 22}, {NULL}};
const Segment SGI23919[] = {{&S16254[30], 6}, {NULL}};
const Segment SGI23920[] = {{&S16254[116], 3}, {NULL}};
const Segment SGI23921[] = {{&S16254[82], 5}, {NULL}};
const Segment SGI23922[] = {{&S16256[44], 8}, {NULL}};
const Segment SGI23923[] = {{&S16256[44], 19}, {NULL}};
const Segment SGI23924[] = {{&S16256[30], 6}, {NULL}};
const Segment SGI23925[] = {{&S16256[111], 3}, {NULL}};
const Segment SGI23926[] = {{&S16256[77], 5}, {NULL}};
const Segment SGI23927[] = {{&S16263[44], 9}, {NULL}};
const Segment SGI23928[] = {{&S16263[44], 34}, {NULL}};
const Segment SGI23929[] = {{&S16263[30], 6}, {NULL}};
const Segment SGI23930[] = {{&S16263[126], 3}, {NULL}};
const Segment SGI23931[] = {{&S16263[92], 5}, {NULL}};
const Segment SGI23932[] = {{&S16265[44], 6}, {NULL}};
const Segment SGI23933[] = {{&S16265[44], 17}, {NULL}};
const Segment SGI23934[] = {{&S16265[30], 6}, {NULL}};
const Segment SGI23935[] = {{&S16265[109], 3}, {NULL}};
const Segment SGI23936[] = {{&S16265[75], 5}, {NULL}};
const Segment SGI23937[] = {{&S16268[44], 6}, {NULL}};
const Segment SGI23938[] = {{&S16268[44], 17}, {NULL}};
const Segment SGI23939[] = {{&S16268[30], 6}, {NULL}};
const Segment SGI23940[] = {{&S16268[78], 4}, {NULL}};
const Segment SGI23941[] = {{&S16268[78], 3}, {NULL}};
const Segment SGI23942[] = {{&S16272[44], 13}, {NULL}};
const Segment SGI23943[] = {{&S16272[44], 30}, {NULL}};
const Segment SGI23944[] = {{&S16272[30], 6}, {NULL}};
const Segment SGI23945[] = {{&S16272[122], 3}, {NULL}};
const Segment SGI23946[] = {{&S16272[88], 5}, {NULL}};
const Segment SGI23947[] = {{&S16274[44], 9}, {NULL}};
const Segment SGI23948[] = {{&S16274[44], 36}, {NULL}};
const Segment SGI23949[] = {{&S16274[30], 6}, {NULL}};
const Segment SGI23950[] = {{&S16274[128], 3}, {NULL}};
const Segment SGI23951[] = {{&S16274[94], 5}, {NULL}};
const Segment SGI23952[] = {{&S16277[44], 6}, {NULL}};
const Segment SGI23953[] = {{&S16277[44], 17}, {NULL}};
const Segment SGI23954[] = {{&S16277[30], 6}, {NULL}};
const Segment SGI23955[] = {{&S16277[109], 3}, {NULL}};
const Segment SGI23956[] = {{&S16277[75], 5}, {NULL}};
const Segment SGI23957[] = {{&S16286[44], 17}, {NULL}};
const Segment SGI23958[] = {{&S16286[44], 28}, {NULL}};
const Segment SGI23959[] = {{&S16286[30], 6}, {NULL}};
const Segment SGI23960[] = {{&S16286[120], 3}, {NULL}};
const Segment SGI23961[] = {{&S16286[86], 5}, {NULL}};
const Segment SGI23962[] = {{&S16621[44], 12}, {NULL}};
const Segment SGI23963[] = {{&S16621[44], 29}, {NULL}};
const Segment SGI23964[] = {{&S16621[30], 6}, {NULL}};
const Segment SGI23965[] = {{&S16621[121], 3}, {NULL}};
const Segment SGI23966[] = {{&S16621[87], 5}, {NULL}};
const Segment SGI23967[] = {{&S16623[44], 13}, {NULL}};
const Segment SGI23968[] = {{&S16623[44], 24}, {NULL}};
const Segment SGI23969[] = {{&S16623[30], 6}, {NULL}};
const Segment SGI23970[] = {{&S16623[116], 3}, {NULL}};
const Segment SGI23971[] = {{&S16623[82], 5}, {NULL}};
const Segment SGI23972[] = {{&S16630[44], 10}, {NULL}};
const Segment SGI23973[] = {{&S16630[44], 27}, {NULL}};
const Segment SGI23974[] = {{&S16630[30], 6}, {NULL}};
const Segment SGI23975[] = {{&S16630[119], 3}, {NULL}};
const Segment SGI23976[] = {{&S16630[85], 5}, {NULL}};
const Segment SGI23977[] = {{&S16649[44], 11}, {NULL}};
const Segment SGI23978[] = {{&S16649[44], 22}, {NULL}};
const Segment SGI23979[] = {{&S16649[30], 6}, {NULL}};
const Segment SGI23980[] = {{&S16649[114], 3}, {NULL}};
const Segment SGI23981[] = {{&S16649[80], 5}, {NULL}};
const Segment SGI23982[] = {{&S16669[44], 9}, {NULL}};
const Segment SGI23983[] = {{&S16669[44], 24}, {NULL}};
const Segment SGI23984[] = {{&S16669[30], 6}, {NULL}};
const Segment SGI23985[] = {{&S16669[116], 3}, {NULL}};
const Segment SGI23986[] = {{&S16669[82], 5}, {NULL}};
const Segment SGI23987[] = {{&S16692[44], 15}, {NULL}};
const Segment SGI23988[] = {{&S16692[44], 26}, {NULL}};
const Segment SGI23989[] = {{&S16692[30], 6}, {NULL}};
const Segment SGI23990[] = {{&S16692[118], 3}, {NULL}};
const Segment SGI23991[] = {{&S16692[84], 5}, {NULL}};
const Segment SGI23992[] = {{&S16698[44], 7}, {NULL}};
const Segment SGI23993[] = {{&S16698[44], 18}, {NULL}};
const Segment SGI23994[] = {{&S16698[30], 6}, {NULL}};
const Segment SGI23995[] = {{&S16698[110], 3}, {NULL}};
const Segment SGI23996[] = {{&S16698[76], 5}, {NULL}};
const Segment SGI23997[] = {{&S16737[44], 10}, {NULL}};
const Segment SGI23998[] = {{&S16737[44], 27}, {NULL}};
const Segment SGI23999[] = {{&S16737[30], 6}, {NULL}};
const Segment SGI24000[] = {{&S16737[119], 3}, {NULL}};
const Segment SGI24001[] = {{&S16737[85], 5}, {NULL}};
const Segment SGI24002[] = {{&S16768[44], 6}, {NULL}};
const Segment SGI24003[] = {{&S16768[44], 23}, {NULL}};
const Segment SGI24004[] = {{&S16768[30], 6}, {NULL}};
const Segment SGI24005[] = {{&S16768[115], 3}, {NULL}};
const Segment SGI24006[] = {{&S16768[81], 5}, {NULL}};
const Segment SGI24007[] = {{&S16786[44], 6}, {NULL}};
const Segment SGI24008[] = {{&S16786[44], 17}, {NULL}};
const Segment SGI24009[] = {{&S16786[30], 6}, {NULL}};
const Segment SGI24010[] = {{&S16786[109], 3}, {NULL}};
const Segment SGI24011[] = {{&S16786[75], 5}, {NULL}};
const Segment SGI24012[] = {{&S16287[44], 7}, {NULL}};
const Segment SGI24013[] = {{&S16287[44], 18}, {NULL}};
const Segment SGI24014[] = {{&S16287[30], 6}, {NULL}};
const Segment SGI24015[] = {{&S16287[110], 3}, {NULL}};
const Segment SGI24016[] = {{&S16287[76], 5}, {NULL}};
const Segment SGI24017[] = {{&S16294[44], 20}, {NULL}};
const Segment SGI24018[] = {{&S16294[44], 31}, {NULL}};
const Segment SGI24019[] = {{&S16294[30], 6}, {NULL}};
const Segment SGI24020[] = {{&S16294[123], 3}, {NULL}};
const Segment SGI24021[] = {{&S16294[89], 5}, {NULL}};
const Segment SGI24022[] = {{&S16296[44], 8}, {NULL}};
const Segment SGI24023[] = {{&S16296[44], 18}, {NULL}};
const Segment SGI24024[] = {{&S16296[30], 6}, {NULL}};
const Segment SGI24025[] = {{&S16296[110], 3}, {NULL}};
const Segment SGI24026[] = {{&S16296[76], 5}, {NULL}};
const Segment SGI24027[] = {{&S16300[44], 7}, {NULL}};
const Segment SGI24028[] = {{&S16300[44], 18}, {NULL}};
const Segment SGI24029[] = {{&S16300[30], 6}, {NULL}};
const Segment SGI24030[] = {{&S16300[79], 6}, {NULL}};
const Segment SGI24031[] = {{&S16301[44], 6}, {NULL}};
const Segment SGI24032[] = {{&S16301[44], 17}, {NULL}};
const Segment SGI24033[] = {{&S16301[30], 6}, {NULL}};
const Segment SGI24034[] = {{&S16301[109], 3}, {NULL}};
const Segment SGI24035[] = {{&S16301[75], 5}, {NULL}};
const Segment SGI24036[] = {{&S16816[44], 5}, {NULL}};
const Segment SGI24037[] = {{&S16816[44], 16}, {NULL}};
const Segment SGI24038[] = {{&S16816[30], 6}, {NULL}};
const Segment SGI24039[] = {{&S16816[108], 3}, {NULL}};
const Segment SGI24040[] = {{&S16816[74], 5}, {NULL}};
const Segment SGI24041[] = {{&S16312[44], 14}, {NULL}};
const Segment SGI24042[] = {{&S16312[44], 31}, {NULL}};
const Segment SGI24043[] = {{&S16312[30], 6}, {NULL}};
const Segment SGI24044[] = {{&S16312[123], 3}, {NULL}};
const Segment SGI24045[] = {{&S16312[89], 5}, {NULL}};
const Segment SGI24046[] = {{&S16315[44], 6}, {NULL}};
const Segment SGI24047[] = {{&S16315[44], 17}, {NULL}};
const Segment SGI24048[] = {{&S16315[30], 6}, {NULL}};
const Segment SGI24049[] = {{&S16315[109], 3}, {NULL}};
const Segment SGI24050[] = {{&S16315[75], 5}, {NULL}};
const Segment SGI24051[] = {{&S16327[44], 12}, {NULL}};
const Segment SGI24052[] = {{&S16327[44], 23}, {NULL}};
const Segment SGI24053[] = {{&S16327[30], 6}, {NULL}};
const Segment SGI24054[] = {{&S16327[115], 3}, {NULL}};
const Segment SGI24055[] = {{&S16327[81], 5}, {NULL}};
const Segment SGI24056[] = {{&S16331[44], 5}, {NULL}};
const Segment SGI24057[] = {{&S16331[44], 16}, {NULL}};
const Segment SGI24058[] = {{&S16331[30], 6}, {NULL}};
const Segment SGI24059[] = {{&S16331[108], 3}, {NULL}};
const Segment SGI24060[] = {{&S16331[74], 5}, {NULL}};
const Segment SGI24061[] = {{&S16333[44], 7}, {NULL}};
const Segment SGI24062[] = {{&S16333[44], 18}, {NULL}};
const Segment SGI24063[] = {{&S16333[30], 6}, {NULL}};
const Segment SGI24064[] = {{&S16333[110], 3}, {NULL}};
const Segment SGI24065[] = {{&S16333[76], 5}, {NULL}};
const Segment SGI24066[] = {{&S16337[44], 6}, {NULL}};
const Segment SGI24067[] = {{&S16337[44], 17}, {NULL}};
const Segment SGI24068[] = {{&S16337[30], 6}, {NULL}};
const Segment SGI24069[] = {{&S16337[109], 3}, {NULL}};
const Segment SGI24070[] = {{&S16337[75], 5}, {NULL}};
const Segment SGI24071[] = {{&S16338[44], 16}, {NULL}};
const Segment SGI24072[] = {{&S16338[44], 27}, {NULL}};
const Segment SGI24073[] = {{&S16338[30], 6}, {NULL}};
const Segment SGI24074[] = {{&S16338[119], 3}, {NULL}};
const Segment SGI24075[] = {{&S16338[85], 5}, {NULL}};
const Segment SGI24076[] = {{&S16343[44], 7}, {NULL}};
const Segment SGI24077[] = {{&S16343[44], 22}, {NULL}};
const Segment SGI24078[] = {{&S16343[30], 6}, {NULL}};
const Segment SGI24079[] = {{&S16343[114], 3}, {NULL}};
const Segment SGI24080[] = {{&S16343[80], 5}, {NULL}};
const Segment SGI24081[] = {{&S16346[44], 17}, {NULL}};
const Segment SGI24082[] = {{&S16346[44], 34}, {NULL}};
const Segment SGI24083[] = {{&S16346[30], 6}, {NULL}};
const Segment SGI24084[] = {{&S16346[95], 4}, {NULL}};
const Segment SGI24085[] = {{&S16346[95], 3}, {NULL}};
const Segment SGI24086[] = {{&S16348[44], 6}, {NULL}};
const Segment SGI24087[] = {{&S16348[44], 17}, {NULL}};
const Segment SGI24088[] = {{&S16348[30], 6}, {NULL}};
const Segment SGI24089[] = {{&S26792[44], 10}, {NULL}};
const Segment SGI24090[] = {{&S16348[109], 3}, {NULL}};
const Segment SGI24091[] = {{&S26792[44], 22}, {NULL}};
const Segment SGI24092[] = {{&S26792[30], 6}, {NULL}};
const Segment SGI24093[] = {{&S16348[75], 5}, {NULL}};
const Segment SGI24094[] = {{&S26792[114], 3}, {NULL}};
const Segment SGI24095[] = {{&S26792[80], 5}, {NULL}};
const Segment SGI24096[] = {{&S26793[44], 9}, {NULL}};
const Segment SGI24097[] = {{&S26793[44], 21}, {NULL}};
const Segment SGI24098[] = {{&S26793[30], 6}, {NULL}};
const Segment SGI24099[] = {{&S26793[113], 3}, {NULL}};
const Segment SGI24100[] = {{&S26793[79], 5}, {NULL}};
const Segment SGI24101[] = {{&S26794[44], 14}, {NULL}};
const Segment SGI24102[] = {{&S16871[44], 5}, {NULL}};
const Segment SGI24103[] = {{&S26794[44], 26}, {NULL}};
const Segment SGI24104[] = {{&S16871[44], 16}, {NULL}};
const Segment SGI24105[] = {{&S26794[30], 6}, {NULL}};
const Segment SGI24106[] = {{&S16871[30], 6}, {NULL}};
const Segment SGI24107[] = {{&S26794[118], 3}, {NULL}};
const Segment SGI24108[] = {{&S16871[108], 3}, {NULL}};
const Segment SGI24109[] = {{&S26794[84], 5}, {NULL}};
const Segment SGI24110[] = {{&S16871[74], 5}, {NULL}};
const Segment SGI24111[] = {{&S26795[44], 10}, {NULL}};
const Segment SGI24112[] = {{&S26795[44], 22}, {NULL}};
const Segment SGI24113[] = {{&S26795[30], 6}, {NULL}};
const Segment SGI24114[] = {{&S26795[114], 3}, {NULL}};
const Segment SGI24115[] = {{&S26795[80], 5}, {NULL}};
const Segment SGI24116[] = {{&S16873[44], 6}, {NULL}};
const Segment SGI24117[] = {{&S16873[44], 17}, {NULL}};
const Segment SGI24118[] = {{&S16873[30], 6}, {NULL}};
const Segment SGI24119[] = {{&S16873[109], 3}, {NULL}};
const Segment SGI24120[] = {{&S16873[75], 5}, {NULL}};
const Segment SGI24121[] = {{&S26796[44], 10}, {NULL}};
const Segment SGI24122[] = {{&S26796[44], 22}, {NULL}};
const Segment SGI24123[] = {{&S26796[30], 6}, {NULL}};
const Segment SGI24124[] = {{&S26796[114], 3}, {NULL}};
const Segment SGI24125[] = {{&S26796[80], 5}, {NULL}};
const Segment SGI24126[] = {{&S26797[44], 9}, {NULL}};
const Segment SGI24127[] = {{&S26797[44], 21}, {NULL}};
const Segment SGI24128[] = {{&S26797[30], 6}, {NULL}};
const Segment SGI24129[] = {{&S26797[113], 3}, {NULL}};
const Segment SGI24130[] = {{&S26797[79], 5}, {NULL}};
const Segment SGI24131[] = {{&S26798[44], 16}, {NULL}};
const Segment SGI24132[] = {{&S26798[44], 28}, {NULL}};
const Segment SGI24133[] = {{&S26798[30], 6}, {NULL}};
const Segment SGI24134[] = {{&S16362[44], 7}, {NULL}};
const Segment SGI24135[] = {{&S26798[120], 3}, {NULL}};
const Segment SGI24136[] = {{&S16362[44], 18}, {NULL}};
const Segment SGI24137[] = {{&S16362[30], 6}, {NULL}};
const Segment SGI24138[] = {{&S26798[86], 5}, {NULL}};
const Segment SGI24139[] = {{&S16362[110], 3}, {NULL}};
const Segment SGI24140[] = {{&S16362[76], 5}, {NULL}};
const Segment SGI24141[] = {{&S26799[8], 1}, {NULL}};
const Segment SGI24142[] = {{&S26799[53], 13}, {NULL}};
const Segment SGI24143[] = {{&S26799[39], 6}, {NULL}};
const Segment SGI24144[] = {{&S26799[114], 3}, {NULL}};
const Segment SGI24145[] = {{&S26799[80], 5}, {NULL}};
const Segment SGI24146[] = {{&S16364[44], 6}, {NULL}};
const Segment SGI24147[] = {{&S16364[44], 17}, {NULL}};
const Segment SGI24148[] = {{&S16364[30], 6}, {NULL}};
const Segment SGI24149[] = {{&S16364[78], 6}, {NULL}};
const Segment SGI24150[] = {{&S26800[44], 16}, {NULL}};
const Segment SGI24151[] = {{&S26800[44], 28}, {NULL}};
const Segment SGI24152[] = {{&S26800[30], 6}, {NULL}};
const Segment SGI24153[] = {{&S16882[44], 6}, {NULL}};
const Segment SGI24154[] = {{&S16882[44], 17}, {NULL}};
const Segment SGI24155[] = {{&S16882[30], 6}, {NULL}};
const Segment SGI24156[] = {{&S26800[120], 3}, {NULL}};
const Segment SGI24157[] = {{&S16882[109], 3}, {NULL}};
const Segment SGI24158[] = {{&S26800[86], 5}, {NULL}};
const Segment SGI24159[] = {{&S16882[75], 5}, {NULL}};
const Segment SGI24160[] = {{&S16366[44], 13}, {NULL}};
const Segment SGI24161[] = {{&S16366[44], 24}, {NULL}};
const Segment SGI24162[] = {{&S16366[30], 6}, {NULL}};
const Segment SGI24163[] = {{&S26801[41], 14}, {NULL}};
const Segment SGI24164[] = {{&S16883[44], 11}, {NULL}};
const Segment SGI24165[] = {{&S26801[30], 6}, {NULL}};
const Segment SGI24166[] = {{&S16883[44], 22}, {NULL}};
const Segment SGI24167[] = {{&S16366[116], 3}, {NULL}};
const Segment SGI24168[] = {{&S16883[30], 6}, {NULL}};
const Segment SGI24169[] = {{&S16366[82], 5}, {NULL}};
const Segment SGI24170[] = {{&S26801[115], 3}, {NULL}};
const Segment SGI24171[] = {{&S16883[114], 3}, {NULL}};
const Segment SGI24172[] = {{&S26801[81], 5}, {NULL}};
const Segment SGI24173[] = {{&S16883[80], 5}, {NULL}};
const Segment SGI24174[] = {{&S26802[44], 14}, {NULL}};
const Segment SGI24175[] = {{&S26802[44], 26}, {NULL}};
const Segment SGI24176[] = {{&S26802[30], 6}, {NULL}};
const Segment SGI24177[] = {{&S16368[44], 7}, {NULL}};
const Segment SGI24178[] = {{&S26802[118], 3}, {NULL}};
const Segment SGI24179[] = {{&S16368[44], 18}, {NULL}};
const Segment SGI24180[] = {{&S16368[30], 6}, {NULL}};
const Segment SGI24181[] = {{&S26802[84], 5}, {NULL}};
const Segment SGI24182[] = {{&S16368[110], 3}, {NULL}};
const Segment SGI24183[] = {{&S16368[76], 5}, {NULL}};
const Segment SGI24184[] = {{&S26803[44], 14}, {NULL}};
const Segment SGI24185[] = {{&S26803[44], 26}, {NULL}};
const Segment SGI24186[] = {{&S26803[30], 6}, {NULL}};
const Segment SGI24187[] = {{&S26803[118], 3}, {NULL}};
const Segment SGI24188[] = {{&S26803[84], 5}, {NULL}};
const Segment SGI24189[] = {{&S26804[44], 14}, {NULL}};
const Segment SGI24190[] = {{&S26804[44], 26}, {NULL}};
const Segment SGI24191[] = {{&S16371[44], 9}, {NULL}};
const Segment SGI24192[] = {{&S26804[30], 6}, {NULL}};
const Segment SGI24193[] = {{&S16371[44], 20}, {NULL}};
const Segment SGI24194[] = {{&S16371[30], 6}, {NULL}};
const Segment SGI24195[] = {{&S26804[118], 3}, {NULL}};
const Segment SGI24196[] = {{&S16371[112], 3}, {NULL}};
const Segment SGI24197[] = {{&S16371[78], 5}, {NULL}};
const Segment SGI24198[] = {{&S26804[84], 5}, {NULL}};
const Segment SGI24199[] = {{&S26805[44], 14}, {NULL}};
const Segment SGI24200[] = {{&S26805[44], 26}, {NULL}};
const Segment SGI24201[] = {{&S26805[30], 6}, {NULL}};
const Segment SGI24202[] = {{&S26805[118], 3}, {NULL}};
const Segment SGI24203[] = {{&S26805[84], 5}, {NULL}};
const Segment SGI24204[] = {{&S26806[41], 10}, {NULL}};
const Segment SGI24205[] = {{&S26806[30], 6}, {NULL}};
const Segment SGI24206[] = {{&S26806[111], 3}, {NULL}};
const Segment SGI24207[] = {{&S16375[44], 8}, {NULL}};
const Segment SGI24208[] = {{&S16375[44], 19}, {NULL}};
const Segment SGI24209[] = {{&S26806[77], 5}, {NULL}};
const Segment SGI24210[] = {{&S16375[30], 6}, {NULL}};
const Segment SGI24211[] = {{&S16375[111], 3}, {NULL}};
const Segment SGI24212[] = {{&S16375[77], 5}, {NULL}};
const Segment SGI24213[] = {{&S26807[41], 14}, {NULL}};
const Segment SGI24214[] = {{&S26807[30], 6}, {NULL}};
const Segment SGI24215[] = {{&S26807[115], 3}, {NULL}};
const Segment SGI24216[] = {{&S26807[81], 5}, {NULL}};
const Segment SGI24217[] = {{&S26808[44], 14}, {NULL}};
const Segment SGI24218[] = {{&S26808[44], 26}, {NULL}};
const Segment SGI24219[] = {{&S16378[44], 7}, {NULL}};
const Segment SGI24220[] = {{&S26808[30], 6}, {NULL}};
const Segment SGI24221[] = {{&S16378[44], 36}, {NULL}};
const Segment SGI24222[] = {{&S16378[30], 6}, {NULL}};
const Segment SGI24223[] = {{&S26808[118], 3}, {NULL}};
const Segment SGI24224[] = {{&S16378[128], 3}, {NULL}};
const Segment SGI24225[] = {{&S26808[84], 5}, {NULL}};
const Segment SGI24226[] = {{&S16378[94], 5}, {NULL}};
const Segment SGI24227[] = {{&S26809[44], 14}, {NULL}};
const Segment SGI24228[] = {{&S26809[44], 26}, {NULL}};
const Segment SGI24229[] = {{&S26809[30], 6}, {NULL}};
const Segment SGI24230[] = {{&S26809[118], 3}, {NULL}};
const Segment SGI24231[] = {{&S26809[84], 5}, {NULL}};
const Segment SGI24232[] = {{&S26810[38], 9}, {NULL}};
const Segment SGI24233[] = {{&S26810[31], 6}, {NULL}};
const Segment SGI24234[] = {{&S26811[38], 10}, {NULL}};
const Segment SGI24235[] = {{&S26811[31], 6}, {NULL}};
const Segment SGI24236[] = {{&S26812[44], 9}, {NULL}};
const Segment SGI24237[] = {{&S26812[44], 21}, {NULL}};
const Segment SGI24238[] = {{&S26812[30], 6}, {NULL}};
const Segment SGI24239[] = {{&S26812[82], 4}, {NULL}};
const Segment SGI24240[] = {{&S26812[82], 3}, {NULL}};
const Segment SGI24241[] = {{&S26813[41], 9}, {NULL}};
const Segment SGI24242[] = {{&S26813[30], 6}, {NULL}};
const Segment SGI24243[] = {{&S26813[110], 3}, {NULL}};
const Segment SGI24244[] = {{&S26813[76], 5}, {NULL}};
const Segment SGI24245[] = {{&S26814[44], 9}, {NULL}};
const Segment SGI24246[] = {{&S26814[44], 21}, {NULL}};
const Segment SGI24247[] = {{&S26814[30], 6}, {NULL}};
const Segment SGI24248[] = {{&S26814[82], 6}, {NULL}};
const Segment SGI24249[] = {{&S26815[38], 10}, {NULL}};
const Segment SGI24250[] = {{&S26815[31], 6}, {NULL}};
const Segment SGI24251[] = {{&S26816[41], 14}, {NULL}};
const Segment SGI24252[] = {{&S26816[30], 6}, {NULL}};
const Segment SGI24253[] = {{&S26816[115], 3}, {NULL}};
const Segment SGI24254[] = {{&S26816[81], 5}, {NULL}};
const Segment SGI24255[] = {{&S26817[44], 9}, {NULL}};
const Segment SGI24256[] = {{&S26817[44], 26}, {NULL}};
const Segment SGI24257[] = {{&S26817[30], 6}, {NULL}};
const Segment SGI24258[] = {{&S26817[87], 6}, {NULL}};
const Segment SGI24259[] = {{&S26818[44], 10}, {NULL}};
const Segment SGI24260[] = {{&S26818[44], 22}, {NULL}};
const Segment SGI24261[] = {{&S26818[30], 6}, {NULL}};
const Segment SGI24262[] = {{&S26818[80], 5}, {NULL}};
const Segment SGI24263[] = {{&S16399[44], 7}, {NULL}};
const Segment SGI24264[] = {{&S16399[44], 18}, {NULL}};
const Segment SGI24265[] = {{&S16399[30], 6}, {NULL}};
const Segment SGI24266[] = {{&S16399[110], 3}, {NULL}};
const Segment SGI24267[] = {{&S16399[76], 5}, {NULL}};
const Segment SGI24268[] = {{&S16401[44], 8}, {NULL}};
const Segment SGI24269[] = {{&S16401[44], 25}, {NULL}};
const Segment SGI24270[] = {{&S16401[30], 6}, {NULL}};
const Segment SGI24271[] = {{&S16401[117], 3}, {NULL}};
const Segment SGI24272[] = {{&S16401[83], 5}, {NULL}};
const Segment SGI24273[] = {{&S16402[44], 7}, {NULL}};
const Segment SGI24274[] = {{&S16402[44], 18}, {NULL}};
const Segment SGI24275[] = {{&S16402[30], 6}, {NULL}};
const Segment SGI24276[] = {{&S16402[110], 3}, {NULL}};
const Segment SGI24277[] = {{&S16402[76], 5}, {NULL}};
const Segment SGI24278[] = {{&S16424[44], 14}, {NULL}};
const Segment SGI24279[] = {{&S16424[44], 25}, {NULL}};
const Segment SGI24280[] = {{&S16424[30], 6}, {NULL}};
const Segment SGI24281[] = {{&S16424[117], 3}, {NULL}};
const Segment SGI24282[] = {{&S16424[83], 5}, {NULL}};
const Segment SGI24283[] = {{&S16429[44], 8}, {NULL}};
const Segment SGI24284[] = {{&S16429[44], 19}, {NULL}};
const Segment SGI24285[] = {{&S16429[30], 6}, {NULL}};
const Segment SGI24286[] = {{&S16429[111], 3}, {NULL}};
const Segment SGI24287[] = {{&S16429[77], 5}, {NULL}};
const Segment SGI24288[] = {{&S19000[44], 7}, {NULL}};
const Segment SGI24289[] = {{&S19000[44], 24}, {NULL}};
const Segment SGI24290[] = {{&S19000[30], 6}, {NULL}};
const Segment SGI24291[] = {{&S19000[116], 3}, {NULL}};
const Segment SGI24292[] = {{&S19000[82], 5}, {NULL}};
const Segment SGI24293[] = {{&S19001[44], 20}, {NULL}};
const Segment SGI24294[] = {{&S19001[44], 37}, {NULL}};
const Segment SGI24295[] = {{&S19001[30], 6}, {NULL}};
const Segment SGI24296[] = {{&S19001[129], 3}, {NULL}};
const Segment SGI24297[] = {{&S19001[95], 5}, {NULL}};
const Segment SGI24298[] = {{&S19003[44], 19}, {NULL}};
const Segment SGI24299[] = {{&S19003[44], 36}, {NULL}};
const Segment SGI24300[] = {{&S19003[30], 6}, {NULL}};
const Segment SGI24301[] = {{&S19003[128], 3}, {NULL}};
const Segment SGI24302[] = {{&S19003[94], 5}, {NULL}};
const Segment SGI24303[] = {{&S19005[44], 21}, {NULL}};
const Segment SGI24304[] = {{&S19005[44], 38}, {NULL}};
const Segment SGI24305[] = {{&S19005[30], 6}, {NULL}};
const Segment SGI24306[] = {{&S22789[44], 10}, {NULL}};
const Segment SGI24307[] = {{&S19005[130], 3}, {NULL}};
const Segment SGI24308[] = {{&S22789[44], 21}, {NULL}};
const Segment SGI24309[] = {{&S22789[30], 6}, {NULL}};
const Segment SGI24310[] = {{&S19005[96], 5}, {NULL}};
const Segment SGI24311[] = {{&S22789[113], 3}, {NULL}};
const Segment SGI24312[] = {{&S22789[79], 5}, {NULL}};
const Segment SGI24313[] = {{&S19006[44], 19}, {NULL}};
const Segment SGI24314[] = {{&S19006[44], 36}, {NULL}};
const Segment SGI24315[] = {{&S19006[30], 6}, {NULL}};
const Segment SGI24316[] = {{&S19006[128], 3}, {NULL}};
const Segment SGI24317[] = {{&S19006[94], 5}, {NULL}};
const Segment SGI24318[] = {{&S19007[44], 19}, {NULL}};
const Segment SGI24319[] = {{&S19007[44], 36}, {NULL}};
const Segment SGI24320[] = {{&S19007[30], 6}, {NULL}};
const Segment SGI24321[] = {{&S19007[128], 3}, {NULL}};
const Segment SGI24322[] = {{&S19007[94], 5}, {NULL}};
const Segment SGI24323[] = {{&S19008[44], 19}, {NULL}};
const Segment SGI24324[] = {{&S19008[44], 35}, {NULL}};
const Segment SGI24325[] = {{&S19008[30], 6}, {NULL}};
const Segment SGI24326[] = {{&S19008[127], 3}, {NULL}};
const Segment SGI24327[] = {{&S19008[93], 5}, {NULL}};
const Segment SGI24328[] = {{&S19009[44], 7}, {NULL}};
const Segment SGI24329[] = {{&S19009[44], 24}, {NULL}};
const Segment SGI24330[] = {{&S19009[30], 6}, {NULL}};
const Segment SGI24331[] = {{&S19009[116], 3}, {NULL}};
const Segment SGI24332[] = {{&S19009[82], 5}, {NULL}};
const Segment SGI24333[] = {{&S19010[44], 19}, {NULL}};
const Segment SGI24334[] = {{&S19010[44], 36}, {NULL}};
const Segment SGI24335[] = {{&S19010[30], 6}, {NULL}};
const Segment SGI24336[] = {{&S19010[128], 3}, {NULL}};
const Segment SGI24337[] = {{&S19010[94], 5}, {NULL}};
const Segment SGI24338[] = {{&S19025[44], 18}, {NULL}};
const Segment SGI24339[] = {{&S19025[44], 39}, {NULL}};
const Segment SGI24340[] = {{&S19025[30], 6}, {NULL}};
const Segment SGI24341[] = {{&S19025[131], 3}, {NULL}};
const Segment SGI24342[] = {{&S19025[97], 5}, {NULL}};
const Segment SGI24343[] = {{&S19032[44], 21}, {NULL}};
const Segment SGI24344[] = {{&S19032[44], 38}, {NULL}};
const Segment SGI24345[] = {{&S19032[30], 6}, {NULL}};
const Segment SGI24346[] = {{&S19032[130], 3}, {NULL}};
const Segment SGI24347[] = {{&S19032[96], 5}, {NULL}};
const Segment SGI24348[] = {{&S19035[44], 19}, {NULL}};
const Segment SGI24349[] = {{&S19035[44], 34}, {NULL}};
const Segment SGI24350[] = {{&S19035[30], 6}, {NULL}};
const Segment SGI24351[] = {{&S19035[126], 3}, {NULL}};
const Segment SGI24352[] = {{&S19035[92], 5}, {NULL}};
const Segment SGI24353[] = {{&S19041[44], 19}, {NULL}};
const Segment SGI24354[] = {{&S19041[44], 36}, {NULL}};
const Segment SGI24355[] = {{&S19041[30], 6}, {NULL}};
const Segment SGI24356[] = {{&S19041[128], 3}, {NULL}};
const Segment SGI24357[] = {{&S19041[94], 5}, {NULL}};
const Segment SGI24358[] = {{&S15965[44], 22}, {NULL}};
const Segment SGI24359[] = {{&S15965[44], 39}, {NULL}};
const Segment SGI24360[] = {{&S15965[30], 6}, {NULL}};
const Segment SGI24361[] = {{&S15965[131], 3}, {NULL}};
const Segment SGI24362[] = {{&S15965[97], 5}, {NULL}};
const Segment SGI24363[] = {{&S19046[44], 18}, {NULL}};
const Segment SGI24364[] = {{&S19046[44], 39}, {NULL}};
const Segment SGI24365[] = {{&S19046[30], 6}, {NULL}};
const Segment SGI24366[] = {{&S19046[131], 3}, {NULL}};
const Segment SGI24367[] = {{&S15969[44], 11}, {NULL}};
const Segment SGI24368[] = {{&S15969[44], 23}, {NULL}};
const Segment SGI24369[] = {{&S19046[97], 5}, {NULL}};
const Segment SGI24370[] = {{&S15969[30], 6}, {NULL}};
const Segment SGI24371[] = {{&S15969[115], 3}, {NULL}};
const Segment SGI24372[] = {{&S15969[81], 5}, {NULL}};
const Segment SGI24373[] = {{&S15970[44], 11}, {NULL}};
const Segment SGI24374[] = {{&S15970[44], 22}, {NULL}};
const Segment SGI24375[] = {{&S15970[30], 6}, {NULL}};
const Segment SGI24376[] = {{&S15970[114], 3}, {NULL}};
const Segment SGI24377[] = {{&S15970[80], 5}, {NULL}};
const Segment SGI24378[] = {{&S15971[44], 13}, {NULL}};
const Segment SGI24379[] = {{&S15971[44], 25}, {NULL}};
const Segment SGI24380[] = {{&S15971[30], 6}, {NULL}};
const Segment SGI24381[] = {{&S15971[117], 3}, {NULL}};
const Segment SGI24382[] = {{&S15971[83], 5}, {NULL}};
const Segment SGI24383[] = {{&S15973[44], 5}, {NULL}};
const Segment SGI24384[] = {{&S15973[44], 16}, {NULL}};
const Segment SGI24385[] = {{&S15973[30], 6}, {NULL}};
const Segment SGI24386[] = {{&S15973[108], 3}, {NULL}};
const Segment SGI24387[] = {{&S15973[74], 5}, {NULL}};
const Segment SGI24388[] = {{&S19054[44], 19}, {NULL}};
const Segment SGI24389[] = {{&S19054[44], 36}, {NULL}};
const Segment SGI24390[] = {{&S19054[30], 6}, {NULL}};
const Segment SGI24391[] = {{&S19054[128], 3}, {NULL}};
const Segment SGI24392[] = {{&S19054[94], 5}, {NULL}};
const Segment SGI24393[] = {{&S15975[44], 10}, {NULL}};
const Segment SGI24394[] = {{&S16430[44], 7}, {NULL}};
const Segment SGI24395[] = {{&S16430[44], 18}, {NULL}};
const Segment SGI24396[] = {{&S16430[30], 6}, {NULL}};
const Segment SGI24397[] = {{&S16430[79], 6}, {NULL}};
const Segment SGI24398[] = {{&S16431[44], 11}, {NULL}};
const Segment SGI24399[] = {{&S16431[44], 22}, {NULL}};
const Segment SGI24400[] = {{&S16431[30], 6}, {NULL}};
const Segment SGI24401[] = {{&S16431[83], 6}, {NULL}};
const Segment SGI24402[] = {{&S19060[38], 7}, {NULL}};
const Segment SGI24403[] = {{&S19060[31], 6}, {NULL}};
const Segment SGI24404[] = {{&S19066[44], 18}, {NULL}};
const Segment SGI24405[] = {{&S19066[44], 39}, {NULL}};
const Segment SGI24406[] = {{&S19066[30], 6}, {NULL}};
const Segment SGI24407[] = {{&S19066[131], 3}, {NULL}};
const Segment SGI24408[] = {{&S19066[97], 5}, {NULL}};
const Segment SGI24409[] = {{&S16446[44], 34}, {NULL}};
const Segment SGI24410[] = {{&S16446[30], 6}, {NULL}};
const Segment SGI24411[] = {{&S16459[44], 10}, {NULL}};
const Segment SGI24412[] = {{&S16459[44], 21}, {NULL}};
const Segment SGI24413[] = {{&S16459[30], 6}, {NULL}};
const Segment SGI24414[] = {{&S16459[113], 3}, {NULL}};
const Segment SGI24415[] = {{&S16459[79], 5}, {NULL}};
const Segment SGI24416[] = {{&S16467[44], 8}, {NULL}};
const Segment SGI24417[] = {{&S16467[44], 19}, {NULL}};
const Segment SGI24418[] = {{&S16467[30], 6}, {NULL}};
const Segment SGI24419[] = {{&S16467[111], 3}, {NULL}};
const Segment SGI24420[] = {{&S16467[77], 5}, {NULL}};
const Segment SGI24421[] = {{&S16472[44], 7}, {NULL}};
const Segment SGI24422[] = {{&S16472[44], 18}, {NULL}};
const Segment SGI24423[] = {{&S16472[30], 6}, {NULL}};
const Segment SGI24424[] = {{&S16472[110], 3}, {NULL}};
const Segment SGI24425[] = {{&S16472[76], 5}, {NULL}};
const Segment SGI24426[] = {{&S15975[44], 21}, {NULL}};
const Segment SGI24427[] = {{&S15975[30], 6}, {NULL}};
const Segment SGI24428[] = {{&S15975[113], 3}, {NULL}};
const Segment SGI24429[] = {{&S15975[79], 5}, {NULL}};
const Segment SGI24430[] = {{&S17121[38], 5}, {NULL}};
const Segment SGI24431[] = {{&S17121[31], 6}, {NULL}};
const Segment SGI24432[] = {{&S15980[44], 14}, {NULL}};
const Segment SGI24433[] = {{&S15980[44], 25}, {NULL}};
const Segment SGI24434[] = {{&S15980[30], 6}, {NULL}};
const Segment SGI24435[] = {{&S15980[117], 3}, {NULL}};
const Segment SGI24436[] = {{&S15980[83], 5}, {NULL}};
const Segment SGI24437[] = {{&S16481[44], 13}, {NULL}};
const Segment SGI24438[] = {{&S16481[44], 30}, {NULL}};
const Segment SGI24439[] = {{&S16481[30], 6}, {NULL}};
const Segment SGI24440[] = {{&S16481[122], 3}, {NULL}};
const Segment SGI24441[] = {{&S16481[88], 5}, {NULL}};
const Segment SGI24442[] = {{&S16483[44], 24}, {NULL}};
const Segment SGI24443[] = {{&S16483[44], 41}, {NULL}};
const Segment SGI24444[] = {{&S16483[30], 6}, {NULL}};
const Segment SGI24445[] = {{&S16483[134], 3}, {NULL}};
const Segment SGI24446[] = {{&S16483[99], 6}, {NULL}};
const Segment SGI24447[] = {{&S16005[44], 8}, {NULL}};
const Segment SGI24448[] = {{&S16005[44], 19}, {NULL}};
const Segment SGI24449[] = {{&S16005[30], 6}, {NULL}};
const Segment SGI24450[] = {{&S16005[111], 3}, {NULL}};
const Segment SGI24451[] = {{&S16005[77], 5}, {NULL}};
const Segment SGI24452[] = {{&S16017[44], 13}, {NULL}};
const Segment SGI24453[] = {{&S16017[44], 24}, {NULL}};
const Segment SGI24454[] = {{&S16017[30], 6}, {NULL}};
const Segment SGI24455[] = {{&S16017[116], 3}, {NULL}};
const Segment SGI24456[] = {{&S16017[82], 5}, {NULL}};
const Segment SGI24457[] = {{&S16023[44], 28}, {NULL}};
const Segment SGI24458[] = {{&S16023[30], 6}, {NULL}};
const Segment SGI24459[] = {{&S16023[59], 3}, {NULL}};
const Segment SGI24460[] = {{&S16023[105], 5}, {NULL}};
const Segment SGI24461[] = {{&S16524[44], 7}, {NULL}};
const Segment SGI24462[] = {{&S16524[44], 18}, {NULL}};
const Segment SGI24463[] = {{&S16524[30], 6}, {NULL}};
const Segment SGI24464[] = {{&S16524[110], 3}, {NULL}};
const Segment SGI24465[] = {{&S16524[76], 5}, {NULL}};
const Segment SGI24466[] = {{&S16525[44], 10}, {NULL}};
const Segment SGI24467[] = {{&S16525[44], 21}, {NULL}};
const Segment SGI24468[] = {{&S16525[30], 6}, {NULL}};
const Segment SGI24469[] = {{&S16525[113], 3}, {NULL}};
const Segment SGI24470[] = {{&S16525[79], 5}, {NULL}};
const Segment SGI24471[] = {{&S16526[44], 11}, {NULL}};
const Segment SGI24472[] = {{&S16526[44], 22}, {NULL}};
const Segment SGI24473[] = {{&S16526[30], 6}, {NULL}};
const Segment SGI24474[] = {{&S16526[114], 3}, {NULL}};
const Segment SGI24475[] = {{&S16526[80], 5}, {NULL}};
const Segment SGI24476[] = {{&S16039[44], 9}, {NULL}};
const Segment SGI24477[] = {{&S16039[44], 20}, {NULL}};
const Segment SGI24478[] = {{&S16039[30], 6}, {NULL}};
const Segment SGI24479[] = {{&S16039[112], 3}, {NULL}};
const Segment SGI24480[] = {{&S16039[78], 5}, {NULL}};
const Segment SGI24481[] = {{&S16529[44], 14}, {NULL}};
const Segment SGI24482[] = {{&S16529[30], 6}, {NULL}};
const Segment SGI24483[] = {{&S16529[106], 3}, {NULL}};
const Segment SGI24484[] = {{&S16529[72], 5}, {NULL}};
const Segment SGI24485[] = {{&S16530[44], 7}, {NULL}};
const Segment SGI24486[] = {{&S16530[44], 18}, {NULL}};
const Segment SGI24487[] = {{&S16530[30], 6}, {NULL}};
const Segment SGI24488[] = {{&S16530[110], 3}, {NULL}};
const Segment SGI24489[] = {{&S16530[76], 5}, {NULL}};
const Segment SGI24490[] = {{&S16046[44], 15}, {NULL}};
const Segment SGI24491[] = {{&S16046[44], 26}, {NULL}};
const Segment SGI24492[] = {{&S16046[30], 6}, {NULL}};
const Segment SGI24493[] = {{&S16046[118], 3}, {NULL}};
const Segment SGI24494[] = {{&S16046[84], 5}, {NULL}};
const Segment SGI24495[] = {{&S16539[44], 4}, {NULL}};
const Segment SGI24496[] = {{&S16539[44], 15}, {NULL}};
const Segment SGI24497[] = {{&S16539[30], 6}, {NULL}};
const Segment SGI24498[] = {{&S16539[107], 3}, {NULL}};
const Segment SGI24499[] = {{&S16539[73], 5}, {NULL}};
const Segment SGI24500[] = {{&S16054[44], 12}, {NULL}};
const Segment SGI24501[] = {{&S16054[44], 23}, {NULL}};
const Segment SGI24502[] = {{&S16054[30], 6}, {NULL}};
const Segment SGI24503[] = {{&S16054[115], 3}, {NULL}};
const Segment SGI24504[] = {{&S17203[44], 22}, {NULL}};
const Segment SGI24505[] = {{&S17203[44], 33}, {NULL}};
const Segment SGI24506[] = {{&S16054[81], 5}, {NULL}};
const Segment SGI24507[] = {{&S17203[30], 6}, {NULL}};
const Segment SGI24508[] = {{&S17203[125], 3}, {NULL}};
const Segment SGI24509[] = {{&S17203[91], 5}, {NULL}};
const Segment SGI24510[] = {{&S16055[44], 5}, {NULL}};
const Segment SGI24511[] = {{&S16055[44], 16}, {NULL}};
const Segment SGI24512[] = {{&S16055[30], 6}, {NULL}};
const Segment SGI24513[] = {{&S16055[108], 3}, {NULL}};
const Segment SGI24514[] = {{&S16055[74], 5}, {NULL}};
const Segment SGI24515[] = {{&S16058[44], 17}, {NULL}};
const Segment SGI24516[] = {{&S16058[44], 28}, {NULL}};
const Segment SGI24517[] = {{&S16058[30], 6}, {NULL}};
const Segment SGI24518[] = {{&S16058[120], 3}, {NULL}};
const Segment SGI24519[] = {{&S16058[86], 5}, {NULL}};
const Segment SGI24520[] = {{&S16059[44], 12}, {NULL}};
const Segment SGI24521[] = {{&S16059[44], 23}, {NULL}};
const Segment SGI24522[] = {{&S16059[30], 6}, {NULL}};
const Segment SGI24523[] = {{&S16059[115], 3}, {NULL}};
const Segment SGI24524[] = {{&S16059[81], 5}, {NULL}};
const Segment SGI24525[] = {{&S17213[44], 11}, {NULL}};
const Segment SGI24526[] = {{&S17213[44], 22}, {NULL}};
const Segment SGI24527[] = {{&S17213[30], 6}, {NULL}};
const Segment SGI24528[] = {{&S17213[114], 3}, {NULL}};
const Segment SGI24529[] = {{&S16061[44], 6}, {NULL}};
const Segment SGI24530[] = {{&S17213[80], 5}, {NULL}};
const Segment SGI24531[] = {{&S16061[44], 23}, {NULL}};
const Segment SGI24532[] = {{&S16061[30], 6}, {NULL}};
const Segment SGI24533[] = {{&S16061[115], 3}, {NULL}};
const Segment SGI24534[] = {{&S16061[81], 5}, {NULL}};
const Segment SGI24535[] = {{&S16556[44], 15}, {NULL}};
const Segment SGI24536[] = {{&S16556[44], 26}, {NULL}};
const Segment SGI24537[] = {{&S16556[30], 6}, {NULL}};
const Segment SGI24538[] = {{&S16556[118], 3}, {NULL}};
const Segment SGI24539[] = {{&S16556[84], 5}, {NULL}};
const Segment SGI24540[] = {{&S16065[44], 12}, {NULL}};
const Segment SGI24541[] = {{&S16065[44], 29}, {NULL}};
const Segment SGI24542[] = {{&S16065[30], 6}, {NULL}};
const Segment SGI24543[] = {{&S16065[121], 3}, {NULL}};
const Segment SGI24544[] = {{&S16065[87], 5}, {NULL}};
const Segment SGI24545[] = {{&S16076[44], 6}, {NULL}};
const Segment SGI24546[] = {{&S16076[44], 17}, {NULL}};
const Segment SGI24547[] = {{&S16076[30], 6}, {NULL}};
const Segment SGI24548[] = {{&S16076[78], 4}, {NULL}};
const Segment SGI24549[] = {{&S16076[78], 3}, {NULL}};
const Segment SGI24550[] = {{&S17234[44], 10}, {NULL}};
const Segment SGI24551[] = {{&S17234[44], 21}, {NULL}};
const Segment SGI24552[] = {{&S17234[30], 6}, {NULL}};
const Segment SGI24553[] = {{&S16079[44], 6}, {NULL}};
const Segment SGI24554[] = {{&S16079[44], 17}, {NULL}};
const Segment SGI24555[] = {{&S16079[30], 6}, {NULL}};
const Segment SGI24556[] = {{&S17234[113], 3}, {NULL}};
const Segment SGI24557[] = {{&S17234[79], 5}, {NULL}};
const Segment SGI24558[] = {{&S16079[109], 3}, {NULL}};
const Segment SGI24559[] = {{&S16575[44], 11}, {NULL}};
const Segment SGI24560[] = {{&S16079[75], 5}, {NULL}};
const Segment SGI24561[] = {{&S16575[44], 34}, {NULL}};
const Segment SGI24562[] = {{&S16575[30], 6}, {NULL}};
const Segment SGI24563[] = {{&S16575[126], 3}, {NULL}};
const Segment SGI24564[] = {{&S16575[92], 5}, {NULL}};
const Segment SGI24565[] = {{&S16592[44], 6}, {NULL}};
const Segment SGI24566[] = {{&S16592[44], 17}, {NULL}};
const Segment SGI24567[] = {{&S16592[30], 6}, {NULL}};
const Segment SGI24568[] = {{&S16098[44], 5}, {NULL}};
const Segment SGI24569[] = {{&S16592[109], 3}, {NULL}};
const Segment SGI24570[] = {{&S16098[44], 16}, {NULL}};
const Segment SGI24571[] = {{&S16098[30], 6}, {NULL}};
const Segment SGI24572[] = {{&S16592[75], 5}, {NULL}};
const Segment SGI24573[] = {{&S16098[108], 3}, {NULL}};
const Segment SGI24574[] = {{&S16098[74], 5}, {NULL}};
const Segment SGI24575[] = {{&S16595[44], 29}, {&S16595[74], 8}, {NULL}};
const Segment SGI24576[] = {{&S16595[30], 6}, {NULL}};
const Segment SGI24577[] = {{&S16595[139], 3}, {NULL}};
const Segment SGI24578[] = {{&S16595[105], 5}, {NULL}};
const Segment SGI24579[] = {{&S16105[44], 11}, {NULL}};
const Segment SGI24580[] = {{&S16105[44], 28}, {NULL}};
const Segment SGI24581[] = {{&S16105[30], 6}, {NULL}};
const Segment SGI24582[] = {{&S16105[120], 3}, {NULL}};
const Segment SGI24583[] = {{&S16105[86], 5}, {NULL}};
const Segment SGI24584[] = {{&S16106[44], 13}, {NULL}};
const Segment SGI24585[] = {{&S16106[44], 24}, {NULL}};
const Segment SGI24586[] = {{&S16106[30], 6}, {NULL}};
const Segment SGI24587[] = {{&S16106[116], 3}, {NULL}};
const Segment SGI24588[] = {{&S16106[82], 5}, {NULL}};
const Segment SGI24589[] = {{&S16112[44], 8}, {NULL}};
const Segment SGI24590[] = {{&S16112[44], 19}, {NULL}};
const Segment SGI24591[] = {{&S16112[30], 6}, {NULL}};
const Segment SGI24592[] = {{&S16112[111], 3}, {NULL}};
const Segment SGI24593[] = {{&S16112[77], 5}, {NULL}};
const Segment SGI24594[] = {{&S16610[44], 8}, {NULL}};
const Segment SGI24595[] = {{&S16610[44], 25}, {NULL}};
const Segment SGI24596[] = {{&S16610[30], 6}, {NULL}};
const Segment SGI24597[] = {{&S16113[44], 11}, {NULL}};
const Segment SGI24598[] = {{&S16113[44], 22}, {NULL}};
const Segment SGI24599[] = {{&S16610[117], 3}, {NULL}};
const Segment SGI24600[] = {{&S16113[30], 6}, {NULL}};
const Segment SGI24601[] = {{&S16610[83], 5}, {NULL}};
const Segment SGI24602[] = {{&S16113[114], 3}, {NULL}};
const Segment SGI24603[] = {{&S16113[80], 5}, {NULL}};
const Segment SGI24604[] = {{&S16124[44], 25}, {NULL}};
const Segment SGI24605[] = {{&S16124[30], 6}, {NULL}};
const Segment SGI24606[] = {{&S16124[136], 3}, {NULL}};
const Segment SGI24607[] = {{&S16124[102], 5}, {NULL}};
const Segment SGI24608[] = {{&S16125[44], 5}, {NULL}};
const Segment SGI24609[] = {{&S16125[44], 31}, {NULL}};
const Segment SGI24610[] = {{&S16125[30], 6}, {NULL}};
const Segment SGI24611[] = {{&S16125[123], 3}, {NULL}};
const Segment SGI24612[] = {{&S16125[89], 5}, {NULL}};
const Segment SGI24613[] = {{&S16624[44], 17}, {NULL}};
const Segment SGI24614[] = {{&S16624[44], 34}, {NULL}};
const Segment SGI24615[] = {{&S16624[30], 6}, {NULL}};
const Segment SGI24616[] = {{&S16624[95], 6}, {NULL}};
const Segment SGI24617[] = {{&S16127[44], 10}, {NULL}};
const Segment SGI24618[] = {{&S16127[44], 27}, {NULL}};
const Segment SGI24619[] = {{&S16127[30], 6}, {NULL}};
const Segment SGI24620[] = {{&S16127[119], 3}, {NULL}};
const Segment SGI24621[] = {{&S16625[44], 6}, {NULL}};
const Segment SGI24622[] = {{&S16625[44], 17}, {NULL}};
const Segment SGI24623[] = {{&S16127[85], 5}, {NULL}};
const Segment SGI24624[] = {{&S16625[30], 6}, {NULL}};
const Segment SGI24625[] = {{&S16625[109], 3}, {NULL}};
const Segment SGI24626[] = {{&S16625[75], 5}, {NULL}};
const Segment SGI24627[] = {{&S16631[44], 33}, {NULL}};
const Segment SGI24628[] = {{&S16631[30], 6}, {NULL}};
const Segment SGI24629[] = {{&S16631[144], 3}, {NULL}};
const Segment SGI24630[] = {{&S16631[110], 5}, {NULL}};
const Segment SGI24631[] = {{&S16137[44], 9}, {NULL}};
const Segment SGI24632[] = {{&S16137[44], 20}, {NULL}};
const Segment SGI24633[] = {{&S16137[30], 6}, {NULL}};
const Segment SGI24634[] = {{&S16137[112], 3}, {NULL}};
const Segment SGI24635[] = {{&S16137[78], 5}, {NULL}};
const Segment SGI24636[] = {{&S16138[44], 12}, {NULL}};
const Segment SGI24637[] = {{&S16138[44], 23}, {NULL}};
const Segment SGI24638[] = {{&S16138[30], 6}, {NULL}};
const Segment SGI24639[] = {{&S16634[44], 17}, {NULL}};
const Segment SGI24640[] = {{&S16634[44], 34}, {NULL}};
const Segment SGI24641[] = {{&S16138[115], 3}, {NULL}};
const Segment SGI24642[] = {{&S16634[30], 6}, {NULL}};
const Segment SGI24643[] = {{&S16138[81], 5}, {NULL}};
const Segment SGI24644[] = {{&S16634[95], 6}, {NULL}};
const Segment SGI24645[] = {{&S16635[44], 9}, {NULL}};
const Segment SGI24646[] = {{&S16635[44], 26}, {NULL}};
const Segment SGI24647[] = {{&S16635[30], 6}, {NULL}};
const Segment SGI24648[] = {{&S16635[118], 3}, {NULL}};
const Segment SGI24649[] = {{&S16635[84], 5}, {NULL}};
const Segment SGI24650[] = {{&S16149[44], 9}, {NULL}};
const Segment SGI24651[] = {{&S16149[44], 20}, {NULL}};
const Segment SGI24652[] = {{&S16149[30], 6}, {NULL}};
const Segment SGI24653[] = {{&S16149[112], 3}, {NULL}};
const Segment SGI24654[] = {{&S16149[78], 5}, {NULL}};
const Segment SGI24655[] = {{&S16152[44], 25}, {NULL}};
const Segment SGI24656[] = {{&S16152[44], 32}, {NULL}};
const Segment SGI24657[] = {{&S16152[30], 6}, {NULL}};
const Segment SGI24658[] = {{&S16152[124], 3}, {NULL}};
const Segment SGI24659[] = {{&S16152[90], 5}, {NULL}};
const Segment SGI24660[] = {{&S16653[44], 8}, {NULL}};
const Segment SGI24661[] = {{&S16653[44], 19}, {NULL}};
const Segment SGI24662[] = {{&S16653[30], 6}, {NULL}};
const Segment SGI24663[] = {{&S16157[44], 10}, {NULL}};
const Segment SGI24664[] = {{&S16157[44], 21}, {NULL}};
const Segment SGI24665[] = {{&S16157[30], 6}, {NULL}};
const Segment SGI24666[] = {{&S16653[111], 3}, {NULL}};
const Segment SGI24667[] = {{&S16653[77], 5}, {NULL}};
const Segment SGI24668[] = {{&S16157[113], 3}, {NULL}};
const Segment SGI24669[] = {{&S16157[79], 5}, {NULL}};
const Segment SGI24670[] = {{&S16654[44], 8}, {NULL}};
const Segment SGI24671[] = {{&S16654[44], 19}, {NULL}};
const Segment SGI24672[] = {{&S16654[30], 6}, {NULL}};
const Segment SGI24673[] = {{&S16654[111], 3}, {NULL}};
const Segment SGI24674[] = {{&S16654[77], 5}, {NULL}};
const Segment SGI24675[] = {{&S16167[44], 18}, {NULL}};
const Segment SGI24676[] = {{&S16167[44], 29}, {NULL}};
const Segment SGI24677[] = {{&S16167[30], 6}, {NULL}};
const Segment SGI24678[] = {{&S16167[121], 3}, {NULL}};
const Segment SGI24679[] = {{&S16167[87], 5}, {NULL}};
const Segment SGI24680[] = {{&S16663[44], 5}, {NULL}};
const Segment SGI24681[] = {{&S16663[44], 16}, {NULL}};
const Segment SGI24682[] = {{&S16663[30], 6}, {NULL}};
const Segment SGI24683[] = {{&S16663[108], 3}, {NULL}};
const Segment SGI24684[] = {{&S16663[74], 5}, {NULL}};
const Segment SGI24685[] = {{&S16193[44], 18}, {NULL}};
const Segment SGI24686[] = {{&S16193[44], 29}, {NULL}};
const Segment SGI24687[] = {{&S16193[30], 6}, {NULL}};
const Segment SGI24688[] = {{&S16193[121], 3}, {NULL}};
const Segment SGI24689[] = {{&S16193[87], 5}, {NULL}};
const Segment SGI24690[] = {{&S16203[44], 15}, {NULL}};
const Segment SGI24691[] = {{&S16203[44], 26}, {NULL}};
const Segment SGI24692[] = {{&S16203[30], 6}, {NULL}};
const Segment SGI24693[] = {{&S16203[118], 3}, {NULL}};
const Segment SGI24694[] = {{&S16696[44], 17}, {NULL}};
const Segment SGI24695[] = {{&S16696[44], 28}, {NULL}};
const Segment SGI24696[] = {{&S16203[84], 5}, {NULL}};
const Segment SGI24697[] = {{&S16696[30], 6}, {NULL}};
const Segment SGI24698[] = {{&S16696[120], 3}, {NULL}};
const Segment SGI24699[] = {{&S16696[86], 5}, {NULL}};
const Segment SGI24700[] = {{&S16204[44], 7}, {NULL}};
const Segment SGI24701[] = {{&S16204[44], 18}, {NULL}};
const Segment SGI24702[] = {{&S16204[30], 6}, {NULL}};
const Segment SGI24703[] = {{&S16204[110], 3}, {NULL}};
const Segment SGI24704[] = {{&S16204[76], 5}, {NULL}};
const Segment SGI24705[] = {{&S16697[44], 14}, {NULL}};
const Segment SGI24706[] = {{&S16697[44], 31}, {NULL}};
const Segment SGI24707[] = {{&S16697[30], 6}, {NULL}};
const Segment SGI24708[] = {{&S16697[123], 3}, {NULL}};
const Segment SGI24709[] = {{&S16697[89], 5}, {NULL}};
const Segment SGI24710[] = {{&S16700[44], 8}, {NULL}};
const Segment SGI24711[] = {{&S16700[44], 35}, {NULL}};
const Segment SGI24712[] = {{&S16700[30], 6}, {NULL}};
const Segment SGI24713[] = {{&S16700[127], 3}, {NULL}};
const Segment SGI24714[] = {{&S16700[93], 5}, {NULL}};
const Segment SGI24715[] = {{&S16210[44], 12}, {NULL}};
const Segment SGI24716[] = {{&S16210[44], 23}, {NULL}};
const Segment SGI24717[] = {{&S16210[30], 6}, {NULL}};
const Segment SGI24718[] = {{&S16210[115], 3}, {NULL}};
const Segment SGI24719[] = {{&S16210[81], 5}, {NULL}};
const Segment SGI24720[] = {{&S16703[44], 7}, {NULL}};
const Segment SGI24721[] = {{&S16703[44], 18}, {NULL}};
const Segment SGI24722[] = {{&S16703[30], 6}, {NULL}};
const Segment SGI24723[] = {{&S16703[110], 3}, {NULL}};
const Segment SGI24724[] = {{&S16703[76], 5}, {NULL}};
const Segment SGI24725[] = {{&S16222[44], 9}, {NULL}};
const Segment SGI24726[] = {{&S16222[44], 20}, {NULL}};
const Segment SGI24727[] = {{&S16222[30], 6}, {NULL}};
const Segment SGI24728[] = {{&S16222[112], 3}, {NULL}};
const Segment SGI24729[] = {{&S16222[78], 5}, {NULL}};
const Segment SGI24730[] = {{&S16225[44], 14}, {NULL}};
const Segment SGI24731[] = {{&S16225[44], 25}, {NULL}};
const Segment SGI24732[] = {{&S16225[30], 6}, {NULL}};
const Segment SGI24733[] = {{&S16225[117], 3}, {NULL}};
const Segment SGI24734[] = {{&S16225[83], 5}, {NULL}};
const Segment SGI24735[] = {{&S16229[44], 10}, {NULL}};
const Segment SGI24736[] = {{&S16229[44], 36}, {NULL}};
const Segment SGI24737[] = {{&S16229[30], 6}, {NULL}};
const Segment SGI24738[] = {{&S16229[128], 3}, {NULL}};
const Segment SGI24739[] = {{&S16229[94], 5}, {NULL}};
const Segment SGI24740[] = {{&S16232[44], 7}, {NULL}};
const Segment SGI24741[] = {{&S16232[44], 18}, {NULL}};
const Segment SGI24742[] = {{&S16232[30], 6}, {NULL}};
const Segment SGI24743[] = {{&S16232[110], 3}, {NULL}};
const Segment SGI24744[] = {{&S16232[76], 5}, {NULL}};
const Segment SGI24745[] = {{&S16233[44], 10}, {NULL}};
const Segment SGI24746[] = {{&S16233[44], 21}, {NULL}};
const Segment SGI24747[] = {{&S16233[30], 6}, {NULL}};
const Segment SGI24748[] = {{&S16233[113], 3}, {NULL}};
const Segment SGI24749[] = {{&S16233[79], 5}, {NULL}};
const Segment SGI24750[] = {{&S16726[44], 15}, {NULL}};
const Segment SGI24751[] = {{&S16726[44], 30}, {NULL}};
const Segment SGI24752[] = {{&S16726[30], 6}, {NULL}};
const Segment SGI24753[] = {{&S16726[122], 3}, {NULL}};
const Segment SGI24754[] = {{&S16726[88], 5}, {NULL}};
const Segment SGI24755[] = {{&S16238[44], 9}, {NULL}};
const Segment SGI24756[] = {{&S16238[44], 20}, {NULL}};
const Segment SGI24757[] = {{&S16238[30], 6}, {NULL}};
const Segment SGI24758[] = {{&S16238[112], 3}, {NULL}};
const Segment SGI24759[] = {{&S16238[78], 5}, {NULL}};
const Segment SGI24760[] = {{&S16239[44], 12}, {NULL}};
const Segment SGI24761[] = {{&S16239[44], 23}, {NULL}};
const Segment SGI24762[] = {{&S16239[30], 6}, {NULL}};
const Segment SGI24763[] = {{&S16239[84], 4}, {NULL}};
const Segment SGI24764[] = {{&S16239[84], 3}, {NULL}};
const Segment SGI24765[] = {{&S16243[44], 25}, {NULL}};
const Segment SGI24766[] = {{&S16243[30], 6}, {NULL}};
const Segment SGI24767[] = {{&S16243[56], 3}, {NULL}};
const Segment SGI24768[] = {{&S16243[102], 5}, {NULL}};
const Segment SGI24769[] = {{&S16244[44], 10}, {NULL}};
const Segment SGI24770[] = {{&S16244[44], 36}, {NULL}};
const Segment SGI24771[] = {{&S16244[30], 6}, {NULL}};
const Segment SGI24772[] = {{&S16244[128], 3}, {NULL}};
const Segment SGI24773[] = {{&S16244[94], 5}, {NULL}};
const Segment SGI24774[] = {{&S16253[44], 14}, {NULL}};
const Segment SGI24775[] = {{&S16253[44], 25}, {NULL}};
const Segment SGI24776[] = {{&S16253[30], 6}, {NULL}};
const Segment SGI24777[] = {{&S16253[117], 3}, {NULL}};
const Segment SGI24778[] = {{&S16253[83], 5}, {NULL}};
const Segment SGI24779[] = {{&S16255[44], 6}, {NULL}};
const Segment SGI24780[] = {{&S16255[44], 17}, {NULL}};
const Segment SGI24781[] = {{&S16255[30], 6}, {NULL}};
const Segment SGI24782[] = {{&S16255[109], 3}, {NULL}};
const Segment SGI24783[] = {{&S16255[75], 5}, {NULL}};
const Segment SGI24784[] = {{&S16753[44], 23}, {NULL}};
const Segment SGI24785[] = {{&S16753[44], 34}, {NULL}};
const Segment SGI24786[] = {{&S16753[30], 6}, {NULL}};
const Segment SGI24787[] = {{&S16753[126], 3}, {NULL}};
const Segment SGI24788[] = {{&S16753[92], 5}, {NULL}};
const Segment SGI24789[] = {{&S16259[44], 5}, {NULL}};
const Segment SGI24790[] = {{&S16259[44], 16}, {NULL}};
const Segment SGI24791[] = {{&S16259[30], 6}, {NULL}};
const Segment SGI24792[] = {{&S16259[108], 3}, {NULL}};
const Segment SGI24793[] = {{&S16259[74], 5}, {NULL}};
const Segment SGI24794[] = {{&S16261[44], 10}, {NULL}};
const Segment SGI24795[] = {{&S16261[44], 17}, {NULL}};
const Segment SGI24796[] = {{&S16261[30], 6}, {NULL}};
const Segment SGI24797[] = {{&S16261[109], 3}, {NULL}};
const Segment SGI24798[] = {{&S16261[75], 5}, {NULL}};
const Segment SGI24799[] = {{&S16761[31], 6}, {NULL}};
const Segment SGI24800[] = {{&S16765[44], 35}, {NULL}};
const Segment SGI24801[] = {{&S16765[30], 6}, {NULL}};
const Segment SGI24802[] = {{&S16765[145], 3}, {NULL}};
const Segment SGI24803[] = {{&S16765[111], 5}, {NULL}};
const Segment SGI24804[] = {{&S16275[44], 8}, {NULL}};
const Segment SGI24805[] = {{&S16275[44], 19}, {NULL}};
const Segment SGI24806[] = {{&S16275[30], 6}, {NULL}};
const Segment SGI24807[] = {{&S16275[111], 3}, {NULL}};
const Segment SGI24808[] = {{&S16275[77], 5}, {NULL}};
const Segment SGI24809[] = {{&S16278[44], 4}, {NULL}};
const Segment SGI24810[] = {{&S16278[44], 15}, {NULL}};
const Segment SGI24811[] = {{&S16278[30], 6}, {NULL}};
const Segment SGI24812[] = {{&S16278[107], 3}, {NULL}};
const Segment SGI24813[] = {{&S16278[73], 5}, {NULL}};
const Segment SGI24814[] = {{&S16279[44], 6}, {NULL}};
const Segment SGI24815[] = {{&S16279[44], 17}, {NULL}};
const Segment SGI24816[] = {{&S16279[30], 6}, {NULL}};
const Segment SGI24817[] = {{&S16279[109], 3}, {NULL}};
const Segment SGI24818[] = {{&S16279[75], 5}, {NULL}};
const Segment SGI24819[] = {{&S16284[44], 16}, {NULL}};
const Segment SGI24820[] = {{&S16284[44], 27}, {NULL}};
const Segment SGI24821[] = {{&S16284[30], 6}, {NULL}};
const Segment SGI24822[] = {{&S16284[119], 3}, {NULL}};
const Segment SGI24823[] = {{&S16284[85], 5}, {NULL}};
const Segment SGI24824[] = {{&S16285[44], 5}, {NULL}};
const Segment SGI24825[] = {{&S16285[44], 16}, {NULL}};
const Segment SGI24826[] = {{&S16285[30], 6}, {NULL}};
const Segment SGI24827[] = {{&S16285[108], 3}, {NULL}};
const Segment SGI24828[] = {{&S16285[74], 5}, {NULL}};
const Segment SGI24829[] = {{&S16787[44], 10}, {NULL}};
const Segment SGI24830[] = {{&S16787[44], 21}, {NULL}};
const Segment SGI24831[] = {{&S16787[30], 6}, {NULL}};
const Segment SGI24832[] = {{&S16787[113], 3}, {NULL}};
const Segment SGI24833[] = {{&S16787[79], 5}, {NULL}};
const Segment SGI24834[] = {{&S16291[44], 13}, {NULL}};
const Segment SGI24835[] = {{&S16291[44], 24}, {NULL}};
const Segment SGI24836[] = {{&S16291[30], 6}, {NULL}};
const Segment SGI24837[] = {{&S16291[116], 3}, {NULL}};
const Segment SGI24838[] = {{&S16291[82], 5}, {NULL}};
const Segment SGI24839[] = {{&S16792[44], 6}, {NULL}};
const Segment SGI24840[] = {{&S16792[44], 17}, {NULL}};
const Segment SGI24841[] = {{&S16792[30], 6}, {NULL}};
const Segment SGI24842[] = {{&S16792[78], 6}, {NULL}};
const Segment SGI24843[] = {{&S16297[44], 11}, {NULL}};
const Segment SGI24844[] = {{&S16297[44], 22}, {NULL}};
const Segment SGI24845[] = {{&S16297[30], 6}, {NULL}};
const Segment SGI24846[] = {{&S16297[83], 4}, {NULL}};
const Segment SGI24847[] = {{&S16297[83], 3}, {NULL}};
const Segment SGI24848[] = {{&S16298[44], 7}, {NULL}};
const Segment SGI24849[] = {{&S16298[44], 18}, {NULL}};
const Segment SGI24850[] = {{&S16298[30], 6}, {NULL}};
const Segment SGI24851[] = {{&S16298[110], 3}, {NULL}};
const Segment SGI24852[] = {{&S16298[76], 5}, {NULL}};
const Segment SGI24853[] = {{&S16302[44], 6}, {NULL}};
const Segment SGI24854[] = {{&S16302[44], 32}, {NULL}};
const Segment SGI24855[] = {{&S16302[30], 6}, {NULL}};
const Segment SGI24856[] = {{&S16302[124], 3}, {NULL}};
const Segment SGI24857[] = {{&S16302[90], 5}, {NULL}};
const Segment SGI24858[] = {{&S16303[44], 7}, {NULL}};
const Segment SGI24859[] = {{&S16303[44], 18}, {NULL}};
const Segment SGI24860[] = {{&S16303[30], 6}, {NULL}};
const Segment SGI24861[] = {{&S16303[110], 3}, {NULL}};
const Segment SGI24862[] = {{&S16303[76], 5}, {NULL}};
const Segment SGI24863[] = {{&S16808[44], 5}, {NULL}};
const Segment SGI24864[] = {{&S16808[44], 33}, {NULL}};
const Segment SGI24865[] = {{&S16808[30], 6}, {NULL}};
const Segment SGI24866[] = {{&S16808[125], 3}, {NULL}};
const Segment SGI24867[] = {{&S16808[91], 5}, {NULL}};
const Segment SGI24868[] = {{&S16314[44], 5}, {NULL}};
const Segment SGI24869[] = {{&S16314[44], 16}, {NULL}};
const Segment SGI24870[] = {{&S16314[30], 6}, {NULL}};
const Segment SGI24871[] = {{&S16314[108], 3}, {NULL}};
const Segment SGI24872[] = {{&S16314[74], 5}, {NULL}};
const Segment SGI24873[] = {{&S16318[44], 11}, {NULL}};
const Segment SGI24874[] = {{&S16318[44], 22}, {NULL}};
const Segment SGI24875[] = {{&S16318[30], 6}, {NULL}};
const Segment SGI24876[] = {{&S16318[114], 3}, {NULL}};
const Segment SGI24877[] = {{&S16318[80], 5}, {NULL}};
const Segment SGI24878[] = {{&S16322[44], 4}, {NULL}};
const Segment SGI24879[] = {{&S16322[44], 15}, {NULL}};
const Segment SGI24880[] = {{&S16322[30], 6}, {NULL}};
const Segment SGI24881[] = {{&S16825[44], 13}, {NULL}};
const Segment SGI24882[] = {{&S16825[44], 24}, {NULL}};
const Segment SGI24883[] = {{&S16825[30], 6}, {NULL}};
const Segment SGI24884[] = {{&S16322[107], 3}, {NULL}};
const Segment SGI24885[] = {{&S16322[73], 5}, {NULL}};
const Segment SGI24886[] = {{&S16825[85], 6}, {NULL}};
const Segment SGI24887[] = {{&S16328[44], 9}, {NULL}};
const Segment SGI24888[] = {{&S16328[44], 20}, {NULL}};
const Segment SGI24889[] = {{&S16328[30], 6}, {NULL}};
const Segment SGI24890[] = {{&S16328[112], 3}, {NULL}};
const Segment SGI24891[] = {{&S16328[78], 5}, {NULL}};
const Segment SGI24892[] = {{&S16329[44], 5}, {NULL}};
const Segment SGI24893[] = {{&S16329[44], 16}, {NULL}};
const Segment SGI24894[] = {{&S16329[30], 6}, {NULL}};
const Segment SGI24895[] = {{&S16329[108], 3}, {NULL}};
const Segment SGI24896[] = {{&S16329[74], 5}, {NULL}};
const Segment SGI24897[] = {{&S16842[44], 7}, {NULL}};
const Segment SGI24898[] = {{&S16842[44], 23}, {NULL}};
const Segment SGI24899[] = {{&S16842[30], 6}, {NULL}};
const Segment SGI24900[] = {{&S16842[115], 3}, {NULL}};
const Segment SGI24901[] = {{&S16842[81], 5}, {NULL}};
const Segment SGI24902[] = {{&S16339[44], 5}, {NULL}};
const Segment SGI24903[] = {{&S16339[44], 16}, {NULL}};
const Segment SGI24904[] = {{&S16339[30], 6}, {NULL}};
const Segment SGI24905[] = {{&S16339[108], 3}, {NULL}};
const Segment SGI24906[] = {{&S16339[74], 5}, {NULL}};
const Segment SGI24907[] = {{&S16342[44], 6}, {NULL}};
const Segment SGI24908[] = {{&S16342[44], 17}, {NULL}};
const Segment SGI24909[] = {{&S16342[30], 6}, {NULL}};
const Segment SGI24910[] = {{&S16342[109], 3}, {NULL}};
const Segment SGI24911[] = {{&S16342[75], 5}, {NULL}};
const Segment SGI24912[] = {{&S16344[44], 23}, {NULL}};
const Segment SGI24913[] = {{&S16344[30], 6}, {NULL}};
const Segment SGI24914[] = {{&S16344[115], 3}, {NULL}};
const Segment SGI24915[] = {{&S16344[81], 5}, {NULL}};
const Segment SGI24916[] = {{&S27330[44], 7}, {NULL}};
const Segment SGI24917[] = {{&S27330[44], 23}, {NULL}};
const Segment SGI24918[] = {{&S27330[30], 6}, {NULL}};
const Segment SGI24919[] = {{&S27330[115], 3}, {NULL}};
const Segment SGI24920[] = {{&S27330[81], 5}, {NULL}};
const Segment SGI24921[] = {{&S16345[44], 15}, {NULL}};
const Segment SGI24922[] = {{&S16345[44], 26}, {NULL}};
const Segment SGI24923[] = {{&S16345[30], 6}, {NULL}};
const Segment SGI24924[] = {{&S16345[118], 3}, {NULL}};
const Segment SGI24925[] = {{&S16345[84], 5}, {NULL}};
const Segment SGI24926[] = {{&S16347[44], 5}, {NULL}};
const Segment SGI24927[] = {{&S16347[44], 16}, {NULL}};
const Segment SGI24928[] = {{&S16347[30], 6}, {NULL}};
const Segment SGI24929[] = {{&S16347[108], 3}, {NULL}};
const Segment SGI24930[] = {{&S16347[74], 5}, {NULL}};
const Segment SGI24931[] = {{&S27339[44], 7}, {NULL}};
const Segment SGI24932[] = {{&S27339[44], 22}, {NULL}};
const Segment SGI24933[] = {{&S27339[30], 6}, {NULL}};
const Segment SGI24934[] = {{&S27339[114], 3}, {NULL}};
const Segment SGI24935[] = {{&S27339[80], 5}, {NULL}};
const Segment SGI24936[] = {{&S27341[44], 7}, {NULL}};
const Segment SGI24937[] = {{&S27341[44], 22}, {NULL}};
const Segment SGI24938[] = {{&S27341[30], 6}, {NULL}};
const Segment SGI24939[] = {{&S27341[114], 3}, {NULL}};
const Segment SGI24940[] = {{&S27341[80], 5}, {NULL}};
const Segment SGI24941[] = {{&S16356[44], 11}, {NULL}};
const Segment SGI24942[] = {{&S27342[44], 7}, {NULL}};
const Segment SGI24943[] = {{&S16356[44], 22}, {NULL}};
const Segment SGI24944[] = {{&S27342[44], 22}, {NULL}};
const Segment SGI24945[] = {{&S16356[30], 6}, {NULL}};
const Segment SGI24946[] = {{&S27342[30], 6}, {NULL}};
const Segment SGI24947[] = {{&S16356[114], 3}, {NULL}};
const Segment SGI24948[] = {{&S27342[114], 3}, {NULL}};
const Segment SGI24949[] = {{&S16356[80], 5}, {NULL}};
const Segment SGI24950[] = {{&S27342[80], 5}, {NULL}};
const Segment SGI24951[] = {{&S16359[44], 5}, {NULL}};
const Segment SGI24952[] = {{&S16359[44], 16}, {NULL}};
const Segment SGI24953[] = {{&S16359[30], 6}, {NULL}};
const Segment SGI24954[] = {{&S16359[108], 3}, {NULL}};
const Segment SGI24955[] = {{&S16359[74], 5}, {NULL}};
const Segment SGI24956[] = {{&S27347[44], 7}, {NULL}};
const Segment SGI24957[] = {{&S27347[44], 29}, {NULL}};
const Segment SGI24958[] = {{&S27347[30], 6}, {NULL}};
const Segment SGI24959[] = {{&S27347[121], 3}, {NULL}};
const Segment SGI24960[] = {{&S27347[87], 5}, {NULL}};
const Segment SGI24961[] = {{&S27348[42], 7}, {NULL}};
const Segment SGI24962[] = {{&S27348[30], 6}, {NULL}};
const Segment SGI24963[] = {{&S27348[112], 3}, {NULL}};
const Segment SGI24964[] = {{&S27348[78], 5}, {NULL}};
const Segment SGI24965[] = {{&S16363[44], 13}, {NULL}};
const Segment SGI24966[] = {{&S16363[44], 24}, {NULL}};
const Segment SGI24967[] = {{&S16363[30], 6}, {NULL}};
const Segment SGI24968[] = {{&S16363[116], 3}, {NULL}};
const Segment SGI24969[] = {{&S16363[82], 5}, {NULL}};
const Segment SGI24970[] = {{&S16877[44], 6}, {NULL}};
const Segment SGI24971[] = {{&S16877[44], 17}, {NULL}};
const Segment SGI24972[] = {{&S16877[30], 6}, {NULL}};
const Segment SGI24973[] = {{&S16877[109], 3}, {NULL}};
const Segment SGI24974[] = {{&S27350[44], 7}, {NULL}};
const Segment SGI24975[] = {{&S27350[44], 23}, {NULL}};
const Segment SGI24976[] = {{&S27350[30], 6}, {NULL}};
const Segment SGI24977[] = {{&S16877[75], 5}, {NULL}};
const Segment SGI24978[] = {{&S27350[115], 3}, {NULL}};
const Segment SGI24979[] = {{&S27350[81], 5}, {NULL}};
const Segment SGI24980[] = {{&S16879[44], 19}, {NULL}};
const Segment SGI24981[] = {{&S16879[44], 26}, {NULL}};
const Segment SGI24982[] = {{&S27352[44], 7}, {NULL}};
const Segment SGI24983[] = {{&S16879[30], 6}, {NULL}};
const Segment SGI24984[] = {{&S27352[44], 24}, {NULL}};
const Segment SGI24985[] = {{&S27352[30], 6}, {NULL}};
const Segment SGI24986[] = {{&S16879[118], 3}, {NULL}};
const Segment SGI24987[] = {{&S27352[116], 3}, {NULL}};
const Segment SGI24988[] = {{&S16879[84], 5}, {NULL}};
const Segment SGI24989[] = {{&S27352[82], 5}, {NULL}};
const Segment SGI24990[] = {{&S16370[38], 6}, {NULL}};
const Segment SGI24991[] = {{&S16370[31], 6}, {NULL}};
const Segment SGI24992[] = {{&S27355[44], 7}, {NULL}};
const Segment SGI24993[] = {{&S27355[44], 29}, {NULL}};
const Segment SGI24994[] = {{&S27355[30], 6}, {NULL}};
const Segment SGI24995[] = {{&S27355[121], 3}, {NULL}};
const Segment SGI24996[] = {{&S27355[87], 5}, {NULL}};
const Segment SGI24997[] = {{&S27356[44], 11}, {NULL}};
const Segment SGI24998[] = {{&S27356[44], 26}, {NULL}};
const Segment SGI24999[] = {{&S27356[30], 6}, {NULL}};
const Segment SGI25000[] = {{&S27356[118], 3}, {NULL}};
const Segment SGI25001[] = {{&S27356[84], 5}, {NULL}};
const Segment SGI25002[] = {{&S27357[44], 7}, {NULL}};
const Segment SGI25003[] = {{&S16886[44], 5}, {NULL}};
const Segment SGI25004[] = {{&S27357[44], 25}, {NULL}};
const Segment SGI25005[] = {{&S16886[44], 33}, {NULL}};
const Segment SGI25006[] = {{&S27357[30], 6}, {NULL}};
const Segment SGI25007[] = {{&S16886[30], 6}, {NULL}};
const Segment SGI25008[] = {{&S27357[117], 3}, {NULL}};
const Segment SGI25009[] = {{&S16886[125], 3}, {NULL}};
const Segment SGI25010[] = {{&S27357[83], 5}, {NULL}};
const Segment SGI25011[] = {{&S16886[91], 5}, {NULL}};
const Segment SGI25012[] = {{&S16376[44], 29}, {NULL}};
const Segment SGI25013[] = {{&S16376[44], 40}, {NULL}};
const Segment SGI25014[] = {{&S16376[30], 6}, {NULL}};
const Segment SGI25015[] = {{&S27358[44], 7}, {NULL}};
const Segment SGI25016[] = {{&S27358[44], 27}, {NULL}};
const Segment SGI25017[] = {{&S16887[44], 11}, {NULL}};
const Segment SGI25018[] = {{&S27358[30], 6}, {NULL}};
const Segment SGI25019[] = {{&S16887[44], 28}, {NULL}};
const Segment SGI25020[] = {{&S16376[132], 3}, {NULL}};
const Segment SGI25021[] = {{&S16887[30], 6}, {NULL}};
const Segment SGI25022[] = {{&S16376[98], 5}, {NULL}};
const Segment SGI25023[] = {{&S27358[119], 3}, {NULL}};
const Segment SGI25024[] = {{&S16887[120], 3}, {NULL}};
const Segment SGI25025[] = {{&S27358[85], 5}, {NULL}};
const Segment SGI25026[] = {{&S16887[86], 5}, {NULL}};
const Segment SGI25027[] = {{&S27359[44], 7}, {NULL}};
const Segment SGI25028[] = {{&S27359[44], 30}, {NULL}};
const Segment SGI25029[] = {{&S27359[30], 6}, {NULL}};
const Segment SGI25030[] = {{&S27359[122], 3}, {NULL}};
const Segment SGI25031[] = {{&S27359[88], 5}, {NULL}};
const Segment SGI25032[] = {{&S27362[44], 7}, {NULL}};
const Segment SGI25033[] = {{&S27362[44], 22}, {NULL}};
const Segment SGI25034[] = {{&S27362[30], 6}, {NULL}};
const Segment SGI25035[] = {{&S27362[114], 3}, {NULL}};
const Segment SGI25036[] = {{&S27362[80], 5}, {NULL}};
const Segment SGI25037[] = {{&S16893[44], 7}, {NULL}};
const Segment SGI25038[] = {{&S16893[44], 18}, {NULL}};
const Segment SGI25039[] = {{&S16893[30], 6}, {NULL}};
const Segment SGI25040[] = {{&S16893[110], 3}, {NULL}};
const Segment SGI25041[] = {{&S16893[76], 5}, {NULL}};
const Segment SGI25042[] = {{&S27363[44], 7}, {NULL}};
const Segment SGI25043[] = {{&S27363[44], 27}, {NULL}};
const Segment SGI25044[] = {{&S27363[30], 6}, {NULL}};
const Segment SGI25045[] = {{&S27363[119], 3}, {NULL}};
const Segment SGI25046[] = {{&S27363[85], 5}, {NULL}};
const Segment SGI25047[] = {{&S27365[44], 7}, {NULL}};
const Segment SGI25048[] = {{&S27365[44], 25}, {NULL}};
const Segment SGI25049[] = {{&S27365[30], 6}, {NULL}};
const Segment SGI25050[] = {{&S27365[117], 3}, {NULL}};
const Segment SGI25051[] = {{&S27365[83], 5}, {NULL}};
const Segment SGI25052[] = {{&S16387[44], 9}, {NULL}};
const Segment SGI25053[] = {{&S16387[44], 20}, {NULL}};
const Segment SGI25054[] = {{&S16387[30], 6}, {NULL}};
const Segment SGI25055[] = {{&S16387[112], 3}, {NULL}};
const Segment SGI25056[] = {{&S16387[78], 5}, {NULL}};
const Segment SGI25057[] = {{&S27366[44], 7}, {NULL}};
const Segment SGI25058[] = {{&S27366[44], 29}, {NULL}};
const Segment SGI25059[] = {{&S27366[30], 6}, {NULL}};
const Segment SGI25060[] = {{&S27366[121], 3}, {NULL}};
const Segment SGI25061[] = {{&S27366[87], 5}, {NULL}};
const Segment SGI25062[] = {{&S16899[44], 50}, {NULL}};
const Segment SGI25063[] = {{&S16899[30], 6}, {NULL}};
const Segment SGI25064[] = {{&S16899[142], 3}, {NULL}};
const Segment SGI25065[] = {{&S27367[44], 7}, {NULL}};
const Segment SGI25066[] = {{&S16899[108], 5}, {NULL}};
const Segment SGI25067[] = {{&S27367[44], 25}, {NULL}};
const Segment SGI25068[] = {{&S27367[30], 6}, {NULL}};
const Segment SGI25069[] = {{&S27367[117], 3}, {NULL}};
const Segment SGI25070[] = {{&S27367[83], 5}, {NULL}};
const Segment SGI25071[] = {{&S16900[44], 6}, {NULL}};
const Segment SGI25072[] = {{&S16900[44], 17}, {NULL}};
const Segment SGI25073[] = {{&S16900[30], 6}, {NULL}};
const Segment SGI25074[] = {{&S16900[109], 3}, {NULL}};
const Segment SGI25075[] = {{&S16900[75], 5}, {NULL}};
const Segment SGI25076[] = {{&S27368[101], 7}, {NULL}};
const Segment SGI25077[] = {{&S27368[101], 33}, {NULL}};
const Segment SGI25078[] = {{&S27368[87], 6}, {NULL}};
const Segment SGI25079[] = {{&S16391[44], 8}, {NULL}};
const Segment SGI25080[] = {{&S16391[44], 29}, {NULL}};
const Segment SGI25081[] = {{&S16391[30], 6}, {NULL}};
const Segment SGI25082[] = {{&S27368[182], 3}, {NULL}};
const Segment SGI25083[] = {{&S16391[121], 3}, {NULL}};
const Segment SGI25084[] = {{&S27368[148], 5}, {NULL}};
const Segment SGI25085[] = {{&S16391[87], 5}, {NULL}};
const Segment SGI25086[] = {{&S27369[97], 7}, {NULL}};
const Segment SGI25087[] = {{&S16392[44], 8}, {NULL}};
const Segment SGI25088[] = {{&S27369[97], 33}, {NULL}};
const Segment SGI25089[] = {{&S16392[44], 19}, {NULL}};
const Segment SGI25090[] = {{&S27369[83], 6}, {NULL}};
const Segment SGI25091[] = {{&S16392[30], 6}, {NULL}};
const Segment SGI25092[] = {{&S16392[111], 3}, {NULL}};
const Segment SGI25093[] = {{&S27369[178], 3}, {NULL}};
const Segment SGI25094[] = {{&S16392[77], 5}, {NULL}};
const Segment SGI25095[] = {{&S27369[144], 5}, {NULL}};
const Segment SGI25096[] = {{&S16393[44], 10}, {NULL}};
const Segment SGI25097[] = {{&S16393[44], 21}, {NULL}};
const Segment SGI25098[] = {{&S16393[30], 6}, {NULL}};
const Segment SGI25099[] = {{&S16393[113], 3}, {NULL}};
const Segment SGI25100[] = {{&S16905[44], 7}, {NULL}};
const Segment SGI25101[] = {{&S16905[44], 18}, {NULL}};
const Segment SGI25102[] = {{&S16905[30], 6}, {NULL}};
const Segment SGI25103[] = {{&S16393[79], 5}, {NULL}};
const Segment SGI25104[] = {{&S27371[38], 7}, {NULL}};
const Segment SGI25105[] = {{&S27371[31], 6}, {NULL}};
const Segment SGI25106[] = {{&S16905[110], 3}, {NULL}};
const Segment SGI25107[] = {{&S16905[76], 5}, {NULL}};
const Segment SGI25108[] = {{&S16394[44], 12}, {NULL}};
const Segment SGI25109[] = {{&S16394[44], 29}, {NULL}};
const Segment SGI25110[] = {{&S16394[30], 6}, {NULL}};
const Segment SGI25111[] = {{&S27372[44], 7}, {NULL}};
const Segment SGI25112[] = {{&S16394[121], 3}, {NULL}};
const Segment SGI25113[] = {{&S27372[44], 27}, {NULL}};
const Segment SGI25114[] = {{&S27372[30], 6}, {NULL}};
const Segment SGI25115[] = {{&S16394[87], 5}, {NULL}};
const Segment SGI25116[] = {{&S27372[88], 4}, {NULL}};
const Segment SGI25117[] = {{&S27372[88], 3}, {NULL}};
const Segment SGI25118[] = {{&S16907[44], 35}, {NULL}};
const Segment SGI25119[] = {{&S16907[30], 6}, {NULL}};
const Segment SGI25120[] = {{&S16907[145], 3}, {NULL}};
const Segment SGI25121[] = {{&S16907[111], 5}, {NULL}};
const Segment SGI25122[] = {{&S27374[44], 7}, {NULL}};
const Segment SGI25123[] = {{&S27374[44], 24}, {NULL}};
const Segment SGI25124[] = {{&S27374[30], 6}, {NULL}};
const Segment SGI25125[] = {{&S27374[85], 4}, {NULL}};
const Segment SGI25126[] = {{&S27374[85], 3}, {NULL}};
const Segment SGI25127[] = {{&S27375[44], 7}, {NULL}};
const Segment SGI25128[] = {{&S27375[44], 24}, {NULL}};
const Segment SGI25129[] = {{&S27375[30], 6}, {NULL}};
const Segment SGI25130[] = {{&S27375[85], 4}, {NULL}};
const Segment SGI25131[] = {{&S27375[85], 3}, {NULL}};
const Segment SGI25132[] = {{&S27376[44], 7}, {NULL}};
const Segment SGI25133[] = {{&S27376[44], 24}, {NULL}};
const Segment SGI25134[] = {{&S27376[30], 6}, {NULL}};
const Segment SGI25135[] = {{&S27376[116], 3}, {NULL}};
const Segment SGI25136[] = {{&S27376[82], 5}, {NULL}};
const Segment SGI25137[] = {{&S27377[94], 7}, {NULL}};
const Segment SGI25138[] = {{&S27377[94], 29}, {NULL}};
const Segment SGI25139[] = {{&S27377[80], 6}, {NULL}};
const Segment SGI25140[] = {{&S16914[44], 24}, {NULL}};
const Segment SGI25141[] = {{&S16914[30], 6}, {NULL}};
const Segment SGI25142[] = {{&S27377[171], 3}, {NULL}};
const Segment SGI25143[] = {{&S27377[137], 5}, {NULL}};
const Segment SGI25144[] = {{&S16404[44], 19}, {NULL}};
const Segment SGI25145[] = {{&S16404[44], 36}, {NULL}};
const Segment SGI25146[] = {{&S16404[30], 6}, {NULL}};
const Segment SGI25147[] = {{&S27378[101], 7}, {NULL}};
const Segment SGI25148[] = {{&S16404[128], 3}, {NULL}};
const Segment SGI25149[] = {{&S27378[101], 29}, {NULL}};
const Segment SGI25150[] = {{&S16915[44], 35}, {NULL}};
const Segment SGI25151[] = {{&S16915[30], 6}, {NULL}};
const Segment SGI25152[] = {{&S27378[87], 6}, {NULL}};
const Segment SGI25153[] = {{&S16404[94], 5}, {NULL}};
const Segment SGI25154[] = {{&S16915[145], 3}, {NULL}};
const Segment SGI25155[] = {{&S27378[178], 3}, {NULL}};
const Segment SGI25156[] = {{&S16915[111], 5}, {NULL}};
const Segment SGI25157[] = {{&S27378[144], 5}, {NULL}};
const Segment SGI25158[] = {{&S27380[44], 7}, {NULL}};
const Segment SGI25159[] = {{&S16917[44], 7}, {NULL}};
const Segment SGI25160[] = {{&S27380[44], 29}, {NULL}};
const Segment SGI25161[] = {{&S16917[44], 18}, {NULL}};
const Segment SGI25162[] = {{&S27380[30], 6}, {NULL}};
const Segment SGI25163[] = {{&S16917[30], 6}, {NULL}};
const Segment SGI25164[] = {{&S16917[110], 3}, {NULL}};
const Segment SGI25165[] = {{&S27380[90], 4}, {NULL}};
const Segment SGI25166[] = {{&S27380[90], 3}, {NULL}};
const Segment SGI25167[] = {{&S16917[76], 5}, {NULL}};
const Segment SGI25168[] = {{&S27382[44], 7}, {NULL}};
const Segment SGI25169[] = {{&S27382[44], 24}, {NULL}};
const Segment SGI25170[] = {{&S27382[30], 6}, {NULL}};
const Segment SGI25171[] = {{&S27382[85], 6}, {NULL}};
const Segment SGI25172[] = {{&S27383[101], 7}, {NULL}};
const Segment SGI25173[] = {{&S27383[101], 29}, {NULL}};
const Segment SGI25174[] = {{&S27383[87], 6}, {NULL}};
const Segment SGI25175[] = {{&S27383[178], 3}, {NULL}};
const Segment SGI25176[] = {{&S27383[144], 5}, {NULL}};
const Segment SGI25177[] = {{&S27384[44], 7}, {NULL}};
const Segment SGI25178[] = {{&S27384[44], 22}, {NULL}};
const Segment SGI25179[] = {{&S27384[30], 6}, {NULL}};
const Segment SGI25180[] = {{&S27384[83], 4}, {NULL}};
const Segment SGI25181[] = {{&S16414[44], 8}, {NULL}};
const Segment SGI25182[] = {{&S27384[83], 3}, {NULL}};
const Segment SGI25183[] = {{&S16414[44], 19}, {NULL}};
const Segment SGI25184[] = {{&S16414[30], 6}, {NULL}};
const Segment SGI25185[] = {{&S16414[111], 3}, {NULL}};
const Segment SGI25186[] = {{&S16414[77], 5}, {NULL}};
const Segment SGI25187[] = {{&S27385[44], 7}, {NULL}};
const Segment SGI25188[] = {{&S27385[44], 23}, {NULL}};
const Segment SGI25189[] = {{&S27385[30], 6}, {NULL}};
const Segment SGI25190[] = {{&S27385[84], 4}, {NULL}};
const Segment SGI25191[] = {{&S27385[84], 3}, {NULL}};
const Segment SGI25192[] = {{&S16415[44], 9}, {NULL}};
const Segment SGI25193[] = {{&S16415[44], 20}, {NULL}};
const Segment SGI25194[] = {{&S16415[30], 6}, {NULL}};
const Segment SGI25195[] = {{&S16415[112], 3}, {NULL}};
const Segment SGI25196[] = {{&S16415[78], 5}, {NULL}};
const Segment SGI25197[] = {{&S16416[44], 6}, {NULL}};
const Segment SGI25198[] = {{&S16416[44], 23}, {NULL}};
const Segment SGI25199[] = {{&S16416[30], 6}, {NULL}};
const Segment SGI25200[] = {{&S16416[115], 3}, {NULL}};
const Segment SGI25201[] = {{&S16416[81], 5}, {NULL}};
const Segment SGI25202[] = {{&S27388[44], 7}, {NULL}};
const Segment SGI25203[] = {{&S27388[44], 24}, {NULL}};
const Segment SGI25204[] = {{&S27388[30], 6}, {NULL}};
const Segment SGI25205[] = {{&S27388[85], 6}, {NULL}};
const Segment SGI25206[] = {{&S16417[44], 14}, {NULL}};
const Segment SGI25207[] = {{&S16417[44], 25}, {NULL}};
const Segment SGI25208[] = {{&S16417[30], 6}, {NULL}};
const Segment SGI25209[] = {{&S16417[117], 3}, {NULL}};
const Segment SGI25210[] = {{&S16417[83], 5}, {NULL}};
const Segment SGI25211[] = {{&S27389[44], 7}, {NULL}};
const Segment SGI25212[] = {{&S27389[44], 29}, {NULL}};
const Segment SGI25213[] = {{&S27389[30], 6}, {NULL}};
const Segment SGI25214[] = {{&S27389[90], 6}, {NULL}};
const Segment SGI25215[] = {{&S16419[44], 8}, {NULL}};
const Segment SGI25216[] = {{&S16419[44], 19}, {NULL}};
const Segment SGI25217[] = {{&S16419[30], 6}, {NULL}};
const Segment SGI25218[] = {{&S16419[111], 3}, {NULL}};
const Segment SGI25219[] = {{&S16419[77], 5}, {NULL}};
const Segment SGI25220[] = {{&S27392[44], 7}, {NULL}};
const Segment SGI25221[] = {{&S27392[44], 22}, {NULL}};
const Segment SGI25222[] = {{&S16420[44], 13}, {NULL}};
const Segment SGI25223[] = {{&S27392[30], 6}, {NULL}};
const Segment SGI25224[] = {{&S16420[44], 30}, {NULL}};
const Segment SGI25225[] = {{&S16420[30], 6}, {NULL}};
const Segment SGI25226[] = {{&S16420[122], 3}, {NULL}};
const Segment SGI25227[] = {{&S27392[83], 6}, {NULL}};
const Segment SGI25228[] = {{&S16420[88], 5}, {NULL}};
const Segment SGI25229[] = {{&S27393[44], 7}, {NULL}};
const Segment SGI25230[] = {{&S27393[44], 27}, {NULL}};
const Segment SGI25231[] = {{&S27393[30], 6}, {NULL}};
const Segment SGI25232[] = {{&S27393[88], 6}, {NULL}};
const Segment SGI25233[] = {{&S27394[44], 7}, {NULL}};
const Segment SGI25234[] = {{&S27394[44], 24}, {NULL}};
const Segment SGI25235[] = {{&S27394[30], 6}, {NULL}};
const Segment SGI25236[] = {{&S27394[116], 3}, {NULL}};
const Segment SGI25237[] = {{&S27394[82], 5}, {NULL}};
const Segment SGI25238[] = {{&S27395[44], 7}, {NULL}};
const Segment SGI25239[] = {{&S27395[44], 23}, {NULL}};
const Segment SGI25240[] = {{&S27395[30], 6}, {NULL}};
const Segment SGI25241[] = {{&S27395[84], 6}, {NULL}};
const Segment SGI25242[] = {{&S16427[44], 11}, {NULL}};
const Segment SGI25243[] = {{&S16427[44], 22}, {NULL}};
const Segment SGI25244[] = {{&S16427[30], 6}, {NULL}};
const Segment SGI25245[] = {{&S16427[114], 3}, {NULL}};
const Segment SGI25246[] = {{&S16427[80], 5}, {NULL}};
const Segment SGI25247[] = {{&S16958[44], 12}, {NULL}};
const Segment SGI25248[] = {{&S16958[44], 23}, {NULL}};
const Segment SGI25249[] = {{&S16958[30], 6}, {NULL}};
const Segment SGI25250[] = {{&S16958[115], 3}, {NULL}};
const Segment SGI25251[] = {{&S16958[81], 5}, {NULL}};
const Segment SGI25252[] = {{&S16444[44], 10}, {NULL}};
const Segment SGI25253[] = {{&S16444[44], 21}, {NULL}};
const Segment SGI25254[] = {{&S16444[30], 6}, {NULL}};
const Segment SGI25255[] = {{&S16444[113], 3}, {NULL}};
const Segment SGI25256[] = {{&S16444[79], 5}, {NULL}};
const Segment SGI25257[] = {{&S16445[44], 10}, {NULL}};
const Segment SGI25258[] = {{&S16445[44], 21}, {NULL}};
const Segment SGI25259[] = {{&S16445[30], 6}, {NULL}};
const Segment SGI25260[] = {{&S16445[113], 3}, {NULL}};
const Segment SGI25261[] = {{&S16445[79], 5}, {NULL}};
const Segment SGI25262[] = {{&S16963[44], 11}, {NULL}};
const Segment SGI25263[] = {{&S16963[44], 26}, {NULL}};
const Segment SGI25264[] = {{&S16963[30], 6}, {NULL}};
const Segment SGI25265[] = {{&S16963[118], 3}, {NULL}};
const Segment SGI25266[] = {{&S16963[84], 5}, {NULL}};
const Segment SGI25267[] = {{&S16460[44], 10}, {NULL}};
const Segment SGI25268[] = {{&S16460[44], 21}, {NULL}};
const Segment SGI25269[] = {{&S16460[30], 6}, {NULL}};
const Segment SGI25270[] = {{&S16460[113], 3}, {NULL}};
const Segment SGI25271[] = {{&S16460[79], 5}, {NULL}};
const Segment SGI25272[] = {{&S27442[44], 11}, {NULL}};
const Segment SGI25273[] = {{&S27442[44], 22}, {NULL}};
const Segment SGI25274[] = {{&S27442[30], 6}, {NULL}};
const Segment SGI25275[] = {{&S27442[114], 3}, {NULL}};
const Segment SGI25276[] = {{&S27442[80], 5}, {NULL}};
const Segment SGI25277[] = {{&S27443[44], 11}, {NULL}};
const Segment SGI25278[] = {{&S27443[44], 22}, {NULL}};
const Segment SGI25279[] = {{&S27443[30], 6}, {NULL}};
const Segment SGI25280[] = {{&S27443[114], 3}, {NULL}};
const Segment SGI25281[] = {{&S27443[80], 5}, {NULL}};
const Segment SGI25282[] = {{&S27448[38], 11}, {NULL}};
const Segment SGI25283[] = {{&S27448[31], 6}, {NULL}};
const Segment SGI25284[] = {{&S16993[38], 7}, {NULL}};
const Segment SGI25285[] = {{&S16476[44], 6}, {NULL}};
const Segment SGI25286[] = {{&S16993[31], 6}, {NULL}};
const Segment SGI25287[] = {{&S16476[44], 17}, {NULL}};
const Segment SGI25288[] = {{&S16476[30], 6}, {NULL}};
const Segment SGI25289[] = {{&S16476[109], 3}, {NULL}};
const Segment SGI25290[] = {{&S16476[75], 5}, {NULL}};
const Segment SGI25291[] = {{&S16998[44], 5}, {NULL}};
const Segment SGI25292[] = {{&S16998[44], 33}, {NULL}};
const Segment SGI25293[] = {{&S16998[30], 6}, {NULL}};
const Segment SGI25294[] = {{&S16998[125], 3}, {NULL}};
const Segment SGI25295[] = {{&S16998[91], 5}, {NULL}};
const Segment SGI25296[] = {{&S17024[44], 12}, {NULL}};
const Segment SGI25297[] = {{&S17024[44], 23}, {NULL}};
const Segment SGI25298[] = {{&S17024[30], 6}, {NULL}};
const Segment SGI25299[] = {{&S17024[115], 3}, {NULL}};
const Segment SGI25300[] = {{&S17024[81], 5}, {NULL}};
const Segment SGI25301[] = {{&S17031[42], 8}, {NULL}};
const Segment SGI25302[] = {{&S17031[30], 6}, {NULL}};
const Segment SGI25303[] = {{&S17031[125], 3}, {NULL}};
const Segment SGI25304[] = {{&S17031[91], 5}, {NULL}};
const Segment SGI25305[] = {{&S17049[44], 10}, {NULL}};
const Segment SGI25306[] = {{&S17049[44], 22}, {NULL}};
const Segment SGI25307[] = {{&S17049[30], 6}, {NULL}};
const Segment SGI25308[] = {{&S17049[114], 3}, {NULL}};
const Segment SGI25309[] = {{&S17049[80], 5}, {NULL}};
const Segment SGI25310[] = {{&S17056[44], 15}, {NULL}};
const Segment SGI25311[] = {{&S17056[44], 26}, {NULL}};
const Segment SGI25312[] = {{&S17056[30], 6}, {NULL}};
const Segment SGI25313[] = {{&S17056[118], 3}, {NULL}};
const Segment SGI25314[] = {{&S17056[84], 5}, {NULL}};
const Segment SGI25315[] = {{&S17057[44], 11}, {NULL}};
const Segment SGI25316[] = {{&S17057[44], 22}, {NULL}};
const Segment SGI25317[] = {{&S17057[30], 6}, {NULL}};
const Segment SGI25318[] = {{&S17057[114], 3}, {NULL}};
const Segment SGI25319[] = {{&S17057[80], 5}, {NULL}};
const Segment SGI25320[] = {{&S17058[44], 6}, {NULL}};
const Segment SGI25321[] = {{&S17058[44], 17}, {NULL}};
const Segment SGI25322[] = {{&S17058[30], 6}, {NULL}};
const Segment SGI25323[] = {{&S17058[109], 3}, {NULL}};
const Segment SGI25324[] = {{&S17058[75], 5}, {NULL}};
const Segment SGI25325[] = {{&S17060[44], 9}, {NULL}};
const Segment SGI25326[] = {{&S17060[44], 20}, {NULL}};
const Segment SGI25327[] = {{&S17060[30], 6}, {NULL}};
const Segment SGI25328[] = {{&S17060[112], 3}, {NULL}};
const Segment SGI25329[] = {{&S17060[78], 5}, {NULL}};
const Segment SGI25330[] = {{&S17065[44], 34}, {NULL}};
const Segment SGI25331[] = {{&S17065[30], 6}, {NULL}};
const Segment SGI25332[] = {{&S17065[126], 3}, {NULL}};
const Segment SGI25333[] = {{&S17065[92], 5}, {NULL}};
const Segment SGI25334[] = {{&S17069[44], 9}, {NULL}};
const Segment SGI25335[] = {{&S17069[44], 20}, {NULL}};
const Segment SGI25336[] = {{&S17069[30], 6}, {NULL}};
const Segment SGI25337[] = {{&S17069[112], 3}, {NULL}};
const Segment SGI25338[] = {{&S17069[78], 5}, {NULL}};
const Segment SGI25339[] = {{&S17094[44], 10}, {NULL}};
const Segment SGI25340[] = {{&S17094[44], 21}, {NULL}};
const Segment SGI25341[] = {{&S17094[30], 6}, {NULL}};
const Segment SGI25342[] = {{&S17094[113], 3}, {NULL}};
const Segment SGI25343[] = {{&S17094[79], 5}, {NULL}};
const Segment SGI25344[] = {{&S17098[44], 10}, {NULL}};
const Segment SGI25345[] = {{&S17098[44], 27}, {NULL}};
const Segment SGI25346[] = {{&S17098[30], 6}, {NULL}};
const Segment SGI25347[] = {{&S17098[119], 3}, {NULL}};
const Segment SGI25348[] = {{&S17098[85], 5}, {NULL}};
const Segment SGI25349[] = {{&S16484[44], 12}, {NULL}};
const Segment SGI25350[] = {{&S16484[44], 23}, {NULL}};
const Segment SGI25351[] = {{&S16484[30], 6}, {NULL}};
const Segment SGI25352[] = {{&S16484[115], 3}, {NULL}};
const Segment SGI25353[] = {{&S16484[81], 5}, {NULL}};
const Segment SGI25354[] = {{&S16488[44], 10}, {NULL}};
const Segment SGI25355[] = {{&S16488[44], 27}, {NULL}};
const Segment SGI25356[] = {{&S16488[30], 6}, {NULL}};
const Segment SGI25357[] = {{&S16488[119], 3}, {NULL}};
const Segment SGI25358[] = {{&S16488[85], 5}, {NULL}};
const Segment SGI25359[] = {{&S16490[44], 9}, {NULL}};
const Segment SGI25360[] = {{&S16490[44], 20}, {NULL}};
const Segment SGI25361[] = {{&S16490[30], 6}, {NULL}};
const Segment SGI25362[] = {{&S16490[112], 3}, {NULL}};
const Segment SGI25363[] = {{&S16490[78], 5}, {NULL}};
const Segment SGI25364[] = {{&S16491[44], 7}, {NULL}};
const Segment SGI25365[] = {{&S16491[44], 18}, {NULL}};
const Segment SGI25366[] = {{&S16491[30], 6}, {NULL}};
const Segment SGI25367[] = {{&S16491[110], 3}, {NULL}};
const Segment SGI25368[] = {{&S16491[76], 5}, {NULL}};
const Segment SGI25369[] = {{&S16492[44], 6}, {NULL}};
const Segment SGI25370[] = {{&S16492[44], 17}, {NULL}};
const Segment SGI25371[] = {{&S16492[30], 6}, {NULL}};
const Segment SGI25372[] = {{&S16492[109], 3}, {NULL}};
const Segment SGI25373[] = {{&S16492[75], 5}, {NULL}};
const Segment SGI25374[] = {{&S16493[44], 8}, {NULL}};
const Segment SGI25375[] = {{&S16493[44], 29}, {NULL}};
const Segment SGI25376[] = {{&S16493[30], 6}, {NULL}};
const Segment SGI25377[] = {{&S16493[121], 3}, {NULL}};
const Segment SGI25378[] = {{&S16493[87], 5}, {NULL}};
const Segment SGI25379[] = {{&S16503[44], 4}, {NULL}};
const Segment SGI25380[] = {{&S16503[44], 15}, {NULL}};
const Segment SGI25381[] = {{&S16503[30], 6}, {NULL}};
const Segment SGI25382[] = {{&S16503[107], 3}, {NULL}};
const Segment SGI25383[] = {{&S16503[73], 5}, {NULL}};
const Segment SGI25384[] = {{&S16507[44], 10}, {NULL}};
const Segment SGI25385[] = {{&S16507[44], 21}, {NULL}};
const Segment SGI25386[] = {{&S16507[30], 6}, {NULL}};
const Segment SGI25387[] = {{&S16507[113], 3}, {NULL}};
const Segment SGI25388[] = {{&S16507[79], 5}, {NULL}};
const Segment SGI25389[] = {{&S16512[44], 10}, {NULL}};
const Segment SGI25390[] = {{&S16512[44], 38}, {NULL}};
const Segment SGI25391[] = {{&S16512[30], 6}, {NULL}};
const Segment SGI25392[] = {{&S16512[130], 3}, {NULL}};
const Segment SGI25393[] = {{&S16512[96], 5}, {NULL}};
const Segment SGI25394[] = {{&S16513[44], 15}, {NULL}};
const Segment SGI25395[] = {{&S16513[44], 26}, {NULL}};
const Segment SGI25396[] = {{&S16513[30], 6}, {NULL}};
const Segment SGI25397[] = {{&S16513[118], 3}, {NULL}};
const Segment SGI25398[] = {{&S16513[84], 5}, {NULL}};
const Segment SGI25399[] = {{&S16514[44], 8}, {NULL}};
const Segment SGI25400[] = {{&S16514[44], 19}, {NULL}};
const Segment SGI25401[] = {{&S16514[30], 6}, {NULL}};
const Segment SGI25402[] = {{&S16514[111], 3}, {NULL}};
const Segment SGI25403[] = {{&S16514[77], 5}, {NULL}};
const Segment SGI25404[] = {{&S16522[44], 9}, {NULL}};
const Segment SGI25405[] = {{&S16522[44], 30}, {NULL}};
const Segment SGI25406[] = {{&S16522[30], 6}, {NULL}};
const Segment SGI25407[] = {{&S16522[122], 3}, {NULL}};
const Segment SGI25408[] = {{&S16522[88], 5}, {NULL}};
const Segment SGI25409[] = {{&S16527[44], 5}, {NULL}};
const Segment SGI25410[] = {{&S16527[44], 16}, {NULL}};
const Segment SGI25411[] = {{&S16527[30], 6}, {NULL}};
const Segment SGI25412[] = {{&S16527[108], 3}, {NULL}};
const Segment SGI25413[] = {{&S17155[44], 10}, {NULL}};
const Segment SGI25414[] = {{&S17155[44], 21}, {NULL}};
const Segment SGI25415[] = {{&S16527[74], 5}, {NULL}};
const Segment SGI25416[] = {{&S17155[30], 6}, {NULL}};
const Segment SGI25417[] = {{&S17155[113], 3}, {NULL}};
const Segment SGI25418[] = {{&S17155[79], 5}, {NULL}};
const Segment SGI25419[] = {{&S17156[44], 5}, {NULL}};
const Segment SGI25420[] = {{&S17156[44], 16}, {NULL}};
const Segment SGI25421[] = {{&S17156[30], 6}, {NULL}};
const Segment SGI25422[] = {{&S17156[108], 3}, {NULL}};
const Segment SGI25423[] = {{&S17156[74], 5}, {NULL}};
const Segment SGI25424[] = {{&S16533[44], 9}, {NULL}};
const Segment SGI25425[] = {{&S16533[44], 20}, {NULL}};
const Segment SGI25426[] = {{&S16533[30], 6}, {NULL}};
const Segment SGI25427[] = {{&S16533[112], 3}, {NULL}};
const Segment SGI25428[] = {{&S16533[78], 5}, {NULL}};
const Segment SGI25429[] = {{&S16535[44], 14}, {NULL}};
const Segment SGI25430[] = {{&S16535[44], 25}, {NULL}};
const Segment SGI25431[] = {{&S16535[30], 6}, {NULL}};
const Segment SGI25432[] = {{&S16535[117], 3}, {NULL}};
const Segment SGI25433[] = {{&S16535[83], 5}, {NULL}};
const Segment SGI25434[] = {{&S17171[44], 5}, {NULL}};
const Segment SGI25435[] = {{&S17171[44], 33}, {NULL}};
const Segment SGI25436[] = {{&S17171[30], 6}, {NULL}};
const Segment SGI25437[] = {{&S17171[125], 3}, {NULL}};
const Segment SGI25438[] = {{&S17171[91], 5}, {NULL}};
const Segment SGI25439[] = {{&S17172[44], 6}, {&S17172[51], 6}, {&S17172[58], 8}, {NULL}};
const Segment SGI25440[] = {{&S17172[30], 6}, {NULL}};
const Segment SGI25441[] = {{&S17172[114], 3}, {NULL}};
const Segment SGI25442[] = {{&S17172[80], 5}, {NULL}};
const Segment SGI25443[] = {{&S16546[44], 9}, {NULL}};
const Segment SGI25444[] = {{&S16546[44], 20}, {NULL}};
const Segment SGI25445[] = {{&S16546[30], 6}, {NULL}};
const Segment SGI25446[] = {{&S16546[112], 3}, {NULL}};
const Segment SGI25447[] = {{&S16546[78], 5}, {NULL}};
const Segment SGI25448[] = {{&S16550[44], 15}, {NULL}};
const Segment SGI25449[] = {{&S16550[44], 26}, {NULL}};
const Segment SGI25450[] = {{&S16550[30], 6}, {NULL}};
const Segment SGI25451[] = {{&S16550[118], 3}, {NULL}};
const Segment SGI25452[] = {{&S16550[84], 5}, {NULL}};
const Segment SGI25453[] = {{&S16558[44], 9}, {NULL}};
const Segment SGI25454[] = {{&S16558[44], 35}, {NULL}};
const Segment SGI25455[] = {{&S16558[30], 6}, {NULL}};
const Segment SGI25456[] = {{&S16558[127], 3}, {NULL}};
const Segment SGI25457[] = {{&S16558[93], 5}, {NULL}};
const Segment SGI25458[] = {{&S16563[44], 4}, {NULL}};
const Segment SGI25459[] = {{&S16563[44], 25}, {NULL}};
const Segment SGI25460[] = {{&S16563[30], 6}, {NULL}};
const Segment SGI25461[] = {{&S16563[117], 3}, {NULL}};
const Segment SGI25462[] = {{&S16563[83], 5}, {NULL}};
const Segment SGI25463[] = {{&S16569[64], 5}, {NULL}};
const Segment SGI25464[] = {{&S16570[44], 7}, {NULL}};
const Segment SGI25465[] = {{&S16570[44], 28}, {NULL}};
const Segment SGI25466[] = {{&S16570[30], 6}, {NULL}};
const Segment SGI25467[] = {{&S16570[120], 3}, {NULL}};
const Segment SGI25468[] = {{&S16570[86], 5}, {NULL}};
const Segment SGI25469[] = {{&S16572[44], 15}, {NULL}};
const Segment SGI25470[] = {{&S16572[44], 26}, {NULL}};
const Segment SGI25471[] = {{&S16572[30], 6}, {NULL}};
const Segment SGI25472[] = {{&S16572[118], 3}, {NULL}};
const Segment SGI25473[] = {{&S16572[84], 5}, {NULL}};
const Segment SGI25474[] = {{&S16576[44], 22}, {NULL}};
const Segment SGI25475[] = {{&S16576[44], 33}, {NULL}};
const Segment SGI25476[] = {{&S16576[30], 6}, {NULL}};
const Segment SGI25477[] = {{&S16576[125], 3}, {NULL}};
const Segment SGI25478[] = {{&S16576[91], 5}, {NULL}};
const Segment SGI25479[] = {{&S16577[44], 11}, {NULL}};
const Segment SGI25480[] = {{&S16577[44], 22}, {NULL}};
const Segment SGI25481[] = {{&S16577[30], 6}, {NULL}};
const Segment SGI25482[] = {{&S16577[114], 3}, {NULL}};
const Segment SGI25483[] = {{&S16577[80], 5}, {NULL}};
const Segment SGI25484[] = {{&S16581[44], 31}, {NULL}};
const Segment SGI25485[] = {{&S16581[30], 6}, {NULL}};
const Segment SGI25486[] = {{&S16582[44], 9}, {NULL}};
const Segment SGI25487[] = {{&S16582[44], 30}, {NULL}};
const Segment SGI25488[] = {{&S16582[30], 6}, {NULL}};
const Segment SGI25489[] = {{&S16582[91], 6}, {NULL}};
const Segment SGI25490[] = {{&S16585[44], 10}, {NULL}};
const Segment SGI25491[] = {{&S16585[44], 21}, {NULL}};
const Segment SGI25492[] = {{&S16585[30], 6}, {NULL}};
const Segment SGI25493[] = {{&S16585[113], 3}, {NULL}};
const Segment SGI25494[] = {{&S16585[79], 5}, {NULL}};
const Segment SGI25495[] = {{&S16587[44], 5}, {NULL}};
const Segment SGI25496[] = {{&S16587[44], 26}, {NULL}};
const Segment SGI25497[] = {{&S16587[30], 6}, {NULL}};
const Segment SGI25498[] = {{&S16587[118], 3}, {NULL}};
const Segment SGI25499[] = {{&S16587[84], 5}, {NULL}};
const Segment SGI25500[] = {{&S16593[44], 9}, {NULL}};
const Segment SGI25501[] = {{&S16593[44], 20}, {NULL}};
const Segment SGI25502[] = {{&S16593[30], 6}, {NULL}};
const Segment SGI25503[] = {{&S16593[112], 3}, {NULL}};
const Segment SGI25504[] = {{&S16593[78], 5}, {NULL}};
const Segment SGI25505[] = {{&S17244[44], 7}, {NULL}};
const Segment SGI25506[] = {{&S17244[44], 24}, {NULL}};
const Segment SGI25507[] = {{&S17244[30], 6}, {NULL}};
const Segment SGI25508[] = {{&S17244[116], 3}, {NULL}};
const Segment SGI25509[] = {{&S17244[82], 5}, {NULL}};
const Segment SGI25510[] = {{&S16598[44], 12}, {NULL}};
const Segment SGI25511[] = {{&S16598[44], 23}, {NULL}};
const Segment SGI25512[] = {{&S16598[30], 6}, {NULL}};
const Segment SGI25513[] = {{&S16598[115], 3}, {NULL}};
const Segment SGI25514[] = {{&S16598[81], 5}, {NULL}};
const Segment SGI25515[] = {{&S16600[44], 11}, {NULL}};
const Segment SGI25516[] = {{&S16600[44], 37}, {NULL}};
const Segment SGI25517[] = {{&S16600[30], 6}, {NULL}};
const Segment SGI25518[] = {{&S16600[129], 3}, {NULL}};
const Segment SGI25519[] = {{&S16600[95], 5}, {NULL}};
const Segment SGI25520[] = {{&S17251[44], 5}, {NULL}};
const Segment SGI25521[] = {{&S17251[44], 22}, {NULL}};
const Segment SGI25522[] = {{&S17251[30], 6}, {NULL}};
const Segment SGI25523[] = {{&S17251[114], 3}, {NULL}};
const Segment SGI25524[] = {{&S17251[80], 5}, {NULL}};
const Segment SGI25525[] = {{&S16605[44], 9}, {NULL}};
const Segment SGI25526[] = {{&S16605[44], 20}, {NULL}};
const Segment SGI25527[] = {{&S16605[30], 6}, {NULL}};
const Segment SGI25528[] = {{&S16605[112], 3}, {NULL}};
const Segment SGI25529[] = {{&S16605[78], 5}, {NULL}};
const Segment SGI25530[] = {{&S17254[44], 11}, {NULL}};
const Segment SGI25531[] = {{&S17254[44], 22}, {NULL}};
const Segment SGI25532[] = {{&S17254[30], 6}, {NULL}};
const Segment SGI25533[] = {{&S17254[114], 3}, {NULL}};
const Segment SGI25534[] = {{&S17254[80], 5}, {NULL}};
const Segment SGI25535[] = {{&S16612[44], 10}, {NULL}};
const Segment SGI25536[] = {{&S16612[44], 21}, {NULL}};
const Segment SGI25537[] = {{&S16612[30], 6}, {NULL}};
const Segment SGI25538[] = {{&S16612[113], 3}, {NULL}};
const Segment SGI25539[] = {{&S16612[79], 5}, {NULL}};
const Segment SGI25540[] = {{&S16614[44], 24}, {NULL}};
const Segment SGI25541[] = {{&S16614[30], 6}, {NULL}};
const Segment SGI25542[] = {{&S16614[134], 3}, {NULL}};
const Segment SGI25543[] = {{&S16614[100], 5}, {NULL}};
const Segment SGI25544[] = {{&S16616[44], 6}, {NULL}};
const Segment SGI25545[] = {{&S16616[44], 23}, {NULL}};
const Segment SGI25546[] = {{&S16616[30], 6}, {NULL}};
const Segment SGI25547[] = {{&S16616[115], 3}, {NULL}};
const Segment SGI25548[] = {{&S16616[81], 5}, {NULL}};
const Segment SGI25549[] = {{&S16620[44], 7}, {NULL}};
const Segment SGI25550[] = {{&S16620[44], 18}, {NULL}};
const Segment SGI25551[] = {{&S16620[30], 6}, {NULL}};
const Segment SGI25552[] = {{&S16620[110], 3}, {NULL}};
const Segment SGI25553[] = {{&S16620[76], 5}, {NULL}};
const Segment SGI25554[] = {{&S16626[44], 9}, {NULL}};
const Segment SGI25555[] = {{&S16626[44], 20}, {NULL}};
const Segment SGI25556[] = {{&S16626[30], 6}, {NULL}};
const Segment SGI25557[] = {{&S16626[112], 3}, {NULL}};
const Segment SGI25558[] = {{&S16626[78], 5}, {NULL}};
const Segment SGI25559[] = {{&S16629[44], 10}, {NULL}};
const Segment SGI25560[] = {{&S16629[44], 21}, {NULL}};
const Segment SGI25561[] = {{&S16629[30], 6}, {NULL}};
const Segment SGI25562[] = {{&S16629[113], 3}, {NULL}};
const Segment SGI25563[] = {{&S16629[79], 5}, {NULL}};
const Segment SGI25564[] = {{&S16637[44], 10}, {NULL}};
const Segment SGI25565[] = {{&S16637[44], 21}, {NULL}};
const Segment SGI25566[] = {{&S16637[30], 6}, {NULL}};
const Segment SGI25567[] = {{&S16637[113], 3}, {NULL}};
const Segment SGI25568[] = {{&S16637[79], 5}, {NULL}};
const Segment SGI25569[] = {{&S16641[44], 13}, {NULL}};
const Segment SGI25570[] = {{&S16641[44], 24}, {NULL}};
const Segment SGI25571[] = {{&S16641[30], 6}, {NULL}};
const Segment SGI25572[] = {{&S16641[116], 3}, {NULL}};
const Segment SGI25573[] = {{&S16641[82], 5}, {NULL}};
const Segment SGI25574[] = {{&S16644[44], 12}, {NULL}};
const Segment SGI25575[] = {{&S16644[44], 23}, {NULL}};
const Segment SGI25576[] = {{&S16644[30], 6}, {NULL}};
const Segment SGI25577[] = {{&S16644[115], 3}, {NULL}};
const Segment SGI25578[] = {{&S16644[81], 5}, {NULL}};
const Segment SGI25579[] = {{&S16645[44], 27}, {NULL}};
const Segment SGI25580[] = {{&S16645[30], 6}, {NULL}};
const Segment SGI25581[] = {{&S16645[138], 3}, {NULL}};
const Segment SGI25582[] = {{&S16645[104], 5}, {NULL}};
const Segment SGI25583[] = {{&S16650[44], 10}, {NULL}};
const Segment SGI25584[] = {{&S16650[44], 27}, {NULL}};
const Segment SGI25585[] = {{&S16650[30], 6}, {NULL}};
const Segment SGI25586[] = {{&S16650[119], 3}, {NULL}};
const Segment SGI25587[] = {{&S16650[85], 5}, {NULL}};
const Segment SGI25588[] = {{&S16652[44], 9}, {NULL}};
const Segment SGI25589[] = {{&S16652[44], 30}, {NULL}};
const Segment SGI25590[] = {{&S16652[30], 6}, {NULL}};
const Segment SGI25591[] = {{&S16652[122], 3}, {NULL}};
const Segment SGI25592[] = {{&S16652[88], 5}, {NULL}};
const Segment SGI25593[] = {{&S16657[44], 7}, {NULL}};
const Segment SGI25594[] = {{&S16657[44], 28}, {NULL}};
const Segment SGI25595[] = {{&S16657[30], 6}, {NULL}};
const Segment SGI25596[] = {{&S16657[120], 3}, {NULL}};
const Segment SGI25597[] = {{&S16657[86], 5}, {NULL}};
const Segment SGI25598[] = {{&S16666[44], 6}, {NULL}};
const Segment SGI25599[] = {{&S16666[44], 17}, {NULL}};
const Segment SGI25600[] = {{&S16666[30], 6}, {NULL}};
const Segment SGI25601[] = {{&S16666[109], 3}, {NULL}};
const Segment SGI25602[] = {{&S16666[75], 5}, {NULL}};
const Segment SGI25603[] = {{&S16667[44], 17}, {NULL}};
const Segment SGI25604[] = {{&S16667[44], 28}, {NULL}};
const Segment SGI25605[] = {{&S16667[30], 6}, {NULL}};
const Segment SGI25606[] = {{&S16667[120], 3}, {NULL}};
const Segment SGI25607[] = {{&S16667[86], 5}, {NULL}};
const Segment SGI25608[] = {{&S16671[44], 5}, {NULL}};
const Segment SGI25609[] = {{&S16671[44], 16}, {NULL}};
const Segment SGI25610[] = {{&S16671[30], 6}, {NULL}};
const Segment SGI25611[] = {{&S16671[108], 3}, {NULL}};
const Segment SGI25612[] = {{&S16671[74], 5}, {NULL}};
const Segment SGI25613[] = {{&S16673[44], 7}, {NULL}};
const Segment SGI25614[] = {{&S16673[44], 19}, {NULL}};
const Segment SGI25615[] = {{&S16673[30], 6}, {NULL}};
const Segment SGI25616[] = {{&S16673[111], 3}, {NULL}};
const Segment SGI25617[] = {{&S16673[77], 5}, {NULL}};
const Segment SGI25618[] = {{&S16685[44], 5}, {NULL}};
const Segment SGI25619[] = {{&S16685[44], 16}, {NULL}};
const Segment SGI25620[] = {{&S16685[30], 6}, {NULL}};
const Segment SGI25621[] = {{&S16685[108], 3}, {NULL}};
const Segment SGI25622[] = {{&S16685[74], 5}, {NULL}};
const Segment SGI25623[] = {{&S16694[44], 10}, {NULL}};
const Segment SGI25624[] = {{&S16694[44], 21}, {NULL}};
const Segment SGI25625[] = {{&S16694[30], 6}, {NULL}};
const Segment SGI25626[] = {{&S16694[113], 3}, {NULL}};
const Segment SGI25627[] = {{&S16694[79], 5}, {NULL}};
const Segment SGI25628[] = {{&S17261[44], 11}, {NULL}};
const Segment SGI25629[] = {{&S17261[44], 22}, {NULL}};
const Segment SGI25630[] = {{&S17261[30], 6}, {NULL}};
const Segment SGI25631[] = {{&S17261[114], 3}, {NULL}};
const Segment SGI25632[] = {{&S17261[80], 5}, {NULL}};
const Segment SGI25633[] = {{&S16702[44], 7}, {NULL}};
const Segment SGI25634[] = {{&S16702[44], 24}, {NULL}};
const Segment SGI25635[] = {{&S16702[30], 6}, {NULL}};
const Segment SGI25636[] = {{&S16702[116], 3}, {NULL}};
const Segment SGI25637[] = {{&S16702[82], 5}, {NULL}};
const Segment SGI25638[] = {{&S16706[41], 6}, {NULL}};
const Segment SGI25639[] = {{&S16706[41], 18}, {NULL}};
const Segment SGI25640[] = {{&S16706[107], 3}, {NULL}};
const Segment SGI25641[] = {{&S16706[73], 5}, {NULL}};
const Segment SGI25642[] = {{&S16707[44], 12}, {NULL}};
const Segment SGI25643[] = {{&S16707[44], 23}, {NULL}};
const Segment SGI25644[] = {{&S16707[30], 6}, {NULL}};
const Segment SGI25645[] = {{&S16707[115], 3}, {NULL}};
const Segment SGI25646[] = {{&S16707[81], 5}, {NULL}};
const Segment SGI25647[] = {{&S16709[44], 10}, {NULL}};
const Segment SGI25648[] = {{&S16709[44], 21}, {NULL}};
const Segment SGI25649[] = {{&S16709[30], 6}, {NULL}};
const Segment SGI25650[] = {{&S16709[82], 6}, {NULL}};
const Segment SGI25651[] = {{&S16710[44], 12}, {NULL}};
const Segment SGI25652[] = {{&S16710[44], 23}, {NULL}};
const Segment SGI25653[] = {{&S16710[30], 6}, {NULL}};
const Segment SGI25654[] = {{&S16710[84], 6}, {NULL}};
const Segment SGI25655[] = {{&S16715[44], 12}, {NULL}};
const Segment SGI25656[] = {{&S16715[44], 23}, {NULL}};
const Segment SGI25657[] = {{&S16715[30], 6}, {NULL}};
const Segment SGI25658[] = {{&S16715[115], 3}, {NULL}};
const Segment SGI25659[] = {{&S16715[81], 5}, {NULL}};
const Segment SGI25660[] = {{&S16719[44], 10}, {NULL}};
const Segment SGI25661[] = {{&S16719[44], 36}, {NULL}};
const Segment SGI25662[] = {{&S16719[30], 6}, {NULL}};
const Segment SGI25663[] = {{&S16719[128], 3}, {NULL}};
const Segment SGI25664[] = {{&S16719[94], 5}, {NULL}};
const Segment SGI25665[] = {{&S16721[44], 9}, {NULL}};
const Segment SGI25666[] = {{&S16721[44], 20}, {NULL}};
const Segment SGI25667[] = {{&S16721[30], 6}, {NULL}};
const Segment SGI25668[] = {{&S16721[112], 3}, {NULL}};
const Segment SGI25669[] = {{&S16721[78], 5}, {NULL}};
const Segment SGI25670[] = {{&S16723[44], 27}, {NULL}};
const Segment SGI25671[] = {{&S16723[30], 6}, {NULL}};
const Segment SGI25672[] = {{&S16723[138], 3}, {NULL}};
const Segment SGI25673[] = {{&S16723[104], 5}, {NULL}};
const Segment SGI25674[] = {{&S16725[44], 20}, {NULL}};
const Segment SGI25675[] = {{&S16725[44], 31}, {NULL}};
const Segment SGI25676[] = {{&S16725[30], 6}, {NULL}};
const Segment SGI25677[] = {{&S16725[123], 3}, {NULL}};
const Segment SGI25678[] = {{&S16725[89], 5}, {NULL}};
const Segment SGI25679[] = {{&S16729[44], 28}, {NULL}};
const Segment SGI25680[] = {{&S16729[30], 6}, {NULL}};
const Segment SGI25681[] = {{&S16729[139], 3}, {NULL}};
const Segment SGI25682[] = {{&S16729[105], 5}, {NULL}};
const Segment SGI25683[] = {{&S16733[44], 7}, {NULL}};
const Segment SGI25684[] = {{&S16733[44], 18}, {NULL}};
const Segment SGI25685[] = {{&S16733[30], 6}, {NULL}};
const Segment SGI25686[] = {{&S16733[110], 3}, {NULL}};
const Segment SGI25687[] = {{&S16733[76], 5}, {NULL}};
const Segment SGI25688[] = {{&S16734[44], 6}, {NULL}};
const Segment SGI25689[] = {{&S16734[44], 17}, {NULL}};
const Segment SGI25690[] = {{&S16734[30], 6}, {NULL}};
const Segment SGI25691[] = {{&S16734[109], 3}, {NULL}};
const Segment SGI25692[] = {{&S16734[75], 5}, {NULL}};
const Segment SGI25693[] = {{&S16740[44], 18}, {NULL}};
const Segment SGI25694[] = {{&S16740[44], 35}, {NULL}};
const Segment SGI25695[] = {{&S16740[30], 6}, {NULL}};
const Segment SGI25696[] = {{&S16740[127], 3}, {NULL}};
const Segment SGI25697[] = {{&S16740[93], 5}, {NULL}};
const Segment SGI25698[] = {{&S16742[98], 6}, {NULL}};
const Segment SGI25699[] = {{&S16742[152], 3}, {NULL}};
const Segment SGI25700[] = {{&S16742[118], 5}, {NULL}};
const Segment SGI25701[] = {{&S16746[44], 6}, {NULL}};
const Segment SGI25702[] = {{&S16746[44], 27}, {NULL}};
const Segment SGI25703[] = {{&S16746[30], 6}, {NULL}};
const Segment SGI25704[] = {{&S16746[119], 3}, {NULL}};
const Segment SGI25705[] = {{&S16746[85], 5}, {NULL}};
const Segment SGI25706[] = {{&S16748[44], 14}, {NULL}};
const Segment SGI25707[] = {{&S16748[44], 25}, {NULL}};
const Segment SGI25708[] = {{&S16748[30], 6}, {NULL}};
const Segment SGI25709[] = {{&S16748[117], 3}, {NULL}};
const Segment SGI25710[] = {{&S16748[83], 5}, {NULL}};
const Segment SGI25711[] = {{&S16749[44], 10}, {NULL}};
const Segment SGI25712[] = {{&S16749[44], 21}, {NULL}};
const Segment SGI25713[] = {{&S16749[30], 6}, {NULL}};
const Segment SGI25714[] = {{&S16749[113], 3}, {NULL}};
const Segment SGI25715[] = {{&S16749[79], 5}, {NULL}};
const Segment SGI25716[] = {{&S16751[44], 4}, {NULL}};
const Segment SGI25717[] = {{&S16751[44], 15}, {NULL}};
const Segment SGI25718[] = {{&S16751[30], 6}, {NULL}};
const Segment SGI25719[] = {{&S16751[107], 3}, {NULL}};
const Segment SGI25720[] = {{&S16751[73], 5}, {NULL}};
const Segment SGI25721[] = {{&S17299[44], 10}, {NULL}};
const Segment SGI25722[] = {{&S17299[44], 21}, {NULL}};
const Segment SGI25723[] = {{&S17299[30], 6}, {NULL}};
const Segment SGI25724[] = {{&S17299[113], 3}, {NULL}};
const Segment SGI25725[] = {{&S17299[79], 5}, {NULL}};
const Segment SGI25726[] = {{&S17300[44], 5}, {NULL}};
const Segment SGI25727[] = {{&S17300[44], 23}, {NULL}};
const Segment SGI25728[] = {{&S17300[30], 6}, {NULL}};
const Segment SGI25729[] = {{&S17300[55], 3}, {NULL}};
const Segment SGI25730[] = {{&S17300[81], 5}, {NULL}};
const Segment SGI25731[] = {{&S16754[44], 9}, {NULL}};
const Segment SGI25732[] = {{&S16754[44], 26}, {NULL}};
const Segment SGI25733[] = {{&S16754[30], 6}, {NULL}};
const Segment SGI25734[] = {{&S16754[118], 3}, {NULL}};
const Segment SGI25735[] = {{&S16754[84], 5}, {NULL}};
const Segment SGI25736[] = {{&S16755[44], 18}, {NULL}};
const Segment SGI25737[] = {{&S16755[30], 6}, {NULL}};
const Segment SGI25738[] = {{&S17306[44], 5}, {NULL}};
const Segment SGI25739[] = {{&S17306[44], 33}, {NULL}};
const Segment SGI25740[] = {{&S17306[30], 6}, {NULL}};
const Segment SGI25741[] = {{&S17306[125], 3}, {NULL}};
const Segment SGI25742[] = {{&S17306[91], 5}, {NULL}};
const Segment SGI25743[] = {{&S16763[44], 34}, {NULL}};
const Segment SGI25744[] = {{&S16763[30], 6}, {NULL}};
const Segment SGI25745[] = {{&S16763[145], 3}, {NULL}};
const Segment SGI25746[] = {{&S16763[111], 5}, {NULL}};
const Segment SGI25747[] = {{&S16764[44], 23}, {NULL}};
const Segment SGI25748[] = {{&S16764[44], 34}, {NULL}};
const Segment SGI25749[] = {{&S16764[30], 6}, {NULL}};
const Segment SGI25750[] = {{&S16764[126], 3}, {NULL}};
const Segment SGI25751[] = {{&S16764[92], 5}, {NULL}};
const Segment SGI25752[] = {{&S16770[44], 9}, {NULL}};
const Segment SGI25753[] = {{&S16770[44], 20}, {NULL}};
const Segment SGI25754[] = {{&S16770[30], 6}, {NULL}};
const Segment SGI25755[] = {{&S16770[112], 3}, {NULL}};
const Segment SGI25756[] = {{&S16770[78], 5}, {NULL}};
const Segment SGI25757[] = {{&S16773[116], 3}, {NULL}};
const Segment SGI25758[] = {{&S16773[83], 5}, {NULL}};
const Segment SGI25759[] = {{&S16774[44], 10}, {NULL}};
const Segment SGI25760[] = {{&S16774[44], 31}, {NULL}};
const Segment SGI25761[] = {{&S16774[30], 6}, {NULL}};
const Segment SGI25762[] = {{&S16774[123], 3}, {NULL}};
const Segment SGI25763[] = {{&S16774[89], 5}, {NULL}};
const Segment SGI25764[] = {{&S16776[44], 12}, {NULL}};
const Segment SGI25765[] = {{&S16776[44], 23}, {NULL}};
const Segment SGI25766[] = {{&S16776[30], 6}, {NULL}};
const Segment SGI25767[] = {{&S16776[115], 3}, {NULL}};
const Segment SGI25768[] = {{&S16776[81], 5}, {NULL}};
const Segment SGI25769[] = {{&S16777[44], 34}, {NULL}};
const Segment SGI25770[] = {{&S16777[30], 6}, {NULL}};
const Segment SGI25771[] = {{&S16777[145], 3}, {NULL}};
const Segment SGI25772[] = {{&S16777[111], 5}, {NULL}};
const Segment SGI25773[] = {{&S16782[44], 14}, {NULL}};
const Segment SGI25774[] = {{&S16782[44], 25}, {NULL}};
const Segment SGI25775[] = {{&S16782[30], 6}, {NULL}};
const Segment SGI25776[] = {{&S16782[117], 3}, {NULL}};
const Segment SGI25777[] = {{&S16782[83], 5}, {NULL}};
const Segment SGI25778[] = {{&S16793[44], 10}, {NULL}};
const Segment SGI25779[] = {{&S16793[44], 27}, {NULL}};
const Segment SGI25780[] = {{&S16793[30], 6}, {NULL}};
const Segment SGI25781[] = {{&S16793[119], 3}, {NULL}};
const Segment SGI25782[] = {{&S16793[85], 5}, {NULL}};
const Segment SGI25783[] = {{&S16795[44], 10}, {NULL}};
const Segment SGI25784[] = {{&S16795[44], 21}, {NULL}};
const Segment SGI25785[] = {{&S16795[30], 6}, {NULL}};
const Segment SGI25786[] = {{&S16795[113], 3}, {NULL}};
const Segment SGI25787[] = {{&S16795[79], 5}, {NULL}};
const Segment SGI25788[] = {{&S16797[44], 9}, {NULL}};
const Segment SGI25789[] = {{&S16797[44], 20}, {NULL}};
const Segment SGI25790[] = {{&S16797[30], 6}, {NULL}};
const Segment SGI25791[] = {{&S16797[112], 3}, {NULL}};
const Segment SGI25792[] = {{&S16797[78], 5}, {NULL}};
const Segment SGI25793[] = {{&S16798[44], 12}, {NULL}};
const Segment SGI25794[] = {{&S16798[44], 33}, {NULL}};
const Segment SGI25795[] = {{&S16798[30], 6}, {NULL}};
const Segment SGI25796[] = {{&S16798[125], 3}, {NULL}};
const Segment SGI25797[] = {{&S16798[91], 5}, {NULL}};
const Segment SGI25798[] = {{&S16799[44], 20}, {NULL}};
const Segment SGI25799[] = {{&S16799[44], 31}, {NULL}};
const Segment SGI25800[] = {{&S16799[30], 6}, {NULL}};
const Segment SGI25801[] = {{&S16799[123], 3}, {NULL}};
const Segment SGI25802[] = {{&S16799[89], 5}, {NULL}};
const Segment SGI25803[] = {{&S16803[44], 5}, {NULL}};
const Segment SGI25804[] = {{&S16803[44], 26}, {NULL}};
const Segment SGI25805[] = {{&S16803[30], 6}, {NULL}};
const Segment SGI25806[] = {{&S16803[118], 3}, {NULL}};
const Segment SGI25807[] = {{&S16803[84], 5}, {NULL}};
const Segment SGI25808[] = {{&S16811[44], 10}, {NULL}};
const Segment SGI25809[] = {{&S16811[44], 21}, {NULL}};
const Segment SGI25810[] = {{&S16811[30], 6}, {NULL}};
const Segment SGI25811[] = {{&S16811[113], 3}, {NULL}};
const Segment SGI25812[] = {{&S16811[79], 5}, {NULL}};
const Segment SGI25813[] = {{&S16812[44], 35}, {NULL}};
const Segment SGI25814[] = {{&S16812[30], 6}, {NULL}};
const Segment SGI25815[] = {{&S16812[146], 3}, {NULL}};
const Segment SGI25816[] = {{&S16812[112], 5}, {NULL}};
const Segment SGI25817[] = {{&S16815[44], 6}, {NULL}};
const Segment SGI25818[] = {{&S16815[44], 17}, {NULL}};
const Segment SGI25819[] = {{&S16815[30], 6}, {NULL}};
const Segment SGI25820[] = {{&S16815[109], 3}, {NULL}};
const Segment SGI25821[] = {{&S16815[75], 5}, {NULL}};
const Segment SGI25822[] = {{&S16819[44], 6}, {NULL}};
const Segment SGI25823[] = {{&S16819[44], 17}, {NULL}};
const Segment SGI25824[] = {{&S16819[30], 6}, {NULL}};
const Segment SGI25825[] = {{&S16819[109], 3}, {NULL}};
const Segment SGI25826[] = {{&S16819[75], 5}, {NULL}};
const Segment SGI25827[] = {{&S16820[44], 8}, {NULL}};
const Segment SGI25828[] = {{&S16820[44], 29}, {NULL}};
const Segment SGI25829[] = {{&S16820[30], 6}, {NULL}};
const Segment SGI25830[] = {{&S16820[121], 3}, {NULL}};
const Segment SGI25831[] = {{&S16820[87], 5}, {NULL}};
const Segment SGI25832[] = {{&S16822[44], 9}, {NULL}};
const Segment SGI25833[] = {{&S16822[44], 26}, {NULL}};
const Segment SGI25834[] = {{&S16822[30], 6}, {NULL}};
const Segment SGI25835[] = {{&S16822[118], 3}, {NULL}};
const Segment SGI25836[] = {{&S16822[84], 5}, {NULL}};
const Segment SGI25837[] = {{&S16824[44], 8}, {NULL}};
const Segment SGI25838[] = {{&S16824[44], 19}, {NULL}};
const Segment SGI25839[] = {{&S16824[30], 6}, {NULL}};
const Segment SGI25840[] = {{&S16824[111], 3}, {NULL}};
const Segment SGI25841[] = {{&S16824[77], 5}, {NULL}};
const Segment SGI25842[] = {{&S16829[44], 35}, {NULL}};
const Segment SGI25843[] = {{&S16829[30], 6}, {NULL}};
const Segment SGI25844[] = {{&S16829[146], 3}, {NULL}};
const Segment SGI25845[] = {{&S16829[112], 5}, {NULL}};
const Segment SGI25846[] = {{&S16833[44], 7}, {NULL}};
const Segment SGI25847[] = {{&S16833[44], 18}, {NULL}};
const Segment SGI25848[] = {{&S16833[30], 6}, {NULL}};
const Segment SGI25849[] = {{&S16833[110], 3}, {NULL}};
const Segment SGI25850[] = {{&S16833[76], 5}, {NULL}};
const Segment SGI25851[] = {{&S16834[44], 31}, {NULL}};
const Segment SGI25852[] = {{&S16834[30], 6}, {NULL}};
const Segment SGI25853[] = {{&S16836[44], 34}, {NULL}};
const Segment SGI25854[] = {{&S16836[30], 6}, {NULL}};
const Segment SGI25855[] = {{&S16836[145], 3}, {NULL}};
const Segment SGI25856[] = {{&S16836[111], 5}, {NULL}};
const Segment SGI25857[] = {{&S16839[44], 7}, {NULL}};
const Segment SGI25858[] = {{&S16839[44], 28}, {NULL}};
const Segment SGI25859[] = {{&S16839[30], 6}, {NULL}};
const Segment SGI25860[] = {{&S16839[120], 3}, {NULL}};
const Segment SGI25861[] = {{&S16839[86], 5}, {NULL}};
const Segment SGI25862[] = {{&S16840[44], 12}, {NULL}};
const Segment SGI25863[] = {{&S16840[44], 33}, {NULL}};
const Segment SGI25864[] = {{&S16840[30], 6}, {NULL}};
const Segment SGI25865[] = {{&S16840[125], 3}, {NULL}};
const Segment SGI25866[] = {{&S16840[91], 5}, {NULL}};
const Segment SGI25867[] = {{&S16843[44], 14}, {NULL}};
const Segment SGI25868[] = {{&S16843[44], 31}, {NULL}};
const Segment SGI25869[] = {{&S16843[30], 6}, {NULL}};
const Segment SGI25870[] = {{&S16843[123], 3}, {NULL}};
const Segment SGI25871[] = {{&S16843[89], 5}, {NULL}};
const Segment SGI25872[] = {{&S16844[44], 14}, {NULL}};
const Segment SGI25873[] = {{&S16844[44], 25}, {NULL}};
const Segment SGI25874[] = {{&S16844[30], 6}, {NULL}};
const Segment SGI25875[] = {{&S16844[117], 3}, {NULL}};
const Segment SGI25876[] = {{&S16844[83], 5}, {NULL}};
const Segment SGI25877[] = {{&S16849[44], 12}, {NULL}};
const Segment SGI25878[] = {{&S16849[44], 23}, {NULL}};
const Segment SGI25879[] = {{&S16849[30], 6}, {NULL}};
const Segment SGI25880[] = {{&S16849[84], 6}, {NULL}};
const Segment SGI25881[] = {{&S16855[44], 6}, {NULL}};
const Segment SGI25882[] = {{&S16855[44], 27}, {NULL}};
const Segment SGI25883[] = {{&S16855[30], 6}, {NULL}};
const Segment SGI25884[] = {{&S16855[119], 3}, {NULL}};
const Segment SGI25885[] = {{&S16855[85], 5}, {NULL}};
const Segment SGI25886[] = {{&S16856[44], 10}, {NULL}};
const Segment SGI25887[] = {{&S16856[44], 17}, {NULL}};
const Segment SGI25888[] = {{&S16856[30], 6}, {NULL}};
const Segment SGI25889[] = {{&S16856[109], 3}, {NULL}};
const Segment SGI25890[] = {{&S16856[75], 5}, {NULL}};
const Segment SGI25891[] = {{&S16857[44], 12}, {NULL}};
const Segment SGI25892[] = {{&S16857[44], 23}, {NULL}};
const Segment SGI25893[] = {{&S16857[30], 6}, {NULL}};
const Segment SGI25894[] = {{&S16857[115], 3}, {NULL}};
const Segment SGI25895[] = {{&S16857[81], 5}, {NULL}};
const Segment SGI25896[] = {{&S16859[44], 12}, {NULL}};
const Segment SGI25897[] = {{&S16859[44], 23}, {NULL}};
const Segment SGI25898[] = {{&S16859[30], 6}, {NULL}};
const Segment SGI25899[] = {{&S16859[115], 3}, {NULL}};
const Segment SGI25900[] = {{&S16859[81], 5}, {NULL}};
const Segment SGI25901[] = {{&S16861[44], 15}, {NULL}};
const Segment SGI25902[] = {{&S16861[44], 32}, {NULL}};
const Segment SGI25903[] = {{&S16861[30], 6}, {NULL}};
const Segment SGI25904[] = {{&S16861[124], 3}, {NULL}};
const Segment SGI25905[] = {{&S16861[90], 5}, {NULL}};
const Segment SGI25906[] = {{&S16864[44], 6}, {NULL}};
const Segment SGI25907[] = {{&S16864[44], 17}, {NULL}};
const Segment SGI25908[] = {{&S16864[30], 6}, {NULL}};
const Segment SGI25909[] = {{&S16864[109], 3}, {NULL}};
const Segment SGI25910[] = {{&S16864[75], 5}, {NULL}};
const Segment SGI25911[] = {{&S16866[44], 24}, {NULL}};
const Segment SGI25912[] = {{&S16866[44], 35}, {NULL}};
const Segment SGI25913[] = {{&S16866[30], 6}, {NULL}};
const Segment SGI25914[] = {{&S16866[127], 3}, {NULL}};
const Segment SGI25915[] = {{&S16866[93], 5}, {NULL}};
const Segment SGI25916[] = {{&S16867[42], 7}, {NULL}};
const Segment SGI25917[] = {{&S16867[30], 6}, {NULL}};
const Segment SGI25918[] = {{&S16867[108], 3}, {NULL}};
const Segment SGI25919[] = {{&S16867[74], 5}, {NULL}};
const Segment SGI25920[] = {{&S16875[44], 7}, {NULL}};
const Segment SGI25921[] = {{&S16875[44], 18}, {NULL}};
const Segment SGI25922[] = {{&S16875[30], 6}, {NULL}};
const Segment SGI25923[] = {{&S16875[110], 3}, {NULL}};
const Segment SGI25924[] = {{&S16875[76], 5}, {NULL}};
const Segment SGI25925[] = {{&S16888[44], 7}, {NULL}};
const Segment SGI25926[] = {{&S16888[44], 18}, {NULL}};
const Segment SGI25927[] = {{&S16888[30], 6}, {NULL}};
const Segment SGI25928[] = {{&S16888[110], 3}, {NULL}};
const Segment SGI25929[] = {{&S16888[76], 5}, {NULL}};
const Segment SGI25930[] = {{&S16892[44], 10}, {NULL}};
const Segment SGI25931[] = {{&S16892[44], 21}, {NULL}};
const Segment SGI25932[] = {{&S16892[30], 6}, {NULL}};
const Segment SGI25933[] = {{&S16892[82], 4}, {NULL}};
const Segment SGI25934[] = {{&S16892[82], 3}, {NULL}};
const Segment SGI25935[] = {{&S16894[44], 7}, {NULL}};
const Segment SGI25936[] = {{&S16894[44], 18}, {NULL}};
const Segment SGI25937[] = {{&S16894[30], 6}, {NULL}};
const Segment SGI25938[] = {{&S16894[79], 6}, {NULL}};
const Segment SGI25939[] = {{&S16895[44], 17}, {NULL}};
const Segment SGI25940[] = {{&S16895[44], 28}, {NULL}};
const Segment SGI25941[] = {{&S16895[30], 6}, {NULL}};
const Segment SGI25942[] = {{&S16895[89], 6}, {NULL}};
const Segment SGI25943[] = {{&S16896[44], 12}, {NULL}};
const Segment SGI25944[] = {{&S16896[44], 23}, {NULL}};
const Segment SGI25945[] = {{&S16896[30], 6}, {NULL}};
const Segment SGI25946[] = {{&S16896[115], 3}, {NULL}};
const Segment SGI25947[] = {{&S16896[81], 5}, {NULL}};
const Segment SGI25948[] = {{&S16903[44], 11}, {NULL}};
const Segment SGI25949[] = {{&S16903[44], 53}, {NULL}};
const Segment SGI25950[] = {{&S16903[30], 6}, {NULL}};
const Segment SGI25951[] = {{&S16903[145], 3}, {NULL}};
const Segment SGI25952[] = {{&S16903[111], 5}, {NULL}};
const Segment SGI25953[] = {{&S16904[44], 12}, {NULL}};
const Segment SGI25954[] = {{&S16904[44], 29}, {NULL}};
const Segment SGI25955[] = {{&S16904[30], 6}, {NULL}};
const Segment SGI25956[] = {{&S16904[121], 3}, {NULL}};
const Segment SGI25957[] = {{&S16904[87], 5}, {NULL}};
const Segment SGI25958[] = {{&S16911[44], 8}, {NULL}};
const Segment SGI25959[] = {{&S16911[44], 19}, {NULL}};
const Segment SGI25960[] = {{&S16911[30], 6}, {NULL}};
const Segment SGI25961[] = {{&S16911[80], 6}, {NULL}};
const Segment SGI25962[] = {{&S16918[44], 11}, {NULL}};
const Segment SGI25963[] = {{&S16918[44], 22}, {NULL}};
const Segment SGI25964[] = {{&S16918[30], 6}, {NULL}};
const Segment SGI25965[] = {{&S16918[114], 3}, {NULL}};
const Segment SGI25966[] = {{&S16918[80], 5}, {NULL}};
const Segment SGI25967[] = {{&S16919[44], 6}, {NULL}};
const Segment SGI25968[] = {{&S16919[44], 17}, {NULL}};
const Segment SGI25969[] = {{&S16919[30], 6}, {NULL}};
const Segment SGI25970[] = {{&S16919[109], 3}, {NULL}};
const Segment SGI25971[] = {{&S16919[75], 5}, {NULL}};
const Segment SGI25972[] = {{&S16925[44], 5}, {NULL}};
const Segment SGI25973[] = {{&S16925[44], 16}, {NULL}};
const Segment SGI25974[] = {{&S16925[30], 6}, {NULL}};
const Segment SGI25975[] = {{&S16925[108], 3}, {NULL}};
const Segment SGI25976[] = {{&S16925[74], 5}, {NULL}};
const Segment SGI25977[] = {{&S16929[44], 11}, {NULL}};
const Segment SGI25978[] = {{&S16929[44], 22}, {NULL}};
const Segment SGI25979[] = {{&S16929[30], 6}, {NULL}};
const Segment SGI25980[] = {{&S16929[83], 6}, {NULL}};
const Segment SGI25981[] = {{&S16931[44], 7}, {NULL}};
const Segment SGI25982[] = {{&S16931[44], 18}, {NULL}};
const Segment SGI25983[] = {{&S16931[30], 6}, {NULL}};
const Segment SGI25984[] = {{&S16931[110], 3}, {NULL}};
const Segment SGI25985[] = {{&S16931[76], 5}, {NULL}};
const Segment SGI25986[] = {{&S16935[44], 6}, {NULL}};
const Segment SGI25987[] = {{&S16935[44], 17}, {NULL}};
const Segment SGI25988[] = {{&S16935[30], 6}, {NULL}};
const Segment SGI25989[] = {{&S16935[109], 3}, {NULL}};
const Segment SGI25990[] = {{&S23157[44], 9}, {NULL}};
const Segment SGI25991[] = {{&S23157[44], 20}, {NULL}};
const Segment SGI25992[] = {{&S16935[75], 5}, {NULL}};
const Segment SGI25993[] = {{&S23157[30], 6}, {NULL}};
const Segment SGI25994[] = {{&S23157[112], 3}, {NULL}};
const Segment SGI25995[] = {{&S23157[78], 5}, {NULL}};
const Segment SGI25996[] = {{&S16938[44], 6}, {NULL}};
const Segment SGI25997[] = {{&S16938[44], 17}, {NULL}};
const Segment SGI25998[] = {{&S16938[30], 6}, {NULL}};
const Segment SGI25999[] = {{&S16938[109], 3}, {NULL}};
const Segment SGI26000[] = {{&S16938[75], 5}, {NULL}};
const Segment SGI26001[] = {{&S16941[44], 6}, {NULL}};
const Segment SGI26002[] = {{&S16941[44], 17}, {NULL}};
const Segment SGI26003[] = {{&S16941[30], 6}, {NULL}};
const Segment SGI26004[] = {{&S16941[78], 4}, {NULL}};
const Segment SGI26005[] = {{&S16941[78], 3}, {NULL}};
const Segment SGI26006[] = {{&S16943[44], 8}, {NULL}};
const Segment SGI26007[] = {{&S16943[44], 29}, {NULL}};
const Segment SGI26008[] = {{&S16943[30], 6}, {NULL}};
const Segment SGI26009[] = {{&S16943[90], 4}, {NULL}};
const Segment SGI26010[] = {{&S16943[90], 3}, {NULL}};
const Segment SGI26011[] = {{&S16948[44], 10}, {NULL}};
const Segment SGI26012[] = {{&S16948[44], 31}, {NULL}};
const Segment SGI26013[] = {{&S16948[30], 6}, {NULL}};
const Segment SGI26014[] = {{&S16948[123], 3}, {NULL}};
const Segment SGI26015[] = {{&S16948[89], 5}, {NULL}};
const Segment SGI26016[] = {{&S16952[44], 12}, {NULL}};
const Segment SGI26017[] = {{&S16952[44], 23}, {NULL}};
const Segment SGI26018[] = {{&S16952[30], 6}, {NULL}};
const Segment SGI26019[] = {{&S16952[115], 3}, {NULL}};
const Segment SGI26020[] = {{&S16952[81], 5}, {NULL}};
const Segment SGI26021[] = {{&S16965[44], 5}, {NULL}};
const Segment SGI26022[] = {{&S16965[44], 16}, {NULL}};
const Segment SGI26023[] = {{&S16965[30], 6}, {NULL}};
const Segment SGI26024[] = {{&S16965[108], 3}, {NULL}};
const Segment SGI26025[] = {{&S16965[74], 5}, {NULL}};
const Segment SGI26026[] = {{&S16966[44], 6}, {NULL}};
const Segment SGI26027[] = {{&S16966[44], 17}, {NULL}};
const Segment SGI26028[] = {{&S16966[30], 6}, {NULL}};
const Segment SGI26029[] = {{&S16966[109], 3}, {NULL}};
const Segment SGI26030[] = {{&S16966[75], 5}, {NULL}};
const Segment SGI26031[] = {{&S16976[44], 6}, {NULL}};
const Segment SGI26032[] = {{&S16976[44], 17}, {NULL}};
const Segment SGI26033[] = {{&S16976[30], 6}, {NULL}};
const Segment SGI26034[] = {{&S16976[109], 3}, {NULL}};
const Segment SGI26035[] = {{&S16976[75], 5}, {NULL}};
const Segment SGI26036[] = {{&S16978[42], 9}, {NULL}};
const Segment SGI26037[] = {{&S16978[30], 6}, {NULL}};
const Segment SGI26038[] = {{&S16978[110], 3}, {NULL}};
const Segment SGI26039[] = {{&S16978[76], 5}, {NULL}};
const Segment SGI26040[] = {{&S16979[44], 5}, {NULL}};
const Segment SGI26041[] = {{&S16979[44], 26}, {NULL}};
const Segment SGI26042[] = {{&S16979[30], 6}, {NULL}};
const Segment SGI26043[] = {{&S16979[87], 4}, {NULL}};
const Segment SGI26044[] = {{&S16979[87], 3}, {NULL}};
const Segment SGI26045[] = {{&S16982[44], 9}, {NULL}};
const Segment SGI26046[] = {{&S16982[44], 30}, {NULL}};
const Segment SGI26047[] = {{&S16982[30], 6}, {NULL}};
const Segment SGI26048[] = {{&S16982[122], 3}, {NULL}};
const Segment SGI26049[] = {{&S16982[88], 5}, {NULL}};
const Segment SGI26050[] = {{&S16985[55], 6}, {NULL}};
const Segment SGI26051[] = {{&S16986[44], 12}, {NULL}};
const Segment SGI26052[] = {{&S16986[44], 33}, {NULL}};
const Segment SGI26053[] = {{&S16986[30], 6}, {NULL}};
const Segment SGI26054[] = {{&S16986[125], 3}, {NULL}};
const Segment SGI26055[] = {{&S16986[91], 5}, {NULL}};
const Segment SGI26056[] = {{&S16987[44], 12}, {NULL}};
const Segment SGI26057[] = {{&S16987[44], 33}, {NULL}};
const Segment SGI26058[] = {{&S16987[30], 6}, {NULL}};
const Segment SGI26059[] = {{&S16987[125], 3}, {NULL}};
const Segment SGI26060[] = {{&S16987[91], 5}, {NULL}};
const Segment SGI26061[] = {{&S16989[44], 10}, {NULL}};
const Segment SGI26062[] = {{&S16989[44], 21}, {NULL}};
const Segment SGI26063[] = {{&S16989[30], 6}, {NULL}};
const Segment SGI26064[] = {{&S16989[113], 3}, {NULL}};
const Segment SGI26065[] = {{&S16989[79], 5}, {NULL}};
const Segment SGI26066[] = {{&S16990[61], 14}, {NULL}};
const Segment SGI26067[] = {{&S16990[61], 35}, {NULL}};
const Segment SGI26068[] = {{&S16990[47], 6}, {NULL}};
const Segment SGI26069[] = {{&S16990[144], 3}, {NULL}};
const Segment SGI26070[] = {{&S16990[110], 5}, {NULL}};
const Segment SGI26071[] = {{&S16991[44], 10}, {NULL}};
const Segment SGI26072[] = {{&S16991[44], 21}, {NULL}};
const Segment SGI26073[] = {{&S16991[30], 6}, {NULL}};
const Segment SGI26074[] = {{&S16991[113], 3}, {NULL}};
const Segment SGI26075[] = {{&S16991[79], 5}, {NULL}};
const Segment SGI26076[] = {{&S16995[44], 9}, {NULL}};
const Segment SGI26077[] = {{&S16995[44], 20}, {NULL}};
const Segment SGI26078[] = {{&S16995[30], 6}, {NULL}};
const Segment SGI26079[] = {{&S16995[112], 3}, {NULL}};
const Segment SGI26080[] = {{&S16995[78], 5}, {NULL}};
const Segment SGI26081[] = {{&S16996[44], 17}, {NULL}};
const Segment SGI26082[] = {{&S16996[30], 6}, {NULL}};
const Segment SGI26083[] = {{&S16996[109], 3}, {NULL}};
const Segment SGI26084[] = {{&S16996[75], 5}, {NULL}};
const Segment SGI26085[] = {{&S16997[44], 9}, {NULL}};
const Segment SGI26086[] = {{&S16997[44], 20}, {NULL}};
const Segment SGI26087[] = {{&S16997[30], 6}, {NULL}};
const Segment SGI26088[] = {{&S16997[112], 3}, {NULL}};
const Segment SGI26089[] = {{&S16997[78], 5}, {NULL}};
const Segment SGI26090[] = {{&S16999[44], 11}, {NULL}};
const Segment SGI26091[] = {{&S16999[44], 32}, {NULL}};
const Segment SGI26092[] = {{&S16999[30], 6}, {NULL}};
const Segment SGI26093[] = {{&S16999[124], 3}, {NULL}};
const Segment SGI26094[] = {{&S16999[90], 5}, {NULL}};
const Segment SGI26095[] = {{&S17000[44], 10}, {NULL}};
const Segment SGI26096[] = {{&S17000[44], 31}, {NULL}};
const Segment SGI26097[] = {{&S17000[30], 6}, {NULL}};
const Segment SGI26098[] = {{&S17000[92], 6}, {NULL}};
const Segment SGI26099[] = {{&S17002[44], 7}, {NULL}};
const Segment SGI26100[] = {{&S17002[44], 28}, {NULL}};
const Segment SGI26101[] = {{&S17002[30], 6}, {NULL}};
const Segment SGI26102[] = {{&S17002[120], 3}, {NULL}};
const Segment SGI26103[] = {{&S17002[86], 5}, {NULL}};
const Segment SGI26104[] = {{&S17003[44], 12}, {NULL}};
const Segment SGI26105[] = {{&S17003[44], 29}, {NULL}};
const Segment SGI26106[] = {{&S17003[30], 6}, {NULL}};
const Segment SGI26107[] = {{&S17003[121], 3}, {NULL}};
const Segment SGI26108[] = {{&S17003[87], 5}, {NULL}};
const Segment SGI26109[] = {{&S17004[44], 10}, {NULL}};
const Segment SGI26110[] = {{&S17004[44], 21}, {NULL}};
const Segment SGI26111[] = {{&S17004[30], 6}, {NULL}};
const Segment SGI26112[] = {{&S17004[113], 3}, {NULL}};
const Segment SGI26113[] = {{&S17004[79], 5}, {NULL}};
const Segment SGI26114[] = {{&S17006[44], 10}, {NULL}};
const Segment SGI26115[] = {{&S17006[44], 21}, {NULL}};
const Segment SGI26116[] = {{&S17006[30], 6}, {NULL}};
const Segment SGI26117[] = {{&S17006[113], 3}, {NULL}};
const Segment SGI26118[] = {{&S17006[79], 5}, {NULL}};
const Segment SGI26119[] = {{&S17007[44], 27}, {NULL}};
const Segment SGI26120[] = {{&S17007[30], 6}, {NULL}};
const Segment SGI26121[] = {{&S17007[138], 3}, {NULL}};
const Segment SGI26122[] = {{&S17007[104], 5}, {NULL}};
const Segment SGI26123[] = {{&S17010[44], 11}, {NULL}};
const Segment SGI26124[] = {{&S17010[44], 22}, {NULL}};
const Segment SGI26125[] = {{&S17010[30], 6}, {NULL}};
const Segment SGI26126[] = {{&S17010[114], 3}, {NULL}};
const Segment SGI26127[] = {{&S17010[80], 5}, {NULL}};
const Segment SGI26128[] = {{&S17011[44], 6}, {NULL}};
const Segment SGI26129[] = {{&S17011[44], 17}, {NULL}};
const Segment SGI26130[] = {{&S17011[30], 6}, {NULL}};
const Segment SGI26131[] = {{&S17011[109], 3}, {NULL}};
const Segment SGI26132[] = {{&S17011[75], 5}, {NULL}};
const Segment SGI26133[] = {{&S17012[44], 7}, {NULL}};
const Segment SGI26134[] = {{&S17012[44], 28}, {NULL}};
const Segment SGI26135[] = {{&S17012[30], 6}, {NULL}};
const Segment SGI26136[] = {{&S17012[120], 3}, {NULL}};
const Segment SGI26137[] = {{&S17012[86], 5}, {NULL}};
const Segment SGI26138[] = {{&S17019[44], 7}, {NULL}};
const Segment SGI26139[] = {{&S17019[44], 28}, {NULL}};
const Segment SGI26140[] = {{&S17019[30], 6}, {NULL}};
const Segment SGI26141[] = {{&S17019[120], 3}, {NULL}};
const Segment SGI26142[] = {{&S17019[86], 5}, {NULL}};
const Segment SGI26143[] = {{&S17023[44], 12}, {NULL}};
const Segment SGI26144[] = {{&S17023[44], 23}, {NULL}};
const Segment SGI26145[] = {{&S17023[30], 6}, {NULL}};
const Segment SGI26146[] = {{&S17023[115], 3}, {NULL}};
const Segment SGI26147[] = {{&S17023[81], 5}, {NULL}};
const Segment SGI26148[] = {{&S17033[44], 27}, {NULL}};
const Segment SGI26149[] = {{&S17033[30], 6}, {NULL}};
const Segment SGI26150[] = {{&S17033[138], 3}, {NULL}};
const Segment SGI26151[] = {{&S17033[104], 5}, {NULL}};
const Segment SGI26152[] = {{&S17036[44], 14}, {NULL}};
const Segment SGI26153[] = {{&S17036[44], 31}, {NULL}};
const Segment SGI26154[] = {{&S17036[30], 6}, {NULL}};
const Segment SGI26155[] = {{&S17036[123], 3}, {NULL}};
const Segment SGI26156[] = {{&S17036[89], 5}, {NULL}};
const Segment SGI26157[] = {{&S17038[44], 7}, {NULL}};
const Segment SGI26158[] = {{&S17038[44], 18}, {NULL}};
const Segment SGI26159[] = {{&S17038[30], 6}, {NULL}};
const Segment SGI26160[] = {{&S17038[110], 3}, {NULL}};
const Segment SGI26161[] = {{&S17038[76], 5}, {NULL}};
const Segment SGI26162[] = {{&S17040[44], 11}, {NULL}};
const Segment SGI26163[] = {{&S17040[44], 22}, {NULL}};
const Segment SGI26164[] = {{&S17040[30], 6}, {NULL}};
const Segment SGI26165[] = {{&S17040[114], 3}, {NULL}};
const Segment SGI26166[] = {{&S17040[80], 5}, {NULL}};
const Segment SGI26167[] = {{&S17042[44], 12}, {NULL}};
const Segment SGI26168[] = {{&S17042[44], 33}, {NULL}};
const Segment SGI26169[] = {{&S17042[30], 6}, {NULL}};
const Segment SGI26170[] = {{&S17042[125], 3}, {NULL}};
const Segment SGI26171[] = {{&S17042[91], 5}, {NULL}};
const Segment SGI26172[] = {{&S17044[44], 9}, {NULL}};
const Segment SGI26173[] = {{&S17044[44], 30}, {NULL}};
const Segment SGI26174[] = {{&S17044[30], 6}, {NULL}};
const Segment SGI26175[] = {{&S17044[122], 3}, {NULL}};
const Segment SGI26176[] = {{&S17044[88], 5}, {NULL}};
const Segment SGI26177[] = {{&S17048[44], 5}, {NULL}};
const Segment SGI26178[] = {{&S17048[44], 26}, {NULL}};
const Segment SGI26179[] = {{&S17048[30], 6}, {NULL}};
const Segment SGI26180[] = {{&S17048[118], 3}, {NULL}};
const Segment SGI26181[] = {{&S17048[84], 5}, {NULL}};
const Segment SGI26182[] = {{&S17053[61], 7}, {NULL}};
const Segment SGI26183[] = {{&S17053[61], 28}, {NULL}};
const Segment SGI26184[] = {{&S17053[47], 6}, {NULL}};
const Segment SGI26185[] = {{&S17053[137], 3}, {NULL}};
const Segment SGI26186[] = {{&S17053[103], 5}, {NULL}};
const Segment SGI26187[] = {{&S17066[44], 10}, {NULL}};
const Segment SGI26188[] = {{&S17066[44], 21}, {NULL}};
const Segment SGI26189[] = {{&S17066[30], 6}, {NULL}};
const Segment SGI26190[] = {{&S17066[113], 3}, {NULL}};
const Segment SGI26191[] = {{&S17066[79], 5}, {NULL}};
const Segment SGI26192[] = {{&S17067[44], 5}, {NULL}};
const Segment SGI26193[] = {{&S17067[44], 26}, {NULL}};
const Segment SGI26194[] = {{&S17067[30], 6}, {NULL}};
const Segment SGI26195[] = {{&S17067[118], 3}, {NULL}};
const Segment SGI26196[] = {{&S17067[84], 5}, {NULL}};
const Segment SGI26197[] = {{&S17070[44], 8}, {NULL}};
const Segment SGI26198[] = {{&S17070[44], 29}, {NULL}};
const Segment SGI26199[] = {{&S17070[30], 6}, {NULL}};
const Segment SGI26200[] = {{&S17070[121], 3}, {NULL}};
const Segment SGI26201[] = {{&S17070[87], 5}, {NULL}};
const Segment SGI26202[] = {{&S17078[44], 7}, {NULL}};
const Segment SGI26203[] = {{&S17078[44], 28}, {NULL}};
const Segment SGI26204[] = {{&S17078[30], 6}, {NULL}};
const Segment SGI26205[] = {{&S17078[120], 3}, {NULL}};
const Segment SGI26206[] = {{&S17078[86], 5}, {NULL}};
const Segment SGI26207[] = {{&S17080[44], 10}, {NULL}};
const Segment SGI26208[] = {{&S17080[44], 31}, {NULL}};
const Segment SGI26209[] = {{&S17080[30], 6}, {NULL}};
const Segment SGI26210[] = {{&S17080[92], 4}, {NULL}};
const Segment SGI26211[] = {{&S17080[92], 3}, {NULL}};
const Segment SGI26212[] = {{&S17085[44], 7}, {NULL}};
const Segment SGI26213[] = {{&S17085[44], 18}, {NULL}};
const Segment SGI26214[] = {{&S17085[30], 6}, {NULL}};
const Segment SGI26215[] = {{&S17085[110], 3}, {NULL}};
const Segment SGI26216[] = {{&S17085[76], 5}, {NULL}};
const Segment SGI26217[] = {{&S17088[44], 8}, {NULL}};
const Segment SGI26218[] = {{&S17088[44], 19}, {NULL}};
const Segment SGI26219[] = {{&S17088[30], 6}, {NULL}};
const Segment SGI26220[] = {{&S17088[80], 4}, {NULL}};
const Segment SGI26221[] = {{&S17088[80], 3}, {NULL}};
const Segment SGI26222[] = {{&S17092[44], 10}, {NULL}};
const Segment SGI26223[] = {{&S17092[44], 21}, {NULL}};
const Segment SGI26224[] = {{&S17092[30], 6}, {NULL}};
const Segment SGI26225[] = {{&S17092[113], 3}, {NULL}};
const Segment SGI26226[] = {{&S17092[79], 5}, {NULL}};
const Segment SGI26227[] = {{&S17095[44], 21}, {NULL}};
const Segment SGI26228[] = {{&S17095[44], 32}, {NULL}};
const Segment SGI26229[] = {{&S17095[30], 6}, {NULL}};
const Segment SGI26230[] = {{&S17095[124], 3}, {NULL}};
const Segment SGI26231[] = {{&S17095[90], 5}, {NULL}};
const Segment SGI26232[] = {{&S17101[44], 7}, {NULL}};
const Segment SGI26233[] = {{&S17101[44], 18}, {NULL}};
const Segment SGI26234[] = {{&S17101[30], 6}, {NULL}};
const Segment SGI26235[] = {{&S17101[110], 3}, {NULL}};
const Segment SGI26236[] = {{&S17101[76], 5}, {NULL}};
const Segment SGI26237[] = {{&S17102[44], 34}, {NULL}};
const Segment SGI26238[] = {{&S17102[30], 6}, {NULL}};
const Segment SGI26239[] = {{&S17102[145], 3}, {NULL}};
const Segment SGI26240[] = {{&S17102[111], 5}, {NULL}};
const Segment SGI26241[] = {{&S17104[44], 8}, {NULL}};
const Segment SGI26242[] = {{&S17104[44], 19}, {NULL}};
const Segment SGI26243[] = {{&S17104[30], 6}, {NULL}};
const Segment SGI26244[] = {{&S17104[111], 3}, {NULL}};
const Segment SGI26245[] = {{&S17104[77], 5}, {NULL}};
const Segment SGI26246[] = {{&S17105[44], 33}, {NULL}};
const Segment SGI26247[] = {{&S17105[30], 6}, {NULL}};
const Segment SGI26248[] = {{&S17105[144], 3}, {NULL}};
const Segment SGI26249[] = {{&S17105[110], 5}, {NULL}};
const Segment SGI26250[] = {{&S17108[50], 6}, {&S17108[50], 6}, {NULL}};
const Segment SGI26251[] = {{&S17108[159], 3}, {NULL}};
const Segment SGI26252[] = {{&S17108[125], 5}, {NULL}};
const Segment SGI26253[] = {{&S17110[61], 8}, {NULL}};
const Segment SGI26254[] = {{&S17110[61], 29}, {NULL}};
const Segment SGI26255[] = {{&S17110[47], 6}, {NULL}};
const Segment SGI26256[] = {{&S17110[138], 3}, {NULL}};
const Segment SGI26257[] = {{&S17110[104], 5}, {NULL}};
const Segment SGI26258[] = {{&S17115[44], 9}, {NULL}};
const Segment SGI26259[] = {{&S17115[44], 20}, {NULL}};
const Segment SGI26260[] = {{&S23169[42], 18}, {NULL}};
const Segment SGI26261[] = {{&S17115[30], 6}, {NULL}};
const Segment SGI26262[] = {{&S23169[42], 29}, {NULL}};
const Segment SGI26263[] = {{&S23169[30], 4}, {NULL}};
const Segment SGI26264[] = {{&S17115[112], 3}, {NULL}};
const Segment SGI26265[] = {{&S23169[119], 3}, {NULL}};
const Segment SGI26266[] = {{&S17115[78], 5}, {NULL}};
const Segment SGI26267[] = {{&S23169[85], 5}, {NULL}};
const Segment SGI26268[] = {{&S17116[44], 13}, {NULL}};
const Segment SGI26269[] = {{&S17116[44], 25}, {NULL}};
const Segment SGI26270[] = {{&S17116[30], 6}, {NULL}};
const Segment SGI26271[] = {{&S17116[117], 3}, {NULL}};
const Segment SGI26272[] = {{&S17116[83], 5}, {NULL}};
const Segment SGI26273[] = {{&S17118[44], 6}, {NULL}};
const Segment SGI26274[] = {{&S17118[44], 27}, {NULL}};
const Segment SGI26275[] = {{&S17118[30], 6}, {NULL}};
const Segment SGI26276[] = {{&S17118[119], 3}, {NULL}};
const Segment SGI26277[] = {{&S17118[85], 5}, {NULL}};
const Segment SGI26278[] = {{&S17119[44], 10}, {NULL}};
const Segment SGI26279[] = {{&S17119[44], 31}, {NULL}};
const Segment SGI26280[] = {{&S17119[30], 6}, {NULL}};
const Segment SGI26281[] = {{&S17119[123], 3}, {NULL}};
const Segment SGI26282[] = {{&S17119[89], 5}, {NULL}};
const Segment SGI26283[] = {{&S17120[44], 77}, {NULL}};
const Segment SGI26284[] = {{&S17120[44], 98}, {NULL}};
const Segment SGI26285[] = {{&S17120[30], 6}, {NULL}};
const Segment SGI26286[] = {{&S23178[42], 9}, {NULL}};
const Segment SGI26287[] = {{&S23178[42], 20}, {NULL}};
const Segment SGI26288[] = {{&S23178[30], 4}, {NULL}};
const Segment SGI26289[] = {{&S17120[189], 3}, {NULL}};
const Segment SGI26290[] = {{&S23178[110], 3}, {NULL}};
const Segment SGI26291[] = {{&S17120[115], 5}, {NULL}};
const Segment SGI26292[] = {{&S23178[76], 5}, {NULL}};
const Segment SGI26293[] = {{&S17125[44], 15}, {NULL}};
const Segment SGI26294[] = {{&S17125[44], 26}, {NULL}};
const Segment SGI26295[] = {{&S17125[30], 6}, {NULL}};
const Segment SGI26296[] = {{&S17125[118], 3}, {NULL}};
const Segment SGI26297[] = {{&S17125[84], 5}, {NULL}};
const Segment SGI26298[] = {{&S17128[44], 7}, {NULL}};
const Segment SGI26299[] = {{&S17128[44], 28}, {NULL}};
const Segment SGI26300[] = {{&S17128[30], 6}, {NULL}};
const Segment SGI26301[] = {{&S17128[120], 3}, {NULL}};
const Segment SGI26302[] = {{&S17128[86], 5}, {NULL}};
const Segment SGI26303[] = {{&S17130[44], 80}, {NULL}};
const Segment SGI26304[] = {{&S17130[44], 101}, {NULL}};
const Segment SGI26305[] = {{&S17130[30], 6}, {NULL}};
const Segment SGI26306[] = {{&S17130[192], 3}, {NULL}};
const Segment SGI26307[] = {{&S17130[118], 5}, {NULL}};
const Segment SGI26308[] = {{&S17131[44], 80}, {NULL}};
const Segment SGI26309[] = {{&S17131[44], 101}, {NULL}};
const Segment SGI26310[] = {{&S17131[30], 6}, {NULL}};
const Segment SGI26311[] = {{&S17131[192], 3}, {NULL}};
const Segment SGI26312[] = {{&S17131[118], 5}, {NULL}};
const Segment SGI26313[] = {{&S17132[44], 15}, {NULL}};
const Segment SGI26314[] = {{&S17132[44], 26}, {NULL}};
const Segment SGI26315[] = {{&S17132[30], 6}, {NULL}};
const Segment SGI26316[] = {{&S17132[118], 3}, {NULL}};
const Segment SGI26317[] = {{&S17132[84], 5}, {NULL}};
const Segment SGI26318[] = {{&S17136[44], 7}, {NULL}};
const Segment SGI26319[] = {{&S17136[44], 28}, {NULL}};
const Segment SGI26320[] = {{&S17136[30], 6}, {NULL}};
const Segment SGI26321[] = {{&S17136[120], 3}, {NULL}};
const Segment SGI26322[] = {{&S17136[86], 5}, {NULL}};
const Segment SGI26323[] = {{&S17141[44], 11}, {NULL}};
const Segment SGI26324[] = {{&S17141[44], 32}, {NULL}};
const Segment SGI26325[] = {{&S17141[30], 6}, {NULL}};
const Segment SGI26326[] = {{&S17141[124], 3}, {NULL}};
const Segment SGI26327[] = {{&S17141[90], 5}, {NULL}};
const Segment SGI26328[] = {{&S17146[44], 10}, {NULL}};
const Segment SGI26329[] = {{&S17146[44], 31}, {NULL}};
const Segment SGI26330[] = {{&S17146[30], 6}, {NULL}};
const Segment SGI26331[] = {{&S17146[123], 3}, {NULL}};
const Segment SGI26332[] = {{&S17146[89], 5}, {NULL}};
const Segment SGI26333[] = {{&S17147[44], 7}, {NULL}};
const Segment SGI26334[] = {{&S17147[44], 18}, {NULL}};
const Segment SGI26335[] = {{&S17147[30], 6}, {NULL}};
const Segment SGI26336[] = {{&S17147[110], 3}, {NULL}};
const Segment SGI26337[] = {{&S17147[76], 5}, {NULL}};
const Segment SGI26338[] = {{&S17148[44], 12}, {NULL}};
const Segment SGI26339[] = {{&S17148[44], 33}, {NULL}};
const Segment SGI26340[] = {{&S17148[30], 6}, {NULL}};
const Segment SGI26341[] = {{&S17148[125], 3}, {NULL}};
const Segment SGI26342[] = {{&S17148[91], 5}, {NULL}};
const Segment SGI26343[] = {{&S17151[44], 13}, {NULL}};
const Segment SGI26344[] = {{&S17151[44], 24}, {NULL}};
const Segment SGI26345[] = {{&S17151[30], 6}, {NULL}};
const Segment SGI26346[] = {{&S17151[116], 3}, {NULL}};
const Segment SGI26347[] = {{&S17151[82], 5}, {NULL}};
const Segment SGI26348[] = {{&S17159[44], 9}, {NULL}};
const Segment SGI26349[] = {{&S17159[44], 16}, {NULL}};
const Segment SGI26350[] = {{&S17159[30], 6}, {NULL}};
const Segment SGI26351[] = {{&S17159[108], 3}, {NULL}};
const Segment SGI26352[] = {{&S17159[74], 5}, {NULL}};
const Segment SGI26353[] = {{&S17161[44], 17}, {NULL}};
const Segment SGI26354[] = {{&S17161[44], 32}, {NULL}};
const Segment SGI26355[] = {{&S17161[30], 6}, {NULL}};
const Segment SGI26356[] = {{&S17161[124], 3}, {NULL}};
const Segment SGI26357[] = {{&S17161[90], 5}, {NULL}};
const Segment SGI26358[] = {{&S17164[44], 10}, {NULL}};
const Segment SGI26359[] = {{&S17164[44], 36}, {NULL}};
const Segment SGI26360[] = {{&S17164[30], 6}, {NULL}};
const Segment SGI26361[] = {{&S17164[128], 3}, {NULL}};
const Segment SGI26362[] = {{&S17164[94], 5}, {NULL}};
const Segment SGI26363[] = {{&S17165[44], 10}, {NULL}};
const Segment SGI26364[] = {{&S17165[44], 21}, {NULL}};
const Segment SGI26365[] = {{&S17165[30], 6}, {NULL}};
const Segment SGI26366[] = {{&S17165[113], 3}, {NULL}};
const Segment SGI26367[] = {{&S17165[79], 5}, {NULL}};
const Segment SGI26368[] = {{&S17170[44], 7}, {NULL}};
const Segment SGI26369[] = {{&S17170[44], 28}, {NULL}};
const Segment SGI26370[] = {{&S17170[30], 6}, {NULL}};
const Segment SGI26371[] = {{&S17170[89], 4}, {NULL}};
const Segment SGI26372[] = {{&S17170[89], 3}, {NULL}};
const Segment SGI26373[] = {{&S17175[44], 10}, {NULL}};
const Segment SGI26374[] = {{&S17175[44], 21}, {NULL}};
const Segment SGI26375[] = {{&S17175[30], 6}, {NULL}};
const Segment SGI26376[] = {{&S17175[113], 3}, {NULL}};
const Segment SGI26377[] = {{&S17175[79], 5}, {NULL}};
const Segment SGI26378[] = {{&S17176[44], 13}, {NULL}};
const Segment SGI26379[] = {{&S17176[44], 34}, {NULL}};
const Segment SGI26380[] = {{&S17176[30], 6}, {NULL}};
const Segment SGI26381[] = {{&S17176[95], 6}, {NULL}};
const Segment SGI26382[] = {{&S17177[44], 7}, {NULL}};
const Segment SGI26383[] = {{&S17177[44], 18}, {NULL}};
const Segment SGI26384[] = {{&S17177[30], 6}, {NULL}};
const Segment SGI26385[] = {{&S17177[110], 3}, {NULL}};
const Segment SGI26386[] = {{&S17177[76], 5}, {NULL}};
const Segment SGI26387[] = {{&S17178[44], 34}, {NULL}};
const Segment SGI26388[] = {{&S17178[30], 6}, {NULL}};
const Segment SGI26389[] = {{&S17178[145], 3}, {NULL}};
const Segment SGI26390[] = {{&S17178[111], 5}, {NULL}};
const Segment SGI26391[] = {{&S17182[44], 29}, {NULL}};
const Segment SGI26392[] = {{&S17182[30], 6}, {NULL}};
const Segment SGI26393[] = {{&S17182[140], 3}, {NULL}};
const Segment SGI26394[] = {{&S17182[106], 5}, {NULL}};
const Segment SGI26395[] = {{&S17188[44], 9}, {NULL}};
const Segment SGI26396[] = {{&S17188[44], 20}, {NULL}};
const Segment SGI26397[] = {{&S17188[30], 6}, {NULL}};
const Segment SGI26398[] = {{&S17188[112], 3}, {NULL}};
const Segment SGI26399[] = {{&S17188[78], 5}, {NULL}};
const Segment SGI26400[] = {{&S17189[44], 83}, {NULL}};
const Segment SGI26401[] = {{&S17189[44], 104}, {NULL}};
const Segment SGI26402[] = {{&S17189[30], 6}, {NULL}};
const Segment SGI26403[] = {{&S17189[195], 3}, {NULL}};
const Segment SGI26404[] = {{&S17189[121], 5}, {NULL}};
const Segment SGI26405[] = {{&S17190[44], 12}, {NULL}};
const Segment SGI26406[] = {{&S17190[44], 33}, {NULL}};
const Segment SGI26407[] = {{&S17190[30], 6}, {NULL}};
const Segment SGI26408[] = {{&S17190[125], 3}, {NULL}};
const Segment SGI26409[] = {{&S17190[91], 5}, {NULL}};
const Segment SGI26410[] = {{&S17191[44], 7}, {NULL}};
const Segment SGI26411[] = {{&S17191[44], 28}, {NULL}};
const Segment SGI26412[] = {{&S17191[30], 6}, {NULL}};
const Segment SGI26413[] = {{&S17191[120], 3}, {NULL}};
const Segment SGI26414[] = {{&S17191[86], 5}, {NULL}};
const Segment SGI26415[] = {{&S17192[44], 31}, {NULL}};
const Segment SGI26416[] = {{&S17192[30], 6}, {NULL}};
const Segment SGI26417[] = {{&S17195[44], 77}, {NULL}};
const Segment SGI26418[] = {{&S17195[44], 98}, {NULL}};
const Segment SGI26419[] = {{&S17195[30], 6}, {NULL}};
const Segment SGI26420[] = {{&S17195[189], 3}, {NULL}};
const Segment SGI26421[] = {{&S17195[115], 5}, {NULL}};
const Segment SGI26422[] = {{&S17196[44], 19}, {NULL}};
const Segment SGI26423[] = {{&S17196[44], 40}, {NULL}};
const Segment SGI26424[] = {{&S17196[30], 6}, {NULL}};
const Segment SGI26425[] = {{&S17196[132], 3}, {NULL}};
const Segment SGI26426[] = {{&S17196[98], 5}, {NULL}};
const Segment SGI26427[] = {{&S17197[44], 12}, {NULL}};
const Segment SGI26428[] = {{&S17197[44], 33}, {NULL}};
const Segment SGI26429[] = {{&S17197[30], 6}, {NULL}};
const Segment SGI26430[] = {{&S17197[125], 3}, {NULL}};
const Segment SGI26431[] = {{&S17197[91], 5}, {NULL}};
const Segment SGI26432[] = {{&S17198[44], 79}, {NULL}};
const Segment SGI26433[] = {{&S17198[44], 100}, {NULL}};
const Segment SGI26434[] = {{&S17198[30], 6}, {NULL}};
const Segment SGI26435[] = {{&S17198[191], 3}, {NULL}};
const Segment SGI26436[] = {{&S17198[117], 5}, {NULL}};
const Segment SGI26437[] = {{&S17199[44], 12}, {NULL}};
const Segment SGI26438[] = {{&S17199[44], 33}, {NULL}};
const Segment SGI26439[] = {{&S17199[30], 6}, {NULL}};
const Segment SGI26440[] = {{&S17199[125], 3}, {NULL}};
const Segment SGI26441[] = {{&S17199[91], 5}, {NULL}};
const Segment SGI26442[] = {{&S17200[44], 8}, {NULL}};
const Segment SGI26443[] = {{&S17200[44], 29}, {NULL}};
const Segment SGI26444[] = {{&S17200[30], 6}, {NULL}};
const Segment SGI26445[] = {{&S17200[121], 3}, {NULL}};
const Segment SGI26446[] = {{&S17200[87], 5}, {NULL}};
const Segment SGI26447[] = {{&S17201[44], 82}, {NULL}};
const Segment SGI26448[] = {{&S17201[44], 103}, {NULL}};
const Segment SGI26449[] = {{&S17201[30], 6}, {NULL}};
const Segment SGI26450[] = {{&S17201[194], 3}, {NULL}};
const Segment SGI26451[] = {{&S17201[120], 5}, {NULL}};
const Segment SGI26452[] = {{&S17202[44], 10}, {NULL}};
const Segment SGI26453[] = {{&S17202[44], 31}, {NULL}};
const Segment SGI26454[] = {{&S17202[30], 6}, {NULL}};
const Segment SGI26455[] = {{&S17202[123], 3}, {NULL}};
const Segment SGI26456[] = {{&S17202[89], 5}, {NULL}};
const Segment SGI26457[] = {{&S17206[44], 6}, {NULL}};
const Segment SGI26458[] = {{&S17206[44], 27}, {NULL}};
const Segment SGI26459[] = {{&S17206[30], 6}, {NULL}};
const Segment SGI26460[] = {{&S17206[88], 4}, {NULL}};
const Segment SGI26461[] = {{&S17206[88], 3}, {NULL}};
const Segment SGI26462[] = {{&S17208[44], 29}, {NULL}};
const Segment SGI26463[] = {{&S17208[30], 6}, {NULL}};
const Segment SGI26464[] = {{&S17208[140], 3}, {NULL}};
const Segment SGI26465[] = {{&S17208[106], 5}, {NULL}};
const Segment SGI26466[] = {{&S17212[38], 8}, {NULL}};
const Segment SGI26467[] = {{&S17212[31], 6}, {NULL}};
const Segment SGI26468[] = {{&S17214[44], 6}, {NULL}};
const Segment SGI26469[] = {{&S17214[44], 27}, {NULL}};
const Segment SGI26470[] = {{&S17214[30], 6}, {NULL}};
const Segment SGI26471[] = {{&S17214[119], 3}, {NULL}};
const Segment SGI26472[] = {{&S17214[85], 5}, {NULL}};
const Segment SGI26473[] = {{&S17216[44], 13}, {NULL}};
const Segment SGI26474[] = {{&S17216[44], 34}, {NULL}};
const Segment SGI26475[] = {{&S17216[30], 6}, {NULL}};
const Segment SGI26476[] = {{&S17216[126], 3}, {NULL}};
const Segment SGI26477[] = {{&S17216[92], 5}, {NULL}};
const Segment SGI26478[] = {{&S17220[44], 14}, {NULL}};
const Segment SGI26479[] = {{&S17220[44], 35}, {NULL}};
const Segment SGI26480[] = {{&S17220[30], 6}, {NULL}};
const Segment SGI26481[] = {{&S17220[127], 3}, {NULL}};
const Segment SGI26482[] = {{&S17220[93], 5}, {NULL}};
const Segment SGI26483[] = {{&S17221[44], 11}, {NULL}};
const Segment SGI26484[] = {{&S17221[44], 22}, {NULL}};
const Segment SGI26485[] = {{&S17221[30], 6}, {NULL}};
const Segment SGI26486[] = {{&S17221[114], 3}, {NULL}};
const Segment SGI26487[] = {{&S17221[80], 5}, {NULL}};
const Segment SGI26488[] = {{&S17223[44], 10}, {NULL}};
const Segment SGI26489[] = {{&S17223[44], 31}, {NULL}};
const Segment SGI26490[] = {{&S17223[30], 6}, {NULL}};
const Segment SGI26491[] = {{&S17223[92], 6}, {NULL}};
const Segment SGI26492[] = {{&S20355[37], 7}, {NULL}};
const Segment SGI26493[] = {{&S20355[30], 6}, {NULL}};
const Segment SGI26494[] = {{&S17225[44], 8}, {NULL}};
const Segment SGI26495[] = {{&S20355[109], 3}, {NULL}};
const Segment SGI26496[] = {{&S17225[44], 29}, {NULL}};
const Segment SGI26497[] = {{&S17225[30], 6}, {NULL}};
const Segment SGI26498[] = {{&S20355[75], 5}, {NULL}};
const Segment SGI26499[] = {{&S17225[121], 3}, {NULL}};
const Segment SGI26500[] = {{&S17225[87], 5}, {NULL}};
const Segment SGI26501[] = {{&S17226[44], 6}, {NULL}};
const Segment SGI26502[] = {{&S17226[44], 17}, {NULL}};
const Segment SGI26503[] = {{&S17226[30], 6}, {NULL}};
const Segment SGI26504[] = {{&S17226[109], 3}, {NULL}};
const Segment SGI26505[] = {{&S17226[75], 5}, {NULL}};
const Segment SGI26506[] = {{&S17228[44], 74}, {NULL}};
const Segment SGI26507[] = {{&S17228[44], 95}, {NULL}};
const Segment SGI26508[] = {{&S17228[30], 6}, {NULL}};
const Segment SGI26509[] = {{&S17228[186], 3}, {NULL}};
const Segment SGI26510[] = {{&S17228[112], 5}, {NULL}};
const Segment SGI26511[] = {{&S20362[37], 7}, {NULL}};
const Segment SGI26512[] = {{&S20362[30], 6}, {NULL}};
const Segment SGI26513[] = {{&S17230[44], 79}, {NULL}};
const Segment SGI26514[] = {{&S17230[44], 100}, {NULL}};
const Segment SGI26515[] = {{&S20362[109], 3}, {NULL}};
const Segment SGI26516[] = {{&S17230[30], 6}, {NULL}};
const Segment SGI26517[] = {{&S20362[75], 5}, {NULL}};
const Segment SGI26518[] = {{&S17230[192], 3}, {NULL}};
const Segment SGI26519[] = {{&S17230[117], 5}, {NULL}};
const Segment SGI26520[] = {{&S20365[37], 7}, {NULL}};
const Segment SGI26521[] = {{&S20365[30], 6}, {NULL}};
const Segment SGI26522[] = {{&S20365[109], 3}, {NULL}};
const Segment SGI26523[] = {{&S17233[44], 14}, {NULL}};
const Segment SGI26524[] = {{&S20365[75], 5}, {NULL}};
const Segment SGI26525[] = {{&S17233[44], 35}, {NULL}};
const Segment SGI26526[] = {{&S17233[30], 6}, {NULL}};
const Segment SGI26527[] = {{&S17233[127], 3}, {NULL}};
const Segment SGI26528[] = {{&S17233[93], 5}, {NULL}};
const Segment SGI26529[] = {{&S17238[44], 27}, {NULL}};
const Segment SGI26530[] = {{&S17238[30], 6}, {NULL}};
const Segment SGI26531[] = {{&S17238[138], 3}, {NULL}};
const Segment SGI26532[] = {{&S17238[104], 5}, {NULL}};
const Segment SGI26533[] = {{&S17239[38], 17}, {NULL}};
const Segment SGI26534[] = {{&S17239[31], 6}, {NULL}};
const Segment SGI26535[] = {{&S17241[44], 10}, {NULL}};
const Segment SGI26536[] = {{&S17241[44], 31}, {NULL}};
const Segment SGI26537[] = {{&S17241[30], 6}, {NULL}};
const Segment SGI26538[] = {{&S17241[123], 3}, {NULL}};
const Segment SGI26539[] = {{&S17241[89], 5}, {NULL}};
const Segment SGI26540[] = {{&S17242[44], 84}, {NULL}};
const Segment SGI26541[] = {{&S17242[44], 105}, {NULL}};
const Segment SGI26542[] = {{&S17242[30], 6}, {NULL}};
const Segment SGI26543[] = {{&S17242[196], 3}, {NULL}};
const Segment SGI26544[] = {{&S17242[122], 5}, {NULL}};
const Segment SGI26545[] = {{&S17243[44], 7}, {NULL}};
const Segment SGI26546[] = {{&S17243[44], 18}, {NULL}};
const Segment SGI26547[] = {{&S17243[30], 6}, {NULL}};
const Segment SGI26548[] = {{&S17243[110], 3}, {NULL}};
const Segment SGI26549[] = {{&S17243[76], 5}, {NULL}};
const Segment SGI26550[] = {{&S17245[44], 80}, {NULL}};
const Segment SGI26551[] = {{&S17245[44], 101}, {NULL}};
const Segment SGI26552[] = {{&S17245[30], 6}, {NULL}};
const Segment SGI26553[] = {{&S17245[193], 3}, {NULL}};
const Segment SGI26554[] = {{&S17245[118], 5}, {NULL}};
const Segment SGI26555[] = {{&S17246[44], 37}, {NULL}};
const Segment SGI26556[] = {{&S17246[30], 6}, {NULL}};
const Segment SGI26557[] = {{&S17246[147], 3}, {NULL}};
const Segment SGI26558[] = {{&S17246[113], 5}, {NULL}};
const Segment SGI26559[] = {{&S17247[44], 76}, {NULL}};
const Segment SGI26560[] = {{&S17247[44], 97}, {NULL}};
const Segment SGI26561[] = {{&S17247[30], 6}, {NULL}};
const Segment SGI26562[] = {{&S17247[188], 3}, {NULL}};
const Segment SGI26563[] = {{&S17247[114], 5}, {NULL}};
const Segment SGI26564[] = {{&S17249[37], 81}, {NULL}};
const Segment SGI26565[] = {{&S17249[30], 6}, {NULL}};
const Segment SGI26566[] = {{&S17249[177], 3}, {NULL}};
const Segment SGI26567[] = {{&S17249[112], 5}, {NULL}};
const Segment SGI26568[] = {{&S17250[44], 8}, {NULL}};
const Segment SGI26569[] = {{&S17250[44], 19}, {NULL}};
const Segment SGI26570[] = {{&S17250[30], 6}, {NULL}};
const Segment SGI26571[] = {{&S17250[111], 3}, {NULL}};
const Segment SGI26572[] = {{&S17250[77], 5}, {NULL}};
const Segment SGI26573[] = {{&S20394[37], 7}, {NULL}};
const Segment SGI26574[] = {{&S20394[30], 6}, {NULL}};
const Segment SGI26575[] = {{&S20394[109], 3}, {NULL}};
const Segment SGI26576[] = {{&S20394[75], 5}, {NULL}};
const Segment SGI26577[] = {{&S20395[37], 7}, {NULL}};
const Segment SGI26578[] = {{&S20395[30], 6}, {NULL}};
const Segment SGI26579[] = {{&S20395[109], 3}, {NULL}};
const Segment SGI26580[] = {{&S20395[75], 5}, {NULL}};
const Segment SGI26581[] = {{&S17255[44], 7}, {NULL}};
const Segment SGI26582[] = {{&S17255[44], 18}, {NULL}};
const Segment SGI26583[] = {{&S17255[30], 6}, {NULL}};
const Segment SGI26584[] = {{&S17255[110], 3}, {NULL}};
const Segment SGI26585[] = {{&S17255[76], 5}, {NULL}};
const Segment SGI26586[] = {{&S17257[44], 27}, {NULL}};
const Segment SGI26587[] = {{&S17257[30], 6}, {NULL}};
const Segment SGI26588[] = {{&S17257[138], 3}, {NULL}};
const Segment SGI26589[] = {{&S17257[104], 5}, {NULL}};
const Segment SGI26590[] = {{&S17258[44], 35}, {NULL}};
const Segment SGI26591[] = {{&S17258[30], 6}, {NULL}};
const Segment SGI26592[] = {{&S17258[146], 3}, {NULL}};
const Segment SGI26593[] = {{&S17258[112], 5}, {NULL}};
const Segment SGI26594[] = {{&S17262[44], 13}, {NULL}};
const Segment SGI26595[] = {{&S17262[44], 34}, {NULL}};
const Segment SGI26596[] = {{&S17262[30], 6}, {NULL}};
const Segment SGI26597[] = {{&S17262[95], 4}, {NULL}};
const Segment SGI26598[] = {{&S17262[95], 3}, {NULL}};
const Segment SGI26599[] = {{&S17266[44], 13}, {NULL}};
const Segment SGI26600[] = {{&S17266[44], 24}, {NULL}};
const Segment SGI26601[] = {{&S17266[30], 6}, {NULL}};
const Segment SGI26602[] = {{&S17266[116], 3}, {NULL}};
const Segment SGI26603[] = {{&S17266[82], 5}, {NULL}};
const Segment SGI26604[] = {{&S20420[37], 7}, {NULL}};
const Segment SGI26605[] = {{&S20420[30], 6}, {NULL}};
const Segment SGI26606[] = {{&S20420[109], 3}, {NULL}};
const Segment SGI26607[] = {{&S20420[75], 5}, {NULL}};
const Segment SGI26608[] = {{&S17273[44], 6}, {NULL}};
const Segment SGI26609[] = {{&S17273[44], 17}, {NULL}};
const Segment SGI26610[] = {{&S17273[30], 6}, {NULL}};
const Segment SGI26611[] = {{&S17273[109], 3}, {NULL}};
const Segment SGI26612[] = {{&S17273[75], 5}, {NULL}};
const Segment SGI26613[] = {{&S17280[44], 6}, {NULL}};
const Segment SGI26614[] = {{&S17280[44], 27}, {NULL}};
const Segment SGI26615[] = {{&S17280[30], 6}, {NULL}};
const Segment SGI26616[] = {{&S17280[119], 3}, {NULL}};
const Segment SGI26617[] = {{&S17280[85], 5}, {NULL}};
const Segment SGI26618[] = {{&S17281[44], 79}, {NULL}};
const Segment SGI26619[] = {{&S17281[44], 101}, {NULL}};
const Segment SGI26620[] = {{&S17281[30], 6}, {NULL}};
const Segment SGI26621[] = {{&S20433[37], 7}, {NULL}};
const Segment SGI26622[] = {{&S20433[30], 6}, {NULL}};
const Segment SGI26623[] = {{&S20433[109], 3}, {NULL}};
const Segment SGI26624[] = {{&S17281[193], 3}, {NULL}};
const Segment SGI26625[] = {{&S20433[75], 5}, {NULL}};
const Segment SGI26626[] = {{&S17281[117], 5}, {NULL}};
const Segment SGI26627[] = {{&S17284[44], 80}, {NULL}};
const Segment SGI26628[] = {{&S17284[44], 101}, {NULL}};
const Segment SGI26629[] = {{&S17284[30], 6}, {NULL}};
const Segment SGI26630[] = {{&S17284[192], 3}, {NULL}};
const Segment SGI26631[] = {{&S17284[118], 5}, {NULL}};
const Segment SGI26632[] = {{&S17286[44], 18}, {NULL}};
const Segment SGI26633[] = {{&S17286[44], 29}, {NULL}};
const Segment SGI26634[] = {{&S17286[30], 6}, {NULL}};
const Segment SGI26635[] = {{&S17286[121], 3}, {NULL}};
const Segment SGI26636[] = {{&S17286[87], 5}, {NULL}};
const Segment SGI26637[] = {{&S17289[44], 7}, {NULL}};
const Segment SGI26638[] = {{&S17289[44], 28}, {NULL}};
const Segment SGI26639[] = {{&S17289[30], 6}, {NULL}};
const Segment SGI26640[] = {{&S17289[120], 3}, {NULL}};
const Segment SGI26641[] = {{&S17289[86], 5}, {NULL}};
const Segment SGI26642[] = {{&S17292[44], 6}, {NULL}};
const Segment SGI26643[] = {{&S17292[44], 22}, {NULL}};
const Segment SGI26644[] = {{&S17292[30], 6}, {NULL}};
const Segment SGI26645[] = {{&S17292[114], 3}, {NULL}};
const Segment SGI26646[] = {{&S17292[80], 5}, {NULL}};
const Segment SGI26647[] = {{&S17308[44], 6}, {NULL}};
const Segment SGI26648[] = {{&S17308[44], 27}, {NULL}};
const Segment SGI26649[] = {{&S17308[30], 6}, {NULL}};
const Segment SGI26650[] = {{&S17308[119], 3}, {NULL}};
const Segment SGI26651[] = {{&S17308[85], 5}, {NULL}};
const Segment SGI26652[] = {{&S17311[44], 17}, {NULL}};
const Segment SGI26653[] = {{&S17311[44], 38}, {NULL}};
const Segment SGI26654[] = {{&S17311[30], 6}, {NULL}};
const Segment SGI26655[] = {{&S17311[130], 3}, {NULL}};
const Segment SGI26656[] = {{&S17311[96], 5}, {NULL}};
const Segment SGI26657[] = {{&S17312[44], 84}, {NULL}};
const Segment SGI26658[] = {{&S17312[44], 105}, {NULL}};
const Segment SGI26659[] = {{&S17312[30], 6}, {NULL}};
const Segment SGI26660[] = {{&S17312[196], 3}, {NULL}};
const Segment SGI26661[] = {{&S17312[122], 5}, {NULL}};
const Segment SGI26662[] = {{&S17314[44], 10}, {NULL}};
const Segment SGI26663[] = {{&S17314[44], 31}, {NULL}};
const Segment SGI26664[] = {{&S17314[30], 6}, {NULL}};
const Segment SGI26665[] = {{&S17314[123], 3}, {NULL}};
const Segment SGI26666[] = {{&S17314[89], 5}, {NULL}};
const Segment SGI26667[] = {{&S17316[44], 8}, {NULL}};
const Segment SGI26668[] = {{&S17316[44], 29}, {NULL}};
const Segment SGI26669[] = {{&S17316[30], 6}, {NULL}};
const Segment SGI26670[] = {{&S17316[121], 3}, {NULL}};
const Segment SGI26671[] = {{&S17316[87], 5}, {NULL}};
const Segment SGI26672[] = {{&S17318[44], 81}, {NULL}};
const Segment SGI26673[] = {{&S17318[44], 102}, {NULL}};
const Segment SGI26674[] = {{&S17318[30], 6}, {NULL}};
const Segment SGI26675[] = {{&S17318[193], 3}, {NULL}};
const Segment SGI26676[] = {{&S17318[119], 5}, {NULL}};
const Segment SGI26677[] = {{&S20613[37], 7}, {NULL}};
const Segment SGI26678[] = {{&S20613[30], 6}, {NULL}};
const Segment SGI26679[] = {{&S20613[109], 3}, {NULL}};
const Segment SGI26680[] = {{&S20613[75], 5}, {NULL}};
const Segment SGI26681[] = {{&S18165[44], 10}, {NULL}};
const Segment SGI26682[] = {{&S18165[44], 27}, {NULL}};
const Segment SGI26683[] = {{&S18165[30], 6}, {NULL}};
const Segment SGI26684[] = {{&S18165[119], 3}, {NULL}};
const Segment SGI26685[] = {{&S18165[85], 5}, {NULL}};
const Segment SGI26686[] = {{&S18166[38], 10}, {NULL}};
const Segment SGI26687[] = {{&S18166[31], 6}, {NULL}};
const Segment SGI26688[] = {{&S18168[44], 10}, {NULL}};
const Segment SGI26689[] = {{&S18168[44], 21}, {NULL}};
const Segment SGI26690[] = {{&S18168[30], 6}, {NULL}};
const Segment SGI26691[] = {{&S18168[113], 3}, {NULL}};
const Segment SGI26692[] = {{&S18168[79], 5}, {NULL}};
const Segment SGI26693[] = {{&S19011[44], 19}, {NULL}};
const Segment SGI26694[] = {{&S19011[44], 38}, {NULL}};
const Segment SGI26695[] = {{&S19011[30], 6}, {NULL}};
const Segment SGI26696[] = {{&S19011[130], 3}, {NULL}};
const Segment SGI26697[] = {{&S19011[96], 5}, {NULL}};
const Segment SGI26698[] = {{&S19012[44], 7}, {NULL}};
const Segment SGI26699[] = {{&S19012[44], 26}, {NULL}};
const Segment SGI26700[] = {{&S19012[30], 6}, {NULL}};
const Segment SGI26701[] = {{&S19012[118], 3}, {NULL}};
const Segment SGI26702[] = {{&S19012[84], 5}, {NULL}};
const Segment SGI26703[] = {{&S19013[44], 21}, {NULL}};
const Segment SGI26704[] = {{&S19013[44], 40}, {NULL}};
const Segment SGI26705[] = {{&S19013[30], 6}, {NULL}};
const Segment SGI26706[] = {{&S19013[132], 3}, {NULL}};
const Segment SGI26707[] = {{&S19013[98], 5}, {NULL}};
const Segment SGI26708[] = {{&S19014[44], 19}, {NULL}};
const Segment SGI26709[] = {{&S19014[44], 38}, {NULL}};
const Segment SGI26710[] = {{&S19014[30], 6}, {NULL}};
const Segment SGI26711[] = {{&S19014[130], 3}, {NULL}};
const Segment SGI26712[] = {{&S19014[96], 5}, {NULL}};
const Segment SGI26713[] = {{&S19015[44], 7}, {NULL}};
const Segment SGI26714[] = {{&S19015[44], 26}, {NULL}};
const Segment SGI26715[] = {{&S19015[30], 6}, {NULL}};
const Segment SGI26716[] = {{&S19015[118], 3}, {NULL}};
const Segment SGI26717[] = {{&S19015[84], 5}, {NULL}};
const Segment SGI26718[] = {{&S19017[44], 7}, {NULL}};
const Segment SGI26719[] = {{&S19017[44], 24}, {NULL}};
const Segment SGI26720[] = {{&S19017[30], 6}, {NULL}};
const Segment SGI26721[] = {{&S19017[57], 3}, {NULL}};
const Segment SGI26722[] = {{&S19017[82], 5}, {NULL}};
const Segment SGI26723[] = {{&S19019[44], 19}, {NULL}};
const Segment SGI26724[] = {{&S19019[44], 36}, {NULL}};
const Segment SGI26725[] = {{&S19019[30], 6}, {NULL}};
const Segment SGI26726[] = {{&S19019[69], 3}, {NULL}};
const Segment SGI26727[] = {{&S19019[94], 5}, {NULL}};
const Segment SGI26728[] = {{&S19021[44], 20}, {NULL}};
const Segment SGI26729[] = {{&S19021[44], 37}, {NULL}};
const Segment SGI26730[] = {{&S19021[30], 6}, {NULL}};
const Segment SGI26731[] = {{&S19021[70], 3}, {NULL}};
const Segment SGI26732[] = {{&S19021[95], 5}, {NULL}};
const Segment SGI26733[] = {{&S19022[44], 19}, {NULL}};
const Segment SGI26734[] = {{&S19022[44], 36}, {NULL}};
const Segment SGI26735[] = {{&S19022[30], 6}, {NULL}};
const Segment SGI26736[] = {{&S19022[69], 3}, {NULL}};
const Segment SGI26737[] = {{&S19022[94], 5}, {NULL}};
const Segment SGI26738[] = {{&S19023[42], 19}, {NULL}};
const Segment SGI26739[] = {{&S19023[30], 6}, {NULL}};
const Segment SGI26740[] = {{&S19023[67], 3}, {NULL}};
const Segment SGI26741[] = {{&S19023[92], 5}, {NULL}};
const Segment SGI26742[] = {{&S19030[44], 20}, {NULL}};
const Segment SGI26743[] = {{&S19030[44], 39}, {NULL}};
const Segment SGI26744[] = {{&S19030[30], 6}, {NULL}};
const Segment SGI26745[] = {{&S19030[131], 3}, {NULL}};
const Segment SGI26746[] = {{&S19030[97], 5}, {NULL}};
const Segment SGI26747[] = {{&S19031[44], 19}, {NULL}};
const Segment SGI26748[] = {{&S19031[44], 38}, {NULL}};
const Segment SGI26749[] = {{&S19031[30], 6}, {NULL}};
const Segment SGI26750[] = {{&S19031[130], 3}, {NULL}};
const Segment SGI26751[] = {{&S19031[96], 5}, {NULL}};
const Segment SGI26752[] = {{&S19033[44], 7}, {NULL}};
const Segment SGI26753[] = {{&S19033[44], 26}, {NULL}};
const Segment SGI26754[] = {{&S19033[30], 6}, {NULL}};
const Segment SGI26755[] = {{&S19033[118], 3}, {NULL}};
const Segment SGI26756[] = {{&S19033[84], 5}, {NULL}};
const Segment SGI26757[] = {{&S19034[44], 7}, {NULL}};
const Segment SGI26758[] = {{&S19034[44], 26}, {NULL}};
const Segment SGI26759[] = {{&S19034[30], 6}, {NULL}};
const Segment SGI26760[] = {{&S19034[118], 3}, {NULL}};
const Segment SGI26761[] = {{&S19034[84], 5}, {NULL}};
const Segment SGI26762[] = {{&S19036[44], 19}, {NULL}};
const Segment SGI26763[] = {{&S19036[44], 36}, {NULL}};
const Segment SGI26764[] = {{&S19036[30], 6}, {NULL}};
const Segment SGI26765[] = {{&S19036[69], 3}, {NULL}};
const Segment SGI26766[] = {{&S19036[94], 5}, {NULL}};
const Segment SGI26767[] = {{&S19037[44], 21}, {NULL}};
const Segment SGI26768[] = {{&S19037[44], 40}, {NULL}};
const Segment SGI26769[] = {{&S19037[30], 6}, {NULL}};
const Segment SGI26770[] = {{&S19037[132], 3}, {NULL}};
const Segment SGI26771[] = {{&S19037[98], 5}, {NULL}};
const Segment SGI26772[] = {{&S19038[44], 20}, {NULL}};
const Segment SGI26773[] = {{&S19038[44], 39}, {NULL}};
const Segment SGI26774[] = {{&S19038[30], 6}, {NULL}};
const Segment SGI26775[] = {{&S19038[131], 3}, {NULL}};
const Segment SGI26776[] = {{&S19038[97], 5}, {NULL}};
const Segment SGI26777[] = {{&S19039[44], 19}, {NULL}};
const Segment SGI26778[] = {{&S19039[44], 38}, {NULL}};
const Segment SGI26779[] = {{&S19039[30], 6}, {NULL}};
const Segment SGI26780[] = {{&S19039[130], 3}, {NULL}};
const Segment SGI26781[] = {{&S19039[96], 5}, {NULL}};
const Segment SGI26782[] = {{&S19040[44], 19}, {NULL}};
const Segment SGI26783[] = {{&S19040[44], 36}, {NULL}};
const Segment SGI26784[] = {{&S19040[30], 6}, {NULL}};
const Segment SGI26785[] = {{&S19040[69], 3}, {NULL}};
const Segment SGI26786[] = {{&S19040[94], 5}, {NULL}};
const Segment SGI26787[] = {{&S19042[44], 19}, {NULL}};
const Segment SGI26788[] = {{&S19042[44], 36}, {NULL}};
const Segment SGI26789[] = {{&S19042[30], 6}, {NULL}};
const Segment SGI26790[] = {{&S19042[69], 3}, {NULL}};
const Segment SGI26791[] = {{&S19042[94], 5}, {NULL}};
const Segment SGI26792[] = {{&S19043[44], 19}, {NULL}};
const Segment SGI26793[] = {{&S19043[44], 36}, {NULL}};
const Segment SGI26794[] = {{&S19043[30], 6}, {NULL}};
const Segment SGI26795[] = {{&S19043[69], 3}, {NULL}};
const Segment SGI26796[] = {{&S19043[94], 5}, {NULL}};
const Segment SGI26797[] = {{&S19044[44], 19}, {NULL}};
const Segment SGI26798[] = {{&S19044[44], 36}, {NULL}};
const Segment SGI26799[] = {{&S19044[30], 6}, {NULL}};
const Segment SGI26800[] = {{&S19044[69], 3}, {NULL}};
const Segment SGI26801[] = {{&S19044[94], 5}, {NULL}};
const Segment SGI26802[] = {{&S19052[44], 19}, {NULL}};
const Segment SGI26803[] = {{&S19052[44], 36}, {NULL}};
const Segment SGI26804[] = {{&S19052[30], 6}, {NULL}};
const Segment SGI26805[] = {{&S19052[69], 3}, {NULL}};
const Segment SGI26806[] = {{&S19052[94], 5}, {NULL}};
const Segment SGI26807[] = {{&S19057[44], 19}, {NULL}};
const Segment SGI26808[] = {{&S19057[44], 36}, {NULL}};
const Segment SGI26809[] = {{&S19057[30], 6}, {NULL}};
const Segment SGI26810[] = {{&S19057[69], 3}, {NULL}};
const Segment SGI26811[] = {{&S19057[94], 5}, {NULL}};
const Segment SGI26812[] = {{&S19058[44], 19}, {NULL}};
const Segment SGI26813[] = {{&S19058[44], 38}, {NULL}};
const Segment SGI26814[] = {{&S19058[30], 6}, {NULL}};
const Segment SGI26815[] = {{&S19058[130], 3}, {NULL}};
const Segment SGI26816[] = {{&S19058[96], 5}, {NULL}};
const Segment SGI26817[] = {{&S19059[44], 19}, {NULL}};
const Segment SGI26818[] = {{&S19059[44], 35}, {NULL}};
const Segment SGI26819[] = {{&S19059[30], 6}, {NULL}};
const Segment SGI26820[] = {{&S19059[69], 3}, {NULL}};
const Segment SGI26821[] = {{&S19059[93], 5}, {NULL}};
const Segment SGI26822[] = {{&S19061[38], 7}, {NULL}};
const Segment SGI26823[] = {{&S19061[31], 6}, {NULL}};
const Segment SGI26824[] = {{&S19075[44], 7}, {NULL}};
const Segment SGI26825[] = {{&S19075[44], 26}, {NULL}};
const Segment SGI26826[] = {{&S19075[30], 6}, {NULL}};
const Segment SGI26827[] = {{&S19075[87], 4}, {NULL}};
const Segment SGI26828[] = {{&S19075[87], 3}, {NULL}};
const Segment SGI26829[] = {{&S19079[44], 7}, {NULL}};
const Segment SGI26830[] = {{&S19079[44], 24}, {NULL}};
const Segment SGI26831[] = {{&S19079[30], 6}, {NULL}};
const Segment SGI26832[] = {{&S19079[85], 4}, {NULL}};
const Segment SGI26833[] = {{&S19079[85], 3}, {NULL}};
const Segment SGI26834[] = {{&S19083[44], 7}, {NULL}};
const Segment SGI26835[] = {{&S19083[44], 26}, {NULL}};
const Segment SGI26836[] = {{&S19083[30], 6}, {NULL}};
const Segment SGI26837[] = {{&S19083[87], 6}, {NULL}};
const Segment SGI26838[] = {{&S29034[44], 6}, {NULL}};
const Segment SGI26839[] = {{&S29034[44], 22}, {NULL}};
const Segment SGI26840[] = {{&S29034[30], 6}, {NULL}};
const Segment SGI26841[] = {{&S29034[114], 3}, {NULL}};
const Segment SGI26842[] = {{&S29034[80], 5}, {NULL}};
const Segment SGI26843[] = {{&S29038[44], 13}, {NULL}};
const Segment SGI26844[] = {{&S29038[44], 25}, {NULL}};
const Segment SGI26845[] = {{&S29038[30], 6}, {NULL}};
const Segment SGI26846[] = {{&S29038[117], 3}, {NULL}};
const Segment SGI26847[] = {{&S29038[83], 5}, {NULL}};
const Segment SGI26848[] = {{&S29044[44], 5}, {NULL}};
const Segment SGI26849[] = {{&S29044[44], 16}, {NULL}};
const Segment SGI26850[] = {{&S29044[30], 6}, {NULL}};
const Segment SGI26851[] = {{&S29044[108], 3}, {NULL}};
const Segment SGI26852[] = {{&S29044[74], 5}, {NULL}};
const Segment SGI26853[] = {{&S29052[44], 11}, {NULL}};
const Segment SGI26854[] = {{&S29052[44], 23}, {NULL}};
const Segment SGI26855[] = {{&S29052[30], 6}, {NULL}};
const Segment SGI26856[] = {{&S29052[115], 3}, {NULL}};
const Segment SGI26857[] = {{&S29052[81], 5}, {NULL}};
const Segment SGI26858[] = {{&S29056[44], 6}, {NULL}};
const Segment SGI26859[] = {{&S29056[44], 17}, {NULL}};
const Segment SGI26860[] = {{&S29056[30], 6}, {NULL}};
const Segment SGI26861[] = {{&S29056[109], 3}, {NULL}};
const Segment SGI26862[] = {{&S29056[75], 5}, {NULL}};
const Segment SGI26863[] = {{&S29057[44], 6}, {NULL}};
const Segment SGI26864[] = {{&S29057[44], 18}, {NULL}};
const Segment SGI26865[] = {{&S29057[30], 6}, {NULL}};
const Segment SGI26866[] = {{&S29057[110], 3}, {NULL}};
const Segment SGI26867[] = {{&S29057[76], 5}, {NULL}};
const Segment SGI26868[] = {{&S29061[44], 11}, {NULL}};
const Segment SGI26869[] = {{&S29061[44], 35}, {NULL}};
const Segment SGI26870[] = {{&S29061[30], 6}, {NULL}};
const Segment SGI26871[] = {{&S29061[127], 3}, {NULL}};
const Segment SGI26872[] = {{&S29061[93], 5}, {NULL}};
const Segment SGI26873[] = {{&S29062[47], 9}, {NULL}};
const Segment SGI26874[] = {{&S29062[47], 26}, {NULL}};
const Segment SGI26875[] = {{&S29062[33], 6}, {NULL}};
const Segment SGI26876[] = {{&S29062[121], 3}, {NULL}};
const Segment SGI26877[] = {{&S29062[87], 5}, {NULL}};
const Segment SGI26878[] = {{&S29064[44], 34}, {NULL}};
const Segment SGI26879[] = {{&S29064[30], 6}, {NULL}};
const Segment SGI26880[] = {{&S29064[136], 3}, {NULL}};
const Segment SGI26881[] = {{&S29064[101], 6}, {NULL}};
const Segment SGI26882[] = {{&S29075[44], 9}, {NULL}};
const Segment SGI26883[] = {{&S29075[44], 21}, {NULL}};
const Segment SGI26884[] = {{&S29075[30], 6}, {NULL}};
const Segment SGI26885[] = {{&S29075[113], 3}, {NULL}};
const Segment SGI26886[] = {{&S29075[79], 5}, {NULL}};
const Segment SGI26887[] = {{&S29101[44], 12}, {NULL}};
const Segment SGI26888[] = {{&S29101[44], 23}, {NULL}};
const Segment SGI26889[] = {{&S29101[30], 6}, {NULL}};
const Segment SGI26890[] = {{&S29101[115], 3}, {NULL}};
const Segment SGI26891[] = {{&S29101[81], 5}, {NULL}};
const Segment SGI26892[] = {{&S29102[44], 9}, {NULL}};
const Segment SGI26893[] = {{&S29102[44], 21}, {NULL}};
const Segment SGI26894[] = {{&S29102[30], 6}, {NULL}};
const Segment SGI26895[] = {{&S29102[122], 3}, {NULL}};
const Segment SGI26896[] = {{&S29102[87], 6}, {NULL}};
const Segment SGI26897[] = {{&S29102[142], 8}, {NULL}};
const Segment SGI26898[] = {{&S29103[44], 6}, {NULL}};
const Segment SGI26899[] = {{&S29103[44], 18}, {NULL}};
const Segment SGI26900[] = {{&S29103[30], 6}, {NULL}};
const Segment SGI26901[] = {{&S29103[110], 3}, {NULL}};
const Segment SGI26902[] = {{&S29103[76], 5}, {NULL}};
const Segment SGI26903[] = {{&S29104[44], 9}, {NULL}};
const Segment SGI26904[] = {{&S29104[44], 21}, {NULL}};
const Segment SGI26905[] = {{&S29104[30], 6}, {NULL}};
const Segment SGI26906[] = {{&S29104[113], 3}, {NULL}};
const Segment SGI26907[] = {{&S29104[79], 5}, {NULL}};
const Segment SGI26908[] = {{&S29108[42], 7}, {NULL}};
const Segment SGI26909[] = {{&S29108[30], 6}, {NULL}};
const Segment SGI26910[] = {{&S29108[109], 3}, {NULL}};
const Segment SGI26911[] = {{&S29108[75], 5}, {NULL}};
const Segment SGI26912[] = {{&S29112[44], 14}, {NULL}};
const Segment SGI26913[] = {{&S29112[44], 26}, {NULL}};
const Segment SGI26914[] = {{&S29112[30], 6}, {NULL}};
const Segment SGI26915[] = {{&S29112[118], 3}, {NULL}};
const Segment SGI26916[] = {{&S29112[84], 5}, {NULL}};
const Segment SGI26917[] = {{&S29118[44], 8}, {NULL}};
const Segment SGI26918[] = {{&S29118[44], 20}, {NULL}};
const Segment SGI26919[] = {{&S29118[30], 6}, {NULL}};
const Segment SGI26920[] = {{&S29118[81], 4}, {NULL}};
const Segment SGI26921[] = {{&S29118[81], 3}, {NULL}};
const Segment SGI26922[] = {{&S29123[44], 8}, {NULL}};
const Segment SGI26923[] = {{&S29123[44], 20}, {NULL}};
const Segment SGI26924[] = {{&S29123[30], 6}, {NULL}};
const Segment SGI26925[] = {{&S29123[112], 3}, {NULL}};
const Segment SGI26926[] = {{&S29123[78], 5}, {NULL}};
const Segment SGI26927[] = {{&S29131[44], 13}, {NULL}};
const Segment SGI26928[] = {{&S29131[44], 25}, {NULL}};
const Segment SGI26929[] = {{&S29131[30], 6}, {NULL}};
const Segment SGI26930[] = {{&S29131[117], 3}, {NULL}};
const Segment SGI26931[] = {{&S29131[83], 5}, {NULL}};
const Segment SGI26932[] = {{&S29133[44], 7}, {NULL}};
const Segment SGI26933[] = {{&S29133[44], 18}, {NULL}};
const Segment SGI26934[] = {{&S29133[30], 6}, {NULL}};
const Segment SGI26935[] = {{&S29133[110], 3}, {NULL}};
const Segment SGI26936[] = {{&S29133[76], 5}, {NULL}};
const Segment SGI26937[] = {{&S29136[44], 7}, {NULL}};
const Segment SGI26938[] = {{&S29136[44], 19}, {NULL}};
const Segment SGI26939[] = {{&S29136[30], 6}, {NULL}};
const Segment SGI26940[] = {{&S29136[111], 3}, {NULL}};
const Segment SGI26941[] = {{&S29136[77], 5}, {NULL}};
const Segment SGI26942[] = {{&S29142[44], 7}, {NULL}};
const Segment SGI26943[] = {{&S29142[44], 18}, {NULL}};
const Segment SGI26944[] = {{&S29142[30], 6}, {NULL}};
const Segment SGI26945[] = {{&S29142[110], 3}, {NULL}};
const Segment SGI26946[] = {{&S29142[76], 5}, {NULL}};
const Segment SGI26947[] = {{&S29143[44], 7}, {NULL}};
const Segment SGI26948[] = {{&S29143[44], 19}, {NULL}};
const Segment SGI26949[] = {{&S29143[30], 6}, {NULL}};
const Segment SGI26950[] = {{&S29143[111], 3}, {NULL}};
const Segment SGI26951[] = {{&S29143[77], 5}, {NULL}};
const Segment SGI26952[] = {{&S29149[44], 27}, {NULL}};
const Segment SGI26953[] = {{&S29149[30], 6}, {NULL}};
const Segment SGI26954[] = {{&S29149[137], 3}, {NULL}};
const Segment SGI26955[] = {{&S29149[103], 5}, {NULL}};
const Segment SGI26956[] = {{&S29153[44], 5}, {NULL}};
const Segment SGI26957[] = {{&S29153[44], 16}, {NULL}};
const Segment SGI26958[] = {{&S29153[30], 6}, {NULL}};
const Segment SGI26959[] = {{&S29153[108], 3}, {NULL}};
const Segment SGI26960[] = {{&S29153[74], 5}, {NULL}};
const Segment SGI26961[] = {{&S29155[44], 10}, {NULL}};
const Segment SGI26962[] = {{&S29155[44], 21}, {NULL}};
const Segment SGI26963[] = {{&S29155[30], 6}, {NULL}};
const Segment SGI26964[] = {{&S29155[113], 3}, {NULL}};
const Segment SGI26965[] = {{&S29155[79], 5}, {NULL}};
const Segment SGI26966[] = {{&S29158[44], 6}, {NULL}};
const Segment SGI26967[] = {{&S29158[44], 45}, {NULL}};
const Segment SGI26968[] = {{&S29158[30], 6}, {NULL}};
const Segment SGI26969[] = {{&S29158[137], 3}, {NULL}};
const Segment SGI26970[] = {{&S29158[103], 5}, {NULL}};
const Segment SGI26971[] = {{&S29164[44], 16}, {NULL}};
const Segment SGI26972[] = {{&S29164[44], 27}, {NULL}};
const Segment SGI26973[] = {{&S29164[30], 6}, {NULL}};
const Segment SGI26974[] = {{&S29164[119], 3}, {NULL}};
const Segment SGI26975[] = {{&S29164[85], 5}, {NULL}};
const Segment SGI26976[] = {{&S29169[44], 12}, {NULL}};
const Segment SGI26977[] = {{&S29169[44], 36}, {NULL}};
const Segment SGI26978[] = {{&S29169[30], 6}, {NULL}};
const Segment SGI26979[] = {{&S29169[128], 3}, {NULL}};
const Segment SGI26980[] = {{&S29169[94], 5}, {NULL}};
const Segment SGI26981[] = {{&S29171[44], 11}, {NULL}};
const Segment SGI26982[] = {{&S29171[44], 23}, {NULL}};
const Segment SGI26983[] = {{&S29171[30], 6}, {NULL}};
const Segment SGI26984[] = {{&S29171[115], 3}, {NULL}};
const Segment SGI26985[] = {{&S29171[81], 5}, {NULL}};
const Segment SGI26986[] = {{&S29174[44], 13}, {NULL}};
const Segment SGI26987[] = {{&S29174[44], 25}, {NULL}};
const Segment SGI26988[] = {{&S29174[30], 6}, {NULL}};
const Segment SGI26989[] = {{&S29174[117], 3}, {NULL}};
const Segment SGI26990[] = {{&S29174[83], 5}, {NULL}};
const Segment SGI26991[] = {{&S29177[44], 9}, {NULL}};
const Segment SGI26992[] = {{&S29177[44], 21}, {NULL}};
const Segment SGI26993[] = {{&S29177[30], 6}, {NULL}};
const Segment SGI26994[] = {{&S29177[113], 3}, {NULL}};
const Segment SGI26995[] = {{&S29177[79], 5}, {NULL}};
const Segment SGI26996[] = {{&S29188[44], 4}, {NULL}};
const Segment SGI26997[] = {{&S29188[44], 15}, {NULL}};
const Segment SGI26998[] = {{&S29188[30], 6}, {NULL}};
const Segment SGI26999[] = {{&S29188[107], 3}, {NULL}};
const Segment SGI27000[] = {{&S29188[73], 5}, {NULL}};
const Segment SGI27001[] = {{&S29189[44], 7}, {NULL}};
const Segment SGI27002[] = {{&S29189[44], 19}, {NULL}};
const Segment SGI27003[] = {{&S29189[30], 6}, {NULL}};
const Segment SGI27004[] = {{&S29189[111], 3}, {NULL}};
const Segment SGI27005[] = {{&S29189[77], 5}, {NULL}};
const Segment SGI27006[] = {{&S29191[44], 8}, {NULL}};
const Segment SGI27007[] = {{&S29191[44], 19}, {NULL}};
const Segment SGI27008[] = {{&S29191[30], 6}, {NULL}};
const Segment SGI27009[] = {{&S29191[111], 3}, {NULL}};
const Segment SGI27010[] = {{&S29191[77], 5}, {NULL}};
const Segment SGI27011[] = {{&S29194[44], 6}, {NULL}};
const Segment SGI27012[] = {{&S29194[44], 17}, {NULL}};
const Segment SGI27013[] = {{&S29194[30], 6}, {NULL}};
const Segment SGI27014[] = {{&S29194[109], 3}, {NULL}};
const Segment SGI27015[] = {{&S29194[75], 5}, {NULL}};
const Segment SGI27016[] = {{&S29197[44], 6}, {NULL}};
const Segment SGI27017[] = {{&S29197[44], 18}, {NULL}};
const Segment SGI27018[] = {{&S29197[30], 6}, {NULL}};
const Segment SGI27019[] = {{&S29197[110], 3}, {NULL}};
const Segment SGI27020[] = {{&S29197[76], 5}, {NULL}};
const Segment SGI27021[] = {{&S29199[44], 6}, {NULL}};
const Segment SGI27022[] = {{&S29199[44], 18}, {NULL}};
const Segment SGI27023[] = {{&S29199[30], 6}, {NULL}};
const Segment SGI27024[] = {{&S29199[110], 3}, {NULL}};
const Segment SGI27025[] = {{&S29199[76], 5}, {NULL}};
const Segment SGI27026[] = {{&S29200[44], 6}, {NULL}};
const Segment SGI27027[] = {{&S29200[44], 18}, {NULL}};
const Segment SGI27028[] = {{&S29200[30], 6}, {NULL}};
const Segment SGI27029[] = {{&S29200[110], 3}, {NULL}};
const Segment SGI27030[] = {{&S29200[76], 5}, {NULL}};
const Segment SGI27031[] = {{&S29201[44], 7}, {NULL}};
const Segment SGI27032[] = {{&S29201[44], 19}, {NULL}};
const Segment SGI27033[] = {{&S29201[30], 6}, {NULL}};
const Segment SGI27034[] = {{&S29201[111], 3}, {NULL}};
const Segment SGI27035[] = {{&S29201[77], 5}, {NULL}};
const Segment SGI27036[] = {{&S29203[44], 6}, {NULL}};
const Segment SGI27037[] = {{&S29203[44], 18}, {NULL}};
const Segment SGI27038[] = {{&S29203[30], 6}, {NULL}};
const Segment SGI27039[] = {{&S29203[110], 3}, {NULL}};
const Segment SGI27040[] = {{&S29203[76], 5}, {NULL}};
const Segment SGI27041[] = {{&S29204[44], 9}, {NULL}};
const Segment SGI27042[] = {{&S29204[44], 21}, {NULL}};
const Segment SGI27043[] = {{&S29204[30], 6}, {NULL}};
const Segment SGI27044[] = {{&S29204[113], 3}, {NULL}};
const Segment SGI27045[] = {{&S29204[79], 5}, {NULL}};
const Segment SGI27046[] = {{&S29205[44], 6}, {NULL}};
const Segment SGI27047[] = {{&S29205[44], 18}, {NULL}};
const Segment SGI27048[] = {{&S29205[30], 6}, {NULL}};
const Segment SGI27049[] = {{&S29205[110], 3}, {NULL}};
const Segment SGI27050[] = {{&S29205[76], 5}, {NULL}};
const Segment SGI27051[] = {{&S29208[44], 8}, {NULL}};
const Segment SGI27052[] = {{&S29208[44], 47}, {NULL}};
const Segment SGI27053[] = {{&S29208[30], 6}, {NULL}};
const Segment SGI27054[] = {{&S29208[139], 3}, {NULL}};
const Segment SGI27055[] = {{&S29208[105], 5}, {NULL}};
const Segment SGI27056[] = {{&S29210[44], 12}, {NULL}};
const Segment SGI27057[] = {{&S29210[44], 35}, {NULL}};
const Segment SGI27058[] = {{&S29210[30], 6}, {NULL}};
const Segment SGI27059[] = {{&S29210[127], 3}, {NULL}};
const Segment SGI27060[] = {{&S29210[93], 5}, {NULL}};
const Segment SGI27061[] = {{&S29211[44], 12}, {NULL}};
const Segment SGI27062[] = {{&S29211[44], 24}, {NULL}};
const Segment SGI27063[] = {{&S29211[30], 6}, {NULL}};
const Segment SGI27064[] = {{&S29211[116], 3}, {NULL}};
const Segment SGI27065[] = {{&S29211[82], 5}, {NULL}};
const Segment SGI27066[] = {{&S29212[44], 7}, {NULL}};
const Segment SGI27067[] = {{&S29212[44], 28}, {NULL}};
const Segment SGI27068[] = {{&S29212[30], 6}, {NULL}};
const Segment SGI27069[] = {{&S29212[120], 3}, {NULL}};
const Segment SGI27070[] = {{&S29212[86], 5}, {NULL}};
const Segment SGI27071[] = {{&S29213[44], 6}, {NULL}};
const Segment SGI27072[] = {{&S29213[44], 18}, {NULL}};
const Segment SGI27073[] = {{&S29213[30], 6}, {NULL}};
const Segment SGI27074[] = {{&S29213[110], 3}, {NULL}};
const Segment SGI27075[] = {{&S29213[76], 5}, {NULL}};
const Segment SGI27076[] = {{&S29214[44], 34}, {NULL}};
const Segment SGI27077[] = {{&S29214[30], 6}, {NULL}};
const Segment SGI27078[] = {{&S29214[136], 3}, {NULL}};
const Segment SGI27079[] = {{&S29214[101], 6}, {NULL}};
const Segment SGI27080[] = {{&S29217[44], 6}, {NULL}};
const Segment SGI27081[] = {{&S29217[44], 18}, {NULL}};
const Segment SGI27082[] = {{&S29217[30], 6}, {NULL}};
const Segment SGI27083[] = {{&S29217[110], 3}, {NULL}};
const Segment SGI27084[] = {{&S29217[76], 5}, {NULL}};
const Segment SGI27085[] = {{&S29218[44], 18}, {NULL}};
const Segment SGI27086[] = {{&S29218[44], 30}, {NULL}};
const Segment SGI27087[] = {{&S29218[30], 6}, {NULL}};
const Segment SGI27088[] = {{&S29218[122], 3}, {NULL}};
const Segment SGI27089[] = {{&S29218[88], 5}, {NULL}};
const Segment SGI27090[] = {{&S29231[44], 9}, {NULL}};
const Segment SGI27091[] = {{&S29231[44], 20}, {NULL}};
const Segment SGI27092[] = {{&S29231[30], 6}, {NULL}};
const Segment SGI27093[] = {{&S29231[112], 3}, {NULL}};
const Segment SGI27094[] = {{&S29231[78], 5}, {NULL}};
const Segment SGI27095[] = {{&S29240[44], 7}, {NULL}};
const Segment SGI27096[] = {{&S29240[44], 30}, {NULL}};
const Segment SGI27097[] = {{&S29240[30], 6}, {NULL}};
const Segment SGI27098[] = {{&S29240[122], 3}, {NULL}};
const Segment SGI27099[] = {{&S29240[88], 5}, {NULL}};
const Segment SGI27100[] = {{&S29245[44], 9}, {NULL}};
const Segment SGI27101[] = {{&S29245[44], 21}, {NULL}};
const Segment SGI27102[] = {{&S29245[30], 6}, {NULL}};
const Segment SGI27103[] = {{&S29245[113], 3}, {NULL}};
const Segment SGI27104[] = {{&S29245[79], 5}, {NULL}};
const Segment SGI27105[] = {{&S29248[44], 4}, {NULL}};
const Segment SGI27106[] = {{&S29248[44], 16}, {NULL}};
const Segment SGI27107[] = {{&S29248[30], 6}, {NULL}};
const Segment SGI27108[] = {{&S29248[108], 3}, {NULL}};
const Segment SGI27109[] = {{&S29248[74], 5}, {NULL}};
const Segment SGI27110[] = {{&S29249[44], 9}, {NULL}};
const Segment SGI27111[] = {{&S29249[44], 21}, {NULL}};
const Segment SGI27112[] = {{&S29249[30], 6}, {NULL}};
const Segment SGI27113[] = {{&S29249[113], 3}, {NULL}};
const Segment SGI27114[] = {{&S29249[79], 5}, {NULL}};
const Segment SGI27115[] = {{&S29252[44], 9}, {NULL}};
const Segment SGI27116[] = {{&S29252[44], 28}, {NULL}};
const Segment SGI27117[] = {{&S29252[30], 6}, {NULL}};
const Segment SGI27118[] = {{&S29252[120], 3}, {NULL}};
const Segment SGI27119[] = {{&S29252[86], 5}, {NULL}};
const Segment SGI27120[] = {{&S29261[44], 10}, {NULL}};
const Segment SGI27121[] = {{&S29261[44], 30}, {NULL}};
const Segment SGI27122[] = {{&S29261[30], 6}, {NULL}};
const Segment SGI27123[] = {{&S29261[122], 3}, {NULL}};
const Segment SGI27124[] = {{&S29261[88], 5}, {NULL}};
const Segment SGI27125[] = {{&S29267[44], 12}, {NULL}};
const Segment SGI27126[] = {{&S29267[44], 23}, {NULL}};
const Segment SGI27127[] = {{&S29267[30], 6}, {NULL}};
const Segment SGI27128[] = {{&S29267[84], 4}, {NULL}};
const Segment SGI27129[] = {{&S29267[84], 3}, {NULL}};
const Segment SGI27130[] = {{&S29271[44], 9}, {NULL}};
const Segment SGI27131[] = {{&S29271[44], 21}, {NULL}};
const Segment SGI27132[] = {{&S29271[30], 6}, {NULL}};
const Segment SGI27133[] = {{&S29271[113], 3}, {NULL}};
const Segment SGI27134[] = {{&S29271[79], 5}, {NULL}};
const Segment SGI27135[] = {{&S29273[44], 5}, {NULL}};
const Segment SGI27136[] = {{&S29273[44], 24}, {NULL}};
const Segment SGI27137[] = {{&S29273[30], 6}, {NULL}};
const Segment SGI27138[] = {{&S29273[116], 3}, {NULL}};
const Segment SGI27139[] = {{&S29273[82], 5}, {NULL}};
const Segment SGI27140[] = {{&S29276[44], 6}, {NULL}};
const Segment SGI27141[] = {{&S29276[44], 18}, {NULL}};
const Segment SGI27142[] = {{&S29276[30], 6}, {NULL}};
const Segment SGI27143[] = {{&S29276[110], 3}, {NULL}};
const Segment SGI27144[] = {{&S29276[76], 5}, {NULL}};
const Segment SGI27145[] = {{&S29280[44], 15}, {NULL}};
const Segment SGI27146[] = {{&S29280[44], 27}, {NULL}};
const Segment SGI27147[] = {{&S29280[30], 6}, {NULL}};
const Segment SGI27148[] = {{&S29280[88], 4}, {NULL}};
const Segment SGI27149[] = {{&S29280[88], 3}, {NULL}};
const Segment SGI27150[] = {{&S29281[44], 16}, {NULL}};
const Segment SGI27151[] = {{&S29281[44], 56}, {NULL}};
const Segment SGI27152[] = {{&S29281[30], 6}, {NULL}};
const Segment SGI27153[] = {{&S29281[148], 3}, {NULL}};
const Segment SGI27154[] = {{&S29281[114], 5}, {NULL}};
const Segment SGI27155[] = {{&S29283[44], 10}, {NULL}};
const Segment SGI27156[] = {{&S29283[44], 22}, {NULL}};
const Segment SGI27157[] = {{&S29283[30], 6}, {NULL}};
const Segment SGI27158[] = {{&S29283[114], 3}, {NULL}};
const Segment SGI27159[] = {{&S29283[80], 5}, {NULL}};
const Segment SGI27160[] = {{&S29285[44], 14}, {NULL}};
const Segment SGI27161[] = {{&S29285[44], 33}, {NULL}};
const Segment SGI27162[] = {{&S29285[30], 6}, {NULL}};
const Segment SGI27163[] = {{&S29285[125], 3}, {NULL}};
const Segment SGI27164[] = {{&S29285[91], 5}, {NULL}};
const Segment SGI27165[] = {{&S29289[44], 12}, {NULL}};
const Segment SGI27166[] = {{&S29289[44], 24}, {NULL}};
const Segment SGI27167[] = {{&S29289[30], 6}, {NULL}};
const Segment SGI27168[] = {{&S29289[116], 3}, {NULL}};
const Segment SGI27169[] = {{&S29289[82], 5}, {NULL}};
const Segment SGI27170[] = {{&S29290[44], 12}, {NULL}};
const Segment SGI27171[] = {{&S29290[44], 24}, {NULL}};
const Segment SGI27172[] = {{&S29290[30], 6}, {NULL}};
const Segment SGI27173[] = {{&S29290[116], 3}, {NULL}};
const Segment SGI27174[] = {{&S29290[82], 5}, {NULL}};
const Segment SGI27175[] = {{&S29292[44], 8}, {NULL}};
const Segment SGI27176[] = {{&S29292[44], 19}, {NULL}};
const Segment SGI27177[] = {{&S29292[30], 6}, {NULL}};
const Segment SGI27178[] = {{&S29292[111], 3}, {NULL}};
const Segment SGI27179[] = {{&S29292[77], 5}, {NULL}};
const Segment SGI27180[] = {{&S29293[44], 14}, {NULL}};
const Segment SGI27181[] = {{&S29293[44], 26}, {NULL}};
const Segment SGI27182[] = {{&S29293[30], 6}, {NULL}};
const Segment SGI27183[] = {{&S29293[118], 3}, {NULL}};
const Segment SGI27184[] = {{&S29293[84], 5}, {NULL}};
const Segment SGI27185[] = {{&S29295[44], 12}, {NULL}};
const Segment SGI27186[] = {{&S29295[44], 24}, {NULL}};
const Segment SGI27187[] = {{&S29295[30], 6}, {NULL}};
const Segment SGI27188[] = {{&S29295[116], 3}, {NULL}};
const Segment SGI27189[] = {{&S29295[82], 5}, {NULL}};
const Segment SGI27190[] = {{&S29297[44], 10}, {NULL}};
const Segment SGI27191[] = {{&S29297[44], 29}, {NULL}};
const Segment SGI27192[] = {{&S29297[30], 6}, {NULL}};
const Segment SGI27193[] = {{&S29297[120], 3}, {NULL}};
const Segment SGI27194[] = {{&S29297[86], 5}, {NULL}};
const Segment SGI27195[] = {{&S29302[44], 8}, {NULL}};
const Segment SGI27196[] = {{&S29302[44], 20}, {NULL}};
const Segment SGI27197[] = {{&S29302[30], 6}, {NULL}};
const Segment SGI27198[] = {{&S29302[112], 3}, {NULL}};
const Segment SGI27199[] = {{&S29302[78], 5}, {NULL}};
const Segment SGI27200[] = {{&S29303[44], 12}, {NULL}};
const Segment SGI27201[] = {{&S29303[44], 30}, {NULL}};
const Segment SGI27202[] = {{&S29303[30], 6}, {NULL}};
const Segment SGI27203[] = {{&S18128[44], 10}, {NULL}};
const Segment SGI27204[] = {{&S18128[44], 27}, {NULL}};
const Segment SGI27205[] = {{&S29303[122], 3}, {NULL}};
const Segment SGI27206[] = {{&S18128[30], 6}, {NULL}};
const Segment SGI27207[] = {{&S29303[88], 5}, {NULL}};
const Segment SGI27208[] = {{&S18128[119], 3}, {NULL}};
const Segment SGI27209[] = {{&S18128[85], 5}, {NULL}};
const Segment SGI27210[] = {{&S29305[44], 6}, {NULL}};
const Segment SGI27211[] = {{&S29305[44], 18}, {NULL}};
const Segment SGI27212[] = {{&S29305[30], 6}, {NULL}};
const Segment SGI27213[] = {{&S29305[110], 3}, {NULL}};
const Segment SGI27214[] = {{&S29305[76], 5}, {NULL}};
const Segment SGI27215[] = {{&S29306[47], 10}, {NULL}};
const Segment SGI27216[] = {{&S29306[47], 35}, {NULL}};
const Segment SGI27217[] = {{&S29306[33], 6}, {NULL}};
const Segment SGI27218[] = {{&S29306[130], 3}, {NULL}};
const Segment SGI27219[] = {{&S29306[96], 5}, {NULL}};
const Segment SGI27220[] = {{&S29310[44], 7}, {NULL}};
const Segment SGI27221[] = {{&S29310[44], 30}, {NULL}};
const Segment SGI27222[] = {{&S29310[30], 6}, {NULL}};
const Segment SGI27223[] = {{&S29310[91], 4}, {NULL}};
const Segment SGI27224[] = {{&S29310[91], 3}, {NULL}};
const Segment SGI27225[] = {{&S29311[44], 9}, {NULL}};
const Segment SGI27226[] = {{&S29311[44], 21}, {NULL}};
const Segment SGI27227[] = {{&S29311[30], 6}, {NULL}};
const Segment SGI27228[] = {{&S29311[113], 3}, {NULL}};
const Segment SGI27229[] = {{&S29311[79], 5}, {NULL}};
const Segment SGI27230[] = {{&S29313[44], 11}, {NULL}};
const Segment SGI27231[] = {{&S29313[44], 23}, {NULL}};
const Segment SGI27232[] = {{&S29313[30], 6}, {NULL}};
const Segment SGI27233[] = {{&S29313[115], 3}, {NULL}};
const Segment SGI27234[] = {{&S29313[81], 5}, {NULL}};
const Segment SGI27235[] = {{&S29314[44], 8}, {NULL}};
const Segment SGI27236[] = {{&S29314[44], 47}, {NULL}};
const Segment SGI27237[] = {{&S29314[30], 6}, {NULL}};
const Segment SGI27238[] = {{&S29314[139], 3}, {NULL}};
const Segment SGI27239[] = {{&S29314[105], 5}, {NULL}};
const Segment SGI27240[] = {{&S29315[44], 8}, {NULL}};
const Segment SGI27241[] = {{&S29315[44], 19}, {NULL}};
const Segment SGI27242[] = {{&S29315[30], 6}, {NULL}};
const Segment SGI27243[] = {{&S29315[111], 3}, {NULL}};
const Segment SGI27244[] = {{&S29315[77], 5}, {NULL}};
const Segment SGI27245[] = {{&S29317[44], 32}, {NULL}};
const Segment SGI27246[] = {{&S29317[30], 6}, {NULL}};
const Segment SGI27247[] = {{&S29317[142], 3}, {NULL}};
const Segment SGI27248[] = {{&S29317[107], 6}, {NULL}};
const Segment SGI27249[] = {{&S29323[44], 15}, {NULL}};
const Segment SGI27250[] = {{&S29323[44], 27}, {NULL}};
const Segment SGI27251[] = {{&S29323[30], 6}, {NULL}};
const Segment SGI27252[] = {{&S29323[119], 3}, {NULL}};
const Segment SGI27253[] = {{&S29323[85], 5}, {NULL}};
const Segment SGI27254[] = {{&S29324[44], 32}, {NULL}};
const Segment SGI27255[] = {{&S29324[30], 6}, {NULL}};
const Segment SGI27256[] = {{&S29324[142], 3}, {NULL}};
const Segment SGI27257[] = {{&S29324[107], 6}, {NULL}};
const Segment SGI27258[] = {{&S29325[44], 8}, {NULL}};
const Segment SGI27259[] = {{&S29325[44], 20}, {NULL}};
const Segment SGI27260[] = {{&S29325[30], 6}, {NULL}};
const Segment SGI27261[] = {{&S29325[81], 6}, {NULL}};
const Segment SGI27262[] = {{&S29326[44], 12}, {NULL}};
const Segment SGI27263[] = {{&S29326[44], 32}, {NULL}};
const Segment SGI27264[] = {{&S29326[30], 6}, {NULL}};
const Segment SGI27265[] = {{&S29326[124], 3}, {NULL}};
const Segment SGI27266[] = {{&S29326[90], 5}, {NULL}};
const Segment SGI27267[] = {{&S29327[44], 7}, {NULL}};
const Segment SGI27268[] = {{&S29327[44], 19}, {NULL}};
const Segment SGI27269[] = {{&S29327[30], 6}, {NULL}};
const Segment SGI27270[] = {{&S29327[111], 3}, {NULL}};
const Segment SGI27271[] = {{&S29327[77], 5}, {NULL}};
const Segment SGI27272[] = {{&S29329[44], 7}, {NULL}};
const Segment SGI27273[] = {{&S29329[44], 24}, {NULL}};
const Segment SGI27274[] = {{&S29329[30], 6}, {NULL}};
const Segment SGI27275[] = {{&S29329[116], 3}, {NULL}};
const Segment SGI27276[] = {{&S29329[82], 5}, {NULL}};
const Segment SGI27277[] = {{&S29330[44], 5}, {NULL}};
const Segment SGI27278[] = {{&S29330[44], 17}, {NULL}};
const Segment SGI27279[] = {{&S29330[30], 6}, {NULL}};
const Segment SGI27280[] = {{&S29330[78], 6}, {NULL}};
const Segment SGI27281[] = {{&S29331[44], 6}, {NULL}};
const Segment SGI27282[] = {{&S29331[44], 36}, {NULL}};
const Segment SGI27283[] = {{&S29331[30], 6}, {NULL}};
const Segment SGI27284[] = {{&S29331[76], 3}, {NULL}};
const Segment SGI27285[] = {{&S29331[94], 5}, {NULL}};
const Segment SGI27286[] = {{&S29336[44], 12}, {NULL}};
const Segment SGI27287[] = {{&S29336[44], 35}, {NULL}};
const Segment SGI27288[] = {{&S29336[30], 6}, {NULL}};
const Segment SGI27289[] = {{&S29336[127], 3}, {NULL}};
const Segment SGI27290[] = {{&S29336[93], 5}, {NULL}};
const Segment SGI27291[] = {{&S29340[44], 15}, {NULL}};
const Segment SGI27292[] = {{&S29340[44], 27}, {NULL}};
const Segment SGI27293[] = {{&S29340[30], 6}, {NULL}};
const Segment SGI27294[] = {{&S29340[88], 6}, {NULL}};
const Segment SGI27295[] = {{&S29341[44], 19}, {NULL}};
const Segment SGI27296[] = {{&S29341[44], 31}, {NULL}};
const Segment SGI27297[] = {{&S29341[30], 6}, {NULL}};
const Segment SGI27298[] = {{&S29341[123], 3}, {NULL}};
const Segment SGI27299[] = {{&S29341[89], 5}, {NULL}};
const Segment SGI27300[] = {{&S29342[44], 6}, {NULL}};
const Segment SGI27301[] = {{&S29342[44], 18}, {NULL}};
const Segment SGI27302[] = {{&S29342[30], 6}, {NULL}};
const Segment SGI27303[] = {{&S29342[110], 3}, {NULL}};
const Segment SGI27304[] = {{&S29342[76], 5}, {NULL}};
const Segment SGI27305[] = {{&S24450[44], 10}, {NULL}};
const Segment SGI27306[] = {{&S24450[44], 27}, {NULL}};
const Segment SGI27307[] = {{&S24450[30], 6}, {NULL}};
const Segment SGI27308[] = {{&S24450[119], 3}, {NULL}};
const Segment SGI27309[] = {{&S24450[85], 5}, {NULL}};
const Segment SGI27310[] = {{&S29346[44], 7}, {NULL}};
const Segment SGI27311[] = {{&S29346[44], 30}, {NULL}};
const Segment SGI27312[] = {{&S29346[30], 6}, {NULL}};
const Segment SGI27313[] = {{&S29346[91], 4}, {NULL}};
const Segment SGI27314[] = {{&S29346[91], 3}, {NULL}};
const Segment SGI27315[] = {{&S18142[38], 10}, {NULL}};
const Segment SGI27316[] = {{&S18142[31], 6}, {NULL}};
const Segment SGI27317[] = {{&S29348[44], 5}, {NULL}};
const Segment SGI27318[] = {{&S29348[44], 17}, {NULL}};
const Segment SGI27319[] = {{&S29348[30], 6}, {NULL}};
const Segment SGI27320[] = {{&S29348[78], 4}, {NULL}};
const Segment SGI27321[] = {{&S29348[78], 3}, {NULL}};
const Segment SGI27322[] = {{&S29349[44], 33}, {NULL}};
const Segment SGI27323[] = {{&S29349[30], 6}, {NULL}};
const Segment SGI27324[] = {{&S29349[135], 3}, {NULL}};
const Segment SGI27325[] = {{&S29349[100], 6}, {NULL}};
const Segment SGI27326[] = {{&S24459[38], 10}, {NULL}};
const Segment SGI27327[] = {{&S29351[44], 8}, {NULL}};
const Segment SGI27328[] = {{&S24459[31], 6}, {NULL}};
const Segment SGI27329[] = {{&S29351[44], 20}, {NULL}};
const Segment SGI27330[] = {{&S29351[30], 6}, {NULL}};
const Segment SGI27331[] = {{&S29351[112], 3}, {NULL}};
const Segment SGI27332[] = {{&S29351[78], 5}, {NULL}};
const Segment SGI27333[] = {{&S18149[44], 10}, {NULL}};
const Segment SGI27334[] = {{&S18149[44], 27}, {NULL}};
const Segment SGI27335[] = {{&S18149[30], 6}, {NULL}};
const Segment SGI27336[] = {{&S18149[88], 4}, {NULL}};
const Segment SGI27337[] = {{&S18149[88], 3}, {NULL}};
const Segment SGI27338[] = {{&S29357[44], 37}, {NULL}};
const Segment SGI27339[] = {{&S29357[30], 6}, {NULL}};
const Segment SGI27340[] = {{&S29357[113], 5}, {NULL}};
const Segment SGI27341[] = {{&S18155[44], 10}, {NULL}};
const Segment SGI27342[] = {{&S29358[44], 12}, {NULL}};
const Segment SGI27343[] = {{&S18155[44], 27}, {NULL}};
const Segment SGI27344[] = {{&S18155[30], 6}, {NULL}};
const Segment SGI27345[] = {{&S29358[44], 24}, {NULL}};
const Segment SGI27346[] = {{&S29358[30], 6}, {NULL}};
const Segment SGI27347[] = {{&S29358[116], 3}, {NULL}};
const Segment SGI27348[] = {{&S18155[88], 6}, {NULL}};
const Segment SGI27349[] = {{&S29358[82], 5}, {NULL}};
const Segment SGI27350[] = {{&S29359[44], 9}, {NULL}};
const Segment SGI27351[] = {{&S29359[44], 20}, {NULL}};
const Segment SGI27352[] = {{&S29359[30], 6}, {NULL}};
const Segment SGI27353[] = {{&S29359[112], 3}, {NULL}};
const Segment SGI27354[] = {{&S29359[78], 5}, {NULL}};
const Segment SGI27355[] = {{&S29360[44], 13}, {NULL}};
const Segment SGI27356[] = {{&S29360[44], 25}, {NULL}};
const Segment SGI27357[] = {{&S29360[30], 6}, {NULL}};
const Segment SGI27358[] = {{&S29360[117], 3}, {NULL}};
const Segment SGI27359[] = {{&S29360[83], 5}, {NULL}};
const Segment SGI27360[] = {{&S29363[44], 6}, {NULL}};
const Segment SGI27361[] = {{&S29363[44], 25}, {NULL}};
const Segment SGI27362[] = {{&S29363[30], 6}, {NULL}};
const Segment SGI27363[] = {{&S29363[117], 3}, {NULL}};
const Segment SGI27364[] = {{&S29363[83], 5}, {NULL}};
const Segment SGI27365[] = {{&S18164[44], 27}, {NULL}};
const Segment SGI27366[] = {{&S18164[30], 6}, {NULL}};
const Segment SGI27367[] = {{&S29367[44], 7}, {NULL}};
const Segment SGI27368[] = {{&S29367[44], 30}, {NULL}};
const Segment SGI27369[] = {{&S29367[30], 6}, {NULL}};
const Segment SGI27370[] = {{&S29367[91], 6}, {NULL}};
const Segment SGI27371[] = {{&S18169[42], 10}, {NULL}};
const Segment SGI27372[] = {{&S18169[30], 6}, {NULL}};
const Segment SGI27373[] = {{&S18169[117], 3}, {NULL}};
const Segment SGI27374[] = {{&S18169[83], 5}, {NULL}};
const Segment SGI27375[] = {{&S29371[44], 7}, {NULL}};
const Segment SGI27376[] = {{&S29371[44], 19}, {NULL}};
const Segment SGI27377[] = {{&S29371[30], 6}, {NULL}};
const Segment SGI27378[] = {{&S29371[111], 3}, {NULL}};
const Segment SGI27379[] = {{&S29371[77], 5}, {NULL}};
const Segment SGI27380[] = {{&S29372[44], 7}, {NULL}};
const Segment SGI27381[] = {{&S29372[44], 19}, {NULL}};
const Segment SGI27382[] = {{&S29372[30], 6}, {NULL}};
const Segment SGI27383[] = {{&S29372[111], 3}, {NULL}};
const Segment SGI27384[] = {{&S29372[77], 5}, {NULL}};
const Segment SGI27385[] = {{&S29373[44], 6}, {NULL}};
const Segment SGI27386[] = {{&S29373[44], 25}, {NULL}};
const Segment SGI27387[] = {{&S29373[30], 6}, {NULL}};
const Segment SGI27388[] = {{&S29373[117], 3}, {NULL}};
const Segment SGI27389[] = {{&S29373[83], 5}, {NULL}};
const Segment SGI27390[] = {{&S29374[44], 7}, {NULL}};
const Segment SGI27391[] = {{&S29374[44], 19}, {NULL}};
const Segment SGI27392[] = {{&S29374[30], 6}, {NULL}};
const Segment SGI27393[] = {{&S29374[111], 3}, {NULL}};
const Segment SGI27394[] = {{&S29374[77], 5}, {NULL}};
const Segment SGI27395[] = {{&S29376[44], 8}, {NULL}};
const Segment SGI27396[] = {{&S29376[44], 19}, {NULL}};
const Segment SGI27397[] = {{&S29376[30], 6}, {NULL}};
const Segment SGI27398[] = {{&S29376[111], 3}, {NULL}};
const Segment SGI27399[] = {{&S29376[77], 5}, {NULL}};
const Segment SGI27400[] = {{&S29377[44], 11}, {NULL}};
const Segment SGI27401[] = {{&S29377[44], 20}, {NULL}};
const Segment SGI27402[] = {{&S29377[30], 6}, {NULL}};
const Segment SGI27403[] = {{&S29377[112], 3}, {NULL}};
const Segment SGI27404[] = {{&S29377[78], 5}, {NULL}};
const Segment SGI27405[] = {{&S29382[44], 5}, {NULL}};
const Segment SGI27406[] = {{&S29382[44], 16}, {NULL}};
const Segment SGI27407[] = {{&S29382[30], 6}, {NULL}};
const Segment SGI27408[] = {{&S29382[108], 3}, {NULL}};
const Segment SGI27409[] = {{&S29382[74], 5}, {NULL}};
const Segment SGI27410[] = {{&S29383[44], 6}, {NULL}};
const Segment SGI27411[] = {{&S29383[44], 17}, {NULL}};
const Segment SGI27412[] = {{&S29383[30], 6}, {NULL}};
const Segment SGI27413[] = {{&S29383[109], 3}, {NULL}};
const Segment SGI27414[] = {{&S29383[75], 5}, {NULL}};
const Segment SGI27415[] = {{&S29387[44], 6}, {NULL}};
const Segment SGI27416[] = {{&S29387[44], 18}, {NULL}};
const Segment SGI27417[] = {{&S29387[30], 6}, {NULL}};
const Segment SGI27418[] = {{&S29387[110], 3}, {NULL}};
const Segment SGI27419[] = {{&S29387[76], 5}, {NULL}};
const Segment SGI27420[] = {{&S29389[44], 10}, {NULL}};
const Segment SGI27421[] = {{&S29389[44], 29}, {NULL}};
const Segment SGI27422[] = {{&S29389[30], 6}, {NULL}};
const Segment SGI27423[] = {{&S29389[121], 3}, {NULL}};
const Segment SGI27424[] = {{&S29389[87], 5}, {NULL}};
const Segment SGI27425[] = {{&S29392[44], 15}, {NULL}};
const Segment SGI27426[] = {{&S29392[44], 27}, {NULL}};
const Segment SGI27427[] = {{&S29392[30], 6}, {NULL}};
const Segment SGI27428[] = {{&S29392[119], 3}, {NULL}};
const Segment SGI27429[] = {{&S29392[85], 5}, {NULL}};
const Segment SGI27430[] = {{&S29394[44], 12}, {NULL}};
const Segment SGI27431[] = {{&S29394[44], 36}, {NULL}};
const Segment SGI27432[] = {{&S29394[30], 6}, {NULL}};
const Segment SGI27433[] = {{&S29394[128], 3}, {NULL}};
const Segment SGI27434[] = {{&S29394[94], 5}, {NULL}};
const Segment SGI27435[] = {{&S29396[44], 23}, {NULL}};
const Segment SGI27436[] = {{&S29396[44], 32}, {NULL}};
const Segment SGI27437[] = {{&S29396[30], 6}, {NULL}};
const Segment SGI27438[] = {{&S29396[124], 3}, {NULL}};
const Segment SGI27439[] = {{&S29396[90], 5}, {NULL}};
const Segment SGI27440[] = {{&S29398[44], 8}, {NULL}};
const Segment SGI27441[] = {{&S29398[44], 20}, {NULL}};
const Segment SGI27442[] = {{&S29398[30], 6}, {NULL}};
const Segment SGI27443[] = {{&S29398[111], 3}, {NULL}};
const Segment SGI27444[] = {{&S29398[78], 5}, {NULL}};
const Segment SGI27445[] = {{&S29399[44], 9}, {NULL}};
const Segment SGI27446[] = {{&S29399[44], 21}, {NULL}};
const Segment SGI27447[] = {{&S29399[30], 6}, {NULL}};
const Segment SGI27448[] = {{&S29399[112], 3}, {NULL}};
const Segment SGI27449[] = {{&S29399[79], 5}, {NULL}};
const Segment SGI27450[] = {{&S29400[44], 24}, {NULL}};
const Segment SGI27451[] = {{&S29400[44], 45}, {NULL}};
const Segment SGI27452[] = {{&S29400[30], 6}, {NULL}};
const Segment SGI27453[] = {{&S29400[136], 3}, {NULL}};
const Segment SGI27454[] = {{&S29400[103], 5}, {NULL}};
const Segment SGI27455[] = {{&S29403[44], 7}, {NULL}};
const Segment SGI27456[] = {{&S29403[44], 18}, {NULL}};
const Segment SGI27457[] = {{&S29403[30], 6}, {NULL}};
const Segment SGI27458[] = {{&S29403[110], 3}, {NULL}};
const Segment SGI27459[] = {{&S29403[76], 5}, {NULL}};
const Segment SGI27460[] = {{&S29406[44], 34}, {NULL}};
const Segment SGI27461[] = {{&S29406[30], 6}, {NULL}};
const Segment SGI27462[] = {{&S29406[136], 3}, {NULL}};
const Segment SGI27463[] = {{&S29406[101], 6}, {NULL}};
const Segment SGI27464[] = {{&S29407[44], 6}, {NULL}};
const Segment SGI27465[] = {{&S29407[44], 17}, {NULL}};
const Segment SGI27466[] = {{&S29407[30], 6}, {NULL}};
const Segment SGI27467[] = {{&S29407[109], 3}, {NULL}};
const Segment SGI27468[] = {{&S29407[75], 5}, {NULL}};
const Segment SGI27469[] = {{&S29408[44], 10}, {NULL}};
const Segment SGI27470[] = {{&S29408[44], 29}, {NULL}};
const Segment SGI27471[] = {{&S29408[30], 6}, {NULL}};
const Segment SGI27472[] = {{&S29408[120], 3}, {NULL}};
const Segment SGI27473[] = {{&S29408[87], 5}, {NULL}};
const Segment SGI27474[] = {{&S29409[44], 10}, {NULL}};
const Segment SGI27475[] = {{&S29409[44], 29}, {NULL}};
const Segment SGI27476[] = {{&S29409[30], 6}, {NULL}};
const Segment SGI27477[] = {{&S29409[119], 3}, {NULL}};
const Segment SGI27478[] = {{&S29409[86], 5}, {NULL}};
const Segment SGI27479[] = {{&S29410[44], 37}, {NULL}};
const Segment SGI27480[] = {{&S29410[30], 6}, {NULL}};
const Segment SGI27481[] = {{&S29410[113], 5}, {NULL}};
const Segment SGI27482[] = {{&S29411[44], 27}, {NULL}};
const Segment SGI27483[] = {{&S29411[30], 6}, {NULL}};
const Segment SGI27484[] = {{&S29411[137], 3}, {NULL}};
const Segment SGI27485[] = {{&S29411[103], 5}, {NULL}};
const Segment SGI27486[] = {{&S29413[44], 7}, {NULL}};
const Segment SGI27487[] = {{&S29413[44], 19}, {NULL}};
const Segment SGI27488[] = {{&S29413[30], 6}, {NULL}};
const Segment SGI27489[] = {{&S29413[111], 3}, {NULL}};
const Segment SGI27490[] = {{&S29413[77], 5}, {NULL}};
const Segment SGI27491[] = {{&S29415[44], 10}, {NULL}};
const Segment SGI27492[] = {{&S29415[44], 22}, {NULL}};
const Segment SGI27493[] = {{&S29415[30], 6}, {NULL}};
const Segment SGI27494[] = {{&S29415[114], 3}, {NULL}};
const Segment SGI27495[] = {{&S29415[80], 5}, {NULL}};
const Segment SGI27496[] = {{&S29417[44], 12}, {NULL}};
const Segment SGI27497[] = {{&S29417[44], 33}, {NULL}};
const Segment SGI27498[] = {{&S29417[30], 6}, {NULL}};
const Segment SGI27499[] = {{&S29417[125], 3}, {NULL}};
const Segment SGI27500[] = {{&S29417[91], 5}, {NULL}};
const Segment SGI27501[] = {{&S29418[44], 14}, {NULL}};
const Segment SGI27502[] = {{&S29418[44], 26}, {NULL}};
const Segment SGI27503[] = {{&S29418[30], 6}, {NULL}};
const Segment SGI27504[] = {{&S29418[118], 3}, {NULL}};
const Segment SGI27505[] = {{&S29418[84], 5}, {NULL}};
const Segment SGI27506[] = {{&S29421[47], 10}, {NULL}};
const Segment SGI27507[] = {{&S29421[47], 31}, {NULL}};
const Segment SGI27508[] = {{&S29421[33], 6}, {NULL}};
const Segment SGI27509[] = {{&S29421[126], 3}, {NULL}};
const Segment SGI27510[] = {{&S29421[92], 5}, {NULL}};
const Segment SGI27511[] = {{&S29423[44], 15}, {NULL}};
const Segment SGI27512[] = {{&S29423[44], 24}, {NULL}};
const Segment SGI27513[] = {{&S29423[30], 6}, {NULL}};
const Segment SGI27514[] = {{&S29423[116], 3}, {NULL}};
const Segment SGI27515[] = {{&S29423[82], 5}, {NULL}};
const Segment SGI27516[] = {{&S29428[44], 12}, {NULL}};
const Segment SGI27517[] = {{&S29428[44], 24}, {NULL}};
const Segment SGI27518[] = {{&S29428[30], 6}, {NULL}};
const Segment SGI27519[] = {{&S29428[116], 3}, {NULL}};
const Segment SGI27520[] = {{&S29428[82], 5}, {NULL}};
const Segment SGI27521[] = {{&S29431[44], 7}, {NULL}};
const Segment SGI27522[] = {{&S29431[44], 30}, {NULL}};
const Segment SGI27523[] = {{&S29431[30], 6}, {NULL}};
const Segment SGI27524[] = {{&S29431[91], 6}, {NULL}};
const Segment SGI27525[] = {{&S29436[38], 15}, {NULL}};
const Segment SGI27526[] = {{&S29436[31], 6}, {NULL}};
const Segment SGI27527[] = {{&S29439[44], 12}, {NULL}};
const Segment SGI27528[] = {{&S29439[44], 32}, {NULL}};
const Segment SGI27529[] = {{&S29439[30], 6}, {NULL}};
const Segment SGI27530[] = {{&S29439[124], 3}, {NULL}};
const Segment SGI27531[] = {{&S29439[90], 5}, {NULL}};
const Segment SGI27532[] = {{&S29440[44], 8}, {NULL}};
const Segment SGI27533[] = {{&S29440[44], 20}, {NULL}};
const Segment SGI27534[] = {{&S29440[30], 6}, {NULL}};
const Segment SGI27535[] = {{&S29440[112], 3}, {NULL}};
const Segment SGI27536[] = {{&S29440[78], 5}, {NULL}};
const Segment SGI27537[] = {{&S29441[44], 6}, {NULL}};
const Segment SGI27538[] = {{&S29441[44], 25}, {NULL}};
const Segment SGI27539[] = {{&S29441[30], 6}, {NULL}};
const Segment SGI27540[] = {{&S29441[117], 3}, {NULL}};
const Segment SGI27541[] = {{&S29441[83], 5}, {NULL}};
const Segment SGI27542[] = {{&S29443[44], 13}, {NULL}};
const Segment SGI27543[] = {{&S29443[44], 52}, {NULL}};
const Segment SGI27544[] = {{&S29443[30], 6}, {NULL}};
const Segment SGI27545[] = {{&S29443[144], 3}, {NULL}};
const Segment SGI27546[] = {{&S29443[110], 5}, {NULL}};
const Segment SGI27547[] = {{&S29446[44], 6}, {NULL}};
const Segment SGI27548[] = {{&S29446[44], 25}, {NULL}};
const Segment SGI27549[] = {{&S29446[30], 6}, {NULL}};
const Segment SGI27550[] = {{&S29446[117], 3}, {NULL}};
const Segment SGI27551[] = {{&S29446[83], 5}, {NULL}};
const Segment SGI27552[] = {{&S19016[44], 19}, {NULL}};
const Segment SGI27553[] = {{&S19016[44], 38}, {NULL}};
const Segment SGI27554[] = {{&S19016[30], 6}, {NULL}};
const Segment SGI27555[] = {{&S19016[69], 3}, {NULL}};
const Segment SGI27556[] = {{&S19016[96], 5}, {NULL}};
const Segment SGI27557[] = {{&S19018[44], 19}, {NULL}};
const Segment SGI27558[] = {{&S19018[44], 38}, {NULL}};
const Segment SGI27559[] = {{&S19018[30], 6}, {NULL}};
const Segment SGI27560[] = {{&S19018[69], 3}, {NULL}};
const Segment SGI27561[] = {{&S19018[96], 5}, {NULL}};
const Segment SGI27562[] = {{&S19020[44], 19}, {NULL}};
const Segment SGI27563[] = {{&S19020[44], 38}, {NULL}};
const Segment SGI27564[] = {{&S19020[30], 6}, {NULL}};
const Segment SGI27565[] = {{&S19020[69], 3}, {NULL}};
const Segment SGI27566[] = {{&S19020[96], 5}, {NULL}};
const Segment SGI27567[] = {{&S19024[44], 19}, {NULL}};
const Segment SGI27568[] = {{&S19024[44], 38}, {NULL}};
const Segment SGI27569[] = {{&S19024[30], 6}, {NULL}};
const Segment SGI27570[] = {{&S19024[69], 3}, {NULL}};
const Segment SGI27571[] = {{&S19024[96], 5}, {NULL}};
const Segment SGI27572[] = {{&S19026[44], 19}, {NULL}};
const Segment SGI27573[] = {{&S19026[44], 38}, {NULL}};
const Segment SGI27574[] = {{&S19026[30], 6}, {NULL}};
const Segment SGI27575[] = {{&S19026[69], 3}, {NULL}};
const Segment SGI27576[] = {{&S19026[96], 5}, {NULL}};
const Segment SGI27577[] = {{&S19027[44], 19}, {NULL}};
const Segment SGI27578[] = {{&S19027[44], 38}, {NULL}};
const Segment SGI27579[] = {{&S19027[30], 6}, {NULL}};
const Segment SGI27580[] = {{&S19027[69], 3}, {NULL}};
const Segment SGI27581[] = {{&S19027[96], 5}, {NULL}};
const Segment SGI27582[] = {{&S19028[44], 19}, {NULL}};
const Segment SGI27583[] = {{&S19028[44], 38}, {NULL}};
const Segment SGI27584[] = {{&S19028[30], 6}, {NULL}};
const Segment SGI27585[] = {{&S19028[69], 3}, {NULL}};
const Segment SGI27586[] = {{&S19028[96], 5}, {NULL}};
const Segment SGI27587[] = {{&S19029[44], 19}, {NULL}};
const Segment SGI27588[] = {{&S19029[44], 36}, {NULL}};
const Segment SGI27589[] = {{&S19029[30], 6}, {NULL}};
const Segment SGI27590[] = {{&S19029[69], 3}, {NULL}};
const Segment SGI27591[] = {{&S19029[94], 5}, {NULL}};
const Segment SGI27592[] = {{&S19045[44], 19}, {NULL}};
const Segment SGI27593[] = {{&S19045[44], 32}, {NULL}};
const Segment SGI27594[] = {{&S19045[30], 6}, {NULL}};
const Segment SGI27595[] = {{&S19045[124], 3}, {NULL}};
const Segment SGI27596[] = {{&S19045[90], 5}, {NULL}};
const Segment SGI27597[] = {{&S19047[44], 19}, {NULL}};
const Segment SGI27598[] = {{&S19047[44], 38}, {NULL}};
const Segment SGI27599[] = {{&S19047[30], 6}, {NULL}};
const Segment SGI27600[] = {{&S19047[69], 3}, {NULL}};
const Segment SGI27601[] = {{&S19047[96], 5}, {NULL}};
const Segment SGI27602[] = {{&S19048[44], 19}, {NULL}};
const Segment SGI27603[] = {{&S19048[44], 38}, {NULL}};
const Segment SGI27604[] = {{&S19048[30], 6}, {NULL}};
const Segment SGI27605[] = {{&S19048[69], 3}, {NULL}};
const Segment SGI27606[] = {{&S19048[96], 5}, {NULL}};
const Segment SGI27607[] = {{&S19049[44], 19}, {NULL}};
const Segment SGI27608[] = {{&S19049[44], 38}, {NULL}};
const Segment SGI27609[] = {{&S19049[30], 6}, {NULL}};
const Segment SGI27610[] = {{&S19049[69], 3}, {NULL}};
const Segment SGI27611[] = {{&S19049[96], 5}, {NULL}};
const Segment SGI27612[] = {{&S19051[44], 7}, {NULL}};
const Segment SGI27613[] = {{&S19051[44], 26}, {NULL}};
const Segment SGI27614[] = {{&S19051[30], 6}, {NULL}};
const Segment SGI27615[] = {{&S19051[57], 3}, {NULL}};
const Segment SGI27616[] = {{&S19051[84], 5}, {NULL}};
const Segment SGI27617[] = {{&S19053[44], 19}, {NULL}};
const Segment SGI27618[] = {{&S19053[44], 36}, {NULL}};
const Segment SGI27619[] = {{&S19053[30], 6}, {NULL}};
const Segment SGI27620[] = {{&S19053[69], 3}, {NULL}};
const Segment SGI27621[] = {{&S19053[94], 5}, {NULL}};
const Segment SGI27622[] = {{&S19055[44], 19}, {NULL}};
const Segment SGI27623[] = {{&S19055[44], 37}, {NULL}};
const Segment SGI27624[] = {{&S19055[30], 6}, {NULL}};
const Segment SGI27625[] = {{&S19055[69], 3}, {NULL}};
const Segment SGI27626[] = {{&S19055[95], 5}, {NULL}};
const Segment SGI27627[] = {{&S19056[44], 19}, {NULL}};
const Segment SGI27628[] = {{&S19056[44], 37}, {NULL}};
const Segment SGI27629[] = {{&S19056[30], 6}, {NULL}};
const Segment SGI27630[] = {{&S19056[69], 3}, {NULL}};
const Segment SGI27631[] = {{&S19056[95], 5}, {NULL}};
const Segment SGI27632[] = {{&S19062[44], 7}, {NULL}};
const Segment SGI27633[] = {{&S19062[44], 25}, {NULL}};
const Segment SGI27634[] = {{&S19062[30], 6}, {NULL}};
const Segment SGI27635[] = {{&S19062[57], 3}, {NULL}};
const Segment SGI27636[] = {{&S19062[83], 5}, {NULL}};
const Segment SGI27637[] = {{&S19063[44], 21}, {NULL}};
const Segment SGI27638[] = {{&S19063[44], 39}, {NULL}};
const Segment SGI27639[] = {{&S19063[30], 6}, {NULL}};
const Segment SGI27640[] = {{&S19063[71], 3}, {NULL}};
const Segment SGI27641[] = {{&S19063[97], 5}, {NULL}};
const Segment SGI27642[] = {{&S19064[44], 20}, {NULL}};
const Segment SGI27643[] = {{&S19064[44], 38}, {NULL}};
const Segment SGI27644[] = {{&S19064[30], 6}, {NULL}};
const Segment SGI27645[] = {{&S19064[70], 3}, {NULL}};
const Segment SGI27646[] = {{&S19064[96], 5}, {NULL}};
const Segment SGI27647[] = {{&S19065[44], 19}, {NULL}};
const Segment SGI27648[] = {{&S19065[44], 37}, {NULL}};
const Segment SGI27649[] = {{&S19065[30], 6}, {NULL}};
const Segment SGI27650[] = {{&S19065[69], 3}, {NULL}};
const Segment SGI27651[] = {{&S19065[95], 5}, {NULL}};
const Segment SGI27652[] = {{&S19067[44], 19}, {NULL}};
const Segment SGI27653[] = {{&S19067[44], 37}, {NULL}};
const Segment SGI27654[] = {{&S19067[30], 6}, {NULL}};
const Segment SGI27655[] = {{&S19067[69], 3}, {NULL}};
const Segment SGI27656[] = {{&S19067[95], 5}, {NULL}};
const Segment SGI27657[] = {{&S19068[44], 7}, {NULL}};
const Segment SGI27658[] = {{&S19068[44], 25}, {NULL}};
const Segment SGI27659[] = {{&S19068[30], 6}, {NULL}};
const Segment SGI27660[] = {{&S19068[86], 4}, {NULL}};
const Segment SGI27661[] = {{&S19068[86], 3}, {NULL}};
const Segment SGI27662[] = {{&S19069[44], 20}, {NULL}};
const Segment SGI27663[] = {{&S19069[44], 38}, {NULL}};
const Segment SGI27664[] = {{&S19069[30], 6}, {NULL}};
const Segment SGI27665[] = {{&S19069[70], 3}, {NULL}};
const Segment SGI27666[] = {{&S19069[96], 5}, {NULL}};
const Segment SGI27667[] = {{&S19070[44], 21}, {NULL}};
const Segment SGI27668[] = {{&S19070[44], 39}, {NULL}};
const Segment SGI27669[] = {{&S19070[30], 6}, {NULL}};
const Segment SGI27670[] = {{&S19070[71], 3}, {NULL}};
const Segment SGI27671[] = {{&S19070[97], 5}, {NULL}};
const Segment SGI27672[] = {{&S19071[44], 7}, {NULL}};
const Segment SGI27673[] = {{&S19071[44], 25}, {NULL}};
const Segment SGI27674[] = {{&S19071[30], 6}, {NULL}};
const Segment SGI27675[] = {{&S19071[57], 3}, {NULL}};
const Segment SGI27676[] = {{&S19071[83], 5}, {NULL}};
const Segment SGI27677[] = {{&S19072[38], 7}, {NULL}};
const Segment SGI27678[] = {{&S19072[31], 6}, {NULL}};
const Segment SGI27679[] = {{&S19074[44], 7}, {NULL}};
const Segment SGI27680[] = {{&S19074[44], 25}, {NULL}};
const Segment SGI27681[] = {{&S19074[30], 6}, {NULL}};
const Segment SGI27682[] = {{&S19074[57], 3}, {NULL}};
const Segment SGI27683[] = {{&S19074[83], 5}, {NULL}};
const Segment SGI27684[] = {{&S19076[44], 7}, {NULL}};
const Segment SGI27685[] = {{&S19076[44], 25}, {NULL}};
const Segment SGI27686[] = {{&S19076[30], 6}, {NULL}};
const Segment SGI27687[] = {{&S19076[86], 4}, {NULL}};
const Segment SGI27688[] = {{&S19076[86], 3}, {NULL}};
const Segment SGI27689[] = {{&S19077[44], 7}, {NULL}};
const Segment SGI27690[] = {{&S19077[44], 25}, {NULL}};
const Segment SGI27691[] = {{&S19077[30], 6}, {NULL}};
const Segment SGI27692[] = {{&S19077[86], 6}, {NULL}};
const Segment SGI27693[] = {{&S19078[44], 7}, {NULL}};
const Segment SGI27694[] = {{&S19078[44], 25}, {NULL}};
const Segment SGI27695[] = {{&S19078[30], 6}, {NULL}};
const Segment SGI27696[] = {{&S19078[57], 3}, {NULL}};
const Segment SGI27697[] = {{&S19078[83], 5}, {NULL}};
const Segment SGI27698[] = {{&S19080[44], 7}, {NULL}};
const Segment SGI27699[] = {{&S19080[44], 25}, {NULL}};
const Segment SGI27700[] = {{&S19080[30], 6}, {NULL}};
const Segment SGI27701[] = {{&S19080[86], 4}, {NULL}};
const Segment SGI27702[] = {{&S19080[86], 3}, {NULL}};
const Segment SGI27703[] = {{&S19081[44], 7}, {NULL}};
const Segment SGI27704[] = {{&S19081[44], 25}, {NULL}};
const Segment SGI27705[] = {{&S19081[30], 6}, {NULL}};
const Segment SGI27706[] = {{&S19081[86], 6}, {NULL}};
const Segment SGI27707[] = {{&S19082[42], 19}, {NULL}};
const Segment SGI27708[] = {{&S19082[30], 6}, {NULL}};
const Segment SGI27709[] = {{&S19082[67], 3}, {NULL}};
const Segment SGI27710[] = {{&S19082[93], 5}, {NULL}};
const Segment SGI27711[] = {{&S19088[44], 19}, {NULL}};
const Segment SGI27712[] = {{&S19088[44], 37}, {NULL}};
const Segment SGI27713[] = {{&S19088[30], 6}, {NULL}};
const Segment SGI27714[] = {{&S19088[69], 3}, {NULL}};
const Segment SGI27715[] = {{&S19088[95], 5}, {NULL}};
const Segment SGI27716[] = {{&S19089[44], 20}, {NULL}};
const Segment SGI27717[] = {{&S19089[44], 38}, {NULL}};
const Segment SGI27718[] = {{&S19089[30], 6}, {NULL}};
const Segment SGI27719[] = {{&S19089[70], 3}, {NULL}};
const Segment SGI27720[] = {{&S19089[96], 5}, {NULL}};
const Segment SGI27721[] = {{&S24443[42], 10}, {NULL}};
const Segment SGI27722[] = {{&S24443[42], 27}, {NULL}};
const Segment SGI27723[] = {{&S24443[30], 4}, {NULL}};
const Segment SGI27724[] = {{&S24443[117], 3}, {NULL}};
const Segment SGI27725[] = {{&S24443[83], 5}, {NULL}};
const Segment SGI27726[] = {{&S20209[44], 19}, {NULL}};
const Segment SGI27727[] = {{&S20209[44], 36}, {NULL}};
const Segment SGI27728[] = {{&S20209[30], 6}, {NULL}};
const Segment SGI27729[] = {{&S20209[69], 3}, {NULL}};
const Segment SGI27730[] = {{&S20209[94], 5}, {NULL}};
const Segment SGI27731[] = {{&S20212[44], 19}, {NULL}};
const Segment SGI27732[] = {{&S20212[44], 36}, {NULL}};
const Segment SGI27733[] = {{&S20212[30], 6}, {NULL}};
const Segment SGI27734[] = {{&S20212[69], 3}, {NULL}};
const Segment SGI27735[] = {{&S20212[94], 5}, {NULL}};
const Segment SGI27736[] = {{&S20213[44], 19}, {NULL}};
const Segment SGI27737[] = {{&S20213[44], 36}, {NULL}};
const Segment SGI27738[] = {{&S20213[30], 6}, {NULL}};
const Segment SGI27739[] = {{&S20213[69], 3}, {NULL}};
const Segment SGI27740[] = {{&S20213[94], 5}, {NULL}};
const Segment SGI27741[] = {{&S20214[44], 19}, {NULL}};
const Segment SGI27742[] = {{&S20214[44], 36}, {NULL}};
const Segment SGI27743[] = {{&S20214[30], 6}, {NULL}};
const Segment SGI27744[] = {{&S20214[69], 3}, {NULL}};
const Segment SGI27745[] = {{&S20214[94], 5}, {NULL}};
const Segment SGI27746[] = {{&S20215[44], 7}, {NULL}};
const Segment SGI27747[] = {{&S20215[44], 24}, {NULL}};
const Segment SGI27748[] = {{&S20215[30], 6}, {NULL}};
const Segment SGI27749[] = {{&S20215[57], 3}, {NULL}};
const Segment SGI27750[] = {{&S20215[82], 5}, {NULL}};
const Segment SGI27751[] = {{&S20216[44], 20}, {NULL}};
const Segment SGI27752[] = {{&S20216[44], 36}, {NULL}};
const Segment SGI27753[] = {{&S20216[30], 6}, {NULL}};
const Segment SGI27754[] = {{&S20216[70], 3}, {NULL}};
const Segment SGI27755[] = {{&S20216[94], 5}, {NULL}};
const Segment SGI27756[] = {{&S20219[44], 19}, {NULL}};
const Segment SGI27757[] = {{&S20219[44], 36}, {NULL}};
const Segment SGI27758[] = {{&S20219[30], 6}, {NULL}};
const Segment SGI27759[] = {{&S20219[69], 3}, {NULL}};
const Segment SGI27760[] = {{&S20219[94], 5}, {NULL}};
const Segment SGI27761[] = {{&S20223[44], 20}, {NULL}};
const Segment SGI27762[] = {{&S20223[44], 37}, {NULL}};
const Segment SGI27763[] = {{&S20223[30], 6}, {NULL}};
const Segment SGI27764[] = {{&S20223[70], 3}, {NULL}};
const Segment SGI27765[] = {{&S20223[95], 5}, {NULL}};
const Segment SGI27766[] = {{&S20225[44], 20}, {NULL}};
const Segment SGI27767[] = {{&S20225[44], 37}, {NULL}};
const Segment SGI27768[] = {{&S20225[30], 6}, {NULL}};
const Segment SGI27769[] = {{&S20225[70], 3}, {NULL}};
const Segment SGI27770[] = {{&S20225[95], 5}, {NULL}};
const Segment SGI27771[] = {{&S20227[44], 19}, {NULL}};
const Segment SGI27772[] = {{&S20227[44], 36}, {NULL}};
const Segment SGI27773[] = {{&S20227[30], 6}, {NULL}};
const Segment SGI27774[] = {{&S20227[69], 3}, {NULL}};
const Segment SGI27775[] = {{&S20227[94], 5}, {NULL}};
const Segment SGI27776[] = {{&S20241[38], 7}, {NULL}};
const Segment SGI27777[] = {{&S20241[31], 6}, {NULL}};
const Segment SGI27778[] = {{&S20245[44], 7}, {NULL}};
const Segment SGI27779[] = {{&S20245[44], 24}, {NULL}};
const Segment SGI27780[] = {{&S20245[30], 6}, {NULL}};
const Segment SGI27781[] = {{&S20245[57], 3}, {NULL}};
const Segment SGI27782[] = {{&S20245[82], 5}, {NULL}};
const Segment SGI27783[] = {{&S20247[44], 19}, {NULL}};
const Segment SGI27784[] = {{&S20247[44], 36}, {NULL}};
const Segment SGI27785[] = {{&S20247[30], 6}, {NULL}};
const Segment SGI27786[] = {{&S20247[69], 3}, {NULL}};
const Segment SGI27787[] = {{&S20247[94], 5}, {NULL}};
const Segment SGI27788[] = {{&S20251[38], 7}, {NULL}};
const Segment SGI27789[] = {{&S20251[31], 6}, {NULL}};
const Segment SGI27790[] = {{&S20254[44], 7}, {NULL}};
const Segment SGI27791[] = {{&S20254[44], 24}, {NULL}};
const Segment SGI27792[] = {{&S20254[30], 6}, {NULL}};
const Segment SGI27793[] = {{&S20254[85], 4}, {NULL}};
const Segment SGI27794[] = {{&S20254[85], 3}, {NULL}};
const Segment SGI27795[] = {{&S20354[30], 6}, {NULL}};
const Segment SGI27796[] = {{&S20354[91], 3}, {NULL}};
const Segment SGI27797[] = {{&S20354[57], 5}, {NULL}};
const Segment SGI27798[] = {{&S20354[102], 11}, {NULL}};
const Segment SGI27799[] = {{&S20366[37], 7}, {NULL}};
const Segment SGI27800[] = {{&S20366[30], 6}, {NULL}};
const Segment SGI27801[] = {{&S20366[108], 3}, {NULL}};
const Segment SGI27802[] = {{&S20366[74], 5}, {NULL}};
const Segment SGI27803[] = {{&S20368[44], 45}, {NULL}};
const Segment SGI27804[] = {{&S20368[44], 56}, {NULL}};
const Segment SGI27805[] = {{&S20368[30], 6}, {&S20368[52], 48}, {NULL}};
const Segment SGI27806[] = {{&S20368[148], 3}, {NULL}};
const Segment SGI27807[] = {{&S20368[114], 5}, {NULL}};
const Segment SGI27808[] = {{&S20375[44], 10}, {NULL}};
const Segment SGI27809[] = {{&S20375[44], 21}, {NULL}};
const Segment SGI27810[] = {{&S20375[30], 6}, {NULL}};
const Segment SGI27811[] = {{&S20375[113], 3}, {NULL}};
const Segment SGI27812[] = {{&S20375[79], 5}, {NULL}};
const Segment SGI27813[] = {{&S20385[44], 48}, {NULL}};
const Segment SGI27814[] = {{&S20385[44], 59}, {NULL}};
const Segment SGI27815[] = {{&S20385[30], 6}, {&S20385[52], 51}, {NULL}};
const Segment SGI27816[] = {{&S20385[151], 3}, {NULL}};
const Segment SGI27817[] = {{&S20385[117], 5}, {NULL}};
const Segment SGI27818[] = {{&S20401[44], 5}, {NULL}};
const Segment SGI27819[] = {{&S20401[44], 16}, {NULL}};
const Segment SGI27820[] = {{&S20401[30], 6}, {NULL}};
const Segment SGI27821[] = {{&S20401[108], 3}, {NULL}};
const Segment SGI27822[] = {{&S20401[74], 5}, {NULL}};
const Segment SGI27823[] = {{&S20422[44], 6}, {NULL}};
const Segment SGI27824[] = {{&S20422[44], 23}, {NULL}};
const Segment SGI27825[] = {{&S20422[30], 6}, {NULL}};
const Segment SGI27826[] = {{&S20422[115], 3}, {NULL}};
const Segment SGI27827[] = {{&S20422[81], 5}, {NULL}};
const Segment SGI27828[] = {{&S20435[38], 45}, {NULL}};
const Segment SGI27829[] = {{&S20435[31], 6}, {&S20435[46], 48}, {NULL}};
const Segment SGI27830[] = {{&S20438[44], 5}, {NULL}};
const Segment SGI27831[] = {{&S20438[44], 14}, {NULL}};
const Segment SGI27832[] = {{&S20438[30], 6}, {NULL}};
const Segment SGI27833[] = {{&S20438[106], 3}, {NULL}};
const Segment SGI27834[] = {{&S20438[72], 5}, {NULL}};
const Segment SGI27835[] = {{&S20505[30], 6}, {NULL}};
const Segment SGI27836[] = {{&S20505[104], 3}, {NULL}};
const Segment SGI27837[] = {{&S20505[57], 6}, {NULL}};
const Segment SGI27838[] = {{&S20506[38], 37}, {NULL}};
const Segment SGI27839[] = {{&S20506[31], 6}, {NULL}};
const Segment SGI27840[] = {{&S20530[44], 15}, {NULL}};
const Segment SGI27841[] = {{&S20530[44], 28}, {NULL}};
const Segment SGI27842[] = {{&S20530[30], 6}, {NULL}};
const Segment SGI27843[] = {{&S20530[120], 3}, {NULL}};
const Segment SGI27844[] = {{&S20530[86], 5}, {NULL}};
const Segment SGI27845[] = {{&S20541[44], 8}, {NULL}};
const Segment SGI27846[] = {{&S20541[44], 25}, {NULL}};
const Segment SGI27847[] = {{&S20541[30], 6}, {NULL}};
const Segment SGI27848[] = {{&S20541[117], 3}, {NULL}};
const Segment SGI27849[] = {{&S20541[83], 5}, {NULL}};
const Segment SGI27850[] = {{&S20542[44], 6}, {NULL}};
const Segment SGI27851[] = {{&S20542[44], 17}, {NULL}};
const Segment SGI27852[] = {{&S20542[30], 6}, {NULL}};
const Segment SGI27853[] = {{&S20542[109], 3}, {NULL}};
const Segment SGI27854[] = {{&S20542[75], 5}, {NULL}};
const Segment SGI27855[] = {{&S20589[38], 6}, {NULL}};
const Segment SGI27856[] = {{&S20589[31], 6}, {NULL}};
const Segment SGI27857[] = {{&S20210[44], 19}, {NULL}};
const Segment SGI27858[] = {{&S20210[44], 38}, {NULL}};
const Segment SGI27859[] = {{&S20210[30], 6}, {NULL}};
const Segment SGI27860[] = {{&S20210[69], 3}, {NULL}};
const Segment SGI27861[] = {{&S20210[96], 5}, {NULL}};
const Segment SGI27862[] = {{&S20211[44], 19}, {NULL}};
const Segment SGI27863[] = {{&S20211[44], 38}, {NULL}};
const Segment SGI27864[] = {{&S20211[30], 6}, {NULL}};
const Segment SGI27865[] = {{&S20211[69], 3}, {NULL}};
const Segment SGI27866[] = {{&S20211[96], 5}, {NULL}};
const Segment SGI27867[] = {{&S25864[44], 19}, {NULL}};
const Segment SGI27868[] = {{&S25864[44], 36}, {NULL}};
const Segment SGI27869[] = {{&S25864[30], 6}, {NULL}};
const Segment SGI27870[] = {{&S25864[128], 3}, {NULL}};
const Segment SGI27871[] = {{&S25864[94], 5}, {NULL}};
const Segment SGI27872[] = {{&S20217[44], 19}, {NULL}};
const Segment SGI27873[] = {{&S20217[44], 38}, {NULL}};
const Segment SGI27874[] = {{&S20217[30], 6}, {NULL}};
const Segment SGI27875[] = {{&S20217[69], 3}, {NULL}};
const Segment SGI27876[] = {{&S20217[96], 5}, {NULL}};
const Segment SGI27877[] = {{&S25865[44], 7}, {NULL}};
const Segment SGI27878[] = {{&S25865[44], 24}, {NULL}};
const Segment SGI27879[] = {{&S25865[30], 6}, {NULL}};
const Segment SGI27880[] = {{&S25865[116], 3}, {NULL}};
const Segment SGI27881[] = {{&S25865[82], 5}, {NULL}};
const Segment SGI27882[] = {{&S20218[44], 19}, {NULL}};
const Segment SGI27883[] = {{&S20218[44], 38}, {NULL}};
const Segment SGI27884[] = {{&S20218[30], 6}, {NULL}};
const Segment SGI27885[] = {{&S20218[69], 3}, {NULL}};
const Segment SGI27886[] = {{&S20218[96], 5}, {NULL}};
const Segment SGI27887[] = {{&S20220[44], 19}, {NULL}};
const Segment SGI27888[] = {{&S20220[44], 38}, {NULL}};
const Segment SGI27889[] = {{&S20220[30], 6}, {NULL}};
const Segment SGI27890[] = {{&S20220[69], 3}, {NULL}};
const Segment SGI27891[] = {{&S20220[96], 5}, {NULL}};
const Segment SGI27892[] = {{&S20221[44], 7}, {NULL}};
const Segment SGI27893[] = {{&S20221[44], 26}, {NULL}};
const Segment SGI27894[] = {{&S20221[30], 6}, {NULL}};
const Segment SGI27895[] = {{&S20221[57], 3}, {NULL}};
const Segment SGI27896[] = {{&S20221[84], 5}, {NULL}};
const Segment SGI27897[] = {{&S20222[44], 7}, {NULL}};
const Segment SGI27898[] = {{&S20222[44], 25}, {NULL}};
const Segment SGI27899[] = {{&S20222[30], 6}, {NULL}};
const Segment SGI27900[] = {{&S20222[57], 3}, {NULL}};
const Segment SGI27901[] = {{&S20222[83], 5}, {NULL}};
const Segment SGI27902[] = {{&S20224[44], 19}, {NULL}};
const Segment SGI27903[] = {{&S20224[44], 37}, {NULL}};
const Segment SGI27904[] = {{&S20224[30], 6}, {NULL}};
const Segment SGI27905[] = {{&S20224[69], 3}, {NULL}};
const Segment SGI27906[] = {{&S20224[95], 5}, {NULL}};
const Segment SGI27907[] = {{&S20226[44], 19}, {NULL}};
const Segment SGI27908[] = {{&S20226[44], 37}, {NULL}};
const Segment SGI27909[] = {{&S20226[30], 6}, {NULL}};
const Segment SGI27910[] = {{&S20226[69], 3}, {NULL}};
const Segment SGI27911[] = {{&S20226[95], 5}, {NULL}};
const Segment SGI27912[] = {{&S25872[44], 19}, {NULL}};
const Segment SGI27913[] = {{&S25872[44], 35}, {NULL}};
const Segment SGI27914[] = {{&S25872[30], 6}, {NULL}};
const Segment SGI27915[] = {{&S25872[127], 3}, {NULL}};
const Segment SGI27916[] = {{&S25872[93], 5}, {NULL}};
const Segment SGI27917[] = {{&S20228[44], 19}, {NULL}};
const Segment SGI27918[] = {{&S20228[44], 37}, {NULL}};
const Segment SGI27919[] = {{&S20228[30], 6}, {NULL}};
const Segment SGI27920[] = {{&S20228[69], 3}, {NULL}};
const Segment SGI27921[] = {{&S20228[95], 5}, {NULL}};
const Segment SGI27922[] = {{&S20229[44], 20}, {NULL}};
const Segment SGI27923[] = {{&S20229[44], 38}, {NULL}};
const Segment SGI27924[] = {{&S20229[30], 6}, {NULL}};
const Segment SGI27925[] = {{&S20229[70], 3}, {NULL}};
const Segment SGI27926[] = {{&S20229[96], 5}, {NULL}};
const Segment SGI27927[] = {{&S25876[44], 19}, {NULL}};
const Segment SGI27928[] = {{&S25876[44], 35}, {NULL}};
const Segment SGI27929[] = {{&S25876[30], 6}, {NULL}};
const Segment SGI27930[] = {{&S20230[44], 19}, {NULL}};
const Segment SGI27931[] = {{&S25876[127], 3}, {NULL}};
const Segment SGI27932[] = {{&S20230[44], 38}, {NULL}};
const Segment SGI27933[] = {{&S20230[30], 6}, {NULL}};
const Segment SGI27934[] = {{&S25876[93], 5}, {NULL}};
const Segment SGI27935[] = {{&S20230[69], 3}, {NULL}};
const Segment SGI27936[] = {{&S20230[96], 5}, {NULL}};
const Segment SGI27937[] = {{&S20231[44], 19}, {NULL}};
const Segment SGI27938[] = {{&S20231[44], 37}, {NULL}};
const Segment SGI27939[] = {{&S20231[30], 6}, {NULL}};
const Segment SGI27940[] = {{&S20231[69], 3}, {NULL}};
const Segment SGI27941[] = {{&S20231[95], 5}, {NULL}};
const Segment SGI27942[] = {{&S20232[44], 7}, {NULL}};
const Segment SGI27943[] = {{&S20232[44], 25}, {NULL}};
const Segment SGI27944[] = {{&S20232[30], 6}, {NULL}};
const Segment SGI27945[] = {{&S20232[57], 3}, {NULL}};
const Segment SGI27946[] = {{&S20232[83], 5}, {NULL}};
const Segment SGI27947[] = {{&S20233[44], 20}, {NULL}};
const Segment SGI27948[] = {{&S20233[44], 38}, {NULL}};
const Segment SGI27949[] = {{&S20233[30], 6}, {NULL}};
const Segment SGI27950[] = {{&S20233[70], 3}, {NULL}};
const Segment SGI27951[] = {{&S20233[96], 5}, {NULL}};
const Segment SGI27952[] = {{&S20234[38], 7}, {NULL}};
const Segment SGI27953[] = {{&S20234[31], 6}, {NULL}};
const Segment SGI27954[] = {{&S20235[44], 20}, {NULL}};
const Segment SGI27955[] = {{&S20235[44], 38}, {NULL}};
const Segment SGI27956[] = {{&S20235[30], 6}, {NULL}};
const Segment SGI27957[] = {{&S20235[70], 3}, {NULL}};
const Segment SGI27958[] = {{&S20235[96], 5}, {NULL}};
const Segment SGI27959[] = {{&S20236[44], 20}, {NULL}};
const Segment SGI27960[] = {{&S20236[44], 37}, {NULL}};
const Segment SGI27961[] = {{&S20236[30], 6}, {NULL}};
const Segment SGI27962[] = {{&S20236[70], 3}, {NULL}};
const Segment SGI27963[] = {{&S20236[95], 5}, {NULL}};
const Segment SGI27964[] = {{&S20237[44], 19}, {NULL}};
const Segment SGI27965[] = {{&S20237[44], 37}, {NULL}};
const Segment SGI27966[] = {{&S20237[30], 6}, {NULL}};
const Segment SGI27967[] = {{&S20237[69], 3}, {NULL}};
const Segment SGI27968[] = {{&S20237[95], 5}, {NULL}};
const Segment SGI27969[] = {{&S20238[44], 7}, {NULL}};
const Segment SGI27970[] = {{&S20238[44], 26}, {NULL}};
const Segment SGI27971[] = {{&S20238[30], 6}, {NULL}};
const Segment SGI27972[] = {{&S20238[57], 3}, {NULL}};
const Segment SGI27973[] = {{&S20238[84], 5}, {NULL}};
const Segment SGI27974[] = {{&S20239[44], 20}, {NULL}};
const Segment SGI27975[] = {{&S20239[44], 38}, {NULL}};
const Segment SGI27976[] = {{&S20239[30], 6}, {NULL}};
const Segment SGI27977[] = {{&S20239[70], 3}, {NULL}};
const Segment SGI27978[] = {{&S20239[96], 5}, {NULL}};
const Segment SGI27979[] = {{&S20240[44], 7}, {NULL}};
const Segment SGI27980[] = {{&S20240[44], 25}, {NULL}};
const Segment SGI27981[] = {{&S20240[30], 6}, {NULL}};
const Segment SGI27982[] = {{&S20240[57], 3}, {NULL}};
const Segment SGI27983[] = {{&S20240[83], 5}, {NULL}};
const Segment SGI27984[] = {{&S20242[44], 7}, {NULL}};
const Segment SGI27985[] = {{&S20242[44], 25}, {NULL}};
const Segment SGI27986[] = {{&S20242[30], 6}, {NULL}};
const Segment SGI27987[] = {{&S20242[86], 4}, {NULL}};
const Segment SGI27988[] = {{&S20242[86], 3}, {NULL}};
const Segment SGI27989[] = {{&S20243[44], 19}, {NULL}};
const Segment SGI27990[] = {{&S20243[44], 37}, {NULL}};
const Segment SGI27991[] = {{&S20243[30], 6}, {NULL}};
const Segment SGI27992[] = {{&S20243[69], 3}, {NULL}};
const Segment SGI27993[] = {{&S20243[95], 5}, {NULL}};
const Segment SGI27994[] = {{&S20244[44], 7}, {NULL}};
const Segment SGI27995[] = {{&S20244[44], 25}, {NULL}};
const Segment SGI27996[] = {{&S20244[30], 6}, {NULL}};
const Segment SGI27997[] = {{&S20244[57], 3}, {NULL}};
const Segment SGI27998[] = {{&S20244[83], 5}, {NULL}};
const Segment SGI27999[] = {{&S20246[44], 7}, {NULL}};
const Segment SGI28000[] = {{&S20246[44], 25}, {NULL}};
const Segment SGI28001[] = {{&S20246[30], 6}, {NULL}};
const Segment SGI28002[] = {{&S20246[57], 3}, {NULL}};
const Segment SGI28003[] = {{&S20246[83], 5}, {NULL}};
const Segment SGI28004[] = {{&S20248[44], 7}, {NULL}};
const Segment SGI28005[] = {{&S20248[44], 25}, {NULL}};
const Segment SGI28006[] = {{&S20248[30], 6}, {NULL}};
const Segment SGI28007[] = {{&S20248[86], 6}, {NULL}};
const Segment SGI28008[] = {{&S20250[44], 7}, {NULL}};
const Segment SGI28009[] = {{&S20250[44], 25}, {NULL}};
const Segment SGI28010[] = {{&S20250[30], 6}, {NULL}};
const Segment SGI28011[] = {{&S20250[57], 3}, {NULL}};
const Segment SGI28012[] = {{&S20250[83], 5}, {NULL}};
const Segment SGI28013[] = {{&S20252[44], 7}, {NULL}};
const Segment SGI28014[] = {{&S20252[44], 25}, {NULL}};
const Segment SGI28015[] = {{&S20252[30], 6}, {NULL}};
const Segment SGI28016[] = {{&S20252[86], 4}, {NULL}};
const Segment SGI28017[] = {{&S20252[86], 3}, {NULL}};
const Segment SGI28018[] = {{&S20253[44], 7}, {NULL}};
const Segment SGI28019[] = {{&S20253[44], 25}, {NULL}};
const Segment SGI28020[] = {{&S20253[30], 6}, {NULL}};
const Segment SGI28021[] = {{&S20253[86], 4}, {NULL}};
const Segment SGI28022[] = {{&S20253[86], 3}, {NULL}};
const Segment SGI28023[] = {{&S20255[44], 19}, {NULL}};
const Segment SGI28024[] = {{&S20255[44], 37}, {NULL}};
const Segment SGI28025[] = {{&S20255[30], 6}, {NULL}};
const Segment SGI28026[] = {{&S20255[69], 3}, {NULL}};
const Segment SGI28027[] = {{&S20255[95], 5}, {NULL}};
const Segment SGI28028[] = {{&S20256[44], 20}, {NULL}};
const Segment SGI28029[] = {{&S20256[44], 38}, {NULL}};
const Segment SGI28030[] = {{&S20256[30], 6}, {NULL}};
const Segment SGI28031[] = {{&S20256[70], 3}, {NULL}};
const Segment SGI28032[] = {{&S20256[96], 5}, {NULL}};
const Segment SGI28033[] = {{&S20257[44], 7}, {NULL}};
const Segment SGI28034[] = {{&S20257[44], 25}, {NULL}};
const Segment SGI28035[] = {{&S20257[30], 6}, {NULL}};
const Segment SGI28036[] = {{&S20257[86], 6}, {NULL}};
const Segment SGI28037[] = {{&S20258[42], 19}, {NULL}};
const Segment SGI28038[] = {{&S20258[30], 6}, {NULL}};
const Segment SGI28039[] = {{&S20258[67], 3}, {NULL}};
const Segment SGI28040[] = {{&S20258[93], 5}, {NULL}};
const Segment SGI28041[] = {{&S20259[44], 20}, {NULL}};
const Segment SGI28042[] = {{&S20259[44], 38}, {NULL}};
const Segment SGI28043[] = {{&S20259[30], 6}, {NULL}};
const Segment SGI28044[] = {{&S20259[70], 3}, {NULL}};
const Segment SGI28045[] = {{&S20259[96], 5}, {NULL}};
const Segment SGI28046[] = {{&S20260[38], 7}, {NULL}};
const Segment SGI28047[] = {{&S20260[31], 6}, {NULL}};
const Segment SGI28048[] = {{&S20353[30], 6}, {NULL}};
const Segment SGI28049[] = {{&S20353[91], 3}, {NULL}};
const Segment SGI28050[] = {{&S20353[57], 5}, {NULL}};
const Segment SGI28051[] = {{&S20353[102], 11}, {NULL}};
const Segment SGI28052[] = {{&S20358[30], 6}, {NULL}};
const Segment SGI28053[] = {{&S20358[114], 3}, {NULL}};
const Segment SGI28054[] = {{&S20358[59], 6}, {NULL}};
const Segment SGI28055[] = {{&S20360[44], 27}, {NULL}};
const Segment SGI28056[] = {{&S20360[44], 38}, {NULL}};
const Segment SGI28057[] = {{&S20360[30], 6}, {NULL}};
const Segment SGI28058[] = {{&S20360[130], 3}, {NULL}};
const Segment SGI28059[] = {{&S20360[96], 5}, {NULL}};
const Segment SGI28060[] = {{&S20361[44], 9}, {NULL}};
const Segment SGI28061[] = {{&S20361[44], 24}, {NULL}};
const Segment SGI28062[] = {{&S20361[30], 6}, {NULL}};
const Segment SGI28063[] = {{&S20361[116], 3}, {NULL}};
const Segment SGI28064[] = {{&S20361[82], 5}, {NULL}};
const Segment SGI28065[] = {{&S25633[44], 7}, {NULL}};
const Segment SGI28066[] = {{&S25633[44], 31}, {NULL}};
const Segment SGI28067[] = {{&S25633[30], 6}, {NULL}};
const Segment SGI28068[] = {{&S25633[146], 3}, {NULL}};
const Segment SGI28069[] = {{&S25633[112], 5}, {NULL}};
const Segment SGI28070[] = {{&S20367[30], 6}, {NULL}};
const Segment SGI28071[] = {{&S20367[91], 3}, {NULL}};
const Segment SGI28072[] = {{&S20367[57], 5}, {NULL}};
const Segment SGI28073[] = {{&S20370[37], 7}, {NULL}};
const Segment SGI28074[] = {{&S20370[30], 6}, {NULL}};
const Segment SGI28075[] = {{&S20370[108], 3}, {NULL}};
const Segment SGI28076[] = {{&S20370[74], 5}, {NULL}};
const Segment SGI28077[] = {{&S20371[37], 7}, {NULL}};
const Segment SGI28078[] = {{&S20371[30], 6}, {NULL}};
const Segment SGI28079[] = {{&S20371[108], 3}, {NULL}};
const Segment SGI28080[] = {{&S20371[74], 5}, {NULL}};
const Segment SGI28081[] = {{&S20373[30], 6}, {NULL}};
const Segment SGI28082[] = {{&S20373[113], 3}, {NULL}};
const Segment SGI28083[] = {{&S20373[59], 5}, {NULL}};
const Segment SGI28084[] = {{&S20374[30], 6}, {NULL}};
const Segment SGI28085[] = {{&S20374[114], 3}, {NULL}};
const Segment SGI28086[] = {{&S20374[59], 6}, {NULL}};
const Segment SGI28087[] = {{&S20378[30], 6}, {NULL}};
const Segment SGI28088[] = {{&S20378[113], 3}, {NULL}};
const Segment SGI28089[] = {{&S20378[59], 5}, {NULL}};
const Segment SGI28090[] = {{&S20380[22], 9}, {NULL}};
const Segment SGI28091[] = {{&S20380[44], 20}, {NULL}};
const Segment SGI28092[] = {{&S20380[30], 6}, {&S20380[52], 12}, {NULL}};
const Segment SGI28093[] = {{&S20380[112], 3}, {NULL}};
const Segment SGI28094[] = {{&S20380[78], 5}, {NULL}};
const Segment SGI28095[] = {{&S20384[44], 21}, {NULL}};
const Segment SGI28096[] = {{&S20384[44], 32}, {NULL}};
const Segment SGI28097[] = {{&S20384[30], 6}, {NULL}};
const Segment SGI28098[] = {{&S20384[124], 3}, {NULL}};
const Segment SGI28099[] = {{&S20384[90], 5}, {NULL}};
const Segment SGI28100[] = {{&S20386[44], 13}, {NULL}};
const Segment SGI28101[] = {{&S20386[44], 40}, {NULL}};
const Segment SGI28102[] = {{&S20386[30], 6}, {NULL}};
const Segment SGI28103[] = {{&S20386[132], 3}, {NULL}};
const Segment SGI28104[] = {{&S20386[98], 5}, {NULL}};
const Segment SGI28105[] = {{&S20388[44], 4}, {NULL}};
const Segment SGI28106[] = {{&S20388[44], 15}, {NULL}};
const Segment SGI28107[] = {{&S20388[30], 6}, {NULL}};
const Segment SGI28108[] = {{&S20388[107], 3}, {NULL}};
const Segment SGI28109[] = {{&S20388[73], 5}, {NULL}};
const Segment SGI28110[] = {{&S20389[44], 16}, {NULL}};
const Segment SGI28111[] = {{&S20389[44], 25}, {NULL}};
const Segment SGI28112[] = {{&S20389[30], 6}, {NULL}};
const Segment SGI28113[] = {{&S20389[117], 3}, {NULL}};
const Segment SGI28114[] = {{&S20389[83], 5}, {NULL}};
const Segment SGI28115[] = {{&S20390[44], 22}, {NULL}};
const Segment SGI28116[] = {{&S20390[44], 33}, {NULL}};
const Segment SGI28117[] = {{&S20390[30], 6}, {NULL}};
const Segment SGI28118[] = {{&S20390[125], 3}, {NULL}};
const Segment SGI28119[] = {{&S20390[91], 5}, {NULL}};
const Segment SGI28120[] = {{&S20392[22], 9}, {NULL}};
const Segment SGI28121[] = {{&S20392[44], 21}, {NULL}};
const Segment SGI28122[] = {{&S20392[30], 6}, {&S20392[52], 13}, {NULL}};
const Segment SGI28123[] = {{&S20392[113], 3}, {NULL}};
const Segment SGI28124[] = {{&S20392[79], 5}, {NULL}};
const Segment SGI28125[] = {{&S20397[44], 5}, {NULL}};
const Segment SGI28126[] = {{&S20397[44], 16}, {NULL}};
const Segment SGI28127[] = {{&S20397[30], 6}, {NULL}};
const Segment SGI28128[] = {{&S20397[108], 3}, {NULL}};
const Segment SGI28129[] = {{&S20397[74], 5}, {NULL}};
const Segment SGI28130[] = {{&S20406[44], 24}, {NULL}};
const Segment SGI28131[] = {{&S20406[44], 35}, {NULL}};
const Segment SGI28132[] = {{&S20406[30], 6}, {NULL}};
const Segment SGI28133[] = {{&S20406[127], 3}, {NULL}};
const Segment SGI28134[] = {{&S20406[93], 5}, {NULL}};
const Segment SGI28135[] = {{&S20407[44], 17}, {NULL}};
const Segment SGI28136[] = {{&S20407[44], 28}, {NULL}};
const Segment SGI28137[] = {{&S20407[30], 6}, {NULL}};
const Segment SGI28138[] = {{&S20407[120], 3}, {NULL}};
const Segment SGI28139[] = {{&S20407[86], 5}, {NULL}};
const Segment SGI28140[] = {{&S20408[44], 22}, {NULL}};
const Segment SGI28141[] = {{&S20408[44], 33}, {NULL}};
const Segment SGI28142[] = {{&S20408[30], 6}, {NULL}};
const Segment SGI28143[] = {{&S20408[125], 3}, {NULL}};
const Segment SGI28144[] = {{&S20408[91], 5}, {NULL}};
const Segment SGI28145[] = {{&S20410[44], 16}, {NULL}};
const Segment SGI28146[] = {{&S20410[44], 27}, {NULL}};
const Segment SGI28147[] = {{&S20410[30], 6}, {NULL}};
const Segment SGI28148[] = {{&S20410[119], 3}, {NULL}};
const Segment SGI28149[] = {{&S20410[85], 5}, {NULL}};
const Segment SGI28150[] = {{&S20411[44], 27}, {NULL}};
const Segment SGI28151[] = {{&S20411[44], 38}, {NULL}};
const Segment SGI28152[] = {{&S20411[30], 6}, {NULL}};
const Segment SGI28153[] = {{&S20411[130], 3}, {NULL}};
const Segment SGI28154[] = {{&S20411[96], 5}, {NULL}};
const Segment SGI28155[] = {{&S20416[44], 9}, {NULL}};
const Segment SGI28156[] = {{&S20416[44], 20}, {NULL}};
const Segment SGI28157[] = {{&S20416[30], 6}, {NULL}};
const Segment SGI28158[] = {{&S20416[112], 3}, {NULL}};
const Segment SGI28159[] = {{&S20416[78], 5}, {NULL}};
const Segment SGI28160[] = {{&S20421[44], 22}, {NULL}};
const Segment SGI28161[] = {{&S20421[44], 33}, {NULL}};
const Segment SGI28162[] = {{&S20421[30], 6}, {NULL}};
const Segment SGI28163[] = {{&S20421[125], 3}, {NULL}};
const Segment SGI28164[] = {{&S20421[91], 5}, {NULL}};
const Segment SGI28165[] = {{&S20431[44], 14}, {NULL}};
const Segment SGI28166[] = {{&S20431[44], 23}, {NULL}};
const Segment SGI28167[] = {{&S20431[30], 6}, {NULL}};
const Segment SGI28168[] = {{&S20431[115], 3}, {NULL}};
const Segment SGI28169[] = {{&S20431[81], 5}, {NULL}};
const Segment SGI28170[] = {{&S20439[22], 9}, {NULL}};
const Segment SGI28171[] = {{&S20439[44], 20}, {NULL}};
const Segment SGI28172[] = {{&S20439[30], 6}, {&S20439[52], 12}, {NULL}};
const Segment SGI28173[] = {{&S20439[81], 4}, {NULL}};
const Segment SGI28174[] = {{&S20439[81], 3}, {NULL}};
const Segment SGI28175[] = {{&S20440[23], 9}, {NULL}};
const Segment SGI28176[] = {{&S20440[31], 6}, {&S20440[46], 12}, {NULL}};
const Segment SGI28177[] = {{&S20441[38], 13}, {NULL}};
const Segment SGI28178[] = {{&S20441[31], 6}, {NULL}};
const Segment SGI28179[] = {{&S20442[31], 6}, {NULL}};
const Segment SGI28180[] = {{&S20444[44], 24}, {NULL}};
const Segment SGI28181[] = {{&S20444[44], 35}, {NULL}};
const Segment SGI28182[] = {{&S20444[30], 6}, {NULL}};
const Segment SGI28183[] = {{&S20444[127], 3}, {NULL}};
const Segment SGI28184[] = {{&S20444[93], 5}, {NULL}};
const Segment SGI28185[] = {{&S20445[38], 5}, {NULL}};
const Segment SGI28186[] = {{&S20445[31], 6}, {NULL}};
const Segment SGI28187[] = {{&S20447[44], 19}, {NULL}};
const Segment SGI28188[] = {{&S20447[44], 30}, {NULL}};
const Segment SGI28189[] = {{&S20447[30], 6}, {NULL}};
const Segment SGI28190[] = {{&S20447[122], 3}, {NULL}};
const Segment SGI28191[] = {{&S20447[88], 5}, {NULL}};
const Segment SGI28192[] = {{&S20453[44], 18}, {NULL}};
const Segment SGI28193[] = {{&S20453[44], 29}, {NULL}};
const Segment SGI28194[] = {{&S20453[30], 6}, {NULL}};
const Segment SGI28195[] = {{&S20453[121], 3}, {NULL}};
const Segment SGI28196[] = {{&S20453[87], 5}, {NULL}};
const Segment SGI28197[] = {{&S20455[44], 19}, {NULL}};
const Segment SGI28198[] = {{&S20455[44], 30}, {NULL}};
const Segment SGI28199[] = {{&S20455[30], 6}, {NULL}};
const Segment SGI28200[] = {{&S20455[122], 3}, {NULL}};
const Segment SGI28201[] = {{&S20455[88], 5}, {NULL}};
const Segment SGI28202[] = {{&S20457[44], 7}, {NULL}};
const Segment SGI28203[] = {{&S20457[44], 18}, {NULL}};
const Segment SGI28204[] = {{&S20457[30], 6}, {NULL}};
const Segment SGI28205[] = {{&S20457[110], 3}, {NULL}};
const Segment SGI28206[] = {{&S20457[76], 5}, {NULL}};
const Segment SGI28207[] = {{&S20464[44], 19}, {NULL}};
const Segment SGI28208[] = {{&S20464[44], 30}, {NULL}};
const Segment SGI28209[] = {{&S20464[30], 6}, {NULL}};
const Segment SGI28210[] = {{&S20464[122], 3}, {NULL}};
const Segment SGI28211[] = {{&S20464[88], 5}, {NULL}};
const Segment SGI28212[] = {{&S20468[44], 21}, {NULL}};
const Segment SGI28213[] = {{&S20468[44], 32}, {NULL}};
const Segment SGI28214[] = {{&S20468[30], 6}, {NULL}};
const Segment SGI28215[] = {{&S20468[124], 3}, {NULL}};
const Segment SGI28216[] = {{&S20468[90], 5}, {NULL}};
const Segment SGI28217[] = {{&S20474[44], 22}, {NULL}};
const Segment SGI28218[] = {{&S20474[44], 33}, {NULL}};
const Segment SGI28219[] = {{&S20474[30], 6}, {NULL}};
const Segment SGI28220[] = {{&S20474[125], 3}, {NULL}};
const Segment SGI28221[] = {{&S20474[91], 5}, {NULL}};
const Segment SGI28222[] = {{&S20479[44], 16}, {NULL}};
const Segment SGI28223[] = {{&S20479[44], 27}, {NULL}};
const Segment SGI28224[] = {{&S20479[30], 6}, {NULL}};
const Segment SGI28225[] = {{&S20479[119], 3}, {NULL}};
const Segment SGI28226[] = {{&S20479[85], 5}, {NULL}};
const Segment SGI28227[] = {{&S20487[30], 6}, {NULL}};
const Segment SGI28228[] = {{&S20487[113], 3}, {NULL}};
const Segment SGI28229[] = {{&S20487[59], 5}, {NULL}};
const Segment SGI28230[] = {{&S20492[38], 9}, {NULL}};
const Segment SGI28231[] = {{&S20492[31], 6}, {NULL}};
const Segment SGI28232[] = {{&S20494[30], 6}, {NULL}};
const Segment SGI28233[] = {{&S20494[104], 3}, {NULL}};
const Segment SGI28234[] = {{&S20494[57], 6}, {NULL}};
const Segment SGI28235[] = {{&S20497[44], 5}, {NULL}};
const Segment SGI28236[] = {{&S20497[44], 14}, {NULL}};
const Segment SGI28237[] = {{&S20497[30], 6}, {NULL}};
const Segment SGI28238[] = {{&S20497[106], 3}, {NULL}};
const Segment SGI28239[] = {{&S20497[72], 5}, {NULL}};
const Segment SGI28240[] = {{&S23151[44], 9}, {NULL}};
const Segment SGI28241[] = {{&S23151[44], 20}, {NULL}};
const Segment SGI28242[] = {{&S23151[30], 6}, {NULL}};
const Segment SGI28243[] = {{&S23151[112], 3}, {NULL}};
const Segment SGI28244[] = {{&S23151[78], 5}, {NULL}};
const Segment SGI28245[] = {{&S20516[44], 8}, {NULL}};
const Segment SGI28246[] = {{&S20516[44], 19}, {NULL}};
const Segment SGI28247[] = {{&S20516[30], 6}, {NULL}};
const Segment SGI28248[] = {{&S20516[111], 3}, {NULL}};
const Segment SGI28249[] = {{&S20516[77], 5}, {NULL}};
const Segment SGI28250[] = {{&S20521[44], 10}, {NULL}};
const Segment SGI28251[] = {{&S20521[44], 27}, {NULL}};
const Segment SGI28252[] = {{&S20521[30], 6}, {NULL}};
const Segment SGI28253[] = {{&S20521[119], 3}, {NULL}};
const Segment SGI28254[] = {{&S20521[85], 5}, {NULL}};
const Segment SGI28255[] = {{&S20526[44], 7}, {NULL}};
const Segment SGI28256[] = {{&S20526[44], 23}, {NULL}};
const Segment SGI28257[] = {{&S20526[30], 6}, {NULL}};
const Segment SGI28258[] = {{&S20526[115], 3}, {NULL}};
const Segment SGI28259[] = {{&S20526[81], 5}, {NULL}};
const Segment SGI28260[] = {{&S20540[44], 35}, {NULL}};
const Segment SGI28261[] = {{&S20540[30], 6}, {NULL}};
const Segment SGI28262[] = {{&S20540[145], 3}, {NULL}};
const Segment SGI28263[] = {{&S20540[111], 5}, {NULL}};
const Segment SGI28264[] = {{&S20556[44], 19}, {NULL}};
const Segment SGI28265[] = {{&S20556[44], 30}, {NULL}};
const Segment SGI28266[] = {{&S20556[30], 6}, {NULL}};
const Segment SGI28267[] = {{&S20556[122], 3}, {NULL}};
const Segment SGI28268[] = {{&S20556[88], 5}, {NULL}};
const Segment SGI28269[] = {{&S20591[44], 12}, {NULL}};
const Segment SGI28270[] = {{&S20591[44], 23}, {NULL}};
const Segment SGI28271[] = {{&S20591[30], 6}, {NULL}};
const Segment SGI28272[] = {{&S20591[115], 3}, {NULL}};
const Segment SGI28273[] = {{&S20591[81], 5}, {NULL}};
const Segment SGI28274[] = {{&S20595[44], 2}, {NULL}};
const Segment SGI28275[] = {{&S20595[44], 13}, {NULL}};
const Segment SGI28276[] = {{&S20595[30], 6}, {NULL}};
const Segment SGI28277[] = {{&S20595[105], 3}, {NULL}};
const Segment SGI28278[] = {{&S20595[71], 5}, {NULL}};
const Segment SGI28279[] = {{&S20598[38], 22}, {NULL}};
const Segment SGI28280[] = {{&S20598[31], 6}, {NULL}};
const Segment SGI28281[] = {{&S20609[44], 5}, {NULL}};
const Segment SGI28282[] = {{&S20609[44], 16}, {NULL}};
const Segment SGI28283[] = {{&S20609[30], 6}, {NULL}};
const Segment SGI28284[] = {{&S20609[108], 3}, {NULL}};
const Segment SGI28285[] = {{&S20609[74], 5}, {NULL}};
const Segment SGI28286[] = {{&S20621[44], 10}, {NULL}};
const Segment SGI28287[] = {{&S20621[44], 32}, {NULL}};
const Segment SGI28288[] = {{&S20621[30], 6}, {NULL}};
const Segment SGI28289[] = {{&S20621[123], 3}, {NULL}};
const Segment SGI28290[] = {{&S20621[90], 5}, {NULL}};
const Segment SGI28291[] = {{&S21964[44], 19}, {NULL}};
const Segment SGI28292[] = {{&S21964[44], 36}, {NULL}};
const Segment SGI28293[] = {{&S21964[30], 6}, {NULL}};
const Segment SGI28294[] = {{&S21964[69], 3}, {NULL}};
const Segment SGI28295[] = {{&S21964[94], 5}, {NULL}};
const Segment SGI28296[] = {{&S21965[44], 19}, {NULL}};
const Segment SGI28297[] = {{&S21965[44], 36}, {NULL}};
const Segment SGI28298[] = {{&S21965[30], 6}, {NULL}};
const Segment SGI28299[] = {{&S21965[69], 3}, {NULL}};
const Segment SGI28300[] = {{&S21965[94], 5}, {NULL}};
const Segment SGI28301[] = {{&S21966[44], 19}, {NULL}};
const Segment SGI28302[] = {{&S21966[44], 36}, {NULL}};
const Segment SGI28303[] = {{&S21966[30], 6}, {NULL}};
const Segment SGI28304[] = {{&S21966[69], 3}, {NULL}};
const Segment SGI28305[] = {{&S21966[94], 5}, {NULL}};
const Segment SGI28306[] = {{&S21967[44], 20}, {NULL}};
const Segment SGI28307[] = {{&S21967[44], 37}, {NULL}};
const Segment SGI28308[] = {{&S21967[30], 6}, {NULL}};
const Segment SGI28309[] = {{&S21967[70], 3}, {NULL}};
const Segment SGI28310[] = {{&S21967[95], 5}, {NULL}};
const Segment SGI28311[] = {{&S21968[44], 7}, {NULL}};
const Segment SGI28312[] = {{&S21968[44], 24}, {NULL}};
const Segment SGI28313[] = {{&S21968[30], 6}, {NULL}};
const Segment SGI28314[] = {{&S21968[57], 3}, {NULL}};
const Segment SGI28315[] = {{&S21968[82], 5}, {NULL}};
const Segment SGI28316[] = {{&S21969[44], 19}, {NULL}};
const Segment SGI28317[] = {{&S21969[44], 36}, {NULL}};
const Segment SGI28318[] = {{&S21969[30], 6}, {NULL}};
const Segment SGI28319[] = {{&S21969[69], 3}, {NULL}};
const Segment SGI28320[] = {{&S21969[94], 5}, {NULL}};
const Segment SGI28321[] = {{&S21970[44], 21}, {NULL}};
const Segment SGI28322[] = {{&S21970[44], 38}, {NULL}};
const Segment SGI28323[] = {{&S21970[30], 6}, {NULL}};
const Segment SGI28324[] = {{&S21970[71], 3}, {NULL}};
const Segment SGI28325[] = {{&S21970[96], 5}, {NULL}};
const Segment SGI28326[] = {{&S21971[44], 7}, {NULL}};
const Segment SGI28327[] = {{&S21971[44], 24}, {NULL}};
const Segment SGI28328[] = {{&S21971[30], 6}, {NULL}};
const Segment SGI28329[] = {{&S21971[57], 3}, {NULL}};
const Segment SGI28330[] = {{&S21971[82], 5}, {NULL}};
const Segment SGI28331[] = {{&S21972[44], 19}, {NULL}};
const Segment SGI28332[] = {{&S21972[44], 36}, {NULL}};
const Segment SGI28333[] = {{&S21972[30], 6}, {NULL}};
const Segment SGI28334[] = {{&S21972[69], 3}, {NULL}};
const Segment SGI28335[] = {{&S21972[94], 5}, {NULL}};
const Segment SGI28336[] = {{&S21973[44], 21}, {NULL}};
const Segment SGI28337[] = {{&S21973[44], 38}, {NULL}};
const Segment SGI28338[] = {{&S21973[30], 6}, {NULL}};
const Segment SGI28339[] = {{&S21973[71], 3}, {NULL}};
const Segment SGI28340[] = {{&S21973[96], 5}, {NULL}};
const Segment SGI28341[] = {{&S21974[44], 19}, {NULL}};
const Segment SGI28342[] = {{&S21974[44], 36}, {NULL}};
const Segment SGI28343[] = {{&S21974[30], 6}, {NULL}};
const Segment SGI28344[] = {{&S21974[69], 3}, {NULL}};
const Segment SGI28345[] = {{&S21974[94], 5}, {NULL}};
const Segment SGI28346[] = {{&S21975[44], 7}, {NULL}};
const Segment SGI28347[] = {{&S21975[44], 24}, {NULL}};
const Segment SGI28348[] = {{&S21975[30], 6}, {NULL}};
const Segment SGI28349[] = {{&S21975[57], 3}, {NULL}};
const Segment SGI28350[] = {{&S21975[82], 5}, {NULL}};
const Segment SGI28351[] = {{&S21977[38], 7}, {NULL}};
const Segment SGI28352[] = {{&S21977[31], 6}, {NULL}};
const Segment SGI28353[] = {{&S21987[44], 19}, {NULL}};
const Segment SGI28354[] = {{&S21987[44], 36}, {NULL}};
const Segment SGI28355[] = {{&S21987[30], 6}, {NULL}};
const Segment SGI28356[] = {{&S21987[69], 3}, {NULL}};
const Segment SGI28357[] = {{&S21987[94], 5}, {NULL}};
const Segment SGI28358[] = {{&S21993[38], 7}, {NULL}};
const Segment SGI28359[] = {{&S21993[31], 6}, {NULL}};
const Segment SGI28360[] = {{&S21998[44], 7}, {NULL}};
const Segment SGI28361[] = {{&S21998[44], 24}, {NULL}};
const Segment SGI28362[] = {{&S21998[30], 6}, {NULL}};
const Segment SGI28363[] = {{&S21998[57], 3}, {NULL}};
const Segment SGI28364[] = {{&S21998[82], 5}, {NULL}};
const Segment SGI28365[] = {{&S20391[22], 9}, {NULL}};
const Segment SGI28366[] = {{&S20391[44], 20}, {NULL}};
const Segment SGI28367[] = {{&S20391[30], 6}, {&S20391[52], 12}, {NULL}};
const Segment SGI28368[] = {{&S20391[112], 3}, {NULL}};
const Segment SGI28369[] = {{&S20391[78], 5}, {NULL}};
const Segment SGI28370[] = {{&S20396[44], 14}, {NULL}};
const Segment SGI28371[] = {{&S20396[44], 31}, {NULL}};
const Segment SGI28372[] = {{&S20396[30], 6}, {NULL}};
const Segment SGI28373[] = {{&S20396[123], 3}, {NULL}};
const Segment SGI28374[] = {{&S20396[89], 5}, {NULL}};
const Segment SGI28375[] = {{&S20400[44], 7}, {NULL}};
const Segment SGI28376[] = {{&S20400[44], 18}, {NULL}};
const Segment SGI28377[] = {{&S20400[30], 6}, {NULL}};
const Segment SGI28378[] = {{&S20400[110], 3}, {NULL}};
const Segment SGI28379[] = {{&S20400[76], 5}, {NULL}};
const Segment SGI28380[] = {{&S20427[44], 8}, {NULL}};
const Segment SGI28381[] = {{&S20427[44], 19}, {NULL}};
const Segment SGI28382[] = {{&S20427[30], 6}, {NULL}};
const Segment SGI28383[] = {{&S20427[111], 3}, {NULL}};
const Segment SGI28384[] = {{&S20427[77], 5}, {NULL}};
const Segment SGI28385[] = {{&S20460[44], 6}, {NULL}};
const Segment SGI28386[] = {{&S20460[44], 17}, {NULL}};
const Segment SGI28387[] = {{&S20460[30], 6}, {NULL}};
const Segment SGI28388[] = {{&S20460[78], 6}, {NULL}};
const Segment SGI28389[] = {{&S20466[44], 9}, {NULL}};
const Segment SGI28390[] = {{&S20466[44], 22}, {NULL}};
const Segment SGI28391[] = {{&S20466[30], 6}, {NULL}};
const Segment SGI28392[] = {{&S20466[114], 3}, {NULL}};
const Segment SGI28393[] = {{&S20466[80], 5}, {NULL}};
const Segment SGI28394[] = {{&S20467[23], 9}, {NULL}};
const Segment SGI28395[] = {{&S20467[31], 6}, {&S20467[46], 12}, {NULL}};
const Segment SGI28396[] = {{&S20478[44], 8}, {NULL}};
const Segment SGI28397[] = {{&S20478[44], 19}, {NULL}};
const Segment SGI28398[] = {{&S20478[30], 6}, {NULL}};
const Segment SGI28399[] = {{&S20478[111], 3}, {NULL}};
const Segment SGI28400[] = {{&S20478[77], 5}, {NULL}};
const Segment SGI28401[] = {{&S20482[64], 5}, {NULL}};
const Segment SGI28402[] = {{&S20489[30], 6}, {NULL}};
const Segment SGI28403[] = {{&S20489[104], 3}, {NULL}};
const Segment SGI28404[] = {{&S20489[57], 6}, {NULL}};
const Segment SGI28405[] = {{&S20499[44], 9}, {NULL}};
const Segment SGI28406[] = {{&S20499[44], 26}, {NULL}};
const Segment SGI28407[] = {{&S20499[30], 6}, {NULL}};
const Segment SGI28408[] = {{&S20499[118], 3}, {NULL}};
const Segment SGI28409[] = {{&S20499[84], 5}, {NULL}};
const Segment SGI28410[] = {{&S20502[44], 27}, {NULL}};
const Segment SGI28411[] = {{&S20502[30], 6}, {NULL}};
const Segment SGI28412[] = {{&S20502[137], 3}, {NULL}};
const Segment SGI28413[] = {{&S20502[103], 5}, {NULL}};
const Segment SGI28414[] = {{&S20509[44], 12}, {NULL}};
const Segment SGI28415[] = {{&S20509[44], 23}, {NULL}};
const Segment SGI28416[] = {{&S20509[30], 6}, {NULL}};
const Segment SGI28417[] = {{&S20509[115], 3}, {NULL}};
const Segment SGI28418[] = {{&S20509[81], 5}, {NULL}};
const Segment SGI28419[] = {{&S20510[44], 8}, {NULL}};
const Segment SGI28420[] = {{&S20510[44], 19}, {NULL}};
const Segment SGI28421[] = {{&S20510[30], 6}, {NULL}};
const Segment SGI28422[] = {{&S20510[111], 3}, {NULL}};
const Segment SGI28423[] = {{&S20510[77], 5}, {NULL}};
const Segment SGI28424[] = {{&S20511[44], 10}, {NULL}};
const Segment SGI28425[] = {{&S20511[44], 21}, {NULL}};
const Segment SGI28426[] = {{&S20511[30], 6}, {NULL}};
const Segment SGI28427[] = {{&S20511[113], 3}, {NULL}};
const Segment SGI28428[] = {{&S20511[79], 5}, {NULL}};
const Segment SGI28429[] = {{&S20515[44], 13}, {NULL}};
const Segment SGI28430[] = {{&S20515[44], 24}, {NULL}};
const Segment SGI28431[] = {{&S20515[30], 6}, {NULL}};
const Segment SGI28432[] = {{&S20515[115], 3}, {NULL}};
const Segment SGI28433[] = {{&S20515[82], 5}, {NULL}};
const Segment SGI28434[] = {{&S20519[44], 34}, {NULL}};
const Segment SGI28435[] = {{&S20519[30], 6}, {NULL}};
const Segment SGI28436[] = {{&S20519[145], 3}, {NULL}};
const Segment SGI28437[] = {{&S20519[111], 5}, {NULL}};
const Segment SGI28438[] = {{&S20524[44], 8}, {NULL}};
const Segment SGI28439[] = {{&S20524[44], 19}, {NULL}};
const Segment SGI28440[] = {{&S20524[30], 6}, {NULL}};
const Segment SGI28441[] = {{&S20524[111], 3}, {NULL}};
const Segment SGI28442[] = {{&S20524[77], 5}, {NULL}};
const Segment SGI28443[] = {{&S20529[44], 6}, {NULL}};
const Segment SGI28444[] = {{&S20529[44], 17}, {NULL}};
const Segment SGI28445[] = {{&S20529[30], 6}, {NULL}};
const Segment SGI28446[] = {{&S20529[109], 3}, {NULL}};
const Segment SGI28447[] = {{&S20529[75], 5}, {NULL}};
const Segment SGI28448[] = {{&S20536[44], 12}, {NULL}};
const Segment SGI28449[] = {{&S20536[44], 38}, {NULL}};
const Segment SGI28450[] = {{&S20536[30], 6}, {NULL}};
const Segment SGI28451[] = {{&S20536[130], 3}, {NULL}};
const Segment SGI28452[] = {{&S20536[96], 5}, {NULL}};
const Segment SGI28453[] = {{&S20537[44], 9}, {NULL}};
const Segment SGI28454[] = {{&S20537[44], 35}, {NULL}};
const Segment SGI28455[] = {{&S20537[30], 6}, {NULL}};
const Segment SGI28456[] = {{&S20537[127], 3}, {NULL}};
const Segment SGI28457[] = {{&S20537[93], 5}, {NULL}};
const Segment SGI28458[] = {{&S20548[44], 15}, {NULL}};
const Segment SGI28459[] = {{&S20548[44], 32}, {NULL}};
const Segment SGI28460[] = {{&S20548[30], 6}, {NULL}};
const Segment SGI28461[] = {{&S20548[124], 3}, {NULL}};
const Segment SGI28462[] = {{&S20548[90], 5}, {NULL}};
const Segment SGI28463[] = {{&S20558[44], 5}, {NULL}};
const Segment SGI28464[] = {{&S20558[44], 22}, {NULL}};
const Segment SGI28465[] = {{&S20558[30], 6}, {NULL}};
const Segment SGI28466[] = {{&S20558[114], 3}, {NULL}};
const Segment SGI28467[] = {{&S20558[80], 5}, {NULL}};
const Segment SGI28468[] = {{&S20560[44], 7}, {NULL}};
const Segment SGI28469[] = {{&S20560[44], 18}, {NULL}};
const Segment SGI28470[] = {{&S20560[30], 6}, {NULL}};
const Segment SGI28471[] = {{&S20560[110], 3}, {NULL}};
const Segment SGI28472[] = {{&S20560[76], 5}, {NULL}};
const Segment SGI28473[] = {{&S20571[34], 6}, {NULL}};
const Segment SGI28474[] = {{&S20583[44], 14}, {NULL}};
const Segment SGI28475[] = {{&S20583[44], 25}, {NULL}};
const Segment SGI28476[] = {{&S20583[30], 6}, {NULL}};
const Segment SGI28477[] = {{&S20583[117], 3}, {NULL}};
const Segment SGI28478[] = {{&S20583[83], 5}, {NULL}};
const Segment SGI28479[] = {{&S20600[38], 8}, {NULL}};
const Segment SGI28480[] = {{&S20600[31], 6}, {NULL}};
const Segment SGI28481[] = {{&S20603[22], 9}, {NULL}};
const Segment SGI28482[] = {{&S20603[44], 20}, {NULL}};
const Segment SGI28483[] = {{&S20603[30], 6}, {&S20603[52], 12}, {NULL}};
const Segment SGI28484[] = {{&S20603[81], 6}, {NULL}};
const Segment SGI28485[] = {{&S22592[44], 10}, {NULL}};
const Segment SGI28486[] = {{&S22592[44], 27}, {NULL}};
const Segment SGI28487[] = {{&S22592[30], 6}, {NULL}};
const Segment SGI28488[] = {{&S22592[119], 3}, {NULL}};
const Segment SGI28489[] = {{&S22592[85], 5}, {NULL}};
const Segment SGI28490[] = {{&S22596[38], 10}, {NULL}};
const Segment SGI28491[] = {{&S22596[31], 6}, {NULL}};
const Segment SGI28492[] = {{&S22609[44], 10}, {NULL}};
const Segment SGI28493[] = {{&S22609[44], 27}, {NULL}};
const Segment SGI28494[] = {{&S22609[30], 6}, {NULL}};
const Segment SGI28495[] = {{&S22609[88], 4}, {NULL}};
const Segment SGI28496[] = {{&S22609[88], 3}, {NULL}};
const Segment SGI28497[] = {{&S22611[42], 10}, {NULL}};
const Segment SGI28498[] = {{&S22611[30], 6}, {NULL}};
const Segment SGI28499[] = {{&S22611[117], 3}, {NULL}};
const Segment SGI28500[] = {{&S22611[83], 5}, {NULL}};
const Segment SGI28501[] = {{&S32117[44], 16}, {NULL}};
const Segment SGI28502[] = {{&S32117[44], 27}, {NULL}};
const Segment SGI28503[] = {{&S32117[30], 6}, {NULL}};
const Segment SGI28504[] = {{&S32117[119], 3}, {NULL}};
const Segment SGI28505[] = {{&S32117[85], 5}, {NULL}};
const Segment SGI28506[] = {{&S32118[44], 19}, {NULL}};
const Segment SGI28507[] = {{&S32118[30], 6}, {NULL}};
const Segment SGI28508[] = {{&S32118[111], 3}, {NULL}};
const Segment SGI28509[] = {{&S32118[77], 5}, {NULL}};
const Segment SGI28510[] = {{&S32119[44], 9}, {NULL}};
const Segment SGI28511[] = {{&S32119[44], 20}, {NULL}};
const Segment SGI28512[] = {{&S32119[30], 6}, {NULL}};
const Segment SGI28513[] = {{&S32119[112], 3}, {NULL}};
const Segment SGI28514[] = {{&S32119[78], 5}, {NULL}};
const Segment SGI28515[] = {{&S32120[44], 14}, {NULL}};
const Segment SGI28516[] = {{&S32120[44], 25}, {NULL}};
const Segment SGI28517[] = {{&S32120[30], 6}, {NULL}};
const Segment SGI28518[] = {{&S32120[117], 3}, {NULL}};
const Segment SGI28519[] = {{&S32120[83], 5}, {NULL}};
const Segment SGI28520[] = {{&S32121[44], 10}, {NULL}};
const Segment SGI28521[] = {{&S32121[44], 22}, {NULL}};
const Segment SGI28522[] = {{&S32121[30], 6}, {NULL}};
const Segment SGI28523[] = {{&S32121[114], 3}, {NULL}};
const Segment SGI28524[] = {{&S32121[80], 5}, {NULL}};
const Segment SGI28525[] = {{&S32123[44], 19}, {NULL}};
const Segment SGI28526[] = {{&S32123[30], 6}, {NULL}};
const Segment SGI28527[] = {{&S32123[111], 3}, {NULL}};
const Segment SGI28528[] = {{&S32123[77], 5}, {NULL}};
const Segment SGI28529[] = {{&S32124[44], 9}, {NULL}};
const Segment SGI28530[] = {{&S32124[44], 21}, {NULL}};
const Segment SGI28531[] = {{&S32124[30], 6}, {NULL}};
const Segment SGI28532[] = {{&S32124[113], 3}, {NULL}};
const Segment SGI28533[] = {{&S32124[79], 5}, {NULL}};
const Segment SGI28534[] = {{&S32125[44], 10}, {NULL}};
const Segment SGI28535[] = {{&S32125[44], 22}, {NULL}};
const Segment SGI28536[] = {{&S32125[30], 6}, {NULL}};
const Segment SGI28537[] = {{&S32125[114], 3}, {NULL}};
const Segment SGI28538[] = {{&S32125[80], 5}, {NULL}};
const Segment SGI28539[] = {{&S32126[44], 14}, {NULL}};
const Segment SGI28540[] = {{&S32126[44], 25}, {NULL}};
const Segment SGI28541[] = {{&S32126[30], 6}, {NULL}};
const Segment SGI28542[] = {{&S32126[117], 3}, {NULL}};
const Segment SGI28543[] = {{&S32126[83], 5}, {NULL}};
const Segment SGI28544[] = {{&S32127[41], 9}, {NULL}};
const Segment SGI28545[] = {{&S32127[30], 6}, {NULL}};
const Segment SGI28546[] = {{&S32127[109], 3}, {NULL}};
const Segment SGI28547[] = {{&S32127[75], 5}, {NULL}};
const Segment SGI28548[] = {{&S32128[38], 9}, {NULL}};
const Segment SGI28549[] = {{&S32128[31], 6}, {NULL}};
const Segment SGI28550[] = {{&S32129[44], 14}, {NULL}};
const Segment SGI28551[] = {{&S32129[44], 25}, {NULL}};
const Segment SGI28552[] = {{&S32129[30], 6}, {NULL}};
const Segment SGI28553[] = {{&S32129[117], 3}, {NULL}};
const Segment SGI28554[] = {{&S32129[83], 5}, {NULL}};
const Segment SGI28555[] = {{&S32130[44], 9}, {NULL}};
const Segment SGI28556[] = {{&S32130[44], 20}, {NULL}};
const Segment SGI28557[] = {{&S32130[30], 6}, {NULL}};
const Segment SGI28558[] = {{&S32130[81], 4}, {NULL}};
const Segment SGI28559[] = {{&S32130[81], 3}, {NULL}};
const Segment SGI28560[] = {{&S32131[38], 10}, {NULL}};
const Segment SGI28561[] = {{&S32131[31], 6}, {NULL}};
const Segment SGI28562[] = {{&S32132[38], 10}, {NULL}};
const Segment SGI28563[] = {{&S32132[31], 6}, {NULL}};
const Segment SGI28564[] = {{&S32133[44], 14}, {NULL}};
const Segment SGI28565[] = {{&S32133[44], 25}, {NULL}};
const Segment SGI28566[] = {{&S32133[30], 6}, {NULL}};
const Segment SGI28567[] = {{&S32133[117], 3}, {NULL}};
const Segment SGI28568[] = {{&S32133[83], 5}, {NULL}};
const Segment SGI28569[] = {{&S32136[44], 14}, {NULL}};
const Segment SGI28570[] = {{&S32136[44], 25}, {NULL}};
const Segment SGI28571[] = {{&S32136[30], 6}, {NULL}};
const Segment SGI28572[] = {{&S32136[117], 3}, {NULL}};
const Segment SGI28573[] = {{&S32136[83], 5}, {NULL}};
const Segment SGI28574[] = {{&S32340[44], 9}, {NULL}};
const Segment SGI28575[] = {{&S32340[44], 21}, {NULL}};
const Segment SGI28576[] = {{&S32340[30], 6}, {NULL}};
const Segment SGI28577[] = {{&S32340[113], 3}, {NULL}};
const Segment SGI28578[] = {{&S32340[79], 5}, {NULL}};
const Segment SGI28579[] = {{&S32341[44], 13}, {NULL}};
const Segment SGI28580[] = {{&S32341[44], 53}, {NULL}};
const Segment SGI28581[] = {{&S32341[30], 6}, {NULL}};
const Segment SGI28582[] = {{&S32341[145], 3}, {NULL}};
const Segment SGI28583[] = {{&S32341[111], 5}, {NULL}};
const Segment SGI28584[] = {{&S32344[44], 13}, {NULL}};
const Segment SGI28585[] = {{&S32344[44], 52}, {NULL}};
const Segment SGI28586[] = {{&S32344[30], 6}, {NULL}};
const Segment SGI28587[] = {{&S32344[144], 3}, {NULL}};
const Segment SGI28588[] = {{&S32344[110], 5}, {NULL}};
const Segment SGI28589[] = {{&S32345[44], 13}, {NULL}};
const Segment SGI28590[] = {{&S32345[44], 25}, {NULL}};
const Segment SGI28591[] = {{&S32345[30], 6}, {NULL}};
const Segment SGI28592[] = {{&S32345[117], 3}, {NULL}};
const Segment SGI28593[] = {{&S32345[83], 5}, {NULL}};
const Segment SGI28594[] = {{&S32349[44], 12}, {NULL}};
const Segment SGI28595[] = {{&S32349[44], 32}, {NULL}};
const Segment SGI28596[] = {{&S32349[30], 6}, {NULL}};
const Segment SGI28597[] = {{&S32349[124], 3}, {NULL}};
const Segment SGI28598[] = {{&S32349[90], 5}, {NULL}};
const Segment SGI28599[] = {{&S32351[44], 13}, {NULL}};
const Segment SGI28600[] = {{&S32351[44], 53}, {NULL}};
const Segment SGI28601[] = {{&S32351[30], 6}, {NULL}};
const Segment SGI28602[] = {{&S32351[145], 3}, {NULL}};
const Segment SGI28603[] = {{&S32351[111], 5}, {NULL}};
const Segment SGI28604[] = {{&S32353[44], 7}, {NULL}};
const Segment SGI28605[] = {{&S32353[44], 19}, {NULL}};
const Segment SGI28606[] = {{&S32353[30], 6}, {NULL}};
const Segment SGI28607[] = {{&S32353[111], 3}, {NULL}};
const Segment SGI28608[] = {{&S32353[77], 5}, {NULL}};
const Segment SGI28609[] = {{&S32354[44], 13}, {NULL}};
const Segment SGI28610[] = {{&S32354[44], 52}, {NULL}};
const Segment SGI28611[] = {{&S32354[30], 6}, {NULL}};
const Segment SGI28612[] = {{&S32354[144], 3}, {NULL}};
const Segment SGI28613[] = {{&S32354[110], 5}, {NULL}};
const Segment SGI28614[] = {{&S27762[44], 12}, {NULL}};
const Segment SGI28615[] = {{&S27762[44], 23}, {NULL}};
const Segment SGI28616[] = {{&S27762[30], 6}, {NULL}};
const Segment SGI28617[] = {{&S27762[115], 3}, {NULL}};
const Segment SGI28618[] = {{&S27762[81], 5}, {NULL}};
const Segment SGI28619[] = {{&S32355[44], 13}, {NULL}};
const Segment SGI28620[] = {{&S32355[44], 52}, {NULL}};
const Segment SGI28621[] = {{&S32355[30], 6}, {NULL}};
const Segment SGI28622[] = {{&S32355[144], 3}, {NULL}};
const Segment SGI28623[] = {{&S32355[110], 5}, {NULL}};
const Segment SGI28624[] = {{&S32356[44], 13}, {NULL}};
const Segment SGI28625[] = {{&S32356[44], 52}, {NULL}};
const Segment SGI28626[] = {{&S32356[30], 6}, {NULL}};
const Segment SGI28627[] = {{&S32356[144], 3}, {NULL}};
const Segment SGI28628[] = {{&S32356[110], 5}, {NULL}};
const Segment SGI28629[] = {{&S32359[44], 13}, {NULL}};
const Segment SGI28630[] = {{&S32359[44], 53}, {NULL}};
const Segment SGI28631[] = {{&S32359[30], 6}, {NULL}};
const Segment SGI28632[] = {{&S32359[145], 3}, {NULL}};
const Segment SGI28633[] = {{&S32359[111], 5}, {NULL}};
const Segment SGI28634[] = {{&S27771[44], 12}, {NULL}};
const Segment SGI28635[] = {{&S27771[44], 35}, {NULL}};
const Segment SGI28636[] = {{&S27771[30], 6}, {NULL}};
const Segment SGI28637[] = {{&S27771[127], 3}, {NULL}};
const Segment SGI28638[] = {{&S27771[93], 5}, {NULL}};
const Segment SGI28639[] = {{&S32361[44], 13}, {NULL}};
const Segment SGI28640[] = {{&S32361[44], 53}, {NULL}};
const Segment SGI28641[] = {{&S32361[30], 6}, {NULL}};
const Segment SGI28642[] = {{&S32361[145], 3}, {NULL}};
const Segment SGI28643[] = {{&S32361[111], 5}, {NULL}};
const Segment SGI28644[] = {{&S32370[44], 13}, {NULL}};
const Segment SGI28645[] = {{&S32370[44], 52}, {NULL}};
const Segment SGI28646[] = {{&S32370[30], 6}, {NULL}};
const Segment SGI28647[] = {{&S32370[144], 3}, {NULL}};
const Segment SGI28648[] = {{&S32370[110], 5}, {NULL}};
const Segment SGI28649[] = {{&S32372[44], 13}, {NULL}};
const Segment SGI28650[] = {{&S32372[44], 52}, {NULL}};
const Segment SGI28651[] = {{&S32372[30], 6}, {NULL}};
const Segment SGI28652[] = {{&S32372[144], 3}, {NULL}};
const Segment SGI28653[] = {{&S32372[110], 5}, {NULL}};
const Segment SGI28654[] = {{&S32375[44], 13}, {NULL}};
const Segment SGI28655[] = {{&S32375[44], 54}, {NULL}};
const Segment SGI28656[] = {{&S32375[30], 6}, {NULL}};
const Segment SGI28657[] = {{&S32375[146], 3}, {NULL}};
const Segment SGI28658[] = {{&S32375[112], 5}, {NULL}};
const Segment SGI28659[] = {{&S32376[44], 13}, {NULL}};
const Segment SGI28660[] = {{&S32376[44], 53}, {NULL}};
const Segment SGI28661[] = {{&S32376[30], 6}, {NULL}};
const Segment SGI28662[] = {{&S32376[145], 3}, {NULL}};
const Segment SGI28663[] = {{&S32376[111], 5}, {NULL}};
const Segment SGI28664[] = {{&S32378[44], 13}, {NULL}};
const Segment SGI28665[] = {{&S32378[44], 62}, {NULL}};
const Segment SGI28666[] = {{&S32378[30], 6}, {NULL}};
const Segment SGI28667[] = {{&S32378[155], 3}, {NULL}};
const Segment SGI28668[] = {{&S32378[120], 6}, {NULL}};
const Segment SGI28669[] = {{&S32378[175], 8}, {NULL}};
const Segment SGI28670[] = {{&S32380[44], 9}, {NULL}};
const Segment SGI28671[] = {{&S32380[44], 21}, {NULL}};
const Segment SGI28672[] = {{&S32380[30], 6}, {NULL}};
const Segment SGI28673[] = {{&S32380[113], 3}, {NULL}};
const Segment SGI28674[] = {{&S32380[79], 5}, {NULL}};
const Segment SGI28675[] = {{&S32381[30], 6}, {NULL}};
const Segment SGI28676[] = {{&S32381[92], 3}, {NULL}};
const Segment SGI28677[] = {{&S32381[57], 6}, {NULL}};
const Segment SGI28678[] = {{&S32383[44], 13}, {NULL}};
const Segment SGI28679[] = {{&S32383[44], 53}, {NULL}};
const Segment SGI28680[] = {{&S32383[30], 6}, {NULL}};
const Segment SGI28681[] = {{&S32383[145], 3}, {NULL}};
const Segment SGI28682[] = {{&S32383[111], 5}, {NULL}};
const Segment SGI28683[] = {{&S32384[44], 13}, {NULL}};
const Segment SGI28684[] = {{&S32384[44], 53}, {NULL}};
const Segment SGI28685[] = {{&S32384[30], 6}, {NULL}};
const Segment SGI28686[] = {{&S32384[145], 3}, {NULL}};
const Segment SGI28687[] = {{&S32384[111], 5}, {NULL}};
const Segment SGI28688[] = {{&S32386[44], 13}, {NULL}};
const Segment SGI28689[] = {{&S32386[44], 53}, {NULL}};
const Segment SGI28690[] = {{&S32386[30], 6}, {NULL}};
const Segment SGI28691[] = {{&S32386[145], 3}, {NULL}};
const Segment SGI28692[] = {{&S32386[111], 5}, {NULL}};
const Segment SGI28693[] = {{&S32389[44], 9}, {NULL}};
const Segment SGI28694[] = {{&S32389[44], 21}, {NULL}};
const Segment SGI28695[] = {{&S32389[30], 6}, {NULL}};
const Segment SGI28696[] = {{&S32389[113], 3}, {NULL}};
const Segment SGI28697[] = {{&S32389[79], 5}, {NULL}};
const Segment SGI28698[] = {{&S32390[44], 24}, {NULL}};
const Segment SGI28699[] = {{&S32390[44], 47}, {NULL}};
const Segment SGI28700[] = {{&S32390[30], 6}, {NULL}};
const Segment SGI28701[] = {{&S32390[139], 3}, {NULL}};
const Segment SGI28702[] = {{&S32390[105], 5}, {NULL}};
const Segment SGI28703[] = {{&S32395[44], 6}, {NULL}};
const Segment SGI28704[] = {{&S32395[44], 18}, {NULL}};
const Segment SGI28705[] = {{&S32395[30], 6}, {NULL}};
const Segment SGI28706[] = {{&S32395[110], 3}, {NULL}};
const Segment SGI28707[] = {{&S32395[76], 5}, {NULL}};
const Segment SGI28708[] = {{&S32396[37], 6}, {NULL}};
const Segment SGI28709[] = {{&S32396[137], 5}, {NULL}};
const Segment SGI28710[] = {{&S32402[44], 13}, {NULL}};
const Segment SGI28711[] = {{&S32402[44], 52}, {NULL}};
const Segment SGI28712[] = {{&S32402[30], 6}, {NULL}};
const Segment SGI28713[] = {{&S32402[144], 3}, {NULL}};
const Segment SGI28714[] = {{&S32402[110], 5}, {NULL}};
const Segment SGI28715[] = {{&S32474[44], 8}, {NULL}};
const Segment SGI28716[] = {{&S32474[44], 20}, {NULL}};
const Segment SGI28717[] = {{&S32474[30], 6}, {NULL}};
const Segment SGI28718[] = {{&S32474[112], 3}, {NULL}};
const Segment SGI28719[] = {{&S32474[78], 5}, {NULL}};
const Segment SGI28720[] = {{&S32475[43], 9}, {NULL}};
const Segment SGI28721[] = {{&S32475[43], 21}, {NULL}};
const Segment SGI28722[] = {{&S32475[30], 5}, {NULL}};
const Segment SGI28723[] = {{&S32475[112], 3}, {NULL}};
const Segment SGI28724[] = {{&S32475[78], 5}, {NULL}};
const Segment SGI28725[] = {{&S32477[44], 9}, {NULL}};
const Segment SGI28726[] = {{&S32477[44], 21}, {NULL}};
const Segment SGI28727[] = {{&S32477[30], 6}, {NULL}};
const Segment SGI28728[] = {{&S32477[113], 3}, {NULL}};
const Segment SGI28729[] = {{&S32477[79], 5}, {NULL}};
const Segment SGI28730[] = {{&S32478[43], 9}, {NULL}};
const Segment SGI28731[] = {{&S32478[43], 21}, {NULL}};
const Segment SGI28732[] = {{&S32478[112], 3}, {NULL}};
const Segment SGI28733[] = {{&S32478[78], 5}, {NULL}};
const Segment SGI28734[] = {{&S32479[44], 8}, {NULL}};
const Segment SGI28735[] = {{&S32479[44], 20}, {NULL}};
const Segment SGI28736[] = {{&S32479[30], 6}, {NULL}};
const Segment SGI28737[] = {{&S32479[112], 3}, {NULL}};
const Segment SGI28738[] = {{&S32479[78], 5}, {NULL}};
const Segment SGI28739[] = {{&S32480[44], 9}, {NULL}};
const Segment SGI28740[] = {{&S32480[44], 21}, {NULL}};
const Segment SGI28741[] = {{&S32480[30], 6}, {NULL}};
const Segment SGI28742[] = {{&S32480[113], 3}, {NULL}};
const Segment SGI28743[] = {{&S32480[79], 5}, {NULL}};
const Segment SGI28744[] = {{&S32483[38], 9}, {NULL}};
const Segment SGI28745[] = {{&S32483[31], 6}, {NULL}};
const Segment SGI28746[] = {{&S32484[44], 9}, {NULL}};
const Segment SGI28747[] = {{&S32484[44], 20}, {NULL}};
const Segment SGI28748[] = {{&S32484[30], 6}, {NULL}};
const Segment SGI28749[] = {{&S32484[112], 3}, {NULL}};
const Segment SGI28750[] = {{&S32484[78], 5}, {NULL}};
const Segment SGI28751[] = {{&S21976[44], 7}, {NULL}};
const Segment SGI28752[] = {{&S21976[44], 25}, {NULL}};
const Segment SGI28753[] = {{&S21976[30], 6}, {NULL}};
const Segment SGI28754[] = {{&S21976[57], 3}, {NULL}};
const Segment SGI28755[] = {{&S21976[83], 5}, {NULL}};
const Segment SGI28756[] = {{&S21978[44], 19}, {NULL}};
const Segment SGI28757[] = {{&S21978[44], 37}, {NULL}};
const Segment SGI28758[] = {{&S21978[30], 6}, {NULL}};
const Segment SGI28759[] = {{&S21978[69], 3}, {NULL}};
const Segment SGI28760[] = {{&S21978[95], 5}, {NULL}};
const Segment SGI28761[] = {{&S21979[44], 19}, {NULL}};
const Segment SGI28762[] = {{&S21979[44], 37}, {NULL}};
const Segment SGI28763[] = {{&S21979[30], 6}, {NULL}};
const Segment SGI28764[] = {{&S21979[69], 3}, {NULL}};
const Segment SGI28765[] = {{&S21979[95], 5}, {NULL}};
const Segment SGI28766[] = {{&S21980[44], 21}, {NULL}};
const Segment SGI28767[] = {{&S21980[44], 39}, {NULL}};
const Segment SGI28768[] = {{&S21980[30], 6}, {NULL}};
const Segment SGI28769[] = {{&S21980[71], 3}, {NULL}};
const Segment SGI28770[] = {{&S21980[97], 5}, {NULL}};
const Segment SGI28771[] = {{&S21981[44], 19}, {NULL}};
const Segment SGI28772[] = {{&S21981[44], 37}, {NULL}};
const Segment SGI28773[] = {{&S21981[30], 6}, {NULL}};
const Segment SGI28774[] = {{&S21981[69], 3}, {NULL}};
const Segment SGI28775[] = {{&S21981[95], 5}, {NULL}};
const Segment SGI28776[] = {{&S21982[44], 7}, {NULL}};
const Segment SGI28777[] = {{&S21982[44], 25}, {NULL}};
const Segment SGI28778[] = {{&S21982[30], 6}, {NULL}};
const Segment SGI28779[] = {{&S21982[57], 3}, {NULL}};
const Segment SGI28780[] = {{&S21982[83], 5}, {NULL}};
const Segment SGI28781[] = {{&S21983[44], 19}, {NULL}};
const Segment SGI28782[] = {{&S21983[44], 37}, {NULL}};
const Segment SGI28783[] = {{&S21983[30], 6}, {NULL}};
const Segment SGI28784[] = {{&S21983[69], 3}, {NULL}};
const Segment SGI28785[] = {{&S21983[95], 5}, {NULL}};
const Segment SGI28786[] = {{&S21984[38], 7}, {NULL}};
const Segment SGI28787[] = {{&S21984[31], 6}, {NULL}};
const Segment SGI28788[] = {{&S21985[44], 19}, {NULL}};
const Segment SGI28789[] = {{&S21985[44], 37}, {NULL}};
const Segment SGI28790[] = {{&S21985[30], 6}, {NULL}};
const Segment SGI28791[] = {{&S21985[69], 3}, {NULL}};
const Segment SGI28792[] = {{&S21985[95], 5}, {NULL}};
const Segment SGI28793[] = {{&S21986[44], 19}, {NULL}};
const Segment SGI28794[] = {{&S21986[44], 37}, {NULL}};
const Segment SGI28795[] = {{&S21986[30], 6}, {NULL}};
const Segment SGI28796[] = {{&S21986[69], 3}, {NULL}};
const Segment SGI28797[] = {{&S21986[95], 5}, {NULL}};
const Segment SGI28798[] = {{&S21988[44], 21}, {NULL}};
const Segment SGI28799[] = {{&S21988[44], 39}, {NULL}};
const Segment SGI28800[] = {{&S21988[30], 6}, {NULL}};
const Segment SGI28801[] = {{&S21988[71], 3}, {NULL}};
const Segment SGI28802[] = {{&S21988[97], 5}, {NULL}};
const Segment SGI28803[] = {{&S21989[44], 20}, {NULL}};
const Segment SGI28804[] = {{&S21989[44], 38}, {NULL}};
const Segment SGI28805[] = {{&S21989[30], 6}, {NULL}};
const Segment SGI28806[] = {{&S21989[70], 3}, {NULL}};
const Segment SGI28807[] = {{&S21989[96], 5}, {NULL}};
const Segment SGI28808[] = {{&S21990[44], 7}, {NULL}};
const Segment SGI28809[] = {{&S21990[44], 25}, {NULL}};
const Segment SGI28810[] = {{&S21990[30], 6}, {NULL}};
const Segment SGI28811[] = {{&S21990[57], 3}, {NULL}};
const Segment SGI28812[] = {{&S21990[83], 5}, {NULL}};
const Segment SGI28813[] = {{&S21991[44], 21}, {NULL}};
const Segment SGI28814[] = {{&S21991[44], 39}, {NULL}};
const Segment SGI28815[] = {{&S21991[30], 6}, {NULL}};
const Segment SGI28816[] = {{&S21991[71], 3}, {NULL}};
const Segment SGI28817[] = {{&S21991[97], 5}, {NULL}};
const Segment SGI28818[] = {{&S21992[44], 20}, {NULL}};
const Segment SGI28819[] = {{&S21992[44], 38}, {NULL}};
const Segment SGI28820[] = {{&S21992[30], 6}, {NULL}};
const Segment SGI28821[] = {{&S21992[70], 3}, {NULL}};
const Segment SGI28822[] = {{&S21992[96], 5}, {NULL}};
const Segment SGI28823[] = {{&S21994[44], 7}, {NULL}};
const Segment SGI28824[] = {{&S21994[44], 25}, {NULL}};
const Segment SGI28825[] = {{&S21994[30], 6}, {NULL}};
const Segment SGI28826[] = {{&S21994[57], 3}, {NULL}};
const Segment SGI28827[] = {{&S21994[83], 5}, {NULL}};
const Segment SGI28828[] = {{&S21995[44], 7}, {NULL}};
const Segment SGI28829[] = {{&S21995[44], 25}, {NULL}};
const Segment SGI28830[] = {{&S21995[30], 6}, {NULL}};
const Segment SGI28831[] = {{&S21995[57], 3}, {NULL}};
const Segment SGI28832[] = {{&S21995[83], 5}, {NULL}};
const Segment SGI28833[] = {{&S21996[38], 7}, {NULL}};
const Segment SGI28834[] = {{&S21996[31], 6}, {NULL}};
const Segment SGI28835[] = {{&S21997[44], 7}, {NULL}};
const Segment SGI28836[] = {{&S21997[44], 25}, {NULL}};
const Segment SGI28837[] = {{&S21997[30], 6}, {NULL}};
const Segment SGI28838[] = {{&S21997[57], 3}, {NULL}};
const Segment SGI28839[] = {{&S21997[83], 5}, {NULL}};
const Segment SGI28840[] = {{&S21999[44], 7}, {NULL}};
const Segment SGI28841[] = {{&S21999[44], 25}, {NULL}};
const Segment SGI28842[] = {{&S21999[30], 6}, {NULL}};
const Segment SGI28843[] = {{&S21999[86], 4}, {NULL}};
const Segment SGI28844[] = {{&S21999[86], 3}, {NULL}};
const Segment SGI28845[] = {{&S22000[44], 20}, {NULL}};
const Segment SGI28846[] = {{&S22000[44], 38}, {NULL}};
const Segment SGI28847[] = {{&S22000[30], 6}, {NULL}};
const Segment SGI28848[] = {{&S22000[70], 3}, {NULL}};
const Segment SGI28849[] = {{&S22000[96], 5}, {NULL}};
const Segment SGI28850[] = {{&S22001[44], 7}, {NULL}};
const Segment SGI28851[] = {{&S22001[44], 25}, {NULL}};
const Segment SGI28852[] = {{&S22001[30], 6}, {NULL}};
const Segment SGI28853[] = {{&S22001[86], 4}, {NULL}};
const Segment SGI28854[] = {{&S22001[86], 3}, {NULL}};
const Segment SGI28855[] = {{&S32806[44], 10}, {NULL}};
const Segment SGI28856[] = {{&S32806[44], 27}, {NULL}};
const Segment SGI28857[] = {{&S32806[30], 6}, {NULL}};
const Segment SGI28858[] = {{&S32806[119], 3}, {NULL}};
const Segment SGI28859[] = {{&S32806[85], 5}, {NULL}};
const Segment SGI28860[] = {{&S32807[44], 7}, {NULL}};
const Segment SGI28861[] = {{&S32807[44], 27}, {NULL}};
const Segment SGI28862[] = {{&S32807[30], 6}, {NULL}};
const Segment SGI28863[] = {{&S32807[119], 3}, {NULL}};
const Segment SGI28864[] = {{&S32807[85], 5}, {NULL}};
const Segment SGI28865[] = {{&S23145[44], 18}, {NULL}};
const Segment SGI28866[] = {{&S23145[44], 29}, {NULL}};
const Segment SGI28867[] = {{&S23145[30], 6}, {NULL}};
const Segment SGI28868[] = {{&S23145[121], 3}, {NULL}};
const Segment SGI28869[] = {{&S23145[87], 5}, {NULL}};
const Segment SGI28870[] = {{&S32808[44], 17}, {NULL}};
const Segment SGI28871[] = {{&S32808[44], 34}, {NULL}};
const Segment SGI28872[] = {{&S32808[30], 6}, {NULL}};
const Segment SGI28873[] = {{&S32808[126], 3}, {NULL}};
const Segment SGI28874[] = {{&S32808[92], 5}, {NULL}};
const Segment SGI28875[] = {{&S23146[44], 9}, {NULL}};
const Segment SGI28876[] = {{&S23146[44], 20}, {NULL}};
const Segment SGI28877[] = {{&S23146[30], 6}, {NULL}};
const Segment SGI28878[] = {{&S23146[112], 3}, {NULL}};
const Segment SGI28879[] = {{&S23146[78], 5}, {NULL}};
const Segment SGI28880[] = {{&S32809[44], 17}, {NULL}};
const Segment SGI28881[] = {{&S32809[44], 34}, {NULL}};
const Segment SGI28882[] = {{&S32809[30], 6}, {NULL}};
const Segment SGI28883[] = {{&S32809[126], 3}, {NULL}};
const Segment SGI28884[] = {{&S32809[92], 5}, {NULL}};
const Segment SGI28885[] = {{&S23148[42], 18}, {NULL}};
const Segment SGI28886[] = {{&S23148[30], 6}, {NULL}};
const Segment SGI28887[] = {{&S23148[119], 3}, {NULL}};
const Segment SGI28888[] = {{&S23148[85], 5}, {NULL}};
const Segment SGI28889[] = {{&S32813[42], 10}, {NULL}};
const Segment SGI28890[] = {{&S32813[30], 6}, {NULL}};
const Segment SGI28891[] = {{&S32813[117], 3}, {NULL}};
const Segment SGI28892[] = {{&S32813[83], 5}, {NULL}};
const Segment SGI28893[] = {{&S23152[44], 9}, {NULL}};
const Segment SGI28894[] = {{&S23152[44], 20}, {NULL}};
const Segment SGI28895[] = {{&S23152[30], 6}, {NULL}};
const Segment SGI28896[] = {{&S23152[112], 3}, {NULL}};
const Segment SGI28897[] = {{&S23152[78], 5}, {NULL}};
const Segment SGI28898[] = {{&S32816[38], 17}, {NULL}};
const Segment SGI28899[] = {{&S32816[31], 6}, {NULL}};
const Segment SGI28900[] = {{&S23153[38], 9}, {NULL}};
const Segment SGI28901[] = {{&S23153[31], 6}, {NULL}};
const Segment SGI28902[] = {{&S32817[44], 7}, {NULL}};
const Segment SGI28903[] = {{&S32817[44], 24}, {NULL}};
const Segment SGI28904[] = {{&S32817[30], 6}, {NULL}};
const Segment SGI28905[] = {{&S32817[116], 3}, {NULL}};
const Segment SGI28906[] = {{&S32817[82], 5}, {NULL}};
const Segment SGI28907[] = {{&S32818[44], 10}, {NULL}};
const Segment SGI28908[] = {{&S32818[44], 30}, {NULL}};
const Segment SGI28909[] = {{&S32818[30], 6}, {NULL}};
const Segment SGI28910[] = {{&S32818[122], 3}, {NULL}};
const Segment SGI28911[] = {{&S32818[88], 5}, {NULL}};
const Segment SGI28912[] = {{&S32819[44], 17}, {NULL}};
const Segment SGI28913[] = {{&S32819[44], 37}, {NULL}};
const Segment SGI28914[] = {{&S32819[30], 6}, {NULL}};
const Segment SGI28915[] = {{&S32819[129], 3}, {NULL}};
const Segment SGI28916[] = {{&S32819[95], 5}, {NULL}};
const Segment SGI28917[] = {{&S32820[44], 7}, {NULL}};
const Segment SGI28918[] = {{&S32820[44], 24}, {NULL}};
const Segment SGI28919[] = {{&S32820[30], 6}, {NULL}};
const Segment SGI28920[] = {{&S32820[116], 3}, {NULL}};
const Segment SGI28921[] = {{&S32820[82], 5}, {NULL}};
const Segment SGI28922[] = {{&S32821[44], 9}, {NULL}};
const Segment SGI28923[] = {{&S32821[44], 26}, {NULL}};
const Segment SGI28924[] = {{&S32821[30], 6}, {NULL}};
const Segment SGI28925[] = {{&S32821[118], 3}, {NULL}};
const Segment SGI28926[] = {{&S32821[84], 5}, {NULL}};
const Segment SGI28927[] = {{&S32822[44], 17}, {NULL}};
const Segment SGI28928[] = {{&S32822[44], 34}, {NULL}};
const Segment SGI28929[] = {{&S32822[30], 6}, {NULL}};
const Segment SGI28930[] = {{&S32822[126], 3}, {NULL}};
const Segment SGI28931[] = {{&S32822[92], 5}, {NULL}};
const Segment SGI28932[] = {{&S23171[44], 9}, {NULL}};
const Segment SGI28933[] = {{&S23171[44], 20}, {NULL}};
const Segment SGI28934[] = {{&S23171[30], 6}, {NULL}};
const Segment SGI28935[] = {{&S23171[114], 3}, {NULL}};
const Segment SGI28936[] = {{&S23171[78], 7}, {NULL}};
const Segment SGI28937[] = {{&S23171[134], 8}, {NULL}};
const Segment SGI28938[] = {{&S23172[42], 9}, {NULL}};
const Segment SGI28939[] = {{&S23172[30], 6}, {NULL}};
const Segment SGI28940[] = {{&S23172[110], 3}, {NULL}};
const Segment SGI28941[] = {{&S23172[76], 5}, {NULL}};
const Segment SGI28942[] = {{&S23175[44], 9}, {NULL}};
const Segment SGI28943[] = {{&S23175[44], 20}, {NULL}};
const Segment SGI28944[] = {{&S23175[30], 6}, {NULL}};
const Segment SGI28945[] = {{&S23175[81], 4}, {NULL}};
const Segment SGI28946[] = {{&S23175[81], 3}, {NULL}};
const Segment SGI28947[] = {{&S23176[44], 9}, {NULL}};
const Segment SGI28948[] = {{&S23176[44], 26}, {NULL}};
const Segment SGI28949[] = {{&S23176[30], 6}, {NULL}};
const Segment SGI28950[] = {{&S23176[119], 3}, {NULL}};
const Segment SGI28951[] = {{&S23176[84], 6}, {NULL}};
const Segment SGI28952[] = {{&S23177[44], 9}, {NULL}};
const Segment SGI28953[] = {{&S23177[44], 20}, {NULL}};
const Segment SGI28954[] = {{&S23177[30], 6}, {NULL}};
const Segment SGI28955[] = {{&S23177[81], 6}, {NULL}};
const Segment SGI28956[] = {{&S32913[44], 14}, {NULL}};
const Segment SGI28957[] = {{&S32913[44], 26}, {NULL}};
const Segment SGI28958[] = {{&S32913[30], 6}, {NULL}};
const Segment SGI28959[] = {{&S32913[118], 3}, {NULL}};
const Segment SGI28960[] = {{&S32913[84], 5}, {NULL}};
const Segment SGI28961[] = {{&S32914[44], 9}, {NULL}};
const Segment SGI28962[] = {{&S32914[44], 21}, {NULL}};
const Segment SGI28963[] = {{&S32914[30], 6}, {NULL}};
const Segment SGI28964[] = {{&S32914[113], 3}, {NULL}};
const Segment SGI28965[] = {{&S32914[79], 5}, {NULL}};
const Segment SGI28966[] = {{&S32915[44], 14}, {NULL}};
const Segment SGI28967[] = {{&S32915[44], 26}, {NULL}};
const Segment SGI28968[] = {{&S32915[30], 6}, {NULL}};
const Segment SGI28969[] = {{&S32915[118], 3}, {NULL}};
const Segment SGI28970[] = {{&S32915[84], 5}, {NULL}};
const Segment SGI28971[] = {{&S32917[44], 10}, {NULL}};
const Segment SGI28972[] = {{&S32917[44], 22}, {NULL}};
const Segment SGI28973[] = {{&S32917[30], 6}, {NULL}};
const Segment SGI28974[] = {{&S32917[114], 3}, {NULL}};
const Segment SGI28975[] = {{&S32917[80], 5}, {NULL}};
const Segment SGI28976[] = {{&S32919[44], 11}, {NULL}};
const Segment SGI28977[] = {{&S32919[44], 23}, {NULL}};
const Segment SGI28978[] = {{&S32919[30], 6}, {NULL}};
const Segment SGI28979[] = {{&S32919[115], 3}, {NULL}};
const Segment SGI28980[] = {{&S32919[81], 5}, {NULL}};
const Segment SGI28981[] = {{&S32921[38], 9}, {NULL}};
const Segment SGI28982[] = {{&S32921[31], 6}, {NULL}};
const Segment SGI28983[] = {{&S32923[38], 10}, {NULL}};
const Segment SGI28984[] = {{&S32923[31], 6}, {NULL}};
const Segment SGI28985[] = {{&S32925[38], 11}, {NULL}};
const Segment SGI28986[] = {{&S32925[31], 6}, {NULL}};
const Segment SGI28987[] = {{&S21671[44], 20}, {NULL}};
const Segment SGI28988[] = {{&S21671[44], 31}, {NULL}};
const Segment SGI28989[] = {{&S21671[30], 6}, {NULL}};
const Segment SGI28990[] = {{&S21671[123], 3}, {NULL}};
const Segment SGI28991[] = {{&S21671[89], 5}, {NULL}};
const Segment SGI28992[] = {{&S21672[44], 20}, {NULL}};
const Segment SGI28993[] = {{&S21672[44], 31}, {NULL}};
const Segment SGI28994[] = {{&S21672[30], 6}, {NULL}};
const Segment SGI28995[] = {{&S21672[123], 3}, {NULL}};
const Segment SGI28996[] = {{&S21672[89], 5}, {NULL}};
const Segment SGI28997[] = {{&S21673[44], 20}, {NULL}};
const Segment SGI28998[] = {{&S21673[44], 31}, {NULL}};
const Segment SGI28999[] = {{&S21673[30], 6}, {NULL}};
const Segment SGI29000[] = {{&S21673[123], 3}, {NULL}};
const Segment SGI29001[] = {{&S21673[89], 5}, {NULL}};
const Segment SGI29002[] = {{&S21674[44], 20}, {NULL}};
const Segment SGI29003[] = {{&S21674[44], 31}, {NULL}};
const Segment SGI29004[] = {{&S21674[30], 6}, {NULL}};
const Segment SGI29005[] = {{&S21674[123], 3}, {NULL}};
const Segment SGI29006[] = {{&S21674[89], 5}, {NULL}};
const Segment SGI29007[] = {{&S21675[44], 14}, {NULL}};
const Segment SGI29008[] = {{&S21675[44], 25}, {NULL}};
const Segment SGI29009[] = {{&S21675[30], 6}, {NULL}};
const Segment SGI29010[] = {{&S21675[117], 3}, {NULL}};
const Segment SGI29011[] = {{&S21675[83], 5}, {NULL}};
const Segment SGI29012[] = {{&S21676[44], 14}, {NULL}};
const Segment SGI29013[] = {{&S21676[44], 25}, {NULL}};
const Segment SGI29014[] = {{&S21676[30], 6}, {NULL}};
const Segment SGI29015[] = {{&S21676[117], 3}, {NULL}};
const Segment SGI29016[] = {{&S21676[83], 5}, {NULL}};
const Segment SGI29017[] = {{&S21677[44], 25}, {NULL}};
const Segment SGI29018[] = {{&S21677[44], 36}, {NULL}};
const Segment SGI29019[] = {{&S21677[30], 6}, {NULL}};
const Segment SGI29020[] = {{&S21677[128], 3}, {NULL}};
const Segment SGI29021[] = {{&S21677[94], 5}, {NULL}};
const Segment SGI29022[] = {{&S21678[44], 24}, {NULL}};
const Segment SGI29023[] = {{&S21678[44], 35}, {NULL}};
const Segment SGI29024[] = {{&S21678[30], 6}, {NULL}};
const Segment SGI29025[] = {{&S21678[127], 3}, {NULL}};
const Segment SGI29026[] = {{&S21678[93], 5}, {NULL}};
const Segment SGI29027[] = {{&S21679[44], 25}, {NULL}};
const Segment SGI29028[] = {{&S21679[44], 36}, {NULL}};
const Segment SGI29029[] = {{&S21679[30], 6}, {NULL}};
const Segment SGI29030[] = {{&S21679[128], 3}, {NULL}};
const Segment SGI29031[] = {{&S21679[94], 5}, {NULL}};
const Segment SGI29032[] = {{&S21680[38], 20}, {NULL}};
const Segment SGI29033[] = {{&S21680[31], 6}, {NULL}};
const Segment SGI29034[] = {{&S21681[44], 25}, {NULL}};
const Segment SGI29035[] = {{&S21681[44], 36}, {NULL}};
const Segment SGI29036[] = {{&S21681[30], 6}, {NULL}};
const Segment SGI29037[] = {{&S21681[128], 3}, {NULL}};
const Segment SGI29038[] = {{&S21681[94], 5}, {NULL}};
const Segment SGI29039[] = {{&S21682[44], 20}, {NULL}};
const Segment SGI29040[] = {{&S21682[44], 31}, {NULL}};
const Segment SGI29041[] = {{&S21682[30], 6}, {NULL}};
const Segment SGI29042[] = {{&S21682[92], 4}, {NULL}};
const Segment SGI29043[] = {{&S21682[92], 3}, {NULL}};
const Segment SGI29044[] = {{&S21683[44], 25}, {NULL}};
const Segment SGI29045[] = {{&S21683[44], 36}, {NULL}};
const Segment SGI29046[] = {{&S21683[30], 6}, {NULL}};
const Segment SGI29047[] = {{&S21683[128], 3}, {NULL}};
const Segment SGI29048[] = {{&S21683[94], 5}, {NULL}};
const Segment SGI29049[] = {{&S21684[44], 25}, {NULL}};
const Segment SGI29050[] = {{&S21684[44], 36}, {NULL}};
const Segment SGI29051[] = {{&S21684[30], 6}, {NULL}};
const Segment SGI29052[] = {{&S21684[128], 3}, {NULL}};
const Segment SGI29053[] = {{&S21684[94], 5}, {NULL}};
const Segment SGI29054[] = {{&S21685[44], 25}, {NULL}};
const Segment SGI29055[] = {{&S21685[44], 36}, {NULL}};
const Segment SGI29056[] = {{&S21685[30], 6}, {NULL}};
const Segment SGI29057[] = {{&S21685[128], 3}, {NULL}};
const Segment SGI29058[] = {{&S21685[94], 5}, {NULL}};
const Segment SGI29059[] = {{&S21686[44], 25}, {NULL}};
const Segment SGI29060[] = {{&S21686[44], 36}, {NULL}};
const Segment SGI29061[] = {{&S21686[30], 6}, {NULL}};
const Segment SGI29062[] = {{&S21686[128], 3}, {NULL}};
const Segment SGI29063[] = {{&S21686[94], 5}, {NULL}};
const Segment SGI29064[] = {{&S21687[38], 14}, {NULL}};
const Segment SGI29065[] = {{&S21687[31], 6}, {NULL}};
const Segment SGI29066[] = {{&S21688[44], 25}, {NULL}};
const Segment SGI29067[] = {{&S21688[44], 36}, {NULL}};
const Segment SGI29068[] = {{&S21688[30], 6}, {NULL}};
const Segment SGI29069[] = {{&S21688[128], 3}, {NULL}};
const Segment SGI29070[] = {{&S21688[94], 5}, {NULL}};
const Segment SGI29071[] = {{&S21689[44], 25}, {NULL}};
const Segment SGI29072[] = {{&S21689[44], 36}, {NULL}};
const Segment SGI29073[] = {{&S21689[30], 6}, {NULL}};
const Segment SGI29074[] = {{&S21689[128], 3}, {NULL}};
const Segment SGI29075[] = {{&S21689[94], 5}, {NULL}};
const Segment SGI29076[] = {{&S21690[44], 11}, {NULL}};
const Segment SGI29077[] = {{&S21690[44], 22}, {NULL}};
const Segment SGI29078[] = {{&S21690[30], 6}, {NULL}};
const Segment SGI29079[] = {{&S21690[83], 4}, {NULL}};
const Segment SGI29080[] = {{&S21690[83], 3}, {NULL}};
const Segment SGI29081[] = {{&S21691[38], 20}, {NULL}};
const Segment SGI29082[] = {{&S21691[31], 6}, {NULL}};
const Segment SGI29083[] = {{&S21692[44], 11}, {NULL}};
const Segment SGI29084[] = {{&S21692[44], 22}, {NULL}};
const Segment SGI29085[] = {{&S21692[30], 6}, {NULL}};
const Segment SGI29086[] = {{&S21692[114], 3}, {NULL}};
const Segment SGI29087[] = {{&S21692[80], 5}, {NULL}};
const Segment SGI29088[] = {{&S21693[44], 11}, {NULL}};
const Segment SGI29089[] = {{&S21693[44], 22}, {NULL}};
const Segment SGI29090[] = {{&S21693[30], 6}, {NULL}};
const Segment SGI29091[] = {{&S21693[83], 6}, {NULL}};
const Segment SGI29092[] = {{&S21694[44], 11}, {NULL}};
const Segment SGI29093[] = {{&S21694[44], 22}, {NULL}};
const Segment SGI29094[] = {{&S21694[30], 6}, {NULL}};
const Segment SGI29095[] = {{&S21694[114], 3}, {NULL}};
const Segment SGI29096[] = {{&S21694[80], 5}, {NULL}};
const Segment SGI29097[] = {{&S21695[44], 20}, {NULL}};
const Segment SGI29098[] = {{&S21695[44], 31}, {NULL}};
const Segment SGI29099[] = {{&S21695[30], 6}, {NULL}};
const Segment SGI29100[] = {{&S21695[92], 6}, {NULL}};
const Segment SGI29101[] = {{&S21696[44], 25}, {NULL}};
const Segment SGI29102[] = {{&S21696[44], 36}, {NULL}};
const Segment SGI29103[] = {{&S21696[30], 6}, {NULL}};
const Segment SGI29104[] = {{&S21696[128], 3}, {NULL}};
const Segment SGI29105[] = {{&S21696[94], 5}, {NULL}};
const Segment SGI29106[] = {{&S21697[44], 25}, {NULL}};
const Segment SGI29107[] = {{&S21697[44], 36}, {NULL}};
const Segment SGI29108[] = {{&S21697[30], 6}, {NULL}};
const Segment SGI29109[] = {{&S21697[128], 3}, {NULL}};
const Segment SGI29110[] = {{&S21697[94], 5}, {NULL}};
const Segment SGI29111[] = {{&S29058[44], 6}, {NULL}};
const Segment SGI29112[] = {{&S29058[44], 17}, {NULL}};
const Segment SGI29113[] = {{&S29058[30], 6}, {NULL}};
const Segment SGI29114[] = {{&S29058[109], 3}, {NULL}};
const Segment SGI29115[] = {{&S29058[75], 5}, {NULL}};
const Segment SGI29116[] = {{&S22591[44], 10}, {NULL}};
const Segment SGI29117[] = {{&S22591[44], 27}, {NULL}};
const Segment SGI29118[] = {{&S22591[30], 6}, {NULL}};
const Segment SGI29119[] = {{&S22591[119], 3}, {NULL}};
const Segment SGI29120[] = {{&S22591[85], 5}, {NULL}};
const Segment SGI29121[] = {{&S22594[44], 11}, {NULL}};
const Segment SGI29122[] = {{&S22594[44], 28}, {NULL}};
const Segment SGI29123[] = {{&S22594[30], 6}, {NULL}};
const Segment SGI29124[] = {{&S22594[120], 3}, {NULL}};
const Segment SGI29125[] = {{&S22594[86], 5}, {NULL}};
const Segment SGI29126[] = {{&S22595[44], 11}, {NULL}};
const Segment SGI29127[] = {{&S22595[44], 28}, {NULL}};
const Segment SGI29128[] = {{&S22595[30], 6}, {NULL}};
const Segment SGI29129[] = {{&S22595[120], 3}, {NULL}};
const Segment SGI29130[] = {{&S22595[86], 5}, {NULL}};
const Segment SGI29131[] = {{&S23530[44], 7}, {NULL}};
const Segment SGI29132[] = {{&S23530[44], 18}, {NULL}};
const Segment SGI29133[] = {{&S23530[30], 6}, {NULL}};
const Segment SGI29134[] = {{&S23530[110], 3}, {NULL}};
const Segment SGI29135[] = {{&S22597[38], 10}, {NULL}};
const Segment SGI29136[] = {{&S22597[31], 6}, {NULL}};
const Segment SGI29137[] = {{&S23530[76], 5}, {NULL}};
const Segment SGI29138[] = {{&S22599[44], 31}, {NULL}};
const Segment SGI29139[] = {{&S22599[44], 48}, {NULL}};
const Segment SGI29140[] = {{&S22599[30], 6}, {NULL}};
const Segment SGI29141[] = {{&S22599[140], 3}, {NULL}};
const Segment SGI29142[] = {{&S22599[106], 5}, {NULL}};
const Segment SGI29143[] = {{&S22600[44], 31}, {NULL}};
const Segment SGI29144[] = {{&S22600[44], 48}, {NULL}};
const Segment SGI29145[] = {{&S22600[30], 6}, {NULL}};
const Segment SGI29146[] = {{&S22600[140], 3}, {NULL}};
const Segment SGI29147[] = {{&S22600[106], 5}, {NULL}};
const Segment SGI29148[] = {{&S22602[44], 10}, {NULL}};
const Segment SGI29149[] = {{&S22602[44], 27}, {NULL}};
const Segment SGI29150[] = {{&S22602[30], 6}, {NULL}};
const Segment SGI29151[] = {{&S23539[44], 7}, {NULL}};
const Segment SGI29152[] = {{&S23539[44], 19}, {NULL}};
const Segment SGI29153[] = {{&S22602[88], 4}, {NULL}};
const Segment SGI29154[] = {{&S23539[30], 6}, {NULL}};
const Segment SGI29155[] = {{&S22602[88], 3}, {NULL}};
const Segment SGI29156[] = {{&S23539[111], 3}, {NULL}};
const Segment SGI29157[] = {{&S23539[77], 5}, {NULL}};
const Segment SGI29158[] = {{&S22603[38], 11}, {NULL}};
const Segment SGI29159[] = {{&S22603[31], 6}, {NULL}};
const Segment SGI29160[] = {{&S22607[44], 10}, {NULL}};
const Segment SGI29161[] = {{&S22607[44], 27}, {NULL}};
const Segment SGI29162[] = {{&S22607[30], 6}, {NULL}};
const Segment SGI29163[] = {{&S22607[88], 6}, {NULL}};
const Segment SGI29164[] = {{&S22608[38], 11}, {NULL}};
const Segment SGI29165[] = {{&S22608[31], 6}, {NULL}};
const Segment SGI29166[] = {{&S23547[44], 7}, {NULL}};
const Segment SGI29167[] = {{&S23547[44], 19}, {NULL}};
const Segment SGI29168[] = {{&S23547[30], 6}, {NULL}};
const Segment SGI29169[] = {{&S23547[111], 3}, {NULL}};
const Segment SGI29170[] = {{&S23547[77], 5}, {NULL}};
const Segment SGI29171[] = {{&S23553[44], 7}, {NULL}};
const Segment SGI29172[] = {{&S23553[44], 21}, {NULL}};
const Segment SGI29173[] = {{&S23553[30], 6}, {NULL}};
const Segment SGI29174[] = {{&S23553[113], 3}, {NULL}};
const Segment SGI29175[] = {{&S23553[79], 5}, {NULL}};
const Segment SGI29176[] = {{&S27941[44], 12}, {NULL}};
const Segment SGI29177[] = {{&S27941[44], 28}, {NULL}};
const Segment SGI29178[] = {{&S27941[30], 6}, {NULL}};
const Segment SGI29179[] = {{&S27941[120], 3}, {NULL}};
const Segment SGI29180[] = {{&S27941[86], 5}, {NULL}};
const Segment SGI29181[] = {{&S27946[44], 12}, {NULL}};
const Segment SGI29182[] = {{&S27946[44], 28}, {NULL}};
const Segment SGI29183[] = {{&S27946[30], 6}, {NULL}};
const Segment SGI29184[] = {{&S27946[120], 3}, {NULL}};
const Segment SGI29185[] = {{&S27946[86], 5}, {NULL}};
const Segment SGI29186[] = {{&S29321[44], 5}, {NULL}};
const Segment SGI29187[] = {{&S29321[44], 19}, {NULL}};
const Segment SGI29188[] = {{&S29321[30], 6}, {NULL}};
const Segment SGI29189[] = {{&S29321[111], 3}, {NULL}};
const Segment SGI29190[] = {{&S29321[77], 5}, {NULL}};
const Segment SGI29191[] = {{&S23827[44], 12}, {NULL}};
const Segment SGI29192[] = {{&S23827[44], 23}, {NULL}};
const Segment SGI29193[] = {{&S23827[30], 6}, {NULL}};
const Segment SGI29194[] = {{&S23827[115], 3}, {NULL}};
const Segment SGI29195[] = {{&S23827[81], 5}, {NULL}};
const Segment SGI29196[] = {{&S23828[44], 23}, {NULL}};
const Segment SGI29197[] = {{&S23828[44], 34}, {NULL}};
const Segment SGI29198[] = {{&S23828[30], 6}, {NULL}};
const Segment SGI29199[] = {{&S23828[126], 3}, {NULL}};
const Segment SGI29200[] = {{&S23828[92], 5}, {NULL}};
const Segment SGI29201[] = {{&S23829[44], 23}, {NULL}};
const Segment SGI29202[] = {{&S23829[44], 34}, {NULL}};
const Segment SGI29203[] = {{&S23829[30], 6}, {NULL}};
const Segment SGI29204[] = {{&S23829[126], 3}, {NULL}};
const Segment SGI29205[] = {{&S29381[44], 12}, {NULL}};
const Segment SGI29206[] = {{&S23829[92], 5}, {NULL}};
const Segment SGI29207[] = {{&S29381[44], 24}, {NULL}};
const Segment SGI29208[] = {{&S29381[30], 6}, {NULL}};
const Segment SGI29209[] = {{&S29381[116], 3}, {NULL}};
const Segment SGI29210[] = {{&S29381[82], 5}, {NULL}};
const Segment SGI29211[] = {{&S23830[44], 21}, {NULL}};
const Segment SGI29212[] = {{&S23830[44], 32}, {NULL}};
const Segment SGI29213[] = {{&S23830[30], 6}, {NULL}};
const Segment SGI29214[] = {{&S23830[124], 3}, {NULL}};
const Segment SGI29215[] = {{&S23830[90], 5}, {NULL}};
const Segment SGI29216[] = {{&S23831[44], 18}, {NULL}};
const Segment SGI29217[] = {{&S23831[44], 29}, {NULL}};
const Segment SGI29218[] = {{&S23831[30], 6}, {NULL}};
const Segment SGI29219[] = {{&S23831[121], 3}, {NULL}};
const Segment SGI29220[] = {{&S23831[87], 5}, {NULL}};
const Segment SGI29221[] = {{&S23832[44], 18}, {NULL}};
const Segment SGI29222[] = {{&S23832[44], 29}, {NULL}};
const Segment SGI29223[] = {{&S23832[30], 6}, {NULL}};
const Segment SGI29224[] = {{&S23832[121], 3}, {NULL}};
const Segment SGI29225[] = {{&S23832[87], 5}, {NULL}};
const Segment SGI29226[] = {{&S23833[44], 22}, {NULL}};
const Segment SGI29227[] = {{&S23833[44], 33}, {NULL}};
const Segment SGI29228[] = {{&S23833[30], 6}, {NULL}};
const Segment SGI29229[] = {{&S23833[125], 3}, {NULL}};
const Segment SGI29230[] = {{&S23833[91], 5}, {NULL}};
const Segment SGI29231[] = {{&S23834[44], 18}, {NULL}};
const Segment SGI29232[] = {{&S23834[44], 29}, {NULL}};
const Segment SGI29233[] = {{&S23834[30], 6}, {NULL}};
const Segment SGI29234[] = {{&S23834[121], 3}, {NULL}};
const Segment SGI29235[] = {{&S23834[87], 5}, {NULL}};
const Segment SGI29236[] = {{&S23835[44], 23}, {NULL}};
const Segment SGI29237[] = {{&S23835[44], 34}, {NULL}};
const Segment SGI29238[] = {{&S23835[30], 6}, {NULL}};
const Segment SGI29239[] = {{&S23835[126], 3}, {NULL}};
const Segment SGI29240[] = {{&S23835[92], 5}, {NULL}};
const Segment SGI29241[] = {{&S23836[44], 23}, {NULL}};
const Segment SGI29242[] = {{&S23836[44], 34}, {NULL}};
const Segment SGI29243[] = {{&S23836[30], 6}, {NULL}};
const Segment SGI29244[] = {{&S23836[126], 3}, {NULL}};
const Segment SGI29245[] = {{&S23836[92], 5}, {NULL}};
const Segment SGI29246[] = {{&S29397[44], 10}, {NULL}};
const Segment SGI29247[] = {{&S29397[44], 31}, {NULL}};
const Segment SGI29248[] = {{&S29397[30], 6}, {NULL}};
const Segment SGI29249[] = {{&S23837[42], 18}, {NULL}};
const Segment SGI29250[] = {{&S29397[123], 3}, {NULL}};
const Segment SGI29251[] = {{&S23837[30], 6}, {NULL}};
const Segment SGI29252[] = {{&S29397[89], 5}, {NULL}};
const Segment SGI29253[] = {{&S23837[119], 3}, {NULL}};
const Segment SGI29254[] = {{&S23837[85], 5}, {NULL}};
const Segment SGI29255[] = {{&S23839[44], 23}, {NULL}};
const Segment SGI29256[] = {{&S23839[44], 34}, {NULL}};
const Segment SGI29257[] = {{&S23839[30], 6}, {NULL}};
const Segment SGI29258[] = {{&S23839[126], 3}, {NULL}};
const Segment SGI29259[] = {{&S23839[92], 5}, {NULL}};
const Segment SGI29260[] = {{&S23840[35], 1}, {NULL}};
const Segment SGI29261[] = {{&S23840[59], 13}, {NULL}};
const Segment SGI29262[] = {{&S23840[46], 6}, {NULL}};
const Segment SGI29263[] = {{&S23840[120], 3}, {NULL}};
const Segment SGI29264[] = {{&S23840[86], 5}, {NULL}};
const Segment SGI29265[] = {{&S23841[44], 18}, {NULL}};
const Segment SGI29266[] = {{&S23841[44], 29}, {NULL}};
const Segment SGI29267[] = {{&S23841[30], 6}, {NULL}};
const Segment SGI29268[] = {{&S23841[121], 3}, {NULL}};
const Segment SGI29269[] = {{&S23841[87], 5}, {NULL}};
const Segment SGI29270[] = {{&S23844[38], 18}, {NULL}};
const Segment SGI29271[] = {{&S23844[31], 6}, {NULL}};
const Segment SGI29272[] = {{&S23855[44], 21}, {NULL}};
const Segment SGI29273[] = {{&S23855[44], 32}, {NULL}};
const Segment SGI29274[] = {{&S23855[30], 6}, {NULL}};
const Segment SGI29275[] = {{&S23855[124], 3}, {NULL}};
const Segment SGI29276[] = {{&S23855[90], 5}, {NULL}};
const Segment SGI29277[] = {{&S29427[44], 26}, {NULL}};
const Segment SGI29278[] = {{&S29427[44], 38}, {NULL}};
const Segment SGI29279[] = {{&S29427[30], 6}, {NULL}};
const Segment SGI29280[] = {{&S29427[130], 3}, {NULL}};
const Segment SGI29281[] = {{&S29427[96], 5}, {NULL}};
const Segment SGI29282[] = {{&S23859[44], 18}, {NULL}};
const Segment SGI29283[] = {{&S23859[44], 29}, {NULL}};
const Segment SGI29284[] = {{&S23859[30], 6}, {NULL}};
const Segment SGI29285[] = {{&S23859[90], 4}, {NULL}};
const Segment SGI29286[] = {{&S23859[90], 3}, {NULL}};
const Segment SGI29287[] = {{&S29432[44], 37}, {NULL}};
const Segment SGI29288[] = {{&S29432[30], 6}, {NULL}};
const Segment SGI29289[] = {{&S29432[113], 5}, {NULL}};
const Segment SGI29290[] = {{&S23868[44], 18}, {NULL}};
const Segment SGI29291[] = {{&S23868[44], 29}, {NULL}};
const Segment SGI29292[] = {{&S23868[30], 6}, {NULL}};
const Segment SGI29293[] = {{&S23868[90], 6}, {NULL}};
const Segment SGI29294[] = {{&S33684[44], 7}, {NULL}};
const Segment SGI29295[] = {{&S33684[44], 19}, {NULL}};
const Segment SGI29296[] = {{&S33684[30], 6}, {NULL}};
const Segment SGI29297[] = {{&S33684[111], 3}, {NULL}};
const Segment SGI29298[] = {{&S33684[77], 5}, {NULL}};
const Segment SGI29299[] = {{&S33688[38], 7}, {NULL}};
const Segment SGI29300[] = {{&S33688[31], 6}, {NULL}};
const Segment SGI29301[] = {{&S24026[44], 101}, {NULL}};
const Segment SGI29302[] = {{&S24026[44], 112}, {NULL}};
const Segment SGI29303[] = {{&S24026[30], 6}, {NULL}};
const Segment SGI29304[] = {{&S24026[204], 3}, {NULL}};
const Segment SGI29305[] = {{&S24026[170], 5}, {NULL}};
const Segment SGI29306[] = {{&S33692[44], 7}, {NULL}};
const Segment SGI29307[] = {{&S33692[44], 19}, {NULL}};
const Segment SGI29308[] = {{&S33692[30], 6}, {NULL}};
const Segment SGI29309[] = {{&S33692[113], 3}, {NULL}};
const Segment SGI29310[] = {{&S33692[77], 7}, {NULL}};
const Segment SGI29311[] = {{&S33692[133], 8}, {NULL}};
const Segment SGI29312[] = {{&S33693[42], 7}, {NULL}};
const Segment SGI29313[] = {{&S33693[30], 6}, {NULL}};
const Segment SGI29314[] = {{&S33693[109], 3}, {NULL}};
const Segment SGI29315[] = {{&S33693[75], 5}, {NULL}};
const Segment SGI29316[] = {{&S33694[44], 7}, {NULL}};
const Segment SGI29317[] = {{&S33694[44], 22}, {NULL}};
const Segment SGI29318[] = {{&S33694[30], 6}, {NULL}};
const Segment SGI29319[] = {{&S33694[114], 3}, {NULL}};
const Segment SGI29320[] = {{&S33694[80], 5}, {NULL}};
const Segment SGI29321[] = {{&S24034[44], 17}, {NULL}};
const Segment SGI29322[] = {{&S24034[44], 28}, {NULL}};
const Segment SGI29323[] = {{&S24034[30], 6}, {NULL}};
const Segment SGI29324[] = {{&S24034[120], 3}, {NULL}};
const Segment SGI29325[] = {{&S24034[86], 5}, {NULL}};
const Segment SGI29326[] = {{&S24036[44], 18}, {NULL}};
const Segment SGI29327[] = {{&S24036[44], 29}, {NULL}};
const Segment SGI29328[] = {{&S24036[30], 6}, {NULL}};
const Segment SGI29329[] = {{&S24036[121], 3}, {NULL}};
const Segment SGI29330[] = {{&S24036[87], 5}, {NULL}};
const Segment SGI29331[] = {{&S24038[44], 21}, {NULL}};
const Segment SGI29332[] = {{&S24038[44], 32}, {NULL}};
const Segment SGI29333[] = {{&S24038[30], 6}, {NULL}};
const Segment SGI29334[] = {{&S24038[124], 3}, {NULL}};
const Segment SGI29335[] = {{&S24038[90], 5}, {NULL}};
const Segment SGI29336[] = {{&S33704[44], 10}, {NULL}};
const Segment SGI29337[] = {{&S33704[44], 22}, {NULL}};
const Segment SGI29338[] = {{&S33704[30], 6}, {NULL}};
const Segment SGI29339[] = {{&S33704[114], 3}, {NULL}};
const Segment SGI29340[] = {{&S33704[80], 5}, {NULL}};
const Segment SGI29341[] = {{&S24039[44], 21}, {NULL}};
const Segment SGI29342[] = {{&S24039[44], 32}, {NULL}};
const Segment SGI29343[] = {{&S24039[30], 6}, {NULL}};
const Segment SGI29344[] = {{&S24039[124], 3}, {NULL}};
const Segment SGI29345[] = {{&S24039[90], 5}, {NULL}};
const Segment SGI29346[] = {{&S24040[44], 17}, {NULL}};
const Segment SGI29347[] = {{&S24040[44], 28}, {NULL}};
const Segment SGI29348[] = {{&S24040[30], 6}, {NULL}};
const Segment SGI29349[] = {{&S24040[120], 3}, {NULL}};
const Segment SGI29350[] = {{&S24040[86], 5}, {NULL}};
const Segment SGI29351[] = {{&S24041[44], 18}, {NULL}};
const Segment SGI29352[] = {{&S24041[44], 29}, {NULL}};
const Segment SGI29353[] = {{&S24041[30], 6}, {NULL}};
const Segment SGI29354[] = {{&S24041[121], 3}, {NULL}};
const Segment SGI29355[] = {{&S24041[87], 5}, {NULL}};
const Segment SGI29356[] = {{&S33710[38], 10}, {NULL}};
const Segment SGI29357[] = {{&S33710[31], 6}, {NULL}};
const Segment SGI29358[] = {{&S24042[44], 18}, {NULL}};
const Segment SGI29359[] = {{&S24042[44], 29}, {NULL}};
const Segment SGI29360[] = {{&S24042[30], 6}, {NULL}};
const Segment SGI29361[] = {{&S24042[121], 3}, {NULL}};
const Segment SGI29362[] = {{&S24042[87], 5}, {NULL}};
const Segment SGI29363[] = {{&S33712[42], 10}, {NULL}};
const Segment SGI29364[] = {{&S33712[30], 6}, {NULL}};
const Segment SGI29365[] = {{&S33712[112], 3}, {NULL}};
const Segment SGI29366[] = {{&S33712[78], 5}, {NULL}};
const Segment SGI29367[] = {{&S24043[44], 17}, {NULL}};
const Segment SGI29368[] = {{&S24043[44], 28}, {NULL}};
const Segment SGI29369[] = {{&S24043[30], 6}, {NULL}};
const Segment SGI29370[] = {{&S24043[120], 3}, {NULL}};
const Segment SGI29371[] = {{&S24043[86], 5}, {NULL}};
const Segment SGI29372[] = {{&S24044[44], 18}, {NULL}};
const Segment SGI29373[] = {{&S24044[44], 29}, {NULL}};
const Segment SGI29374[] = {{&S24044[30], 6}, {NULL}};
const Segment SGI29375[] = {{&S24044[121], 3}, {NULL}};
const Segment SGI29376[] = {{&S24044[87], 5}, {NULL}};
const Segment SGI29377[] = {{&S24045[44], 17}, {NULL}};
const Segment SGI29378[] = {{&S24045[44], 28}, {NULL}};
const Segment SGI29379[] = {{&S24045[30], 6}, {NULL}};
const Segment SGI29380[] = {{&S24045[120], 3}, {NULL}};
const Segment SGI29381[] = {{&S24045[86], 5}, {NULL}};
const Segment SGI29382[] = {{&S24046[43], 18}, {NULL}};
const Segment SGI29383[] = {{&S24046[43], 29}, {NULL}};
const Segment SGI29384[] = {{&S24046[30], 5}, {NULL}};
const Segment SGI29385[] = {{&S24046[120], 3}, {NULL}};
const Segment SGI29386[] = {{&S24046[86], 5}, {NULL}};
const Segment SGI29387[] = {{&S24047[43], 18}, {NULL}};
const Segment SGI29388[] = {{&S24047[43], 29}, {NULL}};
const Segment SGI29389[] = {{&S24047[30], 5}, {NULL}};
const Segment SGI29390[] = {{&S24047[120], 3}, {NULL}};
const Segment SGI29391[] = {{&S24047[86], 5}, {NULL}};
const Segment SGI29392[] = {{&S24048[44], 17}, {NULL}};
const Segment SGI29393[] = {{&S24048[44], 28}, {NULL}};
const Segment SGI29394[] = {{&S24048[30], 6}, {NULL}};
const Segment SGI29395[] = {{&S24048[120], 3}, {NULL}};
const Segment SGI29396[] = {{&S24048[86], 5}, {NULL}};
const Segment SGI29397[] = {{&S24058[44], 14}, {NULL}};
const Segment SGI29398[] = {{&S24058[44], 25}, {NULL}};
const Segment SGI29399[] = {{&S24058[30], 6}, {NULL}};
const Segment SGI29400[] = {{&S24058[8], 3}, {NULL}};
const Segment SGI29401[] = {{&S24058[83], 5}, {NULL}};
const Segment SGI29402[] = {{&S24059[38], 21}, {NULL}};
const Segment SGI29403[] = {{&S24059[31], 6}, {NULL}};
const Segment SGI29404[] = {{&S24060[43], 14}, {NULL}};
const Segment SGI29405[] = {{&S24060[43], 25}, {NULL}};
const Segment SGI29406[] = {{&S24060[8], 3}, {NULL}};
const Segment SGI29407[] = {{&S24060[82], 5}, {NULL}};
const Segment SGI29408[] = {{&S24061[43], 14}, {NULL}};
const Segment SGI29409[] = {{&S24061[43], 25}, {NULL}};
const Segment SGI29410[] = {{&S24061[30], 5}, {NULL}};
const Segment SGI29411[] = {{&S24061[8], 3}, {NULL}};
const Segment SGI29412[] = {{&S24061[82], 5}, {NULL}};
const Segment SGI29413[] = {{&S24062[44], 13}, {NULL}};
const Segment SGI29414[] = {{&S24062[44], 24}, {NULL}};
const Segment SGI29415[] = {{&S24062[30], 6}, {NULL}};
const Segment SGI29416[] = {{&S24062[8], 3}, {NULL}};
const Segment SGI29417[] = {{&S24062[82], 5}, {NULL}};
const Segment SGI29418[] = {{&S24063[44], 13}, {NULL}};
const Segment SGI29419[] = {{&S24063[44], 24}, {NULL}};
const Segment SGI29420[] = {{&S24063[30], 6}, {NULL}};
const Segment SGI29421[] = {{&S25853[44], 7}, {NULL}};
const Segment SGI29422[] = {{&S25853[44], 24}, {NULL}};
const Segment SGI29423[] = {{&S25853[30], 6}, {NULL}};
const Segment SGI29424[] = {{&S24063[8], 3}, {NULL}};
const Segment SGI29425[] = {{&S24063[82], 5}, {NULL}};
const Segment SGI29426[] = {{&S25853[116], 3}, {NULL}};
const Segment SGI29427[] = {{&S25853[82], 5}, {NULL}};
const Segment SGI29428[] = {{&S24066[38], 18}, {NULL}};
const Segment SGI29429[] = {{&S24066[31], 6}, {NULL}};
const Segment SGI29430[] = {{&S25858[44], 19}, {NULL}};
const Segment SGI29431[] = {{&S25858[44], 36}, {NULL}};
const Segment SGI29432[] = {{&S25858[30], 6}, {NULL}};
const Segment SGI29433[] = {{&S25858[128], 3}, {NULL}};
const Segment SGI29434[] = {{&S25858[94], 5}, {NULL}};
const Segment SGI29435[] = {{&S25860[44], 7}, {NULL}};
const Segment SGI29436[] = {{&S25860[44], 24}, {NULL}};
const Segment SGI29437[] = {{&S25860[30], 6}, {NULL}};
const Segment SGI29438[] = {{&S25860[116], 3}, {NULL}};
const Segment SGI29439[] = {{&S25860[82], 5}, {NULL}};
const Segment SGI29440[] = {{&S24073[44], 21}, {NULL}};
const Segment SGI29441[] = {{&S24073[44], 32}, {NULL}};
const Segment SGI29442[] = {{&S24073[30], 6}, {NULL}};
const Segment SGI29443[] = {{&S24073[93], 4}, {NULL}};
const Segment SGI29444[] = {{&S24073[93], 3}, {NULL}};
const Segment SGI29445[] = {{&S23149[44], 9}, {NULL}};
const Segment SGI29446[] = {{&S23149[44], 20}, {NULL}};
const Segment SGI29447[] = {{&S23149[30], 6}, {NULL}};
const Segment SGI29448[] = {{&S23149[112], 3}, {NULL}};
const Segment SGI29449[] = {{&S23149[78], 5}, {NULL}};
const Segment SGI29450[] = {{&S23150[44], 18}, {NULL}};
const Segment SGI29451[] = {{&S23150[44], 29}, {NULL}};
const Segment SGI29452[] = {{&S23150[30], 6}, {NULL}};
const Segment SGI29453[] = {{&S23150[121], 3}, {NULL}};
const Segment SGI29454[] = {{&S23150[87], 5}, {NULL}};
const Segment SGI29455[] = {{&S24079[44], 21}, {NULL}};
const Segment SGI29456[] = {{&S24079[44], 32}, {NULL}};
const Segment SGI29457[] = {{&S24079[30], 6}, {NULL}};
const Segment SGI29458[] = {{&S24079[93], 6}, {NULL}};
const Segment SGI29459[] = {{&S23154[44], 9}, {NULL}};
const Segment SGI29460[] = {{&S23154[44], 20}, {NULL}};
const Segment SGI29461[] = {{&S23154[30], 6}, {NULL}};
const Segment SGI29462[] = {{&S23154[112], 3}, {NULL}};
const Segment SGI29463[] = {{&S23154[78], 5}, {NULL}};
const Segment SGI29464[] = {{&S24081[43], 18}, {NULL}};
const Segment SGI29465[] = {{&S24081[43], 29}, {NULL}};
const Segment SGI29466[] = {{&S24081[120], 3}, {NULL}};
const Segment SGI29467[] = {{&S24081[86], 5}, {NULL}};
const Segment SGI29468[] = {{&S25873[44], 19}, {NULL}};
const Segment SGI29469[] = {{&S23158[44], 9}, {NULL}};
const Segment SGI29470[] = {{&S25873[44], 35}, {NULL}};
const Segment SGI29471[] = {{&S23158[44], 20}, {NULL}};
const Segment SGI29472[] = {{&S25873[30], 6}, {NULL}};
const Segment SGI29473[] = {{&S23158[30], 6}, {NULL}};
const Segment SGI29474[] = {{&S25873[127], 3}, {NULL}};
const Segment SGI29475[] = {{&S23158[112], 3}, {NULL}};
const Segment SGI29476[] = {{&S23158[78], 5}, {NULL}};
const Segment SGI29477[] = {{&S25873[93], 5}, {NULL}};
const Segment SGI29478[] = {{&S23162[44], 18}, {NULL}};
const Segment SGI29479[] = {{&S23162[44], 29}, {NULL}};
const Segment SGI29480[] = {{&S23162[30], 6}, {NULL}};
const Segment SGI29481[] = {{&S23162[121], 3}, {NULL}};
const Segment SGI29482[] = {{&S23162[87], 5}, {NULL}};
const Segment SGI29483[] = {{&S25879[44], 19}, {NULL}};
const Segment SGI29484[] = {{&S25879[44], 36}, {NULL}};
const Segment SGI29485[] = {{&S25879[30], 6}, {NULL}};
const Segment SGI29486[] = {{&S25879[128], 3}, {NULL}};
const Segment SGI29487[] = {{&S25879[94], 5}, {NULL}};
const Segment SGI29488[] = {{&S25881[44], 21}, {NULL}};
const Segment SGI29489[] = {{&S25881[44], 38}, {NULL}};
const Segment SGI29490[] = {{&S25881[30], 6}, {NULL}};
const Segment SGI29491[] = {{&S23168[38], 9}, {NULL}};
const Segment SGI29492[] = {{&S25881[130], 3}, {NULL}};
const Segment SGI29493[] = {{&S23168[31], 6}, {NULL}};
const Segment SGI29494[] = {{&S25881[96], 5}, {NULL}};
const Segment SGI29495[] = {{&S25882[44], 19}, {NULL}};
const Segment SGI29496[] = {{&S25882[44], 36}, {NULL}};
const Segment SGI29497[] = {{&S25882[30], 6}, {NULL}};
const Segment SGI29498[] = {{&S25882[128], 3}, {NULL}};
const Segment SGI29499[] = {{&S25882[94], 5}, {NULL}};
const Segment SGI29500[] = {{&S25900[44], 20}, {NULL}};
const Segment SGI29501[] = {{&S25900[44], 37}, {NULL}};
const Segment SGI29502[] = {{&S25900[30], 6}, {NULL}};
const Segment SGI29503[] = {{&S25900[129], 3}, {NULL}};
const Segment SGI29504[] = {{&S25900[95], 5}, {NULL}};
const Segment SGI29505[] = {{&S22593[44], 10}, {NULL}};
const Segment SGI29506[] = {{&S22593[44], 27}, {NULL}};
const Segment SGI29507[] = {{&S22593[30], 6}, {NULL}};
const Segment SGI29508[] = {{&S22593[119], 3}, {NULL}};
const Segment SGI29509[] = {{&S22593[85], 5}, {NULL}};
const Segment SGI29510[] = {{&S22606[38], 10}, {NULL}};
const Segment SGI29511[] = {{&S22606[31], 6}, {NULL}};
const Segment SGI29512[] = {{&S24425[44], 11}, {NULL}};
const Segment SGI29513[] = {{&S24425[44], 28}, {NULL}};
const Segment SGI29514[] = {{&S24425[30], 6}, {NULL}};
const Segment SGI29515[] = {{&S24425[120], 3}, {NULL}};
const Segment SGI29516[] = {{&S24425[86], 5}, {NULL}};
const Segment SGI29517[] = {{&S24426[44], 10}, {NULL}};
const Segment SGI29518[] = {{&S24426[44], 27}, {NULL}};
const Segment SGI29519[] = {{&S24426[30], 6}, {NULL}};
const Segment SGI29520[] = {{&S24426[119], 3}, {NULL}};
const Segment SGI29521[] = {{&S24426[85], 5}, {NULL}};
const Segment SGI29522[] = {{&S24427[44], 17}, {NULL}};
const Segment SGI29523[] = {{&S24427[30], 6}, {NULL}};
const Segment SGI29524[] = {{&S24427[123], 3}, {NULL}};
const Segment SGI29525[] = {{&S24427[89], 5}, {NULL}};
const Segment SGI29526[] = {{&S24428[44], 11}, {NULL}};
const Segment SGI29527[] = {{&S24428[44], 28}, {NULL}};
const Segment SGI29528[] = {{&S24428[30], 6}, {NULL}};
const Segment SGI29529[] = {{&S24428[120], 3}, {NULL}};
const Segment SGI29530[] = {{&S24428[86], 5}, {NULL}};
const Segment SGI29531[] = {{&S24429[44], 11}, {NULL}};
const Segment SGI29532[] = {{&S24429[44], 28}, {NULL}};
const Segment SGI29533[] = {{&S24429[30], 6}, {NULL}};
const Segment SGI29534[] = {{&S24429[120], 3}, {NULL}};
const Segment SGI29535[] = {{&S24429[86], 5}, {NULL}};
const Segment SGI29536[] = {{&S24430[44], 11}, {NULL}};
const Segment SGI29537[] = {{&S24430[44], 28}, {NULL}};
const Segment SGI29538[] = {{&S24430[30], 6}, {NULL}};
const Segment SGI29539[] = {{&S24430[120], 3}, {NULL}};
const Segment SGI29540[] = {{&S24430[86], 5}, {NULL}};
const Segment SGI29541[] = {{&S24431[44], 11}, {NULL}};
const Segment SGI29542[] = {{&S24431[44], 28}, {NULL}};
const Segment SGI29543[] = {{&S24431[30], 6}, {NULL}};
const Segment SGI29544[] = {{&S24431[120], 3}, {NULL}};
const Segment SGI29545[] = {{&S24431[86], 5}, {NULL}};
const Segment SGI29546[] = {{&S24432[38], 10}, {NULL}};
const Segment SGI29547[] = {{&S24432[31], 6}, {NULL}};
const Segment SGI29548[] = {{&S24437[44], 10}, {NULL}};
const Segment SGI29549[] = {{&S24437[44], 21}, {NULL}};
const Segment SGI29550[] = {{&S24437[30], 6}, {NULL}};
const Segment SGI29551[] = {{&S24437[113], 3}, {NULL}};
const Segment SGI29552[] = {{&S24437[79], 5}, {NULL}};
const Segment SGI29553[] = {{&S24439[44], 10}, {NULL}};
const Segment SGI29554[] = {{&S24439[44], 21}, {NULL}};
const Segment SGI29555[] = {{&S24439[30], 6}, {NULL}};
const Segment SGI29556[] = {{&S24439[113], 3}, {NULL}};
const Segment SGI29557[] = {{&S24439[79], 5}, {NULL}};
const Segment SGI29558[] = {{&S24441[38], 11}, {NULL}};
const Segment SGI29559[] = {{&S24441[31], 6}, {NULL}};
const Segment SGI29560[] = {{&S24444[44], 10}, {NULL}};
const Segment SGI29561[] = {{&S24444[44], 27}, {NULL}};
const Segment SGI29562[] = {{&S24444[30], 6}, {NULL}};
const Segment SGI29563[] = {{&S24444[88], 4}, {NULL}};
const Segment SGI29564[] = {{&S24444[88], 3}, {NULL}};
const Segment SGI29565[] = {{&S24445[38], 11}, {NULL}};
const Segment SGI29566[] = {{&S24445[31], 6}, {NULL}};
const Segment SGI29567[] = {{&S24446[38], 11}, {NULL}};
const Segment SGI29568[] = {{&S24446[31], 6}, {NULL}};
const Segment SGI29569[] = {{&S24447[38], 11}, {NULL}};
const Segment SGI29570[] = {{&S24447[31], 6}, {NULL}};
const Segment SGI29571[] = {{&S24448[42], 10}, {NULL}};
const Segment SGI29572[] = {{&S24448[30], 6}, {NULL}};
const Segment SGI29573[] = {{&S24448[117], 3}, {NULL}};
const Segment SGI29574[] = {{&S24448[83], 5}, {NULL}};
const Segment SGI29575[] = {{&S24457[44], 10}, {NULL}};
const Segment SGI29576[] = {{&S24457[44], 27}, {NULL}};
const Segment SGI29577[] = {{&S24457[30], 6}, {NULL}};
const Segment SGI29578[] = {{&S24457[88], 6}, {NULL}};
const Segment SGI29579[] = {{&S24461[38], 11}, {NULL}};
const Segment SGI29580[] = {{&S24461[31], 6}, {NULL}};
const Segment SGI29581[] = {{&S24462[39], 8}, {NULL}};
const Segment SGI29582[] = {{&S24462[93], 3}, {NULL}};
const Segment SGI29583[] = {{&S24462[70], 6}, {NULL}};
const Segment SGI29584[] = {{&S24465[44], 10}, {NULL}};
const Segment SGI29585[] = {{&S24465[44], 27}, {NULL}};
const Segment SGI29586[] = {{&S24465[30], 6}, {NULL}};
const Segment SGI29587[] = {{&S24465[120], 3}, {NULL}};
const Segment SGI29588[] = {{&S24465[85], 6}, {NULL}};
const Segment SGI29589[] = {{&S24467[43], 10}, {NULL}};
const Segment SGI29590[] = {{&S24467[43], 27}, {NULL}};
const Segment SGI29591[] = {{&S24467[30], 5}, {NULL}};
const Segment SGI29592[] = {{&S24467[118], 3}, {NULL}};
const Segment SGI29593[] = {{&S24467[84], 5}, {NULL}};
const Segment SGI29594[] = {{&S24468[42], 11}, {NULL}};
const Segment SGI29595[] = {{&S24468[30], 6}, {NULL}};
const Segment SGI29596[] = {{&S24468[118], 3}, {NULL}};
const Segment SGI29597[] = {{&S24468[84], 5}, {NULL}};
const Segment SGI29598[] = {{&S24471[44], 10}, {NULL}};
const Segment SGI29599[] = {{&S24471[44], 21}, {NULL}};
const Segment SGI29600[] = {{&S24471[30], 6}, {NULL}};
const Segment SGI29601[] = {{&S24471[113], 3}, {NULL}};
const Segment SGI29602[] = {{&S24471[79], 5}, {NULL}};
const Segment SGI29603[] = {{&S24473[44], 10}, {NULL}};
const Segment SGI29604[] = {{&S24473[44], 40}, {NULL}};
const Segment SGI29605[] = {{&S24473[30], 6}, {NULL}};
const Segment SGI29606[] = {{&S24477[44], 9}, {NULL}};
const Segment SGI29607[] = {{&S24477[44], 26}, {NULL}};
const Segment SGI29608[] = {{&S24477[30], 6}, {NULL}};
const Segment SGI29609[] = {{&S24477[118], 3}, {NULL}};
const Segment SGI29610[] = {{&S24477[84], 5}, {NULL}};
const Segment SGI29611[] = {{&S24479[44], 27}, {NULL}};
const Segment SGI29612[] = {{&S24479[30], 6}, {NULL}};
const Segment SGI29613[] = {{&S24485[44], 10}, {NULL}};
const Segment SGI29614[] = {{&S24485[44], 19}, {NULL}};
const Segment SGI29615[] = {{&S24485[30], 6}, {NULL}};
const Segment SGI29616[] = {{&S24485[111], 3}, {NULL}};
const Segment SGI29617[] = {{&S24485[77], 5}, {NULL}};
const Segment SGI29618[] = {{&S24486[44], 18}, {NULL}};
const Segment SGI29619[] = {{&S24486[44], 35}, {NULL}};
const Segment SGI29620[] = {{&S24486[30], 6}, {NULL}};
const Segment SGI29621[] = {{&S24486[127], 3}, {NULL}};
const Segment SGI29622[] = {{&S24486[93], 5}, {NULL}};
const Segment SGI29623[] = {{&S24488[44], 10}, {NULL}};
const Segment SGI29624[] = {{&S24488[44], 27}, {&S24488[217], 7}, {NULL}};
const Segment SGI29625[] = {{&S24488[30], 6}, {NULL}};
const Segment SGI29626[] = {{&S24488[119], 3}, {NULL}};
const Segment SGI29627[] = {{&S24488[85], 5}, {NULL}};
const Segment SGI29628[] = {{&S24489[44], 10}, {NULL}};
const Segment SGI29629[] = {{&S24489[44], 27}, {NULL}};
const Segment SGI29630[] = {{&S24489[30], 6}, {NULL}};
const Segment SGI29631[] = {{&S24489[130], 3}, {NULL}};
const Segment SGI29632[] = {{&S24489[95], 6}, {NULL}};
const Segment SGI29633[] = {{&S24489[150], 8}, {NULL}};
const Segment SGI29634[] = {{&S24492[44], 11}, {NULL}};
const Segment SGI29635[] = {{&S24492[44], 28}, {NULL}};
const Segment SGI29636[] = {{&S24492[30], 6}, {NULL}};
const Segment SGI29637[] = {{&S24492[86], 6}, {NULL}};
const Segment SGI29638[] = {{&S24493[44], 10}, {NULL}};
const Segment SGI29639[] = {{&S24493[44], 33}, {NULL}};
const Segment SGI29640[] = {{&S24493[30], 6}, {NULL}};
const Segment SGI29641[] = {{&S24495[44], 10}, {NULL}};
const Segment SGI29642[] = {{&S24495[44], 33}, {NULL}};
const Segment SGI29643[] = {{&S24495[30], 6}, {NULL}};
const Segment SGI29644[] = {{&S24497[44], 11}, {NULL}};
const Segment SGI29645[] = {{&S24497[44], 28}, {NULL}};
const Segment SGI29646[] = {{&S24497[30], 6}, {NULL}};
const Segment SGI29647[] = {{&S24497[89], 6}, {NULL}};
const Segment SGI29648[] = {{&S24498[45], 11}, {NULL}};
const Segment SGI29649[] = {{&S24498[45], 34}, {NULL}};
const Segment SGI29650[] = {{&S24498[31], 6}, {NULL}};
const Segment SGI29651[] = {{&S24500[44], 10}, {NULL}};
const Segment SGI29652[] = {{&S24500[44], 33}, {NULL}};
const Segment SGI29653[] = {{&S24500[30], 6}, {NULL}};
const Segment SGI29654[] = {{&S24501[43], 10}, {NULL}};
const Segment SGI29655[] = {{&S24501[43], 27}, {NULL}};
const Segment SGI29656[] = {{&S24501[118], 3}, {NULL}};
const Segment SGI29657[] = {{&S24501[84], 5}, {NULL}};
const Segment SGI29658[] = {{&S24502[44], 10}, {NULL}};
const Segment SGI29659[] = {{&S24502[44], 33}, {NULL}};
const Segment SGI29660[] = {{&S24502[30], 6}, {NULL}};
const Segment SGI29661[] = {{&S24503[44], 10}, {NULL}};
const Segment SGI29662[] = {{&S24503[44], 33}, {NULL}};
const Segment SGI29663[] = {{&S24503[30], 6}, {NULL}};
const Segment SGI29664[] = {{&S24505[44], 10}, {NULL}};
const Segment SGI29665[] = {{&S24505[44], 27}, {NULL}};
const Segment SGI29666[] = {{&S24505[30], 6}, {NULL}};
const Segment SGI29667[] = {{&S24505[119], 3}, {NULL}};
const Segment SGI29668[] = {{&S24505[85], 5}, {NULL}};
const Segment SGI29669[] = {{&S24506[44], 9}, {NULL}};
const Segment SGI29670[] = {{&S24506[30], 6}, {NULL}};
const Segment SGI29671[] = {{&S24507[44], 10}, {NULL}};
const Segment SGI29672[] = {{&S24507[44], 33}, {NULL}};
const Segment SGI29673[] = {{&S24507[30], 6}, {NULL}};
const Segment SGI29674[] = {{&S24509[44], 10}, {NULL}};
const Segment SGI29675[] = {{&S24509[44], 33}, {NULL}};
const Segment SGI29676[] = {{&S24509[30], 6}, {NULL}};
const Segment SGI29677[] = {{&S24510[44], 10}, {NULL}};
const Segment SGI29678[] = {{&S24510[44], 33}, {NULL}};
const Segment SGI29679[] = {{&S24510[30], 6}, {NULL}};
const Segment SGI29680[] = {{&S24511[42], 11}, {NULL}};
const Segment SGI29681[] = {{&S24511[30], 6}, {NULL}};
const Segment SGI29682[] = {{&S24511[118], 3}, {NULL}};
const Segment SGI29683[] = {{&S24511[84], 5}, {NULL}};
const Segment SGI29684[] = {{&S24512[44], 10}, {NULL}};
const Segment SGI29685[] = {{&S24512[44], 22}, {NULL}};
const Segment SGI29686[] = {{&S24512[30], 6}, {NULL}};
const Segment SGI29687[] = {{&S24514[44], 10}, {NULL}};
const Segment SGI29688[] = {{&S24514[44], 33}, {NULL}};
const Segment SGI29689[] = {{&S24514[30], 6}, {NULL}};
const Segment SGI29690[] = {{&S24515[44], 11}, {NULL}};
const Segment SGI29691[] = {{&S24515[44], 28}, {NULL}};
const Segment SGI29692[] = {{&S24515[30], 6}, {NULL}};
const Segment SGI29693[] = {{&S24515[86], 6}, {NULL}};
const Segment SGI29694[] = {{&S24522[52], 11}, {NULL}};
const Segment SGI29695[] = {{&S24522[52], 28}, {NULL}};
const Segment SGI29696[] = {{&S24522[38], 6}, {NULL}};
const Segment SGI29697[] = {{&S24523[64], 10}, {NULL}};
const Segment SGI29698[] = {{&S24523[64], 27}, {NULL}};
const Segment SGI29699[] = {{&S24523[50], 6}, {NULL}};
const Segment SGI29700[] = {{&S24524[44], 10}, {NULL}};
const Segment SGI29701[] = {{&S24524[44], 27}, {NULL}};
const Segment SGI29702[] = {{&S24524[30], 6}, {NULL}};
const Segment SGI29703[] = {{&S24524[121], 3}, {NULL}};
const Segment SGI29704[] = {{&S24524[85], 7}, {NULL}};
const Segment SGI29705[] = {{&S24524[141], 8}, {NULL}};
const Segment SGI29706[] = {{&S24525[44], 10}, {NULL}};
const Segment SGI29707[] = {{&S24525[44], 27}, {&S24525[281], 7}, {NULL}};
const Segment SGI29708[] = {{&S24525[30], 6}, {NULL}};
const Segment SGI29709[] = {{&S24525[119], 3}, {NULL}};
const Segment SGI29710[] = {{&S24525[85], 5}, {NULL}};
const Segment SGI29711[] = {{&S24526[44], 21}, {NULL}};
const Segment SGI29712[] = {{&S24526[44], 33}, {NULL}};
const Segment SGI29713[] = {{&S24526[30], 6}, {NULL}};
const Segment SGI29714[] = {{&S24527[44], 10}, {NULL}};
const Segment SGI29715[] = {{&S24527[44], 33}, {NULL}};
const Segment SGI29716[] = {{&S24527[30], 6}, {NULL}};
const Segment SGI29717[] = {{&S24530[44], 11}, {NULL}};
const Segment SGI29718[] = {{&S24530[44], 22}, {NULL}};
const Segment SGI29719[] = {{&S24530[30], 6}, {NULL}};
const Segment SGI29720[] = {{&S24530[114], 3}, {NULL}};
const Segment SGI29721[] = {{&S24530[80], 5}, {NULL}};
const Segment SGI29722[] = {{&S24534[44], 10}, {NULL}};
const Segment SGI29723[] = {{&S24534[44], 33}, {NULL}};
const Segment SGI29724[] = {{&S24534[30], 6}, {NULL}};
const Segment SGI29725[] = {{&S24538[45], 11}, {NULL}};
const Segment SGI29726[] = {{&S24538[45], 25}, {NULL}};
const Segment SGI29727[] = {{&S24538[31], 6}, {NULL}};
const Segment SGI29728[] = {{&S24542[47], 11}, {NULL}};
const Segment SGI29729[] = {{&S24542[47], 34}, {NULL}};
const Segment SGI29730[] = {{&S24542[33], 6}, {NULL}};
const Segment SGI29731[] = {{&S24543[44], 10}, {NULL}};
const Segment SGI29732[] = {{&S24543[30], 6}, {NULL}};
const Segment SGI29733[] = {{&S24549[44], 10}, {NULL}};
const Segment SGI29734[] = {{&S24549[44], 28}, {NULL}};
const Segment SGI29735[] = {{&S24549[30], 6}, {NULL}};
const Segment SGI29736[] = {{&S24549[82], 3}, {NULL}};
const Segment SGI29737[] = {{&S24549[100], 5}, {NULL}};
const Segment SGI29738[] = {{&S24552[60], 11}, {NULL}};
const Segment SGI29739[] = {{&S24552[60], 34}, {NULL}};
const Segment SGI29740[] = {{&S24552[46], 6}, {NULL}};
const Segment SGI29741[] = {{&S24553[42], 11}, {NULL}};
const Segment SGI29742[] = {{&S24553[30], 6}, {NULL}};
const Segment SGI29743[] = {{&S24553[118], 3}, {NULL}};
const Segment SGI29744[] = {{&S24553[84], 5}, {NULL}};
const Segment SGI29745[] = {{&S24554[99], 29}, {NULL}};
const Segment SGI29746[] = {{&S24556[46], 10}, {NULL}};
const Segment SGI29747[] = {{&S24556[46], 38}, {NULL}};
const Segment SGI29748[] = {{&S24556[32], 6}, {NULL}};
const Segment SGI29749[] = {{&S24557[44], 11}, {NULL}};
const Segment SGI29750[] = {{&S24557[44], 20}, {NULL}};
const Segment SGI29751[] = {{&S24557[30], 6}, {NULL}};
const Segment SGI29752[] = {{&S24557[112], 3}, {NULL}};
const Segment SGI29753[] = {{&S24557[78], 5}, {NULL}};
const Segment SGI29754[] = {{&S24561[44], 11}, {NULL}};
const Segment SGI29755[] = {{&S24561[44], 28}, {NULL}};
const Segment SGI29756[] = {{&S24561[30], 6}, {NULL}};
const Segment SGI29757[] = {{&S24561[121], 3}, {NULL}};
const Segment SGI29758[] = {{&S24561[86], 6}, {NULL}};
const Segment SGI29759[] = {{&S24563[65], 11}, {NULL}};
const Segment SGI29760[] = {{&S24563[65], 28}, {NULL}};
const Segment SGI29761[] = {{&S24563[51], 6}, {NULL}};
const Segment SGI29762[] = {{&S24564[44], 10}, {NULL}};
const Segment SGI29763[] = {{&S24564[44], 27}, {NULL}};
const Segment SGI29764[] = {{&S24564[30], 6}, {NULL}};
const Segment SGI29765[] = {{&S24564[132], 3}, {NULL}};
const Segment SGI29766[] = {{&S24564[85], 6}, {NULL}};
const Segment SGI29767[] = {{&S24565[44], 9}, {NULL}};
const Segment SGI29768[] = {{&S24565[30], 6}, {NULL}};
const Segment SGI29769[] = {{&S24565[103], 3}, {NULL}};
const Segment SGI29770[] = {{&S24565[67], 7}, {NULL}};
const Segment SGI29771[] = {{&S24565[123], 8}, {NULL}};
const Segment SGI29772[] = {{&S24566[44], 10}, {NULL}};
const Segment SGI29773[] = {{&S24566[30], 6}, {NULL}};
const Segment SGI29774[] = {{&S24568[45], 10}, {NULL}};
const Segment SGI29775[] = {{&S24568[45], 33}, {NULL}};
const Segment SGI29776[] = {{&S24568[31], 6}, {NULL}};
const Segment SGI29777[] = {{&S24569[44], 11}, {NULL}};
const Segment SGI29778[] = {{&S24569[44], 28}, {NULL}};
const Segment SGI29779[] = {{&S24569[30], 6}, {NULL}};
const Segment SGI29780[] = {{&S24569[133], 3}, {NULL}};
const Segment SGI29781[] = {{&S24569[86], 6}, {NULL}};
const Segment SGI29782[] = {{&S24574[44], 10}, {NULL}};
const Segment SGI29783[] = {{&S24574[30], 6}, {NULL}};
const Segment SGI29784[] = {{&S24575[44], 11}, {NULL}};
const Segment SGI29785[] = {{&S24575[44], 28}, {NULL}};
const Segment SGI29786[] = {{&S24575[30], 6}, {NULL}};
const Segment SGI29787[] = {{&S24575[122], 3}, {NULL}};
const Segment SGI29788[] = {{&S24575[86], 7}, {NULL}};
const Segment SGI29789[] = {{&S24575[142], 8}, {NULL}};
const Segment SGI29790[] = {{&S24576[44], 10}, {NULL}};
const Segment SGI29791[] = {{&S24576[44], 27}, {NULL}};
const Segment SGI29792[] = {{&S24576[30], 6}, {NULL}};
const Segment SGI29793[] = {{&S24576[85], 6}, {NULL}};
const Segment SGI29794[] = {{&S24578[37], 6}, {NULL}};
const Segment SGI29795[] = {{&S24579[42], 11}, {NULL}};
const Segment SGI29796[] = {{&S24579[30], 6}, {NULL}};
const Segment SGI29797[] = {{&S24579[118], 3}, {NULL}};
const Segment SGI29798[] = {{&S24579[84], 5}, {NULL}};
const Segment SGI29799[] = {{&S24580[42], 18}, {NULL}};
const Segment SGI29800[] = {{&S24580[30], 6}, {NULL}};
const Segment SGI29801[] = {{&S24580[125], 3}, {NULL}};
const Segment SGI29802[] = {{&S24580[91], 5}, {NULL}};
const Segment SGI29803[] = {{&S24581[38], 11}, {NULL}};
const Segment SGI29804[] = {{&S24581[117], 6}, {NULL}};
const Segment SGI29805[] = {{&S24581[31], 6}, {NULL}};
const Segment SGI29806[] = {{&S24582[44], 10}, {NULL}};
const Segment SGI29807[] = {{&S24582[44], 27}, {NULL}};
const Segment SGI29808[] = {{&S24582[30], 6}, {NULL}};
const Segment SGI29809[] = {{&S24582[120], 3}, {NULL}};
const Segment SGI29810[] = {{&S24582[85], 6}, {NULL}};
const Segment SGI29811[] = {{&S23531[44], 7}, {NULL}};
const Segment SGI29812[] = {{&S23531[44], 29}, {NULL}};
const Segment SGI29813[] = {{&S23531[30], 6}, {NULL}};
const Segment SGI29814[] = {{&S23531[121], 3}, {NULL}};
const Segment SGI29815[] = {{&S23531[87], 5}, {NULL}};
const Segment SGI29816[] = {{&S23533[44], 7}, {NULL}};
const Segment SGI29817[] = {{&S23533[44], 26}, {NULL}};
const Segment SGI29818[] = {{&S23533[30], 6}, {NULL}};
const Segment SGI29819[] = {{&S23533[118], 3}, {NULL}};
const Segment SGI29820[] = {{&S23533[84], 5}, {NULL}};
const Segment SGI29821[] = {{&S23534[44], 7}, {NULL}};
const Segment SGI29822[] = {{&S23534[44], 29}, {NULL}};
const Segment SGI29823[] = {{&S23534[30], 6}, {NULL}};
const Segment SGI29824[] = {{&S23534[121], 3}, {NULL}};
const Segment SGI29825[] = {{&S23534[87], 5}, {NULL}};
const Segment SGI29826[] = {{&S23535[44], 7}, {NULL}};
const Segment SGI29827[] = {{&S23535[44], 22}, {NULL}};
const Segment SGI29828[] = {{&S23535[30], 6}, {NULL}};
const Segment SGI29829[] = {{&S23535[114], 3}, {NULL}};
const Segment SGI29830[] = {{&S23535[80], 5}, {NULL}};
const Segment SGI29831[] = {{&S23536[44], 7}, {NULL}};
const Segment SGI29832[] = {{&S23536[44], 19}, {NULL}};
const Segment SGI29833[] = {{&S23536[30], 6}, {NULL}};
const Segment SGI29834[] = {{&S23536[111], 3}, {NULL}};
const Segment SGI29835[] = {{&S23536[77], 5}, {NULL}};
const Segment SGI29836[] = {{&S23537[44], 7}, {NULL}};
const Segment SGI29837[] = {{&S23537[44], 28}, {NULL}};
const Segment SGI29838[] = {{&S23537[30], 6}, {NULL}};
const Segment SGI29839[] = {{&S23537[120], 3}, {NULL}};
const Segment SGI29840[] = {{&S23537[86], 5}, {NULL}};
const Segment SGI29841[] = {{&S23538[44], 7}, {NULL}};
const Segment SGI29842[] = {{&S23538[44], 22}, {NULL}};
const Segment SGI29843[] = {{&S23538[30], 6}, {NULL}};
const Segment SGI29844[] = {{&S23538[114], 3}, {NULL}};
const Segment SGI29845[] = {{&S23538[80], 5}, {NULL}};
const Segment SGI29846[] = {{&S23540[44], 7}, {NULL}};
const Segment SGI29847[] = {{&S23540[44], 29}, {NULL}};
const Segment SGI29848[] = {{&S23540[30], 6}, {NULL}};
const Segment SGI29849[] = {{&S23540[121], 3}, {NULL}};
const Segment SGI29850[] = {{&S23540[87], 5}, {NULL}};
const Segment SGI29851[] = {{&S23541[44], 7}, {NULL}};
const Segment SGI29852[] = {{&S23541[44], 29}, {NULL}};
const Segment SGI29853[] = {{&S23541[30], 6}, {NULL}};
const Segment SGI29854[] = {{&S23541[121], 3}, {NULL}};
const Segment SGI29855[] = {{&S23541[87], 5}, {NULL}};
const Segment SGI29856[] = {{&S23542[44], 7}, {NULL}};
const Segment SGI29857[] = {{&S23542[44], 29}, {NULL}};
const Segment SGI29858[] = {{&S23542[30], 6}, {NULL}};
const Segment SGI29859[] = {{&S23542[121], 3}, {NULL}};
const Segment SGI29860[] = {{&S23542[87], 5}, {NULL}};
const Segment SGI29861[] = {{&S23543[44], 7}, {NULL}};
const Segment SGI29862[] = {{&S23543[44], 28}, {NULL}};
const Segment SGI29863[] = {{&S23543[30], 6}, {NULL}};
const Segment SGI29864[] = {{&S23543[120], 3}, {NULL}};
const Segment SGI29865[] = {{&S23543[86], 5}, {NULL}};
const Segment SGI29866[] = {{&S23544[44], 7}, {NULL}};
const Segment SGI29867[] = {{&S23544[44], 29}, {NULL}};
const Segment SGI29868[] = {{&S23544[30], 6}, {NULL}};
const Segment SGI29869[] = {{&S23544[121], 3}, {NULL}};
const Segment SGI29870[] = {{&S23544[87], 5}, {NULL}};
const Segment SGI29871[] = {{&S23545[44], 7}, {NULL}};
const Segment SGI29872[] = {{&S23545[44], 35}, {NULL}};
const Segment SGI29873[] = {{&S23545[30], 6}, {NULL}};
const Segment SGI29874[] = {{&S23545[127], 3}, {NULL}};
const Segment SGI29875[] = {{&S23545[93], 5}, {NULL}};
const Segment SGI29876[] = {{&S23546[44], 7}, {NULL}};
const Segment SGI29877[] = {{&S23546[44], 29}, {NULL}};
const Segment SGI29878[] = {{&S23546[30], 6}, {NULL}};
const Segment SGI29879[] = {{&S23546[121], 3}, {NULL}};
const Segment SGI29880[] = {{&S23546[87], 5}, {NULL}};
const Segment SGI29881[] = {{&S23548[44], 7}, {NULL}};
const Segment SGI29882[] = {{&S23548[44], 19}, {NULL}};
const Segment SGI29883[] = {{&S23548[30], 6}, {NULL}};
const Segment SGI29884[] = {{&S23548[111], 3}, {NULL}};
const Segment SGI29885[] = {{&S23548[77], 5}, {NULL}};
const Segment SGI29886[] = {{&S23549[44], 7}, {NULL}};
const Segment SGI29887[] = {{&S23549[44], 19}, {NULL}};
const Segment SGI29888[] = {{&S23549[30], 6}, {NULL}};
const Segment SGI29889[] = {{&S23549[111], 3}, {NULL}};
const Segment SGI29890[] = {{&S23549[77], 5}, {NULL}};
const Segment SGI29891[] = {{&S23550[44], 7}, {NULL}};
const Segment SGI29892[] = {{&S23550[44], 27}, {NULL}};
const Segment SGI29893[] = {{&S23550[30], 6}, {NULL}};
const Segment SGI29894[] = {{&S23550[119], 3}, {NULL}};
const Segment SGI29895[] = {{&S23550[85], 5}, {NULL}};
const Segment SGI29896[] = {{&S23551[44], 7}, {NULL}};
const Segment SGI29897[] = {{&S23551[44], 22}, {NULL}};
const Segment SGI29898[] = {{&S23551[30], 6}, {NULL}};
const Segment SGI29899[] = {{&S23551[114], 3}, {NULL}};
const Segment SGI29900[] = {{&S23551[80], 5}, {NULL}};
const Segment SGI29901[] = {{&S23552[44], 7}, {NULL}};
const Segment SGI29902[] = {{&S23552[44], 19}, {NULL}};
const Segment SGI29903[] = {{&S23552[30], 6}, {NULL}};
const Segment SGI29904[] = {{&S23552[111], 3}, {NULL}};
const Segment SGI29905[] = {{&S23552[77], 5}, {NULL}};
const Segment SGI29906[] = {{&S23554[44], 7}, {NULL}};
const Segment SGI29907[] = {{&S23554[44], 29}, {NULL}};
const Segment SGI29908[] = {{&S23554[30], 6}, {NULL}};
const Segment SGI29909[] = {{&S23554[121], 3}, {NULL}};
const Segment SGI29910[] = {{&S23554[87], 5}, {NULL}};
const Segment SGI29911[] = {{&S23556[44], 7}, {NULL}};
const Segment SGI29912[] = {{&S23556[44], 29}, {NULL}};
const Segment SGI29913[] = {{&S23556[30], 6}, {NULL}};
const Segment SGI29914[] = {{&S23556[121], 3}, {NULL}};
const Segment SGI29915[] = {{&S23556[87], 5}, {NULL}};
const Segment SGI29916[] = {{&S23557[44], 7}, {NULL}};
const Segment SGI29917[] = {{&S23557[44], 21}, {NULL}};
const Segment SGI29918[] = {{&S23557[30], 6}, {NULL}};
const Segment SGI29919[] = {{&S23557[113], 3}, {NULL}};
const Segment SGI29920[] = {{&S23557[79], 5}, {NULL}};
const Segment SGI29921[] = {{&S23558[44], 7}, {NULL}};
const Segment SGI29922[] = {{&S23558[44], 29}, {NULL}};
const Segment SGI29923[] = {{&S23558[30], 6}, {NULL}};
const Segment SGI29924[] = {{&S23558[121], 3}, {NULL}};
const Segment SGI29925[] = {{&S23558[87], 5}, {NULL}};
const Segment SGI29926[] = {{&S23565[44], 7}, {NULL}};
const Segment SGI29927[] = {{&S23565[44], 29}, {NULL}};
const Segment SGI29928[] = {{&S23565[30], 6}, {NULL}};
const Segment SGI29929[] = {{&S23565[121], 3}, {NULL}};
const Segment SGI29930[] = {{&S23565[87], 5}, {NULL}};
const Segment SGI29931[] = {{&S23566[44], 7}, {NULL}};
const Segment SGI29932[] = {{&S23566[44], 19}, {NULL}};
const Segment SGI29933[] = {{&S23566[30], 6}, {NULL}};
const Segment SGI29934[] = {{&S23566[77], 6}, {NULL}};
const Segment SGI29935[] = {{&S23568[44], 7}, {NULL}};
const Segment SGI29936[] = {{&S23568[44], 29}, {NULL}};
const Segment SGI29937[] = {{&S23568[30], 6}, {NULL}};
const Segment SGI29938[] = {{&S23568[121], 3}, {NULL}};
const Segment SGI29939[] = {{&S23568[87], 5}, {NULL}};
const Segment SGI29940[] = {{&S23569[44], 7}, {NULL}};
const Segment SGI29941[] = {{&S23569[44], 31}, {NULL}};
const Segment SGI29942[] = {{&S23569[30], 6}, {NULL}};
const Segment SGI29943[] = {{&S23569[123], 3}, {NULL}};
const Segment SGI29944[] = {{&S23569[89], 5}, {NULL}};
const Segment SGI29945[] = {{&S23573[44], 7}, {NULL}};
const Segment SGI29946[] = {{&S23573[44], 29}, {NULL}};
const Segment SGI29947[] = {{&S23573[30], 6}, {NULL}};
const Segment SGI29948[] = {{&S23573[121], 3}, {NULL}};
const Segment SGI29949[] = {{&S23573[87], 5}, {NULL}};
const Segment SGI29950[] = {{&S23578[44], 7}, {NULL}};
const Segment SGI29951[] = {{&S23578[44], 31}, {NULL}};
const Segment SGI29952[] = {{&S23578[30], 6}, {NULL}};
const Segment SGI29953[] = {{&S23578[123], 3}, {NULL}};
const Segment SGI29954[] = {{&S23578[89], 5}, {NULL}};
const Segment SGI29955[] = {{&S23579[44], 7}, {NULL}};
const Segment SGI29956[] = {{&S23579[44], 32}, {NULL}};
const Segment SGI29957[] = {{&S23579[30], 6}, {NULL}};
const Segment SGI29958[] = {{&S23579[124], 3}, {NULL}};
const Segment SGI29959[] = {{&S23579[90], 5}, {NULL}};
const Segment SGI29960[] = {{&S23580[44], 7}, {NULL}};
const Segment SGI29961[] = {{&S23580[44], 32}, {NULL}};
const Segment SGI29962[] = {{&S23580[30], 6}, {NULL}};
const Segment SGI29963[] = {{&S23580[124], 3}, {NULL}};
const Segment SGI29964[] = {{&S23580[90], 5}, {NULL}};
const Segment SGI29965[] = {{&S23581[44], 7}, {NULL}};
const Segment SGI29966[] = {{&S23581[44], 29}, {NULL}};
const Segment SGI29967[] = {{&S23581[30], 6}, {NULL}};
const Segment SGI29968[] = {{&S23581[121], 3}, {NULL}};
const Segment SGI29969[] = {{&S23581[87], 5}, {NULL}};
const Segment SGI29970[] = {{&S23582[38], 7}, {NULL}};
const Segment SGI29971[] = {{&S23582[31], 6}, {NULL}};
const Segment SGI29972[] = {{&S23583[44], 7}, {NULL}};
const Segment SGI29973[] = {{&S23583[44], 29}, {NULL}};
const Segment SGI29974[] = {{&S23583[30], 6}, {NULL}};
const Segment SGI29975[] = {{&S23583[121], 3}, {NULL}};
const Segment SGI29976[] = {{&S23583[87], 5}, {NULL}};
const Segment SGI29977[] = {{&S23585[44], 7}, {NULL}};
const Segment SGI29978[] = {{&S23585[44], 29}, {NULL}};
const Segment SGI29979[] = {{&S23585[30], 6}, {NULL}};
const Segment SGI29980[] = {{&S23585[121], 3}, {NULL}};
const Segment SGI29981[] = {{&S23585[87], 5}, {NULL}};
const Segment SGI29982[] = {{&S23587[44], 7}, {NULL}};
const Segment SGI29983[] = {{&S23587[44], 31}, {NULL}};
const Segment SGI29984[] = {{&S23587[30], 6}, {NULL}};
const Segment SGI29985[] = {{&S23587[123], 3}, {NULL}};
const Segment SGI29986[] = {{&S23587[89], 5}, {NULL}};
const Segment SGI29987[] = {{&S23593[42], 7}, {NULL}};
const Segment SGI29988[] = {{&S23593[30], 6}, {NULL}};
const Segment SGI29989[] = {{&S23593[119], 3}, {NULL}};
const Segment SGI29990[] = {{&S23593[85], 5}, {NULL}};
const Segment SGI29991[] = {{&S23626[44], 19}, {NULL}};
const Segment SGI29992[] = {{&S23626[44], 37}, {NULL}};
const Segment SGI29993[] = {{&S23626[30], 6}, {NULL}};
const Segment SGI29994[] = {{&S23626[69], 3}, {NULL}};
const Segment SGI29995[] = {{&S23626[95], 5}, {NULL}};
const Segment SGI29996[] = {{&S23627[44], 19}, {NULL}};
const Segment SGI29997[] = {{&S23627[44], 38}, {NULL}};
const Segment SGI29998[] = {{&S23627[30], 6}, {NULL}};
const Segment SGI29999[] = {{&S23627[69], 3}, {NULL}};
const Segment SGI30000[] = {{&S23627[96], 5}, {NULL}};
const Segment SGI30001[] = {{&S23628[44], 21}, {NULL}};
const Segment SGI30002[] = {{&S23628[44], 40}, {NULL}};
const Segment SGI30003[] = {{&S23628[30], 6}, {NULL}};
const Segment SGI30004[] = {{&S23628[71], 3}, {NULL}};
const Segment SGI30005[] = {{&S23628[98], 5}, {NULL}};
const Segment SGI30006[] = {{&S23629[44], 20}, {NULL}};
const Segment SGI30007[] = {{&S23629[44], 39}, {NULL}};
const Segment SGI30008[] = {{&S23629[30], 6}, {NULL}};
const Segment SGI30009[] = {{&S23629[70], 3}, {NULL}};
const Segment SGI30010[] = {{&S23629[97], 5}, {NULL}};
const Segment SGI30011[] = {{&S23630[44], 19}, {NULL}};
const Segment SGI30012[] = {{&S23630[44], 37}, {NULL}};
const Segment SGI30013[] = {{&S23630[30], 6}, {NULL}};
const Segment SGI30014[] = {{&S23630[69], 3}, {NULL}};
const Segment SGI30015[] = {{&S23630[95], 5}, {NULL}};
const Segment SGI30016[] = {{&S23631[44], 20}, {NULL}};
const Segment SGI30017[] = {{&S23631[44], 38}, {NULL}};
const Segment SGI30018[] = {{&S23631[30], 6}, {NULL}};
const Segment SGI30019[] = {{&S23631[70], 3}, {NULL}};
const Segment SGI30020[] = {{&S23631[96], 5}, {NULL}};
const Segment SGI30021[] = {{&S23633[44], 7}, {NULL}};
const Segment SGI30022[] = {{&S23633[44], 24}, {NULL}};
const Segment SGI30023[] = {{&S23633[30], 6}, {NULL}};
const Segment SGI30024[] = {{&S23633[57], 3}, {NULL}};
const Segment SGI30025[] = {{&S23633[82], 5}, {NULL}};
const Segment SGI30026[] = {{&S23634[44], 20}, {NULL}};
const Segment SGI30027[] = {{&S23634[44], 38}, {NULL}};
const Segment SGI30028[] = {{&S23634[30], 6}, {NULL}};
const Segment SGI30029[] = {{&S23634[70], 3}, {NULL}};
const Segment SGI30030[] = {{&S23634[96], 5}, {NULL}};
const Segment SGI30031[] = {{&S23635[38], 7}, {NULL}};
const Segment SGI30032[] = {{&S23635[31], 6}, {NULL}};
const Segment SGI30033[] = {{&S23636[44], 21}, {NULL}};
const Segment SGI30034[] = {{&S23636[44], 39}, {NULL}};
const Segment SGI30035[] = {{&S23636[30], 6}, {NULL}};
const Segment SGI30036[] = {{&S23636[71], 3}, {NULL}};
const Segment SGI30037[] = {{&S23636[97], 5}, {NULL}};
const Segment SGI30038[] = {{&S23637[44], 19}, {NULL}};
const Segment SGI30039[] = {{&S23637[44], 37}, {NULL}};
const Segment SGI30040[] = {{&S23637[30], 6}, {NULL}};
const Segment SGI30041[] = {{&S23637[69], 3}, {NULL}};
const Segment SGI30042[] = {{&S23637[95], 5}, {NULL}};
const Segment SGI30043[] = {{&S23638[44], 19}, {NULL}};
const Segment SGI30044[] = {{&S23638[44], 37}, {NULL}};
const Segment SGI30045[] = {{&S23638[30], 6}, {NULL}};
const Segment SGI30046[] = {{&S23638[69], 3}, {NULL}};
const Segment SGI30047[] = {{&S23638[95], 5}, {NULL}};
const Segment SGI30048[] = {{&S23639[44], 21}, {NULL}};
const Segment SGI30049[] = {{&S23639[44], 39}, {NULL}};
const Segment SGI30050[] = {{&S23639[30], 6}, {NULL}};
const Segment SGI30051[] = {{&S23639[71], 3}, {NULL}};
const Segment SGI30052[] = {{&S23639[97], 5}, {NULL}};
const Segment SGI30053[] = {{&S23640[44], 19}, {NULL}};
const Segment SGI30054[] = {{&S23640[44], 36}, {NULL}};
const Segment SGI30055[] = {{&S23640[30], 6}, {NULL}};
const Segment SGI30056[] = {{&S23640[69], 3}, {NULL}};
const Segment SGI30057[] = {{&S23640[94], 5}, {NULL}};
const Segment SGI30058[] = {{&S23641[44], 7}, {NULL}};
const Segment SGI30059[] = {{&S23641[44], 25}, {NULL}};
const Segment SGI30060[] = {{&S23641[30], 6}, {NULL}};
const Segment SGI30061[] = {{&S23641[57], 3}, {NULL}};
const Segment SGI30062[] = {{&S23641[83], 5}, {NULL}};
const Segment SGI30063[] = {{&S23642[44], 7}, {NULL}};
const Segment SGI30064[] = {{&S23642[44], 26}, {NULL}};
const Segment SGI30065[] = {{&S23642[30], 6}, {NULL}};
const Segment SGI30066[] = {{&S23642[57], 3}, {NULL}};
const Segment SGI30067[] = {{&S23642[84], 5}, {NULL}};
const Segment SGI30068[] = {{&S23643[44], 19}, {NULL}};
const Segment SGI30069[] = {{&S23643[44], 36}, {NULL}};
const Segment SGI30070[] = {{&S23643[30], 6}, {NULL}};
const Segment SGI30071[] = {{&S23643[69], 3}, {NULL}};
const Segment SGI30072[] = {{&S23643[94], 5}, {NULL}};
const Segment SGI30073[] = {{&S23644[44], 7}, {NULL}};
const Segment SGI30074[] = {{&S23644[44], 25}, {NULL}};
const Segment SGI30075[] = {{&S23644[30], 6}, {NULL}};
const Segment SGI30076[] = {{&S23644[57], 3}, {NULL}};
const Segment SGI30077[] = {{&S23644[83], 5}, {NULL}};
const Segment SGI30078[] = {{&S23645[44], 7}, {NULL}};
const Segment SGI30079[] = {{&S23645[44], 25}, {NULL}};
const Segment SGI30080[] = {{&S23645[30], 6}, {NULL}};
const Segment SGI30081[] = {{&S23645[86], 4}, {NULL}};
const Segment SGI30082[] = {{&S23645[86], 3}, {NULL}};
const Segment SGI30083[] = {{&S23646[44], 7}, {NULL}};
const Segment SGI30084[] = {{&S23646[44], 26}, {NULL}};
const Segment SGI30085[] = {{&S23646[30], 6}, {NULL}};
const Segment SGI30086[] = {{&S23646[57], 3}, {NULL}};
const Segment SGI30087[] = {{&S23646[84], 5}, {NULL}};
const Segment SGI30088[] = {{&S23647[44], 19}, {NULL}};
const Segment SGI30089[] = {{&S23647[44], 38}, {NULL}};
const Segment SGI30090[] = {{&S23647[30], 6}, {NULL}};
const Segment SGI30091[] = {{&S23647[69], 3}, {NULL}};
const Segment SGI30092[] = {{&S23647[96], 5}, {NULL}};
const Segment SGI30093[] = {{&S23648[44], 19}, {NULL}};
const Segment SGI30094[] = {{&S23648[44], 37}, {NULL}};
const Segment SGI30095[] = {{&S23648[30], 6}, {NULL}};
const Segment SGI30096[] = {{&S23648[69], 3}, {NULL}};
const Segment SGI30097[] = {{&S23648[95], 5}, {NULL}};
const Segment SGI30098[] = {{&S23649[44], 7}, {NULL}};
const Segment SGI30099[] = {{&S23649[44], 25}, {NULL}};
const Segment SGI30100[] = {{&S23649[30], 6}, {NULL}};
const Segment SGI30101[] = {{&S23649[57], 3}, {NULL}};
const Segment SGI30102[] = {{&S23649[83], 5}, {NULL}};
const Segment SGI30103[] = {{&S23650[44], 7}, {NULL}};
const Segment SGI30104[] = {{&S23650[44], 25}, {NULL}};
const Segment SGI30105[] = {{&S23650[30], 6}, {NULL}};
const Segment SGI30106[] = {{&S23650[57], 3}, {NULL}};
const Segment SGI30107[] = {{&S23650[83], 5}, {NULL}};
const Segment SGI30108[] = {{&S23651[44], 19}, {NULL}};
const Segment SGI30109[] = {{&S23651[44], 37}, {NULL}};
const Segment SGI30110[] = {{&S23651[30], 6}, {NULL}};
const Segment SGI30111[] = {{&S23651[69], 3}, {NULL}};
const Segment SGI30112[] = {{&S23651[95], 5}, {NULL}};
const Segment SGI30113[] = {{&S23652[44], 7}, {NULL}};
const Segment SGI30114[] = {{&S23652[44], 25}, {NULL}};
const Segment SGI30115[] = {{&S23652[30], 6}, {NULL}};
const Segment SGI30116[] = {{&S23652[86], 4}, {NULL}};
const Segment SGI30117[] = {{&S23652[86], 3}, {NULL}};
const Segment SGI30118[] = {{&S23653[44], 7}, {NULL}};
const Segment SGI30119[] = {{&S23653[44], 25}, {NULL}};
const Segment SGI30120[] = {{&S23653[30], 6}, {NULL}};
const Segment SGI30121[] = {{&S23653[57], 3}, {NULL}};
const Segment SGI30122[] = {{&S23653[83], 5}, {NULL}};
const Segment SGI30123[] = {{&S23654[44], 7}, {NULL}};
const Segment SGI30124[] = {{&S23654[44], 24}, {NULL}};
const Segment SGI30125[] = {{&S23654[30], 6}, {NULL}};
const Segment SGI30126[] = {{&S23654[57], 3}, {NULL}};
const Segment SGI30127[] = {{&S23654[82], 5}, {NULL}};
const Segment SGI30128[] = {{&S23655[44], 7}, {NULL}};
const Segment SGI30129[] = {{&S23655[44], 25}, {NULL}};
const Segment SGI30130[] = {{&S23655[30], 6}, {NULL}};
const Segment SGI30131[] = {{&S23655[57], 3}, {NULL}};
const Segment SGI30132[] = {{&S23655[83], 5}, {NULL}};
const Segment SGI30133[] = {{&S23656[44], 7}, {NULL}};
const Segment SGI30134[] = {{&S23656[44], 25}, {NULL}};
const Segment SGI30135[] = {{&S23656[30], 6}, {NULL}};
const Segment SGI30136[] = {{&S23656[86], 6}, {NULL}};
const Segment SGI30137[] = {{&S23657[38], 7}, {NULL}};
const Segment SGI30138[] = {{&S23657[31], 6}, {NULL}};
const Segment SGI30139[] = {{&S23658[42], 19}, {NULL}};
const Segment SGI30140[] = {{&S23658[30], 6}, {NULL}};
const Segment SGI30141[] = {{&S23658[67], 3}, {NULL}};
const Segment SGI30142[] = {{&S23658[93], 5}, {NULL}};
const Segment SGI30143[] = {{&S26733[44], 19}, {NULL}};
const Segment SGI30144[] = {{&S26733[44], 36}, {NULL}};
const Segment SGI30145[] = {{&S26733[30], 6}, {NULL}};
const Segment SGI30146[] = {{&S26733[128], 3}, {NULL}};
const Segment SGI30147[] = {{&S26733[94], 5}, {NULL}};
const Segment SGI30148[] = {{&S26734[44], 7}, {NULL}};
const Segment SGI30149[] = {{&S26734[44], 24}, {NULL}};
const Segment SGI30150[] = {{&S26734[30], 6}, {NULL}};
const Segment SGI30151[] = {{&S26734[116], 3}, {NULL}};
const Segment SGI30152[] = {{&S26734[82], 5}, {NULL}};
const Segment SGI30153[] = {{&S26736[44], 7}, {NULL}};
const Segment SGI30154[] = {{&S26736[44], 24}, {NULL}};
const Segment SGI30155[] = {{&S26736[30], 6}, {NULL}};
const Segment SGI30156[] = {{&S26736[116], 3}, {NULL}};
const Segment SGI30157[] = {{&S26736[82], 5}, {NULL}};
const Segment SGI30158[] = {{&S26737[44], 19}, {NULL}};
const Segment SGI30159[] = {{&S26737[44], 36}, {NULL}};
const Segment SGI30160[] = {{&S26737[30], 6}, {NULL}};
const Segment SGI30161[] = {{&S26737[128], 3}, {NULL}};
const Segment SGI30162[] = {{&S26737[94], 5}, {NULL}};
const Segment SGI30163[] = {{&S26738[44], 19}, {NULL}};
const Segment SGI30164[] = {{&S26738[44], 36}, {NULL}};
const Segment SGI30165[] = {{&S26738[30], 6}, {NULL}};
const Segment SGI30166[] = {{&S26738[128], 3}, {NULL}};
const Segment SGI30167[] = {{&S26738[94], 5}, {NULL}};
const Segment SGI30168[] = {{&S26755[44], 19}, {NULL}};
const Segment SGI30169[] = {{&S26755[44], 36}, {NULL}};
const Segment SGI30170[] = {{&S26755[30], 6}, {NULL}};
const Segment SGI30171[] = {{&S26755[128], 3}, {NULL}};
const Segment SGI30172[] = {{&S26755[94], 5}, {NULL}};
const Segment SGI30173[] = {{&S26769[38], 7}, {NULL}};
const Segment SGI30174[] = {{&S26769[31], 6}, {NULL}};
const Segment SGI30175[] = {{&S23147[44], 18}, {NULL}};
const Segment SGI30176[] = {{&S23147[44], 29}, {NULL}};
const Segment SGI30177[] = {{&S23147[30], 6}, {NULL}};
const Segment SGI30178[] = {{&S23147[121], 3}, {NULL}};
const Segment SGI30179[] = {{&S23147[87], 5}, {NULL}};
const Segment SGI30180[] = {{&S23155[44], 9}, {NULL}};
const Segment SGI30181[] = {{&S23155[44], 20}, {NULL}};
const Segment SGI30182[] = {{&S23155[30], 6}, {NULL}};
const Segment SGI30183[] = {{&S23155[112], 3}, {NULL}};
const Segment SGI30184[] = {{&S23155[78], 5}, {NULL}};
const Segment SGI30185[] = {{&S23159[42], 9}, {NULL}};
const Segment SGI30186[] = {{&S23159[30], 6}, {NULL}};
const Segment SGI30187[] = {{&S23159[110], 3}, {NULL}};
const Segment SGI30188[] = {{&S23159[76], 5}, {NULL}};
const Segment SGI30189[] = {{&S23160[44], 9}, {NULL}};
const Segment SGI30190[] = {{&S23160[44], 20}, {NULL}};
const Segment SGI30191[] = {{&S23160[30], 6}, {NULL}};
const Segment SGI30192[] = {{&S23160[114], 3}, {NULL}};
const Segment SGI30193[] = {{&S23160[78], 7}, {NULL}};
const Segment SGI30194[] = {{&S23160[134], 8}, {NULL}};
const Segment SGI30195[] = {{&S23161[44], 9}, {NULL}};
const Segment SGI30196[] = {{&S23161[44], 25}, {NULL}};
const Segment SGI30197[] = {{&S23161[30], 6}, {NULL}};
const Segment SGI30198[] = {{&S23163[44], 18}, {NULL}};
const Segment SGI30199[] = {{&S23163[44], 29}, {NULL}};
const Segment SGI30200[] = {{&S23163[30], 6}, {NULL}};
const Segment SGI30201[] = {{&S23163[121], 3}, {NULL}};
const Segment SGI30202[] = {{&S23163[87], 5}, {NULL}};
const Segment SGI30203[] = {{&S23164[44], 9}, {NULL}};
const Segment SGI30204[] = {{&S23164[44], 20}, {NULL}};
const Segment SGI30205[] = {{&S23164[30], 6}, {NULL}};
const Segment SGI30206[] = {{&S23164[112], 3}, {NULL}};
const Segment SGI30207[] = {{&S23164[78], 5}, {NULL}};
const Segment SGI30208[] = {{&S23165[38], 9}, {NULL}};
const Segment SGI30209[] = {{&S23165[31], 6}, {NULL}};
const Segment SGI30210[] = {{&S23167[44], 9}, {NULL}};
const Segment SGI30211[] = {{&S23167[44], 20}, {NULL}};
const Segment SGI30212[] = {{&S23167[30], 6}, {NULL}};
const Segment SGI30213[] = {{&S23167[112], 3}, {NULL}};
const Segment SGI30214[] = {{&S23167[78], 5}, {NULL}};
const Segment SGI30215[] = {{&S23170[42], 18}, {NULL}};
const Segment SGI30216[] = {{&S23170[30], 6}, {NULL}};
const Segment SGI30217[] = {{&S23170[119], 3}, {NULL}};
const Segment SGI30218[] = {{&S23170[85], 5}, {NULL}};
const Segment SGI30219[] = {{&S23173[44], 13}, {NULL}};
const Segment SGI30220[] = {{&S23173[44], 24}, {NULL}};
const Segment SGI30221[] = {{&S23173[30], 6}, {NULL}};
const Segment SGI30222[] = {{&S23173[116], 3}, {NULL}};
const Segment SGI30223[] = {{&S23173[82], 5}, {NULL}};
const Segment SGI30224[] = {{&S23179[44], 9}, {NULL}};
const Segment SGI30225[] = {{&S23179[44], 20}, {NULL}};
const Segment SGI30226[] = {{&S23179[30], 6}, {NULL}};
const Segment SGI30227[] = {{&S23179[78], 6}, {NULL}};
const Segment SGI30228[] = {{&S23858[44], 11}, {NULL}};
const Segment SGI30229[] = {{&S23858[44], 22}, {NULL}};
const Segment SGI30230[] = {{&S23858[30], 6}, {NULL}};
const Segment SGI30231[] = {{&S23858[114], 3}, {NULL}};
const Segment SGI30232[] = {{&S23858[80], 5}, {NULL}};
const Segment SGI30233[] = {{&S23861[44], 11}, {NULL}};
const Segment SGI30234[] = {{&S23861[44], 22}, {NULL}};
const Segment SGI30235[] = {{&S23861[30], 6}, {NULL}};
const Segment SGI30236[] = {{&S23861[83], 4}, {NULL}};
const Segment SGI30237[] = {{&S23861[83], 3}, {NULL}};
const Segment SGI30238[] = {{&S23863[44], 10}, {NULL}};
const Segment SGI30239[] = {{&S23863[44], 21}, {NULL}};
const Segment SGI30240[] = {{&S23863[30], 6}, {NULL}};
const Segment SGI30241[] = {{&S23863[113], 3}, {NULL}};
const Segment SGI30242[] = {{&S23863[79], 5}, {NULL}};
const Segment SGI30243[] = {{&S23867[44], 11}, {NULL}};
const Segment SGI30244[] = {{&S23867[44], 22}, {NULL}};
const Segment SGI30245[] = {{&S23867[30], 6}, {NULL}};
const Segment SGI30246[] = {{&S23867[83], 6}, {NULL}};
const Segment SGI30247[] = {{&S24023[44], 21}, {NULL}};
const Segment SGI30248[] = {{&S24023[44], 32}, {NULL}};
const Segment SGI30249[] = {{&S24023[30], 6}, {NULL}};
const Segment SGI30250[] = {{&S24023[124], 3}, {NULL}};
const Segment SGI30251[] = {{&S24023[90], 5}, {NULL}};
const Segment SGI30252[] = {{&S24024[44], 21}, {NULL}};
const Segment SGI30253[] = {{&S24024[44], 32}, {NULL}};
const Segment SGI30254[] = {{&S24024[30], 6}, {NULL}};
const Segment SGI30255[] = {{&S24024[124], 3}, {NULL}};
const Segment SGI30256[] = {{&S24024[90], 5}, {NULL}};
const Segment SGI30257[] = {{&S24025[44], 21}, {NULL}};
const Segment SGI30258[] = {{&S24025[44], 32}, {NULL}};
const Segment SGI30259[] = {{&S24025[30], 6}, {NULL}};
const Segment SGI30260[] = {{&S24025[124], 3}, {NULL}};
const Segment SGI30261[] = {{&S24025[90], 5}, {NULL}};
const Segment SGI30262[] = {{&S24029[43], 18}, {NULL}};
const Segment SGI30263[] = {{&S24029[43], 29}, {NULL}};
const Segment SGI30264[] = {{&S24029[120], 3}, {NULL}};
const Segment SGI30265[] = {{&S24029[86], 5}, {NULL}};
const Segment SGI30266[] = {{&S24030[44], 17}, {NULL}};
const Segment SGI30267[] = {{&S24030[44], 28}, {NULL}};
const Segment SGI30268[] = {{&S24030[30], 6}, {NULL}};
const Segment SGI30269[] = {{&S24030[120], 3}, {NULL}};
const Segment SGI30270[] = {{&S24030[86], 5}, {NULL}};
const Segment SGI30271[] = {{&S24031[44], 18}, {NULL}};
const Segment SGI30272[] = {{&S24031[44], 29}, {NULL}};
const Segment SGI30273[] = {{&S24031[30], 6}, {NULL}};
const Segment SGI30274[] = {{&S24031[121], 3}, {NULL}};
const Segment SGI30275[] = {{&S24031[87], 5}, {NULL}};
const Segment SGI30276[] = {{&S24032[43], 18}, {NULL}};
const Segment SGI30277[] = {{&S24032[43], 29}, {NULL}};
const Segment SGI30278[] = {{&S24032[30], 5}, {NULL}};
const Segment SGI30279[] = {{&S24032[120], 3}, {NULL}};
const Segment SGI30280[] = {{&S24032[86], 5}, {NULL}};
const Segment SGI30281[] = {{&S24033[43], 18}, {NULL}};
const Segment SGI30282[] = {{&S24033[43], 29}, {NULL}};
const Segment SGI30283[] = {{&S24033[120], 3}, {NULL}};
const Segment SGI30284[] = {{&S24033[86], 5}, {NULL}};
const Segment SGI30285[] = {{&S24035[44], 18}, {NULL}};
const Segment SGI30286[] = {{&S24035[44], 29}, {NULL}};
const Segment SGI30287[] = {{&S24035[30], 6}, {NULL}};
const Segment SGI30288[] = {{&S24035[121], 3}, {NULL}};
const Segment SGI30289[] = {{&S24035[87], 5}, {NULL}};
const Segment SGI30290[] = {{&S24037[44], 18}, {NULL}};
const Segment SGI30291[] = {{&S24037[44], 29}, {NULL}};
const Segment SGI30292[] = {{&S24037[30], 6}, {NULL}};
const Segment SGI30293[] = {{&S24037[121], 3}, {NULL}};
const Segment SGI30294[] = {{&S24037[87], 5}, {NULL}};
const Segment SGI30295[] = {{&S34837[44], 9}, {NULL}};
const Segment SGI30296[] = {{&S34837[44], 21}, {NULL}};
const Segment SGI30297[] = {{&S34837[30], 6}, {NULL}};
const Segment SGI30298[] = {{&S34837[113], 3}, {NULL}};
const Segment SGI30299[] = {{&S34837[79], 5}, {NULL}};
const Segment SGI30300[] = {{&S34838[44], 20}, {NULL}};
const Segment SGI30301[] = {{&S34838[30], 6}, {NULL}};
const Segment SGI30302[] = {{&S34838[112], 3}, {NULL}};
const Segment SGI30303[] = {{&S34838[78], 5}, {NULL}};
const Segment SGI30304[] = {{&S34839[8], 1}, {NULL}};
const Segment SGI30305[] = {{&S34839[53], 13}, {NULL}};
const Segment SGI30306[] = {{&S34839[39], 6}, {NULL}};
const Segment SGI30307[] = {{&S34839[114], 3}, {NULL}};
const Segment SGI30308[] = {{&S34839[80], 5}, {NULL}};
const Segment SGI30309[] = {{&S34840[44], 15}, {NULL}};
const Segment SGI30310[] = {{&S34840[44], 27}, {NULL}};
const Segment SGI30311[] = {{&S34840[30], 6}, {NULL}};
const Segment SGI30312[] = {{&S34840[119], 3}, {NULL}};
const Segment SGI30313[] = {{&S34840[85], 5}, {NULL}};
const Segment SGI30314[] = {{&S34841[44], 20}, {NULL}};
const Segment SGI30315[] = {{&S34841[30], 6}, {NULL}};
const Segment SGI30316[] = {{&S34841[112], 3}, {NULL}};
const Segment SGI30317[] = {{&S34841[78], 5}, {NULL}};
const Segment SGI30318[] = {{&S34842[44], 15}, {NULL}};
const Segment SGI30319[] = {{&S34842[44], 27}, {NULL}};
const Segment SGI30320[] = {{&S34842[30], 6}, {NULL}};
const Segment SGI30321[] = {{&S34842[119], 3}, {NULL}};
const Segment SGI30322[] = {{&S24049[44], 18}, {NULL}};
const Segment SGI30323[] = {{&S34842[85], 5}, {NULL}};
const Segment SGI30324[] = {{&S24049[44], 29}, {NULL}};
const Segment SGI30325[] = {{&S24049[30], 6}, {NULL}};
const Segment SGI30326[] = {{&S24049[121], 3}, {NULL}};
const Segment SGI30327[] = {{&S24049[87], 5}, {NULL}};
const Segment SGI30328[] = {{&S34843[7], 1}, {NULL}};
const Segment SGI30329[] = {{&S34843[52], 8}, {NULL}};
const Segment SGI30330[] = {{&S34843[38], 6}, {NULL}};
const Segment SGI30331[] = {{&S34843[122], 3}, {NULL}};
const Segment SGI30332[] = {{&S24050[44], 17}, {NULL}};
const Segment SGI30333[] = {{&S34843[88], 5}, {NULL}};
const Segment SGI30334[] = {{&S24050[44], 28}, {NULL}};
const Segment SGI30335[] = {{&S24050[30], 6}, {NULL}};
const Segment SGI30336[] = {{&S24050[120], 3}, {NULL}};
const Segment SGI30337[] = {{&S24050[86], 5}, {NULL}};
const Segment SGI30338[] = {{&S34844[44], 9}, {NULL}};
const Segment SGI30339[] = {{&S34844[44], 21}, {NULL}};
const Segment SGI30340[] = {{&S34844[30], 6}, {NULL}};
const Segment SGI30341[] = {{&S34844[113], 3}, {NULL}};
const Segment SGI30342[] = {{&S24051[44], 17}, {NULL}};
const Segment SGI30343[] = {{&S34844[79], 5}, {NULL}};
const Segment SGI30344[] = {{&S24051[44], 28}, {NULL}};
const Segment SGI30345[] = {{&S24051[30], 6}, {NULL}};
const Segment SGI30346[] = {{&S24051[120], 3}, {NULL}};
const Segment SGI30347[] = {{&S24051[86], 5}, {NULL}};
const Segment SGI30348[] = {{&S34845[44], 16}, {NULL}};
const Segment SGI30349[] = {{&S34845[44], 28}, {NULL}};
const Segment SGI30350[] = {{&S34845[30], 6}, {NULL}};
const Segment SGI30351[] = {{&S34845[120], 3}, {NULL}};
const Segment SGI30352[] = {{&S24052[44], 17}, {NULL}};
const Segment SGI30353[] = {{&S34845[86], 5}, {NULL}};
const Segment SGI30354[] = {{&S24052[44], 28}, {NULL}};
const Segment SGI30355[] = {{&S24052[30], 6}, {NULL}};
const Segment SGI30356[] = {{&S24052[120], 3}, {NULL}};
const Segment SGI30357[] = {{&S24052[86], 5}, {NULL}};
const Segment SGI30358[] = {{&S34846[44], 15}, {NULL}};
const Segment SGI30359[] = {{&S34846[44], 27}, {NULL}};
const Segment SGI30360[] = {{&S34846[30], 6}, {NULL}};
const Segment SGI30361[] = {{&S34846[119], 3}, {NULL}};
const Segment SGI30362[] = {{&S34846[85], 5}, {NULL}};
const Segment SGI30363[] = {{&S34847[38], 9}, {NULL}};
const Segment SGI30364[] = {{&S34847[31], 6}, {NULL}};
const Segment SGI30365[] = {{&S34848[44], 10}, {NULL}};
const Segment SGI30366[] = {{&S34848[44], 22}, {NULL}};
const Segment SGI30367[] = {{&S34848[30], 6}, {NULL}};
const Segment SGI30368[] = {{&S34848[114], 3}, {NULL}};
const Segment SGI30369[] = {{&S34848[80], 5}, {NULL}};
const Segment SGI30370[] = {{&S24057[42], 18}, {NULL}};
const Segment SGI30371[] = {{&S24057[30], 6}, {NULL}};
const Segment SGI30372[] = {{&S24057[119], 3}, {NULL}};
const Segment SGI30373[] = {{&S24057[85], 5}, {NULL}};
const Segment SGI30374[] = {{&S34849[44], 20}, {NULL}};
const Segment SGI30375[] = {{&S34849[30], 6}, {NULL}};
const Segment SGI30376[] = {{&S34849[112], 3}, {NULL}};
const Segment SGI30377[] = {{&S34849[78], 5}, {NULL}};
const Segment SGI30378[] = {{&S34850[44], 15}, {NULL}};
const Segment SGI30379[] = {{&S34850[44], 27}, {NULL}};
const Segment SGI30380[] = {{&S34850[30], 6}, {NULL}};
const Segment SGI30381[] = {{&S34850[119], 3}, {NULL}};
const Segment SGI30382[] = {{&S34850[85], 5}, {NULL}};
const Segment SGI30383[] = {{&S34851[38], 10}, {NULL}};
const Segment SGI30384[] = {{&S34851[31], 6}, {NULL}};
const Segment SGI30385[] = {{&S34852[44], 15}, {NULL}};
const Segment SGI30386[] = {{&S34852[44], 27}, {NULL}};
const Segment SGI30387[] = {{&S34852[30], 6}, {NULL}};
const Segment SGI30388[] = {{&S34852[119], 3}, {NULL}};
const Segment SGI30389[] = {{&S24064[38], 21}, {NULL}};
const Segment SGI30390[] = {{&S34852[85], 5}, {NULL}};
const Segment SGI30391[] = {{&S24064[31], 6}, {NULL}};
const Segment SGI30392[] = {{&S34853[44], 9}, {NULL}};
const Segment SGI30393[] = {{&S24065[38], 18}, {NULL}};
const Segment SGI30394[] = {{&S34853[44], 21}, {NULL}};
const Segment SGI30395[] = {{&S34853[30], 6}, {NULL}};
const Segment SGI30396[] = {{&S24065[31], 6}, {NULL}};
const Segment SGI30397[] = {{&S34853[82], 6}, {NULL}};
const Segment SGI30398[] = {{&S34854[44], 20}, {NULL}};
const Segment SGI30399[] = {{&S34854[30], 6}, {NULL}};
const Segment SGI30400[] = {{&S24067[44], 18}, {NULL}};
const Segment SGI30401[] = {{&S24067[44], 29}, {NULL}};
const Segment SGI30402[] = {{&S34854[112], 3}, {NULL}};
const Segment SGI30403[] = {{&S24067[30], 6}, {NULL}};
const Segment SGI30404[] = {{&S34854[78], 5}, {NULL}};
const Segment SGI30405[] = {{&S24068[43], 18}, {NULL}};
const Segment SGI30406[] = {{&S24068[43], 29}, {NULL}};
const Segment SGI30407[] = {{&S24068[30], 5}, {NULL}};
const Segment SGI30408[] = {{&S24068[120], 3}, {NULL}};
const Segment SGI30409[] = {{&S24068[86], 5}, {NULL}};
const Segment SGI30410[] = {{&S24069[44], 17}, {NULL}};
const Segment SGI30411[] = {{&S24069[44], 28}, {NULL}};
const Segment SGI30412[] = {{&S24069[30], 6}, {NULL}};
const Segment SGI30413[] = {{&S24069[120], 3}, {NULL}};
const Segment SGI30414[] = {{&S24069[86], 5}, {NULL}};
const Segment SGI30415[] = {{&S24070[44], 17}, {NULL}};
const Segment SGI30416[] = {{&S24070[44], 28}, {NULL}};
const Segment SGI30417[] = {{&S24070[30], 6}, {NULL}};
const Segment SGI30418[] = {{&S24070[120], 3}, {NULL}};
const Segment SGI30419[] = {{&S24070[86], 5}, {NULL}};
const Segment SGI30420[] = {{&S24072[44], 21}, {NULL}};
const Segment SGI30421[] = {{&S24072[44], 32}, {NULL}};
const Segment SGI30422[] = {{&S24072[30], 6}, {NULL}};
const Segment SGI30423[] = {{&S24072[93], 4}, {NULL}};
const Segment SGI30424[] = {{&S24072[93], 3}, {NULL}};
const Segment SGI30425[] = {{&S24075[44], 11}, {NULL}};
const Segment SGI30426[] = {{&S24075[44], 22}, {NULL}};
const Segment SGI30427[] = {{&S24075[30], 6}, {NULL}};
const Segment SGI30428[] = {{&S24075[83], 4}, {NULL}};
const Segment SGI30429[] = {{&S24075[83], 3}, {NULL}};
const Segment SGI30430[] = {{&S24077[44], 11}, {NULL}};
const Segment SGI30431[] = {{&S24077[44], 22}, {NULL}};
const Segment SGI30432[] = {{&S24077[30], 6}, {NULL}};
const Segment SGI30433[] = {{&S24077[114], 3}, {NULL}};
const Segment SGI30434[] = {{&S24077[80], 5}, {NULL}};
const Segment SGI30435[] = {{&S24080[44], 21}, {NULL}};
const Segment SGI30436[] = {{&S24080[44], 32}, {NULL}};
const Segment SGI30437[] = {{&S24080[30], 6}, {NULL}};
const Segment SGI30438[] = {{&S24080[93], 6}, {NULL}};
const Segment SGI30439[] = {{&S24082[44], 18}, {NULL}};
const Segment SGI30440[] = {{&S24082[44], 29}, {NULL}};
const Segment SGI30441[] = {{&S24082[30], 6}, {NULL}};
const Segment SGI30442[] = {{&S24082[123], 3}, {NULL}};
const Segment SGI30443[] = {{&S24082[87], 7}, {NULL}};
const Segment SGI30444[] = {{&S24082[143], 8}, {NULL}};
const Segment SGI30445[] = {{&S24083[42], 21}, {NULL}};
const Segment SGI30446[] = {{&S24083[30], 6}, {NULL}};
const Segment SGI30447[] = {{&S24083[122], 3}, {NULL}};
const Segment SGI30448[] = {{&S24083[88], 5}, {NULL}};
const Segment SGI30449[] = {{&S24086[44], 18}, {NULL}};
const Segment SGI30450[] = {{&S24086[44], 29}, {NULL}};
const Segment SGI30451[] = {{&S24086[30], 6}, {NULL}};
const Segment SGI30452[] = {{&S24086[90], 4}, {NULL}};
const Segment SGI30453[] = {{&S24086[90], 3}, {NULL}};
const Segment SGI30454[] = {{&S24087[42], 18}, {NULL}};
const Segment SGI30455[] = {{&S24087[30], 6}, {NULL}};
const Segment SGI30456[] = {{&S24087[119], 3}, {NULL}};
const Segment SGI30457[] = {{&S24087[85], 5}, {NULL}};
const Segment SGI30458[] = {{&S24090[44], 11}, {NULL}};
const Segment SGI30459[] = {{&S24090[44], 22}, {NULL}};
const Segment SGI30460[] = {{&S24090[30], 6}, {NULL}};
const Segment SGI30461[] = {{&S24090[83], 6}, {NULL}};
const Segment SGI30462[] = {{&S23532[44], 7}, {NULL}};
const Segment SGI30463[] = {{&S23532[44], 29}, {NULL}};
const Segment SGI30464[] = {{&S23532[30], 6}, {NULL}};
const Segment SGI30465[] = {{&S23532[121], 3}, {NULL}};
const Segment SGI30466[] = {{&S23532[87], 5}, {NULL}};
const Segment SGI30467[] = {{&S23555[44], 7}, {NULL}};
const Segment SGI30468[] = {{&S23555[44], 20}, {NULL}};
const Segment SGI30469[] = {{&S23555[30], 6}, {NULL}};
const Segment SGI30470[] = {{&S23555[112], 3}, {NULL}};
const Segment SGI30471[] = {{&S23555[78], 5}, {NULL}};
const Segment SGI30472[] = {{&S23559[44], 7}, {NULL}};
const Segment SGI30473[] = {{&S23559[44], 20}, {NULL}};
const Segment SGI30474[] = {{&S23559[30], 6}, {NULL}};
const Segment SGI30475[] = {{&S23559[112], 3}, {NULL}};
const Segment SGI30476[] = {{&S23559[78], 5}, {NULL}};
const Segment SGI30477[] = {{&S23560[44], 7}, {NULL}};
const Segment SGI30478[] = {{&S23560[44], 20}, {NULL}};
const Segment SGI30479[] = {{&S23560[30], 6}, {NULL}};
const Segment SGI30480[] = {{&S23560[112], 3}, {NULL}};
const Segment SGI30481[] = {{&S23560[78], 5}, {NULL}};
const Segment SGI30482[] = {{&S23561[44], 7}, {NULL}};
const Segment SGI30483[] = {{&S23561[44], 19}, {NULL}};
const Segment SGI30484[] = {{&S23561[30], 6}, {NULL}};
const Segment SGI30485[] = {{&S23561[111], 3}, {NULL}};
const Segment SGI30486[] = {{&S23561[77], 5}, {NULL}};
const Segment SGI30487[] = {{&S23575[44], 7}, {NULL}};
const Segment SGI30488[] = {{&S23575[44], 27}, {NULL}};
const Segment SGI30489[] = {{&S23575[30], 6}, {NULL}};
const Segment SGI30490[] = {{&S23575[119], 3}, {NULL}};
const Segment SGI30491[] = {{&S23575[85], 5}, {NULL}};
const Segment SGI30492[] = {{&S23576[44], 7}, {NULL}};
const Segment SGI30493[] = {{&S23576[44], 20}, {NULL}};
const Segment SGI30494[] = {{&S23576[30], 6}, {NULL}};
const Segment SGI30495[] = {{&S23576[112], 3}, {NULL}};
const Segment SGI30496[] = {{&S23576[78], 5}, {NULL}};
const Segment SGI30497[] = {{&S23586[38], 7}, {NULL}};
const Segment SGI30498[] = {{&S23586[31], 6}, {NULL}};
const Segment SGI30499[] = {{&S25610[44], 7}, {NULL}};
const Segment SGI30500[] = {{&S25610[44], 29}, {NULL}};
const Segment SGI30501[] = {{&S25610[30], 6}, {NULL}};
const Segment SGI30502[] = {{&S25610[121], 3}, {NULL}};
const Segment SGI30503[] = {{&S25610[87], 5}, {NULL}};
const Segment SGI30504[] = {{&S24449[44], 10}, {NULL}};
const Segment SGI30505[] = {{&S24449[44], 27}, {NULL}};
const Segment SGI30506[] = {{&S24449[30], 6}, {NULL}};
const Segment SGI30507[] = {{&S24449[88], 4}, {NULL}};
const Segment SGI30508[] = {{&S24449[88], 3}, {NULL}};
const Segment SGI30509[] = {{&S24455[44], 10}, {NULL}};
const Segment SGI30510[] = {{&S24455[44], 27}, {NULL}};
const Segment SGI30511[] = {{&S24455[30], 6}, {NULL}};
const Segment SGI30512[] = {{&S24455[119], 3}, {NULL}};
const Segment SGI30513[] = {{&S24455[85], 5}, {NULL}};
const Segment SGI30514[] = {{&S24460[44], 10}, {NULL}};
const Segment SGI30515[] = {{&S24460[44], 21}, {NULL}};
const Segment SGI30516[] = {{&S24460[30], 6}, {NULL}};
const Segment SGI30517[] = {{&S24460[113], 3}, {NULL}};
const Segment SGI30518[] = {{&S24460[79], 5}, {NULL}};
const Segment SGI30519[] = {{&S24466[44], 10}, {NULL}};
const Segment SGI30520[] = {{&S24466[44], 21}, {NULL}};
const Segment SGI30521[] = {{&S24466[30], 6}, {NULL}};
const Segment SGI30522[] = {{&S24466[113], 3}, {NULL}};
const Segment SGI30523[] = {{&S24466[79], 5}, {NULL}};
const Segment SGI30524[] = {{&S24470[44], 10}, {NULL}};
const Segment SGI30525[] = {{&S24470[44], 27}, {NULL}};
const Segment SGI30526[] = {{&S24470[30], 6}, {NULL}};
const Segment SGI30527[] = {{&S24470[88], 6}, {NULL}};
const Segment SGI30528[] = {{&S24480[44], 15}, {NULL}};
const Segment SGI30529[] = {{&S24480[44], 32}, {NULL}};
const Segment SGI30530[] = {{&S24480[30], 6}, {NULL}};
const Segment SGI30531[] = {{&S24480[124], 3}, {NULL}};
const Segment SGI30532[] = {{&S24480[90], 5}, {NULL}};
const Segment SGI30533[] = {{&S24483[38], 10}, {NULL}};
const Segment SGI30534[] = {{&S24483[31], 6}, {NULL}};
const Segment SGI30535[] = {{&S24494[44], 10}, {NULL}};
const Segment SGI30536[] = {{&S24494[44], 20}, {NULL}};
const Segment SGI30537[] = {{&S24494[30], 6}, {NULL}};
const Segment SGI30538[] = {{&S24494[112], 3}, {NULL}};
const Segment SGI30539[] = {{&S24494[78], 5}, {NULL}};
const Segment SGI30540[] = {{&S24513[42], 10}, {NULL}};
const Segment SGI30541[] = {{&S24513[30], 6}, {NULL}};
const Segment SGI30542[] = {{&S24513[117], 3}, {NULL}};
const Segment SGI30543[] = {{&S24513[83], 5}, {NULL}};
const Segment SGI30544[] = {{&S30608[44], 11}, {NULL}};
const Segment SGI30545[] = {{&S30608[44], 28}, {NULL}};
const Segment SGI30546[] = {{&S30608[30], 6}, {NULL}};
const Segment SGI30547[] = {{&S30608[120], 3}, {NULL}};
const Segment SGI30548[] = {{&S30608[86], 5}, {NULL}};
const Segment SGI30549[] = {{&S30609[44], 11}, {NULL}};
const Segment SGI30550[] = {{&S30609[44], 22}, {NULL}};
const Segment SGI30551[] = {{&S30609[30], 6}, {NULL}};
const Segment SGI30552[] = {{&S30609[114], 3}, {NULL}};
const Segment SGI30553[] = {{&S30609[80], 5}, {NULL}};
const Segment SGI30554[] = {{&S30620[44], 11}, {NULL}};
const Segment SGI30555[] = {{&S30620[44], 23}, {NULL}};
const Segment SGI30556[] = {{&S30620[30], 6}, {NULL}};
const Segment SGI30557[] = {{&S30620[115], 3}, {NULL}};
const Segment SGI30558[] = {{&S30620[81], 5}, {NULL}};
const Segment SGI30559[] = {{&S24533[44], 10}, {NULL}};
const Segment SGI30560[] = {{&S24533[44], 21}, {NULL}};
const Segment SGI30561[] = {{&S24533[30], 6}, {NULL}};
const Segment SGI30562[] = {{&S24533[113], 3}, {NULL}};
const Segment SGI30563[] = {{&S24533[79], 5}, {NULL}};
const Segment SGI30564[] = {{&S35131[44], 14}, {NULL}};
const Segment SGI30565[] = {{&S35131[44], 26}, {NULL}};
const Segment SGI30566[] = {{&S35131[30], 6}, {NULL}};
const Segment SGI30567[] = {{&S35131[118], 3}, {NULL}};
const Segment SGI30568[] = {{&S35131[84], 5}, {NULL}};
const Segment SGI30569[] = {{&S35132[44], 14}, {NULL}};
const Segment SGI30570[] = {{&S35132[44], 26}, {NULL}};
const Segment SGI30571[] = {{&S35132[30], 6}, {NULL}};
const Segment SGI30572[] = {{&S35132[118], 3}, {NULL}};
const Segment SGI30573[] = {{&S35132[84], 5}, {NULL}};
const Segment SGI30574[] = {{&S35133[44], 14}, {NULL}};
const Segment SGI30575[] = {{&S35133[44], 26}, {NULL}};
const Segment SGI30576[] = {{&S35133[30], 6}, {NULL}};
const Segment SGI30577[] = {{&S35133[118], 3}, {NULL}};
const Segment SGI30578[] = {{&S35133[84], 5}, {NULL}};
const Segment SGI30579[] = {{&S35134[44], 14}, {NULL}};
const Segment SGI30580[] = {{&S35134[44], 26}, {NULL}};
const Segment SGI30581[] = {{&S35134[30], 6}, {NULL}};
const Segment SGI30582[] = {{&S35134[118], 3}, {NULL}};
const Segment SGI30583[] = {{&S35134[84], 5}, {NULL}};
const Segment SGI30584[] = {{&S23838[44], 18}, {NULL}};
const Segment SGI30585[] = {{&S23838[44], 29}, {NULL}};
const Segment SGI30586[] = {{&S23838[30], 6}, {NULL}};
const Segment SGI30587[] = {{&S23838[121], 3}, {NULL}};
const Segment SGI30588[] = {{&S23838[87], 5}, {NULL}};
const Segment SGI30589[] = {{&S35135[44], 9}, {NULL}};
const Segment SGI30590[] = {{&S35135[44], 21}, {NULL}};
const Segment SGI30591[] = {{&S35135[30], 6}, {NULL}};
const Segment SGI30592[] = {{&S35135[113], 3}, {NULL}};
const Segment SGI30593[] = {{&S35135[79], 5}, {NULL}};
const Segment SGI30594[] = {{&S35136[41], 9}, {NULL}};
const Segment SGI30595[] = {{&S35136[30], 6}, {NULL}};
const Segment SGI30596[] = {{&S35136[110], 3}, {NULL}};
const Segment SGI30597[] = {{&S35136[76], 5}, {NULL}};
const Segment SGI30598[] = {{&S23842[44], 21}, {NULL}};
const Segment SGI30599[] = {{&S23842[44], 32}, {NULL}};
const Segment SGI30600[] = {{&S23842[30], 6}, {NULL}};
const Segment SGI30601[] = {{&S35137[44], 14}, {NULL}};
const Segment SGI30602[] = {{&S35137[44], 26}, {NULL}};
const Segment SGI30603[] = {{&S35137[30], 6}, {NULL}};
const Segment SGI30604[] = {{&S23842[124], 3}, {NULL}};
const Segment SGI30605[] = {{&S23842[90], 5}, {NULL}};
const Segment SGI30606[] = {{&S35137[118], 3}, {NULL}};
const Segment SGI30607[] = {{&S35137[84], 5}, {NULL}};
const Segment SGI30608[] = {{&S23843[44], 25}, {NULL}};
const Segment SGI30609[] = {{&S23843[44], 36}, {NULL}};
const Segment SGI30610[] = {{&S23843[30], 6}, {NULL}};
const Segment SGI30611[] = {{&S35138[44], 10}, {NULL}};
const Segment SGI30612[] = {{&S35138[44], 22}, {NULL}};
const Segment SGI30613[] = {{&S23843[128], 3}, {NULL}};
const Segment SGI30614[] = {{&S35138[30], 6}, {NULL}};
const Segment SGI30615[] = {{&S23843[94], 5}, {NULL}};
const Segment SGI30616[] = {{&S35138[114], 3}, {NULL}};
const Segment SGI30617[] = {{&S35138[80], 5}, {NULL}};
const Segment SGI30618[] = {{&S35139[44], 11}, {NULL}};
const Segment SGI30619[] = {{&S35139[44], 23}, {NULL}};
const Segment SGI30620[] = {{&S35139[30], 6}, {NULL}};
const Segment SGI30621[] = {{&S23845[38], 18}, {NULL}};
const Segment SGI30622[] = {{&S23845[31], 6}, {NULL}};
const Segment SGI30623[] = {{&S35139[115], 3}, {NULL}};
const Segment SGI30624[] = {{&S35139[81], 5}, {NULL}};
const Segment SGI30625[] = {{&S35140[44], 10}, {NULL}};
const Segment SGI30626[] = {{&S23846[44], 18}, {NULL}};
const Segment SGI30627[] = {{&S35140[44], 22}, {NULL}};
const Segment SGI30628[] = {{&S23846[44], 29}, {NULL}};
const Segment SGI30629[] = {{&S35140[30], 6}, {NULL}};
const Segment SGI30630[] = {{&S23846[30], 6}, {NULL}};
const Segment SGI30631[] = {{&S35140[114], 3}, {NULL}};
const Segment SGI30632[] = {{&S23846[121], 3}, {NULL}};
const Segment SGI30633[] = {{&S23846[87], 5}, {NULL}};
const Segment SGI30634[] = {{&S35140[80], 5}, {NULL}};
const Segment SGI30635[] = {{&S23847[44], 18}, {NULL}};
const Segment SGI30636[] = {{&S23847[44], 29}, {NULL}};
const Segment SGI30637[] = {{&S23847[30], 6}, {NULL}};
const Segment SGI30638[] = {{&S23847[121], 3}, {NULL}};
const Segment SGI30639[] = {{&S23847[87], 5}, {NULL}};
const Segment SGI30640[] = {{&S35142[44], 14}, {NULL}};
const Segment SGI30641[] = {{&S35142[44], 26}, {NULL}};
const Segment SGI30642[] = {{&S35142[30], 6}, {NULL}};
const Segment SGI30643[] = {{&S35142[118], 3}, {NULL}};
const Segment SGI30644[] = {{&S23848[44], 21}, {NULL}};
const Segment SGI30645[] = {{&S23848[44], 32}, {NULL}};
const Segment SGI30646[] = {{&S23848[30], 6}, {NULL}};
const Segment SGI30647[] = {{&S35142[84], 5}, {NULL}};
const Segment SGI30648[] = {{&S23848[124], 3}, {NULL}};
const Segment SGI30649[] = {{&S23848[90], 5}, {NULL}};
const Segment SGI30650[] = {{&S35143[38], 9}, {NULL}};
const Segment SGI30651[] = {{&S35143[31], 6}, {NULL}};
const Segment SGI30652[] = {{&S23849[44], 18}, {NULL}};
const Segment SGI30653[] = {{&S23849[44], 29}, {NULL}};
const Segment SGI30654[] = {{&S23849[30], 6}, {NULL}};
const Segment SGI30655[] = {{&S23849[121], 3}, {NULL}};
const Segment SGI30656[] = {{&S23849[87], 5}, {NULL}};
const Segment SGI30657[] = {{&S35144[44], 9}, {NULL}};
const Segment SGI30658[] = {{&S35144[44], 29}, {NULL}};
const Segment SGI30659[] = {{&S35144[30], 6}, {NULL}};
const Segment SGI30660[] = {{&S35144[121], 3}, {NULL}};
const Segment SGI30661[] = {{&S35144[87], 5}, {NULL}};
const Segment SGI30662[] = {{&S23850[44], 23}, {NULL}};
const Segment SGI30663[] = {{&S23850[44], 34}, {NULL}};
const Segment SGI30664[] = {{&S23850[30], 6}, {NULL}};
const Segment SGI30665[] = {{&S23850[126], 3}, {NULL}};
const Segment SGI30666[] = {{&S35145[44], 9}, {NULL}};
const Segment SGI30667[] = {{&S35145[44], 21}, {NULL}};
const Segment SGI30668[] = {{&S23850[92], 5}, {NULL}};
const Segment SGI30669[] = {{&S35145[30], 6}, {NULL}};
const Segment SGI30670[] = {{&S35145[82], 4}, {NULL}};
const Segment SGI30671[] = {{&S35145[82], 3}, {NULL}};
const Segment SGI30672[] = {{&S23851[44], 12}, {NULL}};
const Segment SGI30673[] = {{&S23851[44], 23}, {NULL}};
const Segment SGI30674[] = {{&S23851[30], 6}, {NULL}};
const Segment SGI30675[] = {{&S35146[38], 11}, {NULL}};
const Segment SGI30676[] = {{&S35146[31], 6}, {NULL}};
const Segment SGI30677[] = {{&S23851[115], 3}, {NULL}};
const Segment SGI30678[] = {{&S23851[81], 5}, {NULL}};
const Segment SGI30679[] = {{&S35147[38], 10}, {NULL}};
const Segment SGI30680[] = {{&S35147[31], 6}, {NULL}};
const Segment SGI30681[] = {{&S23852[44], 23}, {NULL}};
const Segment SGI30682[] = {{&S23852[44], 34}, {NULL}};
const Segment SGI30683[] = {{&S23852[30], 6}, {NULL}};
const Segment SGI30684[] = {{&S23852[126], 3}, {NULL}};
const Segment SGI30685[] = {{&S23852[92], 5}, {NULL}};
const Segment SGI30686[] = {{&S35148[44], 9}, {NULL}};
const Segment SGI30687[] = {{&S35148[44], 21}, {NULL}};
const Segment SGI30688[] = {{&S35148[30], 6}, {NULL}};
const Segment SGI30689[] = {{&S35148[82], 6}, {NULL}};
const Segment SGI30690[] = {{&S23853[44], 22}, {NULL}};
const Segment SGI30691[] = {{&S23853[44], 33}, {NULL}};
const Segment SGI30692[] = {{&S23853[30], 6}, {NULL}};
const Segment SGI30693[] = {{&S23853[125], 3}, {NULL}};
const Segment SGI30694[] = {{&S23853[91], 5}, {NULL}};
const Segment SGI30695[] = {{&S23854[44], 23}, {NULL}};
const Segment SGI30696[] = {{&S23854[44], 34}, {NULL}};
const Segment SGI30697[] = {{&S23854[30], 6}, {NULL}};
const Segment SGI30698[] = {{&S23854[126], 3}, {NULL}};
const Segment SGI30699[] = {{&S23854[92], 5}, {NULL}};
const Segment SGI30700[] = {{&S23856[44], 23}, {NULL}};
const Segment SGI30701[] = {{&S23856[44], 34}, {NULL}};
const Segment SGI30702[] = {{&S23856[30], 6}, {NULL}};
const Segment SGI30703[] = {{&S23856[126], 3}, {NULL}};
const Segment SGI30704[] = {{&S23856[92], 5}, {NULL}};
const Segment SGI30705[] = {{&S23857[35], 1}, {NULL}};
const Segment SGI30706[] = {{&S23857[59], 13}, {NULL}};
const Segment SGI30707[] = {{&S23857[46], 6}, {NULL}};
const Segment SGI30708[] = {{&S23857[120], 3}, {NULL}};
const Segment SGI30709[] = {{&S23857[86], 5}, {NULL}};
const Segment SGI30710[] = {{&S23860[44], 23}, {NULL}};
const Segment SGI30711[] = {{&S23860[44], 34}, {NULL}};
const Segment SGI30712[] = {{&S23860[30], 6}, {NULL}};
const Segment SGI30713[] = {{&S23860[126], 3}, {NULL}};
const Segment SGI30714[] = {{&S23860[92], 5}, {NULL}};
const Segment SGI30715[] = {{&S23862[44], 23}, {NULL}};
const Segment SGI30716[] = {{&S23862[44], 34}, {NULL}};
const Segment SGI30717[] = {{&S23862[30], 6}, {NULL}};
const Segment SGI30718[] = {{&S23862[126], 3}, {NULL}};
const Segment SGI30719[] = {{&S23862[92], 5}, {NULL}};
const Segment SGI30720[] = {{&S23864[44], 23}, {NULL}};
const Segment SGI30721[] = {{&S23864[44], 34}, {NULL}};
const Segment SGI30722[] = {{&S23864[30], 6}, {NULL}};
const Segment SGI30723[] = {{&S23864[126], 3}, {NULL}};
const Segment SGI30724[] = {{&S23864[92], 5}, {NULL}};
const Segment SGI30725[] = {{&S23865[44], 18}, {NULL}};
const Segment SGI30726[] = {{&S23865[44], 29}, {NULL}};
const Segment SGI30727[] = {{&S23865[30], 6}, {NULL}};
const Segment SGI30728[] = {{&S23865[90], 4}, {NULL}};
const Segment SGI30729[] = {{&S23865[90], 3}, {NULL}};
const Segment SGI30730[] = {{&S23866[44], 23}, {NULL}};
const Segment SGI30731[] = {{&S23866[44], 34}, {NULL}};
const Segment SGI30732[] = {{&S23866[30], 6}, {NULL}};
const Segment SGI30733[] = {{&S23866[126], 3}, {NULL}};
const Segment SGI30734[] = {{&S23866[92], 5}, {NULL}};
const Segment SGI30735[] = {{&S23869[44], 23}, {NULL}};
const Segment SGI30736[] = {{&S23869[44], 34}, {NULL}};
const Segment SGI30737[] = {{&S23869[30], 6}, {NULL}};
const Segment SGI30738[] = {{&S23869[126], 3}, {NULL}};
const Segment SGI30739[] = {{&S23869[92], 5}, {NULL}};
const Segment SGI30740[] = {{&S23871[44], 18}, {NULL}};
const Segment SGI30741[] = {{&S23871[44], 29}, {NULL}};
const Segment SGI30742[] = {{&S23871[30], 6}, {NULL}};
const Segment SGI30743[] = {{&S23871[90], 6}, {NULL}};
const Segment SGI30744[] = {{&S23872[44], 23}, {NULL}};
const Segment SGI30745[] = {{&S23872[44], 34}, {NULL}};
const Segment SGI30746[] = {{&S23872[30], 6}, {NULL}};
const Segment SGI30747[] = {{&S23872[126], 3}, {NULL}};
const Segment SGI30748[] = {{&S23872[92], 5}, {NULL}};
const Segment SGI30749[] = {{&S23873[44], 23}, {NULL}};
const Segment SGI30750[] = {{&S23873[44], 34}, {NULL}};
const Segment SGI30751[] = {{&S23873[30], 6}, {NULL}};
const Segment SGI30752[] = {{&S23873[126], 3}, {NULL}};
const Segment SGI30753[] = {{&S23873[92], 5}, {NULL}};
const Segment SGI30754[] = {{&S23874[44], 23}, {NULL}};
const Segment SGI30755[] = {{&S23874[44], 34}, {NULL}};
const Segment SGI30756[] = {{&S23874[30], 6}, {NULL}};
const Segment SGI30757[] = {{&S23874[126], 3}, {NULL}};
const Segment SGI30758[] = {{&S23874[92], 5}, {NULL}};
const Segment SGI30759[] = {{&S24573[44], 10}, {NULL}};
const Segment SGI30760[] = {{&S24573[44], 27}, {NULL}};
const Segment SGI30761[] = {{&S24573[30], 6}, {NULL}};
const Segment SGI30762[] = {{&S24573[132], 3}, {NULL}};
const Segment SGI30763[] = {{&S24573[85], 6}, {NULL}};
const Segment SGI30764[] = {{&S25852[44], 19}, {NULL}};
const Segment SGI30765[] = {{&S25852[44], 38}, {NULL}};
const Segment SGI30766[] = {{&S25852[30], 6}, {NULL}};
const Segment SGI30767[] = {{&S25852[130], 3}, {NULL}};
const Segment SGI30768[] = {{&S25852[96], 5}, {NULL}};
const Segment SGI30769[] = {{&S25854[44], 19}, {NULL}};
const Segment SGI30770[] = {{&S25854[44], 38}, {NULL}};
const Segment SGI30771[] = {{&S25854[30], 6}, {NULL}};
const Segment SGI30772[] = {{&S25854[130], 3}, {NULL}};
const Segment SGI30773[] = {{&S25854[96], 5}, {NULL}};
const Segment SGI30774[] = {{&S25855[44], 19}, {NULL}};
const Segment SGI30775[] = {{&S25855[44], 37}, {NULL}};
const Segment SGI30776[] = {{&S25855[30], 6}, {NULL}};
const Segment SGI30777[] = {{&S25855[129], 3}, {NULL}};
const Segment SGI30778[] = {{&S25855[95], 5}, {NULL}};
const Segment SGI30779[] = {{&S25856[44], 19}, {NULL}};
const Segment SGI30780[] = {{&S25856[44], 38}, {NULL}};
const Segment SGI30781[] = {{&S25856[30], 6}, {NULL}};
const Segment SGI30782[] = {{&S25856[130], 3}, {NULL}};
const Segment SGI30783[] = {{&S25856[96], 5}, {NULL}};
const Segment SGI30784[] = {{&S25857[44], 19}, {NULL}};
const Segment SGI30785[] = {{&S25857[44], 38}, {NULL}};
const Segment SGI30786[] = {{&S25857[30], 6}, {NULL}};
const Segment SGI30787[] = {{&S25857[130], 3}, {NULL}};
const Segment SGI30788[] = {{&S25857[96], 5}, {NULL}};
const Segment SGI30789[] = {{&S25859[44], 19}, {NULL}};
const Segment SGI30790[] = {{&S25859[44], 36}, {NULL}};
const Segment SGI30791[] = {{&S25859[30], 6}, {NULL}};
const Segment SGI30792[] = {{&S25859[69], 3}, {NULL}};
const Segment SGI30793[] = {{&S25859[94], 5}, {NULL}};
const Segment SGI30794[] = {{&S25861[44], 7}, {NULL}};
const Segment SGI30795[] = {{&S25861[44], 26}, {NULL}};
const Segment SGI30796[] = {{&S25861[30], 6}, {NULL}};
const Segment SGI30797[] = {{&S25861[118], 3}, {NULL}};
const Segment SGI30798[] = {{&S25861[84], 5}, {NULL}};
const Segment SGI30799[] = {{&S25862[44], 20}, {NULL}};
const Segment SGI30800[] = {{&S25862[44], 39}, {NULL}};
const Segment SGI30801[] = {{&S25862[30], 6}, {NULL}};
const Segment SGI30802[] = {{&S25862[131], 3}, {NULL}};
const Segment SGI30803[] = {{&S25862[97], 5}, {NULL}};
const Segment SGI30804[] = {{&S25863[44], 21}, {NULL}};
const Segment SGI30805[] = {{&S25863[44], 40}, {NULL}};
const Segment SGI30806[] = {{&S25863[30], 6}, {NULL}};
const Segment SGI30807[] = {{&S25863[132], 3}, {NULL}};
const Segment SGI30808[] = {{&S25863[98], 5}, {NULL}};
const Segment SGI30809[] = {{&S25866[44], 20}, {NULL}};
const Segment SGI30810[] = {{&S25866[44], 37}, {NULL}};
const Segment SGI30811[] = {{&S25866[30], 6}, {NULL}};
const Segment SGI30812[] = {{&S25866[70], 3}, {NULL}};
const Segment SGI30813[] = {{&S25866[95], 5}, {NULL}};
const Segment SGI30814[] = {{&S27769[44], 12}, {NULL}};
const Segment SGI30815[] = {{&S27769[44], 23}, {NULL}};
const Segment SGI30816[] = {{&S27769[30], 6}, {NULL}};
const Segment SGI30817[] = {{&S27769[115], 3}, {NULL}};
const Segment SGI30818[] = {{&S27769[81], 5}, {NULL}};
const Segment SGI30819[] = {{&S25867[44], 19}, {NULL}};
const Segment SGI30820[] = {{&S25867[44], 36}, {NULL}};
const Segment SGI30821[] = {{&S25867[30], 6}, {NULL}};
const Segment SGI30822[] = {{&S25867[69], 3}, {NULL}};
const Segment SGI30823[] = {{&S25867[94], 5}, {NULL}};
const Segment SGI30824[] = {{&S27772[44], 17}, {NULL}};
const Segment SGI30825[] = {{&S27772[44], 28}, {NULL}};
const Segment SGI30826[] = {{&S27772[30], 6}, {NULL}};
const Segment SGI30827[] = {{&S27772[120], 3}, {NULL}};
const Segment SGI30828[] = {{&S27772[86], 5}, {NULL}};
const Segment SGI30829[] = {{&S25874[44], 20}, {NULL}};
const Segment SGI30830[] = {{&S25874[44], 39}, {NULL}};
const Segment SGI30831[] = {{&S25874[30], 6}, {NULL}};
const Segment SGI30832[] = {{&S25874[131], 3}, {NULL}};
const Segment SGI30833[] = {{&S25874[97], 5}, {NULL}};
const Segment SGI30834[] = {{&S25875[44], 19}, {NULL}};
const Segment SGI30835[] = {{&S25875[44], 38}, {NULL}};
const Segment SGI30836[] = {{&S25875[30], 6}, {NULL}};
const Segment SGI30837[] = {{&S25875[130], 3}, {NULL}};
const Segment SGI30838[] = {{&S25875[96], 5}, {NULL}};
const Segment SGI30839[] = {{&S25877[44], 19}, {NULL}};
const Segment SGI30840[] = {{&S25877[44], 38}, {NULL}};
const Segment SGI30841[] = {{&S25877[30], 6}, {NULL}};
const Segment SGI30842[] = {{&S25877[130], 3}, {NULL}};
const Segment SGI30843[] = {{&S25877[96], 5}, {NULL}};
const Segment SGI30844[] = {{&S25878[44], 19}, {NULL}};
const Segment SGI30845[] = {{&S25878[44], 38}, {NULL}};
const Segment SGI30846[] = {{&S25878[30], 6}, {NULL}};
const Segment SGI30847[] = {{&S25878[130], 3}, {NULL}};
const Segment SGI30848[] = {{&S25878[96], 5}, {NULL}};
const Segment SGI30849[] = {{&S25880[44], 7}, {NULL}};
const Segment SGI30850[] = {{&S25880[44], 26}, {NULL}};
const Segment SGI30851[] = {{&S25880[30], 6}, {NULL}};
const Segment SGI30852[] = {{&S25880[118], 3}, {NULL}};
const Segment SGI30853[] = {{&S25880[84], 5}, {NULL}};
const Segment SGI30854[] = {{&S25883[44], 7}, {NULL}};
const Segment SGI30855[] = {{&S25883[44], 26}, {NULL}};
const Segment SGI30856[] = {{&S25883[30], 6}, {NULL}};
const Segment SGI30857[] = {{&S25883[118], 3}, {NULL}};
const Segment SGI30858[] = {{&S25883[84], 5}, {NULL}};
const Segment SGI30859[] = {{&S25884[44], 19}, {NULL}};
const Segment SGI30860[] = {{&S25884[44], 38}, {NULL}};
const Segment SGI30861[] = {{&S25884[30], 6}, {NULL}};
const Segment SGI30862[] = {{&S25884[130], 3}, {NULL}};
const Segment SGI30863[] = {{&S25884[96], 5}, {NULL}};
const Segment SGI30864[] = {{&S25885[44], 19}, {NULL}};
const Segment SGI30865[] = {{&S25885[44], 36}, {NULL}};
const Segment SGI30866[] = {{&S25885[30], 6}, {NULL}};
const Segment SGI30867[] = {{&S25885[69], 3}, {NULL}};
const Segment SGI30868[] = {{&S25885[94], 5}, {NULL}};
const Segment SGI30869[] = {{&S25886[44], 19}, {NULL}};
const Segment SGI30870[] = {{&S25886[44], 36}, {NULL}};
const Segment SGI30871[] = {{&S25886[30], 6}, {NULL}};
const Segment SGI30872[] = {{&S25886[69], 3}, {NULL}};
const Segment SGI30873[] = {{&S25886[94], 5}, {NULL}};
const Segment SGI30874[] = {{&S25887[44], 7}, {NULL}};
const Segment SGI30875[] = {{&S25887[44], 24}, {NULL}};
const Segment SGI30876[] = {{&S25887[30], 6}, {NULL}};
const Segment SGI30877[] = {{&S25887[57], 3}, {NULL}};
const Segment SGI30878[] = {{&S25887[82], 5}, {NULL}};
const Segment SGI30879[] = {{&S25888[44], 19}, {NULL}};
const Segment SGI30880[] = {{&S25888[44], 36}, {NULL}};
const Segment SGI30881[] = {{&S25888[30], 6}, {NULL}};
const Segment SGI30882[] = {{&S25888[69], 3}, {NULL}};
const Segment SGI30883[] = {{&S25888[94], 5}, {NULL}};
const Segment SGI30884[] = {{&S25889[44], 19}, {NULL}};
const Segment SGI30885[] = {{&S25889[44], 36}, {NULL}};
const Segment SGI30886[] = {{&S25889[30], 6}, {NULL}};
const Segment SGI30887[] = {{&S25889[69], 3}, {NULL}};
const Segment SGI30888[] = {{&S25889[94], 5}, {NULL}};
const Segment SGI30889[] = {{&S25890[44], 19}, {NULL}};
const Segment SGI30890[] = {{&S25890[44], 36}, {NULL}};
const Segment SGI30891[] = {{&S25890[30], 6}, {NULL}};
const Segment SGI30892[] = {{&S25890[69], 3}, {NULL}};
const Segment SGI30893[] = {{&S25890[94], 5}, {NULL}};
const Segment SGI30894[] = {{&S25892[44], 19}, {NULL}};
const Segment SGI30895[] = {{&S25892[44], 36}, {NULL}};
const Segment SGI30896[] = {{&S25892[30], 6}, {NULL}};
const Segment SGI30897[] = {{&S25892[69], 3}, {NULL}};
const Segment SGI30898[] = {{&S25892[94], 5}, {NULL}};
const Segment SGI30899[] = {{&S25893[44], 21}, {NULL}};
const Segment SGI30900[] = {{&S25893[44], 40}, {NULL}};
const Segment SGI30901[] = {{&S25893[30], 6}, {NULL}};
const Segment SGI30902[] = {{&S25893[132], 3}, {NULL}};
const Segment SGI30903[] = {{&S25893[98], 5}, {NULL}};
const Segment SGI30904[] = {{&S25897[44], 7}, {NULL}};
const Segment SGI30905[] = {{&S25897[44], 26}, {NULL}};
const Segment SGI30906[] = {{&S25897[30], 6}, {NULL}};
const Segment SGI30907[] = {{&S25897[118], 3}, {NULL}};
const Segment SGI30908[] = {{&S25897[84], 5}, {NULL}};
const Segment SGI30909[] = {{&S25898[44], 7}, {NULL}};
const Segment SGI30910[] = {{&S25898[44], 24}, {NULL}};
const Segment SGI30911[] = {{&S25898[30], 6}, {NULL}};
const Segment SGI30912[] = {{&S25898[57], 3}, {NULL}};
const Segment SGI30913[] = {{&S25898[82], 5}, {NULL}};
const Segment SGI30914[] = {{&S25899[40], 6}, {NULL}};
const Segment SGI30915[] = {{&S25899[26], 6}, {NULL}};
const Segment SGI30916[] = {{&S25899[83], 3}, {NULL}};
const Segment SGI30917[] = {{&S25899[60], 6}, {NULL}};
const Segment SGI30918[] = {{&S25901[38], 7}, {NULL}};
const Segment SGI30919[] = {{&S25901[31], 6}, {NULL}};
const Segment SGI30920[] = {{&S25917[44], 7}, {NULL}};
const Segment SGI30921[] = {{&S25917[44], 26}, {NULL}};
const Segment SGI30922[] = {{&S25917[30], 6}, {NULL}};
const Segment SGI30923[] = {{&S25917[87], 4}, {NULL}};
const Segment SGI30924[] = {{&S25917[87], 3}, {NULL}};
const Segment SGI30925[] = {{&S25918[44], 7}, {NULL}};
const Segment SGI30926[] = {{&S25918[44], 24}, {NULL}};
const Segment SGI30927[] = {{&S25918[30], 6}, {NULL}};
const Segment SGI30928[] = {{&S25918[85], 4}, {NULL}};
const Segment SGI30929[] = {{&S25918[85], 3}, {NULL}};
const Segment SGI30930[] = {{&S25919[44], 7}, {NULL}};
const Segment SGI30931[] = {{&S25919[44], 26}, {NULL}};
const Segment SGI30932[] = {{&S25919[30], 6}, {NULL}};
const Segment SGI30933[] = {{&S25919[87], 6}, {NULL}};
const Segment SGI30934[] = {{&S26032[44], 6}, {NULL}};
const Segment SGI30935[] = {{&S26032[44], 22}, {NULL}};
const Segment SGI30936[] = {{&S26032[30], 6}, {NULL}};
const Segment SGI30937[] = {{&S26032[114], 3}, {NULL}};
const Segment SGI30938[] = {{&S26032[80], 5}, {NULL}};
const Segment SGI30939[] = {{&S26039[38], 6}, {NULL}};
const Segment SGI30940[] = {{&S26039[31], 6}, {NULL}};
const Segment SGI30941[] = {{&S24027[44], 21}, {NULL}};
const Segment SGI30942[] = {{&S24027[44], 32}, {NULL}};
const Segment SGI30943[] = {{&S24027[30], 6}, {NULL}};
const Segment SGI30944[] = {{&S24027[124], 3}, {NULL}};
const Segment SGI30945[] = {{&S24027[90], 5}, {NULL}};
const Segment SGI30946[] = {{&S24028[44], 21}, {NULL}};
const Segment SGI30947[] = {{&S24028[44], 32}, {NULL}};
const Segment SGI30948[] = {{&S24028[30], 6}, {NULL}};
const Segment SGI30949[] = {{&S24028[124], 3}, {NULL}};
const Segment SGI30950[] = {{&S24028[90], 5}, {NULL}};
const Segment SGI30951[] = {{&S24053[44], 21}, {NULL}};
const Segment SGI30952[] = {{&S24053[44], 32}, {NULL}};
const Segment SGI30953[] = {{&S24053[30], 6}, {NULL}};
const Segment SGI30954[] = {{&S24053[124], 3}, {NULL}};
const Segment SGI30955[] = {{&S24053[90], 5}, {NULL}};
const Segment SGI30956[] = {{&S24054[44], 21}, {NULL}};
const Segment SGI30957[] = {{&S24054[44], 32}, {NULL}};
const Segment SGI30958[] = {{&S24054[30], 6}, {NULL}};
const Segment SGI30959[] = {{&S24054[124], 3}, {NULL}};
const Segment SGI30960[] = {{&S24054[90], 5}, {NULL}};
const Segment SGI30961[] = {{&S24055[44], 18}, {NULL}};
const Segment SGI30962[] = {{&S24055[44], 29}, {NULL}};
const Segment SGI30963[] = {{&S24055[30], 6}, {NULL}};
const Segment SGI30964[] = {{&S24055[121], 3}, {NULL}};
const Segment SGI30965[] = {{&S24055[87], 5}, {NULL}};
const Segment SGI30966[] = {{&S24056[44], 18}, {NULL}};
const Segment SGI30967[] = {{&S24056[44], 29}, {NULL}};
const Segment SGI30968[] = {{&S24056[30], 6}, {NULL}};
const Segment SGI30969[] = {{&S24056[121], 3}, {NULL}};
const Segment SGI30970[] = {{&S24056[87], 5}, {NULL}};
const Segment SGI30971[] = {{&S24071[44], 38}, {NULL}};
const Segment SGI30972[] = {{&S24071[44], 49}, {NULL}};
const Segment SGI30973[] = {{&S24071[30], 6}, {NULL}};
const Segment SGI30974[] = {{&S24071[141], 3}, {NULL}};
const Segment SGI30975[] = {{&S24071[107], 5}, {NULL}};
const Segment SGI30976[] = {{&S24074[38], 21}, {NULL}};
const Segment SGI30977[] = {{&S24074[31], 6}, {NULL}};
const Segment SGI30978[] = {{&S24076[44], 18}, {NULL}};
const Segment SGI30979[] = {{&S24076[44], 29}, {NULL}};
const Segment SGI30980[] = {{&S24076[30], 6}, {NULL}};
const Segment SGI30981[] = {{&S24076[121], 3}, {NULL}};
const Segment SGI30982[] = {{&S24076[87], 5}, {NULL}};
const Segment SGI30983[] = {{&S24078[44], 21}, {NULL}};
const Segment SGI30984[] = {{&S24078[44], 32}, {NULL}};
const Segment SGI30985[] = {{&S24078[30], 6}, {NULL}};
const Segment SGI30986[] = {{&S24078[93], 4}, {NULL}};
const Segment SGI30987[] = {{&S24078[93], 3}, {NULL}};
const Segment SGI30988[] = {{&S24084[44], 29}, {NULL}};
const Segment SGI30989[] = {{&S24084[44], 40}, {NULL}};
const Segment SGI30990[] = {{&S24084[30], 6}, {NULL}};
const Segment SGI30991[] = {{&S24084[132], 3}, {NULL}};
const Segment SGI30992[] = {{&S24084[98], 5}, {NULL}};
const Segment SGI30993[] = {{&S24085[44], 21}, {NULL}};
const Segment SGI30994[] = {{&S24085[44], 32}, {NULL}};
const Segment SGI30995[] = {{&S24085[30], 6}, {NULL}};
const Segment SGI30996[] = {{&S24085[124], 3}, {NULL}};
const Segment SGI30997[] = {{&S24085[90], 5}, {NULL}};
const Segment SGI30998[] = {{&S24088[44], 38}, {NULL}};
const Segment SGI30999[] = {{&S24088[44], 49}, {NULL}};
const Segment SGI31000[] = {{&S24088[30], 6}, {NULL}};
const Segment SGI31001[] = {{&S24088[141], 3}, {NULL}};
const Segment SGI31002[] = {{&S24088[107], 5}, {NULL}};
const Segment SGI31003[] = {{&S24089[44], 21}, {NULL}};
const Segment SGI31004[] = {{&S24089[44], 32}, {NULL}};
const Segment SGI31005[] = {{&S24089[30], 6}, {NULL}};
const Segment SGI31006[] = {{&S24089[93], 6}, {NULL}};
const Segment SGI31007[] = {{&S24091[44], 38}, {NULL}};
const Segment SGI31008[] = {{&S24091[44], 49}, {NULL}};
const Segment SGI31009[] = {{&S24091[30], 6}, {NULL}};
const Segment SGI31010[] = {{&S24091[110], 6}, {NULL}};
const Segment SGI31011[] = {{&S26211[43], 6}, {NULL}};
const Segment SGI31012[] = {{&S26211[43], 20}, {NULL}};
const Segment SGI31013[] = {{&S26211[30], 5}, {NULL}};
const Segment SGI31014[] = {{&S26211[111], 3}, {NULL}};
const Segment SGI31015[] = {{&S26211[77], 5}, {NULL}};
const Segment SGI31016[] = {{&S26212[44], 6}, {NULL}};
const Segment SGI31017[] = {{&S26212[44], 20}, {NULL}};
const Segment SGI31018[] = {{&S26212[30], 6}, {NULL}};
const Segment SGI31019[] = {{&S26212[112], 3}, {NULL}};
const Segment SGI31020[] = {{&S26212[78], 5}, {NULL}};
const Segment SGI31021[] = {{&S26213[44], 6}, {NULL}};
const Segment SGI31022[] = {{&S26213[44], 20}, {NULL}};
const Segment SGI31023[] = {{&S26213[30], 6}, {NULL}};
const Segment SGI31024[] = {{&S26213[112], 3}, {NULL}};
const Segment SGI31025[] = {{&S26213[78], 5}, {NULL}};
const Segment SGI31026[] = {{&S26214[44], 6}, {NULL}};
const Segment SGI31027[] = {{&S26214[44], 23}, {NULL}};
const Segment SGI31028[] = {{&S26214[30], 6}, {NULL}};
const Segment SGI31029[] = {{&S26215[44], 6}, {NULL}};
const Segment SGI31030[] = {{&S26215[44], 20}, {NULL}};
const Segment SGI31031[] = {{&S26215[30], 6}, {NULL}};
const Segment SGI31032[] = {{&S26215[112], 3}, {NULL}};
const Segment SGI31033[] = {{&S26215[78], 5}, {NULL}};
const Segment SGI31034[] = {{&S26216[44], 6}, {NULL}};
const Segment SGI31035[] = {{&S26216[44], 20}, {NULL}};
const Segment SGI31036[] = {{&S26216[30], 6}, {NULL}};
const Segment SGI31037[] = {{&S26216[112], 3}, {NULL}};
const Segment SGI31038[] = {{&S26216[78], 5}, {NULL}};
const Segment SGI31039[] = {{&S26217[44], 6}, {NULL}};
const Segment SGI31040[] = {{&S26217[44], 20}, {NULL}};
const Segment SGI31041[] = {{&S26217[30], 6}, {NULL}};
const Segment SGI31042[] = {{&S26217[112], 3}, {NULL}};
const Segment SGI31043[] = {{&S26217[78], 5}, {NULL}};
const Segment SGI31044[] = {{&S26218[43], 6}, {NULL}};
const Segment SGI31045[] = {{&S26218[43], 20}, {NULL}};
const Segment SGI31046[] = {{&S26218[30], 5}, {NULL}};
const Segment SGI31047[] = {{&S26218[111], 3}, {NULL}};
const Segment SGI31048[] = {{&S26218[77], 5}, {NULL}};
const Segment SGI31049[] = {{&S26219[44], 6}, {NULL}};
const Segment SGI31050[] = {{&S26219[44], 20}, {NULL}};
const Segment SGI31051[] = {{&S26219[30], 6}, {NULL}};
const Segment SGI31052[] = {{&S26219[114], 3}, {NULL}};
const Segment SGI31053[] = {{&S26219[78], 7}, {NULL}};
const Segment SGI31054[] = {{&S26219[134], 8}, {NULL}};
const Segment SGI31055[] = {{&S26220[44], 6}, {NULL}};
const Segment SGI31056[] = {{&S26220[44], 20}, {NULL}};
const Segment SGI31057[] = {{&S26220[30], 6}, {NULL}};
const Segment SGI31058[] = {{&S26220[112], 3}, {NULL}};
const Segment SGI31059[] = {{&S26220[78], 5}, {NULL}};
const Segment SGI31060[] = {{&S26221[44], 5}, {NULL}};
const Segment SGI31061[] = {{&S26221[44], 19}, {NULL}};
const Segment SGI31062[] = {{&S26221[30], 6}, {NULL}};
const Segment SGI31063[] = {{&S26221[111], 3}, {NULL}};
const Segment SGI31064[] = {{&S26221[77], 5}, {NULL}};
const Segment SGI31065[] = {{&S26222[44], 5}, {NULL}};
const Segment SGI31066[] = {{&S26222[44], 19}, {NULL}};
const Segment SGI31067[] = {{&S26222[30], 6}, {NULL}};
const Segment SGI31068[] = {{&S26222[111], 3}, {NULL}};
const Segment SGI31069[] = {{&S26222[77], 5}, {NULL}};
const Segment SGI31070[] = {{&S26223[44], 5}, {NULL}};
const Segment SGI31071[] = {{&S26223[44], 19}, {NULL}};
const Segment SGI31072[] = {{&S26223[30], 6}, {NULL}};
const Segment SGI31073[] = {{&S26223[111], 3}, {NULL}};
const Segment SGI31074[] = {{&S26223[77], 5}, {NULL}};
const Segment SGI31075[] = {{&S26224[43], 6}, {NULL}};
const Segment SGI31076[] = {{&S26224[43], 20}, {NULL}};
const Segment SGI31077[] = {{&S26224[111], 3}, {NULL}};
const Segment SGI31078[] = {{&S26224[77], 5}, {NULL}};
const Segment SGI31079[] = {{&S26225[44], 5}, {NULL}};
const Segment SGI31080[] = {{&S26225[44], 19}, {NULL}};
const Segment SGI31081[] = {{&S26225[30], 6}, {NULL}};
const Segment SGI31082[] = {{&S26225[111], 3}, {NULL}};
const Segment SGI31083[] = {{&S26225[77], 5}, {NULL}};
const Segment SGI31084[] = {{&S26226[43], 6}, {NULL}};
const Segment SGI31085[] = {{&S26226[43], 20}, {NULL}};
const Segment SGI31086[] = {{&S26226[111], 3}, {NULL}};
const Segment SGI31087[] = {{&S26226[77], 5}, {NULL}};
const Segment SGI31088[] = {{&S26227[43], 6}, {NULL}};
const Segment SGI31089[] = {{&S26227[43], 20}, {NULL}};
const Segment SGI31090[] = {{&S26227[111], 3}, {NULL}};
const Segment SGI31091[] = {{&S26227[77], 5}, {NULL}};
const Segment SGI31092[] = {{&S26228[43], 6}, {NULL}};
const Segment SGI31093[] = {{&S26228[43], 20}, {NULL}};
const Segment SGI31094[] = {{&S26228[111], 3}, {NULL}};
const Segment SGI31095[] = {{&S26228[77], 5}, {NULL}};
const Segment SGI31096[] = {{&S26229[44], 5}, {NULL}};
const Segment SGI31097[] = {{&S26229[44], 19}, {NULL}};
const Segment SGI31098[] = {{&S26229[30], 6}, {NULL}};
const Segment SGI31099[] = {{&S26229[111], 3}, {NULL}};
const Segment SGI31100[] = {{&S26229[77], 5}, {NULL}};
const Segment SGI31101[] = {{&S26230[44], 6}, {NULL}};
const Segment SGI31102[] = {{&S26230[44], 20}, {NULL}};
const Segment SGI31103[] = {{&S26230[30], 6}, {NULL}};
const Segment SGI31104[] = {{&S26230[112], 3}, {NULL}};
const Segment SGI31105[] = {{&S26230[78], 5}, {NULL}};
const Segment SGI31106[] = {{&S26231[44], 6}, {NULL}};
const Segment SGI31107[] = {{&S26231[44], 20}, {NULL}};
const Segment SGI31108[] = {{&S26231[30], 6}, {NULL}};
const Segment SGI31109[] = {{&S26231[114], 3}, {NULL}};
const Segment SGI31110[] = {{&S26231[78], 7}, {NULL}};
const Segment SGI31111[] = {{&S26231[134], 8}, {NULL}};
const Segment SGI31112[] = {{&S26232[44], 5}, {NULL}};
const Segment SGI31113[] = {{&S26232[44], 19}, {NULL}};
const Segment SGI31114[] = {{&S26232[30], 6}, {NULL}};
const Segment SGI31115[] = {{&S26232[111], 3}, {NULL}};
const Segment SGI31116[] = {{&S26232[77], 5}, {NULL}};
const Segment SGI31117[] = {{&S26233[43], 6}, {NULL}};
const Segment SGI31118[] = {{&S26233[43], 20}, {NULL}};
const Segment SGI31119[] = {{&S26233[30], 5}, {NULL}};
const Segment SGI31120[] = {{&S26233[111], 3}, {NULL}};
const Segment SGI31121[] = {{&S26233[77], 5}, {NULL}};
const Segment SGI31122[] = {{&S26234[44], 6}, {NULL}};
const Segment SGI31123[] = {{&S26234[44], 20}, {NULL}};
const Segment SGI31124[] = {{&S26234[30], 6}, {NULL}};
const Segment SGI31125[] = {{&S26234[112], 3}, {NULL}};
const Segment SGI31126[] = {{&S26234[78], 5}, {NULL}};
const Segment SGI31127[] = {{&S26235[44], 5}, {NULL}};
const Segment SGI31128[] = {{&S26235[44], 19}, {NULL}};
const Segment SGI31129[] = {{&S26235[30], 6}, {NULL}};
const Segment SGI31130[] = {{&S26235[111], 3}, {NULL}};
const Segment SGI31131[] = {{&S26235[77], 5}, {NULL}};
const Segment SGI31132[] = {{&S26236[43], 6}, {NULL}};
const Segment SGI31133[] = {{&S26236[43], 20}, {NULL}};
const Segment SGI31134[] = {{&S26236[30], 5}, {NULL}};
const Segment SGI31135[] = {{&S26236[111], 3}, {NULL}};
const Segment SGI31136[] = {{&S26236[77], 5}, {NULL}};
const Segment SGI31137[] = {{&S26237[44], 6}, {NULL}};
const Segment SGI31138[] = {{&S26237[44], 20}, {NULL}};
const Segment SGI31139[] = {{&S26237[30], 6}, {NULL}};
const Segment SGI31140[] = {{&S26237[112], 3}, {NULL}};
const Segment SGI31141[] = {{&S26237[78], 5}, {NULL}};
const Segment SGI31142[] = {{&S26238[38], 6}, {NULL}};
const Segment SGI31143[] = {{&S26238[31], 6}, {NULL}};
const Segment SGI31144[] = {{&S26239[44], 5}, {NULL}};
const Segment SGI31145[] = {{&S26239[44], 19}, {NULL}};
const Segment SGI31146[] = {{&S26239[30], 6}, {NULL}};
const Segment SGI31147[] = {{&S26239[111], 3}, {NULL}};
const Segment SGI31148[] = {{&S26239[77], 5}, {NULL}};
const Segment SGI31149[] = {{&S26240[44], 5}, {NULL}};
const Segment SGI31150[] = {{&S26240[44], 19}, {NULL}};
const Segment SGI31151[] = {{&S26240[30], 6}, {NULL}};
const Segment SGI31152[] = {{&S26240[111], 3}, {NULL}};
const Segment SGI31153[] = {{&S26240[77], 5}, {NULL}};
const Segment SGI31154[] = {{&S26241[43], 6}, {NULL}};
const Segment SGI31155[] = {{&S26241[43], 20}, {NULL}};
const Segment SGI31156[] = {{&S26241[111], 3}, {NULL}};
const Segment SGI31157[] = {{&S26241[77], 5}, {NULL}};
const Segment SGI31158[] = {{&S26242[43], 6}, {NULL}};
const Segment SGI31159[] = {{&S26242[43], 20}, {NULL}};
const Segment SGI31160[] = {{&S26242[30], 5}, {NULL}};
const Segment SGI31161[] = {{&S26242[111], 3}, {NULL}};
const Segment SGI31162[] = {{&S26242[77], 5}, {NULL}};
const Segment SGI31163[] = {{&S26243[44], 6}, {NULL}};
const Segment SGI31164[] = {{&S26243[44], 20}, {NULL}};
const Segment SGI31165[] = {{&S26243[30], 6}, {NULL}};
const Segment SGI31166[] = {{&S26243[112], 3}, {NULL}};
const Segment SGI31167[] = {{&S26243[78], 5}, {NULL}};
const Segment SGI31168[] = {{&S26244[44], 6}, {NULL}};
const Segment SGI31169[] = {{&S26244[44], 20}, {NULL}};
const Segment SGI31170[] = {{&S26244[30], 6}, {NULL}};
const Segment SGI31171[] = {{&S26244[112], 3}, {NULL}};
const Segment SGI31172[] = {{&S26244[78], 5}, {NULL}};
const Segment SGI31173[] = {{&S35834[44], 11}, {NULL}};
const Segment SGI31174[] = {{&S35834[44], 22}, {NULL}};
const Segment SGI31175[] = {{&S35834[30], 6}, {NULL}};
const Segment SGI31176[] = {{&S35834[114], 3}, {NULL}};
const Segment SGI31177[] = {{&S35834[80], 5}, {NULL}};
const Segment SGI31178[] = {{&S35840[44], 11}, {NULL}};
const Segment SGI31179[] = {{&S35840[44], 22}, {NULL}};
const Segment SGI31180[] = {{&S35840[30], 6}, {NULL}};
const Segment SGI31181[] = {{&S35840[114], 3}, {NULL}};
const Segment SGI31182[] = {{&S35840[80], 5}, {NULL}};
const Segment SGI31183[] = {{&S35843[38], 11}, {NULL}};
const Segment SGI31184[] = {{&S35843[31], 6}, {NULL}};
const Segment SGI31185[] = {{&S35845[38], 11}, {NULL}};
const Segment SGI31186[] = {{&S35845[31], 6}, {NULL}};
const Segment SGI31187[] = {{&S35846[44], 10}, {NULL}};
const Segment SGI31188[] = {{&S35846[44], 21}, {NULL}};
const Segment SGI31189[] = {{&S35846[30], 6}, {NULL}};
const Segment SGI31190[] = {{&S35846[113], 3}, {NULL}};
const Segment SGI31191[] = {{&S35846[79], 5}, {NULL}};
const Segment SGI31192[] = {{&S24438[44], 10}, {NULL}};
const Segment SGI31193[] = {{&S24438[44], 27}, {NULL}};
const Segment SGI31194[] = {{&S24438[30], 6}, {NULL}};
const Segment SGI31195[] = {{&S24438[119], 3}, {NULL}};
const Segment SGI31196[] = {{&S24438[85], 5}, {NULL}};
const Segment SGI31197[] = {{&S24476[38], 10}, {NULL}};
const Segment SGI31198[] = {{&S24476[31], 6}, {NULL}};
const Segment SGI31199[] = {{&S24496[42], 10}, {NULL}};
const Segment SGI31200[] = {{&S24496[30], 6}, {NULL}};
const Segment SGI31201[] = {{&S24496[117], 3}, {NULL}};
const Segment SGI31202[] = {{&S24496[83], 5}, {NULL}};
const Segment SGI31203[] = {{&S24499[44], 10}, {NULL}};
const Segment SGI31204[] = {{&S24499[44], 26}, {NULL}};
const Segment SGI31205[] = {{&S24499[30], 6}, {NULL}};
const Segment SGI31206[] = {{&S24499[118], 3}, {NULL}};
const Segment SGI31207[] = {{&S24499[84], 5}, {NULL}};
const Segment SGI31208[] = {{&S24504[44], 10}, {NULL}};
const Segment SGI31209[] = {{&S24504[44], 21}, {NULL}};
const Segment SGI31210[] = {{&S24504[30], 6}, {NULL}};
const Segment SGI31211[] = {{&S24504[117], 3}, {NULL}};
const Segment SGI31212[] = {{&S24504[79], 9}, {NULL}};
const Segment SGI31213[] = {{&S24547[44], 10}, {NULL}};
const Segment SGI31214[] = {{&S24547[44], 21}, {NULL}};
const Segment SGI31215[] = {{&S24547[30], 6}, {NULL}};
const Segment SGI31216[] = {{&S24547[113], 3}, {NULL}};
const Segment SGI31217[] = {{&S24547[79], 5}, {NULL}};
const Segment SGI31218[] = {{&S30614[42], 11}, {NULL}};
const Segment SGI31219[] = {{&S30614[42], 22}, {NULL}};
const Segment SGI31220[] = {{&S30614[30], 4}, {NULL}};
const Segment SGI31221[] = {{&S30614[112], 3}, {NULL}};
const Segment SGI31222[] = {{&S30614[78], 5}, {NULL}};
const Segment SGI31223[] = {{&S30616[43], 11}, {NULL}};
const Segment SGI31224[] = {{&S30616[43], 22}, {NULL}};
const Segment SGI31225[] = {{&S30616[31], 4}, {NULL}};
const Segment SGI31226[] = {{&S30616[113], 3}, {NULL}};
const Segment SGI31227[] = {{&S30616[79], 5}, {NULL}};
const Segment SGI31228[] = {{&S30629[42], 11}, {NULL}};
const Segment SGI31229[] = {{&S30629[42], 22}, {NULL}};
const Segment SGI31230[] = {{&S30629[30], 4}, {NULL}};
const Segment SGI31231[] = {{&S30629[112], 3}, {NULL}};
const Segment SGI31232[] = {{&S30629[78], 5}, {NULL}};
const Segment SGI31233[] = {{&S30630[42], 11}, {NULL}};
const Segment SGI31234[] = {{&S30630[42], 22}, {NULL}};
const Segment SGI31235[] = {{&S30630[30], 4}, {NULL}};
const Segment SGI31236[] = {{&S24583[44], 10}, {NULL}};
const Segment SGI31237[] = {{&S24583[44], 21}, {NULL}};
const Segment SGI31238[] = {{&S24583[30], 6}, {NULL}};
const Segment SGI31239[] = {{&S30630[112], 3}, {NULL}};
const Segment SGI31240[] = {{&S30630[78], 5}, {NULL}};
const Segment SGI31241[] = {{&S24583[113], 3}, {NULL}};
const Segment SGI31242[] = {{&S24583[79], 5}, {NULL}};
const Segment SGI31243[] = {{&S26735[44], 19}, {NULL}};
const Segment SGI31244[] = {{&S26735[44], 37}, {NULL}};
const Segment SGI31245[] = {{&S26735[30], 6}, {NULL}};
const Segment SGI31246[] = {{&S26735[129], 3}, {NULL}};
const Segment SGI31247[] = {{&S26735[95], 5}, {NULL}};
const Segment SGI31248[] = {{&S26739[42], 19}, {NULL}};
const Segment SGI31249[] = {{&S26739[30], 6}, {NULL}};
const Segment SGI31250[] = {{&S26739[128], 3}, {NULL}};
const Segment SGI31251[] = {{&S26739[94], 5}, {NULL}};
const Segment SGI31252[] = {{&S26740[44], 19}, {NULL}};
const Segment SGI31253[] = {{&S26740[44], 38}, {NULL}};
const Segment SGI31254[] = {{&S26740[30], 6}, {NULL}};
const Segment SGI31255[] = {{&S26740[130], 3}, {NULL}};
const Segment SGI31256[] = {{&S26740[96], 5}, {NULL}};
const Segment SGI31257[] = {{&S26741[44], 19}, {NULL}};
const Segment SGI31258[] = {{&S26741[44], 38}, {NULL}};
const Segment SGI31259[] = {{&S26741[30], 6}, {NULL}};
const Segment SGI31260[] = {{&S26741[130], 3}, {NULL}};
const Segment SGI31261[] = {{&S26741[96], 5}, {NULL}};
const Segment SGI31262[] = {{&S26742[44], 7}, {NULL}};
const Segment SGI31263[] = {{&S26742[44], 26}, {NULL}};
const Segment SGI31264[] = {{&S26742[30], 6}, {NULL}};
const Segment SGI31265[] = {{&S26742[118], 3}, {NULL}};
const Segment SGI31266[] = {{&S26742[84], 5}, {NULL}};
const Segment SGI31267[] = {{&S26744[44], 19}, {NULL}};
const Segment SGI31268[] = {{&S26744[44], 36}, {NULL}};
const Segment SGI31269[] = {{&S26744[30], 6}, {NULL}};
const Segment SGI31270[] = {{&S26744[69], 3}, {NULL}};
const Segment SGI31271[] = {{&S26744[94], 5}, {NULL}};
const Segment SGI31272[] = {{&S26747[44], 7}, {NULL}};
const Segment SGI31273[] = {{&S26747[44], 24}, {NULL}};
const Segment SGI31274[] = {{&S26747[30], 6}, {NULL}};
const Segment SGI31275[] = {{&S26747[57], 3}, {NULL}};
const Segment SGI31276[] = {{&S26747[82], 5}, {NULL}};
const Segment SGI31277[] = {{&S26748[44], 19}, {NULL}};
const Segment SGI31278[] = {{&S26748[44], 36}, {NULL}};
const Segment SGI31279[] = {{&S26748[30], 6}, {NULL}};
const Segment SGI31280[] = {{&S26748[69], 3}, {NULL}};
const Segment SGI31281[] = {{&S26748[94], 5}, {NULL}};
const Segment SGI31282[] = {{&S26752[44], 19}, {NULL}};
const Segment SGI31283[] = {{&S26752[44], 37}, {NULL}};
const Segment SGI31284[] = {{&S26752[30], 6}, {NULL}};
const Segment SGI31285[] = {{&S26752[129], 3}, {NULL}};
const Segment SGI31286[] = {{&S26752[95], 5}, {NULL}};
const Segment SGI31287[] = {{&S25597[44], 7}, {NULL}};
const Segment SGI31288[] = {{&S25597[44], 29}, {NULL}};
const Segment SGI31289[] = {{&S26753[44], 7}, {NULL}};
const Segment SGI31290[] = {{&S25597[30], 6}, {NULL}};
const Segment SGI31291[] = {{&S26753[44], 26}, {NULL}};
const Segment SGI31292[] = {{&S26753[30], 6}, {NULL}};
const Segment SGI31293[] = {{&S25597[121], 3}, {NULL}};
const Segment SGI31294[] = {{&S26753[118], 3}, {NULL}};
const Segment SGI31295[] = {{&S25597[87], 5}, {NULL}};
const Segment SGI31296[] = {{&S26753[84], 5}, {NULL}};
const Segment SGI31297[] = {{&S25598[44], 7}, {NULL}};
const Segment SGI31298[] = {{&S25598[44], 29}, {NULL}};
const Segment SGI31299[] = {{&S26754[44], 19}, {NULL}};
const Segment SGI31300[] = {{&S25598[30], 6}, {NULL}};
const Segment SGI31301[] = {{&S26754[44], 38}, {NULL}};
const Segment SGI31302[] = {{&S26754[30], 6}, {NULL}};
const Segment SGI31303[] = {{&S25598[121], 3}, {NULL}};
const Segment SGI31304[] = {{&S26754[130], 3}, {NULL}};
const Segment SGI31305[] = {{&S25598[87], 5}, {NULL}};
const Segment SGI31306[] = {{&S26754[96], 5}, {NULL}};
const Segment SGI31307[] = {{&S25599[44], 7}, {NULL}};
const Segment SGI31308[] = {{&S25599[44], 34}, {NULL}};
const Segment SGI31309[] = {{&S25599[30], 6}, {NULL}};
const Segment SGI31310[] = {{&S25599[126], 3}, {NULL}};
const Segment SGI31311[] = {{&S25599[92], 5}, {NULL}};
const Segment SGI31312[] = {{&S26756[44], 7}, {NULL}};
const Segment SGI31313[] = {{&S26756[44], 26}, {NULL}};
const Segment SGI31314[] = {{&S26756[30], 6}, {NULL}};
const Segment SGI31315[] = {{&S26756[118], 3}, {NULL}};
const Segment SGI31316[] = {{&S26756[84], 5}, {NULL}};
const Segment SGI31317[] = {{&S25600[44], 7}, {NULL}};
const Segment SGI31318[] = {{&S25600[44], 29}, {NULL}};
const Segment SGI31319[] = {{&S25600[30], 6}, {NULL}};
const Segment SGI31320[] = {{&S25600[121], 3}, {NULL}};
const Segment SGI31321[] = {{&S25600[87], 5}, {NULL}};
const Segment SGI31322[] = {{&S26757[44], 19}, {NULL}};
const Segment SGI31323[] = {{&S26757[44], 38}, {NULL}};
const Segment SGI31324[] = {{&S26757[30], 6}, {NULL}};
const Segment SGI31325[] = {{&S26757[130], 3}, {NULL}};
const Segment SGI31326[] = {{&S26757[96], 5}, {NULL}};
const Segment SGI31327[] = {{&S25601[44], 7}, {NULL}};
const Segment SGI31328[] = {{&S25601[44], 31}, {NULL}};
const Segment SGI31329[] = {{&S25601[30], 6}, {NULL}};
const Segment SGI31330[] = {{&S25601[123], 3}, {NULL}};
const Segment SGI31331[] = {{&S25601[89], 5}, {NULL}};
const Segment SGI31332[] = {{&S26758[44], 19}, {NULL}};
const Segment SGI31333[] = {{&S26758[44], 37}, {NULL}};
const Segment SGI31334[] = {{&S26758[30], 6}, {NULL}};
const Segment SGI31335[] = {{&S26758[129], 3}, {NULL}};
const Segment SGI31336[] = {{&S25602[44], 7}, {NULL}};
const Segment SGI31337[] = {{&S26758[95], 5}, {NULL}};
const Segment SGI31338[] = {{&S25602[44], 29}, {NULL}};
const Segment SGI31339[] = {{&S25602[30], 6}, {NULL}};
const Segment SGI31340[] = {{&S25602[121], 3}, {NULL}};
const Segment SGI31341[] = {{&S25602[87], 5}, {NULL}};
const Segment SGI31342[] = {{&S26759[44], 19}, {NULL}};
const Segment SGI31343[] = {{&S26759[44], 36}, {NULL}};
const Segment SGI31344[] = {{&S26759[30], 6}, {NULL}};
const Segment SGI31345[] = {{&S25603[44], 7}, {NULL}};
const Segment SGI31346[] = {{&S26759[69], 3}, {NULL}};
const Segment SGI31347[] = {{&S25603[44], 34}, {NULL}};
const Segment SGI31348[] = {{&S25603[30], 6}, {NULL}};
const Segment SGI31349[] = {{&S26759[94], 5}, {NULL}};
const Segment SGI31350[] = {{&S25603[126], 3}, {NULL}};
const Segment SGI31351[] = {{&S25603[92], 5}, {NULL}};
const Segment SGI31352[] = {{&S26760[44], 19}, {NULL}};
const Segment SGI31353[] = {{&S26760[44], 36}, {NULL}};
const Segment SGI31354[] = {{&S26760[30], 6}, {NULL}};
const Segment SGI31355[] = {{&S25604[44], 7}, {NULL}};
const Segment SGI31356[] = {{&S25604[44], 31}, {NULL}};
const Segment SGI31357[] = {{&S25604[30], 6}, {NULL}};
const Segment SGI31358[] = {{&S26760[69], 3}, {NULL}};
const Segment SGI31359[] = {{&S26760[94], 5}, {NULL}};
const Segment SGI31360[] = {{&S25604[123], 3}, {NULL}};
const Segment SGI31361[] = {{&S25604[89], 5}, {NULL}};
const Segment SGI31362[] = {{&S26761[44], 19}, {NULL}};
const Segment SGI31363[] = {{&S25605[44], 7}, {NULL}};
const Segment SGI31364[] = {{&S26761[44], 36}, {NULL}};
const Segment SGI31365[] = {{&S25605[44], 29}, {NULL}};
const Segment SGI31366[] = {{&S26761[30], 6}, {NULL}};
const Segment SGI31367[] = {{&S25605[30], 6}, {NULL}};
const Segment SGI31368[] = {{&S26761[129], 3}, {NULL}};
const Segment SGI31369[] = {{&S25605[121], 3}, {NULL}};
const Segment SGI31370[] = {{&S26761[95], 5}, {NULL}};
const Segment SGI31371[] = {{&S25605[87], 5}, {NULL}};
const Segment SGI31372[] = {{&S25606[44], 20}, {NULL}};
const Segment SGI31373[] = {{&S25606[44], 45}, {NULL}};
const Segment SGI31374[] = {{&S25606[30], 6}, {NULL}};
const Segment SGI31375[] = {{&S25606[137], 3}, {NULL}};
const Segment SGI31376[] = {{&S25606[103], 5}, {NULL}};
const Segment SGI31377[] = {{&S25607[44], 20}, {NULL}};
const Segment SGI31378[] = {{&S25607[44], 45}, {NULL}};
const Segment SGI31379[] = {{&S25607[30], 6}, {NULL}};
const Segment SGI31380[] = {{&S25607[137], 3}, {NULL}};
const Segment SGI31381[] = {{&S25607[103], 5}, {NULL}};
const Segment SGI31382[] = {{&S26766[38], 7}, {NULL}};
const Segment SGI31383[] = {{&S26766[31], 6}, {NULL}};
const Segment SGI31384[] = {{&S25608[44], 7}, {NULL}};
const Segment SGI31385[] = {{&S25608[44], 29}, {NULL}};
const Segment SGI31386[] = {{&S25608[30], 6}, {NULL}};
const Segment SGI31387[] = {{&S25608[121], 3}, {NULL}};
const Segment SGI31388[] = {{&S25608[87], 5}, {NULL}};
const Segment SGI31389[] = {{&S25609[44], 7}, {NULL}};
const Segment SGI31390[] = {{&S25609[44], 29}, {NULL}};
const Segment SGI31391[] = {{&S25609[30], 6}, {NULL}};
const Segment SGI31392[] = {{&S25609[121], 3}, {NULL}};
const Segment SGI31393[] = {{&S25609[87], 5}, {NULL}};
const Segment SGI31394[] = {{&S25611[44], 7}, {NULL}};
const Segment SGI31395[] = {{&S25611[44], 31}, {NULL}};
const Segment SGI31396[] = {{&S25611[30], 6}, {NULL}};
const Segment SGI31397[] = {{&S25611[123], 3}, {NULL}};
const Segment SGI31398[] = {{&S25611[89], 5}, {NULL}};
const Segment SGI31399[] = {{&S25612[44], 7}, {NULL}};
const Segment SGI31400[] = {{&S25612[44], 31}, {NULL}};
const Segment SGI31401[] = {{&S25612[30], 6}, {NULL}};
const Segment SGI31402[] = {{&S25612[123], 3}, {NULL}};
const Segment SGI31403[] = {{&S25612[89], 5}, {NULL}};
const Segment SGI31404[] = {{&S25613[44], 7}, {NULL}};
const Segment SGI31405[] = {{&S25613[44], 31}, {NULL}};
const Segment SGI31406[] = {{&S25613[30], 6}, {NULL}};
const Segment SGI31407[] = {{&S25613[123], 3}, {NULL}};
const Segment SGI31408[] = {{&S25613[89], 5}, {NULL}};
const Segment SGI31409[] = {{&S36155[38], 10}, {NULL}};
const Segment SGI31410[] = {{&S36155[31], 6}, {NULL}};
const Segment SGI31411[] = {{&S25614[44], 7}, {NULL}};
const Segment SGI31412[] = {{&S25614[44], 35}, {NULL}};
const Segment SGI31413[] = {{&S25614[30], 6}, {NULL}};
const Segment SGI31414[] = {{&S25614[127], 3}, {NULL}};
const Segment SGI31415[] = {{&S25614[93], 5}, {NULL}};
const Segment SGI31416[] = {{&S36156[44], 10}, {NULL}};
const Segment SGI31417[] = {{&S36156[44], 21}, {NULL}};
const Segment SGI31418[] = {{&S36156[30], 6}, {NULL}};
const Segment SGI31419[] = {{&S25615[44], 7}, {NULL}};
const Segment SGI31420[] = {{&S25615[44], 31}, {NULL}};
const Segment SGI31421[] = {{&S25615[30], 6}, {NULL}};
const Segment SGI31422[] = {{&S36156[113], 3}, {NULL}};
const Segment SGI31423[] = {{&S36156[79], 5}, {NULL}};
const Segment SGI31424[] = {{&S25616[38], 7}, {NULL}};
const Segment SGI31425[] = {{&S25616[31], 6}, {NULL}};
const Segment SGI31426[] = {{&S36158[44], 9}, {NULL}};
const Segment SGI31427[] = {{&S36158[44], 20}, {NULL}};
const Segment SGI31428[] = {{&S36158[30], 6}, {NULL}};
const Segment SGI31429[] = {{&S25617[44], 7}, {NULL}};
const Segment SGI31430[] = {{&S36158[112], 3}, {NULL}};
const Segment SGI31431[] = {{&S25617[44], 31}, {NULL}};
const Segment SGI31432[] = {{&S25617[30], 6}, {NULL}};
const Segment SGI31433[] = {{&S36158[78], 5}, {NULL}};
const Segment SGI31434[] = {{&S25617[123], 3}, {NULL}};
const Segment SGI31435[] = {{&S25617[89], 5}, {NULL}};
const Segment SGI31436[] = {{&S25620[38], 20}, {NULL}};
const Segment SGI31437[] = {{&S25620[31], 6}, {NULL}};
const Segment SGI31438[] = {{&S36164[44], 9}, {NULL}};
const Segment SGI31439[] = {{&S36164[44], 20}, {NULL}};
const Segment SGI31440[] = {{&S36164[30], 6}, {NULL}};
const Segment SGI31441[] = {{&S36164[112], 3}, {NULL}};
const Segment SGI31442[] = {{&S36164[78], 5}, {NULL}};
const Segment SGI31443[] = {{&S36165[43], 10}, {NULL}};
const Segment SGI31444[] = {{&S36165[43], 21}, {NULL}};
const Segment SGI31445[] = {{&S36165[112], 3}, {NULL}};
const Segment SGI31446[] = {{&S36165[78], 5}, {NULL}};
const Segment SGI31447[] = {{&S36166[43], 10}, {NULL}};
const Segment SGI31448[] = {{&S36166[43], 21}, {NULL}};
const Segment SGI31449[] = {{&S36166[30], 5}, {NULL}};
const Segment SGI31450[] = {{&S36166[112], 3}, {NULL}};
const Segment SGI31451[] = {{&S36166[78], 5}, {NULL}};
const Segment SGI31452[] = {{&S36167[42], 10}, {NULL}};
const Segment SGI31453[] = {{&S36167[30], 6}, {NULL}};
const Segment SGI31454[] = {{&S36167[111], 3}, {NULL}};
const Segment SGI31455[] = {{&S36167[77], 5}, {NULL}};
const Segment SGI31456[] = {{&S26873[44], 7}, {NULL}};
const Segment SGI31457[] = {{&S26873[44], 28}, {NULL}};
const Segment SGI31458[] = {{&S26873[30], 6}, {NULL}};
const Segment SGI31459[] = {{&S26873[120], 3}, {NULL}};
const Segment SGI31460[] = {{&S26873[86], 5}, {NULL}};
const Segment SGI31461[] = {{&S26874[43], 8}, {NULL}};
const Segment SGI31462[] = {{&S26874[43], 29}, {NULL}};
const Segment SGI31463[] = {{&S26874[120], 3}, {NULL}};
const Segment SGI31464[] = {{&S26874[86], 5}, {NULL}};
const Segment SGI31465[] = {{&S26875[44], 8}, {NULL}};
const Segment SGI31466[] = {{&S26875[44], 29}, {NULL}};
const Segment SGI31467[] = {{&S26875[30], 6}, {NULL}};
const Segment SGI31468[] = {{&S26875[121], 3}, {NULL}};
const Segment SGI31469[] = {{&S26875[87], 5}, {NULL}};
const Segment SGI31470[] = {{&S26876[44], 8}, {NULL}};
const Segment SGI31471[] = {{&S26876[44], 29}, {NULL}};
const Segment SGI31472[] = {{&S26876[30], 6}, {NULL}};
const Segment SGI31473[] = {{&S32364[44], 9}, {NULL}};
const Segment SGI31474[] = {{&S26876[121], 3}, {NULL}};
const Segment SGI31475[] = {{&S32364[44], 21}, {NULL}};
const Segment SGI31476[] = {{&S32364[30], 6}, {NULL}};
const Segment SGI31477[] = {{&S26876[87], 5}, {NULL}};
const Segment SGI31478[] = {{&S32364[113], 3}, {NULL}};
const Segment SGI31479[] = {{&S32364[79], 5}, {NULL}};
const Segment SGI31480[] = {{&S26877[44], 8}, {NULL}};
const Segment SGI31481[] = {{&S26877[44], 29}, {NULL}};
const Segment SGI31482[] = {{&S26877[30], 6}, {NULL}};
const Segment SGI31483[] = {{&S26877[121], 3}, {NULL}};
const Segment SGI31484[] = {{&S26877[87], 5}, {NULL}};
const Segment SGI31485[] = {{&S26878[43], 8}, {NULL}};
const Segment SGI31486[] = {{&S26878[43], 29}, {NULL}};
const Segment SGI31487[] = {{&S26878[30], 5}, {NULL}};
const Segment SGI31488[] = {{&S26878[120], 3}, {NULL}};
const Segment SGI31489[] = {{&S26878[86], 5}, {NULL}};
const Segment SGI31490[] = {{&S26879[44], 7}, {NULL}};
const Segment SGI31491[] = {{&S26879[44], 28}, {NULL}};
const Segment SGI31492[] = {{&S26879[30], 6}, {NULL}};
const Segment SGI31493[] = {{&S26879[120], 3}, {NULL}};
const Segment SGI31494[] = {{&S26879[86], 5}, {NULL}};
const Segment SGI31495[] = {{&S26880[44], 8}, {NULL}};
const Segment SGI31496[] = {{&S26880[44], 29}, {NULL}};
const Segment SGI31497[] = {{&S26880[30], 6}, {NULL}};
const Segment SGI31498[] = {{&S26880[121], 3}, {NULL}};
const Segment SGI31499[] = {{&S26880[87], 5}, {NULL}};
const Segment SGI31500[] = {{&S26881[44], 8}, {NULL}};
const Segment SGI31501[] = {{&S26881[44], 29}, {NULL}};
const Segment SGI31502[] = {{&S26881[30], 6}, {NULL}};
const Segment SGI31503[] = {{&S26881[121], 3}, {NULL}};
const Segment SGI31504[] = {{&S26881[87], 5}, {NULL}};
const Segment SGI31505[] = {{&S26882[44], 7}, {NULL}};
const Segment SGI31506[] = {{&S26882[44], 28}, {NULL}};
const Segment SGI31507[] = {{&S26882[30], 6}, {NULL}};
const Segment SGI31508[] = {{&S26882[120], 3}, {NULL}};
const Segment SGI31509[] = {{&S26882[86], 5}, {NULL}};
const Segment SGI31510[] = {{&S26883[44], 8}, {NULL}};
const Segment SGI31511[] = {{&S26883[44], 29}, {NULL}};
const Segment SGI31512[] = {{&S26883[30], 6}, {NULL}};
const Segment SGI31513[] = {{&S26883[121], 3}, {NULL}};
const Segment SGI31514[] = {{&S26883[87], 5}, {NULL}};
const Segment SGI31515[] = {{&S26884[44], 7}, {NULL}};
const Segment SGI31516[] = {{&S26884[44], 28}, {NULL}};
const Segment SGI31517[] = {{&S26884[30], 6}, {NULL}};
const Segment SGI31518[] = {{&S26884[120], 3}, {NULL}};
const Segment SGI31519[] = {{&S26884[86], 5}, {NULL}};
const Segment SGI31520[] = {{&S26885[43], 8}, {NULL}};
const Segment SGI31521[] = {{&S26885[43], 29}, {NULL}};
const Segment SGI31522[] = {{&S26885[30], 5}, {NULL}};
const Segment SGI31523[] = {{&S26885[120], 3}, {NULL}};
const Segment SGI31524[] = {{&S26885[86], 5}, {NULL}};
const Segment SGI31525[] = {{&S26886[44], 7}, {NULL}};
const Segment SGI31526[] = {{&S26886[44], 28}, {NULL}};
const Segment SGI31527[] = {{&S26886[30], 6}, {NULL}};
const Segment SGI31528[] = {{&S26886[120], 3}, {NULL}};
const Segment SGI31529[] = {{&S26886[86], 5}, {NULL}};
const Segment SGI31530[] = {{&S26887[43], 8}, {NULL}};
const Segment SGI31531[] = {{&S26887[43], 29}, {NULL}};
const Segment SGI31532[] = {{&S26887[120], 3}, {NULL}};
const Segment SGI31533[] = {{&S26887[86], 5}, {NULL}};
const Segment SGI31534[] = {{&S26888[44], 8}, {NULL}};
const Segment SGI31535[] = {{&S26888[44], 29}, {NULL}};
const Segment SGI31536[] = {{&S26888[30], 6}, {NULL}};
const Segment SGI31537[] = {{&S26888[121], 3}, {NULL}};
const Segment SGI31538[] = {{&S26888[87], 5}, {NULL}};
const Segment SGI31539[] = {{&S26889[44], 8}, {NULL}};
const Segment SGI31540[] = {{&S26889[44], 28}, {NULL}};
const Segment SGI31541[] = {{&S26889[30], 6}, {NULL}};
const Segment SGI31542[] = {{&S26889[120], 3}, {NULL}};
const Segment SGI31543[] = {{&S26889[86], 5}, {NULL}};
const Segment SGI31544[] = {{&S26890[44], 8}, {NULL}};
const Segment SGI31545[] = {{&S26890[44], 29}, {NULL}};
const Segment SGI31546[] = {{&S26890[30], 6}, {NULL}};
const Segment SGI31547[] = {{&S26890[121], 3}, {NULL}};
const Segment SGI31548[] = {{&S26890[87], 5}, {NULL}};
const Segment SGI31549[] = {{&S26893[38], 8}, {NULL}};
const Segment SGI31550[] = {{&S26893[31], 6}, {NULL}};
const Segment SGI31551[] = {{&S26895[44], 8}, {NULL}};
const Segment SGI31552[] = {{&S26895[44], 29}, {NULL}};
const Segment SGI31553[] = {{&S26895[30], 6}, {NULL}};
const Segment SGI31554[] = {{&S26895[121], 3}, {NULL}};
const Segment SGI31555[] = {{&S26895[87], 5}, {NULL}};
const Segment SGI31556[] = {{&S26897[44], 8}, {NULL}};
const Segment SGI31557[] = {{&S26897[44], 29}, {NULL}};
const Segment SGI31558[] = {{&S26897[30], 6}, {NULL}};
const Segment SGI31559[] = {{&S26897[123], 3}, {NULL}};
const Segment SGI31560[] = {{&S26897[87], 7}, {NULL}};
const Segment SGI31561[] = {{&S26897[143], 8}, {NULL}};
const Segment SGI31562[] = {{&S26899[44], 8}, {NULL}};
const Segment SGI31563[] = {{&S26899[44], 29}, {NULL}};
const Segment SGI31564[] = {{&S26899[30], 6}, {NULL}};
const Segment SGI31565[] = {{&S26899[121], 3}, {NULL}};
const Segment SGI31566[] = {{&S26899[87], 5}, {NULL}};
const Segment SGI31567[] = {{&S36317[44], 7}, {NULL}};
const Segment SGI31568[] = {{&S36317[44], 19}, {NULL}};
const Segment SGI31569[] = {{&S36317[30], 6}, {NULL}};
const Segment SGI31570[] = {{&S36317[111], 3}, {NULL}};
const Segment SGI31571[] = {{&S36317[77], 5}, {NULL}};
const Segment SGI31572[] = {{&S36318[38], 7}, {NULL}};
const Segment SGI31573[] = {{&S36318[31], 6}, {NULL}};
const Segment SGI31574[] = {{&S36392[44], 7}, {NULL}};
const Segment SGI31575[] = {{&S36392[44], 28}, {NULL}};
const Segment SGI31576[] = {{&S36392[30], 6}, {NULL}};
const Segment SGI31577[] = {{&S36392[69], 3}, {NULL}};
const Segment SGI31578[] = {{&S36392[86], 5}, {NULL}};
const Segment SGI31579[] = {{&S36393[55], 7}, {NULL}};
const Segment SGI31580[] = {{&S36393[55], 28}, {NULL}};
const Segment SGI31581[] = {{&S36393[41], 6}, {NULL}};
const Segment SGI31582[] = {{&S36393[131], 3}, {NULL}};
const Segment SGI31583[] = {{&S36393[97], 5}, {NULL}};
const Segment SGI31584[] = {{&S36394[44], 7}, {NULL}};
const Segment SGI31585[] = {{&S36394[44], 61}, {NULL}};
const Segment SGI31586[] = {{&S36394[30], 6}, {NULL}};
const Segment SGI31587[] = {{&S36394[153], 3}, {NULL}};
const Segment SGI31588[] = {{&S36394[119], 5}, {NULL}};
const Segment SGI31589[] = {{&S36395[44], 7}, {NULL}};
const Segment SGI31590[] = {{&S36395[44], 57}, {NULL}};
const Segment SGI31591[] = {{&S36395[30], 6}, {NULL}};
const Segment SGI31592[] = {{&S36395[149], 3}, {NULL}};
const Segment SGI31593[] = {{&S36395[115], 5}, {NULL}};
const Segment SGI31594[] = {{&S36399[44], 7}, {NULL}};
const Segment SGI31595[] = {{&S36399[44], 27}, {NULL}};
const Segment SGI31596[] = {{&S36399[30], 6}, {NULL}};
const Segment SGI31597[] = {{&S36399[119], 3}, {NULL}};
const Segment SGI31598[] = {{&S36399[85], 5}, {NULL}};
const Segment SGI31599[] = {{&S36400[44], 7}, {NULL}};
const Segment SGI31600[] = {{&S36400[44], 58}, {NULL}};
const Segment SGI31601[] = {{&S36400[30], 6}, {NULL}};
const Segment SGI31602[] = {{&S36400[150], 3}, {NULL}};
const Segment SGI31603[] = {{&S36400[116], 5}, {NULL}};
const Segment SGI31604[] = {{&S25868[44], 19}, {NULL}};
const Segment SGI31605[] = {{&S25868[44], 36}, {NULL}};
const Segment SGI31606[] = {{&S25868[30], 6}, {NULL}};
const Segment SGI31607[] = {{&S25868[69], 3}, {NULL}};
const Segment SGI31608[] = {{&S25868[94], 5}, {NULL}};
const Segment SGI31609[] = {{&S25869[44], 19}, {NULL}};
const Segment SGI31610[] = {{&S25869[44], 38}, {NULL}};
const Segment SGI31611[] = {{&S25869[30], 6}, {NULL}};
const Segment SGI31612[] = {{&S25869[69], 3}, {NULL}};
const Segment SGI31613[] = {{&S25869[96], 5}, {NULL}};
const Segment SGI31614[] = {{&S25870[44], 19}, {NULL}};
const Segment SGI31615[] = {{&S25870[44], 37}, {NULL}};
const Segment SGI31616[] = {{&S25870[30], 6}, {NULL}};
const Segment SGI31617[] = {{&S25870[69], 3}, {NULL}};
const Segment SGI31618[] = {{&S25870[95], 5}, {NULL}};
const Segment SGI31619[] = {{&S25871[44], 19}, {NULL}};
const Segment SGI31620[] = {{&S25871[44], 38}, {NULL}};
const Segment SGI31621[] = {{&S25871[30], 6}, {NULL}};
const Segment SGI31622[] = {{&S25871[69], 3}, {NULL}};
const Segment SGI31623[] = {{&S25871[96], 5}, {NULL}};
const Segment SGI31624[] = {{&S36410[44], 7}, {NULL}};
const Segment SGI31625[] = {{&S36410[44], 58}, {NULL}};
const Segment SGI31626[] = {{&S36410[30], 6}, {NULL}};
const Segment SGI31627[] = {{&S36410[150], 3}, {NULL}};
const Segment SGI31628[] = {{&S36410[116], 5}, {NULL}};
const Segment SGI31629[] = {{&S36411[44], 7}, {NULL}};
const Segment SGI31630[] = {{&S36411[44], 58}, {NULL}};
const Segment SGI31631[] = {{&S36411[30], 6}, {NULL}};
const Segment SGI31632[] = {{&S36411[150], 3}, {NULL}};
const Segment SGI31633[] = {{&S36411[116], 5}, {NULL}};
const Segment SGI31634[] = {{&S36415[44], 7}, {NULL}};
const Segment SGI31635[] = {{&S36415[44], 27}, {NULL}};
const Segment SGI31636[] = {{&S36415[30], 6}, {NULL}};
const Segment SGI31637[] = {{&S36415[119], 3}, {NULL}};
const Segment SGI31638[] = {{&S36415[85], 5}, {NULL}};
const Segment SGI31639[] = {{&S36416[44], 7}, {NULL}};
const Segment SGI31640[] = {{&S36416[44], 62}, {NULL}};
const Segment SGI31641[] = {{&S36416[30], 6}, {NULL}};
const Segment SGI31642[] = {{&S36416[70], 3}, {NULL}};
const Segment SGI31643[] = {{&S36416[120], 5}, {NULL}};
const Segment SGI31644[] = {{&S36417[55], 7}, {NULL}};
const Segment SGI31645[] = {{&S36417[55], 28}, {NULL}};
const Segment SGI31646[] = {{&S36417[41], 6}, {NULL}};
const Segment SGI31647[] = {{&S36417[80], 3}, {NULL}};
const Segment SGI31648[] = {{&S36417[97], 5}, {NULL}};
const Segment SGI31649[] = {{&S36418[44], 7}, {NULL}};
const Segment SGI31650[] = {{&S36418[44], 27}, {NULL}};
const Segment SGI31651[] = {{&S36418[30], 6}, {NULL}};
const Segment SGI31652[] = {{&S36418[119], 3}, {NULL}};
const Segment SGI31653[] = {{&S36418[85], 5}, {NULL}};
const Segment SGI31654[] = {{&S36419[44], 7}, {NULL}};
const Segment SGI31655[] = {{&S36419[44], 63}, {NULL}};
const Segment SGI31656[] = {{&S36419[30], 6}, {NULL}};
const Segment SGI31657[] = {{&S36419[155], 3}, {NULL}};
const Segment SGI31658[] = {{&S36419[121], 5}, {NULL}};
const Segment SGI31659[] = {{&S25891[44], 19}, {NULL}};
const Segment SGI31660[] = {{&S25891[44], 38}, {NULL}};
const Segment SGI31661[] = {{&S25891[30], 6}, {NULL}};
const Segment SGI31662[] = {{&S36420[44], 7}, {NULL}};
const Segment SGI31663[] = {{&S25891[69], 3}, {NULL}};
const Segment SGI31664[] = {{&S36420[44], 27}, {NULL}};
const Segment SGI31665[] = {{&S36420[30], 6}, {NULL}};
const Segment SGI31666[] = {{&S25891[96], 5}, {NULL}};
const Segment SGI31667[] = {{&S36420[119], 3}, {NULL}};
const Segment SGI31668[] = {{&S36420[85], 5}, {NULL}};
const Segment SGI31669[] = {{&S24957[44], 10}, {NULL}};
const Segment SGI31670[] = {{&S24957[44], 27}, {NULL}};
const Segment SGI31671[] = {{&S24957[30], 6}, {NULL}};
const Segment SGI31672[] = {{&S24957[119], 3}, {NULL}};
const Segment SGI31673[] = {{&S24957[85], 5}, {NULL}};
const Segment SGI31674[] = {{&S36421[44], 7}, {NULL}};
const Segment SGI31675[] = {{&S36421[44], 27}, {NULL}};
const Segment SGI31676[] = {{&S36421[30], 6}, {NULL}};
const Segment SGI31677[] = {{&S36421[119], 3}, {NULL}};
const Segment SGI31678[] = {{&S24958[38], 10}, {NULL}};
const Segment SGI31679[] = {{&S36421[85], 5}, {NULL}};
const Segment SGI31680[] = {{&S24958[31], 6}, {NULL}};
const Segment SGI31681[] = {{&S25894[44], 19}, {NULL}};
const Segment SGI31682[] = {{&S25894[44], 38}, {NULL}};
const Segment SGI31683[] = {{&S36422[44], 7}, {NULL}};
const Segment SGI31684[] = {{&S25894[30], 6}, {NULL}};
const Segment SGI31685[] = {{&S36422[44], 65}, {NULL}};
const Segment SGI31686[] = {{&S36422[30], 6}, {NULL}};
const Segment SGI31687[] = {{&S25894[69], 3}, {NULL}};
const Segment SGI31688[] = {{&S25894[96], 5}, {NULL}};
const Segment SGI31689[] = {{&S36422[157], 3}, {NULL}};
const Segment SGI31690[] = {{&S36422[123], 5}, {NULL}};
const Segment SGI31691[] = {{&S25895[44], 7}, {NULL}};
const Segment SGI31692[] = {{&S25895[44], 26}, {NULL}};
const Segment SGI31693[] = {{&S25895[30], 6}, {NULL}};
const Segment SGI31694[] = {{&S36423[42], 7}, {NULL}};
const Segment SGI31695[] = {{&S36423[30], 6}, {NULL}};
const Segment SGI31696[] = {{&S25895[57], 3}, {NULL}};
const Segment SGI31697[] = {{&S25895[84], 5}, {NULL}};
const Segment SGI31698[] = {{&S36423[151], 3}, {NULL}};
const Segment SGI31699[] = {{&S24962[44], 11}, {NULL}};
const Segment SGI31700[] = {{&S36423[117], 5}, {NULL}};
const Segment SGI31701[] = {{&S24962[44], 28}, {NULL}};
const Segment SGI31702[] = {{&S24962[30], 6}, {NULL}};
const Segment SGI31703[] = {{&S24962[120], 3}, {NULL}};
const Segment SGI31704[] = {{&S24962[86], 5}, {NULL}};
const Segment SGI31705[] = {{&S25896[44], 19}, {NULL}};
const Segment SGI31706[] = {{&S36424[44], 7}, {NULL}};
const Segment SGI31707[] = {{&S25896[44], 38}, {NULL}};
const Segment SGI31708[] = {{&S36424[44], 62}, {NULL}};
const Segment SGI31709[] = {{&S25896[30], 6}, {NULL}};
const Segment SGI31710[] = {{&S36424[30], 6}, {NULL}};
const Segment SGI31711[] = {{&S25896[69], 3}, {NULL}};
const Segment SGI31712[] = {{&S36424[154], 3}, {NULL}};
const Segment SGI31713[] = {{&S25896[96], 5}, {NULL}};
const Segment SGI31714[] = {{&S36424[120], 5}, {NULL}};
const Segment SGI31715[] = {{&S36425[44], 7}, {NULL}};
const Segment SGI31716[] = {{&S36425[44], 61}, {NULL}};
const Segment SGI31717[] = {{&S36425[30], 6}, {NULL}};
const Segment SGI31718[] = {{&S36425[153], 3}, {NULL}};
const Segment SGI31719[] = {{&S36425[119], 5}, {NULL}};
const Segment SGI31720[] = {{&S36426[44], 7}, {NULL}};
const Segment SGI31721[] = {{&S36426[44], 61}, {NULL}};
const Segment SGI31722[] = {{&S24967[44], 10}, {NULL}};
const Segment SGI31723[] = {{&S36426[30], 6}, {NULL}};
const Segment SGI31724[] = {{&S24967[44], 27}, {NULL}};
const Segment SGI31725[] = {{&S24967[30], 6}, {NULL}};
const Segment SGI31726[] = {{&S36426[153], 3}, {NULL}};
const Segment SGI31727[] = {{&S24967[88], 6}, {NULL}};
const Segment SGI31728[] = {{&S36426[119], 5}, {NULL}};
const Segment SGI31729[] = {{&S36427[44], 7}, {NULL}};
const Segment SGI31730[] = {{&S36427[44], 61}, {NULL}};
const Segment SGI31731[] = {{&S36427[30], 6}, {NULL}};
const Segment SGI31732[] = {{&S36427[153], 3}, {NULL}};
const Segment SGI31733[] = {{&S25902[44], 19}, {NULL}};
const Segment SGI31734[] = {{&S25902[44], 37}, {NULL}};
const Segment SGI31735[] = {{&S36427[119], 5}, {NULL}};
const Segment SGI31736[] = {{&S25902[30], 6}, {NULL}};
const Segment SGI31737[] = {{&S25902[69], 3}, {NULL}};
const Segment SGI31738[] = {{&S25902[95], 5}, {NULL}};
const Segment SGI31739[] = {{&S36428[44], 7}, {NULL}};
const Segment SGI31740[] = {{&S36428[44], 62}, {NULL}};
const Segment SGI31741[] = {{&S36428[30], 6}, {NULL}};
const Segment SGI31742[] = {{&S25903[44], 7}, {NULL}};
const Segment SGI31743[] = {{&S36428[154], 3}, {NULL}};
const Segment SGI31744[] = {{&S25903[44], 25}, {NULL}};
const Segment SGI31745[] = {{&S25903[30], 6}, {NULL}};
const Segment SGI31746[] = {{&S36428[120], 5}, {NULL}};
const Segment SGI31747[] = {{&S25903[57], 3}, {NULL}};
const Segment SGI31748[] = {{&S25903[83], 5}, {NULL}};
const Segment SGI31749[] = {{&S36429[44], 7}, {NULL}};
const Segment SGI31750[] = {{&S36429[44], 63}, {NULL}};
const Segment SGI31751[] = {{&S36429[30], 6}, {NULL}};
const Segment SGI31752[] = {{&S25904[44], 20}, {NULL}};
const Segment SGI31753[] = {{&S36429[155], 3}, {NULL}};
const Segment SGI31754[] = {{&S25904[44], 38}, {NULL}};
const Segment SGI31755[] = {{&S25904[30], 6}, {NULL}};
const Segment SGI31756[] = {{&S36429[121], 5}, {NULL}};
const Segment SGI31757[] = {{&S25904[70], 3}, {NULL}};
const Segment SGI31758[] = {{&S25904[96], 5}, {NULL}};
const Segment SGI31759[] = {{&S36430[44], 7}, {NULL}};
const Segment SGI31760[] = {{&S36430[44], 27}, {NULL}};
const Segment SGI31761[] = {{&S36430[30], 6}, {NULL}};
const Segment SGI31762[] = {{&S36430[119], 3}, {NULL}};
const Segment SGI31763[] = {{&S25905[44], 21}, {NULL}};
const Segment SGI31764[] = {{&S36430[85], 5}, {NULL}};
const Segment SGI31765[] = {{&S25905[44], 39}, {NULL}};
const Segment SGI31766[] = {{&S25905[30], 6}, {NULL}};
const Segment SGI31767[] = {{&S25905[71], 3}, {NULL}};
const Segment SGI31768[] = {{&S25905[97], 5}, {NULL}};
const Segment SGI31769[] = {{&S36431[44], 7}, {NULL}};
const Segment SGI31770[] = {{&S36431[44], 29}, {NULL}};
const Segment SGI31771[] = {{&S36431[30], 6}, {NULL}};
const Segment SGI31772[] = {{&S36431[121], 3}, {NULL}};
const Segment SGI31773[] = {{&S25906[44], 21}, {NULL}};
const Segment SGI31774[] = {{&S36431[87], 5}, {NULL}};
const Segment SGI31775[] = {{&S25906[44], 39}, {NULL}};
const Segment SGI31776[] = {{&S36432[55], 7}, {NULL}};
const Segment SGI31777[] = {{&S36432[55], 32}, {NULL}};
const Segment SGI31778[] = {{&S36432[41], 6}, {NULL}};
const Segment SGI31779[] = {{&S36432[135], 3}, {NULL}};
const Segment SGI31780[] = {{&S36432[101], 5}, {NULL}};
const Segment SGI31781[] = {{&S36433[44], 7}, {NULL}};
const Segment SGI31782[] = {{&S36433[44], 68}, {NULL}};
const Segment SGI31783[] = {{&S36433[30], 6}, {NULL}};
const Segment SGI31784[] = {{&S36433[160], 3}, {NULL}};
const Segment SGI31785[] = {{&S36433[126], 5}, {NULL}};
const Segment SGI31786[] = {{&S36434[44], 7}, {NULL}};
const Segment SGI31787[] = {{&S36434[44], 62}, {NULL}};
const Segment SGI31788[] = {{&S36434[30], 6}, {NULL}};
const Segment SGI31789[] = {{&S36434[154], 3}, {NULL}};
const Segment SGI31790[] = {{&S36434[120], 5}, {NULL}};
const Segment SGI31791[] = {{&S36435[44], 7}, {NULL}};
const Segment SGI31792[] = {{&S36435[44], 59}, {NULL}};
const Segment SGI31793[] = {{&S36435[30], 6}, {NULL}};
const Segment SGI31794[] = {{&S36435[151], 3}, {NULL}};
const Segment SGI31795[] = {{&S36435[117], 5}, {NULL}};
const Segment SGI31796[] = {{&S36436[55], 7}, {NULL}};
const Segment SGI31797[] = {{&S36436[55], 32}, {NULL}};
const Segment SGI31798[] = {{&S36436[41], 6}, {NULL}};
const Segment SGI31799[] = {{&S36436[135], 3}, {NULL}};
const Segment SGI31800[] = {{&S36436[101], 5}, {NULL}};
const Segment SGI31801[] = {{&S36438[44], 7}, {NULL}};
const Segment SGI31802[] = {{&S36438[44], 28}, {NULL}};
const Segment SGI31803[] = {{&S36438[30], 6}, {NULL}};
const Segment SGI31804[] = {{&S36438[120], 3}, {NULL}};
const Segment SGI31805[] = {{&S36438[86], 5}, {NULL}};
const Segment SGI31806[] = {{&S36439[44], 7}, {NULL}};
const Segment SGI31807[] = {{&S36439[44], 27}, {NULL}};
const Segment SGI31808[] = {{&S36439[30], 6}, {NULL}};
const Segment SGI31809[] = {{&S36439[119], 3}, {NULL}};
const Segment SGI31810[] = {{&S36439[85], 5}, {NULL}};
const Segment SGI31811[] = {{&S36440[44], 7}, {NULL}};
const Segment SGI31812[] = {{&S36440[44], 27}, {NULL}};
const Segment SGI31813[] = {{&S36440[30], 6}, {NULL}};
const Segment SGI31814[] = {{&S36440[119], 3}, {NULL}};
const Segment SGI31815[] = {{&S36440[85], 5}, {NULL}};
const Segment SGI31816[] = {{&S36441[44], 7}, {NULL}};
const Segment SGI31817[] = {{&S36441[44], 62}, {NULL}};
const Segment SGI31818[] = {{&S36441[30], 6}, {NULL}};
const Segment SGI31819[] = {{&S36441[154], 3}, {NULL}};
const Segment SGI31820[] = {{&S36441[120], 5}, {NULL}};
const Segment SGI31821[] = {{&S36442[44], 7}, {NULL}};
const Segment SGI31822[] = {{&S36442[44], 59}, {NULL}};
const Segment SGI31823[] = {{&S36442[30], 6}, {NULL}};
const Segment SGI31824[] = {{&S36442[151], 3}, {NULL}};
const Segment SGI31825[] = {{&S36442[117], 5}, {NULL}};
const Segment SGI31826[] = {{&S36444[44], 7}, {NULL}};
const Segment SGI31827[] = {{&S36444[44], 66}, {NULL}};
const Segment SGI31828[] = {{&S36444[30], 6}, {NULL}};
const Segment SGI31829[] = {{&S36444[158], 3}, {NULL}};
const Segment SGI31830[] = {{&S36444[124], 5}, {NULL}};
const Segment SGI31831[] = {{&S36445[44], 7}, {NULL}};
const Segment SGI31832[] = {{&S36445[44], 58}, {NULL}};
const Segment SGI31833[] = {{&S36445[30], 6}, {NULL}};
const Segment SGI31834[] = {{&S36445[150], 3}, {NULL}};
const Segment SGI31835[] = {{&S36445[116], 5}, {NULL}};
const Segment SGI31836[] = {{&S36446[44], 7}, {NULL}};
const Segment SGI31837[] = {{&S36446[44], 27}, {NULL}};
const Segment SGI31838[] = {{&S36446[30], 6}, {NULL}};
const Segment SGI31839[] = {{&S36446[119], 3}, {NULL}};
const Segment SGI31840[] = {{&S36446[85], 5}, {NULL}};
const Segment SGI31841[] = {{&S36447[44], 7}, {NULL}};
const Segment SGI31842[] = {{&S36447[44], 61}, {NULL}};
const Segment SGI31843[] = {{&S36447[30], 6}, {NULL}};
const Segment SGI31844[] = {{&S36447[153], 3}, {NULL}};
const Segment SGI31845[] = {{&S36447[119], 5}, {NULL}};
const Segment SGI31846[] = {{&S36448[44], 7}, {NULL}};
const Segment SGI31847[] = {{&S36448[44], 59}, {NULL}};
const Segment SGI31848[] = {{&S36448[30], 6}, {NULL}};
const Segment SGI31849[] = {{&S36448[151], 3}, {NULL}};
const Segment SGI31850[] = {{&S36448[117], 5}, {NULL}};
const Segment SGI31851[] = {{&S36450[44], 7}, {NULL}};
const Segment SGI31852[] = {{&S36450[44], 62}, {NULL}};
const Segment SGI31853[] = {{&S36450[30], 6}, {NULL}};
const Segment SGI31854[] = {{&S36450[154], 3}, {NULL}};
const Segment SGI31855[] = {{&S36450[120], 5}, {NULL}};
const Segment SGI31856[] = {{&S36451[55], 7}, {NULL}};
const Segment SGI31857[] = {{&S36451[55], 32}, {NULL}};
const Segment SGI31858[] = {{&S36451[41], 6}, {NULL}};
const Segment SGI31859[] = {{&S36451[80], 3}, {NULL}};
const Segment SGI31860[] = {{&S36451[101], 5}, {NULL}};
const Segment SGI31861[] = {{&S36452[44], 7}, {NULL}};
const Segment SGI31862[] = {{&S36452[44], 65}, {NULL}};
const Segment SGI31863[] = {{&S36452[30], 6}, {NULL}};
const Segment SGI31864[] = {{&S36452[157], 3}, {NULL}};
const Segment SGI31865[] = {{&S36452[123], 5}, {NULL}};
const Segment SGI31866[] = {{&S36453[44], 7}, {NULL}};
const Segment SGI31867[] = {{&S36453[44], 63}, {NULL}};
const Segment SGI31868[] = {{&S36453[30], 6}, {NULL}};
const Segment SGI31869[] = {{&S36453[155], 3}, {NULL}};
const Segment SGI31870[] = {{&S36453[121], 5}, {NULL}};
const Segment SGI31871[] = {{&S36454[44], 7}, {NULL}};
const Segment SGI31872[] = {{&S36454[44], 27}, {NULL}};
const Segment SGI31873[] = {{&S36454[30], 6}, {NULL}};
const Segment SGI31874[] = {{&S36454[119], 3}, {NULL}};
const Segment SGI31875[] = {{&S36454[85], 5}, {NULL}};
const Segment SGI31876[] = {{&S36455[44], 7}, {NULL}};
const Segment SGI31877[] = {{&S36455[44], 27}, {NULL}};
const Segment SGI31878[] = {{&S36455[30], 6}, {NULL}};
const Segment SGI31879[] = {{&S36455[119], 3}, {NULL}};
const Segment SGI31880[] = {{&S36455[85], 5}, {NULL}};
const Segment SGI31881[] = {{&S36456[44], 7}, {NULL}};
const Segment SGI31882[] = {{&S36456[44], 61}, {NULL}};
const Segment SGI31883[] = {{&S36456[30], 6}, {NULL}};
const Segment SGI31884[] = {{&S36456[153], 3}, {NULL}};
const Segment SGI31885[] = {{&S36456[119], 5}, {NULL}};
const Segment SGI31886[] = {{&S36457[44], 7}, {NULL}};
const Segment SGI31887[] = {{&S36457[44], 61}, {NULL}};
const Segment SGI31888[] = {{&S36457[30], 6}, {NULL}};
const Segment SGI31889[] = {{&S36457[153], 3}, {NULL}};
const Segment SGI31890[] = {{&S36457[119], 5}, {NULL}};
const Segment SGI31891[] = {{&S36458[44], 7}, {NULL}};
const Segment SGI31892[] = {{&S36458[44], 27}, {NULL}};
const Segment SGI31893[] = {{&S36458[30], 6}, {NULL}};
const Segment SGI31894[] = {{&S36458[119], 3}, {NULL}};
const Segment SGI31895[] = {{&S36458[85], 5}, {NULL}};
const Segment SGI31896[] = {{&S36459[44], 7}, {NULL}};
const Segment SGI31897[] = {{&S36459[44], 57}, {NULL}};
const Segment SGI31898[] = {{&S36459[30], 6}, {NULL}};
const Segment SGI31899[] = {{&S36459[149], 3}, {NULL}};
const Segment SGI31900[] = {{&S36459[115], 5}, {NULL}};
const Segment SGI31901[] = {{&S27139[44], 10}, {NULL}};
const Segment SGI31902[] = {{&S27139[44], 27}, {NULL}};
const Segment SGI31903[] = {{&S27139[30], 6}, {NULL}};
const Segment SGI31904[] = {{&S36460[44], 7}, {NULL}};
const Segment SGI31905[] = {{&S36460[44], 61}, {NULL}};
const Segment SGI31906[] = {{&S27139[119], 3}, {NULL}};
const Segment SGI31907[] = {{&S36460[30], 6}, {NULL}};
const Segment SGI31908[] = {{&S27139[85], 5}, {NULL}};
const Segment SGI31909[] = {{&S36460[153], 3}, {NULL}};
const Segment SGI31910[] = {{&S36460[119], 5}, {NULL}};
const Segment SGI31911[] = {{&S36461[55], 7}, {NULL}};
const Segment SGI31912[] = {{&S36461[55], 32}, {NULL}};
const Segment SGI31913[] = {{&S36461[41], 6}, {NULL}};
const Segment SGI31914[] = {{&S36461[80], 3}, {NULL}};
const Segment SGI31915[] = {{&S36461[101], 5}, {NULL}};
const Segment SGI31916[] = {{&S36462[44], 7}, {NULL}};
const Segment SGI31917[] = {{&S36462[44], 67}, {NULL}};
const Segment SGI31918[] = {{&S36462[30], 6}, {NULL}};
const Segment SGI31919[] = {{&S36462[159], 3}, {NULL}};
const Segment SGI31920[] = {{&S36462[125], 5}, {NULL}};
const Segment SGI31921[] = {{&S36463[44], 7}, {NULL}};
const Segment SGI31922[] = {{&S36463[44], 58}, {NULL}};
const Segment SGI31923[] = {{&S36463[30], 6}, {NULL}};
const Segment SGI31924[] = {{&S36463[150], 3}, {NULL}};
const Segment SGI31925[] = {{&S36463[116], 5}, {NULL}};
const Segment SGI31926[] = {{&S36464[55], 7}, {NULL}};
const Segment SGI31927[] = {{&S36464[55], 28}, {NULL}};
const Segment SGI31928[] = {{&S36464[41], 6}, {NULL}};
const Segment SGI31929[] = {{&S36464[131], 3}, {NULL}};
const Segment SGI31930[] = {{&S36464[97], 5}, {NULL}};
const Segment SGI31931[] = {{&S36465[44], 7}, {NULL}};
const Segment SGI31932[] = {{&S36465[44], 63}, {NULL}};
const Segment SGI31933[] = {{&S36465[30], 6}, {NULL}};
const Segment SGI31934[] = {{&S36465[155], 3}, {NULL}};
const Segment SGI31935[] = {{&S36465[121], 5}, {NULL}};
const Segment SGI31936[] = {{&S36467[38], 7}, {NULL}};
const Segment SGI31937[] = {{&S36467[31], 6}, {NULL}};
const Segment SGI31938[] = {{&S36468[44], 7}, {NULL}};
const Segment SGI31939[] = {{&S36468[44], 61}, {NULL}};
const Segment SGI31940[] = {{&S36468[30], 6}, {NULL}};
const Segment SGI31941[] = {{&S36468[153], 3}, {NULL}};
const Segment SGI31942[] = {{&S36468[119], 5}, {NULL}};
const Segment SGI31943[] = {{&S36470[44], 7}, {NULL}};
const Segment SGI31944[] = {{&S36470[44], 27}, {NULL}};
const Segment SGI31945[] = {{&S36470[30], 6}, {NULL}};
const Segment SGI31946[] = {{&S36470[88], 4}, {NULL}};
const Segment SGI31947[] = {{&S36470[88], 3}, {NULL}};
const Segment SGI31948[] = {{&S36471[44], 7}, {NULL}};
const Segment SGI31949[] = {{&S36471[44], 27}, {NULL}};
const Segment SGI31950[] = {{&S36471[30], 6}, {NULL}};
const Segment SGI31951[] = {{&S36471[119], 3}, {NULL}};
const Segment SGI31952[] = {{&S36471[85], 5}, {NULL}};
const Segment SGI31953[] = {{&S36472[55], 7}, {NULL}};
const Segment SGI31954[] = {{&S36472[55], 32}, {NULL}};
const Segment SGI31955[] = {{&S36472[41], 6}, {NULL}};
const Segment SGI31956[] = {{&S36472[80], 3}, {NULL}};
const Segment SGI31957[] = {{&S36472[101], 5}, {NULL}};
const Segment SGI31958[] = {{&S36474[44], 7}, {NULL}};
const Segment SGI31959[] = {{&S36474[44], 27}, {NULL}};
const Segment SGI31960[] = {{&S36474[30], 6}, {NULL}};
const Segment SGI31961[] = {{&S36474[88], 6}, {NULL}};
const Segment SGI31962[] = {{&S36475[44], 7}, {NULL}};
const Segment SGI31963[] = {{&S36475[44], 27}, {NULL}};
const Segment SGI31964[] = {{&S36475[30], 6}, {NULL}};
const Segment SGI31965[] = {{&S36475[119], 3}, {NULL}};
const Segment SGI31966[] = {{&S36475[85], 5}, {NULL}};
const Segment SGI31967[] = {{&S36476[44], 7}, {NULL}};
const Segment SGI31968[] = {{&S36476[44], 61}, {NULL}};
const Segment SGI31969[] = {{&S36476[30], 6}, {NULL}};
const Segment SGI31970[] = {{&S36476[153], 3}, {NULL}};
const Segment SGI31971[] = {{&S36476[119], 5}, {NULL}};
const Segment SGI31972[] = {{&S36477[44], 7}, {NULL}};
const Segment SGI31973[] = {{&S36477[44], 27}, {NULL}};
const Segment SGI31974[] = {{&S36477[30], 6}, {NULL}};
const Segment SGI31975[] = {{&S36477[88], 6}, {NULL}};
const Segment SGI31976[] = {{&S25906[30], 6}, {NULL}};
const Segment SGI31977[] = {{&S25906[71], 3}, {NULL}};
const Segment SGI31978[] = {{&S25906[97], 5}, {NULL}};
const Segment SGI31979[] = {{&S25907[44], 19}, {NULL}};
const Segment SGI31980[] = {{&S25907[44], 37}, {NULL}};
const Segment SGI31981[] = {{&S25907[30], 6}, {NULL}};
const Segment SGI31982[] = {{&S25907[69], 3}, {NULL}};
const Segment SGI31983[] = {{&S25907[95], 5}, {NULL}};
const Segment SGI31984[] = {{&S25908[44], 7}, {NULL}};
const Segment SGI31985[] = {{&S25908[44], 25}, {NULL}};
const Segment SGI31986[] = {{&S25908[30], 6}, {NULL}};
const Segment SGI31987[] = {{&S25908[57], 3}, {NULL}};
const Segment SGI31988[] = {{&S25908[83], 5}, {NULL}};
const Segment SGI31989[] = {{&S25909[38], 7}, {NULL}};
const Segment SGI31990[] = {{&S25909[31], 6}, {NULL}};
const Segment SGI31991[] = {{&S25910[44], 19}, {NULL}};
const Segment SGI31992[] = {{&S25910[44], 37}, {NULL}};
const Segment SGI31993[] = {{&S25910[30], 6}, {NULL}};
const Segment SGI31994[] = {{&S25910[69], 3}, {NULL}};
const Segment SGI31995[] = {{&S25910[95], 5}, {NULL}};
const Segment SGI31996[] = {{&S25911[44], 20}, {NULL}};
const Segment SGI31997[] = {{&S25911[44], 38}, {NULL}};
const Segment SGI31998[] = {{&S25911[30], 6}, {NULL}};
const Segment SGI31999[] = {{&S25911[70], 3}, {NULL}};
const Segment SGI32000[] = {{&S25911[96], 5}, {NULL}};
const Segment SGI32001[] = {{&S25912[44], 7}, {NULL}};
const Segment SGI32002[] = {{&S25912[44], 25}, {NULL}};
const Segment SGI32003[] = {{&S25912[30], 6}, {NULL}};
const Segment SGI32004[] = {{&S25912[86], 4}, {NULL}};
const Segment SGI32005[] = {{&S25912[86], 3}, {NULL}};
const Segment SGI32006[] = {{&S25913[44], 7}, {NULL}};
const Segment SGI32007[] = {{&S25913[44], 25}, {NULL}};
const Segment SGI32008[] = {{&S25913[30], 6}, {NULL}};
const Segment SGI32009[] = {{&S25913[57], 3}, {NULL}};
const Segment SGI32010[] = {{&S25913[83], 5}, {NULL}};
const Segment SGI32011[] = {{&S25914[44], 7}, {NULL}};
const Segment SGI32012[] = {{&S25914[44], 25}, {NULL}};
const Segment SGI32013[] = {{&S25914[30], 6}, {NULL}};
const Segment SGI32014[] = {{&S25914[86], 4}, {NULL}};
const Segment SGI32015[] = {{&S25914[86], 3}, {NULL}};
const Segment SGI32016[] = {{&S25915[44], 19}, {NULL}};
const Segment SGI32017[] = {{&S25915[44], 37}, {NULL}};
const Segment SGI32018[] = {{&S25915[30], 6}, {NULL}};
const Segment SGI32019[] = {{&S25915[69], 3}, {NULL}};
const Segment SGI32020[] = {{&S25915[95], 5}, {NULL}};
const Segment SGI32021[] = {{&S25916[44], 7}, {NULL}};
const Segment SGI32022[] = {{&S25916[44], 25}, {NULL}};
const Segment SGI32023[] = {{&S25916[30], 6}, {NULL}};
const Segment SGI32024[] = {{&S25916[86], 6}, {NULL}};
const Segment SGI32025[] = {{&S25920[44], 7}, {NULL}};
const Segment SGI32026[] = {{&S25920[44], 25}, {NULL}};
const Segment SGI32027[] = {{&S25920[30], 6}, {NULL}};
const Segment SGI32028[] = {{&S25920[86], 6}, {NULL}};
const Segment SGI32029[] = {{&S27252[44], 11}, {NULL}};
const Segment SGI32030[] = {{&S27252[44], 28}, {NULL}};
const Segment SGI32031[] = {{&S27252[30], 6}, {NULL}};
const Segment SGI32032[] = {{&S27252[120], 3}, {NULL}};
const Segment SGI32033[] = {{&S27252[86], 5}, {NULL}};
const Segment SGI32034[] = {{&S27253[44], 11}, {NULL}};
const Segment SGI32035[] = {{&S27253[44], 28}, {NULL}};
const Segment SGI32036[] = {{&S27253[30], 6}, {NULL}};
const Segment SGI32037[] = {{&S27253[120], 3}, {NULL}};
const Segment SGI32038[] = {{&S27253[86], 5}, {NULL}};
const Segment SGI32039[] = {{&S27254[44], 11}, {NULL}};
const Segment SGI32040[] = {{&S27254[44], 28}, {NULL}};
const Segment SGI32041[] = {{&S27254[30], 6}, {NULL}};
const Segment SGI32042[] = {{&S27254[120], 3}, {NULL}};
const Segment SGI32043[] = {{&S27254[86], 5}, {NULL}};
const Segment SGI32044[] = {{&S27255[38], 11}, {NULL}};
const Segment SGI32045[] = {{&S27255[31], 6}, {NULL}};
const Segment SGI32046[] = {{&S27271[38], 11}, {NULL}};
const Segment SGI32047[] = {{&S27271[31], 6}, {NULL}};
const Segment SGI32048[] = {{&S27272[38], 11}, {NULL}};
const Segment SGI32049[] = {{&S27272[31], 6}, {NULL}};
const Segment SGI32050[] = {{&S26033[44], 6}, {NULL}};
const Segment SGI32051[] = {{&S26033[44], 22}, {NULL}};
const Segment SGI32052[] = {{&S26033[30], 6}, {NULL}};
const Segment SGI32053[] = {{&S26033[114], 3}, {NULL}};
const Segment SGI32054[] = {{&S26033[80], 5}, {NULL}};
const Segment SGI32055[] = {{&S36904[44], 8}, {NULL}};
const Segment SGI32056[] = {{&S36904[44], 20}, {NULL}};
const Segment SGI32057[] = {{&S36904[30], 6}, {NULL}};
const Segment SGI32058[] = {{&S36904[112], 3}, {NULL}};
const Segment SGI32059[] = {{&S36904[78], 5}, {NULL}};
const Segment SGI32060[] = {{&S36905[38], 8}, {NULL}};
const Segment SGI32061[] = {{&S36905[31], 6}, {NULL}};
const Segment SGI32062[] = {{&S36908[44], 8}, {NULL}};
const Segment SGI32063[] = {{&S36908[44], 20}, {NULL}};
const Segment SGI32064[] = {{&S36908[30], 6}, {NULL}};
const Segment SGI32065[] = {{&S36909[44], 7}, {NULL}};
const Segment SGI32066[] = {{&S36909[44], 19}, {NULL}};
const Segment SGI32067[] = {{&S36909[30], 6}, {NULL}};
const Segment SGI32068[] = {{&S36909[111], 3}, {NULL}};
const Segment SGI32069[] = {{&S36909[77], 5}, {NULL}};
const Segment SGI32070[] = {{&S36910[44], 8}, {NULL}};
const Segment SGI32071[] = {{&S36910[44], 20}, {NULL}};
const Segment SGI32072[] = {{&S36910[30], 6}, {NULL}};
const Segment SGI32073[] = {{&S36910[112], 3}, {NULL}};
const Segment SGI32074[] = {{&S36910[78], 5}, {NULL}};
const Segment SGI32075[] = {{&S36911[43], 8}, {NULL}};
const Segment SGI32076[] = {{&S36911[43], 20}, {NULL}};
const Segment SGI32077[] = {{&S36911[111], 3}, {NULL}};
const Segment SGI32078[] = {{&S36911[77], 5}, {NULL}};
const Segment SGI32079[] = {{&S36912[44], 7}, {NULL}};
const Segment SGI32080[] = {{&S36912[44], 19}, {NULL}};
const Segment SGI32081[] = {{&S36912[30], 6}, {NULL}};
const Segment SGI32082[] = {{&S36912[111], 3}, {NULL}};
const Segment SGI32083[] = {{&S36912[77], 5}, {NULL}};
const Segment SGI32084[] = {{&S36913[43], 8}, {NULL}};
const Segment SGI32085[] = {{&S36913[43], 20}, {NULL}};
const Segment SGI32086[] = {{&S36913[30], 5}, {NULL}};
const Segment SGI32087[] = {{&S36913[111], 3}, {NULL}};
const Segment SGI32088[] = {{&S36913[77], 5}, {NULL}};
const Segment SGI32089[] = {{&S36914[44], 8}, {NULL}};
const Segment SGI32090[] = {{&S36914[44], 20}, {NULL}};
const Segment SGI32091[] = {{&S36914[30], 6}, {NULL}};
const Segment SGI32092[] = {{&S36914[114], 3}, {NULL}};
const Segment SGI32093[] = {{&S36914[78], 7}, {NULL}};
const Segment SGI32094[] = {{&S36914[134], 8}, {NULL}};
const Segment SGI32095[] = {{&S36916[42], 8}, {NULL}};
const Segment SGI32096[] = {{&S36916[30], 6}, {NULL}};
const Segment SGI32097[] = {{&S36916[110], 3}, {NULL}};
const Segment SGI32098[] = {{&S36916[76], 5}, {NULL}};
const Segment SGI32099[] = {{&S27716[44], 12}, {NULL}};
const Segment SGI32100[] = {{&S27716[44], 23}, {NULL}};
const Segment SGI32101[] = {{&S27716[30], 6}, {NULL}};
const Segment SGI32102[] = {{&S27716[115], 3}, {NULL}};
const Segment SGI32103[] = {{&S27716[81], 5}, {NULL}};
const Segment SGI32104[] = {{&S27717[44], 14}, {NULL}};
const Segment SGI32105[] = {{&S27717[44], 25}, {NULL}};
const Segment SGI32106[] = {{&S27717[30], 6}, {NULL}};
const Segment SGI32107[] = {{&S27717[117], 3}, {NULL}};
const Segment SGI32108[] = {{&S27717[83], 5}, {NULL}};
const Segment SGI32109[] = {{&S27718[44], 18}, {NULL}};
const Segment SGI32110[] = {{&S27718[44], 29}, {NULL}};
const Segment SGI32111[] = {{&S27718[30], 6}, {NULL}};
const Segment SGI32112[] = {{&S27718[121], 3}, {NULL}};
const Segment SGI32113[] = {{&S27718[87], 5}, {NULL}};
const Segment SGI32114[] = {{&S27719[44], 22}, {NULL}};
const Segment SGI32115[] = {{&S27719[44], 33}, {NULL}};
const Segment SGI32116[] = {{&S27719[30], 6}, {NULL}};
const Segment SGI32117[] = {{&S27719[125], 3}, {NULL}};
const Segment SGI32118[] = {{&S27719[91], 5}, {NULL}};
const Segment SGI32119[] = {{&S27720[44], 23}, {NULL}};
const Segment SGI32120[] = {{&S27720[44], 34}, {NULL}};
const Segment SGI32121[] = {{&S27720[30], 6}, {NULL}};
const Segment SGI32122[] = {{&S27720[126], 3}, {NULL}};
const Segment SGI32123[] = {{&S27720[92], 5}, {NULL}};
const Segment SGI32124[] = {{&S27721[44], 30}, {NULL}};
const Segment SGI32125[] = {{&S27721[44], 41}, {NULL}};
const Segment SGI32126[] = {{&S27721[30], 6}, {NULL}};
const Segment SGI32127[] = {{&S27721[133], 3}, {NULL}};
const Segment SGI32128[] = {{&S27721[99], 5}, {NULL}};
const Segment SGI32129[] = {{&S27722[44], 8}, {NULL}};
const Segment SGI32130[] = {{&S27722[44], 19}, {NULL}};
const Segment SGI32131[] = {{&S27722[30], 6}, {NULL}};
const Segment SGI32132[] = {{&S27722[111], 3}, {NULL}};
const Segment SGI32133[] = {{&S27722[77], 5}, {NULL}};
const Segment SGI32134[] = {{&S27725[42], 18}, {NULL}};
const Segment SGI32135[] = {{&S27725[30], 6}, {NULL}};
const Segment SGI32136[] = {{&S27725[119], 3}, {NULL}};
const Segment SGI32137[] = {{&S27725[85], 5}, {NULL}};
const Segment SGI32138[] = {{&S27726[44], 12}, {NULL}};
const Segment SGI32139[] = {{&S27726[44], 23}, {NULL}};
const Segment SGI32140[] = {{&S27726[30], 6}, {NULL}};
const Segment SGI32141[] = {{&S27726[115], 3}, {NULL}};
const Segment SGI32142[] = {{&S27726[81], 5}, {NULL}};
const Segment SGI32143[] = {{&S27727[44], 17}, {NULL}};
const Segment SGI32144[] = {{&S27727[44], 28}, {NULL}};
const Segment SGI32145[] = {{&S27727[30], 6}, {NULL}};
const Segment SGI32146[] = {{&S27727[120], 3}, {NULL}};
const Segment SGI32147[] = {{&S27727[86], 5}, {NULL}};
const Segment SGI32148[] = {{&S27728[38], 12}, {NULL}};
const Segment SGI32149[] = {{&S27728[31], 6}, {NULL}};
const Segment SGI32150[] = {{&S27739[44], 14}, {NULL}};
const Segment SGI32151[] = {{&S27739[44], 25}, {NULL}};
const Segment SGI32152[] = {{&S27739[30], 6}, {NULL}};
const Segment SGI32153[] = {{&S27739[117], 3}, {NULL}};
const Segment SGI32154[] = {{&S27739[83], 5}, {NULL}};
const Segment SGI32155[] = {{&S27752[44], 12}, {NULL}};
const Segment SGI32156[] = {{&S27752[44], 23}, {NULL}};
const Segment SGI32157[] = {{&S27752[30], 6}, {NULL}};
const Segment SGI32158[] = {{&S27752[115], 3}, {NULL}};
const Segment SGI32159[] = {{&S27752[81], 5}, {NULL}};
const Segment SGI32160[] = {{&S25618[44], 20}, {NULL}};
const Segment SGI32161[] = {{&S25618[44], 45}, {NULL}};
const Segment SGI32162[] = {{&S25618[30], 6}, {NULL}};
const Segment SGI32163[] = {{&S25618[137], 3}, {NULL}};
const Segment SGI32164[] = {{&S25618[103], 5}, {NULL}};
const Segment SGI32165[] = {{&S25619[38], 20}, {NULL}};
const Segment SGI32166[] = {{&S25619[31], 6}, {NULL}};
const Segment SGI32167[] = {{&S27759[44], 12}, {NULL}};
const Segment SGI32168[] = {{&S27759[44], 23}, {NULL}};
const Segment SGI32169[] = {{&S27759[30], 6}, {NULL}};
const Segment SGI32170[] = {{&S25621[44], 7}, {NULL}};
const Segment SGI32171[] = {{&S25621[44], 32}, {NULL}};
const Segment SGI32172[] = {{&S27759[84], 4}, {NULL}};
const Segment SGI32173[] = {{&S25621[30], 6}, {NULL}};
const Segment SGI32174[] = {{&S27759[84], 3}, {NULL}};
const Segment SGI32175[] = {{&S25621[124], 3}, {NULL}};
const Segment SGI32176[] = {{&S25621[90], 5}, {NULL}};
const Segment SGI32177[] = {{&S25622[44], 7}, {NULL}};
const Segment SGI32178[] = {{&S25622[44], 32}, {NULL}};
const Segment SGI32179[] = {{&S25622[30], 6}, {NULL}};
const Segment SGI32180[] = {{&S25622[124], 3}, {NULL}};
const Segment SGI32181[] = {{&S25622[90], 5}, {NULL}};
const Segment SGI32182[] = {{&S25623[38], 7}, {NULL}};
const Segment SGI32183[] = {{&S25623[31], 6}, {NULL}};
const Segment SGI32184[] = {{&S25624[44], 7}, {NULL}};
const Segment SGI32185[] = {{&S25624[44], 37}, {NULL}};
const Segment SGI32186[] = {{&S25624[30], 6}, {NULL}};
const Segment SGI32187[] = {{&S25624[129], 3}, {NULL}};
const Segment SGI32188[] = {{&S25624[95], 5}, {NULL}};
const Segment SGI32189[] = {{&S25625[44], 7}, {NULL}};
const Segment SGI32190[] = {{&S25625[44], 31}, {NULL}};
const Segment SGI32191[] = {{&S25625[30], 6}, {NULL}};
const Segment SGI32192[] = {{&S25625[123], 3}, {NULL}};
const Segment SGI32193[] = {{&S25625[89], 5}, {NULL}};
const Segment SGI32194[] = {{&S25626[44], 7}, {NULL}};
const Segment SGI32195[] = {{&S25626[44], 37}, {NULL}};
const Segment SGI32196[] = {{&S25626[30], 6}, {NULL}};
const Segment SGI32197[] = {{&S25626[129], 3}, {NULL}};
const Segment SGI32198[] = {{&S25626[95], 5}, {NULL}};
const Segment SGI32199[] = {{&S25627[44], 20}, {NULL}};
const Segment SGI32200[] = {{&S25627[44], 45}, {NULL}};
const Segment SGI32201[] = {{&S25627[30], 6}, {NULL}};
const Segment SGI32202[] = {{&S25627[139], 3}, {NULL}};
const Segment SGI32203[] = {{&S25627[103], 7}, {NULL}};
const Segment SGI32204[] = {{&S25627[159], 8}, {NULL}};
const Segment SGI32205[] = {{&S27773[44], 12}, {NULL}};
const Segment SGI32206[] = {{&S27773[44], 35}, {NULL}};
const Segment SGI32207[] = {{&S27773[30], 6}, {NULL}};
const Segment SGI32208[] = {{&S27773[127], 3}, {NULL}};
const Segment SGI32209[] = {{&S27773[93], 5}, {NULL}};
const Segment SGI32210[] = {{&S25628[44], 20}, {NULL}};
const Segment SGI32211[] = {{&S25628[44], 45}, {NULL}};
const Segment SGI32212[] = {{&S25628[30], 6}, {NULL}};
const Segment SGI32213[] = {{&S25628[103], 6}, {NULL}};
const Segment SGI32214[] = {{&S27774[44], 12}, {NULL}};
const Segment SGI32215[] = {{&S27774[44], 35}, {NULL}};
const Segment SGI32216[] = {{&S27774[30], 6}, {NULL}};
const Segment SGI32217[] = {{&S27774[127], 3}, {NULL}};
const Segment SGI32218[] = {{&S27774[93], 5}, {NULL}};
const Segment SGI32219[] = {{&S25629[44], 13}, {NULL}};
const Segment SGI32220[] = {{&S25629[44], 38}, {NULL}};
const Segment SGI32221[] = {{&S25629[30], 6}, {NULL}};
const Segment SGI32222[] = {{&S25629[130], 3}, {NULL}};
const Segment SGI32223[] = {{&S25629[96], 5}, {NULL}};
const Segment SGI32224[] = {{&S25630[44], 7}, {NULL}};
const Segment SGI32225[] = {{&S25630[44], 32}, {NULL}};
const Segment SGI32226[] = {{&S25630[30], 6}, {NULL}};
const Segment SGI32227[] = {{&S25630[90], 6}, {NULL}};
const Segment SGI32228[] = {{&S27777[44], 17}, {NULL}};
const Segment SGI32229[] = {{&S27777[44], 34}, {NULL}};
const Segment SGI32230[] = {{&S27777[30], 6}, {NULL}};
const Segment SGI32231[] = {{&S27777[126], 3}, {NULL}};
const Segment SGI32232[] = {{&S27777[92], 5}, {NULL}};
const Segment SGI32233[] = {{&S25631[42], 7}, {NULL}};
const Segment SGI32234[] = {{&S25631[30], 6}, {NULL}};
const Segment SGI32235[] = {{&S25631[122], 3}, {NULL}};
const Segment SGI32236[] = {{&S25631[88], 5}, {NULL}};
const Segment SGI32237[] = {{&S25632[44], 7}, {NULL}};
const Segment SGI32238[] = {{&S25632[44], 32}, {NULL}};
const Segment SGI32239[] = {{&S25632[30], 6}, {NULL}};
const Segment SGI32240[] = {{&S25632[124], 3}, {NULL}};
const Segment SGI32241[] = {{&S25632[90], 5}, {NULL}};
const Segment SGI32242[] = {{&S27780[44], 12}, {NULL}};
const Segment SGI32243[] = {{&S27780[44], 23}, {NULL}};
const Segment SGI32244[] = {{&S27780[30], 6}, {NULL}};
const Segment SGI32245[] = {{&S27780[84], 4}, {NULL}};
const Segment SGI32246[] = {{&S27780[84], 3}, {NULL}};
const Segment SGI32247[] = {{&S27785[44], 18}, {NULL}};
const Segment SGI32248[] = {{&S27785[44], 43}, {NULL}};
const Segment SGI32249[] = {{&S27785[30], 6}, {NULL}};
const Segment SGI32250[] = {{&S27785[135], 3}, {NULL}};
const Segment SGI32251[] = {{&S27785[101], 5}, {NULL}};
const Segment SGI32252[] = {{&S27788[44], 12}, {NULL}};
const Segment SGI32253[] = {{&S27788[44], 23}, {NULL}};
const Segment SGI32254[] = {{&S27788[30], 6}, {NULL}};
const Segment SGI32255[] = {{&S27788[84], 6}, {NULL}};
const Segment SGI32256[] = {{&S27797[44], 18}, {NULL}};
const Segment SGI32257[] = {{&S27797[44], 43}, {NULL}};
const Segment SGI32258[] = {{&S27797[30], 6}, {NULL}};
const Segment SGI32259[] = {{&S27797[135], 3}, {NULL}};
const Segment SGI32260[] = {{&S27797[101], 5}, {NULL}};
const Segment SGI32261[] = {{&S27915[44], 18}, {NULL}};
const Segment SGI32262[] = {{&S27915[44], 31}, {NULL}};
const Segment SGI32263[] = {{&S27915[30], 6}, {NULL}};
const Segment SGI32264[] = {{&S27915[123], 3}, {NULL}};
const Segment SGI32265[] = {{&S27915[89], 5}, {NULL}};
const Segment SGI32266[] = {{&S27916[44], 18}, {NULL}};
const Segment SGI32267[] = {{&S27916[44], 36}, {NULL}};
const Segment SGI32268[] = {{&S27916[30], 6}, {NULL}};
const Segment SGI32269[] = {{&S27916[128], 3}, {NULL}};
const Segment SGI32270[] = {{&S27916[94], 5}, {NULL}};
const Segment SGI32271[] = {{&S27917[44], 18}, {NULL}};
const Segment SGI32272[] = {{&S27917[44], 43}, {NULL}};
const Segment SGI32273[] = {{&S27917[30], 6}, {NULL}};
const Segment SGI32274[] = {{&S27917[135], 3}, {NULL}};
const Segment SGI32275[] = {{&S27917[101], 5}, {NULL}};
const Segment SGI32276[] = {{&S27918[44], 19}, {NULL}};
const Segment SGI32277[] = {{&S27918[44], 37}, {NULL}};
const Segment SGI32278[] = {{&S27918[30], 6}, {NULL}};
const Segment SGI32279[] = {{&S27918[129], 3}, {NULL}};
const Segment SGI32280[] = {{&S27918[95], 5}, {NULL}};
const Segment SGI32281[] = {{&S27919[44], 18}, {NULL}};
const Segment SGI32282[] = {{&S27919[44], 36}, {NULL}};
const Segment SGI32283[] = {{&S27919[30], 6}, {NULL}};
const Segment SGI32284[] = {{&S27919[128], 3}, {NULL}};
const Segment SGI32285[] = {{&S27919[94], 5}, {NULL}};
const Segment SGI32286[] = {{&S27920[44], 20}, {NULL}};
const Segment SGI32287[] = {{&S27920[44], 38}, {NULL}};
const Segment SGI32288[] = {{&S27920[30], 6}, {NULL}};
const Segment SGI32289[] = {{&S27920[130], 3}, {NULL}};
const Segment SGI32290[] = {{&S27920[96], 5}, {NULL}};
const Segment SGI32291[] = {{&S27921[44], 18}, {NULL}};
const Segment SGI32292[] = {{&S27921[44], 35}, {NULL}};
const Segment SGI32293[] = {{&S27921[30], 6}, {NULL}};
const Segment SGI32294[] = {{&S27921[68], 3}, {NULL}};
const Segment SGI32295[] = {{&S27921[93], 5}, {NULL}};
const Segment SGI32296[] = {{&S27922[44], 6}, {NULL}};
const Segment SGI32297[] = {{&S27922[44], 24}, {NULL}};
const Segment SGI32298[] = {{&S27922[30], 6}, {NULL}};
const Segment SGI32299[] = {{&S27922[116], 3}, {NULL}};
const Segment SGI32300[] = {{&S27922[82], 5}, {NULL}};
const Segment SGI32301[] = {{&S27923[44], 18}, {NULL}};
const Segment SGI32302[] = {{&S27923[44], 36}, {NULL}};
const Segment SGI32303[] = {{&S27923[30], 6}, {NULL}};
const Segment SGI32304[] = {{&S27923[128], 3}, {NULL}};
const Segment SGI32305[] = {{&S27923[94], 5}, {NULL}};
const Segment SGI32306[] = {{&S27924[44], 18}, {NULL}};
const Segment SGI32307[] = {{&S27924[44], 36}, {NULL}};
const Segment SGI32308[] = {{&S27924[30], 6}, {NULL}};
const Segment SGI32309[] = {{&S27924[128], 3}, {NULL}};
const Segment SGI32310[] = {{&S27924[94], 5}, {NULL}};
const Segment SGI32311[] = {{&S27925[44], 18}, {NULL}};
const Segment SGI32312[] = {{&S27925[44], 36}, {NULL}};
const Segment SGI32313[] = {{&S27925[30], 6}, {NULL}};
const Segment SGI32314[] = {{&S27925[128], 3}, {NULL}};
const Segment SGI32315[] = {{&S27925[94], 5}, {NULL}};
const Segment SGI32316[] = {{&S27926[44], 18}, {NULL}};
const Segment SGI32317[] = {{&S27926[44], 36}, {NULL}};
const Segment SGI32318[] = {{&S27926[30], 6}, {NULL}};
const Segment SGI32319[] = {{&S27926[128], 3}, {NULL}};
const Segment SGI32320[] = {{&S27926[94], 5}, {NULL}};
const Segment SGI32321[] = {{&S27927[40], 5}, {NULL}};
const Segment SGI32322[] = {{&S27927[26], 6}, {NULL}};
const Segment SGI32323[] = {{&S27927[82], 3}, {NULL}};
const Segment SGI32324[] = {{&S27927[59], 6}, {NULL}};
const Segment SGI32325[] = {{&S27928[39], 4}, {NULL}};
const Segment SGI32326[] = {{&S27928[88], 3}, {NULL}};
const Segment SGI32327[] = {{&S27928[65], 6}, {NULL}};
const Segment SGI32328[] = {{&S27930[38], 6}, {NULL}};
const Segment SGI32329[] = {{&S27930[31], 6}, {NULL}};
const Segment SGI32330[] = {{&S27931[44], 6}, {NULL}};
const Segment SGI32331[] = {{&S27931[44], 24}, {NULL}};
const Segment SGI32332[] = {{&S27931[30], 6}, {NULL}};
const Segment SGI32333[] = {{&S27931[85], 4}, {NULL}};
const Segment SGI32334[] = {{&S27931[85], 3}, {NULL}};
const Segment SGI32335[] = {{&S27936[44], 6}, {NULL}};
const Segment SGI32336[] = {{&S27936[44], 24}, {NULL}};
const Segment SGI32337[] = {{&S27936[30], 6}, {NULL}};
const Segment SGI32338[] = {{&S27936[85], 6}, {NULL}};
const Segment SGI32339[] = {{&S27939[42], 18}, {NULL}};
const Segment SGI32340[] = {{&S27939[30], 6}, {NULL}};
const Segment SGI32341[] = {{&S27939[126], 3}, {NULL}};
const Segment SGI32342[] = {{&S27939[92], 5}, {NULL}};
const Segment SGI32343[] = {{&S27948[44], 12}, {NULL}};
const Segment SGI32344[] = {{&S27948[44], 29}, {NULL}};
const Segment SGI32345[] = {{&S27948[30], 6}, {NULL}};
const Segment SGI32346[] = {{&S27948[121], 3}, {NULL}};
const Segment SGI32347[] = {{&S27948[87], 5}, {NULL}};
const Segment SGI32348[] = {{&S27951[44], 10}, {NULL}};
const Segment SGI32349[] = {{&S27951[44], 33}, {NULL}};
const Segment SGI32350[] = {{&S27951[30], 6}, {NULL}};
const Segment SGI32351[] = {{&S27951[125], 3}, {NULL}};
const Segment SGI32352[] = {{&S27951[91], 5}, {NULL}};
const Segment SGI32353[] = {{&S27952[43], 10}, {NULL}};
const Segment SGI32354[] = {{&S27952[43], 33}, {NULL}};
const Segment SGI32355[] = {{&S27952[124], 3}, {NULL}};
const Segment SGI32356[] = {{&S27952[90], 5}, {NULL}};
const Segment SGI32357[] = {{&S27954[44], 9}, {NULL}};
const Segment SGI32358[] = {{&S27954[44], 32}, {NULL}};
const Segment SGI32359[] = {{&S27954[30], 6}, {NULL}};
const Segment SGI32360[] = {{&S27954[124], 3}, {NULL}};
const Segment SGI32361[] = {{&S27954[90], 5}, {NULL}};
const Segment SGI32362[] = {{&S27955[44], 9}, {NULL}};
const Segment SGI32363[] = {{&S27955[44], 32}, {NULL}};
const Segment SGI32364[] = {{&S27955[30], 6}, {NULL}};
const Segment SGI32365[] = {{&S27955[124], 3}, {NULL}};
const Segment SGI32366[] = {{&S27955[90], 5}, {NULL}};
const Segment SGI32367[] = {{&S27958[44], 10}, {NULL}};
const Segment SGI32368[] = {{&S27958[44], 33}, {NULL}};
const Segment SGI32369[] = {{&S27958[30], 6}, {NULL}};
const Segment SGI32370[] = {{&S27958[125], 3}, {NULL}};
const Segment SGI32371[] = {{&S27958[91], 5}, {NULL}};
const Segment SGI32372[] = {{&S27959[44], 10}, {NULL}};
const Segment SGI32373[] = {{&S27959[44], 24}, {NULL}};
const Segment SGI32374[] = {{&S27959[30], 6}, {NULL}};
const Segment SGI32375[] = {{&S27959[116], 3}, {NULL}};
const Segment SGI32376[] = {{&S27959[82], 5}, {NULL}};
const Segment SGI32377[] = {{&S27960[44], 10}, {NULL}};
const Segment SGI32378[] = {{&S27960[44], 33}, {NULL}};
const Segment SGI32379[] = {{&S27960[30], 6}, {NULL}};
const Segment SGI32380[] = {{&S27960[125], 3}, {NULL}};
const Segment SGI32381[] = {{&S27960[91], 5}, {NULL}};
const Segment SGI32382[] = {{&S27962[44], 10}, {NULL}};
const Segment SGI32383[] = {{&S27962[44], 32}, {NULL}};
const Segment SGI32384[] = {{&S27962[30], 6}, {NULL}};
const Segment SGI32385[] = {{&S27962[124], 3}, {NULL}};
const Segment SGI32386[] = {{&S27962[90], 5}, {NULL}};
const Segment SGI32387[] = {{&S27964[44], 10}, {NULL}};
const Segment SGI32388[] = {{&S27964[44], 32}, {NULL}};
const Segment SGI32389[] = {{&S27964[30], 6}, {NULL}};
const Segment SGI32390[] = {{&S27964[124], 3}, {NULL}};
const Segment SGI32391[] = {{&S27964[90], 5}, {NULL}};
const Segment SGI32392[] = {{&S27965[44], 10}, {NULL}};
const Segment SGI32393[] = {{&S27965[44], 33}, {NULL}};
const Segment SGI32394[] = {{&S27965[30], 6}, {NULL}};
const Segment SGI32395[] = {{&S27965[125], 3}, {NULL}};
const Segment SGI32396[] = {{&S27965[91], 5}, {NULL}};
const Segment SGI32397[] = {{&S27969[44], 10}, {NULL}};
const Segment SGI32398[] = {{&S27969[44], 32}, {NULL}};
const Segment SGI32399[] = {{&S27969[30], 6}, {NULL}};
const Segment SGI32400[] = {{&S27969[124], 3}, {NULL}};
const Segment SGI32401[] = {{&S27969[90], 5}, {NULL}};
const Segment SGI32402[] = {{&S27975[38], 10}, {NULL}};
const Segment SGI32403[] = {{&S27975[31], 6}, {NULL}};
const Segment SGI32404[] = {{&S28049[43], 12}, {NULL}};
const Segment SGI32405[] = {{&S28049[43], 23}, {NULL}};
const Segment SGI32406[] = {{&S28049[30], 5}, {NULL}};
const Segment SGI32407[] = {{&S28049[114], 3}, {NULL}};
const Segment SGI32408[] = {{&S28049[80], 5}, {NULL}};
const Segment SGI32409[] = {{&S28050[44], 12}, {NULL}};
const Segment SGI32410[] = {{&S28050[44], 23}, {NULL}};
const Segment SGI32411[] = {{&S28050[30], 6}, {NULL}};
const Segment SGI32412[] = {{&S28050[115], 3}, {NULL}};
const Segment SGI32413[] = {{&S28050[81], 5}, {NULL}};
const Segment SGI32414[] = {{&S28051[44], 11}, {NULL}};
const Segment SGI32415[] = {{&S28051[44], 22}, {NULL}};
const Segment SGI32416[] = {{&S28051[30], 6}, {NULL}};
const Segment SGI32417[] = {{&S28051[114], 3}, {NULL}};
const Segment SGI32418[] = {{&S28051[80], 5}, {NULL}};
const Segment SGI32419[] = {{&S28053[42], 12}, {NULL}};
const Segment SGI32420[] = {{&S28053[30], 6}, {NULL}};
const Segment SGI32421[] = {{&S28053[113], 3}, {NULL}};
const Segment SGI32422[] = {{&S28053[79], 5}, {NULL}};
const Segment SGI32423[] = {{&S28055[44], 11}, {NULL}};
const Segment SGI32424[] = {{&S28055[44], 22}, {NULL}};
const Segment SGI32425[] = {{&S28055[30], 6}, {NULL}};
const Segment SGI32426[] = {{&S28055[114], 3}, {NULL}};
const Segment SGI32427[] = {{&S28055[80], 5}, {NULL}};
const Segment SGI32428[] = {{&S26743[44], 19}, {NULL}};
const Segment SGI32429[] = {{&S26743[44], 38}, {NULL}};
const Segment SGI32430[] = {{&S26743[30], 6}, {NULL}};
const Segment SGI32431[] = {{&S26743[69], 3}, {NULL}};
const Segment SGI32432[] = {{&S26743[96], 5}, {NULL}};
const Segment SGI32433[] = {{&S28056[43], 12}, {NULL}};
const Segment SGI32434[] = {{&S28056[43], 23}, {NULL}};
const Segment SGI32435[] = {{&S28056[114], 3}, {NULL}};
const Segment SGI32436[] = {{&S28056[80], 5}, {NULL}};
const Segment SGI32437[] = {{&S28057[44], 12}, {NULL}};
const Segment SGI32438[] = {{&S28057[44], 23}, {NULL}};
const Segment SGI32439[] = {{&S28057[30], 6}, {NULL}};
const Segment SGI32440[] = {{&S26745[44], 19}, {NULL}};
const Segment SGI32441[] = {{&S26745[44], 38}, {NULL}};
const Segment SGI32442[] = {{&S26745[30], 6}, {NULL}};
const Segment SGI32443[] = {{&S28057[115], 3}, {NULL}};
const Segment SGI32444[] = {{&S28057[81], 5}, {NULL}};
const Segment SGI32445[] = {{&S26745[69], 3}, {NULL}};
const Segment SGI32446[] = {{&S26745[96], 5}, {NULL}};
const Segment SGI32447[] = {{&S28058[44], 9}, {NULL}};
const Segment SGI32448[] = {{&S28058[44], 20}, {NULL}};
const Segment SGI32449[] = {{&S28058[30], 6}, {NULL}};
const Segment SGI32450[] = {{&S26746[44], 19}, {NULL}};
const Segment SGI32451[] = {{&S28058[112], 3}, {NULL}};
const Segment SGI32452[] = {{&S26746[44], 38}, {NULL}};
const Segment SGI32453[] = {{&S26746[30], 6}, {NULL}};
const Segment SGI32454[] = {{&S28058[78], 5}, {NULL}};
const Segment SGI32455[] = {{&S26746[69], 3}, {NULL}};
const Segment SGI32456[] = {{&S26746[96], 5}, {NULL}};
const Segment SGI32457[] = {{&S26749[44], 19}, {NULL}};
const Segment SGI32458[] = {{&S26749[44], 38}, {NULL}};
const Segment SGI32459[] = {{&S26749[30], 6}, {NULL}};
const Segment SGI32460[] = {{&S26749[69], 3}, {NULL}};
const Segment SGI32461[] = {{&S26749[96], 5}, {NULL}};
const Segment SGI32462[] = {{&S26750[44], 19}, {NULL}};
const Segment SGI32463[] = {{&S26750[44], 36}, {NULL}};
const Segment SGI32464[] = {{&S26750[30], 6}, {NULL}};
const Segment SGI32465[] = {{&S26750[69], 3}, {NULL}};
const Segment SGI32466[] = {{&S26750[94], 5}, {NULL}};
const Segment SGI32467[] = {{&S26751[44], 19}, {NULL}};
const Segment SGI32468[] = {{&S26751[44], 37}, {NULL}};
const Segment SGI32469[] = {{&S26751[30], 6}, {NULL}};
const Segment SGI32470[] = {{&S26751[69], 3}, {NULL}};
const Segment SGI32471[] = {{&S28068[38], 9}, {NULL}};
const Segment SGI32472[] = {{&S26751[95], 5}, {NULL}};
const Segment SGI32473[] = {{&S28068[31], 6}, {NULL}};
const Segment SGI32474[] = {{&S28075[44], 9}, {NULL}};
const Segment SGI32475[] = {{&S28075[44], 20}, {NULL}};
const Segment SGI32476[] = {{&S28075[30], 6}, {NULL}};
const Segment SGI32477[] = {{&S28075[81], 4}, {NULL}};
const Segment SGI32478[] = {{&S28075[81], 3}, {NULL}};
const Segment SGI32479[] = {{&S26762[44], 19}, {NULL}};
const Segment SGI32480[] = {{&S26762[44], 38}, {NULL}};
const Segment SGI32481[] = {{&S26762[30], 6}, {NULL}};
const Segment SGI32482[] = {{&S26762[69], 3}, {NULL}};
const Segment SGI32483[] = {{&S26762[96], 5}, {NULL}};
const Segment SGI32484[] = {{&S28081[44], 9}, {NULL}};
const Segment SGI32485[] = {{&S28081[44], 20}, {NULL}};
const Segment SGI32486[] = {{&S28081[30], 6}, {NULL}};
const Segment SGI32487[] = {{&S26763[44], 19}, {NULL}};
const Segment SGI32488[] = {{&S26763[44], 37}, {NULL}};
const Segment SGI32489[] = {{&S28081[81], 6}, {NULL}};
const Segment SGI32490[] = {{&S26763[30], 6}, {NULL}};
const Segment SGI32491[] = {{&S26763[69], 3}, {NULL}};
const Segment SGI32492[] = {{&S26763[95], 5}, {NULL}};
const Segment SGI32493[] = {{&S26764[44], 7}, {NULL}};
const Segment SGI32494[] = {{&S26764[44], 25}, {NULL}};
const Segment SGI32495[] = {{&S26764[30], 6}, {NULL}};
const Segment SGI32496[] = {{&S26764[57], 3}, {NULL}};
const Segment SGI32497[] = {{&S26764[83], 5}, {NULL}};
const Segment SGI32498[] = {{&S26765[44], 19}, {NULL}};
const Segment SGI32499[] = {{&S26765[44], 37}, {NULL}};
const Segment SGI32500[] = {{&S26765[30], 6}, {NULL}};
const Segment SGI32501[] = {{&S26765[69], 3}, {NULL}};
const Segment SGI32502[] = {{&S26765[95], 5}, {NULL}};
const Segment SGI32503[] = {{&S26767[44], 19}, {NULL}};
const Segment SGI32504[] = {{&S26767[44], 37}, {NULL}};
const Segment SGI32505[] = {{&S26767[30], 6}, {NULL}};
const Segment SGI32506[] = {{&S26767[69], 3}, {NULL}};
const Segment SGI32507[] = {{&S26767[95], 5}, {NULL}};
const Segment SGI32508[] = {{&S26768[44], 19}, {NULL}};
const Segment SGI32509[] = {{&S26768[44], 37}, {NULL}};
const Segment SGI32510[] = {{&S26768[30], 6}, {NULL}};
const Segment SGI32511[] = {{&S26768[69], 3}, {NULL}};
const Segment SGI32512[] = {{&S26768[95], 5}, {NULL}};
const Segment SGI32513[] = {{&S26770[38], 7}, {NULL}};
const Segment SGI32514[] = {{&S26770[31], 6}, {NULL}};
const Segment SGI32515[] = {{&S26771[44], 19}, {NULL}};
const Segment SGI32516[] = {{&S26771[44], 37}, {NULL}};
const Segment SGI32517[] = {{&S26771[30], 6}, {NULL}};
const Segment SGI32518[] = {{&S26771[69], 3}, {NULL}};
const Segment SGI32519[] = {{&S26771[95], 5}, {NULL}};
const Segment SGI32520[] = {{&S26772[42], 19}, {NULL}};
const Segment SGI32521[] = {{&S26772[30], 6}, {NULL}};
const Segment SGI32522[] = {{&S26772[67], 3}, {NULL}};
const Segment SGI32523[] = {{&S26772[93], 5}, {NULL}};
const Segment SGI32524[] = {{&S26773[44], 7}, {NULL}};
const Segment SGI32525[] = {{&S26773[44], 25}, {NULL}};
const Segment SGI32526[] = {{&S26773[30], 6}, {NULL}};
const Segment SGI32527[] = {{&S26773[57], 3}, {NULL}};
const Segment SGI32528[] = {{&S26773[83], 5}, {NULL}};
const Segment SGI32529[] = {{&S26774[44], 7}, {NULL}};
const Segment SGI32530[] = {{&S26774[44], 25}, {NULL}};
const Segment SGI32531[] = {{&S26774[30], 6}, {NULL}};
const Segment SGI32532[] = {{&S26774[57], 3}, {NULL}};
const Segment SGI32533[] = {{&S26774[83], 5}, {NULL}};
const Segment SGI32534[] = {{&S26777[44], 7}, {NULL}};
const Segment SGI32535[] = {{&S26777[44], 25}, {NULL}};
const Segment SGI32536[] = {{&S26777[30], 6}, {NULL}};
const Segment SGI32537[] = {{&S26777[57], 3}, {NULL}};
const Segment SGI32538[] = {{&S26777[83], 5}, {NULL}};
const Segment SGI32539[] = {{&S26779[44], 7}, {NULL}};
const Segment SGI32540[] = {{&S26779[44], 25}, {NULL}};
const Segment SGI32541[] = {{&S26779[30], 6}, {NULL}};
const Segment SGI32542[] = {{&S26779[86], 6}, {NULL}};
const Segment SGI32543[] = {{&S26780[44], 7}, {NULL}};
const Segment SGI32544[] = {{&S26780[44], 25}, {NULL}};
const Segment SGI32545[] = {{&S26780[30], 6}, {NULL}};
const Segment SGI32546[] = {{&S26780[86], 6}, {NULL}};
const Segment SGI32547[] = {{&S26781[44], 7}, {NULL}};
const Segment SGI32548[] = {{&S26781[44], 25}, {NULL}};
const Segment SGI32549[] = {{&S26781[30], 6}, {NULL}};
const Segment SGI32550[] = {{&S26781[86], 4}, {NULL}};
const Segment SGI32551[] = {{&S26781[86], 3}, {NULL}};
const Segment SGI32552[] = {{&S26784[44], 7}, {NULL}};
const Segment SGI32553[] = {{&S26784[44], 25}, {NULL}};
const Segment SGI32554[] = {{&S26784[30], 6}, {NULL}};
const Segment SGI32555[] = {{&S26784[57], 3}, {NULL}};
const Segment SGI32556[] = {{&S26784[83], 5}, {NULL}};
const Segment SGI32557[] = {{&S26896[44], 8}, {NULL}};
const Segment SGI32558[] = {{&S26896[44], 17}, {NULL}};
const Segment SGI32559[] = {{&S26896[30], 6}, {NULL}};
const Segment SGI32560[] = {{&S26896[109], 3}, {NULL}};
const Segment SGI32561[] = {{&S26896[75], 5}, {NULL}};
const Segment SGI32562[] = {{&S26898[44], 8}, {NULL}};
const Segment SGI32563[] = {{&S26898[44], 29}, {NULL}};
const Segment SGI32564[] = {{&S26898[30], 6}, {NULL}};
const Segment SGI32565[] = {{&S26898[121], 3}, {NULL}};
const Segment SGI32566[] = {{&S26898[87], 5}, {NULL}};
const Segment SGI32567[] = {{&S26900[38], 8}, {NULL}};
const Segment SGI32568[] = {{&S26900[31], 6}, {NULL}};
const Segment SGI32569[] = {{&S26901[43], 8}, {NULL}};
const Segment SGI32570[] = {{&S26901[43], 29}, {NULL}};
const Segment SGI32571[] = {{&S26901[30], 5}, {NULL}};
const Segment SGI32572[] = {{&S26901[120], 3}, {NULL}};
const Segment SGI32573[] = {{&S26901[86], 5}, {NULL}};
const Segment SGI32574[] = {{&S26902[44], 8}, {NULL}};
const Segment SGI32575[] = {{&S26902[44], 29}, {NULL}};
const Segment SGI32576[] = {{&S26902[30], 6}, {NULL}};
const Segment SGI32577[] = {{&S26902[121], 3}, {NULL}};
const Segment SGI32578[] = {{&S26902[87], 5}, {NULL}};
const Segment SGI32579[] = {{&S26903[44], 7}, {NULL}};
const Segment SGI32580[] = {{&S26903[44], 28}, {NULL}};
const Segment SGI32581[] = {{&S26903[30], 6}, {NULL}};
const Segment SGI32582[] = {{&S26903[120], 3}, {NULL}};
const Segment SGI32583[] = {{&S26903[86], 5}, {NULL}};
const Segment SGI32584[] = {{&S37461[44], 10}, {NULL}};
const Segment SGI32585[] = {{&S37461[44], 21}, {NULL}};
const Segment SGI32586[] = {{&S37461[30], 6}, {NULL}};
const Segment SGI32587[] = {{&S26904[43], 8}, {NULL}};
const Segment SGI32588[] = {{&S26904[43], 29}, {NULL}};
const Segment SGI32589[] = {{&S37461[113], 3}, {NULL}};
const Segment SGI32590[] = {{&S37461[79], 5}, {NULL}};
const Segment SGI32591[] = {{&S26904[120], 3}, {NULL}};
const Segment SGI32592[] = {{&S26904[86], 5}, {NULL}};
const Segment SGI32593[] = {{&S37462[43], 10}, {NULL}};
const Segment SGI32594[] = {{&S37462[43], 21}, {NULL}};
const Segment SGI32595[] = {{&S37462[30], 5}, {NULL}};
const Segment SGI32596[] = {{&S37462[112], 3}, {NULL}};
const Segment SGI32597[] = {{&S26905[44], 7}, {NULL}};
const Segment SGI32598[] = {{&S26905[44], 28}, {NULL}};
const Segment SGI32599[] = {{&S37462[78], 5}, {NULL}};
const Segment SGI32600[] = {{&S26905[30], 6}, {NULL}};
const Segment SGI32601[] = {{&S26905[120], 3}, {NULL}};
const Segment SGI32602[] = {{&S26905[86], 5}, {NULL}};
const Segment SGI32603[] = {{&S37463[44], 9}, {NULL}};
const Segment SGI32604[] = {{&S37463[44], 20}, {NULL}};
const Segment SGI32605[] = {{&S37463[30], 6}, {NULL}};
const Segment SGI32606[] = {{&S37463[112], 3}, {NULL}};
const Segment SGI32607[] = {{&S37463[78], 5}, {NULL}};
const Segment SGI32608[] = {{&S26906[44], 8}, {NULL}};
const Segment SGI32609[] = {{&S26906[44], 20}, {NULL}};
const Segment SGI32610[] = {{&S26906[30], 6}, {NULL}};
const Segment SGI32611[] = {{&S26906[112], 3}, {NULL}};
const Segment SGI32612[] = {{&S26906[78], 5}, {NULL}};
const Segment SGI32613[] = {{&S37464[43], 10}, {NULL}};
const Segment SGI32614[] = {{&S37464[43], 21}, {NULL}};
const Segment SGI32615[] = {{&S37464[112], 3}, {NULL}};
const Segment SGI32616[] = {{&S37464[78], 5}, {NULL}};
const Segment SGI32617[] = {{&S37465[44], 11}, {NULL}};
const Segment SGI32618[] = {{&S37465[44], 22}, {NULL}};
const Segment SGI32619[] = {{&S37465[30], 6}, {NULL}};
const Segment SGI32620[] = {{&S37465[114], 3}, {NULL}};
const Segment SGI32621[] = {{&S37465[80], 5}, {NULL}};
const Segment SGI32622[] = {{&S37466[44], 9}, {NULL}};
const Segment SGI32623[] = {{&S37466[44], 20}, {NULL}};
const Segment SGI32624[] = {{&S37466[30], 6}, {NULL}};
const Segment SGI32625[] = {{&S37466[112], 3}, {NULL}};
const Segment SGI32626[] = {{&S37466[78], 5}, {NULL}};
const Segment SGI32627[] = {{&S26911[44], 8}, {NULL}};
const Segment SGI32628[] = {{&S26911[44], 29}, {NULL}};
const Segment SGI32629[] = {{&S26911[30], 6}, {NULL}};
const Segment SGI32630[] = {{&S37467[44], 10}, {NULL}};
const Segment SGI32631[] = {{&S37467[44], 21}, {NULL}};
const Segment SGI32632[] = {{&S26911[123], 3}, {NULL}};
const Segment SGI32633[] = {{&S37467[30], 6}, {NULL}};
const Segment SGI32634[] = {{&S26911[87], 7}, {NULL}};
const Segment SGI32635[] = {{&S26911[143], 8}, {NULL}};
const Segment SGI32636[] = {{&S37467[113], 3}, {NULL}};
const Segment SGI32637[] = {{&S37467[79], 5}, {NULL}};
const Segment SGI32638[] = {{&S37468[38], 10}, {NULL}};
const Segment SGI32639[] = {{&S37468[31], 6}, {NULL}};
const Segment SGI32640[] = {{&S37469[38], 11}, {NULL}};
const Segment SGI32641[] = {{&S26914[44], 8}, {NULL}};
const Segment SGI32642[] = {{&S37469[31], 6}, {NULL}};
const Segment SGI32643[] = {{&S26914[44], 29}, {NULL}};
const Segment SGI32644[] = {{&S26914[30], 6}, {NULL}};
const Segment SGI32645[] = {{&S26914[121], 3}, {NULL}};
const Segment SGI32646[] = {{&S26914[87], 5}, {NULL}};
const Segment SGI32647[] = {{&S26915[42], 8}, {NULL}};
const Segment SGI32648[] = {{&S26915[30], 6}, {NULL}};
const Segment SGI32649[] = {{&S26915[119], 3}, {NULL}};
const Segment SGI32650[] = {{&S26915[85], 5}, {NULL}};
const Segment SGI32651[] = {{&S26916[44], 8}, {NULL}};
const Segment SGI32652[] = {{&S26916[44], 29}, {NULL}};
const Segment SGI32653[] = {{&S26916[30], 6}, {NULL}};
const Segment SGI32654[] = {{&S26916[87], 6}, {NULL}};
const Segment SGI32655[] = {{&S26917[44], 8}, {NULL}};
const Segment SGI32656[] = {{&S26917[44], 29}, {NULL}};
const Segment SGI32657[] = {{&S26917[30], 6}, {NULL}};
const Segment SGI32658[] = {{&S26917[121], 3}, {NULL}};
const Segment SGI32659[] = {{&S26917[87], 5}, {NULL}};
const Segment SGI32660[] = {{&S26948[44], 19}, {NULL}};
const Segment SGI32661[] = {{&S26948[44], 37}, {NULL}};
const Segment SGI32662[] = {{&S26948[30], 6}, {NULL}};
const Segment SGI32663[] = {{&S26948[69], 3}, {NULL}};
const Segment SGI32664[] = {{&S26948[95], 5}, {NULL}};
const Segment SGI32665[] = {{&S26949[44], 19}, {NULL}};
const Segment SGI32666[] = {{&S26949[44], 38}, {NULL}};
const Segment SGI32667[] = {{&S26949[30], 6}, {NULL}};
const Segment SGI32668[] = {{&S26949[69], 3}, {NULL}};
const Segment SGI32669[] = {{&S26949[96], 5}, {NULL}};
const Segment SGI32670[] = {{&S26950[44], 19}, {NULL}};
const Segment SGI32671[] = {{&S26950[44], 38}, {NULL}};
const Segment SGI32672[] = {{&S26950[30], 6}, {NULL}};
const Segment SGI32673[] = {{&S26950[69], 3}, {NULL}};
const Segment SGI32674[] = {{&S26950[96], 5}, {NULL}};
const Segment SGI32675[] = {{&S26951[44], 7}, {NULL}};
const Segment SGI32676[] = {{&S26951[44], 26}, {NULL}};
const Segment SGI32677[] = {{&S26951[30], 6}, {NULL}};
const Segment SGI32678[] = {{&S26951[57], 3}, {NULL}};
const Segment SGI32679[] = {{&S26951[84], 5}, {NULL}};
const Segment SGI32680[] = {{&S26952[44], 19}, {NULL}};
const Segment SGI32681[] = {{&S26952[44], 38}, {NULL}};
const Segment SGI32682[] = {{&S26952[30], 6}, {NULL}};
const Segment SGI32683[] = {{&S26952[69], 3}, {NULL}};
const Segment SGI32684[] = {{&S26952[96], 5}, {NULL}};
const Segment SGI32685[] = {{&S26953[44], 19}, {NULL}};
const Segment SGI32686[] = {{&S26953[44], 36}, {NULL}};
const Segment SGI32687[] = {{&S26953[30], 6}, {NULL}};
const Segment SGI32688[] = {{&S26953[69], 3}, {NULL}};
const Segment SGI32689[] = {{&S26953[94], 5}, {NULL}};
const Segment SGI32690[] = {{&S26954[44], 7}, {NULL}};
const Segment SGI32691[] = {{&S26954[44], 25}, {NULL}};
const Segment SGI32692[] = {{&S26954[30], 6}, {NULL}};
const Segment SGI32693[] = {{&S26954[57], 3}, {NULL}};
const Segment SGI32694[] = {{&S26954[83], 5}, {NULL}};
const Segment SGI32695[] = {{&S26955[44], 7}, {NULL}};
const Segment SGI32696[] = {{&S26955[44], 25}, {NULL}};
const Segment SGI32697[] = {{&S26955[30], 6}, {NULL}};
const Segment SGI32698[] = {{&S26955[86], 4}, {NULL}};
const Segment SGI32699[] = {{&S26955[86], 3}, {NULL}};
const Segment SGI32700[] = {{&S26956[42], 19}, {NULL}};
const Segment SGI32701[] = {{&S26956[30], 6}, {NULL}};
const Segment SGI32702[] = {{&S26956[67], 3}, {NULL}};
const Segment SGI32703[] = {{&S26956[93], 5}, {NULL}};
const Segment SGI32704[] = {{&S26957[44], 19}, {NULL}};
const Segment SGI32705[] = {{&S26957[44], 37}, {NULL}};
const Segment SGI32706[] = {{&S26957[30], 6}, {NULL}};
const Segment SGI32707[] = {{&S26957[69], 3}, {NULL}};
const Segment SGI32708[] = {{&S26957[95], 5}, {NULL}};
const Segment SGI32709[] = {{&S26958[44], 21}, {NULL}};
const Segment SGI32710[] = {{&S26958[44], 39}, {NULL}};
const Segment SGI32711[] = {{&S26958[30], 6}, {NULL}};
const Segment SGI32712[] = {{&S26958[71], 3}, {NULL}};
const Segment SGI32713[] = {{&S26958[97], 5}, {NULL}};
const Segment SGI32714[] = {{&S26959[44], 7}, {NULL}};
const Segment SGI32715[] = {{&S26959[44], 25}, {NULL}};
const Segment SGI32716[] = {{&S26959[30], 6}, {NULL}};
const Segment SGI32717[] = {{&S26959[57], 3}, {NULL}};
const Segment SGI32718[] = {{&S26959[83], 5}, {NULL}};
const Segment SGI32719[] = {{&S26960[38], 7}, {NULL}};
const Segment SGI32720[] = {{&S26960[31], 6}, {NULL}};
const Segment SGI32721[] = {{&S26961[44], 19}, {NULL}};
const Segment SGI32722[] = {{&S26961[44], 37}, {NULL}};
const Segment SGI32723[] = {{&S26961[30], 6}, {NULL}};
const Segment SGI32724[] = {{&S26961[69], 3}, {NULL}};
const Segment SGI32725[] = {{&S26961[95], 5}, {NULL}};
const Segment SGI32726[] = {{&S26962[44], 7}, {NULL}};
const Segment SGI32727[] = {{&S26962[44], 24}, {NULL}};
const Segment SGI32728[] = {{&S26962[30], 6}, {NULL}};
const Segment SGI32729[] = {{&S26962[57], 3}, {NULL}};
const Segment SGI32730[] = {{&S26962[82], 5}, {NULL}};
const Segment SGI32731[] = {{&S26963[44], 19}, {NULL}};
const Segment SGI32732[] = {{&S26963[44], 37}, {NULL}};
const Segment SGI32733[] = {{&S26963[30], 6}, {NULL}};
const Segment SGI32734[] = {{&S26963[69], 3}, {NULL}};
const Segment SGI32735[] = {{&S26963[95], 5}, {NULL}};
const Segment SGI32736[] = {{&S26964[44], 20}, {NULL}};
const Segment SGI32737[] = {{&S26964[44], 38}, {NULL}};
const Segment SGI32738[] = {{&S26964[30], 6}, {NULL}};
const Segment SGI32739[] = {{&S26964[70], 3}, {NULL}};
const Segment SGI32740[] = {{&S26964[96], 5}, {NULL}};
const Segment SGI32741[] = {{&S26965[44], 20}, {NULL}};
const Segment SGI32742[] = {{&S26965[44], 38}, {NULL}};
const Segment SGI32743[] = {{&S26965[30], 6}, {NULL}};
const Segment SGI32744[] = {{&S26965[70], 3}, {NULL}};
const Segment SGI32745[] = {{&S26965[96], 5}, {NULL}};
const Segment SGI32746[] = {{&S26966[44], 7}, {NULL}};
const Segment SGI32747[] = {{&S26966[44], 25}, {NULL}};
const Segment SGI32748[] = {{&S26966[30], 6}, {NULL}};
const Segment SGI32749[] = {{&S26966[86], 4}, {NULL}};
const Segment SGI32750[] = {{&S26966[86], 3}, {NULL}};
const Segment SGI32751[] = {{&S26967[44], 19}, {NULL}};
const Segment SGI32752[] = {{&S26967[44], 37}, {NULL}};
const Segment SGI32753[] = {{&S26967[30], 6}, {NULL}};
const Segment SGI32754[] = {{&S26967[69], 3}, {NULL}};
const Segment SGI32755[] = {{&S26967[95], 5}, {NULL}};
const Segment SGI32756[] = {{&S26968[44], 20}, {NULL}};
const Segment SGI32757[] = {{&S26968[44], 39}, {NULL}};
const Segment SGI32758[] = {{&S26968[30], 6}, {NULL}};
const Segment SGI32759[] = {{&S26968[70], 3}, {NULL}};
const Segment SGI32760[] = {{&S26968[97], 5}, {NULL}};
const Segment SGI32761[] = {{&S26969[44], 21}, {NULL}};
const Segment SGI32762[] = {{&S26969[44], 39}, {NULL}};
const Segment SGI32763[] = {{&S26969[30], 6}, {NULL}};
const Segment SGI32764[] = {{&S26969[71], 3}, {NULL}};
const Segment SGI32765[] = {{&S26969[97], 5}, {NULL}};
const Segment SGI32766[] = {{&S26970[44], 7}, {NULL}};
const Segment SGI32767[] = {{&S26970[44], 25}, {NULL}};
const Segment SGI32768[] = {{&S26970[30], 6}, {NULL}};
const Segment SGI32769[] = {{&S26970[57], 3}, {NULL}};
const Segment SGI32770[] = {{&S26970[83], 5}, {NULL}};
const Segment SGI32771[] = {{&S26971[44], 20}, {NULL}};
const Segment SGI32772[] = {{&S26971[44], 38}, {NULL}};
const Segment SGI32773[] = {{&S26971[30], 6}, {NULL}};
const Segment SGI32774[] = {{&S26971[70], 3}, {NULL}};
const Segment SGI32775[] = {{&S26971[96], 5}, {NULL}};
const Segment SGI32776[] = {{&S26972[44], 7}, {NULL}};
const Segment SGI32777[] = {{&S26972[44], 25}, {NULL}};
const Segment SGI32778[] = {{&S26972[30], 6}, {NULL}};
const Segment SGI32779[] = {{&S26972[86], 6}, {NULL}};
const Segment SGI32780[] = {{&S26973[44], 7}, {NULL}};
const Segment SGI32781[] = {{&S26973[44], 26}, {NULL}};
const Segment SGI32782[] = {{&S26973[30], 6}, {NULL}};
const Segment SGI32783[] = {{&S26973[87], 6}, {NULL}};
const Segment SGI32784[] = {{&S26034[44], 6}, {NULL}};
const Segment SGI32785[] = {{&S26034[44], 22}, {NULL}};
const Segment SGI32786[] = {{&S26034[30], 6}, {NULL}};
const Segment SGI32787[] = {{&S26034[114], 3}, {NULL}};
const Segment SGI32788[] = {{&S26034[80], 5}, {NULL}};
const Segment SGI32789[] = {{&S26035[44], 13}, {NULL}};
const Segment SGI32790[] = {{&S26035[44], 29}, {NULL}};
const Segment SGI32791[] = {{&S26035[30], 6}, {NULL}};
const Segment SGI32792[] = {{&S26035[121], 3}, {NULL}};
const Segment SGI32793[] = {{&S26035[87], 5}, {NULL}};
const Segment SGI32794[] = {{&S26037[44], 13}, {NULL}};
const Segment SGI32795[] = {{&S26037[44], 29}, {NULL}};
const Segment SGI32796[] = {{&S26037[30], 6}, {NULL}};
const Segment SGI32797[] = {{&S26037[123], 3}, {NULL}};
const Segment SGI32798[] = {{&S26037[87], 7}, {NULL}};
const Segment SGI32799[] = {{&S26037[143], 8}, {NULL}};
const Segment SGI32800[] = {{&S26038[44], 6}, {NULL}};
const Segment SGI32801[] = {{&S26038[44], 22}, {NULL}};
const Segment SGI32802[] = {{&S26038[30], 6}, {NULL}};
const Segment SGI32803[] = {{&S26038[114], 3}, {&S26038[31], 3}, {NULL}};
const Segment SGI32804[] = {{&S26038[80], 5}, {NULL}};
const Segment SGI32805[] = {{&S26041[38], 6}, {NULL}};
const Segment SGI32806[] = {{&S26041[31], 6}, {NULL}};
const Segment SGI32807[] = {{&S26045[38], 13}, {NULL}};
const Segment SGI32808[] = {{&S26045[31], 6}, {NULL}};
const Segment SGI32809[] = {{&S37558[44], 10}, {NULL}};
const Segment SGI32810[] = {{&S37558[44], 21}, {NULL}};
const Segment SGI32811[] = {{&S37558[30], 6}, {NULL}};
const Segment SGI32812[] = {{&S37558[113], 3}, {NULL}};
const Segment SGI32813[] = {{&S37558[79], 5}, {NULL}};
const Segment SGI32814[] = {{&S37561[38], 10}, {NULL}};
const Segment SGI32815[] = {{&S37561[31], 6}, {NULL}};
const Segment SGI32816[] = {{&S37562[44], 10}, {NULL}};
const Segment SGI32817[] = {{&S37562[44], 21}, {NULL}};
const Segment SGI32818[] = {{&S37562[30], 6}, {NULL}};
const Segment SGI32819[] = {{&S37562[82], 4}, {NULL}};
const Segment SGI32820[] = {{&S37562[82], 3}, {NULL}};
const Segment SGI32821[] = {{&S37564[44], 10}, {NULL}};
const Segment SGI32822[] = {{&S37564[44], 21}, {NULL}};
const Segment SGI32823[] = {{&S37564[30], 6}, {NULL}};
const Segment SGI32824[] = {{&S37564[82], 6}, {NULL}};
const Segment SGI32825[] = {{&S37571[46], 10}, {NULL}};
const Segment SGI32826[] = {{&S37571[46], 27}, {NULL}};
const Segment SGI32827[] = {{&S37571[32], 6}, {NULL}};
const Segment SGI32828[] = {{&S37598[44], 9}, {NULL}};
const Segment SGI32829[] = {{&S37598[44], 20}, {NULL}};
const Segment SGI32830[] = {{&S37598[30], 6}, {NULL}};
const Segment SGI32831[] = {{&S37598[112], 3}, {NULL}};
const Segment SGI32832[] = {{&S37598[78], 5}, {NULL}};
const Segment SGI32833[] = {{&S37599[44], 10}, {NULL}};
const Segment SGI32834[] = {{&S37599[44], 21}, {NULL}};
const Segment SGI32835[] = {{&S37599[30], 6}, {NULL}};
const Segment SGI32836[] = {{&S37599[113], 3}, {NULL}};
const Segment SGI32837[] = {{&S37599[79], 5}, {NULL}};
const Segment SGI32838[] = {{&S27140[44], 10}, {NULL}};
const Segment SGI32839[] = {{&S27140[44], 27}, {NULL}};
const Segment SGI32840[] = {{&S27140[30], 6}, {NULL}};
const Segment SGI32841[] = {{&S27140[119], 3}, {NULL}};
const Segment SGI32842[] = {{&S37600[44], 11}, {NULL}};
const Segment SGI32843[] = {{&S27140[85], 5}, {NULL}};
const Segment SGI32844[] = {{&S37600[44], 22}, {NULL}};
const Segment SGI32845[] = {{&S37600[30], 6}, {NULL}};
const Segment SGI32846[] = {{&S37600[114], 3}, {NULL}};
const Segment SGI32847[] = {{&S37600[80], 5}, {NULL}};
const Segment SGI32848[] = {{&S37601[43], 10}, {NULL}};
const Segment SGI32849[] = {{&S37601[43], 21}, {NULL}};
const Segment SGI32850[] = {{&S37601[112], 3}, {NULL}};
const Segment SGI32851[] = {{&S37601[78], 5}, {NULL}};
const Segment SGI32852[] = {{&S27143[44], 10}, {NULL}};
const Segment SGI32853[] = {{&S27143[44], 21}, {NULL}};
const Segment SGI32854[] = {{&S37602[43], 10}, {NULL}};
const Segment SGI32855[] = {{&S27143[30], 6}, {NULL}};
const Segment SGI32856[] = {{&S37602[43], 21}, {NULL}};
const Segment SGI32857[] = {{&S37602[30], 5}, {NULL}};
const Segment SGI32858[] = {{&S27143[113], 3}, {NULL}};
const Segment SGI32859[] = {{&S37602[112], 3}, {NULL}};
const Segment SGI32860[] = {{&S27143[79], 5}, {NULL}};
const Segment SGI32861[] = {{&S37602[78], 5}, {NULL}};
const Segment SGI32862[] = {{&S37603[44], 10}, {NULL}};
const Segment SGI32863[] = {{&S37603[44], 21}, {NULL}};
const Segment SGI32864[] = {{&S37603[30], 6}, {NULL}};
const Segment SGI32865[] = {{&S37603[113], 3}, {NULL}};
const Segment SGI32866[] = {{&S37603[79], 5}, {NULL}};
const Segment SGI32867[] = {{&S37604[44], 9}, {NULL}};
const Segment SGI32868[] = {{&S37604[44], 20}, {NULL}};
const Segment SGI32869[] = {{&S37604[30], 6}, {NULL}};
const Segment SGI32870[] = {{&S37604[112], 3}, {NULL}};
const Segment SGI32871[] = {{&S37604[78], 5}, {NULL}};
const Segment SGI32872[] = {{&S37605[44], 11}, {NULL}};
const Segment SGI32873[] = {{&S37605[44], 22}, {NULL}};
const Segment SGI32874[] = {{&S37605[30], 6}, {NULL}};
const Segment SGI32875[] = {{&S27149[38], 10}, {NULL}};
const Segment SGI32876[] = {{&S27149[31], 6}, {NULL}};
const Segment SGI32877[] = {{&S37605[114], 3}, {NULL}};
const Segment SGI32878[] = {{&S37605[80], 5}, {NULL}};
const Segment SGI32879[] = {{&S37606[44], 10}, {NULL}};
const Segment SGI32880[] = {{&S37606[44], 21}, {NULL}};
const Segment SGI32881[] = {{&S37606[30], 6}, {NULL}};
const Segment SGI32882[] = {{&S37606[115], 3}, {NULL}};
const Segment SGI32883[] = {{&S37606[79], 7}, {NULL}};
const Segment SGI32884[] = {{&S37606[135], 8}, {NULL}};
const Segment SGI32885[] = {{&S37607[38], 10}, {NULL}};
const Segment SGI32886[] = {{&S37607[31], 6}, {NULL}};
const Segment SGI32887[] = {{&S37608[44], 10}, {NULL}};
const Segment SGI32888[] = {{&S37608[44], 21}, {NULL}};
const Segment SGI32889[] = {{&S37608[30], 6}, {NULL}};
const Segment SGI32890[] = {{&S37608[82], 4}, {NULL}};
const Segment SGI32891[] = {{&S37608[82], 3}, {NULL}};
const Segment SGI32892[] = {{&S37609[38], 11}, {NULL}};
const Segment SGI32893[] = {{&S37609[31], 6}, {NULL}};
const Segment SGI32894[] = {{&S27209[44], 10}, {NULL}};
const Segment SGI32895[] = {{&S27209[44], 27}, {NULL}};
const Segment SGI32896[] = {{&S27209[30], 6}, {NULL}};
const Segment SGI32897[] = {{&S27209[119], 3}, {NULL}};
const Segment SGI32898[] = {{&S27209[85], 5}, {NULL}};
const Segment SGI32899[] = {{&S28525[44], 15}, {NULL}};
const Segment SGI32900[] = {{&S28525[44], 26}, {NULL}};
const Segment SGI32901[] = {{&S28525[30], 6}, {NULL}};
const Segment SGI32902[] = {{&S28525[118], 3}, {NULL}};
const Segment SGI32903[] = {{&S28525[84], 5}, {NULL}};
const Segment SGI32904[] = {{&S28526[44], 12}, {NULL}};
const Segment SGI32905[] = {{&S28526[44], 23}, {NULL}};
const Segment SGI32906[] = {{&S28526[30], 6}, {NULL}};
const Segment SGI32907[] = {{&S28526[115], 3}, {NULL}};
const Segment SGI32908[] = {{&S28526[81], 5}, {NULL}};
const Segment SGI32909[] = {{&S28527[43], 15}, {NULL}};
const Segment SGI32910[] = {{&S28527[43], 26}, {NULL}};
const Segment SGI32911[] = {{&S28527[117], 3}, {NULL}};
const Segment SGI32912[] = {{&S28527[83], 5}, {NULL}};
const Segment SGI32913[] = {{&S28528[43], 15}, {NULL}};
const Segment SGI32914[] = {{&S28528[43], 26}, {NULL}};
const Segment SGI32915[] = {{&S28528[30], 5}, {NULL}};
const Segment SGI32916[] = {{&S28528[117], 3}, {NULL}};
const Segment SGI32917[] = {{&S28528[83], 5}, {NULL}};
const Segment SGI32918[] = {{&S28529[44], 14}, {NULL}};
const Segment SGI32919[] = {{&S28529[44], 25}, {NULL}};
const Segment SGI32920[] = {{&S28529[30], 6}, {NULL}};
const Segment SGI32921[] = {{&S28529[117], 3}, {NULL}};
const Segment SGI32922[] = {{&S28529[83], 5}, {NULL}};
const Segment SGI32923[] = {{&S28530[44], 17}, {NULL}};
const Segment SGI32924[] = {{&S28530[44], 28}, {NULL}};
const Segment SGI32925[] = {{&S28530[30], 6}, {NULL}};
const Segment SGI32926[] = {{&S28530[120], 3}, {NULL}};
const Segment SGI32927[] = {{&S28530[86], 5}, {NULL}};
const Segment SGI32928[] = {{&S28531[44], 11}, {NULL}};
const Segment SGI32929[] = {{&S28531[44], 22}, {NULL}};
const Segment SGI32930[] = {{&S28531[30], 6}, {NULL}};
const Segment SGI32931[] = {{&S28531[114], 3}, {NULL}};
const Segment SGI32932[] = {{&S28531[80], 5}, {NULL}};
const Segment SGI32933[] = {{&S28533[44], 16}, {NULL}};
const Segment SGI32934[] = {{&S28533[44], 27}, {NULL}};
const Segment SGI32935[] = {{&S28533[30], 6}, {NULL}};
const Segment SGI32936[] = {{&S28533[119], 3}, {NULL}};
const Segment SGI32937[] = {{&S28533[85], 5}, {NULL}};
const Segment SGI32938[] = {{&S27256[44], 11}, {NULL}};
const Segment SGI32939[] = {{&S27256[44], 28}, {NULL}};
const Segment SGI32940[] = {{&S27256[30], 6}, {NULL}};
const Segment SGI32941[] = {{&S27256[120], 3}, {NULL}};
const Segment SGI32942[] = {{&S27256[86], 5}, {NULL}};
const Segment SGI32943[] = {{&S27257[44], 11}, {NULL}};
const Segment SGI32944[] = {{&S27257[44], 28}, {NULL}};
const Segment SGI32945[] = {{&S27257[30], 6}, {NULL}};
const Segment SGI32946[] = {{&S27257[120], 3}, {NULL}};
const Segment SGI32947[] = {{&S28544[38], 12}, {NULL}};
const Segment SGI32948[] = {{&S28544[31], 6}, {NULL}};
const Segment SGI32949[] = {{&S27257[86], 5}, {NULL}};
const Segment SGI32950[] = {{&S27258[44], 11}, {NULL}};
const Segment SGI32951[] = {{&S27258[44], 28}, {NULL}};
const Segment SGI32952[] = {{&S27258[30], 6}, {NULL}};
const Segment SGI32953[] = {{&S27258[120], 3}, {NULL}};
const Segment SGI32954[] = {{&S27258[86], 5}, {NULL}};
const Segment SGI32955[] = {{&S28549[44], 15}, {NULL}};
const Segment SGI32956[] = {{&S28549[44], 26}, {NULL}};
const Segment SGI32957[] = {{&S28549[30], 6}, {NULL}};
const Segment SGI32958[] = {{&S28549[118], 3}, {NULL}};
const Segment SGI32959[] = {{&S28549[84], 5}, {NULL}};
const Segment SGI32960[] = {{&S28550[44], 14}, {NULL}};
const Segment SGI32961[] = {{&S28550[44], 25}, {NULL}};
const Segment SGI32962[] = {{&S28550[30], 6}, {NULL}};
const Segment SGI32963[] = {{&S28550[117], 3}, {NULL}};
const Segment SGI32964[] = {{&S28550[83], 5}, {NULL}};
const Segment SGI32965[] = {{&S27265[42], 11}, {NULL}};
const Segment SGI32966[] = {{&S27265[30], 6}, {NULL}};
const Segment SGI32967[] = {{&S27265[118], 3}, {NULL}};
const Segment SGI32968[] = {{&S27265[84], 5}, {NULL}};
const Segment SGI32969[] = {{&S27266[38], 11}, {NULL}};
const Segment SGI32970[] = {{&S27266[31], 6}, {NULL}};
const Segment SGI32971[] = {{&S27273[44], 11}, {NULL}};
const Segment SGI32972[] = {{&S27273[44], 28}, {NULL}};
const Segment SGI32973[] = {{&S27273[30], 6}, {NULL}};
const Segment SGI32974[] = {{&S27273[120], 3}, {NULL}};
const Segment SGI32975[] = {{&S27273[86], 5}, {NULL}};
const Segment SGI32976[] = {{&S27274[38], 11}, {NULL}};
const Segment SGI32977[] = {{&S27274[31], 6}, {NULL}};
const Segment SGI32978[] = {{&S27275[38], 11}, {NULL}};
const Segment SGI32979[] = {{&S27275[31], 6}, {NULL}};
const Segment SGI32980[] = {{&S27276[38], 11}, {NULL}};
const Segment SGI32981[] = {{&S27276[31], 6}, {NULL}};
const Segment SGI32982[] = {{&S30602[44], 11}, {NULL}};
const Segment SGI32983[] = {{&S30602[44], 23}, {NULL}};
const Segment SGI32984[] = {{&S30602[30], 6}, {NULL}};
const Segment SGI32985[] = {{&S30602[115], 3}, {NULL}};
const Segment SGI32986[] = {{&S30602[81], 5}, {NULL}};
const Segment SGI32987[] = {{&S30631[44], 11}, {NULL}};
const Segment SGI32988[] = {{&S30631[44], 23}, {NULL}};
const Segment SGI32989[] = {{&S30631[30], 6}, {NULL}};
const Segment SGI32990[] = {{&S30631[114], 3}, {NULL}};
const Segment SGI32991[] = {{&S30631[81], 5}, {NULL}};
const Segment SGI32992[] = {{&S26891[44], 8}, {NULL}};
const Segment SGI32993[] = {{&S26891[44], 20}, {NULL}};
const Segment SGI32994[] = {{&S26891[30], 6}, {NULL}};
const Segment SGI32995[] = {{&S26891[112], 3}, {NULL}};
const Segment SGI32996[] = {{&S26891[78], 5}, {NULL}};
const Segment SGI32997[] = {{&S26892[44], 8}, {NULL}};
const Segment SGI32998[] = {{&S26892[44], 17}, {NULL}};
const Segment SGI32999[] = {{&S26892[30], 6}, {NULL}};
const Segment SGI33000[] = {{&S26892[109], 3}, {NULL}};
const Segment SGI33001[] = {{&S26892[75], 5}, {NULL}};
const Segment SGI33002[] = {{&S27729[44], 12}, {NULL}};
const Segment SGI33003[] = {{&S27729[44], 23}, {NULL}};
const Segment SGI33004[] = {{&S27729[30], 6}, {NULL}};
const Segment SGI33005[] = {{&S27729[115], 3}, {NULL}};
const Segment SGI33006[] = {{&S27729[81], 5}, {NULL}};
const Segment SGI33007[] = {{&S27730[44], 12}, {NULL}};
const Segment SGI33008[] = {{&S27730[44], 23}, {NULL}};
const Segment SGI33009[] = {{&S27730[30], 6}, {NULL}};
const Segment SGI33010[] = {{&S27730[115], 3}, {NULL}};
const Segment SGI33011[] = {{&S27730[81], 5}, {NULL}};
const Segment SGI33012[] = {{&S27740[44], 17}, {NULL}};
const Segment SGI33013[] = {{&S27740[44], 28}, {NULL}};
const Segment SGI33014[] = {{&S27740[30], 6}, {NULL}};
const Segment SGI33015[] = {{&S27740[120], 3}, {NULL}};
const Segment SGI33016[] = {{&S27740[86], 5}, {NULL}};
const Segment SGI33017[] = {{&S26907[44], 8}, {NULL}};
const Segment SGI33018[] = {{&S26907[44], 20}, {NULL}};
const Segment SGI33019[] = {{&S26907[30], 6}, {NULL}};
const Segment SGI33020[] = {{&S26907[112], 3}, {NULL}};
const Segment SGI33021[] = {{&S26907[78], 5}, {NULL}};
const Segment SGI33022[] = {{&S27741[44], 17}, {NULL}};
const Segment SGI33023[] = {{&S27741[44], 28}, {NULL}};
const Segment SGI33024[] = {{&S27741[30], 6}, {NULL}};
const Segment SGI33025[] = {{&S27741[120], 3}, {NULL}};
const Segment SGI33026[] = {{&S27741[86], 5}, {NULL}};
const Segment SGI33027[] = {{&S27742[44], 9}, {NULL}};
const Segment SGI33028[] = {{&S27742[44], 20}, {NULL}};
const Segment SGI33029[] = {{&S27742[30], 6}, {NULL}};
const Segment SGI33030[] = {{&S27742[112], 3}, {NULL}};
const Segment SGI33031[] = {{&S27742[78], 5}, {NULL}};
const Segment SGI33032[] = {{&S26912[38], 8}, {NULL}};
const Segment SGI33033[] = {{&S26912[31], 6}, {NULL}};
const Segment SGI33034[] = {{&S27749[44], 12}, {NULL}};
const Segment SGI33035[] = {{&S27749[44], 23}, {NULL}};
const Segment SGI33036[] = {{&S27749[30], 6}, {NULL}};
const Segment SGI33037[] = {{&S27749[84], 4}, {NULL}};
const Segment SGI33038[] = {{&S27749[84], 3}, {NULL}};
const Segment SGI33039[] = {{&S27750[44], 12}, {NULL}};
const Segment SGI33040[] = {{&S27750[44], 23}, {NULL}};
const Segment SGI33041[] = {{&S27750[30], 6}, {NULL}};
const Segment SGI33042[] = {{&S27750[115], 3}, {NULL}};
const Segment SGI33043[] = {{&S27750[81], 5}, {NULL}};
const Segment SGI33044[] = {{&S27753[44], 12}, {NULL}};
const Segment SGI33045[] = {{&S27753[44], 35}, {NULL}};
const Segment SGI33046[] = {{&S27753[30], 6}, {NULL}};
const Segment SGI33047[] = {{&S27753[127], 3}, {NULL}};
const Segment SGI33048[] = {{&S27753[93], 5}, {NULL}};
const Segment SGI33049[] = {{&S27754[44], 12}, {NULL}};
const Segment SGI33050[] = {{&S27754[44], 35}, {NULL}};
const Segment SGI33051[] = {{&S27754[30], 6}, {NULL}};
const Segment SGI33052[] = {{&S27754[127], 3}, {NULL}};
const Segment SGI33053[] = {{&S27754[93], 5}, {NULL}};
const Segment SGI33054[] = {{&S27755[44], 12}, {NULL}};
const Segment SGI33055[] = {{&S27755[44], 34}, {NULL}};
const Segment SGI33056[] = {{&S27755[30], 6}, {NULL}};
const Segment SGI33057[] = {{&S27755[126], 3}, {NULL}};
const Segment SGI33058[] = {{&S27755[92], 5}, {NULL}};
const Segment SGI33059[] = {{&S27757[38], 12}, {NULL}};
const Segment SGI33060[] = {{&S27757[31], 6}, {NULL}};
const Segment SGI33061[] = {{&S38227[44], 14}, {NULL}};
const Segment SGI33062[] = {{&S38227[44], 26}, {NULL}};
const Segment SGI33063[] = {{&S38227[30], 6}, {NULL}};
const Segment SGI33064[] = {{&S38227[118], 3}, {NULL}};
const Segment SGI33065[] = {{&S38227[84], 5}, {NULL}};
const Segment SGI33066[] = {{&S38228[44], 14}, {NULL}};
const Segment SGI33067[] = {{&S38228[44], 26}, {NULL}};
const Segment SGI33068[] = {{&S38228[30], 6}, {NULL}};
const Segment SGI33069[] = {{&S38228[118], 3}, {NULL}};
const Segment SGI33070[] = {{&S38228[84], 5}, {NULL}};
const Segment SGI33071[] = {{&S38229[44], 9}, {NULL}};
const Segment SGI33072[] = {{&S38229[44], 21}, {NULL}};
const Segment SGI33073[] = {{&S38229[30], 6}, {NULL}};
const Segment SGI33074[] = {{&S38229[113], 3}, {NULL}};
const Segment SGI33075[] = {{&S38229[79], 5}, {NULL}};
const Segment SGI33076[] = {{&S38230[44], 14}, {NULL}};
const Segment SGI33077[] = {{&S38230[44], 26}, {NULL}};
const Segment SGI33078[] = {{&S38230[30], 6}, {NULL}};
const Segment SGI33079[] = {{&S38230[118], 3}, {NULL}};
const Segment SGI33080[] = {{&S27770[44], 14}, {NULL}};
const Segment SGI33081[] = {{&S27770[44], 25}, {NULL}};
const Segment SGI33082[] = {{&S27770[30], 6}, {NULL}};
const Segment SGI33083[] = {{&S38230[84], 5}, {NULL}};
const Segment SGI33084[] = {{&S27770[117], 3}, {NULL}};
const Segment SGI33085[] = {{&S27770[83], 5}, {NULL}};
const Segment SGI33086[] = {{&S38231[44], 14}, {NULL}};
const Segment SGI33087[] = {{&S38231[44], 26}, {NULL}};
const Segment SGI33088[] = {{&S38231[30], 6}, {NULL}};
const Segment SGI33089[] = {{&S38231[118], 3}, {NULL}};
const Segment SGI33090[] = {{&S38231[84], 5}, {NULL}};
const Segment SGI33091[] = {{&S38232[38], 9}, {NULL}};
const Segment SGI33092[] = {{&S38232[31], 6}, {NULL}};
const Segment SGI33093[] = {{&S38233[44], 14}, {NULL}};
const Segment SGI33094[] = {{&S38233[44], 26}, {NULL}};
const Segment SGI33095[] = {{&S38233[30], 6}, {NULL}};
const Segment SGI33096[] = {{&S38233[118], 3}, {NULL}};
const Segment SGI33097[] = {{&S38233[84], 5}, {NULL}};
const Segment SGI33098[] = {{&S27775[44], 12}, {NULL}};
const Segment SGI33099[] = {{&S27775[44], 35}, {NULL}};
const Segment SGI33100[] = {{&S27775[30], 6}, {NULL}};
const Segment SGI33101[] = {{&S27775[127], 3}, {NULL}};
const Segment SGI33102[] = {{&S27775[93], 5}, {NULL}};
const Segment SGI33103[] = {{&S38234[42], 14}, {NULL}};
const Segment SGI33104[] = {{&S38234[30], 6}, {NULL}};
const Segment SGI33105[] = {{&S38234[116], 3}, {NULL}};
const Segment SGI33106[] = {{&S38234[82], 5}, {NULL}};
const Segment SGI33107[] = {{&S27776[44], 12}, {NULL}};
const Segment SGI33108[] = {{&S27776[44], 35}, {NULL}};
const Segment SGI33109[] = {{&S27776[30], 6}, {NULL}};
const Segment SGI33110[] = {{&S27776[127], 3}, {NULL}};
const Segment SGI33111[] = {{&S27776[93], 5}, {NULL}};
const Segment SGI33112[] = {{&S27937[44], 18}, {NULL}};
const Segment SGI33113[] = {{&S27937[44], 36}, {NULL}};
const Segment SGI33114[] = {{&S27937[30], 6}, {NULL}};
const Segment SGI33115[] = {{&S27937[128], 3}, {NULL}};
const Segment SGI33116[] = {{&S27937[94], 5}, {NULL}};
const Segment SGI33117[] = {{&S27943[44], 18}, {NULL}};
const Segment SGI33118[] = {{&S27943[44], 36}, {NULL}};
const Segment SGI33119[] = {{&S27943[30], 6}, {NULL}};
const Segment SGI33120[] = {{&S27943[128], 3}, {NULL}};
const Segment SGI33121[] = {{&S27943[94], 5}, {NULL}};
const Segment SGI33122[] = {{&S27947[44], 17}, {NULL}};
const Segment SGI33123[] = {{&S27947[44], 35}, {NULL}};
const Segment SGI33124[] = {{&S27947[30], 6}, {NULL}};
const Segment SGI33125[] = {{&S27947[127], 3}, {NULL}};
const Segment SGI33126[] = {{&S27947[93], 5}, {NULL}};
const Segment SGI33127[] = {{&S27953[44], 10}, {NULL}};
const Segment SGI33128[] = {{&S27953[44], 33}, {NULL}};
const Segment SGI33129[] = {{&S27953[30], 6}, {NULL}};
const Segment SGI33130[] = {{&S27953[125], 3}, {NULL}};
const Segment SGI33131[] = {{&S27953[91], 5}, {NULL}};
const Segment SGI33132[] = {{&S27956[44], 10}, {NULL}};
const Segment SGI33133[] = {{&S27956[44], 33}, {NULL}};
const Segment SGI33134[] = {{&S27956[30], 6}, {NULL}};
const Segment SGI33135[] = {{&S27956[125], 3}, {NULL}};
const Segment SGI33136[] = {{&S27956[91], 5}, {NULL}};
const Segment SGI33137[] = {{&S27957[44], 10}, {NULL}};
const Segment SGI33138[] = {{&S27957[44], 33}, {NULL}};
const Segment SGI33139[] = {{&S27957[30], 6}, {NULL}};
const Segment SGI33140[] = {{&S27957[125], 3}, {NULL}};
const Segment SGI33141[] = {{&S27957[91], 5}, {NULL}};
const Segment SGI33142[] = {{&S27961[44], 10}, {NULL}};
const Segment SGI33143[] = {{&S27961[44], 33}, {NULL}};
const Segment SGI33144[] = {{&S27961[30], 6}, {NULL}};
const Segment SGI33145[] = {{&S27961[125], 3}, {NULL}};
const Segment SGI33146[] = {{&S27961[91], 5}, {NULL}};
const Segment SGI33147[] = {{&S27963[44], 10}, {NULL}};
const Segment SGI33148[] = {{&S27963[44], 33}, {NULL}};
const Segment SGI33149[] = {{&S27963[30], 6}, {NULL}};
const Segment SGI33150[] = {{&S27963[125], 3}, {NULL}};
const Segment SGI33151[] = {{&S27963[91], 5}, {NULL}};
const Segment SGI33152[] = {{&S27966[44], 10}, {NULL}};
const Segment SGI33153[] = {{&S27966[44], 22}, {NULL}};
const Segment SGI33154[] = {{&S27966[30], 6}, {NULL}};
const Segment SGI33155[] = {{&S27966[114], 3}, {NULL}};
const Segment SGI33156[] = {{&S27966[80], 5}, {NULL}};
const Segment SGI33157[] = {{&S27967[44], 10}, {NULL}};
const Segment SGI33158[] = {{&S27967[44], 33}, {NULL}};
const Segment SGI33159[] = {{&S27967[30], 6}, {NULL}};
const Segment SGI33160[] = {{&S27967[125], 3}, {NULL}};
const Segment SGI33161[] = {{&S27967[91], 5}, {NULL}};
const Segment SGI33162[] = {{&S27968[43], 10}, {NULL}};
const Segment SGI33163[] = {{&S27968[43], 33}, {NULL}};
const Segment SGI33164[] = {{&S27968[30], 5}, {NULL}};
const Segment SGI33165[] = {{&S27968[124], 3}, {NULL}};
const Segment SGI33166[] = {{&S27968[90], 5}, {NULL}};
const Segment SGI33167[] = {{&S27970[44], 9}, {NULL}};
const Segment SGI33168[] = {{&S27970[44], 32}, {NULL}};
const Segment SGI33169[] = {{&S27970[30], 6}, {NULL}};
const Segment SGI33170[] = {{&S27970[124], 3}, {NULL}};
const Segment SGI33171[] = {{&S27970[90], 5}, {NULL}};
const Segment SGI33172[] = {{&S27971[44], 9}, {NULL}};
const Segment SGI33173[] = {{&S27971[44], 32}, {NULL}};
const Segment SGI33174[] = {{&S27971[30], 6}, {NULL}};
const Segment SGI33175[] = {{&S27971[124], 3}, {NULL}};
const Segment SGI33176[] = {{&S27971[90], 5}, {NULL}};
const Segment SGI33177[] = {{&S27972[43], 10}, {NULL}};
const Segment SGI33178[] = {{&S27972[43], 33}, {NULL}};
const Segment SGI33179[] = {{&S27972[124], 3}, {NULL}};
const Segment SGI33180[] = {{&S27972[90], 5}, {NULL}};
const Segment SGI33181[] = {{&S27973[44], 10}, {NULL}};
const Segment SGI33182[] = {{&S27973[44], 33}, {NULL}};
const Segment SGI33183[] = {{&S27973[30], 6}, {NULL}};
const Segment SGI33184[] = {{&S27973[125], 3}, {NULL}};
const Segment SGI33185[] = {{&S27973[91], 5}, {NULL}};
const Segment SGI33186[] = {{&S27974[44], 9}, {NULL}};
const Segment SGI33187[] = {{&S27974[44], 32}, {NULL}};
const Segment SGI33188[] = {{&S27974[30], 6}, {NULL}};
const Segment SGI33189[] = {{&S27974[124], 3}, {NULL}};
const Segment SGI33190[] = {{&S27974[90], 5}, {NULL}};
const Segment SGI33191[] = {{&S27976[38], 10}, {NULL}};
const Segment SGI33192[] = {{&S27976[31], 6}, {NULL}};
const Segment SGI33193[] = {{&S27990[44], 14}, {NULL}};
const Segment SGI33194[] = {{&S27990[44], 27}, {NULL}};
const Segment SGI33195[] = {{&S27990[30], 6}, {NULL}};
const Segment SGI33196[] = {{&S27990[119], 3}, {NULL}};
const Segment SGI33197[] = {{&S27990[85], 5}, {NULL}};
const Segment SGI33198[] = {{&S27995[44], 10}, {NULL}};
const Segment SGI33199[] = {{&S27995[44], 33}, {NULL}};
const Segment SGI33200[] = {{&S27995[30], 6}, {NULL}};
const Segment SGI33201[] = {{&S27995[94], 6}, {NULL}};
const Segment SGI33202[] = {{&S28052[44], 12}, {NULL}};
const Segment SGI33203[] = {{&S28052[44], 23}, {NULL}};
const Segment SGI33204[] = {{&S28052[30], 6}, {NULL}};
const Segment SGI33205[] = {{&S28052[115], 3}, {NULL}};
const Segment SGI33206[] = {{&S28052[81], 5}, {NULL}};
const Segment SGI33207[] = {{&S27142[44], 10}, {NULL}};
const Segment SGI33208[] = {{&S27142[44], 27}, {NULL}};
const Segment SGI33209[] = {{&S27142[30], 6}, {NULL}};
const Segment SGI33210[] = {{&S28059[44], 9}, {NULL}};
const Segment SGI33211[] = {{&S27142[119], 3}, {NULL}};
const Segment SGI33212[] = {{&S28059[44], 20}, {NULL}};
const Segment SGI33213[] = {{&S28059[30], 6}, {NULL}};
const Segment SGI33214[] = {{&S27142[85], 5}, {NULL}};
const Segment SGI33215[] = {{&S28059[112], 3}, {NULL}};
const Segment SGI33216[] = {{&S28059[78], 5}, {NULL}};
const Segment SGI33217[] = {{&S28060[44], 12}, {NULL}};
const Segment SGI33218[] = {{&S28060[44], 23}, {NULL}};
const Segment SGI33219[] = {{&S28060[30], 6}, {NULL}};
const Segment SGI33220[] = {{&S28060[115], 3}, {NULL}};
const Segment SGI33221[] = {{&S28060[81], 5}, {NULL}};
const Segment SGI33222[] = {{&S27144[44], 10}, {NULL}};
const Segment SGI33223[] = {{&S27144[44], 21}, {NULL}};
const Segment SGI33224[] = {{&S27144[30], 6}, {NULL}};
const Segment SGI33225[] = {{&S27144[113], 3}, {NULL}};
const Segment SGI33226[] = {{&S27144[79], 5}, {NULL}};
const Segment SGI33227[] = {{&S28061[43], 9}, {NULL}};
const Segment SGI33228[] = {{&S28061[43], 20}, {NULL}};
const Segment SGI33229[] = {{&S28061[30], 5}, {NULL}};
const Segment SGI33230[] = {{&S28061[111], 3}, {NULL}};
const Segment SGI33231[] = {{&S28061[77], 5}, {NULL}};
const Segment SGI33232[] = {{&S27145[43], 10}, {NULL}};
const Segment SGI33233[] = {{&S27145[43], 27}, {NULL}};
const Segment SGI33234[] = {{&S27145[30], 5}, {NULL}};
const Segment SGI33235[] = {{&S27145[118], 3}, {NULL}};
const Segment SGI33236[] = {{&S27145[84], 5}, {NULL}};
const Segment SGI33237[] = {{&S28062[44], 9}, {NULL}};
const Segment SGI33238[] = {{&S28062[44], 20}, {NULL}};
const Segment SGI33239[] = {{&S28062[30], 6}, {NULL}};
const Segment SGI33240[] = {{&S28062[112], 3}, {NULL}};
const Segment SGI33241[] = {{&S28062[78], 5}, {NULL}};
const Segment SGI33242[] = {{&S27146[44], 9}, {NULL}};
const Segment SGI33243[] = {{&S27146[44], 26}, {NULL}};
const Segment SGI33244[] = {{&S27146[30], 6}, {NULL}};
const Segment SGI33245[] = {{&S27146[118], 3}, {NULL}};
const Segment SGI33246[] = {{&S27146[84], 5}, {NULL}};
const Segment SGI33247[] = {{&S28063[44], 11}, {NULL}};
const Segment SGI33248[] = {{&S28063[44], 22}, {NULL}};
const Segment SGI33249[] = {{&S28063[30], 6}, {NULL}};
const Segment SGI33250[] = {{&S28063[114], 3}, {NULL}};
const Segment SGI33251[] = {{&S28063[80], 5}, {NULL}};
const Segment SGI33252[] = {{&S27147[42], 10}, {NULL}};
const Segment SGI33253[] = {{&S27147[30], 6}, {NULL}};
const Segment SGI33254[] = {{&S27147[117], 3}, {NULL}};
const Segment SGI33255[] = {{&S27147[83], 5}, {NULL}};
const Segment SGI33256[] = {{&S28064[43], 12}, {NULL}};
const Segment SGI33257[] = {{&S28064[43], 23}, {NULL}};
const Segment SGI33258[] = {{&S28064[30], 5}, {NULL}};
const Segment SGI33259[] = {{&S28064[114], 3}, {NULL}};
const Segment SGI33260[] = {{&S28064[80], 5}, {NULL}};
const Segment SGI33261[] = {{&S27148[44], 10}, {NULL}};
const Segment SGI33262[] = {{&S27148[44], 32}, {NULL}};
const Segment SGI33263[] = {{&S27148[30], 6}, {NULL}};
const Segment SGI33264[] = {{&S28065[43], 12}, {NULL}};
const Segment SGI33265[] = {{&S28065[43], 23}, {NULL}};
const Segment SGI33266[] = {{&S28065[114], 3}, {NULL}};
const Segment SGI33267[] = {{&S28065[80], 5}, {NULL}};
const Segment SGI33268[] = {{&S28066[44], 11}, {NULL}};
const Segment SGI33269[] = {{&S28066[44], 22}, {NULL}};
const Segment SGI33270[] = {{&S28066[30], 6}, {NULL}};
const Segment SGI33271[] = {{&S28066[114], 3}, {NULL}};
const Segment SGI33272[] = {{&S28066[80], 5}, {NULL}};
const Segment SGI33273[] = {{&S27151[38], 10}, {NULL}};
const Segment SGI33274[] = {{&S27151[31], 6}, {NULL}};
const Segment SGI33275[] = {{&S28067[42], 12}, {NULL}};
const Segment SGI33276[] = {{&S28067[30], 6}, {NULL}};
const Segment SGI33277[] = {{&S27152[43], 10}, {NULL}};
const Segment SGI33278[] = {{&S28067[113], 3}, {NULL}};
const Segment SGI33279[] = {{&S27152[43], 27}, {NULL}};
const Segment SGI33280[] = {{&S28067[79], 5}, {NULL}};
const Segment SGI33281[] = {{&S27152[118], 3}, {NULL}};
const Segment SGI33282[] = {{&S27152[84], 5}, {NULL}};
const Segment SGI33283[] = {{&S27153[44], 9}, {NULL}};
const Segment SGI33284[] = {{&S27153[44], 26}, {NULL}};
const Segment SGI33285[] = {{&S27153[30], 6}, {NULL}};
const Segment SGI33286[] = {{&S28069[38], 9}, {NULL}};
const Segment SGI33287[] = {{&S28069[31], 6}, {NULL}};
const Segment SGI33288[] = {{&S27153[118], 3}, {NULL}};
const Segment SGI33289[] = {{&S27153[84], 5}, {NULL}};
const Segment SGI33290[] = {{&S27154[44], 10}, {NULL}};
const Segment SGI33291[] = {{&S28070[44], 9}, {NULL}};
const Segment SGI33292[] = {{&S28070[44], 20}, {NULL}};
const Segment SGI33293[] = {{&S27154[44], 27}, {NULL}};
const Segment SGI33294[] = {{&S28070[30], 6}, {NULL}};
const Segment SGI33295[] = {{&S27154[30], 6}, {NULL}};
const Segment SGI33296[] = {{&S27154[119], 3}, {NULL}};
const Segment SGI33297[] = {{&S27154[85], 5}, {NULL}};
const Segment SGI33298[] = {{&S28071[44], 8}, {NULL}};
const Segment SGI33299[] = {{&S28071[44], 19}, {NULL}};
const Segment SGI33300[] = {{&S28071[30], 6}, {NULL}};
const Segment SGI33301[] = {{&S28071[111], 3}, {NULL}};
const Segment SGI33302[] = {{&S28071[77], 5}, {NULL}};
const Segment SGI33303[] = {{&S27156[44], 10}, {NULL}};
const Segment SGI33304[] = {{&S27156[44], 27}, {NULL}};
const Segment SGI33305[] = {{&S27156[30], 6}, {NULL}};
const Segment SGI33306[] = {{&S27156[121], 3}, {NULL}};
const Segment SGI33307[] = {{&S27156[85], 7}, {NULL}};
const Segment SGI33308[] = {{&S27156[141], 8}, {NULL}};
const Segment SGI33309[] = {{&S28076[44], 8}, {NULL}};
const Segment SGI33310[] = {{&S28076[44], 19}, {NULL}};
const Segment SGI33311[] = {{&S28076[30], 6}, {NULL}};
const Segment SGI33312[] = {{&S28076[111], 3}, {NULL}};
const Segment SGI33313[] = {{&S28076[77], 5}, {NULL}};
const Segment SGI33314[] = {{&S28077[44], 9}, {NULL}};
const Segment SGI33315[] = {{&S28077[44], 20}, {NULL}};
const Segment SGI33316[] = {{&S28077[30], 6}, {NULL}};
const Segment SGI33317[] = {{&S28077[114], 3}, {NULL}};
const Segment SGI33318[] = {{&S28077[78], 7}, {NULL}};
const Segment SGI33319[] = {{&S28077[134], 8}, {NULL}};
const Segment SGI33320[] = {{&S28078[43], 9}, {NULL}};
const Segment SGI33321[] = {{&S28078[43], 20}, {NULL}};
const Segment SGI33322[] = {{&S28078[111], 3}, {NULL}};
const Segment SGI33323[] = {{&S28078[77], 5}, {NULL}};
const Segment SGI33324[] = {{&S28079[42], 9}, {NULL}};
const Segment SGI33325[] = {{&S28079[30], 6}, {NULL}};
const Segment SGI33326[] = {{&S28079[110], 3}, {NULL}};
const Segment SGI33327[] = {{&S28079[76], 5}, {NULL}};
const Segment SGI33328[] = {{&S28082[44], 9}, {NULL}};
const Segment SGI33329[] = {{&S28082[44], 20}, {NULL}};
const Segment SGI33330[] = {{&S28082[30], 6}, {NULL}};
const Segment SGI33331[] = {{&S28082[78], 6}, {NULL}};
const Segment SGI33332[] = {{&S27210[44], 10}, {NULL}};
const Segment SGI33333[] = {{&S27210[44], 27}, {NULL}};
const Segment SGI33334[] = {{&S27210[30], 6}, {NULL}};
const Segment SGI33335[] = {{&S27210[119], 3}, {NULL}};
const Segment SGI33336[] = {{&S27210[85], 5}, {NULL}};
const Segment SGI33337[] = {{&S27211[44], 11}, {NULL}};
const Segment SGI33338[] = {{&S27211[44], 28}, {NULL}};
const Segment SGI33339[] = {{&S27211[30], 6}, {NULL}};
const Segment SGI33340[] = {{&S27211[120], 3}, {NULL}};
const Segment SGI33341[] = {{&S27211[86], 5}, {NULL}};
const Segment SGI33342[] = {{&S27212[44], 9}, {NULL}};
const Segment SGI33343[] = {{&S27212[44], 26}, {NULL}};
const Segment SGI33344[] = {{&S27212[30], 6}, {NULL}};
const Segment SGI33345[] = {{&S27212[118], 3}, {NULL}};
const Segment SGI33346[] = {{&S27212[84], 5}, {NULL}};
const Segment SGI33347[] = {{&S27213[43], 10}, {NULL}};
const Segment SGI33348[] = {{&S27213[43], 27}, {NULL}};
const Segment SGI33349[] = {{&S27213[30], 5}, {NULL}};
const Segment SGI33350[] = {{&S27213[118], 3}, {NULL}};
const Segment SGI33351[] = {{&S27213[84], 5}, {NULL}};
const Segment SGI33352[] = {{&S27214[44], 9}, {NULL}};
const Segment SGI33353[] = {{&S27214[44], 26}, {NULL}};
const Segment SGI33354[] = {{&S27214[30], 6}, {NULL}};
const Segment SGI33355[] = {{&S27214[118], 3}, {NULL}};
const Segment SGI33356[] = {{&S27214[84], 5}, {NULL}};
const Segment SGI33357[] = {{&S27222[38], 11}, {NULL}};
const Segment SGI33358[] = {{&S27222[31], 6}, {NULL}};
const Segment SGI33359[] = {{&S27224[38], 10}, {NULL}};
const Segment SGI33360[] = {{&S27224[31], 6}, {NULL}};
const Segment SGI33361[] = {{&S27230[43], 10}, {NULL}};
const Segment SGI33362[] = {{&S27230[43], 27}, {NULL}};
const Segment SGI33363[] = {{&S27230[118], 3}, {NULL}};
const Segment SGI33364[] = {{&S27230[84], 5}, {NULL}};
const Segment SGI33365[] = {{&S27235[42], 11}, {NULL}};
const Segment SGI33366[] = {{&S27235[30], 6}, {NULL}};
const Segment SGI33367[] = {{&S27235[118], 3}, {NULL}};
const Segment SGI33368[] = {{&S27235[84], 5}, {NULL}};
const Segment SGI33369[] = {{&S27259[44], 11}, {NULL}};
const Segment SGI33370[] = {{&S27259[44], 28}, {NULL}};
const Segment SGI33371[] = {{&S27259[30], 6}, {NULL}};
const Segment SGI33372[] = {{&S27259[120], 3}, {NULL}};
const Segment SGI33373[] = {{&S27259[86], 5}, {NULL}};
const Segment SGI33374[] = {{&S27260[44], 11}, {NULL}};
const Segment SGI33375[] = {{&S27260[44], 28}, {NULL}};
const Segment SGI33376[] = {{&S27260[30], 6}, {NULL}};
const Segment SGI33377[] = {{&S27260[120], 3}, {NULL}};
const Segment SGI33378[] = {{&S27260[86], 5}, {NULL}};
const Segment SGI33379[] = {{&S27261[44], 11}, {NULL}};
const Segment SGI33380[] = {{&S27261[44], 28}, {NULL}};
const Segment SGI33381[] = {{&S27261[30], 6}, {NULL}};
const Segment SGI33382[] = {{&S27261[120], 3}, {NULL}};
const Segment SGI33383[] = {{&S27261[86], 5}, {NULL}};
const Segment SGI33384[] = {{&S38683[38], 6}, {NULL}};
const Segment SGI33385[] = {{&S38683[31], 6}, {NULL}};
const Segment SGI33386[] = {{&S38684[44], 6}, {NULL}};
const Segment SGI33387[] = {{&S38684[44], 18}, {NULL}};
const Segment SGI33388[] = {{&S38684[30], 6}, {NULL}};
const Segment SGI33389[] = {{&S38684[110], 3}, {NULL}};
const Segment SGI33390[] = {{&S38684[76], 5}, {NULL}};
const Segment SGI33391[] = {{&S27267[38], 11}, {NULL}};
const Segment SGI33392[] = {{&S27267[31], 6}, {NULL}};
const Segment SGI33393[] = {{&S38685[44], 6}, {NULL}};
const Segment SGI33394[] = {{&S38685[44], 18}, {NULL}};
const Segment SGI33395[] = {{&S38685[30], 6}, {NULL}};
const Segment SGI33396[] = {{&S38685[110], 3}, {NULL}};
const Segment SGI33397[] = {{&S38685[76], 5}, {NULL}};
const Segment SGI33398[] = {{&S27268[38], 11}, {NULL}};
const Segment SGI33399[] = {{&S27268[31], 6}, {NULL}};
const Segment SGI33400[] = {{&S38687[43], 6}, {NULL}};
const Segment SGI33401[] = {{&S38687[43], 18}, {NULL}};
const Segment SGI33402[] = {{&S38687[109], 3}, {NULL}};
const Segment SGI33403[] = {{&S38687[75], 5}, {NULL}};
const Segment SGI33404[] = {{&S38690[43], 6}, {NULL}};
const Segment SGI33405[] = {{&S38690[43], 18}, {NULL}};
const Segment SGI33406[] = {{&S38690[30], 5}, {NULL}};
const Segment SGI33407[] = {{&S38690[109], 3}, {NULL}};
const Segment SGI33408[] = {{&S38690[75], 5}, {NULL}};
const Segment SGI33409[] = {{&S38691[44], 5}, {NULL}};
const Segment SGI33410[] = {{&S38691[44], 17}, {NULL}};
const Segment SGI33411[] = {{&S38691[30], 6}, {NULL}};
const Segment SGI33412[] = {{&S38691[109], 3}, {NULL}};
const Segment SGI33413[] = {{&S38691[75], 5}, {NULL}};
const Segment SGI33414[] = {{&S27277[38], 11}, {NULL}};
const Segment SGI33415[] = {{&S27277[31], 6}, {NULL}};
const Segment SGI33416[] = {{&S38692[44], 6}, {NULL}};
const Segment SGI33417[] = {{&S38692[44], 18}, {NULL}};
const Segment SGI33418[] = {{&S38692[30], 6}, {NULL}};
const Segment SGI33419[] = {{&S38692[110], 3}, {NULL}};
const Segment SGI33420[] = {{&S38692[76], 5}, {NULL}};
const Segment SGI33421[] = {{&S38693[44], 5}, {NULL}};
const Segment SGI33422[] = {{&S38693[44], 17}, {NULL}};
const Segment SGI33423[] = {{&S38693[30], 6}, {NULL}};
const Segment SGI33424[] = {{&S38693[109], 3}, {NULL}};
const Segment SGI33425[] = {{&S38693[75], 5}, {NULL}};
const Segment SGI33426[] = {{&S38696[44], 6}, {NULL}};
const Segment SGI33427[] = {{&S38696[44], 18}, {NULL}};
const Segment SGI33428[] = {{&S38696[30], 6}, {NULL}};
const Segment SGI33429[] = {{&S38696[112], 3}, {NULL}};
const Segment SGI33430[] = {{&S38696[76], 7}, {NULL}};
const Segment SGI33431[] = {{&S38696[132], 8}, {NULL}};
const Segment SGI33432[] = {{&S29698[44], 13}, {NULL}};
const Segment SGI33433[] = {{&S29698[44], 22}, {NULL}};
const Segment SGI33434[] = {{&S29698[30], 6}, {NULL}};
const Segment SGI33435[] = {{&S29698[114], 3}, {NULL}};
const Segment SGI33436[] = {{&S29698[80], 5}, {NULL}};
const Segment SGI33437[] = {{&S38847[44], 14}, {NULL}};
const Segment SGI33438[] = {{&S38847[44], 38}, {NULL}};
const Segment SGI33439[] = {{&S38847[30], 6}, {NULL}};
const Segment SGI33440[] = {{&S38847[130], 3}, {NULL}};
const Segment SGI33441[] = {{&S38847[96], 5}, {NULL}};
const Segment SGI33442[] = {{&S38848[44], 14}, {NULL}};
const Segment SGI33443[] = {{&S38848[44], 38}, {NULL}};
const Segment SGI33444[] = {{&S38848[30], 6}, {NULL}};
const Segment SGI33445[] = {{&S38848[130], 3}, {NULL}};
const Segment SGI33446[] = {{&S38848[96], 5}, {NULL}};
const Segment SGI33447[] = {{&S38854[44], 14}, {NULL}};
const Segment SGI33448[] = {{&S38854[44], 38}, {NULL}};
const Segment SGI33449[] = {{&S38854[30], 6}, {NULL}};
const Segment SGI33450[] = {{&S38854[130], 3}, {NULL}};
const Segment SGI33451[] = {{&S38854[96], 5}, {NULL}};
const Segment SGI33452[] = {{&S38855[44], 14}, {NULL}};
const Segment SGI33453[] = {{&S38855[44], 38}, {NULL}};
const Segment SGI33454[] = {{&S38855[30], 6}, {NULL}};
const Segment SGI33455[] = {{&S38855[130], 3}, {NULL}};
const Segment SGI33456[] = {{&S38855[96], 5}, {NULL}};
const Segment SGI33457[] = {{&S38856[44], 14}, {NULL}};
const Segment SGI33458[] = {{&S38856[44], 42}, {NULL}};
const Segment SGI33459[] = {{&S38856[30], 6}, {NULL}};
const Segment SGI33460[] = {{&S38856[134], 3}, {NULL}};
const Segment SGI33461[] = {{&S38856[100], 5}, {NULL}};
const Segment SGI33462[] = {{&S38857[38], 14}, {NULL}};
const Segment SGI33463[] = {{&S38857[31], 6}, {NULL}};
const Segment SGI33464[] = {{&S38859[44], 14}, {NULL}};
const Segment SGI33465[] = {{&S38859[44], 38}, {NULL}};
const Segment SGI33466[] = {{&S38859[30], 6}, {NULL}};
const Segment SGI33467[] = {{&S38859[130], 3}, {NULL}};
const Segment SGI33468[] = {{&S38859[96], 5}, {NULL}};
const Segment SGI33469[] = {{&S29779[44], 19}, {NULL}};
const Segment SGI33470[] = {{&S29779[44], 30}, {NULL}};
const Segment SGI33471[] = {{&S29779[30], 6}, {NULL}};
const Segment SGI33472[] = {{&S29779[122], 3}, {NULL}};
const Segment SGI33473[] = {{&S29779[88], 5}, {NULL}};
const Segment SGI33474[] = {{&S38860[44], 14}, {NULL}};
const Segment SGI33475[] = {{&S38860[44], 38}, {NULL}};
const Segment SGI33476[] = {{&S38860[30], 6}, {NULL}};
const Segment SGI33477[] = {{&S38860[130], 3}, {NULL}};
const Segment SGI33478[] = {{&S38860[96], 5}, {NULL}};
const Segment SGI33479[] = {{&S29780[44], 25}, {NULL}};
const Segment SGI33480[] = {{&S29780[44], 36}, {NULL}};
const Segment SGI33481[] = {{&S29780[30], 6}, {NULL}};
const Segment SGI33482[] = {{&S29780[128], 3}, {NULL}};
const Segment SGI33483[] = {{&S29780[94], 5}, {NULL}};
const Segment SGI33484[] = {{&S38861[44], 14}, {NULL}};
const Segment SGI33485[] = {{&S38861[44], 38}, {NULL}};
const Segment SGI33486[] = {{&S38861[30], 6}, {NULL}};
const Segment SGI33487[] = {{&S38861[130], 3}, {NULL}};
const Segment SGI33488[] = {{&S38861[96], 5}, {NULL}};
const Segment SGI33489[] = {{&S29781[44], 26}, {NULL}};
const Segment SGI33490[] = {{&S29781[44], 37}, {NULL}};
const Segment SGI33491[] = {{&S29781[30], 6}, {NULL}};
const Segment SGI33492[] = {{&S29781[129], 3}, {NULL}};
const Segment SGI33493[] = {{&S29781[95], 5}, {NULL}};
const Segment SGI33494[] = {{&S38862[44], 14}, {NULL}};
const Segment SGI33495[] = {{&S38862[44], 38}, {NULL}};
const Segment SGI33496[] = {{&S38862[30], 6}, {NULL}};
const Segment SGI33497[] = {{&S38862[130], 3}, {NULL}};
const Segment SGI33498[] = {{&S38862[96], 5}, {NULL}};
const Segment SGI33499[] = {{&S29782[44], 25}, {NULL}};
const Segment SGI33500[] = {{&S29782[44], 36}, {NULL}};
const Segment SGI33501[] = {{&S29782[30], 6}, {NULL}};
const Segment SGI33502[] = {{&S29782[128], 3}, {NULL}};
const Segment SGI33503[] = {{&S29782[94], 5}, {NULL}};
const Segment SGI33504[] = {{&S38863[44], 14}, {NULL}};
const Segment SGI33505[] = {{&S38863[44], 38}, {NULL}};
const Segment SGI33506[] = {{&S38863[30], 6}, {NULL}};
const Segment SGI33507[] = {{&S38863[130], 3}, {NULL}};
const Segment SGI33508[] = {{&S38863[96], 5}, {NULL}};
const Segment SGI33509[] = {{&S29783[44], 25}, {NULL}};
const Segment SGI33510[] = {{&S29783[44], 36}, {NULL}};
const Segment SGI33511[] = {{&S29783[30], 6}, {NULL}};
const Segment SGI33512[] = {{&S29783[128], 3}, {NULL}};
const Segment SGI33513[] = {{&S38864[44], 14}, {NULL}};
const Segment SGI33514[] = {{&S38864[44], 38}, {NULL}};
const Segment SGI33515[] = {{&S29783[94], 5}, {NULL}};
const Segment SGI33516[] = {{&S38864[30], 6}, {NULL}};
const Segment SGI33517[] = {{&S38864[130], 3}, {NULL}};
const Segment SGI33518[] = {{&S38864[96], 5}, {NULL}};
const Segment SGI33519[] = {{&S29784[44], 25}, {NULL}};
const Segment SGI33520[] = {{&S29784[44], 36}, {NULL}};
const Segment SGI33521[] = {{&S29784[30], 6}, {NULL}};
const Segment SGI33522[] = {{&S29784[128], 3}, {NULL}};
const Segment SGI33523[] = {{&S38865[44], 14}, {NULL}};
const Segment SGI33524[] = {{&S38865[44], 42}, {NULL}};
const Segment SGI33525[] = {{&S29784[94], 5}, {NULL}};
const Segment SGI33526[] = {{&S38865[30], 6}, {NULL}};
const Segment SGI33527[] = {{&S38865[134], 3}, {NULL}};
const Segment SGI33528[] = {{&S38865[100], 5}, {NULL}};
const Segment SGI33529[] = {{&S29785[44], 25}, {NULL}};
const Segment SGI33530[] = {{&S29785[44], 36}, {NULL}};
const Segment SGI33531[] = {{&S29785[30], 6}, {NULL}};
const Segment SGI33532[] = {{&S38866[44], 12}, {NULL}};
const Segment SGI33533[] = {{&S29785[128], 3}, {NULL}};
const Segment SGI33534[] = {{&S38866[44], 36}, {NULL}};
const Segment SGI33535[] = {{&S38866[30], 6}, {NULL}};
const Segment SGI33536[] = {{&S29785[94], 5}, {NULL}};
const Segment SGI33537[] = {{&S38866[128], 3}, {NULL}};
const Segment SGI33538[] = {{&S38866[94], 5}, {NULL}};
const Segment SGI33539[] = {{&S29786[44], 11}, {NULL}};
const Segment SGI33540[] = {{&S29786[44], 22}, {NULL}};
const Segment SGI33541[] = {{&S29786[30], 6}, {NULL}};
const Segment SGI33542[] = {{&S38867[44], 14}, {NULL}};
const Segment SGI33543[] = {{&S38867[44], 42}, {NULL}};
const Segment SGI33544[] = {{&S38867[30], 6}, {NULL}};
const Segment SGI33545[] = {{&S29786[114], 3}, {NULL}};
const Segment SGI33546[] = {{&S29786[80], 5}, {NULL}};
const Segment SGI33547[] = {{&S38867[134], 3}, {NULL}};
const Segment SGI33548[] = {{&S38867[100], 5}, {NULL}};
const Segment SGI33549[] = {{&S38868[44], 14}, {NULL}};
const Segment SGI33550[] = {{&S38868[44], 38}, {NULL}};
const Segment SGI33551[] = {{&S38868[30], 6}, {NULL}};
const Segment SGI33552[] = {{&S38868[130], 3}, {NULL}};
const Segment SGI33553[] = {{&S29788[44], 25}, {NULL}};
const Segment SGI33554[] = {{&S38868[96], 5}, {NULL}};
const Segment SGI33555[] = {{&S29788[44], 36}, {NULL}};
const Segment SGI33556[] = {{&S29788[30], 6}, {NULL}};
const Segment SGI33557[] = {{&S29788[128], 3}, {NULL}};
const Segment SGI33558[] = {{&S29788[94], 5}, {NULL}};
const Segment SGI33559[] = {{&S29789[44], 26}, {NULL}};
const Segment SGI33560[] = {{&S29789[44], 37}, {NULL}};
const Segment SGI33561[] = {{&S29789[30], 6}, {NULL}};
const Segment SGI33562[] = {{&S29789[129], 3}, {NULL}};
const Segment SGI33563[] = {{&S29789[95], 5}, {NULL}};
const Segment SGI33564[] = {{&S29790[44], 25}, {NULL}};
const Segment SGI33565[] = {{&S29790[44], 36}, {NULL}};
const Segment SGI33566[] = {{&S29790[30], 6}, {NULL}};
const Segment SGI33567[] = {{&S29790[128], 3}, {NULL}};
const Segment SGI33568[] = {{&S29790[94], 5}, {NULL}};
const Segment SGI33569[] = {{&S29791[44], 25}, {NULL}};
const Segment SGI33570[] = {{&S29791[44], 36}, {NULL}};
const Segment SGI33571[] = {{&S29791[30], 6}, {NULL}};
const Segment SGI33572[] = {{&S29791[128], 3}, {NULL}};
const Segment SGI33573[] = {{&S29791[94], 5}, {NULL}};
const Segment SGI33574[] = {{&S29792[44], 26}, {NULL}};
const Segment SGI33575[] = {{&S29792[44], 37}, {NULL}};
const Segment SGI33576[] = {{&S29792[30], 6}, {NULL}};
const Segment SGI33577[] = {{&S29792[129], 3}, {NULL}};
const Segment SGI33578[] = {{&S29792[95], 5}, {NULL}};
const Segment SGI33579[] = {{&S29793[44], 26}, {NULL}};
const Segment SGI33580[] = {{&S29793[44], 37}, {NULL}};
const Segment SGI33581[] = {{&S29793[30], 6}, {NULL}};
const Segment SGI33582[] = {{&S29793[129], 3}, {NULL}};
const Segment SGI33583[] = {{&S29793[95], 5}, {NULL}};
const Segment SGI33584[] = {{&S38956[44], 7}, {NULL}};
const Segment SGI33585[] = {{&S38956[44], 19}, {NULL}};
const Segment SGI33586[] = {{&S38956[30], 6}, {NULL}};
const Segment SGI33587[] = {{&S38956[111], 3}, {NULL}};
const Segment SGI33588[] = {{&S38956[77], 5}, {NULL}};
const Segment SGI33589[] = {{&S29879[44], 7}, {NULL}};
const Segment SGI33590[] = {{&S29879[44], 24}, {NULL}};
const Segment SGI33591[] = {{&S29879[30], 6}, {NULL}};
const Segment SGI33592[] = {{&S29879[57], 3}, {NULL}};
const Segment SGI33593[] = {{&S29879[82], 5}, {NULL}};
const Segment SGI33594[] = {{&S29880[44], 19}, {NULL}};
const Segment SGI33595[] = {{&S29880[44], 36}, {NULL}};
const Segment SGI33596[] = {{&S29880[30], 6}, {NULL}};
const Segment SGI33597[] = {{&S29880[69], 3}, {NULL}};
const Segment SGI33598[] = {{&S29880[94], 5}, {NULL}};
const Segment SGI33599[] = {{&S29883[44], 19}, {NULL}};
const Segment SGI33600[] = {{&S29883[44], 36}, {NULL}};
const Segment SGI33601[] = {{&S29883[30], 6}, {NULL}};
const Segment SGI33602[] = {{&S29883[69], 3}, {NULL}};
const Segment SGI33603[] = {{&S29883[94], 5}, {NULL}};
const Segment SGI33604[] = {{&S29884[44], 19}, {NULL}};
const Segment SGI33605[] = {{&S29884[44], 36}, {NULL}};
const Segment SGI33606[] = {{&S29884[30], 6}, {NULL}};
const Segment SGI33607[] = {{&S29884[69], 3}, {NULL}};
const Segment SGI33608[] = {{&S29884[94], 5}, {NULL}};
const Segment SGI33609[] = {{&S29885[44], 19}, {NULL}};
const Segment SGI33610[] = {{&S29885[44], 36}, {NULL}};
const Segment SGI33611[] = {{&S29885[30], 6}, {NULL}};
const Segment SGI33612[] = {{&S29885[69], 3}, {NULL}};
const Segment SGI33613[] = {{&S29885[94], 5}, {NULL}};
const Segment SGI33614[] = {{&S29886[44], 19}, {NULL}};
const Segment SGI33615[] = {{&S29886[44], 36}, {NULL}};
const Segment SGI33616[] = {{&S29886[30], 6}, {NULL}};
const Segment SGI33617[] = {{&S29886[69], 3}, {NULL}};
const Segment SGI33618[] = {{&S29886[94], 5}, {NULL}};
const Segment SGI33619[] = {{&S29887[44], 19}, {NULL}};
const Segment SGI33620[] = {{&S29887[44], 36}, {NULL}};
const Segment SGI33621[] = {{&S29887[30], 6}, {NULL}};
const Segment SGI33622[] = {{&S29887[69], 3}, {NULL}};
const Segment SGI33623[] = {{&S29887[94], 5}, {NULL}};
const Segment SGI33624[] = {{&S29888[44], 19}, {NULL}};
const Segment SGI33625[] = {{&S29888[44], 36}, {NULL}};
const Segment SGI33626[] = {{&S29888[30], 6}, {NULL}};
const Segment SGI33627[] = {{&S29888[69], 3}, {NULL}};
const Segment SGI33628[] = {{&S29888[94], 5}, {NULL}};
const Segment SGI33629[] = {{&S29889[44], 7}, {NULL}};
const Segment SGI33630[] = {{&S29889[44], 24}, {NULL}};
const Segment SGI33631[] = {{&S29889[30], 6}, {NULL}};
const Segment SGI33632[] = {{&S29889[57], 3}, {NULL}};
const Segment SGI33633[] = {{&S29889[82], 5}, {NULL}};
const Segment SGI33634[] = {{&S29890[44], 19}, {NULL}};
const Segment SGI33635[] = {{&S29890[44], 36}, {NULL}};
const Segment SGI33636[] = {{&S29890[30], 6}, {NULL}};
const Segment SGI33637[] = {{&S29890[69], 3}, {NULL}};
const Segment SGI33638[] = {{&S29890[94], 5}, {NULL}};
const Segment SGI33639[] = {{&S29893[38], 7}, {NULL}};
const Segment SGI33640[] = {{&S29893[31], 6}, {NULL}};
const Segment SGI33641[] = {{&S29900[44], 19}, {NULL}};
const Segment SGI33642[] = {{&S29900[44], 36}, {NULL}};
const Segment SGI33643[] = {{&S29900[30], 6}, {NULL}};
const Segment SGI33644[] = {{&S29900[69], 3}, {NULL}};
const Segment SGI33645[] = {{&S29900[94], 5}, {NULL}};
const Segment SGI33646[] = {{&S29902[44], 7}, {NULL}};
const Segment SGI33647[] = {{&S29902[44], 24}, {NULL}};
const Segment SGI33648[] = {{&S29902[30], 6}, {NULL}};
const Segment SGI33649[] = {{&S29902[57], 3}, {NULL}};
const Segment SGI33650[] = {{&S29902[82], 5}, {NULL}};
const Segment SGI33651[] = {{&S29907[38], 7}, {NULL}};
const Segment SGI33652[] = {{&S29907[31], 6}, {NULL}};
const Segment SGI33653[] = {{&S29916[44], 7}, {NULL}};
const Segment SGI33654[] = {{&S29916[44], 24}, {NULL}};
const Segment SGI33655[] = {{&S29916[30], 6}, {NULL}};
const Segment SGI33656[] = {{&S29916[57], 3}, {NULL}};
const Segment SGI33657[] = {{&S29916[82], 5}, {NULL}};
const Segment SGI33658[] = {{&S29969[44], 10}, {NULL}};
const Segment SGI33659[] = {{&S29969[44], 27}, {NULL}};
const Segment SGI33660[] = {{&S29969[30], 6}, {NULL}};
const Segment SGI33661[] = {{&S29969[120], 3}, {NULL}};
const Segment SGI33662[] = {{&S29969[85], 6}, {NULL}};
const Segment SGI33663[] = {{&S28532[44], 11}, {NULL}};
const Segment SGI33664[] = {{&S28532[44], 22}, {NULL}};
const Segment SGI33665[] = {{&S28532[30], 6}, {NULL}};
const Segment SGI33666[] = {{&S28532[114], 3}, {NULL}};
const Segment SGI33667[] = {{&S28532[80], 5}, {NULL}};
const Segment SGI33668[] = {{&S28534[44], 12}, {NULL}};
const Segment SGI33669[] = {{&S28534[44], 23}, {NULL}};
const Segment SGI33670[] = {{&S28534[30], 6}, {NULL}};
const Segment SGI33671[] = {{&S28534[115], 3}, {NULL}};
const Segment SGI33672[] = {{&S28534[81], 5}, {NULL}};
const Segment SGI33673[] = {{&S28535[44], 15}, {NULL}};
const Segment SGI33674[] = {{&S28535[44], 26}, {NULL}};
const Segment SGI33675[] = {{&S28535[30], 6}, {NULL}};
const Segment SGI33676[] = {{&S28535[118], 3}, {NULL}};
const Segment SGI33677[] = {{&S28535[84], 5}, {NULL}};
const Segment SGI33678[] = {{&S28538[44], 12}, {NULL}};
const Segment SGI33679[] = {{&S28538[44], 24}, {NULL}};
const Segment SGI33680[] = {{&S30001[44], 9}, {NULL}};
const Segment SGI33681[] = {{&S28538[30], 6}, {NULL}};
const Segment SGI33682[] = {{&S30001[44], 20}, {NULL}};
const Segment SGI33683[] = {{&S30001[30], 6}, {NULL}};
const Segment SGI33684[] = {{&S28538[116], 3}, {NULL}};
const Segment SGI33685[] = {{&S28538[82], 5}, {NULL}};
const Segment SGI33686[] = {{&S30001[112], 3}, {NULL}};
const Segment SGI33687[] = {{&S30001[78], 5}, {NULL}};
const Segment SGI33688[] = {{&S28539[44], 12}, {NULL}};
const Segment SGI33689[] = {{&S28539[44], 23}, {NULL}};
const Segment SGI33690[] = {{&S28539[30], 6}, {NULL}};
const Segment SGI33691[] = {{&S28539[115], 3}, {NULL}};
const Segment SGI33692[] = {{&S28539[81], 5}, {NULL}};
const Segment SGI33693[] = {{&S27723[44], 12}, {NULL}};
const Segment SGI33694[] = {{&S27723[44], 23}, {NULL}};
const Segment SGI33695[] = {{&S27723[30], 6}, {NULL}};
const Segment SGI33696[] = {{&S27723[115], 3}, {NULL}};
const Segment SGI33697[] = {{&S27723[81], 5}, {NULL}};
const Segment SGI33698[] = {{&S27724[44], 18}, {NULL}};
const Segment SGI33699[] = {{&S27724[44], 29}, {NULL}};
const Segment SGI33700[] = {{&S27724[30], 6}, {NULL}};
const Segment SGI33701[] = {{&S27724[121], 3}, {NULL}};
const Segment SGI33702[] = {{&S27724[87], 5}, {NULL}};
const Segment SGI33703[] = {{&S27731[44], 14}, {NULL}};
const Segment SGI33704[] = {{&S27731[44], 25}, {NULL}};
const Segment SGI33705[] = {{&S27731[30], 6}, {NULL}};
const Segment SGI33706[] = {{&S27731[117], 3}, {NULL}};
const Segment SGI33707[] = {{&S27731[83], 5}, {NULL}};
const Segment SGI33708[] = {{&S27732[44], 12}, {NULL}};
const Segment SGI33709[] = {{&S27732[44], 23}, {NULL}};
const Segment SGI33710[] = {{&S27732[30], 6}, {NULL}};
const Segment SGI33711[] = {{&S27732[115], 3}, {NULL}};
const Segment SGI33712[] = {{&S27732[81], 5}, {NULL}};
const Segment SGI33713[] = {{&S27733[44], 12}, {NULL}};
const Segment SGI33714[] = {{&S27733[44], 35}, {NULL}};
const Segment SGI33715[] = {{&S27733[30], 6}, {NULL}};
const Segment SGI33716[] = {{&S27733[127], 3}, {NULL}};
const Segment SGI33717[] = {{&S27733[93], 5}, {NULL}};
const Segment SGI33718[] = {{&S30002[44], 8}, {NULL}};
const Segment SGI33719[] = {{&S30002[44], 19}, {NULL}};
const Segment SGI33720[] = {{&S30002[30], 6}, {NULL}};
const Segment SGI33721[] = {{&S30002[111], 3}, {NULL}};
const Segment SGI33722[] = {{&S30002[77], 5}, {NULL}};
const Segment SGI33723[] = {{&S30003[44], 9}, {NULL}};
const Segment SGI33724[] = {{&S30003[44], 20}, {NULL}};
const Segment SGI33725[] = {{&S30003[30], 6}, {NULL}};
const Segment SGI33726[] = {{&S30003[112], 3}, {NULL}};
const Segment SGI33727[] = {{&S30003[78], 5}, {NULL}};
const Segment SGI33728[] = {{&S27737[44], 14}, {NULL}};
const Segment SGI33729[] = {{&S27737[44], 25}, {NULL}};
const Segment SGI33730[] = {{&S27737[30], 6}, {NULL}};
const Segment SGI33731[] = {{&S27737[117], 3}, {NULL}};
const Segment SGI33732[] = {{&S27737[83], 5}, {NULL}};
const Segment SGI33733[] = {{&S30004[43], 9}, {NULL}};
const Segment SGI33734[] = {{&S30004[43], 20}, {NULL}};
const Segment SGI33735[] = {{&S30004[111], 3}, {NULL}};
const Segment SGI33736[] = {{&S30004[77], 5}, {NULL}};
const Segment SGI33737[] = {{&S30005[44], 8}, {NULL}};
const Segment SGI33738[] = {{&S30005[44], 19}, {NULL}};
const Segment SGI33739[] = {{&S30005[30], 6}, {NULL}};
const Segment SGI33740[] = {{&S30005[111], 3}, {NULL}};
const Segment SGI33741[] = {{&S30005[77], 5}, {NULL}};
const Segment SGI33742[] = {{&S30006[43], 9}, {NULL}};
const Segment SGI33743[] = {{&S30006[43], 20}, {NULL}};
const Segment SGI33744[] = {{&S30006[30], 5}, {NULL}};
const Segment SGI33745[] = {{&S30006[111], 3}, {NULL}};
const Segment SGI33746[] = {{&S30006[77], 5}, {NULL}};
const Segment SGI33747[] = {{&S27743[44], 17}, {NULL}};
const Segment SGI33748[] = {{&S27743[44], 28}, {NULL}};
const Segment SGI33749[] = {{&S27743[30], 6}, {NULL}};
const Segment SGI33750[] = {{&S27743[120], 3}, {NULL}};
const Segment SGI33751[] = {{&S27743[86], 5}, {NULL}};
const Segment SGI33752[] = {{&S30009[38], 9}, {NULL}};
const Segment SGI33753[] = {{&S30009[31], 6}, {NULL}};
const Segment SGI33754[] = {{&S30015[44], 9}, {NULL}};
const Segment SGI33755[] = {{&S30015[44], 21}, {NULL}};
const Segment SGI33756[] = {{&S30015[30], 6}, {NULL}};
const Segment SGI33757[] = {{&S30015[113], 3}, {NULL}};
const Segment SGI33758[] = {{&S30015[79], 5}, {NULL}};
const Segment SGI33759[] = {{&S30016[44], 9}, {NULL}};
const Segment SGI33760[] = {{&S30016[44], 20}, {NULL}};
const Segment SGI33761[] = {{&S30016[30], 6}, {NULL}};
const Segment SGI33762[] = {{&S30016[114], 3}, {NULL}};
const Segment SGI33763[] = {{&S30016[78], 7}, {NULL}};
const Segment SGI33764[] = {{&S30016[134], 8}, {NULL}};
const Segment SGI33765[] = {{&S27758[44], 17}, {NULL}};
const Segment SGI33766[] = {{&S27758[44], 28}, {NULL}};
const Segment SGI33767[] = {{&S27758[30], 6}, {NULL}};
const Segment SGI33768[] = {{&S27758[120], 3}, {NULL}};
const Segment SGI33769[] = {{&S27758[86], 5}, {NULL}};
const Segment SGI33770[] = {{&S27761[38], 12}, {NULL}};
const Segment SGI33771[] = {{&S27761[31], 6}, {NULL}};
const Segment SGI33772[] = {{&S27763[44], 12}, {NULL}};
const Segment SGI33773[] = {{&S27763[44], 29}, {NULL}};
const Segment SGI33774[] = {{&S27763[30], 6}, {NULL}};
const Segment SGI33775[] = {{&S27763[121], 3}, {NULL}};
const Segment SGI33776[] = {{&S27763[87], 5}, {NULL}};
const Segment SGI33777[] = {{&S30027[42], 9}, {NULL}};
const Segment SGI33778[] = {{&S30027[30], 6}, {NULL}};
const Segment SGI33779[] = {{&S30027[110], 3}, {NULL}};
const Segment SGI33780[] = {{&S30027[76], 5}, {NULL}};
const Segment SGI33781[] = {{&S27765[44], 12}, {NULL}};
const Segment SGI33782[] = {{&S27765[44], 35}, {NULL}};
const Segment SGI33783[] = {{&S27765[30], 6}, {NULL}};
const Segment SGI33784[] = {{&S27765[127], 3}, {NULL}};
const Segment SGI33785[] = {{&S27765[93], 5}, {NULL}};
const Segment SGI33786[] = {{&S27778[44], 12}, {NULL}};
const Segment SGI33787[] = {{&S27778[44], 35}, {NULL}};
const Segment SGI33788[] = {{&S27778[30], 6}, {NULL}};
const Segment SGI33789[] = {{&S27778[127], 3}, {NULL}};
const Segment SGI33790[] = {{&S27778[93], 5}, {NULL}};
const Segment SGI33791[] = {{&S27779[44], 12}, {NULL}};
const Segment SGI33792[] = {{&S27779[44], 35}, {NULL}};
const Segment SGI33793[] = {{&S27779[30], 6}, {NULL}};
const Segment SGI33794[] = {{&S27779[127], 3}, {NULL}};
const Segment SGI33795[] = {{&S27779[93], 5}, {NULL}};
const Segment SGI33796[] = {{&S27781[44], 12}, {NULL}};
const Segment SGI33797[] = {{&S27781[44], 23}, {NULL}};
const Segment SGI33798[] = {{&S27781[30], 6}, {NULL}};
const Segment SGI33799[] = {{&S27781[115], 3}, {NULL}};
const Segment SGI33800[] = {{&S27781[81], 5}, {NULL}};
const Segment SGI33801[] = {{&S27784[44], 8}, {NULL}};
const Segment SGI33802[] = {{&S27784[44], 19}, {NULL}};
const Segment SGI33803[] = {{&S27784[30], 6}, {NULL}};
const Segment SGI33804[] = {{&S27784[111], 3}, {NULL}};
const Segment SGI33805[] = {{&S27784[77], 5}, {NULL}};
const Segment SGI33806[] = {{&S27793[44], 7}, {NULL}};
const Segment SGI33807[] = {{&S27793[44], 18}, {NULL}};
const Segment SGI33808[] = {{&S27793[30], 6}, {NULL}};
const Segment SGI33809[] = {{&S27793[110], 3}, {NULL}};
const Segment SGI33810[] = {{&S27793[76], 5}, {NULL}};
const Segment SGI33811[] = {{&S28672[44], 13}, {NULL}};
const Segment SGI33812[] = {{&S28672[44], 22}, {NULL}};
const Segment SGI33813[] = {{&S28672[30], 6}, {NULL}};
const Segment SGI33814[] = {{&S28672[114], 3}, {NULL}};
const Segment SGI33815[] = {{&S28672[80], 5}, {NULL}};
const Segment SGI33816[] = {{&S28674[44], 13}, {NULL}};
const Segment SGI33817[] = {{&S28674[44], 22}, {NULL}};
const Segment SGI33818[] = {{&S28674[30], 6}, {NULL}};
const Segment SGI33819[] = {{&S28674[114], 3}, {NULL}};
const Segment SGI33820[] = {{&S28674[80], 5}, {NULL}};
const Segment SGI33821[] = {{&S28768[44], 18}, {NULL}};
const Segment SGI33822[] = {{&S28768[44], 35}, {NULL}};
const Segment SGI33823[] = {{&S28768[30], 6}, {NULL}};
const Segment SGI33824[] = {{&S28768[127], 3}, {NULL}};
const Segment SGI33825[] = {{&S28768[93], 5}, {NULL}};
const Segment SGI33826[] = {{&S28781[38], 13}, {NULL}};
const Segment SGI33827[] = {{&S28781[31], 6}, {NULL}};
const Segment SGI33828[] = {{&S28782[44], 14}, {NULL}};
const Segment SGI33829[] = {{&S28782[44], 31}, {NULL}};
const Segment SGI33830[] = {{&S28782[30], 6}, {NULL}};
const Segment SGI33831[] = {{&S28782[123], 3}, {NULL}};
const Segment SGI33832[] = {{&S28782[89], 5}, {NULL}};
const Segment SGI33833[] = {{&S28783[44], 9}, {NULL}};
const Segment SGI33834[] = {{&S28783[44], 26}, {NULL}};
const Segment SGI33835[] = {{&S28783[30], 6}, {NULL}};
const Segment SGI33836[] = {{&S28783[118], 3}, {NULL}};
const Segment SGI33837[] = {{&S28783[84], 5}, {NULL}};
const Segment SGI33838[] = {{&S28784[44], 15}, {NULL}};
const Segment SGI33839[] = {{&S28784[44], 32}, {NULL}};
const Segment SGI33840[] = {{&S28784[30], 6}, {NULL}};
const Segment SGI33841[] = {{&S28784[124], 3}, {NULL}};
const Segment SGI33842[] = {{&S28784[90], 5}, {NULL}};
const Segment SGI33843[] = {{&S28785[44], 10}, {NULL}};
const Segment SGI33844[] = {{&S28785[44], 27}, {NULL}};
const Segment SGI33845[] = {{&S28785[30], 6}, {NULL}};
const Segment SGI33846[] = {{&S28785[119], 3}, {NULL}};
const Segment SGI33847[] = {{&S28785[85], 5}, {NULL}};
const Segment SGI33848[] = {{&S28786[38], 9}, {NULL}};
const Segment SGI33849[] = {{&S28786[31], 6}, {NULL}};
const Segment SGI33850[] = {{&S28787[38], 10}, {NULL}};
const Segment SGI33851[] = {{&S28787[31], 6}, {NULL}};
const Segment SGI33852[] = {{&S28788[44], 15}, {NULL}};
const Segment SGI33853[] = {{&S28788[44], 38}, {NULL}};
const Segment SGI33854[] = {{&S28788[30], 6}, {NULL}};
const Segment SGI33855[] = {{&S28788[130], 3}, {NULL}};
const Segment SGI33856[] = {{&S28788[96], 5}, {NULL}};
const Segment SGI33857[] = {{&S28789[44], 14}, {NULL}};
const Segment SGI33858[] = {{&S28789[44], 37}, {NULL}};
const Segment SGI33859[] = {{&S28789[30], 6}, {NULL}};
const Segment SGI33860[] = {{&S28789[129], 3}, {NULL}};
const Segment SGI33861[] = {{&S28789[95], 5}, {NULL}};
const Segment SGI33862[] = {{&S28790[44], 15}, {NULL}};
const Segment SGI33863[] = {{&S28790[44], 38}, {NULL}};
const Segment SGI33864[] = {{&S28790[30], 6}, {NULL}};
const Segment SGI33865[] = {{&S28790[130], 3}, {NULL}};
const Segment SGI33866[] = {{&S28790[96], 5}, {NULL}};
const Segment SGI33867[] = {{&S28791[44], 14}, {NULL}};
const Segment SGI33868[] = {{&S28791[44], 37}, {NULL}};
const Segment SGI33869[] = {{&S28791[30], 6}, {NULL}};
const Segment SGI33870[] = {{&S28791[129], 3}, {NULL}};
const Segment SGI33871[] = {{&S28791[95], 5}, {NULL}};
const Segment SGI33872[] = {{&S27929[58], 18}, {NULL}};
const Segment SGI33873[] = {{&S27929[58], 36}, {NULL}};
const Segment SGI33874[] = {{&S27929[30], 20}, {NULL}};
const Segment SGI33875[] = {{&S27929[142], 3}, {NULL}};
const Segment SGI33876[] = {{&S27929[108], 5}, {NULL}};
const Segment SGI33877[] = {{&S27945[44], 6}, {NULL}};
const Segment SGI33878[] = {{&S27945[44], 24}, {NULL}};
const Segment SGI33879[] = {{&S27945[30], 6}, {NULL}};
const Segment SGI33880[] = {{&S27945[116], 3}, {NULL}};
const Segment SGI33881[] = {{&S27945[82], 5}, {NULL}};
const Segment SGI33882[] = {{&S27979[44], 10}, {NULL}};
const Segment SGI33883[] = {{&S27979[44], 33}, {NULL}};
const Segment SGI33884[] = {{&S27979[30], 6}, {NULL}};
const Segment SGI33885[] = {{&S27979[125], 3}, {NULL}};
const Segment SGI33886[] = {{&S27979[91], 5}, {NULL}};
const Segment SGI33887[] = {{&S28916[44], 10}, {NULL}};
const Segment SGI33888[] = {{&S28916[44], 27}, {NULL}};
const Segment SGI33889[] = {{&S28916[30], 6}, {NULL}};
const Segment SGI33890[] = {{&S27980[44], 14}, {NULL}};
const Segment SGI33891[] = {{&S27980[44], 27}, {NULL}};
const Segment SGI33892[] = {{&S27980[30], 6}, {NULL}};
const Segment SGI33893[] = {{&S28916[119], 3}, {NULL}};
const Segment SGI33894[] = {{&S28916[85], 5}, {NULL}};
const Segment SGI33895[] = {{&S27980[119], 3}, {NULL}};
const Segment SGI33896[] = {{&S27980[85], 5}, {NULL}};
const Segment SGI33897[] = {{&S27981[44], 10}, {NULL}};
const Segment SGI33898[] = {{&S27981[44], 33}, {NULL}};
const Segment SGI33899[] = {{&S27981[30], 6}, {NULL}};
const Segment SGI33900[] = {{&S27981[125], 3}, {NULL}};
const Segment SGI33901[] = {{&S27981[91], 5}, {NULL}};
const Segment SGI33902[] = {{&S28919[38], 10}, {NULL}};
const Segment SGI33903[] = {{&S28919[31], 6}, {NULL}};
const Segment SGI33904[] = {{&S27982[38], 10}, {NULL}};
const Segment SGI33905[] = {{&S27982[31], 6}, {NULL}};
const Segment SGI33906[] = {{&S27983[43], 10}, {NULL}};
const Segment SGI33907[] = {{&S27983[43], 33}, {NULL}};
const Segment SGI33908[] = {{&S27983[30], 5}, {NULL}};
const Segment SGI33909[] = {{&S27983[124], 3}, {NULL}};
const Segment SGI33910[] = {{&S27983[90], 5}, {NULL}};
const Segment SGI33911[] = {{&S27984[44], 10}, {NULL}};
const Segment SGI33912[] = {{&S27984[44], 33}, {NULL}};
const Segment SGI33913[] = {{&S27984[30], 6}, {NULL}};
const Segment SGI33914[] = {{&S27984[125], 3}, {NULL}};
const Segment SGI33915[] = {{&S27984[91], 5}, {NULL}};
const Segment SGI33916[] = {{&S27985[44], 9}, {NULL}};
const Segment SGI33917[] = {{&S27985[44], 32}, {NULL}};
const Segment SGI33918[] = {{&S27985[30], 6}, {NULL}};
const Segment SGI33919[] = {{&S27985[124], 3}, {NULL}};
const Segment SGI33920[] = {{&S27985[90], 5}, {NULL}};
const Segment SGI33921[] = {{&S27986[43], 10}, {NULL}};
const Segment SGI33922[] = {{&S27986[43], 33}, {NULL}};
const Segment SGI33923[] = {{&S27986[124], 3}, {NULL}};
const Segment SGI33924[] = {{&S27986[90], 5}, {NULL}};
const Segment SGI33925[] = {{&S27987[44], 9}, {NULL}};
const Segment SGI33926[] = {{&S27987[44], 32}, {NULL}};
const Segment SGI33927[] = {{&S27987[30], 6}, {NULL}};
const Segment SGI33928[] = {{&S27987[124], 3}, {NULL}};
const Segment SGI33929[] = {{&S27987[90], 5}, {NULL}};
const Segment SGI33930[] = {{&S28930[44], 11}, {NULL}};
const Segment SGI33931[] = {{&S28930[44], 28}, {NULL}};
const Segment SGI33932[] = {{&S28930[30], 6}, {NULL}};
const Segment SGI33933[] = {{&S28930[120], 3}, {NULL}};
const Segment SGI33934[] = {{&S28930[86], 5}, {NULL}};
const Segment SGI33935[] = {{&S27988[44], 10}, {NULL}};
const Segment SGI33936[] = {{&S27988[44], 33}, {NULL}};
const Segment SGI33937[] = {{&S27988[30], 6}, {NULL}};
const Segment SGI33938[] = {{&S27988[125], 3}, {NULL}};
const Segment SGI33939[] = {{&S27988[91], 5}, {NULL}};
const Segment SGI33940[] = {{&S27989[44], 9}, {NULL}};
const Segment SGI33941[] = {{&S27989[44], 32}, {NULL}};
const Segment SGI33942[] = {{&S27989[30], 6}, {NULL}};
const Segment SGI33943[] = {{&S27989[124], 3}, {NULL}};
const Segment SGI33944[] = {{&S27989[90], 5}, {NULL}};
const Segment SGI33945[] = {{&S27991[38], 14}, {NULL}};
const Segment SGI33946[] = {{&S27991[31], 6}, {NULL}};
const Segment SGI33947[] = {{&S27994[42], 10}, {NULL}};
const Segment SGI33948[] = {{&S27994[30], 6}, {NULL}};
const Segment SGI33949[] = {{&S27994[123], 3}, {NULL}};
const Segment SGI33950[] = {{&S27994[89], 5}, {NULL}};
const Segment SGI33951[] = {{&S27996[44], 10}, {NULL}};
const Segment SGI33952[] = {{&S27996[44], 33}, {NULL}};
const Segment SGI33953[] = {{&S27996[30], 6}, {NULL}};
const Segment SGI33954[] = {{&S27996[91], 6}, {NULL}};
const Segment SGI33955[] = {{&S28054[44], 12}, {NULL}};
const Segment SGI33956[] = {{&S28054[44], 23}, {NULL}};
const Segment SGI33957[] = {{&S28054[30], 6}, {NULL}};
const Segment SGI33958[] = {{&S28054[115], 3}, {NULL}};
const Segment SGI33959[] = {{&S28054[81], 5}, {NULL}};
const Segment SGI33960[] = {{&S28072[44], 9}, {NULL}};
const Segment SGI33961[] = {{&S28072[44], 20}, {NULL}};
const Segment SGI33962[] = {{&S28072[30], 6}, {NULL}};
const Segment SGI33963[] = {{&S28072[112], 3}, {NULL}};
const Segment SGI33964[] = {{&S28072[78], 5}, {NULL}};
const Segment SGI33965[] = {{&S28073[44], 12}, {NULL}};
const Segment SGI33966[] = {{&S28073[44], 23}, {NULL}};
const Segment SGI33967[] = {{&S28073[30], 6}, {NULL}};
const Segment SGI33968[] = {{&S28073[115], 3}, {NULL}};
const Segment SGI33969[] = {{&S28073[81], 5}, {NULL}};
const Segment SGI33970[] = {{&S28074[44], 12}, {NULL}};
const Segment SGI33971[] = {{&S28074[44], 23}, {NULL}};
const Segment SGI33972[] = {{&S28074[30], 6}, {NULL}};
const Segment SGI33973[] = {{&S28074[115], 3}, {NULL}};
const Segment SGI33974[] = {{&S28074[81], 5}, {NULL}};
const Segment SGI33975[] = {{&S28080[38], 9}, {NULL}};
const Segment SGI33976[] = {{&S28080[31], 6}, {NULL}};
const Segment SGI33977[] = {{&S28209[44], 10}, {NULL}};
const Segment SGI33978[] = {{&S28209[44], 27}, {NULL}};
const Segment SGI33979[] = {{&S28209[30], 6}, {NULL}};
const Segment SGI33980[] = {{&S28209[119], 3}, {NULL}};
const Segment SGI33981[] = {{&S28209[85], 5}, {NULL}};
const Segment SGI33982[] = {{&S28210[44], 10}, {NULL}};
const Segment SGI33983[] = {{&S28210[44], 34}, {NULL}};
const Segment SGI33984[] = {{&S28210[30], 6}, {NULL}};
const Segment SGI33985[] = {{&S28210[126], 3}, {NULL}};
const Segment SGI33986[] = {{&S28210[92], 5}, {NULL}};
const Segment SGI33987[] = {{&S28211[44], 10}, {NULL}};
const Segment SGI33988[] = {{&S28211[44], 33}, {NULL}};
const Segment SGI33989[] = {{&S28211[30], 6}, {NULL}};
const Segment SGI33990[] = {{&S28211[125], 3}, {NULL}};
const Segment SGI33991[] = {{&S28211[91], 5}, {NULL}};
const Segment SGI33992[] = {{&S28213[44], 11}, {NULL}};
const Segment SGI33993[] = {{&S28213[44], 28}, {NULL}};
const Segment SGI33994[] = {{&S28213[30], 6}, {NULL}};
const Segment SGI33995[] = {{&S28213[120], 3}, {NULL}};
const Segment SGI33996[] = {{&S28213[86], 5}, {NULL}};
const Segment SGI33997[] = {{&S28214[38], 10}, {NULL}};
const Segment SGI33998[] = {{&S28214[31], 6}, {NULL}};
const Segment SGI33999[] = {{&S28215[44], 9}, {NULL}};
const Segment SGI34000[] = {{&S28215[44], 26}, {NULL}};
const Segment SGI34001[] = {{&S28215[30], 6}, {NULL}};
const Segment SGI34002[] = {{&S28215[118], 3}, {NULL}};
const Segment SGI34003[] = {{&S28215[84], 5}, {NULL}};
const Segment SGI34004[] = {{&S28216[43], 10}, {NULL}};
const Segment SGI34005[] = {{&S28216[43], 27}, {NULL}};
const Segment SGI34006[] = {{&S28216[30], 5}, {NULL}};
const Segment SGI34007[] = {{&S28216[118], 3}, {NULL}};
const Segment SGI34008[] = {{&S28216[84], 5}, {NULL}};
const Segment SGI34009[] = {{&S28217[43], 10}, {NULL}};
const Segment SGI34010[] = {{&S28217[43], 27}, {NULL}};
const Segment SGI34011[] = {{&S28217[118], 3}, {NULL}};
const Segment SGI34012[] = {{&S28217[84], 5}, {NULL}};
const Segment SGI34013[] = {{&S28218[44], 9}, {NULL}};
const Segment SGI34014[] = {{&S28218[44], 26}, {NULL}};
const Segment SGI34015[] = {{&S28218[30], 6}, {NULL}};
const Segment SGI34016[] = {{&S28218[118], 3}, {NULL}};
const Segment SGI34017[] = {{&S28218[84], 5}, {NULL}};
const Segment SGI34018[] = {{&S28219[44], 10}, {NULL}};
const Segment SGI34019[] = {{&S28219[44], 27}, {NULL}};
const Segment SGI34020[] = {{&S28219[30], 6}, {NULL}};
const Segment SGI34021[] = {{&S28219[119], 3}, {NULL}};
const Segment SGI34022[] = {{&S28219[85], 5}, {NULL}};
const Segment SGI34023[] = {{&S28220[44], 10}, {NULL}};
const Segment SGI34024[] = {{&S28220[44], 34}, {NULL}};
const Segment SGI34025[] = {{&S28220[30], 6}, {NULL}};
const Segment SGI34026[] = {{&S28220[126], 3}, {NULL}};
const Segment SGI34027[] = {{&S28220[92], 5}, {NULL}};
const Segment SGI34028[] = {{&S28222[38], 11}, {NULL}};
const Segment SGI34029[] = {{&S28222[31], 6}, {NULL}};
const Segment SGI34030[] = {{&S28230[44], 10}, {NULL}};
const Segment SGI34031[] = {{&S28230[44], 26}, {NULL}};
const Segment SGI34032[] = {{&S28230[30], 6}, {NULL}};
const Segment SGI34033[] = {{&S28230[118], 3}, {NULL}};
const Segment SGI34034[] = {{&S28230[84], 5}, {NULL}};
const Segment SGI34035[] = {{&S28233[42], 11}, {NULL}};
const Segment SGI34036[] = {{&S28233[30], 6}, {NULL}};
const Segment SGI34037[] = {{&S28233[118], 3}, {NULL}};
const Segment SGI34038[] = {{&S28233[84], 5}, {NULL}};
const Segment SGI34039[] = {{&S28234[44], 10}, {NULL}};
const Segment SGI34040[] = {{&S28234[44], 27}, {NULL}};
const Segment SGI34041[] = {{&S28234[30], 6}, {NULL}};
const Segment SGI34042[] = {{&S28234[88], 6}, {NULL}};
const Segment SGI34043[] = {{&S34957[44], 9}, {NULL}};
const Segment SGI34044[] = {{&S34957[44], 20}, {NULL}};
const Segment SGI34045[] = {{&S34957[30], 6}, {NULL}};
const Segment SGI34046[] = {{&S34957[112], 3}, {NULL}};
const Segment SGI34047[] = {{&S34957[78], 5}, {NULL}};
const Segment SGI34048[] = {{&S30600[44], 11}, {NULL}};
const Segment SGI34049[] = {{&S30600[44], 22}, {NULL}};
const Segment SGI34050[] = {{&S30600[30], 6}, {NULL}};
const Segment SGI34051[] = {{&S30600[114], 3}, {NULL}};
const Segment SGI34052[] = {{&S30600[80], 5}, {NULL}};
const Segment SGI34053[] = {{&S30610[38], 11}, {NULL}};
const Segment SGI34054[] = {{&S30610[31], 6}, {NULL}};
const Segment SGI34055[] = {{&S30622[44], 11}, {NULL}};
const Segment SGI34056[] = {{&S30622[44], 22}, {NULL}};
const Segment SGI34057[] = {{&S30622[30], 6}, {NULL}};
const Segment SGI34058[] = {{&S30622[114], 3}, {NULL}};
const Segment SGI34059[] = {{&S30622[80], 5}, {NULL}};
const Segment SGI34060[] = {{&S30635[44], 11}, {NULL}};
const Segment SGI34061[] = {{&S30635[44], 22}, {NULL}};
const Segment SGI34062[] = {{&S30635[30], 6}, {NULL}};
const Segment SGI34063[] = {{&S30635[113], 3}, {NULL}};
const Segment SGI34064[] = {{&S30635[80], 5}, {NULL}};
const Segment SGI34065[] = {{&S35054[44], 9}, {NULL}};
const Segment SGI34066[] = {{&S35054[44], 20}, {NULL}};
const Segment SGI34067[] = {{&S35054[30], 6}, {NULL}};
const Segment SGI34068[] = {{&S35054[112], 3}, {NULL}};
const Segment SGI34069[] = {{&S35054[78], 5}, {NULL}};
const Segment SGI34070[] = {{&S35079[38], 9}, {NULL}};
const Segment SGI34071[] = {{&S35079[31], 6}, {NULL}};
const Segment SGI34072[] = {{&S35080[44], 17}, {NULL}};
const Segment SGI34073[] = {{&S35080[44], 28}, {NULL}};
const Segment SGI34074[] = {{&S35080[30], 6}, {NULL}};
const Segment SGI34075[] = {{&S35080[120], 3}, {NULL}};
const Segment SGI34076[] = {{&S35080[86], 5}, {NULL}};
const Segment SGI34077[] = {{&S35081[42], 9}, {NULL}};
const Segment SGI34078[] = {{&S35081[30], 6}, {NULL}};
const Segment SGI34079[] = {{&S35081[110], 3}, {NULL}};
const Segment SGI34080[] = {{&S35081[76], 5}, {NULL}};
const Segment SGI34081[] = {{&S39972[44], 11}, {NULL}};
const Segment SGI34082[] = {{&S39972[44], 23}, {NULL}};
const Segment SGI34083[] = {{&S39972[30], 6}, {NULL}};
const Segment SGI34084[] = {{&S39972[115], 3}, {NULL}};
const Segment SGI34085[] = {{&S39972[81], 5}, {NULL}};
const Segment SGI34086[] = {{&S28536[44], 16}, {NULL}};
const Segment SGI34087[] = {{&S28536[44], 27}, {NULL}};
const Segment SGI34088[] = {{&S28536[30], 6}, {NULL}};
const Segment SGI34089[] = {{&S28536[119], 3}, {NULL}};
const Segment SGI34090[] = {{&S28536[85], 5}, {NULL}};
const Segment SGI34091[] = {{&S28537[44], 15}, {NULL}};
const Segment SGI34092[] = {{&S28537[44], 26}, {NULL}};
const Segment SGI34093[] = {{&S28537[30], 6}, {NULL}};
const Segment SGI34094[] = {{&S28537[118], 3}, {NULL}};
const Segment SGI34095[] = {{&S28537[84], 5}, {NULL}};
const Segment SGI34096[] = {{&S39985[44], 11}, {NULL}};
const Segment SGI34097[] = {{&S39985[44], 22}, {NULL}};
const Segment SGI34098[] = {{&S39985[30], 6}, {NULL}};
const Segment SGI34099[] = {{&S28540[44], 12}, {NULL}};
const Segment SGI34100[] = {{&S28540[44], 23}, {NULL}};
const Segment SGI34101[] = {{&S28540[30], 6}, {NULL}};
const Segment SGI34102[] = {{&S39985[114], 3}, {NULL}};
const Segment SGI34103[] = {{&S39985[80], 5}, {NULL}};
const Segment SGI34104[] = {{&S28540[115], 3}, {NULL}};
const Segment SGI34105[] = {{&S28540[81], 5}, {NULL}};
const Segment SGI34106[] = {{&S28541[38], 12}, {NULL}};
const Segment SGI34107[] = {{&S28541[31], 6}, {NULL}};
const Segment SGI34108[] = {{&S28542[44], 12}, {NULL}};
const Segment SGI34109[] = {{&S28542[44], 23}, {NULL}};
const Segment SGI34110[] = {{&S28542[30], 6}, {NULL}};
const Segment SGI34111[] = {{&S28542[115], 3}, {NULL}};
const Segment SGI34112[] = {{&S28542[81], 5}, {NULL}};
const Segment SGI34113[] = {{&S28543[44], 15}, {NULL}};
const Segment SGI34114[] = {{&S28543[44], 26}, {NULL}};
const Segment SGI34115[] = {{&S28543[30], 6}, {NULL}};
const Segment SGI34116[] = {{&S28543[118], 3}, {NULL}};
const Segment SGI34117[] = {{&S28543[84], 5}, {NULL}};
const Segment SGI34118[] = {{&S28547[44], 11}, {NULL}};
const Segment SGI34119[] = {{&S28547[44], 22}, {NULL}};
const Segment SGI34120[] = {{&S28547[30], 6}, {NULL}};
const Segment SGI34121[] = {{&S28547[114], 3}, {NULL}};
const Segment SGI34122[] = {{&S28547[80], 5}, {NULL}};
const Segment SGI34123[] = {{&S35053[42], 9}, {NULL}};
const Segment SGI34124[] = {{&S35053[42], 20}, {NULL}};
const Segment SGI34125[] = {{&S35053[30], 4}, {NULL}};
const Segment SGI34126[] = {{&S35053[110], 3}, {NULL}};
const Segment SGI34127[] = {{&S35053[76], 5}, {NULL}};
const Segment SGI34128[] = {{&S28552[44], 15}, {NULL}};
const Segment SGI34129[] = {{&S28552[44], 26}, {NULL}};
const Segment SGI34130[] = {{&S28552[30], 6}, {NULL}};
const Segment SGI34131[] = {{&S28552[118], 3}, {NULL}};
const Segment SGI34132[] = {{&S28552[84], 5}, {NULL}};
const Segment SGI34133[] = {{&S28553[38], 16}, {NULL}};
const Segment SGI34134[] = {{&S28553[31], 6}, {NULL}};
const Segment SGI34135[] = {{&S28555[44], 15}, {NULL}};
const Segment SGI34136[] = {{&S28555[44], 26}, {NULL}};
const Segment SGI34137[] = {{&S28555[30], 6}, {NULL}};
const Segment SGI34138[] = {{&S28555[118], 3}, {NULL}};
const Segment SGI34139[] = {{&S28555[84], 5}, {NULL}};
const Segment SGI34140[] = {{&S35082[42], 9}, {NULL}};
const Segment SGI34141[] = {{&S35082[42], 21}, {NULL}};
const Segment SGI34142[] = {{&S35082[30], 4}, {NULL}};
const Segment SGI34143[] = {{&S35082[111], 3}, {NULL}};
const Segment SGI34144[] = {{&S35082[77], 5}, {NULL}};
const Segment SGI34145[] = {{&S35083[42], 9}, {NULL}};
const Segment SGI34146[] = {{&S35083[42], 20}, {NULL}};
const Segment SGI34147[] = {{&S35083[30], 4}, {NULL}};
const Segment SGI34148[] = {{&S35083[110], 3}, {NULL}};
const Segment SGI34149[] = {{&S35083[76], 5}, {NULL}};
const Segment SGI34150[] = {{&S28583[44], 11}, {NULL}};
const Segment SGI34151[] = {{&S28583[44], 31}, {NULL}};
const Segment SGI34152[] = {{&S28583[30], 6}, {NULL}};
const Segment SGI34153[] = {{&S28583[123], 3}, {NULL}};
const Segment SGI34154[] = {{&S28583[89], 5}, {NULL}};
const Segment SGI34155[] = {{&S28584[44], 11}, {NULL}};
const Segment SGI34156[] = {{&S28584[44], 32}, {NULL}};
const Segment SGI34157[] = {{&S28584[30], 6}, {NULL}};
const Segment SGI34158[] = {{&S28584[124], 3}, {NULL}};
const Segment SGI34159[] = {{&S28584[90], 5}, {NULL}};
const Segment SGI34160[] = {{&S28585[44], 11}, {NULL}};
const Segment SGI34161[] = {{&S28585[44], 32}, {NULL}};
const Segment SGI34162[] = {{&S28585[30], 6}, {NULL}};
const Segment SGI34163[] = {{&S28585[124], 3}, {NULL}};
const Segment SGI34164[] = {{&S28585[90], 5}, {NULL}};
const Segment SGI34165[] = {{&S28586[44], 7}, {NULL}};
const Segment SGI34166[] = {{&S28586[44], 33}, {NULL}};
const Segment SGI34167[] = {{&S28586[30], 6}, {NULL}};
const Segment SGI34168[] = {{&S28586[125], 3}, {NULL}};
const Segment SGI34169[] = {{&S28586[91], 5}, {NULL}};
const Segment SGI34170[] = {{&S28589[44], 11}, {NULL}};
const Segment SGI34171[] = {{&S28589[44], 31}, {NULL}};
const Segment SGI34172[] = {{&S28589[30], 6}, {NULL}};
const Segment SGI34173[] = {{&S28589[123], 3}, {NULL}};
const Segment SGI34174[] = {{&S28589[89], 5}, {NULL}};
const Segment SGI34175[] = {{&S28590[44], 7}, {NULL}};
const Segment SGI34176[] = {{&S28590[44], 32}, {NULL}};
const Segment SGI34177[] = {{&S28590[30], 6}, {NULL}};
const Segment SGI34178[] = {{&S28590[93], 4}, {NULL}};
const Segment SGI34179[] = {{&S28590[93], 3}, {NULL}};
const Segment SGI34180[] = {{&S28594[44], 7}, {NULL}};
const Segment SGI34181[] = {{&S28594[44], 28}, {NULL}};
const Segment SGI34182[] = {{&S28594[30], 6}, {NULL}};
const Segment SGI34183[] = {{&S28594[120], 3}, {NULL}};
const Segment SGI34184[] = {{&S28594[86], 5}, {NULL}};
const Segment SGI34185[] = {{&S28597[44], 11}, {NULL}};
const Segment SGI34186[] = {{&S28597[44], 36}, {NULL}};
const Segment SGI34187[] = {{&S28597[30], 6}, {NULL}};
const Segment SGI34188[] = {{&S28597[128], 3}, {NULL}};
const Segment SGI34189[] = {{&S28597[94], 5}, {NULL}};
const Segment SGI34190[] = {{&S28598[44], 11}, {NULL}};
const Segment SGI34191[] = {{&S28598[44], 32}, {NULL}};
const Segment SGI34192[] = {{&S28598[30], 6}, {NULL}};
const Segment SGI34193[] = {{&S28598[124], 3}, {NULL}};
const Segment SGI34194[] = {{&S28598[90], 5}, {NULL}};
const Segment SGI34195[] = {{&S28599[44], 7}, {NULL}};
const Segment SGI34196[] = {{&S28599[44], 32}, {NULL}};
const Segment SGI34197[] = {{&S28599[30], 6}, {NULL}};
const Segment SGI34198[] = {{&S28599[124], 3}, {NULL}};
const Segment SGI34199[] = {{&S28599[90], 5}, {NULL}};
const Segment SGI34200[] = {{&S28600[44], 11}, {NULL}};
const Segment SGI34201[] = {{&S28600[44], 37}, {NULL}};
const Segment SGI34202[] = {{&S28600[30], 6}, {NULL}};
const Segment SGI34203[] = {{&S28600[129], 3}, {NULL}};
const Segment SGI34204[] = {{&S28600[95], 5}, {NULL}};
const Segment SGI34205[] = {{&S28602[44], 11}, {NULL}};
const Segment SGI34206[] = {{&S28602[44], 36}, {NULL}};
const Segment SGI34207[] = {{&S28602[30], 6}, {NULL}};
const Segment SGI34208[] = {{&S28602[128], 3}, {NULL}};
const Segment SGI34209[] = {{&S28602[94], 5}, {NULL}};
const Segment SGI34210[] = {{&S28605[44], 7}, {NULL}};
const Segment SGI34211[] = {{&S28605[44], 28}, {NULL}};
const Segment SGI34212[] = {{&S28605[30], 6}, {NULL}};
const Segment SGI34213[] = {{&S28605[120], 3}, {NULL}};
const Segment SGI34214[] = {{&S28605[86], 5}, {NULL}};
const Segment SGI34215[] = {{&S28609[44], 11}, {NULL}};
const Segment SGI34216[] = {{&S28609[44], 36}, {NULL}};
const Segment SGI34217[] = {{&S28609[30], 6}, {NULL}};
const Segment SGI34218[] = {{&S28609[128], 3}, {NULL}};
const Segment SGI34219[] = {{&S28609[94], 5}, {NULL}};
const Segment SGI34220[] = {{&S28610[44], 11}, {NULL}};
const Segment SGI34221[] = {{&S28610[44], 34}, {NULL}};
const Segment SGI34222[] = {{&S28610[30], 6}, {NULL}};
const Segment SGI34223[] = {{&S28610[126], 3}, {NULL}};
const Segment SGI34224[] = {{&S28610[92], 5}, {NULL}};
const Segment SGI34225[] = {{&S28611[44], 7}, {NULL}};
const Segment SGI34226[] = {{&S28611[44], 27}, {NULL}};
const Segment SGI34227[] = {{&S28611[30], 6}, {NULL}};
const Segment SGI34228[] = {{&S28611[119], 3}, {NULL}};
const Segment SGI34229[] = {{&S28611[85], 5}, {NULL}};
const Segment SGI34230[] = {{&S28612[44], 7}, {NULL}};
const Segment SGI34231[] = {{&S28612[44], 32}, {NULL}};
const Segment SGI34232[] = {{&S28612[30], 6}, {NULL}};
const Segment SGI34233[] = {{&S28612[124], 3}, {NULL}};
const Segment SGI34234[] = {{&S28612[90], 5}, {NULL}};
const Segment SGI34235[] = {{&S28613[44], 11}, {NULL}};
const Segment SGI34236[] = {{&S28613[44], 35}, {NULL}};
const Segment SGI34237[] = {{&S28613[30], 6}, {NULL}};
const Segment SGI34238[] = {{&S28613[127], 3}, {NULL}};
const Segment SGI34239[] = {{&S28613[93], 5}, {NULL}};
const Segment SGI34240[] = {{&S28614[44], 11}, {NULL}};
const Segment SGI34241[] = {{&S28614[44], 36}, {NULL}};
const Segment SGI34242[] = {{&S28614[30], 6}, {NULL}};
const Segment SGI34243[] = {{&S28614[128], 3}, {NULL}};
const Segment SGI34244[] = {{&S28614[94], 5}, {NULL}};
const Segment SGI34245[] = {{&S28617[44], 7}, {NULL}};
const Segment SGI34246[] = {{&S28617[44], 28}, {NULL}};
const Segment SGI34247[] = {{&S28617[30], 6}, {NULL}};
const Segment SGI34248[] = {{&S28617[120], 3}, {NULL}};
const Segment SGI34249[] = {{&S28617[86], 5}, {NULL}};
const Segment SGI34250[] = {{&S28619[44], 11}, {NULL}};
const Segment SGI34251[] = {{&S28619[44], 34}, {NULL}};
const Segment SGI34252[] = {{&S28619[30], 6}, {NULL}};
const Segment SGI34253[] = {{&S28619[126], 3}, {NULL}};
const Segment SGI34254[] = {{&S28619[92], 5}, {NULL}};
const Segment SGI34255[] = {{&S28620[44], 11}, {NULL}};
const Segment SGI34256[] = {{&S28620[44], 36}, {NULL}};
const Segment SGI34257[] = {{&S28620[30], 6}, {NULL}};
const Segment SGI34258[] = {{&S28620[128], 3}, {NULL}};
const Segment SGI34259[] = {{&S28620[94], 5}, {NULL}};
const Segment SGI34260[] = {{&S28621[44], 11}, {NULL}};
const Segment SGI34261[] = {{&S28621[44], 36}, {NULL}};
const Segment SGI34262[] = {{&S28621[30], 6}, {NULL}};
const Segment SGI34263[] = {{&S28621[128], 3}, {NULL}};
const Segment SGI34264[] = {{&S28621[94], 5}, {NULL}};
const Segment SGI34265[] = {{&S28622[44], 8}, {NULL}};
const Segment SGI34266[] = {{&S28622[44], 34}, {NULL}};
const Segment SGI34267[] = {{&S28622[30], 6}, {NULL}};
const Segment SGI34268[] = {{&S28622[126], 3}, {NULL}};
const Segment SGI34269[] = {{&S28622[92], 5}, {NULL}};
const Segment SGI34270[] = {{&S28623[44], 11}, {NULL}};
const Segment SGI34271[] = {{&S28623[44], 40}, {NULL}};
const Segment SGI34272[] = {{&S28623[30], 6}, {NULL}};
const Segment SGI34273[] = {{&S28623[132], 3}, {NULL}};
const Segment SGI34274[] = {{&S28623[98], 5}, {NULL}};
const Segment SGI34275[] = {{&S28624[44], 11}, {NULL}};
const Segment SGI34276[] = {{&S28624[44], 32}, {NULL}};
const Segment SGI34277[] = {{&S28624[30], 6}, {NULL}};
const Segment SGI34278[] = {{&S28624[124], 3}, {NULL}};
const Segment SGI34279[] = {{&S28624[90], 5}, {NULL}};
const Segment SGI34280[] = {{&S28626[44], 7}, {NULL}};
const Segment SGI34281[] = {{&S28626[44], 31}, {NULL}};
const Segment SGI34282[] = {{&S28626[30], 6}, {NULL}};
const Segment SGI34283[] = {{&S28626[123], 3}, {NULL}};
const Segment SGI34284[] = {{&S28626[89], 5}, {NULL}};
const Segment SGI34285[] = {{&S28627[44], 7}, {NULL}};
const Segment SGI34286[] = {{&S28627[44], 32}, {NULL}};
const Segment SGI34287[] = {{&S28627[30], 6}, {NULL}};
const Segment SGI34288[] = {{&S28627[124], 3}, {NULL}};
const Segment SGI34289[] = {{&S28627[90], 5}, {NULL}};
const Segment SGI34290[] = {{&S28628[44], 11}, {NULL}};
const Segment SGI34291[] = {{&S28628[44], 41}, {NULL}};
const Segment SGI34292[] = {{&S28628[30], 6}, {NULL}};
const Segment SGI34293[] = {{&S28628[133], 3}, {NULL}};
const Segment SGI34294[] = {{&S28628[99], 5}, {NULL}};
const Segment SGI34295[] = {{&S28629[44], 11}, {NULL}};
const Segment SGI34296[] = {{&S28629[44], 34}, {NULL}};
const Segment SGI34297[] = {{&S28629[30], 6}, {NULL}};
const Segment SGI34298[] = {{&S28629[126], 3}, {NULL}};
const Segment SGI34299[] = {{&S28629[92], 5}, {NULL}};
const Segment SGI34300[] = {{&S28638[44], 8}, {NULL}};
const Segment SGI34301[] = {{&S28638[44], 34}, {NULL}};
const Segment SGI34302[] = {{&S28638[30], 6}, {NULL}};
const Segment SGI34303[] = {{&S28638[126], 3}, {NULL}};
const Segment SGI34304[] = {{&S28638[92], 5}, {NULL}};
const Segment SGI34305[] = {{&S28640[44], 11}, {NULL}};
const Segment SGI34306[] = {{&S28640[44], 36}, {NULL}};
const Segment SGI34307[] = {{&S28640[30], 6}, {NULL}};
const Segment SGI34308[] = {{&S28640[128], 3}, {NULL}};
const Segment SGI34309[] = {{&S28640[94], 5}, {NULL}};
const Segment SGI34310[] = {{&S28641[44], 7}, {NULL}};
const Segment SGI34311[] = {{&S28641[44], 32}, {NULL}};
const Segment SGI34312[] = {{&S28641[30], 6}, {NULL}};
const Segment SGI34313[] = {{&S28641[124], 3}, {NULL}};
const Segment SGI34314[] = {{&S28641[90], 5}, {NULL}};
const Segment SGI34315[] = {{&S28642[44], 7}, {NULL}};
const Segment SGI34316[] = {{&S28642[44], 37}, {NULL}};
const Segment SGI34317[] = {{&S28642[30], 6}, {NULL}};
const Segment SGI34318[] = {{&S28642[129], 3}, {NULL}};
const Segment SGI34319[] = {{&S28642[95], 5}, {NULL}};
const Segment SGI34320[] = {{&S28643[44], 7}, {NULL}};
const Segment SGI34321[] = {{&S28643[44], 30}, {NULL}};
const Segment SGI34322[] = {{&S28643[30], 6}, {NULL}};
const Segment SGI34323[] = {{&S28643[122], 3}, {NULL}};
const Segment SGI34324[] = {{&S28643[88], 5}, {NULL}};
const Segment SGI34325[] = {{&S28647[38], 7}, {NULL}};
const Segment SGI34326[] = {{&S28647[31], 6}, {NULL}};
const Segment SGI34327[] = {{&S28653[44], 7}, {NULL}};
const Segment SGI34328[] = {{&S28653[44], 36}, {NULL}};
const Segment SGI34329[] = {{&S28653[30], 6}, {NULL}};
const Segment SGI34330[] = {{&S28653[128], 3}, {NULL}};
const Segment SGI34331[] = {{&S28653[94], 5}, {NULL}};
const Segment SGI34332[] = {{&S28654[44], 11}, {NULL}};
const Segment SGI34333[] = {{&S28654[44], 36}, {NULL}};
const Segment SGI34334[] = {{&S28654[30], 6}, {NULL}};
const Segment SGI34335[] = {{&S28654[128], 3}, {NULL}};
const Segment SGI34336[] = {{&S28654[94], 5}, {NULL}};
const Segment SGI34337[] = {{&S28658[44], 7}, {NULL}};
const Segment SGI34338[] = {{&S28658[44], 30}, {NULL}};
const Segment SGI34339[] = {{&S28658[30], 6}, {NULL}};
const Segment SGI34340[] = {{&S28658[122], 3}, {NULL}};
const Segment SGI34341[] = {{&S28658[88], 5}, {NULL}};
const Segment SGI34342[] = {{&S28659[44], 7}, {NULL}};
const Segment SGI34343[] = {{&S28659[44], 30}, {NULL}};
const Segment SGI34344[] = {{&S28659[30], 6}, {NULL}};
const Segment SGI34345[] = {{&S36322[44], 7}, {NULL}};
const Segment SGI34346[] = {{&S36322[44], 18}, {NULL}};
const Segment SGI34347[] = {{&S28659[122], 3}, {NULL}};
const Segment SGI34348[] = {{&S36322[30], 6}, {NULL}};
const Segment SGI34349[] = {{&S28659[88], 5}, {NULL}};
const Segment SGI34350[] = {{&S36323[44], 7}, {NULL}};
const Segment SGI34351[] = {{&S36323[44], 18}, {NULL}};
const Segment SGI34352[] = {{&S36323[30], 6}, {NULL}};
const Segment SGI34353[] = {{&S36323[110], 3}, {NULL}};
const Segment SGI34354[] = {{&S36323[76], 5}, {NULL}};
const Segment SGI34355[] = {{&S40119[44], 10}, {NULL}};
const Segment SGI34356[] = {{&S40119[44], 22}, {NULL}};
const Segment SGI34357[] = {{&S40119[30], 6}, {NULL}};
const Segment SGI34358[] = {{&S36324[38], 7}, {NULL}};
const Segment SGI34359[] = {{&S36324[31], 6}, {NULL}};
const Segment SGI34360[] = {{&S40119[114], 3}, {NULL}};
const Segment SGI34361[] = {{&S40119[80], 5}, {NULL}};
const Segment SGI34362[] = {{&S28663[44], 11}, {NULL}};
const Segment SGI34363[] = {{&S28663[44], 40}, {NULL}};
const Segment SGI34364[] = {{&S28663[30], 6}, {NULL}};
const Segment SGI34365[] = {{&S28663[132], 3}, {NULL}};
const Segment SGI34366[] = {{&S28663[98], 5}, {NULL}};
const Segment SGI34367[] = {{&S40120[44], 10}, {NULL}};
const Segment SGI34368[] = {{&S40120[44], 21}, {NULL}};
const Segment SGI34369[] = {{&S40120[30], 6}, {NULL}};
const Segment SGI34370[] = {{&S40120[113], 3}, {NULL}};
const Segment SGI34371[] = {{&S36326[44], 6}, {NULL}};
const Segment SGI34372[] = {{&S36326[44], 17}, {NULL}};
const Segment SGI34373[] = {{&S40120[79], 5}, {NULL}};
const Segment SGI34374[] = {{&S36326[30], 6}, {NULL}};
const Segment SGI34375[] = {{&S36326[109], 3}, {NULL}};
const Segment SGI34376[] = {{&S36326[75], 5}, {NULL}};
const Segment SGI34377[] = {{&S36327[44], 7}, {NULL}};
const Segment SGI34378[] = {{&S36327[44], 18}, {NULL}};
const Segment SGI34379[] = {{&S36327[30], 6}, {NULL}};
const Segment SGI34380[] = {{&S36327[110], 3}, {NULL}};
const Segment SGI34381[] = {{&S36327[76], 5}, {NULL}};
const Segment SGI34382[] = {{&S28667[44], 7}, {NULL}};
const Segment SGI34383[] = {{&S28667[44], 32}, {NULL}};
const Segment SGI34384[] = {{&S28667[30], 6}, {NULL}};
const Segment SGI34385[] = {{&S36328[43], 7}, {NULL}};
const Segment SGI34386[] = {{&S36328[43], 18}, {NULL}};
const Segment SGI34387[] = {{&S36328[30], 5}, {NULL}};
const Segment SGI34388[] = {{&S28667[93], 4}, {NULL}};
const Segment SGI34389[] = {{&S28667[93], 3}, {NULL}};
const Segment SGI34390[] = {{&S36328[109], 3}, {NULL}};
const Segment SGI34391[] = {{&S36328[75], 5}, {NULL}};
const Segment SGI34392[] = {{&S28668[44], 7}, {NULL}};
const Segment SGI34393[] = {{&S28668[44], 32}, {NULL}};
const Segment SGI34394[] = {{&S36329[43], 7}, {NULL}};
const Segment SGI34395[] = {{&S28668[30], 6}, {NULL}};
const Segment SGI34396[] = {{&S36329[43], 18}, {NULL}};
const Segment SGI34397[] = {{&S36329[109], 3}, {NULL}};
const Segment SGI34398[] = {{&S28668[93], 6}, {NULL}};
const Segment SGI34399[] = {{&S36329[75], 5}, {NULL}};
const Segment SGI34400[] = {{&S36334[42], 7}, {NULL}};
const Segment SGI34401[] = {{&S36334[30], 6}, {NULL}};
const Segment SGI34402[] = {{&S36334[108], 3}, {NULL}};
const Segment SGI34403[] = {{&S28673[44], 13}, {NULL}};
const Segment SGI34404[] = {{&S28673[44], 22}, {NULL}};
const Segment SGI34405[] = {{&S36334[74], 5}, {NULL}};
const Segment SGI34406[] = {{&S28673[30], 6}, {NULL}};
const Segment SGI34407[] = {{&S28673[114], 3}, {NULL}};
const Segment SGI34408[] = {{&S28673[80], 5}, {NULL}};
const Segment SGI34409[] = {{&S36335[44], 7}, {NULL}};
const Segment SGI34410[] = {{&S36335[44], 18}, {NULL}};
const Segment SGI34411[] = {{&S36335[30], 6}, {NULL}};
const Segment SGI34412[] = {{&S36335[112], 3}, {NULL}};
const Segment SGI34413[] = {{&S36335[76], 7}, {NULL}};
const Segment SGI34414[] = {{&S36335[132], 8}, {NULL}};
const Segment SGI34415[] = {{&S28675[44], 13}, {NULL}};
const Segment SGI34416[] = {{&S28675[44], 22}, {NULL}};
const Segment SGI34417[] = {{&S28675[30], 6}, {NULL}};
const Segment SGI34418[] = {{&S28675[114], 3}, {NULL}};
const Segment SGI34419[] = {{&S28675[80], 5}, {NULL}};
const Segment SGI34420[] = {{&S28676[44], 10}, {NULL}};
const Segment SGI34421[] = {{&S28676[44], 27}, {NULL}};
const Segment SGI34422[] = {{&S28676[30], 6}, {NULL}};
const Segment SGI34423[] = {{&S28676[119], 3}, {NULL}};
const Segment SGI34424[] = {{&S28676[85], 5}, {NULL}};
const Segment SGI34425[] = {{&S28677[38], 10}, {NULL}};
const Segment SGI34426[] = {{&S28677[31], 6}, {NULL}};
const Segment SGI34427[] = {{&S28678[44], 10}, {NULL}};
const Segment SGI34428[] = {{&S28678[44], 27}, {NULL}};
const Segment SGI34429[] = {{&S28678[30], 6}, {NULL}};
const Segment SGI34430[] = {{&S28678[121], 3}, {NULL}};
const Segment SGI34431[] = {{&S28678[85], 7}, {NULL}};
const Segment SGI34432[] = {{&S28678[141], 8}, {NULL}};
const Segment SGI34433[] = {{&S40185[44], 13}, {NULL}};
const Segment SGI34434[] = {{&S40185[44], 25}, {NULL}};
const Segment SGI34435[] = {{&S40185[30], 6}, {NULL}};
const Segment SGI34436[] = {{&S40185[117], 3}, {NULL}};
const Segment SGI34437[] = {{&S40185[83], 5}, {NULL}};
const Segment SGI34438[] = {{&S40186[42], 13}, {NULL}};
const Segment SGI34439[] = {{&S40186[30], 6}, {NULL}};
const Segment SGI34440[] = {{&S40186[115], 3}, {NULL}};
const Segment SGI34441[] = {{&S40186[81], 5}, {NULL}};
const Segment SGI34442[] = {{&S40187[44], 7}, {NULL}};
const Segment SGI34443[] = {{&S40187[44], 19}, {NULL}};
const Segment SGI34444[] = {{&S40187[30], 6}, {NULL}};
const Segment SGI34445[] = {{&S40187[111], 3}, {NULL}};
const Segment SGI34446[] = {{&S40187[77], 5}, {NULL}};
const Segment SGI34447[] = {{&S40188[38], 13}, {NULL}};
const Segment SGI34448[] = {{&S40188[31], 6}, {NULL}};
const Segment SGI34449[] = {{&S40191[38], 7}, {NULL}};
const Segment SGI34450[] = {{&S40191[31], 6}, {NULL}};
const Segment SGI34451[] = {{&S40192[44], 13}, {NULL}};
const Segment SGI34452[] = {{&S40192[44], 25}, {NULL}};
const Segment SGI34453[] = {{&S40192[30], 6}, {NULL}};
const Segment SGI34454[] = {{&S40192[86], 4}, {NULL}};
const Segment SGI34455[] = {{&S40192[86], 3}, {NULL}};
const Segment SGI34456[] = {{&S40194[44], 13}, {NULL}};
const Segment SGI34457[] = {{&S40194[44], 25}, {NULL}};
const Segment SGI34458[] = {{&S40194[30], 6}, {NULL}};
const Segment SGI34459[] = {{&S40194[86], 6}, {NULL}};
const Segment SGI34460[] = {{&S40196[44], 10}, {NULL}};
const Segment SGI34461[] = {{&S40196[44], 21}, {NULL}};
const Segment SGI34462[] = {{&S40196[30], 6}, {NULL}};
const Segment SGI34463[] = {{&S40196[113], 3}, {NULL}};
const Segment SGI34464[] = {{&S40196[79], 5}, {NULL}};
const Segment SGI34465[] = {{&S40197[42], 10}, {NULL}};
const Segment SGI34466[] = {{&S40197[30], 6}, {NULL}};
const Segment SGI34467[] = {{&S40197[111], 3}, {NULL}};
const Segment SGI34468[] = {{&S40197[77], 5}, {NULL}};
const Segment SGI34469[] = {{&S29686[44], 13}, {NULL}};
const Segment SGI34470[] = {{&S29686[44], 23}, {NULL}};
const Segment SGI34471[] = {{&S29686[30], 6}, {NULL}};
const Segment SGI34472[] = {{&S29686[115], 3}, {NULL}};
const Segment SGI34473[] = {{&S29686[81], 5}, {NULL}};
const Segment SGI34474[] = {{&S40199[38], 10}, {NULL}};
const Segment SGI34475[] = {{&S40199[31], 6}, {NULL}};
const Segment SGI34476[] = {{&S29687[44], 10}, {NULL}};
const Segment SGI34477[] = {{&S29687[44], 27}, {NULL}};
const Segment SGI34478[] = {{&S29687[30], 6}, {NULL}};
const Segment SGI34479[] = {{&S29687[119], 3}, {NULL}};
const Segment SGI34480[] = {{&S29687[85], 5}, {NULL}};
const Segment SGI34481[] = {{&S40200[44], 10}, {NULL}};
const Segment SGI34482[] = {{&S40200[44], 26}, {NULL}};
const Segment SGI34483[] = {{&S40200[30], 6}, {NULL}};
const Segment SGI34484[] = {{&S29691[44], 13}, {NULL}};
const Segment SGI34485[] = {{&S29691[44], 22}, {NULL}};
const Segment SGI34486[] = {{&S29691[30], 6}, {NULL}};
const Segment SGI34487[] = {{&S29691[114], 3}, {NULL}};
const Segment SGI34488[] = {{&S29691[80], 5}, {NULL}};
const Segment SGI34489[] = {{&S29693[44], 13}, {NULL}};
const Segment SGI34490[] = {{&S29693[44], 23}, {NULL}};
const Segment SGI34491[] = {{&S29693[30], 6}, {NULL}};
const Segment SGI34492[] = {{&S29693[115], 3}, {NULL}};
const Segment SGI34493[] = {{&S29693[81], 5}, {NULL}};
const Segment SGI34494[] = {{&S29694[44], 13}, {NULL}};
const Segment SGI34495[] = {{&S29694[44], 24}, {NULL}};
const Segment SGI34496[] = {{&S29694[30], 6}, {NULL}};
const Segment SGI34497[] = {{&S29694[65], 3}, {NULL}};
const Segment SGI34498[] = {{&S29694[82], 5}, {NULL}};
const Segment SGI34499[] = {{&S40344[44], 7}, {NULL}};
const Segment SGI34500[] = {{&S40344[44], 24}, {NULL}};
const Segment SGI34501[] = {{&S40344[30], 6}, {NULL}};
const Segment SGI34502[] = {{&S40344[116], 3}, {NULL}};
const Segment SGI34503[] = {{&S40344[82], 5}, {NULL}};
const Segment SGI34504[] = {{&S40345[44], 11}, {NULL}};
const Segment SGI34505[] = {{&S40345[44], 28}, {NULL}};
const Segment SGI34506[] = {{&S40345[30], 6}, {NULL}};
const Segment SGI34507[] = {{&S40345[120], 3}, {NULL}};
const Segment SGI34508[] = {{&S40345[86], 5}, {NULL}};
const Segment SGI34509[] = {{&S40346[44], 11}, {NULL}};
const Segment SGI34510[] = {{&S40346[44], 28}, {NULL}};
const Segment SGI34511[] = {{&S40346[30], 6}, {NULL}};
const Segment SGI34512[] = {{&S40346[120], 3}, {NULL}};
const Segment SGI34513[] = {{&S40346[86], 5}, {NULL}};
const Segment SGI34514[] = {{&S40347[44], 12}, {NULL}};
const Segment SGI34515[] = {{&S40347[44], 29}, {NULL}};
const Segment SGI34516[] = {{&S40347[30], 6}, {NULL}};
const Segment SGI34517[] = {{&S40347[121], 3}, {NULL}};
const Segment SGI34518[] = {{&S40347[87], 5}, {NULL}};
const Segment SGI34519[] = {{&S40348[44], 14}, {NULL}};
const Segment SGI34520[] = {{&S40348[44], 31}, {NULL}};
const Segment SGI34521[] = {{&S40348[30], 6}, {NULL}};
const Segment SGI34522[] = {{&S40348[123], 3}, {NULL}};
const Segment SGI34523[] = {{&S40348[89], 5}, {NULL}};
const Segment SGI34524[] = {{&S40349[44], 10}, {NULL}};
const Segment SGI34525[] = {{&S40349[44], 27}, {NULL}};
const Segment SGI34526[] = {{&S40349[30], 6}, {NULL}};
const Segment SGI34527[] = {{&S40349[119], 3}, {NULL}};
const Segment SGI34528[] = {{&S40349[85], 5}, {NULL}};
const Segment SGI34529[] = {{&S40350[44], 11}, {NULL}};
const Segment SGI34530[] = {{&S40350[44], 28}, {NULL}};
const Segment SGI34531[] = {{&S40350[30], 6}, {NULL}};
const Segment SGI34532[] = {{&S40350[120], 3}, {NULL}};
const Segment SGI34533[] = {{&S40350[86], 5}, {NULL}};
const Segment SGI34534[] = {{&S40351[44], 7}, {NULL}};
const Segment SGI34535[] = {{&S40351[44], 24}, {NULL}};
const Segment SGI34536[] = {{&S40351[30], 6}, {NULL}};
const Segment SGI34537[] = {{&S40351[85], 4}, {NULL}};
const Segment SGI34538[] = {{&S40351[85], 3}, {NULL}};
const Segment SGI34539[] = {{&S40352[38], 7}, {NULL}};
const Segment SGI34540[] = {{&S40352[31], 6}, {NULL}};
const Segment SGI34541[] = {{&S40353[44], 17}, {NULL}};
const Segment SGI34542[] = {{&S40353[44], 34}, {NULL}};
const Segment SGI34543[] = {{&S40353[30], 6}, {NULL}};
const Segment SGI34544[] = {{&S40353[126], 3}, {NULL}};
const Segment SGI34545[] = {{&S29787[44], 19}, {NULL}};
const Segment SGI34546[] = {{&S40353[92], 5}, {NULL}};
const Segment SGI34547[] = {{&S29787[44], 30}, {NULL}};
const Segment SGI34548[] = {{&S29787[30], 6}, {NULL}};
const Segment SGI34549[] = {{&S29787[122], 3}, {NULL}};
const Segment SGI34550[] = {{&S29787[88], 5}, {NULL}};
const Segment SGI34551[] = {{&S40354[44], 15}, {NULL}};
const Segment SGI34552[] = {{&S40354[44], 32}, {NULL}};
const Segment SGI34553[] = {{&S40354[30], 6}, {NULL}};
const Segment SGI34554[] = {{&S40354[124], 3}, {NULL}};
const Segment SGI34555[] = {{&S40354[90], 5}, {NULL}};
const Segment SGI34556[] = {{&S28769[44], 18}, {NULL}};
const Segment SGI34557[] = {{&S28769[44], 35}, {NULL}};
const Segment SGI34558[] = {{&S28769[30], 6}, {NULL}};
const Segment SGI34559[] = {{&S28769[127], 3}, {NULL}};
const Segment SGI34560[] = {{&S28769[93], 5}, {NULL}};
const Segment SGI34561[] = {{&S28770[38], 18}, {NULL}};
const Segment SGI34562[] = {{&S28770[31], 6}, {NULL}};
const Segment SGI34563[] = {{&S28776[42], 18}, {NULL}};
const Segment SGI34564[] = {{&S28776[30], 6}, {NULL}};
const Segment SGI34565[] = {{&S28776[125], 3}, {NULL}};
const Segment SGI34566[] = {{&S28776[91], 5}, {NULL}};
const Segment SGI34567[] = {{&S28777[44], 10}, {NULL}};
const Segment SGI34568[] = {{&S28777[44], 27}, {NULL}};
const Segment SGI34569[] = {{&S28777[30], 6}, {NULL}};
const Segment SGI34570[] = {{&S28777[119], 3}, {NULL}};
const Segment SGI34571[] = {{&S28777[85], 5}, {NULL}};
const Segment SGI34572[] = {{&S29878[44], 19}, {NULL}};
const Segment SGI34573[] = {{&S29878[44], 38}, {NULL}};
const Segment SGI34574[] = {{&S29878[30], 6}, {NULL}};
const Segment SGI34575[] = {{&S29878[69], 3}, {NULL}};
const Segment SGI34576[] = {{&S29878[96], 5}, {NULL}};
const Segment SGI34577[] = {{&S29881[44], 19}, {NULL}};
const Segment SGI34578[] = {{&S29881[44], 38}, {NULL}};
const Segment SGI34579[] = {{&S29881[30], 6}, {NULL}};
const Segment SGI34580[] = {{&S29881[69], 3}, {NULL}};
const Segment SGI34581[] = {{&S29881[96], 5}, {NULL}};
const Segment SGI34582[] = {{&S29882[44], 19}, {NULL}};
const Segment SGI34583[] = {{&S29882[44], 37}, {NULL}};
const Segment SGI34584[] = {{&S29882[30], 6}, {NULL}};
const Segment SGI34585[] = {{&S29882[69], 3}, {NULL}};
const Segment SGI34586[] = {{&S29882[95], 5}, {NULL}};
const Segment SGI34587[] = {{&S29891[44], 7}, {NULL}};
const Segment SGI34588[] = {{&S29891[44], 26}, {NULL}};
const Segment SGI34589[] = {{&S29891[30], 6}, {NULL}};
const Segment SGI34590[] = {{&S29891[57], 3}, {NULL}};
const Segment SGI34591[] = {{&S29891[84], 5}, {NULL}};
const Segment SGI34592[] = {{&S29892[44], 19}, {NULL}};
const Segment SGI34593[] = {{&S29892[44], 37}, {NULL}};
const Segment SGI34594[] = {{&S29892[30], 6}, {NULL}};
const Segment SGI34595[] = {{&S29892[69], 3}, {NULL}};
const Segment SGI34596[] = {{&S29892[95], 5}, {NULL}};
const Segment SGI34597[] = {{&S31439[44], 14}, {NULL}};
const Segment SGI34598[] = {{&S31439[44], 25}, {NULL}};
const Segment SGI34599[] = {{&S31439[30], 6}, {NULL}};
const Segment SGI34600[] = {{&S31439[117], 3}, {NULL}};
const Segment SGI34601[] = {{&S29894[44], 7}, {NULL}};
const Segment SGI34602[] = {{&S29894[44], 25}, {NULL}};
const Segment SGI34603[] = {{&S31439[83], 5}, {NULL}};
const Segment SGI34604[] = {{&S29894[30], 6}, {NULL}};
const Segment SGI34605[] = {{&S29894[86], 4}, {NULL}};
const Segment SGI34606[] = {{&S29894[86], 3}, {NULL}};
const Segment SGI34607[] = {{&S31440[44], 9}, {NULL}};
const Segment SGI34608[] = {{&S29895[38], 7}, {NULL}};
const Segment SGI34609[] = {{&S31440[44], 20}, {NULL}};
const Segment SGI34610[] = {{&S31440[30], 6}, {NULL}};
const Segment SGI34611[] = {{&S29895[31], 6}, {NULL}};
const Segment SGI34612[] = {{&S31440[111], 3}, {NULL}};
const Segment SGI34613[] = {{&S31440[78], 6}, {NULL}};
const Segment SGI34614[] = {{&S31440[131], 8}, {NULL}};
const Segment SGI34615[] = {{&S29896[44], 19}, {NULL}};
const Segment SGI34616[] = {{&S29896[44], 37}, {NULL}};
const Segment SGI34617[] = {{&S29896[30], 6}, {NULL}};
const Segment SGI34618[] = {{&S31441[44], 9}, {NULL}};
const Segment SGI34619[] = {{&S29896[69], 3}, {NULL}};
const Segment SGI34620[] = {{&S31441[44], 22}, {NULL}};
const Segment SGI34621[] = {{&S31441[30], 6}, {NULL}};
const Segment SGI34622[] = {{&S29896[95], 5}, {NULL}};
const Segment SGI34623[] = {{&S31441[114], 3}, {NULL}};
const Segment SGI34624[] = {{&S31441[80], 5}, {NULL}};
const Segment SGI34625[] = {{&S29897[44], 19}, {NULL}};
const Segment SGI34626[] = {{&S29897[44], 37}, {NULL}};
const Segment SGI34627[] = {{&S29897[30], 6}, {NULL}};
const Segment SGI34628[] = {{&S29897[69], 3}, {NULL}};
const Segment SGI34629[] = {{&S31442[44], 9}, {NULL}};
const Segment SGI34630[] = {{&S29897[95], 5}, {NULL}};
const Segment SGI34631[] = {{&S31442[44], 38}, {NULL}};
const Segment SGI34632[] = {{&S31442[30], 6}, {NULL}};
const Segment SGI34633[] = {{&S31442[130], 3}, {NULL}};
const Segment SGI34634[] = {{&S31442[96], 5}, {NULL}};
const Segment SGI34635[] = {{&S29898[44], 7}, {NULL}};
const Segment SGI34636[] = {{&S29898[44], 25}, {NULL}};
const Segment SGI34637[] = {{&S29898[30], 6}, {NULL}};
const Segment SGI34638[] = {{&S29898[57], 3}, {NULL}};
const Segment SGI34639[] = {{&S29898[83], 5}, {NULL}};
const Segment SGI34640[] = {{&S31443[56], 9}, {NULL}};
const Segment SGI34641[] = {{&S31443[56], 20}, {NULL}};
const Segment SGI34642[] = {{&S31443[30], 18}, {NULL}};
const Segment SGI34643[] = {{&S31443[124], 3}, {NULL}};
const Segment SGI34644[] = {{&S31443[90], 5}, {NULL}};
const Segment SGI34645[] = {{&S29899[44], 19}, {NULL}};
const Segment SGI34646[] = {{&S29899[44], 37}, {NULL}};
const Segment SGI34647[] = {{&S29899[30], 6}, {NULL}};
const Segment SGI34648[] = {{&S29899[69], 3}, {NULL}};
const Segment SGI34649[] = {{&S29899[95], 5}, {NULL}};
const Segment SGI34650[] = {{&S31444[44], 15}, {NULL}};
const Segment SGI34651[] = {{&S31444[44], 32}, {NULL}};
const Segment SGI34652[] = {{&S31444[30], 6}, {NULL}};
const Segment SGI34653[] = {{&S31444[124], 3}, {NULL}};
const Segment SGI34654[] = {{&S31444[90], 5}, {NULL}};
const Segment SGI34655[] = {{&S29901[44], 7}, {NULL}};
const Segment SGI34656[] = {{&S29901[44], 25}, {NULL}};
const Segment SGI34657[] = {{&S29901[30], 6}, {NULL}};
const Segment SGI34658[] = {{&S29901[57], 3}, {NULL}};
const Segment SGI34659[] = {{&S29901[83], 5}, {NULL}};
const Segment SGI34660[] = {{&S31445[44], 14}, {NULL}};
const Segment SGI34661[] = {{&S31445[44], 25}, {NULL}};
const Segment SGI34662[] = {{&S31445[30], 6}, {NULL}};
const Segment SGI34663[] = {{&S31445[117], 3}, {NULL}};
const Segment SGI34664[] = {{&S31445[83], 5}, {NULL}};
const Segment SGI34665[] = {{&S31446[44], 9}, {NULL}};
const Segment SGI34666[] = {{&S29903[44], 7}, {NULL}};
const Segment SGI34667[] = {{&S31446[44], 26}, {NULL}};
const Segment SGI34668[] = {{&S29903[44], 25}, {NULL}};
const Segment SGI34669[] = {{&S31446[30], 6}, {NULL}};
const Segment SGI34670[] = {{&S29903[30], 6}, {NULL}};
const Segment SGI34671[] = {{&S29903[57], 3}, {NULL}};
const Segment SGI34672[] = {{&S31446[118], 3}, {NULL}};
const Segment SGI34673[] = {{&S29903[83], 5}, {NULL}};
const Segment SGI34674[] = {{&S31446[84], 5}, {NULL}};
const Segment SGI34675[] = {{&S29904[44], 19}, {NULL}};
const Segment SGI34676[] = {{&S29904[44], 37}, {NULL}};
const Segment SGI34677[] = {{&S29904[30], 6}, {NULL}};
const Segment SGI34678[] = {{&S31447[44], 9}, {NULL}};
const Segment SGI34679[] = {{&S31447[44], 20}, {NULL}};
const Segment SGI34680[] = {{&S31447[30], 6}, {NULL}};
const Segment SGI34681[] = {{&S29904[69], 3}, {NULL}};
const Segment SGI34682[] = {{&S29904[95], 5}, {NULL}};
const Segment SGI34683[] = {{&S31447[112], 3}, {NULL}};
const Segment SGI34684[] = {{&S31447[78], 5}, {NULL}};
const Segment SGI34685[] = {{&S29905[44], 20}, {NULL}};
const Segment SGI34686[] = {{&S29905[44], 38}, {NULL}};
const Segment SGI34687[] = {{&S29905[30], 6}, {NULL}};
const Segment SGI34688[] = {{&S29905[70], 3}, {NULL}};
const Segment SGI34689[] = {{&S29905[96], 5}, {NULL}};
const Segment SGI34690[] = {{&S31449[44], 14}, {NULL}};
const Segment SGI34691[] = {{&S31449[44], 25}, {NULL}};
const Segment SGI34692[] = {{&S31449[30], 6}, {NULL}};
const Segment SGI34693[] = {{&S31449[117], 3}, {NULL}};
const Segment SGI34694[] = {{&S31449[83], 5}, {NULL}};
const Segment SGI34695[] = {{&S29906[44], 19}, {NULL}};
const Segment SGI34696[] = {{&S29906[44], 37}, {NULL}};
const Segment SGI34697[] = {{&S29906[30], 6}, {NULL}};
const Segment SGI34698[] = {{&S29906[69], 3}, {NULL}};
const Segment SGI34699[] = {{&S29906[95], 5}, {NULL}};
const Segment SGI34700[] = {{&S29908[44], 19}, {NULL}};
const Segment SGI34701[] = {{&S31452[38], 9}, {NULL}};
const Segment SGI34702[] = {{&S29908[44], 37}, {NULL}};
const Segment SGI34703[] = {{&S31452[31], 6}, {NULL}};
const Segment SGI34704[] = {{&S29908[30], 6}, {NULL}};
const Segment SGI34705[] = {{&S29908[69], 3}, {NULL}};
const Segment SGI34706[] = {{&S31454[44], 10}, {NULL}};
const Segment SGI34707[] = {{&S31454[44], 21}, {NULL}};
const Segment SGI34708[] = {{&S31454[30], 6}, {NULL}};
const Segment SGI34709[] = {{&S31454[113], 3}, {NULL}};
const Segment SGI34710[] = {{&S31454[79], 5}, {NULL}};
const Segment SGI34711[] = {{&S31455[44], 15}, {NULL}};
const Segment SGI34712[] = {{&S31455[44], 32}, {NULL}};
const Segment SGI34713[] = {{&S31455[30], 6}, {NULL}};
const Segment SGI34714[] = {{&S31455[124], 3}, {NULL}};
const Segment SGI34715[] = {{&S31455[90], 5}, {NULL}};
const Segment SGI34716[] = {{&S31456[38], 10}, {NULL}};
const Segment SGI34717[] = {{&S31456[31], 6}, {NULL}};
const Segment SGI34718[] = {{&S31457[42], 17}, {NULL}};
const Segment SGI34719[] = {{&S31457[30], 6}, {NULL}};
const Segment SGI34720[] = {{&S31457[118], 3}, {NULL}};
const Segment SGI34721[] = {{&S31457[84], 5}, {NULL}};
const Segment SGI34722[] = {{&S31458[44], 17}, {NULL}};
const Segment SGI34723[] = {{&S31458[44], 28}, {NULL}};
const Segment SGI34724[] = {{&S31458[30], 6}, {NULL}};
const Segment SGI34725[] = {{&S31458[120], 3}, {NULL}};
const Segment SGI34726[] = {{&S31458[86], 5}, {NULL}};
const Segment SGI34727[] = {{&S31460[44], 9}, {NULL}};
const Segment SGI34728[] = {{&S31460[44], 20}, {NULL}};
const Segment SGI34729[] = {{&S31460[30], 6}, {NULL}};
const Segment SGI34730[] = {{&S31460[81], 4}, {NULL}};
const Segment SGI34731[] = {{&S31460[81], 3}, {NULL}};
const Segment SGI34732[] = {{&S31461[44], 14}, {NULL}};
const Segment SGI34733[] = {{&S31461[44], 25}, {NULL}};
const Segment SGI34734[] = {{&S31461[30], 6}, {NULL}};
const Segment SGI34735[] = {{&S31461[117], 3}, {NULL}};
const Segment SGI34736[] = {{&S31461[83], 5}, {NULL}};
const Segment SGI34737[] = {{&S31462[42], 10}, {NULL}};
const Segment SGI34738[] = {{&S31462[30], 6}, {NULL}};
const Segment SGI34739[] = {{&S31462[111], 3}, {NULL}};
const Segment SGI34740[] = {{&S31462[77], 5}, {NULL}};
const Segment SGI34741[] = {{&S28917[44], 10}, {NULL}};
const Segment SGI34742[] = {{&S28917[44], 27}, {NULL}};
const Segment SGI34743[] = {{&S28917[30], 6}, {NULL}};
const Segment SGI34744[] = {{&S28917[119], 3}, {NULL}};
const Segment SGI34745[] = {{&S28917[85], 5}, {NULL}};
const Segment SGI34746[] = {{&S31464[44], 21}, {NULL}};
const Segment SGI34747[] = {{&S31464[44], 50}, {NULL}};
const Segment SGI34748[] = {{&S31464[30], 6}, {NULL}};
const Segment SGI34749[] = {{&S31464[142], 3}, {NULL}};
const Segment SGI34750[] = {{&S31464[108], 5}, {NULL}};
const Segment SGI34751[] = {{&S31465[44], 19}, {NULL}};
const Segment SGI34752[] = {{&S31465[44], 30}, {NULL}};
const Segment SGI34753[] = {{&S31465[30], 6}, {NULL}};
const Segment SGI34754[] = {{&S31465[122], 3}, {NULL}};
const Segment SGI34755[] = {{&S31465[88], 5}, {NULL}};
const Segment SGI34756[] = {{&S28922[38], 10}, {NULL}};
const Segment SGI34757[] = {{&S28922[31], 6}, {NULL}};
const Segment SGI34758[] = {{&S28923[44], 10}, {NULL}};
const Segment SGI34759[] = {{&S28923[44], 27}, {NULL}};
const Segment SGI34760[] = {{&S28923[30], 6}, {NULL}};
const Segment SGI34761[] = {{&S31468[44], 10}, {NULL}};
const Segment SGI34762[] = {{&S31468[44], 21}, {NULL}};
const Segment SGI34763[] = {{&S31468[30], 6}, {NULL}};
const Segment SGI34764[] = {{&S28923[88], 4}, {NULL}};
const Segment SGI34765[] = {{&S28923[88], 3}, {NULL}};
const Segment SGI34766[] = {{&S31468[79], 5}, {NULL}};
const Segment SGI34767[] = {{&S28925[44], 10}, {NULL}};
const Segment SGI34768[] = {{&S28925[44], 27}, {NULL}};
const Segment SGI34769[] = {{&S28925[30], 6}, {NULL}};
const Segment SGI34770[] = {{&S28925[88], 6}, {NULL}};
const Segment SGI34771[] = {{&S28929[42], 10}, {NULL}};
const Segment SGI34772[] = {{&S28929[30], 6}, {NULL}};
const Segment SGI34773[] = {{&S28929[117], 3}, {NULL}};
const Segment SGI34774[] = {{&S28929[83], 5}, {NULL}};
const Segment SGI34775[] = {{&S29908[95], 5}, {NULL}};
const Segment SGI34776[] = {{&S29909[44], 19}, {NULL}};
const Segment SGI34777[] = {{&S29909[44], 37}, {NULL}};
const Segment SGI34778[] = {{&S29909[30], 6}, {NULL}};
const Segment SGI34779[] = {{&S29909[69], 3}, {NULL}};
const Segment SGI34780[] = {{&S29909[95], 5}, {NULL}};
const Segment SGI34781[] = {{&S29910[44], 20}, {NULL}};
const Segment SGI34782[] = {{&S29910[44], 38}, {NULL}};
const Segment SGI34783[] = {{&S29910[30], 6}, {NULL}};
const Segment SGI34784[] = {{&S29910[70], 3}, {NULL}};
const Segment SGI34785[] = {{&S29910[96], 5}, {NULL}};
const Segment SGI34786[] = {{&S29911[44], 7}, {NULL}};
const Segment SGI34787[] = {{&S29911[44], 25}, {NULL}};
const Segment SGI34788[] = {{&S29911[30], 6}, {NULL}};
const Segment SGI34789[] = {{&S29911[57], 3}, {NULL}};
const Segment SGI34790[] = {{&S29911[83], 5}, {NULL}};
const Segment SGI34791[] = {{&S29912[44], 7}, {NULL}};
const Segment SGI34792[] = {{&S29912[44], 25}, {NULL}};
const Segment SGI34793[] = {{&S29912[30], 6}, {NULL}};
const Segment SGI34794[] = {{&S29912[57], 3}, {NULL}};
const Segment SGI34795[] = {{&S29912[83], 5}, {NULL}};
const Segment SGI34796[] = {{&S29913[44], 7}, {NULL}};
const Segment SGI34797[] = {{&S29913[44], 25}, {NULL}};
const Segment SGI34798[] = {{&S29913[30], 6}, {NULL}};
const Segment SGI34799[] = {{&S29913[57], 3}, {NULL}};
const Segment SGI34800[] = {{&S29913[83], 5}, {NULL}};
const Segment SGI34801[] = {{&S29914[44], 20}, {NULL}};
const Segment SGI34802[] = {{&S29914[44], 38}, {NULL}};
const Segment SGI34803[] = {{&S29914[30], 6}, {NULL}};
const Segment SGI34804[] = {{&S29914[70], 3}, {NULL}};
const Segment SGI34805[] = {{&S29914[96], 5}, {NULL}};
const Segment SGI34806[] = {{&S29915[44], 7}, {NULL}};
const Segment SGI34807[] = {{&S29915[44], 25}, {NULL}};
const Segment SGI34808[] = {{&S29915[30], 6}, {NULL}};
const Segment SGI34809[] = {{&S29915[57], 3}, {NULL}};
const Segment SGI34810[] = {{&S29915[83], 5}, {NULL}};
const Segment SGI34811[] = {{&S29917[42], 19}, {NULL}};
const Segment SGI34812[] = {{&S29917[30], 6}, {NULL}};
const Segment SGI34813[] = {{&S29917[67], 3}, {NULL}};
const Segment SGI34814[] = {{&S29917[93], 5}, {NULL}};
const Segment SGI34815[] = {{&S29961[44], 10}, {NULL}};
const Segment SGI34816[] = {{&S29961[44], 27}, {NULL}};
const Segment SGI34817[] = {{&S29961[30], 6}, {NULL}};
const Segment SGI34818[] = {{&S29961[119], 3}, {NULL}};
const Segment SGI34819[] = {{&S29961[85], 5}, {NULL}};
const Segment SGI34820[] = {{&S29962[38], 10}, {NULL}};
const Segment SGI34821[] = {{&S29962[31], 6}, {NULL}};
const Segment SGI34822[] = {{&S29963[44], 10}, {NULL}};
const Segment SGI34823[] = {{&S29963[44], 32}, {NULL}};
const Segment SGI34824[] = {{&S29963[30], 6}, {NULL}};
const Segment SGI34825[] = {{&S29964[44], 10}, {NULL}};
const Segment SGI34826[] = {{&S29964[44], 27}, {NULL}};
const Segment SGI34827[] = {{&S29964[30], 6}, {NULL}};
const Segment SGI34828[] = {{&S29964[121], 3}, {NULL}};
const Segment SGI34829[] = {{&S29964[85], 7}, {NULL}};
const Segment SGI34830[] = {{&S29964[141], 8}, {NULL}};
const Segment SGI34831[] = {{&S29967[42], 10}, {NULL}};
const Segment SGI34832[] = {{&S29967[30], 6}, {NULL}};
const Segment SGI34833[] = {{&S29967[117], 3}, {NULL}};
const Segment SGI34834[] = {{&S29967[83], 5}, {NULL}};
const Segment SGI34835[] = {{&S29970[44], 10}, {NULL}};
const Segment SGI34836[] = {{&S29970[44], 27}, {NULL}};
const Segment SGI34837[] = {{&S29970[30], 6}, {NULL}};
const Segment SGI34838[] = {{&S29970[88], 4}, {NULL}};
const Segment SGI34839[] = {{&S29970[88], 3}, {NULL}};
const Segment SGI34840[] = {{&S29973[44], 10}, {NULL}};
const Segment SGI34841[] = {{&S29973[44], 25}, {NULL}};
const Segment SGI34842[] = {{&S29973[30], 6}, {NULL}};
const Segment SGI34843[] = {{&S29973[117], 3}, {NULL}};
const Segment SGI34844[] = {{&S29973[83], 5}, {NULL}};
const Segment SGI34845[] = {{&S29976[44], 10}, {NULL}};
const Segment SGI34846[] = {{&S29976[44], 27}, {NULL}};
const Segment SGI34847[] = {{&S29976[30], 6}, {NULL}};
const Segment SGI34848[] = {{&S29976[119], 3}, {&S29976[178], 3}, {NULL}};
const Segment SGI34849[] = {{&S29976[85], 5}, {NULL}};
const Segment SGI34850[] = {{&S40724[44], 9}, {NULL}};
const Segment SGI34851[] = {{&S40724[44], 21}, {NULL}};
const Segment SGI34852[] = {{&S40724[30], 6}, {NULL}};
const Segment SGI34853[] = {{&S40724[113], 3}, {NULL}};
const Segment SGI34854[] = {{&S40724[79], 5}, {NULL}};
const Segment SGI34855[] = {{&S40725[43], 9}, {NULL}};
const Segment SGI34856[] = {{&S40725[43], 21}, {NULL}};
const Segment SGI34857[] = {{&S40725[112], 3}, {NULL}};
const Segment SGI34858[] = {{&S40725[78], 5}, {NULL}};
const Segment SGI34859[] = {{&S40726[41], 9}, {NULL}};
const Segment SGI34860[] = {{&S40726[30], 6}, {NULL}};
const Segment SGI34861[] = {{&S40726[110], 3}, {NULL}};
const Segment SGI34862[] = {{&S40726[76], 5}, {NULL}};
const Segment SGI34863[] = {{&S40728[44], 8}, {NULL}};
const Segment SGI34864[] = {{&S40728[44], 20}, {NULL}};
const Segment SGI34865[] = {{&S40728[30], 6}, {NULL}};
const Segment SGI34866[] = {{&S40728[112], 3}, {NULL}};
const Segment SGI34867[] = {{&S40728[78], 5}, {NULL}};
const Segment SGI34868[] = {{&S40729[43], 9}, {NULL}};
const Segment SGI34869[] = {{&S40729[43], 21}, {NULL}};
const Segment SGI34870[] = {{&S40729[30], 5}, {NULL}};
const Segment SGI34871[] = {{&S40729[112], 3}, {NULL}};
const Segment SGI34872[] = {{&S40729[78], 5}, {NULL}};
const Segment SGI34873[] = {{&S40730[44], 8}, {NULL}};
const Segment SGI34874[] = {{&S40730[44], 20}, {NULL}};
const Segment SGI34875[] = {{&S40730[30], 6}, {NULL}};
const Segment SGI34876[] = {{&S40730[112], 3}, {NULL}};
const Segment SGI34877[] = {{&S40730[78], 5}, {NULL}};
const Segment SGI34878[] = {{&S40731[44], 9}, {NULL}};
const Segment SGI34879[] = {{&S40731[44], 21}, {NULL}};
const Segment SGI34880[] = {{&S40731[30], 6}, {NULL}};
const Segment SGI34881[] = {{&S40731[113], 3}, {NULL}};
const Segment SGI34882[] = {{&S40731[79], 5}, {NULL}};
const Segment SGI34883[] = {{&S40732[38], 9}, {NULL}};
const Segment SGI34884[] = {{&S40732[31], 6}, {NULL}};
const Segment SGI34885[] = {{&S30007[44], 9}, {NULL}};
const Segment SGI34886[] = {{&S30007[44], 21}, {NULL}};
const Segment SGI34887[] = {{&S30007[30], 6}, {NULL}};
const Segment SGI34888[] = {{&S30007[113], 3}, {NULL}};
const Segment SGI34889[] = {{&S30007[79], 5}, {NULL}};
const Segment SGI34890[] = {{&S40748[44], 8}, {NULL}};
const Segment SGI34891[] = {{&S40748[44], 19}, {NULL}};
const Segment SGI34892[] = {{&S40748[30], 6}, {NULL}};
const Segment SGI34893[] = {{&S40748[111], 3}, {NULL}};
const Segment SGI34894[] = {{&S40748[77], 5}, {NULL}};
const Segment SGI34895[] = {{&S40749[38], 8}, {NULL}};
const Segment SGI34896[] = {{&S40749[31], 6}, {NULL}};
const Segment SGI34897[] = {{&S40750[42], 8}, {NULL}};
const Segment SGI34898[] = {{&S40750[30], 6}, {NULL}};
const Segment SGI34899[] = {{&S40750[109], 3}, {NULL}};
const Segment SGI34900[] = {{&S40750[75], 5}, {NULL}};
const Segment SGI34901[] = {{&S30013[44], 9}, {NULL}};
const Segment SGI34902[] = {{&S30013[44], 20}, {NULL}};
const Segment SGI34903[] = {{&S30013[30], 6}, {NULL}};
const Segment SGI34904[] = {{&S30013[112], 3}, {NULL}};
const Segment SGI34905[] = {{&S30013[78], 5}, {NULL}};
const Segment SGI34906[] = {{&S30017[38], 9}, {NULL}};
const Segment SGI34907[] = {{&S30017[31], 6}, {NULL}};
const Segment SGI34908[] = {{&S30018[44], 9}, {NULL}};
const Segment SGI34909[] = {{&S30018[44], 20}, {NULL}};
const Segment SGI34910[] = {{&S30018[30], 6}, {NULL}};
const Segment SGI34911[] = {{&S30018[112], 3}, {NULL}};
const Segment SGI34912[] = {{&S30018[78], 5}, {NULL}};
const Segment SGI34913[] = {{&S30021[44], 9}, {NULL}};
const Segment SGI34914[] = {{&S30021[44], 20}, {NULL}};
const Segment SGI34915[] = {{&S30021[30], 6}, {NULL}};
const Segment SGI34916[] = {{&S30021[112], 3}, {NULL}};
const Segment SGI34917[] = {{&S30021[78], 5}, {NULL}};
const Segment SGI34918[] = {{&S30022[44], 8}, {NULL}};
const Segment SGI34919[] = {{&S30022[44], 19}, {NULL}};
const Segment SGI34920[] = {{&S30022[30], 6}, {NULL}};
const Segment SGI34921[] = {{&S30022[111], 3}, {NULL}};
const Segment SGI34922[] = {{&S30022[77], 5}, {NULL}};
const Segment SGI34923[] = {{&S30023[43], 9}, {NULL}};
const Segment SGI34924[] = {{&S30023[43], 20}, {NULL}};
const Segment SGI34925[] = {{&S30023[30], 5}, {NULL}};
const Segment SGI34926[] = {{&S30023[111], 3}, {NULL}};
const Segment SGI34927[] = {{&S30023[77], 5}, {NULL}};
const Segment SGI34928[] = {{&S30024[43], 9}, {NULL}};
const Segment SGI34929[] = {{&S30024[43], 20}, {NULL}};
const Segment SGI34930[] = {{&S30024[111], 3}, {NULL}};
const Segment SGI34931[] = {{&S30024[77], 5}, {NULL}};
const Segment SGI34932[] = {{&S30025[44], 8}, {NULL}};
const Segment SGI34933[] = {{&S30025[44], 19}, {NULL}};
const Segment SGI34934[] = {{&S30025[30], 6}, {NULL}};
const Segment SGI34935[] = {{&S30025[111], 3}, {NULL}};
const Segment SGI34936[] = {{&S30025[77], 5}, {NULL}};
const Segment SGI34937[] = {{&S30026[44], 9}, {NULL}};
const Segment SGI34938[] = {{&S30026[44], 22}, {NULL}};
const Segment SGI34939[] = {{&S30026[30], 6}, {NULL}};
const Segment SGI34940[] = {{&S30026[114], 3}, {NULL}};
const Segment SGI34941[] = {{&S30026[80], 5}, {NULL}};
const Segment SGI34942[] = {{&S30029[64], 9}, {NULL}};
const Segment SGI34943[] = {{&S30029[64], 20}, {NULL}};
const Segment SGI34944[] = {{&S30029[50], 6}, {NULL}};
const Segment SGI34945[] = {{&S40958[44], 10}, {NULL}};
const Segment SGI34946[] = {{&S40958[44], 22}, {NULL}};
const Segment SGI34947[] = {{&S40958[30], 6}, {NULL}};
const Segment SGI34948[] = {{&S40958[114], 3}, {NULL}};
const Segment SGI34949[] = {{&S40958[80], 5}, {NULL}};
const Segment SGI34950[] = {{&S40960[38], 10}, {NULL}};
const Segment SGI34951[] = {{&S40960[31], 6}, {NULL}};
const Segment SGI34952[] = {{&S33486[44], 19}, {NULL}};
const Segment SGI34953[] = {{&S33486[44], 35}, {NULL}};
const Segment SGI34954[] = {{&S33486[30], 6}, {NULL}};
const Segment SGI34955[] = {{&S33486[127], 3}, {NULL}};
const Segment SGI34956[] = {{&S33486[93], 5}, {NULL}};
const Segment SGI34957[] = {{&S33487[44], 7}, {NULL}};
const Segment SGI34958[] = {{&S33487[44], 23}, {NULL}};
const Segment SGI34959[] = {{&S33487[30], 6}, {NULL}};
const Segment SGI34960[] = {{&S33487[115], 3}, {NULL}};
const Segment SGI34961[] = {{&S33487[81], 5}, {NULL}};
const Segment SGI34962[] = {{&S33490[44], 19}, {NULL}};
const Segment SGI34963[] = {{&S33490[44], 35}, {NULL}};
const Segment SGI34964[] = {{&S33490[30], 6}, {NULL}};
const Segment SGI34965[] = {{&S33490[127], 3}, {NULL}};
const Segment SGI34966[] = {{&S33490[93], 5}, {NULL}};
const Segment SGI34967[] = {{&S33491[38], 7}, {NULL}};
const Segment SGI34968[] = {{&S33491[31], 6}, {NULL}};
const Segment SGI34969[] = {{&S41017[44], 13}, {NULL}};
const Segment SGI34970[] = {{&S41017[44], 29}, {NULL}};
const Segment SGI34971[] = {{&S41017[30], 6}, {NULL}};
const Segment SGI34972[] = {{&S41017[121], 3}, {NULL}};
const Segment SGI34973[] = {{&S41017[87], 5}, {NULL}};
const Segment SGI34974[] = {{&S41018[42], 13}, {NULL}};
const Segment SGI34975[] = {{&S41018[30], 6}, {NULL}};
const Segment SGI34976[] = {{&S41018[119], 3}, {NULL}};
const Segment SGI34977[] = {{&S41018[85], 5}, {NULL}};
const Segment SGI34978[] = {{&S41019[44], 13}, {NULL}};
const Segment SGI34979[] = {{&S41019[44], 29}, {NULL}};
const Segment SGI34980[] = {{&S41019[30], 6}, {NULL}};
const Segment SGI34981[] = {{&S41019[121], 3}, {&S41019[180], 3}, {NULL}};
const Segment SGI34982[] = {{&S41019[87], 5}, {NULL}};
const Segment SGI34983[] = {{&S41020[38], 13}, {NULL}};
const Segment SGI34984[] = {{&S41020[31], 6}, {NULL}};
const Segment SGI34985[] = {{&S41021[44], 13}, {NULL}};
const Segment SGI34986[] = {{&S41021[44], 29}, {NULL}};
const Segment SGI34987[] = {{&S41021[30], 6}, {NULL}};
const Segment SGI34988[] = {{&S41021[123], 3}, {NULL}};
const Segment SGI34989[] = {{&S41021[87], 7}, {NULL}};
const Segment SGI34990[] = {{&S41021[143], 8}, {NULL}};
const Segment SGI34991[] = {{&S41022[44], 13}, {NULL}};
const Segment SGI34992[] = {{&S41022[44], 29}, {NULL}};
const Segment SGI34993[] = {{&S41022[30], 6}, {NULL}};
const Segment SGI34994[] = {{&S41022[121], 3}, {&S41022[180], 3}, {NULL}};
const Segment SGI34995[] = {{&S41022[87], 5}, {NULL}};
const Segment SGI34996[] = {{&S41081[44], 10}, {NULL}};
const Segment SGI34997[] = {{&S41081[44], 21}, {NULL}};
const Segment SGI34998[] = {{&S41081[30], 6}, {NULL}};
const Segment SGI34999[] = {{&S41081[113], 3}, {NULL}};
const Segment SGI35000[] = {{&S41081[79], 5}, {NULL}};
const Segment SGI35001[] = {{&S41082[38], 10}, {NULL}};
const Segment SGI35002[] = {{&S41082[31], 6}, {NULL}};
const Segment SGI35003[] = {{&S36249[42], 26}, {NULL}};
const Segment SGI35004[] = {{&S36249[42], 37}, {NULL}};
const Segment SGI35005[] = {{&S36249[30], 4}, {NULL}};
const Segment SGI35006[] = {{&S36249[127], 3}, {NULL}};
const Segment SGI35007[] = {{&S36249[93], 5}, {NULL}};
const Segment SGI35008[] = {{&S41300[44], 23}, {NULL}};
const Segment SGI35009[] = {{&S41300[44], 35}, {NULL}};
const Segment SGI35010[] = {{&S41300[30], 6}, {NULL}};
const Segment SGI35011[] = {{&S41300[127], 3}, {NULL}};
const Segment SGI35012[] = {{&S41300[93], 5}, {NULL}};
const Segment SGI35013[] = {{&S41301[44], 23}, {NULL}};
const Segment SGI35014[] = {{&S41301[44], 35}, {NULL}};
const Segment SGI35015[] = {{&S41301[30], 6}, {NULL}};
const Segment SGI35016[] = {{&S41301[127], 3}, {NULL}};
const Segment SGI35017[] = {{&S41301[93], 5}, {NULL}};
const Segment SGI35018[] = {{&S41302[44], 24}, {NULL}};
const Segment SGI35019[] = {{&S41302[44], 36}, {NULL}};
const Segment SGI35020[] = {{&S41302[30], 6}, {NULL}};
const Segment SGI35021[] = {{&S41302[128], 3}, {NULL}};
const Segment SGI35022[] = {{&S41302[94], 5}, {NULL}};
const Segment SGI35023[] = {{&S41303[44], 24}, {NULL}};
const Segment SGI35024[] = {{&S41303[44], 36}, {NULL}};
const Segment SGI35025[] = {{&S41303[30], 6}, {NULL}};
const Segment SGI35026[] = {{&S41303[128], 3}, {NULL}};
const Segment SGI35027[] = {{&S41303[94], 5}, {NULL}};
const Segment SGI35028[] = {{&S41304[44], 30}, {NULL}};
const Segment SGI35029[] = {{&S41304[44], 42}, {NULL}};
const Segment SGI35030[] = {{&S41304[30], 6}, {NULL}};
const Segment SGI35031[] = {{&S41304[134], 3}, {NULL}};
const Segment SGI35032[] = {{&S41304[100], 5}, {NULL}};
const Segment SGI35033[] = {{&S41305[44], 24}, {NULL}};
const Segment SGI35034[] = {{&S41305[44], 36}, {NULL}};
const Segment SGI35035[] = {{&S41305[30], 6}, {NULL}};
const Segment SGI35036[] = {{&S41305[128], 3}, {NULL}};
const Segment SGI35037[] = {{&S41305[94], 5}, {NULL}};
const Segment SGI35038[] = {{&S29688[44], 13}, {NULL}};
const Segment SGI35039[] = {{&S29688[44], 22}, {NULL}};
const Segment SGI35040[] = {{&S29688[30], 6}, {NULL}};
const Segment SGI35041[] = {{&S41306[38], 23}, {NULL}};
const Segment SGI35042[] = {{&S29688[114], 3}, {NULL}};
const Segment SGI35043[] = {{&S41306[31], 6}, {NULL}};
const Segment SGI35044[] = {{&S29688[80], 5}, {NULL}};
const Segment SGI35045[] = {{&S41307[38], 23}, {NULL}};
const Segment SGI35046[] = {{&S41307[31], 6}, {NULL}};
const Segment SGI35047[] = {{&S29689[44], 10}, {NULL}};
const Segment SGI35048[] = {{&S29689[44], 27}, {NULL}};
const Segment SGI35049[] = {{&S29689[30], 6}, {NULL}};
const Segment SGI35050[] = {{&S29689[119], 3}, {NULL}};
const Segment SGI35051[] = {{&S29689[85], 5}, {NULL}};
const Segment SGI35052[] = {{&S41308[38], 24}, {NULL}};
const Segment SGI35053[] = {{&S41308[31], 6}, {NULL}};
const Segment SGI35054[] = {{&S41309[38], 24}, {NULL}};
const Segment SGI35055[] = {{&S41309[31], 6}, {NULL}};
const Segment SGI35056[] = {{&S41494[44], 10}, {NULL}};
const Segment SGI35057[] = {{&S41494[44], 21}, {NULL}};
const Segment SGI35058[] = {{&S41494[30], 6}, {NULL}};
const Segment SGI35059[] = {{&S41494[113], 3}, {NULL}};
const Segment SGI35060[] = {{&S41494[79], 5}, {NULL}};
const Segment SGI35061[] = {{&S41495[38], 10}, {NULL}};
const Segment SGI35062[] = {{&S41495[31], 6}, {NULL}};
const Segment SGI35063[] = {{&S41496[44], 9}, {NULL}};
const Segment SGI35064[] = {{&S41496[44], 20}, {NULL}};
const Segment SGI35065[] = {{&S41496[30], 6}, {NULL}};
const Segment SGI35066[] = {{&S41496[112], 3}, {NULL}};
const Segment SGI35067[] = {{&S41496[78], 5}, {NULL}};
const Segment SGI35068[] = {{&S41497[43], 10}, {NULL}};
const Segment SGI35069[] = {{&S41497[43], 21}, {NULL}};
const Segment SGI35070[] = {{&S41497[30], 5}, {NULL}};
const Segment SGI35071[] = {{&S41497[112], 3}, {NULL}};
const Segment SGI35072[] = {{&S41497[78], 5}, {NULL}};
const Segment SGI35073[] = {{&S41498[43], 10}, {NULL}};
const Segment SGI35074[] = {{&S41498[43], 21}, {NULL}};
const Segment SGI35075[] = {{&S41498[112], 3}, {NULL}};
const Segment SGI35076[] = {{&S41498[78], 5}, {NULL}};
const Segment SGI35077[] = {{&S41499[44], 9}, {NULL}};
const Segment SGI35078[] = {{&S41499[44], 20}, {NULL}};
const Segment SGI35079[] = {{&S41499[30], 6}, {NULL}};
const Segment SGI35080[] = {{&S41499[112], 3}, {NULL}};
const Segment SGI35081[] = {{&S41499[78], 5}, {NULL}};
const Segment SGI35082[] = {{&S30601[44], 11}, {NULL}};
const Segment SGI35083[] = {{&S30601[44], 22}, {NULL}};
const Segment SGI35084[] = {{&S30601[30], 6}, {NULL}};
const Segment SGI35085[] = {{&S30601[114], 3}, {NULL}};
const Segment SGI35086[] = {{&S30601[80], 5}, {NULL}};
const Segment SGI35087[] = {{&S30603[38], 11}, {NULL}};
const Segment SGI35088[] = {{&S30603[31], 6}, {NULL}};
const Segment SGI35089[] = {{&S30615[44], 11}, {NULL}};
const Segment SGI35090[] = {{&S30615[44], 20}, {NULL}};
const Segment SGI35091[] = {{&S30615[30], 6}, {NULL}};
const Segment SGI35092[] = {{&S30615[112], 3}, {NULL}};
const Segment SGI35093[] = {{&S30615[78], 5}, {NULL}};
const Segment SGI35094[] = {{&S30621[44], 11}, {NULL}};
const Segment SGI35095[] = {{&S30621[44], 22}, {NULL}};
const Segment SGI35096[] = {{&S30621[30], 6}, {NULL}};
const Segment SGI35097[] = {{&S30621[114], 3}, {NULL}};
const Segment SGI35098[] = {{&S30621[80], 5}, {NULL}};
const Segment SGI35099[] = {{&S30633[44], 11}, {NULL}};
const Segment SGI35100[] = {{&S30633[44], 22}, {NULL}};
const Segment SGI35101[] = {{&S30633[30], 6}, {NULL}};
const Segment SGI35102[] = {{&S30633[113], 3}, {NULL}};
const Segment SGI35103[] = {{&S30633[80], 5}, {NULL}};
const Segment SGI35104[] = {{&S30008[44], 9}, {NULL}};
const Segment SGI35105[] = {{&S30008[44], 21}, {NULL}};
const Segment SGI35106[] = {{&S30008[30], 6}, {NULL}};
const Segment SGI35107[] = {{&S30008[113], 3}, {NULL}};
const Segment SGI35108[] = {{&S30008[79], 5}, {NULL}};
const Segment SGI35109[] = {{&S30010[44], 9}, {NULL}};
const Segment SGI35110[] = {{&S30010[44], 20}, {NULL}};
const Segment SGI35111[] = {{&S30010[30], 6}, {NULL}};
const Segment SGI35112[] = {{&S30010[112], 3}, {NULL}};
const Segment SGI35113[] = {{&S30010[78], 5}, {NULL}};
const Segment SGI35114[] = {{&S36673[44], 19}, {NULL}};
const Segment SGI35115[] = {{&S36673[44], 36}, {NULL}};
const Segment SGI35116[] = {{&S36673[30], 6}, {NULL}};
const Segment SGI35117[] = {{&S36673[128], 3}, {NULL}};
const Segment SGI35118[] = {{&S36673[94], 5}, {NULL}};
const Segment SGI35119[] = {{&S41929[44], 12}, {NULL}};
const Segment SGI35120[] = {{&S41929[44], 23}, {NULL}};
const Segment SGI35121[] = {{&S41929[30], 6}, {NULL}};
const Segment SGI35122[] = {{&S41929[115], 3}, {NULL}};
const Segment SGI35123[] = {{&S41929[81], 5}, {NULL}};
const Segment SGI35124[] = {{&S41930[38], 12}, {NULL}};
const Segment SGI35125[] = {{&S41930[31], 6}, {NULL}};
const Segment SGI35126[] = {{&S41931[50], 12}, {NULL}};
const Segment SGI35127[] = {{&S41931[50], 23}, {NULL}};
const Segment SGI35128[] = {{&S41931[30], 11}, {NULL}};
const Segment SGI35129[] = {{&S41931[121], 3}, {NULL}};
const Segment SGI35130[] = {{&S41931[87], 5}, {NULL}};
const Segment SGI35131[] = {{&S41932[42], 12}, {NULL}};
const Segment SGI35132[] = {{&S41932[30], 6}, {NULL}};
const Segment SGI35133[] = {{&S41932[113], 3}, {NULL}};
const Segment SGI35134[] = {{&S41932[79], 5}, {NULL}};
const Segment SGI35135[] = {{&S30448[38], 10}, {NULL}};
const Segment SGI35136[] = {{&S30448[31], 6}, {NULL}};
const Segment SGI35137[] = {{&S30449[44], 10}, {NULL}};
const Segment SGI35138[] = {{&S30449[44], 27}, {NULL}};
const Segment SGI35139[] = {{&S30449[30], 6}, {NULL}};
const Segment SGI35140[] = {{&S30449[119], 3}, {NULL}};
const Segment SGI35141[] = {{&S30449[85], 5}, {NULL}};
const Segment SGI35142[] = {{&S33114[44], 8}, {NULL}};
const Segment SGI35143[] = {{&S33114[44], 29}, {NULL}};
const Segment SGI35144[] = {{&S33114[30], 6}, {NULL}};
const Segment SGI35145[] = {{&S33114[121], 3}, {NULL}};
const Segment SGI35146[] = {{&S33114[87], 5}, {NULL}};
const Segment SGI35147[] = {{&S31199[44], 9}, {NULL}};
const Segment SGI35148[] = {{&S31199[44], 20}, {NULL}};
const Segment SGI35149[] = {{&S31199[30], 6}, {NULL}};
const Segment SGI35150[] = {{&S31199[112], 3}, {NULL}};
const Segment SGI35151[] = {{&S31199[78], 5}, {NULL}};
const Segment SGI35152[] = {{&S31200[44], 10}, {NULL}};
const Segment SGI35153[] = {{&S31200[44], 21}, {NULL}};
const Segment SGI35154[] = {{&S31200[30], 6}, {NULL}};
const Segment SGI35155[] = {{&S31200[113], 3}, {NULL}};
const Segment SGI35156[] = {{&S31200[79], 5}, {NULL}};
const Segment SGI35157[] = {{&S31201[44], 9}, {NULL}};
const Segment SGI35158[] = {{&S31201[44], 20}, {NULL}};
const Segment SGI35159[] = {{&S31201[30], 6}, {NULL}};
const Segment SGI35160[] = {{&S31201[114], 3}, {NULL}};
const Segment SGI35161[] = {{&S31201[78], 7}, {NULL}};
const Segment SGI35162[] = {{&S31201[134], 8}, {NULL}};
const Segment SGI35163[] = {{&S31202[38], 9}, {NULL}};
const Segment SGI35164[] = {{&S31202[31], 6}, {NULL}};
const Segment SGI35165[] = {{&S31203[43], 9}, {NULL}};
const Segment SGI35166[] = {{&S31203[43], 20}, {NULL}};
const Segment SGI35167[] = {{&S31203[111], 3}, {NULL}};
const Segment SGI35168[] = {{&S31203[77], 5}, {NULL}};
const Segment SGI35169[] = {{&S31204[44], 8}, {NULL}};
const Segment SGI35170[] = {{&S31204[44], 19}, {NULL}};
const Segment SGI35171[] = {{&S31204[30], 6}, {NULL}};
const Segment SGI35172[] = {{&S31204[111], 3}, {NULL}};
const Segment SGI35173[] = {{&S31204[77], 5}, {NULL}};
const Segment SGI35174[] = {{&S31205[44], 8}, {NULL}};
const Segment SGI35175[] = {{&S31205[44], 19}, {NULL}};
const Segment SGI35176[] = {{&S31205[30], 6}, {NULL}};
const Segment SGI35177[] = {{&S31205[111], 3}, {NULL}};
const Segment SGI35178[] = {{&S31205[77], 5}, {NULL}};
const Segment SGI35179[] = {{&S31206[44], 9}, {NULL}};
const Segment SGI35180[] = {{&S31206[44], 20}, {NULL}};
const Segment SGI35181[] = {{&S31206[30], 6}, {NULL}};
const Segment SGI35182[] = {{&S31206[112], 3}, {NULL}};
const Segment SGI35183[] = {{&S31206[78], 5}, {NULL}};
const Segment SGI35184[] = {{&S31207[43], 9}, {NULL}};
const Segment SGI35185[] = {{&S31207[43], 20}, {NULL}};
const Segment SGI35186[] = {{&S31207[30], 5}, {NULL}};
const Segment SGI35187[] = {{&S31207[111], 3}, {NULL}};
const Segment SGI35188[] = {{&S31207[77], 5}, {NULL}};
const Segment SGI35189[] = {{&S31215[38], 10}, {NULL}};
const Segment SGI35190[] = {{&S31215[31], 6}, {NULL}};
const Segment SGI35191[] = {{&S31229[44], 7}, {NULL}};
const Segment SGI35192[] = {{&S31229[44], 22}, {NULL}};
const Segment SGI35193[] = {{&S31229[30], 6}, {NULL}};
const Segment SGI35194[] = {{&S31229[114], 3}, {NULL}};
const Segment SGI35195[] = {{&S31229[80], 5}, {NULL}};
const Segment SGI35196[] = {{&S31230[44], 7}, {NULL}};
const Segment SGI35197[] = {{&S31230[44], 29}, {NULL}};
const Segment SGI35198[] = {{&S31230[30], 6}, {NULL}};
const Segment SGI35199[] = {{&S31230[121], 3}, {NULL}};
const Segment SGI35200[] = {{&S31230[87], 5}, {NULL}};
const Segment SGI35201[] = {{&S31231[44], 7}, {NULL}};
const Segment SGI35202[] = {{&S31231[44], 29}, {NULL}};
const Segment SGI35203[] = {{&S31231[30], 6}, {NULL}};
const Segment SGI35204[] = {{&S31231[121], 3}, {NULL}};
const Segment SGI35205[] = {{&S31231[87], 5}, {NULL}};
const Segment SGI35206[] = {{&S31232[44], 7}, {NULL}};
const Segment SGI35207[] = {{&S31232[44], 29}, {NULL}};
const Segment SGI35208[] = {{&S31232[30], 6}, {NULL}};
const Segment SGI35209[] = {{&S31232[121], 3}, {NULL}};
const Segment SGI35210[] = {{&S31232[87], 5}, {NULL}};
const Segment SGI35211[] = {{&S31233[44], 7}, {NULL}};
const Segment SGI35212[] = {{&S31233[44], 29}, {NULL}};
const Segment SGI35213[] = {{&S31233[30], 6}, {NULL}};
const Segment SGI35214[] = {{&S31233[121], 3}, {NULL}};
const Segment SGI35215[] = {{&S31233[87], 5}, {NULL}};
const Segment SGI35216[] = {{&S31239[44], 7}, {NULL}};
const Segment SGI35217[] = {{&S31239[44], 30}, {NULL}};
const Segment SGI35218[] = {{&S31239[30], 6}, {NULL}};
const Segment SGI35219[] = {{&S31239[122], 3}, {NULL}};
const Segment SGI35220[] = {{&S31239[88], 5}, {NULL}};
const Segment SGI35221[] = {{&S31246[44], 7}, {NULL}};
const Segment SGI35222[] = {{&S31246[44], 29}, {NULL}};
const Segment SGI35223[] = {{&S31246[30], 6}, {NULL}};
const Segment SGI35224[] = {{&S31246[121], 3}, {NULL}};
const Segment SGI35225[] = {{&S31246[87], 5}, {NULL}};
const Segment SGI35226[] = {{&S31248[44], 7}, {NULL}};
const Segment SGI35227[] = {{&S31248[44], 28}, {NULL}};
const Segment SGI35228[] = {{&S31248[30], 6}, {NULL}};
const Segment SGI35229[] = {{&S31248[120], 3}, {NULL}};
const Segment SGI35230[] = {{&S31248[86], 5}, {NULL}};
const Segment SGI35231[] = {{&S31250[44], 7}, {NULL}};
const Segment SGI35232[] = {{&S31250[44], 30}, {NULL}};
const Segment SGI35233[] = {{&S31250[30], 6}, {NULL}};
const Segment SGI35234[] = {{&S31250[122], 3}, {NULL}};
const Segment SGI35235[] = {{&S31250[88], 5}, {NULL}};
const Segment SGI35236[] = {{&S31252[44], 7}, {NULL}};
const Segment SGI35237[] = {{&S31252[44], 30}, {NULL}};
const Segment SGI35238[] = {{&S31252[30], 6}, {NULL}};
const Segment SGI35239[] = {{&S31252[122], 3}, {NULL}};
const Segment SGI35240[] = {{&S31252[88], 5}, {NULL}};
const Segment SGI35241[] = {{&S31255[44], 7}, {NULL}};
const Segment SGI35242[] = {{&S31255[44], 30}, {NULL}};
const Segment SGI35243[] = {{&S31255[30], 6}, {NULL}};
const Segment SGI35244[] = {{&S31255[122], 3}, {NULL}};
const Segment SGI35245[] = {{&S31255[88], 5}, {NULL}};
const Segment SGI35246[] = {{&S31260[44], 7}, {NULL}};
const Segment SGI35247[] = {{&S31260[44], 29}, {NULL}};
const Segment SGI35248[] = {{&S31260[30], 6}, {NULL}};
const Segment SGI35249[] = {{&S31260[121], 3}, {NULL}};
const Segment SGI35250[] = {{&S31260[87], 5}, {NULL}};
const Segment SGI35251[] = {{&S31262[38], 7}, {NULL}};
const Segment SGI35252[] = {{&S31262[31], 6}, {NULL}};
const Segment SGI35253[] = {{&S31263[44], 7}, {NULL}};
const Segment SGI35254[] = {{&S31263[44], 29}, {NULL}};
const Segment SGI35255[] = {{&S31263[30], 6}, {NULL}};
const Segment SGI35256[] = {{&S31263[121], 3}, {NULL}};
const Segment SGI35257[] = {{&S31263[87], 5}, {NULL}};
const Segment SGI35258[] = {{&S33266[44], 23}, {NULL}};
const Segment SGI35259[] = {{&S33266[44], 34}, {NULL}};
const Segment SGI35260[] = {{&S33266[30], 6}, {NULL}};
const Segment SGI35261[] = {{&S33266[126], 3}, {NULL}};
const Segment SGI35262[] = {{&S33266[92], 5}, {NULL}};
const Segment SGI35263[] = {{&S33268[44], 23}, {NULL}};
const Segment SGI35264[] = {{&S33268[44], 34}, {NULL}};
const Segment SGI35265[] = {{&S33268[30], 6}, {NULL}};
const Segment SGI35266[] = {{&S33268[126], 3}, {NULL}};
const Segment SGI35267[] = {{&S33268[92], 5}, {NULL}};
const Segment SGI35268[] = {{&S30604[44], 11}, {NULL}};
const Segment SGI35269[] = {{&S30604[44], 22}, {NULL}};
const Segment SGI35270[] = {{&S30604[30], 6}, {NULL}};
const Segment SGI35271[] = {{&S30604[114], 3}, {NULL}};
const Segment SGI35272[] = {{&S30604[80], 5}, {NULL}};
const Segment SGI35273[] = {{&S30605[44], 11}, {NULL}};
const Segment SGI35274[] = {{&S30605[44], 22}, {NULL}};
const Segment SGI35275[] = {{&S30605[30], 6}, {NULL}};
const Segment SGI35276[] = {{&S30605[114], 3}, {NULL}};
const Segment SGI35277[] = {{&S30605[80], 5}, {NULL}};
const Segment SGI35278[] = {{&S30623[38], 11}, {NULL}};
const Segment SGI35279[] = {{&S30623[31], 6}, {NULL}};
const Segment SGI35280[] = {{&S30632[44], 11}, {NULL}};
const Segment SGI35281[] = {{&S30632[44], 22}, {NULL}};
const Segment SGI35282[] = {{&S30632[30], 6}, {NULL}};
const Segment SGI35283[] = {{&S30632[113], 3}, {NULL}};
const Segment SGI35284[] = {{&S30632[80], 5}, {NULL}};
const Segment SGI35285[] = {{&S30636[44], 11}, {NULL}};
const Segment SGI35286[] = {{&S30636[44], 28}, {NULL}};
const Segment SGI35287[] = {{&S30636[30], 6}, {NULL}};
const Segment SGI35288[] = {{&S30636[120], 3}, {NULL}};
const Segment SGI35289[] = {{&S30636[86], 5}, {NULL}};
const Segment SGI35290[] = {{&S31448[44], 9}, {NULL}};
const Segment SGI35291[] = {{&S31448[44], 26}, {NULL}};
const Segment SGI35292[] = {{&S31448[30], 6}, {NULL}};
const Segment SGI35293[] = {{&S31448[118], 3}, {NULL}};
const Segment SGI35294[] = {{&S31448[84], 5}, {NULL}};
const Segment SGI35295[] = {{&S31463[38], 9}, {NULL}};
const Segment SGI35296[] = {{&S31463[31], 6}, {NULL}};
const Segment SGI35297[] = {{&S33485[44], 19}, {NULL}};
const Segment SGI35298[] = {{&S33485[44], 38}, {NULL}};
const Segment SGI35299[] = {{&S33485[30], 6}, {NULL}};
const Segment SGI35300[] = {{&S33485[130], 3}, {NULL}};
const Segment SGI35301[] = {{&S33485[96], 5}, {NULL}};
const Segment SGI35302[] = {{&S33488[44], 19}, {NULL}};
const Segment SGI35303[] = {{&S33488[44], 37}, {NULL}};
const Segment SGI35304[] = {{&S33488[30], 6}, {NULL}};
const Segment SGI35305[] = {{&S33488[129], 3}, {NULL}};
const Segment SGI35306[] = {{&S33488[95], 5}, {NULL}};
const Segment SGI35307[] = {{&S33489[44], 7}, {NULL}};
const Segment SGI35308[] = {{&S33489[44], 25}, {NULL}};
const Segment SGI35309[] = {{&S33489[30], 6}, {NULL}};
const Segment SGI35310[] = {{&S33489[117], 3}, {NULL}};
const Segment SGI35311[] = {{&S33489[83], 5}, {NULL}};
const Segment SGI35312[] = {{&S33493[44], 67}, {NULL}};
const Segment SGI35313[] = {{&S33493[44], 85}, {NULL}};
const Segment SGI35314[] = {{&S33493[30], 6}, {NULL}};
const Segment SGI35315[] = {{&S33493[177], 3}, {NULL}};
const Segment SGI35316[] = {{&S33493[143], 5}, {NULL}};
const Segment SGI35317[] = {{&S33495[38], 7}, {NULL}};
const Segment SGI35318[] = {{&S33495[31], 6}, {NULL}};
const Segment SGI35319[] = {{&S33496[44], 7}, {NULL}};
const Segment SGI35320[] = {{&S33496[44], 25}, {NULL}};
const Segment SGI35321[] = {{&S33496[30], 6}, {NULL}};
const Segment SGI35322[] = {{&S33496[117], 3}, {NULL}};
const Segment SGI35323[] = {{&S33496[83], 5}, {NULL}};
const Segment SGI35324[] = {{&S33498[44], 19}, {NULL}};
const Segment SGI35325[] = {{&S33498[44], 37}, {NULL}};
const Segment SGI35326[] = {{&S33498[30], 6}, {NULL}};
const Segment SGI35327[] = {{&S33498[129], 3}, {NULL}};
const Segment SGI35328[] = {{&S33498[95], 5}, {NULL}};
const Segment SGI35329[] = {{&S33499[44], 7}, {NULL}};
const Segment SGI35330[] = {{&S33499[44], 25}, {NULL}};
const Segment SGI35331[] = {{&S33499[30], 6}, {NULL}};
const Segment SGI35332[] = {{&S33499[57], 3}, {NULL}};
const Segment SGI35333[] = {{&S33499[83], 7}, {NULL}};
const Segment SGI35334[] = {{&S33499[139], 8}, {NULL}};
const Segment SGI35335[] = {{&S34946[44], 9}, {NULL}};
const Segment SGI35336[] = {{&S34946[44], 20}, {NULL}};
const Segment SGI35337[] = {{&S34946[30], 6}, {NULL}};
const Segment SGI35338[] = {{&S34946[112], 3}, {NULL}};
const Segment SGI35339[] = {{&S34946[78], 5}, {NULL}};
const Segment SGI35340[] = {{&S31690[44], 11}, {NULL}};
const Segment SGI35341[] = {{&S31690[44], 28}, {NULL}};
const Segment SGI35342[] = {{&S31690[30], 6}, {NULL}};
const Segment SGI35343[] = {{&S31690[120], 3}, {NULL}};
const Segment SGI35344[] = {{&S31690[86], 5}, {NULL}};
const Segment SGI35345[] = {{&S31691[44], 30}, {NULL}};
const Segment SGI35346[] = {{&S31691[44], 47}, {NULL}};
const Segment SGI35347[] = {{&S31691[30], 6}, {NULL}};
const Segment SGI35348[] = {{&S31691[139], 3}, {NULL}};
const Segment SGI35349[] = {{&S31691[105], 5}, {NULL}};
const Segment SGI35350[] = {{&S31694[38], 11}, {NULL}};
const Segment SGI35351[] = {{&S31694[31], 6}, {NULL}};
const Segment SGI35352[] = {{&S35052[44], 9}, {NULL}};
const Segment SGI35353[] = {{&S35052[44], 20}, {NULL}};
const Segment SGI35354[] = {{&S35052[30], 6}, {NULL}};
const Segment SGI35355[] = {{&S35052[112], 3}, {NULL}};
const Segment SGI35356[] = {{&S31733[44], 10}, {NULL}};
const Segment SGI35357[] = {{&S35052[78], 5}, {NULL}};
const Segment SGI35358[] = {{&S31733[44], 27}, {NULL}};
const Segment SGI35359[] = {{&S31733[30], 6}, {NULL}};
const Segment SGI35360[] = {{&S31733[119], 3}, {NULL}};
const Segment SGI35361[] = {{&S31733[85], 5}, {NULL}};
const Segment SGI35362[] = {{&S31734[38], 10}, {NULL}};
const Segment SGI35363[] = {{&S31734[31], 6}, {NULL}};
const Segment SGI35364[] = {{&S35055[44], 9}, {NULL}};
const Segment SGI35365[] = {{&S35055[44], 21}, {NULL}};
const Segment SGI35366[] = {{&S35055[30], 6}, {NULL}};
const Segment SGI35367[] = {{&S31735[44], 11}, {NULL}};
const Segment SGI35368[] = {{&S31735[44], 28}, {NULL}};
const Segment SGI35369[] = {{&S35055[113], 3}, {NULL}};
const Segment SGI35370[] = {{&S31735[30], 6}, {NULL}};
const Segment SGI35371[] = {{&S35055[79], 5}, {NULL}};
const Segment SGI35372[] = {{&S31735[120], 3}, {NULL}};
const Segment SGI35373[] = {{&S31735[86], 5}, {NULL}};
const Segment SGI35374[] = {{&S31740[44], 10}, {NULL}};
const Segment SGI35375[] = {{&S31740[44], 27}, {NULL}};
const Segment SGI35376[] = {{&S31740[30], 6}, {NULL}};
const Segment SGI35377[] = {{&S31740[88], 4}, {NULL}};
const Segment SGI35378[] = {{&S31740[88], 3}, {NULL}};
const Segment SGI35379[] = {{&S31746[38], 11}, {NULL}};
const Segment SGI35380[] = {{&S31746[31], 6}, {NULL}};
const Segment SGI35381[] = {{&S31747[44], 10}, {NULL}};
const Segment SGI35382[] = {{&S31747[44], 27}, {NULL}};
const Segment SGI35383[] = {{&S31747[30], 6}, {NULL}};
const Segment SGI35384[] = {{&S31747[88], 6}, {NULL}};
const Segment SGI35385[] = {{&S31748[42], 10}, {NULL}};
const Segment SGI35386[] = {{&S31748[30], 6}, {NULL}};
const Segment SGI35387[] = {{&S31748[117], 3}, {NULL}};
const Segment SGI35388[] = {{&S31748[83], 5}, {NULL}};
const Segment SGI35389[] = {{&S33734[44], 23}, {NULL}};
const Segment SGI35390[] = {{&S33734[44], 34}, {NULL}};
const Segment SGI35391[] = {{&S33734[30], 6}, {NULL}};
const Segment SGI35392[] = {{&S33734[126], 3}, {NULL}};
const Segment SGI35393[] = {{&S33734[92], 5}, {NULL}};
const Segment SGI35394[] = {{&S33735[44], 101}, {NULL}};
const Segment SGI35395[] = {{&S33735[44], 112}, {NULL}};
const Segment SGI35396[] = {{&S33735[30], 6}, {NULL}};
const Segment SGI35397[] = {{&S33735[204], 3}, {NULL}};
const Segment SGI35398[] = {{&S33735[170], 5}, {NULL}};
const Segment SGI35399[] = {{&S33736[44], 18}, {NULL}};
const Segment SGI35400[] = {{&S33736[44], 29}, {NULL}};
const Segment SGI35401[] = {{&S33736[30], 6}, {NULL}};
const Segment SGI35402[] = {{&S33736[121], 3}, {NULL}};
const Segment SGI35403[] = {{&S33736[87], 5}, {NULL}};
const Segment SGI35404[] = {{&S33737[44], 18}, {NULL}};
const Segment SGI35405[] = {{&S33737[44], 30}, {NULL}};
const Segment SGI35406[] = {{&S33737[30], 6}, {NULL}};
const Segment SGI35407[] = {{&S33737[122], 3}, {NULL}};
const Segment SGI35408[] = {{&S33737[88], 5}, {NULL}};
const Segment SGI35409[] = {{&S33738[42], 23}, {NULL}};
const Segment SGI35410[] = {{&S33738[30], 6}, {NULL}};
const Segment SGI35411[] = {{&S33738[124], 3}, {NULL}};
const Segment SGI35412[] = {{&S33738[90], 5}, {NULL}};
const Segment SGI35413[] = {{&S33740[38], 18}, {NULL}};
const Segment SGI35414[] = {{&S33740[31], 6}, {NULL}};
const Segment SGI35415[] = {{&S33741[44], 18}, {NULL}};
const Segment SGI35416[] = {{&S33741[44], 29}, {NULL}};
const Segment SGI35417[] = {{&S33741[30], 6}, {NULL}};
const Segment SGI35418[] = {{&S33741[90], 4}, {NULL}};
const Segment SGI35419[] = {{&S33741[90], 3}, {NULL}};
const Segment SGI35420[] = {{&S33752[44], 11}, {NULL}};
const Segment SGI35421[] = {{&S33752[44], 22}, {NULL}};
const Segment SGI35422[] = {{&S33752[30], 6}, {NULL}};
const Segment SGI35423[] = {{&S33752[114], 3}, {NULL}};
const Segment SGI35424[] = {{&S33752[80], 5}, {NULL}};
const Segment SGI35425[] = {{&S33754[44], 18}, {NULL}};
const Segment SGI35426[] = {{&S33754[44], 29}, {NULL}};
const Segment SGI35427[] = {{&S33754[30], 6}, {NULL}};
const Segment SGI35428[] = {{&S33754[90], 6}, {NULL}};
const Segment SGI35429[] = {{&S33757[44], 29}, {NULL}};
const Segment SGI35430[] = {{&S33757[30], 6}, {NULL}};
const Segment SGI35431[] = {{&S31820[38], 9}, {NULL}};
const Segment SGI35432[] = {{&S31820[31], 6}, {NULL}};
const Segment SGI35433[] = {{&S31821[41], 9}, {NULL}};
const Segment SGI35434[] = {{&S31821[30], 6}, {NULL}};
const Segment SGI35435[] = {{&S31821[109], 3}, {NULL}};
const Segment SGI35436[] = {{&S31821[75], 5}, {NULL}};
const Segment SGI35437[] = {{&S31822[44], 17}, {NULL}};
const Segment SGI35438[] = {{&S31822[44], 28}, {NULL}};
const Segment SGI35439[] = {{&S31822[30], 6}, {NULL}};
const Segment SGI35440[] = {{&S31822[120], 3}, {NULL}};
const Segment SGI35441[] = {{&S31822[86], 5}, {NULL}};
const Segment SGI35442[] = {{&S31823[44], 16}, {NULL}};
const Segment SGI35443[] = {{&S31823[44], 27}, {NULL}};
const Segment SGI35444[] = {{&S31823[30], 6}, {NULL}};
const Segment SGI35445[] = {{&S31823[119], 3}, {NULL}};
const Segment SGI35446[] = {{&S31823[85], 5}, {NULL}};
const Segment SGI35447[] = {{&S31824[44], 9}, {NULL}};
const Segment SGI35448[] = {{&S31824[44], 20}, {NULL}};
const Segment SGI35449[] = {{&S31824[30], 6}, {NULL}};
const Segment SGI35450[] = {{&S31824[112], 3}, {NULL}};
const Segment SGI35451[] = {{&S31824[78], 5}, {NULL}};
const Segment SGI35452[] = {{&S31825[44], 10}, {NULL}};
const Segment SGI35453[] = {{&S31825[44], 21}, {NULL}};
const Segment SGI35454[] = {{&S31825[30], 6}, {NULL}};
const Segment SGI35455[] = {{&S31825[113], 3}, {NULL}};
const Segment SGI35456[] = {{&S31825[79], 5}, {NULL}};
const Segment SGI35457[] = {{&S31826[44], 14}, {NULL}};
const Segment SGI35458[] = {{&S31826[44], 25}, {NULL}};
const Segment SGI35459[] = {{&S31826[30], 6}, {NULL}};
const Segment SGI35460[] = {{&S31826[117], 3}, {NULL}};
const Segment SGI35461[] = {{&S31826[83], 5}, {NULL}};
const Segment SGI35462[] = {{&S31827[44], 10}, {NULL}};
const Segment SGI35463[] = {{&S31827[44], 21}, {NULL}};
const Segment SGI35464[] = {{&S31827[30], 6}, {NULL}};
const Segment SGI35465[] = {{&S31827[113], 3}, {NULL}};
const Segment SGI35466[] = {{&S31827[79], 5}, {NULL}};
const Segment SGI35467[] = {{&S33792[44], 8}, {NULL}};
const Segment SGI35468[] = {{&S33792[44], 29}, {NULL}};
const Segment SGI35469[] = {{&S33792[30], 6}, {NULL}};
const Segment SGI35470[] = {{&S33792[121], 3}, {NULL}};
const Segment SGI35471[] = {{&S33792[87], 5}, {NULL}};
const Segment SGI35472[] = {{&S31828[44], 21}, {NULL}};
const Segment SGI35473[] = {{&S31828[44], 32}, {NULL}};
const Segment SGI35474[] = {{&S31828[30], 6}, {NULL}};
const Segment SGI35475[] = {{&S31828[124], 3}, {NULL}};
const Segment SGI35476[] = {{&S31828[90], 5}, {NULL}};
const Segment SGI35477[] = {{&S33793[44], 7}, {NULL}};
const Segment SGI35478[] = {{&S33793[44], 28}, {NULL}};
const Segment SGI35479[] = {{&S33793[30], 6}, {NULL}};
const Segment SGI35480[] = {{&S33793[120], 3}, {NULL}};
const Segment SGI35481[] = {{&S33793[86], 5}, {NULL}};
const Segment SGI35482[] = {{&S31829[44], 9}, {NULL}};
const Segment SGI35483[] = {{&S31829[44], 20}, {NULL}};
const Segment SGI35484[] = {{&S31829[30], 6}, {NULL}};
const Segment SGI35485[] = {{&S31829[81], 4}, {NULL}};
const Segment SGI35486[] = {{&S31829[81], 3}, {NULL}};
const Segment SGI35487[] = {{&S33794[44], 8}, {NULL}};
const Segment SGI35488[] = {{&S33794[44], 29}, {NULL}};
const Segment SGI35489[] = {{&S33794[30], 6}, {NULL}};
const Segment SGI35490[] = {{&S33794[121], 3}, {NULL}};
const Segment SGI35491[] = {{&S31830[44], 14}, {NULL}};
const Segment SGI35492[] = {{&S33794[87], 5}, {NULL}};
const Segment SGI35493[] = {{&S31830[44], 25}, {NULL}};
const Segment SGI35494[] = {{&S31830[30], 6}, {NULL}};
const Segment SGI35495[] = {{&S31830[117], 3}, {NULL}};
const Segment SGI35496[] = {{&S31830[83], 5}, {NULL}};
const Segment SGI35497[] = {{&S33795[43], 8}, {NULL}};
const Segment SGI35498[] = {{&S33795[43], 29}, {NULL}};
const Segment SGI35499[] = {{&S33795[120], 3}, {NULL}};
const Segment SGI35500[] = {{&S31831[44], 14}, {NULL}};
const Segment SGI35501[] = {{&S33795[86], 5}, {NULL}};
const Segment SGI35502[] = {{&S31831[44], 25}, {NULL}};
const Segment SGI35503[] = {{&S31831[30], 6}, {NULL}};
const Segment SGI35504[] = {{&S31831[117], 3}, {NULL}};
const Segment SGI35505[] = {{&S31831[83], 5}, {NULL}};
const Segment SGI35506[] = {{&S33796[44], 8}, {NULL}};
const Segment SGI35507[] = {{&S33796[44], 29}, {NULL}};
const Segment SGI35508[] = {{&S33796[30], 6}, {NULL}};
const Segment SGI35509[] = {{&S33796[121], 3}, {NULL}};
const Segment SGI35510[] = {{&S33796[87], 5}, {NULL}};
const Segment SGI35511[] = {{&S31832[44], 18}, {NULL}};
const Segment SGI35512[] = {{&S31832[44], 29}, {NULL}};
const Segment SGI35513[] = {{&S31832[30], 6}, {NULL}};
const Segment SGI35514[] = {{&S31832[121], 3}, {NULL}};
const Segment SGI35515[] = {{&S31832[87], 5}, {NULL}};
const Segment SGI35516[] = {{&S33797[44], 7}, {NULL}};
const Segment SGI35517[] = {{&S33797[44], 28}, {NULL}};
const Segment SGI35518[] = {{&S33797[30], 6}, {NULL}};
const Segment SGI35519[] = {{&S33797[120], 3}, {NULL}};
const Segment SGI35520[] = {{&S33797[86], 5}, {NULL}};
const Segment SGI35521[] = {{&S31833[44], 9}, {NULL}};
const Segment SGI35522[] = {{&S31833[44], 20}, {NULL}};
const Segment SGI35523[] = {{&S31833[30], 6}, {NULL}};
const Segment SGI35524[] = {{&S31833[81], 6}, {NULL}};
const Segment SGI35525[] = {{&S33798[44], 8}, {NULL}};
const Segment SGI35526[] = {{&S33798[44], 29}, {NULL}};
const Segment SGI35527[] = {{&S33798[30], 6}, {NULL}};
const Segment SGI35528[] = {{&S33798[121], 3}, {NULL}};
const Segment SGI35529[] = {{&S33798[87], 5}, {NULL}};
const Segment SGI35530[] = {{&S31834[44], 18}, {NULL}};
const Segment SGI35531[] = {{&S31834[30], 6}, {NULL}};
const Segment SGI35532[] = {{&S31834[123], 3}, {NULL}};
const Segment SGI35533[] = {{&S31834[89], 5}, {NULL}};
const Segment SGI35534[] = {{&S33799[44], 8}, {NULL}};
const Segment SGI35535[] = {{&S33799[44], 25}, {NULL}};
const Segment SGI35536[] = {{&S33799[30], 6}, {NULL}};
const Segment SGI35537[] = {{&S33799[118], 3}, {NULL}};
const Segment SGI35538[] = {{&S33799[83], 6}, {NULL}};
const Segment SGI35539[] = {{&S31835[44], 14}, {NULL}};
const Segment SGI35540[] = {{&S31835[44], 25}, {NULL}};
const Segment SGI35541[] = {{&S31835[30], 6}, {NULL}};
const Segment SGI35542[] = {{&S31835[117], 3}, {NULL}};
const Segment SGI35543[] = {{&S31835[83], 5}, {NULL}};
const Segment SGI35544[] = {{&S33800[44], 8}, {NULL}};
const Segment SGI35545[] = {{&S33800[44], 18}, {NULL}};
const Segment SGI35546[] = {{&S33800[30], 6}, {NULL}};
const Segment SGI35547[] = {{&S33800[110], 3}, {NULL}};
const Segment SGI35548[] = {{&S33800[76], 5}, {NULL}};
const Segment SGI35549[] = {{&S31836[38], 10}, {NULL}};
const Segment SGI35550[] = {{&S31836[31], 6}, {NULL}};
const Segment SGI35551[] = {{&S33801[38], 8}, {NULL}};
const Segment SGI35552[] = {{&S33801[31], 6}, {NULL}};
const Segment SGI35553[] = {{&S31837[38], 10}, {NULL}};
const Segment SGI35554[] = {{&S31837[31], 6}, {NULL}};
const Segment SGI35555[] = {{&S31838[44], 21}, {NULL}};
const Segment SGI35556[] = {{&S31838[44], 32}, {NULL}};
const Segment SGI35557[] = {{&S31838[30], 6}, {NULL}};
const Segment SGI35558[] = {{&S31838[124], 3}, {NULL}};
const Segment SGI35559[] = {{&S31838[90], 5}, {NULL}};
const Segment SGI35560[] = {{&S31839[44], 14}, {NULL}};
const Segment SGI35561[] = {{&S31839[44], 25}, {NULL}};
const Segment SGI35562[] = {{&S31839[30], 6}, {NULL}};
const Segment SGI35563[] = {{&S31839[117], 3}, {NULL}};
const Segment SGI35564[] = {{&S31839[83], 5}, {NULL}};
const Segment SGI35565[] = {{&S33807[44], 8}, {NULL}};
const Segment SGI35566[] = {{&S33807[44], 29}, {NULL}};
const Segment SGI35567[] = {{&S33807[30], 6}, {NULL}};
const Segment SGI35568[] = {{&S33807[123], 3}, {NULL}};
const Segment SGI35569[] = {{&S33807[87], 7}, {NULL}};
const Segment SGI35570[] = {{&S31840[8], 1}, {NULL}};
const Segment SGI35571[] = {{&S33807[143], 8}, {NULL}};
const Segment SGI35572[] = {{&S31840[53], 12}, {NULL}};
const Segment SGI35573[] = {{&S31840[39], 6}, {NULL}};
const Segment SGI35574[] = {{&S31840[113], 3}, {NULL}};
const Segment SGI35575[] = {{&S31840[79], 5}, {NULL}};
const Segment SGI35576[] = {{&S31842[44], 10}, {NULL}};
const Segment SGI35577[] = {{&S31842[44], 21}, {NULL}};
const Segment SGI35578[] = {{&S31842[30], 6}, {NULL}};
const Segment SGI35579[] = {{&S31842[79], 5}, {NULL}};
const Segment SGI35580[] = {{&S31242[44], 7}, {NULL}};
const Segment SGI35581[] = {{&S31242[44], 30}, {NULL}};
const Segment SGI35582[] = {{&S31242[30], 6}, {NULL}};
const Segment SGI35583[] = {{&S31242[122], 3}, {NULL}};
const Segment SGI35584[] = {{&S31242[88], 5}, {NULL}};
const Segment SGI35585[] = {{&S31245[44], 7}, {NULL}};
const Segment SGI35586[] = {{&S31245[44], 28}, {NULL}};
const Segment SGI35587[] = {{&S31245[30], 6}, {NULL}};
const Segment SGI35588[] = {{&S31245[120], 3}, {NULL}};
const Segment SGI35589[] = {{&S31245[86], 5}, {NULL}};
const Segment SGI35590[] = {{&S31247[44], 7}, {NULL}};
const Segment SGI35591[] = {{&S31247[44], 28}, {NULL}};
const Segment SGI35592[] = {{&S31247[30], 6}, {NULL}};
const Segment SGI35593[] = {{&S31247[120], 3}, {NULL}};
const Segment SGI35594[] = {{&S31247[86], 5}, {NULL}};
const Segment SGI35595[] = {{&S31249[44], 11}, {NULL}};
const Segment SGI35596[] = {{&S31249[44], 35}, {NULL}};
const Segment SGI35597[] = {{&S31249[30], 6}, {NULL}};
const Segment SGI35598[] = {{&S31249[127], 3}, {NULL}};
const Segment SGI35599[] = {{&S31249[93], 5}, {NULL}};
const Segment SGI35600[] = {{&S31259[44], 7}, {NULL}};
const Segment SGI35601[] = {{&S31259[44], 31}, {NULL}};
const Segment SGI35602[] = {{&S31259[30], 6}, {NULL}};
const Segment SGI35603[] = {{&S31259[123], 3}, {NULL}};
const Segment SGI35604[] = {{&S31259[89], 5}, {NULL}};
const Segment SGI35605[] = {{&S31271[38], 7}, {NULL}};
const Segment SGI35606[] = {{&S31271[31], 6}, {NULL}};
const Segment SGI35607[] = {{&S31274[44], 7}, {NULL}};
const Segment SGI35608[] = {{&S31274[44], 31}, {NULL}};
const Segment SGI35609[] = {{&S31274[30], 6}, {NULL}};
const Segment SGI35610[] = {{&S31274[123], 3}, {NULL}};
const Segment SGI35611[] = {{&S31274[89], 5}, {NULL}};
const Segment SGI35612[] = {{&S31275[44], 7}, {NULL}};
const Segment SGI35613[] = {{&S31275[44], 30}, {NULL}};
const Segment SGI35614[] = {{&S31275[30], 6}, {NULL}};
const Segment SGI35615[] = {{&S31275[122], 3}, {NULL}};
const Segment SGI35616[] = {{&S31275[88], 5}, {NULL}};
const Segment SGI35617[] = {{&S31450[44], 9}, {NULL}};
const Segment SGI35618[] = {{&S31450[44], 20}, {NULL}};
const Segment SGI35619[] = {{&S31450[30], 6}, {NULL}};
const Segment SGI35620[] = {{&S31450[112], 3}, {NULL}};
const Segment SGI35621[] = {{&S31450[78], 5}, {NULL}};
const Segment SGI35622[] = {{&S31466[38], 9}, {NULL}};
const Segment SGI35623[] = {{&S31466[31], 6}, {NULL}};
const Segment SGI35624[] = {{&S43255[44], 7}, {NULL}};
const Segment SGI35625[] = {{&S43255[44], 26}, {NULL}};
const Segment SGI35626[] = {{&S43255[30], 6}, {NULL}};
const Segment SGI35627[] = {{&S43255[118], 3}, {NULL}};
const Segment SGI35628[] = {{&S43255[84], 5}, {NULL}};
const Segment SGI35629[] = {{&S43256[44], 7}, {NULL}};
const Segment SGI35630[] = {{&S43256[44], 27}, {NULL}};
const Segment SGI35631[] = {{&S43256[30], 6}, {NULL}};
const Segment SGI35632[] = {{&S43256[119], 3}, {NULL}};
const Segment SGI35633[] = {{&S43256[85], 5}, {NULL}};
const Segment SGI35634[] = {{&S43257[44], 7}, {NULL}};
const Segment SGI35635[] = {{&S43257[44], 26}, {NULL}};
const Segment SGI35636[] = {{&S43257[30], 6}, {NULL}};
const Segment SGI35637[] = {{&S43257[118], 3}, {NULL}};
const Segment SGI35638[] = {{&S43257[84], 5}, {NULL}};
const Segment SGI35639[] = {{&S43258[44], 7}, {NULL}};
const Segment SGI35640[] = {{&S43258[44], 26}, {NULL}};
const Segment SGI35641[] = {{&S43258[30], 6}, {NULL}};
const Segment SGI35642[] = {{&S43258[118], 3}, {NULL}};
const Segment SGI35643[] = {{&S43258[84], 5}, {NULL}};
const Segment SGI35644[] = {{&S43259[44], 7}, {NULL}};
const Segment SGI35645[] = {{&S43259[44], 27}, {NULL}};
const Segment SGI35646[] = {{&S43259[30], 6}, {NULL}};
const Segment SGI35647[] = {{&S43259[119], 3}, {NULL}};
const Segment SGI35648[] = {{&S43259[85], 5}, {NULL}};
const Segment SGI35649[] = {{&S43260[38], 7}, {NULL}};
const Segment SGI35650[] = {{&S43260[31], 6}, {NULL}};
const Segment SGI35651[] = {{&S43308[44], 10}, {NULL}};
const Segment SGI35652[] = {{&S43308[44], 21}, {NULL}};
const Segment SGI35653[] = {{&S43308[30], 6}, {NULL}};
const Segment SGI35654[] = {{&S43308[113], 3}, {NULL}};
const Segment SGI35655[] = {{&S43308[79], 5}, {NULL}};
const Segment SGI35656[] = {{&S43309[44], 11}, {NULL}};
const Segment SGI35657[] = {{&S43309[44], 22}, {NULL}};
const Segment SGI35658[] = {{&S43309[30], 6}, {NULL}};
const Segment SGI35659[] = {{&S43309[114], 3}, {NULL}};
const Segment SGI35660[] = {{&S43309[80], 5}, {NULL}};
const Segment SGI35661[] = {{&S43310[44], 11}, {NULL}};
const Segment SGI35662[] = {{&S43310[44], 22}, {NULL}};
const Segment SGI35663[] = {{&S43310[30], 6}, {NULL}};
const Segment SGI35664[] = {{&S43310[114], 3}, {NULL}};
const Segment SGI35665[] = {{&S43310[80], 5}, {NULL}};
const Segment SGI35666[] = {{&S43311[38], 10}, {NULL}};
const Segment SGI35667[] = {{&S43311[31], 6}, {NULL}};
const Segment SGI35668[] = {{&S43312[44], 10}, {NULL}};
const Segment SGI35669[] = {{&S43312[44], 21}, {NULL}};
const Segment SGI35670[] = {{&S43312[30], 6}, {NULL}};
const Segment SGI35671[] = {{&S43312[82], 4}, {NULL}};
const Segment SGI35672[] = {{&S43312[82], 3}, {NULL}};
const Segment SGI35673[] = {{&S43313[38], 11}, {NULL}};
const Segment SGI35674[] = {{&S43313[31], 6}, {NULL}};
const Segment SGI35675[] = {{&S43314[42], 10}, {NULL}};
const Segment SGI35676[] = {{&S43314[30], 6}, {NULL}};
const Segment SGI35677[] = {{&S43314[111], 3}, {NULL}};
const Segment SGI35678[] = {{&S43314[77], 5}, {NULL}};
const Segment SGI35679[] = {{&S31692[44], 11}, {NULL}};
const Segment SGI35680[] = {{&S31692[44], 28}, {NULL}};
const Segment SGI35681[] = {{&S31692[30], 6}, {NULL}};
const Segment SGI35682[] = {{&S31692[120], 3}, {NULL}};
const Segment SGI35683[] = {{&S31692[86], 5}, {NULL}};
const Segment SGI35684[] = {{&S31697[38], 11}, {NULL}};
const Segment SGI35685[] = {{&S31697[31], 6}, {NULL}};
const Segment SGI35686[] = {{&S31698[42], 11}, {NULL}};
const Segment SGI35687[] = {{&S31698[30], 6}, {NULL}};
const Segment SGI35688[] = {{&S31698[118], 3}, {NULL}};
const Segment SGI35689[] = {{&S31698[84], 5}, {NULL}};
const Segment SGI35690[] = {{&S31736[44], 10}, {NULL}};
const Segment SGI35691[] = {{&S31736[44], 27}, {NULL}};
const Segment SGI35692[] = {{&S31736[30], 6}, {NULL}};
const Segment SGI35693[] = {{&S31736[119], 3}, {NULL}};
const Segment SGI35694[] = {{&S31736[85], 5}, {NULL}};
const Segment SGI35695[] = {{&S31739[44], 11}, {NULL}};
const Segment SGI35696[] = {{&S31739[44], 28}, {NULL}};
const Segment SGI35697[] = {{&S31739[30], 6}, {NULL}};
const Segment SGI35698[] = {{&S31739[120], 3}, {NULL}};
const Segment SGI35699[] = {{&S31739[86], 5}, {NULL}};
const Segment SGI35700[] = {{&S43587[44], 13}, {NULL}};
const Segment SGI35701[] = {{&S43587[44], 25}, {NULL}};
const Segment SGI35702[] = {{&S43587[30], 6}, {NULL}};
const Segment SGI35703[] = {{&S43587[117], 3}, {NULL}};
const Segment SGI35704[] = {{&S43587[83], 5}, {NULL}};
const Segment SGI35705[] = {{&S31742[38], 10}, {NULL}};
const Segment SGI35706[] = {{&S31742[31], 6}, {NULL}};
const Segment SGI35707[] = {{&S43588[44], 13}, {NULL}};
const Segment SGI35708[] = {{&S43588[44], 25}, {NULL}};
const Segment SGI35709[] = {{&S43588[30], 6}, {NULL}};
const Segment SGI35710[] = {{&S43588[117], 3}, {&S43588[159], 3}, {NULL}};
const Segment SGI35711[] = {{&S43588[83], 5}, {NULL}};
const Segment SGI35712[] = {{&S43590[38], 13}, {NULL}};
const Segment SGI35713[] = {{&S43590[31], 6}, {NULL}};
const Segment SGI35714[] = {{&S43591[44], 13}, {NULL}};
const Segment SGI35715[] = {{&S43591[44], 29}, {NULL}};
const Segment SGI35716[] = {{&S43591[30], 6}, {NULL}};
const Segment SGI35717[] = {{&S43591[121], 3}, {NULL}};
const Segment SGI35718[] = {{&S43591[87], 5}, {NULL}};
const Segment SGI35719[] = {{&S32579[44], 17}, {NULL}};
const Segment SGI35720[] = {{&S32579[44], 38}, {NULL}};
const Segment SGI35721[] = {{&S32579[30], 6}, {NULL}};
const Segment SGI35722[] = {{&S32579[130], 3}, {NULL}};
const Segment SGI35723[] = {{&S32579[96], 5}, {NULL}};
const Segment SGI35724[] = {{&S32580[44], 17}, {NULL}};
const Segment SGI35725[] = {{&S32580[44], 38}, {NULL}};
const Segment SGI35726[] = {{&S32580[30], 6}, {NULL}};
const Segment SGI35727[] = {{&S32580[130], 3}, {NULL}};
const Segment SGI35728[] = {{&S32580[96], 5}, {NULL}};
const Segment SGI35729[] = {{&S32581[44], 14}, {NULL}};
const Segment SGI35730[] = {{&S32581[44], 35}, {NULL}};
const Segment SGI35731[] = {{&S32581[30], 6}, {NULL}};
const Segment SGI35732[] = {{&S32581[127], 3}, {NULL}};
const Segment SGI35733[] = {{&S32581[93], 5}, {NULL}};
const Segment SGI35734[] = {{&S32582[44], 17}, {NULL}};
const Segment SGI35735[] = {{&S32582[44], 38}, {NULL}};
const Segment SGI35736[] = {{&S32582[30], 6}, {NULL}};
const Segment SGI35737[] = {{&S32582[130], 3}, {NULL}};
const Segment SGI35738[] = {{&S32582[96], 5}, {NULL}};
const Segment SGI35739[] = {{&S32583[44], 14}, {NULL}};
const Segment SGI35740[] = {{&S32583[44], 35}, {NULL}};
const Segment SGI35741[] = {{&S32583[30], 6}, {NULL}};
const Segment SGI35742[] = {{&S32583[127], 3}, {NULL}};
const Segment SGI35743[] = {{&S32583[93], 5}, {NULL}};
const Segment SGI35744[] = {{&S32584[44], 17}, {NULL}};
const Segment SGI35745[] = {{&S32584[44], 48}, {NULL}};
const Segment SGI35746[] = {{&S32584[30], 6}, {NULL}};
const Segment SGI35747[] = {{&S32584[140], 3}, {NULL}};
const Segment SGI35748[] = {{&S32584[106], 5}, {NULL}};
const Segment SGI35749[] = {{&S32585[44], 14}, {NULL}};
const Segment SGI35750[] = {{&S32585[44], 35}, {NULL}};
const Segment SGI35751[] = {{&S32585[30], 6}, {NULL}};
const Segment SGI35752[] = {{&S32585[127], 3}, {NULL}};
const Segment SGI35753[] = {{&S32585[93], 5}, {NULL}};
const Segment SGI35754[] = {{&S32586[44], 14}, {NULL}};
const Segment SGI35755[] = {{&S32586[44], 35}, {NULL}};
const Segment SGI35756[] = {{&S32586[30], 6}, {NULL}};
const Segment SGI35757[] = {{&S32586[127], 3}, {NULL}};
const Segment SGI35758[] = {{&S32586[93], 5}, {NULL}};
const Segment SGI35759[] = {{&S32587[44], 14}, {NULL}};
const Segment SGI35760[] = {{&S32587[44], 35}, {NULL}};
const Segment SGI35761[] = {{&S32587[30], 6}, {NULL}};
const Segment SGI35762[] = {{&S32587[127], 3}, {NULL}};
const Segment SGI35763[] = {{&S32587[93], 5}, {NULL}};
const Segment SGI35764[] = {{&S32588[44], 14}, {NULL}};
const Segment SGI35765[] = {{&S32588[44], 35}, {NULL}};
const Segment SGI35766[] = {{&S32588[30], 6}, {NULL}};
const Segment SGI35767[] = {{&S32588[127], 3}, {NULL}};
const Segment SGI35768[] = {{&S32588[93], 5}, {NULL}};
const Segment SGI35769[] = {{&S32589[44], 17}, {NULL}};
const Segment SGI35770[] = {{&S32589[44], 38}, {NULL}};
const Segment SGI35771[] = {{&S32589[30], 6}, {NULL}};
const Segment SGI35772[] = {{&S32589[130], 3}, {NULL}};
const Segment SGI35773[] = {{&S32589[96], 5}, {NULL}};
const Segment SGI35774[] = {{&S32590[44], 14}, {NULL}};
const Segment SGI35775[] = {{&S32590[44], 45}, {NULL}};
const Segment SGI35776[] = {{&S32590[30], 6}, {NULL}};
const Segment SGI35777[] = {{&S32590[137], 3}, {NULL}};
const Segment SGI35778[] = {{&S32590[103], 5}, {NULL}};
const Segment SGI35779[] = {{&S32591[44], 17}, {NULL}};
const Segment SGI35780[] = {{&S32591[44], 38}, {NULL}};
const Segment SGI35781[] = {{&S32591[30], 6}, {NULL}};
const Segment SGI35782[] = {{&S32591[130], 3}, {NULL}};
const Segment SGI35783[] = {{&S32591[96], 5}, {NULL}};
const Segment SGI35784[] = {{&S32592[44], 17}, {NULL}};
const Segment SGI35785[] = {{&S32592[44], 38}, {NULL}};
const Segment SGI35786[] = {{&S32592[30], 6}, {NULL}};
const Segment SGI35787[] = {{&S32592[130], 3}, {NULL}};
const Segment SGI35788[] = {{&S32592[96], 5}, {NULL}};
const Segment SGI35789[] = {{&S32593[44], 17}, {NULL}};
const Segment SGI35790[] = {{&S32593[44], 38}, {NULL}};
const Segment SGI35791[] = {{&S32593[30], 6}, {NULL}};
const Segment SGI35792[] = {{&S32593[130], 3}, {NULL}};
const Segment SGI35793[] = {{&S32593[96], 5}, {NULL}};
const Segment SGI35794[] = {{&S32594[44], 17}, {NULL}};
const Segment SGI35795[] = {{&S32594[44], 38}, {NULL}};
const Segment SGI35796[] = {{&S32594[30], 6}, {NULL}};
const Segment SGI35797[] = {{&S32594[130], 3}, {NULL}};
const Segment SGI35798[] = {{&S32594[96], 5}, {NULL}};
const Segment SGI35799[] = {{&S32595[44], 14}, {NULL}};
const Segment SGI35800[] = {{&S32595[44], 35}, {NULL}};
const Segment SGI35801[] = {{&S32595[30], 6}, {NULL}};
const Segment SGI35802[] = {{&S32595[127], 3}, {NULL}};
const Segment SGI35803[] = {{&S32595[93], 5}, {NULL}};
const Segment SGI35804[] = {{&S32596[44], 17}, {NULL}};
const Segment SGI35805[] = {{&S32596[44], 38}, {NULL}};
const Segment SGI35806[] = {{&S32596[30], 6}, {NULL}};
const Segment SGI35807[] = {{&S32596[130], 3}, {NULL}};
const Segment SGI35808[] = {{&S32596[96], 5}, {NULL}};
const Segment SGI35809[] = {{&S32597[44], 17}, {NULL}};
const Segment SGI35810[] = {{&S32597[44], 38}, {NULL}};
const Segment SGI35811[] = {{&S32597[30], 6}, {NULL}};
const Segment SGI35812[] = {{&S32597[130], 3}, {NULL}};
const Segment SGI35813[] = {{&S32597[96], 5}, {NULL}};
const Segment SGI35814[] = {{&S32598[44], 14}, {NULL}};
const Segment SGI35815[] = {{&S32598[44], 35}, {NULL}};
const Segment SGI35816[] = {{&S32598[30], 6}, {NULL}};
const Segment SGI35817[] = {{&S32598[127], 3}, {NULL}};
const Segment SGI35818[] = {{&S32598[93], 5}, {NULL}};
const Segment SGI35819[] = {{&S32599[44], 17}, {NULL}};
const Segment SGI35820[] = {{&S32599[44], 44}, {NULL}};
const Segment SGI35821[] = {{&S32599[30], 6}, {NULL}};
const Segment SGI35822[] = {{&S32599[136], 3}, {NULL}};
const Segment SGI35823[] = {{&S32599[102], 5}, {NULL}};
const Segment SGI35824[] = {{&S32600[44], 17}, {NULL}};
const Segment SGI35825[] = {{&S32600[44], 38}, {NULL}};
const Segment SGI35826[] = {{&S32600[30], 6}, {NULL}};
const Segment SGI35827[] = {{&S32600[130], 3}, {NULL}};
const Segment SGI35828[] = {{&S32600[96], 5}, {NULL}};
const Segment SGI35829[] = {{&S32601[44], 14}, {NULL}};
const Segment SGI35830[] = {{&S32601[44], 35}, {NULL}};
const Segment SGI35831[] = {{&S32601[30], 6}, {NULL}};
const Segment SGI35832[] = {{&S32601[127], 3}, {NULL}};
const Segment SGI35833[] = {{&S32601[93], 5}, {NULL}};
const Segment SGI35834[] = {{&S32602[44], 17}, {NULL}};
const Segment SGI35835[] = {{&S32602[44], 38}, {NULL}};
const Segment SGI35836[] = {{&S32602[30], 6}, {NULL}};
const Segment SGI35837[] = {{&S32602[130], 3}, {NULL}};
const Segment SGI35838[] = {{&S32602[96], 5}, {NULL}};
const Segment SGI35839[] = {{&S32603[44], 14}, {NULL}};
const Segment SGI35840[] = {{&S32603[44], 35}, {NULL}};
const Segment SGI35841[] = {{&S32603[30], 6}, {NULL}};
const Segment SGI35842[] = {{&S32603[127], 3}, {NULL}};
const Segment SGI35843[] = {{&S32603[93], 5}, {NULL}};
const Segment SGI35844[] = {{&S32604[44], 17}, {NULL}};
const Segment SGI35845[] = {{&S32604[44], 38}, {NULL}};
const Segment SGI35846[] = {{&S32604[30], 6}, {NULL}};
const Segment SGI35847[] = {{&S32604[130], 3}, {NULL}};
const Segment SGI35848[] = {{&S32604[96], 5}, {NULL}};
const Segment SGI35849[] = {{&S32605[44], 14}, {NULL}};
const Segment SGI35850[] = {{&S32605[44], 35}, {NULL}};
const Segment SGI35851[] = {{&S32605[30], 6}, {NULL}};
const Segment SGI35852[] = {{&S32605[127], 3}, {NULL}};
const Segment SGI35853[] = {{&S32605[93], 5}, {NULL}};
const Segment SGI35854[] = {{&S32606[44], 14}, {NULL}};
const Segment SGI35855[] = {{&S32606[44], 41}, {NULL}};
const Segment SGI35856[] = {{&S32606[30], 6}, {NULL}};
const Segment SGI35857[] = {{&S32606[133], 3}, {NULL}};
const Segment SGI35858[] = {{&S32606[99], 5}, {NULL}};
const Segment SGI35859[] = {{&S32607[44], 17}, {NULL}};
const Segment SGI35860[] = {{&S32607[44], 43}, {NULL}};
const Segment SGI35861[] = {{&S32607[30], 6}, {NULL}};
const Segment SGI35862[] = {{&S32607[135], 3}, {NULL}};
const Segment SGI35863[] = {{&S32607[101], 5}, {NULL}};
const Segment SGI35864[] = {{&S32608[44], 14}, {NULL}};
const Segment SGI35865[] = {{&S32608[44], 40}, {NULL}};
const Segment SGI35866[] = {{&S32608[30], 6}, {NULL}};
const Segment SGI35867[] = {{&S32608[132], 3}, {NULL}};
const Segment SGI35868[] = {{&S32608[98], 5}, {NULL}};
const Segment SGI35869[] = {{&S32609[44], 17}, {NULL}};
const Segment SGI35870[] = {{&S32609[44], 38}, {NULL}};
const Segment SGI35871[] = {{&S32609[30], 6}, {NULL}};
const Segment SGI35872[] = {{&S32609[130], 3}, {NULL}};
const Segment SGI35873[] = {{&S32609[96], 5}, {NULL}};
const Segment SGI35874[] = {{&S32610[44], 14}, {NULL}};
const Segment SGI35875[] = {{&S32610[44], 35}, {NULL}};
const Segment SGI35876[] = {{&S32610[30], 6}, {NULL}};
const Segment SGI35877[] = {{&S32610[127], 3}, {NULL}};
const Segment SGI35878[] = {{&S32610[93], 5}, {NULL}};
const Segment SGI35879[] = {{&S32611[38], 14}, {NULL}};
const Segment SGI35880[] = {{&S32611[31], 6}, {NULL}};
const Segment SGI35881[] = {{&S32612[44], 17}, {NULL}};
const Segment SGI35882[] = {{&S32612[44], 43}, {NULL}};
const Segment SGI35883[] = {{&S32612[30], 6}, {NULL}};
const Segment SGI35884[] = {{&S32612[135], 3}, {NULL}};
const Segment SGI35885[] = {{&S32612[101], 5}, {NULL}};
const Segment SGI35886[] = {{&S32613[44], 17}, {NULL}};
const Segment SGI35887[] = {{&S32613[44], 38}, {NULL}};
const Segment SGI35888[] = {{&S32613[30], 6}, {NULL}};
const Segment SGI35889[] = {{&S32613[130], 3}, {NULL}};
const Segment SGI35890[] = {{&S32613[96], 5}, {NULL}};
const Segment SGI35891[] = {{&S32614[44], 17}, {NULL}};
const Segment SGI35892[] = {{&S32614[44], 38}, {NULL}};
const Segment SGI35893[] = {{&S32614[30], 6}, {NULL}};
const Segment SGI35894[] = {{&S32614[130], 3}, {NULL}};
const Segment SGI35895[] = {{&S32614[96], 5}, {NULL}};
const Segment SGI35896[] = {{&S32615[44], 14}, {NULL}};
const Segment SGI35897[] = {{&S32615[44], 35}, {NULL}};
const Segment SGI35898[] = {{&S32615[30], 6}, {NULL}};
const Segment SGI35899[] = {{&S32615[93], 6}, {NULL}};
const Segment SGI35900[] = {{&S32627[44], 10}, {NULL}};
const Segment SGI35901[] = {{&S32627[44], 27}, {NULL}};
const Segment SGI35902[] = {{&S32627[30], 6}, {NULL}};
const Segment SGI35903[] = {{&S32627[119], 3}, {NULL}};
const Segment SGI35904[] = {{&S32627[85], 5}, {NULL}};
const Segment SGI35905[] = {{&S32628[44], 13}, {NULL}};
const Segment SGI35906[] = {{&S32628[44], 30}, {NULL}};
const Segment SGI35907[] = {{&S32628[30], 6}, {NULL}};
const Segment SGI35908[] = {{&S32628[122], 3}, {NULL}};
const Segment SGI35909[] = {{&S32628[88], 5}, {NULL}};
const Segment SGI35910[] = {{&S32630[38], 10}, {NULL}};
const Segment SGI35911[] = {{&S32630[31], 6}, {NULL}};
const Segment SGI35912[] = {{&S32632[38], 13}, {NULL}};
const Segment SGI35913[] = {{&S32632[31], 6}, {NULL}};
const Segment SGI35914[] = {{&S32635[44], 13}, {NULL}};
const Segment SGI35915[] = {{&S32635[44], 30}, {NULL}};
const Segment SGI35916[] = {{&S32635[30], 6}, {NULL}};
const Segment SGI35917[] = {{&S32635[124], 3}, {NULL}};
const Segment SGI35918[] = {{&S32635[88], 7}, {NULL}};
const Segment SGI35919[] = {{&S32635[144], 8}, {NULL}};
const Segment SGI35920[] = {{&S32673[44], 18}, {NULL}};
const Segment SGI35921[] = {{&S32673[44], 35}, {NULL}};
const Segment SGI35922[] = {{&S32673[30], 6}, {NULL}};
const Segment SGI35923[] = {{&S32673[127], 3}, {NULL}};
const Segment SGI35924[] = {{&S32673[93], 5}, {NULL}};
const Segment SGI35925[] = {{&S32679[38], 18}, {NULL}};
const Segment SGI35926[] = {{&S32679[31], 6}, {NULL}};
const Segment SGI35927[] = {{&S32683[44], 10}, {NULL}};
const Segment SGI35928[] = {{&S32683[44], 24}, {NULL}};
const Segment SGI35929[] = {{&S32683[30], 6}, {NULL}};
const Segment SGI35930[] = {{&S32683[116], 3}, {NULL}};
const Segment SGI35931[] = {{&S32683[82], 5}, {NULL}};
const Segment SGI35932[] = {{&S34651[44], 11}, {NULL}};
const Segment SGI35933[] = {{&S34651[44], 22}, {NULL}};
const Segment SGI35934[] = {{&S34651[30], 6}, {NULL}};
const Segment SGI35935[] = {{&S34651[114], 3}, {NULL}};
const Segment SGI35936[] = {{&S34651[80], 5}, {NULL}};
const Segment SGI35937[] = {{&S34665[44], 23}, {NULL}};
const Segment SGI35938[] = {{&S34665[44], 34}, {NULL}};
const Segment SGI35939[] = {{&S34665[30], 6}, {NULL}};
const Segment SGI35940[] = {{&S34665[126], 3}, {NULL}};
const Segment SGI35941[] = {{&S34665[92], 5}, {NULL}};
const Segment SGI35942[] = {{&S34666[44], 22}, {NULL}};
const Segment SGI35943[] = {{&S34666[44], 33}, {NULL}};
const Segment SGI35944[] = {{&S34666[30], 6}, {NULL}};
const Segment SGI35945[] = {{&S34666[125], 3}, {NULL}};
const Segment SGI35946[] = {{&S34666[91], 5}, {NULL}};
const Segment SGI35947[] = {{&S34667[44], 23}, {NULL}};
const Segment SGI35948[] = {{&S34667[44], 34}, {NULL}};
const Segment SGI35949[] = {{&S34667[30], 6}, {NULL}};
const Segment SGI35950[] = {{&S34667[126], 3}, {NULL}};
const Segment SGI35951[] = {{&S34667[92], 5}, {NULL}};
const Segment SGI35952[] = {{&S34668[44], 21}, {NULL}};
const Segment SGI35953[] = {{&S34668[44], 32}, {NULL}};
const Segment SGI35954[] = {{&S34668[30], 6}, {NULL}};
const Segment SGI35955[] = {{&S34668[124], 3}, {NULL}};
const Segment SGI35956[] = {{&S34668[90], 5}, {NULL}};
const Segment SGI35957[] = {{&S34669[44], 11}, {NULL}};
const Segment SGI35958[] = {{&S34669[44], 22}, {NULL}};
const Segment SGI35959[] = {{&S34669[30], 6}, {NULL}};
const Segment SGI35960[] = {{&S34669[114], 3}, {NULL}};
const Segment SGI35961[] = {{&S34669[80], 5}, {NULL}};
const Segment SGI35962[] = {{&S34672[38], 22}, {NULL}};
const Segment SGI35963[] = {{&S34672[31], 6}, {NULL}};
const Segment SGI35964[] = {{&S34674[38], 11}, {NULL}};
const Segment SGI35965[] = {{&S34674[31], 6}, {NULL}};
const Segment SGI35966[] = {{&S34677[44], 22}, {NULL}};
const Segment SGI35967[] = {{&S34677[30], 6}, {NULL}};
const Segment SGI35968[] = {{&S34677[116], 3}, {NULL}};
const Segment SGI35969[] = {{&S34677[82], 5}, {NULL}};
const Segment SGI35970[] = {{&S34678[44], 10}, {NULL}};
const Segment SGI35971[] = {{&S34678[30], 6}, {NULL}};
const Segment SGI35972[] = {{&S34678[104], 3}, {NULL}};
const Segment SGI35973[] = {{&S34678[70], 5}, {NULL}};
const Segment SGI35974[] = {{&S34680[44], 11}, {NULL}};
const Segment SGI35975[] = {{&S34680[44], 22}, {NULL}};
const Segment SGI35976[] = {{&S34680[30], 6}, {NULL}};
const Segment SGI35977[] = {{&S34680[83], 4}, {NULL}};
const Segment SGI35978[] = {{&S34680[83], 3}, {NULL}};
const Segment SGI35979[] = {{&S34684[44], 11}, {NULL}};
const Segment SGI35980[] = {{&S34684[44], 22}, {NULL}};
const Segment SGI35981[] = {{&S34684[30], 6}, {NULL}};
const Segment SGI35982[] = {{&S34684[83], 6}, {NULL}};
const Segment SGI35983[] = {{&S40202[44], 10}, {NULL}};
const Segment SGI35984[] = {{&S40202[44], 21}, {NULL}};
const Segment SGI35985[] = {{&S40202[30], 6}, {NULL}};
const Segment SGI35986[] = {{&S40202[113], 3}, {NULL}};
const Segment SGI35987[] = {{&S40202[79], 5}, {NULL}};
const Segment SGI35988[] = {{&S40203[38], 10}, {NULL}};
const Segment SGI35989[] = {{&S40203[31], 6}, {NULL}};
const Segment SGI35990[] = {{&S40204[42], 10}, {NULL}};
const Segment SGI35991[] = {{&S40204[30], 6}, {NULL}};
const Segment SGI35992[] = {{&S40204[111], 3}, {NULL}};
const Segment SGI35993[] = {{&S40204[77], 5}, {NULL}};
const Segment SGI35994[] = {{&S34780[44], 11}, {NULL}};
const Segment SGI35995[] = {{&S34780[44], 28}, {NULL}};
const Segment SGI35996[] = {{&S34780[30], 6}, {NULL}};
const Segment SGI35997[] = {{&S34780[120], 3}, {NULL}};
const Segment SGI35998[] = {{&S34780[86], 5}, {NULL}};
const Segment SGI35999[] = {{&S34791[38], 11}, {NULL}};
const Segment SGI36000[] = {{&S34791[31], 6}, {NULL}};
const Segment SGI36001[] = {{&S44079[44], 7}, {NULL}};
const Segment SGI36002[] = {{&S44079[44], 32}, {NULL}};
const Segment SGI36003[] = {{&S44079[30], 6}, {NULL}};
const Segment SGI36004[] = {{&S44079[124], 3}, {NULL}};
const Segment SGI36005[] = {{&S44079[90], 5}, {NULL}};
const Segment SGI36006[] = {{&S44084[44], 7}, {NULL}};
const Segment SGI36007[] = {{&S44084[44], 62}, {NULL}};
const Segment SGI36008[] = {{&S44084[30], 6}, {NULL}};
const Segment SGI36009[] = {{&S44084[154], 3}, {NULL}};
const Segment SGI36010[] = {{&S44084[120], 5}, {NULL}};
const Segment SGI36011[] = {{&S44085[44], 7}, {NULL}};
const Segment SGI36012[] = {{&S44085[44], 58}, {NULL}};
const Segment SGI36013[] = {{&S44085[30], 6}, {NULL}};
const Segment SGI36014[] = {{&S44085[150], 3}, {NULL}};
const Segment SGI36015[] = {{&S44085[116], 5}, {NULL}};
const Segment SGI36016[] = {{&S44087[38], 7}, {NULL}};
const Segment SGI36017[] = {{&S44087[31], 6}, {NULL}};
const Segment SGI36018[] = {{&S38935[44], 12}, {NULL}};
const Segment SGI36019[] = {{&S38935[44], 23}, {NULL}};
const Segment SGI36020[] = {{&S38935[30], 6}, {NULL}};
const Segment SGI36021[] = {{&S38935[115], 3}, {NULL}};
const Segment SGI36022[] = {{&S38935[81], 5}, {NULL}};
const Segment SGI36023[] = {{&S34944[44], 9}, {NULL}};
const Segment SGI36024[] = {{&S34944[44], 20}, {NULL}};
const Segment SGI36025[] = {{&S34944[30], 6}, {NULL}};
const Segment SGI36026[] = {{&S34944[112], 3}, {NULL}};
const Segment SGI36027[] = {{&S34944[78], 5}, {NULL}};
const Segment SGI36028[] = {{&S34947[44], 9}, {NULL}};
const Segment SGI36029[] = {{&S34947[44], 20}, {NULL}};
const Segment SGI36030[] = {{&S34947[30], 6}, {NULL}};
const Segment SGI36031[] = {{&S34947[112], 3}, {NULL}};
const Segment SGI36032[] = {{&S34947[78], 5}, {NULL}};
const Segment SGI36033[] = {{&S36669[44], 19}, {NULL}};
const Segment SGI36034[] = {{&S36669[44], 36}, {NULL}};
const Segment SGI36035[] = {{&S36669[30], 6}, {NULL}};
const Segment SGI36036[] = {{&S36669[128], 3}, {NULL}};
const Segment SGI36037[] = {{&S36669[94], 5}, {NULL}};
const Segment SGI36038[] = {{&S34954[38], 9}, {NULL}};
const Segment SGI36039[] = {{&S34954[31], 6}, {NULL}};
const Segment SGI36040[] = {{&S36671[44], 19}, {NULL}};
const Segment SGI36041[] = {{&S36671[44], 36}, {NULL}};
const Segment SGI36042[] = {{&S36671[30], 6}, {NULL}};
const Segment SGI36043[] = {{&S36671[128], 3}, {NULL}};
const Segment SGI36044[] = {{&S36671[94], 5}, {NULL}};
const Segment SGI36045[] = {{&S36672[44], 19}, {NULL}};
const Segment SGI36046[] = {{&S36672[44], 36}, {NULL}};
const Segment SGI36047[] = {{&S36672[30], 6}, {NULL}};
const Segment SGI36048[] = {{&S36672[128], 3}, {NULL}};
const Segment SGI36049[] = {{&S36672[94], 5}, {NULL}};
const Segment SGI36050[] = {{&S36691[44], 7}, {NULL}};
const Segment SGI36051[] = {{&S36691[44], 24}, {NULL}};
const Segment SGI36052[] = {{&S36691[30], 6}, {NULL}};
const Segment SGI36053[] = {{&S36691[116], 3}, {NULL}};
const Segment SGI36054[] = {{&S36691[82], 5}, {NULL}};
const Segment SGI36055[] = {{&S33106[44], 7}, {NULL}};
const Segment SGI36056[] = {{&S33106[44], 33}, {NULL}};
const Segment SGI36057[] = {{&S33106[30], 6}, {NULL}};
const Segment SGI36058[] = {{&S33106[125], 3}, {NULL}};
const Segment SGI36059[] = {{&S36693[44], 7}, {NULL}};
const Segment SGI36060[] = {{&S33106[91], 5}, {NULL}};
const Segment SGI36061[] = {{&S36693[44], 24}, {NULL}};
const Segment SGI36062[] = {{&S36693[30], 6}, {NULL}};
const Segment SGI36063[] = {{&S36693[116], 3}, {NULL}};
const Segment SGI36064[] = {{&S36693[82], 5}, {NULL}};
const Segment SGI36065[] = {{&S33107[44], 8}, {NULL}};
const Segment SGI36066[] = {{&S33107[44], 29}, {NULL}};
const Segment SGI36067[] = {{&S33107[30], 6}, {NULL}};
const Segment SGI36068[] = {{&S33107[121], 3}, {NULL}};
const Segment SGI36069[] = {{&S33107[87], 5}, {NULL}};
const Segment SGI36070[] = {{&S33108[44], 8}, {NULL}};
const Segment SGI36071[] = {{&S33108[44], 34}, {NULL}};
const Segment SGI36072[] = {{&S33108[30], 6}, {NULL}};
const Segment SGI36073[] = {{&S33108[126], 3}, {NULL}};
const Segment SGI36074[] = {{&S33108[92], 5}, {NULL}};
const Segment SGI36075[] = {{&S33109[44], 8}, {NULL}};
const Segment SGI36076[] = {{&S33109[44], 29}, {NULL}};
const Segment SGI36077[] = {{&S33109[30], 6}, {NULL}};
const Segment SGI36078[] = {{&S33109[121], 3}, {NULL}};
const Segment SGI36079[] = {{&S33109[87], 5}, {NULL}};
const Segment SGI36080[] = {{&S33110[44], 8}, {NULL}};
const Segment SGI36081[] = {{&S33110[44], 34}, {NULL}};
const Segment SGI36082[] = {{&S33110[30], 6}, {NULL}};
const Segment SGI36083[] = {{&S33110[126], 3}, {NULL}};
const Segment SGI36084[] = {{&S33110[92], 5}, {NULL}};
const Segment SGI36085[] = {{&S33111[44], 8}, {NULL}};
const Segment SGI36086[] = {{&S33111[44], 29}, {NULL}};
const Segment SGI36087[] = {{&S33111[30], 6}, {NULL}};
const Segment SGI36088[] = {{&S33111[121], 3}, {NULL}};
const Segment SGI36089[] = {{&S33111[87], 5}, {NULL}};
const Segment SGI36090[] = {{&S32284[44], 10}, {NULL}};
const Segment SGI36091[] = {{&S32284[44], 27}, {NULL}};
const Segment SGI36092[] = {{&S32284[30], 6}, {NULL}};
const Segment SGI36093[] = {{&S32284[119], 3}, {NULL}};
const Segment SGI36094[] = {{&S33112[43], 8}, {NULL}};
const Segment SGI36095[] = {{&S32284[85], 5}, {NULL}};
const Segment SGI36096[] = {{&S33112[43], 34}, {NULL}};
const Segment SGI36097[] = {{&S33112[125], 3}, {NULL}};
const Segment SGI36098[] = {{&S33112[91], 5}, {NULL}};
const Segment SGI36099[] = {{&S33113[44], 8}, {NULL}};
const Segment SGI36100[] = {{&S33113[44], 29}, {NULL}};
const Segment SGI36101[] = {{&S33113[30], 6}, {NULL}};
const Segment SGI36102[] = {{&S33113[121], 3}, {NULL}};
const Segment SGI36103[] = {{&S33113[87], 5}, {NULL}};
const Segment SGI36104[] = {{&S32287[44], 11}, {NULL}};
const Segment SGI36105[] = {{&S32287[44], 28}, {NULL}};
const Segment SGI36106[] = {{&S32287[30], 6}, {NULL}};
const Segment SGI36107[] = {{&S32287[120], 3}, {NULL}};
const Segment SGI36108[] = {{&S32287[86], 5}, {NULL}};
const Segment SGI36109[] = {{&S33115[44], 8}, {NULL}};
const Segment SGI36110[] = {{&S33115[44], 22}, {NULL}};
const Segment SGI36111[] = {{&S33115[30], 6}, {NULL}};
const Segment SGI36112[] = {{&S33115[114], 3}, {NULL}};
const Segment SGI36113[] = {{&S33115[80], 5}, {NULL}};
const Segment SGI36114[] = {{&S32290[44], 11}, {NULL}};
const Segment SGI36115[] = {{&S32290[44], 28}, {NULL}};
const Segment SGI36116[] = {{&S32290[30], 6}, {NULL}};
const Segment SGI36117[] = {{&S32290[120], 3}, {NULL}};
const Segment SGI36118[] = {{&S32290[86], 5}, {NULL}};
const Segment SGI36119[] = {{&S33116[44], 8}, {NULL}};
const Segment SGI36120[] = {{&S33116[44], 35}, {NULL}};
const Segment SGI36121[] = {{&S33116[30], 6}, {NULL}};
const Segment SGI36122[] = {{&S33116[127], 3}, {NULL}};
const Segment SGI36123[] = {{&S33116[93], 5}, {NULL}};
const Segment SGI36124[] = {{&S33117[44], 8}, {NULL}};
const Segment SGI36125[] = {{&S33117[44], 34}, {NULL}};
const Segment SGI36126[] = {{&S33117[30], 6}, {NULL}};
const Segment SGI36127[] = {{&S33117[126], 3}, {NULL}};
const Segment SGI36128[] = {{&S33117[92], 5}, {NULL}};
const Segment SGI36129[] = {{&S33118[43], 8}, {NULL}};
const Segment SGI36130[] = {{&S33118[43], 29}, {NULL}};
const Segment SGI36131[] = {{&S33118[30], 5}, {NULL}};
const Segment SGI36132[] = {{&S33118[120], 3}, {NULL}};
const Segment SGI36133[] = {{&S33118[86], 5}, {NULL}};
const Segment SGI36134[] = {{&S33119[44], 8}, {NULL}};
const Segment SGI36135[] = {{&S33119[44], 29}, {NULL}};
const Segment SGI36136[] = {{&S33119[30], 6}, {NULL}};
const Segment SGI36137[] = {{&S32297[38], 10}, {NULL}};
const Segment SGI36138[] = {{&S32297[31], 6}, {NULL}};
const Segment SGI36139[] = {{&S33119[121], 3}, {NULL}};
const Segment SGI36140[] = {{&S33119[87], 5}, {NULL}};
const Segment SGI36141[] = {{&S32298[38], 11}, {NULL}};
const Segment SGI36142[] = {{&S32298[31], 6}, {NULL}};
const Segment SGI36143[] = {{&S33120[44], 7}, {NULL}};
const Segment SGI36144[] = {{&S33120[44], 28}, {NULL}};
const Segment SGI36145[] = {{&S33120[30], 6}, {NULL}};
const Segment SGI36146[] = {{&S33120[120], 3}, {NULL}};
const Segment SGI36147[] = {{&S33120[86], 5}, {NULL}};
const Segment SGI36148[] = {{&S33121[44], 7}, {NULL}};
const Segment SGI36149[] = {{&S33121[44], 28}, {NULL}};
const Segment SGI36150[] = {{&S33121[30], 6}, {NULL}};
const Segment SGI36151[] = {{&S33121[120], 3}, {NULL}};
const Segment SGI36152[] = {{&S33121[86], 5}, {NULL}};
const Segment SGI36153[] = {{&S33122[44], 8}, {NULL}};
const Segment SGI36154[] = {{&S33122[44], 34}, {NULL}};
const Segment SGI36155[] = {{&S33122[30], 6}, {NULL}};
const Segment SGI36156[] = {{&S33122[126], 3}, {NULL}};
const Segment SGI36157[] = {{&S33122[92], 5}, {NULL}};
const Segment SGI36158[] = {{&S33123[43], 8}, {NULL}};
const Segment SGI36159[] = {{&S33123[43], 29}, {NULL}};
const Segment SGI36160[] = {{&S33123[120], 3}, {NULL}};
const Segment SGI36161[] = {{&S33123[86], 5}, {NULL}};
const Segment SGI36162[] = {{&S33126[44], 11}, {NULL}};
const Segment SGI36163[] = {{&S33126[44], 32}, {NULL}};
const Segment SGI36164[] = {{&S33126[30], 6}, {NULL}};
const Segment SGI36165[] = {{&S33126[124], 3}, {NULL}};
const Segment SGI36166[] = {{&S33126[90], 5}, {NULL}};
const Segment SGI36167[] = {{&S33128[44], 8}, {NULL}};
const Segment SGI36168[] = {{&S33128[44], 34}, {NULL}};
const Segment SGI36169[] = {{&S33128[30], 6}, {NULL}};
const Segment SGI36170[] = {{&S33128[126], 3}, {NULL}};
const Segment SGI36171[] = {{&S33128[92], 5}, {NULL}};
const Segment SGI36172[] = {{&S33130[44], 8}, {NULL}};
const Segment SGI36173[] = {{&S33130[44], 35}, {NULL}};
const Segment SGI36174[] = {{&S33130[30], 6}, {NULL}};
const Segment SGI36175[] = {{&S33130[127], 3}, {NULL}};
const Segment SGI36176[] = {{&S33130[93], 5}, {NULL}};
const Segment SGI36177[] = {{&S33131[44], 8}, {NULL}};
const Segment SGI36178[] = {{&S33131[44], 29}, {NULL}};
const Segment SGI36179[] = {{&S33131[30], 6}, {NULL}};
const Segment SGI36180[] = {{&S33131[121], 3}, {NULL}};
const Segment SGI36181[] = {{&S33131[87], 5}, {NULL}};
const Segment SGI36182[] = {{&S33132[38], 8}, {NULL}};
const Segment SGI36183[] = {{&S33132[31], 6}, {NULL}};
const Segment SGI36184[] = {{&S33133[44], 8}, {NULL}};
const Segment SGI36185[] = {{&S33133[44], 34}, {NULL}};
const Segment SGI36186[] = {{&S33133[30], 6}, {NULL}};
const Segment SGI36187[] = {{&S33133[126], 3}, {NULL}};
const Segment SGI36188[] = {{&S33133[92], 5}, {NULL}};
const Segment SGI36189[] = {{&S33134[44], 8}, {NULL}};
const Segment SGI36190[] = {{&S33134[44], 35}, {NULL}};
const Segment SGI36191[] = {{&S33134[30], 6}, {NULL}};
const Segment SGI36192[] = {{&S33134[127], 3}, {NULL}};
const Segment SGI36193[] = {{&S33134[93], 5}, {NULL}};
const Segment SGI36194[] = {{&S33135[43], 8}, {NULL}};
const Segment SGI36195[] = {{&S33135[43], 35}, {NULL}};
const Segment SGI36196[] = {{&S33135[126], 3}, {NULL}};
const Segment SGI36197[] = {{&S33135[92], 5}, {NULL}};
const Segment SGI36198[] = {{&S33136[44], 7}, {NULL}};
const Segment SGI36199[] = {{&S33136[44], 34}, {NULL}};
const Segment SGI36200[] = {{&S33136[30], 6}, {NULL}};
const Segment SGI36201[] = {{&S33136[126], 3}, {NULL}};
const Segment SGI36202[] = {{&S33136[92], 5}, {NULL}};
const Segment SGI36203[] = {{&S33137[43], 8}, {NULL}};
const Segment SGI36204[] = {{&S33137[43], 35}, {NULL}};
const Segment SGI36205[] = {{&S33137[30], 5}, {NULL}};
const Segment SGI36206[] = {{&S33137[126], 3}, {NULL}};
const Segment SGI36207[] = {{&S33137[92], 5}, {NULL}};
const Segment SGI36208[] = {{&S33138[44], 11}, {NULL}};
const Segment SGI36209[] = {{&S33138[44], 38}, {NULL}};
const Segment SGI36210[] = {{&S33138[30], 6}, {NULL}};
const Segment SGI36211[] = {{&S33138[130], 3}, {NULL}};
const Segment SGI36212[] = {{&S33138[96], 5}, {NULL}};
const Segment SGI36213[] = {{&S33139[44], 7}, {NULL}};
const Segment SGI36214[] = {{&S33139[44], 34}, {NULL}};
const Segment SGI36215[] = {{&S33139[30], 6}, {NULL}};
const Segment SGI36216[] = {{&S33139[126], 3}, {NULL}};
const Segment SGI36217[] = {{&S33139[92], 5}, {NULL}};
const Segment SGI36218[] = {{&S33140[44], 8}, {NULL}};
const Segment SGI36219[] = {{&S33140[44], 25}, {NULL}};
const Segment SGI36220[] = {{&S33140[30], 6}, {NULL}};
const Segment SGI36221[] = {{&S33140[117], 3}, {NULL}};
const Segment SGI36222[] = {{&S33140[83], 5}, {NULL}};
const Segment SGI36223[] = {{&S35051[44], 9}, {NULL}};
const Segment SGI36224[] = {{&S35051[44], 26}, {NULL}};
const Segment SGI36225[] = {{&S35051[30], 6}, {NULL}};
const Segment SGI36226[] = {{&S35051[118], 3}, {NULL}};
const Segment SGI36227[] = {{&S35051[84], 5}, {NULL}};
const Segment SGI36228[] = {{&S33141[44], 13}, {NULL}};
const Segment SGI36229[] = {{&S33141[44], 30}, {NULL}};
const Segment SGI36230[] = {{&S33141[30], 6}, {NULL}};
const Segment SGI36231[] = {{&S33141[122], 3}, {NULL}};
const Segment SGI36232[] = {{&S33141[88], 5}, {NULL}};
const Segment SGI36233[] = {{&S33142[44], 7}, {NULL}};
const Segment SGI36234[] = {{&S33142[44], 26}, {NULL}};
const Segment SGI36235[] = {{&S33142[30], 6}, {NULL}};
const Segment SGI36236[] = {{&S33142[118], 3}, {NULL}};
const Segment SGI36237[] = {{&S33142[84], 5}, {NULL}};
const Segment SGI36238[] = {{&S35056[44], 9}, {NULL}};
const Segment SGI36239[] = {{&S35056[44], 20}, {NULL}};
const Segment SGI36240[] = {{&S35056[30], 6}, {NULL}};
const Segment SGI36241[] = {{&S35056[112], 3}, {NULL}};
const Segment SGI36242[] = {{&S35056[78], 5}, {NULL}};
const Segment SGI36243[] = {{&S35057[44], 9}, {NULL}};
const Segment SGI36244[] = {{&S35057[44], 20}, {NULL}};
const Segment SGI36245[] = {{&S35057[30], 6}, {NULL}};
const Segment SGI36246[] = {{&S35057[112], 3}, {NULL}};
const Segment SGI36247[] = {{&S35057[78], 5}, {NULL}};
const Segment SGI36248[] = {{&S35069[38], 9}, {NULL}};
const Segment SGI36249[] = {{&S35069[31], 6}, {NULL}};
const Segment SGI36250[] = {{&S35075[44], 9}, {NULL}};
const Segment SGI36251[] = {{&S35075[44], 20}, {NULL}};
const Segment SGI36252[] = {{&S35075[30], 6}, {NULL}};
const Segment SGI36253[] = {{&S35075[112], 3}, {NULL}};
const Segment SGI36254[] = {{&S35075[78], 5}, {NULL}};
const Segment SGI36255[] = {{&S35091[44], 9}, {NULL}};
const Segment SGI36256[] = {{&S35091[44], 20}, {NULL}};
const Segment SGI36257[] = {{&S35091[30], 6}, {NULL}};
const Segment SGI36258[] = {{&S35091[112], 3}, {NULL}};
const Segment SGI36259[] = {{&S35091[78], 5}, {NULL}};
const Segment SGI36260[] = {{&S35095[44], 13}, {NULL}};
const Segment SGI36261[] = {{&S35095[44], 24}, {NULL}};
const Segment SGI36262[] = {{&S35095[30], 6}, {NULL}};
const Segment SGI36263[] = {{&S35095[116], 3}, {NULL}};
const Segment SGI36264[] = {{&S35095[82], 5}, {NULL}};
const Segment SGI36265[] = {{&S39194[44], 13}, {NULL}};
const Segment SGI36266[] = {{&S39194[44], 24}, {NULL}};
const Segment SGI36267[] = {{&S39194[30], 6}, {NULL}};
const Segment SGI36268[] = {{&S39194[116], 3}, {NULL}};
const Segment SGI36269[] = {{&S39194[82], 5}, {NULL}};
const Segment SGI36270[] = {{&S33233[44], 9}, {NULL}};
const Segment SGI36271[] = {{&S33233[44], 20}, {NULL}};
const Segment SGI36272[] = {{&S33233[30], 6}, {NULL}};
const Segment SGI36273[] = {{&S33233[112], 3}, {NULL}};
const Segment SGI36274[] = {{&S33233[78], 5}, {NULL}};
const Segment SGI36275[] = {{&S33235[41], 9}, {NULL}};
const Segment SGI36276[] = {{&S33235[30], 6}, {NULL}};
const Segment SGI36277[] = {{&S33235[109], 3}, {NULL}};
const Segment SGI36278[] = {{&S33235[75], 5}, {NULL}};
const Segment SGI36279[] = {{&S33236[38], 9}, {NULL}};
const Segment SGI36280[] = {{&S33236[31], 6}, {NULL}};
const Segment SGI36281[] = {{&S33237[44], 11}, {NULL}};
const Segment SGI36282[] = {{&S33237[44], 22}, {NULL}};
const Segment SGI36283[] = {{&S33237[30], 6}, {NULL}};
const Segment SGI36284[] = {{&S33237[114], 3}, {NULL}};
const Segment SGI36285[] = {{&S33237[80], 5}, {NULL}};
const Segment SGI36286[] = {{&S33238[42], 12}, {NULL}};
const Segment SGI36287[] = {{&S33238[30], 6}, {NULL}};
const Segment SGI36288[] = {{&S33238[113], 3}, {NULL}};
const Segment SGI36289[] = {{&S33238[79], 5}, {NULL}};
const Segment SGI36290[] = {{&S33240[44], 14}, {NULL}};
const Segment SGI36291[] = {{&S33240[44], 25}, {NULL}};
const Segment SGI36292[] = {{&S33240[30], 6}, {NULL}};
const Segment SGI36293[] = {{&S33240[117], 3}, {NULL}};
const Segment SGI36294[] = {{&S33240[83], 5}, {NULL}};
const Segment SGI36295[] = {{&S33241[44], 9}, {NULL}};
const Segment SGI36296[] = {{&S33241[44], 20}, {NULL}};
const Segment SGI36297[] = {{&S33241[30], 6}, {NULL}};
const Segment SGI36298[] = {{&S33241[112], 3}, {NULL}};
const Segment SGI36299[] = {{&S33241[78], 5}, {NULL}};
const Segment SGI36300[] = {{&S33243[44], 14}, {NULL}};
const Segment SGI36301[] = {{&S33243[44], 25}, {NULL}};
const Segment SGI36302[] = {{&S33243[30], 6}, {NULL}};
const Segment SGI36303[] = {{&S33243[117], 3}, {NULL}};
const Segment SGI36304[] = {{&S33243[83], 5}, {NULL}};
const Segment SGI36305[] = {{&S33244[44], 12}, {NULL}};
const Segment SGI36306[] = {{&S33244[44], 24}, {NULL}};
const Segment SGI36307[] = {{&S33244[30], 6}, {NULL}};
const Segment SGI36308[] = {{&S33244[116], 3}, {NULL}};
const Segment SGI36309[] = {{&S33244[82], 5}, {NULL}};
const Segment SGI36310[] = {{&S33246[44], 10}, {NULL}};
const Segment SGI36311[] = {{&S33246[44], 21}, {NULL}};
const Segment SGI36312[] = {{&S33246[30], 6}, {NULL}};
const Segment SGI36313[] = {{&S33246[113], 3}, {NULL}};
const Segment SGI36314[] = {{&S33246[79], 5}, {NULL}};
const Segment SGI36315[] = {{&S33248[44], 10}, {NULL}};
const Segment SGI36316[] = {{&S33248[44], 21}, {NULL}};
const Segment SGI36317[] = {{&S33248[30], 6}, {NULL}};
const Segment SGI36318[] = {{&S33248[113], 3}, {NULL}};
const Segment SGI36319[] = {{&S33248[79], 5}, {NULL}};
const Segment SGI36320[] = {{&S33249[44], 14}, {NULL}};
const Segment SGI36321[] = {{&S33249[44], 25}, {NULL}};
const Segment SGI36322[] = {{&S33249[30], 6}, {NULL}};
const Segment SGI36323[] = {{&S33249[117], 3}, {NULL}};
const Segment SGI36324[] = {{&S33249[83], 5}, {NULL}};
const Segment SGI36325[] = {{&S33250[44], 9}, {NULL}};
const Segment SGI36326[] = {{&S33250[44], 20}, {NULL}};
const Segment SGI36327[] = {{&S33250[30], 6}, {NULL}};
const Segment SGI36328[] = {{&S33250[81], 4}, {NULL}};
const Segment SGI36329[] = {{&S33250[81], 3}, {NULL}};
const Segment SGI36330[] = {{&S33252[38], 11}, {NULL}};
const Segment SGI36331[] = {{&S33252[31], 6}, {NULL}};
const Segment SGI36332[] = {{&S33253[44], 9}, {NULL}};
const Segment SGI36333[] = {{&S33253[44], 20}, {NULL}};
const Segment SGI36334[] = {{&S33253[30], 6}, {NULL}};
const Segment SGI36335[] = {{&S33253[81], 6}, {NULL}};
const Segment SGI36336[] = {{&S33255[41], 11}, {NULL}};
const Segment SGI36337[] = {{&S33255[30], 6}, {NULL}};
const Segment SGI36338[] = {{&S33255[111], 3}, {NULL}};
const Segment SGI36339[] = {{&S33255[77], 5}, {NULL}};
const Segment SGI36340[] = {{&S33256[44], 14}, {NULL}};
const Segment SGI36341[] = {{&S33256[44], 25}, {NULL}};
const Segment SGI36342[] = {{&S33256[30], 6}, {NULL}};
const Segment SGI36343[] = {{&S33256[117], 3}, {NULL}};
const Segment SGI36344[] = {{&S33256[83], 5}, {NULL}};
const Segment SGI36345[] = {{&S33257[41], 14}, {NULL}};
const Segment SGI36346[] = {{&S33257[30], 6}, {NULL}};
const Segment SGI36347[] = {{&S33257[114], 3}, {NULL}};
const Segment SGI36348[] = {{&S33257[80], 5}, {NULL}};
const Segment SGI36349[] = {{&S33265[44], 17}, {NULL}};
const Segment SGI36350[] = {{&S33265[44], 28}, {NULL}};
const Segment SGI36351[] = {{&S33265[30], 6}, {NULL}};
const Segment SGI36352[] = {{&S33265[120], 3}, {NULL}};
const Segment SGI36353[] = {{&S33265[86], 5}, {NULL}};
const Segment SGI36354[] = {{&S44537[44], 11}, {NULL}};
const Segment SGI36355[] = {{&S44537[44], 22}, {NULL}};
const Segment SGI36356[] = {{&S44537[30], 6}, {NULL}};
const Segment SGI36357[] = {{&S44537[114], 3}, {NULL}};
const Segment SGI36358[] = {{&S44537[80], 5}, {NULL}};
const Segment SGI36359[] = {{&S44538[44], 11}, {NULL}};
const Segment SGI36360[] = {{&S44538[44], 22}, {NULL}};
const Segment SGI36361[] = {{&S44538[30], 6}, {NULL}};
const Segment SGI36362[] = {{&S44538[83], 4}, {NULL}};
const Segment SGI36363[] = {{&S44538[83], 3}, {NULL}};
const Segment SGI36364[] = {{&S44539[38], 11}, {NULL}};
const Segment SGI36365[] = {{&S44539[31], 6}, {NULL}};
const Segment SGI36366[] = {{&S44562[44], 10}, {NULL}};
const Segment SGI36367[] = {{&S44562[44], 22}, {NULL}};
const Segment SGI36368[] = {{&S44562[30], 6}, {NULL}};
const Segment SGI36369[] = {{&S44562[114], 3}, {NULL}};
const Segment SGI36370[] = {{&S44562[80], 5}, {NULL}};
const Segment SGI36371[] = {{&S44565[38], 10}, {NULL}};
const Segment SGI36372[] = {{&S44565[31], 6}, {NULL}};
const Segment SGI36373[] = {{&S32674[44], 18}, {NULL}};
const Segment SGI36374[] = {{&S32674[44], 35}, {NULL}};
const Segment SGI36375[] = {{&S32674[30], 6}, {NULL}};
const Segment SGI36376[] = {{&S32674[127], 3}, {NULL}};
const Segment SGI36377[] = {{&S32674[93], 5}, {NULL}};
const Segment SGI36378[] = {{&S44685[90], 6}, {NULL}};
const Segment SGI36379[] = {{&S44685[90], 27}, {NULL}};
const Segment SGI36380[] = {{&S44685[76], 6}, {NULL}};
const Segment SGI36381[] = {{&S44685[165], 3}, {NULL}};
const Segment SGI36382[] = {{&S44685[131], 5}, {NULL}};
const Segment SGI36383[] = {{&S44688[90], 6}, {NULL}};
const Segment SGI36384[] = {{&S44688[90], 27}, {NULL}};
const Segment SGI36385[] = {{&S44688[76], 6}, {NULL}};
const Segment SGI36386[] = {{&S44688[165], 3}, {NULL}};
const Segment SGI36387[] = {{&S44688[131], 5}, {NULL}};
const Segment SGI36388[] = {{&S44692[44], 10}, {NULL}};
const Segment SGI36389[] = {{&S44692[44], 31}, {NULL}};
const Segment SGI36390[] = {{&S44692[30], 6}, {NULL}};
const Segment SGI36391[] = {{&S44692[123], 3}, {NULL}};
const Segment SGI36392[] = {{&S44692[89], 5}, {NULL}};
const Segment SGI36393[] = {{&S44694[44], 10}, {NULL}};
const Segment SGI36394[] = {{&S44694[44], 30}, {NULL}};
const Segment SGI36395[] = {{&S44694[30], 6}, {NULL}};
const Segment SGI36396[] = {{&S44694[122], 3}, {NULL}};
const Segment SGI36397[] = {{&S44694[88], 5}, {NULL}};
const Segment SGI36398[] = {{&S44696[44], 10}, {NULL}};
const Segment SGI36399[] = {{&S44696[44], 31}, {NULL}};
const Segment SGI36400[] = {{&S44696[30], 6}, {NULL}};
const Segment SGI36401[] = {{&S44696[123], 3}, {NULL}};
const Segment SGI36402[] = {{&S44696[89], 5}, {NULL}};
const Segment SGI36403[] = {{&S44697[88], 6}, {NULL}};
const Segment SGI36404[] = {{&S44697[88], 27}, {NULL}};
const Segment SGI36405[] = {{&S44697[74], 6}, {NULL}};
const Segment SGI36406[] = {{&S44697[25], 3}, {NULL}};
const Segment SGI36407[] = {{&S44697[129], 5}, {NULL}};
const Segment SGI36408[] = {{&S44701[90], 6}, {NULL}};
const Segment SGI36409[] = {{&S44701[90], 27}, {NULL}};
const Segment SGI36410[] = {{&S44701[76], 6}, {NULL}};
const Segment SGI36411[] = {{&S44701[165], 3}, {NULL}};
const Segment SGI36412[] = {{&S44701[131], 5}, {NULL}};
const Segment SGI36413[] = {{&S44703[38], 10}, {NULL}};
const Segment SGI36414[] = {{&S44703[31], 6}, {NULL}};
const Segment SGI36415[] = {{&S44769[44], 10}, {NULL}};
const Segment SGI36416[] = {{&S44769[44], 22}, {NULL}};
const Segment SGI36417[] = {{&S44769[30], 6}, {NULL}};
const Segment SGI36418[] = {{&S44769[114], 3}, {NULL}};
const Segment SGI36419[] = {{&S44769[80], 5}, {NULL}};
const Segment SGI36420[] = {{&S33492[44], 19}, {NULL}};
const Segment SGI36421[] = {{&S33492[44], 37}, {NULL}};
const Segment SGI36422[] = {{&S33492[30], 6}, {NULL}};
const Segment SGI36423[] = {{&S33492[69], 3}, {NULL}};
const Segment SGI36424[] = {{&S33492[95], 5}, {NULL}};
const Segment SGI36425[] = {{&S44774[44], 18}, {NULL}};
const Segment SGI36426[] = {{&S44774[44], 30}, {NULL}};
const Segment SGI36427[] = {{&S44774[30], 6}, {NULL}};
const Segment SGI36428[] = {{&S44774[122], 3}, {NULL}};
const Segment SGI36429[] = {{&S44774[88], 5}, {NULL}};
const Segment SGI36430[] = {{&S33494[44], 7}, {NULL}};
const Segment SGI36431[] = {{&S33494[44], 25}, {NULL}};
const Segment SGI36432[] = {{&S33494[30], 6}, {NULL}};
const Segment SGI36433[] = {{&S44775[44], 10}, {NULL}};
const Segment SGI36434[] = {{&S44775[44], 22}, {NULL}};
const Segment SGI36435[] = {{&S44775[30], 6}, {NULL}};
const Segment SGI36436[] = {{&S33494[57], 3}, {NULL}};
const Segment SGI36437[] = {{&S33494[83], 5}, {NULL}};
const Segment SGI36438[] = {{&S44775[116], 3}, {NULL}};
const Segment SGI36439[] = {{&S44775[80], 7}, {NULL}};
const Segment SGI36440[] = {{&S44775[136], 8}, {NULL}};
const Segment SGI36441[] = {{&S44776[38], 10}, {NULL}};
const Segment SGI36442[] = {{&S44776[31], 6}, {NULL}};
const Segment SGI36443[] = {{&S33497[38], 7}, {NULL}};
const Segment SGI36444[] = {{&S33497[31], 6}, {NULL}};
const Segment SGI36445[] = {{&S33500[44], 67}, {NULL}};
const Segment SGI36446[] = {{&S33500[44], 85}, {NULL}};
const Segment SGI36447[] = {{&S33500[30], 6}, {NULL}};
const Segment SGI36448[] = {{&S33500[117], 3}, {NULL}};
const Segment SGI36449[] = {{&S33500[143], 5}, {NULL}};
const Segment SGI36450[] = {{&S33501[44], 19}, {NULL}};
const Segment SGI36451[] = {{&S33501[44], 37}, {NULL}};
const Segment SGI36452[] = {{&S33501[30], 6}, {NULL}};
const Segment SGI36453[] = {{&S33501[69], 3}, {NULL}};
const Segment SGI36454[] = {{&S33501[95], 5}, {NULL}};
const Segment SGI36455[] = {{&S33502[44], 18}, {NULL}};
const Segment SGI36456[] = {{&S33502[44], 36}, {NULL}};
const Segment SGI36457[] = {{&S33502[30], 6}, {NULL}};
const Segment SGI36458[] = {{&S33502[68], 3}, {NULL}};
const Segment SGI36459[] = {{&S33502[94], 5}, {NULL}};
const Segment SGI36460[] = {{&S33503[43], 19}, {NULL}};
const Segment SGI36461[] = {{&S33503[43], 37}, {NULL}};
const Segment SGI36462[] = {{&S33503[68], 3}, {NULL}};
const Segment SGI36463[] = {{&S33503[94], 5}, {NULL}};
const Segment SGI36464[] = {{&S33504[44], 18}, {NULL}};
const Segment SGI36465[] = {{&S33504[44], 36}, {NULL}};
const Segment SGI36466[] = {{&S33504[30], 6}, {NULL}};
const Segment SGI36467[] = {{&S33504[68], 3}, {NULL}};
const Segment SGI36468[] = {{&S33504[94], 5}, {NULL}};
const Segment SGI36469[] = {{&S33505[44], 7}, {NULL}};
const Segment SGI36470[] = {{&S33505[44], 25}, {NULL}};
const Segment SGI36471[] = {{&S33505[30], 6}, {NULL}};
const Segment SGI36472[] = {{&S33505[57], 3}, {NULL}};
const Segment SGI36473[] = {{&S33505[83], 5}, {NULL}};
const Segment SGI36474[] = {{&S44932[44], 15}, {NULL}};
const Segment SGI36475[] = {{&S44932[44], 27}, {NULL}};
const Segment SGI36476[] = {{&S44932[30], 6}, {NULL}};
const Segment SGI36477[] = {{&S44932[119], 3}, {NULL}};
const Segment SGI36478[] = {{&S44932[85], 5}, {NULL}};
const Segment SGI36479[] = {{&S44934[38], 15}, {NULL}};
const Segment SGI36480[] = {{&S44934[31], 6}, {NULL}};
const Segment SGI36481[] = {{&S44935[44], 15}, {NULL}};
const Segment SGI36482[] = {{&S44935[44], 27}, {NULL}};
const Segment SGI36483[] = {{&S44935[30], 6}, {NULL}};
const Segment SGI36484[] = {{&S44935[88], 4}, {NULL}};
const Segment SGI36485[] = {{&S44935[88], 3}, {NULL}};
const Segment SGI36486[] = {{&S35571[44], 10}, {NULL}};
const Segment SGI36487[] = {{&S35571[44], 27}, {NULL}};
const Segment SGI36488[] = {{&S35571[30], 6}, {NULL}};
const Segment SGI36489[] = {{&S35571[119], 3}, {NULL}};
const Segment SGI36490[] = {{&S35571[85], 5}, {NULL}};
const Segment SGI36491[] = {{&S35585[38], 10}, {NULL}};
const Segment SGI36492[] = {{&S35585[31], 6}, {NULL}};
const Segment SGI36493[] = {{&S35587[44], 10}, {NULL}};
const Segment SGI36494[] = {{&S35587[44], 27}, {NULL}};
const Segment SGI36495[] = {{&S35587[30], 6}, {NULL}};
const Segment SGI36496[] = {{&S35587[120], 3}, {NULL}};
const Segment SGI36497[] = {{&S35587[85], 6}, {NULL}};
const Segment SGI36498[] = {{&S35711[44], 7}, {NULL}};
const Segment SGI36499[] = {{&S35711[44], 24}, {NULL}};
const Segment SGI36500[] = {{&S35711[30], 6}, {NULL}};
const Segment SGI36501[] = {{&S35711[116], 3}, {NULL}};
const Segment SGI36502[] = {{&S35711[82], 5}, {NULL}};
const Segment SGI36503[] = {{&S35712[44], 20}, {NULL}};
const Segment SGI36504[] = {{&S35712[44], 37}, {NULL}};
const Segment SGI36505[] = {{&S35712[30], 6}, {NULL}};
const Segment SGI36506[] = {{&S35712[129], 3}, {NULL}};
const Segment SGI36507[] = {{&S35712[95], 5}, {NULL}};
const Segment SGI36508[] = {{&S35713[44], 14}, {NULL}};
const Segment SGI36509[] = {{&S35713[44], 31}, {NULL}};
const Segment SGI36510[] = {{&S35713[30], 6}, {NULL}};
const Segment SGI36511[] = {{&S35713[123], 3}, {NULL}};
const Segment SGI36512[] = {{&S35713[89], 5}, {NULL}};
const Segment SGI36513[] = {{&S35720[38], 7}, {NULL}};
const Segment SGI36514[] = {{&S35720[31], 6}, {NULL}};
const Segment SGI36515[] = {{&S35734[44], 9}, {NULL}};
const Segment SGI36516[] = {{&S35734[44], 26}, {NULL}};
const Segment SGI36517[] = {{&S35734[30], 6}, {NULL}};
const Segment SGI36518[] = {{&S35734[118], 3}, {NULL}};
const Segment SGI36519[] = {{&S35734[84], 5}, {NULL}};
const Segment SGI36520[] = {{&S35735[38], 14}, {NULL}};
const Segment SGI36521[] = {{&S35735[31], 6}, {NULL}};
const Segment SGI36522[] = {{&S35742[44], 13}, {NULL}};
const Segment SGI36523[] = {{&S35742[44], 30}, {NULL}};
const Segment SGI36524[] = {{&S35742[30], 6}, {NULL}};
const Segment SGI36525[] = {{&S35742[122], 3}, {NULL}};
const Segment SGI36526[] = {{&S35742[88], 5}, {NULL}};
const Segment SGI36527[] = {{&S33739[44], 18}, {NULL}};
const Segment SGI36528[] = {{&S33739[44], 29}, {NULL}};
const Segment SGI36529[] = {{&S33739[30], 6}, {NULL}};
const Segment SGI36530[] = {{&S33739[121], 3}, {NULL}};
const Segment SGI36531[] = {{&S33739[87], 5}, {NULL}};
const Segment SGI36532[] = {{&S33758[44], 14}, {NULL}};
const Segment SGI36533[] = {{&S33758[44], 25}, {NULL}};
const Segment SGI36534[] = {{&S33758[30], 6}, {NULL}};
const Segment SGI36535[] = {{&S33758[117], 3}, {NULL}};
const Segment SGI36536[] = {{&S33758[83], 5}, {NULL}};
const Segment SGI36537[] = {{&S35804[44], 18}, {NULL}};
const Segment SGI36538[] = {{&S35804[44], 29}, {NULL}};
const Segment SGI36539[] = {{&S35804[30], 6}, {NULL}};
const Segment SGI36540[] = {{&S35804[121], 3}, {NULL}};
const Segment SGI36541[] = {{&S35804[87], 5}, {NULL}};
const Segment SGI36542[] = {{&S35805[44], 9}, {NULL}};
const Segment SGI36543[] = {{&S35805[44], 20}, {NULL}};
const Segment SGI36544[] = {{&S35805[30], 6}, {NULL}};
const Segment SGI36545[] = {{&S35805[112], 3}, {NULL}};
const Segment SGI36546[] = {{&S35805[78], 5}, {NULL}};
const Segment SGI36547[] = {{&S33124[44], 8}, {NULL}};
const Segment SGI36548[] = {{&S33124[44], 29}, {NULL}};
const Segment SGI36549[] = {{&S33124[30], 6}, {NULL}};
const Segment SGI36550[] = {{&S33124[121], 3}, {NULL}};
const Segment SGI36551[] = {{&S33124[87], 5}, {NULL}};
const Segment SGI36552[] = {{&S33125[44], 8}, {NULL}};
const Segment SGI36553[] = {{&S33125[44], 29}, {NULL}};
const Segment SGI36554[] = {{&S33125[30], 6}, {NULL}};
const Segment SGI36555[] = {{&S33125[121], 3}, {NULL}};
const Segment SGI36556[] = {{&S35812[38], 9}, {NULL}};
const Segment SGI36557[] = {{&S35812[31], 6}, {NULL}};
const Segment SGI36558[] = {{&S33125[87], 5}, {NULL}};
const Segment SGI36559[] = {{&S37664[44], 19}, {NULL}};
const Segment SGI36560[] = {{&S37664[44], 36}, {NULL}};
const Segment SGI36561[] = {{&S37664[30], 6}, {NULL}};
const Segment SGI36562[] = {{&S37664[128], 3}, {NULL}};
const Segment SGI36563[] = {{&S37664[94], 5}, {NULL}};
const Segment SGI36564[] = {{&S37669[44], 7}, {NULL}};
const Segment SGI36565[] = {{&S37669[44], 24}, {NULL}};
const Segment SGI36566[] = {{&S37669[30], 6}, {NULL}};
const Segment SGI36567[] = {{&S37669[116], 3}, {NULL}};
const Segment SGI36568[] = {{&S37669[82], 5}, {NULL}};
const Segment SGI36569[] = {{&S37670[44], 19}, {NULL}};
const Segment SGI36570[] = {{&S37670[44], 36}, {NULL}};
const Segment SGI36571[] = {{&S37670[30], 6}, {NULL}};
const Segment SGI36572[] = {{&S37670[128], 3}, {NULL}};
const Segment SGI36573[] = {{&S37670[94], 5}, {NULL}};
const Segment SGI36574[] = {{&S37672[44], 7}, {NULL}};
const Segment SGI36575[] = {{&S37672[44], 24}, {NULL}};
const Segment SGI36576[] = {{&S37672[30], 6}, {NULL}};
const Segment SGI36577[] = {{&S37672[116], 3}, {NULL}};
const Segment SGI36578[] = {{&S37672[82], 5}, {NULL}};
const Segment SGI36579[] = {{&S37679[38], 7}, {NULL}};
const Segment SGI36580[] = {{&S37679[31], 6}, {NULL}};
const Segment SGI36581[] = {{&S33803[44], 8}, {NULL}};
const Segment SGI36582[] = {{&S33803[44], 17}, {NULL}};
const Segment SGI36583[] = {{&S33803[30], 6}, {NULL}};
const Segment SGI36584[] = {{&S33239[44], 12}, {NULL}};
const Segment SGI36585[] = {{&S33239[44], 24}, {NULL}};
const Segment SGI36586[] = {{&S33803[109], 3}, {NULL}};
const Segment SGI36587[] = {{&S33239[30], 6}, {NULL}};
const Segment SGI36588[] = {{&S33803[75], 5}, {NULL}};
const Segment SGI36589[] = {{&S33239[116], 3}, {NULL}};
const Segment SGI36590[] = {{&S33239[82], 5}, {NULL}};
const Segment SGI36591[] = {{&S33806[44], 9}, {NULL}};
const Segment SGI36592[] = {{&S33806[44], 21}, {NULL}};
const Segment SGI36593[] = {{&S33806[30], 6}, {NULL}};
const Segment SGI36594[] = {{&S33806[113], 3}, {NULL}};
const Segment SGI36595[] = {{&S33806[79], 5}, {NULL}};
const Segment SGI36596[] = {{&S33808[38], 8}, {NULL}};
const Segment SGI36597[] = {{&S33808[31], 6}, {NULL}};
const Segment SGI36598[] = {{&S33809[43], 8}, {NULL}};
const Segment SGI36599[] = {{&S33809[43], 29}, {NULL}};
const Segment SGI36600[] = {{&S33809[120], 3}, {NULL}};
const Segment SGI36601[] = {{&S33809[86], 5}, {NULL}};
const Segment SGI36602[] = {{&S33810[44], 7}, {NULL}};
const Segment SGI36603[] = {{&S33810[44], 28}, {NULL}};
const Segment SGI36604[] = {{&S33810[30], 6}, {NULL}};
const Segment SGI36605[] = {{&S33810[120], 3}, {NULL}};
const Segment SGI36606[] = {{&S33810[86], 5}, {NULL}};
const Segment SGI36607[] = {{&S33811[44], 8}, {NULL}};
const Segment SGI36608[] = {{&S33811[44], 29}, {NULL}};
const Segment SGI36609[] = {{&S33811[30], 6}, {NULL}};
const Segment SGI36610[] = {{&S33811[121], 3}, {NULL}};
const Segment SGI36611[] = {{&S33811[87], 5}, {NULL}};
const Segment SGI36612[] = {{&S33812[44], 8}, {NULL}};
const Segment SGI36613[] = {{&S33812[44], 29}, {NULL}};
const Segment SGI36614[] = {{&S33812[30], 6}, {NULL}};
const Segment SGI36615[] = {{&S33812[121], 3}, {NULL}};
const Segment SGI36616[] = {{&S33812[87], 5}, {NULL}};
const Segment SGI36617[] = {{&S33813[44], 7}, {NULL}};
const Segment SGI36618[] = {{&S33813[44], 28}, {NULL}};
const Segment SGI36619[] = {{&S33813[30], 6}, {NULL}};
const Segment SGI36620[] = {{&S33813[120], 3}, {NULL}};
const Segment SGI36621[] = {{&S33813[86], 5}, {NULL}};
const Segment SGI36622[] = {{&S33814[44], 8}, {NULL}};
const Segment SGI36623[] = {{&S33814[44], 29}, {NULL}};
const Segment SGI36624[] = {{&S33814[30], 6}, {NULL}};
const Segment SGI36625[] = {{&S33814[121], 3}, {NULL}};
const Segment SGI36626[] = {{&S33814[87], 5}, {NULL}};
const Segment SGI36627[] = {{&S33815[43], 8}, {NULL}};
const Segment SGI36628[] = {{&S33815[43], 29}, {NULL}};
const Segment SGI36629[] = {{&S33815[30], 5}, {NULL}};
const Segment SGI36630[] = {{&S33815[120], 3}, {NULL}};
const Segment SGI36631[] = {{&S33815[86], 5}, {NULL}};
const Segment SGI36632[] = {{&S33267[44], 17}, {NULL}};
const Segment SGI36633[] = {{&S33267[44], 28}, {NULL}};
const Segment SGI36634[] = {{&S33267[30], 6}, {NULL}};
const Segment SGI36635[] = {{&S33267[120], 3}, {NULL}};
const Segment SGI36636[] = {{&S33267[86], 5}, {NULL}};
const Segment SGI36637[] = {{&S33269[44], 17}, {NULL}};
const Segment SGI36638[] = {{&S33269[44], 28}, {NULL}};
const Segment SGI36639[] = {{&S33269[30], 6}, {NULL}};
const Segment SGI36640[] = {{&S33269[120], 3}, {NULL}};
const Segment SGI36641[] = {{&S33269[86], 5}, {NULL}};
const Segment SGI36642[] = {{&S33270[44], 17}, {NULL}};
const Segment SGI36643[] = {{&S33270[44], 28}, {NULL}};
const Segment SGI36644[] = {{&S33270[30], 6}, {NULL}};
const Segment SGI36645[] = {{&S33270[120], 3}, {NULL}};
const Segment SGI36646[] = {{&S33270[86], 5}, {NULL}};
const Segment SGI36647[] = {{&S33271[44], 23}, {NULL}};
const Segment SGI36648[] = {{&S33271[44], 34}, {NULL}};
const Segment SGI36649[] = {{&S33271[30], 6}, {NULL}};
const Segment SGI36650[] = {{&S33271[126], 3}, {NULL}};
const Segment SGI36651[] = {{&S33271[92], 5}, {NULL}};
const Segment SGI36652[] = {{&S33272[44], 17}, {NULL}};
const Segment SGI36653[] = {{&S33272[44], 28}, {NULL}};
const Segment SGI36654[] = {{&S33272[30], 6}, {NULL}};
const Segment SGI36655[] = {{&S33272[120], 3}, {NULL}};
const Segment SGI36656[] = {{&S33272[86], 5}, {NULL}};
const Segment SGI36657[] = {{&S33273[44], 17}, {NULL}};
const Segment SGI36658[] = {{&S33273[44], 28}, {NULL}};
const Segment SGI36659[] = {{&S33273[30], 6}, {NULL}};
const Segment SGI36660[] = {{&S33273[120], 3}, {NULL}};
const Segment SGI36661[] = {{&S33273[86], 5}, {NULL}};
const Segment SGI36662[] = {{&S33274[38], 17}, {NULL}};
const Segment SGI36663[] = {{&S33274[31], 6}, {NULL}};
const Segment SGI36664[] = {{&S33275[44], 17}, {NULL}};
const Segment SGI36665[] = {{&S33275[44], 28}, {NULL}};
const Segment SGI36666[] = {{&S33275[30], 6}, {NULL}};
const Segment SGI36667[] = {{&S33275[89], 4}, {NULL}};
const Segment SGI36668[] = {{&S33275[89], 3}, {NULL}};
const Segment SGI36669[] = {{&S33276[44], 23}, {NULL}};
const Segment SGI36670[] = {{&S33276[44], 34}, {NULL}};
const Segment SGI36671[] = {{&S33276[30], 6}, {NULL}};
const Segment SGI36672[] = {{&S33276[126], 3}, {NULL}};
const Segment SGI36673[] = {{&S33276[92], 5}, {NULL}};
const Segment SGI36674[] = {{&S33277[44], 11}, {NULL}};
const Segment SGI36675[] = {{&S33277[44], 22}, {NULL}};
const Segment SGI36676[] = {{&S33277[30], 6}, {NULL}};
const Segment SGI36677[] = {{&S33277[114], 3}, {NULL}};
const Segment SGI36678[] = {{&S33277[80], 5}, {NULL}};
const Segment SGI36679[] = {{&S33278[44], 11}, {NULL}};
const Segment SGI36680[] = {{&S33278[44], 22}, {NULL}};
const Segment SGI36681[] = {{&S33278[30], 6}, {NULL}};
const Segment SGI36682[] = {{&S33278[114], 3}, {NULL}};
const Segment SGI36683[] = {{&S33278[80], 5}, {NULL}};
const Segment SGI36684[] = {{&S33279[44], 23}, {NULL}};
const Segment SGI36685[] = {{&S33279[44], 34}, {NULL}};
const Segment SGI36686[] = {{&S33279[30], 6}, {NULL}};
const Segment SGI36687[] = {{&S33279[126], 3}, {NULL}};
const Segment SGI36688[] = {{&S33279[92], 5}, {NULL}};
const Segment SGI36689[] = {{&S33280[44], 11}, {NULL}};
const Segment SGI36690[] = {{&S33280[44], 22}, {NULL}};
const Segment SGI36691[] = {{&S33280[30], 6}, {NULL}};
const Segment SGI36692[] = {{&S33280[83], 4}, {NULL}};
const Segment SGI36693[] = {{&S33280[83], 3}, {NULL}};
const Segment SGI36694[] = {{&S33281[44], 17}, {NULL}};
const Segment SGI36695[] = {{&S33281[44], 28}, {NULL}};
const Segment SGI36696[] = {{&S33281[30], 6}, {NULL}};
const Segment SGI36697[] = {{&S33281[89], 6}, {NULL}};
const Segment SGI36698[] = {{&S33282[44], 11}, {NULL}};
const Segment SGI36699[] = {{&S33282[44], 22}, {NULL}};
const Segment SGI36700[] = {{&S33282[30], 6}, {NULL}};
const Segment SGI36701[] = {{&S33282[83], 6}, {NULL}};
const Segment SGI36702[] = {{&S33283[42], 17}, {NULL}};
const Segment SGI36703[] = {{&S33283[30], 6}, {NULL}};
const Segment SGI36704[] = {{&S33283[118], 3}, {NULL}};
const Segment SGI36705[] = {{&S33283[84], 5}, {NULL}};
const Segment SGI36706[] = {{&S33990[44], 9}, {NULL}};
const Segment SGI36707[] = {{&S33990[44], 20}, {NULL}};
const Segment SGI36708[] = {{&S33990[30], 6}, {NULL}};
const Segment SGI36709[] = {{&S33990[112], 3}, {NULL}};
const Segment SGI36710[] = {{&S33990[78], 5}, {NULL}};
const Segment SGI36711[] = {{&S33991[38], 9}, {NULL}};
const Segment SGI36712[] = {{&S33991[31], 6}, {NULL}};
const Segment SGI36713[] = {{&S33992[44], 8}, {NULL}};
const Segment SGI36714[] = {{&S33992[44], 19}, {NULL}};
const Segment SGI36715[] = {{&S33992[30], 6}, {NULL}};
const Segment SGI36716[] = {{&S33992[111], 3}, {NULL}};
const Segment SGI36717[] = {{&S33992[77], 5}, {NULL}};
const Segment SGI36718[] = {{&S33993[44], 8}, {NULL}};
const Segment SGI36719[] = {{&S33993[44], 19}, {NULL}};
const Segment SGI36720[] = {{&S33993[30], 6}, {NULL}};
const Segment SGI36721[] = {{&S33993[111], 3}, {NULL}};
const Segment SGI36722[] = {{&S33993[77], 5}, {NULL}};
const Segment SGI36723[] = {{&S33995[43], 9}, {NULL}};
const Segment SGI36724[] = {{&S33995[43], 20}, {NULL}};
const Segment SGI36725[] = {{&S33995[111], 3}, {NULL}};
const Segment SGI36726[] = {{&S33995[77], 5}, {NULL}};
const Segment SGI36727[] = {{&S33997[43], 9}, {NULL}};
const Segment SGI36728[] = {{&S33997[43], 20}, {NULL}};
const Segment SGI36729[] = {{&S33997[30], 5}, {NULL}};
const Segment SGI36730[] = {{&S33997[111], 3}, {NULL}};
const Segment SGI36731[] = {{&S33997[77], 5}, {NULL}};
const Segment SGI36732[] = {{&S33998[44], 9}, {NULL}};
const Segment SGI36733[] = {{&S33998[44], 20}, {NULL}};
const Segment SGI36734[] = {{&S33998[30], 6}, {NULL}};
const Segment SGI36735[] = {{&S33998[112], 3}, {NULL}};
const Segment SGI36736[] = {{&S33998[78], 5}, {NULL}};
const Segment SGI36737[] = {{&S34002[44], 9}, {NULL}};
const Segment SGI36738[] = {{&S34002[44], 20}, {NULL}};
const Segment SGI36739[] = {{&S34002[30], 6}, {NULL}};
const Segment SGI36740[] = {{&S34002[78], 5}, {NULL}};
const Segment SGI36741[] = {{&S45501[44], 10}, {NULL}};
const Segment SGI36742[] = {{&S45501[44], 21}, {NULL}};
const Segment SGI36743[] = {{&S45501[30], 6}, {NULL}};
const Segment SGI36744[] = {{&S45501[113], 3}, {NULL}};
const Segment SGI36745[] = {{&S45501[79], 5}, {NULL}};
const Segment SGI36746[] = {{&S45502[42], 10}, {NULL}};
const Segment SGI36747[] = {{&S45502[30], 6}, {NULL}};
const Segment SGI36748[] = {{&S45502[111], 3}, {NULL}};
const Segment SGI36749[] = {{&S45502[77], 5}, {NULL}};
const Segment SGI36750[] = {{&S45504[38], 10}, {NULL}};
const Segment SGI36751[] = {{&S45504[31], 6}, {NULL}};
const Segment SGI36752[] = {{&S45505[44], 10}, {NULL}};
const Segment SGI36753[] = {{&S45505[44], 21}, {NULL}};
const Segment SGI36754[] = {{&S45505[30], 6}, {NULL}};
const Segment SGI36755[] = {{&S45505[115], 3}, {NULL}};
const Segment SGI36756[] = {{&S45505[79], 7}, {NULL}};
const Segment SGI36757[] = {{&S45505[135], 8}, {NULL}};
const Segment SGI36758[] = {{&S36210[44], 11}, {NULL}};
const Segment SGI36759[] = {{&S36210[44], 22}, {NULL}};
const Segment SGI36760[] = {{&S36210[30], 6}, {NULL}};
const Segment SGI36761[] = {{&S36210[114], 3}, {NULL}};
const Segment SGI36762[] = {{&S36210[80], 5}, {NULL}};
const Segment SGI36763[] = {{&S36211[44], 15}, {NULL}};
const Segment SGI36764[] = {{&S45573[44], 10}, {NULL}};
const Segment SGI36765[] = {{&S36211[44], 26}, {NULL}};
const Segment SGI36766[] = {{&S45573[44], 22}, {NULL}};
const Segment SGI36767[] = {{&S36211[30], 6}, {NULL}};
const Segment SGI36768[] = {{&S45573[30], 6}, {NULL}};
const Segment SGI36769[] = {{&S36211[118], 3}, {NULL}};
const Segment SGI36770[] = {{&S45573[114], 3}, {NULL}};
const Segment SGI36771[] = {{&S36211[84], 5}, {NULL}};
const Segment SGI36772[] = {{&S45573[80], 5}, {NULL}};
const Segment SGI36773[] = {{&S36212[44], 15}, {NULL}};
const Segment SGI36774[] = {{&S36212[44], 26}, {NULL}};
const Segment SGI36775[] = {{&S36212[30], 6}, {NULL}};
const Segment SGI36776[] = {{&S36212[118], 3}, {NULL}};
const Segment SGI36777[] = {{&S36212[84], 5}, {NULL}};
const Segment SGI36778[] = {{&S36213[44], 26}, {NULL}};
const Segment SGI36779[] = {{&S36213[44], 37}, {NULL}};
const Segment SGI36780[] = {{&S36213[30], 6}, {NULL}};
const Segment SGI36781[] = {{&S36213[129], 3}, {NULL}};
const Segment SGI36782[] = {{&S45578[38], 10}, {NULL}};
const Segment SGI36783[] = {{&S45578[31], 6}, {NULL}};
const Segment SGI36784[] = {{&S36213[95], 5}, {NULL}};
const Segment SGI36785[] = {{&S36221[44], 26}, {NULL}};
const Segment SGI36786[] = {{&S36221[44], 37}, {NULL}};
const Segment SGI36787[] = {{&S36221[30], 6}, {NULL}};
const Segment SGI36788[] = {{&S36221[129], 3}, {NULL}};
const Segment SGI36789[] = {{&S36221[95], 5}, {NULL}};
const Segment SGI36790[] = {{&S36222[44], 21}, {NULL}};
const Segment SGI36791[] = {{&S36222[44], 32}, {NULL}};
const Segment SGI36792[] = {{&S36222[30], 6}, {NULL}};
const Segment SGI36793[] = {{&S36222[124], 3}, {NULL}};
const Segment SGI36794[] = {{&S36222[90], 5}, {NULL}};
const Segment SGI36795[] = {{&S36223[44], 26}, {NULL}};
const Segment SGI36796[] = {{&S36223[44], 37}, {NULL}};
const Segment SGI36797[] = {{&S36223[30], 6}, {NULL}};
const Segment SGI36798[] = {{&S36223[129], 3}, {NULL}};
const Segment SGI36799[] = {{&S36223[95], 5}, {NULL}};
const Segment SGI36800[] = {{&S36224[44], 26}, {NULL}};
const Segment SGI36801[] = {{&S36224[44], 37}, {NULL}};
const Segment SGI36802[] = {{&S36224[30], 6}, {NULL}};
const Segment SGI36803[] = {{&S36224[129], 3}, {NULL}};
const Segment SGI36804[] = {{&S36224[95], 5}, {NULL}};
const Segment SGI36805[] = {{&S36225[44], 26}, {NULL}};
const Segment SGI36806[] = {{&S36225[44], 37}, {NULL}};
const Segment SGI36807[] = {{&S36225[30], 6}, {NULL}};
const Segment SGI36808[] = {{&S36225[129], 3}, {NULL}};
const Segment SGI36809[] = {{&S36225[95], 5}, {NULL}};
const Segment SGI36810[] = {{&S36226[44], 26}, {NULL}};
const Segment SGI36811[] = {{&S36226[44], 37}, {NULL}};
const Segment SGI36812[] = {{&S36226[30], 6}, {NULL}};
const Segment SGI36813[] = {{&S36226[129], 3}, {NULL}};
const Segment SGI36814[] = {{&S36226[95], 5}, {NULL}};
const Segment SGI36815[] = {{&S36227[44], 26}, {NULL}};
const Segment SGI36816[] = {{&S36227[44], 37}, {NULL}};
const Segment SGI36817[] = {{&S36227[30], 6}, {NULL}};
const Segment SGI36818[] = {{&S36227[129], 3}, {NULL}};
const Segment SGI36819[] = {{&S36227[95], 5}, {NULL}};
const Segment SGI36820[] = {{&S36228[44], 25}, {NULL}};
const Segment SGI36821[] = {{&S36228[44], 36}, {NULL}};
const Segment SGI36822[] = {{&S36228[30], 6}, {NULL}};
const Segment SGI36823[] = {{&S36228[128], 3}, {NULL}};
const Segment SGI36824[] = {{&S36228[94], 5}, {NULL}};
const Segment SGI36825[] = {{&S36229[44], 26}, {NULL}};
const Segment SGI36826[] = {{&S36229[44], 37}, {NULL}};
const Segment SGI36827[] = {{&S36229[30], 6}, {NULL}};
const Segment SGI36828[] = {{&S36229[129], 3}, {NULL}};
const Segment SGI36829[] = {{&S36229[95], 5}, {NULL}};
const Segment SGI36830[] = {{&S36230[44], 26}, {NULL}};
const Segment SGI36831[] = {{&S36230[44], 37}, {NULL}};
const Segment SGI36832[] = {{&S36230[30], 6}, {NULL}};
const Segment SGI36833[] = {{&S36230[129], 3}, {NULL}};
const Segment SGI36834[] = {{&S36230[95], 5}, {NULL}};
const Segment SGI36835[] = {{&S36231[44], 26}, {NULL}};
const Segment SGI36836[] = {{&S36231[44], 37}, {NULL}};
const Segment SGI36837[] = {{&S36231[30], 6}, {NULL}};
const Segment SGI36838[] = {{&S36231[129], 3}, {NULL}};
const Segment SGI36839[] = {{&S36231[95], 5}, {NULL}};
const Segment SGI36840[] = {{&S36232[44], 26}, {NULL}};
const Segment SGI36841[] = {{&S36232[44], 37}, {NULL}};
const Segment SGI36842[] = {{&S36232[30], 6}, {NULL}};
const Segment SGI36843[] = {{&S36232[129], 3}, {NULL}};
const Segment SGI36844[] = {{&S36232[95], 5}, {NULL}};
const Segment SGI36845[] = {{&S36233[44], 26}, {NULL}};
const Segment SGI36846[] = {{&S36233[44], 37}, {NULL}};
const Segment SGI36847[] = {{&S36233[30], 6}, {NULL}};
const Segment SGI36848[] = {{&S36233[129], 3}, {NULL}};
const Segment SGI36849[] = {{&S36233[95], 5}, {NULL}};
const Segment SGI36850[] = {{&S36234[44], 21}, {NULL}};
const Segment SGI36851[] = {{&S36234[44], 32}, {NULL}};
const Segment SGI36852[] = {{&S36234[30], 6}, {NULL}};
const Segment SGI36853[] = {{&S36234[124], 3}, {NULL}};
const Segment SGI36854[] = {{&S36234[90], 5}, {NULL}};
const Segment SGI36855[] = {{&S36235[44], 23}, {NULL}};
const Segment SGI36856[] = {{&S36235[44], 34}, {NULL}};
const Segment SGI36857[] = {{&S36235[30], 6}, {NULL}};
const Segment SGI36858[] = {{&S36235[126], 3}, {NULL}};
const Segment SGI36859[] = {{&S36235[92], 5}, {NULL}};
const Segment SGI36860[] = {{&S36239[44], 26}, {NULL}};
const Segment SGI36861[] = {{&S36239[44], 37}, {NULL}};
const Segment SGI36862[] = {{&S36239[30], 6}, {NULL}};
const Segment SGI36863[] = {{&S36239[129], 3}, {NULL}};
const Segment SGI36864[] = {{&S36239[95], 5}, {NULL}};
const Segment SGI36865[] = {{&S36257[38], 11}, {NULL}};
const Segment SGI36866[] = {{&S36257[31], 6}, {NULL}};
const Segment SGI36867[] = {{&S36261[44], 26}, {NULL}};
const Segment SGI36868[] = {{&S36261[44], 37}, {NULL}};
const Segment SGI36869[] = {{&S36261[30], 6}, {NULL}};
const Segment SGI36870[] = {{&S36261[129], 3}, {NULL}};
const Segment SGI36871[] = {{&S36261[95], 5}, {NULL}};
const Segment SGI36872[] = {{&S36262[38], 21}, {NULL}};
const Segment SGI36873[] = {{&S36262[31], 6}, {NULL}};
const Segment SGI36874[] = {{&S36263[38], 15}, {NULL}};
const Segment SGI36875[] = {{&S36263[31], 6}, {NULL}};
const Segment SGI36876[] = {{&S36264[44], 26}, {NULL}};
const Segment SGI36877[] = {{&S36264[44], 37}, {NULL}};
const Segment SGI36878[] = {{&S36264[30], 6}, {NULL}};
const Segment SGI36879[] = {{&S36264[129], 3}, {NULL}};
const Segment SGI36880[] = {{&S36264[95], 5}, {NULL}};
const Segment SGI36881[] = {{&S36280[44], 15}, {NULL}};
const Segment SGI36882[] = {{&S36280[44], 36}, {NULL}};
const Segment SGI36883[] = {{&S36280[30], 6}, {NULL}};
const Segment SGI36884[] = {{&S36280[128], 3}, {NULL}};
const Segment SGI36885[] = {{&S36280[94], 5}, {NULL}};
const Segment SGI36886[] = {{&S36281[44], 15}, {NULL}};
const Segment SGI36887[] = {{&S36281[44], 36}, {NULL}};
const Segment SGI36888[] = {{&S36281[30], 6}, {NULL}};
const Segment SGI36889[] = {{&S36281[128], 3}, {NULL}};
const Segment SGI36890[] = {{&S36281[94], 5}, {NULL}};
const Segment SGI36891[] = {{&S36282[44], 15}, {NULL}};
const Segment SGI36892[] = {{&S36282[44], 36}, {NULL}};
const Segment SGI36893[] = {{&S36282[30], 6}, {NULL}};
const Segment SGI36894[] = {{&S36282[128], 3}, {NULL}};
const Segment SGI36895[] = {{&S36282[94], 5}, {NULL}};
const Segment SGI36896[] = {{&S36283[44], 15}, {NULL}};
const Segment SGI36897[] = {{&S36283[44], 36}, {NULL}};
const Segment SGI36898[] = {{&S36283[30], 6}, {NULL}};
const Segment SGI36899[] = {{&S36283[128], 3}, {NULL}};
const Segment SGI36900[] = {{&S36283[94], 5}, {NULL}};
const Segment SGI36901[] = {{&S36284[44], 15}, {NULL}};
const Segment SGI36902[] = {{&S36284[44], 36}, {NULL}};
const Segment SGI36903[] = {{&S36284[30], 6}, {NULL}};
const Segment SGI36904[] = {{&S36284[128], 3}, {NULL}};
const Segment SGI36905[] = {{&S36284[94], 5}, {NULL}};
const Segment SGI36906[] = {{&S36285[44], 14}, {NULL}};
const Segment SGI36907[] = {{&S36285[44], 35}, {NULL}};
const Segment SGI36908[] = {{&S36285[30], 6}, {NULL}};
const Segment SGI36909[] = {{&S36285[127], 3}, {NULL}};
const Segment SGI36910[] = {{&S36285[93], 5}, {NULL}};
const Segment SGI36911[] = {{&S36287[38], 15}, {NULL}};
const Segment SGI36912[] = {{&S36287[31], 6}, {NULL}};
const Segment SGI36913[] = {{&S36291[43], 15}, {NULL}};
const Segment SGI36914[] = {{&S36291[43], 36}, {NULL}};
const Segment SGI36915[] = {{&S36291[30], 5}, {NULL}};
const Segment SGI36916[] = {{&S36291[127], 3}, {NULL}};
const Segment SGI36917[] = {{&S36291[93], 5}, {NULL}};
const Segment SGI36918[] = {{&S36292[44], 14}, {NULL}};
const Segment SGI36919[] = {{&S36292[44], 35}, {NULL}};
const Segment SGI36920[] = {{&S36292[30], 6}, {NULL}};
const Segment SGI36921[] = {{&S36292[127], 3}, {NULL}};
const Segment SGI36922[] = {{&S36292[93], 5}, {NULL}};
const Segment SGI36923[] = {{&S36293[44], 14}, {NULL}};
const Segment SGI36924[] = {{&S36293[44], 35}, {NULL}};
const Segment SGI36925[] = {{&S36293[30], 6}, {NULL}};
const Segment SGI36926[] = {{&S36293[127], 3}, {NULL}};
const Segment SGI36927[] = {{&S36293[93], 5}, {NULL}};
const Segment SGI36928[] = {{&S36297[43], 15}, {NULL}};
const Segment SGI36929[] = {{&S36297[43], 36}, {NULL}};
const Segment SGI36930[] = {{&S36297[127], 3}, {NULL}};
const Segment SGI36931[] = {{&S36297[93], 5}, {NULL}};
const Segment SGI36932[] = {{&S36298[44], 15}, {NULL}};
const Segment SGI36933[] = {{&S36298[44], 36}, {NULL}};
const Segment SGI36934[] = {{&S36298[30], 6}, {NULL}};
const Segment SGI36935[] = {{&S36298[128], 3}, {NULL}};
const Segment SGI36936[] = {{&S36298[94], 5}, {NULL}};
const Segment SGI36937[] = {{&S36304[44], 15}, {NULL}};
const Segment SGI36938[] = {{&S36304[44], 36}, {NULL}};
const Segment SGI36939[] = {{&S36304[30], 6}, {NULL}};
const Segment SGI36940[] = {{&S36304[97], 4}, {NULL}};
const Segment SGI36941[] = {{&S36304[97], 3}, {NULL}};
const Segment SGI36942[] = {{&S36305[44], 15}, {NULL}};
const Segment SGI36943[] = {{&S36305[44], 36}, {NULL}};
const Segment SGI36944[] = {{&S36305[30], 6}, {NULL}};
const Segment SGI36945[] = {{&S36305[97], 6}, {NULL}};
const Segment SGI36946[] = {{&S33742[44], 23}, {NULL}};
const Segment SGI36947[] = {{&S33742[44], 34}, {NULL}};
const Segment SGI36948[] = {{&S33742[30], 6}, {NULL}};
const Segment SGI36949[] = {{&S33742[126], 3}, {NULL}};
const Segment SGI36950[] = {{&S33742[92], 5}, {NULL}};
const Segment SGI36951[] = {{&S33743[44], 101}, {NULL}};
const Segment SGI36952[] = {{&S33743[44], 112}, {NULL}};
const Segment SGI36953[] = {{&S33743[30], 6}, {NULL}};
const Segment SGI36954[] = {{&S33743[204], 3}, {NULL}};
const Segment SGI36955[] = {{&S33743[170], 5}, {NULL}};
const Segment SGI36956[] = {{&S33744[44], 18}, {NULL}};
const Segment SGI36957[] = {{&S33744[44], 29}, {NULL}};
const Segment SGI36958[] = {{&S33744[30], 6}, {NULL}};
const Segment SGI36959[] = {{&S33744[121], 3}, {NULL}};
const Segment SGI36960[] = {{&S33744[87], 5}, {NULL}};
const Segment SGI36961[] = {{&S33745[38], 18}, {NULL}};
const Segment SGI36962[] = {{&S33745[31], 6}, {NULL}};
const Segment SGI36963[] = {{&S33746[44], 18}, {NULL}};
const Segment SGI36964[] = {{&S33746[44], 42}, {NULL}};
const Segment SGI36965[] = {{&S33746[30], 6}, {NULL}};
const Segment SGI36966[] = {{&S33746[134], 3}, {NULL}};
const Segment SGI36967[] = {{&S33746[100], 5}, {NULL}};
const Segment SGI36968[] = {{&S33747[44], 18}, {NULL}};
const Segment SGI36969[] = {{&S33747[44], 29}, {NULL}};
const Segment SGI36970[] = {{&S33747[30], 6}, {NULL}};
const Segment SGI36971[] = {{&S33747[90], 4}, {NULL}};
const Segment SGI36972[] = {{&S33747[90], 3}, {NULL}};
const Segment SGI36973[] = {{&S33749[44], 23}, {NULL}};
const Segment SGI36974[] = {{&S33749[44], 47}, {NULL}};
const Segment SGI36975[] = {{&S33749[30], 6}, {NULL}};
const Segment SGI36976[] = {{&S33749[139], 3}, {NULL}};
const Segment SGI36977[] = {{&S33749[105], 5}, {NULL}};
const Segment SGI36978[] = {{&S33750[42], 23}, {NULL}};
const Segment SGI36979[] = {{&S33750[30], 6}, {NULL}};
const Segment SGI36980[] = {{&S33750[124], 3}, {NULL}};
const Segment SGI36981[] = {{&S33750[90], 5}, {NULL}};
const Segment SGI36982[] = {{&S33751[44], 18}, {NULL}};
const Segment SGI36983[] = {{&S33751[44], 29}, {NULL}};
const Segment SGI36984[] = {{&S33751[30], 6}, {NULL}};
const Segment SGI36985[] = {{&S33751[90], 6}, {NULL}};
const Segment SGI36986[] = {{&S33753[44], 11}, {NULL}};
const Segment SGI36987[] = {{&S33753[44], 22}, {NULL}};
const Segment SGI36988[] = {{&S33753[30], 6}, {NULL}};
const Segment SGI36989[] = {{&S33753[114], 3}, {NULL}};
const Segment SGI36990[] = {{&S33753[80], 5}, {NULL}};
const Segment SGI36991[] = {{&S33756[44], 29}, {NULL}};
const Segment SGI36992[] = {{&S33756[30], 6}, {NULL}};
const Segment SGI36993[] = {{&S45883[44], 7}, {NULL}};
const Segment SGI36994[] = {{&S45883[44], 23}, {NULL}};
const Segment SGI36995[] = {{&S45883[30], 6}, {NULL}};
const Segment SGI36996[] = {{&S45883[115], 3}, {NULL}};
const Segment SGI36997[] = {{&S45883[81], 5}, {NULL}};
const Segment SGI36998[] = {{&S33804[44], 8}, {NULL}};
const Segment SGI36999[] = {{&S33804[44], 17}, {NULL}};
const Segment SGI37000[] = {{&S33804[30], 6}, {NULL}};
const Segment SGI37001[] = {{&S33804[109], 3}, {NULL}};
const Segment SGI37002[] = {{&S33804[75], 5}, {NULL}};
const Segment SGI37003[] = {{&S45885[44], 7}, {NULL}};
const Segment SGI37004[] = {{&S45885[44], 23}, {NULL}};
const Segment SGI37005[] = {{&S45885[30], 6}, {NULL}};
const Segment SGI37006[] = {{&S45885[115], 3}, {NULL}};
const Segment SGI37007[] = {{&S45885[81], 5}, {NULL}};
const Segment SGI37008[] = {{&S33805[44], 8}, {NULL}};
const Segment SGI37009[] = {{&S33805[44], 20}, {NULL}};
const Segment SGI37010[] = {{&S33805[30], 6}, {NULL}};
const Segment SGI37011[] = {{&S33805[112], 3}, {NULL}};
const Segment SGI37012[] = {{&S33805[78], 5}, {NULL}};
const Segment SGI37013[] = {{&S45890[44], 6}, {NULL}};
const Segment SGI37014[] = {{&S45890[44], 22}, {NULL}};
const Segment SGI37015[] = {{&S45890[30], 6}, {NULL}};
const Segment SGI37016[] = {{&S45890[114], 3}, {NULL}};
const Segment SGI37017[] = {{&S45890[80], 5}, {NULL}};
const Segment SGI37018[] = {{&S45892[43], 7}, {NULL}};
const Segment SGI37019[] = {{&S45892[43], 23}, {NULL}};
const Segment SGI37020[] = {{&S45892[114], 3}, {NULL}};
const Segment SGI37021[] = {{&S45892[80], 5}, {NULL}};
const Segment SGI37022[] = {{&S45893[43], 7}, {NULL}};
const Segment SGI37023[] = {{&S45893[43], 23}, {NULL}};
const Segment SGI37024[] = {{&S45893[30], 5}, {NULL}};
const Segment SGI37025[] = {{&S45893[114], 3}, {NULL}};
const Segment SGI37026[] = {{&S45893[80], 5}, {NULL}};
const Segment SGI37027[] = {{&S45895[44], 7}, {NULL}};
const Segment SGI37028[] = {{&S45895[44], 23}, {NULL}};
const Segment SGI37029[] = {{&S45895[30], 6}, {NULL}};
const Segment SGI37030[] = {{&S45895[115], 3}, {NULL}};
const Segment SGI37031[] = {{&S45895[81], 5}, {NULL}};
const Segment SGI37032[] = {{&S45899[38], 7}, {NULL}};
const Segment SGI37033[] = {{&S45899[31], 6}, {NULL}};
const Segment SGI37034[] = {{&S34409[44], 6}, {NULL}};
const Segment SGI37035[] = {{&S34409[44], 23}, {NULL}};
const Segment SGI37036[] = {{&S34409[30], 6}, {NULL}};
const Segment SGI37037[] = {{&S34409[115], 3}, {NULL}};
const Segment SGI37038[] = {{&S34409[81], 5}, {NULL}};
const Segment SGI37039[] = {{&S34410[44], 10}, {NULL}};
const Segment SGI37040[] = {{&S34410[44], 21}, {NULL}};
const Segment SGI37041[] = {{&S34410[30], 6}, {NULL}};
const Segment SGI37042[] = {{&S34410[113], 3}, {NULL}};
const Segment SGI37043[] = {{&S34410[79], 5}, {NULL}};
const Segment SGI37044[] = {{&S34411[38], 6}, {NULL}};
const Segment SGI37045[] = {{&S34411[31], 6}, {NULL}};
const Segment SGI37046[] = {{&S34412[38], 10}, {NULL}};
const Segment SGI37047[] = {{&S34412[31], 6}, {NULL}};
const Segment SGI37048[] = {{&S34414[42], 6}, {NULL}};
const Segment SGI37049[] = {{&S34414[30], 6}, {NULL}};
const Segment SGI37050[] = {{&S34414[113], 3}, {NULL}};
const Segment SGI37051[] = {{&S34414[79], 5}, {NULL}};
const Segment SGI37052[] = {{&S34415[44], 6}, {NULL}};
const Segment SGI37053[] = {{&S34415[44], 23}, {NULL}};
const Segment SGI37054[] = {{&S34415[30], 6}, {NULL}};
const Segment SGI37055[] = {{&S34415[81], 5}, {NULL}};
const Segment SGI37056[] = {{&S33946[38], 14}, {NULL}};
const Segment SGI37057[] = {{&S33946[31], 6}, {NULL}};
const Segment SGI37058[] = {{&S33949[44], 14}, {NULL}};
const Segment SGI37059[] = {{&S33949[44], 31}, {NULL}};
const Segment SGI37060[] = {{&S33949[30], 6}, {NULL}};
const Segment SGI37061[] = {{&S33949[123], 3}, {NULL}};
const Segment SGI37062[] = {{&S33949[89], 5}, {NULL}};
const Segment SGI37063[] = {{&S33950[44], 14}, {NULL}};
const Segment SGI37064[] = {{&S33950[44], 32}, {NULL}};
const Segment SGI37065[] = {{&S33950[30], 6}, {NULL}};
const Segment SGI37066[] = {{&S33950[124], 3}, {NULL}};
const Segment SGI37067[] = {{&S33950[90], 5}, {NULL}};
const Segment SGI37068[] = {{&S33953[42], 14}, {NULL}};
const Segment SGI37069[] = {{&S33953[30], 6}, {NULL}};
const Segment SGI37070[] = {{&S33953[121], 3}, {NULL}};
const Segment SGI37071[] = {{&S33953[87], 5}, {NULL}};
const Segment SGI37072[] = {{&S36670[44], 19}, {NULL}};
const Segment SGI37073[] = {{&S36670[44], 36}, {NULL}};
const Segment SGI37074[] = {{&S36670[30], 6}, {NULL}};
const Segment SGI37075[] = {{&S36670[69], 3}, {NULL}};
const Segment SGI37076[] = {{&S36670[94], 5}, {NULL}};
const Segment SGI37077[] = {{&S36674[44], 7}, {NULL}};
const Segment SGI37078[] = {{&S36674[44], 26}, {NULL}};
const Segment SGI37079[] = {{&S36674[30], 6}, {NULL}};
const Segment SGI37080[] = {{&S36674[118], 3}, {NULL}};
const Segment SGI37081[] = {{&S36674[84], 5}, {NULL}};
const Segment SGI37082[] = {{&S36675[44], 19}, {NULL}};
const Segment SGI37083[] = {{&S36675[44], 38}, {NULL}};
const Segment SGI37084[] = {{&S36675[30], 6}, {NULL}};
const Segment SGI37085[] = {{&S36675[130], 3}, {NULL}};
const Segment SGI37086[] = {{&S36675[96], 5}, {NULL}};
const Segment SGI37087[] = {{&S36676[44], 19}, {NULL}};
const Segment SGI37088[] = {{&S36676[44], 36}, {NULL}};
const Segment SGI37089[] = {{&S36676[30], 6}, {NULL}};
const Segment SGI37090[] = {{&S36676[69], 3}, {NULL}};
const Segment SGI37091[] = {{&S36676[94], 5}, {NULL}};
const Segment SGI37092[] = {{&S36677[44], 20}, {NULL}};
const Segment SGI37093[] = {{&S36677[44], 39}, {NULL}};
const Segment SGI37094[] = {{&S36677[30], 6}, {NULL}};
const Segment SGI37095[] = {{&S36677[131], 3}, {NULL}};
const Segment SGI37096[] = {{&S36677[97], 5}, {NULL}};
const Segment SGI37097[] = {{&S36680[38], 7}, {NULL}};
const Segment SGI37098[] = {{&S36680[31], 6}, {NULL}};
const Segment SGI37099[] = {{&S36688[44], 7}, {NULL}};
const Segment SGI37100[] = {{&S36688[44], 24}, {NULL}};
const Segment SGI37101[] = {{&S36688[30], 6}, {NULL}};
const Segment SGI37102[] = {{&S36688[57], 3}, {NULL}};
const Segment SGI37103[] = {{&S36688[82], 5}, {NULL}};
const Segment SGI37104[] = {{&S38548[44], 10}, {NULL}};
const Segment SGI37105[] = {{&S38548[44], 27}, {NULL}};
const Segment SGI37106[] = {{&S38548[30], 6}, {NULL}};
const Segment SGI37107[] = {{&S38548[119], 3}, {NULL}};
const Segment SGI37108[] = {{&S38548[85], 5}, {NULL}};
const Segment SGI37109[] = {{&S41045[42], 10}, {NULL}};
const Segment SGI37110[] = {{&S41045[42], 21}, {NULL}};
const Segment SGI37111[] = {{&S41045[30], 4}, {NULL}};
const Segment SGI37112[] = {{&S41045[111], 3}, {NULL}};
const Segment SGI37113[] = {{&S41045[77], 5}, {NULL}};
const Segment SGI37114[] = {{&S36826[44], 12}, {NULL}};
const Segment SGI37115[] = {{&S36826[44], 23}, {NULL}};
const Segment SGI37116[] = {{&S36826[30], 6}, {NULL}};
const Segment SGI37117[] = {{&S36826[115], 3}, {NULL}};
const Segment SGI37118[] = {{&S36826[81], 5}, {NULL}};
const Segment SGI37119[] = {{&S36827[44], 9}, {NULL}};
const Segment SGI37120[] = {{&S36827[44], 20}, {NULL}};
const Segment SGI37121[] = {{&S36827[30], 6}, {NULL}};
const Segment SGI37122[] = {{&S36827[112], 3}, {NULL}};
const Segment SGI37123[] = {{&S36827[78], 5}, {NULL}};
const Segment SGI37124[] = {{&S36828[38], 9}, {NULL}};
const Segment SGI37125[] = {{&S36828[31], 6}, {NULL}};
const Segment SGI37126[] = {{&S36829[44], 12}, {NULL}};
const Segment SGI37127[] = {{&S36829[44], 28}, {NULL}};
const Segment SGI37128[] = {{&S36829[30], 6}, {NULL}};
const Segment SGI37129[] = {{&S36829[120], 3}, {NULL}};
const Segment SGI37130[] = {{&S36829[86], 5}, {NULL}};
const Segment SGI37131[] = {{&S36830[44], 9}, {NULL}};
const Segment SGI37132[] = {{&S36830[44], 25}, {NULL}};
const Segment SGI37133[] = {{&S36830[30], 6}, {NULL}};
const Segment SGI37134[] = {{&S36830[117], 3}, {NULL}};
const Segment SGI37135[] = {{&S36830[83], 5}, {NULL}};
const Segment SGI37136[] = {{&S36831[42], 12}, {NULL}};
const Segment SGI37137[] = {{&S36831[30], 6}, {NULL}};
const Segment SGI37138[] = {{&S36831[113], 3}, {NULL}};
const Segment SGI37139[] = {{&S36831[79], 5}, {NULL}};
const Segment SGI37140[] = {{&S34652[43], 11}, {NULL}};
const Segment SGI37141[] = {{&S34652[43], 22}, {NULL}};
const Segment SGI37142[] = {{&S34652[113], 3}, {NULL}};
const Segment SGI37143[] = {{&S34652[79], 5}, {NULL}};
const Segment SGI37144[] = {{&S34653[44], 11}, {NULL}};
const Segment SGI37145[] = {{&S34653[44], 22}, {NULL}};
const Segment SGI37146[] = {{&S34653[30], 6}, {NULL}};
const Segment SGI37147[] = {{&S34653[114], 3}, {NULL}};
const Segment SGI37148[] = {{&S34653[80], 5}, {NULL}};
const Segment SGI37149[] = {{&S34654[44], 10}, {NULL}};
const Segment SGI37150[] = {{&S34654[44], 21}, {NULL}};
const Segment SGI37151[] = {{&S34654[30], 6}, {NULL}};
const Segment SGI37152[] = {{&S34654[113], 3}, {NULL}};
const Segment SGI37153[] = {{&S34654[79], 5}, {NULL}};
const Segment SGI37154[] = {{&S34655[44], 10}, {NULL}};
const Segment SGI37155[] = {{&S34655[44], 21}, {NULL}};
const Segment SGI37156[] = {{&S34655[30], 6}, {NULL}};
const Segment SGI37157[] = {{&S34655[113], 3}, {NULL}};
const Segment SGI37158[] = {{&S34655[79], 5}, {NULL}};
const Segment SGI37159[] = {{&S34656[43], 11}, {NULL}};
const Segment SGI37160[] = {{&S34656[43], 22}, {NULL}};
const Segment SGI37161[] = {{&S34656[30], 5}, {NULL}};
const Segment SGI37162[] = {{&S34656[113], 3}, {NULL}};
const Segment SGI37163[] = {{&S34656[79], 5}, {NULL}};
const Segment SGI37164[] = {{&S34657[44], 11}, {NULL}};
const Segment SGI37165[] = {{&S34657[44], 22}, {NULL}};
const Segment SGI37166[] = {{&S34657[30], 6}, {NULL}};
const Segment SGI37167[] = {{&S34657[114], 3}, {NULL}};
const Segment SGI37168[] = {{&S34657[80], 5}, {NULL}};
const Segment SGI37169[] = {{&S34658[44], 11}, {NULL}};
const Segment SGI37170[] = {{&S34658[44], 22}, {NULL}};
const Segment SGI37171[] = {{&S34658[30], 6}, {NULL}};
const Segment SGI37172[] = {{&S34658[114], 3}, {NULL}};
const Segment SGI37173[] = {{&S34658[80], 5}, {NULL}};
const Segment SGI37174[] = {{&S34670[44], 11}, {NULL}};
const Segment SGI37175[] = {{&S34670[44], 22}, {NULL}};
const Segment SGI37176[] = {{&S34670[30], 6}, {NULL}};
const Segment SGI37177[] = {{&S34670[114], 3}, {NULL}};
const Segment SGI37178[] = {{&S34670[80], 5}, {NULL}};
const Segment SGI37179[] = {{&S34673[38], 11}, {NULL}};
const Segment SGI37180[] = {{&S34673[31], 6}, {NULL}};
const Segment SGI37181[] = {{&S34681[44], 11}, {NULL}};
const Segment SGI37182[] = {{&S34681[44], 35}, {NULL}};
const Segment SGI37183[] = {{&S34681[30], 6}, {NULL}};
const Segment SGI37184[] = {{&S34681[127], 3}, {NULL}};
const Segment SGI37185[] = {{&S34681[93], 5}, {NULL}};
const Segment SGI37186[] = {{&S34779[44], 11}, {NULL}};
const Segment SGI37187[] = {{&S34779[44], 28}, {NULL}};
const Segment SGI37188[] = {{&S34779[30], 6}, {NULL}};
const Segment SGI37189[] = {{&S34779[120], 3}, {NULL}};
const Segment SGI37190[] = {{&S34779[86], 5}, {NULL}};
const Segment SGI37191[] = {{&S34792[38], 11}, {NULL}};
const Segment SGI37192[] = {{&S34792[31], 6}, {NULL}};
const Segment SGI37193[] = {{&S42230[44], 21}, {NULL}};
const Segment SGI37194[] = {{&S42230[44], 33}, {NULL}};
const Segment SGI37195[] = {{&S42230[30], 6}, {NULL}};
const Segment SGI37196[] = {{&S42230[125], 3}, {NULL}};
const Segment SGI37197[] = {{&S42230[91], 5}, {NULL}};
const Segment SGI37198[] = {{&S42231[38], 21}, {NULL}};
const Segment SGI37199[] = {{&S42231[31], 6}, {NULL}};
const Segment SGI37200[] = {{&S42233[44], 20}, {NULL}};
const Segment SGI37201[] = {{&S42233[44], 32}, {NULL}};
const Segment SGI37202[] = {{&S42233[30], 6}, {NULL}};
const Segment SGI37203[] = {{&S42233[124], 3}, {NULL}};
const Segment SGI37204[] = {{&S42233[90], 5}, {NULL}};
const Segment SGI37205[] = {{&S42234[43], 21}, {NULL}};
const Segment SGI37206[] = {{&S42234[43], 33}, {NULL}};
const Segment SGI37207[] = {{&S42234[30], 5}, {NULL}};
const Segment SGI37208[] = {{&S42234[124], 3}, {NULL}};
const Segment SGI37209[] = {{&S42234[90], 5}, {NULL}};
const Segment SGI37210[] = {{&S42235[44], 20}, {NULL}};
const Segment SGI37211[] = {{&S42235[44], 32}, {NULL}};
const Segment SGI37212[] = {{&S42235[30], 6}, {NULL}};
const Segment SGI37213[] = {{&S42235[124], 3}, {NULL}};
const Segment SGI37214[] = {{&S42235[90], 5}, {NULL}};
const Segment SGI37215[] = {{&S37091[44], 10}, {NULL}};
const Segment SGI37216[] = {{&S37091[44], 27}, {NULL}};
const Segment SGI37217[] = {{&S37091[30], 6}, {NULL}};
const Segment SGI37218[] = {{&S42236[42], 21}, {NULL}};
const Segment SGI37219[] = {{&S42236[30], 6}, {NULL}};
const Segment SGI37220[] = {{&S37091[119], 3}, {NULL}};
const Segment SGI37221[] = {{&S42236[123], 3}, {NULL}};
const Segment SGI37222[] = {{&S37091[85], 5}, {NULL}};
const Segment SGI37223[] = {{&S42236[89], 5}, {NULL}};
const Segment SGI37224[] = {{&S42237[43], 21}, {NULL}};
const Segment SGI37225[] = {{&S42237[43], 33}, {NULL}};
const Segment SGI37226[] = {{&S42237[124], 3}, {NULL}};
const Segment SGI37227[] = {{&S42237[90], 5}, {NULL}};
const Segment SGI37228[] = {{&S34938[44], 9}, {NULL}};
const Segment SGI37229[] = {{&S34938[44], 20}, {NULL}};
const Segment SGI37230[] = {{&S34938[30], 6}, {NULL}};
const Segment SGI37231[] = {{&S34938[112], 3}, {NULL}};
const Segment SGI37232[] = {{&S34938[78], 5}, {NULL}};
const Segment SGI37233[] = {{&S34939[44], 8}, {NULL}};
const Segment SGI37234[] = {{&S34939[44], 19}, {NULL}};
const Segment SGI37235[] = {{&S34939[30], 6}, {NULL}};
const Segment SGI37236[] = {{&S34939[111], 3}, {NULL}};
const Segment SGI37237[] = {{&S34939[77], 5}, {NULL}};
const Segment SGI37238[] = {{&S34940[43], 9}, {NULL}};
const Segment SGI37239[] = {{&S34940[43], 20}, {NULL}};
const Segment SGI37240[] = {{&S34940[30], 5}, {NULL}};
const Segment SGI37241[] = {{&S34940[111], 3}, {NULL}};
const Segment SGI37242[] = {{&S34940[77], 5}, {NULL}};
const Segment SGI37243[] = {{&S34941[43], 9}, {NULL}};
const Segment SGI37244[] = {{&S34941[43], 20}, {NULL}};
const Segment SGI37245[] = {{&S34941[111], 3}, {NULL}};
const Segment SGI37246[] = {{&S34941[77], 5}, {NULL}};
const Segment SGI37247[] = {{&S34942[44], 8}, {NULL}};
const Segment SGI37248[] = {{&S34942[44], 19}, {NULL}};
const Segment SGI37249[] = {{&S34942[30], 6}, {NULL}};
const Segment SGI37250[] = {{&S34942[111], 3}, {NULL}};
const Segment SGI37251[] = {{&S34942[77], 5}, {NULL}};
const Segment SGI37252[] = {{&S34943[44], 9}, {NULL}};
const Segment SGI37253[] = {{&S34943[44], 20}, {NULL}};
const Segment SGI37254[] = {{&S34943[30], 6}, {NULL}};
const Segment SGI37255[] = {{&S34943[112], 3}, {NULL}};
const Segment SGI37256[] = {{&S34943[78], 5}, {NULL}};
const Segment SGI37257[] = {{&S46680[44], 7}, {NULL}};
const Segment SGI37258[] = {{&S46680[44], 24}, {NULL}};
const Segment SGI37259[] = {{&S46680[30], 6}, {NULL}};
const Segment SGI37260[] = {{&S46680[116], 3}, {NULL}};
const Segment SGI37261[] = {{&S46680[82], 5}, {NULL}};
const Segment SGI37262[] = {{&S34948[44], 9}, {NULL}};
const Segment SGI37263[] = {{&S34948[44], 20}, {NULL}};
const Segment SGI37264[] = {{&S34948[30], 6}, {NULL}};
const Segment SGI37265[] = {{&S46683[44], 5}, {NULL}};
const Segment SGI37266[] = {{&S46683[44], 22}, {NULL}};
const Segment SGI37267[] = {{&S46683[30], 6}, {NULL}};
const Segment SGI37268[] = {{&S34948[112], 3}, {NULL}};
const Segment SGI37269[] = {{&S34948[78], 5}, {NULL}};
const Segment SGI37270[] = {{&S46683[114], 3}, {NULL}};
const Segment SGI37271[] = {{&S46683[80], 5}, {NULL}};
const Segment SGI37272[] = {{&S34949[38], 9}, {NULL}};
const Segment SGI37273[] = {{&S34949[31], 6}, {NULL}};
const Segment SGI37274[] = {{&S46685[38], 7}, {NULL}};
const Segment SGI37275[] = {{&S46685[31], 6}, {NULL}};
const Segment SGI37276[] = {{&S46686[44], 9}, {NULL}};
const Segment SGI37277[] = {{&S46686[44], 26}, {NULL}};
const Segment SGI37278[] = {{&S46686[30], 6}, {NULL}};
const Segment SGI37279[] = {{&S34952[44], 9}, {NULL}};
const Segment SGI37280[] = {{&S34952[44], 20}, {NULL}};
const Segment SGI37281[] = {{&S46686[118], 3}, {NULL}};
const Segment SGI37282[] = {{&S34952[30], 6}, {NULL}};
const Segment SGI37283[] = {{&S46686[84], 5}, {NULL}};
const Segment SGI37284[] = {{&S46687[44], 20}, {NULL}};
const Segment SGI37285[] = {{&S46687[44], 37}, {NULL}};
const Segment SGI37286[] = {{&S46687[30], 6}, {NULL}};
const Segment SGI37287[] = {{&S46687[129], 3}, {NULL}};
const Segment SGI37288[] = {{&S46687[95], 5}, {NULL}};
const Segment SGI37289[] = {{&S34955[44], 9}, {NULL}};
const Segment SGI37290[] = {{&S46688[44], 8}, {NULL}};
const Segment SGI37291[] = {{&S34955[44], 20}, {NULL}};
const Segment SGI37292[] = {{&S34955[30], 6}, {NULL}};
const Segment SGI37293[] = {{&S46688[44], 25}, {NULL}};
const Segment SGI37294[] = {{&S46688[30], 6}, {NULL}};
const Segment SGI37295[] = {{&S34955[114], 3}, {NULL}};
const Segment SGI37296[] = {{&S46688[117], 3}, {NULL}};
const Segment SGI37297[] = {{&S34955[78], 7}, {NULL}};
const Segment SGI37298[] = {{&S34955[134], 8}, {NULL}};
const Segment SGI37299[] = {{&S46688[83], 5}, {NULL}};
const Segment SGI37300[] = {{&S34958[42], 9}, {NULL}};
const Segment SGI37301[] = {{&S34958[30], 6}, {NULL}};
const Segment SGI37302[] = {{&S34958[110], 3}, {NULL}};
const Segment SGI37303[] = {{&S34958[76], 5}, {NULL}};
const Segment SGI37304[] = {{&S38942[44], 12}, {NULL}};
const Segment SGI37305[] = {{&S38942[44], 23}, {NULL}};
const Segment SGI37306[] = {{&S38942[30], 6}, {NULL}};
const Segment SGI37307[] = {{&S38942[115], 3}, {NULL}};
const Segment SGI37308[] = {{&S38942[81], 5}, {NULL}};
const Segment SGI37309[] = {{&S34659[44], 15}, {NULL}};
const Segment SGI37310[] = {{&S34659[44], 26}, {NULL}};
const Segment SGI37311[] = {{&S34659[30], 6}, {NULL}};
const Segment SGI37312[] = {{&S34659[118], 3}, {NULL}};
const Segment SGI37313[] = {{&S34659[84], 5}, {NULL}};
const Segment SGI37314[] = {{&S34660[44], 11}, {NULL}};
const Segment SGI37315[] = {{&S34660[44], 22}, {NULL}};
const Segment SGI37316[] = {{&S34660[30], 6}, {NULL}};
const Segment SGI37317[] = {{&S34660[114], 3}, {NULL}};
const Segment SGI37318[] = {{&S34660[80], 5}, {NULL}};
const Segment SGI37319[] = {{&S34671[44], 21}, {NULL}};
const Segment SGI37320[] = {{&S34671[44], 32}, {NULL}};
const Segment SGI37321[] = {{&S34671[30], 6}, {NULL}};
const Segment SGI37322[] = {{&S34671[124], 3}, {NULL}};
const Segment SGI37323[] = {{&S34671[90], 5}, {NULL}};
const Segment SGI37324[] = {{&S34675[44], 23}, {NULL}};
const Segment SGI37325[] = {{&S34675[44], 34}, {NULL}};
const Segment SGI37326[] = {{&S34675[30], 6}, {NULL}};
const Segment SGI37327[] = {{&S34675[126], 3}, {NULL}};
const Segment SGI37328[] = {{&S34675[92], 5}, {NULL}};
const Segment SGI37329[] = {{&S34676[44], 22}, {NULL}};
const Segment SGI37330[] = {{&S34676[44], 33}, {NULL}};
const Segment SGI37331[] = {{&S34676[30], 6}, {NULL}};
const Segment SGI37332[] = {{&S34676[125], 3}, {NULL}};
const Segment SGI37333[] = {{&S34676[91], 5}, {NULL}};
const Segment SGI37334[] = {{&S34679[38], 22}, {NULL}};
const Segment SGI37335[] = {{&S34679[31], 6}, {NULL}};
const Segment SGI37336[] = {{&S34682[44], 23}, {NULL}};
const Segment SGI37337[] = {{&S34682[44], 34}, {NULL}};
const Segment SGI37338[] = {{&S34682[30], 6}, {NULL}};
const Segment SGI37339[] = {{&S34682[126], 3}, {NULL}};
const Segment SGI37340[] = {{&S34682[92], 5}, {NULL}};
const Segment SGI37341[] = {{&S34683[44], 23}, {NULL}};
const Segment SGI37342[] = {{&S34683[44], 34}, {NULL}};
const Segment SGI37343[] = {{&S34683[30], 6}, {NULL}};
const Segment SGI37344[] = {{&S34683[126], 3}, {NULL}};
const Segment SGI37345[] = {{&S34683[92], 5}, {NULL}};
const Segment SGI37346[] = {{&S35058[44], 9}, {NULL}};
const Segment SGI37347[] = {{&S35058[44], 20}, {NULL}};
const Segment SGI37348[] = {{&S35058[30], 6}, {NULL}};
const Segment SGI37349[] = {{&S35058[112], 3}, {NULL}};
const Segment SGI37350[] = {{&S35058[78], 5}, {NULL}};
const Segment SGI37351[] = {{&S35059[38], 9}, {NULL}};
const Segment SGI37352[] = {{&S35059[31], 6}, {NULL}};
const Segment SGI37353[] = {{&S35060[44], 9}, {NULL}};
const Segment SGI37354[] = {{&S35060[44], 20}, {NULL}};
const Segment SGI37355[] = {{&S35060[30], 6}, {NULL}};
const Segment SGI37356[] = {{&S35060[112], 3}, {NULL}};
const Segment SGI37357[] = {{&S35060[78], 5}, {NULL}};
const Segment SGI37358[] = {{&S35062[44], 9}, {NULL}};
const Segment SGI37359[] = {{&S35062[44], 20}, {NULL}};
const Segment SGI37360[] = {{&S35062[30], 6}, {NULL}};
const Segment SGI37361[] = {{&S35062[112], 3}, {NULL}};
const Segment SGI37362[] = {{&S35062[78], 5}, {NULL}};
const Segment SGI37363[] = {{&S35086[44], 14}, {NULL}};
const Segment SGI37364[] = {{&S35086[44], 31}, {NULL}};
const Segment SGI37365[] = {{&S35086[30], 6}, {NULL}};
const Segment SGI37366[] = {{&S35086[123], 3}, {NULL}};
const Segment SGI37367[] = {{&S35086[89], 5}, {NULL}};
const Segment SGI37368[] = {{&S35087[44], 9}, {NULL}};
const Segment SGI37369[] = {{&S35087[44], 20}, {NULL}};
const Segment SGI37370[] = {{&S35087[30], 6}, {NULL}};
const Segment SGI37371[] = {{&S35087[81], 4}, {NULL}};
const Segment SGI37372[] = {{&S35087[81], 3}, {NULL}};
const Segment SGI37373[] = {{&S35089[44], 9}, {NULL}};
const Segment SGI37374[] = {{&S35089[44], 26}, {NULL}};
const Segment SGI37375[] = {{&S35089[30], 6}, {NULL}};
const Segment SGI37376[] = {{&S35089[118], 3}, {NULL}};
const Segment SGI37377[] = {{&S35089[84], 5}, {NULL}};
const Segment SGI37378[] = {{&S35097[44], 10}, {NULL}};
const Segment SGI37379[] = {{&S35097[44], 21}, {NULL}};
const Segment SGI37380[] = {{&S35097[30], 6}, {NULL}};
const Segment SGI37381[] = {{&S35097[113], 3}, {NULL}};
const Segment SGI37382[] = {{&S35097[79], 5}, {NULL}};
const Segment SGI37383[] = {{&S35098[44], 13}, {NULL}};
const Segment SGI37384[] = {{&S35098[44], 24}, {NULL}};
const Segment SGI37385[] = {{&S35098[30], 6}, {NULL}};
const Segment SGI37386[] = {{&S35098[116], 3}, {NULL}};
const Segment SGI37387[] = {{&S35098[82], 5}, {NULL}};
const Segment SGI37388[] = {{&S34781[44], 11}, {NULL}};
const Segment SGI37389[] = {{&S34781[44], 28}, {NULL}};
const Segment SGI37390[] = {{&S34781[30], 6}, {NULL}};
const Segment SGI37391[] = {{&S34781[120], 3}, {NULL}};
const Segment SGI37392[] = {{&S35100[38], 10}, {NULL}};
const Segment SGI37393[] = {{&S37211[44], 7}, {NULL}};
const Segment SGI37394[] = {{&S35100[31], 6}, {NULL}};
const Segment SGI37395[] = {{&S37211[44], 18}, {NULL}};
const Segment SGI37396[] = {{&S34781[86], 5}, {NULL}};
const Segment SGI37397[] = {{&S37211[30], 6}, {NULL}};
const Segment SGI37398[] = {{&S37211[110], 3}, {NULL}};
const Segment SGI37399[] = {{&S37211[76], 5}, {NULL}};
const Segment SGI37400[] = {{&S35101[44], 13}, {NULL}};
const Segment SGI37401[] = {{&S35101[44], 25}, {NULL}};
const Segment SGI37402[] = {{&S35101[30], 6}, {NULL}};
const Segment SGI37403[] = {{&S35101[117], 3}, {NULL}};
const Segment SGI37404[] = {{&S35101[83], 5}, {NULL}};
const Segment SGI37405[] = {{&S35102[44], 13}, {NULL}};
const Segment SGI37406[] = {{&S35102[44], 24}, {NULL}};
const Segment SGI37407[] = {{&S35102[30], 6}, {NULL}};
const Segment SGI37408[] = {{&S35102[116], 3}, {NULL}};
const Segment SGI37409[] = {{&S35102[82], 5}, {NULL}};
const Segment SGI37410[] = {{&S37214[44], 7}, {NULL}};
const Segment SGI37411[] = {{&S37214[44], 24}, {NULL}};
const Segment SGI37412[] = {{&S37214[30], 6}, {NULL}};
const Segment SGI37413[] = {{&S37214[116], 3}, {NULL}};
const Segment SGI37414[] = {{&S37214[82], 5}, {NULL}};
const Segment SGI37415[] = {{&S37215[38], 7}, {NULL}};
const Segment SGI37416[] = {{&S37215[31], 6}, {NULL}};
const Segment SGI37417[] = {{&S35109[44], 10}, {NULL}};
const Segment SGI37418[] = {{&S35109[44], 21}, {NULL}};
const Segment SGI37419[] = {{&S35109[30], 6}, {NULL}};
const Segment SGI37420[] = {{&S37220[44], 7}, {NULL}};
const Segment SGI37421[] = {{&S34793[38], 11}, {NULL}};
const Segment SGI37422[] = {{&S37220[44], 18}, {NULL}};
const Segment SGI37423[] = {{&S34793[31], 6}, {NULL}};
const Segment SGI37424[] = {{&S37220[30], 6}, {NULL}};
const Segment SGI37425[] = {{&S37220[112], 3}, {NULL}};
const Segment SGI37426[] = {{&S37220[76], 7}, {NULL}};
const Segment SGI37427[] = {{&S37220[132], 8}, {NULL}};
const Segment SGI37428[] = {{&S37221[42], 7}, {NULL}};
const Segment SGI37429[] = {{&S37221[30], 6}, {NULL}};
const Segment SGI37430[] = {{&S35112[44], 13}, {NULL}};
const Segment SGI37431[] = {{&S35112[44], 24}, {NULL}};
const Segment SGI37432[] = {{&S37221[108], 3}, {NULL}};
const Segment SGI37433[] = {{&S35112[30], 6}, {NULL}};
const Segment SGI37434[] = {{&S37221[74], 5}, {NULL}};
const Segment SGI37435[] = {{&S35112[116], 3}, {NULL}};
const Segment SGI37436[] = {{&S35112[82], 5}, {NULL}};
const Segment SGI37437[] = {{&S35114[44], 12}, {NULL}};
const Segment SGI37438[] = {{&S35114[44], 23}, {NULL}};
const Segment SGI37439[] = {{&S35114[30], 6}, {NULL}};
const Segment SGI37440[] = {{&S35114[115], 3}, {NULL}};
const Segment SGI37441[] = {{&S35114[81], 5}, {NULL}};
const Segment SGI37442[] = {{&S35115[44], 12}, {NULL}};
const Segment SGI37443[] = {{&S35115[44], 23}, {NULL}};
const Segment SGI37444[] = {{&S35115[30], 6}, {NULL}};
const Segment SGI37445[] = {{&S35115[115], 3}, {NULL}};
const Segment SGI37446[] = {{&S35115[81], 5}, {NULL}};
const Segment SGI37447[] = {{&S35116[43], 13}, {NULL}};
const Segment SGI37448[] = {{&S35116[43], 24}, {NULL}};
const Segment SGI37449[] = {{&S35116[30], 5}, {NULL}};
const Segment SGI37450[] = {{&S35116[115], 3}, {NULL}};
const Segment SGI37451[] = {{&S35116[81], 5}, {NULL}};
const Segment SGI37452[] = {{&S35117[43], 13}, {NULL}};
const Segment SGI37453[] = {{&S35117[43], 24}, {NULL}};
const Segment SGI37454[] = {{&S35117[115], 3}, {NULL}};
const Segment SGI37455[] = {{&S35117[81], 5}, {NULL}};
const Segment SGI37456[] = {{&S35118[44], 12}, {NULL}};
const Segment SGI37457[] = {{&S35118[44], 23}, {NULL}};
const Segment SGI37458[] = {{&S35118[30], 6}, {NULL}};
const Segment SGI37459[] = {{&S35118[115], 3}, {NULL}};
const Segment SGI37460[] = {{&S35118[81], 5}, {NULL}};
const Segment SGI37461[] = {{&S35119[42], 13}, {NULL}};
const Segment SGI37462[] = {{&S35119[30], 6}, {NULL}};
const Segment SGI37463[] = {{&S35119[114], 3}, {NULL}};
const Segment SGI37464[] = {{&S35119[80], 5}, {NULL}};
const Segment SGI37465[] = {{&S35122[44], 10}, {NULL}};
const Segment SGI37466[] = {{&S35122[44], 21}, {NULL}};
const Segment SGI37467[] = {{&S35122[30], 6}, {NULL}};
const Segment SGI37468[] = {{&S35122[115], 3}, {NULL}};
const Segment SGI37469[] = {{&S35122[79], 7}, {NULL}};
const Segment SGI37470[] = {{&S35122[135], 8}, {NULL}};
const Segment SGI37471[] = {{&S35125[44], 17}, {NULL}};
const Segment SGI37472[] = {{&S35125[44], 28}, {NULL}};
const Segment SGI37473[] = {{&S35125[30], 6}, {NULL}};
const Segment SGI37474[] = {{&S35125[120], 3}, {NULL}};
const Segment SGI37475[] = {{&S35125[86], 5}, {NULL}};
const Segment SGI37476[] = {{&S35126[42], 10}, {NULL}};
const Segment SGI37477[] = {{&S35126[30], 6}, {NULL}};
const Segment SGI37478[] = {{&S35126[111], 3}, {NULL}};
const Segment SGI37479[] = {{&S35126[77], 5}, {NULL}};
const Segment SGI37480[] = {{&S35127[44], 10}, {NULL}};
const Segment SGI37481[] = {{&S35127[44], 21}, {NULL}};
const Segment SGI37482[] = {{&S35127[30], 6}, {NULL}};
const Segment SGI37483[] = {{&S35127[82], 4}, {NULL}};
const Segment SGI37484[] = {{&S35127[82], 3}, {NULL}};
const Segment SGI37485[] = {{&S35128[44], 10}, {NULL}};
const Segment SGI37486[] = {{&S35128[44], 21}, {NULL}};
const Segment SGI37487[] = {{&S35128[30], 6}, {NULL}};
const Segment SGI37488[] = {{&S35128[82], 6}, {NULL}};
const Segment SGI37489[] = {{&S35129[44], 10}, {NULL}};
const Segment SGI37490[] = {{&S35129[44], 21}, {NULL}};
const Segment SGI37491[] = {{&S35129[30], 6}, {NULL}};
const Segment SGI37492[] = {{&S35129[79], 6}, {NULL}};
const Segment SGI37493[] = {{&S46809[95], 7}, {NULL}};
const Segment SGI37494[] = {{&S46809[95], 23}, {NULL}};
const Segment SGI37495[] = {{&S46809[81], 6}, {NULL}};
const Segment SGI37496[] = {{&S46809[166], 3}, {NULL}};
const Segment SGI37497[] = {{&S46809[132], 5}, {NULL}};
const Segment SGI37498[] = {{&S46810[95], 7}, {NULL}};
const Segment SGI37499[] = {{&S46810[95], 23}, {NULL}};
const Segment SGI37500[] = {{&S46810[81], 6}, {NULL}};
const Segment SGI37501[] = {{&S46810[166], 3}, {NULL}};
const Segment SGI37502[] = {{&S46810[132], 5}, {NULL}};
const Segment SGI37503[] = {{&S46812[44], 7}, {NULL}};
const Segment SGI37504[] = {{&S46812[44], 26}, {NULL}};
const Segment SGI37505[] = {{&S46812[30], 6}, {NULL}};
const Segment SGI37506[] = {{&S46812[118], 3}, {NULL}};
const Segment SGI37507[] = {{&S46812[84], 5}, {NULL}};
const Segment SGI37508[] = {{&S46820[93], 7}, {NULL}};
const Segment SGI37509[] = {{&S46820[93], 24}, {NULL}};
const Segment SGI37510[] = {{&S46820[79], 6}, {NULL}};
const Segment SGI37511[] = {{&S46820[165], 3}, {NULL}};
const Segment SGI37512[] = {{&S46820[131], 5}, {NULL}};
const Segment SGI37513[] = {{&S46821[99], 7}, {NULL}};
const Segment SGI37514[] = {{&S46821[99], 23}, {NULL}};
const Segment SGI37515[] = {{&S46821[85], 6}, {NULL}};
const Segment SGI37516[] = {{&S46821[170], 3}, {NULL}};
const Segment SGI37517[] = {{&S46821[136], 5}, {NULL}};
const Segment SGI37518[] = {{&S46822[99], 7}, {NULL}};
const Segment SGI37519[] = {{&S46822[99], 23}, {NULL}};
const Segment SGI37520[] = {{&S46822[85], 6}, {NULL}};
const Segment SGI37521[] = {{&S46822[170], 3}, {NULL}};
const Segment SGI37522[] = {{&S46822[136], 5}, {NULL}};
const Segment SGI37523[] = {{&S34945[44], 9}, {NULL}};
const Segment SGI37524[] = {{&S34945[44], 20}, {NULL}};
const Segment SGI37525[] = {{&S34945[30], 6}, {NULL}};
const Segment SGI37526[] = {{&S34945[112], 3}, {NULL}};
const Segment SGI37527[] = {{&S34945[78], 5}, {NULL}};
const Segment SGI37528[] = {{&S34951[44], 9}, {NULL}};
const Segment SGI37529[] = {{&S34951[44], 20}, {NULL}};
const Segment SGI37530[] = {{&S34951[30], 6}, {NULL}};
const Segment SGI37531[] = {{&S34951[112], 3}, {NULL}};
const Segment SGI37532[] = {{&S34951[78], 5}, {NULL}};
const Segment SGI37533[] = {{&S37375[44], 18}, {NULL}};
const Segment SGI37534[] = {{&S37375[44], 29}, {NULL}};
const Segment SGI37535[] = {{&S37375[30], 6}, {NULL}};
const Segment SGI37536[] = {{&S37375[121], 3}, {NULL}};
const Segment SGI37537[] = {{&S37375[87], 5}, {NULL}};
const Segment SGI37538[] = {{&S39185[44], 13}, {NULL}};
const Segment SGI37539[] = {{&S39185[44], 24}, {NULL}};
const Segment SGI37540[] = {{&S39185[30], 6}, {NULL}};
const Segment SGI37541[] = {{&S39185[116], 3}, {NULL}};
const Segment SGI37542[] = {{&S39185[82], 5}, {NULL}};
const Segment SGI37543[] = {{&S37470[44], 13}, {NULL}};
const Segment SGI37544[] = {{&S37470[44], 24}, {NULL}};
const Segment SGI37545[] = {{&S37470[30], 6}, {NULL}};
const Segment SGI37546[] = {{&S37470[116], 3}, {NULL}};
const Segment SGI37547[] = {{&S37470[82], 5}, {NULL}};
const Segment SGI37548[] = {{&S37473[44], 13}, {NULL}};
const Segment SGI37549[] = {{&S37473[44], 24}, {NULL}};
const Segment SGI37550[] = {{&S37473[30], 6}, {NULL}};
const Segment SGI37551[] = {{&S37473[116], 3}, {NULL}};
const Segment SGI37552[] = {{&S37473[82], 5}, {NULL}};
const Segment SGI37553[] = {{&S37478[44], 12}, {NULL}};
const Segment SGI37554[] = {{&S37478[44], 23}, {NULL}};
const Segment SGI37555[] = {{&S37478[30], 6}, {NULL}};
const Segment SGI37556[] = {{&S37478[115], 3}, {NULL}};
const Segment SGI37557[] = {{&S37478[81], 5}, {NULL}};
const Segment SGI37558[] = {{&S37479[43], 13}, {NULL}};
const Segment SGI37559[] = {{&S37479[43], 24}, {NULL}};
const Segment SGI37560[] = {{&S37479[30], 5}, {NULL}};
const Segment SGI37561[] = {{&S37479[115], 3}, {NULL}};
const Segment SGI37562[] = {{&S37479[81], 5}, {NULL}};
const Segment SGI37563[] = {{&S37480[44], 12}, {NULL}};
const Segment SGI37564[] = {{&S37480[44], 23}, {NULL}};
const Segment SGI37565[] = {{&S37480[30], 6}, {NULL}};
const Segment SGI37566[] = {{&S37480[115], 3}, {NULL}};
const Segment SGI37567[] = {{&S37480[81], 5}, {NULL}};
const Segment SGI37568[] = {{&S35061[44], 9}, {NULL}};
const Segment SGI37569[] = {{&S35061[44], 20}, {NULL}};
const Segment SGI37570[] = {{&S35061[30], 6}, {NULL}};
const Segment SGI37571[] = {{&S35061[112], 3}, {NULL}};
const Segment SGI37572[] = {{&S35061[78], 5}, {NULL}};
const Segment SGI37573[] = {{&S37483[43], 13}, {NULL}};
const Segment SGI37574[] = {{&S37483[43], 24}, {NULL}};
const Segment SGI37575[] = {{&S37483[115], 3}, {NULL}};
const Segment SGI37576[] = {{&S37483[81], 5}, {NULL}};
const Segment SGI37577[] = {{&S37485[38], 13}, {NULL}};
const Segment SGI37578[] = {{&S37485[31], 6}, {NULL}};
const Segment SGI37579[] = {{&S35077[44], 9}, {NULL}};
const Segment SGI37580[] = {{&S35077[44], 20}, {NULL}};
const Segment SGI37581[] = {{&S35077[30], 6}, {NULL}};
const Segment SGI37582[] = {{&S35077[112], 3}, {NULL}};
const Segment SGI37583[] = {{&S35077[78], 5}, {NULL}};
const Segment SGI37584[] = {{&S35084[44], 9}, {NULL}};
const Segment SGI37585[] = {{&S35084[44], 22}, {NULL}};
const Segment SGI37586[] = {{&S35084[30], 6}, {NULL}};
const Segment SGI37587[] = {{&S35084[114], 3}, {NULL}};
const Segment SGI37588[] = {{&S35084[80], 5}, {NULL}};
const Segment SGI37589[] = {{&S35093[44], 10}, {NULL}};
const Segment SGI37590[] = {{&S35093[44], 21}, {NULL}};
const Segment SGI37591[] = {{&S35093[30], 6}, {NULL}};
const Segment SGI37592[] = {{&S35093[113], 3}, {NULL}};
const Segment SGI37593[] = {{&S35093[79], 5}, {NULL}};
const Segment SGI37594[] = {{&S35094[44], 13}, {NULL}};
const Segment SGI37595[] = {{&S35094[44], 24}, {NULL}};
const Segment SGI37596[] = {{&S35094[30], 6}, {NULL}};
const Segment SGI37597[] = {{&S35094[116], 3}, {NULL}};
const Segment SGI37598[] = {{&S35094[82], 5}, {NULL}};
const Segment SGI37599[] = {{&S35096[44], 13}, {NULL}};
const Segment SGI37600[] = {{&S35096[44], 24}, {NULL}};
const Segment SGI37601[] = {{&S35096[30], 6}, {NULL}};
const Segment SGI37602[] = {{&S35096[116], 3}, {NULL}};
const Segment SGI37603[] = {{&S35096[82], 5}, {NULL}};
const Segment SGI37604[] = {{&S35099[44], 13}, {NULL}};
const Segment SGI37605[] = {{&S35099[44], 24}, {NULL}};
const Segment SGI37606[] = {{&S35099[30], 6}, {NULL}};
const Segment SGI37607[] = {{&S35099[116], 3}, {NULL}};
const Segment SGI37608[] = {{&S35099[82], 5}, {NULL}};
const Segment SGI37609[] = {{&S35103[44], 13}, {NULL}};
const Segment SGI37610[] = {{&S35103[44], 25}, {NULL}};
const Segment SGI37611[] = {{&S35103[30], 6}, {NULL}};
const Segment SGI37612[] = {{&S35103[117], 3}, {NULL}};
const Segment SGI37613[] = {{&S35103[83], 5}, {NULL}};
const Segment SGI37614[] = {{&S35104[44], 10}, {NULL}};
const Segment SGI37615[] = {{&S35104[44], 22}, {NULL}};
const Segment SGI37616[] = {{&S35104[30], 6}, {NULL}};
const Segment SGI37617[] = {{&S35104[114], 3}, {NULL}};
const Segment SGI37618[] = {{&S35104[80], 5}, {NULL}};
const Segment SGI37619[] = {{&S35107[44], 17}, {NULL}};
const Segment SGI37620[] = {{&S35107[44], 29}, {NULL}};
const Segment SGI37621[] = {{&S35107[30], 6}, {NULL}};
const Segment SGI37622[] = {{&S35107[121], 3}, {NULL}};
const Segment SGI37623[] = {{&S35107[87], 5}, {NULL}};
const Segment SGI37624[] = {{&S35110[44], 17}, {NULL}};
const Segment SGI37625[] = {{&S35110[44], 28}, {NULL}};
const Segment SGI37626[] = {{&S35110[30], 6}, {NULL}};
const Segment SGI37627[] = {{&S35110[120], 3}, {NULL}};
const Segment SGI37628[] = {{&S35110[86], 5}, {NULL}};
const Segment SGI37629[] = {{&S35120[44], 16}, {NULL}};
const Segment SGI37630[] = {{&S35120[44], 28}, {NULL}};
const Segment SGI37631[] = {{&S35120[30], 6}, {NULL}};
const Segment SGI37632[] = {{&S35120[120], 3}, {NULL}};
const Segment SGI37633[] = {{&S35120[86], 5}, {NULL}};
const Segment SGI37634[] = {{&S47132[44], 13}, {NULL}};
const Segment SGI37635[] = {{&S47132[44], 25}, {NULL}};
const Segment SGI37636[] = {{&S47132[30], 6}, {NULL}};
const Segment SGI37637[] = {{&S47132[117], 3}, {NULL}};
const Segment SGI37638[] = {{&S47132[83], 5}, {NULL}};
const Segment SGI37639[] = {{&S47133[44], 13}, {NULL}};
const Segment SGI37640[] = {{&S47133[44], 23}, {NULL}};
const Segment SGI37641[] = {{&S47133[30], 6}, {NULL}};
const Segment SGI37642[] = {{&S47133[115], 3}, {NULL}};
const Segment SGI37643[] = {{&S47133[81], 5}, {NULL}};
const Segment SGI37644[] = {{&S47134[44], 13}, {NULL}};
const Segment SGI37645[] = {{&S47134[44], 21}, {NULL}};
const Segment SGI37646[] = {{&S47134[30], 6}, {NULL}};
const Segment SGI37647[] = {{&S47134[113], 3}, {NULL}};
const Segment SGI37648[] = {{&S47134[79], 5}, {NULL}};
const Segment SGI37649[] = {{&S47136[38], 13}, {NULL}};
const Segment SGI37650[] = {{&S47136[31], 6}, {NULL}};
const Segment SGI37651[] = {{&S35512[44], 18}, {NULL}};
const Segment SGI37652[] = {{&S35512[30], 6}, {NULL}};
const Segment SGI37653[] = {{&S35512[110], 3}, {NULL}};
const Segment SGI37654[] = {{&S35512[76], 5}, {NULL}};
const Segment SGI37655[] = {{&S35513[44], 17}, {NULL}};
const Segment SGI37656[] = {{&S35513[30], 6}, {NULL}};
const Segment SGI37657[] = {{&S35513[109], 3}, {NULL}};
const Segment SGI37658[] = {{&S35513[75], 5}, {NULL}};
const Segment SGI37659[] = {{&S35514[44], 14}, {NULL}};
const Segment SGI37660[] = {{&S35514[30], 6}, {NULL}};
const Segment SGI37661[] = {{&S35514[106], 3}, {NULL}};
const Segment SGI37662[] = {{&S35514[72], 5}, {NULL}};
const Segment SGI37663[] = {{&S35515[44], 14}, {NULL}};
const Segment SGI37664[] = {{&S35515[30], 6}, {NULL}};
const Segment SGI37665[] = {{&S35515[106], 3}, {NULL}};
const Segment SGI37666[] = {{&S35515[72], 5}, {NULL}};
const Segment SGI37667[] = {{&S35516[44], 17}, {NULL}};
const Segment SGI37668[] = {{&S35516[30], 6}, {NULL}};
const Segment SGI37669[] = {{&S35516[109], 3}, {NULL}};
const Segment SGI37670[] = {{&S35516[75], 5}, {NULL}};
const Segment SGI37671[] = {{&S35517[44], 18}, {NULL}};
const Segment SGI37672[] = {{&S35517[30], 6}, {NULL}};
const Segment SGI37673[] = {{&S35517[110], 3}, {NULL}};
const Segment SGI37674[] = {{&S35517[76], 5}, {NULL}};
const Segment SGI37675[] = {{&S35520[44], 7}, {NULL}};
const Segment SGI37676[] = {{&S35520[44], 22}, {NULL}};
const Segment SGI37677[] = {{&S35520[30], 6}, {NULL}};
const Segment SGI37678[] = {{&S35520[114], 3}, {NULL}};
const Segment SGI37679[] = {{&S35520[80], 5}, {NULL}};
const Segment SGI37680[] = {{&S35521[44], 7}, {NULL}};
const Segment SGI37681[] = {{&S35521[44], 22}, {NULL}};
const Segment SGI37682[] = {{&S35521[30], 6}, {NULL}};
const Segment SGI37683[] = {{&S35521[114], 3}, {NULL}};
const Segment SGI37684[] = {{&S35521[80], 5}, {NULL}};
const Segment SGI37685[] = {{&S35522[44], 7}, {NULL}};
const Segment SGI37686[] = {{&S35522[44], 22}, {NULL}};
const Segment SGI37687[] = {{&S35522[30], 6}, {NULL}};
const Segment SGI37688[] = {{&S35522[114], 3}, {NULL}};
const Segment SGI37689[] = {{&S35522[80], 5}, {NULL}};
const Segment SGI37690[] = {{&S35573[44], 10}, {NULL}};
const Segment SGI37691[] = {{&S35573[44], 27}, {NULL}};
const Segment SGI37692[] = {{&S35573[30], 6}, {NULL}};
const Segment SGI37693[] = {{&S35573[119], 3}, {NULL}};
const Segment SGI37694[] = {{&S35573[85], 5}, {NULL}};
const Segment SGI37695[] = {{&S35580[38], 10}, {NULL}};
const Segment SGI37696[] = {{&S35580[31], 6}, {NULL}};
const Segment SGI37697[] = {{&S35298[44], 15}, {NULL}};
const Segment SGI37698[] = {{&S35298[44], 35}, {NULL}};
const Segment SGI37699[] = {{&S35298[30], 6}, {NULL}};
const Segment SGI37700[] = {{&S35298[127], 3}, {NULL}};
const Segment SGI37701[] = {{&S35298[93], 5}, {NULL}};
const Segment SGI37702[] = {{&S35299[44], 15}, {NULL}};
const Segment SGI37703[] = {{&S35299[44], 36}, {NULL}};
const Segment SGI37704[] = {{&S35299[30], 6}, {NULL}};
const Segment SGI37705[] = {{&S35299[128], 3}, {NULL}};
const Segment SGI37706[] = {{&S35299[94], 5}, {NULL}};
const Segment SGI37707[] = {{&S35300[44], 12}, {NULL}};
const Segment SGI37708[] = {{&S35300[44], 33}, {NULL}};
const Segment SGI37709[] = {{&S35300[30], 6}, {NULL}};
const Segment SGI37710[] = {{&S35300[125], 3}, {NULL}};
const Segment SGI37711[] = {{&S35300[91], 5}, {NULL}};
const Segment SGI37712[] = {{&S35301[44], 15}, {NULL}};
const Segment SGI37713[] = {{&S35301[44], 41}, {NULL}};
const Segment SGI37714[] = {{&S35301[30], 6}, {NULL}};
const Segment SGI37715[] = {{&S35301[133], 3}, {NULL}};
const Segment SGI37716[] = {{&S35301[99], 5}, {NULL}};
const Segment SGI37717[] = {{&S35302[38], 12}, {NULL}};
const Segment SGI37718[] = {{&S35302[31], 6}, {NULL}};
const Segment SGI37719[] = {{&S35303[44], 15}, {NULL}};
const Segment SGI37720[] = {{&S35303[44], 42}, {NULL}};
const Segment SGI37721[] = {{&S35303[30], 6}, {NULL}};
const Segment SGI37722[] = {{&S35303[134], 3}, {NULL}};
const Segment SGI37723[] = {{&S35303[100], 5}, {NULL}};
const Segment SGI37724[] = {{&S35307[44], 15}, {NULL}};
const Segment SGI37725[] = {{&S35307[44], 41}, {NULL}};
const Segment SGI37726[] = {{&S35307[30], 6}, {NULL}};
const Segment SGI37727[] = {{&S35307[133], 3}, {NULL}};
const Segment SGI37728[] = {{&S35307[99], 5}, {NULL}};
const Segment SGI37729[] = {{&S35308[44], 15}, {NULL}};
const Segment SGI37730[] = {{&S35308[44], 41}, {NULL}};
const Segment SGI37731[] = {{&S35308[30], 6}, {NULL}};
const Segment SGI37732[] = {{&S35308[133], 3}, {NULL}};
const Segment SGI37733[] = {{&S35308[99], 5}, {NULL}};
const Segment SGI37734[] = {{&S35311[44], 15}, {NULL}};
const Segment SGI37735[] = {{&S35311[44], 41}, {NULL}};
const Segment SGI37736[] = {{&S35311[30], 6}, {NULL}};
const Segment SGI37737[] = {{&S35311[133], 3}, {NULL}};
const Segment SGI37738[] = {{&S35311[99], 5}, {NULL}};
const Segment SGI37739[] = {{&S35314[44], 12}, {NULL}};
const Segment SGI37740[] = {{&S35314[44], 38}, {NULL}};
const Segment SGI37741[] = {{&S35314[30], 6}, {NULL}};
const Segment SGI37742[] = {{&S35314[130], 3}, {NULL}};
const Segment SGI37743[] = {{&S35314[96], 5}, {NULL}};
const Segment SGI37744[] = {{&S35315[44], 12}, {NULL}};
const Segment SGI37745[] = {{&S35315[44], 38}, {NULL}};
const Segment SGI37746[] = {{&S35315[30], 6}, {NULL}};
const Segment SGI37747[] = {{&S35315[130], 3}, {NULL}};
const Segment SGI37748[] = {{&S35315[96], 5}, {NULL}};
const Segment SGI37749[] = {{&S35317[44], 15}, {NULL}};
const Segment SGI37750[] = {{&S35317[44], 41}, {NULL}};
const Segment SGI37751[] = {{&S35317[30], 6}, {NULL}};
const Segment SGI37752[] = {{&S35317[133], 3}, {NULL}};
const Segment SGI37753[] = {{&S35317[99], 5}, {NULL}};
const Segment SGI37754[] = {{&S35318[44], 15}, {NULL}};
const Segment SGI37755[] = {{&S35318[44], 41}, {NULL}};
const Segment SGI37756[] = {{&S35318[30], 6}, {NULL}};
const Segment SGI37757[] = {{&S35318[133], 3}, {NULL}};
const Segment SGI37758[] = {{&S35318[99], 5}, {NULL}};
const Segment SGI37759[] = {{&S35319[44], 15}, {NULL}};
const Segment SGI37760[] = {{&S35319[44], 36}, {NULL}};
const Segment SGI37761[] = {{&S35319[30], 6}, {NULL}};
const Segment SGI37762[] = {{&S35319[128], 3}, {NULL}};
const Segment SGI37763[] = {{&S35319[94], 5}, {NULL}};
const Segment SGI37764[] = {{&S35320[44], 15}, {NULL}};
const Segment SGI37765[] = {{&S35320[44], 42}, {NULL}};
const Segment SGI37766[] = {{&S35320[30], 6}, {NULL}};
const Segment SGI37767[] = {{&S35320[134], 3}, {NULL}};
const Segment SGI37768[] = {{&S35320[100], 5}, {NULL}};
const Segment SGI37769[] = {{&S35324[44], 12}, {NULL}};
const Segment SGI37770[] = {{&S35324[44], 38}, {NULL}};
const Segment SGI37771[] = {{&S35324[30], 6}, {NULL}};
const Segment SGI37772[] = {{&S35324[130], 3}, {NULL}};
const Segment SGI37773[] = {{&S35324[96], 5}, {NULL}};
const Segment SGI37774[] = {{&S35326[44], 12}, {NULL}};
const Segment SGI37775[] = {{&S35326[44], 38}, {NULL}};
const Segment SGI37776[] = {{&S35326[30], 6}, {NULL}};
const Segment SGI37777[] = {{&S35326[130], 3}, {NULL}};
const Segment SGI37778[] = {{&S35326[96], 5}, {NULL}};
const Segment SGI37779[] = {{&S35327[44], 15}, {NULL}};
const Segment SGI37780[] = {{&S35327[44], 41}, {NULL}};
const Segment SGI37781[] = {{&S35327[30], 6}, {NULL}};
const Segment SGI37782[] = {{&S35327[133], 3}, {NULL}};
const Segment SGI37783[] = {{&S35327[99], 5}, {NULL}};
const Segment SGI37784[] = {{&S37644[44], 11}, {NULL}};
const Segment SGI37785[] = {{&S37644[44], 33}, {NULL}};
const Segment SGI37786[] = {{&S37644[30], 6}, {NULL}};
const Segment SGI37787[] = {{&S37644[125], 3}, {NULL}};
const Segment SGI37788[] = {{&S37644[91], 5}, {NULL}};
const Segment SGI37789[] = {{&S37645[44], 11}, {NULL}};
const Segment SGI37790[] = {{&S37645[44], 33}, {NULL}};
const Segment SGI37791[] = {{&S37645[30], 6}, {NULL}};
const Segment SGI37792[] = {{&S37645[125], 3}, {NULL}};
const Segment SGI37793[] = {{&S37645[91], 5}, {NULL}};
const Segment SGI37794[] = {{&S37646[44], 10}, {NULL}};
const Segment SGI37795[] = {{&S37646[44], 32}, {NULL}};
const Segment SGI37796[] = {{&S37646[30], 6}, {NULL}};
const Segment SGI37797[] = {{&S37646[124], 3}, {NULL}};
const Segment SGI37798[] = {{&S37646[90], 5}, {NULL}};
const Segment SGI37799[] = {{&S37647[43], 11}, {NULL}};
const Segment SGI37800[] = {{&S37647[43], 33}, {NULL}};
const Segment SGI37801[] = {{&S37647[124], 3}, {NULL}};
const Segment SGI37802[] = {{&S37647[90], 5}, {NULL}};
const Segment SGI37803[] = {{&S47372[44], 18}, {NULL}};
const Segment SGI37804[] = {{&S47372[44], 29}, {NULL}};
const Segment SGI37805[] = {{&S47372[30], 6}, {NULL}};
const Segment SGI37806[] = {{&S47372[121], 3}, {NULL}};
const Segment SGI37807[] = {{&S47372[87], 5}, {NULL}};
const Segment SGI37808[] = {{&S37648[44], 11}, {NULL}};
const Segment SGI37809[] = {{&S37648[44], 33}, {NULL}};
const Segment SGI37810[] = {{&S37648[30], 6}, {NULL}};
const Segment SGI37811[] = {{&S37648[125], 3}, {NULL}};
const Segment SGI37812[] = {{&S37648[91], 5}, {NULL}};
const Segment SGI37813[] = {{&S47373[38], 18}, {NULL}};
const Segment SGI37814[] = {{&S47373[31], 6}, {NULL}};
const Segment SGI37815[] = {{&S35716[44], 7}, {NULL}};
const Segment SGI37816[] = {{&S35716[44], 28}, {NULL}};
const Segment SGI37817[] = {{&S35716[30], 6}, {NULL}};
const Segment SGI37818[] = {{&S35716[120], 3}, {NULL}};
const Segment SGI37819[] = {{&S35716[86], 5}, {NULL}};
const Segment SGI37820[] = {{&S35721[44], 7}, {NULL}};
const Segment SGI37821[] = {{&S35721[44], 32}, {NULL}};
const Segment SGI37822[] = {{&S35721[30], 6}, {NULL}};
const Segment SGI37823[] = {{&S35721[124], 3}, {NULL}};
const Segment SGI37824[] = {{&S35721[90], 5}, {NULL}};
const Segment SGI37825[] = {{&S35342[44], 12}, {NULL}};
const Segment SGI37826[] = {{&S35342[44], 33}, {NULL}};
const Segment SGI37827[] = {{&S35342[30], 6}, {NULL}};
const Segment SGI37828[] = {{&S35342[125], 3}, {NULL}};
const Segment SGI37829[] = {{&S35342[91], 5}, {NULL}};
const Segment SGI37830[] = {{&S35722[44], 7}, {NULL}};
const Segment SGI37831[] = {{&S35722[44], 24}, {NULL}};
const Segment SGI37832[] = {{&S35722[30], 6}, {NULL}};
const Segment SGI37833[] = {{&S35722[116], 3}, {NULL}};
const Segment SGI37834[] = {{&S35722[82], 5}, {NULL}};
const Segment SGI37835[] = {{&S35728[44], 15}, {NULL}};
const Segment SGI37836[] = {{&S35728[44], 32}, {NULL}};
const Segment SGI37837[] = {{&S35728[30], 6}, {NULL}};
const Segment SGI37838[] = {{&S35728[124], 3}, {NULL}};
const Segment SGI37839[] = {{&S35728[90], 5}, {NULL}};
const Segment SGI37840[] = {{&S35732[44], 9}, {NULL}};
const Segment SGI37841[] = {{&S35732[44], 26}, {NULL}};
const Segment SGI37842[] = {{&S35732[30], 6}, {NULL}};
const Segment SGI37843[] = {{&S35732[118], 3}, {NULL}};
const Segment SGI37844[] = {{&S35732[84], 5}, {NULL}};
const Segment SGI37845[] = {{&S35733[38], 7}, {NULL}};
const Segment SGI37846[] = {{&S35733[31], 6}, {NULL}};
const Segment SGI37847[] = {{&S37650[38], 11}, {NULL}};
const Segment SGI37848[] = {{&S37650[31], 6}, {NULL}};
const Segment SGI37849[] = {{&S37653[44], 10}, {NULL}};
const Segment SGI37850[] = {{&S37653[44], 32}, {NULL}};
const Segment SGI37851[] = {{&S37653[30], 6}, {NULL}};
const Segment SGI37852[] = {{&S37653[124], 3}, {NULL}};
const Segment SGI37853[] = {{&S37653[90], 5}, {NULL}};
const Segment SGI37854[] = {{&S37654[44], 10}, {NULL}};
const Segment SGI37855[] = {{&S37654[44], 32}, {NULL}};
const Segment SGI37856[] = {{&S37654[30], 6}, {NULL}};
const Segment SGI37857[] = {{&S37654[124], 3}, {NULL}};
const Segment SGI37858[] = {{&S37654[90], 5}, {NULL}};
const Segment SGI37859[] = {{&S37655[44], 11}, {NULL}};
const Segment SGI37860[] = {{&S37655[44], 33}, {NULL}};
const Segment SGI37861[] = {{&S37655[30], 6}, {NULL}};
const Segment SGI37862[] = {{&S37655[125], 3}, {NULL}};
const Segment SGI37863[] = {{&S37655[91], 5}, {NULL}};
const Segment SGI37864[] = {{&S37660[43], 11}, {NULL}};
const Segment SGI37865[] = {{&S37660[43], 33}, {NULL}};
const Segment SGI37866[] = {{&S37660[30], 5}, {NULL}};
const Segment SGI37867[] = {{&S37660[124], 3}, {NULL}};
const Segment SGI37868[] = {{&S37660[90], 5}, {NULL}};
const Segment SGI37869[] = {{&S37665[44], 19}, {NULL}};
const Segment SGI37870[] = {{&S37665[44], 38}, {NULL}};
const Segment SGI37871[] = {{&S37665[30], 6}, {NULL}};
const Segment SGI37872[] = {{&S37665[130], 3}, {NULL}};
const Segment SGI37873[] = {{&S37665[96], 5}, {NULL}};
const Segment SGI37874[] = {{&S37666[44], 19}, {NULL}};
const Segment SGI37875[] = {{&S37666[44], 38}, {NULL}};
const Segment SGI37876[] = {{&S37666[30], 6}, {NULL}};
const Segment SGI37877[] = {{&S37666[130], 3}, {NULL}};
const Segment SGI37878[] = {{&S37666[96], 5}, {NULL}};
const Segment SGI37879[] = {{&S37667[44], 7}, {NULL}};
const Segment SGI37880[] = {{&S37667[44], 26}, {NULL}};
const Segment SGI37881[] = {{&S37667[30], 6}, {NULL}};
const Segment SGI37882[] = {{&S37667[118], 3}, {NULL}};
const Segment SGI37883[] = {{&S37667[84], 5}, {NULL}};
const Segment SGI37884[] = {{&S37668[44], 19}, {NULL}};
const Segment SGI37885[] = {{&S37668[44], 36}, {NULL}};
const Segment SGI37886[] = {{&S37668[30], 6}, {NULL}};
const Segment SGI37887[] = {{&S37668[69], 3}, {NULL}};
const Segment SGI37888[] = {{&S37668[94], 5}, {NULL}};
const Segment SGI37889[] = {{&S37671[44], 19}, {NULL}};
const Segment SGI37890[] = {{&S37671[44], 38}, {NULL}};
const Segment SGI37891[] = {{&S37671[30], 6}, {NULL}};
const Segment SGI37892[] = {{&S37671[130], 3}, {NULL}};
const Segment SGI37893[] = {{&S37671[96], 5}, {NULL}};
const Segment SGI37894[] = {{&S37673[44], 19}, {NULL}};
const Segment SGI37895[] = {{&S37673[44], 36}, {NULL}};
const Segment SGI37896[] = {{&S37673[30], 6}, {NULL}};
const Segment SGI37897[] = {{&S37673[128], 3}, {NULL}};
const Segment SGI37898[] = {{&S37673[94], 5}, {NULL}};
const Segment SGI37899[] = {{&S37678[38], 7}, {NULL}};
const Segment SGI37900[] = {{&S37678[31], 6}, {NULL}};
const Segment SGI37901[] = {{&S37680[44], 19}, {NULL}};
const Segment SGI37902[] = {{&S37680[44], 38}, {NULL}};
const Segment SGI37903[] = {{&S37680[30], 6}, {NULL}};
const Segment SGI37904[] = {{&S37680[130], 3}, {NULL}};
const Segment SGI37905[] = {{&S37680[96], 5}, {NULL}};
const Segment SGI37906[] = {{&S37681[44], 7}, {NULL}};
const Segment SGI37907[] = {{&S37681[44], 24}, {NULL}};
const Segment SGI37908[] = {{&S37681[30], 6}, {NULL}};
const Segment SGI37909[] = {{&S37681[57], 3}, {NULL}};
const Segment SGI37910[] = {{&S37681[82], 5}, {NULL}};
const Segment SGI37911[] = {{&S35448[44], 8}, {NULL}};
const Segment SGI37912[] = {{&S35448[44], 25}, {NULL}};
const Segment SGI37913[] = {{&S35448[30], 6}, {NULL}};
const Segment SGI37914[] = {{&S35448[117], 3}, {NULL}};
const Segment SGI37915[] = {{&S35448[83], 5}, {NULL}};
const Segment SGI37916[] = {{&S35449[44], 8}, {NULL}};
const Segment SGI37917[] = {{&S35449[44], 31}, {NULL}};
const Segment SGI37918[] = {{&S35449[30], 6}, {NULL}};
const Segment SGI37919[] = {{&S35450[38], 8}, {NULL}};
const Segment SGI37920[] = {{&S35450[31], 6}, {NULL}};
const Segment SGI37921[] = {{&S35453[42], 8}, {NULL}};
const Segment SGI37922[] = {{&S35453[30], 6}, {NULL}};
const Segment SGI37923[] = {{&S35453[115], 3}, {NULL}};
const Segment SGI37924[] = {{&S35453[81], 5}, {NULL}};
const Segment SGI37925[] = {{&S47526[44], 12}, {NULL}};
const Segment SGI37926[] = {{&S47526[44], 24}, {NULL}};
const Segment SGI37927[] = {{&S47526[30], 6}, {NULL}};
const Segment SGI37928[] = {{&S47526[116], 3}, {NULL}};
const Segment SGI37929[] = {{&S47526[82], 5}, {NULL}};
const Segment SGI37930[] = {{&S47527[44], 12}, {NULL}};
const Segment SGI37931[] = {{&S47527[44], 24}, {NULL}};
const Segment SGI37932[] = {{&S47527[30], 6}, {NULL}};
const Segment SGI37933[] = {{&S47527[116], 3}, {NULL}};
const Segment SGI37934[] = {{&S47527[82], 5}, {NULL}};
const Segment SGI37935[] = {{&S47528[38], 12}, {NULL}};
const Segment SGI37936[] = {{&S47528[31], 6}, {NULL}};
const Segment SGI37937[] = {{&S47546[44], 10}, {NULL}};
const Segment SGI37938[] = {{&S47546[44], 21}, {NULL}};
const Segment SGI37939[] = {{&S47546[30], 6}, {NULL}};
const Segment SGI37940[] = {{&S47546[113], 3}, {NULL}};
const Segment SGI37941[] = {{&S47546[79], 5}, {NULL}};
const Segment SGI37942[] = {{&S35519[44], 7}, {NULL}};
const Segment SGI37943[] = {{&S35519[44], 31}, {NULL}};
const Segment SGI37944[] = {{&S35519[30], 6}, {NULL}};
const Segment SGI37945[] = {{&S35519[123], 3}, {NULL}};
const Segment SGI37946[] = {{&S35519[89], 5}, {NULL}};
const Segment SGI37947[] = {{&S35523[44], 7}, {NULL}};
const Segment SGI37948[] = {{&S35523[44], 32}, {NULL}};
const Segment SGI37949[] = {{&S35523[30], 6}, {NULL}};
const Segment SGI37950[] = {{&S35523[124], 3}, {NULL}};
const Segment SGI37951[] = {{&S35523[90], 5}, {NULL}};
const Segment SGI37952[] = {{&S35524[44], 7}, {NULL}};
const Segment SGI37953[] = {{&S35524[44], 32}, {NULL}};
const Segment SGI37954[] = {{&S35524[30], 6}, {NULL}};
const Segment SGI37955[] = {{&S35524[124], 3}, {NULL}};
const Segment SGI37956[] = {{&S35524[90], 5}, {NULL}};
const Segment SGI37957[] = {{&S35525[44], 7}, {NULL}};
const Segment SGI37958[] = {{&S35525[44], 32}, {NULL}};
const Segment SGI37959[] = {{&S35525[30], 6}, {NULL}};
const Segment SGI37960[] = {{&S35525[124], 3}, {NULL}};
const Segment SGI37961[] = {{&S35525[90], 5}, {NULL}};
const Segment SGI37962[] = {{&S35526[44], 7}, {NULL}};
const Segment SGI37963[] = {{&S35526[44], 31}, {NULL}};
const Segment SGI37964[] = {{&S35526[30], 6}, {NULL}};
const Segment SGI37965[] = {{&S35526[123], 3}, {NULL}};
const Segment SGI37966[] = {{&S35526[89], 5}, {NULL}};
const Segment SGI37967[] = {{&S35527[44], 7}, {NULL}};
const Segment SGI37968[] = {{&S35527[44], 32}, {NULL}};
const Segment SGI37969[] = {{&S35527[30], 6}, {NULL}};
const Segment SGI37970[] = {{&S35527[124], 3}, {NULL}};
const Segment SGI37971[] = {{&S35527[90], 5}, {NULL}};
const Segment SGI37972[] = {{&S35528[44], 7}, {NULL}};
const Segment SGI37973[] = {{&S35528[44], 30}, {NULL}};
const Segment SGI37974[] = {{&S35528[30], 6}, {NULL}};
const Segment SGI37975[] = {{&S35528[122], 3}, {NULL}};
const Segment SGI37976[] = {{&S35528[88], 5}, {NULL}};
const Segment SGI37977[] = {{&S35534[38], 7}, {NULL}};
const Segment SGI37978[] = {{&S35534[31], 6}, {NULL}};
const Segment SGI37979[] = {{&S47596[44], 10}, {NULL}};
const Segment SGI37980[] = {{&S47596[44], 21}, {NULL}};
const Segment SGI37981[] = {{&S47596[30], 6}, {NULL}};
const Segment SGI37982[] = {{&S47596[113], 3}, {NULL}};
const Segment SGI37983[] = {{&S47596[79], 5}, {NULL}};
const Segment SGI37984[] = {{&S47598[38], 10}, {NULL}};
const Segment SGI37985[] = {{&S47598[31], 6}, {NULL}};
const Segment SGI37986[] = {{&S35546[44], 7}, {NULL}};
const Segment SGI37987[] = {{&S35546[44], 30}, {NULL}};
const Segment SGI37988[] = {{&S35546[30], 6}, {NULL}};
const Segment SGI37989[] = {{&S35546[122], 3}, {NULL}};
const Segment SGI37990[] = {{&S35546[88], 5}, {NULL}};
const Segment SGI37991[] = {{&S35547[44], 7}, {NULL}};
const Segment SGI37992[] = {{&S35547[44], 35}, {NULL}};
const Segment SGI37993[] = {{&S35547[30], 6}, {NULL}};
const Segment SGI37994[] = {{&S35547[127], 3}, {NULL}};
const Segment SGI37995[] = {{&S35547[93], 5}, {NULL}};
const Segment SGI37996[] = {{&S35575[38], 10}, {NULL}};
const Segment SGI37997[] = {{&S35575[31], 6}, {NULL}};
const Segment SGI37998[] = {{&S35582[44], 10}, {NULL}};
const Segment SGI37999[] = {{&S35582[44], 27}, {NULL}};
const Segment SGI38000[] = {{&S35582[30], 6}, {NULL}};
const Segment SGI38001[] = {{&S35582[119], 3}, {NULL}};
const Segment SGI38002[] = {{&S35582[85], 5}, {NULL}};
const Segment SGI38003[] = {{&S35806[44], 23}, {NULL}};
const Segment SGI38004[] = {{&S35806[44], 34}, {NULL}};
const Segment SGI38005[] = {{&S35806[30], 6}, {NULL}};
const Segment SGI38006[] = {{&S35806[126], 3}, {NULL}};
const Segment SGI38007[] = {{&S35806[92], 5}, {NULL}};
const Segment SGI38008[] = {{&S35807[44], 23}, {NULL}};
const Segment SGI38009[] = {{&S35807[44], 34}, {NULL}};
const Segment SGI38010[] = {{&S35807[30], 6}, {NULL}};
const Segment SGI38011[] = {{&S35807[126], 3}, {NULL}};
const Segment SGI38012[] = {{&S35807[92], 5}, {NULL}};
const Segment SGI38013[] = {{&S35808[44], 23}, {NULL}};
const Segment SGI38014[] = {{&S35808[44], 34}, {NULL}};
const Segment SGI38015[] = {{&S35808[30], 6}, {NULL}};
const Segment SGI38016[] = {{&S35808[126], 3}, {NULL}};
const Segment SGI38017[] = {{&S35808[92], 5}, {NULL}};
const Segment SGI38018[] = {{&S35810[44], 9}, {NULL}};
const Segment SGI38019[] = {{&S35810[44], 20}, {NULL}};
const Segment SGI38020[] = {{&S35810[30], 6}, {NULL}};
const Segment SGI38021[] = {{&S35810[112], 3}, {NULL}};
const Segment SGI38022[] = {{&S35810[78], 5}, {NULL}};
const Segment SGI38023[] = {{&S35811[44], 18}, {NULL}};
const Segment SGI38024[] = {{&S35811[44], 29}, {NULL}};
const Segment SGI38025[] = {{&S35811[30], 6}, {NULL}};
const Segment SGI38026[] = {{&S35811[121], 3}, {NULL}};
const Segment SGI38027[] = {{&S35811[87], 5}, {NULL}};
const Segment SGI38028[] = {{&S35813[38], 9}, {NULL}};
const Segment SGI38029[] = {{&S35813[31], 6}, {NULL}};
const Segment SGI38030[] = {{&S35815[44], 9}, {NULL}};
const Segment SGI38031[] = {{&S35815[44], 20}, {NULL}};
const Segment SGI38032[] = {{&S35815[30], 6}, {NULL}};
const Segment SGI38033[] = {{&S35816[44], 23}, {NULL}};
const Segment SGI38034[] = {{&S35816[44], 34}, {NULL}};
const Segment SGI38035[] = {{&S35816[30], 6}, {NULL}};
const Segment SGI38036[] = {{&S35816[126], 3}, {NULL}};
const Segment SGI38037[] = {{&S35816[92], 5}, {NULL}};
const Segment SGI38038[] = {{&S35817[44], 9}, {NULL}};
const Segment SGI38039[] = {{&S35817[44], 20}, {NULL}};
const Segment SGI38040[] = {{&S35817[30], 6}, {NULL}};
const Segment SGI38041[] = {{&S35817[112], 3}, {NULL}};
const Segment SGI38042[] = {{&S35817[78], 5}, {NULL}};
const Segment SGI38043[] = {{&S35818[44], 9}, {NULL}};
const Segment SGI38044[] = {{&S35818[44], 20}, {NULL}};
const Segment SGI38045[] = {{&S35818[30], 6}, {NULL}};
const Segment SGI38046[] = {{&S35818[78], 6}, {NULL}};
const Segment SGI38047[] = {{&S37978[44], 19}, {NULL}};
const Segment SGI38048[] = {{&S37978[44], 36}, {NULL}};
const Segment SGI38049[] = {{&S37978[30], 6}, {NULL}};
const Segment SGI38050[] = {{&S37978[69], 3}, {NULL}};
const Segment SGI38051[] = {{&S37978[94], 5}, {NULL}};
const Segment SGI38052[] = {{&S35820[44], 23}, {NULL}};
const Segment SGI38053[] = {{&S35820[44], 34}, {NULL}};
const Segment SGI38054[] = {{&S35820[30], 6}, {NULL}};
const Segment SGI38055[] = {{&S35820[126], 3}, {NULL}};
const Segment SGI38056[] = {{&S35820[92], 5}, {NULL}};
const Segment SGI38057[] = {{&S37980[44], 19}, {NULL}};
const Segment SGI38058[] = {{&S37980[44], 36}, {NULL}};
const Segment SGI38059[] = {{&S37980[30], 6}, {NULL}};
const Segment SGI38060[] = {{&S37980[69], 3}, {NULL}};
const Segment SGI38061[] = {{&S37980[94], 5}, {NULL}};
const Segment SGI38062[] = {{&S35822[44], 23}, {NULL}};
const Segment SGI38063[] = {{&S35822[44], 34}, {NULL}};
const Segment SGI38064[] = {{&S35822[30], 6}, {NULL}};
const Segment SGI38065[] = {{&S35822[126], 3}, {NULL}};
const Segment SGI38066[] = {{&S35822[92], 5}, {NULL}};
const Segment SGI38067[] = {{&S37982[44], 19}, {NULL}};
const Segment SGI38068[] = {{&S37982[44], 36}, {NULL}};
const Segment SGI38069[] = {{&S37982[30], 6}, {NULL}};
const Segment SGI38070[] = {{&S37982[69], 3}, {NULL}};
const Segment SGI38071[] = {{&S35823[44], 23}, {NULL}};
const Segment SGI38072[] = {{&S35823[44], 34}, {NULL}};
const Segment SGI38073[] = {{&S35823[30], 6}, {NULL}};
const Segment SGI38074[] = {{&S37982[94], 5}, {NULL}};
const Segment SGI38075[] = {{&S35823[126], 3}, {NULL}};
const Segment SGI38076[] = {{&S35823[92], 5}, {NULL}};
const Segment SGI38077[] = {{&S37983[44], 19}, {NULL}};
const Segment SGI38078[] = {{&S37983[44], 36}, {NULL}};
const Segment SGI38079[] = {{&S37983[30], 6}, {NULL}};
const Segment SGI38080[] = {{&S35824[44], 9}, {NULL}};
const Segment SGI38081[] = {{&S35824[44], 20}, {NULL}};
const Segment SGI38082[] = {{&S35824[30], 6}, {NULL}};
const Segment SGI38083[] = {{&S37983[69], 3}, {NULL}};
const Segment SGI38084[] = {{&S37983[94], 5}, {NULL}};
const Segment SGI38085[] = {{&S35824[114], 3}, {NULL}};
const Segment SGI38086[] = {{&S35824[78], 7}, {NULL}};
const Segment SGI38087[] = {{&S35824[134], 8}, {NULL}};
const Segment SGI38088[] = {{&S35825[42], 9}, {NULL}};
const Segment SGI38089[] = {{&S35825[30], 6}, {NULL}};
const Segment SGI38090[] = {{&S35825[110], 3}, {NULL}};
const Segment SGI38091[] = {{&S35825[76], 5}, {NULL}};
const Segment SGI38092[] = {{&S35826[38], 9}, {NULL}};
const Segment SGI38093[] = {{&S35826[31], 6}, {NULL}};
const Segment SGI38094[] = {{&S35827[44], 23}, {NULL}};
const Segment SGI38095[] = {{&S35827[44], 34}, {NULL}};
const Segment SGI38096[] = {{&S35827[30], 6}, {NULL}};
const Segment SGI38097[] = {{&S35827[126], 3}, {NULL}};
const Segment SGI38098[] = {{&S35827[92], 5}, {NULL}};
const Segment SGI38099[] = {{&S35828[44], 11}, {NULL}};
const Segment SGI38100[] = {{&S35828[44], 22}, {NULL}};
const Segment SGI38101[] = {{&S35828[30], 6}, {NULL}};
const Segment SGI38102[] = {{&S35828[114], 3}, {NULL}};
const Segment SGI38103[] = {{&S35828[80], 5}, {NULL}};
const Segment SGI38104[] = {{&S35829[44], 9}, {NULL}};
const Segment SGI38105[] = {{&S35829[44], 20}, {NULL}};
const Segment SGI38106[] = {{&S35829[30], 6}, {NULL}};
const Segment SGI38107[] = {{&S35829[81], 6}, {NULL}};
const Segment SGI38108[] = {{&S35714[38], 20}, {NULL}};
const Segment SGI38109[] = {{&S35714[31], 6}, {NULL}};
const Segment SGI38110[] = {{&S35715[44], 20}, {NULL}};
const Segment SGI38111[] = {{&S35715[44], 37}, {NULL}};
const Segment SGI38112[] = {{&S35715[30], 6}, {NULL}};
const Segment SGI38113[] = {{&S35715[129], 3}, {NULL}};
const Segment SGI38114[] = {{&S35715[95], 5}, {NULL}};
const Segment SGI38115[] = {{&S47777[44], 8}, {NULL}};
const Segment SGI38116[] = {{&S47777[44], 25}, {NULL}};
const Segment SGI38117[] = {{&S35717[44], 7}, {NULL}};
const Segment SGI38118[] = {{&S47777[30], 6}, {NULL}};
const Segment SGI38119[] = {{&S35717[44], 24}, {NULL}};
const Segment SGI38120[] = {{&S35717[30], 6}, {NULL}};
const Segment SGI38121[] = {{&S47777[117], 3}, {NULL}};
const Segment SGI38122[] = {{&S35717[116], 3}, {NULL}};
const Segment SGI38123[] = {{&S47777[83], 5}, {NULL}};
const Segment SGI38124[] = {{&S35717[82], 5}, {NULL}};
const Segment SGI38125[] = {{&S35718[44], 7}, {NULL}};
const Segment SGI38126[] = {{&S47778[44], 7}, {NULL}};
const Segment SGI38127[] = {{&S35718[44], 24}, {NULL}};
const Segment SGI38128[] = {{&S47778[44], 24}, {NULL}};
const Segment SGI38129[] = {{&S35718[30], 6}, {NULL}};
const Segment SGI38130[] = {{&S47778[30], 6}, {NULL}};
const Segment SGI38131[] = {{&S47778[116], 3}, {NULL}};
const Segment SGI38132[] = {{&S35718[85], 4}, {NULL}};
const Segment SGI38133[] = {{&S35718[85], 3}, {NULL}};
const Segment SGI38134[] = {{&S47778[82], 5}, {NULL}};
const Segment SGI38135[] = {{&S47779[44], 11}, {NULL}};
const Segment SGI38136[] = {{&S47779[44], 28}, {NULL}};
const Segment SGI38137[] = {{&S47779[30], 6}, {NULL}};
const Segment SGI38138[] = {{&S47779[120], 3}, {NULL}};
const Segment SGI38139[] = {{&S47779[86], 5}, {NULL}};
const Segment SGI38140[] = {{&S47780[38], 7}, {NULL}};
const Segment SGI38141[] = {{&S47780[31], 6}, {NULL}};
const Segment SGI38142[] = {{&S47781[44], 16}, {NULL}};
const Segment SGI38143[] = {{&S47781[44], 33}, {NULL}};
const Segment SGI38144[] = {{&S47781[30], 6}, {NULL}};
const Segment SGI38145[] = {{&S47781[125], 3}, {NULL}};
const Segment SGI38146[] = {{&S47781[91], 5}, {NULL}};
const Segment SGI38147[] = {{&S35724[44], 13}, {NULL}};
const Segment SGI38148[] = {{&S35724[44], 30}, {NULL}};
const Segment SGI38149[] = {{&S35724[30], 6}, {NULL}};
const Segment SGI38150[] = {{&S35724[122], 3}, {NULL}};
const Segment SGI38151[] = {{&S35724[88], 5}, {NULL}};
const Segment SGI38152[] = {{&S47782[44], 10}, {NULL}};
const Segment SGI38153[] = {{&S47782[44], 27}, {NULL}};
const Segment SGI38154[] = {{&S47782[30], 6}, {NULL}};
const Segment SGI38155[] = {{&S47782[119], 3}, {NULL}};
const Segment SGI38156[] = {{&S47782[85], 5}, {NULL}};
const Segment SGI38157[] = {{&S35725[44], 7}, {NULL}};
const Segment SGI38158[] = {{&S35725[44], 24}, {NULL}};
const Segment SGI38159[] = {{&S35725[30], 6}, {NULL}};
const Segment SGI38160[] = {{&S35725[116], 3}, {NULL}};
const Segment SGI38161[] = {{&S35725[82], 5}, {NULL}};
const Segment SGI38162[] = {{&S35726[44], 7}, {NULL}};
const Segment SGI38163[] = {{&S35726[44], 24}, {NULL}};
const Segment SGI38164[] = {{&S35726[30], 6}, {NULL}};
const Segment SGI38165[] = {{&S35726[116], 3}, {NULL}};
const Segment SGI38166[] = {{&S35726[82], 5}, {NULL}};
const Segment SGI38167[] = {{&S35727[44], 20}, {NULL}};
const Segment SGI38168[] = {{&S35727[44], 37}, {NULL}};
const Segment SGI38169[] = {{&S35727[30], 6}, {NULL}};
const Segment SGI38170[] = {{&S35727[129], 3}, {NULL}};
const Segment SGI38171[] = {{&S35727[95], 5}, {NULL}};
const Segment SGI38172[] = {{&S35729[44], 9}, {NULL}};
const Segment SGI38173[] = {{&S35729[44], 26}, {NULL}};
const Segment SGI38174[] = {{&S35729[30], 6}, {NULL}};
const Segment SGI38175[] = {{&S35729[118], 3}, {NULL}};
const Segment SGI38176[] = {{&S35729[84], 5}, {NULL}};
const Segment SGI38177[] = {{&S35730[38], 7}, {NULL}};
const Segment SGI38178[] = {{&S35730[31], 6}, {NULL}};
const Segment SGI38179[] = {{&S35736[44], 7}, {NULL}};
const Segment SGI38180[] = {{&S35736[44], 24}, {NULL}};
const Segment SGI38181[] = {{&S35736[30], 6}, {NULL}};
const Segment SGI38182[] = {{&S35736[85], 4}, {NULL}};
const Segment SGI38183[] = {{&S35736[85], 3}, {NULL}};
const Segment SGI38184[] = {{&S35738[44], 7}, {NULL}};
const Segment SGI38185[] = {{&S35738[44], 24}, {NULL}};
const Segment SGI38186[] = {{&S35738[30], 6}, {NULL}};
const Segment SGI38187[] = {{&S35738[85], 6}, {NULL}};
const Segment SGI38188[] = {{&S35739[38], 7}, {NULL}};
const Segment SGI38189[] = {{&S35739[31], 6}, {NULL}};
const Segment SGI38190[] = {{&S35740[44], 9}, {NULL}};
const Segment SGI38191[] = {{&S35740[44], 26}, {NULL}};
const Segment SGI38192[] = {{&S35740[30], 6}, {NULL}};
const Segment SGI38193[] = {{&S35740[118], 3}, {NULL}};
const Segment SGI38194[] = {{&S35740[84], 5}, {NULL}};
const Segment SGI38195[] = {{&S35743[44], 7}, {NULL}};
const Segment SGI38196[] = {{&S35743[44], 24}, {NULL}};
const Segment SGI38197[] = {{&S35743[30], 6}, {NULL}};
const Segment SGI38198[] = {{&S35743[85], 6}, {NULL}};
const Segment SGI38199[] = {{&S35746[44], 7}, {NULL}};
const Segment SGI38200[] = {{&S35746[44], 18}, {NULL}};
const Segment SGI38201[] = {{&S35746[30], 6}, {NULL}};
const Segment SGI38202[] = {{&S35746[110], 3}, {NULL}};
const Segment SGI38203[] = {{&S35746[76], 5}, {NULL}};
const Segment SGI38204[] = {{&S35748[44], 25}, {NULL}};
const Segment SGI38205[] = {{&S35748[44], 46}, {NULL}};
const Segment SGI38206[] = {{&S35748[30], 6}, {NULL}};
const Segment SGI38207[] = {{&S35748[138], 3}, {NULL}};
const Segment SGI38208[] = {{&S35748[104], 5}, {NULL}};
const Segment SGI38209[] = {{&S35809[44], 18}, {NULL}};
const Segment SGI38210[] = {{&S35809[44], 29}, {NULL}};
const Segment SGI38211[] = {{&S35809[30], 6}, {NULL}};
const Segment SGI38212[] = {{&S35809[121], 3}, {NULL}};
const Segment SGI38213[] = {{&S35809[87], 5}, {NULL}};
const Segment SGI38214[] = {{&S35814[44], 9}, {NULL}};
const Segment SGI38215[] = {{&S35814[44], 20}, {NULL}};
const Segment SGI38216[] = {{&S35814[30], 6}, {NULL}};
const Segment SGI38217[] = {{&S35814[112], 3}, {NULL}};
const Segment SGI38218[] = {{&S35814[78], 5}, {NULL}};
const Segment SGI38219[] = {{&S35819[44], 18}, {NULL}};
const Segment SGI38220[] = {{&S35819[44], 29}, {NULL}};
const Segment SGI38221[] = {{&S35819[30], 6}, {NULL}};
const Segment SGI38222[] = {{&S35819[121], 3}, {NULL}};
const Segment SGI38223[] = {{&S35819[87], 5}, {NULL}};
const Segment SGI38224[] = {{&S35821[44], 9}, {NULL}};
const Segment SGI38225[] = {{&S35821[44], 20}, {NULL}};
const Segment SGI38226[] = {{&S35821[30], 6}, {NULL}};
const Segment SGI38227[] = {{&S35821[112], 3}, {NULL}};
const Segment SGI38228[] = {{&S35821[78], 5}, {NULL}};
const Segment SGI38229[] = {{&S35964[44], 10}, {NULL}};
const Segment SGI38230[] = {{&S35964[44], 21}, {NULL}};
const Segment SGI38231[] = {{&S35964[30], 6}, {NULL}};
const Segment SGI38232[] = {{&S35964[113], 3}, {NULL}};
const Segment SGI38233[] = {{&S35964[79], 5}, {NULL}};
const Segment SGI38234[] = {{&S35965[38], 9}, {NULL}};
const Segment SGI38235[] = {{&S35965[31], 6}, {NULL}};
const Segment SGI38236[] = {{&S35966[44], 10}, {NULL}};
const Segment SGI38237[] = {{&S35966[44], 21}, {NULL}};
const Segment SGI38238[] = {{&S35966[30], 6}, {NULL}};
const Segment SGI38239[] = {{&S35966[113], 3}, {NULL}};
const Segment SGI38240[] = {{&S35966[79], 5}, {NULL}};
const Segment SGI38241[] = {{&S35968[38], 10}, {NULL}};
const Segment SGI38242[] = {{&S35968[31], 6}, {NULL}};
const Segment SGI38243[] = {{&S35969[44], 9}, {NULL}};
const Segment SGI38244[] = {{&S35969[44], 20}, {NULL}};
const Segment SGI38245[] = {{&S35969[30], 6}, {NULL}};
const Segment SGI38246[] = {{&S35969[112], 3}, {NULL}};
const Segment SGI38247[] = {{&S35969[78], 5}, {NULL}};
const Segment SGI38248[] = {{&S35972[44], 20}, {NULL}};
const Segment SGI38249[] = {{&S35972[44], 31}, {NULL}};
const Segment SGI38250[] = {{&S35972[30], 6}, {NULL}};
const Segment SGI38251[] = {{&S35972[123], 3}, {NULL}};
const Segment SGI38252[] = {{&S35972[89], 5}, {NULL}};
const Segment SGI38253[] = {{&S47931[44], 15}, {NULL}};
const Segment SGI38254[] = {{&S47931[44], 27}, {NULL}};
const Segment SGI38255[] = {{&S47931[30], 6}, {NULL}};
const Segment SGI38256[] = {{&S35974[38], 10}, {NULL}};
const Segment SGI38257[] = {{&S47931[119], 3}, {NULL}};
const Segment SGI38258[] = {{&S35974[31], 6}, {NULL}};
const Segment SGI38259[] = {{&S47931[85], 5}, {NULL}};
const Segment SGI38260[] = {{&S47932[38], 15}, {NULL}};
const Segment SGI38261[] = {{&S47932[31], 6}, {NULL}};
const Segment SGI38262[] = {{&S47933[43], 15}, {NULL}};
const Segment SGI38263[] = {{&S47933[43], 27}, {NULL}};
const Segment SGI38264[] = {{&S47933[118], 3}, {NULL}};
const Segment SGI38265[] = {{&S47933[84], 5}, {NULL}};
const Segment SGI38266[] = {{&S35859[44], 6}, {NULL}};
const Segment SGI38267[] = {{&S35859[44], 23}, {NULL}};
const Segment SGI38268[] = {{&S35859[30], 6}, {NULL}};
const Segment SGI38269[] = {{&S47934[44], 14}, {NULL}};
const Segment SGI38270[] = {{&S47934[44], 26}, {NULL}};
const Segment SGI38271[] = {{&S47934[30], 6}, {NULL}};
const Segment SGI38272[] = {{&S35859[117], 3}, {NULL}};
const Segment SGI38273[] = {{&S35859[81], 7}, {NULL}};
const Segment SGI38274[] = {{&S35859[137], 8}, {NULL}};
const Segment SGI38275[] = {{&S47934[118], 3}, {NULL}};
const Segment SGI38276[] = {{&S47934[84], 5}, {NULL}};
const Segment SGI38277[] = {{&S35860[44], 6}, {NULL}};
const Segment SGI38278[] = {{&S35860[44], 23}, {NULL}};
const Segment SGI38279[] = {{&S35860[30], 6}, {NULL}};
const Segment SGI38280[] = {{&S47935[44], 14}, {NULL}};
const Segment SGI38281[] = {{&S47935[44], 26}, {NULL}};
const Segment SGI38282[] = {{&S35860[115], 3}, {NULL}};
const Segment SGI38283[] = {{&S47935[30], 6}, {NULL}};
const Segment SGI38284[] = {{&S35860[81], 5}, {NULL}};
const Segment SGI38285[] = {{&S47935[118], 3}, {NULL}};
const Segment SGI38286[] = {{&S47935[84], 5}, {NULL}};
const Segment SGI38287[] = {{&S35861[38], 6}, {NULL}};
const Segment SGI38288[] = {{&S35861[31], 6}, {NULL}};
const Segment SGI38289[] = {{&S47936[44], 14}, {NULL}};
const Segment SGI38290[] = {{&S47936[44], 26}, {NULL}};
const Segment SGI38291[] = {{&S47936[30], 6}, {NULL}};
const Segment SGI38292[] = {{&S47936[118], 3}, {NULL}};
const Segment SGI38293[] = {{&S47936[84], 5}, {NULL}};
const Segment SGI38294[] = {{&S35862[44], 6}, {NULL}};
const Segment SGI38295[] = {{&S35862[44], 25}, {NULL}};
const Segment SGI38296[] = {{&S35862[30], 6}, {NULL}};
const Segment SGI38297[] = {{&S35862[117], 3}, {NULL}};
const Segment SGI38298[] = {{&S35862[83], 5}, {NULL}};
const Segment SGI38299[] = {{&S47937[43], 15}, {NULL}};
const Segment SGI38300[] = {{&S47937[43], 27}, {NULL}};
const Segment SGI38301[] = {{&S47937[30], 5}, {NULL}};
const Segment SGI38302[] = {{&S47937[118], 3}, {NULL}};
const Segment SGI38303[] = {{&S47937[84], 5}, {NULL}};
const Segment SGI38304[] = {{&S35863[44], 6}, {NULL}};
const Segment SGI38305[] = {{&S35863[44], 25}, {NULL}};
const Segment SGI38306[] = {{&S35863[30], 6}, {NULL}};
const Segment SGI38307[] = {{&S35863[117], 3}, {NULL}};
const Segment SGI38308[] = {{&S35863[83], 5}, {NULL}};
const Segment SGI38309[] = {{&S35864[42], 6}, {NULL}};
const Segment SGI38310[] = {{&S35864[30], 6}, {NULL}};
const Segment SGI38311[] = {{&S35864[113], 3}, {NULL}};
const Segment SGI38312[] = {{&S35864[79], 5}, {NULL}};
const Segment SGI38313[] = {{&S35865[44], 6}, {NULL}};
const Segment SGI38314[] = {{&S35865[44], 20}, {NULL}};
const Segment SGI38315[] = {{&S35865[30], 6}, {NULL}};
const Segment SGI38316[] = {{&S35865[112], 3}, {NULL}};
const Segment SGI38317[] = {{&S35865[78], 5}, {NULL}};
const Segment SGI38318[] = {{&S47970[44], 12}, {NULL}};
const Segment SGI38319[] = {{&S47970[44], 24}, {NULL}};
const Segment SGI38320[] = {{&S47970[30], 6}, {NULL}};
const Segment SGI38321[] = {{&S47970[116], 3}, {NULL}};
const Segment SGI38322[] = {{&S47970[82], 5}, {NULL}};
const Segment SGI38323[] = {{&S47971[38], 12}, {NULL}};
const Segment SGI38324[] = {{&S47971[31], 6}, {NULL}};
const Segment SGI38325[] = {{&S35866[44], 6}, {NULL}};
const Segment SGI38326[] = {{&S35866[44], 27}, {NULL}};
const Segment SGI38327[] = {{&S35866[30], 6}, {NULL}};
const Segment SGI38328[] = {{&S35866[119], 3}, {NULL}};
const Segment SGI38329[] = {{&S35866[85], 5}, {NULL}};
const Segment SGI38330[] = {{&S36005[44], 24}, {NULL}};
const Segment SGI38331[] = {{&S36005[44], 35}, {NULL}};
const Segment SGI38332[] = {{&S36005[30], 6}, {NULL}};
const Segment SGI38333[] = {{&S36005[127], 3}, {NULL}};
const Segment SGI38334[] = {{&S36005[93], 5}, {NULL}};
const Segment SGI38335[] = {{&S36006[44], 24}, {NULL}};
const Segment SGI38336[] = {{&S36006[44], 35}, {NULL}};
const Segment SGI38337[] = {{&S36006[30], 6}, {NULL}};
const Segment SGI38338[] = {{&S36006[127], 3}, {NULL}};
const Segment SGI38339[] = {{&S36006[93], 5}, {NULL}};
const Segment SGI38340[] = {{&S36007[44], 24}, {NULL}};
const Segment SGI38341[] = {{&S36007[44], 35}, {NULL}};
const Segment SGI38342[] = {{&S36007[30], 6}, {NULL}};
const Segment SGI38343[] = {{&S36007[127], 3}, {NULL}};
const Segment SGI38344[] = {{&S36007[93], 5}, {NULL}};
const Segment SGI38345[] = {{&S36013[44], 17}, {NULL}};
const Segment SGI38346[] = {{&S36013[44], 28}, {NULL}};
const Segment SGI38347[] = {{&S36013[30], 6}, {NULL}};
const Segment SGI38348[] = {{&S36013[120], 3}, {NULL}};
const Segment SGI38349[] = {{&S36013[86], 5}, {NULL}};
const Segment SGI38350[] = {{&S36016[38], 24}, {NULL}};
const Segment SGI38351[] = {{&S36016[31], 6}, {NULL}};
const Segment SGI38352[] = {{&S38193[44], 11}, {NULL}};
const Segment SGI38353[] = {{&S38193[44], 28}, {NULL}};
const Segment SGI38354[] = {{&S38193[30], 6}, {NULL}};
const Segment SGI38355[] = {{&S38193[120], 3}, {NULL}};
const Segment SGI38356[] = {{&S38193[86], 5}, {NULL}};
const Segment SGI38357[] = {{&S38198[38], 11}, {NULL}};
const Segment SGI38358[] = {{&S38198[31], 6}, {NULL}};
const Segment SGI38359[] = {{&S38202[44], 11}, {NULL}};
const Segment SGI38360[] = {{&S38202[44], 28}, {NULL}};
const Segment SGI38361[] = {{&S38202[30], 6}, {NULL}};
const Segment SGI38362[] = {{&S38202[121], 3}, {NULL}};
const Segment SGI38363[] = {{&S38202[86], 6}, {NULL}};
const Segment SGI38364[] = {{&S38206[44], 11}, {NULL}};
const Segment SGI38365[] = {{&S38206[44], 28}, {NULL}};
const Segment SGI38366[] = {{&S38206[30], 6}, {NULL}};
const Segment SGI38367[] = {{&S38206[89], 6}, {NULL}};
const Segment SGI38368[] = {{&S36214[44], 26}, {NULL}};
const Segment SGI38369[] = {{&S36214[44], 37}, {NULL}};
const Segment SGI38370[] = {{&S36214[30], 6}, {NULL}};
const Segment SGI38371[] = {{&S36214[129], 3}, {NULL}};
const Segment SGI38372[] = {{&S38318[44], 18}, {NULL}};
const Segment SGI38373[] = {{&S38318[44], 29}, {NULL}};
const Segment SGI38374[] = {{&S36214[95], 5}, {NULL}};
const Segment SGI38375[] = {{&S38318[30], 6}, {NULL}};
const Segment SGI38376[] = {{&S38318[121], 3}, {NULL}};
const Segment SGI38377[] = {{&S38318[87], 5}, {NULL}};
const Segment SGI38378[] = {{&S36215[44], 26}, {NULL}};
const Segment SGI38379[] = {{&S36215[44], 37}, {NULL}};
const Segment SGI38380[] = {{&S36215[30], 6}, {NULL}};
const Segment SGI38381[] = {{&S36215[129], 3}, {NULL}};
const Segment SGI38382[] = {{&S36215[95], 5}, {NULL}};
const Segment SGI38383[] = {{&S36216[44], 26}, {NULL}};
const Segment SGI38384[] = {{&S36216[44], 37}, {NULL}};
const Segment SGI38385[] = {{&S36216[30], 6}, {NULL}};
const Segment SGI38386[] = {{&S36216[129], 3}, {NULL}};
const Segment SGI38387[] = {{&S36216[95], 5}, {NULL}};
const Segment SGI38388[] = {{&S36217[44], 26}, {NULL}};
const Segment SGI38389[] = {{&S36217[44], 37}, {NULL}};
const Segment SGI38390[] = {{&S36217[30], 6}, {NULL}};
const Segment SGI38391[] = {{&S36217[129], 3}, {NULL}};
const Segment SGI38392[] = {{&S36217[95], 5}, {NULL}};
const Segment SGI38393[] = {{&S36218[44], 21}, {NULL}};
const Segment SGI38394[] = {{&S36218[44], 32}, {NULL}};
const Segment SGI38395[] = {{&S36218[30], 6}, {NULL}};
const Segment SGI38396[] = {{&S36218[124], 3}, {NULL}};
const Segment SGI38397[] = {{&S36218[90], 5}, {NULL}};
const Segment SGI38398[] = {{&S36219[44], 11}, {NULL}};
const Segment SGI38399[] = {{&S36219[44], 22}, {NULL}};
const Segment SGI38400[] = {{&S36219[30], 6}, {NULL}};
const Segment SGI38401[] = {{&S36219[114], 3}, {NULL}};
const Segment SGI38402[] = {{&S36219[80], 5}, {NULL}};
const Segment SGI38403[] = {{&S36220[44], 23}, {NULL}};
const Segment SGI38404[] = {{&S36220[44], 34}, {NULL}};
const Segment SGI38405[] = {{&S36220[30], 6}, {NULL}};
const Segment SGI38406[] = {{&S36220[126], 3}, {NULL}};
const Segment SGI38407[] = {{&S36220[92], 5}, {NULL}};
const Segment SGI38408[] = {{&S36241[44], 11}, {NULL}};
const Segment SGI38409[] = {{&S36241[44], 22}, {NULL}};
const Segment SGI38410[] = {{&S36241[30], 6}, {NULL}};
const Segment SGI38411[] = {{&S36241[114], 3}, {NULL}};
const Segment SGI38412[] = {{&S36241[80], 5}, {NULL}};
const Segment SGI38413[] = {{&S36247[44], 11}, {NULL}};
const Segment SGI38414[] = {{&S36247[44], 22}, {NULL}};
const Segment SGI38415[] = {{&S36247[30], 6}, {NULL}};
const Segment SGI38416[] = {{&S36247[114], 3}, {NULL}};
const Segment SGI38417[] = {{&S36247[80], 5}, {NULL}};
const Segment SGI38418[] = {{&S36236[44], 26}, {NULL}};
const Segment SGI38419[] = {{&S36236[44], 37}, {NULL}};
const Segment SGI38420[] = {{&S36236[30], 6}, {NULL}};
const Segment SGI38421[] = {{&S36236[129], 3}, {NULL}};
const Segment SGI38422[] = {{&S36236[95], 5}, {NULL}};
const Segment SGI38423[] = {{&S36237[44], 26}, {NULL}};
const Segment SGI38424[] = {{&S36237[44], 37}, {NULL}};
const Segment SGI38425[] = {{&S36237[30], 6}, {NULL}};
const Segment SGI38426[] = {{&S36237[129], 3}, {NULL}};
const Segment SGI38427[] = {{&S36237[95], 5}, {NULL}};
const Segment SGI38428[] = {{&S36238[44], 25}, {NULL}};
const Segment SGI38429[] = {{&S36238[44], 36}, {NULL}};
const Segment SGI38430[] = {{&S36238[30], 6}, {NULL}};
const Segment SGI38431[] = {{&S36238[128], 3}, {NULL}};
const Segment SGI38432[] = {{&S36238[94], 5}, {NULL}};
const Segment SGI38433[] = {{&S36240[44], 21}, {NULL}};
const Segment SGI38434[] = {{&S36240[44], 32}, {NULL}};
const Segment SGI38435[] = {{&S36240[30], 6}, {NULL}};
const Segment SGI38436[] = {{&S36240[124], 3}, {NULL}};
const Segment SGI38437[] = {{&S36240[90], 5}, {NULL}};
const Segment SGI38438[] = {{&S36243[44], 15}, {NULL}};
const Segment SGI38439[] = {{&S36243[44], 26}, {NULL}};
const Segment SGI38440[] = {{&S36243[30], 6}, {NULL}};
const Segment SGI38441[] = {{&S36243[118], 3}, {NULL}};
const Segment SGI38442[] = {{&S36243[84], 5}, {NULL}};
const Segment SGI38443[] = {{&S36244[44], 15}, {NULL}};
const Segment SGI38444[] = {{&S36244[44], 26}, {NULL}};
const Segment SGI38445[] = {{&S36244[30], 6}, {NULL}};
const Segment SGI38446[] = {{&S36244[118], 3}, {NULL}};
const Segment SGI38447[] = {{&S36244[84], 5}, {NULL}};
const Segment SGI38448[] = {{&S36268[44], 39}, {NULL}};
const Segment SGI38449[] = {{&S36268[44], 50}, {NULL}};
const Segment SGI38450[] = {{&S36268[30], 6}, {NULL}};
const Segment SGI38451[] = {{&S36268[142], 3}, {NULL}};
const Segment SGI38452[] = {{&S36268[108], 5}, {NULL}};
const Segment SGI38453[] = {{&S36245[44], 26}, {NULL}};
const Segment SGI38454[] = {{&S36245[44], 37}, {NULL}};
const Segment SGI38455[] = {{&S36245[30], 6}, {NULL}};
const Segment SGI38456[] = {{&S36245[129], 3}, {NULL}};
const Segment SGI38457[] = {{&S36245[95], 5}, {NULL}};
const Segment SGI38458[] = {{&S36246[44], 23}, {NULL}};
const Segment SGI38459[] = {{&S36246[44], 34}, {NULL}};
const Segment SGI38460[] = {{&S36246[30], 6}, {NULL}};
const Segment SGI38461[] = {{&S36246[126], 3}, {NULL}};
const Segment SGI38462[] = {{&S36246[92], 5}, {NULL}};
const Segment SGI38463[] = {{&S36248[44], 26}, {NULL}};
const Segment SGI38464[] = {{&S36248[44], 37}, {NULL}};
const Segment SGI38465[] = {{&S36248[30], 6}, {NULL}};
const Segment SGI38466[] = {{&S36248[129], 3}, {NULL}};
const Segment SGI38467[] = {{&S36248[95], 5}, {NULL}};
const Segment SGI38468[] = {{&S36250[44], 26}, {NULL}};
const Segment SGI38469[] = {{&S36250[44], 37}, {NULL}};
const Segment SGI38470[] = {{&S36250[30], 6}, {NULL}};
const Segment SGI38471[] = {{&S36250[129], 3}, {NULL}};
const Segment SGI38472[] = {{&S36250[95], 5}, {NULL}};
const Segment SGI38473[] = {{&S36251[44], 26}, {NULL}};
const Segment SGI38474[] = {{&S36251[44], 37}, {NULL}};
const Segment SGI38475[] = {{&S36251[30], 6}, {NULL}};
const Segment SGI38476[] = {{&S36251[129], 3}, {NULL}};
const Segment SGI38477[] = {{&S36251[95], 5}, {NULL}};
const Segment SGI38478[] = {{&S36252[44], 26}, {NULL}};
const Segment SGI38479[] = {{&S36252[44], 37}, {NULL}};
const Segment SGI38480[] = {{&S36252[30], 6}, {NULL}};
const Segment SGI38481[] = {{&S36252[129], 3}, {NULL}};
const Segment SGI38482[] = {{&S36252[95], 5}, {NULL}};
const Segment SGI38483[] = {{&S36253[44], 26}, {NULL}};
const Segment SGI38484[] = {{&S36253[44], 37}, {NULL}};
const Segment SGI38485[] = {{&S36253[30], 6}, {NULL}};
const Segment SGI38486[] = {{&S36253[129], 3}, {NULL}};
const Segment SGI38487[] = {{&S36253[95], 5}, {NULL}};
const Segment SGI38488[] = {{&S36286[44], 15}, {NULL}};
const Segment SGI38489[] = {{&S36286[44], 25}, {NULL}};
const Segment SGI38490[] = {{&S36286[30], 6}, {NULL}};
const Segment SGI38491[] = {{&S36286[117], 3}, {NULL}};
const Segment SGI38492[] = {{&S36254[44], 26}, {NULL}};
const Segment SGI38493[] = {{&S36254[44], 37}, {NULL}};
const Segment SGI38494[] = {{&S36286[83], 5}, {NULL}};
const Segment SGI38495[] = {{&S36254[30], 6}, {NULL}};
const Segment SGI38496[] = {{&S36254[129], 3}, {NULL}};
const Segment SGI38497[] = {{&S36254[95], 5}, {NULL}};
const Segment SGI38498[] = {{&S36255[38], 15}, {NULL}};
const Segment SGI38499[] = {{&S36255[31], 6}, {NULL}};
const Segment SGI38500[] = {{&S36256[44], 26}, {NULL}};
const Segment SGI38501[] = {{&S36256[44], 37}, {NULL}};
const Segment SGI38502[] = {{&S36256[30], 6}, {NULL}};
const Segment SGI38503[] = {{&S36256[129], 3}, {NULL}};
const Segment SGI38504[] = {{&S36256[95], 5}, {NULL}};
const Segment SGI38505[] = {{&S36258[44], 26}, {NULL}};
const Segment SGI38506[] = {{&S36258[44], 37}, {NULL}};
const Segment SGI38507[] = {{&S36258[30], 6}, {NULL}};
const Segment SGI38508[] = {{&S36258[129], 3}, {NULL}};
const Segment SGI38509[] = {{&S36258[95], 5}, {NULL}};
const Segment SGI38510[] = {{&S36259[44], 26}, {NULL}};
const Segment SGI38511[] = {{&S36259[44], 37}, {NULL}};
const Segment SGI38512[] = {{&S36259[30], 6}, {NULL}};
const Segment SGI38513[] = {{&S36259[129], 3}, {NULL}};
const Segment SGI38514[] = {{&S36259[95], 5}, {NULL}};
const Segment SGI38515[] = {{&S36260[44], 26}, {NULL}};
const Segment SGI38516[] = {{&S36260[44], 37}, {NULL}};
const Segment SGI38517[] = {{&S36260[30], 6}, {NULL}};
const Segment SGI38518[] = {{&S36260[129], 3}, {NULL}};
const Segment SGI38519[] = {{&S36260[95], 5}, {NULL}};
const Segment SGI38520[] = {{&S36265[38], 11}, {NULL}};
const Segment SGI38521[] = {{&S36265[31], 6}, {NULL}};
const Segment SGI38522[] = {{&S36266[44], 26}, {NULL}};
const Segment SGI38523[] = {{&S36266[44], 37}, {NULL}};
const Segment SGI38524[] = {{&S36266[30], 6}, {NULL}};
const Segment SGI38525[] = {{&S36266[129], 3}, {NULL}};
const Segment SGI38526[] = {{&S36266[95], 5}, {NULL}};
const Segment SGI38527[] = {{&S36269[44], 26}, {NULL}};
const Segment SGI38528[] = {{&S36269[44], 37}, {NULL}};
const Segment SGI38529[] = {{&S36269[30], 6}, {NULL}};
const Segment SGI38530[] = {{&S36269[129], 3}, {NULL}};
const Segment SGI38531[] = {{&S36269[95], 5}, {NULL}};
const Segment SGI38532[] = {{&S36270[44], 21}, {NULL}};
const Segment SGI38533[] = {{&S36270[44], 32}, {NULL}};
const Segment SGI38534[] = {{&S36270[30], 6}, {NULL}};
const Segment SGI38535[] = {{&S36270[93], 4}, {NULL}};
const Segment SGI38536[] = {{&S36270[93], 3}, {NULL}};
const Segment SGI38537[] = {{&S36271[38], 21}, {NULL}};
const Segment SGI38538[] = {{&S36271[31], 6}, {NULL}};
const Segment SGI38539[] = {{&S36289[44], 15}, {NULL}};
const Segment SGI38540[] = {{&S36289[44], 37}, {NULL}};
const Segment SGI38541[] = {{&S36289[30], 6}, {NULL}};
const Segment SGI38542[] = {{&S36289[129], 3}, {NULL}};
const Segment SGI38543[] = {{&S36289[95], 5}, {NULL}};
const Segment SGI38544[] = {{&S36290[44], 14}, {NULL}};
const Segment SGI38545[] = {{&S36290[44], 36}, {NULL}};
const Segment SGI38546[] = {{&S36290[30], 6}, {NULL}};
const Segment SGI38547[] = {{&S36290[128], 3}, {NULL}};
const Segment SGI38548[] = {{&S36290[94], 5}, {NULL}};
const Segment SGI38549[] = {{&S36294[43], 15}, {NULL}};
const Segment SGI38550[] = {{&S36294[43], 37}, {NULL}};
const Segment SGI38551[] = {{&S36294[30], 5}, {NULL}};
const Segment SGI38552[] = {{&S36294[128], 3}, {NULL}};
const Segment SGI38553[] = {{&S36294[94], 5}, {NULL}};
const Segment SGI38554[] = {{&S36295[38], 15}, {NULL}};
const Segment SGI38555[] = {{&S36295[31], 6}, {NULL}};
const Segment SGI38556[] = {{&S36296[44], 14}, {NULL}};
const Segment SGI38557[] = {{&S36296[44], 36}, {NULL}};
const Segment SGI38558[] = {{&S36296[30], 6}, {NULL}};
const Segment SGI38559[] = {{&S36296[128], 3}, {NULL}};
const Segment SGI38560[] = {{&S36296[94], 5}, {NULL}};
const Segment SGI38561[] = {{&S36299[44], 15}, {NULL}};
const Segment SGI38562[] = {{&S36299[44], 37}, {NULL}};
const Segment SGI38563[] = {{&S36299[30], 6}, {NULL}};
const Segment SGI38564[] = {{&S36299[129], 3}, {NULL}};
const Segment SGI38565[] = {{&S36299[95], 5}, {NULL}};
const Segment SGI38566[] = {{&S36300[44], 14}, {NULL}};
const Segment SGI38567[] = {{&S36300[44], 36}, {NULL}};
const Segment SGI38568[] = {{&S36300[30], 6}, {NULL}};
const Segment SGI38569[] = {{&S36300[128], 3}, {NULL}};
const Segment SGI38570[] = {{&S36300[94], 5}, {NULL}};
const Segment SGI38571[] = {{&S36301[43], 15}, {NULL}};
const Segment SGI38572[] = {{&S36301[43], 37}, {NULL}};
const Segment SGI38573[] = {{&S36301[128], 3}, {NULL}};
const Segment SGI38574[] = {{&S36301[94], 5}, {NULL}};
const Segment SGI38575[] = {{&S36303[44], 15}, {NULL}};
const Segment SGI38576[] = {{&S36303[44], 26}, {NULL}};
const Segment SGI38577[] = {{&S36303[30], 6}, {NULL}};
const Segment SGI38578[] = {{&S36303[118], 3}, {NULL}};
const Segment SGI38579[] = {{&S36303[84], 5}, {NULL}};
const Segment SGI38580[] = {{&S38542[44], 10}, {NULL}};
const Segment SGI38581[] = {{&S38542[44], 27}, {NULL}};
const Segment SGI38582[] = {{&S38542[30], 6}, {NULL}};
const Segment SGI38583[] = {{&S38542[119], 3}, {NULL}};
const Segment SGI38584[] = {{&S38542[85], 5}, {NULL}};
const Segment SGI38585[] = {{&S38543[44], 9}, {NULL}};
const Segment SGI38586[] = {{&S38543[44], 26}, {NULL}};
const Segment SGI38587[] = {{&S38543[30], 6}, {NULL}};
const Segment SGI38588[] = {{&S38543[118], 3}, {NULL}};
const Segment SGI38589[] = {{&S38543[84], 5}, {NULL}};
const Segment SGI38590[] = {{&S38544[44], 10}, {NULL}};
const Segment SGI38591[] = {{&S38544[44], 27}, {NULL}};
const Segment SGI38592[] = {{&S38544[30], 6}, {NULL}};
const Segment SGI38593[] = {{&S38544[119], 3}, {NULL}};
const Segment SGI38594[] = {{&S38544[85], 5}, {NULL}};
const Segment SGI38595[] = {{&S38545[43], 10}, {NULL}};
const Segment SGI38596[] = {{&S38545[43], 27}, {NULL}};
const Segment SGI38597[] = {{&S38545[30], 5}, {NULL}};
const Segment SGI38598[] = {{&S38545[118], 3}, {NULL}};
const Segment SGI38599[] = {{&S38545[84], 5}, {NULL}};
const Segment SGI38600[] = {{&S38546[43], 10}, {NULL}};
const Segment SGI38601[] = {{&S38546[43], 27}, {NULL}};
const Segment SGI38602[] = {{&S38546[118], 3}, {NULL}};
const Segment SGI38603[] = {{&S38546[84], 5}, {NULL}};
const Segment SGI38604[] = {{&S38547[44], 9}, {NULL}};
const Segment SGI38605[] = {{&S38547[44], 26}, {NULL}};
const Segment SGI38606[] = {{&S38547[30], 6}, {NULL}};
const Segment SGI38607[] = {{&S38547[118], 3}, {NULL}};
const Segment SGI38608[] = {{&S38547[84], 5}, {NULL}};
const Segment SGI38609[] = {{&S38549[44], 10}, {NULL}};
const Segment SGI38610[] = {{&S38549[44], 27}, {NULL}};
const Segment SGI38611[] = {{&S38549[30], 6}, {NULL}};
const Segment SGI38612[] = {{&S38549[119], 3}, {&S38549[161], 3}, {NULL}};
const Segment SGI38613[] = {{&S38549[85], 5}, {NULL}};
const Segment SGI38614[] = {{&S38550[38], 10}, {NULL}};
const Segment SGI38615[] = {{&S38550[31], 6}, {NULL}};
const Segment SGI38616[] = {{&S38551[44], 10}, {NULL}};
const Segment SGI38617[] = {{&S38551[44], 33}, {NULL}};
const Segment SGI38618[] = {{&S38551[30], 6}, {NULL}};
const Segment SGI38619[] = {{&S38553[44], 10}, {NULL}};
const Segment SGI38620[] = {{&S38553[44], 27}, {NULL}};
const Segment SGI38621[] = {{&S38553[30], 6}, {NULL}};
const Segment SGI38622[] = {{&S38553[121], 3}, {NULL}};
const Segment SGI38623[] = {{&S38553[85], 7}, {NULL}};
const Segment SGI38624[] = {{&S38553[141], 8}, {NULL}};
const Segment SGI38625[] = {{&S38585[44], 10}, {NULL}};
const Segment SGI38626[] = {{&S38585[44], 27}, {NULL}};
const Segment SGI38627[] = {{&S38585[30], 6}, {NULL}};
const Segment SGI38628[] = {{&S38585[119], 3}, {NULL}};
const Segment SGI38629[] = {{&S38585[85], 5}, {NULL}};
const Segment SGI38630[] = {{&S38586[44], 18}, {NULL}};
const Segment SGI38631[] = {{&S38586[44], 35}, {NULL}};
const Segment SGI38632[] = {{&S38586[30], 6}, {NULL}};
const Segment SGI38633[] = {{&S38586[127], 3}, {NULL}};
const Segment SGI38634[] = {{&S38586[93], 5}, {NULL}};
const Segment SGI38635[] = {{&S38597[44], 10}, {NULL}};
const Segment SGI38636[] = {{&S38597[44], 21}, {NULL}};
const Segment SGI38637[] = {{&S38597[30], 6}, {NULL}};
const Segment SGI38638[] = {{&S38597[113], 3}, {NULL}};
const Segment SGI38639[] = {{&S38597[79], 5}, {NULL}};
const Segment SGI38640[] = {{&S38599[38], 18}, {NULL}};
const Segment SGI38641[] = {{&S38599[31], 6}, {NULL}};
const Segment SGI38642[] = {{&S38601[38], 10}, {NULL}};
const Segment SGI38643[] = {{&S38601[31], 6}, {NULL}};
const Segment SGI38644[] = {{&S36565[44], 19}, {NULL}};
const Segment SGI38645[] = {{&S36565[44], 37}, {NULL}};
const Segment SGI38646[] = {{&S36565[30], 6}, {NULL}};
const Segment SGI38647[] = {{&S36565[69], 3}, {NULL}};
const Segment SGI38648[] = {{&S36565[95], 5}, {NULL}};
const Segment SGI38649[] = {{&S36566[44], 19}, {NULL}};
const Segment SGI38650[] = {{&S36566[44], 38}, {NULL}};
const Segment SGI38651[] = {{&S36566[30], 6}, {NULL}};
const Segment SGI38652[] = {{&S36566[69], 3}, {NULL}};
const Segment SGI38653[] = {{&S36566[96], 5}, {NULL}};
const Segment SGI38654[] = {{&S36567[44], 19}, {NULL}};
const Segment SGI38655[] = {{&S36567[44], 38}, {NULL}};
const Segment SGI38656[] = {{&S36567[30], 6}, {NULL}};
const Segment SGI38657[] = {{&S36567[69], 3}, {NULL}};
const Segment SGI38658[] = {{&S36567[96], 5}, {NULL}};
const Segment SGI38659[] = {{&S36568[44], 19}, {NULL}};
const Segment SGI38660[] = {{&S36568[44], 38}, {NULL}};
const Segment SGI38661[] = {{&S36568[30], 6}, {NULL}};
const Segment SGI38662[] = {{&S36568[69], 3}, {NULL}};
const Segment SGI38663[] = {{&S36568[96], 5}, {NULL}};
const Segment SGI38664[] = {{&S36569[44], 19}, {NULL}};
const Segment SGI38665[] = {{&S36569[44], 38}, {NULL}};
const Segment SGI38666[] = {{&S36569[30], 6}, {NULL}};
const Segment SGI38667[] = {{&S36569[69], 3}, {NULL}};
const Segment SGI38668[] = {{&S36569[96], 5}, {NULL}};
const Segment SGI38669[] = {{&S36570[44], 19}, {NULL}};
const Segment SGI38670[] = {{&S36570[44], 38}, {NULL}};
const Segment SGI38671[] = {{&S36570[30], 6}, {NULL}};
const Segment SGI38672[] = {{&S36570[69], 3}, {NULL}};
const Segment SGI38673[] = {{&S36570[96], 5}, {NULL}};
const Segment SGI38674[] = {{&S36571[44], 19}, {NULL}};
const Segment SGI38675[] = {{&S36571[44], 38}, {NULL}};
const Segment SGI38676[] = {{&S36571[30], 6}, {NULL}};
const Segment SGI38677[] = {{&S36571[69], 3}, {NULL}};
const Segment SGI38678[] = {{&S36571[96], 5}, {NULL}};
const Segment SGI38679[] = {{&S36572[44], 19}, {NULL}};
const Segment SGI38680[] = {{&S36572[44], 38}, {NULL}};
const Segment SGI38681[] = {{&S36572[30], 6}, {NULL}};
const Segment SGI38682[] = {{&S36572[69], 3}, {NULL}};
const Segment SGI38683[] = {{&S36572[96], 5}, {NULL}};
const Segment SGI38684[] = {{&S36573[44], 19}, {NULL}};
const Segment SGI38685[] = {{&S36573[44], 37}, {NULL}};
const Segment SGI38686[] = {{&S36573[30], 6}, {NULL}};
const Segment SGI38687[] = {{&S36573[69], 3}, {NULL}};
const Segment SGI38688[] = {{&S36573[95], 5}, {NULL}};
const Segment SGI38689[] = {{&S36574[44], 19}, {NULL}};
const Segment SGI38690[] = {{&S36574[44], 37}, {NULL}};
const Segment SGI38691[] = {{&S36574[30], 6}, {NULL}};
const Segment SGI38692[] = {{&S36574[69], 3}, {NULL}};
const Segment SGI38693[] = {{&S36574[95], 5}, {NULL}};
const Segment SGI38694[] = {{&S38697[44], 7}, {NULL}};
const Segment SGI38695[] = {{&S38697[44], 18}, {NULL}};
const Segment SGI38696[] = {{&S38697[30], 6}, {NULL}};
const Segment SGI38697[] = {{&S38697[110], 3}, {NULL}};
const Segment SGI38698[] = {{&S36575[44], 19}, {NULL}};
const Segment SGI38699[] = {{&S38697[76], 5}, {NULL}};
const Segment SGI38700[] = {{&S36575[44], 37}, {NULL}};
const Segment SGI38701[] = {{&S36575[30], 6}, {NULL}};
const Segment SGI38702[] = {{&S36575[69], 3}, {NULL}};
const Segment SGI38703[] = {{&S36575[95], 5}, {NULL}};
const Segment SGI38704[] = {{&S36576[44], 19}, {NULL}};
const Segment SGI38705[] = {{&S36576[44], 37}, {NULL}};
const Segment SGI38706[] = {{&S36576[30], 6}, {NULL}};
const Segment SGI38707[] = {{&S36576[69], 3}, {NULL}};
const Segment SGI38708[] = {{&S36576[95], 5}, {NULL}};
const Segment SGI38709[] = {{&S36577[44], 19}, {NULL}};
const Segment SGI38710[] = {{&S36577[44], 36}, {NULL}};
const Segment SGI38711[] = {{&S36577[30], 6}, {NULL}};
const Segment SGI38712[] = {{&S36577[69], 3}, {NULL}};
const Segment SGI38713[] = {{&S36577[94], 5}, {NULL}};
const Segment SGI38714[] = {{&S36578[44], 19}, {NULL}};
const Segment SGI38715[] = {{&S36578[44], 38}, {NULL}};
const Segment SGI38716[] = {{&S36578[30], 6}, {NULL}};
const Segment SGI38717[] = {{&S36578[69], 3}, {NULL}};
const Segment SGI38718[] = {{&S36578[96], 5}, {NULL}};
const Segment SGI38719[] = {{&S36579[44], 7}, {NULL}};
const Segment SGI38720[] = {{&S36579[44], 25}, {NULL}};
const Segment SGI38721[] = {{&S36579[30], 6}, {NULL}};
const Segment SGI38722[] = {{&S36579[57], 3}, {NULL}};
const Segment SGI38723[] = {{&S36579[83], 5}, {NULL}};
const Segment SGI38724[] = {{&S36580[44], 7}, {NULL}};
const Segment SGI38725[] = {{&S36580[44], 26}, {NULL}};
const Segment SGI38726[] = {{&S36580[30], 6}, {NULL}};
const Segment SGI38727[] = {{&S36580[57], 3}, {NULL}};
const Segment SGI38728[] = {{&S36580[84], 5}, {NULL}};
const Segment SGI38729[] = {{&S36581[44], 19}, {NULL}};
const Segment SGI38730[] = {{&S36581[44], 37}, {NULL}};
const Segment SGI38731[] = {{&S36581[30], 6}, {NULL}};
const Segment SGI38732[] = {{&S36581[69], 3}, {NULL}};
const Segment SGI38733[] = {{&S36581[95], 5}, {NULL}};
const Segment SGI38734[] = {{&S36582[44], 19}, {NULL}};
const Segment SGI38735[] = {{&S36582[44], 37}, {NULL}};
const Segment SGI38736[] = {{&S36582[30], 6}, {NULL}};
const Segment SGI38737[] = {{&S36582[69], 3}, {NULL}};
const Segment SGI38738[] = {{&S36582[95], 5}, {NULL}};
const Segment SGI38739[] = {{&S36583[44], 19}, {NULL}};
const Segment SGI38740[] = {{&S36583[44], 37}, {NULL}};
const Segment SGI38741[] = {{&S36583[30], 6}, {NULL}};
const Segment SGI38742[] = {{&S36583[69], 3}, {NULL}};
const Segment SGI38743[] = {{&S36583[95], 5}, {NULL}};
const Segment SGI38744[] = {{&S36584[38], 7}, {NULL}};
const Segment SGI38745[] = {{&S36584[31], 6}, {NULL}};
const Segment SGI38746[] = {{&S36585[44], 20}, {NULL}};
const Segment SGI38747[] = {{&S36585[44], 38}, {NULL}};
const Segment SGI38748[] = {{&S36585[30], 6}, {NULL}};
const Segment SGI38749[] = {{&S48667[44], 11}, {NULL}};
const Segment SGI38750[] = {{&S48667[44], 23}, {NULL}};
const Segment SGI38751[] = {{&S48667[30], 6}, {NULL}};
const Segment SGI38752[] = {{&S36585[70], 3}, {NULL}};
const Segment SGI38753[] = {{&S38720[44], 10}, {NULL}};
const Segment SGI38754[] = {{&S36585[96], 5}, {NULL}};
const Segment SGI38755[] = {{&S38720[44], 27}, {NULL}};
const Segment SGI38756[] = {{&S48667[115], 3}, {NULL}};
const Segment SGI38757[] = {{&S38720[30], 6}, {NULL}};
const Segment SGI38758[] = {{&S48667[81], 5}, {NULL}};
const Segment SGI38759[] = {{&S38720[119], 3}, {NULL}};
const Segment SGI38760[] = {{&S38720[85], 5}, {NULL}};
const Segment SGI38761[] = {{&S36586[44], 19}, {NULL}};
const Segment SGI38762[] = {{&S36586[44], 37}, {NULL}};
const Segment SGI38763[] = {{&S36586[30], 6}, {NULL}};
const Segment SGI38764[] = {{&S36586[69], 3}, {NULL}};
const Segment SGI38765[] = {{&S36586[95], 5}, {NULL}};
const Segment SGI38766[] = {{&S36587[44], 19}, {NULL}};
const Segment SGI38767[] = {{&S36587[44], 37}, {NULL}};
const Segment SGI38768[] = {{&S36587[30], 6}, {NULL}};
const Segment SGI38769[] = {{&S36587[69], 3}, {NULL}};
const Segment SGI38770[] = {{&S36587[95], 5}, {NULL}};
const Segment SGI38771[] = {{&S36588[44], 7}, {NULL}};
const Segment SGI38772[] = {{&S36588[44], 25}, {NULL}};
const Segment SGI38773[] = {{&S36588[30], 6}, {NULL}};
const Segment SGI38774[] = {{&S36588[57], 3}, {NULL}};
const Segment SGI38775[] = {{&S36588[83], 5}, {NULL}};
const Segment SGI38776[] = {{&S36589[44], 19}, {NULL}};
const Segment SGI38777[] = {{&S36589[44], 37}, {NULL}};
const Segment SGI38778[] = {{&S36589[30], 6}, {NULL}};
const Segment SGI38779[] = {{&S36589[69], 3}, {NULL}};
const Segment SGI38780[] = {{&S36589[95], 5}, {NULL}};
const Segment SGI38781[] = {{&S36590[44], 7}, {NULL}};
const Segment SGI38782[] = {{&S36590[44], 25}, {NULL}};
const Segment SGI38783[] = {{&S36590[30], 6}, {NULL}};
const Segment SGI38784[] = {{&S36590[57], 3}, {NULL}};
const Segment SGI38785[] = {{&S36590[83], 5}, {NULL}};
const Segment SGI38786[] = {{&S48678[38], 11}, {NULL}};
const Segment SGI38787[] = {{&S48678[31], 6}, {NULL}};
const Segment SGI38788[] = {{&S36591[44], 7}, {NULL}};
const Segment SGI38789[] = {{&S36591[44], 26}, {NULL}};
const Segment SGI38790[] = {{&S36591[30], 6}, {NULL}};
const Segment SGI38791[] = {{&S36591[57], 3}, {NULL}};
const Segment SGI38792[] = {{&S36591[84], 5}, {NULL}};
const Segment SGI38793[] = {{&S36592[44], 7}, {NULL}};
const Segment SGI38794[] = {{&S36592[44], 25}, {NULL}};
const Segment SGI38795[] = {{&S36592[30], 6}, {NULL}};
const Segment SGI38796[] = {{&S36592[57], 3}, {NULL}};
const Segment SGI38797[] = {{&S36592[83], 5}, {NULL}};
const Segment SGI38798[] = {{&S36593[44], 19}, {NULL}};
const Segment SGI38799[] = {{&S36593[44], 37}, {NULL}};
const Segment SGI38800[] = {{&S36593[30], 6}, {NULL}};
const Segment SGI38801[] = {{&S36593[69], 3}, {NULL}};
const Segment SGI38802[] = {{&S38737[44], 18}, {NULL}};
const Segment SGI38803[] = {{&S36593[95], 5}, {NULL}};
const Segment SGI38804[] = {{&S38737[44], 35}, {NULL}};
const Segment SGI38805[] = {{&S38737[30], 6}, {NULL}};
const Segment SGI38806[] = {{&S38737[127], 3}, {NULL}};
const Segment SGI38807[] = {{&S38737[93], 5}, {NULL}};
const Segment SGI38808[] = {{&S36594[44], 19}, {NULL}};
const Segment SGI38809[] = {{&S36594[44], 37}, {NULL}};
const Segment SGI38810[] = {{&S36594[30], 6}, {NULL}};
const Segment SGI38811[] = {{&S36594[69], 3}, {NULL}};
const Segment SGI38812[] = {{&S36594[95], 5}, {NULL}};
const Segment SGI38813[] = {{&S38739[44], 10}, {NULL}};
const Segment SGI38814[] = {{&S38739[44], 27}, {NULL}};
const Segment SGI38815[] = {{&S38739[30], 6}, {NULL}};
const Segment SGI38816[] = {{&S38739[119], 3}, {NULL}};
const Segment SGI38817[] = {{&S36595[44], 7}, {NULL}};
const Segment SGI38818[] = {{&S38739[85], 5}, {NULL}};
const Segment SGI38819[] = {{&S36595[44], 25}, {NULL}};
const Segment SGI38820[] = {{&S36595[30], 6}, {NULL}};
const Segment SGI38821[] = {{&S36595[57], 3}, {NULL}};
const Segment SGI38822[] = {{&S36595[83], 5}, {NULL}};
const Segment SGI38823[] = {{&S38748[38], 18}, {NULL}};
const Segment SGI38824[] = {{&S38748[31], 6}, {NULL}};
const Segment SGI38825[] = {{&S36678[44], 7}, {NULL}};
const Segment SGI38826[] = {{&S36678[44], 25}, {NULL}};
const Segment SGI38827[] = {{&S36678[30], 6}, {NULL}};
const Segment SGI38828[] = {{&S36678[57], 3}, {NULL}};
const Segment SGI38829[] = {{&S36678[83], 5}, {NULL}};
const Segment SGI38830[] = {{&S36679[44], 19}, {NULL}};
const Segment SGI38831[] = {{&S36679[44], 37}, {NULL}};
const Segment SGI38832[] = {{&S36679[30], 6}, {NULL}};
const Segment SGI38833[] = {{&S36679[69], 3}, {NULL}};
const Segment SGI38834[] = {{&S36679[95], 5}, {NULL}};
const Segment SGI38835[] = {{&S36681[44], 19}, {NULL}};
const Segment SGI38836[] = {{&S36681[44], 37}, {NULL}};
const Segment SGI38837[] = {{&S36681[30], 6}, {NULL}};
const Segment SGI38838[] = {{&S36681[69], 3}, {NULL}};
const Segment SGI38839[] = {{&S36681[95], 5}, {NULL}};
const Segment SGI38840[] = {{&S36682[44], 19}, {NULL}};
const Segment SGI38841[] = {{&S36682[44], 36}, {NULL}};
const Segment SGI38842[] = {{&S36682[30], 6}, {NULL}};
const Segment SGI38843[] = {{&S36682[69], 3}, {NULL}};
const Segment SGI38844[] = {{&S36682[94], 5}, {NULL}};
const Segment SGI38845[] = {{&S36683[44], 19}, {NULL}};
const Segment SGI38846[] = {{&S36683[44], 36}, {NULL}};
const Segment SGI38847[] = {{&S36683[30], 6}, {NULL}};
const Segment SGI38848[] = {{&S36683[69], 3}, {NULL}};
const Segment SGI38849[] = {{&S36683[94], 5}, {NULL}};
const Segment SGI38850[] = {{&S36684[44], 20}, {NULL}};
const Segment SGI38851[] = {{&S36684[44], 38}, {NULL}};
const Segment SGI38852[] = {{&S36684[30], 6}, {NULL}};
const Segment SGI38853[] = {{&S36684[70], 3}, {NULL}};
const Segment SGI38854[] = {{&S36684[96], 5}, {NULL}};
const Segment SGI38855[] = {{&S36685[44], 7}, {NULL}};
const Segment SGI38856[] = {{&S36685[44], 25}, {NULL}};
const Segment SGI38857[] = {{&S36685[30], 6}, {NULL}};
const Segment SGI38858[] = {{&S36685[57], 3}, {NULL}};
const Segment SGI38859[] = {{&S36685[83], 5}, {NULL}};
const Segment SGI38860[] = {{&S36686[38], 7}, {NULL}};
const Segment SGI38861[] = {{&S36686[31], 6}, {NULL}};
const Segment SGI38862[] = {{&S36687[44], 20}, {NULL}};
const Segment SGI38863[] = {{&S36687[44], 38}, {NULL}};
const Segment SGI38864[] = {{&S36687[30], 6}, {NULL}};
const Segment SGI38865[] = {{&S36687[70], 3}, {NULL}};
const Segment SGI38866[] = {{&S36687[96], 5}, {NULL}};
const Segment SGI38867[] = {{&S36689[44], 7}, {NULL}};
const Segment SGI38868[] = {{&S36689[44], 24}, {NULL}};
const Segment SGI38869[] = {{&S36689[30], 6}, {NULL}};
const Segment SGI38870[] = {{&S36812[44], 11}, {NULL}};
const Segment SGI38871[] = {{&S36689[57], 3}, {NULL}};
const Segment SGI38872[] = {{&S36812[44], 28}, {NULL}};
const Segment SGI38873[] = {{&S36812[30], 6}, {NULL}};
const Segment SGI38874[] = {{&S36689[82], 5}, {NULL}};
const Segment SGI38875[] = {{&S36812[120], 3}, {NULL}};
const Segment SGI38876[] = {{&S36812[86], 5}, {NULL}};
const Segment SGI38877[] = {{&S36690[44], 19}, {NULL}};
const Segment SGI38878[] = {{&S36690[44], 37}, {NULL}};
const Segment SGI38879[] = {{&S36690[30], 6}, {NULL}};
const Segment SGI38880[] = {{&S36813[44], 11}, {NULL}};
const Segment SGI38881[] = {{&S36813[44], 28}, {NULL}};
const Segment SGI38882[] = {{&S36813[30], 6}, {NULL}};
const Segment SGI38883[] = {{&S36690[69], 3}, {NULL}};
const Segment SGI38884[] = {{&S36813[120], 3}, {NULL}};
const Segment SGI38885[] = {{&S36690[95], 5}, {NULL}};
const Segment SGI38886[] = {{&S36813[86], 5}, {NULL}};
const Segment SGI38887[] = {{&S36814[38], 11}, {NULL}};
const Segment SGI38888[] = {{&S36814[31], 6}, {NULL}};
const Segment SGI38889[] = {{&S36692[44], 7}, {NULL}};
const Segment SGI38890[] = {{&S36692[44], 24}, {NULL}};
const Segment SGI38891[] = {{&S36692[30], 6}, {NULL}};
const Segment SGI38892[] = {{&S36692[57], 3}, {NULL}};
const Segment SGI38893[] = {{&S36815[44], 11}, {NULL}};
const Segment SGI38894[] = {{&S36692[82], 5}, {NULL}};
const Segment SGI38895[] = {{&S36815[44], 28}, {NULL}};
const Segment SGI38896[] = {{&S36815[30], 6}, {NULL}};
const Segment SGI38897[] = {{&S36815[86], 6}, {NULL}};
const Segment SGI38898[] = {{&S36819[38], 11}, {NULL}};
const Segment SGI38899[] = {{&S36819[31], 6}, {NULL}};
const Segment SGI38900[] = {{&S36820[42], 11}, {NULL}};
const Segment SGI38901[] = {{&S36820[30], 6}, {NULL}};
const Segment SGI38902[] = {{&S36820[118], 3}, {NULL}};
const Segment SGI38903[] = {{&S36820[84], 5}, {NULL}};
const Segment SGI38904[] = {{&S38934[44], 12}, {NULL}};
const Segment SGI38905[] = {{&S38934[44], 23}, {NULL}};
const Segment SGI38906[] = {{&S38934[30], 6}, {NULL}};
const Segment SGI38907[] = {{&S38934[115], 3}, {NULL}};
const Segment SGI38908[] = {{&S38934[81], 5}, {NULL}};
const Segment SGI38909[] = {{&S38936[44], 12}, {NULL}};
const Segment SGI38910[] = {{&S38936[44], 23}, {NULL}};
const Segment SGI38911[] = {{&S38936[30], 6}, {NULL}};
const Segment SGI38912[] = {{&S38936[115], 3}, {NULL}};
const Segment SGI38913[] = {{&S38936[81], 5}, {NULL}};
const Segment SGI38914[] = {{&S38938[38], 12}, {NULL}};
const Segment SGI38915[] = {{&S38938[31], 6}, {NULL}};
const Segment SGI38916[] = {{&S38948[44], 12}, {NULL}};
const Segment SGI38917[] = {{&S38948[44], 23}, {NULL}};
const Segment SGI38918[] = {{&S38948[30], 6}, {NULL}};
const Segment SGI38919[] = {{&S38948[84], 4}, {NULL}};
const Segment SGI38920[] = {{&S38948[84], 3}, {NULL}};
const Segment SGI38921[] = {{&S38949[44], 12}, {NULL}};
const Segment SGI38922[] = {{&S38949[44], 23}, {NULL}};
const Segment SGI38923[] = {{&S38949[30], 6}, {NULL}};
const Segment SGI38924[] = {{&S38949[84], 6}, {NULL}};
const Segment SGI38925[] = {{&S36962[44], 9}, {NULL}};
const Segment SGI38926[] = {{&S36962[44], 20}, {NULL}};
const Segment SGI38927[] = {{&S36962[30], 6}, {NULL}};
const Segment SGI38928[] = {{&S36962[112], 3}, {NULL}};
const Segment SGI38929[] = {{&S36962[78], 5}, {NULL}};
const Segment SGI38930[] = {{&S48995[44], 16}, {NULL}};
const Segment SGI38931[] = {{&S48995[44], 36}, {NULL}};
const Segment SGI38932[] = {{&S48995[30], 6}, {NULL}};
const Segment SGI38933[] = {{&S48995[128], 3}, {NULL}};
const Segment SGI38934[] = {{&S48995[94], 5}, {NULL}};
const Segment SGI38935[] = {{&S48996[44], 16}, {NULL}};
const Segment SGI38936[] = {{&S48996[44], 36}, {NULL}};
const Segment SGI38937[] = {{&S48996[30], 6}, {NULL}};
const Segment SGI38938[] = {{&S48996[128], 3}, {NULL}};
const Segment SGI38939[] = {{&S48996[94], 5}, {NULL}};
const Segment SGI38940[] = {{&S48997[38], 16}, {NULL}};
const Segment SGI38941[] = {{&S48997[31], 6}, {NULL}};
const Segment SGI38942[] = {{&S48998[44], 16}, {NULL}};
const Segment SGI38943[] = {{&S48998[44], 36}, {NULL}};
const Segment SGI38944[] = {{&S48998[30], 6}, {NULL}};
const Segment SGI38945[] = {{&S48998[128], 3}, {NULL}};
const Segment SGI38946[] = {{&S48998[94], 5}, {NULL}};
const Segment SGI38947[] = {{&S39176[44], 13}, {NULL}};
const Segment SGI38948[] = {{&S39176[44], 24}, {NULL}};
const Segment SGI38949[] = {{&S39176[30], 6}, {NULL}};
const Segment SGI38950[] = {{&S39176[116], 3}, {NULL}};
const Segment SGI38951[] = {{&S39176[82], 5}, {NULL}};
const Segment SGI38952[] = {{&S39180[44], 13}, {NULL}};
const Segment SGI38953[] = {{&S39180[44], 24}, {NULL}};
const Segment SGI38954[] = {{&S39180[30], 6}, {NULL}};
const Segment SGI38955[] = {{&S39180[116], 3}, {NULL}};
const Segment SGI38956[] = {{&S39180[82], 5}, {NULL}};
const Segment SGI38957[] = {{&S39186[44], 13}, {NULL}};
const Segment SGI38958[] = {{&S39186[44], 24}, {NULL}};
const Segment SGI38959[] = {{&S39186[30], 6}, {NULL}};
const Segment SGI38960[] = {{&S39186[116], 3}, {NULL}};
const Segment SGI38961[] = {{&S39186[82], 5}, {NULL}};
const Segment SGI38962[] = {{&S39195[38], 13}, {NULL}};
const Segment SGI38963[] = {{&S39195[31], 6}, {NULL}};
const Segment SGI38964[] = {{&S37087[44], 10}, {NULL}};
const Segment SGI38965[] = {{&S37087[44], 21}, {NULL}};
const Segment SGI38966[] = {{&S37087[30], 6}, {NULL}};
const Segment SGI38967[] = {{&S37087[113], 3}, {NULL}};
const Segment SGI38968[] = {{&S37087[79], 5}, {NULL}};
const Segment SGI38969[] = {{&S37090[38], 10}, {NULL}};
const Segment SGI38970[] = {{&S37090[31], 6}, {NULL}};
const Segment SGI38971[] = {{&S39206[44], 13}, {NULL}};
const Segment SGI38972[] = {{&S39206[44], 24}, {NULL}};
const Segment SGI38973[] = {{&S39206[30], 6}, {NULL}};
const Segment SGI38974[] = {{&S39206[85], 6}, {NULL}};
const Segment SGI38975[] = {{&S37213[44], 10}, {NULL}};
const Segment SGI38976[] = {{&S37213[44], 17}, {NULL}};
const Segment SGI38977[] = {{&S37213[30], 6}, {NULL}};
const Segment SGI38978[] = {{&S37213[109], 3}, {NULL}};
const Segment SGI38979[] = {{&S37213[75], 5}, {NULL}};
const Segment SGI38980[] = {{&S37216[44], 10}, {NULL}};
const Segment SGI38981[] = {{&S37216[44], 19}, {NULL}};
const Segment SGI38982[] = {{&S37216[30], 6}, {NULL}};
const Segment SGI38983[] = {{&S37216[111], 3}, {NULL}};
const Segment SGI38984[] = {{&S37216[77], 5}, {NULL}};
const Segment SGI38985[] = {{&S37217[44], 10}, {NULL}};
const Segment SGI38986[] = {{&S37217[44], 19}, {NULL}};
const Segment SGI38987[] = {{&S37217[30], 6}, {NULL}};
const Segment SGI38988[] = {{&S37217[111], 3}, {NULL}};
const Segment SGI38989[] = {{&S37217[77], 5}, {NULL}};
const Segment SGI38990[] = {{&S37218[44], 10}, {NULL}};
const Segment SGI38991[] = {{&S37218[44], 19}, {NULL}};
const Segment SGI38992[] = {{&S37218[30], 6}, {NULL}};
const Segment SGI38993[] = {{&S37218[111], 3}, {NULL}};
const Segment SGI38994[] = {{&S37218[77], 5}, {NULL}};
const Segment SGI38995[] = {{&S37219[44], 10}, {NULL}};
const Segment SGI38996[] = {{&S37219[44], 26}, {NULL}};
const Segment SGI38997[] = {{&S37219[30], 6}, {NULL}};
const Segment SGI38998[] = {{&S37219[118], 3}, {NULL}};
const Segment SGI38999[] = {{&S37219[84], 5}, {NULL}};
const Segment SGI39000[] = {{&S37222[44], 10}, {NULL}};
const Segment SGI39001[] = {{&S37222[44], 21}, {NULL}};
const Segment SGI39002[] = {{&S37222[30], 6}, {NULL}};
const Segment SGI39003[] = {{&S37222[113], 3}, {NULL}};
const Segment SGI39004[] = {{&S37222[79], 5}, {NULL}};
const Segment SGI39005[] = {{&S37223[44], 10}, {NULL}};
const Segment SGI39006[] = {{&S37223[44], 27}, {NULL}};
const Segment SGI39007[] = {{&S37223[30], 6}, {NULL}};
const Segment SGI39008[] = {{&S37223[119], 3}, {NULL}};
const Segment SGI39009[] = {{&S37223[85], 5}, {NULL}};
const Segment SGI39010[] = {{&S49267[42], 7}, {NULL}};
const Segment SGI39011[] = {{&S49267[30], 6}, {NULL}};
const Segment SGI39012[] = {{&S49267[109], 3}, {NULL}};
const Segment SGI39013[] = {{&S49267[75], 5}, {NULL}};
const Segment SGI39014[] = {{&S49268[44], 7}, {NULL}};
const Segment SGI39015[] = {{&S49268[44], 19}, {NULL}};
const Segment SGI39016[] = {{&S49268[30], 6}, {NULL}};
const Segment SGI39017[] = {{&S49268[111], 3}, {NULL}};
const Segment SGI39018[] = {{&S49268[77], 5}, {NULL}};
const Segment SGI39019[] = {{&S49273[38], 7}, {NULL}};
const Segment SGI39020[] = {{&S49273[31], 6}, {NULL}};
const Segment SGI39021[] = {{&S49275[44], 7}, {NULL}};
const Segment SGI39022[] = {{&S49275[44], 19}, {NULL}};
const Segment SGI39023[] = {{&S49275[30], 6}, {NULL}};
const Segment SGI39024[] = {{&S49275[113], 3}, {NULL}};
const Segment SGI39025[] = {{&S49275[77], 7}, {NULL}};
const Segment SGI39026[] = {{&S49275[133], 8}, {NULL}};
const Segment SGI39027[] = {{&S37212[44], 10}, {NULL}};
const Segment SGI39028[] = {{&S37212[44], 27}, {NULL}};
const Segment SGI39029[] = {{&S37212[30], 6}, {NULL}};
const Segment SGI39030[] = {{&S37212[119], 3}, {NULL}};
const Segment SGI39031[] = {{&S37212[85], 5}, {NULL}};
const Segment SGI39032[] = {{&S49328[44], 7}, {NULL}};
const Segment SGI39033[] = {{&S49328[44], 19}, {NULL}};
const Segment SGI39034[] = {{&S49328[30], 6}, {NULL}};
const Segment SGI39035[] = {{&S49328[111], 3}, {NULL}};
const Segment SGI39036[] = {{&S49328[77], 5}, {NULL}};
const Segment SGI39037[] = {{&S49329[44], 7}, {NULL}};
const Segment SGI39038[] = {{&S49329[44], 19}, {NULL}};
const Segment SGI39039[] = {{&S49329[30], 6}, {NULL}};
const Segment SGI39040[] = {{&S49329[111], 3}, {NULL}};
const Segment SGI39041[] = {{&S49329[77], 5}, {NULL}};
const Segment SGI39042[] = {{&S49330[38], 7}, {NULL}};
const Segment SGI39043[] = {{&S49330[31], 6}, {NULL}};
const Segment SGI39044[] = {{&S49331[38], 7}, {NULL}};
const Segment SGI39045[] = {{&S49331[31], 6}, {NULL}};
const Segment SGI39046[] = {{&S37338[44], 11}, {NULL}};
const Segment SGI39047[] = {{&S37338[44], 22}, {NULL}};
const Segment SGI39048[] = {{&S37338[30], 6}, {NULL}};
const Segment SGI39049[] = {{&S37338[114], 3}, {NULL}};
const Segment SGI39050[] = {{&S37338[80], 5}, {NULL}};
const Segment SGI39051[] = {{&S37339[44], 14}, {NULL}};
const Segment SGI39052[] = {{&S37339[44], 25}, {NULL}};
const Segment SGI39053[] = {{&S37339[30], 6}, {NULL}};
const Segment SGI39054[] = {{&S37339[117], 3}, {NULL}};
const Segment SGI39055[] = {{&S37339[83], 5}, {NULL}};
const Segment SGI39056[] = {{&S37341[44], 20}, {NULL}};
const Segment SGI39057[] = {{&S37341[44], 33}, {NULL}};
const Segment SGI39058[] = {{&S37341[30], 6}, {NULL}};
const Segment SGI39059[] = {{&S37341[125], 3}, {NULL}};
const Segment SGI39060[] = {{&S37341[91], 5}, {NULL}};
const Segment SGI39061[] = {{&S37342[44], 20}, {NULL}};
const Segment SGI39062[] = {{&S37342[44], 33}, {NULL}};
const Segment SGI39063[] = {{&S37342[30], 6}, {NULL}};
const Segment SGI39064[] = {{&S37342[125], 3}, {NULL}};
const Segment SGI39065[] = {{&S37342[91], 5}, {NULL}};
const Segment SGI39066[] = {{&S37343[44], 20}, {NULL}};
const Segment SGI39067[] = {{&S37343[44], 33}, {NULL}};
const Segment SGI39068[] = {{&S37343[30], 6}, {NULL}};
const Segment SGI39069[] = {{&S37343[125], 3}, {NULL}};
const Segment SGI39070[] = {{&S37343[91], 5}, {NULL}};
const Segment SGI39071[] = {{&S37344[44], 11}, {NULL}};
const Segment SGI39072[] = {{&S37344[44], 26}, {NULL}};
const Segment SGI39073[] = {{&S37344[30], 6}, {NULL}};
const Segment SGI39074[] = {{&S37344[118], 3}, {NULL}};
const Segment SGI39075[] = {{&S37344[84], 5}, {NULL}};
const Segment SGI39076[] = {{&S37345[44], 20}, {NULL}};
const Segment SGI39077[] = {{&S37345[44], 33}, {NULL}};
const Segment SGI39078[] = {{&S37345[30], 6}, {NULL}};
const Segment SGI39079[] = {{&S37345[125], 3}, {NULL}};
const Segment SGI39080[] = {{&S37345[91], 5}, {NULL}};
const Segment SGI39081[] = {{&S37346[44], 20}, {NULL}};
const Segment SGI39082[] = {{&S37346[44], 33}, {NULL}};
const Segment SGI39083[] = {{&S37346[30], 6}, {NULL}};
const Segment SGI39084[] = {{&S37346[125], 3}, {NULL}};
const Segment SGI39085[] = {{&S37346[91], 5}, {NULL}};
const Segment SGI39086[] = {{&S37347[44], 20}, {NULL}};
const Segment SGI39087[] = {{&S37347[44], 33}, {NULL}};
const Segment SGI39088[] = {{&S37347[30], 6}, {NULL}};
const Segment SGI39089[] = {{&S37347[125], 3}, {NULL}};
const Segment SGI39090[] = {{&S37347[91], 5}, {NULL}};
const Segment SGI39091[] = {{&S37348[44], 11}, {NULL}};
const Segment SGI39092[] = {{&S37348[44], 22}, {NULL}};
const Segment SGI39093[] = {{&S37348[30], 6}, {NULL}};
const Segment SGI39094[] = {{&S37348[114], 3}, {NULL}};
const Segment SGI39095[] = {{&S37348[80], 5}, {NULL}};
const Segment SGI39096[] = {{&S37350[44], 11}, {NULL}};
const Segment SGI39097[] = {{&S37350[44], 22}, {NULL}};
const Segment SGI39098[] = {{&S37350[30], 6}, {NULL}};
const Segment SGI39099[] = {{&S37350[114], 3}, {NULL}};
const Segment SGI39100[] = {{&S37350[80], 5}, {NULL}};
const Segment SGI39101[] = {{&S37351[44], 20}, {NULL}};
const Segment SGI39102[] = {{&S37351[44], 33}, {NULL}};
const Segment SGI39103[] = {{&S37351[30], 6}, {NULL}};
const Segment SGI39104[] = {{&S37351[125], 3}, {NULL}};
const Segment SGI39105[] = {{&S37351[91], 5}, {NULL}};
const Segment SGI39106[] = {{&S37352[44], 11}, {NULL}};
const Segment SGI39107[] = {{&S37352[44], 26}, {NULL}};
const Segment SGI39108[] = {{&S37352[30], 6}, {NULL}};
const Segment SGI39109[] = {{&S37352[118], 3}, {NULL}};
const Segment SGI39110[] = {{&S37352[84], 5}, {NULL}};
const Segment SGI39111[] = {{&S37353[44], 11}, {NULL}};
const Segment SGI39112[] = {{&S37353[44], 22}, {NULL}};
const Segment SGI39113[] = {{&S37353[30], 6}, {NULL}};
const Segment SGI39114[] = {{&S37353[114], 3}, {NULL}};
const Segment SGI39115[] = {{&S37353[80], 5}, {NULL}};
const Segment SGI39116[] = {{&S37356[38], 11}, {NULL}};
const Segment SGI39117[] = {{&S37356[31], 6}, {NULL}};
const Segment SGI39118[] = {{&S37359[44], 20}, {NULL}};
const Segment SGI39119[] = {{&S37359[44], 33}, {NULL}};
const Segment SGI39120[] = {{&S37359[30], 6}, {NULL}};
const Segment SGI39121[] = {{&S37359[125], 3}, {NULL}};
const Segment SGI39122[] = {{&S37359[91], 5}, {NULL}};
const Segment SGI39123[] = {{&S37360[44], 20}, {NULL}};
const Segment SGI39124[] = {{&S37360[44], 33}, {NULL}};
const Segment SGI39125[] = {{&S37360[30], 6}, {NULL}};
const Segment SGI39126[] = {{&S37360[125], 3}, {NULL}};
const Segment SGI39127[] = {{&S37360[91], 5}, {NULL}};
const Segment SGI39128[] = {{&S37361[38], 20}, {NULL}};
const Segment SGI39129[] = {{&S37361[31], 6}, {NULL}};
const Segment SGI39130[] = {{&S37362[44], 20}, {NULL}};
const Segment SGI39131[] = {{&S37362[44], 33}, {NULL}};
const Segment SGI39132[] = {{&S37362[30], 6}, {NULL}};
const Segment SGI39133[] = {{&S37362[125], 3}, {NULL}};
const Segment SGI39134[] = {{&S37362[91], 5}, {NULL}};
const Segment SGI39135[] = {{&S37364[44], 11}, {NULL}};
const Segment SGI39136[] = {{&S37364[44], 24}, {NULL}};
const Segment SGI39137[] = {{&S37364[30], 6}, {NULL}};
const Segment SGI39138[] = {{&S37364[116], 3}, {NULL}};
const Segment SGI39139[] = {{&S37364[82], 5}, {NULL}};
const Segment SGI39140[] = {{&S37365[44], 11}, {NULL}};
const Segment SGI39141[] = {{&S37365[44], 22}, {NULL}};
const Segment SGI39142[] = {{&S37365[30], 6}, {NULL}};
const Segment SGI39143[] = {{&S37365[114], 3}, {NULL}};
const Segment SGI39144[] = {{&S37365[80], 5}, {NULL}};
const Segment SGI39145[] = {{&S37366[44], 12}, {NULL}};
const Segment SGI39146[] = {{&S37366[44], 23}, {NULL}};
const Segment SGI39147[] = {{&S37366[30], 6}, {NULL}};
const Segment SGI39148[] = {{&S37366[115], 3}, {NULL}};
const Segment SGI39149[] = {{&S37366[81], 5}, {NULL}};
const Segment SGI39150[] = {{&S37368[44], 20}, {NULL}};
const Segment SGI39151[] = {{&S37368[44], 33}, {NULL}};
const Segment SGI39152[] = {{&S37368[30], 6}, {NULL}};
const Segment SGI39153[] = {{&S37368[125], 3}, {NULL}};
const Segment SGI39154[] = {{&S37368[91], 5}, {NULL}};
const Segment SGI39155[] = {{&S37377[44], 18}, {NULL}};
const Segment SGI39156[] = {{&S37377[44], 29}, {NULL}};
const Segment SGI39157[] = {{&S37377[30], 6}, {NULL}};
const Segment SGI39158[] = {{&S37377[121], 3}, {NULL}};
const Segment SGI39159[] = {{&S37377[87], 5}, {NULL}};
const Segment SGI39160[] = {{&S37335[44], 20}, {NULL}};
const Segment SGI39161[] = {{&S37335[44], 33}, {NULL}};
const Segment SGI39162[] = {{&S37335[30], 6}, {NULL}};
const Segment SGI39163[] = {{&S37335[125], 3}, {NULL}};
const Segment SGI39164[] = {{&S37335[91], 5}, {NULL}};
const Segment SGI39165[] = {{&S44380[44], 9}, {NULL}};
const Segment SGI39166[] = {{&S44380[44], 20}, {NULL}};
const Segment SGI39167[] = {{&S44380[30], 6}, {NULL}};
const Segment SGI39168[] = {{&S44380[112], 3}, {NULL}};
const Segment SGI39169[] = {{&S37336[44], 20}, {NULL}};
const Segment SGI39170[] = {{&S44380[78], 5}, {NULL}};
const Segment SGI39171[] = {{&S37336[44], 33}, {NULL}};
const Segment SGI39172[] = {{&S37336[30], 6}, {NULL}};
const Segment SGI39173[] = {{&S37336[125], 3}, {NULL}};
const Segment SGI39174[] = {{&S37336[91], 5}, {NULL}};
const Segment SGI39175[] = {{&S37337[44], 20}, {NULL}};
const Segment SGI39176[] = {{&S37337[44], 33}, {NULL}};
const Segment SGI39177[] = {{&S37337[30], 6}, {NULL}};
const Segment SGI39178[] = {{&S37337[125], 3}, {NULL}};
const Segment SGI39179[] = {{&S37337[91], 5}, {NULL}};
const Segment SGI39180[] = {{&S37340[44], 20}, {NULL}};
const Segment SGI39181[] = {{&S37340[44], 33}, {NULL}};
const Segment SGI39182[] = {{&S37340[30], 6}, {NULL}};
const Segment SGI39183[] = {{&S37340[125], 3}, {NULL}};
const Segment SGI39184[] = {{&S37340[91], 5}, {NULL}};
const Segment SGI39185[] = {{&S37349[44], 11}, {NULL}};
const Segment SGI39186[] = {{&S37349[44], 24}, {NULL}};
const Segment SGI39187[] = {{&S37349[30], 6}, {NULL}};
const Segment SGI39188[] = {{&S37349[116], 3}, {NULL}};
const Segment SGI39189[] = {{&S37349[82], 5}, {NULL}};
const Segment SGI39190[] = {{&S44397[44], 9}, {NULL}};
const Segment SGI39191[] = {{&S44397[44], 20}, {NULL}};
const Segment SGI39192[] = {{&S44397[30], 6}, {NULL}};
const Segment SGI39193[] = {{&S44397[112], 3}, {NULL}};
const Segment SGI39194[] = {{&S44397[78], 5}, {NULL}};
const Segment SGI39195[] = {{&S37354[38], 20}, {NULL}};
const Segment SGI39196[] = {{&S37354[31], 6}, {NULL}};
const Segment SGI39197[] = {{&S44403[44], 9}, {NULL}};
const Segment SGI39198[] = {{&S44403[44], 20}, {NULL}};
const Segment SGI39199[] = {{&S37355[44], 7}, {NULL}};
const Segment SGI39200[] = {{&S44403[30], 6}, {NULL}};
const Segment SGI39201[] = {{&S37355[44], 15}, {NULL}};
const Segment SGI39202[] = {{&S37355[30], 6}, {NULL}};
const Segment SGI39203[] = {{&S44403[112], 3}, {NULL}};
const Segment SGI39204[] = {{&S37355[107], 3}, {NULL}};
const Segment SGI39205[] = {{&S44403[78], 5}, {NULL}};
const Segment SGI39206[] = {{&S37355[73], 5}, {NULL}};
const Segment SGI39207[] = {{&S37358[44], 11}, {NULL}};
const Segment SGI39208[] = {{&S37358[44], 22}, {NULL}};
const Segment SGI39209[] = {{&S37358[30], 6}, {NULL}};
const Segment SGI39210[] = {{&S37358[114], 3}, {NULL}};
const Segment SGI39211[] = {{&S37358[80], 5}, {NULL}};
const Segment SGI39212[] = {{&S37591[44], 18}, {NULL}};
const Segment SGI39213[] = {{&S37591[44], 35}, {NULL}};
const Segment SGI39214[] = {{&S37591[30], 6}, {NULL}};
const Segment SGI39215[] = {{&S44409[38], 9}, {NULL}};
const Segment SGI39216[] = {{&S44409[31], 6}, {NULL}};
const Segment SGI39217[] = {{&S37591[127], 3}, {NULL}};
const Segment SGI39218[] = {{&S37591[93], 5}, {NULL}};
const Segment SGI39219[] = {{&S44410[44], 5}, {NULL}};
const Segment SGI39220[] = {{&S44410[44], 16}, {NULL}};
const Segment SGI39221[] = {{&S44410[30], 6}, {NULL}};
const Segment SGI39222[] = {{&S37592[38], 18}, {NULL}};
const Segment SGI39223[] = {{&S37592[31], 6}, {NULL}};
const Segment SGI39224[] = {{&S44410[108], 3}, {NULL}};
const Segment SGI39225[] = {{&S44410[74], 5}, {NULL}};
const Segment SGI39226[] = {{&S37363[44], 14}, {NULL}};
const Segment SGI39227[] = {{&S37363[44], 25}, {NULL}};
const Segment SGI39228[] = {{&S37363[30], 6}, {NULL}};
const Segment SGI39229[] = {{&S37363[117], 3}, {NULL}};
const Segment SGI39230[] = {{&S37363[83], 5}, {NULL}};
const Segment SGI39231[] = {{&S37376[44], 18}, {NULL}};
const Segment SGI39232[] = {{&S37376[44], 29}, {NULL}};
const Segment SGI39233[] = {{&S37376[30], 6}, {NULL}};
const Segment SGI39234[] = {{&S37376[121], 3}, {NULL}};
const Segment SGI39235[] = {{&S37376[87], 5}, {NULL}};
const Segment SGI39236[] = {{&S37624[38], 6}, {NULL}};
const Segment SGI39237[] = {{&S37624[31], 6}, {NULL}};
const Segment SGI39238[] = {{&S37625[44], 6}, {NULL}};
const Segment SGI39239[] = {{&S37625[44], 22}, {NULL}};
const Segment SGI39240[] = {{&S37625[30], 6}, {NULL}};
const Segment SGI39241[] = {{&S37625[114], 3}, {NULL}};
const Segment SGI39242[] = {{&S37625[80], 5}, {NULL}};
const Segment SGI39243[] = {{&S37631[44], 7}, {NULL}};
const Segment SGI39244[] = {{&S37631[44], 30}, {NULL}};
const Segment SGI39245[] = {{&S37631[30], 6}, {NULL}};
const Segment SGI39246[] = {{&S37631[122], 3}, {NULL}};
const Segment SGI39247[] = {{&S37631[88], 5}, {NULL}};
const Segment SGI39248[] = {{&S37632[44], 7}, {NULL}};
const Segment SGI39249[] = {{&S37632[44], 30}, {NULL}};
const Segment SGI39250[] = {{&S37632[30], 6}, {NULL}};
const Segment SGI39251[] = {{&S37632[122], 3}, {NULL}};
const Segment SGI39252[] = {{&S37632[88], 5}, {NULL}};
const Segment SGI39253[] = {{&S37633[44], 7}, {NULL}};
const Segment SGI39254[] = {{&S37633[44], 30}, {NULL}};
const Segment SGI39255[] = {{&S37633[30], 6}, {NULL}};
const Segment SGI39256[] = {{&S37633[122], 3}, {NULL}};
const Segment SGI39257[] = {{&S37633[88], 5}, {NULL}};
const Segment SGI39258[] = {{&S37634[44], 7}, {NULL}};
const Segment SGI39259[] = {{&S37634[44], 30}, {NULL}};
const Segment SGI39260[] = {{&S37634[30], 6}, {NULL}};
const Segment SGI39261[] = {{&S37634[122], 3}, {NULL}};
const Segment SGI39262[] = {{&S37634[88], 5}, {NULL}};
const Segment SGI39263[] = {{&S37635[44], 7}, {NULL}};
const Segment SGI39264[] = {{&S37635[44], 30}, {NULL}};
const Segment SGI39265[] = {{&S37635[30], 6}, {NULL}};
const Segment SGI39266[] = {{&S37635[122], 3}, {NULL}};
const Segment SGI39267[] = {{&S37635[88], 5}, {NULL}};
const Segment SGI39268[] = {{&S37636[44], 7}, {NULL}};
const Segment SGI39269[] = {{&S37636[44], 30}, {NULL}};
const Segment SGI39270[] = {{&S37636[30], 6}, {NULL}};
const Segment SGI39271[] = {{&S37636[122], 3}, {NULL}};
const Segment SGI39272[] = {{&S37636[88], 5}, {NULL}};
const Segment SGI39273[] = {{&S37637[38], 7}, {NULL}};
const Segment SGI39274[] = {{&S37637[31], 6}, {NULL}};
const Segment SGI39275[] = {{&S37662[44], 11}, {NULL}};
const Segment SGI39276[] = {{&S37662[44], 21}, {NULL}};
const Segment SGI39277[] = {{&S37662[30], 6}, {NULL}};
const Segment SGI39278[] = {{&S37662[113], 3}, {NULL}};
const Segment SGI39279[] = {{&S37662[79], 5}, {NULL}};
const Segment SGI39280[] = {{&S37682[44], 13}, {NULL}};
const Segment SGI39281[] = {{&S37682[44], 24}, {NULL}};
const Segment SGI39282[] = {{&S37682[30], 6}, {NULL}};
const Segment SGI39283[] = {{&S37471[44], 12}, {NULL}};
const Segment SGI39284[] = {{&S37471[44], 23}, {NULL}};
const Segment SGI39285[] = {{&S37682[116], 3}, {NULL}};
const Segment SGI39286[] = {{&S37471[30], 6}, {NULL}};
const Segment SGI39287[] = {{&S37682[82], 5}, {NULL}};
const Segment SGI39288[] = {{&S37471[115], 3}, {NULL}};
const Segment SGI39289[] = {{&S37471[81], 5}, {NULL}};
const Segment SGI39290[] = {{&S37683[44], 9}, {NULL}};
const Segment SGI39291[] = {{&S37472[43], 13}, {NULL}};
const Segment SGI39292[] = {{&S37683[44], 20}, {NULL}};
const Segment SGI39293[] = {{&S37472[43], 24}, {NULL}};
const Segment SGI39294[] = {{&S37683[30], 6}, {NULL}};
const Segment SGI39295[] = {{&S37472[115], 3}, {NULL}};
const Segment SGI39296[] = {{&S37683[112], 3}, {NULL}};
const Segment SGI39297[] = {{&S37472[81], 5}, {NULL}};
const Segment SGI39298[] = {{&S37683[78], 5}, {NULL}};
const Segment SGI39299[] = {{&S37684[44], 13}, {NULL}};
const Segment SGI39300[] = {{&S37684[44], 24}, {NULL}};
const Segment SGI39301[] = {{&S37684[30], 6}, {NULL}};
const Segment SGI39302[] = {{&S37684[116], 3}, {NULL}};
const Segment SGI39303[] = {{&S37474[44], 13}, {NULL}};
const Segment SGI39304[] = {{&S37474[44], 24}, {NULL}};
const Segment SGI39305[] = {{&S37684[82], 5}, {NULL}};
const Segment SGI39306[] = {{&S37474[30], 6}, {NULL}};
const Segment SGI39307[] = {{&S37474[116], 3}, {NULL}};
const Segment SGI39308[] = {{&S37474[82], 5}, {NULL}};
const Segment SGI39309[] = {{&S37685[38], 9}, {NULL}};
const Segment SGI39310[] = {{&S37685[31], 6}, {NULL}};
const Segment SGI39311[] = {{&S37475[44], 12}, {NULL}};
const Segment SGI39312[] = {{&S37475[44], 23}, {NULL}};
const Segment SGI39313[] = {{&S37475[30], 6}, {NULL}};
const Segment SGI39314[] = {{&S37475[115], 3}, {NULL}};
const Segment SGI39315[] = {{&S37475[81], 5}, {NULL}};
const Segment SGI39316[] = {{&S37686[44], 14}, {NULL}};
const Segment SGI39317[] = {{&S37686[44], 25}, {NULL}};
const Segment SGI39318[] = {{&S37686[30], 6}, {NULL}};
const Segment SGI39319[] = {{&S37686[117], 3}, {NULL}};
const Segment SGI39320[] = {{&S37686[83], 5}, {NULL}};
const Segment SGI39321[] = {{&S37476[44], 12}, {NULL}};
const Segment SGI39322[] = {{&S37476[44], 23}, {NULL}};
const Segment SGI39323[] = {{&S37476[30], 6}, {NULL}};
const Segment SGI39324[] = {{&S37476[115], 3}, {NULL}};
const Segment SGI39325[] = {{&S37476[81], 5}, {NULL}};
const Segment SGI39326[] = {{&S37687[44], 14}, {NULL}};
const Segment SGI39327[] = {{&S37687[44], 25}, {NULL}};
const Segment SGI39328[] = {{&S37687[30], 6}, {NULL}};
const Segment SGI39329[] = {{&S37687[117], 3}, {NULL}};
const Segment SGI39330[] = {{&S37477[43], 13}, {NULL}};
const Segment SGI39331[] = {{&S37687[83], 5}, {NULL}};
const Segment SGI39332[] = {{&S37477[43], 24}, {NULL}};
const Segment SGI39333[] = {{&S37477[30], 5}, {NULL}};
const Segment SGI39334[] = {{&S37477[115], 3}, {NULL}};
const Segment SGI39335[] = {{&S37477[81], 5}, {NULL}};
const Segment SGI39336[] = {{&S37689[44], 10}, {NULL}};
const Segment SGI39337[] = {{&S37689[44], 21}, {NULL}};
const Segment SGI39338[] = {{&S37689[30], 6}, {NULL}};
const Segment SGI39339[] = {{&S37689[113], 3}, {NULL}};
const Segment SGI39340[] = {{&S37689[79], 5}, {NULL}};
const Segment SGI39341[] = {{&S37690[44], 13}, {NULL}};
const Segment SGI39342[] = {{&S37690[44], 24}, {NULL}};
const Segment SGI39343[] = {{&S37690[30], 6}, {NULL}};
const Segment SGI39344[] = {{&S37481[44], 13}, {NULL}};
const Segment SGI39345[] = {{&S37481[44], 24}, {NULL}};
const Segment SGI39346[] = {{&S37481[30], 6}, {NULL}};
const Segment SGI39347[] = {{&S37690[116], 3}, {NULL}};
const Segment SGI39348[] = {{&S37481[116], 3}, {NULL}};
const Segment SGI39349[] = {{&S37690[82], 5}, {NULL}};
const Segment SGI39350[] = {{&S37481[82], 5}, {NULL}};
const Segment SGI39351[] = {{&S37691[44], 13}, {NULL}};
const Segment SGI39352[] = {{&S37482[42], 13}, {NULL}};
const Segment SGI39353[] = {{&S37691[44], 24}, {NULL}};
const Segment SGI39354[] = {{&S37691[30], 6}, {NULL}};
const Segment SGI39355[] = {{&S37482[30], 6}, {NULL}};
const Segment SGI39356[] = {{&S37482[114], 3}, {NULL}};
const Segment SGI39357[] = {{&S37691[116], 3}, {NULL}};
const Segment SGI39358[] = {{&S37482[80], 5}, {NULL}};
const Segment SGI39359[] = {{&S37691[82], 5}, {NULL}};
const Segment SGI39360[] = {{&S37484[38], 13}, {NULL}};
const Segment SGI39361[] = {{&S37484[31], 6}, {NULL}};
const Segment SGI39362[] = {{&S37694[44], 14}, {NULL}};
const Segment SGI39363[] = {{&S37694[44], 25}, {NULL}};
const Segment SGI39364[] = {{&S37694[30], 6}, {NULL}};
const Segment SGI39365[] = {{&S37486[44], 13}, {NULL}};
const Segment SGI39366[] = {{&S37486[44], 24}, {NULL}};
const Segment SGI39367[] = {{&S37486[30], 6}, {NULL}};
const Segment SGI39368[] = {{&S37694[117], 3}, {NULL}};
const Segment SGI39369[] = {{&S37486[118], 3}, {NULL}};
const Segment SGI39370[] = {{&S37694[83], 5}, {NULL}};
const Segment SGI39371[] = {{&S37486[82], 7}, {NULL}};
const Segment SGI39372[] = {{&S37486[138], 8}, {NULL}};
const Segment SGI39373[] = {{&S37695[44], 14}, {NULL}};
const Segment SGI39374[] = {{&S37487[44], 13}, {NULL}};
const Segment SGI39375[] = {{&S37695[44], 25}, {NULL}};
const Segment SGI39376[] = {{&S37487[44], 24}, {NULL}};
const Segment SGI39377[] = {{&S37695[30], 6}, {NULL}};
const Segment SGI39378[] = {{&S37487[30], 6}, {NULL}};
const Segment SGI39379[] = {{&S37695[117], 3}, {NULL}};
const Segment SGI39380[] = {{&S37487[82], 6}, {NULL}};
const Segment SGI39381[] = {{&S37695[83], 5}, {NULL}};
const Segment SGI39382[] = {{&S37696[44], 9}, {NULL}};
const Segment SGI39383[] = {{&S37696[44], 20}, {NULL}};
const Segment SGI39384[] = {{&S37696[30], 6}, {NULL}};
const Segment SGI39385[] = {{&S37696[113], 3}, {NULL}};
const Segment SGI39386[] = {{&S37696[78], 6}, {NULL}};
const Segment SGI39387[] = {{&S37696[133], 8}, {NULL}};
const Segment SGI39388[] = {{&S37697[44], 9}, {NULL}};
const Segment SGI39389[] = {{&S37697[44], 41}, {NULL}};
const Segment SGI39390[] = {{&S37697[30], 6}, {NULL}};
const Segment SGI39391[] = {{&S37697[133], 3}, {NULL}};
const Segment SGI39392[] = {{&S37697[99], 5}, {NULL}};
const Segment SGI39393[] = {{&S37698[44], 16}, {NULL}};
const Segment SGI39394[] = {{&S37698[44], 27}, {NULL}};
const Segment SGI39395[] = {{&S37698[30], 6}, {NULL}};
const Segment SGI39396[] = {{&S37698[119], 3}, {NULL}};
const Segment SGI39397[] = {{&S37698[85], 5}, {NULL}};
const Segment SGI39398[] = {{&S37699[44], 9}, {NULL}};
const Segment SGI39399[] = {{&S37699[44], 20}, {NULL}};
const Segment SGI39400[] = {{&S37699[30], 6}, {NULL}};
const Segment SGI39401[] = {{&S37699[81], 6}, {NULL}};
const Segment SGI39402[] = {{&S37700[44], 13}, {NULL}};
const Segment SGI39403[] = {{&S37700[44], 24}, {NULL}};
const Segment SGI39404[] = {{&S37700[30], 6}, {NULL}};
const Segment SGI39405[] = {{&S37700[116], 3}, {NULL}};
const Segment SGI39406[] = {{&S37700[82], 5}, {NULL}};
const Segment SGI39407[] = {{&S37701[44], 9}, {NULL}};
const Segment SGI39408[] = {{&S37701[44], 41}, {NULL}};
const Segment SGI39409[] = {{&S37701[30], 6}, {NULL}};
const Segment SGI39410[] = {{&S37701[133], 3}, {NULL}};
const Segment SGI39411[] = {{&S37701[99], 5}, {NULL}};
const Segment SGI39412[] = {{&S37702[44], 14}, {NULL}};
const Segment SGI39413[] = {{&S37702[44], 25}, {NULL}};
const Segment SGI39414[] = {{&S37702[30], 6}, {NULL}};
const Segment SGI39415[] = {{&S37702[117], 3}, {NULL}};
const Segment SGI39416[] = {{&S37702[83], 5}, {NULL}};
const Segment SGI39417[] = {{&S49755[44], 11}, {NULL}};
const Segment SGI39418[] = {{&S49755[44], 22}, {NULL}};
const Segment SGI39419[] = {{&S49755[30], 6}, {NULL}};
const Segment SGI39420[] = {{&S49755[114], 3}, {NULL}};
const Segment SGI39421[] = {{&S49755[80], 5}, {NULL}};
const Segment SGI39422[] = {{&S49756[38], 11}, {NULL}};
const Segment SGI39423[] = {{&S49756[31], 6}, {NULL}};
const Segment SGI39424[] = {{&S49775[44], 9}, {NULL}};
const Segment SGI39425[] = {{&S49775[44], 21}, {NULL}};
const Segment SGI39426[] = {{&S49775[30], 6}, {NULL}};
const Segment SGI39427[] = {{&S49775[113], 3}, {NULL}};
const Segment SGI39428[] = {{&S49775[79], 5}, {NULL}};
const Segment SGI39429[] = {{&S49776[38], 9}, {NULL}};
const Segment SGI39430[] = {{&S49776[31], 6}, {NULL}};
const Segment SGI39431[] = {{&S49777[44], 8}, {NULL}};
const Segment SGI39432[] = {{&S49777[44], 20}, {NULL}};
const Segment SGI39433[] = {{&S49777[30], 6}, {NULL}};
const Segment SGI39434[] = {{&S49777[112], 3}, {NULL}};
const Segment SGI39435[] = {{&S49777[78], 5}, {NULL}};
const Segment SGI39436[] = {{&S49778[44], 9}, {NULL}};
const Segment SGI39437[] = {{&S49778[44], 21}, {NULL}};
const Segment SGI39438[] = {{&S49778[30], 6}, {NULL}};
const Segment SGI39439[] = {{&S49778[113], 3}, {NULL}};
const Segment SGI39440[] = {{&S49778[79], 5}, {NULL}};
const Segment SGI39441[] = {{&S49779[43], 9}, {NULL}};
const Segment SGI39442[] = {{&S49779[43], 21}, {NULL}};
const Segment SGI39443[] = {{&S49779[30], 5}, {NULL}};
const Segment SGI39444[] = {{&S49779[112], 3}, {NULL}};
const Segment SGI39445[] = {{&S49779[78], 5}, {NULL}};
const Segment SGI39446[] = {{&S39775[44], 7}, {NULL}};
const Segment SGI39447[] = {{&S39775[44], 21}, {NULL}};
const Segment SGI39448[] = {{&S39775[30], 6}, {NULL}};
const Segment SGI39449[] = {{&S39775[113], 3}, {NULL}};
const Segment SGI39450[] = {{&S39775[79], 5}, {NULL}};
const Segment SGI39451[] = {{&S39778[44], 7}, {NULL}};
const Segment SGI39452[] = {{&S39778[44], 22}, {NULL}};
const Segment SGI39453[] = {{&S39778[30], 6}, {NULL}};
const Segment SGI39454[] = {{&S39778[114], 3}, {NULL}};
const Segment SGI39455[] = {{&S39778[80], 5}, {NULL}};
const Segment SGI39456[] = {{&S39801[44], 7}, {NULL}};
const Segment SGI39457[] = {{&S39801[44], 19}, {NULL}};
const Segment SGI39458[] = {{&S39801[30], 6}, {NULL}};
const Segment SGI39459[] = {{&S39801[111], 3}, {NULL}};
const Segment SGI39460[] = {{&S39801[77], 5}, {NULL}};
const Segment SGI39461[] = {{&S37590[44], 18}, {NULL}};
const Segment SGI39462[] = {{&S37590[44], 35}, {NULL}};
const Segment SGI39463[] = {{&S37590[30], 6}, {NULL}};
const Segment SGI39464[] = {{&S37590[127], 3}, {NULL}};
const Segment SGI39465[] = {{&S37590[93], 5}, {NULL}};
const Segment SGI39466[] = {{&S37649[44], 11}, {NULL}};
const Segment SGI39467[] = {{&S37649[44], 33}, {NULL}};
const Segment SGI39468[] = {{&S37649[30], 6}, {NULL}};
const Segment SGI39469[] = {{&S37649[125], 3}, {NULL}};
const Segment SGI39470[] = {{&S37649[91], 5}, {NULL}};
const Segment SGI39471[] = {{&S37651[44], 11}, {NULL}};
const Segment SGI39472[] = {{&S37651[44], 33}, {NULL}};
const Segment SGI39473[] = {{&S37651[30], 6}, {NULL}};
const Segment SGI39474[] = {{&S37651[125], 3}, {NULL}};
const Segment SGI39475[] = {{&S37651[91], 5}, {NULL}};
const Segment SGI39476[] = {{&S37656[44], 10}, {NULL}};
const Segment SGI39477[] = {{&S37656[44], 32}, {NULL}};
const Segment SGI39478[] = {{&S37656[30], 6}, {NULL}};
const Segment SGI39479[] = {{&S37656[124], 3}, {NULL}};
const Segment SGI39480[] = {{&S37656[90], 5}, {NULL}};
const Segment SGI39481[] = {{&S37657[44], 10}, {NULL}};
const Segment SGI39482[] = {{&S37657[44], 32}, {NULL}};
const Segment SGI39483[] = {{&S37657[30], 6}, {NULL}};
const Segment SGI39484[] = {{&S37657[124], 3}, {NULL}};
const Segment SGI39485[] = {{&S37657[90], 5}, {NULL}};
const Segment SGI39486[] = {{&S37658[44], 10}, {NULL}};
const Segment SGI39487[] = {{&S37658[44], 32}, {NULL}};
const Segment SGI39488[] = {{&S37658[30], 6}, {NULL}};
const Segment SGI39489[] = {{&S37658[124], 3}, {NULL}};
const Segment SGI39490[] = {{&S37658[90], 5}, {NULL}};
const Segment SGI39491[] = {{&S37659[43], 11}, {NULL}};
const Segment SGI39492[] = {{&S37659[43], 33}, {NULL}};
const Segment SGI39493[] = {{&S44936[44], 15}, {NULL}};
const Segment SGI39494[] = {{&S37659[30], 5}, {NULL}};
const Segment SGI39495[] = {{&S44936[44], 27}, {NULL}};
const Segment SGI39496[] = {{&S44936[30], 6}, {NULL}};
const Segment SGI39497[] = {{&S37659[124], 3}, {NULL}};
const Segment SGI39498[] = {{&S44936[119], 3}, {NULL}};
const Segment SGI39499[] = {{&S37659[90], 5}, {NULL}};
const Segment SGI39500[] = {{&S44936[85], 5}, {NULL}};
const Segment SGI39501[] = {{&S37661[43], 11}, {NULL}};
const Segment SGI39502[] = {{&S37661[43], 33}, {NULL}};
const Segment SGI39503[] = {{&S37661[124], 3}, {NULL}};
const Segment SGI39504[] = {{&S37661[90], 5}, {NULL}};
const Segment SGI39505[] = {{&S37663[38], 11}, {NULL}};
const Segment SGI39506[] = {{&S37663[31], 6}, {NULL}};
const Segment SGI39507[] = {{&S37674[38], 7}, {NULL}};
const Segment SGI39508[] = {{&S37674[31], 6}, {NULL}};
const Segment SGI39509[] = {{&S37675[44], 19}, {NULL}};
const Segment SGI39510[] = {{&S37675[44], 37}, {NULL}};
const Segment SGI39511[] = {{&S37675[30], 6}, {NULL}};
const Segment SGI39512[] = {{&S37675[69], 3}, {NULL}};
const Segment SGI39513[] = {{&S37675[95], 5}, {NULL}};
const Segment SGI39514[] = {{&S37676[44], 7}, {NULL}};
const Segment SGI39515[] = {{&S37676[44], 25}, {NULL}};
const Segment SGI39516[] = {{&S37676[30], 6}, {NULL}};
const Segment SGI39517[] = {{&S37676[57], 3}, {NULL}};
const Segment SGI39518[] = {{&S37676[83], 5}, {NULL}};
const Segment SGI39519[] = {{&S37677[44], 19}, {NULL}};
const Segment SGI39520[] = {{&S37677[44], 38}, {NULL}};
const Segment SGI39521[] = {{&S37677[30], 6}, {NULL}};
const Segment SGI39522[] = {{&S37677[69], 3}, {NULL}};
const Segment SGI39523[] = {{&S37677[96], 5}, {NULL}};
const Segment SGI39524[] = {{&S40023[44], 13}, {NULL}};
const Segment SGI39525[] = {{&S40023[44], 24}, {NULL}};
const Segment SGI39526[] = {{&S40023[30], 6}, {NULL}};
const Segment SGI39527[] = {{&S40023[116], 3}, {NULL}};
const Segment SGI39528[] = {{&S40023[82], 5}, {NULL}};
const Segment SGI39529[] = {{&S44820[44], 5}, {NULL}};
const Segment SGI39530[] = {{&S44820[44], 22}, {NULL}};
const Segment SGI39531[] = {{&S44820[30], 6}, {NULL}};
const Segment SGI39532[] = {{&S44820[114], 3}, {NULL}};
const Segment SGI39533[] = {{&S44820[80], 5}, {NULL}};
const Segment SGI39534[] = {{&S37979[44], 19}, {NULL}};
const Segment SGI39535[] = {{&S37979[44], 38}, {NULL}};
const Segment SGI39536[] = {{&S37979[30], 6}, {NULL}};
const Segment SGI39537[] = {{&S37979[69], 3}, {NULL}};
const Segment SGI39538[] = {{&S37979[96], 5}, {NULL}};
const Segment SGI39539[] = {{&S37981[44], 19}, {NULL}};
const Segment SGI39540[] = {{&S37981[44], 38}, {NULL}};
const Segment SGI39541[] = {{&S37981[30], 6}, {NULL}};
const Segment SGI39542[] = {{&S37981[69], 3}, {NULL}};
const Segment SGI39543[] = {{&S37981[96], 5}, {NULL}};
const Segment SGI39544[] = {{&S37984[44], 19}, {NULL}};
const Segment SGI39545[] = {{&S37984[44], 38}, {NULL}};
const Segment SGI39546[] = {{&S37984[30], 6}, {NULL}};
const Segment SGI39547[] = {{&S37984[69], 3}, {NULL}};
const Segment SGI39548[] = {{&S37984[96], 5}, {NULL}};
const Segment SGI39549[] = {{&S37985[44], 7}, {NULL}};
const Segment SGI39550[] = {{&S37985[44], 26}, {NULL}};
const Segment SGI39551[] = {{&S37985[30], 6}, {NULL}};
const Segment SGI39552[] = {{&S37985[57], 3}, {NULL}};
const Segment SGI39553[] = {{&S37985[84], 5}, {NULL}};
const Segment SGI39554[] = {{&S37986[44], 7}, {NULL}};
const Segment SGI39555[] = {{&S37986[44], 25}, {NULL}};
const Segment SGI39556[] = {{&S37986[30], 6}, {NULL}};
const Segment SGI39557[] = {{&S37986[57], 3}, {NULL}};
const Segment SGI39558[] = {{&S37986[83], 5}, {NULL}};
const Segment SGI39559[] = {{&S37987[44], 19}, {NULL}};
const Segment SGI39560[] = {{&S37987[44], 37}, {NULL}};
const Segment SGI39561[] = {{&S37987[30], 6}, {NULL}};
const Segment SGI39562[] = {{&S37987[69], 3}, {NULL}};
const Segment SGI39563[] = {{&S37987[95], 5}, {NULL}};
const Segment SGI39564[] = {{&S37988[38], 7}, {NULL}};
const Segment SGI39565[] = {{&S37988[31], 6}, {NULL}};
const Segment SGI39566[] = {{&S37989[44], 19}, {NULL}};
const Segment SGI39567[] = {{&S37989[44], 37}, {NULL}};
const Segment SGI39568[] = {{&S37989[30], 6}, {NULL}};
const Segment SGI39569[] = {{&S37989[69], 3}, {NULL}};
const Segment SGI39570[] = {{&S37989[95], 5}, {NULL}};
const Segment SGI39571[] = {{&S37990[44], 19}, {NULL}};
const Segment SGI39572[] = {{&S37990[44], 37}, {NULL}};
const Segment SGI39573[] = {{&S37990[30], 6}, {NULL}};
const Segment SGI39574[] = {{&S37990[69], 3}, {NULL}};
const Segment SGI39575[] = {{&S37990[95], 5}, {NULL}};
const Segment SGI39576[] = {{&S37991[44], 7}, {NULL}};
const Segment SGI39577[] = {{&S37991[44], 25}, {NULL}};
const Segment SGI39578[] = {{&S37991[30], 6}, {NULL}};
const Segment SGI39579[] = {{&S37991[57], 3}, {NULL}};
const Segment SGI39580[] = {{&S37991[83], 5}, {NULL}};
const Segment SGI39581[] = {{&S37992[44], 7}, {NULL}};
const Segment SGI39582[] = {{&S37992[44], 25}, {NULL}};
const Segment SGI39583[] = {{&S37992[30], 6}, {NULL}};
const Segment SGI39584[] = {{&S37992[57], 3}, {NULL}};
const Segment SGI39585[] = {{&S37992[83], 5}, {NULL}};
const Segment SGI39586[] = {{&S37993[44], 20}, {NULL}};
const Segment SGI39587[] = {{&S37993[44], 38}, {NULL}};
const Segment SGI39588[] = {{&S37993[30], 6}, {NULL}};
const Segment SGI39589[] = {{&S37993[70], 3}, {NULL}};
const Segment SGI39590[] = {{&S37993[96], 5}, {NULL}};
const Segment SGI39591[] = {{&S37994[44], 19}, {NULL}};
const Segment SGI39592[] = {{&S37994[44], 37}, {NULL}};
const Segment SGI39593[] = {{&S37994[30], 6}, {NULL}};
const Segment SGI39594[] = {{&S37994[69], 3}, {NULL}};
const Segment SGI39595[] = {{&S37994[95], 5}, {NULL}};
const Segment SGI39596[] = {{&S37995[44], 7}, {NULL}};
const Segment SGI39597[] = {{&S37995[44], 25}, {NULL}};
const Segment SGI39598[] = {{&S37995[30], 6}, {NULL}};
const Segment SGI39599[] = {{&S37995[86], 4}, {NULL}};
const Segment SGI39600[] = {{&S37995[86], 3}, {NULL}};
const Segment SGI39601[] = {{&S38150[44], 15}, {NULL}};
const Segment SGI39602[] = {{&S38150[44], 26}, {NULL}};
const Segment SGI39603[] = {{&S38150[30], 6}, {NULL}};
const Segment SGI39604[] = {{&S38150[118], 3}, {NULL}};
const Segment SGI39605[] = {{&S38150[84], 5}, {NULL}};
const Segment SGI39606[] = {{&S38151[44], 15}, {NULL}};
const Segment SGI39607[] = {{&S38151[44], 26}, {NULL}};
const Segment SGI39608[] = {{&S38151[30], 6}, {NULL}};
const Segment SGI39609[] = {{&S38151[118], 3}, {NULL}};
const Segment SGI39610[] = {{&S38151[84], 5}, {NULL}};
const Segment SGI39611[] = {{&S38152[44], 41}, {NULL}};
const Segment SGI39612[] = {{&S38152[44], 52}, {NULL}};
const Segment SGI39613[] = {{&S38152[30], 6}, {NULL}};
const Segment SGI39614[] = {{&S38152[144], 3}, {NULL}};
const Segment SGI39615[] = {{&S38152[110], 5}, {NULL}};
const Segment SGI39616[] = {{&S38153[42], 41}, {NULL}};
const Segment SGI39617[] = {{&S38153[30], 6}, {NULL}};
const Segment SGI39618[] = {{&S38153[142], 3}, {NULL}};
const Segment SGI39619[] = {{&S38153[108], 5}, {NULL}};
const Segment SGI39620[] = {{&S38154[44], 15}, {NULL}};
const Segment SGI39621[] = {{&S38154[44], 26}, {NULL}};
const Segment SGI39622[] = {{&S38154[30], 6}, {NULL}};
const Segment SGI39623[] = {{&S38154[118], 3}, {NULL}};
const Segment SGI39624[] = {{&S38154[84], 5}, {NULL}};
const Segment SGI39625[] = {{&S38155[44], 15}, {NULL}};
const Segment SGI39626[] = {{&S38155[44], 26}, {NULL}};
const Segment SGI39627[] = {{&S38155[30], 6}, {NULL}};
const Segment SGI39628[] = {{&S38155[118], 3}, {NULL}};
const Segment SGI39629[] = {{&S38155[84], 5}, {NULL}};
const Segment SGI39630[] = {{&S38156[44], 29}, {NULL}};
const Segment SGI39631[] = {{&S38156[44], 40}, {NULL}};
const Segment SGI39632[] = {{&S38156[30], 6}, {NULL}};
const Segment SGI39633[] = {{&S38156[132], 3}, {NULL}};
const Segment SGI39634[] = {{&S38156[98], 5}, {NULL}};
const Segment SGI39635[] = {{&S38157[44], 41}, {NULL}};
const Segment SGI39636[] = {{&S38157[44], 52}, {NULL}};
const Segment SGI39637[] = {{&S38157[30], 6}, {NULL}};
const Segment SGI39638[] = {{&S38157[144], 3}, {NULL}};
const Segment SGI39639[] = {{&S38157[110], 5}, {NULL}};
const Segment SGI39640[] = {{&S38158[44], 29}, {NULL}};
const Segment SGI39641[] = {{&S38158[44], 40}, {NULL}};
const Segment SGI39642[] = {{&S38158[30], 6}, {NULL}};
const Segment SGI39643[] = {{&S38158[132], 3}, {NULL}};
const Segment SGI39644[] = {{&S38158[98], 5}, {NULL}};
const Segment SGI39645[] = {{&S38159[44], 34}, {NULL}};
const Segment SGI39646[] = {{&S38159[44], 45}, {NULL}};
const Segment SGI39647[] = {{&S38159[30], 6}, {NULL}};
const Segment SGI39648[] = {{&S38159[137], 3}, {NULL}};
const Segment SGI39649[] = {{&S38159[103], 5}, {NULL}};
const Segment SGI39650[] = {{&S38160[38], 41}, {NULL}};
const Segment SGI39651[] = {{&S38160[31], 6}, {NULL}};
const Segment SGI39652[] = {{&S38161[44], 23}, {NULL}};
const Segment SGI39653[] = {{&S38161[44], 34}, {NULL}};
const Segment SGI39654[] = {{&S38161[30], 6}, {NULL}};
const Segment SGI39655[] = {{&S38161[126], 3}, {NULL}};
const Segment SGI39656[] = {{&S38161[92], 5}, {NULL}};
const Segment SGI39657[] = {{&S38162[44], 33}, {NULL}};
const Segment SGI39658[] = {{&S38162[44], 44}, {NULL}};
const Segment SGI39659[] = {{&S38162[30], 6}, {NULL}};
const Segment SGI39660[] = {{&S38162[136], 3}, {NULL}};
const Segment SGI39661[] = {{&S38162[102], 5}, {NULL}};
const Segment SGI39662[] = {{&S38163[44], 41}, {NULL}};
const Segment SGI39663[] = {{&S38163[44], 52}, {NULL}};
const Segment SGI39664[] = {{&S38163[30], 6}, {NULL}};
const Segment SGI39665[] = {{&S38163[113], 4}, {NULL}};
const Segment SGI39666[] = {{&S38163[113], 3}, {NULL}};
const Segment SGI39667[] = {{&S38164[44], 11}, {NULL}};
const Segment SGI39668[] = {{&S38164[44], 22}, {NULL}};
const Segment SGI39669[] = {{&S38164[30], 6}, {NULL}};
const Segment SGI39670[] = {{&S38164[114], 3}, {NULL}};
const Segment SGI39671[] = {{&S38164[80], 5}, {NULL}};
const Segment SGI39672[] = {{&S38165[44], 31}, {NULL}};
const Segment SGI39673[] = {{&S38165[44], 42}, {NULL}};
const Segment SGI39674[] = {{&S38165[30], 6}, {NULL}};
const Segment SGI39675[] = {{&S38165[134], 3}, {NULL}};
const Segment SGI39676[] = {{&S38165[100], 5}, {NULL}};
const Segment SGI39677[] = {{&S38166[38], 41}, {NULL}};
const Segment SGI39678[] = {{&S38166[31], 6}, {NULL}};
const Segment SGI39679[] = {{&S38167[44], 41}, {NULL}};
const Segment SGI39680[] = {{&S38167[44], 52}, {NULL}};
const Segment SGI39681[] = {{&S38167[30], 6}, {NULL}};
const Segment SGI39682[] = {{&S38167[113], 6}, {NULL}};
const Segment SGI39683[] = {{&S38168[38], 15}, {NULL}};
const Segment SGI39684[] = {{&S38168[31], 6}, {NULL}};
const Segment SGI39685[] = {{&S38169[44], 41}, {NULL}};
const Segment SGI39686[] = {{&S38169[44], 52}, {NULL}};
const Segment SGI39687[] = {{&S38169[30], 6}, {NULL}};
const Segment SGI39688[] = {{&S38169[113], 4}, {NULL}};
const Segment SGI39689[] = {{&S38169[113], 3}, {NULL}};
const Segment SGI39690[] = {{&S38170[44], 11}, {NULL}};
const Segment SGI39691[] = {{&S38170[44], 22}, {NULL}};
const Segment SGI39692[] = {{&S38170[30], 6}, {NULL}};
const Segment SGI39693[] = {{&S38170[114], 3}, {NULL}};
const Segment SGI39694[] = {{&S38170[80], 5}, {NULL}};
const Segment SGI39695[] = {{&S38171[44], 41}, {NULL}};
const Segment SGI39696[] = {{&S38171[44], 52}, {NULL}};
const Segment SGI39697[] = {{&S38171[30], 6}, {NULL}};
const Segment SGI39698[] = {{&S38171[113], 6}, {NULL}};
const Segment SGI39699[] = {{&S38172[38], 11}, {NULL}};
const Segment SGI39700[] = {{&S38172[31], 6}, {NULL}};
const Segment SGI39701[] = {{&S38173[44], 11}, {NULL}};
const Segment SGI39702[] = {{&S38173[44], 22}, {NULL}};
const Segment SGI39703[] = {{&S38173[30], 6}, {NULL}};
const Segment SGI39704[] = {{&S38173[83], 6}, {NULL}};
const Segment SGI39705[] = {{&S38174[44], 11}, {NULL}};
const Segment SGI39706[] = {{&S38174[44], 22}, {NULL}};
const Segment SGI39707[] = {{&S38174[30], 6}, {NULL}};
const Segment SGI39708[] = {{&S38174[114], 3}, {NULL}};
const Segment SGI39709[] = {{&S38174[80], 5}, {NULL}};
const Segment SGI39710[] = {{&S38199[44], 11}, {NULL}};
const Segment SGI39711[] = {{&S38199[44], 28}, {NULL}};
const Segment SGI39712[] = {{&S38199[30], 6}, {NULL}};
const Segment SGI39713[] = {{&S38199[120], 3}, {NULL}};
const Segment SGI39714[] = {{&S38199[86], 5}, {NULL}};
const Segment SGI39715[] = {{&S38200[38], 11}, {NULL}};
const Segment SGI39716[] = {{&S38200[31], 6}, {NULL}};
const Segment SGI39717[] = {{&S38204[44], 11}, {NULL}};
const Segment SGI39718[] = {{&S38204[44], 28}, {NULL}};
const Segment SGI39719[] = {{&S38204[30], 6}, {NULL}};
const Segment SGI39720[] = {{&S38204[121], 3}, {NULL}};
const Segment SGI39721[] = {{&S38204[86], 6}, {NULL}};
const Segment SGI39722[] = {{&S38221[44], 10}, {NULL}};
const Segment SGI39723[] = {{&S38221[44], 27}, {NULL}};
const Segment SGI39724[] = {{&S38221[30], 6}, {NULL}};
const Segment SGI39725[] = {{&S38221[119], 3}, {NULL}};
const Segment SGI39726[] = {{&S38221[85], 5}, {NULL}};
const Segment SGI39727[] = {{&S38223[38], 10}, {NULL}};
const Segment SGI39728[] = {{&S38223[31], 6}, {NULL}};
const Segment SGI39729[] = {{&S40395[44], 20}, {NULL}};
const Segment SGI39730[] = {{&S40395[44], 39}, {NULL}};
const Segment SGI39731[] = {{&S40395[30], 6}, {NULL}};
const Segment SGI39732[] = {{&S40395[131], 3}, {NULL}};
const Segment SGI39733[] = {{&S40395[97], 5}, {NULL}};
const Segment SGI39734[] = {{&S40396[44], 8}, {NULL}};
const Segment SGI39735[] = {{&S40396[44], 27}, {NULL}};
const Segment SGI39736[] = {{&S40396[30], 6}, {NULL}};
const Segment SGI39737[] = {{&S40396[119], 3}, {NULL}};
const Segment SGI39738[] = {{&S40396[85], 5}, {NULL}};
const Segment SGI39739[] = {{&S40397[44], 20}, {NULL}};
const Segment SGI39740[] = {{&S40397[44], 39}, {NULL}};
const Segment SGI39741[] = {{&S40397[30], 6}, {NULL}};
const Segment SGI39742[] = {{&S40397[131], 3}, {NULL}};
const Segment SGI39743[] = {{&S40397[97], 5}, {NULL}};
const Segment SGI39744[] = {{&S40398[44], 20}, {NULL}};
const Segment SGI39745[] = {{&S40398[44], 39}, {NULL}};
const Segment SGI39746[] = {{&S40398[30], 6}, {NULL}};
const Segment SGI39747[] = {{&S40398[131], 3}, {NULL}};
const Segment SGI39748[] = {{&S40398[97], 5}, {NULL}};
const Segment SGI39749[] = {{&S40399[44], 20}, {NULL}};
const Segment SGI39750[] = {{&S40399[44], 39}, {NULL}};
const Segment SGI39751[] = {{&S40399[30], 6}, {NULL}};
const Segment SGI39752[] = {{&S40399[131], 3}, {NULL}};
const Segment SGI39753[] = {{&S40399[97], 5}, {NULL}};
const Segment SGI39754[] = {{&S40400[44], 8}, {NULL}};
const Segment SGI39755[] = {{&S40400[44], 27}, {NULL}};
const Segment SGI39756[] = {{&S40400[30], 6}, {NULL}};
const Segment SGI39757[] = {{&S40400[119], 3}, {NULL}};
const Segment SGI39758[] = {{&S40400[85], 5}, {NULL}};
const Segment SGI39759[] = {{&S40401[44], 20}, {NULL}};
const Segment SGI39760[] = {{&S40401[44], 39}, {NULL}};
const Segment SGI39761[] = {{&S40401[30], 6}, {NULL}};
const Segment SGI39762[] = {{&S40401[131], 3}, {NULL}};
const Segment SGI39763[] = {{&S40401[97], 5}, {NULL}};
const Segment SGI39764[] = {{&S38270[44], 10}, {NULL}};
const Segment SGI39765[] = {{&S38270[44], 27}, {NULL}};
const Segment SGI39766[] = {{&S38270[30], 6}, {NULL}};
const Segment SGI39767[] = {{&S38270[119], 3}, {NULL}};
const Segment SGI39768[] = {{&S38270[85], 5}, {NULL}};
const Segment SGI39769[] = {{&S40402[38], 8}, {NULL}};
const Segment SGI39770[] = {{&S40402[31], 6}, {NULL}};
const Segment SGI39771[] = {{&S38272[38], 10}, {NULL}};
const Segment SGI39772[] = {{&S38272[31], 6}, {NULL}};
const Segment SGI39773[] = {{&S38276[44], 10}, {NULL}};
const Segment SGI39774[] = {{&S38276[44], 27}, {NULL}};
const Segment SGI39775[] = {{&S38276[30], 6}, {NULL}};
const Segment SGI39776[] = {{&S38276[88], 4}, {NULL}};
const Segment SGI39777[] = {{&S38276[88], 3}, {NULL}};
const Segment SGI39778[] = {{&S38278[44], 10}, {NULL}};
const Segment SGI39779[] = {{&S38278[44], 27}, {NULL}};
const Segment SGI39780[] = {{&S38278[30], 6}, {NULL}};
const Segment SGI39781[] = {{&S38278[88], 6}, {NULL}};
const Segment SGI39782[] = {{&S38280[42], 10}, {NULL}};
const Segment SGI39783[] = {{&S38280[30], 6}, {NULL}};
const Segment SGI39784[] = {{&S38280[117], 3}, {NULL}};
const Segment SGI39785[] = {{&S38280[83], 5}, {NULL}};
const Segment SGI39786[] = {{&S38327[44], 18}, {NULL}};
const Segment SGI39787[] = {{&S38327[44], 29}, {NULL}};
const Segment SGI39788[] = {{&S38327[30], 6}, {NULL}};
const Segment SGI39789[] = {{&S38327[121], 3}, {NULL}};
const Segment SGI39790[] = {{&S38327[87], 5}, {NULL}};
const Segment SGI39791[] = {{&S38194[38], 11}, {NULL}};
const Segment SGI39792[] = {{&S38194[31], 6}, {NULL}};
const Segment SGI39793[] = {{&S38195[44], 11}, {NULL}};
const Segment SGI39794[] = {{&S38195[44], 28}, {NULL}};
const Segment SGI39795[] = {{&S38195[30], 6}, {NULL}};
const Segment SGI39796[] = {{&S38195[120], 3}, {NULL}};
const Segment SGI39797[] = {{&S38195[86], 5}, {NULL}};
const Segment SGI39798[] = {{&S38203[44], 11}, {NULL}};
const Segment SGI39799[] = {{&S38203[44], 28}, {NULL}};
const Segment SGI39800[] = {{&S38203[30], 6}, {NULL}};
const Segment SGI39801[] = {{&S38203[121], 3}, {NULL}};
const Segment SGI39802[] = {{&S38203[86], 6}, {NULL}};
const Segment SGI39803[] = {{&S38205[44], 11}, {NULL}};
const Segment SGI39804[] = {{&S38205[44], 28}, {NULL}};
const Segment SGI39805[] = {{&S38205[30], 6}, {NULL}};
const Segment SGI39806[] = {{&S38205[89], 4}, {NULL}};
const Segment SGI39807[] = {{&S38205[89], 3}, {NULL}};
const Segment SGI39808[] = {{&S38207[42], 11}, {NULL}};
const Segment SGI39809[] = {{&S38207[30], 6}, {NULL}};
const Segment SGI39810[] = {{&S38207[118], 3}, {NULL}};
const Segment SGI39811[] = {{&S38207[84], 5}, {NULL}};
const Segment SGI39812[] = {{&S38208[44], 11}, {NULL}};
const Segment SGI39813[] = {{&S38208[44], 28}, {NULL}};
const Segment SGI39814[] = {{&S38208[30], 6}, {NULL}};
const Segment SGI39815[] = {{&S38208[89], 6}, {NULL}};
const Segment SGI39816[] = {{&S38269[44], 10}, {NULL}};
const Segment SGI39817[] = {{&S38269[44], 27}, {NULL}};
const Segment SGI39818[] = {{&S38269[30], 6}, {NULL}};
const Segment SGI39819[] = {{&S38269[119], 3}, {NULL}};
const Segment SGI39820[] = {{&S38269[85], 5}, {NULL}};
const Segment SGI39821[] = {{&S38271[38], 10}, {NULL}};
const Segment SGI39822[] = {{&S38271[31], 6}, {NULL}};
const Segment SGI39823[] = {{&S50714[44], 10}, {NULL}};
const Segment SGI39824[] = {{&S50714[44], 22}, {NULL}};
const Segment SGI39825[] = {{&S50714[30], 6}, {NULL}};
const Segment SGI39826[] = {{&S50714[114], 3}, {NULL}};
const Segment SGI39827[] = {{&S50714[80], 5}, {NULL}};
const Segment SGI39828[] = {{&S50715[44], 10}, {NULL}};
const Segment SGI39829[] = {{&S50715[44], 22}, {NULL}};
const Segment SGI39830[] = {{&S50715[30], 6}, {NULL}};
const Segment SGI39831[] = {{&S50715[114], 3}, {NULL}};
const Segment SGI39832[] = {{&S50715[80], 5}, {NULL}};
const Segment SGI39833[] = {{&S50716[38], 10}, {NULL}};
const Segment SGI39834[] = {{&S50716[31], 6}, {NULL}};
const Segment SGI39835[] = {{&S38279[44], 10}, {NULL}};
const Segment SGI39836[] = {{&S38279[44], 27}, {NULL}};
const Segment SGI39837[] = {{&S38279[30], 6}, {NULL}};
const Segment SGI39838[] = {{&S38279[88], 6}, {NULL}};
const Segment SGI39839[] = {{&S38428[44], 14}, {NULL}};
const Segment SGI39840[] = {{&S38428[44], 25}, {NULL}};
const Segment SGI39841[] = {{&S38428[30], 6}, {NULL}};
const Segment SGI39842[] = {{&S38428[117], 3}, {NULL}};
const Segment SGI39843[] = {{&S38428[83], 5}, {NULL}};
const Segment SGI39844[] = {{&S38429[38], 14}, {NULL}};
const Segment SGI39845[] = {{&S38429[31], 6}, {NULL}};
const Segment SGI39846[] = {{&S38430[42], 14}, {NULL}};
const Segment SGI39847[] = {{&S38430[30], 6}, {NULL}};
const Segment SGI39848[] = {{&S38430[115], 3}, {NULL}};
const Segment SGI39849[] = {{&S38430[81], 5}, {NULL}};
const Segment SGI39850[] = {{&S38319[44], 18}, {NULL}};
const Segment SGI39851[] = {{&S38319[44], 29}, {NULL}};
const Segment SGI39852[] = {{&S38319[30], 6}, {NULL}};
const Segment SGI39853[] = {{&S38319[121], 3}, {NULL}};
const Segment SGI39854[] = {{&S38319[87], 5}, {NULL}};
const Segment SGI39855[] = {{&S38320[43], 18}, {NULL}};
const Segment SGI39856[] = {{&S38320[43], 29}, {NULL}};
const Segment SGI39857[] = {{&S38320[120], 3}, {NULL}};
const Segment SGI39858[] = {{&S38320[86], 5}, {NULL}};
const Segment SGI39859[] = {{&S38321[44], 17}, {NULL}};
const Segment SGI39860[] = {{&S38321[44], 28}, {NULL}};
const Segment SGI39861[] = {{&S38321[30], 6}, {NULL}};
const Segment SGI39862[] = {{&S38321[120], 3}, {NULL}};
const Segment SGI39863[] = {{&S38321[86], 5}, {NULL}};
const Segment SGI39864[] = {{&S38322[43], 18}, {NULL}};
const Segment SGI39865[] = {{&S38322[43], 29}, {NULL}};
const Segment SGI39866[] = {{&S38322[30], 5}, {NULL}};
const Segment SGI39867[] = {{&S38322[120], 3}, {NULL}};
const Segment SGI39868[] = {{&S38322[86], 5}, {NULL}};
const Segment SGI39869[] = {{&S38323[44], 17}, {NULL}};
const Segment SGI39870[] = {{&S38323[44], 28}, {NULL}};
const Segment SGI39871[] = {{&S38323[30], 6}, {NULL}};
const Segment SGI39872[] = {{&S38323[120], 3}, {NULL}};
const Segment SGI39873[] = {{&S38323[86], 5}, {NULL}};
const Segment SGI39874[] = {{&S38324[44], 18}, {NULL}};
const Segment SGI39875[] = {{&S38324[44], 29}, {NULL}};
const Segment SGI39876[] = {{&S38324[30], 6}, {NULL}};
const Segment SGI39877[] = {{&S38324[121], 3}, {NULL}};
const Segment SGI39878[] = {{&S38324[87], 5}, {NULL}};
const Segment SGI39879[] = {{&S38325[44], 18}, {NULL}};
const Segment SGI39880[] = {{&S38325[44], 29}, {NULL}};
const Segment SGI39881[] = {{&S38325[30], 6}, {NULL}};
const Segment SGI39882[] = {{&S38326[38], 18}, {NULL}};
const Segment SGI39883[] = {{&S38326[31], 6}, {NULL}};
const Segment SGI39884[] = {{&S38328[44], 18}, {NULL}};
const Segment SGI39885[] = {{&S38328[44], 29}, {NULL}};
const Segment SGI39886[] = {{&S38328[30], 6}, {NULL}};
const Segment SGI39887[] = {{&S38328[123], 3}, {NULL}};
const Segment SGI39888[] = {{&S38328[87], 7}, {NULL}};
const Segment SGI39889[] = {{&S38328[143], 8}, {NULL}};
const Segment SGI39890[] = {{&S38329[42], 18}, {NULL}};
const Segment SGI39891[] = {{&S38329[30], 6}, {NULL}};
const Segment SGI39892[] = {{&S38329[119], 3}, {NULL}};
const Segment SGI39893[] = {{&S38329[85], 5}, {NULL}};
const Segment SGI39894[] = {{&S38503[44], 14}, {NULL}};
const Segment SGI39895[] = {{&S38503[44], 25}, {NULL}};
const Segment SGI39896[] = {{&S38503[30], 6}, {NULL}};
const Segment SGI39897[] = {{&S38503[117], 3}, {NULL}};
const Segment SGI39898[] = {{&S38503[83], 5}, {NULL}};
const Segment SGI39899[] = {{&S38504[44], 9}, {NULL}};
const Segment SGI39900[] = {{&S38504[44], 20}, {NULL}};
const Segment SGI39901[] = {{&S38504[30], 6}, {NULL}};
const Segment SGI39902[] = {{&S38504[112], 3}, {NULL}};
const Segment SGI39903[] = {{&S38504[78], 5}, {NULL}};
const Segment SGI39904[] = {{&S38505[44], 14}, {NULL}};
const Segment SGI39905[] = {{&S38505[44], 25}, {NULL}};
const Segment SGI39906[] = {{&S38505[30], 6}, {NULL}};
const Segment SGI39907[] = {{&S38505[117], 3}, {NULL}};
const Segment SGI39908[] = {{&S38505[83], 5}, {NULL}};
const Segment SGI39909[] = {{&S38506[44], 14}, {NULL}};
const Segment SGI39910[] = {{&S38506[44], 25}, {NULL}};
const Segment SGI39911[] = {{&S38506[30], 6}, {NULL}};
const Segment SGI39912[] = {{&S38506[117], 3}, {NULL}};
const Segment SGI39913[] = {{&S38506[83], 5}, {NULL}};
const Segment SGI39914[] = {{&S38507[38], 9}, {NULL}};
const Segment SGI39915[] = {{&S38507[31], 6}, {NULL}};
const Segment SGI39916[] = {{&S38524[44], 14}, {NULL}};
const Segment SGI39917[] = {{&S38524[44], 35}, {NULL}};
const Segment SGI39918[] = {{&S38524[30], 6}, {NULL}};
const Segment SGI39919[] = {{&S38524[127], 3}, {NULL}};
const Segment SGI39920[] = {{&S38524[93], 5}, {NULL}};
const Segment SGI39921[] = {{&S38526[44], 10}, {NULL}};
const Segment SGI39922[] = {{&S38526[44], 30}, {NULL}};
const Segment SGI39923[] = {{&S38526[30], 6}, {NULL}};
const Segment SGI39924[] = {{&S38526[122], 3}, {NULL}};
const Segment SGI39925[] = {{&S38526[88], 5}, {NULL}};
const Segment SGI39926[] = {{&S38527[38], 14}, {NULL}};
const Segment SGI39927[] = {{&S38527[31], 6}, {NULL}};
const Segment SGI39928[] = {{&S50841[44], 10}, {NULL}};
const Segment SGI39929[] = {{&S50841[44], 24}, {NULL}};
const Segment SGI39930[] = {{&S50841[30], 6}, {NULL}};
const Segment SGI39931[] = {{&S50841[116], 3}, {NULL}};
const Segment SGI39932[] = {{&S50841[82], 5}, {NULL}};
const Segment SGI39933[] = {{&S50842[44], 10}, {NULL}};
const Segment SGI39934[] = {{&S50842[44], 24}, {NULL}};
const Segment SGI39935[] = {{&S50842[30], 6}, {NULL}};
const Segment SGI39936[] = {{&S50842[116], 3}, {NULL}};
const Segment SGI39937[] = {{&S50842[82], 5}, {NULL}};
const Segment SGI39938[] = {{&S50843[38], 10}, {NULL}};
const Segment SGI39939[] = {{&S50843[31], 6}, {NULL}};
const Segment SGI39940[] = {{&S50844[44], 10}, {NULL}};
const Segment SGI39941[] = {{&S50844[44], 24}, {NULL}};
const Segment SGI39942[] = {{&S50844[30], 6}, {NULL}};
const Segment SGI39943[] = {{&S50844[116], 3}, {NULL}};
const Segment SGI39944[] = {{&S50844[82], 5}, {NULL}};
const Segment SGI39945[] = {{&S50845[44], 10}, {NULL}};
const Segment SGI39946[] = {{&S50845[44], 24}, {NULL}};
const Segment SGI39947[] = {{&S50845[30], 6}, {NULL}};
const Segment SGI39948[] = {{&S50845[116], 3}, {NULL}};
const Segment SGI39949[] = {{&S50845[82], 5}, {NULL}};
const Segment SGI39950[] = {{&S38427[44], 14}, {NULL}};
const Segment SGI39951[] = {{&S38427[44], 31}, {NULL}};
const Segment SGI39952[] = {{&S38427[30], 6}, {NULL}};
const Segment SGI39953[] = {{&S38427[123], 3}, {NULL}};
const Segment SGI39954[] = {{&S38427[89], 5}, {NULL}};
const Segment SGI39955[] = {{&S45605[44], 8}, {NULL}};
const Segment SGI39956[] = {{&S45605[44], 25}, {NULL}};
const Segment SGI39957[] = {{&S45605[30], 6}, {NULL}};
const Segment SGI39958[] = {{&S45605[117], 3}, {NULL}};
const Segment SGI39959[] = {{&S45605[83], 5}, {NULL}};
const Segment SGI39960[] = {{&S45606[44], 10}, {NULL}};
const Segment SGI39961[] = {{&S45606[44], 27}, {NULL}};
const Segment SGI39962[] = {{&S45606[30], 6}, {NULL}};
const Segment SGI39963[] = {{&S45606[119], 3}, {NULL}};
const Segment SGI39964[] = {{&S45606[85], 5}, {NULL}};
const Segment SGI39965[] = {{&S45607[44], 19}, {NULL}};
const Segment SGI39966[] = {{&S45607[44], 36}, {NULL}};
const Segment SGI39967[] = {{&S45607[30], 6}, {NULL}};
const Segment SGI39968[] = {{&S45607[128], 3}, {NULL}};
const Segment SGI39969[] = {{&S45607[94], 5}, {NULL}};
const Segment SGI39970[] = {{&S45608[38], 10}, {NULL}};
const Segment SGI39971[] = {{&S45608[31], 6}, {NULL}};
const Segment SGI39972[] = {{&S45609[42], 10}, {NULL}};
const Segment SGI39973[] = {{&S45609[30], 6}, {NULL}};
const Segment SGI39974[] = {{&S45609[117], 3}, {NULL}};
const Segment SGI39975[] = {{&S45609[83], 5}, {NULL}};
const Segment SGI39976[] = {{&S38441[44], 13}, {NULL}};
const Segment SGI39977[] = {{&S38441[44], 30}, {NULL}};
const Segment SGI39978[] = {{&S38441[30], 6}, {NULL}};
const Segment SGI39979[] = {{&S38441[122], 3}, {NULL}};
const Segment SGI39980[] = {{&S38441[88], 5}, {NULL}};
const Segment SGI39981[] = {{&S38442[44], 13}, {NULL}};
const Segment SGI39982[] = {{&S38442[44], 30}, {NULL}};
const Segment SGI39983[] = {{&S38442[30], 6}, {NULL}};
const Segment SGI39984[] = {{&S38442[133], 3}, {NULL}};
const Segment SGI39985[] = {{&S38442[98], 6}, {NULL}};
const Segment SGI39986[] = {{&S38442[153], 8}, {NULL}};
const Segment SGI39987[] = {{&S38443[44], 13}, {NULL}};
const Segment SGI39988[] = {{&S38443[44], 30}, {NULL}};
const Segment SGI39989[] = {{&S38443[30], 6}, {NULL}};
const Segment SGI39990[] = {{&S38443[88], 6}, {NULL}};
const Segment SGI39991[] = {{&S38444[38], 13}, {NULL}};
const Segment SGI39992[] = {{&S38444[31], 6}, {NULL}};
const Segment SGI39993[] = {{&S38445[42], 13}, {NULL}};
const Segment SGI39994[] = {{&S38445[30], 6}, {NULL}};
const Segment SGI39995[] = {{&S38445[120], 3}, {NULL}};
const Segment SGI39996[] = {{&S38445[86], 5}, {NULL}};
const Segment SGI39997[] = {{&S50947[44], 17}, {NULL}};
const Segment SGI39998[] = {{&S50947[44], 29}, {NULL}};
const Segment SGI39999[] = {{&S50947[30], 6}, {NULL}};
const Segment SGI40000[] = {{&S50947[121], 3}, {NULL}};
const Segment SGI40001[] = {{&S50947[87], 5}, {NULL}};
const Segment SGI40002[] = {{&S50949[38], 17}, {NULL}};
const Segment SGI40003[] = {{&S50949[31], 6}, {NULL}};
const Segment SGI40004[] = {{&S38588[44], 18}, {NULL}};
const Segment SGI40005[] = {{&S38588[44], 35}, {NULL}};
const Segment SGI40006[] = {{&S38588[30], 6}, {NULL}};
const Segment SGI40007[] = {{&S38588[127], 3}, {NULL}};
const Segment SGI40008[] = {{&S38588[93], 5}, {NULL}};
const Segment SGI40009[] = {{&S38591[44], 10}, {NULL}};
const Segment SGI40010[] = {{&S38591[44], 21}, {NULL}};
const Segment SGI40011[] = {{&S38591[30], 6}, {NULL}};
const Segment SGI40012[] = {{&S38591[113], 3}, {NULL}};
const Segment SGI40013[] = {{&S38591[79], 5}, {NULL}};
const Segment SGI40014[] = {{&S38593[44], 10}, {NULL}};
const Segment SGI40015[] = {{&S38593[44], 27}, {NULL}};
const Segment SGI40016[] = {{&S38593[30], 6}, {NULL}};
const Segment SGI40017[] = {{&S38593[119], 3}, {NULL}};
const Segment SGI40018[] = {{&S38593[85], 5}, {NULL}};
const Segment SGI40019[] = {{&S38596[38], 18}, {NULL}};
const Segment SGI40020[] = {{&S38596[31], 6}, {NULL}};
const Segment SGI40021[] = {{&S38602[38], 10}, {NULL}};
const Segment SGI40022[] = {{&S38602[31], 6}, {NULL}};
const Segment SGI40023[] = {{&S38523[44], 14}, {NULL}};
const Segment SGI40024[] = {{&S38523[44], 35}, {NULL}};
const Segment SGI40025[] = {{&S38523[30], 6}, {NULL}};
const Segment SGI40026[] = {{&S38523[127], 3}, {NULL}};
const Segment SGI40027[] = {{&S38523[93], 5}, {NULL}};
const Segment SGI40028[] = {{&S38525[44], 10}, {NULL}};
const Segment SGI40029[] = {{&S38525[44], 30}, {NULL}};
const Segment SGI40030[] = {{&S38525[30], 6}, {NULL}};
const Segment SGI40031[] = {{&S38525[122], 3}, {NULL}};
const Segment SGI40032[] = {{&S38525[88], 5}, {NULL}};
const Segment SGI40033[] = {{&S38528[44], 7}, {NULL}};
const Segment SGI40034[] = {{&S38528[44], 24}, {NULL}};
const Segment SGI40035[] = {{&S38528[30], 6}, {NULL}};
const Segment SGI40036[] = {{&S38528[116], 3}, {NULL}};
const Segment SGI40037[] = {{&S38528[82], 5}, {NULL}};
const Segment SGI40038[] = {{&S38529[38], 14}, {NULL}};
const Segment SGI40039[] = {{&S38529[31], 6}, {NULL}};
const Segment SGI40040[] = {{&S38552[44], 10}, {NULL}};
const Segment SGI40041[] = {{&S38552[44], 27}, {NULL}};
const Segment SGI40042[] = {{&S38552[30], 6}, {NULL}};
const Segment SGI40043[] = {{&S38552[119], 3}, {NULL}};
const Segment SGI40044[] = {{&S38552[85], 5}, {NULL}};
const Segment SGI40045[] = {{&S38587[44], 18}, {NULL}};
const Segment SGI40046[] = {{&S38587[44], 35}, {NULL}};
const Segment SGI40047[] = {{&S38587[30], 6}, {NULL}};
const Segment SGI40048[] = {{&S38587[127], 3}, {NULL}};
const Segment SGI40049[] = {{&S38587[93], 5}, {NULL}};
const Segment SGI40050[] = {{&S38589[44], 10}, {NULL}};
const Segment SGI40051[] = {{&S38589[44], 21}, {NULL}};
const Segment SGI40052[] = {{&S38589[30], 6}, {NULL}};
const Segment SGI40053[] = {{&S38589[113], 3}, {NULL}};
const Segment SGI40054[] = {{&S38589[79], 5}, {NULL}};
const Segment SGI40055[] = {{&S38590[44], 10}, {NULL}};
const Segment SGI40056[] = {{&S38590[44], 27}, {NULL}};
const Segment SGI40057[] = {{&S38590[30], 6}, {NULL}};
const Segment SGI40058[] = {{&S38590[119], 3}, {NULL}};
const Segment SGI40059[] = {{&S38590[85], 5}, {NULL}};
const Segment SGI40060[] = {{&S38594[38], 18}, {NULL}};
const Segment SGI40061[] = {{&S38594[31], 6}, {NULL}};
const Segment SGI40062[] = {{&S38598[38], 10}, {NULL}};
const Segment SGI40063[] = {{&S38598[31], 6}, {NULL}};
const Segment SGI40064[] = {{&S38700[44], 7}, {NULL}};
const Segment SGI40065[] = {{&S38700[44], 18}, {NULL}};
const Segment SGI40066[] = {{&S38700[30], 6}, {NULL}};
const Segment SGI40067[] = {{&S38700[110], 3}, {NULL}};
const Segment SGI40068[] = {{&S38700[76], 5}, {NULL}};
const Segment SGI40069[] = {{&S38716[44], 10}, {NULL}};
const Segment SGI40070[] = {{&S38716[44], 27}, {NULL}};
const Segment SGI40071[] = {{&S38716[30], 6}, {NULL}};
const Segment SGI40072[] = {{&S38716[119], 3}, {NULL}};
const Segment SGI40073[] = {{&S38716[85], 5}, {NULL}};
const Segment SGI40074[] = {{&S38717[44], 10}, {NULL}};
const Segment SGI40075[] = {{&S38717[44], 32}, {NULL}};
const Segment SGI40076[] = {{&S38717[30], 6}, {NULL}};
const Segment SGI40077[] = {{&S38718[42], 10}, {NULL}};
const Segment SGI40078[] = {{&S38718[30], 6}, {NULL}};
const Segment SGI40079[] = {{&S38718[117], 3}, {NULL}};
const Segment SGI40080[] = {{&S38718[83], 5}, {NULL}};
const Segment SGI40081[] = {{&S38719[64], 10}, {NULL}};
const Segment SGI40082[] = {{&S38719[64], 27}, {NULL}};
const Segment SGI40083[] = {{&S38719[50], 6}, {NULL}};
const Segment SGI40084[] = {{&S38721[44], 10}, {NULL}};
const Segment SGI40085[] = {{&S38721[44], 27}, {NULL}};
const Segment SGI40086[] = {{&S38721[30], 6}, {NULL}};
const Segment SGI40087[] = {{&S38721[85], 6}, {NULL}};
const Segment SGI40088[] = {{&S51105[44], 10}, {NULL}};
const Segment SGI40089[] = {{&S51105[44], 22}, {NULL}};
const Segment SGI40090[] = {{&S51105[30], 6}, {NULL}};
const Segment SGI40091[] = {{&S38722[38], 10}, {NULL}};
const Segment SGI40092[] = {{&S38722[31], 6}, {NULL}};
const Segment SGI40093[] = {{&S51105[114], 3}, {NULL}};
const Segment SGI40094[] = {{&S51105[80], 5}, {NULL}};
const Segment SGI40095[] = {{&S38723[43], 10}, {NULL}};
const Segment SGI40096[] = {{&S38723[43], 27}, {NULL}};
const Segment SGI40097[] = {{&S51106[44], 4}, {NULL}};
const Segment SGI40098[] = {{&S51106[44], 16}, {NULL}};
const Segment SGI40099[] = {{&S51106[30], 6}, {NULL}};
const Segment SGI40100[] = {{&S38723[118], 3}, {NULL}};
const Segment SGI40101[] = {{&S38723[84], 5}, {NULL}};
const Segment SGI40102[] = {{&S51106[108], 3}, {NULL}};
const Segment SGI40103[] = {{&S51106[74], 5}, {NULL}};
const Segment SGI40104[] = {{&S38660[38], 10}, {NULL}};
const Segment SGI40105[] = {{&S38660[31], 6}, {NULL}};
const Segment SGI40106[] = {{&S38724[44], 10}, {NULL}};
const Segment SGI40107[] = {{&S38724[44], 27}, {NULL}};
const Segment SGI40108[] = {{&S38724[30], 6}, {NULL}};
const Segment SGI40109[] = {{&S38724[119], 3}, {NULL}};
const Segment SGI40110[] = {{&S38724[85], 5}, {NULL}};
const Segment SGI40111[] = {{&S38661[44], 10}, {NULL}};
const Segment SGI40112[] = {{&S38661[44], 21}, {NULL}};
const Segment SGI40113[] = {{&S38661[30], 6}, {NULL}};
const Segment SGI40114[] = {{&S38661[113], 3}, {NULL}};
const Segment SGI40115[] = {{&S38661[79], 5}, {NULL}};
const Segment SGI40116[] = {{&S38725[43], 10}, {NULL}};
const Segment SGI40117[] = {{&S38725[43], 27}, {NULL}};
const Segment SGI40118[] = {{&S38725[30], 5}, {NULL}};
const Segment SGI40119[] = {{&S38725[118], 3}, {NULL}};
const Segment SGI40120[] = {{&S38662[44], 10}, {NULL}};
const Segment SGI40121[] = {{&S38725[84], 5}, {NULL}};
const Segment SGI40122[] = {{&S38662[44], 27}, {NULL}};
const Segment SGI40123[] = {{&S38662[30], 6}, {NULL}};
const Segment SGI40124[] = {{&S38662[119], 3}, {NULL}};
const Segment SGI40125[] = {{&S38662[85], 5}, {NULL}};
const Segment SGI40126[] = {{&S38726[44], 9}, {NULL}};
const Segment SGI40127[] = {{&S38726[44], 26}, {NULL}};
const Segment SGI40128[] = {{&S38726[30], 6}, {NULL}};
const Segment SGI40129[] = {{&S38663[44], 10}, {NULL}};
const Segment SGI40130[] = {{&S38663[44], 26}, {NULL}};
const Segment SGI40131[] = {{&S38726[118], 3}, {NULL}};
const Segment SGI40132[] = {{&S38663[30], 6}, {NULL}};
const Segment SGI40133[] = {{&S38726[84], 5}, {NULL}};
const Segment SGI40134[] = {{&S38663[118], 3}, {NULL}};
const Segment SGI40135[] = {{&S38663[84], 5}, {NULL}};
const Segment SGI40136[] = {{&S38727[44], 9}, {NULL}};
const Segment SGI40137[] = {{&S38727[44], 26}, {NULL}};
const Segment SGI40138[] = {{&S38727[30], 6}, {NULL}};
const Segment SGI40139[] = {{&S38664[44], 10}, {NULL}};
const Segment SGI40140[] = {{&S38664[44], 32}, {NULL}};
const Segment SGI40141[] = {{&S38664[30], 6}, {NULL}};
const Segment SGI40142[] = {{&S38727[118], 3}, {NULL}};
const Segment SGI40143[] = {{&S38727[84], 5}, {NULL}};
const Segment SGI40144[] = {{&S38664[124], 3}, {NULL}};
const Segment SGI40145[] = {{&S38664[90], 5}, {NULL}};
const Segment SGI40146[] = {{&S38665[44], 10}, {NULL}};
const Segment SGI40147[] = {{&S38665[44], 26}, {NULL}};
const Segment SGI40148[] = {{&S38665[30], 6}, {NULL}};
const Segment SGI40149[] = {{&S38665[118], 3}, {NULL}};
const Segment SGI40150[] = {{&S38665[84], 5}, {NULL}};
const Segment SGI40151[] = {{&S38738[44], 18}, {NULL}};
const Segment SGI40152[] = {{&S38738[44], 35}, {NULL}};
const Segment SGI40153[] = {{&S38738[30], 6}, {NULL}};
const Segment SGI40154[] = {{&S38738[127], 3}, {NULL}};
const Segment SGI40155[] = {{&S38738[93], 5}, {NULL}};
const Segment SGI40156[] = {{&S38740[44], 10}, {NULL}};
const Segment SGI40157[] = {{&S38740[44], 21}, {NULL}};
const Segment SGI40158[] = {{&S38740[30], 6}, {NULL}};
const Segment SGI40159[] = {{&S38740[113], 3}, {NULL}};
const Segment SGI40160[] = {{&S38740[79], 5}, {NULL}};
const Segment SGI40161[] = {{&S38746[44], 10}, {NULL}};
const Segment SGI40162[] = {{&S38746[44], 27}, {NULL}};
const Segment SGI40163[] = {{&S38746[30], 6}, {NULL}};
const Segment SGI40164[] = {{&S38746[119], 3}, {NULL}};
const Segment SGI40165[] = {{&S38746[85], 5}, {NULL}};
const Segment SGI40166[] = {{&S38749[38], 18}, {NULL}};
const Segment SGI40167[] = {{&S38749[31], 6}, {NULL}};
const Segment SGI40168[] = {{&S38698[44], 7}, {NULL}};
const Segment SGI40169[] = {{&S38698[44], 18}, {NULL}};
const Segment SGI40170[] = {{&S38698[30], 6}, {NULL}};
const Segment SGI40171[] = {{&S38698[110], 3}, {NULL}};
const Segment SGI40172[] = {{&S38698[76], 5}, {NULL}};
const Segment SGI40173[] = {{&S38715[44], 10}, {NULL}};
const Segment SGI40174[] = {{&S38715[44], 27}, {NULL}};
const Segment SGI40175[] = {{&S38715[30], 6}, {NULL}};
const Segment SGI40176[] = {{&S38715[119], 3}, {NULL}};
const Segment SGI40177[] = {{&S38715[85], 5}, {NULL}};
const Segment SGI40178[] = {{&S38736[44], 18}, {NULL}};
const Segment SGI40179[] = {{&S38736[44], 35}, {NULL}};
const Segment SGI40180[] = {{&S38736[30], 6}, {NULL}};
const Segment SGI40181[] = {{&S38736[127], 3}, {NULL}};
const Segment SGI40182[] = {{&S38736[93], 5}, {NULL}};
const Segment SGI40183[] = {{&S38741[44], 10}, {NULL}};
const Segment SGI40184[] = {{&S38741[44], 27}, {NULL}};
const Segment SGI40185[] = {{&S38741[30], 6}, {NULL}};
const Segment SGI40186[] = {{&S38741[119], 3}, {NULL}};
const Segment SGI40187[] = {{&S38741[85], 5}, {NULL}};
const Segment SGI40188[] = {{&S51109[38], 10}, {NULL}};
const Segment SGI40189[] = {{&S51109[31], 6}, {NULL}};
const Segment SGI40190[] = {{&S51110[44], 19}, {NULL}};
const Segment SGI40191[] = {{&S51110[44], 31}, {NULL}};
const Segment SGI40192[] = {{&S51110[30], 6}, {NULL}};
const Segment SGI40193[] = {{&S51110[123], 3}, {NULL}};
const Segment SGI40194[] = {{&S51110[89], 5}, {NULL}};
const Segment SGI40195[] = {{&S40894[44], 10}, {NULL}};
const Segment SGI40196[] = {{&S40894[44], 27}, {NULL}};
const Segment SGI40197[] = {{&S40894[30], 6}, {NULL}};
const Segment SGI40198[] = {{&S40894[119], 3}, {NULL}};
const Segment SGI40199[] = {{&S40894[85], 5}, {NULL}};
const Segment SGI40200[] = {{&S38825[44], 7}, {NULL}};
const Segment SGI40201[] = {{&S38825[44], 24}, {NULL}};
const Segment SGI40202[] = {{&S38825[30], 6}, {NULL}};
const Segment SGI40203[] = {{&S38825[116], 3}, {NULL}};
const Segment SGI40204[] = {{&S38825[82], 5}, {NULL}};
const Segment SGI40205[] = {{&S38826[44], 7}, {NULL}};
const Segment SGI40206[] = {{&S38826[44], 24}, {NULL}};
const Segment SGI40207[] = {{&S38826[30], 6}, {NULL}};
const Segment SGI40208[] = {{&S38826[116], 3}, {NULL}};
const Segment SGI40209[] = {{&S38826[82], 5}, {NULL}};
const Segment SGI40210[] = {{&S38827[38], 7}, {NULL}};
const Segment SGI40211[] = {{&S38827[31], 6}, {NULL}};
const Segment SGI40212[] = {{&S38828[38], 7}, {NULL}};
const Segment SGI40213[] = {{&S38828[31], 6}, {NULL}};
const Segment SGI40214[] = {{&S51211[44], 12}, {NULL}};
const Segment SGI40215[] = {{&S51211[44], 24}, {NULL}};
const Segment SGI40216[] = {{&S40964[44], 7}, {NULL}};
const Segment SGI40217[] = {{&S51211[30], 6}, {NULL}};
const Segment SGI40218[] = {{&S40964[44], 24}, {NULL}};
const Segment SGI40219[] = {{&S40964[30], 6}, {NULL}};
const Segment SGI40220[] = {{&S51211[116], 3}, {NULL}};
const Segment SGI40221[] = {{&S40964[116], 3}, {NULL}};
const Segment SGI40222[] = {{&S51211[82], 5}, {NULL}};
const Segment SGI40223[] = {{&S40964[82], 5}, {NULL}};
const Segment SGI40224[] = {{&S40965[44], 7}, {NULL}};
const Segment SGI40225[] = {{&S40965[44], 24}, {NULL}};
const Segment SGI40226[] = {{&S40965[30], 6}, {NULL}};
const Segment SGI40227[] = {{&S51213[38], 12}, {NULL}};
const Segment SGI40228[] = {{&S40965[116], 3}, {NULL}};
const Segment SGI40229[] = {{&S51213[31], 6}, {NULL}};
const Segment SGI40230[] = {{&S40965[82], 5}, {NULL}};
const Segment SGI40231[] = {{&S40966[38], 7}, {NULL}};
const Segment SGI40232[] = {{&S40966[31], 6}, {NULL}};
const Segment SGI40233[] = {{&S38937[44], 12}, {NULL}};
const Segment SGI40234[] = {{&S38937[44], 23}, {NULL}};
const Segment SGI40235[] = {{&S38937[30], 6}, {NULL}};
const Segment SGI40236[] = {{&S38937[115], 3}, {NULL}};
const Segment SGI40237[] = {{&S38937[81], 5}, {NULL}};
const Segment SGI40238[] = {{&S38939[44], 12}, {NULL}};
const Segment SGI40239[] = {{&S38939[44], 23}, {NULL}};
const Segment SGI40240[] = {{&S38939[30], 6}, {NULL}};
const Segment SGI40241[] = {{&S38939[115], 3}, {NULL}};
const Segment SGI40242[] = {{&S38939[81], 5}, {NULL}};
const Segment SGI40243[] = {{&S38943[38], 12}, {NULL}};
const Segment SGI40244[] = {{&S38943[31], 6}, {NULL}};
const Segment SGI40245[] = {{&S38947[44], 12}, {NULL}};
const Segment SGI40246[] = {{&S38947[44], 23}, {NULL}};
const Segment SGI40247[] = {{&S38947[30], 6}, {NULL}};
const Segment SGI40248[] = {{&S38947[115], 3}, {NULL}};
const Segment SGI40249[] = {{&S38947[81], 5}, {NULL}};
const Segment SGI40250[] = {{&S41046[44], 11}, {NULL}};
const Segment SGI40251[] = {{&S41046[44], 28}, {NULL}};
const Segment SGI40252[] = {{&S41046[30], 6}, {NULL}};
const Segment SGI40253[] = {{&S41046[120], 3}, {NULL}};
const Segment SGI40254[] = {{&S41046[86], 5}, {NULL}};
const Segment SGI40255[] = {{&S41058[38], 11}, {NULL}};
const Segment SGI40256[] = {{&S41058[31], 6}, {NULL}};
const Segment SGI40257[] = {{&S41059[44], 10}, {NULL}};
const Segment SGI40258[] = {{&S41059[44], 21}, {NULL}};
const Segment SGI40259[] = {{&S41059[30], 6}, {NULL}};
const Segment SGI40260[] = {{&S41059[113], 3}, {NULL}};
const Segment SGI40261[] = {{&S41059[79], 5}, {NULL}};
const Segment SGI40262[] = {{&S51371[44], 15}, {NULL}};
const Segment SGI40263[] = {{&S51371[44], 27}, {NULL}};
const Segment SGI40264[] = {{&S51371[30], 6}, {NULL}};
const Segment SGI40265[] = {{&S51371[119], 3}, {NULL}};
const Segment SGI40266[] = {{&S51371[85], 5}, {NULL}};
const Segment SGI40267[] = {{&S51374[43], 15}, {NULL}};
const Segment SGI40268[] = {{&S51374[43], 27}, {NULL}};
const Segment SGI40269[] = {{&S51374[30], 5}, {NULL}};
const Segment SGI40270[] = {{&S51374[118], 3}, {NULL}};
const Segment SGI40271[] = {{&S51374[84], 5}, {NULL}};
const Segment SGI40272[] = {{&S38940[44], 12}, {NULL}};
const Segment SGI40273[] = {{&S38940[44], 23}, {NULL}};
const Segment SGI40274[] = {{&S38940[30], 6}, {NULL}};
const Segment SGI40275[] = {{&S38940[115], 3}, {NULL}};
const Segment SGI40276[] = {{&S38940[81], 5}, {NULL}};
const Segment SGI40277[] = {{&S38944[44], 12}, {NULL}};
const Segment SGI40278[] = {{&S38944[44], 23}, {NULL}};
const Segment SGI40279[] = {{&S38944[30], 6}, {NULL}};
const Segment SGI40280[] = {{&S38944[115], 3}, {NULL}};
const Segment SGI40281[] = {{&S38944[81], 5}, {NULL}};
const Segment SGI40282[] = {{&S38945[44], 12}, {NULL}};
const Segment SGI40283[] = {{&S38945[44], 23}, {NULL}};
const Segment SGI40284[] = {{&S38945[30], 6}, {NULL}};
const Segment SGI40285[] = {{&S38945[115], 3}, {NULL}};
const Segment SGI40286[] = {{&S38945[81], 5}, {NULL}};
const Segment SGI40287[] = {{&S38984[44], 7}, {NULL}};
const Segment SGI40288[] = {{&S38984[44], 32}, {NULL}};
const Segment SGI40289[] = {{&S38984[30], 6}, {NULL}};
const Segment SGI40290[] = {{&S38984[124], 3}, {NULL}};
const Segment SGI40291[] = {{&S38984[90], 5}, {NULL}};
const Segment SGI40292[] = {{&S38985[44], 7}, {NULL}};
const Segment SGI40293[] = {{&S38985[44], 29}, {NULL}};
const Segment SGI40294[] = {{&S38985[30], 6}, {NULL}};
const Segment SGI40295[] = {{&S38985[121], 3}, {NULL}};
const Segment SGI40296[] = {{&S38985[87], 5}, {NULL}};
const Segment SGI40297[] = {{&S38986[38], 7}, {NULL}};
const Segment SGI40298[] = {{&S38986[31], 6}, {NULL}};
const Segment SGI40299[] = {{&S51473[44], 16}, {NULL}};
const Segment SGI40300[] = {{&S51473[44], 27}, {NULL}};
const Segment SGI40301[] = {{&S51473[30], 6}, {NULL}};
const Segment SGI40302[] = {{&S51473[119], 3}, {NULL}};
const Segment SGI40303[] = {{&S51473[85], 5}, {NULL}};
const Segment SGI40304[] = {{&S51474[44], 9}, {NULL}};
const Segment SGI40305[] = {{&S51474[44], 20}, {NULL}};
const Segment SGI40306[] = {{&S51474[30], 6}, {NULL}};
const Segment SGI40307[] = {{&S51474[112], 3}, {NULL}};
const Segment SGI40308[] = {{&S51474[78], 5}, {NULL}};
const Segment SGI40309[] = {{&S51475[38], 9}, {NULL}};
const Segment SGI40310[] = {{&S51475[31], 6}, {NULL}};
const Segment SGI40311[] = {{&S39177[43], 13}, {NULL}};
const Segment SGI40312[] = {{&S39177[43], 24}, {NULL}};
const Segment SGI40313[] = {{&S39177[30], 5}, {NULL}};
const Segment SGI40314[] = {{&S39177[115], 3}, {NULL}};
const Segment SGI40315[] = {{&S39177[81], 5}, {NULL}};
const Segment SGI40316[] = {{&S39178[44], 12}, {NULL}};
const Segment SGI40317[] = {{&S39178[44], 23}, {NULL}};
const Segment SGI40318[] = {{&S39178[30], 6}, {NULL}};
const Segment SGI40319[] = {{&S39178[115], 3}, {NULL}};
const Segment SGI40320[] = {{&S39178[81], 5}, {NULL}};
const Segment SGI40321[] = {{&S39179[44], 12}, {NULL}};
const Segment SGI40322[] = {{&S39179[44], 23}, {NULL}};
const Segment SGI40323[] = {{&S39179[30], 6}, {NULL}};
const Segment SGI40324[] = {{&S39179[115], 3}, {NULL}};
const Segment SGI40325[] = {{&S39179[81], 5}, {NULL}};
const Segment SGI40326[] = {{&S39181[44], 13}, {NULL}};
const Segment SGI40327[] = {{&S39181[44], 24}, {NULL}};
const Segment SGI40328[] = {{&S39181[30], 6}, {NULL}};
const Segment SGI40329[] = {{&S39181[116], 3}, {NULL}};
const Segment SGI40330[] = {{&S39181[82], 5}, {NULL}};
const Segment SGI40331[] = {{&S39182[44], 13}, {NULL}};
const Segment SGI40332[] = {{&S39182[44], 24}, {NULL}};
const Segment SGI40333[] = {{&S39182[30], 6}, {NULL}};
const Segment SGI40334[] = {{&S39182[116], 3}, {NULL}};
const Segment SGI40335[] = {{&S39182[82], 5}, {NULL}};
const Segment SGI40336[] = {{&S39183[43], 13}, {NULL}};
const Segment SGI40337[] = {{&S39183[43], 24}, {NULL}};
const Segment SGI40338[] = {{&S39183[115], 3}, {NULL}};
const Segment SGI40339[] = {{&S39183[81], 5}, {NULL}};
const Segment SGI40340[] = {{&S39187[44], 13}, {NULL}};
const Segment SGI40341[] = {{&S39187[44], 24}, {NULL}};
const Segment SGI40342[] = {{&S39187[30], 6}, {NULL}};
const Segment SGI40343[] = {{&S39187[116], 3}, {NULL}};
const Segment SGI40344[] = {{&S39187[82], 5}, {NULL}};
const Segment SGI40345[] = {{&S39189[38], 13}, {NULL}};
const Segment SGI40346[] = {{&S39189[31], 6}, {NULL}};
const Segment SGI40347[] = {{&S39191[44], 13}, {NULL}};
const Segment SGI40348[] = {{&S39191[44], 24}, {NULL}};
const Segment SGI40349[] = {{&S39191[30], 6}, {NULL}};
const Segment SGI40350[] = {{&S39192[44], 12}, {NULL}};
const Segment SGI40351[] = {{&S39192[44], 23}, {NULL}};
const Segment SGI40352[] = {{&S39192[30], 6}, {NULL}};
const Segment SGI40353[] = {{&S39192[115], 3}, {NULL}};
const Segment SGI40354[] = {{&S39192[81], 5}, {NULL}};
const Segment SGI40355[] = {{&S39197[44], 13}, {NULL}};
const Segment SGI40356[] = {{&S39197[44], 24}, {NULL}};
const Segment SGI40357[] = {{&S39197[30], 6}, {NULL}};
const Segment SGI40358[] = {{&S39197[118], 3}, {NULL}};
const Segment SGI40359[] = {{&S39197[82], 7}, {NULL}};
const Segment SGI40360[] = {{&S39197[138], 8}, {NULL}};
const Segment SGI40361[] = {{&S39199[42], 13}, {NULL}};
const Segment SGI40362[] = {{&S39199[30], 6}, {NULL}};
const Segment SGI40363[] = {{&S39199[114], 3}, {NULL}};
const Segment SGI40364[] = {{&S39199[80], 5}, {NULL}};
const Segment SGI40365[] = {{&S39202[44], 17}, {NULL}};
const Segment SGI40366[] = {{&S39202[44], 28}, {NULL}};
const Segment SGI40367[] = {{&S39202[30], 6}, {NULL}};
const Segment SGI40368[] = {{&S39202[120], 3}, {NULL}};
const Segment SGI40369[] = {{&S39202[86], 5}, {NULL}};
const Segment SGI40370[] = {{&S39204[44], 13}, {NULL}};
const Segment SGI40371[] = {{&S39204[44], 24}, {NULL}};
const Segment SGI40372[] = {{&S39204[30], 6}, {NULL}};
const Segment SGI40373[] = {{&S39204[82], 6}, {NULL}};
const Segment SGI40374[] = {{&S46224[42], 8}, {NULL}};
const Segment SGI40375[] = {{&S46224[42], 25}, {NULL}};
const Segment SGI40376[] = {{&S46224[30], 4}, {NULL}};
const Segment SGI40377[] = {{&S46224[116], 3}, {NULL}};
const Segment SGI40378[] = {{&S46224[81], 6}, {NULL}};
const Segment SGI40379[] = {{&S39184[44], 13}, {NULL}};
const Segment SGI40380[] = {{&S39184[44], 24}, {NULL}};
const Segment SGI40381[] = {{&S39184[30], 6}, {NULL}};
const Segment SGI40382[] = {{&S39184[116], 3}, {NULL}};
const Segment SGI40383[] = {{&S39184[82], 5}, {NULL}};
const Segment SGI40384[] = {{&S39188[44], 13}, {NULL}};
const Segment SGI40385[] = {{&S39188[44], 24}, {NULL}};
const Segment SGI40386[] = {{&S39188[30], 6}, {NULL}};
const Segment SGI40387[] = {{&S39188[116], 3}, {NULL}};
const Segment SGI40388[] = {{&S39188[82], 5}, {NULL}};
const Segment SGI40389[] = {{&S39193[44], 13}, {NULL}};
const Segment SGI40390[] = {{&S39193[44], 24}, {NULL}};
const Segment SGI40391[] = {{&S39193[30], 6}, {NULL}};
const Segment SGI40392[] = {{&S39193[116], 3}, {NULL}};
const Segment SGI40393[] = {{&S39193[82], 5}, {NULL}};
const Segment SGI40394[] = {{&S39196[44], 13}, {NULL}};
const Segment SGI40395[] = {{&S39196[44], 24}, {NULL}};
const Segment SGI40396[] = {{&S39196[30], 6}, {NULL}};
const Segment SGI40397[] = {{&S46511[6], 4}, {NULL}};
const Segment SGI40398[] = {{&S46511[83], 5}, {NULL}};
const Segment SGI40399[] = {{&S39196[116], 3}, {NULL}};
const Segment SGI40400[] = {{&S39196[82], 5}, {NULL}};
const Segment SGI40401[] = {{&S39200[38], 13}, {NULL}};
const Segment SGI40402[] = {{&S39200[31], 6}, {NULL}};
const Segment SGI40403[] = {{&S51772[44], 13}, {NULL}};
const Segment SGI40404[] = {{&S51772[44], 25}, {NULL}};
const Segment SGI40405[] = {{&S51772[30], 6}, {NULL}};
const Segment SGI40406[] = {{&S51772[117], 3}, {NULL}};
const Segment SGI40407[] = {{&S51772[83], 5}, {NULL}};
const Segment SGI40408[] = {{&S51773[44], 8}, {NULL}};
const Segment SGI40409[] = {{&S51773[44], 20}, {NULL}};
const Segment SGI40410[] = {{&S51773[30], 6}, {NULL}};
const Segment SGI40411[] = {{&S51773[112], 3}, {NULL}};
const Segment SGI40412[] = {{&S51773[78], 5}, {NULL}};
const Segment SGI40413[] = {{&S51778[38], 13}, {NULL}};
const Segment SGI40414[] = {{&S51778[31], 6}, {NULL}};
const Segment SGI40415[] = {{&S39465[44], 10}, {NULL}};
const Segment SGI40416[] = {{&S39465[44], 27}, {NULL}};
const Segment SGI40417[] = {{&S39465[30], 6}, {NULL}};
const Segment SGI40418[] = {{&S39465[119], 3}, {NULL}};
const Segment SGI40419[] = {{&S39465[85], 5}, {NULL}};
const Segment SGI40420[] = {{&S39466[38], 10}, {NULL}};
const Segment SGI40421[] = {{&S39466[31], 6}, {NULL}};
const Segment SGI40422[] = {{&S39467[44], 10}, {NULL}};
const Segment SGI40423[] = {{&S39467[44], 28}, {NULL}};
const Segment SGI40424[] = {{&S39467[30], 6}, {NULL}};
const Segment SGI40425[] = {{&S39467[120], 3}, {NULL}};
const Segment SGI40426[] = {{&S39467[86], 5}, {NULL}};
const Segment SGI40427[] = {{&S39468[42], 10}, {NULL}};
const Segment SGI40428[] = {{&S39468[30], 6}, {NULL}};
const Segment SGI40429[] = {{&S39468[117], 3}, {NULL}};
const Segment SGI40430[] = {{&S39468[83], 5}, {NULL}};
const Segment SGI40431[] = {{&S39469[44], 10}, {NULL}};
const Segment SGI40432[] = {{&S39469[44], 27}, {NULL}};
const Segment SGI40433[] = {{&S39469[30], 6}, {NULL}};
const Segment SGI40434[] = {{&S39469[85], 6}, {NULL}};
const Segment SGI40435[] = {{&S46510[6], 4}, {NULL}};
const Segment SGI40436[] = {{&S46510[83], 5}, {NULL}};
const Segment SGI40437[] = {{&S39769[44], 11}, {NULL}};
const Segment SGI40438[] = {{&S39769[44], 22}, {NULL}};
const Segment SGI40439[] = {{&S39769[30], 6}, {NULL}};
const Segment SGI40440[] = {{&S39769[114], 3}, {NULL}};
const Segment SGI40441[] = {{&S39769[80], 5}, {NULL}};
const Segment SGI40442[] = {{&S39770[38], 11}, {NULL}};
const Segment SGI40443[] = {{&S39770[31], 6}, {NULL}};
const Segment SGI40444[] = {{&S39771[44], 10}, {NULL}};
const Segment SGI40445[] = {{&S39771[44], 21}, {NULL}};
const Segment SGI40446[] = {{&S39771[30], 6}, {NULL}};
const Segment SGI40447[] = {{&S39771[113], 3}, {NULL}};
const Segment SGI40448[] = {{&S39771[79], 5}, {NULL}};
const Segment SGI40449[] = {{&S39772[43], 11}, {NULL}};
const Segment SGI40450[] = {{&S39772[43], 22}, {NULL}};
const Segment SGI40451[] = {{&S39772[113], 3}, {NULL}};
const Segment SGI40452[] = {{&S39772[79], 5}, {NULL}};
const Segment SGI40453[] = {{&S39773[44], 11}, {NULL}};
const Segment SGI40454[] = {{&S39773[44], 22}, {NULL}};
const Segment SGI40455[] = {{&S39773[30], 6}, {NULL}};
const Segment SGI40456[] = {{&S39773[114], 3}, {NULL}};
const Segment SGI40457[] = {{&S39773[80], 5}, {NULL}};
const Segment SGI40458[] = {{&S39774[43], 11}, {NULL}};
const Segment SGI40459[] = {{&S39774[43], 22}, {NULL}};
const Segment SGI40460[] = {{&S39774[30], 5}, {NULL}};
const Segment SGI40461[] = {{&S39774[113], 3}, {NULL}};
const Segment SGI40462[] = {{&S39774[79], 5}, {NULL}};
const Segment SGI40463[] = {{&S39776[44], 8}, {NULL}};
const Segment SGI40464[] = {{&S39776[44], 22}, {NULL}};
const Segment SGI40465[] = {{&S39776[30], 6}, {NULL}};
const Segment SGI40466[] = {{&S39776[114], 3}, {NULL}};
const Segment SGI40467[] = {{&S39776[80], 5}, {NULL}};
const Segment SGI40468[] = {{&S39777[44], 7}, {NULL}};
const Segment SGI40469[] = {{&S39777[44], 29}, {NULL}};
const Segment SGI40470[] = {{&S39777[30], 6}, {NULL}};
const Segment SGI40471[] = {{&S39777[121], 3}, {NULL}};
const Segment SGI40472[] = {{&S39777[87], 5}, {NULL}};
const Segment SGI40473[] = {{&S39779[44], 8}, {NULL}};
const Segment SGI40474[] = {{&S39779[44], 29}, {NULL}};
const Segment SGI40475[] = {{&S39779[30], 6}, {NULL}};
const Segment SGI40476[] = {{&S39779[121], 3}, {NULL}};
const Segment SGI40477[] = {{&S39779[87], 5}, {NULL}};
const Segment SGI40478[] = {{&S39780[44], 8}, {NULL}};
const Segment SGI40479[] = {{&S39780[44], 30}, {NULL}};
const Segment SGI40480[] = {{&S39780[30], 6}, {NULL}};
const Segment SGI40481[] = {{&S39780[122], 3}, {NULL}};
const Segment SGI40482[] = {{&S39780[88], 5}, {NULL}};
const Segment SGI40483[] = {{&S39783[44], 7}, {NULL}};
const Segment SGI40484[] = {{&S39783[44], 18}, {NULL}};
const Segment SGI40485[] = {{&S39783[30], 6}, {NULL}};
const Segment SGI40486[] = {{&S39783[110], 3}, {NULL}};
const Segment SGI40487[] = {{&S39783[76], 5}, {NULL}};
const Segment SGI40488[] = {{&S52155[44], 23}, {NULL}};
const Segment SGI40489[] = {{&S52155[44], 35}, {NULL}};
const Segment SGI40490[] = {{&S52155[30], 6}, {NULL}};
const Segment SGI40491[] = {{&S52155[127], 3}, {NULL}};
const Segment SGI40492[] = {{&S52155[93], 5}, {NULL}};
const Segment SGI40493[] = {{&S39793[44], 8}, {NULL}};
const Segment SGI40494[] = {{&S39793[44], 29}, {NULL}};
const Segment SGI40495[] = {{&S39793[30], 6}, {NULL}};
const Segment SGI40496[] = {{&S52156[44], 24}, {NULL}};
const Segment SGI40497[] = {{&S52156[44], 36}, {NULL}};
const Segment SGI40498[] = {{&S39793[90], 4}, {NULL}};
const Segment SGI40499[] = {{&S52156[30], 6}, {NULL}};
const Segment SGI40500[] = {{&S39793[90], 3}, {NULL}};
const Segment SGI40501[] = {{&S52156[128], 3}, {NULL}};
const Segment SGI40502[] = {{&S52156[94], 5}, {NULL}};
const Segment SGI40503[] = {{&S39794[44], 7}, {NULL}};
const Segment SGI40504[] = {{&S39794[44], 18}, {NULL}};
const Segment SGI40505[] = {{&S39794[30], 6}, {NULL}};
const Segment SGI40506[] = {{&S39794[79], 4}, {NULL}};
const Segment SGI40507[] = {{&S52157[44], 24}, {NULL}};
const Segment SGI40508[] = {{&S39794[79], 3}, {NULL}};
const Segment SGI40509[] = {{&S52157[44], 36}, {NULL}};
const Segment SGI40510[] = {{&S52157[30], 6}, {NULL}};
const Segment SGI40511[] = {{&S52157[128], 3}, {NULL}};
const Segment SGI40512[] = {{&S52157[94], 5}, {NULL}};
const Segment SGI40513[] = {{&S52158[38], 23}, {NULL}};
const Segment SGI40514[] = {{&S52158[31], 6}, {NULL}};
const Segment SGI40515[] = {{&S39796[44], 8}, {NULL}};
const Segment SGI40516[] = {{&S39796[44], 29}, {NULL}};
const Segment SGI40517[] = {{&S39796[30], 6}, {NULL}};
const Segment SGI40518[] = {{&S39796[90], 6}, {NULL}};
const Segment SGI40519[] = {{&S39802[38], 7}, {NULL}};
const Segment SGI40520[] = {{&S39802[31], 6}, {NULL}};
const Segment SGI40521[] = {{&S39805[44], 7}, {NULL}};
const Segment SGI40522[] = {{&S39805[44], 19}, {NULL}};
const Segment SGI40523[] = {{&S39805[30], 6}, {NULL}};
const Segment SGI40524[] = {{&S39805[111], 3}, {NULL}};
const Segment SGI40525[] = {{&S39805[77], 5}, {NULL}};
const Segment SGI40526[] = {{&S39807[44], 7}, {NULL}};
const Segment SGI40527[] = {{&S39807[44], 29}, {NULL}};
const Segment SGI40528[] = {{&S39807[30], 6}, {NULL}};
const Segment SGI40529[] = {{&S39807[121], 3}, {NULL}};
const Segment SGI40530[] = {{&S39807[87], 5}, {NULL}};
const Segment SGI40531[] = {{&S39809[44], 8}, {NULL}};
const Segment SGI40532[] = {{&S39809[44], 30}, {NULL}};
const Segment SGI40533[] = {{&S39809[30], 6}, {NULL}};
const Segment SGI40534[] = {{&S39809[91], 4}, {NULL}};
const Segment SGI40535[] = {{&S39809[91], 3}, {NULL}};
const Segment SGI40536[] = {{&S41797[44], 17}, {NULL}};
const Segment SGI40537[] = {{&S41797[44], 52}, {NULL}};
const Segment SGI40538[] = {{&S41797[30], 6}, {NULL}};
const Segment SGI40539[] = {{&S41797[144], 3}, {NULL}};
const Segment SGI40540[] = {{&S41797[110], 5}, {NULL}};
const Segment SGI40541[] = {{&S41798[44], 17}, {NULL}};
const Segment SGI40542[] = {{&S41798[44], 28}, {NULL}};
const Segment SGI40543[] = {{&S41798[30], 6}, {NULL}};
const Segment SGI40544[] = {{&S41798[120], 3}, {NULL}};
const Segment SGI40545[] = {{&S41798[86], 5}, {NULL}};
const Segment SGI40546[] = {{&S41799[44], 17}, {NULL}};
const Segment SGI40547[] = {{&S41799[44], 28}, {NULL}};
const Segment SGI40548[] = {{&S41799[30], 6}, {NULL}};
const Segment SGI40549[] = {{&S41799[120], 3}, {NULL}};
const Segment SGI40550[] = {{&S41799[86], 5}, {NULL}};
const Segment SGI40551[] = {{&S41800[44], 17}, {NULL}};
const Segment SGI40552[] = {{&S41800[44], 28}, {NULL}};
const Segment SGI40553[] = {{&S41800[30], 6}, {NULL}};
const Segment SGI40554[] = {{&S41800[120], 3}, {NULL}};
const Segment SGI40555[] = {{&S41800[86], 5}, {NULL}};
const Segment SGI40556[] = {{&S41801[44], 17}, {NULL}};
const Segment SGI40557[] = {{&S41801[44], 28}, {NULL}};
const Segment SGI40558[] = {{&S41801[30], 6}, {NULL}};
const Segment SGI40559[] = {{&S41801[120], 3}, {NULL}};
const Segment SGI40560[] = {{&S41801[86], 5}, {NULL}};
const Segment SGI40561[] = {{&S41802[38], 17}, {NULL}};
const Segment SGI40562[] = {{&S41802[31], 6}, {NULL}};
const Segment SGI40563[] = {{&S41803[38], 17}, {NULL}};
const Segment SGI40564[] = {{&S41803[31], 6}, {NULL}};
const Segment SGI40565[] = {{&S41804[44], 11}, {NULL}};
const Segment SGI40566[] = {{&S41804[44], 22}, {NULL}};
const Segment SGI40567[] = {{&S41804[30], 6}, {NULL}};
const Segment SGI40568[] = {{&S41804[114], 3}, {NULL}};
const Segment SGI40569[] = {{&S41804[80], 5}, {NULL}};
const Segment SGI40570[] = {{&S41806[44], 10}, {NULL}};
const Segment SGI40571[] = {{&S41806[44], 21}, {NULL}};
const Segment SGI40572[] = {{&S41806[30], 6}, {NULL}};
const Segment SGI40573[] = {{&S41806[113], 3}, {NULL}};
const Segment SGI40574[] = {{&S41806[79], 5}, {NULL}};
const Segment SGI40575[] = {{&S41807[44], 17}, {NULL}};
const Segment SGI40576[] = {{&S41807[44], 28}, {NULL}};
const Segment SGI40577[] = {{&S41807[30], 6}, {NULL}};
const Segment SGI40578[] = {{&S41807[89], 4}, {NULL}};
const Segment SGI40579[] = {{&S41807[89], 3}, {NULL}};
const Segment SGI40580[] = {{&S41808[44], 11}, {NULL}};
const Segment SGI40581[] = {{&S41808[44], 22}, {NULL}};
const Segment SGI40582[] = {{&S41808[30], 6}, {NULL}};
const Segment SGI40583[] = {{&S41808[83], 4}, {NULL}};
const Segment SGI40584[] = {{&S41808[83], 3}, {NULL}};
const Segment SGI40585[] = {{&S41812[44], 17}, {NULL}};
const Segment SGI40586[] = {{&S41812[44], 28}, {NULL}};
const Segment SGI40587[] = {{&S41812[30], 6}, {NULL}};
const Segment SGI40588[] = {{&S41812[89], 6}, {NULL}};
const Segment SGI40589[] = {{&S41813[44], 11}, {NULL}};
const Segment SGI40590[] = {{&S41813[44], 22}, {NULL}};
const Segment SGI40591[] = {{&S41813[30], 6}, {NULL}};
const Segment SGI40592[] = {{&S41813[83], 6}, {NULL}};
const Segment SGI40593[] = {{&S39818[38], 8}, {NULL}};
const Segment SGI40594[] = {{&S39818[31], 6}, {NULL}};
const Segment SGI40595[] = {{&S39847[44], 7}, {NULL}};
const Segment SGI40596[] = {{&S39847[44], 31}, {NULL}};
const Segment SGI40597[] = {{&S39847[30], 6}, {NULL}};
const Segment SGI40598[] = {{&S39847[123], 3}, {NULL}};
const Segment SGI40599[] = {{&S39847[89], 5}, {NULL}};
const Segment SGI40600[] = {{&S39848[44], 7}, {NULL}};
const Segment SGI40601[] = {{&S39848[44], 31}, {NULL}};
const Segment SGI40602[] = {{&S39848[30], 6}, {NULL}};
const Segment SGI40603[] = {{&S39848[123], 3}, {NULL}};
const Segment SGI40604[] = {{&S39848[89], 5}, {NULL}};
const Segment SGI40605[] = {{&S39849[44], 7}, {NULL}};
const Segment SGI40606[] = {{&S39849[44], 29}, {NULL}};
const Segment SGI40607[] = {{&S39849[30], 6}, {NULL}};
const Segment SGI40608[] = {{&S39849[121], 3}, {NULL}};
const Segment SGI40609[] = {{&S39849[87], 5}, {NULL}};
const Segment SGI40610[] = {{&S39854[44], 7}, {NULL}};
const Segment SGI40611[] = {{&S39854[44], 29}, {NULL}};
const Segment SGI40612[] = {{&S39854[30], 6}, {NULL}};
const Segment SGI40613[] = {{&S39854[121], 3}, {NULL}};
const Segment SGI40614[] = {{&S39854[87], 5}, {NULL}};
const Segment SGI40615[] = {{&S39855[44], 7}, {NULL}};
const Segment SGI40616[] = {{&S39855[44], 29}, {NULL}};
const Segment SGI40617[] = {{&S39855[30], 6}, {NULL}};
const Segment SGI40618[] = {{&S39855[121], 3}, {NULL}};
const Segment SGI40619[] = {{&S39855[87], 5}, {NULL}};
const Segment SGI40620[] = {{&S39856[44], 7}, {NULL}};
const Segment SGI40621[] = {{&S39856[44], 29}, {NULL}};
const Segment SGI40622[] = {{&S39856[30], 6}, {NULL}};
const Segment SGI40623[] = {{&S39856[121], 3}, {NULL}};
const Segment SGI40624[] = {{&S39856[87], 5}, {NULL}};
const Segment SGI40625[] = {{&S39861[44], 7}, {NULL}};
const Segment SGI40626[] = {{&S39861[44], 31}, {NULL}};
const Segment SGI40627[] = {{&S39861[30], 6}, {NULL}};
const Segment SGI40628[] = {{&S39861[123], 3}, {NULL}};
const Segment SGI40629[] = {{&S39861[89], 5}, {NULL}};
const Segment SGI40630[] = {{&S39865[44], 7}, {NULL}};
const Segment SGI40631[] = {{&S39865[44], 29}, {NULL}};
const Segment SGI40632[] = {{&S39865[30], 6}, {NULL}};
const Segment SGI40633[] = {{&S39865[121], 3}, {NULL}};
const Segment SGI40634[] = {{&S39865[87], 5}, {NULL}};
const Segment SGI40635[] = {{&S39869[44], 7}, {NULL}};
const Segment SGI40636[] = {{&S39869[44], 34}, {NULL}};
const Segment SGI40637[] = {{&S39869[30], 6}, {NULL}};
const Segment SGI40638[] = {{&S39869[126], 3}, {NULL}};
const Segment SGI40639[] = {{&S39869[92], 5}, {NULL}};
const Segment SGI40640[] = {{&S39781[44], 7}, {NULL}};
const Segment SGI40641[] = {{&S39781[44], 18}, {NULL}};
const Segment SGI40642[] = {{&S39781[30], 6}, {NULL}};
const Segment SGI40643[] = {{&S39781[110], 3}, {NULL}};
const Segment SGI40644[] = {{&S39781[76], 5}, {NULL}};
const Segment SGI40645[] = {{&S39803[44], 7}, {NULL}};
const Segment SGI40646[] = {{&S39803[44], 19}, {NULL}};
const Segment SGI40647[] = {{&S39803[30], 6}, {NULL}};
const Segment SGI40648[] = {{&S39803[111], 3}, {NULL}};
const Segment SGI40649[] = {{&S39803[77], 5}, {NULL}};
const Segment SGI40650[] = {{&S39804[38], 7}, {NULL}};
const Segment SGI40651[] = {{&S39804[31], 6}, {NULL}};
const Segment SGI40652[] = {{&S39806[44], 7}, {NULL}};
const Segment SGI40653[] = {{&S39806[44], 19}, {NULL}};
const Segment SGI40654[] = {{&S39806[30], 6}, {NULL}};
const Segment SGI40655[] = {{&S39806[80], 4}, {NULL}};
const Segment SGI40656[] = {{&S39806[80], 3}, {NULL}};
const Segment SGI40657[] = {{&S39857[44], 7}, {NULL}};
const Segment SGI40658[] = {{&S39857[44], 27}, {NULL}};
const Segment SGI40659[] = {{&S39857[30], 6}, {NULL}};
const Segment SGI40660[] = {{&S39857[119], 3}, {NULL}};
const Segment SGI40661[] = {{&S39857[85], 5}, {NULL}};
const Segment SGI40662[] = {{&S39858[44], 7}, {NULL}};
const Segment SGI40663[] = {{&S39858[44], 30}, {NULL}};
const Segment SGI40664[] = {{&S39858[30], 6}, {NULL}};
const Segment SGI40665[] = {{&S39858[122], 3}, {NULL}};
const Segment SGI40666[] = {{&S39858[88], 5}, {NULL}};
const Segment SGI40667[] = {{&S39859[44], 7}, {NULL}};
const Segment SGI40668[] = {{&S39859[44], 30}, {NULL}};
const Segment SGI40669[] = {{&S39859[30], 6}, {NULL}};
const Segment SGI40670[] = {{&S39859[122], 3}, {NULL}};
const Segment SGI40671[] = {{&S39859[88], 5}, {NULL}};
const Segment SGI40672[] = {{&S39860[44], 7}, {NULL}};
const Segment SGI40673[] = {{&S39860[44], 30}, {NULL}};
const Segment SGI40674[] = {{&S39860[30], 6}, {NULL}};
const Segment SGI40675[] = {{&S41966[44], 9}, {NULL}};
const Segment SGI40676[] = {{&S41966[44], 20}, {NULL}};
const Segment SGI40677[] = {{&S39860[122], 3}, {NULL}};
const Segment SGI40678[] = {{&S41966[30], 6}, {NULL}};
const Segment SGI40679[] = {{&S39860[88], 5}, {NULL}};
const Segment SGI40680[] = {{&S41966[112], 3}, {NULL}};
const Segment SGI40681[] = {{&S39995[44], 7}, {NULL}};
const Segment SGI40682[] = {{&S41966[78], 5}, {NULL}};
const Segment SGI40683[] = {{&S39995[44], 18}, {NULL}};
const Segment SGI40684[] = {{&S39995[30], 6}, {NULL}};
const Segment SGI40685[] = {{&S39995[110], 3}, {NULL}};
const Segment SGI40686[] = {{&S39995[76], 5}, {NULL}};
const Segment SGI40687[] = {{&S39996[38], 7}, {NULL}};
const Segment SGI40688[] = {{&S39996[31], 6}, {NULL}};
const Segment SGI40689[] = {{&S39863[44], 7}, {NULL}};
const Segment SGI40690[] = {{&S39863[44], 27}, {NULL}};
const Segment SGI40691[] = {{&S39863[30], 6}, {NULL}};
const Segment SGI40692[] = {{&S39863[119], 3}, {NULL}};
const Segment SGI40693[] = {{&S39863[85], 5}, {NULL}};
const Segment SGI40694[] = {{&S39997[44], 7}, {NULL}};
const Segment SGI40695[] = {{&S39997[44], 18}, {NULL}};
const Segment SGI40696[] = {{&S39997[30], 6}, {NULL}};
const Segment SGI40697[] = {{&S39997[79], 4}, {NULL}};
const Segment SGI40698[] = {{&S39997[79], 3}, {NULL}};
const Segment SGI40699[] = {{&S39864[44], 7}, {NULL}};
const Segment SGI40700[] = {{&S39864[44], 27}, {NULL}};
const Segment SGI40701[] = {{&S39864[30], 6}, {NULL}};
const Segment SGI40702[] = {{&S39864[119], 3}, {NULL}};
const Segment SGI40703[] = {{&S39864[85], 5}, {NULL}};
const Segment SGI40704[] = {{&S39998[44], 7}, {NULL}};
const Segment SGI40705[] = {{&S39998[44], 18}, {NULL}};
const Segment SGI40706[] = {{&S39998[30], 6}, {NULL}};
const Segment SGI40707[] = {{&S39998[121], 3}, {NULL}};
const Segment SGI40708[] = {{&S39998[86], 6}, {NULL}};
const Segment SGI40709[] = {{&S39998[141], 8}, {NULL}};
const Segment SGI40710[] = {{&S39999[44], 7}, {NULL}};
const Segment SGI40711[] = {{&S39999[44], 18}, {NULL}};
const Segment SGI40712[] = {{&S39999[30], 6}, {NULL}};
const Segment SGI40713[] = {{&S39999[79], 6}, {NULL}};
const Segment SGI40714[] = {{&S39870[44], 27}, {NULL}};
const Segment SGI40715[] = {{&S39870[30], 6}, {NULL}};
const Segment SGI40716[] = {{&S39872[44], 10}, {NULL}};
const Segment SGI40717[] = {{&S39872[44], 27}, {NULL}};
const Segment SGI40718[] = {{&S39872[30], 6}, {NULL}};
const Segment SGI40719[] = {{&S39872[119], 3}, {NULL}};
const Segment SGI40720[] = {{&S39872[85], 5}, {NULL}};
const Segment SGI40721[] = {{&S39874[38], 10}, {NULL}};
const Segment SGI40722[] = {{&S39874[31], 6}, {NULL}};
const Segment SGI40723[] = {{&S40019[44], 18}, {NULL}};
const Segment SGI40724[] = {{&S40019[44], 29}, {NULL}};
const Segment SGI40725[] = {{&S40019[30], 6}, {NULL}};
const Segment SGI40726[] = {{&S40019[121], 3}, {NULL}};
const Segment SGI40727[] = {{&S40019[87], 5}, {NULL}};
const Segment SGI40728[] = {{&S40020[44], 17}, {NULL}};
const Segment SGI40729[] = {{&S40020[44], 28}, {NULL}};
const Segment SGI40730[] = {{&S40020[30], 6}, {NULL}};
const Segment SGI40731[] = {{&S40020[120], 3}, {NULL}};
const Segment SGI40732[] = {{&S40020[86], 5}, {NULL}};
const Segment SGI40733[] = {{&S40021[44], 17}, {NULL}};
const Segment SGI40734[] = {{&S40021[44], 28}, {NULL}};
const Segment SGI40735[] = {{&S40021[30], 6}, {NULL}};
const Segment SGI40736[] = {{&S40021[120], 3}, {NULL}};
const Segment SGI40737[] = {{&S40021[86], 5}, {NULL}};
const Segment SGI40738[] = {{&S40022[44], 13}, {NULL}};
const Segment SGI40739[] = {{&S40022[44], 24}, {NULL}};
const Segment SGI40740[] = {{&S40022[30], 6}, {NULL}};
const Segment SGI40741[] = {{&S40022[116], 3}, {NULL}};
const Segment SGI40742[] = {{&S40022[82], 5}, {NULL}};
const Segment SGI40743[] = {{&S40024[44], 11}, {NULL}};
const Segment SGI40744[] = {{&S40024[44], 22}, {NULL}};
const Segment SGI40745[] = {{&S40024[30], 6}, {NULL}};
const Segment SGI40746[] = {{&S40024[114], 3}, {NULL}};
const Segment SGI40747[] = {{&S40024[80], 5}, {NULL}};
const Segment SGI40748[] = {{&S40025[44], 23}, {NULL}};
const Segment SGI40749[] = {{&S40025[44], 34}, {NULL}};
const Segment SGI40750[] = {{&S40025[30], 6}, {NULL}};
const Segment SGI40751[] = {{&S40025[126], 3}, {NULL}};
const Segment SGI40752[] = {{&S40025[92], 5}, {NULL}};
const Segment SGI40753[] = {{&S40026[44], 18}, {NULL}};
const Segment SGI40754[] = {{&S40026[44], 29}, {NULL}};
const Segment SGI40755[] = {{&S40026[30], 6}, {NULL}};
const Segment SGI40756[] = {{&S40026[121], 3}, {NULL}};
const Segment SGI40757[] = {{&S40026[87], 5}, {NULL}};
const Segment SGI40758[] = {{&S40030[38], 11}, {NULL}};
const Segment SGI40759[] = {{&S40030[31], 6}, {NULL}};
const Segment SGI40760[] = {{&S40031[44], 18}, {NULL}};
const Segment SGI40761[] = {{&S40031[44], 29}, {NULL}};
const Segment SGI40762[] = {{&S40031[30], 6}, {NULL}};
const Segment SGI40763[] = {{&S40031[87], 6}, {NULL}};
const Segment SGI40764[] = {{&S40032[38], 18}, {NULL}};
const Segment SGI40765[] = {{&S40032[31], 6}, {NULL}};
const Segment SGI40766[] = {{&S40033[38], 18}, {NULL}};
const Segment SGI40767[] = {{&S40033[31], 6}, {NULL}};
const Segment SGI40768[] = {{&S40034[44], 18}, {NULL}};
const Segment SGI40769[] = {{&S40034[44], 29}, {NULL}};
const Segment SGI40770[] = {{&S40034[30], 6}, {NULL}};
const Segment SGI40771[] = {{&S40034[90], 4}, {NULL}};
const Segment SGI40772[] = {{&S40034[90], 3}, {NULL}};
const Segment SGI40773[] = {{&S40035[44], 23}, {NULL}};
const Segment SGI40774[] = {{&S40035[44], 34}, {NULL}};
const Segment SGI40775[] = {{&S40035[30], 6}, {NULL}};
const Segment SGI40776[] = {{&S40035[126], 3}, {NULL}};
const Segment SGI40777[] = {{&S40035[92], 5}, {NULL}};
const Segment SGI40778[] = {{&S40036[44], 18}, {NULL}};
const Segment SGI40779[] = {{&S40036[44], 29}, {NULL}};
const Segment SGI40780[] = {{&S40036[30], 6}, {NULL}};
const Segment SGI40781[] = {{&S40036[90], 6}, {NULL}};
const Segment SGI40782[] = {{&S40037[44], 11}, {NULL}};
const Segment SGI40783[] = {{&S40037[44], 22}, {NULL}};
const Segment SGI40784[] = {{&S40037[30], 6}, {NULL}};
const Segment SGI40785[] = {{&S40037[83], 4}, {NULL}};
const Segment SGI40786[] = {{&S40037[83], 3}, {NULL}};
const Segment SGI40787[] = {{&S40038[44], 11}, {NULL}};
const Segment SGI40788[] = {{&S40038[44], 22}, {NULL}};
const Segment SGI40789[] = {{&S40038[30], 6}, {NULL}};
const Segment SGI40790[] = {{&S40038[114], 3}, {NULL}};
const Segment SGI40791[] = {{&S40038[80], 5}, {NULL}};
const Segment SGI40792[] = {{&S40041[44], 11}, {NULL}};
const Segment SGI40793[] = {{&S40041[44], 22}, {NULL}};
const Segment SGI40794[] = {{&S40041[30], 6}, {NULL}};
const Segment SGI40795[] = {{&S40041[83], 6}, {NULL}};
const Segment SGI40796[] = {{&S40043[44], 29}, {NULL}};
const Segment SGI40797[] = {{&S40043[30], 6}, {NULL}};
const Segment SGI40798[] = {{&S41968[38], 9}, {NULL}};
const Segment SGI40799[] = {{&S41968[31], 6}, {NULL}};
const Segment SGI40800[] = {{&S41970[44], 9}, {NULL}};
const Segment SGI40801[] = {{&S41970[44], 20}, {NULL}};
const Segment SGI40802[] = {{&S41970[30], 6}, {NULL}};
const Segment SGI40803[] = {{&S41970[112], 3}, {NULL}};
const Segment SGI40804[] = {{&S41970[78], 5}, {NULL}};
const Segment SGI40805[] = {{&S41976[44], 10}, {NULL}};
const Segment SGI40806[] = {{&S41976[44], 21}, {NULL}};
const Segment SGI40807[] = {{&S41976[30], 6}, {NULL}};
const Segment SGI40808[] = {{&S41976[113], 3}, {NULL}};
const Segment SGI40809[] = {{&S41976[79], 5}, {NULL}};
const Segment SGI40810[] = {{&S40039[44], 18}, {NULL}};
const Segment SGI40811[] = {{&S40039[44], 29}, {NULL}};
const Segment SGI40812[] = {{&S40039[30], 6}, {NULL}};
const Segment SGI40813[] = {{&S40039[90], 6}, {NULL}};
const Segment SGI40814[] = {{&S40042[44], 18}, {NULL}};
const Segment SGI40815[] = {{&S40042[44], 29}, {NULL}};
const Segment SGI40816[] = {{&S40042[30], 6}, {NULL}};
const Segment SGI40817[] = {{&S40042[90], 4}, {NULL}};
const Segment SGI40818[] = {{&S40042[90], 3}, {NULL}};
const Segment SGI40819[] = {{&S40044[44], 18}, {NULL}};
const Segment SGI40820[] = {{&S40044[44], 29}, {NULL}};
const Segment SGI40821[] = {{&S40044[30], 6}, {NULL}};
const Segment SGI40822[] = {{&S40044[121], 3}, {NULL}};
const Segment SGI40823[] = {{&S40044[87], 5}, {NULL}};
const Segment SGI40824[] = {{&S44381[44], 9}, {NULL}};
const Segment SGI40825[] = {{&S44381[44], 21}, {NULL}};
const Segment SGI40826[] = {{&S44381[30], 6}, {NULL}};
const Segment SGI40827[] = {{&S44381[113], 3}, {NULL}};
const Segment SGI40828[] = {{&S44381[79], 5}, {NULL}};
const Segment SGI40829[] = {{&S40087[44], 11}, {NULL}};
const Segment SGI40830[] = {{&S40087[44], 22}, {NULL}};
const Segment SGI40831[] = {{&S40087[30], 6}, {NULL}};
const Segment SGI40832[] = {{&S40087[114], 3}, {NULL}};
const Segment SGI40833[] = {{&S40087[80], 5}, {NULL}};
const Segment SGI40834[] = {{&S40088[38], 11}, {NULL}};
const Segment SGI40835[] = {{&S40088[31], 6}, {NULL}};
const Segment SGI40836[] = {{&S40089[42], 11}, {NULL}};
const Segment SGI40837[] = {{&S40089[30], 6}, {NULL}};
const Segment SGI40838[] = {{&S40089[112], 3}, {NULL}};
const Segment SGI40839[] = {{&S40089[78], 5}, {NULL}};
const Segment SGI40840[] = {{&S44392[44], 9}, {NULL}};
const Segment SGI40841[] = {{&S44392[44], 20}, {NULL}};
const Segment SGI40842[] = {{&S44392[30], 6}, {NULL}};
const Segment SGI40843[] = {{&S44392[112], 3}, {NULL}};
const Segment SGI40844[] = {{&S44392[78], 5}, {NULL}};
const Segment SGI40845[] = {{&S44402[44], 5}, {NULL}};
const Segment SGI40846[] = {{&S44402[44], 17}, {NULL}};
const Segment SGI40847[] = {{&S44402[30], 6}, {NULL}};
const Segment SGI40848[] = {{&S44402[109], 3}, {NULL}};
const Segment SGI40849[] = {{&S44402[75], 5}, {NULL}};
const Segment SGI40850[] = {{&S40313[44], 8}, {NULL}};
const Segment SGI40851[] = {{&S40313[44], 19}, {NULL}};
const Segment SGI40852[] = {{&S40313[30], 6}, {NULL}};
const Segment SGI40853[] = {{&S40313[111], 3}, {NULL}};
const Segment SGI40854[] = {{&S40313[77], 5}, {NULL}};
const Segment SGI40855[] = {{&S40314[44], 8}, {NULL}};
const Segment SGI40856[] = {{&S40314[44], 19}, {NULL}};
const Segment SGI40857[] = {{&S40314[30], 6}, {NULL}};
const Segment SGI40858[] = {{&S40314[113], 3}, {NULL}};
const Segment SGI40859[] = {{&S40314[77], 7}, {NULL}};
const Segment SGI40860[] = {{&S44405[44], 21}, {NULL}};
const Segment SGI40861[] = {{&S40314[133], 8}, {NULL}};
const Segment SGI40862[] = {{&S44405[44], 32}, {NULL}};
const Segment SGI40863[] = {{&S44405[30], 6}, {NULL}};
const Segment SGI40864[] = {{&S44405[124], 3}, {NULL}};
const Segment SGI40865[] = {{&S44405[90], 5}, {NULL}};
const Segment SGI40866[] = {{&S40315[38], 8}, {NULL}};
const Segment SGI40867[] = {{&S40315[31], 6}, {NULL}};
const Segment SGI40868[] = {{&S40316[44], 8}, {NULL}};
const Segment SGI40869[] = {{&S40316[44], 27}, {NULL}};
const Segment SGI40870[] = {{&S40316[30], 6}, {NULL}};
const Segment SGI40871[] = {{&S40316[119], 3}, {NULL}};
const Segment SGI40872[] = {{&S40316[85], 5}, {NULL}};
const Segment SGI40873[] = {{&S40317[44], 8}, {NULL}};
const Segment SGI40874[] = {{&S40317[44], 27}, {NULL}};
const Segment SGI40875[] = {{&S40317[30], 6}, {NULL}};
const Segment SGI40876[] = {{&S40317[119], 3}, {NULL}};
const Segment SGI40877[] = {{&S40317[85], 5}, {NULL}};
const Segment SGI40878[] = {{&S40318[44], 8}, {NULL}};
const Segment SGI40879[] = {{&S40318[44], 25}, {NULL}};
const Segment SGI40880[] = {{&S40318[30], 6}, {NULL}};
const Segment SGI40881[] = {{&S40318[117], 3}, {NULL}};
const Segment SGI40882[] = {{&S40318[83], 5}, {NULL}};
const Segment SGI40883[] = {{&S40319[44], 8}, {NULL}};
const Segment SGI40884[] = {{&S40319[44], 27}, {NULL}};
const Segment SGI40885[] = {{&S40319[30], 6}, {NULL}};
const Segment SGI40886[] = {{&S40319[119], 3}, {NULL}};
const Segment SGI40887[] = {{&S40319[85], 5}, {NULL}};
const Segment SGI40888[] = {{&S40320[44], 8}, {NULL}};
const Segment SGI40889[] = {{&S40320[44], 27}, {NULL}};
const Segment SGI40890[] = {{&S40320[30], 6}, {NULL}};
const Segment SGI40891[] = {{&S40320[119], 3}, {NULL}};
const Segment SGI40892[] = {{&S40320[85], 5}, {NULL}};
const Segment SGI40893[] = {{&S42345[44], 19}, {NULL}};
const Segment SGI40894[] = {{&S42345[44], 36}, {NULL}};
const Segment SGI40895[] = {{&S42345[30], 6}, {NULL}};
const Segment SGI40896[] = {{&S42345[69], 3}, {NULL}};
const Segment SGI40897[] = {{&S42345[94], 5}, {NULL}};
const Segment SGI40898[] = {{&S42346[44], 7}, {NULL}};
const Segment SGI40899[] = {{&S42346[44], 24}, {NULL}};
const Segment SGI40900[] = {{&S42346[30], 6}, {NULL}};
const Segment SGI40901[] = {{&S42346[57], 3}, {NULL}};
const Segment SGI40902[] = {{&S42346[82], 5}, {NULL}};
const Segment SGI40903[] = {{&S42355[38], 7}, {NULL}};
const Segment SGI40904[] = {{&S42355[31], 6}, {NULL}};
const Segment SGI40905[] = {{&S40573[44], 10}, {NULL}};
const Segment SGI40906[] = {{&S40573[44], 21}, {NULL}};
const Segment SGI40907[] = {{&S40573[30], 6}, {NULL}};
const Segment SGI40908[] = {{&S40573[113], 3}, {NULL}};
const Segment SGI40909[] = {{&S40573[79], 5}, {NULL}};
const Segment SGI40910[] = {{&S40574[38], 10}, {NULL}};
const Segment SGI40911[] = {{&S40574[31], 6}, {NULL}};
const Segment SGI40912[] = {{&S40575[44], 9}, {NULL}};
const Segment SGI40913[] = {{&S40575[44], 20}, {NULL}};
const Segment SGI40914[] = {{&S40575[30], 6}, {NULL}};
const Segment SGI40915[] = {{&S40575[112], 3}, {NULL}};
const Segment SGI40916[] = {{&S40575[78], 5}, {NULL}};
const Segment SGI40917[] = {{&S40576[44], 10}, {NULL}};
const Segment SGI40918[] = {{&S40576[44], 21}, {NULL}};
const Segment SGI40919[] = {{&S40576[30], 6}, {NULL}};
const Segment SGI40920[] = {{&S40576[113], 3}, {NULL}};
const Segment SGI40921[] = {{&S40576[79], 5}, {NULL}};
const Segment SGI40922[] = {{&S40577[43], 10}, {NULL}};
const Segment SGI40923[] = {{&S40577[43], 21}, {NULL}};
const Segment SGI40924[] = {{&S40577[112], 3}, {NULL}};
const Segment SGI40925[] = {{&S40577[78], 5}, {NULL}};
const Segment SGI40926[] = {{&S40578[43], 10}, {NULL}};
const Segment SGI40927[] = {{&S40578[43], 21}, {NULL}};
const Segment SGI40928[] = {{&S40578[30], 5}, {NULL}};
const Segment SGI40929[] = {{&S40578[112], 3}, {NULL}};
const Segment SGI40930[] = {{&S40578[78], 5}, {NULL}};
const Segment SGI40931[] = {{&S40579[44], 9}, {NULL}};
const Segment SGI40932[] = {{&S40579[44], 20}, {NULL}};
const Segment SGI40933[] = {{&S40579[30], 6}, {NULL}};
const Segment SGI40934[] = {{&S40579[112], 3}, {NULL}};
const Segment SGI40935[] = {{&S40579[78], 5}, {NULL}};
const Segment SGI40936[] = {{&S40580[44], 10}, {NULL}};
const Segment SGI40937[] = {{&S40580[44], 21}, {NULL}};
const Segment SGI40938[] = {{&S40580[30], 6}, {NULL}};
const Segment SGI40939[] = {{&S40580[115], 3}, {NULL}};
const Segment SGI40940[] = {{&S40580[79], 7}, {NULL}};
const Segment SGI40941[] = {{&S40580[135], 8}, {NULL}};
const Segment SGI40942[] = {{&S40581[42], 10}, {NULL}};
const Segment SGI40943[] = {{&S40581[30], 6}, {NULL}};
const Segment SGI40944[] = {{&S40581[111], 3}, {NULL}};
const Segment SGI40945[] = {{&S40581[77], 5}, {NULL}};
const Segment SGI40946[] = {{&S40893[44], 10}, {NULL}};
const Segment SGI40947[] = {{&S40893[44], 27}, {NULL}};
const Segment SGI40948[] = {{&S40893[30], 6}, {NULL}};
const Segment SGI40949[] = {{&S40893[119], 3}, {NULL}};
const Segment SGI40950[] = {{&S40893[85], 5}, {NULL}};
const Segment SGI40951[] = {{&S40896[38], 10}, {NULL}};
const Segment SGI40952[] = {{&S40896[31], 6}, {NULL}};
const Segment SGI40953[] = {{&S40905[44], 19}, {NULL}};
const Segment SGI40954[] = {{&S40905[44], 37}, {NULL}};
const Segment SGI40955[] = {{&S40905[30], 6}, {NULL}};
const Segment SGI40956[] = {{&S40905[69], 3}, {NULL}};
const Segment SGI40957[] = {{&S40905[95], 5}, {NULL}};
const Segment SGI40958[] = {{&S40906[44], 19}, {NULL}};
const Segment SGI40959[] = {{&S40906[44], 38}, {NULL}};
const Segment SGI40960[] = {{&S40906[30], 6}, {NULL}};
const Segment SGI40961[] = {{&S40906[69], 3}, {NULL}};
const Segment SGI40962[] = {{&S40906[96], 5}, {NULL}};
const Segment SGI40963[] = {{&S40907[44], 19}, {NULL}};
const Segment SGI40964[] = {{&S40907[44], 37}, {NULL}};
const Segment SGI40965[] = {{&S40907[30], 6}, {NULL}};
const Segment SGI40966[] = {{&S40907[69], 3}, {NULL}};
const Segment SGI40967[] = {{&S40907[95], 5}, {NULL}};
const Segment SGI40968[] = {{&S40908[44], 7}, {NULL}};
const Segment SGI40969[] = {{&S40908[44], 26}, {NULL}};
const Segment SGI40970[] = {{&S40908[30], 6}, {NULL}};
const Segment SGI40971[] = {{&S40908[57], 3}, {NULL}};
const Segment SGI40972[] = {{&S40908[84], 5}, {NULL}};
const Segment SGI40973[] = {{&S40909[44], 7}, {NULL}};
const Segment SGI40974[] = {{&S40909[44], 25}, {NULL}};
const Segment SGI40975[] = {{&S40909[30], 6}, {NULL}};
const Segment SGI40976[] = {{&S40909[57], 3}, {NULL}};
const Segment SGI40977[] = {{&S40909[83], 5}, {NULL}};
const Segment SGI40978[] = {{&S40910[44], 19}, {NULL}};
const Segment SGI40979[] = {{&S40910[44], 37}, {NULL}};
const Segment SGI40980[] = {{&S40910[30], 6}, {NULL}};
const Segment SGI40981[] = {{&S40910[69], 3}, {NULL}};
const Segment SGI40982[] = {{&S40910[95], 5}, {NULL}};
const Segment SGI40983[] = {{&S40911[44], 19}, {NULL}};
const Segment SGI40984[] = {{&S40911[44], 37}, {NULL}};
const Segment SGI40985[] = {{&S40911[30], 6}, {NULL}};
const Segment SGI40986[] = {{&S40911[69], 3}, {NULL}};
const Segment SGI40987[] = {{&S40911[95], 5}, {NULL}};
const Segment SGI40988[] = {{&S40912[44], 7}, {NULL}};
const Segment SGI40989[] = {{&S40912[44], 25}, {NULL}};
const Segment SGI40990[] = {{&S40912[30], 6}, {NULL}};
const Segment SGI40991[] = {{&S40912[57], 3}, {NULL}};
const Segment SGI40992[] = {{&S40912[83], 5}, {NULL}};
const Segment SGI40993[] = {{&S40913[38], 7}, {NULL}};
const Segment SGI40994[] = {{&S40913[31], 6}, {NULL}};
const Segment SGI40995[] = {{&S40914[44], 19}, {NULL}};
const Segment SGI40996[] = {{&S40914[44], 36}, {NULL}};
const Segment SGI40997[] = {{&S40914[30], 6}, {NULL}};
const Segment SGI40998[] = {{&S40914[69], 3}, {NULL}};
const Segment SGI40999[] = {{&S40914[94], 5}, {NULL}};
const Segment SGI41000[] = {{&S40915[44], 7}, {NULL}};
const Segment SGI41001[] = {{&S40915[44], 25}, {NULL}};
const Segment SGI41002[] = {{&S40915[30], 6}, {NULL}};
const Segment SGI41003[] = {{&S40915[57], 3}, {NULL}};
const Segment SGI41004[] = {{&S40915[83], 5}, {NULL}};
const Segment SGI41005[] = {{&S40916[44], 7}, {NULL}};
const Segment SGI41006[] = {{&S40916[44], 24}, {NULL}};
const Segment SGI41007[] = {{&S40916[30], 6}, {NULL}};
const Segment SGI41008[] = {{&S40916[57], 3}, {NULL}};
const Segment SGI41009[] = {{&S40916[82], 5}, {NULL}};
const Segment SGI41010[] = {{&S40917[44], 7}, {NULL}};
const Segment SGI41011[] = {{&S40917[44], 25}, {NULL}};
const Segment SGI41012[] = {{&S40917[30], 6}, {NULL}};
const Segment SGI41013[] = {{&S40917[57], 3}, {NULL}};
const Segment SGI41014[] = {{&S40917[83], 5}, {NULL}};
const Segment SGI41015[] = {{&S48329[6], 4}, {NULL}};
const Segment SGI41016[] = {{&S48329[76], 5}, {NULL}};
const Segment SGI41017[] = {{&S40816[44], 10}, {NULL}};
const Segment SGI41018[] = {{&S40816[44], 21}, {NULL}};
const Segment SGI41019[] = {{&S40816[30], 6}, {NULL}};
const Segment SGI41020[] = {{&S40816[113], 3}, {NULL}};
const Segment SGI41021[] = {{&S40816[79], 5}, {NULL}};
const Segment SGI41022[] = {{&S40963[44], 18}, {NULL}};
const Segment SGI41023[] = {{&S40963[44], 35}, {NULL}};
const Segment SGI41024[] = {{&S40963[30], 6}, {NULL}};
const Segment SGI41025[] = {{&S40963[127], 3}, {NULL}};
const Segment SGI41026[] = {{&S40963[93], 5}, {NULL}};
const Segment SGI41027[] = {{&S40967[44], 7}, {NULL}};
const Segment SGI41028[] = {{&S40967[44], 24}, {NULL}};
const Segment SGI41029[] = {{&S40967[30], 6}, {NULL}};
const Segment SGI41030[] = {{&S40967[116], 3}, {NULL}};
const Segment SGI41031[] = {{&S40967[82], 5}, {NULL}};
const Segment SGI41032[] = {{&S40970[38], 7}, {NULL}};
const Segment SGI41033[] = {{&S40970[31], 6}, {NULL}};
const Segment SGI41034[] = {{&S48394[44], 19}, {NULL}};
const Segment SGI41035[] = {{&S48394[44], 35}, {NULL}};
const Segment SGI41036[] = {{&S48394[30], 6}, {NULL}};
const Segment SGI41037[] = {{&S48394[127], 3}, {NULL}};
const Segment SGI41038[] = {{&S48394[93], 5}, {NULL}};
const Segment SGI41039[] = {{&S41047[44], 10}, {NULL}};
const Segment SGI41040[] = {{&S41047[44], 21}, {NULL}};
const Segment SGI41041[] = {{&S41047[30], 6}, {NULL}};
const Segment SGI41042[] = {{&S41047[113], 3}, {NULL}};
const Segment SGI41043[] = {{&S41047[79], 5}, {NULL}};
const Segment SGI41044[] = {{&S41049[44], 11}, {NULL}};
const Segment SGI41045[] = {{&S41049[44], 28}, {NULL}};
const Segment SGI41046[] = {{&S41049[30], 6}, {NULL}};
const Segment SGI41047[] = {{&S41049[120], 3}, {NULL}};
const Segment SGI41048[] = {{&S41049[86], 5}, {NULL}};
const Segment SGI41049[] = {{&S41051[44], 10}, {NULL}};
const Segment SGI41050[] = {{&S41051[44], 27}, {NULL}};
const Segment SGI41051[] = {{&S41051[30], 6}, {NULL}};
const Segment SGI41052[] = {{&S41051[119], 3}, {NULL}};
const Segment SGI41053[] = {{&S41051[85], 5}, {NULL}};
const Segment SGI41054[] = {{&S41056[38], 11}, {NULL}};
const Segment SGI41055[] = {{&S41056[31], 6}, {NULL}};
const Segment SGI41056[] = {{&S40968[44], 7}, {NULL}};
const Segment SGI41057[] = {{&S40968[44], 24}, {NULL}};
const Segment SGI41058[] = {{&S40968[30], 6}, {NULL}};
const Segment SGI41059[] = {{&S40968[116], 3}, {NULL}};
const Segment SGI41060[] = {{&S40968[82], 5}, {NULL}};
const Segment SGI41061[] = {{&S40969[38], 7}, {NULL}};
const Segment SGI41062[] = {{&S40969[31], 6}, {NULL}};
const Segment SGI41063[] = {{&S40971[44], 13}, {NULL}};
const Segment SGI41064[] = {{&S40971[44], 30}, {NULL}};
const Segment SGI41065[] = {{&S40971[30], 6}, {NULL}};
const Segment SGI41066[] = {{&S40971[122], 3}, {NULL}};
const Segment SGI41067[] = {{&S40971[88], 5}, {NULL}};
const Segment SGI41068[] = {{&S53511[44], 9}, {NULL}};
const Segment SGI41069[] = {{&S53511[44], 21}, {NULL}};
const Segment SGI41070[] = {{&S53511[30], 6}, {NULL}};
const Segment SGI41071[] = {{&S53511[113], 3}, {NULL}};
const Segment SGI41072[] = {{&S53511[79], 5}, {NULL}};
const Segment SGI41073[] = {{&S53512[44], 9}, {NULL}};
const Segment SGI41074[] = {{&S53512[44], 21}, {NULL}};
const Segment SGI41075[] = {{&S53512[30], 6}, {NULL}};
const Segment SGI41076[] = {{&S53512[113], 3}, {NULL}};
const Segment SGI41077[] = {{&S53512[79], 5}, {NULL}};
const Segment SGI41078[] = {{&S41048[44], 10}, {NULL}};
const Segment SGI41079[] = {{&S41048[44], 21}, {NULL}};
const Segment SGI41080[] = {{&S41048[30], 6}, {NULL}};
const Segment SGI41081[] = {{&S41048[113], 3}, {NULL}};
const Segment SGI41082[] = {{&S41048[79], 5}, {NULL}};
const Segment SGI41083[] = {{&S41053[44], 6}, {NULL}};
const Segment SGI41084[] = {{&S41053[44], 20}, {NULL}};
const Segment SGI41085[] = {{&S41053[30], 6}, {NULL}};
const Segment SGI41086[] = {{&S41053[112], 3}, {NULL}};
const Segment SGI41087[] = {{&S41053[78], 5}, {NULL}};
const Segment SGI41088[] = {{&S41055[44], 10}, {NULL}};
const Segment SGI41089[] = {{&S41055[44], 27}, {NULL}};
const Segment SGI41090[] = {{&S41055[30], 6}, {NULL}};
const Segment SGI41091[] = {{&S41055[119], 3}, {NULL}};
const Segment SGI41092[] = {{&S41055[85], 5}, {NULL}};
const Segment SGI41093[] = {{&S41197[44], 13}, {NULL}};
const Segment SGI41094[] = {{&S41197[44], 34}, {NULL}};
const Segment SGI41095[] = {{&S41197[30], 6}, {NULL}};
const Segment SGI41096[] = {{&S41197[126], 3}, {NULL}};
const Segment SGI41097[] = {{&S41197[92], 5}, {NULL}};
const Segment SGI41098[] = {{&S41198[44], 13}, {NULL}};
const Segment SGI41099[] = {{&S41198[44], 34}, {NULL}};
const Segment SGI41100[] = {{&S41198[30], 6}, {NULL}};
const Segment SGI41101[] = {{&S41198[126], 3}, {NULL}};
const Segment SGI41102[] = {{&S41198[92], 5}, {NULL}};
const Segment SGI41103[] = {{&S41199[44], 13}, {NULL}};
const Segment SGI41104[] = {{&S41199[44], 34}, {NULL}};
const Segment SGI41105[] = {{&S41199[30], 6}, {NULL}};
const Segment SGI41106[] = {{&S41199[126], 3}, {NULL}};
const Segment SGI41107[] = {{&S41199[92], 5}, {NULL}};
const Segment SGI41108[] = {{&S41200[38], 13}, {NULL}};
const Segment SGI41109[] = {{&S41200[31], 6}, {NULL}};
const Segment SGI41110[] = {{&S41201[44], 13}, {NULL}};
const Segment SGI41111[] = {{&S41201[44], 34}, {NULL}};
const Segment SGI41112[] = {{&S41201[30], 6}, {NULL}};
const Segment SGI41113[] = {{&S41202[44], 13}, {NULL}};
const Segment SGI41114[] = {{&S41202[44], 34}, {NULL}};
const Segment SGI41115[] = {{&S41202[30], 6}, {NULL}};
const Segment SGI41116[] = {{&S41202[126], 3}, {NULL}};
const Segment SGI41117[] = {{&S41202[92], 5}, {NULL}};
const Segment SGI41118[] = {{&S43147[44], 8}, {NULL}};
const Segment SGI41119[] = {{&S43147[44], 19}, {NULL}};
const Segment SGI41120[] = {{&S43147[30], 6}, {NULL}};
const Segment SGI41121[] = {{&S43147[111], 3}, {NULL}};
const Segment SGI41122[] = {{&S43147[77], 5}, {NULL}};
const Segment SGI41123[] = {{&S41359[44], 8}, {NULL}};
const Segment SGI41124[] = {{&S41359[44], 19}, {NULL}};
const Segment SGI41125[] = {{&S41359[30], 6}, {NULL}};
const Segment SGI41126[] = {{&S41359[111], 3}, {NULL}};
const Segment SGI41127[] = {{&S41359[77], 5}, {NULL}};
const Segment SGI41128[] = {{&S41360[38], 8}, {NULL}};
const Segment SGI41129[] = {{&S41360[31], 6}, {NULL}};
const Segment SGI41130[] = {{&S41467[44], 14}, {NULL}};
const Segment SGI41131[] = {{&S41467[30], 6}, {NULL}};
const Segment SGI41132[] = {{&S41467[106], 3}, {NULL}};
const Segment SGI41133[] = {{&S41467[72], 5}, {NULL}};
const Segment SGI41134[] = {{&S41468[44], 17}, {NULL}};
const Segment SGI41135[] = {{&S41468[30], 6}, {NULL}};
const Segment SGI41136[] = {{&S41468[109], 3}, {NULL}};
const Segment SGI41137[] = {{&S41468[75], 5}, {NULL}};
const Segment SGI41138[] = {{&S41469[44], 14}, {NULL}};
const Segment SGI41139[] = {{&S41469[30], 6}, {NULL}};
const Segment SGI41140[] = {{&S41469[106], 3}, {NULL}};
const Segment SGI41141[] = {{&S41469[72], 5}, {NULL}};
const Segment SGI41142[] = {{&S41470[44], 18}, {NULL}};
const Segment SGI41143[] = {{&S41470[30], 6}, {NULL}};
const Segment SGI41144[] = {{&S41470[110], 3}, {NULL}};
const Segment SGI41145[] = {{&S41470[76], 5}, {NULL}};
const Segment SGI41146[] = {{&S41471[44], 18}, {NULL}};
const Segment SGI41147[] = {{&S41471[30], 6}, {NULL}};
const Segment SGI41148[] = {{&S41471[110], 3}, {NULL}};
const Segment SGI41149[] = {{&S41471[76], 5}, {NULL}};
const Segment SGI41150[] = {{&S48330[6], 4}, {NULL}};
const Segment SGI41151[] = {{&S48330[76], 5}, {NULL}};
const Segment SGI41152[] = {{&S54141[44], 6}, {NULL}};
const Segment SGI41153[] = {{&S54141[44], 18}, {NULL}};
const Segment SGI41154[] = {{&S54141[30], 6}, {NULL}};
const Segment SGI41155[] = {{&S41680[44], 8}, {NULL}};
const Segment SGI41156[] = {{&S54141[110], 3}, {NULL}};
const Segment SGI41157[] = {{&S41680[44], 19}, {NULL}};
const Segment SGI41158[] = {{&S41680[30], 6}, {NULL}};
const Segment SGI41159[] = {{&S54141[76], 5}, {NULL}};
const Segment SGI41160[] = {{&S41680[111], 3}, {NULL}};
const Segment SGI41161[] = {{&S41680[77], 5}, {NULL}};
const Segment SGI41162[] = {{&S54142[38], 6}, {NULL}};
const Segment SGI41163[] = {{&S54142[31], 6}, {NULL}};
const Segment SGI41164[] = {{&S41681[38], 8}, {NULL}};
const Segment SGI41165[] = {{&S41681[31], 6}, {NULL}};
const Segment SGI41166[] = {{&S41805[44], 11}, {NULL}};
const Segment SGI41167[] = {{&S41805[44], 22}, {NULL}};
const Segment SGI41168[] = {{&S41805[30], 6}, {NULL}};
const Segment SGI41169[] = {{&S41805[114], 3}, {NULL}};
const Segment SGI41170[] = {{&S41805[80], 5}, {NULL}};
const Segment SGI41171[] = {{&S41809[44], 10}, {NULL}};
const Segment SGI41172[] = {{&S41809[44], 21}, {NULL}};
const Segment SGI41173[] = {{&S41809[30], 6}, {NULL}};
const Segment SGI41174[] = {{&S41809[113], 3}, {NULL}};
const Segment SGI41175[] = {{&S41809[79], 5}, {NULL}};
const Segment SGI41176[] = {{&S41810[44], 11}, {NULL}};
const Segment SGI41177[] = {{&S41810[44], 22}, {NULL}};
const Segment SGI41178[] = {{&S41810[30], 6}, {NULL}};
const Segment SGI41179[] = {{&S41810[83], 4}, {NULL}};
const Segment SGI41180[] = {{&S41810[83], 3}, {NULL}};
const Segment SGI41181[] = {{&S41811[44], 11}, {NULL}};
const Segment SGI41182[] = {{&S41811[44], 22}, {NULL}};
const Segment SGI41183[] = {{&S41811[30], 6}, {NULL}};
const Segment SGI41184[] = {{&S41811[83], 6}, {NULL}};
const Segment SGI41185[] = {{&S41967[44], 9}, {NULL}};
const Segment SGI41186[] = {{&S41967[44], 20}, {NULL}};
const Segment SGI41187[] = {{&S41967[30], 6}, {NULL}};
const Segment SGI41188[] = {{&S41967[112], 3}, {NULL}};
const Segment SGI41189[] = {{&S41967[78], 5}, {NULL}};
const Segment SGI41190[] = {{&S41972[44], 9}, {NULL}};
const Segment SGI41191[] = {{&S41972[44], 20}, {NULL}};
const Segment SGI41192[] = {{&S41972[30], 6}, {NULL}};
const Segment SGI41193[] = {{&S41972[112], 3}, {NULL}};
const Segment SGI41194[] = {{&S41972[78], 5}, {NULL}};
const Segment SGI41195[] = {{&S41973[38], 9}, {NULL}};
const Segment SGI41196[] = {{&S41973[31], 6}, {NULL}};
const Segment SGI41197[] = {{&S41974[44], 9}, {NULL}};
const Segment SGI41198[] = {{&S41974[44], 20}, {NULL}};
const Segment SGI41199[] = {{&S41974[30], 6}, {NULL}};
const Segment SGI41200[] = {{&S41974[112], 3}, {NULL}};
const Segment SGI41201[] = {{&S41974[78], 5}, {NULL}};
const Segment SGI41202[] = {{&S41977[44], 10}, {NULL}};
const Segment SGI41203[] = {{&S41977[44], 21}, {NULL}};
const Segment SGI41204[] = {{&S41977[30], 6}, {NULL}};
const Segment SGI41205[] = {{&S41977[113], 3}, {NULL}};
const Segment SGI41206[] = {{&S41977[79], 5}, {NULL}};
const Segment SGI41207[] = {{&S41978[38], 10}, {NULL}};
const Segment SGI41208[] = {{&S41978[31], 6}, {NULL}};
const Segment SGI41209[] = {{&S41979[44], 10}, {NULL}};
const Segment SGI41210[] = {{&S41979[44], 26}, {NULL}};
const Segment SGI41211[] = {{&S41979[30], 6}, {NULL}};
const Segment SGI41212[] = {{&S41979[118], 3}, {NULL}};
const Segment SGI41213[] = {{&S41979[84], 5}, {NULL}};
const Segment SGI41214[] = {{&S41980[44], 10}, {NULL}};
const Segment SGI41215[] = {{&S41980[44], 32}, {NULL}};
const Segment SGI41216[] = {{&S41980[30], 6}, {NULL}};
const Segment SGI41217[] = {{&S41980[124], 3}, {NULL}};
const Segment SGI41218[] = {{&S41980[90], 5}, {NULL}};
const Segment SGI41219[] = {{&S41981[44], 10}, {NULL}};
const Segment SGI41220[] = {{&S41981[44], 26}, {NULL}};
const Segment SGI41221[] = {{&S41981[30], 6}, {NULL}};
const Segment SGI41222[] = {{&S41981[118], 3}, {NULL}};
const Segment SGI41223[] = {{&S41981[84], 5}, {NULL}};
const Segment SGI41224[] = {{&S43923[44], 24}, {NULL}};
const Segment SGI41225[] = {{&S43923[44], 35}, {NULL}};
const Segment SGI41226[] = {{&S43923[30], 6}, {NULL}};
const Segment SGI41227[] = {{&S43923[127], 3}, {NULL}};
const Segment SGI41228[] = {{&S43923[93], 5}, {NULL}};
const Segment SGI41229[] = {{&S42095[44], 9}, {NULL}};
const Segment SGI41230[] = {{&S42095[44], 20}, {NULL}};
const Segment SGI41231[] = {{&S42095[30], 6}, {NULL}};
const Segment SGI41232[] = {{&S42095[112], 3}, {NULL}};
const Segment SGI41233[] = {{&S42095[78], 5}, {NULL}};
const Segment SGI41234[] = {{&S54610[44], 7}, {NULL}};
const Segment SGI41235[] = {{&S54610[44], 18}, {NULL}};
const Segment SGI41236[] = {{&S54610[30], 6}, {NULL}};
const Segment SGI41237[] = {{&S54610[110], 3}, {NULL}};
const Segment SGI41238[] = {{&S42096[44], 8}, {NULL}};
const Segment SGI41239[] = {{&S54610[76], 5}, {NULL}};
const Segment SGI41240[] = {{&S42096[44], 19}, {NULL}};
const Segment SGI41241[] = {{&S42096[30], 6}, {NULL}};
const Segment SGI41242[] = {{&S42096[111], 3}, {NULL}};
const Segment SGI41243[] = {{&S42096[77], 5}, {NULL}};
const Segment SGI41244[] = {{&S42097[43], 9}, {NULL}};
const Segment SGI41245[] = {{&S42097[43], 20}, {NULL}};
const Segment SGI41246[] = {{&S42097[111], 3}, {NULL}};
const Segment SGI41247[] = {{&S42097[77], 5}, {NULL}};
const Segment SGI41248[] = {{&S42098[38], 9}, {NULL}};
const Segment SGI41249[] = {{&S42098[31], 6}, {NULL}};
const Segment SGI41250[] = {{&S42099[44], 9}, {NULL}};
const Segment SGI41251[] = {{&S42099[44], 20}, {NULL}};
const Segment SGI41252[] = {{&S42099[30], 6}, {NULL}};
const Segment SGI41253[] = {{&S42099[112], 3}, {NULL}};
const Segment SGI41254[] = {{&S42099[78], 5}, {NULL}};
const Segment SGI41255[] = {{&S42100[44], 8}, {NULL}};
const Segment SGI41256[] = {{&S42100[44], 19}, {NULL}};
const Segment SGI41257[] = {{&S42100[30], 6}, {NULL}};
const Segment SGI41258[] = {{&S42100[111], 3}, {NULL}};
const Segment SGI41259[] = {{&S42100[77], 5}, {NULL}};
const Segment SGI41260[] = {{&S42101[43], 9}, {NULL}};
const Segment SGI41261[] = {{&S42101[43], 20}, {NULL}};
const Segment SGI41262[] = {{&S42101[30], 5}, {NULL}};
const Segment SGI41263[] = {{&S42101[111], 3}, {NULL}};
const Segment SGI41264[] = {{&S42101[77], 5}, {NULL}};
const Segment SGI41265[] = {{&S54669[44], 10}, {NULL}};
const Segment SGI41266[] = {{&S54669[44], 30}, {NULL}};
const Segment SGI41267[] = {{&S54669[30], 6}, {NULL}};
const Segment SGI41268[] = {{&S54669[122], 3}, {NULL}};
const Segment SGI41269[] = {{&S54669[88], 5}, {NULL}};
const Segment SGI41270[] = {{&S54670[44], 11}, {NULL}};
const Segment SGI41271[] = {{&S54670[44], 31}, {NULL}};
const Segment SGI41272[] = {{&S54670[30], 6}, {NULL}};
const Segment SGI41273[] = {{&S54670[123], 3}, {NULL}};
const Segment SGI41274[] = {{&S54670[89], 5}, {NULL}};
const Segment SGI41275[] = {{&S54671[44], 10}, {NULL}};
const Segment SGI41276[] = {{&S54671[44], 34}, {NULL}};
const Segment SGI41277[] = {{&S54671[30], 6}, {NULL}};
const Segment SGI41278[] = {{&S54671[126], 3}, {NULL}};
const Segment SGI41279[] = {{&S54671[92], 5}, {NULL}};
const Segment SGI41280[] = {{&S54672[44], 10}, {NULL}};
const Segment SGI41281[] = {{&S54672[44], 30}, {NULL}};
const Segment SGI41282[] = {{&S54672[30], 6}, {NULL}};
const Segment SGI41283[] = {{&S54672[122], 3}, {NULL}};
const Segment SGI41284[] = {{&S54672[88], 5}, {NULL}};
const Segment SGI41285[] = {{&S54673[44], 8}, {NULL}};
const Segment SGI41286[] = {{&S54673[44], 28}, {NULL}};
const Segment SGI41287[] = {{&S54673[30], 6}, {NULL}};
const Segment SGI41288[] = {{&S54673[120], 3}, {NULL}};
const Segment SGI41289[] = {{&S54673[86], 5}, {NULL}};
const Segment SGI41290[] = {{&S54674[44], 8}, {NULL}};
const Segment SGI41291[] = {{&S54674[44], 28}, {NULL}};
const Segment SGI41292[] = {{&S54674[30], 6}, {NULL}};
const Segment SGI41293[] = {{&S54674[120], 3}, {NULL}};
const Segment SGI41294[] = {{&S54674[86], 5}, {NULL}};
const Segment SGI41295[] = {{&S54675[44], 9}, {NULL}};
const Segment SGI41296[] = {{&S54675[44], 29}, {NULL}};
const Segment SGI41297[] = {{&S54675[30], 6}, {NULL}};
const Segment SGI41298[] = {{&S54675[121], 3}, {NULL}};
const Segment SGI41299[] = {{&S54675[87], 5}, {NULL}};
const Segment SGI41300[] = {{&S42179[44], 14}, {NULL}};
const Segment SGI41301[] = {{&S42179[44], 25}, {NULL}};
const Segment SGI41302[] = {{&S42179[30], 6}, {NULL}};
const Segment SGI41303[] = {{&S42179[117], 3}, {NULL}};
const Segment SGI41304[] = {{&S42179[83], 5}, {NULL}};
const Segment SGI41305[] = {{&S42180[44], 14}, {NULL}};
const Segment SGI41306[] = {{&S42180[44], 25}, {NULL}};
const Segment SGI41307[] = {{&S42180[30], 6}, {NULL}};
const Segment SGI41308[] = {{&S42180[117], 3}, {NULL}};
const Segment SGI41309[] = {{&S42180[83], 5}, {NULL}};
const Segment SGI41310[] = {{&S42181[44], 14}, {NULL}};
const Segment SGI41311[] = {{&S42181[44], 25}, {NULL}};
const Segment SGI41312[] = {{&S42181[30], 6}, {NULL}};
const Segment SGI41313[] = {{&S42181[117], 3}, {NULL}};
const Segment SGI41314[] = {{&S42181[83], 5}, {NULL}};
const Segment SGI41315[] = {{&S42182[38], 14}, {NULL}};
const Segment SGI41316[] = {{&S42182[31], 6}, {NULL}};
const Segment SGI41317[] = {{&S42183[44], 13}, {NULL}};
const Segment SGI41318[] = {{&S42183[44], 24}, {NULL}};
const Segment SGI41319[] = {{&S42183[30], 6}, {NULL}};
const Segment SGI41320[] = {{&S42183[116], 3}, {NULL}};
const Segment SGI41321[] = {{&S42183[82], 5}, {NULL}};
const Segment SGI41322[] = {{&S42184[44], 10}, {NULL}};
const Segment SGI41323[] = {{&S42184[44], 21}, {NULL}};
const Segment SGI41324[] = {{&S42184[30], 6}, {NULL}};
const Segment SGI41325[] = {{&S42184[113], 3}, {NULL}};
const Segment SGI41326[] = {{&S42184[79], 5}, {NULL}};
const Segment SGI41327[] = {{&S42185[43], 14}, {NULL}};
const Segment SGI41328[] = {{&S42185[43], 25}, {NULL}};
const Segment SGI41329[] = {{&S42185[116], 3}, {NULL}};
const Segment SGI41330[] = {{&S42185[82], 5}, {NULL}};
const Segment SGI41331[] = {{&S42186[38], 14}, {NULL}};
const Segment SGI41332[] = {{&S42186[31], 6}, {NULL}};
const Segment SGI41333[] = {{&S42187[44], 13}, {NULL}};
const Segment SGI41334[] = {{&S42187[44], 24}, {NULL}};
const Segment SGI41335[] = {{&S42187[30], 6}, {NULL}};
const Segment SGI41336[] = {{&S42187[116], 3}, {NULL}};
const Segment SGI41337[] = {{&S42187[82], 5}, {NULL}};
const Segment SGI41338[] = {{&S42188[44], 13}, {NULL}};
const Segment SGI41339[] = {{&S42188[44], 24}, {NULL}};
const Segment SGI41340[] = {{&S42188[30], 6}, {NULL}};
const Segment SGI41341[] = {{&S42188[116], 3}, {NULL}};
const Segment SGI41342[] = {{&S42188[82], 5}, {NULL}};
const Segment SGI41343[] = {{&S42189[44], 13}, {NULL}};
const Segment SGI41344[] = {{&S42189[44], 24}, {NULL}};
const Segment SGI41345[] = {{&S42189[30], 6}, {NULL}};
const Segment SGI41346[] = {{&S42189[116], 3}, {NULL}};
const Segment SGI41347[] = {{&S42189[82], 5}, {NULL}};
const Segment SGI41348[] = {{&S42190[38], 10}, {NULL}};
const Segment SGI41349[] = {{&S42190[31], 6}, {NULL}};
const Segment SGI41350[] = {{&S42192[44], 10}, {NULL}};
const Segment SGI41351[] = {{&S42192[44], 21}, {NULL}};
const Segment SGI41352[] = {{&S42192[30], 6}, {NULL}};
const Segment SGI41353[] = {{&S42192[82], 6}, {NULL}};
const Segment SGI41354[] = {{&S42193[44], 11}, {NULL}};
const Segment SGI41355[] = {{&S42193[44], 22}, {NULL}};
const Segment SGI41356[] = {{&S42193[30], 6}, {NULL}};
const Segment SGI41357[] = {{&S42193[114], 3}, {NULL}};
const Segment SGI41358[] = {{&S42193[80], 5}, {NULL}};
const Segment SGI41359[] = {{&S42194[44], 10}, {NULL}};
const Segment SGI41360[] = {{&S42194[44], 21}, {NULL}};
const Segment SGI41361[] = {{&S42194[30], 6}, {NULL}};
const Segment SGI41362[] = {{&S42194[113], 3}, {NULL}};
const Segment SGI41363[] = {{&S42194[79], 5}, {NULL}};
const Segment SGI41364[] = {{&S41971[44], 9}, {NULL}};
const Segment SGI41365[] = {{&S41971[44], 20}, {NULL}};
const Segment SGI41366[] = {{&S41971[30], 6}, {NULL}};
const Segment SGI41367[] = {{&S41971[112], 3}, {NULL}};
const Segment SGI41368[] = {{&S41971[78], 5}, {NULL}};
const Segment SGI41369[] = {{&S41975[44], 9}, {NULL}};
const Segment SGI41370[] = {{&S41975[44], 20}, {NULL}};
const Segment SGI41371[] = {{&S41975[30], 6}, {NULL}};
const Segment SGI41372[] = {{&S41975[112], 3}, {NULL}};
const Segment SGI41373[] = {{&S41975[78], 5}, {NULL}};
const Segment SGI41374[] = {{&S42255[44], 10}, {NULL}};
const Segment SGI41375[] = {{&S42255[44], 27}, {NULL}};
const Segment SGI41376[] = {{&S42255[30], 6}, {NULL}};
const Segment SGI41377[] = {{&S42255[119], 3}, {NULL}};
const Segment SGI41378[] = {{&S42255[85], 5}, {NULL}};
const Segment SGI41379[] = {{&S42256[38], 10}, {NULL}};
const Segment SGI41380[] = {{&S42256[31], 6}, {NULL}};
const Segment SGI41381[] = {{&S42262[44], 11}, {NULL}};
const Segment SGI41382[] = {{&S42262[44], 22}, {NULL}};
const Segment SGI41383[] = {{&S42262[30], 6}, {NULL}};
const Segment SGI41384[] = {{&S42262[114], 3}, {NULL}};
const Segment SGI41385[] = {{&S42262[80], 5}, {NULL}};
const Segment SGI41386[] = {{&S42347[44], 19}, {NULL}};
const Segment SGI41387[] = {{&S42347[44], 37}, {NULL}};
const Segment SGI41388[] = {{&S42347[30], 6}, {NULL}};
const Segment SGI41389[] = {{&S42347[69], 3}, {NULL}};
const Segment SGI41390[] = {{&S42347[95], 5}, {NULL}};
const Segment SGI41391[] = {{&S42348[44], 19}, {NULL}};
const Segment SGI41392[] = {{&S42348[44], 37}, {NULL}};
const Segment SGI41393[] = {{&S42348[30], 6}, {NULL}};
const Segment SGI41394[] = {{&S42348[69], 3}, {NULL}};
const Segment SGI41395[] = {{&S42348[95], 5}, {NULL}};
const Segment SGI41396[] = {{&S42349[44], 7}, {NULL}};
const Segment SGI41397[] = {{&S42349[44], 26}, {NULL}};
const Segment SGI41398[] = {{&S42349[30], 6}, {NULL}};
const Segment SGI41399[] = {{&S42349[57], 3}, {NULL}};
const Segment SGI41400[] = {{&S42349[84], 5}, {NULL}};
const Segment SGI41401[] = {{&S42350[44], 19}, {NULL}};
const Segment SGI41402[] = {{&S42350[44], 38}, {NULL}};
const Segment SGI41403[] = {{&S42350[30], 6}, {NULL}};
const Segment SGI41404[] = {{&S42350[69], 3}, {NULL}};
const Segment SGI41405[] = {{&S42350[96], 5}, {NULL}};
const Segment SGI41406[] = {{&S42351[44], 7}, {NULL}};
const Segment SGI41407[] = {{&S42351[44], 25}, {NULL}};
const Segment SGI41408[] = {{&S42351[30], 6}, {NULL}};
const Segment SGI41409[] = {{&S42351[57], 3}, {NULL}};
const Segment SGI41410[] = {{&S42351[83], 5}, {NULL}};
const Segment SGI41411[] = {{&S42352[44], 7}, {NULL}};
const Segment SGI41412[] = {{&S42352[44], 25}, {NULL}};
const Segment SGI41413[] = {{&S42352[30], 6}, {NULL}};
const Segment SGI41414[] = {{&S42352[57], 3}, {NULL}};
const Segment SGI41415[] = {{&S42352[83], 5}, {NULL}};
const Segment SGI41416[] = {{&S54932[44], 8}, {NULL}};
const Segment SGI41417[] = {{&S54932[44], 19}, {NULL}};
const Segment SGI41418[] = {{&S54932[30], 6}, {NULL}};
const Segment SGI41419[] = {{&S54932[111], 3}, {NULL}};
const Segment SGI41420[] = {{&S54932[77], 5}, {NULL}};
const Segment SGI41421[] = {{&S54933[44], 7}, {NULL}};
const Segment SGI41422[] = {{&S54933[44], 18}, {NULL}};
const Segment SGI41423[] = {{&S54933[30], 6}, {NULL}};
const Segment SGI41424[] = {{&S54933[110], 3}, {NULL}};
const Segment SGI41425[] = {{&S54933[76], 5}, {NULL}};
const Segment SGI41426[] = {{&S54934[38], 8}, {NULL}};
const Segment SGI41427[] = {{&S54934[31], 6}, {NULL}};
const Segment SGI41428[] = {{&S54936[38], 7}, {NULL}};
const Segment SGI41429[] = {{&S54936[31], 6}, {NULL}};
const Segment SGI41430[] = {{&S44382[44], 9}, {NULL}};
const Segment SGI41431[] = {{&S44382[44], 20}, {NULL}};
const Segment SGI41432[] = {{&S44382[30], 6}, {NULL}};
const Segment SGI41433[] = {{&S44382[112], 3}, {NULL}};
const Segment SGI41434[] = {{&S44382[78], 5}, {NULL}};
const Segment SGI41435[] = {{&S44383[44], 9}, {NULL}};
const Segment SGI41436[] = {{&S44383[44], 20}, {NULL}};
const Segment SGI41437[] = {{&S44383[30], 6}, {NULL}};
const Segment SGI41438[] = {{&S44383[112], 3}, {NULL}};
const Segment SGI41439[] = {{&S44383[78], 5}, {NULL}};
const Segment SGI41440[] = {{&S44384[44], 9}, {NULL}};
const Segment SGI41441[] = {{&S44384[44], 20}, {NULL}};
const Segment SGI41442[] = {{&S44384[30], 6}, {NULL}};
const Segment SGI41443[] = {{&S44384[112], 3}, {NULL}};
const Segment SGI41444[] = {{&S44384[78], 5}, {NULL}};
const Segment SGI41445[] = {{&S44393[44], 9}, {NULL}};
const Segment SGI41446[] = {{&S44393[44], 21}, {NULL}};
const Segment SGI41447[] = {{&S44393[30], 6}, {NULL}};
const Segment SGI41448[] = {{&S44393[113], 3}, {NULL}};
const Segment SGI41449[] = {{&S44393[79], 5}, {NULL}};
const Segment SGI41450[] = {{&S44395[44], 9}, {NULL}};
const Segment SGI41451[] = {{&S44395[44], 32}, {NULL}};
const Segment SGI41452[] = {{&S44395[30], 6}, {NULL}};
const Segment SGI41453[] = {{&S44395[124], 3}, {NULL}};
const Segment SGI41454[] = {{&S44395[90], 5}, {NULL}};
const Segment SGI41455[] = {{&S44396[44], 17}, {NULL}};
const Segment SGI41456[] = {{&S44396[44], 29}, {NULL}};
const Segment SGI41457[] = {{&S44396[30], 6}, {NULL}};
const Segment SGI41458[] = {{&S44396[121], 3}, {NULL}};
const Segment SGI41459[] = {{&S44396[87], 5}, {NULL}};
const Segment SGI41460[] = {{&S44406[38], 9}, {NULL}};
const Segment SGI41461[] = {{&S44406[31], 6}, {NULL}};
const Segment SGI41462[] = {{&S44441[44], 14}, {NULL}};
const Segment SGI41463[] = {{&S44441[44], 31}, {NULL}};
const Segment SGI41464[] = {{&S44441[30], 6}, {NULL}};
const Segment SGI41465[] = {{&S44441[123], 3}, {NULL}};
const Segment SGI41466[] = {{&S44441[89], 5}, {NULL}};
const Segment SGI41467[] = {{&S55083[44], 10}, {NULL}};
const Segment SGI41468[] = {{&S55083[44], 34}, {NULL}};
const Segment SGI41469[] = {{&S55083[30], 6}, {NULL}};
const Segment SGI41470[] = {{&S55083[126], 3}, {NULL}};
const Segment SGI41471[] = {{&S55083[92], 5}, {NULL}};
const Segment SGI41472[] = {{&S55084[44], 10}, {NULL}};
const Segment SGI41473[] = {{&S55084[44], 34}, {NULL}};
const Segment SGI41474[] = {{&S55084[30], 6}, {NULL}};
const Segment SGI41475[] = {{&S55084[126], 3}, {NULL}};
const Segment SGI41476[] = {{&S55084[92], 5}, {NULL}};
const Segment SGI41477[] = {{&S55086[43], 10}, {NULL}};
const Segment SGI41478[] = {{&S55086[43], 34}, {NULL}};
const Segment SGI41479[] = {{&S55086[125], 3}, {NULL}};
const Segment SGI41480[] = {{&S42195[44], 19}, {NULL}};
const Segment SGI41481[] = {{&S55086[91], 5}, {NULL}};
const Segment SGI41482[] = {{&S42195[44], 40}, {NULL}};
const Segment SGI41483[] = {{&S42195[30], 6}, {NULL}};
const Segment SGI41484[] = {{&S42195[132], 3}, {NULL}};
const Segment SGI41485[] = {{&S42195[98], 5}, {NULL}};
const Segment SGI41486[] = {{&S55087[44], 9}, {NULL}};
const Segment SGI41487[] = {{&S55087[44], 33}, {NULL}};
const Segment SGI41488[] = {{&S55087[30], 6}, {NULL}};
const Segment SGI41489[] = {{&S55087[125], 3}, {NULL}};
const Segment SGI41490[] = {{&S55087[91], 5}, {NULL}};
const Segment SGI41491[] = {{&S55088[44], 9}, {NULL}};
const Segment SGI41492[] = {{&S55088[44], 33}, {NULL}};
const Segment SGI41493[] = {{&S55088[30], 6}, {NULL}};
const Segment SGI41494[] = {{&S55088[125], 3}, {NULL}};
const Segment SGI41495[] = {{&S55088[91], 5}, {NULL}};
const Segment SGI41496[] = {{&S55089[43], 10}, {NULL}};
const Segment SGI41497[] = {{&S55089[43], 34}, {NULL}};
const Segment SGI41498[] = {{&S55089[30], 5}, {NULL}};
const Segment SGI41499[] = {{&S55089[125], 3}, {NULL}};
const Segment SGI41500[] = {{&S55089[91], 5}, {NULL}};
const Segment SGI41501[] = {{&S55090[44], 10}, {NULL}};
const Segment SGI41502[] = {{&S55090[44], 32}, {NULL}};
const Segment SGI41503[] = {{&S55090[30], 6}, {NULL}};
const Segment SGI41504[] = {{&S55090[124], 3}, {NULL}};
const Segment SGI41505[] = {{&S55090[90], 5}, {NULL}};
const Segment SGI41506[] = {{&S55091[44], 9}, {NULL}};
const Segment SGI41507[] = {{&S55091[44], 33}, {NULL}};
const Segment SGI41508[] = {{&S55091[30], 6}, {NULL}};
const Segment SGI41509[] = {{&S55091[125], 3}, {NULL}};
const Segment SGI41510[] = {{&S55091[91], 5}, {NULL}};
const Segment SGI41511[] = {{&S55092[38], 10}, {NULL}};
const Segment SGI41512[] = {{&S55092[31], 6}, {NULL}};
const Segment SGI41513[] = {{&S42353[44], 19}, {NULL}};
const Segment SGI41514[] = {{&S42353[44], 37}, {NULL}};
const Segment SGI41515[] = {{&S42353[30], 6}, {NULL}};
const Segment SGI41516[] = {{&S42353[69], 3}, {NULL}};
const Segment SGI41517[] = {{&S42353[95], 5}, {NULL}};
const Segment SGI41518[] = {{&S42354[38], 7}, {NULL}};
const Segment SGI41519[] = {{&S42354[31], 6}, {NULL}};
const Segment SGI41520[] = {{&S46217[44], 20}, {NULL}};
const Segment SGI41521[] = {{&S46217[44], 36}, {NULL}};
const Segment SGI41522[] = {{&S46217[30], 6}, {NULL}};
const Segment SGI41523[] = {{&S46217[128], 3}, {NULL}};
const Segment SGI41524[] = {{&S46217[94], 5}, {NULL}};
const Segment SGI41525[] = {{&S46218[44], 8}, {NULL}};
const Segment SGI41526[] = {{&S46218[44], 24}, {NULL}};
const Segment SGI41527[] = {{&S46218[30], 6}, {NULL}};
const Segment SGI41528[] = {{&S46218[116], 3}, {NULL}};
const Segment SGI41529[] = {{&S46218[82], 5}, {NULL}};
const Segment SGI41530[] = {{&S46219[44], 20}, {NULL}};
const Segment SGI41531[] = {{&S46219[44], 37}, {NULL}};
const Segment SGI41532[] = {{&S46219[30], 6}, {NULL}};
const Segment SGI41533[] = {{&S46219[129], 3}, {NULL}};
const Segment SGI41534[] = {{&S46219[95], 5}, {NULL}};
const Segment SGI41535[] = {{&S46227[43], 20}, {NULL}};
const Segment SGI41536[] = {{&S46227[43], 36}, {NULL}};
const Segment SGI41537[] = {{&S46227[126], 3}, {NULL}};
const Segment SGI41538[] = {{&S46227[92], 5}, {NULL}};
const Segment SGI41539[] = {{&S46235[38], 8}, {NULL}};
const Segment SGI41540[] = {{&S46235[31], 6}, {NULL}};
const Segment SGI41541[] = {{&S42435[44], 29}, {NULL}};
const Segment SGI41542[] = {{&S42435[44], 40}, {NULL}};
const Segment SGI41543[] = {{&S42435[30], 6}, {NULL}};
const Segment SGI41544[] = {{&S42435[132], 3}, {NULL}};
const Segment SGI41545[] = {{&S42435[98], 5}, {NULL}};
const Segment SGI41546[] = {{&S42436[44], 9}, {NULL}};
const Segment SGI41547[] = {{&S42436[44], 20}, {NULL}};
const Segment SGI41548[] = {{&S42436[30], 6}, {NULL}};
const Segment SGI41549[] = {{&S42436[112], 3}, {NULL}};
const Segment SGI41550[] = {{&S42436[78], 5}, {NULL}};
const Segment SGI41551[] = {{&S42437[38], 9}, {NULL}};
const Segment SGI41552[] = {{&S42437[31], 6}, {NULL}};
const Segment SGI41553[] = {{&S42260[44], 10}, {NULL}};
const Segment SGI41554[] = {{&S42260[44], 27}, {NULL}};
const Segment SGI41555[] = {{&S42260[30], 6}, {NULL}};
const Segment SGI41556[] = {{&S42260[119], 3}, {NULL}};
const Segment SGI41557[] = {{&S42260[85], 5}, {NULL}};
const Segment SGI41558[] = {{&S44707[44], 11}, {NULL}};
const Segment SGI41559[] = {{&S44707[44], 28}, {NULL}};
const Segment SGI41560[] = {{&S44707[30], 6}, {NULL}};
const Segment SGI41561[] = {{&S44707[120], 3}, {NULL}};
const Segment SGI41562[] = {{&S44707[86], 5}, {NULL}};
const Segment SGI41563[] = {{&S44708[44], 11}, {NULL}};
const Segment SGI41564[] = {{&S44708[44], 28}, {NULL}};
const Segment SGI41565[] = {{&S44708[30], 6}, {NULL}};
const Segment SGI41566[] = {{&S44708[120], 3}, {NULL}};
const Segment SGI41567[] = {{&S44708[86], 5}, {NULL}};
const Segment SGI41568[] = {{&S44714[44], 11}, {NULL}};
const Segment SGI41569[] = {{&S44714[44], 28}, {NULL}};
const Segment SGI41570[] = {{&S44714[30], 6}, {NULL}};
const Segment SGI41571[] = {{&S44714[120], 3}, {NULL}};
const Segment SGI41572[] = {{&S44714[86], 5}, {NULL}};
const Segment SGI41573[] = {{&S44720[38], 11}, {NULL}};
const Segment SGI41574[] = {{&S44720[31], 6}, {NULL}};
const Segment SGI41575[] = {{&S44811[44], 18}, {NULL}};
const Segment SGI41576[] = {{&S44811[44], 36}, {NULL}};
const Segment SGI41577[] = {{&S44811[30], 6}, {NULL}};
const Segment SGI41578[] = {{&S44811[128], 3}, {NULL}};
const Segment SGI41579[] = {{&S44811[94], 5}, {NULL}};
const Segment SGI41580[] = {{&S44812[44], 6}, {NULL}};
const Segment SGI41581[] = {{&S44812[44], 24}, {NULL}};
const Segment SGI41582[] = {{&S44812[30], 6}, {NULL}};
const Segment SGI41583[] = {{&S44812[116], 3}, {NULL}};
const Segment SGI41584[] = {{&S44812[82], 5}, {NULL}};
const Segment SGI41585[] = {{&S44813[44], 18}, {NULL}};
const Segment SGI41586[] = {{&S44813[44], 36}, {NULL}};
const Segment SGI41587[] = {{&S44813[30], 6}, {NULL}};
const Segment SGI41588[] = {{&S44813[128], 3}, {NULL}};
const Segment SGI41589[] = {{&S44813[94], 5}, {NULL}};
const Segment SGI41590[] = {{&S44815[38], 6}, {NULL}};
const Segment SGI41591[] = {{&S44815[31], 6}, {NULL}};
const Segment SGI41592[] = {{&S46509[6], 4}, {NULL}};
const Segment SGI41593[] = {{&S46509[86], 5}, {NULL}};
const Segment SGI41594[] = {{&S42625[44], 15}, {NULL}};
const Segment SGI41595[] = {{&S42625[44], 26}, {NULL}};
const Segment SGI41596[] = {{&S42625[30], 6}, {NULL}};
const Segment SGI41597[] = {{&S42625[118], 3}, {NULL}};
const Segment SGI41598[] = {{&S42625[84], 5}, {NULL}};
const Segment SGI41599[] = {{&S42626[44], 12}, {NULL}};
const Segment SGI41600[] = {{&S42626[44], 23}, {NULL}};
const Segment SGI41601[] = {{&S42626[30], 6}, {NULL}};
const Segment SGI41602[] = {{&S42626[115], 3}, {NULL}};
const Segment SGI41603[] = {{&S42626[81], 5}, {NULL}};
const Segment SGI41604[] = {{&S42627[38], 12}, {NULL}};
const Segment SGI41605[] = {{&S42627[31], 6}, {NULL}};
const Segment SGI41606[] = {{&S42628[44], 12}, {NULL}};
const Segment SGI41607[] = {{&S42628[44], 23}, {NULL}};
const Segment SGI41608[] = {{&S42628[30], 6}, {NULL}};
const Segment SGI41609[] = {{&S42628[84], 6}, {NULL}};
const Segment SGI41610[] = {{&S50538[44], 10}, {NULL}};
const Segment SGI41611[] = {{&S50538[44], 27}, {NULL}};
const Segment SGI41612[] = {{&S50538[30], 6}, {NULL}};
const Segment SGI41613[] = {{&S50538[119], 3}, {NULL}};
const Segment SGI41614[] = {{&S50538[85], 5}, {NULL}};
const Segment SGI41615[] = {{&S50539[44], 12}, {NULL}};
const Segment SGI41616[] = {{&S50539[44], 29}, {NULL}};
const Segment SGI41617[] = {{&S50539[30], 6}, {NULL}};
const Segment SGI41618[] = {{&S50539[121], 3}, {NULL}};
const Segment SGI41619[] = {{&S50539[87], 5}, {NULL}};
const Segment SGI41620[] = {{&S50540[38], 10}, {NULL}};
const Segment SGI41621[] = {{&S50540[31], 6}, {NULL}};
const Segment SGI41622[] = {{&S55435[44], 14}, {NULL}};
const Segment SGI41623[] = {{&S55435[44], 26}, {NULL}};
const Segment SGI41624[] = {{&S55435[30], 6}, {NULL}};
const Segment SGI41625[] = {{&S55435[118], 3}, {NULL}};
const Segment SGI41626[] = {{&S55435[84], 5}, {NULL}};
const Segment SGI41627[] = {{&S55436[44], 19}, {NULL}};
const Segment SGI41628[] = {{&S55436[44], 31}, {NULL}};
const Segment SGI41629[] = {{&S55436[30], 6}, {NULL}};
const Segment SGI41630[] = {{&S55436[123], 3}, {NULL}};
const Segment SGI41631[] = {{&S55436[89], 5}, {NULL}};
const Segment SGI41632[] = {{&S44979[44], 6}, {NULL}};
const Segment SGI41633[] = {{&S44979[44], 17}, {NULL}};
const Segment SGI41634[] = {{&S44979[30], 6}, {NULL}};
const Segment SGI41635[] = {{&S44979[109], 3}, {NULL}};
const Segment SGI41636[] = {{&S44979[75], 5}, {NULL}};
const Segment SGI41637[] = {{&S42867[44], 10}, {NULL}};
const Segment SGI41638[] = {{&S42867[44], 21}, {NULL}};
const Segment SGI41639[] = {{&S42867[30], 6}, {NULL}};
const Segment SGI41640[] = {{&S44980[38], 6}, {NULL}};
const Segment SGI41641[] = {{&S44980[31], 6}, {NULL}};
const Segment SGI41642[] = {{&S42867[113], 3}, {NULL}};
const Segment SGI41643[] = {{&S42867[79], 5}, {NULL}};
const Segment SGI41644[] = {{&S42868[38], 10}, {NULL}};
const Segment SGI41645[] = {{&S42868[31], 6}, {NULL}};
const Segment SGI41646[] = {{&S42869[44], 10}, {NULL}};
const Segment SGI41647[] = {{&S42869[44], 27}, {NULL}};
const Segment SGI41648[] = {{&S42869[30], 6}, {NULL}};
const Segment SGI41649[] = {{&S42869[119], 3}, {NULL}};
const Segment SGI41650[] = {{&S42869[85], 5}, {NULL}};
const Segment SGI41651[] = {{&S45002[44], 10}, {NULL}};
const Segment SGI41652[] = {{&S45002[44], 21}, {NULL}};
const Segment SGI41653[] = {{&S45002[30], 6}, {NULL}};
const Segment SGI41654[] = {{&S45002[113], 3}, {NULL}};
const Segment SGI41655[] = {{&S45002[79], 5}, {NULL}};
const Segment SGI41656[] = {{&S45171[44], 19}, {NULL}};
const Segment SGI41657[] = {{&S45171[44], 36}, {NULL}};
const Segment SGI41658[] = {{&S45171[30], 6}, {NULL}};
const Segment SGI41659[] = {{&S45171[128], 3}, {NULL}};
const Segment SGI41660[] = {{&S45171[94], 5}, {NULL}};
const Segment SGI41661[] = {{&S45172[38], 19}, {NULL}};
const Segment SGI41662[] = {{&S45172[31], 6}, {NULL}};
const Segment SGI41663[] = {{&S46888[44], 11}, {NULL}};
const Segment SGI41664[] = {{&S46888[44], 23}, {NULL}};
const Segment SGI41665[] = {{&S45173[44], 19}, {NULL}};
const Segment SGI41666[] = {{&S46888[30], 6}, {NULL}};
const Segment SGI41667[] = {{&S45173[44], 36}, {NULL}};
const Segment SGI41668[] = {{&S45173[30], 6}, {NULL}};
const Segment SGI41669[] = {{&S46888[115], 3}, {NULL}};
const Segment SGI41670[] = {{&S45173[129], 3}, {NULL}};
const Segment SGI41671[] = {{&S46888[81], 5}, {NULL}};
const Segment SGI41672[] = {{&S45173[94], 6}, {NULL}};
const Segment SGI41673[] = {{&S45174[44], 19}, {NULL}};
const Segment SGI41674[] = {{&S45174[44], 36}, {NULL}};
const Segment SGI41675[] = {{&S45174[30], 6}, {NULL}};
const Segment SGI41676[] = {{&S45174[97], 6}, {NULL}};
const Segment SGI41677[] = {{&S45175[42], 19}, {NULL}};
const Segment SGI41678[] = {{&S45175[30], 6}, {NULL}};
const Segment SGI41679[] = {{&S45175[126], 3}, {NULL}};
const Segment SGI41680[] = {{&S45175[92], 5}, {NULL}};
const Segment SGI41681[] = {{&S45188[44], 11}, {NULL}};
const Segment SGI41682[] = {{&S45188[44], 28}, {NULL}};
const Segment SGI41683[] = {{&S45188[30], 6}, {NULL}};
const Segment SGI41684[] = {{&S45188[120], 3}, {NULL}};
const Segment SGI41685[] = {{&S45188[86], 5}, {NULL}};
const Segment SGI41686[] = {{&S45189[38], 11}, {NULL}};
const Segment SGI41687[] = {{&S45189[31], 6}, {NULL}};
const Segment SGI41688[] = {{&S42937[44], 8}, {NULL}};
const Segment SGI41689[] = {{&S42937[44], 19}, {NULL}};
const Segment SGI41690[] = {{&S42937[30], 6}, {NULL}};
const Segment SGI41691[] = {{&S42937[111], 3}, {NULL}};
const Segment SGI41692[] = {{&S42937[77], 5}, {NULL}};
const Segment SGI41693[] = {{&S42938[38], 8}, {NULL}};
const Segment SGI41694[] = {{&S42938[31], 6}, {NULL}};
const Segment SGI41695[] = {{&S43148[44], 9}, {NULL}};
const Segment SGI41696[] = {{&S43148[44], 20}, {NULL}};
const Segment SGI41697[] = {{&S43148[30], 6}, {NULL}};
const Segment SGI41698[] = {{&S43148[112], 3}, {NULL}};
const Segment SGI41699[] = {{&S43148[78], 5}, {NULL}};
const Segment SGI41700[] = {{&S43149[44], 8}, {NULL}};
const Segment SGI41701[] = {{&S43149[44], 19}, {NULL}};
const Segment SGI41702[] = {{&S43149[30], 6}, {NULL}};
const Segment SGI41703[] = {{&S43149[111], 3}, {NULL}};
const Segment SGI41704[] = {{&S43149[77], 5}, {NULL}};
const Segment SGI41705[] = {{&S43150[44], 10}, {NULL}};
const Segment SGI41706[] = {{&S43150[44], 21}, {NULL}};
const Segment SGI41707[] = {{&S43150[30], 6}, {NULL}};
const Segment SGI41708[] = {{&S43150[113], 3}, {NULL}};
const Segment SGI41709[] = {{&S43150[79], 5}, {NULL}};
const Segment SGI41710[] = {{&S43151[44], 14}, {NULL}};
const Segment SGI41711[] = {{&S43151[30], 6}, {NULL}};
const Segment SGI41712[] = {{&S43151[106], 3}, {NULL}};
const Segment SGI41713[] = {{&S43151[72], 5}, {NULL}};
const Segment SGI41714[] = {{&S43152[44], 9}, {NULL}};
const Segment SGI41715[] = {{&S43152[44], 20}, {NULL}};
const Segment SGI41716[] = {{&S43152[30], 6}, {NULL}};
const Segment SGI41717[] = {{&S43152[112], 3}, {NULL}};
const Segment SGI41718[] = {{&S43152[78], 5}, {NULL}};
const Segment SGI41719[] = {{&S43153[44], 10}, {NULL}};
const Segment SGI41720[] = {{&S43153[44], 21}, {NULL}};
const Segment SGI41721[] = {{&S43153[30], 6}, {NULL}};
const Segment SGI41722[] = {{&S43153[113], 3}, {NULL}};
const Segment SGI41723[] = {{&S43153[79], 5}, {NULL}};
const Segment SGI41724[] = {{&S43154[44], 14}, {NULL}};
const Segment SGI41725[] = {{&S43154[30], 6}, {NULL}};
const Segment SGI41726[] = {{&S43154[106], 3}, {NULL}};
const Segment SGI41727[] = {{&S43154[72], 5}, {NULL}};
const Segment SGI41728[] = {{&S43155[42], 8}, {NULL}};
const Segment SGI41729[] = {{&S43155[30], 6}, {NULL}};
const Segment SGI41730[] = {{&S43155[109], 3}, {NULL}};
const Segment SGI41731[] = {{&S43155[75], 5}, {NULL}};
const Segment SGI41732[] = {{&S43156[38], 10}, {NULL}};
const Segment SGI41733[] = {{&S43156[31], 6}, {NULL}};
const Segment SGI41734[] = {{&S43157[38], 9}, {NULL}};
const Segment SGI41735[] = {{&S43157[31], 6}, {NULL}};
const Segment SGI41736[] = {{&S43158[38], 10}, {NULL}};
const Segment SGI41737[] = {{&S43158[31], 6}, {NULL}};
const Segment SGI41738[] = {{&S43159[42], 9}, {NULL}};
const Segment SGI41739[] = {{&S43159[30], 6}, {NULL}};
const Segment SGI41740[] = {{&S43159[110], 3}, {NULL}};
const Segment SGI41741[] = {{&S43159[76], 5}, {NULL}};
const Segment SGI41742[] = {{&S43160[44], 10}, {NULL}};
const Segment SGI41743[] = {{&S43160[44], 21}, {NULL}};
const Segment SGI41744[] = {{&S43160[30], 6}, {NULL}};
const Segment SGI41745[] = {{&S43160[79], 5}, {NULL}};
const Segment SGI41746[] = {{&S43161[44], 6}, {NULL}};
const Segment SGI41747[] = {{&S43161[44], 23}, {NULL}};
const Segment SGI41748[] = {{&S43161[30], 6}, {NULL}};
const Segment SGI41749[] = {{&S43161[115], 3}, {NULL}};
const Segment SGI41750[] = {{&S43161[81], 5}, {NULL}};
const Segment SGI41751[] = {{&S43331[44], 10}, {NULL}};
const Segment SGI41752[] = {{&S43331[44], 27}, {NULL}};
const Segment SGI41753[] = {{&S43331[30], 6}, {NULL}};
const Segment SGI41754[] = {{&S43331[119], 3}, {NULL}};
const Segment SGI41755[] = {{&S43331[85], 5}, {NULL}};
const Segment SGI41756[] = {{&S43332[42], 10}, {NULL}};
const Segment SGI41757[] = {{&S43332[30], 6}, {NULL}};
const Segment SGI41758[] = {{&S43332[117], 3}, {NULL}};
const Segment SGI41759[] = {{&S43332[83], 5}, {NULL}};
const Segment SGI41760[] = {{&S43333[44], 10}, {NULL}};
const Segment SGI41761[] = {{&S43333[44], 32}, {NULL}};
const Segment SGI41762[] = {{&S43333[30], 6}, {NULL}};
const Segment SGI41763[] = {{&S43334[38], 10}, {NULL}};
const Segment SGI41764[] = {{&S43334[31], 6}, {NULL}};
const Segment SGI41765[] = {{&S43168[44], 14}, {NULL}};
const Segment SGI41766[] = {{&S43168[44], 31}, {NULL}};
const Segment SGI41767[] = {{&S43168[30], 6}, {NULL}};
const Segment SGI41768[] = {{&S43168[123], 3}, {NULL}};
const Segment SGI41769[] = {{&S43168[89], 5}, {NULL}};
const Segment SGI41770[] = {{&S43335[44], 7}, {NULL}};
const Segment SGI41771[] = {{&S43335[44], 19}, {NULL}};
const Segment SGI41772[] = {{&S43335[30], 6}, {NULL}};
const Segment SGI41773[] = {{&S43169[44], 7}, {NULL}};
const Segment SGI41774[] = {{&S43169[44], 24}, {NULL}};
const Segment SGI41775[] = {{&S43335[111], 3}, {NULL}};
const Segment SGI41776[] = {{&S43169[30], 6}, {NULL}};
const Segment SGI41777[] = {{&S43335[77], 5}, {NULL}};
const Segment SGI41778[] = {{&S43169[116], 3}, {NULL}};
const Segment SGI41779[] = {{&S43169[82], 5}, {NULL}};
const Segment SGI41780[] = {{&S43338[44], 7}, {NULL}};
const Segment SGI41781[] = {{&S43338[44], 19}, {NULL}};
const Segment SGI41782[] = {{&S43338[30], 6}, {NULL}};
const Segment SGI41783[] = {{&S43338[111], 3}, {NULL}};
const Segment SGI41784[] = {{&S43338[77], 5}, {NULL}};
const Segment SGI41785[] = {{&S43340[44], 7}, {NULL}};
const Segment SGI41786[] = {{&S43340[44], 19}, {NULL}};
const Segment SGI41787[] = {{&S43340[30], 6}, {NULL}};
const Segment SGI41788[] = {{&S43340[111], 3}, {NULL}};
const Segment SGI41789[] = {{&S43340[77], 5}, {NULL}};
const Segment SGI41790[] = {{&S43342[44], 7}, {NULL}};
const Segment SGI41791[] = {{&S43342[44], 19}, {NULL}};
const Segment SGI41792[] = {{&S43342[30], 6}, {NULL}};
const Segment SGI41793[] = {{&S43342[111], 3}, {NULL}};
const Segment SGI41794[] = {{&S43342[77], 5}, {NULL}};
const Segment SGI41795[] = {{&S43343[44], 7}, {NULL}};
const Segment SGI41796[] = {{&S43343[44], 18}, {NULL}};
const Segment SGI41797[] = {{&S43343[30], 6}, {NULL}};
const Segment SGI41798[] = {{&S43343[110], 3}, {NULL}};
const Segment SGI41799[] = {{&S43343[76], 5}, {NULL}};
const Segment SGI41800[] = {{&S43350[44], 7}, {NULL}};
const Segment SGI41801[] = {{&S43350[44], 18}, {NULL}};
const Segment SGI41802[] = {{&S43350[30], 6}, {NULL}};
const Segment SGI41803[] = {{&S43350[79], 4}, {NULL}};
const Segment SGI41804[] = {{&S43350[79], 3}, {NULL}};
const Segment SGI41805[] = {{&S43352[44], 7}, {NULL}};
const Segment SGI41806[] = {{&S43352[44], 18}, {NULL}};
const Segment SGI41807[] = {{&S43352[30], 6}, {NULL}};
const Segment SGI41808[] = {{&S43352[79], 6}, {NULL}};
const Segment SGI41809[] = {{&S43422[44], 13}, {NULL}};
const Segment SGI41810[] = {{&S43422[44], 24}, {NULL}};
const Segment SGI41811[] = {{&S43422[30], 6}, {NULL}};
const Segment SGI41812[] = {{&S43422[116], 3}, {NULL}};
const Segment SGI41813[] = {{&S43422[82], 5}, {NULL}};
const Segment SGI41814[] = {{&S43423[44], 24}, {NULL}};
const Segment SGI41815[] = {{&S43423[44], 35}, {NULL}};
const Segment SGI41816[] = {{&S43423[30], 6}, {NULL}};
const Segment SGI41817[] = {{&S43423[127], 3}, {NULL}};
const Segment SGI41818[] = {{&S43423[93], 5}, {NULL}};
const Segment SGI41819[] = {{&S43424[44], 24}, {NULL}};
const Segment SGI41820[] = {{&S43424[44], 35}, {NULL}};
const Segment SGI41821[] = {{&S43424[30], 6}, {NULL}};
const Segment SGI41822[] = {{&S43424[127], 3}, {NULL}};
const Segment SGI41823[] = {{&S43424[93], 5}, {NULL}};
const Segment SGI41824[] = {{&S43425[44], 24}, {NULL}};
const Segment SGI41825[] = {{&S43425[44], 35}, {NULL}};
const Segment SGI41826[] = {{&S43425[30], 6}, {NULL}};
const Segment SGI41827[] = {{&S43425[127], 3}, {NULL}};
const Segment SGI41828[] = {{&S43425[93], 5}, {NULL}};
const Segment SGI41829[] = {{&S43426[44], 24}, {NULL}};
const Segment SGI41830[] = {{&S43426[44], 35}, {NULL}};
const Segment SGI41831[] = {{&S43426[30], 6}, {NULL}};
const Segment SGI41832[] = {{&S43426[127], 3}, {NULL}};
const Segment SGI41833[] = {{&S43426[93], 5}, {NULL}};
const Segment SGI41834[] = {{&S43428[44], 24}, {NULL}};
const Segment SGI41835[] = {{&S43428[44], 35}, {NULL}};
const Segment SGI41836[] = {{&S43428[30], 6}, {NULL}};
const Segment SGI41837[] = {{&S43428[127], 3}, {NULL}};
const Segment SGI41838[] = {{&S43428[93], 5}, {NULL}};
const Segment SGI41839[] = {{&S43429[38], 13}, {NULL}};
const Segment SGI41840[] = {{&S43429[31], 6}, {NULL}};
const Segment SGI41841[] = {{&S43263[44], 10}, {NULL}};
const Segment SGI41842[] = {{&S43263[44], 21}, {NULL}};
const Segment SGI41843[] = {{&S43263[30], 6}, {NULL}};
const Segment SGI41844[] = {{&S43263[113], 3}, {NULL}};
const Segment SGI41845[] = {{&S43263[79], 5}, {NULL}};
const Segment SGI41846[] = {{&S43264[38], 10}, {NULL}};
const Segment SGI41847[] = {{&S43264[31], 6}, {NULL}};
const Segment SGI41848[] = {{&S43510[44], 41}, {NULL}};
const Segment SGI41849[] = {{&S43510[44], 52}, {NULL}};
const Segment SGI41850[] = {{&S43510[30], 6}, {NULL}};
const Segment SGI41851[] = {{&S43510[144], 3}, {NULL}};
const Segment SGI41852[] = {{&S43510[110], 5}, {NULL}};
const Segment SGI41853[] = {{&S56065[44], 11}, {NULL}};
const Segment SGI41854[] = {{&S56065[44], 23}, {NULL}};
const Segment SGI41855[] = {{&S56065[30], 6}, {NULL}};
const Segment SGI41856[] = {{&S43511[44], 35}, {NULL}};
const Segment SGI41857[] = {{&S56065[115], 3}, {NULL}};
const Segment SGI41858[] = {{&S43511[44], 46}, {NULL}};
const Segment SGI41859[] = {{&S43511[30], 6}, {NULL}};
const Segment SGI41860[] = {{&S56065[81], 5}, {NULL}};
const Segment SGI41861[] = {{&S43511[138], 3}, {NULL}};
const Segment SGI41862[] = {{&S43511[104], 5}, {NULL}};
const Segment SGI41863[] = {{&S56066[38], 11}, {NULL}};
const Segment SGI41864[] = {{&S56066[31], 6}, {NULL}};
const Segment SGI41865[] = {{&S43512[44], 29}, {NULL}};
const Segment SGI41866[] = {{&S43512[44], 40}, {NULL}};
const Segment SGI41867[] = {{&S43512[30], 6}, {NULL}};
const Segment SGI41868[] = {{&S43512[132], 3}, {NULL}};
const Segment SGI41869[] = {{&S43512[98], 5}, {NULL}};
const Segment SGI41870[] = {{&S43514[44], 27}, {NULL}};
const Segment SGI41871[] = {{&S43514[44], 38}, {NULL}};
const Segment SGI41872[] = {{&S43514[30], 6}, {NULL}};
const Segment SGI41873[] = {{&S43514[130], 3}, {NULL}};
const Segment SGI41874[] = {{&S43514[96], 5}, {NULL}};
const Segment SGI41875[] = {{&S43515[38], 41}, {NULL}};
const Segment SGI41876[] = {{&S43515[31], 6}, {NULL}};
const Segment SGI41877[] = {{&S43516[44], 29}, {NULL}};
const Segment SGI41878[] = {{&S43516[44], 40}, {NULL}};
const Segment SGI41879[] = {{&S43516[30], 6}, {NULL}};
const Segment SGI41880[] = {{&S43516[132], 3}, {NULL}};
const Segment SGI41881[] = {{&S43516[98], 5}, {NULL}};
const Segment SGI41882[] = {{&S43517[44], 41}, {NULL}};
const Segment SGI41883[] = {{&S43517[44], 52}, {NULL}};
const Segment SGI41884[] = {{&S43517[30], 6}, {NULL}};
const Segment SGI41885[] = {{&S43517[144], 3}, {NULL}};
const Segment SGI41886[] = {{&S43517[110], 5}, {NULL}};
const Segment SGI41887[] = {{&S43518[44], 41}, {NULL}};
const Segment SGI41888[] = {{&S43518[44], 52}, {NULL}};
const Segment SGI41889[] = {{&S43518[30], 6}, {NULL}};
const Segment SGI41890[] = {{&S43518[113], 4}, {NULL}};
const Segment SGI41891[] = {{&S43518[113], 3}, {NULL}};
const Segment SGI41892[] = {{&S43519[38], 35}, {NULL}};
const Segment SGI41893[] = {{&S43519[31], 6}, {NULL}};
const Segment SGI41894[] = {{&S43520[44], 23}, {NULL}};
const Segment SGI41895[] = {{&S43520[44], 34}, {NULL}};
const Segment SGI41896[] = {{&S43520[30], 6}, {NULL}};
const Segment SGI41897[] = {{&S43520[126], 3}, {NULL}};
const Segment SGI41898[] = {{&S43520[92], 5}, {NULL}};
const Segment SGI41899[] = {{&S43521[71], 12}, {NULL}};
const Segment SGI41900[] = {{&S43521[58], 6}, {NULL}};
const Segment SGI41901[] = {{&S43521[36], 3}, {NULL}};
const Segment SGI41902[] = {{&S43521[97], 6}, {NULL}};
const Segment SGI41903[] = {{&S43522[44], 41}, {NULL}};
const Segment SGI41904[] = {{&S43522[44], 52}, {NULL}};
const Segment SGI41905[] = {{&S43522[30], 6}, {NULL}};
const Segment SGI41906[] = {{&S43522[113], 6}, {NULL}};
const Segment SGI41907[] = {{&S43523[44], 11}, {NULL}};
const Segment SGI41908[] = {{&S43523[44], 22}, {NULL}};
const Segment SGI41909[] = {{&S43523[30], 6}, {NULL}};
const Segment SGI41910[] = {{&S43523[83], 4}, {NULL}};
const Segment SGI41911[] = {{&S43523[83], 3}, {NULL}};
const Segment SGI41912[] = {{&S43524[44], 38}, {NULL}};
const Segment SGI41913[] = {{&S43524[44], 49}, {NULL}};
const Segment SGI41914[] = {{&S43524[30], 6}, {NULL}};
const Segment SGI41915[] = {{&S43524[141], 3}, {NULL}};
const Segment SGI41916[] = {{&S43524[107], 5}, {NULL}};
const Segment SGI41917[] = {{&S43525[44], 11}, {NULL}};
const Segment SGI41918[] = {{&S43525[44], 22}, {NULL}};
const Segment SGI41919[] = {{&S43525[30], 6}, {NULL}};
const Segment SGI41920[] = {{&S43525[114], 3}, {NULL}};
const Segment SGI41921[] = {{&S43525[80], 5}, {NULL}};
const Segment SGI41922[] = {{&S43526[44], 11}, {NULL}};
const Segment SGI41923[] = {{&S43526[44], 22}, {NULL}};
const Segment SGI41924[] = {{&S43526[30], 6}, {NULL}};
const Segment SGI41925[] = {{&S43526[83], 6}, {NULL}};
const Segment SGI41926[] = {{&S43336[44], 7}, {NULL}};
const Segment SGI41927[] = {{&S43336[44], 19}, {NULL}};
const Segment SGI41928[] = {{&S43336[30], 6}, {NULL}};
const Segment SGI41929[] = {{&S43336[111], 3}, {NULL}};
const Segment SGI41930[] = {{&S43336[77], 5}, {NULL}};
const Segment SGI41931[] = {{&S43337[44], 7}, {NULL}};
const Segment SGI41932[] = {{&S43337[44], 19}, {NULL}};
const Segment SGI41933[] = {{&S43337[30], 6}, {NULL}};
const Segment SGI41934[] = {{&S43337[111], 3}, {NULL}};
const Segment SGI41935[] = {{&S43337[77], 5}, {NULL}};
const Segment SGI41936[] = {{&S43339[54], 7}, {NULL}};
const Segment SGI41937[] = {{&S43339[54], 19}, {NULL}};
const Segment SGI41938[] = {{&S43339[40], 6}, {NULL}};
const Segment SGI41939[] = {{&S43339[121], 3}, {NULL}};
const Segment SGI41940[] = {{&S43339[87], 5}, {NULL}};
const Segment SGI41941[] = {{&S43341[44], 7}, {NULL}};
const Segment SGI41942[] = {{&S43341[44], 19}, {NULL}};
const Segment SGI41943[] = {{&S43341[30], 6}, {NULL}};
const Segment SGI41944[] = {{&S43341[111], 3}, {NULL}};
const Segment SGI41945[] = {{&S43341[77], 5}, {NULL}};
const Segment SGI41946[] = {{&S43344[44], 7}, {NULL}};
const Segment SGI41947[] = {{&S43344[44], 19}, {NULL}};
const Segment SGI41948[] = {{&S43344[30], 6}, {NULL}};
const Segment SGI41949[] = {{&S43344[111], 3}, {NULL}};
const Segment SGI41950[] = {{&S43344[77], 5}, {NULL}};
const Segment SGI41951[] = {{&S43345[44], 7}, {NULL}};
const Segment SGI41952[] = {{&S43345[44], 19}, {NULL}};
const Segment SGI41953[] = {{&S43345[30], 6}, {NULL}};
const Segment SGI41954[] = {{&S43345[111], 3}, {NULL}};
const Segment SGI41955[] = {{&S43345[77], 5}, {NULL}};
const Segment SGI41956[] = {{&S43346[44], 7}, {NULL}};
const Segment SGI41957[] = {{&S43346[44], 19}, {NULL}};
const Segment SGI41958[] = {{&S43346[30], 6}, {NULL}};
const Segment SGI41959[] = {{&S43346[111], 3}, {NULL}};
const Segment SGI41960[] = {{&S43346[77], 5}, {NULL}};
const Segment SGI41961[] = {{&S43347[44], 7}, {NULL}};
const Segment SGI41962[] = {{&S43347[44], 19}, {NULL}};
const Segment SGI41963[] = {{&S43347[30], 6}, {NULL}};
const Segment SGI41964[] = {{&S43347[111], 3}, {NULL}};
const Segment SGI41965[] = {{&S43347[77], 5}, {NULL}};
const Segment SGI41966[] = {{&S43348[44], 7}, {NULL}};
const Segment SGI41967[] = {{&S43348[44], 19}, {NULL}};
const Segment SGI41968[] = {{&S43348[30], 6}, {NULL}};
const Segment SGI41969[] = {{&S43348[111], 3}, {NULL}};
const Segment SGI41970[] = {{&S43348[77], 5}, {NULL}};
const Segment SGI41971[] = {{&S43349[44], 7}, {NULL}};
const Segment SGI41972[] = {{&S43349[44], 19}, {NULL}};
const Segment SGI41973[] = {{&S43349[30], 6}, {NULL}};
const Segment SGI41974[] = {{&S43349[80], 4}, {NULL}};
const Segment SGI41975[] = {{&S43349[80], 3}, {NULL}};
const Segment SGI41976[] = {{&S43351[44], 7}, {NULL}};
const Segment SGI41977[] = {{&S43351[44], 19}, {NULL}};
const Segment SGI41978[] = {{&S43351[30], 6}, {NULL}};
const Segment SGI41979[] = {{&S43351[111], 3}, {NULL}};
const Segment SGI41980[] = {{&S43351[77], 5}, {NULL}};
const Segment SGI41981[] = {{&S43353[38], 7}, {NULL}};
const Segment SGI41982[] = {{&S43353[31], 6}, {NULL}};
const Segment SGI41983[] = {{&S43602[44], 12}, {NULL}};
const Segment SGI41984[] = {{&S43602[44], 23}, {NULL}};
const Segment SGI41985[] = {{&S43602[30], 6}, {NULL}};
const Segment SGI41986[] = {{&S43602[115], 3}, {NULL}};
const Segment SGI41987[] = {{&S43602[81], 5}, {NULL}};
const Segment SGI41988[] = {{&S43603[44], 15}, {NULL}};
const Segment SGI41989[] = {{&S43603[44], 26}, {NULL}};
const Segment SGI41990[] = {{&S43603[30], 6}, {NULL}};
const Segment SGI41991[] = {{&S43603[118], 3}, {NULL}};
const Segment SGI41992[] = {{&S43603[84], 5}, {NULL}};
const Segment SGI41993[] = {{&S43605[38], 12}, {NULL}};
const Segment SGI41994[] = {{&S43605[31], 6}, {NULL}};
const Segment SGI41995[] = {{&S43607[44], 14}, {NULL}};
const Segment SGI41996[] = {{&S43607[44], 25}, {NULL}};
const Segment SGI41997[] = {{&S43607[30], 6}, {NULL}};
const Segment SGI41998[] = {{&S43607[117], 3}, {NULL}};
const Segment SGI41999[] = {{&S43607[83], 5}, {NULL}};
const Segment SGI42000[] = {{&S43610[43], 15}, {NULL}};
const Segment SGI42001[] = {{&S43610[43], 26}, {NULL}};
const Segment SGI42002[] = {{&S43610[30], 5}, {NULL}};
const Segment SGI42003[] = {{&S43610[117], 3}, {NULL}};
const Segment SGI42004[] = {{&S43610[83], 5}, {NULL}};
const Segment SGI42005[] = {{&S43513[44], 27}, {NULL}};
const Segment SGI42006[] = {{&S43513[44], 38}, {NULL}};
const Segment SGI42007[] = {{&S43513[30], 6}, {NULL}};
const Segment SGI42008[] = {{&S43513[130], 3}, {NULL}};
const Segment SGI42009[] = {{&S43513[96], 5}, {NULL}};
const Segment SGI42010[] = {{&S56317[44], 14}, {NULL}};
const Segment SGI42011[] = {{&S56317[44], 31}, {NULL}};
const Segment SGI42012[] = {{&S56317[30], 6}, {NULL}};
const Segment SGI42013[] = {{&S56317[123], 3}, {NULL}};
const Segment SGI42014[] = {{&S56317[89], 5}, {NULL}};
const Segment SGI42015[] = {{&S56318[38], 14}, {NULL}};
const Segment SGI42016[] = {{&S56318[31], 6}, {NULL}};
const Segment SGI42017[] = {{&S43570[44], 10}, {NULL}};
const Segment SGI42018[] = {{&S43570[44], 21}, {NULL}};
const Segment SGI42019[] = {{&S43570[30], 6}, {NULL}};
const Segment SGI42020[] = {{&S43570[113], 3}, {NULL}};
const Segment SGI42021[] = {{&S43570[79], 5}, {NULL}};
const Segment SGI42022[] = {{&S43571[38], 10}, {NULL}};
const Segment SGI42023[] = {{&S43571[31], 6}, {NULL}};
const Segment SGI42024[] = {{&S43654[44], 7}, {NULL}};
const Segment SGI42025[] = {{&S43654[44], 24}, {NULL}};
const Segment SGI42026[] = {{&S43654[30], 6}, {NULL}};
const Segment SGI42027[] = {{&S43654[116], 3}, {NULL}};
const Segment SGI42028[] = {{&S43654[82], 5}, {NULL}};
const Segment SGI42029[] = {{&S43655[38], 7}, {NULL}};
const Segment SGI42030[] = {{&S43655[31], 6}, {NULL}};
const Segment SGI42031[] = {{&S43658[42], 7}, {NULL}};
const Segment SGI42032[] = {{&S43658[30], 6}, {NULL}};
const Segment SGI42033[] = {{&S43658[114], 3}, {NULL}};
const Segment SGI42034[] = {{&S43658[80], 5}, {NULL}};
const Segment SGI42035[] = {{&S43729[44], 11}, {NULL}};
const Segment SGI42036[] = {{&S43729[44], 38}, {NULL}};
const Segment SGI42037[] = {{&S43729[30], 6}, {NULL}};
const Segment SGI42038[] = {{&S43729[130], 3}, {NULL}};
const Segment SGI42039[] = {{&S43729[96], 5}, {NULL}};
const Segment SGI42040[] = {{&S43737[44], 11}, {NULL}};
const Segment SGI42041[] = {{&S43737[44], 33}, {NULL}};
const Segment SGI42042[] = {{&S43737[30], 6}, {NULL}};
const Segment SGI42043[] = {{&S43737[125], 3}, {NULL}};
const Segment SGI42044[] = {{&S43737[91], 5}, {NULL}};
const Segment SGI42045[] = {{&S43738[44], 11}, {NULL}};
const Segment SGI42046[] = {{&S43738[44], 33}, {NULL}};
const Segment SGI42047[] = {{&S43738[30], 6}, {NULL}};
const Segment SGI42048[] = {{&S43738[125], 3}, {NULL}};
const Segment SGI42049[] = {{&S43738[91], 5}, {NULL}};
const Segment SGI42050[] = {{&S46041[44], 8}, {NULL}};
const Segment SGI42051[] = {{&S46041[44], 25}, {NULL}};
const Segment SGI42052[] = {{&S46041[30], 6}, {NULL}};
const Segment SGI42053[] = {{&S46041[117], 3}, {NULL}};
const Segment SGI42054[] = {{&S46041[83], 5}, {NULL}};
const Segment SGI42055[] = {{&S46043[42], 8}, {NULL}};
const Segment SGI42056[] = {{&S46043[30], 6}, {NULL}};
const Segment SGI42057[] = {{&S46043[115], 3}, {NULL}};
const Segment SGI42058[] = {{&S46043[81], 5}, {NULL}};
const Segment SGI42059[] = {{&S46045[38], 8}, {NULL}};
const Segment SGI42060[] = {{&S46045[31], 6}, {NULL}};
const Segment SGI42061[] = {{&S46050[44], 8}, {NULL}};
const Segment SGI42062[] = {{&S46050[44], 25}, {NULL}};
const Segment SGI42063[] = {{&S46050[30], 6}, {NULL}};
const Segment SGI42064[] = {{&S46050[118], 3}, {NULL}};
const Segment SGI42065[] = {{&S46050[83], 6}, {NULL}};
const Segment SGI42066[] = {{&S43924[44], 24}, {NULL}};
const Segment SGI42067[] = {{&S43924[44], 35}, {NULL}};
const Segment SGI42068[] = {{&S43924[30], 6}, {NULL}};
const Segment SGI42069[] = {{&S43924[127], 3}, {NULL}};
const Segment SGI42070[] = {{&S43924[93], 5}, {NULL}};
const Segment SGI42071[] = {{&S43925[44], 24}, {NULL}};
const Segment SGI42072[] = {{&S43925[44], 35}, {NULL}};
const Segment SGI42073[] = {{&S43925[30], 6}, {NULL}};
const Segment SGI42074[] = {{&S43925[127], 3}, {NULL}};
const Segment SGI42075[] = {{&S43925[93], 5}, {NULL}};
const Segment SGI42076[] = {{&S46171[44], 7}, {NULL}};
const Segment SGI42077[] = {{&S46171[44], 18}, {NULL}};
const Segment SGI42078[] = {{&S46171[30], 6}, {NULL}};
const Segment SGI42079[] = {{&S46171[110], 3}, {NULL}};
const Segment SGI42080[] = {{&S46171[76], 5}, {NULL}};
const Segment SGI42081[] = {{&S47855[44], 20}, {NULL}};
const Segment SGI42082[] = {{&S47855[44], 37}, {NULL}};
const Segment SGI42083[] = {{&S47855[30], 6}, {NULL}};
const Segment SGI42084[] = {{&S47855[129], 3}, {NULL}};
const Segment SGI42085[] = {{&S47855[95], 5}, {NULL}};
const Segment SGI42086[] = {{&S47858[44], 20}, {NULL}};
const Segment SGI42087[] = {{&S47858[44], 37}, {NULL}};
const Segment SGI42088[] = {{&S47858[30], 6}, {NULL}};
const Segment SGI42089[] = {{&S47858[129], 3}, {NULL}};
const Segment SGI42090[] = {{&S47858[95], 5}, {NULL}};
const Segment SGI42091[] = {{&S46196[44], 10}, {NULL}};
const Segment SGI42092[] = {{&S46196[44], 21}, {NULL}};
const Segment SGI42093[] = {{&S46196[30], 6}, {NULL}};
const Segment SGI42094[] = {{&S46196[113], 3}, {NULL}};
const Segment SGI42095[] = {{&S46196[79], 5}, {NULL}};
const Segment SGI42096[] = {{&S46220[44], 8}, {NULL}};
const Segment SGI42097[] = {{&S46220[44], 26}, {NULL}};
const Segment SGI42098[] = {{&S46220[30], 6}, {NULL}};
const Segment SGI42099[] = {{&S46220[118], 3}, {NULL}};
const Segment SGI42100[] = {{&S46220[84], 5}, {NULL}};
const Segment SGI42101[] = {{&S46221[44], 20}, {NULL}};
const Segment SGI42102[] = {{&S46221[44], 38}, {NULL}};
const Segment SGI42103[] = {{&S46221[30], 6}, {NULL}};
const Segment SGI42104[] = {{&S46221[130], 3}, {NULL}};
const Segment SGI42105[] = {{&S46221[96], 5}, {NULL}};
const Segment SGI42106[] = {{&S46222[44], 8}, {NULL}};
const Segment SGI42107[] = {{&S46222[44], 27}, {NULL}};
const Segment SGI42108[] = {{&S46222[30], 6}, {NULL}};
const Segment SGI42109[] = {{&S46222[119], 3}, {NULL}};
const Segment SGI42110[] = {{&S46222[85], 5}, {NULL}};
const Segment SGI42111[] = {{&S46223[44], 20}, {NULL}};
const Segment SGI42112[] = {{&S46223[44], 39}, {NULL}};
const Segment SGI42113[] = {{&S46223[30], 6}, {NULL}};
const Segment SGI42114[] = {{&S46223[131], 3}, {NULL}};
const Segment SGI42115[] = {{&S46223[97], 5}, {NULL}};
const Segment SGI42116[] = {{&S46225[44], 8}, {NULL}};
const Segment SGI42117[] = {{&S46225[44], 27}, {NULL}};
const Segment SGI42118[] = {{&S46225[30], 6}, {NULL}};
const Segment SGI42119[] = {{&S46225[119], 3}, {NULL}};
const Segment SGI42120[] = {{&S46225[85], 5}, {NULL}};
const Segment SGI42121[] = {{&S46226[44], 20}, {NULL}};
const Segment SGI42122[] = {{&S46226[44], 39}, {NULL}};
const Segment SGI42123[] = {{&S46226[30], 6}, {NULL}};
const Segment SGI42124[] = {{&S46226[131], 3}, {NULL}};
const Segment SGI42125[] = {{&S46226[97], 5}, {NULL}};
const Segment SGI42126[] = {{&S46232[38], 8}, {NULL}};
const Segment SGI42127[] = {{&S46232[31], 6}, {NULL}};
const Segment SGI42128[] = {{&S46352[44], 13}, {NULL}};
const Segment SGI42129[] = {{&S46352[44], 24}, {NULL}};
const Segment SGI42130[] = {{&S46352[30], 6}, {NULL}};
const Segment SGI42131[] = {{&S46352[116], 3}, {NULL}};
const Segment SGI42132[] = {{&S46352[82], 5}, {NULL}};
const Segment SGI42133[] = {{&S46353[38], 13}, {NULL}};
const Segment SGI42134[] = {{&S46353[31], 6}, {NULL}};
const Segment SGI42135[] = {{&S44385[44], 5}, {NULL}};
const Segment SGI42136[] = {{&S44385[44], 16}, {NULL}};
const Segment SGI42137[] = {{&S44385[30], 6}, {NULL}};
const Segment SGI42138[] = {{&S44385[108], 3}, {NULL}};
const Segment SGI42139[] = {{&S44385[74], 5}, {NULL}};
const Segment SGI42140[] = {{&S44386[44], 9}, {NULL}};
const Segment SGI42141[] = {{&S44386[44], 20}, {NULL}};
const Segment SGI42142[] = {{&S44386[30], 6}, {NULL}};
const Segment SGI42143[] = {{&S44386[112], 3}, {NULL}};
const Segment SGI42144[] = {{&S44386[78], 5}, {NULL}};
const Segment SGI42145[] = {{&S44394[44], 9}, {NULL}};
const Segment SGI42146[] = {{&S44394[44], 20}, {NULL}};
const Segment SGI42147[] = {{&S44394[30], 6}, {NULL}};
const Segment SGI42148[] = {{&S44394[112], 3}, {NULL}};
const Segment SGI42149[] = {{&S44394[78], 5}, {NULL}};
const Segment SGI42150[] = {{&S44398[38], 9}, {NULL}};
const Segment SGI42151[] = {{&S44398[31], 6}, {NULL}};
const Segment SGI42152[] = {{&S44404[44], 9}, {NULL}};
const Segment SGI42153[] = {{&S44404[44], 20}, {NULL}};
const Segment SGI42154[] = {{&S44404[30], 6}, {NULL}};
const Segment SGI42155[] = {{&S44404[112], 3}, {NULL}};
const Segment SGI42156[] = {{&S44404[78], 5}, {NULL}};
const Segment SGI42157[] = {{&S44418[44], 7}, {NULL}};
const Segment SGI42158[] = {{&S44418[44], 18}, {NULL}};
const Segment SGI42159[] = {{&S44418[30], 6}, {NULL}};
const Segment SGI42160[] = {{&S44418[110], 3}, {NULL}};
const Segment SGI42161[] = {{&S44418[76], 5}, {NULL}};
const Segment SGI42162[] = {{&S44442[44], 14}, {NULL}};
const Segment SGI42163[] = {{&S44442[44], 31}, {NULL}};
const Segment SGI42164[] = {{&S44442[30], 6}, {NULL}};
const Segment SGI42165[] = {{&S44442[123], 3}, {NULL}};
const Segment SGI42166[] = {{&S44442[89], 5}, {NULL}};
const Segment SGI42167[] = {{&S44443[38], 14}, {NULL}};
const Segment SGI42168[] = {{&S44443[31], 6}, {NULL}};
const Segment SGI42169[] = {{&S57029[6], 4}, {NULL}};
const Segment SGI42170[] = {{&S57029[83], 5}, {NULL}};
const Segment SGI42171[] = {{&S46504[6], 4}, {NULL}};
const Segment SGI42172[] = {{&S46504[83], 5}, {NULL}};
const Segment SGI42173[] = {{&S44507[44], 18}, {NULL}};
const Segment SGI42174[] = {{&S44507[44], 29}, {NULL}};
const Segment SGI42175[] = {{&S44507[30], 6}, {NULL}};
const Segment SGI42176[] = {{&S44507[121], 3}, {NULL}};
const Segment SGI42177[] = {{&S44507[87], 5}, {NULL}};
const Segment SGI42178[] = {{&S44508[44], 9}, {NULL}};
const Segment SGI42179[] = {{&S44508[44], 20}, {NULL}};
const Segment SGI42180[] = {{&S44508[30], 6}, {NULL}};
const Segment SGI42181[] = {{&S44508[112], 3}, {NULL}};
const Segment SGI42182[] = {{&S44508[78], 5}, {NULL}};
const Segment SGI42183[] = {{&S44509[38], 9}, {NULL}};
const Segment SGI42184[] = {{&S44509[31], 6}, {NULL}};
const Segment SGI42185[] = {{&S44510[44], 22}, {NULL}};
const Segment SGI42186[] = {{&S44510[44], 33}, {NULL}};
const Segment SGI42187[] = {{&S44510[30], 6}, {NULL}};
const Segment SGI42188[] = {{&S44510[125], 3}, {NULL}};
const Segment SGI42189[] = {{&S44510[91], 5}, {NULL}};
const Segment SGI42190[] = {{&S44511[44], 22}, {NULL}};
const Segment SGI42191[] = {{&S44511[44], 33}, {NULL}};
const Segment SGI42192[] = {{&S44511[30], 6}, {NULL}};
const Segment SGI42193[] = {{&S44511[125], 3}, {NULL}};
const Segment SGI42194[] = {{&S44511[91], 5}, {NULL}};
const Segment SGI42195[] = {{&S46577[44], 8}, {NULL}};
const Segment SGI42196[] = {{&S46577[44], 27}, {NULL}};
const Segment SGI42197[] = {{&S46577[30], 6}, {NULL}};
const Segment SGI42198[] = {{&S46577[119], 3}, {NULL}};
const Segment SGI42199[] = {{&S46577[85], 5}, {NULL}};
const Segment SGI42200[] = {{&S46578[44], 20}, {NULL}};
const Segment SGI42201[] = {{&S46578[44], 39}, {NULL}};
const Segment SGI42202[] = {{&S46578[30], 6}, {NULL}};
const Segment SGI42203[] = {{&S46578[131], 3}, {NULL}};
const Segment SGI42204[] = {{&S46578[97], 5}, {NULL}};
const Segment SGI42205[] = {{&S46579[44], 20}, {NULL}};
const Segment SGI42206[] = {{&S46579[44], 38}, {NULL}};
const Segment SGI42207[] = {{&S46579[30], 6}, {NULL}};
const Segment SGI42208[] = {{&S46579[130], 3}, {NULL}};
const Segment SGI42209[] = {{&S46579[96], 5}, {NULL}};
const Segment SGI42210[] = {{&S46580[44], 8}, {NULL}};
const Segment SGI42211[] = {{&S46580[44], 27}, {NULL}};
const Segment SGI42212[] = {{&S46580[30], 6}, {NULL}};
const Segment SGI42213[] = {{&S46580[119], 3}, {NULL}};
const Segment SGI42214[] = {{&S46580[85], 5}, {NULL}};
const Segment SGI42215[] = {{&S46581[44], 20}, {NULL}};
const Segment SGI42216[] = {{&S46581[44], 39}, {NULL}};
const Segment SGI42217[] = {{&S46581[30], 6}, {NULL}};
const Segment SGI42218[] = {{&S46581[131], 3}, {NULL}};
const Segment SGI42219[] = {{&S46581[97], 5}, {NULL}};
const Segment SGI42220[] = {{&S46586[38], 8}, {NULL}};
const Segment SGI42221[] = {{&S46586[31], 6}, {NULL}};
const Segment SGI42222[] = {{&S46589[44], 20}, {NULL}};
const Segment SGI42223[] = {{&S46589[44], 38}, {NULL}};
const Segment SGI42224[] = {{&S46589[30], 6}, {NULL}};
const Segment SGI42225[] = {{&S44709[44], 11}, {NULL}};
const Segment SGI42226[] = {{&S44709[44], 28}, {NULL}};
const Segment SGI42227[] = {{&S44709[30], 6}, {NULL}};
const Segment SGI42228[] = {{&S44709[120], 3}, {NULL}};
const Segment SGI42229[] = {{&S44709[86], 5}, {NULL}};
const Segment SGI42230[] = {{&S44710[44], 11}, {NULL}};
const Segment SGI42231[] = {{&S44710[44], 28}, {NULL}};
const Segment SGI42232[] = {{&S44710[30], 6}, {NULL}};
const Segment SGI42233[] = {{&S44363[44], 14}, {NULL}};
const Segment SGI42234[] = {{&S44710[120], 3}, {NULL}};
const Segment SGI42235[] = {{&S44363[44], 25}, {NULL}};
const Segment SGI42236[] = {{&S44363[30], 6}, {NULL}};
const Segment SGI42237[] = {{&S44710[86], 5}, {NULL}};
const Segment SGI42238[] = {{&S44363[117], 3}, {NULL}};
const Segment SGI42239[] = {{&S44363[83], 5}, {NULL}};
const Segment SGI42240[] = {{&S44711[44], 11}, {NULL}};
const Segment SGI42241[] = {{&S44711[44], 28}, {NULL}};
const Segment SGI42242[] = {{&S44711[30], 6}, {NULL}};
const Segment SGI42243[] = {{&S44364[44], 9}, {NULL}};
const Segment SGI42244[] = {{&S44711[120], 3}, {NULL}};
const Segment SGI42245[] = {{&S44364[44], 20}, {NULL}};
const Segment SGI42246[] = {{&S44364[30], 6}, {NULL}};
const Segment SGI42247[] = {{&S44711[86], 5}, {NULL}};
const Segment SGI42248[] = {{&S44364[112], 3}, {NULL}};
const Segment SGI42249[] = {{&S44364[78], 5}, {NULL}};
const Segment SGI42250[] = {{&S44712[44], 11}, {NULL}};
const Segment SGI42251[] = {{&S44712[44], 28}, {NULL}};
const Segment SGI42252[] = {{&S44365[38], 9}, {NULL}};
const Segment SGI42253[] = {{&S44712[30], 6}, {NULL}};
const Segment SGI42254[] = {{&S44365[31], 6}, {NULL}};
const Segment SGI42255[] = {{&S44712[120], 3}, {NULL}};
const Segment SGI42256[] = {{&S44712[86], 5}, {NULL}};
const Segment SGI42257[] = {{&S44366[44], 14}, {NULL}};
const Segment SGI42258[] = {{&S44366[44], 48}, {NULL}};
const Segment SGI42259[] = {{&S44366[30], 6}, {NULL}};
const Segment SGI42260[] = {{&S44366[140], 3}, {NULL}};
const Segment SGI42261[] = {{&S44366[106], 5}, {NULL}};
const Segment SGI42262[] = {{&S44715[38], 11}, {NULL}};
const Segment SGI42263[] = {{&S44367[44], 9}, {NULL}};
const Segment SGI42264[] = {{&S44715[31], 6}, {NULL}};
const Segment SGI42265[] = {{&S44367[44], 20}, {NULL}};
const Segment SGI42266[] = {{&S44367[30], 6}, {NULL}};
const Segment SGI42267[] = {{&S44367[78], 5}, {NULL}};
const Segment SGI42268[] = {{&S44721[38], 11}, {NULL}};
const Segment SGI42269[] = {{&S44721[31], 6}, {NULL}};
const Segment SGI42270[] = {{&S48327[6], 4}, {NULL}};
const Segment SGI42271[] = {{&S48327[76], 5}, {NULL}};
const Segment SGI42272[] = {{&S44387[44], 9}, {NULL}};
const Segment SGI42273[] = {{&S44387[44], 20}, {NULL}};
const Segment SGI42274[] = {{&S44387[30], 6}, {NULL}};
const Segment SGI42275[] = {{&S44387[112], 3}, {NULL}};
const Segment SGI42276[] = {{&S44387[78], 5}, {NULL}};
const Segment SGI42277[] = {{&S44388[44], 9}, {NULL}};
const Segment SGI42278[] = {{&S44388[44], 20}, {NULL}};
const Segment SGI42279[] = {{&S44388[30], 6}, {NULL}};
const Segment SGI42280[] = {{&S44388[112], 3}, {NULL}};
const Segment SGI42281[] = {{&S44388[78], 5}, {NULL}};
const Segment SGI42282[] = {{&S44389[44], 5}, {NULL}};
const Segment SGI42283[] = {{&S44389[44], 16}, {NULL}};
const Segment SGI42284[] = {{&S44389[30], 6}, {NULL}};
const Segment SGI42285[] = {{&S44389[108], 3}, {NULL}};
const Segment SGI42286[] = {{&S44389[74], 5}, {NULL}};
const Segment SGI42287[] = {{&S44400[44], 9}, {NULL}};
const Segment SGI42288[] = {{&S44400[44], 20}, {NULL}};
const Segment SGI42289[] = {{&S44400[30], 6}, {NULL}};
const Segment SGI42290[] = {{&S44400[112], 3}, {NULL}};
const Segment SGI42291[] = {{&S44400[78], 5}, {NULL}};
const Segment SGI42292[] = {{&S44411[44], 9}, {NULL}};
const Segment SGI42293[] = {{&S44411[44], 29}, {NULL}};
const Segment SGI42294[] = {{&S44411[30], 6}, {NULL}};
const Segment SGI42295[] = {{&S44411[121], 3}, {NULL}};
const Segment SGI42296[] = {{&S44411[87], 5}, {NULL}};
const Segment SGI42297[] = {{&S44412[44], 9}, {NULL}};
const Segment SGI42298[] = {{&S44412[44], 31}, {NULL}};
const Segment SGI42299[] = {{&S44412[30], 6}, {NULL}};
const Segment SGI42300[] = {{&S44412[123], 3}, {NULL}};
const Segment SGI42301[] = {{&S44412[89], 5}, {NULL}};
const Segment SGI42302[] = {{&S44413[44], 5}, {NULL}};
const Segment SGI42303[] = {{&S44413[44], 16}, {NULL}};
const Segment SGI42304[] = {{&S44413[30], 6}, {NULL}};
const Segment SGI42305[] = {{&S44413[108], 3}, {NULL}};
const Segment SGI42306[] = {{&S44413[74], 5}, {NULL}};
const Segment SGI42307[] = {{&S44415[38], 9}, {NULL}};
const Segment SGI42308[] = {{&S44415[31], 6}, {NULL}};
const Segment SGI42309[] = {{&S48393[44], 19}, {NULL}};
const Segment SGI42310[] = {{&S48393[44], 36}, {NULL}};
const Segment SGI42311[] = {{&S48393[30], 6}, {NULL}};
const Segment SGI42312[] = {{&S48393[128], 3}, {NULL}};
const Segment SGI42313[] = {{&S48393[94], 5}, {NULL}};
const Segment SGI42314[] = {{&S48404[44], 19}, {NULL}};
const Segment SGI42315[] = {{&S48404[44], 36}, {NULL}};
const Segment SGI42316[] = {{&S48404[30], 6}, {NULL}};
const Segment SGI42317[] = {{&S48404[128], 3}, {NULL}};
const Segment SGI42318[] = {{&S48404[94], 5}, {NULL}};
const Segment SGI42319[] = {{&S44866[44], 14}, {NULL}};
const Segment SGI42320[] = {{&S44866[44], 31}, {NULL}};
const Segment SGI42321[] = {{&S44866[30], 6}, {NULL}};
const Segment SGI42322[] = {{&S44866[123], 3}, {NULL}};
const Segment SGI42323[] = {{&S44866[89], 5}, {NULL}};
const Segment SGI42324[] = {{&S44867[44], 14}, {NULL}};
const Segment SGI42325[] = {{&S44867[44], 25}, {NULL}};
const Segment SGI42326[] = {{&S44867[30], 6}, {NULL}};
const Segment SGI42327[] = {{&S44867[117], 3}, {NULL}};
const Segment SGI42328[] = {{&S44867[83], 5}, {NULL}};
const Segment SGI42329[] = {{&S44868[44], 9}, {NULL}};
const Segment SGI42330[] = {{&S44868[44], 20}, {NULL}};
const Segment SGI42331[] = {{&S44868[30], 6}, {NULL}};
const Segment SGI42332[] = {{&S44868[112], 3}, {NULL}};
const Segment SGI42333[] = {{&S44868[78], 5}, {NULL}};
const Segment SGI42334[] = {{&S44869[44], 10}, {NULL}};
const Segment SGI42335[] = {{&S44869[44], 21}, {NULL}};
const Segment SGI42336[] = {{&S44869[30], 6}, {NULL}};
const Segment SGI42337[] = {{&S44869[113], 3}, {NULL}};
const Segment SGI42338[] = {{&S44869[79], 5}, {NULL}};
const Segment SGI42339[] = {{&S57317[44], 10}, {NULL}};
const Segment SGI42340[] = {{&S57317[44], 21}, {NULL}};
const Segment SGI42341[] = {{&S57317[30], 6}, {NULL}};
const Segment SGI42342[] = {{&S57317[113], 3}, {NULL}};
const Segment SGI42343[] = {{&S44870[44], 14}, {NULL}};
const Segment SGI42344[] = {{&S57317[79], 5}, {NULL}};
const Segment SGI42345[] = {{&S44870[44], 25}, {NULL}};
const Segment SGI42346[] = {{&S44870[30], 6}, {NULL}};
const Segment SGI42347[] = {{&S44870[117], 3}, {NULL}};
const Segment SGI42348[] = {{&S44870[83], 5}, {NULL}};
const Segment SGI42349[] = {{&S57318[38], 10}, {NULL}};
const Segment SGI42350[] = {{&S57318[31], 6}, {NULL}};
const Segment SGI42351[] = {{&S44871[38], 9}, {NULL}};
const Segment SGI42352[] = {{&S44871[31], 6}, {NULL}};
const Segment SGI42353[] = {{&S44872[44], 14}, {NULL}};
const Segment SGI42354[] = {{&S44872[44], 25}, {NULL}};
const Segment SGI42355[] = {{&S44872[30], 6}, {NULL}};
const Segment SGI42356[] = {{&S44872[117], 3}, {NULL}};
const Segment SGI42357[] = {{&S44872[83], 5}, {NULL}};
const Segment SGI42358[] = {{&S44873[44], 14}, {NULL}};
const Segment SGI42359[] = {{&S44873[44], 25}, {NULL}};
const Segment SGI42360[] = {{&S44873[30], 6}, {NULL}};
const Segment SGI42361[] = {{&S44873[117], 3}, {NULL}};
const Segment SGI42362[] = {{&S44873[83], 5}, {NULL}};
const Segment SGI42363[] = {{&S44874[44], 14}, {NULL}};
const Segment SGI42364[] = {{&S44874[44], 25}, {NULL}};
const Segment SGI42365[] = {{&S44874[30], 6}, {NULL}};
const Segment SGI42366[] = {{&S44874[117], 3}, {NULL}};
const Segment SGI42367[] = {{&S44874[83], 5}, {NULL}};
const Segment SGI42368[] = {{&S44875[44], 10}, {NULL}};
const Segment SGI42369[] = {{&S44875[44], 21}, {NULL}};
const Segment SGI42370[] = {{&S44875[30], 6}, {NULL}};
const Segment SGI42371[] = {{&S44875[79], 5}, {NULL}};
const Segment SGI42372[] = {{&S46881[44], 17}, {NULL}};
const Segment SGI42373[] = {{&S46881[44], 28}, {NULL}};
const Segment SGI42374[] = {{&S46881[30], 6}, {NULL}};
const Segment SGI42375[] = {{&S46881[120], 3}, {NULL}};
const Segment SGI42376[] = {{&S46881[86], 5}, {NULL}};
const Segment SGI42377[] = {{&S46882[44], 16}, {NULL}};
const Segment SGI42378[] = {{&S46882[44], 27}, {NULL}};
const Segment SGI42379[] = {{&S46882[30], 6}, {NULL}};
const Segment SGI42380[] = {{&S46882[119], 3}, {NULL}};
const Segment SGI42381[] = {{&S46882[85], 5}, {NULL}};
const Segment SGI42382[] = {{&S46884[44], 17}, {NULL}};
const Segment SGI42383[] = {{&S52377[44], 5}, {NULL}};
const Segment SGI42384[] = {{&S46884[44], 28}, {NULL}};
const Segment SGI42385[] = {{&S52377[44], 16}, {NULL}};
const Segment SGI42386[] = {{&S46884[30], 6}, {NULL}};
const Segment SGI42387[] = {{&S52377[30], 6}, {NULL}};
const Segment SGI42388[] = {{&S46884[120], 3}, {NULL}};
const Segment SGI42389[] = {{&S52377[108], 3}, {NULL}};
const Segment SGI42390[] = {{&S46884[86], 5}, {NULL}};
const Segment SGI42391[] = {{&S52377[74], 5}, {NULL}};
const Segment SGI42392[] = {{&S46885[38], 16}, {NULL}};
const Segment SGI42393[] = {{&S46885[31], 6}, {NULL}};
const Segment SGI42394[] = {{&S46886[38], 17}, {NULL}};
const Segment SGI42395[] = {{&S46886[31], 6}, {NULL}};
const Segment SGI42396[] = {{&S52381[44], 9}, {NULL}};
const Segment SGI42397[] = {{&S52381[44], 26}, {NULL}};
const Segment SGI42398[] = {{&S52381[30], 6}, {NULL}};
const Segment SGI42399[] = {{&S52381[118], 3}, {NULL}};
const Segment SGI42400[] = {{&S52381[84], 5}, {NULL}};
const Segment SGI42401[] = {{&S52385[38], 5}, {NULL}};
const Segment SGI42402[] = {{&S52385[31], 6}, {NULL}};
const Segment SGI42403[] = {{&S52386[44], 12}, {NULL}};
const Segment SGI42404[] = {{&S52386[44], 23}, {NULL}};
const Segment SGI42405[] = {{&S52386[30], 6}, {NULL}};
const Segment SGI42406[] = {{&S52386[115], 3}, {NULL}};
const Segment SGI42407[] = {{&S52386[81], 5}, {NULL}};
const Segment SGI42408[] = {{&S46894[44], 11}, {NULL}};
const Segment SGI42409[] = {{&S46894[44], 22}, {NULL}};
const Segment SGI42410[] = {{&S46894[30], 6}, {NULL}};
const Segment SGI42411[] = {{&S52387[38], 9}, {NULL}};
const Segment SGI42412[] = {{&S52387[31], 6}, {NULL}};
const Segment SGI42413[] = {{&S46894[114], 3}, {NULL}};
const Segment SGI42414[] = {{&S46894[80], 5}, {NULL}};
const Segment SGI42415[] = {{&S52388[44], 9}, {NULL}};
const Segment SGI42416[] = {{&S52388[44], 26}, {NULL}};
const Segment SGI42417[] = {{&S52388[30], 6}, {NULL}};
const Segment SGI42418[] = {{&S52388[118], 3}, {NULL}};
const Segment SGI42419[] = {{&S52388[84], 5}, {NULL}};
const Segment SGI42420[] = {{&S46897[44], 17}, {NULL}};
const Segment SGI42421[] = {{&S46897[44], 28}, {NULL}};
const Segment SGI42422[] = {{&S46897[30], 6}, {NULL}};
const Segment SGI42423[] = {{&S46897[89], 4}, {NULL}};
const Segment SGI42424[] = {{&S46897[89], 3}, {NULL}};
const Segment SGI42425[] = {{&S46898[44], 11}, {NULL}};
const Segment SGI42426[] = {{&S46898[44], 22}, {NULL}};
const Segment SGI42427[] = {{&S46898[30], 6}, {NULL}};
const Segment SGI42428[] = {{&S46898[114], 3}, {NULL}};
const Segment SGI42429[] = {{&S46898[80], 5}, {NULL}};
const Segment SGI42430[] = {{&S46899[45], 17}, {NULL}};
const Segment SGI42431[] = {{&S46899[30], 6}, {NULL}};
const Segment SGI42432[] = {{&S46899[121], 3}, {NULL}};
const Segment SGI42433[] = {{&S46899[87], 5}, {NULL}};
const Segment SGI42434[] = {{&S46902[44], 14}, {NULL}};
const Segment SGI42435[] = {{&S46902[44], 25}, {NULL}};
const Segment SGI42436[] = {{&S46902[30], 6}, {NULL}};
const Segment SGI42437[] = {{&S46902[116], 3}, {NULL}};
const Segment SGI42438[] = {{&S46902[83], 5}, {NULL}};
const Segment SGI42439[] = {{&S46903[44], 16}, {NULL}};
const Segment SGI42440[] = {{&S46903[44], 27}, {NULL}};
const Segment SGI42441[] = {{&S46903[30], 6}, {NULL}};
const Segment SGI42442[] = {{&S46903[118], 3}, {NULL}};
const Segment SGI42443[] = {{&S46903[85], 5}, {NULL}};
const Segment SGI42444[] = {{&S46905[44], 15}, {NULL}};
const Segment SGI42445[] = {{&S46905[44], 26}, {NULL}};
const Segment SGI42446[] = {{&S46905[30], 6}, {NULL}};
const Segment SGI42447[] = {{&S46905[117], 3}, {NULL}};
const Segment SGI42448[] = {{&S46905[84], 5}, {NULL}};
const Segment SGI42449[] = {{&S44981[44], 5}, {NULL}};
const Segment SGI42450[] = {{&S44981[44], 16}, {NULL}};
const Segment SGI42451[] = {{&S44981[30], 6}, {NULL}};
const Segment SGI42452[] = {{&S44981[108], 3}, {NULL}};
const Segment SGI42453[] = {{&S44981[74], 5}, {NULL}};
const Segment SGI42454[] = {{&S44982[38], 5}, {NULL}};
const Segment SGI42455[] = {{&S44982[31], 6}, {NULL}};
const Segment SGI42456[] = {{&S47070[44], 9}, {NULL}};
const Segment SGI42457[] = {{&S47070[44], 26}, {NULL}};
const Segment SGI42458[] = {{&S47070[30], 6}, {NULL}};
const Segment SGI42459[] = {{&S47070[118], 3}, {NULL}};
const Segment SGI42460[] = {{&S47070[84], 5}, {NULL}};
const Segment SGI42461[] = {{&S47071[44], 9}, {NULL}};
const Segment SGI42462[] = {{&S47071[44], 20}, {NULL}};
const Segment SGI42463[] = {{&S47071[30], 6}, {NULL}};
const Segment SGI42464[] = {{&S47071[112], 3}, {NULL}};
const Segment SGI42465[] = {{&S47071[78], 5}, {NULL}};
const Segment SGI42466[] = {{&S45003[44], 10}, {NULL}};
const Segment SGI42467[] = {{&S45003[44], 21}, {NULL}};
const Segment SGI42468[] = {{&S45003[30], 6}, {NULL}};
const Segment SGI42469[] = {{&S45003[113], 3}, {NULL}};
const Segment SGI42470[] = {{&S45003[79], 5}, {NULL}};
const Segment SGI42471[] = {{&S45004[38], 10}, {NULL}};
const Segment SGI42472[] = {{&S45004[31], 6}, {NULL}};
const Segment SGI42473[] = {{&S52754[44], 19}, {NULL}};
const Segment SGI42474[] = {{&S52754[30], 6}, {NULL}};
const Segment SGI42475[] = {{&S52754[111], 3}, {NULL}};
const Segment SGI42476[] = {{&S52754[77], 5}, {NULL}};
const Segment SGI42477[] = {{&S44814[44], 5}, {NULL}};
const Segment SGI42478[] = {{&S44814[44], 16}, {NULL}};
const Segment SGI42479[] = {{&S44814[30], 6}, {NULL}};
const Segment SGI42480[] = {{&S44814[108], 3}, {NULL}};
const Segment SGI42481[] = {{&S44814[74], 5}, {NULL}};
const Segment SGI42482[] = {{&S44816[44], 22}, {NULL}};
const Segment SGI42483[] = {{&S44816[44], 37}, {NULL}};
const Segment SGI42484[] = {{&S44816[30], 6}, {NULL}};
const Segment SGI42485[] = {{&S44816[129], 3}, {NULL}};
const Segment SGI42486[] = {{&S44816[95], 5}, {NULL}};
const Segment SGI42487[] = {{&S44817[44], 14}, {NULL}};
const Segment SGI42488[] = {{&S44817[44], 25}, {NULL}};
const Segment SGI42489[] = {{&S44817[30], 6}, {NULL}};
const Segment SGI42490[] = {{&S44817[117], 3}, {NULL}};
const Segment SGI42491[] = {{&S44817[83], 5}, {NULL}};
const Segment SGI42492[] = {{&S58042[44], 14}, {NULL}};
const Segment SGI42493[] = {{&S58042[44], 25}, {NULL}};
const Segment SGI42494[] = {{&S58042[30], 6}, {NULL}};
const Segment SGI42495[] = {{&S58042[117], 3}, {NULL}};
const Segment SGI42496[] = {{&S58042[83], 5}, {NULL}};
const Segment SGI42497[] = {{&S52276[42], 12}, {NULL}};
const Segment SGI42498[] = {{&S52276[42], 23}, {NULL}};
const Segment SGI42499[] = {{&S52276[30], 4}, {NULL}};
const Segment SGI42500[] = {{&S52276[114], 3}, {NULL}};
const Segment SGI42501[] = {{&S52276[79], 5}, {NULL}};
const Segment SGI42502[] = {{&S58044[38], 14}, {NULL}};
const Segment SGI42503[] = {{&S58044[31], 6}, {NULL}};
const Segment SGI42504[] = {{&S47642[44], 11}, {NULL}};
const Segment SGI42505[] = {{&S47642[44], 22}, {NULL}};
const Segment SGI42506[] = {{&S47642[30], 6}, {NULL}};
const Segment SGI42507[] = {{&S47642[114], 3}, {NULL}};
const Segment SGI42508[] = {{&S47642[80], 5}, {NULL}};
const Segment SGI42509[] = {{&S47750[44], 11}, {NULL}};
const Segment SGI42510[] = {{&S47750[44], 22}, {NULL}};
const Segment SGI42511[] = {{&S47750[30], 6}, {NULL}};
const Segment SGI42512[] = {{&S47750[114], 3}, {NULL}};
const Segment SGI42513[] = {{&S47750[80], 5}, {NULL}};
const Segment SGI42514[] = {{&S47759[38], 11}, {NULL}};
const Segment SGI42515[] = {{&S47759[31], 6}, {NULL}};
const Segment SGI42516[] = {{&S52383[42], 5}, {NULL}};
const Segment SGI42517[] = {{&S52383[42], 16}, {NULL}};
const Segment SGI42518[] = {{&S52383[30], 4}, {NULL}};
const Segment SGI42519[] = {{&S52383[106], 3}, {NULL}};
const Segment SGI42520[] = {{&S52383[72], 5}, {NULL}};
const Segment SGI42521[] = {{&S47856[44], 20}, {NULL}};
const Segment SGI42522[] = {{&S47856[44], 39}, {NULL}};
const Segment SGI42523[] = {{&S47856[30], 6}, {NULL}};
const Segment SGI42524[] = {{&S47856[131], 3}, {NULL}};
const Segment SGI42525[] = {{&S47856[97], 5}, {NULL}};
const Segment SGI42526[] = {{&S47857[44], 20}, {NULL}};
const Segment SGI42527[] = {{&S47857[44], 39}, {NULL}};
const Segment SGI42528[] = {{&S47857[30], 6}, {NULL}};
const Segment SGI42529[] = {{&S47857[131], 3}, {NULL}};
const Segment SGI42530[] = {{&S47857[97], 5}, {NULL}};
const Segment SGI42531[] = {{&S47859[44], 20}, {NULL}};
const Segment SGI42532[] = {{&S47859[44], 37}, {NULL}};
const Segment SGI42533[] = {{&S47859[30], 6}, {NULL}};
const Segment SGI42534[] = {{&S47859[70], 3}, {NULL}};
const Segment SGI42535[] = {{&S47859[95], 5}, {NULL}};
const Segment SGI42536[] = {{&S53342[44], 19}, {NULL}};
const Segment SGI42537[] = {{&S53342[30], 6}, {NULL}};
const Segment SGI42538[] = {{&S53342[111], 3}, {NULL}};
const Segment SGI42539[] = {{&S53342[77], 5}, {NULL}};
const Segment SGI42540[] = {{&S45809[44], 13}, {NULL}};
const Segment SGI42541[] = {{&S45809[44], 30}, {NULL}};
const Segment SGI42542[] = {{&S45809[30], 6}, {NULL}};
const Segment SGI42543[] = {{&S45809[121], 3}, {NULL}};
const Segment SGI42544[] = {{&S45809[88], 5}, {NULL}};
const Segment SGI42545[] = {{&S45835[44], 10}, {NULL}};
const Segment SGI42546[] = {{&S45835[44], 27}, {NULL}};
const Segment SGI42547[] = {{&S45835[30], 6}, {NULL}};
const Segment SGI42548[] = {{&S45835[119], 3}, {NULL}};
const Segment SGI42549[] = {{&S45835[85], 5}, {NULL}};
const Segment SGI42550[] = {{&S45836[38], 10}, {NULL}};
const Segment SGI42551[] = {{&S45836[31], 6}, {NULL}};
const Segment SGI42552[] = {{&S45543[44], 9}, {NULL}};
const Segment SGI42553[] = {{&S45543[44], 20}, {NULL}};
const Segment SGI42554[] = {{&S45543[30], 6}, {NULL}};
const Segment SGI42555[] = {{&S45543[112], 3}, {NULL}};
const Segment SGI42556[] = {{&S45543[78], 5}, {NULL}};
const Segment SGI42557[] = {{&S45544[38], 9}, {NULL}};
const Segment SGI42558[] = {{&S45544[31], 6}, {NULL}};
const Segment SGI42559[] = {{&S48198[44], 19}, {NULL}};
const Segment SGI42560[] = {{&S48198[44], 37}, {NULL}};
const Segment SGI42561[] = {{&S48198[30], 6}, {NULL}};
const Segment SGI42562[] = {{&S48198[129], 3}, {NULL}};
const Segment SGI42563[] = {{&S48198[95], 5}, {NULL}};
const Segment SGI42564[] = {{&S48199[44], 7}, {NULL}};
const Segment SGI42565[] = {{&S48199[44], 25}, {NULL}};
const Segment SGI42566[] = {{&S48199[30], 6}, {NULL}};
const Segment SGI42567[] = {{&S48199[117], 3}, {NULL}};
const Segment SGI42568[] = {{&S48199[83], 5}, {NULL}};
const Segment SGI42569[] = {{&S48200[44], 7}, {NULL}};
const Segment SGI42570[] = {{&S48200[44], 24}, {NULL}};
const Segment SGI42571[] = {{&S48200[30], 6}, {NULL}};
const Segment SGI42572[] = {{&S48200[116], 3}, {NULL}};
const Segment SGI42573[] = {{&S48200[82], 5}, {NULL}};
const Segment SGI42574[] = {{&S48201[44], 19}, {NULL}};
const Segment SGI42575[] = {{&S48201[44], 36}, {NULL}};
const Segment SGI42576[] = {{&S48201[30], 6}, {NULL}};
const Segment SGI42577[] = {{&S48201[128], 3}, {NULL}};
const Segment SGI42578[] = {{&S48201[94], 5}, {NULL}};
const Segment SGI42579[] = {{&S48202[44], 7}, {NULL}};
const Segment SGI42580[] = {{&S48202[44], 24}, {NULL}};
const Segment SGI42581[] = {{&S48202[30], 6}, {NULL}};
const Segment SGI42582[] = {{&S48202[116], 3}, {NULL}};
const Segment SGI42583[] = {{&S48202[82], 5}, {NULL}};
const Segment SGI42584[] = {{&S48203[44], 19}, {NULL}};
const Segment SGI42585[] = {{&S48203[44], 36}, {NULL}};
const Segment SGI42586[] = {{&S48203[30], 6}, {NULL}};
const Segment SGI42587[] = {{&S48203[128], 3}, {NULL}};
const Segment SGI42588[] = {{&S48203[94], 5}, {NULL}};
const Segment SGI42589[] = {{&S48204[44], 7}, {NULL}};
const Segment SGI42590[] = {{&S48204[44], 25}, {NULL}};
const Segment SGI42591[] = {{&S48204[30], 6}, {NULL}};
const Segment SGI42592[] = {{&S48204[117], 3}, {NULL}};
const Segment SGI42593[] = {{&S48204[83], 5}, {NULL}};
const Segment SGI42594[] = {{&S48205[44], 19}, {NULL}};
const Segment SGI42595[] = {{&S48205[44], 37}, {NULL}};
const Segment SGI42596[] = {{&S48205[30], 6}, {NULL}};
const Segment SGI42597[] = {{&S48205[129], 3}, {NULL}};
const Segment SGI42598[] = {{&S48205[95], 5}, {NULL}};
const Segment SGI42599[] = {{&S48206[44], 19}, {NULL}};
const Segment SGI42600[] = {{&S48206[44], 37}, {NULL}};
const Segment SGI42601[] = {{&S48206[30], 6}, {NULL}};
const Segment SGI42602[] = {{&S48206[129], 3}, {NULL}};
const Segment SGI42603[] = {{&S48206[95], 5}, {NULL}};
const Segment SGI42604[] = {{&S48207[44], 7}, {NULL}};
const Segment SGI42605[] = {{&S48207[44], 25}, {NULL}};
const Segment SGI42606[] = {{&S48207[30], 6}, {NULL}};
const Segment SGI42607[] = {{&S48207[117], 3}, {NULL}};
const Segment SGI42608[] = {{&S48207[83], 5}, {NULL}};
const Segment SGI42609[] = {{&S48324[6], 4}, {NULL}};
const Segment SGI42610[] = {{&S48324[76], 5}, {NULL}};
const Segment SGI42611[] = {{&S46172[44], 7}, {NULL}};
const Segment SGI42612[] = {{&S46172[44], 19}, {NULL}};
const Segment SGI42613[] = {{&S46172[30], 6}, {NULL}};
const Segment SGI42614[] = {{&S46172[111], 3}, {NULL}};
const Segment SGI42615[] = {{&S46172[77], 5}, {NULL}};
const Segment SGI42616[] = {{&S46173[44], 7}, {NULL}};
const Segment SGI42617[] = {{&S46173[44], 19}, {NULL}};
const Segment SGI42618[] = {{&S46173[30], 6}, {NULL}};
const Segment SGI42619[] = {{&S46173[111], 3}, {NULL}};
const Segment SGI42620[] = {{&S46173[77], 5}, {NULL}};
const Segment SGI42621[] = {{&S46174[44], 7}, {NULL}};
const Segment SGI42622[] = {{&S46174[44], 19}, {NULL}};
const Segment SGI42623[] = {{&S46174[30], 6}, {NULL}};
const Segment SGI42624[] = {{&S46174[111], 3}, {NULL}};
const Segment SGI42625[] = {{&S46174[77], 5}, {NULL}};
const Segment SGI42626[] = {{&S46175[44], 7}, {NULL}};
const Segment SGI42627[] = {{&S46175[44], 19}, {NULL}};
const Segment SGI42628[] = {{&S46175[30], 6}, {NULL}};
const Segment SGI42629[] = {{&S46175[111], 3}, {NULL}};
const Segment SGI42630[] = {{&S46175[77], 5}, {NULL}};
const Segment SGI42631[] = {{&S46176[54], 7}, {NULL}};
const Segment SGI42632[] = {{&S46176[54], 18}, {NULL}};
const Segment SGI42633[] = {{&S46176[40], 6}, {NULL}};
const Segment SGI42634[] = {{&S46176[120], 3}, {NULL}};
const Segment SGI42635[] = {{&S46176[86], 5}, {NULL}};
const Segment SGI42636[] = {{&S46177[44], 7}, {NULL}};
const Segment SGI42637[] = {{&S46177[44], 19}, {NULL}};
const Segment SGI42638[] = {{&S46177[30], 6}, {NULL}};
const Segment SGI42639[] = {{&S46177[111], 3}, {NULL}};
const Segment SGI42640[] = {{&S46177[77], 5}, {NULL}};
const Segment SGI42641[] = {{&S46178[44], 7}, {NULL}};
const Segment SGI42642[] = {{&S46178[44], 19}, {NULL}};
const Segment SGI42643[] = {{&S46178[30], 6}, {NULL}};
const Segment SGI42644[] = {{&S46178[111], 3}, {NULL}};
const Segment SGI42645[] = {{&S46178[77], 5}, {NULL}};
const Segment SGI42646[] = {{&S46179[44], 7}, {NULL}};
const Segment SGI42647[] = {{&S46179[44], 19}, {NULL}};
const Segment SGI42648[] = {{&S46179[30], 6}, {NULL}};
const Segment SGI42649[] = {{&S46179[111], 3}, {NULL}};
const Segment SGI42650[] = {{&S46179[77], 5}, {NULL}};
const Segment SGI42651[] = {{&S46181[44], 7}, {NULL}};
const Segment SGI42652[] = {{&S46181[44], 19}, {NULL}};
const Segment SGI42653[] = {{&S46181[30], 6}, {NULL}};
const Segment SGI42654[] = {{&S46181[111], 3}, {NULL}};
const Segment SGI42655[] = {{&S46181[77], 5}, {NULL}};
const Segment SGI42656[] = {{&S46184[44], 7}, {NULL}};
const Segment SGI42657[] = {{&S46184[44], 19}, {NULL}};
const Segment SGI42658[] = {{&S46184[30], 6}, {NULL}};
const Segment SGI42659[] = {{&S46184[111], 3}, {NULL}};
const Segment SGI42660[] = {{&S46184[77], 5}, {NULL}};
const Segment SGI42661[] = {{&S46185[44], 7}, {NULL}};
const Segment SGI42662[] = {{&S46185[44], 19}, {NULL}};
const Segment SGI42663[] = {{&S46185[30], 6}, {NULL}};
const Segment SGI42664[] = {{&S46185[111], 3}, {NULL}};
const Segment SGI42665[] = {{&S46185[77], 5}, {NULL}};
const Segment SGI42666[] = {{&S46186[44], 7}, {NULL}};
const Segment SGI42667[] = {{&S46186[44], 19}, {NULL}};
const Segment SGI42668[] = {{&S46186[30], 6}, {NULL}};
const Segment SGI42669[] = {{&S46186[111], 3}, {NULL}};
const Segment SGI42670[] = {{&S46186[77], 5}, {NULL}};
const Segment SGI42671[] = {{&S53513[44], 9}, {NULL}};
const Segment SGI42672[] = {{&S53513[44], 21}, {NULL}};
const Segment SGI42673[] = {{&S53513[30], 6}, {NULL}};
const Segment SGI42674[] = {{&S53513[113], 3}, {NULL}};
const Segment SGI42675[] = {{&S53513[79], 5}, {NULL}};
const Segment SGI42676[] = {{&S46187[44], 7}, {NULL}};
const Segment SGI42677[] = {{&S46187[44], 19}, {NULL}};
const Segment SGI42678[] = {{&S46187[30], 6}, {NULL}};
const Segment SGI42679[] = {{&S46187[111], 3}, {NULL}};
const Segment SGI42680[] = {{&S53514[44], 9}, {NULL}};
const Segment SGI42681[] = {{&S46187[77], 5}, {NULL}};
const Segment SGI42682[] = {{&S53514[44], 21}, {NULL}};
const Segment SGI42683[] = {{&S53514[30], 6}, {NULL}};
const Segment SGI42684[] = {{&S53514[113], 3}, {NULL}};
const Segment SGI42685[] = {{&S53514[79], 5}, {NULL}};
const Segment SGI42686[] = {{&S45803[44], 13}, {NULL}};
const Segment SGI42687[] = {{&S45803[44], 30}, {NULL}};
const Segment SGI42688[] = {{&S45803[30], 6}, {NULL}};
const Segment SGI42689[] = {{&S45803[122], 3}, {NULL}};
const Segment SGI42690[] = {{&S45803[88], 5}, {NULL}};
const Segment SGI42691[] = {{&S45804[44], 13}, {NULL}};
const Segment SGI42692[] = {{&S45804[44], 30}, {NULL}};
const Segment SGI42693[] = {{&S45804[30], 6}, {NULL}};
const Segment SGI42694[] = {{&S45804[122], 3}, {NULL}};
const Segment SGI42695[] = {{&S45804[88], 5}, {NULL}};
const Segment SGI42696[] = {{&S45805[44], 5}, {NULL}};
const Segment SGI42697[] = {{&S45805[44], 22}, {NULL}};
const Segment SGI42698[] = {{&S45805[30], 6}, {NULL}};
const Segment SGI42699[] = {{&S45805[114], 3}, {NULL}};
const Segment SGI42700[] = {{&S45805[80], 5}, {NULL}};
const Segment SGI42701[] = {{&S45806[38], 13}, {NULL}};
const Segment SGI42702[] = {{&S45806[31], 6}, {NULL}};
const Segment SGI42703[] = {{&S45807[44], 5}, {NULL}};
const Segment SGI42704[] = {{&S45807[44], 22}, {NULL}};
const Segment SGI42705[] = {{&S45807[30], 6}, {NULL}};
const Segment SGI42706[] = {{&S45807[114], 3}, {NULL}};
const Segment SGI42707[] = {{&S45807[80], 5}, {NULL}};
const Segment SGI42708[] = {{&S45808[44], 9}, {NULL}};
const Segment SGI42709[] = {{&S45808[44], 26}, {NULL}};
const Segment SGI42710[] = {{&S45808[30], 6}, {NULL}};
const Segment SGI42711[] = {{&S45808[118], 3}, {NULL}};
const Segment SGI42712[] = {{&S45808[84], 5}, {NULL}};
const Segment SGI42713[] = {{&S48391[44], 7}, {NULL}};
const Segment SGI42714[] = {{&S48391[44], 26}, {NULL}};
const Segment SGI42715[] = {{&S48391[30], 6}, {NULL}};
const Segment SGI42716[] = {{&S48391[118], 3}, {NULL}};
const Segment SGI42717[] = {{&S48391[84], 5}, {NULL}};
const Segment SGI42718[] = {{&S45810[38], 5}, {NULL}};
const Segment SGI42719[] = {{&S45810[31], 6}, {NULL}};
const Segment SGI42720[] = {{&S48392[44], 19}, {NULL}};
const Segment SGI42721[] = {{&S48392[44], 38}, {NULL}};
const Segment SGI42722[] = {{&S48392[30], 6}, {NULL}};
const Segment SGI42723[] = {{&S48392[130], 3}, {NULL}};
const Segment SGI42724[] = {{&S48392[96], 5}, {NULL}};
const Segment SGI42725[] = {{&S48395[44], 19}, {NULL}};
const Segment SGI42726[] = {{&S48395[44], 36}, {NULL}};
const Segment SGI42727[] = {{&S48395[30], 6}, {NULL}};
const Segment SGI42728[] = {{&S48395[69], 3}, {NULL}};
const Segment SGI42729[] = {{&S48395[94], 5}, {NULL}};
const Segment SGI42730[] = {{&S48396[44], 7}, {NULL}};
const Segment SGI42731[] = {{&S48396[44], 24}, {NULL}};
const Segment SGI42732[] = {{&S48396[30], 6}, {NULL}};
const Segment SGI42733[] = {{&S48396[57], 3}, {NULL}};
const Segment SGI42734[] = {{&S48396[82], 5}, {NULL}};
const Segment SGI42735[] = {{&S48397[44], 19}, {NULL}};
const Segment SGI42736[] = {{&S48397[44], 36}, {NULL}};
const Segment SGI42737[] = {{&S48397[30], 6}, {NULL}};
const Segment SGI42738[] = {{&S48397[69], 3}, {NULL}};
const Segment SGI42739[] = {{&S48397[94], 5}, {NULL}};
const Segment SGI42740[] = {{&S46228[44], 20}, {NULL}};
const Segment SGI42741[] = {{&S46228[44], 37}, {NULL}};
const Segment SGI42742[] = {{&S46228[30], 6}, {NULL}};
const Segment SGI42743[] = {{&S46228[70], 3}, {NULL}};
const Segment SGI42744[] = {{&S46228[95], 5}, {NULL}};
const Segment SGI42745[] = {{&S46229[44], 8}, {NULL}};
const Segment SGI42746[] = {{&S46229[44], 25}, {NULL}};
const Segment SGI42747[] = {{&S46229[30], 6}, {NULL}};
const Segment SGI42748[] = {{&S46229[58], 3}, {NULL}};
const Segment SGI42749[] = {{&S46229[83], 5}, {NULL}};
const Segment SGI42750[] = {{&S48403[38], 7}, {NULL}};
const Segment SGI42751[] = {{&S48403[31], 6}, {NULL}};
const Segment SGI42752[] = {{&S46230[44], 20}, {NULL}};
const Segment SGI42753[] = {{&S46230[44], 38}, {NULL}};
const Segment SGI42754[] = {{&S46230[30], 6}, {NULL}};
const Segment SGI42755[] = {{&S46230[70], 3}, {NULL}};
const Segment SGI42756[] = {{&S46230[96], 5}, {NULL}};
const Segment SGI42757[] = {{&S46231[44], 8}, {NULL}};
const Segment SGI42758[] = {{&S46231[44], 26}, {NULL}};
const Segment SGI42759[] = {{&S46231[30], 6}, {NULL}};
const Segment SGI42760[] = {{&S46231[58], 3}, {NULL}};
const Segment SGI42761[] = {{&S46231[84], 5}, {NULL}};
const Segment SGI42762[] = {{&S46233[44], 20}, {NULL}};
const Segment SGI42763[] = {{&S46233[44], 38}, {NULL}};
const Segment SGI42764[] = {{&S46233[30], 6}, {NULL}};
const Segment SGI42765[] = {{&S46233[70], 3}, {NULL}};
const Segment SGI42766[] = {{&S46233[96], 5}, {NULL}};
const Segment SGI42767[] = {{&S46234[44], 8}, {NULL}};
const Segment SGI42768[] = {{&S46234[44], 26}, {NULL}};
const Segment SGI42769[] = {{&S46234[30], 6}, {NULL}};
const Segment SGI42770[] = {{&S46234[58], 3}, {NULL}};
const Segment SGI42771[] = {{&S46234[84], 5}, {NULL}};
const Segment SGI42772[] = {{&S46236[44], 20}, {NULL}};
const Segment SGI42773[] = {{&S46236[44], 38}, {NULL}};
const Segment SGI42774[] = {{&S46236[30], 6}, {NULL}};
const Segment SGI42775[] = {{&S46236[70], 3}, {NULL}};
const Segment SGI42776[] = {{&S46236[96], 5}, {NULL}};
const Segment SGI42777[] = {{&S46237[44], 8}, {NULL}};
const Segment SGI42778[] = {{&S46237[44], 26}, {NULL}};
const Segment SGI42779[] = {{&S46237[30], 6}, {NULL}};
const Segment SGI42780[] = {{&S46237[58], 3}, {NULL}};
const Segment SGI42781[] = {{&S46237[84], 5}, {NULL}};
const Segment SGI42782[] = {{&S46238[38], 8}, {NULL}};
const Segment SGI42783[] = {{&S46238[31], 6}, {NULL}};
const Segment SGI42784[] = {{&S58955[44], 9}, {NULL}};
const Segment SGI42785[] = {{&S58955[44], 21}, {NULL}};
const Segment SGI42786[] = {{&S46508[6], 4}, {NULL}};
const Segment SGI42787[] = {{&S58955[30], 6}, {NULL}};
const Segment SGI42788[] = {{&S46508[83], 5}, {NULL}};
const Segment SGI42789[] = {{&S58955[113], 3}, {NULL}};
const Segment SGI42790[] = {{&S58955[79], 5}, {NULL}};
const Segment SGI42791[] = {{&S58958[44], 7}, {NULL}};
const Segment SGI42792[] = {{&S58958[44], 19}, {NULL}};
const Segment SGI42793[] = {{&S58958[30], 6}, {NULL}};
const Segment SGI42794[] = {{&S58958[111], 3}, {NULL}};
const Segment SGI42795[] = {{&S58958[77], 5}, {NULL}};
const Segment SGI42796[] = {{&S58959[38], 7}, {NULL}};
const Segment SGI42797[] = {{&S58959[31], 6}, {NULL}};
const Segment SGI42798[] = {{&S46519[44], 7}, {NULL}};
const Segment SGI42799[] = {{&S46519[44], 24}, {NULL}};
const Segment SGI42800[] = {{&S46519[30], 6}, {NULL}};
const Segment SGI42801[] = {{&S46519[116], 3}, {NULL}};
const Segment SGI42802[] = {{&S46519[82], 5}, {NULL}};
const Segment SGI42803[] = {{&S46042[44], 8}, {NULL}};
const Segment SGI42804[] = {{&S46042[44], 25}, {NULL}};
const Segment SGI42805[] = {{&S46042[30], 6}, {NULL}};
const Segment SGI42806[] = {{&S46042[117], 3}, {NULL}};
const Segment SGI42807[] = {{&S46042[83], 5}, {NULL}};
const Segment SGI42808[] = {{&S46046[42], 8}, {NULL}};
const Segment SGI42809[] = {{&S46046[30], 6}, {NULL}};
const Segment SGI42810[] = {{&S46046[115], 3}, {NULL}};
const Segment SGI42811[] = {{&S46046[81], 5}, {NULL}};
const Segment SGI42812[] = {{&S46049[38], 8}, {NULL}};
const Segment SGI42813[] = {{&S46049[31], 6}, {NULL}};
const Segment SGI42814[] = {{&S46582[44], 20}, {NULL}};
const Segment SGI42815[] = {{&S46582[44], 37}, {NULL}};
const Segment SGI42816[] = {{&S46582[30], 6}, {NULL}};
const Segment SGI42817[] = {{&S46582[70], 3}, {NULL}};
const Segment SGI42818[] = {{&S46582[95], 5}, {NULL}};
const Segment SGI42819[] = {{&S46583[44], 20}, {NULL}};
const Segment SGI42820[] = {{&S46583[44], 38}, {NULL}};
const Segment SGI42821[] = {{&S46583[30], 6}, {NULL}};
const Segment SGI42822[] = {{&S46583[70], 3}, {NULL}};
const Segment SGI42823[] = {{&S59030[44], 24}, {NULL}};
const Segment SGI42824[] = {{&S46583[96], 5}, {NULL}};
const Segment SGI42825[] = {{&S59030[44], 36}, {NULL}};
const Segment SGI42826[] = {{&S59030[30], 6}, {NULL}};
const Segment SGI42827[] = {{&S59030[128], 3}, {NULL}};
const Segment SGI42828[] = {{&S59030[94], 5}, {NULL}};
const Segment SGI42829[] = {{&S46584[44], 8}, {NULL}};
const Segment SGI42830[] = {{&S46584[44], 26}, {NULL}};
const Segment SGI42831[] = {{&S46584[30], 6}, {NULL}};
const Segment SGI42832[] = {{&S46584[58], 3}, {NULL}};
const Segment SGI42833[] = {{&S46584[84], 5}, {NULL}};
const Segment SGI42834[] = {{&S59031[44], 24}, {NULL}};
const Segment SGI42835[] = {{&S59031[44], 36}, {NULL}};
const Segment SGI42836[] = {{&S59031[30], 6}, {NULL}};
const Segment SGI42837[] = {{&S59031[128], 3}, {NULL}};
const Segment SGI42838[] = {{&S59031[94], 5}, {NULL}};
const Segment SGI42839[] = {{&S46585[44], 8}, {NULL}};
const Segment SGI42840[] = {{&S46585[44], 25}, {NULL}};
const Segment SGI42841[] = {{&S46585[30], 6}, {NULL}};
const Segment SGI42842[] = {{&S46585[58], 3}, {NULL}};
const Segment SGI42843[] = {{&S59032[44], 24}, {NULL}};
const Segment SGI42844[] = {{&S46585[83], 5}, {NULL}};
const Segment SGI42845[] = {{&S59032[44], 36}, {NULL}};
const Segment SGI42846[] = {{&S59032[30], 6}, {NULL}};
const Segment SGI42847[] = {{&S59032[97], 4}, {NULL}};
const Segment SGI42848[] = {{&S59032[97], 3}, {NULL}};
const Segment SGI42849[] = {{&S59033[38], 24}, {NULL}};
const Segment SGI42850[] = {{&S59033[31], 6}, {NULL}};
const Segment SGI42851[] = {{&S46587[44], 8}, {NULL}};
const Segment SGI42852[] = {{&S46587[44], 26}, {NULL}};
const Segment SGI42853[] = {{&S46587[30], 6}, {NULL}};
const Segment SGI42854[] = {{&S46587[58], 3}, {NULL}};
const Segment SGI42855[] = {{&S46587[84], 5}, {NULL}};
const Segment SGI42856[] = {{&S46588[44], 20}, {NULL}};
const Segment SGI42857[] = {{&S46588[44], 38}, {NULL}};
const Segment SGI42858[] = {{&S46588[30], 6}, {NULL}};
const Segment SGI42859[] = {{&S46588[70], 3}, {NULL}};
const Segment SGI42860[] = {{&S46588[96], 5}, {NULL}};
const Segment SGI42861[] = {{&S46590[38], 8}, {NULL}};
const Segment SGI42862[] = {{&S46590[31], 6}, {NULL}};
const Segment SGI42863[] = {{&S53293[42], 14}, {NULL}};
const Segment SGI42864[] = {{&S53293[42], 25}, {NULL}};
const Segment SGI42865[] = {{&S53293[30], 4}, {NULL}};
const Segment SGI42866[] = {{&S53293[115], 3}, {NULL}};
const Segment SGI42867[] = {{&S53293[81], 5}, {NULL}};
const Segment SGI42868[] = {{&S48828[44], 9}, {NULL}};
const Segment SGI42869[] = {{&S48828[44], 20}, {NULL}};
const Segment SGI42870[] = {{&S48828[30], 6}, {NULL}};
const Segment SGI42871[] = {{&S48828[112], 3}, {NULL}};
const Segment SGI42872[] = {{&S48828[78], 5}, {NULL}};
const Segment SGI42873[] = {{&S48829[44], 9}, {NULL}};
const Segment SGI42874[] = {{&S48829[44], 20}, {NULL}};
const Segment SGI42875[] = {{&S48829[30], 6}, {NULL}};
const Segment SGI42876[] = {{&S48829[114], 3}, {NULL}};
const Segment SGI42877[] = {{&S48829[78], 7}, {NULL}};
const Segment SGI42878[] = {{&S48829[134], 8}, {NULL}};
const Segment SGI42879[] = {{&S48830[38], 9}, {NULL}};
const Segment SGI42880[] = {{&S48830[31], 6}, {NULL}};
const Segment SGI42881[] = {{&S48831[44], 9}, {NULL}};
const Segment SGI42882[] = {{&S48831[44], 24}, {NULL}};
const Segment SGI42883[] = {{&S48831[30], 6}, {NULL}};
const Segment SGI42884[] = {{&S48832[44], 10}, {NULL}};
const Segment SGI42885[] = {{&S48832[44], 21}, {NULL}};
const Segment SGI42886[] = {{&S48832[30], 6}, {NULL}};
const Segment SGI42887[] = {{&S48832[113], 3}, {NULL}};
const Segment SGI42888[] = {{&S48832[79], 5}, {NULL}};
const Segment SGI42889[] = {{&S46180[44], 7}, {NULL}};
const Segment SGI42890[] = {{&S46180[44], 19}, {NULL}};
const Segment SGI42891[] = {{&S46180[30], 6}, {NULL}};
const Segment SGI42892[] = {{&S46180[111], 3}, {NULL}};
const Segment SGI42893[] = {{&S46180[77], 5}, {NULL}};
const Segment SGI42894[] = {{&S46182[54], 7}, {NULL}};
const Segment SGI42895[] = {{&S46182[54], 19}, {NULL}};
const Segment SGI42896[] = {{&S46182[40], 6}, {NULL}};
const Segment SGI42897[] = {{&S46182[121], 3}, {NULL}};
const Segment SGI42898[] = {{&S46182[87], 5}, {NULL}};
const Segment SGI42899[] = {{&S46183[44], 7}, {NULL}};
const Segment SGI42900[] = {{&S46183[44], 19}, {NULL}};
const Segment SGI42901[] = {{&S46183[30], 6}, {NULL}};
const Segment SGI42902[] = {{&S46183[111], 3}, {NULL}};
const Segment SGI42903[] = {{&S46183[77], 5}, {NULL}};
const Segment SGI42904[] = {{&S46188[44], 7}, {NULL}};
const Segment SGI42905[] = {{&S46188[44], 19}, {NULL}};
const Segment SGI42906[] = {{&S46188[30], 6}, {NULL}};
const Segment SGI42907[] = {{&S46188[111], 3}, {NULL}};
const Segment SGI42908[] = {{&S46188[77], 5}, {NULL}};
const Segment SGI42909[] = {{&S46189[44], 7}, {NULL}};
const Segment SGI42910[] = {{&S46189[44], 19}, {NULL}};
const Segment SGI42911[] = {{&S46189[30], 6}, {NULL}};
const Segment SGI42912[] = {{&S46189[111], 3}, {NULL}};
const Segment SGI42913[] = {{&S46189[77], 5}, {NULL}};
const Segment SGI42914[] = {{&S46190[115], 7}, {NULL}};
const Segment SGI42915[] = {{&S46190[115], 18}, {NULL}};
const Segment SGI42916[] = {{&S46190[101], 6}, {NULL}};
const Segment SGI42917[] = {{&S46190[181], 3}, {NULL}};
const Segment SGI42918[] = {{&S46190[65], 5}, {NULL}};
const Segment SGI42919[] = {{&S53454[41], 9}, {NULL}};
const Segment SGI42920[] = {{&S53454[41], 20}, {NULL}};
const Segment SGI42921[] = {{&S53454[30], 4}, {NULL}};
const Segment SGI42922[] = {{&S53454[109], 3}, {NULL}};
const Segment SGI42923[] = {{&S46191[57], 7}, {NULL}};
const Segment SGI42924[] = {{&S46191[57], 19}, {NULL}};
const Segment SGI42925[] = {{&S53454[75], 5}, {NULL}};
const Segment SGI42926[] = {{&S46191[30], 19}, {NULL}};
const Segment SGI42927[] = {{&S46191[124], 3}, {NULL}};
const Segment SGI42928[] = {{&S46191[90], 5}, {NULL}};
const Segment SGI42929[] = {{&S53458[41], 10}, {NULL}};
const Segment SGI42930[] = {{&S53458[41], 21}, {NULL}};
const Segment SGI42931[] = {{&S53458[30], 4}, {NULL}};
const Segment SGI42932[] = {{&S53458[110], 3}, {NULL}};
const Segment SGI42933[] = {{&S53458[76], 5}, {NULL}};
const Segment SGI42934[] = {{&S46197[44], 10}, {NULL}};
const Segment SGI42935[] = {{&S46197[44], 21}, {NULL}};
const Segment SGI42936[] = {{&S46197[30], 6}, {NULL}};
const Segment SGI42937[] = {{&S46197[113], 3}, {NULL}};
const Segment SGI42938[] = {{&S46197[79], 5}, {NULL}};
const Segment SGI42939[] = {{&S59245[44], 7}, {NULL}};
const Segment SGI42940[] = {{&S59245[44], 60}, {NULL}};
const Segment SGI42941[] = {{&S59245[30], 6}, {NULL}};
const Segment SGI42942[] = {{&S59245[152], 3}, {NULL}};
const Segment SGI42943[] = {{&S59245[118], 5}, {NULL}};
const Segment SGI42944[] = {{&S59246[44], 7}, {NULL}};
const Segment SGI42945[] = {{&S59246[44], 36}, {NULL}};
const Segment SGI42946[] = {{&S59246[30], 6}, {NULL}};
const Segment SGI42947[] = {{&S59246[128], 3}, {NULL}};
const Segment SGI42948[] = {{&S59246[94], 5}, {NULL}};
const Segment SGI42949[] = {{&S54371[44], 13}, {NULL}};
const Segment SGI42950[] = {{&S54371[44], 30}, {NULL}};
const Segment SGI42951[] = {{&S54371[30], 6}, {NULL}};
const Segment SGI42952[] = {{&S54371[122], 3}, {NULL}};
const Segment SGI42953[] = {{&S54371[88], 5}, {NULL}};
const Segment SGI42954[] = {{&S46880[44], 17}, {NULL}};
const Segment SGI42955[] = {{&S46880[44], 28}, {NULL}};
const Segment SGI42956[] = {{&S46880[30], 6}, {NULL}};
const Segment SGI42957[] = {{&S46880[120], 3}, {NULL}};
const Segment SGI42958[] = {{&S46880[86], 5}, {NULL}};
const Segment SGI42959[] = {{&S54374[38], 13}, {NULL}};
const Segment SGI42960[] = {{&S54374[31], 6}, {NULL}};
const Segment SGI42961[] = {{&S46883[44], 11}, {NULL}};
const Segment SGI42962[] = {{&S46883[44], 22}, {NULL}};
const Segment SGI42963[] = {{&S46883[30], 6}, {NULL}};
const Segment SGI42964[] = {{&S46887[38], 17}, {NULL}};
const Segment SGI42965[] = {{&S46887[31], 6}, {NULL}};
const Segment SGI42966[] = {{&S46889[44], 19}, {NULL}};
const Segment SGI42967[] = {{&S46889[44], 30}, {NULL}};
const Segment SGI42968[] = {{&S46889[30], 6}, {NULL}};
const Segment SGI42969[] = {{&S46889[122], 3}, {NULL}};
const Segment SGI42970[] = {{&S46889[88], 5}, {NULL}};
const Segment SGI42971[] = {{&S46890[38], 19}, {NULL}};
const Segment SGI42972[] = {{&S46890[31], 6}, {NULL}};
const Segment SGI42973[] = {{&S46891[44], 16}, {NULL}};
const Segment SGI42974[] = {{&S46891[44], 27}, {NULL}};
const Segment SGI42975[] = {{&S46891[30], 6}, {NULL}};
const Segment SGI42976[] = {{&S46891[119], 3}, {NULL}};
const Segment SGI42977[] = {{&S46891[85], 5}, {NULL}};
const Segment SGI42978[] = {{&S46892[44], 11}, {NULL}};
const Segment SGI42979[] = {{&S46892[44], 22}, {NULL}};
const Segment SGI42980[] = {{&S46892[30], 6}, {NULL}};
const Segment SGI42981[] = {{&S46892[114], 3}, {NULL}};
const Segment SGI42982[] = {{&S46892[80], 5}, {NULL}};
const Segment SGI42983[] = {{&S46893[38], 16}, {NULL}};
const Segment SGI42984[] = {{&S46893[31], 6}, {NULL}};
const Segment SGI42985[] = {{&S46895[44], 16}, {NULL}};
const Segment SGI42986[] = {{&S46895[44], 27}, {NULL}};
const Segment SGI42987[] = {{&S46895[30], 6}, {NULL}};
const Segment SGI42988[] = {{&S46895[119], 3}, {NULL}};
const Segment SGI42989[] = {{&S46895[85], 5}, {NULL}};
const Segment SGI42990[] = {{&S46896[44], 11}, {NULL}};
const Segment SGI42991[] = {{&S46896[44], 22}, {NULL}};
const Segment SGI42992[] = {{&S46896[30], 6}, {NULL}};
const Segment SGI42993[] = {{&S46896[114], 3}, {NULL}};
const Segment SGI42994[] = {{&S46896[80], 5}, {NULL}};
const Segment SGI42995[] = {{&S46900[44], 17}, {NULL}};
const Segment SGI42996[] = {{&S46900[44], 28}, {NULL}};
const Segment SGI42997[] = {{&S46900[30], 6}, {NULL}};
const Segment SGI42998[] = {{&S46900[119], 3}, {NULL}};
const Segment SGI42999[] = {{&S46900[86], 5}, {NULL}};
const Segment SGI43000[] = {{&S46901[44], 15}, {NULL}};
const Segment SGI43001[] = {{&S46901[44], 26}, {NULL}};
const Segment SGI43002[] = {{&S46901[30], 6}, {NULL}};
const Segment SGI43003[] = {{&S46901[117], 3}, {NULL}};
const Segment SGI43004[] = {{&S46901[84], 5}, {NULL}};
const Segment SGI43005[] = {{&S46904[44], 10}, {NULL}};
const Segment SGI43006[] = {{&S46904[44], 21}, {NULL}};
const Segment SGI43007[] = {{&S46904[30], 6}, {NULL}};
const Segment SGI43008[] = {{&S46904[112], 3}, {NULL}};
const Segment SGI43009[] = {{&S46904[79], 5}, {NULL}};
const Segment SGI43010[] = {{&S47069[44], 9}, {NULL}};
const Segment SGI43011[] = {{&S47069[44], 20}, {NULL}};
const Segment SGI43012[] = {{&S47069[30], 6}, {NULL}};
const Segment SGI43013[] = {{&S47069[112], 3}, {NULL}};
const Segment SGI43014[] = {{&S47069[78], 5}, {NULL}};
const Segment SGI43015[] = {{&S47072[44], 20}, {NULL}};
const Segment SGI43016[] = {{&S47072[44], 31}, {NULL}};
const Segment SGI43017[] = {{&S47072[30], 6}, {NULL}};
const Segment SGI43018[] = {{&S47072[123], 3}, {NULL}};
const Segment SGI43019[] = {{&S47072[89], 5}, {NULL}};
const Segment SGI43020[] = {{&S47073[44], 9}, {NULL}};
const Segment SGI43021[] = {{&S47073[44], 20}, {NULL}};
const Segment SGI43022[] = {{&S47073[30], 6}, {NULL}};
const Segment SGI43023[] = {{&S47073[81], 4}, {NULL}};
const Segment SGI43024[] = {{&S47073[81], 3}, {NULL}};
const Segment SGI43025[] = {{&S47074[38], 9}, {NULL}};
const Segment SGI43026[] = {{&S47074[31], 6}, {NULL}};
const Segment SGI43027[] = {{&S47075[41], 9}, {NULL}};
const Segment SGI43028[] = {{&S47075[30], 6}, {NULL}};
const Segment SGI43029[] = {{&S47075[109], 3}, {NULL}};
const Segment SGI43030[] = {{&S47075[75], 5}, {NULL}};
const Segment SGI43031[] = {{&S47077[44], 13}, {NULL}};
const Segment SGI43032[] = {{&S47077[44], 24}, {NULL}};
const Segment SGI43033[] = {{&S47077[30], 6}, {NULL}};
const Segment SGI43034[] = {{&S47077[116], 3}, {NULL}};
const Segment SGI43035[] = {{&S47077[82], 5}, {NULL}};
const Segment SGI43036[] = {{&S47082[44], 41}, {NULL}};
const Segment SGI43037[] = {{&S47082[44], 52}, {NULL}};
const Segment SGI43038[] = {{&S47082[30], 6}, {NULL}};
const Segment SGI43039[] = {{&S47082[151], 3}, {NULL}};
const Segment SGI43040[] = {{&S47082[117], 5}, {NULL}};
const Segment SGI43041[] = {{&S47082[96], 8}, {NULL}};
const Segment SGI43042[] = {{&S47083[44], 35}, {NULL}};
const Segment SGI43043[] = {{&S47083[44], 46}, {NULL}};
const Segment SGI43044[] = {{&S47083[30], 6}, {NULL}};
const Segment SGI43045[] = {{&S47083[138], 3}, {NULL}};
const Segment SGI43046[] = {{&S47083[104], 5}, {NULL}};
const Segment SGI43047[] = {{&S47083[162], 14}, {NULL}};
const Segment SGI43048[] = {{&S47083[169], 7}, {NULL}};
const Segment SGI43049[] = {{&S47084[44], 35}, {NULL}};
const Segment SGI43050[] = {{&S47084[44], 46}, {NULL}};
const Segment SGI43051[] = {{&S47084[30], 6}, {NULL}};
const Segment SGI43052[] = {{&S47084[138], 3}, {NULL}};
const Segment SGI43053[] = {{&S47084[104], 5}, {NULL}};
const Segment SGI43054[] = {{&S47084[162], 14}, {&S47084[162], 14}, {NULL}};
const Segment SGI43055[] = {{&S47084[169], 7}, {NULL}};
const Segment SGI43056[] = {{&S46505[6], 4}, {NULL}};
const Segment SGI43057[] = {{&S46505[83], 5}, {NULL}};
const Segment SGI43058[] = {{&S46517[44], 7}, {NULL}};
const Segment SGI43059[] = {{&S46517[44], 24}, {NULL}};
const Segment SGI43060[] = {{&S46517[30], 6}, {NULL}};
const Segment SGI43061[] = {{&S46517[116], 3}, {NULL}};
const Segment SGI43062[] = {{&S46517[82], 5}, {NULL}};
const Segment SGI43063[] = {{&S46518[44], 14}, {NULL}};
const Segment SGI43064[] = {{&S46518[44], 31}, {NULL}};
const Segment SGI43065[] = {{&S46518[30], 6}, {NULL}};
const Segment SGI43066[] = {{&S46518[123], 3}, {NULL}};
const Segment SGI43067[] = {{&S46518[89], 5}, {NULL}};
const Segment SGI43068[] = {{&S46521[38], 14}, {NULL}};
const Segment SGI43069[] = {{&S46521[31], 6}, {NULL}};
const Segment SGI43070[] = {{&S49205[44], 9}, {NULL}};
const Segment SGI43071[] = {{&S49205[44], 19}, {NULL}};
const Segment SGI43072[] = {{&S49205[30], 6}, {NULL}};
const Segment SGI43073[] = {{&S49205[111], 3}, {NULL}};
const Segment SGI43074[] = {{&S49205[77], 5}, {NULL}};
const Segment SGI43075[] = {{&S49280[44], 11}, {NULL}};
const Segment SGI43076[] = {{&S49280[44], 22}, {NULL}};
const Segment SGI43077[] = {{&S49280[30], 6}, {NULL}};
const Segment SGI43078[] = {{&S49280[114], 3}, {NULL}};
const Segment SGI43079[] = {{&S49280[80], 5}, {NULL}};
const Segment SGI43080[] = {{&S59717[44], 6}, {NULL}};
const Segment SGI43081[] = {{&S59717[44], 18}, {NULL}};
const Segment SGI43082[] = {{&S59717[30], 6}, {NULL}};
const Segment SGI43083[] = {{&S59717[110], 3}, {NULL}};
const Segment SGI43084[] = {{&S59717[76], 5}, {NULL}};
const Segment SGI43085[] = {{&S59718[44], 6}, {NULL}};
const Segment SGI43086[] = {{&S59718[44], 18}, {NULL}};
const Segment SGI43087[] = {{&S59718[30], 6}, {NULL}};
const Segment SGI43088[] = {{&S59718[110], 3}, {NULL}};
const Segment SGI43089[] = {{&S59718[76], 5}, {NULL}};
const Segment SGI43090[] = {{&S59719[38], 6}, {NULL}};
const Segment SGI43091[] = {{&S59719[31], 6}, {NULL}};
const Segment SGI43092[] = {{&S47200[44], 11}, {NULL}};
const Segment SGI43093[] = {{&S47200[44], 28}, {NULL}};
const Segment SGI43094[] = {{&S47200[30], 6}, {NULL}};
const Segment SGI43095[] = {{&S47200[120], 3}, {NULL}};
const Segment SGI43096[] = {{&S47200[86], 5}, {NULL}};
const Segment SGI43097[] = {{&S47201[38], 11}, {NULL}};
const Segment SGI43098[] = {{&S47201[31], 6}, {NULL}};
const Segment SGI43099[] = {{&S47258[44], 7}, {NULL}};
const Segment SGI43100[] = {{&S47258[44], 24}, {NULL}};
const Segment SGI43101[] = {{&S47258[30], 6}, {NULL}};
const Segment SGI43102[] = {{&S47258[116], 3}, {NULL}};
const Segment SGI43103[] = {{&S47258[82], 5}, {NULL}};
const Segment SGI43104[] = {{&S47259[38], 7}, {NULL}};
const Segment SGI43105[] = {{&S47259[31], 6}, {NULL}};
const Segment SGI43106[] = {{&S47305[44], 13}, {NULL}};
const Segment SGI43107[] = {{&S47305[44], 24}, {NULL}};
const Segment SGI43108[] = {{&S47305[30], 6}, {NULL}};
const Segment SGI43109[] = {{&S47305[116], 3}, {NULL}};
const Segment SGI43110[] = {{&S47305[82], 5}, {NULL}};
const Segment SGI43111[] = {{&S59861[44], 10}, {NULL}};
const Segment SGI43112[] = {{&S59861[44], 21}, {NULL}};
const Segment SGI43113[] = {{&S59861[30], 6}, {NULL}};
const Segment SGI43114[] = {{&S59861[113], 3}, {NULL}};
const Segment SGI43115[] = {{&S59861[79], 5}, {NULL}};
const Segment SGI43116[] = {{&S59969[44], 4}, {NULL}};
const Segment SGI43117[] = {{&S59969[44], 15}, {NULL}};
const Segment SGI43118[] = {{&S59969[30], 6}, {NULL}};
const Segment SGI43119[] = {{&S59969[107], 3}, {NULL}};
const Segment SGI43120[] = {{&S59969[73], 5}, {NULL}};
const Segment SGI43121[] = {{&S47481[44], 13}, {NULL}};
const Segment SGI43122[] = {{&S47481[44], 30}, {NULL}};
const Segment SGI43123[] = {{&S47481[30], 6}, {NULL}};
const Segment SGI43124[] = {{&S47481[150], 3}, {NULL}};
const Segment SGI43125[] = {{&S47481[116], 5}, {NULL}};
const Segment SGI43126[] = {{&S47481[76], 8}, {NULL}};
const Segment SGI43127[] = {{&S49532[44], 9}, {NULL}};
const Segment SGI43128[] = {{&S49532[44], 20}, {NULL}};
const Segment SGI43129[] = {{&S49532[30], 6}, {NULL}};
const Segment SGI43130[] = {{&S49532[112], 3}, {NULL}};
const Segment SGI43131[] = {{&S49532[78], 5}, {NULL}};
const Segment SGI43132[] = {{&S49533[44], 9}, {NULL}};
const Segment SGI43133[] = {{&S49533[44], 24}, {NULL}};
const Segment SGI43134[] = {{&S49533[30], 6}, {NULL}};
const Segment SGI43135[] = {{&S49534[38], 9}, {NULL}};
const Segment SGI43136[] = {{&S49534[31], 6}, {NULL}};
const Segment SGI43137[] = {{&S54173[44], 78}, {NULL}};
const Segment SGI43138[] = {{&S54173[44], 99}, {NULL}};
const Segment SGI43139[] = {{&S54173[30], 6}, {NULL}};
const Segment SGI43140[] = {{&S54173[190], 3}, {NULL}};
const Segment SGI43141[] = {{&S54173[116], 5}, {NULL}};
const Segment SGI43142[] = {{&S47551[44], 10}, {NULL}};
const Segment SGI43143[] = {{&S47551[44], 21}, {NULL}};
const Segment SGI43144[] = {{&S47551[30], 6}, {NULL}};
const Segment SGI43145[] = {{&S47551[113], 3}, {NULL}};
const Segment SGI43146[] = {{&S47551[79], 5}, {NULL}};
const Segment SGI43147[] = {{&S47552[44], 9}, {NULL}};
const Segment SGI43148[] = {{&S47552[44], 20}, {NULL}};
const Segment SGI43149[] = {{&S47552[30], 6}, {NULL}};
const Segment SGI43150[] = {{&S47552[112], 3}, {NULL}};
const Segment SGI43151[] = {{&S47552[78], 5}, {NULL}};
const Segment SGI43152[] = {{&S47553[38], 9}, {NULL}};
const Segment SGI43153[] = {{&S47553[31], 6}, {NULL}};
const Segment SGI43154[] = {{&S47554[41], 9}, {NULL}};
const Segment SGI43155[] = {{&S47554[30], 6}, {NULL}};
const Segment SGI43156[] = {{&S47554[109], 3}, {NULL}};
const Segment SGI43157[] = {{&S47554[75], 5}, {NULL}};
const Segment SGI43158[] = {{&S47555[38], 10}, {NULL}};
const Segment SGI43159[] = {{&S47555[31], 6}, {NULL}};
const Segment SGI43160[] = {{&S60156[44], 10}, {NULL}};
const Segment SGI43161[] = {{&S60156[44], 22}, {NULL}};
const Segment SGI43162[] = {{&S60156[30], 6}, {NULL}};
const Segment SGI43163[] = {{&S60156[114], 3}, {NULL}};
const Segment SGI43164[] = {{&S60156[80], 5}, {NULL}};
const Segment SGI43165[] = {{&S60157[38], 10}, {NULL}};
const Segment SGI43166[] = {{&S60157[31], 6}, {NULL}};
const Segment SGI43167[] = {{&S47128[44], 6}, {NULL}};
const Segment SGI43168[] = {{&S47128[44], 17}, {NULL}};
const Segment SGI43169[] = {{&S47128[30], 6}, {NULL}};
const Segment SGI43170[] = {{&S47128[109], 3}, {NULL}};
const Segment SGI43171[] = {{&S47128[75], 5}, {NULL}};
const Segment SGI43172[] = {{&S47129[38], 6}, {NULL}};
const Segment SGI43173[] = {{&S47129[31], 6}, {NULL}};
const Segment SGI43174[] = {{&S49677[47], 10}, {NULL}};
const Segment SGI43175[] = {{&S49677[47], 19}, {NULL}};
const Segment SGI43176[] = {{&S49677[33], 6}, {NULL}};
const Segment SGI43177[] = {{&S49677[114], 3}, {NULL}};
const Segment SGI43178[] = {{&S49677[80], 5}, {NULL}};
const Segment SGI43179[] = {{&S49678[47], 10}, {NULL}};
const Segment SGI43180[] = {{&S49678[47], 19}, {NULL}};
const Segment SGI43181[] = {{&S49678[33], 6}, {NULL}};
const Segment SGI43182[] = {{&S49678[114], 3}, {NULL}};
const Segment SGI43183[] = {{&S49678[80], 5}, {NULL}};
const Segment SGI43184[] = {{&S49679[47], 10}, {NULL}};
const Segment SGI43185[] = {{&S49679[47], 24}, {NULL}};
const Segment SGI43186[] = {{&S49679[33], 6}, {NULL}};
const Segment SGI43187[] = {{&S49679[119], 3}, {NULL}};
const Segment SGI43188[] = {{&S49679[85], 5}, {NULL}};
const Segment SGI43189[] = {{&S49680[47], 10}, {NULL}};
const Segment SGI43190[] = {{&S49680[47], 24}, {NULL}};
const Segment SGI43191[] = {{&S49680[33], 6}, {NULL}};
const Segment SGI43192[] = {{&S49680[119], 3}, {NULL}};
const Segment SGI43193[] = {{&S49680[85], 5}, {NULL}};
const Segment SGI43194[] = {{&S49681[47], 10}, {NULL}};
const Segment SGI43195[] = {{&S49681[47], 24}, {NULL}};
const Segment SGI43196[] = {{&S49681[33], 6}, {NULL}};
const Segment SGI43197[] = {{&S49681[119], 3}, {NULL}};
const Segment SGI43198[] = {{&S49681[85], 5}, {NULL}};
const Segment SGI43199[] = {{&S49682[44], 10}, {NULL}};
const Segment SGI43200[] = {{&S49682[44], 27}, {NULL}};
const Segment SGI43201[] = {{&S49682[30], 6}, {NULL}};
const Segment SGI43202[] = {{&S49682[120], 3}, {NULL}};
const Segment SGI43203[] = {{&S49682[85], 6}, {NULL}};
const Segment SGI43204[] = {{&S49683[47], 10}, {NULL}};
const Segment SGI43205[] = {{&S49683[47], 24}, {NULL}};
const Segment SGI43206[] = {{&S49683[33], 6}, {NULL}};
const Segment SGI43207[] = {{&S49683[119], 3}, {NULL}};
const Segment SGI43208[] = {{&S49683[85], 5}, {NULL}};
const Segment SGI43209[] = {{&S49684[47], 10}, {NULL}};
const Segment SGI43210[] = {{&S49684[47], 25}, {NULL}};
const Segment SGI43211[] = {{&S49684[33], 6}, {NULL}};
const Segment SGI43212[] = {{&S49684[120], 3}, {NULL}};
const Segment SGI43213[] = {{&S49684[86], 5}, {NULL}};
const Segment SGI43214[] = {{&S49685[47], 10}, {NULL}};
const Segment SGI43215[] = {{&S49685[47], 25}, {NULL}};
const Segment SGI43216[] = {{&S49685[33], 6}, {NULL}};
const Segment SGI43217[] = {{&S49685[120], 3}, {NULL}};
const Segment SGI43218[] = {{&S49685[86], 5}, {NULL}};
const Segment SGI43219[] = {{&S49686[47], 10}, {NULL}};
const Segment SGI43220[] = {{&S49686[47], 26}, {NULL}};
const Segment SGI43221[] = {{&S49686[33], 6}, {NULL}};
const Segment SGI43222[] = {{&S49686[121], 3}, {NULL}};
const Segment SGI43223[] = {{&S49686[87], 5}, {NULL}};
const Segment SGI43224[] = {{&S49687[47], 10}, {NULL}};
const Segment SGI43225[] = {{&S49687[47], 26}, {NULL}};
const Segment SGI43226[] = {{&S49687[33], 6}, {NULL}};
const Segment SGI43227[] = {{&S49687[121], 3}, {NULL}};
const Segment SGI43228[] = {{&S49687[87], 5}, {NULL}};
const Segment SGI43229[] = {{&S49688[47], 10}, {NULL}};
const Segment SGI43230[] = {{&S49688[47], 26}, {NULL}};
const Segment SGI43231[] = {{&S49688[33], 6}, {NULL}};
const Segment SGI43232[] = {{&S49688[121], 3}, {NULL}};
const Segment SGI43233[] = {{&S49688[87], 5}, {NULL}};
const Segment SGI43234[] = {{&S49689[47], 10}, {NULL}};
const Segment SGI43235[] = {{&S49689[47], 26}, {NULL}};
const Segment SGI43236[] = {{&S49689[33], 6}, {NULL}};
const Segment SGI43237[] = {{&S49689[121], 3}, {NULL}};
const Segment SGI43238[] = {{&S49689[87], 5}, {NULL}};
const Segment SGI43239[] = {{&S49690[47], 10}, {NULL}};
const Segment SGI43240[] = {{&S49690[47], 26}, {NULL}};
const Segment SGI43241[] = {{&S49690[33], 6}, {NULL}};
const Segment SGI43242[] = {{&S49690[121], 3}, {NULL}};
const Segment SGI43243[] = {{&S49690[87], 5}, {NULL}};
const Segment SGI43244[] = {{&S49691[47], 10}, {NULL}};
const Segment SGI43245[] = {{&S49691[47], 25}, {NULL}};
const Segment SGI43246[] = {{&S49691[33], 6}, {NULL}};
const Segment SGI43247[] = {{&S49691[120], 3}, {NULL}};
const Segment SGI43248[] = {{&S49691[86], 5}, {NULL}};
const Segment SGI43249[] = {{&S49692[47], 10}, {NULL}};
const Segment SGI43250[] = {{&S49692[47], 26}, {NULL}};
const Segment SGI43251[] = {{&S49692[33], 6}, {NULL}};
const Segment SGI43252[] = {{&S49692[121], 3}, {NULL}};
const Segment SGI43253[] = {{&S49692[87], 5}, {NULL}};
const Segment SGI43254[] = {{&S49693[47], 10}, {NULL}};
const Segment SGI43255[] = {{&S49693[47], 25}, {NULL}};
const Segment SGI43256[] = {{&S49693[33], 6}, {NULL}};
const Segment SGI43257[] = {{&S49693[120], 3}, {NULL}};
const Segment SGI43258[] = {{&S49693[86], 5}, {NULL}};
const Segment SGI43259[] = {{&S54372[42], 13}, {NULL}};
const Segment SGI43260[] = {{&S54372[42], 30}, {NULL}};
const Segment SGI43261[] = {{&S54372[30], 4}, {NULL}};
const Segment SGI43262[] = {{&S54372[120], 3}, {NULL}};
const Segment SGI43263[] = {{&S54372[86], 5}, {NULL}};
const Segment SGI43264[] = {{&S54373[36], 13}, {NULL}};
const Segment SGI43265[] = {{&S54373[31], 4}, {NULL}};
const Segment SGI43266[] = {{&S47751[44], 11}, {NULL}};
const Segment SGI43267[] = {{&S47751[44], 22}, {NULL}};
const Segment SGI43268[] = {{&S47751[30], 6}, {NULL}};
const Segment SGI43269[] = {{&S47751[114], 3}, {NULL}};
const Segment SGI43270[] = {{&S47751[80], 5}, {NULL}};
const Segment SGI43271[] = {{&S47754[44], 11}, {NULL}};
const Segment SGI43272[] = {{&S47754[44], 22}, {NULL}};
const Segment SGI43273[] = {{&S47754[30], 6}, {NULL}};
const Segment SGI43274[] = {{&S47754[114], 3}, {NULL}};
const Segment SGI43275[] = {{&S47754[80], 5}, {NULL}};
const Segment SGI43276[] = {{&S47760[38], 11}, {NULL}};
const Segment SGI43277[] = {{&S47760[31], 6}, {NULL}};
const Segment SGI43278[] = {{&S47840[44], 10}, {NULL}};
const Segment SGI43279[] = {{&S47840[44], 21}, {NULL}};
const Segment SGI43280[] = {{&S47840[30], 6}, {NULL}};
const Segment SGI43281[] = {{&S47840[113], 3}, {NULL}};
const Segment SGI43282[] = {{&S47840[79], 5}, {NULL}};
const Segment SGI43283[] = {{&S47841[38], 10}, {NULL}};
const Segment SGI43284[] = {{&S47841[31], 6}, {NULL}};
const Segment SGI43285[] = {{&S47860[44], 20}, {NULL}};
const Segment SGI43286[] = {{&S47860[44], 38}, {NULL}};
const Segment SGI43287[] = {{&S47860[30], 6}, {NULL}};
const Segment SGI43288[] = {{&S47860[70], 3}, {NULL}};
const Segment SGI43289[] = {{&S47860[96], 5}, {NULL}};
const Segment SGI43290[] = {{&S47861[44], 20}, {NULL}};
const Segment SGI43291[] = {{&S47861[44], 38}, {NULL}};
const Segment SGI43292[] = {{&S47861[30], 6}, {NULL}};
const Segment SGI43293[] = {{&S47861[70], 3}, {NULL}};
const Segment SGI43294[] = {{&S47861[96], 5}, {NULL}};
const Segment SGI43295[] = {{&S47862[44], 20}, {NULL}};
const Segment SGI43296[] = {{&S47862[44], 38}, {NULL}};
const Segment SGI43297[] = {{&S47862[30], 6}, {NULL}};
const Segment SGI43298[] = {{&S47862[70], 3}, {NULL}};
const Segment SGI43299[] = {{&S47862[96], 5}, {NULL}};
const Segment SGI43300[] = {{&S49862[44], 12}, {NULL}};
const Segment SGI43301[] = {{&S49862[44], 22}, {NULL}};
const Segment SGI43302[] = {{&S49862[30], 6}, {NULL}};
const Segment SGI43303[] = {{&S49862[114], 3}, {NULL}};
const Segment SGI43304[] = {{&S49862[80], 5}, {NULL}};
const Segment SGI43305[] = {{&S49863[44], 12}, {NULL}};
const Segment SGI43306[] = {{&S49863[44], 51}, {NULL}};
const Segment SGI43307[] = {{&S49863[30], 6}, {NULL}};
const Segment SGI43308[] = {{&S49863[143], 3}, {NULL}};
const Segment SGI43309[] = {{&S49863[109], 5}, {NULL}};
const Segment SGI43310[] = {{&S47632[44], 4}, {NULL}};
const Segment SGI43311[] = {{&S47632[44], 18}, {NULL}};
const Segment SGI43312[] = {{&S47632[30], 6}, {NULL}};
const Segment SGI43313[] = {{&S47632[110], 3}, {NULL}};
const Segment SGI43314[] = {{&S47632[76], 5}, {NULL}};
const Segment SGI43315[] = {{&S47633[44], 4}, {NULL}};
const Segment SGI43316[] = {{&S47633[44], 15}, {NULL}};
const Segment SGI43317[] = {{&S47633[30], 6}, {NULL}};
const Segment SGI43318[] = {{&S47633[107], 3}, {NULL}};
const Segment SGI43319[] = {{&S47633[73], 5}, {NULL}};
const Segment SGI43320[] = {{&S47643[38], 4}, {NULL}};
const Segment SGI43321[] = {{&S47643[31], 6}, {NULL}};
const Segment SGI43322[] = {{&S47644[44], 4}, {NULL}};
const Segment SGI43323[] = {{&S47644[44], 15}, {NULL}};
const Segment SGI43324[] = {{&S47644[30], 6}, {NULL}};
const Segment SGI43325[] = {{&S47644[107], 3}, {NULL}};
const Segment SGI43326[] = {{&S47644[73], 5}, {NULL}};
const Segment SGI43327[] = {{&S47645[44], 11}, {NULL}};
const Segment SGI43328[] = {{&S47645[44], 19}, {NULL}};
const Segment SGI43329[] = {{&S47645[30], 6}, {NULL}};
const Segment SGI43330[] = {{&S47645[111], 3}, {NULL}};
const Segment SGI43331[] = {{&S47645[77], 5}, {NULL}};
const Segment SGI43332[] = {{&S47647[44], 11}, {NULL}};
const Segment SGI43333[] = {{&S47647[44], 19}, {NULL}};
const Segment SGI43334[] = {{&S47647[30], 6}, {NULL}};
const Segment SGI43335[] = {{&S47647[111], 3}, {NULL}};
const Segment SGI43336[] = {{&S47647[77], 5}, {NULL}};
const Segment SGI43337[] = {{&S47655[44], 11}, {NULL}};
const Segment SGI43338[] = {{&S47655[44], 24}, {NULL}};
const Segment SGI43339[] = {{&S47655[30], 6}, {NULL}};
const Segment SGI43340[] = {{&S47655[116], 3}, {NULL}};
const Segment SGI43341[] = {{&S47655[82], 5}, {NULL}};
const Segment SGI43342[] = {{&S50231[44], 12}, {NULL}};
const Segment SGI43343[] = {{&S50231[44], 23}, {NULL}};
const Segment SGI43344[] = {{&S50231[30], 6}, {NULL}};
const Segment SGI43345[] = {{&S50231[115], 3}, {NULL}};
const Segment SGI43346[] = {{&S50231[81], 5}, {NULL}};
const Segment SGI43347[] = {{&S50232[44], 12}, {NULL}};
const Segment SGI43348[] = {{&S50232[44], 23}, {NULL}};
const Segment SGI43349[] = {{&S50232[30], 6}, {NULL}};
const Segment SGI43350[] = {{&S50232[84], 4}, {NULL}};
const Segment SGI43351[] = {{&S50232[84], 3}, {NULL}};
const Segment SGI43352[] = {{&S50233[38], 12}, {NULL}};
const Segment SGI43353[] = {{&S50233[31], 6}, {NULL}};
const Segment SGI43354[] = {{&S50237[44], 12}, {NULL}};
const Segment SGI43355[] = {{&S50237[44], 23}, {NULL}};
const Segment SGI43356[] = {{&S50237[30], 6}, {NULL}};
const Segment SGI43357[] = {{&S50237[84], 6}, {NULL}};
const Segment SGI43358[] = {{&S48176[44], 6}, {NULL}};
const Segment SGI43359[] = {{&S48176[44], 23}, {NULL}};
const Segment SGI43360[] = {{&S48176[30], 6}, {NULL}};
const Segment SGI43361[] = {{&S48176[115], 3}, {NULL}};
const Segment SGI43362[] = {{&S48176[81], 5}, {NULL}};
const Segment SGI43363[] = {{&S55797[44], 25}, {NULL}};
const Segment SGI43364[] = {{&S55797[44], 42}, {NULL}};
const Segment SGI43365[] = {{&S55797[30], 6}, {NULL}};
const Segment SGI43366[] = {{&S55797[134], 3}, {NULL}};
const Segment SGI43367[] = {{&S55797[100], 5}, {NULL}};
const Segment SGI43368[] = {{&S55798[44], 25}, {NULL}};
const Segment SGI43369[] = {{&S55798[44], 40}, {NULL}};
const Segment SGI43370[] = {{&S55798[30], 6}, {NULL}};
const Segment SGI43371[] = {{&S55798[132], 3}, {NULL}};
const Segment SGI43372[] = {{&S55798[98], 5}, {NULL}};
const Segment SGI43373[] = {{&S55799[44], 25}, {NULL}};
const Segment SGI43374[] = {{&S55799[44], 37}, {NULL}};
const Segment SGI43375[] = {{&S55799[30], 6}, {NULL}};
const Segment SGI43376[] = {{&S55799[129], 3}, {NULL}};
const Segment SGI43377[] = {{&S55799[95], 5}, {NULL}};
const Segment SGI43378[] = {{&S55800[38], 25}, {NULL}};
const Segment SGI43379[] = {{&S55800[31], 6}, {NULL}};
const Segment SGI43380[] = {{&S48208[44], 19}, {NULL}};
const Segment SGI43381[] = {{&S48208[44], 38}, {NULL}};
const Segment SGI43382[] = {{&S48208[30], 6}, {NULL}};
const Segment SGI43383[] = {{&S48208[69], 3}, {NULL}};
const Segment SGI43384[] = {{&S48208[96], 5}, {NULL}};
const Segment SGI43385[] = {{&S48209[44], 7}, {NULL}};
const Segment SGI43386[] = {{&S48209[44], 26}, {NULL}};
const Segment SGI43387[] = {{&S48209[30], 6}, {NULL}};
const Segment SGI43388[] = {{&S48209[57], 3}, {NULL}};
const Segment SGI43389[] = {{&S48209[84], 5}, {NULL}};
const Segment SGI43390[] = {{&S48210[44], 7}, {NULL}};
const Segment SGI43391[] = {{&S48210[44], 26}, {NULL}};
const Segment SGI43392[] = {{&S48210[30], 6}, {NULL}};
const Segment SGI43393[] = {{&S48210[57], 3}, {NULL}};
const Segment SGI43394[] = {{&S48210[84], 5}, {NULL}};
const Segment SGI43395[] = {{&S48211[44], 19}, {NULL}};
const Segment SGI43396[] = {{&S48211[44], 38}, {NULL}};
const Segment SGI43397[] = {{&S48211[30], 6}, {NULL}};
const Segment SGI43398[] = {{&S48211[69], 3}, {NULL}};
const Segment SGI43399[] = {{&S48211[96], 5}, {NULL}};
const Segment SGI43400[] = {{&S48212[44], 19}, {NULL}};
const Segment SGI43401[] = {{&S48212[44], 38}, {NULL}};
const Segment SGI43402[] = {{&S48212[30], 6}, {NULL}};
const Segment SGI43403[] = {{&S48212[69], 3}, {NULL}};
const Segment SGI43404[] = {{&S48212[96], 5}, {NULL}};
const Segment SGI43405[] = {{&S48213[44], 7}, {NULL}};
const Segment SGI43406[] = {{&S48213[44], 26}, {NULL}};
const Segment SGI43407[] = {{&S48213[30], 6}, {NULL}};
const Segment SGI43408[] = {{&S48213[57], 3}, {NULL}};
const Segment SGI43409[] = {{&S48213[84], 5}, {NULL}};
const Segment SGI43410[] = {{&S48214[38], 7}, {NULL}};
const Segment SGI43411[] = {{&S48214[31], 6}, {NULL}};
const Segment SGI43412[] = {{&S50454[44], 12}, {NULL}};
const Segment SGI43413[] = {{&S50454[44], 23}, {NULL}};
const Segment SGI43414[] = {{&S50454[30], 6}, {NULL}};
const Segment SGI43415[] = {{&S50454[115], 3}, {NULL}};
const Segment SGI43416[] = {{&S50454[81], 5}, {NULL}};
const Segment SGI43417[] = {{&S50455[38], 12}, {NULL}};
const Segment SGI43418[] = {{&S50455[31], 6}, {NULL}};
const Segment SGI43419[] = {{&S47752[44], 11}, {NULL}};
const Segment SGI43420[] = {{&S47752[44], 22}, {NULL}};
const Segment SGI43421[] = {{&S47752[30], 6}, {NULL}};
const Segment SGI43422[] = {{&S47752[114], 3}, {NULL}};
const Segment SGI43423[] = {{&S47752[80], 5}, {NULL}};
const Segment SGI43424[] = {{&S47761[38], 11}, {NULL}};
const Segment SGI43425[] = {{&S47761[31], 6}, {NULL}};
const Segment SGI43426[] = {{&S47762[46], 12}, {NULL}};
const Segment SGI43427[] = {{&S47762[46], 23}, {NULL}};
const Segment SGI43428[] = {{&S47762[30], 8}, {NULL}};
const Segment SGI43429[] = {{&S47762[117], 3}, {NULL}};
const Segment SGI43430[] = {{&S47762[83], 5}, {NULL}};
const Segment SGI43431[] = {{&S50493[44], 10}, {NULL}};
const Segment SGI43432[] = {{&S50493[44], 21}, {NULL}};
const Segment SGI43433[] = {{&S50493[30], 6}, {NULL}};
const Segment SGI43434[] = {{&S50493[113], 3}, {NULL}};
const Segment SGI43435[] = {{&S50493[79], 5}, {NULL}};
const Segment SGI43436[] = {{&S50497[44], 9}, {NULL}};
const Segment SGI43437[] = {{&S50497[44], 20}, {NULL}};
const Segment SGI43438[] = {{&S50497[30], 6}, {NULL}};
const Segment SGI43439[] = {{&S50497[112], 3}, {NULL}};
const Segment SGI43440[] = {{&S50497[78], 5}, {NULL}};
const Segment SGI43441[] = {{&S50498[38], 9}, {NULL}};
const Segment SGI43442[] = {{&S50498[31], 6}, {NULL}};
const Segment SGI43443[] = {{&S50499[44], 9}, {NULL}};
const Segment SGI43444[] = {{&S50499[44], 24}, {NULL}};
const Segment SGI43445[] = {{&S50499[30], 6}, {NULL}};
const Segment SGI43446[] = {{&S48323[6], 4}, {NULL}};
const Segment SGI43447[] = {{&S48323[76], 5}, {NULL}};
const Segment SGI43448[] = {{&S55953[44], 10}, {NULL}};
const Segment SGI43449[] = {{&S55953[44], 27}, {NULL}};
const Segment SGI43450[] = {{&S55953[30], 6}, {NULL}};
const Segment SGI43451[] = {{&S55953[119], 3}, {NULL}};
const Segment SGI43452[] = {{&S55953[85], 5}, {NULL}};
const Segment SGI43453[] = {{&S55954[44], 12}, {NULL}};
const Segment SGI43454[] = {{&S55954[44], 29}, {NULL}};
const Segment SGI43455[] = {{&S55954[30], 6}, {NULL}};
const Segment SGI43456[] = {{&S55954[121], 3}, {NULL}};
const Segment SGI43457[] = {{&S55954[87], 5}, {NULL}};
const Segment SGI43458[] = {{&S48398[44], 19}, {NULL}};
const Segment SGI43459[] = {{&S48398[44], 37}, {NULL}};
const Segment SGI43460[] = {{&S48398[30], 6}, {NULL}};
const Segment SGI43461[] = {{&S48398[69], 3}, {NULL}};
const Segment SGI43462[] = {{&S48398[95], 5}, {NULL}};
const Segment SGI43463[] = {{&S48399[44], 7}, {NULL}};
const Segment SGI43464[] = {{&S48399[44], 25}, {NULL}};
const Segment SGI43465[] = {{&S48399[30], 6}, {NULL}};
const Segment SGI43466[] = {{&S48399[57], 3}, {NULL}};
const Segment SGI43467[] = {{&S48399[83], 5}, {NULL}};
const Segment SGI43468[] = {{&S48400[44], 19}, {NULL}};
const Segment SGI43469[] = {{&S48400[44], 37}, {NULL}};
const Segment SGI43470[] = {{&S48400[30], 6}, {NULL}};
const Segment SGI43471[] = {{&S48400[69], 3}, {NULL}};
const Segment SGI43472[] = {{&S48400[95], 5}, {NULL}};
const Segment SGI43473[] = {{&S48401[44], 7}, {NULL}};
const Segment SGI43474[] = {{&S48401[44], 25}, {NULL}};
const Segment SGI43475[] = {{&S48401[30], 6}, {NULL}};
const Segment SGI43476[] = {{&S48401[57], 3}, {NULL}};
const Segment SGI43477[] = {{&S48401[83], 5}, {NULL}};
const Segment SGI43478[] = {{&S48402[38], 7}, {NULL}};
const Segment SGI43479[] = {{&S48402[31], 6}, {NULL}};
const Segment SGI43480[] = {{&S61074[44], 9}, {NULL}};
const Segment SGI43481[] = {{&S61074[44], 21}, {NULL}};
const Segment SGI43482[] = {{&S61074[30], 6}, {NULL}};
const Segment SGI43483[] = {{&S61074[113], 3}, {NULL}};
const Segment SGI43484[] = {{&S61074[79], 5}, {NULL}};
const Segment SGI43485[] = {{&S61076[38], 9}, {NULL}};
const Segment SGI43486[] = {{&S61076[31], 6}, {NULL}};
const Segment SGI43487[] = {{&S48504[44], 7}, {NULL}};
const Segment SGI43488[] = {{&S48504[44], 18}, {NULL}};
const Segment SGI43489[] = {{&S48504[30], 6}, {NULL}};
const Segment SGI43490[] = {{&S48504[110], 3}, {NULL}};
const Segment SGI43491[] = {{&S48504[76], 5}, {NULL}};
const Segment SGI43492[] = {{&S48505[54], 7}, {NULL}};
const Segment SGI43493[] = {{&S48505[54], 19}, {NULL}};
const Segment SGI43494[] = {{&S48505[40], 6}, {NULL}};
const Segment SGI43495[] = {{&S48505[121], 3}, {NULL}};
const Segment SGI43496[] = {{&S48505[87], 5}, {NULL}};
const Segment SGI43497[] = {{&S48506[54], 7}, {NULL}};
const Segment SGI43498[] = {{&S48506[54], 19}, {NULL}};
const Segment SGI43499[] = {{&S48506[40], 6}, {NULL}};
const Segment SGI43500[] = {{&S48506[121], 3}, {NULL}};
const Segment SGI43501[] = {{&S48506[87], 5}, {NULL}};
const Segment SGI43502[] = {{&S48507[54], 7}, {NULL}};
const Segment SGI43503[] = {{&S48507[54], 19}, {NULL}};
const Segment SGI43504[] = {{&S48507[40], 6}, {NULL}};
const Segment SGI43505[] = {{&S48507[121], 3}, {NULL}};
const Segment SGI43506[] = {{&S48507[87], 5}, {NULL}};
const Segment SGI43507[] = {{&S48508[54], 7}, {NULL}};
const Segment SGI43508[] = {{&S48508[54], 19}, {NULL}};
const Segment SGI43509[] = {{&S48508[40], 6}, {NULL}};
const Segment SGI43510[] = {{&S48508[121], 3}, {NULL}};
const Segment SGI43511[] = {{&S48508[87], 5}, {NULL}};
const Segment SGI43512[] = {{&S48510[54], 7}, {NULL}};
const Segment SGI43513[] = {{&S48510[54], 19}, {NULL}};
const Segment SGI43514[] = {{&S48510[40], 6}, {NULL}};
const Segment SGI43515[] = {{&S48510[121], 3}, {NULL}};
const Segment SGI43516[] = {{&S48510[87], 5}, {NULL}};
const Segment SGI43517[] = {{&S48511[54], 7}, {NULL}};
const Segment SGI43518[] = {{&S48511[54], 19}, {NULL}};
const Segment SGI43519[] = {{&S48511[40], 6}, {NULL}};
const Segment SGI43520[] = {{&S48511[121], 3}, {NULL}};
const Segment SGI43521[] = {{&S48511[87], 5}, {NULL}};
const Segment SGI43522[] = {{&S48514[44], 7}, {NULL}};
const Segment SGI43523[] = {{&S48514[44], 19}, {NULL}};
const Segment SGI43524[] = {{&S48514[30], 6}, {NULL}};
const Segment SGI43525[] = {{&S48514[111], 3}, {NULL}};
const Segment SGI43526[] = {{&S48514[77], 5}, {NULL}};
const Segment SGI43527[] = {{&S48516[54], 7}, {NULL}};
const Segment SGI43528[] = {{&S48516[54], 19}, {NULL}};
const Segment SGI43529[] = {{&S48516[40], 6}, {NULL}};
const Segment SGI43530[] = {{&S48516[121], 3}, {NULL}};
const Segment SGI43531[] = {{&S48516[87], 5}, {NULL}};
const Segment SGI43532[] = {{&S50779[44], 8}, {NULL}};
const Segment SGI43533[] = {{&S50779[44], 19}, {NULL}};
const Segment SGI43534[] = {{&S50779[30], 6}, {NULL}};
const Segment SGI43535[] = {{&S50779[111], 3}, {NULL}};
const Segment SGI43536[] = {{&S50779[77], 5}, {NULL}};
const Segment SGI43537[] = {{&S50780[44], 8}, {NULL}};
const Segment SGI43538[] = {{&S50780[44], 19}, {NULL}};
const Segment SGI43539[] = {{&S50780[30], 6}, {NULL}};
const Segment SGI43540[] = {{&S50780[111], 3}, {NULL}};
const Segment SGI43541[] = {{&S50780[77], 5}, {NULL}};
const Segment SGI43542[] = {{&S50781[44], 8}, {NULL}};
const Segment SGI43543[] = {{&S50781[44], 19}, {NULL}};
const Segment SGI43544[] = {{&S50781[30], 6}, {NULL}};
const Segment SGI43545[] = {{&S50781[111], 3}, {NULL}};
const Segment SGI43546[] = {{&S50781[77], 5}, {NULL}};
const Segment SGI43547[] = {{&S50782[44], 8}, {NULL}};
const Segment SGI43548[] = {{&S50782[44], 19}, {NULL}};
const Segment SGI43549[] = {{&S50782[30], 6}, {NULL}};
const Segment SGI43550[] = {{&S50782[111], 3}, {NULL}};
const Segment SGI43551[] = {{&S50782[77], 5}, {NULL}};
const Segment SGI43552[] = {{&S50783[44], 8}, {NULL}};
const Segment SGI43553[] = {{&S50783[44], 19}, {NULL}};
const Segment SGI43554[] = {{&S50783[30], 6}, {NULL}};
const Segment SGI43555[] = {{&S50783[111], 3}, {NULL}};
const Segment SGI43556[] = {{&S50783[77], 5}, {NULL}};
const Segment SGI43557[] = {{&S48242[44], 24}, {NULL}};
const Segment SGI43558[] = {{&S48242[44], 35}, {NULL}};
const Segment SGI43559[] = {{&S48242[30], 6}, {NULL}};
const Segment SGI43560[] = {{&S50786[38], 8}, {NULL}};
const Segment SGI43561[] = {{&S50786[31], 6}, {NULL}};
const Segment SGI43562[] = {{&S48242[127], 3}, {NULL}};
const Segment SGI43563[] = {{&S48242[93], 5}, {NULL}};
const Segment SGI43564[] = {{&S48244[44], 24}, {NULL}};
const Segment SGI43565[] = {{&S48244[44], 35}, {NULL}};
const Segment SGI43566[] = {{&S48244[30], 6}, {NULL}};
const Segment SGI43567[] = {{&S48244[127], 3}, {NULL}};
const Segment SGI43568[] = {{&S50789[44], 8}, {NULL}};
const Segment SGI43569[] = {{&S48244[93], 5}, {NULL}};
const Segment SGI43570[] = {{&S50789[44], 19}, {NULL}};
const Segment SGI43571[] = {{&S50789[30], 6}, {NULL}};
const Segment SGI43572[] = {{&S50789[111], 3}, {NULL}};
const Segment SGI43573[] = {{&S50789[77], 5}, {NULL}};
const Segment SGI43574[] = {{&S50790[44], 8}, {NULL}};
const Segment SGI43575[] = {{&S50790[44], 19}, {NULL}};
const Segment SGI43576[] = {{&S50790[30], 6}, {NULL}};
const Segment SGI43577[] = {{&S48325[6], 4}, {NULL}};
const Segment SGI43578[] = {{&S48325[76], 5}, {NULL}};
const Segment SGI43579[] = {{&S48896[44], 14}, {NULL}};
const Segment SGI43580[] = {{&S48896[44], 25}, {NULL}};
const Segment SGI43581[] = {{&S48896[30], 6}, {NULL}};
const Segment SGI43582[] = {{&S48896[117], 3}, {NULL}};
const Segment SGI43583[] = {{&S48896[83], 5}, {NULL}};
const Segment SGI43584[] = {{&S48897[44], 9}, {NULL}};
const Segment SGI43585[] = {{&S48897[44], 20}, {NULL}};
const Segment SGI43586[] = {{&S48897[30], 6}, {NULL}};
const Segment SGI43587[] = {{&S48897[112], 3}, {NULL}};
const Segment SGI43588[] = {{&S48897[78], 5}, {NULL}};
const Segment SGI43589[] = {{&S48898[44], 10}, {NULL}};
const Segment SGI43590[] = {{&S48898[44], 21}, {NULL}};
const Segment SGI43591[] = {{&S48898[30], 6}, {NULL}};
const Segment SGI43592[] = {{&S48898[113], 3}, {NULL}};
const Segment SGI43593[] = {{&S48898[79], 5}, {NULL}};
const Segment SGI43594[] = {{&S48899[38], 9}, {NULL}};
const Segment SGI43595[] = {{&S48899[31], 6}, {NULL}};
const Segment SGI43596[] = {{&S48900[44], 11}, {NULL}};
const Segment SGI43597[] = {{&S48900[44], 22}, {NULL}};
const Segment SGI43598[] = {{&S48900[30], 6}, {NULL}};
const Segment SGI43599[] = {{&S48900[114], 3}, {NULL}};
const Segment SGI43600[] = {{&S48900[80], 5}, {NULL}};
const Segment SGI43601[] = {{&S48901[44], 9}, {NULL}};
const Segment SGI43602[] = {{&S48901[44], 20}, {NULL}};
const Segment SGI43603[] = {{&S48901[30], 6}, {NULL}};
const Segment SGI43604[] = {{&S48901[78], 5}, {NULL}};
const Segment SGI43605[] = {{&S61636[44], 10}, {NULL}};
const Segment SGI43606[] = {{&S61636[44], 21}, {NULL}};
const Segment SGI43607[] = {{&S61636[30], 6}, {NULL}};
const Segment SGI43608[] = {{&S61636[113], 3}, {NULL}};
const Segment SGI43609[] = {{&S61636[79], 5}, {NULL}};
const Segment SGI43610[] = {{&S61639[44], 10}, {NULL}};
const Segment SGI43611[] = {{&S61639[44], 21}, {NULL}};
const Segment SGI43612[] = {{&S61639[30], 6}, {NULL}};
const Segment SGI43613[] = {{&S61639[82], 4}, {NULL}};
const Segment SGI43614[] = {{&S61639[82], 3}, {NULL}};
const Segment SGI43615[] = {{&S61644[44], 10}, {NULL}};
const Segment SGI43616[] = {{&S61644[44], 21}, {NULL}};
const Segment SGI43617[] = {{&S61644[30], 6}, {NULL}};
const Segment SGI43618[] = {{&S61644[82], 6}, {NULL}};
const Segment SGI43619[] = {{&S61646[38], 10}, {NULL}};
const Segment SGI43620[] = {{&S61646[31], 6}, {NULL}};
const Segment SGI43621[] = {{&S52472[44], 7}, {NULL}};
const Segment SGI43622[] = {{&S52472[44], 19}, {NULL}};
const Segment SGI43623[] = {{&S52472[30], 6}, {NULL}};
const Segment SGI43624[] = {{&S52472[111], 3}, {NULL}};
const Segment SGI43625[] = {{&S52472[77], 5}, {NULL}};
const Segment SGI43626[] = {{&S52473[44], 7}, {NULL}};
const Segment SGI43627[] = {{&S52473[44], 24}, {NULL}};
const Segment SGI43628[] = {{&S52473[30], 6}, {NULL}};
const Segment SGI43629[] = {{&S52473[116], 3}, {NULL}};
const Segment SGI43630[] = {{&S52473[82], 5}, {NULL}};
const Segment SGI43631[] = {{&S48509[54], 7}, {NULL}};
const Segment SGI43632[] = {{&S48509[54], 19}, {NULL}};
const Segment SGI43633[] = {{&S48509[40], 6}, {NULL}};
const Segment SGI43634[] = {{&S48509[121], 3}, {NULL}};
const Segment SGI43635[] = {{&S48509[87], 5}, {NULL}};
const Segment SGI43636[] = {{&S49039[44], 7}, {NULL}};
const Segment SGI43637[] = {{&S49039[44], 33}, {NULL}};
const Segment SGI43638[] = {{&S49039[30], 6}, {NULL}};
const Segment SGI43639[] = {{&S49039[125], 3}, {NULL}};
const Segment SGI43640[] = {{&S49039[91], 5}, {NULL}};
const Segment SGI43641[] = {{&S48512[44], 7}, {NULL}};
const Segment SGI43642[] = {{&S48512[44], 19}, {NULL}};
const Segment SGI43643[] = {{&S48512[30], 6}, {NULL}};
const Segment SGI43644[] = {{&S48512[111], 3}, {NULL}};
const Segment SGI43645[] = {{&S48512[77], 5}, {NULL}};
const Segment SGI43646[] = {{&S48513[44], 7}, {NULL}};
const Segment SGI43647[] = {{&S48513[44], 19}, {NULL}};
const Segment SGI43648[] = {{&S48513[30], 6}, {NULL}};
const Segment SGI43649[] = {{&S48513[111], 3}, {NULL}};
const Segment SGI43650[] = {{&S48513[77], 5}, {NULL}};
const Segment SGI43651[] = {{&S48515[54], 7}, {NULL}};
const Segment SGI43652[] = {{&S48515[54], 19}, {NULL}};
const Segment SGI43653[] = {{&S48515[40], 6}, {NULL}};
const Segment SGI43654[] = {{&S48515[121], 3}, {NULL}};
const Segment SGI43655[] = {{&S48515[87], 5}, {NULL}};
const Segment SGI43656[] = {{&S51111[44], 10}, {NULL}};
const Segment SGI43657[] = {{&S51111[44], 27}, {NULL}};
const Segment SGI43658[] = {{&S51111[30], 6}, {NULL}};
const Segment SGI43659[] = {{&S51111[119], 3}, {NULL}};
const Segment SGI43660[] = {{&S51111[85], 5}, {NULL}};
const Segment SGI43661[] = {{&S51112[38], 10}, {NULL}};
const Segment SGI43662[] = {{&S51112[31], 6}, {NULL}};
const Segment SGI43663[] = {{&S49200[44], 9}, {NULL}};
const Segment SGI43664[] = {{&S49200[44], 19}, {NULL}};
const Segment SGI43665[] = {{&S49200[30], 6}, {NULL}};
const Segment SGI43666[] = {{&S49200[111], 3}, {NULL}};
const Segment SGI43667[] = {{&S49200[77], 5}, {NULL}};
const Segment SGI43668[] = {{&S49210[38], 9}, {NULL}};
const Segment SGI43669[] = {{&S49210[31], 6}, {NULL}};
const Segment SGI43670[] = {{&S49211[44], 9}, {NULL}};
const Segment SGI43671[] = {{&S49211[44], 33}, {NULL}};
const Segment SGI43672[] = {{&S49211[30], 6}, {NULL}};
const Segment SGI43673[] = {{&S49211[125], 3}, {NULL}};
const Segment SGI43674[] = {{&S49211[91], 5}, {NULL}};
const Segment SGI43675[] = {{&S49214[44], 9}, {NULL}};
const Segment SGI43676[] = {{&S49214[44], 19}, {NULL}};
const Segment SGI43677[] = {{&S49214[30], 6}, {NULL}};
const Segment SGI43678[] = {{&S49214[80], 4}, {NULL}};
const Segment SGI43679[] = {{&S49214[80], 3}, {NULL}};
const Segment SGI43680[] = {{&S49279[44], 11}, {NULL}};
const Segment SGI43681[] = {{&S49279[44], 22}, {NULL}};
const Segment SGI43682[] = {{&S49279[30], 6}, {NULL}};
const Segment SGI43683[] = {{&S49279[114], 3}, {NULL}};
const Segment SGI43684[] = {{&S49279[80], 5}, {NULL}};
const Segment SGI43685[] = {{&S51203[44], 9}, {NULL}};
const Segment SGI43686[] = {{&S51203[44], 24}, {NULL}};
const Segment SGI43687[] = {{&S51203[30], 6}, {NULL}};
const Segment SGI43688[] = {{&S51204[44], 9}, {NULL}};
const Segment SGI43689[] = {{&S51204[44], 20}, {NULL}};
const Segment SGI43690[] = {{&S51204[30], 6}, {NULL}};
const Segment SGI43691[] = {{&S51204[112], 3}, {NULL}};
const Segment SGI43692[] = {{&S51204[78], 5}, {NULL}};
const Segment SGI43693[] = {{&S51205[38], 9}, {NULL}};
const Segment SGI43694[] = {{&S51205[31], 6}, {NULL}};
const Segment SGI43695[] = {{&S61950[44], 7}, {NULL}};
const Segment SGI43696[] = {{&S61950[44], 24}, {NULL}};
const Segment SGI43697[] = {{&S61950[30], 6}, {NULL}};
const Segment SGI43698[] = {{&S61950[116], 3}, {NULL}};
const Segment SGI43699[] = {{&S61950[82], 5}, {NULL}};
const Segment SGI43700[] = {{&S61951[44], 7}, {NULL}};
const Segment SGI43701[] = {{&S61951[44], 24}, {NULL}};
const Segment SGI43702[] = {{&S61951[30], 6}, {NULL}};
const Segment SGI43703[] = {{&S61951[116], 3}, {NULL}};
const Segment SGI43704[] = {{&S61951[82], 5}, {NULL}};
const Segment SGI43705[] = {{&S61952[38], 7}, {NULL}};
const Segment SGI43706[] = {{&S61952[31], 6}, {NULL}};
const Segment SGI43707[] = {{&S49360[44], 24}, {NULL}};
const Segment SGI43708[] = {{&S49360[44], 41}, {NULL}};
const Segment SGI43709[] = {{&S49360[30], 6}, {NULL}};
const Segment SGI43710[] = {{&S49360[133], 3}, {NULL}};
const Segment SGI43711[] = {{&S49360[99], 5}, {NULL}};
const Segment SGI43712[] = {{&S49361[44], 24}, {NULL}};
const Segment SGI43713[] = {{&S49361[44], 41}, {NULL}};
const Segment SGI43714[] = {{&S49361[30], 6}, {NULL}};
const Segment SGI43715[] = {{&S49361[133], 3}, {NULL}};
const Segment SGI43716[] = {{&S49361[99], 5}, {NULL}};
const Segment SGI43717[] = {{&S49362[44], 24}, {NULL}};
const Segment SGI43718[] = {{&S49362[44], 41}, {NULL}};
const Segment SGI43719[] = {{&S49362[30], 6}, {NULL}};
const Segment SGI43720[] = {{&S49362[133], 3}, {NULL}};
const Segment SGI43721[] = {{&S49362[99], 5}, {NULL}};
const Segment SGI43722[] = {{&S62359[44], 6}, {NULL}};
const Segment SGI43723[] = {{&S62359[44], 18}, {NULL}};
const Segment SGI43724[] = {{&S62359[30], 6}, {NULL}};
const Segment SGI43725[] = {{&S62359[110], 3}, {NULL}};
const Segment SGI43726[] = {{&S62359[76], 5}, {NULL}};
const Segment SGI43727[] = {{&S48837[44], 7}, {NULL}};
const Segment SGI43728[] = {{&S48837[44], 20}, {NULL}};
const Segment SGI43729[] = {{&S48837[30], 6}, {NULL}};
const Segment SGI43730[] = {{&S48837[112], 3}, {NULL}};
const Segment SGI43731[] = {{&S48837[78], 5}, {NULL}};
const Segment SGI43732[] = {{&S48838[44], 7}, {NULL}};
const Segment SGI43733[] = {{&S48838[44], 20}, {NULL}};
const Segment SGI43734[] = {{&S48838[30], 6}, {NULL}};
const Segment SGI43735[] = {{&S48838[112], 3}, {NULL}};
const Segment SGI43736[] = {{&S48838[78], 5}, {NULL}};
const Segment SGI43737[] = {{&S48839[44], 7}, {NULL}};
const Segment SGI43738[] = {{&S48839[44], 18}, {NULL}};
const Segment SGI43739[] = {{&S48839[30], 6}, {NULL}};
const Segment SGI43740[] = {{&S48839[110], 3}, {NULL}};
const Segment SGI43741[] = {{&S48839[76], 5}, {NULL}};
const Segment SGI43742[] = {{&S48840[44], 7}, {NULL}};
const Segment SGI43743[] = {{&S48840[44], 20}, {NULL}};
const Segment SGI43744[] = {{&S48840[30], 6}, {NULL}};
const Segment SGI43745[] = {{&S48840[112], 3}, {NULL}};
const Segment SGI43746[] = {{&S48840[78], 5}, {NULL}};
const Segment SGI43747[] = {{&S48841[44], 7}, {NULL}};
const Segment SGI43748[] = {{&S48841[44], 20}, {NULL}};
const Segment SGI43749[] = {{&S48841[30], 6}, {NULL}};
const Segment SGI43750[] = {{&S48841[112], 3}, {NULL}};
const Segment SGI43751[] = {{&S48841[78], 5}, {NULL}};
const Segment SGI43752[] = {{&S48842[44], 7}, {NULL}};
const Segment SGI43753[] = {{&S48842[44], 20}, {NULL}};
const Segment SGI43754[] = {{&S48842[30], 6}, {NULL}};
const Segment SGI43755[] = {{&S48842[112], 3}, {NULL}};
const Segment SGI43756[] = {{&S48842[78], 5}, {NULL}};
const Segment SGI43757[] = {{&S48843[38], 7}, {NULL}};
const Segment SGI43758[] = {{&S48843[31], 6}, {NULL}};
const Segment SGI43759[] = {{&S48844[44], 7}, {NULL}};
const Segment SGI43760[] = {{&S48844[44], 20}, {NULL}};
const Segment SGI43761[] = {{&S48844[30], 6}, {NULL}};
const Segment SGI43762[] = {{&S48844[112], 3}, {NULL}};
const Segment SGI43763[] = {{&S48844[78], 5}, {NULL}};
const Segment SGI43764[] = {{&S48845[44], 7}, {NULL}};
const Segment SGI43765[] = {{&S48845[44], 20}, {NULL}};
const Segment SGI43766[] = {{&S48845[30], 6}, {NULL}};
const Segment SGI43767[] = {{&S48845[112], 3}, {NULL}};
const Segment SGI43768[] = {{&S48845[78], 5}, {NULL}};
const Segment SGI43769[] = {{&S57027[6], 4}, {NULL}};
const Segment SGI43770[] = {{&S57027[83], 5}, {NULL}};
const Segment SGI43771[] = {{&S49040[44], 7}, {NULL}};
const Segment SGI43772[] = {{&S49040[44], 32}, {NULL}};
const Segment SGI43773[] = {{&S49040[30], 6}, {NULL}};
const Segment SGI43774[] = {{&S49040[124], 3}, {NULL}};
const Segment SGI43775[] = {{&S49040[90], 5}, {NULL}};
const Segment SGI43776[] = {{&S49041[44], 7}, {NULL}};
const Segment SGI43777[] = {{&S49041[44], 35}, {NULL}};
const Segment SGI43778[] = {{&S49041[30], 6}, {NULL}};
const Segment SGI43779[] = {{&S49041[127], 3}, {NULL}};
const Segment SGI43780[] = {{&S49041[93], 5}, {NULL}};
const Segment SGI43781[] = {{&S49043[44], 7}, {NULL}};
const Segment SGI43782[] = {{&S49043[44], 35}, {NULL}};
const Segment SGI43783[] = {{&S49043[30], 6}, {NULL}};
const Segment SGI43784[] = {{&S49043[127], 3}, {NULL}};
const Segment SGI43785[] = {{&S49043[93], 5}, {NULL}};
const Segment SGI43786[] = {{&S49044[44], 7}, {NULL}};
const Segment SGI43787[] = {{&S49044[44], 35}, {NULL}};
const Segment SGI43788[] = {{&S49044[30], 6}, {NULL}};
const Segment SGI43789[] = {{&S49044[127], 3}, {NULL}};
const Segment SGI43790[] = {{&S49044[93], 5}, {NULL}};
const Segment SGI43791[] = {{&S49856[44], 12}, {NULL}};
const Segment SGI43792[] = {{&S49856[44], 22}, {NULL}};
const Segment SGI43793[] = {{&S49856[30], 6}, {NULL}};
const Segment SGI43794[] = {{&S49045[44], 7}, {NULL}};
const Segment SGI43795[] = {{&S49045[44], 33}, {NULL}};
const Segment SGI43796[] = {{&S49045[30], 6}, {NULL}};
const Segment SGI43797[] = {{&S49856[114], 3}, {NULL}};
const Segment SGI43798[] = {{&S49856[80], 5}, {NULL}};
const Segment SGI43799[] = {{&S49045[125], 3}, {NULL}};
const Segment SGI43800[] = {{&S49045[91], 5}, {NULL}};
const Segment SGI43801[] = {{&S49046[44], 7}, {NULL}};
const Segment SGI43802[] = {{&S49046[44], 38}, {NULL}};
const Segment SGI43803[] = {{&S49046[30], 6}, {NULL}};
const Segment SGI43804[] = {{&S49046[130], 3}, {NULL}};
const Segment SGI43805[] = {{&S49858[44], 12}, {NULL}};
const Segment SGI43806[] = {{&S49858[44], 36}, {NULL}};
const Segment SGI43807[] = {{&S49858[30], 6}, {NULL}};
const Segment SGI43808[] = {{&S49046[96], 5}, {NULL}};
const Segment SGI43809[] = {{&S49858[128], 3}, {NULL}};
const Segment SGI43810[] = {{&S49858[94], 5}, {NULL}};
const Segment SGI43811[] = {{&S49050[44], 7}, {NULL}};
const Segment SGI43812[] = {{&S49050[44], 35}, {NULL}};
const Segment SGI43813[] = {{&S49050[30], 6}, {NULL}};
const Segment SGI43814[] = {{&S49050[127], 3}, {NULL}};
const Segment SGI43815[] = {{&S49050[93], 5}, {NULL}};
const Segment SGI43816[] = {{&S49051[44], 7}, {NULL}};
const Segment SGI43817[] = {{&S49051[44], 35}, {NULL}};
const Segment SGI43818[] = {{&S49051[30], 6}, {NULL}};
const Segment SGI43819[] = {{&S49051[127], 3}, {NULL}};
const Segment SGI43820[] = {{&S49051[93], 5}, {NULL}};
const Segment SGI43821[] = {{&S49053[44], 7}, {NULL}};
const Segment SGI43822[] = {{&S49053[44], 35}, {NULL}};
const Segment SGI43823[] = {{&S49053[30], 6}, {NULL}};
const Segment SGI43824[] = {{&S49053[127], 3}, {NULL}};
const Segment SGI43825[] = {{&S49053[93], 5}, {NULL}};
const Segment SGI43826[] = {{&S49868[44], 17}, {NULL}};
const Segment SGI43827[] = {{&S49868[44], 27}, {NULL}};
const Segment SGI43828[] = {{&S49868[30], 6}, {NULL}};
const Segment SGI43829[] = {{&S49868[119], 3}, {NULL}};
const Segment SGI43830[] = {{&S49868[85], 5}, {NULL}};
const Segment SGI43831[] = {{&S49054[44], 7}, {NULL}};
const Segment SGI43832[] = {{&S49054[44], 35}, {NULL}};
const Segment SGI43833[] = {{&S49054[30], 6}, {NULL}};
const Segment SGI43834[] = {{&S49054[127], 3}, {NULL}};
const Segment SGI43835[] = {{&S49054[93], 5}, {NULL}};
const Segment SGI43836[] = {{&S49869[44], 12}, {NULL}};
const Segment SGI43837[] = {{&S49869[44], 51}, {NULL}};
const Segment SGI43838[] = {{&S49869[30], 6}, {NULL}};
const Segment SGI43839[] = {{&S49869[143], 3}, {NULL}};
const Segment SGI43840[] = {{&S49869[109], 5}, {NULL}};
const Segment SGI43841[] = {{&S49892[58], 10}, {NULL}};
const Segment SGI43842[] = {{&S49892[58], 21}, {NULL}};
const Segment SGI43843[] = {{&S49892[44], 6}, {NULL}};
const Segment SGI43844[] = {{&S49892[127], 3}, {NULL}};
const Segment SGI43845[] = {{&S49892[93], 5}, {NULL}};
const Segment SGI43846[] = {{&S49893[41], 10}, {NULL}};
const Segment SGI43847[] = {{&S49893[30], 6}, {NULL}};
const Segment SGI43848[] = {{&S49893[110], 3}, {NULL}};
const Segment SGI43849[] = {{&S49893[76], 5}, {NULL}};
const Segment SGI43850[] = {{&S49894[38], 10}, {NULL}};
const Segment SGI43851[] = {{&S49894[31], 6}, {NULL}};
const Segment SGI43852[] = {{&S51961[44], 10}, {NULL}};
const Segment SGI43853[] = {{&S51961[44], 27}, {NULL}};
const Segment SGI43854[] = {{&S51961[30], 6}, {NULL}};
const Segment SGI43855[] = {{&S51961[119], 3}, {NULL}};
const Segment SGI43856[] = {{&S51961[85], 5}, {NULL}};
const Segment SGI43857[] = {{&S51965[38], 10}, {NULL}};
const Segment SGI43858[] = {{&S51965[31], 6}, {NULL}};
const Segment SGI43859[] = {{&S51966[42], 10}, {NULL}};
const Segment SGI43860[] = {{&S51966[30], 6}, {NULL}};
const Segment SGI43861[] = {{&S51966[117], 3}, {NULL}};
const Segment SGI43862[] = {{&S51966[83], 5}, {NULL}};
const Segment SGI43863[] = {{&S62708[44], 8}, {NULL}};
const Segment SGI43864[] = {{&S62708[44], 19}, {NULL}};
const Segment SGI43865[] = {{&S62708[30], 6}, {NULL}};
const Segment SGI43866[] = {{&S62708[111], 3}, {NULL}};
const Segment SGI43867[] = {{&S62708[77], 5}, {NULL}};
const Segment SGI43868[] = {{&S62709[44], 8}, {NULL}};
const Segment SGI43869[] = {{&S62709[44], 19}, {NULL}};
const Segment SGI43870[] = {{&S62709[30], 6}, {NULL}};
const Segment SGI43871[] = {{&S62709[111], 3}, {NULL}};
const Segment SGI43872[] = {{&S62709[77], 5}, {NULL}};
const Segment SGI43873[] = {{&S62710[44], 8}, {NULL}};
const Segment SGI43874[] = {{&S62710[44], 19}, {NULL}};
const Segment SGI43875[] = {{&S62710[30], 6}, {NULL}};
const Segment SGI43876[] = {{&S62710[111], 3}, {NULL}};
const Segment SGI43877[] = {{&S62710[77], 5}, {NULL}};
const Segment SGI43878[] = {{&S62711[38], 8}, {NULL}};
const Segment SGI43879[] = {{&S62711[31], 6}, {NULL}};
const Segment SGI43880[] = {{&S62712[44], 8}, {NULL}};
const Segment SGI43881[] = {{&S62712[44], 19}, {NULL}};
const Segment SGI43882[] = {{&S62712[30], 6}, {NULL}};
const Segment SGI43883[] = {{&S62712[111], 3}, {NULL}};
const Segment SGI43884[] = {{&S62712[77], 5}, {NULL}};
const Segment SGI43885[] = {{&S49201[44], 9}, {NULL}};
const Segment SGI43886[] = {{&S49201[44], 19}, {NULL}};
const Segment SGI43887[] = {{&S49201[30], 6}, {NULL}};
const Segment SGI43888[] = {{&S49201[111], 3}, {NULL}};
const Segment SGI43889[] = {{&S49201[77], 5}, {NULL}};
const Segment SGI43890[] = {{&S49203[44], 5}, {NULL}};
const Segment SGI43891[] = {{&S49203[44], 15}, {NULL}};
const Segment SGI43892[] = {{&S49203[30], 6}, {NULL}};
const Segment SGI43893[] = {{&S49203[107], 3}, {NULL}};
const Segment SGI43894[] = {{&S49203[73], 5}, {NULL}};
const Segment SGI43895[] = {{&S49207[38], 9}, {NULL}};
const Segment SGI43896[] = {{&S49207[31], 6}, {NULL}};
const Segment SGI43897[] = {{&S49215[44], 9}, {NULL}};
const Segment SGI43898[] = {{&S49215[44], 19}, {NULL}};
const Segment SGI43899[] = {{&S49215[30], 6}, {NULL}};
const Segment SGI43900[] = {{&S49215[80], 6}, {NULL}};
const Segment SGI43901[] = {{&S52133[44], 7}, {NULL}};
const Segment SGI43902[] = {{&S52133[44], 18}, {NULL}};
const Segment SGI43903[] = {{&S52133[30], 6}, {NULL}};
const Segment SGI43904[] = {{&S52133[110], 3}, {NULL}};
const Segment SGI43905[] = {{&S52133[76], 5}, {NULL}};
const Segment SGI43906[] = {{&S52140[38], 7}, {NULL}};
const Segment SGI43907[] = {{&S52140[31], 6}, {NULL}};
const Segment SGI43908[] = {{&S49281[44], 11}, {NULL}};
const Segment SGI43909[] = {{&S49281[44], 22}, {NULL}};
const Segment SGI43910[] = {{&S49281[30], 6}, {NULL}};
const Segment SGI43911[] = {{&S49281[114], 3}, {NULL}};
const Segment SGI43912[] = {{&S49281[80], 5}, {NULL}};
const Segment SGI43913[] = {{&S49363[44], 29}, {NULL}};
const Segment SGI43914[] = {{&S49363[44], 46}, {NULL}};
const Segment SGI43915[] = {{&S49363[30], 6}, {NULL}};
const Segment SGI43916[] = {{&S49363[138], 3}, {NULL}};
const Segment SGI43917[] = {{&S49363[104], 5}, {NULL}};
const Segment SGI43918[] = {{&S63004[44], 31}, {NULL}};
const Segment SGI43919[] = {{&S63004[44], 43}, {NULL}};
const Segment SGI43920[] = {{&S63004[30], 6}, {NULL}};
const Segment SGI43921[] = {{&S63004[135], 3}, {NULL}};
const Segment SGI43922[] = {{&S63004[101], 5}, {NULL}};
const Segment SGI43923[] = {{&S63005[44], 31}, {NULL}};
const Segment SGI43924[] = {{&S63005[44], 43}, {NULL}};
const Segment SGI43925[] = {{&S63005[30], 6}, {NULL}};
const Segment SGI43926[] = {{&S63005[135], 3}, {NULL}};
const Segment SGI43927[] = {{&S63005[101], 5}, {NULL}};
const Segment SGI43928[] = {{&S63006[44], 31}, {NULL}};
const Segment SGI43929[] = {{&S63006[44], 43}, {NULL}};
const Segment SGI43930[] = {{&S63006[30], 6}, {NULL}};
const Segment SGI43931[] = {{&S63006[135], 3}, {NULL}};
const Segment SGI43932[] = {{&S63006[101], 5}, {NULL}};
const Segment SGI43933[] = {{&S63007[44], 24}, {NULL}};
const Segment SGI43934[] = {{&S63007[44], 36}, {NULL}};
const Segment SGI43935[] = {{&S63007[30], 6}, {NULL}};
const Segment SGI43936[] = {{&S57651[6], 4}, {NULL}};
const Segment SGI43937[] = {{&S57651[85], 5}, {NULL}};
const Segment SGI43938[] = {{&S63007[128], 3}, {NULL}};
const Segment SGI43939[] = {{&S63007[94], 5}, {NULL}};
const Segment SGI43940[] = {{&S63008[44], 24}, {NULL}};
const Segment SGI43941[] = {{&S63008[44], 36}, {NULL}};
const Segment SGI43942[] = {{&S63008[30], 6}, {NULL}};
const Segment SGI43943[] = {{&S63008[128], 3}, {NULL}};
const Segment SGI43944[] = {{&S63008[94], 5}, {NULL}};
const Segment SGI43945[] = {{&S52228[34], 10}, {NULL}};
const Segment SGI43946[] = {{&S52228[27], 6}, {NULL}};
const Segment SGI43947[] = {{&S52228[109], 2}, {NULL}};
const Segment SGI43948[] = {{&S52228[75], 6}, {NULL}};
const Segment SGI43949[] = {{&S52228[143], 8}, {NULL}};
const Segment SGI43950[] = {{&S52234[34], 11}, {NULL}};
const Segment SGI43951[] = {{&S52234[27], 6}, {NULL}};
const Segment SGI43952[] = {{&S52234[110], 2}, {NULL}};
const Segment SGI43953[] = {{&S52234[76], 6}, {NULL}};
const Segment SGI43954[] = {{&S52234[144], 8}, {NULL}};
const Segment SGI43955[] = {{&S52235[34], 11}, {NULL}};
const Segment SGI43956[] = {{&S52235[27], 6}, {NULL}};
const Segment SGI43957[] = {{&S52235[110], 2}, {NULL}};
const Segment SGI43958[] = {{&S52235[76], 6}, {NULL}};
const Segment SGI43959[] = {{&S52235[144], 8}, {NULL}};
const Segment SGI43960[] = {{&S57154[42], 9}, {NULL}};
const Segment SGI43961[] = {{&S57154[42], 26}, {NULL}};
const Segment SGI43962[] = {{&S57154[30], 4}, {NULL}};
const Segment SGI43963[] = {{&S57154[116], 3}, {NULL}};
const Segment SGI43964[] = {{&S57154[82], 5}, {NULL}};
const Segment SGI43965[] = {{&S57155[42], 13}, {NULL}};
const Segment SGI43966[] = {{&S57155[42], 30}, {NULL}};
const Segment SGI43967[] = {{&S57155[30], 4}, {NULL}};
const Segment SGI43968[] = {{&S57155[120], 3}, {NULL}};
const Segment SGI43969[] = {{&S57155[86], 5}, {NULL}};
const Segment SGI43970[] = {{&S57156[36], 9}, {NULL}};
const Segment SGI43971[] = {{&S57156[31], 4}, {NULL}};
const Segment SGI43972[] = {{&S57157[36], 13}, {NULL}};
const Segment SGI43973[] = {{&S57157[31], 4}, {NULL}};
const Segment SGI43974[] = {{&S50405[44], 10}, {NULL}};
const Segment SGI43975[] = {{&S50405[44], 21}, {NULL}};
const Segment SGI43976[] = {{&S50405[30], 6}, {NULL}};
const Segment SGI43977[] = {{&S50405[113], 3}, {NULL}};
const Segment SGI43978[] = {{&S50405[79], 5}, {NULL}};
const Segment SGI43979[] = {{&S50406[38], 10}, {NULL}};
const Segment SGI43980[] = {{&S50406[31], 6}, {NULL}};
const Segment SGI43981[] = {{&S57709[6], 4}, {NULL}};
const Segment SGI43982[] = {{&S57709[78], 5}, {NULL}};
const Segment SGI43983[] = {{&S52320[44], 11}, {NULL}};
const Segment SGI43984[] = {{&S52320[44], 28}, {NULL}};
const Segment SGI43985[] = {{&S52320[30], 6}, {NULL}};
const Segment SGI43986[] = {{&S52320[120], 3}, {NULL}};
const Segment SGI43987[] = {{&S52320[86], 5}, {NULL}};
const Segment SGI43988[] = {{&S52323[44], 11}, {NULL}};
const Segment SGI43989[] = {{&S52323[44], 28}, {NULL}};
const Segment SGI43990[] = {{&S52323[30], 6}, {NULL}};
const Segment SGI43991[] = {{&S52323[120], 3}, {NULL}};
const Segment SGI43992[] = {{&S52323[86], 5}, {NULL}};
const Segment SGI43993[] = {{&S52326[38], 11}, {NULL}};
const Segment SGI43994[] = {{&S52326[31], 6}, {NULL}};
const Segment SGI43995[] = {{&S50492[44], 10}, {NULL}};
const Segment SGI43996[] = {{&S50492[44], 27}, {NULL}};
const Segment SGI43997[] = {{&S50492[30], 6}, {NULL}};
const Segment SGI43998[] = {{&S50492[119], 3}, {NULL}};
const Segment SGI43999[] = {{&S50492[85], 5}, {NULL}};
const Segment SGI44000[] = {{&S52378[44], 5}, {NULL}};
const Segment SGI44001[] = {{&S52378[44], 16}, {NULL}};
const Segment SGI44002[] = {{&S52378[30], 6}, {NULL}};
const Segment SGI44003[] = {{&S52378[108], 3}, {NULL}};
const Segment SGI44004[] = {{&S52378[74], 5}, {NULL}};
const Segment SGI44005[] = {{&S52380[38], 5}, {NULL}};
const Segment SGI44006[] = {{&S52380[31], 6}, {NULL}};
const Segment SGI44007[] = {{&S52382[44], 5}, {NULL}};
const Segment SGI44008[] = {{&S52382[44], 16}, {NULL}};
const Segment SGI44009[] = {{&S52382[30], 6}, {NULL}};
const Segment SGI44010[] = {{&S52382[108], 3}, {NULL}};
const Segment SGI44011[] = {{&S52382[74], 5}, {NULL}};
const Segment SGI44012[] = {{&S52466[44], 7}, {NULL}};
const Segment SGI44013[] = {{&S52466[44], 19}, {NULL}};
const Segment SGI44014[] = {{&S52466[30], 6}, {NULL}};
const Segment SGI44015[] = {{&S52466[111], 3}, {NULL}};
const Segment SGI44016[] = {{&S52466[77], 5}, {NULL}};
const Segment SGI44017[] = {{&S52474[44], 7}, {NULL}};
const Segment SGI44018[] = {{&S52474[44], 19}, {NULL}};
const Segment SGI44019[] = {{&S52474[30], 6}, {NULL}};
const Segment SGI44020[] = {{&S52474[111], 3}, {NULL}};
const Segment SGI44021[] = {{&S52474[77], 5}, {NULL}};
const Segment SGI44022[] = {{&S52477[44], 7}, {NULL}};
const Segment SGI44023[] = {{&S52477[44], 18}, {NULL}};
const Segment SGI44024[] = {{&S52477[30], 6}, {NULL}};
const Segment SGI44025[] = {{&S52477[110], 3}, {NULL}};
const Segment SGI44026[] = {{&S52477[76], 5}, {NULL}};
const Segment SGI44027[] = {{&S52480[38], 7}, {NULL}};
const Segment SGI44028[] = {{&S52480[31], 6}, {NULL}};
const Segment SGI44029[] = {{&S50778[44], 7}, {NULL}};
const Segment SGI44030[] = {{&S50778[44], 18}, {NULL}};
const Segment SGI44031[] = {{&S50778[30], 6}, {NULL}};
const Segment SGI44032[] = {{&S50778[110], 3}, {NULL}};
const Segment SGI44033[] = {{&S50778[76], 5}, {NULL}};
const Segment SGI44034[] = {{&S50784[44], 7}, {NULL}};
const Segment SGI44035[] = {{&S50784[44], 18}, {NULL}};
const Segment SGI44036[] = {{&S50784[30], 6}, {NULL}};
const Segment SGI44037[] = {{&S50784[110], 3}, {NULL}};
const Segment SGI44038[] = {{&S50784[76], 5}, {NULL}};
const Segment SGI44039[] = {{&S50785[44], 7}, {NULL}};
const Segment SGI44040[] = {{&S50785[44], 18}, {NULL}};
const Segment SGI44041[] = {{&S50785[30], 6}, {NULL}};
const Segment SGI44042[] = {{&S50785[110], 3}, {NULL}};
const Segment SGI44043[] = {{&S50785[76], 5}, {NULL}};
const Segment SGI44044[] = {{&S50787[44], 7}, {NULL}};
const Segment SGI44045[] = {{&S50787[44], 18}, {NULL}};
const Segment SGI44046[] = {{&S50787[30], 6}, {NULL}};
const Segment SGI44047[] = {{&S50787[110], 3}, {NULL}};
const Segment SGI44048[] = {{&S50787[76], 5}, {NULL}};
const Segment SGI44049[] = {{&S50788[44], 7}, {NULL}};
const Segment SGI44050[] = {{&S50788[44], 18}, {NULL}};
const Segment SGI44051[] = {{&S50788[30], 6}, {NULL}};
const Segment SGI44052[] = {{&S50788[110], 3}, {NULL}};
const Segment SGI44053[] = {{&S50788[76], 5}, {NULL}};
const Segment SGI44054[] = {{&S49857[44], 12}, {NULL}};
const Segment SGI44055[] = {{&S49857[44], 22}, {NULL}};
const Segment SGI44056[] = {{&S49857[30], 6}, {NULL}};
const Segment SGI44057[] = {{&S49857[114], 3}, {NULL}};
const Segment SGI44058[] = {{&S49857[80], 5}, {NULL}};
const Segment SGI44059[] = {{&S49860[44], 12}, {NULL}};
const Segment SGI44060[] = {{&S49860[44], 25}, {NULL}};
const Segment SGI44061[] = {{&S49860[30], 6}, {NULL}};
const Segment SGI44062[] = {{&S49860[117], 3}, {NULL}};
const Segment SGI44063[] = {{&S49860[83], 5}, {NULL}};
const Segment SGI44064[] = {{&S49864[38], 12}, {NULL}};
const Segment SGI44065[] = {{&S49864[31], 6}, {NULL}};
const Segment SGI44066[] = {{&S49865[44], 17}, {NULL}};
const Segment SGI44067[] = {{&S49865[44], 27}, {NULL}};
const Segment SGI44068[] = {{&S49865[30], 6}, {NULL}};
const Segment SGI44069[] = {{&S49865[119], 3}, {NULL}};
const Segment SGI44070[] = {{&S49865[85], 5}, {NULL}};
const Segment SGI44071[] = {{&S49872[44], 12}, {NULL}};
const Segment SGI44072[] = {{&S49872[44], 22}, {NULL}};
const Segment SGI44073[] = {{&S49872[30], 6}, {NULL}};
const Segment SGI44074[] = {{&S49872[83], 4}, {NULL}};
const Segment SGI44075[] = {{&S49872[83], 3}, {NULL}};
const Segment SGI44076[] = {{&S49873[44], 12}, {NULL}};
const Segment SGI44077[] = {{&S49873[44], 24}, {NULL}};
const Segment SGI44078[] = {{&S49873[30], 6}, {NULL}};
const Segment SGI44079[] = {{&S49873[116], 3}, {NULL}};
const Segment SGI44080[] = {{&S49873[82], 5}, {NULL}};
const Segment SGI44081[] = {{&S58113[44], 11}, {NULL}};
const Segment SGI44082[] = {{&S58113[44], 28}, {NULL}};
const Segment SGI44083[] = {{&S58113[30], 6}, {NULL}};
const Segment SGI44084[] = {{&S58113[120], 3}, {NULL}};
const Segment SGI44085[] = {{&S58113[86], 5}, {NULL}};
const Segment SGI44086[] = {{&S63196[44], 12}, {NULL}};
const Segment SGI44087[] = {{&S63196[44], 24}, {NULL}};
const Segment SGI44088[] = {{&S63196[30], 6}, {NULL}};
const Segment SGI44089[] = {{&S63196[116], 3}, {NULL}};
const Segment SGI44090[] = {{&S63196[82], 5}, {NULL}};
const Segment SGI44091[] = {{&S58114[44], 8}, {NULL}};
const Segment SGI44092[] = {{&S58114[44], 25}, {NULL}};
const Segment SGI44093[] = {{&S58114[30], 6}, {NULL}};
const Segment SGI44094[] = {{&S58114[117], 3}, {NULL}};
const Segment SGI44095[] = {{&S58114[83], 5}, {NULL}};
const Segment SGI44096[] = {{&S58115[38], 8}, {NULL}};
const Segment SGI44097[] = {{&S58115[31], 6}, {NULL}};
const Segment SGI44098[] = {{&S58116[44], 11}, {NULL}};
const Segment SGI44099[] = {{&S58116[44], 28}, {NULL}};
const Segment SGI44100[] = {{&S58116[30], 6}, {NULL}};
const Segment SGI44101[] = {{&S58116[120], 3}, {NULL}};
const Segment SGI44102[] = {{&S58116[86], 5}, {NULL}};
const Segment SGI44103[] = {{&S58117[44], 11}, {NULL}};
const Segment SGI44104[] = {{&S58117[44], 28}, {NULL}};
const Segment SGI44105[] = {{&S58117[30], 6}, {NULL}};
const Segment SGI44106[] = {{&S58117[120], 3}, {NULL}};
const Segment SGI44107[] = {{&S58117[86], 5}, {NULL}};
const Segment SGI44108[] = {{&S50883[44], 10}, {NULL}};
const Segment SGI44109[] = {{&S50883[44], 21}, {NULL}};
const Segment SGI44110[] = {{&S50883[30], 6}, {NULL}};
const Segment SGI44111[] = {{&S50883[113], 3}, {NULL}};
const Segment SGI44112[] = {{&S50883[79], 5}, {NULL}};
const Segment SGI44113[] = {{&S50884[38], 10}, {NULL}};
const Segment SGI44114[] = {{&S50884[31], 6}, {NULL}};
const Segment SGI44115[] = {{&S63332[44], 10}, {NULL}};
const Segment SGI44116[] = {{&S63332[44], 22}, {NULL}};
const Segment SGI44117[] = {{&S63332[30], 6}, {NULL}};
const Segment SGI44118[] = {{&S63332[114], 3}, {NULL}};
const Segment SGI44119[] = {{&S63332[80], 5}, {NULL}};
const Segment SGI44120[] = {{&S63333[38], 10}, {NULL}};
const Segment SGI44121[] = {{&S63333[31], 6}, {NULL}};
const Segment SGI44122[] = {{&S63334[42], 10}, {NULL}};
const Segment SGI44123[] = {{&S63334[30], 6}, {NULL}};
const Segment SGI44124[] = {{&S63334[112], 3}, {NULL}};
const Segment SGI44125[] = {{&S63334[78], 5}, {NULL}};
const Segment SGI44126[] = {{&S50230[44], 12}, {NULL}};
const Segment SGI44127[] = {{&S50230[44], 23}, {NULL}};
const Segment SGI44128[] = {{&S50230[30], 6}, {NULL}};
const Segment SGI44129[] = {{&S50230[115], 3}, {NULL}};
const Segment SGI44130[] = {{&S50230[81], 5}, {NULL}};
const Segment SGI44131[] = {{&S50234[38], 12}, {NULL}};
const Segment SGI44132[] = {{&S50234[31], 6}, {NULL}};
const Segment SGI44133[] = {{&S50235[44], 12}, {NULL}};
const Segment SGI44134[] = {{&S50235[44], 23}, {NULL}};
const Segment SGI44135[] = {{&S50235[30], 6}, {NULL}};
const Segment SGI44136[] = {{&S50235[84], 4}, {NULL}};
const Segment SGI44137[] = {{&S50235[84], 3}, {NULL}};
const Segment SGI44138[] = {{&S50236[44], 12}, {NULL}};
const Segment SGI44139[] = {{&S50236[44], 23}, {NULL}};
const Segment SGI44140[] = {{&S50236[30], 6}, {NULL}};
const Segment SGI44141[] = {{&S50236[84], 6}, {NULL}};
const Segment SGI44142[] = {{&S63487[44], 7}, {NULL}};
const Segment SGI44143[] = {{&S63487[44], 19}, {NULL}};
const Segment SGI44144[] = {{&S63487[30], 6}, {NULL}};
const Segment SGI44145[] = {{&S63487[111], 3}, {NULL}};
const Segment SGI44146[] = {{&S63487[77], 5}, {NULL}};
const Segment SGI44147[] = {{&S63488[38], 7}, {NULL}};
const Segment SGI44148[] = {{&S63488[31], 6}, {NULL}};
const Segment SGI44149[] = {{&S63642[44], 6}, {NULL}};
const Segment SGI44150[] = {{&S63642[44], 18}, {NULL}};
const Segment SGI44151[] = {{&S63642[30], 6}, {NULL}};
const Segment SGI44152[] = {{&S63642[110], 3}, {NULL}};
const Segment SGI44153[] = {{&S63642[76], 5}, {NULL}};
const Segment SGI44154[] = {{&S63643[38], 6}, {NULL}};
const Segment SGI44155[] = {{&S63643[31], 6}, {NULL}};
const Segment SGI44156[] = {{&S58518[44], 31}, {NULL}};
const Segment SGI44157[] = {{&S58518[30], 6}, {NULL}};
const Segment SGI44158[] = {{&S58518[123], 3}, {NULL}};
const Segment SGI44159[] = {{&S58518[89], 5}, {NULL}};
const Segment SGI44160[] = {{&S50456[44], 12}, {NULL}};
const Segment SGI44161[] = {{&S50456[44], 23}, {NULL}};
const Segment SGI44162[] = {{&S50456[30], 6}, {NULL}};
const Segment SGI44163[] = {{&S50456[115], 3}, {NULL}};
const Segment SGI44164[] = {{&S50456[81], 5}, {NULL}};
const Segment SGI44165[] = {{&S50457[38], 12}, {NULL}};
const Segment SGI44166[] = {{&S50457[31], 6}, {NULL}};
const Segment SGI44167[] = {{&S63749[44], 12}, {NULL}};
const Segment SGI44168[] = {{&S63749[44], 31}, {NULL}};
const Segment SGI44169[] = {{&S63749[30], 6}, {NULL}};
const Segment SGI44170[] = {{&S63749[122], 3}, {NULL}};
const Segment SGI44171[] = {{&S63749[88], 5}, {NULL}};
const Segment SGI44172[] = {{&S63750[44], 15}, {NULL}};
const Segment SGI44173[] = {{&S63750[44], 34}, {NULL}};
const Segment SGI44174[] = {{&S63750[30], 6}, {NULL}};
const Segment SGI44175[] = {{&S63750[72], 3}, {NULL}};
const Segment SGI44176[] = {{&S63750[91], 5}, {NULL}};
const Segment SGI44177[] = {{&S63751[44], 15}, {NULL}};
const Segment SGI44178[] = {{&S63751[44], 34}, {NULL}};
const Segment SGI44179[] = {{&S63751[30], 6}, {NULL}};
const Segment SGI44180[] = {{&S63751[125], 3}, {NULL}};
const Segment SGI44181[] = {{&S63751[91], 5}, {NULL}};
const Segment SGI44182[] = {{&S63752[38], 12}, {NULL}};
const Segment SGI44183[] = {{&S50575[44], 19}, {NULL}};
const Segment SGI44184[] = {{&S50575[44], 36}, {NULL}};
const Segment SGI44185[] = {{&S50575[30], 6}, {NULL}};
const Segment SGI44186[] = {{&S50575[128], 3}, {NULL}};
const Segment SGI44187[] = {{&S50575[94], 5}, {NULL}};
const Segment SGI44188[] = {{&S50576[38], 19}, {NULL}};
const Segment SGI44189[] = {{&S50576[31], 6}, {NULL}};
const Segment SGI44190[] = {{&S63752[31], 6}, {NULL}};
const Segment SGI44191[] = {{&S63753[38], 15}, {NULL}};
const Segment SGI44192[] = {{&S63753[31], 6}, {NULL}};
const Segment SGI44193[] = {{&S63754[44], 12}, {NULL}};
const Segment SGI44194[] = {{&S63754[44], 31}, {NULL}};
const Segment SGI44195[] = {{&S63754[30], 6}, {NULL}};
const Segment SGI44196[] = {{&S63754[121], 3}, {NULL}};
const Segment SGI44197[] = {{&S63754[88], 5}, {NULL}};
const Segment SGI44198[] = {{&S63755[44], 15}, {NULL}};
const Segment SGI44199[] = {{&S63755[44], 34}, {NULL}};
const Segment SGI44200[] = {{&S63755[30], 6}, {NULL}};
const Segment SGI44201[] = {{&S63755[124], 3}, {NULL}};
const Segment SGI44202[] = {{&S63755[91], 5}, {NULL}};
const Segment SGI44203[] = {{&S58739[44], 10}, {NULL}};
const Segment SGI44204[] = {{&S58739[44], 27}, {NULL}};
const Segment SGI44205[] = {{&S58739[30], 6}, {NULL}};
const Segment SGI44206[] = {{&S58739[119], 3}, {NULL}};
const Segment SGI44207[] = {{&S58739[85], 5}, {NULL}};
const Segment SGI44208[] = {{&S50851[44], 11}, {NULL}};
const Segment SGI44209[] = {{&S50851[44], 22}, {NULL}};
const Segment SGI44210[] = {{&S50851[30], 6}, {NULL}};
const Segment SGI44211[] = {{&S50851[114], 3}, {NULL}};
const Segment SGI44212[] = {{&S50851[80], 5}, {NULL}};
const Segment SGI44213[] = {{&S50852[38], 11}, {NULL}};
const Segment SGI44214[] = {{&S50852[31], 6}, {NULL}};
const Segment SGI44215[] = {{&S50853[44], 11}, {NULL}};
const Segment SGI44216[] = {{&S50853[44], 22}, {NULL}};
const Segment SGI44217[] = {{&S50853[30], 6}, {NULL}};
const Segment SGI44218[] = {{&S50853[83], 4}, {NULL}};
const Segment SGI44219[] = {{&S50853[83], 3}, {NULL}};
const Segment SGI44220[] = {{&S50854[44], 11}, {NULL}};
const Segment SGI44221[] = {{&S50854[44], 22}, {NULL}};
const Segment SGI44222[] = {{&S50854[30], 6}, {NULL}};
const Segment SGI44223[] = {{&S50854[83], 6}, {NULL}};
const Segment SGI44224[] = {{&S51699[44], 11}, {NULL}};
const Segment SGI44225[] = {{&S51699[44], 22}, {NULL}};
const Segment SGI44226[] = {{&S51699[30], 6}, {NULL}};
const Segment SGI44227[] = {{&S51699[114], 3}, {NULL}};
const Segment SGI44228[] = {{&S51699[80], 5}, {NULL}};
const Segment SGI44229[] = {{&S51700[44], 18}, {NULL}};
const Segment SGI44230[] = {{&S51700[44], 29}, {NULL}};
const Segment SGI44231[] = {{&S51700[30], 6}, {NULL}};
const Segment SGI44232[] = {{&S51700[121], 3}, {NULL}};
const Segment SGI44233[] = {{&S51700[87], 5}, {NULL}};
const Segment SGI44234[] = {{&S51701[38], 11}, {NULL}};
const Segment SGI44235[] = {{&S51701[31], 6}, {NULL}};
const Segment SGI44236[] = {{&S64071[6], 4}, {NULL}};
const Segment SGI44237[] = {{&S64071[85], 5}, {NULL}};
const Segment SGI44238[] = {{&S53599[44], 9}, {NULL}};
const Segment SGI44239[] = {{&S53599[44], 26}, {NULL}};
const Segment SGI44240[] = {{&S53599[30], 6}, {NULL}};
const Segment SGI44241[] = {{&S53599[118], 3}, {NULL}};
const Segment SGI44242[] = {{&S53599[84], 5}, {NULL}};
const Segment SGI44243[] = {{&S51962[44], 10}, {NULL}};
const Segment SGI44244[] = {{&S51962[44], 27}, {NULL}};
const Segment SGI44245[] = {{&S51962[30], 6}, {NULL}};
const Segment SGI44246[] = {{&S51962[119], 3}, {NULL}};
const Segment SGI44247[] = {{&S51962[85], 5}, {NULL}};
const Segment SGI44248[] = {{&S51963[38], 10}, {NULL}};
const Segment SGI44249[] = {{&S51963[31], 6}, {NULL}};
const Segment SGI44250[] = {{&S51964[42], 10}, {NULL}};
const Segment SGI44251[] = {{&S51964[30], 6}, {NULL}};
const Segment SGI44252[] = {{&S51964[117], 3}, {NULL}};
const Segment SGI44253[] = {{&S51964[83], 5}, {NULL}};
const Segment SGI44254[] = {{&S53649[44], 8}, {NULL}};
const Segment SGI44255[] = {{&S53649[44], 25}, {NULL}};
const Segment SGI44256[] = {{&S53649[30], 6}, {NULL}};
const Segment SGI44257[] = {{&S53649[117], 3}, {NULL}};
const Segment SGI44258[] = {{&S53649[83], 5}, {NULL}};
const Segment SGI44259[] = {{&S53651[38], 8}, {NULL}};
const Segment SGI44260[] = {{&S53651[31], 6}, {NULL}};
const Segment SGI44261[] = {{&S52131[44], 7}, {NULL}};
const Segment SGI44262[] = {{&S52131[44], 18}, {NULL}};
const Segment SGI44263[] = {{&S52131[30], 6}, {NULL}};
const Segment SGI44264[] = {{&S52131[110], 3}, {NULL}};
const Segment SGI44265[] = {{&S52131[76], 5}, {NULL}};
const Segment SGI44266[] = {{&S52132[44], 11}, {NULL}};
const Segment SGI44267[] = {{&S52132[44], 22}, {NULL}};
const Segment SGI44268[] = {{&S52132[30], 6}, {NULL}};
const Segment SGI44269[] = {{&S52132[114], 3}, {NULL}};
const Segment SGI44270[] = {{&S52132[80], 5}, {NULL}};
const Segment SGI44271[] = {{&S52139[44], 7}, {NULL}};
const Segment SGI44272[] = {{&S52139[44], 18}, {NULL}};
const Segment SGI44273[] = {{&S52139[30], 6}, {NULL}};
const Segment SGI44274[] = {{&S52139[110], 3}, {NULL}};
const Segment SGI44275[] = {{&S52139[76], 5}, {NULL}};
const Segment SGI44276[] = {{&S52229[34], 10}, {NULL}};
const Segment SGI44277[] = {{&S52229[27], 6}, {NULL}};
const Segment SGI44278[] = {{&S52229[109], 2}, {NULL}};
const Segment SGI44279[] = {{&S52229[75], 6}, {NULL}};
const Segment SGI44280[] = {{&S52229[143], 8}, {NULL}};
const Segment SGI44281[] = {{&S54023[44], 10}, {NULL}};
const Segment SGI44282[] = {{&S54023[44], 27}, {NULL}};
const Segment SGI44283[] = {{&S54023[30], 6}, {NULL}};
const Segment SGI44284[] = {{&S54023[119], 3}, {NULL}};
const Segment SGI44285[] = {{&S54023[85], 5}, {NULL}};
const Segment SGI44286[] = {{&S54024[38], 10}, {NULL}};
const Segment SGI44287[] = {{&S54024[31], 6}, {NULL}};
const Segment SGI44288[] = {{&S54025[42], 10}, {NULL}};
const Segment SGI44289[] = {{&S54025[30], 6}, {NULL}};
const Segment SGI44290[] = {{&S54025[117], 3}, {NULL}};
const Segment SGI44291[] = {{&S54025[83], 5}, {NULL}};
const Segment SGI44292[] = {{&S51468[44], 12}, {NULL}};
const Segment SGI44293[] = {{&S51468[44], 33}, {NULL}};
const Segment SGI44294[] = {{&S51468[30], 6}, {NULL}};
const Segment SGI44295[] = {{&S51468[125], 3}, {NULL}};
const Segment SGI44296[] = {{&S51468[91], 5}, {NULL}};
const Segment SGI44297[] = {{&S64754[44], 24}, {NULL}};
const Segment SGI44298[] = {{&S64754[44], 33}, {NULL}};
const Segment SGI44299[] = {{&S64754[30], 6}, {NULL}};
const Segment SGI44300[] = {{&S64754[125], 3}, {NULL}};
const Segment SGI44301[] = {{&S64754[91], 5}, {NULL}};
const Segment SGI44302[] = {{&S52267[44], 12}, {NULL}};
const Segment SGI44303[] = {{&S52267[44], 24}, {NULL}};
const Segment SGI44304[] = {{&S52267[30], 6}, {NULL}};
const Segment SGI44305[] = {{&S52267[116], 3}, {NULL}};
const Segment SGI44306[] = {{&S52267[82], 5}, {NULL}};
const Segment SGI44307[] = {{&S52268[44], 12}, {NULL}};
const Segment SGI44308[] = {{&S52268[44], 23}, {NULL}};
const Segment SGI44309[] = {{&S52268[30], 6}, {NULL}};
const Segment SGI44310[] = {{&S52268[115], 3}, {NULL}};
const Segment SGI44311[] = {{&S52268[81], 5}, {NULL}};
const Segment SGI44312[] = {{&S52273[38], 12}, {NULL}};
const Segment SGI44313[] = {{&S52273[31], 6}, {NULL}};
const Segment SGI44314[] = {{&S54129[44], 13}, {NULL}};
const Segment SGI44315[] = {{&S54129[44], 24}, {NULL}};
const Segment SGI44316[] = {{&S54129[30], 6}, {NULL}};
const Segment SGI44317[] = {{&S54129[116], 3}, {NULL}};
const Segment SGI44318[] = {{&S54129[82], 5}, {NULL}};
const Segment SGI44319[] = {{&S54130[44], 13}, {NULL}};
const Segment SGI44320[] = {{&S54130[44], 24}, {NULL}};
const Segment SGI44321[] = {{&S54130[30], 6}, {NULL}};
const Segment SGI44322[] = {{&S54130[116], 3}, {NULL}};
const Segment SGI44323[] = {{&S54130[82], 5}, {NULL}};
const Segment SGI44324[] = {{&S54131[44], 13}, {NULL}};
const Segment SGI44325[] = {{&S54131[44], 24}, {NULL}};
const Segment SGI44326[] = {{&S54131[30], 6}, {NULL}};
const Segment SGI44327[] = {{&S54131[116], 3}, {NULL}};
const Segment SGI44328[] = {{&S54131[82], 5}, {NULL}};
const Segment SGI44329[] = {{&S54132[44], 13}, {NULL}};
const Segment SGI44330[] = {{&S54132[44], 24}, {NULL}};
const Segment SGI44331[] = {{&S54132[30], 6}, {NULL}};
const Segment SGI44332[] = {{&S54132[116], 3}, {NULL}};
const Segment SGI44333[] = {{&S54132[82], 5}, {NULL}};
const Segment SGI44334[] = {{&S54133[44], 13}, {NULL}};
const Segment SGI44335[] = {{&S54133[44], 24}, {NULL}};
const Segment SGI44336[] = {{&S54133[30], 6}, {NULL}};
const Segment SGI44337[] = {{&S54133[116], 3}, {NULL}};
const Segment SGI44338[] = {{&S54133[82], 5}, {NULL}};
const Segment SGI44339[] = {{&S54134[44], 13}, {NULL}};
const Segment SGI44340[] = {{&S54134[44], 24}, {NULL}};
const Segment SGI44341[] = {{&S54134[30], 6}, {NULL}};
const Segment SGI44342[] = {{&S54134[116], 3}, {NULL}};
const Segment SGI44343[] = {{&S54134[82], 5}, {NULL}};
const Segment SGI44344[] = {{&S54135[44], 13}, {NULL}};
const Segment SGI44345[] = {{&S54135[44], 24}, {NULL}};
const Segment SGI44346[] = {{&S54135[30], 6}, {NULL}};
const Segment SGI44347[] = {{&S54135[116], 3}, {NULL}};
const Segment SGI44348[] = {{&S54135[82], 5}, {NULL}};
const Segment SGI44349[] = {{&S54136[44], 13}, {NULL}};
const Segment SGI44350[] = {{&S54136[44], 24}, {NULL}};
const Segment SGI44351[] = {{&S54136[30], 6}, {NULL}};
const Segment SGI44352[] = {{&S54136[116], 3}, {NULL}};
const Segment SGI44353[] = {{&S54136[82], 5}, {NULL}};
const Segment SGI44354[] = {{&S54137[44], 13}, {NULL}};
const Segment SGI44355[] = {{&S54137[44], 24}, {NULL}};
const Segment SGI44356[] = {{&S54137[30], 6}, {NULL}};
const Segment SGI44357[] = {{&S54137[116], 3}, {NULL}};
const Segment SGI44358[] = {{&S54137[82], 5}, {NULL}};
const Segment SGI44359[] = {{&S54138[38], 13}, {NULL}};
const Segment SGI44360[] = {{&S54138[31], 6}, {NULL}};
const Segment SGI44361[] = {{&S52321[44], 11}, {NULL}};
const Segment SGI44362[] = {{&S52321[44], 28}, {NULL}};
const Segment SGI44363[] = {{&S52321[30], 6}, {NULL}};
const Segment SGI44364[] = {{&S52321[120], 3}, {NULL}};
const Segment SGI44365[] = {{&S52321[86], 5}, {NULL}};
const Segment SGI44366[] = {{&S52324[44], 11}, {NULL}};
const Segment SGI44367[] = {{&S52324[44], 28}, {NULL}};
const Segment SGI44368[] = {{&S52324[30], 6}, {NULL}};
const Segment SGI44369[] = {{&S52324[120], 3}, {NULL}};
const Segment SGI44370[] = {{&S52324[86], 5}, {NULL}};
const Segment SGI44371[] = {{&S52327[38], 11}, {NULL}};
const Segment SGI44372[] = {{&S52327[31], 6}, {NULL}};
const Segment SGI44373[] = {{&S52328[38], 11}, {NULL}};
const Segment SGI44374[] = {{&S52328[31], 6}, {NULL}};
const Segment SGI44375[] = {{&S52335[44], 10}, {NULL}};
const Segment SGI44376[] = {{&S52335[44], 27}, {NULL}};
const Segment SGI44377[] = {{&S52335[30], 6}, {NULL}};
const Segment SGI44378[] = {{&S52335[119], 3}, {NULL}};
const Segment SGI44379[] = {{&S52335[85], 5}, {NULL}};
const Segment SGI44380[] = {{&S52340[44], 11}, {NULL}};
const Segment SGI44381[] = {{&S52340[44], 28}, {NULL}};
const Segment SGI44382[] = {{&S52340[30], 6}, {NULL}};
const Segment SGI44383[] = {{&S52340[120], 3}, {NULL}};
const Segment SGI44384[] = {{&S52340[86], 5}, {NULL}};
const Segment SGI44385[] = {{&S52379[44], 12}, {NULL}};
const Segment SGI44386[] = {{&S52379[44], 29}, {NULL}};
const Segment SGI44387[] = {{&S52379[30], 6}, {NULL}};
const Segment SGI44388[] = {{&S52379[121], 3}, {NULL}};
const Segment SGI44389[] = {{&S52379[87], 5}, {NULL}};
const Segment SGI44390[] = {{&S52384[38], 12}, {NULL}};
const Segment SGI44391[] = {{&S52384[31], 6}, {NULL}};
const Segment SGI44392[] = {{&S52391[44], 5}, {NULL}};
const Segment SGI44393[] = {{&S52391[44], 16}, {NULL}};
const Segment SGI44394[] = {{&S52391[30], 6}, {NULL}};
const Segment SGI44395[] = {{&S52391[108], 3}, {NULL}};
const Segment SGI44396[] = {{&S52391[74], 5}, {NULL}};
const Segment SGI44397[] = {{&S52467[44], 7}, {NULL}};
const Segment SGI44398[] = {{&S52467[44], 19}, {NULL}};
const Segment SGI44399[] = {{&S52467[30], 6}, {NULL}};
const Segment SGI44400[] = {{&S52467[111], 3}, {NULL}};
const Segment SGI44401[] = {{&S52467[77], 5}, {NULL}};
const Segment SGI44402[] = {{&S52468[44], 7}, {NULL}};
const Segment SGI44403[] = {{&S52468[44], 19}, {NULL}};
const Segment SGI44404[] = {{&S52468[30], 6}, {NULL}};
const Segment SGI44405[] = {{&S52468[111], 3}, {NULL}};
const Segment SGI44406[] = {{&S52468[77], 5}, {NULL}};
const Segment SGI44407[] = {{&S52475[44], 7}, {NULL}};
const Segment SGI44408[] = {{&S52475[44], 18}, {NULL}};
const Segment SGI44409[] = {{&S52475[30], 6}, {NULL}};
const Segment SGI44410[] = {{&S52475[110], 3}, {NULL}};
const Segment SGI44411[] = {{&S52475[76], 5}, {NULL}};
const Segment SGI44412[] = {{&S52476[38], 7}, {NULL}};
const Segment SGI44413[] = {{&S52476[31], 6}, {NULL}};
const Segment SGI44414[] = {{&S54428[44], 37}, {NULL}};
const Segment SGI44415[] = {{&S54428[30], 6}, {NULL}};
const Segment SGI44416[] = {{&S54428[113], 5}, {NULL}};
const Segment SGI44417[] = {{&S54429[44], 7}, {NULL}};
const Segment SGI44418[] = {{&S54429[44], 24}, {NULL}};
const Segment SGI44419[] = {{&S54429[30], 6}, {NULL}};
const Segment SGI44420[] = {{&S54429[85], 4}, {NULL}};
const Segment SGI44421[] = {{&S54429[85], 3}, {NULL}};
const Segment SGI44422[] = {{&S54433[44], 7}, {NULL}};
const Segment SGI44423[] = {{&S54433[44], 24}, {NULL}};
const Segment SGI44424[] = {{&S54433[30], 6}, {NULL}};
const Segment SGI44425[] = {{&S54433[85], 6}, {NULL}};
const Segment SGI44426[] = {{&S54434[38], 7}, {NULL}};
const Segment SGI44427[] = {{&S54434[31], 6}, {NULL}};
const Segment SGI44428[] = {{&S54439[44], 24}, {NULL}};
const Segment SGI44429[] = {{&S54439[30], 6}, {NULL}};
const Segment SGI44430[] = {{&S52650[44], 8}, {NULL}};
const Segment SGI44431[] = {{&S52650[44], 33}, {NULL}};
const Segment SGI44432[] = {{&S52650[30], 6}, {NULL}};
const Segment SGI44433[] = {{&S52650[125], 3}, {NULL}};
const Segment SGI44434[] = {{&S52650[91], 5}, {NULL}};
const Segment SGI44435[] = {{&S54440[44], 7}, {NULL}};
const Segment SGI44436[] = {{&S54440[44], 24}, {NULL}};
const Segment SGI44437[] = {{&S54440[30], 6}, {NULL}};
const Segment SGI44438[] = {{&S54440[117], 3}, {NULL}};
const Segment SGI44439[] = {{&S54440[82], 6}, {NULL}};
const Segment SGI44440[] = {{&S59472[42], 14}, {NULL}};
const Segment SGI44441[] = {{&S59472[42], 25}, {NULL}};
const Segment SGI44442[] = {{&S59472[30], 4}, {NULL}};
const Segment SGI44443[] = {{&S59472[115], 3}, {NULL}};
const Segment SGI44444[] = {{&S59472[81], 5}, {NULL}};
const Segment SGI44445[] = {{&S59474[42], 20}, {NULL}};
const Segment SGI44446[] = {{&S59474[42], 31}, {NULL}};
const Segment SGI44447[] = {{&S59474[30], 4}, {NULL}};
const Segment SGI44448[] = {{&S59474[121], 3}, {NULL}};
const Segment SGI44449[] = {{&S59474[87], 5}, {NULL}};
const Segment SGI44450[] = {{&S59476[42], 27}, {NULL}};
const Segment SGI44451[] = {{&S59476[42], 38}, {NULL}};
const Segment SGI44452[] = {{&S59476[30], 4}, {NULL}};
const Segment SGI44453[] = {{&S59476[128], 3}, {NULL}};
const Segment SGI44454[] = {{&S59476[94], 5}, {NULL}};
const Segment SGI44455[] = {{&S65213[44], 7}, {NULL}};
const Segment SGI44456[] = {{&S65213[44], 18}, {NULL}};
const Segment SGI44457[] = {{&S65213[30], 6}, {NULL}};
const Segment SGI44458[] = {{&S65213[110], 3}, {NULL}};
const Segment SGI44459[] = {{&S65213[76], 5}, {NULL}};
const Segment SGI44460[] = {{&S54505[44], 9}, {NULL}};
const Segment SGI44461[] = {{&S54505[44], 20}, {NULL}};
const Segment SGI44462[] = {{&S54505[30], 6}, {NULL}};
const Segment SGI44463[] = {{&S54505[112], 3}, {NULL}};
const Segment SGI44464[] = {{&S54505[78], 5}, {NULL}};
const Segment SGI44465[] = {{&S54506[51], 9}, {NULL}};
const Segment SGI44466[] = {{&S54506[51], 24}, {NULL}};
const Segment SGI44467[] = {{&S54506[37], 6}, {NULL}};
const Segment SGI44468[] = {{&S54507[38], 9}, {NULL}};
const Segment SGI44469[] = {{&S54507[31], 6}, {NULL}};
const Segment SGI44470[] = {{&S51933[44], 28}, {NULL}};
const Segment SGI44471[] = {{&S51933[44], 45}, {NULL}};
const Segment SGI44472[] = {{&S51933[30], 6}, {NULL}};
const Segment SGI44473[] = {{&S51933[137], 3}, {NULL}};
const Segment SGI44474[] = {{&S51933[103], 5}, {NULL}};
const Segment SGI44475[] = {{&S51934[44], 23}, {NULL}};
const Segment SGI44476[] = {{&S51934[44], 40}, {NULL}};
const Segment SGI44477[] = {{&S51934[30], 6}, {NULL}};
const Segment SGI44478[] = {{&S51934[132], 3}, {NULL}};
const Segment SGI44479[] = {{&S51934[98], 5}, {NULL}};
const Segment SGI44480[] = {{&S51935[38], 28}, {NULL}};
const Segment SGI44481[] = {{&S51935[31], 6}, {NULL}};
const Segment SGI44482[] = {{&S65292[44], 12}, {NULL}};
const Segment SGI44483[] = {{&S65292[44], 23}, {NULL}};
const Segment SGI44484[] = {{&S65292[30], 6}, {NULL}};
const Segment SGI44485[] = {{&S65292[115], 3}, {NULL}};
const Segment SGI44486[] = {{&S65292[81], 5}, {NULL}};
const Segment SGI44487[] = {{&S65293[44], 6}, {NULL}};
const Segment SGI44488[] = {{&S65293[44], 17}, {NULL}};
const Segment SGI44489[] = {{&S65293[30], 6}, {NULL}};
const Segment SGI44490[] = {{&S65293[109], 3}, {NULL}};
const Segment SGI44491[] = {{&S65293[75], 5}, {NULL}};
const Segment SGI44492[] = {{&S65294[44], 12}, {NULL}};
const Segment SGI44493[] = {{&S65294[44], 23}, {NULL}};
const Segment SGI44494[] = {{&S65294[30], 6}, {NULL}};
const Segment SGI44495[] = {{&S65294[115], 3}, {NULL}};
const Segment SGI44496[] = {{&S65294[81], 5}, {NULL}};
const Segment SGI44497[] = {{&S65295[44], 12}, {NULL}};
const Segment SGI44498[] = {{&S65295[44], 23}, {NULL}};
const Segment SGI44499[] = {{&S65295[30], 6}, {NULL}};
const Segment SGI44500[] = {{&S65295[115], 3}, {NULL}};
const Segment SGI44501[] = {{&S65295[81], 5}, {NULL}};
const Segment SGI44502[] = {{&S65296[6], 4}, {NULL}};
const Segment SGI44503[] = {{&S65296[82], 5}, {NULL}};
const Segment SGI44504[] = {{&S51967[44], 15}, {NULL}};
const Segment SGI44505[] = {{&S51967[44], 32}, {NULL}};
const Segment SGI44506[] = {{&S51967[30], 6}, {NULL}};
const Segment SGI44507[] = {{&S51967[124], 3}, {NULL}};
const Segment SGI44508[] = {{&S51967[90], 5}, {NULL}};
const Segment SGI44509[] = {{&S65356[44], 11}, {NULL}};
const Segment SGI44510[] = {{&S65356[44], 23}, {NULL}};
const Segment SGI44511[] = {{&S65356[30], 6}, {NULL}};
const Segment SGI44512[] = {{&S65356[115], 3}, {NULL}};
const Segment SGI44513[] = {{&S65356[81], 5}, {NULL}};
const Segment SGI44514[] = {{&S65357[38], 11}, {NULL}};
const Segment SGI44515[] = {{&S65357[31], 6}, {NULL}};
const Segment SGI44516[] = {{&S52842[44], 11}, {NULL}};
const Segment SGI44517[] = {{&S52842[44], 24}, {NULL}};
const Segment SGI44518[] = {{&S52842[30], 6}, {NULL}};
const Segment SGI44519[] = {{&S52842[116], 3}, {NULL}};
const Segment SGI44520[] = {{&S52842[82], 5}, {NULL}};
const Segment SGI44521[] = {{&S52843[44], 11}, {NULL}};
const Segment SGI44522[] = {{&S52843[44], 22}, {NULL}};
const Segment SGI44523[] = {{&S52843[30], 6}, {NULL}};
const Segment SGI44524[] = {{&S52843[114], 3}, {NULL}};
const Segment SGI44525[] = {{&S52843[80], 5}, {NULL}};
const Segment SGI44526[] = {{&S52847[38], 11}, {NULL}};
const Segment SGI44527[] = {{&S52847[31], 6}, {NULL}};
const Segment SGI44528[] = {{&S54728[44], 9}, {NULL}};
const Segment SGI44529[] = {{&S54728[44], 20}, {NULL}};
const Segment SGI44530[] = {{&S54728[30], 6}, {NULL}};
const Segment SGI44531[] = {{&S54728[112], 3}, {NULL}};
const Segment SGI44532[] = {{&S54728[78], 5}, {NULL}};
const Segment SGI44533[] = {{&S54729[44], 9}, {NULL}};
const Segment SGI44534[] = {{&S54729[44], 24}, {NULL}};
const Segment SGI44535[] = {{&S54729[30], 6}, {NULL}};
const Segment SGI44536[] = {{&S54730[38], 9}, {NULL}};
const Segment SGI44537[] = {{&S54730[31], 6}, {NULL}};
const Segment SGI44538[] = {{&S52134[44], 7}, {NULL}};
const Segment SGI44539[] = {{&S52134[44], 18}, {NULL}};
const Segment SGI44540[] = {{&S52134[30], 6}, {NULL}};
const Segment SGI44541[] = {{&S52134[110], 3}, {NULL}};
const Segment SGI44542[] = {{&S52134[76], 5}, {NULL}};
const Segment SGI44543[] = {{&S52136[44], 11}, {NULL}};
const Segment SGI44544[] = {{&S52136[44], 22}, {NULL}};
const Segment SGI44545[] = {{&S52136[30], 6}, {NULL}};
const Segment SGI44546[] = {{&S52136[114], 3}, {NULL}};
const Segment SGI44547[] = {{&S52136[80], 5}, {NULL}};
const Segment SGI44548[] = {{&S52227[34], 10}, {NULL}};
const Segment SGI44549[] = {{&S52227[27], 6}, {NULL}};
const Segment SGI44550[] = {{&S52227[109], 2}, {NULL}};
const Segment SGI44551[] = {{&S52227[75], 6}, {NULL}};
const Segment SGI44552[] = {{&S52227[143], 8}, {NULL}};
const Segment SGI44553[] = {{&S65556[44], 6}, {NULL}};
const Segment SGI44554[] = {{&S65556[44], 18}, {NULL}};
const Segment SGI44555[] = {{&S65556[30], 6}, {NULL}};
const Segment SGI44556[] = {{&S65556[110], 3}, {NULL}};
const Segment SGI44557[] = {{&S65556[76], 5}, {NULL}};
const Segment SGI44558[] = {{&S52232[34], 11}, {NULL}};
const Segment SGI44559[] = {{&S52232[27], 6}, {NULL}};
const Segment SGI44560[] = {{&S52232[110], 2}, {NULL}};
const Segment SGI44561[] = {{&S52232[76], 6}, {NULL}};
const Segment SGI44562[] = {{&S52232[144], 8}, {NULL}};
const Segment SGI44563[] = {{&S52269[44], 12}, {NULL}};
const Segment SGI44564[] = {{&S52269[44], 23}, {NULL}};
const Segment SGI44565[] = {{&S52269[30], 6}, {NULL}};
const Segment SGI44566[] = {{&S52269[115], 3}, {NULL}};
const Segment SGI44567[] = {{&S52269[81], 5}, {NULL}};
const Segment SGI44568[] = {{&S52274[38], 12}, {NULL}};
const Segment SGI44569[] = {{&S52274[31], 6}, {NULL}};
const Segment SGI44570[] = {{&S52275[44], 12}, {NULL}};
const Segment SGI44571[] = {{&S52275[44], 23}, {NULL}};
const Segment SGI44572[] = {{&S52275[30], 6}, {NULL}};
const Segment SGI44573[] = {{&S52275[115], 3}, {NULL}};
const Segment SGI44574[] = {{&S52275[81], 5}, {NULL}};
const Segment SGI44575[] = {{&S52390[44], 5}, {NULL}};
const Segment SGI44576[] = {{&S52390[44], 16}, {NULL}};
const Segment SGI44577[] = {{&S52390[30], 6}, {NULL}};
const Segment SGI44578[] = {{&S52390[108], 3}, {NULL}};
const Segment SGI44579[] = {{&S52390[74], 5}, {NULL}};
const Segment SGI44580[] = {{&S65776[44], 10}, {NULL}};
const Segment SGI44581[] = {{&S65776[44], 21}, {NULL}};
const Segment SGI44582[] = {{&S65776[30], 6}, {NULL}};
const Segment SGI44583[] = {{&S65776[113], 3}, {NULL}};
const Segment SGI44584[] = {{&S65776[79], 5}, {NULL}};
const Segment SGI44585[] = {{&S65777[44], 10}, {NULL}};
const Segment SGI44586[] = {{&S65777[44], 22}, {NULL}};
const Segment SGI44587[] = {{&S65777[30], 6}, {NULL}};
const Segment SGI44588[] = {{&S65777[114], 3}, {NULL}};
const Segment SGI44589[] = {{&S65777[80], 5}, {NULL}};
const Segment SGI44590[] = {{&S65779[38], 10}, {NULL}};
const Segment SGI44591[] = {{&S65779[31], 6}, {NULL}};
const Segment SGI44592[] = {{&S52469[44], 7}, {NULL}};
const Segment SGI44593[] = {{&S52469[44], 19}, {NULL}};
const Segment SGI44594[] = {{&S52469[30], 6}, {NULL}};
const Segment SGI44595[] = {{&S52469[111], 3}, {NULL}};
const Segment SGI44596[] = {{&S52469[77], 5}, {NULL}};
const Segment SGI44597[] = {{&S52482[38], 7}, {NULL}};
const Segment SGI44598[] = {{&S52482[31], 6}, {NULL}};
const Segment SGI44599[] = {{&S52484[44], 19}, {NULL}};
const Segment SGI44600[] = {{&S52484[44], 30}, {NULL}};
const Segment SGI44601[] = {{&S52484[30], 6}, {NULL}};
const Segment SGI44602[] = {{&S52484[122], 3}, {NULL}};
const Segment SGI44603[] = {{&S52484[88], 5}, {NULL}};
const Segment SGI44604[] = {{&S52485[44], 19}, {NULL}};
const Segment SGI44605[] = {{&S52485[44], 30}, {NULL}};
const Segment SGI44606[] = {{&S52485[30], 6}, {NULL}};
const Segment SGI44607[] = {{&S52485[122], 3}, {NULL}};
const Segment SGI44608[] = {{&S52485[88], 5}, {NULL}};
const Segment SGI44609[] = {{&S52486[44], 19}, {NULL}};
const Segment SGI44610[] = {{&S52486[44], 30}, {NULL}};
const Segment SGI44611[] = {{&S52486[30], 6}, {NULL}};
const Segment SGI44612[] = {{&S52486[122], 3}, {NULL}};
const Segment SGI44613[] = {{&S52486[88], 5}, {NULL}};
const Segment SGI44614[] = {{&S52487[44], 19}, {NULL}};
const Segment SGI44615[] = {{&S52487[44], 30}, {NULL}};
const Segment SGI44616[] = {{&S52487[30], 6}, {NULL}};
const Segment SGI44617[] = {{&S52487[122], 3}, {NULL}};
const Segment SGI44618[] = {{&S52487[88], 5}, {NULL}};
const Segment SGI44619[] = {{&S52488[44], 19}, {NULL}};
const Segment SGI44620[] = {{&S52488[44], 30}, {NULL}};
const Segment SGI44621[] = {{&S52488[30], 6}, {NULL}};
const Segment SGI44622[] = {{&S52488[122], 3}, {NULL}};
const Segment SGI44623[] = {{&S52488[88], 5}, {NULL}};
const Segment SGI44624[] = {{&S52489[44], 19}, {NULL}};
const Segment SGI44625[] = {{&S52489[44], 30}, {NULL}};
const Segment SGI44626[] = {{&S52489[30], 6}, {NULL}};
const Segment SGI44627[] = {{&S52489[122], 3}, {NULL}};
const Segment SGI44628[] = {{&S52489[88], 5}, {NULL}};
const Segment SGI44629[] = {{&S52490[44], 19}, {NULL}};
const Segment SGI44630[] = {{&S52490[44], 30}, {NULL}};
const Segment SGI44631[] = {{&S52490[30], 6}, {NULL}};
const Segment SGI44632[] = {{&S52490[122], 3}, {NULL}};
const Segment SGI44633[] = {{&S52490[88], 5}, {NULL}};
const Segment SGI44634[] = {{&S52491[44], 19}, {NULL}};
const Segment SGI44635[] = {{&S52491[44], 30}, {NULL}};
const Segment SGI44636[] = {{&S52491[30], 6}, {NULL}};
const Segment SGI44637[] = {{&S52491[122], 3}, {NULL}};
const Segment SGI44638[] = {{&S52491[88], 5}, {NULL}};
const Segment SGI44639[] = {{&S52492[44], 19}, {NULL}};
const Segment SGI44640[] = {{&S52492[44], 30}, {NULL}};
const Segment SGI44641[] = {{&S52492[30], 6}, {NULL}};
const Segment SGI44642[] = {{&S52492[122], 3}, {NULL}};
const Segment SGI44643[] = {{&S52492[88], 5}, {NULL}};
const Segment SGI44644[] = {{&S52493[44], 19}, {NULL}};
const Segment SGI44645[] = {{&S52493[44], 30}, {NULL}};
const Segment SGI44646[] = {{&S52493[30], 6}, {NULL}};
const Segment SGI44647[] = {{&S52493[122], 3}, {NULL}};
const Segment SGI44648[] = {{&S52493[88], 5}, {NULL}};
const Segment SGI44649[] = {{&S66043[44], 10}, {NULL}};
const Segment SGI44650[] = {{&S66043[44], 21}, {NULL}};
const Segment SGI44651[] = {{&S66043[30], 6}, {NULL}};
const Segment SGI44652[] = {{&S66043[113], 3}, {NULL}};
const Segment SGI44653[] = {{&S66043[79], 5}, {NULL}};
const Segment SGI44654[] = {{&S66045[44], 12}, {NULL}};
const Segment SGI44655[] = {{&S66045[44], 23}, {NULL}};
const Segment SGI44656[] = {{&S66045[30], 6}, {NULL}};
const Segment SGI44657[] = {{&S66045[115], 3}, {NULL}};
const Segment SGI44658[] = {{&S66045[81], 5}, {NULL}};
const Segment SGI44659[] = {{&S66046[44], 12}, {NULL}};
const Segment SGI44660[] = {{&S66046[44], 23}, {NULL}};
const Segment SGI44661[] = {{&S66046[30], 6}, {NULL}};
const Segment SGI44662[] = {{&S66046[84], 4}, {NULL}};
const Segment SGI44663[] = {{&S66046[84], 3}, {NULL}};
const Segment SGI44664[] = {{&S66048[44], 12}, {NULL}};
const Segment SGI44665[] = {{&S66048[44], 23}, {NULL}};
const Segment SGI44666[] = {{&S66048[30], 6}, {NULL}};
const Segment SGI44667[] = {{&S66048[84], 6}, {NULL}};
const Segment SGI44668[] = {{&S66049[38], 12}, {NULL}};
const Segment SGI44669[] = {{&S66049[31], 6}, {NULL}};
const Segment SGI44670[] = {{&S53518[44], 6}, {NULL}};
const Segment SGI44671[] = {{&S53518[44], 17}, {NULL}};
const Segment SGI44672[] = {{&S53518[30], 6}, {NULL}};
const Segment SGI44673[] = {{&S53518[109], 3}, {NULL}};
const Segment SGI44674[] = {{&S53518[75], 5}, {NULL}};
const Segment SGI44675[] = {{&S55388[44], 13}, {NULL}};
const Segment SGI44676[] = {{&S55388[44], 24}, {NULL}};
const Segment SGI44677[] = {{&S55388[30], 6}, {NULL}};
const Segment SGI44678[] = {{&S55388[116], 3}, {NULL}};
const Segment SGI44679[] = {{&S55388[82], 5}, {NULL}};
const Segment SGI44680[] = {{&S55389[44], 13}, {NULL}};
const Segment SGI44681[] = {{&S55389[44], 24}, {NULL}};
const Segment SGI44682[] = {{&S55389[30], 6}, {NULL}};
const Segment SGI44683[] = {{&S55389[85], 4}, {NULL}};
const Segment SGI44684[] = {{&S55389[85], 3}, {NULL}};
const Segment SGI44685[] = {{&S55390[38], 13}, {NULL}};
const Segment SGI44686[] = {{&S55390[31], 6}, {NULL}};
const Segment SGI44687[] = {{&S55391[44], 13}, {NULL}};
const Segment SGI44688[] = {{&S55391[44], 30}, {NULL}};
const Segment SGI44689[] = {{&S55391[30], 6}, {NULL}};
const Segment SGI44690[] = {{&S55391[122], 3}, {NULL}};
const Segment SGI44691[] = {{&S55391[88], 5}, {NULL}};
const Segment SGI44692[] = {{&S53600[44], 9}, {NULL}};
const Segment SGI44693[] = {{&S53600[44], 26}, {NULL}};
const Segment SGI44694[] = {{&S53600[30], 6}, {NULL}};
const Segment SGI44695[] = {{&S53600[118], 3}, {NULL}};
const Segment SGI44696[] = {{&S53600[84], 5}, {NULL}};
const Segment SGI44697[] = {{&S53601[44], 9}, {NULL}};
const Segment SGI44698[] = {{&S53601[44], 26}, {NULL}};
const Segment SGI44699[] = {{&S53601[30], 6}, {NULL}};
const Segment SGI44700[] = {{&S53601[118], 3}, {NULL}};
const Segment SGI44701[] = {{&S53601[84], 5}, {NULL}};
const Segment SGI44702[] = {{&S53602[44], 10}, {NULL}};
const Segment SGI44703[] = {{&S53602[44], 27}, {NULL}};
const Segment SGI44704[] = {{&S53602[30], 6}, {NULL}};
const Segment SGI44705[] = {{&S53602[119], 3}, {NULL}};
const Segment SGI44706[] = {{&S53602[85], 5}, {NULL}};
const Segment SGI44707[] = {{&S53603[38], 9}, {NULL}};
const Segment SGI44708[] = {{&S53603[31], 6}, {NULL}};
const Segment SGI44709[] = {{&S53604[38], 9}, {NULL}};
const Segment SGI44710[] = {{&S53604[31], 6}, {NULL}};
const Segment SGI44711[] = {{&S66231[44], 10}, {NULL}};
const Segment SGI44712[] = {{&S66231[44], 22}, {NULL}};
const Segment SGI44713[] = {{&S66231[30], 6}, {NULL}};
const Segment SGI44714[] = {{&S66231[114], 3}, {NULL}};
const Segment SGI44715[] = {{&S66231[80], 5}, {NULL}};
const Segment SGI44716[] = {{&S61042[44], 10}, {NULL}};
const Segment SGI44717[] = {{&S61042[44], 27}, {NULL}};
const Segment SGI44718[] = {{&S61042[30], 6}, {NULL}};
const Segment SGI44719[] = {{&S61042[119], 3}, {NULL}};
const Segment SGI44720[] = {{&S61042[85], 5}, {NULL}};
const Segment SGI44721[] = {{&S52844[44], 11}, {NULL}};
const Segment SGI44722[] = {{&S52844[44], 22}, {NULL}};
const Segment SGI44723[] = {{&S52844[30], 6}, {NULL}};
const Segment SGI44724[] = {{&S52844[114], 3}, {NULL}};
const Segment SGI44725[] = {{&S52844[80], 5}, {NULL}};
const Segment SGI44726[] = {{&S52845[38], 11}, {NULL}};
const Segment SGI44727[] = {{&S52845[31], 6}, {NULL}};
const Segment SGI44728[] = {{&S52846[44], 11}, {NULL}};
const Segment SGI44729[] = {{&S52846[44], 22}, {NULL}};
const Segment SGI44730[] = {{&S52846[30], 6}, {NULL}};
const Segment SGI44731[] = {{&S52846[114], 3}, {NULL}};
const Segment SGI44732[] = {{&S52846[80], 5}, {NULL}};
const Segment SGI44733[] = {{&S52848[44], 11}, {NULL}};
const Segment SGI44734[] = {{&S52848[44], 22}, {NULL}};
const Segment SGI44735[] = {{&S52848[30], 6}, {NULL}};
const Segment SGI44736[] = {{&S52848[83], 4}, {NULL}};
const Segment SGI44737[] = {{&S52848[83], 3}, {NULL}};
const Segment SGI44738[] = {{&S52919[44], 10}, {NULL}};
const Segment SGI44739[] = {{&S52919[44], 30}, {NULL}};
const Segment SGI44740[] = {{&S52919[30], 6}, {NULL}};
const Segment SGI44741[] = {{&S52919[91], 4}, {NULL}};
const Segment SGI44742[] = {{&S52919[91], 3}, {NULL}};
const Segment SGI44743[] = {{&S52920[44], 10}, {NULL}};
const Segment SGI44744[] = {{&S52920[44], 30}, {NULL}};
const Segment SGI44745[] = {{&S52920[30], 6}, {NULL}};
const Segment SGI44746[] = {{&S52920[122], 3}, {NULL}};
const Segment SGI44747[] = {{&S52920[88], 5}, {NULL}};
const Segment SGI44748[] = {{&S52921[38], 10}, {NULL}};
const Segment SGI44749[] = {{&S52921[31], 6}, {NULL}};
const Segment SGI44750[] = {{&S52922[44], 10}, {NULL}};
const Segment SGI44751[] = {{&S52922[44], 30}, {NULL}};
const Segment SGI44752[] = {{&S52922[30], 6}, {NULL}};
const Segment SGI44753[] = {{&S52922[91], 6}, {NULL}};
const Segment SGI44754[] = {{&S53836[44], 10}, {NULL}};
const Segment SGI44755[] = {{&S53836[44], 31}, {NULL}};
const Segment SGI44756[] = {{&S53836[30], 6}, {NULL}};
const Segment SGI44757[] = {{&S53836[123], 3}, {NULL}};
const Segment SGI44758[] = {{&S53836[89], 5}, {NULL}};
const Segment SGI44759[] = {{&S53840[38], 10}, {NULL}};
const Segment SGI44760[] = {{&S53840[31], 6}, {NULL}};
const Segment SGI44761[] = {{&S60597[44], 10}, {NULL}};
const Segment SGI44762[] = {{&S60597[44], 20}, {NULL}};
const Segment SGI44763[] = {{&S60597[30], 6}, {NULL}};
const Segment SGI44764[] = {{&S60597[112], 3}, {NULL}};
const Segment SGI44765[] = {{&S60597[78], 5}, {NULL}};
const Segment SGI44766[] = {{&S53186[44], 13}, {NULL}};
const Segment SGI44767[] = {{&S53186[44], 30}, {NULL}};
const Segment SGI44768[] = {{&S53186[30], 6}, {NULL}};
const Segment SGI44769[] = {{&S53186[122], 3}, {NULL}};
const Segment SGI44770[] = {{&S53186[88], 5}, {NULL}};
const Segment SGI44771[] = {{&S53187[44], 13}, {NULL}};
const Segment SGI44772[] = {{&S53187[44], 30}, {NULL}};
const Segment SGI44773[] = {{&S53187[30], 6}, {NULL}};
const Segment SGI44774[] = {{&S53187[91], 4}, {NULL}};
const Segment SGI44775[] = {{&S53187[91], 3}, {NULL}};
const Segment SGI44776[] = {{&S53188[44], 13}, {NULL}};
const Segment SGI44777[] = {{&S53188[44], 30}, {NULL}};
const Segment SGI44778[] = {{&S53188[30], 6}, {NULL}};
const Segment SGI44779[] = {{&S53188[91], 6}, {NULL}};
const Segment SGI44780[] = {{&S53189[38], 13}, {NULL}};
const Segment SGI44781[] = {{&S53189[31], 6}, {NULL}};
const Segment SGI44782[] = {{&S53190[44], 30}, {NULL}};
const Segment SGI44783[] = {{&S53190[30], 6}, {NULL}};
const Segment SGI44784[] = {{&S53193[44], 8}, {NULL}};
const Segment SGI44785[] = {{&S53193[44], 25}, {NULL}};
const Segment SGI44786[] = {{&S53193[30], 6}, {NULL}};
const Segment SGI44787[] = {{&S53193[117], 3}, {NULL}};
const Segment SGI44788[] = {{&S53193[83], 5}, {NULL}};
const Segment SGI44789[] = {{&S56093[44], 8}, {NULL}};
const Segment SGI44790[] = {{&S56093[44], 25}, {NULL}};
const Segment SGI44791[] = {{&S56093[30], 6}, {NULL}};
const Segment SGI44792[] = {{&S56093[117], 3}, {NULL}};
const Segment SGI44793[] = {{&S56093[83], 5}, {NULL}};
const Segment SGI44794[] = {{&S56094[38], 8}, {NULL}};
const Segment SGI44795[] = {{&S56094[31], 6}, {NULL}};
const Segment SGI44796[] = {{&S60856[42], 14}, {NULL}};
const Segment SGI44797[] = {{&S60856[42], 25}, {NULL}};
const Segment SGI44798[] = {{&S60856[30], 4}, {NULL}};
const Segment SGI44799[] = {{&S60856[115], 3}, {NULL}};
const Segment SGI44800[] = {{&S60856[81], 5}, {NULL}};
const Segment SGI44801[] = {{&S53419[44], 10}, {NULL}};
const Segment SGI44802[] = {{&S53419[44], 21}, {NULL}};
const Segment SGI44803[] = {{&S53419[30], 6}, {NULL}};
const Segment SGI44804[] = {{&S53419[113], 3}, {NULL}};
const Segment SGI44805[] = {{&S53419[79], 5}, {NULL}};
const Segment SGI44806[] = {{&S53420[38], 10}, {NULL}};
const Segment SGI44807[] = {{&S53420[31], 6}, {NULL}};
const Segment SGI44808[] = {{&S53421[42], 10}, {NULL}};
const Segment SGI44809[] = {{&S53421[30], 6}, {NULL}};
const Segment SGI44810[] = {{&S53421[111], 3}, {NULL}};
const Segment SGI44811[] = {{&S53421[77], 5}, {NULL}};
const Segment SGI44812[] = {{&S53422[44], 10}, {NULL}};
const Segment SGI44813[] = {{&S53422[44], 21}, {NULL}};
const Segment SGI44814[] = {{&S53422[30], 6}, {NULL}};
const Segment SGI44815[] = {{&S53422[82], 6}, {NULL}};
const Segment SGI44816[] = {{&S53423[44], 10}, {NULL}};
const Segment SGI44817[] = {{&S53423[44], 34}, {NULL}};
const Segment SGI44818[] = {{&S53423[30], 6}, {NULL}};
const Segment SGI44819[] = {{&S53423[126], 3}, {NULL}};
const Segment SGI44820[] = {{&S53423[92], 5}, {NULL}};
const Segment SGI44821[] = {{&S53424[44], 10}, {NULL}};
const Segment SGI44822[] = {{&S53424[44], 56}, {NULL}};
const Segment SGI44823[] = {{&S53424[30], 6}, {NULL}};
const Segment SGI44824[] = {{&S53424[148], 3}, {NULL}};
const Segment SGI44825[] = {{&S53424[114], 5}, {NULL}};
const Segment SGI44826[] = {{&S53425[44], 10}, {NULL}};
const Segment SGI44827[] = {{&S53425[44], 46}, {NULL}};
const Segment SGI44828[] = {{&S53425[30], 6}, {NULL}};
const Segment SGI44829[] = {{&S53425[138], 3}, {NULL}};
const Segment SGI44830[] = {{&S53425[104], 5}, {NULL}};
const Segment SGI44831[] = {{&S56184[44], 8}, {NULL}};
const Segment SGI44832[] = {{&S56184[44], 22}, {NULL}};
const Segment SGI44833[] = {{&S56184[30], 6}, {NULL}};
const Segment SGI44834[] = {{&S56184[114], 3}, {NULL}};
const Segment SGI44835[] = {{&S56184[80], 5}, {NULL}};
const Segment SGI44836[] = {{&S56185[44], 8}, {NULL}};
const Segment SGI44837[] = {{&S56185[44], 22}, {NULL}};
const Segment SGI44838[] = {{&S56185[30], 6}, {NULL}};
const Segment SGI44839[] = {{&S56185[114], 3}, {NULL}};
const Segment SGI44840[] = {{&S56185[80], 5}, {NULL}};
const Segment SGI44841[] = {{&S56186[44], 8}, {NULL}};
const Segment SGI44842[] = {{&S56186[44], 22}, {NULL}};
const Segment SGI44843[] = {{&S56186[30], 6}, {NULL}};
const Segment SGI44844[] = {{&S56186[114], 3}, {NULL}};
const Segment SGI44845[] = {{&S56186[80], 5}, {NULL}};
const Segment SGI44846[] = {{&S67041[44], 12}, {NULL}};
const Segment SGI44847[] = {{&S67041[44], 23}, {NULL}};
const Segment SGI44848[] = {{&S67041[30], 6}, {NULL}};
const Segment SGI44849[] = {{&S67041[115], 3}, {NULL}};
const Segment SGI44850[] = {{&S67041[81], 5}, {NULL}};
const Segment SGI44851[] = {{&S56187[44], 8}, {NULL}};
const Segment SGI44852[] = {{&S56187[44], 22}, {NULL}};
const Segment SGI44853[] = {{&S56187[30], 6}, {NULL}};
const Segment SGI44854[] = {{&S56187[114], 3}, {NULL}};
const Segment SGI44855[] = {{&S56187[80], 5}, {NULL}};
const Segment SGI44856[] = {{&S67042[38], 12}, {NULL}};
const Segment SGI44857[] = {{&S67042[31], 6}, {NULL}};
const Segment SGI44858[] = {{&S56188[44], 8}, {NULL}};
const Segment SGI44859[] = {{&S56188[44], 22}, {NULL}};
const Segment SGI44860[] = {{&S56188[30], 6}, {NULL}};
const Segment SGI44861[] = {{&S67043[44], 12}, {NULL}};
const Segment SGI44862[] = {{&S56188[114], 3}, {NULL}};
const Segment SGI44863[] = {{&S67043[44], 23}, {NULL}};
const Segment SGI44864[] = {{&S67043[30], 6}, {NULL}};
const Segment SGI44865[] = {{&S56188[80], 5}, {NULL}};
const Segment SGI44866[] = {{&S67043[84], 4}, {NULL}};
const Segment SGI44867[] = {{&S67043[84], 3}, {NULL}};
const Segment SGI44868[] = {{&S56189[44], 8}, {NULL}};
const Segment SGI44869[] = {{&S56189[44], 22}, {NULL}};
const Segment SGI44870[] = {{&S56189[30], 6}, {NULL}};
const Segment SGI44871[] = {{&S56189[114], 3}, {NULL}};
const Segment SGI44872[] = {{&S56189[80], 5}, {NULL}};
const Segment SGI44873[] = {{&S56190[44], 8}, {NULL}};
const Segment SGI44874[] = {{&S56190[44], 22}, {NULL}};
const Segment SGI44875[] = {{&S56190[30], 6}, {NULL}};
const Segment SGI44876[] = {{&S56190[114], 3}, {NULL}};
const Segment SGI44877[] = {{&S56190[80], 5}, {NULL}};
const Segment SGI44878[] = {{&S56191[44], 8}, {NULL}};
const Segment SGI44879[] = {{&S56191[44], 22}, {NULL}};
const Segment SGI44880[] = {{&S56191[30], 6}, {NULL}};
const Segment SGI44881[] = {{&S56191[114], 3}, {NULL}};
const Segment SGI44882[] = {{&S56191[80], 5}, {NULL}};
const Segment SGI44883[] = {{&S56192[44], 8}, {NULL}};
const Segment SGI44884[] = {{&S56192[44], 22}, {NULL}};
const Segment SGI44885[] = {{&S56192[30], 6}, {NULL}};
const Segment SGI44886[] = {{&S56192[114], 3}, {NULL}};
const Segment SGI44887[] = {{&S56192[80], 5}, {NULL}};
const Segment SGI44888[] = {{&S56193[44], 8}, {NULL}};
const Segment SGI44889[] = {{&S56193[44], 22}, {NULL}};
const Segment SGI44890[] = {{&S56193[30], 6}, {NULL}};
const Segment SGI44891[] = {{&S56193[114], 3}, {NULL}};
const Segment SGI44892[] = {{&S56193[80], 5}, {NULL}};
const Segment SGI44893[] = {{&S54432[44], 7}, {NULL}};
const Segment SGI44894[] = {{&S54432[44], 24}, {NULL}};
const Segment SGI44895[] = {{&S54432[30], 6}, {NULL}};
const Segment SGI44896[] = {{&S54432[116], 3}, {NULL}};
const Segment SGI44897[] = {{&S54432[82], 5}, {NULL}};
const Segment SGI44898[] = {{&S53516[44], 6}, {NULL}};
const Segment SGI44899[] = {{&S53516[44], 17}, {NULL}};
const Segment SGI44900[] = {{&S53516[30], 6}, {NULL}};
const Segment SGI44901[] = {{&S53516[109], 3}, {NULL}};
const Segment SGI44902[] = {{&S53516[75], 5}, {NULL}};
const Segment SGI44903[] = {{&S53517[38], 6}, {NULL}};
const Segment SGI44904[] = {{&S53517[31], 6}, {NULL}};
const Segment SGI44905[] = {{&S53605[44], 14}, {NULL}};
const Segment SGI44906[] = {{&S53605[44], 31}, {NULL}};
const Segment SGI44907[] = {{&S53605[30], 6}, {NULL}};
const Segment SGI44908[] = {{&S53605[123], 3}, {NULL}};
const Segment SGI44909[] = {{&S53605[89], 5}, {NULL}};
const Segment SGI44910[] = {{&S53607[38], 14}, {NULL}};
const Segment SGI44911[] = {{&S53607[31], 6}, {NULL}};
const Segment SGI44912[] = {{&S53608[44], 14}, {NULL}};
const Segment SGI44913[] = {{&S53608[44], 31}, {NULL}};
const Segment SGI44914[] = {{&S53608[30], 6}, {NULL}};
const Segment SGI44915[] = {{&S53608[92], 4}, {NULL}};
const Segment SGI44916[] = {{&S53608[92], 3}, {NULL}};
const Segment SGI44917[] = {{&S61510[44], 17}, {NULL}};
const Segment SGI44918[] = {{&S61510[44], 33}, {NULL}};
const Segment SGI44919[] = {{&S61510[30], 6}, {NULL}};
const Segment SGI44920[] = {{&S61510[125], 3}, {NULL}};
const Segment SGI44921[] = {{&S61510[91], 5}, {NULL}};
const Segment SGI44922[] = {{&S53710[44], 6}, {NULL}};
const Segment SGI44923[] = {{&S53710[44], 22}, {NULL}};
const Segment SGI44924[] = {{&S53710[30], 6}, {NULL}};
const Segment SGI44925[] = {{&S53710[114], 3}, {NULL}};
const Segment SGI44926[] = {{&S53710[80], 5}, {NULL}};
const Segment SGI44927[] = {{&S67320[44], 14}, {NULL}};
const Segment SGI44928[] = {{&S67320[44], 25}, {NULL}};
const Segment SGI44929[] = {{&S67320[30], 6}, {NULL}};
const Segment SGI44930[] = {{&S67320[117], 3}, {NULL}};
const Segment SGI44931[] = {{&S67320[83], 5}, {NULL}};
const Segment SGI44932[] = {{&S67321[38], 11}, {NULL}};
const Segment SGI44933[] = {{&S67321[31], 6}, {NULL}};
const Segment SGI44934[] = {{&S67322[44], 11}, {NULL}};
const Segment SGI44935[] = {{&S67322[44], 22}, {NULL}};
const Segment SGI44936[] = {{&S67322[30], 6}, {NULL}};
const Segment SGI44937[] = {{&S67322[114], 3}, {NULL}};
const Segment SGI44938[] = {{&S67322[80], 5}, {NULL}};
const Segment SGI44939[] = {{&S61641[44], 10}, {NULL}};
const Segment SGI44940[] = {{&S61641[44], 21}, {NULL}};
const Segment SGI44941[] = {{&S61641[30], 6}, {NULL}};
const Segment SGI44942[] = {{&S61641[113], 3}, {NULL}};
const Segment SGI44943[] = {{&S61641[79], 5}, {NULL}};
const Segment SGI44944[] = {{&S61643[44], 10}, {NULL}};
const Segment SGI44945[] = {{&S61643[44], 21}, {NULL}};
const Segment SGI44946[] = {{&S61643[30], 6}, {NULL}};
const Segment SGI44947[] = {{&S61643[82], 6}, {NULL}};
const Segment SGI44948[] = {{&S53755[44], 10}, {NULL}};
const Segment SGI44949[] = {{&S53755[44], 27}, {NULL}};
const Segment SGI44950[] = {{&S53755[30], 6}, {NULL}};
const Segment SGI44951[] = {{&S53755[149], 4}, {NULL}};
const Segment SGI44952[] = {{&S53755[175], 3}, {NULL}};
const Segment SGI44953[] = {{&S53755[119], 3}, {&S53755[224], 5}, {NULL}};
const Segment SGI44954[] = {{&S53755[85], 5}, {NULL}};
const Segment SGI44955[] = {{&S53931[44], 9}, {NULL}};
const Segment SGI44956[] = {{&S53931[44], 20}, {NULL}};
const Segment SGI44957[] = {{&S53931[30], 6}, {NULL}};
const Segment SGI44958[] = {{&S53931[112], 3}, {NULL}};
const Segment SGI44959[] = {{&S53931[78], 5}, {NULL}};
const Segment SGI44960[] = {{&S53932[38], 9}, {NULL}};
const Segment SGI44961[] = {{&S53932[31], 6}, {NULL}};
const Segment SGI44962[] = {{&S61414[41], 12}, {NULL}};
const Segment SGI44963[] = {{&S61414[41], 23}, {NULL}};
const Segment SGI44964[] = {{&S61414[30], 4}, {NULL}};
const Segment SGI44965[] = {{&S61414[112], 3}, {NULL}};
const Segment SGI44966[] = {{&S61414[78], 5}, {NULL}};
const Segment SGI44967[] = {{&S53967[44], 14}, {NULL}};
const Segment SGI44968[] = {{&S53967[44], 26}, {NULL}};
const Segment SGI44969[] = {{&S53967[30], 6}, {NULL}};
const Segment SGI44970[] = {{&S53967[120], 3}, {NULL}};
const Segment SGI44971[] = {{&S53967[84], 6}, {NULL}};
const Segment SGI44972[] = {{&S53968[38], 14}, {NULL}};
const Segment SGI44973[] = {{&S53968[31], 6}, {NULL}};
const Segment SGI44974[] = {{&S67575[44], 5}, {NULL}};
const Segment SGI44975[] = {{&S67575[44], 46}, {NULL}};
const Segment SGI44976[] = {{&S67575[30], 6}, {NULL}};
const Segment SGI44977[] = {{&S67575[138], 3}, {NULL}};
const Segment SGI44978[] = {{&S67575[104], 5}, {NULL}};
const Segment SGI44979[] = {{&S67576[44], 5}, {NULL}};
const Segment SGI44980[] = {{&S67576[44], 45}, {NULL}};
const Segment SGI44981[] = {{&S67576[30], 6}, {NULL}};
const Segment SGI44982[] = {{&S67576[137], 3}, {NULL}};
const Segment SGI44983[] = {{&S67576[103], 5}, {NULL}};
const Segment SGI44984[] = {{&S67577[44], 5}, {NULL}};
const Segment SGI44985[] = {{&S67577[44], 44}, {NULL}};
const Segment SGI44986[] = {{&S67577[30], 6}, {NULL}};
const Segment SGI44987[] = {{&S67577[136], 3}, {NULL}};
const Segment SGI44988[] = {{&S67577[102], 5}, {NULL}};
const Segment SGI44989[] = {{&S67578[44], 5}, {NULL}};
const Segment SGI44990[] = {{&S67578[44], 43}, {NULL}};
const Segment SGI44991[] = {{&S67578[30], 6}, {NULL}};
const Segment SGI44992[] = {{&S67578[135], 3}, {NULL}};
const Segment SGI44993[] = {{&S67578[101], 5}, {NULL}};
const Segment SGI44994[] = {{&S67712[44], 11}, {NULL}};
const Segment SGI44995[] = {{&S67712[44], 23}, {NULL}};
const Segment SGI44996[] = {{&S67712[30], 6}, {NULL}};
const Segment SGI44997[] = {{&S67712[115], 3}, {NULL}};
const Segment SGI44998[] = {{&S67712[81], 5}, {NULL}};
const Segment SGI44999[] = {{&S67837[44], 18}, {NULL}};
const Segment SGI45000[] = {{&S67837[44], 30}, {NULL}};
const Segment SGI45001[] = {{&S67837[30], 6}, {NULL}};
const Segment SGI45002[] = {{&S67837[122], 3}, {NULL}};
const Segment SGI45003[] = {{&S67837[88], 5}, {NULL}};
const Segment SGI45004[] = {{&S61771[41], 10}, {NULL}};
const Segment SGI45005[] = {{&S61771[41], 21}, {NULL}};
const Segment SGI45006[] = {{&S61771[30], 4}, {NULL}};
const Segment SGI45007[] = {{&S61771[110], 3}, {NULL}};
const Segment SGI45008[] = {{&S61771[76], 5}, {NULL}};
const Segment SGI45009[] = {{&S54242[44], 18}, {NULL}};
const Segment SGI45010[] = {{&S54242[44], 31}, {NULL}};
const Segment SGI45011[] = {{&S54242[30], 6}, {NULL}};
const Segment SGI45012[] = {{&S54242[123], 3}, {NULL}};
const Segment SGI45013[] = {{&S54242[89], 5}, {NULL}};
const Segment SGI45014[] = {{&S54243[44], 18}, {NULL}};
const Segment SGI45015[] = {{&S54243[44], 31}, {NULL}};
const Segment SGI45016[] = {{&S54243[30], 6}, {NULL}};
const Segment SGI45017[] = {{&S54243[123], 3}, {NULL}};
const Segment SGI45018[] = {{&S54243[89], 5}, {NULL}};
const Segment SGI45019[] = {{&S54244[38], 18}, {NULL}};
const Segment SGI45020[] = {{&S54244[31], 6}, {NULL}};
const Segment SGI45021[] = {{&S54245[44], 18}, {NULL}};
const Segment SGI45022[] = {{&S54245[44], 31}, {NULL}};
const Segment SGI45023[] = {{&S55116[44], 8}, {NULL}};
const Segment SGI45024[] = {{&S54245[30], 6}, {NULL}};
const Segment SGI45025[] = {{&S55116[44], 19}, {NULL}};
const Segment SGI45026[] = {{&S55116[30], 6}, {NULL}};
const Segment SGI45027[] = {{&S54245[123], 3}, {NULL}};
const Segment SGI45028[] = {{&S54245[89], 5}, {NULL}};
const Segment SGI45029[] = {{&S55116[111], 3}, {NULL}};
const Segment SGI45030[] = {{&S55116[77], 5}, {NULL}};
const Segment SGI45031[] = {{&S54246[44], 18}, {NULL}};
const Segment SGI45032[] = {{&S55117[38], 8}, {NULL}};
const Segment SGI45033[] = {{&S54246[44], 31}, {NULL}};
const Segment SGI45034[] = {{&S55117[31], 6}, {NULL}};
const Segment SGI45035[] = {{&S54246[30], 6}, {NULL}};
const Segment SGI45036[] = {{&S54246[123], 3}, {NULL}};
const Segment SGI45037[] = {{&S54246[89], 5}, {NULL}};
const Segment SGI45038[] = {{&S62903[44], 7}, {NULL}};
const Segment SGI45039[] = {{&S62903[44], 24}, {NULL}};
const Segment SGI45040[] = {{&S62903[30], 6}, {NULL}};
const Segment SGI45041[] = {{&S62903[116], 3}, {NULL}};
const Segment SGI45042[] = {{&S62903[82], 5}, {NULL}};
const Segment SGI45043[] = {{&S62904[38], 7}, {NULL}};
const Segment SGI45044[] = {{&S62904[31], 6}, {NULL}};
const Segment SGI45045[] = {{&S62934[44], 11}, {NULL}};
const Segment SGI45046[] = {{&S62934[44], 28}, {NULL}};
const Segment SGI45047[] = {{&S62934[30], 6}, {NULL}};
const Segment SGI45048[] = {{&S62934[120], 3}, {NULL}};
const Segment SGI45049[] = {{&S62934[86], 5}, {NULL}};
const Segment SGI45050[] = {{&S62935[44], 10}, {NULL}};
const Segment SGI45051[] = {{&S62935[44], 27}, {NULL}};
const Segment SGI45052[] = {{&S62935[30], 6}, {NULL}};
const Segment SGI45053[] = {{&S62935[119], 3}, {NULL}};
const Segment SGI45054[] = {{&S62935[85], 5}, {NULL}};
const Segment SGI45055[] = {{&S62936[44], 10}, {NULL}};
const Segment SGI45056[] = {{&S62936[44], 27}, {NULL}};
const Segment SGI45057[] = {{&S62936[30], 6}, {NULL}};
const Segment SGI45058[] = {{&S62936[119], 3}, {NULL}};
const Segment SGI45059[] = {{&S62936[85], 5}, {NULL}};
const Segment SGI45060[] = {{&S54375[44], 8}, {NULL}};
const Segment SGI45061[] = {{&S54375[44], 19}, {NULL}};
const Segment SGI45062[] = {{&S54375[30], 6}, {NULL}};
const Segment SGI45063[] = {{&S54375[111], 3}, {NULL}};
const Segment SGI45064[] = {{&S54375[77], 5}, {NULL}};
const Segment SGI45065[] = {{&S54437[44], 7}, {NULL}};
const Segment SGI45066[] = {{&S54437[44], 24}, {NULL}};
const Segment SGI45067[] = {{&S54437[30], 6}, {NULL}};
const Segment SGI45068[] = {{&S54437[116], 3}, {NULL}};
const Segment SGI45069[] = {{&S54437[82], 5}, {NULL}};
const Segment SGI45070[] = {{&S54456[44], 8}, {NULL}};
const Segment SGI45071[] = {{&S54456[44], 24}, {NULL}};
const Segment SGI45072[] = {{&S54456[30], 6}, {NULL}};
const Segment SGI45073[] = {{&S54456[116], 3}, {NULL}};
const Segment SGI45074[] = {{&S54456[82], 5}, {NULL}};
const Segment SGI45075[] = {{&S54457[44], 8}, {NULL}};
const Segment SGI45076[] = {{&S54457[44], 24}, {NULL}};
const Segment SGI45077[] = {{&S54457[30], 6}, {NULL}};
const Segment SGI45078[] = {{&S54457[116], 3}, {NULL}};
const Segment SGI45079[] = {{&S54457[82], 5}, {NULL}};
const Segment SGI45080[] = {{&S54458[44], 8}, {NULL}};
const Segment SGI45081[] = {{&S54458[44], 24}, {NULL}};
const Segment SGI45082[] = {{&S54458[30], 6}, {NULL}};
const Segment SGI45083[] = {{&S54458[116], 3}, {NULL}};
const Segment SGI45084[] = {{&S54458[82], 5}, {NULL}};
const Segment SGI45085[] = {{&S54459[44], 8}, {NULL}};
const Segment SGI45086[] = {{&S54459[44], 24}, {NULL}};
const Segment SGI45087[] = {{&S54459[30], 6}, {NULL}};
const Segment SGI45088[] = {{&S54459[116], 3}, {NULL}};
const Segment SGI45089[] = {{&S54459[82], 5}, {NULL}};
const Segment SGI45090[] = {{&S54460[44], 8}, {NULL}};
const Segment SGI45091[] = {{&S54460[44], 24}, {NULL}};
const Segment SGI45092[] = {{&S54460[30], 6}, {NULL}};
const Segment SGI45093[] = {{&S54460[116], 3}, {NULL}};
const Segment SGI45094[] = {{&S54460[82], 5}, {NULL}};
const Segment SGI45095[] = {{&S54461[44], 8}, {NULL}};
const Segment SGI45096[] = {{&S54461[44], 24}, {NULL}};
const Segment SGI45097[] = {{&S54461[30], 6}, {NULL}};
const Segment SGI45098[] = {{&S54461[116], 3}, {NULL}};
const Segment SGI45099[] = {{&S54461[82], 5}, {NULL}};
const Segment SGI45100[] = {{&S54462[44], 8}, {NULL}};
const Segment SGI45101[] = {{&S54462[44], 24}, {NULL}};
const Segment SGI45102[] = {{&S54462[30], 6}, {NULL}};
const Segment SGI45103[] = {{&S54462[116], 3}, {NULL}};
const Segment SGI45104[] = {{&S54462[82], 5}, {NULL}};
const Segment SGI45105[] = {{&S54463[38], 8}, {NULL}};
const Segment SGI45106[] = {{&S54463[31], 6}, {NULL}};
const Segment SGI45107[] = {{&S57195[44], 15}, {NULL}};
const Segment SGI45108[] = {{&S57195[44], 26}, {NULL}};
const Segment SGI45109[] = {{&S57195[30], 6}, {NULL}};
const Segment SGI45110[] = {{&S57195[118], 3}, {NULL}};
const Segment SGI45111[] = {{&S57195[84], 5}, {NULL}};
const Segment SGI45112[] = {{&S68244[44], 12}, {NULL}};
const Segment SGI45113[] = {{&S68244[44], 23}, {NULL}};
const Segment SGI45114[] = {{&S68244[30], 6}, {NULL}};
const Segment SGI45115[] = {{&S68244[115], 3}, {NULL}};
const Segment SGI45116[] = {{&S68244[81], 5}, {NULL}};
const Segment SGI45117[] = {{&S68245[44], 23}, {NULL}};
const Segment SGI45118[] = {{&S68245[44], 34}, {NULL}};
const Segment SGI45119[] = {{&S68245[30], 6}, {NULL}};
const Segment SGI45120[] = {{&S68245[126], 3}, {NULL}};
const Segment SGI45121[] = {{&S68245[92], 5}, {NULL}};
const Segment SGI45122[] = {{&S68247[44], 22}, {NULL}};
const Segment SGI45123[] = {{&S68247[44], 33}, {NULL}};
const Segment SGI45124[] = {{&S68247[30], 6}, {NULL}};
const Segment SGI45125[] = {{&S68247[125], 3}, {NULL}};
const Segment SGI45126[] = {{&S68247[91], 5}, {NULL}};
const Segment SGI45127[] = {{&S54633[44], 10}, {NULL}};
const Segment SGI45128[] = {{&S54633[44], 21}, {NULL}};
const Segment SGI45129[] = {{&S54633[30], 6}, {NULL}};
const Segment SGI45130[] = {{&S54633[113], 3}, {NULL}};
const Segment SGI45131[] = {{&S54633[79], 5}, {NULL}};
const Segment SGI45132[] = {{&S54634[38], 10}, {NULL}};
const Segment SGI45133[] = {{&S54634[31], 6}, {NULL}};
const Segment SGI45134[] = {{&S68412[44], 9}, {NULL}};
const Segment SGI45135[] = {{&S68412[44], 21}, {NULL}};
const Segment SGI45136[] = {{&S68412[30], 6}, {NULL}};
const Segment SGI45137[] = {{&S68412[113], 3}, {NULL}};
const Segment SGI45138[] = {{&S68412[79], 5}, {NULL}};
const Segment SGI45139[] = {{&S68414[38], 9}, {NULL}};
const Segment SGI45140[] = {{&S68414[31], 6}, {NULL}};
const Segment SGI45141[] = {{&S54714[44], 14}, {NULL}};
const Segment SGI45142[] = {{&S54714[44], 31}, {NULL}};
const Segment SGI45143[] = {{&S54714[30], 6}, {NULL}};
const Segment SGI45144[] = {{&S54714[123], 3}, {NULL}};
const Segment SGI45145[] = {{&S54714[89], 5}, {NULL}};
const Segment SGI45146[] = {{&S54715[38], 14}, {NULL}};
const Segment SGI45147[] = {{&S54715[31], 6}, {NULL}};
const Segment SGI45148[] = {{&S54723[44], 16}, {NULL}};
const Segment SGI45149[] = {{&S54723[44], 27}, {NULL}};
const Segment SGI45150[] = {{&S54723[30], 6}, {NULL}};
const Segment SGI45151[] = {{&S54723[119], 3}, {NULL}};
const Segment SGI45152[] = {{&S54723[85], 5}, {NULL}};
const Segment SGI45153[] = {{&S54724[38], 16}, {NULL}};
const Segment SGI45154[] = {{&S54724[31], 6}, {NULL}};
const Segment SGI45155[] = {{&S54809[45], 10}, {NULL}};
const Segment SGI45156[] = {{&S54809[30], 6}, {NULL}};
const Segment SGI45157[] = {{&S54809[120], 3}, {NULL}};
const Segment SGI45158[] = {{&S54809[86], 5}, {NULL}};
const Segment SGI45159[] = {{&S54809[144], 13}, {NULL}};
const Segment SGI45160[] = {{&S68521[43], 23}, {NULL}};
const Segment SGI45161[] = {{&S68521[30], 6}, {NULL}};
const Segment SGI45162[] = {{&S68521[114], 3}, {NULL}};
const Segment SGI45163[] = {{&S68521[80], 5}, {NULL}};
const Segment SGI45164[] = {{&S68522[44], 7}, {NULL}};
const Segment SGI45165[] = {{&S68522[44], 24}, {NULL}};
const Segment SGI45166[] = {{&S68522[30], 6}, {NULL}};
const Segment SGI45167[] = {{&S68522[116], 3}, {NULL}};
const Segment SGI45168[] = {{&S68522[82], 5}, {NULL}};
const Segment SGI45169[] = {{&S68523[43], 22}, {NULL}};
const Segment SGI45170[] = {{&S68523[30], 6}, {NULL}};
const Segment SGI45171[] = {{&S68523[113], 3}, {NULL}};
const Segment SGI45172[] = {{&S68523[79], 5}, {NULL}};
const Segment SGI45173[] = {{&S55671[44], 23}, {NULL}};
const Segment SGI45174[] = {{&S55671[44], 40}, {NULL}};
const Segment SGI45175[] = {{&S55671[30], 6}, {NULL}};
const Segment SGI45176[] = {{&S55671[132], 3}, {NULL}};
const Segment SGI45177[] = {{&S55671[98], 5}, {NULL}};
const Segment SGI45178[] = {{&S55672[44], 23}, {NULL}};
const Segment SGI45179[] = {{&S55672[44], 40}, {NULL}};
const Segment SGI45180[] = {{&S55672[30], 6}, {NULL}};
const Segment SGI45181[] = {{&S55672[132], 3}, {NULL}};
const Segment SGI45182[] = {{&S55672[98], 5}, {NULL}};
const Segment SGI45183[] = {{&S55673[44], 24}, {NULL}};
const Segment SGI45184[] = {{&S55673[44], 41}, {NULL}};
const Segment SGI45185[] = {{&S55673[30], 6}, {NULL}};
const Segment SGI45186[] = {{&S55673[133], 3}, {NULL}};
const Segment SGI45187[] = {{&S55673[99], 5}, {NULL}};
const Segment SGI45188[] = {{&S55674[44], 23}, {NULL}};
const Segment SGI45189[] = {{&S55674[44], 40}, {NULL}};
const Segment SGI45190[] = {{&S55674[30], 6}, {NULL}};
const Segment SGI45191[] = {{&S55674[132], 3}, {NULL}};
const Segment SGI45192[] = {{&S55674[98], 5}, {NULL}};
const Segment SGI45193[] = {{&S55675[44], 24}, {NULL}};
const Segment SGI45194[] = {{&S55675[44], 41}, {NULL}};
const Segment SGI45195[] = {{&S55675[30], 6}, {NULL}};
const Segment SGI45196[] = {{&S55675[133], 3}, {NULL}};
const Segment SGI45197[] = {{&S55675[99], 5}, {NULL}};
const Segment SGI45198[] = {{&S55676[44], 29}, {NULL}};
const Segment SGI45199[] = {{&S55676[44], 46}, {NULL}};
const Segment SGI45200[] = {{&S55676[30], 6}, {NULL}};
const Segment SGI45201[] = {{&S55676[138], 3}, {NULL}};
const Segment SGI45202[] = {{&S55676[104], 5}, {NULL}};
const Segment SGI45203[] = {{&S55677[44], 24}, {NULL}};
const Segment SGI45204[] = {{&S55677[44], 41}, {NULL}};
const Segment SGI45205[] = {{&S55677[30], 6}, {NULL}};
const Segment SGI45206[] = {{&S55677[133], 3}, {NULL}};
const Segment SGI45207[] = {{&S55677[99], 5}, {NULL}};
const Segment SGI45208[] = {{&S55678[44], 24}, {NULL}};
const Segment SGI45209[] = {{&S55678[44], 41}, {NULL}};
const Segment SGI45210[] = {{&S55678[30], 6}, {NULL}};
const Segment SGI45211[] = {{&S55678[133], 3}, {NULL}};
const Segment SGI45212[] = {{&S55678[99], 5}, {NULL}};
const Segment SGI45213[] = {{&S57650[6], 4}, {NULL}};
const Segment SGI45214[] = {{&S57650[85], 5}, {NULL}};
const Segment SGI45215[] = {{&S55748[44], 35}, {NULL}};
const Segment SGI45216[] = {{&S55748[44], 46}, {NULL}};
const Segment SGI45217[] = {{&S55748[30], 6}, {&S55748[52], 38}, {NULL}};
const Segment SGI45218[] = {{&S55748[138], 3}, {NULL}};
const Segment SGI45219[] = {{&S55748[104], 5}, {NULL}};
const Segment SGI45220[] = {{&S57707[6], 4}, {NULL}};
const Segment SGI45221[] = {{&S57707[78], 5}, {NULL}};
const Segment SGI45222[] = {{&S68896[44], 14}, {NULL}};
const Segment SGI45223[] = {{&S68896[44], 25}, {NULL}};
const Segment SGI45224[] = {{&S68896[30], 6}, {NULL}};
const Segment SGI45225[] = {{&S68896[117], 3}, {NULL}};
const Segment SGI45226[] = {{&S68896[83], 5}, {NULL}};
const Segment SGI45227[] = {{&S68900[44], 4}, {NULL}};
const Segment SGI45228[] = {{&S68900[44], 15}, {NULL}};
const Segment SGI45229[] = {{&S68900[30], 6}, {NULL}};
const Segment SGI45230[] = {{&S68900[107], 3}, {NULL}};
const Segment SGI45231[] = {{&S68900[73], 5}, {NULL}};
const Segment SGI45232[] = {{&S68901[44], 11}, {NULL}};
const Segment SGI45233[] = {{&S68901[44], 22}, {NULL}};
const Segment SGI45234[] = {{&S68901[30], 6}, {NULL}};
const Segment SGI45235[] = {{&S68901[114], 3}, {NULL}};
const Segment SGI45236[] = {{&S68901[80], 5}, {NULL}};
const Segment SGI45237[] = {{&S69083[44], 11}, {NULL}};
const Segment SGI45238[] = {{&S69083[44], 23}, {NULL}};
const Segment SGI45239[] = {{&S69083[30], 6}, {NULL}};
const Segment SGI45240[] = {{&S69083[115], 3}, {NULL}};
const Segment SGI45241[] = {{&S69083[81], 5}, {NULL}};
const Segment SGI45242[] = {{&S69085[38], 11}, {NULL}};
const Segment SGI45243[] = {{&S69085[31], 6}, {NULL}};
const Segment SGI45244[] = {{&S58050[44], 9}, {NULL}};
const Segment SGI45245[] = {{&S58050[44], 20}, {NULL}};
const Segment SGI45246[] = {{&S58050[30], 6}, {NULL}};
const Segment SGI45247[] = {{&S58050[112], 3}, {NULL}};
const Segment SGI45248[] = {{&S58050[78], 5}, {NULL}};
const Segment SGI45249[] = {{&S58051[38], 9}, {NULL}};
const Segment SGI45250[] = {{&S58051[31], 6}, {NULL}};
const Segment SGI45251[] = {{&S58052[44], 9}, {NULL}};
const Segment SGI45252[] = {{&S58052[44], 24}, {NULL}};
const Segment SGI45253[] = {{&S58052[30], 6}, {NULL}};
const Segment SGI45254[] = {{&S55253[44], 10}, {NULL}};
const Segment SGI45255[] = {{&S55253[44], 22}, {NULL}};
const Segment SGI45256[] = {{&S55253[30], 6}, {NULL}};
const Segment SGI45257[] = {{&S55253[114], 3}, {NULL}};
const Segment SGI45258[] = {{&S55253[80], 5}, {NULL}};
const Segment SGI45259[] = {{&S55254[44], 11}, {NULL}};
const Segment SGI45260[] = {{&S55254[44], 23}, {NULL}};
const Segment SGI45261[] = {{&S55254[30], 6}, {NULL}};
const Segment SGI45262[] = {{&S55254[115], 3}, {NULL}};
const Segment SGI45263[] = {{&S55254[81], 5}, {NULL}};
const Segment SGI45264[] = {{&S55255[38], 10}, {NULL}};
const Segment SGI45265[] = {{&S55255[31], 6}, {NULL}};
const Segment SGI45266[] = {{&S55257[45], 10}, {NULL}};
const Segment SGI45267[] = {{&S55257[30], 6}, {NULL}};
const Segment SGI45268[] = {{&S55257[120], 3}, {NULL}};
const Segment SGI45269[] = {{&S55257[86], 5}, {NULL}};
const Segment SGI45270[] = {{&S55257[144], 13}, {NULL}};
const Segment SGI45271[] = {{&S69188[44], 9}, {NULL}};
const Segment SGI45272[] = {{&S69188[44], 20}, {NULL}};
const Segment SGI45273[] = {{&S69188[30], 6}, {NULL}};
const Segment SGI45274[] = {{&S69188[112], 3}, {NULL}};
const Segment SGI45275[] = {{&S69188[78], 5}, {NULL}};
const Segment SGI45276[] = {{&S58317[44], 14}, {NULL}};
const Segment SGI45277[] = {{&S58317[44], 25}, {NULL}};
const Segment SGI45278[] = {{&S58317[30], 6}, {NULL}};
const Segment SGI45279[] = {{&S58317[117], 3}, {NULL}};
const Segment SGI45280[] = {{&S58317[83], 5}, {NULL}};
const Segment SGI45281[] = {{&S58320[38], 14}, {NULL}};
const Segment SGI45282[] = {{&S58320[31], 6}, {NULL}};
const Segment SGI45283[] = {{&S69502[44], 11}, {NULL}};
const Segment SGI45284[] = {{&S69502[44], 28}, {NULL}};
const Segment SGI45285[] = {{&S69502[30], 6}, {NULL}};
const Segment SGI45286[] = {{&S69502[120], 3}, {NULL}};
const Segment SGI45287[] = {{&S69502[86], 5}, {NULL}};
const Segment SGI45288[] = {{&S55618[44], 10}, {NULL}};
const Segment SGI45289[] = {{&S55618[44], 21}, {NULL}};
const Segment SGI45290[] = {{&S55618[30], 6}, {NULL}};
const Segment SGI45291[] = {{&S55618[113], 3}, {NULL}};
const Segment SGI45292[] = {{&S55618[79], 5}, {NULL}};
const Segment SGI45293[] = {{&S55619[42], 10}, {NULL}};
const Segment SGI45294[] = {{&S55619[30], 6}, {NULL}};
const Segment SGI45295[] = {{&S55619[111], 3}, {NULL}};
const Segment SGI45296[] = {{&S55619[77], 5}, {NULL}};
const Segment SGI45297[] = {{&S55620[38], 10}, {NULL}};
const Segment SGI45298[] = {{&S55620[31], 6}, {NULL}};
const Segment SGI45299[] = {{&S55621[44], 10}, {NULL}};
const Segment SGI45300[] = {{&S55621[44], 24}, {NULL}};
const Segment SGI45301[] = {{&S55621[30], 6}, {NULL}};
const Segment SGI45302[] = {{&S55621[130], 3}, {NULL}};
const Segment SGI45303[] = {{&S55621[96], 5}, {NULL}};
const Segment SGI45304[] = {{&S55622[44], 10}, {NULL}};
const Segment SGI45305[] = {{&S55622[44], 40}, {NULL}};
const Segment SGI45306[] = {{&S55622[30], 6}, {NULL}};
const Segment SGI45307[] = {{&S55622[132], 3}, {NULL}};
const Segment SGI45308[] = {{&S55622[98], 5}, {NULL}};
const Segment SGI45309[] = {{&S69574[44], 15}, {NULL}};
const Segment SGI45310[] = {{&S69574[44], 27}, {NULL}};
const Segment SGI45311[] = {{&S69574[30], 6}, {NULL}};
const Segment SGI45312[] = {{&S69574[119], 3}, {NULL}};
const Segment SGI45313[] = {{&S69574[85], 5}, {NULL}};
const Segment SGI45314[] = {{&S55643[44], 14}, {NULL}};
const Segment SGI45315[] = {{&S55643[44], 25}, {NULL}};
const Segment SGI45316[] = {{&S55643[30], 6}, {NULL}};
const Segment SGI45317[] = {{&S55643[117], 3}, {NULL}};
const Segment SGI45318[] = {{&S55643[83], 5}, {NULL}};
const Segment SGI45319[] = {{&S69633[44], 9}, {NULL}};
const Segment SGI45320[] = {{&S69633[44], 21}, {NULL}};
const Segment SGI45321[] = {{&S69633[30], 6}, {NULL}};
const Segment SGI45322[] = {{&S69633[113], 3}, {NULL}};
const Segment SGI45323[] = {{&S69633[79], 5}, {NULL}};
const Segment SGI45324[] = {{&S69634[38], 9}, {NULL}};
const Segment SGI45325[] = {{&S69634[31], 6}, {NULL}};
const Segment SGI45326[] = {{&S55749[44], 35}, {NULL}};
const Segment SGI45327[] = {{&S55749[44], 46}, {NULL}};
const Segment SGI45328[] = {{&S55749[30], 6}, {&S55749[52], 38}, {NULL}};
const Segment SGI45329[] = {{&S55749[138], 3}, {NULL}};
const Segment SGI45330[] = {{&S55749[104], 5}, {NULL}};
const Segment SGI45331[] = {{&S69691[44], 9}, {NULL}};
const Segment SGI45332[] = {{&S69691[44], 20}, {NULL}};
const Segment SGI45333[] = {{&S69691[30], 6}, {NULL}};
const Segment SGI45334[] = {{&S69691[112], 3}, {NULL}};
const Segment SGI45335[] = {{&S69691[78], 5}, {NULL}};
const Segment SGI45336[] = {{&S69692[38], 9}, {NULL}};
const Segment SGI45337[] = {{&S69692[31], 6}, {NULL}};
const Segment SGI45338[] = {{&S55775[42], 10}, {NULL}};
const Segment SGI45339[] = {{&S55775[30], 6}, {NULL}};
const Segment SGI45340[] = {{&S55775[111], 3}, {NULL}};
const Segment SGI45341[] = {{&S55775[77], 5}, {NULL}};
const Segment SGI45342[] = {{&S55776[44], 10}, {NULL}};
const Segment SGI45343[] = {{&S55776[44], 21}, {NULL}};
const Segment SGI45344[] = {{&S55776[30], 6}, {NULL}};
const Segment SGI45345[] = {{&S55776[113], 3}, {NULL}};
const Segment SGI45346[] = {{&S55776[79], 5}, {NULL}};
const Segment SGI45347[] = {{&S55777[44], 10}, {NULL}};
const Segment SGI45348[] = {{&S55777[44], 21}, {NULL}};
const Segment SGI45349[] = {{&S55777[30], 6}, {NULL}};
const Segment SGI45350[] = {{&S55777[113], 3}, {NULL}};
const Segment SGI45351[] = {{&S55777[79], 5}, {NULL}};
const Segment SGI45352[] = {{&S55778[38], 10}, {NULL}};
const Segment SGI45353[] = {{&S55778[31], 6}, {NULL}};
const Segment SGI45354[] = {{&S55779[38], 10}, {NULL}};
const Segment SGI45355[] = {{&S55779[31], 6}, {NULL}};
const Segment SGI45356[] = {{&S55780[42], 10}, {NULL}};
const Segment SGI45357[] = {{&S55780[30], 6}, {NULL}};
const Segment SGI45358[] = {{&S55780[111], 3}, {NULL}};
const Segment SGI45359[] = {{&S55780[77], 5}, {NULL}};
const Segment SGI45360[] = {{&S55781[44], 10}, {NULL}};
const Segment SGI45361[] = {{&S55781[44], 44}, {NULL}};
const Segment SGI45362[] = {{&S55781[30], 6}, {NULL}};
const Segment SGI45363[] = {{&S55781[136], 3}, {NULL}};
const Segment SGI45364[] = {{&S55781[102], 5}, {NULL}};
const Segment SGI45365[] = {{&S55788[44], 7}, {NULL}};
const Segment SGI45366[] = {{&S55788[44], 24}, {NULL}};
const Segment SGI45367[] = {{&S55788[30], 6}, {NULL}};
const Segment SGI45368[] = {{&S55788[116], 3}, {NULL}};
const Segment SGI45369[] = {{&S55788[82], 5}, {NULL}};
const Segment SGI45370[] = {{&S55789[44], 14}, {NULL}};
const Segment SGI45371[] = {{&S55789[44], 25}, {NULL}};
const Segment SGI45372[] = {{&S55789[30], 6}, {NULL}};
const Segment SGI45373[] = {{&S55789[117], 3}, {NULL}};
const Segment SGI45374[] = {{&S55789[83], 5}, {NULL}};
const Segment SGI45375[] = {{&S64281[44], 7}, {NULL}};
const Segment SGI45376[] = {{&S64281[44], 24}, {NULL}};
const Segment SGI45377[] = {{&S64281[30], 6}, {NULL}};
const Segment SGI45378[] = {{&S64281[116], 3}, {NULL}};
const Segment SGI45379[] = {{&S64281[82], 5}, {NULL}};
const Segment SGI45380[] = {{&S55790[38], 7}, {NULL}};
const Segment SGI45381[] = {{&S55790[31], 6}, {NULL}};
const Segment SGI45382[] = {{&S64282[38], 7}, {NULL}};
const Segment SGI45383[] = {{&S64282[31], 6}, {NULL}};
const Segment SGI45384[] = {{&S55791[44], 7}, {NULL}};
const Segment SGI45385[] = {{&S55791[44], 24}, {NULL}};
const Segment SGI45386[] = {{&S55791[30], 6}, {NULL}};
const Segment SGI45387[] = {{&S55791[85], 4}, {NULL}};
const Segment SGI45388[] = {{&S55791[85], 3}, {NULL}};
const Segment SGI45389[] = {{&S63431[41], 9}, {NULL}};
const Segment SGI45390[] = {{&S63431[41], 20}, {NULL}};
const Segment SGI45391[] = {{&S63431[30], 4}, {NULL}};
const Segment SGI45392[] = {{&S63431[108], 3}, {NULL}};
const Segment SGI45393[] = {{&S63431[74], 5}, {NULL}};
const Segment SGI45394[] = {{&S58659[44], 15}, {NULL}};
const Segment SGI45395[] = {{&S58659[44], 25}, {NULL}};
const Segment SGI45396[] = {{&S58659[30], 6}, {NULL}};
const Segment SGI45397[] = {{&S58659[117], 3}, {NULL}};
const Segment SGI45398[] = {{&S58659[83], 5}, {NULL}};
const Segment SGI45399[] = {{&S55964[44], 14}, {NULL}};
const Segment SGI45400[] = {{&S55964[44], 35}, {NULL}};
const Segment SGI45401[] = {{&S55964[30], 6}, {NULL}};
const Segment SGI45402[] = {{&S55964[96], 6}, {NULL}};
const Segment SGI45403[] = {{&S55965[44], 14}, {NULL}};
const Segment SGI45404[] = {{&S55965[44], 35}, {NULL}};
const Segment SGI45405[] = {{&S55965[30], 6}, {NULL}};
const Segment SGI45406[] = {{&S55965[127], 3}, {NULL}};
const Segment SGI45407[] = {{&S55965[93], 5}, {NULL}};
const Segment SGI45408[] = {{&S55966[44], 35}, {NULL}};
const Segment SGI45409[] = {{&S55966[30], 6}, {NULL}};
const Segment SGI45410[] = {{&S55976[38], 14}, {NULL}};
const Segment SGI45411[] = {{&S55976[31], 6}, {NULL}};
const Segment SGI45412[] = {{&S63707[41], 12}, {NULL}};
const Segment SGI45413[] = {{&S63707[41], 23}, {NULL}};
const Segment SGI45414[] = {{&S63707[30], 4}, {NULL}};
const Segment SGI45415[] = {{&S63707[111], 3}, {NULL}};
const Segment SGI45416[] = {{&S63707[77], 5}, {NULL}};
const Segment SGI45417[] = {{&S63708[41], 13}, {NULL}};
const Segment SGI45418[] = {{&S63708[41], 24}, {NULL}};
const Segment SGI45419[] = {{&S63708[30], 4}, {NULL}};
const Segment SGI45420[] = {{&S63708[113], 3}, {NULL}};
const Segment SGI45421[] = {{&S63708[79], 5}, {NULL}};
const Segment SGI45422[] = {{&S59001[44], 10}, {NULL}};
const Segment SGI45423[] = {{&S59001[44], 27}, {NULL}};
const Segment SGI45424[] = {{&S59001[30], 6}, {NULL}};
const Segment SGI45425[] = {{&S59001[119], 3}, {NULL}};
const Segment SGI45426[] = {{&S59001[85], 5}, {NULL}};
const Segment SGI45427[] = {{&S56124[44], 10}, {NULL}};
const Segment SGI45428[] = {{&S59002[38], 10}, {NULL}};
const Segment SGI45429[] = {{&S56124[44], 27}, {NULL}};
const Segment SGI45430[] = {{&S59002[31], 6}, {NULL}};
const Segment SGI45431[] = {{&S56124[30], 6}, {NULL}};
const Segment SGI45432[] = {{&S56124[85], 6}, {NULL}};
const Segment SGI45433[] = {{&S56124[112], 13}, {NULL}};
const Segment SGI45434[] = {{&S56195[45], 10}, {NULL}};
const Segment SGI45435[] = {{&S56195[30], 6}, {NULL}};
const Segment SGI45436[] = {{&S56195[120], 3}, {NULL}};
const Segment SGI45437[] = {{&S56195[86], 5}, {NULL}};
const Segment SGI45438[] = {{&S56195[144], 13}, {NULL}};
const Segment SGI45439[] = {{&S64801[44], 12}, {NULL}};
const Segment SGI45440[] = {{&S64801[44], 29}, {NULL}};
const Segment SGI45441[] = {{&S64801[30], 6}, {NULL}};
const Segment SGI45442[] = {{&S64801[121], 3}, {NULL}};
const Segment SGI45443[] = {{&S64801[87], 5}, {NULL}};
const Segment SGI45444[] = {{&S64802[44], 10}, {NULL}};
const Segment SGI45445[] = {{&S64802[44], 27}, {NULL}};
const Segment SGI45446[] = {{&S64802[30], 6}, {NULL}};
const Segment SGI45447[] = {{&S64802[119], 3}, {NULL}};
const Segment SGI45448[] = {{&S64802[85], 5}, {NULL}};
const Segment SGI45449[] = {{&S70260[44], 10}, {NULL}};
const Segment SGI45450[] = {{&S70260[44], 22}, {NULL}};
const Segment SGI45451[] = {{&S70260[30], 6}, {NULL}};
const Segment SGI45452[] = {{&S70260[114], 3}, {NULL}};
const Segment SGI45453[] = {{&S70260[80], 5}, {NULL}};
const Segment SGI45454[] = {{&S70262[42], 10}, {NULL}};
const Segment SGI45455[] = {{&S70262[30], 6}, {NULL}};
const Segment SGI45456[] = {{&S70262[112], 3}, {NULL}};
const Segment SGI45457[] = {{&S70262[78], 5}, {NULL}};
const Segment SGI45458[] = {{&S59239[44], 10}, {NULL}};
const Segment SGI45459[] = {{&S59239[44], 22}, {NULL}};
const Segment SGI45460[] = {{&S59239[30], 6}, {NULL}};
const Segment SGI45461[] = {{&S59239[114], 3}, {NULL}};
const Segment SGI45462[] = {{&S59239[80], 5}, {NULL}};
const Segment SGI45463[] = {{&S59240[38], 10}, {NULL}};
const Segment SGI45464[] = {{&S59240[31], 6}, {NULL}};
const Segment SGI45465[] = {{&S56883[44], 7}, {NULL}};
const Segment SGI45466[] = {{&S56883[44], 17}, {NULL}};
const Segment SGI45467[] = {{&S56883[30], 6}, {NULL}};
const Segment SGI45468[] = {{&S56883[109], 3}, {NULL}};
const Segment SGI45469[] = {{&S56883[75], 5}, {NULL}};
const Segment SGI45470[] = {{&S56884[44], 7}, {NULL}};
const Segment SGI45471[] = {{&S56884[44], 20}, {NULL}};
const Segment SGI45472[] = {{&S56884[30], 6}, {NULL}};
const Segment SGI45473[] = {{&S56884[112], 3}, {NULL}};
const Segment SGI45474[] = {{&S56884[78], 5}, {NULL}};
const Segment SGI45475[] = {{&S59313[44], 14}, {NULL}};
const Segment SGI45476[] = {{&S59313[44], 31}, {NULL}};
const Segment SGI45477[] = {{&S59313[30], 6}, {NULL}};
const Segment SGI45478[] = {{&S59313[123], 3}, {NULL}};
const Segment SGI45479[] = {{&S59313[89], 5}, {NULL}};
const Segment SGI45480[] = {{&S56898[44], 11}, {NULL}};
const Segment SGI45481[] = {{&S56898[44], 28}, {NULL}};
const Segment SGI45482[] = {{&S56898[30], 6}, {NULL}};
const Segment SGI45483[] = {{&S56898[120], 3}, {NULL}};
const Segment SGI45484[] = {{&S56898[86], 5}, {NULL}};
const Segment SGI45485[] = {{&S56899[38], 11}, {NULL}};
const Segment SGI45486[] = {{&S56899[31], 6}, {NULL}};
const Segment SGI45487[] = {{&S56902[44], 10}, {NULL}};
const Segment SGI45488[] = {{&S56902[44], 21}, {NULL}};
const Segment SGI45489[] = {{&S56902[30], 6}, {NULL}};
const Segment SGI45490[] = {{&S56902[113], 3}, {NULL}};
const Segment SGI45491[] = {{&S56902[79], 5}, {NULL}};
const Segment SGI45492[] = {{&S59593[44], 9}, {NULL}};
const Segment SGI45493[] = {{&S59593[44], 20}, {NULL}};
const Segment SGI45494[] = {{&S59593[30], 6}, {NULL}};
const Segment SGI45495[] = {{&S59593[112], 3}, {NULL}};
const Segment SGI45496[] = {{&S59593[78], 5}, {NULL}};
const Segment SGI45497[] = {{&S59594[44], 9}, {NULL}};
const Segment SGI45498[] = {{&S59594[44], 24}, {NULL}};
const Segment SGI45499[] = {{&S59594[30], 6}, {NULL}};
const Segment SGI45500[] = {{&S70781[44], 10}, {NULL}};
const Segment SGI45501[] = {{&S70781[44], 21}, {NULL}};
const Segment SGI45502[] = {{&S70781[30], 6}, {NULL}};
const Segment SGI45503[] = {{&S70781[113], 3}, {NULL}};
const Segment SGI45504[] = {{&S70781[79], 5}, {NULL}};
const Segment SGI45505[] = {{&S70783[44], 10}, {NULL}};
const Segment SGI45506[] = {{&S70783[44], 21}, {NULL}};
const Segment SGI45507[] = {{&S70783[30], 6}, {NULL}};
const Segment SGI45508[] = {{&S70783[82], 4}, {NULL}};
const Segment SGI45509[] = {{&S70783[82], 3}, {NULL}};
const Segment SGI45510[] = {{&S70786[44], 10}, {NULL}};
const Segment SGI45511[] = {{&S70786[44], 21}, {NULL}};
const Segment SGI45512[] = {{&S70786[30], 6}, {NULL}};
const Segment SGI45513[] = {{&S70786[82], 6}, {NULL}};
const Segment SGI45514[] = {{&S65385[44], 4}, {NULL}};
const Segment SGI45515[] = {{&S65385[44], 21}, {NULL}};
const Segment SGI45516[] = {{&S65385[30], 6}, {NULL}};
const Segment SGI45517[] = {{&S65385[82], 4}, {NULL}};
const Segment SGI45518[] = {{&S65385[82], 3}, {NULL}};
const Segment SGI45519[] = {{&S65390[44], 4}, {NULL}};
const Segment SGI45520[] = {{&S65390[44], 21}, {NULL}};
const Segment SGI45521[] = {{&S65390[30], 6}, {NULL}};
const Segment SGI45522[] = {{&S65390[82], 6}, {NULL}};
const Segment SGI45523[] = {{&S57196[44], 15}, {NULL}};
const Segment SGI45524[] = {{&S57196[44], 26}, {NULL}};
const Segment SGI45525[] = {{&S57196[30], 6}, {NULL}};
const Segment SGI45526[] = {{&S57196[118], 3}, {NULL}};
const Segment SGI45527[] = {{&S57196[84], 5}, {NULL}};
const Segment SGI45528[] = {{&S57198[44], 6}, {NULL}};
const Segment SGI45529[] = {{&S57198[44], 17}, {NULL}};
const Segment SGI45530[] = {{&S57198[30], 6}, {NULL}};
const Segment SGI45531[] = {{&S57198[109], 3}, {NULL}};
const Segment SGI45532[] = {{&S57198[75], 5}, {NULL}};
const Segment SGI45533[] = {{&S59727[44], 9}, {NULL}};
const Segment SGI45534[] = {{&S59727[44], 20}, {NULL}};
const Segment SGI45535[] = {{&S59727[30], 6}, {NULL}};
const Segment SGI45536[] = {{&S59727[112], 3}, {NULL}};
const Segment SGI45537[] = {{&S59727[78], 5}, {NULL}};
const Segment SGI45538[] = {{&S59728[38], 9}, {NULL}};
const Segment SGI45539[] = {{&S59728[31], 6}, {NULL}};
const Segment SGI45540[] = {{&S59729[44], 9}, {NULL}};
const Segment SGI45541[] = {{&S59729[44], 24}, {NULL}};
const Segment SGI45542[] = {{&S59729[30], 6}, {NULL}};
const Segment SGI45543[] = {{&S64602[41], 4}, {NULL}};
const Segment SGI45544[] = {{&S64602[41], 15}, {NULL}};
const Segment SGI45545[] = {{&S64602[30], 4}, {NULL}};
const Segment SGI45546[] = {{&S64602[104], 3}, {NULL}};
const Segment SGI45547[] = {{&S64602[70], 5}, {NULL}};
const Segment SGI45548[] = {{&S65694[6], 4}, {NULL}};
const Segment SGI45549[] = {{&S65694[82], 5}, {NULL}};
const Segment SGI45550[] = {{&S56808[44], 14}, {NULL}};
const Segment SGI45551[] = {{&S56808[44], 26}, {NULL}};
const Segment SGI45552[] = {{&S56808[30], 6}, {NULL}};
const Segment SGI45553[] = {{&S56808[120], 3}, {NULL}};
const Segment SGI45554[] = {{&S56808[84], 6}, {NULL}};
const Segment SGI45555[] = {{&S56809[38], 14}, {NULL}};
const Segment SGI45556[] = {{&S56809[31], 6}, {NULL}};
const Segment SGI45557[] = {{&S64656[42], 9}, {NULL}};
const Segment SGI45558[] = {{&S64656[42], 26}, {NULL}};
const Segment SGI45559[] = {{&S64656[30], 4}, {NULL}};
const Segment SGI45560[] = {{&S64656[116], 3}, {NULL}};
const Segment SGI45561[] = {{&S64656[82], 5}, {NULL}};
const Segment SGI45562[] = {{&S64657[36], 9}, {NULL}};
const Segment SGI45563[] = {{&S64657[31], 4}, {NULL}};
const Segment SGI45564[] = {{&S59992[44], 7}, {NULL}};
const Segment SGI45565[] = {{&S59992[44], 23}, {NULL}};
const Segment SGI45566[] = {{&S59992[30], 6}, {NULL}};
const Segment SGI45567[] = {{&S59992[115], 3}, {NULL}};
const Segment SGI45568[] = {{&S59992[81], 5}, {NULL}};
const Segment SGI45569[] = {{&S56876[44], 7}, {NULL}};
const Segment SGI45570[] = {{&S56876[44], 17}, {NULL}};
const Segment SGI45571[] = {{&S56876[30], 6}, {NULL}};
const Segment SGI45572[] = {{&S56876[109], 3}, {NULL}};
const Segment SGI45573[] = {{&S56876[75], 5}, {NULL}};
const Segment SGI45574[] = {{&S56879[44], 7}, {NULL}};
const Segment SGI45575[] = {{&S56879[44], 20}, {NULL}};
const Segment SGI45576[] = {{&S56879[30], 6}, {NULL}};
const Segment SGI45577[] = {{&S56879[112], 3}, {NULL}};
const Segment SGI45578[] = {{&S56879[78], 5}, {NULL}};
const Segment SGI45579[] = {{&S60024[44], 10}, {NULL}};
const Segment SGI45580[] = {{&S60024[44], 27}, {NULL}};
const Segment SGI45581[] = {{&S60024[30], 6}, {NULL}};
const Segment SGI45582[] = {{&S60024[119], 3}, {NULL}};
const Segment SGI45583[] = {{&S60024[85], 5}, {NULL}};
const Segment SGI45584[] = {{&S60025[38], 10}, {NULL}};
const Segment SGI45585[] = {{&S60025[31], 6}, {NULL}};
const Segment SGI45586[] = {{&S60026[42], 10}, {NULL}};
const Segment SGI45587[] = {{&S60026[30], 6}, {NULL}};
const Segment SGI45588[] = {{&S60026[117], 3}, {NULL}};
const Segment SGI45589[] = {{&S60026[83], 5}, {NULL}};
const Segment SGI45590[] = {{&S71199[44], 11}, {NULL}};
const Segment SGI45591[] = {{&S71199[44], 22}, {NULL}};
const Segment SGI45592[] = {{&S71199[30], 6}, {NULL}};
const Segment SGI45593[] = {{&S71199[114], 3}, {NULL}};
const Segment SGI45594[] = {{&S71199[80], 5}, {NULL}};
const Segment SGI45595[] = {{&S60062[44], 10}, {NULL}};
const Segment SGI45596[] = {{&S60062[44], 21}, {NULL}};
const Segment SGI45597[] = {{&S60062[30], 6}, {NULL}};
const Segment SGI45598[] = {{&S60062[113], 3}, {NULL}};
const Segment SGI45599[] = {{&S60062[79], 5}, {NULL}};
const Segment SGI45600[] = {{&S65829[44], 10}, {NULL}};
const Segment SGI45601[] = {{&S65829[44], 27}, {NULL}};
const Segment SGI45602[] = {{&S65829[30], 6}, {NULL}};
const Segment SGI45603[] = {{&S65829[119], 3}, {NULL}};
const Segment SGI45604[] = {{&S65829[85], 5}, {NULL}};
const Segment SGI45605[] = {{&S60065[44], 14}, {NULL}};
const Segment SGI45606[] = {{&S65830[38], 10}, {NULL}};
const Segment SGI45607[] = {{&S60065[44], 25}, {NULL}};
const Segment SGI45608[] = {{&S65830[31], 6}, {NULL}};
const Segment SGI45609[] = {{&S60065[30], 6}, {NULL}};
const Segment SGI45610[] = {{&S60065[117], 3}, {NULL}};
const Segment SGI45611[] = {{&S60065[83], 5}, {NULL}};
const Segment SGI45612[] = {{&S60069[44], 7}, {NULL}};
const Segment SGI45613[] = {{&S60069[44], 18}, {NULL}};
const Segment SGI45614[] = {{&S60069[30], 6}, {NULL}};
const Segment SGI45615[] = {{&S60069[110], 3}, {NULL}};
const Segment SGI45616[] = {{&S60069[76], 5}, {NULL}};
const Segment SGI45617[] = {{&S56953[44], 19}, {NULL}};
const Segment SGI45618[] = {{&S56953[44], 30}, {NULL}};
const Segment SGI45619[] = {{&S56953[30], 6}, {NULL}};
const Segment SGI45620[] = {{&S56953[122], 3}, {NULL}};
const Segment SGI45621[] = {{&S56953[88], 5}, {NULL}};
const Segment SGI45622[] = {{&S56954[38], 19}, {NULL}};
const Segment SGI45623[] = {{&S56954[31], 6}, {NULL}};
const Segment SGI45624[] = {{&S56955[44], 10}, {NULL}};
const Segment SGI45625[] = {{&S56955[44], 21}, {NULL}};
const Segment SGI45626[] = {{&S56955[30], 6}, {NULL}};
const Segment SGI45627[] = {{&S56955[113], 3}, {NULL}};
const Segment SGI45628[] = {{&S56955[79], 5}, {NULL}};
const Segment SGI45629[] = {{&S65299[6], 4}, {NULL}};
const Segment SGI45630[] = {{&S65299[82], 5}, {NULL}};
const Segment SGI45631[] = {{&S57503[44], 7}, {NULL}};
const Segment SGI45632[] = {{&S57503[44], 18}, {NULL}};
const Segment SGI45633[] = {{&S57503[30], 6}, {NULL}};
const Segment SGI45634[] = {{&S57503[110], 3}, {NULL}};
const Segment SGI45635[] = {{&S57503[76], 5}, {NULL}};
const Segment SGI45636[] = {{&S57504[44], 17}, {NULL}};
const Segment SGI45637[] = {{&S57504[44], 27}, {NULL}};
const Segment SGI45638[] = {{&S57504[30], 6}, {NULL}};
const Segment SGI45639[] = {{&S57504[119], 3}, {NULL}};
const Segment SGI45640[] = {{&S57504[85], 5}, {NULL}};
const Segment SGI45641[] = {{&S56985[44], 14}, {NULL}};
const Segment SGI45642[] = {{&S56985[44], 26}, {NULL}};
const Segment SGI45643[] = {{&S56985[30], 6}, {NULL}};
const Segment SGI45644[] = {{&S57505[44], 17}, {NULL}};
const Segment SGI45645[] = {{&S57505[44], 27}, {NULL}};
const Segment SGI45646[] = {{&S57505[30], 6}, {NULL}};
const Segment SGI45647[] = {{&S56985[118], 3}, {NULL}};
const Segment SGI45648[] = {{&S56985[84], 5}, {NULL}};
const Segment SGI45649[] = {{&S57505[119], 3}, {NULL}};
const Segment SGI45650[] = {{&S57505[85], 5}, {NULL}};
const Segment SGI45651[] = {{&S56986[44], 18}, {NULL}};
const Segment SGI45652[] = {{&S56986[44], 29}, {NULL}};
const Segment SGI45653[] = {{&S56986[30], 6}, {NULL}};
const Segment SGI45654[] = {{&S57506[38], 17}, {NULL}};
const Segment SGI45655[] = {{&S57506[31], 6}, {NULL}};
const Segment SGI45656[] = {{&S56986[121], 3}, {NULL}};
const Segment SGI45657[] = {{&S56986[87], 5}, {NULL}};
const Segment SGI45658[] = {{&S56987[44], 14}, {NULL}};
const Segment SGI45659[] = {{&S56987[44], 25}, {NULL}};
const Segment SGI45660[] = {{&S56987[30], 6}, {NULL}};
const Segment SGI45661[] = {{&S56987[117], 3}, {NULL}};
const Segment SGI45662[] = {{&S56987[83], 5}, {NULL}};
const Segment SGI45663[] = {{&S56988[44], 19}, {NULL}};
const Segment SGI45664[] = {{&S56988[44], 30}, {NULL}};
const Segment SGI45665[] = {{&S56988[30], 6}, {NULL}};
const Segment SGI45666[] = {{&S56988[122], 3}, {NULL}};
const Segment SGI45667[] = {{&S56988[88], 5}, {NULL}};
const Segment SGI45668[] = {{&S56989[44], 10}, {NULL}};
const Segment SGI45669[] = {{&S56989[44], 21}, {NULL}};
const Segment SGI45670[] = {{&S56989[30], 6}, {NULL}};
const Segment SGI45671[] = {{&S56989[113], 3}, {NULL}};
const Segment SGI45672[] = {{&S56989[79], 5}, {NULL}};
const Segment SGI45673[] = {{&S57515[44], 8}, {NULL}};
const Segment SGI45674[] = {{&S57515[44], 35}, {NULL}};
const Segment SGI45675[] = {{&S57515[30], 6}, {NULL}};
const Segment SGI45676[] = {{&S57515[127], 3}, {NULL}};
const Segment SGI45677[] = {{&S57515[93], 5}, {NULL}};
const Segment SGI45678[] = {{&S57516[44], 8}, {NULL}};
const Segment SGI45679[] = {{&S57516[44], 36}, {NULL}};
const Segment SGI45680[] = {{&S57516[30], 6}, {NULL}};
const Segment SGI45681[] = {{&S57516[128], 3}, {NULL}};
const Segment SGI45682[] = {{&S57516[94], 5}, {NULL}};
const Segment SGI45683[] = {{&S57517[44], 8}, {NULL}};
const Segment SGI45684[] = {{&S57517[44], 36}, {NULL}};
const Segment SGI45685[] = {{&S57517[30], 6}, {NULL}};
const Segment SGI45686[] = {{&S57517[128], 3}, {NULL}};
const Segment SGI45687[] = {{&S57517[94], 5}, {NULL}};
const Segment SGI45688[] = {{&S57518[44], 8}, {NULL}};
const Segment SGI45689[] = {{&S57518[44], 36}, {NULL}};
const Segment SGI45690[] = {{&S57518[30], 6}, {NULL}};
const Segment SGI45691[] = {{&S57518[128], 3}, {NULL}};
const Segment SGI45692[] = {{&S57518[94], 5}, {NULL}};
const Segment SGI45693[] = {{&S57519[44], 8}, {NULL}};
const Segment SGI45694[] = {{&S57519[44], 35}, {NULL}};
const Segment SGI45695[] = {{&S57519[30], 6}, {NULL}};
const Segment SGI45696[] = {{&S57519[127], 3}, {NULL}};
const Segment SGI45697[] = {{&S57519[93], 5}, {NULL}};
const Segment SGI45698[] = {{&S57520[38], 8}, {NULL}};
const Segment SGI45699[] = {{&S57520[31], 6}, {NULL}};
const Segment SGI45700[] = {{&S57086[44], 9}, {NULL}};
const Segment SGI45701[] = {{&S57086[44], 20}, {NULL}};
const Segment SGI45702[] = {{&S57086[30], 6}, {NULL}};
const Segment SGI45703[] = {{&S57086[112], 3}, {NULL}};
const Segment SGI45704[] = {{&S57086[78], 5}, {NULL}};
const Segment SGI45705[] = {{&S57087[44], 8}, {NULL}};
const Segment SGI45706[] = {{&S57087[44], 19}, {NULL}};
const Segment SGI45707[] = {{&S57087[30], 6}, {NULL}};
const Segment SGI45708[] = {{&S57087[111], 3}, {NULL}};
const Segment SGI45709[] = {{&S57087[77], 5}, {NULL}};
const Segment SGI45710[] = {{&S60295[44], 6}, {NULL}};
const Segment SGI45711[] = {{&S60295[44], 17}, {NULL}};
const Segment SGI45712[] = {{&S60295[30], 6}, {NULL}};
const Segment SGI45713[] = {{&S60295[109], 3}, {NULL}};
const Segment SGI45714[] = {{&S60295[75], 5}, {NULL}};
const Segment SGI45715[] = {{&S60296[44], 6}, {NULL}};
const Segment SGI45716[] = {{&S60296[44], 17}, {NULL}};
const Segment SGI45717[] = {{&S60296[30], 6}, {NULL}};
const Segment SGI45718[] = {{&S60296[109], 3}, {NULL}};
const Segment SGI45719[] = {{&S60296[75], 5}, {NULL}};
const Segment SGI45720[] = {{&S60299[38], 6}, {NULL}};
const Segment SGI45721[] = {{&S60299[31], 6}, {NULL}};
const Segment SGI45722[] = {{&S60312[44], 8}, {NULL}};
const Segment SGI45723[] = {{&S60312[44], 19}, {NULL}};
const Segment SGI45724[] = {{&S60312[30], 6}, {NULL}};
const Segment SGI45725[] = {{&S60312[111], 3}, {NULL}};
const Segment SGI45726[] = {{&S60312[77], 5}, {NULL}};
const Segment SGI45727[] = {{&S60313[44], 8}, {NULL}};
const Segment SGI45728[] = {{&S60313[44], 19}, {NULL}};
const Segment SGI45729[] = {{&S60313[30], 6}, {NULL}};
const Segment SGI45730[] = {{&S60313[111], 3}, {NULL}};
const Segment SGI45731[] = {{&S60313[77], 5}, {NULL}};
const Segment SGI45732[] = {{&S60314[44], 8}, {NULL}};
const Segment SGI45733[] = {{&S60314[44], 19}, {NULL}};
const Segment SGI45734[] = {{&S60314[30], 6}, {NULL}};
const Segment SGI45735[] = {{&S60314[111], 3}, {NULL}};
const Segment SGI45736[] = {{&S60314[77], 5}, {NULL}};
const Segment SGI45737[] = {{&S60315[44], 8}, {NULL}};
const Segment SGI45738[] = {{&S60315[44], 19}, {NULL}};
const Segment SGI45739[] = {{&S60315[30], 6}, {NULL}};
const Segment SGI45740[] = {{&S60315[111], 3}, {NULL}};
const Segment SGI45741[] = {{&S60315[77], 5}, {NULL}};
const Segment SGI45742[] = {{&S60316[44], 8}, {NULL}};
const Segment SGI45743[] = {{&S60316[44], 19}, {NULL}};
const Segment SGI45744[] = {{&S60316[30], 6}, {NULL}};
const Segment SGI45745[] = {{&S60316[111], 3}, {NULL}};
const Segment SGI45746[] = {{&S60316[77], 5}, {NULL}};
const Segment SGI45747[] = {{&S60317[44], 8}, {NULL}};
const Segment SGI45748[] = {{&S60317[44], 19}, {NULL}};
const Segment SGI45749[] = {{&S60317[30], 6}, {NULL}};
const Segment SGI45750[] = {{&S60317[77], 6}, {NULL}};
const Segment SGI45751[] = {{&S60318[44], 8}, {NULL}};
const Segment SGI45752[] = {{&S60318[44], 19}, {NULL}};
const Segment SGI45753[] = {{&S60318[30], 6}, {NULL}};
const Segment SGI45754[] = {{&S60318[111], 3}, {NULL}};
const Segment SGI45755[] = {{&S60318[77], 5}, {NULL}};
const Segment SGI45756[] = {{&S60319[38], 8}, {NULL}};
const Segment SGI45757[] = {{&S60319[31], 6}, {NULL}};
const Segment SGI45758[] = {{&S57136[44], 7}, {NULL}};
const Segment SGI45759[] = {{&S57136[44], 20}, {NULL}};
const Segment SGI45760[] = {{&S57136[30], 6}, {NULL}};
const Segment SGI45761[] = {{&S57136[112], 3}, {NULL}};
const Segment SGI45762[] = {{&S57136[78], 5}, {NULL}};
const Segment SGI45763[] = {{&S57137[44], 7}, {NULL}};
const Segment SGI45764[] = {{&S57137[44], 18}, {NULL}};
const Segment SGI45765[] = {{&S57137[30], 6}, {NULL}};
const Segment SGI45766[] = {{&S57137[110], 3}, {NULL}};
const Segment SGI45767[] = {{&S57137[76], 5}, {NULL}};
const Segment SGI45768[] = {{&S57138[44], 7}, {NULL}};
const Segment SGI45769[] = {{&S57138[44], 20}, {NULL}};
const Segment SGI45770[] = {{&S57138[30], 6}, {NULL}};
const Segment SGI45771[] = {{&S57138[112], 3}, {NULL}};
const Segment SGI45772[] = {{&S57138[78], 5}, {NULL}};
const Segment SGI45773[] = {{&S57139[44], 7}, {NULL}};
const Segment SGI45774[] = {{&S57139[44], 21}, {NULL}};
const Segment SGI45775[] = {{&S57139[30], 6}, {NULL}};
const Segment SGI45776[] = {{&S57139[113], 3}, {NULL}};
const Segment SGI45777[] = {{&S57139[79], 5}, {NULL}};
const Segment SGI45778[] = {{&S57140[44], 7}, {NULL}};
const Segment SGI45779[] = {{&S57140[44], 21}, {NULL}};
const Segment SGI45780[] = {{&S57140[30], 6}, {NULL}};
const Segment SGI45781[] = {{&S57140[113], 3}, {NULL}};
const Segment SGI45782[] = {{&S57140[79], 5}, {NULL}};
const Segment SGI45783[] = {{&S57141[38], 7}, {NULL}};
const Segment SGI45784[] = {{&S57141[31], 6}, {NULL}};
const Segment SGI45785[] = {{&S57142[44], 7}, {NULL}};
const Segment SGI45786[] = {{&S57142[44], 20}, {NULL}};
const Segment SGI45787[] = {{&S57142[30], 6}, {NULL}};
const Segment SGI45788[] = {{&S57142[112], 3}, {NULL}};
const Segment SGI45789[] = {{&S57142[78], 5}, {NULL}};
const Segment SGI45790[] = {{&S57143[44], 7}, {NULL}};
const Segment SGI45791[] = {{&S57143[44], 20}, {NULL}};
const Segment SGI45792[] = {{&S57143[30], 6}, {NULL}};
const Segment SGI45793[] = {{&S57143[112], 3}, {NULL}};
const Segment SGI45794[] = {{&S57143[78], 5}, {NULL}};
const Segment SGI45795[] = {{&S57649[6], 4}, {NULL}};
const Segment SGI45796[] = {{&S57649[85], 5}, {NULL}};
const Segment SGI45797[] = {{&S57708[6], 4}, {NULL}};
const Segment SGI45798[] = {{&S57708[78], 5}, {NULL}};
const Segment SGI45799[] = {{&S57197[44], 6}, {NULL}};
const Segment SGI45800[] = {{&S57197[44], 17}, {NULL}};
const Segment SGI45801[] = {{&S57197[30], 6}, {NULL}};
const Segment SGI45802[] = {{&S57197[109], 3}, {NULL}};
const Segment SGI45803[] = {{&S57197[75], 5}, {NULL}};
const Segment SGI45804[] = {{&S57199[44], 6}, {NULL}};
const Segment SGI45805[] = {{&S57199[44], 17}, {NULL}};
const Segment SGI45806[] = {{&S57199[30], 6}, {NULL}};
const Segment SGI45807[] = {{&S57199[109], 3}, {NULL}};
const Segment SGI45808[] = {{&S57199[75], 5}, {NULL}};
const Segment SGI45809[] = {{&S57200[44], 13}, {NULL}};
const Segment SGI45810[] = {{&S57200[44], 29}, {NULL}};
const Segment SGI45811[] = {{&S57200[30], 6}, {NULL}};
const Segment SGI45812[] = {{&S57200[121], 3}, {NULL}};
const Segment SGI45813[] = {{&S57200[87], 5}, {NULL}};
const Segment SGI45814[] = {{&S57210[44], 10}, {NULL}};
const Segment SGI45815[] = {{&S57210[44], 27}, {NULL}};
const Segment SGI45816[] = {{&S57210[30], 6}, {NULL}};
const Segment SGI45817[] = {{&S57210[149], 4}, {NULL}};
const Segment SGI45818[] = {{&S57210[175], 3}, {NULL}};
const Segment SGI45819[] = {{&S57210[119], 3}, {&S57210[224], 5}, {NULL}};
const Segment SGI45820[] = {{&S57210[85], 5}, {NULL}};
const Segment SGI45821[] = {{&S71755[44], 13}, {NULL}};
const Segment SGI45822[] = {{&S71755[44], 25}, {NULL}};
const Segment SGI45823[] = {{&S71755[30], 6}, {NULL}};
const Segment SGI45824[] = {{&S71755[117], 3}, {NULL}};
const Segment SGI45825[] = {{&S71755[83], 5}, {NULL}};
const Segment SGI45826[] = {{&S65780[44], 10}, {NULL}};
const Segment SGI45827[] = {{&S65780[44], 22}, {NULL}};
const Segment SGI45828[] = {{&S65780[30], 6}, {NULL}};
const Segment SGI45829[] = {{&S65780[114], 3}, {NULL}};
const Segment SGI45830[] = {{&S65780[80], 5}, {NULL}};
const Segment SGI45831[] = {{&S71791[44], 6}, {NULL}};
const Segment SGI45832[] = {{&S71791[44], 18}, {NULL}};
const Segment SGI45833[] = {{&S71791[30], 6}, {NULL}};
const Segment SGI45834[] = {{&S71791[110], 3}, {NULL}};
const Segment SGI45835[] = {{&S71791[76], 5}, {NULL}};
const Segment SGI45836[] = {{&S57911[44], 10}, {NULL}};
const Segment SGI45837[] = {{&S57911[44], 27}, {NULL}};
const Segment SGI45838[] = {{&S57911[30], 6}, {NULL}};
const Segment SGI45839[] = {{&S57911[119], 3}, {NULL}};
const Segment SGI45840[] = {{&S57911[85], 5}, {NULL}};
const Segment SGI45841[] = {{&S57912[38], 10}, {NULL}};
const Segment SGI45842[] = {{&S57912[31], 6}, {NULL}};
const Segment SGI45843[] = {{&S57913[42], 10}, {NULL}};
const Segment SGI45844[] = {{&S57913[30], 6}, {NULL}};
const Segment SGI45845[] = {{&S57913[117], 3}, {NULL}};
const Segment SGI45846[] = {{&S57913[83], 5}, {NULL}};
const Segment SGI45847[] = {{&S57914[44], 10}, {NULL}};
const Segment SGI45848[] = {{&S57914[44], 27}, {NULL}};
const Segment SGI45849[] = {{&S57914[30], 6}, {NULL}};
const Segment SGI45850[] = {{&S57914[88], 6}, {NULL}};
const Segment SGI45851[] = {{&S62079[6], 4}, {NULL}};
const Segment SGI45852[] = {{&S62079[85], 5}, {NULL}};
const Segment SGI45853[] = {{&S60739[44], 10}, {NULL}};
const Segment SGI45854[] = {{&S60739[44], 21}, {NULL}};
const Segment SGI45855[] = {{&S60739[30], 6}, {NULL}};
const Segment SGI45856[] = {{&S60739[113], 3}, {NULL}};
const Segment SGI45857[] = {{&S60739[79], 5}, {NULL}};
const Segment SGI45858[] = {{&S60740[44], 10}, {NULL}};
const Segment SGI45859[] = {{&S60740[44], 21}, {NULL}};
const Segment SGI45860[] = {{&S60740[30], 6}, {NULL}};
const Segment SGI45861[] = {{&S60740[113], 3}, {NULL}};
const Segment SGI45862[] = {{&S60740[79], 5}, {NULL}};
const Segment SGI45863[] = {{&S60741[44], 10}, {NULL}};
const Segment SGI45864[] = {{&S60741[44], 21}, {NULL}};
const Segment SGI45865[] = {{&S60741[30], 6}, {NULL}};
const Segment SGI45866[] = {{&S60741[113], 3}, {NULL}};
const Segment SGI45867[] = {{&S60741[79], 5}, {NULL}};
const Segment SGI45868[] = {{&S60742[44], 10}, {NULL}};
const Segment SGI45869[] = {{&S60742[44], 21}, {NULL}};
const Segment SGI45870[] = {{&S60742[30], 6}, {NULL}};
const Segment SGI45871[] = {{&S60742[113], 3}, {NULL}};
const Segment SGI45872[] = {{&S60742[79], 5}, {NULL}};
const Segment SGI45873[] = {{&S60743[44], 10}, {NULL}};
const Segment SGI45874[] = {{&S60743[44], 21}, {NULL}};
const Segment SGI45875[] = {{&S60743[30], 6}, {NULL}};
const Segment SGI45876[] = {{&S60743[113], 3}, {NULL}};
const Segment SGI45877[] = {{&S60743[79], 5}, {NULL}};
const Segment SGI45878[] = {{&S60744[38], 10}, {NULL}};
const Segment SGI45879[] = {{&S60744[31], 6}, {NULL}};
const Segment SGI45880[] = {{&S60745[44], 10}, {NULL}};
const Segment SGI45881[] = {{&S60745[44], 21}, {NULL}};
const Segment SGI45882[] = {{&S60745[30], 6}, {NULL}};
const Segment SGI45883[] = {{&S60745[113], 3}, {NULL}};
const Segment SGI45884[] = {{&S60745[79], 5}, {NULL}};
const Segment SGI45885[] = {{&S60746[44], 10}, {NULL}};
const Segment SGI45886[] = {{&S60746[44], 21}, {NULL}};
const Segment SGI45887[] = {{&S60746[30], 6}, {NULL}};
const Segment SGI45888[] = {{&S60746[113], 3}, {NULL}};
const Segment SGI45889[] = {{&S60746[79], 5}, {NULL}};
const Segment SGI45890[] = {{&S57497[44], 10}, {NULL}};
const Segment SGI45891[] = {{&S57497[44], 27}, {NULL}};
const Segment SGI45892[] = {{&S57497[30], 6}, {NULL}};
const Segment SGI45893[] = {{&S57497[119], 3}, {NULL}};
const Segment SGI45894[] = {{&S57497[85], 5}, {NULL}};
const Segment SGI45895[] = {{&S71853[44], 43}, {NULL}};
const Segment SGI45896[] = {{&S71853[30], 6}, {NULL}};
const Segment SGI45897[] = {{&S71853[119], 5}, {NULL}};
const Segment SGI45898[] = {{&S71854[44], 39}, {NULL}};
const Segment SGI45899[] = {{&S71854[30], 6}, {NULL}};
const Segment SGI45900[] = {{&S71854[115], 5}, {NULL}};
const Segment SGI45901[] = {{&S58184[44], 8}, {NULL}};
const Segment SGI45902[] = {{&S58184[44], 19}, {NULL}};
const Segment SGI45903[] = {{&S58184[30], 6}, {NULL}};
const Segment SGI45904[] = {{&S71947[44], 12}, {NULL}};
const Segment SGI45905[] = {{&S71947[44], 52}, {NULL}};
const Segment SGI45906[] = {{&S71947[30], 6}, {NULL}};
const Segment SGI45907[] = {{&S58184[111], 3}, {NULL}};
const Segment SGI45908[] = {{&S58184[77], 5}, {NULL}};
const Segment SGI45909[] = {{&S71947[144], 3}, {NULL}};
const Segment SGI45910[] = {{&S71947[110], 5}, {NULL}};
const Segment SGI45911[] = {{&S58185[38], 8}, {NULL}};
const Segment SGI45912[] = {{&S58185[31], 6}, {NULL}};
const Segment SGI45913[] = {{&S57560[44], 8}, {NULL}};
const Segment SGI45914[] = {{&S57560[44], 19}, {NULL}};
const Segment SGI45915[] = {{&S57560[30], 6}, {NULL}};
const Segment SGI45916[] = {{&S57560[111], 3}, {NULL}};
const Segment SGI45917[] = {{&S57560[77], 5}, {NULL}};
const Segment SGI45918[] = {{&S57561[44], 8}, {NULL}};
const Segment SGI45919[] = {{&S57561[44], 19}, {NULL}};
const Segment SGI45920[] = {{&S57561[30], 6}, {NULL}};
const Segment SGI45921[] = {{&S57561[111], 3}, {NULL}};
const Segment SGI45922[] = {{&S57561[77], 5}, {NULL}};
const Segment SGI45923[] = {{&S57562[44], 8}, {NULL}};
const Segment SGI45924[] = {{&S57562[44], 19}, {NULL}};
const Segment SGI45925[] = {{&S57562[30], 6}, {NULL}};
const Segment SGI45926[] = {{&S57562[111], 3}, {NULL}};
const Segment SGI45927[] = {{&S57562[77], 5}, {NULL}};
const Segment SGI45928[] = {{&S57563[44], 8}, {NULL}};
const Segment SGI45929[] = {{&S57563[44], 19}, {NULL}};
const Segment SGI45930[] = {{&S57563[30], 6}, {NULL}};
const Segment SGI45931[] = {{&S57563[111], 3}, {NULL}};
const Segment SGI45932[] = {{&S57563[77], 5}, {NULL}};
const Segment SGI45933[] = {{&S57564[44], 8}, {NULL}};
const Segment SGI45934[] = {{&S57564[44], 19}, {NULL}};
const Segment SGI45935[] = {{&S57564[30], 6}, {NULL}};
const Segment SGI45936[] = {{&S57564[111], 3}, {NULL}};
const Segment SGI45937[] = {{&S57564[77], 5}, {NULL}};
const Segment SGI45938[] = {{&S57565[38], 8}, {NULL}};
const Segment SGI45939[] = {{&S57565[31], 6}, {NULL}};
const Segment SGI45940[] = {{&S65692[6], 4}, {NULL}};
const Segment SGI45941[] = {{&S65692[80], 5}, {NULL}};
const Segment SGI45942[] = {{&S57652[6], 4}, {NULL}};
const Segment SGI45943[] = {{&S57652[85], 5}, {NULL}};
const Segment SGI45944[] = {{&S58233[44], 5}, {NULL}};
const Segment SGI45945[] = {{&S58233[44], 16}, {NULL}};
const Segment SGI45946[] = {{&S58233[30], 6}, {NULL}};
const Segment SGI45947[] = {{&S58233[108], 3}, {NULL}};
const Segment SGI45948[] = {{&S58233[74], 5}, {NULL}};
const Segment SGI45949[] = {{&S58234[38], 5}, {NULL}};
const Segment SGI45950[] = {{&S58234[31], 6}, {NULL}};
const Segment SGI45951[] = {{&S61129[44], 10}, {NULL}};
const Segment SGI45952[] = {{&S61129[44], 21}, {NULL}};
const Segment SGI45953[] = {{&S61129[30], 6}, {NULL}};
const Segment SGI45954[] = {{&S61129[113], 3}, {NULL}};
const Segment SGI45955[] = {{&S61129[79], 5}, {NULL}};
const Segment SGI45956[] = {{&S61130[44], 10}, {NULL}};
const Segment SGI45957[] = {{&S61130[44], 21}, {NULL}};
const Segment SGI45958[] = {{&S61130[30], 6}, {NULL}};
const Segment SGI45959[] = {{&S61130[113], 3}, {NULL}};
const Segment SGI45960[] = {{&S61130[79], 5}, {NULL}};
const Segment SGI45961[] = {{&S61131[44], 10}, {NULL}};
const Segment SGI45962[] = {{&S61131[44], 21}, {NULL}};
const Segment SGI45963[] = {{&S61131[30], 6}, {NULL}};
const Segment SGI45964[] = {{&S61131[113], 3}, {NULL}};
const Segment SGI45965[] = {{&S61131[79], 5}, {NULL}};
const Segment SGI45966[] = {{&S61132[44], 11}, {NULL}};
const Segment SGI45967[] = {{&S61132[44], 22}, {NULL}};
const Segment SGI45968[] = {{&S61132[30], 6}, {NULL}};
const Segment SGI45969[] = {{&S61132[114], 3}, {NULL}};
const Segment SGI45970[] = {{&S61132[80], 5}, {NULL}};
const Segment SGI45971[] = {{&S61133[44], 11}, {NULL}};
const Segment SGI45972[] = {{&S61133[44], 22}, {NULL}};
const Segment SGI45973[] = {{&S61133[30], 6}, {NULL}};
const Segment SGI45974[] = {{&S61133[114], 3}, {NULL}};
const Segment SGI45975[] = {{&S61133[80], 5}, {NULL}};
const Segment SGI45976[] = {{&S61134[44], 10}, {NULL}};
const Segment SGI45977[] = {{&S61134[44], 21}, {NULL}};
const Segment SGI45978[] = {{&S61134[30], 6}, {NULL}};
const Segment SGI45979[] = {{&S61134[113], 3}, {NULL}};
const Segment SGI45980[] = {{&S61134[79], 5}, {NULL}};
const Segment SGI45981[] = {{&S61135[44], 10}, {NULL}};
const Segment SGI45982[] = {{&S61135[44], 21}, {NULL}};
const Segment SGI45983[] = {{&S61135[30], 6}, {NULL}};
const Segment SGI45984[] = {{&S61135[113], 3}, {NULL}};
const Segment SGI45985[] = {{&S61135[79], 5}, {NULL}};
const Segment SGI45986[] = {{&S61136[44], 11}, {NULL}};
const Segment SGI45987[] = {{&S61136[44], 22}, {NULL}};
const Segment SGI45988[] = {{&S61136[30], 6}, {NULL}};
const Segment SGI45989[] = {{&S61136[114], 3}, {NULL}};
const Segment SGI45990[] = {{&S61136[80], 5}, {NULL}};
const Segment SGI45991[] = {{&S61137[44], 11}, {NULL}};
const Segment SGI45992[] = {{&S61137[44], 22}, {NULL}};
const Segment SGI45993[] = {{&S61137[30], 6}, {NULL}};
const Segment SGI45994[] = {{&S61137[114], 3}, {NULL}};
const Segment SGI45995[] = {{&S61137[80], 5}, {NULL}};
const Segment SGI45996[] = {{&S61138[44], 11}, {NULL}};
const Segment SGI45997[] = {{&S61138[44], 22}, {NULL}};
const Segment SGI45998[] = {{&S61138[30], 6}, {NULL}};
const Segment SGI45999[] = {{&S61138[114], 3}, {NULL}};
const Segment SGI46000[] = {{&S61138[80], 5}, {NULL}};
const Segment SGI46001[] = {{&S61139[44], 11}, {NULL}};
const Segment SGI46002[] = {{&S61139[44], 22}, {NULL}};
const Segment SGI46003[] = {{&S61139[30], 6}, {NULL}};
const Segment SGI46004[] = {{&S61139[114], 3}, {NULL}};
const Segment SGI46005[] = {{&S61139[80], 5}, {NULL}};
const Segment SGI46006[] = {{&S61140[44], 11}, {NULL}};
const Segment SGI46007[] = {{&S61140[44], 22}, {NULL}};
const Segment SGI46008[] = {{&S61140[30], 6}, {NULL}};
const Segment SGI46009[] = {{&S61140[114], 3}, {NULL}};
const Segment SGI46010[] = {{&S61140[80], 5}, {NULL}};
const Segment SGI46011[] = {{&S61141[38], 10}, {NULL}};
const Segment SGI46012[] = {{&S61141[31], 6}, {NULL}};
const Segment SGI46013[] = {{&S61142[44], 10}, {NULL}};
const Segment SGI46014[] = {{&S61142[44], 21}, {NULL}};
const Segment SGI46015[] = {{&S61142[30], 6}, {NULL}};
const Segment SGI46016[] = {{&S61142[113], 3}, {NULL}};
const Segment SGI46017[] = {{&S61142[79], 5}, {NULL}};
const Segment SGI46018[] = {{&S58286[44], 10}, {NULL}};
const Segment SGI46019[] = {{&S58286[44], 21}, {NULL}};
const Segment SGI46020[] = {{&S58286[30], 6}, {NULL}};
const Segment SGI46021[] = {{&S58286[113], 3}, {NULL}};
const Segment SGI46022[] = {{&S58286[79], 5}, {NULL}};
const Segment SGI46023[] = {{&S58287[44], 10}, {NULL}};
const Segment SGI46024[] = {{&S58287[44], 21}, {NULL}};
const Segment SGI46025[] = {{&S58287[30], 6}, {NULL}};
const Segment SGI46026[] = {{&S58287[113], 3}, {NULL}};
const Segment SGI46027[] = {{&S58287[79], 5}, {NULL}};
const Segment SGI46028[] = {{&S58288[44], 10}, {NULL}};
const Segment SGI46029[] = {{&S58288[44], 21}, {NULL}};
const Segment SGI46030[] = {{&S58288[30], 6}, {NULL}};
const Segment SGI46031[] = {{&S58288[113], 3}, {NULL}};
const Segment SGI46032[] = {{&S58288[79], 5}, {NULL}};
const Segment SGI46033[] = {{&S58289[44], 10}, {NULL}};
const Segment SGI46034[] = {{&S58289[44], 21}, {NULL}};
const Segment SGI46035[] = {{&S58289[30], 6}, {NULL}};
const Segment SGI46036[] = {{&S58289[79], 6}, {NULL}};
const Segment SGI46037[] = {{&S58290[44], 10}, {NULL}};
const Segment SGI46038[] = {{&S58290[44], 21}, {NULL}};
const Segment SGI46039[] = {{&S58290[30], 6}, {NULL}};
const Segment SGI46040[] = {{&S58290[113], 3}, {NULL}};
const Segment SGI46041[] = {{&S58290[79], 5}, {NULL}};
const Segment SGI46042[] = {{&S58291[38], 10}, {NULL}};
const Segment SGI46043[] = {{&S58291[31], 6}, {NULL}};
const Segment SGI46044[] = {{&S58292[44], 10}, {NULL}};
const Segment SGI46045[] = {{&S58292[44], 21}, {NULL}};
const Segment SGI46046[] = {{&S58292[30], 6}, {NULL}};
const Segment SGI46047[] = {{&S58292[113], 3}, {NULL}};
const Segment SGI46048[] = {{&S58292[79], 5}, {NULL}};
const Segment SGI46049[] = {{&S58293[44], 10}, {NULL}};
const Segment SGI46050[] = {{&S58293[44], 21}, {NULL}};
const Segment SGI46051[] = {{&S58293[30], 6}, {NULL}};
const Segment SGI46052[] = {{&S58293[113], 3}, {NULL}};
const Segment SGI46053[] = {{&S58293[79], 5}, {NULL}};
const Segment SGI46054[] = {{&S58318[44], 14}, {NULL}};
const Segment SGI46055[] = {{&S58318[44], 25}, {NULL}};
const Segment SGI46056[] = {{&S58318[30], 6}, {NULL}};
const Segment SGI46057[] = {{&S58318[117], 3}, {NULL}};
const Segment SGI46058[] = {{&S58318[83], 5}, {NULL}};
const Segment SGI46059[] = {{&S57703[44], 14}, {NULL}};
const Segment SGI46060[] = {{&S57703[44], 31}, {NULL}};
const Segment SGI46061[] = {{&S57703[30], 6}, {NULL}};
const Segment SGI46062[] = {{&S57703[123], 3}, {NULL}};
const Segment SGI46063[] = {{&S57703[89], 5}, {NULL}};
const Segment SGI46064[] = {{&S57704[44], 5}, {NULL}};
const Segment SGI46065[] = {{&S57704[44], 22}, {NULL}};
const Segment SGI46066[] = {{&S57704[30], 6}, {NULL}};
const Segment SGI46067[] = {{&S57704[114], 3}, {NULL}};
const Segment SGI46068[] = {{&S57704[80], 5}, {NULL}};
const Segment SGI46069[] = {{&S57705[38], 5}, {NULL}};
const Segment SGI46070[] = {{&S57705[31], 6}, {NULL}};
const Segment SGI46071[] = {{&S57711[6], 4}, {NULL}};
const Segment SGI46072[] = {{&S57711[78], 5}, {NULL}};
const Segment SGI46073[] = {{&S58430[44], 12}, {NULL}};
const Segment SGI46074[] = {{&S58430[44], 67}, {NULL}};
const Segment SGI46075[] = {{&S58430[30], 6}, {NULL}};
const Segment SGI46076[] = {{&S58430[159], 3}, {NULL}};
const Segment SGI46077[] = {{&S58430[125], 5}, {NULL}};
const Segment SGI46078[] = {{&S58432[38], 7}, {NULL}};
const Segment SGI46079[] = {{&S58432[31], 6}, {NULL}};
const Segment SGI46080[] = {{&S58433[44], 7}, {NULL}};
const Segment SGI46081[] = {{&S58433[44], 29}, {NULL}};
const Segment SGI46082[] = {{&S58433[30], 6}, {NULL}};
const Segment SGI46083[] = {{&S58433[121], 3}, {NULL}};
const Segment SGI46084[] = {{&S58433[87], 5}, {NULL}};
const Segment SGI46085[] = {{&S58435[44], 12}, {NULL}};
const Segment SGI46086[] = {{&S58435[44], 67}, {NULL}};
const Segment SGI46087[] = {{&S58435[30], 6}, {NULL}};
const Segment SGI46088[] = {{&S58435[159], 3}, {NULL}};
const Segment SGI46089[] = {{&S58435[125], 5}, {NULL}};
const Segment SGI46090[] = {{&S58436[44], 7}, {NULL}};
const Segment SGI46091[] = {{&S58436[44], 29}, {NULL}};
const Segment SGI46092[] = {{&S58436[30], 6}, {NULL}};
const Segment SGI46093[] = {{&S58436[90], 4}, {NULL}};
const Segment SGI46094[] = {{&S58436[90], 3}, {NULL}};
const Segment SGI46095[] = {{&S58437[44], 7}, {NULL}};
const Segment SGI46096[] = {{&S58437[44], 28}, {NULL}};
const Segment SGI46097[] = {{&S58437[30], 6}, {NULL}};
const Segment SGI46098[] = {{&S58437[120], 3}, {NULL}};
const Segment SGI46099[] = {{&S58437[86], 5}, {NULL}};
const Segment SGI46100[] = {{&S58438[44], 7}, {NULL}};
const Segment SGI46101[] = {{&S58438[44], 29}, {NULL}};
const Segment SGI46102[] = {{&S58438[30], 6}, {NULL}};
const Segment SGI46103[] = {{&S58438[90], 6}, {NULL}};
const Segment SGI46104[] = {{&S58440[44], 7}, {NULL}};
const Segment SGI46105[] = {{&S58440[44], 29}, {NULL}};
const Segment SGI46106[] = {{&S58440[30], 6}, {NULL}};
const Segment SGI46107[] = {{&S58440[121], 3}, {NULL}};
const Segment SGI46108[] = {{&S58440[87], 5}, {NULL}};
const Segment SGI46109[] = {{&S57915[44], 10}, {NULL}};
const Segment SGI46110[] = {{&S57915[44], 39}, {NULL}};
const Segment SGI46111[] = {{&S57915[30], 6}, {NULL}};
const Segment SGI46112[] = {{&S57915[131], 3}, {NULL}};
const Segment SGI46113[] = {{&S57915[97], 5}, {NULL}};
const Segment SGI46114[] = {{&S58655[44], 15}, {NULL}};
const Segment SGI46115[] = {{&S58655[44], 25}, {NULL}};
const Segment SGI46116[] = {{&S58655[30], 6}, {NULL}};
const Segment SGI46117[] = {{&S58655[117], 3}, {NULL}};
const Segment SGI46118[] = {{&S58655[83], 5}, {NULL}};
const Segment SGI46119[] = {{&S61431[44], 10}, {NULL}};
const Segment SGI46120[] = {{&S61431[44], 27}, {NULL}};
const Segment SGI46121[] = {{&S61431[30], 6}, {NULL}};
const Segment SGI46122[] = {{&S61431[119], 3}, {NULL}};
const Segment SGI46123[] = {{&S61431[85], 5}, {NULL}};
const Segment SGI46124[] = {{&S61432[44], 10}, {NULL}};
const Segment SGI46125[] = {{&S61432[44], 27}, {NULL}};
const Segment SGI46126[] = {{&S61432[30], 6}, {NULL}};
const Segment SGI46127[] = {{&S61432[119], 3}, {NULL}};
const Segment SGI46128[] = {{&S61432[85], 5}, {NULL}};
const Segment SGI46129[] = {{&S61433[42], 10}, {NULL}};
const Segment SGI46130[] = {{&S61433[30], 6}, {NULL}};
const Segment SGI46131[] = {{&S61433[117], 3}, {NULL}};
const Segment SGI46132[] = {{&S61433[83], 5}, {NULL}};
const Segment SGI46133[] = {{&S61434[38], 10}, {NULL}};
const Segment SGI46134[] = {{&S61434[31], 6}, {NULL}};
const Segment SGI46135[] = {{&S61435[42], 10}, {NULL}};
const Segment SGI46136[] = {{&S61435[30], 6}, {NULL}};
const Segment SGI46137[] = {{&S61435[117], 3}, {NULL}};
const Segment SGI46138[] = {{&S61435[83], 5}, {NULL}};
const Segment SGI46139[] = {{&S61512[44], 8}, {NULL}};
const Segment SGI46140[] = {{&S61512[44], 25}, {NULL}};
const Segment SGI46141[] = {{&S61512[30], 6}, {NULL}};
const Segment SGI46142[] = {{&S61512[117], 3}, {NULL}};
const Segment SGI46143[] = {{&S61512[83], 5}, {NULL}};
const Segment SGI46144[] = {{&S61513[38], 8}, {NULL}};
const Segment SGI46145[] = {{&S61513[31], 6}, {NULL}};
const Segment SGI46146[] = {{&S72569[6], 4}, {NULL}};
const Segment SGI46147[] = {{&S72569[82], 5}, {NULL}};
const Segment SGI46148[] = {{&S58122[31], 8}, {NULL}};
const Segment SGI46149[] = {{&S58122[54], 3}, {NULL}};
const Segment SGI46150[] = {{&S72690[44], 26}, {NULL}};
const Segment SGI46151[] = {{&S72690[44], 41}, {NULL}};
const Segment SGI46152[] = {{&S72690[30], 6}, {&S72690[52], 33}, {NULL}};
const Segment SGI46153[] = {{&S72690[133], 3}, {NULL}};
const Segment SGI46154[] = {{&S72690[99], 5}, {NULL}};
const Segment SGI46155[] = {{&S72691[44], 26}, {NULL}};
const Segment SGI46156[] = {{&S72691[44], 41}, {NULL}};
const Segment SGI46157[] = {{&S72691[30], 6}, {&S72691[52], 33}, {NULL}};
const Segment SGI46158[] = {{&S72691[133], 3}, {NULL}};
const Segment SGI46159[] = {{&S72691[99], 5}, {NULL}};
const Segment SGI46160[] = {{&S58235[44], 11}, {NULL}};
const Segment SGI46161[] = {{&S58235[44], 22}, {NULL}};
const Segment SGI46162[] = {{&S58235[30], 6}, {NULL}};
const Segment SGI46163[] = {{&S58235[113], 3}, {NULL}};
const Segment SGI46164[] = {{&S58235[80], 5}, {NULL}};
const Segment SGI46165[] = {{&S61708[38], 9}, {NULL}};
const Segment SGI46166[] = {{&S61708[31], 6}, {NULL}};
const Segment SGI46167[] = {{&S61709[44], 9}, {NULL}};
const Segment SGI46168[] = {{&S61709[44], 20}, {NULL}};
const Segment SGI46169[] = {{&S61709[30], 6}, {NULL}};
const Segment SGI46170[] = {{&S61709[112], 3}, {NULL}};
const Segment SGI46171[] = {{&S61709[78], 5}, {NULL}};
const Segment SGI46172[] = {{&S61710[44], 10}, {NULL}};
const Segment SGI46173[] = {{&S61710[44], 21}, {NULL}};
const Segment SGI46174[] = {{&S61710[30], 6}, {NULL}};
const Segment SGI46175[] = {{&S61710[113], 3}, {NULL}};
const Segment SGI46176[] = {{&S61710[79], 5}, {NULL}};
const Segment SGI46177[] = {{&S72771[44], 11}, {NULL}};
const Segment SGI46178[] = {{&S72771[44], 23}, {NULL}};
const Segment SGI46179[] = {{&S72771[30], 6}, {NULL}};
const Segment SGI46180[] = {{&S72771[115], 3}, {NULL}};
const Segment SGI46181[] = {{&S72771[81], 5}, {NULL}};
const Segment SGI46182[] = {{&S72772[38], 11}, {NULL}};
const Segment SGI46183[] = {{&S72772[31], 6}, {NULL}};
const Segment SGI46184[] = {{&S72773[44], 12}, {NULL}};
const Segment SGI46185[] = {{&S72773[44], 23}, {NULL}};
const Segment SGI46186[] = {{&S72773[30], 6}, {NULL}};
const Segment SGI46187[] = {{&S72773[115], 3}, {NULL}};
const Segment SGI46188[] = {{&S72773[81], 5}, {NULL}};
const Segment SGI46189[] = {{&S72774[44], 12}, {NULL}};
const Segment SGI46190[] = {{&S72774[44], 23}, {NULL}};
const Segment SGI46191[] = {{&S72774[30], 6}, {NULL}};
const Segment SGI46192[] = {{&S72774[115], 3}, {NULL}};
const Segment SGI46193[] = {{&S72774[81], 5}, {NULL}};
const Segment SGI46194[] = {{&S72775[44], 6}, {NULL}};
const Segment SGI46195[] = {{&S72775[44], 17}, {NULL}};
const Segment SGI46196[] = {{&S72775[30], 6}, {NULL}};
const Segment SGI46197[] = {{&S72775[109], 3}, {NULL}};
const Segment SGI46198[] = {{&S72775[75], 5}, {NULL}};
const Segment SGI46199[] = {{&S58321[44], 14}, {NULL}};
const Segment SGI46200[] = {{&S58321[44], 25}, {NULL}};
const Segment SGI46201[] = {{&S58321[30], 6}, {NULL}};
const Segment SGI46202[] = {{&S58321[117], 3}, {NULL}};
const Segment SGI46203[] = {{&S58321[83], 5}, {NULL}};
const Segment SGI46204[] = {{&S58397[44], 7}, {NULL}};
const Segment SGI46205[] = {{&S58397[44], 20}, {NULL}};
const Segment SGI46206[] = {{&S58397[30], 6}, {NULL}};
const Segment SGI46207[] = {{&S58397[112], 3}, {NULL}};
const Segment SGI46208[] = {{&S58397[78], 5}, {NULL}};
const Segment SGI46209[] = {{&S58398[44], 7}, {NULL}};
const Segment SGI46210[] = {{&S58398[44], 18}, {NULL}};
const Segment SGI46211[] = {{&S58398[30], 6}, {NULL}};
const Segment SGI46212[] = {{&S58398[110], 3}, {NULL}};
const Segment SGI46213[] = {{&S58398[76], 5}, {NULL}};
const Segment SGI46214[] = {{&S58399[44], 7}, {NULL}};
const Segment SGI46215[] = {{&S58399[44], 20}, {NULL}};
const Segment SGI46216[] = {{&S58399[30], 6}, {NULL}};
const Segment SGI46217[] = {{&S58399[112], 3}, {NULL}};
const Segment SGI46218[] = {{&S58399[78], 5}, {NULL}};
const Segment SGI46219[] = {{&S58400[44], 7}, {NULL}};
const Segment SGI46220[] = {{&S58400[44], 20}, {NULL}};
const Segment SGI46221[] = {{&S58400[30], 6}, {NULL}};
const Segment SGI46222[] = {{&S58400[112], 3}, {NULL}};
const Segment SGI46223[] = {{&S58400[78], 5}, {NULL}};
const Segment SGI46224[] = {{&S58401[44], 7}, {NULL}};
const Segment SGI46225[] = {{&S58401[44], 20}, {NULL}};
const Segment SGI46226[] = {{&S58401[30], 6}, {NULL}};
const Segment SGI46227[] = {{&S58401[112], 3}, {NULL}};
const Segment SGI46228[] = {{&S58401[78], 5}, {NULL}};
const Segment SGI46229[] = {{&S58402[38], 7}, {NULL}};
const Segment SGI46230[] = {{&S58402[31], 6}, {NULL}};
const Segment SGI46231[] = {{&S59197[44], 7}, {NULL}};
const Segment SGI46232[] = {{&S59197[44], 18}, {NULL}};
const Segment SGI46233[] = {{&S59197[30], 6}, {NULL}};
const Segment SGI46234[] = {{&S59197[110], 3}, {NULL}};
const Segment SGI46235[] = {{&S59197[76], 5}, {NULL}};
const Segment SGI46236[] = {{&S59198[38], 7}, {NULL}};
const Segment SGI46237[] = {{&S59198[31], 6}, {NULL}};
const Segment SGI46238[] = {{&S58431[44], 7}, {NULL}};
const Segment SGI46239[] = {{&S58431[44], 29}, {NULL}};
const Segment SGI46240[] = {{&S58431[30], 6}, {NULL}};
const Segment SGI46241[] = {{&S58431[121], 3}, {NULL}};
const Segment SGI46242[] = {{&S58431[87], 5}, {NULL}};
const Segment SGI46243[] = {{&S58434[44], 7}, {NULL}};
const Segment SGI46244[] = {{&S58434[44], 29}, {NULL}};
const Segment SGI46245[] = {{&S58434[30], 6}, {NULL}};
const Segment SGI46246[] = {{&S58434[90], 4}, {NULL}};
const Segment SGI46247[] = {{&S58434[90], 3}, {NULL}};
const Segment SGI46248[] = {{&S58439[44], 7}, {NULL}};
const Segment SGI46249[] = {{&S58439[44], 29}, {NULL}};
const Segment SGI46250[] = {{&S58439[30], 6}, {NULL}};
const Segment SGI46251[] = {{&S58439[90], 6}, {NULL}};
const Segment SGI46252[] = {{&S59231[44], 11}, {NULL}};
const Segment SGI46253[] = {{&S59231[44], 22}, {NULL}};
const Segment SGI46254[] = {{&S59231[30], 6}, {NULL}};
const Segment SGI46255[] = {{&S59231[114], 3}, {NULL}};
const Segment SGI46256[] = {{&S59231[80], 5}, {NULL}};
const Segment SGI46257[] = {{&S59232[38], 11}, {NULL}};
const Segment SGI46258[] = {{&S59232[31], 6}, {NULL}};
const Segment SGI46259[] = {{&S72943[44], 8}, {NULL}};
const Segment SGI46260[] = {{&S72943[44], 20}, {NULL}};
const Segment SGI46261[] = {{&S72943[30], 6}, {NULL}};
const Segment SGI46262[] = {{&S72943[112], 3}, {NULL}};
const Segment SGI46263[] = {{&S72943[78], 5}, {NULL}};
const Segment SGI46264[] = {{&S72964[6], 4}, {NULL}};
const Segment SGI46265[] = {{&S72964[76], 5}, {NULL}};
const Segment SGI46266[] = {{&S62078[6], 4}, {NULL}};
const Segment SGI46267[] = {{&S62078[82], 5}, {NULL}};
const Segment SGI46268[] = {{&S58656[44], 15}, {NULL}};
const Segment SGI46269[] = {{&S58656[44], 25}, {NULL}};
const Segment SGI46270[] = {{&S58656[30], 6}, {NULL}};
const Segment SGI46271[] = {{&S58656[117], 3}, {NULL}};
const Segment SGI46272[] = {{&S58656[83], 5}, {NULL}};
const Segment SGI46273[] = {{&S59314[44], 14}, {NULL}};
const Segment SGI46274[] = {{&S59314[44], 31}, {NULL}};
const Segment SGI46275[] = {{&S59314[30], 6}, {NULL}};
const Segment SGI46276[] = {{&S59314[123], 3}, {NULL}};
const Segment SGI46277[] = {{&S59314[89], 5}, {NULL}};
const Segment SGI46278[] = {{&S58658[44], 11}, {NULL}};
const Segment SGI46279[] = {{&S58658[44], 21}, {NULL}};
const Segment SGI46280[] = {{&S58658[30], 6}, {NULL}};
const Segment SGI46281[] = {{&S58658[113], 3}, {NULL}};
const Segment SGI46282[] = {{&S58658[79], 5}, {NULL}};
const Segment SGI46283[] = {{&S59353[44], 6}, {NULL}};
const Segment SGI46284[] = {{&S59353[44], 17}, {NULL}};
const Segment SGI46285[] = {{&S59353[30], 6}, {NULL}};
const Segment SGI46286[] = {{&S59353[109], 3}, {NULL}};
const Segment SGI46287[] = {{&S59353[75], 5}, {NULL}};
const Segment SGI46288[] = {{&S59354[44], 6}, {NULL}};
const Segment SGI46289[] = {{&S59354[44], 17}, {NULL}};
const Segment SGI46290[] = {{&S59354[30], 6}, {NULL}};
const Segment SGI46291[] = {{&S59354[109], 3}, {NULL}};
const Segment SGI46292[] = {{&S59354[75], 5}, {NULL}};
const Segment SGI46293[] = {{&S59355[42], 6}, {NULL}};
const Segment SGI46294[] = {{&S59355[30], 6}, {NULL}};
const Segment SGI46295[] = {{&S59355[107], 3}, {NULL}};
const Segment SGI46296[] = {{&S59355[73], 5}, {NULL}};
const Segment SGI46297[] = {{&S59356[44], 6}, {NULL}};
const Segment SGI46298[] = {{&S59356[44], 17}, {NULL}};
const Segment SGI46299[] = {{&S59356[30], 6}, {NULL}};
const Segment SGI46300[] = {{&S59356[109], 3}, {NULL}};
const Segment SGI46301[] = {{&S59356[75], 5}, {NULL}};
const Segment SGI46302[] = {{&S62188[44], 10}, {NULL}};
const Segment SGI46303[] = {{&S62188[44], 27}, {NULL}};
const Segment SGI46304[] = {{&S62188[30], 6}, {NULL}};
const Segment SGI46305[] = {{&S58749[44], 10}, {NULL}};
const Segment SGI46306[] = {{&S58749[44], 21}, {NULL}};
const Segment SGI46307[] = {{&S58749[30], 6}, {NULL}};
const Segment SGI46308[] = {{&S62188[85], 6}, {NULL}};
const Segment SGI46309[] = {{&S62188[122], 3}, {NULL}};
const Segment SGI46310[] = {{&S58749[113], 3}, {NULL}};
const Segment SGI46311[] = {{&S58749[79], 5}, {NULL}};
const Segment SGI46312[] = {{&S58750[38], 10}, {NULL}};
const Segment SGI46313[] = {{&S58750[31], 6}, {NULL}};
const Segment SGI46314[] = {{&S62204[30], 6}, {NULL}};
const Segment SGI46315[] = {{&S62204[92], 2}, {NULL}};
const Segment SGI46316[] = {{&S62204[57], 6}, {NULL}};
const Segment SGI46317[] = {{&S73144[44], 8}, {NULL}};
const Segment SGI46318[] = {{&S73144[44], 31}, {NULL}};
const Segment SGI46319[] = {{&S73144[30], 6}, {NULL}};
const Segment SGI46320[] = {{&S73144[123], 3}, {NULL}};
const Segment SGI46321[] = {{&S73144[89], 5}, {NULL}};
const Segment SGI46322[] = {{&S73145[44], 7}, {NULL}};
const Segment SGI46323[] = {{&S73145[44], 28}, {NULL}};
const Segment SGI46324[] = {{&S73145[30], 6}, {NULL}};
const Segment SGI46325[] = {{&S73145[89], 4}, {NULL}};
const Segment SGI46326[] = {{&S73145[89], 3}, {NULL}};
const Segment SGI46327[] = {{&S73146[44], 26}, {NULL}};
const Segment SGI46328[] = {{&S73146[30], 6}, {NULL}};
const Segment SGI46329[] = {{&S73146[136], 3}, {NULL}};
const Segment SGI46330[] = {{&S73146[102], 5}, {NULL}};
const Segment SGI46331[] = {{&S73147[38], 8}, {NULL}};
const Segment SGI46332[] = {{&S73147[31], 6}, {NULL}};
const Segment SGI46333[] = {{&S73149[44], 7}, {NULL}};
const Segment SGI46334[] = {{&S73149[44], 28}, {NULL}};
const Segment SGI46335[] = {{&S73149[30], 6}, {NULL}};
const Segment SGI46336[] = {{&S73149[120], 3}, {NULL}};
const Segment SGI46337[] = {{&S73149[86], 5}, {NULL}};
const Segment SGI46338[] = {{&S73193[44], 11}, {NULL}};
const Segment SGI46339[] = {{&S73193[44], 22}, {NULL}};
const Segment SGI46340[] = {{&S73193[30], 6}, {NULL}};
const Segment SGI46341[] = {{&S73193[114], 3}, {NULL}};
const Segment SGI46342[] = {{&S73193[80], 5}, {NULL}};
const Segment SGI46343[] = {{&S73194[44], 11}, {NULL}};
const Segment SGI46344[] = {{&S73194[44], 22}, {NULL}};
const Segment SGI46345[] = {{&S73194[30], 6}, {NULL}};
const Segment SGI46346[] = {{&S73194[83], 4}, {NULL}};
const Segment SGI46347[] = {{&S73194[83], 3}, {NULL}};
const Segment SGI46348[] = {{&S66664[56], 14}, {NULL}};
const Segment SGI46349[] = {{&S66664[56], 33}, {NULL}};
const Segment SGI46350[] = {{&S66664[30], 18}, {NULL}};
const Segment SGI46351[] = {{&S66664[137], 3}, {NULL}};
const Segment SGI46352[] = {{&S66664[103], 5}, {NULL}};
const Segment SGI46353[] = {{&S73228[44], 11}, {NULL}};
const Segment SGI46354[] = {{&S73228[44], 22}, {NULL}};
const Segment SGI46355[] = {{&S73228[30], 6}, {NULL}};
const Segment SGI46356[] = {{&S73228[83], 4}, {NULL}};
const Segment SGI46357[] = {{&S73228[83], 3}, {NULL}};
const Segment SGI46358[] = {{&S73229[44], 11}, {NULL}};
const Segment SGI46359[] = {{&S73229[44], 22}, {NULL}};
const Segment SGI46360[] = {{&S73229[30], 6}, {NULL}};
const Segment SGI46361[] = {{&S73229[114], 3}, {NULL}};
const Segment SGI46362[] = {{&S73229[80], 5}, {NULL}};
const Segment SGI46363[] = {{&S73230[44], 11}, {NULL}};
const Segment SGI46364[] = {{&S73230[44], 22}, {NULL}};
const Segment SGI46365[] = {{&S73230[30], 6}, {NULL}};
const Segment SGI46366[] = {{&S73230[83], 6}, {NULL}};
const Segment SGI46367[] = {{&S73233[38], 11}, {NULL}};
const Segment SGI46368[] = {{&S73233[31], 6}, {NULL}};
const Segment SGI46369[] = {{&S62240[44], 8}, {NULL}};
const Segment SGI46370[] = {{&S62240[44], 19}, {NULL}};
const Segment SGI46371[] = {{&S62240[30], 6}, {NULL}};
const Segment SGI46372[] = {{&S62240[111], 3}, {NULL}};
const Segment SGI46373[] = {{&S62240[77], 5}, {NULL}};
const Segment SGI46374[] = {{&S62241[44], 8}, {NULL}};
const Segment SGI46375[] = {{&S62241[44], 19}, {NULL}};
const Segment SGI46376[] = {{&S62241[30], 6}, {NULL}};
const Segment SGI46377[] = {{&S62241[111], 3}, {NULL}};
const Segment SGI46378[] = {{&S62241[77], 5}, {NULL}};
const Segment SGI46379[] = {{&S62242[44], 8}, {NULL}};
const Segment SGI46380[] = {{&S62242[44], 19}, {NULL}};
const Segment SGI46381[] = {{&S62242[30], 6}, {NULL}};
const Segment SGI46382[] = {{&S62242[111], 3}, {NULL}};
const Segment SGI46383[] = {{&S62242[77], 5}, {NULL}};
const Segment SGI46384[] = {{&S62243[44], 8}, {NULL}};
const Segment SGI46385[] = {{&S62243[44], 19}, {NULL}};
const Segment SGI46386[] = {{&S62243[30], 6}, {NULL}};
const Segment SGI46387[] = {{&S62243[111], 3}, {NULL}};
const Segment SGI46388[] = {{&S62243[77], 5}, {NULL}};
const Segment SGI46389[] = {{&S59555[44], 8}, {NULL}};
const Segment SGI46390[] = {{&S59555[44], 19}, {NULL}};
const Segment SGI46391[] = {{&S59555[30], 6}, {NULL}};
const Segment SGI46392[] = {{&S59555[111], 3}, {NULL}};
const Segment SGI46393[] = {{&S59555[77], 5}, {NULL}};
const Segment SGI46394[] = {{&S62244[44], 8}, {NULL}};
const Segment SGI46395[] = {{&S62244[44], 19}, {NULL}};
const Segment SGI46396[] = {{&S62244[30], 6}, {NULL}};
const Segment SGI46397[] = {{&S62244[111], 3}, {NULL}};
const Segment SGI46398[] = {{&S62244[77], 5}, {NULL}};
const Segment SGI46399[] = {{&S59556[44], 8}, {NULL}};
const Segment SGI46400[] = {{&S59556[44], 19}, {NULL}};
const Segment SGI46401[] = {{&S59556[30], 6}, {NULL}};
const Segment SGI46402[] = {{&S59556[111], 3}, {NULL}};
const Segment SGI46403[] = {{&S59556[77], 5}, {NULL}};
const Segment SGI46404[] = {{&S62245[44], 8}, {NULL}};
const Segment SGI46405[] = {{&S62245[44], 19}, {NULL}};
const Segment SGI46406[] = {{&S62245[30], 6}, {NULL}};
const Segment SGI46407[] = {{&S62245[111], 3}, {NULL}};
const Segment SGI46408[] = {{&S62245[77], 5}, {NULL}};
const Segment SGI46409[] = {{&S59557[44], 8}, {NULL}};
const Segment SGI46410[] = {{&S59557[44], 19}, {NULL}};
const Segment SGI46411[] = {{&S59557[30], 6}, {NULL}};
const Segment SGI46412[] = {{&S59557[111], 3}, {NULL}};
const Segment SGI46413[] = {{&S59557[77], 5}, {NULL}};
const Segment SGI46414[] = {{&S62246[44], 8}, {NULL}};
const Segment SGI46415[] = {{&S62246[44], 19}, {NULL}};
const Segment SGI46416[] = {{&S62246[30], 6}, {NULL}};
const Segment SGI46417[] = {{&S62246[111], 3}, {NULL}};
const Segment SGI46418[] = {{&S62246[77], 5}, {NULL}};
const Segment SGI46419[] = {{&S59558[44], 8}, {NULL}};
const Segment SGI46420[] = {{&S59558[44], 19}, {NULL}};
const Segment SGI46421[] = {{&S59558[30], 6}, {NULL}};
const Segment SGI46422[] = {{&S59558[111], 3}, {NULL}};
const Segment SGI46423[] = {{&S59558[77], 5}, {NULL}};
const Segment SGI46424[] = {{&S59559[44], 8}, {NULL}};
const Segment SGI46425[] = {{&S59559[44], 19}, {NULL}};
const Segment SGI46426[] = {{&S59559[30], 6}, {NULL}};
const Segment SGI46427[] = {{&S59559[111], 3}, {NULL}};
const Segment SGI46428[] = {{&S59559[77], 5}, {NULL}};
const Segment SGI46429[] = {{&S59560[38], 8}, {NULL}};
const Segment SGI46430[] = {{&S59560[31], 6}, {NULL}};
const Segment SGI46431[] = {{&S59003[44], 13}, {NULL}};
const Segment SGI46432[] = {{&S59003[44], 24}, {NULL}};
const Segment SGI46433[] = {{&S59003[30], 6}, {NULL}};
const Segment SGI46434[] = {{&S59003[116], 3}, {NULL}};
const Segment SGI46435[] = {{&S59003[82], 5}, {NULL}};
const Segment SGI46436[] = {{&S59005[44], 13}, {NULL}};
const Segment SGI46437[] = {{&S59005[44], 24}, {NULL}};
const Segment SGI46438[] = {{&S59005[30], 6}, {NULL}};
const Segment SGI46439[] = {{&S59005[85], 6}, {NULL}};
const Segment SGI46440[] = {{&S59006[38], 13}, {NULL}};
const Segment SGI46441[] = {{&S59006[31], 6}, {NULL}};
const Segment SGI46442[] = {{&S73251[44], 10}, {NULL}};
const Segment SGI46443[] = {{&S73251[44], 22}, {NULL}};
const Segment SGI46444[] = {{&S73251[30], 6}, {NULL}};
const Segment SGI46445[] = {{&S73251[114], 3}, {NULL}};
const Segment SGI46446[] = {{&S73251[80], 5}, {NULL}};
const Segment SGI46447[] = {{&S67941[44], 7}, {NULL}};
const Segment SGI46448[] = {{&S67941[44], 24}, {NULL}};
const Segment SGI46449[] = {{&S67941[30], 6}, {NULL}};
const Segment SGI46450[] = {{&S67941[116], 3}, {NULL}};
const Segment SGI46451[] = {{&S67941[82], 5}, {NULL}};
const Segment SGI46452[] = {{&S67942[44], 9}, {NULL}};
const Segment SGI46453[] = {{&S67942[44], 26}, {NULL}};
const Segment SGI46454[] = {{&S67942[30], 6}, {NULL}};
const Segment SGI46455[] = {{&S67942[118], 3}, {NULL}};
const Segment SGI46456[] = {{&S67942[84], 5}, {NULL}};
const Segment SGI46457[] = {{&S59819[44], 7}, {NULL}};
const Segment SGI46458[] = {{&S59819[44], 18}, {NULL}};
const Segment SGI46459[] = {{&S59819[30], 6}, {NULL}};
const Segment SGI46460[] = {{&S59819[110], 3}, {NULL}};
const Segment SGI46461[] = {{&S59819[76], 5}, {NULL}};
const Segment SGI46462[] = {{&S59843[44], 18}, {NULL}};
const Segment SGI46463[] = {{&S59843[44], 29}, {NULL}};
const Segment SGI46464[] = {{&S59843[30], 6}, {NULL}};
const Segment SGI46465[] = {{&S59843[121], 3}, {NULL}};
const Segment SGI46466[] = {{&S59843[87], 5}, {NULL}};
const Segment SGI46467[] = {{&S59847[44], 18}, {NULL}};
const Segment SGI46468[] = {{&S59847[44], 29}, {NULL}};
const Segment SGI46469[] = {{&S59847[30], 6}, {NULL}};
const Segment SGI46470[] = {{&S59847[90], 4}, {NULL}};
const Segment SGI46471[] = {{&S59847[90], 3}, {NULL}};
const Segment SGI46472[] = {{&S59850[44], 18}, {NULL}};
const Segment SGI46473[] = {{&S59850[44], 29}, {NULL}};
const Segment SGI46474[] = {{&S59850[30], 6}, {NULL}};
const Segment SGI46475[] = {{&S59850[121], 3}, {NULL}};
const Segment SGI46476[] = {{&S59850[87], 5}, {NULL}};
const Segment SGI46477[] = {{&S59990[44], 7}, {NULL}};
const Segment SGI46478[] = {{&S59990[44], 23}, {NULL}};
const Segment SGI46479[] = {{&S59990[30], 6}, {NULL}};
const Segment SGI46480[] = {{&S59990[115], 3}, {NULL}};
const Segment SGI46481[] = {{&S59990[81], 5}, {NULL}};
const Segment SGI46482[] = {{&S59991[44], 7}, {NULL}};
const Segment SGI46483[] = {{&S59991[44], 23}, {NULL}};
const Segment SGI46484[] = {{&S59991[30], 6}, {NULL}};
const Segment SGI46485[] = {{&S59991[115], 3}, {NULL}};
const Segment SGI46486[] = {{&S59991[81], 5}, {NULL}};
const Segment SGI46487[] = {{&S59994[44], 7}, {NULL}};
const Segment SGI46488[] = {{&S59994[44], 33}, {NULL}};
const Segment SGI46489[] = {{&S59994[30], 6}, {NULL}};
const Segment SGI46490[] = {{&S59994[125], 3}, {NULL}};
const Segment SGI46491[] = {{&S59994[91], 5}, {NULL}};
const Segment SGI46492[] = {{&S59301[44], 8}, {NULL}};
const Segment SGI46493[] = {{&S59301[44], 19}, {NULL}};
const Segment SGI46494[] = {{&S59301[30], 6}, {NULL}};
const Segment SGI46495[] = {{&S59301[111], 3}, {NULL}};
const Segment SGI46496[] = {{&S59301[77], 5}, {NULL}};
const Segment SGI46497[] = {{&S59302[44], 8}, {NULL}};
const Segment SGI46498[] = {{&S59302[44], 19}, {NULL}};
const Segment SGI46499[] = {{&S59302[30], 6}, {NULL}};
const Segment SGI46500[] = {{&S59302[111], 3}, {NULL}};
const Segment SGI46501[] = {{&S59302[77], 5}, {NULL}};
const Segment SGI46502[] = {{&S59303[44], 8}, {NULL}};
const Segment SGI46503[] = {{&S59303[44], 19}, {NULL}};
const Segment SGI46504[] = {{&S59303[30], 6}, {NULL}};
const Segment SGI46505[] = {{&S59303[111], 3}, {NULL}};
const Segment SGI46506[] = {{&S59303[77], 5}, {NULL}};
const Segment SGI46507[] = {{&S59304[44], 12}, {NULL}};
const Segment SGI46508[] = {{&S59304[44], 23}, {NULL}};
const Segment SGI46509[] = {{&S59304[30], 6}, {NULL}};
const Segment SGI46510[] = {{&S59304[115], 3}, {NULL}};
const Segment SGI46511[] = {{&S59304[81], 5}, {NULL}};
const Segment SGI46512[] = {{&S59305[44], 12}, {NULL}};
const Segment SGI46513[] = {{&S59305[44], 23}, {NULL}};
const Segment SGI46514[] = {{&S59305[30], 6}, {NULL}};
const Segment SGI46515[] = {{&S59305[115], 3}, {NULL}};
const Segment SGI46516[] = {{&S59305[81], 5}, {NULL}};
const Segment SGI46517[] = {{&S59306[44], 8}, {NULL}};
const Segment SGI46518[] = {{&S59306[44], 19}, {NULL}};
const Segment SGI46519[] = {{&S59306[30], 6}, {NULL}};
const Segment SGI46520[] = {{&S59306[111], 3}, {NULL}};
const Segment SGI46521[] = {{&S59306[77], 5}, {NULL}};
const Segment SGI46522[] = {{&S60057[44], 11}, {NULL}};
const Segment SGI46523[] = {{&S60057[44], 28}, {NULL}};
const Segment SGI46524[] = {{&S60057[30], 6}, {NULL}};
const Segment SGI46525[] = {{&S60057[120], 3}, {NULL}};
const Segment SGI46526[] = {{&S60057[86], 5}, {NULL}};
const Segment SGI46527[] = {{&S59307[38], 8}, {NULL}};
const Segment SGI46528[] = {{&S59307[31], 6}, {NULL}};
const Segment SGI46529[] = {{&S59308[44], 12}, {NULL}};
const Segment SGI46530[] = {{&S59308[44], 23}, {NULL}};
const Segment SGI46531[] = {{&S59308[30], 6}, {NULL}};
const Segment SGI46532[] = {{&S59308[115], 3}, {NULL}};
const Segment SGI46533[] = {{&S59308[81], 5}, {NULL}};
const Segment SGI46534[] = {{&S62777[44], 7}, {NULL}};
const Segment SGI46535[] = {{&S62777[44], 18}, {NULL}};
const Segment SGI46536[] = {{&S62777[30], 6}, {NULL}};
const Segment SGI46537[] = {{&S62777[110], 3}, {NULL}};
const Segment SGI46538[] = {{&S62777[76], 5}, {NULL}};
const Segment SGI46539[] = {{&S59315[44], 14}, {NULL}};
const Segment SGI46540[] = {{&S59315[44], 31}, {NULL}};
const Segment SGI46541[] = {{&S59315[30], 6}, {NULL}};
const Segment SGI46542[] = {{&S59315[123], 3}, {NULL}};
const Segment SGI46543[] = {{&S59315[89], 5}, {NULL}};
const Segment SGI46544[] = {{&S62778[38], 7}, {NULL}};
const Segment SGI46545[] = {{&S62778[31], 6}, {NULL}};
const Segment SGI46546[] = {{&S60059[38], 11}, {NULL}};
const Segment SGI46547[] = {{&S60059[31], 6}, {NULL}};
const Segment SGI46548[] = {{&S60063[44], 10}, {NULL}};
const Segment SGI46549[] = {{&S60063[44], 27}, {NULL}};
const Segment SGI46550[] = {{&S60063[30], 6}, {NULL}};
const Segment SGI46551[] = {{&S60063[119], 3}, {NULL}};
const Segment SGI46552[] = {{&S60063[85], 5}, {NULL}};
const Segment SGI46553[] = {{&S60067[44], 10}, {NULL}};
const Segment SGI46554[] = {{&S60067[44], 21}, {NULL}};
const Segment SGI46555[] = {{&S60067[30], 6}, {NULL}};
const Segment SGI46556[] = {{&S60067[113], 3}, {NULL}};
const Segment SGI46557[] = {{&S60067[79], 5}, {NULL}};
const Segment SGI46558[] = {{&S60072[44], 6}, {NULL}};
const Segment SGI46559[] = {{&S60072[44], 17}, {NULL}};
const Segment SGI46560[] = {{&S60072[30], 6}, {NULL}};
const Segment SGI46561[] = {{&S60072[109], 3}, {NULL}};
const Segment SGI46562[] = {{&S60072[75], 5}, {NULL}};
const Segment SGI46563[] = {{&S73785[6], 4}, {NULL}};
const Segment SGI46564[] = {{&S73785[84], 5}, {NULL}};
const Segment SGI46565[] = {{&S73885[44], 14}, {NULL}};
const Segment SGI46566[] = {{&S73885[44], 25}, {NULL}};
const Segment SGI46567[] = {{&S73885[30], 6}, {NULL}};
const Segment SGI46568[] = {{&S73885[86], 4}, {NULL}};
const Segment SGI46569[] = {{&S73885[86], 3}, {NULL}};
const Segment SGI46570[] = {{&S73886[44], 14}, {NULL}};
const Segment SGI46571[] = {{&S73886[44], 25}, {NULL}};
const Segment SGI46572[] = {{&S73886[30], 6}, {NULL}};
const Segment SGI46573[] = {{&S73886[117], 3}, {NULL}};
const Segment SGI46574[] = {{&S73886[83], 5}, {NULL}};
const Segment SGI46575[] = {{&S73887[44], 14}, {NULL}};
const Segment SGI46576[] = {{&S73887[44], 25}, {NULL}};
const Segment SGI46577[] = {{&S73887[30], 6}, {NULL}};
const Segment SGI46578[] = {{&S73887[86], 6}, {NULL}};
const Segment SGI46579[] = {{&S67501[41], 10}, {NULL}};
const Segment SGI46580[] = {{&S67501[41], 21}, {NULL}};
const Segment SGI46581[] = {{&S67501[30], 4}, {NULL}};
const Segment SGI46582[] = {{&S67501[110], 3}, {NULL}};
const Segment SGI46583[] = {{&S67501[76], 5}, {NULL}};
const Segment SGI46584[] = {{&S59595[44], 7}, {NULL}};
const Segment SGI46585[] = {{&S59595[44], 18}, {NULL}};
const Segment SGI46586[] = {{&S59595[30], 6}, {NULL}};
const Segment SGI46587[] = {{&S59595[110], 3}, {NULL}};
const Segment SGI46588[] = {{&S59595[76], 5}, {NULL}};
const Segment SGI46589[] = {{&S59596[38], 7}, {NULL}};
const Segment SGI46590[] = {{&S59596[31], 6}, {NULL}};
const Segment SGI46591[] = {{&S60297[44], 6}, {NULL}};
const Segment SGI46592[] = {{&S60297[44], 17}, {NULL}};
const Segment SGI46593[] = {{&S60297[30], 6}, {NULL}};
const Segment SGI46594[] = {{&S60297[109], 3}, {NULL}};
const Segment SGI46595[] = {{&S60297[75], 5}, {NULL}};
const Segment SGI46596[] = {{&S60298[44], 9}, {NULL}};
const Segment SGI46597[] = {{&S60298[44], 26}, {NULL}};
const Segment SGI46598[] = {{&S60298[30], 6}, {NULL}};
const Segment SGI46599[] = {{&S60298[118], 3}, {NULL}};
const Segment SGI46600[] = {{&S60298[84], 5}, {NULL}};
const Segment SGI46601[] = {{&S60300[38], 9}, {NULL}};
const Segment SGI46602[] = {{&S60300[31], 6}, {NULL}};
const Segment SGI46603[] = {{&S59770[44], 7}, {NULL}};
const Segment SGI46604[] = {{&S59770[44], 24}, {NULL}};
const Segment SGI46605[] = {{&S59770[30], 6}, {NULL}};
const Segment SGI46606[] = {{&S59770[116], 3}, {NULL}};
const Segment SGI46607[] = {{&S59770[82], 5}, {NULL}};
const Segment SGI46608[] = {{&S60436[44], 10}, {NULL}};
const Segment SGI46609[] = {{&S60436[44], 27}, {NULL}};
const Segment SGI46610[] = {{&S60436[30], 6}, {NULL}};
const Segment SGI46611[] = {{&S60436[119], 3}, {NULL}};
const Segment SGI46612[] = {{&S60436[85], 5}, {NULL}};
const Segment SGI46613[] = {{&S60437[42], 10}, {NULL}};
const Segment SGI46614[] = {{&S60437[30], 6}, {NULL}};
const Segment SGI46615[] = {{&S60437[117], 3}, {NULL}};
const Segment SGI46616[] = {{&S60437[83], 5}, {NULL}};
const Segment SGI46617[] = {{&S74185[44], 7}, {NULL}};
const Segment SGI46618[] = {{&S60438[38], 10}, {NULL}};
const Segment SGI46619[] = {{&S74185[44], 28}, {NULL}};
const Segment SGI46620[] = {{&S60438[31], 6}, {NULL}};
const Segment SGI46621[] = {{&S74185[30], 6}, {NULL}};
const Segment SGI46622[] = {{&S74185[120], 3}, {NULL}};
const Segment SGI46623[] = {{&S74185[86], 5}, {NULL}};
const Segment SGI46624[] = {{&S74186[44], 7}, {NULL}};
const Segment SGI46625[] = {{&S74186[44], 28}, {NULL}};
const Segment SGI46626[] = {{&S74186[30], 6}, {NULL}};
const Segment SGI46627[] = {{&S74186[120], 3}, {NULL}};
const Segment SGI46628[] = {{&S74186[86], 5}, {NULL}};
const Segment SGI46629[] = {{&S74187[44], 12}, {NULL}};
const Segment SGI46630[] = {{&S74187[44], 67}, {NULL}};
const Segment SGI46631[] = {{&S74187[30], 6}, {NULL}};
const Segment SGI46632[] = {{&S74187[75], 3}, {NULL}};
const Segment SGI46633[] = {{&S74187[125], 5}, {NULL}};
const Segment SGI46634[] = {{&S74188[44], 7}, {NULL}};
const Segment SGI46635[] = {{&S74188[44], 28}, {NULL}};
const Segment SGI46636[] = {{&S74188[30], 6}, {NULL}};
const Segment SGI46637[] = {{&S74188[120], 3}, {NULL}};
const Segment SGI46638[] = {{&S74188[86], 5}, {NULL}};
const Segment SGI46639[] = {{&S74189[44], 12}, {NULL}};
const Segment SGI46640[] = {{&S74189[44], 67}, {NULL}};
const Segment SGI46641[] = {{&S74189[30], 6}, {NULL}};
const Segment SGI46642[] = {{&S74189[75], 3}, {NULL}};
const Segment SGI46643[] = {{&S74189[125], 5}, {NULL}};
const Segment SGI46644[] = {{&S74190[44], 7}, {NULL}};
const Segment SGI46645[] = {{&S74190[44], 28}, {NULL}};
const Segment SGI46646[] = {{&S74190[30], 6}, {NULL}};
const Segment SGI46647[] = {{&S74190[120], 3}, {NULL}};
const Segment SGI46648[] = {{&S74190[86], 5}, {NULL}};
const Segment SGI46649[] = {{&S74191[44], 7}, {NULL}};
const Segment SGI46650[] = {{&S74191[44], 28}, {NULL}};
const Segment SGI46651[] = {{&S74191[30], 6}, {NULL}};
const Segment SGI46652[] = {{&S74191[120], 3}, {NULL}};
const Segment SGI46653[] = {{&S74191[86], 5}, {NULL}};
const Segment SGI46654[] = {{&S74192[44], 7}, {NULL}};
const Segment SGI46655[] = {{&S74192[44], 28}, {NULL}};
const Segment SGI46656[] = {{&S74192[30], 6}, {NULL}};
const Segment SGI46657[] = {{&S74192[120], 3}, {NULL}};
const Segment SGI46658[] = {{&S74192[86], 5}, {NULL}};
const Segment SGI46659[] = {{&S74193[44], 7}, {NULL}};
const Segment SGI46660[] = {{&S74193[44], 28}, {NULL}};
const Segment SGI46661[] = {{&S74193[30], 6}, {NULL}};
const Segment SGI46662[] = {{&S74193[89], 4}, {NULL}};
const Segment SGI46663[] = {{&S74193[89], 3}, {NULL}};
const Segment SGI46664[] = {{&S59820[44], 7}, {NULL}};
const Segment SGI46665[] = {{&S59820[44], 18}, {NULL}};
const Segment SGI46666[] = {{&S59820[30], 6}, {NULL}};
const Segment SGI46667[] = {{&S59820[110], 3}, {NULL}};
const Segment SGI46668[] = {{&S59820[76], 5}, {NULL}};
const Segment SGI46669[] = {{&S60548[44], 23}, {NULL}};
const Segment SGI46670[] = {{&S60548[44], 34}, {NULL}};
const Segment SGI46671[] = {{&S60548[30], 6}, {NULL}};
const Segment SGI46672[] = {{&S60548[126], 3}, {NULL}};
const Segment SGI46673[] = {{&S60548[92], 5}, {NULL}};
const Segment SGI46674[] = {{&S60553[44], 24}, {NULL}};
const Segment SGI46675[] = {{&S60553[44], 35}, {NULL}};
const Segment SGI46676[] = {{&S60553[30], 6}, {NULL}};
const Segment SGI46677[] = {{&S60553[127], 3}, {NULL}};
const Segment SGI46678[] = {{&S60553[93], 5}, {NULL}};
const Segment SGI46679[] = {{&S67839[44], 13}, {NULL}};
const Segment SGI46680[] = {{&S67839[44], 34}, {NULL}};
const Segment SGI46681[] = {{&S67839[30], 6}, {NULL}};
const Segment SGI46682[] = {{&S67839[126], 3}, {NULL}};
const Segment SGI46683[] = {{&S67839[92], 5}, {NULL}};
const Segment SGI46684[] = {{&S60592[44], 10}, {NULL}};
const Segment SGI46685[] = {{&S60592[44], 20}, {NULL}};
const Segment SGI46686[] = {{&S60592[30], 6}, {NULL}};
const Segment SGI46687[] = {{&S60592[112], 3}, {NULL}};
const Segment SGI46688[] = {{&S60592[78], 5}, {NULL}};
const Segment SGI46689[] = {{&S63386[44], 12}, {NULL}};
const Segment SGI46690[] = {{&S63386[44], 23}, {NULL}};
const Segment SGI46691[] = {{&S63386[30], 6}, {NULL}};
const Segment SGI46692[] = {{&S63386[115], 3}, {NULL}};
const Segment SGI46693[] = {{&S63386[81], 5}, {NULL}};
const Segment SGI46694[] = {{&S63387[44], 14}, {NULL}};
const Segment SGI46695[] = {{&S63387[44], 25}, {NULL}};
const Segment SGI46696[] = {{&S63387[30], 6}, {NULL}};
const Segment SGI46697[] = {{&S63387[117], 3}, {NULL}};
const Segment SGI46698[] = {{&S63387[83], 5}, {NULL}};
const Segment SGI46699[] = {{&S69051[44], 9}, {NULL}};
const Segment SGI46700[] = {{&S69051[44], 26}, {NULL}};
const Segment SGI46701[] = {{&S69051[30], 6}, {NULL}};
const Segment SGI46702[] = {{&S69051[118], 3}, {NULL}};
const Segment SGI46703[] = {{&S69051[84], 5}, {NULL}};
const Segment SGI46704[] = {{&S60010[44], 9}, {NULL}};
const Segment SGI46705[] = {{&S60010[44], 20}, {NULL}};
const Segment SGI46706[] = {{&S60010[30], 6}, {NULL}};
const Segment SGI46707[] = {{&S60010[112], 3}, {NULL}};
const Segment SGI46708[] = {{&S60010[78], 5}, {NULL}};
const Segment SGI46709[] = {{&S63547[44], 9}, {NULL}};
const Segment SGI46710[] = {{&S63547[44], 20}, {NULL}};
const Segment SGI46711[] = {{&S63547[30], 6}, {NULL}};
const Segment SGI46712[] = {{&S63547[112], 3}, {NULL}};
const Segment SGI46713[] = {{&S63547[78], 5}, {NULL}};
const Segment SGI46714[] = {{&S63548[44], 9}, {NULL}};
const Segment SGI46715[] = {{&S63548[44], 24}, {NULL}};
const Segment SGI46716[] = {{&S63548[30], 6}, {NULL}};
const Segment SGI46717[] = {{&S60064[44], 10}, {NULL}};
const Segment SGI46718[] = {{&S60064[44], 21}, {NULL}};
const Segment SGI46719[] = {{&S60064[30], 6}, {NULL}};
const Segment SGI46720[] = {{&S60064[113], 3}, {NULL}};
const Segment SGI46721[] = {{&S60064[79], 5}, {NULL}};
const Segment SGI46722[] = {{&S60066[44], 10}, {NULL}};
const Segment SGI46723[] = {{&S60066[44], 27}, {NULL}};
const Segment SGI46724[] = {{&S60066[30], 6}, {NULL}};
const Segment SGI46725[] = {{&S60066[119], 3}, {NULL}};
const Segment SGI46726[] = {{&S60066[85], 5}, {NULL}};
const Segment SGI46727[] = {{&S60857[44], 5}, {NULL}};
const Segment SGI46728[] = {{&S60857[44], 16}, {NULL}};
const Segment SGI46729[] = {{&S60857[30], 6}, {NULL}};
const Segment SGI46730[] = {{&S60857[108], 3}, {NULL}};
const Segment SGI46731[] = {{&S60857[74], 5}, {NULL}};
const Segment SGI46732[] = {{&S60858[44], 5}, {NULL}};
const Segment SGI46733[] = {{&S60858[44], 16}, {NULL}};
const Segment SGI46734[] = {{&S60858[30], 6}, {NULL}};
const Segment SGI46735[] = {{&S60858[77], 4}, {NULL}};
const Segment SGI46736[] = {{&S60858[77], 3}, {NULL}};
const Segment SGI46737[] = {{&S74722[44], 11}, {NULL}};
const Segment SGI46738[] = {{&S74722[44], 23}, {NULL}};
const Segment SGI46739[] = {{&S74722[30], 6}, {NULL}};
const Segment SGI46740[] = {{&S74722[115], 3}, {NULL}};
const Segment SGI46741[] = {{&S74722[81], 5}, {NULL}};
const Segment SGI46742[] = {{&S60859[38], 5}, {NULL}};
const Segment SGI46743[] = {{&S60859[31], 6}, {NULL}};
const Segment SGI46744[] = {{&S68624[44], 12}, {NULL}};
const Segment SGI46745[] = {{&S68624[44], 24}, {NULL}};
const Segment SGI46746[] = {{&S68624[30], 6}, {NULL}};
const Segment SGI46747[] = {{&S68624[116], 3}, {NULL}};
const Segment SGI46748[] = {{&S68624[82], 5}, {NULL}};
const Segment SGI46749[] = {{&S75043[44], 14}, {NULL}};
const Segment SGI46750[] = {{&S75043[44], 25}, {NULL}};
const Segment SGI46751[] = {{&S75043[30], 6}, {NULL}};
const Segment SGI46752[] = {{&S75043[117], 3}, {NULL}};
const Segment SGI46753[] = {{&S75043[83], 5}, {NULL}};
const Segment SGI46754[] = {{&S61326[44], 4}, {NULL}};
const Segment SGI46755[] = {{&S61326[44], 15}, {NULL}};
const Segment SGI46756[] = {{&S61326[30], 6}, {NULL}};
const Segment SGI46757[] = {{&S61326[107], 3}, {NULL}};
const Segment SGI46758[] = {{&S61326[73], 5}, {NULL}};
const Segment SGI46759[] = {{&S60590[44], 10}, {NULL}};
const Segment SGI46760[] = {{&S60590[44], 20}, {NULL}};
const Segment SGI46761[] = {{&S60590[30], 6}, {NULL}};
const Segment SGI46762[] = {{&S60590[112], 3}, {NULL}};
const Segment SGI46763[] = {{&S60590[78], 5}, {NULL}};
const Segment SGI46764[] = {{&S60595[38], 10}, {NULL}};
const Segment SGI46765[] = {{&S60595[31], 6}, {NULL}};
const Segment SGI46766[] = {{&S61415[44], 10}, {NULL}};
const Segment SGI46767[] = {{&S61415[44], 21}, {NULL}};
const Segment SGI46768[] = {{&S61415[30], 6}, {NULL}};
const Segment SGI46769[] = {{&S61415[113], 3}, {NULL}};
const Segment SGI46770[] = {{&S61415[79], 5}, {NULL}};
const Segment SGI46771[] = {{&S61416[38], 10}, {NULL}};
const Segment SGI46772[] = {{&S61416[31], 6}, {NULL}};
const Segment SGI46773[] = {{&S61430[44], 9}, {NULL}};
const Segment SGI46774[] = {{&S61430[44], 20}, {NULL}};
const Segment SGI46775[] = {{&S61430[30], 6}, {NULL}};
const Segment SGI46776[] = {{&S61430[112], 3}, {NULL}};
const Segment SGI46777[] = {{&S61430[78], 5}, {NULL}};
const Segment SGI46778[] = {{&S61436[44], 10}, {NULL}};
const Segment SGI46779[] = {{&S61436[44], 21}, {NULL}};
const Segment SGI46780[] = {{&S61436[30], 6}, {NULL}};
const Segment SGI46781[] = {{&S61436[113], 3}, {NULL}};
const Segment SGI46782[] = {{&S61436[79], 5}, {NULL}};
const Segment SGI46783[] = {{&S61437[44], 10}, {NULL}};
const Segment SGI46784[] = {{&S61437[44], 21}, {NULL}};
const Segment SGI46785[] = {{&S61437[30], 6}, {NULL}};
const Segment SGI46786[] = {{&S61437[113], 3}, {NULL}};
const Segment SGI46787[] = {{&S61437[79], 5}, {NULL}};
const Segment SGI46788[] = {{&S61438[44], 10}, {NULL}};
const Segment SGI46789[] = {{&S61438[44], 21}, {NULL}};
const Segment SGI46790[] = {{&S61438[30], 6}, {NULL}};
const Segment SGI46791[] = {{&S61438[113], 3}, {NULL}};
const Segment SGI46792[] = {{&S61438[79], 5}, {NULL}};
const Segment SGI46793[] = {{&S61439[44], 10}, {NULL}};
const Segment SGI46794[] = {{&S61439[44], 21}, {NULL}};
const Segment SGI46795[] = {{&S61439[30], 6}, {NULL}};
const Segment SGI46796[] = {{&S61439[113], 3}, {NULL}};
const Segment SGI46797[] = {{&S61439[79], 5}, {NULL}};
const Segment SGI46798[] = {{&S61440[44], 10}, {NULL}};
const Segment SGI46799[] = {{&S61440[44], 21}, {NULL}};
const Segment SGI46800[] = {{&S61440[30], 6}, {NULL}};
const Segment SGI46801[] = {{&S61440[113], 3}, {NULL}};
const Segment SGI46802[] = {{&S61440[79], 5}, {NULL}};
const Segment SGI46803[] = {{&S61441[44], 11}, {NULL}};
const Segment SGI46804[] = {{&S61441[44], 22}, {NULL}};
const Segment SGI46805[] = {{&S61441[30], 6}, {NULL}};
const Segment SGI46806[] = {{&S61441[114], 3}, {NULL}};
const Segment SGI46807[] = {{&S61441[80], 5}, {NULL}};
const Segment SGI46808[] = {{&S61442[44], 11}, {NULL}};
const Segment SGI46809[] = {{&S61442[44], 22}, {NULL}};
const Segment SGI46810[] = {{&S61442[30], 6}, {NULL}};
const Segment SGI46811[] = {{&S61442[114], 3}, {NULL}};
const Segment SGI46812[] = {{&S61442[80], 5}, {NULL}};
const Segment SGI46813[] = {{&S61443[44], 11}, {NULL}};
const Segment SGI46814[] = {{&S61443[44], 22}, {NULL}};
const Segment SGI46815[] = {{&S61443[30], 6}, {NULL}};
const Segment SGI46816[] = {{&S61443[114], 3}, {NULL}};
const Segment SGI46817[] = {{&S61443[80], 5}, {NULL}};
const Segment SGI46818[] = {{&S61444[44], 11}, {NULL}};
const Segment SGI46819[] = {{&S61444[44], 22}, {NULL}};
const Segment SGI46820[] = {{&S61444[30], 6}, {NULL}};
const Segment SGI46821[] = {{&S61444[114], 3}, {NULL}};
const Segment SGI46822[] = {{&S61444[80], 5}, {NULL}};
const Segment SGI46823[] = {{&S61445[38], 10}, {NULL}};
const Segment SGI46824[] = {{&S61445[31], 6}, {NULL}};
const Segment SGI46825[] = {{&S61446[44], 10}, {NULL}};
const Segment SGI46826[] = {{&S61446[44], 21}, {NULL}};
const Segment SGI46827[] = {{&S61446[30], 6}, {NULL}};
const Segment SGI46828[] = {{&S61446[113], 3}, {NULL}};
const Segment SGI46829[] = {{&S61446[79], 5}, {NULL}};
const Segment SGI46830[] = {{&S61447[44], 10}, {NULL}};
const Segment SGI46831[] = {{&S61447[44], 21}, {NULL}};
const Segment SGI46832[] = {{&S61447[30], 6}, {NULL}};
const Segment SGI46833[] = {{&S61447[113], 3}, {NULL}};
const Segment SGI46834[] = {{&S61447[79], 5}, {NULL}};
const Segment SGI46835[] = {{&S61448[44], 11}, {NULL}};
const Segment SGI46836[] = {{&S61448[44], 22}, {NULL}};
const Segment SGI46837[] = {{&S61448[30], 6}, {NULL}};
const Segment SGI46838[] = {{&S61448[114], 3}, {NULL}};
const Segment SGI46839[] = {{&S61448[80], 5}, {NULL}};
const Segment SGI46840[] = {{&S61449[44], 11}, {NULL}};
const Segment SGI46841[] = {{&S61449[44], 22}, {NULL}};
const Segment SGI46842[] = {{&S61449[30], 6}, {NULL}};
const Segment SGI46843[] = {{&S61449[114], 3}, {NULL}};
const Segment SGI46844[] = {{&S61449[80], 5}, {NULL}};
const Segment SGI46845[] = {{&S64221[44], 19}, {NULL}};
const Segment SGI46846[] = {{&S64221[44], 30}, {NULL}};
const Segment SGI46847[] = {{&S64221[30], 6}, {NULL}};
const Segment SGI46848[] = {{&S64221[122], 3}, {NULL}};
const Segment SGI46849[] = {{&S64221[88], 5}, {NULL}};
const Segment SGI46850[] = {{&S68810[42], 20}, {NULL}};
const Segment SGI46851[] = {{&S68810[42], 31}, {NULL}};
const Segment SGI46852[] = {{&S68810[30], 4}, {NULL}};
const Segment SGI46853[] = {{&S68810[121], 3}, {NULL}};
const Segment SGI46854[] = {{&S68810[87], 5}, {NULL}};
const Segment SGI46855[] = {{&S61492[44], 7}, {NULL}};
const Segment SGI46856[] = {{&S61492[44], 23}, {NULL}};
const Segment SGI46857[] = {{&S61492[30], 6}, {NULL}};
const Segment SGI46858[] = {{&S61492[115], 3}, {NULL}};
const Segment SGI46859[] = {{&S61492[81], 5}, {NULL}};
const Segment SGI46860[] = {{&S61493[44], 7}, {NULL}};
const Segment SGI46861[] = {{&S61493[44], 23}, {NULL}};
const Segment SGI46862[] = {{&S61493[30], 6}, {NULL}};
const Segment SGI46863[] = {{&S61493[115], 3}, {NULL}};
const Segment SGI46864[] = {{&S61493[81], 5}, {NULL}};
const Segment SGI46865[] = {{&S61494[44], 7}, {NULL}};
const Segment SGI46866[] = {{&S61494[44], 23}, {NULL}};
const Segment SGI46867[] = {{&S61494[30], 6}, {NULL}};
const Segment SGI46868[] = {{&S61494[115], 3}, {NULL}};
const Segment SGI46869[] = {{&S61494[81], 5}, {NULL}};
const Segment SGI46870[] = {{&S61495[44], 7}, {NULL}};
const Segment SGI46871[] = {{&S61495[44], 23}, {NULL}};
const Segment SGI46872[] = {{&S61495[30], 6}, {NULL}};
const Segment SGI46873[] = {{&S61495[115], 3}, {NULL}};
const Segment SGI46874[] = {{&S61495[81], 5}, {NULL}};
const Segment SGI46875[] = {{&S61496[44], 7}, {NULL}};
const Segment SGI46876[] = {{&S61496[44], 23}, {NULL}};
const Segment SGI46877[] = {{&S61496[30], 6}, {NULL}};
const Segment SGI46878[] = {{&S61496[115], 3}, {NULL}};
const Segment SGI46879[] = {{&S61496[81], 5}, {NULL}};
const Segment SGI46880[] = {{&S61497[38], 7}, {NULL}};
const Segment SGI46881[] = {{&S61497[31], 6}, {NULL}};
const Segment SGI46882[] = {{&S65693[6], 4}, {NULL}};
const Segment SGI46883[] = {{&S65693[85], 5}, {NULL}};
const Segment SGI46884[] = {{&S75278[44], 27}, {NULL}};
const Segment SGI46885[] = {{&S75278[44], 38}, {NULL}};
const Segment SGI46886[] = {{&S75278[30], 6}, {&S75278[52], 30}, {NULL}};
const Segment SGI46887[] = {{&S75278[130], 3}, {NULL}};
const Segment SGI46888[] = {{&S75278[96], 5}, {NULL}};
const Segment SGI46889[] = {{&S64272[44], 14}, {NULL}};
const Segment SGI46890[] = {{&S64272[44], 25}, {NULL}};
const Segment SGI46891[] = {{&S64272[30], 6}, {NULL}};
const Segment SGI46892[] = {{&S64272[117], 3}, {NULL}};
const Segment SGI46893[] = {{&S64272[83], 5}, {NULL}};
const Segment SGI46894[] = {{&S60862[44], 7}, {NULL}};
const Segment SGI46895[] = {{&S60862[44], 24}, {NULL}};
const Segment SGI46896[] = {{&S60862[30], 6}, {NULL}};
const Segment SGI46897[] = {{&S60862[116], 3}, {NULL}};
const Segment SGI46898[] = {{&S60862[82], 5}, {NULL}};
const Segment SGI46899[] = {{&S64380[44], 12}, {NULL}};
const Segment SGI46900[] = {{&S64380[44], 23}, {NULL}};
const Segment SGI46901[] = {{&S60863[38], 7}, {NULL}};
const Segment SGI46902[] = {{&S64380[30], 6}, {NULL}};
const Segment SGI46903[] = {{&S60863[31], 6}, {NULL}};
const Segment SGI46904[] = {{&S64380[115], 3}, {NULL}};
const Segment SGI46905[] = {{&S64380[81], 5}, {NULL}};
const Segment SGI46906[] = {{&S60864[49], 7}, {NULL}};
const Segment SGI46907[] = {{&S64381[38], 12}, {NULL}};
const Segment SGI46908[] = {{&S60864[49], 24}, {NULL}};
const Segment SGI46909[] = {{&S60864[30], 11}, {NULL}};
const Segment SGI46910[] = {{&S64381[31], 6}, {NULL}};
const Segment SGI46911[] = {{&S60864[121], 3}, {NULL}};
const Segment SGI46912[] = {{&S60864[87], 5}, {NULL}};
const Segment SGI46913[] = {{&S65874[44], 10}, {NULL}};
const Segment SGI46914[] = {{&S65874[44], 22}, {NULL}};
const Segment SGI46915[] = {{&S65874[30], 6}, {NULL}};
const Segment SGI46916[] = {{&S65874[114], 3}, {NULL}};
const Segment SGI46917[] = {{&S65874[80], 5}, {NULL}};
const Segment SGI46918[] = {{&S65875[38], 10}, {NULL}};
const Segment SGI46919[] = {{&S65875[31], 6}, {NULL}};
const Segment SGI46920[] = {{&S60957[44], 9}, {NULL}};
const Segment SGI46921[] = {{&S60957[44], 20}, {NULL}};
const Segment SGI46922[] = {{&S60957[30], 6}, {NULL}};
const Segment SGI46923[] = {{&S60957[112], 3}, {NULL}};
const Segment SGI46924[] = {{&S60957[78], 5}, {NULL}};
const Segment SGI46925[] = {{&S60958[44], 17}, {NULL}};
const Segment SGI46926[] = {{&S60958[44], 28}, {NULL}};
const Segment SGI46927[] = {{&S60958[30], 6}, {NULL}};
const Segment SGI46928[] = {{&S60958[120], 3}, {NULL}};
const Segment SGI46929[] = {{&S60958[86], 5}, {NULL}};
const Segment SGI46930[] = {{&S75510[44], 6}, {NULL}};
const Segment SGI46931[] = {{&S75510[44], 18}, {NULL}};
const Segment SGI46932[] = {{&S75510[30], 6}, {NULL}};
const Segment SGI46933[] = {{&S75510[110], 3}, {NULL}};
const Segment SGI46934[] = {{&S75510[76], 5}, {NULL}};
const Segment SGI46935[] = {{&S61037[44], 12}, {NULL}};
const Segment SGI46936[] = {{&S61037[44], 33}, {NULL}};
const Segment SGI46937[] = {{&S61037[30], 6}, {NULL}};
const Segment SGI46938[] = {{&S61037[125], 3}, {NULL}};
const Segment SGI46939[] = {{&S61037[91], 5}, {NULL}};
const Segment SGI46940[] = {{&S75587[44], 14}, {NULL}};
const Segment SGI46941[] = {{&S75587[44], 25}, {NULL}};
const Segment SGI46942[] = {{&S75587[30], 6}, {NULL}};
const Segment SGI46943[] = {{&S75587[117], 3}, {NULL}};
const Segment SGI46944[] = {{&S75587[83], 5}, {NULL}};
const Segment SGI46945[] = {{&S75588[44], 14}, {NULL}};
const Segment SGI46946[] = {{&S75588[44], 25}, {NULL}};
const Segment SGI46947[] = {{&S75588[30], 6}, {NULL}};
const Segment SGI46948[] = {{&S75588[86], 4}, {NULL}};
const Segment SGI46949[] = {{&S75588[86], 3}, {NULL}};
const Segment SGI46950[] = {{&S75589[44], 14}, {NULL}};
const Segment SGI46951[] = {{&S75589[44], 25}, {NULL}};
const Segment SGI46952[] = {{&S75589[30], 6}, {NULL}};
const Segment SGI46953[] = {{&S75589[86], 6}, {NULL}};
const Segment SGI46954[] = {{&S61108[44], 10}, {NULL}};
const Segment SGI46955[] = {{&S61108[44], 21}, {NULL}};
const Segment SGI46956[] = {{&S61108[30], 6}, {NULL}};
const Segment SGI46957[] = {{&S61108[113], 3}, {NULL}};
const Segment SGI46958[] = {{&S61108[79], 5}, {NULL}};
const Segment SGI46959[] = {{&S61904[52], 9}, {NULL}};
const Segment SGI46960[] = {{&S61904[52], 20}, {NULL}};
const Segment SGI46961[] = {{&S61904[38], 6}, {NULL}};
const Segment SGI46962[] = {{&S61904[120], 3}, {NULL}};
const Segment SGI46963[] = {{&S61904[86], 5}, {NULL}};
const Segment SGI46964[] = {{&S61109[44], 10}, {NULL}};
const Segment SGI46965[] = {{&S61109[44], 21}, {NULL}};
const Segment SGI46966[] = {{&S61109[30], 6}, {NULL}};
const Segment SGI46967[] = {{&S61109[113], 3}, {NULL}};
const Segment SGI46968[] = {{&S61109[79], 5}, {NULL}};
const Segment SGI46969[] = {{&S61110[44], 10}, {NULL}};
const Segment SGI46970[] = {{&S61110[44], 21}, {NULL}};
const Segment SGI46971[] = {{&S61110[30], 6}, {NULL}};
const Segment SGI46972[] = {{&S61110[113], 3}, {NULL}};
const Segment SGI46973[] = {{&S61110[79], 5}, {NULL}};
const Segment SGI46974[] = {{&S61111[44], 10}, {NULL}};
const Segment SGI46975[] = {{&S61111[44], 21}, {NULL}};
const Segment SGI46976[] = {{&S61111[30], 6}, {NULL}};
const Segment SGI46977[] = {{&S61111[113], 3}, {NULL}};
const Segment SGI46978[] = {{&S61111[79], 5}, {NULL}};
const Segment SGI46979[] = {{&S61112[44], 10}, {NULL}};
const Segment SGI46980[] = {{&S61112[44], 21}, {NULL}};
const Segment SGI46981[] = {{&S61112[30], 6}, {NULL}};
const Segment SGI46982[] = {{&S61112[113], 3}, {NULL}};
const Segment SGI46983[] = {{&S61112[79], 5}, {NULL}};
const Segment SGI46984[] = {{&S61113[38], 10}, {NULL}};
const Segment SGI46985[] = {{&S61113[31], 6}, {NULL}};
const Segment SGI46986[] = {{&S61114[44], 11}, {NULL}};
const Segment SGI46987[] = {{&S61114[44], 22}, {NULL}};
const Segment SGI46988[] = {{&S61114[30], 6}, {NULL}};
const Segment SGI46989[] = {{&S61114[114], 3}, {NULL}};
const Segment SGI46990[] = {{&S61114[80], 5}, {NULL}};
const Segment SGI46991[] = {{&S61115[38], 11}, {NULL}};
const Segment SGI46992[] = {{&S61115[31], 6}, {NULL}};
const Segment SGI46993[] = {{&S61953[44], 8}, {NULL}};
const Segment SGI46994[] = {{&S61953[44], 25}, {NULL}};
const Segment SGI46995[] = {{&S61953[30], 6}, {NULL}};
const Segment SGI46996[] = {{&S61953[117], 3}, {NULL}};
const Segment SGI46997[] = {{&S61953[83], 5}, {NULL}};
const Segment SGI46998[] = {{&S75770[44], 7}, {NULL}};
const Segment SGI46999[] = {{&S75770[44], 24}, {NULL}};
const Segment SGI47000[] = {{&S75770[30], 6}, {NULL}};
const Segment SGI47001[] = {{&S75770[116], 3}, {NULL}};
const Segment SGI47002[] = {{&S75770[82], 5}, {NULL}};
const Segment SGI47003[] = {{&S61293[44], 7}, {NULL}};
const Segment SGI47004[] = {{&S61293[44], 18}, {NULL}};
const Segment SGI47005[] = {{&S61293[30], 6}, {NULL}};
const Segment SGI47006[] = {{&S61293[110], 3}, {NULL}};
const Segment SGI47007[] = {{&S61293[76], 5}, {NULL}};
const Segment SGI47008[] = {{&S61294[44], 7}, {NULL}};
const Segment SGI47009[] = {{&S61294[44], 20}, {NULL}};
const Segment SGI47010[] = {{&S61294[30], 6}, {NULL}};
const Segment SGI47011[] = {{&S66271[44], 27}, {NULL}};
const Segment SGI47012[] = {{&S61294[112], 3}, {NULL}};
const Segment SGI47013[] = {{&S66271[44], 38}, {NULL}};
const Segment SGI47014[] = {{&S66271[30], 6}, {NULL}};
const Segment SGI47015[] = {{&S61294[78], 5}, {NULL}};
const Segment SGI47016[] = {{&S66271[130], 3}, {NULL}};
const Segment SGI47017[] = {{&S66271[96], 5}, {NULL}};
const Segment SGI47018[] = {{&S61295[44], 7}, {NULL}};
const Segment SGI47019[] = {{&S61295[44], 20}, {NULL}};
const Segment SGI47020[] = {{&S61295[30], 6}, {NULL}};
const Segment SGI47021[] = {{&S61295[112], 3}, {NULL}};
const Segment SGI47022[] = {{&S61295[78], 5}, {NULL}};
const Segment SGI47023[] = {{&S61296[38], 7}, {NULL}};
const Segment SGI47024[] = {{&S61296[31], 6}, {NULL}};
const Segment SGI47025[] = {{&S62080[6], 4}, {NULL}};
const Segment SGI47026[] = {{&S62080[87], 5}, {NULL}};
const Segment SGI47027[] = {{&S61328[44], 10}, {NULL}};
const Segment SGI47028[] = {{&S61328[44], 31}, {NULL}};
const Segment SGI47029[] = {{&S61328[30], 6}, {NULL}};
const Segment SGI47030[] = {{&S61328[123], 3}, {NULL}};
const Segment SGI47031[] = {{&S61328[89], 5}, {NULL}};
const Segment SGI47032[] = {{&S75824[44], 5}, {NULL}};
const Segment SGI47033[] = {{&S75824[44], 24}, {NULL}};
const Segment SGI47034[] = {{&S75824[30], 6}, {NULL}};
const Segment SGI47035[] = {{&S75824[116], 3}, {NULL}};
const Segment SGI47036[] = {{&S75824[82], 5}, {NULL}};
const Segment SGI47037[] = {{&S75825[44], 5}, {NULL}};
const Segment SGI47038[] = {{&S75825[44], 24}, {NULL}};
const Segment SGI47039[] = {{&S75825[30], 6}, {NULL}};
const Segment SGI47040[] = {{&S75825[62], 3}, {NULL}};
const Segment SGI47041[] = {{&S75825[82], 5}, {NULL}};
const Segment SGI47042[] = {{&S62161[44], 9}, {NULL}};
const Segment SGI47043[] = {{&S62161[44], 20}, {NULL}};
const Segment SGI47044[] = {{&S62161[30], 6}, {NULL}};
const Segment SGI47045[] = {{&S62161[112], 3}, {NULL}};
const Segment SGI47046[] = {{&S62161[78], 5}, {NULL}};
const Segment SGI47047[] = {{&S64741[44], 7}, {NULL}};
const Segment SGI47048[] = {{&S64741[44], 18}, {NULL}};
const Segment SGI47049[] = {{&S64741[30], 6}, {NULL}};
const Segment SGI47050[] = {{&S64741[110], 3}, {NULL}};
const Segment SGI47051[] = {{&S64741[76], 5}, {NULL}};
const Segment SGI47052[] = {{&S64742[38], 7}, {NULL}};
const Segment SGI47053[] = {{&S64742[31], 6}, {NULL}};
const Segment SGI47054[] = {{&S62193[44], 10}, {NULL}};
const Segment SGI47055[] = {{&S62193[44], 27}, {NULL}};
const Segment SGI47056[] = {{&S62193[30], 6}, {NULL}};
const Segment SGI47057[] = {{&S62193[85], 6}, {NULL}};
const Segment SGI47058[] = {{&S62193[112], 13}, {NULL}};
const Segment SGI47059[] = {{&S62194[44], 11}, {NULL}};
const Segment SGI47060[] = {{&S62194[44], 28}, {NULL}};
const Segment SGI47061[] = {{&S62194[30], 6}, {NULL}};
const Segment SGI47062[] = {{&S62194[86], 6}, {NULL}};
const Segment SGI47063[] = {{&S62194[113], 13}, {NULL}};
const Segment SGI47064[] = {{&S62195[44], 7}, {NULL}};
const Segment SGI47065[] = {{&S62195[44], 19}, {NULL}};
const Segment SGI47066[] = {{&S62195[30], 6}, {NULL}};
const Segment SGI47067[] = {{&S62195[111], 3}, {NULL}};
const Segment SGI47068[] = {{&S62195[77], 5}, {NULL}};
const Segment SGI47069[] = {{&S62196[44], 7}, {NULL}};
const Segment SGI47070[] = {{&S62196[44], 25}, {NULL}};
const Segment SGI47071[] = {{&S62196[30], 6}, {NULL}};
const Segment SGI47072[] = {{&S62196[117], 3}, {NULL}};
const Segment SGI47073[] = {{&S62196[83], 5}, {NULL}};
const Segment SGI47074[] = {{&S62198[38], 7}, {NULL}};
const Segment SGI47075[] = {{&S62198[31], 6}, {NULL}};
const Segment SGI47076[] = {{&S62201[30], 6}, {NULL}};
const Segment SGI47077[] = {{&S62201[92], 2}, {NULL}};
const Segment SGI47078[] = {{&S62201[57], 6}, {NULL}};
const Segment SGI47079[] = {{&S62308[44], 10}, {NULL}};
const Segment SGI47080[] = {{&S62308[44], 21}, {NULL}};
const Segment SGI47081[] = {{&S62308[30], 6}, {NULL}};
const Segment SGI47082[] = {{&S62308[113], 3}, {NULL}};
const Segment SGI47083[] = {{&S62308[79], 5}, {NULL}};
const Segment SGI47084[] = {{&S62309[38], 10}, {NULL}};
const Segment SGI47085[] = {{&S62309[31], 6}, {NULL}};
const Segment SGI47086[] = {{&S64776[44], 10}, {NULL}};
const Segment SGI47087[] = {{&S64776[44], 21}, {NULL}};
const Segment SGI47088[] = {{&S64776[30], 6}, {NULL}};
const Segment SGI47089[] = {{&S64776[114], 3}, {NULL}};
const Segment SGI47090[] = {{&S64776[79], 6}, {NULL}};
const Segment SGI47091[] = {{&S64776[79], 5}, {NULL}};
const Segment SGI47092[] = {{&S62333[44], 10}, {NULL}};
const Segment SGI47093[] = {{&S62333[44], 27}, {NULL}};
const Segment SGI47094[] = {{&S62333[30], 6}, {NULL}};
const Segment SGI47095[] = {{&S62333[149], 4}, {NULL}};
const Segment SGI47096[] = {{&S62333[175], 3}, {NULL}};
const Segment SGI47097[] = {{&S62333[119], 3}, {&S62333[224], 5}, {NULL}};
const Segment SGI47098[] = {{&S62333[85], 5}, {NULL}};
const Segment SGI47099[] = {{&S76185[44], 7}, {NULL}};
const Segment SGI47100[] = {{&S76185[44], 24}, {NULL}};
const Segment SGI47101[] = {{&S76185[30], 6}, {NULL}};
const Segment SGI47102[] = {{&S76185[116], 3}, {NULL}};
const Segment SGI47103[] = {{&S76185[82], 5}, {NULL}};
const Segment SGI47104[] = {{&S70643[44], 11}, {NULL}};
const Segment SGI47105[] = {{&S70643[44], 28}, {NULL}};
const Segment SGI47106[] = {{&S70643[30], 6}, {NULL}};
const Segment SGI47107[] = {{&S70643[120], 3}, {NULL}};
const Segment SGI47108[] = {{&S70643[86], 5}, {NULL}};
const Segment SGI47109[] = {{&S70644[44], 9}, {NULL}};
const Segment SGI47110[] = {{&S70644[44], 26}, {NULL}};
const Segment SGI47111[] = {{&S70644[30], 6}, {NULL}};
const Segment SGI47112[] = {{&S70644[118], 3}, {NULL}};
const Segment SGI47113[] = {{&S61772[44], 10}, {NULL}};
const Segment SGI47114[] = {{&S70644[84], 5}, {NULL}};
const Segment SGI47115[] = {{&S61772[44], 21}, {NULL}};
const Segment SGI47116[] = {{&S61772[30], 6}, {NULL}};
const Segment SGI47117[] = {{&S61772[113], 3}, {NULL}};
const Segment SGI47118[] = {{&S61772[79], 5}, {NULL}};
const Segment SGI47119[] = {{&S76265[44], 6}, {NULL}};
const Segment SGI47120[] = {{&S76265[44], 17}, {NULL}};
const Segment SGI47121[] = {{&S76265[30], 6}, {NULL}};
const Segment SGI47122[] = {{&S76265[109], 3}, {NULL}};
const Segment SGI47123[] = {{&S76265[75], 5}, {NULL}};
const Segment SGI47124[] = {{&S65035[44], 10}, {NULL}};
const Segment SGI47125[] = {{&S65035[44], 27}, {NULL}};
const Segment SGI47126[] = {{&S65035[30], 6}, {NULL}};
const Segment SGI47127[] = {{&S65035[85], 6}, {NULL}};
const Segment SGI47128[] = {{&S65035[122], 3}, {NULL}};
const Segment SGI47129[] = {{&S70806[38], 10}, {NULL}};
const Segment SGI47130[] = {{&S70806[31], 6}, {NULL}};
const Segment SGI47131[] = {{&S69972[42], 12}, {NULL}};
const Segment SGI47132[] = {{&S69972[42], 23}, {NULL}};
const Segment SGI47133[] = {{&S69972[30], 4}, {NULL}};
const Segment SGI47134[] = {{&S69972[113], 3}, {NULL}};
const Segment SGI47135[] = {{&S69972[79], 5}, {NULL}};
const Segment SGI47136[] = {{&S62664[44], 14}, {NULL}};
const Segment SGI47137[] = {{&S62664[44], 25}, {NULL}};
const Segment SGI47138[] = {{&S62664[30], 6}, {NULL}};
const Segment SGI47139[] = {{&S62664[117], 3}, {NULL}};
const Segment SGI47140[] = {{&S62664[83], 5}, {NULL}};
const Segment SGI47141[] = {{&S70784[44], 10}, {NULL}};
const Segment SGI47142[] = {{&S70784[44], 21}, {NULL}};
const Segment SGI47143[] = {{&S70784[30], 6}, {NULL}};
const Segment SGI47144[] = {{&S70784[113], 3}, {NULL}};
const Segment SGI47145[] = {{&S70784[79], 5}, {NULL}};
const Segment SGI47146[] = {{&S62008[44], 11}, {NULL}};
const Segment SGI47147[] = {{&S62008[44], 28}, {NULL}};
const Segment SGI47148[] = {{&S62008[30], 6}, {NULL}};
const Segment SGI47149[] = {{&S70787[44], 10}, {NULL}};
const Segment SGI47150[] = {{&S70787[44], 21}, {NULL}};
const Segment SGI47151[] = {{&S62008[120], 3}, {NULL}};
const Segment SGI47152[] = {{&S70787[30], 6}, {NULL}};
const Segment SGI47153[] = {{&S62008[86], 5}, {NULL}};
const Segment SGI47154[] = {{&S70787[82], 6}, {NULL}};
const Segment SGI47155[] = {{&S62009[44], 11}, {NULL}};
const Segment SGI47156[] = {{&S62009[44], 28}, {NULL}};
const Segment SGI47157[] = {{&S62009[30], 6}, {NULL}};
const Segment SGI47158[] = {{&S62009[89], 4}, {NULL}};
const Segment SGI47159[] = {{&S62009[89], 3}, {NULL}};
const Segment SGI47160[] = {{&S62010[44], 11}, {NULL}};
const Segment SGI47161[] = {{&S62010[44], 28}, {NULL}};
const Segment SGI47162[] = {{&S62010[30], 6}, {NULL}};
const Segment SGI47163[] = {{&S62010[89], 6}, {NULL}};
const Segment SGI47164[] = {{&S62011[38], 11}, {NULL}};
const Segment SGI47165[] = {{&S62011[31], 6}, {NULL}};
const Segment SGI47166[] = {{&S62012[44], 28}, {NULL}};
const Segment SGI47167[] = {{&S62012[30], 6}, {NULL}};
const Segment SGI47168[] = {{&S70098[42], 9}, {NULL}};
const Segment SGI47169[] = {{&S70098[42], 26}, {NULL}};
const Segment SGI47170[] = {{&S70098[30], 4}, {NULL}};
const Segment SGI47171[] = {{&S70098[116], 3}, {NULL}};
const Segment SGI47172[] = {{&S70098[82], 5}, {NULL}};
const Segment SGI47173[] = {{&S70099[36], 9}, {NULL}};
const Segment SGI47174[] = {{&S70099[31], 4}, {NULL}};
const Segment SGI47175[] = {{&S62077[6], 4}, {NULL}};
const Segment SGI47176[] = {{&S65324[34], 11}, {NULL}};
const Segment SGI47177[] = {{&S62077[82], 5}, {NULL}};
const Segment SGI47178[] = {{&S65324[27], 6}, {NULL}};
const Segment SGI47179[] = {{&S65324[104], 2}, {NULL}};
const Segment SGI47180[] = {{&S65324[70], 6}, {NULL}};
const Segment SGI47181[] = {{&S65324[138], 8}, {NULL}};
const Segment SGI47182[] = {{&S62162[44], 9}, {NULL}};
const Segment SGI47183[] = {{&S62162[44], 20}, {NULL}};
const Segment SGI47184[] = {{&S62162[30], 6}, {NULL}};
const Segment SGI47185[] = {{&S62162[112], 3}, {NULL}};
const Segment SGI47186[] = {{&S62162[78], 5}, {NULL}};
const Segment SGI47187[] = {{&S62163[38], 9}, {NULL}};
const Segment SGI47188[] = {{&S62163[31], 6}, {NULL}};
const Segment SGI47189[] = {{&S62190[44], 10}, {NULL}};
const Segment SGI47190[] = {{&S62190[44], 27}, {NULL}};
const Segment SGI47191[] = {{&S62190[30], 6}, {NULL}};
const Segment SGI47192[] = {{&S62190[85], 6}, {NULL}};
const Segment SGI47193[] = {{&S62190[112], 13}, {NULL}};
const Segment SGI47194[] = {{&S62197[44], 7}, {NULL}};
const Segment SGI47195[] = {{&S62197[44], 26}, {NULL}};
const Segment SGI47196[] = {{&S62197[30], 6}, {NULL}};
const Segment SGI47197[] = {{&S62197[118], 3}, {NULL}};
const Segment SGI47198[] = {{&S62197[84], 5}, {NULL}};
const Segment SGI47199[] = {{&S62199[38], 7}, {NULL}};
const Segment SGI47200[] = {{&S62199[31], 6}, {NULL}};
const Segment SGI47201[] = {{&S62202[30], 6}, {NULL}};
const Segment SGI47202[] = {{&S62202[92], 2}, {NULL}};
const Segment SGI47203[] = {{&S62202[57], 6}, {NULL}};
const Segment SGI47204[] = {{&S65512[44], 10}, {NULL}};
const Segment SGI47205[] = {{&S65512[44], 27}, {NULL}};
const Segment SGI47206[] = {{&S65512[30], 6}, {NULL}};
const Segment SGI47207[] = {{&S65512[85], 6}, {NULL}};
const Segment SGI47208[] = {{&S65512[122], 3}, {NULL}};
const Segment SGI47209[] = {{&S62277[44], 8}, {NULL}};
const Segment SGI47210[] = {{&S62277[44], 19}, {NULL}};
const Segment SGI47211[] = {{&S62277[30], 6}, {NULL}};
const Segment SGI47212[] = {{&S62277[111], 3}, {NULL}};
const Segment SGI47213[] = {{&S62277[77], 5}, {NULL}};
const Segment SGI47214[] = {{&S62278[44], 8}, {NULL}};
const Segment SGI47215[] = {{&S62278[44], 19}, {NULL}};
const Segment SGI47216[] = {{&S62278[30], 6}, {NULL}};
const Segment SGI47217[] = {{&S62278[111], 3}, {NULL}};
const Segment SGI47218[] = {{&S62278[77], 5}, {NULL}};
const Segment SGI47219[] = {{&S76802[44], 8}, {NULL}};
const Segment SGI47220[] = {{&S76802[44], 20}, {NULL}};
const Segment SGI47221[] = {{&S76802[30], 6}, {NULL}};
const Segment SGI47222[] = {{&S76802[112], 3}, {NULL}};
const Segment SGI47223[] = {{&S76802[78], 5}, {NULL}};
const Segment SGI47224[] = {{&S62279[44], 8}, {NULL}};
const Segment SGI47225[] = {{&S62279[44], 19}, {NULL}};
const Segment SGI47226[] = {{&S62279[30], 6}, {NULL}};
const Segment SGI47227[] = {{&S62279[111], 3}, {NULL}};
const Segment SGI47228[] = {{&S62279[77], 5}, {NULL}};
const Segment SGI47229[] = {{&S76803[44], 7}, {NULL}};
const Segment SGI47230[] = {{&S76803[44], 19}, {NULL}};
const Segment SGI47231[] = {{&S76803[30], 6}, {NULL}};
const Segment SGI47232[] = {{&S76803[111], 3}, {NULL}};
const Segment SGI47233[] = {{&S76803[77], 5}, {NULL}};
const Segment SGI47234[] = {{&S62280[44], 8}, {NULL}};
const Segment SGI47235[] = {{&S62280[44], 19}, {NULL}};
const Segment SGI47236[] = {{&S62280[30], 6}, {NULL}};
const Segment SGI47237[] = {{&S62280[111], 3}, {NULL}};
const Segment SGI47238[] = {{&S62280[77], 5}, {NULL}};
const Segment SGI47239[] = {{&S62281[44], 8}, {NULL}};
const Segment SGI47240[] = {{&S62281[44], 19}, {NULL}};
const Segment SGI47241[] = {{&S62281[30], 6}, {NULL}};
const Segment SGI47242[] = {{&S62281[111], 3}, {NULL}};
const Segment SGI47243[] = {{&S62281[77], 5}, {NULL}};
const Segment SGI47244[] = {{&S76806[44], 27}, {NULL}};
const Segment SGI47245[] = {{&S76806[30], 6}, {NULL}};
const Segment SGI47246[] = {{&S76806[138], 3}, {NULL}};
const Segment SGI47247[] = {{&S76806[103], 6}, {NULL}};
const Segment SGI47248[] = {{&S62282[44], 8}, {NULL}};
const Segment SGI47249[] = {{&S62282[44], 19}, {NULL}};
const Segment SGI47250[] = {{&S62282[30], 6}, {NULL}};
const Segment SGI47251[] = {{&S62282[111], 3}, {NULL}};
const Segment SGI47252[] = {{&S62282[77], 5}, {NULL}};
const Segment SGI47253[] = {{&S76807[44], 27}, {NULL}};
const Segment SGI47254[] = {{&S76807[30], 6}, {NULL}};
const Segment SGI47255[] = {{&S76807[138], 3}, {NULL}};
const Segment SGI47256[] = {{&S76807[103], 6}, {NULL}};
const Segment SGI47257[] = {{&S62283[44], 8}, {NULL}};
const Segment SGI47258[] = {{&S62283[44], 19}, {NULL}};
const Segment SGI47259[] = {{&S62283[30], 6}, {NULL}};
const Segment SGI47260[] = {{&S62283[111], 3}, {NULL}};
const Segment SGI47261[] = {{&S62283[77], 5}, {NULL}};
const Segment SGI47262[] = {{&S76808[44], 27}, {NULL}};
const Segment SGI47263[] = {{&S76808[30], 6}, {NULL}};
const Segment SGI47264[] = {{&S76808[138], 3}, {NULL}};
const Segment SGI47265[] = {{&S76808[103], 6}, {NULL}};
const Segment SGI47266[] = {{&S76809[44], 14}, {NULL}};
const Segment SGI47267[] = {{&S76809[44], 26}, {NULL}};
const Segment SGI47268[] = {{&S76809[30], 6}, {NULL}};
const Segment SGI47269[] = {{&S76809[118], 3}, {NULL}};
const Segment SGI47270[] = {{&S76809[84], 5}, {NULL}};
const Segment SGI47271[] = {{&S76810[44], 21}, {NULL}};
const Segment SGI47272[] = {{&S76810[30], 6}, {NULL}};
const Segment SGI47273[] = {{&S76810[123], 3}, {NULL}};
const Segment SGI47274[] = {{&S76810[88], 6}, {NULL}};
const Segment SGI47275[] = {{&S76811[44], 27}, {NULL}};
const Segment SGI47276[] = {{&S76811[30], 6}, {NULL}};
const Segment SGI47277[] = {{&S76811[138], 3}, {NULL}};
const Segment SGI47278[] = {{&S76811[103], 6}, {NULL}};
const Segment SGI47279[] = {{&S76812[44], 27}, {NULL}};
const Segment SGI47280[] = {{&S76812[30], 6}, {NULL}};
const Segment SGI47281[] = {{&S76812[138], 3}, {NULL}};
const Segment SGI47282[] = {{&S76812[103], 6}, {NULL}};
const Segment SGI47283[] = {{&S76813[44], 14}, {NULL}};
const Segment SGI47284[] = {{&S76813[44], 26}, {NULL}};
const Segment SGI47285[] = {{&S76813[30], 6}, {NULL}};
const Segment SGI47286[] = {{&S76813[87], 6}, {NULL}};
const Segment SGI47287[] = {{&S63079[44], 4}, {NULL}};
const Segment SGI47288[] = {{&S63079[44], 15}, {NULL}};
const Segment SGI47289[] = {{&S63079[30], 6}, {NULL}};
const Segment SGI47290[] = {{&S63079[107], 3}, {NULL}};
const Segment SGI47291[] = {{&S63079[73], 5}, {NULL}};
const Segment SGI47292[] = {{&S63080[44], 4}, {NULL}};
const Segment SGI47293[] = {{&S63080[44], 15}, {NULL}};
const Segment SGI47294[] = {{&S63080[30], 6}, {NULL}};
const Segment SGI47295[] = {{&S63080[107], 3}, {NULL}};
const Segment SGI47296[] = {{&S63080[73], 5}, {NULL}};
const Segment SGI47297[] = {{&S76860[44], 12}, {NULL}};
const Segment SGI47298[] = {{&S76860[44], 29}, {NULL}};
const Segment SGI47299[] = {{&S76860[30], 6}, {NULL}};
const Segment SGI47300[] = {{&S76860[121], 3}, {NULL}};
const Segment SGI47301[] = {{&S76860[87], 5}, {NULL}};
const Segment SGI47302[] = {{&S65668[44], 17}, {NULL}};
const Segment SGI47303[] = {{&S65668[44], 34}, {NULL}};
const Segment SGI47304[] = {{&S65668[30], 6}, {NULL}};
const Segment SGI47305[] = {{&S63097[44], 8}, {NULL}};
const Segment SGI47306[] = {{&S65668[126], 3}, {NULL}};
const Segment SGI47307[] = {{&S63097[44], 18}, {NULL}};
const Segment SGI47308[] = {{&S63097[30], 6}, {NULL}};
const Segment SGI47309[] = {{&S65668[92], 5}, {NULL}};
const Segment SGI47310[] = {{&S63097[110], 3}, {NULL}};
const Segment SGI47311[] = {{&S63097[76], 5}, {NULL}};
const Segment SGI47312[] = {{&S65669[38], 17}, {NULL}};
const Segment SGI47313[] = {{&S65669[31], 6}, {NULL}};
const Segment SGI47314[] = {{&S65688[6], 4}, {NULL}};
const Segment SGI47315[] = {{&S65688[82], 5}, {NULL}};
const Segment SGI47316[] = {{&S65719[44], 9}, {NULL}};
const Segment SGI47317[] = {{&S65719[44], 20}, {NULL}};
const Segment SGI47318[] = {{&S65719[30], 6}, {NULL}};
const Segment SGI47319[] = {{&S65719[112], 3}, {NULL}};
const Segment SGI47320[] = {{&S65719[78], 5}, {NULL}};
const Segment SGI47321[] = {{&S65720[38], 9}, {NULL}};
const Segment SGI47322[] = {{&S65720[31], 6}, {NULL}};
const Segment SGI47323[] = {{&S63163[44], 4}, {NULL}};
const Segment SGI47324[] = {{&S63163[44], 15}, {NULL}};
const Segment SGI47325[] = {{&S63163[30], 6}, {NULL}};
const Segment SGI47326[] = {{&S63163[107], 3}, {NULL}};
const Segment SGI47327[] = {{&S63163[73], 5}, {NULL}};
const Segment SGI47328[] = {{&S62495[44], 16}, {NULL}};
const Segment SGI47329[] = {{&S62495[44], 27}, {NULL}};
const Segment SGI47330[] = {{&S62495[30], 6}, {NULL}};
const Segment SGI47331[] = {{&S62495[119], 3}, {NULL}};
const Segment SGI47332[] = {{&S62495[85], 5}, {NULL}};
const Segment SGI47333[] = {{&S62534[86], 5}, {NULL}};
const Segment SGI47334[] = {{&S62536[44], 14}, {NULL}};
const Segment SGI47335[] = {{&S62536[44], 35}, {NULL}};
const Segment SGI47336[] = {{&S62536[30], 6}, {NULL}};
const Segment SGI47337[] = {{&S62536[127], 3}, {NULL}};
const Segment SGI47338[] = {{&S62536[93], 5}, {NULL}};
const Segment SGI47339[] = {{&S62537[44], 14}, {NULL}};
const Segment SGI47340[] = {{&S62537[44], 35}, {NULL}};
const Segment SGI47341[] = {{&S62537[30], 6}, {NULL}};
const Segment SGI47342[] = {{&S62537[96], 6}, {NULL}};
const Segment SGI47343[] = {{&S62538[38], 14}, {NULL}};
const Segment SGI47344[] = {{&S62538[31], 6}, {NULL}};
const Segment SGI47345[] = {{&S65852[44], 9}, {NULL}};
const Segment SGI47346[] = {{&S65852[44], 20}, {NULL}};
const Segment SGI47347[] = {{&S65852[30], 6}, {NULL}};
const Segment SGI47348[] = {{&S65852[112], 3}, {NULL}};
const Segment SGI47349[] = {{&S65852[78], 5}, {NULL}};
const Segment SGI47350[] = {{&S65853[38], 9}, {NULL}};
const Segment SGI47351[] = {{&S65853[31], 6}, {NULL}};
const Segment SGI47352[] = {{&S63345[44], 11}, {NULL}};
const Segment SGI47353[] = {{&S63345[44], 21}, {NULL}};
const Segment SGI47354[] = {{&S63345[30], 6}, {NULL}};
const Segment SGI47355[] = {{&S63345[113], 3}, {NULL}};
const Segment SGI47356[] = {{&S63345[79], 5}, {NULL}};
const Segment SGI47357[] = {{&S65934[44], 12}, {NULL}};
const Segment SGI47358[] = {{&S65934[44], 29}, {NULL}};
const Segment SGI47359[] = {{&S65934[30], 6}, {NULL}};
const Segment SGI47360[] = {{&S65934[121], 3}, {NULL}};
const Segment SGI47361[] = {{&S65934[87], 5}, {NULL}};
const Segment SGI47362[] = {{&S62665[44], 14}, {NULL}};
const Segment SGI47363[] = {{&S62665[44], 35}, {NULL}};
const Segment SGI47364[] = {{&S62665[30], 6}, {NULL}};
const Segment SGI47365[] = {{&S62665[127], 3}, {NULL}};
const Segment SGI47366[] = {{&S62665[93], 5}, {NULL}};
const Segment SGI47367[] = {{&S66188[45], 10}, {NULL}};
const Segment SGI47368[] = {{&S66188[30], 6}, {NULL}};
const Segment SGI47369[] = {{&S66188[120], 3}, {NULL}};
const Segment SGI47370[] = {{&S66188[86], 5}, {NULL}};
const Segment SGI47371[] = {{&S66188[154], 3}, {NULL}};
const Segment SGI47372[] = {{&S62793[44], 10}, {NULL}};
const Segment SGI47373[] = {{&S62793[44], 21}, {NULL}};
const Segment SGI47374[] = {{&S62793[30], 6}, {NULL}};
const Segment SGI47375[] = {{&S62793[113], 3}, {NULL}};
const Segment SGI47376[] = {{&S62793[79], 5}, {NULL}};
const Segment SGI47377[] = {{&S62794[38], 10}, {NULL}};
const Segment SGI47378[] = {{&S62794[31], 6}, {NULL}};
const Segment SGI47379[] = {{&S66225[44], 12}, {NULL}};
const Segment SGI47380[] = {{&S66225[44], 23}, {NULL}};
const Segment SGI47381[] = {{&S66225[30], 6}, {NULL}};
const Segment SGI47382[] = {{&S66225[115], 3}, {NULL}};
const Segment SGI47383[] = {{&S66225[81], 5}, {NULL}};
const Segment SGI47384[] = {{&S66226[44], 8}, {NULL}};
const Segment SGI47385[] = {{&S66226[44], 19}, {NULL}};
const Segment SGI47386[] = {{&S66226[30], 6}, {NULL}};
const Segment SGI47387[] = {{&S66226[111], 3}, {NULL}};
const Segment SGI47388[] = {{&S66226[77], 5}, {NULL}};
const Segment SGI47389[] = {{&S66227[44], 12}, {NULL}};
const Segment SGI47390[] = {{&S66227[44], 23}, {NULL}};
const Segment SGI47391[] = {{&S66227[30], 6}, {NULL}};
const Segment SGI47392[] = {{&S66227[115], 3}, {NULL}};
const Segment SGI47393[] = {{&S66227[81], 5}, {NULL}};
const Segment SGI47394[] = {{&S66272[44], 27}, {NULL}};
const Segment SGI47395[] = {{&S66272[44], 38}, {NULL}};
const Segment SGI47396[] = {{&S66272[30], 6}, {NULL}};
const Segment SGI47397[] = {{&S66272[130], 3}, {NULL}};
const Segment SGI47398[] = {{&S66272[96], 5}, {NULL}};
const Segment SGI47399[] = {{&S66276[44], 27}, {NULL}};
const Segment SGI47400[] = {{&S66276[44], 38}, {NULL}};
const Segment SGI47401[] = {{&S66276[30], 6}, {NULL}};
const Segment SGI47402[] = {{&S66276[99], 4}, {NULL}};
const Segment SGI47403[] = {{&S66276[99], 3}, {NULL}};
const Segment SGI47404[] = {{&S62889[44], 8}, {NULL}};
const Segment SGI47405[] = {{&S62889[44], 20}, {NULL}};
const Segment SGI47406[] = {{&S62889[30], 6}, {NULL}};
const Segment SGI47407[] = {{&S62889[112], 3}, {NULL}};
const Segment SGI47408[] = {{&S62889[78], 5}, {NULL}};
const Segment SGI47409[] = {{&S62890[38], 8}, {NULL}};
const Segment SGI47410[] = {{&S62890[31], 6}, {NULL}};
const Segment SGI47411[] = {{&S63737[44], 10}, {NULL}};
const Segment SGI47412[] = {{&S63737[44], 21}, {NULL}};
const Segment SGI47413[] = {{&S63737[30], 6}, {NULL}};
const Segment SGI47414[] = {{&S63737[113], 3}, {NULL}};
const Segment SGI47415[] = {{&S63737[79], 5}, {NULL}};
const Segment SGI47416[] = {{&S63738[38], 10}, {NULL}};
const Segment SGI47417[] = {{&S63738[31], 6}, {NULL}};
const Segment SGI47418[] = {{&S62969[44], 14}, {NULL}};
const Segment SGI47419[] = {{&S62969[44], 35}, {NULL}};
const Segment SGI47420[] = {{&S62969[30], 6}, {NULL}};
const Segment SGI47421[] = {{&S62969[96], 6}, {NULL}};
const Segment SGI47422[] = {{&S62970[44], 35}, {NULL}};
const Segment SGI47423[] = {{&S62970[30], 6}, {NULL}};
const Segment SGI47424[] = {{&S62971[44], 14}, {NULL}};
const Segment SGI47425[] = {{&S62971[44], 35}, {NULL}};
const Segment SGI47426[] = {{&S62971[30], 6}, {NULL}};
const Segment SGI47427[] = {{&S62971[127], 3}, {NULL}};
const Segment SGI47428[] = {{&S62971[93], 5}, {NULL}};
const Segment SGI47429[] = {{&S62975[38], 14}, {NULL}};
const Segment SGI47430[] = {{&S62975[31], 6}, {NULL}};
const Segment SGI47431[] = {{&S63081[44], 4}, {NULL}};
const Segment SGI47432[] = {{&S63081[44], 15}, {NULL}};
const Segment SGI47433[] = {{&S63081[30], 6}, {NULL}};
const Segment SGI47434[] = {{&S63081[107], 3}, {NULL}};
const Segment SGI47435[] = {{&S63081[73], 5}, {NULL}};
const Segment SGI47436[] = {{&S63085[44], 8}, {NULL}};
const Segment SGI47437[] = {{&S63085[44], 19}, {NULL}};
const Segment SGI47438[] = {{&S63085[30], 6}, {NULL}};
const Segment SGI47439[] = {{&S63085[111], 3}, {NULL}};
const Segment SGI47440[] = {{&S63085[77], 5}, {NULL}};
const Segment SGI47441[] = {{&S63086[44], 4}, {NULL}};
const Segment SGI47442[] = {{&S63086[44], 21}, {NULL}};
const Segment SGI47443[] = {{&S63086[30], 6}, {NULL}};
const Segment SGI47444[] = {{&S63086[113], 3}, {NULL}};
const Segment SGI47445[] = {{&S63086[79], 5}, {NULL}};
const Segment SGI47446[] = {{&S63096[44], 8}, {NULL}};
const Segment SGI47447[] = {{&S63096[44], 18}, {NULL}};
const Segment SGI47448[] = {{&S63096[30], 6}, {NULL}};
const Segment SGI47449[] = {{&S63096[110], 3}, {NULL}};
const Segment SGI47450[] = {{&S63096[76], 5}, {NULL}};
const Segment SGI47451[] = {{&S77585[6], 4}, {NULL}};
const Segment SGI47452[] = {{&S77585[84], 5}, {NULL}};
const Segment SGI47453[] = {{&S63160[44], 4}, {NULL}};
const Segment SGI47454[] = {{&S63160[44], 15}, {NULL}};
const Segment SGI47455[] = {{&S63160[30], 6}, {NULL}};
const Segment SGI47456[] = {{&S63160[107], 3}, {NULL}};
const Segment SGI47457[] = {{&S63160[73], 5}, {NULL}};
const Segment SGI47458[] = {{&S63161[41], 9}, {NULL}};
const Segment SGI47459[] = {{&S63161[41], 21}, {NULL}};
const Segment SGI47460[] = {{&S63161[110], 3}, {NULL}};
const Segment SGI47461[] = {{&S63161[76], 5}, {NULL}};
const Segment SGI47462[] = {{&S63162[44], 14}, {NULL}};
const Segment SGI47463[] = {{&S63162[44], 25}, {NULL}};
const Segment SGI47464[] = {{&S63162[30], 6}, {NULL}};
const Segment SGI47465[] = {{&S63162[117], 3}, {NULL}};
const Segment SGI47466[] = {{&S63162[83], 5}, {NULL}};
const Segment SGI47467[] = {{&S63164[44], 9}, {NULL}};
const Segment SGI47468[] = {{&S63164[44], 20}, {NULL}};
const Segment SGI47469[] = {{&S63164[30], 6}, {NULL}};
const Segment SGI47470[] = {{&S63164[112], 3}, {NULL}};
const Segment SGI47471[] = {{&S63164[78], 5}, {NULL}};
const Segment SGI47472[] = {{&S63165[38], 4}, {NULL}};
const Segment SGI47473[] = {{&S63165[31], 6}, {NULL}};
const Segment SGI47474[] = {{&S63166[44], 10}, {NULL}};
const Segment SGI47475[] = {{&S63166[44], 31}, {NULL}};
const Segment SGI47476[] = {{&S63166[30], 6}, {NULL}};
const Segment SGI47477[] = {{&S63166[92], 4}, {NULL}};
const Segment SGI47478[] = {{&S63166[92], 3}, {NULL}};
const Segment SGI47479[] = {{&S63344[44], 11}, {NULL}};
const Segment SGI47480[] = {{&S63344[44], 21}, {NULL}};
const Segment SGI47481[] = {{&S63344[30], 6}, {NULL}};
const Segment SGI47482[] = {{&S63344[113], 3}, {NULL}};
const Segment SGI47483[] = {{&S63344[79], 5}, {NULL}};
const Segment SGI47484[] = {{&S64165[44], 10}, {NULL}};
const Segment SGI47485[] = {{&S64165[44], 27}, {NULL}};
const Segment SGI47486[] = {{&S64165[30], 6}, {NULL}};
const Segment SGI47487[] = {{&S64165[119], 3}, {NULL}};
const Segment SGI47488[] = {{&S64165[85], 5}, {NULL}};
const Segment SGI47489[] = {{&S64166[44], 10}, {NULL}};
const Segment SGI47490[] = {{&S64166[44], 27}, {NULL}};
const Segment SGI47491[] = {{&S64166[30], 6}, {NULL}};
const Segment SGI47492[] = {{&S64166[119], 3}, {NULL}};
const Segment SGI47493[] = {{&S64166[85], 5}, {NULL}};
const Segment SGI47494[] = {{&S64167[42], 10}, {NULL}};
const Segment SGI47495[] = {{&S64167[30], 6}, {NULL}};
const Segment SGI47496[] = {{&S64167[117], 3}, {NULL}};
const Segment SGI47497[] = {{&S64167[83], 5}, {NULL}};
const Segment SGI47498[] = {{&S64168[38], 10}, {NULL}};
const Segment SGI47499[] = {{&S64168[31], 6}, {NULL}};
const Segment SGI47500[] = {{&S63418[44], 7}, {NULL}};
const Segment SGI47501[] = {{&S63418[44], 21}, {NULL}};
const Segment SGI47502[] = {{&S63418[30], 6}, {NULL}};
const Segment SGI47503[] = {{&S63418[113], 3}, {NULL}};
const Segment SGI47504[] = {{&S63418[79], 5}, {NULL}};
const Segment SGI47505[] = {{&S63419[44], 7}, {NULL}};
const Segment SGI47506[] = {{&S63419[44], 21}, {NULL}};
const Segment SGI47507[] = {{&S63419[30], 6}, {NULL}};
const Segment SGI47508[] = {{&S63419[113], 3}, {NULL}};
const Segment SGI47509[] = {{&S63419[79], 5}, {NULL}};
const Segment SGI47510[] = {{&S63420[44], 7}, {NULL}};
const Segment SGI47511[] = {{&S63420[44], 21}, {NULL}};
const Segment SGI47512[] = {{&S63420[30], 6}, {NULL}};
const Segment SGI47513[] = {{&S63420[113], 3}, {NULL}};
const Segment SGI47514[] = {{&S63420[79], 5}, {NULL}};
const Segment SGI47515[] = {{&S63421[44], 7}, {NULL}};
const Segment SGI47516[] = {{&S63421[44], 21}, {NULL}};
const Segment SGI47517[] = {{&S63421[30], 6}, {NULL}};
const Segment SGI47518[] = {{&S63421[113], 3}, {NULL}};
const Segment SGI47519[] = {{&S63421[79], 5}, {NULL}};
const Segment SGI47520[] = {{&S63422[44], 7}, {NULL}};
const Segment SGI47521[] = {{&S63422[44], 21}, {NULL}};
const Segment SGI47522[] = {{&S63422[30], 6}, {NULL}};
const Segment SGI47523[] = {{&S63422[113], 3}, {NULL}};
const Segment SGI47524[] = {{&S63422[79], 5}, {NULL}};
const Segment SGI47525[] = {{&S63423[44], 7}, {NULL}};
const Segment SGI47526[] = {{&S63423[44], 21}, {NULL}};
const Segment SGI47527[] = {{&S63423[30], 6}, {NULL}};
const Segment SGI47528[] = {{&S63423[113], 3}, {NULL}};
const Segment SGI47529[] = {{&S63423[79], 5}, {NULL}};
const Segment SGI47530[] = {{&S63424[44], 7}, {NULL}};
const Segment SGI47531[] = {{&S63424[44], 21}, {NULL}};
const Segment SGI47532[] = {{&S63424[30], 6}, {NULL}};
const Segment SGI47533[] = {{&S63424[113], 3}, {NULL}};
const Segment SGI47534[] = {{&S63424[79], 5}, {NULL}};
const Segment SGI47535[] = {{&S63432[44], 9}, {NULL}};
const Segment SGI47536[] = {{&S63432[44], 20}, {NULL}};
const Segment SGI47537[] = {{&S63432[30], 6}, {NULL}};
const Segment SGI47538[] = {{&S63432[112], 3}, {NULL}};
const Segment SGI47539[] = {{&S63432[78], 5}, {NULL}};
const Segment SGI47540[] = {{&S64220[44], 19}, {NULL}};
const Segment SGI47541[] = {{&S64220[44], 30}, {NULL}};
const Segment SGI47542[] = {{&S64220[30], 6}, {NULL}};
const Segment SGI47543[] = {{&S64220[122], 3}, {NULL}};
const Segment SGI47544[] = {{&S64220[88], 5}, {NULL}};
const Segment SGI47545[] = {{&S72648[44], 19}, {NULL}};
const Segment SGI47546[] = {{&S72648[44], 36}, {NULL}};
const Segment SGI47547[] = {{&S72648[30], 6}, {NULL}};
const Segment SGI47548[] = {{&S72648[128], 3}, {NULL}};
const Segment SGI47549[] = {{&S72648[94], 5}, {NULL}};
const Segment SGI47550[] = {{&S66770[44], 6}, {NULL}};
const Segment SGI47551[] = {{&S66770[44], 17}, {NULL}};
const Segment SGI47552[] = {{&S66770[30], 6}, {NULL}};
const Segment SGI47553[] = {{&S66770[109], 3}, {NULL}};
const Segment SGI47554[] = {{&S66770[75], 5}, {NULL}};
const Segment SGI47555[] = {{&S64273[44], 14}, {NULL}};
const Segment SGI47556[] = {{&S64273[44], 25}, {NULL}};
const Segment SGI47557[] = {{&S64273[30], 6}, {NULL}};
const Segment SGI47558[] = {{&S64273[117], 3}, {NULL}};
const Segment SGI47559[] = {{&S64273[83], 5}, {NULL}};
const Segment SGI47560[] = {{&S66835[45], 10}, {NULL}};
const Segment SGI47561[] = {{&S66835[30], 6}, {NULL}};
const Segment SGI47562[] = {{&S66835[120], 3}, {NULL}};
const Segment SGI47563[] = {{&S66835[86], 5}, {NULL}};
const Segment SGI47564[] = {{&S66835[154], 3}, {NULL}};
const Segment SGI47565[] = {{&S78219[44], 10}, {NULL}};
const Segment SGI47566[] = {{&S78219[44], 21}, {NULL}};
const Segment SGI47567[] = {{&S78219[30], 6}, {NULL}};
const Segment SGI47568[] = {{&S78219[113], 3}, {NULL}};
const Segment SGI47569[] = {{&S78219[79], 5}, {NULL}};
const Segment SGI47570[] = {{&S78220[44], 10}, {NULL}};
const Segment SGI47571[] = {{&S78220[44], 21}, {NULL}};
const Segment SGI47572[] = {{&S78220[30], 6}, {NULL}};
const Segment SGI47573[] = {{&S78220[82], 4}, {NULL}};
const Segment SGI47574[] = {{&S78220[82], 3}, {NULL}};
const Segment SGI47575[] = {{&S78221[44], 10}, {NULL}};
const Segment SGI47576[] = {{&S78221[44], 21}, {NULL}};
const Segment SGI47577[] = {{&S78221[30], 6}, {NULL}};
const Segment SGI47578[] = {{&S78221[82], 6}, {NULL}};
const Segment SGI47579[] = {{&S67161[44], 26}, {NULL}};
const Segment SGI47580[] = {{&S67161[30], 6}, {NULL}};
const Segment SGI47581[] = {{&S67161[136], 3}, {NULL}};
const Segment SGI47582[] = {{&S67161[102], 5}, {NULL}};
const Segment SGI47583[] = {{&S67162[44], 7}, {NULL}};
const Segment SGI47584[] = {{&S67162[44], 28}, {NULL}};
const Segment SGI47585[] = {{&S67162[30], 6}, {NULL}};
const Segment SGI47586[] = {{&S67162[120], 3}, {NULL}};
const Segment SGI47587[] = {{&S67162[86], 5}, {NULL}};
const Segment SGI47588[] = {{&S67167[44], 7}, {NULL}};
const Segment SGI47589[] = {{&S67167[44], 28}, {NULL}};
const Segment SGI47590[] = {{&S67167[30], 6}, {NULL}};
const Segment SGI47591[] = {{&S67167[89], 4}, {NULL}};
const Segment SGI47592[] = {{&S67167[89], 3}, {NULL}};
const Segment SGI47593[] = {{&S78424[44], 11}, {NULL}};
const Segment SGI47594[] = {{&S78424[44], 22}, {NULL}};
const Segment SGI47595[] = {{&S78424[30], 6}, {NULL}};
const Segment SGI47596[] = {{&S78424[114], 3}, {NULL}};
const Segment SGI47597[] = {{&S78424[80], 5}, {NULL}};
const Segment SGI47598[] = {{&S63857[44], 12}, {NULL}};
const Segment SGI47599[] = {{&S63857[44], 23}, {NULL}};
const Segment SGI47600[] = {{&S63857[30], 6}, {NULL}};
const Segment SGI47601[] = {{&S63857[115], 3}, {NULL}};
const Segment SGI47602[] = {{&S63857[81], 5}, {NULL}};
const Segment SGI47603[] = {{&S63858[44], 12}, {NULL}};
const Segment SGI47604[] = {{&S63858[44], 23}, {NULL}};
const Segment SGI47605[] = {{&S63858[30], 6}, {NULL}};
const Segment SGI47606[] = {{&S63858[84], 4}, {NULL}};
const Segment SGI47607[] = {{&S63858[84], 3}, {NULL}};
const Segment SGI47608[] = {{&S63859[44], 12}, {NULL}};
const Segment SGI47609[] = {{&S63859[44], 23}, {NULL}};
const Segment SGI47610[] = {{&S63859[30], 6}, {NULL}};
const Segment SGI47611[] = {{&S63859[84], 6}, {NULL}};
const Segment SGI47612[] = {{&S63860[38], 12}, {NULL}};
const Segment SGI47613[] = {{&S63860[31], 6}, {NULL}};
const Segment SGI47614[] = {{&S67324[44], 8}, {NULL}};
const Segment SGI47615[] = {{&S67324[44], 19}, {NULL}};
const Segment SGI47616[] = {{&S67324[30], 6}, {NULL}};
const Segment SGI47617[] = {{&S67324[111], 3}, {NULL}};
const Segment SGI47618[] = {{&S67324[77], 5}, {NULL}};
const Segment SGI47619[] = {{&S73272[44], 6}, {NULL}};
const Segment SGI47620[] = {{&S73272[44], 23}, {NULL}};
const Segment SGI47621[] = {{&S73272[30], 6}, {NULL}};
const Segment SGI47622[] = {{&S73272[115], 3}, {NULL}};
const Segment SGI47623[] = {{&S73272[81], 5}, {NULL}};
const Segment SGI47624[] = {{&S78582[44], 20}, {NULL}};
const Segment SGI47625[] = {{&S78582[44], 39}, {NULL}};
const Segment SGI47626[] = {{&S78582[30], 6}, {NULL}};
const Segment SGI47627[] = {{&S78582[131], 3}, {NULL}};
const Segment SGI47628[] = {{&S78582[97], 5}, {NULL}};
const Segment SGI47629[] = {{&S72965[6], 4}, {NULL}};
const Segment SGI47630[] = {{&S72965[76], 5}, {NULL}};
const Segment SGI47631[] = {{&S78665[44], 12}, {NULL}};
const Segment SGI47632[] = {{&S78665[44], 23}, {NULL}};
const Segment SGI47633[] = {{&S78665[30], 6}, {NULL}};
const Segment SGI47634[] = {{&S78665[115], 3}, {NULL}};
const Segment SGI47635[] = {{&S78665[81], 5}, {NULL}};
const Segment SGI47636[] = {{&S78666[44], 10}, {NULL}};
const Segment SGI47637[] = {{&S78666[44], 21}, {NULL}};
const Segment SGI47638[] = {{&S78666[30], 6}, {NULL}};
const Segment SGI47639[] = {{&S78666[113], 3}, {NULL}};
const Segment SGI47640[] = {{&S78666[79], 5}, {NULL}};
const Segment SGI47641[] = {{&S73397[44], 12}, {NULL}};
const Segment SGI47642[] = {{&S73397[44], 29}, {NULL}};
const Segment SGI47643[] = {{&S73397[30], 6}, {NULL}};
const Segment SGI47644[] = {{&S73397[121], 3}, {NULL}};
const Segment SGI47645[] = {{&S73397[87], 5}, {NULL}};
const Segment SGI47646[] = {{&S71948[42], 12}, {NULL}};
const Segment SGI47647[] = {{&S71948[42], 23}, {NULL}};
const Segment SGI47648[] = {{&S71948[30], 4}, {NULL}};
const Segment SGI47649[] = {{&S71948[113], 3}, {NULL}};
const Segment SGI47650[] = {{&S71948[79], 5}, {NULL}};
const Segment SGI47651[] = {{&S78767[44], 29}, {NULL}};
const Segment SGI47652[] = {{&S78767[30], 6}, {NULL}};
const Segment SGI47653[] = {{&S78767[139], 3}, {NULL}};
const Segment SGI47654[] = {{&S78767[105], 5}, {NULL}};
const Segment SGI47655[] = {{&S78771[44], 8}, {NULL}};
const Segment SGI47656[] = {{&S78771[44], 31}, {NULL}};
const Segment SGI47657[] = {{&S78771[30], 6}, {NULL}};
const Segment SGI47658[] = {{&S78771[123], 3}, {NULL}};
const Segment SGI47659[] = {{&S78771[89], 5}, {NULL}};
const Segment SGI47660[] = {{&S78773[44], 14}, {NULL}};
const Segment SGI47661[] = {{&S78773[44], 42}, {NULL}};
const Segment SGI47662[] = {{&S78773[30], 6}, {NULL}};
const Segment SGI47663[] = {{&S78773[134], 3}, {NULL}};
const Segment SGI47664[] = {{&S78773[100], 5}, {NULL}};
const Segment SGI47665[] = {{&S65031[44], 7}, {NULL}};
const Segment SGI47666[] = {{&S65031[44], 24}, {NULL}};
const Segment SGI47667[] = {{&S65031[30], 6}, {NULL}};
const Segment SGI47668[] = {{&S65031[116], 3}, {NULL}};
const Segment SGI47669[] = {{&S65031[82], 5}, {NULL}};
const Segment SGI47670[] = {{&S65032[44], 7}, {NULL}};
const Segment SGI47671[] = {{&S65032[44], 24}, {NULL}};
const Segment SGI47672[] = {{&S65032[30], 6}, {NULL}};
const Segment SGI47673[] = {{&S65032[82], 5}, {NULL}};
const Segment SGI47674[] = {{&S64275[44], 14}, {NULL}};
const Segment SGI47675[] = {{&S64275[44], 25}, {NULL}};
const Segment SGI47676[] = {{&S64275[30], 6}, {NULL}};
const Segment SGI47677[] = {{&S64275[117], 3}, {NULL}};
const Segment SGI47678[] = {{&S64275[83], 5}, {NULL}};
const Segment SGI47679[] = {{&S64276[44], 17}, {NULL}};
const Segment SGI47680[] = {{&S64276[44], 28}, {NULL}};
const Segment SGI47681[] = {{&S64276[30], 6}, {NULL}};
const Segment SGI47682[] = {{&S64276[120], 3}, {NULL}};
const Segment SGI47683[] = {{&S64276[86], 5}, {NULL}};
const Segment SGI47684[] = {{&S64277[44], 14}, {NULL}};
const Segment SGI47685[] = {{&S64277[44], 25}, {NULL}};
const Segment SGI47686[] = {{&S64277[30], 6}, {NULL}};
const Segment SGI47687[] = {{&S64277[117], 3}, {NULL}};
const Segment SGI47688[] = {{&S64277[83], 5}, {NULL}};
const Segment SGI47689[] = {{&S64360[44], 7}, {NULL}};
const Segment SGI47690[] = {{&S64360[44], 20}, {NULL}};
const Segment SGI47691[] = {{&S64360[30], 6}, {NULL}};
const Segment SGI47692[] = {{&S64360[112], 3}, {NULL}};
const Segment SGI47693[] = {{&S64360[78], 5}, {NULL}};
const Segment SGI47694[] = {{&S64361[44], 7}, {NULL}};
const Segment SGI47695[] = {{&S64361[44], 20}, {NULL}};
const Segment SGI47696[] = {{&S64361[30], 6}, {NULL}};
const Segment SGI47697[] = {{&S64361[112], 3}, {NULL}};
const Segment SGI47698[] = {{&S64361[78], 5}, {NULL}};
const Segment SGI47699[] = {{&S64362[44], 7}, {NULL}};
const Segment SGI47700[] = {{&S64362[44], 18}, {NULL}};
const Segment SGI47701[] = {{&S64362[30], 6}, {NULL}};
const Segment SGI47702[] = {{&S64362[110], 3}, {NULL}};
const Segment SGI47703[] = {{&S64362[76], 5}, {NULL}};
const Segment SGI47704[] = {{&S73231[44], 11}, {NULL}};
const Segment SGI47705[] = {{&S73231[44], 22}, {NULL}};
const Segment SGI47706[] = {{&S73231[30], 6}, {NULL}};
const Segment SGI47707[] = {{&S64363[38], 7}, {NULL}};
const Segment SGI47708[] = {{&S64363[31], 6}, {NULL}};
const Segment SGI47709[] = {{&S73231[83], 6}, {NULL}};
const Segment SGI47710[] = {{&S69131[44], 9}, {NULL}};
const Segment SGI47711[] = {{&S69131[44], 20}, {NULL}};
const Segment SGI47712[] = {{&S69131[30], 6}, {NULL}};
const Segment SGI47713[] = {{&S73232[44], 11}, {NULL}};
const Segment SGI47714[] = {{&S69131[112], 3}, {NULL}};
const Segment SGI47715[] = {{&S73232[44], 22}, {NULL}};
const Segment SGI47716[] = {{&S73232[30], 6}, {NULL}};
const Segment SGI47717[] = {{&S64364[44], 7}, {NULL}};
const Segment SGI47718[] = {{&S69131[78], 5}, {NULL}};
const Segment SGI47719[] = {{&S64364[44], 20}, {NULL}};
const Segment SGI47720[] = {{&S64364[30], 6}, {NULL}};
const Segment SGI47721[] = {{&S73232[114], 3}, {NULL}};
const Segment SGI47722[] = {{&S73232[80], 5}, {NULL}};
const Segment SGI47723[] = {{&S64364[112], 3}, {NULL}};
const Segment SGI47724[] = {{&S64364[78], 5}, {NULL}};
const Segment SGI47725[] = {{&S67565[44], 13}, {NULL}};
const Segment SGI47726[] = {{&S67565[44], 25}, {NULL}};
const Segment SGI47727[] = {{&S67565[30], 6}, {NULL}};
const Segment SGI47728[] = {{&S67565[117], 3}, {NULL}};
const Segment SGI47729[] = {{&S67565[83], 5}, {NULL}};
const Segment SGI47730[] = {{&S64417[44], 10}, {NULL}};
const Segment SGI47731[] = {{&S64417[44], 21}, {NULL}};
const Segment SGI47732[] = {{&S64417[30], 6}, {NULL}};
const Segment SGI47733[] = {{&S64417[113], 3}, {NULL}};
const Segment SGI47734[] = {{&S64417[79], 5}, {NULL}};
const Segment SGI47735[] = {{&S64418[44], 17}, {NULL}};
const Segment SGI47736[] = {{&S64418[44], 28}, {NULL}};
const Segment SGI47737[] = {{&S64418[30], 6}, {NULL}};
const Segment SGI47738[] = {{&S64418[120], 3}, {NULL}};
const Segment SGI47739[] = {{&S64418[86], 5}, {NULL}};
const Segment SGI47740[] = {{&S78983[44], 12}, {NULL}};
const Segment SGI47741[] = {{&S78983[44], 24}, {NULL}};
const Segment SGI47742[] = {{&S78983[30], 6}, {NULL}};
const Segment SGI47743[] = {{&S78983[85], 4}, {NULL}};
const Segment SGI47744[] = {{&S78983[85], 3}, {NULL}};
const Segment SGI47745[] = {{&S78984[44], 12}, {NULL}};
const Segment SGI47746[] = {{&S78984[44], 24}, {NULL}};
const Segment SGI47747[] = {{&S78984[30], 6}, {NULL}};
const Segment SGI47748[] = {{&S78984[85], 6}, {NULL}};
const Segment SGI47749[] = {{&S78985[44], 24}, {NULL}};
const Segment SGI47750[] = {{&S78985[30], 6}, {NULL}};
const Segment SGI47751[] = {{&S65264[45], 10}, {NULL}};
const Segment SGI47752[] = {{&S65264[30], 6}, {NULL}};
const Segment SGI47753[] = {{&S65264[120], 3}, {NULL}};
const Segment SGI47754[] = {{&S65264[86], 5}, {NULL}};
const Segment SGI47755[] = {{&S65264[144], 13}, {NULL}};
const Segment SGI47756[] = {{&S72275[42], 13}, {NULL}};
const Segment SGI47757[] = {{&S72275[42], 24}, {NULL}};
const Segment SGI47758[] = {{&S72275[30], 4}, {NULL}};
const Segment SGI47759[] = {{&S72275[114], 3}, {NULL}};
const Segment SGI47760[] = {{&S72275[80], 5}, {NULL}};
const Segment SGI47761[] = {{&S65283[44], 10}, {NULL}};
const Segment SGI47762[] = {{&S65283[44], 21}, {NULL}};
const Segment SGI47763[] = {{&S65283[30], 6}, {NULL}};
const Segment SGI47764[] = {{&S65283[113], 3}, {NULL}};
const Segment SGI47765[] = {{&S65283[79], 5}, {NULL}};
const Segment SGI47766[] = {{&S65284[44], 10}, {NULL}};
const Segment SGI47767[] = {{&S65284[44], 21}, {NULL}};
const Segment SGI47768[] = {{&S65284[30], 6}, {NULL}};
const Segment SGI47769[] = {{&S65284[113], 3}, {NULL}};
const Segment SGI47770[] = {{&S65284[79], 5}, {NULL}};
const Segment SGI47771[] = {{&S65285[44], 10}, {NULL}};
const Segment SGI47772[] = {{&S65285[44], 21}, {NULL}};
const Segment SGI47773[] = {{&S65285[30], 6}, {NULL}};
const Segment SGI47774[] = {{&S65285[113], 3}, {NULL}};
const Segment SGI47775[] = {{&S65285[79], 5}, {NULL}};
const Segment SGI47776[] = {{&S65286[44], 10}, {NULL}};
const Segment SGI47777[] = {{&S65286[44], 21}, {NULL}};
const Segment SGI47778[] = {{&S65286[30], 6}, {NULL}};
const Segment SGI47779[] = {{&S65286[113], 3}, {NULL}};
const Segment SGI47780[] = {{&S65286[79], 5}, {NULL}};
const Segment SGI47781[] = {{&S65287[38], 10}, {NULL}};
const Segment SGI47782[] = {{&S65287[31], 6}, {NULL}};
const Segment SGI47783[] = {{&S65288[44], 10}, {NULL}};
const Segment SGI47784[] = {{&S65288[44], 21}, {NULL}};
const Segment SGI47785[] = {{&S65288[30], 6}, {NULL}};
const Segment SGI47786[] = {{&S65288[113], 3}, {NULL}};
const Segment SGI47787[] = {{&S65288[79], 5}, {NULL}};
const Segment SGI47788[] = {{&S65289[44], 10}, {NULL}};
const Segment SGI47789[] = {{&S65289[44], 21}, {NULL}};
const Segment SGI47790[] = {{&S65289[30], 6}, {NULL}};
const Segment SGI47791[] = {{&S65289[113], 3}, {NULL}};
const Segment SGI47792[] = {{&S65289[79], 5}, {NULL}};
const Segment SGI47793[] = {{&S67684[45], 10}, {NULL}};
const Segment SGI47794[] = {{&S67684[30], 6}, {NULL}};
const Segment SGI47795[] = {{&S67684[120], 3}, {NULL}};
const Segment SGI47796[] = {{&S67684[86], 5}, {NULL}};
const Segment SGI47797[] = {{&S67684[154], 3}, {NULL}};
const Segment SGI47798[] = {{&S79023[44], 28}, {NULL}};
const Segment SGI47799[] = {{&S79023[30], 6}, {NULL}};
const Segment SGI47800[] = {{&S79023[138], 3}, {NULL}};
const Segment SGI47801[] = {{&S79023[104], 5}, {NULL}};
const Segment SGI47802[] = {{&S79024[44], 28}, {NULL}};
const Segment SGI47803[] = {{&S79024[30], 6}, {NULL}};
const Segment SGI47804[] = {{&S79024[138], 3}, {NULL}};
const Segment SGI47805[] = {{&S79024[104], 5}, {NULL}};
const Segment SGI47806[] = {{&S72478[42], 12}, {NULL}};
const Segment SGI47807[] = {{&S72478[42], 23}, {NULL}};
const Segment SGI47808[] = {{&S72478[30], 4}, {NULL}};
const Segment SGI47809[] = {{&S72478[113], 3}, {NULL}};
const Segment SGI47810[] = {{&S72478[79], 5}, {NULL}};
const Segment SGI47811[] = {{&S65511[44], 10}, {NULL}};
const Segment SGI47812[] = {{&S65511[44], 27}, {NULL}};
const Segment SGI47813[] = {{&S65511[30], 6}, {NULL}};
const Segment SGI47814[] = {{&S65511[85], 6}, {NULL}};
const Segment SGI47815[] = {{&S65511[112], 13}, {NULL}};
const Segment SGI47816[] = {{&S67963[44], 10}, {NULL}};
const Segment SGI47817[] = {{&S67963[44], 21}, {NULL}};
const Segment SGI47818[] = {{&S67963[30], 6}, {NULL}};
const Segment SGI47819[] = {{&S67963[113], 3}, {NULL}};
const Segment SGI47820[] = {{&S67963[79], 5}, {NULL}};
const Segment SGI47821[] = {{&S67964[44], 10}, {NULL}};
const Segment SGI47822[] = {{&S67964[44], 21}, {NULL}};
const Segment SGI47823[] = {{&S67964[30], 6}, {NULL}};
const Segment SGI47824[] = {{&S67964[114], 3}, {NULL}};
const Segment SGI47825[] = {{&S67964[79], 6}, {NULL}};
const Segment SGI47826[] = {{&S72647[42], 19}, {NULL}};
const Segment SGI47827[] = {{&S72647[42], 36}, {NULL}};
const Segment SGI47828[] = {{&S72647[30], 4}, {NULL}};
const Segment SGI47829[] = {{&S72647[126], 3}, {NULL}};
const Segment SGI47830[] = {{&S72647[92], 5}, {NULL}};
const Segment SGI47831[] = {{&S72649[42], 19}, {NULL}};
const Segment SGI47832[] = {{&S72649[42], 36}, {NULL}};
const Segment SGI47833[] = {{&S72649[30], 4}, {NULL}};
const Segment SGI47834[] = {{&S72649[126], 3}, {NULL}};
const Segment SGI47835[] = {{&S72649[92], 5}, {NULL}};
const Segment SGI47836[] = {{&S79358[44], 7}, {NULL}};
const Segment SGI47837[] = {{&S79358[44], 26}, {NULL}};
const Segment SGI47838[] = {{&S79358[30], 6}, {NULL}};
const Segment SGI47839[] = {{&S79358[118], 3}, {NULL}};
const Segment SGI47840[] = {{&S79358[84], 5}, {NULL}};
const Segment SGI47841[] = {{&S79359[44], 7}, {NULL}};
const Segment SGI47842[] = {{&S79359[44], 25}, {NULL}};
const Segment SGI47843[] = {{&S79359[30], 6}, {NULL}};
const Segment SGI47844[] = {{&S79359[117], 3}, {NULL}};
const Segment SGI47845[] = {{&S79359[83], 5}, {NULL}};
const Segment SGI47846[] = {{&S79360[44], 7}, {NULL}};
const Segment SGI47847[] = {{&S79360[44], 25}, {NULL}};
const Segment SGI47848[] = {{&S79360[30], 6}, {NULL}};
const Segment SGI47849[] = {{&S79360[117], 3}, {NULL}};
const Segment SGI47850[] = {{&S79360[83], 5}, {NULL}};
const Segment SGI47851[] = {{&S79362[44], 7}, {NULL}};
const Segment SGI47852[] = {{&S79362[44], 25}, {NULL}};
const Segment SGI47853[] = {{&S79362[30], 6}, {NULL}};
const Segment SGI47854[] = {{&S79362[86], 4}, {NULL}};
const Segment SGI47855[] = {{&S79362[86], 3}, {NULL}};
const Segment SGI47856[] = {{&S79363[44], 7}, {NULL}};
const Segment SGI47857[] = {{&S79363[44], 25}, {NULL}};
const Segment SGI47858[] = {{&S79363[30], 6}, {NULL}};
const Segment SGI47859[] = {{&S79363[86], 6}, {NULL}};
const Segment SGI47860[] = {{&S65690[6], 4}, {NULL}};
const Segment SGI47861[] = {{&S65690[82], 5}, {NULL}};
const Segment SGI47862[] = {{&S64539[44], 14}, {NULL}};
const Segment SGI47863[] = {{&S64539[44], 31}, {NULL}};
const Segment SGI47864[] = {{&S64539[30], 6}, {NULL}};
const Segment SGI47865[] = {{&S64539[123], 3}, {NULL}};
const Segment SGI47866[] = {{&S64539[89], 5}, {NULL}};
const Segment SGI47867[] = {{&S64540[44], 5}, {NULL}};
const Segment SGI47868[] = {{&S64540[44], 22}, {NULL}};
const Segment SGI47869[] = {{&S64540[30], 6}, {NULL}};
const Segment SGI47870[] = {{&S64540[114], 3}, {NULL}};
const Segment SGI47871[] = {{&S64540[80], 5}, {NULL}};
const Segment SGI47872[] = {{&S68155[44], 10}, {NULL}};
const Segment SGI47873[] = {{&S68155[44], 27}, {NULL}};
const Segment SGI47874[] = {{&S68155[30], 6}, {NULL}};
const Segment SGI47875[] = {{&S68155[85], 6}, {NULL}};
const Segment SGI47876[] = {{&S68155[122], 3}, {NULL}};
const Segment SGI47877[] = {{&S72794[42], 14}, {NULL}};
const Segment SGI47878[] = {{&S72794[42], 25}, {NULL}};
const Segment SGI47879[] = {{&S72794[30], 4}, {NULL}};
const Segment SGI47880[] = {{&S72794[115], 3}, {NULL}};
const Segment SGI47881[] = {{&S72794[81], 5}, {NULL}};
const Segment SGI47882[] = {{&S64658[44], 7}, {NULL}};
const Segment SGI47883[] = {{&S64658[44], 18}, {NULL}};
const Segment SGI47884[] = {{&S64658[30], 6}, {NULL}};
const Segment SGI47885[] = {{&S64658[110], 3}, {NULL}};
const Segment SGI47886[] = {{&S64658[76], 5}, {NULL}};
const Segment SGI47887[] = {{&S64659[38], 7}, {NULL}};
const Segment SGI47888[] = {{&S64659[31], 6}, {NULL}};
const Segment SGI47889[] = {{&S72894[42], 9}, {NULL}};
const Segment SGI47890[] = {{&S72894[42], 26}, {NULL}};
const Segment SGI47891[] = {{&S72894[30], 4}, {NULL}};
const Segment SGI47892[] = {{&S72894[116], 3}, {NULL}};
const Segment SGI47893[] = {{&S72894[82], 5}, {NULL}};
const Segment SGI47894[] = {{&S65955[44], 10}, {NULL}};
const Segment SGI47895[] = {{&S65955[44], 27}, {NULL}};
const Segment SGI47896[] = {{&S65955[30], 6}, {NULL}};
const Segment SGI47897[] = {{&S65955[85], 6}, {NULL}};
const Segment SGI47898[] = {{&S65955[112], 13}, {NULL}};
const Segment SGI47899[] = {{&S64773[44], 13}, {NULL}};
const Segment SGI47900[] = {{&S64773[44], 24}, {NULL}};
const Segment SGI47901[] = {{&S64773[30], 6}, {NULL}};
const Segment SGI47902[] = {{&S64773[116], 3}, {NULL}};
const Segment SGI47903[] = {{&S64773[82], 5}, {NULL}};
const Segment SGI47904[] = {{&S66011[44], 8}, {NULL}};
const Segment SGI47905[] = {{&S66011[44], 25}, {NULL}};
const Segment SGI47906[] = {{&S66011[30], 6}, {NULL}};
const Segment SGI47907[] = {{&S66011[117], 3}, {NULL}};
const Segment SGI47908[] = {{&S66011[83], 5}, {NULL}};
const Segment SGI47909[] = {{&S65037[44], 10}, {NULL}};
const Segment SGI47910[] = {{&S65037[44], 27}, {NULL}};
const Segment SGI47911[] = {{&S65037[30], 6}, {NULL}};
const Segment SGI47912[] = {{&S65037[85], 6}, {NULL}};
const Segment SGI47913[] = {{&S65037[112], 13}, {NULL}};
const Segment SGI47914[] = {{&S66187[45], 10}, {NULL}};
const Segment SGI47915[] = {{&S66187[30], 6}, {NULL}};
const Segment SGI47916[] = {{&S66187[120], 3}, {NULL}};
const Segment SGI47917[] = {{&S66187[86], 5}, {NULL}};
const Segment SGI47918[] = {{&S66187[144], 13}, {NULL}};
const Segment SGI47919[] = {{&S66191[45], 11}, {NULL}};
const Segment SGI47920[] = {{&S66191[30], 6}, {NULL}};
const Segment SGI47921[] = {{&S66191[121], 3}, {NULL}};
const Segment SGI47922[] = {{&S66191[87], 5}, {NULL}};
const Segment SGI47923[] = {{&S66191[145], 13}, {NULL}};
const Segment SGI47924[] = {{&S65067[44], 8}, {NULL}};
const Segment SGI47925[] = {{&S65067[44], 19}, {NULL}};
const Segment SGI47926[] = {{&S65067[30], 6}, {NULL}};
const Segment SGI47927[] = {{&S65067[111], 3}, {NULL}};
const Segment SGI47928[] = {{&S65067[77], 5}, {NULL}};
const Segment SGI47929[] = {{&S65068[44], 8}, {NULL}};
const Segment SGI47930[] = {{&S65068[44], 19}, {NULL}};
const Segment SGI47931[] = {{&S65068[30], 6}, {NULL}};
const Segment SGI47932[] = {{&S65068[111], 3}, {NULL}};
const Segment SGI47933[] = {{&S65068[77], 5}, {NULL}};
const Segment SGI47934[] = {{&S65069[44], 8}, {NULL}};
const Segment SGI47935[] = {{&S65069[44], 19}, {NULL}};
const Segment SGI47936[] = {{&S65069[30], 6}, {NULL}};
const Segment SGI47937[] = {{&S65069[111], 3}, {NULL}};
const Segment SGI47938[] = {{&S65069[77], 5}, {NULL}};
const Segment SGI47939[] = {{&S65070[44], 8}, {NULL}};
const Segment SGI47940[] = {{&S65070[44], 19}, {NULL}};
const Segment SGI47941[] = {{&S65070[30], 6}, {NULL}};
const Segment SGI47942[] = {{&S65070[111], 3}, {NULL}};
const Segment SGI47943[] = {{&S65070[77], 5}, {NULL}};
const Segment SGI47944[] = {{&S65071[44], 8}, {NULL}};
const Segment SGI47945[] = {{&S65071[44], 19}, {NULL}};
const Segment SGI47946[] = {{&S65071[30], 6}, {NULL}};
const Segment SGI47947[] = {{&S65071[111], 3}, {NULL}};
const Segment SGI47948[] = {{&S65071[77], 5}, {NULL}};
const Segment SGI47949[] = {{&S65072[44], 8}, {NULL}};
const Segment SGI47950[] = {{&S65072[44], 19}, {NULL}};
const Segment SGI47951[] = {{&S65072[30], 6}, {NULL}};
const Segment SGI47952[] = {{&S65072[111], 3}, {NULL}};
const Segment SGI47953[] = {{&S65072[77], 5}, {NULL}};
const Segment SGI47954[] = {{&S74448[38], 11}, {NULL}};
const Segment SGI47955[] = {{&S74448[31], 6}, {NULL}};
const Segment SGI47956[] = {{&S66273[44], 27}, {NULL}};
const Segment SGI47957[] = {{&S66273[44], 38}, {NULL}};
const Segment SGI47958[] = {{&S66273[30], 6}, {NULL}};
const Segment SGI47959[] = {{&S66273[130], 3}, {NULL}};
const Segment SGI47960[] = {{&S66273[96], 5}, {NULL}};
const Segment SGI47961[] = {{&S66302[45], 10}, {NULL}};
const Segment SGI47962[] = {{&S66302[30], 6}, {NULL}};
const Segment SGI47963[] = {{&S66302[120], 3}, {NULL}};
const Segment SGI47964[] = {{&S66302[86], 5}, {NULL}};
const Segment SGI47965[] = {{&S66302[144], 13}, {NULL}};
const Segment SGI47966[] = {{&S68889[45], 10}, {NULL}};
const Segment SGI47967[] = {{&S68889[30], 6}, {NULL}};
const Segment SGI47968[] = {{&S68889[120], 3}, {NULL}};
const Segment SGI47969[] = {{&S68889[86], 5}, {NULL}};
const Segment SGI47970[] = {{&S68889[154], 3}, {NULL}};
const Segment SGI47971[] = {{&S65230[59], 10}, {NULL}};
const Segment SGI47972[] = {{&S65230[59], 27}, {NULL}};
const Segment SGI47973[] = {{&S65230[45], 6}, {NULL}};
const Segment SGI47974[] = {{&S65230[134], 3}, {NULL}};
const Segment SGI47975[] = {{&S65230[100], 5}, {NULL}};
const Segment SGI47976[] = {{&S65230[0], 14}, {NULL}};
const Segment SGI47977[] = {{&S80141[44], 10}, {NULL}};
const Segment SGI47978[] = {{&S80141[44], 21}, {NULL}};
const Segment SGI47979[] = {{&S80141[30], 6}, {NULL}};
const Segment SGI47980[] = {{&S80141[113], 3}, {NULL}};
const Segment SGI47981[] = {{&S80141[79], 5}, {NULL}};
const Segment SGI47982[] = {{&S80142[44], 10}, {NULL}};
const Segment SGI47983[] = {{&S80142[44], 21}, {NULL}};
const Segment SGI47984[] = {{&S80142[30], 6}, {NULL}};
const Segment SGI47985[] = {{&S80142[82], 4}, {NULL}};
const Segment SGI47986[] = {{&S80142[82], 3}, {NULL}};
const Segment SGI47987[] = {{&S73578[44], 17}, {NULL}};
const Segment SGI47988[] = {{&S73578[44], 38}, {NULL}};
const Segment SGI47989[] = {{&S73578[30], 6}, {NULL}};
const Segment SGI47990[] = {{&S73578[130], 3}, {NULL}};
const Segment SGI47991[] = {{&S73578[96], 5}, {NULL}};
const Segment SGI47992[] = {{&S80273[44], 8}, {NULL}};
const Segment SGI47993[] = {{&S80273[44], 20}, {NULL}};
const Segment SGI47994[] = {{&S80273[30], 6}, {NULL}};
const Segment SGI47995[] = {{&S80273[112], 3}, {NULL}};
const Segment SGI47996[] = {{&S80273[78], 5}, {NULL}};
const Segment SGI47997[] = {{&S80274[38], 8}, {NULL}};
const Segment SGI47998[] = {{&S80274[31], 6}, {NULL}};
const Segment SGI47999[] = {{&S65323[34], 21}, {NULL}};
const Segment SGI48000[] = {{&S65323[27], 6}, {NULL}};
const Segment SGI48001[] = {{&S65323[114], 2}, {NULL}};
const Segment SGI48002[] = {{&S65323[80], 6}, {NULL}};
const Segment SGI48003[] = {{&S65323[148], 8}, {NULL}};
const Segment SGI48004[] = {{&S65325[34], 11}, {NULL}};
const Segment SGI48005[] = {{&S65325[27], 6}, {NULL}};
const Segment SGI48006[] = {{&S65325[104], 2}, {NULL}};
const Segment SGI48007[] = {{&S65325[70], 6}, {NULL}};
const Segment SGI48008[] = {{&S65325[138], 8}, {NULL}};
const Segment SGI48009[] = {{&S69127[44], 9}, {NULL}};
const Segment SGI48010[] = {{&S69127[44], 20}, {NULL}};
const Segment SGI48011[] = {{&S69127[30], 6}, {NULL}};
const Segment SGI48012[] = {{&S69127[112], 3}, {NULL}};
const Segment SGI48013[] = {{&S69127[78], 5}, {NULL}};
const Segment SGI48014[] = {{&S65386[44], 4}, {NULL}};
const Segment SGI48015[] = {{&S65386[44], 15}, {NULL}};
const Segment SGI48016[] = {{&S65386[30], 6}, {NULL}};
const Segment SGI48017[] = {{&S65386[76], 4}, {NULL}};
const Segment SGI48018[] = {{&S65386[76], 3}, {NULL}};
const Segment SGI48019[] = {{&S65387[44], 4}, {NULL}};
const Segment SGI48020[] = {{&S65387[44], 15}, {NULL}};
const Segment SGI48021[] = {{&S65387[30], 6}, {NULL}};
const Segment SGI48022[] = {{&S65387[107], 3}, {NULL}};
const Segment SGI48023[] = {{&S65387[73], 5}, {NULL}};
const Segment SGI48024[] = {{&S65388[44], 4}, {NULL}};
const Segment SGI48025[] = {{&S65388[44], 15}, {NULL}};
const Segment SGI48026[] = {{&S65388[30], 6}, {NULL}};
const Segment SGI48027[] = {{&S65388[76], 6}, {NULL}};
const Segment SGI48028[] = {{&S65389[38], 4}, {NULL}};
const Segment SGI48029[] = {{&S65389[31], 6}, {NULL}};
const Segment SGI48030[] = {{&S66626[44], 7}, {NULL}};
const Segment SGI48031[] = {{&S66626[44], 19}, {NULL}};
const Segment SGI48032[] = {{&S66626[30], 6}, {NULL}};
const Segment SGI48033[] = {{&S66626[111], 3}, {NULL}};
const Segment SGI48034[] = {{&S66626[77], 5}, {NULL}};
const Segment SGI48035[] = {{&S66627[44], 7}, {NULL}};
const Segment SGI48036[] = {{&S66627[44], 19}, {NULL}};
const Segment SGI48037[] = {{&S66627[30], 6}, {NULL}};
const Segment SGI48038[] = {{&S66627[111], 3}, {NULL}};
const Segment SGI48039[] = {{&S66627[77], 5}, {NULL}};
const Segment SGI48040[] = {{&S65510[44], 10}, {NULL}};
const Segment SGI48041[] = {{&S65510[44], 27}, {NULL}};
const Segment SGI48042[] = {{&S65510[30], 6}, {NULL}};
const Segment SGI48043[] = {{&S65510[85], 6}, {NULL}};
const Segment SGI48044[] = {{&S65510[112], 13}, {NULL}};
const Segment SGI48045[] = {{&S69271[44], 6}, {NULL}};
const Segment SGI48046[] = {{&S69271[44], 23}, {NULL}};
const Segment SGI48047[] = {{&S69271[30], 6}, {NULL}};
const Segment SGI48048[] = {{&S69271[115], 3}, {NULL}};
const Segment SGI48049[] = {{&S69271[81], 5}, {NULL}};
const Segment SGI48050[] = {{&S80515[44], 7}, {NULL}};
const Segment SGI48051[] = {{&S80515[44], 19}, {NULL}};
const Segment SGI48052[] = {{&S80515[30], 6}, {NULL}};
const Segment SGI48053[] = {{&S80515[111], 3}, {NULL}};
const Segment SGI48054[] = {{&S80515[77], 5}, {NULL}};
const Segment SGI48055[] = {{&S65604[45], 10}, {NULL}};
const Segment SGI48056[] = {{&S65604[30], 6}, {NULL}};
const Segment SGI48057[] = {{&S65604[120], 3}, {NULL}};
const Segment SGI48058[] = {{&S65604[86], 5}, {NULL}};
const Segment SGI48059[] = {{&S65604[144], 13}, {NULL}};
const Segment SGI48060[] = {{&S65631[44], 10}, {NULL}};
const Segment SGI48061[] = {{&S65631[44], 22}, {NULL}};
const Segment SGI48062[] = {{&S65631[30], 6}, {NULL}};
const Segment SGI48063[] = {{&S65631[114], 3}, {NULL}};
const Segment SGI48064[] = {{&S65631[80], 5}, {NULL}};
const Segment SGI48065[] = {{&S75218[6], 4}, {NULL}};
const Segment SGI48066[] = {{&S75218[84], 5}, {NULL}};
const Segment SGI48067[] = {{&S69363[44], 9}, {NULL}};
const Segment SGI48068[] = {{&S69363[44], 19}, {NULL}};
const Segment SGI48069[] = {{&S69363[30], 6}, {NULL}};
const Segment SGI48070[] = {{&S69363[111], 3}, {NULL}};
const Segment SGI48071[] = {{&S69363[77], 5}, {NULL}};
const Segment SGI48072[] = {{&S80658[44], 6}, {NULL}};
const Segment SGI48073[] = {{&S80658[44], 17}, {NULL}};
const Segment SGI48074[] = {{&S80658[30], 6}, {NULL}};
const Segment SGI48075[] = {{&S80658[109], 3}, {NULL}};
const Segment SGI48076[] = {{&S80658[75], 5}, {NULL}};
const Segment SGI48077[] = {{&S74027[44], 10}, {NULL}};
const Segment SGI48078[] = {{&S74027[44], 20}, {NULL}};
const Segment SGI48079[] = {{&S74027[30], 6}, {NULL}};
const Segment SGI48080[] = {{&S74027[112], 3}, {NULL}};
const Segment SGI48081[] = {{&S74027[78], 5}, {NULL}};
const Segment SGI48082[] = {{&S66838[45], 10}, {NULL}};
const Segment SGI48083[] = {{&S66838[30], 6}, {NULL}};
const Segment SGI48084[] = {{&S66838[120], 3}, {NULL}};
const Segment SGI48085[] = {{&S66838[86], 5}, {NULL}};
const Segment SGI48086[] = {{&S66838[144], 13}, {NULL}};
const Segment SGI48087[] = {{&S66839[45], 11}, {NULL}};
const Segment SGI48088[] = {{&S66839[30], 6}, {NULL}};
const Segment SGI48089[] = {{&S66839[121], 3}, {NULL}};
const Segment SGI48090[] = {{&S66839[87], 5}, {NULL}};
const Segment SGI48091[] = {{&S66839[145], 13}, {NULL}};
const Segment SGI48092[] = {{&S65691[6], 4}, {NULL}};
const Segment SGI48093[] = {{&S65691[82], 5}, {NULL}};
const Segment SGI48094[] = {{&S65713[44], 14}, {NULL}};
const Segment SGI48095[] = {{&S65713[44], 35}, {NULL}};
const Segment SGI48096[] = {{&S65713[30], 6}, {NULL}};
const Segment SGI48097[] = {{&S65713[127], 3}, {NULL}};
const Segment SGI48098[] = {{&S65713[93], 5}, {NULL}};
const Segment SGI48099[] = {{&S80805[44], 5}, {NULL}};
const Segment SGI48100[] = {{&S80805[44], 16}, {NULL}};
const Segment SGI48101[] = {{&S80805[30], 6}, {NULL}};
const Segment SGI48102[] = {{&S80805[108], 3}, {NULL}};
const Segment SGI48103[] = {{&S80805[74], 5}, {NULL}};
const Segment SGI48104[] = {{&S66934[44], 7}, {NULL}};
const Segment SGI48105[] = {{&S66934[44], 18}, {NULL}};
const Segment SGI48106[] = {{&S66934[30], 6}, {NULL}};
const Segment SGI48107[] = {{&S66934[110], 3}, {NULL}};
const Segment SGI48108[] = {{&S66934[76], 5}, {NULL}};
const Segment SGI48109[] = {{&S66935[38], 7}, {NULL}};
const Segment SGI48110[] = {{&S66935[31], 6}, {NULL}};
const Segment SGI48111[] = {{&S65956[44], 10}, {NULL}};
const Segment SGI48112[] = {{&S65956[44], 27}, {NULL}};
const Segment SGI48113[] = {{&S65956[30], 6}, {NULL}};
const Segment SGI48114[] = {{&S65956[85], 6}, {NULL}};
const Segment SGI48115[] = {{&S65956[112], 13}, {NULL}};
const Segment SGI48116[] = {{&S67163[44], 26}, {NULL}};
const Segment SGI48117[] = {{&S67163[30], 6}, {NULL}};
const Segment SGI48118[] = {{&S67163[136], 3}, {NULL}};
const Segment SGI48119[] = {{&S67163[102], 5}, {NULL}};
const Segment SGI48120[] = {{&S67164[44], 7}, {NULL}};
const Segment SGI48121[] = {{&S67164[44], 28}, {NULL}};
const Segment SGI48122[] = {{&S67164[30], 6}, {NULL}};
const Segment SGI48123[] = {{&S67164[120], 3}, {NULL}};
const Segment SGI48124[] = {{&S67164[86], 5}, {NULL}};
const Segment SGI48125[] = {{&S67168[44], 7}, {NULL}};
const Segment SGI48126[] = {{&S67168[44], 28}, {NULL}};
const Segment SGI48127[] = {{&S67168[30], 6}, {NULL}};
const Segment SGI48128[] = {{&S67168[89], 4}, {NULL}};
const Segment SGI48129[] = {{&S67168[89], 3}, {NULL}};
const Segment SGI48130[] = {{&S67171[44], 7}, {NULL}};
const Segment SGI48131[] = {{&S67171[44], 28}, {NULL}};
const Segment SGI48132[] = {{&S67171[30], 6}, {NULL}};
const Segment SGI48133[] = {{&S67171[89], 6}, {NULL}};
const Segment SGI48134[] = {{&S81066[44], 9}, {NULL}};
const Segment SGI48135[] = {{&S81066[44], 21}, {NULL}};
const Segment SGI48136[] = {{&S81066[30], 6}, {NULL}};
const Segment SGI48137[] = {{&S81066[113], 3}, {NULL}};
const Segment SGI48138[] = {{&S81066[79], 5}, {NULL}};
const Segment SGI48139[] = {{&S66039[44], 14}, {NULL}};
const Segment SGI48140[] = {{&S66039[44], 35}, {NULL}};
const Segment SGI48141[] = {{&S66039[30], 6}, {NULL}};
const Segment SGI48142[] = {{&S66039[127], 3}, {NULL}};
const Segment SGI48143[] = {{&S66039[93], 5}, {NULL}};
const Segment SGI48144[] = {{&S66189[45], 10}, {NULL}};
const Segment SGI48145[] = {{&S66189[30], 6}, {NULL}};
const Segment SGI48146[] = {{&S66189[120], 3}, {NULL}};
const Segment SGI48147[] = {{&S66189[86], 5}, {NULL}};
const Segment SGI48148[] = {{&S66189[144], 13}, {NULL}};
const Segment SGI48149[] = {{&S74447[41], 32}, {NULL}};
const Segment SGI48150[] = {{&S74447[30], 4}, {NULL}};
const Segment SGI48151[] = {{&S74447[121], 3}, {NULL}};
const Segment SGI48152[] = {{&S74447[87], 5}, {NULL}};
const Segment SGI48153[] = {{&S66244[68], 6}, {NULL}};
const Segment SGI48154[] = {{&S66244[122], 3}, {NULL}};
const Segment SGI48155[] = {{&S66244[88], 5}, {NULL}};
const Segment SGI48156[] = {{&S66275[44], 27}, {NULL}};
const Segment SGI48157[] = {{&S66275[44], 38}, {NULL}};
const Segment SGI48158[] = {{&S66275[30], 6}, {NULL}};
const Segment SGI48159[] = {{&S66275[130], 3}, {NULL}};
const Segment SGI48160[] = {{&S66275[96], 5}, {NULL}};
const Segment SGI48161[] = {{&S66304[45], 10}, {NULL}};
const Segment SGI48162[] = {{&S66304[30], 6}, {NULL}};
const Segment SGI48163[] = {{&S66304[120], 3}, {NULL}};
const Segment SGI48164[] = {{&S66304[86], 5}, {NULL}};
const Segment SGI48165[] = {{&S66304[144], 13}, {NULL}};
const Segment SGI48166[] = {{&S70100[44], 9}, {NULL}};
const Segment SGI48167[] = {{&S70100[44], 26}, {NULL}};
const Segment SGI48168[] = {{&S70100[30], 6}, {NULL}};
const Segment SGI48169[] = {{&S70100[118], 3}, {NULL}};
const Segment SGI48170[] = {{&S70100[84], 5}, {NULL}};
const Segment SGI48171[] = {{&S81481[6], 4}, {NULL}};
const Segment SGI48172[] = {{&S81481[78], 5}, {NULL}};
const Segment SGI48173[] = {{&S81519[44], 12}, {NULL}};
const Segment SGI48174[] = {{&S81519[44], 23}, {NULL}};
const Segment SGI48175[] = {{&S81519[30], 6}, {NULL}};
const Segment SGI48176[] = {{&S81519[115], 3}, {NULL}};
const Segment SGI48177[] = {{&S81519[81], 5}, {NULL}};
const Segment SGI48178[] = {{&S67561[44], 13}, {NULL}};
const Segment SGI48179[] = {{&S67561[44], 23}, {NULL}};
const Segment SGI48180[] = {{&S67561[30], 6}, {NULL}};
const Segment SGI48181[] = {{&S67561[115], 3}, {NULL}};
const Segment SGI48182[] = {{&S67561[81], 5}, {NULL}};
const Segment SGI48183[] = {{&S67564[44], 21}, {NULL}};
const Segment SGI48184[] = {{&S67564[44], 41}, {NULL}};
const Segment SGI48185[] = {{&S67564[30], 6}, {NULL}};
const Segment SGI48186[] = {{&S67564[133], 3}, {NULL}};
const Segment SGI48187[] = {{&S67564[99], 5}, {NULL}};
const Segment SGI48188[] = {{&S67566[44], 21}, {NULL}};
const Segment SGI48189[] = {{&S67566[44], 46}, {NULL}};
const Segment SGI48190[] = {{&S67566[30], 6}, {NULL}};
const Segment SGI48191[] = {{&S67566[138], 3}, {NULL}};
const Segment SGI48192[] = {{&S67566[104], 5}, {NULL}};
const Segment SGI48193[] = {{&S67567[44], 21}, {NULL}};
const Segment SGI48194[] = {{&S67567[44], 46}, {NULL}};
const Segment SGI48195[] = {{&S67567[30], 6}, {NULL}};
const Segment SGI48196[] = {{&S67567[138], 3}, {NULL}};
const Segment SGI48197[] = {{&S67567[104], 5}, {NULL}};
const Segment SGI48198[] = {{&S70193[45], 10}, {NULL}};
const Segment SGI48199[] = {{&S70193[30], 6}, {NULL}};
const Segment SGI48200[] = {{&S70193[120], 3}, {NULL}};
const Segment SGI48201[] = {{&S70193[86], 5}, {NULL}};
const Segment SGI48202[] = {{&S70193[154], 3}, {NULL}};
const Segment SGI48203[] = {{&S66448[44], 10}, {NULL}};
const Segment SGI48204[] = {{&S66448[44], 21}, {NULL}};
const Segment SGI48205[] = {{&S66448[30], 6}, {NULL}};
const Segment SGI48206[] = {{&S66448[113], 3}, {NULL}};
const Segment SGI48207[] = {{&S66448[79], 5}, {NULL}};
const Segment SGI48208[] = {{&S66449[44], 10}, {NULL}};
const Segment SGI48209[] = {{&S66449[44], 21}, {NULL}};
const Segment SGI48210[] = {{&S66449[30], 6}, {NULL}};
const Segment SGI48211[] = {{&S66449[113], 3}, {NULL}};
const Segment SGI48212[] = {{&S66449[79], 5}, {NULL}};
const Segment SGI48213[] = {{&S66450[38], 10}, {NULL}};
const Segment SGI48214[] = {{&S66450[31], 6}, {NULL}};
const Segment SGI48215[] = {{&S66451[44], 10}, {NULL}};
const Segment SGI48216[] = {{&S66451[44], 21}, {NULL}};
const Segment SGI48217[] = {{&S66451[30], 6}, {NULL}};
const Segment SGI48218[] = {{&S66451[113], 3}, {NULL}};
const Segment SGI48219[] = {{&S66451[79], 5}, {NULL}};
const Segment SGI48220[] = {{&S81614[44], 10}, {NULL}};
const Segment SGI48221[] = {{&S70264[44], 9}, {NULL}};
const Segment SGI48222[] = {{&S81614[44], 21}, {NULL}};
const Segment SGI48223[] = {{&S70264[44], 20}, {NULL}};
const Segment SGI48224[] = {{&S81614[30], 6}, {NULL}};
const Segment SGI48225[] = {{&S70264[30], 6}, {NULL}};
const Segment SGI48226[] = {{&S81614[143], 4}, {NULL}};
const Segment SGI48227[] = {{&S70264[112], 3}, {NULL}};
const Segment SGI48228[] = {{&S81614[169], 3}, {NULL}};
const Segment SGI48229[] = {{&S70264[78], 5}, {NULL}};
const Segment SGI48230[] = {{&S81614[113], 3}, {&S81614[218], 5}, {NULL}};
const Segment SGI48231[] = {{&S81614[79], 5}, {NULL}};
const Segment SGI48232[] = {{&S70265[38], 9}, {NULL}};
const Segment SGI48233[] = {{&S70265[31], 6}, {NULL}};
const Segment SGI48234[] = {{&S70266[44], 10}, {NULL}};
const Segment SGI48235[] = {{&S70266[44], 21}, {NULL}};
const Segment SGI48236[] = {{&S70266[30], 6}, {NULL}};
const Segment SGI48237[] = {{&S70266[113], 3}, {NULL}};
const Segment SGI48238[] = {{&S70266[79], 5}, {NULL}};
const Segment SGI48239[] = {{&S67686[45], 11}, {NULL}};
const Segment SGI48240[] = {{&S67686[30], 6}, {NULL}};
const Segment SGI48241[] = {{&S67686[121], 3}, {NULL}};
const Segment SGI48242[] = {{&S67686[87], 5}, {NULL}};
const Segment SGI48243[] = {{&S67686[145], 13}, {NULL}};
const Segment SGI48244[] = {{&S67688[45], 10}, {NULL}};
const Segment SGI48245[] = {{&S67688[30], 6}, {NULL}};
const Segment SGI48246[] = {{&S67688[120], 3}, {NULL}};
const Segment SGI48247[] = {{&S67688[86], 5}, {NULL}};
const Segment SGI48248[] = {{&S67688[144], 13}, {NULL}};
const Segment SGI48249[] = {{&S66558[44], 5}, {NULL}};
const Segment SGI48250[] = {{&S66558[44], 16}, {NULL}};
const Segment SGI48251[] = {{&S66558[30], 6}, {NULL}};
const Segment SGI48252[] = {{&S66558[108], 3}, {NULL}};
const Segment SGI48253[] = {{&S66558[74], 5}, {NULL}};
const Segment SGI48254[] = {{&S81743[44], 4}, {NULL}};
const Segment SGI48255[] = {{&S81743[44], 16}, {NULL}};
const Segment SGI48256[] = {{&S81743[30], 6}, {NULL}};
const Segment SGI48257[] = {{&S81743[108], 3}, {NULL}};
const Segment SGI48258[] = {{&S81743[74], 5}, {NULL}};
const Segment SGI48259[] = {{&S66771[44], 6}, {NULL}};
const Segment SGI48260[] = {{&S66771[44], 27}, {NULL}};
const Segment SGI48261[] = {{&S66771[30], 6}, {NULL}};
const Segment SGI48262[] = {{&S81860[44], 37}, {NULL}};
const Segment SGI48263[] = {{&S81860[30], 6}, {NULL}};
const Segment SGI48264[] = {{&S66771[119], 3}, {NULL}};
const Segment SGI48265[] = {{&S66771[85], 5}, {NULL}};
const Segment SGI48266[] = {{&S81860[113], 5}, {NULL}};
const Segment SGI48267[] = {{&S81863[44], 7}, {NULL}};
const Segment SGI48268[] = {{&S81863[44], 19}, {NULL}};
const Segment SGI48269[] = {{&S81863[30], 6}, {NULL}};
const Segment SGI48270[] = {{&S81863[80], 4}, {NULL}};
const Segment SGI48271[] = {{&S81863[80], 3}, {NULL}};
const Segment SGI48272[] = {{&S66836[45], 10}, {NULL}};
const Segment SGI48273[] = {{&S66836[30], 6}, {NULL}};
const Segment SGI48274[] = {{&S66836[120], 3}, {NULL}};
const Segment SGI48275[] = {{&S66836[86], 5}, {NULL}};
const Segment SGI48276[] = {{&S66836[144], 13}, {NULL}};
const Segment SGI48277[] = {{&S68021[31], 8}, {NULL}};
const Segment SGI48278[] = {{&S68021[54], 3}, {NULL}};
const Segment SGI48279[] = {{&S82022[44], 14}, {NULL}};
const Segment SGI48280[] = {{&S82022[44], 26}, {NULL}};
const Segment SGI48281[] = {{&S82022[30], 6}, {NULL}};
const Segment SGI48282[] = {{&S82022[118], 3}, {NULL}};
const Segment SGI48283[] = {{&S82022[84], 5}, {NULL}};
const Segment SGI48284[] = {{&S82041[44], 12}, {NULL}};
const Segment SGI48285[] = {{&S82041[44], 23}, {NULL}};
const Segment SGI48286[] = {{&S82041[30], 6}, {NULL}};
const Segment SGI48287[] = {{&S82041[84], 4}, {NULL}};
const Segment SGI48288[] = {{&S82041[84], 3}, {NULL}};
const Segment SGI48289[] = {{&S82042[44], 12}, {NULL}};
const Segment SGI48290[] = {{&S82042[44], 23}, {NULL}};
const Segment SGI48291[] = {{&S82042[30], 6}, {NULL}};
const Segment SGI48292[] = {{&S82042[115], 3}, {NULL}};
const Segment SGI48293[] = {{&S82042[81], 5}, {NULL}};
const Segment SGI48294[] = {{&S82043[44], 12}, {NULL}};
const Segment SGI48295[] = {{&S82043[44], 23}, {NULL}};
const Segment SGI48296[] = {{&S82043[30], 6}, {NULL}};
const Segment SGI48297[] = {{&S82043[84], 6}, {NULL}};
const Segment SGI48298[] = {{&S66962[56], 6}, {NULL}};
const Segment SGI48299[] = {{&S66962[110], 3}, {NULL}};
const Segment SGI48300[] = {{&S66962[76], 5}, {NULL}};
const Segment SGI48301[] = {{&S66963[44], 14}, {NULL}};
const Segment SGI48302[] = {{&S66963[44], 35}, {NULL}};
const Segment SGI48303[] = {{&S66963[30], 6}, {NULL}};
const Segment SGI48304[] = {{&S66963[96], 6}, {NULL}};
const Segment SGI48305[] = {{&S66964[44], 35}, {NULL}};
const Segment SGI48306[] = {{&S66964[30], 6}, {NULL}};
const Segment SGI48307[] = {{&S66965[38], 14}, {NULL}};
const Segment SGI48308[] = {{&S66965[31], 6}, {NULL}};
const Segment SGI48309[] = {{&S68128[44], 14}, {NULL}};
const Segment SGI48310[] = {{&S68128[44], 24}, {NULL}};
const Segment SGI48311[] = {{&S68128[30], 6}, {NULL}};
const Segment SGI48312[] = {{&S68128[116], 3}, {NULL}};
const Segment SGI48313[] = {{&S68128[82], 5}, {NULL}};
const Segment SGI48314[] = {{&S70795[31], 8}, {NULL}};
const Segment SGI48315[] = {{&S70795[54], 3}, {NULL}};
const Segment SGI48316[] = {{&S67256[44], 7}, {NULL}};
const Segment SGI48317[] = {{&S67256[44], 18}, {NULL}};
const Segment SGI48318[] = {{&S67256[30], 6}, {NULL}};
const Segment SGI48319[] = {{&S67256[110], 3}, {NULL}};
const Segment SGI48320[] = {{&S67256[76], 5}, {NULL}};
const Segment SGI48321[] = {{&S82515[44], 8}, {NULL}};
const Segment SGI48322[] = {{&S82515[44], 20}, {NULL}};
const Segment SGI48323[] = {{&S82515[30], 6}, {NULL}};
const Segment SGI48324[] = {{&S82515[112], 3}, {NULL}};
const Segment SGI48325[] = {{&S82515[78], 5}, {NULL}};
const Segment SGI48326[] = {{&S82516[44], 8}, {NULL}};
const Segment SGI48327[] = {{&S82516[44], 19}, {NULL}};
const Segment SGI48328[] = {{&S82516[30], 6}, {NULL}};
const Segment SGI48329[] = {{&S82516[111], 3}, {NULL}};
const Segment SGI48330[] = {{&S82516[77], 5}, {NULL}};
const Segment SGI48331[] = {{&S72563[44], 9}, {NULL}};
const Segment SGI48332[] = {{&S72563[44], 20}, {NULL}};
const Segment SGI48333[] = {{&S72563[30], 6}, {NULL}};
const Segment SGI48334[] = {{&S72563[112], 3}, {NULL}};
const Segment SGI48335[] = {{&S72563[78], 5}, {NULL}};
const Segment SGI48336[] = {{&S77069[44], 8}, {NULL}};
const Segment SGI48337[] = {{&S77069[44], 25}, {NULL}};
const Segment SGI48338[] = {{&S77069[30], 6}, {NULL}};
const Segment SGI48339[] = {{&S77069[117], 3}, {NULL}};
const Segment SGI48340[] = {{&S77069[83], 5}, {NULL}};
const Segment SGI48341[] = {{&S77070[44], 13}, {NULL}};
const Segment SGI48342[] = {{&S77070[44], 30}, {NULL}};
const Segment SGI48343[] = {{&S77070[30], 6}, {NULL}};
const Segment SGI48344[] = {{&S77070[122], 3}, {NULL}};
const Segment SGI48345[] = {{&S77070[88], 5}, {NULL}};
const Segment SGI48346[] = {{&S82669[41], 9}, {NULL}};
const Segment SGI48347[] = {{&S82669[30], 6}, {NULL}};
const Segment SGI48348[] = {{&S82669[110], 3}, {NULL}};
const Segment SGI48349[] = {{&S82669[76], 5}, {NULL}};
const Segment SGI48350[] = {{&S82670[44], 9}, {NULL}};
const Segment SGI48351[] = {{&S82670[44], 21}, {NULL}};
const Segment SGI48352[] = {{&S82670[30], 6}, {NULL}};
const Segment SGI48353[] = {{&S82670[113], 3}, {NULL}};
const Segment SGI48354[] = {{&S82670[79], 5}, {NULL}};
const Segment SGI48355[] = {{&S82671[44], 9}, {NULL}};
const Segment SGI48356[] = {{&S82671[44], 21}, {NULL}};
const Segment SGI48357[] = {{&S82671[30], 6}, {NULL}};
const Segment SGI48358[] = {{&S82671[82], 6}, {NULL}};
const Segment SGI48359[] = {{&S82672[39], 7}, {NULL}};
const Segment SGI48360[] = {{&S82672[102], 3}, {NULL}};
const Segment SGI48361[] = {{&S82672[91], 3}, {NULL}};
const Segment SGI48362[] = {{&S82672[68], 6}, {NULL}};
const Segment SGI48363[] = {{&S82673[31], 7}, {NULL}};
const Segment SGI48364[] = {{&S82673[53], 3}, {NULL}};
const Segment SGI48365[] = {{&S67562[44], 13}, {NULL}};
const Segment SGI48366[] = {{&S67562[44], 23}, {NULL}};
const Segment SGI48367[] = {{&S67562[30], 6}, {NULL}};
const Segment SGI48368[] = {{&S67562[115], 3}, {NULL}};
const Segment SGI48369[] = {{&S67562[81], 5}, {NULL}};
const Segment SGI48370[] = {{&S67563[44], 13}, {NULL}};
const Segment SGI48371[] = {{&S67563[44], 35}, {NULL}};
const Segment SGI48372[] = {{&S67563[30], 6}, {NULL}};
const Segment SGI48373[] = {{&S67563[127], 3}, {NULL}};
const Segment SGI48374[] = {{&S67563[93], 5}, {NULL}};
const Segment SGI48375[] = {{&S67568[44], 13}, {NULL}};
const Segment SGI48376[] = {{&S67568[44], 30}, {NULL}};
const Segment SGI48377[] = {{&S67568[30], 6}, {NULL}};
const Segment SGI48378[] = {{&S67568[122], 3}, {NULL}};
const Segment SGI48379[] = {{&S67568[88], 5}, {NULL}};
const Segment SGI48380[] = {{&S82826[44], 10}, {NULL}};
const Segment SGI48381[] = {{&S82826[44], 50}, {NULL}};
const Segment SGI48382[] = {{&S82826[30], 6}, {NULL}};
const Segment SGI48383[] = {{&S82826[142], 3}, {NULL}};
const Segment SGI48384[] = {{&S82826[108], 5}, {NULL}};
const Segment SGI48385[] = {{&S67683[45], 10}, {NULL}};
const Segment SGI48386[] = {{&S67683[30], 6}, {NULL}};
const Segment SGI48387[] = {{&S67683[120], 3}, {NULL}};
const Segment SGI48388[] = {{&S82827[44], 10}, {NULL}};
const Segment SGI48389[] = {{&S82827[44], 50}, {NULL}};
const Segment SGI48390[] = {{&S82827[30], 6}, {NULL}};
const Segment SGI48391[] = {{&S67683[86], 5}, {NULL}};
const Segment SGI48392[] = {{&S67683[144], 13}, {NULL}};
const Segment SGI48393[] = {{&S82827[142], 3}, {NULL}};
const Segment SGI48394[] = {{&S82827[108], 5}, {NULL}};
const Segment SGI48395[] = {{&S82828[44], 10}, {NULL}};
const Segment SGI48396[] = {{&S82828[44], 50}, {NULL}};
const Segment SGI48397[] = {{&S82828[30], 6}, {NULL}};
const Segment SGI48398[] = {{&S82828[142], 3}, {NULL}};
const Segment SGI48399[] = {{&S82828[108], 5}, {NULL}};
const Segment SGI48400[] = {{&S72888[44], 15}, {NULL}};
const Segment SGI48401[] = {{&S72888[44], 27}, {NULL}};
const Segment SGI48402[] = {{&S72888[30], 6}, {NULL}};
const Segment SGI48403[] = {{&S72888[119], 3}, {NULL}};
const Segment SGI48404[] = {{&S72888[85], 5}, {NULL}};
const Segment SGI48405[] = {{&S71342[44], 11}, {NULL}};
const Segment SGI48406[] = {{&S71342[44], 22}, {NULL}};
const Segment SGI48407[] = {{&S71342[30], 6}, {NULL}};
const Segment SGI48408[] = {{&S71342[114], 3}, {NULL}};
const Segment SGI48409[] = {{&S71342[80], 5}, {NULL}};
const Segment SGI48410[] = {{&S67811[44], 10}, {NULL}};
const Segment SGI48411[] = {{&S67811[44], 21}, {NULL}};
const Segment SGI48412[] = {{&S67811[30], 6}, {NULL}};
const Segment SGI48413[] = {{&S67811[113], 3}, {NULL}};
const Segment SGI48414[] = {{&S67811[79], 5}, {NULL}};
const Segment SGI48415[] = {{&S67812[44], 10}, {NULL}};
const Segment SGI48416[] = {{&S67812[44], 21}, {NULL}};
const Segment SGI48417[] = {{&S67812[30], 6}, {NULL}};
const Segment SGI48418[] = {{&S67812[82], 4}, {NULL}};
const Segment SGI48419[] = {{&S67812[82], 3}, {NULL}};
const Segment SGI48420[] = {{&S67813[42], 10}, {NULL}};
const Segment SGI48421[] = {{&S67813[30], 6}, {NULL}};
const Segment SGI48422[] = {{&S67813[111], 3}, {NULL}};
const Segment SGI48423[] = {{&S67813[77], 5}, {NULL}};
const Segment SGI48424[] = {{&S67814[38], 10}, {NULL}};
const Segment SGI48425[] = {{&S67814[31], 6}, {NULL}};
const Segment SGI48426[] = {{&S82955[44], 11}, {NULL}};
const Segment SGI48427[] = {{&S82955[44], 23}, {NULL}};
const Segment SGI48428[] = {{&S67815[44], 10}, {NULL}};
const Segment SGI48429[] = {{&S82955[30], 6}, {NULL}};
const Segment SGI48430[] = {{&S67815[44], 21}, {NULL}};
const Segment SGI48431[] = {{&S67815[30], 6}, {NULL}};
const Segment SGI48432[] = {{&S82955[115], 3}, {NULL}};
const Segment SGI48433[] = {{&S67815[82], 6}, {NULL}};
const Segment SGI48434[] = {{&S82955[81], 5}, {NULL}};
const Segment SGI48435[] = {{&S67863[44], 10}, {NULL}};
const Segment SGI48436[] = {{&S67863[44], 26}, {NULL}};
const Segment SGI48437[] = {{&S67863[30], 6}, {NULL}};
const Segment SGI48438[] = {{&S67863[118], 3}, {NULL}};
const Segment SGI48439[] = {{&S67863[84], 5}, {NULL}};
const Segment SGI48440[] = {{&S67864[44], 10}, {NULL}};
const Segment SGI48441[] = {{&S67864[44], 26}, {NULL}};
const Segment SGI48442[] = {{&S67864[30], 6}, {NULL}};
const Segment SGI48443[] = {{&S67864[118], 3}, {NULL}};
const Segment SGI48444[] = {{&S67864[84], 5}, {NULL}};
const Segment SGI48445[] = {{&S67866[44], 10}, {NULL}};
const Segment SGI48446[] = {{&S67866[44], 27}, {NULL}};
const Segment SGI48447[] = {{&S67866[30], 6}, {NULL}};
const Segment SGI48448[] = {{&S67866[119], 3}, {NULL}};
const Segment SGI48449[] = {{&S67866[85], 5}, {NULL}};
const Segment SGI48450[] = {{&S67867[44], 10}, {NULL}};
const Segment SGI48451[] = {{&S67867[44], 27}, {NULL}};
const Segment SGI48452[] = {{&S67867[30], 6}, {NULL}};
const Segment SGI48453[] = {{&S67867[119], 3}, {NULL}};
const Segment SGI48454[] = {{&S67867[85], 5}, {NULL}};
const Segment SGI48455[] = {{&S67868[44], 10}, {NULL}};
const Segment SGI48456[] = {{&S67868[44], 27}, {NULL}};
const Segment SGI48457[] = {{&S67868[30], 6}, {NULL}};
const Segment SGI48458[] = {{&S67868[119], 3}, {NULL}};
const Segment SGI48459[] = {{&S67868[85], 5}, {NULL}};
const Segment SGI48460[] = {{&S68830[44], 16}, {NULL}};
const Segment SGI48461[] = {{&S68830[44], 26}, {NULL}};
const Segment SGI48462[] = {{&S68830[30], 6}, {NULL}};
const Segment SGI48463[] = {{&S68830[118], 3}, {NULL}};
const Segment SGI48464[] = {{&S68830[84], 5}, {NULL}};
const Segment SGI48465[] = {{&S68831[44], 11}, {NULL}};
const Segment SGI48466[] = {{&S68831[44], 22}, {NULL}};
const Segment SGI48467[] = {{&S68831[30], 6}, {NULL}};
const Segment SGI48468[] = {{&S68831[114], 3}, {NULL}};
const Segment SGI48469[] = {{&S68831[80], 5}, {NULL}};
const Segment SGI48470[] = {{&S68832[44], 11}, {NULL}};
const Segment SGI48471[] = {{&S68832[44], 35}, {NULL}};
const Segment SGI48472[] = {{&S68832[30], 6}, {NULL}};
const Segment SGI48473[] = {{&S68833[38], 11}, {NULL}};
const Segment SGI48474[] = {{&S68833[31], 6}, {NULL}};
const Segment SGI48475[] = {{&S83050[45], 9}, {NULL}};
const Segment SGI48476[] = {{&S83050[30], 6}, {NULL}};
const Segment SGI48477[] = {{&S83050[113], 3}, {NULL}};
const Segment SGI48478[] = {{&S83050[79], 5}, {NULL}};
const Segment SGI48479[] = {{&S68834[44], 11}, {NULL}};
const Segment SGI48480[] = {{&S83050[137], 13}, {NULL}};
const Segment SGI48481[] = {{&S68834[44], 22}, {NULL}};
const Segment SGI48482[] = {{&S68834[30], 6}, {NULL}};
const Segment SGI48483[] = {{&S68834[127], 3}, {NULL}};
const Segment SGI48484[] = {{&S68834[80], 6}, {NULL}};
const Segment SGI48485[] = {{&S68888[45], 10}, {NULL}};
const Segment SGI48486[] = {{&S68888[30], 6}, {NULL}};
const Segment SGI48487[] = {{&S68888[120], 3}, {NULL}};
const Segment SGI48488[] = {{&S68888[86], 5}, {NULL}};
const Segment SGI48489[] = {{&S68888[144], 13}, {NULL}};
const Segment SGI48490[] = {{&S68890[45], 11}, {NULL}};
const Segment SGI48491[] = {{&S68890[30], 6}, {NULL}};
const Segment SGI48492[] = {{&S68890[121], 3}, {NULL}};
const Segment SGI48493[] = {{&S68890[87], 5}, {NULL}};
const Segment SGI48494[] = {{&S68890[145], 13}, {NULL}};
const Segment SGI48495[] = {{&S75973[42], 13}, {NULL}};
const Segment SGI48496[] = {{&S75973[42], 24}, {NULL}};
const Segment SGI48497[] = {{&S75973[30], 4}, {NULL}};
const Segment SGI48498[] = {{&S75973[114], 3}, {NULL}};
const Segment SGI48499[] = {{&S75973[80], 5}, {NULL}};
const Segment SGI48500[] = {{&S75974[42], 14}, {NULL}};
const Segment SGI48501[] = {{&S75974[42], 25}, {NULL}};
const Segment SGI48502[] = {{&S75974[30], 4}, {NULL}};
const Segment SGI48503[] = {{&S75974[115], 3}, {NULL}};
const Segment SGI48504[] = {{&S75974[81], 5}, {NULL}};
const Segment SGI48505[] = {{&S67975[44], 5}, {NULL}};
const Segment SGI48506[] = {{&S67975[44], 16}, {NULL}};
const Segment SGI48507[] = {{&S67975[30], 6}, {NULL}};
const Segment SGI48508[] = {{&S67975[108], 3}, {NULL}};
const Segment SGI48509[] = {{&S67975[74], 5}, {NULL}};
const Segment SGI48510[] = {{&S67976[44], 14}, {NULL}};
const Segment SGI48511[] = {{&S67976[44], 25}, {NULL}};
const Segment SGI48512[] = {{&S67976[30], 6}, {NULL}};
const Segment SGI48513[] = {{&S67976[117], 3}, {NULL}};
const Segment SGI48514[] = {{&S67976[83], 5}, {NULL}};
const Segment SGI48515[] = {{&S71570[44], 37}, {NULL}};
const Segment SGI48516[] = {{&S71570[30], 6}, {NULL}};
const Segment SGI48517[] = {{&S71570[113], 5}, {NULL}};
const Segment SGI48518[] = {{&S71571[44], 7}, {NULL}};
const Segment SGI48519[] = {{&S71571[44], 24}, {NULL}};
const Segment SGI48520[] = {{&S71571[30], 6}, {NULL}};
const Segment SGI48521[] = {{&S71571[85], 4}, {NULL}};
const Segment SGI48522[] = {{&S71571[85], 3}, {NULL}};
const Segment SGI48523[] = {{&S71577[44], 12}, {NULL}};
const Segment SGI48524[] = {{&S71577[44], 29}, {NULL}};
const Segment SGI48525[] = {{&S71577[30], 6}, {NULL}};
const Segment SGI48526[] = {{&S71577[121], 3}, {NULL}};
const Segment SGI48527[] = {{&S71577[87], 5}, {NULL}};
const Segment SGI48528[] = {{&S71578[44], 25}, {NULL}};
const Segment SGI48529[] = {{&S71578[44], 42}, {NULL}};
const Segment SGI48530[] = {{&S71578[30], 6}, {NULL}};
const Segment SGI48531[] = {{&S71578[134], 3}, {NULL}};
const Segment SGI48532[] = {{&S71578[100], 5}, {NULL}};
const Segment SGI48533[] = {{&S68022[31], 8}, {NULL}};
const Segment SGI48534[] = {{&S68022[54], 3}, {NULL}};
const Segment SGI48535[] = {{&S68049[44], 11}, {NULL}};
const Segment SGI48536[] = {{&S68049[44], 28}, {NULL}};
const Segment SGI48537[] = {{&S68049[30], 6}, {NULL}};
const Segment SGI48538[] = {{&S68049[120], 3}, {NULL}};
const Segment SGI48539[] = {{&S68049[86], 5}, {NULL}};
const Segment SGI48540[] = {{&S68050[44], 10}, {NULL}};
const Segment SGI48541[] = {{&S68050[44], 27}, {NULL}};
const Segment SGI48542[] = {{&S68050[30], 6}, {NULL}};
const Segment SGI48543[] = {{&S68050[119], 3}, {NULL}};
const Segment SGI48544[] = {{&S68050[85], 5}, {NULL}};
const Segment SGI48545[] = {{&S71657[45], 10}, {NULL}};
const Segment SGI48546[] = {{&S71657[30], 6}, {NULL}};
const Segment SGI48547[] = {{&S71657[120], 3}, {NULL}};
const Segment SGI48548[] = {{&S71657[86], 5}, {NULL}};
const Segment SGI48549[] = {{&S71657[154], 3}, {NULL}};
const Segment SGI48550[] = {{&S69049[44], 11}, {NULL}};
const Segment SGI48551[] = {{&S69049[44], 28}, {NULL}};
const Segment SGI48552[] = {{&S69049[30], 6}, {NULL}};
const Segment SGI48553[] = {{&S69049[120], 3}, {NULL}};
const Segment SGI48554[] = {{&S69049[86], 5}, {NULL}};
const Segment SGI48555[] = {{&S69050[38], 11}, {NULL}};
const Segment SGI48556[] = {{&S69050[31], 6}, {NULL}};
const Segment SGI48557[] = {{&S77683[44], 5}, {NULL}};
const Segment SGI48558[] = {{&S77683[44], 22}, {NULL}};
const Segment SGI48559[] = {{&S77683[30], 6}, {NULL}};
const Segment SGI48560[] = {{&S77683[114], 3}, {NULL}};
const Segment SGI48561[] = {{&S77683[80], 5}, {NULL}};
const Segment SGI48562[] = {{&S69101[44], 9}, {NULL}};
const Segment SGI48563[] = {{&S69101[44], 20}, {NULL}};
const Segment SGI48564[] = {{&S69101[30], 6}, {NULL}};
const Segment SGI48565[] = {{&S69101[112], 3}, {NULL}};
const Segment SGI48566[] = {{&S69101[78], 5}, {NULL}};
const Segment SGI48567[] = {{&S69114[45], 10}, {NULL}};
const Segment SGI48568[] = {{&S69114[30], 6}, {NULL}};
const Segment SGI48569[] = {{&S69114[120], 3}, {NULL}};
const Segment SGI48570[] = {{&S69114[86], 5}, {NULL}};
const Segment SGI48571[] = {{&S69114[144], 13}, {NULL}};
const Segment SGI48572[] = {{&S83390[44], 10}, {NULL}};
const Segment SGI48573[] = {{&S83390[44], 22}, {NULL}};
const Segment SGI48574[] = {{&S83390[30], 6}, {NULL}};
const Segment SGI48575[] = {{&S69125[44], 9}, {NULL}};
const Segment SGI48576[] = {{&S69125[44], 20}, {NULL}};
const Segment SGI48577[] = {{&S69125[30], 6}, {NULL}};
const Segment SGI48578[] = {{&S83390[114], 3}, {NULL}};
const Segment SGI48579[] = {{&S83390[80], 5}, {NULL}};
const Segment SGI48580[] = {{&S69125[112], 3}, {NULL}};
const Segment SGI48581[] = {{&S69125[78], 5}, {NULL}};
const Segment SGI48582[] = {{&S69126[44], 9}, {NULL}};
const Segment SGI48583[] = {{&S69126[44], 20}, {NULL}};
const Segment SGI48584[] = {{&S69126[30], 6}, {NULL}};
const Segment SGI48585[] = {{&S69126[112], 3}, {NULL}};
const Segment SGI48586[] = {{&S69126[78], 5}, {NULL}};
const Segment SGI48587[] = {{&S68127[44], 14}, {NULL}};
const Segment SGI48588[] = {{&S68127[44], 24}, {NULL}};
const Segment SGI48589[] = {{&S68127[30], 6}, {NULL}};
const Segment SGI48590[] = {{&S68127[116], 3}, {NULL}};
const Segment SGI48591[] = {{&S68127[82], 5}, {NULL}};
const Segment SGI48592[] = {{&S83404[44], 7}, {NULL}};
const Segment SGI48593[] = {{&S83404[44], 19}, {NULL}};
const Segment SGI48594[] = {{&S83404[30], 6}, {NULL}};
const Segment SGI48595[] = {{&S83404[111], 3}, {NULL}};
const Segment SGI48596[] = {{&S83404[77], 5}, {NULL}};
const Segment SGI48597[] = {{&S76356[42], 13}, {NULL}};
const Segment SGI48598[] = {{&S76356[42], 24}, {NULL}};
const Segment SGI48599[] = {{&S76356[30], 4}, {NULL}};
const Segment SGI48600[] = {{&S76356[114], 3}, {NULL}};
const Segment SGI48601[] = {{&S76356[80], 5}, {NULL}};
const Segment SGI48602[] = {{&S69205[44], 11}, {NULL}};
const Segment SGI48603[] = {{&S69205[44], 22}, {NULL}};
const Segment SGI48604[] = {{&S69205[30], 6}, {NULL}};
const Segment SGI48605[] = {{&S69205[114], 3}, {NULL}};
const Segment SGI48606[] = {{&S69205[80], 5}, {NULL}};
const Segment SGI48607[] = {{&S69206[38], 11}, {NULL}};
const Segment SGI48608[] = {{&S69206[31], 6}, {NULL}};
const Segment SGI48609[] = {{&S83480[44], 7}, {NULL}};
const Segment SGI48610[] = {{&S83480[44], 58}, {NULL}};
const Segment SGI48611[] = {{&S83480[30], 6}, {NULL}};
const Segment SGI48612[] = {{&S83480[150], 3}, {NULL}};
const Segment SGI48613[] = {{&S83480[116], 5}, {NULL}};
const Segment SGI48614[] = {{&S83481[44], 7}, {NULL}};
const Segment SGI48615[] = {{&S83481[44], 28}, {NULL}};
const Segment SGI48616[] = {{&S83481[30], 6}, {NULL}};
const Segment SGI48617[] = {{&S83481[120], 3}, {NULL}};
const Segment SGI48618[] = {{&S83481[86], 5}, {NULL}};
const Segment SGI48619[] = {{&S83482[44], 7}, {NULL}};
const Segment SGI48620[] = {{&S83482[44], 28}, {NULL}};
const Segment SGI48621[] = {{&S83482[30], 6}, {NULL}};
const Segment SGI48622[] = {{&S83482[120], 3}, {NULL}};
const Segment SGI48623[] = {{&S83482[86], 5}, {NULL}};
const Segment SGI48624[] = {{&S83483[44], 7}, {NULL}};
const Segment SGI48625[] = {{&S83483[44], 58}, {NULL}};
const Segment SGI48626[] = {{&S83483[30], 6}, {NULL}};
const Segment SGI48627[] = {{&S83483[150], 3}, {NULL}};
const Segment SGI48628[] = {{&S83483[116], 5}, {NULL}};
const Segment SGI48629[] = {{&S83484[44], 7}, {NULL}};
const Segment SGI48630[] = {{&S83484[44], 28}, {NULL}};
const Segment SGI48631[] = {{&S83484[30], 6}, {NULL}};
const Segment SGI48632[] = {{&S83484[120], 3}, {NULL}};
const Segment SGI48633[] = {{&S83484[86], 5}, {NULL}};
const Segment SGI48634[] = {{&S83485[44], 7}, {NULL}};
const Segment SGI48635[] = {{&S83485[44], 58}, {NULL}};
const Segment SGI48636[] = {{&S83485[30], 6}, {NULL}};
const Segment SGI48637[] = {{&S83485[150], 3}, {NULL}};
const Segment SGI48638[] = {{&S83485[116], 5}, {NULL}};
const Segment SGI48639[] = {{&S71917[44], 10}, {NULL}};
const Segment SGI48640[] = {{&S71917[44], 27}, {NULL}};
const Segment SGI48641[] = {{&S71917[30], 6}, {NULL}};
const Segment SGI48642[] = {{&S71917[119], 3}, {NULL}};
const Segment SGI48643[] = {{&S71917[85], 5}, {NULL}};
const Segment SGI48644[] = {{&S68156[44], 10}, {NULL}};
const Segment SGI48645[] = {{&S68156[44], 27}, {NULL}};
const Segment SGI48646[] = {{&S68156[30], 6}, {NULL}};
const Segment SGI48647[] = {{&S68156[85], 6}, {NULL}};
const Segment SGI48648[] = {{&S68156[112], 13}, {NULL}};
const Segment SGI48649[] = {{&S76453[44], 9}, {NULL}};
const Segment SGI48650[] = {{&S76453[44], 19}, {NULL}};
const Segment SGI48651[] = {{&S76453[30], 6}, {NULL}};
const Segment SGI48652[] = {{&S76453[111], 3}, {NULL}};
const Segment SGI48653[] = {{&S76453[77], 5}, {NULL}};
const Segment SGI48654[] = {{&S68185[44], 8}, {NULL}};
const Segment SGI48655[] = {{&S68185[44], 19}, {NULL}};
const Segment SGI48656[] = {{&S68185[30], 6}, {NULL}};
const Segment SGI48657[] = {{&S68185[111], 3}, {NULL}};
const Segment SGI48658[] = {{&S68185[77], 5}, {NULL}};
const Segment SGI48659[] = {{&S68186[44], 8}, {NULL}};
const Segment SGI48660[] = {{&S68186[44], 19}, {NULL}};
const Segment SGI48661[] = {{&S68186[30], 6}, {NULL}};
const Segment SGI48662[] = {{&S68186[111], 3}, {NULL}};
const Segment SGI48663[] = {{&S68186[77], 5}, {NULL}};
const Segment SGI48664[] = {{&S68187[44], 8}, {NULL}};
const Segment SGI48665[] = {{&S68187[44], 19}, {NULL}};
const Segment SGI48666[] = {{&S68187[30], 6}, {NULL}};
const Segment SGI48667[] = {{&S68187[111], 3}, {NULL}};
const Segment SGI48668[] = {{&S68187[77], 5}, {NULL}};
const Segment SGI48669[] = {{&S68188[44], 8}, {NULL}};
const Segment SGI48670[] = {{&S68188[44], 19}, {NULL}};
const Segment SGI48671[] = {{&S68188[30], 6}, {NULL}};
const Segment SGI48672[] = {{&S68188[111], 3}, {NULL}};
const Segment SGI48673[] = {{&S68188[77], 5}, {NULL}};
const Segment SGI48674[] = {{&S68189[44], 8}, {NULL}};
const Segment SGI48675[] = {{&S68189[44], 19}, {NULL}};
const Segment SGI48676[] = {{&S68189[30], 6}, {NULL}};
const Segment SGI48677[] = {{&S68189[111], 3}, {NULL}};
const Segment SGI48678[] = {{&S68189[77], 5}, {NULL}};
const Segment SGI48679[] = {{&S68190[38], 8}, {NULL}};
const Segment SGI48680[] = {{&S68190[31], 6}, {NULL}};
const Segment SGI48681[] = {{&S68191[44], 8}, {NULL}};
const Segment SGI48682[] = {{&S68191[44], 19}, {NULL}};
const Segment SGI48683[] = {{&S68191[30], 6}, {NULL}};
const Segment SGI48684[] = {{&S68191[111], 3}, {NULL}};
const Segment SGI48685[] = {{&S68191[77], 5}, {NULL}};
const Segment SGI48686[] = {{&S69364[44], 9}, {NULL}};
const Segment SGI48687[] = {{&S69364[44], 19}, {NULL}};
const Segment SGI48688[] = {{&S69364[30], 6}, {NULL}};
const Segment SGI48689[] = {{&S69364[111], 3}, {NULL}};
const Segment SGI48690[] = {{&S69364[77], 5}, {NULL}};
const Segment SGI48691[] = {{&S72147[44], 9}, {NULL}};
const Segment SGI48692[] = {{&S72147[44], 20}, {NULL}};
const Segment SGI48693[] = {{&S72147[30], 6}, {NULL}};
const Segment SGI48694[] = {{&S72147[112], 3}, {NULL}};
const Segment SGI48695[] = {{&S72147[78], 5}, {NULL}};
const Segment SGI48696[] = {{&S83773[44], 5}, {NULL}};
const Segment SGI48697[] = {{&S83773[44], 16}, {NULL}};
const Segment SGI48698[] = {{&S83773[30], 6}, {NULL}};
const Segment SGI48699[] = {{&S83773[108], 3}, {NULL}};
const Segment SGI48700[] = {{&S83773[74], 5}, {NULL}};
const Segment SGI48701[] = {{&S69499[44], 11}, {NULL}};
const Segment SGI48702[] = {{&S69499[44], 28}, {NULL}};
const Segment SGI48703[] = {{&S69499[30], 6}, {NULL}};
const Segment SGI48704[] = {{&S69499[120], 3}, {NULL}};
const Segment SGI48705[] = {{&S69499[86], 5}, {NULL}};
const Segment SGI48706[] = {{&S69503[44], 11}, {NULL}};
const Segment SGI48707[] = {{&S69503[44], 22}, {NULL}};
const Segment SGI48708[] = {{&S69503[30], 6}, {NULL}};
const Segment SGI48709[] = {{&S69503[114], 3}, {NULL}};
const Segment SGI48710[] = {{&S69503[80], 5}, {NULL}};
const Segment SGI48711[] = {{&S69504[38], 11}, {NULL}};
const Segment SGI48712[] = {{&S69504[31], 6}, {NULL}};
const Segment SGI48713[] = {{&S69540[44], 20}, {NULL}};
const Segment SGI48714[] = {{&S69540[44], 27}, {NULL}};
const Segment SGI48715[] = {{&S69540[30], 6}, {NULL}};
const Segment SGI48716[] = {{&S69540[119], 3}, {NULL}};
const Segment SGI48717[] = {{&S69540[85], 5}, {NULL}};
const Segment SGI48718[] = {{&S68495[44], 27}, {NULL}};
const Segment SGI48719[] = {{&S68495[30], 6}, {NULL}};
const Segment SGI48720[] = {{&S68495[137], 3}, {NULL}};
const Segment SGI48721[] = {{&S68495[103], 5}, {NULL}};
const Segment SGI48722[] = {{&S68496[44], 27}, {NULL}};
const Segment SGI48723[] = {{&S68496[30], 6}, {NULL}};
const Segment SGI48724[] = {{&S68496[137], 3}, {NULL}};
const Segment SGI48725[] = {{&S68496[103], 5}, {NULL}};
const Segment SGI48726[] = {{&S68497[44], 14}, {NULL}};
const Segment SGI48727[] = {{&S68497[44], 25}, {NULL}};
const Segment SGI48728[] = {{&S68497[30], 6}, {NULL}};
const Segment SGI48729[] = {{&S68497[117], 3}, {NULL}};
const Segment SGI48730[] = {{&S68497[83], 5}, {NULL}};
const Segment SGI48731[] = {{&S68498[44], 18}, {NULL}};
const Segment SGI48732[] = {{&S68498[44], 35}, {NULL}};
const Segment SGI48733[] = {{&S68498[30], 6}, {NULL}};
const Segment SGI48734[] = {{&S68498[127], 3}, {NULL}};
const Segment SGI48735[] = {{&S68498[93], 5}, {NULL}};
const Segment SGI48736[] = {{&S68499[44], 14}, {NULL}};
const Segment SGI48737[] = {{&S68499[44], 25}, {NULL}};
const Segment SGI48738[] = {{&S68499[30], 6}, {NULL}};
const Segment SGI48739[] = {{&S68499[86], 6}, {NULL}};
const Segment SGI48740[] = {{&S68500[44], 18}, {NULL}};
const Segment SGI48741[] = {{&S68500[44], 35}, {NULL}};
const Segment SGI48742[] = {{&S68500[30], 6}, {NULL}};
const Segment SGI48743[] = {{&S68500[127], 3}, {NULL}};
const Segment SGI48744[] = {{&S68500[93], 5}, {NULL}};
const Segment SGI48745[] = {{&S68501[44], 15}, {NULL}};
const Segment SGI48746[] = {{&S68501[44], 26}, {NULL}};
const Segment SGI48747[] = {{&S68501[30], 6}, {NULL}};
const Segment SGI48748[] = {{&S68501[118], 3}, {NULL}};
const Segment SGI48749[] = {{&S68501[84], 5}, {NULL}};
const Segment SGI48750[] = {{&S72295[31], 8}, {NULL}};
const Segment SGI48751[] = {{&S72295[54], 3}, {NULL}};
const Segment SGI48752[] = {{&S72298[39], 8}, {NULL}};
const Segment SGI48753[] = {{&S72298[103], 3}, {NULL}};
const Segment SGI48754[] = {{&S72298[92], 3}, {NULL}};
const Segment SGI48755[] = {{&S72298[69], 6}, {NULL}};
const Segment SGI48756[] = {{&S69605[44], 11}, {NULL}};
const Segment SGI48757[] = {{&S69605[44], 22}, {NULL}};
const Segment SGI48758[] = {{&S69605[30], 6}, {NULL}};
const Segment SGI48759[] = {{&S69605[114], 3}, {NULL}};
const Segment SGI48760[] = {{&S69605[80], 5}, {NULL}};
const Segment SGI48761[] = {{&S69606[38], 11}, {NULL}};
const Segment SGI48762[] = {{&S69606[31], 6}, {NULL}};
const Segment SGI48763[] = {{&S83927[6], 4}, {NULL}};
const Segment SGI48764[] = {{&S83927[87], 5}, {NULL}};
const Segment SGI48765[] = {{&S72406[44], 18}, {NULL}};
const Segment SGI48766[] = {{&S72406[44], 29}, {NULL}};
const Segment SGI48767[] = {{&S72406[30], 6}, {NULL}};
const Segment SGI48768[] = {{&S72406[121], 3}, {NULL}};
const Segment SGI48769[] = {{&S72406[87], 5}, {NULL}};
const Segment SGI48770[] = {{&S69712[44], 6}, {NULL}};
const Segment SGI48771[] = {{&S69712[44], 22}, {NULL}};
const Segment SGI48772[] = {{&S69712[30], 6}, {NULL}};
const Segment SGI48773[] = {{&S69712[114], 3}, {NULL}};
const Segment SGI48774[] = {{&S69712[80], 5}, {NULL}};
const Segment SGI48775[] = {{&S69713[44], 6}, {NULL}};
const Segment SGI48776[] = {{&S69713[44], 22}, {NULL}};
const Segment SGI48777[] = {{&S69713[30], 6}, {NULL}};
const Segment SGI48778[] = {{&S69713[114], 3}, {NULL}};
const Segment SGI48779[] = {{&S69713[80], 5}, {NULL}};
const Segment SGI48780[] = {{&S69714[44], 6}, {NULL}};
const Segment SGI48781[] = {{&S69714[44], 22}, {NULL}};
const Segment SGI48782[] = {{&S69714[30], 6}, {NULL}};
const Segment SGI48783[] = {{&S69714[114], 3}, {NULL}};
const Segment SGI48784[] = {{&S69714[80], 5}, {NULL}};
const Segment SGI48785[] = {{&S69715[44], 6}, {NULL}};
const Segment SGI48786[] = {{&S69715[44], 22}, {NULL}};
const Segment SGI48787[] = {{&S69715[30], 6}, {NULL}};
const Segment SGI48788[] = {{&S69715[60], 3}, {NULL}};
const Segment SGI48789[] = {{&S69715[80], 5}, {NULL}};
const Segment SGI48790[] = {{&S69716[38], 6}, {NULL}};
const Segment SGI48791[] = {{&S69716[31], 6}, {NULL}};
const Segment SGI48792[] = {{&S69717[44], 6}, {NULL}};
const Segment SGI48793[] = {{&S69717[44], 22}, {NULL}};
const Segment SGI48794[] = {{&S69717[30], 6}, {NULL}};
const Segment SGI48795[] = {{&S69717[114], 3}, {NULL}};
const Segment SGI48796[] = {{&S69717[80], 5}, {NULL}};
const Segment SGI48797[] = {{&S69718[44], 6}, {NULL}};
const Segment SGI48798[] = {{&S69718[44], 22}, {NULL}};
const Segment SGI48799[] = {{&S69718[30], 6}, {NULL}};
const Segment SGI48800[] = {{&S69718[114], 3}, {NULL}};
const Segment SGI48801[] = {{&S69718[80], 5}, {NULL}};
const Segment SGI48802[] = {{&S72458[6], 4}, {NULL}};
const Segment SGI48803[] = {{&S72458[81], 5}, {NULL}};
const Segment SGI48804[] = {{&S69788[44], 9}, {NULL}};
const Segment SGI48805[] = {{&S69788[44], 20}, {NULL}};
const Segment SGI48806[] = {{&S69788[30], 6}, {NULL}};
const Segment SGI48807[] = {{&S69788[112], 3}, {NULL}};
const Segment SGI48808[] = {{&S69788[78], 5}, {NULL}};
const Segment SGI48809[] = {{&S69789[44], 9}, {NULL}};
const Segment SGI48810[] = {{&S69789[44], 20}, {NULL}};
const Segment SGI48811[] = {{&S69789[30], 6}, {NULL}};
const Segment SGI48812[] = {{&S69789[112], 3}, {NULL}};
const Segment SGI48813[] = {{&S69789[78], 5}, {NULL}};
const Segment SGI48814[] = {{&S69790[44], 9}, {NULL}};
const Segment SGI48815[] = {{&S69790[44], 20}, {NULL}};
const Segment SGI48816[] = {{&S69790[30], 6}, {NULL}};
const Segment SGI48817[] = {{&S69790[112], 3}, {NULL}};
const Segment SGI48818[] = {{&S69790[78], 5}, {NULL}};
const Segment SGI48819[] = {{&S69791[44], 9}, {NULL}};
const Segment SGI48820[] = {{&S69791[44], 20}, {NULL}};
const Segment SGI48821[] = {{&S69791[30], 6}, {NULL}};
const Segment SGI48822[] = {{&S69791[112], 3}, {NULL}};
const Segment SGI48823[] = {{&S69791[78], 5}, {NULL}};
const Segment SGI48824[] = {{&S69792[38], 9}, {NULL}};
const Segment SGI48825[] = {{&S69792[31], 6}, {NULL}};
const Segment SGI48826[] = {{&S69793[44], 9}, {NULL}};
const Segment SGI48827[] = {{&S69793[44], 20}, {NULL}};
const Segment SGI48828[] = {{&S69793[30], 6}, {NULL}};
const Segment SGI48829[] = {{&S69793[112], 3}, {NULL}};
const Segment SGI48830[] = {{&S69793[78], 5}, {NULL}};
const Segment SGI48831[] = {{&S69794[44], 9}, {NULL}};
const Segment SGI48832[] = {{&S69794[44], 20}, {NULL}};
const Segment SGI48833[] = {{&S69794[30], 6}, {NULL}};
const Segment SGI48834[] = {{&S69794[112], 3}, {NULL}};
const Segment SGI48835[] = {{&S69794[78], 5}, {NULL}};
const Segment SGI48836[] = {{&S72561[44], 9}, {NULL}};
const Segment SGI48837[] = {{&S72561[44], 20}, {NULL}};
const Segment SGI48838[] = {{&S72561[30], 6}, {NULL}};
const Segment SGI48839[] = {{&S72561[112], 3}, {NULL}};
const Segment SGI48840[] = {{&S72561[78], 5}, {NULL}};
const Segment SGI48841[] = {{&S72592[59], 10}, {NULL}};
const Segment SGI48842[] = {{&S72592[59], 27}, {NULL}};
const Segment SGI48843[] = {{&S72592[45], 6}, {NULL}};
const Segment SGI48844[] = {{&S72592[134], 3}, {NULL}};
const Segment SGI48845[] = {{&S72592[100], 5}, {NULL}};
const Segment SGI48846[] = {{&S72592[0], 14}, {NULL}};
const Segment SGI48847[] = {{&S77075[41], 25}, {NULL}};
const Segment SGI48848[] = {{&S77075[30], 4}, {NULL}};
const Segment SGI48849[] = {{&S77075[114], 3}, {NULL}};
const Segment SGI48850[] = {{&S77075[80], 5}, {NULL}};
const Segment SGI48851[] = {{&S72638[44], 11}, {NULL}};
const Segment SGI48852[] = {{&S72638[44], 22}, {NULL}};
const Segment SGI48853[] = {{&S72638[30], 6}, {NULL}};
const Segment SGI48854[] = {{&S72638[80], 6}, {NULL}};
const Segment SGI48855[] = {{&S72638[117], 3}, {NULL}};
const Segment SGI48856[] = {{&S68892[45], 10}, {NULL}};
const Segment SGI48857[] = {{&S68892[30], 6}, {NULL}};
const Segment SGI48858[] = {{&S68892[120], 3}, {NULL}};
const Segment SGI48859[] = {{&S68892[86], 5}, {NULL}};
const Segment SGI48860[] = {{&S68892[144], 13}, {NULL}};
const Segment SGI48861[] = {{&S68909[44], 7}, {NULL}};
const Segment SGI48862[] = {{&S68909[44], 18}, {NULL}};
const Segment SGI48863[] = {{&S68909[30], 6}, {NULL}};
const Segment SGI48864[] = {{&S68909[110], 3}, {NULL}};
const Segment SGI48865[] = {{&S68909[76], 5}, {NULL}};
const Segment SGI48866[] = {{&S68910[44], 9}, {NULL}};
const Segment SGI48867[] = {{&S68910[44], 20}, {NULL}};
const Segment SGI48868[] = {{&S68910[30], 6}, {NULL}};
const Segment SGI48869[] = {{&S68910[112], 3}, {NULL}};
const Segment SGI48870[] = {{&S68910[78], 5}, {NULL}};
const Segment SGI48871[] = {{&S69074[44], 10}, {NULL}};
const Segment SGI48872[] = {{&S69074[44], 21}, {NULL}};
const Segment SGI48873[] = {{&S69074[30], 6}, {NULL}};
const Segment SGI48874[] = {{&S69074[113], 3}, {NULL}};
const Segment SGI48875[] = {{&S69074[79], 5}, {NULL}};
const Segment SGI48876[] = {{&S69075[44], 10}, {NULL}};
const Segment SGI48877[] = {{&S69075[44], 21}, {NULL}};
const Segment SGI48878[] = {{&S69075[30], 6}, {NULL}};
const Segment SGI48879[] = {{&S69075[113], 3}, {NULL}};
const Segment SGI48880[] = {{&S69075[79], 5}, {NULL}};
const Segment SGI48881[] = {{&S69076[44], 10}, {NULL}};
const Segment SGI48882[] = {{&S69076[44], 21}, {NULL}};
const Segment SGI48883[] = {{&S69076[30], 6}, {NULL}};
const Segment SGI48884[] = {{&S69076[113], 3}, {NULL}};
const Segment SGI48885[] = {{&S69076[79], 5}, {NULL}};
const Segment SGI48886[] = {{&S69077[44], 10}, {NULL}};
const Segment SGI48887[] = {{&S69077[44], 21}, {NULL}};
const Segment SGI48888[] = {{&S69077[30], 6}, {NULL}};
const Segment SGI48889[] = {{&S69077[113], 3}, {NULL}};
const Segment SGI48890[] = {{&S69077[79], 5}, {NULL}};
const Segment SGI48891[] = {{&S84379[44], 11}, {NULL}};
const Segment SGI48892[] = {{&S84379[44], 22}, {NULL}};
const Segment SGI48893[] = {{&S84379[30], 6}, {NULL}};
const Segment SGI48894[] = {{&S84379[114], 3}, {NULL}};
const Segment SGI48895[] = {{&S84379[80], 5}, {NULL}};
const Segment SGI48896[] = {{&S69100[44], 10}, {NULL}};
const Segment SGI48897[] = {{&S69100[44], 27}, {NULL}};
const Segment SGI48898[] = {{&S69100[30], 6}, {NULL}};
const Segment SGI48899[] = {{&S69100[119], 3}, {NULL}};
const Segment SGI48900[] = {{&S69100[85], 5}, {NULL}};
const Segment SGI48901[] = {{&S72883[44], 10}, {NULL}};
const Segment SGI48902[] = {{&S72883[44], 21}, {NULL}};
const Segment SGI48903[] = {{&S72883[30], 6}, {NULL}};
const Segment SGI48904[] = {{&S72883[113], 3}, {NULL}};
const Segment SGI48905[] = {{&S72883[79], 5}, {NULL}};
const Segment SGI48906[] = {{&S72884[44], 15}, {NULL}};
const Segment SGI48907[] = {{&S72884[44], 26}, {NULL}};
const Segment SGI48908[] = {{&S72884[30], 6}, {NULL}};
const Segment SGI48909[] = {{&S72884[118], 3}, {NULL}};
const Segment SGI48910[] = {{&S72884[84], 5}, {NULL}};
const Segment SGI48911[] = {{&S72890[44], 10}, {NULL}};
const Segment SGI48912[] = {{&S72890[44], 21}, {NULL}};
const Segment SGI48913[] = {{&S72890[30], 6}, {NULL}};
const Segment SGI48914[] = {{&S72890[113], 3}, {NULL}};
const Segment SGI48915[] = {{&S72890[79], 5}, {NULL}};
const Segment SGI48916[] = {{&S69993[44], 24}, {NULL}};
const Segment SGI48917[] = {{&S69993[30], 6}, {NULL}};
const Segment SGI48918[] = {{&S69993[134], 3}, {NULL}};
const Segment SGI48919[] = {{&S69993[100], 5}, {NULL}};
const Segment SGI48920[] = {{&S69995[44], 7}, {NULL}};
const Segment SGI48921[] = {{&S69995[44], 26}, {NULL}};
const Segment SGI48922[] = {{&S69995[30], 6}, {NULL}};
const Segment SGI48923[] = {{&S69995[118], 3}, {NULL}};
const Segment SGI48924[] = {{&S69995[84], 5}, {NULL}};
const Segment SGI48925[] = {{&S69996[44], 24}, {NULL}};
const Segment SGI48926[] = {{&S69996[30], 6}, {NULL}};
const Segment SGI48927[] = {{&S69996[134], 3}, {NULL}};
const Segment SGI48928[] = {{&S69996[100], 5}, {NULL}};
const Segment SGI48929[] = {{&S69997[44], 7}, {NULL}};
const Segment SGI48930[] = {{&S69997[44], 26}, {NULL}};
const Segment SGI48931[] = {{&S69997[30], 6}, {NULL}};
const Segment SGI48932[] = {{&S69997[87], 4}, {NULL}};
const Segment SGI48933[] = {{&S69997[87], 3}, {NULL}};
const Segment SGI48934[] = {{&S69998[44], 7}, {NULL}};
const Segment SGI48935[] = {{&S69998[44], 26}, {NULL}};
const Segment SGI48936[] = {{&S69998[30], 6}, {NULL}};
const Segment SGI48937[] = {{&S69998[87], 6}, {NULL}};
const Segment SGI48938[] = {{&S69130[44], 9}, {NULL}};
const Segment SGI48939[] = {{&S69130[44], 20}, {NULL}};
const Segment SGI48940[] = {{&S69130[30], 6}, {NULL}};
const Segment SGI48941[] = {{&S69130[112], 3}, {NULL}};
const Segment SGI48942[] = {{&S69130[78], 5}, {NULL}};
const Segment SGI48943[] = {{&S70038[44], 6}, {NULL}};
const Segment SGI48944[] = {{&S70038[44], 16}, {NULL}};
const Segment SGI48945[] = {{&S70038[30], 6}, {NULL}};
const Segment SGI48946[] = {{&S70038[108], 3}, {NULL}};
const Segment SGI48947[] = {{&S70038[74], 5}, {NULL}};
const Segment SGI48948[] = {{&S70041[44], 10}, {NULL}};
const Segment SGI48949[] = {{&S70041[44], 20}, {NULL}};
const Segment SGI48950[] = {{&S70041[30], 6}, {NULL}};
const Segment SGI48951[] = {{&S70041[112], 3}, {NULL}};
const Segment SGI48952[] = {{&S70041[78], 5}, {NULL}};
const Segment SGI48953[] = {{&S70068[57], 13}, {NULL}};
const Segment SGI48954[] = {{&S70068[57], 40}, {NULL}};
const Segment SGI48955[] = {{&S70068[43], 6}, {NULL}};
const Segment SGI48956[] = {{&S70068[145], 3}, {NULL}};
const Segment SGI48957[] = {{&S70068[111], 5}, {NULL}};
const Segment SGI48958[] = {{&S70069[57], 15}, {NULL}};
const Segment SGI48959[] = {{&S70069[57], 42}, {NULL}};
const Segment SGI48960[] = {{&S70069[43], 6}, {NULL}};
const Segment SGI48961[] = {{&S70069[147], 3}, {NULL}};
const Segment SGI48962[] = {{&S70069[113], 5}, {NULL}};
const Segment SGI48963[] = {{&S77377[42], 101}, {NULL}};
const Segment SGI48964[] = {{&S77377[42], 112}, {NULL}};
const Segment SGI48965[] = {{&S77377[30], 4}, {NULL}};
const Segment SGI48966[] = {{&S77377[202], 3}, {NULL}};
const Segment SGI48967[] = {{&S77377[168], 5}, {NULL}};
const Segment SGI48968[] = {{&S77378[42], 11}, {NULL}};
const Segment SGI48969[] = {{&S77378[42], 22}, {NULL}};
const Segment SGI48970[] = {{&S77378[30], 4}, {NULL}};
const Segment SGI48971[] = {{&S77378[112], 3}, {NULL}};
const Segment SGI48972[] = {{&S77378[78], 5}, {NULL}};
const Segment SGI48973[] = {{&S77379[42], 11}, {NULL}};
const Segment SGI48974[] = {{&S77379[42], 22}, {NULL}};
const Segment SGI48975[] = {{&S77379[30], 4}, {NULL}};
const Segment SGI48976[] = {{&S77379[81], 4}, {NULL}};
const Segment SGI48977[] = {{&S77379[81], 3}, {NULL}};
const Segment SGI48978[] = {{&S77380[42], 11}, {NULL}};
const Segment SGI48979[] = {{&S77380[42], 22}, {NULL}};
const Segment SGI48980[] = {{&S77380[30], 4}, {NULL}};
const Segment SGI48981[] = {{&S77380[81], 6}, {NULL}};
const Segment SGI48982[] = {{&S69263[44], 13}, {NULL}};
const Segment SGI48983[] = {{&S69263[44], 34}, {NULL}};
const Segment SGI48984[] = {{&S69263[30], 6}, {NULL}};
const Segment SGI48985[] = {{&S69263[126], 3}, {NULL}};
const Segment SGI48986[] = {{&S69263[92], 5}, {NULL}};
const Segment SGI48987[] = {{&S70194[45], 10}, {NULL}};
const Segment SGI48988[] = {{&S70194[30], 6}, {NULL}};
const Segment SGI48989[] = {{&S70194[120], 3}, {NULL}};
const Segment SGI48990[] = {{&S70194[86], 5}, {NULL}};
const Segment SGI48991[] = {{&S70194[144], 13}, {NULL}};
const Segment SGI48992[] = {{&S70201[30], 6}, {NULL}};
const Segment SGI48993[] = {{&S70201[92], 2}, {NULL}};
const Segment SGI48994[] = {{&S70201[57], 6}, {NULL}};
const Segment SGI48995[] = {{&S84597[44], 7}, {NULL}};
const Segment SGI48996[] = {{&S84597[44], 48}, {NULL}};
const Segment SGI48997[] = {{&S84597[30], 6}, {NULL}};
const Segment SGI48998[] = {{&S84597[140], 3}, {NULL}};
const Segment SGI48999[] = {{&S84597[106], 5}, {NULL}};
const Segment SGI49000[] = {{&S69365[44], 9}, {NULL}};
const Segment SGI49001[] = {{&S69365[44], 19}, {NULL}};
const Segment SGI49002[] = {{&S69365[30], 6}, {NULL}};
const Segment SGI49003[] = {{&S69365[111], 3}, {NULL}};
const Segment SGI49004[] = {{&S69365[77], 5}, {NULL}};
const Segment SGI49005[] = {{&S69413[44], 18}, {NULL}};
const Segment SGI49006[] = {{&S69413[44], 35}, {NULL}};
const Segment SGI49007[] = {{&S69413[30], 6}, {NULL}};
const Segment SGI49008[] = {{&S69413[127], 3}, {NULL}};
const Segment SGI49009[] = {{&S69413[93], 5}, {NULL}};
const Segment SGI49010[] = {{&S69414[44], 18}, {NULL}};
const Segment SGI49011[] = {{&S69414[44], 35}, {NULL}};
const Segment SGI49012[] = {{&S69414[30], 6}, {NULL}};
const Segment SGI49013[] = {{&S69414[96], 6}, {NULL}};
const Segment SGI49014[] = {{&S69415[44], 18}, {NULL}};
const Segment SGI49015[] = {{&S69415[44], 35}, {NULL}};
const Segment SGI49016[] = {{&S69415[30], 6}, {NULL}};
const Segment SGI49017[] = {{&S69415[96], 4}, {NULL}};
const Segment SGI49018[] = {{&S69415[96], 3}, {NULL}};
const Segment SGI49019[] = {{&S69416[38], 18}, {NULL}};
const Segment SGI49020[] = {{&S69416[31], 6}, {NULL}};
const Segment SGI49021[] = {{&S69417[44], 35}, {NULL}};
const Segment SGI49022[] = {{&S69417[30], 6}, {NULL}};
const Segment SGI49023[] = {{&S84695[44], 6}, {NULL}};
const Segment SGI49024[] = {{&S84695[44], 18}, {NULL}};
const Segment SGI49025[] = {{&S84695[30], 6}, {NULL}};
const Segment SGI49026[] = {{&S84695[110], 3}, {NULL}};
const Segment SGI49027[] = {{&S84695[76], 5}, {NULL}};
const Segment SGI49028[] = {{&S70333[44], 16}, {NULL}};
const Segment SGI49029[] = {{&S70333[44], 26}, {NULL}};
const Segment SGI49030[] = {{&S70333[30], 6}, {NULL}};
const Segment SGI49031[] = {{&S70333[118], 3}, {NULL}};
const Segment SGI49032[] = {{&S70333[84], 5}, {NULL}};
const Segment SGI49033[] = {{&S70334[44], 16}, {NULL}};
const Segment SGI49034[] = {{&S70334[44], 26}, {NULL}};
const Segment SGI49035[] = {{&S70334[30], 6}, {NULL}};
const Segment SGI49036[] = {{&S70334[118], 3}, {NULL}};
const Segment SGI49037[] = {{&S70334[84], 5}, {NULL}};
const Segment SGI49038[] = {{&S70335[44], 16}, {NULL}};
const Segment SGI49039[] = {{&S70335[44], 26}, {NULL}};
const Segment SGI49040[] = {{&S70335[30], 6}, {NULL}};
const Segment SGI49041[] = {{&S70335[118], 3}, {NULL}};
const Segment SGI49042[] = {{&S70335[84], 5}, {NULL}};
const Segment SGI49043[] = {{&S70336[44], 16}, {NULL}};
const Segment SGI49044[] = {{&S70336[44], 26}, {NULL}};
const Segment SGI49045[] = {{&S70336[30], 6}, {NULL}};
const Segment SGI49046[] = {{&S70336[118], 3}, {NULL}};
const Segment SGI49047[] = {{&S70336[84], 5}, {NULL}};
const Segment SGI49048[] = {{&S73270[44], 7}, {NULL}};
const Segment SGI49049[] = {{&S73270[44], 18}, {NULL}};
const Segment SGI49050[] = {{&S73270[30], 6}, {NULL}};
const Segment SGI49051[] = {{&S73270[110], 3}, {NULL}};
const Segment SGI49052[] = {{&S73270[76], 5}, {NULL}};
const Segment SGI49053[] = {{&S73271[38], 7}, {NULL}};
const Segment SGI49054[] = {{&S73271[31], 6}, {NULL}};
const Segment SGI49055[] = {{&S69541[44], 4}, {NULL}};
const Segment SGI49056[] = {{&S69541[44], 15}, {NULL}};
const Segment SGI49057[] = {{&S69541[30], 6}, {NULL}};
const Segment SGI49058[] = {{&S69541[107], 3}, {NULL}};
const Segment SGI49059[] = {{&S69541[73], 5}, {NULL}};
const Segment SGI49060[] = {{&S69542[44], 9}, {NULL}};
const Segment SGI49061[] = {{&S69542[44], 30}, {NULL}};
const Segment SGI49062[] = {{&S69542[30], 6}, {NULL}};
const Segment SGI49063[] = {{&S69542[122], 3}, {NULL}};
const Segment SGI49064[] = {{&S69542[88], 5}, {NULL}};
const Segment SGI49065[] = {{&S69583[44], 13}, {NULL}};
const Segment SGI49066[] = {{&S69583[44], 24}, {NULL}};
const Segment SGI49067[] = {{&S69583[30], 6}, {NULL}};
const Segment SGI49068[] = {{&S69583[116], 3}, {NULL}};
const Segment SGI49069[] = {{&S69583[82], 5}, {NULL}};
const Segment SGI49070[] = {{&S69584[44], 13}, {NULL}};
const Segment SGI49071[] = {{&S69584[44], 24}, {NULL}};
const Segment SGI49072[] = {{&S69584[30], 6}, {NULL}};
const Segment SGI49073[] = {{&S69584[116], 3}, {NULL}};
const Segment SGI49074[] = {{&S69584[82], 5}, {NULL}};
const Segment SGI49075[] = {{&S69585[44], 13}, {NULL}};
const Segment SGI49076[] = {{&S69585[44], 24}, {NULL}};
const Segment SGI49077[] = {{&S69585[30], 6}, {NULL}};
const Segment SGI49078[] = {{&S69585[116], 3}, {NULL}};
const Segment SGI49079[] = {{&S69585[82], 5}, {NULL}};
const Segment SGI49080[] = {{&S69586[44], 13}, {NULL}};
const Segment SGI49081[] = {{&S69586[44], 24}, {NULL}};
const Segment SGI49082[] = {{&S69586[30], 6}, {NULL}};
const Segment SGI49083[] = {{&S69586[116], 3}, {NULL}};
const Segment SGI49084[] = {{&S69586[82], 5}, {NULL}};
const Segment SGI49085[] = {{&S69587[44], 13}, {NULL}};
const Segment SGI49086[] = {{&S69587[44], 24}, {NULL}};
const Segment SGI49087[] = {{&S69587[30], 6}, {NULL}};
const Segment SGI49088[] = {{&S69587[116], 3}, {NULL}};
const Segment SGI49089[] = {{&S69587[82], 5}, {NULL}};
const Segment SGI49090[] = {{&S69588[44], 14}, {NULL}};
const Segment SGI49091[] = {{&S69588[44], 25}, {NULL}};
const Segment SGI49092[] = {{&S69588[30], 6}, {NULL}};
const Segment SGI49093[] = {{&S69588[117], 3}, {NULL}};
const Segment SGI49094[] = {{&S69588[83], 5}, {NULL}};
const Segment SGI49095[] = {{&S73556[44], 8}, {NULL}};
const Segment SGI49096[] = {{&S73556[44], 20}, {NULL}};
const Segment SGI49097[] = {{&S73556[30], 6}, {NULL}};
const Segment SGI49098[] = {{&S73556[112], 3}, {NULL}};
const Segment SGI49099[] = {{&S73556[78], 5}, {NULL}};
const Segment SGI49100[] = {{&S73676[44], 10}, {NULL}};
const Segment SGI49101[] = {{&S73676[44], 27}, {NULL}};
const Segment SGI49102[] = {{&S73676[30], 6}, {NULL}};
const Segment SGI49103[] = {{&S73676[118], 3}, {NULL}};
const Segment SGI49104[] = {{&S73676[85], 5}, {NULL}};
const Segment SGI49105[] = {{&S73676[122], 14}, {NULL}};
const Segment SGI49106[] = {{&S79193[44], 11}, {NULL}};
const Segment SGI49107[] = {{&S79193[44], 28}, {NULL}};
const Segment SGI49108[] = {{&S79193[30], 6}, {NULL}};
const Segment SGI49109[] = {{&S79193[120], 3}, {NULL}};
const Segment SGI49110[] = {{&S79193[86], 5}, {NULL}};
const Segment SGI49111[] = {{&S70794[31], 8}, {NULL}};
const Segment SGI49112[] = {{&S70794[54], 3}, {NULL}};
const Segment SGI49113[] = {{&S69807[44], 10}, {NULL}};
const Segment SGI49114[] = {{&S69807[44], 27}, {NULL}};
const Segment SGI49115[] = {{&S69807[30], 6}, {NULL}};
const Segment SGI49116[] = {{&S69807[119], 3}, {NULL}};
const Segment SGI49117[] = {{&S69807[85], 5}, {NULL}};
const Segment SGI49118[] = {{&S70807[44], 10}, {NULL}};
const Segment SGI49119[] = {{&S70807[44], 33}, {NULL}};
const Segment SGI49120[] = {{&S70807[30], 6}, {NULL}};
const Segment SGI49121[] = {{&S70807[125], 3}, {NULL}};
const Segment SGI49122[] = {{&S70807[91], 5}, {NULL}};
const Segment SGI49123[] = {{&S70808[44], 10}, {NULL}};
const Segment SGI49124[] = {{&S70808[44], 33}, {NULL}};
const Segment SGI49125[] = {{&S70808[30], 6}, {NULL}};
const Segment SGI49126[] = {{&S70808[125], 3}, {NULL}};
const Segment SGI49127[] = {{&S70808[91], 5}, {NULL}};
const Segment SGI49128[] = {{&S70809[44], 10}, {NULL}};
const Segment SGI49129[] = {{&S70809[44], 33}, {NULL}};
const Segment SGI49130[] = {{&S70809[30], 6}, {NULL}};
const Segment SGI49131[] = {{&S70809[125], 3}, {NULL}};
const Segment SGI49132[] = {{&S70809[91], 5}, {NULL}};
const Segment SGI49133[] = {{&S70810[44], 10}, {NULL}};
const Segment SGI49134[] = {{&S70810[44], 33}, {NULL}};
const Segment SGI49135[] = {{&S70810[30], 6}, {NULL}};
const Segment SGI49136[] = {{&S70810[125], 3}, {NULL}};
const Segment SGI49137[] = {{&S70810[91], 5}, {NULL}};
const Segment SGI49138[] = {{&S70811[44], 10}, {NULL}};
const Segment SGI49139[] = {{&S70811[44], 33}, {NULL}};
const Segment SGI49140[] = {{&S70811[30], 6}, {NULL}};
const Segment SGI49141[] = {{&S70811[125], 3}, {NULL}};
const Segment SGI49142[] = {{&S70811[91], 5}, {NULL}};
const Segment SGI49143[] = {{&S79233[44], 11}, {NULL}};
const Segment SGI49144[] = {{&S79233[44], 28}, {NULL}};
const Segment SGI49145[] = {{&S79233[30], 6}, {NULL}};
const Segment SGI49146[] = {{&S70812[44], 10}, {NULL}};
const Segment SGI49147[] = {{&S79233[120], 3}, {NULL}};
const Segment SGI49148[] = {{&S70812[44], 33}, {NULL}};
const Segment SGI49149[] = {{&S70812[30], 6}, {NULL}};
const Segment SGI49150[] = {{&S79233[86], 5}, {NULL}};
const Segment SGI49151[] = {{&S70812[125], 3}, {NULL}};
const Segment SGI49152[] = {{&S70812[91], 5}, {NULL}};
const Segment SGI49153[] = {{&S69857[44], 13}, {NULL}};
const Segment SGI49154[] = {{&S69857[44], 24}, {NULL}};
const Segment SGI49155[] = {{&S69857[30], 6}, {NULL}};
const Segment SGI49156[] = {{&S73750[44], 9}, {NULL}};
const Segment SGI49157[] = {{&S73750[44], 20}, {NULL}};
const Segment SGI49158[] = {{&S73750[30], 6}, {NULL}};
const Segment SGI49159[] = {{&S69857[117], 3}, {NULL}};
const Segment SGI49160[] = {{&S73750[112], 3}, {NULL}};
const Segment SGI49161[] = {{&S69857[82], 6}, {NULL}};
const Segment SGI49162[] = {{&S69857[135], 5}, {NULL}};
const Segment SGI49163[] = {{&S73750[78], 5}, {NULL}};
const Segment SGI49164[] = {{&S69858[44], 14}, {NULL}};
const Segment SGI49165[] = {{&S69858[44], 25}, {NULL}};
const Segment SGI49166[] = {{&S69858[30], 6}, {NULL}};
const Segment SGI49167[] = {{&S69858[117], 3}, {NULL}};
const Segment SGI49168[] = {{&S69858[83], 5}, {NULL}};
const Segment SGI49169[] = {{&S85281[59], 6}, {NULL}};
const Segment SGI49170[] = {{&S85281[59], 25}, {NULL}};
const Segment SGI49171[] = {{&S85281[45], 6}, {NULL}};
const Segment SGI49172[] = {{&S85281[132], 3}, {NULL}};
const Segment SGI49173[] = {{&S85281[98], 5}, {NULL}};
const Segment SGI49174[] = {{&S85281[0], 14}, {NULL}};
const Segment SGI49175[] = {{&S85282[59], 11}, {NULL}};
const Segment SGI49176[] = {{&S85282[59], 20}, {NULL}};
const Segment SGI49177[] = {{&S85282[45], 6}, {NULL}};
const Segment SGI49178[] = {{&S85282[127], 3}, {NULL}};
const Segment SGI49179[] = {{&S85282[93], 5}, {NULL}};
const Segment SGI49180[] = {{&S85282[0], 14}, {NULL}};
const Segment SGI49181[] = {{&S69900[44], 20}, {NULL}};
const Segment SGI49182[] = {{&S69900[44], 31}, {NULL}};
const Segment SGI49183[] = {{&S69900[30], 6}, {NULL}};
const Segment SGI49184[] = {{&S69900[123], 3}, {NULL}};
const Segment SGI49185[] = {{&S69900[89], 5}, {NULL}};
const Segment SGI49186[] = {{&S69901[44], 19}, {NULL}};
const Segment SGI49187[] = {{&S69901[44], 36}, {NULL}};
const Segment SGI49188[] = {{&S69901[30], 6}, {NULL}};
const Segment SGI49189[] = {{&S69901[128], 3}, {NULL}};
const Segment SGI49190[] = {{&S69901[94], 5}, {NULL}};
const Segment SGI49191[] = {{&S73806[45], 22}, {NULL}};
const Segment SGI49192[] = {{&S73806[30], 6}, {NULL}};
const Segment SGI49193[] = {{&S73806[126], 3}, {NULL}};
const Segment SGI49194[] = {{&S73806[92], 5}, {NULL}};
const Segment SGI49195[] = {{&S73806[160], 3}, {NULL}};
const Segment SGI49196[] = {{&S85336[44], 11}, {NULL}};
const Segment SGI49197[] = {{&S85336[44], 22}, {NULL}};
const Segment SGI49198[] = {{&S85336[30], 6}, {NULL}};
const Segment SGI49199[] = {{&S85336[114], 3}, {NULL}};
const Segment SGI49200[] = {{&S85336[80], 5}, {NULL}};
const Segment SGI49201[] = {{&S69931[44], 7}, {NULL}};
const Segment SGI49202[] = {{&S69931[44], 18}, {NULL}};
const Segment SGI49203[] = {{&S69931[30], 6}, {NULL}};
const Segment SGI49204[] = {{&S69931[110], 3}, {NULL}};
const Segment SGI49205[] = {{&S69931[76], 5}, {NULL}};
const Segment SGI49206[] = {{&S69973[44], 14}, {NULL}};
const Segment SGI49207[] = {{&S69973[44], 31}, {NULL}};
const Segment SGI49208[] = {{&S69973[30], 6}, {NULL}};
const Segment SGI49209[] = {{&S69973[123], 3}, {NULL}};
const Segment SGI49210[] = {{&S70970[44], 10}, {NULL}};
const Segment SGI49211[] = {{&S69973[89], 5}, {NULL}};
const Segment SGI49212[] = {{&S70970[44], 21}, {NULL}};
const Segment SGI49213[] = {{&S70970[30], 6}, {NULL}};
const Segment SGI49214[] = {{&S70970[113], 3}, {NULL}};
const Segment SGI49215[] = {{&S70970[79], 5}, {NULL}};
const Segment SGI49216[] = {{&S69974[44], 14}, {NULL}};
const Segment SGI49217[] = {{&S69974[44], 31}, {NULL}};
const Segment SGI49218[] = {{&S69974[30], 6}, {NULL}};
const Segment SGI49219[] = {{&S69974[92], 6}, {NULL}};
const Segment SGI49220[] = {{&S69975[44], 15}, {NULL}};
const Segment SGI49221[] = {{&S69975[44], 27}, {NULL}};
const Segment SGI49222[] = {{&S69975[30], 6}, {NULL}};
const Segment SGI49223[] = {{&S69975[119], 3}, {NULL}};
const Segment SGI49224[] = {{&S69975[85], 5}, {NULL}};
const Segment SGI49225[] = {{&S69976[44], 17}, {NULL}};
const Segment SGI49226[] = {{&S69976[30], 6}, {NULL}};
const Segment SGI49227[] = {{&S69976[109], 3}, {NULL}};
const Segment SGI49228[] = {{&S69976[75], 5}, {NULL}};
const Segment SGI49229[] = {{&S73880[44], 6}, {NULL}};
const Segment SGI49230[] = {{&S73880[44], 17}, {NULL}};
const Segment SGI49231[] = {{&S73880[30], 6}, {NULL}};
const Segment SGI49232[] = {{&S73880[109], 3}, {NULL}};
const Segment SGI49233[] = {{&S73880[75], 5}, {NULL}};
const Segment SGI49234[] = {{&S69977[44], 13}, {NULL}};
const Segment SGI49235[] = {{&S69977[44], 24}, {NULL}};
const Segment SGI49236[] = {{&S69977[30], 6}, {NULL}};
const Segment SGI49237[] = {{&S69977[85], 6}, {NULL}};
const Segment SGI49238[] = {{&S73881[38], 6}, {NULL}};
const Segment SGI49239[] = {{&S73881[31], 6}, {NULL}};
const Segment SGI49240[] = {{&S69978[44], 13}, {NULL}};
const Segment SGI49241[] = {{&S69978[44], 24}, {NULL}};
const Segment SGI49242[] = {{&S69978[30], 6}, {NULL}};
const Segment SGI49243[] = {{&S69978[116], 3}, {NULL}};
const Segment SGI49244[] = {{&S69978[82], 5}, {NULL}};
const Segment SGI49245[] = {{&S69979[44], 13}, {NULL}};
const Segment SGI49246[] = {{&S69979[44], 24}, {NULL}};
const Segment SGI49247[] = {{&S69979[30], 6}, {NULL}};
const Segment SGI49248[] = {{&S69979[85], 4}, {NULL}};
const Segment SGI49249[] = {{&S69979[85], 3}, {NULL}};
const Segment SGI49250[] = {{&S69980[38], 13}, {NULL}};
const Segment SGI49251[] = {{&S69980[31], 6}, {NULL}};
const Segment SGI49252[] = {{&S69981[44], 17}, {NULL}};
const Segment SGI49253[] = {{&S69981[44], 28}, {NULL}};
const Segment SGI49254[] = {{&S69981[30], 6}, {NULL}};
const Segment SGI49255[] = {{&S69981[120], 3}, {NULL}};
const Segment SGI49256[] = {{&S69981[86], 5}, {NULL}};
const Segment SGI49257[] = {{&S85432[44], 9}, {NULL}};
const Segment SGI49258[] = {{&S85432[44], 21}, {NULL}};
const Segment SGI49259[] = {{&S85432[30], 6}, {NULL}};
const Segment SGI49260[] = {{&S85432[113], 3}, {NULL}};
const Segment SGI49261[] = {{&S85432[79], 5}, {NULL}};
const Segment SGI49262[] = {{&S70036[44], 6}, {NULL}};
const Segment SGI49263[] = {{&S70036[44], 16}, {NULL}};
const Segment SGI49264[] = {{&S70036[30], 6}, {NULL}};
const Segment SGI49265[] = {{&S70036[108], 3}, {NULL}};
const Segment SGI49266[] = {{&S70036[74], 5}, {NULL}};
const Segment SGI49267[] = {{&S70121[44], 7}, {NULL}};
const Segment SGI49268[] = {{&S70121[44], 19}, {NULL}};
const Segment SGI49269[] = {{&S70121[30], 6}, {NULL}};
const Segment SGI49270[] = {{&S70121[111], 3}, {NULL}};
const Segment SGI49271[] = {{&S70121[77], 5}, {NULL}};
const Segment SGI49272[] = {{&S71139[44], 9}, {NULL}};
const Segment SGI49273[] = {{&S71139[44], 19}, {NULL}};
const Segment SGI49274[] = {{&S71139[30], 6}, {NULL}};
const Segment SGI49275[] = {{&S71139[111], 3}, {NULL}};
const Segment SGI49276[] = {{&S71139[77], 5}, {NULL}};
const Segment SGI49277[] = {{&S70195[45], 10}, {NULL}};
const Segment SGI49278[] = {{&S70195[30], 6}, {NULL}};
const Segment SGI49279[] = {{&S70195[120], 3}, {NULL}};
const Segment SGI49280[] = {{&S70195[86], 5}, {NULL}};
const Segment SGI49281[] = {{&S70195[144], 13}, {NULL}};
const Segment SGI49282[] = {{&S85675[44], 25}, {NULL}};
const Segment SGI49283[] = {{&S85675[30], 6}, {NULL}};
const Segment SGI49284[] = {{&S85675[117], 3}, {NULL}};
const Segment SGI49285[] = {{&S85675[83], 5}, {NULL}};
const Segment SGI49286[] = {{&S85676[44], 25}, {NULL}};
const Segment SGI49287[] = {{&S85676[30], 6}, {NULL}};
const Segment SGI49288[] = {{&S85676[117], 3}, {NULL}};
const Segment SGI49289[] = {{&S85676[83], 5}, {NULL}};
const Segment SGI49290[] = {{&S85677[44], 25}, {NULL}};
const Segment SGI49291[] = {{&S85677[30], 6}, {NULL}};
const Segment SGI49292[] = {{&S85677[117], 3}, {NULL}};
const Segment SGI49293[] = {{&S85677[83], 5}, {NULL}};
const Segment SGI49294[] = {{&S85678[44], 12}, {NULL}};
const Segment SGI49295[] = {{&S85678[44], 23}, {NULL}};
const Segment SGI49296[] = {{&S85678[30], 6}, {NULL}};
const Segment SGI49297[] = {{&S85678[84], 6}, {NULL}};
const Segment SGI49298[] = {{&S85681[45], 11}, {NULL}};
const Segment SGI49299[] = {{&S85681[30], 6}, {NULL}};
const Segment SGI49300[] = {{&S85681[115], 3}, {NULL}};
const Segment SGI49301[] = {{&S85681[81], 5}, {NULL}};
const Segment SGI49302[] = {{&S85681[139], 13}, {NULL}};
const Segment SGI49303[] = {{&S70303[44], 17}, {NULL}};
const Segment SGI49304[] = {{&S70303[44], 28}, {NULL}};
const Segment SGI49305[] = {{&S70303[30], 6}, {NULL}};
const Segment SGI49306[] = {{&S70303[120], 3}, {NULL}};
const Segment SGI49307[] = {{&S70303[86], 5}, {NULL}};
const Segment SGI49308[] = {{&S71341[44], 11}, {NULL}};
const Segment SGI49309[] = {{&S71341[44], 22}, {NULL}};
const Segment SGI49310[] = {{&S71341[30], 6}, {NULL}};
const Segment SGI49311[] = {{&S71341[114], 3}, {NULL}};
const Segment SGI49312[] = {{&S71341[80], 5}, {NULL}};
const Segment SGI49313[] = {{&S70304[44], 31}, {NULL}};
const Segment SGI49314[] = {{&S70304[30], 6}, {NULL}};
const Segment SGI49315[] = {{&S71343[44], 18}, {NULL}};
const Segment SGI49316[] = {{&S71343[44], 35}, {NULL}};
const Segment SGI49317[] = {{&S71343[30], 6}, {NULL}};
const Segment SGI49318[] = {{&S70305[44], 32}, {NULL}};
const Segment SGI49319[] = {{&S70305[30], 6}, {NULL}};
const Segment SGI49320[] = {{&S71343[127], 3}, {NULL}};
const Segment SGI49321[] = {{&S71343[93], 5}, {NULL}};
const Segment SGI49322[] = {{&S71344[44], 18}, {NULL}};
const Segment SGI49323[] = {{&S70306[44], 18}, {NULL}};
const Segment SGI49324[] = {{&S71344[44], 69}, {NULL}};
const Segment SGI49325[] = {{&S70306[44], 29}, {NULL}};
const Segment SGI49326[] = {{&S71344[30], 6}, {NULL}};
const Segment SGI49327[] = {{&S70306[30], 6}, {NULL}};
const Segment SGI49328[] = {{&S70306[121], 3}, {NULL}};
const Segment SGI49329[] = {{&S71344[161], 3}, {NULL}};
const Segment SGI49330[] = {{&S70306[87], 5}, {NULL}};
const Segment SGI49331[] = {{&S71344[127], 5}, {NULL}};
const Segment SGI49332[] = {{&S70343[44], 14}, {NULL}};
const Segment SGI49333[] = {{&S70343[44], 31}, {NULL}};
const Segment SGI49334[] = {{&S70343[30], 6}, {NULL}};
const Segment SGI49335[] = {{&S70343[123], 3}, {NULL}};
const Segment SGI49336[] = {{&S70343[89], 5}, {NULL}};
const Segment SGI49337[] = {{&S70390[44], 10}, {NULL}};
const Segment SGI49338[] = {{&S70390[44], 27}, {NULL}};
const Segment SGI49339[] = {{&S70390[30], 6}, {NULL}};
const Segment SGI49340[] = {{&S70390[85], 6}, {NULL}};
const Segment SGI49341[] = {{&S70390[112], 13}, {NULL}};
const Segment SGI49342[] = {{&S85914[44], 9}, {NULL}};
const Segment SGI49343[] = {{&S85914[44], 21}, {NULL}};
const Segment SGI49344[] = {{&S85914[30], 6}, {NULL}};
const Segment SGI49345[] = {{&S85914[113], 3}, {NULL}};
const Segment SGI49346[] = {{&S85914[79], 5}, {NULL}};
const Segment SGI49347[] = {{&S85916[44], 6}, {NULL}};
const Segment SGI49348[] = {{&S85916[44], 18}, {NULL}};
const Segment SGI49349[] = {{&S85916[30], 6}, {NULL}};
const Segment SGI49350[] = {{&S85916[110], 3}, {NULL}};
const Segment SGI49351[] = {{&S85916[76], 5}, {NULL}};
const Segment SGI49352[] = {{&S85917[44], 6}, {NULL}};
const Segment SGI49353[] = {{&S85917[44], 18}, {NULL}};
const Segment SGI49354[] = {{&S85917[30], 6}, {NULL}};
const Segment SGI49355[] = {{&S85917[79], 4}, {NULL}};
const Segment SGI49356[] = {{&S85917[79], 3}, {NULL}};
const Segment SGI49357[] = {{&S85918[44], 6}, {NULL}};
const Segment SGI49358[] = {{&S85918[44], 18}, {NULL}};
const Segment SGI49359[] = {{&S85918[30], 6}, {NULL}};
const Segment SGI49360[] = {{&S85918[79], 6}, {NULL}};
const Segment SGI49361[] = {{&S75216[6], 4}, {NULL}};
const Segment SGI49362[] = {{&S75216[84], 5}, {NULL}};
const Segment SGI49363[] = {{&S80143[44], 10}, {NULL}};
const Segment SGI49364[] = {{&S80143[44], 21}, {NULL}};
const Segment SGI49365[] = {{&S80143[30], 6}, {NULL}};
const Segment SGI49366[] = {{&S80143[113], 3}, {NULL}};
const Segment SGI49367[] = {{&S80143[79], 5}, {NULL}};
const Segment SGI49368[] = {{&S75404[44], 11}, {NULL}};
const Segment SGI49369[] = {{&S75404[44], 27}, {NULL}};
const Segment SGI49370[] = {{&S75404[30], 6}, {NULL}};
const Segment SGI49371[] = {{&S75404[119], 3}, {NULL}};
const Segment SGI49372[] = {{&S75404[85], 5}, {NULL}};
const Segment SGI49373[] = {{&S74575[44], 8}, {NULL}};
const Segment SGI49374[] = {{&S74575[44], 24}, {NULL}};
const Segment SGI49375[] = {{&S74575[30], 6}, {NULL}};
const Segment SGI49376[] = {{&S74575[58], 3}, {NULL}};
const Segment SGI49377[] = {{&S74575[82], 5}, {NULL}};
const Segment SGI49378[] = {{&S74580[47], 6}, {NULL}};
const Segment SGI49379[] = {{&S74580[182], 3}, {NULL}};
const Segment SGI49380[] = {{&S74580[148], 5}, {NULL}};
const Segment SGI49381[] = {{&S78989[42], 14}, {NULL}};
const Segment SGI49382[] = {{&S78989[42], 25}, {NULL}};
const Segment SGI49383[] = {{&S78989[30], 4}, {NULL}};
const Segment SGI49384[] = {{&S78989[115], 3}, {NULL}};
const Segment SGI49385[] = {{&S78989[81], 5}, {NULL}};
const Segment SGI49386[] = {{&S74610[44], 10}, {NULL}};
const Segment SGI49387[] = {{&S74610[44], 27}, {NULL}};
const Segment SGI49388[] = {{&S74610[30], 6}, {NULL}};
const Segment SGI49389[] = {{&S74610[85], 6}, {NULL}};
const Segment SGI49390[] = {{&S74610[122], 3}, {NULL}};
const Segment SGI49391[] = {{&S74616[44], 11}, {NULL}};
const Segment SGI49392[] = {{&S74616[44], 22}, {NULL}};
const Segment SGI49393[] = {{&S74616[30], 6}, {NULL}};
const Segment SGI49394[] = {{&S74616[80], 6}, {NULL}};
const Segment SGI49395[] = {{&S74616[117], 3}, {NULL}};
const Segment SGI49396[] = {{&S70797[31], 8}, {NULL}};
const Segment SGI49397[] = {{&S70797[54], 3}, {NULL}};
const Segment SGI49398[] = {{&S86219[39], 7}, {NULL}};
const Segment SGI49399[] = {{&S86219[102], 3}, {NULL}};
const Segment SGI49400[] = {{&S86219[91], 3}, {NULL}};
const Segment SGI49401[] = {{&S86219[68], 6}, {NULL}};
const Segment SGI49402[] = {{&S86221[31], 7}, {NULL}};
const Segment SGI49403[] = {{&S86221[53], 3}, {NULL}};
const Segment SGI49404[] = {{&S74728[44], 14}, {NULL}};
const Segment SGI49405[] = {{&S74728[44], 25}, {NULL}};
const Segment SGI49406[] = {{&S74728[30], 6}, {NULL}};
const Segment SGI49407[] = {{&S74728[117], 3}, {NULL}};
const Segment SGI49408[] = {{&S74728[83], 5}, {NULL}};
const Segment SGI49409[] = {{&S70883[44], 12}, {NULL}};
const Segment SGI49410[] = {{&S70883[44], 29}, {NULL}};
const Segment SGI49411[] = {{&S70883[30], 6}, {NULL}};
const Segment SGI49412[] = {{&S70883[121], 3}, {NULL}};
const Segment SGI49413[] = {{&S70883[87], 5}, {NULL}};
const Segment SGI49414[] = {{&S86329[44], 6}, {NULL}};
const Segment SGI49415[] = {{&S86329[44], 18}, {NULL}};
const Segment SGI49416[] = {{&S86329[30], 6}, {NULL}};
const Segment SGI49417[] = {{&S86329[79], 4}, {NULL}};
const Segment SGI49418[] = {{&S86329[79], 3}, {NULL}};
const Segment SGI49419[] = {{&S86330[44], 6}, {NULL}};
const Segment SGI49420[] = {{&S86330[44], 18}, {NULL}};
const Segment SGI49421[] = {{&S86330[30], 6}, {NULL}};
const Segment SGI49422[] = {{&S86330[79], 6}, {NULL}};
const Segment SGI49423[] = {{&S86331[44], 6}, {NULL}};
const Segment SGI49424[] = {{&S86331[44], 18}, {NULL}};
const Segment SGI49425[] = {{&S86331[30], 6}, {NULL}};
const Segment SGI49426[] = {{&S86331[110], 3}, {NULL}};
const Segment SGI49427[] = {{&S86331[76], 5}, {NULL}};
const Segment SGI49428[] = {{&S86332[44], 18}, {NULL}};
const Segment SGI49429[] = {{&S86332[30], 6}, {NULL}};
const Segment SGI49430[] = {{&S70924[44], 31}, {NULL}};
const Segment SGI49431[] = {{&S70924[30], 6}, {NULL}};
const Segment SGI49432[] = {{&S70925[44], 6}, {NULL}};
const Segment SGI49433[] = {{&S70925[44], 17}, {NULL}};
const Segment SGI49434[] = {{&S70925[30], 6}, {NULL}};
const Segment SGI49435[] = {{&S70925[78], 4}, {NULL}};
const Segment SGI49436[] = {{&S70925[78], 3}, {NULL}};
const Segment SGI49437[] = {{&S70926[44], 6}, {NULL}};
const Segment SGI49438[] = {{&S70926[44], 17}, {NULL}};
const Segment SGI49439[] = {{&S70926[30], 6}, {NULL}};
const Segment SGI49440[] = {{&S70926[109], 3}, {NULL}};
const Segment SGI49441[] = {{&S70926[75], 5}, {NULL}};
const Segment SGI49442[] = {{&S74813[44], 22}, {NULL}};
const Segment SGI49443[] = {{&S74813[44], 33}, {NULL}};
const Segment SGI49444[] = {{&S70927[44], 33}, {NULL}};
const Segment SGI49445[] = {{&S74813[30], 6}, {NULL}};
const Segment SGI49446[] = {{&S70927[30], 6}, {NULL}};
const Segment SGI49447[] = {{&S74813[91], 6}, {NULL}};
const Segment SGI49448[] = {{&S74813[128], 3}, {NULL}};
const Segment SGI49449[] = {{&S70928[44], 6}, {NULL}};
const Segment SGI49450[] = {{&S70928[44], 17}, {NULL}};
const Segment SGI49451[] = {{&S70928[30], 6}, {NULL}};
const Segment SGI49452[] = {{&S70928[78], 6}, {NULL}};
const Segment SGI49453[] = {{&S70929[44], 17}, {NULL}};
const Segment SGI49454[] = {{&S70929[30], 6}, {NULL}};
const Segment SGI49455[] = {{&S70930[44], 33}, {NULL}};
const Segment SGI49456[] = {{&S70930[30], 6}, {NULL}};
const Segment SGI49457[] = {{&S71569[44], 7}, {NULL}};
const Segment SGI49458[] = {{&S71569[44], 24}, {NULL}};
const Segment SGI49459[] = {{&S71569[30], 6}, {NULL}};
const Segment SGI49460[] = {{&S71569[116], 3}, {NULL}};
const Segment SGI49461[] = {{&S71569[82], 5}, {NULL}};
const Segment SGI49462[] = {{&S70986[44], 10}, {NULL}};
const Segment SGI49463[] = {{&S70986[44], 21}, {NULL}};
const Segment SGI49464[] = {{&S70986[30], 6}, {NULL}};
const Segment SGI49465[] = {{&S70986[113], 3}, {NULL}};
const Segment SGI49466[] = {{&S70986[79], 5}, {NULL}};
const Segment SGI49467[] = {{&S74895[44], 11}, {NULL}};
const Segment SGI49468[] = {{&S74895[44], 22}, {NULL}};
const Segment SGI49469[] = {{&S74895[30], 6}, {NULL}};
const Segment SGI49470[] = {{&S74895[80], 6}, {NULL}};
const Segment SGI49471[] = {{&S74895[117], 3}, {NULL}};
const Segment SGI49472[] = {{&S71029[44], 10}, {NULL}};
const Segment SGI49473[] = {{&S71029[44], 27}, {NULL}};
const Segment SGI49474[] = {{&S71029[30], 6}, {NULL}};
const Segment SGI49475[] = {{&S71029[119], 3}, {NULL}};
const Segment SGI49476[] = {{&S71029[85], 5}, {NULL}};
const Segment SGI49477[] = {{&S79196[42], 14}, {NULL}};
const Segment SGI49478[] = {{&S79196[42], 25}, {NULL}};
const Segment SGI49479[] = {{&S79196[30], 4}, {NULL}};
const Segment SGI49480[] = {{&S79196[115], 3}, {NULL}};
const Segment SGI49481[] = {{&S79196[81], 5}, {NULL}};
const Segment SGI49482[] = {{&S71032[44], 10}, {NULL}};
const Segment SGI49483[] = {{&S71032[44], 27}, {NULL}};
const Segment SGI49484[] = {{&S71032[30], 6}, {NULL}};
const Segment SGI49485[] = {{&S71032[88], 4}, {NULL}};
const Segment SGI49486[] = {{&S71032[88], 3}, {NULL}};
const Segment SGI49487[] = {{&S71035[44], 10}, {NULL}};
const Segment SGI49488[] = {{&S71035[44], 27}, {NULL}};
const Segment SGI49489[] = {{&S71035[30], 6}, {NULL}};
const Segment SGI49490[] = {{&S71035[88], 6}, {NULL}};
const Segment SGI49491[] = {{&S71644[44], 9}, {NULL}};
const Segment SGI49492[] = {{&S71644[44], 26}, {NULL}};
const Segment SGI49493[] = {{&S71644[30], 6}, {NULL}};
const Segment SGI49494[] = {{&S71644[118], 3}, {NULL}};
const Segment SGI49495[] = {{&S71644[84], 5}, {NULL}};
const Segment SGI49496[] = {{&S71098[44], 7}, {NULL}};
const Segment SGI49497[] = {{&S71098[44], 18}, {NULL}};
const Segment SGI49498[] = {{&S71098[30], 6}, {NULL}};
const Segment SGI49499[] = {{&S71098[110], 3}, {NULL}};
const Segment SGI49500[] = {{&S71098[76], 5}, {NULL}};
const Segment SGI49501[] = {{&S71099[38], 7}, {NULL}};
const Segment SGI49502[] = {{&S71099[31], 6}, {NULL}};
const Segment SGI49503[] = {{&S71103[44], 9}, {NULL}};
const Segment SGI49504[] = {{&S71103[44], 20}, {NULL}};
const Segment SGI49505[] = {{&S71103[30], 6}, {NULL}};
const Segment SGI49506[] = {{&S71103[112], 3}, {NULL}};
const Segment SGI49507[] = {{&S71103[78], 5}, {NULL}};
const Segment SGI49508[] = {{&S71127[44], 6}, {NULL}};
const Segment SGI49509[] = {{&S71127[44], 17}, {NULL}};
const Segment SGI49510[] = {{&S71127[30], 6}, {NULL}};
const Segment SGI49511[] = {{&S71127[109], 3}, {NULL}};
const Segment SGI49512[] = {{&S71127[75], 5}, {NULL}};
const Segment SGI49513[] = {{&S71140[44], 9}, {NULL}};
const Segment SGI49514[] = {{&S71140[44], 19}, {NULL}};
const Segment SGI49515[] = {{&S71140[30], 6}, {NULL}};
const Segment SGI49516[] = {{&S71140[111], 3}, {NULL}};
const Segment SGI49517[] = {{&S71140[77], 5}, {NULL}};
const Segment SGI49518[] = {{&S71146[44], 12}, {NULL}};
const Segment SGI49519[] = {{&S71146[44], 23}, {NULL}};
const Segment SGI49520[] = {{&S71146[30], 6}, {NULL}};
const Segment SGI49521[] = {{&S71146[115], 3}, {NULL}};
const Segment SGI49522[] = {{&S71146[81], 5}, {NULL}};
const Segment SGI49523[] = {{&S71233[44], 7}, {NULL}};
const Segment SGI49524[] = {{&S71233[44], 24}, {NULL}};
const Segment SGI49525[] = {{&S71233[30], 6}, {NULL}};
const Segment SGI49526[] = {{&S71233[116], 3}, {NULL}};
const Segment SGI49527[] = {{&S71233[82], 5}, {NULL}};
const Segment SGI49528[] = {{&S75195[45], 10}, {NULL}};
const Segment SGI49529[] = {{&S75195[30], 6}, {NULL}};
const Segment SGI49530[] = {{&S75195[120], 3}, {NULL}};
const Segment SGI49531[] = {{&S75195[86], 5}, {NULL}};
const Segment SGI49532[] = {{&S75195[154], 3}, {NULL}};
const Segment SGI49533[] = {{&S75215[6], 4}, {NULL}};
const Segment SGI49534[] = {{&S75215[84], 5}, {NULL}};
const Segment SGI49535[] = {{&S71324[44], 5}, {NULL}};
const Segment SGI49536[] = {{&S71324[44], 16}, {NULL}};
const Segment SGI49537[] = {{&S71324[30], 6}, {NULL}};
const Segment SGI49538[] = {{&S71324[108], 3}, {NULL}};
const Segment SGI49539[] = {{&S71324[74], 5}, {NULL}};
const Segment SGI49540[] = {{&S86884[44], 10}, {NULL}};
const Segment SGI49541[] = {{&S86884[44], 21}, {NULL}};
const Segment SGI49542[] = {{&S86884[30], 6}, {NULL}};
const Segment SGI49543[] = {{&S86884[113], 3}, {NULL}};
const Segment SGI49544[] = {{&S86884[79], 5}, {NULL}};
const Segment SGI49545[] = {{&S86887[76], 10}, {NULL}};
const Segment SGI49546[] = {{&S86887[76], 21}, {NULL}};
const Segment SGI49547[] = {{&S86887[30], 38}, {NULL}};
const Segment SGI49548[] = {{&S86887[145], 3}, {NULL}};
const Segment SGI49549[] = {{&S86887[111], 5}, {NULL}};
const Segment SGI49550[] = {{&S86888[69], 10}, {NULL}};
const Segment SGI49551[] = {{&S86888[69], 21}, {NULL}};
const Segment SGI49552[] = {{&S86888[30], 31}, {NULL}};
const Segment SGI49553[] = {{&S86888[138], 3}, {NULL}};
const Segment SGI49554[] = {{&S86888[104], 5}, {NULL}};
const Segment SGI49555[] = {{&S75367[44], 10}, {NULL}};
const Segment SGI49556[] = {{&S75367[44], 27}, {NULL}};
const Segment SGI49557[] = {{&S75367[30], 6}, {NULL}};
const Segment SGI49558[] = {{&S75367[85], 6}, {NULL}};
const Segment SGI49559[] = {{&S71463[44], 10}, {NULL}};
const Segment SGI49560[] = {{&S71463[44], 31}, {NULL}};
const Segment SGI49561[] = {{&S75367[122], 3}, {NULL}};
const Segment SGI49562[] = {{&S71463[30], 6}, {NULL}};
const Segment SGI49563[] = {{&S71463[123], 3}, {NULL}};
const Segment SGI49564[] = {{&S71463[89], 5}, {NULL}};
const Segment SGI49565[] = {{&S75402[44], 11}, {NULL}};
const Segment SGI49566[] = {{&S75402[44], 27}, {NULL}};
const Segment SGI49567[] = {{&S75402[30], 6}, {NULL}};
const Segment SGI49568[] = {{&S75402[119], 3}, {NULL}};
const Segment SGI49569[] = {{&S75402[85], 5}, {NULL}};
const Segment SGI49570[] = {{&S71487[44], 6}, {NULL}};
const Segment SGI49571[] = {{&S79638[46], 4}, {NULL}};
const Segment SGI49572[] = {{&S71487[44], 17}, {NULL}};
const Segment SGI49573[] = {{&S71487[30], 6}, {NULL}};
const Segment SGI49574[] = {{&S71487[109], 3}, {NULL}};
const Segment SGI49575[] = {{&S79638[99], 5}, {NULL}};
const Segment SGI49576[] = {{&S71487[75], 5}, {NULL}};
const Segment SGI49577[] = {{&S71488[38], 6}, {NULL}};
const Segment SGI49578[] = {{&S71488[31], 6}, {NULL}};
const Segment SGI49579[] = {{&S86976[44], 9}, {NULL}};
const Segment SGI49580[] = {{&S86976[44], 20}, {NULL}};
const Segment SGI49581[] = {{&S86976[30], 6}, {NULL}};
const Segment SGI49582[] = {{&S86976[78], 6}, {NULL}};
const Segment SGI49583[] = {{&S86976[105], 13}, {NULL}};
const Segment SGI49584[] = {{&S71499[34], 14}, {NULL}};
const Segment SGI49585[] = {{&S71499[27], 6}, {NULL}};
const Segment SGI49586[] = {{&S71499[107], 2}, {NULL}};
const Segment SGI49587[] = {{&S71499[73], 6}, {NULL}};
const Segment SGI49588[] = {{&S71499[141], 8}, {NULL}};
const Segment SGI49589[] = {{&S87033[44], 10}, {NULL}};
const Segment SGI49590[] = {{&S87033[44], 21}, {NULL}};
const Segment SGI49591[] = {{&S87033[30], 6}, {NULL}};
const Segment SGI49592[] = {{&S87033[113], 3}, {NULL}};
const Segment SGI49593[] = {{&S87033[79], 5}, {NULL}};
const Segment SGI49594[] = {{&S87034[44], 10}, {NULL}};
const Segment SGI49595[] = {{&S87034[44], 21}, {NULL}};
const Segment SGI49596[] = {{&S87034[30], 6}, {NULL}};
const Segment SGI49597[] = {{&S87034[82], 4}, {NULL}};
const Segment SGI49598[] = {{&S87034[82], 3}, {NULL}};
const Segment SGI49599[] = {{&S71572[44], 7}, {NULL}};
const Segment SGI49600[] = {{&S71572[44], 24}, {NULL}};
const Segment SGI49601[] = {{&S71572[30], 6}, {NULL}};
const Segment SGI49602[] = {{&S71572[116], 3}, {NULL}};
const Segment SGI49603[] = {{&S71572[82], 5}, {NULL}};
const Segment SGI49604[] = {{&S72032[31], 8}, {NULL}};
const Segment SGI49605[] = {{&S72032[54], 3}, {NULL}};
const Segment SGI49606[] = {{&S87064[44], 13}, {NULL}};
const Segment SGI49607[] = {{&S87064[44], 25}, {NULL}};
const Segment SGI49608[] = {{&S87064[30], 6}, {NULL}};
const Segment SGI49609[] = {{&S87064[86], 4}, {NULL}};
const Segment SGI49610[] = {{&S87064[86], 3}, {NULL}};
const Segment SGI49611[] = {{&S71580[44], 8}, {NULL}};
const Segment SGI49612[] = {{&S71580[44], 19}, {NULL}};
const Segment SGI49613[] = {{&S71580[30], 6}, {NULL}};
const Segment SGI49614[] = {{&S71580[111], 3}, {NULL}};
const Segment SGI49615[] = {{&S71580[77], 5}, {NULL}};
const Segment SGI49616[] = {{&S79734[42], 15}, {NULL}};
const Segment SGI49617[] = {{&S79734[42], 26}, {NULL}};
const Segment SGI49618[] = {{&S79734[30], 4}, {NULL}};
const Segment SGI49619[] = {{&S79734[116], 3}, {NULL}};
const Segment SGI49620[] = {{&S79734[82], 5}, {NULL}};
const Segment SGI49621[] = {{&S71633[44], 10}, {NULL}};
const Segment SGI49622[] = {{&S71633[44], 27}, {NULL}};
const Segment SGI49623[] = {{&S71633[30], 6}, {NULL}};
const Segment SGI49624[] = {{&S71633[85], 6}, {NULL}};
const Segment SGI49625[] = {{&S71633[112], 13}, {NULL}};
const Segment SGI49626[] = {{&S71658[45], 10}, {NULL}};
const Segment SGI49627[] = {{&S71658[30], 6}, {NULL}};
const Segment SGI49628[] = {{&S71658[120], 3}, {NULL}};
const Segment SGI49629[] = {{&S71658[86], 5}, {NULL}};
const Segment SGI49630[] = {{&S71658[144], 13}, {NULL}};
const Segment SGI49631[] = {{&S72148[44], 9}, {NULL}};
const Segment SGI49632[] = {{&S72148[44], 20}, {NULL}};
const Segment SGI49633[] = {{&S72148[30], 6}, {NULL}};
const Segment SGI49634[] = {{&S72148[112], 3}, {NULL}};
const Segment SGI49635[] = {{&S72148[78], 5}, {NULL}};
const Segment SGI49636[] = {{&S72149[38], 9}, {NULL}};
const Segment SGI49637[] = {{&S72149[31], 6}, {NULL}};
const Segment SGI49638[] = {{&S72173[44], 11}, {NULL}};
const Segment SGI49639[] = {{&S72173[44], 22}, {NULL}};
const Segment SGI49640[] = {{&S72173[30], 6}, {NULL}};
const Segment SGI49641[] = {{&S72173[114], 3}, {NULL}};
const Segment SGI49642[] = {{&S72173[80], 5}, {NULL}};
const Segment SGI49643[] = {{&S72174[38], 11}, {NULL}};
const Segment SGI49644[] = {{&S72174[31], 6}, {NULL}};
const Segment SGI49645[] = {{&S75739[45], 10}, {NULL}};
const Segment SGI49646[] = {{&S75739[30], 6}, {NULL}};
const Segment SGI49647[] = {{&S75739[120], 3}, {NULL}};
const Segment SGI49648[] = {{&S75739[86], 5}, {NULL}};
const Segment SGI49649[] = {{&S75739[154], 3}, {NULL}};
const Segment SGI49650[] = {{&S87288[44], 9}, {NULL}};
const Segment SGI49651[] = {{&S87288[44], 20}, {NULL}};
const Segment SGI49652[] = {{&S87288[30], 6}, {NULL}};
const Segment SGI49653[] = {{&S87288[112], 3}, {NULL}};
const Segment SGI49654[] = {{&S87288[78], 5}, {NULL}};
const Segment SGI49655[] = {{&S76438[44], 12}, {NULL}};
const Segment SGI49656[] = {{&S76438[44], 23}, {NULL}};
const Segment SGI49657[] = {{&S76438[30], 6}, {NULL}};
const Segment SGI49658[] = {{&S76438[115], 3}, {NULL}};
const Segment SGI49659[] = {{&S76438[81], 5}, {NULL}};
const Segment SGI49660[] = {{&S87289[44], 9}, {NULL}};
const Segment SGI49661[] = {{&S87289[44], 21}, {NULL}};
const Segment SGI49662[] = {{&S87289[30], 6}, {NULL}};
const Segment SGI49663[] = {{&S87289[113], 3}, {NULL}};
const Segment SGI49664[] = {{&S87289[79], 5}, {NULL}};
const Segment SGI49665[] = {{&S71747[44], 14}, {NULL}};
const Segment SGI49666[] = {{&S71747[44], 25}, {NULL}};
const Segment SGI49667[] = {{&S71747[30], 6}, {NULL}};
const Segment SGI49668[] = {{&S71747[117], 3}, {NULL}};
const Segment SGI49669[] = {{&S71747[83], 5}, {NULL}};
const Segment SGI49670[] = {{&S75836[44], 10}, {NULL}};
const Segment SGI49671[] = {{&S71776[44], 20}, {NULL}};
const Segment SGI49672[] = {{&S75836[44], 27}, {NULL}};
const Segment SGI49673[] = {{&S71776[44], 31}, {NULL}};
const Segment SGI49674[] = {{&S71776[30], 6}, {NULL}};
const Segment SGI49675[] = {{&S75836[30], 6}, {NULL}};
const Segment SGI49676[] = {{&S75836[149], 4}, {NULL}};
const Segment SGI49677[] = {{&S75836[175], 3}, {NULL}};
const Segment SGI49678[] = {{&S71776[89], 6}, {NULL}};
const Segment SGI49679[] = {{&S75836[119], 3}, {&S75836[224], 5}, {NULL}};
const Segment SGI49680[] = {{&S71776[116], 13}, {NULL}};
const Segment SGI49681[] = {{&S75836[85], 5}, {NULL}};
const Segment SGI49682[] = {{&S81249[44], 9}, {NULL}};
const Segment SGI49683[] = {{&S81249[44], 26}, {NULL}};
const Segment SGI49684[] = {{&S81249[30], 6}, {NULL}};
const Segment SGI49685[] = {{&S81249[118], 3}, {NULL}};
const Segment SGI49686[] = {{&S81249[84], 5}, {NULL}};
const Segment SGI49687[] = {{&S81250[44], 12}, {NULL}};
const Segment SGI49688[] = {{&S81250[44], 29}, {NULL}};
const Segment SGI49689[] = {{&S81250[30], 6}, {NULL}};
const Segment SGI49690[] = {{&S81250[121], 3}, {NULL}};
const Segment SGI49691[] = {{&S81250[87], 5}, {NULL}};
const Segment SGI49692[] = {{&S72297[31], 9}, {NULL}};
const Segment SGI49693[] = {{&S72297[55], 3}, {NULL}};
const Segment SGI49694[] = {{&S87460[45], 9}, {NULL}};
const Segment SGI49695[] = {{&S87460[30], 6}, {NULL}};
const Segment SGI49696[] = {{&S87460[113], 3}, {NULL}};
const Segment SGI49697[] = {{&S87460[79], 5}, {NULL}};
const Segment SGI49698[] = {{&S87460[137], 13}, {NULL}};
const Segment SGI49699[] = {{&S87461[44], 9}, {NULL}};
const Segment SGI49700[] = {{&S87461[44], 20}, {NULL}};
const Segment SGI49701[] = {{&S87461[30], 6}, {NULL}};
const Segment SGI49702[] = {{&S87461[113], 3}, {NULL}};
const Segment SGI49703[] = {{&S87461[78], 6}, {NULL}};
const Segment SGI49704[] = {{&S87461[133], 8}, {NULL}};
const Segment SGI49705[] = {{&S87461[142], 13}, {NULL}};
const Segment SGI49706[] = {{&S72344[44], 10}, {NULL}};
const Segment SGI49707[] = {{&S72344[44], 27}, {NULL}};
const Segment SGI49708[] = {{&S72344[30], 6}, {NULL}};
const Segment SGI49709[] = {{&S72344[119], 3}, {NULL}};
const Segment SGI49710[] = {{&S72344[85], 5}, {NULL}};
const Segment SGI49711[] = {{&S80064[42], 12}, {NULL}};
const Segment SGI49712[] = {{&S80064[42], 23}, {NULL}};
const Segment SGI49713[] = {{&S80064[30], 4}, {NULL}};
const Segment SGI49714[] = {{&S80064[113], 3}, {NULL}};
const Segment SGI49715[] = {{&S80064[79], 5}, {NULL}};
const Segment SGI49716[] = {{&S76022[34], 10}, {NULL}};
const Segment SGI49717[] = {{&S76022[27], 6}, {NULL}};
const Segment SGI49718[] = {{&S76022[109], 2}, {NULL}};
const Segment SGI49719[] = {{&S76022[75], 6}, {NULL}};
const Segment SGI49720[] = {{&S76022[143], 8}, {NULL}};
const Segment SGI49721[] = {{&S87524[44], 14}, {NULL}};
const Segment SGI49722[] = {{&S87524[44], 26}, {NULL}};
const Segment SGI49723[] = {{&S87524[30], 6}, {NULL}};
const Segment SGI49724[] = {{&S87524[118], 3}, {NULL}};
const Segment SGI49725[] = {{&S87524[84], 5}, {NULL}};
const Segment SGI49726[] = {{&S72407[44], 18}, {NULL}};
const Segment SGI49727[] = {{&S72407[44], 29}, {NULL}};
const Segment SGI49728[] = {{&S72407[30], 6}, {NULL}};
const Segment SGI49729[] = {{&S72407[121], 3}, {NULL}};
const Segment SGI49730[] = {{&S72407[87], 5}, {NULL}};
const Segment SGI49731[] = {{&S87592[45], 12}, {NULL}};
const Segment SGI49732[] = {{&S87592[30], 6}, {NULL}};
const Segment SGI49733[] = {{&S87592[117], 3}, {NULL}};
const Segment SGI49734[] = {{&S87592[83], 5}, {NULL}};
const Segment SGI49735[] = {{&S87592[141], 13}, {NULL}};
const Segment SGI49736[] = {{&S72455[6], 4}, {NULL}};
const Segment SGI49737[] = {{&S72455[81], 5}, {NULL}};
const Segment SGI49738[] = {{&S72479[44], 8}, {NULL}};
const Segment SGI49739[] = {{&S72479[44], 19}, {NULL}};
const Segment SGI49740[] = {{&S72479[30], 6}, {NULL}};
const Segment SGI49741[] = {{&S72479[111], 3}, {NULL}};
const Segment SGI49742[] = {{&S72479[77], 5}, {NULL}};
const Segment SGI49743[] = {{&S72141[34], 10}, {NULL}};
const Segment SGI49744[] = {{&S72141[27], 6}, {NULL}};
const Segment SGI49745[] = {{&S72141[109], 2}, {NULL}};
const Segment SGI49746[] = {{&S72141[75], 6}, {NULL}};
const Segment SGI49747[] = {{&S72141[143], 8}, {NULL}};
const Segment SGI49748[] = {{&S72564[44], 9}, {NULL}};
const Segment SGI49749[] = {{&S72564[44], 20}, {NULL}};
const Segment SGI49750[] = {{&S72564[30], 6}, {NULL}};
const Segment SGI49751[] = {{&S72564[112], 3}, {NULL}};
const Segment SGI49752[] = {{&S72564[78], 5}, {NULL}};
const Segment SGI49753[] = {{&S72577[44], 11}, {NULL}};
const Segment SGI49754[] = {{&S72577[44], 22}, {NULL}};
const Segment SGI49755[] = {{&S72577[30], 6}, {NULL}};
const Segment SGI49756[] = {{&S72577[114], 3}, {NULL}};
const Segment SGI49757[] = {{&S72577[80], 5}, {NULL}};
const Segment SGI49758[] = {{&S72178[44], 6}, {NULL}};
const Segment SGI49759[] = {{&S72178[44], 17}, {NULL}};
const Segment SGI49760[] = {{&S72178[30], 6}, {NULL}};
const Segment SGI49761[] = {{&S72178[109], 3}, {NULL}};
const Segment SGI49762[] = {{&S72578[44], 11}, {NULL}};
const Segment SGI49763[] = {{&S72578[44], 22}, {NULL}};
const Segment SGI49764[] = {{&S72178[75], 5}, {NULL}};
const Segment SGI49765[] = {{&S72578[30], 6}, {NULL}};
const Segment SGI49766[] = {{&S72578[83], 4}, {NULL}};
const Segment SGI49767[] = {{&S72578[83], 3}, {NULL}};
const Segment SGI49768[] = {{&S72179[44], 14}, {NULL}};
const Segment SGI49769[] = {{&S72179[44], 25}, {NULL}};
const Segment SGI49770[] = {{&S72179[30], 6}, {NULL}};
const Segment SGI49771[] = {{&S72579[44], 11}, {NULL}};
const Segment SGI49772[] = {{&S72579[44], 22}, {NULL}};
const Segment SGI49773[] = {{&S72179[86], 4}, {NULL}};
const Segment SGI49774[] = {{&S72579[30], 6}, {NULL}};
const Segment SGI49775[] = {{&S72179[86], 3}, {NULL}};
const Segment SGI49776[] = {{&S72579[114], 3}, {NULL}};
const Segment SGI49777[] = {{&S72579[80], 5}, {NULL}};
const Segment SGI49778[] = {{&S72180[44], 14}, {NULL}};
const Segment SGI49779[] = {{&S72180[44], 25}, {NULL}};
const Segment SGI49780[] = {{&S72180[30], 6}, {NULL}};
const Segment SGI49781[] = {{&S72180[117], 3}, {NULL}};
const Segment SGI49782[] = {{&S72580[44], 11}, {NULL}};
const Segment SGI49783[] = {{&S72580[44], 22}, {NULL}};
const Segment SGI49784[] = {{&S72580[30], 6}, {NULL}};
const Segment SGI49785[] = {{&S72180[83], 5}, {NULL}};
const Segment SGI49786[] = {{&S72580[83], 6}, {NULL}};
const Segment SGI49787[] = {{&S72591[59], 10}, {NULL}};
const Segment SGI49788[] = {{&S72591[59], 27}, {NULL}};
const Segment SGI49789[] = {{&S72591[45], 6}, {NULL}};
const Segment SGI49790[] = {{&S72591[134], 3}, {NULL}};
const Segment SGI49791[] = {{&S72591[100], 5}, {NULL}};
const Segment SGI49792[] = {{&S72591[0], 14}, {NULL}};
const Segment SGI49793[] = {{&S72597[59], 11}, {NULL}};
const Segment SGI49794[] = {{&S72597[59], 28}, {NULL}};
const Segment SGI49795[] = {{&S72597[45], 6}, {NULL}};
const Segment SGI49796[] = {{&S72597[135], 3}, {NULL}};
const Segment SGI49797[] = {{&S72597[101], 5}, {NULL}};
const Segment SGI49798[] = {{&S72597[0], 14}, {NULL}};
const Segment SGI49799[] = {{&S72613[44], 11}, {NULL}};
const Segment SGI49800[] = {{&S72613[44], 21}, {NULL}};
const Segment SGI49801[] = {{&S72613[30], 6}, {NULL}};
const Segment SGI49802[] = {{&S72613[113], 3}, {NULL}};
const Segment SGI49803[] = {{&S72613[79], 5}, {NULL}};
const Segment SGI49804[] = {{&S72296[31], 8}, {NULL}};
const Segment SGI49805[] = {{&S72296[54], 3}, {NULL}};
const Segment SGI49806[] = {{&S76302[44], 10}, {NULL}};
const Segment SGI49807[] = {{&S76302[44], 27}, {NULL}};
const Segment SGI49808[] = {{&S76302[30], 6}, {NULL}};
const Segment SGI49809[] = {{&S76302[85], 6}, {NULL}};
const Segment SGI49810[] = {{&S76302[122], 3}, {NULL}};
const Segment SGI49811[] = {{&S72318[44], 9}, {NULL}};
const Segment SGI49812[] = {{&S72318[44], 20}, {NULL}};
const Segment SGI49813[] = {{&S72318[30], 6}, {NULL}};
const Segment SGI49814[] = {{&S72318[112], 3}, {NULL}};
const Segment SGI49815[] = {{&S72318[78], 5}, {NULL}};
const Segment SGI49816[] = {{&S72319[44], 9}, {NULL}};
const Segment SGI49817[] = {{&S72319[44], 20}, {NULL}};
const Segment SGI49818[] = {{&S72319[30], 6}, {NULL}};
const Segment SGI49819[] = {{&S72319[112], 3}, {NULL}};
const Segment SGI49820[] = {{&S72319[78], 5}, {NULL}};
const Segment SGI49821[] = {{&S72320[38], 9}, {NULL}};
const Segment SGI49822[] = {{&S72320[31], 6}, {NULL}};
const Segment SGI49823[] = {{&S72321[44], 9}, {NULL}};
const Segment SGI49824[] = {{&S72321[44], 20}, {NULL}};
const Segment SGI49825[] = {{&S72321[30], 6}, {NULL}};
const Segment SGI49826[] = {{&S72321[112], 3}, {NULL}};
const Segment SGI49827[] = {{&S72321[78], 5}, {NULL}};
const Segment SGI49828[] = {{&S76333[59], 10}, {NULL}};
const Segment SGI49829[] = {{&S76333[59], 27}, {NULL}};
const Segment SGI49830[] = {{&S76333[45], 6}, {NULL}};
const Segment SGI49831[] = {{&S76333[134], 3}, {NULL}};
const Segment SGI49832[] = {{&S76333[100], 5}, {NULL}};
const Segment SGI49833[] = {{&S76333[0], 14}, {NULL}};
const Segment SGI49834[] = {{&S72799[34], 7}, {NULL}};
const Segment SGI49835[] = {{&S72799[27], 6}, {NULL}};
const Segment SGI49836[] = {{&S72799[111], 2}, {NULL}};
const Segment SGI49837[] = {{&S72799[77], 6}, {NULL}};
const Segment SGI49838[] = {{&S72799[145], 8}, {NULL}};
const Segment SGI49839[] = {{&S87908[44], 14}, {NULL}};
const Segment SGI49840[] = {{&S87908[44], 26}, {NULL}};
const Segment SGI49841[] = {{&S87908[30], 6}, {NULL}};
const Segment SGI49842[] = {{&S87908[118], 3}, {NULL}};
const Segment SGI49843[] = {{&S87908[84], 5}, {NULL}};
const Segment SGI49844[] = {{&S72427[44], 15}, {NULL}};
const Segment SGI49845[] = {{&S72427[44], 26}, {NULL}};
const Segment SGI49846[] = {{&S72427[30], 6}, {NULL}};
const Segment SGI49847[] = {{&S72427[118], 3}, {NULL}};
const Segment SGI49848[] = {{&S76433[44], 12}, {NULL}};
const Segment SGI49849[] = {{&S72427[84], 5}, {NULL}};
const Segment SGI49850[] = {{&S76433[44], 23}, {NULL}};
const Segment SGI49851[] = {{&S76433[30], 6}, {NULL}};
const Segment SGI49852[] = {{&S76433[115], 3}, {NULL}};
const Segment SGI49853[] = {{&S76433[81], 5}, {NULL}};
const Segment SGI49854[] = {{&S72428[38], 15}, {NULL}};
const Segment SGI49855[] = {{&S72428[31], 6}, {NULL}};
const Segment SGI49856[] = {{&S76434[44], 12}, {NULL}};
const Segment SGI49857[] = {{&S76434[44], 23}, {NULL}};
const Segment SGI49858[] = {{&S76434[30], 6}, {NULL}};
const Segment SGI49859[] = {{&S76434[115], 3}, {NULL}};
const Segment SGI49860[] = {{&S76434[81], 5}, {NULL}};
const Segment SGI49861[] = {{&S72457[6], 4}, {NULL}};
const Segment SGI49862[] = {{&S72457[81], 5}, {NULL}};
const Segment SGI49863[] = {{&S72887[44], 6}, {NULL}};
const Segment SGI49864[] = {{&S72887[44], 17}, {NULL}};
const Segment SGI49865[] = {{&S72887[30], 6}, {NULL}};
const Segment SGI49866[] = {{&S72887[109], 3}, {NULL}};
const Segment SGI49867[] = {{&S72887[75], 5}, {NULL}};
const Segment SGI49868[] = {{&S72889[44], 10}, {NULL}};
const Segment SGI49869[] = {{&S72889[44], 23}, {NULL}};
const Segment SGI49870[] = {{&S72889[30], 6}, {NULL}};
const Segment SGI49871[] = {{&S72889[115], 3}, {NULL}};
const Segment SGI49872[] = {{&S72889[81], 5}, {NULL}};
const Segment SGI49873[] = {{&S72485[45], 20}, {NULL}};
const Segment SGI49874[] = {{&S72485[30], 6}, {NULL}};
const Segment SGI49875[] = {{&S72485[124], 3}, {NULL}};
const Segment SGI49876[] = {{&S72485[90], 5}, {NULL}};
const Segment SGI49877[] = {{&S72485[148], 13}, {NULL}};
const Segment SGI49878[] = {{&S72486[44], 20}, {NULL}};
const Segment SGI49879[] = {{&S72486[44], 31}, {NULL}};
const Segment SGI49880[] = {{&S72486[30], 6}, {NULL}};
const Segment SGI49881[] = {{&S72486[124], 3}, {NULL}};
const Segment SGI49882[] = {{&S72486[89], 6}, {NULL}};
const Segment SGI49883[] = {{&S72486[144], 8}, {NULL}};
const Segment SGI49884[] = {{&S72486[153], 13}, {NULL}};
const Segment SGI49885[] = {{&S72903[44], 13}, {NULL}};
const Segment SGI49886[] = {{&S72903[44], 30}, {NULL}};
const Segment SGI49887[] = {{&S72903[30], 6}, {NULL}};
const Segment SGI49888[] = {{&S72903[122], 3}, {NULL}};
const Segment SGI49889[] = {{&S72903[88], 5}, {NULL}};
const Segment SGI49890[] = {{&S72903[156], 3}, {NULL}};
const Segment SGI49891[] = {{&S72927[44], 7}, {NULL}};
const Segment SGI49892[] = {{&S72927[44], 24}, {NULL}};
const Segment SGI49893[] = {{&S72927[30], 6}, {NULL}};
const Segment SGI49894[] = {{&S72927[116], 3}, {NULL}};
const Segment SGI49895[] = {{&S72927[82], 5}, {NULL}};
const Segment SGI49896[] = {{&S72931[44], 8}, {NULL}};
const Segment SGI49897[] = {{&S72931[44], 25}, {NULL}};
const Segment SGI49898[] = {{&S72931[30], 6}, {NULL}};
const Segment SGI49899[] = {{&S72931[117], 3}, {NULL}};
const Segment SGI49900[] = {{&S72931[83], 5}, {NULL}};
const Segment SGI49901[] = {{&S72565[44], 9}, {NULL}};
const Segment SGI49902[] = {{&S72565[44], 20}, {NULL}};
const Segment SGI49903[] = {{&S72565[30], 6}, {NULL}};
const Segment SGI49904[] = {{&S72565[112], 3}, {NULL}};
const Segment SGI49905[] = {{&S72565[78], 5}, {NULL}};
const Segment SGI49906[] = {{&S72589[59], 10}, {NULL}};
const Segment SGI49907[] = {{&S72589[59], 27}, {NULL}};
const Segment SGI49908[] = {{&S72589[45], 6}, {NULL}};
const Segment SGI49909[] = {{&S72589[134], 3}, {NULL}};
const Segment SGI49910[] = {{&S72589[100], 5}, {NULL}};
const Segment SGI49911[] = {{&S72589[0], 14}, {NULL}};
const Segment SGI49912[] = {{&S72590[59], 11}, {NULL}};
const Segment SGI49913[] = {{&S72590[59], 28}, {NULL}};
const Segment SGI49914[] = {{&S72590[45], 6}, {NULL}};
const Segment SGI49915[] = {{&S72590[135], 3}, {NULL}};
const Segment SGI49916[] = {{&S72590[101], 5}, {NULL}};
const Segment SGI49917[] = {{&S72590[0], 14}, {NULL}};
const Segment SGI49918[] = {{&S72988[44], 24}, {NULL}};
const Segment SGI49919[] = {{&S72988[44], 41}, {NULL}};
const Segment SGI49920[] = {{&S72988[30], 6}, {NULL}};
const Segment SGI49921[] = {{&S72988[133], 3}, {NULL}};
const Segment SGI49922[] = {{&S72988[99], 5}, {NULL}};
const Segment SGI49923[] = {{&S72989[44], 23}, {NULL}};
const Segment SGI49924[] = {{&S72989[44], 40}, {NULL}};
const Segment SGI49925[] = {{&S72989[30], 6}, {NULL}};
const Segment SGI49926[] = {{&S72989[132], 3}, {NULL}};
const Segment SGI49927[] = {{&S72989[98], 5}, {NULL}};
const Segment SGI49928[] = {{&S72612[44], 11}, {NULL}};
const Segment SGI49929[] = {{&S72612[44], 21}, {NULL}};
const Segment SGI49930[] = {{&S72612[30], 6}, {NULL}};
const Segment SGI49931[] = {{&S72612[113], 3}, {NULL}};
const Segment SGI49932[] = {{&S72612[79], 5}, {NULL}};
const Segment SGI49933[] = {{&S80605[54], 6}, {NULL}};
const Segment SGI49934[] = {{&S80605[54], 25}, {NULL}};
const Segment SGI49935[] = {{&S80605[150], 3}, {NULL}};
const Segment SGI49936[] = {{&S80605[116], 5}, {NULL}};
const Segment SGI49937[] = {{&S72637[44], 21}, {NULL}};
const Segment SGI49938[] = {{&S72637[44], 32}, {NULL}};
const Segment SGI49939[] = {{&S72637[30], 6}, {NULL}};
const Segment SGI49940[] = {{&S72637[90], 6}, {NULL}};
const Segment SGI49941[] = {{&S72637[117], 13}, {NULL}};
const Segment SGI49942[] = {{&S72662[44], 6}, {NULL}};
const Segment SGI49943[] = {{&S72662[44], 17}, {NULL}};
const Segment SGI49944[] = {{&S72662[30], 6}, {NULL}};
const Segment SGI49945[] = {{&S72662[109], 3}, {NULL}};
const Segment SGI49946[] = {{&S72662[75], 5}, {NULL}};
const Segment SGI49947[] = {{&S88179[44], 15}, {NULL}};
const Segment SGI49948[] = {{&S88179[44], 27}, {NULL}};
const Segment SGI49949[] = {{&S88179[30], 6}, {NULL}};
const Segment SGI49950[] = {{&S88179[119], 3}, {NULL}};
const Segment SGI49951[] = {{&S88179[85], 5}, {NULL}};
const Segment SGI49952[] = {{&S73069[44], 8}, {NULL}};
const Segment SGI49953[] = {{&S73069[44], 19}, {NULL}};
const Segment SGI49954[] = {{&S73069[30], 6}, {NULL}};
const Segment SGI49955[] = {{&S73069[111], 3}, {NULL}};
const Segment SGI49956[] = {{&S73069[77], 5}, {NULL}};
const Segment SGI49957[] = {{&S73070[44], 8}, {NULL}};
const Segment SGI49958[] = {{&S73070[44], 19}, {NULL}};
const Segment SGI49959[] = {{&S73070[30], 6}, {NULL}};
const Segment SGI49960[] = {{&S73070[111], 3}, {NULL}};
const Segment SGI49961[] = {{&S73070[77], 5}, {NULL}};
const Segment SGI49962[] = {{&S73071[38], 8}, {NULL}};
const Segment SGI49963[] = {{&S73071[31], 6}, {NULL}};
const Segment SGI49964[] = {{&S73072[44], 8}, {NULL}};
const Segment SGI49965[] = {{&S73072[44], 19}, {NULL}};
const Segment SGI49966[] = {{&S73072[30], 6}, {NULL}};
const Segment SGI49967[] = {{&S73072[111], 3}, {NULL}};
const Segment SGI49968[] = {{&S73072[77], 5}, {NULL}};
const Segment SGI49969[] = {{&S88202[44], 10}, {NULL}};
const Segment SGI49970[] = {{&S88202[44], 21}, {NULL}};
const Segment SGI49971[] = {{&S88202[30], 6}, {NULL}};
const Segment SGI49972[] = {{&S88202[113], 3}, {NULL}};
const Segment SGI49973[] = {{&S88202[79], 5}, {NULL}};
const Segment SGI49974[] = {{&S88203[44], 10}, {NULL}};
const Segment SGI49975[] = {{&S88203[44], 21}, {NULL}};
const Segment SGI49976[] = {{&S88203[30], 6}, {NULL}};
const Segment SGI49977[] = {{&S88203[82], 4}, {NULL}};
const Segment SGI49978[] = {{&S88203[82], 3}, {NULL}};
const Segment SGI49979[] = {{&S88204[44], 10}, {NULL}};
const Segment SGI49980[] = {{&S88204[44], 21}, {NULL}};
const Segment SGI49981[] = {{&S88204[30], 6}, {NULL}};
const Segment SGI49982[] = {{&S88204[82], 6}, {NULL}};
const Segment SGI49983[] = {{&S72736[45], 10}, {NULL}};
const Segment SGI49984[] = {{&S72736[30], 6}, {NULL}};
const Segment SGI49985[] = {{&S72736[120], 3}, {NULL}};
const Segment SGI49986[] = {{&S72736[86], 5}, {NULL}};
const Segment SGI49987[] = {{&S72736[144], 13}, {NULL}};
const Segment SGI49988[] = {{&S88227[45], 9}, {NULL}};
const Segment SGI49989[] = {{&S88227[30], 6}, {NULL}};
const Segment SGI49990[] = {{&S88227[113], 3}, {NULL}};
const Segment SGI49991[] = {{&S88227[79], 5}, {NULL}};
const Segment SGI49992[] = {{&S88227[137], 13}, {NULL}};
const Segment SGI49993[] = {{&S82035[44], 14}, {NULL}};
const Segment SGI49994[] = {{&S82035[44], 31}, {NULL}};
const Segment SGI49995[] = {{&S82035[30], 6}, {NULL}};
const Segment SGI49996[] = {{&S82035[123], 3}, {NULL}};
const Segment SGI49997[] = {{&S82035[89], 5}, {NULL}};
const Segment SGI49998[] = {{&S88303[58], 12}, {NULL}};
const Segment SGI49999[] = {{&S88303[58], 23}, {NULL}};
const Segment SGI50000[] = {{&S88303[44], 6}, {NULL}};
const Segment SGI50001[] = {{&S88303[129], 3}, {NULL}};
const Segment SGI50002[] = {{&S88303[95], 5}, {NULL}};
const Segment SGI50003[] = {{&S72819[44], 6}, {NULL}};
const Segment SGI50004[] = {{&S72819[44], 17}, {NULL}};
const Segment SGI50005[] = {{&S72819[30], 6}, {NULL}};
const Segment SGI50006[] = {{&S72819[109], 3}, {NULL}};
const Segment SGI50007[] = {{&S72819[75], 5}, {NULL}};
const Segment SGI50008[] = {{&S72820[38], 6}, {NULL}};
const Segment SGI50009[] = {{&S72820[31], 6}, {NULL}};
const Segment SGI50010[] = {{&S72863[44], 19}, {NULL}};
const Segment SGI50011[] = {{&S73225[44], 6}, {NULL}};
const Segment SGI50012[] = {{&S73225[44], 23}, {NULL}};
const Segment SGI50013[] = {{&S73225[30], 6}, {NULL}};
const Segment SGI50014[] = {{&S73225[115], 3}, {NULL}};
const Segment SGI50015[] = {{&S73225[81], 5}, {NULL}};
const Segment SGI50016[] = {{&S73226[38], 6}, {NULL}};
const Segment SGI50017[] = {{&S73226[31], 6}, {NULL}};
const Segment SGI50018[] = {{&S88432[44], 13}, {NULL}};
const Segment SGI50019[] = {{&S88432[44], 24}, {NULL}};
const Segment SGI50020[] = {{&S88432[30], 6}, {NULL}};
const Segment SGI50021[] = {{&S88432[85], 4}, {NULL}};
const Segment SGI50022[] = {{&S88432[85], 3}, {NULL}};
const Segment SGI50023[] = {{&S88433[44], 13}, {NULL}};
const Segment SGI50024[] = {{&S88433[44], 24}, {NULL}};
const Segment SGI50025[] = {{&S88433[30], 6}, {NULL}};
const Segment SGI50026[] = {{&S88433[116], 3}, {NULL}};
const Segment SGI50027[] = {{&S88433[82], 5}, {NULL}};
const Segment SGI50028[] = {{&S88434[44], 13}, {NULL}};
const Segment SGI50029[] = {{&S88434[44], 24}, {NULL}};
const Segment SGI50030[] = {{&S88434[30], 6}, {NULL}};
const Segment SGI50031[] = {{&S88434[85], 6}, {NULL}};
const Segment SGI50032[] = {{&S72863[44], 32}, {NULL}};
const Segment SGI50033[] = {{&S72863[30], 6}, {NULL}};
const Segment SGI50034[] = {{&S72863[124], 3}, {NULL}};
const Segment SGI50035[] = {{&S72863[90], 5}, {NULL}};
const Segment SGI50036[] = {{&S72865[44], 6}, {NULL}};
const Segment SGI50037[] = {{&S72865[44], 26}, {NULL}};
const Segment SGI50038[] = {{&S72865[30], 6}, {NULL}};
const Segment SGI50039[] = {{&S72865[118], 3}, {NULL}};
const Segment SGI50040[] = {{&S72865[84], 5}, {NULL}};
const Segment SGI50041[] = {{&S72885[44], 6}, {NULL}};
const Segment SGI50042[] = {{&S72885[44], 17}, {NULL}};
const Segment SGI50043[] = {{&S72885[30], 6}, {NULL}};
const Segment SGI50044[] = {{&S72885[109], 3}, {NULL}};
const Segment SGI50045[] = {{&S72885[75], 5}, {NULL}};
const Segment SGI50046[] = {{&S72896[44], 18}, {NULL}};
const Segment SGI50047[] = {{&S72896[44], 28}, {NULL}};
const Segment SGI50048[] = {{&S72896[30], 6}, {NULL}};
const Segment SGI50049[] = {{&S72896[120], 3}, {NULL}};
const Segment SGI50050[] = {{&S72896[86], 5}, {NULL}};
const Segment SGI50051[] = {{&S76910[44], 9}, {NULL}};
const Segment SGI50052[] = {{&S76910[44], 26}, {NULL}};
const Segment SGI50053[] = {{&S76910[30], 6}, {NULL}};
const Segment SGI50054[] = {{&S76910[118], 3}, {NULL}};
const Segment SGI50055[] = {{&S76910[84], 5}, {NULL}};
const Segment SGI50056[] = {{&S76911[44], 9}, {NULL}};
const Segment SGI50057[] = {{&S76911[44], 26}, {NULL}};
const Segment SGI50058[] = {{&S76911[30], 6}, {NULL}};
const Segment SGI50059[] = {{&S76911[87], 4}, {NULL}};
const Segment SGI50060[] = {{&S76911[87], 3}, {NULL}};
const Segment SGI50061[] = {{&S76912[44], 9}, {NULL}};
const Segment SGI50062[] = {{&S76912[44], 26}, {NULL}};
const Segment SGI50063[] = {{&S76912[30], 6}, {NULL}};
const Segment SGI50064[] = {{&S76912[87], 6}, {NULL}};
const Segment SGI50065[] = {{&S76913[31], 20}, {NULL}};
const Segment SGI50066[] = {{&S76913[66], 3}, {NULL}};
const Segment SGI50067[] = {{&S76915[39], 20}, {NULL}};
const Segment SGI50068[] = {{&S76915[115], 3}, {NULL}};
const Segment SGI50069[] = {{&S76915[104], 3}, {NULL}};
const Segment SGI50070[] = {{&S76915[81], 6}, {NULL}};
const Segment SGI50071[] = {{&S73373[44], 21}, {NULL}};
const Segment SGI50072[] = {{&S73373[44], 34}, {NULL}};
const Segment SGI50073[] = {{&S73373[30], 6}, {NULL}};
const Segment SGI50074[] = {{&S73373[126], 3}, {NULL}};
const Segment SGI50075[] = {{&S73373[92], 5}, {NULL}};
const Segment SGI50076[] = {{&S72997[44], 10}, {NULL}};
const Segment SGI50077[] = {{&S72997[44], 21}, {NULL}};
const Segment SGI50078[] = {{&S72997[30], 6}, {NULL}};
const Segment SGI50079[] = {{&S72997[113], 3}, {NULL}};
const Segment SGI50080[] = {{&S72997[79], 5}, {NULL}};
const Segment SGI50081[] = {{&S72998[44], 10}, {NULL}};
const Segment SGI50082[] = {{&S72998[44], 21}, {NULL}};
const Segment SGI50083[] = {{&S72998[30], 6}, {NULL}};
const Segment SGI50084[] = {{&S72998[113], 3}, {NULL}};
const Segment SGI50085[] = {{&S72998[79], 5}, {NULL}};
const Segment SGI50086[] = {{&S72999[44], 10}, {NULL}};
const Segment SGI50087[] = {{&S72999[44], 21}, {NULL}};
const Segment SGI50088[] = {{&S72999[30], 6}, {NULL}};
const Segment SGI50089[] = {{&S72999[113], 3}, {NULL}};
const Segment SGI50090[] = {{&S72999[79], 5}, {NULL}};
const Segment SGI50091[] = {{&S73000[38], 10}, {NULL}};
const Segment SGI50092[] = {{&S73000[31], 6}, {NULL}};
const Segment SGI50093[] = {{&S73001[44], 10}, {NULL}};
const Segment SGI50094[] = {{&S73001[44], 21}, {NULL}};
const Segment SGI50095[] = {{&S73001[30], 6}, {NULL}};
const Segment SGI50096[] = {{&S73001[113], 3}, {NULL}};
const Segment SGI50097[] = {{&S73001[79], 5}, {NULL}};
const Segment SGI50098[] = {{&S73002[44], 10}, {NULL}};
const Segment SGI50099[] = {{&S73002[44], 21}, {NULL}};
const Segment SGI50100[] = {{&S73002[30], 6}, {NULL}};
const Segment SGI50101[] = {{&S73002[113], 3}, {NULL}};
const Segment SGI50102[] = {{&S73002[79], 5}, {NULL}};
const Segment SGI50103[] = {{&S73025[44], 5}, {NULL}};
const Segment SGI50104[] = {{&S73025[44], 16}, {NULL}};
const Segment SGI50105[] = {{&S73025[30], 6}, {NULL}};
const Segment SGI50106[] = {{&S73025[108], 3}, {NULL}};
const Segment SGI50107[] = {{&S73025[74], 5}, {NULL}};
const Segment SGI50108[] = {{&S73435[45], 10}, {NULL}};
const Segment SGI50109[] = {{&S73435[30], 6}, {NULL}};
const Segment SGI50110[] = {{&S73435[120], 3}, {NULL}};
const Segment SGI50111[] = {{&S73435[86], 5}, {NULL}};
const Segment SGI50112[] = {{&S73435[144], 13}, {NULL}};
const Segment SGI50113[] = {{&S73056[44], 10}, {NULL}};
const Segment SGI50114[] = {{&S73056[44], 27}, {NULL}};
const Segment SGI50115[] = {{&S73056[30], 6}, {NULL}};
const Segment SGI50116[] = {{&S73056[119], 3}, {NULL}};
const Segment SGI50117[] = {{&S73056[85], 5}, {NULL}};
const Segment SGI50118[] = {{&S73093[34], 7}, {NULL}};
const Segment SGI50119[] = {{&S73093[27], 6}, {NULL}};
const Segment SGI50120[] = {{&S73093[114], 2}, {NULL}};
const Segment SGI50121[] = {{&S73093[80], 6}, {NULL}};
const Segment SGI50122[] = {{&S73093[148], 8}, {NULL}};
const Segment SGI50123[] = {{&S73094[34], 20}, {NULL}};
const Segment SGI50124[] = {{&S73094[27], 6}, {NULL}};
const Segment SGI50125[] = {{&S73094[127], 2}, {NULL}};
const Segment SGI50126[] = {{&S73094[93], 6}, {NULL}};
const Segment SGI50127[] = {{&S73094[161], 8}, {NULL}};
const Segment SGI50128[] = {{&S73161[44], 12}, {NULL}};
const Segment SGI50129[] = {{&S73161[44], 23}, {NULL}};
const Segment SGI50130[] = {{&S73161[30], 6}, {NULL}};
const Segment SGI50131[] = {{&S73161[115], 3}, {NULL}};
const Segment SGI50132[] = {{&S73161[81], 5}, {NULL}};
const Segment SGI50133[] = {{&S73561[44], 22}, {NULL}};
const Segment SGI50134[] = {{&S73561[30], 6}, {NULL}};
const Segment SGI50135[] = {{&S73561[116], 3}, {NULL}};
const Segment SGI50136[] = {{&S73561[82], 5}, {NULL}};
const Segment SGI50137[] = {{&S73562[44], 22}, {NULL}};
const Segment SGI50138[] = {{&S73562[30], 6}, {NULL}};
const Segment SGI50139[] = {{&S73562[116], 3}, {NULL}};
const Segment SGI50140[] = {{&S73562[82], 5}, {NULL}};
const Segment SGI50141[] = {{&S73563[44], 10}, {NULL}};
const Segment SGI50142[] = {{&S73563[30], 6}, {NULL}};
const Segment SGI50143[] = {{&S73563[104], 3}, {NULL}};
const Segment SGI50144[] = {{&S73563[70], 5}, {NULL}};
const Segment SGI50145[] = {{&S73564[44], 11}, {NULL}};
const Segment SGI50146[] = {{&S73564[44], 22}, {NULL}};
const Segment SGI50147[] = {{&S73564[30], 6}, {NULL}};
const Segment SGI50148[] = {{&S73564[83], 4}, {NULL}};
const Segment SGI50149[] = {{&S73564[83], 3}, {NULL}};
const Segment SGI50150[] = {{&S73565[44], 22}, {NULL}};
const Segment SGI50151[] = {{&S73565[30], 6}, {NULL}};
const Segment SGI50152[] = {{&S73565[116], 3}, {NULL}};
const Segment SGI50153[] = {{&S73565[82], 5}, {NULL}};
const Segment SGI50154[] = {{&S73566[44], 22}, {NULL}};
const Segment SGI50155[] = {{&S73566[30], 6}, {NULL}};
const Segment SGI50156[] = {{&S73566[116], 3}, {NULL}};
const Segment SGI50157[] = {{&S73178[44], 9}, {NULL}};
const Segment SGI50158[] = {{&S73566[82], 5}, {NULL}};
const Segment SGI50159[] = {{&S73178[44], 20}, {NULL}};
const Segment SGI50160[] = {{&S73178[30], 6}, {NULL}};
const Segment SGI50161[] = {{&S73178[112], 3}, {NULL}};
const Segment SGI50162[] = {{&S73178[78], 5}, {NULL}};
const Segment SGI50163[] = {{&S73568[44], 11}, {NULL}};
const Segment SGI50164[] = {{&S73568[44], 22}, {NULL}};
const Segment SGI50165[] = {{&S73568[30], 6}, {NULL}};
const Segment SGI50166[] = {{&S73568[83], 6}, {NULL}};
const Segment SGI50167[] = {{&S88845[44], 15}, {NULL}};
const Segment SGI50168[] = {{&S88845[44], 27}, {NULL}};
const Segment SGI50169[] = {{&S88845[30], 6}, {NULL}};
const Segment SGI50170[] = {{&S88845[119], 3}, {NULL}};
const Segment SGI50171[] = {{&S88845[85], 5}, {NULL}};
const Segment SGI50172[] = {{&S73673[44], 10}, {NULL}};
const Segment SGI50173[] = {{&S73673[44], 27}, {NULL}};
const Segment SGI50174[] = {{&S73673[30], 6}, {NULL}};
const Segment SGI50175[] = {{&S73673[118], 3}, {NULL}};
const Segment SGI50176[] = {{&S73673[85], 5}, {NULL}};
const Segment SGI50177[] = {{&S73673[122], 14}, {NULL}};
const Segment SGI50178[] = {{&S73675[44], 11}, {NULL}};
const Segment SGI50179[] = {{&S73675[44], 28}, {NULL}};
const Segment SGI50180[] = {{&S73675[30], 6}, {NULL}};
const Segment SGI50181[] = {{&S73675[119], 3}, {NULL}};
const Segment SGI50182[] = {{&S73675[86], 5}, {NULL}};
const Segment SGI50183[] = {{&S73675[123], 14}, {NULL}};
const Segment SGI50184[] = {{&S77196[44], 8}, {NULL}};
const Segment SGI50185[] = {{&S77196[44], 19}, {NULL}};
const Segment SGI50186[] = {{&S77196[30], 6}, {NULL}};
const Segment SGI50187[] = {{&S77196[111], 3}, {NULL}};
const Segment SGI50188[] = {{&S77196[77], 5}, {NULL}};
const Segment SGI50189[] = {{&S88974[44], 11}, {NULL}};
const Segment SGI50190[] = {{&S88974[44], 22}, {NULL}};
const Segment SGI50191[] = {{&S88974[30], 6}, {NULL}};
const Segment SGI50192[] = {{&S88974[114], 3}, {NULL}};
const Segment SGI50193[] = {{&S88974[80], 5}, {NULL}};
const Segment SGI50194[] = {{&S73751[44], 9}, {NULL}};
const Segment SGI50195[] = {{&S73751[44], 20}, {NULL}};
const Segment SGI50196[] = {{&S73751[30], 6}, {NULL}};
const Segment SGI50197[] = {{&S73751[112], 3}, {NULL}};
const Segment SGI50198[] = {{&S73751[78], 5}, {NULL}};
const Segment SGI50199[] = {{&S73352[44], 14}, {NULL}};
const Segment SGI50200[] = {{&S73352[44], 35}, {NULL}};
const Segment SGI50201[] = {{&S73352[30], 6}, {NULL}};
const Segment SGI50202[] = {{&S73352[127], 3}, {NULL}};
const Segment SGI50203[] = {{&S73352[93], 5}, {NULL}};
const Segment SGI50204[] = {{&S73753[44], 9}, {NULL}};
const Segment SGI50205[] = {{&S73753[44], 20}, {NULL}};
const Segment SGI50206[] = {{&S73753[30], 6}, {NULL}};
const Segment SGI50207[] = {{&S73753[112], 3}, {NULL}};
const Segment SGI50208[] = {{&S73753[78], 5}, {NULL}};
const Segment SGI50209[] = {{&S73374[44], 21}, {NULL}};
const Segment SGI50210[] = {{&S73374[44], 34}, {NULL}};
const Segment SGI50211[] = {{&S73374[30], 6}, {NULL}};
const Segment SGI50212[] = {{&S73374[126], 3}, {NULL}};
const Segment SGI50213[] = {{&S73374[92], 5}, {NULL}};
const Segment SGI50214[] = {{&S73375[44], 21}, {NULL}};
const Segment SGI50215[] = {{&S73375[44], 34}, {NULL}};
const Segment SGI50216[] = {{&S73375[30], 6}, {NULL}};
const Segment SGI50217[] = {{&S73375[126], 3}, {NULL}};
const Segment SGI50218[] = {{&S73375[92], 5}, {NULL}};
const Segment SGI50219[] = {{&S73376[44], 21}, {NULL}};
const Segment SGI50220[] = {{&S73376[44], 34}, {NULL}};
const Segment SGI50221[] = {{&S73376[30], 6}, {NULL}};
const Segment SGI50222[] = {{&S73376[126], 3}, {NULL}};
const Segment SGI50223[] = {{&S73376[92], 5}, {NULL}};
const Segment SGI50224[] = {{&S73377[44], 21}, {NULL}};
const Segment SGI50225[] = {{&S73377[44], 34}, {NULL}};
const Segment SGI50226[] = {{&S73377[30], 6}, {NULL}};
const Segment SGI50227[] = {{&S73377[126], 3}, {NULL}};
const Segment SGI50228[] = {{&S73377[92], 5}, {NULL}};
const Segment SGI50229[] = {{&S73418[52], 10}, {NULL}};
const Segment SGI50230[] = {{&S73418[52], 27}, {NULL}};
const Segment SGI50231[] = {{&S73418[38], 6}, {NULL}};
const Segment SGI50232[] = {{&S73418[93], 5}, {NULL}};
const Segment SGI50233[] = {{&S73418[0], 14}, {NULL}};
const Segment SGI50234[] = {{&S82901[44], 10}, {NULL}};
const Segment SGI50235[] = {{&S82901[44], 27}, {NULL}};
const Segment SGI50236[] = {{&S82901[30], 6}, {NULL}};
const Segment SGI50237[] = {{&S82901[119], 3}, {NULL}};
const Segment SGI50238[] = {{&S82901[85], 5}, {NULL}};
const Segment SGI50239[] = {{&S89126[44], 9}, {NULL}};
const Segment SGI50240[] = {{&S89126[44], 21}, {NULL}};
const Segment SGI50241[] = {{&S89126[30], 6}, {NULL}};
const Segment SGI50242[] = {{&S89126[113], 3}, {NULL}};
const Segment SGI50243[] = {{&S89126[79], 5}, {NULL}};
const Segment SGI50244[] = {{&S77284[45], 22}, {NULL}};
const Segment SGI50245[] = {{&S77284[30], 6}, {NULL}};
const Segment SGI50246[] = {{&S77284[126], 3}, {NULL}};
const Segment SGI50247[] = {{&S77284[92], 5}, {NULL}};
const Segment SGI50248[] = {{&S77284[160], 3}, {NULL}};
const Segment SGI50249[] = {{&S73437[44], 13}, {NULL}};
const Segment SGI50250[] = {{&S73437[44], 25}, {NULL}};
const Segment SGI50251[] = {{&S73437[30], 6}, {NULL}};
const Segment SGI50252[] = {{&S73437[117], 3}, {NULL}};
const Segment SGI50253[] = {{&S73437[83], 5}, {NULL}};
const Segment SGI50254[] = {{&S73500[44], 11}, {NULL}};
const Segment SGI50255[] = {{&S73500[44], 23}, {NULL}};
const Segment SGI50256[] = {{&S73500[30], 6}, {NULL}};
const Segment SGI50257[] = {{&S73500[115], 3}, {NULL}};
const Segment SGI50258[] = {{&S73500[81], 5}, {NULL}};
const Segment SGI50259[] = {{&S77383[31], 8}, {NULL}};
const Segment SGI50260[] = {{&S77383[54], 3}, {NULL}};
const Segment SGI50261[] = {{&S73547[44], 6}, {NULL}};
const Segment SGI50262[] = {{&S73547[44], 23}, {NULL}};
const Segment SGI50263[] = {{&S73547[30], 6}, {NULL}};
const Segment SGI50264[] = {{&S73547[115], 3}, {NULL}};
const Segment SGI50265[] = {{&S73547[81], 5}, {NULL}};
const Segment SGI50266[] = {{&S77436[45], 11}, {NULL}};
const Segment SGI50267[] = {{&S77436[30], 6}, {NULL}};
const Segment SGI50268[] = {{&S77436[115], 3}, {NULL}};
const Segment SGI50269[] = {{&S77436[81], 5}, {NULL}};
const Segment SGI50270[] = {{&S77436[149], 3}, {NULL}};
const Segment SGI50271[] = {{&S73574[44], 30}, {NULL}};
const Segment SGI50272[] = {{&S73574[30], 6}, {NULL}};
const Segment SGI50273[] = {{&S73575[44], 19}, {NULL}};
const Segment SGI50274[] = {{&S73575[44], 30}, {NULL}};
const Segment SGI50275[] = {{&S73575[30], 6}, {NULL}};
const Segment SGI50276[] = {{&S73575[122], 3}, {NULL}};
const Segment SGI50277[] = {{&S73575[88], 5}, {NULL}};
const Segment SGI50278[] = {{&S73577[44], 17}, {NULL}};
const Segment SGI50279[] = {{&S73577[44], 28}, {NULL}};
const Segment SGI50280[] = {{&S73577[30], 6}, {NULL}};
const Segment SGI50281[] = {{&S73577[120], 3}, {NULL}};
const Segment SGI50282[] = {{&S73577[86], 5}, {NULL}};
const Segment SGI50283[] = {{&S73579[44], 17}, {NULL}};
const Segment SGI50284[] = {{&S73579[44], 38}, {NULL}};
const Segment SGI50285[] = {{&S73579[30], 6}, {NULL}};
const Segment SGI50286[] = {{&S73579[130], 3}, {NULL}};
const Segment SGI50287[] = {{&S73579[96], 5}, {NULL}};
const Segment SGI50288[] = {{&S74056[44], 14}, {NULL}};
const Segment SGI50289[] = {{&S74056[44], 24}, {NULL}};
const Segment SGI50290[] = {{&S74056[30], 6}, {NULL}};
const Segment SGI50291[] = {{&S74056[116], 3}, {NULL}};
const Segment SGI50292[] = {{&S74056[82], 5}, {NULL}};
const Segment SGI50293[] = {{&S77527[44], 13}, {NULL}};
const Segment SGI50294[] = {{&S77527[44], 24}, {NULL}};
const Segment SGI50295[] = {{&S77527[30], 6}, {NULL}};
const Segment SGI50296[] = {{&S77527[116], 3}, {NULL}};
const Segment SGI50297[] = {{&S77527[82], 5}, {NULL}};
const Segment SGI50298[] = {{&S77528[44], 13}, {NULL}};
const Segment SGI50299[] = {{&S77528[44], 24}, {NULL}};
const Segment SGI50300[] = {{&S77528[30], 6}, {NULL}};
const Segment SGI50301[] = {{&S77528[116], 3}, {NULL}};
const Segment SGI50302[] = {{&S77528[82], 5}, {NULL}};
const Segment SGI50303[] = {{&S77529[44], 13}, {NULL}};
const Segment SGI50304[] = {{&S77529[44], 24}, {NULL}};
const Segment SGI50305[] = {{&S77529[30], 6}, {NULL}};
const Segment SGI50306[] = {{&S77529[116], 3}, {NULL}};
const Segment SGI50307[] = {{&S77529[82], 5}, {NULL}};
const Segment SGI50308[] = {{&S73670[44], 10}, {NULL}};
const Segment SGI50309[] = {{&S73670[44], 27}, {NULL}};
const Segment SGI50310[] = {{&S73670[30], 6}, {NULL}};
const Segment SGI50311[] = {{&S73670[118], 3}, {NULL}};
const Segment SGI50312[] = {{&S73670[85], 5}, {NULL}};
const Segment SGI50313[] = {{&S73670[122], 14}, {NULL}};
const Segment SGI50314[] = {{&S73672[44], 11}, {NULL}};
const Segment SGI50315[] = {{&S73672[44], 28}, {NULL}};
const Segment SGI50316[] = {{&S73672[30], 6}, {NULL}};
const Segment SGI50317[] = {{&S73672[119], 3}, {NULL}};
const Segment SGI50318[] = {{&S73672[86], 5}, {NULL}};
const Segment SGI50319[] = {{&S73672[123], 14}, {NULL}};
const Segment SGI50320[] = {{&S73717[44], 14}, {NULL}};
const Segment SGI50321[] = {{&S73717[44], 25}, {NULL}};
const Segment SGI50322[] = {{&S73717[30], 6}, {NULL}};
const Segment SGI50323[] = {{&S73717[117], 3}, {NULL}};
const Segment SGI50324[] = {{&S73717[83], 5}, {NULL}};
const Segment SGI50325[] = {{&S89414[31], 8}, {NULL}};
const Segment SGI50326[] = {{&S89414[54], 3}, {NULL}};
const Segment SGI50327[] = {{&S73755[44], 9}, {NULL}};
const Segment SGI50328[] = {{&S73755[44], 20}, {NULL}};
const Segment SGI50329[] = {{&S73755[30], 6}, {NULL}};
const Segment SGI50330[] = {{&S73755[112], 3}, {NULL}};
const Segment SGI50331[] = {{&S73755[78], 5}, {NULL}};
const Segment SGI50332[] = {{&S77621[44], 10}, {NULL}};
const Segment SGI50333[] = {{&S77621[44], 27}, {NULL}};
const Segment SGI50334[] = {{&S77621[30], 6}, {NULL}};
const Segment SGI50335[] = {{&S77621[85], 6}, {NULL}};
const Segment SGI50336[] = {{&S77621[122], 3}, {NULL}};
const Segment SGI50337[] = {{&S77640[45], 10}, {NULL}};
const Segment SGI50338[] = {{&S77640[30], 6}, {NULL}};
const Segment SGI50339[] = {{&S77640[120], 3}, {NULL}};
const Segment SGI50340[] = {{&S77640[86], 5}, {NULL}};
const Segment SGI50341[] = {{&S77640[154], 3}, {NULL}};
const Segment SGI50342[] = {{&S74270[44], 7}, {NULL}};
const Segment SGI50343[] = {{&S74270[44], 19}, {NULL}};
const Segment SGI50344[] = {{&S74270[30], 6}, {NULL}};
const Segment SGI50345[] = {{&S74270[111], 3}, {NULL}};
const Segment SGI50346[] = {{&S74270[77], 5}, {NULL}};
const Segment SGI50347[] = {{&S74271[44], 7}, {NULL}};
const Segment SGI50348[] = {{&S74271[44], 19}, {NULL}};
const Segment SGI50349[] = {{&S74271[30], 6}, {NULL}};
const Segment SGI50350[] = {{&S74271[111], 3}, {NULL}};
const Segment SGI50351[] = {{&S74271[77], 5}, {NULL}};
const Segment SGI50352[] = {{&S77657[44], 14}, {NULL}};
const Segment SGI50353[] = {{&S77657[44], 25}, {NULL}};
const Segment SGI50354[] = {{&S77657[30], 6}, {NULL}};
const Segment SGI50355[] = {{&S77657[83], 6}, {NULL}};
const Segment SGI50356[] = {{&S77657[120], 3}, {NULL}};
const Segment SGI50357[] = {{&S77662[34], 10}, {NULL}};
const Segment SGI50358[] = {{&S77662[27], 6}, {NULL}};
const Segment SGI50359[] = {{&S77662[109], 2}, {NULL}};
const Segment SGI50360[] = {{&S77662[75], 6}, {NULL}};
const Segment SGI50361[] = {{&S77662[143], 8}, {NULL}};
const Segment SGI50362[] = {{&S89570[44], 14}, {NULL}};
const Segment SGI50363[] = {{&S89570[44], 26}, {NULL}};
const Segment SGI50364[] = {{&S89570[30], 6}, {NULL}};
const Segment SGI50365[] = {{&S89570[87], 4}, {NULL}};
const Segment SGI50366[] = {{&S89570[87], 3}, {NULL}};
const Segment SGI50367[] = {{&S89571[44], 37}, {NULL}};
const Segment SGI50368[] = {{&S89571[30], 6}, {NULL}};
const Segment SGI50369[] = {{&S89571[148], 3}, {NULL}};
const Segment SGI50370[] = {{&S89571[113], 6}, {NULL}};
const Segment SGI50371[] = {{&S89572[44], 37}, {NULL}};
const Segment SGI50372[] = {{&S89572[30], 6}, {NULL}};
const Segment SGI50373[] = {{&S89572[148], 3}, {NULL}};
const Segment SGI50374[] = {{&S89572[113], 6}, {NULL}};
const Segment SGI50375[] = {{&S73805[45], 22}, {NULL}};
const Segment SGI50376[] = {{&S73805[30], 6}, {NULL}};
const Segment SGI50377[] = {{&S73805[126], 3}, {NULL}};
const Segment SGI50378[] = {{&S73805[92], 5}, {NULL}};
const Segment SGI50379[] = {{&S73805[150], 13}, {NULL}};
const Segment SGI50380[] = {{&S73807[44], 22}, {NULL}};
const Segment SGI50381[] = {{&S73807[44], 33}, {NULL}};
const Segment SGI50382[] = {{&S73807[30], 6}, {NULL}};
const Segment SGI50383[] = {{&S73807[126], 3}, {NULL}};
const Segment SGI50384[] = {{&S73807[91], 6}, {NULL}};
const Segment SGI50385[] = {{&S73807[146], 8}, {NULL}};
const Segment SGI50386[] = {{&S73807[155], 13}, {NULL}};
const Segment SGI50387[] = {{&S74316[31], 5}, {NULL}};
const Segment SGI50388[] = {{&S74316[51], 3}, {NULL}};
const Segment SGI50389[] = {{&S74317[39], 5}, {NULL}};
const Segment SGI50390[] = {{&S74317[100], 3}, {NULL}};
const Segment SGI50391[] = {{&S74317[89], 3}, {NULL}};
const Segment SGI50392[] = {{&S74317[66], 6}, {NULL}};
const Segment SGI50393[] = {{&S77776[45], 10}, {NULL}};
const Segment SGI50394[] = {{&S77776[30], 6}, {NULL}};
const Segment SGI50395[] = {{&S77776[120], 3}, {NULL}};
const Segment SGI50396[] = {{&S77776[86], 5}, {NULL}};
const Segment SGI50397[] = {{&S77776[154], 3}, {NULL}};
const Segment SGI50398[] = {{&S74348[44], 13}, {NULL}};
const Segment SGI50399[] = {{&S74348[44], 24}, {NULL}};
const Segment SGI50400[] = {{&S74348[30], 6}, {NULL}};
const Segment SGI50401[] = {{&S74348[116], 3}, {NULL}};
const Segment SGI50402[] = {{&S74348[82], 5}, {NULL}};
const Segment SGI50403[] = {{&S73898[44], 20}, {NULL}};
const Segment SGI50404[] = {{&S73898[44], 31}, {NULL}};
const Segment SGI50405[] = {{&S73898[30], 6}, {NULL}};
const Segment SGI50406[] = {{&S73898[89], 6}, {NULL}};
const Segment SGI50407[] = {{&S73898[116], 13}, {NULL}};
const Segment SGI50408[] = {{&S74395[44], 10}, {NULL}};
const Segment SGI50409[] = {{&S74395[44], 27}, {NULL}};
const Segment SGI50410[] = {{&S74395[30], 6}, {NULL}};
const Segment SGI50411[] = {{&S74395[85], 6}, {NULL}};
const Segment SGI50412[] = {{&S74395[112], 13}, {NULL}};
const Segment SGI50413[] = {{&S89700[44], 16}, {NULL}};
const Segment SGI50414[] = {{&S89700[44], 28}, {NULL}};
const Segment SGI50415[] = {{&S89700[30], 6}, {NULL}};
const Segment SGI50416[] = {{&S89700[120], 3}, {NULL}};
const Segment SGI50417[] = {{&S89700[86], 5}, {NULL}};
const Segment SGI50418[] = {{&S74443[44], 10}, {NULL}};
const Segment SGI50419[] = {{&S74443[44], 27}, {NULL}};
const Segment SGI50420[] = {{&S74443[30], 6}, {NULL}};
const Segment SGI50421[] = {{&S74443[119], 3}, {NULL}};
const Segment SGI50422[] = {{&S74443[85], 5}, {NULL}};
const Segment SGI50423[] = {{&S74444[44], 10}, {NULL}};
const Segment SGI50424[] = {{&S74444[44], 27}, {NULL}};
const Segment SGI50425[] = {{&S74444[30], 6}, {NULL}};
const Segment SGI50426[] = {{&S74444[88], 4}, {NULL}};
const Segment SGI50427[] = {{&S74444[88], 3}, {NULL}};
const Segment SGI50428[] = {{&S74445[44], 10}, {NULL}};
const Segment SGI50429[] = {{&S74445[44], 27}, {NULL}};
const Segment SGI50430[] = {{&S74445[30], 6}, {NULL}};
const Segment SGI50431[] = {{&S74445[88], 6}, {NULL}};
const Segment SGI50432[] = {{&S74024[44], 10}, {NULL}};
const Segment SGI50433[] = {{&S74024[44], 20}, {NULL}};
const Segment SGI50434[] = {{&S74024[30], 6}, {NULL}};
const Segment SGI50435[] = {{&S74024[112], 3}, {NULL}};
const Segment SGI50436[] = {{&S74024[78], 5}, {NULL}};
const Segment SGI50437[] = {{&S74055[44], 14}, {NULL}};
const Segment SGI50438[] = {{&S74055[44], 24}, {NULL}};
const Segment SGI50439[] = {{&S74055[30], 6}, {NULL}};
const Segment SGI50440[] = {{&S74055[116], 3}, {NULL}};
const Segment SGI50441[] = {{&S74055[82], 5}, {NULL}};
const Segment SGI50442[] = {{&S74061[44], 15}, {NULL}};
const Segment SGI50443[] = {{&S74061[44], 26}, {NULL}};
const Segment SGI50444[] = {{&S74061[30], 6}, {NULL}};
const Segment SGI50445[] = {{&S74061[118], 3}, {NULL}};
const Segment SGI50446[] = {{&S74061[84], 5}, {NULL}};
const Segment SGI50447[] = {{&S74062[44], 7}, {NULL}};
const Segment SGI50448[] = {{&S74062[44], 18}, {NULL}};
const Segment SGI50449[] = {{&S74062[30], 6}, {NULL}};
const Segment SGI50450[] = {{&S74062[110], 3}, {NULL}};
const Segment SGI50451[] = {{&S74062[76], 5}, {NULL}};
const Segment SGI50452[] = {{&S89754[44], 10}, {NULL}};
const Segment SGI50453[] = {{&S89754[44], 29}, {NULL}};
const Segment SGI50454[] = {{&S89754[30], 6}, {NULL}};
const Segment SGI50455[] = {{&S89754[121], 3}, {NULL}};
const Segment SGI50456[] = {{&S89754[87], 5}, {NULL}};
const Segment SGI50457[] = {{&S74576[44], 8}, {NULL}};
const Segment SGI50458[] = {{&S74576[44], 26}, {NULL}};
const Segment SGI50459[] = {{&S74576[30], 6}, {NULL}};
const Segment SGI50460[] = {{&S74576[58], 3}, {NULL}};
const Segment SGI50461[] = {{&S74576[84], 5}, {NULL}};
const Segment SGI50462[] = {{&S74577[44], 8}, {NULL}};
const Segment SGI50463[] = {{&S74577[44], 25}, {NULL}};
const Segment SGI50464[] = {{&S74577[30], 6}, {NULL}};
const Segment SGI50465[] = {{&S74577[58], 3}, {NULL}};
const Segment SGI50466[] = {{&S74577[83], 5}, {NULL}};
const Segment SGI50467[] = {{&S74578[44], 8}, {NULL}};
const Segment SGI50468[] = {{&S74578[44], 26}, {NULL}};
const Segment SGI50469[] = {{&S74578[30], 6}, {NULL}};
const Segment SGI50470[] = {{&S74578[58], 3}, {NULL}};
const Segment SGI50471[] = {{&S74578[84], 5}, {NULL}};
const Segment SGI50472[] = {{&S74579[38], 8}, {NULL}};
const Segment SGI50473[] = {{&S74579[31], 6}, {NULL}};
const Segment SGI50474[] = {{&S74581[44], 7}, {NULL}};
const Segment SGI50475[] = {{&S74581[44], 21}, {NULL}};
const Segment SGI50476[] = {{&S74581[30], 6}, {NULL}};
const Segment SGI50477[] = {{&S74581[113], 3}, {NULL}};
const Segment SGI50478[] = {{&S74581[79], 5}, {NULL}};
const Segment SGI50479[] = {{&S74582[44], 7}, {NULL}};
const Segment SGI50480[] = {{&S74582[44], 21}, {NULL}};
const Segment SGI50481[] = {{&S74582[30], 6}, {NULL}};
const Segment SGI50482[] = {{&S74582[82], 4}, {NULL}};
const Segment SGI50483[] = {{&S74582[82], 3}, {NULL}};
const Segment SGI50484[] = {{&S74107[44], 7}, {NULL}};
const Segment SGI50485[] = {{&S74107[44], 18}, {NULL}};
const Segment SGI50486[] = {{&S74107[30], 6}, {NULL}};
const Segment SGI50487[] = {{&S74107[110], 3}, {NULL}};
const Segment SGI50488[] = {{&S74107[76], 5}, {NULL}};
const Segment SGI50489[] = {{&S74114[45], 20}, {NULL}};
const Segment SGI50490[] = {{&S74114[30], 6}, {NULL}};
const Segment SGI50491[] = {{&S74114[124], 3}, {NULL}};
const Segment SGI50492[] = {{&S74114[90], 5}, {NULL}};
const Segment SGI50493[] = {{&S74114[148], 13}, {NULL}};
const Segment SGI50494[] = {{&S89789[44], 10}, {NULL}};
const Segment SGI50495[] = {{&S89789[44], 21}, {NULL}};
const Segment SGI50496[] = {{&S89789[30], 6}, {NULL}};
const Segment SGI50497[] = {{&S89789[79], 6}, {NULL}};
const Segment SGI50498[] = {{&S89789[106], 13}, {NULL}};
const Segment SGI50499[] = {{&S74609[44], 10}, {NULL}};
const Segment SGI50500[] = {{&S74609[44], 27}, {NULL}};
const Segment SGI50501[] = {{&S74609[30], 6}, {NULL}};
const Segment SGI50502[] = {{&S74609[85], 6}, {NULL}};
const Segment SGI50503[] = {{&S74609[112], 13}, {NULL}};
const Segment SGI50504[] = {{&S74148[44], 14}, {NULL}};
const Segment SGI50505[] = {{&S74148[44], 40}, {NULL}};
const Segment SGI50506[] = {{&S74148[30], 6}, {NULL}};
const Segment SGI50507[] = {{&S74148[132], 3}, {NULL}};
const Segment SGI50508[] = {{&S74148[98], 5}, {NULL}};
const Segment SGI50509[] = {{&S74208[34], 11}, {NULL}};
const Segment SGI50510[] = {{&S74208[27], 6}, {NULL}};
const Segment SGI50511[] = {{&S74208[110], 2}, {NULL}};
const Segment SGI50512[] = {{&S74208[76], 6}, {NULL}};
const Segment SGI50513[] = {{&S74208[144], 8}, {NULL}};
const Segment SGI50514[] = {{&S89886[44], 14}, {NULL}};
const Segment SGI50515[] = {{&S89886[44], 25}, {NULL}};
const Segment SGI50516[] = {{&S89886[30], 6}, {NULL}};
const Segment SGI50517[] = {{&S89886[117], 3}, {NULL}};
const Segment SGI50518[] = {{&S89886[83], 5}, {NULL}};
const Segment SGI50519[] = {{&S74272[44], 7}, {NULL}};
const Segment SGI50520[] = {{&S74272[44], 19}, {NULL}};
const Segment SGI50521[] = {{&S74272[30], 6}, {NULL}};
const Segment SGI50522[] = {{&S74272[111], 3}, {NULL}};
const Segment SGI50523[] = {{&S74272[77], 5}, {NULL}};
const Segment SGI50524[] = {{&S74273[44], 7}, {NULL}};
const Segment SGI50525[] = {{&S74273[44], 19}, {NULL}};
const Segment SGI50526[] = {{&S74273[30], 6}, {NULL}};
const Segment SGI50527[] = {{&S74273[80], 6}, {NULL}};
const Segment SGI50528[] = {{&S74274[44], 7}, {NULL}};
const Segment SGI50529[] = {{&S74274[44], 19}, {NULL}};
const Segment SGI50530[] = {{&S74274[30], 6}, {NULL}};
const Segment SGI50531[] = {{&S74274[111], 3}, {NULL}};
const Segment SGI50532[] = {{&S74274[77], 5}, {NULL}};
const Segment SGI50533[] = {{&S74784[44], 7}, {NULL}};
const Segment SGI50534[] = {{&S74784[44], 25}, {NULL}};
const Segment SGI50535[] = {{&S74784[30], 6}, {NULL}};
const Segment SGI50536[] = {{&S74784[83], 6}, {NULL}};
const Segment SGI50537[] = {{&S74784[110], 13}, {NULL}};
const Segment SGI50538[] = {{&S74840[44], 11}, {NULL}};
const Segment SGI50539[] = {{&S74840[44], 22}, {NULL}};
const Segment SGI50540[] = {{&S74840[30], 6}, {NULL}};
const Segment SGI50541[] = {{&S74840[114], 3}, {NULL}};
const Segment SGI50542[] = {{&S74840[80], 5}, {NULL}};
const Segment SGI50543[] = {{&S74859[44], 7}, {NULL}};
const Segment SGI50544[] = {{&S74859[44], 18}, {NULL}};
const Segment SGI50545[] = {{&S74859[30], 6}, {NULL}};
const Segment SGI50546[] = {{&S74859[110], 3}, {NULL}};
const Segment SGI50547[] = {{&S74859[76], 5}, {NULL}};
const Segment SGI50548[] = {{&S78188[44], 14}, {NULL}};
const Segment SGI50549[] = {{&S78188[44], 25}, {NULL}};
const Segment SGI50550[] = {{&S78188[30], 6}, {NULL}};
const Segment SGI50551[] = {{&S78188[117], 3}, {NULL}};
const Segment SGI50552[] = {{&S78188[83], 5}, {NULL}};
const Segment SGI50553[] = {{&S78191[44], 5}, {NULL}};
const Segment SGI50554[] = {{&S78191[44], 22}, {NULL}};
const Segment SGI50555[] = {{&S78191[30], 6}, {NULL}};
const Segment SGI50556[] = {{&S78191[114], 3}, {NULL}};
const Segment SGI50557[] = {{&S78191[80], 5}, {NULL}};
const Segment SGI50558[] = {{&S74480[44], 6}, {NULL}};
const Segment SGI50559[] = {{&S74480[44], 17}, {NULL}};
const Segment SGI50560[] = {{&S74480[30], 6}, {NULL}};
const Segment SGI50561[] = {{&S74480[109], 3}, {NULL}};
const Segment SGI50562[] = {{&S74480[75], 5}, {NULL}};
const Segment SGI50563[] = {{&S74536[57], 10}, {NULL}};
const Segment SGI50564[] = {{&S74536[57], 27}, {NULL}};
const Segment SGI50565[] = {{&S74536[43], 6}, {NULL}};
const Segment SGI50566[] = {{&S74536[132], 3}, {NULL}};
const Segment SGI50567[] = {{&S74536[98], 5}, {NULL}};
const Segment SGI50568[] = {{&S74536[0], 12}, {NULL}};
const Segment SGI50569[] = {{&S74583[44], 7}, {NULL}};
const Segment SGI50570[] = {{&S74583[44], 21}, {NULL}};
const Segment SGI50571[] = {{&S74583[30], 6}, {NULL}};
const Segment SGI50572[] = {{&S74583[82], 4}, {NULL}};
const Segment SGI50573[] = {{&S74583[82], 3}, {NULL}};
const Segment SGI50574[] = {{&S74586[44], 7}, {NULL}};
const Segment SGI50575[] = {{&S74586[44], 21}, {NULL}};
const Segment SGI50576[] = {{&S74586[30], 6}, {NULL}};
const Segment SGI50577[] = {{&S74586[82], 6}, {NULL}};
const Segment SGI50578[] = {{&S74587[44], 7}, {NULL}};
const Segment SGI50579[] = {{&S74587[44], 21}, {NULL}};
const Segment SGI50580[] = {{&S74587[30], 6}, {NULL}};
const Segment SGI50581[] = {{&S74587[113], 3}, {NULL}};
const Segment SGI50582[] = {{&S74587[79], 5}, {NULL}};
const Segment SGI50583[] = {{&S74607[44], 10}, {NULL}};
const Segment SGI50584[] = {{&S74607[44], 27}, {NULL}};
const Segment SGI50585[] = {{&S74607[30], 6}, {NULL}};
const Segment SGI50586[] = {{&S74607[85], 6}, {NULL}};
const Segment SGI50587[] = {{&S74607[112], 13}, {NULL}};
const Segment SGI50588[] = {{&S74615[44], 21}, {NULL}};
const Segment SGI50589[] = {{&S74615[44], 32}, {NULL}};
const Segment SGI50590[] = {{&S74615[30], 6}, {NULL}};
const Segment SGI50591[] = {{&S74615[90], 6}, {NULL}};
const Segment SGI50592[] = {{&S74615[117], 13}, {NULL}};
const Segment SGI50593[] = {{&S74627[44], 14}, {NULL}};
const Segment SGI50594[] = {{&S74627[44], 24}, {NULL}};
const Segment SGI50595[] = {{&S74627[30], 6}, {NULL}};
const Segment SGI50596[] = {{&S74627[116], 3}, {NULL}};
const Segment SGI50597[] = {{&S74627[82], 5}, {NULL}};
const Segment SGI50598[] = {{&S74643[44], 13}, {NULL}};
const Segment SGI50599[] = {{&S74643[44], 24}, {NULL}};
const Segment SGI50600[] = {{&S74643[30], 6}, {NULL}};
const Segment SGI50601[] = {{&S74643[116], 3}, {NULL}};
const Segment SGI50602[] = {{&S74643[82], 5}, {NULL}};
const Segment SGI50603[] = {{&S74644[44], 13}, {NULL}};
const Segment SGI50604[] = {{&S74644[44], 24}, {NULL}};
const Segment SGI50605[] = {{&S74644[30], 6}, {NULL}};
const Segment SGI50606[] = {{&S74644[116], 3}, {NULL}};
const Segment SGI50607[] = {{&S74644[82], 5}, {NULL}};
const Segment SGI50608[] = {{&S75196[45], 10}, {NULL}};
const Segment SGI50609[] = {{&S75196[30], 6}, {NULL}};
const Segment SGI50610[] = {{&S75196[120], 3}, {NULL}};
const Segment SGI50611[] = {{&S75196[86], 5}, {NULL}};
const Segment SGI50612[] = {{&S75196[144], 13}, {NULL}};
const Segment SGI50613[] = {{&S84090[44], 8}, {NULL}};
const Segment SGI50614[] = {{&S84090[44], 25}, {NULL}};
const Segment SGI50615[] = {{&S84090[30], 6}, {NULL}};
const Segment SGI50616[] = {{&S84090[117], 3}, {NULL}};
const Segment SGI50617[] = {{&S84090[83], 5}, {NULL}};
const Segment SGI50618[] = {{&S74726[44], 10}, {NULL}};
const Segment SGI50619[] = {{&S74726[44], 21}, {NULL}};
const Segment SGI50620[] = {{&S74726[30], 6}, {NULL}};
const Segment SGI50621[] = {{&S74726[113], 3}, {NULL}};
const Segment SGI50622[] = {{&S74726[79], 5}, {NULL}};
const Segment SGI50623[] = {{&S74729[44], 10}, {NULL}};
const Segment SGI50624[] = {{&S74729[44], 30}, {NULL}};
const Segment SGI50625[] = {{&S74729[30], 6}, {NULL}};
const Segment SGI50626[] = {{&S74729[122], 3}, {NULL}};
const Segment SGI50627[] = {{&S74729[88], 5}, {NULL}};
const Segment SGI50628[] = {{&S75217[6], 4}, {NULL}};
const Segment SGI50629[] = {{&S75217[84], 5}, {NULL}};
const Segment SGI50630[] = {{&S90270[44], 11}, {NULL}};
const Segment SGI50631[] = {{&S90270[44], 36}, {NULL}};
const Segment SGI50632[] = {{&S90270[30], 6}, {NULL}};
const Segment SGI50633[] = {{&S90270[128], 3}, {NULL}};
const Segment SGI50634[] = {{&S90270[94], 5}, {NULL}};
const Segment SGI50635[] = {{&S90271[44], 11}, {NULL}};
const Segment SGI50636[] = {{&S90271[44], 36}, {NULL}};
const Segment SGI50637[] = {{&S90271[30], 6}, {NULL}};
const Segment SGI50638[] = {{&S90271[128], 3}, {NULL}};
const Segment SGI50639[] = {{&S90271[94], 5}, {NULL}};
const Segment SGI50640[] = {{&S74812[44], 22}, {NULL}};
const Segment SGI50641[] = {{&S74812[44], 33}, {NULL}};
const Segment SGI50642[] = {{&S74812[30], 6}, {NULL}};
const Segment SGI50643[] = {{&S74812[91], 6}, {NULL}};
const Segment SGI50644[] = {{&S74812[118], 13}, {NULL}};
const Segment SGI50645[] = {{&S78335[44], 10}, {NULL}};
const Segment SGI50646[] = {{&S78335[44], 27}, {NULL}};
const Segment SGI50647[] = {{&S78335[30], 6}, {NULL}};
const Segment SGI50648[] = {{&S78335[126], 3}, {NULL}};
const Segment SGI50649[] = {{&S78335[92], 5}, {NULL}};
const Segment SGI50650[] = {{&S78335[71], 8}, {NULL}};
const Segment SGI50651[] = {{&S78338[44], 10}, {NULL}};
const Segment SGI50652[] = {{&S78338[44], 27}, {NULL}};
const Segment SGI50653[] = {{&S78338[30], 6}, {NULL}};
const Segment SGI50654[] = {{&S78338[119], 3}, {NULL}};
const Segment SGI50655[] = {{&S78338[85], 5}, {NULL}};
const Segment SGI50656[] = {{&S78338[143], 7}, {NULL}};
const Segment SGI50657[] = {{&S74861[44], 7}, {NULL}};
const Segment SGI50658[] = {{&S74861[44], 18}, {NULL}};
const Segment SGI50659[] = {{&S74861[30], 6}, {NULL}};
const Segment SGI50660[] = {{&S74861[110], 3}, {NULL}};
const Segment SGI50661[] = {{&S74861[76], 5}, {NULL}};
const Segment SGI50662[] = {{&S74894[44], 21}, {NULL}};
const Segment SGI50663[] = {{&S74894[44], 32}, {NULL}};
const Segment SGI50664[] = {{&S74894[30], 6}, {NULL}};
const Segment SGI50665[] = {{&S74894[90], 6}, {NULL}};
const Segment SGI50666[] = {{&S74894[117], 13}, {NULL}};
const Segment SGI50667[] = {{&S74896[44], 11}, {NULL}};
const Segment SGI50668[] = {{&S74896[44], 22}, {NULL}};
const Segment SGI50669[] = {{&S74896[30], 6}, {NULL}};
const Segment SGI50670[] = {{&S74896[80], 6}, {NULL}};
const Segment SGI50671[] = {{&S74896[107], 13}, {NULL}};
const Segment SGI50672[] = {{&S90367[44], 15}, {NULL}};
const Segment SGI50673[] = {{&S90367[44], 27}, {NULL}};
const Segment SGI50674[] = {{&S90367[30], 6}, {NULL}};
const Segment SGI50675[] = {{&S90367[119], 3}, {NULL}};
const Segment SGI50676[] = {{&S90367[85], 5}, {NULL}};
const Segment SGI50677[] = {{&S78434[44], 7}, {NULL}};
const Segment SGI50678[] = {{&S78434[44], 18}, {NULL}};
const Segment SGI50679[] = {{&S78434[30], 6}, {NULL}};
const Segment SGI50680[] = {{&S78434[110], 3}, {NULL}};
const Segment SGI50681[] = {{&S78434[76], 5}, {NULL}};
const Segment SGI50682[] = {{&S75324[44], 7}, {NULL}};
const Segment SGI50683[] = {{&S75324[44], 24}, {NULL}};
const Segment SGI50684[] = {{&S75324[30], 6}, {NULL}};
const Segment SGI50685[] = {{&S75324[85], 4}, {NULL}};
const Segment SGI50686[] = {{&S75324[85], 3}, {NULL}};
const Segment SGI50687[] = {{&S75325[44], 37}, {NULL}};
const Segment SGI50688[] = {{&S75325[30], 6}, {NULL}};
const Segment SGI50689[] = {{&S75325[112], 5}, {NULL}};
const Segment SGI50690[] = {{&S75326[44], 7}, {NULL}};
const Segment SGI50691[] = {{&S75326[44], 24}, {NULL}};
const Segment SGI50692[] = {{&S75326[30], 6}, {NULL}};
const Segment SGI50693[] = {{&S75326[85], 6}, {NULL}};
const Segment SGI50694[] = {{&S75327[44], 7}, {NULL}};
const Segment SGI50695[] = {{&S75327[44], 24}, {NULL}};
const Segment SGI50696[] = {{&S75327[30], 6}, {NULL}};
const Segment SGI50697[] = {{&S75327[116], 3}, {NULL}};
const Segment SGI50698[] = {{&S75327[82], 5}, {NULL}};
const Segment SGI50699[] = {{&S75400[44], 11}, {NULL}};
const Segment SGI50700[] = {{&S75400[44], 27}, {NULL}};
const Segment SGI50701[] = {{&S75400[30], 6}, {NULL}};
const Segment SGI50702[] = {{&S75400[119], 3}, {NULL}};
const Segment SGI50703[] = {{&S75400[85], 5}, {NULL}};
const Segment SGI50704[] = {{&S75401[38], 11}, {NULL}};
const Segment SGI50705[] = {{&S75401[31], 6}, {NULL}};
const Segment SGI50706[] = {{&S75430[44], 7}, {NULL}};
const Segment SGI50707[] = {{&S75430[44], 25}, {NULL}};
const Segment SGI50708[] = {{&S75430[30], 6}, {NULL}};
const Segment SGI50709[] = {{&S75430[83], 6}, {NULL}};
const Segment SGI50710[] = {{&S75430[110], 13}, {NULL}};
const Segment SGI50711[] = {{&S75053[45], 20}, {NULL}};
const Segment SGI50712[] = {{&S75053[30], 6}, {NULL}};
const Segment SGI50713[] = {{&S75053[124], 3}, {NULL}};
const Segment SGI50714[] = {{&S75053[90], 5}, {NULL}};
const Segment SGI50715[] = {{&S75053[148], 13}, {NULL}};
const Segment SGI50716[] = {{&S90405[45], 12}, {NULL}};
const Segment SGI50717[] = {{&S90405[30], 6}, {NULL}};
const Segment SGI50718[] = {{&S90405[116], 3}, {NULL}};
const Segment SGI50719[] = {{&S90405[82], 5}, {NULL}};
const Segment SGI50720[] = {{&S90405[140], 13}, {NULL}};
const Segment SGI50721[] = {{&S90416[59], 12}, {NULL}};
const Segment SGI50722[] = {{&S90416[59], 23}, {NULL}};
const Segment SGI50723[] = {{&S90416[45], 6}, {NULL}};
const Segment SGI50724[] = {{&S90416[130], 3}, {NULL}};
const Segment SGI50725[] = {{&S90416[96], 5}, {NULL}};
const Segment SGI50726[] = {{&S90416[0], 14}, {NULL}};
const Segment SGI50727[] = {{&S78679[59], 12}, {NULL}};
const Segment SGI50728[] = {{&S78679[59], 23}, {NULL}};
const Segment SGI50729[] = {{&S78679[45], 6}, {NULL}};
const Segment SGI50730[] = {{&S78679[130], 3}, {NULL}};
const Segment SGI50731[] = {{&S78679[96], 5}, {NULL}};
const Segment SGI50732[] = {{&S78679[0], 14}, {NULL}};
const Segment SGI50733[] = {{&S75200[44], 9}, {NULL}};
const Segment SGI50734[] = {{&S75200[44], 30}, {NULL}};
const Segment SGI50735[] = {{&S75200[30], 6}, {NULL}};
const Segment SGI50736[] = {{&S75200[122], 3}, {NULL}};
const Segment SGI50737[] = {{&S75200[88], 5}, {NULL}};
const Segment SGI50738[] = {{&S83231[42], 17}, {NULL}};
const Segment SGI50739[] = {{&S83231[42], 28}, {NULL}};
const Segment SGI50740[] = {{&S83231[30], 4}, {NULL}};
const Segment SGI50741[] = {{&S83231[118], 3}, {NULL}};
const Segment SGI50742[] = {{&S83231[84], 5}, {NULL}};
const Segment SGI50743[] = {{&S78880[45], 10}, {NULL}};
const Segment SGI50744[] = {{&S78880[30], 6}, {NULL}};
const Segment SGI50745[] = {{&S78880[120], 3}, {NULL}};
const Segment SGI50746[] = {{&S78880[86], 5}, {NULL}};
const Segment SGI50747[] = {{&S78880[154], 3}, {NULL}};
const Segment SGI50748[] = {{&S75363[44], 10}, {NULL}};
const Segment SGI50749[] = {{&S75363[44], 27}, {NULL}};
const Segment SGI50750[] = {{&S75363[30], 6}, {NULL}};
const Segment SGI50751[] = {{&S75363[85], 6}, {NULL}};
const Segment SGI50752[] = {{&S75363[112], 13}, {NULL}};
const Segment SGI50753[] = {{&S75405[44], 11}, {NULL}};
const Segment SGI50754[] = {{&S75405[44], 27}, {NULL}};
const Segment SGI50755[] = {{&S75405[30], 6}, {NULL}};
const Segment SGI50756[] = {{&S75405[119], 3}, {NULL}};
const Segment SGI50757[] = {{&S75405[85], 5}, {NULL}};
const Segment SGI50758[] = {{&S75740[45], 11}, {NULL}};
const Segment SGI50759[] = {{&S75740[30], 6}, {NULL}};
const Segment SGI50760[] = {{&S75740[121], 3}, {NULL}};
const Segment SGI50761[] = {{&S75740[87], 5}, {NULL}};
const Segment SGI50762[] = {{&S75740[145], 13}, {NULL}};
const Segment SGI50763[] = {{&S75741[45], 10}, {NULL}};
const Segment SGI50764[] = {{&S75741[30], 6}, {NULL}};
const Segment SGI50765[] = {{&S75741[120], 3}, {NULL}};
const Segment SGI50766[] = {{&S75741[86], 5}, {NULL}};
const Segment SGI50767[] = {{&S75741[144], 13}, {NULL}};
const Segment SGI50768[] = {{&S90752[44], 7}, {NULL}};
const Segment SGI50769[] = {{&S90752[44], 19}, {NULL}};
const Segment SGI50770[] = {{&S90752[30], 6}, {NULL}};
const Segment SGI50771[] = {{&S90752[111], 3}, {NULL}};
const Segment SGI50772[] = {{&S90752[77], 5}, {NULL}};
const Segment SGI50773[] = {{&S75838[44], 11}, {NULL}};
const Segment SGI50774[] = {{&S75838[44], 28}, {NULL}};
const Segment SGI50775[] = {{&S75838[30], 6}, {NULL}};
const Segment SGI50776[] = {{&S75838[150], 4}, {NULL}};
const Segment SGI50777[] = {{&S75838[176], 3}, {NULL}};
const Segment SGI50778[] = {{&S75838[120], 3}, {&S75838[225], 5}, {NULL}};
const Segment SGI50779[] = {{&S75838[86], 5}, {NULL}};
const Segment SGI50780[] = {{&S79000[47], 10}, {NULL}};
const Segment SGI50781[] = {{&S79000[47], 25}, {NULL}};
const Segment SGI50782[] = {{&S79000[33], 6}, {NULL}};
const Segment SGI50783[] = {{&S79000[127], 3}, {NULL}};
const Segment SGI50784[] = {{&S79000[93], 5}, {NULL}};
const Segment SGI50785[] = {{&S79000[72], 8}, {NULL}};
const Segment SGI50786[] = {{&S79001[47], 10}, {NULL}};
const Segment SGI50787[] = {{&S79001[47], 26}, {NULL}};
const Segment SGI50788[] = {{&S79001[33], 6}, {NULL}};
const Segment SGI50789[] = {{&S79001[128], 3}, {NULL}};
const Segment SGI50790[] = {{&S79001[94], 5}, {NULL}};
const Segment SGI50791[] = {{&S79001[73], 8}, {NULL}};
const Segment SGI50792[] = {{&S79002[47], 10}, {NULL}};
const Segment SGI50793[] = {{&S79002[47], 26}, {NULL}};
const Segment SGI50794[] = {{&S79002[33], 6}, {NULL}};
const Segment SGI50795[] = {{&S79002[128], 3}, {NULL}};
const Segment SGI50796[] = {{&S79002[94], 5}, {NULL}};
const Segment SGI50797[] = {{&S79002[73], 8}, {NULL}};
const Segment SGI50798[] = {{&S79003[47], 10}, {NULL}};
const Segment SGI50799[] = {{&S79003[47], 26}, {NULL}};
const Segment SGI50800[] = {{&S79003[33], 6}, {NULL}};
const Segment SGI50801[] = {{&S75550[44], 14}, {NULL}};
const Segment SGI50802[] = {{&S79003[128], 3}, {NULL}};
const Segment SGI50803[] = {{&S75550[44], 24}, {NULL}};
const Segment SGI50804[] = {{&S75550[30], 6}, {NULL}};
const Segment SGI50805[] = {{&S79003[94], 5}, {NULL}};
const Segment SGI50806[] = {{&S75550[116], 3}, {NULL}};
const Segment SGI50807[] = {{&S79003[73], 8}, {NULL}};
const Segment SGI50808[] = {{&S75550[82], 5}, {NULL}};
const Segment SGI50809[] = {{&S75594[59], 10}, {NULL}};
const Segment SGI50810[] = {{&S75594[59], 27}, {NULL}};
const Segment SGI50811[] = {{&S75594[45], 6}, {NULL}};
const Segment SGI50812[] = {{&S75594[134], 3}, {NULL}};
const Segment SGI50813[] = {{&S75594[100], 5}, {NULL}};
const Segment SGI50814[] = {{&S75594[0], 14}, {NULL}};
const Segment SGI50815[] = {{&S75630[44], 18}, {NULL}};
const Segment SGI50816[] = {{&S75630[44], 47}, {NULL}};
const Segment SGI50817[] = {{&S75630[30], 6}, {NULL}};
const Segment SGI50818[] = {{&S75630[139], 3}, {NULL}};
const Segment SGI50819[] = {{&S75630[105], 5}, {NULL}};
const Segment SGI50820[] = {{&S75631[44], 18}, {NULL}};
const Segment SGI50821[] = {{&S75631[44], 47}, {NULL}};
const Segment SGI50822[] = {{&S75631[30], 6}, {NULL}};
const Segment SGI50823[] = {{&S75631[139], 3}, {NULL}};
const Segment SGI50824[] = {{&S75631[105], 5}, {NULL}};
const Segment SGI50825[] = {{&S75632[44], 18}, {NULL}};
const Segment SGI50826[] = {{&S75632[44], 47}, {NULL}};
const Segment SGI50827[] = {{&S75632[30], 6}, {NULL}};
const Segment SGI50828[] = {{&S75632[139], 3}, {NULL}};
const Segment SGI50829[] = {{&S75632[105], 5}, {NULL}};
const Segment SGI50830[] = {{&S75633[44], 18}, {NULL}};
const Segment SGI50831[] = {{&S75633[44], 47}, {NULL}};
const Segment SGI50832[] = {{&S75633[30], 6}, {NULL}};
const Segment SGI50833[] = {{&S75633[139], 3}, {NULL}};
const Segment SGI50834[] = {{&S75633[105], 5}, {NULL}};
const Segment SGI50835[] = {{&S75634[44], 18}, {NULL}};
const Segment SGI50836[] = {{&S75634[44], 47}, {NULL}};
const Segment SGI50837[] = {{&S75634[30], 6}, {NULL}};
const Segment SGI50838[] = {{&S75634[139], 3}, {NULL}};
const Segment SGI50839[] = {{&S75634[105], 5}, {NULL}};
const Segment SGI50840[] = {{&S75970[44], 6}, {NULL}};
const Segment SGI50841[] = {{&S75970[44], 23}, {NULL}};
const Segment SGI50842[] = {{&S75970[30], 6}, {NULL}};
const Segment SGI50843[] = {{&S75970[115], 3}, {NULL}};
const Segment SGI50844[] = {{&S75970[81], 5}, {NULL}};
const Segment SGI50845[] = {{&S75971[38], 6}, {NULL}};
const Segment SGI50846[] = {{&S75971[31], 6}, {NULL}};
const Segment SGI50847[] = {{&S75975[44], 7}, {NULL}};
const Segment SGI50848[] = {{&S75975[44], 18}, {NULL}};
const Segment SGI50849[] = {{&S75975[30], 6}, {NULL}};
const Segment SGI50850[] = {{&S75975[79], 4}, {NULL}};
const Segment SGI50851[] = {{&S75975[79], 3}, {NULL}};
const Segment SGI50852[] = {{&S75976[44], 7}, {NULL}};
const Segment SGI50853[] = {{&S75976[44], 18}, {NULL}};
const Segment SGI50854[] = {{&S75976[30], 6}, {NULL}};
const Segment SGI50855[] = {{&S75976[110], 3}, {NULL}};
const Segment SGI50856[] = {{&S75976[76], 5}, {NULL}};
const Segment SGI50857[] = {{&S75977[44], 7}, {NULL}};
const Segment SGI50858[] = {{&S75977[44], 18}, {NULL}};
const Segment SGI50859[] = {{&S75977[30], 6}, {NULL}};
const Segment SGI50860[] = {{&S75977[79], 6}, {NULL}};
const Segment SGI50861[] = {{&S75978[38], 7}, {NULL}};
const Segment SGI50862[] = {{&S75978[31], 6}, {NULL}};
const Segment SGI50863[] = {{&S75738[45], 10}, {NULL}};
const Segment SGI50864[] = {{&S75738[30], 6}, {NULL}};
const Segment SGI50865[] = {{&S75738[120], 3}, {NULL}};
const Segment SGI50866[] = {{&S75738[86], 5}, {NULL}};
const Segment SGI50867[] = {{&S75738[144], 13}, {NULL}};
const Segment SGI50868[] = {{&S90969[45], 12}, {NULL}};
const Segment SGI50869[] = {{&S90969[30], 6}, {NULL}};
const Segment SGI50870[] = {{&S90969[117], 3}, {NULL}};
const Segment SGI50871[] = {{&S90969[83], 5}, {NULL}};
const Segment SGI50872[] = {{&S90969[141], 13}, {NULL}};
const Segment SGI50873[] = {{&S75812[44], 11}, {NULL}};
const Segment SGI50874[] = {{&S75812[44], 35}, {NULL}};
const Segment SGI50875[] = {{&S75812[30], 6}, {NULL}};
const Segment SGI50876[] = {{&S75812[127], 3}, {NULL}};
const Segment SGI50877[] = {{&S75812[93], 5}, {NULL}};
const Segment SGI50878[] = {{&S75813[44], 11}, {NULL}};
const Segment SGI50879[] = {{&S75813[44], 35}, {NULL}};
const Segment SGI50880[] = {{&S75813[30], 6}, {NULL}};
const Segment SGI50881[] = {{&S75813[127], 3}, {NULL}};
const Segment SGI50882[] = {{&S75813[93], 5}, {NULL}};
const Segment SGI50883[] = {{&S75814[44], 11}, {NULL}};
const Segment SGI50884[] = {{&S75814[44], 35}, {NULL}};
const Segment SGI50885[] = {{&S75814[30], 6}, {NULL}};
const Segment SGI50886[] = {{&S75814[127], 3}, {NULL}};
const Segment SGI50887[] = {{&S75814[93], 5}, {NULL}};
const Segment SGI50888[] = {{&S76171[44], 9}, {NULL}};
const Segment SGI50889[] = {{&S76171[44], 20}, {NULL}};
const Segment SGI50890[] = {{&S76171[30], 6}, {NULL}};
const Segment SGI50891[] = {{&S76171[112], 3}, {NULL}};
const Segment SGI50892[] = {{&S76171[78], 5}, {NULL}};
const Segment SGI50893[] = {{&S76172[44], 9}, {NULL}};
const Segment SGI50894[] = {{&S76172[44], 20}, {NULL}};
const Segment SGI50895[] = {{&S76172[30], 6}, {NULL}};
const Segment SGI50896[] = {{&S76172[112], 3}, {NULL}};
const Segment SGI50897[] = {{&S76172[78], 5}, {NULL}};
const Segment SGI50898[] = {{&S76173[44], 9}, {NULL}};
const Segment SGI50899[] = {{&S76173[44], 20}, {NULL}};
const Segment SGI50900[] = {{&S76173[30], 6}, {NULL}};
const Segment SGI50901[] = {{&S76173[112], 3}, {NULL}};
const Segment SGI50902[] = {{&S76173[78], 5}, {NULL}};
const Segment SGI50903[] = {{&S76174[44], 9}, {NULL}};
const Segment SGI50904[] = {{&S76174[44], 20}, {NULL}};
const Segment SGI50905[] = {{&S76174[30], 6}, {NULL}};
const Segment SGI50906[] = {{&S76174[112], 3}, {NULL}};
const Segment SGI50907[] = {{&S76174[78], 5}, {NULL}};
const Segment SGI50908[] = {{&S76175[44], 9}, {NULL}};
const Segment SGI50909[] = {{&S76175[44], 20}, {NULL}};
const Segment SGI50910[] = {{&S76175[30], 6}, {NULL}};
const Segment SGI50911[] = {{&S76175[112], 3}, {NULL}};
const Segment SGI50912[] = {{&S76175[78], 5}, {NULL}};
const Segment SGI50913[] = {{&S76176[44], 9}, {NULL}};
const Segment SGI50914[] = {{&S76176[44], 20}, {NULL}};
const Segment SGI50915[] = {{&S76176[30], 6}, {NULL}};
const Segment SGI50916[] = {{&S76176[112], 3}, {NULL}};
const Segment SGI50917[] = {{&S76176[78], 5}, {NULL}};
const Segment SGI50918[] = {{&S79324[44], 10}, {NULL}};
const Segment SGI50919[] = {{&S79324[44], 27}, {NULL}};
const Segment SGI50920[] = {{&S79324[30], 6}, {NULL}};
const Segment SGI50921[] = {{&S79324[149], 4}, {NULL}};
const Segment SGI50922[] = {{&S79324[175], 3}, {NULL}};
const Segment SGI50923[] = {{&S79324[119], 3}, {&S79324[224], 5}, {NULL}};
const Segment SGI50924[] = {{&S79324[85], 5}, {NULL}};
const Segment SGI50925[] = {{&S79327[45], 18}, {NULL}};
const Segment SGI50926[] = {{&S79327[30], 6}, {NULL}};
const Segment SGI50927[] = {{&S79327[121], 3}, {NULL}};
const Segment SGI50928[] = {{&S79327[88], 5}, {NULL}};
const Segment SGI50929[] = {{&S79327[155], 3}, {NULL}};
const Segment SGI50930[] = {{&S76304[44], 10}, {NULL}};
const Segment SGI50931[] = {{&S76304[44], 27}, {NULL}};
const Segment SGI50932[] = {{&S76304[30], 6}, {NULL}};
const Segment SGI50933[] = {{&S76304[85], 6}, {NULL}};
const Segment SGI50934[] = {{&S76304[112], 13}, {NULL}};
const Segment SGI50935[] = {{&S76021[34], 10}, {NULL}};
const Segment SGI50936[] = {{&S76021[27], 6}, {NULL}};
const Segment SGI50937[] = {{&S76021[109], 2}, {NULL}};
const Segment SGI50938[] = {{&S76021[75], 6}, {NULL}};
const Segment SGI50939[] = {{&S76021[143], 8}, {NULL}};
const Segment SGI50940[] = {{&S76330[59], 10}, {NULL}};
const Segment SGI50941[] = {{&S76330[59], 27}, {NULL}};
const Segment SGI50942[] = {{&S76330[45], 6}, {NULL}};
const Segment SGI50943[] = {{&S76330[134], 3}, {NULL}};
const Segment SGI50944[] = {{&S76330[100], 5}, {NULL}};
const Segment SGI50945[] = {{&S76330[0], 14}, {NULL}};
const Segment SGI50946[] = {{&S76336[59], 11}, {NULL}};
const Segment SGI50947[] = {{&S76336[59], 28}, {NULL}};
const Segment SGI50948[] = {{&S76336[45], 6}, {NULL}};
const Segment SGI50949[] = {{&S76336[135], 3}, {NULL}};
const Segment SGI50950[] = {{&S76336[101], 5}, {NULL}};
const Segment SGI50951[] = {{&S76336[0], 14}, {NULL}};
const Segment SGI50952[] = {{&S76445[44], 6}, {NULL}};
const Segment SGI50953[] = {{&S76445[44], 16}, {NULL}};
const Segment SGI50954[] = {{&S76445[30], 6}, {NULL}};
const Segment SGI50955[] = {{&S76445[108], 3}, {NULL}};
const Segment SGI50956[] = {{&S76445[74], 5}, {NULL}};
const Segment SGI50957[] = {{&S76447[44], 10}, {NULL}};
const Segment SGI50958[] = {{&S76447[44], 20}, {NULL}};
const Segment SGI50959[] = {{&S76447[30], 6}, {NULL}};
const Segment SGI50960[] = {{&S76447[112], 3}, {NULL}};
const Segment SGI50961[] = {{&S76447[78], 5}, {NULL}};
const Segment SGI50962[] = {{&S76490[44], 25}, {NULL}};
const Segment SGI50963[] = {{&S76490[30], 6}, {NULL}};
const Segment SGI50964[] = {{&S76490[135], 3}, {NULL}};
const Segment SGI50965[] = {{&S76490[101], 5}, {NULL}};
const Segment SGI50966[] = {{&S76492[44], 25}, {NULL}};
const Segment SGI50967[] = {{&S76492[30], 6}, {NULL}};
const Segment SGI50968[] = {{&S76492[135], 3}, {NULL}};
const Segment SGI50969[] = {{&S76492[101], 5}, {NULL}};
const Segment SGI50970[] = {{&S76493[44], 12}, {NULL}};
const Segment SGI50971[] = {{&S76493[44], 23}, {NULL}};
const Segment SGI50972[] = {{&S76493[30], 6}, {NULL}};
const Segment SGI50973[] = {{&S76493[84], 4}, {NULL}};
const Segment SGI50974[] = {{&S76493[84], 3}, {NULL}};
const Segment SGI50975[] = {{&S76496[44], 12}, {NULL}};
const Segment SGI50976[] = {{&S76496[44], 23}, {NULL}};
const Segment SGI50977[] = {{&S76496[30], 6}, {NULL}};
const Segment SGI50978[] = {{&S76496[84], 6}, {NULL}};
const Segment SGI50979[] = {{&S79482[45], 11}, {NULL}};
const Segment SGI50980[] = {{&S79482[30], 6}, {NULL}};
const Segment SGI50981[] = {{&S79482[115], 3}, {NULL}};
const Segment SGI50982[] = {{&S79482[81], 5}, {NULL}};
const Segment SGI50983[] = {{&S79482[149], 3}, {NULL}};
const Segment SGI50984[] = {{&S76559[44], 6}, {NULL}};
const Segment SGI50985[] = {{&S76559[44], 17}, {NULL}};
const Segment SGI50986[] = {{&S76559[30], 6}, {NULL}};
const Segment SGI50987[] = {{&S76559[109], 3}, {NULL}};
const Segment SGI50988[] = {{&S76559[75], 5}, {NULL}};
const Segment SGI50989[] = {{&S79537[45], 10}, {NULL}};
const Segment SGI50990[] = {{&S79537[30], 6}, {NULL}};
const Segment SGI50991[] = {{&S79537[120], 3}, {NULL}};
const Segment SGI50992[] = {{&S79537[86], 5}, {NULL}};
const Segment SGI50993[] = {{&S79537[154], 3}, {NULL}};
const Segment SGI50994[] = {{&S76609[44], 17}, {NULL}};
const Segment SGI50995[] = {{&S76609[30], 6}, {NULL}};
const Segment SGI50996[] = {{&S76609[109], 3}, {NULL}};
const Segment SGI50997[] = {{&S76609[75], 5}, {NULL}};
const Segment SGI50998[] = {{&S76239[44], 6}, {NULL}};
const Segment SGI50999[] = {{&S76239[44], 23}, {NULL}};
const Segment SGI51000[] = {{&S76239[30], 6}, {NULL}};
const Segment SGI51001[] = {{&S76239[122], 3}, {NULL}};
const Segment SGI51002[] = {{&S76239[88], 5}, {NULL}};
const Segment SGI51003[] = {{&S76239[67], 8}, {NULL}};
const Segment SGI51004[] = {{&S76240[44], 6}, {NULL}};
const Segment SGI51005[] = {{&S76240[44], 23}, {NULL}};
const Segment SGI51006[] = {{&S76240[30], 6}, {NULL}};
const Segment SGI51007[] = {{&S76240[115], 3}, {NULL}};
const Segment SGI51008[] = {{&S76240[81], 5}, {NULL}};
const Segment SGI51009[] = {{&S76240[139], 7}, {NULL}};
const Segment SGI51010[] = {{&S76693[44], 14}, {NULL}};
const Segment SGI51011[] = {{&S76693[44], 31}, {NULL}};
const Segment SGI51012[] = {{&S76693[30], 6}, {NULL}};
const Segment SGI51013[] = {{&S76693[123], 3}, {NULL}};
const Segment SGI51014[] = {{&S76693[89], 5}, {NULL}};
const Segment SGI51015[] = {{&S79691[31], 9}, {NULL}};
const Segment SGI51016[] = {{&S79691[55], 3}, {NULL}};
const Segment SGI51017[] = {{&S91536[59], 9}, {NULL}};
const Segment SGI51018[] = {{&S91536[59], 20}, {NULL}};
const Segment SGI51019[] = {{&S91536[45], 6}, {NULL}};
const Segment SGI51020[] = {{&S91536[127], 3}, {NULL}};
const Segment SGI51021[] = {{&S91536[93], 5}, {NULL}};
const Segment SGI51022[] = {{&S76780[44], 14}, {NULL}};
const Segment SGI51023[] = {{&S76780[44], 25}, {NULL}};
const Segment SGI51024[] = {{&S76780[30], 6}, {NULL}};
const Segment SGI51025[] = {{&S76780[117], 3}, {NULL}};
const Segment SGI51026[] = {{&S76780[83], 5}, {NULL}};
const Segment SGI51027[] = {{&S76781[44], 5}, {NULL}};
const Segment SGI51028[] = {{&S76781[44], 16}, {NULL}};
const Segment SGI51029[] = {{&S76781[30], 6}, {NULL}};
const Segment SGI51030[] = {{&S76781[108], 3}, {NULL}};
const Segment SGI51031[] = {{&S76781[74], 5}, {NULL}};
const Segment SGI51032[] = {{&S76797[44], 10}, {NULL}};
const Segment SGI51033[] = {{&S76797[44], 21}, {NULL}};
const Segment SGI51034[] = {{&S76797[30], 6}, {NULL}};
const Segment SGI51035[] = {{&S76797[113], 3}, {NULL}};
const Segment SGI51036[] = {{&S76797[79], 5}, {NULL}};
const Segment SGI51037[] = {{&S76798[44], 6}, {NULL}};
const Segment SGI51038[] = {{&S76798[44], 17}, {NULL}};
const Segment SGI51039[] = {{&S76798[30], 6}, {NULL}};
const Segment SGI51040[] = {{&S76798[78], 4}, {NULL}};
const Segment SGI51041[] = {{&S76798[78], 3}, {NULL}};
const Segment SGI51042[] = {{&S76799[44], 6}, {NULL}};
const Segment SGI51043[] = {{&S76799[44], 17}, {NULL}};
const Segment SGI51044[] = {{&S76799[30], 6}, {NULL}};
const Segment SGI51045[] = {{&S76799[78], 6}, {NULL}};
const Segment SGI51046[] = {{&S76800[38], 6}, {NULL}};
const Segment SGI51047[] = {{&S76800[31], 6}, {NULL}};
const Segment SGI51048[] = {{&S76331[59], 11}, {NULL}};
const Segment SGI51049[] = {{&S76331[59], 28}, {NULL}};
const Segment SGI51050[] = {{&S76331[45], 6}, {NULL}};
const Segment SGI51051[] = {{&S76331[135], 3}, {NULL}};
const Segment SGI51052[] = {{&S76331[101], 5}, {NULL}};
const Segment SGI51053[] = {{&S76331[0], 14}, {NULL}};
const Segment SGI51054[] = {{&S76332[59], 10}, {NULL}};
const Segment SGI51055[] = {{&S76332[59], 27}, {NULL}};
const Segment SGI51056[] = {{&S76332[45], 6}, {NULL}};
const Segment SGI51057[] = {{&S76332[134], 3}, {NULL}};
const Segment SGI51058[] = {{&S76332[100], 5}, {NULL}};
const Segment SGI51059[] = {{&S76332[0], 14}, {NULL}};
const Segment SGI51060[] = {{&S76388[44], 11}, {NULL}};
const Segment SGI51061[] = {{&S76388[44], 22}, {NULL}};
const Segment SGI51062[] = {{&S76388[30], 6}, {NULL}};
const Segment SGI51063[] = {{&S76388[114], 3}, {NULL}};
const Segment SGI51064[] = {{&S76388[80], 5}, {NULL}};
const Segment SGI51065[] = {{&S76389[44], 10}, {NULL}};
const Segment SGI51066[] = {{&S76389[44], 22}, {NULL}};
const Segment SGI51067[] = {{&S76389[30], 6}, {NULL}};
const Segment SGI51068[] = {{&S76389[50], 3}, {NULL}};
const Segment SGI51069[] = {{&S76389[80], 5}, {NULL}};
const Segment SGI51070[] = {{&S79868[44], 10}, {NULL}};
const Segment SGI51071[] = {{&S79868[44], 27}, {NULL}};
const Segment SGI51072[] = {{&S79868[30], 6}, {NULL}};
const Segment SGI51073[] = {{&S79868[85], 6}, {NULL}};
const Segment SGI51074[] = {{&S79868[122], 3}, {NULL}};
const Segment SGI51075[] = {{&S76437[44], 22}, {NULL}};
const Segment SGI51076[] = {{&S76437[44], 33}, {NULL}};
const Segment SGI51077[] = {{&S76437[30], 6}, {NULL}};
const Segment SGI51078[] = {{&S76437[125], 3}, {NULL}};
const Segment SGI51079[] = {{&S76437[91], 5}, {NULL}};
const Segment SGI51080[] = {{&S76439[44], 12}, {NULL}};
const Segment SGI51081[] = {{&S76439[44], 23}, {NULL}};
const Segment SGI51082[] = {{&S76439[30], 6}, {NULL}};
const Segment SGI51083[] = {{&S76439[115], 3}, {NULL}};
const Segment SGI51084[] = {{&S76439[81], 5}, {NULL}};
const Segment SGI51085[] = {{&S76446[44], 6}, {NULL}};
const Segment SGI51086[] = {{&S76446[44], 16}, {NULL}};
const Segment SGI51087[] = {{&S76446[30], 6}, {NULL}};
const Segment SGI51088[] = {{&S76446[108], 3}, {NULL}};
const Segment SGI51089[] = {{&S76446[74], 5}, {NULL}};
const Segment SGI51090[] = {{&S76449[44], 9}, {NULL}};
const Segment SGI51091[] = {{&S76449[44], 19}, {NULL}};
const Segment SGI51092[] = {{&S76449[30], 6}, {NULL}};
const Segment SGI51093[] = {{&S76449[111], 3}, {NULL}};
const Segment SGI51094[] = {{&S76449[77], 5}, {NULL}};
const Segment SGI51095[] = {{&S76452[44], 9}, {NULL}};
const Segment SGI51096[] = {{&S76452[44], 33}, {NULL}};
const Segment SGI51097[] = {{&S76452[30], 6}, {NULL}};
const Segment SGI51098[] = {{&S76452[125], 3}, {NULL}};
const Segment SGI51099[] = {{&S76452[91], 5}, {NULL}};
const Segment SGI51100[] = {{&S76454[44], 7}, {NULL}};
const Segment SGI51101[] = {{&S76454[44], 18}, {NULL}};
const Segment SGI51102[] = {{&S76454[30], 6}, {NULL}};
const Segment SGI51103[] = {{&S76454[110], 3}, {NULL}};
const Segment SGI51104[] = {{&S76454[76], 5}, {NULL}};
const Segment SGI51105[] = {{&S76455[38], 7}, {NULL}};
const Segment SGI51106[] = {{&S76455[31], 6}, {NULL}};
const Segment SGI51107[] = {{&S76491[44], 25}, {NULL}};
const Segment SGI51108[] = {{&S76491[30], 6}, {NULL}};
const Segment SGI51109[] = {{&S76491[135], 3}, {NULL}};
const Segment SGI51110[] = {{&S76491[101], 5}, {NULL}};
const Segment SGI51111[] = {{&S76494[44], 12}, {NULL}};
const Segment SGI51112[] = {{&S76494[44], 23}, {NULL}};
const Segment SGI51113[] = {{&S76494[30], 6}, {NULL}};
const Segment SGI51114[] = {{&S76494[84], 4}, {NULL}};
const Segment SGI51115[] = {{&S76494[84], 3}, {NULL}};
const Segment SGI51116[] = {{&S76495[44], 25}, {NULL}};
const Segment SGI51117[] = {{&S76495[30], 6}, {NULL}};
const Segment SGI51118[] = {{&S76495[135], 3}, {NULL}};
const Segment SGI51119[] = {{&S76495[101], 5}, {NULL}};
const Segment SGI51120[] = {{&S76497[44], 12}, {NULL}};
const Segment SGI51121[] = {{&S76497[44], 23}, {NULL}};
const Segment SGI51122[] = {{&S76497[30], 6}, {NULL}};
const Segment SGI51123[] = {{&S76497[84], 6}, {NULL}};
const Segment SGI51124[] = {{&S76498[44], 25}, {NULL}};
const Segment SGI51125[] = {{&S76498[30], 6}, {NULL}};
const Segment SGI51126[] = {{&S76498[135], 3}, {NULL}};
const Segment SGI51127[] = {{&S76498[101], 5}, {NULL}};
const Segment SGI51128[] = {{&S80006[59], 10}, {NULL}};
const Segment SGI51129[] = {{&S80006[59], 27}, {NULL}};
const Segment SGI51130[] = {{&S80006[45], 6}, {NULL}};
const Segment SGI51131[] = {{&S80006[134], 3}, {NULL}};
const Segment SGI51132[] = {{&S80006[100], 5}, {NULL}};
const Segment SGI51133[] = {{&S80006[0], 12}, {NULL}};
const Segment SGI51134[] = {{&S76539[44], 10}, {NULL}};
const Segment SGI51135[] = {{&S76539[44], 27}, {NULL}};
const Segment SGI51136[] = {{&S76539[30], 6}, {NULL}};
const Segment SGI51137[] = {{&S76539[85], 6}, {NULL}};
const Segment SGI51138[] = {{&S76539[112], 13}, {NULL}};
const Segment SGI51139[] = {{&S76608[44], 13}, {NULL}};
const Segment SGI51140[] = {{&S76608[44], 24}, {NULL}};
const Segment SGI51141[] = {{&S76608[30], 6}, {NULL}};
const Segment SGI51142[] = {{&S76608[85], 4}, {NULL}};
const Segment SGI51143[] = {{&S76608[85], 3}, {NULL}};
const Segment SGI51144[] = {{&S76610[44], 13}, {NULL}};
const Segment SGI51145[] = {{&S76610[44], 24}, {NULL}};
const Segment SGI51146[] = {{&S76610[30], 6}, {NULL}};
const Segment SGI51147[] = {{&S76610[116], 3}, {NULL}};
const Segment SGI51148[] = {{&S76610[82], 5}, {NULL}};
const Segment SGI51149[] = {{&S76611[44], 13}, {NULL}};
const Segment SGI51150[] = {{&S76611[44], 24}, {NULL}};
const Segment SGI51151[] = {{&S76611[30], 6}, {NULL}};
const Segment SGI51152[] = {{&S76611[85], 6}, {NULL}};
const Segment SGI51153[] = {{&S80115[44], 14}, {NULL}};
const Segment SGI51154[] = {{&S80115[44], 25}, {NULL}};
const Segment SGI51155[] = {{&S80115[30], 6}, {NULL}};
const Segment SGI51156[] = {{&S80115[83], 6}, {NULL}};
const Segment SGI51157[] = {{&S80115[120], 3}, {NULL}};
const Segment SGI51158[] = {{&S80162[44], 10}, {NULL}};
const Segment SGI51159[] = {{&S80162[44], 27}, {NULL}};
const Segment SGI51160[] = {{&S80162[30], 6}, {NULL}};
const Segment SGI51161[] = {{&S80162[119], 3}, {NULL}};
const Segment SGI51162[] = {{&S80162[85], 5}, {NULL}};
const Segment SGI51163[] = {{&S80162[143], 11}, {NULL}};
const Segment SGI51164[] = {{&S76703[44], 9}, {NULL}};
const Segment SGI51165[] = {{&S76703[44], 20}, {NULL}};
const Segment SGI51166[] = {{&S76703[30], 6}, {NULL}};
const Segment SGI51167[] = {{&S76703[112], 3}, {NULL}};
const Segment SGI51168[] = {{&S76703[78], 5}, {NULL}};
const Segment SGI51169[] = {{&S76704[44], 9}, {NULL}};
const Segment SGI51170[] = {{&S76704[44], 20}, {NULL}};
const Segment SGI51171[] = {{&S76704[30], 6}, {NULL}};
const Segment SGI51172[] = {{&S76704[81], 6}, {NULL}};
const Segment SGI51173[] = {{&S92079[44], 9}, {NULL}};
const Segment SGI51174[] = {{&S92079[44], 20}, {NULL}};
const Segment SGI51175[] = {{&S92079[30], 6}, {NULL}};
const Segment SGI51176[] = {{&S92079[112], 3}, {NULL}};
const Segment SGI51177[] = {{&S92079[78], 5}, {NULL}};
const Segment SGI51178[] = {{&S92080[41], 9}, {NULL}};
const Segment SGI51179[] = {{&S92080[30], 6}, {NULL}};
const Segment SGI51180[] = {{&S92080[109], 3}, {NULL}};
const Segment SGI51181[] = {{&S92080[75], 5}, {NULL}};
const Segment SGI51182[] = {{&S76789[44], 13}, {NULL}};
const Segment SGI51183[] = {{&S76789[44], 30}, {NULL}};
const Segment SGI51184[] = {{&S76789[30], 6}, {NULL}};
const Segment SGI51185[] = {{&S76789[88], 6}, {NULL}};
const Segment SGI51186[] = {{&S76789[115], 13}, {NULL}};
const Segment SGI51187[] = {{&S92125[44], 6}, {NULL}};
const Segment SGI51188[] = {{&S92125[44], 46}, {NULL}};
const Segment SGI51189[] = {{&S92125[30], 6}, {NULL}};
const Segment SGI51190[] = {{&S92125[138], 3}, {NULL}};
const Segment SGI51191[] = {{&S92125[104], 5}, {NULL}};
const Segment SGI51192[] = {{&S77228[45], 10}, {NULL}};
const Segment SGI51193[] = {{&S77228[30], 6}, {NULL}};
const Segment SGI51194[] = {{&S77228[120], 3}, {NULL}};
const Segment SGI51195[] = {{&S77228[86], 5}, {NULL}};
const Segment SGI51196[] = {{&S77228[144], 13}, {NULL}};
const Segment SGI51197[] = {{&S76881[97], 6}, {NULL}};
const Segment SGI51198[] = {{&S76881[151], 3}, {NULL}};
const Segment SGI51199[] = {{&S76881[117], 5}, {NULL}};
const Segment SGI51200[] = {{&S76886[44], 12}, {NULL}};
const Segment SGI51201[] = {{&S76886[44], 23}, {NULL}};
const Segment SGI51202[] = {{&S76886[30], 6}, {NULL}};
const Segment SGI51203[] = {{&S76886[115], 3}, {NULL}};
const Segment SGI51204[] = {{&S76886[81], 5}, {NULL}};
const Segment SGI51205[] = {{&S76914[31], 20}, {NULL}};
const Segment SGI51206[] = {{&S92236[44], 12}, {NULL}};
const Segment SGI51207[] = {{&S76914[66], 3}, {NULL}};
const Segment SGI51208[] = {{&S92236[44], 24}, {NULL}};
const Segment SGI51209[] = {{&S92236[30], 6}, {NULL}};
const Segment SGI51210[] = {{&S92236[82], 6}, {NULL}};
const Segment SGI51211[] = {{&S92236[109], 13}, {NULL}};
const Segment SGI51212[] = {{&S77374[44], 16}, {NULL}};
const Segment SGI51213[] = {{&S77374[44], 26}, {NULL}};
const Segment SGI51214[] = {{&S77374[30], 6}, {NULL}};
const Segment SGI51215[] = {{&S77374[118], 3}, {NULL}};
const Segment SGI51216[] = {{&S77374[84], 5}, {NULL}};
const Segment SGI51217[] = {{&S77390[45], 7}, {NULL}};
const Segment SGI51218[] = {{&S77390[30], 6}, {NULL}};
const Segment SGI51219[] = {{&S77390[58], 3}, {NULL}};
const Segment SGI51220[] = {{&S77390[84], 5}, {NULL}};
const Segment SGI51221[] = {{&S77390[142], 13}, {NULL}};
const Segment SGI51222[] = {{&S77019[44], 9}, {NULL}};
const Segment SGI51223[] = {{&S77019[44], 20}, {NULL}};
const Segment SGI51224[] = {{&S77019[30], 6}, {NULL}};
const Segment SGI51225[] = {{&S77019[112], 3}, {NULL}};
const Segment SGI51226[] = {{&S77019[78], 5}, {NULL}};
const Segment SGI51227[] = {{&S77020[44], 9}, {NULL}};
const Segment SGI51228[] = {{&S77020[44], 20}, {NULL}};
const Segment SGI51229[] = {{&S77020[30], 6}, {NULL}};
const Segment SGI51230[] = {{&S77020[81], 6}, {NULL}};
const Segment SGI51231[] = {{&S80402[44], 5}, {NULL}};
const Segment SGI51232[] = {{&S80402[44], 16}, {NULL}};
const Segment SGI51233[] = {{&S80402[30], 6}, {NULL}};
const Segment SGI51234[] = {{&S80402[108], 3}, {NULL}};
const Segment SGI51235[] = {{&S80402[74], 5}, {NULL}};
const Segment SGI51236[] = {{&S77074[44], 12}, {NULL}};
const Segment SGI51237[] = {{&S77074[44], 33}, {NULL}};
const Segment SGI51238[] = {{&S77074[30], 6}, {NULL}};
const Segment SGI51239[] = {{&S77074[125], 3}, {NULL}};
const Segment SGI51240[] = {{&S77074[91], 5}, {NULL}};
const Segment SGI51241[] = {{&S77538[45], 7}, {NULL}};
const Segment SGI51242[] = {{&S77538[30], 6}, {NULL}};
const Segment SGI51243[] = {{&S77538[58], 3}, {NULL}};
const Segment SGI51244[] = {{&S77538[84], 5}, {NULL}};
const Segment SGI51245[] = {{&S77538[142], 13}, {NULL}};
const Segment SGI51246[] = {{&S92510[45], 11}, {NULL}};
const Segment SGI51247[] = {{&S92510[30], 6}, {NULL}};
const Segment SGI51248[] = {{&S92510[115], 3}, {NULL}};
const Segment SGI51249[] = {{&S92510[81], 5}, {NULL}};
const Segment SGI51250[] = {{&S92510[139], 13}, {NULL}};
const Segment SGI51251[] = {{&S80524[44], 10}, {NULL}};
const Segment SGI51252[] = {{&S80524[44], 27}, {NULL}};
const Segment SGI51253[] = {{&S80524[30], 6}, {NULL}};
const Segment SGI51254[] = {{&S80524[119], 3}, {NULL}};
const Segment SGI51255[] = {{&S80524[85], 5}, {NULL}};
const Segment SGI51256[] = {{&S77552[44], 7}, {NULL}};
const Segment SGI51257[] = {{&S77552[44], 25}, {NULL}};
const Segment SGI51258[] = {{&S77552[30], 6}, {NULL}};
const Segment SGI51259[] = {{&S77552[83], 6}, {NULL}};
const Segment SGI51260[] = {{&S77552[110], 13}, {NULL}};
const Segment SGI51261[] = {{&S77553[44], 7}, {NULL}};
const Segment SGI51262[] = {{&S77553[44], 25}, {NULL}};
const Segment SGI51263[] = {{&S77553[30], 6}, {NULL}};
const Segment SGI51264[] = {{&S77553[83], 6}, {NULL}};
const Segment SGI51265[] = {{&S77553[110], 13}, {NULL}};
const Segment SGI51266[] = {{&S77283[45], 22}, {NULL}};
const Segment SGI51267[] = {{&S77283[30], 6}, {NULL}};
const Segment SGI51268[] = {{&S77283[126], 3}, {NULL}};
const Segment SGI51269[] = {{&S77283[92], 5}, {NULL}};
const Segment SGI51270[] = {{&S77283[150], 13}, {NULL}};
const Segment SGI51271[] = {{&S86921[44], 12}, {NULL}};
const Segment SGI51272[] = {{&S86921[44], 29}, {NULL}};
const Segment SGI51273[] = {{&S86921[30], 6}, {NULL}};
const Segment SGI51274[] = {{&S86921[121], 3}, {NULL}};
const Segment SGI51275[] = {{&S86921[87], 5}, {NULL}};
const Segment SGI51276[] = {{&S77338[45], 22}, {NULL}};
const Segment SGI51277[] = {{&S77338[30], 6}, {NULL}};
const Segment SGI51278[] = {{&S77338[126], 3}, {NULL}};
const Segment SGI51279[] = {{&S77338[92], 5}, {NULL}};
const Segment SGI51280[] = {{&S77338[150], 13}, {NULL}};
const Segment SGI51281[] = {{&S85227[41], 7}, {NULL}};
const Segment SGI51282[] = {{&S85227[41], 18}, {NULL}};
const Segment SGI51283[] = {{&S85227[30], 4}, {NULL}};
const Segment SGI51284[] = {{&S85227[107], 3}, {NULL}};
const Segment SGI51285[] = {{&S85227[73], 5}, {NULL}};
const Segment SGI51286[] = {{&S77626[44], 7}, {NULL}};
const Segment SGI51287[] = {{&S77626[44], 18}, {NULL}};
const Segment SGI51288[] = {{&S77626[30], 6}, {NULL}};
const Segment SGI51289[] = {{&S77626[110], 3}, {NULL}};
const Segment SGI51290[] = {{&S77626[76], 5}, {NULL}};
const Segment SGI51291[] = {{&S77371[44], 16}, {NULL}};
const Segment SGI51292[] = {{&S77371[44], 26}, {NULL}};
const Segment SGI51293[] = {{&S77371[30], 6}, {NULL}};
const Segment SGI51294[] = {{&S77371[118], 3}, {NULL}};
const Segment SGI51295[] = {{&S77371[84], 5}, {NULL}};
const Segment SGI51296[] = {{&S77642[45], 10}, {NULL}};
const Segment SGI51297[] = {{&S77642[30], 6}, {NULL}};
const Segment SGI51298[] = {{&S77642[120], 3}, {NULL}};
const Segment SGI51299[] = {{&S77642[86], 5}, {NULL}};
const Segment SGI51300[] = {{&S77642[144], 13}, {NULL}};
const Segment SGI51301[] = {{&S77373[44], 9}, {NULL}};
const Segment SGI51302[] = {{&S77373[44], 19}, {NULL}};
const Segment SGI51303[] = {{&S77373[30], 6}, {NULL}};
const Segment SGI51304[] = {{&S77373[111], 3}, {NULL}};
const Segment SGI51305[] = {{&S77373[77], 5}, {NULL}};
const Segment SGI51306[] = {{&S80725[39], 8}, {NULL}};
const Segment SGI51307[] = {{&S80725[103], 3}, {NULL}};
const Segment SGI51308[] = {{&S80725[92], 3}, {NULL}};
const Segment SGI51309[] = {{&S80725[69], 6}, {NULL}};
const Segment SGI51310[] = {{&S80726[31], 8}, {NULL}};
const Segment SGI51311[] = {{&S80726[54], 3}, {NULL}};
const Segment SGI51312[] = {{&S77423[44], 11}, {NULL}};
const Segment SGI51313[] = {{&S77423[44], 22}, {NULL}};
const Segment SGI51314[] = {{&S77423[30], 6}, {NULL}};
const Segment SGI51315[] = {{&S77423[114], 3}, {NULL}};
const Segment SGI51316[] = {{&S77423[80], 5}, {NULL}};
const Segment SGI51317[] = {{&S77424[44], 17}, {NULL}};
const Segment SGI51318[] = {{&S77424[44], 28}, {NULL}};
const Segment SGI51319[] = {{&S77424[30], 6}, {NULL}};
const Segment SGI51320[] = {{&S77424[120], 3}, {NULL}};
const Segment SGI51321[] = {{&S77424[86], 5}, {NULL}};
const Segment SGI51322[] = {{&S77434[45], 21}, {NULL}};
const Segment SGI51323[] = {{&S77434[30], 6}, {NULL}};
const Segment SGI51324[] = {{&S77434[125], 3}, {NULL}};
const Segment SGI51325[] = {{&S77434[91], 5}, {NULL}};
const Segment SGI51326[] = {{&S77434[149], 13}, {NULL}};
const Segment SGI51327[] = {{&S77435[45], 11}, {NULL}};
const Segment SGI51328[] = {{&S77435[30], 6}, {NULL}};
const Segment SGI51329[] = {{&S77435[115], 3}, {NULL}};
const Segment SGI51330[] = {{&S77435[81], 5}, {NULL}};
const Segment SGI51331[] = {{&S77435[139], 13}, {NULL}};
const Segment SGI51332[] = {{&S87055[44], 12}, {NULL}};
const Segment SGI51333[] = {{&S87055[44], 29}, {NULL}};
const Segment SGI51334[] = {{&S87055[30], 6}, {NULL}};
const Segment SGI51335[] = {{&S87055[121], 3}, {NULL}};
const Segment SGI51336[] = {{&S87055[87], 5}, {NULL}};
const Segment SGI51337[] = {{&S80803[39], 4}, {NULL}};
const Segment SGI51338[] = {{&S80803[99], 3}, {NULL}};
const Segment SGI51339[] = {{&S80803[88], 3}, {NULL}};
const Segment SGI51340[] = {{&S80803[65], 6}, {NULL}};
const Segment SGI51341[] = {{&S80804[31], 4}, {NULL}};
const Segment SGI51342[] = {{&S80804[50], 3}, {NULL}};
const Segment SGI51343[] = {{&S77735[44], 10}, {NULL}};
const Segment SGI51344[] = {{&S77735[44], 21}, {NULL}};
const Segment SGI51345[] = {{&S77735[30], 6}, {NULL}};
const Segment SGI51346[] = {{&S77735[113], 3}, {NULL}};
const Segment SGI51347[] = {{&S77735[79], 5}, {NULL}};
const Segment SGI51348[] = {{&S77525[44], 13}, {NULL}};
const Segment SGI51349[] = {{&S77525[44], 24}, {NULL}};
const Segment SGI51350[] = {{&S77525[30], 6}, {NULL}};
const Segment SGI51351[] = {{&S77525[116], 3}, {NULL}};
const Segment SGI51352[] = {{&S77525[82], 5}, {NULL}};
const Segment SGI51353[] = {{&S77526[44], 13}, {NULL}};
const Segment SGI51354[] = {{&S77526[44], 24}, {NULL}};
const Segment SGI51355[] = {{&S77526[30], 6}, {NULL}};
const Segment SGI51356[] = {{&S77526[116], 3}, {NULL}};
const Segment SGI51357[] = {{&S77526[82], 5}, {NULL}};
const Segment SGI51358[] = {{&S77534[44], 7}, {NULL}};
const Segment SGI51359[] = {{&S77534[44], 18}, {NULL}};
const Segment SGI51360[] = {{&S77534[30], 6}, {NULL}};
const Segment SGI51361[] = {{&S77534[79], 6}, {NULL}};
const Segment SGI51362[] = {{&S77535[44], 7}, {NULL}};
const Segment SGI51363[] = {{&S77535[44], 18}, {NULL}};
const Segment SGI51364[] = {{&S77535[30], 6}, {NULL}};
const Segment SGI51365[] = {{&S77535[110], 3}, {NULL}};
const Segment SGI51366[] = {{&S77535[76], 5}, {NULL}};
const Segment SGI51367[] = {{&S77536[44], 18}, {NULL}};
const Segment SGI51368[] = {{&S77536[30], 6}, {NULL}};
const Segment SGI51369[] = {{&S77537[38], 7}, {NULL}};
const Segment SGI51370[] = {{&S77537[31], 6}, {NULL}};
const Segment SGI51371[] = {{&S77774[45], 10}, {NULL}};
const Segment SGI51372[] = {{&S77774[30], 6}, {NULL}};
const Segment SGI51373[] = {{&S77774[120], 3}, {NULL}};
const Segment SGI51374[] = {{&S77774[86], 5}, {NULL}};
const Segment SGI51375[] = {{&S77774[144], 13}, {NULL}};
const Segment SGI51376[] = {{&S87056[44], 9}, {NULL}};
const Segment SGI51377[] = {{&S87056[44], 26}, {NULL}};
const Segment SGI51378[] = {{&S87056[30], 6}, {NULL}};
const Segment SGI51379[] = {{&S87056[118], 3}, {NULL}};
const Segment SGI51380[] = {{&S87056[84], 5}, {NULL}};
const Segment SGI51381[] = {{&S80939[31], 8}, {NULL}};
const Segment SGI51382[] = {{&S80939[54], 3}, {NULL}};
const Segment SGI51383[] = {{&S77606[44], 14}, {NULL}};
const Segment SGI51384[] = {{&S77606[44], 25}, {NULL}};
const Segment SGI51385[] = {{&S77606[30], 6}, {NULL}};
const Segment SGI51386[] = {{&S77606[117], 3}, {NULL}};
const Segment SGI51387[] = {{&S77606[83], 5}, {NULL}};
const Segment SGI51388[] = {{&S77619[44], 10}, {NULL}};
const Segment SGI51389[] = {{&S77619[44], 27}, {NULL}};
const Segment SGI51390[] = {{&S77619[30], 6}, {NULL}};
const Segment SGI51391[] = {{&S77619[85], 6}, {NULL}};
const Segment SGI51392[] = {{&S77619[112], 13}, {NULL}};
const Segment SGI51393[] = {{&S77627[44], 7}, {NULL}};
const Segment SGI51394[] = {{&S77627[44], 18}, {NULL}};
const Segment SGI51395[] = {{&S77627[30], 6}, {NULL}};
const Segment SGI51396[] = {{&S77627[110], 3}, {NULL}};
const Segment SGI51397[] = {{&S77627[76], 5}, {NULL}};
const Segment SGI51398[] = {{&S82031[44], 14}, {NULL}};
const Segment SGI51399[] = {{&S82031[44], 25}, {NULL}};
const Segment SGI51400[] = {{&S82031[30], 6}, {NULL}};
const Segment SGI51401[] = {{&S82031[117], 3}, {NULL}};
const Segment SGI51402[] = {{&S82031[83], 5}, {NULL}};
const Segment SGI51403[] = {{&S81033[44], 14}, {NULL}};
const Segment SGI51404[] = {{&S81033[44], 25}, {NULL}};
const Segment SGI51405[] = {{&S81033[30], 6}, {NULL}};
const Segment SGI51406[] = {{&S81033[83], 6}, {NULL}};
const Segment SGI51407[] = {{&S81033[120], 3}, {NULL}};
const Segment SGI51408[] = {{&S77658[44], 14}, {NULL}};
const Segment SGI51409[] = {{&S77658[44], 25}, {NULL}};
const Segment SGI51410[] = {{&S77658[30], 6}, {NULL}};
const Segment SGI51411[] = {{&S77658[83], 6}, {NULL}};
const Segment SGI51412[] = {{&S77658[110], 13}, {NULL}};
const Segment SGI51413[] = {{&S77671[44], 13}, {NULL}};
const Segment SGI51414[] = {{&S77671[44], 24}, {NULL}};
const Segment SGI51415[] = {{&S77671[30], 6}, {NULL}};
const Segment SGI51416[] = {{&S81070[45], 6}, {NULL}};
const Segment SGI51417[] = {{&S81070[30], 6}, {NULL}};
const Segment SGI51418[] = {{&S81070[117], 3}, {NULL}};
const Segment SGI51419[] = {{&S81070[83], 5}, {NULL}};
const Segment SGI51420[] = {{&S81070[151], 3}, {NULL}};
const Segment SGI51421[] = {{&S81072[45], 10}, {NULL}};
const Segment SGI51422[] = {{&S81072[30], 6}, {NULL}};
const Segment SGI51423[] = {{&S81072[120], 3}, {NULL}};
const Segment SGI51424[] = {{&S81072[86], 5}, {NULL}};
const Segment SGI51425[] = {{&S93078[45], 9}, {NULL}};
const Segment SGI51426[] = {{&S81072[154], 3}, {NULL}};
const Segment SGI51427[] = {{&S93078[30], 6}, {NULL}};
const Segment SGI51428[] = {{&S93078[113], 3}, {NULL}};
const Segment SGI51429[] = {{&S93078[79], 5}, {NULL}};
const Segment SGI51430[] = {{&S93078[137], 13}, {NULL}};
const Segment SGI51431[] = {{&S81077[45], 11}, {NULL}};
const Segment SGI51432[] = {{&S93083[44], 20}, {NULL}};
const Segment SGI51433[] = {{&S81077[30], 6}, {NULL}};
const Segment SGI51434[] = {{&S93083[44], 32}, {NULL}};
const Segment SGI51435[] = {{&S93083[30], 6}, {NULL}};
const Segment SGI51436[] = {{&S81077[121], 3}, {NULL}};
const Segment SGI51437[] = {{&S93083[124], 3}, {NULL}};
const Segment SGI51438[] = {{&S81077[87], 5}, {NULL}};
const Segment SGI51439[] = {{&S93083[90], 5}, {NULL}};
const Segment SGI51440[] = {{&S81077[155], 3}, {NULL}};
const Segment SGI51441[] = {{&S77671[116], 3}, {NULL}};
const Segment SGI51442[] = {{&S77671[82], 5}, {NULL}};
const Segment SGI51443[] = {{&S77690[45], 10}, {NULL}};
const Segment SGI51444[] = {{&S93135[45], 9}, {NULL}};
const Segment SGI51445[] = {{&S77690[30], 6}, {NULL}};
const Segment SGI51446[] = {{&S93135[30], 6}, {NULL}};
const Segment SGI51447[] = {{&S77690[120], 3}, {NULL}};
const Segment SGI51448[] = {{&S93135[114], 3}, {NULL}};
const Segment SGI51449[] = {{&S77690[86], 5}, {NULL}};
const Segment SGI51450[] = {{&S93135[80], 5}, {NULL}};
const Segment SGI51451[] = {{&S77690[144], 13}, {NULL}};
const Segment SGI51452[] = {{&S93135[138], 13}, {NULL}};
const Segment SGI51453[] = {{&S93155[45], 7}, {NULL}};
const Segment SGI51454[] = {{&S93155[30], 6}, {NULL}};
const Segment SGI51455[] = {{&S93155[117], 3}, {NULL}};
const Segment SGI51456[] = {{&S93155[83], 5}, {NULL}};
const Segment SGI51457[] = {{&S93155[141], 13}, {NULL}};
const Segment SGI51458[] = {{&S81167[44], 11}, {NULL}};
const Segment SGI51459[] = {{&S81167[44], 22}, {NULL}};
const Segment SGI51460[] = {{&S81167[30], 6}, {NULL}};
const Segment SGI51461[] = {{&S81167[121], 3}, {NULL}};
const Segment SGI51462[] = {{&S81167[87], 5}, {NULL}};
const Segment SGI51463[] = {{&S81167[66], 8}, {NULL}};
const Segment SGI51464[] = {{&S81169[44], 11}, {NULL}};
const Segment SGI51465[] = {{&S81169[44], 22}, {NULL}};
const Segment SGI51466[] = {{&S81169[30], 6}, {NULL}};
const Segment SGI51467[] = {{&S81169[114], 3}, {NULL}};
const Segment SGI51468[] = {{&S81169[80], 5}, {NULL}};
const Segment SGI51469[] = {{&S81169[138], 7}, {NULL}};
const Segment SGI51470[] = {{&S78130[31], 5}, {NULL}};
const Segment SGI51471[] = {{&S78130[51], 3}, {NULL}};
const Segment SGI51472[] = {{&S78131[39], 5}, {NULL}};
const Segment SGI51473[] = {{&S78131[100], 3}, {NULL}};
const Segment SGI51474[] = {{&S78131[89], 3}, {NULL}};
const Segment SGI51475[] = {{&S78131[66], 6}, {NULL}};
const Segment SGI51476[] = {{&S93275[44], 9}, {NULL}};
const Segment SGI51477[] = {{&S93275[44], 20}, {NULL}};
const Segment SGI51478[] = {{&S93275[30], 6}, {NULL}};
const Segment SGI51479[] = {{&S93275[112], 3}, {NULL}};
const Segment SGI51480[] = {{&S93275[78], 5}, {NULL}};
const Segment SGI51481[] = {{&S77775[45], 10}, {NULL}};
const Segment SGI51482[] = {{&S77775[30], 6}, {NULL}};
const Segment SGI51483[] = {{&S77775[120], 3}, {NULL}};
const Segment SGI51484[] = {{&S77775[86], 5}, {NULL}};
const Segment SGI51485[] = {{&S77775[144], 13}, {NULL}};
const Segment SGI51486[] = {{&S78284[44], 7}, {NULL}};
const Segment SGI51487[] = {{&S78284[44], 18}, {NULL}};
const Segment SGI51488[] = {{&S78284[30], 6}, {NULL}};
const Segment SGI51489[] = {{&S78284[110], 3}, {NULL}};
const Segment SGI51490[] = {{&S78284[76], 5}, {NULL}};
const Segment SGI51491[] = {{&S78286[44], 7}, {NULL}};
const Segment SGI51492[] = {{&S78286[44], 18}, {NULL}};
const Segment SGI51493[] = {{&S78286[30], 6}, {NULL}};
const Segment SGI51494[] = {{&S78286[110], 3}, {NULL}};
const Segment SGI51495[] = {{&S78286[76], 5}, {NULL}};
const Segment SGI51496[] = {{&S78287[42], 7}, {NULL}};
const Segment SGI51497[] = {{&S78287[30], 6}, {NULL}};
const Segment SGI51498[] = {{&S78287[108], 3}, {NULL}};
const Segment SGI51499[] = {{&S78287[74], 5}, {NULL}};
const Segment SGI51500[] = {{&S77870[44], 10}, {NULL}};
const Segment SGI51501[] = {{&S77870[44], 20}, {NULL}};
const Segment SGI51502[] = {{&S77870[30], 6}, {NULL}};
const Segment SGI51503[] = {{&S77870[112], 3}, {NULL}};
const Segment SGI51504[] = {{&S77870[78], 5}, {NULL}};
const Segment SGI51505[] = {{&S77874[52], 10}, {NULL}};
const Segment SGI51506[] = {{&S77874[52], 20}, {NULL}};
const Segment SGI51507[] = {{&S77874[30], 14}, {NULL}};
const Segment SGI51508[] = {{&S77874[120], 3}, {NULL}};
const Segment SGI51509[] = {{&S77874[86], 5}, {NULL}};
const Segment SGI51510[] = {{&S93452[44], 9}, {NULL}};
const Segment SGI51511[] = {{&S93452[44], 20}, {NULL}};
const Segment SGI51512[] = {{&S93452[30], 6}, {NULL}};
const Segment SGI51513[] = {{&S93452[78], 6}, {NULL}};
const Segment SGI51514[] = {{&S93452[105], 13}, {NULL}};
const Segment SGI51515[] = {{&S87574[44], 10}, {NULL}};
const Segment SGI51516[] = {{&S87574[44], 22}, {NULL}};
const Segment SGI51517[] = {{&S87574[30], 6}, {NULL}};
const Segment SGI51518[] = {{&S87574[114], 3}, {NULL}};
const Segment SGI51519[] = {{&S87574[80], 5}, {NULL}};
const Segment SGI51520[] = {{&S87575[44], 10}, {NULL}};
const Segment SGI51521[] = {{&S87575[44], 22}, {NULL}};
const Segment SGI51522[] = {{&S87575[30], 6}, {NULL}};
const Segment SGI51523[] = {{&S87575[80], 6}, {NULL}};
const Segment SGI51524[] = {{&S81484[44], 18}, {NULL}};
const Segment SGI51525[] = {{&S81484[44], 29}, {NULL}};
const Segment SGI51526[] = {{&S81484[30], 6}, {NULL}};
const Segment SGI51527[] = {{&S81484[121], 3}, {NULL}};
const Segment SGI51528[] = {{&S81484[87], 5}, {NULL}};
const Segment SGI51529[] = {{&S78426[44], 19}, {NULL}};
const Segment SGI51530[] = {{&S78426[44], 37}, {NULL}};
const Segment SGI51531[] = {{&S78426[30], 6}, {NULL}};
const Segment SGI51532[] = {{&S78426[69], 3}, {NULL}};
const Segment SGI51533[] = {{&S78426[95], 5}, {NULL}};
const Segment SGI51534[] = {{&S78427[44], 19}, {NULL}};
const Segment SGI51535[] = {{&S78427[44], 37}, {NULL}};
const Segment SGI51536[] = {{&S78427[30], 6}, {NULL}};
const Segment SGI51537[] = {{&S78427[69], 3}, {NULL}};
const Segment SGI51538[] = {{&S78427[95], 5}, {NULL}};
const Segment SGI51539[] = {{&S78428[44], 19}, {NULL}};
const Segment SGI51540[] = {{&S78428[44], 37}, {NULL}};
const Segment SGI51541[] = {{&S78428[30], 6}, {NULL}};
const Segment SGI51542[] = {{&S78428[69], 3}, {NULL}};
const Segment SGI51543[] = {{&S78428[95], 5}, {NULL}};
const Segment SGI51544[] = {{&S78429[44], 7}, {NULL}};
const Segment SGI51545[] = {{&S78429[44], 25}, {NULL}};
const Segment SGI51546[] = {{&S78429[30], 6}, {NULL}};
const Segment SGI51547[] = {{&S78429[57], 3}, {NULL}};
const Segment SGI51548[] = {{&S78429[83], 5}, {NULL}};
const Segment SGI51549[] = {{&S81531[59], 10}, {NULL}};
const Segment SGI51550[] = {{&S81531[59], 27}, {NULL}};
const Segment SGI51551[] = {{&S81531[45], 6}, {NULL}};
const Segment SGI51552[] = {{&S81531[134], 3}, {NULL}};
const Segment SGI51553[] = {{&S81531[100], 5}, {NULL}};
const Segment SGI51554[] = {{&S81531[0], 12}, {NULL}};
const Segment SGI51555[] = {{&S77996[44], 10}, {NULL}};
const Segment SGI51556[] = {{&S77996[44], 27}, {NULL}};
const Segment SGI51557[] = {{&S77996[30], 6}, {NULL}};
const Segment SGI51558[] = {{&S77996[85], 6}, {NULL}};
const Segment SGI51559[] = {{&S77996[112], 13}, {NULL}};
const Segment SGI51560[] = {{&S93562[44], 4}, {NULL}};
const Segment SGI51561[] = {{&S93562[44], 43}, {NULL}};
const Segment SGI51562[] = {{&S93562[30], 6}, {NULL}};
const Segment SGI51563[] = {{&S93562[135], 3}, {NULL}};
const Segment SGI51564[] = {{&S93562[101], 5}, {NULL}};
const Segment SGI51565[] = {{&S81555[44], 6}, {NULL}};
const Segment SGI51566[] = {{&S81555[44], 24}, {NULL}};
const Segment SGI51567[] = {{&S81555[30], 6}, {NULL}};
const Segment SGI51568[] = {{&S81555[82], 6}, {NULL}};
const Segment SGI51569[] = {{&S78016[47], 6}, {NULL}};
const Segment SGI51570[] = {{&S81555[119], 3}, {NULL}};
const Segment SGI51571[] = {{&S78016[195], 3}, {NULL}};
const Segment SGI51572[] = {{&S78016[161], 5}, {NULL}};
const Segment SGI51573[] = {{&S93587[45], 11}, {NULL}};
const Segment SGI51574[] = {{&S93587[30], 6}, {NULL}};
const Segment SGI51575[] = {{&S93587[115], 3}, {NULL}};
const Segment SGI51576[] = {{&S93587[81], 5}, {NULL}};
const Segment SGI51577[] = {{&S93587[139], 13}, {NULL}};
const Segment SGI51578[] = {{&S78027[44], 12}, {NULL}};
const Segment SGI51579[] = {{&S78027[44], 33}, {NULL}};
const Segment SGI51580[] = {{&S78027[30], 6}, {NULL}};
const Segment SGI51581[] = {{&S78027[125], 3}, {NULL}};
const Segment SGI51582[] = {{&S78027[91], 5}, {NULL}};
const Segment SGI51583[] = {{&S78485[44], 7}, {NULL}};
const Segment SGI51584[] = {{&S78485[44], 18}, {NULL}};
const Segment SGI51585[] = {{&S78485[30], 6}, {NULL}};
const Segment SGI51586[] = {{&S78485[110], 3}, {NULL}};
const Segment SGI51587[] = {{&S78485[76], 5}, {NULL}};
const Segment SGI51588[] = {{&S78490[44], 8}, {NULL}};
const Segment SGI51589[] = {{&S78490[44], 19}, {NULL}};
const Segment SGI51590[] = {{&S78490[30], 6}, {NULL}};
const Segment SGI51591[] = {{&S78490[111], 3}, {NULL}};
const Segment SGI51592[] = {{&S78490[77], 5}, {NULL}};
const Segment SGI51593[] = {{&S78497[44], 8}, {NULL}};
const Segment SGI51594[] = {{&S78497[44], 35}, {NULL}};
const Segment SGI51595[] = {{&S78497[30], 6}, {NULL}};
const Segment SGI51596[] = {{&S78497[127], 3}, {NULL}};
const Segment SGI51597[] = {{&S78497[93], 5}, {NULL}};
const Segment SGI51598[] = {{&S78497[151], 7}, {NULL}};
const Segment SGI51599[] = {{&S78189[44], 14}, {NULL}};
const Segment SGI51600[] = {{&S78189[44], 25}, {NULL}};
const Segment SGI51601[] = {{&S78189[30], 6}, {NULL}};
const Segment SGI51602[] = {{&S78189[117], 3}, {NULL}};
const Segment SGI51603[] = {{&S78189[83], 5}, {NULL}};
const Segment SGI51604[] = {{&S78640[45], 7}, {NULL}};
const Segment SGI51605[] = {{&S78640[30], 6}, {NULL}};
const Segment SGI51606[] = {{&S78640[58], 3}, {NULL}};
const Segment SGI51607[] = {{&S78640[84], 5}, {NULL}};
const Segment SGI51608[] = {{&S78640[142], 13}, {NULL}};
const Segment SGI51609[] = {{&S93809[44], 4}, {NULL}};
const Segment SGI51610[] = {{&S93809[44], 16}, {NULL}};
const Segment SGI51611[] = {{&S93809[30], 6}, {NULL}};
const Segment SGI51612[] = {{&S93809[108], 3}, {NULL}};
const Segment SGI51613[] = {{&S93809[74], 5}, {NULL}};
const Segment SGI51614[] = {{&S78285[44], 7}, {NULL}};
const Segment SGI51615[] = {{&S78285[44], 18}, {NULL}};
const Segment SGI51616[] = {{&S78285[30], 6}, {NULL}};
const Segment SGI51617[] = {{&S78285[110], 3}, {NULL}};
const Segment SGI51618[] = {{&S78285[76], 5}, {NULL}};
const Segment SGI51619[] = {{&S78288[38], 8}, {NULL}};
const Segment SGI51620[] = {{&S78288[31], 6}, {NULL}};
const Segment SGI51621[] = {{&S78289[44], 8}, {NULL}};
const Segment SGI51622[] = {{&S78289[44], 19}, {NULL}};
const Segment SGI51623[] = {{&S78289[30], 6}, {NULL}};
const Segment SGI51624[] = {{&S78289[111], 3}, {NULL}};
const Segment SGI51625[] = {{&S78289[77], 5}, {NULL}};
const Segment SGI51626[] = {{&S81689[45], 10}, {NULL}};
const Segment SGI51627[] = {{&S81689[30], 6}, {NULL}};
const Segment SGI51628[] = {{&S81689[120], 3}, {NULL}};
const Segment SGI51629[] = {{&S81689[86], 5}, {NULL}};
const Segment SGI51630[] = {{&S81689[154], 3}, {NULL}};
const Segment SGI51631[] = {{&S81690[45], 11}, {NULL}};
const Segment SGI51632[] = {{&S81690[30], 6}, {NULL}};
const Segment SGI51633[] = {{&S81690[121], 3}, {NULL}};
const Segment SGI51634[] = {{&S81690[87], 5}, {NULL}};
const Segment SGI51635[] = {{&S81690[155], 3}, {NULL}};
const Segment SGI51636[] = {{&S81692[45], 11}, {NULL}};
const Segment SGI51637[] = {{&S81692[30], 6}, {NULL}};
const Segment SGI51638[] = {{&S81692[121], 3}, {NULL}};
const Segment SGI51639[] = {{&S81692[87], 5}, {NULL}};
const Segment SGI51640[] = {{&S81692[155], 3}, {NULL}};
const Segment SGI51641[] = {{&S81694[45], 11}, {NULL}};
const Segment SGI51642[] = {{&S81694[30], 6}, {NULL}};
const Segment SGI51643[] = {{&S81694[121], 3}, {NULL}};
const Segment SGI51644[] = {{&S81694[87], 5}, {NULL}};
const Segment SGI51645[] = {{&S81694[155], 3}, {NULL}};
const Segment SGI51646[] = {{&S78337[44], 10}, {NULL}};
const Segment SGI51647[] = {{&S78337[44], 27}, {NULL}};
const Segment SGI51648[] = {{&S78337[30], 6}, {NULL}};
const Segment SGI51649[] = {{&S78337[126], 3}, {NULL}};
const Segment SGI51650[] = {{&S78337[92], 5}, {NULL}};
const Segment SGI51651[] = {{&S78337[71], 8}, {NULL}};
const Segment SGI51652[] = {{&S78340[44], 10}, {NULL}};
const Segment SGI51653[] = {{&S78340[44], 27}, {NULL}};
const Segment SGI51654[] = {{&S78340[30], 6}, {NULL}};
const Segment SGI51655[] = {{&S78340[119], 3}, {NULL}};
const Segment SGI51656[] = {{&S78340[85], 5}, {NULL}};
const Segment SGI51657[] = {{&S78340[143], 7}, {NULL}};
const Segment SGI51658[] = {{&S78817[45], 10}, {NULL}};
const Segment SGI51659[] = {{&S78817[30], 6}, {NULL}};
const Segment SGI51660[] = {{&S78817[120], 3}, {NULL}};
const Segment SGI51661[] = {{&S78817[86], 5}, {NULL}};
const Segment SGI51662[] = {{&S78817[144], 13}, {NULL}};
const Segment SGI51663[] = {{&S81747[44], 11}, {NULL}};
const Segment SGI51664[] = {{&S81747[44], 28}, {NULL}};
const Segment SGI51665[] = {{&S81747[30], 6}, {NULL}};
const Segment SGI51666[] = {{&S81747[120], 3}, {NULL}};
const Segment SGI51667[] = {{&S81747[86], 5}, {NULL}};
const Segment SGI51668[] = {{&S78870[44], 20}, {NULL}};
const Segment SGI51669[] = {{&S78870[44], 38}, {NULL}};
const Segment SGI51670[] = {{&S78870[30], 6}, {NULL}};
const Segment SGI51671[] = {{&S78870[70], 3}, {NULL}};
const Segment SGI51672[] = {{&S78870[96], 5}, {NULL}};
const Segment SGI51673[] = {{&S78871[44], 20}, {NULL}};
const Segment SGI51674[] = {{&S78871[44], 38}, {NULL}};
const Segment SGI51675[] = {{&S78871[30], 6}, {NULL}};
const Segment SGI51676[] = {{&S78871[70], 3}, {NULL}};
const Segment SGI51677[] = {{&S78871[96], 5}, {NULL}};
const Segment SGI51678[] = {{&S78872[44], 8}, {NULL}};
const Segment SGI51679[] = {{&S78872[44], 26}, {NULL}};
const Segment SGI51680[] = {{&S78872[30], 6}, {NULL}};
const Segment SGI51681[] = {{&S78872[58], 3}, {NULL}};
const Segment SGI51682[] = {{&S78872[84], 5}, {NULL}};
const Segment SGI51683[] = {{&S78873[44], 20}, {NULL}};
const Segment SGI51684[] = {{&S78873[44], 38}, {NULL}};
const Segment SGI51685[] = {{&S78873[30], 6}, {NULL}};
const Segment SGI51686[] = {{&S78873[70], 3}, {NULL}};
const Segment SGI51687[] = {{&S78873[96], 5}, {NULL}};
const Segment SGI51688[] = {{&S78879[45], 10}, {NULL}};
const Segment SGI51689[] = {{&S78879[30], 6}, {NULL}};
const Segment SGI51690[] = {{&S78879[120], 3}, {NULL}};
const Segment SGI51691[] = {{&S78879[86], 5}, {NULL}};
const Segment SGI51692[] = {{&S78879[144], 13}, {NULL}};
const Segment SGI51693[] = {{&S78472[44], 9}, {NULL}};
const Segment SGI51694[] = {{&S78472[44], 20}, {NULL}};
const Segment SGI51695[] = {{&S78472[30], 6}, {NULL}};
const Segment SGI51696[] = {{&S78472[112], 3}, {NULL}};
const Segment SGI51697[] = {{&S78472[78], 5}, {NULL}};
const Segment SGI51698[] = {{&S78472[136], 11}, {NULL}};
const Segment SGI51699[] = {{&S78487[44], 7}, {NULL}};
const Segment SGI51700[] = {{&S78487[44], 18}, {NULL}};
const Segment SGI51701[] = {{&S78487[30], 6}, {NULL}};
const Segment SGI51702[] = {{&S78487[110], 3}, {NULL}};
const Segment SGI51703[] = {{&S78487[76], 5}, {NULL}};
const Segment SGI51704[] = {{&S78491[44], 8}, {NULL}};
const Segment SGI51705[] = {{&S78491[44], 19}, {NULL}};
const Segment SGI51706[] = {{&S78491[30], 6}, {NULL}};
const Segment SGI51707[] = {{&S78491[111], 3}, {NULL}};
const Segment SGI51708[] = {{&S78491[77], 5}, {NULL}};
const Segment SGI51709[] = {{&S78943[44], 10}, {NULL}};
const Segment SGI51710[] = {{&S78943[44], 27}, {NULL}};
const Segment SGI51711[] = {{&S78943[30], 6}, {NULL}};
const Segment SGI51712[] = {{&S78943[119], 3}, {NULL}};
const Segment SGI51713[] = {{&S78943[85], 5}, {NULL}};
const Segment SGI51714[] = {{&S81957[44], 14}, {NULL}};
const Segment SGI51715[] = {{&S81957[44], 31}, {NULL}};
const Segment SGI51716[] = {{&S81957[30], 6}, {NULL}};
const Segment SGI51717[] = {{&S81957[123], 3}, {NULL}};
const Segment SGI51718[] = {{&S81957[89], 5}, {NULL}};
const Segment SGI51719[] = {{&S82029[44], 6}, {NULL}};
const Segment SGI51720[] = {{&S82029[44], 17}, {NULL}};
const Segment SGI51721[] = {{&S82029[30], 6}, {NULL}};
const Segment SGI51722[] = {{&S82029[109], 3}, {NULL}};
const Segment SGI51723[] = {{&S82029[75], 5}, {NULL}};
const Segment SGI51724[] = {{&S82030[44], 6}, {NULL}};
const Segment SGI51725[] = {{&S82030[44], 17}, {NULL}};
const Segment SGI51726[] = {{&S82030[30], 6}, {NULL}};
const Segment SGI51727[] = {{&S82030[109], 3}, {NULL}};
const Segment SGI51728[] = {{&S82030[75], 5}, {NULL}};
const Segment SGI51729[] = {{&S82032[44], 14}, {NULL}};
const Segment SGI51730[] = {{&S82032[44], 25}, {NULL}};
const Segment SGI51731[] = {{&S82032[30], 6}, {NULL}};
const Segment SGI51732[] = {{&S82032[117], 3}, {NULL}};
const Segment SGI51733[] = {{&S82032[83], 5}, {NULL}};
const Segment SGI51734[] = {{&S82111[44], 9}, {NULL}};
const Segment SGI51735[] = {{&S82111[44], 20}, {NULL}};
const Segment SGI51736[] = {{&S82111[30], 6}, {NULL}};
const Segment SGI51737[] = {{&S82111[119], 3}, {NULL}};
const Segment SGI51738[] = {{&S82111[85], 5}, {NULL}};
const Segment SGI51739[] = {{&S82111[64], 8}, {NULL}};
const Segment SGI51740[] = {{&S82113[44], 9}, {NULL}};
const Segment SGI51741[] = {{&S82113[44], 20}, {NULL}};
const Segment SGI51742[] = {{&S82113[30], 6}, {NULL}};
const Segment SGI51743[] = {{&S82113[112], 3}, {NULL}};
const Segment SGI51744[] = {{&S82113[78], 5}, {NULL}};
const Segment SGI51745[] = {{&S82113[136], 7}, {NULL}};
const Segment SGI51746[] = {{&S78837[44], 10}, {NULL}};
const Segment SGI51747[] = {{&S78837[44], 22}, {NULL}};
const Segment SGI51748[] = {{&S78837[30], 6}, {NULL}};
const Segment SGI51749[] = {{&S78837[80], 6}, {NULL}};
const Segment SGI51750[] = {{&S78837[107], 13}, {NULL}};
const Segment SGI51751[] = {{&S79197[44], 5}, {NULL}};
const Segment SGI51752[] = {{&S79197[44], 16}, {NULL}};
const Segment SGI51753[] = {{&S79197[30], 6}, {NULL}};
const Segment SGI51754[] = {{&S79197[108], 3}, {NULL}};
const Segment SGI51755[] = {{&S79197[74], 5}, {NULL}};
const Segment SGI51756[] = {{&S78878[45], 10}, {NULL}};
const Segment SGI51757[] = {{&S78878[30], 6}, {NULL}};
const Segment SGI51758[] = {{&S78878[120], 3}, {NULL}};
const Segment SGI51759[] = {{&S78878[86], 5}, {NULL}};
const Segment SGI51760[] = {{&S78878[144], 13}, {NULL}};
const Segment SGI51761[] = {{&S78940[44], 6}, {NULL}};
const Segment SGI51762[] = {{&S78940[44], 14}, {NULL}};
const Segment SGI51763[] = {{&S78940[30], 6}, {NULL}};
const Segment SGI51764[] = {{&S78940[106], 3}, {NULL}};
const Segment SGI51765[] = {{&S78940[72], 5}, {NULL}};
const Segment SGI51766[] = {{&S78941[38], 6}, {NULL}};
const Segment SGI51767[] = {{&S78941[31], 6}, {NULL}};
const Segment SGI51768[] = {{&S82317[45], 10}, {NULL}};
const Segment SGI51769[] = {{&S82317[30], 6}, {NULL}};
const Segment SGI51770[] = {{&S82317[120], 3}, {NULL}};
const Segment SGI51771[] = {{&S82317[86], 5}, {NULL}};
const Segment SGI51772[] = {{&S82317[154], 3}, {NULL}};
const Segment SGI51773[] = {{&S79311[44], 14}, {NULL}};
const Segment SGI51774[] = {{&S79311[44], 31}, {NULL}};
const Segment SGI51775[] = {{&S79311[30], 6}, {NULL}};
const Segment SGI51776[] = {{&S79311[123], 3}, {NULL}};
const Segment SGI51777[] = {{&S79311[89], 5}, {NULL}};
const Segment SGI51778[] = {{&S79340[34], 11}, {NULL}};
const Segment SGI51779[] = {{&S79340[27], 6}, {NULL}};
const Segment SGI51780[] = {{&S79340[110], 2}, {NULL}};
const Segment SGI51781[] = {{&S79340[76], 6}, {NULL}};
const Segment SGI51782[] = {{&S79340[144], 8}, {NULL}};
const Segment SGI51783[] = {{&S79376[45], 7}, {NULL}};
const Segment SGI51784[] = {{&S79376[30], 6}, {NULL}};
const Segment SGI51785[] = {{&S79376[58], 3}, {NULL}};
const Segment SGI51786[] = {{&S79376[84], 5}, {NULL}};
const Segment SGI51787[] = {{&S79376[142], 13}, {NULL}};
const Segment SGI51788[] = {{&S79418[44], 9}, {NULL}};
const Segment SGI51789[] = {{&S79418[44], 26}, {NULL}};
const Segment SGI51790[] = {{&S79418[30], 6}, {NULL}};
const Segment SGI51791[] = {{&S79418[118], 3}, {NULL}};
const Segment SGI51792[] = {{&S79418[84], 5}, {NULL}};
const Segment SGI51793[] = {{&S79149[44], 28}, {NULL}};
const Segment SGI51794[] = {{&S79149[44], 39}, {NULL}};
const Segment SGI51795[] = {{&S79149[30], 6}, {NULL}};
const Segment SGI51796[] = {{&S79149[130], 3}, {NULL}};
const Segment SGI51797[] = {{&S79149[97], 5}, {NULL}};
const Segment SGI51798[] = {{&S94490[45], 11}, {NULL}};
const Segment SGI51799[] = {{&S94490[30], 6}, {NULL}};
const Segment SGI51800[] = {{&S94490[115], 3}, {NULL}};
const Segment SGI51801[] = {{&S94490[81], 5}, {NULL}};
const Segment SGI51802[] = {{&S94490[139], 13}, {NULL}};
const Segment SGI51803[] = {{&S79199[44], 10}, {NULL}};
const Segment SGI51804[] = {{&S79199[44], 21}, {NULL}};
const Segment SGI51805[] = {{&S79199[30], 6}, {NULL}};
const Segment SGI51806[] = {{&S79199[113], 3}, {NULL}};
const Segment SGI51807[] = {{&S79199[79], 5}, {NULL}};
const Segment SGI51808[] = {{&S79200[44], 4}, {NULL}};
const Segment SGI51809[] = {{&S79200[44], 25}, {NULL}};
const Segment SGI51810[] = {{&S79200[30], 6}, {NULL}};
const Segment SGI51811[] = {{&S79200[117], 3}, {NULL}};
const Segment SGI51812[] = {{&S79200[83], 5}, {NULL}};
const Segment SGI51813[] = {{&S79534[45], 10}, {NULL}};
const Segment SGI51814[] = {{&S79534[30], 6}, {NULL}};
const Segment SGI51815[] = {{&S79534[120], 3}, {NULL}};
const Segment SGI51816[] = {{&S79534[86], 5}, {NULL}};
const Segment SGI51817[] = {{&S79534[144], 13}, {NULL}};
const Segment SGI51818[] = {{&S79535[45], 11}, {NULL}};
const Segment SGI51819[] = {{&S79535[30], 6}, {NULL}};
const Segment SGI51820[] = {{&S79535[121], 3}, {NULL}};
const Segment SGI51821[] = {{&S79535[87], 5}, {NULL}};
const Segment SGI51822[] = {{&S79535[145], 13}, {NULL}};
const Segment SGI51823[] = {{&S82478[45], 10}, {NULL}};
const Segment SGI51824[] = {{&S82478[30], 6}, {NULL}};
const Segment SGI51825[] = {{&S82478[120], 3}, {NULL}};
const Segment SGI51826[] = {{&S82478[86], 5}, {NULL}};
const Segment SGI51827[] = {{&S82478[154], 3}, {NULL}};
const Segment SGI51828[] = {{&S79341[34], 11}, {NULL}};
const Segment SGI51829[] = {{&S79341[27], 6}, {NULL}};
const Segment SGI51830[] = {{&S79341[110], 2}, {NULL}};
const Segment SGI51831[] = {{&S79341[76], 6}, {NULL}};
const Segment SGI51832[] = {{&S79341[144], 8}, {NULL}};
const Segment SGI51833[] = {{&S79660[34], 9}, {NULL}};
const Segment SGI51834[] = {{&S79660[27], 6}, {NULL}};
const Segment SGI51835[] = {{&S79660[102], 2}, {NULL}};
const Segment SGI51836[] = {{&S79660[68], 6}, {NULL}};
const Segment SGI51837[] = {{&S79660[136], 8}, {NULL}};
const Segment SGI51838[] = {{&S79384[44], 21}, {NULL}};
const Segment SGI51839[] = {{&S79384[44], 42}, {NULL}};
const Segment SGI51840[] = {{&S79384[30], 6}, {NULL}};
const Segment SGI51841[] = {{&S79384[134], 3}, {NULL}};
const Segment SGI51842[] = {{&S79384[100], 5}, {NULL}};
const Segment SGI51843[] = {{&S94698[44], 13}, {NULL}};
const Segment SGI51844[] = {{&S94698[44], 25}, {NULL}};
const Segment SGI51845[] = {{&S94698[30], 6}, {NULL}};
const Segment SGI51846[] = {{&S94698[117], 3}, {NULL}};
const Segment SGI51847[] = {{&S94698[83], 5}, {NULL}};
const Segment SGI51848[] = {{&S79773[45], 10}, {NULL}};
const Segment SGI51849[] = {{&S79773[30], 6}, {NULL}};
const Segment SGI51850[] = {{&S79773[120], 3}, {NULL}};
const Segment SGI51851[] = {{&S79773[86], 5}, {NULL}};
const Segment SGI51852[] = {{&S79773[144], 13}, {NULL}};
const Segment SGI51853[] = {{&S79481[45], 21}, {NULL}};
const Segment SGI51854[] = {{&S79481[30], 6}, {NULL}};
const Segment SGI51855[] = {{&S79481[125], 3}, {NULL}};
const Segment SGI51856[] = {{&S79481[91], 5}, {NULL}};
const Segment SGI51857[] = {{&S79481[149], 13}, {NULL}};
const Segment SGI51858[] = {{&S79828[44], 7}, {NULL}};
const Segment SGI51859[] = {{&S79828[44], 31}, {NULL}};
const Segment SGI51860[] = {{&S79828[30], 6}, {NULL}};
const Segment SGI51861[] = {{&S79828[130], 3}, {NULL}};
const Segment SGI51862[] = {{&S79828[96], 5}, {NULL}};
const Segment SGI51863[] = {{&S79828[75], 8}, {NULL}};
const Segment SGI51864[] = {{&S79866[44], 10}, {NULL}};
const Segment SGI51865[] = {{&S79866[44], 27}, {NULL}};
const Segment SGI51866[] = {{&S79866[30], 6}, {NULL}};
const Segment SGI51867[] = {{&S79866[85], 6}, {NULL}};
const Segment SGI51868[] = {{&S79866[112], 13}, {NULL}};
const Segment SGI51869[] = {{&S79533[45], 10}, {NULL}};
const Segment SGI51870[] = {{&S79533[30], 6}, {NULL}};
const Segment SGI51871[] = {{&S79533[120], 3}, {NULL}};
const Segment SGI51872[] = {{&S79533[86], 5}, {NULL}};
const Segment SGI51873[] = {{&S79533[144], 13}, {NULL}};
const Segment SGI51874[] = {{&S79564[34], 10}, {NULL}};
const Segment SGI51875[] = {{&S79564[27], 6}, {NULL}};
const Segment SGI51876[] = {{&S79564[109], 2}, {NULL}};
const Segment SGI51877[] = {{&S79564[75], 6}, {NULL}};
const Segment SGI51878[] = {{&S79564[143], 8}, {NULL}};
const Segment SGI51879[] = {{&S79591[31], 18}, {NULL}};
const Segment SGI51880[] = {{&S79591[64], 3}, {NULL}};
const Segment SGI51881[] = {{&S79599[44], 18}, {NULL}};
const Segment SGI51882[] = {{&S79599[44], 39}, {NULL}};
const Segment SGI51883[] = {{&S79599[30], 6}, {NULL}};
const Segment SGI51884[] = {{&S79599[100], 6}, {NULL}};
const Segment SGI51885[] = {{&S79600[44], 39}, {NULL}};
const Segment SGI51886[] = {{&S79600[30], 6}, {NULL}};
const Segment SGI51887[] = {{&S79601[44], 18}, {NULL}};
const Segment SGI51888[] = {{&S79601[44], 39}, {NULL}};
const Segment SGI51889[] = {{&S79601[30], 6}, {NULL}};
const Segment SGI51890[] = {{&S79601[131], 3}, {NULL}};
const Segment SGI51891[] = {{&S79601[97], 5}, {NULL}};
const Segment SGI51892[] = {{&S94922[31], 12}, {NULL}};
const Segment SGI51893[] = {{&S94922[58], 3}, {NULL}};
const Segment SGI51894[] = {{&S79626[44], 7}, {NULL}};
const Segment SGI51895[] = {{&S79626[44], 18}, {NULL}};
const Segment SGI51896[] = {{&S79626[30], 6}, {NULL}};
const Segment SGI51897[] = {{&S79626[110], 3}, {NULL}};
const Segment SGI51898[] = {{&S79626[76], 5}, {NULL}};
const Segment SGI51899[] = {{&S79627[38], 7}, {NULL}};
const Segment SGI51900[] = {{&S79627[31], 6}, {NULL}};
const Segment SGI51901[] = {{&S80005[59], 10}, {NULL}};
const Segment SGI51902[] = {{&S80005[59], 27}, {NULL}};
const Segment SGI51903[] = {{&S80005[45], 6}, {NULL}};
const Segment SGI51904[] = {{&S80005[134], 3}, {NULL}};
const Segment SGI51905[] = {{&S80005[100], 5}, {NULL}};
const Segment SGI51906[] = {{&S80005[0], 14}, {NULL}};
const Segment SGI51907[] = {{&S82841[45], 10}, {NULL}};
const Segment SGI51908[] = {{&S82841[30], 6}, {NULL}};
const Segment SGI51909[] = {{&S82841[120], 3}, {NULL}};
const Segment SGI51910[] = {{&S82841[86], 5}, {NULL}};
const Segment SGI51911[] = {{&S82841[154], 3}, {NULL}};
const Segment SGI51912[] = {{&S80065[44], 5}, {NULL}};
const Segment SGI51913[] = {{&S80065[44], 16}, {NULL}};
const Segment SGI51914[] = {{&S80065[30], 6}, {NULL}};
const Segment SGI51915[] = {{&S80065[108], 3}, {NULL}};
const Segment SGI51916[] = {{&S80065[74], 5}, {NULL}};
const Segment SGI51917[] = {{&S79690[31], 19}, {NULL}};
const Segment SGI51918[] = {{&S79690[65], 3}, {NULL}};
const Segment SGI51919[] = {{&S79712[44], 21}, {NULL}};
const Segment SGI51920[] = {{&S79712[44], 32}, {NULL}};
const Segment SGI51921[] = {{&S79712[30], 6}, {NULL}};
const Segment SGI51922[] = {{&S79712[124], 3}, {NULL}};
const Segment SGI51923[] = {{&S79712[90], 5}, {NULL}};
const Segment SGI51924[] = {{&S79712[148], 11}, {NULL}};
const Segment SGI51925[] = {{&S79713[44], 11}, {NULL}};
const Segment SGI51926[] = {{&S79713[44], 22}, {NULL}};
const Segment SGI51927[] = {{&S79713[30], 6}, {NULL}};
const Segment SGI51928[] = {{&S79713[114], 3}, {NULL}};
const Segment SGI51929[] = {{&S79713[80], 5}, {NULL}};
const Segment SGI51930[] = {{&S79713[138], 11}, {NULL}};
const Segment SGI51931[] = {{&S79714[44], 15}, {NULL}};
const Segment SGI51932[] = {{&S79714[44], 26}, {NULL}};
const Segment SGI51933[] = {{&S79714[30], 6}, {NULL}};
const Segment SGI51934[] = {{&S79714[118], 3}, {NULL}};
const Segment SGI51935[] = {{&S79714[84], 5}, {NULL}};
const Segment SGI51936[] = {{&S79714[142], 11}, {NULL}};
const Segment SGI51937[] = {{&S95206[59], 12}, {NULL}};
const Segment SGI51938[] = {{&S95206[59], 23}, {NULL}};
const Segment SGI51939[] = {{&S95206[45], 6}, {NULL}};
const Segment SGI51940[] = {{&S95206[130], 3}, {NULL}};
const Segment SGI51941[] = {{&S95206[96], 5}, {NULL}};
const Segment SGI51942[] = {{&S83049[44], 17}, {NULL}};
const Segment SGI51943[] = {{&S83049[44], 28}, {NULL}};
const Segment SGI51944[] = {{&S83049[30], 6}, {NULL}};
const Segment SGI51945[] = {{&S83049[120], 3}, {NULL}};
const Segment SGI51946[] = {{&S83049[86], 5}, {NULL}};
const Segment SGI51947[] = {{&S80299[45], 7}, {NULL}};
const Segment SGI51948[] = {{&S80299[30], 6}, {NULL}};
const Segment SGI51949[] = {{&S80299[58], 3}, {NULL}};
const Segment SGI51950[] = {{&S80299[84], 5}, {NULL}};
const Segment SGI51951[] = {{&S80299[142], 13}, {NULL}};
const Segment SGI51952[] = {{&S79829[44], 20}, {NULL}};
const Segment SGI51953[] = {{&S79829[44], 45}, {NULL}};
const Segment SGI51954[] = {{&S79829[30], 6}, {NULL}};
const Segment SGI51955[] = {{&S79829[144], 3}, {NULL}};
const Segment SGI51956[] = {{&S79829[110], 5}, {NULL}};
const Segment SGI51957[] = {{&S79829[89], 8}, {NULL}};
const Segment SGI51958[] = {{&S79830[44], 7}, {NULL}};
const Segment SGI51959[] = {{&S79830[44], 32}, {NULL}};
const Segment SGI51960[] = {{&S79830[30], 6}, {NULL}};
const Segment SGI51961[] = {{&S79830[124], 3}, {NULL}};
const Segment SGI51962[] = {{&S79830[90], 5}, {NULL}};
const Segment SGI51963[] = {{&S79830[148], 7}, {NULL}};
const Segment SGI51964[] = {{&S79831[44], 20}, {NULL}};
const Segment SGI51965[] = {{&S79831[44], 45}, {NULL}};
const Segment SGI51966[] = {{&S79831[30], 6}, {NULL}};
const Segment SGI51967[] = {{&S79831[137], 3}, {NULL}};
const Segment SGI51968[] = {{&S79831[103], 5}, {NULL}};
const Segment SGI51969[] = {{&S79831[161], 7}, {NULL}};
const Segment SGI51970[] = {{&S79865[44], 10}, {NULL}};
const Segment SGI51971[] = {{&S79865[44], 27}, {NULL}};
const Segment SGI51972[] = {{&S79865[30], 6}, {NULL}};
const Segment SGI51973[] = {{&S79865[85], 6}, {NULL}};
const Segment SGI51974[] = {{&S79865[112], 13}, {NULL}};
const Segment SGI51975[] = {{&S80404[44], 5}, {NULL}};
const Segment SGI51976[] = {{&S80404[44], 16}, {NULL}};
const Segment SGI51977[] = {{&S80404[30], 6}, {NULL}};
const Segment SGI51978[] = {{&S80404[108], 3}, {NULL}};
const Segment SGI51979[] = {{&S80404[74], 5}, {NULL}};
const Segment SGI51980[] = {{&S79893[44], 9}, {NULL}};
const Segment SGI51981[] = {{&S79893[44], 20}, {NULL}};
const Segment SGI51982[] = {{&S79893[30], 6}, {NULL}};
const Segment SGI51983[] = {{&S79893[112], 3}, {NULL}};
const Segment SGI51984[] = {{&S79893[78], 5}, {NULL}};
const Segment SGI51985[] = {{&S79894[44], 9}, {NULL}};
const Segment SGI51986[] = {{&S79894[44], 20}, {NULL}};
const Segment SGI51987[] = {{&S79894[30], 6}, {NULL}};
const Segment SGI51988[] = {{&S79894[81], 6}, {NULL}};
const Segment SGI51989[] = {{&S84669[31], 5}, {NULL}};
const Segment SGI51990[] = {{&S84669[51], 3}, {NULL}};
const Segment SGI51991[] = {{&S95396[31], 10}, {NULL}};
const Segment SGI51992[] = {{&S95396[56], 3}, {NULL}};
const Segment SGI51993[] = {{&S95427[44], 10}, {NULL}};
const Segment SGI51994[] = {{&S95427[44], 22}, {NULL}};
const Segment SGI51995[] = {{&S95427[30], 6}, {NULL}};
const Segment SGI51996[] = {{&S95427[114], 3}, {NULL}};
const Segment SGI51997[] = {{&S95427[80], 5}, {NULL}};
const Segment SGI51998[] = {{&S80471[44], 7}, {NULL}};
const Segment SGI51999[] = {{&S80471[44], 18}, {NULL}};
const Segment SGI52000[] = {{&S80471[30], 6}, {NULL}};
const Segment SGI52001[] = {{&S80471[110], 3}, {NULL}};
const Segment SGI52002[] = {{&S80471[76], 5}, {NULL}};
const Segment SGI52003[] = {{&S80472[44], 13}, {NULL}};
const Segment SGI52004[] = {{&S80472[44], 30}, {NULL}};
const Segment SGI52005[] = {{&S80472[30], 6}, {NULL}};
const Segment SGI52006[] = {{&S80472[123], 3}, {NULL}};
const Segment SGI52007[] = {{&S80472[88], 5}, {NULL}};
const Segment SGI52008[] = {{&S88316[42], 12}, {NULL}};
const Segment SGI52009[] = {{&S88316[42], 23}, {NULL}};
const Segment SGI52010[] = {{&S88316[30], 4}, {NULL}};
const Segment SGI52011[] = {{&S88316[113], 3}, {NULL}};
const Segment SGI52012[] = {{&S88316[79], 5}, {NULL}};
const Segment SGI52013[] = {{&S80600[45], 7}, {NULL}};
const Segment SGI52014[] = {{&S80600[30], 6}, {NULL}};
const Segment SGI52015[] = {{&S80600[58], 3}, {NULL}};
const Segment SGI52016[] = {{&S80600[84], 5}, {NULL}};
const Segment SGI52017[] = {{&S80600[142], 13}, {NULL}};
const Segment SGI52018[] = {{&S80603[44], 6}, {NULL}};
const Segment SGI52019[] = {{&S80603[44], 17}, {NULL}};
const Segment SGI52020[] = {{&S80603[30], 6}, {NULL}};
const Segment SGI52021[] = {{&S80603[109], 3}, {NULL}};
const Segment SGI52022[] = {{&S80603[75], 5}, {NULL}};
const Segment SGI52023[] = {{&S80604[44], 6}, {NULL}};
const Segment SGI52024[] = {{&S80604[44], 17}, {NULL}};
const Segment SGI52025[] = {{&S80604[30], 6}, {NULL}};
const Segment SGI52026[] = {{&S80604[78], 6}, {NULL}};
const Segment SGI52027[] = {{&S80114[44], 14}, {NULL}};
const Segment SGI52028[] = {{&S80114[44], 25}, {NULL}};
const Segment SGI52029[] = {{&S80114[30], 6}, {NULL}};
const Segment SGI52030[] = {{&S80114[83], 6}, {NULL}};
const Segment SGI52031[] = {{&S80114[110], 13}, {NULL}};
const Segment SGI52032[] = {{&S80666[45], 7}, {NULL}};
const Segment SGI52033[] = {{&S80666[30], 6}, {NULL}};
const Segment SGI52034[] = {{&S80666[58], 3}, {NULL}};
const Segment SGI52035[] = {{&S80666[84], 5}, {NULL}};
const Segment SGI52036[] = {{&S80666[142], 13}, {NULL}};
const Segment SGI52037[] = {{&S80164[44], 10}, {NULL}};
const Segment SGI52038[] = {{&S89733[44], 12}, {NULL}};
const Segment SGI52039[] = {{&S80164[44], 27}, {NULL}};
const Segment SGI52040[] = {{&S89733[44], 29}, {NULL}};
const Segment SGI52041[] = {{&S80164[30], 6}, {NULL}};
const Segment SGI52042[] = {{&S89733[30], 6}, {NULL}};
const Segment SGI52043[] = {{&S80164[119], 3}, {NULL}};
const Segment SGI52044[] = {{&S89733[121], 3}, {NULL}};
const Segment SGI52045[] = {{&S80164[85], 5}, {NULL}};
const Segment SGI52046[] = {{&S89733[87], 5}, {NULL}};
const Segment SGI52047[] = {{&S80164[143], 11}, {NULL}};
const Segment SGI52048[] = {{&S89734[44], 12}, {NULL}};
const Segment SGI52049[] = {{&S89734[44], 29}, {NULL}};
const Segment SGI52050[] = {{&S89734[30], 6}, {NULL}};
const Segment SGI52051[] = {{&S89734[121], 3}, {NULL}};
const Segment SGI52052[] = {{&S89734[87], 5}, {NULL}};
const Segment SGI52053[] = {{&S80224[44], 17}, {NULL}};
const Segment SGI52054[] = {{&S80224[30], 6}, {NULL}};
const Segment SGI52055[] = {{&S80224[109], 3}, {NULL}};
const Segment SGI52056[] = {{&S80224[75], 5}, {NULL}};
const Segment SGI52057[] = {{&S95736[59], 12}, {NULL}};
const Segment SGI52058[] = {{&S95736[59], 23}, {NULL}};
const Segment SGI52059[] = {{&S95736[45], 6}, {NULL}};
const Segment SGI52060[] = {{&S95736[130], 3}, {NULL}};
const Segment SGI52061[] = {{&S95736[96], 5}, {NULL}};
const Segment SGI52062[] = {{&S95736[0], 14}, {NULL}};
const Segment SGI52063[] = {{&S95759[59], 11}, {NULL}};
const Segment SGI52064[] = {{&S95759[59], 22}, {NULL}};
const Segment SGI52065[] = {{&S95759[45], 6}, {NULL}};
const Segment SGI52066[] = {{&S95759[129], 3}, {NULL}};
const Segment SGI52067[] = {{&S95759[95], 5}, {NULL}};
const Segment SGI52068[] = {{&S95762[45], 9}, {NULL}};
const Segment SGI52069[] = {{&S95762[30], 6}, {NULL}};
const Segment SGI52070[] = {{&S95762[113], 3}, {NULL}};
const Segment SGI52071[] = {{&S95762[79], 5}, {NULL}};
const Segment SGI52072[] = {{&S95762[137], 13}, {NULL}};
const Segment SGI52073[] = {{&S80947[31], 5}, {NULL}};
const Segment SGI52074[] = {{&S80947[51], 3}, {NULL}};
const Segment SGI52075[] = {{&S83558[44], 10}, {NULL}};
const Segment SGI52076[] = {{&S83558[44], 27}, {NULL}};
const Segment SGI52077[] = {{&S83558[30], 6}, {NULL}};
const Segment SGI52078[] = {{&S83558[85], 6}, {NULL}};
const Segment SGI52079[] = {{&S83558[122], 3}, {NULL}};
const Segment SGI52080[] = {{&S95789[45], 7}, {NULL}};
const Segment SGI52081[] = {{&S95789[30], 6}, {NULL}};
const Segment SGI52082[] = {{&S95789[117], 3}, {NULL}};
const Segment SGI52083[] = {{&S95789[83], 5}, {NULL}};
const Segment SGI52084[] = {{&S95789[141], 13}, {NULL}};
const Segment SGI52085[] = {{&S80979[44], 10}, {NULL}};
const Segment SGI52086[] = {{&S80979[44], 27}, {NULL}};
const Segment SGI52087[] = {{&S80979[30], 6}, {NULL}};
const Segment SGI52088[] = {{&S80979[85], 6}, {NULL}};
const Segment SGI52089[] = {{&S80979[112], 13}, {NULL}};
const Segment SGI52090[] = {{&S80556[44], 14}, {NULL}};
const Segment SGI52091[] = {{&S80556[44], 35}, {NULL}};
const Segment SGI52092[] = {{&S80556[30], 6}, {NULL}};
const Segment SGI52093[] = {{&S80556[93], 6}, {NULL}};
const Segment SGI52094[] = {{&S80556[120], 13}, {NULL}};
const Segment SGI52095[] = {{&S80565[44], 79}, {NULL}};
const Segment SGI52096[] = {{&S80565[44], 100}, {NULL}};
const Segment SGI52097[] = {{&S80565[30], 6}, {NULL}};
const Segment SGI52098[] = {{&S80565[191], 3}, {NULL}};
const Segment SGI52099[] = {{&S80565[117], 5}, {NULL}};
const Segment SGI52100[] = {{&S81076[45], 10}, {NULL}};
const Segment SGI52101[] = {{&S81076[30], 6}, {NULL}};
const Segment SGI52102[] = {{&S81076[120], 3}, {NULL}};
const Segment SGI52103[] = {{&S81076[86], 5}, {NULL}};
const Segment SGI52104[] = {{&S81076[144], 13}, {NULL}};
const Segment SGI52105[] = {{&S83685[45], 9}, {NULL}};
const Segment SGI52106[] = {{&S83685[30], 6}, {NULL}};
const Segment SGI52107[] = {{&S83685[113], 3}, {NULL}};
const Segment SGI52108[] = {{&S83685[79], 5}, {NULL}};
const Segment SGI52109[] = {{&S83685[147], 3}, {NULL}};
const Segment SGI52110[] = {{&S80584[44], 12}, {NULL}};
const Segment SGI52111[] = {{&S80584[44], 22}, {NULL}};
const Segment SGI52112[] = {{&S80584[30], 6}, {NULL}};
const Segment SGI52113[] = {{&S80584[114], 3}, {NULL}};
const Segment SGI52114[] = {{&S80584[80], 5}, {NULL}};
const Segment SGI52115[] = {{&S80586[44], 16}, {NULL}};
const Segment SGI52116[] = {{&S80586[44], 26}, {NULL}};
const Segment SGI52117[] = {{&S80586[30], 6}, {NULL}};
const Segment SGI52118[] = {{&S80586[118], 3}, {NULL}};
const Segment SGI52119[] = {{&S80586[84], 5}, {NULL}};
const Segment SGI52120[] = {{&S80587[44], 11}, {NULL}};
const Segment SGI52121[] = {{&S80587[44], 22}, {NULL}};
const Segment SGI52122[] = {{&S80587[30], 6}, {NULL}};
const Segment SGI52123[] = {{&S80587[133], 3}, {NULL}};
const Segment SGI52124[] = {{&S80587[99], 5}, {NULL}};
const Segment SGI52125[] = {{&S81177[44], 27}, {NULL}};
const Segment SGI52126[] = {{&S81177[30], 6}, {NULL}};
const Segment SGI52127[] = {{&S81177[137], 3}, {NULL}};
const Segment SGI52128[] = {{&S81177[103], 5}, {NULL}};
const Segment SGI52129[] = {{&S81238[44], 7}, {NULL}};
const Segment SGI52130[] = {{&S81238[44], 28}, {NULL}};
const Segment SGI52131[] = {{&S81238[30], 6}, {NULL}};
const Segment SGI52132[] = {{&S81238[120], 3}, {NULL}};
const Segment SGI52133[] = {{&S81238[86], 5}, {NULL}};
const Segment SGI52134[] = {{&S81239[44], 7}, {NULL}};
const Segment SGI52135[] = {{&S81239[44], 21}, {NULL}};
const Segment SGI52136[] = {{&S81239[30], 6}, {NULL}};
const Segment SGI52137[] = {{&S81239[113], 3}, {NULL}};
const Segment SGI52138[] = {{&S81239[79], 5}, {NULL}};
const Segment SGI52139[] = {{&S81240[44], 7}, {NULL}};
const Segment SGI52140[] = {{&S81240[44], 28}, {NULL}};
const Segment SGI52141[] = {{&S81240[30], 6}, {NULL}};
const Segment SGI52142[] = {{&S81240[120], 3}, {NULL}};
const Segment SGI52143[] = {{&S81240[86], 5}, {NULL}};
const Segment SGI52144[] = {{&S81241[44], 7}, {NULL}};
const Segment SGI52145[] = {{&S81241[44], 28}, {NULL}};
const Segment SGI52146[] = {{&S81241[30], 6}, {NULL}};
const Segment SGI52147[] = {{&S81241[89], 4}, {NULL}};
const Segment SGI52148[] = {{&S81241[89], 3}, {NULL}};
const Segment SGI52149[] = {{&S81242[44], 7}, {NULL}};
const Segment SGI52150[] = {{&S81242[44], 28}, {NULL}};
const Segment SGI52151[] = {{&S81242[30], 6}, {NULL}};
const Segment SGI52152[] = {{&S81242[89], 6}, {NULL}};
const Segment SGI52153[] = {{&S81243[44], 7}, {NULL}};
const Segment SGI52154[] = {{&S81243[44], 28}, {NULL}};
const Segment SGI52155[] = {{&S81243[30], 6}, {NULL}};
const Segment SGI52156[] = {{&S81243[89], 4}, {NULL}};
const Segment SGI52157[] = {{&S81243[89], 3}, {NULL}};
const Segment SGI52158[] = {{&S81246[44], 10}, {NULL}};
const Segment SGI52159[] = {{&S81246[44], 27}, {NULL}};
const Segment SGI52160[] = {{&S81246[30], 6}, {NULL}};
const Segment SGI52161[] = {{&S81246[119], 3}, {NULL}};
const Segment SGI52162[] = {{&S81246[85], 5}, {NULL}};
const Segment SGI52163[] = {{&S81247[44], 10}, {NULL}};
const Segment SGI52164[] = {{&S81247[44], 27}, {NULL}};
const Segment SGI52165[] = {{&S81247[30], 6}, {NULL}};
const Segment SGI52166[] = {{&S81247[88], 4}, {NULL}};
const Segment SGI52167[] = {{&S81247[88], 3}, {NULL}};
const Segment SGI52168[] = {{&S81263[45], 7}, {NULL}};
const Segment SGI52169[] = {{&S81263[30], 6}, {NULL}};
const Segment SGI52170[] = {{&S81263[58], 3}, {NULL}};
const Segment SGI52171[] = {{&S81263[84], 5}, {NULL}};
const Segment SGI52172[] = {{&S81263[142], 13}, {NULL}};
const Segment SGI52173[] = {{&S80767[44], 12}, {NULL}};
const Segment SGI52174[] = {{&S80767[44], 23}, {NULL}};
const Segment SGI52175[] = {{&S80767[30], 6}, {NULL}};
const Segment SGI52176[] = {{&S80767[115], 3}, {NULL}};
const Segment SGI52177[] = {{&S80767[81], 5}, {NULL}};
const Segment SGI52178[] = {{&S88990[42], 13}, {NULL}};
const Segment SGI52179[] = {{&S88990[42], 24}, {NULL}};
const Segment SGI52180[] = {{&S88990[30], 4}, {NULL}};
const Segment SGI52181[] = {{&S88990[114], 3}, {NULL}};
const Segment SGI52182[] = {{&S88990[80], 5}, {NULL}};
const Segment SGI52183[] = {{&S81323[44], 26}, {NULL}};
const Segment SGI52184[] = {{&S81323[30], 6}, {NULL}};
const Segment SGI52185[] = {{&S81323[136], 3}, {NULL}};
const Segment SGI52186[] = {{&S81323[102], 5}, {NULL}};
const Segment SGI52187[] = {{&S96178[44], 9}, {NULL}};
const Segment SGI52188[] = {{&S96178[44], 21}, {NULL}};
const Segment SGI52189[] = {{&S96178[30], 6}, {NULL}};
const Segment SGI52190[] = {{&S96178[79], 6}, {NULL}};
const Segment SGI52191[] = {{&S96178[106], 13}, {NULL}};
const Segment SGI52192[] = {{&S83835[45], 10}, {NULL}};
const Segment SGI52193[] = {{&S83835[30], 6}, {NULL}};
const Segment SGI52194[] = {{&S83835[120], 3}, {NULL}};
const Segment SGI52195[] = {{&S83835[86], 5}, {NULL}};
const Segment SGI52196[] = {{&S83835[154], 3}, {NULL}};
const Segment SGI52197[] = {{&S81387[44], 9}, {NULL}};
const Segment SGI52198[] = {{&S81387[44], 20}, {NULL}};
const Segment SGI52199[] = {{&S81387[30], 6}, {NULL}};
const Segment SGI52200[] = {{&S81387[112], 3}, {NULL}};
const Segment SGI52201[] = {{&S81387[78], 5}, {NULL}};
const Segment SGI52202[] = {{&S81404[44], 17}, {NULL}};
const Segment SGI52203[] = {{&S81404[30], 6}, {NULL}};
const Segment SGI52204[] = {{&S81404[109], 3}, {NULL}};
const Segment SGI52205[] = {{&S81404[75], 5}, {NULL}};
const Segment SGI52206[] = {{&S80940[31], 8}, {NULL}};
const Segment SGI52207[] = {{&S80940[54], 3}, {NULL}};
const Segment SGI52208[] = {{&S80942[44], 8}, {NULL}};
const Segment SGI52209[] = {{&S80942[44], 19}, {NULL}};
const Segment SGI52210[] = {{&S80942[30], 6}, {NULL}};
const Segment SGI52211[] = {{&S80942[111], 3}, {NULL}};
const Segment SGI52212[] = {{&S80942[77], 5}, {NULL}};
const Segment SGI52213[] = {{&S80943[38], 8}, {NULL}};
const Segment SGI52214[] = {{&S80943[31], 6}, {NULL}};
const Segment SGI52215[] = {{&S80968[44], 20}, {NULL}};
const Segment SGI52216[] = {{&S80968[44], 31}, {NULL}};
const Segment SGI52217[] = {{&S80968[30], 6}, {NULL}};
const Segment SGI52218[] = {{&S80968[89], 6}, {NULL}};
const Segment SGI52219[] = {{&S80968[116], 13}, {NULL}};
const Segment SGI52220[] = {{&S80977[44], 10}, {NULL}};
const Segment SGI52221[] = {{&S80977[44], 27}, {NULL}};
const Segment SGI52222[] = {{&S83942[44], 10}, {NULL}};
const Segment SGI52223[] = {{&S80977[30], 6}, {NULL}};
const Segment SGI52224[] = {{&S83942[44], 27}, {NULL}};
const Segment SGI52225[] = {{&S83942[30], 6}, {NULL}};
const Segment SGI52226[] = {{&S80977[85], 6}, {NULL}};
const Segment SGI52227[] = {{&S83942[149], 4}, {NULL}};
const Segment SGI52228[] = {{&S83942[175], 3}, {NULL}};
const Segment SGI52229[] = {{&S80977[112], 13}, {NULL}};
const Segment SGI52230[] = {{&S83942[119], 3}, {&S83942[224], 5}, {NULL}};
const Segment SGI52231[] = {{&S83942[85], 5}, {NULL}};
const Segment SGI52232[] = {{&S81514[44], 7}, {NULL}};
const Segment SGI52233[] = {{&S81514[44], 25}, {NULL}};
const Segment SGI52234[] = {{&S81514[30], 6}, {NULL}};
const Segment SGI52235[] = {{&S81514[83], 6}, {NULL}};
const Segment SGI52236[] = {{&S81514[110], 13}, {NULL}};
const Segment SGI52237[] = {{&S81032[44], 14}, {NULL}};
const Segment SGI52238[] = {{&S81032[44], 25}, {NULL}};
const Segment SGI52239[] = {{&S81032[30], 6}, {NULL}};
const Segment SGI52240[] = {{&S81032[83], 6}, {NULL}};
const Segment SGI52241[] = {{&S81032[110], 13}, {NULL}};
const Segment SGI52242[] = {{&S96360[45], 10}, {NULL}};
const Segment SGI52243[] = {{&S96360[30], 6}, {NULL}};
const Segment SGI52244[] = {{&S81074[45], 10}, {NULL}};
const Segment SGI52245[] = {{&S81074[30], 6}, {NULL}};
const Segment SGI52246[] = {{&S81074[120], 3}, {NULL}};
const Segment SGI52247[] = {{&S84037[44], 10}, {NULL}};
const Segment SGI52248[] = {{&S81074[86], 5}, {NULL}};
const Segment SGI52249[] = {{&S84037[44], 27}, {NULL}};
const Segment SGI52250[] = {{&S84037[30], 6}, {NULL}};
const Segment SGI52251[] = {{&S81074[144], 13}, {NULL}};
const Segment SGI52252[] = {{&S84037[85], 6}, {NULL}};
const Segment SGI52253[] = {{&S84037[122], 3}, {NULL}};
const Segment SGI52254[] = {{&S96360[114], 3}, {NULL}};
const Segment SGI52255[] = {{&S96360[80], 5}, {NULL}};
const Segment SGI52256[] = {{&S96360[138], 13}, {NULL}};
const Segment SGI52257[] = {{&S96368[45], 11}, {NULL}};
const Segment SGI52258[] = {{&S96368[30], 6}, {NULL}};
const Segment SGI52259[] = {{&S96368[121], 3}, {NULL}};
const Segment SGI52260[] = {{&S96368[87], 5}, {NULL}};
const Segment SGI52261[] = {{&S96368[145], 13}, {NULL}};
const Segment SGI52262[] = {{&S81142[44], 22}, {NULL}};
const Segment SGI52263[] = {{&S81142[44], 33}, {NULL}};
const Segment SGI52264[] = {{&S81142[30], 6}, {NULL}};
const Segment SGI52265[] = {{&S81142[125], 3}, {NULL}};
const Segment SGI52266[] = {{&S81142[91], 5}, {NULL}};
const Segment SGI52267[] = {{&S81168[44], 21}, {NULL}};
const Segment SGI52268[] = {{&S81168[44], 32}, {NULL}};
const Segment SGI52269[] = {{&S81168[30], 6}, {NULL}};
const Segment SGI52270[] = {{&S81168[124], 3}, {NULL}};
const Segment SGI52271[] = {{&S81168[90], 5}, {NULL}};
const Segment SGI52272[] = {{&S81168[148], 7}, {NULL}};
const Segment SGI52273[] = {{&S81170[44], 11}, {NULL}};
const Segment SGI52274[] = {{&S81170[44], 22}, {NULL}};
const Segment SGI52275[] = {{&S81170[30], 6}, {NULL}};
const Segment SGI52276[] = {{&S81170[114], 3}, {NULL}};
const Segment SGI52277[] = {{&S81170[80], 5}, {NULL}};
const Segment SGI52278[] = {{&S81170[138], 7}, {NULL}};
const Segment SGI52279[] = {{&S81172[44], 10}, {NULL}};
const Segment SGI52280[] = {{&S81172[44], 21}, {NULL}};
const Segment SGI52281[] = {{&S81172[30], 6}, {NULL}};
const Segment SGI52282[] = {{&S81172[113], 3}, {NULL}};
const Segment SGI52283[] = {{&S81172[79], 5}, {NULL}};
const Segment SGI52284[] = {{&S81679[31], 5}, {NULL}};
const Segment SGI52285[] = {{&S81679[51], 3}, {NULL}};
const Segment SGI52286[] = {{&S81198[44], 13}, {NULL}};
const Segment SGI52287[] = {{&S81198[44], 30}, {NULL}};
const Segment SGI52288[] = {{&S81198[30], 6}, {NULL}};
const Segment SGI52289[] = {{&S81198[88], 6}, {NULL}};
const Segment SGI52290[] = {{&S81198[115], 13}, {NULL}};
const Segment SGI52291[] = {{&S81696[45], 10}, {NULL}};
const Segment SGI52292[] = {{&S81696[30], 6}, {NULL}};
const Segment SGI52293[] = {{&S81696[120], 3}, {NULL}};
const Segment SGI52294[] = {{&S81696[86], 5}, {NULL}};
const Segment SGI52295[] = {{&S81696[144], 13}, {NULL}};
const Segment SGI52296[] = {{&S81279[45], 14}, {NULL}};
const Segment SGI52297[] = {{&S81279[30], 6}, {NULL}};
const Segment SGI52298[] = {{&S81279[124], 3}, {NULL}};
const Segment SGI52299[] = {{&S81279[90], 5}, {NULL}};
const Segment SGI52300[] = {{&S81279[148], 13}, {NULL}};
const Segment SGI52301[] = {{&S81280[44], 14}, {NULL}};
const Segment SGI52302[] = {{&S81280[44], 31}, {NULL}};
const Segment SGI52303[] = {{&S81280[30], 6}, {NULL}};
const Segment SGI52304[] = {{&S81280[124], 3}, {NULL}};
const Segment SGI52305[] = {{&S81280[89], 6}, {NULL}};
const Segment SGI52306[] = {{&S81280[144], 8}, {NULL}};
const Segment SGI52307[] = {{&S81280[153], 13}, {NULL}};
const Segment SGI52308[] = {{&S84315[44], 10}, {NULL}};
const Segment SGI52309[] = {{&S84315[44], 27}, {NULL}};
const Segment SGI52310[] = {{&S84315[30], 6}, {NULL}};
const Segment SGI52311[] = {{&S84315[85], 6}, {NULL}};
const Segment SGI52312[] = {{&S84315[122], 3}, {NULL}};
const Segment SGI52313[] = {{&S81366[45], 14}, {NULL}};
const Segment SGI52314[] = {{&S81366[30], 6}, {NULL}};
const Segment SGI52315[] = {{&S81366[128], 3}, {NULL}};
const Segment SGI52316[] = {{&S81366[94], 5}, {NULL}};
const Segment SGI52317[] = {{&S81366[152], 13}, {NULL}};
const Segment SGI52318[] = {{&S81367[44], 14}, {NULL}};
const Segment SGI52319[] = {{&S81367[44], 35}, {NULL}};
const Segment SGI52320[] = {{&S81367[30], 6}, {NULL}};
const Segment SGI52321[] = {{&S81367[128], 3}, {NULL}};
const Segment SGI52322[] = {{&S81367[93], 6}, {NULL}};
const Segment SGI52323[] = {{&S81367[148], 8}, {NULL}};
const Segment SGI52324[] = {{&S81367[157], 13}, {NULL}};
const Segment SGI52325[] = {{&S81864[44], 14}, {NULL}};
const Segment SGI52326[] = {{&S81864[44], 31}, {NULL}};
const Segment SGI52327[] = {{&S81864[30], 6}, {NULL}};
const Segment SGI52328[] = {{&S81864[123], 3}, {NULL}};
const Segment SGI52329[] = {{&S81864[89], 5}, {NULL}};
const Segment SGI52330[] = {{&S81376[44], 22}, {NULL}};
const Segment SGI52331[] = {{&S81376[44], 33}, {NULL}};
const Segment SGI52332[] = {{&S81376[30], 6}, {NULL}};
const Segment SGI52333[] = {{&S81376[91], 6}, {NULL}};
const Segment SGI52334[] = {{&S81376[118], 13}, {NULL}};
const Segment SGI52335[] = {{&S81388[44], 9}, {NULL}};
const Segment SGI52336[] = {{&S81388[44], 20}, {NULL}};
const Segment SGI52337[] = {{&S81388[30], 6}, {NULL}};
const Segment SGI52338[] = {{&S81388[112], 3}, {NULL}};
const Segment SGI52339[] = {{&S81388[78], 5}, {NULL}};
const Segment SGI52340[] = {{&S81883[44], 12}, {NULL}};
const Segment SGI52341[] = {{&S81883[44], 23}, {NULL}};
const Segment SGI52342[] = {{&S81883[30], 6}, {NULL}};
const Segment SGI52343[] = {{&S81883[115], 3}, {NULL}};
const Segment SGI52344[] = {{&S81883[81], 5}, {NULL}};
const Segment SGI52345[] = {{&S81389[44], 15}, {NULL}};
const Segment SGI52346[] = {{&S81389[44], 26}, {NULL}};
const Segment SGI52347[] = {{&S81389[30], 6}, {NULL}};
const Segment SGI52348[] = {{&S81389[118], 3}, {NULL}};
const Segment SGI52349[] = {{&S81389[84], 5}, {NULL}};
const Segment SGI52350[] = {{&S81392[44], 7}, {NULL}};
const Segment SGI52351[] = {{&S81392[44], 29}, {NULL}};
const Segment SGI52352[] = {{&S81392[30], 6}, {NULL}};
const Segment SGI52353[] = {{&S81392[121], 3}, {NULL}};
const Segment SGI52354[] = {{&S81392[87], 5}, {NULL}};
const Segment SGI52355[] = {{&S81393[44], 7}, {NULL}};
const Segment SGI52356[] = {{&S81393[44], 30}, {NULL}};
const Segment SGI52357[] = {{&S81393[30], 6}, {NULL}};
const Segment SGI52358[] = {{&S81393[122], 3}, {NULL}};
const Segment SGI52359[] = {{&S81393[88], 5}, {NULL}};
const Segment SGI52360[] = {{&S81394[44], 7}, {NULL}};
const Segment SGI52361[] = {{&S81394[44], 27}, {NULL}};
const Segment SGI52362[] = {{&S81394[30], 6}, {NULL}};
const Segment SGI52363[] = {{&S81394[119], 3}, {NULL}};
const Segment SGI52364[] = {{&S81394[85], 5}, {NULL}};
const Segment SGI52365[] = {{&S81395[31], 8}, {NULL}};
const Segment SGI52366[] = {{&S81395[54], 3}, {NULL}};
const Segment SGI52367[] = {{&S81405[44], 13}, {NULL}};
const Segment SGI52368[] = {{&S81405[44], 24}, {NULL}};
const Segment SGI52369[] = {{&S81405[30], 6}, {NULL}};
const Segment SGI52370[] = {{&S81405[116], 3}, {NULL}};
const Segment SGI52371[] = {{&S81405[82], 5}, {NULL}};
const Segment SGI52372[] = {{&S81406[44], 13}, {NULL}};
const Segment SGI52373[] = {{&S81406[44], 24}, {NULL}};
const Segment SGI52374[] = {{&S81406[30], 6}, {NULL}};
const Segment SGI52375[] = {{&S81406[85], 6}, {NULL}};
const Segment SGI52376[] = {{&S96647[45], 12}, {NULL}};
const Segment SGI52377[] = {{&S96647[30], 6}, {NULL}};
const Segment SGI52378[] = {{&S96647[116], 3}, {NULL}};
const Segment SGI52379[] = {{&S96647[82], 5}, {NULL}};
const Segment SGI52380[] = {{&S96647[140], 13}, {NULL}};
const Segment SGI52381[] = {{&S81450[44], 9}, {NULL}};
const Segment SGI52382[] = {{&S81450[44], 20}, {NULL}};
const Segment SGI52383[] = {{&S81450[30], 6}, {NULL}};
const Segment SGI52384[] = {{&S81450[112], 3}, {NULL}};
const Segment SGI52385[] = {{&S81450[78], 5}, {NULL}};
const Segment SGI52386[] = {{&S81959[44], 10}, {NULL}};
const Segment SGI52387[] = {{&S81959[44], 27}, {NULL}};
const Segment SGI52388[] = {{&S81959[30], 6}, {NULL}};
const Segment SGI52389[] = {{&S81451[44], 9}, {NULL}};
const Segment SGI52390[] = {{&S81451[44], 20}, {NULL}};
const Segment SGI52391[] = {{&S81451[30], 6}, {NULL}};
const Segment SGI52392[] = {{&S81959[119], 3}, {NULL}};
const Segment SGI52393[] = {{&S81959[85], 5}, {NULL}};
const Segment SGI52394[] = {{&S81451[112], 3}, {NULL}};
const Segment SGI52395[] = {{&S81451[78], 5}, {NULL}};
const Segment SGI52396[] = {{&S81960[44], 10}, {NULL}};
const Segment SGI52397[] = {{&S81960[44], 27}, {NULL}};
const Segment SGI52398[] = {{&S81960[30], 6}, {NULL}};
const Segment SGI52399[] = {{&S81452[44], 9}, {NULL}};
const Segment SGI52400[] = {{&S81452[44], 20}, {NULL}};
const Segment SGI52401[] = {{&S81452[30], 6}, {NULL}};
const Segment SGI52402[] = {{&S81960[88], 6}, {NULL}};
const Segment SGI52403[] = {{&S81452[112], 3}, {NULL}};
const Segment SGI52404[] = {{&S81452[78], 5}, {NULL}};
const Segment SGI52405[] = {{&S81453[44], 9}, {NULL}};
const Segment SGI52406[] = {{&S81453[44], 20}, {NULL}};
const Segment SGI52407[] = {{&S81453[30], 6}, {NULL}};
const Segment SGI52408[] = {{&S81453[112], 3}, {NULL}};
const Segment SGI52409[] = {{&S81453[78], 5}, {NULL}};
const Segment SGI52410[] = {{&S81454[44], 9}, {NULL}};
const Segment SGI52411[] = {{&S81454[44], 20}, {NULL}};
const Segment SGI52412[] = {{&S81454[30], 6}, {NULL}};
const Segment SGI52413[] = {{&S81454[112], 3}, {NULL}};
const Segment SGI52414[] = {{&S81454[78], 5}, {NULL}};
const Segment SGI52415[] = {{&S81993[44], 7}, {NULL}};
const Segment SGI52416[] = {{&S81993[44], 25}, {NULL}};
const Segment SGI52417[] = {{&S81993[30], 6}, {NULL}};
const Segment SGI52418[] = {{&S81993[83], 6}, {NULL}};
const Segment SGI52419[] = {{&S81993[110], 13}, {NULL}};
const Segment SGI52420[] = {{&S81492[44], 5}, {NULL}};
const Segment SGI52421[] = {{&S81492[44], 22}, {NULL}};
const Segment SGI52422[] = {{&S81492[30], 6}, {NULL}};
const Segment SGI52423[] = {{&S81492[114], 3}, {NULL}};
const Segment SGI52424[] = {{&S81492[80], 5}, {NULL}};
const Segment SGI52425[] = {{&S82025[44], 6}, {NULL}};
const Segment SGI52426[] = {{&S82025[44], 17}, {NULL}};
const Segment SGI52427[] = {{&S82025[30], 6}, {NULL}};
const Segment SGI52428[] = {{&S82025[109], 3}, {NULL}};
const Segment SGI52429[] = {{&S82025[75], 5}, {NULL}};
const Segment SGI52430[] = {{&S82028[44], 14}, {NULL}};
const Segment SGI52431[] = {{&S82028[44], 25}, {NULL}};
const Segment SGI52432[] = {{&S82028[30], 6}, {NULL}};
const Segment SGI52433[] = {{&S82028[117], 3}, {NULL}};
const Segment SGI52434[] = {{&S82028[83], 5}, {NULL}};
const Segment SGI52435[] = {{&S96734[45], 12}, {NULL}};
const Segment SGI52436[] = {{&S96734[30], 6}, {NULL}};
const Segment SGI52437[] = {{&S96734[116], 3}, {NULL}};
const Segment SGI52438[] = {{&S96734[82], 5}, {NULL}};
const Segment SGI52439[] = {{&S96734[140], 13}, {NULL}};
const Segment SGI52440[] = {{&S81527[44], 13}, {NULL}};
const Segment SGI52441[] = {{&S81527[44], 24}, {NULL}};
const Segment SGI52442[] = {{&S81527[30], 6}, {NULL}};
const Segment SGI52443[] = {{&S81527[116], 3}, {NULL}};
const Segment SGI52444[] = {{&S81527[82], 5}, {NULL}};
const Segment SGI52445[] = {{&S81529[44], 11}, {NULL}};
const Segment SGI52446[] = {{&S81529[44], 22}, {NULL}};
const Segment SGI52447[] = {{&S81529[30], 6}, {NULL}};
const Segment SGI52448[] = {{&S81529[114], 3}, {NULL}};
const Segment SGI52449[] = {{&S81529[80], 5}, {NULL}};
const Segment SGI52450[] = {{&S81532[59], 10}, {NULL}};
const Segment SGI52451[] = {{&S81532[59], 27}, {NULL}};
const Segment SGI52452[] = {{&S81532[45], 6}, {NULL}};
const Segment SGI52453[] = {{&S81532[134], 3}, {NULL}};
const Segment SGI52454[] = {{&S81532[100], 5}, {NULL}};
const Segment SGI52455[] = {{&S81532[0], 14}, {NULL}};
const Segment SGI52456[] = {{&S81537[45], 21}, {NULL}};
const Segment SGI52457[] = {{&S81537[30], 6}, {NULL}};
const Segment SGI52458[] = {{&S81537[125], 3}, {NULL}};
const Segment SGI52459[] = {{&S81537[91], 5}, {NULL}};
const Segment SGI52460[] = {{&S81537[149], 13}, {NULL}};
const Segment SGI52461[] = {{&S81556[44], 5}, {NULL}};
const Segment SGI52462[] = {{&S81556[44], 22}, {NULL}};
const Segment SGI52463[] = {{&S81556[30], 6}, {NULL}};
const Segment SGI52464[] = {{&S81556[80], 6}, {NULL}};
const Segment SGI52465[] = {{&S81556[107], 13}, {NULL}};
const Segment SGI52466[] = {{&S84540[44], 10}, {NULL}};
const Segment SGI52467[] = {{&S84540[44], 27}, {NULL}};
const Segment SGI52468[] = {{&S84540[30], 6}, {NULL}};
const Segment SGI52469[] = {{&S84540[85], 6}, {NULL}};
const Segment SGI52470[] = {{&S84540[122], 3}, {NULL}};
const Segment SGI52471[] = {{&S81603[45], 20}, {NULL}};
const Segment SGI52472[] = {{&S81603[30], 6}, {NULL}};
const Segment SGI52473[] = {{&S81603[124], 3}, {NULL}};
const Segment SGI52474[] = {{&S81603[90], 5}, {NULL}};
const Segment SGI52475[] = {{&S81603[148], 13}, {NULL}};
const Segment SGI52476[] = {{&S81623[45], 10}, {NULL}};
const Segment SGI52477[] = {{&S81623[30], 6}, {NULL}};
const Segment SGI52478[] = {{&S81623[120], 3}, {NULL}};
const Segment SGI52479[] = {{&S81623[86], 5}, {NULL}};
const Segment SGI52480[] = {{&S81623[144], 13}, {NULL}};
const Segment SGI52481[] = {{&S84620[44], 10}, {NULL}};
const Segment SGI52482[] = {{&S84620[44], 27}, {NULL}};
const Segment SGI52483[] = {{&S84620[30], 6}, {NULL}};
const Segment SGI52484[] = {{&S84620[119], 3}, {NULL}};
const Segment SGI52485[] = {{&S84620[85], 5}, {NULL}};
const Segment SGI52486[] = {{&S84620[153], 3}, {NULL}};
const Segment SGI52487[] = {{&S84623[44], 11}, {NULL}};
const Segment SGI52488[] = {{&S84623[44], 28}, {NULL}};
const Segment SGI52489[] = {{&S84623[30], 6}, {NULL}};
const Segment SGI52490[] = {{&S84623[120], 3}, {NULL}};
const Segment SGI52491[] = {{&S84623[86], 5}, {NULL}};
const Segment SGI52492[] = {{&S84623[154], 3}, {NULL}};
const Segment SGI52493[] = {{&S84624[44], 11}, {NULL}};
const Segment SGI52494[] = {{&S84624[44], 28}, {NULL}};
const Segment SGI52495[] = {{&S84624[30], 6}, {NULL}};
const Segment SGI52496[] = {{&S84624[120], 3}, {NULL}};
const Segment SGI52497[] = {{&S84624[86], 5}, {NULL}};
const Segment SGI52498[] = {{&S84624[154], 3}, {NULL}};
const Segment SGI52499[] = {{&S81695[45], 10}, {NULL}};
const Segment SGI52500[] = {{&S81695[30], 6}, {NULL}};
const Segment SGI52501[] = {{&S81695[120], 3}, {NULL}};
const Segment SGI52502[] = {{&S81695[86], 5}, {NULL}};
const Segment SGI52503[] = {{&S81695[144], 13}, {NULL}};
const Segment SGI52504[] = {{&S84671[31], 5}, {NULL}};
const Segment SGI52505[] = {{&S84671[51], 3}, {NULL}};
const Segment SGI52506[] = {{&S84691[44], 7}, {NULL}};
const Segment SGI52507[] = {{&S84691[44], 15}, {NULL}};
const Segment SGI52508[] = {{&S84691[30], 6}, {NULL}};
const Segment SGI52509[] = {{&S84691[107], 3}, {NULL}};
const Segment SGI52510[] = {{&S84691[73], 5}, {NULL}};
const Segment SGI52511[] = {{&S84692[38], 7}, {NULL}};
const Segment SGI52512[] = {{&S84692[31], 6}, {NULL}};
const Segment SGI52513[] = {{&S89968[42], 11}, {NULL}};
const Segment SGI52514[] = {{&S89968[42], 22}, {NULL}};
const Segment SGI52515[] = {{&S89968[30], 4}, {NULL}};
const Segment SGI52516[] = {{&S89968[112], 3}, {NULL}};
const Segment SGI52517[] = {{&S89968[78], 5}, {NULL}};
const Segment SGI52518[] = {{&S81750[44], 9}, {NULL}};
const Segment SGI52519[] = {{&S81750[44], 20}, {NULL}};
const Segment SGI52520[] = {{&S81750[30], 6}, {NULL}};
const Segment SGI52521[] = {{&S81750[112], 3}, {NULL}};
const Segment SGI52522[] = {{&S81750[78], 5}, {NULL}};
const Segment SGI52523[] = {{&S96985[31], 10}, {&S96985[42], 7}, {NULL}};
const Segment SGI52524[] = {{&S96985[56], 3}, {NULL}};
const Segment SGI52525[] = {{&S82293[44], 14}, {NULL}};
const Segment SGI52526[] = {{&S82293[44], 25}, {NULL}};
const Segment SGI52527[] = {{&S82293[30], 6}, {NULL}};
const Segment SGI52528[] = {{&S82293[117], 3}, {NULL}};
const Segment SGI52529[] = {{&S82293[83], 5}, {NULL}};
const Segment SGI52530[] = {{&S82319[45], 10}, {NULL}};
const Segment SGI52531[] = {{&S82319[30], 6}, {NULL}};
const Segment SGI52532[] = {{&S82319[120], 3}, {NULL}};
const Segment SGI52533[] = {{&S82319[86], 5}, {NULL}};
const Segment SGI52534[] = {{&S82319[144], 13}, {NULL}};
const Segment SGI52535[] = {{&S84885[45], 10}, {NULL}};
const Segment SGI52536[] = {{&S84885[30], 6}, {NULL}};
const Segment SGI52537[] = {{&S84885[120], 3}, {NULL}};
const Segment SGI52538[] = {{&S84885[86], 5}, {NULL}};
const Segment SGI52539[] = {{&S84885[154], 3}, {NULL}};
const Segment SGI52540[] = {{&S82379[44], 13}, {NULL}};
const Segment SGI52541[] = {{&S82379[44], 24}, {NULL}};
const Segment SGI52542[] = {{&S82379[30], 6}, {NULL}};
const Segment SGI52543[] = {{&S82379[116], 3}, {NULL}};
const Segment SGI52544[] = {{&S82379[82], 5}, {NULL}};
const Segment SGI52545[] = {{&S82380[38], 13}, {NULL}};
const Segment SGI52546[] = {{&S82380[31], 6}, {NULL}};
const Segment SGI52547[] = {{&S82026[44], 6}, {NULL}};
const Segment SGI52548[] = {{&S82026[44], 17}, {NULL}};
const Segment SGI52549[] = {{&S82026[30], 6}, {NULL}};
const Segment SGI52550[] = {{&S82026[109], 3}, {NULL}};
const Segment SGI52551[] = {{&S82026[75], 5}, {NULL}};
const Segment SGI52552[] = {{&S82477[45], 10}, {NULL}};
const Segment SGI52553[] = {{&S82477[30], 6}, {NULL}};
const Segment SGI52554[] = {{&S82477[120], 3}, {NULL}};
const Segment SGI52555[] = {{&S82477[86], 5}, {NULL}};
const Segment SGI52556[] = {{&S82477[144], 13}, {NULL}};
const Segment SGI52557[] = {{&S97364[45], 10}, {NULL}};
const Segment SGI52558[] = {{&S97364[30], 6}, {NULL}};
const Segment SGI52559[] = {{&S97364[114], 3}, {NULL}};
const Segment SGI52560[] = {{&S97364[80], 5}, {NULL}};
const Segment SGI52561[] = {{&S97364[138], 13}, {NULL}};
const Segment SGI52562[] = {{&S82110[44], 9}, {NULL}};
const Segment SGI52563[] = {{&S82110[44], 20}, {NULL}};
const Segment SGI52564[] = {{&S82110[30], 6}, {NULL}};
const Segment SGI52565[] = {{&S82110[119], 3}, {NULL}};
const Segment SGI52566[] = {{&S82110[85], 5}, {NULL}};
const Segment SGI52567[] = {{&S82110[64], 8}, {NULL}};
const Segment SGI52568[] = {{&S82112[44], 9}, {NULL}};
const Segment SGI52569[] = {{&S82112[44], 20}, {NULL}};
const Segment SGI52570[] = {{&S82112[30], 6}, {NULL}};
const Segment SGI52571[] = {{&S82112[112], 3}, {NULL}};
const Segment SGI52572[] = {{&S82112[78], 5}, {NULL}};
const Segment SGI52573[] = {{&S82112[136], 7}, {NULL}};
const Segment SGI52574[] = {{&S82561[45], 7}, {NULL}};
const Segment SGI52575[] = {{&S82561[30], 6}, {NULL}};
const Segment SGI52576[] = {{&S82561[58], 3}, {NULL}};
const Segment SGI52577[] = {{&S82561[84], 5}, {NULL}};
const Segment SGI52578[] = {{&S82561[142], 13}, {NULL}};
const Segment SGI52579[] = {{&S85012[44], 18}, {NULL}};
const Segment SGI52580[] = {{&S85012[44], 29}, {NULL}};
const Segment SGI52581[] = {{&S85012[30], 6}, {NULL}};
const Segment SGI52582[] = {{&S85012[87], 6}, {NULL}};
const Segment SGI52583[] = {{&S85012[124], 3}, {NULL}};
const Segment SGI52584[] = {{&S85042[44], 11}, {NULL}};
const Segment SGI52585[] = {{&S85042[44], 21}, {NULL}};
const Segment SGI52586[] = {{&S85042[30], 6}, {NULL}};
const Segment SGI52587[] = {{&S85042[113], 3}, {NULL}};
const Segment SGI52588[] = {{&S85042[79], 5}, {NULL}};
const Segment SGI52589[] = {{&S82681[44], 35}, {NULL}};
const Segment SGI52590[] = {{&S82681[30], 6}, {NULL}};
const Segment SGI52591[] = {{&S82682[44], 13}, {NULL}};
const Segment SGI52592[] = {{&S82682[44], 37}, {NULL}};
const Segment SGI52593[] = {{&S82682[30], 6}, {NULL}};
const Segment SGI52594[] = {{&S82682[129], 3}, {NULL}};
const Segment SGI52595[] = {{&S82682[95], 5}, {NULL}};
const Segment SGI52596[] = {{&S90400[42], 11}, {NULL}};
const Segment SGI52597[] = {{&S90400[42], 22}, {NULL}};
const Segment SGI52598[] = {{&S90400[30], 4}, {NULL}};
const Segment SGI52599[] = {{&S82683[44], 13}, {NULL}};
const Segment SGI52600[] = {{&S82683[44], 37}, {NULL}};
const Segment SGI52601[] = {{&S90400[112], 3}, {NULL}};
const Segment SGI52602[] = {{&S82683[30], 6}, {NULL}};
const Segment SGI52603[] = {{&S90400[78], 5}, {NULL}};
const Segment SGI52604[] = {{&S82683[129], 3}, {NULL}};
const Segment SGI52605[] = {{&S82683[95], 5}, {NULL}};
const Segment SGI52606[] = {{&S82684[44], 13}, {NULL}};
const Segment SGI52607[] = {{&S82684[44], 37}, {NULL}};
const Segment SGI52608[] = {{&S82684[30], 6}, {NULL}};
const Segment SGI52609[] = {{&S82684[129], 3}, {NULL}};
const Segment SGI52610[] = {{&S82684[95], 5}, {NULL}};
const Segment SGI52611[] = {{&S82699[45], 7}, {NULL}};
const Segment SGI52612[] = {{&S82220[44], 12}, {NULL}};
const Segment SGI52613[] = {{&S82220[44], 23}, {NULL}};
const Segment SGI52614[] = {{&S82699[30], 6}, {NULL}};
const Segment SGI52615[] = {{&S82220[30], 6}, {NULL}};
const Segment SGI52616[] = {{&S82699[58], 3}, {NULL}};
const Segment SGI52617[] = {{&S82220[115], 3}, {NULL}};
const Segment SGI52618[] = {{&S82220[81], 5}, {NULL}};
const Segment SGI52619[] = {{&S82699[84], 5}, {NULL}};
const Segment SGI52620[] = {{&S82699[142], 13}, {NULL}};
const Segment SGI52621[] = {{&S85157[45], 22}, {NULL}};
const Segment SGI52622[] = {{&S85157[30], 6}, {NULL}};
const Segment SGI52623[] = {{&S85157[126], 3}, {NULL}};
const Segment SGI52624[] = {{&S85157[92], 5}, {NULL}};
const Segment SGI52625[] = {{&S85157[160], 3}, {NULL}};
const Segment SGI52626[] = {{&S82233[44], 14}, {NULL}};
const Segment SGI52627[] = {{&S82233[44], 35}, {NULL}};
const Segment SGI52628[] = {{&S82233[30], 6}, {NULL}};
const Segment SGI52629[] = {{&S82233[93], 6}, {NULL}};
const Segment SGI52630[] = {{&S82233[120], 13}, {NULL}};
const Segment SGI52631[] = {{&S82273[44], 7}, {NULL}};
const Segment SGI52632[] = {{&S82273[44], 17}, {NULL}};
const Segment SGI52633[] = {{&S82273[30], 6}, {NULL}};
const Segment SGI52634[] = {{&S82273[109], 3}, {NULL}};
const Segment SGI52635[] = {{&S82273[75], 5}, {NULL}};
const Segment SGI52636[] = {{&S82320[45], 10}, {NULL}};
const Segment SGI52637[] = {{&S82320[30], 6}, {NULL}};
const Segment SGI52638[] = {{&S82320[120], 3}, {NULL}};
const Segment SGI52639[] = {{&S82320[86], 5}, {NULL}};
const Segment SGI52640[] = {{&S82320[144], 13}, {NULL}};
const Segment SGI52641[] = {{&S97695[44], 4}, {NULL}};
const Segment SGI52642[] = {{&S97695[44], 16}, {NULL}};
const Segment SGI52643[] = {{&S97695[30], 6}, {NULL}};
const Segment SGI52644[] = {{&S97695[108], 3}, {NULL}};
const Segment SGI52645[] = {{&S97695[74], 5}, {NULL}};
const Segment SGI52646[] = {{&S85228[44], 7}, {NULL}};
const Segment SGI52647[] = {{&S85228[44], 24}, {NULL}};
const Segment SGI52648[] = {{&S85228[30], 6}, {NULL}};
const Segment SGI52649[] = {{&S85228[117], 3}, {NULL}};
const Segment SGI52650[] = {{&S85228[82], 6}, {NULL}};
const Segment SGI52651[] = {{&S97699[44], 13}, {NULL}};
const Segment SGI52652[] = {{&S97699[44], 36}, {NULL}};
const Segment SGI52653[] = {{&S97699[30], 6}, {NULL}};
const Segment SGI52654[] = {{&S97699[97], 4}, {NULL}};
const Segment SGI52655[] = {{&S97699[97], 3}, {NULL}};
const Segment SGI52656[] = {{&S97700[44], 13}, {NULL}};
const Segment SGI52657[] = {{&S97700[44], 36}, {NULL}};
const Segment SGI52658[] = {{&S97700[30], 6}, {NULL}};
const Segment SGI52659[] = {{&S97700[97], 4}, {NULL}};
const Segment SGI52660[] = {{&S97700[97], 3}, {NULL}};
const Segment SGI52661[] = {{&S82341[44], 14}, {NULL}};
const Segment SGI52662[] = {{&S82341[44], 35}, {NULL}};
const Segment SGI52663[] = {{&S82341[30], 6}, {NULL}};
const Segment SGI52664[] = {{&S82341[127], 3}, {NULL}};
const Segment SGI52665[] = {{&S82341[93], 5}, {NULL}};
const Segment SGI52666[] = {{&S82857[44], 7}, {NULL}};
const Segment SGI52667[] = {{&S82857[44], 25}, {NULL}};
const Segment SGI52668[] = {{&S82857[30], 6}, {NULL}};
const Segment SGI52669[] = {{&S82857[83], 6}, {NULL}};
const Segment SGI52670[] = {{&S82857[110], 13}, {NULL}};
const Segment SGI52671[] = {{&S97730[45], 10}, {NULL}};
const Segment SGI52672[] = {{&S97730[30], 6}, {NULL}};
const Segment SGI52673[] = {{&S97730[114], 3}, {NULL}};
const Segment SGI52674[] = {{&S97730[80], 5}, {NULL}};
const Segment SGI52675[] = {{&S97730[138], 13}, {NULL}};
const Segment SGI52676[] = {{&S82889[44], 13}, {NULL}};
const Segment SGI52677[] = {{&S82889[44], 30}, {NULL}};
const Segment SGI52678[] = {{&S82889[30], 6}, {NULL}};
const Segment SGI52679[] = {{&S82889[122], 3}, {NULL}};
const Segment SGI52680[] = {{&S82889[88], 5}, {NULL}};
const Segment SGI52681[] = {{&S82889[146], 14}, {NULL}};
const Segment SGI52682[] = {{&S82889[153], 7}, {NULL}};
const Segment SGI52683[] = {{&S82890[44], 13}, {NULL}};
const Segment SGI52684[] = {{&S82890[44], 30}, {NULL}};
const Segment SGI52685[] = {{&S82890[30], 6}, {NULL}};
const Segment SGI52686[] = {{&S82890[129], 3}, {NULL}};
const Segment SGI52687[] = {{&S82890[95], 5}, {NULL}};
const Segment SGI52688[] = {{&S82890[74], 8}, {NULL}};
const Segment SGI52689[] = {{&S82891[44], 13}, {NULL}};
const Segment SGI52690[] = {{&S82891[44], 30}, {NULL}};
const Segment SGI52691[] = {{&S82891[30], 6}, {NULL}};
const Segment SGI52692[] = {{&S82891[122], 3}, {NULL}};
const Segment SGI52693[] = {{&S82891[88], 5}, {NULL}};
const Segment SGI52694[] = {{&S82891[146], 14}, {&S82891[146], 14}, {NULL}};
const Segment SGI52695[] = {{&S82891[153], 7}, {NULL}};
const Segment SGI52696[] = {{&S82898[44], 9}, {NULL}};
const Segment SGI52697[] = {{&S82898[44], 20}, {NULL}};
const Segment SGI52698[] = {{&S82898[30], 6}, {NULL}};
const Segment SGI52699[] = {{&S82898[112], 3}, {NULL}};
const Segment SGI52700[] = {{&S82898[78], 5}, {NULL}};
const Segment SGI52701[] = {{&S82424[34], 9}, {NULL}};
const Segment SGI52702[] = {{&S82424[27], 6}, {NULL}};
const Segment SGI52703[] = {{&S82424[102], 2}, {NULL}};
const Segment SGI52704[] = {{&S82424[68], 6}, {NULL}};
const Segment SGI52705[] = {{&S82424[136], 8}, {NULL}};
const Segment SGI52706[] = {{&S91844[44], 10}, {NULL}};
const Segment SGI52707[] = {{&S91844[44], 27}, {NULL}};
const Segment SGI52708[] = {{&S91844[30], 6}, {NULL}};
const Segment SGI52709[] = {{&S91844[85], 6}, {NULL}};
const Segment SGI52710[] = {{&S82441[44], 14}, {NULL}};
const Segment SGI52711[] = {{&S82441[44], 35}, {NULL}};
const Segment SGI52712[] = {{&S82441[30], 6}, {NULL}};
const Segment SGI52713[] = {{&S82441[93], 6}, {NULL}};
const Segment SGI52714[] = {{&S82441[120], 13}, {NULL}};
const Segment SGI52715[] = {{&S85313[44], 18}, {NULL}};
const Segment SGI52716[] = {{&S85313[44], 29}, {NULL}};
const Segment SGI52717[] = {{&S85313[30], 6}, {NULL}};
const Segment SGI52718[] = {{&S85313[87], 6}, {NULL}};
const Segment SGI52719[] = {{&S85313[124], 3}, {NULL}};
const Segment SGI52720[] = {{&S85363[59], 37}, {NULL}};
const Segment SGI52721[] = {{&S85363[45], 6}, {NULL}};
const Segment SGI52722[] = {{&S85363[128], 5}, {NULL}};
const Segment SGI52723[] = {{&S85363[0], 14}, {NULL}};
const Segment SGI52724[] = {{&S85383[44], 9}, {NULL}};
const Segment SGI52725[] = {{&S85383[44], 20}, {NULL}};
const Segment SGI52726[] = {{&S85383[30], 6}, {NULL}};
const Segment SGI52727[] = {{&S85383[78], 6}, {NULL}};
const Segment SGI52728[] = {{&S85383[115], 3}, {NULL}};
const Segment SGI52729[] = {{&S97894[44], 11}, {NULL}};
const Segment SGI52730[] = {{&S97894[44], 22}, {NULL}};
const Segment SGI52731[] = {{&S97894[30], 6}, {NULL}};
const Segment SGI52732[] = {{&S97894[80], 6}, {NULL}};
const Segment SGI52733[] = {{&S97894[107], 13}, {NULL}};
const Segment SGI52734[] = {{&S82476[45], 10}, {NULL}};
const Segment SGI52735[] = {{&S82476[30], 6}, {NULL}};
const Segment SGI52736[] = {{&S82476[120], 3}, {NULL}};
const Segment SGI52737[] = {{&S82476[86], 5}, {NULL}};
const Segment SGI52738[] = {{&S82476[144], 13}, {NULL}};
const Segment SGI52739[] = {{&S82480[44], 22}, {NULL}};
const Segment SGI52740[] = {{&S82480[44], 33}, {NULL}};
const Segment SGI52741[] = {{&S82480[30], 6}, {NULL}};
const Segment SGI52742[] = {{&S82480[125], 3}, {NULL}};
const Segment SGI52743[] = {{&S82480[91], 5}, {NULL}};
const Segment SGI52744[] = {{&S82500[44], 13}, {NULL}};
const Segment SGI52745[] = {{&S82500[44], 30}, {NULL}};
const Segment SGI52746[] = {{&S82500[30], 6}, {NULL}};
const Segment SGI52747[] = {{&S82500[88], 6}, {NULL}};
const Segment SGI52748[] = {{&S82500[115], 13}, {NULL}};
const Segment SGI52749[] = {{&S82548[44], 9}, {NULL}};
const Segment SGI52750[] = {{&S82548[44], 20}, {NULL}};
const Segment SGI52751[] = {{&S82548[30], 6}, {NULL}};
const Segment SGI52752[] = {{&S82548[112], 3}, {NULL}};
const Segment SGI52753[] = {{&S82548[78], 5}, {NULL}};
const Segment SGI52754[] = {{&S82549[44], 9}, {NULL}};
const Segment SGI52755[] = {{&S82549[44], 20}, {NULL}};
const Segment SGI52756[] = {{&S82549[30], 6}, {NULL}};
const Segment SGI52757[] = {{&S82549[81], 6}, {NULL}};
const Segment SGI52758[] = {{&S82585[44], 14}, {NULL}};
const Segment SGI52759[] = {{&S82585[44], 35}, {NULL}};
const Segment SGI52760[] = {{&S82585[30], 6}, {NULL}};
const Segment SGI52761[] = {{&S82585[127], 3}, {NULL}};
const Segment SGI52762[] = {{&S82585[93], 5}, {NULL}};
const Segment SGI52763[] = {{&S82650[44], 22}, {NULL}};
const Segment SGI52764[] = {{&S82650[44], 33}, {NULL}};
const Segment SGI52765[] = {{&S82650[30], 6}, {NULL}};
const Segment SGI52766[] = {{&S82650[91], 6}, {NULL}};
const Segment SGI52767[] = {{&S82650[118], 13}, {NULL}};
const Segment SGI52768[] = {{&S85705[45], 10}, {NULL}};
const Segment SGI52769[] = {{&S85705[30], 6}, {NULL}};
const Segment SGI52770[] = {{&S85705[120], 3}, {NULL}};
const Segment SGI52771[] = {{&S85705[86], 5}, {NULL}};
const Segment SGI52772[] = {{&S85705[154], 3}, {NULL}};
const Segment SGI52773[] = {{&S98142[45], 9}, {NULL}};
const Segment SGI52774[] = {{&S98142[30], 6}, {NULL}};
const Segment SGI52775[] = {{&S98142[114], 3}, {NULL}};
const Segment SGI52776[] = {{&S98142[80], 5}, {NULL}};
const Segment SGI52777[] = {{&S98142[138], 13}, {NULL}};
const Segment SGI52778[] = {{&S85804[44], 9}, {NULL}};
const Segment SGI52779[] = {{&S85804[44], 17}, {NULL}};
const Segment SGI52780[] = {{&S85804[30], 6}, {NULL}};
const Segment SGI52781[] = {{&S85804[109], 3}, {NULL}};
const Segment SGI52782[] = {{&S85804[75], 5}, {NULL}};
const Segment SGI52783[] = {{&S85817[30], 6}, {NULL}};
const Segment SGI52784[] = {{&S82792[44], 14}, {NULL}};
const Segment SGI52785[] = {{&S82792[44], 31}, {NULL}};
const Segment SGI52786[] = {{&S82792[30], 6}, {NULL}};
const Segment SGI52787[] = {{&S82792[89], 6}, {NULL}};
const Segment SGI52788[] = {{&S82792[116], 13}, {NULL}};
const Segment SGI52789[] = {{&S82800[44], 14}, {NULL}};
const Segment SGI52790[] = {{&S82800[44], 35}, {NULL}};
const Segment SGI52791[] = {{&S82800[30], 6}, {NULL}};
const Segment SGI52792[] = {{&S82800[128], 3}, {NULL}};
const Segment SGI52793[] = {{&S82800[93], 6}, {NULL}};
const Segment SGI52794[] = {{&S82800[148], 8}, {NULL}};
const Segment SGI52795[] = {{&S82800[157], 13}, {NULL}};
const Segment SGI52796[] = {{&S82808[44], 10}, {NULL}};
const Segment SGI52797[] = {{&S82808[44], 27}, {NULL}};
const Segment SGI52798[] = {{&S82808[30], 6}, {NULL}};
const Segment SGI52799[] = {{&S82808[118], 3}, {NULL}};
const Segment SGI52800[] = {{&S82808[85], 5}, {NULL}};
const Segment SGI52801[] = {{&S82808[122], 14}, {NULL}};
const Segment SGI52802[] = {{&S82809[44], 11}, {NULL}};
const Segment SGI52803[] = {{&S82809[44], 28}, {NULL}};
const Segment SGI52804[] = {{&S82809[30], 6}, {NULL}};
const Segment SGI52805[] = {{&S82809[119], 3}, {NULL}};
const Segment SGI52806[] = {{&S82809[86], 5}, {NULL}};
const Segment SGI52807[] = {{&S82809[123], 14}, {NULL}};
const Segment SGI52808[] = {{&S98292[44], 12}, {NULL}};
const Segment SGI52809[] = {{&S98292[44], 23}, {NULL}};
const Segment SGI52810[] = {{&S98292[30], 6}, {NULL}};
const Segment SGI52811[] = {{&S98292[81], 6}, {NULL}};
const Segment SGI52812[] = {{&S98292[108], 13}, {NULL}};
const Segment SGI52813[] = {{&S82899[44], 9}, {NULL}};
const Segment SGI52814[] = {{&S82899[44], 20}, {NULL}};
const Segment SGI52815[] = {{&S82899[30], 6}, {NULL}};
const Segment SGI52816[] = {{&S82899[112], 3}, {NULL}};
const Segment SGI52817[] = {{&S82899[78], 5}, {NULL}};
const Segment SGI52818[] = {{&S85949[45], 6}, {NULL}};
const Segment SGI52819[] = {{&S85949[30], 6}, {NULL}};
const Segment SGI52820[] = {{&S85949[117], 3}, {NULL}};
const Segment SGI52821[] = {{&S85949[83], 5}, {NULL}};
const Segment SGI52822[] = {{&S85949[151], 3}, {NULL}};
const Segment SGI52823[] = {{&S82931[44], 21}, {NULL}};
const Segment SGI52824[] = {{&S82931[44], 32}, {NULL}};
const Segment SGI52825[] = {{&S82931[30], 6}, {NULL}};
const Segment SGI52826[] = {{&S82931[124], 3}, {NULL}};
const Segment SGI52827[] = {{&S82931[90], 5}, {NULL}};
const Segment SGI52828[] = {{&S82932[42], 21}, {NULL}};
const Segment SGI52829[] = {{&S82932[30], 6}, {NULL}};
const Segment SGI52830[] = {{&S82932[122], 3}, {NULL}};
const Segment SGI52831[] = {{&S82932[88], 5}, {NULL}};
const Segment SGI52832[] = {{&S91311[42], 12}, {NULL}};
const Segment SGI52833[] = {{&S91311[42], 23}, {NULL}};
const Segment SGI52834[] = {{&S91311[30], 4}, {NULL}};
const Segment SGI52835[] = {{&S91311[113], 3}, {NULL}};
const Segment SGI52836[] = {{&S91311[79], 5}, {NULL}};
const Segment SGI52837[] = {{&S83505[44], 14}, {NULL}};
const Segment SGI52838[] = {{&S83505[44], 37}, {NULL}};
const Segment SGI52839[] = {{&S83505[30], 6}, {NULL}};
const Segment SGI52840[] = {{&S83505[136], 3}, {NULL}};
const Segment SGI52841[] = {{&S83505[102], 5}, {NULL}};
const Segment SGI52842[] = {{&S83505[81], 8}, {NULL}};
const Segment SGI52843[] = {{&S83506[44], 15}, {NULL}};
const Segment SGI52844[] = {{&S83506[44], 38}, {NULL}};
const Segment SGI52845[] = {{&S83506[30], 6}, {NULL}};
const Segment SGI52846[] = {{&S83506[137], 3}, {NULL}};
const Segment SGI52847[] = {{&S83506[103], 5}, {NULL}};
const Segment SGI52848[] = {{&S83506[82], 8}, {NULL}};
const Segment SGI52849[] = {{&S83512[44], 14}, {NULL}};
const Segment SGI52850[] = {{&S83512[44], 31}, {NULL}};
const Segment SGI52851[] = {{&S83512[30], 6}, {NULL}};
const Segment SGI52852[] = {{&S83512[123], 3}, {NULL}};
const Segment SGI52853[] = {{&S83512[89], 5}, {NULL}};
const Segment SGI52854[] = {{&S83557[44], 7}, {NULL}};
const Segment SGI52855[] = {{&S83557[44], 25}, {NULL}};
const Segment SGI52856[] = {{&S83557[30], 6}, {NULL}};
const Segment SGI52857[] = {{&S83557[83], 6}, {NULL}};
const Segment SGI52858[] = {{&S83557[110], 13}, {NULL}};
const Segment SGI52859[] = {{&S83590[44], 5}, {NULL}};
const Segment SGI52860[] = {{&S83590[44], 16}, {NULL}};
const Segment SGI52861[] = {{&S83590[30], 6}, {NULL}};
const Segment SGI52862[] = {{&S83590[108], 3}, {NULL}};
const Segment SGI52863[] = {{&S83590[74], 5}, {NULL}};
const Segment SGI52864[] = {{&S83591[44], 11}, {NULL}};
const Segment SGI52865[] = {{&S83591[44], 22}, {NULL}};
const Segment SGI52866[] = {{&S83591[30], 6}, {NULL}};
const Segment SGI52867[] = {{&S83591[114], 3}, {NULL}};
const Segment SGI52868[] = {{&S83591[80], 5}, {NULL}};
const Segment SGI52869[] = {{&S83592[44], 7}, {NULL}};
const Segment SGI52870[] = {{&S83592[44], 18}, {NULL}};
const Segment SGI52871[] = {{&S83592[30], 6}, {NULL}};
const Segment SGI52872[] = {{&S83592[110], 3}, {NULL}};
const Segment SGI52873[] = {{&S83592[76], 5}, {NULL}};
const Segment SGI52874[] = {{&S83595[45], 5}, {NULL}};
const Segment SGI52875[] = {{&S83595[30], 6}, {NULL}};
const Segment SGI52876[] = {{&S83595[109], 3}, {NULL}};
const Segment SGI52877[] = {{&S83595[75], 5}, {NULL}};
const Segment SGI52878[] = {{&S83595[133], 13}, {NULL}};
const Segment SGI52879[] = {{&S82999[44], 7}, {NULL}};
const Segment SGI52880[] = {{&S82999[44], 32}, {NULL}};
const Segment SGI52881[] = {{&S82999[30], 6}, {NULL}};
const Segment SGI52882[] = {{&S82999[124], 3}, {NULL}};
const Segment SGI52883[] = {{&S82999[90], 5}, {NULL}};
const Segment SGI52884[] = {{&S82999[148], 11}, {NULL}};
const Segment SGI52885[] = {{&S83607[45], 7}, {NULL}};
const Segment SGI52886[] = {{&S83607[30], 6}, {NULL}};
const Segment SGI52887[] = {{&S83607[58], 3}, {NULL}};
const Segment SGI52888[] = {{&S83607[84], 5}, {NULL}};
const Segment SGI52889[] = {{&S83607[142], 13}, {NULL}};
const Segment SGI52890[] = {{&S98618[45], 9}, {NULL}};
const Segment SGI52891[] = {{&S98618[30], 6}, {NULL}};
const Segment SGI52892[] = {{&S98618[113], 3}, {NULL}};
const Segment SGI52893[] = {{&S98618[79], 5}, {NULL}};
const Segment SGI52894[] = {{&S98618[137], 13}, {NULL}};
const Segment SGI52895[] = {{&S98662[44], 12}, {NULL}};
const Segment SGI52896[] = {{&S98662[44], 24}, {NULL}};
const Segment SGI52897[] = {{&S98662[30], 6}, {NULL}};
const Segment SGI52898[] = {{&S98662[115], 3}, {NULL}};
const Segment SGI52899[] = {{&S98662[82], 5}, {NULL}};
const Segment SGI52900[] = {{&S98662[119], 14}, {NULL}};
const Segment SGI52901[] = {{&S83757[45], 10}, {NULL}};
const Segment SGI52902[] = {{&S83757[30], 6}, {NULL}};
const Segment SGI52903[] = {{&S83757[120], 3}, {NULL}};
const Segment SGI52904[] = {{&S83757[86], 5}, {NULL}};
const Segment SGI52905[] = {{&S83757[144], 13}, {NULL}};
const Segment SGI52906[] = {{&S83765[45], 7}, {NULL}};
const Segment SGI52907[] = {{&S83765[30], 6}, {NULL}};
const Segment SGI52908[] = {{&S83765[58], 3}, {NULL}};
const Segment SGI52909[] = {{&S83765[84], 5}, {NULL}};
const Segment SGI52910[] = {{&S83765[142], 13}, {NULL}};
const Segment SGI52911[] = {{&S83793[34], 13}, {NULL}};
const Segment SGI52912[] = {{&S83793[27], 6}, {NULL}};
const Segment SGI52913[] = {{&S83793[106], 2}, {NULL}};
const Segment SGI52914[] = {{&S83793[72], 6}, {NULL}};
const Segment SGI52915[] = {{&S83793[140], 8}, {NULL}};
const Segment SGI52916[] = {{&S83212[44], 10}, {NULL}};
const Segment SGI52917[] = {{&S83212[44], 21}, {NULL}};
const Segment SGI52918[] = {{&S83212[30], 6}, {NULL}};
const Segment SGI52919[] = {{&S83212[113], 3}, {NULL}};
const Segment SGI52920[] = {{&S83212[79], 5}, {NULL}};
const Segment SGI52921[] = {{&S83836[45], 10}, {NULL}};
const Segment SGI52922[] = {{&S83836[30], 6}, {NULL}};
const Segment SGI52923[] = {{&S83836[120], 3}, {NULL}};
const Segment SGI52924[] = {{&S83836[86], 5}, {NULL}};
const Segment SGI52925[] = {{&S83836[144], 13}, {NULL}};
const Segment SGI52926[] = {{&S86270[45], 9}, {NULL}};
const Segment SGI52927[] = {{&S86270[30], 6}, {NULL}};
const Segment SGI52928[] = {{&S86270[113], 3}, {NULL}};
const Segment SGI52929[] = {{&S86270[79], 5}, {NULL}};
const Segment SGI52930[] = {{&S86270[147], 3}, {NULL}};
const Segment SGI52931[] = {{&S98808[44], 12}, {NULL}};
const Segment SGI52932[] = {{&S98808[44], 24}, {NULL}};
const Segment SGI52933[] = {{&S98808[30], 6}, {NULL}};
const Segment SGI52934[] = {{&S98808[82], 6}, {NULL}};
const Segment SGI52935[] = {{&S98808[109], 13}, {NULL}};
const Segment SGI52936[] = {{&S83323[44], 8}, {NULL}};
const Segment SGI52937[] = {{&S83323[44], 29}, {NULL}};
const Segment SGI52938[] = {{&S83323[30], 6}, {NULL}};
const Segment SGI52939[] = {{&S83323[121], 3}, {NULL}};
const Segment SGI52940[] = {{&S83323[87], 5}, {NULL}};
const Segment SGI52941[] = {{&S83330[45], 10}, {NULL}};
const Segment SGI52942[] = {{&S83330[30], 6}, {NULL}};
const Segment SGI52943[] = {{&S83330[115], 3}, {NULL}};
const Segment SGI52944[] = {{&S83330[81], 5}, {NULL}};
const Segment SGI52945[] = {{&S83330[139], 13}, {NULL}};
const Segment SGI52946[] = {{&S83941[45], 7}, {NULL}};
const Segment SGI52947[] = {{&S83941[30], 6}, {NULL}};
const Segment SGI52948[] = {{&S83941[58], 3}, {NULL}};
const Segment SGI52949[] = {{&S83941[84], 5}, {NULL}};
const Segment SGI52950[] = {{&S83941[142], 13}, {NULL}};
const Segment SGI52951[] = {{&S83983[44], 7}, {NULL}};
const Segment SGI52952[] = {{&S83983[44], 25}, {NULL}};
const Segment SGI52953[] = {{&S83983[30], 6}, {NULL}};
const Segment SGI52954[] = {{&S83983[83], 6}, {NULL}};
const Segment SGI52955[] = {{&S83983[110], 13}, {NULL}};
const Segment SGI52956[] = {{&S83430[41], 6}, {NULL}};
const Segment SGI52957[] = {{&S83430[41], 18}, {NULL}};
const Segment SGI52958[] = {{&S83430[107], 3}, {NULL}};
const Segment SGI52959[] = {{&S83430[73], 5}, {NULL}};
const Segment SGI52960[] = {{&S83431[44], 7}, {NULL}};
const Segment SGI52961[] = {{&S83431[44], 19}, {NULL}};
const Segment SGI52962[] = {{&S83431[30], 6}, {NULL}};
const Segment SGI52963[] = {{&S83431[111], 3}, {NULL}};
const Segment SGI52964[] = {{&S83431[77], 5}, {NULL}};
const Segment SGI52965[] = {{&S83457[44], 74}, {NULL}};
const Segment SGI52966[] = {{&S83457[44], 95}, {NULL}};
const Segment SGI52967[] = {{&S83457[30], 6}, {NULL}};
const Segment SGI52968[] = {{&S83457[186], 3}, {NULL}};
const Segment SGI52969[] = {{&S83457[112], 5}, {NULL}};
const Segment SGI52970[] = {{&S84039[44], 10}, {NULL}};
const Segment SGI52971[] = {{&S84039[44], 27}, {NULL}};
const Segment SGI52972[] = {{&S84039[30], 6}, {NULL}};
const Segment SGI52973[] = {{&S84039[85], 6}, {NULL}};
const Segment SGI52974[] = {{&S84039[112], 13}, {NULL}};
const Segment SGI52975[] = {{&S83508[44], 11}, {NULL}};
const Segment SGI52976[] = {{&S83508[44], 22}, {NULL}};
const Segment SGI52977[] = {{&S83508[30], 6}, {NULL}};
const Segment SGI52978[] = {{&S83508[114], 3}, {NULL}};
const Segment SGI52979[] = {{&S83508[80], 5}, {NULL}};
const Segment SGI52980[] = {{&S83515[45], 14}, {NULL}};
const Segment SGI52981[] = {{&S83515[30], 6}, {NULL}};
const Segment SGI52982[] = {{&S83515[124], 3}, {NULL}};
const Segment SGI52983[] = {{&S83515[90], 5}, {NULL}};
const Segment SGI52984[] = {{&S83515[148], 13}, {NULL}};
const Segment SGI52985[] = {{&S99044[44], 22}, {NULL}};
const Segment SGI52986[] = {{&S99044[30], 6}, {NULL}};
const Segment SGI52987[] = {{&S99044[116], 3}, {NULL}};
const Segment SGI52988[] = {{&S99044[82], 5}, {NULL}};
const Segment SGI52989[] = {{&S99045[44], 22}, {NULL}};
const Segment SGI52990[] = {{&S99045[30], 6}, {NULL}};
const Segment SGI52991[] = {{&S99045[116], 3}, {NULL}};
const Segment SGI52992[] = {{&S99045[82], 5}, {NULL}};
const Segment SGI52993[] = {{&S86558[45], 10}, {NULL}};
const Segment SGI52994[] = {{&S86558[30], 6}, {NULL}};
const Segment SGI52995[] = {{&S86558[120], 3}, {NULL}};
const Segment SGI52996[] = {{&S86558[86], 5}, {NULL}};
const Segment SGI52997[] = {{&S86558[154], 3}, {NULL}};
const Segment SGI52998[] = {{&S83548[44], 10}, {NULL}};
const Segment SGI52999[] = {{&S83548[44], 27}, {NULL}};
const Segment SGI53000[] = {{&S83548[30], 6}, {NULL}};
const Segment SGI53001[] = {{&S83548[85], 6}, {NULL}};
const Segment SGI53002[] = {{&S83548[112], 13}, {NULL}};
const Segment SGI53003[] = {{&S84101[44], 11}, {NULL}};
const Segment SGI53004[] = {{&S84101[44], 22}, {NULL}};
const Segment SGI53005[] = {{&S84101[30], 6}, {NULL}};
const Segment SGI53006[] = {{&S84101[114], 3}, {NULL}};
const Segment SGI53007[] = {{&S84101[80], 5}, {NULL}};
const Segment SGI53008[] = {{&S99090[44], 17}, {NULL}};
const Segment SGI53009[] = {{&S99090[44], 29}, {NULL}};
const Segment SGI53010[] = {{&S99090[30], 6}, {NULL}};
const Segment SGI53011[] = {{&S99090[121], 3}, {NULL}};
const Segment SGI53012[] = {{&S99090[87], 5}, {NULL}};
const Segment SGI53013[] = {{&S83594[59], 20}, {NULL}};
const Segment SGI53014[] = {{&S83594[59], 31}, {NULL}};
const Segment SGI53015[] = {{&S83594[45], 6}, {NULL}};
const Segment SGI53016[] = {{&S83594[138], 3}, {NULL}};
const Segment SGI53017[] = {{&S83594[104], 5}, {NULL}};
const Segment SGI53018[] = {{&S83594[0], 14}, {NULL}};
const Segment SGI53019[] = {{&S88424[44], 17}, {NULL}};
const Segment SGI53020[] = {{&S88424[44], 28}, {NULL}};
const Segment SGI53021[] = {{&S88424[30], 6}, {NULL}};
const Segment SGI53022[] = {{&S88424[120], 3}, {NULL}};
const Segment SGI53023[] = {{&S88424[86], 5}, {NULL}};
const Segment SGI53024[] = {{&S83610[44], 12}, {NULL}};
const Segment SGI53025[] = {{&S83610[44], 33}, {NULL}};
const Segment SGI53026[] = {{&S83610[30], 6}, {NULL}};
const Segment SGI53027[] = {{&S83610[125], 3}, {NULL}};
const Segment SGI53028[] = {{&S83610[91], 5}, {NULL}};
const Segment SGI53029[] = {{&S83697[44], 8}, {NULL}};
const Segment SGI53030[] = {{&S83697[44], 29}, {NULL}};
const Segment SGI53031[] = {{&S83697[30], 6}, {NULL}};
const Segment SGI53032[] = {{&S83697[121], 3}, {NULL}};
const Segment SGI53033[] = {{&S83697[87], 5}, {NULL}};
const Segment SGI53034[] = {{&S99238[44], 12}, {NULL}};
const Segment SGI53035[] = {{&S99238[44], 24}, {NULL}};
const Segment SGI53036[] = {{&S99238[30], 6}, {NULL}};
const Segment SGI53037[] = {{&S99238[82], 6}, {NULL}};
const Segment SGI53038[] = {{&S99238[109], 13}, {NULL}};
const Segment SGI53039[] = {{&S84250[44], 7}, {NULL}};
const Segment SGI53040[] = {{&S84250[44], 25}, {NULL}};
const Segment SGI53041[] = {{&S84250[30], 6}, {NULL}};
const Segment SGI53042[] = {{&S84250[83], 6}, {NULL}};
const Segment SGI53043[] = {{&S84250[110], 13}, {NULL}};
const Segment SGI53044[] = {{&S84251[44], 7}, {NULL}};
const Segment SGI53045[] = {{&S84251[44], 25}, {NULL}};
const Segment SGI53046[] = {{&S84251[30], 6}, {NULL}};
const Segment SGI53047[] = {{&S84251[83], 6}, {NULL}};
const Segment SGI53048[] = {{&S84251[110], 13}, {NULL}};
const Segment SGI53049[] = {{&S86781[39], 8}, {NULL}};
const Segment SGI53050[] = {{&S86781[104], 3}, {NULL}};
const Segment SGI53051[] = {{&S86781[93], 3}, {NULL}};
const Segment SGI53052[] = {{&S86781[70], 6}, {NULL}};
const Segment SGI53053[] = {{&S86782[31], 8}, {NULL}};
const Segment SGI53054[] = {{&S86782[55], 3}, {NULL}};
const Segment SGI53055[] = {{&S83832[44], 12}, {NULL}};
const Segment SGI53056[] = {{&S83832[44], 38}, {NULL}};
const Segment SGI53057[] = {{&S83832[30], 6}, {NULL}};
const Segment SGI53058[] = {{&S83832[130], 3}, {NULL}};
const Segment SGI53059[] = {{&S83832[96], 5}, {NULL}};
const Segment SGI53060[] = {{&S83833[45], 10}, {NULL}};
const Segment SGI53061[] = {{&S83833[30], 6}, {NULL}};
const Segment SGI53062[] = {{&S83833[120], 3}, {NULL}};
const Segment SGI53063[] = {{&S83833[86], 5}, {NULL}};
const Segment SGI53064[] = {{&S83833[144], 13}, {NULL}};
const Segment SGI53065[] = {{&S84380[44], 10}, {NULL}};
const Segment SGI53066[] = {{&S84380[44], 21}, {NULL}};
const Segment SGI53067[] = {{&S84380[30], 6}, {NULL}};
const Segment SGI53068[] = {{&S84380[113], 3}, {NULL}};
const Segment SGI53069[] = {{&S84380[79], 5}, {NULL}};
const Segment SGI53070[] = {{&S84392[31], 5}, {NULL}};
const Segment SGI53071[] = {{&S84392[51], 3}, {NULL}};
const Segment SGI53072[] = {{&S83967[44], 10}, {NULL}};
const Segment SGI53073[] = {{&S83967[44], 27}, {NULL}};
const Segment SGI53074[] = {{&S83967[30], 6}, {NULL}};
const Segment SGI53075[] = {{&S83967[118], 3}, {NULL}};
const Segment SGI53076[] = {{&S83967[85], 5}, {NULL}};
const Segment SGI53077[] = {{&S83967[122], 14}, {NULL}};
const Segment SGI53078[] = {{&S83968[44], 11}, {NULL}};
const Segment SGI53079[] = {{&S83968[44], 28}, {NULL}};
const Segment SGI53080[] = {{&S83968[30], 6}, {NULL}};
const Segment SGI53081[] = {{&S83968[119], 3}, {NULL}};
const Segment SGI53082[] = {{&S83968[86], 5}, {NULL}};
const Segment SGI53083[] = {{&S83968[123], 14}, {NULL}};
const Segment SGI53084[] = {{&S83969[44], 10}, {NULL}};
const Segment SGI53085[] = {{&S83969[44], 31}, {NULL}};
const Segment SGI53086[] = {{&S83969[30], 6}, {NULL}};
const Segment SGI53087[] = {{&S83969[123], 3}, {NULL}};
const Segment SGI53088[] = {{&S83969[89], 5}, {NULL}};
const Segment SGI53089[] = {{&S84490[44], 10}, {NULL}};
const Segment SGI53090[] = {{&S84490[44], 21}, {NULL}};
const Segment SGI53091[] = {{&S84490[30], 6}, {NULL}};
const Segment SGI53092[] = {{&S84490[113], 3}, {NULL}};
const Segment SGI53093[] = {{&S84490[79], 5}, {NULL}};
const Segment SGI53094[] = {{&S84491[38], 10}, {NULL}};
const Segment SGI53095[] = {{&S84491[31], 6}, {NULL}};
const Segment SGI53096[] = {{&S84492[44], 10}, {NULL}};
const Segment SGI53097[] = {{&S84492[44], 21}, {NULL}};
const Segment SGI53098[] = {{&S84492[30], 6}, {NULL}};
const Segment SGI53099[] = {{&S84492[82], 4}, {NULL}};
const Segment SGI53100[] = {{&S84492[82], 3}, {NULL}};
const Segment SGI53101[] = {{&S84493[44], 10}, {NULL}};
const Segment SGI53102[] = {{&S84493[44], 21}, {NULL}};
const Segment SGI53103[] = {{&S84493[30], 6}, {NULL}};
const Segment SGI53104[] = {{&S84493[82], 6}, {NULL}};
const Segment SGI53105[] = {{&S84015[44], 23}, {NULL}};
const Segment SGI53106[] = {{&S84015[44], 34}, {NULL}};
const Segment SGI53107[] = {{&S84015[30], 6}, {NULL}};
const Segment SGI53108[] = {{&S84015[126], 3}, {NULL}};
const Segment SGI53109[] = {{&S84015[92], 5}, {NULL}};
const Segment SGI53110[] = {{&S84531[44], 10}, {NULL}};
const Segment SGI53111[] = {{&S84531[44], 27}, {NULL}};
const Segment SGI53112[] = {{&S84531[30], 6}, {NULL}};
const Segment SGI53113[] = {{&S84531[118], 3}, {NULL}};
const Segment SGI53114[] = {{&S84531[85], 5}, {NULL}};
const Segment SGI53115[] = {{&S84531[122], 14}, {NULL}};
const Segment SGI53116[] = {{&S84541[44], 10}, {NULL}};
const Segment SGI53117[] = {{&S84541[44], 27}, {NULL}};
const Segment SGI53118[] = {{&S84541[30], 6}, {NULL}};
const Segment SGI53119[] = {{&S84541[85], 6}, {NULL}};
const Segment SGI53120[] = {{&S84541[112], 13}, {NULL}};
const Segment SGI53121[] = {{&S84668[39], 5}, {NULL}};
const Segment SGI53122[] = {{&S84668[100], 3}, {NULL}};
const Segment SGI53123[] = {{&S84668[89], 3}, {NULL}};
const Segment SGI53124[] = {{&S84668[66], 6}, {NULL}};
const Segment SGI53125[] = {{&S84670[31], 5}, {NULL}};
const Segment SGI53126[] = {{&S84670[51], 3}, {NULL}};
const Segment SGI53127[] = {{&S84697[44], 15}, {NULL}};
const Segment SGI53128[] = {{&S84697[44], 26}, {NULL}};
const Segment SGI53129[] = {{&S84697[30], 6}, {NULL}};
const Segment SGI53130[] = {{&S84697[118], 3}, {NULL}};
const Segment SGI53131[] = {{&S84697[84], 5}, {NULL}};
const Segment SGI53132[] = {{&S99726[45], 9}, {NULL}};
const Segment SGI53133[] = {{&S99726[30], 6}, {NULL}};
const Segment SGI53134[] = {{&S99726[114], 3}, {NULL}};
const Segment SGI53135[] = {{&S99726[80], 5}, {NULL}};
const Segment SGI53136[] = {{&S99726[138], 13}, {NULL}};
const Segment SGI53137[] = {{&S87081[45], 9}, {NULL}};
const Segment SGI53138[] = {{&S87081[30], 6}, {NULL}};
const Segment SGI53139[] = {{&S87081[113], 3}, {NULL}};
const Segment SGI53140[] = {{&S87081[79], 5}, {NULL}};
const Segment SGI53141[] = {{&S87081[147], 3}, {NULL}};
const Segment SGI53142[] = {{&S84292[44], 6}, {NULL}};
const Segment SGI53143[] = {{&S84292[44], 17}, {NULL}};
const Segment SGI53144[] = {{&S84292[30], 6}, {NULL}};
const Segment SGI53145[] = {{&S84292[109], 3}, {NULL}};
const Segment SGI53146[] = {{&S84292[75], 5}, {NULL}};
const Segment SGI53147[] = {{&S99802[44], 11}, {NULL}};
const Segment SGI53148[] = {{&S99802[44], 22}, {NULL}};
const Segment SGI53149[] = {{&S99802[30], 6}, {NULL}};
const Segment SGI53150[] = {{&S99802[80], 6}, {NULL}};
const Segment SGI53151[] = {{&S99802[107], 13}, {NULL}};
const Segment SGI53152[] = {{&S87113[45], 6}, {NULL}};
const Segment SGI53153[] = {{&S87113[30], 6}, {NULL}};
const Segment SGI53154[] = {{&S87113[117], 3}, {NULL}};
const Segment SGI53155[] = {{&S87113[83], 5}, {NULL}};
const Segment SGI53156[] = {{&S87113[151], 3}, {NULL}};
const Segment SGI53157[] = {{&S99891[44], 11}, {NULL}};
const Segment SGI53158[] = {{&S99891[44], 22}, {NULL}};
const Segment SGI53159[] = {{&S99891[30], 6}, {NULL}};
const Segment SGI53160[] = {{&S99891[80], 6}, {NULL}};
const Segment SGI53161[] = {{&S99891[107], 13}, {NULL}};
const Segment SGI53162[] = {{&S84418[44], 12}, {NULL}};
const Segment SGI53163[] = {{&S84418[44], 23}, {NULL}};
const Segment SGI53164[] = {{&S84418[30], 6}, {NULL}};
const Segment SGI53165[] = {{&S84418[115], 3}, {NULL}};
const Segment SGI53166[] = {{&S84418[81], 5}, {NULL}};
const Segment SGI53167[] = {{&S84915[44], 7}, {NULL}};
const Segment SGI53168[] = {{&S84915[44], 25}, {NULL}};
const Segment SGI53169[] = {{&S84915[30], 6}, {NULL}};
const Segment SGI53170[] = {{&S84915[83], 6}, {NULL}};
const Segment SGI53171[] = {{&S84915[110], 13}, {NULL}};
const Segment SGI53172[] = {{&S84433[44], 17}, {NULL}};
const Segment SGI53173[] = {{&S84433[44], 38}, {NULL}};
const Segment SGI53174[] = {{&S84433[30], 6}, {NULL}};
const Segment SGI53175[] = {{&S84433[130], 3}, {NULL}};
const Segment SGI53176[] = {{&S84433[96], 5}, {NULL}};
const Segment SGI53177[] = {{&S84458[44], 15}, {NULL}};
const Segment SGI53178[] = {{&S84458[44], 36}, {NULL}};
const Segment SGI53179[] = {{&S84458[30], 6}, {NULL}};
const Segment SGI53180[] = {{&S84458[128], 3}, {NULL}};
const Segment SGI53181[] = {{&S84458[94], 5}, {NULL}};
const Segment SGI53182[] = {{&S84986[45], 7}, {NULL}};
const Segment SGI53183[] = {{&S84986[30], 6}, {NULL}};
const Segment SGI53184[] = {{&S84986[58], 3}, {NULL}};
const Segment SGI53185[] = {{&S84986[84], 5}, {NULL}};
const Segment SGI53186[] = {{&S84986[142], 13}, {NULL}};
const Segment SGI53187[] = {{&S84530[44], 10}, {NULL}};
const Segment SGI53188[] = {{&S84530[44], 27}, {NULL}};
const Segment SGI53189[] = {{&S84530[30], 6}, {NULL}};
const Segment SGI53190[] = {{&S84530[118], 3}, {NULL}};
const Segment SGI53191[] = {{&S84530[85], 5}, {NULL}};
const Segment SGI53192[] = {{&S84530[122], 14}, {NULL}};
const Segment SGI53193[] = {{&S84600[44], 9}, {NULL}};
const Segment SGI53194[] = {{&S84600[44], 20}, {NULL}};
const Segment SGI53195[] = {{&S84600[30], 6}, {NULL}};
const Segment SGI53196[] = {{&S84600[112], 3}, {NULL}};
const Segment SGI53197[] = {{&S84600[78], 5}, {NULL}};
const Segment SGI53198[] = {{&S84600[135], 10}, {NULL}};
const Segment SGI53199[] = {{&S92881[42], 7}, {NULL}};
const Segment SGI53200[] = {{&S92881[42], 24}, {NULL}};
const Segment SGI53201[] = {{&S92881[30], 4}, {NULL}};
const Segment SGI53202[] = {{&S92881[114], 3}, {NULL}};
const Segment SGI53203[] = {{&S92881[80], 5}, {NULL}};
const Segment SGI53204[] = {{&S84631[44], 14}, {NULL}};
const Segment SGI53205[] = {{&S84631[44], 31}, {NULL}};
const Segment SGI53206[] = {{&S84631[30], 6}, {NULL}};
const Segment SGI53207[] = {{&S84631[89], 6}, {NULL}};
const Segment SGI53208[] = {{&S84631[116], 13}, {NULL}};
const Segment SGI53209[] = {{&S85045[44], 11}, {NULL}};
const Segment SGI53210[] = {{&S85045[44], 21}, {NULL}};
const Segment SGI53211[] = {{&S85045[30], 6}, {NULL}};
const Segment SGI53212[] = {{&S85045[113], 3}, {NULL}};
const Segment SGI53213[] = {{&S85045[79], 5}, {NULL}};
const Segment SGI53214[] = {{&S100195[31], 7}, {NULL}};
const Segment SGI53215[] = {{&S100195[53], 3}, {NULL}};
const Segment SGI53216[] = {{&S85159[44], 12}, {NULL}};
const Segment SGI53217[] = {{&S85159[44], 23}, {NULL}};
const Segment SGI53218[] = {{&S85159[30], 6}, {NULL}};
const Segment SGI53219[] = {{&S85159[115], 3}, {NULL}};
const Segment SGI53220[] = {{&S85159[81], 5}, {NULL}};
const Segment SGI53221[] = {{&S85160[45], 7}, {NULL}};
const Segment SGI53222[] = {{&S85160[30], 6}, {NULL}};
const Segment SGI53223[] = {{&S85160[58], 3}, {NULL}};
const Segment SGI53224[] = {{&S100247[45], 11}, {NULL}};
const Segment SGI53225[] = {{&S85160[84], 5}, {NULL}};
const Segment SGI53226[] = {{&S100247[30], 6}, {NULL}};
const Segment SGI53227[] = {{&S85160[142], 13}, {NULL}};
const Segment SGI53228[] = {{&S100247[115], 3}, {NULL}};
const Segment SGI53229[] = {{&S100247[81], 5}, {NULL}};
const Segment SGI53230[] = {{&S100247[139], 13}, {NULL}};
const Segment SGI53231[] = {{&S93067[42], 9}, {NULL}};
const Segment SGI53232[] = {{&S93067[42], 26}, {NULL}};
const Segment SGI53233[] = {{&S93067[30], 4}, {NULL}};
const Segment SGI53234[] = {{&S93067[116], 3}, {NULL}};
const Segment SGI53235[] = {{&S93067[82], 5}, {NULL}};
const Segment SGI53236[] = {{&S94104[45], 10}, {NULL}};
const Segment SGI53237[] = {{&S94104[30], 6}, {NULL}};
const Segment SGI53238[] = {{&S94104[120], 3}, {NULL}};
const Segment SGI53239[] = {{&S94104[86], 5}, {NULL}};
const Segment SGI53240[] = {{&S85342[39], 8}, {NULL}};
const Segment SGI53241[] = {{&S85342[103], 3}, {NULL}};
const Segment SGI53242[] = {{&S85342[92], 3}, {NULL}};
const Segment SGI53243[] = {{&S85342[69], 6}, {NULL}};
const Segment SGI53244[] = {{&S85343[31], 8}, {NULL}};
const Segment SGI53245[] = {{&S85343[54], 3}, {NULL}};
const Segment SGI53246[] = {{&S84787[44], 10}, {NULL}};
const Segment SGI53247[] = {{&S84787[44], 27}, {NULL}};
const Segment SGI53248[] = {{&S84787[30], 6}, {NULL}};
const Segment SGI53249[] = {{&S84787[119], 3}, {NULL}};
const Segment SGI53250[] = {{&S84787[85], 5}, {NULL}};
const Segment SGI53251[] = {{&S84788[44], 10}, {NULL}};
const Segment SGI53252[] = {{&S84788[44], 27}, {NULL}};
const Segment SGI53253[] = {{&S84788[30], 6}, {NULL}};
const Segment SGI53254[] = {{&S84788[88], 6}, {NULL}};
const Segment SGI53255[] = {{&S84790[45], 14}, {NULL}};
const Segment SGI53256[] = {{&S84790[30], 6}, {NULL}};
const Segment SGI53257[] = {{&S84790[124], 3}, {NULL}};
const Segment SGI53258[] = {{&S84790[90], 5}, {NULL}};
const Segment SGI53259[] = {{&S84790[148], 13}, {NULL}};
const Segment SGI53260[] = {{&S84827[44], 11}, {NULL}};
const Segment SGI53261[] = {{&S84827[44], 32}, {NULL}};
const Segment SGI53262[] = {{&S84827[30], 6}, {NULL}};
const Segment SGI53263[] = {{&S84827[124], 3}, {NULL}};
const Segment SGI53264[] = {{&S84827[90], 5}, {NULL}};
const Segment SGI53265[] = {{&S84835[44], 13}, {NULL}};
const Segment SGI53266[] = {{&S84835[44], 30}, {NULL}};
const Segment SGI53267[] = {{&S84835[30], 6}, {NULL}};
const Segment SGI53268[] = {{&S84835[88], 6}, {NULL}};
const Segment SGI53269[] = {{&S87717[44], 18}, {NULL}};
const Segment SGI53270[] = {{&S84835[115], 13}, {NULL}};
const Segment SGI53271[] = {{&S87717[44], 29}, {NULL}};
const Segment SGI53272[] = {{&S87717[30], 6}, {NULL}};
const Segment SGI53273[] = {{&S87717[87], 6}, {NULL}};
const Segment SGI53274[] = {{&S87717[124], 3}, {NULL}};
const Segment SGI53275[] = {{&S94263[59], 31}, {NULL}};
const Segment SGI53276[] = {{&S94263[45], 6}, {NULL}};
const Segment SGI53277[] = {{&S94263[138], 3}, {NULL}};
const Segment SGI53278[] = {{&S94263[104], 5}, {NULL}};
const Segment SGI53279[] = {{&S87795[44], 19}, {NULL}};
const Segment SGI53280[] = {{&S87795[44], 30}, {NULL}};
const Segment SGI53281[] = {{&S87795[30], 6}, {NULL}};
const Segment SGI53282[] = {{&S87795[122], 3}, {NULL}};
const Segment SGI53283[] = {{&S87795[88], 5}, {NULL}};
const Segment SGI53284[] = {{&S87843[59], 12}, {NULL}};
const Segment SGI53285[] = {{&S87843[59], 23}, {NULL}};
const Segment SGI53286[] = {{&S87843[45], 6}, {NULL}};
const Segment SGI53287[] = {{&S87843[130], 3}, {NULL}};
const Segment SGI53288[] = {{&S87843[96], 5}, {NULL}};
const Segment SGI53289[] = {{&S87843[0], 14}, {NULL}};
const Segment SGI53290[] = {{&S84990[44], 5}, {NULL}};
const Segment SGI53291[] = {{&S84990[44], 22}, {NULL}};
const Segment SGI53292[] = {{&S84990[30], 6}, {NULL}};
const Segment SGI53293[] = {{&S84990[114], 3}, {NULL}};
const Segment SGI53294[] = {{&S84990[80], 5}, {NULL}};
const Segment SGI53295[] = {{&S85013[44], 18}, {NULL}};
const Segment SGI53296[] = {{&S85013[44], 29}, {NULL}};
const Segment SGI53297[] = {{&S85013[30], 6}, {NULL}};
const Segment SGI53298[] = {{&S85013[87], 6}, {NULL}};
const Segment SGI53299[] = {{&S85013[114], 13}, {NULL}};
const Segment SGI53300[] = {{&S85035[44], 20}, {NULL}};
const Segment SGI53301[] = {{&S85035[44], 31}, {NULL}};
const Segment SGI53302[] = {{&S85035[30], 6}, {NULL}};
const Segment SGI53303[] = {{&S85035[89], 6}, {NULL}};
const Segment SGI53304[] = {{&S85035[116], 13}, {NULL}};
const Segment SGI53305[] = {{&S85044[44], 11}, {NULL}};
const Segment SGI53306[] = {{&S85044[44], 21}, {NULL}};
const Segment SGI53307[] = {{&S85044[30], 6}, {NULL}};
const Segment SGI53308[] = {{&S85044[113], 3}, {NULL}};
const Segment SGI53309[] = {{&S85044[79], 5}, {NULL}};
const Segment SGI53310[] = {{&S85063[31], 8}, {NULL}};
const Segment SGI53311[] = {{&S85063[54], 3}, {NULL}};
const Segment SGI53312[] = {{&S85575[59], 10}, {NULL}};
const Segment SGI53313[] = {{&S85575[59], 27}, {NULL}};
const Segment SGI53314[] = {{&S85575[45], 6}, {NULL}};
const Segment SGI53315[] = {{&S85575[134], 3}, {NULL}};
const Segment SGI53316[] = {{&S85575[100], 5}, {NULL}};
const Segment SGI53317[] = {{&S85575[0], 14}, {NULL}};
const Segment SGI53318[] = {{&S85627[44], 10}, {NULL}};
const Segment SGI53319[] = {{&S85627[44], 27}, {NULL}};
const Segment SGI53320[] = {{&S85627[30], 6}, {NULL}};
const Segment SGI53321[] = {{&S85627[85], 6}, {NULL}};
const Segment SGI53322[] = {{&S85627[112], 13}, {NULL}};
const Segment SGI53323[] = {{&S85156[45], 22}, {NULL}};
const Segment SGI53324[] = {{&S85156[30], 6}, {NULL}};
const Segment SGI53325[] = {{&S85156[126], 3}, {NULL}};
const Segment SGI53326[] = {{&S85156[92], 5}, {NULL}};
const Segment SGI53327[] = {{&S85156[150], 13}, {NULL}};
const Segment SGI53328[] = {{&S85672[44], 11}, {NULL}};
const Segment SGI53329[] = {{&S85672[44], 28}, {NULL}};
const Segment SGI53330[] = {{&S85672[30], 6}, {NULL}};
const Segment SGI53331[] = {{&S85672[120], 3}, {NULL}};
const Segment SGI53332[] = {{&S85672[86], 5}, {NULL}};
const Segment SGI53333[] = {{&S100794[58], 14}, {NULL}};
const Segment SGI53334[] = {{&S100794[58], 26}, {NULL}};
const Segment SGI53335[] = {{&S100794[44], 6}, {NULL}};
const Segment SGI53336[] = {{&S100794[132], 3}, {NULL}};
const Segment SGI53337[] = {{&S100794[98], 5}, {NULL}};
const Segment SGI53338[] = {{&S85257[44], 9}, {NULL}};
const Segment SGI53339[] = {{&S85257[44], 35}, {NULL}};
const Segment SGI53340[] = {{&S85257[30], 6}, {NULL}};
const Segment SGI53341[] = {{&S85257[127], 3}, {NULL}};
const Segment SGI53342[] = {{&S85257[93], 5}, {NULL}};
const Segment SGI53343[] = {{&S85312[44], 18}, {NULL}};
const Segment SGI53344[] = {{&S85312[44], 29}, {NULL}};
const Segment SGI53345[] = {{&S85312[30], 6}, {NULL}};
const Segment SGI53346[] = {{&S85312[87], 6}, {NULL}};
const Segment SGI53347[] = {{&S85312[114], 13}, {NULL}};
const Segment SGI53348[] = {{&S100890[59], 10}, {NULL}};
const Segment SGI53349[] = {{&S100890[59], 21}, {NULL}};
const Segment SGI53350[] = {{&S100890[45], 6}, {NULL}};
const Segment SGI53351[] = {{&S100890[128], 3}, {NULL}};
const Segment SGI53352[] = {{&S100890[94], 5}, {NULL}};
const Segment SGI53353[] = {{&S101086[58], 27}, {NULL}};
const Segment SGI53354[] = {{&S101086[44], 6}, {NULL}};
const Segment SGI53355[] = {{&S101086[151], 3}, {NULL}};
const Segment SGI53356[] = {{&S101086[117], 5}, {NULL}};
const Segment SGI53357[] = {{&S101090[59], 9}, {NULL}};
const Segment SGI53358[] = {{&S101090[59], 20}, {NULL}};
const Segment SGI53359[] = {{&S101090[45], 6}, {NULL}};
const Segment SGI53360[] = {{&S101090[127], 3}, {NULL}};
const Segment SGI53361[] = {{&S101090[93], 5}, {NULL}};
const Segment SGI53362[] = {{&S101090[0], 14}, {NULL}};
const Segment SGI53363[] = {{&S88420[44], 17}, {NULL}};
const Segment SGI53364[] = {{&S88420[44], 28}, {NULL}};
const Segment SGI53365[] = {{&S88420[30], 6}, {NULL}};
const Segment SGI53366[] = {{&S88420[120], 3}, {NULL}};
const Segment SGI53367[] = {{&S88420[86], 5}, {NULL}};
const Segment SGI53368[] = {{&S88425[44], 11}, {NULL}};
const Segment SGI53369[] = {{&S88425[44], 22}, {NULL}};
const Segment SGI53370[] = {{&S88425[30], 6}, {NULL}};
const Segment SGI53371[] = {{&S88425[114], 3}, {NULL}};
const Segment SGI53372[] = {{&S88425[80], 5}, {NULL}};
const Segment SGI53373[] = {{&S85919[44], 11}, {NULL}};
const Segment SGI53374[] = {{&S85919[44], 36}, {NULL}};
const Segment SGI53375[] = {{&S85919[30], 6}, {NULL}};
const Segment SGI53376[] = {{&S85919[128], 3}, {NULL}};
const Segment SGI53377[] = {{&S85919[94], 5}, {NULL}};
const Segment SGI53378[] = {{&S85920[44], 11}, {NULL}};
const Segment SGI53379[] = {{&S85920[44], 36}, {NULL}};
const Segment SGI53380[] = {{&S85920[30], 6}, {NULL}};
const Segment SGI53381[] = {{&S85920[97], 4}, {NULL}};
const Segment SGI53382[] = {{&S85920[97], 3}, {NULL}};
const Segment SGI53383[] = {{&S85709[44], 22}, {NULL}};
const Segment SGI53384[] = {{&S85709[44], 33}, {NULL}};
const Segment SGI53385[] = {{&S85709[30], 6}, {NULL}};
const Segment SGI53386[] = {{&S85709[125], 3}, {NULL}};
const Segment SGI53387[] = {{&S85709[91], 5}, {NULL}};
const Segment SGI53388[] = {{&S85710[45], 14}, {NULL}};
const Segment SGI53389[] = {{&S85710[30], 6}, {NULL}};
const Segment SGI53390[] = {{&S85710[124], 3}, {NULL}};
const Segment SGI53391[] = {{&S85710[90], 5}, {NULL}};
const Segment SGI53392[] = {{&S85710[148], 13}, {NULL}};
const Segment SGI53393[] = {{&S85711[44], 14}, {NULL}};
const Segment SGI53394[] = {{&S85711[44], 31}, {NULL}};
const Segment SGI53395[] = {{&S85711[30], 6}, {NULL}};
const Segment SGI53396[] = {{&S85711[124], 3}, {NULL}};
const Segment SGI53397[] = {{&S85711[89], 6}, {NULL}};
const Segment SGI53398[] = {{&S85711[144], 8}, {NULL}};
const Segment SGI53399[] = {{&S85711[153], 13}, {NULL}};
const Segment SGI53400[] = {{&S85721[44], 4}, {NULL}};
const Segment SGI53401[] = {{&S85721[44], 15}, {NULL}};
const Segment SGI53402[] = {{&S85721[30], 6}, {NULL}};
const Segment SGI53403[] = {{&S85721[107], 3}, {NULL}};
const Segment SGI53404[] = {{&S85721[73], 5}, {NULL}};
const Segment SGI53405[] = {{&S85722[44], 4}, {NULL}};
const Segment SGI53406[] = {{&S85722[44], 25}, {NULL}};
const Segment SGI53407[] = {{&S85722[30], 6}, {NULL}};
const Segment SGI53408[] = {{&S85722[117], 3}, {NULL}};
const Segment SGI53409[] = {{&S85722[83], 5}, {NULL}};
const Segment SGI53410[] = {{&S86040[44], 11}, {NULL}};
const Segment SGI53411[] = {{&S86040[44], 22}, {NULL}};
const Segment SGI53412[] = {{&S86040[30], 6}, {NULL}};
const Segment SGI53413[] = {{&S86040[114], 3}, {NULL}};
const Segment SGI53414[] = {{&S86040[80], 5}, {NULL}};
const Segment SGI53415[] = {{&S86083[34], 7}, {NULL}};
const Segment SGI53416[] = {{&S86083[27], 6}, {NULL}};
const Segment SGI53417[] = {{&S86083[107], 2}, {NULL}};
const Segment SGI53418[] = {{&S86083[73], 6}, {NULL}};
const Segment SGI53419[] = {{&S86083[141], 8}, {NULL}};
const Segment SGI53420[] = {{&S101367[31], 9}, {NULL}};
const Segment SGI53421[] = {{&S101367[55], 3}, {NULL}};
const Segment SGI53422[] = {{&S88554[44], 9}, {NULL}};
const Segment SGI53423[] = {{&S88554[44], 20}, {NULL}};
const Segment SGI53424[] = {{&S88554[30], 6}, {NULL}};
const Segment SGI53425[] = {{&S88554[78], 6}, {NULL}};
const Segment SGI53426[] = {{&S88554[115], 3}, {NULL}};
const Segment SGI53427[] = {{&S101379[57], 9}, {NULL}};
const Segment SGI53428[] = {{&S101379[57], 20}, {NULL}};
const Segment SGI53429[] = {{&S101379[43], 6}, {NULL}};
const Segment SGI53430[] = {{&S101379[125], 3}, {NULL}};
const Segment SGI53431[] = {{&S101379[91], 5}, {NULL}};
const Segment SGI53432[] = {{&S85806[44], 7}, {NULL}};
const Segment SGI53433[] = {{&S85806[44], 18}, {NULL}};
const Segment SGI53434[] = {{&S85806[30], 6}, {NULL}};
const Segment SGI53435[] = {{&S85806[110], 3}, {NULL}};
const Segment SGI53436[] = {{&S85806[76], 5}, {NULL}};
const Segment SGI53437[] = {{&S88579[39], 5}, {NULL}};
const Segment SGI53438[] = {{&S88579[100], 3}, {NULL}};
const Segment SGI53439[] = {{&S88579[89], 3}, {NULL}};
const Segment SGI53440[] = {{&S88579[66], 6}, {NULL}};
const Segment SGI53441[] = {{&S86128[44], 23}, {NULL}};
const Segment SGI53442[] = {{&S86128[44], 40}, {NULL}};
const Segment SGI53443[] = {{&S86128[30], 6}, {NULL}};
const Segment SGI53444[] = {{&S86128[132], 3}, {NULL}};
const Segment SGI53445[] = {{&S86128[98], 5}, {NULL}};
const Segment SGI53446[] = {{&S86129[44], 23}, {NULL}};
const Segment SGI53447[] = {{&S86129[44], 40}, {NULL}};
const Segment SGI53448[] = {{&S86129[30], 6}, {NULL}};
const Segment SGI53449[] = {{&S86129[132], 3}, {NULL}};
const Segment SGI53450[] = {{&S86129[98], 5}, {NULL}};
const Segment SGI53451[] = {{&S85833[44], 10}, {NULL}};
const Segment SGI53452[] = {{&S85833[44], 33}, {NULL}};
const Segment SGI53453[] = {{&S85833[30], 6}, {NULL}};
const Segment SGI53454[] = {{&S85833[125], 3}, {NULL}};
const Segment SGI53455[] = {{&S85833[91], 5}, {NULL}};
const Segment SGI53456[] = {{&S85833[149], 11}, {NULL}};
const Segment SGI53457[] = {{&S86240[45], 7}, {NULL}};
const Segment SGI53458[] = {{&S86240[30], 6}, {NULL}};
const Segment SGI53459[] = {{&S86240[58], 3}, {NULL}};
const Segment SGI53460[] = {{&S86240[84], 5}, {NULL}};
const Segment SGI53461[] = {{&S86240[142], 13}, {NULL}};
const Segment SGI53462[] = {{&S85945[44], 14}, {NULL}};
const Segment SGI53463[] = {{&S85945[44], 35}, {NULL}};
const Segment SGI53464[] = {{&S85945[30], 6}, {NULL}};
const Segment SGI53465[] = {{&S85945[127], 3}, {NULL}};
const Segment SGI53466[] = {{&S85945[93], 5}, {NULL}};
const Segment SGI53467[] = {{&S86284[44], 9}, {NULL}};
const Segment SGI53468[] = {{&S86284[44], 20}, {NULL}};
const Segment SGI53469[] = {{&S86284[30], 6}, {NULL}};
const Segment SGI53470[] = {{&S86284[78], 6}, {NULL}};
const Segment SGI53471[] = {{&S86284[105], 13}, {NULL}};
const Segment SGI53472[] = {{&S85977[44], 14}, {NULL}};
const Segment SGI53473[] = {{&S85977[44], 35}, {NULL}};
const Segment SGI53474[] = {{&S85977[30], 6}, {NULL}};
const Segment SGI53475[] = {{&S85977[93], 6}, {NULL}};
const Segment SGI53476[] = {{&S85977[120], 13}, {NULL}};
const Segment SGI53477[] = {{&S101531[57], 11}, {NULL}};
const Segment SGI53478[] = {{&S101531[57], 22}, {NULL}};
const Segment SGI53479[] = {{&S101531[43], 6}, {NULL}};
const Segment SGI53480[] = {{&S101531[127], 3}, {NULL}};
const Segment SGI53481[] = {{&S101531[93], 5}, {NULL}};
const Segment SGI53482[] = {{&S101533[45], 11}, {NULL}};
const Segment SGI53483[] = {{&S101533[30], 6}, {NULL}};
const Segment SGI53484[] = {{&S101533[115], 3}, {NULL}};
const Segment SGI53485[] = {{&S101533[81], 5}, {NULL}};
const Segment SGI53486[] = {{&S101533[139], 13}, {NULL}};
const Segment SGI53487[] = {{&S86358[44], 13}, {NULL}};
const Segment SGI53488[] = {{&S86358[44], 30}, {NULL}};
const Segment SGI53489[] = {{&S86358[30], 6}, {NULL}};
const Segment SGI53490[] = {{&S86358[122], 3}, {NULL}};
const Segment SGI53491[] = {{&S86358[88], 5}, {NULL}};
const Segment SGI53492[] = {{&S86358[145], 10}, {NULL}};
const Segment SGI53493[] = {{&S86093[44], 14}, {NULL}};
const Segment SGI53494[] = {{&S86093[44], 25}, {NULL}};
const Segment SGI53495[] = {{&S86093[30], 6}, {NULL}};
const Segment SGI53496[] = {{&S86093[117], 3}, {NULL}};
const Segment SGI53497[] = {{&S86093[83], 5}, {NULL}};
const Segment SGI53498[] = {{&S86431[44], 12}, {NULL}};
const Segment SGI53499[] = {{&S86431[44], 23}, {NULL}};
const Segment SGI53500[] = {{&S86431[30], 6}, {NULL}};
const Segment SGI53501[] = {{&S86431[84], 4}, {NULL}};
const Segment SGI53502[] = {{&S86431[84], 3}, {NULL}};
const Segment SGI53503[] = {{&S86432[44], 12}, {NULL}};
const Segment SGI53504[] = {{&S86432[44], 23}, {NULL}};
const Segment SGI53505[] = {{&S86432[30], 6}, {NULL}};
const Segment SGI53506[] = {{&S86432[115], 3}, {NULL}};
const Segment SGI53507[] = {{&S86432[81], 5}, {NULL}};
const Segment SGI53508[] = {{&S86434[44], 12}, {NULL}};
const Segment SGI53509[] = {{&S86434[44], 23}, {NULL}};
const Segment SGI53510[] = {{&S86434[30], 6}, {NULL}};
const Segment SGI53511[] = {{&S86434[84], 6}, {NULL}};
const Segment SGI53512[] = {{&S86164[45], 5}, {NULL}};
const Segment SGI53513[] = {{&S86164[30], 6}, {NULL}};
const Segment SGI53514[] = {{&S86164[115], 3}, {NULL}};
const Segment SGI53515[] = {{&S86164[81], 5}, {NULL}};
const Segment SGI53516[] = {{&S86164[139], 13}, {NULL}};
const Segment SGI53517[] = {{&S86165[44], 5}, {NULL}};
const Segment SGI53518[] = {{&S86165[44], 22}, {NULL}};
const Segment SGI53519[] = {{&S86165[30], 6}, {NULL}};
const Segment SGI53520[] = {{&S86165[115], 3}, {NULL}};
const Segment SGI53521[] = {{&S86165[80], 6}, {NULL}};
const Segment SGI53522[] = {{&S86165[135], 8}, {NULL}};
const Segment SGI53523[] = {{&S86165[144], 13}, {NULL}};
const Segment SGI53524[] = {{&S101731[45], 7}, {NULL}};
const Segment SGI53525[] = {{&S101731[30], 6}, {NULL}};
const Segment SGI53526[] = {{&S101731[117], 3}, {NULL}};
const Segment SGI53527[] = {{&S101731[83], 5}, {NULL}};
const Segment SGI53528[] = {{&S101731[141], 13}, {NULL}};
const Segment SGI53529[] = {{&S101763[45], 10}, {NULL}};
const Segment SGI53530[] = {{&S101763[30], 6}, {NULL}};
const Segment SGI53531[] = {{&S101763[114], 3}, {NULL}};
const Segment SGI53532[] = {{&S101763[80], 5}, {NULL}};
const Segment SGI53533[] = {{&S101763[138], 13}, {NULL}};
const Segment SGI53534[] = {{&S88899[44], 14}, {NULL}};
const Segment SGI53535[] = {{&S88899[44], 25}, {NULL}};
const Segment SGI53536[] = {{&S88899[30], 6}, {NULL}};
const Segment SGI53537[] = {{&S88899[83], 6}, {NULL}};
const Segment SGI53538[] = {{&S88899[120], 3}, {NULL}};
const Segment SGI53539[] = {{&S86561[45], 11}, {NULL}};
const Segment SGI53540[] = {{&S86561[30], 6}, {NULL}};
const Segment SGI53541[] = {{&S86561[121], 3}, {NULL}};
const Segment SGI53542[] = {{&S86561[87], 5}, {NULL}};
const Segment SGI53543[] = {{&S86561[145], 13}, {NULL}};
const Segment SGI53544[] = {{&S86334[44], 9}, {NULL}};
const Segment SGI53545[] = {{&S86334[44], 20}, {NULL}};
const Segment SGI53546[] = {{&S86334[30], 6}, {NULL}};
const Segment SGI53547[] = {{&S86334[112], 3}, {NULL}};
const Segment SGI53548[] = {{&S86334[78], 5}, {NULL}};
const Segment SGI53549[] = {{&S101903[45], 10}, {NULL}};
const Segment SGI53550[] = {{&S101903[30], 6}, {NULL}};
const Segment SGI53551[] = {{&S101903[114], 3}, {NULL}};
const Segment SGI53552[] = {{&S101903[80], 5}, {NULL}};
const Segment SGI53553[] = {{&S101903[138], 13}, {NULL}};
const Segment SGI53554[] = {{&S101911[59], 43}, {NULL}};
const Segment SGI53555[] = {{&S101911[45], 6}, {NULL}};
const Segment SGI53556[] = {{&S86683[11], 1}, {NULL}};
const Segment SGI53557[] = {{&S86683[44], 12}, {NULL}};
const Segment SGI53558[] = {{&S86683[30], 6}, {NULL}};
const Segment SGI53559[] = {{&S101911[134], 5}, {NULL}};
const Segment SGI53560[] = {{&S101911[0], 14}, {NULL}};
const Segment SGI53561[] = {{&S86683[133], 3}, {NULL}};
const Segment SGI53562[] = {{&S86683[99], 5}, {NULL}};
const Segment SGI53563[] = {{&S86350[44], 13}, {NULL}};
const Segment SGI53564[] = {{&S86350[44], 30}, {NULL}};
const Segment SGI53565[] = {{&S86350[30], 6}, {NULL}};
const Segment SGI53566[] = {{&S86350[122], 3}, {NULL}};
const Segment SGI53567[] = {{&S86350[88], 5}, {NULL}};
const Segment SGI53568[] = {{&S86369[44], 10}, {NULL}};
const Segment SGI53569[] = {{&S86369[44], 27}, {NULL}};
const Segment SGI53570[] = {{&S86369[30], 6}, {NULL}};
const Segment SGI53571[] = {{&S86369[85], 6}, {NULL}};
const Segment SGI53572[] = {{&S86369[112], 13}, {NULL}};
const Segment SGI53573[] = {{&S86379[44], 11}, {NULL}};
const Segment SGI53574[] = {{&S86379[44], 28}, {NULL}};
const Segment SGI53575[] = {{&S86379[30], 6}, {NULL}};
const Segment SGI53576[] = {{&S86379[86], 6}, {NULL}};
const Segment SGI53577[] = {{&S86379[113], 13}, {NULL}};
const Segment SGI53578[] = {{&S89032[44], 11}, {NULL}};
const Segment SGI53579[] = {{&S89032[44], 22}, {NULL}};
const Segment SGI53580[] = {{&S89032[30], 6}, {NULL}};
const Segment SGI53581[] = {{&S89032[80], 6}, {NULL}};
const Segment SGI53582[] = {{&S89032[117], 3}, {NULL}};
const Segment SGI53583[] = {{&S89037[45], 10}, {NULL}};
const Segment SGI53584[] = {{&S89037[30], 6}, {NULL}};
const Segment SGI53585[] = {{&S89037[120], 3}, {NULL}};
const Segment SGI53586[] = {{&S89037[86], 5}, {NULL}};
const Segment SGI53587[] = {{&S89037[154], 3}, {NULL}};
const Segment SGI53588[] = {{&S86433[44], 12}, {NULL}};
const Segment SGI53589[] = {{&S86433[44], 23}, {NULL}};
const Segment SGI53590[] = {{&S86433[30], 6}, {NULL}};
const Segment SGI53591[] = {{&S86433[115], 3}, {NULL}};
const Segment SGI53592[] = {{&S86433[81], 5}, {NULL}};
const Segment SGI53593[] = {{&S86785[31], 8}, {NULL}};
const Segment SGI53594[] = {{&S86785[55], 3}, {NULL}};
const Segment SGI53595[] = {{&S86439[45], 10}, {NULL}};
const Segment SGI53596[] = {{&S86439[30], 6}, {NULL}};
const Segment SGI53597[] = {{&S86439[120], 3}, {NULL}};
const Segment SGI53598[] = {{&S86439[86], 5}, {NULL}};
const Segment SGI53599[] = {{&S86439[144], 13}, {NULL}};
const Segment SGI53600[] = {{&S86559[45], 10}, {NULL}};
const Segment SGI53601[] = {{&S86559[30], 6}, {NULL}};
const Segment SGI53602[] = {{&S86559[120], 3}, {NULL}};
const Segment SGI53603[] = {{&S86559[86], 5}, {NULL}};
const Segment SGI53604[] = {{&S86559[144], 13}, {NULL}};
const Segment SGI53605[] = {{&S89219[44], 18}, {NULL}};
const Segment SGI53606[] = {{&S89219[44], 29}, {NULL}};
const Segment SGI53607[] = {{&S89219[30], 6}, {NULL}};
const Segment SGI53608[] = {{&S89219[87], 6}, {NULL}};
const Segment SGI53609[] = {{&S89219[124], 3}, {NULL}};
const Segment SGI53610[] = {{&S86922[44], 16}, {NULL}};
const Segment SGI53611[] = {{&S86922[44], 27}, {NULL}};
const Segment SGI53612[] = {{&S86922[30], 6}, {NULL}};
const Segment SGI53613[] = {{&S86922[119], 3}, {NULL}};
const Segment SGI53614[] = {{&S86922[85], 5}, {NULL}};
const Segment SGI53615[] = {{&S86926[44], 6}, {NULL}};
const Segment SGI53616[] = {{&S86926[44], 23}, {NULL}};
const Segment SGI53617[] = {{&S86926[30], 6}, {NULL}};
const Segment SGI53618[] = {{&S86926[115], 3}, {NULL}};
const Segment SGI53619[] = {{&S86926[81], 5}, {NULL}};
const Segment SGI53620[] = {{&S86673[44], 10}, {NULL}};
const Segment SGI53621[] = {{&S86673[44], 30}, {NULL}};
const Segment SGI53622[] = {{&S86673[30], 6}, {NULL}};
const Segment SGI53623[] = {{&S86673[122], 3}, {NULL}};
const Segment SGI53624[] = {{&S86673[88], 5}, {NULL}};
const Segment SGI53625[] = {{&S86941[59], 7}, {NULL}};
const Segment SGI53626[] = {{&S86941[59], 25}, {NULL}};
const Segment SGI53627[] = {{&S86941[45], 6}, {NULL}};
const Segment SGI53628[] = {{&S86941[72], 3}, {NULL}};
const Segment SGI53629[] = {{&S86941[98], 5}, {NULL}};
const Segment SGI53630[] = {{&S86941[0], 14}, {NULL}};
const Segment SGI53631[] = {{&S86971[44], 7}, {NULL}};
const Segment SGI53632[] = {{&S86971[44], 25}, {NULL}};
const Segment SGI53633[] = {{&S86971[30], 6}, {NULL}};
const Segment SGI53634[] = {{&S86971[83], 6}, {NULL}};
const Segment SGI53635[] = {{&S86971[110], 13}, {NULL}};
const Segment SGI53636[] = {{&S102266[57], 12}, {NULL}};
const Segment SGI53637[] = {{&S102266[57], 23}, {NULL}};
const Segment SGI53638[] = {{&S102266[43], 6}, {NULL}};
const Segment SGI53639[] = {{&S102266[128], 3}, {NULL}};
const Segment SGI53640[] = {{&S102266[94], 5}, {NULL}};
const Segment SGI53641[] = {{&S89412[44], 7}, {NULL}};
const Segment SGI53642[] = {{&S89412[44], 15}, {NULL}};
const Segment SGI53643[] = {{&S89412[30], 6}, {NULL}};
const Segment SGI53644[] = {{&S89412[107], 3}, {NULL}};
const Segment SGI53645[] = {{&S89412[73], 5}, {NULL}};
const Segment SGI53646[] = {{&S89413[38], 7}, {NULL}};
const Segment SGI53647[] = {{&S89413[31], 6}, {NULL}};
const Segment SGI53648[] = {{&S89440[59], 37}, {NULL}};
const Segment SGI53649[] = {{&S89440[45], 6}, {NULL}};
const Segment SGI53650[] = {{&S89440[128], 5}, {NULL}};
const Segment SGI53651[] = {{&S87075[44], 10}, {NULL}};
const Segment SGI53652[] = {{&S87075[44], 21}, {NULL}};
const Segment SGI53653[] = {{&S87075[30], 6}, {NULL}};
const Segment SGI53654[] = {{&S89440[0], 14}, {NULL}};
const Segment SGI53655[] = {{&S87075[113], 3}, {NULL}};
const Segment SGI53656[] = {{&S87075[79], 5}, {NULL}};
const Segment SGI53657[] = {{&S102379[44], 11}, {NULL}};
const Segment SGI53658[] = {{&S102379[44], 22}, {NULL}};
const Segment SGI53659[] = {{&S102379[30], 6}, {NULL}};
const Segment SGI53660[] = {{&S102379[114], 3}, {NULL}};
const Segment SGI53661[] = {{&S102379[80], 5}, {NULL}};
const Segment SGI53662[] = {{&S102384[44], 12}, {NULL}};
const Segment SGI53663[] = {{&S102384[44], 24}, {NULL}};
const Segment SGI53664[] = {{&S102384[30], 6}, {NULL}};
const Segment SGI53665[] = {{&S102384[116], 3}, {NULL}};
const Segment SGI53666[] = {{&S102384[82], 5}, {NULL}};
const Segment SGI53667[] = {{&S102477[45], 9}, {NULL}};
const Segment SGI53668[] = {{&S102477[30], 6}, {NULL}};
const Segment SGI53669[] = {{&S102477[113], 3}, {NULL}};
const Segment SGI53670[] = {{&S102477[79], 5}, {NULL}};
const Segment SGI53671[] = {{&S102477[137], 13}, {NULL}};
const Segment SGI53672[] = {{&S102479[31], 5}, {NULL}};
const Segment SGI53673[] = {{&S102479[51], 3}, {NULL}};
const Segment SGI53674[] = {{&S86829[44], 12}, {NULL}};
const Segment SGI53675[] = {{&S86829[44], 33}, {NULL}};
const Segment SGI53676[] = {{&S86829[30], 6}, {NULL}};
const Segment SGI53677[] = {{&S86829[125], 3}, {NULL}};
const Segment SGI53678[] = {{&S86829[91], 5}, {NULL}};
const Segment SGI53679[] = {{&S87140[44], 7}, {NULL}};
const Segment SGI53680[] = {{&S87140[44], 25}, {NULL}};
const Segment SGI53681[] = {{&S87140[30], 6}, {NULL}};
const Segment SGI53682[] = {{&S87140[83], 6}, {NULL}};
const Segment SGI53683[] = {{&S87140[110], 13}, {NULL}};
const Segment SGI53684[] = {{&S87198[44], 8}, {NULL}};
const Segment SGI53685[] = {{&S87198[44], 29}, {NULL}};
const Segment SGI53686[] = {{&S87198[30], 6}, {NULL}};
const Segment SGI53687[] = {{&S87198[128], 3}, {NULL}};
const Segment SGI53688[] = {{&S87198[94], 5}, {NULL}};
const Segment SGI53689[] = {{&S87198[73], 8}, {NULL}};
const Segment SGI53690[] = {{&S87199[44], 8}, {NULL}};
const Segment SGI53691[] = {{&S87199[44], 29}, {NULL}};
const Segment SGI53692[] = {{&S87199[30], 6}, {NULL}};
const Segment SGI53693[] = {{&S87199[121], 3}, {NULL}};
const Segment SGI53694[] = {{&S87199[87], 5}, {NULL}};
const Segment SGI53695[] = {{&S87199[145], 7}, {NULL}};
const Segment SGI53696[] = {{&S87210[45], 7}, {NULL}};
const Segment SGI53697[] = {{&S87210[30], 6}, {NULL}};
const Segment SGI53698[] = {{&S87210[58], 3}, {NULL}};
const Segment SGI53699[] = {{&S87210[84], 5}, {NULL}};
const Segment SGI53700[] = {{&S87210[142], 13}, {NULL}};
const Segment SGI53701[] = {{&S87221[45], 7}, {NULL}};
const Segment SGI53702[] = {{&S87221[30], 6}, {NULL}};
const Segment SGI53703[] = {{&S87221[58], 3}, {NULL}};
const Segment SGI53704[] = {{&S87221[84], 5}, {NULL}};
const Segment SGI53705[] = {{&S87221[142], 13}, {NULL}};
const Segment SGI53706[] = {{&S89758[59], 14}, {NULL}};
const Segment SGI53707[] = {{&S89758[59], 25}, {NULL}};
const Segment SGI53708[] = {{&S89758[45], 6}, {NULL}};
const Segment SGI53709[] = {{&S89758[132], 3}, {NULL}};
const Segment SGI53710[] = {{&S89758[98], 5}, {NULL}};
const Segment SGI53711[] = {{&S89758[0], 14}, {NULL}};
const Segment SGI53712[] = {{&S91866[44], 12}, {NULL}};
const Segment SGI53713[] = {{&S91866[44], 24}, {NULL}};
const Segment SGI53714[] = {{&S91866[30], 6}, {NULL}};
const Segment SGI53715[] = {{&S91866[116], 3}, {NULL}};
const Segment SGI53716[] = {{&S91866[82], 5}, {NULL}};
const Segment SGI53717[] = {{&S86955[59], 10}, {NULL}};
const Segment SGI53718[] = {{&S86955[59], 27}, {NULL}};
const Segment SGI53719[] = {{&S86955[45], 6}, {NULL}};
const Segment SGI53720[] = {{&S86955[134], 3}, {NULL}};
const Segment SGI53721[] = {{&S86955[100], 5}, {NULL}};
const Segment SGI53722[] = {{&S86955[0], 14}, {NULL}};
const Segment SGI53723[] = {{&S87384[44], 7}, {NULL}};
const Segment SGI53724[] = {{&S87384[44], 25}, {NULL}};
const Segment SGI53725[] = {{&S87384[30], 6}, {NULL}};
const Segment SGI53726[] = {{&S87384[83], 6}, {NULL}};
const Segment SGI53727[] = {{&S87384[110], 13}, {NULL}};
const Segment SGI53728[] = {{&S87068[44], 9}, {NULL}};
const Segment SGI53729[] = {{&S87068[44], 20}, {NULL}};
const Segment SGI53730[] = {{&S87068[30], 6}, {NULL}};
const Segment SGI53731[] = {{&S87068[112], 3}, {NULL}};
const Segment SGI53732[] = {{&S87068[78], 5}, {NULL}};
const Segment SGI53733[] = {{&S102877[45], 10}, {NULL}};
const Segment SGI53734[] = {{&S102877[30], 6}, {NULL}};
const Segment SGI53735[] = {{&S102877[114], 3}, {NULL}};
const Segment SGI53736[] = {{&S102877[80], 5}, {NULL}};
const Segment SGI53737[] = {{&S102877[138], 13}, {NULL}};
const Segment SGI53738[] = {{&S87114[45], 14}, {NULL}};
const Segment SGI53739[] = {{&S87114[30], 6}, {NULL}};
const Segment SGI53740[] = {{&S87114[118], 3}, {NULL}};
const Segment SGI53741[] = {{&S87114[84], 5}, {NULL}};
const Segment SGI53742[] = {{&S87114[142], 13}, {NULL}};
const Segment SGI53743[] = {{&S87139[48], 6}, {NULL}};
const Segment SGI53744[] = {{&S87139[154], 3}, {NULL}};
const Segment SGI53745[] = {{&S87139[121], 5}, {NULL}};
const Segment SGI53746[] = {{&S87560[44], 15}, {NULL}};
const Segment SGI53747[] = {{&S87560[44], 26}, {NULL}};
const Segment SGI53748[] = {{&S87560[30], 6}, {NULL}};
const Segment SGI53749[] = {{&S87560[118], 3}, {NULL}};
const Segment SGI53750[] = {{&S87560[84], 5}, {NULL}};
const Segment SGI53751[] = {{&S87218[44], 5}, {NULL}};
const Segment SGI53752[] = {{&S87218[44], 22}, {NULL}};
const Segment SGI53753[] = {{&S87218[30], 6}, {NULL}};
const Segment SGI53754[] = {{&S87218[114], 3}, {NULL}};
const Segment SGI53755[] = {{&S87218[80], 5}, {NULL}};
const Segment SGI53756[] = {{&S95728[42], 19}, {NULL}};
const Segment SGI53757[] = {{&S95728[42], 36}, {NULL}};
const Segment SGI53758[] = {{&S95728[30], 4}, {NULL}};
const Segment SGI53759[] = {{&S95728[126], 3}, {NULL}};
const Segment SGI53760[] = {{&S95728[92], 5}, {NULL}};
const Segment SGI53761[] = {{&S87273[44], 10}, {NULL}};
const Segment SGI53762[] = {{&S87273[44], 22}, {NULL}};
const Segment SGI53763[] = {{&S87273[30], 6}, {NULL}};
const Segment SGI53764[] = {{&S87273[80], 6}, {NULL}};
const Segment SGI53765[] = {{&S87273[107], 13}, {NULL}};
const Segment SGI53766[] = {{&S103108[44], 12}, {NULL}};
const Segment SGI53767[] = {{&S103108[44], 24}, {NULL}};
const Segment SGI53768[] = {{&S103108[30], 6}, {NULL}};
const Segment SGI53769[] = {{&S103108[123], 3}, {NULL}};
const Segment SGI53770[] = {{&S103108[89], 5}, {NULL}};
const Segment SGI53771[] = {{&S90148[44], 10}, {NULL}};
const Segment SGI53772[] = {{&S90148[44], 27}, {NULL}};
const Segment SGI53773[] = {{&S90148[30], 6}, {NULL}};
const Segment SGI53774[] = {{&S90148[85], 6}, {NULL}};
const Segment SGI53775[] = {{&S90148[122], 3}, {NULL}};
const Segment SGI53776[] = {{&S90152[52], 10}, {NULL}};
const Segment SGI53777[] = {{&S90152[52], 27}, {NULL}};
const Segment SGI53778[] = {{&S90152[38], 6}, {NULL}};
const Segment SGI53779[] = {{&S90152[93], 5}, {NULL}};
const Segment SGI53780[] = {{&S90152[0], 12}, {NULL}};
const Segment SGI53781[] = {{&S87797[44], 8}, {NULL}};
const Segment SGI53782[] = {{&S87797[44], 18}, {NULL}};
const Segment SGI53783[] = {{&S87797[30], 6}, {NULL}};
const Segment SGI53784[] = {{&S87797[110], 3}, {NULL}};
const Segment SGI53785[] = {{&S87797[76], 5}, {NULL}};
const Segment SGI53786[] = {{&S87312[44], 10}, {NULL}};
const Segment SGI53787[] = {{&S87312[44], 21}, {NULL}};
const Segment SGI53788[] = {{&S87312[30], 6}, {NULL}};
const Segment SGI53789[] = {{&S87312[113], 3}, {NULL}};
const Segment SGI53790[] = {{&S87312[79], 5}, {NULL}};
const Segment SGI53791[] = {{&S90245[59], 18}, {NULL}};
const Segment SGI53792[] = {{&S90245[59], 29}, {NULL}};
const Segment SGI53793[] = {{&S90245[45], 6}, {NULL}};
const Segment SGI53794[] = {{&S90245[136], 3}, {NULL}};
const Segment SGI53795[] = {{&S90245[102], 5}, {NULL}};
const Segment SGI53796[] = {{&S90245[0], 14}, {NULL}};
const Segment SGI53797[] = {{&S103338[43], 38}, {NULL}};
const Segment SGI53798[] = {{&S103338[30], 6}, {NULL}};
const Segment SGI53799[] = {{&S103338[147], 3}, {NULL}};
const Segment SGI53800[] = {{&S103338[113], 5}, {NULL}};
const Segment SGI53801[] = {{&S87399[44], 21}, {NULL}};
const Segment SGI53802[] = {{&S87399[44], 32}, {NULL}};
const Segment SGI53803[] = {{&S87399[30], 6}, {NULL}};
const Segment SGI53804[] = {{&S87399[90], 6}, {NULL}};
const Segment SGI53805[] = {{&S87399[117], 13}, {NULL}};
const Segment SGI53806[] = {{&S103390[59], 12}, {NULL}};
const Segment SGI53807[] = {{&S103390[59], 23}, {NULL}};
const Segment SGI53808[] = {{&S103390[45], 6}, {NULL}};
const Segment SGI53809[] = {{&S103390[130], 3}, {NULL}};
const Segment SGI53810[] = {{&S103390[96], 5}, {NULL}};
const Segment SGI53811[] = {{&S103390[0], 14}, {NULL}};
const Segment SGI53812[] = {{&S87489[34], 14}, {NULL}};
const Segment SGI53813[] = {{&S87489[27], 6}, {NULL}};
const Segment SGI53814[] = {{&S87489[107], 2}, {NULL}};
const Segment SGI53815[] = {{&S87489[73], 6}, {NULL}};
const Segment SGI53816[] = {{&S87489[141], 8}, {NULL}};
const Segment SGI53817[] = {{&S88082[44], 8}, {NULL}};
const Segment SGI53818[] = {{&S88082[44], 29}, {NULL}};
const Segment SGI53819[] = {{&S88082[30], 6}, {NULL}};
const Segment SGI53820[] = {{&S88082[121], 3}, {NULL}};
const Segment SGI53821[] = {{&S88082[87], 5}, {NULL}};
const Segment SGI53822[] = {{&S88082[145], 11}, {NULL}};
const Segment SGI53823[] = {{&S88088[44], 14}, {NULL}};
const Segment SGI53824[] = {{&S88088[44], 37}, {NULL}};
const Segment SGI53825[] = {{&S88088[30], 6}, {NULL}};
const Segment SGI53826[] = {{&S88088[129], 3}, {NULL}};
const Segment SGI53827[] = {{&S88088[95], 5}, {NULL}};
const Segment SGI53828[] = {{&S88088[163], 3}, {NULL}};
const Segment SGI53829[] = {{&S103674[30], 6}, {NULL}};
const Segment SGI53830[] = {{&S103674[91], 3}, {NULL}};
const Segment SGI53831[] = {{&S103674[125], 3}, {NULL}};
const Segment SGI53832[] = {{&S103674[57], 5}, {NULL}};
const Segment SGI53833[] = {{&S103707[44], 4}, {NULL}};
const Segment SGI53834[] = {{&S103707[44], 16}, {NULL}};
const Segment SGI53835[] = {{&S103707[30], 6}, {NULL}};
const Segment SGI53836[] = {{&S103707[108], 3}, {NULL}};
const Segment SGI53837[] = {{&S103707[74], 5}, {NULL}};
const Segment SGI53838[] = {{&S88199[44], 10}, {NULL}};
const Segment SGI53839[] = {{&S88199[44], 27}, {NULL}};
const Segment SGI53840[] = {{&S88199[30], 6}, {NULL}};
const Segment SGI53841[] = {{&S88199[119], 3}, {NULL}};
const Segment SGI53842[] = {{&S88199[85], 5}, {NULL}};
const Segment SGI53843[] = {{&S97306[44], 10}, {NULL}};
const Segment SGI53844[] = {{&S97306[44], 27}, {NULL}};
const Segment SGI53845[] = {{&S97306[30], 6}, {NULL}};
const Segment SGI53846[] = {{&S97306[119], 3}, {NULL}};
const Segment SGI53847[] = {{&S97306[85], 5}, {NULL}};
const Segment SGI53848[] = {{&S87600[44], 10}, {NULL}};
const Segment SGI53849[] = {{&S87600[44], 27}, {NULL}};
const Segment SGI53850[] = {{&S87600[30], 6}, {NULL}};
const Segment SGI53851[] = {{&S87600[119], 3}, {NULL}};
const Segment SGI53852[] = {{&S87600[85], 5}, {NULL}};
const Segment SGI53853[] = {{&S87632[52], 7}, {NULL}};
const Segment SGI53854[] = {{&S87632[52], 32}, {NULL}};
const Segment SGI53855[] = {{&S87632[38], 6}, {NULL}};
const Segment SGI53856[] = {{&S87632[98], 5}, {NULL}};
const Segment SGI53857[] = {{&S87632[0], 14}, {NULL}};
const Segment SGI53858[] = {{&S87633[52], 20}, {NULL}};
const Segment SGI53859[] = {{&S87633[52], 45}, {NULL}};
const Segment SGI53860[] = {{&S87633[38], 6}, {NULL}};
const Segment SGI53861[] = {{&S87633[111], 5}, {NULL}};
const Segment SGI53862[] = {{&S87633[0], 14}, {NULL}};
const Segment SGI53863[] = {{&S90750[39], 7}, {NULL}};
const Segment SGI53864[] = {{&S90750[102], 3}, {NULL}};
const Segment SGI53865[] = {{&S90750[91], 3}, {NULL}};
const Segment SGI53866[] = {{&S90750[68], 6}, {NULL}};
const Segment SGI53867[] = {{&S90751[31], 7}, {NULL}};
const Segment SGI53868[] = {{&S90751[53], 3}, {NULL}};
const Segment SGI53869[] = {{&S103884[44], 7}, {NULL}};
const Segment SGI53870[] = {{&S103884[44], 28}, {NULL}};
const Segment SGI53871[] = {{&S103884[30], 6}, {NULL}};
const Segment SGI53872[] = {{&S103884[120], 3}, {NULL}};
const Segment SGI53873[] = {{&S103884[86], 5}, {NULL}};
const Segment SGI53874[] = {{&S90771[6], 4}, {NULL}};
const Segment SGI53875[] = {{&S90771[87], 5}, {NULL}};
const Segment SGI53876[] = {{&S90784[45], 6}, {NULL}};
const Segment SGI53877[] = {{&S90784[30], 6}, {NULL}};
const Segment SGI53878[] = {{&S90784[117], 3}, {NULL}};
const Segment SGI53879[] = {{&S90784[83], 5}, {NULL}};
const Segment SGI53880[] = {{&S90784[151], 3}, {NULL}};
const Segment SGI53881[] = {{&S90787[45], 10}, {NULL}};
const Segment SGI53882[] = {{&S90787[30], 6}, {NULL}};
const Segment SGI53883[] = {{&S90787[120], 3}, {NULL}};
const Segment SGI53884[] = {{&S90787[86], 5}, {NULL}};
const Segment SGI53885[] = {{&S90787[154], 3}, {NULL}};
const Segment SGI53886[] = {{&S87716[44], 18}, {NULL}};
const Segment SGI53887[] = {{&S87716[44], 29}, {NULL}};
const Segment SGI53888[] = {{&S87716[30], 6}, {NULL}};
const Segment SGI53889[] = {{&S87716[87], 6}, {NULL}};
const Segment SGI53890[] = {{&S87716[114], 13}, {NULL}};
const Segment SGI53891[] = {{&S88257[44], 7}, {NULL}};
const Segment SGI53892[] = {{&S88257[44], 25}, {NULL}};
const Segment SGI53893[] = {{&S88257[30], 6}, {NULL}};
const Segment SGI53894[] = {{&S88257[83], 6}, {NULL}};
const Segment SGI53895[] = {{&S88257[110], 13}, {NULL}};
const Segment SGI53896[] = {{&S87725[44], 12}, {NULL}};
const Segment SGI53897[] = {{&S87725[44], 23}, {NULL}};
const Segment SGI53898[] = {{&S87725[30], 6}, {NULL}};
const Segment SGI53899[] = {{&S87725[115], 3}, {NULL}};
const Segment SGI53900[] = {{&S87725[81], 5}, {NULL}};
const Segment SGI53901[] = {{&S90835[59], 10}, {NULL}};
const Segment SGI53902[] = {{&S90835[59], 27}, {NULL}};
const Segment SGI53903[] = {{&S90835[45], 6}, {NULL}};
const Segment SGI53904[] = {{&S90835[134], 3}, {NULL}};
const Segment SGI53905[] = {{&S90835[100], 5}, {NULL}};
const Segment SGI53906[] = {{&S90835[0], 12}, {NULL}};
const Segment SGI53907[] = {{&S90836[57], 10}, {NULL}};
const Segment SGI53908[] = {{&S90836[45], 6}, {NULL}};
const Segment SGI53909[] = {{&S90836[132], 3}, {NULL}};
const Segment SGI53910[] = {{&S90836[98], 5}, {NULL}};
const Segment SGI53911[] = {{&S90836[0], 12}, {NULL}};
const Segment SGI53912[] = {{&S87750[44], 10}, {NULL}};
const Segment SGI53913[] = {{&S87750[44], 22}, {NULL}};
const Segment SGI53914[] = {{&S96642[42], 11}, {NULL}};
const Segment SGI53915[] = {{&S87750[30], 6}, {NULL}};
const Segment SGI53916[] = {{&S96642[42], 22}, {NULL}};
const Segment SGI53917[] = {{&S96642[30], 4}, {NULL}};
const Segment SGI53918[] = {{&S96642[112], 3}, {NULL}};
const Segment SGI53919[] = {{&S87750[80], 6}, {NULL}};
const Segment SGI53920[] = {{&S96642[78], 5}, {NULL}};
const Segment SGI53921[] = {{&S87750[107], 13}, {NULL}};
const Segment SGI53922[] = {{&S88298[44], 12}, {NULL}};
const Segment SGI53923[] = {{&S88298[44], 30}, {NULL}};
const Segment SGI53924[] = {{&S88298[30], 6}, {NULL}};
const Segment SGI53925[] = {{&S88298[122], 3}, {NULL}};
const Segment SGI53926[] = {{&S88298[88], 5}, {NULL}};
const Segment SGI53927[] = {{&S87796[44], 8}, {NULL}};
const Segment SGI53928[] = {{&S87796[44], 18}, {NULL}};
const Segment SGI53929[] = {{&S87796[30], 6}, {NULL}};
const Segment SGI53930[] = {{&S87796[110], 3}, {NULL}};
const Segment SGI53931[] = {{&S87796[76], 5}, {NULL}};
const Segment SGI53932[] = {{&S88341[44], 9}, {NULL}};
const Segment SGI53933[] = {{&S88341[44], 20}, {NULL}};
const Segment SGI53934[] = {{&S88341[30], 6}, {NULL}};
const Segment SGI53935[] = {{&S88341[112], 3}, {NULL}};
const Segment SGI53936[] = {{&S88341[78], 5}, {NULL}};
const Segment SGI53937[] = {{&S88341[136], 11}, {NULL}};
const Segment SGI53938[] = {{&S87818[44], 9}, {NULL}};
const Segment SGI53939[] = {{&S87818[44], 27}, {NULL}};
const Segment SGI53940[] = {{&S87818[30], 6}, {NULL}};
const Segment SGI53941[] = {{&S87818[119], 3}, {NULL}};
const Segment SGI53942[] = {{&S87818[85], 5}, {NULL}};
const Segment SGI53943[] = {{&S87819[44], 11}, {NULL}};
const Segment SGI53944[] = {{&S87819[44], 22}, {NULL}};
const Segment SGI53945[] = {{&S87819[30], 6}, {NULL}};
const Segment SGI53946[] = {{&S87819[114], 3}, {NULL}};
const Segment SGI53947[] = {{&S87819[80], 5}, {NULL}};
const Segment SGI53948[] = {{&S88380[44], 7}, {NULL}};
const Segment SGI53949[] = {{&S88380[44], 25}, {NULL}};
const Segment SGI53950[] = {{&S88380[30], 6}, {NULL}};
const Segment SGI53951[] = {{&S104005[59], 12}, {NULL}};
const Segment SGI53952[] = {{&S104005[59], 23}, {NULL}};
const Segment SGI53953[] = {{&S104005[45], 6}, {NULL}};
const Segment SGI53954[] = {{&S88380[83], 6}, {NULL}};
const Segment SGI53955[] = {{&S104005[130], 3}, {NULL}};
const Segment SGI53956[] = {{&S88380[110], 13}, {NULL}};
const Segment SGI53957[] = {{&S104005[96], 5}, {NULL}};
const Segment SGI53958[] = {{&S104005[0], 14}, {NULL}};
const Segment SGI53959[] = {{&S88422[44], 16}, {NULL}};
const Segment SGI53960[] = {{&S88422[44], 27}, {NULL}};
const Segment SGI53961[] = {{&S88422[30], 6}, {NULL}};
const Segment SGI53962[] = {{&S88422[119], 3}, {NULL}};
const Segment SGI53963[] = {{&S88422[85], 5}, {NULL}};
const Segment SGI53964[] = {{&S87916[44], 7}, {NULL}};
const Segment SGI53965[] = {{&S87916[44], 36}, {NULL}};
const Segment SGI53966[] = {{&S87916[30], 6}, {NULL}};
const Segment SGI53967[] = {{&S87916[128], 3}, {NULL}};
const Segment SGI53968[] = {{&S87916[94], 5}, {NULL}};
const Segment SGI53969[] = {{&S87919[44], 7}, {NULL}};
const Segment SGI53970[] = {{&S87919[44], 36}, {NULL}};
const Segment SGI53971[] = {{&S87919[30], 6}, {NULL}};
const Segment SGI53972[] = {{&S87919[128], 3}, {NULL}};
const Segment SGI53973[] = {{&S87919[94], 5}, {NULL}};
const Segment SGI53974[] = {{&S88480[44], 10}, {NULL}};
const Segment SGI53975[] = {{&S88480[44], 27}, {NULL}};
const Segment SGI53976[] = {{&S88480[30], 6}, {NULL}};
const Segment SGI53977[] = {{&S88480[85], 6}, {NULL}};
const Segment SGI53978[] = {{&S88480[112], 13}, {NULL}};
const Segment SGI53979[] = {{&S87941[44], 6}, {NULL}};
const Segment SGI53980[] = {{&S87941[44], 17}, {NULL}};
const Segment SGI53981[] = {{&S87941[30], 6}, {NULL}};
const Segment SGI53982[] = {{&S87941[109], 3}, {NULL}};
const Segment SGI53983[] = {{&S87941[75], 5}, {NULL}};
const Segment SGI53984[] = {{&S87970[44], 6}, {NULL}};
const Segment SGI53985[] = {{&S87970[44], 23}, {NULL}};
const Segment SGI53986[] = {{&S87970[30], 6}, {NULL}};
const Segment SGI53987[] = {{&S87970[115], 3}, {NULL}};
const Segment SGI53988[] = {{&S87970[81], 5}, {NULL}};
const Segment SGI53989[] = {{&S87970[139], 11}, {NULL}};
const Segment SGI53990[] = {{&S88009[52], 20}, {NULL}};
const Segment SGI53991[] = {{&S88009[52], 31}, {NULL}};
const Segment SGI53992[] = {{&S88009[38], 6}, {NULL}};
const Segment SGI53993[] = {{&S88009[97], 5}, {NULL}};
const Segment SGI53994[] = {{&S88009[0], 14}, {NULL}};
const Segment SGI53995[] = {{&S88580[31], 5}, {NULL}};
const Segment SGI53996[] = {{&S88580[51], 3}, {NULL}};
const Segment SGI53997[] = {{&S104182[44], 9}, {NULL}};
const Segment SGI53998[] = {{&S104182[44], 21}, {NULL}};
const Segment SGI53999[] = {{&S104182[30], 6}, {NULL}};
const Segment SGI54000[] = {{&S97789[45], 10}, {NULL}};
const Segment SGI54001[] = {{&S88600[44], 5}, {NULL}};
const Segment SGI54002[] = {{&S104182[113], 3}, {NULL}};
const Segment SGI54003[] = {{&S88600[44], 16}, {NULL}};
const Segment SGI54004[] = {{&S97789[30], 6}, {NULL}};
const Segment SGI54005[] = {{&S88600[30], 6}, {NULL}};
const Segment SGI54006[] = {{&S104182[79], 5}, {NULL}};
const Segment SGI54007[] = {{&S88600[108], 3}, {NULL}};
const Segment SGI54008[] = {{&S97789[120], 3}, {NULL}};
const Segment SGI54009[] = {{&S88600[74], 5}, {NULL}};
const Segment SGI54010[] = {{&S97789[86], 5}, {NULL}};
const Segment SGI54011[] = {{&S104183[44], 9}, {NULL}};
const Segment SGI54012[] = {{&S104183[44], 21}, {NULL}};
const Segment SGI54013[] = {{&S104183[30], 6}, {NULL}};
const Segment SGI54014[] = {{&S104183[113], 3}, {NULL}};
const Segment SGI54015[] = {{&S104183[79], 5}, {NULL}};
const Segment SGI54016[] = {{&S88619[45], 9}, {NULL}};
const Segment SGI54017[] = {{&S88619[30], 6}, {NULL}};
const Segment SGI54018[] = {{&S88619[113], 3}, {NULL}};
const Segment SGI54019[] = {{&S88619[79], 5}, {NULL}};
const Segment SGI54020[] = {{&S88619[137], 13}, {NULL}};
const Segment SGI54021[] = {{&S88625[44], 13}, {NULL}};
const Segment SGI54022[] = {{&S88625[44], 24}, {NULL}};
const Segment SGI54023[] = {{&S88625[30], 6}, {NULL}};
const Segment SGI54024[] = {{&S88625[116], 3}, {NULL}};
const Segment SGI54025[] = {{&S88625[82], 5}, {NULL}};
const Segment SGI54026[] = {{&S88625[140], 11}, {NULL}};
const Segment SGI54027[] = {{&S104224[44], 7}, {NULL}};
const Segment SGI54028[] = {{&S104224[44], 22}, {NULL}};
const Segment SGI54029[] = {{&S104224[30], 6}, {NULL}};
const Segment SGI54030[] = {{&S104224[158], 3}, {NULL}};
const Segment SGI54031[] = {{&S104224[124], 3}, {NULL}};
const Segment SGI54032[] = {{&S104224[89], 6}, {NULL}};
const Segment SGI54033[] = {{&S104224[144], 8}, {NULL}};
const Segment SGI54034[] = {{&S88128[44], 10}, {NULL}};
const Segment SGI54035[] = {{&S88128[44], 27}, {NULL}};
const Segment SGI54036[] = {{&S88128[30], 6}, {NULL}};
const Segment SGI54037[] = {{&S88128[118], 3}, {NULL}};
const Segment SGI54038[] = {{&S88128[85], 5}, {NULL}};
const Segment SGI54039[] = {{&S91132[45], 22}, {NULL}};
const Segment SGI54040[] = {{&S88128[122], 14}, {NULL}};
const Segment SGI54041[] = {{&S91132[30], 6}, {NULL}};
const Segment SGI54042[] = {{&S91132[126], 3}, {NULL}};
const Segment SGI54043[] = {{&S91132[92], 5}, {NULL}};
const Segment SGI54044[] = {{&S91132[160], 3}, {NULL}};
const Segment SGI54045[] = {{&S88130[44], 14}, {NULL}};
const Segment SGI54046[] = {{&S88130[44], 35}, {NULL}};
const Segment SGI54047[] = {{&S88130[30], 6}, {NULL}};
const Segment SGI54048[] = {{&S88130[93], 6}, {NULL}};
const Segment SGI54049[] = {{&S88130[120], 13}, {NULL}};
const Segment SGI54050[] = {{&S104354[44], 12}, {NULL}};
const Segment SGI54051[] = {{&S104354[44], 23}, {NULL}};
const Segment SGI54052[] = {{&S104354[30], 6}, {NULL}};
const Segment SGI54053[] = {{&S104354[122], 3}, {NULL}};
const Segment SGI54054[] = {{&S104354[88], 5}, {NULL}};
const Segment SGI54055[] = {{&S88765[34], 8}, {NULL}};
const Segment SGI54056[] = {{&S88765[27], 6}, {NULL}};
const Segment SGI54057[] = {{&S88765[111], 2}, {NULL}};
const Segment SGI54058[] = {{&S88765[77], 6}, {NULL}};
const Segment SGI54059[] = {{&S88765[145], 8}, {NULL}};
const Segment SGI54060[] = {{&S88776[44], 13}, {NULL}};
const Segment SGI54061[] = {{&S88776[44], 24}, {NULL}};
const Segment SGI54062[] = {{&S88776[30], 6}, {NULL}};
const Segment SGI54063[] = {{&S88776[82], 6}, {NULL}};
const Segment SGI54064[] = {{&S88776[109], 13}, {NULL}};
const Segment SGI54065[] = {{&S104379[45], 9}, {NULL}};
const Segment SGI54066[] = {{&S104379[30], 6}, {NULL}};
const Segment SGI54067[] = {{&S104379[114], 3}, {NULL}};
const Segment SGI54068[] = {{&S104379[80], 5}, {NULL}};
const Segment SGI54069[] = {{&S104379[138], 13}, {NULL}};
const Segment SGI54070[] = {{&S104383[45], 9}, {NULL}};
const Segment SGI54071[] = {{&S104383[30], 6}, {NULL}};
const Segment SGI54072[] = {{&S104383[114], 3}, {NULL}};
const Segment SGI54073[] = {{&S104383[80], 5}, {NULL}};
const Segment SGI54074[] = {{&S104383[138], 13}, {NULL}};
const Segment SGI54075[] = {{&S104387[45], 11}, {NULL}};
const Segment SGI54076[] = {{&S104387[30], 6}, {NULL}};
const Segment SGI54077[] = {{&S104387[115], 3}, {NULL}};
const Segment SGI54078[] = {{&S104387[81], 5}, {NULL}};
const Segment SGI54079[] = {{&S104387[139], 13}, {NULL}};
const Segment SGI54080[] = {{&S88270[44], 14}, {NULL}};
const Segment SGI54081[] = {{&S88270[44], 35}, {NULL}};
const Segment SGI54082[] = {{&S88270[30], 6}, {NULL}};
const Segment SGI54083[] = {{&S88270[128], 3}, {NULL}};
const Segment SGI54084[] = {{&S88270[93], 6}, {NULL}};
const Segment SGI54085[] = {{&S88270[148], 8}, {NULL}};
const Segment SGI54086[] = {{&S88270[157], 13}, {NULL}};
const Segment SGI54087[] = {{&S88295[44], 12}, {NULL}};
const Segment SGI54088[] = {{&S88295[44], 22}, {NULL}};
const Segment SGI54089[] = {{&S88295[30], 6}, {NULL}};
const Segment SGI54090[] = {{&S88295[114], 3}, {NULL}};
const Segment SGI54091[] = {{&S88295[80], 5}, {NULL}};
const Segment SGI54092[] = {{&S88296[44], 12}, {NULL}};
const Segment SGI54093[] = {{&S88296[44], 30}, {NULL}};
const Segment SGI54094[] = {{&S88296[30], 6}, {NULL}};
const Segment SGI54095[] = {{&S88296[122], 3}, {NULL}};
const Segment SGI54096[] = {{&S88296[88], 5}, {NULL}};
const Segment SGI54097[] = {{&S88329[59], 22}, {NULL}};
const Segment SGI54098[] = {{&S88329[59], 33}, {NULL}};
const Segment SGI54099[] = {{&S88329[45], 6}, {NULL}};
const Segment SGI54100[] = {{&S88329[140], 3}, {NULL}};
const Segment SGI54101[] = {{&S88329[106], 5}, {NULL}};
const Segment SGI54102[] = {{&S88329[0], 14}, {NULL}};
const Segment SGI54103[] = {{&S88421[44], 16}, {NULL}};
const Segment SGI54104[] = {{&S88421[44], 27}, {NULL}};
const Segment SGI54105[] = {{&S88421[30], 6}, {NULL}};
const Segment SGI54106[] = {{&S88421[119], 3}, {NULL}};
const Segment SGI54107[] = {{&S88421[85], 5}, {NULL}};
const Segment SGI54108[] = {{&S88426[44], 16}, {NULL}};
const Segment SGI54109[] = {{&S88426[44], 27}, {NULL}};
const Segment SGI54110[] = {{&S88426[30], 6}, {NULL}};
const Segment SGI54111[] = {{&S88426[119], 3}, {NULL}};
const Segment SGI54112[] = {{&S88426[85], 5}, {NULL}};
const Segment SGI54113[] = {{&S91495[31], 12}, {NULL}};
const Segment SGI54114[] = {{&S91495[58], 3}, {NULL}};
const Segment SGI54115[] = {{&S89079[44], 10}, {NULL}};
const Segment SGI54116[] = {{&S89079[44], 27}, {NULL}};
const Segment SGI54117[] = {{&S89079[30], 6}, {NULL}};
const Segment SGI54118[] = {{&S89079[85], 6}, {NULL}};
const Segment SGI54119[] = {{&S89079[112], 13}, {NULL}};
const Segment SGI54120[] = {{&S91517[31], 16}, {NULL}};
const Segment SGI54121[] = {{&S91517[62], 3}, {NULL}};
const Segment SGI54122[] = {{&S89118[44], 17}, {NULL}};
const Segment SGI54123[] = {{&S89118[44], 28}, {NULL}};
const Segment SGI54124[] = {{&S89118[30], 6}, {NULL}};
const Segment SGI54125[] = {{&S89118[120], 3}, {NULL}};
const Segment SGI54126[] = {{&S89118[86], 5}, {NULL}};
const Segment SGI54127[] = {{&S89134[45], 7}, {NULL}};
const Segment SGI54128[] = {{&S89134[30], 6}, {NULL}};
const Segment SGI54129[] = {{&S89134[58], 3}, {NULL}};
const Segment SGI54130[] = {{&S89134[84], 5}, {NULL}};
const Segment SGI54131[] = {{&S89134[142], 13}, {NULL}};
const Segment SGI54132[] = {{&S88567[44], 22}, {NULL}};
const Segment SGI54133[] = {{&S88567[44], 33}, {NULL}};
const Segment SGI54134[] = {{&S88567[30], 6}, {NULL}};
const Segment SGI54135[] = {{&S88567[91], 6}, {NULL}};
const Segment SGI54136[] = {{&S88567[118], 13}, {NULL}};
const Segment SGI54137[] = {{&S89194[44], 9}, {NULL}};
const Segment SGI54138[] = {{&S89194[44], 20}, {NULL}};
const Segment SGI54139[] = {{&S89194[30], 6}, {NULL}};
const Segment SGI54140[] = {{&S89194[78], 6}, {NULL}};
const Segment SGI54141[] = {{&S89194[105], 13}, {NULL}};
const Segment SGI54142[] = {{&S89236[44], 7}, {NULL}};
const Segment SGI54143[] = {{&S89236[44], 18}, {NULL}};
const Segment SGI54144[] = {{&S89236[30], 6}, {NULL}};
const Segment SGI54145[] = {{&S89236[110], 3}, {NULL}};
const Segment SGI54146[] = {{&S89236[76], 5}, {NULL}};
const Segment SGI54147[] = {{&S89237[44], 7}, {NULL}};
const Segment SGI54148[] = {{&S89237[44], 18}, {NULL}};
const Segment SGI54149[] = {{&S89237[30], 6}, {NULL}};
const Segment SGI54150[] = {{&S89237[79], 4}, {NULL}};
const Segment SGI54151[] = {{&S89237[79], 3}, {NULL}};
const Segment SGI54152[] = {{&S89238[38], 7}, {NULL}};
const Segment SGI54153[] = {{&S89238[31], 6}, {NULL}};
const Segment SGI54154[] = {{&S88703[44], 14}, {NULL}};
const Segment SGI54155[] = {{&S88703[44], 25}, {NULL}};
const Segment SGI54156[] = {{&S88703[30], 6}, {NULL}};
const Segment SGI54157[] = {{&S88703[117], 3}, {NULL}};
const Segment SGI54158[] = {{&S88703[83], 5}, {NULL}};
const Segment SGI54159[] = {{&S89302[45], 9}, {NULL}};
const Segment SGI54160[] = {{&S89302[30], 6}, {NULL}};
const Segment SGI54161[] = {{&S89302[113], 3}, {NULL}};
const Segment SGI54162[] = {{&S89302[79], 5}, {NULL}};
const Segment SGI54163[] = {{&S89302[137], 13}, {NULL}};
const Segment SGI54164[] = {{&S88719[44], 10}, {NULL}};
const Segment SGI54165[] = {{&S88719[44], 21}, {NULL}};
const Segment SGI54166[] = {{&S88719[30], 6}, {NULL}};
const Segment SGI54167[] = {{&S88719[113], 3}, {NULL}};
const Segment SGI54168[] = {{&S88719[79], 5}, {NULL}};
const Segment SGI54169[] = {{&S104832[44], 7}, {NULL}};
const Segment SGI54170[] = {{&S104832[44], 19}, {NULL}};
const Segment SGI54171[] = {{&S104832[30], 6}, {NULL}};
const Segment SGI54172[] = {{&S104832[118], 3}, {NULL}};
const Segment SGI54173[] = {{&S104832[84], 5}, {NULL}};
const Segment SGI54174[] = {{&S104882[59], 10}, {NULL}};
const Segment SGI54175[] = {{&S104882[59], 21}, {NULL}};
const Segment SGI54176[] = {{&S104882[45], 6}, {NULL}};
const Segment SGI54177[] = {{&S104882[128], 3}, {NULL}};
const Segment SGI54178[] = {{&S104882[94], 5}, {NULL}};
const Segment SGI54179[] = {{&S104882[0], 14}, {NULL}};
const Segment SGI54180[] = {{&S91867[44], 12}, {NULL}};
const Segment SGI54181[] = {{&S91867[44], 23}, {NULL}};
const Segment SGI54182[] = {{&S91867[30], 6}, {NULL}};
const Segment SGI54183[] = {{&S91867[115], 3}, {NULL}};
const Segment SGI54184[] = {{&S91867[81], 5}, {NULL}};
const Segment SGI54185[] = {{&S89432[44], 9}, {NULL}};
const Segment SGI54186[] = {{&S89432[44], 26}, {NULL}};
const Segment SGI54187[] = {{&S89432[30], 6}, {NULL}};
const Segment SGI54188[] = {{&S89432[118], 3}, {NULL}};
const Segment SGI54189[] = {{&S89432[84], 5}, {NULL}};
const Segment SGI54190[] = {{&S88844[58], 6}, {NULL}};
const Segment SGI54191[] = {{&S88844[58], 17}, {NULL}};
const Segment SGI54192[] = {{&S88844[44], 6}, {NULL}};
const Segment SGI54193[] = {{&S88844[123], 3}, {NULL}};
const Segment SGI54194[] = {{&S88844[89], 5}, {NULL}};
const Segment SGI54195[] = {{&S88852[45], 10}, {NULL}};
const Segment SGI54196[] = {{&S88852[30], 6}, {NULL}};
const Segment SGI54197[] = {{&S88852[120], 3}, {NULL}};
const Segment SGI54198[] = {{&S88852[86], 5}, {NULL}};
const Segment SGI54199[] = {{&S88852[144], 13}, {NULL}};
const Segment SGI54200[] = {{&S88853[44], 11}, {NULL}};
const Segment SGI54201[] = {{&S88853[44], 22}, {NULL}};
const Segment SGI54202[] = {{&S88853[30], 6}, {NULL}};
const Segment SGI54203[] = {{&S88853[114], 3}, {NULL}};
const Segment SGI54204[] = {{&S88853[80], 5}, {NULL}};
const Segment SGI54205[] = {{&S88898[44], 14}, {NULL}};
const Segment SGI54206[] = {{&S88898[44], 25}, {NULL}};
const Segment SGI54207[] = {{&S88898[30], 6}, {NULL}};
const Segment SGI54208[] = {{&S88898[83], 6}, {NULL}};
const Segment SGI54209[] = {{&S88898[110], 13}, {NULL}};
const Segment SGI54210[] = {{&S91956[59], 22}, {NULL}};
const Segment SGI54211[] = {{&S91956[59], 33}, {NULL}};
const Segment SGI54212[] = {{&S91956[45], 6}, {NULL}};
const Segment SGI54213[] = {{&S89509[44], 13}, {NULL}};
const Segment SGI54214[] = {{&S89509[44], 24}, {NULL}};
const Segment SGI54215[] = {{&S91956[140], 3}, {NULL}};
const Segment SGI54216[] = {{&S89509[30], 6}, {NULL}};
const Segment SGI54217[] = {{&S91956[106], 5}, {NULL}};
const Segment SGI54218[] = {{&S91956[0], 14}, {NULL}};
const Segment SGI54219[] = {{&S89509[82], 6}, {NULL}};
const Segment SGI54220[] = {{&S89509[109], 13}, {NULL}};
const Segment SGI54221[] = {{&S105004[44], 10}, {NULL}};
const Segment SGI54222[] = {{&S105004[44], 21}, {NULL}};
const Segment SGI54223[] = {{&S105004[30], 6}, {NULL}};
const Segment SGI54224[] = {{&S105004[112], 3}, {NULL}};
const Segment SGI54225[] = {{&S105004[79], 5}, {NULL}};
const Segment SGI54226[] = {{&S105004[116], 14}, {NULL}};
const Segment SGI54227[] = {{&S89014[59], 10}, {NULL}};
const Segment SGI54228[] = {{&S89014[59], 31}, {NULL}};
const Segment SGI54229[] = {{&S89014[45], 6}, {NULL}};
const Segment SGI54230[] = {{&S89014[138], 3}, {NULL}};
const Segment SGI54231[] = {{&S89014[104], 5}, {NULL}};
const Segment SGI54232[] = {{&S89014[0], 14}, {NULL}};
const Segment SGI54233[] = {{&S89031[44], 21}, {NULL}};
const Segment SGI54234[] = {{&S89031[44], 32}, {NULL}};
const Segment SGI54235[] = {{&S89031[30], 6}, {NULL}};
const Segment SGI54236[] = {{&S89031[90], 6}, {NULL}};
const Segment SGI54237[] = {{&S89031[117], 13}, {NULL}};
const Segment SGI54238[] = {{&S89720[44], 6}, {NULL}};
const Segment SGI54239[] = {{&S89720[44], 23}, {NULL}};
const Segment SGI54240[] = {{&S89720[30], 6}, {NULL}};
const Segment SGI54241[] = {{&S89720[115], 3}, {NULL}};
const Segment SGI54242[] = {{&S89720[81], 5}, {NULL}};
const Segment SGI54243[] = {{&S89737[45], 9}, {NULL}};
const Segment SGI54244[] = {{&S89737[30], 6}, {NULL}};
const Segment SGI54245[] = {{&S89737[113], 3}, {NULL}};
const Segment SGI54246[] = {{&S89737[79], 5}, {NULL}};
const Segment SGI54247[] = {{&S89737[137], 13}, {NULL}};
const Segment SGI54248[] = {{&S89150[59], 14}, {NULL}};
const Segment SGI54249[] = {{&S89150[59], 31}, {NULL}};
const Segment SGI54250[] = {{&S89150[45], 6}, {NULL}};
const Segment SGI54251[] = {{&S89150[138], 3}, {NULL}};
const Segment SGI54252[] = {{&S89150[104], 5}, {NULL}};
const Segment SGI54253[] = {{&S89150[0], 14}, {NULL}};
const Segment SGI54254[] = {{&S89188[44], 14}, {NULL}};
const Segment SGI54255[] = {{&S89188[44], 35}, {NULL}};
const Segment SGI54256[] = {{&S89188[30], 6}, {NULL}};
const Segment SGI54257[] = {{&S89188[93], 6}, {NULL}};
const Segment SGI54258[] = {{&S89188[120], 13}, {NULL}};
const Segment SGI54259[] = {{&S89802[44], 9}, {NULL}};
const Segment SGI54260[] = {{&S89802[44], 20}, {NULL}};
const Segment SGI54261[] = {{&S89802[30], 6}, {NULL}};
const Segment SGI54262[] = {{&S89802[112], 3}, {NULL}};
const Segment SGI54263[] = {{&S89802[78], 5}, {NULL}};
const Segment SGI54264[] = {{&S89802[136], 11}, {NULL}};
const Segment SGI54265[] = {{&S89198[44], 11}, {NULL}};
const Segment SGI54266[] = {{&S89198[44], 28}, {NULL}};
const Segment SGI54267[] = {{&S89198[30], 6}, {NULL}};
const Segment SGI54268[] = {{&S89198[86], 6}, {NULL}};
const Segment SGI54269[] = {{&S89198[113], 13}, {NULL}};
const Segment SGI54270[] = {{&S89818[44], 7}, {NULL}};
const Segment SGI54271[] = {{&S89818[44], 25}, {NULL}};
const Segment SGI54272[] = {{&S89818[30], 6}, {NULL}};
const Segment SGI54273[] = {{&S89818[83], 6}, {NULL}};
const Segment SGI54274[] = {{&S89818[110], 13}, {NULL}};
const Segment SGI54275[] = {{&S89220[44], 18}, {NULL}};
const Segment SGI54276[] = {{&S89220[44], 29}, {NULL}};
const Segment SGI54277[] = {{&S89220[30], 6}, {NULL}};
const Segment SGI54278[] = {{&S89220[87], 6}, {NULL}};
const Segment SGI54279[] = {{&S89220[114], 13}, {NULL}};
const Segment SGI54280[] = {{&S89257[43], 16}, {NULL}};
const Segment SGI54281[] = {{&S89257[30], 6}, {NULL}};
const Segment SGI54282[] = {{&S89257[107], 3}, {NULL}};
const Segment SGI54283[] = {{&S89257[73], 5}, {NULL}};
const Segment SGI54284[] = {{&S89258[44], 29}, {NULL}};
const Segment SGI54285[] = {{&S89258[30], 6}, {NULL}};
const Segment SGI54286[] = {{&S89258[121], 3}, {NULL}};
const Segment SGI54287[] = {{&S89258[87], 5}, {NULL}};
const Segment SGI54288[] = {{&S105216[58], 7}, {NULL}};
const Segment SGI54289[] = {{&S105216[58], 25}, {NULL}};
const Segment SGI54290[] = {{&S105216[44], 6}, {NULL}};
const Segment SGI54291[] = {{&S105216[131], 3}, {NULL}};
const Segment SGI54292[] = {{&S105216[97], 5}, {NULL}};
const Segment SGI54293[] = {{&S89331[34], 22}, {NULL}};
const Segment SGI54294[] = {{&S89331[27], 6}, {NULL}};
const Segment SGI54295[] = {{&S89331[115], 2}, {NULL}};
const Segment SGI54296[] = {{&S89331[81], 6}, {NULL}};
const Segment SGI54297[] = {{&S92329[45], 11}, {NULL}};
const Segment SGI54298[] = {{&S92329[30], 6}, {NULL}};
const Segment SGI54299[] = {{&S89331[149], 8}, {NULL}};
const Segment SGI54300[] = {{&S92329[115], 3}, {NULL}};
const Segment SGI54301[] = {{&S92329[81], 5}, {NULL}};
const Segment SGI54302[] = {{&S92329[149], 3}, {NULL}};
const Segment SGI54303[] = {{&S92343[44], 10}, {NULL}};
const Segment SGI54304[] = {{&S92343[44], 27}, {NULL}};
const Segment SGI54305[] = {{&S92343[30], 6}, {NULL}};
const Segment SGI54306[] = {{&S92343[149], 4}, {NULL}};
const Segment SGI54307[] = {{&S92343[175], 3}, {NULL}};
const Segment SGI54308[] = {{&S92343[119], 3}, {&S92343[224], 5}, {NULL}};
const Segment SGI54309[] = {{&S92343[85], 5}, {NULL}};
const Segment SGI54310[] = {{&S92345[59], 9}, {NULL}};
const Segment SGI54311[] = {{&S92345[59], 20}, {NULL}};
const Segment SGI54312[] = {{&S92345[45], 6}, {NULL}};
const Segment SGI54313[] = {{&S92345[127], 3}, {NULL}};
const Segment SGI54314[] = {{&S92345[93], 5}, {NULL}};
const Segment SGI54315[] = {{&S92345[0], 12}, {NULL}};
const Segment SGI54316[] = {{&S92364[44], 11}, {NULL}};
const Segment SGI54317[] = {{&S92364[44], 28}, {NULL}};
const Segment SGI54318[] = {{&S92364[30], 6}, {NULL}};
const Segment SGI54319[] = {{&S92364[120], 3}, {NULL}};
const Segment SGI54320[] = {{&S92364[86], 5}, {NULL}};
const Segment SGI54321[] = {{&S92364[144], 11}, {NULL}};
const Segment SGI54322[] = {{&S92366[44], 10}, {NULL}};
const Segment SGI54323[] = {{&S92366[44], 27}, {NULL}};
const Segment SGI54324[] = {{&S92366[30], 6}, {NULL}};
const Segment SGI54325[] = {{&S92366[119], 3}, {NULL}};
const Segment SGI54326[] = {{&S92366[85], 5}, {NULL}};
const Segment SGI54327[] = {{&S92366[143], 11}, {NULL}};
const Segment SGI54328[] = {{&S92367[44], 11}, {NULL}};
const Segment SGI54329[] = {{&S92367[44], 28}, {NULL}};
const Segment SGI54330[] = {{&S92367[30], 6}, {NULL}};
const Segment SGI54331[] = {{&S92367[120], 3}, {NULL}};
const Segment SGI54332[] = {{&S92367[86], 5}, {NULL}};
const Segment SGI54333[] = {{&S92367[144], 11}, {NULL}};
const Segment SGI54334[] = {{&S92368[44], 11}, {NULL}};
const Segment SGI54335[] = {{&S92368[44], 28}, {NULL}};
const Segment SGI54336[] = {{&S92368[30], 6}, {NULL}};
const Segment SGI54337[] = {{&S92368[120], 3}, {NULL}};
const Segment SGI54338[] = {{&S92368[86], 5}, {NULL}};
const Segment SGI54339[] = {{&S92368[144], 11}, {NULL}};
const Segment SGI54340[] = {{&S105354[44], 12}, {NULL}};
const Segment SGI54341[] = {{&S105354[44], 24}, {NULL}};
const Segment SGI54342[] = {{&S105354[30], 6}, {NULL}};
const Segment SGI54343[] = {{&S105354[116], 3}, {NULL}};
const Segment SGI54344[] = {{&S105354[82], 5}, {NULL}};
const Segment SGI54345[] = {{&S89915[45], 9}, {NULL}};
const Segment SGI54346[] = {{&S89915[30], 6}, {NULL}};
const Segment SGI54347[] = {{&S89915[113], 3}, {NULL}};
const Segment SGI54348[] = {{&S89915[79], 5}, {NULL}};
const Segment SGI54349[] = {{&S89915[137], 13}, {NULL}};
const Segment SGI54350[] = {{&S89919[45], 7}, {NULL}};
const Segment SGI54351[] = {{&S89919[30], 6}, {NULL}};
const Segment SGI54352[] = {{&S89919[58], 3}, {NULL}};
const Segment SGI54353[] = {{&S89919[84], 5}, {NULL}};
const Segment SGI54354[] = {{&S89919[142], 13}, {NULL}};
const Segment SGI54355[] = {{&S89932[45], 11}, {NULL}};
const Segment SGI54356[] = {{&S89932[30], 6}, {NULL}};
const Segment SGI54357[] = {{&S89932[121], 3}, {NULL}};
const Segment SGI54358[] = {{&S89932[87], 5}, {NULL}};
const Segment SGI54359[] = {{&S89932[145], 13}, {NULL}};
const Segment SGI54360[] = {{&S105460[58], 14}, {NULL}};
const Segment SGI54361[] = {{&S105460[58], 25}, {NULL}};
const Segment SGI54362[] = {{&S105460[44], 6}, {NULL}};
const Segment SGI54363[] = {{&S105460[131], 3}, {NULL}};
const Segment SGI54364[] = {{&S105460[97], 5}, {NULL}};
const Segment SGI54365[] = {{&S89956[34], 7}, {NULL}};
const Segment SGI54366[] = {{&S89956[27], 6}, {NULL}};
const Segment SGI54367[] = {{&S89956[107], 2}, {NULL}};
const Segment SGI54368[] = {{&S89956[73], 6}, {NULL}};
const Segment SGI54369[] = {{&S89956[141], 8}, {NULL}};
const Segment SGI54370[] = {{&S105536[45], 13}, {NULL}};
const Segment SGI54371[] = {{&S89585[44], 10}, {NULL}};
const Segment SGI54372[] = {{&S105536[30], 6}, {NULL}};
const Segment SGI54373[] = {{&S89585[44], 33}, {NULL}};
const Segment SGI54374[] = {{&S89585[30], 6}, {NULL}};
const Segment SGI54375[] = {{&S105536[118], 3}, {NULL}};
const Segment SGI54376[] = {{&S89585[132], 3}, {NULL}};
const Segment SGI54377[] = {{&S105536[84], 5}, {NULL}};
const Segment SGI54378[] = {{&S89585[98], 5}, {NULL}};
const Segment SGI54379[] = {{&S105536[142], 13}, {NULL}};
const Segment SGI54380[] = {{&S89585[77], 8}, {NULL}};
const Segment SGI54381[] = {{&S89586[44], 10}, {NULL}};
const Segment SGI54382[] = {{&S89586[44], 33}, {NULL}};
const Segment SGI54383[] = {{&S89586[30], 6}, {NULL}};
const Segment SGI54384[] = {{&S89586[125], 3}, {NULL}};
const Segment SGI54385[] = {{&S89586[91], 5}, {NULL}};
const Segment SGI54386[] = {{&S89586[149], 7}, {NULL}};
const Segment SGI54387[] = {{&S92606[44], 9}, {NULL}};
const Segment SGI54388[] = {{&S92606[44], 20}, {NULL}};
const Segment SGI54389[] = {{&S92606[30], 6}, {NULL}};
const Segment SGI54390[] = {{&S92606[78], 6}, {NULL}};
const Segment SGI54391[] = {{&S92606[115], 3}, {NULL}};
const Segment SGI54392[] = {{&S92611[44], 10}, {NULL}};
const Segment SGI54393[] = {{&S92611[44], 27}, {NULL}};
const Segment SGI54394[] = {{&S92611[30], 6}, {NULL}};
const Segment SGI54395[] = {{&S92611[85], 6}, {NULL}};
const Segment SGI54396[] = {{&S92611[122], 3}, {NULL}};
const Segment SGI54397[] = {{&S90060[59], 7}, {NULL}};
const Segment SGI54398[] = {{&S90060[59], 25}, {NULL}};
const Segment SGI54399[] = {{&S90060[45], 6}, {NULL}};
const Segment SGI54400[] = {{&S90060[72], 3}, {NULL}};
const Segment SGI54401[] = {{&S90060[98], 5}, {NULL}};
const Segment SGI54402[] = {{&S90060[0], 14}, {NULL}};
const Segment SGI54403[] = {{&S90121[44], 7}, {NULL}};
const Segment SGI54404[] = {{&S90121[44], 25}, {NULL}};
const Segment SGI54405[] = {{&S90121[30], 6}, {NULL}};
const Segment SGI54406[] = {{&S90121[83], 6}, {NULL}};
const Segment SGI54407[] = {{&S90121[110], 13}, {NULL}};
const Segment SGI54408[] = {{&S92716[59], 10}, {NULL}};
const Segment SGI54409[] = {{&S92716[59], 27}, {NULL}};
const Segment SGI54410[] = {{&S92716[45], 6}, {NULL}};
const Segment SGI54411[] = {{&S92716[134], 3}, {NULL}};
const Segment SGI54412[] = {{&S92716[100], 5}, {NULL}};
const Segment SGI54413[] = {{&S92716[0], 14}, {NULL}};
const Segment SGI54414[] = {{&S89759[59], 14}, {NULL}};
const Segment SGI54415[] = {{&S89759[59], 25}, {NULL}};
const Segment SGI54416[] = {{&S89759[45], 6}, {NULL}};
const Segment SGI54417[] = {{&S89759[132], 3}, {NULL}};
const Segment SGI54418[] = {{&S89759[98], 5}, {NULL}};
const Segment SGI54419[] = {{&S89759[0], 14}, {NULL}};
const Segment SGI54420[] = {{&S89837[44], 14}, {NULL}};
const Segment SGI54421[] = {{&S89837[44], 35}, {NULL}};
const Segment SGI54422[] = {{&S89837[30], 6}, {NULL}};
const Segment SGI54423[] = {{&S89837[128], 3}, {NULL}};
const Segment SGI54424[] = {{&S89837[93], 6}, {NULL}};
const Segment SGI54425[] = {{&S89837[148], 8}, {NULL}};
const Segment SGI54426[] = {{&S89837[157], 13}, {NULL}};
const Segment SGI54427[] = {{&S92871[45], 10}, {NULL}};
const Segment SGI54428[] = {{&S92871[30], 6}, {NULL}};
const Segment SGI54429[] = {{&S92871[120], 3}, {NULL}};
const Segment SGI54430[] = {{&S92871[86], 5}, {NULL}};
const Segment SGI54431[] = {{&S92871[154], 3}, {NULL}};
const Segment SGI54432[] = {{&S90431[44], 11}, {NULL}};
const Segment SGI54433[] = {{&S90431[44], 28}, {NULL}};
const Segment SGI54434[] = {{&S90431[30], 6}, {NULL}};
const Segment SGI54435[] = {{&S90431[120], 3}, {NULL}};
const Segment SGI54436[] = {{&S90431[86], 5}, {NULL}};
const Segment SGI54437[] = {{&S89959[44], 20}, {NULL}};
const Segment SGI54438[] = {{&S89959[44], 31}, {NULL}};
const Segment SGI54439[] = {{&S89959[30], 6}, {NULL}};
const Segment SGI54440[] = {{&S89959[122], 3}, {NULL}};
const Segment SGI54441[] = {{&S89959[89], 5}, {NULL}};
const Segment SGI54442[] = {{&S89959[126], 14}, {NULL}};
const Segment SGI54443[] = {{&S93087[59], 12}, {NULL}};
const Segment SGI54444[] = {{&S93087[59], 23}, {NULL}};
const Segment SGI54445[] = {{&S93087[45], 6}, {NULL}};
const Segment SGI54446[] = {{&S93087[130], 3}, {NULL}};
const Segment SGI54447[] = {{&S93087[96], 5}, {NULL}};
const Segment SGI54448[] = {{&S93087[0], 14}, {NULL}};
const Segment SGI54449[] = {{&S90546[44], 8}, {NULL}};
const Segment SGI54450[] = {{&S90546[44], 18}, {NULL}};
const Segment SGI54451[] = {{&S90546[30], 6}, {NULL}};
const Segment SGI54452[] = {{&S90546[110], 3}, {NULL}};
const Segment SGI54453[] = {{&S90546[76], 5}, {NULL}};
const Segment SGI54454[] = {{&S93133[59], 4}, {NULL}};
const Segment SGI54455[] = {{&S93133[59], 26}, {NULL}};
const Segment SGI54456[] = {{&S93133[45], 6}, {NULL}};
const Segment SGI54457[] = {{&S93133[133], 3}, {NULL}};
const Segment SGI54458[] = {{&S93133[99], 5}, {NULL}};
const Segment SGI54459[] = {{&S93133[0], 12}, {NULL}};
const Segment SGI54460[] = {{&S93181[34], 6}, {NULL}};
const Segment SGI54461[] = {{&S93181[27], 6}, {NULL}};
const Segment SGI54462[] = {{&S93181[106], 2}, {NULL}};
const Segment SGI54463[] = {{&S93181[72], 6}, {NULL}};
const Segment SGI54464[] = {{&S93181[140], 8}, {NULL}};
const Segment SGI54465[] = {{&S90063[59], 14}, {NULL}};
const Segment SGI54466[] = {{&S90063[59], 31}, {NULL}};
const Segment SGI54467[] = {{&S90063[45], 6}, {NULL}};
const Segment SGI54468[] = {{&S90063[138], 3}, {NULL}};
const Segment SGI54469[] = {{&S90063[104], 5}, {NULL}};
const Segment SGI54470[] = {{&S90063[0], 14}, {NULL}};
const Segment SGI54471[] = {{&S90089[45], 6}, {NULL}};
const Segment SGI54472[] = {{&S90089[30], 6}, {NULL}};
const Segment SGI54473[] = {{&S90089[116], 3}, {NULL}};
const Segment SGI54474[] = {{&S90089[82], 5}, {NULL}};
const Segment SGI54475[] = {{&S90089[140], 13}, {NULL}};
const Segment SGI54476[] = {{&S90115[45], 10}, {NULL}};
const Segment SGI54477[] = {{&S90115[30], 6}, {NULL}};
const Segment SGI54478[] = {{&S90115[115], 3}, {NULL}};
const Segment SGI54479[] = {{&S90115[81], 5}, {NULL}};
const Segment SGI54480[] = {{&S90115[139], 13}, {NULL}};
const Segment SGI54481[] = {{&S90153[52], 10}, {NULL}};
const Segment SGI54482[] = {{&S90153[52], 27}, {NULL}};
const Segment SGI54483[] = {{&S90153[38], 6}, {NULL}};
const Segment SGI54484[] = {{&S90153[93], 5}, {NULL}};
const Segment SGI54485[] = {{&S90153[0], 14}, {NULL}};
const Segment SGI54486[] = {{&S93281[44], 11}, {NULL}};
const Segment SGI54487[] = {{&S93281[44], 22}, {NULL}};
const Segment SGI54488[] = {{&S93281[30], 6}, {NULL}};
const Segment SGI54489[] = {{&S93281[114], 3}, {NULL}};
const Segment SGI54490[] = {{&S93281[80], 5}, {NULL}};
const Segment SGI54491[] = {{&S90196[56], 10}, {NULL}};
const Segment SGI54492[] = {{&S90196[56], 27}, {NULL}};
const Segment SGI54493[] = {{&S90196[42], 6}, {NULL}};
const Segment SGI54494[] = {{&S90196[131], 3}, {NULL}};
const Segment SGI54495[] = {{&S90196[97], 5}, {NULL}};
const Segment SGI54496[] = {{&S90196[0], 12}, {NULL}};
const Segment SGI54497[] = {{&S90593[45], 13}, {NULL}};
const Segment SGI54498[] = {{&S90593[30], 6}, {NULL}};
const Segment SGI54499[] = {{&S90593[117], 3}, {NULL}};
const Segment SGI54500[] = {{&S90593[83], 5}, {NULL}};
const Segment SGI54501[] = {{&S90593[141], 13}, {NULL}};
const Segment SGI54502[] = {{&S90212[44], 7}, {NULL}};
const Segment SGI54503[] = {{&S90212[44], 33}, {NULL}};
const Segment SGI54504[] = {{&S90212[30], 6}, {NULL}};
const Segment SGI54505[] = {{&S90212[125], 3}, {NULL}};
const Segment SGI54506[] = {{&S90212[91], 5}, {NULL}};
const Segment SGI54507[] = {{&S90220[41], 11}, {NULL}};
const Segment SGI54508[] = {{&S90220[41], 35}, {NULL}};
const Segment SGI54509[] = {{&S90220[27], 6}, {NULL}};
const Segment SGI54510[] = {{&S90220[124], 3}, {NULL}};
const Segment SGI54511[] = {{&S90220[90], 5}, {NULL}};
const Segment SGI54512[] = {{&S90246[59], 18}, {NULL}};
const Segment SGI54513[] = {{&S90246[59], 29}, {NULL}};
const Segment SGI54514[] = {{&S90246[45], 6}, {NULL}};
const Segment SGI54515[] = {{&S90246[136], 3}, {NULL}};
const Segment SGI54516[] = {{&S90246[102], 5}, {NULL}};
const Segment SGI54517[] = {{&S90246[0], 14}, {NULL}};
const Segment SGI54518[] = {{&S93396[44], 12}, {NULL}};
const Segment SGI54519[] = {{&S93396[44], 23}, {NULL}};
const Segment SGI54520[] = {{&S93396[30], 6}, {NULL}};
const Segment SGI54521[] = {{&S93396[81], 6}, {NULL}};
const Segment SGI54522[] = {{&S93396[118], 3}, {NULL}};
const Segment SGI54523[] = {{&S93408[44], 10}, {NULL}};
const Segment SGI54524[] = {{&S93408[44], 27}, {NULL}};
const Segment SGI54525[] = {{&S93408[30], 6}, {NULL}};
const Segment SGI54526[] = {{&S93408[119], 3}, {NULL}};
const Segment SGI54527[] = {{&S93408[85], 5}, {NULL}};
const Segment SGI54528[] = {{&S93408[143], 11}, {NULL}};
const Segment SGI54529[] = {{&S90294[44], 15}, {NULL}};
const Segment SGI54530[] = {{&S90294[44], 32}, {NULL}};
const Segment SGI54531[] = {{&S90294[30], 6}, {NULL}};
const Segment SGI54532[] = {{&S90294[93], 6}, {NULL}};
const Segment SGI54533[] = {{&S90295[38], 15}, {NULL}};
const Segment SGI54534[] = {{&S90295[31], 6}, {NULL}};
const Segment SGI54535[] = {{&S90307[44], 14}, {NULL}};
const Segment SGI54536[] = {{&S90307[44], 35}, {NULL}};
const Segment SGI54537[] = {{&S90307[30], 6}, {NULL}};
const Segment SGI54538[] = {{&S90307[93], 6}, {NULL}};
const Segment SGI54539[] = {{&S90307[120], 13}, {NULL}};
const Segment SGI54540[] = {{&S99093[42], 11}, {NULL}};
const Segment SGI54541[] = {{&S99093[42], 22}, {NULL}};
const Segment SGI54542[] = {{&S99093[30], 4}, {NULL}};
const Segment SGI54543[] = {{&S99093[81], 4}, {NULL}};
const Segment SGI54544[] = {{&S99093[81], 3}, {NULL}};
const Segment SGI54545[] = {{&S99094[42], 11}, {NULL}};
const Segment SGI54546[] = {{&S99094[42], 22}, {NULL}};
const Segment SGI54547[] = {{&S99094[30], 4}, {NULL}};
const Segment SGI54548[] = {{&S99094[112], 3}, {NULL}};
const Segment SGI54549[] = {{&S99094[78], 5}, {NULL}};
const Segment SGI54550[] = {{&S93539[44], 12}, {NULL}};
const Segment SGI54551[] = {{&S93539[44], 23}, {NULL}};
const Segment SGI54552[] = {{&S93539[30], 6}, {NULL}};
const Segment SGI54553[] = {{&S93539[115], 3}, {NULL}};
const Segment SGI54554[] = {{&S93539[81], 5}, {NULL}};
const Segment SGI54555[] = {{&S93547[30], 6}, {NULL}};
const Segment SGI54556[] = {{&S93547[91], 3}, {NULL}};
const Segment SGI54557[] = {{&S93547[125], 3}, {NULL}};
const Segment SGI54558[] = {{&S93547[57], 5}, {NULL}};
const Segment SGI54559[] = {{&S90377[44], 18}, {NULL}};
const Segment SGI54560[] = {{&S90377[44], 28}, {NULL}};
const Segment SGI54561[] = {{&S90377[30], 6}, {NULL}};
const Segment SGI54562[] = {{&S90377[120], 3}, {NULL}};
const Segment SGI54563[] = {{&S90377[86], 5}, {NULL}};
const Segment SGI54564[] = {{&S93568[44], 10}, {NULL}};
const Segment SGI54565[] = {{&S93568[44], 21}, {NULL}};
const Segment SGI54566[] = {{&S93568[30], 6}, {NULL}};
const Segment SGI54567[] = {{&S93568[113], 3}, {NULL}};
const Segment SGI54568[] = {{&S93568[79], 5}, {NULL}};
const Segment SGI54569[] = {{&S90432[44], 17}, {NULL}};
const Segment SGI54570[] = {{&S90432[44], 37}, {NULL}};
const Segment SGI54571[] = {{&S90432[30], 6}, {NULL}};
const Segment SGI54572[] = {{&S90432[98], 6}, {NULL}};
const Segment SGI54573[] = {{&S90493[44], 13}, {NULL}};
const Segment SGI54574[] = {{&S90493[44], 30}, {NULL}};
const Segment SGI54575[] = {{&S90493[30], 6}, {NULL}};
const Segment SGI54576[] = {{&S90493[121], 3}, {NULL}};
const Segment SGI54577[] = {{&S90493[88], 5}, {NULL}};
const Segment SGI54578[] = {{&S90493[125], 14}, {NULL}};
const Segment SGI54579[] = {{&S90890[44], 13}, {NULL}};
const Segment SGI54580[] = {{&S90890[44], 30}, {NULL}};
const Segment SGI54581[] = {{&S90890[30], 6}, {NULL}};
const Segment SGI54582[] = {{&S90890[88], 6}, {NULL}};
const Segment SGI54583[] = {{&S90890[115], 13}, {NULL}};
const Segment SGI54584[] = {{&S90547[44], 8}, {NULL}};
const Segment SGI54585[] = {{&S90547[44], 18}, {NULL}};
const Segment SGI54586[] = {{&S90547[30], 6}, {NULL}};
const Segment SGI54587[] = {{&S90547[110], 3}, {NULL}};
const Segment SGI54588[] = {{&S90547[76], 5}, {NULL}};
const Segment SGI54589[] = {{&S90636[44], 9}, {NULL}};
const Segment SGI54590[] = {{&S90636[44], 20}, {NULL}};
const Segment SGI54591[] = {{&S90636[30], 6}, {NULL}};
const Segment SGI54592[] = {{&S90636[78], 6}, {NULL}};
const Segment SGI54593[] = {{&S90636[105], 13}, {NULL}};
const Segment SGI54594[] = {{&S91027[44], 5}, {NULL}};
const Segment SGI54595[] = {{&S91027[44], 16}, {NULL}};
const Segment SGI54596[] = {{&S91027[30], 6}, {NULL}};
const Segment SGI54597[] = {{&S91027[74], 6}, {NULL}};
const Segment SGI54598[] = {{&S91027[101], 13}, {NULL}};
const Segment SGI54599[] = {{&S91029[44], 9}, {NULL}};
const Segment SGI54600[] = {{&S91029[44], 20}, {NULL}};
const Segment SGI54601[] = {{&S91029[30], 6}, {NULL}};
const Segment SGI54602[] = {{&S91029[78], 6}, {NULL}};
const Segment SGI54603[] = {{&S91029[105], 13}, {NULL}};
const Segment SGI54604[] = {{&S91073[44], 7}, {NULL}};
const Segment SGI54605[] = {{&S91073[44], 25}, {NULL}};
const Segment SGI54606[] = {{&S91073[30], 6}, {NULL}};
const Segment SGI54607[] = {{&S91073[83], 6}, {NULL}};
const Segment SGI54608[] = {{&S91073[110], 13}, {NULL}};
const Segment SGI54609[] = {{&S90732[44], 10}, {NULL}};
const Segment SGI54610[] = {{&S90732[44], 27}, {NULL}};
const Segment SGI54611[] = {{&S90732[30], 6}, {NULL}};
const Segment SGI54612[] = {{&S90732[85], 6}, {NULL}};
const Segment SGI54613[] = {{&S90732[112], 13}, {NULL}};
const Segment SGI54614[] = {{&S90755[44], 10}, {NULL}};
const Segment SGI54615[] = {{&S90755[44], 20}, {NULL}};
const Segment SGI54616[] = {{&S90755[30], 6}, {NULL}};
const Segment SGI54617[] = {{&S90755[112], 3}, {NULL}};
const Segment SGI54618[] = {{&S90755[78], 5}, {NULL}};
const Segment SGI54619[] = {{&S100255[44], 10}, {NULL}};
const Segment SGI54620[] = {{&S100255[44], 27}, {NULL}};
const Segment SGI54621[] = {{&S100255[30], 6}, {NULL}};
const Segment SGI54622[] = {{&S100255[119], 3}, {NULL}};
const Segment SGI54623[] = {{&S100255[85], 5}, {NULL}};
const Segment SGI54624[] = {{&S91134[44], 8}, {NULL}};
const Segment SGI54625[] = {{&S91134[44], 19}, {NULL}};
const Segment SGI54626[] = {{&S91134[30], 6}, {NULL}};
const Segment SGI54627[] = {{&S91134[111], 3}, {NULL}};
const Segment SGI54628[] = {{&S91134[77], 5}, {NULL}};
const Segment SGI54629[] = {{&S91147[44], 7}, {NULL}};
const Segment SGI54630[] = {{&S91147[44], 18}, {NULL}};
const Segment SGI54631[] = {{&S91147[30], 6}, {NULL}};
const Segment SGI54632[] = {{&S91147[110], 3}, {NULL}};
const Segment SGI54633[] = {{&S91147[76], 5}, {NULL}};
const Segment SGI54634[] = {{&S91221[34], 7}, {NULL}};
const Segment SGI54635[] = {{&S91221[27], 6}, {NULL}};
const Segment SGI54636[] = {{&S91221[107], 2}, {NULL}};
const Segment SGI54637[] = {{&S91221[73], 6}, {NULL}};
const Segment SGI54638[] = {{&S91221[141], 8}, {NULL}};
const Segment SGI54639[] = {{&S91225[44], 9}, {NULL}};
const Segment SGI54640[] = {{&S91225[44], 20}, {NULL}};
const Segment SGI54641[] = {{&S91225[30], 6}, {NULL}};
const Segment SGI54642[] = {{&S91225[112], 3}, {NULL}};
const Segment SGI54643[] = {{&S91225[78], 5}, {NULL}};
const Segment SGI54644[] = {{&S91225[136], 7}, {NULL}};
const Segment SGI54645[] = {{&S90872[34], 17}, {NULL}};
const Segment SGI54646[] = {{&S90872[27], 6}, {NULL}};
const Segment SGI54647[] = {{&S90872[110], 2}, {NULL}};
const Segment SGI54648[] = {{&S90872[76], 6}, {NULL}};
const Segment SGI54649[] = {{&S90872[144], 8}, {NULL}};
const Segment SGI54650[] = {{&S90932[44], 14}, {NULL}};
const Segment SGI54651[] = {{&S90932[44], 25}, {NULL}};
const Segment SGI54652[] = {{&S90932[30], 6}, {NULL}};
const Segment SGI54653[] = {{&S90932[83], 6}, {NULL}};
const Segment SGI54654[] = {{&S90932[110], 13}, {NULL}};
const Segment SGI54655[] = {{&S94020[45], 10}, {NULL}};
const Segment SGI54656[] = {{&S94020[30], 6}, {NULL}};
const Segment SGI54657[] = {{&S94020[120], 3}, {NULL}};
const Segment SGI54658[] = {{&S94020[86], 5}, {NULL}};
const Segment SGI54659[] = {{&S94020[154], 3}, {NULL}};
const Segment SGI54660[] = {{&S94021[45], 11}, {NULL}};
const Segment SGI54661[] = {{&S94021[30], 6}, {NULL}};
const Segment SGI54662[] = {{&S94021[121], 3}, {NULL}};
const Segment SGI54663[] = {{&S94021[87], 5}, {NULL}};
const Segment SGI54664[] = {{&S94021[155], 3}, {NULL}};
const Segment SGI54665[] = {{&S94023[45], 11}, {NULL}};
const Segment SGI54666[] = {{&S94023[30], 6}, {NULL}};
const Segment SGI54667[] = {{&S94023[121], 3}, {NULL}};
const Segment SGI54668[] = {{&S94023[87], 5}, {NULL}};
const Segment SGI54669[] = {{&S94023[155], 3}, {NULL}};
const Segment SGI54670[] = {{&S91028[44], 6}, {NULL}};
const Segment SGI54671[] = {{&S91028[44], 23}, {NULL}};
const Segment SGI54672[] = {{&S91028[30], 6}, {NULL}};
const Segment SGI54673[] = {{&S91028[81], 6}, {NULL}};
const Segment SGI54674[] = {{&S91028[108], 13}, {NULL}};
const Segment SGI54675[] = {{&S91336[44], 11}, {NULL}};
const Segment SGI54676[] = {{&S91336[44], 22}, {NULL}};
const Segment SGI54677[] = {{&S91336[30], 6}, {NULL}};
const Segment SGI54678[] = {{&S91336[114], 3}, {NULL}};
const Segment SGI54679[] = {{&S91336[80], 5}, {NULL}};
const Segment SGI54680[] = {{&S91346[45], 7}, {NULL}};
const Segment SGI54681[] = {{&S91346[30], 6}, {NULL}};
const Segment SGI54682[] = {{&S91346[58], 3}, {NULL}};
const Segment SGI54683[] = {{&S91346[84], 5}, {NULL}};
const Segment SGI54684[] = {{&S91346[142], 13}, {NULL}};
const Segment SGI54685[] = {{&S91131[45], 22}, {NULL}};
const Segment SGI54686[] = {{&S91131[30], 6}, {NULL}};
const Segment SGI54687[] = {{&S91131[126], 3}, {NULL}};
const Segment SGI54688[] = {{&S91131[92], 5}, {NULL}};
const Segment SGI54689[] = {{&S91131[150], 13}, {NULL}};
const Segment SGI54690[] = {{&S91133[44], 22}, {NULL}};
const Segment SGI54691[] = {{&S91133[44], 33}, {NULL}};
const Segment SGI54692[] = {{&S91133[30], 6}, {NULL}};
const Segment SGI54693[] = {{&S91133[126], 3}, {NULL}};
const Segment SGI54694[] = {{&S91133[91], 6}, {NULL}};
const Segment SGI54695[] = {{&S91133[146], 8}, {NULL}};
const Segment SGI54696[] = {{&S91133[155], 13}, {NULL}};
const Segment SGI54697[] = {{&S91146[44], 7}, {NULL}};
const Segment SGI54698[] = {{&S91146[44], 18}, {NULL}};
const Segment SGI54699[] = {{&S91146[30], 6}, {NULL}};
const Segment SGI54700[] = {{&S91146[110], 3}, {NULL}};
const Segment SGI54701[] = {{&S91146[76], 5}, {NULL}};
const Segment SGI54702[] = {{&S94282[44], 11}, {NULL}};
const Segment SGI54703[] = {{&S94282[44], 22}, {NULL}};
const Segment SGI54704[] = {{&S94282[30], 6}, {NULL}};
const Segment SGI54705[] = {{&S94282[114], 3}, {NULL}};
const Segment SGI54706[] = {{&S94282[80], 5}, {NULL}};
const Segment SGI54707[] = {{&S94282[137], 10}, {NULL}};
const Segment SGI54708[] = {{&S91220[44], 18}, {NULL}};
const Segment SGI54709[] = {{&S91220[44], 29}, {NULL}};
const Segment SGI54710[] = {{&S91220[30], 6}, {NULL}};
const Segment SGI54711[] = {{&S91220[87], 6}, {NULL}};
const Segment SGI54712[] = {{&S91220[114], 13}, {NULL}};
const Segment SGI54713[] = {{&S91641[44], 13}, {NULL}};
const Segment SGI54714[] = {{&S91641[44], 24}, {NULL}};
const Segment SGI54715[] = {{&S91641[30], 6}, {NULL}};
const Segment SGI54716[] = {{&S91641[82], 6}, {NULL}};
const Segment SGI54717[] = {{&S91641[109], 13}, {NULL}};
const Segment SGI54718[] = {{&S91657[44], 10}, {NULL}};
const Segment SGI54719[] = {{&S91657[44], 27}, {NULL}};
const Segment SGI54720[] = {{&S91657[30], 6}, {NULL}};
const Segment SGI54721[] = {{&S91657[120], 3}, {NULL}};
const Segment SGI54722[] = {{&S91657[85], 6}, {NULL}};
const Segment SGI54723[] = {{&S91657[140], 8}, {NULL}};
const Segment SGI54724[] = {{&S91657[149], 13}, {NULL}};
const Segment SGI54725[] = {{&S91714[44], 9}, {NULL}};
const Segment SGI54726[] = {{&S91714[44], 20}, {NULL}};
const Segment SGI54727[] = {{&S91714[30], 6}, {NULL}};
const Segment SGI54728[] = {{&S91714[119], 3}, {NULL}};
const Segment SGI54729[] = {{&S91714[85], 5}, {NULL}};
const Segment SGI54730[] = {{&S91714[64], 8}, {NULL}};
const Segment SGI54731[] = {{&S91715[44], 9}, {NULL}};
const Segment SGI54732[] = {{&S91715[44], 20}, {NULL}};
const Segment SGI54733[] = {{&S91715[30], 6}, {NULL}};
const Segment SGI54734[] = {{&S91715[112], 3}, {NULL}};
const Segment SGI54735[] = {{&S91715[78], 5}, {NULL}};
const Segment SGI54736[] = {{&S91715[136], 7}, {NULL}};
const Segment SGI54737[] = {{&S94442[44], 12}, {NULL}};
const Segment SGI54738[] = {{&S94442[44], 23}, {NULL}};
const Segment SGI54739[] = {{&S94442[30], 6}, {NULL}};
const Segment SGI54740[] = {{&S94442[115], 3}, {NULL}};
const Segment SGI54741[] = {{&S94442[81], 5}, {NULL}};
const Segment SGI54742[] = {{&S94443[44], 13}, {NULL}};
const Segment SGI54743[] = {{&S94443[44], 24}, {NULL}};
const Segment SGI54744[] = {{&S94443[30], 6}, {NULL}};
const Segment SGI54745[] = {{&S94443[116], 3}, {NULL}};
const Segment SGI54746[] = {{&S94443[82], 5}, {NULL}};
const Segment SGI54747[] = {{&S94452[59], 18}, {NULL}};
const Segment SGI54748[] = {{&S94452[59], 29}, {NULL}};
const Segment SGI54749[] = {{&S94452[45], 6}, {NULL}};
const Segment SGI54750[] = {{&S94452[136], 3}, {NULL}};
const Segment SGI54751[] = {{&S94452[102], 5}, {NULL}};
const Segment SGI54752[] = {{&S94452[0], 14}, {NULL}};
const Segment SGI54753[] = {{&S91738[59], 10}, {NULL}};
const Segment SGI54754[] = {{&S91738[59], 27}, {NULL}};
const Segment SGI54755[] = {{&S91738[45], 6}, {NULL}};
const Segment SGI54756[] = {{&S91738[134], 3}, {NULL}};
const Segment SGI54757[] = {{&S91738[100], 5}, {NULL}};
const Segment SGI54758[] = {{&S91738[0], 14}, {NULL}};
const Segment SGI54759[] = {{&S91340[44], 10}, {NULL}};
const Segment SGI54760[] = {{&S91340[44], 21}, {NULL}};
const Segment SGI54761[] = {{&S91340[30], 6}, {NULL}};
const Segment SGI54762[] = {{&S91340[113], 3}, {NULL}};
const Segment SGI54763[] = {{&S91340[79], 5}, {NULL}};
const Segment SGI54764[] = {{&S91749[45], 9}, {NULL}};
const Segment SGI54765[] = {{&S91749[30], 6}, {NULL}};
const Segment SGI54766[] = {{&S91749[113], 3}, {NULL}};
const Segment SGI54767[] = {{&S91749[79], 5}, {NULL}};
const Segment SGI54768[] = {{&S91749[137], 13}, {NULL}};
const Segment SGI54769[] = {{&S91343[59], 13}, {NULL}};
const Segment SGI54770[] = {{&S91343[59], 30}, {NULL}};
const Segment SGI54771[] = {{&S91343[45], 6}, {NULL}};
const Segment SGI54772[] = {{&S91343[137], 3}, {NULL}};
const Segment SGI54773[] = {{&S91343[103], 5}, {NULL}};
const Segment SGI54774[] = {{&S91343[0], 14}, {NULL}};
const Segment SGI54775[] = {{&S91352[59], 12}, {NULL}};
const Segment SGI54776[] = {{&S91352[59], 23}, {NULL}};
const Segment SGI54777[] = {{&S91352[45], 6}, {NULL}};
const Segment SGI54778[] = {{&S91352[130], 3}, {NULL}};
const Segment SGI54779[] = {{&S91352[96], 5}, {NULL}};
const Segment SGI54780[] = {{&S91352[0], 14}, {NULL}};
const Segment SGI54781[] = {{&S91409[44], 14}, {NULL}};
const Segment SGI54782[] = {{&S91409[44], 31}, {NULL}};
const Segment SGI54783[] = {{&S91409[30], 6}, {NULL}};
const Segment SGI54784[] = {{&S91409[123], 3}, {NULL}};
const Segment SGI54785[] = {{&S91409[89], 5}, {NULL}};
const Segment SGI54786[] = {{&S91440[44], 11}, {NULL}};
const Segment SGI54787[] = {{&S91440[44], 28}, {NULL}};
const Segment SGI54788[] = {{&S91440[30], 6}, {NULL}};
const Segment SGI54789[] = {{&S91440[86], 6}, {NULL}};
const Segment SGI54790[] = {{&S91440[113], 13}, {NULL}};
const Segment SGI54791[] = {{&S91910[44], 11}, {NULL}};
const Segment SGI54792[] = {{&S91910[44], 28}, {NULL}};
const Segment SGI54793[] = {{&S91910[30], 6}, {NULL}};
const Segment SGI54794[] = {{&S91910[127], 3}, {NULL}};
const Segment SGI54795[] = {{&S91910[93], 5}, {NULL}};
const Segment SGI54796[] = {{&S91910[72], 8}, {NULL}};
const Segment SGI54797[] = {{&S91496[31], 12}, {NULL}};
const Segment SGI54798[] = {{&S91496[58], 3}, {NULL}};
const Segment SGI54799[] = {{&S91931[59], 7}, {NULL}};
const Segment SGI54800[] = {{&S91931[59], 25}, {NULL}};
const Segment SGI54801[] = {{&S91931[45], 6}, {NULL}};
const Segment SGI54802[] = {{&S91931[72], 3}, {NULL}};
const Segment SGI54803[] = {{&S91931[98], 5}, {NULL}};
const Segment SGI54804[] = {{&S91931[0], 14}, {NULL}};
const Segment SGI54805[] = {{&S91932[59], 7}, {NULL}};
const Segment SGI54806[] = {{&S91932[59], 25}, {NULL}};
const Segment SGI54807[] = {{&S91932[45], 6}, {NULL}};
const Segment SGI54808[] = {{&S91932[72], 3}, {NULL}};
const Segment SGI54809[] = {{&S91932[98], 5}, {NULL}};
const Segment SGI54810[] = {{&S91932[0], 14}, {NULL}};
const Segment SGI54811[] = {{&S91518[31], 16}, {NULL}};
const Segment SGI54812[] = {{&S101024[44], 5}, {NULL}};
const Segment SGI54813[] = {{&S91518[62], 3}, {NULL}};
const Segment SGI54814[] = {{&S101024[44], 22}, {NULL}};
const Segment SGI54815[] = {{&S101024[30], 6}, {NULL}};
const Segment SGI54816[] = {{&S101024[114], 3}, {NULL}};
const Segment SGI54817[] = {{&S101024[80], 5}, {NULL}};
const Segment SGI54818[] = {{&S91542[45], 14}, {NULL}};
const Segment SGI54819[] = {{&S91542[30], 6}, {NULL}};
const Segment SGI54820[] = {{&S91542[118], 3}, {NULL}};
const Segment SGI54821[] = {{&S91542[84], 5}, {NULL}};
const Segment SGI54822[] = {{&S91542[142], 13}, {NULL}};
const Segment SGI54823[] = {{&S94672[30], 6}, {NULL}};
const Segment SGI54824[] = {{&S94672[91], 3}, {NULL}};
const Segment SGI54825[] = {{&S94672[125], 3}, {NULL}};
const Segment SGI54826[] = {{&S94672[57], 5}, {NULL}};
const Segment SGI54827[] = {{&S91597[44], 10}, {NULL}};
const Segment SGI54828[] = {{&S91597[44], 27}, {NULL}};
const Segment SGI54829[] = {{&S91597[30], 6}, {NULL}};
const Segment SGI54830[] = {{&S91597[118], 3}, {NULL}};
const Segment SGI54831[] = {{&S91597[85], 5}, {NULL}};
const Segment SGI54832[] = {{&S91597[122], 14}, {NULL}};
const Segment SGI54833[] = {{&S92099[44], 11}, {NULL}};
const Segment SGI54834[] = {{&S92099[44], 22}, {NULL}};
const Segment SGI54835[] = {{&S92099[30], 6}, {NULL}};
const Segment SGI54836[] = {{&S92099[114], 3}, {NULL}};
const Segment SGI54837[] = {{&S92099[80], 5}, {NULL}};
const Segment SGI54838[] = {{&S91697[44], 8}, {NULL}};
const Segment SGI54839[] = {{&S91697[44], 19}, {NULL}};
const Segment SGI54840[] = {{&S91697[30], 6}, {NULL}};
const Segment SGI54841[] = {{&S91697[111], 3}, {NULL}};
const Segment SGI54842[] = {{&S91697[77], 5}, {NULL}};
const Segment SGI54843[] = {{&S92113[45], 7}, {NULL}};
const Segment SGI54844[] = {{&S92113[30], 6}, {NULL}};
const Segment SGI54845[] = {{&S92113[58], 3}, {NULL}};
const Segment SGI54846[] = {{&S92113[84], 5}, {NULL}};
const Segment SGI54847[] = {{&S92113[142], 13}, {NULL}};
const Segment SGI54848[] = {{&S91728[44], 15}, {NULL}};
const Segment SGI54849[] = {{&S91728[44], 26}, {NULL}};
const Segment SGI54850[] = {{&S91728[30], 6}, {NULL}};
const Segment SGI54851[] = {{&S91728[118], 3}, {NULL}};
const Segment SGI54852[] = {{&S91728[84], 5}, {NULL}};
const Segment SGI54853[] = {{&S91779[44], 7}, {NULL}};
const Segment SGI54854[] = {{&S91779[44], 28}, {NULL}};
const Segment SGI54855[] = {{&S91779[30], 6}, {NULL}};
const Segment SGI54856[] = {{&S91779[120], 3}, {NULL}};
const Segment SGI54857[] = {{&S91779[86], 5}, {NULL}};
const Segment SGI54858[] = {{&S92299[44], 10}, {NULL}};
const Segment SGI54859[] = {{&S92299[44], 21}, {NULL}};
const Segment SGI54860[] = {{&S92299[30], 6}, {NULL}};
const Segment SGI54861[] = {{&S92299[113], 3}, {NULL}};
const Segment SGI54862[] = {{&S92299[79], 5}, {NULL}};
const Segment SGI54863[] = {{&S92300[38], 10}, {NULL}};
const Segment SGI54864[] = {{&S92300[31], 6}, {NULL}};
const Segment SGI54865[] = {{&S91913[44], 12}, {NULL}};
const Segment SGI54866[] = {{&S91913[44], 23}, {NULL}};
const Segment SGI54867[] = {{&S91913[30], 6}, {NULL}};
const Segment SGI54868[] = {{&S91913[115], 3}, {NULL}};
const Segment SGI54869[] = {{&S91913[81], 5}, {NULL}};
const Segment SGI54870[] = {{&S91929[31], 12}, {NULL}};
const Segment SGI54871[] = {{&S91929[58], 3}, {NULL}};
const Segment SGI54872[] = {{&S92336[44], 13}, {NULL}};
const Segment SGI54873[] = {{&S92336[44], 24}, {NULL}};
const Segment SGI54874[] = {{&S92336[30], 6}, {NULL}};
const Segment SGI54875[] = {{&S92336[123], 3}, {NULL}};
const Segment SGI54876[] = {{&S92336[89], 5}, {NULL}};
const Segment SGI54877[] = {{&S92336[68], 8}, {NULL}};
const Segment SGI54878[] = {{&S92337[44], 13}, {NULL}};
const Segment SGI54879[] = {{&S92337[44], 24}, {NULL}};
const Segment SGI54880[] = {{&S92337[30], 6}, {NULL}};
const Segment SGI54881[] = {{&S92337[116], 3}, {NULL}};
const Segment SGI54882[] = {{&S92337[82], 5}, {NULL}};
const Segment SGI54883[] = {{&S92337[140], 7}, {NULL}};
const Segment SGI54884[] = {{&S94926[44], 8}, {NULL}};
const Segment SGI54885[] = {{&S94926[44], 19}, {NULL}};
const Segment SGI54886[] = {{&S94926[30], 6}, {NULL}};
const Segment SGI54887[] = {{&S94926[111], 3}, {NULL}};
const Segment SGI54888[] = {{&S94926[77], 5}, {NULL}};
const Segment SGI54889[] = {{&S94937[59], 14}, {NULL}};
const Segment SGI54890[] = {{&S94937[59], 25}, {NULL}};
const Segment SGI54891[] = {{&S94937[45], 6}, {NULL}};
const Segment SGI54892[] = {{&S94937[132], 3}, {NULL}};
const Segment SGI54893[] = {{&S94937[98], 5}, {NULL}};
const Segment SGI54894[] = {{&S94937[0], 14}, {NULL}};
const Segment SGI54895[] = {{&S91957[59], 11}, {NULL}};
const Segment SGI54896[] = {{&S91957[59], 22}, {NULL}};
const Segment SGI54897[] = {{&S91957[45], 6}, {NULL}};
const Segment SGI54898[] = {{&S91957[129], 3}, {NULL}};
const Segment SGI54899[] = {{&S91957[95], 5}, {NULL}};
const Segment SGI54900[] = {{&S91957[0], 14}, {NULL}};
const Segment SGI54901[] = {{&S92016[44], 17}, {NULL}};
const Segment SGI54902[] = {{&S92016[44], 28}, {NULL}};
const Segment SGI54903[] = {{&S92016[30], 6}, {NULL}};
const Segment SGI54904[] = {{&S92016[86], 6}, {NULL}};
const Segment SGI54905[] = {{&S92016[113], 13}, {NULL}};
const Segment SGI54906[] = {{&S92418[44], 13}, {NULL}};
const Segment SGI54907[] = {{&S92418[44], 24}, {NULL}};
const Segment SGI54908[] = {{&S92418[30], 6}, {NULL}};
const Segment SGI54909[] = {{&S92418[82], 6}, {NULL}};
const Segment SGI54910[] = {{&S92418[109], 13}, {NULL}};
const Segment SGI54911[] = {{&S92045[44], 20}, {NULL}};
const Segment SGI54912[] = {{&S92045[44], 31}, {NULL}};
const Segment SGI54913[] = {{&S92045[30], 6}, {NULL}};
const Segment SGI54914[] = {{&S92045[124], 3}, {NULL}};
const Segment SGI54915[] = {{&S92045[89], 6}, {NULL}};
const Segment SGI54916[] = {{&S92045[144], 8}, {NULL}};
const Segment SGI54917[] = {{&S92045[153], 13}, {NULL}};
const Segment SGI54918[] = {{&S92048[44], 10}, {NULL}};
const Segment SGI54919[] = {{&S92048[44], 27}, {NULL}};
const Segment SGI54920[] = {{&S92048[30], 6}, {NULL}};
const Segment SGI54921[] = {{&S92048[118], 3}, {NULL}};
const Segment SGI54922[] = {{&S92048[85], 5}, {NULL}};
const Segment SGI54923[] = {{&S92048[122], 14}, {NULL}};
const Segment SGI54924[] = {{&S92098[44], 7}, {NULL}};
const Segment SGI54925[] = {{&S92098[44], 18}, {NULL}};
const Segment SGI54926[] = {{&S92098[30], 6}, {NULL}};
const Segment SGI54927[] = {{&S92098[110], 3}, {NULL}};
const Segment SGI54928[] = {{&S92098[76], 5}, {NULL}};
const Segment SGI54929[] = {{&S92114[44], 9}, {NULL}};
const Segment SGI54930[] = {{&S92114[44], 19}, {NULL}};
const Segment SGI54931[] = {{&S92114[30], 6}, {NULL}};
const Segment SGI54932[] = {{&S92114[111], 3}, {NULL}};
const Segment SGI54933[] = {{&S92114[77], 5}, {NULL}};
const Segment SGI54934[] = {{&S100762[44], 6}, {NULL}};
const Segment SGI54935[] = {{&S100762[44], 17}, {NULL}};
const Segment SGI54936[] = {{&S100762[30], 6}, {NULL}};
const Segment SGI54937[] = {{&S100762[109], 3}, {NULL}};
const Segment SGI54938[] = {{&S100762[75], 5}, {NULL}};
const Segment SGI54939[] = {{&S101542[45], 10}, {NULL}};
const Segment SGI54940[] = {{&S101542[30], 6}, {NULL}};
const Segment SGI54941[] = {{&S101542[120], 3}, {NULL}};
const Segment SGI54942[] = {{&S101542[86], 5}, {NULL}};
const Segment SGI54943[] = {{&S92500[44], 10}, {NULL}};
const Segment SGI54944[] = {{&S92500[44], 27}, {NULL}};
const Segment SGI54945[] = {{&S92500[30], 6}, {NULL}};
const Segment SGI54946[] = {{&S92500[119], 3}, {NULL}};
const Segment SGI54947[] = {{&S92500[85], 5}, {NULL}};
const Segment SGI54948[] = {{&S92680[45], 10}, {NULL}};
const Segment SGI54949[] = {{&S92680[30], 6}, {NULL}};
const Segment SGI54950[] = {{&S92680[120], 3}, {NULL}};
const Segment SGI54951[] = {{&S92680[86], 5}, {NULL}};
const Segment SGI54952[] = {{&S92680[144], 13}, {NULL}};
const Segment SGI54953[] = {{&S92681[44], 10}, {NULL}};
const Segment SGI54954[] = {{&S92681[44], 27}, {NULL}};
const Segment SGI54955[] = {{&S92681[30], 6}, {NULL}};
const Segment SGI54956[] = {{&S92681[120], 3}, {NULL}};
const Segment SGI54957[] = {{&S92681[85], 6}, {NULL}};
const Segment SGI54958[] = {{&S92681[140], 8}, {NULL}};
const Segment SGI54959[] = {{&S92681[149], 13}, {NULL}};
const Segment SGI54960[] = {{&S92328[45], 21}, {NULL}};
const Segment SGI54961[] = {{&S92328[30], 6}, {NULL}};
const Segment SGI54962[] = {{&S92328[125], 3}, {NULL}};
const Segment SGI54963[] = {{&S92328[91], 5}, {NULL}};
const Segment SGI54964[] = {{&S92328[149], 13}, {NULL}};
const Segment SGI54965[] = {{&S92379[44], 11}, {NULL}};
const Segment SGI54966[] = {{&S92379[44], 28}, {NULL}};
const Segment SGI54967[] = {{&S92379[30], 6}, {NULL}};
const Segment SGI54968[] = {{&S92379[86], 6}, {NULL}};
const Segment SGI54969[] = {{&S92379[113], 13}, {NULL}};
const Segment SGI54970[] = {{&S92406[44], 4}, {NULL}};
const Segment SGI54971[] = {{&S92406[44], 15}, {NULL}};
const Segment SGI54972[] = {{&S92406[30], 6}, {NULL}};
const Segment SGI54973[] = {{&S92406[73], 6}, {NULL}};
const Segment SGI54974[] = {{&S92406[100], 13}, {NULL}};
const Segment SGI54975[] = {{&S92415[45], 7}, {NULL}};
const Segment SGI54976[] = {{&S92415[30], 6}, {NULL}};
const Segment SGI54977[] = {{&S92415[121], 3}, {NULL}};
const Segment SGI54978[] = {{&S92415[87], 5}, {NULL}};
const Segment SGI54979[] = {{&S92415[145], 13}, {NULL}};
const Segment SGI54980[] = {{&S92416[44], 13}, {NULL}};
const Segment SGI54981[] = {{&S92416[44], 24}, {NULL}};
const Segment SGI54982[] = {{&S92416[30], 6}, {NULL}};
const Segment SGI54983[] = {{&S92416[116], 3}, {NULL}};
const Segment SGI54984[] = {{&S92416[82], 5}, {NULL}};
const Segment SGI54985[] = {{&S92759[44], 7}, {NULL}};
const Segment SGI54986[] = {{&S92759[44], 25}, {NULL}};
const Segment SGI54987[] = {{&S92759[30], 6}, {NULL}};
const Segment SGI54988[] = {{&S92759[83], 6}, {NULL}};
const Segment SGI54989[] = {{&S92759[110], 13}, {NULL}};
const Segment SGI54990[] = {{&S92780[44], 6}, {NULL}};
const Segment SGI54991[] = {{&S92780[44], 23}, {NULL}};
const Segment SGI54992[] = {{&S92780[30], 6}, {NULL}};
const Segment SGI54993[] = {{&S92780[115], 3}, {NULL}};
const Segment SGI54994[] = {{&S92780[81], 5}, {NULL}};
const Segment SGI54995[] = {{&S92505[58], 12}, {NULL}};
const Segment SGI54996[] = {{&S92505[58], 23}, {NULL}};
const Segment SGI54997[] = {{&S92505[44], 6}, {NULL}};
const Segment SGI54998[] = {{&S92505[129], 3}, {NULL}};
const Segment SGI54999[] = {{&S92505[95], 5}, {NULL}};
const Segment SGI55000[] = {{&S92548[44], 13}, {NULL}};
const Segment SGI55001[] = {{&S92548[44], 30}, {NULL}};
const Segment SGI55002[] = {{&S92548[30], 6}, {NULL}};
const Segment SGI55003[] = {{&S92548[122], 3}, {NULL}};
const Segment SGI55004[] = {{&S92548[88], 5}, {NULL}};
const Segment SGI55005[] = {{&S92876[45], 7}, {NULL}};
const Segment SGI55006[] = {{&S92876[30], 6}, {NULL}};
const Segment SGI55007[] = {{&S92876[58], 3}, {NULL}};
const Segment SGI55008[] = {{&S92876[84], 5}, {NULL}};
const Segment SGI55009[] = {{&S92876[142], 13}, {NULL}};
const Segment SGI55010[] = {{&S92879[31], 5}, {NULL}};
const Segment SGI55011[] = {{&S92879[51], 3}, {NULL}};
const Segment SGI55012[] = {{&S92629[44], 17}, {NULL}};
const Segment SGI55013[] = {{&S92629[44], 28}, {NULL}};
const Segment SGI55014[] = {{&S92629[30], 6}, {NULL}};
const Segment SGI55015[] = {{&S92629[86], 6}, {NULL}};
const Segment SGI55016[] = {{&S92629[113], 13}, {NULL}};
const Segment SGI55017[] = {{&S92974[44], 10}, {NULL}};
const Segment SGI55018[] = {{&S92974[44], 21}, {NULL}};
const Segment SGI55019[] = {{&S92974[30], 6}, {NULL}};
const Segment SGI55020[] = {{&S92974[79], 6}, {NULL}};
const Segment SGI55021[] = {{&S92974[106], 13}, {NULL}};
const Segment SGI55022[] = {{&S92698[44], 10}, {NULL}};
const Segment SGI55023[] = {{&S92698[44], 33}, {NULL}};
const Segment SGI55024[] = {{&S92698[30], 6}, {NULL}};
const Segment SGI55025[] = {{&S92698[125], 3}, {NULL}};
const Segment SGI55026[] = {{&S92698[91], 5}, {NULL}};
const Segment SGI55027[] = {{&S92698[148], 10}, {NULL}};
const Segment SGI55028[] = {{&S92717[59], 10}, {NULL}};
const Segment SGI55029[] = {{&S92717[59], 27}, {NULL}};
const Segment SGI55030[] = {{&S92717[45], 6}, {NULL}};
const Segment SGI55031[] = {{&S92717[134], 3}, {NULL}};
const Segment SGI55032[] = {{&S92717[100], 5}, {NULL}};
const Segment SGI55033[] = {{&S92717[0], 14}, {NULL}};
const Segment SGI55034[] = {{&S95780[45], 12}, {NULL}};
const Segment SGI55035[] = {{&S95780[30], 6}, {NULL}};
const Segment SGI55036[] = {{&S95780[116], 3}, {NULL}};
const Segment SGI55037[] = {{&S95780[82], 5}, {NULL}};
const Segment SGI55038[] = {{&S95780[150], 3}, {NULL}};
const Segment SGI55039[] = {{&S92769[44], 7}, {NULL}};
const Segment SGI55040[] = {{&S92769[44], 32}, {NULL}};
const Segment SGI55041[] = {{&S92769[30], 6}, {NULL}};
const Segment SGI55042[] = {{&S92769[90], 6}, {NULL}};
const Segment SGI55043[] = {{&S92769[117], 13}, {NULL}};
const Segment SGI55044[] = {{&S92880[59], 13}, {NULL}};
const Segment SGI55045[] = {{&S92880[59], 24}, {NULL}};
const Segment SGI55046[] = {{&S92880[45], 6}, {NULL}};
const Segment SGI55047[] = {{&S92880[131], 3}, {NULL}};
const Segment SGI55048[] = {{&S92880[97], 5}, {NULL}};
const Segment SGI55049[] = {{&S92880[0], 14}, {NULL}};
const Segment SGI55050[] = {{&S93159[44], 7}, {NULL}};
const Segment SGI55051[] = {{&S93159[44], 25}, {NULL}};
const Segment SGI55052[] = {{&S93159[30], 6}, {NULL}};
const Segment SGI55053[] = {{&S93159[57], 3}, {NULL}};
const Segment SGI55054[] = {{&S93159[83], 5}, {NULL}};
const Segment SGI55055[] = {{&S93159[120], 14}, {NULL}};
const Segment SGI55056[] = {{&S93179[34], 9}, {NULL}};
const Segment SGI55057[] = {{&S93179[27], 6}, {NULL}};
const Segment SGI55058[] = {{&S93179[102], 2}, {NULL}};
const Segment SGI55059[] = {{&S93179[68], 6}, {NULL}};
const Segment SGI55060[] = {{&S93179[136], 8}, {NULL}};
const Segment SGI55061[] = {{&S93193[44], 10}, {NULL}};
const Segment SGI55062[] = {{&S93193[44], 27}, {NULL}};
const Segment SGI55063[] = {{&S93193[30], 6}, {NULL}};
const Segment SGI55064[] = {{&S93193[119], 3}, {NULL}};
const Segment SGI55065[] = {{&S93193[85], 5}, {NULL}};
const Segment SGI55066[] = {{&S93193[143], 7}, {NULL}};
const Segment SGI55067[] = {{&S92930[44], 6}, {NULL}};
const Segment SGI55068[] = {{&S92930[44], 17}, {NULL}};
const Segment SGI55069[] = {{&S92930[30], 6}, {NULL}};
const Segment SGI55070[] = {{&S92930[109], 3}, {NULL}};
const Segment SGI55071[] = {{&S92930[75], 5}, {NULL}};
const Segment SGI55072[] = {{&S92932[44], 10}, {NULL}};
const Segment SGI55073[] = {{&S92932[44], 27}, {NULL}};
const Segment SGI55074[] = {{&S92932[30], 6}, {NULL}};
const Segment SGI55075[] = {{&S92932[119], 3}, {NULL}};
const Segment SGI55076[] = {{&S92932[85], 5}, {NULL}};
const Segment SGI55077[] = {{&S92932[130], 14}, {NULL}};
const Segment SGI55078[] = {{&S95974[44], 11}, {NULL}};
const Segment SGI55079[] = {{&S95974[44], 22}, {NULL}};
const Segment SGI55080[] = {{&S95974[30], 6}, {NULL}};
const Segment SGI55081[] = {{&S95974[114], 3}, {NULL}};
const Segment SGI55082[] = {{&S95974[80], 5}, {NULL}};
const Segment SGI55083[] = {{&S95975[44], 11}, {NULL}};
const Segment SGI55084[] = {{&S95975[44], 22}, {NULL}};
const Segment SGI55085[] = {{&S95975[30], 6}, {NULL}};
const Segment SGI55086[] = {{&S95975[114], 3}, {NULL}};
const Segment SGI55087[] = {{&S95975[80], 5}, {NULL}};
const Segment SGI55088[] = {{&S93282[44], 11}, {NULL}};
const Segment SGI55089[] = {{&S93282[44], 22}, {NULL}};
const Segment SGI55090[] = {{&S93282[30], 6}, {NULL}};
const Segment SGI55091[] = {{&S93282[114], 3}, {NULL}};
const Segment SGI55092[] = {{&S93282[80], 5}, {NULL}};
const Segment SGI55093[] = {{&S93041[44], 14}, {NULL}};
const Segment SGI55094[] = {{&S102253[44], 13}, {NULL}};
const Segment SGI55095[] = {{&S93041[44], 35}, {NULL}};
const Segment SGI55096[] = {{&S102253[44], 30}, {NULL}};
const Segment SGI55097[] = {{&S93041[30], 6}, {NULL}};
const Segment SGI55098[] = {{&S102253[30], 6}, {NULL}};
const Segment SGI55099[] = {{&S102253[122], 3}, {NULL}};
const Segment SGI55100[] = {{&S93041[93], 6}, {NULL}};
const Segment SGI55101[] = {{&S102253[88], 5}, {NULL}};
const Segment SGI55102[] = {{&S93041[120], 13}, {NULL}};
const Segment SGI55103[] = {{&S93309[45], 7}, {NULL}};
const Segment SGI55104[] = {{&S93309[30], 6}, {NULL}};
const Segment SGI55105[] = {{&S93309[58], 3}, {NULL}};
const Segment SGI55106[] = {{&S93309[84], 5}, {NULL}};
const Segment SGI55107[] = {{&S93309[142], 13}, {NULL}};
const Segment SGI55108[] = {{&S93076[44], 10}, {NULL}};
const Segment SGI55109[] = {{&S93076[44], 21}, {NULL}};
const Segment SGI55110[] = {{&S93076[30], 6}, {NULL}};
const Segment SGI55111[] = {{&S93076[113], 3}, {NULL}};
const Segment SGI55112[] = {{&S93076[79], 5}, {NULL}};
const Segment SGI55113[] = {{&S93123[58], 5}, {NULL}};
const Segment SGI55114[] = {{&S93123[58], 16}, {NULL}};
const Segment SGI55115[] = {{&S93123[44], 6}, {NULL}};
const Segment SGI55116[] = {{&S93123[122], 3}, {NULL}};
const Segment SGI55117[] = {{&S93123[88], 5}, {NULL}};
const Segment SGI55118[] = {{&S93124[58], 10}, {NULL}};
const Segment SGI55119[] = {{&S93124[58], 21}, {NULL}};
const Segment SGI55120[] = {{&S93124[44], 6}, {NULL}};
const Segment SGI55121[] = {{&S93124[127], 3}, {NULL}};
const Segment SGI55122[] = {{&S93124[93], 5}, {NULL}};
const Segment SGI55123[] = {{&S93134[59], 8}, {NULL}};
const Segment SGI55124[] = {{&S93134[59], 29}, {NULL}};
const Segment SGI55125[] = {{&S93134[45], 6}, {NULL}};
const Segment SGI55126[] = {{&S93134[136], 3}, {NULL}};
const Segment SGI55127[] = {{&S93134[102], 5}, {NULL}};
const Segment SGI55128[] = {{&S93134[0], 14}, {NULL}};
const Segment SGI55129[] = {{&S93173[44], 14}, {NULL}};
const Segment SGI55130[] = {{&S93173[44], 35}, {NULL}};
const Segment SGI55131[] = {{&S93173[30], 6}, {NULL}};
const Segment SGI55132[] = {{&S93173[128], 3}, {NULL}};
const Segment SGI55133[] = {{&S93173[93], 6}, {NULL}};
const Segment SGI55134[] = {{&S93173[148], 8}, {NULL}};
const Segment SGI55135[] = {{&S93173[157], 13}, {NULL}};
const Segment SGI55136[] = {{&S96146[44], 10}, {NULL}};
const Segment SGI55137[] = {{&S96146[44], 27}, {NULL}};
const Segment SGI55138[] = {{&S96146[30], 6}, {NULL}};
const Segment SGI55139[] = {{&S96146[118], 3}, {NULL}};
const Segment SGI55140[] = {{&S96146[85], 5}, {NULL}};
const Segment SGI55141[] = {{&S93196[44], 10}, {NULL}};
const Segment SGI55142[] = {{&S93196[44], 22}, {NULL}};
const Segment SGI55143[] = {{&S93196[30], 6}, {NULL}};
const Segment SGI55144[] = {{&S93196[115], 3}, {NULL}};
const Segment SGI55145[] = {{&S93196[80], 6}, {NULL}};
const Segment SGI55146[] = {{&S93196[135], 8}, {NULL}};
const Segment SGI55147[] = {{&S93196[144], 13}, {NULL}};
const Segment SGI55148[] = {{&S93247[44], 13}, {NULL}};
const Segment SGI55149[] = {{&S93247[44], 30}, {NULL}};
const Segment SGI55150[] = {{&S93247[30], 6}, {NULL}};
const Segment SGI55151[] = {{&S93247[88], 6}, {NULL}};
const Segment SGI55152[] = {{&S93247[115], 13}, {NULL}};
const Segment SGI55153[] = {{&S93544[30], 6}, {NULL}};
const Segment SGI55154[] = {{&S93544[91], 3}, {NULL}};
const Segment SGI55155[] = {{&S93544[125], 3}, {NULL}};
const Segment SGI55156[] = {{&S93544[57], 5}, {NULL}};
const Segment SGI55157[] = {{&S93557[44], 10}, {NULL}};
const Segment SGI55158[] = {{&S93557[44], 26}, {NULL}};
const Segment SGI55159[] = {{&S93557[30], 6}, {NULL}};
const Segment SGI55160[] = {{&S93557[125], 3}, {NULL}};
const Segment SGI55161[] = {{&S93557[91], 5}, {NULL}};
const Segment SGI55162[] = {{&S93557[70], 8}, {NULL}};
const Segment SGI55163[] = {{&S93561[45], 7}, {NULL}};
const Segment SGI55164[] = {{&S93561[30], 6}, {NULL}};
const Segment SGI55165[] = {{&S93561[58], 3}, {NULL}};
const Segment SGI55166[] = {{&S93561[84], 5}, {NULL}};
const Segment SGI55167[] = {{&S93561[142], 13}, {NULL}};
const Segment SGI55168[] = {{&S93563[44], 17}, {NULL}};
const Segment SGI55169[] = {{&S93563[44], 35}, {NULL}};
const Segment SGI55170[] = {{&S93563[30], 6}, {NULL}};
const Segment SGI55171[] = {{&S93563[67], 3}, {NULL}};
const Segment SGI55172[] = {{&S93563[93], 5}, {NULL}};
const Segment SGI55173[] = {{&S93564[44], 5}, {NULL}};
const Segment SGI55174[] = {{&S93564[44], 23}, {NULL}};
const Segment SGI55175[] = {{&S93564[30], 6}, {NULL}};
const Segment SGI55176[] = {{&S93564[84], 6}, {NULL}};
const Segment SGI55177[] = {{&S93569[45], 5}, {NULL}};
const Segment SGI55178[] = {{&S93569[30], 6}, {NULL}};
const Segment SGI55179[] = {{&S93569[109], 3}, {NULL}};
const Segment SGI55180[] = {{&S93569[75], 5}, {NULL}};
const Segment SGI55181[] = {{&S93569[133], 13}, {NULL}};
const Segment SGI55182[] = {{&S93327[44], 6}, {NULL}};
const Segment SGI55183[] = {{&S93327[44], 17}, {NULL}};
const Segment SGI55184[] = {{&S93327[30], 6}, {NULL}};
const Segment SGI55185[] = {{&S93327[109], 3}, {NULL}};
const Segment SGI55186[] = {{&S93327[75], 5}, {NULL}};
const Segment SGI55187[] = {{&S93329[58], 18}, {NULL}};
const Segment SGI55188[] = {{&S93329[58], 35}, {NULL}};
const Segment SGI55189[] = {{&S93329[44], 6}, {NULL}};
const Segment SGI55190[] = {{&S93329[141], 3}, {NULL}};
const Segment SGI55191[] = {{&S93329[107], 5}, {NULL}};
const Segment SGI55192[] = {{&S96233[44], 6}, {NULL}};
const Segment SGI55193[] = {{&S96233[44], 24}, {NULL}};
const Segment SGI55194[] = {{&S96233[30], 6}, {NULL}};
const Segment SGI55195[] = {{&S96233[82], 6}, {NULL}};
const Segment SGI55196[] = {{&S96233[119], 3}, {NULL}};
const Segment SGI55197[] = {{&S96260[44], 10}, {NULL}};
const Segment SGI55198[] = {{&S96260[44], 27}, {NULL}};
const Segment SGI55199[] = {{&S96260[30], 6}, {NULL}};
const Segment SGI55200[] = {{&S96260[119], 3}, {NULL}};
const Segment SGI55201[] = {{&S96260[85], 5}, {NULL}};
const Segment SGI55202[] = {{&S96260[31], 3}, {NULL}};
const Segment SGI55203[] = {{&S93618[45], 7}, {NULL}};
const Segment SGI55204[] = {{&S93618[30], 6}, {NULL}};
const Segment SGI55205[] = {{&S93618[58], 3}, {NULL}};
const Segment SGI55206[] = {{&S93618[84], 5}, {NULL}};
const Segment SGI55207[] = {{&S93618[142], 13}, {NULL}};
const Segment SGI55208[] = {{&S93383[57], 9}, {NULL}};
const Segment SGI55209[] = {{&S93383[57], 20}, {NULL}};
const Segment SGI55210[] = {{&S93383[43], 6}, {NULL}};
const Segment SGI55211[] = {{&S93383[125], 3}, {NULL}};
const Segment SGI55212[] = {{&S93383[91], 5}, {NULL}};
const Segment SGI55213[] = {{&S93383[0], 12}, {NULL}};
const Segment SGI55214[] = {{&S93541[44], 10}, {NULL}};
const Segment SGI55215[] = {{&S93541[44], 27}, {NULL}};
const Segment SGI55216[] = {{&S93541[30], 6}, {NULL}};
const Segment SGI55217[] = {{&S93541[8], 3}, {NULL}};
const Segment SGI55218[] = {{&S93541[85], 5}, {NULL}};
const Segment SGI55219[] = {{&S93541[137], 6}, {NULL}};
const Segment SGI55220[] = {{&S93546[30], 6}, {NULL}};
const Segment SGI55221[] = {{&S93546[91], 3}, {NULL}};
const Segment SGI55222[] = {{&S93546[125], 3}, {NULL}};
const Segment SGI55223[] = {{&S93546[57], 5}, {NULL}};
const Segment SGI55224[] = {{&S93565[44], 14}, {NULL}};
const Segment SGI55225[] = {{&S93565[44], 25}, {NULL}};
const Segment SGI55226[] = {{&S93565[30], 6}, {NULL}};
const Segment SGI55227[] = {{&S93807[44], 12}, {NULL}};
const Segment SGI55228[] = {{&S93807[44], 67}, {NULL}};
const Segment SGI55229[] = {{&S93807[30], 6}, {NULL}};
const Segment SGI55230[] = {{&S93807[159], 3}, {NULL}};
const Segment SGI55231[] = {{&S93807[125], 5}, {NULL}};
const Segment SGI55232[] = {{&S96450[44], 10}, {NULL}};
const Segment SGI55233[] = {{&S96450[44], 27}, {NULL}};
const Segment SGI55234[] = {{&S96450[30], 6}, {NULL}};
const Segment SGI55235[] = {{&S96450[119], 3}, {NULL}};
const Segment SGI55236[] = {{&S96450[85], 5}, {NULL}};
const Segment SGI55237[] = {{&S93893[44], 8}, {NULL}};
const Segment SGI55238[] = {{&S93893[44], 29}, {NULL}};
const Segment SGI55239[] = {{&S93893[30], 6}, {NULL}};
const Segment SGI55240[] = {{&S93893[121], 3}, {NULL}};
const Segment SGI55241[] = {{&S93893[87], 5}, {NULL}};
const Segment SGI55242[] = {{&S93893[144], 10}, {NULL}};
const Segment SGI55243[] = {{&S93565[124], 3}, {NULL}};
const Segment SGI55244[] = {{&S93565[90], 5}, {NULL}};
const Segment SGI55245[] = {{&S93565[69], 8}, {NULL}};
const Segment SGI55246[] = {{&S93566[44], 14}, {NULL}};
const Segment SGI55247[] = {{&S93566[44], 25}, {NULL}};
const Segment SGI55248[] = {{&S93566[30], 6}, {NULL}};
const Segment SGI55249[] = {{&S93566[117], 3}, {NULL}};
const Segment SGI55250[] = {{&S93566[83], 5}, {NULL}};
const Segment SGI55251[] = {{&S93566[141], 7}, {NULL}};
const Segment SGI55252[] = {{&S93575[45], 6}, {NULL}};
const Segment SGI55253[] = {{&S93575[30], 6}, {NULL}};
const Segment SGI55254[] = {{&S93575[116], 3}, {NULL}};
const Segment SGI55255[] = {{&S93575[82], 5}, {NULL}};
const Segment SGI55256[] = {{&S93575[140], 13}, {NULL}};
const Segment SGI55257[] = {{&S96645[44], 8}, {NULL}};
const Segment SGI55258[] = {{&S96645[44], 19}, {NULL}};
const Segment SGI55259[] = {{&S96645[30], 6}, {NULL}};
const Segment SGI55260[] = {{&S96645[111], 3}, {NULL}};
const Segment SGI55261[] = {{&S96645[77], 5}, {NULL}};
const Segment SGI55262[] = {{&S94024[58], 25}, {NULL}};
const Segment SGI55263[] = {{&S94024[44], 6}, {NULL}};
const Segment SGI55264[] = {{&S94024[149], 3}, {NULL}};
const Segment SGI55265[] = {{&S94024[115], 5}, {NULL}};
const Segment SGI55266[] = {{&S94026[58], 11}, {NULL}};
const Segment SGI55267[] = {{&S94026[58], 22}, {NULL}};
const Segment SGI55268[] = {{&S94026[44], 6}, {NULL}};
const Segment SGI55269[] = {{&S94026[128], 3}, {NULL}};
const Segment SGI55270[] = {{&S94026[94], 5}, {NULL}};
const Segment SGI55271[] = {{&S93675[44], 14}, {NULL}};
const Segment SGI55272[] = {{&S93675[44], 35}, {NULL}};
const Segment SGI55273[] = {{&S93675[30], 6}, {NULL}};
const Segment SGI55274[] = {{&S93675[127], 3}, {NULL}};
const Segment SGI55275[] = {{&S93675[93], 5}, {NULL}};
const Segment SGI55276[] = {{&S94035[44], 13}, {NULL}};
const Segment SGI55277[] = {{&S94035[44], 24}, {NULL}};
const Segment SGI55278[] = {{&S94035[30], 6}, {NULL}};
const Segment SGI55279[] = {{&S94035[116], 3}, {NULL}};
const Segment SGI55280[] = {{&S94035[82], 5}, {NULL}};
const Segment SGI55281[] = {{&S96680[59], 6}, {NULL}};
const Segment SGI55282[] = {{&S96680[59], 24}, {NULL}};
const Segment SGI55283[] = {{&S96680[45], 6}, {NULL}};
const Segment SGI55284[] = {{&S96680[131], 3}, {NULL}};
const Segment SGI55285[] = {{&S96680[97], 5}, {NULL}};
const Segment SGI55286[] = {{&S96680[0], 12}, {NULL}};
const Segment SGI55287[] = {{&S94091[45], 5}, {NULL}};
const Segment SGI55288[] = {{&S94091[30], 6}, {NULL}};
const Segment SGI55289[] = {{&S94091[109], 3}, {NULL}};
const Segment SGI55290[] = {{&S94091[75], 5}, {NULL}};
const Segment SGI55291[] = {{&S94091[133], 13}, {NULL}};
const Segment SGI55292[] = {{&S93760[44], 12}, {NULL}};
const Segment SGI55293[] = {{&S93760[44], 22}, {NULL}};
const Segment SGI55294[] = {{&S93760[30], 6}, {NULL}};
const Segment SGI55295[] = {{&S93760[114], 3}, {NULL}};
const Segment SGI55296[] = {{&S93760[80], 5}, {NULL}};
const Segment SGI55297[] = {{&S94154[44], 9}, {NULL}};
const Segment SGI55298[] = {{&S94154[44], 20}, {NULL}};
const Segment SGI55299[] = {{&S94154[30], 6}, {NULL}};
const Segment SGI55300[] = {{&S94154[78], 6}, {NULL}};
const Segment SGI55301[] = {{&S94154[105], 13}, {NULL}};
const Segment SGI55302[] = {{&S93821[45], 20}, {NULL}};
const Segment SGI55303[] = {{&S93821[30], 6}, {NULL}};
const Segment SGI55304[] = {{&S93821[124], 3}, {NULL}};
const Segment SGI55305[] = {{&S93821[90], 5}, {NULL}};
const Segment SGI55306[] = {{&S93821[148], 13}, {NULL}};
const Segment SGI55307[] = {{&S93822[44], 20}, {NULL}};
const Segment SGI55308[] = {{&S93822[44], 31}, {NULL}};
const Segment SGI55309[] = {{&S93822[30], 6}, {NULL}};
const Segment SGI55310[] = {{&S93822[124], 3}, {NULL}};
const Segment SGI55311[] = {{&S93822[89], 6}, {NULL}};
const Segment SGI55312[] = {{&S93822[144], 8}, {NULL}};
const Segment SGI55313[] = {{&S93822[153], 13}, {NULL}};
const Segment SGI55314[] = {{&S93831[44], 13}, {NULL}};
const Segment SGI55315[] = {{&S93831[44], 30}, {NULL}};
const Segment SGI55316[] = {{&S93831[30], 6}, {NULL}};
const Segment SGI55317[] = {{&S93831[121], 3}, {NULL}};
const Segment SGI55318[] = {{&S93831[88], 5}, {NULL}};
const Segment SGI55319[] = {{&S93831[125], 14}, {NULL}};
const Segment SGI55320[] = {{&S96821[34], 10}, {NULL}};
const Segment SGI55321[] = {{&S96821[27], 6}, {NULL}};
const Segment SGI55322[] = {{&S96821[109], 2}, {NULL}};
const Segment SGI55323[] = {{&S96821[75], 6}, {NULL}};
const Segment SGI55324[] = {{&S96821[143], 8}, {NULL}};
const Segment SGI55325[] = {{&S93897[59], 12}, {NULL}};
const Segment SGI55326[] = {{&S93897[59], 33}, {NULL}};
const Segment SGI55327[] = {{&S93897[45], 6}, {NULL}};
const Segment SGI55328[] = {{&S93897[140], 3}, {NULL}};
const Segment SGI55329[] = {{&S93897[106], 5}, {NULL}};
const Segment SGI55330[] = {{&S93897[0], 14}, {NULL}};
const Segment SGI55331[] = {{&S96890[44], 10}, {NULL}};
const Segment SGI55332[] = {{&S96890[44], 27}, {NULL}};
const Segment SGI55333[] = {{&S96890[30], 6}, {NULL}};
const Segment SGI55334[] = {{&S96890[85], 6}, {NULL}};
const Segment SGI55335[] = {{&S96890[122], 3}, {NULL}};
const Segment SGI55336[] = {{&S93926[44], 11}, {NULL}};
const Segment SGI55337[] = {{&S93926[44], 32}, {NULL}};
const Segment SGI55338[] = {{&S93926[30], 6}, {NULL}};
const Segment SGI55339[] = {{&S93926[124], 3}, {NULL}};
const Segment SGI55340[] = {{&S93926[90], 5}, {NULL}};
const Segment SGI55341[] = {{&S96952[31], 16}, {NULL}};
const Segment SGI55342[] = {{&S96952[62], 3}, {NULL}};
const Segment SGI55343[] = {{&S94227[44], 17}, {NULL}};
const Segment SGI55344[] = {{&S94227[44], 28}, {NULL}};
const Segment SGI55345[] = {{&S94227[30], 6}, {NULL}};
const Segment SGI55346[] = {{&S94227[120], 3}, {NULL}};
const Segment SGI55347[] = {{&S94227[86], 5}, {NULL}};
const Segment SGI55348[] = {{&S94228[30], 6}, {NULL}};
const Segment SGI55349[] = {{&S94228[100], 3}, {NULL}};
const Segment SGI55350[] = {{&S94228[66], 5}, {NULL}};
const Segment SGI55351[] = {{&S96978[45], 9}, {NULL}};
const Segment SGI55352[] = {{&S96978[30], 6}, {NULL}};
const Segment SGI55353[] = {{&S96978[113], 3}, {NULL}};
const Segment SGI55354[] = {{&S96978[79], 5}, {NULL}};
const Segment SGI55355[] = {{&S96978[147], 3}, {NULL}};
const Segment SGI55356[] = {{&S96984[44], 12}, {NULL}};
const Segment SGI55357[] = {{&S96984[44], 23}, {NULL}};
const Segment SGI55358[] = {{&S96984[30], 6}, {NULL}};
const Segment SGI55359[] = {{&S96984[115], 3}, {NULL}};
const Segment SGI55360[] = {{&S96984[81], 5}, {NULL}};
const Segment SGI55361[] = {{&S94016[38], 7}, {NULL}};
const Segment SGI55362[] = {{&S94016[31], 6}, {NULL}};
const Segment SGI55363[] = {{&S94017[44], 7}, {NULL}};
const Segment SGI55364[] = {{&S94017[44], 15}, {NULL}};
const Segment SGI55365[] = {{&S94017[30], 6}, {NULL}};
const Segment SGI55366[] = {{&S94017[107], 3}, {NULL}};
const Segment SGI55367[] = {{&S94017[73], 5}, {NULL}};
const Segment SGI55368[] = {{&S94025[58], 25}, {NULL}};
const Segment SGI55369[] = {{&S94025[44], 6}, {NULL}};
const Segment SGI55370[] = {{&S94025[149], 3}, {NULL}};
const Segment SGI55371[] = {{&S94025[115], 5}, {NULL}};
const Segment SGI55372[] = {{&S97013[59], 10}, {NULL}};
const Segment SGI55373[] = {{&S97013[59], 27}, {NULL}};
const Segment SGI55374[] = {{&S97013[45], 6}, {NULL}};
const Segment SGI55375[] = {{&S97013[134], 3}, {NULL}};
const Segment SGI55376[] = {{&S97013[100], 5}, {NULL}};
const Segment SGI55377[] = {{&S97013[0], 12}, {NULL}};
const Segment SGI55378[] = {{&S97041[59], 10}, {NULL}};
const Segment SGI55379[] = {{&S97041[59], 27}, {NULL}};
const Segment SGI55380[] = {{&S97041[45], 6}, {NULL}};
const Segment SGI55381[] = {{&S97041[134], 3}, {NULL}};
const Segment SGI55382[] = {{&S97041[100], 5}, {NULL}};
const Segment SGI55383[] = {{&S97041[0], 14}, {NULL}};
const Segment SGI55384[] = {{&S97042[59], 11}, {NULL}};
const Segment SGI55385[] = {{&S97042[59], 28}, {NULL}};
const Segment SGI55386[] = {{&S97042[45], 6}, {NULL}};
const Segment SGI55387[] = {{&S97042[135], 3}, {NULL}};
const Segment SGI55388[] = {{&S97042[101], 5}, {NULL}};
const Segment SGI55389[] = {{&S97042[0], 14}, {NULL}};
const Segment SGI55390[] = {{&S94314[59], 10}, {NULL}};
const Segment SGI55391[] = {{&S94314[59], 27}, {NULL}};
const Segment SGI55392[] = {{&S94314[45], 6}, {NULL}};
const Segment SGI55393[] = {{&S94314[134], 3}, {NULL}};
const Segment SGI55394[] = {{&S94314[100], 5}, {NULL}};
const Segment SGI55395[] = {{&S94314[0], 14}, {NULL}};
const Segment SGI55396[] = {{&S94172[44], 14}, {NULL}};
const Segment SGI55397[] = {{&S94172[44], 35}, {NULL}};
const Segment SGI55398[] = {{&S94172[30], 6}, {NULL}};
const Segment SGI55399[] = {{&S94172[93], 6}, {NULL}};
const Segment SGI55400[] = {{&S94172[120], 13}, {NULL}};
const Segment SGI55401[] = {{&S94447[44], 12}, {NULL}};
const Segment SGI55402[] = {{&S94447[44], 29}, {NULL}};
const Segment SGI55403[] = {{&S94447[30], 6}, {NULL}};
const Segment SGI55404[] = {{&S94447[122], 3}, {NULL}};
const Segment SGI55405[] = {{&S94447[87], 6}, {NULL}};
const Segment SGI55406[] = {{&S94448[57], 10}, {NULL}};
const Segment SGI55407[] = {{&S94448[57], 27}, {NULL}};
const Segment SGI55408[] = {{&S94448[43], 6}, {NULL}};
const Segment SGI55409[] = {{&S94448[132], 3}, {NULL}};
const Segment SGI55410[] = {{&S94448[98], 5}, {NULL}};
const Segment SGI55411[] = {{&S94448[0], 12}, {NULL}};
const Segment SGI55412[] = {{&S94460[45], 9}, {NULL}};
const Segment SGI55413[] = {{&S94460[30], 6}, {NULL}};
const Segment SGI55414[] = {{&S94460[113], 3}, {NULL}};
const Segment SGI55415[] = {{&S94460[79], 5}, {NULL}};
const Segment SGI55416[] = {{&S94232[45], 14}, {NULL}};
const Segment SGI55417[] = {{&S94232[30], 6}, {NULL}};
const Segment SGI55418[] = {{&S94460[137], 13}, {NULL}};
const Segment SGI55419[] = {{&S94232[118], 3}, {NULL}};
const Segment SGI55420[] = {{&S94232[84], 5}, {NULL}};
const Segment SGI55421[] = {{&S94232[142], 13}, {NULL}};
const Segment SGI55422[] = {{&S94474[45], 9}, {NULL}};
const Segment SGI55423[] = {{&S94474[30], 6}, {NULL}};
const Segment SGI55424[] = {{&S94474[113], 3}, {NULL}};
const Segment SGI55425[] = {{&S94474[79], 5}, {NULL}};
const Segment SGI55426[] = {{&S94474[137], 13}, {NULL}};
const Segment SGI55427[] = {{&S94486[45], 5}, {NULL}};
const Segment SGI55428[] = {{&S94486[30], 6}, {NULL}};
const Segment SGI55429[] = {{&S94486[109], 3}, {NULL}};
const Segment SGI55430[] = {{&S94486[75], 5}, {NULL}};
const Segment SGI55431[] = {{&S94486[133], 13}, {NULL}};
const Segment SGI55432[] = {{&S94499[45], 7}, {NULL}};
const Segment SGI55433[] = {{&S94499[30], 6}, {NULL}};
const Segment SGI55434[] = {{&S94499[58], 3}, {NULL}};
const Segment SGI55435[] = {{&S94499[84], 5}, {NULL}};
const Segment SGI55436[] = {{&S94499[142], 13}, {NULL}};
const Segment SGI55437[] = {{&S94281[44], 11}, {NULL}};
const Segment SGI55438[] = {{&S94281[44], 22}, {NULL}};
const Segment SGI55439[] = {{&S94281[30], 6}, {NULL}};
const Segment SGI55440[] = {{&S94281[114], 3}, {NULL}};
const Segment SGI55441[] = {{&S94281[80], 5}, {NULL}};
const Segment SGI55442[] = {{&S94281[137], 10}, {NULL}};
const Segment SGI55443[] = {{&S94532[34], 13}, {NULL}};
const Segment SGI55444[] = {{&S94532[27], 6}, {NULL}};
const Segment SGI55445[] = {{&S94532[106], 2}, {NULL}};
const Segment SGI55446[] = {{&S94532[72], 6}, {NULL}};
const Segment SGI55447[] = {{&S94532[140], 8}, {NULL}};
const Segment SGI55448[] = {{&S97303[44], 7}, {NULL}};
const Segment SGI55449[] = {{&S97303[44], 18}, {NULL}};
const Segment SGI55450[] = {{&S97303[30], 6}, {NULL}};
const Segment SGI55451[] = {{&S97303[110], 3}, {NULL}};
const Segment SGI55452[] = {{&S97303[76], 5}, {NULL}};
const Segment SGI55453[] = {{&S97304[44], 10}, {NULL}};
const Segment SGI55454[] = {{&S97304[44], 21}, {NULL}};
const Segment SGI55455[] = {{&S97304[30], 6}, {NULL}};
const Segment SGI55456[] = {{&S97304[113], 3}, {NULL}};
const Segment SGI55457[] = {{&S97304[79], 5}, {NULL}};
const Segment SGI55458[] = {{&S97305[44], 10}, {NULL}};
const Segment SGI55459[] = {{&S97305[44], 21}, {NULL}};
const Segment SGI55460[] = {{&S97305[30], 6}, {NULL}};
const Segment SGI55461[] = {{&S97305[113], 3}, {NULL}};
const Segment SGI55462[] = {{&S97305[79], 5}, {NULL}};
const Segment SGI55463[] = {{&S94559[44], 13}, {NULL}};
const Segment SGI55464[] = {{&S94559[44], 24}, {NULL}};
const Segment SGI55465[] = {{&S94559[30], 6}, {NULL}};
const Segment SGI55466[] = {{&S94559[82], 6}, {NULL}};
const Segment SGI55467[] = {{&S94559[109], 13}, {NULL}};
const Segment SGI55468[] = {{&S94351[52], 9}, {NULL}};
const Segment SGI55469[] = {{&S94351[52], 20}, {NULL}};
const Segment SGI55470[] = {{&S94351[38], 6}, {NULL}};
const Segment SGI55471[] = {{&S94351[86], 5}, {NULL}};
const Segment SGI55472[] = {{&S94351[0], 14}, {NULL}};
const Segment SGI55473[] = {{&S94354[44], 14}, {NULL}};
const Segment SGI55474[] = {{&S94572[44], 7}, {NULL}};
const Segment SGI55475[] = {{&S94354[44], 35}, {NULL}};
const Segment SGI55476[] = {{&S94572[44], 25}, {NULL}};
const Segment SGI55477[] = {{&S94354[30], 6}, {NULL}};
const Segment SGI55478[] = {{&S94572[30], 6}, {NULL}};
const Segment SGI55479[] = {{&S94354[128], 3}, {NULL}};
const Segment SGI55480[] = {{&S94354[93], 6}, {NULL}};
const Segment SGI55481[] = {{&S94572[83], 6}, {NULL}};
const Segment SGI55482[] = {{&S94354[148], 8}, {NULL}};
const Segment SGI55483[] = {{&S94572[110], 13}, {NULL}};
const Segment SGI55484[] = {{&S94354[157], 13}, {NULL}};
const Segment SGI55485[] = {{&S94356[44], 14}, {NULL}};
const Segment SGI55486[] = {{&S94356[44], 35}, {NULL}};
const Segment SGI55487[] = {{&S94356[30], 6}, {NULL}};
const Segment SGI55488[] = {{&S94356[128], 3}, {NULL}};
const Segment SGI55489[] = {{&S94356[93], 6}, {NULL}};
const Segment SGI55490[] = {{&S94356[148], 8}, {NULL}};
const Segment SGI55491[] = {{&S94356[157], 13}, {NULL}};
const Segment SGI55492[] = {{&S94671[30], 6}, {NULL}};
const Segment SGI55493[] = {{&S94671[91], 3}, {NULL}};
const Segment SGI55494[] = {{&S94671[125], 3}, {NULL}};
const Segment SGI55495[] = {{&S94671[57], 5}, {NULL}};
const Segment SGI55496[] = {{&S94451[59], 18}, {NULL}};
const Segment SGI55497[] = {{&S94451[59], 29}, {NULL}};
const Segment SGI55498[] = {{&S94451[45], 6}, {NULL}};
const Segment SGI55499[] = {{&S94451[136], 3}, {NULL}};
const Segment SGI55500[] = {{&S94451[102], 5}, {NULL}};
const Segment SGI55501[] = {{&S94451[0], 14}, {NULL}};
const Segment SGI55502[] = {{&S94453[59], 12}, {NULL}};
const Segment SGI55503[] = {{&S94453[59], 23}, {NULL}};
const Segment SGI55504[] = {{&S94453[45], 6}, {NULL}};
const Segment SGI55505[] = {{&S94453[130], 3}, {NULL}};
const Segment SGI55506[] = {{&S94453[96], 5}, {NULL}};
const Segment SGI55507[] = {{&S94453[0], 14}, {NULL}};
const Segment SGI55508[] = {{&S94464[31], 12}, {NULL}};
const Segment SGI55509[] = {{&S94464[58], 3}, {NULL}};
const Segment SGI55510[] = {{&S94500[45], 18}, {NULL}};
const Segment SGI55511[] = {{&S94500[30], 6}, {NULL}};
const Segment SGI55512[] = {{&S94500[122], 3}, {NULL}};
const Segment SGI55513[] = {{&S94500[88], 5}, {NULL}};
const Segment SGI55514[] = {{&S94500[146], 13}, {NULL}};
const Segment SGI55515[] = {{&S94509[44], 14}, {NULL}};
const Segment SGI55516[] = {{&S94509[44], 25}, {NULL}};
const Segment SGI55517[] = {{&S94509[30], 6}, {NULL}};
const Segment SGI55518[] = {{&S94509[117], 3}, {NULL}};
const Segment SGI55519[] = {{&S94509[83], 5}, {NULL}};
const Segment SGI55520[] = {{&S94509[141], 11}, {NULL}};
const Segment SGI55521[] = {{&S97352[45], 9}, {NULL}};
const Segment SGI55522[] = {{&S97352[30], 6}, {NULL}};
const Segment SGI55523[] = {{&S97352[113], 3}, {NULL}};
const Segment SGI55524[] = {{&S97352[79], 5}, {NULL}};
const Segment SGI55525[] = {{&S97352[147], 3}, {NULL}};
const Segment SGI55526[] = {{&S94582[44], 14}, {NULL}};
const Segment SGI55527[] = {{&S94582[44], 35}, {NULL}};
const Segment SGI55528[] = {{&S94582[30], 6}, {NULL}};
const Segment SGI55529[] = {{&S94582[128], 3}, {NULL}};
const Segment SGI55530[] = {{&S94582[93], 6}, {NULL}};
const Segment SGI55531[] = {{&S94582[148], 8}, {NULL}};
const Segment SGI55532[] = {{&S94582[157], 13}, {NULL}};
const Segment SGI55533[] = {{&S97439[44], 7}, {NULL}};
const Segment SGI55534[] = {{&S97439[44], 24}, {NULL}};
const Segment SGI55535[] = {{&S97439[30], 6}, {NULL}};
const Segment SGI55536[] = {{&S97439[82], 6}, {NULL}};
const Segment SGI55537[] = {{&S97439[119], 3}, {NULL}};
const Segment SGI55538[] = {{&S94673[30], 6}, {NULL}};
const Segment SGI55539[] = {{&S94673[91], 3}, {NULL}};
const Segment SGI55540[] = {{&S94673[125], 3}, {NULL}};
const Segment SGI55541[] = {{&S94673[57], 5}, {NULL}};
const Segment SGI55542[] = {{&S94675[44], 6}, {NULL}};
const Segment SGI55543[] = {{&S94675[44], 17}, {NULL}};
const Segment SGI55544[] = {{&S94675[30], 6}, {NULL}};
const Segment SGI55545[] = {{&S94675[109], 3}, {NULL}};
const Segment SGI55546[] = {{&S94675[75], 5}, {NULL}};
const Segment SGI55547[] = {{&S103385[44], 10}, {NULL}};
const Segment SGI55548[] = {{&S103385[44], 21}, {NULL}};
const Segment SGI55549[] = {{&S103385[30], 6}, {NULL}};
const Segment SGI55550[] = {{&S103385[113], 3}, {NULL}};
const Segment SGI55551[] = {{&S103385[79], 5}, {NULL}};
const Segment SGI55552[] = {{&S94813[44], 9}, {NULL}};
const Segment SGI55553[] = {{&S94813[44], 20}, {NULL}};
const Segment SGI55554[] = {{&S94813[30], 6}, {NULL}};
const Segment SGI55555[] = {{&S94813[78], 6}, {NULL}};
const Segment SGI55556[] = {{&S94813[105], 13}, {NULL}};
const Segment SGI55557[] = {{&S94695[57], 20}, {NULL}};
const Segment SGI55558[] = {{&S94695[57], 31}, {NULL}};
const Segment SGI55559[] = {{&S94695[43], 6}, {NULL}};
const Segment SGI55560[] = {{&S94695[136], 3}, {NULL}};
const Segment SGI55561[] = {{&S94695[102], 5}, {NULL}};
const Segment SGI55562[] = {{&S94695[0], 12}, {NULL}};
const Segment SGI55563[] = {{&S97646[58], 12}, {NULL}};
const Segment SGI55564[] = {{&S97646[58], 23}, {NULL}};
const Segment SGI55565[] = {{&S97646[44], 6}, {NULL}};
const Segment SGI55566[] = {{&S97646[129], 3}, {NULL}};
const Segment SGI55567[] = {{&S97646[95], 5}, {NULL}};
const Segment SGI55568[] = {{&S94718[59], 20}, {NULL}};
const Segment SGI55569[] = {{&S94718[59], 31}, {NULL}};
const Segment SGI55570[] = {{&S94718[45], 6}, {NULL}};
const Segment SGI55571[] = {{&S94718[138], 3}, {NULL}};
const Segment SGI55572[] = {{&S94718[104], 5}, {NULL}};
const Segment SGI55573[] = {{&S94718[0], 14}, {NULL}};
const Segment SGI55574[] = {{&S94860[44], 14}, {NULL}};
const Segment SGI55575[] = {{&S94860[44], 35}, {NULL}};
const Segment SGI55576[] = {{&S94860[30], 6}, {NULL}};
const Segment SGI55577[] = {{&S94860[128], 3}, {NULL}};
const Segment SGI55578[] = {{&S94860[93], 6}, {NULL}};
const Segment SGI55579[] = {{&S94860[148], 8}, {NULL}};
const Segment SGI55580[] = {{&S94860[157], 13}, {NULL}};
const Segment SGI55581[] = {{&S94938[59], 14}, {NULL}};
const Segment SGI55582[] = {{&S94938[59], 25}, {NULL}};
const Segment SGI55583[] = {{&S94938[45], 6}, {NULL}};
const Segment SGI55584[] = {{&S94938[132], 3}, {NULL}};
const Segment SGI55585[] = {{&S94938[98], 5}, {NULL}};
const Segment SGI55586[] = {{&S94938[0], 14}, {NULL}};
const Segment SGI55587[] = {{&S95066[52], 7}, {NULL}};
const Segment SGI55588[] = {{&S95066[52], 25}, {NULL}};
const Segment SGI55589[] = {{&S95066[38], 6}, {NULL}};
const Segment SGI55590[] = {{&S95066[91], 5}, {NULL}};
const Segment SGI55591[] = {{&S95066[0], 14}, {NULL}};
const Segment SGI55592[] = {{&S94980[45], 9}, {NULL}};
const Segment SGI55593[] = {{&S94980[30], 6}, {NULL}};
const Segment SGI55594[] = {{&S94980[113], 3}, {NULL}};
const Segment SGI55595[] = {{&S94980[79], 5}, {NULL}};
const Segment SGI55596[] = {{&S94980[137], 13}, {NULL}};
const Segment SGI55597[] = {{&S97878[44], 18}, {NULL}};
const Segment SGI55598[] = {{&S97878[44], 29}, {NULL}};
const Segment SGI55599[] = {{&S97878[30], 6}, {NULL}};
const Segment SGI55600[] = {{&S97878[120], 3}, {NULL}};
const Segment SGI55601[] = {{&S97878[87], 5}, {NULL}};
const Segment SGI55602[] = {{&S97878[124], 14}, {NULL}};
const Segment SGI55603[] = {{&S95207[59], 7}, {NULL}};
const Segment SGI55604[] = {{&S95207[59], 25}, {NULL}};
const Segment SGI55605[] = {{&S95207[45], 6}, {NULL}};
const Segment SGI55606[] = {{&S95207[72], 3}, {NULL}};
const Segment SGI55607[] = {{&S95207[98], 5}, {NULL}};
const Segment SGI55608[] = {{&S95207[0], 14}, {NULL}};
const Segment SGI55609[] = {{&S95061[44], 14}, {NULL}};
const Segment SGI55610[] = {{&S95061[44], 25}, {NULL}};
const Segment SGI55611[] = {{&S95061[30], 6}, {NULL}};
const Segment SGI55612[] = {{&S95061[117], 3}, {NULL}};
const Segment SGI55613[] = {{&S95061[83], 5}, {NULL}};
const Segment SGI55614[] = {{&S95061[141], 7}, {NULL}};
const Segment SGI55615[] = {{&S95227[45], 5}, {NULL}};
const Segment SGI55616[] = {{&S95227[30], 6}, {NULL}};
const Segment SGI55617[] = {{&S95227[109], 3}, {NULL}};
const Segment SGI55618[] = {{&S95227[75], 5}, {NULL}};
const Segment SGI55619[] = {{&S95227[133], 13}, {NULL}};
const Segment SGI55620[] = {{&S95230[45], 9}, {NULL}};
const Segment SGI55621[] = {{&S95230[30], 6}, {NULL}};
const Segment SGI55622[] = {{&S95230[113], 3}, {NULL}};
const Segment SGI55623[] = {{&S95230[79], 5}, {NULL}};
const Segment SGI55624[] = {{&S95230[137], 13}, {NULL}};
const Segment SGI55625[] = {{&S95243[45], 7}, {NULL}};
const Segment SGI55626[] = {{&S95243[30], 6}, {NULL}};
const Segment SGI55627[] = {{&S95243[58], 3}, {NULL}};
const Segment SGI55628[] = {{&S95243[84], 5}, {NULL}};
const Segment SGI55629[] = {{&S95243[142], 13}, {NULL}};
const Segment SGI55630[] = {{&S97982[44], 12}, {NULL}};
const Segment SGI55631[] = {{&S97982[44], 23}, {NULL}};
const Segment SGI55632[] = {{&S97982[30], 6}, {NULL}};
const Segment SGI55633[] = {{&S97982[81], 6}, {NULL}};
const Segment SGI55634[] = {{&S97982[118], 3}, {NULL}};
const Segment SGI55635[] = {{&S95156[44], 20}, {NULL}};
const Segment SGI55636[] = {{&S95156[44], 31}, {NULL}};
const Segment SGI55637[] = {{&S95156[30], 6}, {NULL}};
const Segment SGI55638[] = {{&S95156[123], 3}, {NULL}};
const Segment SGI55639[] = {{&S95156[89], 5}, {NULL}};
const Segment SGI55640[] = {{&S98005[45], 7}, {NULL}};
const Segment SGI55641[] = {{&S98005[30], 6}, {NULL}};
const Segment SGI55642[] = {{&S98005[111], 3}, {NULL}};
const Segment SGI55643[] = {{&S98005[77], 5}, {NULL}};
const Segment SGI55644[] = {{&S98005[145], 3}, {NULL}};
const Segment SGI55645[] = {{&S95191[45], 10}, {NULL}};
const Segment SGI55646[] = {{&S95191[30], 6}, {NULL}};
const Segment SGI55647[] = {{&S95191[120], 3}, {NULL}};
const Segment SGI55648[] = {{&S95191[86], 5}, {NULL}};
const Segment SGI55649[] = {{&S95191[144], 13}, {NULL}};
const Segment SGI55650[] = {{&S103997[42], 9}, {NULL}};
const Segment SGI55651[] = {{&S103997[42], 26}, {NULL}};
const Segment SGI55652[] = {{&S103997[30], 4}, {NULL}};
const Segment SGI55653[] = {{&S103997[116], 3}, {NULL}};
const Segment SGI55654[] = {{&S103997[82], 5}, {NULL}};
const Segment SGI55655[] = {{&S95217[44], 6}, {NULL}};
const Segment SGI55656[] = {{&S95217[44], 17}, {NULL}};
const Segment SGI55657[] = {{&S95217[30], 6}, {NULL}};
const Segment SGI55658[] = {{&S95217[109], 3}, {NULL}};
const Segment SGI55659[] = {{&S95217[75], 5}, {NULL}};
const Segment SGI55660[] = {{&S95279[44], 14}, {NULL}};
const Segment SGI55661[] = {{&S95279[44], 35}, {NULL}};
const Segment SGI55662[] = {{&S95279[30], 6}, {NULL}};
const Segment SGI55663[] = {{&S95279[128], 3}, {NULL}};
const Segment SGI55664[] = {{&S95279[93], 6}, {NULL}};
const Segment SGI55665[] = {{&S95315[44], 10}, {NULL}};
const Segment SGI55666[] = {{&S95279[148], 8}, {NULL}};
const Segment SGI55667[] = {{&S95315[44], 27}, {NULL}};
const Segment SGI55668[] = {{&S95315[30], 6}, {NULL}};
const Segment SGI55669[] = {{&S95279[157], 13}, {NULL}};
const Segment SGI55670[] = {{&S95315[118], 3}, {NULL}};
const Segment SGI55671[] = {{&S95315[85], 5}, {NULL}};
const Segment SGI55672[] = {{&S95315[122], 14}, {NULL}};
const Segment SGI55673[] = {{&S98127[44], 10}, {NULL}};
const Segment SGI55674[] = {{&S98127[44], 27}, {NULL}};
const Segment SGI55675[] = {{&S98127[30], 6}, {NULL}};
const Segment SGI55676[] = {{&S98127[119], 3}, {NULL}};
const Segment SGI55677[] = {{&S98127[85], 5}, {NULL}};
const Segment SGI55678[] = {{&S98127[153], 3}, {NULL}};
const Segment SGI55679[] = {{&S95356[44], 7}, {NULL}};
const Segment SGI55680[] = {{&S95356[44], 25}, {NULL}};
const Segment SGI55681[] = {{&S95356[30], 6}, {NULL}};
const Segment SGI55682[] = {{&S95356[83], 6}, {NULL}};
const Segment SGI55683[] = {{&S95356[110], 13}, {NULL}};
const Segment SGI55684[] = {{&S95320[44], 14}, {NULL}};
const Segment SGI55685[] = {{&S95320[44], 35}, {NULL}};
const Segment SGI55686[] = {{&S95320[30], 6}, {NULL}};
const Segment SGI55687[] = {{&S95320[93], 6}, {NULL}};
const Segment SGI55688[] = {{&S95320[120], 13}, {NULL}};
const Segment SGI55689[] = {{&S95322[44], 14}, {NULL}};
const Segment SGI55690[] = {{&S95322[44], 35}, {NULL}};
const Segment SGI55691[] = {{&S95322[30], 6}, {NULL}};
const Segment SGI55692[] = {{&S95322[126], 3}, {NULL}};
const Segment SGI55693[] = {{&S95322[93], 5}, {NULL}};
const Segment SGI55694[] = {{&S95322[130], 14}, {NULL}};
const Segment SGI55695[] = {{&S95406[44], 9}, {NULL}};
const Segment SGI55696[] = {{&S95406[44], 20}, {NULL}};
const Segment SGI55697[] = {{&S95406[30], 6}, {NULL}};
const Segment SGI55698[] = {{&S95406[112], 3}, {NULL}};
const Segment SGI55699[] = {{&S95406[78], 5}, {NULL}};
const Segment SGI55700[] = {{&S95429[44], 9}, {NULL}};
const Segment SGI55701[] = {{&S95429[30], 6}, {NULL}};
const Segment SGI55702[] = {{&S95429[114], 3}, {NULL}};
const Segment SGI55703[] = {{&S95429[80], 5}, {NULL}};
const Segment SGI55704[] = {{&S95430[31], 6}, {NULL}};
const Segment SGI55705[] = {{&S95431[44], 9}, {NULL}};
const Segment SGI55706[] = {{&S95431[30], 6}, {NULL}};
const Segment SGI55707[] = {{&S95431[83], 4}, {NULL}};
const Segment SGI55708[] = {{&S95431[83], 3}, {NULL}};
const Segment SGI55709[] = {{&S95437[45], 7}, {NULL}};
const Segment SGI55710[] = {{&S95437[30], 6}, {NULL}};
const Segment SGI55711[] = {{&S95437[58], 3}, {NULL}};
const Segment SGI55712[] = {{&S95437[84], 5}, {NULL}};
const Segment SGI55713[] = {{&S95437[142], 13}, {NULL}};
const Segment SGI55714[] = {{&S95447[45], 9}, {NULL}};
const Segment SGI55715[] = {{&S95447[30], 6}, {NULL}};
const Segment SGI55716[] = {{&S95447[113], 3}, {NULL}};
const Segment SGI55717[] = {{&S95447[79], 5}, {NULL}};
const Segment SGI55718[] = {{&S95447[137], 13}, {NULL}};
const Segment SGI55719[] = {{&S95448[44], 11}, {NULL}};
const Segment SGI55720[] = {{&S95448[44], 22}, {NULL}};
const Segment SGI55721[] = {{&S95448[30], 6}, {NULL}};
const Segment SGI55722[] = {{&S95448[114], 3}, {NULL}};
const Segment SGI55723[] = {{&S95448[80], 5}, {NULL}};
const Segment SGI55724[] = {{&S95452[44], 10}, {NULL}};
const Segment SGI55725[] = {{&S95452[44], 27}, {NULL}};
const Segment SGI55726[] = {{&S95452[30], 6}, {NULL}};
const Segment SGI55727[] = {{&S95452[149], 4}, {NULL}};
const Segment SGI55728[] = {{&S95452[175], 3}, {NULL}};
const Segment SGI55729[] = {{&S95452[119], 3}, {&S95452[224], 5}, {NULL}};
const Segment SGI55730[] = {{&S95452[85], 5}, {NULL}};
const Segment SGI55731[] = {{&S95436[59], 10}, {NULL}};
const Segment SGI55732[] = {{&S95436[59], 27}, {NULL}};
const Segment SGI55733[] = {{&S95436[45], 6}, {NULL}};
const Segment SGI55734[] = {{&S95436[134], 3}, {NULL}};
const Segment SGI55735[] = {{&S95436[100], 5}, {NULL}};
const Segment SGI55736[] = {{&S95436[0], 14}, {NULL}};
const Segment SGI55737[] = {{&S95468[45], 14}, {NULL}};
const Segment SGI55738[] = {{&S95468[30], 6}, {NULL}};
const Segment SGI55739[] = {{&S95468[124], 3}, {NULL}};
const Segment SGI55740[] = {{&S95468[90], 5}, {NULL}};
const Segment SGI55741[] = {{&S95468[148], 13}, {NULL}};
const Segment SGI55742[] = {{&S95492[59], 14}, {NULL}};
const Segment SGI55743[] = {{&S95492[59], 35}, {NULL}};
const Segment SGI55744[] = {{&S95492[45], 6}, {NULL}};
const Segment SGI55745[] = {{&S95492[142], 3}, {NULL}};
const Segment SGI55746[] = {{&S95492[108], 5}, {NULL}};
const Segment SGI55747[] = {{&S95492[0], 14}, {NULL}};
const Segment SGI55748[] = {{&S95494[45], 17}, {NULL}};
const Segment SGI55749[] = {{&S95494[30], 6}, {NULL}};
const Segment SGI55750[] = {{&S95494[121], 3}, {NULL}};
const Segment SGI55751[] = {{&S95494[87], 5}, {NULL}};
const Segment SGI55752[] = {{&S95494[145], 13}, {NULL}};
const Segment SGI55753[] = {{&S95550[44], 7}, {NULL}};
const Segment SGI55754[] = {{&S95550[44], 25}, {NULL}};
const Segment SGI55755[] = {{&S95550[30], 6}, {NULL}};
const Segment SGI55756[] = {{&S95550[57], 3}, {NULL}};
const Segment SGI55757[] = {{&S95550[83], 5}, {NULL}};
const Segment SGI55758[] = {{&S95550[120], 14}, {NULL}};
const Segment SGI55759[] = {{&S95564[44], 14}, {NULL}};
const Segment SGI55760[] = {{&S95564[44], 35}, {NULL}};
const Segment SGI55761[] = {{&S95564[30], 6}, {NULL}};
const Segment SGI55762[] = {{&S95564[93], 6}, {NULL}};
const Segment SGI55763[] = {{&S95564[120], 13}, {NULL}};
const Segment SGI55764[] = {{&S104331[44], 24}, {NULL}};
const Segment SGI55765[] = {{&S104331[44], 35}, {NULL}};
const Segment SGI55766[] = {{&S104331[30], 6}, {NULL}};
const Segment SGI55767[] = {{&S104331[127], 3}, {NULL}};
const Segment SGI55768[] = {{&S104331[93], 5}, {NULL}};
const Segment SGI55769[] = {{&S95569[44], 16}, {NULL}};
const Segment SGI55770[] = {{&S95569[44], 27}, {NULL}};
const Segment SGI55771[] = {{&S95569[30], 6}, {NULL}};
const Segment SGI55772[] = {{&S95569[119], 3}, {NULL}};
const Segment SGI55773[] = {{&S95569[85], 5}, {NULL}};
const Segment SGI55774[] = {{&S95569[143], 11}, {NULL}};
const Segment SGI55775[] = {{&S98145[44], 11}, {NULL}};
const Segment SGI55776[] = {{&S98145[44], 22}, {NULL}};
const Segment SGI55777[] = {{&S98145[30], 6}, {NULL}};
const Segment SGI55778[] = {{&S98145[114], 3}, {NULL}};
const Segment SGI55779[] = {{&S98145[80], 5}, {NULL}};
const Segment SGI55780[] = {{&S95579[34], 9}, {NULL}};
const Segment SGI55781[] = {{&S95579[27], 6}, {NULL}};
const Segment SGI55782[] = {{&S95579[102], 2}, {NULL}};
const Segment SGI55783[] = {{&S95579[68], 6}, {NULL}};
const Segment SGI55784[] = {{&S95579[136], 8}, {NULL}};
const Segment SGI55785[] = {{&S95621[44], 16}, {NULL}};
const Segment SGI55786[] = {{&S95621[44], 37}, {NULL}};
const Segment SGI55787[] = {{&S95621[30], 6}, {NULL}};
const Segment SGI55788[] = {{&S95621[130], 3}, {NULL}};
const Segment SGI55789[] = {{&S95621[95], 6}, {NULL}};
const Segment SGI55790[] = {{&S95621[150], 8}, {NULL}};
const Segment SGI55791[] = {{&S95621[159], 13}, {NULL}};
const Segment SGI55792[] = {{&S95642[44], 14}, {NULL}};
const Segment SGI55793[] = {{&S95642[44], 35}, {NULL}};
const Segment SGI55794[] = {{&S95642[30], 6}, {NULL}};
const Segment SGI55795[] = {{&S95642[128], 3}, {NULL}};
const Segment SGI55796[] = {{&S95642[93], 6}, {NULL}};
const Segment SGI55797[] = {{&S95642[148], 8}, {NULL}};
const Segment SGI55798[] = {{&S95642[157], 13}, {NULL}};
const Segment SGI55799[] = {{&S95649[44], 22}, {NULL}};
const Segment SGI55800[] = {{&S95649[44], 33}, {NULL}};
const Segment SGI55801[] = {{&S95649[30], 6}, {NULL}};
const Segment SGI55802[] = {{&S95649[126], 3}, {NULL}};
const Segment SGI55803[] = {{&S95649[91], 6}, {NULL}};
const Segment SGI55804[] = {{&S95649[146], 8}, {NULL}};
const Segment SGI55805[] = {{&S95649[155], 13}, {NULL}};
const Segment SGI55806[] = {{&S95784[59], 7}, {NULL}};
const Segment SGI55807[] = {{&S95784[59], 25}, {NULL}};
const Segment SGI55808[] = {{&S95784[45], 6}, {NULL}};
const Segment SGI55809[] = {{&S95784[72], 3}, {NULL}};
const Segment SGI55810[] = {{&S95784[98], 5}, {NULL}};
const Segment SGI55811[] = {{&S95784[0], 14}, {NULL}};
const Segment SGI55812[] = {{&S95726[44], 8}, {NULL}};
const Segment SGI55813[] = {{&S95726[44], 19}, {NULL}};
const Segment SGI55814[] = {{&S95726[30], 6}, {NULL}};
const Segment SGI55815[] = {{&S95726[111], 3}, {NULL}};
const Segment SGI55816[] = {{&S95726[77], 5}, {NULL}};
const Segment SGI55817[] = {{&S95730[58], 10}, {NULL}};
const Segment SGI55818[] = {{&S95730[58], 27}, {NULL}};
const Segment SGI55819[] = {{&S95730[44], 6}, {NULL}};
const Segment SGI55820[] = {{&S95730[133], 3}, {NULL}};
const Segment SGI55821[] = {{&S95730[99], 5}, {NULL}};
const Segment SGI55822[] = {{&S95737[44], 20}, {NULL}};
const Segment SGI55823[] = {{&S95737[44], 31}, {NULL}};
const Segment SGI55824[] = {{&S95737[30], 6}, {NULL}};
const Segment SGI55825[] = {{&S95737[130], 3}, {NULL}};
const Segment SGI55826[] = {{&S95737[96], 5}, {NULL}};
const Segment SGI55827[] = {{&S95737[75], 8}, {NULL}};
const Segment SGI55828[] = {{&S95738[44], 20}, {NULL}};
const Segment SGI55829[] = {{&S95738[44], 31}, {NULL}};
const Segment SGI55830[] = {{&S95738[30], 6}, {NULL}};
const Segment SGI55831[] = {{&S95738[123], 3}, {NULL}};
const Segment SGI55832[] = {{&S95738[89], 5}, {NULL}};
const Segment SGI55833[] = {{&S95738[147], 7}, {NULL}};
const Segment SGI55834[] = {{&S95739[59], 11}, {NULL}};
const Segment SGI55835[] = {{&S95739[59], 28}, {NULL}};
const Segment SGI55836[] = {{&S95739[45], 6}, {NULL}};
const Segment SGI55837[] = {{&S95739[135], 3}, {NULL}};
const Segment SGI55838[] = {{&S95739[101], 5}, {NULL}};
const Segment SGI55839[] = {{&S95739[0], 14}, {NULL}};
const Segment SGI55840[] = {{&S95740[59], 10}, {NULL}};
const Segment SGI55841[] = {{&S95740[59], 27}, {NULL}};
const Segment SGI55842[] = {{&S95740[45], 6}, {NULL}};
const Segment SGI55843[] = {{&S95740[134], 3}, {NULL}};
const Segment SGI55844[] = {{&S95740[100], 5}, {NULL}};
const Segment SGI55845[] = {{&S95740[0], 14}, {NULL}};
const Segment SGI55846[] = {{&S95769[45], 14}, {NULL}};
const Segment SGI55847[] = {{&S95769[30], 6}, {NULL}};
const Segment SGI55848[] = {{&S95769[124], 3}, {NULL}};
const Segment SGI55849[] = {{&S95769[90], 5}, {NULL}};
const Segment SGI55850[] = {{&S95769[148], 13}, {NULL}};
const Segment SGI55851[] = {{&S95794[45], 11}, {NULL}};
const Segment SGI55852[] = {{&S95794[30], 6}, {NULL}};
const Segment SGI55853[] = {{&S95794[115], 3}, {NULL}};
const Segment SGI55854[] = {{&S95794[81], 5}, {NULL}};
const Segment SGI55855[] = {{&S95794[139], 13}, {NULL}};
const Segment SGI55856[] = {{&S95800[44], 14}, {NULL}};
const Segment SGI55857[] = {{&S95800[44], 35}, {NULL}};
const Segment SGI55858[] = {{&S95800[30], 6}, {NULL}};
const Segment SGI55859[] = {{&S95800[126], 3}, {NULL}};
const Segment SGI55860[] = {{&S95800[93], 5}, {NULL}};
const Segment SGI55861[] = {{&S95800[130], 14}, {NULL}};
const Segment SGI55862[] = {{&S95869[44], 14}, {NULL}};
const Segment SGI55863[] = {{&S95869[44], 35}, {NULL}};
const Segment SGI55864[] = {{&S95869[30], 6}, {NULL}};
const Segment SGI55865[] = {{&S95869[126], 3}, {NULL}};
const Segment SGI55866[] = {{&S95869[93], 5}, {NULL}};
const Segment SGI55867[] = {{&S95869[130], 14}, {NULL}};
const Segment SGI55868[] = {{&S95927[44], 14}, {NULL}};
const Segment SGI55869[] = {{&S95927[44], 31}, {NULL}};
const Segment SGI55870[] = {{&S95927[30], 6}, {NULL}};
const Segment SGI55871[] = {{&S95927[124], 3}, {NULL}};
const Segment SGI55872[] = {{&S95927[89], 6}, {NULL}};
const Segment SGI55873[] = {{&S95927[144], 8}, {NULL}};
const Segment SGI55874[] = {{&S95927[153], 13}, {NULL}};
const Segment SGI55875[] = {{&S95938[44], 10}, {NULL}};
const Segment SGI55876[] = {{&S95938[44], 22}, {NULL}};
const Segment SGI55877[] = {{&S95938[30], 6}, {NULL}};
const Segment SGI55878[] = {{&S95938[80], 6}, {NULL}};
const Segment SGI55879[] = {{&S95938[107], 13}, {NULL}};
const Segment SGI55880[] = {{&S96020[44], 14}, {NULL}};
const Segment SGI55881[] = {{&S96020[44], 37}, {NULL}};
const Segment SGI55882[] = {{&S96020[30], 6}, {NULL}};
const Segment SGI55883[] = {{&S96020[129], 3}, {NULL}};
const Segment SGI55884[] = {{&S96020[95], 5}, {NULL}};
const Segment SGI55885[] = {{&S96020[163], 3}, {NULL}};
const Segment SGI55886[] = {{&S96026[44], 16}, {NULL}};
const Segment SGI55887[] = {{&S96026[44], 29}, {NULL}};
const Segment SGI55888[] = {{&S96026[30], 6}, {NULL}};
const Segment SGI55889[] = {{&S96026[121], 3}, {NULL}};
const Segment SGI55890[] = {{&S96026[87], 5}, {NULL}};
const Segment SGI55891[] = {{&S96036[59], 7}, {NULL}};
const Segment SGI55892[] = {{&S96036[59], 25}, {NULL}};
const Segment SGI55893[] = {{&S96036[45], 6}, {NULL}};
const Segment SGI55894[] = {{&S96036[72], 3}, {NULL}};
const Segment SGI55895[] = {{&S96036[98], 5}, {NULL}};
const Segment SGI55896[] = {{&S96036[0], 14}, {NULL}};
const Segment SGI55897[] = {{&S96045[59], 5}, {NULL}};
const Segment SGI55898[] = {{&S96045[59], 16}, {NULL}};
const Segment SGI55899[] = {{&S96045[45], 6}, {NULL}};
const Segment SGI55900[] = {{&S96045[123], 3}, {NULL}};
const Segment SGI55901[] = {{&S96045[89], 5}, {NULL}};
const Segment SGI55902[] = {{&S96045[0], 14}, {NULL}};
const Segment SGI55903[] = {{&S96049[59], 7}, {NULL}};
const Segment SGI55904[] = {{&S96049[59], 25}, {NULL}};
const Segment SGI55905[] = {{&S96049[45], 6}, {NULL}};
const Segment SGI55906[] = {{&S96049[72], 3}, {NULL}};
const Segment SGI55907[] = {{&S96049[98], 5}, {NULL}};
const Segment SGI55908[] = {{&S96049[0], 14}, {NULL}};
const Segment SGI55909[] = {{&S96058[45], 9}, {NULL}};
const Segment SGI55910[] = {{&S96058[30], 6}, {NULL}};
const Segment SGI55911[] = {{&S96058[113], 3}, {NULL}};
const Segment SGI55912[] = {{&S96058[79], 5}, {NULL}};
const Segment SGI55913[] = {{&S96058[137], 13}, {NULL}};
const Segment SGI55914[] = {{&S98596[57], 10}, {NULL}};
const Segment SGI55915[] = {{&S98596[57], 27}, {NULL}};
const Segment SGI55916[] = {{&S98596[43], 6}, {NULL}};
const Segment SGI55917[] = {{&S98596[132], 3}, {NULL}};
const Segment SGI55918[] = {{&S98596[98], 5}, {NULL}};
const Segment SGI55919[] = {{&S98596[0], 10}, {NULL}};
const Segment SGI55920[] = {{&S96024[44], 16}, {NULL}};
const Segment SGI55921[] = {{&S96024[44], 29}, {NULL}};
const Segment SGI55922[] = {{&S96024[30], 6}, {NULL}};
const Segment SGI55923[] = {{&S96024[121], 3}, {NULL}};
const Segment SGI55924[] = {{&S96024[87], 5}, {NULL}};
const Segment SGI55925[] = {{&S96025[44], 16}, {NULL}};
const Segment SGI55926[] = {{&S96025[44], 29}, {NULL}};
const Segment SGI55927[] = {{&S96025[30], 6}, {NULL}};
const Segment SGI55928[] = {{&S96025[121], 3}, {NULL}};
const Segment SGI55929[] = {{&S96025[87], 5}, {NULL}};
const Segment SGI55930[] = {{&S104829[66], 19}, {NULL}};
const Segment SGI55931[] = {{&S104829[66], 36}, {NULL}};
const Segment SGI55932[] = {{&S104829[54], 4}, {NULL}};
const Segment SGI55933[] = {{&S104829[150], 3}, {NULL}};
const Segment SGI55934[] = {{&S104829[116], 5}, {NULL}};
const Segment SGI55935[] = {{&S96105[45], 9}, {NULL}};
const Segment SGI55936[] = {{&S96105[30], 6}, {NULL}};
const Segment SGI55937[] = {{&S96105[113], 3}, {NULL}};
const Segment SGI55938[] = {{&S96105[79], 5}, {NULL}};
const Segment SGI55939[] = {{&S96105[137], 13}, {NULL}};
const Segment SGI55940[] = {{&S96111[45], 13}, {NULL}};
const Segment SGI55941[] = {{&S96111[30], 6}, {NULL}};
const Segment SGI55942[] = {{&S96111[123], 3}, {NULL}};
const Segment SGI55943[] = {{&S96111[89], 5}, {NULL}};
const Segment SGI55944[] = {{&S96111[147], 13}, {NULL}};
const Segment SGI55945[] = {{&S98647[44], 10}, {NULL}};
const Segment SGI55946[] = {{&S98647[44], 27}, {NULL}};
const Segment SGI55947[] = {{&S98647[30], 6}, {NULL}};
const Segment SGI55948[] = {{&S98647[85], 6}, {NULL}};
const Segment SGI55949[] = {{&S98647[122], 3}, {NULL}};
const Segment SGI55950[] = {{&S98649[44], 11}, {NULL}};
const Segment SGI55951[] = {{&S98649[44], 28}, {NULL}};
const Segment SGI55952[] = {{&S98649[30], 6}, {NULL}};
const Segment SGI55953[] = {{&S98649[86], 6}, {NULL}};
const Segment SGI55954[] = {{&S98649[123], 3}, {NULL}};
const Segment SGI55955[] = {{&S96130[44], 6}, {NULL}};
const Segment SGI55956[] = {{&S96130[44], 17}, {NULL}};
const Segment SGI55957[] = {{&S96130[30], 6}, {NULL}};
const Segment SGI55958[] = {{&S96130[109], 3}, {NULL}};
const Segment SGI55959[] = {{&S96130[75], 5}, {NULL}};
const Segment SGI55960[] = {{&S96098[59], 13}, {NULL}};
const Segment SGI55961[] = {{&S96098[59], 24}, {NULL}};
const Segment SGI55962[] = {{&S96098[45], 6}, {NULL}};
const Segment SGI55963[] = {{&S96098[131], 3}, {NULL}};
const Segment SGI55964[] = {{&S96098[97], 5}, {NULL}};
const Segment SGI55965[] = {{&S96098[0], 14}, {NULL}};
const Segment SGI55966[] = {{&S96106[44], 14}, {NULL}};
const Segment SGI55967[] = {{&S96106[44], 35}, {NULL}};
const Segment SGI55968[] = {{&S96106[30], 6}, {NULL}};
const Segment SGI55969[] = {{&S96106[93], 6}, {NULL}};
const Segment SGI55970[] = {{&S96106[120], 13}, {NULL}};
const Segment SGI55971[] = {{&S4597[44], 10}, {NULL}};
const Segment SGI55972[] = {{&S4597[44], 27}, {NULL}};
const Segment SGI55973[] = {{&S4597[30], 6}, {NULL}};
const Segment SGI55974[] = {{&S4597[119], 3}, {NULL}};
const Segment SGI55975[] = {{&S4597[85], 5}, {NULL}};
const Segment SGI55976[] = {{&S96129[44], 13}, {NULL}};
const Segment SGI55977[] = {{&S96129[44], 30}, {NULL}};
const Segment SGI55978[] = {{&S96129[30], 6}, {NULL}};
const Segment SGI55979[] = {{&S96191[44], 7}, {NULL}};
const Segment SGI55980[] = {{&S96191[44], 25}, {NULL}};
const Segment SGI55981[] = {{&S96129[121], 3}, {NULL}};
const Segment SGI55982[] = {{&S96191[30], 6}, {NULL}};
const Segment SGI55983[] = {{&S96129[88], 5}, {NULL}};
const Segment SGI55984[] = {{&S96129[125], 14}, {NULL}};
const Segment SGI55985[] = {{&S96191[83], 6}, {NULL}};
const Segment SGI55986[] = {{&S96191[110], 13}, {NULL}};
const Segment SGI55987[] = {{&S4613[44], 11}, {NULL}};
const Segment SGI55988[] = {{&S4613[44], 28}, {NULL}};
const Segment SGI55989[] = {{&S4613[30], 6}, {NULL}};
const Segment SGI55990[] = {{&S4613[120], 3}, {NULL}};
const Segment SGI55991[] = {{&S4613[86], 5}, {NULL}};
const Segment SGI55992[] = {{&S4615[38], 10}, {NULL}};
const Segment SGI55993[] = {{&S4615[31], 6}, {NULL}};
const Segment SGI55994[] = {{&S4616[44], 11}, {NULL}};
const Segment SGI55995[] = {{&S4616[44], 28}, {NULL}};
const Segment SGI55996[] = {{&S4616[30], 6}, {NULL}};
const Segment SGI55997[] = {{&S4616[120], 3}, {NULL}};
const Segment SGI55998[] = {{&S4616[86], 5}, {NULL}};
const Segment SGI55999[] = {{&S4617[38], 11}, {NULL}};
const Segment SGI56000[] = {{&S4617[31], 6}, {NULL}};
const Segment SGI56001[] = {{&S4618[44], 28}, {NULL}};
const Segment SGI56002[] = {{&S4618[44], 45}, {NULL}};
const Segment SGI56003[] = {{&S4618[30], 6}, {NULL}};
const Segment SGI56004[] = {{&S4618[137], 3}, {NULL}};
const Segment SGI56005[] = {{&S4618[103], 5}, {NULL}};
const Segment SGI56006[] = {{&S4623[44], 10}, {NULL}};
const Segment SGI56007[] = {{&S4623[44], 22}, {NULL}};
const Segment SGI56008[] = {{&S4623[30], 6}, {NULL}};
const Segment SGI56009[] = {{&S4623[114], 3}, {NULL}};
const Segment SGI56010[] = {{&S4623[80], 5}, {NULL}};
const Segment SGI56011[] = {{&S4631[44], 10}, {NULL}};
const Segment SGI56012[] = {{&S4631[44], 27}, {NULL}};
const Segment SGI56013[] = {{&S4631[30], 6}, {NULL}};
const Segment SGI56014[] = {{&S4631[119], 3}, {NULL}};
const Segment SGI56015[] = {{&S4631[85], 5}, {NULL}};
const Segment SGI56016[] = {{&S4633[44], 10}, {NULL}};
const Segment SGI56017[] = {{&S4633[44], 27}, {NULL}};
const Segment SGI56018[] = {{&S4633[30], 6}, {NULL}};
const Segment SGI56019[] = {{&S4633[88], 4}, {NULL}};
const Segment SGI56020[] = {{&S4633[88], 3}, {NULL}};
const Segment SGI56021[] = {{&S4634[44], 10}, {NULL}};
const Segment SGI56022[] = {{&S4634[44], 27}, {NULL}};
const Segment SGI56023[] = {{&S4634[30], 6}, {NULL}};
const Segment SGI56024[] = {{&S4634[88], 4}, {NULL}};
const Segment SGI56025[] = {{&S4634[88], 3}, {NULL}};
const Segment SGI56026[] = {{&S4638[44], 11}, {NULL}};
const Segment SGI56027[] = {{&S4638[44], 28}, {NULL}};
const Segment SGI56028[] = {{&S4638[30], 6}, {NULL}};
const Segment SGI56029[] = {{&S4638[89], 4}, {NULL}};
const Segment SGI56030[] = {{&S4638[89], 3}, {NULL}};
const Segment SGI56031[] = {{&S4644[44], 11}, {NULL}};
const Segment SGI56032[] = {{&S4644[44], 28}, {NULL}};
const Segment SGI56033[] = {{&S4644[30], 6}, {NULL}};
const Segment SGI56034[] = {{&S4644[89], 6}, {NULL}};
const Segment SGI56035[] = {{&S4651[70], 10}, {NULL}};
const Segment SGI56036[] = {{&S4651[70], 27}, {NULL}};
const Segment SGI56037[] = {{&S4651[30], 32}, {NULL}};
const Segment SGI56038[] = {{&S4651[145], 3}, {NULL}};
const Segment SGI56039[] = {{&S4651[111], 5}, {NULL}};
const Segment SGI56040[] = {{&S98779[34], 9}, {NULL}};
const Segment SGI56041[] = {{&S98779[27], 6}, {NULL}};
const Segment SGI56042[] = {{&S98779[102], 2}, {NULL}};
const Segment SGI56043[] = {{&S96255[44], 7}, {NULL}};
const Segment SGI56044[] = {{&S98779[68], 6}, {NULL}};
const Segment SGI56045[] = {{&S96255[44], 25}, {NULL}};
const Segment SGI56046[] = {{&S96255[30], 6}, {NULL}};
const Segment SGI56047[] = {{&S98779[136], 8}, {NULL}};
const Segment SGI56048[] = {{&S96255[83], 6}, {NULL}};
const Segment SGI56049[] = {{&S96255[110], 13}, {NULL}};
const Segment SGI56050[] = {{&S4654[44], 10}, {NULL}};
const Segment SGI56051[] = {{&S4654[44], 27}, {NULL}};
const Segment SGI56052[] = {{&S4654[30], 6}, {NULL}};
const Segment SGI56053[] = {{&S4654[88], 6}, {NULL}};
const Segment SGI56054[] = {{&S4655[44], 10}, {NULL}};
const Segment SGI56055[] = {{&S4655[44], 27}, {NULL}};
const Segment SGI56056[] = {{&S4655[30], 6}, {NULL}};
const Segment SGI56057[] = {{&S98783[44], 12}, {NULL}};
const Segment SGI56058[] = {{&S98783[44], 23}, {NULL}};
const Segment SGI56059[] = {{&S4655[88], 6}, {NULL}};
const Segment SGI56060[] = {{&S98783[30], 6}, {NULL}};
const Segment SGI56061[] = {{&S98783[81], 6}, {NULL}};
const Segment SGI56062[] = {{&S98783[118], 3}, {NULL}};
const Segment SGI56063[] = {{&S98784[34], 12}, {NULL}};
const Segment SGI56064[] = {{&S98784[27], 6}, {NULL}};
const Segment SGI56065[] = {{&S98784[105], 2}, {NULL}};
const Segment SGI56066[] = {{&S98784[71], 6}, {NULL}};
const Segment SGI56067[] = {{&S98784[139], 8}, {NULL}};
const Segment SGI56068[] = {{&S4660[58], 10}, {NULL}};
const Segment SGI56069[] = {{&S4660[58], 27}, {NULL}};
const Segment SGI56070[] = {{&S4660[30], 20}, {NULL}};
const Segment SGI56071[] = {{&S4660[133], 3}, {NULL}};
const Segment SGI56072[] = {{&S4660[99], 5}, {NULL}};
const Segment SGI56073[] = {{&S4662[42], 10}, {NULL}};
const Segment SGI56074[] = {{&S4662[30], 6}, {NULL}};
const Segment SGI56075[] = {{&S4662[117], 3}, {NULL}};
const Segment SGI56076[] = {{&S4662[83], 5}, {NULL}};
const Segment SGI56077[] = {{&S4663[44], 10}, {NULL}};
const Segment SGI56078[] = {{&S4663[44], 33}, {NULL}};
const Segment SGI56079[] = {{&S4663[30], 6}, {NULL}};
const Segment SGI56080[] = {{&S4663[94], 4}, {NULL}};
const Segment SGI56081[] = {{&S4663[94], 3}, {NULL}};
const Segment SGI56082[] = {{&S4668[44], 10}, {NULL}};
const Segment SGI56083[] = {{&S4668[44], 33}, {NULL}};
const Segment SGI56084[] = {{&S4668[30], 6}, {NULL}};
const Segment SGI56085[] = {{&S4668[125], 3}, {NULL}};
const Segment SGI56086[] = {{&S4668[91], 5}, {NULL}};
const Segment SGI56087[] = {{&S4670[38], 11}, {NULL}};
const Segment SGI56088[] = {{&S4670[31], 6}, {NULL}};
const Segment SGI56089[] = {{&S4671[38], 10}, {NULL}};
const Segment SGI56090[] = {{&S4671[31], 6}, {NULL}};
const Segment SGI56091[] = {{&S96218[44], 14}, {NULL}};
const Segment SGI56092[] = {{&S96218[44], 31}, {NULL}};
const Segment SGI56093[] = {{&S96218[30], 6}, {NULL}};
const Segment SGI56094[] = {{&S96218[89], 6}, {NULL}};
const Segment SGI56095[] = {{&S96218[116], 13}, {NULL}};
const Segment SGI56096[] = {{&S4675[44], 10}, {NULL}};
const Segment SGI56097[] = {{&S4675[44], 33}, {NULL}};
const Segment SGI56098[] = {{&S4675[30], 6}, {NULL}};
const Segment SGI56099[] = {{&S4675[94], 6}, {NULL}};
const Segment SGI56100[] = {{&S4681[44], 11}, {NULL}};
const Segment SGI56101[] = {{&S4681[44], 28}, {NULL}};
const Segment SGI56102[] = {{&S4681[30], 6}, {NULL}};
const Segment SGI56103[] = {{&S4681[120], 3}, {NULL}};
const Segment SGI56104[] = {{&S4681[86], 5}, {NULL}};
const Segment SGI56105[] = {{&S96228[44], 9}, {NULL}};
const Segment SGI56106[] = {{&S4682[42], 11}, {NULL}};
const Segment SGI56107[] = {{&S96228[44], 20}, {NULL}};
const Segment SGI56108[] = {{&S4682[30], 6}, {NULL}};
const Segment SGI56109[] = {{&S96228[30], 6}, {NULL}};
const Segment SGI56110[] = {{&S4682[118], 3}, {NULL}};
const Segment SGI56111[] = {{&S4682[84], 5}, {NULL}};
const Segment SGI56112[] = {{&S96228[78], 6}, {NULL}};
const Segment SGI56113[] = {{&S96228[105], 13}, {NULL}};
const Segment SGI56114[] = {{&S4686[38], 11}, {NULL}};
const Segment SGI56115[] = {{&S4686[31], 6}, {NULL}};
const Segment SGI56116[] = {{&S98833[44], 14}, {NULL}};
const Segment SGI56117[] = {{&S98833[44], 25}, {NULL}};
const Segment SGI56118[] = {{&S98833[30], 6}, {NULL}};
const Segment SGI56119[] = {{&S98833[116], 3}, {NULL}};
const Segment SGI56120[] = {{&S4696[44], 7}, {NULL}};
const Segment SGI56121[] = {{&S4696[44], 19}, {NULL}};
const Segment SGI56122[] = {{&S4696[30], 6}, {NULL}};
const Segment SGI56123[] = {{&S98833[83], 5}, {NULL}};
const Segment SGI56124[] = {{&S4696[111], 3}, {NULL}};
const Segment SGI56125[] = {{&S98833[120], 14}, {NULL}};
const Segment SGI56126[] = {{&S4696[77], 5}, {NULL}};
const Segment SGI56127[] = {{&S4698[42], 11}, {NULL}};
const Segment SGI56128[] = {{&S4698[30], 6}, {NULL}};
const Segment SGI56129[] = {{&S4698[118], 3}, {NULL}};
const Segment SGI56130[] = {{&S4698[84], 5}, {NULL}};
const Segment SGI56131[] = {{&S96249[44], 13}, {NULL}};
const Segment SGI56132[] = {{&S96249[44], 30}, {NULL}};
const Segment SGI56133[] = {{&S96249[30], 6}, {NULL}};
const Segment SGI56134[] = {{&S96249[121], 3}, {NULL}};
const Segment SGI56135[] = {{&S96249[88], 5}, {NULL}};
const Segment SGI56136[] = {{&S96249[125], 14}, {NULL}};
const Segment SGI56137[] = {{&S96346[57], 10}, {NULL}};
const Segment SGI56138[] = {{&S96346[57], 27}, {NULL}};
const Segment SGI56139[] = {{&S96346[43], 6}, {NULL}};
const Segment SGI56140[] = {{&S96346[132], 3}, {NULL}};
const Segment SGI56141[] = {{&S96346[98], 5}, {NULL}};
const Segment SGI56142[] = {{&S96346[0], 12}, {NULL}};
const Segment SGI56143[] = {{&S96417[45], 9}, {NULL}};
const Segment SGI56144[] = {{&S96417[30], 6}, {NULL}};
const Segment SGI56145[] = {{&S96417[113], 3}, {NULL}};
const Segment SGI56146[] = {{&S96417[79], 5}, {NULL}};
const Segment SGI56147[] = {{&S96417[137], 13}, {NULL}};
const Segment SGI56148[] = {{&S96512[44], 9}, {NULL}};
const Segment SGI56149[] = {{&S96512[44], 20}, {NULL}};
const Segment SGI56150[] = {{&S96512[30], 6}, {NULL}};
const Segment SGI56151[] = {{&S96512[112], 3}, {NULL}};
const Segment SGI56152[] = {{&S96512[78], 5}, {NULL}};
const Segment SGI56153[] = {{&S96512[136], 7}, {NULL}};
const Segment SGI56154[] = {{&S96558[44], 10}, {NULL}};
const Segment SGI56155[] = {{&S96558[44], 21}, {NULL}};
const Segment SGI56156[] = {{&S96558[30], 6}, {NULL}};
const Segment SGI56157[] = {{&S96558[79], 6}, {NULL}};
const Segment SGI56158[] = {{&S96558[106], 13}, {NULL}};
const Segment SGI56159[] = {{&S105772[45], 10}, {NULL}};
const Segment SGI56160[] = {{&S105772[30], 6}, {NULL}};
const Segment SGI56161[] = {{&S105772[115], 3}, {NULL}};
const Segment SGI56162[] = {{&S105772[81], 5}, {NULL}};
const Segment SGI56163[] = {{&S96764[44], 9}, {NULL}};
const Segment SGI56164[] = {{&S96764[44], 20}, {NULL}};
const Segment SGI56165[] = {{&S96764[30], 6}, {NULL}};
const Segment SGI56166[] = {{&S96764[111], 3}, {NULL}};
const Segment SGI56167[] = {{&S96764[78], 5}, {NULL}};
const Segment SGI56168[] = {{&S96764[115], 14}, {NULL}};
const Segment SGI56169[] = {{&S99305[44], 10}, {NULL}};
const Segment SGI56170[] = {{&S99305[44], 27}, {NULL}};
const Segment SGI56171[] = {{&S99305[30], 6}, {NULL}};
const Segment SGI56172[] = {{&S99305[85], 6}, {NULL}};
const Segment SGI56173[] = {{&S99305[122], 3}, {NULL}};
const Segment SGI56174[] = {{&S96643[44], 22}, {NULL}};
const Segment SGI56175[] = {{&S96643[44], 33}, {NULL}};
const Segment SGI56176[] = {{&S96643[30], 6}, {NULL}};
const Segment SGI56177[] = {{&S96643[125], 3}, {NULL}};
const Segment SGI56178[] = {{&S96643[91], 5}, {NULL}};
const Segment SGI56179[] = {{&S96805[44], 18}, {NULL}};
const Segment SGI56180[] = {{&S96805[44], 35}, {NULL}};
const Segment SGI56181[] = {{&S96805[30], 6}, {NULL}};
const Segment SGI56182[] = {{&S96644[44], 22}, {NULL}};
const Segment SGI56183[] = {{&S96644[44], 68}, {NULL}};
const Segment SGI56184[] = {{&S96805[126], 3}, {NULL}};
const Segment SGI56185[] = {{&S96644[30], 6}, {NULL}};
const Segment SGI56186[] = {{&S96805[93], 5}, {NULL}};
const Segment SGI56187[] = {{&S96805[130], 14}, {NULL}};
const Segment SGI56188[] = {{&S96644[160], 3}, {NULL}};
const Segment SGI56189[] = {{&S96644[126], 5}, {NULL}};
const Segment SGI56190[] = {{&S99337[44], 12}, {NULL}};
const Segment SGI56191[] = {{&S99337[44], 23}, {NULL}};
const Segment SGI56192[] = {{&S99337[30], 6}, {NULL}};
const Segment SGI56193[] = {{&S99337[81], 6}, {NULL}};
const Segment SGI56194[] = {{&S99337[118], 3}, {NULL}};
const Segment SGI56195[] = {{&S99358[44], 12}, {NULL}};
const Segment SGI56196[] = {{&S99358[44], 23}, {NULL}};
const Segment SGI56197[] = {{&S99358[30], 6}, {NULL}};
const Segment SGI56198[] = {{&S99358[114], 3}, {NULL}};
const Segment SGI56199[] = {{&S99358[81], 5}, {NULL}};
const Segment SGI56200[] = {{&S99358[118], 14}, {NULL}};
const Segment SGI56201[] = {{&S96681[59], 5}, {NULL}};
const Segment SGI56202[] = {{&S96681[59], 16}, {NULL}};
const Segment SGI56203[] = {{&S96681[45], 6}, {NULL}};
const Segment SGI56204[] = {{&S96681[123], 3}, {NULL}};
const Segment SGI56205[] = {{&S96681[89], 5}, {NULL}};
const Segment SGI56206[] = {{&S96681[0], 14}, {NULL}};
const Segment SGI56207[] = {{&S96735[44], 8}, {NULL}};
const Segment SGI56208[] = {{&S96735[44], 35}, {NULL}};
const Segment SGI56209[] = {{&S96735[30], 6}, {NULL}};
const Segment SGI56210[] = {{&S96735[127], 3}, {NULL}};
const Segment SGI56211[] = {{&S96735[93], 5}, {NULL}};
const Segment SGI56212[] = {{&S96735[151], 11}, {NULL}};
const Segment SGI56213[] = {{&S99437[44], 7}, {NULL}};
const Segment SGI56214[] = {{&S99437[44], 18}, {NULL}};
const Segment SGI56215[] = {{&S99437[30], 6}, {NULL}};
const Segment SGI56216[] = {{&S99437[110], 3}, {NULL}};
const Segment SGI56217[] = {{&S99437[76], 5}, {NULL}};
const Segment SGI56218[] = {{&S99437[134], 7}, {NULL}};
const Segment SGI56219[] = {{&S96951[39], 16}, {NULL}};
const Segment SGI56220[] = {{&S96951[111], 3}, {NULL}};
const Segment SGI56221[] = {{&S96951[100], 3}, {NULL}};
const Segment SGI56222[] = {{&S96951[77], 6}, {NULL}};
const Segment SGI56223[] = {{&S96953[31], 16}, {NULL}};
const Segment SGI56224[] = {{&S96953[62], 3}, {NULL}};
const Segment SGI56225[] = {{&S96992[59], 13}, {NULL}};
const Segment SGI56226[] = {{&S96992[59], 30}, {NULL}};
const Segment SGI56227[] = {{&S96992[45], 6}, {NULL}};
const Segment SGI56228[] = {{&S96992[137], 3}, {NULL}};
const Segment SGI56229[] = {{&S96992[103], 5}, {NULL}};
const Segment SGI56230[] = {{&S96992[0], 14}, {NULL}};
const Segment SGI56231[] = {{&S96802[44], 10}, {NULL}};
const Segment SGI56232[] = {{&S96802[44], 27}, {NULL}};
const Segment SGI56233[] = {{&S96802[30], 6}, {NULL}};
const Segment SGI56234[] = {{&S96802[119], 3}, {NULL}};
const Segment SGI56235[] = {{&S96802[85], 5}, {NULL}};
const Segment SGI56236[] = {{&S96802[143], 7}, {NULL}};
const Segment SGI56237[] = {{&S97001[44], 21}, {NULL}};
const Segment SGI56238[] = {{&S97001[44], 32}, {NULL}};
const Segment SGI56239[] = {{&S97001[30], 6}, {NULL}};
const Segment SGI56240[] = {{&S97001[124], 3}, {NULL}};
const Segment SGI56241[] = {{&S97001[90], 5}, {NULL}};
const Segment SGI56242[] = {{&S97001[158], 3}, {NULL}};
const Segment SGI56243[] = {{&S97014[59], 5}, {NULL}};
const Segment SGI56244[] = {{&S97014[59], 16}, {NULL}};
const Segment SGI56245[] = {{&S97014[45], 6}, {NULL}};
const Segment SGI56246[] = {{&S97014[123], 3}, {NULL}};
const Segment SGI56247[] = {{&S97014[89], 5}, {NULL}};
const Segment SGI56248[] = {{&S97014[0], 14}, {NULL}};
const Segment SGI56249[] = {{&S96827[52], 7}, {NULL}};
const Segment SGI56250[] = {{&S96827[52], 25}, {NULL}};
const Segment SGI56251[] = {{&S96827[38], 6}, {NULL}};
const Segment SGI56252[] = {{&S96827[91], 5}, {NULL}};
const Segment SGI56253[] = {{&S96827[0], 14}, {NULL}};
const Segment SGI56254[] = {{&S97067[44], 19}, {NULL}};
const Segment SGI56255[] = {{&S97067[44], 30}, {NULL}};
const Segment SGI56256[] = {{&S97067[30], 6}, {NULL}};
const Segment SGI56257[] = {{&S97067[122], 3}, {NULL}};
const Segment SGI56258[] = {{&S97067[88], 5}, {NULL}};
const Segment SGI56259[] = {{&S96884[44], 7}, {NULL}};
const Segment SGI56260[] = {{&S96884[44], 25}, {NULL}};
const Segment SGI56261[] = {{&S96884[30], 6}, {NULL}};
const Segment SGI56262[] = {{&S96884[83], 6}, {NULL}};
const Segment SGI56263[] = {{&S96884[110], 13}, {NULL}};
const Segment SGI56264[] = {{&S99605[45], 10}, {NULL}};
const Segment SGI56265[] = {{&S99605[30], 6}, {NULL}};
const Segment SGI56266[] = {{&S99605[120], 3}, {NULL}};
const Segment SGI56267[] = {{&S99605[86], 5}, {NULL}};
const Segment SGI56268[] = {{&S99605[154], 3}, {NULL}};
const Segment SGI56269[] = {{&S99633[44], 7}, {NULL}};
const Segment SGI56270[] = {{&S99633[44], 18}, {NULL}};
const Segment SGI56271[] = {{&S99633[30], 6}, {NULL}};
const Segment SGI56272[] = {{&S99633[110], 3}, {NULL}};
const Segment SGI56273[] = {{&S99633[76], 5}, {NULL}};
const Segment SGI56274[] = {{&S97138[59], 18}, {NULL}};
const Segment SGI56275[] = {{&S97138[59], 35}, {NULL}};
const Segment SGI56276[] = {{&S97138[45], 6}, {NULL}};
const Segment SGI56277[] = {{&S97138[142], 3}, {NULL}};
const Segment SGI56278[] = {{&S97138[108], 5}, {NULL}};
const Segment SGI56279[] = {{&S97138[0], 14}, {NULL}};
const Segment SGI56280[] = {{&S97159[52], 10}, {NULL}};
const Segment SGI56281[] = {{&S97159[52], 33}, {NULL}};
const Segment SGI56282[] = {{&S97159[38], 6}, {NULL}};
const Segment SGI56283[] = {{&S97159[99], 5}, {NULL}};
const Segment SGI56284[] = {{&S97159[0], 14}, {NULL}};
const Segment SGI56285[] = {{&S96993[44], 10}, {NULL}};
const Segment SGI56286[] = {{&S96993[44], 26}, {NULL}};
const Segment SGI56287[] = {{&S96993[30], 6}, {NULL}};
const Segment SGI56288[] = {{&S96993[125], 3}, {NULL}};
const Segment SGI56289[] = {{&S96993[91], 5}, {NULL}};
const Segment SGI56290[] = {{&S96993[70], 8}, {NULL}};
const Segment SGI56291[] = {{&S96994[44], 7}, {NULL}};
const Segment SGI56292[] = {{&S96994[44], 24}, {NULL}};
const Segment SGI56293[] = {{&S96994[30], 6}, {NULL}};
const Segment SGI56294[] = {{&S96994[116], 3}, {NULL}};
const Segment SGI56295[] = {{&S96994[82], 5}, {NULL}};
const Segment SGI56296[] = {{&S97186[52], 6}, {NULL}};
const Segment SGI56297[] = {{&S97186[52], 23}, {NULL}};
const Segment SGI56298[] = {{&S97186[38], 6}, {NULL}};
const Segment SGI56299[] = {{&S97186[89], 5}, {NULL}};
const Segment SGI56300[] = {{&S97186[0], 14}, {NULL}};
const Segment SGI56301[] = {{&S99819[44], 17}, {NULL}};
const Segment SGI56302[] = {{&S99819[44], 28}, {NULL}};
const Segment SGI56303[] = {{&S99819[30], 6}, {NULL}};
const Segment SGI56304[] = {{&S99819[86], 6}, {NULL}};
const Segment SGI56305[] = {{&S99819[123], 3}, {NULL}};
const Segment SGI56306[] = {{&S97111[34], 7}, {NULL}};
const Segment SGI56307[] = {{&S97111[27], 6}, {NULL}};
const Segment SGI56308[] = {{&S97111[107], 2}, {NULL}};
const Segment SGI56309[] = {{&S97111[73], 6}, {NULL}};
const Segment SGI56310[] = {{&S97111[141], 8}, {NULL}};
const Segment SGI56311[] = {{&S97471[44], 14}, {NULL}};
const Segment SGI56312[] = {{&S97471[44], 31}, {NULL}};
const Segment SGI56313[] = {{&S97471[30], 6}, {NULL}};
const Segment SGI56314[] = {{&S97471[123], 3}, {NULL}};
const Segment SGI56315[] = {{&S97471[89], 5}, {NULL}};
const Segment SGI56316[] = {{&S97471[147], 13}, {NULL}};
const Segment SGI56317[] = {{&S100020[44], 6}, {NULL}};
const Segment SGI56318[] = {{&S100020[44], 24}, {NULL}};
const Segment SGI56319[] = {{&S100020[30], 6}, {NULL}};
const Segment SGI56320[] = {{&S100020[82], 6}, {NULL}};
const Segment SGI56321[] = {{&S100020[119], 3}, {NULL}};
const Segment SGI56322[] = {{&S97219[44], 7}, {NULL}};
const Segment SGI56323[] = {{&S97219[44], 25}, {NULL}};
const Segment SGI56324[] = {{&S97219[30], 6}, {NULL}};
const Segment SGI56325[] = {{&S97219[83], 6}, {NULL}};
const Segment SGI56326[] = {{&S97219[110], 13}, {NULL}};
const Segment SGI56327[] = {{&S97567[52], 14}, {NULL}};
const Segment SGI56328[] = {{&S97567[52], 35}, {NULL}};
const Segment SGI56329[] = {{&S97567[38], 6}, {NULL}};
const Segment SGI56330[] = {{&S97567[101], 5}, {NULL}};
const Segment SGI56331[] = {{&S97567[0], 14}, {NULL}};
const Segment SGI56332[] = {{&S97573[44], 7}, {NULL}};
const Segment SGI56333[] = {{&S97573[44], 32}, {NULL}};
const Segment SGI56334[] = {{&S97573[30], 6}, {NULL}};
const Segment SGI56335[] = {{&S97573[90], 6}, {NULL}};
const Segment SGI56336[] = {{&S97573[117], 13}, {NULL}};
const Segment SGI56337[] = {{&S97309[57], 7}, {NULL}};
const Segment SGI56338[] = {{&S97309[57], 25}, {NULL}};
const Segment SGI56339[] = {{&S97309[43], 6}, {NULL}};
const Segment SGI56340[] = {{&S97309[70], 3}, {NULL}};
const Segment SGI56341[] = {{&S97309[96], 5}, {NULL}};
const Segment SGI56342[] = {{&S97309[0], 12}, {NULL}};
const Segment SGI56343[] = {{&S97329[31], 5}, {NULL}};
const Segment SGI56344[] = {{&S97329[51], 3}, {NULL}};
const Segment SGI56345[] = {{&S97337[45], 6}, {NULL}};
const Segment SGI56346[] = {{&S97337[30], 6}, {NULL}};
const Segment SGI56347[] = {{&S97337[116], 3}, {NULL}};
const Segment SGI56348[] = {{&S97337[82], 5}, {NULL}};
const Segment SGI56349[] = {{&S97337[140], 13}, {NULL}};
const Segment SGI56350[] = {{&S97365[45], 10}, {NULL}};
const Segment SGI56351[] = {{&S97365[30], 6}, {NULL}};
const Segment SGI56352[] = {{&S97365[120], 3}, {NULL}};
const Segment SGI56353[] = {{&S97365[86], 5}, {NULL}};
const Segment SGI56354[] = {{&S97365[144], 13}, {NULL}};
const Segment SGI56355[] = {{&S100204[44], 9}, {NULL}};
const Segment SGI56356[] = {{&S100204[44], 20}, {NULL}};
const Segment SGI56357[] = {{&S100204[30], 6}, {NULL}};
const Segment SGI56358[] = {{&S100204[112], 3}, {NULL}};
const Segment SGI56359[] = {{&S100204[78], 5}, {NULL}};
const Segment SGI56360[] = {{&S100281[44], 10}, {NULL}};
const Segment SGI56361[] = {{&S100281[44], 27}, {NULL}};
const Segment SGI56362[] = {{&S100281[30], 6}, {NULL}};
const Segment SGI56363[] = {{&S100281[119], 3}, {NULL}};
const Segment SGI56364[] = {{&S100281[85], 5}, {NULL}};
const Segment SGI56365[] = {{&S100281[153], 3}, {NULL}};
const Segment SGI56366[] = {{&S97698[45], 5}, {NULL}};
const Segment SGI56367[] = {{&S97698[30], 6}, {NULL}};
const Segment SGI56368[] = {{&S97698[109], 3}, {NULL}};
const Segment SGI56369[] = {{&S97698[75], 5}, {NULL}};
const Segment SGI56370[] = {{&S97698[133], 13}, {NULL}};
const Segment SGI56371[] = {{&S97517[44], 10}, {NULL}};
const Segment SGI56372[] = {{&S97517[44], 21}, {NULL}};
const Segment SGI56373[] = {{&S97517[30], 6}, {NULL}};
const Segment SGI56374[] = {{&S97517[79], 6}, {NULL}};
const Segment SGI56375[] = {{&S97517[106], 13}, {NULL}};
const Segment SGI56376[] = {{&S97732[45], 6}, {NULL}};
const Segment SGI56377[] = {{&S97732[30], 6}, {NULL}};
const Segment SGI56378[] = {{&S97732[116], 3}, {NULL}};
const Segment SGI56379[] = {{&S97732[82], 5}, {NULL}};
const Segment SGI56380[] = {{&S97732[140], 13}, {NULL}};
const Segment SGI56381[] = {{&S100353[44], 10}, {NULL}};
const Segment SGI56382[] = {{&S100353[44], 27}, {NULL}};
const Segment SGI56383[] = {{&S100353[30], 6}, {NULL}};
const Segment SGI56384[] = {{&S100353[85], 6}, {NULL}};
const Segment SGI56385[] = {{&S100353[122], 3}, {NULL}};
const Segment SGI56386[] = {{&S97824[45], 7}, {NULL}};
const Segment SGI56387[] = {{&S97824[30], 6}, {NULL}};
const Segment SGI56388[] = {{&S97824[125], 3}, {NULL}};
const Segment SGI56389[] = {{&S97824[91], 5}, {NULL}};
const Segment SGI56390[] = {{&S97824[149], 13}, {NULL}};
const Segment SGI56391[] = {{&S97829[44], 41}, {NULL}};
const Segment SGI56392[] = {{&S97829[44], 52}, {NULL}};
const Segment SGI56393[] = {{&S97829[30], 6}, {NULL}};
const Segment SGI56394[] = {{&S97829[110], 6}, {NULL}};
const Segment SGI56395[] = {{&S97829[137], 13}, {NULL}};
const Segment SGI56396[] = {{&S97830[61], 20}, {NULL}};
const Segment SGI56397[] = {{&S97830[61], 31}, {NULL}};
const Segment SGI56398[] = {{&S97830[47], 6}, {NULL}};
const Segment SGI56399[] = {{&S97830[140], 3}, {NULL}};
const Segment SGI56400[] = {{&S97830[106], 5}, {NULL}};
const Segment SGI56401[] = {{&S97830[0], 14}, {NULL}};
const Segment SGI56402[] = {{&S97831[44], 14}, {NULL}};
const Segment SGI56403[] = {{&S97831[44], 35}, {NULL}};
const Segment SGI56404[] = {{&S97831[30], 6}, {NULL}};
const Segment SGI56405[] = {{&S97831[126], 3}, {NULL}};
const Segment SGI56406[] = {{&S97831[93], 5}, {NULL}};
const Segment SGI56407[] = {{&S97831[130], 14}, {NULL}};
const Segment SGI56408[] = {{&S97857[44], 17}, {NULL}};
const Segment SGI56409[] = {{&S97857[44], 28}, {NULL}};
const Segment SGI56410[] = {{&S97857[30], 6}, {NULL}};
const Segment SGI56411[] = {{&S97857[86], 6}, {NULL}};
const Segment SGI56412[] = {{&S97857[113], 13}, {NULL}};
const Segment SGI56413[] = {{&S97716[31], 3}, {NULL}};
const Segment SGI56414[] = {{&S97716[49], 3}, {NULL}};
const Segment SGI56415[] = {{&S97916[44], 5}, {NULL}};
const Segment SGI56416[] = {{&S97916[44], 22}, {NULL}};
const Segment SGI56417[] = {{&S97916[30], 6}, {NULL}};
const Segment SGI56418[] = {{&S97916[80], 6}, {NULL}};
const Segment SGI56419[] = {{&S97916[107], 13}, {NULL}};
const Segment SGI56420[] = {{&S97740[59], 7}, {NULL}};
const Segment SGI56421[] = {{&S97740[59], 25}, {NULL}};
const Segment SGI56422[] = {{&S97740[45], 6}, {NULL}};
const Segment SGI56423[] = {{&S97740[72], 3}, {NULL}};
const Segment SGI56424[] = {{&S97740[98], 5}, {NULL}};
const Segment SGI56425[] = {{&S97740[0], 14}, {NULL}};
const Segment SGI56426[] = {{&S97946[34], 21}, {NULL}};
const Segment SGI56427[] = {{&S97946[27], 6}, {NULL}};
const Segment SGI56428[] = {{&S97946[114], 2}, {NULL}};
const Segment SGI56429[] = {{&S97946[80], 6}, {NULL}};
const Segment SGI56430[] = {{&S97946[148], 8}, {NULL}};
const Segment SGI56431[] = {{&S97778[44], 7}, {NULL}};
const Segment SGI56432[] = {{&S97778[44], 25}, {NULL}};
const Segment SGI56433[] = {{&S97778[30], 6}, {NULL}};
const Segment SGI56434[] = {{&S97778[57], 3}, {NULL}};
const Segment SGI56435[] = {{&S97778[83], 5}, {NULL}};
const Segment SGI56436[] = {{&S97778[120], 14}, {NULL}};
const Segment SGI56437[] = {{&S97979[52], 14}, {NULL}};
const Segment SGI56438[] = {{&S97979[52], 31}, {NULL}};
const Segment SGI56439[] = {{&S97979[38], 6}, {NULL}};
const Segment SGI56440[] = {{&S97979[97], 5}, {NULL}};
const Segment SGI56441[] = {{&S97979[0], 14}, {NULL}};
const Segment SGI56442[] = {{&S97797[45], 7}, {NULL}};
const Segment SGI56443[] = {{&S97797[30], 6}, {NULL}};
const Segment SGI56444[] = {{&S97797[58], 3}, {NULL}};
const Segment SGI56445[] = {{&S97797[84], 5}, {NULL}};
const Segment SGI56446[] = {{&S97797[142], 13}, {NULL}};
const Segment SGI56447[] = {{&S100514[44], 14}, {NULL}};
const Segment SGI56448[] = {{&S100514[44], 25}, {NULL}};
const Segment SGI56449[] = {{&S100514[30], 6}, {NULL}};
const Segment SGI56450[] = {{&S100514[116], 3}, {NULL}};
const Segment SGI56451[] = {{&S100514[83], 5}, {NULL}};
const Segment SGI56452[] = {{&S100514[120], 14}, {NULL}};
const Segment SGI56453[] = {{&S100527[52], 6}, {NULL}};
const Segment SGI56454[] = {{&S100527[52], 24}, {NULL}};
const Segment SGI56455[] = {{&S100527[38], 6}, {NULL}};
const Segment SGI56456[] = {{&S100527[90], 5}, {NULL}};
const Segment SGI56457[] = {{&S100527[0], 12}, {NULL}};
const Segment SGI56458[] = {{&S97826[44], 7}, {NULL}};
const Segment SGI56459[] = {{&S97826[44], 25}, {NULL}};
const Segment SGI56460[] = {{&S97826[30], 6}, {NULL}};
const Segment SGI56461[] = {{&S97826[83], 6}, {NULL}};
const Segment SGI56462[] = {{&S97826[110], 13}, {NULL}};
const Segment SGI56463[] = {{&S97833[44], 7}, {NULL}};
const Segment SGI56464[] = {{&S97833[44], 25}, {NULL}};
const Segment SGI56465[] = {{&S97833[30], 6}, {NULL}};
const Segment SGI56466[] = {{&S97833[57], 3}, {NULL}};
const Segment SGI56467[] = {{&S97833[83], 5}, {NULL}};
const Segment SGI56468[] = {{&S97833[120], 14}, {NULL}};
const Segment SGI56469[] = {{&S97895[44], 7}, {NULL}};
const Segment SGI56470[] = {{&S97895[44], 25}, {NULL}};
const Segment SGI56471[] = {{&S97895[30], 6}, {NULL}};
const Segment SGI56472[] = {{&S97895[57], 3}, {NULL}};
const Segment SGI56473[] = {{&S97895[83], 5}, {NULL}};
const Segment SGI56474[] = {{&S97895[120], 14}, {NULL}};
const Segment SGI56475[] = {{&S97899[34], 18}, {NULL}};
const Segment SGI56476[] = {{&S97899[27], 6}, {NULL}};
const Segment SGI56477[] = {{&S97899[111], 2}, {NULL}};
const Segment SGI56478[] = {{&S97899[77], 6}, {NULL}};
const Segment SGI56479[] = {{&S97899[145], 8}, {NULL}};
const Segment SGI56480[] = {{&S98121[44], 10}, {NULL}};
const Segment SGI56481[] = {{&S98121[44], 27}, {NULL}};
const Segment SGI56482[] = {{&S98121[30], 6}, {NULL}};
const Segment SGI56483[] = {{&S98121[126], 3}, {NULL}};
const Segment SGI56484[] = {{&S98121[92], 5}, {NULL}};
const Segment SGI56485[] = {{&S98121[71], 8}, {NULL}};
const Segment SGI56486[] = {{&S98122[44], 10}, {NULL}};
const Segment SGI56487[] = {{&S98122[44], 27}, {NULL}};
const Segment SGI56488[] = {{&S98122[30], 6}, {NULL}};
const Segment SGI56489[] = {{&S98122[119], 3}, {NULL}};
const Segment SGI56490[] = {{&S98122[85], 5}, {NULL}};
const Segment SGI56491[] = {{&S98122[143], 7}, {NULL}};
const Segment SGI56492[] = {{&S98140[45], 10}, {NULL}};
const Segment SGI56493[] = {{&S98140[30], 6}, {NULL}};
const Segment SGI56494[] = {{&S98140[120], 3}, {NULL}};
const Segment SGI56495[] = {{&S98140[86], 5}, {NULL}};
const Segment SGI56496[] = {{&S98140[144], 13}, {NULL}};
const Segment SGI56497[] = {{&S98155[45], 11}, {NULL}};
const Segment SGI56498[] = {{&S98155[30], 6}, {NULL}};
const Segment SGI56499[] = {{&S98155[115], 3}, {NULL}};
const Segment SGI56500[] = {{&S98155[81], 5}, {NULL}};
const Segment SGI56501[] = {{&S98155[139], 13}, {NULL}};
const Segment SGI56502[] = {{&S98187[45], 14}, {NULL}};
const Segment SGI56503[] = {{&S98187[30], 6}, {NULL}};
const Segment SGI56504[] = {{&S98187[124], 3}, {NULL}};
const Segment SGI56505[] = {{&S98187[90], 5}, {NULL}};
const Segment SGI56506[] = {{&S98187[148], 13}, {NULL}};
const Segment SGI56507[] = {{&S98198[45], 5}, {NULL}};
const Segment SGI56508[] = {{&S98198[30], 6}, {NULL}};
const Segment SGI56509[] = {{&S98198[109], 3}, {NULL}};
const Segment SGI56510[] = {{&S98198[75], 5}, {NULL}};
const Segment SGI56511[] = {{&S98198[133], 13}, {NULL}};
const Segment SGI56512[] = {{&S98241[44], 15}, {NULL}};
const Segment SGI56513[] = {{&S98241[44], 32}, {NULL}};
const Segment SGI56514[] = {{&S98241[30], 6}, {NULL}};
const Segment SGI56515[] = {{&S98241[90], 6}, {NULL}};
const Segment SGI56516[] = {{&S98241[117], 13}, {NULL}};
const Segment SGI56517[] = {{&S98262[44], 14}, {NULL}};
const Segment SGI56518[] = {{&S98262[44], 35}, {NULL}};
const Segment SGI56519[] = {{&S98262[30], 6}, {NULL}};
const Segment SGI56520[] = {{&S98262[126], 3}, {NULL}};
const Segment SGI56521[] = {{&S98262[93], 5}, {NULL}};
const Segment SGI56522[] = {{&S98262[130], 14}, {NULL}};
const Segment SGI56523[] = {{&S98319[45], 5}, {NULL}};
const Segment SGI56524[] = {{&S98319[30], 6}, {NULL}};
const Segment SGI56525[] = {{&S98319[115], 3}, {NULL}};
const Segment SGI56526[] = {{&S98319[81], 5}, {NULL}};
const Segment SGI56527[] = {{&S98319[139], 13}, {NULL}};
const Segment SGI56528[] = {{&S98330[44], 7}, {NULL}};
const Segment SGI56529[] = {{&S98330[44], 32}, {NULL}};
const Segment SGI56530[] = {{&S98330[30], 6}, {NULL}};
const Segment SGI56531[] = {{&S98330[90], 6}, {NULL}};
const Segment SGI56532[] = {{&S98330[117], 13}, {NULL}};
const Segment SGI56533[] = {{&S98367[34], 20}, {NULL}};
const Segment SGI56534[] = {{&S98367[27], 6}, {NULL}};
const Segment SGI56535[] = {{&S98367[113], 2}, {NULL}};
const Segment SGI56536[] = {{&S98367[79], 6}, {NULL}};
const Segment SGI56537[] = {{&S98367[147], 8}, {NULL}};
const Segment SGI56538[] = {{&S98368[34], 11}, {NULL}};
const Segment SGI56539[] = {{&S98368[27], 6}, {NULL}};
const Segment SGI56540[] = {{&S98368[110], 2}, {NULL}};
const Segment SGI56541[] = {{&S98368[76], 6}, {NULL}};
const Segment SGI56542[] = {{&S98368[144], 8}, {NULL}};
const Segment SGI56543[] = {{&S98382[44], 14}, {NULL}};
const Segment SGI56544[] = {{&S98382[44], 25}, {NULL}};
const Segment SGI56545[] = {{&S98382[30], 6}, {NULL}};
const Segment SGI56546[] = {{&S98382[83], 6}, {NULL}};
const Segment SGI56547[] = {{&S98202[45], 10}, {NULL}};
const Segment SGI56548[] = {{&S98202[30], 6}, {NULL}};
const Segment SGI56549[] = {{&S98382[110], 13}, {NULL}};
const Segment SGI56550[] = {{&S98202[114], 3}, {NULL}};
const Segment SGI56551[] = {{&S98202[80], 5}, {NULL}};
const Segment SGI56552[] = {{&S98202[138], 13}, {NULL}};
const Segment SGI56553[] = {{&S98215[44], 18}, {NULL}};
const Segment SGI56554[] = {{&S98215[44], 29}, {NULL}};
const Segment SGI56555[] = {{&S98215[30], 6}, {NULL}};
const Segment SGI56556[] = {{&S98215[87], 6}, {NULL}};
const Segment SGI56557[] = {{&S98215[114], 13}, {NULL}};
const Segment SGI56558[] = {{&S100846[44], 18}, {NULL}};
const Segment SGI56559[] = {{&S100846[44], 29}, {NULL}};
const Segment SGI56560[] = {{&S100846[30], 6}, {NULL}};
const Segment SGI56561[] = {{&S100846[121], 3}, {NULL}};
const Segment SGI56562[] = {{&S100846[87], 5}, {NULL}};
const Segment SGI56563[] = {{&S98439[44], 6}, {NULL}};
const Segment SGI56564[] = {{&S98439[44], 23}, {NULL}};
const Segment SGI56565[] = {{&S98439[30], 6}, {NULL}};
const Segment SGI56566[] = {{&S98439[81], 6}, {NULL}};
const Segment SGI56567[] = {{&S98439[108], 13}, {NULL}};
const Segment SGI56568[] = {{&S98264[44], 10}, {NULL}};
const Segment SGI56569[] = {{&S98264[44], 27}, {NULL}};
const Segment SGI56570[] = {{&S98264[30], 6}, {NULL}};
const Segment SGI56571[] = {{&S98264[118], 3}, {NULL}};
const Segment SGI56572[] = {{&S98264[85], 5}, {NULL}};
const Segment SGI56573[] = {{&S98264[122], 14}, {NULL}};
const Segment SGI56574[] = {{&S98456[44], 14}, {NULL}};
const Segment SGI56575[] = {{&S98456[44], 31}, {NULL}};
const Segment SGI56576[] = {{&S98456[30], 6}, {NULL}};
const Segment SGI56577[] = {{&S98456[89], 6}, {NULL}};
const Segment SGI56578[] = {{&S98456[116], 13}, {NULL}};
const Segment SGI56579[] = {{&S98276[44], 13}, {NULL}};
const Segment SGI56580[] = {{&S98276[44], 24}, {NULL}};
const Segment SGI56581[] = {{&S98276[30], 6}, {NULL}};
const Segment SGI56582[] = {{&S98276[116], 3}, {NULL}};
const Segment SGI56583[] = {{&S98276[82], 5}, {NULL}};
const Segment SGI56584[] = {{&S98276[139], 10}, {NULL}};
const Segment SGI56585[] = {{&S100904[59], 11}, {NULL}};
const Segment SGI56586[] = {{&S100904[59], 22}, {NULL}};
const Segment SGI56587[] = {{&S100904[45], 6}, {NULL}};
const Segment SGI56588[] = {{&S100904[129], 3}, {NULL}};
const Segment SGI56589[] = {{&S100904[95], 5}, {NULL}};
const Segment SGI56590[] = {{&S100904[0], 14}, {NULL}};
const Segment SGI56591[] = {{&S98417[44], 8}, {NULL}};
const Segment SGI56592[] = {{&S98417[44], 19}, {NULL}};
const Segment SGI56593[] = {{&S98417[30], 6}, {NULL}};
const Segment SGI56594[] = {{&S98417[111], 3}, {NULL}};
const Segment SGI56595[] = {{&S98417[77], 5}, {NULL}};
const Segment SGI56596[] = {{&S98417[135], 7}, {NULL}};
const Segment SGI56597[] = {{&S101020[58], 12}, {NULL}};
const Segment SGI56598[] = {{&S101020[58], 23}, {NULL}};
const Segment SGI56599[] = {{&S101020[44], 6}, {NULL}};
const Segment SGI56600[] = {{&S101020[129], 3}, {NULL}};
const Segment SGI56601[] = {{&S101020[95], 5}, {NULL}};
const Segment SGI56602[] = {{&S101027[59], 37}, {NULL}};
const Segment SGI56603[] = {{&S101027[45], 6}, {NULL}};
const Segment SGI56604[] = {{&S101027[128], 5}, {NULL}};
const Segment SGI56605[] = {{&S101027[0], 14}, {NULL}};
const Segment SGI56606[] = {{&S101029[59], 10}, {NULL}};
const Segment SGI56607[] = {{&S101029[59], 27}, {NULL}};
const Segment SGI56608[] = {{&S101029[45], 6}, {NULL}};
const Segment SGI56609[] = {{&S101029[134], 3}, {NULL}};
const Segment SGI56610[] = {{&S101029[100], 5}, {NULL}};
const Segment SGI56611[] = {{&S101029[0], 14}, {NULL}};
const Segment SGI56612[] = {{&S101201[44], 7}, {NULL}};
const Segment SGI56613[] = {{&S101201[44], 18}, {NULL}};
const Segment SGI56614[] = {{&S101201[30], 6}, {NULL}};
const Segment SGI56615[] = {{&S101201[110], 3}, {NULL}};
const Segment SGI56616[] = {{&S101201[76], 5}, {NULL}};
const Segment SGI56617[] = {{&S101234[59], 22}, {NULL}};
const Segment SGI56618[] = {{&S101234[59], 33}, {NULL}};
const Segment SGI56619[] = {{&S101234[45], 6}, {NULL}};
const Segment SGI56620[] = {{&S101234[140], 3}, {NULL}};
const Segment SGI56621[] = {{&S101234[106], 5}, {NULL}};
const Segment SGI56622[] = {{&S101234[0], 14}, {NULL}};
const Segment SGI56623[] = {{&S98595[59], 10}, {NULL}};
const Segment SGI56624[] = {{&S98595[59], 27}, {NULL}};
const Segment SGI56625[] = {{&S98595[45], 6}, {NULL}};
const Segment SGI56626[] = {{&S98595[134], 3}, {NULL}};
const Segment SGI56627[] = {{&S98595[100], 5}, {NULL}};
const Segment SGI56628[] = {{&S98595[0], 14}, {NULL}};
const Segment SGI56629[] = {{&S98698[59], 18}, {NULL}};
const Segment SGI56630[] = {{&S98698[59], 35}, {NULL}};
const Segment SGI56631[] = {{&S98698[45], 6}, {NULL}};
const Segment SGI56632[] = {{&S98698[142], 3}, {NULL}};
const Segment SGI56633[] = {{&S98698[108], 5}, {NULL}};
const Segment SGI56634[] = {{&S98698[0], 14}, {NULL}};
const Segment SGI56635[] = {{&S98646[43], 10}, {NULL}};
const Segment SGI56636[] = {{&S98646[43], 27}, {NULL}};
const Segment SGI56637[] = {{&S98646[29], 6}, {NULL}};
const Segment SGI56638[] = {{&S98646[84], 6}, {NULL}};
const Segment SGI56639[] = {{&S98646[111], 13}, {NULL}};
const Segment SGI56640[] = {{&S101314[44], 20}, {NULL}};
const Segment SGI56641[] = {{&S101314[44], 38}, {NULL}};
const Segment SGI56642[] = {{&S101314[30], 6}, {NULL}};
const Segment SGI56643[] = {{&S101314[130], 3}, {NULL}};
const Segment SGI56644[] = {{&S101314[96], 5}, {NULL}};
const Segment SGI56645[] = {{&S101315[44], 8}, {NULL}};
const Segment SGI56646[] = {{&S101315[44], 26}, {NULL}};
const Segment SGI56647[] = {{&S101315[30], 6}, {NULL}};
const Segment SGI56648[] = {{&S101315[118], 3}, {NULL}};
const Segment SGI56649[] = {{&S101315[84], 5}, {NULL}};
const Segment SGI56650[] = {{&S101362[44], 12}, {NULL}};
const Segment SGI56651[] = {{&S101362[44], 23}, {NULL}};
const Segment SGI56652[] = {{&S101362[30], 6}, {NULL}};
const Segment SGI56653[] = {{&S101362[115], 3}, {NULL}};
const Segment SGI56654[] = {{&S101362[81], 5}, {NULL}};
const Segment SGI56655[] = {{&S101363[44], 24}, {NULL}};
const Segment SGI56656[] = {{&S101363[44], 34}, {NULL}};
const Segment SGI56657[] = {{&S101363[30], 6}, {&S101363[52], 26}, {NULL}};
const Segment SGI56658[] = {{&S101363[126], 3}, {NULL}};
const Segment SGI56659[] = {{&S101363[92], 5}, {NULL}};
const Segment SGI56660[] = {{&S101386[62], 10}, {NULL}};
const Segment SGI56661[] = {{&S101386[55], 6}, {NULL}};
const Segment SGI56662[] = {{&S101386[0], 15}, {NULL}};
const Segment SGI56663[] = {{&S101387[62], 11}, {NULL}};
const Segment SGI56664[] = {{&S101387[55], 6}, {NULL}};
const Segment SGI56665[] = {{&S98812[44], 14}, {NULL}};
const Segment SGI56666[] = {{&S98812[44], 35}, {NULL}};
const Segment SGI56667[] = {{&S101387[0], 15}, {NULL}};
const Segment SGI56668[] = {{&S98812[30], 6}, {NULL}};
const Segment SGI56669[] = {{&S98812[93], 6}, {NULL}};
const Segment SGI56670[] = {{&S98812[120], 13}, {NULL}};
const Segment SGI56671[] = {{&S101388[62], 11}, {NULL}};
const Segment SGI56672[] = {{&S101388[55], 6}, {NULL}};
const Segment SGI56673[] = {{&S101388[0], 15}, {NULL}};
const Segment SGI56674[] = {{&S98824[34], 8}, {NULL}};
const Segment SGI56675[] = {{&S98824[27], 6}, {NULL}};
const Segment SGI56676[] = {{&S98824[107], 2}, {NULL}};
const Segment SGI56677[] = {{&S98824[73], 6}, {NULL}};
const Segment SGI56678[] = {{&S98824[141], 8}, {NULL}};
const Segment SGI56679[] = {{&S98838[44], 21}, {NULL}};
const Segment SGI56680[] = {{&S98838[44], 32}, {NULL}};
const Segment SGI56681[] = {{&S98838[30], 6}, {NULL}};
const Segment SGI56682[] = {{&S98838[123], 3}, {NULL}};
const Segment SGI56683[] = {{&S98838[90], 5}, {NULL}};
const Segment SGI56684[] = {{&S98838[127], 14}, {NULL}};
const Segment SGI56685[] = {{&S98781[34], 10}, {NULL}};
const Segment SGI56686[] = {{&S98781[27], 6}, {NULL}};
const Segment SGI56687[] = {{&S98781[109], 2}, {NULL}};
const Segment SGI56688[] = {{&S98781[75], 6}, {NULL}};
const Segment SGI56689[] = {{&S98781[143], 8}, {NULL}};
const Segment SGI56690[] = {{&S98901[44], 11}, {NULL}};
const Segment SGI56691[] = {{&S98901[44], 22}, {NULL}};
const Segment SGI56692[] = {{&S98901[30], 6}, {NULL}};
const Segment SGI56693[] = {{&S98901[114], 3}, {NULL}};
const Segment SGI56694[] = {{&S98901[80], 5}, {NULL}};
const Segment SGI56695[] = {{&S98901[138], 7}, {NULL}};
const Segment SGI56696[] = {{&S98830[44], 10}, {NULL}};
const Segment SGI56697[] = {{&S98830[44], 27}, {NULL}};
const Segment SGI56698[] = {{&S98830[30], 6}, {NULL}};
const Segment SGI56699[] = {{&S98830[85], 6}, {NULL}};
const Segment SGI56700[] = {{&S98830[112], 13}, {NULL}};
const Segment SGI56701[] = {{&S98872[44], 7}, {NULL}};
const Segment SGI56702[] = {{&S98872[44], 25}, {NULL}};
const Segment SGI56703[] = {{&S98872[30], 6}, {NULL}};
const Segment SGI56704[] = {{&S98872[57], 3}, {NULL}};
const Segment SGI56705[] = {{&S98872[83], 5}, {NULL}};
const Segment SGI56706[] = {{&S98872[120], 14}, {NULL}};
const Segment SGI56707[] = {{&S101554[45], 10}, {NULL}};
const Segment SGI56708[] = {{&S101554[30], 6}, {NULL}};
const Segment SGI56709[] = {{&S101554[120], 3}, {NULL}};
const Segment SGI56710[] = {{&S101554[86], 5}, {NULL}};
const Segment SGI56711[] = {{&S101554[154], 3}, {NULL}};
const Segment SGI56712[] = {{&S101555[44], 10}, {NULL}};
const Segment SGI56713[] = {{&S101555[44], 27}, {NULL}};
const Segment SGI56714[] = {{&S101555[30], 6}, {NULL}};
const Segment SGI56715[] = {{&S101555[120], 3}, {NULL}};
const Segment SGI56716[] = {{&S101555[85], 6}, {NULL}};
const Segment SGI56717[] = {{&S101555[140], 8}, {NULL}};
const Segment SGI56718[] = {{&S101555[159], 3}, {NULL}};
const Segment SGI56719[] = {{&S101567[42], 10}, {NULL}};
const Segment SGI56720[] = {{&S101567[30], 6}, {NULL}};
const Segment SGI56721[] = {{&S101567[117], 3}, {NULL}};
const Segment SGI56722[] = {{&S101567[83], 5}, {NULL}};
const Segment SGI56723[] = {{&S101567[140], 10}, {NULL}};
const Segment SGI56724[] = {{&S101568[44], 10}, {NULL}};
const Segment SGI56725[] = {{&S101568[44], 27}, {NULL}};
const Segment SGI56726[] = {{&S101568[30], 6}, {NULL}};
const Segment SGI56727[] = {{&S101568[119], 3}, {NULL}};
const Segment SGI56728[] = {{&S101568[85], 5}, {NULL}};
const Segment SGI56729[] = {{&S101568[142], 10}, {NULL}};
const Segment SGI56730[] = {{&S101569[44], 11}, {NULL}};
const Segment SGI56731[] = {{&S101569[44], 28}, {NULL}};
const Segment SGI56732[] = {{&S101569[30], 6}, {NULL}};
const Segment SGI56733[] = {{&S101569[120], 3}, {NULL}};
const Segment SGI56734[] = {{&S101569[86], 5}, {NULL}};
const Segment SGI56735[] = {{&S101569[143], 10}, {NULL}};
const Segment SGI56736[] = {{&S101570[44], 11}, {NULL}};
const Segment SGI56737[] = {{&S101570[44], 28}, {NULL}};
const Segment SGI56738[] = {{&S101570[30], 6}, {NULL}};
const Segment SGI56739[] = {{&S101570[120], 3}, {NULL}};
const Segment SGI56740[] = {{&S101570[86], 5}, {NULL}};
const Segment SGI56741[] = {{&S101570[143], 10}, {NULL}};
const Segment SGI56742[] = {{&S101573[44], 11}, {NULL}};
const Segment SGI56743[] = {{&S101573[44], 28}, {NULL}};
const Segment SGI56744[] = {{&S101573[30], 6}, {NULL}};
const Segment SGI56745[] = {{&S101573[120], 3}, {NULL}};
const Segment SGI56746[] = {{&S101573[86], 5}, {NULL}};
const Segment SGI56747[] = {{&S101573[143], 10}, {NULL}};
const Segment SGI56748[] = {{&S98930[34], 7}, {NULL}};
const Segment SGI56749[] = {{&S98930[27], 6}, {NULL}};
const Segment SGI56750[] = {{&S98930[107], 2}, {NULL}};
const Segment SGI56751[] = {{&S98930[73], 6}, {NULL}};
const Segment SGI56752[] = {{&S98930[141], 8}, {NULL}};
const Segment SGI56753[] = {{&S98944[44], 7}, {NULL}};
const Segment SGI56754[] = {{&S98944[44], 25}, {NULL}};
const Segment SGI56755[] = {{&S98944[30], 6}, {NULL}};
const Segment SGI56756[] = {{&S98944[83], 6}, {NULL}};
const Segment SGI56757[] = {{&S98944[110], 13}, {NULL}};
const Segment SGI56758[] = {{&S98964[44], 13}, {NULL}};
const Segment SGI56759[] = {{&S98964[44], 24}, {NULL}};
const Segment SGI56760[] = {{&S98964[30], 6}, {NULL}};
const Segment SGI56761[] = {{&S98964[82], 6}, {NULL}};
const Segment SGI56762[] = {{&S98964[109], 13}, {NULL}};
const Segment SGI56763[] = {{&S101708[59], 37}, {NULL}};
const Segment SGI56764[] = {{&S101708[45], 6}, {NULL}};
const Segment SGI56765[] = {{&S101708[128], 5}, {NULL}};
const Segment SGI56766[] = {{&S101708[0], 14}, {NULL}};
const Segment SGI56767[] = {{&S101737[57], 10}, {NULL}};
const Segment SGI56768[] = {{&S101737[57], 27}, {NULL}};
const Segment SGI56769[] = {{&S101737[43], 6}, {NULL}};
const Segment SGI56770[] = {{&S101737[132], 3}, {NULL}};
const Segment SGI56771[] = {{&S101737[98], 5}, {NULL}};
const Segment SGI56772[] = {{&S101737[0], 10}, {NULL}};
const Segment SGI56773[] = {{&S101738[55], 10}, {NULL}};
const Segment SGI56774[] = {{&S101738[43], 6}, {NULL}};
const Segment SGI56775[] = {{&S101738[130], 3}, {NULL}};
const Segment SGI56776[] = {{&S101738[96], 5}, {NULL}};
const Segment SGI56777[] = {{&S101738[0], 10}, {NULL}};
const Segment SGI56778[] = {{&S101767[44], 10}, {NULL}};
const Segment SGI56779[] = {{&S101767[44], 27}, {NULL}};
const Segment SGI56780[] = {{&S101767[30], 6}, {NULL}};
const Segment SGI56781[] = {{&S101767[118], 3}, {NULL}};
const Segment SGI56782[] = {{&S101767[85], 5}, {NULL}};
const Segment SGI56783[] = {{&S101767[122], 14}, {NULL}};
const Segment SGI56784[] = {{&S99103[59], 7}, {NULL}};
const Segment SGI56785[] = {{&S99103[59], 25}, {NULL}};
const Segment SGI56786[] = {{&S99103[45], 6}, {NULL}};
const Segment SGI56787[] = {{&S99103[72], 3}, {NULL}};
const Segment SGI56788[] = {{&S99103[98], 5}, {NULL}};
const Segment SGI56789[] = {{&S99103[0], 14}, {NULL}};
const Segment SGI56790[] = {{&S99136[45], 7}, {NULL}};
const Segment SGI56791[] = {{&S99136[30], 6}, {NULL}};
const Segment SGI56792[] = {{&S99136[58], 3}, {NULL}};
const Segment SGI56793[] = {{&S99136[84], 5}, {NULL}};
const Segment SGI56794[] = {{&S99136[142], 13}, {NULL}};
const Segment SGI56795[] = {{&S99139[45], 7}, {NULL}};
const Segment SGI56796[] = {{&S99139[30], 6}, {NULL}};
const Segment SGI56797[] = {{&S99139[58], 3}, {NULL}};
const Segment SGI56798[] = {{&S99139[84], 5}, {NULL}};
const Segment SGI56799[] = {{&S99139[142], 13}, {NULL}};
const Segment SGI56800[] = {{&S99185[45], 8}, {NULL}};
const Segment SGI56801[] = {{&S99185[30], 6}, {NULL}};
const Segment SGI56802[] = {{&S99185[122], 3}, {NULL}};
const Segment SGI56803[] = {{&S99185[88], 5}, {NULL}};
const Segment SGI56804[] = {{&S99185[146], 13}, {NULL}};
const Segment SGI56805[] = {{&S101869[44], 16}, {NULL}};
const Segment SGI56806[] = {{&S101869[44], 39}, {NULL}};
const Segment SGI56807[] = {{&S101869[30], 6}, {NULL}};
const Segment SGI56808[] = {{&S101869[131], 3}, {NULL}};
const Segment SGI56809[] = {{&S101869[97], 5}, {NULL}};
const Segment SGI56810[] = {{&S99194[44], 20}, {NULL}};
const Segment SGI56811[] = {{&S99194[44], 31}, {NULL}};
const Segment SGI56812[] = {{&S99194[30], 6}, {NULL}};
const Segment SGI56813[] = {{&S99194[123], 3}, {NULL}};
const Segment SGI56814[] = {{&S99194[89], 5}, {NULL}};
const Segment SGI56815[] = {{&S99194[147], 11}, {NULL}};
const Segment SGI56816[] = {{&S99208[59], 14}, {NULL}};
const Segment SGI56817[] = {{&S99208[59], 31}, {NULL}};
const Segment SGI56818[] = {{&S99208[45], 6}, {NULL}};
const Segment SGI56819[] = {{&S99208[138], 3}, {NULL}};
const Segment SGI56820[] = {{&S99208[104], 5}, {NULL}};
const Segment SGI56821[] = {{&S99208[0], 14}, {NULL}};
const Segment SGI56822[] = {{&S99231[44], 6}, {NULL}};
const Segment SGI56823[] = {{&S99231[44], 17}, {NULL}};
const Segment SGI56824[] = {{&S99231[30], 6}, {NULL}};
const Segment SGI56825[] = {{&S99231[75], 6}, {NULL}};
const Segment SGI56826[] = {{&S99231[102], 13}, {NULL}};
const Segment SGI56827[] = {{&S99283[44], 4}, {NULL}};
const Segment SGI56828[] = {{&S99283[44], 25}, {NULL}};
const Segment SGI56829[] = {{&S99283[30], 6}, {NULL}};
const Segment SGI56830[] = {{&S99283[117], 3}, {NULL}};
const Segment SGI56831[] = {{&S99283[83], 5}, {NULL}};
const Segment SGI56832[] = {{&S99325[34], 6}, {NULL}};
const Segment SGI56833[] = {{&S99325[27], 6}, {NULL}};
const Segment SGI56834[] = {{&S99325[105], 2}, {NULL}};
const Segment SGI56835[] = {{&S99325[71], 6}, {NULL}};
const Segment SGI56836[] = {{&S99325[139], 8}, {NULL}};
const Segment SGI56837[] = {{&S99355[44], 18}, {NULL}};
const Segment SGI56838[] = {{&S99355[44], 35}, {NULL}};
const Segment SGI56839[] = {{&S99355[30], 6}, {NULL}};
const Segment SGI56840[] = {{&S99355[93], 6}, {NULL}};
const Segment SGI56841[] = {{&S99355[120], 13}, {NULL}};
const Segment SGI56842[] = {{&S99291[45], 5}, {NULL}};
const Segment SGI56843[] = {{&S99291[30], 6}, {NULL}};
const Segment SGI56844[] = {{&S99291[109], 3}, {NULL}};
const Segment SGI56845[] = {{&S99291[75], 5}, {NULL}};
const Segment SGI56846[] = {{&S99291[133], 13}, {NULL}};
const Segment SGI56847[] = {{&S102070[59], 18}, {NULL}};
const Segment SGI56848[] = {{&S102070[59], 29}, {NULL}};
const Segment SGI56849[] = {{&S102070[45], 6}, {NULL}};
const Segment SGI56850[] = {{&S102070[136], 3}, {NULL}};
const Segment SGI56851[] = {{&S102070[102], 5}, {NULL}};
const Segment SGI56852[] = {{&S99315[44], 7}, {NULL}};
const Segment SGI56853[] = {{&S99315[44], 25}, {NULL}};
const Segment SGI56854[] = {{&S102070[0], 14}, {NULL}};
const Segment SGI56855[] = {{&S99315[30], 6}, {NULL}};
const Segment SGI56856[] = {{&S99315[83], 6}, {NULL}};
const Segment SGI56857[] = {{&S99315[110], 13}, {NULL}};
const Segment SGI56858[] = {{&S102074[44], 10}, {NULL}};
const Segment SGI56859[] = {{&S102074[44], 27}, {NULL}};
const Segment SGI56860[] = {{&S102074[30], 6}, {NULL}};
const Segment SGI56861[] = {{&S102074[126], 3}, {NULL}};
const Segment SGI56862[] = {{&S102074[92], 5}, {NULL}};
const Segment SGI56863[] = {{&S102074[71], 8}, {NULL}};
const Segment SGI56864[] = {{&S99407[44], 14}, {NULL}};
const Segment SGI56865[] = {{&S99407[44], 25}, {NULL}};
const Segment SGI56866[] = {{&S99407[30], 6}, {NULL}};
const Segment SGI56867[] = {{&S99407[83], 6}, {NULL}};
const Segment SGI56868[] = {{&S99407[110], 13}, {NULL}};
const Segment SGI56869[] = {{&S99333[44], 5}, {NULL}};
const Segment SGI56870[] = {{&S99333[44], 16}, {NULL}};
const Segment SGI56871[] = {{&S99333[30], 6}, {NULL}};
const Segment SGI56872[] = {{&S99333[74], 6}, {NULL}};
const Segment SGI56873[] = {{&S99333[101], 13}, {NULL}};
const Segment SGI56874[] = {{&S99416[52], 5}, {NULL}};
const Segment SGI56875[] = {{&S99416[52], 22}, {NULL}};
const Segment SGI56876[] = {{&S102095[44], 10}, {NULL}};
const Segment SGI56877[] = {{&S102095[44], 27}, {NULL}};
const Segment SGI56878[] = {{&S99416[38], 6}, {NULL}};
const Segment SGI56879[] = {{&S102095[30], 6}, {NULL}};
const Segment SGI56880[] = {{&S102095[164], 3}, {NULL}};
const Segment SGI56881[] = {{&S102095[130], 3}, {NULL}};
const Segment SGI56882[] = {{&S99416[88], 5}, {NULL}};
const Segment SGI56883[] = {{&S102095[95], 6}, {NULL}};
const Segment SGI56884[] = {{&S99416[0], 14}, {NULL}};
const Segment SGI56885[] = {{&S102095[150], 8}, {NULL}};
const Segment SGI56886[] = {{&S99343[44], 7}, {NULL}};
const Segment SGI56887[] = {{&S99343[44], 25}, {NULL}};
const Segment SGI56888[] = {{&S99343[30], 6}, {NULL}};
const Segment SGI56889[] = {{&S99343[83], 6}, {NULL}};
const Segment SGI56890[] = {{&S99343[110], 13}, {NULL}};
const Segment SGI56891[] = {{&S99456[44], 12}, {NULL}};
const Segment SGI56892[] = {{&S99456[44], 33}, {NULL}};
const Segment SGI56893[] = {{&S99456[30], 6}, {NULL}};
const Segment SGI56894[] = {{&S99456[124], 3}, {NULL}};
const Segment SGI56895[] = {{&S99456[91], 5}, {NULL}};
const Segment SGI56896[] = {{&S99456[128], 14}, {NULL}};
const Segment SGI56897[] = {{&S99459[44], 11}, {NULL}};
const Segment SGI56898[] = {{&S99459[44], 28}, {NULL}};
const Segment SGI56899[] = {{&S99459[30], 6}, {NULL}};
const Segment SGI56900[] = {{&S99459[119], 3}, {NULL}};
const Segment SGI56901[] = {{&S99459[86], 5}, {NULL}};
const Segment SGI56902[] = {{&S99459[123], 14}, {NULL}};
const Segment SGI56903[] = {{&S102182[44], 10}, {NULL}};
const Segment SGI56904[] = {{&S102182[44], 27}, {NULL}};
const Segment SGI56905[] = {{&S102182[30], 6}, {NULL}};
const Segment SGI56906[] = {{&S102182[119], 3}, {NULL}};
const Segment SGI56907[] = {{&S102182[85], 5}, {NULL}};
const Segment SGI56908[] = {{&S102241[45], 6}, {NULL}};
const Segment SGI56909[] = {{&S102241[30], 6}, {NULL}};
const Segment SGI56910[] = {{&S102241[117], 3}, {NULL}};
const Segment SGI56911[] = {{&S102241[83], 5}, {NULL}};
const Segment SGI56912[] = {{&S102241[151], 3}, {NULL}};
const Segment SGI56913[] = {{&S102252[44], 14}, {NULL}};
const Segment SGI56914[] = {{&S102252[44], 25}, {NULL}};
const Segment SGI56915[] = {{&S102252[30], 6}, {NULL}};
const Segment SGI56916[] = {{&S102252[161], 3}, {NULL}};
const Segment SGI56917[] = {{&S102252[127], 3}, {NULL}};
const Segment SGI56918[] = {{&S102252[92], 6}, {NULL}};
const Segment SGI56919[] = {{&S102252[147], 8}, {NULL}};
const Segment SGI56920[] = {{&S99607[45], 10}, {NULL}};
const Segment SGI56921[] = {{&S99607[30], 6}, {NULL}};
const Segment SGI56922[] = {{&S99607[120], 3}, {NULL}};
const Segment SGI56923[] = {{&S99607[86], 5}, {NULL}};
const Segment SGI56924[] = {{&S99607[144], 13}, {NULL}};
const Segment SGI56925[] = {{&S99688[59], 9}, {NULL}};
const Segment SGI56926[] = {{&S99688[59], 20}, {NULL}};
const Segment SGI56927[] = {{&S99688[45], 6}, {NULL}};
const Segment SGI56928[] = {{&S99688[127], 3}, {NULL}};
const Segment SGI56929[] = {{&S99688[93], 5}, {NULL}};
const Segment SGI56930[] = {{&S99688[0], 14}, {NULL}};
const Segment SGI56931[] = {{&S99658[57], 7}, {NULL}};
const Segment SGI56932[] = {{&S99658[57], 25}, {NULL}};
const Segment SGI56933[] = {{&S99658[43], 6}, {NULL}};
const Segment SGI56934[] = {{&S99658[70], 3}, {NULL}};
const Segment SGI56935[] = {{&S99658[96], 5}, {NULL}};
const Segment SGI56936[] = {{&S99658[0], 12}, {NULL}};
const Segment SGI56937[] = {{&S99722[59], 22}, {NULL}};
const Segment SGI56938[] = {{&S99722[59], 33}, {NULL}};
const Segment SGI56939[] = {{&S99722[45], 6}, {NULL}};
const Segment SGI56940[] = {{&S99659[45], 10}, {NULL}};
const Segment SGI56941[] = {{&S99659[30], 6}, {NULL}};
const Segment SGI56942[] = {{&S99722[140], 3}, {NULL}};
const Segment SGI56943[] = {{&S99722[106], 5}, {NULL}};
const Segment SGI56944[] = {{&S99659[120], 3}, {NULL}};
const Segment SGI56945[] = {{&S99722[0], 14}, {NULL}};
const Segment SGI56946[] = {{&S99659[86], 5}, {NULL}};
const Segment SGI56947[] = {{&S99659[144], 13}, {NULL}};
const Segment SGI56948[] = {{&S99747[59], 17}, {NULL}};
const Segment SGI56949[] = {{&S99747[59], 28}, {NULL}};
const Segment SGI56950[] = {{&S99747[45], 6}, {NULL}};
const Segment SGI56951[] = {{&S99747[135], 3}, {NULL}};
const Segment SGI56952[] = {{&S99747[101], 5}, {NULL}};
const Segment SGI56953[] = {{&S99747[0], 14}, {NULL}};
const Segment SGI56954[] = {{&S102476[56], 10}, {NULL}};
const Segment SGI56955[] = {{&S102476[56], 27}, {NULL}};
const Segment SGI56956[] = {{&S102476[42], 6}, {NULL}};
const Segment SGI56957[] = {{&S102476[131], 3}, {NULL}};
const Segment SGI56958[] = {{&S102476[97], 5}, {NULL}};
const Segment SGI56959[] = {{&S102476[0], 11}, {NULL}};
const Segment SGI56960[] = {{&S102483[59], 12}, {NULL}};
const Segment SGI56961[] = {{&S102483[59], 23}, {NULL}};
const Segment SGI56962[] = {{&S102483[45], 6}, {NULL}};
const Segment SGI56963[] = {{&S102483[130], 3}, {NULL}};
const Segment SGI56964[] = {{&S102483[96], 5}, {NULL}};
const Segment SGI56965[] = {{&S99720[45], 9}, {NULL}};
const Segment SGI56966[] = {{&S102483[0], 14}, {NULL}};
const Segment SGI56967[] = {{&S99720[30], 6}, {NULL}};
const Segment SGI56968[] = {{&S99720[113], 3}, {NULL}};
const Segment SGI56969[] = {{&S99720[79], 5}, {NULL}};
const Segment SGI56970[] = {{&S99720[137], 13}, {NULL}};
const Segment SGI56971[] = {{&S99723[45], 7}, {NULL}};
const Segment SGI56972[] = {{&S99723[30], 6}, {NULL}};
const Segment SGI56973[] = {{&S99723[58], 3}, {NULL}};
const Segment SGI56974[] = {{&S99723[84], 5}, {NULL}};
const Segment SGI56975[] = {{&S99723[142], 13}, {NULL}};
const Segment SGI56976[] = {{&S99786[44], 21}, {NULL}};
const Segment SGI56977[] = {{&S99786[44], 32}, {NULL}};
const Segment SGI56978[] = {{&S99786[30], 6}, {NULL}};
const Segment SGI56979[] = {{&S102495[57], 6}, {NULL}};
const Segment SGI56980[] = {{&S102495[57], 24}, {NULL}};
const Segment SGI56981[] = {{&S102495[43], 6}, {NULL}};
const Segment SGI56982[] = {{&S99786[123], 3}, {NULL}};
const Segment SGI56983[] = {{&S99786[90], 5}, {NULL}};
const Segment SGI56984[] = {{&S102495[129], 3}, {NULL}};
const Segment SGI56985[] = {{&S99786[127], 14}, {NULL}};
const Segment SGI56986[] = {{&S102495[95], 5}, {NULL}};
const Segment SGI56987[] = {{&S102495[0], 10}, {NULL}};
const Segment SGI56988[] = {{&S99736[59], 7}, {NULL}};
const Segment SGI56989[] = {{&S99736[59], 25}, {NULL}};
const Segment SGI56990[] = {{&S99736[45], 6}, {NULL}};
const Segment SGI56991[] = {{&S99736[72], 3}, {NULL}};
const Segment SGI56992[] = {{&S99736[98], 5}, {NULL}};
const Segment SGI56993[] = {{&S99736[0], 14}, {NULL}};
const Segment SGI56994[] = {{&S99807[44], 13}, {NULL}};
const Segment SGI56995[] = {{&S99807[44], 24}, {NULL}};
const Segment SGI56996[] = {{&S99807[30], 6}, {NULL}};
const Segment SGI56997[] = {{&S99807[82], 6}, {NULL}};
const Segment SGI56998[] = {{&S99807[109], 13}, {NULL}};
const Segment SGI56999[] = {{&S99858[59], 13}, {NULL}};
const Segment SGI57000[] = {{&S99858[59], 24}, {NULL}};
const Segment SGI57001[] = {{&S99858[45], 6}, {NULL}};
const Segment SGI57002[] = {{&S99858[131], 3}, {NULL}};
const Segment SGI57003[] = {{&S99858[97], 5}, {NULL}};
const Segment SGI57004[] = {{&S99858[0], 14}, {NULL}};
const Segment SGI57005[] = {{&S99816[44], 9}, {NULL}};
const Segment SGI57006[] = {{&S99816[44], 20}, {NULL}};
const Segment SGI57007[] = {{&S99816[30], 6}, {NULL}};
const Segment SGI57008[] = {{&S99816[112], 3}, {NULL}};
const Segment SGI57009[] = {{&S99816[78], 5}, {NULL}};
const Segment SGI57010[] = {{&S99816[136], 11}, {NULL}};
const Segment SGI57011[] = {{&S99870[44], 7}, {NULL}};
const Segment SGI57012[] = {{&S99870[44], 32}, {NULL}};
const Segment SGI57013[] = {{&S99870[30], 6}, {NULL}};
const Segment SGI57014[] = {{&S99870[124], 3}, {NULL}};
const Segment SGI57015[] = {{&S99870[90], 5}, {NULL}};
const Segment SGI57016[] = {{&S99870[147], 10}, {NULL}};
const Segment SGI57017[] = {{&S99885[44], 13}, {NULL}};
const Segment SGI57018[] = {{&S99885[44], 24}, {NULL}};
const Segment SGI57019[] = {{&S99885[30], 6}, {NULL}};
const Segment SGI57020[] = {{&S99885[115], 3}, {NULL}};
const Segment SGI57021[] = {{&S99885[82], 5}, {NULL}};
const Segment SGI57022[] = {{&S99885[119], 14}, {NULL}};
const Segment SGI57023[] = {{&S99881[34], 7}, {NULL}};
const Segment SGI57024[] = {{&S99881[27], 6}, {NULL}};
const Segment SGI57025[] = {{&S99881[107], 2}, {NULL}};
const Segment SGI57026[] = {{&S99881[73], 6}, {NULL}};
const Segment SGI57027[] = {{&S99881[141], 8}, {NULL}};
const Segment SGI57028[] = {{&S99925[44], 14}, {NULL}};
const Segment SGI57029[] = {{&S99925[44], 35}, {NULL}};
const Segment SGI57030[] = {{&S99925[30], 6}, {NULL}};
const Segment SGI57031[] = {{&S99925[93], 6}, {NULL}};
const Segment SGI57032[] = {{&S102650[45], 10}, {NULL}};
const Segment SGI57033[] = {{&S99925[120], 13}, {NULL}};
const Segment SGI57034[] = {{&S102650[30], 6}, {NULL}};
const Segment SGI57035[] = {{&S102650[120], 3}, {NULL}};
const Segment SGI57036[] = {{&S102650[86], 5}, {NULL}};
const Segment SGI57037[] = {{&S102650[154], 3}, {NULL}};
const Segment SGI57038[] = {{&S99949[52], 14}, {NULL}};
const Segment SGI57039[] = {{&S99949[52], 35}, {NULL}};
const Segment SGI57040[] = {{&S99949[38], 6}, {NULL}};
const Segment SGI57041[] = {{&S99949[101], 5}, {NULL}};
const Segment SGI57042[] = {{&S99949[0], 14}, {NULL}};
const Segment SGI57043[] = {{&S99989[59], 5}, {NULL}};
const Segment SGI57044[] = {{&S99989[59], 22}, {NULL}};
const Segment SGI57045[] = {{&S99989[45], 6}, {NULL}};
const Segment SGI57046[] = {{&S99989[129], 3}, {NULL}};
const Segment SGI57047[] = {{&S99989[95], 5}, {NULL}};
const Segment SGI57048[] = {{&S99989[0], 14}, {NULL}};
const Segment SGI57049[] = {{&S99990[59], 8}, {NULL}};
const Segment SGI57050[] = {{&S99990[59], 29}, {NULL}};
const Segment SGI57051[] = {{&S99990[45], 6}, {NULL}};
const Segment SGI57052[] = {{&S99990[136], 3}, {NULL}};
const Segment SGI57053[] = {{&S99990[102], 5}, {NULL}};
const Segment SGI57054[] = {{&S99990[0], 14}, {NULL}};
const Segment SGI57055[] = {{&S102724[57], 9}, {NULL}};
const Segment SGI57056[] = {{&S102724[57], 20}, {NULL}};
const Segment SGI57057[] = {{&S102724[43], 6}, {NULL}};
const Segment SGI57058[] = {{&S102724[125], 3}, {NULL}};
const Segment SGI57059[] = {{&S102724[91], 5}, {NULL}};
const Segment SGI57060[] = {{&S102724[0], 10}, {NULL}};
const Segment SGI57061[] = {{&S104817[44], 6}, {NULL}};
const Segment SGI57062[] = {{&S104817[44], 18}, {NULL}};
const Segment SGI57063[] = {{&S104817[30], 6}, {NULL}};
const Segment SGI57064[] = {{&S104817[110], 3}, {NULL}};
const Segment SGI57065[] = {{&S104817[76], 5}, {NULL}};
const Segment SGI57066[] = {{&S100059[44], 10}, {NULL}};
const Segment SGI57067[] = {{&S100059[44], 22}, {NULL}};
const Segment SGI57068[] = {{&S100059[30], 6}, {NULL}};
const Segment SGI57069[] = {{&S100059[115], 3}, {NULL}};
const Segment SGI57070[] = {{&S100059[80], 6}, {NULL}};
const Segment SGI57071[] = {{&S100059[135], 8}, {NULL}};
const Segment SGI57072[] = {{&S100059[144], 13}, {NULL}};
const Segment SGI57073[] = {{&S100062[44], 18}, {NULL}};
const Segment SGI57074[] = {{&S100062[44], 29}, {NULL}};
const Segment SGI57075[] = {{&S100062[30], 6}, {NULL}};
const Segment SGI57076[] = {{&S100062[87], 6}, {NULL}};
const Segment SGI57077[] = {{&S100062[114], 13}, {NULL}};
const Segment SGI57078[] = {{&S100065[44], 14}, {NULL}};
const Segment SGI57079[] = {{&S100065[44], 35}, {NULL}};
const Segment SGI57080[] = {{&S100065[30], 6}, {NULL}};
const Segment SGI57081[] = {{&S100065[93], 6}, {NULL}};
const Segment SGI57082[] = {{&S100065[120], 13}, {NULL}};
const Segment SGI57083[] = {{&S100093[44], 7}, {NULL}};
const Segment SGI57084[] = {{&S100093[44], 25}, {NULL}};
const Segment SGI57085[] = {{&S100093[30], 6}, {NULL}};
const Segment SGI57086[] = {{&S100093[83], 6}, {NULL}};
const Segment SGI57087[] = {{&S100093[110], 13}, {NULL}};
const Segment SGI57088[] = {{&S102874[57], 10}, {NULL}};
const Segment SGI57089[] = {{&S102874[57], 27}, {NULL}};
const Segment SGI57090[] = {{&S102874[43], 6}, {NULL}};
const Segment SGI57091[] = {{&S102874[132], 3}, {NULL}};
const Segment SGI57092[] = {{&S102874[98], 5}, {NULL}};
const Segment SGI57093[] = {{&S102874[0], 10}, {NULL}};
const Segment SGI57094[] = {{&S102875[59], 12}, {NULL}};
const Segment SGI57095[] = {{&S102875[59], 23}, {NULL}};
const Segment SGI57096[] = {{&S102875[45], 6}, {NULL}};
const Segment SGI57097[] = {{&S102875[130], 3}, {NULL}};
const Segment SGI57098[] = {{&S102875[96], 5}, {NULL}};
const Segment SGI57099[] = {{&S102875[0], 14}, {NULL}};
const Segment SGI57100[] = {{&S100201[44], 12}, {NULL}};
const Segment SGI57101[] = {{&S100201[44], 23}, {NULL}};
const Segment SGI57102[] = {{&S100201[30], 6}, {NULL}};
const Segment SGI57103[] = {{&S100201[115], 3}, {NULL}};
const Segment SGI57104[] = {{&S100201[81], 5}, {NULL}};
const Segment SGI57105[] = {{&S100203[45], 7}, {NULL}};
const Segment SGI57106[] = {{&S100203[30], 6}, {NULL}};
const Segment SGI57107[] = {{&S100203[58], 3}, {NULL}};
const Segment SGI57108[] = {{&S100203[84], 5}, {NULL}};
const Segment SGI57109[] = {{&S100203[142], 13}, {NULL}};
const Segment SGI57110[] = {{&S100264[59], 13}, {NULL}};
const Segment SGI57111[] = {{&S100264[59], 24}, {NULL}};
const Segment SGI57112[] = {{&S100264[45], 6}, {NULL}};
const Segment SGI57113[] = {{&S100264[131], 3}, {NULL}};
const Segment SGI57114[] = {{&S100264[97], 5}, {NULL}};
const Segment SGI57115[] = {{&S100264[0], 14}, {NULL}};
const Segment SGI57116[] = {{&S100301[45], 5}, {NULL}};
const Segment SGI57117[] = {{&S100301[30], 6}, {NULL}};
const Segment SGI57118[] = {{&S100301[109], 3}, {NULL}};
const Segment SGI57119[] = {{&S100301[75], 5}, {NULL}};
const Segment SGI57120[] = {{&S100301[133], 13}, {NULL}};
const Segment SGI57121[] = {{&S100332[45], 9}, {NULL}};
const Segment SGI57122[] = {{&S100332[30], 6}, {NULL}};
const Segment SGI57123[] = {{&S100332[113], 3}, {NULL}};
const Segment SGI57124[] = {{&S100332[79], 5}, {NULL}};
const Segment SGI57125[] = {{&S100332[137], 13}, {NULL}};
const Segment SGI57126[] = {{&S100347[45], 9}, {NULL}};
const Segment SGI57127[] = {{&S100347[30], 6}, {NULL}};
const Segment SGI57128[] = {{&S100347[113], 3}, {NULL}};
const Segment SGI57129[] = {{&S100347[79], 5}, {NULL}};
const Segment SGI57130[] = {{&S100347[137], 13}, {NULL}};
const Segment SGI57131[] = {{&S103102[59], 18}, {NULL}};
const Segment SGI57132[] = {{&S103102[59], 29}, {NULL}};
const Segment SGI57133[] = {{&S103102[45], 6}, {NULL}};
const Segment SGI57134[] = {{&S103102[136], 3}, {NULL}};
const Segment SGI57135[] = {{&S103102[102], 5}, {NULL}};
const Segment SGI57136[] = {{&S103102[0], 14}, {NULL}};
const Segment SGI57137[] = {{&S100412[45], 10}, {NULL}};
const Segment SGI57138[] = {{&S100412[30], 6}, {NULL}};
const Segment SGI57139[] = {{&S100412[115], 3}, {NULL}};
const Segment SGI57140[] = {{&S100412[81], 5}, {NULL}};
const Segment SGI57141[] = {{&S100412[139], 13}, {NULL}};
const Segment SGI57142[] = {{&S103168[59], 10}, {NULL}};
const Segment SGI57143[] = {{&S103168[59], 27}, {NULL}};
const Segment SGI57144[] = {{&S103168[45], 6}, {NULL}};
const Segment SGI57145[] = {{&S103168[134], 3}, {NULL}};
const Segment SGI57146[] = {{&S103168[100], 5}, {NULL}};
const Segment SGI57147[] = {{&S103168[0], 14}, {NULL}};
const Segment SGI57148[] = {{&S100426[44], 14}, {NULL}};
const Segment SGI57149[] = {{&S100426[44], 35}, {NULL}};
const Segment SGI57150[] = {{&S100426[30], 6}, {NULL}};
const Segment SGI57151[] = {{&S100426[127], 3}, {NULL}};
const Segment SGI57152[] = {{&S100426[93], 5}, {NULL}};
const Segment SGI57153[] = {{&S100426[151], 13}, {NULL}};
const Segment SGI57154[] = {{&S100409[59], 10}, {NULL}};
const Segment SGI57155[] = {{&S100409[59], 27}, {NULL}};
const Segment SGI57156[] = {{&S100409[45], 6}, {NULL}};
const Segment SGI57157[] = {{&S100409[134], 3}, {NULL}};
const Segment SGI57158[] = {{&S100409[100], 5}, {NULL}};
const Segment SGI57159[] = {{&S100409[0], 14}, {NULL}};
const Segment SGI57160[] = {{&S100438[44], 21}, {NULL}};
const Segment SGI57161[] = {{&S100438[44], 32}, {NULL}};
const Segment SGI57162[] = {{&S100438[30], 6}, {NULL}};
const Segment SGI57163[] = {{&S100438[123], 3}, {NULL}};
const Segment SGI57164[] = {{&S100438[90], 5}, {NULL}};
const Segment SGI57165[] = {{&S100438[127], 14}, {NULL}};
const Segment SGI57166[] = {{&S100436[44], 5}, {NULL}};
const Segment SGI57167[] = {{&S100436[44], 16}, {NULL}};
const Segment SGI57168[] = {{&S100436[30], 6}, {NULL}};
const Segment SGI57169[] = {{&S100436[74], 6}, {NULL}};
const Segment SGI57170[] = {{&S100436[101], 13}, {NULL}};
const Segment SGI57171[] = {{&S100439[44], 9}, {NULL}};
const Segment SGI57172[] = {{&S100439[44], 20}, {NULL}};
const Segment SGI57173[] = {{&S100439[30], 6}, {NULL}};
const Segment SGI57174[] = {{&S100439[78], 6}, {NULL}};
const Segment SGI57175[] = {{&S100439[105], 13}, {NULL}};
const Segment SGI57176[] = {{&S100465[44], 14}, {NULL}};
const Segment SGI57177[] = {{&S100465[44], 31}, {NULL}};
const Segment SGI57178[] = {{&S100465[30], 6}, {NULL}};
const Segment SGI57179[] = {{&S100465[124], 3}, {NULL}};
const Segment SGI57180[] = {{&S100465[89], 6}, {NULL}};
const Segment SGI57181[] = {{&S100465[144], 8}, {NULL}};
const Segment SGI57182[] = {{&S100465[153], 13}, {NULL}};
const Segment SGI57183[] = {{&S100443[44], 9}, {NULL}};
const Segment SGI57184[] = {{&S100443[44], 20}, {NULL}};
const Segment SGI57185[] = {{&S100443[30], 6}, {NULL}};
const Segment SGI57186[] = {{&S100443[78], 6}, {NULL}};
const Segment SGI57187[] = {{&S100443[105], 13}, {NULL}};
const Segment SGI57188[] = {{&S100476[44], 5}, {NULL}};
const Segment SGI57189[] = {{&S100476[44], 22}, {NULL}};
const Segment SGI57190[] = {{&S100476[30], 6}, {NULL}};
const Segment SGI57191[] = {{&S100476[80], 6}, {NULL}};
const Segment SGI57192[] = {{&S100476[107], 13}, {NULL}};
const Segment SGI57193[] = {{&S100487[44], 14}, {NULL}};
const Segment SGI57194[] = {{&S100487[44], 35}, {NULL}};
const Segment SGI57195[] = {{&S100487[30], 6}, {NULL}};
const Segment SGI57196[] = {{&S100487[93], 6}, {NULL}};
const Segment SGI57197[] = {{&S100487[120], 13}, {NULL}};
const Segment SGI57198[] = {{&S100482[34], 7}, {NULL}};
const Segment SGI57199[] = {{&S100482[27], 6}, {NULL}};
const Segment SGI57200[] = {{&S100482[107], 2}, {NULL}};
const Segment SGI57201[] = {{&S100482[73], 6}, {NULL}};
const Segment SGI57202[] = {{&S100482[141], 8}, {NULL}};
const Segment SGI57203[] = {{&S100539[59], 14}, {NULL}};
const Segment SGI57204[] = {{&S100539[59], 31}, {NULL}};
const Segment SGI57205[] = {{&S100539[45], 6}, {NULL}};
const Segment SGI57206[] = {{&S100539[138], 3}, {NULL}};
const Segment SGI57207[] = {{&S100539[104], 5}, {NULL}};
const Segment SGI57208[] = {{&S100539[0], 14}, {NULL}};
const Segment SGI57209[] = {{&S100554[44], 41}, {NULL}};
const Segment SGI57210[] = {{&S100554[44], 52}, {NULL}};
const Segment SGI57211[] = {{&S100554[30], 6}, {NULL}};
const Segment SGI57212[] = {{&S100554[110], 6}, {NULL}};
const Segment SGI57213[] = {{&S100554[137], 13}, {NULL}};
const Segment SGI57214[] = {{&S100585[44], 10}, {NULL}};
const Segment SGI57215[] = {{&S100585[44], 30}, {NULL}};
const Segment SGI57216[] = {{&S100585[30], 6}, {NULL}};
const Segment SGI57217[] = {{&S100585[88], 6}, {NULL}};
const Segment SGI57218[] = {{&S100585[115], 13}, {NULL}};
const Segment SGI57219[] = {{&S100608[59], 14}, {NULL}};
const Segment SGI57220[] = {{&S100608[59], 35}, {NULL}};
const Segment SGI57221[] = {{&S100608[45], 6}, {NULL}};
const Segment SGI57222[] = {{&S100608[142], 3}, {NULL}};
const Segment SGI57223[] = {{&S100608[108], 5}, {NULL}};
const Segment SGI57224[] = {{&S100608[0], 14}, {NULL}};
const Segment SGI57225[] = {{&S100618[44], 5}, {NULL}};
const Segment SGI57226[] = {{&S100618[44], 16}, {NULL}};
const Segment SGI57227[] = {{&S100618[30], 6}, {NULL}};
const Segment SGI57228[] = {{&S100618[74], 6}, {NULL}};
const Segment SGI57229[] = {{&S100618[101], 13}, {NULL}};
const Segment SGI57230[] = {{&S100629[44], 5}, {NULL}};
const Segment SGI57231[] = {{&S100629[44], 22}, {NULL}};
const Segment SGI57232[] = {{&S100629[30], 6}, {NULL}};
const Segment SGI57233[] = {{&S100629[114], 3}, {NULL}};
const Segment SGI57234[] = {{&S100629[80], 5}, {NULL}};
const Segment SGI57235[] = {{&S100629[138], 13}, {NULL}};
const Segment SGI57236[] = {{&S100639[52], 22}, {NULL}};
const Segment SGI57237[] = {{&S100639[52], 33}, {NULL}};
const Segment SGI57238[] = {{&S100639[38], 6}, {NULL}};
const Segment SGI57239[] = {{&S100639[99], 5}, {NULL}};
const Segment SGI57240[] = {{&S100639[0], 14}, {NULL}};
const Segment SGI57241[] = {{&S100641[44], 10}, {NULL}};
const Segment SGI57242[] = {{&S100641[44], 22}, {NULL}};
const Segment SGI57243[] = {{&S100641[30], 6}, {NULL}};
const Segment SGI57244[] = {{&S100641[115], 3}, {NULL}};
const Segment SGI57245[] = {{&S100641[80], 6}, {NULL}};
const Segment SGI57246[] = {{&S100641[135], 8}, {NULL}};
const Segment SGI57247[] = {{&S100641[144], 13}, {NULL}};
const Segment SGI57248[] = {{&S100650[44], 13}, {NULL}};
const Segment SGI57249[] = {{&S100650[44], 34}, {NULL}};
const Segment SGI57250[] = {{&S100650[30], 6}, {NULL}};
const Segment SGI57251[] = {{&S100650[127], 3}, {NULL}};
const Segment SGI57252[] = {{&S100650[92], 6}, {NULL}};
const Segment SGI57253[] = {{&S100650[147], 8}, {NULL}};
const Segment SGI57254[] = {{&S100650[156], 13}, {NULL}};
const Segment SGI57255[] = {{&S100666[44], 14}, {NULL}};
const Segment SGI57256[] = {{&S100666[44], 35}, {NULL}};
const Segment SGI57257[] = {{&S100666[30], 6}, {NULL}};
const Segment SGI57258[] = {{&S100666[128], 3}, {NULL}};
const Segment SGI57259[] = {{&S100666[93], 6}, {NULL}};
const Segment SGI57260[] = {{&S100666[148], 8}, {NULL}};
const Segment SGI57261[] = {{&S100666[157], 13}, {NULL}};
const Segment SGI57262[] = {{&S100675[44], 9}, {NULL}};
const Segment SGI57263[] = {{&S100675[44], 20}, {NULL}};
const Segment SGI57264[] = {{&S100675[30], 6}, {NULL}};
const Segment SGI57265[] = {{&S100675[78], 6}, {NULL}};
const Segment SGI57266[] = {{&S100675[105], 13}, {NULL}};
const Segment SGI57267[] = {{&S100701[44], 7}, {NULL}};
const Segment SGI57268[] = {{&S100701[44], 25}, {NULL}};
const Segment SGI57269[] = {{&S100701[30], 6}, {NULL}};
const Segment SGI57270[] = {{&S100701[83], 6}, {NULL}};
const Segment SGI57271[] = {{&S100701[110], 13}, {NULL}};
const Segment SGI57272[] = {{&S100707[44], 7}, {NULL}};
const Segment SGI57273[] = {{&S100707[44], 25}, {NULL}};
const Segment SGI57274[] = {{&S100707[30], 6}, {NULL}};
const Segment SGI57275[] = {{&S100707[83], 6}, {NULL}};
const Segment SGI57276[] = {{&S100707[110], 13}, {NULL}};
const Segment SGI57277[] = {{&S100765[44], 8}, {NULL}};
const Segment SGI57278[] = {{&S100765[44], 19}, {NULL}};
const Segment SGI57279[] = {{&S100765[30], 6}, {NULL}};
const Segment SGI57280[] = {{&S100765[111], 3}, {NULL}};
const Segment SGI57281[] = {{&S100765[77], 5}, {NULL}};
const Segment SGI57282[] = {{&S100789[44], 8}, {NULL}};
const Segment SGI57283[] = {{&S100789[44], 19}, {NULL}};
const Segment SGI57284[] = {{&S100789[30], 6}, {NULL}};
const Segment SGI57285[] = {{&S100789[111], 3}, {NULL}};
const Segment SGI57286[] = {{&S100789[77], 5}, {NULL}};
const Segment SGI57287[] = {{&S100798[45], 7}, {NULL}};
const Segment SGI57288[] = {{&S100798[30], 6}, {NULL}};
const Segment SGI57289[] = {{&S100798[58], 3}, {NULL}};
const Segment SGI57290[] = {{&S100798[84], 5}, {NULL}};
const Segment SGI57291[] = {{&S100798[142], 13}, {NULL}};
const Segment SGI57292[] = {{&S100804[57], 7}, {NULL}};
const Segment SGI57293[] = {{&S100804[57], 25}, {NULL}};
const Segment SGI57294[] = {{&S100804[43], 6}, {NULL}};
const Segment SGI57295[] = {{&S100804[70], 3}, {NULL}};
const Segment SGI57296[] = {{&S100804[96], 5}, {NULL}};
const Segment SGI57297[] = {{&S100804[0], 12}, {NULL}};
const Segment SGI57298[] = {{&S103382[58], 10}, {NULL}};
const Segment SGI57299[] = {{&S103382[58], 27}, {NULL}};
const Segment SGI57300[] = {{&S103382[44], 6}, {NULL}};
const Segment SGI57301[] = {{&S103382[133], 3}, {NULL}};
const Segment SGI57302[] = {{&S103382[99], 5}, {NULL}};
const Segment SGI57303[] = {{&S100826[45], 7}, {NULL}};
const Segment SGI57304[] = {{&S100826[30], 6}, {NULL}};
const Segment SGI57305[] = {{&S100826[112], 3}, {NULL}};
const Segment SGI57306[] = {{&S100826[78], 5}, {NULL}};
const Segment SGI57307[] = {{&S100826[136], 13}, {NULL}};
const Segment SGI57308[] = {{&S100887[45], 9}, {NULL}};
const Segment SGI57309[] = {{&S100887[30], 6}, {NULL}};
const Segment SGI57310[] = {{&S100887[113], 3}, {NULL}};
const Segment SGI57311[] = {{&S100887[79], 5}, {NULL}};
const Segment SGI57312[] = {{&S100887[137], 13}, {NULL}};
const Segment SGI57313[] = {{&S100892[45], 9}, {NULL}};
const Segment SGI57314[] = {{&S100892[30], 6}, {NULL}};
const Segment SGI57315[] = {{&S100892[113], 3}, {NULL}};
const Segment SGI57316[] = {{&S100892[79], 5}, {NULL}};
const Segment SGI57317[] = {{&S100892[137], 13}, {NULL}};
const Segment SGI57318[] = {{&S100912[45], 9}, {NULL}};
const Segment SGI57319[] = {{&S100912[30], 6}, {NULL}};
const Segment SGI57320[] = {{&S100912[113], 3}, {NULL}};
const Segment SGI57321[] = {{&S100912[79], 5}, {NULL}};
const Segment SGI57322[] = {{&S100912[137], 13}, {NULL}};
const Segment SGI57323[] = {{&S100914[45], 9}, {NULL}};
const Segment SGI57324[] = {{&S100914[30], 6}, {NULL}};
const Segment SGI57325[] = {{&S100914[113], 3}, {NULL}};
const Segment SGI57326[] = {{&S100914[79], 5}, {NULL}};
const Segment SGI57327[] = {{&S100914[137], 13}, {NULL}};
const Segment SGI57328[] = {{&S100925[44], 7}, {NULL}};
const Segment SGI57329[] = {{&S100925[44], 25}, {NULL}};
const Segment SGI57330[] = {{&S100925[30], 6}, {NULL}};
const Segment SGI57331[] = {{&S100925[83], 6}, {NULL}};
const Segment SGI57332[] = {{&S100925[110], 13}, {NULL}};
const Segment SGI57333[] = {{&S100916[44], 11}, {NULL}};
const Segment SGI57334[] = {{&S100916[44], 28}, {NULL}};
const Segment SGI57335[] = {{&S100916[30], 6}, {NULL}};
const Segment SGI57336[] = {{&S100916[119], 3}, {NULL}};
const Segment SGI57337[] = {{&S100916[86], 5}, {NULL}};
const Segment SGI57338[] = {{&S100916[123], 14}, {NULL}};
const Segment SGI57339[] = {{&S100933[44], 11}, {NULL}};
const Segment SGI57340[] = {{&S100933[44], 22}, {NULL}};
const Segment SGI57341[] = {{&S100933[30], 6}, {NULL}};
const Segment SGI57342[] = {{&S100933[80], 6}, {NULL}};
const Segment SGI57343[] = {{&S100933[107], 13}, {NULL}};
const Segment SGI57344[] = {{&S101026[44], 13}, {NULL}};
const Segment SGI57345[] = {{&S101026[44], 24}, {NULL}};
const Segment SGI57346[] = {{&S101026[30], 6}, {NULL}};
const Segment SGI57347[] = {{&S101026[116], 3}, {NULL}};
const Segment SGI57348[] = {{&S101026[82], 5}, {NULL}};
const Segment SGI57349[] = {{&S101030[59], 10}, {NULL}};
const Segment SGI57350[] = {{&S101030[59], 27}, {NULL}};
const Segment SGI57351[] = {{&S101030[45], 6}, {NULL}};
const Segment SGI57352[] = {{&S101030[134], 3}, {NULL}};
const Segment SGI57353[] = {{&S101030[100], 5}, {NULL}};
const Segment SGI57354[] = {{&S101030[0], 14}, {NULL}};
const Segment SGI57355[] = {{&S103563[44], 19}, {NULL}};
const Segment SGI57356[] = {{&S103563[44], 30}, {NULL}};
const Segment SGI57357[] = {{&S103563[30], 6}, {NULL}};
const Segment SGI57358[] = {{&S103563[122], 3}, {NULL}};
const Segment SGI57359[] = {{&S103563[88], 5}, {NULL}};
const Segment SGI57360[] = {{&S103571[44], 10}, {NULL}};
const Segment SGI57361[] = {{&S103571[44], 27}, {NULL}};
const Segment SGI57362[] = {{&S103571[30], 6}, {NULL}};
const Segment SGI57363[] = {{&S103571[163], 3}, {NULL}};
const Segment SGI57364[] = {{&S103571[129], 3}, {NULL}};
const Segment SGI57365[] = {{&S103571[94], 6}, {NULL}};
const Segment SGI57366[] = {{&S103571[149], 8}, {NULL}};
const Segment SGI57367[] = {{&S101096[45], 7}, {NULL}};
const Segment SGI57368[] = {{&S101096[30], 6}, {NULL}};
const Segment SGI57369[] = {{&S101096[58], 3}, {NULL}};
const Segment SGI57370[] = {{&S101096[84], 5}, {NULL}};
const Segment SGI57371[] = {{&S101096[142], 13}, {NULL}};
const Segment SGI57372[] = {{&S101097[44], 14}, {NULL}};
const Segment SGI57373[] = {{&S101097[44], 31}, {NULL}};
const Segment SGI57374[] = {{&S101097[30], 6}, {NULL}};
const Segment SGI57375[] = {{&S101097[167], 3}, {NULL}};
const Segment SGI57376[] = {{&S101097[133], 3}, {NULL}};
const Segment SGI57377[] = {{&S101097[98], 6}, {NULL}};
const Segment SGI57378[] = {{&S101097[153], 8}, {NULL}};
const Segment SGI57379[] = {{&S101103[59], 7}, {NULL}};
const Segment SGI57380[] = {{&S101103[59], 32}, {NULL}};
const Segment SGI57381[] = {{&S101103[45], 6}, {NULL}};
const Segment SGI57382[] = {{&S101103[139], 3}, {NULL}};
const Segment SGI57383[] = {{&S101103[105], 5}, {NULL}};
const Segment SGI57384[] = {{&S101103[0], 14}, {NULL}};
const Segment SGI57385[] = {{&S103702[44], 10}, {NULL}};
const Segment SGI57386[] = {{&S103702[44], 27}, {NULL}};
const Segment SGI57387[] = {{&S103702[30], 6}, {NULL}};
const Segment SGI57388[] = {{&S103702[119], 3}, {NULL}};
const Segment SGI57389[] = {{&S103702[85], 5}, {NULL}};
const Segment SGI57390[] = {{&S103702[153], 3}, {NULL}};
const Segment SGI57391[] = {{&S103708[59], 37}, {NULL}};
const Segment SGI57392[] = {{&S103708[45], 6}, {NULL}};
const Segment SGI57393[] = {{&S103708[128], 5}, {NULL}};
const Segment SGI57394[] = {{&S103708[0], 14}, {NULL}};
const Segment SGI57395[] = {{&S103709[59], 37}, {NULL}};
const Segment SGI57396[] = {{&S103709[45], 6}, {NULL}};
const Segment SGI57397[] = {{&S103709[128], 5}, {NULL}};
const Segment SGI57398[] = {{&S103709[0], 14}, {NULL}};
const Segment SGI57399[] = {{&S101217[31], 19}, {NULL}};
const Segment SGI57400[] = {{&S101217[65], 3}, {NULL}};
const Segment SGI57401[] = {{&S101221[57], 7}, {NULL}};
const Segment SGI57402[] = {{&S101221[57], 25}, {NULL}};
const Segment SGI57403[] = {{&S101221[43], 6}, {NULL}};
const Segment SGI57404[] = {{&S101221[70], 3}, {NULL}};
const Segment SGI57405[] = {{&S101221[96], 5}, {NULL}};
const Segment SGI57406[] = {{&S101221[0], 12}, {NULL}};
const Segment SGI57407[] = {{&S103774[44], 10}, {NULL}};
const Segment SGI57408[] = {{&S103774[44], 27}, {NULL}};
const Segment SGI57409[] = {{&S103774[30], 6}, {NULL}};
const Segment SGI57410[] = {{&S103774[119], 3}, {NULL}};
const Segment SGI57411[] = {{&S103774[85], 5}, {NULL}};
const Segment SGI57412[] = {{&S103774[130], 14}, {NULL}};
const Segment SGI57413[] = {{&S101341[44], 9}, {NULL}};
const Segment SGI57414[] = {{&S101341[44], 20}, {NULL}};
const Segment SGI57415[] = {{&S101341[30], 6}, {NULL}};
const Segment SGI57416[] = {{&S101341[112], 3}, {NULL}};
const Segment SGI57417[] = {{&S101341[78], 5}, {NULL}};
const Segment SGI57418[] = {{&S101366[44], 9}, {NULL}};
const Segment SGI57419[] = {{&S101366[44], 20}, {NULL}};
const Segment SGI57420[] = {{&S101366[30], 6}, {NULL}};
const Segment SGI57421[] = {{&S101366[112], 3}, {NULL}};
const Segment SGI57422[] = {{&S101366[78], 5}, {NULL}};
const Segment SGI57423[] = {{&S101233[31], 3}, {NULL}};
const Segment SGI57424[] = {{&S101233[49], 3}, {NULL}};
const Segment SGI57425[] = {{&S101266[59], 11}, {NULL}};
const Segment SGI57426[] = {{&S101266[59], 28}, {NULL}};
const Segment SGI57427[] = {{&S9539[44], 10}, {NULL}};
const Segment SGI57428[] = {{&S101266[45], 6}, {NULL}};
const Segment SGI57429[] = {{&S9539[44], 27}, {NULL}};
const Segment SGI57430[] = {{&S9539[30], 6}, {NULL}};
const Segment SGI57431[] = {{&S101266[135], 3}, {NULL}};
const Segment SGI57432[] = {{&S9539[119], 3}, {NULL}};
const Segment SGI57433[] = {{&S101266[101], 5}, {NULL}};
const Segment SGI57434[] = {{&S9539[85], 5}, {NULL}};
const Segment SGI57435[] = {{&S101266[0], 14}, {NULL}};
const Segment SGI57436[] = {{&S9543[44], 28}, {NULL}};
const Segment SGI57437[] = {{&S9543[44], 45}, {NULL}};
const Segment SGI57438[] = {{&S9543[30], 6}, {NULL}};
const Segment SGI57439[] = {{&S9543[137], 3}, {NULL}};
const Segment SGI57440[] = {{&S103933[58], 10}, {NULL}};
const Segment SGI57441[] = {{&S103933[58], 27}, {NULL}};
const Segment SGI57442[] = {{&S103933[44], 6}, {NULL}};
const Segment SGI57443[] = {{&S103933[133], 3}, {NULL}};
const Segment SGI57444[] = {{&S103933[99], 5}, {NULL}};
const Segment SGI57445[] = {{&S9543[103], 5}, {NULL}};
const Segment SGI57446[] = {{&S9546[44], 28}, {NULL}};
const Segment SGI57447[] = {{&S103995[44], 11}, {NULL}};
const Segment SGI57448[] = {{&S9546[44], 45}, {NULL}};
const Segment SGI57449[] = {{&S103995[44], 22}, {NULL}};
const Segment SGI57450[] = {{&S9546[30], 6}, {NULL}};
const Segment SGI57451[] = {{&S103995[30], 6}, {NULL}};
const Segment SGI57452[] = {{&S103995[114], 3}, {NULL}};
const Segment SGI57453[] = {{&S9546[137], 3}, {NULL}};
const Segment SGI57454[] = {{&S103995[80], 5}, {NULL}};
const Segment SGI57455[] = {{&S9546[103], 5}, {NULL}};
const Segment SGI57456[] = {{&S9547[44], 11}, {NULL}};
const Segment SGI57457[] = {{&S9547[44], 28}, {NULL}};
const Segment SGI57458[] = {{&S9547[30], 6}, {NULL}};
const Segment SGI57459[] = {{&S9547[120], 3}, {NULL}};
const Segment SGI57460[] = {{&S9547[86], 5}, {NULL}};
const Segment SGI57461[] = {{&S9548[44], 11}, {NULL}};
const Segment SGI57462[] = {{&S9548[44], 28}, {NULL}};
const Segment SGI57463[] = {{&S9548[30], 6}, {NULL}};
const Segment SGI57464[] = {{&S9548[120], 3}, {NULL}};
const Segment SGI57465[] = {{&S103998[59], 37}, {NULL}};
const Segment SGI57466[] = {{&S103998[45], 6}, {NULL}};
const Segment SGI57467[] = {{&S9548[86], 5}, {NULL}};
const Segment SGI57468[] = {{&S103998[128], 5}, {NULL}};
const Segment SGI57469[] = {{&S103998[0], 14}, {NULL}};
const Segment SGI57470[] = {{&S9550[38], 10}, {NULL}};
const Segment SGI57471[] = {{&S9550[31], 6}, {NULL}};
const Segment SGI57472[] = {{&S9553[44], 11}, {NULL}};
const Segment SGI57473[] = {{&S9553[44], 28}, {NULL}};
const Segment SGI57474[] = {{&S9553[30], 6}, {NULL}};
const Segment SGI57475[] = {{&S9553[120], 3}, {NULL}};
const Segment SGI57476[] = {{&S9553[86], 5}, {NULL}};
const Segment SGI57477[] = {{&S9560[44], 11}, {NULL}};
const Segment SGI57478[] = {{&S9560[44], 28}, {NULL}};
const Segment SGI57479[] = {{&S9560[30], 6}, {NULL}};
const Segment SGI57480[] = {{&S9560[120], 3}, {NULL}};
const Segment SGI57481[] = {{&S9560[86], 5}, {NULL}};
const Segment SGI57482[] = {{&S9568[38], 11}, {NULL}};
const Segment SGI57483[] = {{&S9568[31], 6}, {NULL}};
const Segment SGI57484[] = {{&S9571[44], 10}, {NULL}};
const Segment SGI57485[] = {{&S9571[44], 27}, {NULL}};
const Segment SGI57486[] = {{&S9571[30], 6}, {NULL}};
const Segment SGI57487[] = {{&S9571[88], 6}, {NULL}};
const Segment SGI57488[] = {{&S9572[44], 10}, {NULL}};
const Segment SGI57489[] = {{&S9572[44], 27}, {NULL}};
const Segment SGI57490[] = {{&S9572[30], 6}, {NULL}};
const Segment SGI57491[] = {{&S9572[88], 4}, {NULL}};
const Segment SGI57492[] = {{&S9572[88], 3}, {NULL}};
const Segment SGI57493[] = {{&S101337[44], 9}, {NULL}};
const Segment SGI57494[] = {{&S101337[44], 19}, {NULL}};
const Segment SGI57495[] = {{&S101337[30], 6}, {NULL}};
const Segment SGI57496[] = {{&S101337[111], 3}, {NULL}};
const Segment SGI57497[] = {{&S101337[77], 5}, {NULL}};
const Segment SGI57498[] = {{&S9576[44], 10}, {NULL}};
const Segment SGI57499[] = {{&S9576[44], 39}, {NULL}};
const Segment SGI57500[] = {{&S9576[30], 6}, {NULL}};
const Segment SGI57501[] = {{&S9576[131], 3}, {NULL}};
const Segment SGI57502[] = {{&S9576[97], 5}, {NULL}};
const Segment SGI57503[] = {{&S9577[38], 11}, {NULL}};
const Segment SGI57504[] = {{&S9577[31], 6}, {NULL}};
const Segment SGI57505[] = {{&S104032[58], 10}, {NULL}};
const Segment SGI57506[] = {{&S104032[58], 27}, {NULL}};
const Segment SGI57507[] = {{&S9578[38], 11}, {NULL}};
const Segment SGI57508[] = {{&S104032[44], 6}, {NULL}};
const Segment SGI57509[] = {{&S9578[31], 6}, {NULL}};
const Segment SGI57510[] = {{&S104032[133], 3}, {NULL}};
const Segment SGI57511[] = {{&S104032[99], 5}, {NULL}};
const Segment SGI57512[] = {{&S9579[44], 11}, {NULL}};
const Segment SGI57513[] = {{&S9579[44], 28}, {NULL}};
const Segment SGI57514[] = {{&S9579[30], 6}, {NULL}};
const Segment SGI57515[] = {{&S9579[120], 3}, {NULL}};
const Segment SGI57516[] = {{&S9579[86], 5}, {NULL}};
const Segment SGI57517[] = {{&S104034[59], 10}, {NULL}};
const Segment SGI57518[] = {{&S104034[59], 27}, {NULL}};
const Segment SGI57519[] = {{&S104034[45], 6}, {NULL}};
const Segment SGI57520[] = {{&S9580[44], 10}, {NULL}};
const Segment SGI57521[] = {{&S104034[134], 3}, {NULL}};
const Segment SGI57522[] = {{&S9580[44], 39}, {NULL}};
const Segment SGI57523[] = {{&S9580[30], 6}, {NULL}};
const Segment SGI57524[] = {{&S104034[100], 5}, {NULL}};
const Segment SGI57525[] = {{&S9580[131], 3}, {NULL}};
const Segment SGI57526[] = {{&S104034[0], 14}, {NULL}};
const Segment SGI57527[] = {{&S9580[97], 5}, {NULL}};
const Segment SGI57528[] = {{&S9581[44], 11}, {NULL}};
const Segment SGI57529[] = {{&S9581[44], 28}, {NULL}};
const Segment SGI57530[] = {{&S9581[30], 6}, {NULL}};
const Segment SGI57531[] = {{&S9581[120], 3}, {NULL}};
const Segment SGI57532[] = {{&S9581[86], 5}, {NULL}};
const Segment SGI57533[] = {{&S9583[42], 10}, {NULL}};
const Segment SGI57534[] = {{&S9583[30], 6}, {NULL}};
const Segment SGI57535[] = {{&S9583[117], 3}, {NULL}};
const Segment SGI57536[] = {{&S9583[83], 5}, {NULL}};
const Segment SGI57537[] = {{&S9586[44], 29}, {NULL}};
const Segment SGI57538[] = {{&S9586[44], 46}, {NULL}};
const Segment SGI57539[] = {{&S9586[30], 6}, {NULL}};
const Segment SGI57540[] = {{&S9586[138], 3}, {NULL}};
const Segment SGI57541[] = {{&S9586[104], 5}, {NULL}};
const Segment SGI57542[] = {{&S9588[44], 11}, {NULL}};
const Segment SGI57543[] = {{&S104045[59], 10}, {NULL}};
const Segment SGI57544[] = {{&S9588[44], 28}, {NULL}};
const Segment SGI57545[] = {{&S104045[59], 27}, {NULL}};
const Segment SGI57546[] = {{&S9588[30], 6}, {NULL}};
const Segment SGI57547[] = {{&S104045[45], 6}, {NULL}};
const Segment SGI57548[] = {{&S9588[120], 3}, {NULL}};
const Segment SGI57549[] = {{&S104045[134], 3}, {NULL}};
const Segment SGI57550[] = {{&S9588[86], 5}, {NULL}};
const Segment SGI57551[] = {{&S104045[100], 5}, {NULL}};
const Segment SGI57552[] = {{&S104045[0], 14}, {NULL}};
const Segment SGI57553[] = {{&S9589[44], 28}, {NULL}};
const Segment SGI57554[] = {{&S9589[44], 45}, {NULL}};
const Segment SGI57555[] = {{&S9589[30], 6}, {NULL}};
const Segment SGI57556[] = {{&S9589[137], 3}, {NULL}};
const Segment SGI57557[] = {{&S9589[103], 5}, {NULL}};
const Segment SGI57558[] = {{&S9591[44], 10}, {NULL}};
const Segment SGI57559[] = {{&S9591[44], 22}, {NULL}};
const Segment SGI57560[] = {{&S9591[30], 6}, {NULL}};
const Segment SGI57561[] = {{&S9591[114], 3}, {NULL}};
const Segment SGI57562[] = {{&S9591[80], 5}, {NULL}};
const Segment SGI57563[] = {{&S101373[57], 22}, {NULL}};
const Segment SGI57564[] = {{&S101373[57], 33}, {NULL}};
const Segment SGI57565[] = {{&S101373[43], 6}, {NULL}};
const Segment SGI57566[] = {{&S101373[138], 3}, {NULL}};
const Segment SGI57567[] = {{&S101373[104], 5}, {NULL}};
const Segment SGI57568[] = {{&S9595[42], 11}, {NULL}};
const Segment SGI57569[] = {{&S9595[30], 6}, {NULL}};
const Segment SGI57570[] = {{&S101373[0], 12}, {NULL}};
const Segment SGI57571[] = {{&S9595[118], 3}, {NULL}};
const Segment SGI57572[] = {{&S9595[84], 5}, {NULL}};
const Segment SGI57573[] = {{&S9597[38], 11}, {NULL}};
const Segment SGI57574[] = {{&S9597[31], 6}, {NULL}};
const Segment SGI57575[] = {{&S9599[44], 10}, {NULL}};
const Segment SGI57576[] = {{&S9599[44], 21}, {NULL}};
const Segment SGI57577[] = {{&S9599[30], 6}, {NULL}};
const Segment SGI57578[] = {{&S9599[113], 3}, {NULL}};
const Segment SGI57579[] = {{&S9599[79], 5}, {NULL}};
const Segment SGI57580[] = {{&S9600[44], 28}, {NULL}};
const Segment SGI57581[] = {{&S9600[44], 45}, {NULL}};
const Segment SGI57582[] = {{&S9600[30], 6}, {NULL}};
const Segment SGI57583[] = {{&S9600[137], 3}, {NULL}};
const Segment SGI57584[] = {{&S9600[103], 5}, {NULL}};
const Segment SGI57585[] = {{&S101384[45], 5}, {NULL}};
const Segment SGI57586[] = {{&S101384[30], 6}, {NULL}};
const Segment SGI57587[] = {{&S101384[115], 3}, {NULL}};
const Segment SGI57588[] = {{&S101384[81], 5}, {NULL}};
const Segment SGI57589[] = {{&S101384[139], 13}, {NULL}};
const Segment SGI57590[] = {{&S9603[38], 11}, {NULL}};
const Segment SGI57591[] = {{&S9603[31], 6}, {NULL}};
const Segment SGI57592[] = {{&S9605[44], 10}, {NULL}};
const Segment SGI57593[] = {{&S9605[44], 27}, {NULL}};
const Segment SGI57594[] = {{&S9605[30], 6}, {NULL}};
const Segment SGI57595[] = {{&S9605[119], 3}, {NULL}};
const Segment SGI57596[] = {{&S9605[85], 5}, {NULL}};
const Segment SGI57597[] = {{&S9607[44], 10}, {NULL}};
const Segment SGI57598[] = {{&S9607[44], 27}, {NULL}};
const Segment SGI57599[] = {{&S9607[30], 6}, {NULL}};
const Segment SGI57600[] = {{&S9607[119], 3}, {NULL}};
const Segment SGI57601[] = {{&S9607[85], 5}, {NULL}};
const Segment SGI57602[] = {{&S9609[38], 11}, {NULL}};
const Segment SGI57603[] = {{&S9609[31], 6}, {NULL}};
const Segment SGI57604[] = {{&S101411[59], 10}, {NULL}};
const Segment SGI57605[] = {{&S101411[59], 27}, {NULL}};
const Segment SGI57606[] = {{&S101411[45], 6}, {NULL}};
const Segment SGI57607[] = {{&S101411[134], 3}, {NULL}};
const Segment SGI57608[] = {{&S101411[100], 5}, {NULL}};
const Segment SGI57609[] = {{&S101411[0], 14}, {NULL}};
const Segment SGI57610[] = {{&S101529[45], 10}, {NULL}};
const Segment SGI57611[] = {{&S101529[30], 6}, {NULL}};
const Segment SGI57612[] = {{&S101529[120], 3}, {NULL}};
const Segment SGI57613[] = {{&S101529[86], 5}, {NULL}};
const Segment SGI57614[] = {{&S101529[144], 13}, {NULL}};
const Segment SGI57615[] = {{&S101589[57], 7}, {NULL}};
const Segment SGI57616[] = {{&S101589[57], 25}, {NULL}};
const Segment SGI57617[] = {{&S101589[43], 6}, {NULL}};
const Segment SGI57618[] = {{&S101589[70], 3}, {NULL}};
const Segment SGI57619[] = {{&S101589[96], 5}, {NULL}};
const Segment SGI57620[] = {{&S101589[0], 12}, {NULL}};
const Segment SGI57621[] = {{&S104288[58], 10}, {NULL}};
const Segment SGI57622[] = {{&S104288[58], 27}, {NULL}};
const Segment SGI57623[] = {{&S104288[44], 6}, {NULL}};
const Segment SGI57624[] = {{&S104288[133], 3}, {NULL}};
const Segment SGI57625[] = {{&S104288[99], 5}, {NULL}};
const Segment SGI57626[] = {{&S101684[69], 13}, {NULL}};
const Segment SGI57627[] = {{&S101684[69], 40}, {NULL}};
const Segment SGI57628[] = {{&S101684[55], 6}, {NULL}};
const Segment SGI57629[] = {{&S101684[157], 3}, {NULL}};
const Segment SGI57630[] = {{&S101684[123], 5}, {NULL}};
const Segment SGI57631[] = {{&S104371[44], 18}, {NULL}};
const Segment SGI57632[] = {{&S104371[44], 29}, {NULL}};
const Segment SGI57633[] = {{&S104371[30], 6}, {NULL}};
const Segment SGI57634[] = {{&S104371[165], 3}, {NULL}};
const Segment SGI57635[] = {{&S104371[131], 3}, {NULL}};
const Segment SGI57636[] = {{&S104371[96], 6}, {NULL}};
const Segment SGI57637[] = {{&S104371[151], 8}, {NULL}};
const Segment SGI57638[] = {{&S104375[44], 8}, {NULL}};
const Segment SGI57639[] = {{&S104375[44], 19}, {NULL}};
const Segment SGI57640[] = {{&S104375[30], 6}, {NULL}};
const Segment SGI57641[] = {{&S104375[111], 3}, {NULL}};
const Segment SGI57642[] = {{&S104375[77], 5}, {NULL}};
const Segment SGI57643[] = {{&S104382[59], 18}, {NULL}};
const Segment SGI57644[] = {{&S104382[59], 29}, {NULL}};
const Segment SGI57645[] = {{&S104382[45], 6}, {NULL}};
const Segment SGI57646[] = {{&S104382[136], 3}, {NULL}};
const Segment SGI57647[] = {{&S104382[102], 5}, {NULL}};
const Segment SGI57648[] = {{&S104382[0], 14}, {NULL}};
const Segment SGI57649[] = {{&S101766[44], 10}, {NULL}};
const Segment SGI57650[] = {{&S101766[44], 27}, {NULL}};
const Segment SGI57651[] = {{&S101766[30], 6}, {NULL}};
const Segment SGI57652[] = {{&S101766[118], 3}, {NULL}};
const Segment SGI57653[] = {{&S101766[85], 5}, {NULL}};
const Segment SGI57654[] = {{&S101766[122], 14}, {NULL}};
const Segment SGI57655[] = {{&S101769[44], 10}, {NULL}};
const Segment SGI57656[] = {{&S101769[44], 21}, {NULL}};
const Segment SGI57657[] = {{&S101769[30], 6}, {NULL}};
const Segment SGI57658[] = {{&S101769[112], 3}, {NULL}};
const Segment SGI57659[] = {{&S101769[79], 5}, {NULL}};
const Segment SGI57660[] = {{&S101769[116], 14}, {NULL}};
const Segment SGI57661[] = {{&S104414[56], 10}, {NULL}};
const Segment SGI57662[] = {{&S104414[56], 27}, {NULL}};
const Segment SGI57663[] = {{&S104414[42], 6}, {NULL}};
const Segment SGI57664[] = {{&S104414[131], 3}, {NULL}};
const Segment SGI57665[] = {{&S104414[97], 5}, {NULL}};
const Segment SGI57666[] = {{&S104414[0], 11}, {NULL}};
const Segment SGI57667[] = {{&S104417[59], 10}, {NULL}};
const Segment SGI57668[] = {{&S104417[59], 27}, {NULL}};
const Segment SGI57669[] = {{&S104417[45], 6}, {NULL}};
const Segment SGI57670[] = {{&S104417[134], 3}, {NULL}};
const Segment SGI57671[] = {{&S104417[100], 5}, {NULL}};
const Segment SGI57672[] = {{&S104417[0], 14}, {NULL}};
const Segment SGI57673[] = {{&S101528[45], 6}, {NULL}};
const Segment SGI57674[] = {{&S101528[30], 6}, {NULL}};
const Segment SGI57675[] = {{&S101528[116], 3}, {NULL}};
const Segment SGI57676[] = {{&S101528[82], 5}, {NULL}};
const Segment SGI57677[] = {{&S101528[140], 13}, {NULL}};
const Segment SGI57678[] = {{&S101530[45], 10}, {NULL}};
const Segment SGI57679[] = {{&S101530[30], 6}, {NULL}};
const Segment SGI57680[] = {{&S101530[120], 3}, {NULL}};
const Segment SGI57681[] = {{&S101530[86], 5}, {NULL}};
const Segment SGI57682[] = {{&S101530[144], 13}, {NULL}};
const Segment SGI57683[] = {{&S101572[44], 10}, {NULL}};
const Segment SGI57684[] = {{&S101572[44], 27}, {NULL}};
const Segment SGI57685[] = {{&S101572[30], 6}, {NULL}};
const Segment SGI57686[] = {{&S101572[119], 3}, {NULL}};
const Segment SGI57687[] = {{&S101572[85], 5}, {NULL}};
const Segment SGI57688[] = {{&S101572[142], 10}, {NULL}};
const Segment SGI57689[] = {{&S101592[59], 21}, {NULL}};
const Segment SGI57690[] = {{&S101592[59], 32}, {NULL}};
const Segment SGI57691[] = {{&S101592[45], 6}, {NULL}};
const Segment SGI57692[] = {{&S101592[139], 3}, {NULL}};
const Segment SGI57693[] = {{&S101592[105], 5}, {NULL}};
const Segment SGI57694[] = {{&S101592[0], 14}, {NULL}};
const Segment SGI57695[] = {{&S101863[44], 12}, {NULL}};
const Segment SGI57696[] = {{&S101863[44], 23}, {NULL}};
const Segment SGI57697[] = {{&S101863[30], 6}, {NULL}};
const Segment SGI57698[] = {{&S101863[115], 3}, {NULL}};
const Segment SGI57699[] = {{&S101863[81], 5}, {NULL}};
const Segment SGI57700[] = {{&S101864[58], 6}, {NULL}};
const Segment SGI57701[] = {{&S101864[58], 17}, {NULL}};
const Segment SGI57702[] = {{&S101864[44], 6}, {NULL}};
const Segment SGI57703[] = {{&S101864[123], 3}, {NULL}};
const Segment SGI57704[] = {{&S101864[89], 5}, {NULL}};
const Segment SGI57705[] = {{&S101883[59], 18}, {NULL}};
const Segment SGI57706[] = {{&S101883[59], 29}, {NULL}};
const Segment SGI57707[] = {{&S101883[45], 6}, {NULL}};
const Segment SGI57708[] = {{&S101883[136], 3}, {NULL}};
const Segment SGI57709[] = {{&S101883[102], 5}, {NULL}};
const Segment SGI57710[] = {{&S101883[0], 14}, {NULL}};
const Segment SGI57711[] = {{&S101730[59], 13}, {NULL}};
const Segment SGI57712[] = {{&S101730[59], 24}, {NULL}};
const Segment SGI57713[] = {{&S101730[45], 6}, {NULL}};
const Segment SGI57714[] = {{&S101730[131], 3}, {NULL}};
const Segment SGI57715[] = {{&S101730[97], 5}, {NULL}};
const Segment SGI57716[] = {{&S101730[0], 14}, {NULL}};
const Segment SGI57717[] = {{&S104774[61], 11}, {NULL}};
const Segment SGI57718[] = {{&S104774[54], 6}, {NULL}};
const Segment SGI57719[] = {{&S104774[0], 15}, {NULL}};
const Segment SGI57720[] = {{&S101781[59], 25}, {NULL}};
const Segment SGI57721[] = {{&S101781[45], 6}, {NULL}};
const Segment SGI57722[] = {{&S101781[150], 3}, {NULL}};
const Segment SGI57723[] = {{&S101781[116], 5}, {NULL}};
const Segment SGI57724[] = {{&S101781[0], 14}, {NULL}};
const Segment SGI57725[] = {{&S101782[59], 17}, {NULL}};
const Segment SGI57726[] = {{&S101782[59], 28}, {NULL}};
const Segment SGI57727[] = {{&S101782[45], 6}, {NULL}};
const Segment SGI57728[] = {{&S101782[135], 3}, {NULL}};
const Segment SGI57729[] = {{&S101782[101], 5}, {NULL}};
const Segment SGI57730[] = {{&S101782[0], 14}, {NULL}};
const Segment SGI57731[] = {{&S102057[45], 10}, {NULL}};
const Segment SGI57732[] = {{&S102057[30], 6}, {NULL}};
const Segment SGI57733[] = {{&S102057[120], 3}, {NULL}};
const Segment SGI57734[] = {{&S102057[86], 5}, {NULL}};
const Segment SGI57735[] = {{&S102057[144], 13}, {NULL}};
const Segment SGI57736[] = {{&S102076[44], 8}, {NULL}};
const Segment SGI57737[] = {{&S102076[44], 19}, {NULL}};
const Segment SGI57738[] = {{&S102076[30], 6}, {NULL}};
const Segment SGI57739[] = {{&S102076[111], 3}, {NULL}};
const Segment SGI57740[] = {{&S102076[77], 5}, {NULL}};
const Segment SGI57741[] = {{&S102131[57], 7}, {NULL}};
const Segment SGI57742[] = {{&S102131[57], 25}, {NULL}};
const Segment SGI57743[] = {{&S102131[43], 6}, {NULL}};
const Segment SGI57744[] = {{&S102131[70], 3}, {NULL}};
const Segment SGI57745[] = {{&S102131[96], 5}, {NULL}};
const Segment SGI57746[] = {{&S102131[0], 12}, {NULL}};
const Segment SGI57747[] = {{&S104877[59], 10}, {NULL}};
const Segment SGI57748[] = {{&S104877[59], 27}, {NULL}};
const Segment SGI57749[] = {{&S104877[45], 6}, {NULL}};
const Segment SGI57750[] = {{&S104877[134], 3}, {NULL}};
const Segment SGI57751[] = {{&S104877[100], 5}, {NULL}};
const Segment SGI57752[] = {{&S104877[0], 14}, {NULL}};
const Segment SGI57753[] = {{&S104896[44], 10}, {NULL}};
const Segment SGI57754[] = {{&S104896[44], 22}, {NULL}};
const Segment SGI57755[] = {{&S104896[30], 6}, {NULL}};
const Segment SGI57756[] = {{&S104896[114], 3}, {NULL}};
const Segment SGI57757[] = {{&S104896[80], 5}, {NULL}};
const Segment SGI57758[] = {{&S102251[44], 11}, {NULL}};
const Segment SGI57759[] = {{&S102251[44], 22}, {NULL}};
const Segment SGI57760[] = {{&S102251[30], 6}, {NULL}};
const Segment SGI57761[] = {{&S102251[114], 3}, {NULL}};
const Segment SGI57762[] = {{&S102251[80], 5}, {NULL}};
const Segment SGI57763[] = {{&S102254[44], 16}, {NULL}};
const Segment SGI57764[] = {{&S102254[44], 27}, {NULL}};
const Segment SGI57765[] = {{&S102254[30], 6}, {NULL}};
const Segment SGI57766[] = {{&S102254[119], 3}, {NULL}};
const Segment SGI57767[] = {{&S102254[85], 5}, {NULL}};
const Segment SGI57768[] = {{&S102257[59], 10}, {NULL}};
const Segment SGI57769[] = {{&S102257[59], 27}, {NULL}};
const Segment SGI57770[] = {{&S102257[45], 6}, {NULL}};
const Segment SGI57771[] = {{&S102257[134], 3}, {NULL}};
const Segment SGI57772[] = {{&S102257[100], 5}, {NULL}};
const Segment SGI57773[] = {{&S102257[0], 14}, {NULL}};
const Segment SGI57774[] = {{&S102067[6], 4}, {NULL}};
const Segment SGI57775[] = {{&S102067[83], 5}, {NULL}};
const Segment SGI57776[] = {{&S102071[59], 18}, {NULL}};
const Segment SGI57777[] = {{&S102071[59], 29}, {NULL}};
const Segment SGI57778[] = {{&S102071[45], 6}, {NULL}};
const Segment SGI57779[] = {{&S102071[136], 3}, {NULL}};
const Segment SGI57780[] = {{&S102071[102], 5}, {NULL}};
const Segment SGI57781[] = {{&S102071[0], 14}, {NULL}};
const Segment SGI57782[] = {{&S104987[44], 10}, {NULL}};
const Segment SGI57783[] = {{&S104987[44], 27}, {NULL}};
const Segment SGI57784[] = {{&S104987[30], 6}, {NULL}};
const Segment SGI57785[] = {{&S104987[85], 6}, {NULL}};
const Segment SGI57786[] = {{&S104987[122], 3}, {NULL}};
const Segment SGI57787[] = {{&S102472[58], 10}, {NULL}};
const Segment SGI57788[] = {{&S102472[58], 27}, {NULL}};
const Segment SGI57789[] = {{&S102472[44], 6}, {NULL}};
const Segment SGI57790[] = {{&S102472[133], 3}, {NULL}};
const Segment SGI57791[] = {{&S102472[99], 5}, {NULL}};
const Segment SGI57792[] = {{&S102474[44], 9}, {NULL}};
const Segment SGI57793[] = {{&S102474[44], 26}, {NULL}};
const Segment SGI57794[] = {{&S102474[30], 6}, {NULL}};
const Segment SGI57795[] = {{&S102474[118], 3}, {NULL}};
const Segment SGI57796[] = {{&S102474[84], 5}, {NULL}};
const Segment SGI57797[] = {{&S102312[59], 4}, {NULL}};
const Segment SGI57798[] = {{&S102312[59], 15}, {NULL}};
const Segment SGI57799[] = {{&S102312[45], 6}, {NULL}};
const Segment SGI57800[] = {{&S102312[122], 3}, {NULL}};
const Segment SGI57801[] = {{&S102312[88], 5}, {NULL}};
const Segment SGI57802[] = {{&S102312[0], 14}, {NULL}};
const Segment SGI57803[] = {{&S102320[34], 10}, {NULL}};
const Segment SGI57804[] = {{&S102320[27], 6}, {NULL}};
const Segment SGI57805[] = {{&S102320[109], 2}, {NULL}};
const Segment SGI57806[] = {{&S102320[75], 6}, {NULL}};
const Segment SGI57807[] = {{&S102320[143], 8}, {NULL}};
const Segment SGI57808[] = {{&S102663[45], 7}, {NULL}};
const Segment SGI57809[] = {{&S102663[30], 6}, {NULL}};
const Segment SGI57810[] = {{&S102663[58], 3}, {NULL}};
const Segment SGI57811[] = {{&S102663[84], 5}, {NULL}};
const Segment SGI57812[] = {{&S102663[142], 13}, {NULL}};
const Segment SGI57813[] = {{&S102666[31], 16}, {NULL}};
const Segment SGI57814[] = {{&S102666[62], 3}, {NULL}};
const Segment SGI57815[] = {{&S10820[44], 10}, {NULL}};
const Segment SGI57816[] = {{&S10820[44], 27}, {NULL}};
const Segment SGI57817[] = {{&S10820[30], 6}, {NULL}};
const Segment SGI57818[] = {{&S10820[88], 4}, {NULL}};
const Segment SGI57819[] = {{&S10820[88], 3}, {NULL}};
const Segment SGI57820[] = {{&S10821[44], 11}, {NULL}};
const Segment SGI57821[] = {{&S10821[44], 28}, {NULL}};
const Segment SGI57822[] = {{&S10821[30], 6}, {NULL}};
const Segment SGI57823[] = {{&S10821[120], 3}, {NULL}};
const Segment SGI57824[] = {{&S10821[86], 5}, {NULL}};
const Segment SGI57825[] = {{&S10822[44], 11}, {NULL}};
const Segment SGI57826[] = {{&S10822[44], 28}, {NULL}};
const Segment SGI57827[] = {{&S10822[30], 6}, {NULL}};
const Segment SGI57828[] = {{&S10822[120], 3}, {NULL}};
const Segment SGI57829[] = {{&S10822[86], 5}, {NULL}};
const Segment SGI57830[] = {{&S102671[45], 9}, {NULL}};
const Segment SGI57831[] = {{&S102671[30], 6}, {NULL}};
const Segment SGI57832[] = {{&S102671[113], 3}, {NULL}};
const Segment SGI57833[] = {{&S102671[79], 5}, {NULL}};
const Segment SGI57834[] = {{&S102671[137], 13}, {NULL}};
const Segment SGI57835[] = {{&S10825[44], 10}, {NULL}};
const Segment SGI57836[] = {{&S10825[44], 27}, {NULL}};
const Segment SGI57837[] = {{&S10825[30], 6}, {NULL}};
const Segment SGI57838[] = {{&S10825[119], 3}, {NULL}};
const Segment SGI57839[] = {{&S10825[85], 5}, {NULL}};
const Segment SGI57840[] = {{&S10826[44], 29}, {NULL}};
const Segment SGI57841[] = {{&S10826[44], 46}, {NULL}};
const Segment SGI57842[] = {{&S10826[30], 6}, {NULL}};
const Segment SGI57843[] = {{&S10826[138], 3}, {NULL}};
const Segment SGI57844[] = {{&S10826[104], 5}, {NULL}};
const Segment SGI57845[] = {{&S10827[44], 29}, {NULL}};
const Segment SGI57846[] = {{&S10827[44], 46}, {NULL}};
const Segment SGI57847[] = {{&S10827[30], 6}, {NULL}};
const Segment SGI57848[] = {{&S10827[138], 3}, {NULL}};
const Segment SGI57849[] = {{&S10827[104], 5}, {NULL}};
const Segment SGI57850[] = {{&S10828[44], 11}, {NULL}};
const Segment SGI57851[] = {{&S10828[44], 28}, {NULL}};
const Segment SGI57852[] = {{&S10828[30], 6}, {NULL}};
const Segment SGI57853[] = {{&S10828[120], 3}, {NULL}};
const Segment SGI57854[] = {{&S10828[86], 5}, {NULL}};
const Segment SGI57855[] = {{&S10829[38], 10}, {NULL}};
const Segment SGI57856[] = {{&S10829[31], 6}, {NULL}};
const Segment SGI57857[] = {{&S10830[44], 19}, {NULL}};
const Segment SGI57858[] = {{&S10830[44], 36}, {NULL}};
const Segment SGI57859[] = {{&S10830[30], 6}, {NULL}};
const Segment SGI57860[] = {{&S10830[128], 3}, {NULL}};
const Segment SGI57861[] = {{&S10830[94], 5}, {NULL}};
const Segment SGI57862[] = {{&S10832[44], 28}, {NULL}};
const Segment SGI57863[] = {{&S10832[44], 45}, {NULL}};
const Segment SGI57864[] = {{&S10832[30], 6}, {NULL}};
const Segment SGI57865[] = {{&S10832[137], 3}, {NULL}};
const Segment SGI57866[] = {{&S10832[103], 5}, {NULL}};
const Segment SGI57867[] = {{&S10835[44], 10}, {NULL}};
const Segment SGI57868[] = {{&S10835[44], 27}, {NULL}};
const Segment SGI57869[] = {{&S10835[30], 6}, {NULL}};
const Segment SGI57870[] = {{&S10835[119], 3}, {NULL}};
const Segment SGI57871[] = {{&S10835[85], 5}, {NULL}};
const Segment SGI57872[] = {{&S10836[42], 10}, {NULL}};
const Segment SGI57873[] = {{&S10836[30], 6}, {NULL}};
const Segment SGI57874[] = {{&S10836[117], 3}, {NULL}};
const Segment SGI57875[] = {{&S10836[83], 5}, {NULL}};
const Segment SGI57876[] = {{&S102466[45], 14}, {NULL}};
const Segment SGI57877[] = {{&S102466[30], 6}, {NULL}};
const Segment SGI57878[] = {{&S10838[38], 11}, {NULL}};
const Segment SGI57879[] = {{&S10838[31], 6}, {NULL}};
const Segment SGI57880[] = {{&S102466[118], 3}, {NULL}};
const Segment SGI57881[] = {{&S102466[84], 5}, {NULL}};
const Segment SGI57882[] = {{&S102466[142], 13}, {NULL}};
const Segment SGI57883[] = {{&S10839[44], 29}, {NULL}};
const Segment SGI57884[] = {{&S10839[44], 46}, {NULL}};
const Segment SGI57885[] = {{&S10839[30], 6}, {NULL}};
const Segment SGI57886[] = {{&S10839[138], 3}, {NULL}};
const Segment SGI57887[] = {{&S10839[104], 5}, {NULL}};
const Segment SGI57888[] = {{&S10840[44], 27}, {NULL}};
const Segment SGI57889[] = {{&S10840[44], 44}, {NULL}};
const Segment SGI57890[] = {{&S10840[30], 6}, {NULL}};
const Segment SGI57891[] = {{&S10840[136], 3}, {NULL}};
const Segment SGI57892[] = {{&S10840[102], 5}, {NULL}};
const Segment SGI57893[] = {{&S10841[44], 10}, {NULL}};
const Segment SGI57894[] = {{&S10841[44], 27}, {NULL}};
const Segment SGI57895[] = {{&S10841[30], 6}, {NULL}};
const Segment SGI57896[] = {{&S10841[119], 3}, {NULL}};
const Segment SGI57897[] = {{&S10841[85], 5}, {NULL}};
const Segment SGI57898[] = {{&S10842[44], 10}, {NULL}};
const Segment SGI57899[] = {{&S10842[44], 27}, {NULL}};
const Segment SGI57900[] = {{&S10842[30], 6}, {NULL}};
const Segment SGI57901[] = {{&S10842[88], 4}, {NULL}};
const Segment SGI57902[] = {{&S10842[88], 3}, {NULL}};
const Segment SGI57903[] = {{&S10843[44], 28}, {NULL}};
const Segment SGI57904[] = {{&S10843[44], 45}, {NULL}};
const Segment SGI57905[] = {{&S10843[30], 6}, {NULL}};
const Segment SGI57906[] = {{&S10843[137], 3}, {NULL}};
const Segment SGI57907[] = {{&S10843[103], 5}, {NULL}};
const Segment SGI57908[] = {{&S10844[44], 29}, {NULL}};
const Segment SGI57909[] = {{&S10844[44], 46}, {NULL}};
const Segment SGI57910[] = {{&S10844[30], 6}, {NULL}};
const Segment SGI57911[] = {{&S10844[138], 3}, {NULL}};
const Segment SGI57912[] = {{&S10844[104], 5}, {NULL}};
const Segment SGI57913[] = {{&S10845[38], 10}, {NULL}};
const Segment SGI57914[] = {{&S10845[31], 6}, {NULL}};
const Segment SGI57915[] = {{&S10849[38], 11}, {NULL}};
const Segment SGI57916[] = {{&S10849[31], 6}, {NULL}};
const Segment SGI57917[] = {{&S10851[38], 11}, {NULL}};
const Segment SGI57918[] = {{&S10851[31], 6}, {NULL}};
const Segment SGI57919[] = {{&S10852[38], 10}, {NULL}};
const Segment SGI57920[] = {{&S10852[31], 6}, {NULL}};
const Segment SGI57921[] = {{&S10853[44], 10}, {NULL}};
const Segment SGI57922[] = {{&S10853[44], 27}, {NULL}};
const Segment SGI57923[] = {{&S10853[30], 6}, {NULL}};
const Segment SGI57924[] = {{&S10853[88], 6}, {NULL}};
const Segment SGI57925[] = {{&S10854[39], 8}, {NULL}};
const Segment SGI57926[] = {{&S10854[92], 3}, {NULL}};
const Segment SGI57927[] = {{&S10854[69], 6}, {NULL}};
const Segment SGI57928[] = {{&S10855[44], 28}, {NULL}};
const Segment SGI57929[] = {{&S10855[44], 45}, {NULL}};
const Segment SGI57930[] = {{&S10855[30], 6}, {NULL}};
const Segment SGI57931[] = {{&S10855[137], 3}, {NULL}};
const Segment SGI57932[] = {{&S10855[103], 5}, {NULL}};
const Segment SGI57933[] = {{&S10856[44], 10}, {NULL}};
const Segment SGI57934[] = {{&S10856[44], 27}, {NULL}};
const Segment SGI57935[] = {{&S10856[30], 6}, {NULL}};
const Segment SGI57936[] = {{&S10856[88], 6}, {NULL}};
const Segment SGI57937[] = {{&S10857[44], 10}, {NULL}};
const Segment SGI57938[] = {{&S10857[44], 27}, {NULL}};
const Segment SGI57939[] = {{&S10857[30], 6}, {NULL}};
const Segment SGI57940[] = {{&S10857[119], 3}, {NULL}};
const Segment SGI57941[] = {{&S10857[85], 5}, {NULL}};
const Segment SGI57942[] = {{&S10858[44], 29}, {NULL}};
const Segment SGI57943[] = {{&S10858[44], 46}, {NULL}};
const Segment SGI57944[] = {{&S10858[30], 6}, {NULL}};
const Segment SGI57945[] = {{&S10858[138], 3}, {NULL}};
const Segment SGI57946[] = {{&S10858[104], 5}, {NULL}};
const Segment SGI57947[] = {{&S10859[44], 10}, {NULL}};
const Segment SGI57948[] = {{&S10859[44], 27}, {NULL}};
const Segment SGI57949[] = {{&S10859[30], 6}, {NULL}};
const Segment SGI57950[] = {{&S10859[132], 3}, {NULL}};
const Segment SGI57951[] = {{&S10859[85], 6}, {NULL}};
const Segment SGI57952[] = {{&S10860[44], 29}, {NULL}};
const Segment SGI57953[] = {{&S10860[44], 46}, {NULL}};
const Segment SGI57954[] = {{&S10860[30], 6}, {NULL}};
const Segment SGI57955[] = {{&S10860[138], 3}, {NULL}};
const Segment SGI57956[] = {{&S10860[104], 5}, {NULL}};
const Segment SGI57957[] = {{&S10861[44], 29}, {NULL}};
const Segment SGI57958[] = {{&S10861[44], 46}, {NULL}};
const Segment SGI57959[] = {{&S10861[30], 6}, {NULL}};
const Segment SGI57960[] = {{&S10861[138], 3}, {NULL}};
const Segment SGI57961[] = {{&S10861[104], 5}, {NULL}};
const Segment SGI57962[] = {{&S105232[58], 6}, {NULL}};
const Segment SGI57963[] = {{&S105232[58], 24}, {NULL}};
const Segment SGI57964[] = {{&S105232[44], 6}, {NULL}};
const Segment SGI57965[] = {{&S105232[130], 3}, {NULL}};
const Segment SGI57966[] = {{&S105232[96], 5}, {NULL}};
const Segment SGI57967[] = {{&S10862[42], 19}, {NULL}};
const Segment SGI57968[] = {{&S10862[30], 6}, {NULL}};
const Segment SGI57969[] = {{&S10862[126], 3}, {NULL}};
const Segment SGI57970[] = {{&S10862[92], 5}, {NULL}};
const Segment SGI57971[] = {{&S10864[42], 11}, {NULL}};
const Segment SGI57972[] = {{&S10864[30], 6}, {NULL}};
const Segment SGI57973[] = {{&S10864[118], 3}, {NULL}};
const Segment SGI57974[] = {{&S10864[84], 5}, {NULL}};
const Segment SGI57975[] = {{&S10866[44], 28}, {NULL}};
const Segment SGI57976[] = {{&S10866[44], 45}, {NULL}};
const Segment SGI57977[] = {{&S10866[30], 6}, {NULL}};
const Segment SGI57978[] = {{&S10866[137], 3}, {NULL}};
const Segment SGI57979[] = {{&S10866[103], 5}, {NULL}};
const Segment SGI57980[] = {{&S10867[42], 10}, {NULL}};
const Segment SGI57981[] = {{&S10867[30], 6}, {NULL}};
const Segment SGI57982[] = {{&S10867[117], 3}, {NULL}};
const Segment SGI57983[] = {{&S10867[83], 5}, {NULL}};
const Segment SGI57984[] = {{&S10868[44], 35}, {NULL}};
const Segment SGI57985[] = {{&S10868[44], 52}, {NULL}};
const Segment SGI57986[] = {{&S10868[30], 6}, {NULL}};
const Segment SGI57987[] = {{&S10868[144], 3}, {NULL}};
const Segment SGI57988[] = {{&S10868[110], 5}, {NULL}};
const Segment SGI57989[] = {{&S102790[44], 10}, {NULL}};
const Segment SGI57990[] = {{&S102790[44], 21}, {NULL}};
const Segment SGI57991[] = {{&S102790[30], 6}, {NULL}};
const Segment SGI57992[] = {{&S102790[113], 3}, {NULL}};
const Segment SGI57993[] = {{&S102790[79], 5}, {NULL}};
const Segment SGI57994[] = {{&S102796[44], 11}, {NULL}};
const Segment SGI57995[] = {{&S102796[44], 22}, {NULL}};
const Segment SGI57996[] = {{&S102796[30], 6}, {NULL}};
const Segment SGI57997[] = {{&S102796[114], 3}, {NULL}};
const Segment SGI57998[] = {{&S102796[80], 5}, {NULL}};
const Segment SGI57999[] = {{&S105349[44], 10}, {NULL}};
const Segment SGI58000[] = {{&S105349[44], 27}, {NULL}};
const Segment SGI58001[] = {{&S105349[30], 6}, {NULL}};
const Segment SGI58002[] = {{&S105349[119], 3}, {NULL}};
const Segment SGI58003[] = {{&S105349[85], 5}, {NULL}};
const Segment SGI58004[] = {{&S105351[58], 6}, {NULL}};
const Segment SGI58005[] = {{&S105351[58], 24}, {NULL}};
const Segment SGI58006[] = {{&S105351[44], 6}, {NULL}};
const Segment SGI58007[] = {{&S105351[130], 3}, {NULL}};
const Segment SGI58008[] = {{&S105351[96], 5}, {NULL}};
const Segment SGI58009[] = {{&S102632[56], 20}, {NULL}};
const Segment SGI58010[] = {{&S102632[56], 31}, {NULL}};
const Segment SGI58011[] = {{&S102632[42], 6}, {NULL}};
const Segment SGI58012[] = {{&S102632[135], 3}, {NULL}};
const Segment SGI58013[] = {{&S102632[101], 5}, {NULL}};
const Segment SGI58014[] = {{&S102632[0], 12}, {NULL}};
const Segment SGI58015[] = {{&S102868[59], 10}, {NULL}};
const Segment SGI58016[] = {{&S102868[59], 27}, {NULL}};
const Segment SGI58017[] = {{&S102868[45], 6}, {NULL}};
const Segment SGI58018[] = {{&S102868[134], 3}, {NULL}};
const Segment SGI58019[] = {{&S102868[100], 5}, {NULL}};
const Segment SGI58020[] = {{&S102868[0], 14}, {NULL}};
const Segment SGI58021[] = {{&S102871[56], 10}, {NULL}};
const Segment SGI58022[] = {{&S102871[56], 27}, {NULL}};
const Segment SGI58023[] = {{&S102871[42], 6}, {NULL}};
const Segment SGI58024[] = {{&S102871[131], 3}, {NULL}};
const Segment SGI58025[] = {{&S102871[97], 5}, {NULL}};
const Segment SGI58026[] = {{&S102872[45], 9}, {NULL}};
const Segment SGI58027[] = {{&S102872[30], 6}, {NULL}};
const Segment SGI58028[] = {{&S102872[113], 3}, {NULL}};
const Segment SGI58029[] = {{&S102872[79], 5}, {NULL}};
const Segment SGI58030[] = {{&S102872[137], 13}, {NULL}};
const Segment SGI58031[] = {{&S102879[45], 7}, {NULL}};
const Segment SGI58032[] = {{&S102879[30], 6}, {NULL}};
const Segment SGI58033[] = {{&S102879[58], 3}, {NULL}};
const Segment SGI58034[] = {{&S102879[84], 5}, {NULL}};
const Segment SGI58035[] = {{&S102879[142], 13}, {NULL}};
const Segment SGI58036[] = {{&S102896[44], 10}, {NULL}};
const Segment SGI58037[] = {{&S102896[44], 34}, {NULL}};
const Segment SGI58038[] = {{&S102896[30], 6}, {NULL}};
const Segment SGI58039[] = {{&S102896[126], 3}, {NULL}};
const Segment SGI58040[] = {{&S102896[92], 5}, {NULL}};
const Segment SGI58041[] = {{&S102729[34], 21}, {NULL}};
const Segment SGI58042[] = {{&S102729[27], 6}, {NULL}};
const Segment SGI58043[] = {{&S102729[114], 2}, {NULL}};
const Segment SGI58044[] = {{&S102729[80], 6}, {NULL}};
const Segment SGI58045[] = {{&S102729[148], 8}, {NULL}};
const Segment SGI58046[] = {{&S105470[31], 7}, {NULL}};
const Segment SGI58047[] = {{&S105470[53], 3}, {NULL}};
const Segment SGI58048[] = {{&S103089[45], 6}, {NULL}};
const Segment SGI58049[] = {{&S103089[30], 6}, {NULL}};
const Segment SGI58050[] = {{&S105530[45], 9}, {NULL}};
const Segment SGI58051[] = {{&S105530[30], 6}, {NULL}};
const Segment SGI58052[] = {{&S103089[116], 3}, {NULL}};
const Segment SGI58053[] = {{&S105530[113], 3}, {NULL}};
const Segment SGI58054[] = {{&S103089[82], 5}, {NULL}};
const Segment SGI58055[] = {{&S103089[140], 13}, {NULL}};
const Segment SGI58056[] = {{&S105530[79], 5}, {NULL}};
const Segment SGI58057[] = {{&S105530[147], 3}, {NULL}};
const Segment SGI58058[] = {{&S103113[31], 7}, {NULL}};
const Segment SGI58059[] = {{&S103113[53], 3}, {NULL}};
const Segment SGI58060[] = {{&S103116[45], 9}, {NULL}};
const Segment SGI58061[] = {{&S103116[30], 6}, {NULL}};
const Segment SGI58062[] = {{&S103116[113], 3}, {NULL}};
const Segment SGI58063[] = {{&S103116[79], 5}, {NULL}};
const Segment SGI58064[] = {{&S103116[137], 13}, {NULL}};
const Segment SGI58065[] = {{&S103169[59], 10}, {NULL}};
const Segment SGI58066[] = {{&S103169[59], 27}, {NULL}};
const Segment SGI58067[] = {{&S103169[45], 6}, {NULL}};
const Segment SGI58068[] = {{&S103169[134], 3}, {NULL}};
const Segment SGI58069[] = {{&S103169[100], 5}, {NULL}};
const Segment SGI58070[] = {{&S103169[0], 14}, {NULL}};
const Segment SGI58071[] = {{&S4594[44], 10}, {NULL}};
const Segment SGI58072[] = {{&S4594[44], 21}, {NULL}};
const Segment SGI58073[] = {{&S4594[30], 6}, {NULL}};
const Segment SGI58074[] = {{&S4594[113], 3}, {NULL}};
const Segment SGI58075[] = {{&S4594[79], 5}, {NULL}};
const Segment SGI58076[] = {{&S4610[42], 11}, {NULL}};
const Segment SGI58077[] = {{&S4610[42], 28}, {NULL}};
const Segment SGI58078[] = {{&S4610[30], 4}, {NULL}};
const Segment SGI58079[] = {{&S4610[118], 3}, {NULL}};
const Segment SGI58080[] = {{&S4610[84], 5}, {NULL}};
const Segment SGI58081[] = {{&S105646[59], 10}, {NULL}};
const Segment SGI58082[] = {{&S105646[59], 27}, {NULL}};
const Segment SGI58083[] = {{&S105646[45], 6}, {NULL}};
const Segment SGI58084[] = {{&S105646[134], 3}, {NULL}};
const Segment SGI58085[] = {{&S105646[100], 5}, {NULL}};
const Segment SGI58086[] = {{&S105646[0], 14}, {NULL}};
const Segment SGI58087[] = {{&S103101[59], 18}, {NULL}};
const Segment SGI58088[] = {{&S103101[59], 29}, {NULL}};
const Segment SGI58089[] = {{&S103101[45], 6}, {NULL}};
const Segment SGI58090[] = {{&S103101[136], 3}, {NULL}};
const Segment SGI58091[] = {{&S103101[102], 5}, {NULL}};
const Segment SGI58092[] = {{&S103101[0], 14}, {NULL}};
const Segment SGI58093[] = {{&S103104[59], 13}, {NULL}};
const Segment SGI58094[] = {{&S103104[59], 30}, {NULL}};
const Segment SGI58095[] = {{&S103104[45], 6}, {NULL}};
const Segment SGI58096[] = {{&S103104[137], 3}, {NULL}};
const Segment SGI58097[] = {{&S103104[103], 5}, {NULL}};
const Segment SGI58098[] = {{&S103104[0], 14}, {NULL}};
const Segment SGI58099[] = {{&S103105[45], 21}, {NULL}};
const Segment SGI58100[] = {{&S103105[30], 6}, {NULL}};
const Segment SGI58101[] = {{&S103105[125], 3}, {NULL}};
const Segment SGI58102[] = {{&S103105[91], 5}, {NULL}};
const Segment SGI58103[] = {{&S103105[149], 13}, {NULL}};
const Segment SGI58104[] = {{&S103131[59], 15}, {NULL}};
const Segment SGI58105[] = {{&S103131[59], 26}, {NULL}};
const Segment SGI58106[] = {{&S103131[45], 6}, {NULL}};
const Segment SGI58107[] = {{&S103131[133], 3}, {NULL}};
const Segment SGI58108[] = {{&S103131[99], 5}, {NULL}};
const Segment SGI58109[] = {{&S103131[0], 14}, {NULL}};
const Segment SGI58110[] = {{&S4622[38], 10}, {NULL}};
const Segment SGI58111[] = {{&S4622[31], 6}, {NULL}};
const Segment SGI58112[] = {{&S4624[44], 10}, {NULL}};
const Segment SGI58113[] = {{&S4624[44], 21}, {NULL}};
const Segment SGI58114[] = {{&S4624[30], 6}, {NULL}};
const Segment SGI58115[] = {{&S4624[113], 3}, {NULL}};
const Segment SGI58116[] = {{&S4624[79], 5}, {NULL}};
const Segment SGI58117[] = {{&S103384[58], 25}, {NULL}};
const Segment SGI58118[] = {{&S103384[44], 6}, {NULL}};
const Segment SGI58119[] = {{&S103384[149], 3}, {NULL}};
const Segment SGI58120[] = {{&S103384[115], 5}, {NULL}};
const Segment SGI58121[] = {{&S103386[56], 10}, {NULL}};
const Segment SGI58122[] = {{&S103386[56], 27}, {NULL}};
const Segment SGI58123[] = {{&S103386[42], 6}, {NULL}};
const Segment SGI58124[] = {{&S103386[131], 3}, {NULL}};
const Segment SGI58125[] = {{&S103386[97], 5}, {NULL}};
const Segment SGI58126[] = {{&S103389[59], 10}, {NULL}};
const Segment SGI58127[] = {{&S103389[59], 27}, {NULL}};
const Segment SGI58128[] = {{&S103389[45], 6}, {NULL}};
const Segment SGI58129[] = {{&S103389[134], 3}, {NULL}};
const Segment SGI58130[] = {{&S103389[100], 5}, {NULL}};
const Segment SGI58131[] = {{&S103389[0], 14}, {NULL}};
const Segment SGI58132[] = {{&S103393[45], 9}, {NULL}};
const Segment SGI58133[] = {{&S103393[30], 6}, {NULL}};
const Segment SGI58134[] = {{&S103393[113], 3}, {NULL}};
const Segment SGI58135[] = {{&S4647[44], 10}, {NULL}};
const Segment SGI58136[] = {{&S103393[79], 5}, {NULL}};
const Segment SGI58137[] = {{&S4647[44], 23}, {NULL}};
const Segment SGI58138[] = {{&S4647[30], 6}, {NULL}};
const Segment SGI58139[] = {{&S103393[137], 13}, {NULL}};
const Segment SGI58140[] = {{&S4647[115], 3}, {NULL}};
const Segment SGI58141[] = {{&S4647[81], 5}, {NULL}};
const Segment SGI58142[] = {{&S103394[45], 9}, {NULL}};
const Segment SGI58143[] = {{&S103394[30], 6}, {NULL}};
const Segment SGI58144[] = {{&S103394[113], 3}, {NULL}};
const Segment SGI58145[] = {{&S103394[79], 5}, {NULL}};
const Segment SGI58146[] = {{&S103394[137], 13}, {NULL}};
const Segment SGI58147[] = {{&S4649[44], 10}, {NULL}};
const Segment SGI58148[] = {{&S4649[44], 20}, {NULL}};
const Segment SGI58149[] = {{&S4649[30], 6}, {NULL}};
const Segment SGI58150[] = {{&S4649[112], 3}, {NULL}};
const Segment SGI58151[] = {{&S4649[78], 5}, {NULL}};
const Segment SGI58152[] = {{&S4664[44], 10}, {NULL}};
const Segment SGI58153[] = {{&S4664[44], 27}, {NULL}};
const Segment SGI58154[] = {{&S4664[30], 6}, {NULL}};
const Segment SGI58155[] = {{&S4664[119], 3}, {NULL}};
const Segment SGI58156[] = {{&S4664[85], 5}, {NULL}};
const Segment SGI58157[] = {{&S103416[45], 7}, {NULL}};
const Segment SGI58158[] = {{&S103416[30], 6}, {NULL}};
const Segment SGI58159[] = {{&S103416[58], 3}, {NULL}};
const Segment SGI58160[] = {{&S103416[84], 5}, {NULL}};
const Segment SGI58161[] = {{&S103416[142], 13}, {NULL}};
const Segment SGI58162[] = {{&S103478[57], 10}, {NULL}};
const Segment SGI58163[] = {{&S103478[57], 27}, {NULL}};
const Segment SGI58164[] = {{&S103478[43], 6}, {NULL}};
const Segment SGI58165[] = {{&S103478[132], 3}, {NULL}};
const Segment SGI58166[] = {{&S103478[98], 5}, {NULL}};
const Segment SGI58167[] = {{&S103478[0], 12}, {NULL}};
const Segment SGI58168[] = {{&S103391[59], 11}, {NULL}};
const Segment SGI58169[] = {{&S103391[59], 28}, {NULL}};
const Segment SGI58170[] = {{&S103391[45], 6}, {NULL}};
const Segment SGI58171[] = {{&S103391[135], 3}, {NULL}};
const Segment SGI58172[] = {{&S103391[101], 5}, {NULL}};
const Segment SGI58173[] = {{&S103391[0], 14}, {NULL}};
const Segment SGI58174[] = {{&S103392[45], 10}, {NULL}};
const Segment SGI58175[] = {{&S103392[30], 6}, {NULL}};
const Segment SGI58176[] = {{&S103392[120], 3}, {NULL}};
const Segment SGI58177[] = {{&S103392[86], 5}, {NULL}};
const Segment SGI58178[] = {{&S103392[144], 13}, {NULL}};
const Segment SGI58179[] = {{&S103401[59], 13}, {NULL}};
const Segment SGI58180[] = {{&S103401[59], 30}, {NULL}};
const Segment SGI58181[] = {{&S103401[45], 6}, {NULL}};
const Segment SGI58182[] = {{&S103401[137], 3}, {NULL}};
const Segment SGI58183[] = {{&S103401[103], 5}, {NULL}};
const Segment SGI58184[] = {{&S103401[0], 14}, {NULL}};
const Segment SGI58185[] = {{&S103688[44], 6}, {NULL}};
const Segment SGI58186[] = {{&S103688[44], 23}, {NULL}};
const Segment SGI58187[] = {{&S103688[30], 6}, {NULL}};
const Segment SGI58188[] = {{&S103688[122], 3}, {NULL}};
const Segment SGI58189[] = {{&S103688[88], 5}, {NULL}};
const Segment SGI58190[] = {{&S103688[67], 8}, {NULL}};
const Segment SGI58191[] = {{&S103701[44], 10}, {NULL}};
const Segment SGI58192[] = {{&S103701[44], 27}, {NULL}};
const Segment SGI58193[] = {{&S103701[30], 6}, {NULL}};
const Segment SGI58194[] = {{&S103701[119], 3}, {NULL}};
const Segment SGI58195[] = {{&S103701[85], 5}, {NULL}};
const Segment SGI58196[] = {{&S103701[153], 3}, {NULL}};
const Segment SGI58197[] = {{&S103712[45], 7}, {NULL}};
const Segment SGI58198[] = {{&S103712[30], 6}, {NULL}};
const Segment SGI58199[] = {{&S103712[58], 3}, {NULL}};
const Segment SGI58200[] = {{&S103712[84], 5}, {NULL}};
const Segment SGI58201[] = {{&S103712[142], 13}, {NULL}};
const Segment SGI58202[] = {{&S103717[59], 7}, {NULL}};
const Segment SGI58203[] = {{&S103717[59], 25}, {NULL}};
const Segment SGI58204[] = {{&S103717[45], 6}, {NULL}};
const Segment SGI58205[] = {{&S103717[72], 3}, {NULL}};
const Segment SGI58206[] = {{&S103717[98], 5}, {NULL}};
const Segment SGI58207[] = {{&S103717[0], 14}, {NULL}};
const Segment SGI58208[] = {{&S103775[59], 10}, {NULL}};
const Segment SGI58209[] = {{&S103775[59], 27}, {NULL}};
const Segment SGI58210[] = {{&S103775[45], 6}, {NULL}};
const Segment SGI58211[] = {{&S103775[100], 5}, {NULL}};
const Segment SGI58212[] = {{&S103775[0], 14}, {NULL}};
const Segment SGI58213[] = {{&S103776[44], 10}, {NULL}};
const Segment SGI58214[] = {{&S103776[44], 27}, {NULL}};
const Segment SGI58215[] = {{&S103776[30], 6}, {NULL}};
const Segment SGI58216[] = {{&S103776[119], 3}, {NULL}};
const Segment SGI58217[] = {{&S103776[85], 5}, {NULL}};
const Segment SGI58218[] = {{&S103776[130], 14}, {NULL}};
const Segment SGI58219[] = {{&S103956[31], 5}, {NULL}};
const Segment SGI58220[] = {{&S103956[51], 3}, {NULL}};
const Segment SGI58221[] = {{&S103976[44], 10}, {NULL}};
const Segment SGI58222[] = {{&S103976[44], 21}, {NULL}};
const Segment SGI58223[] = {{&S103976[30], 6}, {NULL}};
const Segment SGI58224[] = {{&S103976[120], 3}, {NULL}};
const Segment SGI58225[] = {{&S103976[86], 5}, {NULL}};
const Segment SGI58226[] = {{&S103976[65], 8}, {NULL}};
const Segment SGI58227[] = {{&S103990[44], 13}, {NULL}};
const Segment SGI58228[] = {{&S103990[44], 30}, {NULL}};
const Segment SGI58229[] = {{&S103990[30], 6}, {NULL}};
const Segment SGI58230[] = {{&S103990[122], 3}, {NULL}};
const Segment SGI58231[] = {{&S103990[88], 5}, {NULL}};
const Segment SGI58232[] = {{&S103990[156], 3}, {NULL}};
const Segment SGI58233[] = {{&S103991[45], 9}, {NULL}};
const Segment SGI58234[] = {{&S103991[30], 6}, {NULL}};
const Segment SGI58235[] = {{&S103991[113], 3}, {NULL}};
const Segment SGI58236[] = {{&S103991[79], 5}, {NULL}};
const Segment SGI58237[] = {{&S103991[137], 13}, {NULL}};
const Segment SGI58238[] = {{&S103999[59], 7}, {NULL}};
const Segment SGI58239[] = {{&S103999[59], 18}, {NULL}};
const Segment SGI58240[] = {{&S103999[45], 6}, {NULL}};
const Segment SGI58241[] = {{&S103999[125], 3}, {NULL}};
const Segment SGI58242[] = {{&S103999[91], 5}, {NULL}};
const Segment SGI58243[] = {{&S103999[0], 14}, {NULL}};
const Segment SGI58244[] = {{&S104002[31], 39}, {NULL}};
const Segment SGI58245[] = {{&S104002[85], 3}, {NULL}};
const Segment SGI58246[] = {{&S104004[57], 10}, {NULL}};
const Segment SGI58247[] = {{&S104004[57], 27}, {NULL}};
const Segment SGI58248[] = {{&S104004[43], 6}, {NULL}};
const Segment SGI58249[] = {{&S104004[132], 3}, {NULL}};
const Segment SGI58250[] = {{&S104004[98], 5}, {NULL}};
const Segment SGI58251[] = {{&S104004[0], 12}, {NULL}};
const Segment SGI58252[] = {{&S104010[45], 13}, {NULL}};
const Segment SGI58253[] = {{&S104010[30], 6}, {NULL}};
const Segment SGI58254[] = {{&S104010[117], 3}, {NULL}};
const Segment SGI58255[] = {{&S104010[83], 5}, {NULL}};
const Segment SGI58256[] = {{&S104010[141], 13}, {NULL}};
const Segment SGI58257[] = {{&S104046[59], 10}, {NULL}};
const Segment SGI58258[] = {{&S104046[59], 27}, {NULL}};
const Segment SGI58259[] = {{&S104046[45], 6}, {NULL}};
const Segment SGI58260[] = {{&S104046[134], 3}, {NULL}};
const Segment SGI58261[] = {{&S104046[100], 5}, {NULL}};
const Segment SGI58262[] = {{&S104046[0], 14}, {NULL}};
const Segment SGI58263[] = {{&S12406[38], 11}, {NULL}};
const Segment SGI58264[] = {{&S12406[31], 6}, {NULL}};
const Segment SGI58265[] = {{&S12407[44], 11}, {NULL}};
const Segment SGI58266[] = {{&S12407[44], 28}, {NULL}};
const Segment SGI58267[] = {{&S12407[30], 6}, {NULL}};
const Segment SGI58268[] = {{&S12407[120], 3}, {NULL}};
const Segment SGI58269[] = {{&S12407[86], 5}, {NULL}};
const Segment SGI58270[] = {{&S12408[44], 11}, {NULL}};
const Segment SGI58271[] = {{&S12408[44], 28}, {NULL}};
const Segment SGI58272[] = {{&S12408[30], 6}, {NULL}};
const Segment SGI58273[] = {{&S12408[89], 4}, {NULL}};
const Segment SGI58274[] = {{&S12408[89], 3}, {NULL}};
const Segment SGI58275[] = {{&S12409[44], 30}, {NULL}};
const Segment SGI58276[] = {{&S12409[44], 47}, {NULL}};
const Segment SGI58277[] = {{&S12409[30], 6}, {NULL}};
const Segment SGI58278[] = {{&S12409[139], 3}, {NULL}};
const Segment SGI58279[] = {{&S12409[105], 5}, {NULL}};
const Segment SGI58280[] = {{&S12410[44], 30}, {NULL}};
const Segment SGI58281[] = {{&S12410[44], 47}, {NULL}};
const Segment SGI58282[] = {{&S12410[30], 6}, {NULL}};
const Segment SGI58283[] = {{&S12410[139], 3}, {NULL}};
const Segment SGI58284[] = {{&S12410[105], 5}, {NULL}};
const Segment SGI58285[] = {{&S12411[44], 11}, {NULL}};
const Segment SGI58286[] = {{&S12411[44], 28}, {NULL}};
const Segment SGI58287[] = {{&S12411[30], 6}, {NULL}};
const Segment SGI58288[] = {{&S12411[120], 3}, {NULL}};
const Segment SGI58289[] = {{&S12411[86], 5}, {NULL}};
const Segment SGI58290[] = {{&S12412[44], 31}, {NULL}};
const Segment SGI58291[] = {{&S12412[44], 48}, {NULL}};
const Segment SGI58292[] = {{&S12412[30], 6}, {NULL}};
const Segment SGI58293[] = {{&S12412[140], 3}, {NULL}};
const Segment SGI58294[] = {{&S12412[106], 5}, {NULL}};
const Segment SGI58295[] = {{&S12413[44], 31}, {NULL}};
const Segment SGI58296[] = {{&S12413[44], 48}, {NULL}};
const Segment SGI58297[] = {{&S12413[30], 6}, {NULL}};
const Segment SGI58298[] = {{&S12413[140], 3}, {NULL}};
const Segment SGI58299[] = {{&S12413[106], 5}, {NULL}};
const Segment SGI58300[] = {{&S12414[44], 31}, {NULL}};
const Segment SGI58301[] = {{&S12414[44], 48}, {NULL}};
const Segment SGI58302[] = {{&S12414[30], 6}, {NULL}};
const Segment SGI58303[] = {{&S12414[140], 3}, {NULL}};
const Segment SGI58304[] = {{&S12414[106], 5}, {NULL}};
const Segment SGI58305[] = {{&S12415[44], 31}, {NULL}};
const Segment SGI58306[] = {{&S12415[44], 48}, {NULL}};
const Segment SGI58307[] = {{&S12415[30], 6}, {NULL}};
const Segment SGI58308[] = {{&S12415[140], 3}, {NULL}};
const Segment SGI58309[] = {{&S12415[106], 5}, {NULL}};
const Segment SGI58310[] = {{&S12416[38], 11}, {NULL}};
const Segment SGI58311[] = {{&S12416[31], 6}, {NULL}};
const Segment SGI58312[] = {{&S12417[44], 30}, {NULL}};
const Segment SGI58313[] = {{&S12417[44], 47}, {NULL}};
const Segment SGI58314[] = {{&S12417[30], 6}, {NULL}};
const Segment SGI58315[] = {{&S12417[139], 3}, {NULL}};
const Segment SGI58316[] = {{&S12417[105], 5}, {NULL}};
const Segment SGI58317[] = {{&S12418[44], 31}, {NULL}};
const Segment SGI58318[] = {{&S12418[44], 48}, {NULL}};
const Segment SGI58319[] = {{&S12418[30], 6}, {NULL}};
const Segment SGI58320[] = {{&S12418[140], 3}, {NULL}};
const Segment SGI58321[] = {{&S12418[106], 5}, {NULL}};
const Segment SGI58322[] = {{&S12419[44], 11}, {NULL}};
const Segment SGI58323[] = {{&S12419[44], 28}, {NULL}};
const Segment SGI58324[] = {{&S12419[30], 6}, {NULL}};
const Segment SGI58325[] = {{&S12419[89], 6}, {NULL}};
const Segment SGI58326[] = {{&S12420[44], 31}, {NULL}};
const Segment SGI58327[] = {{&S12420[44], 48}, {NULL}};
const Segment SGI58328[] = {{&S12420[30], 6}, {NULL}};
const Segment SGI58329[] = {{&S12420[140], 3}, {NULL}};
const Segment SGI58330[] = {{&S12420[106], 5}, {NULL}};
const Segment SGI58331[] = {{&S12421[42], 11}, {NULL}};
const Segment SGI58332[] = {{&S12421[30], 6}, {NULL}};
const Segment SGI58333[] = {{&S12421[118], 3}, {NULL}};
const Segment SGI58334[] = {{&S12421[84], 5}, {NULL}};
const Segment SGI58335[] = {{&S12422[44], 30}, {NULL}};
const Segment SGI58336[] = {{&S12422[44], 47}, {NULL}};
const Segment SGI58337[] = {{&S12422[30], 6}, {NULL}};
const Segment SGI58338[] = {{&S12422[139], 3}, {NULL}};
const Segment SGI58339[] = {{&S12422[105], 5}, {NULL}};
const Segment SGI58340[] = {{&S12423[44], 31}, {NULL}};
const Segment SGI58341[] = {{&S12423[44], 48}, {NULL}};
const Segment SGI58342[] = {{&S12423[30], 6}, {NULL}};
const Segment SGI58343[] = {{&S12423[140], 3}, {NULL}};
const Segment SGI58344[] = {{&S12423[106], 5}, {NULL}};
const Segment SGI58345[] = {{&S12424[44], 11}, {NULL}};
const Segment SGI58346[] = {{&S12424[44], 44}, {NULL}};
const Segment SGI58347[] = {{&S12424[30], 6}, {NULL}};
const Segment SGI58348[] = {{&S12424[143], 3}, {NULL}};
const Segment SGI58349[] = {{&S12424[109], 5}, {NULL}};
const Segment SGI58350[] = {{&S12425[55], 11}, {NULL}};
const Segment SGI58351[] = {{&S12425[55], 44}, {NULL}};
const Segment SGI58352[] = {{&S12425[30], 16}, {NULL}};
const Segment SGI58353[] = {{&S12425[154], 3}, {NULL}};
const Segment SGI58354[] = {{&S12425[120], 5}, {NULL}};
const Segment SGI58355[] = {{&S12426[42], 11}, {NULL}};
const Segment SGI58356[] = {{&S12426[30], 6}, {NULL}};
const Segment SGI58357[] = {{&S12426[118], 3}, {NULL}};
const Segment SGI58358[] = {{&S12426[84], 5}, {NULL}};
const Segment SGI58359[] = {{&S12428[44], 11}, {NULL}};
const Segment SGI58360[] = {{&S12428[44], 35}, {NULL}};
const Segment SGI58361[] = {{&S12428[30], 6}, {NULL}};
const Segment SGI58362[] = {{&S12428[127], 3}, {NULL}};
const Segment SGI58363[] = {{&S12428[93], 5}, {NULL}};
const Segment SGI58364[] = {{&S12429[44], 31}, {NULL}};
const Segment SGI58365[] = {{&S12429[44], 48}, {NULL}};
const Segment SGI58366[] = {{&S12429[30], 6}, {NULL}};
const Segment SGI58367[] = {{&S12429[140], 3}, {NULL}};
const Segment SGI58368[] = {{&S12429[106], 5}, {NULL}};
const Segment SGI58369[] = {{&S12430[44], 31}, {NULL}};
const Segment SGI58370[] = {{&S12430[44], 48}, {NULL}};
const Segment SGI58371[] = {{&S12430[30], 6}, {NULL}};
const Segment SGI58372[] = {{&S12430[140], 3}, {NULL}};
const Segment SGI58373[] = {{&S12430[106], 5}, {NULL}};
const Segment SGI58374[] = {{&S12431[44], 31}, {NULL}};
const Segment SGI58375[] = {{&S12431[44], 48}, {NULL}};
const Segment SGI58376[] = {{&S12431[30], 6}, {NULL}};
const Segment SGI58377[] = {{&S12431[140], 3}, {NULL}};
const Segment SGI58378[] = {{&S12431[106], 5}, {NULL}};
const Segment SGI58379[] = {{&S12432[44], 31}, {NULL}};
const Segment SGI58380[] = {{&S12432[44], 48}, {NULL}};
const Segment SGI58381[] = {{&S12432[30], 6}, {NULL}};
const Segment SGI58382[] = {{&S12432[140], 3}, {NULL}};
const Segment SGI58383[] = {{&S12432[106], 5}, {NULL}};
const Segment SGI58384[] = {{&S12433[44], 31}, {NULL}};
const Segment SGI58385[] = {{&S12433[44], 48}, {NULL}};
const Segment SGI58386[] = {{&S104029[44], 15}, {NULL}};
const Segment SGI58387[] = {{&S12433[30], 6}, {NULL}};
const Segment SGI58388[] = {{&S104029[44], 26}, {NULL}};
const Segment SGI58389[] = {{&S104029[30], 6}, {NULL}};
const Segment SGI58390[] = {{&S12433[140], 3}, {NULL}};
const Segment SGI58391[] = {{&S104029[118], 3}, {NULL}};
const Segment SGI58392[] = {{&S12433[106], 5}, {NULL}};
const Segment SGI58393[] = {{&S104029[84], 5}, {NULL}};
const Segment SGI58394[] = {{&S104029[142], 13}, {NULL}};
const Segment SGI58395[] = {{&S12434[44], 11}, {NULL}};
const Segment SGI58396[] = {{&S12434[44], 35}, {NULL}};
const Segment SGI58397[] = {{&S12434[30], 6}, {NULL}};
const Segment SGI58398[] = {{&S104030[45], 15}, {NULL}};
const Segment SGI58399[] = {{&S12434[127], 3}, {NULL}};
const Segment SGI58400[] = {{&S104030[30], 6}, {NULL}};
const Segment SGI58401[] = {{&S12434[93], 5}, {NULL}};
const Segment SGI58402[] = {{&S104030[119], 3}, {NULL}};
const Segment SGI58403[] = {{&S104030[85], 5}, {NULL}};
const Segment SGI58404[] = {{&S104030[143], 13}, {NULL}};
const Segment SGI58405[] = {{&S12435[44], 11}, {NULL}};
const Segment SGI58406[] = {{&S12435[44], 37}, {NULL}};
const Segment SGI58407[] = {{&S12435[30], 6}, {NULL}};
const Segment SGI58408[] = {{&S12435[129], 3}, {NULL}};
const Segment SGI58409[] = {{&S12435[95], 5}, {NULL}};
const Segment SGI58410[] = {{&S12436[44], 11}, {NULL}};
const Segment SGI58411[] = {{&S12436[44], 35}, {NULL}};
const Segment SGI58412[] = {{&S12436[30], 6}, {NULL}};
const Segment SGI58413[] = {{&S12436[127], 3}, {NULL}};
const Segment SGI58414[] = {{&S12436[93], 5}, {NULL}};
const Segment SGI58415[] = {{&S12437[44], 30}, {NULL}};
const Segment SGI58416[] = {{&S12437[44], 47}, {NULL}};
const Segment SGI58417[] = {{&S12437[30], 6}, {NULL}};
const Segment SGI58418[] = {{&S12437[139], 3}, {NULL}};
const Segment SGI58419[] = {{&S12437[105], 5}, {NULL}};
const Segment SGI58420[] = {{&S12438[44], 31}, {NULL}};
const Segment SGI58421[] = {{&S12438[44], 48}, {NULL}};
const Segment SGI58422[] = {{&S12438[30], 6}, {NULL}};
const Segment SGI58423[] = {{&S12438[140], 3}, {NULL}};
const Segment SGI58424[] = {{&S12438[106], 5}, {NULL}};
const Segment SGI58425[] = {{&S12440[44], 28}, {NULL}};
const Segment SGI58426[] = {{&S12440[30], 6}, {NULL}};
const Segment SGI58427[] = {{&S12441[44], 11}, {NULL}};
const Segment SGI58428[] = {{&S12441[44], 23}, {NULL}};
const Segment SGI58429[] = {{&S12441[30], 6}, {NULL}};
const Segment SGI58430[] = {{&S12441[115], 3}, {NULL}};
const Segment SGI58431[] = {{&S12441[81], 5}, {NULL}};
const Segment SGI58432[] = {{&S12442[44], 31}, {NULL}};
const Segment SGI58433[] = {{&S12442[44], 48}, {NULL}};
const Segment SGI58434[] = {{&S12442[30], 6}, {NULL}};
const Segment SGI58435[] = {{&S12442[140], 3}, {NULL}};
const Segment SGI58436[] = {{&S12442[106], 5}, {NULL}};
const Segment SGI58437[] = {{&S12443[44], 31}, {NULL}};
const Segment SGI58438[] = {{&S12443[44], 48}, {NULL}};
const Segment SGI58439[] = {{&S12443[30], 6}, {NULL}};
const Segment SGI58440[] = {{&S12443[140], 3}, {NULL}};
const Segment SGI58441[] = {{&S12443[106], 5}, {NULL}};
const Segment SGI58442[] = {{&S12444[44], 11}, {NULL}};
const Segment SGI58443[] = {{&S12444[44], 27}, {NULL}};
const Segment SGI58444[] = {{&S12444[30], 6}, {NULL}};
const Segment SGI58445[] = {{&S12444[119], 3}, {NULL}};
const Segment SGI58446[] = {{&S12444[85], 5}, {NULL}};
const Segment SGI58447[] = {{&S12445[44], 31}, {NULL}};
const Segment SGI58448[] = {{&S12445[44], 48}, {NULL}};
const Segment SGI58449[] = {{&S12445[30], 6}, {NULL}};
const Segment SGI58450[] = {{&S12445[140], 3}, {NULL}};
const Segment SGI58451[] = {{&S12445[106], 5}, {NULL}};
const Segment SGI58452[] = {{&S12446[44], 11}, {NULL}};
const Segment SGI58453[] = {{&S12446[44], 27}, {NULL}};
const Segment SGI58454[] = {{&S12446[30], 6}, {NULL}};
const Segment SGI58455[] = {{&S12446[119], 3}, {NULL}};
const Segment SGI58456[] = {{&S12446[85], 5}, {NULL}};
const Segment SGI58457[] = {{&S12447[44], 11}, {NULL}};
const Segment SGI58458[] = {{&S12447[44], 31}, {NULL}};
const Segment SGI58459[] = {{&S12447[30], 6}, {NULL}};
const Segment SGI58460[] = {{&S12447[123], 3}, {NULL}};
const Segment SGI58461[] = {{&S12447[89], 5}, {NULL}};
const Segment SGI58462[] = {{&S12448[44], 31}, {NULL}};
const Segment SGI58463[] = {{&S12448[44], 48}, {NULL}};
const Segment SGI58464[] = {{&S12448[30], 6}, {NULL}};
const Segment SGI58465[] = {{&S12448[140], 3}, {NULL}};
const Segment SGI58466[] = {{&S12448[106], 5}, {NULL}};
const Segment SGI58467[] = {{&S12449[44], 11}, {NULL}};
const Segment SGI58468[] = {{&S12449[44], 31}, {NULL}};
const Segment SGI58469[] = {{&S12449[30], 6}, {NULL}};
const Segment SGI58470[] = {{&S12449[123], 3}, {NULL}};
const Segment SGI58471[] = {{&S12449[89], 5}, {NULL}};
const Segment SGI58472[] = {{&S12450[44], 11}, {NULL}};
const Segment SGI58473[] = {{&S12450[44], 31}, {NULL}};
const Segment SGI58474[] = {{&S12450[30], 6}, {NULL}};
const Segment SGI58475[] = {{&S12450[123], 3}, {NULL}};
const Segment SGI58476[] = {{&S12450[89], 5}, {NULL}};
const Segment SGI58477[] = {{&S12451[44], 31}, {NULL}};
const Segment SGI58478[] = {{&S12451[44], 48}, {NULL}};
const Segment SGI58479[] = {{&S12451[30], 6}, {NULL}};
const Segment SGI58480[] = {{&S12451[140], 3}, {NULL}};
const Segment SGI58481[] = {{&S12451[106], 5}, {NULL}};
const Segment SGI58482[] = {{&S12452[44], 11}, {NULL}};
const Segment SGI58483[] = {{&S12452[44], 30}, {NULL}};
const Segment SGI58484[] = {{&S12452[30], 6}, {NULL}};
const Segment SGI58485[] = {{&S12452[122], 3}, {NULL}};
const Segment SGI58486[] = {{&S12452[88], 5}, {NULL}};
const Segment SGI58487[] = {{&S12453[44], 31}, {NULL}};
const Segment SGI58488[] = {{&S12453[44], 48}, {NULL}};
const Segment SGI58489[] = {{&S12453[30], 6}, {NULL}};
const Segment SGI58490[] = {{&S12453[140], 3}, {NULL}};
const Segment SGI58491[] = {{&S12453[106], 5}, {NULL}};
const Segment SGI58492[] = {{&S104321[44], 26}, {NULL}};
const Segment SGI58493[] = {{&S104321[44], 37}, {NULL}};
const Segment SGI58494[] = {{&S104321[30], 6}, {NULL}};
const Segment SGI58495[] = {{&S104321[129], 3}, {NULL}};
const Segment SGI58496[] = {{&S104321[95], 5}, {NULL}};
const Segment SGI58497[] = {{&S104334[30], 6}, {NULL}};
const Segment SGI58498[] = {{&S104334[91], 3}, {NULL}};
const Segment SGI58499[] = {{&S104334[125], 3}, {NULL}};
const Segment SGI58500[] = {{&S104334[57], 5}, {NULL}};
const Segment SGI58501[] = {{&S104381[59], 10}, {NULL}};
const Segment SGI58502[] = {{&S104381[59], 27}, {NULL}};
const Segment SGI58503[] = {{&S104381[45], 6}, {NULL}};
const Segment SGI58504[] = {{&S104381[134], 3}, {NULL}};
const Segment SGI58505[] = {{&S104381[100], 5}, {NULL}};
const Segment SGI58506[] = {{&S104381[0], 14}, {NULL}};
const Segment SGI58507[] = {{&S104415[56], 10}, {NULL}};
const Segment SGI58508[] = {{&S104415[56], 27}, {NULL}};
const Segment SGI58509[] = {{&S104415[42], 6}, {NULL}};
const Segment SGI58510[] = {{&S104415[131], 3}, {NULL}};
const Segment SGI58511[] = {{&S104415[97], 5}, {NULL}};
const Segment SGI58512[] = {{&S104335[30], 6}, {NULL}};
const Segment SGI58513[] = {{&S104335[91], 3}, {NULL}};
const Segment SGI58514[] = {{&S104335[125], 3}, {NULL}};
const Segment SGI58515[] = {{&S104335[57], 5}, {NULL}};
const Segment SGI58516[] = {{&S104376[45], 7}, {NULL}};
const Segment SGI58517[] = {{&S104376[30], 6}, {NULL}};
const Segment SGI58518[] = {{&S104376[112], 3}, {NULL}};
const Segment SGI58519[] = {{&S104376[78], 5}, {NULL}};
const Segment SGI58520[] = {{&S104376[136], 13}, {NULL}};
const Segment SGI58521[] = {{&S104378[31], 39}, {NULL}};
const Segment SGI58522[] = {{&S104378[85], 3}, {NULL}};
const Segment SGI58523[] = {{&S104388[45], 14}, {NULL}};
const Segment SGI58524[] = {{&S104388[30], 6}, {NULL}};
const Segment SGI58525[] = {{&S104388[118], 3}, {NULL}};
const Segment SGI58526[] = {{&S104388[84], 5}, {NULL}};
const Segment SGI58527[] = {{&S104388[142], 13}, {NULL}};
const Segment SGI58528[] = {{&S104389[44], 9}, {NULL}};
const Segment SGI58529[] = {{&S104389[44], 20}, {NULL}};
const Segment SGI58530[] = {{&S104389[30], 6}, {NULL}};
const Segment SGI58531[] = {{&S104389[112], 3}, {NULL}};
const Segment SGI58532[] = {{&S104389[78], 5}, {NULL}};
const Segment SGI58533[] = {{&S104389[146], 3}, {NULL}};
const Segment SGI58534[] = {{&S13011[44], 10}, {NULL}};
const Segment SGI58535[] = {{&S13011[44], 27}, {NULL}};
const Segment SGI58536[] = {{&S13011[30], 6}, {NULL}};
const Segment SGI58537[] = {{&S13011[119], 3}, {NULL}};
const Segment SGI58538[] = {{&S13011[85], 5}, {NULL}};
const Segment SGI58539[] = {{&S13012[44], 28}, {NULL}};
const Segment SGI58540[] = {{&S13012[44], 45}, {NULL}};
const Segment SGI58541[] = {{&S13012[30], 6}, {NULL}};
const Segment SGI58542[] = {{&S13012[137], 3}, {NULL}};
const Segment SGI58543[] = {{&S13012[103], 5}, {NULL}};
const Segment SGI58544[] = {{&S13013[44], 11}, {NULL}};
const Segment SGI58545[] = {{&S13013[44], 28}, {NULL}};
const Segment SGI58546[] = {{&S13013[30], 6}, {NULL}};
const Segment SGI58547[] = {{&S13013[89], 4}, {NULL}};
const Segment SGI58548[] = {{&S13013[89], 3}, {NULL}};
const Segment SGI58549[] = {{&S13019[44], 11}, {NULL}};
const Segment SGI58550[] = {{&S13019[44], 28}, {NULL}};
const Segment SGI58551[] = {{&S13019[30], 6}, {NULL}};
const Segment SGI58552[] = {{&S13019[120], 3}, {NULL}};
const Segment SGI58553[] = {{&S13019[86], 5}, {NULL}};
const Segment SGI58554[] = {{&S13022[44], 11}, {NULL}};
const Segment SGI58555[] = {{&S13022[44], 28}, {NULL}};
const Segment SGI58556[] = {{&S13022[30], 6}, {NULL}};
const Segment SGI58557[] = {{&S13022[120], 3}, {NULL}};
const Segment SGI58558[] = {{&S13022[86], 5}, {NULL}};
const Segment SGI58559[] = {{&S13023[38], 10}, {NULL}};
const Segment SGI58560[] = {{&S13023[31], 6}, {NULL}};
const Segment SGI58561[] = {{&S13025[38], 11}, {NULL}};
const Segment SGI58562[] = {{&S13025[31], 6}, {NULL}};
const Segment SGI58563[] = {{&S13026[44], 11}, {NULL}};
const Segment SGI58564[] = {{&S13026[44], 28}, {NULL}};
const Segment SGI58565[] = {{&S13026[30], 6}, {NULL}};
const Segment SGI58566[] = {{&S13026[120], 3}, {NULL}};
const Segment SGI58567[] = {{&S13026[86], 5}, {NULL}};
const Segment SGI58568[] = {{&S13027[44], 10}, {NULL}};
const Segment SGI58569[] = {{&S13027[44], 27}, {NULL}};
const Segment SGI58570[] = {{&S13027[30], 6}, {NULL}};
const Segment SGI58571[] = {{&S13027[88], 4}, {NULL}};
const Segment SGI58572[] = {{&S13027[88], 3}, {NULL}};
const Segment SGI58573[] = {{&S13033[38], 11}, {NULL}};
const Segment SGI58574[] = {{&S13033[31], 6}, {NULL}};
const Segment SGI58575[] = {{&S13040[38], 11}, {NULL}};
const Segment SGI58576[] = {{&S13040[31], 6}, {NULL}};
const Segment SGI58577[] = {{&S13045[44], 28}, {NULL}};
const Segment SGI58578[] = {{&S13045[44], 45}, {NULL}};
const Segment SGI58579[] = {{&S13045[30], 6}, {NULL}};
const Segment SGI58580[] = {{&S13045[137], 3}, {NULL}};
const Segment SGI58581[] = {{&S13045[103], 5}, {NULL}};
const Segment SGI58582[] = {{&S13046[42], 11}, {NULL}};
const Segment SGI58583[] = {{&S13046[30], 6}, {NULL}};
const Segment SGI58584[] = {{&S13046[118], 3}, {NULL}};
const Segment SGI58585[] = {{&S13046[84], 5}, {NULL}};
const Segment SGI58586[] = {{&S13047[42], 10}, {NULL}};
const Segment SGI58587[] = {{&S13047[30], 6}, {NULL}};
const Segment SGI58588[] = {{&S13047[117], 3}, {NULL}};
const Segment SGI58589[] = {{&S13047[83], 5}, {NULL}};
const Segment SGI58590[] = {{&S13048[44], 29}, {NULL}};
const Segment SGI58591[] = {{&S13048[44], 46}, {NULL}};
const Segment SGI58592[] = {{&S13048[30], 6}, {NULL}};
const Segment SGI58593[] = {{&S13048[138], 3}, {NULL}};
const Segment SGI58594[] = {{&S13048[104], 5}, {NULL}};
const Segment SGI58595[] = {{&S13049[44], 28}, {NULL}};
const Segment SGI58596[] = {{&S13049[44], 45}, {NULL}};
const Segment SGI58597[] = {{&S13049[30], 6}, {NULL}};
const Segment SGI58598[] = {{&S13049[137], 3}, {NULL}};
const Segment SGI58599[] = {{&S13049[103], 5}, {NULL}};
const Segment SGI58600[] = {{&S13051[44], 29}, {NULL}};
const Segment SGI58601[] = {{&S13051[44], 46}, {NULL}};
const Segment SGI58602[] = {{&S13051[30], 6}, {NULL}};
const Segment SGI58603[] = {{&S13051[138], 3}, {NULL}};
const Segment SGI58604[] = {{&S13051[104], 5}, {NULL}};
const Segment SGI58605[] = {{&S13052[44], 29}, {NULL}};
const Segment SGI58606[] = {{&S13052[44], 46}, {NULL}};
const Segment SGI58607[] = {{&S13052[30], 6}, {NULL}};
const Segment SGI58608[] = {{&S13052[138], 3}, {NULL}};
const Segment SGI58609[] = {{&S13052[104], 5}, {NULL}};
const Segment SGI58610[] = {{&S13053[44], 28}, {NULL}};
const Segment SGI58611[] = {{&S13053[44], 45}, {NULL}};
const Segment SGI58612[] = {{&S13053[30], 6}, {NULL}};
const Segment SGI58613[] = {{&S13053[137], 3}, {NULL}};
const Segment SGI58614[] = {{&S13053[103], 5}, {NULL}};
const Segment SGI58615[] = {{&S104773[62], 10}, {NULL}};
const Segment SGI58616[] = {{&S104773[55], 6}, {NULL}};
const Segment SGI58617[] = {{&S104825[31], 8}, {NULL}};
const Segment SGI58618[] = {{&S104825[54], 3}, {NULL}};
const Segment SGI58619[] = {{&S104833[45], 13}, {NULL}};
const Segment SGI58620[] = {{&S104833[30], 6}, {NULL}};
const Segment SGI58621[] = {{&S104833[117], 3}, {NULL}};
const Segment SGI58622[] = {{&S104833[83], 5}, {NULL}};
const Segment SGI58623[] = {{&S104833[141], 13}, {NULL}};
const Segment SGI58624[] = {{&S104834[59], 12}, {NULL}};
const Segment SGI58625[] = {{&S104834[59], 23}, {NULL}};
const Segment SGI58626[] = {{&S104834[45], 6}, {NULL}};
const Segment SGI58627[] = {{&S104834[130], 3}, {NULL}};
const Segment SGI58628[] = {{&S104834[96], 5}, {NULL}};
const Segment SGI58629[] = {{&S104834[0], 14}, {NULL}};
const Segment SGI58630[] = {{&S104837[59], 18}, {NULL}};
const Segment SGI58631[] = {{&S104837[59], 29}, {NULL}};
const Segment SGI58632[] = {{&S104837[45], 6}, {NULL}};
const Segment SGI58633[] = {{&S104837[136], 3}, {NULL}};
const Segment SGI58634[] = {{&S104837[102], 5}, {NULL}};
const Segment SGI58635[] = {{&S104837[0], 14}, {NULL}};
const Segment SGI58636[] = {{&S104858[44], 7}, {NULL}};
const Segment SGI58637[] = {{&S104858[44], 18}, {NULL}};
const Segment SGI58638[] = {{&S104858[30], 6}, {NULL}};
const Segment SGI58639[] = {{&S104858[122], 3}, {NULL}};
const Segment SGI58640[] = {{&S104858[76], 5}, {NULL}};
const Segment SGI58641[] = {{&S104561[34], 8}, {NULL}};
const Segment SGI58642[] = {{&S104561[27], 6}, {NULL}};
const Segment SGI58643[] = {{&S104561[107], 2}, {NULL}};
const Segment SGI58644[] = {{&S104561[73], 6}, {NULL}};
const Segment SGI58645[] = {{&S104561[141], 8}, {NULL}};
const Segment SGI58646[] = {{&S104885[44], 10}, {NULL}};
const Segment SGI58647[] = {{&S104885[44], 27}, {NULL}};
const Segment SGI58648[] = {{&S104885[30], 6}, {NULL}};
const Segment SGI58649[] = {{&S104885[119], 3}, {NULL}};
const Segment SGI58650[] = {{&S104885[85], 5}, {NULL}};
const Segment SGI58651[] = {{&S104895[38], 10}, {NULL}};
const Segment SGI58652[] = {{&S104895[31], 6}, {NULL}};
const Segment SGI58653[] = {{&S104986[44], 10}, {NULL}};
const Segment SGI58654[] = {{&S104986[44], 27}, {NULL}};
const Segment SGI58655[] = {{&S104986[30], 6}, {NULL}};
const Segment SGI58656[] = {{&S104986[85], 6}, {NULL}};
const Segment SGI58657[] = {{&S104986[112], 13}, {NULL}};
const Segment SGI58658[] = {{&S104803[44], 14}, {NULL}};
const Segment SGI58659[] = {{&S104803[44], 25}, {NULL}};
const Segment SGI58660[] = {{&S104803[30], 6}, {NULL}};
const Segment SGI58661[] = {{&S104803[117], 3}, {NULL}};
const Segment SGI58662[] = {{&S104803[83], 5}, {NULL}};
const Segment SGI58663[] = {{&S104803[151], 3}, {NULL}};
const Segment SGI58664[] = {{&S105455[59], 7}, {NULL}};
const Segment SGI58665[] = {{&S105455[59], 19}, {NULL}};
const Segment SGI58666[] = {{&S105455[45], 6}, {NULL}};
const Segment SGI58667[] = {{&S105455[126], 3}, {NULL}};
const Segment SGI58668[] = {{&S105455[92], 5}, {NULL}};
const Segment SGI58669[] = {{&S105455[0], 14}, {NULL}};
const Segment SGI58670[] = {{&S105465[56], 18}, {NULL}};
const Segment SGI58671[] = {{&S105465[56], 29}, {NULL}};
const Segment SGI58672[] = {{&S105465[42], 6}, {NULL}};
const Segment SGI58673[] = {{&S105465[133], 3}, {NULL}};
const Segment SGI58674[] = {{&S105465[99], 5}, {NULL}};
const Segment SGI58675[] = {{&S105512[44], 14}, {NULL}};
const Segment SGI58676[] = {{&S105512[44], 35}, {NULL}};
const Segment SGI58677[] = {{&S105512[30], 6}, {NULL}};
const Segment SGI58678[] = {{&S105512[134], 3}, {NULL}};
const Segment SGI58679[] = {{&S105512[100], 5}, {NULL}};
const Segment SGI58680[] = {{&S105512[79], 8}, {NULL}};
const Segment SGI58681[] = {{&S105534[45], 13}, {NULL}};
const Segment SGI58682[] = {{&S105534[30], 6}, {NULL}};
const Segment SGI58683[] = {{&S105534[117], 3}, {NULL}};
const Segment SGI58684[] = {{&S105534[83], 5}, {NULL}};
const Segment SGI58685[] = {{&S105534[141], 13}, {NULL}};
const Segment SGI58686[] = {{&S105575[45], 11}, {NULL}};
const Segment SGI58687[] = {{&S105575[30], 6}, {NULL}};
const Segment SGI58688[] = {{&S105575[115], 3}, {NULL}};
const Segment SGI58689[] = {{&S105575[81], 5}, {NULL}};
const Segment SGI58690[] = {{&S105575[139], 13}, {NULL}};
const Segment SGI58691[] = {{&S105459[59], 14}, {NULL}};
const Segment SGI58692[] = {{&S105459[59], 25}, {NULL}};
const Segment SGI58693[] = {{&S105459[45], 6}, {NULL}};
const Segment SGI58694[] = {{&S105459[132], 3}, {NULL}};
const Segment SGI58695[] = {{&S105459[98], 5}, {NULL}};
const Segment SGI58696[] = {{&S105459[0], 14}, {NULL}};
const Segment SGI58697[] = {{&S105480[44], 21}, {NULL}};
const Segment SGI58698[] = {{&S105480[44], 32}, {NULL}};
const Segment SGI58699[] = {{&S105480[30], 6}, {NULL}};
const Segment SGI58700[] = {{&S105480[124], 3}, {NULL}};
const Segment SGI58701[] = {{&S105480[90], 5}, {NULL}};
const Segment SGI58702[] = {{&S105480[155], 7}, {NULL}};
const Segment SGI58703[] = {{&S105480[148], 14}, {NULL}};
const Segment SGI58704[] = {{&S105527[31], 8}, {NULL}};
const Segment SGI58705[] = {{&S105527[54], 3}, {NULL}};
const Segment SGI58706[] = {{&S105537[44], 14}, {NULL}};
const Segment SGI58707[] = {{&S105537[44], 25}, {NULL}};
const Segment SGI58708[] = {{&S105537[30], 6}, {NULL}};
const Segment SGI58709[] = {{&S105537[117], 3}, {NULL}};
const Segment SGI58710[] = {{&S105537[83], 5}, {NULL}};
const Segment SGI58711[] = {{&S105537[151], 3}, {NULL}};
const Segment SGI58712[] = {{&S105585[45], 15}, {NULL}};
const Segment SGI58713[] = {{&S105585[30], 6}, {NULL}};
const Segment SGI58714[] = {{&S105585[119], 3}, {NULL}};
const Segment SGI58715[] = {{&S105585[85], 5}, {NULL}};
const Segment SGI58716[] = {{&S105585[143], 13}, {NULL}};
const Segment SGI58717[] = {{&S105586[57], 15}, {NULL}};
const Segment SGI58718[] = {{&S105586[57], 26}, {NULL}};
const Segment SGI58719[] = {{&S105586[43], 6}, {NULL}};
const Segment SGI58720[] = {{&S105586[131], 3}, {NULL}};
const Segment SGI58721[] = {{&S105586[97], 5}, {NULL}};
const Segment SGI58722[] = {{&S105586[0], 12}, {NULL}};
const Segment SGI58723[] = {{&S105587[45], 15}, {NULL}};
const Segment SGI58724[] = {{&S105587[30], 6}, {NULL}};
const Segment SGI58725[] = {{&S105587[119], 3}, {NULL}};
const Segment SGI58726[] = {{&S105587[85], 5}, {NULL}};
const Segment SGI58727[] = {{&S105587[143], 13}, {NULL}};
const Segment SGI58728[] = {{&S105588[31], 13}, {NULL}};
const Segment SGI58729[] = {{&S105588[59], 3}, {NULL}};
const Segment SGI58730[] = {{&S14540[44], 29}, {NULL}};
const Segment SGI58731[] = {{&S14540[44], 46}, {NULL}};
const Segment SGI58732[] = {{&S14540[30], 6}, {NULL}};
const Segment SGI58733[] = {{&S14540[138], 3}, {NULL}};
const Segment SGI58734[] = {{&S14540[104], 5}, {NULL}};
const Segment SGI58735[] = {{&S14541[44], 10}, {NULL}};
const Segment SGI58736[] = {{&S14541[44], 27}, {NULL}};
const Segment SGI58737[] = {{&S14541[30], 6}, {NULL}};
const Segment SGI58738[] = {{&S14541[85], 6}, {NULL}};
const Segment SGI58739[] = {{&S14543[44], 10}, {NULL}};
const Segment SGI58740[] = {{&S14543[44], 27}, {NULL}};
const Segment SGI58741[] = {{&S14543[30], 6}, {NULL}};
const Segment SGI58742[] = {{&S14543[119], 3}, {NULL}};
const Segment SGI58743[] = {{&S14543[85], 5}, {NULL}};
const Segment SGI58744[] = {{&S14545[38], 10}, {NULL}};
const Segment SGI58745[] = {{&S14545[31], 6}, {NULL}};
const Segment SGI58746[] = {{&S14547[44], 10}, {NULL}};
const Segment SGI58747[] = {{&S14547[44], 27}, {NULL}};
const Segment SGI58748[] = {{&S14547[30], 6}, {NULL}};
const Segment SGI58749[] = {{&S14547[88], 4}, {NULL}};
const Segment SGI58750[] = {{&S14547[88], 3}, {NULL}};
const Segment SGI58751[] = {{&S14555[44], 28}, {NULL}};
const Segment SGI58752[] = {{&S14555[44], 45}, {NULL}};
const Segment SGI58753[] = {{&S14555[30], 6}, {NULL}};
const Segment SGI58754[] = {{&S14555[137], 3}, {NULL}};
const Segment SGI58755[] = {{&S14555[103], 5}, {NULL}};
const Segment SGI58756[] = {{&S14556[44], 29}, {NULL}};
const Segment SGI58757[] = {{&S14556[44], 46}, {NULL}};
const Segment SGI58758[] = {{&S14556[30], 6}, {NULL}};
const Segment SGI58759[] = {{&S14556[138], 3}, {NULL}};
const Segment SGI58760[] = {{&S14556[104], 5}, {NULL}};
const Segment SGI58761[] = {{&S14557[44], 10}, {NULL}};
const Segment SGI58762[] = {{&S14557[44], 27}, {NULL}};
const Segment SGI58763[] = {{&S14557[30], 6}, {NULL}};
const Segment SGI58764[] = {{&S14557[88], 6}, {NULL}};
const Segment SGI58765[] = {{&S14559[44], 10}, {NULL}};
const Segment SGI58766[] = {{&S14559[44], 34}, {NULL}};
const Segment SGI58767[] = {{&S14559[30], 6}, {NULL}};
const Segment SGI58768[] = {{&S14559[126], 3}, {NULL}};
const Segment SGI58769[] = {{&S14559[92], 5}, {NULL}};
const Segment SGI58770[] = {{&S105979[34], 10}, {NULL}};
const Segment SGI58771[] = {{&S105979[27], 6}, {NULL}};
const Segment SGI58772[] = {{&S14560[44], 28}, {NULL}};
const Segment SGI58773[] = {{&S14560[44], 45}, {NULL}};
const Segment SGI58774[] = {{&S14560[30], 6}, {NULL}};
const Segment SGI58775[] = {{&S105979[109], 2}, {NULL}};
const Segment SGI58776[] = {{&S14560[137], 3}, {NULL}};
const Segment SGI58777[] = {{&S105979[75], 6}, {NULL}};
const Segment SGI58778[] = {{&S14560[103], 5}, {NULL}};
const Segment SGI58779[] = {{&S105979[143], 8}, {NULL}};
const Segment SGI58780[] = {{&S14561[44], 10}, {NULL}};
const Segment SGI58781[] = {{&S14561[44], 25}, {NULL}};
const Segment SGI58782[] = {{&S14561[30], 6}, {NULL}};
const Segment SGI58783[] = {{&S14561[117], 3}, {NULL}};
const Segment SGI58784[] = {{&S14561[83], 5}, {NULL}};
const Segment SGI58785[] = {{&S105986[34], 10}, {NULL}};
const Segment SGI58786[] = {{&S105986[27], 6}, {NULL}};
const Segment SGI58787[] = {{&S14566[44], 29}, {NULL}};
const Segment SGI58788[] = {{&S14566[44], 46}, {NULL}};
const Segment SGI58789[] = {{&S105986[103], 2}, {NULL}};
const Segment SGI58790[] = {{&S14566[30], 6}, {NULL}};
const Segment SGI58791[] = {{&S105986[69], 6}, {NULL}};
const Segment SGI58792[] = {{&S105986[137], 8}, {NULL}};
const Segment SGI58793[] = {{&S14566[138], 3}, {NULL}};
const Segment SGI58794[] = {{&S14566[104], 5}, {NULL}};
const Segment SGI58795[] = {{&S14569[42], 10}, {NULL}};
const Segment SGI58796[] = {{&S14569[30], 6}, {NULL}};
const Segment SGI58797[] = {{&S14569[117], 3}, {NULL}};
const Segment SGI58798[] = {{&S14569[83], 5}, {NULL}};
const Segment SGI58799[] = {{&S14570[44], 29}, {NULL}};
const Segment SGI58800[] = {{&S14570[44], 46}, {NULL}};
const Segment SGI58801[] = {{&S14570[30], 6}, {NULL}};
const Segment SGI58802[] = {{&S14570[138], 3}, {NULL}};
const Segment SGI58803[] = {{&S14570[104], 5}, {NULL}};
const Segment SGI58804[] = {{&S14572[44], 28}, {NULL}};
const Segment SGI58805[] = {{&S14572[44], 45}, {NULL}};
const Segment SGI58806[] = {{&S14572[30], 6}, {NULL}};
const Segment SGI58807[] = {{&S14572[137], 3}, {NULL}};
const Segment SGI58808[] = {{&S14572[103], 5}, {NULL}};
const Segment SGI58809[] = {{&S14574[44], 28}, {NULL}};
const Segment SGI58810[] = {{&S14574[44], 45}, {NULL}};
const Segment SGI58811[] = {{&S14574[30], 6}, {NULL}};
const Segment SGI58812[] = {{&S14574[137], 3}, {NULL}};
const Segment SGI58813[] = {{&S14574[103], 5}, {NULL}};
const Segment SGI58814[] = {{&S14927[44], 10}, {NULL}};
const Segment SGI58815[] = {{&S14927[44], 27}, {NULL}};
const Segment SGI58816[] = {{&S14927[30], 6}, {NULL}};
const Segment SGI58817[] = {{&S14927[119], 3}, {NULL}};
const Segment SGI58818[] = {{&S14927[85], 5}, {NULL}};
const Segment SGI58819[] = {{&S14928[38], 10}, {NULL}};
const Segment SGI58820[] = {{&S14928[31], 6}, {NULL}};
const Segment SGI58821[] = {{&S14933[47], 10}, {NULL}};
const Segment SGI58822[] = {{&S14933[47], 27}, {NULL}};
const Segment SGI58823[] = {{&S14933[30], 9}, {NULL}};
const Segment SGI58824[] = {{&S14933[122], 3}, {NULL}};
const Segment SGI58825[] = {{&S14933[88], 5}, {NULL}};
const Segment SGI58826[] = {{&S14937[44], 10}, {NULL}};
const Segment SGI58827[] = {{&S14937[44], 27}, {NULL}};
const Segment SGI58828[] = {{&S14937[30], 6}, {NULL}};
const Segment SGI58829[] = {{&S14937[88], 4}, {NULL}};
const Segment SGI58830[] = {{&S14937[88], 3}, {NULL}};
const Segment SGI58831[] = {{&S14938[47], 10}, {NULL}};
const Segment SGI58832[] = {{&S14938[47], 27}, {NULL}};
const Segment SGI58833[] = {{&S14938[30], 9}, {NULL}};
const Segment SGI58834[] = {{&S14938[122], 3}, {NULL}};
const Segment SGI58835[] = {{&S14938[88], 5}, {NULL}};
const Segment SGI58836[] = {{&S14948[44], 10}, {NULL}};
const Segment SGI58837[] = {{&S14948[44], 27}, {NULL}};
const Segment SGI58838[] = {{&S14948[30], 6}, {NULL}};
const Segment SGI58839[] = {{&S14948[88], 6}, {NULL}};
const Segment SGI58840[] = {{&S14949[50], 10}, {NULL}};
const Segment SGI58841[] = {{&S14949[50], 27}, {NULL}};
const Segment SGI58842[] = {{&S14949[30], 12}, {NULL}};
const Segment SGI58843[] = {{&S14949[125], 3}, {NULL}};
const Segment SGI58844[] = {{&S14949[91], 5}, {NULL}};
const Segment SGI58845[] = {{&S14950[44], 10}, {NULL}};
const Segment SGI58846[] = {{&S14950[44], 27}, {NULL}};
const Segment SGI58847[] = {{&S14950[30], 6}, {NULL}};
const Segment SGI58848[] = {{&S14950[120], 3}, {NULL}};
const Segment SGI58849[] = {{&S14950[85], 6}, {NULL}};
const Segment SGI58850[] = {{&S14952[44], 11}, {NULL}};
const Segment SGI58851[] = {{&S14952[44], 28}, {NULL}};
const Segment SGI58852[] = {{&S14952[30], 6}, {NULL}};
const Segment SGI58853[] = {{&S14952[120], 3}, {NULL}};
const Segment SGI58854[] = {{&S14952[86], 5}, {NULL}};
const Segment SGI58855[] = {{&S14953[50], 10}, {NULL}};
const Segment SGI58856[] = {{&S14953[50], 27}, {NULL}};
const Segment SGI58857[] = {{&S14953[30], 12}, {NULL}};
const Segment SGI58858[] = {{&S14953[125], 3}, {NULL}};
const Segment SGI58859[] = {{&S14953[91], 5}, {NULL}};
const Segment SGI58860[] = {{&S14954[47], 10}, {NULL}};
const Segment SGI58861[] = {{&S14954[47], 27}, {NULL}};
const Segment SGI58862[] = {{&S14954[30], 9}, {NULL}};
const Segment SGI58863[] = {{&S14954[122], 3}, {NULL}};
const Segment SGI58864[] = {{&S14954[88], 5}, {NULL}};
const Segment SGI58865[] = {{&S14955[42], 10}, {NULL}};
const Segment SGI58866[] = {{&S14955[30], 6}, {NULL}};
const Segment SGI58867[] = {{&S14955[117], 3}, {NULL}};
const Segment SGI58868[] = {{&S14955[83], 5}, {NULL}};
const Segment SGI58869[] = {{&S14956[44], 29}, {NULL}};
const Segment SGI58870[] = {{&S14956[44], 46}, {NULL}};
const Segment SGI58871[] = {{&S14956[30], 6}, {NULL}};
const Segment SGI58872[] = {{&S14956[138], 3}, {NULL}};
const Segment SGI58873[] = {{&S14956[104], 5}, {NULL}};
const Segment SGI58874[] = {{&S14960[44], 28}, {NULL}};
const Segment SGI58875[] = {{&S14960[44], 45}, {NULL}};
const Segment SGI58876[] = {{&S14960[30], 6}, {NULL}};
const Segment SGI58877[] = {{&S14960[137], 3}, {NULL}};
const Segment SGI58878[] = {{&S14960[103], 5}, {NULL}};
const Segment SGI58879[] = {{&S15046[44], 14}, {NULL}};
const Segment SGI58880[] = {{&S15046[44], 25}, {NULL}};
const Segment SGI58881[] = {{&S15046[30], 6}, {NULL}};
const Segment SGI58882[] = {{&S15046[117], 3}, {NULL}};
const Segment SGI58883[] = {{&S15046[83], 5}, {NULL}};
const Segment SGI58884[] = {{&S16083[44], 7}, {NULL}};
const Segment SGI58885[] = {{&S16083[44], 19}, {NULL}};
const Segment SGI58886[] = {{&S16083[30], 6}, {NULL}};
const Segment SGI58887[] = {{&S16083[111], 3}, {NULL}};
const Segment SGI58888[] = {{&S16083[77], 5}, {NULL}};
const Segment SGI58889[] = {{&S16116[44], 11}, {NULL}};
const Segment SGI58890[] = {{&S16116[44], 30}, {NULL}};
const Segment SGI58891[] = {{&S16116[30], 6}, {NULL}};
const Segment SGI58892[] = {{&S16116[122], 3}, {NULL}};
const Segment SGI58893[] = {{&S16116[88], 5}, {NULL}};
const Segment SGI58894[] = {{&S16130[44], 12}, {NULL}};
const Segment SGI58895[] = {{&S16130[44], 39}, {NULL}};
const Segment SGI58896[] = {{&S16130[30], 6}, {NULL}};
const Segment SGI58897[] = {{&S16130[131], 3}, {NULL}};
const Segment SGI58898[] = {{&S16130[97], 5}, {NULL}};
const Segment SGI58899[] = {{&S16140[44], 8}, {NULL}};
const Segment SGI58900[] = {{&S16140[44], 20}, {NULL}};
const Segment SGI58901[] = {{&S16140[30], 6}, {NULL}};
const Segment SGI58902[] = {{&S16140[112], 3}, {NULL}};
const Segment SGI58903[] = {{&S16140[78], 5}, {NULL}};
const Segment SGI58904[] = {{&S5178[44], 10}, {NULL}};
const Segment SGI58905[] = {{&S5178[44], 27}, {NULL}};
const Segment SGI58906[] = {{&S5178[30], 6}, {NULL}};
const Segment SGI58907[] = {{&S5178[31], 3}, {NULL}};
const Segment SGI58908[] = {{&S5178[85], 5}, {NULL}};
const Segment SGI58909[] = {{&S5199[42], 10}, {NULL}};
const Segment SGI58910[] = {{&S5199[42], 21}, {NULL}};
const Segment SGI58911[] = {{&S5199[30], 4}, {NULL}};
const Segment SGI58912[] = {{&S5199[31], 3}, {NULL}};
const Segment SGI58913[] = {{&S5199[77], 5}, {NULL}};
const Segment SGI58914[] = {{&S5200[42], 10}, {NULL}};
const Segment SGI58915[] = {{&S5200[42], 27}, {NULL}};
const Segment SGI58916[] = {{&S5200[30], 4}, {NULL}};
const Segment SGI58917[] = {{&S5200[31], 3}, {NULL}};
const Segment SGI58918[] = {{&S5200[83], 5}, {NULL}};
const Segment SGI58919[] = {{&S5209[42], 10}, {NULL}};
const Segment SGI58920[] = {{&S5209[42], 22}, {NULL}};
const Segment SGI58921[] = {{&S5209[30], 4}, {NULL}};
const Segment SGI58922[] = {{&S5209[31], 3}, {NULL}};
const Segment SGI58923[] = {{&S5209[78], 5}, {NULL}};
const Segment SGI58924[] = {{&S16215[44], 8}, {NULL}};
const Segment SGI58925[] = {{&S16215[44], 20}, {NULL}};
const Segment SGI58926[] = {{&S16215[30], 6}, {NULL}};
const Segment SGI58927[] = {{&S16215[112], 3}, {NULL}};
const Segment SGI58928[] = {{&S16215[78], 5}, {NULL}};
const Segment SGI58929[] = {{&S16221[44], 13}, {NULL}};
const Segment SGI58930[] = {{&S16221[44], 25}, {NULL}};
const Segment SGI58931[] = {{&S16221[30], 6}, {NULL}};
const Segment SGI58932[] = {{&S16221[117], 3}, {NULL}};
const Segment SGI58933[] = {{&S16221[83], 5}, {NULL}};
const Segment SGI58934[] = {{&S16223[44], 7}, {NULL}};
const Segment SGI58935[] = {{&S16223[44], 19}, {NULL}};
const Segment SGI58936[] = {{&S16223[30], 6}, {NULL}};
const Segment SGI58937[] = {{&S16223[80], 6}, {NULL}};
const Segment SGI58938[] = {{&S16234[44], 11}, {NULL}};
const Segment SGI58939[] = {{&S16234[44], 23}, {NULL}};
const Segment SGI58940[] = {{&S16234[30], 6}, {NULL}};
const Segment SGI58941[] = {{&S16234[115], 3}, {NULL}};
const Segment SGI58942[] = {{&S16234[81], 5}, {NULL}};
const Segment SGI58943[] = {{&S16236[44], 10}, {NULL}};
const Segment SGI58944[] = {{&S16236[44], 22}, {NULL}};
const Segment SGI58945[] = {{&S16236[30], 6}, {NULL}};
const Segment SGI58946[] = {{&S16236[114], 3}, {NULL}};
const Segment SGI58947[] = {{&S16236[80], 5}, {NULL}};
const Segment SGI58948[] = {{&S16245[44], 5}, {NULL}};
const Segment SGI58949[] = {{&S16245[44], 17}, {NULL}};
const Segment SGI58950[] = {{&S16245[30], 6}, {NULL}};
const Segment SGI58951[] = {{&S16245[109], 3}, {NULL}};
const Segment SGI58952[] = {{&S16245[75], 5}, {NULL}};
const Segment SGI58953[] = {{&S16258[44], 15}, {NULL}};
const Segment SGI58954[] = {{&S16258[44], 27}, {NULL}};
const Segment SGI58955[] = {{&S16258[30], 6}, {NULL}};
const Segment SGI58956[] = {{&S16258[119], 3}, {NULL}};
const Segment SGI58957[] = {{&S16258[85], 5}, {NULL}};
const Segment SGI58958[] = {{&S16260[44], 4}, {NULL}};
const Segment SGI58959[] = {{&S16260[44], 12}, {NULL}};
const Segment SGI58960[] = {{&S16260[30], 6}, {NULL}};
const Segment SGI58961[] = {{&S16260[104], 3}, {NULL}};
const Segment SGI58962[] = {{&S16260[70], 5}, {NULL}};
const Segment SGI58963[] = {{&S16264[44], 12}, {NULL}};
const Segment SGI58964[] = {{&S16264[44], 39}, {NULL}};
const Segment SGI58965[] = {{&S16264[30], 6}, {NULL}};
const Segment SGI58966[] = {{&S16264[131], 3}, {NULL}};
const Segment SGI58967[] = {{&S16264[97], 5}, {NULL}};
const Segment SGI58968[] = {{&S16276[44], 16}, {NULL}};
const Segment SGI58969[] = {{&S16276[44], 28}, {NULL}};
const Segment SGI58970[] = {{&S16276[30], 6}, {NULL}};
const Segment SGI58971[] = {{&S16276[120], 3}, {NULL}};
const Segment SGI58972[] = {{&S16276[86], 5}, {NULL}};
const Segment SGI58973[] = {{&S16281[44], 9}, {NULL}};
const Segment SGI58974[] = {{&S16281[44], 26}, {NULL}};
const Segment SGI58975[] = {{&S16281[30], 6}, {NULL}};
const Segment SGI58976[] = {{&S16281[118], 3}, {NULL}};
const Segment SGI58977[] = {{&S16281[84], 5}, {NULL}};
const Segment SGI58978[] = {{&S16288[44], 7}, {NULL}};
const Segment SGI58979[] = {{&S16288[44], 19}, {NULL}};
const Segment SGI58980[] = {{&S16288[30], 6}, {NULL}};
const Segment SGI58981[] = {{&S16288[111], 3}, {NULL}};
const Segment SGI58982[] = {{&S16288[77], 5}, {NULL}};
const Segment SGI58983[] = {{&S16289[44], 6}, {NULL}};
const Segment SGI58984[] = {{&S16289[44], 18}, {NULL}};
const Segment SGI58985[] = {{&S16289[30], 6}, {NULL}};
const Segment SGI58986[] = {{&S16289[110], 3}, {NULL}};
const Segment SGI58987[] = {{&S16289[76], 5}, {NULL}};
const Segment SGI58988[] = {{&S16293[44], 9}, {NULL}};
const Segment SGI58989[] = {{&S16293[44], 26}, {NULL}};
const Segment SGI58990[] = {{&S16293[30], 6}, {NULL}};
const Segment SGI58991[] = {{&S16293[87], 4}, {NULL}};
const Segment SGI58992[] = {{&S16293[87], 3}, {NULL}};
const Segment SGI58993[] = {{&S16299[44], 10}, {NULL}};
const Segment SGI58994[] = {{&S16299[44], 22}, {NULL}};
const Segment SGI58995[] = {{&S16299[30], 6}, {NULL}};
const Segment SGI58996[] = {{&S16299[114], 3}, {NULL}};
const Segment SGI58997[] = {{&S16299[80], 5}, {NULL}};
const Segment SGI58998[] = {{&S16305[44], 9}, {NULL}};
const Segment SGI58999[] = {{&S16305[44], 21}, {NULL}};
const Segment SGI59000[] = {{&S16305[30], 6}, {NULL}};
const Segment SGI59001[] = {{&S16305[113], 3}, {NULL}};
const Segment SGI59002[] = {{&S16305[79], 5}, {NULL}};
const Segment SGI59003[] = {{&S16309[44], 7}, {NULL}};
const Segment SGI59004[] = {{&S16309[44], 19}, {NULL}};
const Segment SGI59005[] = {{&S16309[30], 6}, {NULL}};
const Segment SGI59006[] = {{&S16309[111], 3}, {NULL}};
const Segment SGI59007[] = {{&S16309[77], 5}, {NULL}};
const Segment SGI59008[] = {{&S16310[44], 4}, {NULL}};
const Segment SGI59009[] = {{&S16310[44], 16}, {NULL}};
const Segment SGI59010[] = {{&S16310[30], 6}, {NULL}};
const Segment SGI59011[] = {{&S16310[108], 3}, {NULL}};
const Segment SGI59012[] = {{&S16310[74], 5}, {NULL}};
const Segment SGI59013[] = {{&S16316[44], 10}, {NULL}};
const Segment SGI59014[] = {{&S16316[44], 22}, {NULL}};
const Segment SGI59015[] = {{&S16316[30], 6}, {NULL}};
const Segment SGI59016[] = {{&S16316[114], 3}, {NULL}};
const Segment SGI59017[] = {{&S16316[80], 5}, {NULL}};
const Segment SGI59018[] = {{&S16317[44], 6}, {NULL}};
const Segment SGI59019[] = {{&S16317[44], 18}, {NULL}};
const Segment SGI59020[] = {{&S16317[30], 6}, {NULL}};
const Segment SGI59021[] = {{&S16317[110], 3}, {NULL}};
const Segment SGI59022[] = {{&S16317[76], 5}, {NULL}};
const Segment SGI59023[] = {{&S16319[44], 14}, {NULL}};
const Segment SGI59024[] = {{&S16319[44], 26}, {NULL}};
const Segment SGI59025[] = {{&S16319[30], 6}, {NULL}};
const Segment SGI59026[] = {{&S16319[118], 3}, {NULL}};
const Segment SGI59027[] = {{&S16319[84], 5}, {NULL}};
const Segment SGI59028[] = {{&S16320[44], 8}, {NULL}};
const Segment SGI59029[] = {{&S16320[44], 20}, {NULL}};
const Segment SGI59030[] = {{&S16320[30], 6}, {NULL}};
const Segment SGI59031[] = {{&S16320[112], 3}, {NULL}};
const Segment SGI59032[] = {{&S16320[78], 5}, {NULL}};
const Segment SGI59033[] = {{&S16321[44], 12}, {NULL}};
const Segment SGI59034[] = {{&S16321[44], 39}, {NULL}};
const Segment SGI59035[] = {{&S16321[30], 6}, {NULL}};
const Segment SGI59036[] = {{&S16321[131], 3}, {NULL}};
const Segment SGI59037[] = {{&S16321[97], 5}, {NULL}};
const Segment SGI59038[] = {{&S16326[44], 7}, {NULL}};
const Segment SGI59039[] = {{&S16326[44], 19}, {NULL}};
const Segment SGI59040[] = {{&S16326[30], 6}, {NULL}};
const Segment SGI59041[] = {{&S16326[111], 3}, {NULL}};
const Segment SGI59042[] = {{&S16326[77], 5}, {NULL}};
const Segment SGI59043[] = {{&S16349[44], 5}, {NULL}};
const Segment SGI59044[] = {{&S16349[44], 17}, {NULL}};
const Segment SGI59045[] = {{&S16349[30], 6}, {NULL}};
const Segment SGI59046[] = {{&S16349[109], 3}, {NULL}};
const Segment SGI59047[] = {{&S16349[75], 5}, {NULL}};
const Segment SGI59048[] = {{&S16351[44], 5}, {NULL}};
const Segment SGI59049[] = {{&S16351[44], 17}, {NULL}};
const Segment SGI59050[] = {{&S16351[30], 6}, {NULL}};
const Segment SGI59051[] = {{&S16351[109], 3}, {NULL}};
const Segment SGI59052[] = {{&S16351[75], 5}, {NULL}};
const Segment SGI59053[] = {{&S16352[44], 12}, {NULL}};
const Segment SGI59054[] = {{&S16352[44], 24}, {NULL}};
const Segment SGI59055[] = {{&S16352[30], 6}, {NULL}};
const Segment SGI59056[] = {{&S16352[116], 3}, {NULL}};
const Segment SGI59057[] = {{&S16352[82], 5}, {NULL}};
const Segment SGI59058[] = {{&S16357[44], 6}, {NULL}};
const Segment SGI59059[] = {{&S16357[44], 18}, {NULL}};
const Segment SGI59060[] = {{&S16357[30], 6}, {NULL}};
const Segment SGI59061[] = {{&S16357[110], 3}, {NULL}};
const Segment SGI59062[] = {{&S16357[76], 5}, {NULL}};
const Segment SGI59063[] = {{&S16367[44], 10}, {NULL}};
const Segment SGI59064[] = {{&S16367[44], 22}, {NULL}};
const Segment SGI59065[] = {{&S16367[30], 6}, {NULL}};
const Segment SGI59066[] = {{&S16367[114], 3}, {NULL}};
const Segment SGI59067[] = {{&S16367[80], 5}, {NULL}};
const Segment SGI59068[] = {{&S16372[44], 10}, {NULL}};
const Segment SGI59069[] = {{&S16372[44], 22}, {NULL}};
const Segment SGI59070[] = {{&S16372[30], 6}, {NULL}};
const Segment SGI59071[] = {{&S16372[114], 3}, {NULL}};
const Segment SGI59072[] = {{&S16372[80], 5}, {NULL}};
const Segment SGI59073[] = {{&S16377[44], 15}, {NULL}};
const Segment SGI59074[] = {{&S16377[44], 43}, {NULL}};
const Segment SGI59075[] = {{&S16377[30], 6}, {NULL}};
const Segment SGI59076[] = {{&S16377[135], 3}, {NULL}};
const Segment SGI59077[] = {{&S16377[101], 5}, {NULL}};
const Segment SGI59078[] = {{&S16379[44], 8}, {NULL}};
const Segment SGI59079[] = {{&S16379[44], 20}, {NULL}};
const Segment SGI59080[] = {{&S16379[30], 6}, {NULL}};
const Segment SGI59081[] = {{&S16379[112], 3}, {NULL}};
const Segment SGI59082[] = {{&S16379[78], 5}, {NULL}};
const Segment SGI59083[] = {{&S16380[44], 6}, {NULL}};
const Segment SGI59084[] = {{&S16380[44], 18}, {NULL}};
const Segment SGI59085[] = {{&S16380[30], 6}, {NULL}};
const Segment SGI59086[] = {{&S16380[110], 3}, {NULL}};
const Segment SGI59087[] = {{&S16380[76], 5}, {NULL}};
const Segment SGI59088[] = {{&S16382[44], 5}, {NULL}};
const Segment SGI59089[] = {{&S16382[44], 17}, {NULL}};
const Segment SGI59090[] = {{&S16382[30], 6}, {NULL}};
const Segment SGI59091[] = {{&S16382[109], 3}, {NULL}};
const Segment SGI59092[] = {{&S16382[75], 5}, {NULL}};
const Segment SGI59093[] = {{&S16385[44], 8}, {NULL}};
const Segment SGI59094[] = {{&S16385[44], 20}, {NULL}};
const Segment SGI59095[] = {{&S16385[30], 6}, {NULL}};
const Segment SGI59096[] = {{&S16385[112], 3}, {NULL}};
const Segment SGI59097[] = {{&S16385[78], 5}, {NULL}};
const Segment SGI59098[] = {{&S16386[44], 4}, {NULL}};
const Segment SGI59099[] = {{&S16386[44], 16}, {NULL}};
const Segment SGI59100[] = {{&S16386[30], 6}, {NULL}};
const Segment SGI59101[] = {{&S16386[108], 3}, {NULL}};
const Segment SGI59102[] = {{&S16386[74], 5}, {NULL}};
const Segment SGI59103[] = {{&S16395[44], 7}, {NULL}};
const Segment SGI59104[] = {{&S16395[44], 19}, {NULL}};
const Segment SGI59105[] = {{&S16395[30], 6}, {NULL}};
const Segment SGI59106[] = {{&S16395[111], 3}, {NULL}};
const Segment SGI59107[] = {{&S16395[77], 5}, {NULL}};
const Segment SGI59108[] = {{&S16396[44], 11}, {NULL}};
const Segment SGI59109[] = {{&S16396[44], 23}, {NULL}};
const Segment SGI59110[] = {{&S16396[30], 6}, {NULL}};
const Segment SGI59111[] = {{&S16396[115], 3}, {NULL}};
const Segment SGI59112[] = {{&S16396[81], 5}, {NULL}};
const Segment SGI59113[] = {{&S16403[44], 13}, {NULL}};
const Segment SGI59114[] = {{&S16403[44], 25}, {NULL}};
const Segment SGI59115[] = {{&S16403[30], 6}, {NULL}};
const Segment SGI59116[] = {{&S16403[117], 3}, {NULL}};
const Segment SGI59117[] = {{&S16403[83], 5}, {NULL}};
const Segment SGI59118[] = {{&S16407[44], 5}, {NULL}};
const Segment SGI59119[] = {{&S16407[44], 17}, {NULL}};
const Segment SGI59120[] = {{&S16407[30], 6}, {NULL}};
const Segment SGI59121[] = {{&S16407[109], 3}, {NULL}};
const Segment SGI59122[] = {{&S16407[75], 5}, {NULL}};
const Segment SGI59123[] = {{&S16408[44], 6}, {NULL}};
const Segment SGI59124[] = {{&S16408[44], 18}, {NULL}};
const Segment SGI59125[] = {{&S16408[30], 6}, {NULL}};
const Segment SGI59126[] = {{&S16408[110], 3}, {NULL}};
const Segment SGI59127[] = {{&S16408[76], 5}, {NULL}};
const Segment SGI59128[] = {{&S16410[44], 17}, {NULL}};
const Segment SGI59129[] = {{&S16410[44], 34}, {NULL}};
const Segment SGI59130[] = {{&S16410[30], 6}, {NULL}};
const Segment SGI59131[] = {{&S16410[126], 3}, {NULL}};
const Segment SGI59132[] = {{&S16410[92], 5}, {NULL}};
const Segment SGI59133[] = {{&S16411[44], 9}, {NULL}};
const Segment SGI59134[] = {{&S16411[44], 19}, {NULL}};
const Segment SGI59135[] = {{&S16411[30], 6}, {NULL}};
const Segment SGI59136[] = {{&S16411[111], 3}, {NULL}};
const Segment SGI59137[] = {{&S16411[77], 5}, {NULL}};
const Segment SGI59138[] = {{&S16413[44], 15}, {NULL}};
const Segment SGI59139[] = {{&S16413[44], 27}, {NULL}};
const Segment SGI59140[] = {{&S16413[30], 6}, {NULL}};
const Segment SGI59141[] = {{&S16413[119], 3}, {NULL}};
const Segment SGI59142[] = {{&S16413[85], 5}, {NULL}};
const Segment SGI59143[] = {{&S16421[44], 9}, {NULL}};
const Segment SGI59144[] = {{&S16421[44], 21}, {NULL}};
const Segment SGI59145[] = {{&S16421[30], 6}, {NULL}};
const Segment SGI59146[] = {{&S16421[113], 3}, {NULL}};
const Segment SGI59147[] = {{&S16421[79], 5}, {NULL}};
const Segment SGI59148[] = {{&S16422[44], 10}, {NULL}};
const Segment SGI59149[] = {{&S16422[44], 22}, {NULL}};
const Segment SGI59150[] = {{&S16422[30], 6}, {NULL}};
const Segment SGI59151[] = {{&S16422[114], 3}, {NULL}};
const Segment SGI59152[] = {{&S16422[80], 5}, {NULL}};
const Segment SGI59153[] = {{&S16428[44], 5}, {NULL}};
const Segment SGI59154[] = {{&S16428[44], 17}, {NULL}};
const Segment SGI59155[] = {{&S16428[30], 6}, {NULL}};
const Segment SGI59156[] = {{&S16428[109], 3}, {NULL}};
const Segment SGI59157[] = {{&S16428[75], 5}, {NULL}};
const Segment SGI59158[] = {{&S16432[44], 7}, {NULL}};
const Segment SGI59159[] = {{&S16432[44], 19}, {NULL}};
const Segment SGI59160[] = {{&S16432[30], 6}, {NULL}};
const Segment SGI59161[] = {{&S16432[111], 3}, {NULL}};
const Segment SGI59162[] = {{&S16432[77], 5}, {NULL}};
const Segment SGI59163[] = {{&S16433[44], 7}, {NULL}};
const Segment SGI59164[] = {{&S16433[44], 19}, {NULL}};
const Segment SGI59165[] = {{&S16433[30], 6}, {NULL}};
const Segment SGI59166[] = {{&S16433[111], 3}, {NULL}};
const Segment SGI59167[] = {{&S16433[77], 5}, {NULL}};
const Segment SGI59168[] = {{&S16434[44], 9}, {NULL}};
const Segment SGI59169[] = {{&S16434[44], 21}, {NULL}};
const Segment SGI59170[] = {{&S16434[30], 6}, {NULL}};
const Segment SGI59171[] = {{&S16434[113], 3}, {NULL}};
const Segment SGI59172[] = {{&S16434[79], 5}, {NULL}};
const Segment SGI59173[] = {{&S16436[44], 7}, {NULL}};
const Segment SGI59174[] = {{&S16436[44], 19}, {NULL}};
const Segment SGI59175[] = {{&S16436[30], 6}, {NULL}};
const Segment SGI59176[] = {{&S16436[111], 3}, {NULL}};
const Segment SGI59177[] = {{&S16436[77], 5}, {NULL}};
const Segment SGI59178[] = {{&S16437[44], 13}, {NULL}};
const Segment SGI59179[] = {{&S16437[44], 25}, {NULL}};
const Segment SGI59180[] = {{&S16437[30], 6}, {NULL}};
const Segment SGI59181[] = {{&S16437[117], 3}, {NULL}};
const Segment SGI59182[] = {{&S16437[83], 5}, {NULL}};
const Segment SGI59183[] = {{&S16438[44], 4}, {NULL}};
const Segment SGI59184[] = {{&S16438[44], 16}, {NULL}};
const Segment SGI59185[] = {{&S16438[30], 6}, {NULL}};
const Segment SGI59186[] = {{&S16438[108], 3}, {NULL}};
const Segment SGI59187[] = {{&S16438[74], 5}, {NULL}};
const Segment SGI59188[] = {{&S16440[44], 5}, {NULL}};
const Segment SGI59189[] = {{&S16440[44], 26}, {NULL}};
const Segment SGI59190[] = {{&S16440[30], 6}, {NULL}};
const Segment SGI59191[] = {{&S16440[118], 3}, {NULL}};
const Segment SGI59192[] = {{&S16440[84], 5}, {NULL}};
const Segment SGI59193[] = {{&S9541[44], 10}, {NULL}};
const Segment SGI59194[] = {{&S16441[44], 6}, {NULL}};
const Segment SGI59195[] = {{&S9541[44], 22}, {NULL}};
const Segment SGI59196[] = {{&S9541[30], 6}, {NULL}};
const Segment SGI59197[] = {{&S16441[44], 18}, {NULL}};
const Segment SGI59198[] = {{&S16441[30], 6}, {NULL}};
const Segment SGI59199[] = {{&S16441[110], 3}, {NULL}};
const Segment SGI59200[] = {{&S9541[83], 4}, {NULL}};
const Segment SGI59201[] = {{&S9541[83], 3}, {NULL}};
const Segment SGI59202[] = {{&S16441[76], 5}, {NULL}};
const Segment SGI59203[] = {{&S16442[44], 6}, {NULL}};
const Segment SGI59204[] = {{&S16442[44], 18}, {NULL}};
const Segment SGI59205[] = {{&S16442[30], 6}, {NULL}};
const Segment SGI59206[] = {{&S16442[110], 3}, {NULL}};
const Segment SGI59207[] = {{&S16442[76], 5}, {NULL}};
const Segment SGI59208[] = {{&S16443[44], 14}, {NULL}};
const Segment SGI59209[] = {{&S16443[44], 26}, {NULL}};
const Segment SGI59210[] = {{&S16443[30], 6}, {NULL}};
const Segment SGI59211[] = {{&S16443[118], 3}, {NULL}};
const Segment SGI59212[] = {{&S16443[84], 5}, {NULL}};
const Segment SGI59213[] = {{&S16448[44], 6}, {NULL}};
const Segment SGI59214[] = {{&S16448[44], 18}, {NULL}};
const Segment SGI59215[] = {{&S16448[30], 6}, {NULL}};
const Segment SGI59216[] = {{&S16448[110], 3}, {NULL}};
const Segment SGI59217[] = {{&S16448[76], 5}, {NULL}};
const Segment SGI59218[] = {{&S16449[44], 12}, {NULL}};
const Segment SGI59219[] = {{&S16449[44], 24}, {NULL}};
const Segment SGI59220[] = {{&S16449[30], 6}, {NULL}};
const Segment SGI59221[] = {{&S16449[116], 3}, {NULL}};
const Segment SGI59222[] = {{&S16449[82], 5}, {NULL}};
const Segment SGI59223[] = {{&S16451[44], 6}, {NULL}};
const Segment SGI59224[] = {{&S16451[44], 18}, {NULL}};
const Segment SGI59225[] = {{&S16451[30], 6}, {NULL}};
const Segment SGI59226[] = {{&S16451[110], 3}, {NULL}};
const Segment SGI59227[] = {{&S16451[76], 5}, {NULL}};
const Segment SGI59228[] = {{&S16452[44], 12}, {NULL}};
const Segment SGI59229[] = {{&S16452[44], 24}, {NULL}};
const Segment SGI59230[] = {{&S16452[30], 6}, {NULL}};
const Segment SGI59231[] = {{&S16452[116], 3}, {NULL}};
const Segment SGI59232[] = {{&S16452[82], 5}, {NULL}};
const Segment SGI59233[] = {{&S16453[44], 12}, {NULL}};
const Segment SGI59234[] = {{&S16453[44], 24}, {NULL}};
const Segment SGI59235[] = {{&S16453[30], 6}, {NULL}};
const Segment SGI59236[] = {{&S16453[116], 3}, {NULL}};
const Segment SGI59237[] = {{&S16453[82], 5}, {NULL}};
const Segment SGI59238[] = {{&S16456[44], 11}, {NULL}};
const Segment SGI59239[] = {{&S16456[44], 23}, {NULL}};
const Segment SGI59240[] = {{&S16456[30], 6}, {NULL}};
const Segment SGI59241[] = {{&S16456[115], 3}, {NULL}};
const Segment SGI59242[] = {{&S16456[81], 5}, {NULL}};
const Segment SGI59243[] = {{&S16458[44], 14}, {NULL}};
const Segment SGI59244[] = {{&S16458[44], 26}, {NULL}};
const Segment SGI59245[] = {{&S16458[30], 6}, {NULL}};
const Segment SGI59246[] = {{&S16458[118], 3}, {NULL}};
const Segment SGI59247[] = {{&S16458[84], 5}, {NULL}};
const Segment SGI59248[] = {{&S16462[44], 16}, {NULL}};
const Segment SGI59249[] = {{&S16462[44], 28}, {NULL}};
const Segment SGI59250[] = {{&S16462[30], 6}, {NULL}};
const Segment SGI59251[] = {{&S16462[120], 3}, {NULL}};
const Segment SGI59252[] = {{&S16462[86], 5}, {NULL}};
const Segment SGI59253[] = {{&S16469[44], 9}, {NULL}};
const Segment SGI59254[] = {{&S16469[44], 26}, {NULL}};
const Segment SGI59255[] = {{&S16469[30], 6}, {NULL}};
const Segment SGI59256[] = {{&S16469[87], 6}, {NULL}};
const Segment SGI59257[] = {{&S16470[44], 13}, {NULL}};
const Segment SGI59258[] = {{&S16470[44], 41}, {NULL}};
const Segment SGI59259[] = {{&S16470[30], 6}, {NULL}};
const Segment SGI59260[] = {{&S16470[133], 3}, {NULL}};
const Segment SGI59261[] = {{&S16470[99], 5}, {NULL}};
const Segment SGI59262[] = {{&S16471[44], 14}, {NULL}};
const Segment SGI59263[] = {{&S16471[44], 26}, {NULL}};
const Segment SGI59264[] = {{&S16471[30], 6}, {NULL}};
const Segment SGI59265[] = {{&S16471[118], 3}, {NULL}};
const Segment SGI59266[] = {{&S16471[84], 5}, {NULL}};
const Segment SGI59267[] = {{&S16473[44], 7}, {NULL}};
const Segment SGI59268[] = {{&S16473[44], 19}, {NULL}};
const Segment SGI59269[] = {{&S16473[30], 6}, {NULL}};
const Segment SGI59270[] = {{&S16473[111], 3}, {NULL}};
const Segment SGI59271[] = {{&S16473[77], 5}, {NULL}};
const Segment SGI59272[] = {{&S16474[44], 7}, {NULL}};
const Segment SGI59273[] = {{&S16474[44], 19}, {NULL}};
const Segment SGI59274[] = {{&S16474[30], 6}, {NULL}};
const Segment SGI59275[] = {{&S16474[111], 3}, {NULL}};
const Segment SGI59276[] = {{&S9554[44], 10}, {NULL}};
const Segment SGI59277[] = {{&S16474[77], 5}, {NULL}};
const Segment SGI59278[] = {{&S9554[44], 21}, {NULL}};
const Segment SGI59279[] = {{&S9554[30], 6}, {NULL}};
const Segment SGI59280[] = {{&S9554[113], 3}, {NULL}};
const Segment SGI59281[] = {{&S9554[79], 5}, {NULL}};
const Segment SGI59282[] = {{&S16475[44], 5}, {NULL}};
const Segment SGI59283[] = {{&S16475[44], 26}, {NULL}};
const Segment SGI59284[] = {{&S16475[30], 6}, {NULL}};
const Segment SGI59285[] = {{&S16475[118], 3}, {NULL}};
const Segment SGI59286[] = {{&S16475[84], 5}, {NULL}};
const Segment SGI59287[] = {{&S16480[44], 7}, {NULL}};
const Segment SGI59288[] = {{&S16480[44], 19}, {NULL}};
const Segment SGI59289[] = {{&S16480[30], 6}, {NULL}};
const Segment SGI59290[] = {{&S16480[111], 3}, {NULL}};
const Segment SGI59291[] = {{&S16480[77], 5}, {NULL}};
const Segment SGI59292[] = {{&S9564[38], 10}, {NULL}};
const Segment SGI59293[] = {{&S9564[31], 6}, {NULL}};
const Segment SGI59294[] = {{&S16485[44], 10}, {NULL}};
const Segment SGI59295[] = {{&S16485[44], 27}, {NULL}};
const Segment SGI59296[] = {{&S16485[30], 6}, {NULL}};
const Segment SGI59297[] = {{&S16485[119], 3}, {NULL}};
const Segment SGI59298[] = {{&S16485[85], 5}, {NULL}};
const Segment SGI59299[] = {{&S16486[44], 5}, {NULL}};
const Segment SGI59300[] = {{&S16486[44], 17}, {NULL}};
const Segment SGI59301[] = {{&S16486[30], 6}, {NULL}};
const Segment SGI59302[] = {{&S16486[109], 3}, {NULL}};
const Segment SGI59303[] = {{&S16486[75], 5}, {NULL}};
const Segment SGI59304[] = {{&S9573[44], 10}, {NULL}};
const Segment SGI59305[] = {{&S9573[44], 22}, {NULL}};
const Segment SGI59306[] = {{&S9573[30], 6}, {NULL}};
const Segment SGI59307[] = {{&S16489[91], 5}, {NULL}};
const Segment SGI59308[] = {{&S9573[137], 3}, {NULL}};
const Segment SGI59309[] = {{&S9573[103], 5}, {NULL}};
const Segment SGI59310[] = {{&S16494[44], 19}, {NULL}};
const Segment SGI59311[] = {{&S16494[44], 31}, {NULL}};
const Segment SGI59312[] = {{&S16494[30], 6}, {NULL}};
const Segment SGI59313[] = {{&S16494[123], 3}, {NULL}};
const Segment SGI59314[] = {{&S16494[89], 5}, {NULL}};
const Segment SGI59315[] = {{&S16496[44], 12}, {NULL}};
const Segment SGI59316[] = {{&S16496[44], 24}, {NULL}};
const Segment SGI59317[] = {{&S16496[30], 6}, {NULL}};
const Segment SGI59318[] = {{&S16496[116], 3}, {NULL}};
const Segment SGI59319[] = {{&S16496[82], 5}, {NULL}};
const Segment SGI59320[] = {{&S16500[44], 7}, {NULL}};
const Segment SGI59321[] = {{&S16500[44], 19}, {NULL}};
const Segment SGI59322[] = {{&S16500[30], 6}, {NULL}};
const Segment SGI59323[] = {{&S16500[111], 3}, {NULL}};
const Segment SGI59324[] = {{&S16500[77], 5}, {NULL}};
const Segment SGI59325[] = {{&S9590[44], 10}, {NULL}};
const Segment SGI59326[] = {{&S9590[44], 22}, {NULL}};
const Segment SGI59327[] = {{&S9590[30], 6}, {NULL}};
const Segment SGI59328[] = {{&S9590[83], 6}, {NULL}};
const Segment SGI59329[] = {{&S16506[44], 6}, {NULL}};
const Segment SGI59330[] = {{&S16506[44], 18}, {NULL}};
const Segment SGI59331[] = {{&S16506[30], 6}, {NULL}};
const Segment SGI59332[] = {{&S16506[110], 3}, {NULL}};
const Segment SGI59333[] = {{&S16506[76], 5}, {NULL}};
const Segment SGI59334[] = {{&S9592[44], 10}, {NULL}};
const Segment SGI59335[] = {{&S9592[44], 27}, {NULL}};
const Segment SGI59336[] = {{&S9592[30], 6}, {NULL}};
const Segment SGI59337[] = {{&S9592[119], 3}, {NULL}};
const Segment SGI59338[] = {{&S9592[85], 5}, {NULL}};
const Segment SGI59339[] = {{&S16508[44], 9}, {NULL}};
const Segment SGI59340[] = {{&S16508[44], 21}, {NULL}};
const Segment SGI59341[] = {{&S16508[30], 6}, {NULL}};
const Segment SGI59342[] = {{&S16508[113], 3}, {NULL}};
const Segment SGI59343[] = {{&S16508[79], 5}, {NULL}};
const Segment SGI59344[] = {{&S16509[44], 11}, {NULL}};
const Segment SGI59345[] = {{&S16509[44], 32}, {NULL}};
const Segment SGI59346[] = {{&S16509[30], 6}, {NULL}};
const Segment SGI59347[] = {{&S16509[124], 3}, {NULL}};
const Segment SGI59348[] = {{&S16509[90], 5}, {NULL}};
const Segment SGI59349[] = {{&S9596[44], 10}, {NULL}};
const Segment SGI59350[] = {{&S9596[44], 21}, {NULL}};
const Segment SGI59351[] = {{&S9596[30], 6}, {NULL}};
const Segment SGI59352[] = {{&S9596[113], 3}, {NULL}};
const Segment SGI59353[] = {{&S9596[79], 5}, {NULL}};
const Segment SGI59354[] = {{&S16511[44], 15}, {NULL}};
const Segment SGI59355[] = {{&S16511[44], 41}, {NULL}};
const Segment SGI59356[] = {{&S16511[30], 6}, {NULL}};
const Segment SGI59357[] = {{&S16511[133], 3}, {NULL}};
const Segment SGI59358[] = {{&S16511[99], 5}, {NULL}};
const Segment SGI59359[] = {{&S9601[44], 10}, {NULL}};
const Segment SGI59360[] = {{&S9601[44], 29}, {NULL}};
const Segment SGI59361[] = {{&S9601[30], 6}, {NULL}};
const Segment SGI59362[] = {{&S9601[136], 3}, {NULL}};
const Segment SGI59363[] = {{&S9601[102], 5}, {NULL}};
const Segment SGI59364[] = {{&S16515[44], 25}, {NULL}};
const Segment SGI59365[] = {{&S16515[44], 37}, {NULL}};
const Segment SGI59366[] = {{&S16515[30], 6}, {NULL}};
const Segment SGI59367[] = {{&S16515[129], 3}, {NULL}};
const Segment SGI59368[] = {{&S16515[95], 5}, {NULL}};
const Segment SGI59369[] = {{&S9602[44], 10}, {NULL}};
const Segment SGI59370[] = {{&S9602[44], 26}, {NULL}};
const Segment SGI59371[] = {{&S9602[30], 6}, {NULL}};
const Segment SGI59372[] = {{&S9602[127], 3}, {NULL}};
const Segment SGI59373[] = {{&S9602[93], 5}, {NULL}};
const Segment SGI59374[] = {{&S16516[44], 10}, {NULL}};
const Segment SGI59375[] = {{&S16516[44], 22}, {NULL}};
const Segment SGI59376[] = {{&S16516[30], 6}, {NULL}};
const Segment SGI59377[] = {{&S16516[114], 3}, {NULL}};
const Segment SGI59378[] = {{&S16516[80], 5}, {NULL}};
const Segment SGI59379[] = {{&S16518[44], 15}, {NULL}};
const Segment SGI59380[] = {{&S16518[44], 43}, {NULL}};
const Segment SGI59381[] = {{&S16518[30], 6}, {NULL}};
const Segment SGI59382[] = {{&S16518[135], 3}, {NULL}};
const Segment SGI59383[] = {{&S9606[44], 10}, {NULL}};
const Segment SGI59384[] = {{&S16518[101], 5}, {NULL}};
const Segment SGI59385[] = {{&S9606[44], 50}, {NULL}};
const Segment SGI59386[] = {{&S9606[30], 6}, {NULL}};
const Segment SGI59387[] = {{&S9606[178], 3}, {NULL}};
const Segment SGI59388[] = {{&S9606[144], 5}, {NULL}};
const Segment SGI59389[] = {{&S16519[44], 7}, {NULL}};
const Segment SGI59390[] = {{&S16519[44], 19}, {NULL}};
const Segment SGI59391[] = {{&S16519[30], 6}, {NULL}};
const Segment SGI59392[] = {{&S16519[111], 3}, {NULL}};
const Segment SGI59393[] = {{&S16519[77], 5}, {NULL}};
const Segment SGI59394[] = {{&S9608[44], 10}, {NULL}};
const Segment SGI59395[] = {{&S9608[44], 27}, {&S9608[72], 18}, {NULL}};
const Segment SGI59396[] = {{&S9608[30], 6}, {NULL}};
const Segment SGI59397[] = {{&S9608[139], 3}, {NULL}};
const Segment SGI59398[] = {{&S9608[105], 5}, {NULL}};
const Segment SGI59399[] = {{&S16521[44], 7}, {NULL}};
const Segment SGI59400[] = {{&S16521[44], 19}, {NULL}};
const Segment SGI59401[] = {{&S16521[30], 6}, {NULL}};
const Segment SGI59402[] = {{&S16521[111], 3}, {NULL}};
const Segment SGI59403[] = {{&S16521[77], 5}, {NULL}};
const Segment SGI59404[] = {{&S16523[40], 5}, {NULL}};
const Segment SGI59405[] = {{&S16523[40], 17}, {NULL}};
const Segment SGI59406[] = {{&S16523[101], 3}, {NULL}};
const Segment SGI59407[] = {{&S16523[70], 5}, {NULL}};
const Segment SGI59408[] = {{&S16528[44], 8}, {NULL}};
const Segment SGI59409[] = {{&S16528[44], 20}, {NULL}};
const Segment SGI59410[] = {{&S16528[30], 6}, {NULL}};
const Segment SGI59411[] = {{&S16528[112], 3}, {NULL}};
const Segment SGI59412[] = {{&S16528[78], 5}, {NULL}};
const Segment SGI59413[] = {{&S16534[44], 10}, {NULL}};
const Segment SGI59414[] = {{&S16534[44], 22}, {NULL}};
const Segment SGI59415[] = {{&S16534[30], 6}, {NULL}};
const Segment SGI59416[] = {{&S16534[114], 3}, {NULL}};
const Segment SGI59417[] = {{&S16534[80], 5}, {NULL}};
const Segment SGI59418[] = {{&S16538[44], 7}, {NULL}};
const Segment SGI59419[] = {{&S16538[44], 24}, {NULL}};
const Segment SGI59420[] = {{&S16538[30], 6}, {NULL}};
const Segment SGI59421[] = {{&S16538[116], 3}, {NULL}};
const Segment SGI59422[] = {{&S16538[82], 5}, {NULL}};
const Segment SGI59423[] = {{&S16544[44], 21}, {NULL}};
const Segment SGI59424[] = {{&S16544[44], 33}, {NULL}};
const Segment SGI59425[] = {{&S16544[30], 6}, {&S16544[52], 25}, {NULL}};
const Segment SGI59426[] = {{&S16544[125], 3}, {NULL}};
const Segment SGI59427[] = {{&S16544[91], 5}, {NULL}};
const Segment SGI59428[] = {{&S16545[44], 6}, {NULL}};
const Segment SGI59429[] = {{&S16545[44], 18}, {NULL}};
const Segment SGI59430[] = {{&S16545[30], 6}, {NULL}};
const Segment SGI59431[] = {{&S16545[110], 3}, {NULL}};
const Segment SGI59432[] = {{&S16545[76], 5}, {NULL}};
const Segment SGI59433[] = {{&S16548[44], 11}, {NULL}};
const Segment SGI59434[] = {{&S16548[44], 23}, {NULL}};
const Segment SGI59435[] = {{&S16548[30], 6}, {NULL}};
const Segment SGI59436[] = {{&S16548[115], 3}, {NULL}};
const Segment SGI59437[] = {{&S16548[81], 5}, {NULL}};
const Segment SGI59438[] = {{&S16551[44], 13}, {NULL}};
const Segment SGI59439[] = {{&S16551[44], 28}, {NULL}};
const Segment SGI59440[] = {{&S16551[30], 6}, {NULL}};
const Segment SGI59441[] = {{&S16551[89], 6}, {NULL}};
const Segment SGI59442[] = {{&S16552[44], 7}, {NULL}};
const Segment SGI59443[] = {{&S16552[44], 24}, {NULL}};
const Segment SGI59444[] = {{&S16552[30], 6}, {NULL}};
const Segment SGI59445[] = {{&S16552[116], 3}, {NULL}};
const Segment SGI59446[] = {{&S16552[82], 5}, {NULL}};
const Segment SGI59447[] = {{&S16553[44], 18}, {NULL}};
const Segment SGI59448[] = {{&S16553[44], 30}, {NULL}};
const Segment SGI59449[] = {{&S16553[30], 6}, {NULL}};
const Segment SGI59450[] = {{&S16553[122], 3}, {NULL}};
const Segment SGI59451[] = {{&S16553[88], 5}, {NULL}};
const Segment SGI59452[] = {{&S16554[44], 24}, {NULL}};
const Segment SGI59453[] = {{&S16554[44], 35}, {NULL}};
const Segment SGI59454[] = {{&S16554[30], 6}, {NULL}};
const Segment SGI59455[] = {{&S16554[127], 3}, {NULL}};
const Segment SGI59456[] = {{&S16554[93], 5}, {NULL}};
const Segment SGI59457[] = {{&S16555[44], 7}, {NULL}};
const Segment SGI59458[] = {{&S16555[44], 19}, {NULL}};
const Segment SGI59459[] = {{&S16555[30], 6}, {NULL}};
const Segment SGI59460[] = {{&S16555[111], 3}, {NULL}};
const Segment SGI59461[] = {{&S16555[77], 5}, {NULL}};
const Segment SGI59462[] = {{&S16557[44], 6}, {NULL}};
const Segment SGI59463[] = {{&S16557[44], 27}, {NULL}};
const Segment SGI59464[] = {{&S16557[30], 6}, {NULL}};
const Segment SGI59465[] = {{&S16557[119], 3}, {NULL}};
const Segment SGI59466[] = {{&S16557[85], 5}, {NULL}};
const Segment SGI59467[] = {{&S16559[44], 15}, {NULL}};
const Segment SGI59468[] = {{&S16559[44], 43}, {NULL}};
const Segment SGI59469[] = {{&S16559[30], 6}, {NULL}};
const Segment SGI59470[] = {{&S16559[135], 3}, {NULL}};
const Segment SGI59471[] = {{&S16559[101], 5}, {NULL}};
const Segment SGI59472[] = {{&S16560[44], 7}, {NULL}};
const Segment SGI59473[] = {{&S16560[44], 19}, {NULL}};
const Segment SGI59474[] = {{&S16560[30], 6}, {NULL}};
const Segment SGI59475[] = {{&S16560[111], 3}, {NULL}};
const Segment SGI59476[] = {{&S16560[77], 5}, {NULL}};
const Segment SGI59477[] = {{&S16561[44], 11}, {NULL}};
const Segment SGI59478[] = {{&S16561[44], 23}, {NULL}};
const Segment SGI59479[] = {{&S16561[30], 6}, {NULL}};
const Segment SGI59480[] = {{&S16561[115], 3}, {NULL}};
const Segment SGI59481[] = {{&S16561[81], 5}, {NULL}};
const Segment SGI59482[] = {{&S16564[44], 13}, {NULL}};
const Segment SGI59483[] = {{&S16564[44], 25}, {NULL}};
const Segment SGI59484[] = {{&S16564[30], 6}, {NULL}};
const Segment SGI59485[] = {{&S16564[117], 3}, {NULL}};
const Segment SGI59486[] = {{&S16564[83], 5}, {NULL}};
const Segment SGI59487[] = {{&S16565[44], 8}, {NULL}};
const Segment SGI59488[] = {{&S16565[44], 20}, {NULL}};
const Segment SGI59489[] = {{&S16565[30], 6}, {NULL}};
const Segment SGI59490[] = {{&S16565[112], 3}, {NULL}};
const Segment SGI59491[] = {{&S16565[78], 5}, {NULL}};
const Segment SGI59492[] = {{&S16568[44], 7}, {NULL}};
const Segment SGI59493[] = {{&S16568[44], 19}, {NULL}};
const Segment SGI59494[] = {{&S16568[30], 6}, {NULL}};
const Segment SGI59495[] = {{&S16568[111], 3}, {NULL}};
const Segment SGI59496[] = {{&S16568[77], 5}, {NULL}};
const Segment SGI59497[] = {{&S16571[44], 4}, {NULL}};
const Segment SGI59498[] = {{&S16571[44], 16}, {NULL}};
const Segment SGI59499[] = {{&S16571[30], 6}, {NULL}};
const Segment SGI59500[] = {{&S16571[108], 3}, {NULL}};
const Segment SGI59501[] = {{&S16571[74], 5}, {NULL}};
const Segment SGI59502[] = {{&S16573[44], 7}, {NULL}};
const Segment SGI59503[] = {{&S16573[44], 24}, {NULL}};
const Segment SGI59504[] = {{&S16573[30], 6}, {NULL}};
const Segment SGI59505[] = {{&S16573[116], 3}, {NULL}};
const Segment SGI59506[] = {{&S16573[82], 5}, {NULL}};
const Segment SGI59507[] = {{&S16574[44], 11}, {NULL}};
const Segment SGI59508[] = {{&S16574[44], 23}, {NULL}};
const Segment SGI59509[] = {{&S16574[30], 6}, {NULL}};
const Segment SGI59510[] = {{&S16574[115], 3}, {NULL}};
const Segment SGI59511[] = {{&S16574[81], 5}, {NULL}};
const Segment SGI59512[] = {{&S16580[44], 7}, {NULL}};
const Segment SGI59513[] = {{&S16580[44], 19}, {NULL}};
const Segment SGI59514[] = {{&S16580[30], 6}, {NULL}};
const Segment SGI59515[] = {{&S16580[111], 3}, {NULL}};
const Segment SGI59516[] = {{&S16580[77], 5}, {NULL}};
const Segment SGI59517[] = {{&S16584[44], 11}, {NULL}};
const Segment SGI59518[] = {{&S16584[44], 23}, {NULL}};
const Segment SGI59519[] = {{&S16584[30], 6}, {NULL}};
const Segment SGI59520[] = {{&S16584[115], 3}, {NULL}};
const Segment SGI59521[] = {{&S16584[81], 5}, {NULL}};
const Segment SGI59522[] = {{&S16586[44], 5}, {NULL}};
const Segment SGI59523[] = {{&S16586[44], 17}, {NULL}};
const Segment SGI59524[] = {{&S16586[30], 6}, {NULL}};
const Segment SGI59525[] = {{&S16586[109], 3}, {NULL}};
const Segment SGI59526[] = {{&S16586[75], 5}, {NULL}};
const Segment SGI59527[] = {{&S16590[44], 15}, {NULL}};
const Segment SGI59528[] = {{&S16590[44], 27}, {NULL}};
const Segment SGI59529[] = {{&S16590[30], 6}, {NULL}};
const Segment SGI59530[] = {{&S16590[119], 3}, {NULL}};
const Segment SGI59531[] = {{&S16590[85], 5}, {NULL}};
const Segment SGI59532[] = {{&S16596[44], 7}, {NULL}};
const Segment SGI59533[] = {{&S16596[44], 19}, {NULL}};
const Segment SGI59534[] = {{&S16596[30], 6}, {NULL}};
const Segment SGI59535[] = {{&S16596[111], 3}, {NULL}};
const Segment SGI59536[] = {{&S16596[77], 5}, {NULL}};
const Segment SGI59537[] = {{&S16597[44], 7}, {NULL}};
const Segment SGI59538[] = {{&S16597[44], 19}, {NULL}};
const Segment SGI59539[] = {{&S16597[30], 6}, {NULL}};
const Segment SGI59540[] = {{&S16597[111], 3}, {NULL}};
const Segment SGI59541[] = {{&S16597[77], 5}, {NULL}};
const Segment SGI59542[] = {{&S16601[44], 7}, {NULL}};
const Segment SGI59543[] = {{&S16601[44], 19}, {NULL}};
const Segment SGI59544[] = {{&S16601[30], 6}, {NULL}};
const Segment SGI59545[] = {{&S16601[111], 3}, {NULL}};
const Segment SGI59546[] = {{&S16601[77], 5}, {NULL}};
const Segment SGI59547[] = {{&S16602[44], 10}, {NULL}};
const Segment SGI59548[] = {{&S16602[44], 22}, {NULL}};
const Segment SGI59549[] = {{&S16602[30], 6}, {NULL}};
const Segment SGI59550[] = {{&S16602[114], 3}, {NULL}};
const Segment SGI59551[] = {{&S16602[80], 5}, {NULL}};
const Segment SGI59552[] = {{&S16604[44], 10}, {NULL}};
const Segment SGI59553[] = {{&S16604[44], 22}, {NULL}};
const Segment SGI59554[] = {{&S16604[30], 6}, {NULL}};
const Segment SGI59555[] = {{&S16604[114], 3}, {NULL}};
const Segment SGI59556[] = {{&S16604[80], 5}, {NULL}};
const Segment SGI59557[] = {{&S16606[44], 17}, {NULL}};
const Segment SGI59558[] = {{&S16606[30], 6}, {NULL}};
const Segment SGI59559[] = {{&S16606[109], 3}, {NULL}};
const Segment SGI59560[] = {{&S16606[75], 5}, {NULL}};
const Segment SGI59561[] = {{&S16607[44], 4}, {NULL}};
const Segment SGI59562[] = {{&S16607[44], 16}, {NULL}};
const Segment SGI59563[] = {{&S16607[30], 6}, {NULL}};
const Segment SGI59564[] = {{&S16607[108], 3}, {NULL}};
const Segment SGI59565[] = {{&S16607[74], 5}, {NULL}};
const Segment SGI59566[] = {{&S16609[44], 5}, {NULL}};
const Segment SGI59567[] = {{&S16609[44], 18}, {NULL}};
const Segment SGI59568[] = {{&S16609[30], 6}, {NULL}};
const Segment SGI59569[] = {{&S16609[110], 3}, {NULL}};
const Segment SGI59570[] = {{&S16609[76], 5}, {NULL}};
const Segment SGI59571[] = {{&S16613[44], 10}, {NULL}};
const Segment SGI59572[] = {{&S16613[44], 22}, {NULL}};
const Segment SGI59573[] = {{&S16613[30], 6}, {NULL}};
const Segment SGI59574[] = {{&S16613[114], 3}, {NULL}};
const Segment SGI59575[] = {{&S16613[80], 5}, {NULL}};
const Segment SGI59576[] = {{&S16618[44], 7}, {NULL}};
const Segment SGI59577[] = {{&S16618[44], 19}, {NULL}};
const Segment SGI59578[] = {{&S16618[30], 6}, {NULL}};
const Segment SGI59579[] = {{&S16618[111], 3}, {NULL}};
const Segment SGI59580[] = {{&S16618[77], 5}, {NULL}};
const Segment SGI59581[] = {{&S16633[44], 4}, {NULL}};
const Segment SGI59582[] = {{&S16633[44], 16}, {NULL}};
const Segment SGI59583[] = {{&S16633[30], 6}, {NULL}};
const Segment SGI59584[] = {{&S16633[108], 3}, {NULL}};
const Segment SGI59585[] = {{&S16633[74], 5}, {NULL}};
const Segment SGI59586[] = {{&S16636[44], 7}, {NULL}};
const Segment SGI59587[] = {{&S16636[44], 19}, {NULL}};
const Segment SGI59588[] = {{&S16636[30], 6}, {NULL}};
const Segment SGI59589[] = {{&S16636[111], 3}, {NULL}};
const Segment SGI59590[] = {{&S16636[77], 5}, {NULL}};
const Segment SGI59591[] = {{&S16638[44], 7}, {NULL}};
const Segment SGI59592[] = {{&S16638[44], 19}, {NULL}};
const Segment SGI59593[] = {{&S16638[30], 6}, {NULL}};
const Segment SGI59594[] = {{&S16638[111], 3}, {NULL}};
const Segment SGI59595[] = {{&S16638[77], 5}, {NULL}};
const Segment SGI59596[] = {{&S16639[44], 8}, {NULL}};
const Segment SGI59597[] = {{&S16639[44], 24}, {NULL}};
const Segment SGI59598[] = {{&S16639[30], 6}, {NULL}};
const Segment SGI59599[] = {{&S16639[85], 6}, {NULL}};
const Segment SGI59600[] = {{&S16646[44], 10}, {NULL}};
const Segment SGI59601[] = {{&S16646[44], 22}, {NULL}};
const Segment SGI59602[] = {{&S16646[30], 6}, {NULL}};
const Segment SGI59603[] = {{&S16646[114], 3}, {NULL}};
const Segment SGI59604[] = {{&S16646[80], 5}, {NULL}};
const Segment SGI59605[] = {{&S16647[44], 7}, {NULL}};
const Segment SGI59606[] = {{&S16647[44], 28}, {NULL}};
const Segment SGI59607[] = {{&S16647[30], 6}, {NULL}};
const Segment SGI59608[] = {{&S16647[120], 3}, {NULL}};
const Segment SGI59609[] = {{&S16647[86], 5}, {NULL}};
const Segment SGI59610[] = {{&S16651[44], 13}, {NULL}};
const Segment SGI59611[] = {{&S16651[44], 28}, {NULL}};
const Segment SGI59612[] = {{&S16651[30], 6}, {NULL}};
const Segment SGI59613[] = {{&S16651[120], 3}, {NULL}};
const Segment SGI59614[] = {{&S16651[86], 5}, {NULL}};
const Segment SGI59615[] = {{&S16655[44], 5}, {NULL}};
const Segment SGI59616[] = {{&S16655[44], 17}, {NULL}};
const Segment SGI59617[] = {{&S16655[30], 6}, {NULL}};
const Segment SGI59618[] = {{&S16655[109], 3}, {NULL}};
const Segment SGI59619[] = {{&S16655[75], 5}, {NULL}};
const Segment SGI59620[] = {{&S16656[44], 10}, {NULL}};
const Segment SGI59621[] = {{&S16656[44], 22}, {NULL}};
const Segment SGI59622[] = {{&S16656[30], 6}, {NULL}};
const Segment SGI59623[] = {{&S16656[114], 3}, {NULL}};
const Segment SGI59624[] = {{&S16656[80], 5}, {NULL}};
const Segment SGI59625[] = {{&S16659[44], 16}, {NULL}};
const Segment SGI59626[] = {{&S16659[44], 28}, {NULL}};
const Segment SGI59627[] = {{&S16659[30], 6}, {NULL}};
const Segment SGI59628[] = {{&S16659[120], 3}, {NULL}};
const Segment SGI59629[] = {{&S16659[86], 5}, {NULL}};
const Segment SGI59630[] = {{&S16661[44], 7}, {NULL}};
const Segment SGI59631[] = {{&S16661[44], 19}, {NULL}};
const Segment SGI59632[] = {{&S16661[30], 6}, {NULL}};
const Segment SGI59633[] = {{&S16661[111], 3}, {NULL}};
const Segment SGI59634[] = {{&S16661[77], 5}, {NULL}};
const Segment SGI59635[] = {{&S16662[44], 10}, {NULL}};
const Segment SGI59636[] = {{&S16662[44], 22}, {NULL}};
const Segment SGI59637[] = {{&S16662[30], 6}, {NULL}};
const Segment SGI59638[] = {{&S16662[114], 3}, {NULL}};
const Segment SGI59639[] = {{&S16662[80], 5}, {NULL}};
const Segment SGI59640[] = {{&S16665[44], 11}, {NULL}};
const Segment SGI59641[] = {{&S16665[44], 23}, {NULL}};
const Segment SGI59642[] = {{&S16665[30], 6}, {NULL}};
const Segment SGI59643[] = {{&S16665[115], 3}, {NULL}};
const Segment SGI59644[] = {{&S16665[81], 5}, {NULL}};
const Segment SGI59645[] = {{&S16670[44], 11}, {NULL}};
const Segment SGI59646[] = {{&S16670[44], 23}, {NULL}};
const Segment SGI59647[] = {{&S16670[30], 6}, {NULL}};
const Segment SGI59648[] = {{&S16670[115], 3}, {NULL}};
const Segment SGI59649[] = {{&S16670[81], 5}, {NULL}};
const Segment SGI59650[] = {{&S16672[44], 7}, {NULL}};
const Segment SGI59651[] = {{&S16672[44], 19}, {NULL}};
const Segment SGI59652[] = {{&S16672[30], 6}, {NULL}};
const Segment SGI59653[] = {{&S16672[111], 3}, {NULL}};
const Segment SGI59654[] = {{&S16672[77], 5}, {NULL}};
const Segment SGI59655[] = {{&S16676[44], 7}, {NULL}};
const Segment SGI59656[] = {{&S16676[44], 19}, {NULL}};
const Segment SGI59657[] = {{&S16676[30], 6}, {NULL}};
const Segment SGI59658[] = {{&S16676[111], 3}, {NULL}};
const Segment SGI59659[] = {{&S16676[77], 5}, {NULL}};
const Segment SGI59660[] = {{&S16677[44], 9}, {NULL}};
const Segment SGI59661[] = {{&S16677[44], 21}, {NULL}};
const Segment SGI59662[] = {{&S16677[30], 6}, {NULL}};
const Segment SGI59663[] = {{&S16677[113], 3}, {NULL}};
const Segment SGI59664[] = {{&S16677[79], 5}, {NULL}};
const Segment SGI59665[] = {{&S16679[44], 6}, {NULL}};
const Segment SGI59666[] = {{&S16679[44], 18}, {NULL}};
const Segment SGI59667[] = {{&S16679[30], 6}, {NULL}};
const Segment SGI59668[] = {{&S16679[110], 3}, {NULL}};
const Segment SGI59669[] = {{&S16679[76], 5}, {NULL}};
const Segment SGI59670[] = {{&S16680[44], 12}, {NULL}};
const Segment SGI59671[] = {{&S16680[44], 24}, {NULL}};
const Segment SGI59672[] = {{&S16680[30], 6}, {NULL}};
const Segment SGI59673[] = {{&S16680[116], 3}, {NULL}};
const Segment SGI59674[] = {{&S16680[82], 5}, {NULL}};
const Segment SGI59675[] = {{&S16682[44], 5}, {NULL}};
const Segment SGI59676[] = {{&S16682[44], 26}, {NULL}};
const Segment SGI59677[] = {{&S16682[30], 6}, {NULL}};
const Segment SGI59678[] = {{&S16682[118], 3}, {NULL}};
const Segment SGI59679[] = {{&S16682[84], 5}, {NULL}};
const Segment SGI59680[] = {{&S16687[44], 7}, {NULL}};
const Segment SGI59681[] = {{&S16687[44], 19}, {NULL}};
const Segment SGI59682[] = {{&S16687[30], 6}, {NULL}};
const Segment SGI59683[] = {{&S16687[111], 3}, {NULL}};
const Segment SGI59684[] = {{&S16687[77], 5}, {NULL}};
const Segment SGI59685[] = {{&S16689[44], 6}, {NULL}};
const Segment SGI59686[] = {{&S16689[44], 18}, {NULL}};
const Segment SGI59687[] = {{&S16689[30], 6}, {NULL}};
const Segment SGI59688[] = {{&S16689[110], 3}, {NULL}};
const Segment SGI59689[] = {{&S16689[76], 5}, {NULL}};
const Segment SGI59690[] = {{&S16693[44], 12}, {NULL}};
const Segment SGI59691[] = {{&S16693[44], 24}, {NULL}};
const Segment SGI59692[] = {{&S16693[30], 6}, {NULL}};
const Segment SGI59693[] = {{&S16693[116], 3}, {NULL}};
const Segment SGI59694[] = {{&S16693[82], 5}, {NULL}};
const Segment SGI59695[] = {{&S16695[44], 7}, {NULL}};
const Segment SGI59696[] = {{&S16695[44], 19}, {NULL}};
const Segment SGI59697[] = {{&S16695[30], 6}, {NULL}};
const Segment SGI59698[] = {{&S16695[111], 3}, {NULL}};
const Segment SGI59699[] = {{&S16695[77], 5}, {NULL}};
const Segment SGI59700[] = {{&S16704[44], 7}, {NULL}};
const Segment SGI59701[] = {{&S16704[44], 19}, {NULL}};
const Segment SGI59702[] = {{&S16704[30], 6}, {NULL}};
const Segment SGI59703[] = {{&S16704[111], 3}, {NULL}};
const Segment SGI59704[] = {{&S16704[77], 5}, {NULL}};
const Segment SGI59705[] = {{&S16705[44], 10}, {NULL}};
const Segment SGI59706[] = {{&S16705[44], 22}, {NULL}};
const Segment SGI59707[] = {{&S16705[30], 6}, {NULL}};
const Segment SGI59708[] = {{&S16705[114], 3}, {NULL}};
const Segment SGI59709[] = {{&S16705[80], 5}, {NULL}};
const Segment SGI59710[] = {{&S16708[44], 8}, {NULL}};
const Segment SGI59711[] = {{&S16708[44], 20}, {NULL}};
const Segment SGI59712[] = {{&S16708[30], 6}, {NULL}};
const Segment SGI59713[] = {{&S16708[112], 3}, {NULL}};
const Segment SGI59714[] = {{&S16708[78], 5}, {NULL}};
const Segment SGI59715[] = {{&S16711[44], 13}, {NULL}};
const Segment SGI59716[] = {{&S16711[44], 25}, {NULL}};
const Segment SGI59717[] = {{&S16711[30], 6}, {NULL}};
const Segment SGI59718[] = {{&S16711[117], 3}, {NULL}};
const Segment SGI59719[] = {{&S16711[83], 5}, {NULL}};
const Segment SGI59720[] = {{&S16712[44], 14}, {NULL}};
const Segment SGI59721[] = {{&S16712[44], 26}, {NULL}};
const Segment SGI59722[] = {{&S16712[30], 6}, {NULL}};
const Segment SGI59723[] = {{&S16712[87], 6}, {NULL}};
const Segment SGI59724[] = {{&S16716[44], 11}, {NULL}};
const Segment SGI59725[] = {{&S16716[44], 23}, {NULL}};
const Segment SGI59726[] = {{&S16716[30], 6}, {NULL}};
const Segment SGI59727[] = {{&S16716[115], 3}, {NULL}};
const Segment SGI59728[] = {{&S16716[81], 5}, {NULL}};
const Segment SGI59729[] = {{&S16717[44], 11}, {NULL}};
const Segment SGI59730[] = {{&S16717[44], 37}, {NULL}};
const Segment SGI59731[] = {{&S16717[30], 6}, {NULL}};
const Segment SGI59732[] = {{&S16717[129], 3}, {NULL}};
const Segment SGI59733[] = {{&S16717[95], 5}, {NULL}};
const Segment SGI59734[] = {{&S16718[44], 11}, {NULL}};
const Segment SGI59735[] = {{&S16718[44], 37}, {NULL}};
const Segment SGI59736[] = {{&S16718[30], 6}, {NULL}};
const Segment SGI59737[] = {{&S16718[129], 3}, {NULL}};
const Segment SGI59738[] = {{&S16718[95], 5}, {NULL}};
const Segment SGI59739[] = {{&S16722[44], 6}, {NULL}};
const Segment SGI59740[] = {{&S16722[44], 18}, {NULL}};
const Segment SGI59741[] = {{&S16722[30], 6}, {NULL}};
const Segment SGI59742[] = {{&S16722[110], 3}, {NULL}};
const Segment SGI59743[] = {{&S16722[76], 5}, {NULL}};
const Segment SGI59744[] = {{&S16724[44], 12}, {NULL}};
const Segment SGI59745[] = {{&S16724[44], 24}, {NULL}};
const Segment SGI59746[] = {{&S16724[30], 6}, {NULL}};
const Segment SGI59747[] = {{&S16724[116], 3}, {NULL}};
const Segment SGI59748[] = {{&S16724[82], 5}, {NULL}};
const Segment SGI59749[] = {{&S16727[44], 11}, {NULL}};
const Segment SGI59750[] = {{&S16727[44], 23}, {NULL}};
const Segment SGI59751[] = {{&S16727[30], 6}, {NULL}};
const Segment SGI59752[] = {{&S16727[115], 3}, {NULL}};
const Segment SGI59753[] = {{&S16727[81], 5}, {NULL}};
const Segment SGI59754[] = {{&S16730[44], 21}, {NULL}};
const Segment SGI59755[] = {{&S16730[44], 33}, {NULL}};
const Segment SGI59756[] = {{&S16730[30], 6}, {&S16730[52], 25}, {NULL}};
const Segment SGI59757[] = {{&S16730[125], 3}, {NULL}};
const Segment SGI59758[] = {{&S16730[91], 5}, {NULL}};
const Segment SGI59759[] = {{&S16731[44], 4}, {NULL}};
const Segment SGI59760[] = {{&S16731[44], 16}, {NULL}};
const Segment SGI59761[] = {{&S16731[30], 6}, {NULL}};
const Segment SGI59762[] = {{&S16731[108], 3}, {NULL}};
const Segment SGI59763[] = {{&S16731[74], 5}, {NULL}};
const Segment SGI59764[] = {{&S16732[44], 11}, {NULL}};
const Segment SGI59765[] = {{&S16732[44], 23}, {NULL}};
const Segment SGI59766[] = {{&S16732[30], 6}, {NULL}};
const Segment SGI59767[] = {{&S16732[115], 3}, {NULL}};
const Segment SGI59768[] = {{&S16732[81], 5}, {NULL}};
const Segment SGI59769[] = {{&S16735[44], 7}, {NULL}};
const Segment SGI59770[] = {{&S16735[44], 19}, {NULL}};
const Segment SGI59771[] = {{&S16735[30], 6}, {NULL}};
const Segment SGI59772[] = {{&S16735[111], 3}, {NULL}};
const Segment SGI59773[] = {{&S16735[77], 5}, {NULL}};
const Segment SGI59774[] = {{&S16736[38], 7}, {NULL}};
const Segment SGI59775[] = {{&S16736[31], 6}, {NULL}};
const Segment SGI59776[] = {{&S16738[44], 13}, {NULL}};
const Segment SGI59777[] = {{&S16738[44], 34}, {NULL}};
const Segment SGI59778[] = {{&S16738[30], 6}, {NULL}};
const Segment SGI59779[] = {{&S16738[126], 3}, {NULL}};
const Segment SGI59780[] = {{&S16738[92], 5}, {NULL}};
const Segment SGI59781[] = {{&S16739[44], 15}, {NULL}};
const Segment SGI59782[] = {{&S16739[44], 42}, {NULL}};
const Segment SGI59783[] = {{&S16739[30], 6}, {NULL}};
const Segment SGI59784[] = {{&S16739[134], 3}, {NULL}};
const Segment SGI59785[] = {{&S16739[100], 5}, {NULL}};
const Segment SGI59786[] = {{&S16741[44], 7}, {NULL}};
const Segment SGI59787[] = {{&S16741[44], 35}, {NULL}};
const Segment SGI59788[] = {{&S16741[30], 6}, {NULL}};
const Segment SGI59789[] = {{&S16741[127], 3}, {NULL}};
const Segment SGI59790[] = {{&S16741[93], 5}, {NULL}};
const Segment SGI59791[] = {{&S16744[44], 19}, {NULL}};
const Segment SGI59792[] = {{&S16744[44], 31}, {NULL}};
const Segment SGI59793[] = {{&S16744[30], 6}, {NULL}};
const Segment SGI59794[] = {{&S16744[123], 3}, {NULL}};
const Segment SGI59795[] = {{&S16744[89], 5}, {NULL}};
const Segment SGI59796[] = {{&S16745[44], 8}, {NULL}};
const Segment SGI59797[] = {{&S16745[44], 20}, {NULL}};
const Segment SGI59798[] = {{&S16745[30], 6}, {NULL}};
const Segment SGI59799[] = {{&S16745[112], 3}, {NULL}};
const Segment SGI59800[] = {{&S16745[78], 5}, {NULL}};
const Segment SGI59801[] = {{&S16747[44], 7}, {NULL}};
const Segment SGI59802[] = {{&S16747[44], 19}, {NULL}};
const Segment SGI59803[] = {{&S16747[30], 6}, {NULL}};
const Segment SGI59804[] = {{&S16747[111], 3}, {NULL}};
const Segment SGI59805[] = {{&S16747[77], 5}, {NULL}};
const Segment SGI59806[] = {{&S16750[44], 7}, {NULL}};
const Segment SGI59807[] = {{&S16750[44], 28}, {NULL}};
const Segment SGI59808[] = {{&S16750[30], 6}, {NULL}};
const Segment SGI59809[] = {{&S16750[69], 3}, {NULL}};
const Segment SGI59810[] = {{&S16750[86], 5}, {NULL}};
const Segment SGI59811[] = {{&S16756[44], 15}, {NULL}};
const Segment SGI59812[] = {{&S16756[44], 27}, {NULL}};
const Segment SGI59813[] = {{&S16756[30], 6}, {NULL}};
const Segment SGI59814[] = {{&S16756[119], 3}, {NULL}};
const Segment SGI59815[] = {{&S16756[85], 5}, {NULL}};
const Segment SGI59816[] = {{&S16758[44], 15}, {NULL}};
const Segment SGI59817[] = {{&S16758[44], 27}, {NULL}};
const Segment SGI59818[] = {{&S16758[30], 6}, {NULL}};
const Segment SGI59819[] = {{&S16758[119], 3}, {NULL}};
const Segment SGI59820[] = {{&S16758[85], 5}, {NULL}};
const Segment SGI59821[] = {{&S16759[44], 7}, {NULL}};
const Segment SGI59822[] = {{&S16759[44], 19}, {NULL}};
const Segment SGI59823[] = {{&S16759[30], 6}, {NULL}};
const Segment SGI59824[] = {{&S16759[111], 3}, {NULL}};
const Segment SGI59825[] = {{&S16759[77], 5}, {NULL}};
const Segment SGI59826[] = {{&S16760[44], 6}, {NULL}};
const Segment SGI59827[] = {{&S16760[44], 18}, {NULL}};
const Segment SGI59828[] = {{&S16760[30], 6}, {NULL}};
const Segment SGI59829[] = {{&S16760[110], 3}, {NULL}};
const Segment SGI59830[] = {{&S16760[76], 5}, {NULL}};
const Segment SGI59831[] = {{&S16762[44], 7}, {NULL}};
const Segment SGI59832[] = {{&S16762[44], 19}, {NULL}};
const Segment SGI59833[] = {{&S16762[30], 6}, {NULL}};
const Segment SGI59834[] = {{&S16762[111], 3}, {NULL}};
const Segment SGI59835[] = {{&S16762[77], 5}, {NULL}};
const Segment SGI59836[] = {{&S16766[44], 9}, {NULL}};
const Segment SGI59837[] = {{&S16766[44], 21}, {NULL}};
const Segment SGI59838[] = {{&S16766[30], 6}, {NULL}};
const Segment SGI59839[] = {{&S16766[113], 3}, {NULL}};
const Segment SGI59840[] = {{&S16766[79], 5}, {NULL}};
const Segment SGI59841[] = {{&S16767[59], 9}, {NULL}};
const Segment SGI59842[] = {{&S16767[59], 21}, {NULL}};
const Segment SGI59843[] = {{&S16767[30], 20}, {NULL}};
const Segment SGI59844[] = {{&S16767[128], 3}, {NULL}};
const Segment SGI59845[] = {{&S16767[94], 5}, {NULL}};
const Segment SGI59846[] = {{&S16771[44], 7}, {NULL}};
const Segment SGI59847[] = {{&S16771[44], 19}, {NULL}};
const Segment SGI59848[] = {{&S16771[30], 6}, {NULL}};
const Segment SGI59849[] = {{&S16771[111], 3}, {NULL}};
const Segment SGI59850[] = {{&S16771[77], 5}, {NULL}};
const Segment SGI59851[] = {{&S16772[44], 17}, {NULL}};
const Segment SGI59852[] = {{&S16772[44], 34}, {NULL}};
const Segment SGI59853[] = {{&S16772[30], 6}, {NULL}};
const Segment SGI59854[] = {{&S16772[126], 3}, {NULL}};
const Segment SGI59855[] = {{&S16772[92], 5}, {NULL}};
const Segment SGI59856[] = {{&S16775[44], 7}, {NULL}};
const Segment SGI59857[] = {{&S16775[44], 19}, {NULL}};
const Segment SGI59858[] = {{&S16775[30], 6}, {NULL}};
const Segment SGI59859[] = {{&S16775[111], 3}, {NULL}};
const Segment SGI59860[] = {{&S16775[77], 5}, {NULL}};
const Segment SGI59861[] = {{&S16779[44], 11}, {NULL}};
const Segment SGI59862[] = {{&S16779[44], 25}, {NULL}};
const Segment SGI59863[] = {{&S16779[30], 6}, {NULL}};
const Segment SGI59864[] = {{&S16779[117], 3}, {NULL}};
const Segment SGI59865[] = {{&S16779[83], 5}, {NULL}};
const Segment SGI59866[] = {{&S16780[44], 5}, {NULL}};
const Segment SGI59867[] = {{&S16780[44], 26}, {NULL}};
const Segment SGI59868[] = {{&S16780[30], 6}, {NULL}};
const Segment SGI59869[] = {{&S16780[118], 3}, {NULL}};
const Segment SGI59870[] = {{&S16780[84], 5}, {NULL}};
const Segment SGI59871[] = {{&S16781[44], 6}, {NULL}};
const Segment SGI59872[] = {{&S16781[44], 18}, {NULL}};
const Segment SGI59873[] = {{&S16781[30], 6}, {NULL}};
const Segment SGI59874[] = {{&S16781[76], 5}, {NULL}};
const Segment SGI59875[] = {{&S16783[44], 10}, {NULL}};
const Segment SGI59876[] = {{&S16783[44], 22}, {NULL}};
const Segment SGI59877[] = {{&S16783[30], 6}, {NULL}};
const Segment SGI59878[] = {{&S16783[114], 3}, {NULL}};
const Segment SGI59879[] = {{&S16783[80], 5}, {NULL}};
const Segment SGI59880[] = {{&S9923[38], 16}, {NULL}};
const Segment SGI59881[] = {{&S9923[31], 6}, {NULL}};
const Segment SGI59882[] = {{&S16784[44], 4}, {NULL}};
const Segment SGI59883[] = {{&S16784[44], 25}, {NULL}};
const Segment SGI59884[] = {{&S16784[30], 6}, {NULL}};
const Segment SGI59885[] = {{&S16784[117], 3}, {NULL}};
const Segment SGI59886[] = {{&S16784[83], 5}, {NULL}};
const Segment SGI59887[] = {{&S9927[44], 22}, {NULL}};
const Segment SGI59888[] = {{&S9927[44], 38}, {NULL}};
const Segment SGI59889[] = {{&S9927[30], 6}, {NULL}};
const Segment SGI59890[] = {{&S9927[130], 3}, {NULL}};
const Segment SGI59891[] = {{&S9927[96], 5}, {NULL}};
const Segment SGI59892[] = {{&S16788[44], 6}, {NULL}};
const Segment SGI59893[] = {{&S16788[44], 18}, {NULL}};
const Segment SGI59894[] = {{&S16788[30], 6}, {NULL}};
const Segment SGI59895[] = {{&S16788[110], 3}, {NULL}};
const Segment SGI59896[] = {{&S16788[76], 5}, {NULL}};
const Segment SGI59897[] = {{&S16794[44], 9}, {NULL}};
const Segment SGI59898[] = {{&S16794[44], 21}, {NULL}};
const Segment SGI59899[] = {{&S16794[30], 6}, {NULL}};
const Segment SGI59900[] = {{&S16794[113], 3}, {NULL}};
const Segment SGI59901[] = {{&S16794[79], 5}, {NULL}};
const Segment SGI59902[] = {{&S16796[44], 8}, {NULL}};
const Segment SGI59903[] = {{&S16796[44], 20}, {NULL}};
const Segment SGI59904[] = {{&S16796[30], 6}, {NULL}};
const Segment SGI59905[] = {{&S16796[112], 3}, {NULL}};
const Segment SGI59906[] = {{&S16796[78], 5}, {NULL}};
const Segment SGI59907[] = {{&S16800[44], 18}, {NULL}};
const Segment SGI59908[] = {{&S16800[44], 39}, {NULL}};
const Segment SGI59909[] = {{&S16800[30], 6}, {NULL}};
const Segment SGI59910[] = {{&S16800[131], 3}, {NULL}};
const Segment SGI59911[] = {{&S16800[97], 5}, {NULL}};
const Segment SGI59912[] = {{&S16801[44], 10}, {NULL}};
const Segment SGI59913[] = {{&S16801[44], 22}, {NULL}};
const Segment SGI59914[] = {{&S16801[30], 6}, {NULL}};
const Segment SGI59915[] = {{&S16801[114], 3}, {NULL}};
const Segment SGI59916[] = {{&S16801[80], 5}, {NULL}};
const Segment SGI59917[] = {{&S16802[44], 8}, {NULL}};
const Segment SGI59918[] = {{&S16802[44], 20}, {NULL}};
const Segment SGI59919[] = {{&S16802[30], 6}, {NULL}};
const Segment SGI59920[] = {{&S16802[112], 3}, {NULL}};
const Segment SGI59921[] = {{&S16802[78], 5}, {NULL}};
const Segment SGI59922[] = {{&S16805[44], 12}, {NULL}};
const Segment SGI59923[] = {{&S16805[44], 24}, {NULL}};
const Segment SGI59924[] = {{&S16805[30], 6}, {NULL}};
const Segment SGI59925[] = {{&S16805[116], 3}, {NULL}};
const Segment SGI59926[] = {{&S16805[82], 5}, {NULL}};
const Segment SGI59927[] = {{&S16806[44], 5}, {NULL}};
const Segment SGI59928[] = {{&S16806[44], 17}, {NULL}};
const Segment SGI59929[] = {{&S16806[30], 6}, {NULL}};
const Segment SGI59930[] = {{&S16806[109], 3}, {NULL}};
const Segment SGI59931[] = {{&S16806[75], 5}, {NULL}};
const Segment SGI59932[] = {{&S16807[44], 19}, {NULL}};
const Segment SGI59933[] = {{&S16807[44], 31}, {NULL}};
const Segment SGI59934[] = {{&S16807[30], 6}, {NULL}};
const Segment SGI59935[] = {{&S16807[123], 3}, {NULL}};
const Segment SGI59936[] = {{&S16807[89], 5}, {NULL}};
const Segment SGI59937[] = {{&S16809[44], 10}, {NULL}};
const Segment SGI59938[] = {{&S16809[44], 22}, {NULL}};
const Segment SGI59939[] = {{&S16809[30], 6}, {NULL}};
const Segment SGI59940[] = {{&S16809[114], 3}, {NULL}};
const Segment SGI59941[] = {{&S16809[80], 5}, {NULL}};
const Segment SGI59942[] = {{&S16810[44], 6}, {NULL}};
const Segment SGI59943[] = {{&S16810[44], 18}, {NULL}};
const Segment SGI59944[] = {{&S16810[30], 6}, {NULL}};
const Segment SGI59945[] = {{&S16810[110], 3}, {NULL}};
const Segment SGI59946[] = {{&S16810[76], 5}, {NULL}};
const Segment SGI59947[] = {{&S16813[44], 9}, {NULL}};
const Segment SGI59948[] = {{&S16813[44], 21}, {NULL}};
const Segment SGI59949[] = {{&S16813[30], 6}, {NULL}};
const Segment SGI59950[] = {{&S16813[113], 3}, {NULL}};
const Segment SGI59951[] = {{&S16813[79], 5}, {NULL}};
const Segment SGI59952[] = {{&S16818[44], 5}, {NULL}};
const Segment SGI59953[] = {{&S16818[44], 17}, {NULL}};
const Segment SGI59954[] = {{&S16818[30], 6}, {NULL}};
const Segment SGI59955[] = {{&S16818[109], 3}, {NULL}};
const Segment SGI59956[] = {{&S16818[75], 5}, {NULL}};
const Segment SGI59957[] = {{&S16821[44], 17}, {NULL}};
const Segment SGI59958[] = {{&S16821[44], 29}, {NULL}};
const Segment SGI59959[] = {{&S16821[30], 6}, {NULL}};
const Segment SGI59960[] = {{&S16821[121], 3}, {NULL}};
const Segment SGI59961[] = {{&S16821[87], 5}, {NULL}};
const Segment SGI59962[] = {{&S16826[44], 4}, {NULL}};
const Segment SGI59963[] = {{&S16826[44], 12}, {NULL}};
const Segment SGI59964[] = {{&S16826[30], 6}, {NULL}};
const Segment SGI59965[] = {{&S16826[104], 3}, {NULL}};
const Segment SGI59966[] = {{&S16826[70], 5}, {NULL}};
const Segment SGI59967[] = {{&S16827[44], 8}, {NULL}};
const Segment SGI59968[] = {{&S16827[44], 20}, {NULL}};
const Segment SGI59969[] = {{&S16827[30], 6}, {NULL}};
const Segment SGI59970[] = {{&S16827[112], 3}, {NULL}};
const Segment SGI59971[] = {{&S16827[78], 5}, {NULL}};
const Segment SGI59972[] = {{&S16830[59], 9}, {NULL}};
const Segment SGI59973[] = {{&S16830[59], 21}, {NULL}};
const Segment SGI59974[] = {{&S16830[30], 20}, {NULL}};
const Segment SGI59975[] = {{&S16830[128], 3}, {NULL}};
const Segment SGI59976[] = {{&S16830[94], 5}, {NULL}};
const Segment SGI59977[] = {{&S16831[44], 8}, {NULL}};
const Segment SGI59978[] = {{&S16831[44], 20}, {NULL}};
const Segment SGI59979[] = {{&S16831[30], 6}, {NULL}};
const Segment SGI59980[] = {{&S16831[112], 3}, {NULL}};
const Segment SGI59981[] = {{&S16831[78], 5}, {NULL}};
const Segment SGI59982[] = {{&S16832[44], 19}, {NULL}};
const Segment SGI59983[] = {{&S16832[44], 30}, {NULL}};
const Segment SGI59984[] = {{&S16832[30], 6}, {NULL}};
const Segment SGI59985[] = {{&S16832[122], 3}, {NULL}};
const Segment SGI59986[] = {{&S16832[88], 5}, {NULL}};
const Segment SGI59987[] = {{&S16835[44], 15}, {NULL}};
const Segment SGI59988[] = {{&S16835[44], 42}, {NULL}};
const Segment SGI59989[] = {{&S16835[30], 6}, {NULL}};
const Segment SGI59990[] = {{&S16835[134], 3}, {NULL}};
const Segment SGI59991[] = {{&S16835[100], 5}, {NULL}};
const Segment SGI59992[] = {{&S16837[44], 25}, {NULL}};
const Segment SGI59993[] = {{&S16837[44], 37}, {NULL}};
const Segment SGI59994[] = {{&S16837[30], 6}, {NULL}};
const Segment SGI59995[] = {{&S16837[129], 3}, {NULL}};
const Segment SGI59996[] = {{&S16837[95], 5}, {NULL}};
const Segment SGI59997[] = {{&S16838[44], 6}, {NULL}};
const Segment SGI59998[] = {{&S16838[44], 18}, {NULL}};
const Segment SGI59999[] = {{&S16838[30], 6}, {NULL}};
const Segment SGI60000[] = {{&S16838[110], 3}, {NULL}};
const Segment SGI60001[] = {{&S16838[76], 5}, {NULL}};
const Segment SGI60002[] = {{&S16846[44], 7}, {NULL}};
const Segment SGI60003[] = {{&S16846[44], 19}, {NULL}};
const Segment SGI60004[] = {{&S16846[30], 6}, {NULL}};
const Segment SGI60005[] = {{&S16846[111], 3}, {NULL}};
const Segment SGI60006[] = {{&S16846[77], 5}, {NULL}};
const Segment SGI60007[] = {{&S16847[44], 5}, {NULL}};
const Segment SGI60008[] = {{&S16847[44], 26}, {NULL}};
const Segment SGI60009[] = {{&S16847[30], 6}, {NULL}};
const Segment SGI60010[] = {{&S16847[118], 3}, {NULL}};
const Segment SGI60011[] = {{&S16847[84], 5}, {NULL}};
const Segment SGI60012[] = {{&S16848[44], 18}, {NULL}};
const Segment SGI60013[] = {{&S16848[44], 30}, {NULL}};
const Segment SGI60014[] = {{&S16848[30], 6}, {NULL}};
const Segment SGI60015[] = {{&S16848[122], 3}, {NULL}};
const Segment SGI60016[] = {{&S16848[88], 5}, {NULL}};
const Segment SGI60017[] = {{&S16850[44], 14}, {NULL}};
const Segment SGI60018[] = {{&S16850[44], 26}, {NULL}};
const Segment SGI60019[] = {{&S16850[30], 6}, {NULL}};
const Segment SGI60020[] = {{&S16850[118], 3}, {NULL}};
const Segment SGI60021[] = {{&S16850[84], 5}, {NULL}};
const Segment SGI60022[] = {{&S16852[44], 8}, {NULL}};
const Segment SGI60023[] = {{&S16852[44], 20}, {NULL}};
const Segment SGI60024[] = {{&S16852[30], 6}, {NULL}};
const Segment SGI60025[] = {{&S16852[112], 3}, {NULL}};
const Segment SGI60026[] = {{&S16852[78], 5}, {NULL}};
const Segment SGI60027[] = {{&S16853[44], 11}, {NULL}};
const Segment SGI60028[] = {{&S16853[44], 23}, {NULL}};
const Segment SGI60029[] = {{&S16853[30], 6}, {NULL}};
const Segment SGI60030[] = {{&S16853[115], 3}, {NULL}};
const Segment SGI60031[] = {{&S16853[81], 5}, {NULL}};
const Segment SGI60032[] = {{&S16854[44], 11}, {NULL}};
const Segment SGI60033[] = {{&S16854[44], 23}, {NULL}};
const Segment SGI60034[] = {{&S16854[30], 6}, {NULL}};
const Segment SGI60035[] = {{&S16854[115], 3}, {NULL}};
const Segment SGI60036[] = {{&S16854[81], 5}, {NULL}};
const Segment SGI60037[] = {{&S16858[44], 12}, {NULL}};
const Segment SGI60038[] = {{&S16858[44], 24}, {NULL}};
const Segment SGI60039[] = {{&S16858[30], 6}, {NULL}};
const Segment SGI60040[] = {{&S16858[116], 3}, {NULL}};
const Segment SGI60041[] = {{&S16858[82], 5}, {NULL}};
const Segment SGI60042[] = {{&S16862[44], 15}, {NULL}};
const Segment SGI60043[] = {{&S16862[44], 27}, {NULL}};
const Segment SGI60044[] = {{&S16862[30], 6}, {NULL}};
const Segment SGI60045[] = {{&S16862[119], 3}, {NULL}};
const Segment SGI60046[] = {{&S16862[85], 5}, {NULL}};
const Segment SGI60047[] = {{&S16863[44], 5}, {NULL}};
const Segment SGI60048[] = {{&S16863[44], 17}, {NULL}};
const Segment SGI60049[] = {{&S16863[30], 6}, {NULL}};
const Segment SGI60050[] = {{&S16863[109], 3}, {NULL}};
const Segment SGI60051[] = {{&S16863[75], 5}, {NULL}};
const Segment SGI60052[] = {{&S16868[44], 6}, {NULL}};
const Segment SGI60053[] = {{&S16868[44], 18}, {NULL}};
const Segment SGI60054[] = {{&S16868[30], 6}, {NULL}};
const Segment SGI60055[] = {{&S16868[110], 3}, {NULL}};
const Segment SGI60056[] = {{&S16868[76], 5}, {NULL}};
const Segment SGI60057[] = {{&S16876[44], 15}, {NULL}};
const Segment SGI60058[] = {{&S16876[44], 27}, {NULL}};
const Segment SGI60059[] = {{&S16876[30], 6}, {NULL}};
const Segment SGI60060[] = {{&S16876[119], 3}, {NULL}};
const Segment SGI60061[] = {{&S16876[85], 5}, {NULL}};
const Segment SGI60062[] = {{&S16880[44], 13}, {NULL}};
const Segment SGI60063[] = {{&S16880[44], 32}, {NULL}};
const Segment SGI60064[] = {{&S16880[30], 6}, {NULL}};
const Segment SGI60065[] = {{&S16880[124], 3}, {NULL}};
const Segment SGI60066[] = {{&S16880[90], 5}, {NULL}};
const Segment SGI60067[] = {{&S16884[44], 11}, {NULL}};
const Segment SGI60068[] = {{&S16884[44], 32}, {NULL}};
const Segment SGI60069[] = {{&S16884[30], 6}, {NULL}};
const Segment SGI60070[] = {{&S16884[93], 4}, {NULL}};
const Segment SGI60071[] = {{&S16884[93], 3}, {NULL}};
const Segment SGI60072[] = {{&S16889[44], 11}, {NULL}};
const Segment SGI60073[] = {{&S16889[44], 38}, {NULL}};
const Segment SGI60074[] = {{&S16889[30], 6}, {NULL}};
const Segment SGI60075[] = {{&S16889[130], 3}, {NULL}};
const Segment SGI60076[] = {{&S16889[96], 5}, {NULL}};
const Segment SGI60077[] = {{&S16890[44], 10}, {NULL}};
const Segment SGI60078[] = {{&S16890[44], 23}, {NULL}};
const Segment SGI60079[] = {{&S16890[30], 6}, {NULL}};
const Segment SGI60080[] = {{&S16890[115], 3}, {NULL}};
const Segment SGI60081[] = {{&S16890[81], 5}, {NULL}};
const Segment SGI60082[] = {{&S16891[44], 6}, {NULL}};
const Segment SGI60083[] = {{&S16891[44], 22}, {NULL}};
const Segment SGI60084[] = {{&S16891[30], 6}, {NULL}};
const Segment SGI60085[] = {{&S16891[114], 3}, {NULL}};
const Segment SGI60086[] = {{&S16891[80], 5}, {NULL}};
const Segment SGI60087[] = {{&S16902[44], 6}, {NULL}};
const Segment SGI60088[] = {{&S16902[44], 18}, {NULL}};
const Segment SGI60089[] = {{&S16902[30], 6}, {NULL}};
const Segment SGI60090[] = {{&S16902[110], 3}, {NULL}};
const Segment SGI60091[] = {{&S16902[76], 5}, {NULL}};
const Segment SGI60092[] = {{&S16908[44], 5}, {NULL}};
const Segment SGI60093[] = {{&S16908[44], 17}, {NULL}};
const Segment SGI60094[] = {{&S16908[30], 6}, {NULL}};
const Segment SGI60095[] = {{&S16908[109], 3}, {NULL}};
const Segment SGI60096[] = {{&S16908[75], 5}, {NULL}};
const Segment SGI60097[] = {{&S16909[44], 17}, {NULL}};
const Segment SGI60098[] = {{&S16909[44], 29}, {NULL}};
const Segment SGI60099[] = {{&S16909[30], 6}, {NULL}};
const Segment SGI60100[] = {{&S16909[90], 6}, {NULL}};
const Segment SGI60101[] = {{&S16910[44], 21}, {NULL}};
const Segment SGI60102[] = {{&S16910[44], 33}, {NULL}};
const Segment SGI60103[] = {{&S16910[30], 6}, {&S16910[52], 25}, {NULL}};
const Segment SGI60104[] = {{&S16910[94], 4}, {NULL}};
const Segment SGI60105[] = {{&S16910[94], 3}, {NULL}};
const Segment SGI60106[] = {{&S16913[44], 7}, {NULL}};
const Segment SGI60107[] = {{&S16913[44], 28}, {NULL}};
const Segment SGI60108[] = {{&S16913[30], 6}, {NULL}};
const Segment SGI60109[] = {{&S16913[89], 6}, {NULL}};
const Segment SGI60110[] = {{&S16916[44], 6}, {NULL}};
const Segment SGI60111[] = {{&S16916[44], 18}, {NULL}};
const Segment SGI60112[] = {{&S16916[30], 6}, {NULL}};
const Segment SGI60113[] = {{&S16916[110], 3}, {NULL}};
const Segment SGI60114[] = {{&S16916[76], 5}, {NULL}};
const Segment SGI60115[] = {{&S16920[44], 15}, {NULL}};
const Segment SGI60116[] = {{&S16920[44], 27}, {NULL}};
const Segment SGI60117[] = {{&S16920[30], 6}, {NULL}};
const Segment SGI60118[] = {{&S16920[88], 4}, {NULL}};
const Segment SGI60119[] = {{&S16920[88], 3}, {NULL}};
const Segment SGI60120[] = {{&S16921[44], 7}, {NULL}};
const Segment SGI60121[] = {{&S16921[44], 19}, {NULL}};
const Segment SGI60122[] = {{&S16921[30], 6}, {NULL}};
const Segment SGI60123[] = {{&S16921[111], 3}, {NULL}};
const Segment SGI60124[] = {{&S16921[77], 5}, {NULL}};
const Segment SGI60125[] = {{&S16923[44], 6}, {NULL}};
const Segment SGI60126[] = {{&S16923[44], 18}, {NULL}};
const Segment SGI60127[] = {{&S16923[30], 6}, {NULL}};
const Segment SGI60128[] = {{&S16923[110], 3}, {NULL}};
const Segment SGI60129[] = {{&S16923[76], 5}, {NULL}};
const Segment SGI60130[] = {{&S16924[44], 10}, {NULL}};
const Segment SGI60131[] = {{&S16924[44], 22}, {NULL}};
const Segment SGI60132[] = {{&S16924[30], 6}, {NULL}};
const Segment SGI60133[] = {{&S16924[114], 3}, {NULL}};
const Segment SGI60134[] = {{&S16924[80], 5}, {NULL}};
const Segment SGI60135[] = {{&S16927[44], 14}, {NULL}};
const Segment SGI60136[] = {{&S16927[44], 26}, {NULL}};
const Segment SGI60137[] = {{&S16927[30], 6}, {NULL}};
const Segment SGI60138[] = {{&S16927[118], 3}, {NULL}};
const Segment SGI60139[] = {{&S16927[84], 5}, {NULL}};
const Segment SGI60140[] = {{&S16928[44], 10}, {NULL}};
const Segment SGI60141[] = {{&S16928[44], 22}, {NULL}};
const Segment SGI60142[] = {{&S16928[30], 6}, {NULL}};
const Segment SGI60143[] = {{&S16928[114], 3}, {NULL}};
const Segment SGI60144[] = {{&S16928[80], 5}, {NULL}};
const Segment SGI60145[] = {{&S16930[44], 8}, {NULL}};
const Segment SGI60146[] = {{&S16930[44], 24}, {NULL}};
const Segment SGI60147[] = {{&S16930[30], 6}, {NULL}};
const Segment SGI60148[] = {{&S16930[116], 3}, {NULL}};
const Segment SGI60149[] = {{&S16930[82], 5}, {NULL}};
const Segment SGI60150[] = {{&S16932[44], 9}, {NULL}};
const Segment SGI60151[] = {{&S16932[44], 21}, {NULL}};
const Segment SGI60152[] = {{&S16932[30], 6}, {NULL}};
const Segment SGI60153[] = {{&S16932[113], 3}, {NULL}};
const Segment SGI60154[] = {{&S16932[79], 5}, {NULL}};
const Segment SGI60155[] = {{&S16933[44], 17}, {NULL}};
const Segment SGI60156[] = {{&S16933[44], 29}, {NULL}};
const Segment SGI60157[] = {{&S16933[30], 6}, {NULL}};
const Segment SGI60158[] = {{&S16933[121], 3}, {NULL}};
const Segment SGI60159[] = {{&S16933[87], 5}, {NULL}};
const Segment SGI60160[] = {{&S16934[44], 14}, {NULL}};
const Segment SGI60161[] = {{&S16934[44], 26}, {NULL}};
const Segment SGI60162[] = {{&S16934[30], 6}, {NULL}};
const Segment SGI60163[] = {{&S16934[118], 3}, {NULL}};
const Segment SGI60164[] = {{&S16934[84], 5}, {NULL}};
const Segment SGI60165[] = {{&S16939[44], 4}, {NULL}};
const Segment SGI60166[] = {{&S16939[44], 16}, {NULL}};
const Segment SGI60167[] = {{&S16939[30], 6}, {NULL}};
const Segment SGI60168[] = {{&S16939[108], 3}, {NULL}};
const Segment SGI60169[] = {{&S16939[74], 5}, {NULL}};
const Segment SGI60170[] = {{&S16942[44], 14}, {NULL}};
const Segment SGI60171[] = {{&S16942[44], 26}, {NULL}};
const Segment SGI60172[] = {{&S16942[30], 6}, {NULL}};
const Segment SGI60173[] = {{&S16942[118], 3}, {NULL}};
const Segment SGI60174[] = {{&S16942[84], 5}, {NULL}};
const Segment SGI60175[] = {{&S16944[44], 11}, {NULL}};
const Segment SGI60176[] = {{&S16944[44], 23}, {NULL}};
const Segment SGI60177[] = {{&S16944[30], 6}, {NULL}};
const Segment SGI60178[] = {{&S16944[115], 3}, {NULL}};
const Segment SGI60179[] = {{&S16944[81], 5}, {NULL}};
const Segment SGI60180[] = {{&S16945[44], 8}, {NULL}};
const Segment SGI60181[] = {{&S16945[44], 20}, {NULL}};
const Segment SGI60182[] = {{&S16945[30], 6}, {NULL}};
const Segment SGI60183[] = {{&S16945[112], 3}, {NULL}};
const Segment SGI60184[] = {{&S16945[78], 5}, {NULL}};
const Segment SGI60185[] = {{&S16946[44], 8}, {NULL}};
const Segment SGI60186[] = {{&S16946[44], 20}, {NULL}};
const Segment SGI60187[] = {{&S16946[30], 6}, {NULL}};
const Segment SGI60188[] = {{&S16946[112], 3}, {NULL}};
const Segment SGI60189[] = {{&S16946[78], 5}, {NULL}};
const Segment SGI60190[] = {{&S16947[44], 7}, {NULL}};
const Segment SGI60191[] = {{&S16947[44], 19}, {NULL}};
const Segment SGI60192[] = {{&S16947[30], 6}, {NULL}};
const Segment SGI60193[] = {{&S16947[111], 3}, {NULL}};
const Segment SGI60194[] = {{&S16947[77], 5}, {NULL}};
const Segment SGI60195[] = {{&S16949[44], 16}, {NULL}};
const Segment SGI60196[] = {{&S16949[44], 28}, {NULL}};
const Segment SGI60197[] = {{&S16949[30], 6}, {NULL}};
const Segment SGI60198[] = {{&S16949[120], 3}, {NULL}};
const Segment SGI60199[] = {{&S16949[86], 5}, {NULL}};
const Segment SGI60200[] = {{&S16951[44], 10}, {NULL}};
const Segment SGI60201[] = {{&S16951[44], 27}, {NULL}};
const Segment SGI60202[] = {{&S16951[30], 6}, {NULL}};
const Segment SGI60203[] = {{&S16951[119], 3}, {NULL}};
const Segment SGI60204[] = {{&S16951[85], 5}, {NULL}};
const Segment SGI60205[] = {{&S16953[44], 8}, {NULL}};
const Segment SGI60206[] = {{&S16953[44], 20}, {NULL}};
const Segment SGI60207[] = {{&S16953[30], 6}, {NULL}};
const Segment SGI60208[] = {{&S16953[112], 3}, {NULL}};
const Segment SGI60209[] = {{&S16953[78], 5}, {NULL}};
const Segment SGI60210[] = {{&S16954[44], 7}, {NULL}};
const Segment SGI60211[] = {{&S16954[44], 19}, {NULL}};
const Segment SGI60212[] = {{&S16954[30], 6}, {NULL}};
const Segment SGI60213[] = {{&S16954[80], 4}, {NULL}};
const Segment SGI60214[] = {{&S16954[80], 3}, {NULL}};
const Segment SGI60215[] = {{&S16955[44], 20}, {NULL}};
const Segment SGI60216[] = {{&S16955[44], 32}, {NULL}};
const Segment SGI60217[] = {{&S16955[30], 6}, {&S16955[52], 24}, {NULL}};
const Segment SGI60218[] = {{&S16955[124], 3}, {NULL}};
const Segment SGI60219[] = {{&S16955[90], 5}, {NULL}};
const Segment SGI60220[] = {{&S16956[44], 6}, {NULL}};
const Segment SGI60221[] = {{&S16956[44], 18}, {NULL}};
const Segment SGI60222[] = {{&S16956[30], 6}, {NULL}};
const Segment SGI60223[] = {{&S16956[110], 3}, {NULL}};
const Segment SGI60224[] = {{&S16956[76], 5}, {NULL}};
const Segment SGI60225[] = {{&S16957[44], 6}, {NULL}};
const Segment SGI60226[] = {{&S16957[44], 18}, {NULL}};
const Segment SGI60227[] = {{&S16957[30], 6}, {NULL}};
const Segment SGI60228[] = {{&S16957[110], 3}, {NULL}};
const Segment SGI60229[] = {{&S16957[76], 5}, {NULL}};
const Segment SGI60230[] = {{&S16959[44], 6}, {NULL}};
const Segment SGI60231[] = {{&S16959[44], 18}, {NULL}};
const Segment SGI60232[] = {{&S16959[30], 6}, {NULL}};
const Segment SGI60233[] = {{&S16959[110], 3}, {NULL}};
const Segment SGI60234[] = {{&S16959[76], 5}, {NULL}};
const Segment SGI60235[] = {{&S16960[44], 11}, {NULL}};
const Segment SGI60236[] = {{&S16960[44], 23}, {NULL}};
const Segment SGI60237[] = {{&S16960[30], 6}, {NULL}};
const Segment SGI60238[] = {{&S16960[115], 3}, {NULL}};
const Segment SGI60239[] = {{&S16960[81], 5}, {NULL}};
const Segment SGI60240[] = {{&S16961[44], 10}, {NULL}};
const Segment SGI60241[] = {{&S16961[44], 22}, {NULL}};
const Segment SGI60242[] = {{&S16961[30], 6}, {NULL}};
const Segment SGI60243[] = {{&S16961[83], 6}, {NULL}};
const Segment SGI60244[] = {{&S16962[44], 7}, {NULL}};
const Segment SGI60245[] = {{&S16962[44], 19}, {NULL}};
const Segment SGI60246[] = {{&S16962[30], 6}, {NULL}};
const Segment SGI60247[] = {{&S16962[111], 3}, {NULL}};
const Segment SGI60248[] = {{&S16962[77], 5}, {NULL}};
const Segment SGI60249[] = {{&S16968[38], 7}, {NULL}};
const Segment SGI60250[] = {{&S16968[31], 6}, {NULL}};
const Segment SGI60251[] = {{&S16970[44], 11}, {NULL}};
const Segment SGI60252[] = {{&S16970[44], 23}, {NULL}};
const Segment SGI60253[] = {{&S16970[30], 6}, {NULL}};
const Segment SGI60254[] = {{&S16970[115], 3}, {NULL}};
const Segment SGI60255[] = {{&S16970[81], 5}, {NULL}};
const Segment SGI60256[] = {{&S16971[44], 12}, {NULL}};
const Segment SGI60257[] = {{&S16971[44], 24}, {NULL}};
const Segment SGI60258[] = {{&S16971[30], 6}, {NULL}};
const Segment SGI60259[] = {{&S16971[116], 3}, {NULL}};
const Segment SGI60260[] = {{&S16971[82], 5}, {NULL}};
const Segment SGI60261[] = {{&S16973[44], 7}, {NULL}};
const Segment SGI60262[] = {{&S16973[44], 19}, {NULL}};
const Segment SGI60263[] = {{&S16973[30], 6}, {NULL}};
const Segment SGI60264[] = {{&S16973[111], 3}, {NULL}};
const Segment SGI60265[] = {{&S16973[77], 5}, {NULL}};
const Segment SGI60266[] = {{&S16975[44], 7}, {NULL}};
const Segment SGI60267[] = {{&S16975[44], 19}, {NULL}};
const Segment SGI60268[] = {{&S16975[30], 6}, {NULL}};
const Segment SGI60269[] = {{&S16975[111], 3}, {NULL}};
const Segment SGI60270[] = {{&S16975[77], 5}, {NULL}};
const Segment SGI60271[] = {{&S16977[44], 7}, {NULL}};
const Segment SGI60272[] = {{&S16977[44], 19}, {NULL}};
const Segment SGI60273[] = {{&S16977[30], 6}, {NULL}};
const Segment SGI60274[] = {{&S16977[111], 3}, {NULL}};
const Segment SGI60275[] = {{&S16977[77], 5}, {NULL}};
const Segment SGI60276[] = {{&S16980[44], 21}, {NULL}};
const Segment SGI60277[] = {{&S16980[44], 33}, {NULL}};
const Segment SGI60278[] = {{&S16980[30], 6}, {NULL}};
const Segment SGI60279[] = {{&S16980[125], 3}, {NULL}};
const Segment SGI60280[] = {{&S16980[91], 5}, {NULL}};
const Segment SGI60281[] = {{&S16981[44], 15}, {NULL}};
const Segment SGI60282[] = {{&S16981[44], 27}, {NULL}};
const Segment SGI60283[] = {{&S16981[30], 6}, {NULL}};
const Segment SGI60284[] = {{&S16981[119], 3}, {NULL}};
const Segment SGI60285[] = {{&S16981[85], 5}, {NULL}};
const Segment SGI60286[] = {{&S16983[44], 5}, {NULL}};
const Segment SGI60287[] = {{&S16983[44], 17}, {NULL}};
const Segment SGI60288[] = {{&S16983[30], 6}, {NULL}};
const Segment SGI60289[] = {{&S16983[109], 3}, {NULL}};
const Segment SGI60290[] = {{&S16983[75], 5}, {NULL}};
const Segment SGI60291[] = {{&S16984[44], 19}, {NULL}};
const Segment SGI60292[] = {{&S16984[44], 31}, {NULL}};
const Segment SGI60293[] = {{&S16984[30], 6}, {NULL}};
const Segment SGI60294[] = {{&S16984[123], 3}, {NULL}};
const Segment SGI60295[] = {{&S16984[89], 5}, {NULL}};
const Segment SGI60296[] = {{&S16992[44], 12}, {NULL}};
const Segment SGI60297[] = {{&S16992[44], 24}, {NULL}};
const Segment SGI60298[] = {{&S16992[30], 6}, {NULL}};
const Segment SGI60299[] = {{&S16992[116], 3}, {NULL}};
const Segment SGI60300[] = {{&S16992[82], 5}, {NULL}};
const Segment SGI60301[] = {{&S16994[44], 6}, {NULL}};
const Segment SGI60302[] = {{&S16994[44], 18}, {NULL}};
const Segment SGI60303[] = {{&S16994[30], 6}, {NULL}};
const Segment SGI60304[] = {{&S16994[110], 3}, {NULL}};
const Segment SGI60305[] = {{&S16994[76], 5}, {NULL}};
const Segment SGI60306[] = {{&S17001[44], 15}, {NULL}};
const Segment SGI60307[] = {{&S17001[44], 27}, {NULL}};
const Segment SGI60308[] = {{&S17001[30], 6}, {NULL}};
const Segment SGI60309[] = {{&S17001[119], 3}, {NULL}};
const Segment SGI60310[] = {{&S17001[85], 5}, {NULL}};
const Segment SGI60311[] = {{&S17005[44], 6}, {NULL}};
const Segment SGI60312[] = {{&S17005[44], 18}, {NULL}};
const Segment SGI60313[] = {{&S17005[30], 6}, {NULL}};
const Segment SGI60314[] = {{&S17005[110], 3}, {NULL}};
const Segment SGI60315[] = {{&S17005[76], 5}, {NULL}};
const Segment SGI60316[] = {{&S17008[44], 12}, {NULL}};
const Segment SGI60317[] = {{&S17008[44], 24}, {NULL}};
const Segment SGI60318[] = {{&S17008[30], 6}, {NULL}};
const Segment SGI60319[] = {{&S17008[116], 3}, {NULL}};
const Segment SGI60320[] = {{&S17008[82], 5}, {NULL}};
const Segment SGI60321[] = {{&S17009[44], 10}, {NULL}};
const Segment SGI60322[] = {{&S17009[44], 22}, {NULL}};
const Segment SGI60323[] = {{&S17009[30], 6}, {NULL}};
const Segment SGI60324[] = {{&S17009[114], 3}, {NULL}};
const Segment SGI60325[] = {{&S17009[80], 5}, {NULL}};
const Segment SGI60326[] = {{&S17013[44], 8}, {NULL}};
const Segment SGI60327[] = {{&S17013[44], 20}, {NULL}};
const Segment SGI60328[] = {{&S17013[30], 6}, {NULL}};
const Segment SGI60329[] = {{&S17013[112], 3}, {NULL}};
const Segment SGI60330[] = {{&S17013[78], 5}, {NULL}};
const Segment SGI60331[] = {{&S17014[44], 12}, {NULL}};
const Segment SGI60332[] = {{&S17014[44], 24}, {NULL}};
const Segment SGI60333[] = {{&S17014[30], 6}, {NULL}};
const Segment SGI60334[] = {{&S17014[116], 3}, {NULL}};
const Segment SGI60335[] = {{&S17014[82], 5}, {NULL}};
const Segment SGI60336[] = {{&S17015[44], 6}, {NULL}};
const Segment SGI60337[] = {{&S17015[44], 18}, {NULL}};
const Segment SGI60338[] = {{&S17015[30], 6}, {NULL}};
const Segment SGI60339[] = {{&S17015[110], 3}, {NULL}};
const Segment SGI60340[] = {{&S17015[76], 5}, {NULL}};
const Segment SGI60341[] = {{&S17016[38], 17}, {NULL}};
const Segment SGI60342[] = {{&S17016[31], 6}, {NULL}};
const Segment SGI60343[] = {{&S17017[44], 12}, {NULL}};
const Segment SGI60344[] = {{&S17017[44], 24}, {NULL}};
const Segment SGI60345[] = {{&S17017[30], 6}, {NULL}};
const Segment SGI60346[] = {{&S17017[116], 3}, {NULL}};
const Segment SGI60347[] = {{&S17017[82], 5}, {NULL}};
const Segment SGI60348[] = {{&S17021[44], 73}, {NULL}};
const Segment SGI60349[] = {{&S17021[44], 94}, {NULL}};
const Segment SGI60350[] = {{&S17021[30], 6}, {NULL}};
const Segment SGI60351[] = {{&S17021[82], 3}, {NULL}};
const Segment SGI60352[] = {{&S17021[111], 5}, {NULL}};
const Segment SGI60353[] = {{&S17026[44], 10}, {NULL}};
const Segment SGI60354[] = {{&S17026[44], 28}, {NULL}};
const Segment SGI60355[] = {{&S17026[30], 6}, {NULL}};
const Segment SGI60356[] = {{&S17026[120], 3}, {NULL}};
const Segment SGI60357[] = {{&S17026[86], 5}, {NULL}};
const Segment SGI60358[] = {{&S17028[44], 12}, {NULL}};
const Segment SGI60359[] = {{&S17028[44], 24}, {NULL}};
const Segment SGI60360[] = {{&S17028[30], 6}, {NULL}};
const Segment SGI60361[] = {{&S17028[116], 3}, {NULL}};
const Segment SGI60362[] = {{&S17028[82], 5}, {NULL}};
const Segment SGI60363[] = {{&S17029[44], 8}, {NULL}};
const Segment SGI60364[] = {{&S17029[44], 20}, {NULL}};
const Segment SGI60365[] = {{&S17029[30], 6}, {NULL}};
const Segment SGI60366[] = {{&S17029[112], 3}, {NULL}};
const Segment SGI60367[] = {{&S17029[78], 5}, {NULL}};
const Segment SGI60368[] = {{&S17030[44], 6}, {NULL}};
const Segment SGI60369[] = {{&S17030[44], 18}, {NULL}};
const Segment SGI60370[] = {{&S17030[30], 6}, {NULL}};
const Segment SGI60371[] = {{&S17030[110], 3}, {NULL}};
const Segment SGI60372[] = {{&S17030[76], 5}, {NULL}};
const Segment SGI60373[] = {{&S17032[44], 11}, {NULL}};
const Segment SGI60374[] = {{&S17032[44], 38}, {NULL}};
const Segment SGI60375[] = {{&S17032[30], 6}, {NULL}};
const Segment SGI60376[] = {{&S17032[130], 3}, {NULL}};
const Segment SGI60377[] = {{&S17032[96], 5}, {NULL}};
const Segment SGI60378[] = {{&S17034[44], 12}, {NULL}};
const Segment SGI60379[] = {{&S17034[44], 24}, {NULL}};
const Segment SGI60380[] = {{&S17034[30], 6}, {NULL}};
const Segment SGI60381[] = {{&S17034[116], 3}, {NULL}};
const Segment SGI60382[] = {{&S17034[82], 5}, {NULL}};
const Segment SGI60383[] = {{&S17035[44], 14}, {NULL}};
const Segment SGI60384[] = {{&S17035[44], 42}, {NULL}};
const Segment SGI60385[] = {{&S17035[30], 6}, {NULL}};
const Segment SGI60386[] = {{&S17035[134], 3}, {NULL}};
const Segment SGI60387[] = {{&S17035[100], 5}, {NULL}};
const Segment SGI60388[] = {{&S17039[44], 11}, {NULL}};
const Segment SGI60389[] = {{&S17039[44], 23}, {NULL}};
const Segment SGI60390[] = {{&S17039[30], 6}, {NULL}};
const Segment SGI60391[] = {{&S17039[114], 3}, {NULL}};
const Segment SGI60392[] = {{&S17039[81], 5}, {NULL}};
const Segment SGI60393[] = {{&S17041[42], 9}, {NULL}};
const Segment SGI60394[] = {{&S17041[30], 6}, {NULL}};
const Segment SGI60395[] = {{&S17041[111], 3}, {NULL}};
const Segment SGI60396[] = {{&S17041[77], 5}, {NULL}};
const Segment SGI60397[] = {{&S17043[44], 17}, {NULL}};
const Segment SGI60398[] = {{&S17043[44], 42}, {NULL}};
const Segment SGI60399[] = {{&S17043[30], 6}, {NULL}};
const Segment SGI60400[] = {{&S17043[134], 3}, {NULL}};
const Segment SGI60401[] = {{&S17043[100], 5}, {NULL}};
const Segment SGI60402[] = {{&S17045[44], 7}, {NULL}};
const Segment SGI60403[] = {{&S17045[44], 19}, {NULL}};
const Segment SGI60404[] = {{&S17045[30], 6}, {NULL}};
const Segment SGI60405[] = {{&S10388[44], 11}, {NULL}};
const Segment SGI60406[] = {{&S10388[44], 22}, {NULL}};
const Segment SGI60407[] = {{&S17045[111], 3}, {NULL}};
const Segment SGI60408[] = {{&S10388[30], 6}, {NULL}};
const Segment SGI60409[] = {{&S17045[77], 5}, {NULL}};
const Segment SGI60410[] = {{&S10388[114], 3}, {NULL}};
const Segment SGI60411[] = {{&S10388[80], 5}, {NULL}};
const Segment SGI60412[] = {{&S17046[44], 12}, {NULL}};
const Segment SGI60413[] = {{&S17046[44], 24}, {NULL}};
const Segment SGI60414[] = {{&S17046[30], 6}, {NULL}};
const Segment SGI60415[] = {{&S17046[116], 3}, {NULL}};
const Segment SGI60416[] = {{&S17046[82], 5}, {NULL}};
const Segment SGI60417[] = {{&S10390[44], 15}, {NULL}};
const Segment SGI60418[] = {{&S10390[44], 28}, {NULL}};
const Segment SGI60419[] = {{&S10390[30], 6}, {NULL}};
const Segment SGI60420[] = {{&S17047[44], 9}, {NULL}};
const Segment SGI60421[] = {{&S17047[44], 21}, {NULL}};
const Segment SGI60422[] = {{&S10390[120], 3}, {NULL}};
const Segment SGI60423[] = {{&S17047[30], 6}, {NULL}};
const Segment SGI60424[] = {{&S10390[86], 5}, {NULL}};
const Segment SGI60425[] = {{&S17047[113], 3}, {NULL}};
const Segment SGI60426[] = {{&S17047[79], 5}, {NULL}};
const Segment SGI60427[] = {{&S17050[44], 14}, {NULL}};
const Segment SGI60428[] = {{&S17050[44], 35}, {NULL}};
const Segment SGI60429[] = {{&S17050[30], 6}, {NULL}};
const Segment SGI60430[] = {{&S17050[127], 3}, {NULL}};
const Segment SGI60431[] = {{&S17050[93], 5}, {NULL}};
const Segment SGI60432[] = {{&S10398[38], 11}, {NULL}};
const Segment SGI60433[] = {{&S10398[31], 6}, {NULL}};
const Segment SGI60434[] = {{&S17054[44], 7}, {NULL}};
const Segment SGI60435[] = {{&S17054[44], 19}, {NULL}};
const Segment SGI60436[] = {{&S17054[30], 6}, {NULL}};
const Segment SGI60437[] = {{&S17054[111], 3}, {NULL}};
const Segment SGI60438[] = {{&S17054[77], 5}, {NULL}};
const Segment SGI60439[] = {{&S17059[44], 13}, {NULL}};
const Segment SGI60440[] = {{&S17059[44], 25}, {NULL}};
const Segment SGI60441[] = {{&S17059[30], 6}, {NULL}};
const Segment SGI60442[] = {{&S17059[117], 3}, {NULL}};
const Segment SGI60443[] = {{&S17059[83], 5}, {NULL}};
const Segment SGI60444[] = {{&S17062[44], 14}, {NULL}};
const Segment SGI60445[] = {{&S17062[44], 26}, {NULL}};
const Segment SGI60446[] = {{&S17062[30], 6}, {NULL}};
const Segment SGI60447[] = {{&S17062[118], 3}, {NULL}};
const Segment SGI60448[] = {{&S17062[84], 5}, {NULL}};
const Segment SGI60449[] = {{&S17063[44], 13}, {NULL}};
const Segment SGI60450[] = {{&S17063[44], 25}, {NULL}};
const Segment SGI60451[] = {{&S17063[30], 6}, {NULL}};
const Segment SGI60452[] = {{&S17063[117], 3}, {NULL}};
const Segment SGI60453[] = {{&S17063[83], 5}, {NULL}};
const Segment SGI60454[] = {{&S10414[44], 15}, {NULL}};
const Segment SGI60455[] = {{&S10414[44], 29}, {NULL}};
const Segment SGI60456[] = {{&S10414[30], 6}, {NULL}};
const Segment SGI60457[] = {{&S10414[121], 3}, {NULL}};
const Segment SGI60458[] = {{&S10414[87], 5}, {NULL}};
const Segment SGI60459[] = {{&S10415[44], 11}, {NULL}};
const Segment SGI60460[] = {{&S10415[44], 22}, {NULL}};
const Segment SGI60461[] = {{&S10415[30], 6}, {NULL}};
const Segment SGI60462[] = {{&S10415[114], 3}, {NULL}};
const Segment SGI60463[] = {{&S10415[80], 5}, {NULL}};
const Segment SGI60464[] = {{&S17071[44], 8}, {NULL}};
const Segment SGI60465[] = {{&S17071[44], 20}, {NULL}};
const Segment SGI60466[] = {{&S17071[30], 6}, {NULL}};
const Segment SGI60467[] = {{&S17071[112], 3}, {NULL}};
const Segment SGI60468[] = {{&S17071[78], 5}, {NULL}};
const Segment SGI60469[] = {{&S17072[44], 5}, {NULL}};
const Segment SGI60470[] = {{&S17072[44], 17}, {NULL}};
const Segment SGI60471[] = {{&S17072[30], 6}, {NULL}};
const Segment SGI60472[] = {{&S17072[109], 3}, {NULL}};
const Segment SGI60473[] = {{&S17072[75], 5}, {NULL}};
const Segment SGI60474[] = {{&S10419[53], 11}, {NULL}};
const Segment SGI60475[] = {{&S10419[53], 22}, {NULL}};
const Segment SGI60476[] = {{&S10419[30], 15}, {NULL}};
const Segment SGI60477[] = {{&S10419[138], 3}, {NULL}};
const Segment SGI60478[] = {{&S10419[104], 5}, {NULL}};
const Segment SGI60479[] = {{&S17073[44], 12}, {NULL}};
const Segment SGI60480[] = {{&S17073[44], 39}, {NULL}};
const Segment SGI60481[] = {{&S17073[30], 6}, {NULL}};
const Segment SGI60482[] = {{&S17073[131], 3}, {NULL}};
const Segment SGI60483[] = {{&S17073[97], 5}, {NULL}};
const Segment SGI60484[] = {{&S17074[44], 9}, {NULL}};
const Segment SGI60485[] = {{&S17074[44], 21}, {NULL}};
const Segment SGI60486[] = {{&S17074[30], 6}, {NULL}};
const Segment SGI60487[] = {{&S17074[113], 3}, {NULL}};
const Segment SGI60488[] = {{&S17074[79], 5}, {NULL}};
const Segment SGI60489[] = {{&S17075[44], 4}, {NULL}};
const Segment SGI60490[] = {{&S17075[44], 16}, {NULL}};
const Segment SGI60491[] = {{&S17075[30], 6}, {NULL}};
const Segment SGI60492[] = {{&S17075[108], 3}, {NULL}};
const Segment SGI60493[] = {{&S17075[74], 5}, {NULL}};
const Segment SGI60494[] = {{&S17076[44], 7}, {NULL}};
const Segment SGI60495[] = {{&S17076[44], 19}, {NULL}};
const Segment SGI60496[] = {{&S17076[30], 6}, {NULL}};
const Segment SGI60497[] = {{&S17076[111], 3}, {NULL}};
const Segment SGI60498[] = {{&S17076[77], 5}, {NULL}};
const Segment SGI60499[] = {{&S17077[44], 6}, {NULL}};
const Segment SGI60500[] = {{&S17077[44], 18}, {NULL}};
const Segment SGI60501[] = {{&S17077[30], 6}, {NULL}};
const Segment SGI60502[] = {{&S17077[110], 3}, {NULL}};
const Segment SGI60503[] = {{&S10427[44], 25}, {NULL}};
const Segment SGI60504[] = {{&S17077[76], 5}, {NULL}};
const Segment SGI60505[] = {{&S10427[30], 6}, {NULL}};
const Segment SGI60506[] = {{&S10428[44], 15}, {NULL}};
const Segment SGI60507[] = {{&S10428[44], 32}, {NULL}};
const Segment SGI60508[] = {{&S10428[30], 6}, {NULL}};
const Segment SGI60509[] = {{&S17079[44], 8}, {NULL}};
const Segment SGI60510[] = {{&S17079[44], 26}, {NULL}};
const Segment SGI60511[] = {{&S17079[30], 6}, {NULL}};
const Segment SGI60512[] = {{&S10428[124], 3}, {NULL}};
const Segment SGI60513[] = {{&S10428[90], 5}, {NULL}};
const Segment SGI60514[] = {{&S17079[118], 3}, {NULL}};
const Segment SGI60515[] = {{&S17079[84], 5}, {NULL}};
const Segment SGI60516[] = {{&S10430[44], 11}, {NULL}};
const Segment SGI60517[] = {{&S17081[44], 7}, {NULL}};
const Segment SGI60518[] = {{&S10430[44], 23}, {NULL}};
const Segment SGI60519[] = {{&S17081[44], 19}, {NULL}};
const Segment SGI60520[] = {{&S10430[30], 6}, {NULL}};
const Segment SGI60521[] = {{&S17081[30], 6}, {NULL}};
const Segment SGI60522[] = {{&S10430[115], 3}, {NULL}};
const Segment SGI60523[] = {{&S17081[111], 3}, {NULL}};
const Segment SGI60524[] = {{&S10430[81], 5}, {NULL}};
const Segment SGI60525[] = {{&S17081[77], 5}, {NULL}};
const Segment SGI60526[] = {{&S17083[44], 10}, {NULL}};
const Segment SGI60527[] = {{&S17083[44], 28}, {NULL}};
const Segment SGI60528[] = {{&S17083[30], 6}, {NULL}};
const Segment SGI60529[] = {{&S17083[120], 3}, {NULL}};
const Segment SGI60530[] = {{&S17083[86], 5}, {NULL}};
const Segment SGI60531[] = {{&S17084[44], 7}, {NULL}};
const Segment SGI60532[] = {{&S17084[44], 19}, {NULL}};
const Segment SGI60533[] = {{&S17084[30], 6}, {NULL}};
const Segment SGI60534[] = {{&S17084[111], 3}, {NULL}};
const Segment SGI60535[] = {{&S17084[77], 5}, {NULL}};
const Segment SGI60536[] = {{&S17086[44], 11}, {NULL}};
const Segment SGI60537[] = {{&S17086[44], 23}, {NULL}};
const Segment SGI60538[] = {{&S17086[30], 6}, {NULL}};
const Segment SGI60539[] = {{&S17086[115], 3}, {NULL}};
const Segment SGI60540[] = {{&S17086[81], 5}, {NULL}};
const Segment SGI60541[] = {{&S17087[44], 15}, {NULL}};
const Segment SGI60542[] = {{&S17087[44], 27}, {NULL}};
const Segment SGI60543[] = {{&S17087[30], 6}, {NULL}};
const Segment SGI60544[] = {{&S17087[119], 3}, {NULL}};
const Segment SGI60545[] = {{&S17087[85], 5}, {NULL}};
const Segment SGI60546[] = {{&S17089[44], 15}, {NULL}};
const Segment SGI60547[] = {{&S17089[44], 26}, {NULL}};
const Segment SGI60548[] = {{&S17089[30], 6}, {NULL}};
const Segment SGI60549[] = {{&S17089[118], 3}, {NULL}};
const Segment SGI60550[] = {{&S17089[84], 5}, {NULL}};
const Segment SGI60551[] = {{&S17091[44], 12}, {NULL}};
const Segment SGI60552[] = {{&S17091[44], 24}, {NULL}};
const Segment SGI60553[] = {{&S17091[30], 6}, {NULL}};
const Segment SGI60554[] = {{&S17091[116], 3}, {NULL}};
const Segment SGI60555[] = {{&S17091[82], 5}, {NULL}};
const Segment SGI60556[] = {{&S17093[44], 5}, {NULL}};
const Segment SGI60557[] = {{&S17093[44], 17}, {NULL}};
const Segment SGI60558[] = {{&S17093[30], 6}, {NULL}};
const Segment SGI60559[] = {{&S17093[109], 3}, {NULL}};
const Segment SGI60560[] = {{&S17093[75], 5}, {NULL}};
const Segment SGI60561[] = {{&S17096[44], 4}, {NULL}};
const Segment SGI60562[] = {{&S17096[44], 16}, {NULL}};
const Segment SGI60563[] = {{&S17096[30], 6}, {NULL}};
const Segment SGI60564[] = {{&S17096[108], 3}, {NULL}};
const Segment SGI60565[] = {{&S17096[74], 5}, {NULL}};
const Segment SGI60566[] = {{&S17099[44], 6}, {NULL}};
const Segment SGI60567[] = {{&S17099[44], 49}, {NULL}};
const Segment SGI60568[] = {{&S17099[30], 6}, {NULL}};
const Segment SGI60569[] = {{&S17099[141], 3}, {NULL}};
const Segment SGI60570[] = {{&S17099[107], 5}, {NULL}};
const Segment SGI60571[] = {{&S17103[44], 12}, {NULL}};
const Segment SGI60572[] = {{&S17103[44], 33}, {NULL}};
const Segment SGI60573[] = {{&S17103[30], 6}, {NULL}};
const Segment SGI60574[] = {{&S17103[125], 3}, {NULL}};
const Segment SGI60575[] = {{&S17103[91], 5}, {NULL}};
const Segment SGI60576[] = {{&S17106[44], 11}, {NULL}};
const Segment SGI60577[] = {{&S17106[44], 23}, {NULL}};
const Segment SGI60578[] = {{&S17106[30], 6}, {NULL}};
const Segment SGI60579[] = {{&S17106[115], 3}, {NULL}};
const Segment SGI60580[] = {{&S17106[81], 5}, {NULL}};
const Segment SGI60581[] = {{&S17107[44], 11}, {NULL}};
const Segment SGI60582[] = {{&S17107[44], 23}, {NULL}};
const Segment SGI60583[] = {{&S17107[30], 6}, {NULL}};
const Segment SGI60584[] = {{&S17107[115], 3}, {NULL}};
const Segment SGI60585[] = {{&S17107[81], 5}, {NULL}};
const Segment SGI60586[] = {{&S17109[44], 6}, {NULL}};
const Segment SGI60587[] = {{&S17109[44], 27}, {NULL}};
const Segment SGI60588[] = {{&S17109[30], 6}, {NULL}};
const Segment SGI60589[] = {{&S17109[119], 3}, {NULL}};
const Segment SGI60590[] = {{&S17109[85], 5}, {NULL}};
const Segment SGI60591[] = {{&S17112[42], 13}, {NULL}};
const Segment SGI60592[] = {{&S17112[30], 6}, {NULL}};
const Segment SGI60593[] = {{&S17112[115], 3}, {NULL}};
const Segment SGI60594[] = {{&S17112[81], 5}, {NULL}};
const Segment SGI60595[] = {{&S17117[44], 11}, {NULL}};
const Segment SGI60596[] = {{&S17117[44], 23}, {NULL}};
const Segment SGI60597[] = {{&S17117[30], 6}, {NULL}};
const Segment SGI60598[] = {{&S17117[115], 3}, {NULL}};
const Segment SGI60599[] = {{&S17117[81], 5}, {NULL}};
const Segment SGI60600[] = {{&S17123[44], 9}, {NULL}};
const Segment SGI60601[] = {{&S17123[44], 21}, {NULL}};
const Segment SGI60602[] = {{&S17123[30], 6}, {NULL}};
const Segment SGI60603[] = {{&S17123[113], 3}, {NULL}};
const Segment SGI60604[] = {{&S17123[79], 5}, {NULL}};
const Segment SGI60605[] = {{&S17124[44], 11}, {NULL}};
const Segment SGI60606[] = {{&S17124[44], 23}, {NULL}};
const Segment SGI60607[] = {{&S17124[30], 6}, {NULL}};
const Segment SGI60608[] = {{&S17124[115], 3}, {NULL}};
const Segment SGI60609[] = {{&S17124[81], 5}, {NULL}};
const Segment SGI60610[] = {{&S17126[44], 15}, {NULL}};
const Segment SGI60611[] = {{&S17126[44], 27}, {NULL}};
const Segment SGI60612[] = {{&S17126[30], 6}, {NULL}};
const Segment SGI60613[] = {{&S17126[119], 3}, {NULL}};
const Segment SGI60614[] = {{&S17126[85], 5}, {NULL}};
const Segment SGI60615[] = {{&S17127[44], 5}, {NULL}};
const Segment SGI60616[] = {{&S17127[44], 17}, {NULL}};
const Segment SGI60617[] = {{&S17127[30], 6}, {NULL}};
const Segment SGI60618[] = {{&S17127[109], 3}, {NULL}};
const Segment SGI60619[] = {{&S17127[75], 5}, {NULL}};
const Segment SGI60620[] = {{&S17129[44], 18}, {NULL}};
const Segment SGI60621[] = {{&S17129[44], 30}, {NULL}};
const Segment SGI60622[] = {{&S17129[30], 6}, {NULL}};
const Segment SGI60623[] = {{&S17129[122], 3}, {NULL}};
const Segment SGI60624[] = {{&S17129[88], 5}, {NULL}};
const Segment SGI60625[] = {{&S17133[44], 8}, {NULL}};
const Segment SGI60626[] = {{&S17133[44], 20}, {NULL}};
const Segment SGI60627[] = {{&S17133[30], 6}, {NULL}};
const Segment SGI60628[] = {{&S17133[112], 3}, {NULL}};
const Segment SGI60629[] = {{&S17133[78], 5}, {NULL}};
const Segment SGI60630[] = {{&S17135[44], 13}, {NULL}};
const Segment SGI60631[] = {{&S17135[44], 25}, {NULL}};
const Segment SGI60632[] = {{&S17135[30], 6}, {NULL}};
const Segment SGI60633[] = {{&S17135[117], 3}, {NULL}};
const Segment SGI60634[] = {{&S17135[83], 5}, {NULL}};
const Segment SGI60635[] = {{&S17138[44], 5}, {NULL}};
const Segment SGI60636[] = {{&S17138[44], 17}, {NULL}};
const Segment SGI60637[] = {{&S17138[30], 6}, {NULL}};
const Segment SGI60638[] = {{&S17138[109], 3}, {NULL}};
const Segment SGI60639[] = {{&S17138[75], 5}, {NULL}};
const Segment SGI60640[] = {{&S17139[44], 18}, {NULL}};
const Segment SGI60641[] = {{&S17139[44], 44}, {NULL}};
const Segment SGI60642[] = {{&S17139[30], 6}, {NULL}};
const Segment SGI60643[] = {{&S17139[136], 3}, {NULL}};
const Segment SGI60644[] = {{&S17139[102], 5}, {NULL}};
const Segment SGI60645[] = {{&S17140[44], 4}, {NULL}};
const Segment SGI60646[] = {{&S17140[44], 16}, {NULL}};
const Segment SGI60647[] = {{&S17140[30], 6}, {NULL}};
const Segment SGI60648[] = {{&S17140[108], 3}, {NULL}};
const Segment SGI60649[] = {{&S17140[74], 5}, {NULL}};
const Segment SGI60650[] = {{&S17142[44], 4}, {NULL}};
const Segment SGI60651[] = {{&S17142[44], 21}, {NULL}};
const Segment SGI60652[] = {{&S17142[30], 6}, {NULL}};
const Segment SGI60653[] = {{&S17142[113], 3}, {NULL}};
const Segment SGI60654[] = {{&S17142[79], 5}, {NULL}};
const Segment SGI60655[] = {{&S17144[44], 7}, {NULL}};
const Segment SGI60656[] = {{&S17144[44], 19}, {NULL}};
const Segment SGI60657[] = {{&S17144[30], 6}, {NULL}};
const Segment SGI60658[] = {{&S17144[111], 3}, {NULL}};
const Segment SGI60659[] = {{&S17144[77], 5}, {NULL}};
const Segment SGI60660[] = {{&S17145[44], 7}, {NULL}};
const Segment SGI60661[] = {{&S17145[44], 19}, {NULL}};
const Segment SGI60662[] = {{&S17145[30], 6}, {NULL}};
const Segment SGI60663[] = {{&S17145[111], 3}, {NULL}};
const Segment SGI60664[] = {{&S17145[77], 5}, {NULL}};
const Segment SGI60665[] = {{&S17149[44], 5}, {NULL}};
const Segment SGI60666[] = {{&S17149[44], 26}, {NULL}};
const Segment SGI60667[] = {{&S17149[30], 6}, {NULL}};
const Segment SGI60668[] = {{&S17149[118], 3}, {NULL}};
const Segment SGI60669[] = {{&S17149[84], 5}, {NULL}};
const Segment SGI60670[] = {{&S17152[44], 13}, {NULL}};
const Segment SGI60671[] = {{&S17152[44], 25}, {NULL}};
const Segment SGI60672[] = {{&S17152[30], 6}, {NULL}};
const Segment SGI60673[] = {{&S17152[117], 3}, {NULL}};
const Segment SGI60674[] = {{&S17152[83], 5}, {NULL}};
const Segment SGI60675[] = {{&S17153[44], 13}, {NULL}};
const Segment SGI60676[] = {{&S17153[44], 25}, {NULL}};
const Segment SGI60677[] = {{&S17153[30], 6}, {NULL}};
const Segment SGI60678[] = {{&S17153[117], 3}, {NULL}};
const Segment SGI60679[] = {{&S17153[83], 5}, {NULL}};
const Segment SGI60680[] = {{&S17154[44], 12}, {NULL}};
const Segment SGI60681[] = {{&S17154[44], 24}, {NULL}};
const Segment SGI60682[] = {{&S17154[30], 6}, {NULL}};
const Segment SGI60683[] = {{&S17154[116], 3}, {NULL}};
const Segment SGI60684[] = {{&S17154[82], 5}, {NULL}};
const Segment SGI60685[] = {{&S17157[44], 9}, {NULL}};
const Segment SGI60686[] = {{&S17157[44], 21}, {NULL}};
const Segment SGI60687[] = {{&S17157[30], 6}, {NULL}};
const Segment SGI60688[] = {{&S17157[113], 3}, {NULL}};
const Segment SGI60689[] = {{&S17157[79], 5}, {NULL}};
const Segment SGI60690[] = {{&S17158[44], 9}, {NULL}};
const Segment SGI60691[] = {{&S17158[44], 21}, {NULL}};
const Segment SGI60692[] = {{&S17158[30], 6}, {NULL}};
const Segment SGI60693[] = {{&S17158[113], 3}, {NULL}};
const Segment SGI60694[] = {{&S17158[79], 5}, {NULL}};
const Segment SGI60695[] = {{&S17160[44], 6}, {NULL}};
const Segment SGI60696[] = {{&S17160[44], 18}, {NULL}};
const Segment SGI60697[] = {{&S17160[30], 6}, {NULL}};
const Segment SGI60698[] = {{&S17160[110], 3}, {NULL}};
const Segment SGI60699[] = {{&S17160[76], 5}, {NULL}};
const Segment SGI60700[] = {{&S17162[44], 7}, {NULL}};
const Segment SGI60701[] = {{&S17162[44], 19}, {NULL}};
const Segment SGI60702[] = {{&S17162[30], 6}, {NULL}};
const Segment SGI60703[] = {{&S17162[111], 3}, {NULL}};
const Segment SGI60704[] = {{&S17162[77], 5}, {NULL}};
const Segment SGI60705[] = {{&S17163[44], 9}, {NULL}};
const Segment SGI60706[] = {{&S17163[44], 21}, {NULL}};
const Segment SGI60707[] = {{&S17163[30], 6}, {NULL}};
const Segment SGI60708[] = {{&S17163[82], 4}, {NULL}};
const Segment SGI60709[] = {{&S17163[82], 3}, {NULL}};
const Segment SGI60710[] = {{&S17166[44], 10}, {NULL}};
const Segment SGI60711[] = {{&S17166[44], 22}, {NULL}};
const Segment SGI60712[] = {{&S17166[30], 6}, {NULL}};
const Segment SGI60713[] = {{&S17166[114], 3}, {NULL}};
const Segment SGI60714[] = {{&S17166[80], 5}, {NULL}};
const Segment SGI60715[] = {{&S17167[44], 9}, {NULL}};
const Segment SGI60716[] = {{&S17167[44], 21}, {NULL}};
const Segment SGI60717[] = {{&S17167[30], 6}, {NULL}};
const Segment SGI60718[] = {{&S17167[113], 3}, {NULL}};
const Segment SGI60719[] = {{&S17167[79], 5}, {NULL}};
const Segment SGI60720[] = {{&S17168[44], 4}, {NULL}};
const Segment SGI60721[] = {{&S17168[44], 16}, {NULL}};
const Segment SGI60722[] = {{&S17168[30], 6}, {NULL}};
const Segment SGI60723[] = {{&S17168[108], 3}, {NULL}};
const Segment SGI60724[] = {{&S17168[74], 5}, {NULL}};
const Segment SGI60725[] = {{&S17173[44], 5}, {NULL}};
const Segment SGI60726[] = {{&S17173[44], 17}, {NULL}};
const Segment SGI60727[] = {{&S17173[30], 6}, {NULL}};
const Segment SGI60728[] = {{&S17173[109], 3}, {NULL}};
const Segment SGI60729[] = {{&S17173[75], 5}, {NULL}};
const Segment SGI60730[] = {{&S17174[44], 12}, {NULL}};
const Segment SGI60731[] = {{&S17174[44], 24}, {NULL}};
const Segment SGI60732[] = {{&S17174[30], 6}, {NULL}};
const Segment SGI60733[] = {{&S17174[116], 3}, {NULL}};
const Segment SGI60734[] = {{&S17174[82], 5}, {NULL}};
const Segment SGI60735[] = {{&S17179[44], 9}, {NULL}};
const Segment SGI60736[] = {{&S17179[44], 21}, {NULL}};
const Segment SGI60737[] = {{&S17179[30], 6}, {NULL}};
const Segment SGI60738[] = {{&S17179[113], 3}, {NULL}};
const Segment SGI60739[] = {{&S17179[79], 5}, {NULL}};
const Segment SGI60740[] = {{&S17180[44], 5}, {NULL}};
const Segment SGI60741[] = {{&S17180[44], 17}, {NULL}};
const Segment SGI60742[] = {{&S17180[30], 6}, {NULL}};
const Segment SGI60743[] = {{&S17180[109], 3}, {NULL}};
const Segment SGI60744[] = {{&S17180[75], 5}, {NULL}};
const Segment SGI60745[] = {{&S17181[44], 9}, {NULL}};
const Segment SGI60746[] = {{&S17181[44], 21}, {NULL}};
const Segment SGI60747[] = {{&S17181[30], 6}, {NULL}};
const Segment SGI60748[] = {{&S17181[113], 3}, {NULL}};
const Segment SGI60749[] = {{&S17181[79], 5}, {NULL}};
const Segment SGI60750[] = {{&S17184[44], 11}, {NULL}};
const Segment SGI60751[] = {{&S17184[44], 23}, {NULL}};
const Segment SGI60752[] = {{&S17184[30], 6}, {NULL}};
const Segment SGI60753[] = {{&S17184[115], 3}, {NULL}};
const Segment SGI60754[] = {{&S17184[81], 5}, {NULL}};
const Segment SGI60755[] = {{&S17185[44], 9}, {NULL}};
const Segment SGI60756[] = {{&S17185[44], 21}, {NULL}};
const Segment SGI60757[] = {{&S17185[30], 6}, {NULL}};
const Segment SGI60758[] = {{&S17185[113], 3}, {NULL}};
const Segment SGI60759[] = {{&S17185[79], 5}, {NULL}};
const Segment SGI60760[] = {{&S17186[44], 20}, {NULL}};
const Segment SGI60761[] = {{&S17186[44], 32}, {NULL}};
const Segment SGI60762[] = {{&S17186[30], 6}, {NULL}};
const Segment SGI60763[] = {{&S17186[124], 3}, {NULL}};
const Segment SGI60764[] = {{&S17186[90], 5}, {NULL}};
const Segment SGI60765[] = {{&S17194[44], 73}, {NULL}};
const Segment SGI60766[] = {{&S17194[44], 94}, {NULL}};
const Segment SGI60767[] = {{&S17194[30], 6}, {NULL}};
const Segment SGI60768[] = {{&S17194[185], 3}, {NULL}};
const Segment SGI60769[] = {{&S17194[111], 5}, {NULL}};
const Segment SGI60770[] = {{&S17204[44], 6}, {NULL}};
const Segment SGI60771[] = {{&S17204[44], 19}, {NULL}};
const Segment SGI60772[] = {{&S17204[30], 6}, {NULL}};
const Segment SGI60773[] = {{&S17204[111], 3}, {NULL}};
const Segment SGI60774[] = {{&S17204[77], 5}, {NULL}};
const Segment SGI60775[] = {{&S17205[44], 10}, {NULL}};
const Segment SGI60776[] = {{&S17205[44], 22}, {NULL}};
const Segment SGI60777[] = {{&S17205[30], 6}, {NULL}};
const Segment SGI60778[] = {{&S17205[114], 3}, {NULL}};
const Segment SGI60779[] = {{&S17205[80], 5}, {NULL}};
const Segment SGI60780[] = {{&S17210[44], 7}, {NULL}};
const Segment SGI60781[] = {{&S17210[44], 19}, {NULL}};
const Segment SGI60782[] = {{&S17210[30], 6}, {NULL}};
const Segment SGI60783[] = {{&S17210[111], 3}, {NULL}};
const Segment SGI60784[] = {{&S17210[77], 5}, {NULL}};
const Segment SGI60785[] = {{&S17215[44], 9}, {NULL}};
const Segment SGI60786[] = {{&S17215[44], 21}, {NULL}};
const Segment SGI60787[] = {{&S17215[30], 6}, {NULL}};
const Segment SGI60788[] = {{&S17215[113], 3}, {NULL}};
const Segment SGI60789[] = {{&S17215[79], 5}, {NULL}};
const Segment SGI60790[] = {{&S17217[44], 5}, {NULL}};
const Segment SGI60791[] = {{&S17217[44], 17}, {NULL}};
const Segment SGI60792[] = {{&S17217[30], 6}, {NULL}};
const Segment SGI60793[] = {{&S17217[109], 3}, {NULL}};
const Segment SGI60794[] = {{&S17217[75], 5}, {NULL}};
const Segment SGI60795[] = {{&S17218[44], 28}, {NULL}};
const Segment SGI60796[] = {{&S17218[44], 45}, {NULL}};
const Segment SGI60797[] = {{&S17218[30], 6}, {NULL}};
const Segment SGI60798[] = {{&S17218[137], 3}, {NULL}};
const Segment SGI60799[] = {{&S17218[103], 5}, {NULL}};
const Segment SGI60800[] = {{&S17219[44], 11}, {NULL}};
const Segment SGI60801[] = {{&S17219[44], 23}, {NULL}};
const Segment SGI60802[] = {{&S17219[30], 6}, {NULL}};
const Segment SGI60803[] = {{&S17219[115], 3}, {NULL}};
const Segment SGI60804[] = {{&S17219[81], 5}, {NULL}};
const Segment SGI60805[] = {{&S17222[44], 12}, {NULL}};
const Segment SGI60806[] = {{&S17222[44], 24}, {NULL}};
const Segment SGI60807[] = {{&S17222[30], 6}, {NULL}};
const Segment SGI60808[] = {{&S17222[116], 3}, {NULL}};
const Segment SGI60809[] = {{&S17222[82], 5}, {NULL}};
const Segment SGI60810[] = {{&S17224[42], 10}, {NULL}};
const Segment SGI60811[] = {{&S17224[30], 6}, {NULL}};
const Segment SGI60812[] = {{&S17224[112], 3}, {NULL}};
const Segment SGI60813[] = {{&S17224[78], 5}, {NULL}};
const Segment SGI60814[] = {{&S17227[44], 12}, {NULL}};
const Segment SGI60815[] = {{&S17227[44], 24}, {NULL}};
const Segment SGI60816[] = {{&S17227[30], 6}, {NULL}};
const Segment SGI60817[] = {{&S17227[116], 3}, {NULL}};
const Segment SGI60818[] = {{&S17227[82], 5}, {NULL}};
const Segment SGI60819[] = {{&S17229[44], 7}, {NULL}};
const Segment SGI60820[] = {{&S17229[44], 19}, {NULL}};
const Segment SGI60821[] = {{&S17229[30], 6}, {NULL}};
const Segment SGI60822[] = {{&S17229[111], 3}, {NULL}};
const Segment SGI60823[] = {{&S17229[77], 5}, {NULL}};
const Segment SGI60824[] = {{&S17231[44], 16}, {NULL}};
const Segment SGI60825[] = {{&S17231[44], 28}, {NULL}};
const Segment SGI60826[] = {{&S17231[30], 6}, {NULL}};
const Segment SGI60827[] = {{&S17231[120], 3}, {NULL}};
const Segment SGI60828[] = {{&S17231[86], 5}, {NULL}};
const Segment SGI60829[] = {{&S17236[44], 10}, {NULL}};
const Segment SGI60830[] = {{&S17236[44], 25}, {NULL}};
const Segment SGI60831[] = {{&S17236[30], 6}, {NULL}};
const Segment SGI60832[] = {{&S17236[117], 3}, {NULL}};
const Segment SGI60833[] = {{&S17236[83], 5}, {NULL}};
const Segment SGI60834[] = {{&S17237[44], 7}, {NULL}};
const Segment SGI60835[] = {{&S17237[44], 19}, {NULL}};
const Segment SGI60836[] = {{&S17237[30], 6}, {NULL}};
const Segment SGI60837[] = {{&S17237[111], 3}, {NULL}};
const Segment SGI60838[] = {{&S17237[77], 5}, {NULL}};
const Segment SGI60839[] = {{&S17252[44], 7}, {NULL}};
const Segment SGI60840[] = {{&S17252[44], 28}, {NULL}};
const Segment SGI60841[] = {{&S17252[30], 6}, {NULL}};
const Segment SGI60842[] = {{&S17252[120], 3}, {NULL}};
const Segment SGI60843[] = {{&S17252[86], 5}, {NULL}};
const Segment SGI60844[] = {{&S17259[44], 7}, {NULL}};
const Segment SGI60845[] = {{&S17259[44], 19}, {NULL}};
const Segment SGI60846[] = {{&S17259[30], 6}, {NULL}};
const Segment SGI60847[] = {{&S17259[111], 3}, {NULL}};
const Segment SGI60848[] = {{&S17259[77], 5}, {NULL}};
const Segment SGI60849[] = {{&S17260[44], 13}, {NULL}};
const Segment SGI60850[] = {{&S17260[44], 31}, {NULL}};
const Segment SGI60851[] = {{&S17260[30], 6}, {NULL}};
const Segment SGI60852[] = {{&S17260[123], 3}, {NULL}};
const Segment SGI60853[] = {{&S17260[89], 5}, {NULL}};
const Segment SGI60854[] = {{&S17263[59], 9}, {NULL}};
const Segment SGI60855[] = {{&S17263[59], 21}, {NULL}};
const Segment SGI60856[] = {{&S17263[30], 20}, {NULL}};
const Segment SGI60857[] = {{&S17263[128], 3}, {NULL}};
const Segment SGI60858[] = {{&S17263[94], 5}, {NULL}};
const Segment SGI60859[] = {{&S17264[44], 6}, {NULL}};
const Segment SGI60860[] = {{&S17264[44], 27}, {NULL}};
const Segment SGI60861[] = {{&S17264[30], 6}, {NULL}};
const Segment SGI60862[] = {{&S17264[119], 3}, {NULL}};
const Segment SGI60863[] = {{&S17264[85], 5}, {NULL}};
const Segment SGI60864[] = {{&S17265[44], 5}, {NULL}};
const Segment SGI60865[] = {{&S17265[44], 17}, {NULL}};
const Segment SGI60866[] = {{&S17265[30], 6}, {NULL}};
const Segment SGI60867[] = {{&S17265[109], 3}, {NULL}};
const Segment SGI60868[] = {{&S17265[75], 5}, {NULL}};
const Segment SGI60869[] = {{&S17267[44], 10}, {NULL}};
const Segment SGI60870[] = {{&S17267[44], 26}, {NULL}};
const Segment SGI60871[] = {{&S17267[30], 6}, {NULL}};
const Segment SGI60872[] = {{&S17267[118], 3}, {NULL}};
const Segment SGI60873[] = {{&S17267[84], 5}, {NULL}};
const Segment SGI60874[] = {{&S17268[44], 13}, {NULL}};
const Segment SGI60875[] = {{&S17268[44], 25}, {NULL}};
const Segment SGI60876[] = {{&S17268[30], 6}, {NULL}};
const Segment SGI60877[] = {{&S17268[117], 3}, {NULL}};
const Segment SGI60878[] = {{&S17268[83], 5}, {NULL}};
const Segment SGI60879[] = {{&S17269[38], 4}, {NULL}};
const Segment SGI60880[] = {{&S17269[31], 6}, {NULL}};
const Segment SGI60881[] = {{&S17270[44], 15}, {NULL}};
const Segment SGI60882[] = {{&S17270[44], 27}, {NULL}};
const Segment SGI60883[] = {{&S17270[30], 6}, {NULL}};
const Segment SGI60884[] = {{&S17270[119], 3}, {NULL}};
const Segment SGI60885[] = {{&S17270[85], 5}, {NULL}};
const Segment SGI60886[] = {{&S17272[38], 13}, {NULL}};
const Segment SGI60887[] = {{&S17272[31], 6}, {NULL}};
const Segment SGI60888[] = {{&S17275[44], 9}, {NULL}};
const Segment SGI60889[] = {{&S17275[44], 21}, {NULL}};
const Segment SGI60890[] = {{&S17275[30], 6}, {NULL}};
const Segment SGI60891[] = {{&S17275[113], 3}, {NULL}};
const Segment SGI60892[] = {{&S17275[79], 5}, {NULL}};
const Segment SGI60893[] = {{&S17276[44], 10}, {NULL}};
const Segment SGI60894[] = {{&S17276[44], 22}, {NULL}};
const Segment SGI60895[] = {{&S17276[30], 6}, {NULL}};
const Segment SGI60896[] = {{&S17276[114], 3}, {NULL}};
const Segment SGI60897[] = {{&S17276[80], 5}, {NULL}};
const Segment SGI60898[] = {{&S17278[44], 13}, {NULL}};
const Segment SGI60899[] = {{&S17278[44], 34}, {NULL}};
const Segment SGI60900[] = {{&S17278[30], 6}, {NULL}};
const Segment SGI60901[] = {{&S17278[126], 3}, {NULL}};
const Segment SGI60902[] = {{&S17278[92], 5}, {NULL}};
const Segment SGI60903[] = {{&S17279[44], 14}, {NULL}};
const Segment SGI60904[] = {{&S17279[44], 42}, {NULL}};
const Segment SGI60905[] = {{&S17279[30], 6}, {NULL}};
const Segment SGI60906[] = {{&S17279[134], 3}, {NULL}};
const Segment SGI60907[] = {{&S17279[100], 5}, {NULL}};
const Segment SGI60908[] = {{&S17282[44], 16}, {NULL}};
const Segment SGI60909[] = {{&S17282[44], 28}, {NULL}};
const Segment SGI60910[] = {{&S17282[30], 6}, {NULL}};
const Segment SGI60911[] = {{&S17282[120], 3}, {NULL}};
const Segment SGI60912[] = {{&S17282[86], 5}, {NULL}};
const Segment SGI60913[] = {{&S17285[44], 10}, {NULL}};
const Segment SGI60914[] = {{&S17285[44], 22}, {NULL}};
const Segment SGI60915[] = {{&S17285[30], 6}, {NULL}};
const Segment SGI60916[] = {{&S17285[114], 3}, {NULL}};
const Segment SGI60917[] = {{&S17285[80], 5}, {NULL}};
const Segment SGI60918[] = {{&S17287[44], 72}, {NULL}};
const Segment SGI60919[] = {{&S17287[44], 93}, {NULL}};
const Segment SGI60920[] = {{&S17287[30], 6}, {NULL}};
const Segment SGI60921[] = {{&S17287[184], 3}, {NULL}};
const Segment SGI60922[] = {{&S17287[110], 5}, {NULL}};
const Segment SGI60923[] = {{&S17291[44], 10}, {NULL}};
const Segment SGI60924[] = {{&S17291[44], 22}, {NULL}};
const Segment SGI60925[] = {{&S17291[30], 6}, {NULL}};
const Segment SGI60926[] = {{&S17291[114], 3}, {NULL}};
const Segment SGI60927[] = {{&S17291[80], 5}, {NULL}};
const Segment SGI60928[] = {{&S17293[44], 7}, {NULL}};
const Segment SGI60929[] = {{&S17293[44], 28}, {NULL}};
const Segment SGI60930[] = {{&S17293[30], 6}, {NULL}};
const Segment SGI60931[] = {{&S17293[120], 3}, {NULL}};
const Segment SGI60932[] = {{&S17293[86], 5}, {NULL}};
const Segment SGI60933[] = {{&S17294[44], 17}, {NULL}};
const Segment SGI60934[] = {{&S17294[44], 29}, {NULL}};
const Segment SGI60935[] = {{&S17294[30], 6}, {NULL}};
const Segment SGI60936[] = {{&S17294[121], 3}, {NULL}};
const Segment SGI60937[] = {{&S17294[87], 5}, {NULL}};
const Segment SGI60938[] = {{&S17295[44], 5}, {NULL}};
const Segment SGI60939[] = {{&S17295[44], 26}, {NULL}};
const Segment SGI60940[] = {{&S17295[30], 6}, {NULL}};
const Segment SGI60941[] = {{&S17295[118], 3}, {NULL}};
const Segment SGI60942[] = {{&S17295[84], 5}, {NULL}};
const Segment SGI60943[] = {{&S17298[44], 8}, {NULL}};
const Segment SGI60944[] = {{&S17298[44], 20}, {NULL}};
const Segment SGI60945[] = {{&S17298[30], 6}, {NULL}};
const Segment SGI60946[] = {{&S17298[112], 3}, {NULL}};
const Segment SGI60947[] = {{&S17298[78], 5}, {NULL}};
const Segment SGI60948[] = {{&S17301[44], 12}, {NULL}};
const Segment SGI60949[] = {{&S17301[44], 24}, {NULL}};
const Segment SGI60950[] = {{&S17301[30], 6}, {NULL}};
const Segment SGI60951[] = {{&S17301[116], 3}, {NULL}};
const Segment SGI60952[] = {{&S17301[82], 5}, {NULL}};
const Segment SGI60953[] = {{&S17302[38], 7}, {NULL}};
const Segment SGI60954[] = {{&S17302[31], 6}, {NULL}};
const Segment SGI60955[] = {{&S17303[44], 13}, {NULL}};
const Segment SGI60956[] = {{&S17303[44], 25}, {NULL}};
const Segment SGI60957[] = {{&S17303[30], 6}, {NULL}};
const Segment SGI60958[] = {{&S17303[117], 3}, {NULL}};
const Segment SGI60959[] = {{&S17303[83], 5}, {NULL}};
const Segment SGI60960[] = {{&S17304[44], 7}, {NULL}};
const Segment SGI60961[] = {{&S17304[44], 19}, {NULL}};
const Segment SGI60962[] = {{&S17304[30], 6}, {NULL}};
const Segment SGI60963[] = {{&S17304[111], 3}, {NULL}};
const Segment SGI60964[] = {{&S17304[77], 5}, {NULL}};
const Segment SGI60965[] = {{&S17305[44], 13}, {NULL}};
const Segment SGI60966[] = {{&S17305[44], 29}, {NULL}};
const Segment SGI60967[] = {{&S17305[30], 6}, {NULL}};
const Segment SGI60968[] = {{&S17305[121], 3}, {NULL}};
const Segment SGI60969[] = {{&S17305[87], 5}, {NULL}};
const Segment SGI60970[] = {{&S17307[44], 11}, {NULL}};
const Segment SGI60971[] = {{&S17307[44], 23}, {NULL}};
const Segment SGI60972[] = {{&S17307[30], 6}, {NULL}};
const Segment SGI60973[] = {{&S17307[115], 3}, {NULL}};
const Segment SGI60974[] = {{&S17307[81], 5}, {NULL}};
const Segment SGI60975[] = {{&S17309[44], 72}, {NULL}};
const Segment SGI60976[] = {{&S17309[44], 93}, {NULL}};
const Segment SGI60977[] = {{&S17309[30], 6}, {NULL}};
const Segment SGI60978[] = {{&S17309[81], 3}, {NULL}};
const Segment SGI60979[] = {{&S17309[110], 5}, {NULL}};
const Segment SGI60980[] = {{&S17310[44], 17}, {NULL}};
const Segment SGI60981[] = {{&S17310[44], 29}, {NULL}};
const Segment SGI60982[] = {{&S17310[30], 6}, {NULL}};
const Segment SGI60983[] = {{&S17310[121], 3}, {NULL}};
const Segment SGI60984[] = {{&S17310[87], 5}, {NULL}};
const Segment SGI60985[] = {{&S17313[44], 6}, {NULL}};
const Segment SGI60986[] = {{&S17313[44], 27}, {NULL}};
const Segment SGI60987[] = {{&S17313[30], 6}, {NULL}};
const Segment SGI60988[] = {{&S17313[119], 3}, {NULL}};
const Segment SGI60989[] = {{&S17313[85], 5}, {NULL}};
const Segment SGI60990[] = {{&S17315[44], 17}, {NULL}};
const Segment SGI60991[] = {{&S17315[44], 29}, {NULL}};
const Segment SGI60992[] = {{&S17315[30], 6}, {NULL}};
const Segment SGI60993[] = {{&S17315[121], 3}, {NULL}};
const Segment SGI60994[] = {{&S17315[87], 5}, {NULL}};
const Segment SGI60995[] = {{&S17317[44], 11}, {NULL}};
const Segment SGI60996[] = {{&S17317[44], 28}, {NULL}};
const Segment SGI60997[] = {{&S17317[30], 6}, {NULL}};
const Segment SGI60998[] = {{&S17317[120], 3}, {NULL}};
const Segment SGI60999[] = {{&S17317[86], 5}, {NULL}};
const Segment SGI61000[] = {{&S17320[44], 10}, {NULL}};
const Segment SGI61001[] = {{&S17320[44], 22}, {NULL}};
const Segment SGI61002[] = {{&S17320[30], 6}, {NULL}};
const Segment SGI61003[] = {{&S17320[114], 3}, {NULL}};
const Segment SGI61004[] = {{&S17320[80], 5}, {NULL}};
const Segment SGI61005[] = {{&S6797[42], 6}, {NULL}};
const Segment SGI61006[] = {{&S6797[42], 27}, {NULL}};
const Segment SGI61007[] = {{&S6797[30], 4}, {NULL}};
const Segment SGI61008[] = {{&S6797[31], 3}, {NULL}};
const Segment SGI61009[] = {{&S6797[83], 5}, {NULL}};
const Segment SGI61010[] = {{&S5119[44], 10}, {NULL}};
const Segment SGI61011[] = {{&S5119[44], 19}, {NULL}};
const Segment SGI61012[] = {{&S5119[30], 6}, {NULL}};
const Segment SGI61013[] = {{&S5119[31], 3}, {NULL}};
const Segment SGI61014[] = {{&S5119[77], 5}, {NULL}};
const Segment SGI61015[] = {{&S5119[0], 13}, {NULL}};
const Segment SGI61016[] = {{&S5196[0], 15}, {NULL}};
const Segment SGI61017[] = {{&S5197[44], 10}, {NULL}};
const Segment SGI61018[] = {{&S5197[44], 22}, {NULL}};
const Segment SGI61019[] = {{&S5197[30], 6}, {NULL}};
const Segment SGI61020[] = {{&S5197[31], 3}, {NULL}};
const Segment SGI61021[] = {{&S5197[80], 6}, {NULL}};
const Segment SGI61022[] = {{&S5197[0], 13}, {NULL}};
const Segment SGI61023[] = {{&S6933[42], 12}, {NULL}};
const Segment SGI61024[] = {{&S6933[42], 24}, {NULL}};
const Segment SGI61025[] = {{&S6933[30], 4}, {NULL}};
const Segment SGI61026[] = {{&S6933[31], 3}, {NULL}};
const Segment SGI61027[] = {{&S6933[80], 6}, {NULL}};
const Segment SGI61028[] = {{&S5206[74], 6}, {NULL}};
const Segment SGI61029[] = {{&S5206[0], 9}, {NULL}};
const Segment SGI61030[] = {{&S7043[43], 4}, {NULL}};
const Segment SGI61031[] = {{&S7043[43], 16}, {NULL}};
const Segment SGI61032[] = {{&S7043[30], 4}, {NULL}};
const Segment SGI61033[] = {{&S7043[31], 3}, {NULL}};
const Segment SGI61034[] = {{&S7043[73], 5}, {NULL}};
const Segment SGI61035[] = {{&S7068[42], 14}, {NULL}};
const Segment SGI61036[] = {{&S7068[42], 26}, {NULL}};
const Segment SGI61037[] = {{&S7068[30], 4}, {NULL}};
const Segment SGI61038[] = {{&S7068[31], 3}, {NULL}};
const Segment SGI61039[] = {{&S7068[82], 6}, {NULL}};
const Segment SGI61040[] = {{&S7223[42], 6}, {NULL}};
const Segment SGI61041[] = {{&S7223[42], 27}, {NULL}};
const Segment SGI61042[] = {{&S7223[30], 4}, {NULL}};
const Segment SGI61043[] = {{&S7223[31], 3}, {NULL}};
const Segment SGI61044[] = {{&S7223[83], 5}, {NULL}};
const Segment SGI61045[] = {{&S18129[44], 28}, {NULL}};
const Segment SGI61046[] = {{&S18129[44], 45}, {NULL}};
const Segment SGI61047[] = {{&S18129[30], 6}, {NULL}};
const Segment SGI61048[] = {{&S18129[137], 3}, {NULL}};
const Segment SGI61049[] = {{&S18129[103], 5}, {NULL}};
const Segment SGI61050[] = {{&S18130[44], 11}, {NULL}};
const Segment SGI61051[] = {{&S18130[44], 28}, {NULL}};
const Segment SGI61052[] = {{&S18130[30], 6}, {NULL}};
const Segment SGI61053[] = {{&S18130[120], 3}, {NULL}};
const Segment SGI61054[] = {{&S18130[86], 5}, {NULL}};
const Segment SGI61055[] = {{&S18131[38], 11}, {NULL}};
const Segment SGI61056[] = {{&S18131[31], 6}, {NULL}};
const Segment SGI61057[] = {{&S18132[44], 29}, {NULL}};
const Segment SGI61058[] = {{&S18132[44], 46}, {NULL}};
const Segment SGI61059[] = {{&S18132[30], 6}, {NULL}};
const Segment SGI61060[] = {{&S18132[138], 3}, {NULL}};
const Segment SGI61061[] = {{&S18132[104], 5}, {NULL}};
const Segment SGI61062[] = {{&S18133[44], 10}, {NULL}};
const Segment SGI61063[] = {{&S18133[44], 27}, {NULL}};
const Segment SGI61064[] = {{&S18133[30], 6}, {NULL}};
const Segment SGI61065[] = {{&S18133[119], 3}, {NULL}};
const Segment SGI61066[] = {{&S18133[85], 5}, {NULL}};
const Segment SGI61067[] = {{&S18134[44], 8}, {NULL}};
const Segment SGI61068[] = {{&S18134[44], 25}, {NULL}};
const Segment SGI61069[] = {{&S18134[30], 6}, {NULL}};
const Segment SGI61070[] = {{&S18134[117], 3}, {NULL}};
const Segment SGI61071[] = {{&S18134[83], 5}, {NULL}};
const Segment SGI61072[] = {{&S18135[44], 11}, {NULL}};
const Segment SGI61073[] = {{&S18135[44], 28}, {NULL}};
const Segment SGI61074[] = {{&S18135[30], 6}, {NULL}};
const Segment SGI61075[] = {{&S18135[120], 3}, {NULL}};
const Segment SGI61076[] = {{&S18135[86], 5}, {NULL}};
const Segment SGI61077[] = {{&S18136[44], 11}, {NULL}};
const Segment SGI61078[] = {{&S18136[44], 28}, {NULL}};
const Segment SGI61079[] = {{&S18136[30], 6}, {NULL}};
const Segment SGI61080[] = {{&S18136[120], 3}, {NULL}};
const Segment SGI61081[] = {{&S18136[86], 5}, {NULL}};
const Segment SGI61082[] = {{&S18137[38], 10}, {NULL}};
const Segment SGI61083[] = {{&S18137[31], 6}, {NULL}};
const Segment SGI61084[] = {{&S18138[44], 29}, {NULL}};
const Segment SGI61085[] = {{&S18138[44], 46}, {NULL}};
const Segment SGI61086[] = {{&S18138[30], 6}, {NULL}};
const Segment SGI61087[] = {{&S18138[138], 3}, {NULL}};
const Segment SGI61088[] = {{&S18138[104], 5}, {NULL}};
const Segment SGI61089[] = {{&S18139[44], 29}, {NULL}};
const Segment SGI61090[] = {{&S18139[44], 46}, {NULL}};
const Segment SGI61091[] = {{&S18139[30], 6}, {NULL}};
const Segment SGI61092[] = {{&S18139[138], 3}, {NULL}};
const Segment SGI61093[] = {{&S18139[104], 5}, {NULL}};
const Segment SGI61094[] = {{&S18140[44], 29}, {NULL}};
const Segment SGI61095[] = {{&S18140[44], 46}, {NULL}};
const Segment SGI61096[] = {{&S18140[30], 6}, {NULL}};
const Segment SGI61097[] = {{&S18140[138], 3}, {NULL}};
const Segment SGI61098[] = {{&S18140[104], 5}, {NULL}};
const Segment SGI61099[] = {{&S18141[44], 11}, {NULL}};
const Segment SGI61100[] = {{&S18141[44], 28}, {NULL}};
const Segment SGI61101[] = {{&S18141[30], 6}, {NULL}};
const Segment SGI61102[] = {{&S18141[120], 3}, {NULL}};
const Segment SGI61103[] = {{&S18141[86], 5}, {NULL}};
const Segment SGI61104[] = {{&S18143[38], 11}, {NULL}};
const Segment SGI61105[] = {{&S18143[31], 6}, {NULL}};
const Segment SGI61106[] = {{&S18144[38], 11}, {NULL}};
const Segment SGI61107[] = {{&S18144[31], 6}, {NULL}};
const Segment SGI61108[] = {{&S18145[44], 10}, {NULL}};
const Segment SGI61109[] = {{&S18145[44], 27}, {NULL}};
const Segment SGI61110[] = {{&S18145[30], 6}, {NULL}};
const Segment SGI61111[] = {{&S18145[88], 4}, {NULL}};
const Segment SGI61112[] = {{&S18145[88], 3}, {NULL}};
const Segment SGI61113[] = {{&S18146[44], 10}, {NULL}};
const Segment SGI61114[] = {{&S18146[44], 27}, {NULL}};
const Segment SGI61115[] = {{&S18146[30], 6}, {NULL}};
const Segment SGI61116[] = {{&S18146[119], 3}, {NULL}};
const Segment SGI61117[] = {{&S18146[85], 5}, {NULL}};
const Segment SGI61118[] = {{&S18147[56], 10}, {NULL}};
const Segment SGI61119[] = {{&S18147[56], 27}, {NULL}};
const Segment SGI61120[] = {{&S18147[30], 18}, {NULL}};
const Segment SGI61121[] = {{&S18147[131], 3}, {NULL}};
const Segment SGI61122[] = {{&S18147[97], 5}, {NULL}};
const Segment SGI61123[] = {{&S18148[44], 29}, {NULL}};
const Segment SGI61124[] = {{&S18148[44], 46}, {NULL}};
const Segment SGI61125[] = {{&S18148[30], 6}, {NULL}};
const Segment SGI61126[] = {{&S18148[138], 3}, {NULL}};
const Segment SGI61127[] = {{&S18148[104], 5}, {NULL}};
const Segment SGI61128[] = {{&S18150[42], 10}, {NULL}};
const Segment SGI61129[] = {{&S18150[30], 6}, {NULL}};
const Segment SGI61130[] = {{&S18150[117], 3}, {NULL}};
const Segment SGI61131[] = {{&S18150[83], 5}, {NULL}};
const Segment SGI61132[] = {{&S18151[38], 8}, {NULL}};
const Segment SGI61133[] = {{&S18151[31], 6}, {NULL}};
const Segment SGI61134[] = {{&S18153[44], 28}, {NULL}};
const Segment SGI61135[] = {{&S18153[44], 45}, {NULL}};
const Segment SGI61136[] = {{&S18153[30], 6}, {NULL}};
const Segment SGI61137[] = {{&S18153[137], 3}, {NULL}};
const Segment SGI61138[] = {{&S18153[103], 5}, {NULL}};
const Segment SGI61139[] = {{&S18154[44], 10}, {NULL}};
const Segment SGI61140[] = {{&S18154[44], 27}, {NULL}};
const Segment SGI61141[] = {{&S18154[30], 6}, {NULL}};
const Segment SGI61142[] = {{&S18154[88], 6}, {NULL}};
const Segment SGI61143[] = {{&S18156[38], 11}, {NULL}};
const Segment SGI61144[] = {{&S18156[31], 6}, {NULL}};
const Segment SGI61145[] = {{&S18160[44], 27}, {NULL}};
const Segment SGI61146[] = {{&S18160[30], 6}, {NULL}};
const Segment SGI61147[] = {{&S18161[38], 10}, {NULL}};
const Segment SGI61148[] = {{&S18161[31], 6}, {NULL}};
const Segment SGI61149[] = {{&S18162[44], 28}, {NULL}};
const Segment SGI61150[] = {{&S18162[44], 45}, {NULL}};
const Segment SGI61151[] = {{&S18162[30], 6}, {NULL}};
const Segment SGI61152[] = {{&S18162[137], 3}, {NULL}};
const Segment SGI61153[] = {{&S18162[103], 5}, {NULL}};
const Segment SGI61154[] = {{&S18163[44], 10}, {NULL}};
const Segment SGI61155[] = {{&S18163[44], 27}, {NULL}};
const Segment SGI61156[] = {{&S18163[30], 6}, {NULL}};
const Segment SGI61157[] = {{&S18163[88], 6}, {NULL}};
const Segment SGI61158[] = {{&S7261[42], 14}, {NULL}};
const Segment SGI61159[] = {{&S7261[42], 25}, {NULL}};
const Segment SGI61160[] = {{&S7261[30], 4}, {NULL}};
const Segment SGI61161[] = {{&S7261[31], 3}, {NULL}};
const Segment SGI61162[] = {{&S7261[81], 5}, {NULL}};
const Segment SGI61163[] = {{&S18170[44], 11}, {NULL}};
const Segment SGI61164[] = {{&S18170[44], 28}, {NULL}};
const Segment SGI61165[] = {{&S18170[30], 6}, {NULL}};
const Segment SGI61166[] = {{&S18170[89], 6}, {NULL}};
const Segment SGI61167[] = {{&S7323[42], 13}, {NULL}};
const Segment SGI61168[] = {{&S7323[42], 25}, {NULL}};
const Segment SGI61169[] = {{&S7323[30], 4}, {NULL}};
const Segment SGI61170[] = {{&S7323[31], 3}, {NULL}};
const Segment SGI61171[] = {{&S7323[81], 5}, {NULL}};
const Segment SGI61172[] = {{&S7377[42], 4}, {NULL}};
const Segment SGI61173[] = {{&S7377[42], 15}, {NULL}};
const Segment SGI61174[] = {{&S7377[30], 4}, {NULL}};
const Segment SGI61175[] = {{&S7377[31], 3}, {NULL}};
const Segment SGI61176[] = {{&S7377[71], 5}, {NULL}};
const Segment SGI61177[] = {{&S7522[42], 6}, {NULL}};
const Segment SGI61178[] = {{&S7522[42], 18}, {NULL}};
const Segment SGI61179[] = {{&S7522[30], 4}, {NULL}};
const Segment SGI61180[] = {{&S7522[31], 3}, {NULL}};
const Segment SGI61181[] = {{&S7522[74], 5}, {NULL}};
const Segment SGI61182[] = {{&S5845[44], 32}, {NULL}};
const Segment SGI61183[] = {{&S5845[44], 44}, {NULL}};
const Segment SGI61184[] = {{&S5845[30], 6}, {NULL}};
const Segment SGI61185[] = {{&S5845[31], 3}, {NULL}};
const Segment SGI61186[] = {{&S5845[102], 6}, {NULL}};
const Segment SGI61187[] = {{&S5845[0], 13}, {NULL}};
const Segment SGI61188[] = {{&S7693[67], 4}, {NULL}};
const Segment SGI61189[] = {{&S7693[67], 23}, {NULL}};
const Segment SGI61190[] = {{&S7693[44], 15}, {NULL}};
const Segment SGI61191[] = {{&S7693[45], 3}, {NULL}};
const Segment SGI61192[] = {{&S7693[104], 6}, {NULL}};
const Segment SGI61193[] = {{&S18611[44], 21}, {NULL}};
const Segment SGI61194[] = {{&S18611[44], 33}, {NULL}};
const Segment SGI61195[] = {{&S18611[30], 6}, {NULL}};
const Segment SGI61196[] = {{&S18611[125], 3}, {NULL}};
const Segment SGI61197[] = {{&S18611[91], 5}, {NULL}};
const Segment SGI61198[] = {{&S18612[44], 14}, {NULL}};
const Segment SGI61199[] = {{&S18612[44], 26}, {NULL}};
const Segment SGI61200[] = {{&S18612[30], 6}, {NULL}};
const Segment SGI61201[] = {{&S18612[118], 3}, {NULL}};
const Segment SGI61202[] = {{&S18612[84], 5}, {NULL}};
const Segment SGI61203[] = {{&S18613[44], 9}, {NULL}};
const Segment SGI61204[] = {{&S18613[44], 21}, {NULL}};
const Segment SGI61205[] = {{&S18613[30], 6}, {NULL}};
const Segment SGI61206[] = {{&S18613[113], 3}, {NULL}};
const Segment SGI61207[] = {{&S18613[79], 5}, {NULL}};
const Segment SGI61208[] = {{&S18614[44], 21}, {NULL}};
const Segment SGI61209[] = {{&S18614[44], 33}, {NULL}};
const Segment SGI61210[] = {{&S18614[30], 6}, {NULL}};
const Segment SGI61211[] = {{&S18614[125], 3}, {NULL}};
const Segment SGI61212[] = {{&S18614[91], 5}, {NULL}};
const Segment SGI61213[] = {{&S18615[44], 14}, {NULL}};
const Segment SGI61214[] = {{&S18615[44], 26}, {NULL}};
const Segment SGI61215[] = {{&S18615[30], 6}, {NULL}};
const Segment SGI61216[] = {{&S18615[118], 3}, {NULL}};
const Segment SGI61217[] = {{&S18615[84], 5}, {NULL}};
const Segment SGI61218[] = {{&S18616[52], 6}, {NULL}};
const Segment SGI61219[] = {{&S18616[52], 18}, {NULL}};
const Segment SGI61220[] = {{&S18616[38], 6}, {NULL}};
const Segment SGI61221[] = {{&S18616[118], 3}, {NULL}};
const Segment SGI61222[] = {{&S18616[84], 5}, {NULL}};
const Segment SGI61223[] = {{&S18617[38], 9}, {NULL}};
const Segment SGI61224[] = {{&S18617[31], 6}, {NULL}};
const Segment SGI61225[] = {{&S18618[44], 10}, {NULL}};
const Segment SGI61226[] = {{&S18618[44], 22}, {NULL}};
const Segment SGI61227[] = {{&S18618[30], 6}, {NULL}};
const Segment SGI61228[] = {{&S18618[114], 3}, {NULL}};
const Segment SGI61229[] = {{&S18618[80], 5}, {NULL}};
const Segment SGI61230[] = {{&S18619[44], 14}, {NULL}};
const Segment SGI61231[] = {{&S18619[44], 26}, {NULL}};
const Segment SGI61232[] = {{&S18619[30], 6}, {NULL}};
const Segment SGI61233[] = {{&S18619[118], 3}, {NULL}};
const Segment SGI61234[] = {{&S18619[84], 5}, {NULL}};
const Segment SGI61235[] = {{&S18620[44], 10}, {NULL}};
const Segment SGI61236[] = {{&S18620[44], 22}, {NULL}};
const Segment SGI61237[] = {{&S18620[30], 6}, {NULL}};
const Segment SGI61238[] = {{&S18620[114], 3}, {NULL}};
const Segment SGI61239[] = {{&S18620[80], 5}, {NULL}};
const Segment SGI61240[] = {{&S18621[44], 15}, {NULL}};
const Segment SGI61241[] = {{&S18621[44], 27}, {NULL}};
const Segment SGI61242[] = {{&S18621[30], 6}, {NULL}};
const Segment SGI61243[] = {{&S18621[119], 3}, {NULL}};
const Segment SGI61244[] = {{&S18621[85], 5}, {NULL}};
const Segment SGI61245[] = {{&S18622[44], 21}, {NULL}};
const Segment SGI61246[] = {{&S18622[44], 33}, {NULL}};
const Segment SGI61247[] = {{&S18622[30], 6}, {NULL}};
const Segment SGI61248[] = {{&S18622[125], 3}, {NULL}};
const Segment SGI61249[] = {{&S18622[91], 5}, {NULL}};
const Segment SGI61250[] = {{&S18623[44], 14}, {NULL}};
const Segment SGI61251[] = {{&S18623[44], 26}, {NULL}};
const Segment SGI61252[] = {{&S18623[30], 6}, {NULL}};
const Segment SGI61253[] = {{&S18623[118], 3}, {NULL}};
const Segment SGI61254[] = {{&S18623[84], 5}, {NULL}};
const Segment SGI61255[] = {{&S18624[44], 14}, {NULL}};
const Segment SGI61256[] = {{&S18624[44], 26}, {NULL}};
const Segment SGI61257[] = {{&S18624[30], 6}, {NULL}};
const Segment SGI61258[] = {{&S18624[118], 3}, {NULL}};
const Segment SGI61259[] = {{&S18624[84], 5}, {NULL}};
const Segment SGI61260[] = {{&S18625[44], 14}, {NULL}};
const Segment SGI61261[] = {{&S18625[44], 26}, {NULL}};
const Segment SGI61262[] = {{&S18625[30], 6}, {NULL}};
const Segment SGI61263[] = {{&S18625[118], 3}, {NULL}};
const Segment SGI61264[] = {{&S18625[84], 5}, {NULL}};
const Segment SGI61265[] = {{&S18626[38], 10}, {NULL}};
const Segment SGI61266[] = {{&S18626[31], 6}, {NULL}};
const Segment SGI61267[] = {{&S18627[44], 14}, {NULL}};
const Segment SGI61268[] = {{&S18627[44], 26}, {NULL}};
const Segment SGI61269[] = {{&S18627[30], 6}, {NULL}};
const Segment SGI61270[] = {{&S18627[118], 3}, {NULL}};
const Segment SGI61271[] = {{&S18627[84], 5}, {NULL}};
const Segment SGI61272[] = {{&S18628[44], 15}, {NULL}};
const Segment SGI61273[] = {{&S18628[44], 27}, {NULL}};
const Segment SGI61274[] = {{&S18628[30], 6}, {NULL}};
const Segment SGI61275[] = {{&S18628[119], 3}, {NULL}};
const Segment SGI61276[] = {{&S18628[85], 5}, {NULL}};
const Segment SGI61277[] = {{&S18629[44], 10}, {NULL}};
const Segment SGI61278[] = {{&S18629[44], 22}, {NULL}};
const Segment SGI61279[] = {{&S18629[30], 6}, {NULL}};
const Segment SGI61280[] = {{&S18629[114], 3}, {NULL}};
const Segment SGI61281[] = {{&S18629[80], 5}, {NULL}};
const Segment SGI61282[] = {{&S18630[44], 21}, {NULL}};
const Segment SGI61283[] = {{&S18630[44], 33}, {NULL}};
const Segment SGI61284[] = {{&S18630[30], 6}, {NULL}};
const Segment SGI61285[] = {{&S18630[125], 3}, {NULL}};
const Segment SGI61286[] = {{&S18630[91], 5}, {NULL}};
const Segment SGI61287[] = {{&S18631[44], 14}, {NULL}};
const Segment SGI61288[] = {{&S18631[44], 26}, {NULL}};
const Segment SGI61289[] = {{&S18631[30], 6}, {NULL}};
const Segment SGI61290[] = {{&S18631[118], 3}, {NULL}};
const Segment SGI61291[] = {{&S18631[84], 5}, {NULL}};
const Segment SGI61292[] = {{&S18632[44], 14}, {NULL}};
const Segment SGI61293[] = {{&S18632[44], 26}, {NULL}};
const Segment SGI61294[] = {{&S18632[30], 6}, {NULL}};
const Segment SGI61295[] = {{&S18632[118], 3}, {NULL}};
const Segment SGI61296[] = {{&S18632[84], 5}, {NULL}};
const Segment SGI61297[] = {{&S18633[44], 14}, {NULL}};
const Segment SGI61298[] = {{&S18633[44], 26}, {NULL}};
const Segment SGI61299[] = {{&S18633[30], 6}, {NULL}};
const Segment SGI61300[] = {{&S18633[118], 3}, {NULL}};
const Segment SGI61301[] = {{&S18633[84], 5}, {NULL}};
const Segment SGI61302[] = {{&S18634[44], 14}, {NULL}};
const Segment SGI61303[] = {{&S18634[44], 26}, {NULL}};
const Segment SGI61304[] = {{&S18634[30], 6}, {NULL}};
const Segment SGI61305[] = {{&S18634[118], 3}, {NULL}};
const Segment SGI61306[] = {{&S18634[84], 5}, {NULL}};
const Segment SGI61307[] = {{&S18635[44], 14}, {NULL}};
const Segment SGI61308[] = {{&S18635[44], 26}, {NULL}};
const Segment SGI61309[] = {{&S18635[30], 6}, {NULL}};
const Segment SGI61310[] = {{&S18635[118], 3}, {NULL}};
const Segment SGI61311[] = {{&S18635[84], 5}, {NULL}};
const Segment SGI61312[] = {{&S18636[44], 17}, {NULL}};
const Segment SGI61313[] = {{&S18636[44], 29}, {NULL}};
const Segment SGI61314[] = {{&S18636[30], 6}, {NULL}};
const Segment SGI61315[] = {{&S18636[121], 3}, {NULL}};
const Segment SGI61316[] = {{&S18636[87], 5}, {NULL}};
const Segment SGI61317[] = {{&S18637[38], 10}, {NULL}};
const Segment SGI61318[] = {{&S18637[31], 6}, {NULL}};
const Segment SGI61319[] = {{&S18638[44], 14}, {NULL}};
const Segment SGI61320[] = {{&S18638[44], 26}, {NULL}};
const Segment SGI61321[] = {{&S18638[30], 6}, {NULL}};
const Segment SGI61322[] = {{&S18638[118], 3}, {NULL}};
const Segment SGI61323[] = {{&S18638[84], 5}, {NULL}};
const Segment SGI61324[] = {{&S18639[44], 21}, {NULL}};
const Segment SGI61325[] = {{&S18639[44], 33}, {NULL}};
const Segment SGI61326[] = {{&S18639[30], 6}, {NULL}};
const Segment SGI61327[] = {{&S18639[125], 3}, {NULL}};
const Segment SGI61328[] = {{&S18639[91], 5}, {NULL}};
const Segment SGI61329[] = {{&S18640[42], 10}, {NULL}};
const Segment SGI61330[] = {{&S18640[30], 6}, {NULL}};
const Segment SGI61331[] = {{&S18640[112], 3}, {NULL}};
const Segment SGI61332[] = {{&S18640[78], 5}, {NULL}};
const Segment SGI61333[] = {{&S18641[44], 10}, {NULL}};
const Segment SGI61334[] = {{&S18641[44], 22}, {NULL}};
const Segment SGI61335[] = {{&S18641[30], 6}, {NULL}};
const Segment SGI61336[] = {{&S18641[114], 3}, {NULL}};
const Segment SGI61337[] = {{&S18641[80], 5}, {NULL}};
const Segment SGI61338[] = {{&S18642[44], 21}, {NULL}};
const Segment SGI61339[] = {{&S18642[44], 33}, {NULL}};
const Segment SGI61340[] = {{&S18642[30], 6}, {NULL}};
const Segment SGI61341[] = {{&S18642[125], 3}, {NULL}};
const Segment SGI61342[] = {{&S18642[91], 5}, {NULL}};
const Segment SGI61343[] = {{&S18643[41], 9}, {NULL}};
const Segment SGI61344[] = {{&S18643[30], 6}, {NULL}};
const Segment SGI61345[] = {{&S18643[110], 3}, {NULL}};
const Segment SGI61346[] = {{&S18643[76], 5}, {NULL}};
const Segment SGI61347[] = {{&S18644[38], 10}, {NULL}};
const Segment SGI61348[] = {{&S18644[31], 6}, {NULL}};
const Segment SGI61349[] = {{&S7885[42], 13}, {NULL}};
const Segment SGI61350[] = {{&S7885[42], 25}, {NULL}};
const Segment SGI61351[] = {{&S7885[30], 4}, {NULL}};
const Segment SGI61352[] = {{&S7885[31], 3}, {NULL}};
const Segment SGI61353[] = {{&S7885[81], 5}, {NULL}};
const Segment SGI61354[] = {{&S7960[42], 9}, {NULL}};
const Segment SGI61355[] = {{&S7960[42], 30}, {NULL}};
const Segment SGI61356[] = {{&S7960[30], 4}, {NULL}};
const Segment SGI61357[] = {{&S7960[31], 3}, {NULL}};
const Segment SGI61358[] = {{&S7960[86], 5}, {NULL}};
const Segment SGI61359[] = {{&S12427[44], 11}, {NULL}};
const Segment SGI61360[] = {{&S12427[44], 25}, {NULL}};
const Segment SGI61361[] = {{&S12427[30], 6}, {NULL}};
const Segment SGI61362[] = {{&S12427[117], 3}, {NULL}};
const Segment SGI61363[] = {{&S12427[83], 5}, {NULL}};
const Segment SGI61364[] = {{&S6138[21], 3}, {NULL}};
const Segment SGI61365[] = {{&S6138[47], 3}, {NULL}};
const Segment SGI61366[] = {{&S6138[97], 5}, {NULL}};
const Segment SGI61367[] = {{&S12439[44], 11}, {NULL}};
const Segment SGI61368[] = {{&S12439[44], 43}, {NULL}};
const Segment SGI61369[] = {{&S12439[30], 6}, {NULL}};
const Segment SGI61370[] = {{&S12439[135], 3}, {NULL}};
const Segment SGI61371[] = {{&S12439[101], 5}, {NULL}};
const Segment SGI61372[] = {{&S7990[42], 5}, {NULL}};
const Segment SGI61373[] = {{&S7990[42], 17}, {NULL}};
const Segment SGI61374[] = {{&S7990[30], 4}, {NULL}};
const Segment SGI61375[] = {{&S7990[31], 3}, {NULL}};
const Segment SGI61376[] = {{&S7990[73], 5}, {NULL}};
const Segment SGI61377[] = {{&S7997[42], 5}, {NULL}};
const Segment SGI61378[] = {{&S7997[42], 26}, {NULL}};
const Segment SGI61379[] = {{&S7997[30], 4}, {NULL}};
const Segment SGI61380[] = {{&S7997[31], 3}, {NULL}};
const Segment SGI61381[] = {{&S7997[82], 5}, {NULL}};
const Segment SGI61382[] = {{&S6198[75], 6}, {NULL}};
const Segment SGI61383[] = {{&S6198[0], 10}, {NULL}};
const Segment SGI61384[] = {{&S6202[68], 6}, {NULL}};
const Segment SGI61385[] = {{&S6202[0], 3}, {NULL}};
const Segment SGI61386[] = {{&S8003[40], 14}, {NULL}};
const Segment SGI61387[] = {{&S8003[30], 4}, {NULL}};
const Segment SGI61388[] = {{&S8003[31], 3}, {NULL}};
const Segment SGI61389[] = {{&S8003[80], 5}, {NULL}};
const Segment SGI61390[] = {{&S18892[44], 10}, {NULL}};
const Segment SGI61391[] = {{&S18892[44], 27}, {NULL}};
const Segment SGI61392[] = {{&S18892[30], 6}, {NULL}};
const Segment SGI61393[] = {{&S18892[119], 3}, {NULL}};
const Segment SGI61394[] = {{&S18892[85], 5}, {NULL}};
const Segment SGI61395[] = {{&S18893[38], 10}, {NULL}};
const Segment SGI61396[] = {{&S18893[31], 6}, {NULL}};
const Segment SGI61397[] = {{&S18894[44], 29}, {NULL}};
const Segment SGI61398[] = {{&S18894[44], 46}, {NULL}};
const Segment SGI61399[] = {{&S18894[30], 6}, {NULL}};
const Segment SGI61400[] = {{&S18894[138], 3}, {NULL}};
const Segment SGI61401[] = {{&S18894[104], 5}, {NULL}};
const Segment SGI61402[] = {{&S18895[44], 11}, {NULL}};
const Segment SGI61403[] = {{&S18895[44], 28}, {NULL}};
const Segment SGI61404[] = {{&S18895[30], 6}, {NULL}};
const Segment SGI61405[] = {{&S18895[120], 3}, {NULL}};
const Segment SGI61406[] = {{&S18895[86], 5}, {NULL}};
const Segment SGI61407[] = {{&S18896[44], 30}, {NULL}};
const Segment SGI61408[] = {{&S18896[44], 47}, {NULL}};
const Segment SGI61409[] = {{&S18896[30], 6}, {NULL}};
const Segment SGI61410[] = {{&S18896[139], 3}, {NULL}};
const Segment SGI61411[] = {{&S18896[105], 5}, {NULL}};
const Segment SGI61412[] = {{&S18897[44], 29}, {NULL}};
const Segment SGI61413[] = {{&S18897[44], 46}, {NULL}};
const Segment SGI61414[] = {{&S18897[30], 6}, {NULL}};
const Segment SGI61415[] = {{&S18897[138], 3}, {NULL}};
const Segment SGI61416[] = {{&S18897[104], 5}, {NULL}};
const Segment SGI61417[] = {{&S18898[44], 28}, {NULL}};
const Segment SGI61418[] = {{&S18898[44], 45}, {NULL}};
const Segment SGI61419[] = {{&S18898[30], 6}, {NULL}};
const Segment SGI61420[] = {{&S18898[137], 3}, {NULL}};
const Segment SGI61421[] = {{&S18898[103], 5}, {NULL}};
const Segment SGI61422[] = {{&S18899[44], 31}, {NULL}};
const Segment SGI61423[] = {{&S18899[44], 48}, {NULL}};
const Segment SGI61424[] = {{&S18899[30], 6}, {NULL}};
const Segment SGI61425[] = {{&S18899[140], 3}, {NULL}};
const Segment SGI61426[] = {{&S18899[106], 5}, {NULL}};
const Segment SGI61427[] = {{&S18900[44], 10}, {NULL}};
const Segment SGI61428[] = {{&S18900[44], 27}, {NULL}};
const Segment SGI61429[] = {{&S18900[30], 6}, {NULL}};
const Segment SGI61430[] = {{&S18900[88], 6}, {NULL}};
const Segment SGI61431[] = {{&S18901[44], 30}, {NULL}};
const Segment SGI61432[] = {{&S18901[44], 47}, {NULL}};
const Segment SGI61433[] = {{&S18901[30], 6}, {NULL}};
const Segment SGI61434[] = {{&S18901[139], 3}, {NULL}};
const Segment SGI61435[] = {{&S18901[105], 5}, {NULL}};
const Segment SGI61436[] = {{&S18902[44], 29}, {NULL}};
const Segment SGI61437[] = {{&S18902[44], 46}, {NULL}};
const Segment SGI61438[] = {{&S18902[30], 6}, {NULL}};
const Segment SGI61439[] = {{&S18902[138], 3}, {NULL}};
const Segment SGI61440[] = {{&S18902[104], 5}, {NULL}};
const Segment SGI61441[] = {{&S18903[38], 11}, {NULL}};
const Segment SGI61442[] = {{&S18903[31], 6}, {NULL}};
const Segment SGI61443[] = {{&S18905[44], 29}, {NULL}};
const Segment SGI61444[] = {{&S18905[44], 46}, {NULL}};
const Segment SGI61445[] = {{&S18905[30], 6}, {NULL}};
const Segment SGI61446[] = {{&S18905[138], 3}, {NULL}};
const Segment SGI61447[] = {{&S18905[104], 5}, {NULL}};
const Segment SGI61448[] = {{&S18906[42], 10}, {NULL}};
const Segment SGI61449[] = {{&S18906[30], 6}, {NULL}};
const Segment SGI61450[] = {{&S18906[117], 3}, {NULL}};
const Segment SGI61451[] = {{&S18906[83], 5}, {NULL}};
const Segment SGI61452[] = {{&S18907[56], 10}, {NULL}};
const Segment SGI61453[] = {{&S18907[56], 27}, {NULL}};
const Segment SGI61454[] = {{&S18907[30], 18}, {NULL}};
const Segment SGI61455[] = {{&S18907[131], 3}, {NULL}};
const Segment SGI61456[] = {{&S18907[97], 5}, {NULL}};
const Segment SGI61457[] = {{&S18909[44], 30}, {NULL}};
const Segment SGI61458[] = {{&S18909[44], 47}, {NULL}};
const Segment SGI61459[] = {{&S18909[30], 6}, {NULL}};
const Segment SGI61460[] = {{&S18909[139], 3}, {NULL}};
const Segment SGI61461[] = {{&S18909[105], 5}, {NULL}};
const Segment SGI61462[] = {{&S18910[44], 31}, {NULL}};
const Segment SGI61463[] = {{&S18910[44], 48}, {NULL}};
const Segment SGI61464[] = {{&S18910[30], 6}, {NULL}};
const Segment SGI61465[] = {{&S18910[140], 3}, {NULL}};
const Segment SGI61466[] = {{&S18910[106], 5}, {NULL}};
const Segment SGI61467[] = {{&S18911[44], 29}, {NULL}};
const Segment SGI61468[] = {{&S18911[44], 46}, {NULL}};
const Segment SGI61469[] = {{&S18911[30], 6}, {NULL}};
const Segment SGI61470[] = {{&S18911[138], 3}, {NULL}};
const Segment SGI61471[] = {{&S18911[104], 5}, {NULL}};
const Segment SGI61472[] = {{&S18913[44], 29}, {NULL}};
const Segment SGI61473[] = {{&S18913[44], 46}, {NULL}};
const Segment SGI61474[] = {{&S18913[30], 6}, {NULL}};
const Segment SGI61475[] = {{&S18913[138], 3}, {NULL}};
const Segment SGI61476[] = {{&S18913[104], 5}, {NULL}};
const Segment SGI61477[] = {{&S18914[44], 29}, {NULL}};
const Segment SGI61478[] = {{&S18914[44], 46}, {NULL}};
const Segment SGI61479[] = {{&S18914[30], 6}, {NULL}};
const Segment SGI61480[] = {{&S18914[138], 3}, {NULL}};
const Segment SGI61481[] = {{&S18914[104], 5}, {NULL}};
const Segment SGI61482[] = {{&S18915[44], 29}, {NULL}};
const Segment SGI61483[] = {{&S18915[44], 46}, {NULL}};
const Segment SGI61484[] = {{&S18915[30], 6}, {NULL}};
const Segment SGI61485[] = {{&S18915[138], 3}, {NULL}};
const Segment SGI61486[] = {{&S18915[104], 5}, {NULL}};
const Segment SGI61487[] = {{&S18916[44], 28}, {NULL}};
const Segment SGI61488[] = {{&S18916[44], 45}, {NULL}};
const Segment SGI61489[] = {{&S18916[30], 6}, {NULL}};
const Segment SGI61490[] = {{&S18916[137], 3}, {NULL}};
const Segment SGI61491[] = {{&S18916[103], 5}, {NULL}};
const Segment SGI61492[] = {{&S18917[44], 30}, {NULL}};
const Segment SGI61493[] = {{&S18917[44], 47}, {NULL}};
const Segment SGI61494[] = {{&S18917[30], 6}, {NULL}};
const Segment SGI61495[] = {{&S18917[139], 3}, {NULL}};
const Segment SGI61496[] = {{&S18917[105], 5}, {NULL}};
const Segment SGI61497[] = {{&S18921[54], 10}, {NULL}};
const Segment SGI61498[] = {{&S18921[30], 18}, {NULL}};
const Segment SGI61499[] = {{&S18921[129], 3}, {NULL}};
const Segment SGI61500[] = {{&S18921[95], 5}, {NULL}};
const Segment SGI61501[] = {{&S6385[71], 6}, {NULL}};
const Segment SGI61502[] = {{&S6385[0], 6}, {NULL}};
const Segment SGI61503[] = {{&S8170[42], 9}, {NULL}};
const Segment SGI61504[] = {{&S8170[42], 21}, {NULL}};
const Segment SGI61505[] = {{&S8170[30], 4}, {NULL}};
const Segment SGI61506[] = {{&S8170[31], 3}, {NULL}};
const Segment SGI61507[] = {{&S8170[77], 5}, {NULL}};
const Segment SGI61508[] = {{&S6386[74], 6}, {NULL}};
const Segment SGI61509[] = {{&S6386[0], 9}, {NULL}};
const Segment SGI61510[] = {{&S6422[76], 6}, {NULL}};
const Segment SGI61511[] = {{&S6422[0], 11}, {NULL}};
const Segment SGI61512[] = {{&S6428[69], 6}, {NULL}};
const Segment SGI61513[] = {{&S6428[0], 4}, {NULL}};
const Segment SGI61514[] = {{&S8367[40], 6}, {NULL}};
const Segment SGI61515[] = {{&S8367[30], 4}, {NULL}};
const Segment SGI61516[] = {{&S8367[31], 3}, {NULL}};
const Segment SGI61517[] = {{&S8367[72], 5}, {NULL}};
const Segment SGI61518[] = {{&S19085[44], 12}, {NULL}};
const Segment SGI61519[] = {{&S19085[44], 24}, {NULL}};
const Segment SGI61520[] = {{&S19085[30], 6}, {NULL}};
const Segment SGI61521[] = {{&S19085[116], 3}, {NULL}};
const Segment SGI61522[] = {{&S19085[82], 5}, {NULL}};
const Segment SGI61523[] = {{&S19087[44], 17}, {NULL}};
const Segment SGI61524[] = {{&S19087[44], 29}, {NULL}};
const Segment SGI61525[] = {{&S19087[30], 6}, {NULL}};
const Segment SGI61526[] = {{&S19087[121], 3}, {NULL}};
const Segment SGI61527[] = {{&S19087[87], 5}, {NULL}};
const Segment SGI61528[] = {{&S8383[42], 12}, {NULL}};
const Segment SGI61529[] = {{&S8383[42], 28}, {NULL}};
const Segment SGI61530[] = {{&S8383[30], 4}, {NULL}};
const Segment SGI61531[] = {{&S8383[31], 3}, {NULL}};
const Segment SGI61532[] = {{&S8383[84], 5}, {NULL}};
const Segment SGI61533[] = {{&S6664[69], 6}, {NULL}};
const Segment SGI61534[] = {{&S6664[0], 4}, {NULL}};
const Segment SGI61535[] = {{&S8423[42], 14}, {NULL}};
const Segment SGI61536[] = {{&S8423[42], 26}, {NULL}};
const Segment SGI61537[] = {{&S8423[30], 4}, {NULL}};
const Segment SGI61538[] = {{&S8423[31], 3}, {NULL}};
const Segment SGI61539[] = {{&S8423[82], 5}, {NULL}};
const Segment SGI61540[] = {{&S6669[87], 6}, {NULL}};
const Segment SGI61541[] = {{&S6669[0], 22}, {NULL}};
const Segment SGI61542[] = {{&S8429[42], 6}, {NULL}};
const Segment SGI61543[] = {{&S8429[42], 18}, {NULL}};
const Segment SGI61544[] = {{&S8429[30], 4}, {NULL}};
const Segment SGI61545[] = {{&S8429[31], 3}, {NULL}};
const Segment SGI61546[] = {{&S8429[74], 5}, {NULL}};
const Segment SGI61547[] = {{&S8440[42], 38}, {NULL}};
const Segment SGI61548[] = {{&S8440[30], 4}, {NULL}};
const Segment SGI61549[] = {{&S8440[31], 3}, {NULL}};
const Segment SGI61550[] = {{&S8440[55], 6}, {NULL}};
const Segment SGI61551[] = {{&S8450[42], 5}, {NULL}};
const Segment SGI61552[] = {{&S8450[42], 24}, {NULL}};
const Segment SGI61553[] = {{&S8450[30], 4}, {NULL}};
const Segment SGI61554[] = {{&S8450[31], 3}, {NULL}};
const Segment SGI61555[] = {{&S8450[80], 5}, {NULL}};
const Segment SGI61556[] = {{&S6706[44], 9}, {NULL}};
const Segment SGI61557[] = {{&S6706[44], 21}, {NULL}};
const Segment SGI61558[] = {{&S6706[30], 6}, {NULL}};
const Segment SGI61559[] = {{&S6706[31], 3}, {NULL}};
const Segment SGI61560[] = {{&S6706[79], 6}, {NULL}};
const Segment SGI61561[] = {{&S6706[0], 13}, {NULL}};
const Segment SGI61562[] = {{&S6721[76], 6}, {NULL}};
const Segment SGI61563[] = {{&S6721[0], 11}, {NULL}};
const Segment SGI61564[] = {{&S13016[44], 10}, {NULL}};
const Segment SGI61565[] = {{&S13016[44], 21}, {NULL}};
const Segment SGI61566[] = {{&S13016[30], 6}, {NULL}};
const Segment SGI61567[] = {{&S13016[113], 3}, {NULL}};
const Segment SGI61568[] = {{&S13016[79], 5}, {NULL}};
const Segment SGI61569[] = {{&S6732[44], 8}, {NULL}};
const Segment SGI61570[] = {{&S6732[44], 20}, {NULL}};
const Segment SGI61571[] = {{&S6732[30], 6}, {NULL}};
const Segment SGI61572[] = {{&S6732[31], 3}, {NULL}};
const Segment SGI61573[] = {{&S6732[78], 5}, {NULL}};
const Segment SGI61574[] = {{&S6732[0], 13}, {NULL}};
const Segment SGI61575[] = {{&S6733[75], 6}, {NULL}};
const Segment SGI61576[] = {{&S6733[0], 10}, {NULL}};
const Segment SGI61577[] = {{&S6739[44], 7}, {NULL}};
const Segment SGI61578[] = {{&S6739[44], 19}, {NULL}};
const Segment SGI61579[] = {{&S6739[30], 6}, {NULL}};
const Segment SGI61580[] = {{&S6739[31], 3}, {NULL}};
const Segment SGI61581[] = {{&S6739[77], 5}, {NULL}};
const Segment SGI61582[] = {{&S6739[0], 13}, {NULL}};
const Segment SGI61583[] = {{&S6743[74], 6}, {NULL}};
const Segment SGI61584[] = {{&S6743[0], 9}, {NULL}};
const Segment SGI61585[] = {{&S6755[78], 6}, {NULL}};
const Segment SGI61586[] = {{&S6755[0], 15}, {NULL}};
const Segment SGI61587[] = {{&S8523[42], 10}, {NULL}};
const Segment SGI61588[] = {{&S8523[42], 22}, {NULL}};
const Segment SGI61589[] = {{&S8523[30], 4}, {NULL}};
const Segment SGI61590[] = {{&S8523[31], 3}, {NULL}};
const Segment SGI61591[] = {{&S8523[78], 6}, {NULL}};
const Segment SGI61592[] = {{&S6781[73], 6}, {NULL}};
const Segment SGI61593[] = {{&S6781[0], 8}, {NULL}};
const Segment SGI61594[] = {{&S6803[68], 6}, {NULL}};
const Segment SGI61595[] = {{&S6803[0], 3}, {NULL}};
const Segment SGI61596[] = {{&S6848[70], 6}, {NULL}};
const Segment SGI61597[] = {{&S6848[0], 5}, {NULL}};
const Segment SGI61598[] = {{&S6853[44], 4}, {NULL}};
const Segment SGI61599[] = {{&S6853[44], 16}, {NULL}};
const Segment SGI61600[] = {{&S6853[30], 6}, {NULL}};
const Segment SGI61601[] = {{&S6853[31], 3}, {NULL}};
const Segment SGI61602[] = {{&S6853[74], 5}, {NULL}};
const Segment SGI61603[] = {{&S6853[0], 13}, {NULL}};
const Segment SGI61604[] = {{&S6855[44], 5}, {NULL}};
const Segment SGI61605[] = {{&S6855[44], 17}, {NULL}};
const Segment SGI61606[] = {{&S6855[30], 6}, {NULL}};
const Segment SGI61607[] = {{&S6855[31], 3}, {NULL}};
const Segment SGI61608[] = {{&S6855[75], 5}, {NULL}};
const Segment SGI61609[] = {{&S6855[0], 13}, {NULL}};
const Segment SGI61610[] = {{&S6874[69], 6}, {NULL}};
const Segment SGI61611[] = {{&S6874[0], 4}, {NULL}};
const Segment SGI61612[] = {{&S6876[70], 6}, {NULL}};
const Segment SGI61613[] = {{&S6876[0], 5}, {NULL}};
const Segment SGI61614[] = {{&S13036[46], 10}, {NULL}};
const Segment SGI61615[] = {{&S13036[46], 21}, {NULL}};
const Segment SGI61616[] = {{&S13036[30], 8}, {NULL}};
const Segment SGI61617[] = {{&S13036[115], 3}, {NULL}};
const Segment SGI61618[] = {{&S13036[81], 5}, {NULL}};
const Segment SGI61619[] = {{&S13037[38], 10}, {NULL}};
const Segment SGI61620[] = {{&S13037[31], 6}, {NULL}};
const Segment SGI61621[] = {{&S6963[44], 10}, {NULL}};
const Segment SGI61622[] = {{&S6963[44], 22}, {NULL}};
const Segment SGI61623[] = {{&S6963[30], 6}, {NULL}};
const Segment SGI61624[] = {{&S6963[31], 3}, {NULL}};
const Segment SGI61625[] = {{&S6963[80], 5}, {NULL}};
const Segment SGI61626[] = {{&S6963[0], 13}, {NULL}};
const Segment SGI61627[] = {{&S13054[44], 10}, {NULL}};
const Segment SGI61628[] = {{&S13054[44], 21}, {NULL}};
const Segment SGI61629[] = {{&S13054[30], 6}, {NULL}};
const Segment SGI61630[] = {{&S13054[140], 3}, {NULL}};
const Segment SGI61631[] = {{&S13054[106], 5}, {NULL}};
const Segment SGI61632[] = {{&S13055[44], 10}, {NULL}};
const Segment SGI61633[] = {{&S13055[44], 27}, {NULL}};
const Segment SGI61634[] = {{&S13055[30], 6}, {NULL}};
const Segment SGI61635[] = {{&S13055[136], 3}, {NULL}};
const Segment SGI61636[] = {{&S13055[102], 5}, {NULL}};
const Segment SGI61637[] = {{&S7019[79], 6}, {NULL}};
const Segment SGI61638[] = {{&S7019[0], 14}, {NULL}};
const Segment SGI61639[] = {{&S7045[70], 6}, {NULL}};
const Segment SGI61640[] = {{&S7045[0], 5}, {NULL}};
const Segment SGI61641[] = {{&S7096[77], 6}, {NULL}};
const Segment SGI61642[] = {{&S7096[0], 12}, {NULL}};
const Segment SGI61643[] = {{&S7099[75], 6}, {NULL}};
const Segment SGI61644[] = {{&S7099[0], 10}, {NULL}};
const Segment SGI61645[] = {{&S7125[69], 6}, {NULL}};
const Segment SGI61646[] = {{&S7125[0], 4}, {NULL}};
const Segment SGI61647[] = {{&S7130[44], 6}, {NULL}};
const Segment SGI61648[] = {{&S7130[44], 27}, {NULL}};
const Segment SGI61649[] = {{&S7130[30], 6}, {NULL}};
const Segment SGI61650[] = {{&S7130[31], 3}, {NULL}};
const Segment SGI61651[] = {{&S7130[85], 5}, {NULL}};
const Segment SGI61652[] = {{&S7130[0], 13}, {NULL}};
const Segment SGI61653[] = {{&S7232[44], 13}, {NULL}};
const Segment SGI61654[] = {{&S7232[44], 40}, {NULL}};
const Segment SGI61655[] = {{&S7232[30], 6}, {NULL}};
const Segment SGI61656[] = {{&S7232[31], 3}, {NULL}};
const Segment SGI61657[] = {{&S7232[98], 5}, {NULL}};
const Segment SGI61658[] = {{&S7232[0], 13}, {NULL}};
const Segment SGI61659[] = {{&S19673[44], 10}, {NULL}};
const Segment SGI61660[] = {{&S19673[44], 27}, {NULL}};
const Segment SGI61661[] = {{&S19673[30], 6}, {NULL}};
const Segment SGI61662[] = {{&S19673[119], 3}, {NULL}};
const Segment SGI61663[] = {{&S19673[85], 5}, {NULL}};
const Segment SGI61664[] = {{&S19674[44], 11}, {NULL}};
const Segment SGI61665[] = {{&S19674[44], 28}, {NULL}};
const Segment SGI61666[] = {{&S19674[30], 6}, {NULL}};
const Segment SGI61667[] = {{&S19674[120], 3}, {NULL}};
const Segment SGI61668[] = {{&S19674[86], 5}, {NULL}};
const Segment SGI61669[] = {{&S19675[44], 29}, {NULL}};
const Segment SGI61670[] = {{&S19675[44], 46}, {NULL}};
const Segment SGI61671[] = {{&S19675[30], 6}, {NULL}};
const Segment SGI61672[] = {{&S19675[138], 3}, {NULL}};
const Segment SGI61673[] = {{&S19675[104], 5}, {NULL}};
const Segment SGI61674[] = {{&S19676[38], 10}, {NULL}};
const Segment SGI61675[] = {{&S19676[31], 6}, {NULL}};
const Segment SGI61676[] = {{&S19677[44], 29}, {NULL}};
const Segment SGI61677[] = {{&S19677[44], 46}, {NULL}};
const Segment SGI61678[] = {{&S19677[30], 6}, {NULL}};
const Segment SGI61679[] = {{&S19677[138], 3}, {NULL}};
const Segment SGI61680[] = {{&S19677[104], 5}, {NULL}};
const Segment SGI61681[] = {{&S19678[44], 29}, {NULL}};
const Segment SGI61682[] = {{&S19678[44], 46}, {NULL}};
const Segment SGI61683[] = {{&S19678[30], 6}, {NULL}};
const Segment SGI61684[] = {{&S19678[138], 3}, {NULL}};
const Segment SGI61685[] = {{&S19678[104], 5}, {NULL}};
const Segment SGI61686[] = {{&S19679[44], 11}, {NULL}};
const Segment SGI61687[] = {{&S19679[44], 28}, {NULL}};
const Segment SGI61688[] = {{&S19679[30], 6}, {NULL}};
const Segment SGI61689[] = {{&S19679[120], 3}, {NULL}};
const Segment SGI61690[] = {{&S19679[86], 5}, {NULL}};
const Segment SGI61691[] = {{&S19680[42], 10}, {NULL}};
const Segment SGI61692[] = {{&S19680[30], 6}, {NULL}};
const Segment SGI61693[] = {{&S19680[117], 3}, {NULL}};
const Segment SGI61694[] = {{&S19680[83], 5}, {NULL}};
const Segment SGI61695[] = {{&S19681[44], 10}, {NULL}};
const Segment SGI61696[] = {{&S19681[44], 27}, {NULL}};
const Segment SGI61697[] = {{&S19681[30], 6}, {NULL}};
const Segment SGI61698[] = {{&S19681[88], 6}, {NULL}};
const Segment SGI61699[] = {{&S19682[38], 11}, {NULL}};
const Segment SGI61700[] = {{&S19682[31], 6}, {NULL}};
const Segment SGI61701[] = {{&S19683[44], 29}, {NULL}};
const Segment SGI61702[] = {{&S19683[44], 46}, {NULL}};
const Segment SGI61703[] = {{&S19683[30], 6}, {NULL}};
const Segment SGI61704[] = {{&S19683[138], 3}, {NULL}};
const Segment SGI61705[] = {{&S19683[104], 5}, {NULL}};
const Segment SGI61706[] = {{&S19684[44], 30}, {NULL}};
const Segment SGI61707[] = {{&S19684[44], 47}, {NULL}};
const Segment SGI61708[] = {{&S19684[30], 6}, {NULL}};
const Segment SGI61709[] = {{&S19684[139], 3}, {NULL}};
const Segment SGI61710[] = {{&S19684[105], 5}, {NULL}};
const Segment SGI61711[] = {{&S19685[38], 11}, {NULL}};
const Segment SGI61712[] = {{&S19685[31], 6}, {NULL}};
const Segment SGI61713[] = {{&S19686[44], 29}, {NULL}};
const Segment SGI61714[] = {{&S19686[44], 46}, {NULL}};
const Segment SGI61715[] = {{&S19686[30], 6}, {NULL}};
const Segment SGI61716[] = {{&S19686[138], 3}, {NULL}};
const Segment SGI61717[] = {{&S19686[104], 5}, {NULL}};
const Segment SGI61718[] = {{&S19687[44], 29}, {NULL}};
const Segment SGI61719[] = {{&S19687[44], 46}, {NULL}};
const Segment SGI61720[] = {{&S19687[30], 6}, {NULL}};
const Segment SGI61721[] = {{&S19687[138], 3}, {NULL}};
const Segment SGI61722[] = {{&S19687[104], 5}, {NULL}};
const Segment SGI61723[] = {{&S7284[44], 5}, {NULL}};
const Segment SGI61724[] = {{&S7284[44], 26}, {NULL}};
const Segment SGI61725[] = {{&S7284[30], 6}, {NULL}};
const Segment SGI61726[] = {{&S7284[31], 3}, {NULL}};
const Segment SGI61727[] = {{&S7284[84], 5}, {NULL}};
const Segment SGI61728[] = {{&S7284[0], 13}, {NULL}};
const Segment SGI61729[] = {{&S7288[79], 6}, {NULL}};
const Segment SGI61730[] = {{&S7288[0], 14}, {NULL}};
const Segment SGI61731[] = {{&S7347[76], 6}, {NULL}};
const Segment SGI61732[] = {{&S7347[0], 11}, {NULL}};
const Segment SGI61733[] = {{&S7357[72], 6}, {NULL}};
const Segment SGI61734[] = {{&S7357[0], 7}, {NULL}};
const Segment SGI61735[] = {{&S7364[73], 6}, {NULL}};
const Segment SGI61736[] = {{&S7364[0], 8}, {NULL}};
const Segment SGI61737[] = {{&S7367[44], 6}, {NULL}};
const Segment SGI61738[] = {{&S7367[44], 27}, {NULL}};
const Segment SGI61739[] = {{&S7367[30], 6}, {NULL}};
const Segment SGI61740[] = {{&S7367[31], 3}, {NULL}};
const Segment SGI61741[] = {{&S7367[85], 5}, {NULL}};
const Segment SGI61742[] = {{&S7367[0], 13}, {NULL}};
const Segment SGI61743[] = {{&S7463[75], 6}, {NULL}};
const Segment SGI61744[] = {{&S7463[0], 10}, {NULL}};
const Segment SGI61745[] = {{&S5156[44], 10}, {NULL}};
const Segment SGI61746[] = {{&S5156[44], 21}, {NULL}};
const Segment SGI61747[] = {{&S5156[30], 6}, {NULL}};
const Segment SGI61748[] = {{&S5156[31], 3}, {NULL}};
const Segment SGI61749[] = {{&S5156[79], 5}, {NULL}};
const Segment SGI61750[] = {{&S7473[71], 6}, {NULL}};
const Segment SGI61751[] = {{&S7473[0], 6}, {NULL}};
const Segment SGI61752[] = {{&S7497[78], 6}, {NULL}};
const Segment SGI61753[] = {{&S7497[0], 13}, {NULL}};
const Segment SGI61754[] = {{&S7513[44], 5}, {NULL}};
const Segment SGI61755[] = {{&S7513[44], 26}, {NULL}};
const Segment SGI61756[] = {{&S7513[30], 6}, {NULL}};
const Segment SGI61757[] = {{&S7513[31], 3}, {NULL}};
const Segment SGI61758[] = {{&S7513[84], 5}, {NULL}};
const Segment SGI61759[] = {{&S7513[0], 13}, {NULL}};
const Segment SGI61760[] = {{&S7530[78], 6}, {NULL}};
const Segment SGI61761[] = {{&S7530[0], 13}, {NULL}};
const Segment SGI61762[] = {{&S7540[77], 6}, {NULL}};
const Segment SGI61763[] = {{&S7540[0], 12}, {NULL}};
const Segment SGI61764[] = {{&S7542[75], 6}, {NULL}};
const Segment SGI61765[] = {{&S7542[0], 10}, {NULL}};
const Segment SGI61766[] = {{&S7554[71], 6}, {NULL}};
const Segment SGI61767[] = {{&S7554[0], 6}, {NULL}};
const Segment SGI61768[] = {{&S7576[44], 75}, {NULL}};
const Segment SGI61769[] = {{&S7576[44], 96}, {NULL}};
const Segment SGI61770[] = {{&S7576[30], 6}, {NULL}};
const Segment SGI61771[] = {{&S7576[31], 3}, {NULL}};
const Segment SGI61772[] = {{&S7576[113], 5}, {NULL}};
const Segment SGI61773[] = {{&S7576[0], 13}, {NULL}};
const Segment SGI61774[] = {{&S7584[73], 6}, {NULL}};
const Segment SGI61775[] = {{&S7584[0], 8}, {NULL}};
const Segment SGI61776[] = {{&S7588[70], 6}, {NULL}};
const Segment SGI61777[] = {{&S7588[0], 5}, {NULL}};
const Segment SGI61778[] = {{&S5116[44], 10}, {NULL}};
const Segment SGI61779[] = {{&S5116[44], 22}, {NULL}};
const Segment SGI61780[] = {{&S5116[30], 6}, {NULL}};
const Segment SGI61781[] = {{&S5116[31], 3}, {NULL}};
const Segment SGI61782[] = {{&S5116[80], 6}, {NULL}};
const Segment SGI61783[] = {{&S5117[44], 10}, {NULL}};
const Segment SGI61784[] = {{&S5117[44], 19}, {NULL}};
const Segment SGI61785[] = {{&S5117[30], 6}, {NULL}};
const Segment SGI61786[] = {{&S5117[31], 3}, {NULL}};
const Segment SGI61787[] = {{&S5117[77], 6}, {NULL}};
const Segment SGI61788[] = {{&S5118[44], 10}, {NULL}};
const Segment SGI61789[] = {{&S5118[44], 21}, {NULL}};
const Segment SGI61790[] = {{&S5118[30], 6}, {NULL}};
const Segment SGI61791[] = {{&S5118[31], 3}, {NULL}};
const Segment SGI61792[] = {{&S5118[79], 6}, {NULL}};
const Segment SGI61793[] = {{&S7685[44], 6}, {NULL}};
const Segment SGI61794[] = {{&S7685[44], 27}, {NULL}};
const Segment SGI61795[] = {{&S7685[30], 6}, {NULL}};
const Segment SGI61796[] = {{&S7685[31], 3}, {NULL}};
const Segment SGI61797[] = {{&S7685[85], 5}, {NULL}};
const Segment SGI61798[] = {{&S7685[0], 13}, {NULL}};
const Segment SGI61799[] = {{&S5121[44], 29}, {NULL}};
const Segment SGI61800[] = {{&S5121[44], 41}, {NULL}};
const Segment SGI61801[] = {{&S5121[30], 6}, {NULL}};
const Segment SGI61802[] = {{&S5121[31], 3}, {NULL}};
const Segment SGI61803[] = {{&S5121[99], 6}, {NULL}};
const Segment SGI61804[] = {{&S5122[44], 28}, {NULL}};
const Segment SGI61805[] = {{&S5122[44], 40}, {NULL}};
const Segment SGI61806[] = {{&S5122[30], 6}, {NULL}};
const Segment SGI61807[] = {{&S5122[31], 3}, {NULL}};
const Segment SGI61808[] = {{&S5122[98], 6}, {NULL}};
const Segment SGI61809[] = {{&S5123[44], 29}, {NULL}};
const Segment SGI61810[] = {{&S5123[44], 41}, {NULL}};
const Segment SGI61811[] = {{&S5123[30], 6}, {NULL}};
const Segment SGI61812[] = {{&S5123[31], 3}, {NULL}};
const Segment SGI61813[] = {{&S5123[99], 6}, {NULL}};
const Segment SGI61814[] = {{&S5124[44], 29}, {NULL}};
const Segment SGI61815[] = {{&S5124[44], 41}, {NULL}};
const Segment SGI61816[] = {{&S5124[30], 6}, {NULL}};
const Segment SGI61817[] = {{&S5124[31], 3}, {NULL}};
const Segment SGI61818[] = {{&S5124[99], 6}, {NULL}};
const Segment SGI61819[] = {{&S5126[44], 11}, {NULL}};
const Segment SGI61820[] = {{&S5126[44], 23}, {NULL}};
const Segment SGI61821[] = {{&S5126[30], 6}, {NULL}};
const Segment SGI61822[] = {{&S5126[31], 3}, {NULL}};
const Segment SGI61823[] = {{&S5126[81], 6}, {NULL}};
const Segment SGI61824[] = {{&S5127[44], 11}, {NULL}};
const Segment SGI61825[] = {{&S5127[44], 23}, {NULL}};
const Segment SGI61826[] = {{&S5127[30], 6}, {NULL}};
const Segment SGI61827[] = {{&S5127[31], 3}, {NULL}};
const Segment SGI61828[] = {{&S5127[81], 6}, {NULL}};
const Segment SGI61829[] = {{&S5131[44], 10}, {NULL}};
const Segment SGI61830[] = {{&S5131[44], 22}, {NULL}};
const Segment SGI61831[] = {{&S5131[30], 6}, {NULL}};
const Segment SGI61832[] = {{&S5131[80], 6}, {NULL}};
const Segment SGI61833[] = {{&S7706[78], 6}, {NULL}};
const Segment SGI61834[] = {{&S5132[38], 10}, {NULL}};
const Segment SGI61835[] = {{&S7706[0], 13}, {NULL}};
const Segment SGI61836[] = {{&S5132[31], 6}, {NULL}};
const Segment SGI61837[] = {{&S5134[38], 11}, {NULL}};
const Segment SGI61838[] = {{&S5134[31], 6}, {NULL}};
const Segment SGI61839[] = {{&S5136[44], 10}, {NULL}};
const Segment SGI61840[] = {{&S5136[44], 22}, {NULL}};
const Segment SGI61841[] = {{&S5136[30], 6}, {NULL}};
const Segment SGI61842[] = {{&S5136[83], 4}, {NULL}};
const Segment SGI61843[] = {{&S5136[83], 3}, {NULL}};
const Segment SGI61844[] = {{&S5137[38], 11}, {NULL}};
const Segment SGI61845[] = {{&S5137[31], 6}, {NULL}};
const Segment SGI61846[] = {{&S5138[42], 10}, {NULL}};
const Segment SGI61847[] = {{&S5138[30], 6}, {NULL}};
const Segment SGI61848[] = {{&S5138[31], 3}, {NULL}};
const Segment SGI61849[] = {{&S5138[78], 6}, {NULL}};
const Segment SGI61850[] = {{&S5139[44], 10}, {NULL}};
const Segment SGI61851[] = {{&S5139[44], 22}, {NULL}};
const Segment SGI61852[] = {{&S5139[30], 6}, {NULL}};
const Segment SGI61853[] = {{&S5139[31], 3}, {NULL}};
const Segment SGI61854[] = {{&S5139[80], 6}, {NULL}};
const Segment SGI61855[] = {{&S5140[44], 29}, {NULL}};
const Segment SGI61856[] = {{&S5140[44], 41}, {NULL}};
const Segment SGI61857[] = {{&S5140[30], 6}, {NULL}};
const Segment SGI61858[] = {{&S5140[31], 3}, {NULL}};
const Segment SGI61859[] = {{&S5140[99], 6}, {NULL}};
const Segment SGI61860[] = {{&S5141[45], 10}, {NULL}};
const Segment SGI61861[] = {{&S5141[30], 6}, {NULL}};
const Segment SGI61862[] = {{&S5141[31], 3}, {NULL}};
const Segment SGI61863[] = {{&S5141[81], 5}, {NULL}};
const Segment SGI61864[] = {{&S5141[139], 13}, {NULL}};
const Segment SGI61865[] = {{&S7723[76], 6}, {NULL}};
const Segment SGI61866[] = {{&S7723[0], 11}, {NULL}};
const Segment SGI61867[] = {{&S5143[44], 11}, {NULL}};
const Segment SGI61868[] = {{&S5143[44], 23}, {NULL}};
const Segment SGI61869[] = {{&S5143[30], 6}, {NULL}};
const Segment SGI61870[] = {{&S5143[31], 3}, {NULL}};
const Segment SGI61871[] = {{&S5143[81], 6}, {NULL}};
const Segment SGI61872[] = {{&S5144[38], 11}, {NULL}};
const Segment SGI61873[] = {{&S5144[31], 6}, {NULL}};
const Segment SGI61874[] = {{&S5146[44], 10}, {NULL}};
const Segment SGI61875[] = {{&S5146[44], 22}, {NULL}};
const Segment SGI61876[] = {{&S5146[30], 6}, {NULL}};
const Segment SGI61877[] = {{&S5146[83], 6}, {NULL}};
const Segment SGI61878[] = {{&S5147[44], 29}, {NULL}};
const Segment SGI61879[] = {{&S5147[44], 41}, {NULL}};
const Segment SGI61880[] = {{&S5147[30], 6}, {NULL}};
const Segment SGI61881[] = {{&S5147[31], 3}, {NULL}};
const Segment SGI61882[] = {{&S5147[99], 6}, {NULL}};
const Segment SGI61883[] = {{&S5148[44], 29}, {NULL}};
const Segment SGI61884[] = {{&S5148[44], 41}, {NULL}};
const Segment SGI61885[] = {{&S5148[30], 6}, {NULL}};
const Segment SGI61886[] = {{&S5148[31], 3}, {NULL}};
const Segment SGI61887[] = {{&S5148[99], 6}, {NULL}};
const Segment SGI61888[] = {{&S5149[44], 28}, {NULL}};
const Segment SGI61889[] = {{&S5149[44], 40}, {NULL}};
const Segment SGI61890[] = {{&S5149[30], 6}, {NULL}};
const Segment SGI61891[] = {{&S5149[31], 3}, {NULL}};
const Segment SGI61892[] = {{&S5149[98], 6}, {NULL}};
const Segment SGI61893[] = {{&S5152[44], 10}, {NULL}};
const Segment SGI61894[] = {{&S5152[44], 20}, {NULL}};
const Segment SGI61895[] = {{&S5152[30], 6}, {NULL}};
const Segment SGI61896[] = {{&S5152[31], 3}, {NULL}};
const Segment SGI61897[] = {{&S5152[78], 6}, {NULL}};
const Segment SGI61898[] = {{&S7746[83], 6}, {NULL}};
const Segment SGI61899[] = {{&S7746[0], 18}, {NULL}};
const Segment SGI61900[] = {{&S5161[42], 11}, {NULL}};
const Segment SGI61901[] = {{&S5161[30], 6}, {NULL}};
const Segment SGI61902[] = {{&S5161[31], 3}, {NULL}};
const Segment SGI61903[] = {{&S5161[79], 6}, {NULL}};
const Segment SGI61904[] = {{&S5163[44], 22}, {NULL}};
const Segment SGI61905[] = {{&S5163[30], 6}, {NULL}};
const Segment SGI61906[] = {{&S7755[79], 6}, {NULL}};
const Segment SGI61907[] = {{&S5166[44], 29}, {NULL}};
const Segment SGI61908[] = {{&S5166[44], 41}, {NULL}};
const Segment SGI61909[] = {{&S7755[0], 14}, {NULL}};
const Segment SGI61910[] = {{&S5166[30], 6}, {NULL}};
const Segment SGI61911[] = {{&S5166[31], 3}, {NULL}};
const Segment SGI61912[] = {{&S5166[99], 6}, {NULL}};
const Segment SGI61913[] = {{&S5167[44], 10}, {NULL}};
const Segment SGI61914[] = {{&S5167[44], 27}, {NULL}};
const Segment SGI61915[] = {{&S5167[30], 6}, {NULL}};
const Segment SGI61916[] = {{&S5167[31], 3}, {NULL}};
const Segment SGI61917[] = {{&S5167[85], 5}, {NULL}};
const Segment SGI61918[] = {{&S20377[44], 18}, {NULL}};
const Segment SGI61919[] = {{&S20377[44], 30}, {NULL}};
const Segment SGI61920[] = {{&S20377[30], 6}, {NULL}};
const Segment SGI61921[] = {{&S20377[122], 3}, {NULL}};
const Segment SGI61922[] = {{&S20377[88], 5}, {NULL}};
const Segment SGI61923[] = {{&S5174[42], 11}, {NULL}};
const Segment SGI61924[] = {{&S5174[30], 6}, {NULL}};
const Segment SGI61925[] = {{&S5174[31], 3}, {NULL}};
const Segment SGI61926[] = {{&S5174[79], 6}, {NULL}};
const Segment SGI61927[] = {{&S20381[44], 11}, {NULL}};
const Segment SGI61928[] = {{&S20381[44], 23}, {NULL}};
const Segment SGI61929[] = {{&S20381[30], 6}, {NULL}};
const Segment SGI61930[] = {{&S20381[115], 3}, {NULL}};
const Segment SGI61931[] = {{&S20381[81], 5}, {NULL}};
const Segment SGI61932[] = {{&S20382[44], 19}, {NULL}};
const Segment SGI61933[] = {{&S20382[44], 31}, {NULL}};
const Segment SGI61934[] = {{&S20382[30], 6}, {NULL}};
const Segment SGI61935[] = {{&S5176[44], 10}, {NULL}};
const Segment SGI61936[] = {{&S5176[44], 22}, {NULL}};
const Segment SGI61937[] = {{&S5176[30], 6}, {NULL}};
const Segment SGI61938[] = {{&S20382[123], 3}, {NULL}};
const Segment SGI61939[] = {{&S20382[89], 5}, {NULL}};
const Segment SGI61940[] = {{&S5176[80], 6}, {NULL}};
const Segment SGI61941[] = {{&S20383[44], 16}, {NULL}};
const Segment SGI61942[] = {{&S20383[44], 28}, {NULL}};
const Segment SGI61943[] = {{&S20383[30], 6}, {NULL}};
const Segment SGI61944[] = {{&S20383[120], 3}, {NULL}};
const Segment SGI61945[] = {{&S20383[86], 5}, {NULL}};
const Segment SGI61946[] = {{&S5181[42], 29}, {NULL}};
const Segment SGI61947[] = {{&S5181[30], 6}, {NULL}};
const Segment SGI61948[] = {{&S5181[31], 3}, {NULL}};
const Segment SGI61949[] = {{&S5181[97], 6}, {NULL}};
const Segment SGI61950[] = {{&S5184[44], 11}, {NULL}};
const Segment SGI61951[] = {{&S5184[44], 23}, {NULL}};
const Segment SGI61952[] = {{&S5184[30], 6}, {NULL}};
const Segment SGI61953[] = {{&S5184[84], 6}, {NULL}};
const Segment SGI61954[] = {{&S5186[44], 10}, {NULL}};
const Segment SGI61955[] = {{&S5186[44], 33}, {NULL}};
const Segment SGI61956[] = {{&S5186[30], 6}, {NULL}};
const Segment SGI61957[] = {{&S20393[44], 22}, {NULL}};
const Segment SGI61958[] = {{&S20393[44], 34}, {NULL}};
const Segment SGI61959[] = {{&S20393[30], 6}, {NULL}};
const Segment SGI61960[] = {{&S5186[31], 3}, {NULL}};
const Segment SGI61961[] = {{&S5186[91], 6}, {NULL}};
const Segment SGI61962[] = {{&S20393[126], 3}, {NULL}};
const Segment SGI61963[] = {{&S20393[92], 5}, {NULL}};
const Segment SGI61964[] = {{&S5188[44], 10}, {NULL}};
const Segment SGI61965[] = {{&S5188[44], 27}, {NULL}};
const Segment SGI61966[] = {{&S5188[30], 6}, {NULL}};
const Segment SGI61967[] = {{&S5188[31], 3}, {NULL}};
const Segment SGI61968[] = {{&S5188[85], 6}, {NULL}};
const Segment SGI61969[] = {{&S5189[44], 11}, {NULL}};
const Segment SGI61970[] = {{&S5189[44], 23}, {NULL}};
const Segment SGI61971[] = {{&S5189[30], 6}, {NULL}};
const Segment SGI61972[] = {{&S5189[84], 6}, {NULL}};
const Segment SGI61973[] = {{&S20398[44], 21}, {NULL}};
const Segment SGI61974[] = {{&S20398[44], 33}, {NULL}};
const Segment SGI61975[] = {{&S20398[30], 6}, {NULL}};
const Segment SGI61976[] = {{&S20398[125], 3}, {NULL}};
const Segment SGI61977[] = {{&S20398[91], 5}, {NULL}};
const Segment SGI61978[] = {{&S20399[44], 18}, {NULL}};
const Segment SGI61979[] = {{&S20399[44], 30}, {NULL}};
const Segment SGI61980[] = {{&S20399[30], 6}, {NULL}};
const Segment SGI61981[] = {{&S20399[122], 3}, {NULL}};
const Segment SGI61982[] = {{&S5192[42], 11}, {NULL}};
const Segment SGI61983[] = {{&S20399[88], 5}, {NULL}};
const Segment SGI61984[] = {{&S5192[30], 6}, {NULL}};
const Segment SGI61985[] = {{&S5192[31], 3}, {NULL}};
const Segment SGI61986[] = {{&S5192[79], 6}, {NULL}};
const Segment SGI61987[] = {{&S20402[22], 9}, {NULL}};
const Segment SGI61988[] = {{&S20402[44], 21}, {NULL}};
const Segment SGI61989[] = {{&S20402[30], 6}, {&S20402[52], 13}, {NULL}};
const Segment SGI61990[] = {{&S20402[113], 3}, {NULL}};
const Segment SGI61991[] = {{&S20402[79], 5}, {NULL}};
const Segment SGI61992[] = {{&S20404[44], 24}, {NULL}};
const Segment SGI61993[] = {{&S20404[44], 36}, {NULL}};
const Segment SGI61994[] = {{&S20404[30], 6}, {NULL}};
const Segment SGI61995[] = {{&S20404[128], 3}, {NULL}};
const Segment SGI61996[] = {{&S20404[94], 5}, {NULL}};
const Segment SGI61997[] = {{&S7776[71], 6}, {NULL}};
const Segment SGI61998[] = {{&S20405[44], 20}, {NULL}};
const Segment SGI61999[] = {{&S20405[44], 32}, {NULL}};
const Segment SGI62000[] = {{&S7776[0], 6}, {NULL}};
const Segment SGI62001[] = {{&S20405[30], 6}, {NULL}};
const Segment SGI62002[] = {{&S20405[124], 3}, {NULL}};
const Segment SGI62003[] = {{&S20405[90], 5}, {NULL}};
const Segment SGI62004[] = {{&S20409[44], 22}, {NULL}};
const Segment SGI62005[] = {{&S20409[44], 34}, {NULL}};
const Segment SGI62006[] = {{&S20409[30], 6}, {NULL}};
const Segment SGI62007[] = {{&S20409[126], 3}, {NULL}};
const Segment SGI62008[] = {{&S20409[92], 5}, {NULL}};
const Segment SGI62009[] = {{&S5205[44], 10}, {NULL}};
const Segment SGI62010[] = {{&S5205[44], 29}, {NULL}};
const Segment SGI62011[] = {{&S5205[30], 6}, {NULL}};
const Segment SGI62012[] = {{&S5205[31], 3}, {NULL}};
const Segment SGI62013[] = {{&S5205[87], 6}, {NULL}};
const Segment SGI62014[] = {{&S20412[44], 21}, {NULL}};
const Segment SGI62015[] = {{&S20412[44], 33}, {NULL}};
const Segment SGI62016[] = {{&S20412[30], 6}, {NULL}};
const Segment SGI62017[] = {{&S20412[125], 3}, {NULL}};
const Segment SGI62018[] = {{&S20412[91], 5}, {NULL}};
const Segment SGI62019[] = {{&S20413[44], 20}, {NULL}};
const Segment SGI62020[] = {{&S20413[44], 32}, {NULL}};
const Segment SGI62021[] = {{&S20413[30], 6}, {NULL}};
const Segment SGI62022[] = {{&S20413[124], 3}, {NULL}};
const Segment SGI62023[] = {{&S20413[90], 5}, {NULL}};
const Segment SGI62024[] = {{&S5210[44], 10}, {NULL}};
const Segment SGI62025[] = {{&S5210[44], 19}, {NULL}};
const Segment SGI62026[] = {{&S5210[30], 6}, {NULL}};
const Segment SGI62027[] = {{&S5210[31], 3}, {NULL}};
const Segment SGI62028[] = {{&S20414[44], 16}, {NULL}};
const Segment SGI62029[] = {{&S20414[44], 28}, {NULL}};
const Segment SGI62030[] = {{&S5210[77], 6}, {NULL}};
const Segment SGI62031[] = {{&S20414[30], 6}, {NULL}};
const Segment SGI62032[] = {{&S20414[120], 3}, {NULL}};
const Segment SGI62033[] = {{&S20414[86], 5}, {NULL}};
const Segment SGI62034[] = {{&S20415[44], 17}, {NULL}};
const Segment SGI62035[] = {{&S20415[44], 29}, {NULL}};
const Segment SGI62036[] = {{&S20415[30], 6}, {NULL}};
const Segment SGI62037[] = {{&S20415[121], 3}, {NULL}};
const Segment SGI62038[] = {{&S20415[87], 5}, {NULL}};
const Segment SGI62039[] = {{&S7792[44], 5}, {NULL}};
const Segment SGI62040[] = {{&S7792[44], 26}, {NULL}};
const Segment SGI62041[] = {{&S7792[30], 6}, {NULL}};
const Segment SGI62042[] = {{&S7792[31], 3}, {NULL}};
const Segment SGI62043[] = {{&S7792[84], 5}, {NULL}};
const Segment SGI62044[] = {{&S7792[0], 13}, {NULL}};
const Segment SGI62045[] = {{&S20417[44], 19}, {NULL}};
const Segment SGI62046[] = {{&S20417[44], 31}, {NULL}};
const Segment SGI62047[] = {{&S20417[30], 6}, {NULL}};
const Segment SGI62048[] = {{&S20417[123], 3}, {NULL}};
const Segment SGI62049[] = {{&S20417[89], 5}, {NULL}};
const Segment SGI62050[] = {{&S20418[44], 24}, {NULL}};
const Segment SGI62051[] = {{&S20418[44], 36}, {NULL}};
const Segment SGI62052[] = {{&S20418[30], 6}, {NULL}};
const Segment SGI62053[] = {{&S20418[128], 3}, {NULL}};
const Segment SGI62054[] = {{&S20418[94], 5}, {NULL}};
const Segment SGI62055[] = {{&S20419[44], 29}, {NULL}};
const Segment SGI62056[] = {{&S20419[44], 41}, {NULL}};
const Segment SGI62057[] = {{&S20419[30], 6}, {NULL}};
const Segment SGI62058[] = {{&S20419[133], 3}, {NULL}};
const Segment SGI62059[] = {{&S20419[99], 5}, {NULL}};
const Segment SGI62060[] = {{&S20424[44], 18}, {NULL}};
const Segment SGI62061[] = {{&S20424[44], 30}, {NULL}};
const Segment SGI62062[] = {{&S20424[30], 6}, {NULL}};
const Segment SGI62063[] = {{&S20424[122], 3}, {NULL}};
const Segment SGI62064[] = {{&S20424[88], 5}, {NULL}};
const Segment SGI62065[] = {{&S20425[44], 24}, {NULL}};
const Segment SGI62066[] = {{&S20425[44], 36}, {NULL}};
const Segment SGI62067[] = {{&S20425[30], 6}, {NULL}};
const Segment SGI62068[] = {{&S20425[128], 3}, {NULL}};
const Segment SGI62069[] = {{&S20425[94], 5}, {NULL}};
const Segment SGI62070[] = {{&S20426[44], 12}, {NULL}};
const Segment SGI62071[] = {{&S20426[44], 39}, {NULL}};
const Segment SGI62072[] = {{&S20426[30], 6}, {NULL}};
const Segment SGI62073[] = {{&S20426[131], 3}, {NULL}};
const Segment SGI62074[] = {{&S20426[97], 5}, {NULL}};
const Segment SGI62075[] = {{&S20428[44], 4}, {NULL}};
const Segment SGI62076[] = {{&S20428[44], 16}, {NULL}};
const Segment SGI62077[] = {{&S20428[30], 6}, {NULL}};
const Segment SGI62078[] = {{&S20428[108], 3}, {NULL}};
const Segment SGI62079[] = {{&S20428[74], 5}, {NULL}};
const Segment SGI62080[] = {{&S20429[44], 8}, {NULL}};
const Segment SGI62081[] = {{&S20429[44], 20}, {NULL}};
const Segment SGI62082[] = {{&S20429[30], 6}, {NULL}};
const Segment SGI62083[] = {{&S20429[112], 3}, {NULL}};
const Segment SGI62084[] = {{&S20429[78], 5}, {NULL}};
const Segment SGI62085[] = {{&S20430[44], 23}, {NULL}};
const Segment SGI62086[] = {{&S20430[44], 35}, {NULL}};
const Segment SGI62087[] = {{&S20430[30], 6}, {NULL}};
const Segment SGI62088[] = {{&S20430[127], 3}, {NULL}};
const Segment SGI62089[] = {{&S20430[93], 5}, {NULL}};
const Segment SGI62090[] = {{&S20443[44], 22}, {NULL}};
const Segment SGI62091[] = {{&S20443[44], 34}, {NULL}};
const Segment SGI62092[] = {{&S20443[30], 6}, {NULL}};
const Segment SGI62093[] = {{&S20443[126], 3}, {NULL}};
const Segment SGI62094[] = {{&S20443[92], 5}, {NULL}};
const Segment SGI62095[] = {{&S20448[44], 21}, {NULL}};
const Segment SGI62096[] = {{&S20448[44], 33}, {NULL}};
const Segment SGI62097[] = {{&S20448[30], 6}, {NULL}};
const Segment SGI62098[] = {{&S20448[125], 3}, {NULL}};
const Segment SGI62099[] = {{&S20448[91], 5}, {NULL}};
const Segment SGI62100[] = {{&S20449[44], 19}, {NULL}};
const Segment SGI62101[] = {{&S20449[44], 31}, {NULL}};
const Segment SGI62102[] = {{&S20449[30], 6}, {NULL}};
const Segment SGI62103[] = {{&S20449[123], 3}, {NULL}};
const Segment SGI62104[] = {{&S20449[89], 5}, {NULL}};
const Segment SGI62105[] = {{&S20451[44], 18}, {NULL}};
const Segment SGI62106[] = {{&S20451[44], 30}, {NULL}};
const Segment SGI62107[] = {{&S20451[30], 6}, {NULL}};
const Segment SGI62108[] = {{&S20451[122], 3}, {NULL}};
const Segment SGI62109[] = {{&S20451[88], 5}, {NULL}};
const Segment SGI62110[] = {{&S20452[44], 18}, {NULL}};
const Segment SGI62111[] = {{&S20452[44], 30}, {NULL}};
const Segment SGI62112[] = {{&S20452[30], 6}, {NULL}};
const Segment SGI62113[] = {{&S20452[122], 3}, {NULL}};
const Segment SGI62114[] = {{&S20452[88], 5}, {NULL}};
const Segment SGI62115[] = {{&S20456[44], 22}, {NULL}};
const Segment SGI62116[] = {{&S20456[44], 34}, {NULL}};
const Segment SGI62117[] = {{&S20456[30], 6}, {NULL}};
const Segment SGI62118[] = {{&S20456[126], 3}, {NULL}};
const Segment SGI62119[] = {{&S20456[92], 5}, {NULL}};
const Segment SGI62120[] = {{&S20458[44], 10}, {NULL}};
const Segment SGI62121[] = {{&S20458[44], 25}, {NULL}};
const Segment SGI62122[] = {{&S20458[30], 6}, {NULL}};
const Segment SGI62123[] = {{&S20458[117], 3}, {NULL}};
const Segment SGI62124[] = {{&S20458[83], 5}, {NULL}};
const Segment SGI62125[] = {{&S20459[44], 24}, {NULL}};
const Segment SGI62126[] = {{&S20459[44], 36}, {NULL}};
const Segment SGI62127[] = {{&S20459[30], 6}, {NULL}};
const Segment SGI62128[] = {{&S20459[128], 3}, {NULL}};
const Segment SGI62129[] = {{&S20459[94], 5}, {NULL}};
const Segment SGI62130[] = {{&S20461[44], 12}, {NULL}};
const Segment SGI62131[] = {{&S20461[44], 24}, {NULL}};
const Segment SGI62132[] = {{&S20461[30], 6}, {NULL}};
const Segment SGI62133[] = {{&S20461[116], 3}, {NULL}};
const Segment SGI62134[] = {{&S20461[82], 5}, {NULL}};
const Segment SGI62135[] = {{&S20462[44], 19}, {NULL}};
const Segment SGI62136[] = {{&S20462[44], 31}, {NULL}};
const Segment SGI62137[] = {{&S20462[30], 6}, {NULL}};
const Segment SGI62138[] = {{&S20462[123], 3}, {NULL}};
const Segment SGI62139[] = {{&S20462[89], 5}, {NULL}};
const Segment SGI62140[] = {{&S20465[44], 16}, {NULL}};
const Segment SGI62141[] = {{&S20465[44], 28}, {NULL}};
const Segment SGI62142[] = {{&S20465[30], 6}, {NULL}};
const Segment SGI62143[] = {{&S7861[71], 6}, {NULL}};
const Segment SGI62144[] = {{&S20465[120], 3}, {NULL}};
const Segment SGI62145[] = {{&S7861[0], 6}, {NULL}};
const Segment SGI62146[] = {{&S20465[86], 5}, {NULL}};
const Segment SGI62147[] = {{&S20469[44], 14}, {NULL}};
const Segment SGI62148[] = {{&S20469[44], 26}, {NULL}};
const Segment SGI62149[] = {{&S20469[30], 6}, {NULL}};
const Segment SGI62150[] = {{&S20469[118], 3}, {NULL}};
const Segment SGI62151[] = {{&S20469[84], 5}, {NULL}};
const Segment SGI62152[] = {{&S20470[44], 16}, {NULL}};
const Segment SGI62153[] = {{&S20470[44], 28}, {NULL}};
const Segment SGI62154[] = {{&S20470[30], 6}, {NULL}};
const Segment SGI62155[] = {{&S20470[120], 3}, {NULL}};
const Segment SGI62156[] = {{&S20470[86], 5}, {NULL}};
const Segment SGI62157[] = {{&S20471[44], 20}, {NULL}};
const Segment SGI62158[] = {{&S20471[44], 32}, {NULL}};
const Segment SGI62159[] = {{&S20471[30], 6}, {NULL}};
const Segment SGI62160[] = {{&S20471[124], 3}, {NULL}};
const Segment SGI62161[] = {{&S20471[90], 5}, {NULL}};
const Segment SGI62162[] = {{&S20472[44], 18}, {NULL}};
const Segment SGI62163[] = {{&S20472[44], 30}, {NULL}};
const Segment SGI62164[] = {{&S20472[30], 6}, {NULL}};
const Segment SGI62165[] = {{&S20472[122], 3}, {NULL}};
const Segment SGI62166[] = {{&S20472[88], 5}, {NULL}};
const Segment SGI62167[] = {{&S20473[44], 32}, {NULL}};
const Segment SGI62168[] = {{&S20473[44], 44}, {NULL}};
const Segment SGI62169[] = {{&S20473[30], 6}, {NULL}};
const Segment SGI62170[] = {{&S20473[136], 3}, {NULL}};
const Segment SGI62171[] = {{&S20473[102], 5}, {NULL}};
const Segment SGI62172[] = {{&S20475[44], 25}, {NULL}};
const Segment SGI62173[] = {{&S20475[44], 37}, {NULL}};
const Segment SGI62174[] = {{&S20475[30], 6}, {NULL}};
const Segment SGI62175[] = {{&S20475[129], 3}, {NULL}};
const Segment SGI62176[] = {{&S20475[95], 5}, {NULL}};
const Segment SGI62177[] = {{&S20476[44], 22}, {NULL}};
const Segment SGI62178[] = {{&S20476[44], 34}, {NULL}};
const Segment SGI62179[] = {{&S20476[30], 6}, {NULL}};
const Segment SGI62180[] = {{&S20476[126], 3}, {NULL}};
const Segment SGI62181[] = {{&S20476[92], 5}, {NULL}};
const Segment SGI62182[] = {{&S20477[44], 23}, {NULL}};
const Segment SGI62183[] = {{&S20477[44], 35}, {NULL}};
const Segment SGI62184[] = {{&S20477[30], 6}, {NULL}};
const Segment SGI62185[] = {{&S20477[127], 3}, {NULL}};
const Segment SGI62186[] = {{&S20477[93], 5}, {NULL}};
const Segment SGI62187[] = {{&S20484[30], 6}, {NULL}};
const Segment SGI62188[] = {{&S20484[104], 3}, {NULL}};
const Segment SGI62189[] = {{&S20484[57], 6}, {NULL}};
const Segment SGI62190[] = {{&S20485[44], 12}, {NULL}};
const Segment SGI62191[] = {{&S20485[44], 24}, {NULL}};
const Segment SGI62192[] = {{&S20485[30], 6}, {NULL}};
const Segment SGI62193[] = {{&S20485[116], 3}, {NULL}};
const Segment SGI62194[] = {{&S20485[82], 5}, {NULL}};
const Segment SGI62195[] = {{&S20486[44], 16}, {NULL}};
const Segment SGI62196[] = {{&S20486[44], 37}, {NULL}};
const Segment SGI62197[] = {{&S20486[30], 6}, {NULL}};
const Segment SGI62198[] = {{&S20486[129], 3}, {NULL}};
const Segment SGI62199[] = {{&S20486[95], 5}, {NULL}};
const Segment SGI62200[] = {{&S20488[44], 9}, {NULL}};
const Segment SGI62201[] = {{&S20488[44], 21}, {NULL}};
const Segment SGI62202[] = {{&S20488[30], 6}, {NULL}};
const Segment SGI62203[] = {{&S20488[113], 3}, {NULL}};
const Segment SGI62204[] = {{&S20488[79], 5}, {NULL}};
const Segment SGI62205[] = {{&S20490[44], 20}, {NULL}};
const Segment SGI62206[] = {{&S20490[44], 32}, {NULL}};
const Segment SGI62207[] = {{&S20490[30], 6}, {NULL}};
const Segment SGI62208[] = {{&S20490[124], 3}, {NULL}};
const Segment SGI62209[] = {{&S20490[90], 5}, {NULL}};
const Segment SGI62210[] = {{&S20491[44], 8}, {NULL}};
const Segment SGI62211[] = {{&S7901[72], 6}, {NULL}};
const Segment SGI62212[] = {{&S20491[44], 20}, {NULL}};
const Segment SGI62213[] = {{&S20491[30], 6}, {NULL}};
const Segment SGI62214[] = {{&S7901[0], 7}, {NULL}};
const Segment SGI62215[] = {{&S20491[112], 3}, {NULL}};
const Segment SGI62216[] = {{&S20491[78], 5}, {NULL}};
const Segment SGI62217[] = {{&S20493[44], 8}, {NULL}};
const Segment SGI62218[] = {{&S20493[44], 20}, {NULL}};
const Segment SGI62219[] = {{&S20493[30], 6}, {NULL}};
const Segment SGI62220[] = {{&S20493[112], 3}, {NULL}};
const Segment SGI62221[] = {{&S20493[78], 5}, {NULL}};
const Segment SGI62222[] = {{&S20495[44], 24}, {NULL}};
const Segment SGI62223[] = {{&S20495[44], 36}, {NULL}};
const Segment SGI62224[] = {{&S20495[30], 6}, {NULL}};
const Segment SGI62225[] = {{&S20495[128], 3}, {NULL}};
const Segment SGI62226[] = {{&S20495[94], 5}, {NULL}};
const Segment SGI62227[] = {{&S7907[70], 6}, {NULL}};
const Segment SGI62228[] = {{&S20496[44], 10}, {NULL}};
const Segment SGI62229[] = {{&S7907[0], 5}, {NULL}};
const Segment SGI62230[] = {{&S20496[44], 28}, {NULL}};
const Segment SGI62231[] = {{&S20496[30], 6}, {NULL}};
const Segment SGI62232[] = {{&S20496[120], 3}, {NULL}};
const Segment SGI62233[] = {{&S20496[86], 5}, {NULL}};
const Segment SGI62234[] = {{&S20500[44], 7}, {NULL}};
const Segment SGI62235[] = {{&S20500[44], 24}, {NULL}};
const Segment SGI62236[] = {{&S20500[30], 6}, {NULL}};
const Segment SGI62237[] = {{&S20500[116], 3}, {NULL}};
const Segment SGI62238[] = {{&S20500[82], 5}, {NULL}};
const Segment SGI62239[] = {{&S20501[44], 11}, {NULL}};
const Segment SGI62240[] = {{&S20501[44], 23}, {NULL}};
const Segment SGI62241[] = {{&S20501[30], 6}, {NULL}};
const Segment SGI62242[] = {{&S20501[115], 3}, {NULL}};
const Segment SGI62243[] = {{&S20501[81], 5}, {NULL}};
const Segment SGI62244[] = {{&S20503[44], 7}, {NULL}};
const Segment SGI62245[] = {{&S20503[44], 19}, {NULL}};
const Segment SGI62246[] = {{&S20503[30], 6}, {NULL}};
const Segment SGI62247[] = {{&S20503[111], 3}, {NULL}};
const Segment SGI62248[] = {{&S20503[77], 5}, {NULL}};
const Segment SGI62249[] = {{&S7921[71], 6}, {NULL}};
const Segment SGI62250[] = {{&S7921[0], 6}, {NULL}};
const Segment SGI62251[] = {{&S7948[75], 6}, {NULL}};
const Segment SGI62252[] = {{&S7948[0], 10}, {NULL}};
const Segment SGI62253[] = {{&S7953[74], 6}, {NULL}};
const Segment SGI62254[] = {{&S7953[0], 9}, {NULL}};
const Segment SGI62255[] = {{&S20507[44], 14}, {NULL}};
const Segment SGI62256[] = {{&S20507[44], 25}, {NULL}};
const Segment SGI62257[] = {{&S20507[30], 6}, {NULL}};
const Segment SGI62258[] = {{&S20507[117], 3}, {NULL}};
const Segment SGI62259[] = {{&S20507[83], 5}, {NULL}};
const Segment SGI62260[] = {{&S20508[44], 15}, {NULL}};
const Segment SGI62261[] = {{&S20508[44], 27}, {NULL}};
const Segment SGI62262[] = {{&S20508[30], 6}, {NULL}};
const Segment SGI62263[] = {{&S20508[119], 3}, {NULL}};
const Segment SGI62264[] = {{&S20508[85], 5}, {NULL}};
const Segment SGI62265[] = {{&S20513[44], 30}, {NULL}};
const Segment SGI62266[] = {{&S20513[44], 42}, {NULL}};
const Segment SGI62267[] = {{&S20513[30], 6}, {NULL}};
const Segment SGI62268[] = {{&S20513[134], 3}, {NULL}};
const Segment SGI62269[] = {{&S20513[100], 5}, {NULL}};
const Segment SGI62270[] = {{&S20514[44], 7}, {NULL}};
const Segment SGI62271[] = {{&S20514[44], 19}, {NULL}};
const Segment SGI62272[] = {{&S20514[30], 6}, {NULL}};
const Segment SGI62273[] = {{&S20514[111], 3}, {NULL}};
const Segment SGI62274[] = {{&S20514[77], 5}, {NULL}};
const Segment SGI62275[] = {{&S20517[44], 9}, {NULL}};
const Segment SGI62276[] = {{&S20517[44], 21}, {NULL}};
const Segment SGI62277[] = {{&S20517[30], 6}, {NULL}};
const Segment SGI62278[] = {{&S20517[113], 3}, {NULL}};
const Segment SGI62279[] = {{&S20517[79], 5}, {NULL}};
const Segment SGI62280[] = {{&S20518[44], 7}, {NULL}};
const Segment SGI62281[] = {{&S20518[44], 19}, {NULL}};
const Segment SGI62282[] = {{&S20518[30], 6}, {NULL}};
const Segment SGI62283[] = {{&S20518[111], 3}, {NULL}};
const Segment SGI62284[] = {{&S20518[77], 5}, {NULL}};
const Segment SGI62285[] = {{&S20520[44], 7}, {NULL}};
const Segment SGI62286[] = {{&S20520[44], 19}, {NULL}};
const Segment SGI62287[] = {{&S20520[30], 6}, {NULL}};
const Segment SGI62288[] = {{&S20520[111], 3}, {NULL}};
const Segment SGI62289[] = {{&S20520[77], 5}, {NULL}};
const Segment SGI62290[] = {{&S7984[44], 7}, {NULL}};
const Segment SGI62291[] = {{&S7984[44], 28}, {NULL}};
const Segment SGI62292[] = {{&S7984[30], 6}, {NULL}};
const Segment SGI62293[] = {{&S7984[31], 3}, {NULL}};
const Segment SGI62294[] = {{&S7984[86], 5}, {NULL}};
const Segment SGI62295[] = {{&S7984[0], 13}, {NULL}};
const Segment SGI62296[] = {{&S8006[44], 5}, {NULL}};
const Segment SGI62297[] = {{&S8006[44], 17}, {NULL}};
const Segment SGI62298[] = {{&S8006[30], 6}, {NULL}};
const Segment SGI62299[] = {{&S8006[31], 3}, {NULL}};
const Segment SGI62300[] = {{&S8006[75], 5}, {NULL}};
const Segment SGI62301[] = {{&S8006[0], 13}, {NULL}};
const Segment SGI62302[] = {{&S8022[44], 75}, {NULL}};
const Segment SGI62303[] = {{&S8022[44], 96}, {NULL}};
const Segment SGI62304[] = {{&S8022[30], 6}, {NULL}};
const Segment SGI62305[] = {{&S8022[31], 3}, {NULL}};
const Segment SGI62306[] = {{&S8022[113], 5}, {NULL}};
const Segment SGI62307[] = {{&S8022[0], 13}, {NULL}};
const Segment SGI62308[] = {{&S8037[44], 71}, {NULL}};
const Segment SGI62309[] = {{&S8037[44], 92}, {NULL}};
const Segment SGI62310[] = {{&S8037[30], 6}, {NULL}};
const Segment SGI62311[] = {{&S8037[31], 3}, {NULL}};
const Segment SGI62312[] = {{&S8037[109], 5}, {NULL}};
const Segment SGI62313[] = {{&S8037[0], 13}, {NULL}};
const Segment SGI62314[] = {{&S8044[72], 6}, {NULL}};
const Segment SGI62315[] = {{&S8044[0], 7}, {NULL}};
const Segment SGI62316[] = {{&S8048[75], 6}, {NULL}};
const Segment SGI62317[] = {{&S8048[0], 10}, {NULL}};
const Segment SGI62318[] = {{&S8061[73], 6}, {NULL}};
const Segment SGI62319[] = {{&S8061[0], 8}, {NULL}};
const Segment SGI62320[] = {{&S8070[44], 7}, {NULL}};
const Segment SGI62321[] = {{&S8070[44], 28}, {NULL}};
const Segment SGI62322[] = {{&S8070[30], 6}, {NULL}};
const Segment SGI62323[] = {{&S8070[31], 3}, {NULL}};
const Segment SGI62324[] = {{&S8070[86], 5}, {NULL}};
const Segment SGI62325[] = {{&S8070[0], 13}, {NULL}};
const Segment SGI62326[] = {{&S8074[44], 9}, {NULL}};
const Segment SGI62327[] = {{&S20522[44], 6}, {NULL}};
const Segment SGI62328[] = {{&S8074[44], 30}, {NULL}};
const Segment SGI62329[] = {{&S8074[30], 6}, {NULL}};
const Segment SGI62330[] = {{&S20522[44], 18}, {NULL}};
const Segment SGI62331[] = {{&S20522[30], 6}, {NULL}};
const Segment SGI62332[] = {{&S8074[31], 3}, {NULL}};
const Segment SGI62333[] = {{&S20522[110], 3}, {NULL}};
const Segment SGI62334[] = {{&S8074[88], 5}, {NULL}};
const Segment SGI62335[] = {{&S20522[76], 5}, {NULL}};
const Segment SGI62336[] = {{&S8074[0], 13}, {NULL}};
const Segment SGI62337[] = {{&S8077[44], 8}, {NULL}};
const Segment SGI62338[] = {{&S20525[44], 21}, {NULL}};
const Segment SGI62339[] = {{&S8077[44], 29}, {NULL}};
const Segment SGI62340[] = {{&S8077[30], 6}, {NULL}};
const Segment SGI62341[] = {{&S20525[44], 33}, {NULL}};
const Segment SGI62342[] = {{&S20525[30], 6}, {NULL}};
const Segment SGI62343[] = {{&S8077[31], 3}, {NULL}};
const Segment SGI62344[] = {{&S20525[125], 3}, {NULL}};
const Segment SGI62345[] = {{&S8077[87], 5}, {NULL}};
const Segment SGI62346[] = {{&S20525[91], 5}, {NULL}};
const Segment SGI62347[] = {{&S8077[0], 13}, {NULL}};
const Segment SGI62348[] = {{&S20527[44], 9}, {NULL}};
const Segment SGI62349[] = {{&S20527[44], 21}, {NULL}};
const Segment SGI62350[] = {{&S20527[30], 6}, {NULL}};
const Segment SGI62351[] = {{&S20527[113], 3}, {NULL}};
const Segment SGI62352[] = {{&S20527[79], 5}, {NULL}};
const Segment SGI62353[] = {{&S20528[44], 20}, {NULL}};
const Segment SGI62354[] = {{&S20528[44], 32}, {NULL}};
const Segment SGI62355[] = {{&S20528[30], 6}, {NULL}};
const Segment SGI62356[] = {{&S20528[124], 3}, {NULL}};
const Segment SGI62357[] = {{&S20528[90], 5}, {NULL}};
const Segment SGI62358[] = {{&S20531[44], 13}, {NULL}};
const Segment SGI62359[] = {{&S20531[44], 25}, {NULL}};
const Segment SGI62360[] = {{&S20531[30], 6}, {NULL}};
const Segment SGI62361[] = {{&S20531[117], 3}, {NULL}};
const Segment SGI62362[] = {{&S20531[83], 5}, {NULL}};
const Segment SGI62363[] = {{&S20532[44], 16}, {NULL}};
const Segment SGI62364[] = {{&S20532[44], 33}, {NULL}};
const Segment SGI62365[] = {{&S20532[30], 6}, {NULL}};
const Segment SGI62366[] = {{&S20532[125], 3}, {NULL}};
const Segment SGI62367[] = {{&S20532[91], 5}, {NULL}};
const Segment SGI62368[] = {{&S8088[77], 6}, {NULL}};
const Segment SGI62369[] = {{&S8088[0], 12}, {NULL}};
const Segment SGI62370[] = {{&S20533[44], 5}, {NULL}};
const Segment SGI62371[] = {{&S20533[44], 17}, {NULL}};
const Segment SGI62372[] = {{&S20533[30], 6}, {NULL}};
const Segment SGI62373[] = {{&S20533[109], 3}, {NULL}};
const Segment SGI62374[] = {{&S20533[75], 5}, {NULL}};
const Segment SGI62375[] = {{&S20534[44], 19}, {NULL}};
const Segment SGI62376[] = {{&S20534[44], 30}, {NULL}};
const Segment SGI62377[] = {{&S20534[30], 6}, {NULL}};
const Segment SGI62378[] = {{&S20534[122], 3}, {NULL}};
const Segment SGI62379[] = {{&S20534[88], 5}, {NULL}};
const Segment SGI62380[] = {{&S20535[44], 16}, {NULL}};
const Segment SGI62381[] = {{&S20535[44], 39}, {NULL}};
const Segment SGI62382[] = {{&S20535[30], 6}, {NULL}};
const Segment SGI62383[] = {{&S20535[131], 3}, {NULL}};
const Segment SGI62384[] = {{&S20535[97], 5}, {NULL}};
const Segment SGI62385[] = {{&S20538[44], 10}, {NULL}};
const Segment SGI62386[] = {{&S20538[44], 22}, {NULL}};
const Segment SGI62387[] = {{&S20538[30], 6}, {NULL}};
const Segment SGI62388[] = {{&S20538[114], 3}, {NULL}};
const Segment SGI62389[] = {{&S20538[80], 5}, {NULL}};
const Segment SGI62390[] = {{&S20539[44], 13}, {NULL}};
const Segment SGI62391[] = {{&S20539[44], 25}, {NULL}};
const Segment SGI62392[] = {{&S20539[30], 6}, {NULL}};
const Segment SGI62393[] = {{&S20539[117], 3}, {NULL}};
const Segment SGI62394[] = {{&S20539[83], 5}, {NULL}};
const Segment SGI62395[] = {{&S20543[44], 10}, {NULL}};
const Segment SGI62396[] = {{&S20543[44], 22}, {NULL}};
const Segment SGI62397[] = {{&S20543[30], 6}, {NULL}};
const Segment SGI62398[] = {{&S20543[114], 3}, {NULL}};
const Segment SGI62399[] = {{&S20543[80], 5}, {NULL}};
const Segment SGI62400[] = {{&S20544[44], 5}, {NULL}};
const Segment SGI62401[] = {{&S20544[44], 26}, {NULL}};
const Segment SGI62402[] = {{&S20544[30], 6}, {NULL}};
const Segment SGI62403[] = {{&S8117[44], 6}, {NULL}};
const Segment SGI62404[] = {{&S8117[44], 18}, {NULL}};
const Segment SGI62405[] = {{&S8117[30], 6}, {NULL}};
const Segment SGI62406[] = {{&S8117[31], 3}, {NULL}};
const Segment SGI62407[] = {{&S8117[76], 5}, {NULL}};
const Segment SGI62408[] = {{&S8117[0], 13}, {NULL}};
const Segment SGI62409[] = {{&S8159[69], 6}, {NULL}};
const Segment SGI62410[] = {{&S8159[0], 4}, {NULL}};
const Segment SGI62411[] = {{&S8160[73], 6}, {NULL}};
const Segment SGI62412[] = {{&S8160[0], 8}, {NULL}};
const Segment SGI62413[] = {{&S8171[44], 10}, {NULL}};
const Segment SGI62414[] = {{&S8171[44], 22}, {NULL}};
const Segment SGI62415[] = {{&S8171[30], 6}, {NULL}};
const Segment SGI62416[] = {{&S8171[31], 3}, {NULL}};
const Segment SGI62417[] = {{&S8171[80], 5}, {NULL}};
const Segment SGI62418[] = {{&S8171[0], 13}, {NULL}};
const Segment SGI62419[] = {{&S20544[118], 3}, {NULL}};
const Segment SGI62420[] = {{&S20544[84], 5}, {NULL}};
const Segment SGI62421[] = {{&S20545[44], 13}, {NULL}};
const Segment SGI62422[] = {{&S20545[44], 25}, {NULL}};
const Segment SGI62423[] = {{&S20545[30], 6}, {NULL}};
const Segment SGI62424[] = {{&S20545[117], 3}, {NULL}};
const Segment SGI62425[] = {{&S20545[83], 5}, {NULL}};
const Segment SGI62426[] = {{&S20546[44], 10}, {NULL}};
const Segment SGI62427[] = {{&S20546[44], 27}, {NULL}};
const Segment SGI62428[] = {{&S20546[30], 6}, {NULL}};
const Segment SGI62429[] = {{&S20546[119], 3}, {NULL}};
const Segment SGI62430[] = {{&S20546[85], 5}, {NULL}};
const Segment SGI62431[] = {{&S20547[44], 9}, {NULL}};
const Segment SGI62432[] = {{&S20547[44], 21}, {NULL}};
const Segment SGI62433[] = {{&S20547[30], 6}, {NULL}};
const Segment SGI62434[] = {{&S20547[113], 3}, {NULL}};
const Segment SGI62435[] = {{&S20547[79], 5}, {NULL}};
const Segment SGI62436[] = {{&S20549[44], 4}, {NULL}};
const Segment SGI62437[] = {{&S20549[44], 16}, {NULL}};
const Segment SGI62438[] = {{&S20549[30], 6}, {NULL}};
const Segment SGI62439[] = {{&S20549[108], 3}, {NULL}};
const Segment SGI62440[] = {{&S20549[74], 5}, {NULL}};
const Segment SGI62441[] = {{&S20550[44], 7}, {NULL}};
const Segment SGI62442[] = {{&S20550[44], 19}, {NULL}};
const Segment SGI62443[] = {{&S20550[30], 6}, {NULL}};
const Segment SGI62444[] = {{&S20550[111], 3}, {NULL}};
const Segment SGI62445[] = {{&S20550[77], 5}, {NULL}};
const Segment SGI62446[] = {{&S20552[44], 7}, {NULL}};
const Segment SGI62447[] = {{&S20552[44], 19}, {NULL}};
const Segment SGI62448[] = {{&S20552[30], 6}, {NULL}};
const Segment SGI62449[] = {{&S20552[111], 3}, {NULL}};
const Segment SGI62450[] = {{&S20552[77], 5}, {NULL}};
const Segment SGI62451[] = {{&S20553[44], 9}, {NULL}};
const Segment SGI62452[] = {{&S20553[44], 21}, {NULL}};
const Segment SGI62453[] = {{&S20553[30], 6}, {NULL}};
const Segment SGI62454[] = {{&S20553[113], 3}, {NULL}};
const Segment SGI62455[] = {{&S20553[79], 5}, {NULL}};
const Segment SGI62456[] = {{&S20554[44], 4}, {NULL}};
const Segment SGI62457[] = {{&S20554[44], 16}, {NULL}};
const Segment SGI62458[] = {{&S20554[30], 6}, {NULL}};
const Segment SGI62459[] = {{&S20554[108], 3}, {NULL}};
const Segment SGI62460[] = {{&S20554[74], 5}, {NULL}};
const Segment SGI62461[] = {{&S20557[44], 29}, {NULL}};
const Segment SGI62462[] = {{&S20557[44], 41}, {NULL}};
const Segment SGI62463[] = {{&S20557[30], 6}, {NULL}};
const Segment SGI62464[] = {{&S20557[133], 3}, {NULL}};
const Segment SGI62465[] = {{&S20557[99], 5}, {NULL}};
const Segment SGI62466[] = {{&S20559[44], 20}, {NULL}};
const Segment SGI62467[] = {{&S20559[44], 32}, {NULL}};
const Segment SGI62468[] = {{&S20559[30], 6}, {NULL}};
const Segment SGI62469[] = {{&S20559[124], 3}, {NULL}};
const Segment SGI62470[] = {{&S20559[90], 5}, {NULL}};
const Segment SGI62471[] = {{&S20561[44], 12}, {NULL}};
const Segment SGI62472[] = {{&S20561[44], 24}, {NULL}};
const Segment SGI62473[] = {{&S20561[30], 6}, {NULL}};
const Segment SGI62474[] = {{&S20561[116], 3}, {NULL}};
const Segment SGI62475[] = {{&S20561[82], 5}, {NULL}};
const Segment SGI62476[] = {{&S8243[44], 9}, {NULL}};
const Segment SGI62477[] = {{&S8243[44], 30}, {NULL}};
const Segment SGI62478[] = {{&S8243[30], 6}, {NULL}};
const Segment SGI62479[] = {{&S8243[31], 3}, {NULL}};
const Segment SGI62480[] = {{&S8243[88], 5}, {NULL}};
const Segment SGI62481[] = {{&S8243[0], 13}, {NULL}};
const Segment SGI62482[] = {{&S20562[44], 15}, {NULL}};
const Segment SGI62483[] = {{&S20562[44], 27}, {NULL}};
const Segment SGI62484[] = {{&S20562[30], 6}, {NULL}};
const Segment SGI62485[] = {{&S20562[119], 3}, {NULL}};
const Segment SGI62486[] = {{&S20562[85], 5}, {NULL}};
const Segment SGI62487[] = {{&S20563[44], 18}, {NULL}};
const Segment SGI62488[] = {{&S20563[44], 30}, {NULL}};
const Segment SGI62489[] = {{&S20563[30], 6}, {NULL}};
const Segment SGI62490[] = {{&S20563[122], 3}, {NULL}};
const Segment SGI62491[] = {{&S20563[88], 5}, {NULL}};
const Segment SGI62492[] = {{&S8245[71], 6}, {NULL}};
const Segment SGI62493[] = {{&S8245[0], 6}, {NULL}};
const Segment SGI62494[] = {{&S20564[44], 23}, {NULL}};
const Segment SGI62495[] = {{&S20564[44], 35}, {NULL}};
const Segment SGI62496[] = {{&S20564[30], 6}, {NULL}};
const Segment SGI62497[] = {{&S20564[127], 3}, {NULL}};
const Segment SGI62498[] = {{&S20564[93], 5}, {NULL}};
const Segment SGI62499[] = {{&S20565[44], 6}, {NULL}};
const Segment SGI62500[] = {{&S20565[44], 27}, {NULL}};
const Segment SGI62501[] = {{&S20565[30], 6}, {NULL}};
const Segment SGI62502[] = {{&S20565[119], 3}, {NULL}};
const Segment SGI62503[] = {{&S20565[85], 5}, {NULL}};
const Segment SGI62504[] = {{&S20566[44], 21}, {NULL}};
const Segment SGI62505[] = {{&S20566[44], 33}, {NULL}};
const Segment SGI62506[] = {{&S20566[30], 6}, {NULL}};
const Segment SGI62507[] = {{&S20566[125], 3}, {NULL}};
const Segment SGI62508[] = {{&S20566[91], 5}, {NULL}};
const Segment SGI62509[] = {{&S20567[44], 18}, {NULL}};
const Segment SGI62510[] = {{&S20567[44], 30}, {NULL}};
const Segment SGI62511[] = {{&S20567[30], 6}, {NULL}};
const Segment SGI62512[] = {{&S20567[122], 3}, {NULL}};
const Segment SGI62513[] = {{&S20567[88], 5}, {NULL}};
const Segment SGI62514[] = {{&S20568[44], 26}, {NULL}};
const Segment SGI62515[] = {{&S20568[44], 38}, {NULL}};
const Segment SGI62516[] = {{&S20568[30], 6}, {NULL}};
const Segment SGI62517[] = {{&S20568[130], 3}, {NULL}};
const Segment SGI62518[] = {{&S20568[96], 5}, {NULL}};
const Segment SGI62519[] = {{&S20569[44], 16}, {NULL}};
const Segment SGI62520[] = {{&S20569[44], 28}, {NULL}};
const Segment SGI62521[] = {{&S20569[30], 6}, {NULL}};
const Segment SGI62522[] = {{&S20569[120], 3}, {NULL}};
const Segment SGI62523[] = {{&S20569[86], 5}, {NULL}};
const Segment SGI62524[] = {{&S20570[44], 18}, {NULL}};
const Segment SGI62525[] = {{&S20570[44], 30}, {NULL}};
const Segment SGI62526[] = {{&S20570[30], 6}, {NULL}};
const Segment SGI62527[] = {{&S20570[122], 3}, {NULL}};
const Segment SGI62528[] = {{&S20570[88], 5}, {NULL}};
const Segment SGI62529[] = {{&S20572[44], 12}, {NULL}};
const Segment SGI62530[] = {{&S20572[44], 24}, {NULL}};
const Segment SGI62531[] = {{&S20572[30], 6}, {NULL}};
const Segment SGI62532[] = {{&S20572[116], 3}, {NULL}};
const Segment SGI62533[] = {{&S20572[82], 5}, {NULL}};
const Segment SGI62534[] = {{&S20573[44], 22}, {NULL}};
const Segment SGI62535[] = {{&S20573[44], 34}, {NULL}};
const Segment SGI62536[] = {{&S20573[30], 6}, {NULL}};
const Segment SGI62537[] = {{&S20573[126], 3}, {NULL}};
const Segment SGI62538[] = {{&S20573[92], 5}, {NULL}};
const Segment SGI62539[] = {{&S20574[44], 20}, {NULL}};
const Segment SGI62540[] = {{&S20574[44], 32}, {NULL}};
const Segment SGI62541[] = {{&S20574[30], 6}, {NULL}};
const Segment SGI62542[] = {{&S20574[124], 3}, {NULL}};
const Segment SGI62543[] = {{&S20574[90], 5}, {NULL}};
const Segment SGI62544[] = {{&S20575[44], 19}, {NULL}};
const Segment SGI62545[] = {{&S20575[44], 31}, {NULL}};
const Segment SGI62546[] = {{&S20575[30], 6}, {NULL}};
const Segment SGI62547[] = {{&S20575[123], 3}, {NULL}};
const Segment SGI62548[] = {{&S20575[89], 5}, {NULL}};
const Segment SGI62549[] = {{&S20576[44], 17}, {NULL}};
const Segment SGI62550[] = {{&S20576[44], 42}, {NULL}};
const Segment SGI62551[] = {{&S20576[30], 6}, {NULL}};
const Segment SGI62552[] = {{&S20576[134], 3}, {NULL}};
const Segment SGI62553[] = {{&S20576[100], 5}, {NULL}};
const Segment SGI62554[] = {{&S20578[44], 21}, {NULL}};
const Segment SGI62555[] = {{&S20578[44], 33}, {NULL}};
const Segment SGI62556[] = {{&S20578[30], 6}, {NULL}};
const Segment SGI62557[] = {{&S20578[125], 3}, {NULL}};
const Segment SGI62558[] = {{&S20578[91], 5}, {NULL}};
const Segment SGI62559[] = {{&S20580[44], 18}, {NULL}};
const Segment SGI62560[] = {{&S20580[44], 30}, {NULL}};
const Segment SGI62561[] = {{&S20580[30], 6}, {NULL}};
const Segment SGI62562[] = {{&S20580[122], 3}, {NULL}};
const Segment SGI62563[] = {{&S20580[88], 5}, {NULL}};
const Segment SGI62564[] = {{&S20584[38], 9}, {NULL}};
const Segment SGI62565[] = {{&S20584[31], 6}, {NULL}};
const Segment SGI62566[] = {{&S8282[76], 6}, {NULL}};
const Segment SGI62567[] = {{&S8282[0], 11}, {NULL}};
const Segment SGI62568[] = {{&S20585[44], 6}, {NULL}};
const Segment SGI62569[] = {{&S20585[44], 18}, {NULL}};
const Segment SGI62570[] = {{&S20585[30], 6}, {NULL}};
const Segment SGI62571[] = {{&S20585[110], 3}, {NULL}};
const Segment SGI62572[] = {{&S20585[76], 5}, {NULL}};
const Segment SGI62573[] = {{&S20586[44], 16}, {NULL}};
const Segment SGI62574[] = {{&S20586[44], 28}, {NULL}};
const Segment SGI62575[] = {{&S20586[30], 6}, {NULL}};
const Segment SGI62576[] = {{&S20586[120], 3}, {NULL}};
const Segment SGI62577[] = {{&S20586[86], 5}, {NULL}};
const Segment SGI62578[] = {{&S20587[44], 19}, {NULL}};
const Segment SGI62579[] = {{&S20587[44], 31}, {NULL}};
const Segment SGI62580[] = {{&S20587[30], 6}, {NULL}};
const Segment SGI62581[] = {{&S20587[123], 3}, {NULL}};
const Segment SGI62582[] = {{&S20587[89], 5}, {NULL}};
const Segment SGI62583[] = {{&S20588[44], 27}, {NULL}};
const Segment SGI62584[] = {{&S20588[44], 39}, {NULL}};
const Segment SGI62585[] = {{&S20588[30], 6}, {NULL}};
const Segment SGI62586[] = {{&S20588[131], 3}, {NULL}};
const Segment SGI62587[] = {{&S20588[97], 5}, {NULL}};
const Segment SGI62588[] = {{&S20590[38], 22}, {NULL}};
const Segment SGI62589[] = {{&S20590[31], 6}, {NULL}};
const Segment SGI62590[] = {{&S20592[44], 10}, {NULL}};
const Segment SGI62591[] = {{&S20592[44], 22}, {NULL}};
const Segment SGI62592[] = {{&S20592[30], 6}, {NULL}};
const Segment SGI62593[] = {{&S20592[114], 3}, {NULL}};
const Segment SGI62594[] = {{&S20592[80], 5}, {NULL}};
const Segment SGI62595[] = {{&S20593[38], 8}, {NULL}};
const Segment SGI62596[] = {{&S20593[31], 6}, {NULL}};
const Segment SGI62597[] = {{&S20594[44], 7}, {NULL}};
const Segment SGI62598[] = {{&S20594[44], 19}, {NULL}};
const Segment SGI62599[] = {{&S20594[30], 6}, {NULL}};
const Segment SGI62600[] = {{&S20594[111], 3}, {NULL}};
const Segment SGI62601[] = {{&S20594[77], 5}, {NULL}};
const Segment SGI62602[] = {{&S8297[75], 6}, {NULL}};
const Segment SGI62603[] = {{&S8297[0], 10}, {NULL}};
const Segment SGI62604[] = {{&S20596[44], 18}, {NULL}};
const Segment SGI62605[] = {{&S20596[44], 30}, {NULL}};
const Segment SGI62606[] = {{&S20596[30], 6}, {NULL}};
const Segment SGI62607[] = {{&S20596[122], 3}, {NULL}};
const Segment SGI62608[] = {{&S20596[88], 5}, {NULL}};
const Segment SGI62609[] = {{&S20599[44], 16}, {NULL}};
const Segment SGI62610[] = {{&S20599[44], 28}, {NULL}};
const Segment SGI62611[] = {{&S20599[30], 6}, {NULL}};
const Segment SGI62612[] = {{&S20599[120], 3}, {NULL}};
const Segment SGI62613[] = {{&S20599[86], 5}, {NULL}};
const Segment SGI62614[] = {{&S20601[44], 25}, {NULL}};
const Segment SGI62615[] = {{&S20601[44], 37}, {NULL}};
const Segment SGI62616[] = {{&S20601[30], 6}, {NULL}};
const Segment SGI62617[] = {{&S20601[129], 3}, {NULL}};
const Segment SGI62618[] = {{&S20601[95], 5}, {NULL}};
const Segment SGI62619[] = {{&S20606[44], 21}, {NULL}};
const Segment SGI62620[] = {{&S20606[44], 33}, {NULL}};
const Segment SGI62621[] = {{&S20606[30], 6}, {NULL}};
const Segment SGI62622[] = {{&S20606[125], 3}, {NULL}};
const Segment SGI62623[] = {{&S20606[91], 5}, {NULL}};
const Segment SGI62624[] = {{&S20607[44], 10}, {NULL}};
const Segment SGI62625[] = {{&S20607[44], 22}, {NULL}};
const Segment SGI62626[] = {{&S20607[30], 6}, {NULL}};
const Segment SGI62627[] = {{&S20607[114], 3}, {NULL}};
const Segment SGI62628[] = {{&S20607[80], 5}, {NULL}};
const Segment SGI62629[] = {{&S20608[44], 16}, {NULL}};
const Segment SGI62630[] = {{&S20608[44], 28}, {NULL}};
const Segment SGI62631[] = {{&S20608[30], 6}, {NULL}};
const Segment SGI62632[] = {{&S20608[120], 3}, {NULL}};
const Segment SGI62633[] = {{&S20608[86], 5}, {NULL}};
const Segment SGI62634[] = {{&S20611[44], 19}, {NULL}};
const Segment SGI62635[] = {{&S20611[44], 31}, {NULL}};
const Segment SGI62636[] = {{&S20611[30], 6}, {NULL}};
const Segment SGI62637[] = {{&S20611[123], 3}, {NULL}};
const Segment SGI62638[] = {{&S20611[89], 5}, {NULL}};
const Segment SGI62639[] = {{&S20612[44], 17}, {NULL}};
const Segment SGI62640[] = {{&S20612[44], 29}, {NULL}};
const Segment SGI62641[] = {{&S20612[30], 6}, {NULL}};
const Segment SGI62642[] = {{&S20612[121], 3}, {NULL}};
const Segment SGI62643[] = {{&S20612[87], 5}, {NULL}};
const Segment SGI62644[] = {{&S20614[44], 22}, {NULL}};
const Segment SGI62645[] = {{&S20614[44], 34}, {NULL}};
const Segment SGI62646[] = {{&S20614[30], 6}, {NULL}};
const Segment SGI62647[] = {{&S20614[126], 3}, {NULL}};
const Segment SGI62648[] = {{&S20614[92], 5}, {NULL}};
const Segment SGI62649[] = {{&S20616[44], 19}, {NULL}};
const Segment SGI62650[] = {{&S20616[30], 6}, {NULL}};
const Segment SGI62651[] = {{&S20616[125], 3}, {NULL}};
const Segment SGI62652[] = {{&S20616[91], 5}, {NULL}};
const Segment SGI62653[] = {{&S20617[44], 18}, {NULL}};
const Segment SGI62654[] = {{&S20617[44], 30}, {NULL}};
const Segment SGI62655[] = {{&S20617[30], 6}, {NULL}};
const Segment SGI62656[] = {{&S20617[122], 3}, {NULL}};
const Segment SGI62657[] = {{&S20617[88], 5}, {NULL}};
const Segment SGI62658[] = {{&S20618[44], 23}, {NULL}};
const Segment SGI62659[] = {{&S20618[44], 35}, {NULL}};
const Segment SGI62660[] = {{&S20618[30], 6}, {NULL}};
const Segment SGI62661[] = {{&S20618[127], 3}, {NULL}};
const Segment SGI62662[] = {{&S20618[93], 5}, {NULL}};
const Segment SGI62663[] = {{&S20619[42], 17}, {NULL}};
const Segment SGI62664[] = {{&S20619[30], 6}, {NULL}};
const Segment SGI62665[] = {{&S20619[119], 3}, {NULL}};
const Segment SGI62666[] = {{&S20619[85], 5}, {NULL}};
const Segment SGI62667[] = {{&S20620[44], 26}, {NULL}};
const Segment SGI62668[] = {{&S20620[44], 38}, {NULL}};
const Segment SGI62669[] = {{&S20620[30], 6}, {NULL}};
const Segment SGI62670[] = {{&S20620[130], 3}, {NULL}};
const Segment SGI62671[] = {{&S20620[96], 5}, {NULL}};
const Segment SGI62672[] = {{&S20622[44], 17}, {NULL}};
const Segment SGI62673[] = {{&S20622[44], 29}, {NULL}};
const Segment SGI62674[] = {{&S20622[30], 6}, {NULL}};
const Segment SGI62675[] = {{&S20622[121], 3}, {NULL}};
const Segment SGI62676[] = {{&S20622[87], 5}, {NULL}};
const Segment SGI62677[] = {{&S20623[44], 21}, {NULL}};
const Segment SGI62678[] = {{&S20623[44], 33}, {NULL}};
const Segment SGI62679[] = {{&S20623[30], 6}, {NULL}};
const Segment SGI62680[] = {{&S20623[125], 3}, {NULL}};
const Segment SGI62681[] = {{&S20623[91], 5}, {NULL}};
const Segment SGI62682[] = {{&S20624[44], 22}, {NULL}};
const Segment SGI62683[] = {{&S20624[44], 34}, {NULL}};
const Segment SGI62684[] = {{&S20624[30], 6}, {NULL}};
const Segment SGI62685[] = {{&S20624[126], 3}, {NULL}};
const Segment SGI62686[] = {{&S20624[92], 5}, {NULL}};
const Segment SGI62687[] = {{&S8348[44], 7}, {NULL}};
const Segment SGI62688[] = {{&S8348[44], 28}, {NULL}};
const Segment SGI62689[] = {{&S8348[30], 6}, {NULL}};
const Segment SGI62690[] = {{&S8348[31], 3}, {NULL}};
const Segment SGI62691[] = {{&S8348[86], 5}, {NULL}};
const Segment SGI62692[] = {{&S8348[0], 13}, {NULL}};
const Segment SGI62693[] = {{&S8354[76], 6}, {NULL}};
const Segment SGI62694[] = {{&S8354[0], 11}, {NULL}};
const Segment SGI62695[] = {{&S8368[44], 4}, {NULL}};
const Segment SGI62696[] = {{&S8368[44], 16}, {NULL}};
const Segment SGI62697[] = {{&S8368[30], 6}, {NULL}};
const Segment SGI62698[] = {{&S8368[31], 3}, {NULL}};
const Segment SGI62699[] = {{&S8368[74], 5}, {NULL}};
const Segment SGI62700[] = {{&S8368[0], 13}, {NULL}};
const Segment SGI62701[] = {{&S8373[44], 75}, {NULL}};
const Segment SGI62702[] = {{&S8373[44], 96}, {NULL}};
const Segment SGI62703[] = {{&S8373[30], 6}, {NULL}};
const Segment SGI62704[] = {{&S8373[31], 3}, {NULL}};
const Segment SGI62705[] = {{&S8373[113], 5}, {NULL}};
const Segment SGI62706[] = {{&S8373[0], 13}, {NULL}};
const Segment SGI62707[] = {{&S8390[71], 6}, {NULL}};
const Segment SGI62708[] = {{&S8390[0], 6}, {NULL}};
const Segment SGI62709[] = {{&S5807[44], 25}, {NULL}};
const Segment SGI62710[] = {{&S5807[44], 37}, {NULL}};
const Segment SGI62711[] = {{&S5807[30], 6}, {NULL}};
const Segment SGI62712[] = {{&S5807[31], 3}, {NULL}};
const Segment SGI62713[] = {{&S5807[95], 6}, {NULL}};
const Segment SGI62714[] = {{&S5808[44], 7}, {NULL}};
const Segment SGI62715[] = {{&S5808[44], 19}, {NULL}};
const Segment SGI62716[] = {{&S5808[30], 6}, {NULL}};
const Segment SGI62717[] = {{&S5808[31], 3}, {NULL}};
const Segment SGI62718[] = {{&S5808[77], 6}, {NULL}};
const Segment SGI62719[] = {{&S5809[44], 9}, {NULL}};
const Segment SGI62720[] = {{&S5809[44], 21}, {NULL}};
const Segment SGI62721[] = {{&S5809[30], 6}, {NULL}};
const Segment SGI62722[] = {{&S5809[31], 3}, {NULL}};
const Segment SGI62723[] = {{&S5809[79], 6}, {NULL}};
const Segment SGI62724[] = {{&S5810[44], 12}, {NULL}};
const Segment SGI62725[] = {{&S5810[44], 24}, {NULL}};
const Segment SGI62726[] = {{&S5810[30], 6}, {NULL}};
const Segment SGI62727[] = {{&S5810[31], 3}, {NULL}};
const Segment SGI62728[] = {{&S5810[82], 6}, {NULL}};
const Segment SGI62729[] = {{&S5811[44], 7}, {NULL}};
const Segment SGI62730[] = {{&S5811[44], 19}, {NULL}};
const Segment SGI62731[] = {{&S5811[30], 6}, {NULL}};
const Segment SGI62732[] = {{&S5811[31], 3}, {NULL}};
const Segment SGI62733[] = {{&S5811[77], 6}, {NULL}};
const Segment SGI62734[] = {{&S8409[69], 6}, {NULL}};
const Segment SGI62735[] = {{&S8409[0], 4}, {NULL}};
const Segment SGI62736[] = {{&S5812[44], 9}, {NULL}};
const Segment SGI62737[] = {{&S5812[44], 21}, {NULL}};
const Segment SGI62738[] = {{&S5812[30], 6}, {NULL}};
const Segment SGI62739[] = {{&S5812[31], 3}, {NULL}};
const Segment SGI62740[] = {{&S5812[79], 6}, {NULL}};
const Segment SGI62741[] = {{&S5813[44], 21}, {NULL}};
const Segment SGI62742[] = {{&S5813[44], 33}, {NULL}};
const Segment SGI62743[] = {{&S5813[30], 6}, {NULL}};
const Segment SGI62744[] = {{&S5813[31], 3}, {NULL}};
const Segment SGI62745[] = {{&S5813[91], 6}, {NULL}};
const Segment SGI62746[] = {{&S5814[44], 10}, {NULL}};
const Segment SGI62747[] = {{&S5814[44], 22}, {NULL}};
const Segment SGI62748[] = {{&S5814[30], 6}, {NULL}};
const Segment SGI62749[] = {{&S5814[31], 3}, {NULL}};
const Segment SGI62750[] = {{&S5814[80], 6}, {NULL}};
const Segment SGI62751[] = {{&S5815[44], 7}, {NULL}};
const Segment SGI62752[] = {{&S5815[44], 19}, {NULL}};
const Segment SGI62753[] = {{&S5815[30], 6}, {NULL}};
const Segment SGI62754[] = {{&S5815[31], 3}, {NULL}};
const Segment SGI62755[] = {{&S5815[77], 6}, {NULL}};
const Segment SGI62756[] = {{&S5816[44], 12}, {NULL}};
const Segment SGI62757[] = {{&S5816[44], 24}, {NULL}};
const Segment SGI62758[] = {{&S5816[30], 6}, {NULL}};
const Segment SGI62759[] = {{&S5816[31], 3}, {NULL}};
const Segment SGI62760[] = {{&S5816[82], 6}, {NULL}};
const Segment SGI62761[] = {{&S5818[44], 13}, {NULL}};
const Segment SGI62762[] = {{&S5818[44], 25}, {NULL}};
const Segment SGI62763[] = {{&S5818[30], 6}, {NULL}};
const Segment SGI62764[] = {{&S5818[31], 3}, {NULL}};
const Segment SGI62765[] = {{&S5818[83], 6}, {NULL}};
const Segment SGI62766[] = {{&S5819[44], 4}, {NULL}};
const Segment SGI62767[] = {{&S5819[44], 16}, {NULL}};
const Segment SGI62768[] = {{&S5819[30], 6}, {NULL}};
const Segment SGI62769[] = {{&S5819[31], 3}, {NULL}};
const Segment SGI62770[] = {{&S5819[74], 6}, {NULL}};
const Segment SGI62771[] = {{&S5820[44], 17}, {NULL}};
const Segment SGI62772[] = {{&S5820[44], 29}, {NULL}};
const Segment SGI62773[] = {{&S5820[30], 6}, {NULL}};
const Segment SGI62774[] = {{&S5820[31], 3}, {NULL}};
const Segment SGI62775[] = {{&S5820[87], 6}, {NULL}};
const Segment SGI62776[] = {{&S5821[44], 9}, {NULL}};
const Segment SGI62777[] = {{&S5821[44], 21}, {NULL}};
const Segment SGI62778[] = {{&S5821[30], 6}, {NULL}};
const Segment SGI62779[] = {{&S5821[31], 3}, {NULL}};
const Segment SGI62780[] = {{&S5821[79], 6}, {NULL}};
const Segment SGI62781[] = {{&S8430[44], 7}, {NULL}};
const Segment SGI62782[] = {{&S8430[44], 19}, {NULL}};
const Segment SGI62783[] = {{&S8430[30], 6}, {NULL}};
const Segment SGI62784[] = {{&S8430[31], 3}, {NULL}};
const Segment SGI62785[] = {{&S8430[77], 5}, {NULL}};
const Segment SGI62786[] = {{&S8430[0], 13}, {NULL}};
const Segment SGI62787[] = {{&S5824[44], 13}, {NULL}};
const Segment SGI62788[] = {{&S5824[44], 25}, {NULL}};
const Segment SGI62789[] = {{&S5824[30], 6}, {NULL}};
const Segment SGI62790[] = {{&S5824[31], 3}, {NULL}};
const Segment SGI62791[] = {{&S5824[83], 6}, {NULL}};
const Segment SGI62792[] = {{&S5825[44], 7}, {NULL}};
const Segment SGI62793[] = {{&S5825[44], 19}, {NULL}};
const Segment SGI62794[] = {{&S5825[30], 6}, {NULL}};
const Segment SGI62795[] = {{&S5825[31], 3}, {NULL}};
const Segment SGI62796[] = {{&S5825[77], 6}, {NULL}};
const Segment SGI62797[] = {{&S5826[44], 6}, {NULL}};
const Segment SGI62798[] = {{&S5826[44], 18}, {NULL}};
const Segment SGI62799[] = {{&S5826[30], 6}, {NULL}};
const Segment SGI62800[] = {{&S5826[31], 3}, {NULL}};
const Segment SGI62801[] = {{&S5826[76], 6}, {NULL}};
const Segment SGI62802[] = {{&S5827[44], 9}, {NULL}};
const Segment SGI62803[] = {{&S5827[44], 21}, {NULL}};
const Segment SGI62804[] = {{&S5827[30], 6}, {NULL}};
const Segment SGI62805[] = {{&S5827[31], 3}, {NULL}};
const Segment SGI62806[] = {{&S5827[79], 6}, {NULL}};
const Segment SGI62807[] = {{&S5828[44], 12}, {NULL}};
const Segment SGI62808[] = {{&S5828[44], 24}, {NULL}};
const Segment SGI62809[] = {{&S5828[30], 6}, {NULL}};
const Segment SGI62810[] = {{&S5828[31], 3}, {NULL}};
const Segment SGI62811[] = {{&S5828[82], 6}, {NULL}};
const Segment SGI62812[] = {{&S5829[44], 19}, {NULL}};
const Segment SGI62813[] = {{&S5829[44], 31}, {NULL}};
const Segment SGI62814[] = {{&S5829[30], 6}, {NULL}};
const Segment SGI62815[] = {{&S5829[31], 3}, {NULL}};
const Segment SGI62816[] = {{&S5829[89], 5}, {NULL}};
const Segment SGI62817[] = {{&S5830[44], 10}, {NULL}};
const Segment SGI62818[] = {{&S5830[44], 22}, {NULL}};
const Segment SGI62819[] = {{&S5830[30], 6}, {NULL}};
const Segment SGI62820[] = {{&S5830[31], 3}, {NULL}};
const Segment SGI62821[] = {{&S5830[80], 6}, {NULL}};
const Segment SGI62822[] = {{&S5831[44], 16}, {NULL}};
const Segment SGI62823[] = {{&S5831[44], 28}, {NULL}};
const Segment SGI62824[] = {{&S5831[30], 6}, {NULL}};
const Segment SGI62825[] = {{&S5831[31], 3}, {NULL}};
const Segment SGI62826[] = {{&S5831[86], 6}, {NULL}};
const Segment SGI62827[] = {{&S5832[44], 6}, {NULL}};
const Segment SGI62828[] = {{&S5832[44], 18}, {NULL}};
const Segment SGI62829[] = {{&S5832[30], 6}, {NULL}};
const Segment SGI62830[] = {{&S5832[31], 3}, {NULL}};
const Segment SGI62831[] = {{&S5832[76], 6}, {NULL}};
const Segment SGI62832[] = {{&S5833[44], 11}, {NULL}};
const Segment SGI62833[] = {{&S5833[44], 23}, {NULL}};
const Segment SGI62834[] = {{&S5833[30], 6}, {NULL}};
const Segment SGI62835[] = {{&S5833[31], 3}, {NULL}};
const Segment SGI62836[] = {{&S5833[81], 6}, {NULL}};
const Segment SGI62837[] = {{&S5835[44], 8}, {NULL}};
const Segment SGI62838[] = {{&S5835[44], 20}, {NULL}};
const Segment SGI62839[] = {{&S5835[30], 6}, {NULL}};
const Segment SGI62840[] = {{&S5835[31], 3}, {NULL}};
const Segment SGI62841[] = {{&S5835[78], 6}, {NULL}};
const Segment SGI62842[] = {{&S5836[44], 9}, {NULL}};
const Segment SGI62843[] = {{&S5836[44], 21}, {NULL}};
const Segment SGI62844[] = {{&S5836[30], 6}, {NULL}};
const Segment SGI62845[] = {{&S5836[31], 3}, {NULL}};
const Segment SGI62846[] = {{&S5836[79], 6}, {NULL}};
const Segment SGI62847[] = {{&S5837[44], 10}, {NULL}};
const Segment SGI62848[] = {{&S5837[44], 22}, {NULL}};
const Segment SGI62849[] = {{&S5837[30], 6}, {NULL}};
const Segment SGI62850[] = {{&S5837[31], 3}, {NULL}};
const Segment SGI62851[] = {{&S5837[80], 6}, {NULL}};
const Segment SGI62852[] = {{&S5838[44], 7}, {NULL}};
const Segment SGI62853[] = {{&S5838[44], 19}, {NULL}};
const Segment SGI62854[] = {{&S5838[30], 6}, {NULL}};
const Segment SGI62855[] = {{&S5838[31], 3}, {NULL}};
const Segment SGI62856[] = {{&S5838[77], 6}, {NULL}};
const Segment SGI62857[] = {{&S5840[44], 7}, {NULL}};
const Segment SGI62858[] = {{&S5840[44], 19}, {NULL}};
const Segment SGI62859[] = {{&S5840[30], 6}, {NULL}};
const Segment SGI62860[] = {{&S5840[31], 3}, {NULL}};
const Segment SGI62861[] = {{&S5840[77], 6}, {NULL}};
const Segment SGI62862[] = {{&S5841[44], 7}, {NULL}};
const Segment SGI62863[] = {{&S5841[44], 19}, {NULL}};
const Segment SGI62864[] = {{&S5841[30], 6}, {NULL}};
const Segment SGI62865[] = {{&S5841[31], 3}, {NULL}};
const Segment SGI62866[] = {{&S5841[77], 6}, {NULL}};
const Segment SGI62867[] = {{&S5843[44], 8}, {NULL}};
const Segment SGI62868[] = {{&S5843[44], 20}, {NULL}};
const Segment SGI62869[] = {{&S5843[30], 6}, {NULL}};
const Segment SGI62870[] = {{&S5843[31], 3}, {NULL}};
const Segment SGI62871[] = {{&S5843[78], 6}, {NULL}};
const Segment SGI62872[] = {{&S5844[44], 8}, {NULL}};
const Segment SGI62873[] = {{&S5844[44], 20}, {NULL}};
const Segment SGI62874[] = {{&S5844[30], 6}, {NULL}};
const Segment SGI62875[] = {{&S5844[31], 3}, {NULL}};
const Segment SGI62876[] = {{&S5844[78], 6}, {NULL}};
const Segment SGI62877[] = {{&S5846[44], 12}, {NULL}};
const Segment SGI62878[] = {{&S5846[44], 24}, {NULL}};
const Segment SGI62879[] = {{&S5846[30], 6}, {NULL}};
const Segment SGI62880[] = {{&S5846[31], 3}, {NULL}};
const Segment SGI62881[] = {{&S5846[82], 6}, {NULL}};
const Segment SGI62882[] = {{&S5847[44], 10}, {NULL}};
const Segment SGI62883[] = {{&S5847[44], 22}, {NULL}};
const Segment SGI62884[] = {{&S5847[30], 6}, {NULL}};
const Segment SGI62885[] = {{&S5847[31], 3}, {NULL}};
const Segment SGI62886[] = {{&S5847[80], 6}, {NULL}};
const Segment SGI62887[] = {{&S5850[44], 9}, {NULL}};
const Segment SGI62888[] = {{&S5850[44], 21}, {NULL}};
const Segment SGI62889[] = {{&S5850[30], 6}, {NULL}};
const Segment SGI62890[] = {{&S5850[31], 3}, {NULL}};
const Segment SGI62891[] = {{&S5850[79], 6}, {NULL}};
const Segment SGI62892[] = {{&S5851[44], 13}, {NULL}};
const Segment SGI62893[] = {{&S5851[44], 25}, {NULL}};
const Segment SGI62894[] = {{&S5851[30], 6}, {NULL}};
const Segment SGI62895[] = {{&S5851[31], 3}, {NULL}};
const Segment SGI62896[] = {{&S5851[83], 6}, {NULL}};
const Segment SGI62897[] = {{&S5852[44], 10}, {NULL}};
const Segment SGI62898[] = {{&S5852[44], 22}, {NULL}};
const Segment SGI62899[] = {{&S5852[30], 6}, {NULL}};
const Segment SGI62900[] = {{&S5852[31], 3}, {NULL}};
const Segment SGI62901[] = {{&S5852[80], 6}, {NULL}};
const Segment SGI62902[] = {{&S5853[44], 8}, {NULL}};
const Segment SGI62903[] = {{&S5853[44], 20}, {NULL}};
const Segment SGI62904[] = {{&S5853[30], 6}, {NULL}};
const Segment SGI62905[] = {{&S5853[31], 3}, {NULL}};
const Segment SGI62906[] = {{&S5853[78], 6}, {NULL}};
const Segment SGI62907[] = {{&S5854[44], 12}, {NULL}};
const Segment SGI62908[] = {{&S5854[44], 24}, {NULL}};
const Segment SGI62909[] = {{&S5854[30], 6}, {NULL}};
const Segment SGI62910[] = {{&S5854[31], 3}, {NULL}};
const Segment SGI62911[] = {{&S5854[82], 6}, {NULL}};
const Segment SGI62912[] = {{&S5855[44], 8}, {NULL}};
const Segment SGI62913[] = {{&S5855[44], 20}, {NULL}};
const Segment SGI62914[] = {{&S5855[30], 6}, {NULL}};
const Segment SGI62915[] = {{&S5855[31], 3}, {NULL}};
const Segment SGI62916[] = {{&S5855[78], 6}, {NULL}};
const Segment SGI62917[] = {{&S5856[44], 17}, {NULL}};
const Segment SGI62918[] = {{&S5856[44], 29}, {NULL}};
const Segment SGI62919[] = {{&S5856[30], 6}, {NULL}};
const Segment SGI62920[] = {{&S5856[31], 3}, {NULL}};
const Segment SGI62921[] = {{&S5856[87], 6}, {NULL}};
const Segment SGI62922[] = {{&S5857[44], 6}, {NULL}};
const Segment SGI62923[] = {{&S5857[44], 18}, {NULL}};
const Segment SGI62924[] = {{&S5857[30], 6}, {NULL}};
const Segment SGI62925[] = {{&S5857[31], 3}, {NULL}};
const Segment SGI62926[] = {{&S5857[76], 6}, {NULL}};
const Segment SGI62927[] = {{&S5858[44], 8}, {NULL}};
const Segment SGI62928[] = {{&S5858[44], 20}, {NULL}};
const Segment SGI62929[] = {{&S5858[30], 6}, {NULL}};
const Segment SGI62930[] = {{&S5858[31], 3}, {NULL}};
const Segment SGI62931[] = {{&S5858[78], 6}, {NULL}};
const Segment SGI62932[] = {{&S5860[44], 16}, {NULL}};
const Segment SGI62933[] = {{&S5860[44], 28}, {NULL}};
const Segment SGI62934[] = {{&S5860[30], 6}, {NULL}};
const Segment SGI62935[] = {{&S5860[31], 3}, {NULL}};
const Segment SGI62936[] = {{&S5860[86], 6}, {NULL}};
const Segment SGI62937[] = {{&S5861[44], 14}, {NULL}};
const Segment SGI62938[] = {{&S5861[44], 26}, {NULL}};
const Segment SGI62939[] = {{&S5861[30], 6}, {NULL}};
const Segment SGI62940[] = {{&S5861[31], 3}, {NULL}};
const Segment SGI62941[] = {{&S5861[84], 6}, {NULL}};
const Segment SGI62942[] = {{&S14586[44], 19}, {NULL}};
const Segment SGI62943[] = {{&S14586[44], 36}, {NULL}};
const Segment SGI62944[] = {{&S14586[30], 6}, {NULL}};
const Segment SGI62945[] = {{&S5862[44], 7}, {NULL}};
const Segment SGI62946[] = {{&S5862[44], 19}, {NULL}};
const Segment SGI62947[] = {{&S5862[30], 6}, {NULL}};
const Segment SGI62948[] = {{&S14586[128], 3}, {NULL}};
const Segment SGI62949[] = {{&S5862[31], 3}, {NULL}};
const Segment SGI62950[] = {{&S14586[94], 5}, {NULL}};
const Segment SGI62951[] = {{&S5862[77], 6}, {NULL}};
const Segment SGI62952[] = {{&S14587[44], 7}, {NULL}};
const Segment SGI62953[] = {{&S14587[44], 24}, {NULL}};
const Segment SGI62954[] = {{&S5863[44], 7}, {NULL}};
const Segment SGI62955[] = {{&S14587[30], 6}, {NULL}};
const Segment SGI62956[] = {{&S5863[44], 19}, {NULL}};
const Segment SGI62957[] = {{&S5863[30], 6}, {NULL}};
const Segment SGI62958[] = {{&S14587[116], 3}, {NULL}};
const Segment SGI62959[] = {{&S5863[31], 3}, {NULL}};
const Segment SGI62960[] = {{&S14587[82], 5}, {NULL}};
const Segment SGI62961[] = {{&S5863[77], 6}, {NULL}};
const Segment SGI62962[] = {{&S5864[44], 15}, {NULL}};
const Segment SGI62963[] = {{&S14588[44], 19}, {NULL}};
const Segment SGI62964[] = {{&S5864[44], 27}, {NULL}};
const Segment SGI62965[] = {{&S14588[44], 36}, {NULL}};
const Segment SGI62966[] = {{&S5864[30], 6}, {NULL}};
const Segment SGI62967[] = {{&S14588[30], 6}, {NULL}};
const Segment SGI62968[] = {{&S5864[31], 3}, {NULL}};
const Segment SGI62969[] = {{&S14588[128], 3}, {NULL}};
const Segment SGI62970[] = {{&S5864[85], 6}, {NULL}};
const Segment SGI62971[] = {{&S14588[94], 5}, {NULL}};
const Segment SGI62972[] = {{&S5865[44], 12}, {NULL}};
const Segment SGI62973[] = {{&S5865[44], 24}, {NULL}};
const Segment SGI62974[] = {{&S5865[30], 6}, {NULL}};
const Segment SGI62975[] = {{&S14589[44], 21}, {NULL}};
const Segment SGI62976[] = {{&S14589[44], 38}, {NULL}};
const Segment SGI62977[] = {{&S14589[30], 6}, {NULL}};
const Segment SGI62978[] = {{&S5865[31], 3}, {NULL}};
const Segment SGI62979[] = {{&S5865[82], 6}, {NULL}};
const Segment SGI62980[] = {{&S14589[130], 3}, {NULL}};
const Segment SGI62981[] = {{&S14589[96], 5}, {NULL}};
const Segment SGI62982[] = {{&S5866[44], 12}, {NULL}};
const Segment SGI62983[] = {{&S5866[44], 24}, {NULL}};
const Segment SGI62984[] = {{&S8510[74], 6}, {NULL}};
const Segment SGI62985[] = {{&S5866[30], 6}, {NULL}};
const Segment SGI62986[] = {{&S8510[0], 9}, {NULL}};
const Segment SGI62987[] = {{&S14590[44], 20}, {NULL}};
const Segment SGI62988[] = {{&S5866[31], 3}, {NULL}};
const Segment SGI62989[] = {{&S14590[44], 37}, {NULL}};
const Segment SGI62990[] = {{&S14590[30], 6}, {NULL}};
const Segment SGI62991[] = {{&S5866[82], 6}, {NULL}};
const Segment SGI62992[] = {{&S14590[129], 3}, {NULL}};
const Segment SGI62993[] = {{&S14590[95], 5}, {NULL}};
const Segment SGI62994[] = {{&S5867[44], 6}, {NULL}};
const Segment SGI62995[] = {{&S5867[44], 18}, {NULL}};
const Segment SGI62996[] = {{&S5867[30], 6}, {NULL}};
const Segment SGI62997[] = {{&S5867[31], 3}, {NULL}};
const Segment SGI62998[] = {{&S5867[76], 6}, {NULL}};
const Segment SGI62999[] = {{&S14591[44], 21}, {NULL}};
const Segment SGI63000[] = {{&S14591[44], 38}, {NULL}};
const Segment SGI63001[] = {{&S14591[30], 6}, {NULL}};
const Segment SGI63002[] = {{&S14591[130], 3}, {NULL}};
const Segment SGI63003[] = {{&S14591[96], 5}, {NULL}};
const Segment SGI63004[] = {{&S5868[44], 9}, {NULL}};
const Segment SGI63005[] = {{&S5868[44], 21}, {NULL}};
const Segment SGI63006[] = {{&S5868[30], 6}, {NULL}};
const Segment SGI63007[] = {{&S5868[31], 3}, {NULL}};
const Segment SGI63008[] = {{&S5868[79], 6}, {NULL}};
const Segment SGI63009[] = {{&S14592[44], 19}, {NULL}};
const Segment SGI63010[] = {{&S14592[44], 36}, {NULL}};
const Segment SGI63011[] = {{&S14592[30], 6}, {NULL}};
const Segment SGI63012[] = {{&S14592[128], 3}, {NULL}};
const Segment SGI63013[] = {{&S14592[94], 5}, {NULL}};
const Segment SGI63014[] = {{&S5870[44], 10}, {NULL}};
const Segment SGI63015[] = {{&S5870[44], 22}, {NULL}};
const Segment SGI63016[] = {{&S5870[30], 6}, {NULL}};
const Segment SGI63017[] = {{&S14593[44], 19}, {NULL}};
const Segment SGI63018[] = {{&S14593[44], 36}, {NULL}};
const Segment SGI63019[] = {{&S14593[30], 6}, {NULL}};
const Segment SGI63020[] = {{&S5870[31], 3}, {NULL}};
const Segment SGI63021[] = {{&S5870[80], 6}, {NULL}};
const Segment SGI63022[] = {{&S14593[128], 3}, {NULL}};
const Segment SGI63023[] = {{&S14593[94], 5}, {NULL}};
const Segment SGI63024[] = {{&S5871[44], 9}, {NULL}};
const Segment SGI63025[] = {{&S5871[44], 21}, {NULL}};
const Segment SGI63026[] = {{&S5871[30], 6}, {NULL}};
const Segment SGI63027[] = {{&S14594[44], 19}, {NULL}};
const Segment SGI63028[] = {{&S14594[44], 36}, {NULL}};
const Segment SGI63029[] = {{&S14594[30], 6}, {NULL}};
const Segment SGI63030[] = {{&S5871[31], 3}, {NULL}};
const Segment SGI63031[] = {{&S5871[79], 6}, {NULL}};
const Segment SGI63032[] = {{&S14594[128], 3}, {NULL}};
const Segment SGI63033[] = {{&S14594[94], 5}, {NULL}};
const Segment SGI63034[] = {{&S5872[44], 17}, {NULL}};
const Segment SGI63035[] = {{&S5872[44], 26}, {NULL}};
const Segment SGI63036[] = {{&S5872[30], 6}, {NULL}};
const Segment SGI63037[] = {{&S14595[44], 7}, {NULL}};
const Segment SGI63038[] = {{&S5872[31], 3}, {NULL}};
const Segment SGI63039[] = {{&S14595[44], 24}, {NULL}};
const Segment SGI63040[] = {{&S5872[84], 6}, {NULL}};
const Segment SGI63041[] = {{&S14595[30], 6}, {NULL}};
const Segment SGI63042[] = {{&S14595[116], 3}, {NULL}};
const Segment SGI63043[] = {{&S14595[82], 5}, {NULL}};
const Segment SGI63044[] = {{&S5873[44], 8}, {NULL}};
const Segment SGI63045[] = {{&S5873[44], 20}, {NULL}};
const Segment SGI63046[] = {{&S5873[30], 6}, {NULL}};
const Segment SGI63047[] = {{&S14596[38], 7}, {NULL}};
const Segment SGI63048[] = {{&S5873[31], 3}, {NULL}};
const Segment SGI63049[] = {{&S14596[31], 6}, {NULL}};
const Segment SGI63050[] = {{&S5873[78], 6}, {NULL}};
const Segment SGI63051[] = {{&S5874[44], 8}, {NULL}};
const Segment SGI63052[] = {{&S14597[44], 7}, {NULL}};
const Segment SGI63053[] = {{&S5874[44], 20}, {NULL}};
const Segment SGI63054[] = {{&S5874[30], 6}, {NULL}};
const Segment SGI63055[] = {{&S14597[44], 24}, {NULL}};
const Segment SGI63056[] = {{&S14597[30], 6}, {NULL}};
const Segment SGI63057[] = {{&S5874[31], 3}, {NULL}};
const Segment SGI63058[] = {{&S5874[78], 6}, {NULL}};
const Segment SGI63059[] = {{&S14597[85], 6}, {NULL}};
const Segment SGI63060[] = {{&S8525[73], 6}, {NULL}};
const Segment SGI63061[] = {{&S8525[0], 8}, {NULL}};
const Segment SGI63062[] = {{&S14598[44], 7}, {NULL}};
const Segment SGI63063[] = {{&S14598[44], 24}, {NULL}};
const Segment SGI63064[] = {{&S14598[30], 6}, {NULL}};
const Segment SGI63065[] = {{&S14598[116], 3}, {NULL}};
const Segment SGI63066[] = {{&S14598[82], 5}, {NULL}};
const Segment SGI63067[] = {{&S5877[44], 11}, {NULL}};
const Segment SGI63068[] = {{&S14599[44], 7}, {NULL}};
const Segment SGI63069[] = {{&S5877[44], 23}, {NULL}};
const Segment SGI63070[] = {{&S14599[44], 24}, {NULL}};
const Segment SGI63071[] = {{&S5877[30], 6}, {NULL}};
const Segment SGI63072[] = {{&S14599[30], 6}, {NULL}};
const Segment SGI63073[] = {{&S5877[31], 3}, {NULL}};
const Segment SGI63074[] = {{&S5877[81], 6}, {NULL}};
const Segment SGI63075[] = {{&S14599[85], 6}, {NULL}};
const Segment SGI63076[] = {{&S14600[38], 7}, {NULL}};
const Segment SGI63077[] = {{&S5878[44], 12}, {NULL}};
const Segment SGI63078[] = {{&S14600[31], 6}, {NULL}};
const Segment SGI63079[] = {{&S5878[44], 24}, {NULL}};
const Segment SGI63080[] = {{&S5878[30], 6}, {NULL}};
const Segment SGI63081[] = {{&S5878[31], 3}, {NULL}};
const Segment SGI63082[] = {{&S5878[82], 6}, {NULL}};
const Segment SGI63083[] = {{&S8529[71], 6}, {NULL}};
const Segment SGI63084[] = {{&S8529[0], 6}, {NULL}};
const Segment SGI63085[] = {{&S14601[44], 7}, {NULL}};
const Segment SGI63086[] = {{&S14601[44], 24}, {NULL}};
const Segment SGI63087[] = {{&S14601[30], 6}, {NULL}};
const Segment SGI63088[] = {{&S5879[44], 7}, {NULL}};
const Segment SGI63089[] = {{&S14601[116], 3}, {NULL}};
const Segment SGI63090[] = {{&S5879[44], 19}, {NULL}};
const Segment SGI63091[] = {{&S5879[30], 6}, {NULL}};
const Segment SGI63092[] = {{&S14601[82], 5}, {NULL}};
const Segment SGI63093[] = {{&S5879[31], 3}, {NULL}};
const Segment SGI63094[] = {{&S5879[77], 6}, {NULL}};
const Segment SGI63095[] = {{&S8530[79], 6}, {NULL}};
const Segment SGI63096[] = {{&S8530[0], 14}, {NULL}};
const Segment SGI63097[] = {{&S14602[44], 21}, {NULL}};
const Segment SGI63098[] = {{&S5880[44], 7}, {NULL}};
const Segment SGI63099[] = {{&S14602[44], 38}, {NULL}};
const Segment SGI63100[] = {{&S5880[44], 19}, {NULL}};
const Segment SGI63101[] = {{&S14602[30], 6}, {NULL}};
const Segment SGI63102[] = {{&S5880[30], 6}, {NULL}};
const Segment SGI63103[] = {{&S5880[31], 3}, {NULL}};
const Segment SGI63104[] = {{&S14602[130], 3}, {NULL}};
const Segment SGI63105[] = {{&S5880[77], 6}, {NULL}};
const Segment SGI63106[] = {{&S14602[96], 5}, {NULL}};
const Segment SGI63107[] = {{&S14603[44], 19}, {NULL}};
const Segment SGI63108[] = {{&S5881[44], 9}, {NULL}};
const Segment SGI63109[] = {{&S14603[44], 36}, {NULL}};
const Segment SGI63110[] = {{&S5881[44], 21}, {NULL}};
const Segment SGI63111[] = {{&S14603[30], 6}, {NULL}};
const Segment SGI63112[] = {{&S5881[30], 6}, {NULL}};
const Segment SGI63113[] = {{&S8533[44], 10}, {NULL}};
const Segment SGI63114[] = {{&S8533[44], 22}, {NULL}};
const Segment SGI63115[] = {{&S5881[31], 3}, {NULL}};
const Segment SGI63116[] = {{&S14603[128], 3}, {NULL}};
const Segment SGI63117[] = {{&S8533[30], 6}, {NULL}};
const Segment SGI63118[] = {{&S5881[79], 6}, {NULL}};
const Segment SGI63119[] = {{&S14603[94], 5}, {NULL}};
const Segment SGI63120[] = {{&S8533[31], 3}, {NULL}};
const Segment SGI63121[] = {{&S8533[80], 6}, {NULL}};
const Segment SGI63122[] = {{&S8533[0], 13}, {NULL}};
const Segment SGI63123[] = {{&S5882[44], 7}, {NULL}};
const Segment SGI63124[] = {{&S5882[44], 19}, {NULL}};
const Segment SGI63125[] = {{&S5882[30], 6}, {NULL}};
const Segment SGI63126[] = {{&S14604[44], 20}, {NULL}};
const Segment SGI63127[] = {{&S14604[44], 37}, {NULL}};
const Segment SGI63128[] = {{&S14604[30], 6}, {NULL}};
const Segment SGI63129[] = {{&S5882[31], 3}, {NULL}};
const Segment SGI63130[] = {{&S5882[77], 6}, {NULL}};
const Segment SGI63131[] = {{&S14604[129], 3}, {NULL}};
const Segment SGI63132[] = {{&S14604[95], 5}, {NULL}};
const Segment SGI63133[] = {{&S5883[44], 8}, {NULL}};
const Segment SGI63134[] = {{&S5883[44], 20}, {NULL}};
const Segment SGI63135[] = {{&S5883[30], 6}, {NULL}};
const Segment SGI63136[] = {{&S5883[31], 3}, {NULL}};
const Segment SGI63137[] = {{&S14605[44], 19}, {NULL}};
const Segment SGI63138[] = {{&S5883[78], 6}, {NULL}};
const Segment SGI63139[] = {{&S14605[44], 36}, {NULL}};
const Segment SGI63140[] = {{&S14605[30], 6}, {NULL}};
const Segment SGI63141[] = {{&S8535[78], 6}, {NULL}};
const Segment SGI63142[] = {{&S8535[0], 13}, {NULL}};
const Segment SGI63143[] = {{&S14605[128], 3}, {NULL}};
const Segment SGI63144[] = {{&S14605[94], 5}, {NULL}};
const Segment SGI63145[] = {{&S5884[44], 8}, {NULL}};
const Segment SGI63146[] = {{&S5884[44], 20}, {NULL}};
const Segment SGI63147[] = {{&S5884[30], 6}, {NULL}};
const Segment SGI63148[] = {{&S5884[31], 3}, {NULL}};
const Segment SGI63149[] = {{&S5884[78], 6}, {NULL}};
const Segment SGI63150[] = {{&S14606[44], 20}, {NULL}};
const Segment SGI63151[] = {{&S14606[44], 37}, {NULL}};
const Segment SGI63152[] = {{&S14606[30], 6}, {NULL}};
const Segment SGI63153[] = {{&S14606[129], 3}, {NULL}};
const Segment SGI63154[] = {{&S14606[95], 5}, {NULL}};
const Segment SGI63155[] = {{&S5885[44], 6}, {NULL}};
const Segment SGI63156[] = {{&S5885[44], 18}, {NULL}};
const Segment SGI63157[] = {{&S5885[30], 6}, {NULL}};
const Segment SGI63158[] = {{&S5885[31], 3}, {NULL}};
const Segment SGI63159[] = {{&S5885[76], 6}, {NULL}};
const Segment SGI63160[] = {{&S14607[44], 19}, {NULL}};
const Segment SGI63161[] = {{&S14607[44], 36}, {NULL}};
const Segment SGI63162[] = {{&S14607[30], 6}, {NULL}};
const Segment SGI63163[] = {{&S14607[128], 3}, {NULL}};
const Segment SGI63164[] = {{&S14607[94], 5}, {NULL}};
const Segment SGI63165[] = {{&S14608[44], 7}, {NULL}};
const Segment SGI63166[] = {{&S14608[44], 24}, {NULL}};
const Segment SGI63167[] = {{&S14608[30], 6}, {NULL}};
const Segment SGI63168[] = {{&S14608[116], 3}, {NULL}};
const Segment SGI63169[] = {{&S14608[82], 5}, {NULL}};
const Segment SGI63170[] = {{&S5888[44], 10}, {NULL}};
const Segment SGI63171[] = {{&S5888[44], 22}, {NULL}};
const Segment SGI63172[] = {{&S5888[30], 6}, {NULL}};
const Segment SGI63173[] = {{&S5888[31], 3}, {NULL}};
const Segment SGI63174[] = {{&S5888[80], 6}, {NULL}};
const Segment SGI63175[] = {{&S14609[44], 19}, {NULL}};
const Segment SGI63176[] = {{&S14609[44], 36}, {NULL}};
const Segment SGI63177[] = {{&S14609[30], 6}, {NULL}};
const Segment SGI63178[] = {{&S14609[128], 3}, {NULL}};
const Segment SGI63179[] = {{&S5889[44], 6}, {NULL}};
const Segment SGI63180[] = {{&S5889[44], 18}, {NULL}};
const Segment SGI63181[] = {{&S14609[94], 5}, {NULL}};
const Segment SGI63182[] = {{&S5889[30], 6}, {NULL}};
const Segment SGI63183[] = {{&S5889[31], 3}, {NULL}};
const Segment SGI63184[] = {{&S5889[76], 6}, {NULL}};
const Segment SGI63185[] = {{&S14610[44], 19}, {NULL}};
const Segment SGI63186[] = {{&S14610[44], 36}, {NULL}};
const Segment SGI63187[] = {{&S5890[44], 13}, {NULL}};
const Segment SGI63188[] = {{&S14610[30], 6}, {NULL}};
const Segment SGI63189[] = {{&S5890[44], 27}, {NULL}};
const Segment SGI63190[] = {{&S5890[30], 6}, {NULL}};
const Segment SGI63191[] = {{&S14610[128], 3}, {NULL}};
const Segment SGI63192[] = {{&S5890[31], 3}, {NULL}};
const Segment SGI63193[] = {{&S14610[94], 5}, {NULL}};
const Segment SGI63194[] = {{&S5890[85], 6}, {NULL}};
const Segment SGI63195[] = {{&S14611[44], 7}, {NULL}};
const Segment SGI63196[] = {{&S5891[44], 6}, {NULL}};
const Segment SGI63197[] = {{&S14611[44], 24}, {NULL}};
const Segment SGI63198[] = {{&S5891[44], 18}, {NULL}};
const Segment SGI63199[] = {{&S14611[30], 6}, {NULL}};
const Segment SGI63200[] = {{&S5891[30], 6}, {NULL}};
const Segment SGI63201[] = {{&S14611[116], 3}, {NULL}};
const Segment SGI63202[] = {{&S5891[31], 3}, {NULL}};
const Segment SGI63203[] = {{&S14611[82], 5}, {NULL}};
const Segment SGI63204[] = {{&S5891[76], 5}, {NULL}};
const Segment SGI63205[] = {{&S14612[44], 7}, {NULL}};
const Segment SGI63206[] = {{&S5892[44], 17}, {NULL}};
const Segment SGI63207[] = {{&S14612[44], 24}, {NULL}};
const Segment SGI63208[] = {{&S5892[44], 29}, {NULL}};
const Segment SGI63209[] = {{&S14612[30], 6}, {NULL}};
const Segment SGI63210[] = {{&S5892[30], 6}, {NULL}};
const Segment SGI63211[] = {{&S14612[116], 3}, {NULL}};
const Segment SGI63212[] = {{&S5892[31], 3}, {NULL}};
const Segment SGI63213[] = {{&S14612[82], 5}, {NULL}};
const Segment SGI63214[] = {{&S5892[87], 6}, {NULL}};
const Segment SGI63215[] = {{&S14613[44], 7}, {NULL}};
const Segment SGI63216[] = {{&S14613[44], 24}, {NULL}};
const Segment SGI63217[] = {{&S14613[30], 6}, {NULL}};
const Segment SGI63218[] = {{&S14613[116], 3}, {NULL}};
const Segment SGI63219[] = {{&S14613[82], 5}, {NULL}};
const Segment SGI63220[] = {{&S5894[44], 8}, {NULL}};
const Segment SGI63221[] = {{&S5894[44], 20}, {NULL}};
const Segment SGI63222[] = {{&S5894[30], 6}, {NULL}};
const Segment SGI63223[] = {{&S5894[31], 3}, {NULL}};
const Segment SGI63224[] = {{&S5894[78], 6}, {NULL}};
const Segment SGI63225[] = {{&S14614[44], 7}, {NULL}};
const Segment SGI63226[] = {{&S14614[44], 24}, {NULL}};
const Segment SGI63227[] = {{&S14614[30], 6}, {NULL}};
const Segment SGI63228[] = {{&S14614[116], 3}, {NULL}};
const Segment SGI63229[] = {{&S14614[82], 5}, {NULL}};
const Segment SGI63230[] = {{&S5895[44], 12}, {NULL}};
const Segment SGI63231[] = {{&S5895[44], 24}, {NULL}};
const Segment SGI63232[] = {{&S5895[30], 6}, {NULL}};
const Segment SGI63233[] = {{&S14615[44], 7}, {NULL}};
const Segment SGI63234[] = {{&S14615[44], 24}, {NULL}};
const Segment SGI63235[] = {{&S14615[30], 6}, {NULL}};
const Segment SGI63236[] = {{&S14615[116], 3}, {NULL}};
const Segment SGI63237[] = {{&S14615[82], 5}, {NULL}};
const Segment SGI63238[] = {{&S14616[42], 19}, {NULL}};
const Segment SGI63239[] = {{&S14616[30], 6}, {NULL}};
const Segment SGI63240[] = {{&S14616[126], 3}, {NULL}};
const Segment SGI63241[] = {{&S14616[92], 5}, {NULL}};
const Segment SGI63242[] = {{&S14617[44], 19}, {NULL}};
const Segment SGI63243[] = {{&S14617[44], 43}, {NULL}};
const Segment SGI63244[] = {{&S14617[30], 6}, {NULL}};
const Segment SGI63245[] = {{&S14617[135], 3}, {NULL}};
const Segment SGI63246[] = {{&S14617[101], 5}, {NULL}};
const Segment SGI63247[] = {{&S14618[42], 19}, {NULL}};
const Segment SGI63248[] = {{&S14618[30], 6}, {NULL}};
const Segment SGI63249[] = {{&S14618[126], 3}, {NULL}};
const Segment SGI63250[] = {{&S14618[92], 5}, {NULL}};
const Segment SGI63251[] = {{&S14619[44], 19}, {NULL}};
const Segment SGI63252[] = {{&S14619[44], 36}, {NULL}};
const Segment SGI63253[] = {{&S14619[30], 6}, {NULL}};
const Segment SGI63254[] = {{&S14619[128], 3}, {NULL}};
const Segment SGI63255[] = {{&S14619[94], 5}, {NULL}};
const Segment SGI63256[] = {{&S5895[31], 3}, {NULL}};
const Segment SGI63257[] = {{&S5895[82], 6}, {NULL}};
const Segment SGI63258[] = {{&S5896[44], 10}, {NULL}};
const Segment SGI63259[] = {{&S5896[44], 22}, {NULL}};
const Segment SGI63260[] = {{&S5896[30], 6}, {NULL}};
const Segment SGI63261[] = {{&S5896[31], 3}, {NULL}};
const Segment SGI63262[] = {{&S5896[80], 6}, {NULL}};
const Segment SGI63263[] = {{&S5898[44], 5}, {NULL}};
const Segment SGI63264[] = {{&S5898[44], 17}, {NULL}};
const Segment SGI63265[] = {{&S5898[30], 6}, {NULL}};
const Segment SGI63266[] = {{&S5898[31], 3}, {NULL}};
const Segment SGI63267[] = {{&S5898[75], 6}, {NULL}};
const Segment SGI63268[] = {{&S5899[44], 16}, {NULL}};
const Segment SGI63269[] = {{&S5899[44], 28}, {NULL}};
const Segment SGI63270[] = {{&S5899[30], 6}, {NULL}};
const Segment SGI63271[] = {{&S5899[31], 3}, {NULL}};
const Segment SGI63272[] = {{&S5899[86], 6}, {NULL}};
const Segment SGI63273[] = {{&S5900[44], 9}, {NULL}};
const Segment SGI63274[] = {{&S5900[44], 21}, {NULL}};
const Segment SGI63275[] = {{&S5900[30], 6}, {NULL}};
const Segment SGI63276[] = {{&S5900[31], 3}, {NULL}};
const Segment SGI63277[] = {{&S5900[79], 6}, {NULL}};
const Segment SGI63278[] = {{&S5901[44], 16}, {NULL}};
const Segment SGI63279[] = {{&S5901[44], 28}, {NULL}};
const Segment SGI63280[] = {{&S5901[30], 6}, {NULL}};
const Segment SGI63281[] = {{&S5901[31], 3}, {NULL}};
const Segment SGI63282[] = {{&S5901[86], 6}, {NULL}};
const Segment SGI63283[] = {{&S5902[44], 12}, {NULL}};
const Segment SGI63284[] = {{&S5902[44], 24}, {NULL}};
const Segment SGI63285[] = {{&S5902[30], 6}, {NULL}};
const Segment SGI63286[] = {{&S5902[31], 3}, {NULL}};
const Segment SGI63287[] = {{&S5902[82], 6}, {NULL}};
const Segment SGI63288[] = {{&S5903[44], 8}, {NULL}};
const Segment SGI63289[] = {{&S5903[44], 20}, {NULL}};
const Segment SGI63290[] = {{&S5903[30], 6}, {NULL}};
const Segment SGI63291[] = {{&S5903[31], 3}, {NULL}};
const Segment SGI63292[] = {{&S5903[78], 6}, {NULL}};
const Segment SGI63293[] = {{&S5904[44], 7}, {NULL}};
const Segment SGI63294[] = {{&S5904[44], 19}, {NULL}};
const Segment SGI63295[] = {{&S5904[30], 6}, {NULL}};
const Segment SGI63296[] = {{&S5904[31], 3}, {NULL}};
const Segment SGI63297[] = {{&S5904[77], 6}, {NULL}};
const Segment SGI63298[] = {{&S5905[44], 11}, {NULL}};
const Segment SGI63299[] = {{&S5905[44], 23}, {NULL}};
const Segment SGI63300[] = {{&S5905[30], 6}, {NULL}};
const Segment SGI63301[] = {{&S5905[31], 3}, {NULL}};
const Segment SGI63302[] = {{&S5905[81], 6}, {NULL}};
const Segment SGI63303[] = {{&S5906[44], 6}, {NULL}};
const Segment SGI63304[] = {{&S5906[44], 18}, {NULL}};
const Segment SGI63305[] = {{&S5906[30], 6}, {NULL}};
const Segment SGI63306[] = {{&S5906[31], 3}, {NULL}};
const Segment SGI63307[] = {{&S5906[76], 6}, {NULL}};
const Segment SGI63308[] = {{&S5907[44], 17}, {NULL}};
const Segment SGI63309[] = {{&S5907[44], 29}, {NULL}};
const Segment SGI63310[] = {{&S5907[30], 6}, {NULL}};
const Segment SGI63311[] = {{&S5907[31], 3}, {NULL}};
const Segment SGI63312[] = {{&S5907[87], 6}, {NULL}};
const Segment SGI63313[] = {{&S5908[38], 21}, {NULL}};
const Segment SGI63314[] = {{&S5908[31], 6}, {NULL}};
const Segment SGI63315[] = {{&S5910[44], 6}, {NULL}};
const Segment SGI63316[] = {{&S5910[44], 18}, {NULL}};
const Segment SGI63317[] = {{&S5910[30], 6}, {NULL}};
const Segment SGI63318[] = {{&S5910[31], 3}, {NULL}};
const Segment SGI63319[] = {{&S5910[76], 6}, {NULL}};
const Segment SGI63320[] = {{&S5911[44], 6}, {NULL}};
const Segment SGI63321[] = {{&S5911[44], 18}, {NULL}};
const Segment SGI63322[] = {{&S5911[30], 6}, {NULL}};
const Segment SGI63323[] = {{&S5911[31], 3}, {NULL}};
const Segment SGI63324[] = {{&S5911[76], 6}, {NULL}};
const Segment SGI63325[] = {{&S5912[44], 9}, {NULL}};
const Segment SGI63326[] = {{&S5912[44], 21}, {NULL}};
const Segment SGI63327[] = {{&S5912[30], 6}, {NULL}};
const Segment SGI63328[] = {{&S5912[31], 3}, {NULL}};
const Segment SGI63329[] = {{&S5912[79], 6}, {NULL}};
const Segment SGI63330[] = {{&S5913[44], 6}, {NULL}};
const Segment SGI63331[] = {{&S5913[44], 18}, {NULL}};
const Segment SGI63332[] = {{&S5913[30], 6}, {NULL}};
const Segment SGI63333[] = {{&S5913[31], 3}, {NULL}};
const Segment SGI63334[] = {{&S5913[76], 6}, {NULL}};
const Segment SGI63335[] = {{&S5914[44], 7}, {NULL}};
const Segment SGI63336[] = {{&S5914[44], 19}, {NULL}};
const Segment SGI63337[] = {{&S5914[30], 6}, {NULL}};
const Segment SGI63338[] = {{&S5914[31], 3}, {NULL}};
const Segment SGI63339[] = {{&S5914[77], 6}, {NULL}};
const Segment SGI63340[] = {{&S5915[44], 14}, {NULL}};
const Segment SGI63341[] = {{&S5915[44], 26}, {NULL}};
const Segment SGI63342[] = {{&S5915[30], 6}, {NULL}};
const Segment SGI63343[] = {{&S5915[31], 3}, {NULL}};
const Segment SGI63344[] = {{&S5915[84], 6}, {NULL}};
const Segment SGI63345[] = {{&S5916[44], 15}, {NULL}};
const Segment SGI63346[] = {{&S5916[44], 27}, {NULL}};
const Segment SGI63347[] = {{&S5916[30], 6}, {NULL}};
const Segment SGI63348[] = {{&S5916[31], 3}, {NULL}};
const Segment SGI63349[] = {{&S5916[85], 6}, {NULL}};
const Segment SGI63350[] = {{&S5917[44], 9}, {NULL}};
const Segment SGI63351[] = {{&S5917[44], 21}, {NULL}};
const Segment SGI63352[] = {{&S5917[30], 6}, {NULL}};
const Segment SGI63353[] = {{&S5917[31], 3}, {NULL}};
const Segment SGI63354[] = {{&S5917[79], 6}, {NULL}};
const Segment SGI63355[] = {{&S5918[44], 8}, {NULL}};
const Segment SGI63356[] = {{&S5918[44], 20}, {NULL}};
const Segment SGI63357[] = {{&S5918[30], 6}, {NULL}};
const Segment SGI63358[] = {{&S5918[31], 3}, {NULL}};
const Segment SGI63359[] = {{&S5918[78], 6}, {NULL}};
const Segment SGI63360[] = {{&S5919[44], 4}, {NULL}};
const Segment SGI63361[] = {{&S5919[44], 16}, {NULL}};
const Segment SGI63362[] = {{&S5919[30], 6}, {NULL}};
const Segment SGI63363[] = {{&S5919[31], 3}, {NULL}};
const Segment SGI63364[] = {{&S5919[74], 6}, {NULL}};
const Segment SGI63365[] = {{&S5920[44], 8}, {NULL}};
const Segment SGI63366[] = {{&S5920[44], 20}, {NULL}};
const Segment SGI63367[] = {{&S5920[30], 6}, {NULL}};
const Segment SGI63368[] = {{&S5920[31], 3}, {NULL}};
const Segment SGI63369[] = {{&S5920[78], 6}, {NULL}};
const Segment SGI63370[] = {{&S5921[44], 14}, {NULL}};
const Segment SGI63371[] = {{&S5921[44], 26}, {NULL}};
const Segment SGI63372[] = {{&S5921[30], 6}, {NULL}};
const Segment SGI63373[] = {{&S5921[31], 3}, {NULL}};
const Segment SGI63374[] = {{&S5921[84], 6}, {NULL}};
const Segment SGI63375[] = {{&S5922[44], 9}, {NULL}};
const Segment SGI63376[] = {{&S5922[44], 21}, {NULL}};
const Segment SGI63377[] = {{&S5922[30], 6}, {NULL}};
const Segment SGI63378[] = {{&S5922[31], 3}, {NULL}};
const Segment SGI63379[] = {{&S5922[79], 6}, {NULL}};
const Segment SGI63380[] = {{&S21067[44], 11}, {NULL}};
const Segment SGI63381[] = {{&S21067[44], 28}, {NULL}};
const Segment SGI63382[] = {{&S21067[30], 6}, {NULL}};
const Segment SGI63383[] = {{&S5923[44], 14}, {NULL}};
const Segment SGI63384[] = {{&S5923[44], 26}, {NULL}};
const Segment SGI63385[] = {{&S21067[120], 3}, {NULL}};
const Segment SGI63386[] = {{&S5923[30], 6}, {NULL}};
const Segment SGI63387[] = {{&S21067[86], 5}, {NULL}};
const Segment SGI63388[] = {{&S5923[31], 3}, {NULL}};
const Segment SGI63389[] = {{&S5923[84], 6}, {NULL}};
const Segment SGI63390[] = {{&S21068[44], 29}, {NULL}};
const Segment SGI63391[] = {{&S21068[44], 46}, {NULL}};
const Segment SGI63392[] = {{&S21068[30], 6}, {NULL}};
const Segment SGI63393[] = {{&S5924[44], 4}, {NULL}};
const Segment SGI63394[] = {{&S5924[44], 16}, {NULL}};
const Segment SGI63395[] = {{&S5924[30], 6}, {NULL}};
const Segment SGI63396[] = {{&S21068[138], 3}, {NULL}};
const Segment SGI63397[] = {{&S5924[31], 3}, {NULL}};
const Segment SGI63398[] = {{&S21068[104], 5}, {NULL}};
const Segment SGI63399[] = {{&S5924[74], 6}, {NULL}};
const Segment SGI63400[] = {{&S21069[44], 10}, {NULL}};
const Segment SGI63401[] = {{&S5925[44], 7}, {NULL}};
const Segment SGI63402[] = {{&S21069[44], 27}, {NULL}};
const Segment SGI63403[] = {{&S5925[44], 19}, {NULL}};
const Segment SGI63404[] = {{&S21069[30], 6}, {NULL}};
const Segment SGI63405[] = {{&S5925[30], 6}, {NULL}};
const Segment SGI63406[] = {{&S21069[119], 3}, {NULL}};
const Segment SGI63407[] = {{&S5925[31], 3}, {NULL}};
const Segment SGI63408[] = {{&S5925[77], 6}, {NULL}};
const Segment SGI63409[] = {{&S21069[85], 5}, {NULL}};
const Segment SGI63410[] = {{&S5926[44], 13}, {NULL}};
const Segment SGI63411[] = {{&S5926[44], 25}, {NULL}};
const Segment SGI63412[] = {{&S5926[30], 6}, {NULL}};
const Segment SGI63413[] = {{&S21070[44], 11}, {NULL}};
const Segment SGI63414[] = {{&S21070[44], 28}, {NULL}};
const Segment SGI63415[] = {{&S21070[30], 6}, {NULL}};
const Segment SGI63416[] = {{&S5926[31], 3}, {NULL}};
const Segment SGI63417[] = {{&S5926[83], 6}, {NULL}};
const Segment SGI63418[] = {{&S21070[120], 3}, {NULL}};
const Segment SGI63419[] = {{&S21070[86], 5}, {NULL}};
const Segment SGI63420[] = {{&S5927[44], 5}, {NULL}};
const Segment SGI63421[] = {{&S21071[38], 11}, {NULL}};
const Segment SGI63422[] = {{&S5927[44], 17}, {NULL}};
const Segment SGI63423[] = {{&S21071[31], 6}, {NULL}};
const Segment SGI63424[] = {{&S5927[30], 6}, {NULL}};
const Segment SGI63425[] = {{&S5927[31], 3}, {NULL}};
const Segment SGI63426[] = {{&S5927[75], 6}, {NULL}};
const Segment SGI63427[] = {{&S21072[44], 29}, {NULL}};
const Segment SGI63428[] = {{&S21072[44], 46}, {NULL}};
const Segment SGI63429[] = {{&S21072[30], 6}, {NULL}};
const Segment SGI63430[] = {{&S21072[138], 3}, {NULL}};
const Segment SGI63431[] = {{&S21072[104], 5}, {NULL}};
const Segment SGI63432[] = {{&S5929[44], 7}, {NULL}};
const Segment SGI63433[] = {{&S5929[44], 19}, {NULL}};
const Segment SGI63434[] = {{&S5929[30], 6}, {NULL}};
const Segment SGI63435[] = {{&S5929[31], 3}, {NULL}};
const Segment SGI63436[] = {{&S5929[77], 6}, {NULL}};
const Segment SGI63437[] = {{&S21073[44], 28}, {NULL}};
const Segment SGI63438[] = {{&S21073[44], 45}, {NULL}};
const Segment SGI63439[] = {{&S21073[30], 6}, {NULL}};
const Segment SGI63440[] = {{&S21073[137], 3}, {NULL}};
const Segment SGI63441[] = {{&S21073[103], 5}, {NULL}};
const Segment SGI63442[] = {{&S5930[44], 9}, {NULL}};
const Segment SGI63443[] = {{&S5930[44], 21}, {NULL}};
const Segment SGI63444[] = {{&S5930[30], 6}, {NULL}};
const Segment SGI63445[] = {{&S5930[31], 3}, {NULL}};
const Segment SGI63446[] = {{&S5930[79], 6}, {NULL}};
const Segment SGI63447[] = {{&S21074[44], 11}, {NULL}};
const Segment SGI63448[] = {{&S21074[44], 28}, {NULL}};
const Segment SGI63449[] = {{&S21074[30], 6}, {NULL}};
const Segment SGI63450[] = {{&S21074[120], 3}, {NULL}};
const Segment SGI63451[] = {{&S5931[44], 18}, {NULL}};
const Segment SGI63452[] = {{&S5931[44], 30}, {NULL}};
const Segment SGI63453[] = {{&S21074[86], 5}, {NULL}};
const Segment SGI63454[] = {{&S5931[30], 6}, {NULL}};
const Segment SGI63455[] = {{&S5931[31], 3}, {NULL}};
const Segment SGI63456[] = {{&S5931[88], 6}, {NULL}};
const Segment SGI63457[] = {{&S21075[44], 31}, {NULL}};
const Segment SGI63458[] = {{&S21075[44], 48}, {NULL}};
const Segment SGI63459[] = {{&S21075[30], 6}, {NULL}};
const Segment SGI63460[] = {{&S5932[44], 14}, {NULL}};
const Segment SGI63461[] = {{&S5932[44], 26}, {NULL}};
const Segment SGI63462[] = {{&S21075[140], 3}, {NULL}};
const Segment SGI63463[] = {{&S5932[30], 6}, {NULL}};
const Segment SGI63464[] = {{&S21075[106], 5}, {NULL}};
const Segment SGI63465[] = {{&S5932[31], 3}, {NULL}};
const Segment SGI63466[] = {{&S5932[84], 6}, {NULL}};
const Segment SGI63467[] = {{&S21076[44], 28}, {NULL}};
const Segment SGI63468[] = {{&S21076[44], 45}, {NULL}};
const Segment SGI63469[] = {{&S5933[44], 10}, {NULL}};
const Segment SGI63470[] = {{&S21076[30], 6}, {NULL}};
const Segment SGI63471[] = {{&S5933[44], 22}, {NULL}};
const Segment SGI63472[] = {{&S5933[30], 6}, {NULL}};
const Segment SGI63473[] = {{&S21076[137], 3}, {NULL}};
const Segment SGI63474[] = {{&S5933[31], 3}, {NULL}};
const Segment SGI63475[] = {{&S21076[103], 5}, {NULL}};
const Segment SGI63476[] = {{&S5933[80], 6}, {NULL}};
const Segment SGI63477[] = {{&S5934[44], 9}, {NULL}};
const Segment SGI63478[] = {{&S5934[44], 21}, {NULL}};
const Segment SGI63479[] = {{&S21077[44], 28}, {NULL}};
const Segment SGI63480[] = {{&S5934[30], 6}, {NULL}};
const Segment SGI63481[] = {{&S21077[44], 45}, {NULL}};
const Segment SGI63482[] = {{&S21077[30], 6}, {NULL}};
const Segment SGI63483[] = {{&S5934[31], 3}, {NULL}};
const Segment SGI63484[] = {{&S5934[79], 6}, {NULL}};
const Segment SGI63485[] = {{&S21077[137], 3}, {NULL}};
const Segment SGI63486[] = {{&S21077[103], 5}, {NULL}};
const Segment SGI63487[] = {{&S5935[44], 8}, {NULL}};
const Segment SGI63488[] = {{&S5935[44], 20}, {NULL}};
const Segment SGI63489[] = {{&S5935[30], 6}, {NULL}};
const Segment SGI63490[] = {{&S5935[31], 3}, {NULL}};
const Segment SGI63491[] = {{&S5935[78], 6}, {NULL}};
const Segment SGI63492[] = {{&S21078[44], 29}, {NULL}};
const Segment SGI63493[] = {{&S21078[44], 46}, {NULL}};
const Segment SGI63494[] = {{&S21078[30], 6}, {NULL}};
const Segment SGI63495[] = {{&S21078[138], 3}, {NULL}};
const Segment SGI63496[] = {{&S21078[104], 5}, {NULL}};
const Segment SGI63497[] = {{&S5936[44], 7}, {NULL}};
const Segment SGI63498[] = {{&S5936[44], 19}, {NULL}};
const Segment SGI63499[] = {{&S5936[30], 6}, {NULL}};
const Segment SGI63500[] = {{&S5936[31], 3}, {NULL}};
const Segment SGI63501[] = {{&S5936[77], 6}, {NULL}};
const Segment SGI63502[] = {{&S21079[44], 29}, {NULL}};
const Segment SGI63503[] = {{&S21079[44], 46}, {NULL}};
const Segment SGI63504[] = {{&S21079[30], 6}, {NULL}};
const Segment SGI63505[] = {{&S21079[138], 3}, {NULL}};
const Segment SGI63506[] = {{&S21079[104], 5}, {NULL}};
const Segment SGI63507[] = {{&S5937[44], 20}, {NULL}};
const Segment SGI63508[] = {{&S5937[44], 32}, {NULL}};
const Segment SGI63509[] = {{&S5937[30], 6}, {NULL}};
const Segment SGI63510[] = {{&S5937[31], 3}, {NULL}};
const Segment SGI63511[] = {{&S5937[90], 6}, {NULL}};
const Segment SGI63512[] = {{&S21080[38], 10}, {NULL}};
const Segment SGI63513[] = {{&S21080[31], 6}, {NULL}};
const Segment SGI63514[] = {{&S5938[44], 17}, {NULL}};
const Segment SGI63515[] = {{&S5938[44], 29}, {NULL}};
const Segment SGI63516[] = {{&S5938[30], 6}, {NULL}};
const Segment SGI63517[] = {{&S5938[31], 3}, {NULL}};
const Segment SGI63518[] = {{&S21081[44], 31}, {NULL}};
const Segment SGI63519[] = {{&S5938[87], 6}, {NULL}};
const Segment SGI63520[] = {{&S21081[44], 48}, {NULL}};
const Segment SGI63521[] = {{&S21081[30], 6}, {NULL}};
const Segment SGI63522[] = {{&S21081[140], 3}, {NULL}};
const Segment SGI63523[] = {{&S21081[106], 5}, {NULL}};
const Segment SGI63524[] = {{&S5939[44], 12}, {NULL}};
const Segment SGI63525[] = {{&S5939[44], 24}, {NULL}};
const Segment SGI63526[] = {{&S5939[30], 6}, {NULL}};
const Segment SGI63527[] = {{&S5939[31], 3}, {NULL}};
const Segment SGI63528[] = {{&S5939[82], 6}, {NULL}};
const Segment SGI63529[] = {{&S21082[44], 29}, {NULL}};
const Segment SGI63530[] = {{&S21082[44], 46}, {NULL}};
const Segment SGI63531[] = {{&S21082[30], 6}, {NULL}};
const Segment SGI63532[] = {{&S21082[138], 3}, {NULL}};
const Segment SGI63533[] = {{&S21082[104], 5}, {NULL}};
const Segment SGI63534[] = {{&S5940[44], 6}, {NULL}};
const Segment SGI63535[] = {{&S5940[44], 18}, {NULL}};
const Segment SGI63536[] = {{&S5940[30], 6}, {NULL}};
const Segment SGI63537[] = {{&S5940[31], 3}, {NULL}};
const Segment SGI63538[] = {{&S5940[76], 6}, {NULL}};
const Segment SGI63539[] = {{&S21083[44], 31}, {NULL}};
const Segment SGI63540[] = {{&S21083[44], 48}, {NULL}};
const Segment SGI63541[] = {{&S21083[30], 6}, {NULL}};
const Segment SGI63542[] = {{&S21083[140], 3}, {NULL}};
const Segment SGI63543[] = {{&S21083[106], 5}, {NULL}};
const Segment SGI63544[] = {{&S5942[44], 10}, {NULL}};
const Segment SGI63545[] = {{&S5942[44], 22}, {NULL}};
const Segment SGI63546[] = {{&S5942[30], 6}, {NULL}};
const Segment SGI63547[] = {{&S21084[44], 29}, {NULL}};
const Segment SGI63548[] = {{&S21084[44], 46}, {NULL}};
const Segment SGI63549[] = {{&S5942[31], 3}, {NULL}};
const Segment SGI63550[] = {{&S21084[30], 6}, {NULL}};
const Segment SGI63551[] = {{&S5942[80], 6}, {NULL}};
const Segment SGI63552[] = {{&S21084[138], 3}, {NULL}};
const Segment SGI63553[] = {{&S21084[104], 5}, {NULL}};
const Segment SGI63554[] = {{&S5943[44], 20}, {NULL}};
const Segment SGI63555[] = {{&S5943[44], 32}, {NULL}};
const Segment SGI63556[] = {{&S5943[30], 6}, {NULL}};
const Segment SGI63557[] = {{&S21085[44], 28}, {NULL}};
const Segment SGI63558[] = {{&S5943[31], 3}, {NULL}};
const Segment SGI63559[] = {{&S21085[44], 45}, {NULL}};
const Segment SGI63560[] = {{&S21085[30], 6}, {NULL}};
const Segment SGI63561[] = {{&S5943[90], 6}, {NULL}};
const Segment SGI63562[] = {{&S21085[137], 3}, {NULL}};
const Segment SGI63563[] = {{&S21085[103], 5}, {NULL}};
const Segment SGI63564[] = {{&S5944[44], 9}, {NULL}};
const Segment SGI63565[] = {{&S5944[44], 21}, {NULL}};
const Segment SGI63566[] = {{&S5944[30], 6}, {NULL}};
const Segment SGI63567[] = {{&S5944[31], 3}, {NULL}};
const Segment SGI63568[] = {{&S5944[79], 6}, {NULL}};
const Segment SGI63569[] = {{&S21086[44], 29}, {NULL}};
const Segment SGI63570[] = {{&S21086[44], 46}, {NULL}};
const Segment SGI63571[] = {{&S21086[30], 6}, {NULL}};
const Segment SGI63572[] = {{&S21086[138], 3}, {NULL}};
const Segment SGI63573[] = {{&S21086[104], 5}, {NULL}};
const Segment SGI63574[] = {{&S5945[44], 6}, {NULL}};
const Segment SGI63575[] = {{&S5945[44], 18}, {NULL}};
const Segment SGI63576[] = {{&S5945[30], 6}, {NULL}};
const Segment SGI63577[] = {{&S5945[31], 3}, {NULL}};
const Segment SGI63578[] = {{&S5945[76], 6}, {NULL}};
const Segment SGI63579[] = {{&S21087[44], 31}, {NULL}};
const Segment SGI63580[] = {{&S21087[44], 48}, {NULL}};
const Segment SGI63581[] = {{&S21087[30], 6}, {NULL}};
const Segment SGI63582[] = {{&S21087[140], 3}, {NULL}};
const Segment SGI63583[] = {{&S21087[106], 5}, {NULL}};
const Segment SGI63584[] = {{&S5946[44], 11}, {NULL}};
const Segment SGI63585[] = {{&S5946[44], 23}, {NULL}};
const Segment SGI63586[] = {{&S5946[30], 6}, {NULL}};
const Segment SGI63587[] = {{&S5946[31], 3}, {NULL}};
const Segment SGI63588[] = {{&S5946[81], 6}, {NULL}};
const Segment SGI63589[] = {{&S21088[42], 11}, {NULL}};
const Segment SGI63590[] = {{&S21088[30], 6}, {NULL}};
const Segment SGI63591[] = {{&S21088[118], 3}, {NULL}};
const Segment SGI63592[] = {{&S21088[84], 5}, {NULL}};
const Segment SGI63593[] = {{&S5947[44], 12}, {NULL}};
const Segment SGI63594[] = {{&S5947[44], 24}, {NULL}};
const Segment SGI63595[] = {{&S5947[30], 6}, {NULL}};
const Segment SGI63596[] = {{&S5947[31], 3}, {NULL}};
const Segment SGI63597[] = {{&S5947[82], 6}, {NULL}};
const Segment SGI63598[] = {{&S5948[44], 9}, {NULL}};
const Segment SGI63599[] = {{&S5948[44], 21}, {NULL}};
const Segment SGI63600[] = {{&S5948[30], 6}, {NULL}};
const Segment SGI63601[] = {{&S21090[44], 29}, {NULL}};
const Segment SGI63602[] = {{&S21090[44], 46}, {NULL}};
const Segment SGI63603[] = {{&S21090[30], 6}, {NULL}};
const Segment SGI63604[] = {{&S5948[31], 3}, {NULL}};
const Segment SGI63605[] = {{&S5948[79], 6}, {NULL}};
const Segment SGI63606[] = {{&S21090[138], 3}, {NULL}};
const Segment SGI63607[] = {{&S21090[104], 5}, {NULL}};
const Segment SGI63608[] = {{&S5949[44], 15}, {NULL}};
const Segment SGI63609[] = {{&S5949[44], 27}, {NULL}};
const Segment SGI63610[] = {{&S5949[30], 6}, {NULL}};
const Segment SGI63611[] = {{&S5949[31], 3}, {NULL}};
const Segment SGI63612[] = {{&S5949[85], 6}, {NULL}};
const Segment SGI63613[] = {{&S21091[44], 29}, {NULL}};
const Segment SGI63614[] = {{&S21091[44], 46}, {NULL}};
const Segment SGI63615[] = {{&S21091[30], 6}, {NULL}};
const Segment SGI63616[] = {{&S21091[138], 3}, {NULL}};
const Segment SGI63617[] = {{&S21091[104], 5}, {NULL}};
const Segment SGI63618[] = {{&S5950[44], 4}, {NULL}};
const Segment SGI63619[] = {{&S5950[44], 16}, {NULL}};
const Segment SGI63620[] = {{&S5950[30], 6}, {NULL}};
const Segment SGI63621[] = {{&S5950[31], 3}, {NULL}};
const Segment SGI63622[] = {{&S5950[74], 6}, {NULL}};
const Segment SGI63623[] = {{&S21092[44], 31}, {NULL}};
const Segment SGI63624[] = {{&S21092[44], 48}, {NULL}};
const Segment SGI63625[] = {{&S21092[30], 6}, {NULL}};
const Segment SGI63626[] = {{&S21092[140], 3}, {NULL}};
const Segment SGI63627[] = {{&S5951[44], 7}, {NULL}};
const Segment SGI63628[] = {{&S21092[106], 5}, {NULL}};
const Segment SGI63629[] = {{&S5951[44], 18}, {NULL}};
const Segment SGI63630[] = {{&S5951[30], 6}, {NULL}};
const Segment SGI63631[] = {{&S5951[31], 3}, {NULL}};
const Segment SGI63632[] = {{&S5951[76], 6}, {NULL}};
const Segment SGI63633[] = {{&S21093[38], 11}, {NULL}};
const Segment SGI63634[] = {{&S21093[31], 6}, {NULL}};
const Segment SGI63635[] = {{&S5952[44], 11}, {NULL}};
const Segment SGI63636[] = {{&S5952[44], 23}, {NULL}};
const Segment SGI63637[] = {{&S5952[30], 6}, {NULL}};
const Segment SGI63638[] = {{&S5952[31], 3}, {NULL}};
const Segment SGI63639[] = {{&S5952[81], 6}, {NULL}};
const Segment SGI63640[] = {{&S21094[44], 28}, {NULL}};
const Segment SGI63641[] = {{&S21094[44], 45}, {NULL}};
const Segment SGI63642[] = {{&S21094[30], 6}, {NULL}};
const Segment SGI63643[] = {{&S21094[137], 3}, {NULL}};
const Segment SGI63644[] = {{&S21094[103], 5}, {NULL}};
const Segment SGI63645[] = {{&S5953[44], 10}, {NULL}};
const Segment SGI63646[] = {{&S5953[44], 22}, {NULL}};
const Segment SGI63647[] = {{&S5953[30], 6}, {NULL}};
const Segment SGI63648[] = {{&S5953[31], 3}, {NULL}};
const Segment SGI63649[] = {{&S5953[80], 6}, {NULL}};
const Segment SGI63650[] = {{&S21095[44], 29}, {NULL}};
const Segment SGI63651[] = {{&S21095[44], 46}, {NULL}};
const Segment SGI63652[] = {{&S21095[30], 6}, {NULL}};
const Segment SGI63653[] = {{&S21095[138], 3}, {NULL}};
const Segment SGI63654[] = {{&S21095[104], 5}, {NULL}};
const Segment SGI63655[] = {{&S5955[44], 7}, {NULL}};
const Segment SGI63656[] = {{&S5955[44], 19}, {NULL}};
const Segment SGI63657[] = {{&S5955[30], 6}, {NULL}};
const Segment SGI63658[] = {{&S21096[38], 11}, {NULL}};
const Segment SGI63659[] = {{&S21096[31], 6}, {NULL}};
const Segment SGI63660[] = {{&S5955[31], 3}, {NULL}};
const Segment SGI63661[] = {{&S5955[77], 6}, {NULL}};
const Segment SGI63662[] = {{&S21097[44], 29}, {NULL}};
const Segment SGI63663[] = {{&S21097[44], 46}, {NULL}};
const Segment SGI63664[] = {{&S21097[30], 6}, {NULL}};
const Segment SGI63665[] = {{&S21097[138], 3}, {NULL}};
const Segment SGI63666[] = {{&S21097[104], 5}, {NULL}};
const Segment SGI63667[] = {{&S5957[44], 9}, {NULL}};
const Segment SGI63668[] = {{&S5957[44], 21}, {NULL}};
const Segment SGI63669[] = {{&S5957[30], 6}, {NULL}};
const Segment SGI63670[] = {{&S5957[31], 3}, {NULL}};
const Segment SGI63671[] = {{&S5957[79], 6}, {NULL}};
const Segment SGI63672[] = {{&S21098[44], 11}, {NULL}};
const Segment SGI63673[] = {{&S21098[44], 28}, {NULL}};
const Segment SGI63674[] = {{&S21098[30], 6}, {NULL}};
const Segment SGI63675[] = {{&S21098[89], 6}, {NULL}};
const Segment SGI63676[] = {{&S5958[44], 11}, {NULL}};
const Segment SGI63677[] = {{&S5958[44], 23}, {NULL}};
const Segment SGI63678[] = {{&S5958[30], 6}, {NULL}};
const Segment SGI63679[] = {{&S5958[31], 3}, {NULL}};
const Segment SGI63680[] = {{&S5958[81], 6}, {NULL}};
const Segment SGI63681[] = {{&S5959[44], 7}, {NULL}};
const Segment SGI63682[] = {{&S5959[44], 19}, {NULL}};
const Segment SGI63683[] = {{&S5959[30], 6}, {NULL}};
const Segment SGI63684[] = {{&S21100[44], 29}, {NULL}};
const Segment SGI63685[] = {{&S21100[44], 46}, {NULL}};
const Segment SGI63686[] = {{&S5959[31], 3}, {NULL}};
const Segment SGI63687[] = {{&S21100[30], 6}, {NULL}};
const Segment SGI63688[] = {{&S5959[77], 6}, {NULL}};
const Segment SGI63689[] = {{&S21100[138], 3}, {NULL}};
const Segment SGI63690[] = {{&S21100[104], 5}, {NULL}};
const Segment SGI63691[] = {{&S5960[44], 9}, {NULL}};
const Segment SGI63692[] = {{&S5960[44], 21}, {NULL}};
const Segment SGI63693[] = {{&S5960[30], 6}, {NULL}};
const Segment SGI63694[] = {{&S5960[31], 3}, {NULL}};
const Segment SGI63695[] = {{&S5960[79], 6}, {NULL}};
const Segment SGI63696[] = {{&S5961[44], 10}, {NULL}};
const Segment SGI63697[] = {{&S5961[44], 22}, {NULL}};
const Segment SGI63698[] = {{&S5961[30], 6}, {NULL}};
const Segment SGI63699[] = {{&S5961[31], 3}, {NULL}};
const Segment SGI63700[] = {{&S5961[80], 6}, {NULL}};
const Segment SGI63701[] = {{&S21104[42], 11}, {NULL}};
const Segment SGI63702[] = {{&S21104[30], 6}, {NULL}};
const Segment SGI63703[] = {{&S5962[44], 9}, {NULL}};
const Segment SGI63704[] = {{&S5962[44], 21}, {NULL}};
const Segment SGI63705[] = {{&S21104[118], 3}, {NULL}};
const Segment SGI63706[] = {{&S5962[30], 6}, {NULL}};
const Segment SGI63707[] = {{&S21104[84], 5}, {NULL}};
const Segment SGI63708[] = {{&S5962[31], 3}, {NULL}};
const Segment SGI63709[] = {{&S5962[79], 6}, {NULL}};
const Segment SGI63710[] = {{&S21105[44], 31}, {NULL}};
const Segment SGI63711[] = {{&S21105[44], 48}, {NULL}};
const Segment SGI63712[] = {{&S21105[30], 6}, {NULL}};
const Segment SGI63713[] = {{&S21105[140], 3}, {NULL}};
const Segment SGI63714[] = {{&S21105[106], 5}, {NULL}};
const Segment SGI63715[] = {{&S5965[44], 6}, {NULL}};
const Segment SGI63716[] = {{&S5965[44], 18}, {NULL}};
const Segment SGI63717[] = {{&S5965[30], 6}, {NULL}};
const Segment SGI63718[] = {{&S5965[31], 3}, {NULL}};
const Segment SGI63719[] = {{&S5965[76], 6}, {NULL}};
const Segment SGI63720[] = {{&S5967[44], 8}, {NULL}};
const Segment SGI63721[] = {{&S5967[44], 20}, {NULL}};
const Segment SGI63722[] = {{&S5967[30], 6}, {NULL}};
const Segment SGI63723[] = {{&S5967[31], 3}, {NULL}};
const Segment SGI63724[] = {{&S5967[78], 6}, {NULL}};
const Segment SGI63725[] = {{&S5970[44], 8}, {NULL}};
const Segment SGI63726[] = {{&S5970[44], 20}, {NULL}};
const Segment SGI63727[] = {{&S5970[30], 6}, {NULL}};
const Segment SGI63728[] = {{&S5970[31], 3}, {NULL}};
const Segment SGI63729[] = {{&S5970[78], 6}, {NULL}};
const Segment SGI63730[] = {{&S5971[44], 9}, {NULL}};
const Segment SGI63731[] = {{&S5971[44], 21}, {NULL}};
const Segment SGI63732[] = {{&S5971[30], 6}, {NULL}};
const Segment SGI63733[] = {{&S5971[31], 3}, {NULL}};
const Segment SGI63734[] = {{&S5971[79], 6}, {NULL}};
const Segment SGI63735[] = {{&S5972[44], 12}, {NULL}};
const Segment SGI63736[] = {{&S5972[44], 24}, {NULL}};
const Segment SGI63737[] = {{&S5972[30], 6}, {NULL}};
const Segment SGI63738[] = {{&S5972[31], 3}, {NULL}};
const Segment SGI63739[] = {{&S5972[82], 6}, {NULL}};
const Segment SGI63740[] = {{&S5973[44], 9}, {NULL}};
const Segment SGI63741[] = {{&S5973[44], 21}, {NULL}};
const Segment SGI63742[] = {{&S5973[30], 6}, {NULL}};
const Segment SGI63743[] = {{&S5973[31], 3}, {NULL}};
const Segment SGI63744[] = {{&S5973[79], 6}, {NULL}};
const Segment SGI63745[] = {{&S14924[44], 10}, {NULL}};
const Segment SGI63746[] = {{&S14924[44], 21}, {NULL}};
const Segment SGI63747[] = {{&S14924[30], 6}, {NULL}};
const Segment SGI63748[] = {{&S14924[113], 3}, {NULL}};
const Segment SGI63749[] = {{&S14924[79], 5}, {NULL}};
const Segment SGI63750[] = {{&S5974[44], 4}, {NULL}};
const Segment SGI63751[] = {{&S5974[44], 16}, {NULL}};
const Segment SGI63752[] = {{&S5974[30], 6}, {NULL}};
const Segment SGI63753[] = {{&S5974[31], 3}, {NULL}};
const Segment SGI63754[] = {{&S5974[74], 6}, {NULL}};
const Segment SGI63755[] = {{&S5976[44], 15}, {NULL}};
const Segment SGI63756[] = {{&S5976[44], 27}, {NULL}};
const Segment SGI63757[] = {{&S5976[30], 6}, {NULL}};
const Segment SGI63758[] = {{&S5976[31], 3}, {NULL}};
const Segment SGI63759[] = {{&S5976[85], 6}, {NULL}};
const Segment SGI63760[] = {{&S5977[44], 25}, {NULL}};
const Segment SGI63761[] = {{&S5977[44], 52}, {NULL}};
const Segment SGI63762[] = {{&S5977[30], 6}, {NULL}};
const Segment SGI63763[] = {{&S5977[31], 3}, {NULL}};
const Segment SGI63764[] = {{&S5977[110], 6}, {NULL}};
const Segment SGI63765[] = {{&S14931[44], 10}, {NULL}};
const Segment SGI63766[] = {{&S14931[44], 23}, {NULL}};
const Segment SGI63767[] = {{&S14931[30], 6}, {NULL}};
const Segment SGI63768[] = {{&S14931[115], 3}, {NULL}};
const Segment SGI63769[] = {{&S14931[81], 5}, {NULL}};
const Segment SGI63770[] = {{&S5980[44], 12}, {NULL}};
const Segment SGI63771[] = {{&S5980[44], 24}, {NULL}};
const Segment SGI63772[] = {{&S5980[30], 6}, {NULL}};
const Segment SGI63773[] = {{&S5980[31], 3}, {NULL}};
const Segment SGI63774[] = {{&S5980[82], 6}, {NULL}};
const Segment SGI63775[] = {{&S14935[38], 10}, {NULL}};
const Segment SGI63776[] = {{&S14935[31], 6}, {NULL}};
const Segment SGI63777[] = {{&S5983[44], 8}, {NULL}};
const Segment SGI63778[] = {{&S5983[44], 20}, {NULL}};
const Segment SGI63779[] = {{&S5983[30], 6}, {NULL}};
const Segment SGI63780[] = {{&S5983[31], 3}, {NULL}};
const Segment SGI63781[] = {{&S5983[78], 6}, {NULL}};
const Segment SGI63782[] = {{&S5984[44], 7}, {NULL}};
const Segment SGI63783[] = {{&S5984[44], 19}, {NULL}};
const Segment SGI63784[] = {{&S5984[30], 6}, {NULL}};
const Segment SGI63785[] = {{&S5984[31], 3}, {NULL}};
const Segment SGI63786[] = {{&S5984[77], 6}, {NULL}};
const Segment SGI63787[] = {{&S5985[44], 12}, {NULL}};
const Segment SGI63788[] = {{&S5985[44], 24}, {NULL}};
const Segment SGI63789[] = {{&S5985[30], 6}, {NULL}};
const Segment SGI63790[] = {{&S5985[31], 3}, {NULL}};
const Segment SGI63791[] = {{&S5985[82], 6}, {NULL}};
const Segment SGI63792[] = {{&S5987[44], 7}, {NULL}};
const Segment SGI63793[] = {{&S5987[44], 19}, {NULL}};
const Segment SGI63794[] = {{&S5987[30], 6}, {NULL}};
const Segment SGI63795[] = {{&S5987[31], 3}, {NULL}};
const Segment SGI63796[] = {{&S5987[77], 6}, {NULL}};
const Segment SGI63797[] = {{&S5988[44], 7}, {NULL}};
const Segment SGI63798[] = {{&S5988[44], 19}, {NULL}};
const Segment SGI63799[] = {{&S5988[30], 6}, {NULL}};
const Segment SGI63800[] = {{&S5988[31], 3}, {NULL}};
const Segment SGI63801[] = {{&S5988[77], 6}, {NULL}};
const Segment SGI63802[] = {{&S14946[44], 10}, {NULL}};
const Segment SGI63803[] = {{&S14946[44], 27}, {NULL}};
const Segment SGI63804[] = {{&S14946[30], 6}, {NULL}};
const Segment SGI63805[] = {{&S14946[119], 3}, {NULL}};
const Segment SGI63806[] = {{&S14946[85], 5}, {NULL}};
const Segment SGI63807[] = {{&S5989[44], 6}, {NULL}};
const Segment SGI63808[] = {{&S5989[44], 18}, {NULL}};
const Segment SGI63809[] = {{&S5989[30], 6}, {NULL}};
const Segment SGI63810[] = {{&S5989[31], 3}, {NULL}};
const Segment SGI63811[] = {{&S5989[76], 6}, {NULL}};
const Segment SGI63812[] = {{&S5990[44], 7}, {NULL}};
const Segment SGI63813[] = {{&S5990[44], 19}, {NULL}};
const Segment SGI63814[] = {{&S5990[30], 6}, {NULL}};
const Segment SGI63815[] = {{&S5990[80], 4}, {NULL}};
const Segment SGI63816[] = {{&S5990[80], 3}, {NULL}};
const Segment SGI63817[] = {{&S5991[44], 9}, {NULL}};
const Segment SGI63818[] = {{&S5991[44], 21}, {NULL}};
const Segment SGI63819[] = {{&S5991[30], 6}, {NULL}};
const Segment SGI63820[] = {{&S5991[31], 3}, {NULL}};
const Segment SGI63821[] = {{&S5991[79], 6}, {NULL}};
const Segment SGI63822[] = {{&S5992[44], 34}, {NULL}};
const Segment SGI63823[] = {{&S5992[30], 6}, {NULL}};
const Segment SGI63824[] = {{&S5993[44], 8}, {NULL}};
const Segment SGI63825[] = {{&S5993[44], 20}, {NULL}};
const Segment SGI63826[] = {{&S5993[30], 6}, {NULL}};
const Segment SGI63827[] = {{&S5993[31], 3}, {NULL}};
const Segment SGI63828[] = {{&S5993[78], 6}, {NULL}};
const Segment SGI63829[] = {{&S5994[44], 8}, {NULL}};
const Segment SGI63830[] = {{&S5994[44], 20}, {NULL}};
const Segment SGI63831[] = {{&S5994[30], 6}, {NULL}};
const Segment SGI63832[] = {{&S5994[31], 3}, {NULL}};
const Segment SGI63833[] = {{&S5994[78], 6}, {NULL}};
const Segment SGI63834[] = {{&S5995[44], 10}, {NULL}};
const Segment SGI63835[] = {{&S5995[44], 22}, {NULL}};
const Segment SGI63836[] = {{&S5995[30], 6}, {NULL}};
const Segment SGI63837[] = {{&S5995[31], 3}, {NULL}};
const Segment SGI63838[] = {{&S5995[80], 6}, {NULL}};
const Segment SGI63839[] = {{&S5996[44], 14}, {NULL}};
const Segment SGI63840[] = {{&S5996[44], 26}, {NULL}};
const Segment SGI63841[] = {{&S5996[30], 6}, {NULL}};
const Segment SGI63842[] = {{&S5996[31], 3}, {NULL}};
const Segment SGI63843[] = {{&S5996[84], 6}, {NULL}};
const Segment SGI63844[] = {{&S14963[44], 10}, {NULL}};
const Segment SGI63845[] = {{&S14963[44], 27}, {NULL}};
const Segment SGI63846[] = {{&S14963[30], 6}, {NULL}};
const Segment SGI63847[] = {{&S5997[44], 8}, {NULL}};
const Segment SGI63848[] = {{&S5997[44], 20}, {NULL}};
const Segment SGI63849[] = {{&S14963[119], 3}, {NULL}};
const Segment SGI63850[] = {{&S5997[30], 6}, {NULL}};
const Segment SGI63851[] = {{&S14963[85], 5}, {NULL}};
const Segment SGI63852[] = {{&S5997[31], 3}, {NULL}};
const Segment SGI63853[] = {{&S5997[78], 6}, {NULL}};
const Segment SGI63854[] = {{&S5998[44], 10}, {NULL}};
const Segment SGI63855[] = {{&S5998[44], 22}, {NULL}};
const Segment SGI63856[] = {{&S5998[30], 6}, {NULL}};
const Segment SGI63857[] = {{&S5998[31], 3}, {NULL}};
const Segment SGI63858[] = {{&S5998[80], 6}, {NULL}};
const Segment SGI63859[] = {{&S5999[44], 12}, {NULL}};
const Segment SGI63860[] = {{&S5999[44], 24}, {NULL}};
const Segment SGI63861[] = {{&S5999[30], 6}, {NULL}};
const Segment SGI63862[] = {{&S5999[31], 3}, {NULL}};
const Segment SGI63863[] = {{&S5999[82], 6}, {NULL}};
const Segment SGI63864[] = {{&S6001[44], 6}, {NULL}};
const Segment SGI63865[] = {{&S6001[44], 18}, {NULL}};
const Segment SGI63866[] = {{&S6001[30], 6}, {NULL}};
const Segment SGI63867[] = {{&S6001[31], 3}, {NULL}};
const Segment SGI63868[] = {{&S6001[76], 6}, {NULL}};
const Segment SGI63869[] = {{&S6002[44], 6}, {NULL}};
const Segment SGI63870[] = {{&S6002[44], 18}, {NULL}};
const Segment SGI63871[] = {{&S6002[30], 6}, {NULL}};
const Segment SGI63872[] = {{&S6002[31], 3}, {NULL}};
const Segment SGI63873[] = {{&S6002[76], 6}, {NULL}};
const Segment SGI63874[] = {{&S6003[44], 10}, {NULL}};
const Segment SGI63875[] = {{&S6003[44], 22}, {NULL}};
const Segment SGI63876[] = {{&S6003[30], 6}, {NULL}};
const Segment SGI63877[] = {{&S6003[31], 3}, {NULL}};
const Segment SGI63878[] = {{&S6003[80], 6}, {NULL}};
const Segment SGI63879[] = {{&S6005[44], 7}, {NULL}};
const Segment SGI63880[] = {{&S6005[44], 19}, {NULL}};
const Segment SGI63881[] = {{&S6005[30], 6}, {NULL}};
const Segment SGI63882[] = {{&S6005[31], 3}, {NULL}};
const Segment SGI63883[] = {{&S6005[77], 6}, {NULL}};
const Segment SGI63884[] = {{&S6007[44], 12}, {NULL}};
const Segment SGI63885[] = {{&S6007[44], 24}, {NULL}};
const Segment SGI63886[] = {{&S6007[30], 6}, {NULL}};
const Segment SGI63887[] = {{&S6007[31], 3}, {NULL}};
const Segment SGI63888[] = {{&S6007[82], 6}, {NULL}};
const Segment SGI63889[] = {{&S6010[44], 5}, {NULL}};
const Segment SGI63890[] = {{&S6010[44], 17}, {NULL}};
const Segment SGI63891[] = {{&S6010[30], 6}, {NULL}};
const Segment SGI63892[] = {{&S6010[31], 3}, {NULL}};
const Segment SGI63893[] = {{&S6010[75], 6}, {NULL}};
const Segment SGI63894[] = {{&S6011[44], 12}, {NULL}};
const Segment SGI63895[] = {{&S6011[44], 24}, {NULL}};
const Segment SGI63896[] = {{&S6011[30], 6}, {&S6011[52], 16}, {NULL}};
const Segment SGI63897[] = {{&S6011[31], 3}, {NULL}};
const Segment SGI63898[] = {{&S6011[82], 6}, {NULL}};
const Segment SGI63899[] = {{&S6013[44], 9}, {NULL}};
const Segment SGI63900[] = {{&S6013[44], 21}, {NULL}};
const Segment SGI63901[] = {{&S6013[30], 6}, {NULL}};
const Segment SGI63902[] = {{&S6013[31], 3}, {NULL}};
const Segment SGI63903[] = {{&S6013[79], 6}, {NULL}};
const Segment SGI63904[] = {{&S6014[38], 9}, {NULL}};
const Segment SGI63905[] = {{&S6014[31], 6}, {NULL}};
const Segment SGI63906[] = {{&S6016[44], 9}, {NULL}};
const Segment SGI63907[] = {{&S6016[44], 21}, {NULL}};
const Segment SGI63908[] = {{&S6016[30], 6}, {NULL}};
const Segment SGI63909[] = {{&S6016[31], 3}, {NULL}};
const Segment SGI63910[] = {{&S6016[79], 6}, {NULL}};
const Segment SGI63911[] = {{&S6017[44], 7}, {NULL}};
const Segment SGI63912[] = {{&S6017[44], 19}, {NULL}};
const Segment SGI63913[] = {{&S6017[30], 6}, {NULL}};
const Segment SGI63914[] = {{&S6017[31], 3}, {NULL}};
const Segment SGI63915[] = {{&S6017[77], 6}, {NULL}};
const Segment SGI63916[] = {{&S6018[44], 5}, {NULL}};
const Segment SGI63917[] = {{&S6018[44], 17}, {NULL}};
const Segment SGI63918[] = {{&S6018[30], 6}, {NULL}};
const Segment SGI63919[] = {{&S6018[31], 3}, {NULL}};
const Segment SGI63920[] = {{&S6018[75], 6}, {NULL}};
const Segment SGI63921[] = {{&S6020[44], 9}, {NULL}};
const Segment SGI63922[] = {{&S6020[44], 21}, {NULL}};
const Segment SGI63923[] = {{&S6020[30], 6}, {NULL}};
const Segment SGI63924[] = {{&S6020[31], 3}, {NULL}};
const Segment SGI63925[] = {{&S6020[79], 6}, {NULL}};
const Segment SGI63926[] = {{&S6021[44], 5}, {NULL}};
const Segment SGI63927[] = {{&S6021[44], 17}, {NULL}};
const Segment SGI63928[] = {{&S6021[30], 6}, {NULL}};
const Segment SGI63929[] = {{&S6021[31], 3}, {NULL}};
const Segment SGI63930[] = {{&S6021[75], 6}, {NULL}};
const Segment SGI63931[] = {{&S6022[44], 12}, {NULL}};
const Segment SGI63932[] = {{&S6022[44], 24}, {NULL}};
const Segment SGI63933[] = {{&S6022[30], 6}, {NULL}};
const Segment SGI63934[] = {{&S6022[31], 3}, {NULL}};
const Segment SGI63935[] = {{&S6022[82], 6}, {NULL}};
const Segment SGI63936[] = {{&S6023[44], 16}, {NULL}};
const Segment SGI63937[] = {{&S6023[44], 28}, {NULL}};
const Segment SGI63938[] = {{&S6023[30], 6}, {NULL}};
const Segment SGI63939[] = {{&S6023[31], 3}, {NULL}};
const Segment SGI63940[] = {{&S6023[86], 6}, {NULL}};
const Segment SGI63941[] = {{&S6024[44], 9}, {NULL}};
const Segment SGI63942[] = {{&S6024[44], 21}, {NULL}};
const Segment SGI63943[] = {{&S6024[30], 6}, {NULL}};
const Segment SGI63944[] = {{&S6024[31], 3}, {NULL}};
const Segment SGI63945[] = {{&S6024[79], 6}, {NULL}};
const Segment SGI63946[] = {{&S6026[44], 10}, {NULL}};
const Segment SGI63947[] = {{&S6026[44], 22}, {NULL}};
const Segment SGI63948[] = {{&S6026[30], 6}, {NULL}};
const Segment SGI63949[] = {{&S6026[31], 3}, {NULL}};
const Segment SGI63950[] = {{&S6026[80], 6}, {NULL}};
const Segment SGI63951[] = {{&S6027[44], 16}, {NULL}};
const Segment SGI63952[] = {{&S6027[44], 28}, {NULL}};
const Segment SGI63953[] = {{&S6027[30], 6}, {NULL}};
const Segment SGI63954[] = {{&S6027[31], 3}, {NULL}};
const Segment SGI63955[] = {{&S6027[86], 6}, {NULL}};
const Segment SGI63956[] = {{&S6028[44], 14}, {NULL}};
const Segment SGI63957[] = {{&S6028[44], 26}, {NULL}};
const Segment SGI63958[] = {{&S6028[30], 6}, {NULL}};
const Segment SGI63959[] = {{&S6028[31], 3}, {NULL}};
const Segment SGI63960[] = {{&S6028[84], 6}, {NULL}};
const Segment SGI63961[] = {{&S6400[44], 8}, {NULL}};
const Segment SGI63962[] = {{&S6400[44], 20}, {NULL}};
const Segment SGI63963[] = {{&S6400[30], 6}, {NULL}};
const Segment SGI63964[] = {{&S6400[31], 3}, {NULL}};
const Segment SGI63965[] = {{&S6400[78], 5}, {NULL}};
const Segment SGI63966[] = {{&S6029[44], 12}, {NULL}};
const Segment SGI63967[] = {{&S6029[44], 24}, {NULL}};
const Segment SGI63968[] = {{&S6029[30], 6}, {NULL}};
const Segment SGI63969[] = {{&S6029[31], 3}, {NULL}};
const Segment SGI63970[] = {{&S6029[82], 6}, {NULL}};
const Segment SGI63971[] = {{&S6030[44], 8}, {NULL}};
const Segment SGI63972[] = {{&S6030[44], 20}, {NULL}};
const Segment SGI63973[] = {{&S6030[30], 6}, {NULL}};
const Segment SGI63974[] = {{&S6030[31], 3}, {NULL}};
const Segment SGI63975[] = {{&S6030[78], 6}, {NULL}};
const Segment SGI63976[] = {{&S6031[44], 14}, {NULL}};
const Segment SGI63977[] = {{&S6031[44], 26}, {NULL}};
const Segment SGI63978[] = {{&S6031[30], 6}, {NULL}};
const Segment SGI63979[] = {{&S6031[31], 3}, {NULL}};
const Segment SGI63980[] = {{&S6031[84], 6}, {NULL}};
const Segment SGI63981[] = {{&S15032[44], 14}, {NULL}};
const Segment SGI63982[] = {{&S6035[44], 11}, {NULL}};
const Segment SGI63983[] = {{&S15032[44], 25}, {NULL}};
const Segment SGI63984[] = {{&S6035[44], 23}, {NULL}};
const Segment SGI63985[] = {{&S15032[30], 6}, {NULL}};
const Segment SGI63986[] = {{&S6035[30], 6}, {NULL}};
const Segment SGI63987[] = {{&S15032[117], 3}, {NULL}};
const Segment SGI63988[] = {{&S6035[31], 3}, {NULL}};
const Segment SGI63989[] = {{&S6035[81], 6}, {NULL}};
const Segment SGI63990[] = {{&S15032[83], 5}, {NULL}};
const Segment SGI63991[] = {{&S6036[44], 7}, {NULL}};
const Segment SGI63992[] = {{&S6036[44], 19}, {NULL}};
const Segment SGI63993[] = {{&S6036[30], 6}, {NULL}};
const Segment SGI63994[] = {{&S6036[31], 3}, {NULL}};
const Segment SGI63995[] = {{&S6036[77], 6}, {NULL}};
const Segment SGI63996[] = {{&S6037[44], 5}, {NULL}};
const Segment SGI63997[] = {{&S6037[44], 17}, {NULL}};
const Segment SGI63998[] = {{&S6037[30], 6}, {NULL}};
const Segment SGI63999[] = {{&S6037[31], 3}, {NULL}};
const Segment SGI64000[] = {{&S6037[75], 6}, {NULL}};
const Segment SGI64001[] = {{&S6038[44], 7}, {NULL}};
const Segment SGI64002[] = {{&S6038[44], 19}, {NULL}};
const Segment SGI64003[] = {{&S6038[30], 6}, {NULL}};
const Segment SGI64004[] = {{&S6038[31], 3}, {NULL}};
const Segment SGI64005[] = {{&S6038[77], 6}, {NULL}};
const Segment SGI64006[] = {{&S6039[44], 12}, {NULL}};
const Segment SGI64007[] = {{&S6039[44], 24}, {NULL}};
const Segment SGI64008[] = {{&S6039[30], 6}, {NULL}};
const Segment SGI64009[] = {{&S6039[31], 3}, {NULL}};
const Segment SGI64010[] = {{&S6039[82], 6}, {NULL}};
const Segment SGI64011[] = {{&S15045[38], 14}, {NULL}};
const Segment SGI64012[] = {{&S15045[31], 6}, {NULL}};
const Segment SGI64013[] = {{&S6045[44], 9}, {NULL}};
const Segment SGI64014[] = {{&S6045[44], 21}, {NULL}};
const Segment SGI64015[] = {{&S6045[30], 6}, {NULL}};
const Segment SGI64016[] = {{&S6045[31], 3}, {NULL}};
const Segment SGI64017[] = {{&S6045[79], 6}, {NULL}};
const Segment SGI64018[] = {{&S6046[44], 10}, {NULL}};
const Segment SGI64019[] = {{&S6046[44], 22}, {NULL}};
const Segment SGI64020[] = {{&S6046[30], 6}, {NULL}};
const Segment SGI64021[] = {{&S6046[31], 3}, {NULL}};
const Segment SGI64022[] = {{&S6046[80], 6}, {NULL}};
const Segment SGI64023[] = {{&S6047[44], 10}, {NULL}};
const Segment SGI64024[] = {{&S6047[44], 22}, {NULL}};
const Segment SGI64025[] = {{&S6047[30], 6}, {NULL}};
const Segment SGI64026[] = {{&S6047[31], 3}, {NULL}};
const Segment SGI64027[] = {{&S6047[80], 6}, {NULL}};
const Segment SGI64028[] = {{&S6048[44], 8}, {NULL}};
const Segment SGI64029[] = {{&S6048[44], 20}, {NULL}};
const Segment SGI64030[] = {{&S6048[30], 6}, {NULL}};
const Segment SGI64031[] = {{&S6048[31], 3}, {NULL}};
const Segment SGI64032[] = {{&S6048[78], 6}, {NULL}};
const Segment SGI64033[] = {{&S15053[44], 19}, {NULL}};
const Segment SGI64034[] = {{&S15053[44], 30}, {NULL}};
const Segment SGI64035[] = {{&S15053[30], 6}, {NULL}};
const Segment SGI64036[] = {{&S15053[122], 3}, {NULL}};
const Segment SGI64037[] = {{&S15053[88], 5}, {NULL}};
const Segment SGI64038[] = {{&S6050[44], 13}, {NULL}};
const Segment SGI64039[] = {{&S6050[44], 25}, {NULL}};
const Segment SGI64040[] = {{&S6050[30], 6}, {NULL}};
const Segment SGI64041[] = {{&S6050[31], 3}, {NULL}};
const Segment SGI64042[] = {{&S6050[83], 6}, {NULL}};
const Segment SGI64043[] = {{&S6051[44], 7}, {NULL}};
const Segment SGI64044[] = {{&S6051[44], 24}, {NULL}};
const Segment SGI64045[] = {{&S6051[30], 6}, {NULL}};
const Segment SGI64046[] = {{&S6051[85], 6}, {NULL}};
const Segment SGI64047[] = {{&S6054[44], 12}, {NULL}};
const Segment SGI64048[] = {{&S6054[44], 24}, {NULL}};
const Segment SGI64049[] = {{&S6054[30], 6}, {NULL}};
const Segment SGI64050[] = {{&S6054[31], 3}, {NULL}};
const Segment SGI64051[] = {{&S6054[82], 6}, {NULL}};
const Segment SGI64052[] = {{&S6055[52], 6}, {NULL}};
const Segment SGI64053[] = {{&S6055[114], 6}, {NULL}};
const Segment SGI64054[] = {{&S6063[44], 11}, {NULL}};
const Segment SGI64055[] = {{&S6063[44], 23}, {NULL}};
const Segment SGI64056[] = {{&S6063[30], 6}, {NULL}};
const Segment SGI64057[] = {{&S6063[31], 3}, {NULL}};
const Segment SGI64058[] = {{&S6063[81], 6}, {NULL}};
const Segment SGI64059[] = {{&S6064[44], 25}, {NULL}};
const Segment SGI64060[] = {{&S6064[44], 37}, {NULL}};
const Segment SGI64061[] = {{&S6064[30], 6}, {NULL}};
const Segment SGI64062[] = {{&S6064[31], 3}, {NULL}};
const Segment SGI64063[] = {{&S6064[95], 6}, {NULL}};
const Segment SGI64064[] = {{&S6065[44], 4}, {NULL}};
const Segment SGI64065[] = {{&S6065[44], 16}, {NULL}};
const Segment SGI64066[] = {{&S6065[30], 6}, {NULL}};
const Segment SGI64067[] = {{&S6065[31], 3}, {NULL}};
const Segment SGI64068[] = {{&S6065[74], 6}, {NULL}};
const Segment SGI64069[] = {{&S6066[44], 8}, {NULL}};
const Segment SGI64070[] = {{&S6066[44], 20}, {NULL}};
const Segment SGI64071[] = {{&S6066[30], 6}, {NULL}};
const Segment SGI64072[] = {{&S6066[31], 3}, {NULL}};
const Segment SGI64073[] = {{&S6066[78], 6}, {NULL}};
const Segment SGI64074[] = {{&S6067[44], 18}, {NULL}};
const Segment SGI64075[] = {{&S6067[44], 30}, {NULL}};
const Segment SGI64076[] = {{&S6067[30], 6}, {NULL}};
const Segment SGI64077[] = {{&S6067[31], 3}, {NULL}};
const Segment SGI64078[] = {{&S6067[88], 6}, {NULL}};
const Segment SGI64079[] = {{&S6068[44], 5}, {NULL}};
const Segment SGI64080[] = {{&S6068[44], 17}, {NULL}};
const Segment SGI64081[] = {{&S6068[30], 6}, {NULL}};
const Segment SGI64082[] = {{&S6068[31], 3}, {NULL}};
const Segment SGI64083[] = {{&S6068[75], 6}, {NULL}};
const Segment SGI64084[] = {{&S6070[44], 9}, {NULL}};
const Segment SGI64085[] = {{&S6070[44], 21}, {NULL}};
const Segment SGI64086[] = {{&S6070[30], 6}, {NULL}};
const Segment SGI64087[] = {{&S6070[31], 3}, {NULL}};
const Segment SGI64088[] = {{&S6070[79], 6}, {NULL}};
const Segment SGI64089[] = {{&S6071[44], 14}, {NULL}};
const Segment SGI64090[] = {{&S6071[44], 26}, {NULL}};
const Segment SGI64091[] = {{&S6071[30], 6}, {NULL}};
const Segment SGI64092[] = {{&S6071[31], 3}, {NULL}};
const Segment SGI64093[] = {{&S6071[84], 6}, {NULL}};
const Segment SGI64094[] = {{&S6072[44], 9}, {NULL}};
const Segment SGI64095[] = {{&S6072[44], 21}, {NULL}};
const Segment SGI64096[] = {{&S6072[30], 6}, {NULL}};
const Segment SGI64097[] = {{&S6072[31], 3}, {NULL}};
const Segment SGI64098[] = {{&S6072[79], 6}, {NULL}};
const Segment SGI64099[] = {{&S6081[44], 10}, {NULL}};
const Segment SGI64100[] = {{&S6081[44], 22}, {NULL}};
const Segment SGI64101[] = {{&S6081[30], 6}, {NULL}};
const Segment SGI64102[] = {{&S6081[31], 3}, {NULL}};
const Segment SGI64103[] = {{&S6081[80], 6}, {NULL}};
const Segment SGI64104[] = {{&S6083[44], 6}, {NULL}};
const Segment SGI64105[] = {{&S6083[44], 18}, {NULL}};
const Segment SGI64106[] = {{&S6083[30], 6}, {NULL}};
const Segment SGI64107[] = {{&S6083[31], 3}, {NULL}};
const Segment SGI64108[] = {{&S6083[76], 6}, {NULL}};
const Segment SGI64109[] = {{&S6084[44], 10}, {NULL}};
const Segment SGI64110[] = {{&S6084[44], 22}, {NULL}};
const Segment SGI64111[] = {{&S6084[30], 6}, {NULL}};
const Segment SGI64112[] = {{&S6084[31], 3}, {NULL}};
const Segment SGI64113[] = {{&S6084[80], 6}, {NULL}};
const Segment SGI64114[] = {{&S6085[44], 9}, {NULL}};
const Segment SGI64115[] = {{&S6085[44], 21}, {NULL}};
const Segment SGI64116[] = {{&S6085[30], 6}, {NULL}};
const Segment SGI64117[] = {{&S6085[31], 3}, {NULL}};
const Segment SGI64118[] = {{&S6085[79], 6}, {NULL}};
const Segment SGI64119[] = {{&S6087[44], 6}, {NULL}};
const Segment SGI64120[] = {{&S6087[44], 18}, {NULL}};
const Segment SGI64121[] = {{&S6087[30], 6}, {NULL}};
const Segment SGI64122[] = {{&S6087[31], 3}, {NULL}};
const Segment SGI64123[] = {{&S6087[76], 6}, {NULL}};
const Segment SGI64124[] = {{&S6089[44], 9}, {NULL}};
const Segment SGI64125[] = {{&S6089[44], 21}, {NULL}};
const Segment SGI64126[] = {{&S6089[30], 6}, {NULL}};
const Segment SGI64127[] = {{&S6089[31], 3}, {NULL}};
const Segment SGI64128[] = {{&S6089[79], 6}, {NULL}};
const Segment SGI64129[] = {{&S6090[44], 7}, {NULL}};
const Segment SGI64130[] = {{&S6090[44], 19}, {NULL}};
const Segment SGI64131[] = {{&S6090[30], 6}, {NULL}};
const Segment SGI64132[] = {{&S6090[31], 3}, {NULL}};
const Segment SGI64133[] = {{&S6090[77], 6}, {NULL}};
const Segment SGI64134[] = {{&S6091[44], 13}, {NULL}};
const Segment SGI64135[] = {{&S6091[44], 25}, {NULL}};
const Segment SGI64136[] = {{&S6091[30], 6}, {NULL}};
const Segment SGI64137[] = {{&S6091[31], 3}, {NULL}};
const Segment SGI64138[] = {{&S6091[83], 6}, {NULL}};
const Segment SGI64139[] = {{&S6092[44], 9}, {NULL}};
const Segment SGI64140[] = {{&S6092[44], 21}, {NULL}};
const Segment SGI64141[] = {{&S6092[30], 6}, {NULL}};
const Segment SGI64142[] = {{&S6092[31], 3}, {NULL}};
const Segment SGI64143[] = {{&S6092[79], 6}, {NULL}};
const Segment SGI64144[] = {{&S6093[44], 8}, {NULL}};
const Segment SGI64145[] = {{&S6093[44], 20}, {NULL}};
const Segment SGI64146[] = {{&S6093[30], 6}, {NULL}};
const Segment SGI64147[] = {{&S6093[31], 3}, {NULL}};
const Segment SGI64148[] = {{&S6093[78], 6}, {NULL}};
const Segment SGI64149[] = {{&S6094[44], 7}, {NULL}};
const Segment SGI64150[] = {{&S6094[44], 19}, {NULL}};
const Segment SGI64151[] = {{&S6094[30], 6}, {NULL}};
const Segment SGI64152[] = {{&S6094[31], 3}, {NULL}};
const Segment SGI64153[] = {{&S6094[77], 6}, {NULL}};
const Segment SGI64154[] = {{&S6095[44], 8}, {NULL}};
const Segment SGI64155[] = {{&S6095[44], 20}, {NULL}};
const Segment SGI64156[] = {{&S6095[30], 6}, {NULL}};
const Segment SGI64157[] = {{&S6095[31], 3}, {NULL}};
const Segment SGI64158[] = {{&S6095[78], 6}, {NULL}};
const Segment SGI64159[] = {{&S6102[44], 7}, {NULL}};
const Segment SGI64160[] = {{&S6102[44], 18}, {NULL}};
const Segment SGI64161[] = {{&S6102[30], 6}, {NULL}};
const Segment SGI64162[] = {{&S6102[31], 3}, {NULL}};
const Segment SGI64163[] = {{&S6102[76], 6}, {NULL}};
const Segment SGI64164[] = {{&S6105[44], 11}, {NULL}};
const Segment SGI64165[] = {{&S6105[44], 23}, {NULL}};
const Segment SGI64166[] = {{&S6105[30], 6}, {NULL}};
const Segment SGI64167[] = {{&S6105[31], 3}, {NULL}};
const Segment SGI64168[] = {{&S6105[81], 6}, {NULL}};
const Segment SGI64169[] = {{&S6106[44], 6}, {NULL}};
const Segment SGI64170[] = {{&S6106[44], 18}, {NULL}};
const Segment SGI64171[] = {{&S6106[30], 6}, {NULL}};
const Segment SGI64172[] = {{&S6106[31], 3}, {NULL}};
const Segment SGI64173[] = {{&S6106[76], 6}, {NULL}};
const Segment SGI64174[] = {{&S6107[44], 12}, {NULL}};
const Segment SGI64175[] = {{&S6107[44], 24}, {NULL}};
const Segment SGI64176[] = {{&S6107[30], 6}, {NULL}};
const Segment SGI64177[] = {{&S6107[31], 3}, {NULL}};
const Segment SGI64178[] = {{&S6107[82], 6}, {NULL}};
const Segment SGI64179[] = {{&S6110[44], 7}, {NULL}};
const Segment SGI64180[] = {{&S6110[44], 18}, {NULL}};
const Segment SGI64181[] = {{&S6110[30], 6}, {NULL}};
const Segment SGI64182[] = {{&S6110[31], 3}, {NULL}};
const Segment SGI64183[] = {{&S6110[76], 6}, {NULL}};
const Segment SGI64184[] = {{&S21381[44], 10}, {NULL}};
const Segment SGI64185[] = {{&S21381[44], 27}, {NULL}};
const Segment SGI64186[] = {{&S21381[30], 6}, {NULL}};
const Segment SGI64187[] = {{&S21381[119], 3}, {NULL}};
const Segment SGI64188[] = {{&S21381[85], 5}, {NULL}};
const Segment SGI64189[] = {{&S6113[44], 11}, {NULL}};
const Segment SGI64190[] = {{&S6113[44], 23}, {NULL}};
const Segment SGI64191[] = {{&S6113[30], 6}, {NULL}};
const Segment SGI64192[] = {{&S21382[44], 11}, {NULL}};
const Segment SGI64193[] = {{&S6113[31], 3}, {NULL}};
const Segment SGI64194[] = {{&S21382[44], 28}, {NULL}};
const Segment SGI64195[] = {{&S6113[81], 6}, {NULL}};
const Segment SGI64196[] = {{&S21382[30], 6}, {NULL}};
const Segment SGI64197[] = {{&S21382[120], 3}, {NULL}};
const Segment SGI64198[] = {{&S21382[86], 5}, {NULL}};
const Segment SGI64199[] = {{&S21383[44], 29}, {NULL}};
const Segment SGI64200[] = {{&S21383[44], 46}, {NULL}};
const Segment SGI64201[] = {{&S21383[30], 6}, {NULL}};
const Segment SGI64202[] = {{&S21383[138], 3}, {NULL}};
const Segment SGI64203[] = {{&S21383[104], 5}, {NULL}};
const Segment SGI64204[] = {{&S6117[44], 11}, {NULL}};
const Segment SGI64205[] = {{&S6117[44], 23}, {NULL}};
const Segment SGI64206[] = {{&S6117[30], 6}, {NULL}};
const Segment SGI64207[] = {{&S21384[44], 10}, {NULL}};
const Segment SGI64208[] = {{&S21384[44], 27}, {NULL}};
const Segment SGI64209[] = {{&S21384[30], 6}, {NULL}};
const Segment SGI64210[] = {{&S6117[31], 3}, {NULL}};
const Segment SGI64211[] = {{&S6117[81], 6}, {NULL}};
const Segment SGI64212[] = {{&S21384[88], 4}, {NULL}};
const Segment SGI64213[] = {{&S21384[88], 3}, {NULL}};
const Segment SGI64214[] = {{&S6118[44], 6}, {NULL}};
const Segment SGI64215[] = {{&S21385[38], 10}, {NULL}};
const Segment SGI64216[] = {{&S6118[44], 18}, {NULL}};
const Segment SGI64217[] = {{&S6118[30], 6}, {NULL}};
const Segment SGI64218[] = {{&S21385[31], 6}, {NULL}};
const Segment SGI64219[] = {{&S6118[31], 3}, {NULL}};
const Segment SGI64220[] = {{&S6118[76], 6}, {NULL}};
const Segment SGI64221[] = {{&S21386[44], 29}, {NULL}};
const Segment SGI64222[] = {{&S21386[44], 46}, {NULL}};
const Segment SGI64223[] = {{&S6119[44], 8}, {NULL}};
const Segment SGI64224[] = {{&S21386[30], 6}, {NULL}};
const Segment SGI64225[] = {{&S6119[44], 20}, {NULL}};
const Segment SGI64226[] = {{&S6119[30], 6}, {NULL}};
const Segment SGI64227[] = {{&S21386[138], 3}, {NULL}};
const Segment SGI64228[] = {{&S6119[31], 3}, {NULL}};
const Segment SGI64229[] = {{&S21386[104], 5}, {NULL}};
const Segment SGI64230[] = {{&S6119[78], 6}, {NULL}};
const Segment SGI64231[] = {{&S21387[44], 29}, {NULL}};
const Segment SGI64232[] = {{&S6120[44], 11}, {NULL}};
const Segment SGI64233[] = {{&S21387[44], 46}, {NULL}};
const Segment SGI64234[] = {{&S6120[44], 23}, {NULL}};
const Segment SGI64235[] = {{&S21387[30], 6}, {NULL}};
const Segment SGI64236[] = {{&S6120[30], 6}, {NULL}};
const Segment SGI64237[] = {{&S21387[138], 3}, {NULL}};
const Segment SGI64238[] = {{&S6120[31], 3}, {NULL}};
const Segment SGI64239[] = {{&S6120[81], 6}, {NULL}};
const Segment SGI64240[] = {{&S21387[104], 5}, {NULL}};
const Segment SGI64241[] = {{&S6121[44], 7}, {NULL}};
const Segment SGI64242[] = {{&S6121[44], 19}, {NULL}};
const Segment SGI64243[] = {{&S6121[30], 6}, {NULL}};
const Segment SGI64244[] = {{&S21388[44], 11}, {NULL}};
const Segment SGI64245[] = {{&S21388[44], 28}, {NULL}};
const Segment SGI64246[] = {{&S21388[30], 6}, {NULL}};
const Segment SGI64247[] = {{&S6121[31], 3}, {NULL}};
const Segment SGI64248[] = {{&S6121[77], 6}, {NULL}};
const Segment SGI64249[] = {{&S21388[120], 3}, {NULL}};
const Segment SGI64250[] = {{&S21388[86], 5}, {NULL}};
const Segment SGI64251[] = {{&S6122[44], 11}, {NULL}};
const Segment SGI64252[] = {{&S6122[44], 23}, {NULL}};
const Segment SGI64253[] = {{&S6122[30], 6}, {NULL}};
const Segment SGI64254[] = {{&S6122[31], 3}, {NULL}};
const Segment SGI64255[] = {{&S6122[81], 6}, {NULL}};
const Segment SGI64256[] = {{&S21390[56], 10}, {NULL}};
const Segment SGI64257[] = {{&S21390[56], 27}, {NULL}};
const Segment SGI64258[] = {{&S21390[30], 18}, {NULL}};
const Segment SGI64259[] = {{&S6123[44], 6}, {NULL}};
const Segment SGI64260[] = {{&S21390[131], 3}, {NULL}};
const Segment SGI64261[] = {{&S6123[44], 18}, {NULL}};
const Segment SGI64262[] = {{&S6123[30], 6}, {NULL}};
const Segment SGI64263[] = {{&S21390[97], 5}, {NULL}};
const Segment SGI64264[] = {{&S6123[31], 3}, {NULL}};
const Segment SGI64265[] = {{&S6123[76], 6}, {NULL}};
const Segment SGI64266[] = {{&S21391[44], 10}, {NULL}};
const Segment SGI64267[] = {{&S21391[44], 27}, {NULL}};
const Segment SGI64268[] = {{&S21391[30], 6}, {NULL}};
const Segment SGI64269[] = {{&S6124[44], 10}, {NULL}};
const Segment SGI64270[] = {{&S6124[44], 22}, {NULL}};
const Segment SGI64271[] = {{&S21391[88], 6}, {NULL}};
const Segment SGI64272[] = {{&S6124[30], 6}, {NULL}};
const Segment SGI64273[] = {{&S6124[31], 3}, {NULL}};
const Segment SGI64274[] = {{&S6124[80], 6}, {NULL}};
const Segment SGI64275[] = {{&S21392[42], 10}, {NULL}};
const Segment SGI64276[] = {{&S21392[30], 6}, {NULL}};
const Segment SGI64277[] = {{&S6125[44], 4}, {NULL}};
const Segment SGI64278[] = {{&S6125[44], 16}, {NULL}};
const Segment SGI64279[] = {{&S21392[117], 3}, {NULL}};
const Segment SGI64280[] = {{&S6125[30], 6}, {NULL}};
const Segment SGI64281[] = {{&S21392[83], 5}, {NULL}};
const Segment SGI64282[] = {{&S6125[31], 3}, {NULL}};
const Segment SGI64283[] = {{&S6125[74], 6}, {NULL}};
const Segment SGI64284[] = {{&S21393[44], 11}, {NULL}};
const Segment SGI64285[] = {{&S21393[44], 28}, {NULL}};
const Segment SGI64286[] = {{&S21393[30], 6}, {NULL}};
const Segment SGI64287[] = {{&S21393[120], 3}, {NULL}};
const Segment SGI64288[] = {{&S21393[86], 5}, {NULL}};
const Segment SGI64289[] = {{&S6127[44], 14}, {NULL}};
const Segment SGI64290[] = {{&S6127[44], 26}, {NULL}};
const Segment SGI64291[] = {{&S6127[30], 6}, {NULL}};
const Segment SGI64292[] = {{&S6127[31], 3}, {NULL}};
const Segment SGI64293[] = {{&S21394[38], 11}, {NULL}};
const Segment SGI64294[] = {{&S6127[84], 6}, {NULL}};
const Segment SGI64295[] = {{&S21394[31], 6}, {NULL}};
const Segment SGI64296[] = {{&S6128[44], 8}, {NULL}};
const Segment SGI64297[] = {{&S6128[44], 20}, {NULL}};
const Segment SGI64298[] = {{&S6128[30], 6}, {NULL}};
const Segment SGI64299[] = {{&S6128[31], 3}, {NULL}};
const Segment SGI64300[] = {{&S21395[56], 29}, {NULL}};
const Segment SGI64301[] = {{&S6128[78], 6}, {NULL}};
const Segment SGI64302[] = {{&S21395[56], 46}, {NULL}};
const Segment SGI64303[] = {{&S21395[30], 18}, {NULL}};
const Segment SGI64304[] = {{&S21395[150], 3}, {NULL}};
const Segment SGI64305[] = {{&S21395[116], 5}, {NULL}};
const Segment SGI64306[] = {{&S6130[44], 5}, {NULL}};
const Segment SGI64307[] = {{&S6130[44], 17}, {NULL}};
const Segment SGI64308[] = {{&S6130[30], 6}, {NULL}};
const Segment SGI64309[] = {{&S6130[31], 3}, {NULL}};
const Segment SGI64310[] = {{&S6130[75], 6}, {NULL}};
const Segment SGI64311[] = {{&S21398[44], 29}, {NULL}};
const Segment SGI64312[] = {{&S21398[44], 46}, {NULL}};
const Segment SGI64313[] = {{&S21398[30], 6}, {NULL}};
const Segment SGI64314[] = {{&S21398[138], 3}, {NULL}};
const Segment SGI64315[] = {{&S6131[47], 6}, {NULL}};
const Segment SGI64316[] = {{&S21398[104], 5}, {NULL}};
const Segment SGI64317[] = {{&S6131[25], 3}, {NULL}};
const Segment SGI64318[] = {{&S6131[121], 6}, {NULL}};
const Segment SGI64319[] = {{&S21399[38], 11}, {NULL}};
const Segment SGI64320[] = {{&S21399[31], 6}, {NULL}};
const Segment SGI64321[] = {{&S21400[44], 29}, {NULL}};
const Segment SGI64322[] = {{&S6133[44], 13}, {NULL}};
const Segment SGI64323[] = {{&S21400[44], 46}, {NULL}};
const Segment SGI64324[] = {{&S6133[44], 25}, {NULL}};
const Segment SGI64325[] = {{&S21400[30], 6}, {NULL}};
const Segment SGI64326[] = {{&S6133[30], 6}, {NULL}};
const Segment SGI64327[] = {{&S6133[31], 3}, {NULL}};
const Segment SGI64328[] = {{&S21400[138], 3}, {NULL}};
const Segment SGI64329[] = {{&S6133[83], 6}, {NULL}};
const Segment SGI64330[] = {{&S21400[104], 5}, {NULL}};
const Segment SGI64331[] = {{&S6135[44], 11}, {NULL}};
const Segment SGI64332[] = {{&S6135[44], 32}, {NULL}};
const Segment SGI64333[] = {{&S6135[30], 6}, {NULL}};
const Segment SGI64334[] = {{&S6135[93], 6}, {NULL}};
const Segment SGI64335[] = {{&S6137[44], 31}, {NULL}};
const Segment SGI64336[] = {{&S6137[30], 6}, {NULL}};
const Segment SGI64337[] = {{&S6140[44], 6}, {NULL}};
const Segment SGI64338[] = {{&S6140[44], 18}, {NULL}};
const Segment SGI64339[] = {{&S6140[30], 6}, {NULL}};
const Segment SGI64340[] = {{&S6140[31], 3}, {NULL}};
const Segment SGI64341[] = {{&S6140[76], 6}, {NULL}};
const Segment SGI64342[] = {{&S6141[44], 12}, {NULL}};
const Segment SGI64343[] = {{&S6141[44], 24}, {NULL}};
const Segment SGI64344[] = {{&S6141[30], 6}, {NULL}};
const Segment SGI64345[] = {{&S6141[31], 3}, {NULL}};
const Segment SGI64346[] = {{&S6141[82], 6}, {NULL}};
const Segment SGI64347[] = {{&S6144[44], 8}, {NULL}};
const Segment SGI64348[] = {{&S6144[44], 20}, {NULL}};
const Segment SGI64349[] = {{&S6144[30], 6}, {NULL}};
const Segment SGI64350[] = {{&S6144[31], 3}, {NULL}};
const Segment SGI64351[] = {{&S6144[78], 6}, {NULL}};
const Segment SGI64352[] = {{&S6149[44], 9}, {NULL}};
const Segment SGI64353[] = {{&S6149[44], 21}, {NULL}};
const Segment SGI64354[] = {{&S6149[30], 6}, {NULL}};
const Segment SGI64355[] = {{&S6149[31], 3}, {NULL}};
const Segment SGI64356[] = {{&S6149[79], 6}, {NULL}};
const Segment SGI64357[] = {{&S6150[44], 9}, {NULL}};
const Segment SGI64358[] = {{&S6150[44], 21}, {NULL}};
const Segment SGI64359[] = {{&S6150[30], 6}, {NULL}};
const Segment SGI64360[] = {{&S6150[31], 3}, {NULL}};
const Segment SGI64361[] = {{&S6150[79], 6}, {NULL}};
const Segment SGI64362[] = {{&S6160[44], 5}, {NULL}};
const Segment SGI64363[] = {{&S6160[44], 27}, {NULL}};
const Segment SGI64364[] = {{&S6160[30], 6}, {NULL}};
const Segment SGI64365[] = {{&S6160[31], 3}, {NULL}};
const Segment SGI64366[] = {{&S6160[85], 6}, {NULL}};
const Segment SGI64367[] = {{&S6163[44], 9}, {NULL}};
const Segment SGI64368[] = {{&S6163[44], 21}, {NULL}};
const Segment SGI64369[] = {{&S6163[30], 6}, {NULL}};
const Segment SGI64370[] = {{&S6163[31], 3}, {NULL}};
const Segment SGI64371[] = {{&S6163[79], 6}, {NULL}};
const Segment SGI64372[] = {{&S6165[44], 13}, {NULL}};
const Segment SGI64373[] = {{&S6165[44], 22}, {NULL}};
const Segment SGI64374[] = {{&S6165[30], 6}, {NULL}};
const Segment SGI64375[] = {{&S6165[31], 3}, {NULL}};
const Segment SGI64376[] = {{&S6165[80], 6}, {NULL}};
const Segment SGI64377[] = {{&S6178[44], 8}, {NULL}};
const Segment SGI64378[] = {{&S6178[44], 20}, {NULL}};
const Segment SGI64379[] = {{&S6178[30], 6}, {NULL}};
const Segment SGI64380[] = {{&S6178[31], 3}, {NULL}};
const Segment SGI64381[] = {{&S6178[78], 6}, {NULL}};
const Segment SGI64382[] = {{&S6179[44], 13}, {NULL}};
const Segment SGI64383[] = {{&S6179[44], 25}, {NULL}};
const Segment SGI64384[] = {{&S6179[30], 6}, {NULL}};
const Segment SGI64385[] = {{&S6179[31], 3}, {NULL}};
const Segment SGI64386[] = {{&S6179[83], 6}, {NULL}};
const Segment SGI64387[] = {{&S6181[44], 10}, {NULL}};
const Segment SGI64388[] = {{&S6181[44], 22}, {NULL}};
const Segment SGI64389[] = {{&S6181[30], 6}, {NULL}};
const Segment SGI64390[] = {{&S6181[31], 3}, {NULL}};
const Segment SGI64391[] = {{&S6181[80], 6}, {NULL}};
const Segment SGI64392[] = {{&S6188[44], 6}, {NULL}};
const Segment SGI64393[] = {{&S6188[44], 18}, {NULL}};
const Segment SGI64394[] = {{&S6188[30], 6}, {NULL}};
const Segment SGI64395[] = {{&S6188[31], 3}, {NULL}};
const Segment SGI64396[] = {{&S6188[76], 6}, {NULL}};
const Segment SGI64397[] = {{&S6189[44], 14}, {NULL}};
const Segment SGI64398[] = {{&S6189[44], 26}, {NULL}};
const Segment SGI64399[] = {{&S6189[30], 6}, {NULL}};
const Segment SGI64400[] = {{&S6189[31], 3}, {NULL}};
const Segment SGI64401[] = {{&S6189[84], 6}, {NULL}};
const Segment SGI64402[] = {{&S6192[44], 17}, {NULL}};
const Segment SGI64403[] = {{&S6192[44], 43}, {NULL}};
const Segment SGI64404[] = {{&S6192[30], 6}, {NULL}};
const Segment SGI64405[] = {{&S6192[31], 3}, {NULL}};
const Segment SGI64406[] = {{&S6192[101], 6}, {NULL}};
const Segment SGI64407[] = {{&S6208[44], 30}, {NULL}};
const Segment SGI64408[] = {{&S6208[44], 57}, {NULL}};
const Segment SGI64409[] = {{&S6208[30], 6}, {NULL}};
const Segment SGI64410[] = {{&S6208[31], 3}, {NULL}};
const Segment SGI64411[] = {{&S6208[115], 6}, {NULL}};
const Segment SGI64412[] = {{&S6210[44], 13}, {NULL}};
const Segment SGI64413[] = {{&S6210[44], 25}, {NULL}};
const Segment SGI64414[] = {{&S6210[30], 6}, {NULL}};
const Segment SGI64415[] = {{&S6210[31], 3}, {NULL}};
const Segment SGI64416[] = {{&S6210[83], 6}, {NULL}};
const Segment SGI64417[] = {{&S6214[44], 7}, {NULL}};
const Segment SGI64418[] = {{&S6214[44], 19}, {NULL}};
const Segment SGI64419[] = {{&S6214[30], 6}, {NULL}};
const Segment SGI64420[] = {{&S6214[31], 3}, {NULL}};
const Segment SGI64421[] = {{&S6214[77], 6}, {NULL}};
const Segment SGI64422[] = {{&S6216[44], 17}, {NULL}};
const Segment SGI64423[] = {{&S6216[44], 42}, {NULL}};
const Segment SGI64424[] = {{&S6216[30], 6}, {NULL}};
const Segment SGI64425[] = {{&S6216[31], 3}, {NULL}};
const Segment SGI64426[] = {{&S6216[100], 6}, {NULL}};
const Segment SGI64427[] = {{&S6219[44], 7}, {NULL}};
const Segment SGI64428[] = {{&S6219[44], 19}, {NULL}};
const Segment SGI64429[] = {{&S6219[30], 6}, {NULL}};
const Segment SGI64430[] = {{&S6219[31], 3}, {NULL}};
const Segment SGI64431[] = {{&S6219[77], 6}, {NULL}};
const Segment SGI64432[] = {{&S6221[44], 7}, {NULL}};
const Segment SGI64433[] = {{&S6221[44], 19}, {NULL}};
const Segment SGI64434[] = {{&S6221[30], 6}, {NULL}};
const Segment SGI64435[] = {{&S6221[31], 3}, {NULL}};
const Segment SGI64436[] = {{&S6221[77], 6}, {NULL}};
const Segment SGI64437[] = {{&S6223[44], 13}, {NULL}};
const Segment SGI64438[] = {{&S6223[44], 25}, {NULL}};
const Segment SGI64439[] = {{&S6223[30], 6}, {NULL}};
const Segment SGI64440[] = {{&S6223[31], 3}, {NULL}};
const Segment SGI64441[] = {{&S6223[83], 6}, {NULL}};
const Segment SGI64442[] = {{&S6224[44], 17}, {NULL}};
const Segment SGI64443[] = {{&S6224[44], 29}, {NULL}};
const Segment SGI64444[] = {{&S6224[30], 6}, {NULL}};
const Segment SGI64445[] = {{&S6224[31], 3}, {NULL}};
const Segment SGI64446[] = {{&S6224[87], 6}, {NULL}};
const Segment SGI64447[] = {{&S6225[44], 8}, {NULL}};
const Segment SGI64448[] = {{&S6225[44], 20}, {NULL}};
const Segment SGI64449[] = {{&S6225[30], 6}, {NULL}};
const Segment SGI64450[] = {{&S6225[31], 3}, {NULL}};
const Segment SGI64451[] = {{&S6225[78], 6}, {NULL}};
const Segment SGI64452[] = {{&S6228[44], 9}, {NULL}};
const Segment SGI64453[] = {{&S6228[44], 18}, {NULL}};
const Segment SGI64454[] = {{&S6228[30], 6}, {NULL}};
const Segment SGI64455[] = {{&S6228[31], 3}, {NULL}};
const Segment SGI64456[] = {{&S6228[76], 6}, {NULL}};
const Segment SGI64457[] = {{&S6230[44], 6}, {NULL}};
const Segment SGI64458[] = {{&S6230[44], 18}, {NULL}};
const Segment SGI64459[] = {{&S6230[30], 6}, {NULL}};
const Segment SGI64460[] = {{&S6230[31], 3}, {NULL}};
const Segment SGI64461[] = {{&S6230[76], 6}, {NULL}};
const Segment SGI64462[] = {{&S6231[44], 8}, {NULL}};
const Segment SGI64463[] = {{&S6231[44], 20}, {NULL}};
const Segment SGI64464[] = {{&S6231[30], 6}, {NULL}};
const Segment SGI64465[] = {{&S6231[31], 3}, {NULL}};
const Segment SGI64466[] = {{&S6231[78], 6}, {NULL}};
const Segment SGI64467[] = {{&S6232[44], 6}, {NULL}};
const Segment SGI64468[] = {{&S6232[44], 18}, {NULL}};
const Segment SGI64469[] = {{&S6232[30], 6}, {NULL}};
const Segment SGI64470[] = {{&S6232[31], 3}, {NULL}};
const Segment SGI64471[] = {{&S6232[76], 6}, {NULL}};
const Segment SGI64472[] = {{&S6233[44], 13}, {NULL}};
const Segment SGI64473[] = {{&S6233[44], 25}, {NULL}};
const Segment SGI64474[] = {{&S6233[30], 6}, {NULL}};
const Segment SGI64475[] = {{&S6233[31], 3}, {NULL}};
const Segment SGI64476[] = {{&S6233[83], 6}, {NULL}};
const Segment SGI64477[] = {{&S6235[44], 14}, {NULL}};
const Segment SGI64478[] = {{&S6235[44], 26}, {NULL}};
const Segment SGI64479[] = {{&S6235[30], 6}, {NULL}};
const Segment SGI64480[] = {{&S6235[31], 3}, {NULL}};
const Segment SGI64481[] = {{&S6235[84], 6}, {NULL}};
const Segment SGI64482[] = {{&S6237[44], 10}, {NULL}};
const Segment SGI64483[] = {{&S6237[44], 22}, {NULL}};
const Segment SGI64484[] = {{&S6237[30], 6}, {NULL}};
const Segment SGI64485[] = {{&S6237[31], 3}, {NULL}};
const Segment SGI64486[] = {{&S6237[80], 6}, {NULL}};
const Segment SGI64487[] = {{&S6238[44], 10}, {NULL}};
const Segment SGI64488[] = {{&S6238[44], 22}, {NULL}};
const Segment SGI64489[] = {{&S6238[30], 6}, {NULL}};
const Segment SGI64490[] = {{&S6238[31], 3}, {NULL}};
const Segment SGI64491[] = {{&S6238[80], 6}, {NULL}};
const Segment SGI64492[] = {{&S6240[44], 18}, {NULL}};
const Segment SGI64493[] = {{&S6240[44], 30}, {NULL}};
const Segment SGI64494[] = {{&S6240[30], 6}, {NULL}};
const Segment SGI64495[] = {{&S6240[31], 3}, {NULL}};
const Segment SGI64496[] = {{&S6240[88], 6}, {NULL}};
const Segment SGI64497[] = {{&S6241[44], 7}, {NULL}};
const Segment SGI64498[] = {{&S6241[44], 19}, {NULL}};
const Segment SGI64499[] = {{&S6241[30], 6}, {NULL}};
const Segment SGI64500[] = {{&S6241[31], 3}, {NULL}};
const Segment SGI64501[] = {{&S6241[77], 6}, {NULL}};
const Segment SGI64502[] = {{&S6243[44], 5}, {NULL}};
const Segment SGI64503[] = {{&S6243[44], 17}, {NULL}};
const Segment SGI64504[] = {{&S6243[30], 6}, {NULL}};
const Segment SGI64505[] = {{&S6243[31], 3}, {NULL}};
const Segment SGI64506[] = {{&S6243[75], 6}, {NULL}};
const Segment SGI64507[] = {{&S6244[44], 8}, {NULL}};
const Segment SGI64508[] = {{&S6244[44], 20}, {NULL}};
const Segment SGI64509[] = {{&S6244[30], 6}, {NULL}};
const Segment SGI64510[] = {{&S6244[31], 3}, {NULL}};
const Segment SGI64511[] = {{&S6244[78], 6}, {NULL}};
const Segment SGI64512[] = {{&S6245[44], 6}, {NULL}};
const Segment SGI64513[] = {{&S6245[44], 18}, {NULL}};
const Segment SGI64514[] = {{&S6245[30], 6}, {NULL}};
const Segment SGI64515[] = {{&S6245[31], 3}, {NULL}};
const Segment SGI64516[] = {{&S6245[76], 6}, {NULL}};
const Segment SGI64517[] = {{&S6246[44], 7}, {NULL}};
const Segment SGI64518[] = {{&S6246[44], 19}, {NULL}};
const Segment SGI64519[] = {{&S6246[30], 6}, {NULL}};
const Segment SGI64520[] = {{&S6246[31], 3}, {NULL}};
const Segment SGI64521[] = {{&S6246[77], 6}, {NULL}};
const Segment SGI64522[] = {{&S6257[44], 8}, {NULL}};
const Segment SGI64523[] = {{&S6257[44], 26}, {NULL}};
const Segment SGI64524[] = {{&S6257[30], 6}, {NULL}};
const Segment SGI64525[] = {{&S6257[31], 3}, {NULL}};
const Segment SGI64526[] = {{&S6257[84], 6}, {NULL}};
const Segment SGI64527[] = {{&S6267[44], 10}, {NULL}};
const Segment SGI64528[] = {{&S6267[44], 22}, {NULL}};
const Segment SGI64529[] = {{&S6267[30], 6}, {NULL}};
const Segment SGI64530[] = {{&S6267[31], 3}, {NULL}};
const Segment SGI64531[] = {{&S6267[80], 6}, {NULL}};
const Segment SGI64532[] = {{&S6269[44], 10}, {NULL}};
const Segment SGI64533[] = {{&S6269[44], 22}, {NULL}};
const Segment SGI64534[] = {{&S6269[30], 6}, {NULL}};
const Segment SGI64535[] = {{&S6269[31], 3}, {NULL}};
const Segment SGI64536[] = {{&S6269[80], 6}, {NULL}};
const Segment SGI64537[] = {{&S6280[44], 10}, {NULL}};
const Segment SGI64538[] = {{&S6280[44], 22}, {NULL}};
const Segment SGI64539[] = {{&S6280[30], 6}, {NULL}};
const Segment SGI64540[] = {{&S6280[31], 3}, {NULL}};
const Segment SGI64541[] = {{&S6280[80], 6}, {NULL}};
const Segment SGI64542[] = {{&S6285[44], 13}, {NULL}};
const Segment SGI64543[] = {{&S6285[44], 25}, {NULL}};
const Segment SGI64544[] = {{&S6285[30], 6}, {NULL}};
const Segment SGI64545[] = {{&S6285[31], 3}, {NULL}};
const Segment SGI64546[] = {{&S6285[83], 6}, {NULL}};
const Segment SGI64547[] = {{&S6286[44], 6}, {NULL}};
const Segment SGI64548[] = {{&S6286[44], 35}, {NULL}};
const Segment SGI64549[] = {{&S6286[30], 6}, {NULL}};
const Segment SGI64550[] = {{&S6286[31], 3}, {NULL}};
const Segment SGI64551[] = {{&S6286[93], 6}, {NULL}};
const Segment SGI64552[] = {{&S6287[44], 9}, {NULL}};
const Segment SGI64553[] = {{&S6287[44], 38}, {NULL}};
const Segment SGI64554[] = {{&S6287[30], 6}, {NULL}};
const Segment SGI64555[] = {{&S6287[31], 3}, {NULL}};
const Segment SGI64556[] = {{&S6287[96], 6}, {NULL}};
const Segment SGI64557[] = {{&S6289[44], 9}, {NULL}};
const Segment SGI64558[] = {{&S6289[44], 21}, {NULL}};
const Segment SGI64559[] = {{&S6289[30], 6}, {NULL}};
const Segment SGI64560[] = {{&S6289[31], 3}, {NULL}};
const Segment SGI64561[] = {{&S6289[79], 6}, {NULL}};
const Segment SGI64562[] = {{&S6290[44], 11}, {NULL}};
const Segment SGI64563[] = {{&S6290[44], 23}, {NULL}};
const Segment SGI64564[] = {{&S6290[30], 6}, {NULL}};
const Segment SGI64565[] = {{&S6290[31], 3}, {NULL}};
const Segment SGI64566[] = {{&S6290[81], 6}, {NULL}};
const Segment SGI64567[] = {{&S6291[44], 9}, {NULL}};
const Segment SGI64568[] = {{&S6291[44], 21}, {NULL}};
const Segment SGI64569[] = {{&S6291[30], 6}, {NULL}};
const Segment SGI64570[] = {{&S6291[31], 3}, {NULL}};
const Segment SGI64571[] = {{&S6291[79], 6}, {NULL}};
const Segment SGI64572[] = {{&S6292[44], 9}, {NULL}};
const Segment SGI64573[] = {{&S6292[44], 21}, {NULL}};
const Segment SGI64574[] = {{&S6292[30], 6}, {NULL}};
const Segment SGI64575[] = {{&S6292[31], 3}, {NULL}};
const Segment SGI64576[] = {{&S6292[79], 6}, {NULL}};
const Segment SGI64577[] = {{&S6293[44], 14}, {NULL}};
const Segment SGI64578[] = {{&S6293[44], 26}, {NULL}};
const Segment SGI64579[] = {{&S6293[30], 6}, {NULL}};
const Segment SGI64580[] = {{&S6293[31], 3}, {NULL}};
const Segment SGI64581[] = {{&S6293[84], 6}, {NULL}};
const Segment SGI64582[] = {{&S6295[44], 11}, {NULL}};
const Segment SGI64583[] = {{&S6295[44], 23}, {NULL}};
const Segment SGI64584[] = {{&S6295[30], 6}, {NULL}};
const Segment SGI64585[] = {{&S6295[31], 3}, {NULL}};
const Segment SGI64586[] = {{&S6295[81], 6}, {NULL}};
const Segment SGI64587[] = {{&S6299[44], 14}, {NULL}};
const Segment SGI64588[] = {{&S6299[44], 26}, {NULL}};
const Segment SGI64589[] = {{&S6299[30], 6}, {NULL}};
const Segment SGI64590[] = {{&S6299[31], 3}, {NULL}};
const Segment SGI64591[] = {{&S6299[84], 6}, {NULL}};
const Segment SGI64592[] = {{&S6301[44], 8}, {NULL}};
const Segment SGI64593[] = {{&S6301[44], 20}, {NULL}};
const Segment SGI64594[] = {{&S6301[30], 6}, {NULL}};
const Segment SGI64595[] = {{&S6301[31], 3}, {NULL}};
const Segment SGI64596[] = {{&S6301[78], 6}, {NULL}};
const Segment SGI64597[] = {{&S6302[44], 8}, {NULL}};
const Segment SGI64598[] = {{&S6302[44], 17}, {NULL}};
const Segment SGI64599[] = {{&S6302[30], 6}, {NULL}};
const Segment SGI64600[] = {{&S6302[31], 3}, {NULL}};
const Segment SGI64601[] = {{&S6302[75], 6}, {NULL}};
const Segment SGI64602[] = {{&S6306[44], 10}, {NULL}};
const Segment SGI64603[] = {{&S6306[44], 22}, {NULL}};
const Segment SGI64604[] = {{&S6306[30], 6}, {NULL}};
const Segment SGI64605[] = {{&S6306[31], 3}, {NULL}};
const Segment SGI64606[] = {{&S6306[80], 6}, {NULL}};
const Segment SGI64607[] = {{&S6307[44], 10}, {NULL}};
const Segment SGI64608[] = {{&S6307[44], 22}, {NULL}};
const Segment SGI64609[] = {{&S6307[30], 6}, {NULL}};
const Segment SGI64610[] = {{&S6307[31], 3}, {NULL}};
const Segment SGI64611[] = {{&S6307[80], 6}, {NULL}};
const Segment SGI64612[] = {{&S6310[44], 10}, {NULL}};
const Segment SGI64613[] = {{&S6310[44], 22}, {NULL}};
const Segment SGI64614[] = {{&S6310[30], 6}, {NULL}};
const Segment SGI64615[] = {{&S6310[31], 3}, {NULL}};
const Segment SGI64616[] = {{&S6310[80], 6}, {NULL}};
const Segment SGI64617[] = {{&S6317[44], 7}, {NULL}};
const Segment SGI64618[] = {{&S6317[44], 19}, {NULL}};
const Segment SGI64619[] = {{&S6317[30], 6}, {NULL}};
const Segment SGI64620[] = {{&S6317[31], 3}, {NULL}};
const Segment SGI64621[] = {{&S6317[77], 6}, {NULL}};
const Segment SGI64622[] = {{&S6320[44], 10}, {NULL}};
const Segment SGI64623[] = {{&S6320[44], 22}, {NULL}};
const Segment SGI64624[] = {{&S6320[30], 6}, {NULL}};
const Segment SGI64625[] = {{&S6320[31], 3}, {NULL}};
const Segment SGI64626[] = {{&S6320[80], 6}, {NULL}};
const Segment SGI64627[] = {{&S6325[44], 10}, {NULL}};
const Segment SGI64628[] = {{&S6325[44], 22}, {NULL}};
const Segment SGI64629[] = {{&S6325[30], 6}, {NULL}};
const Segment SGI64630[] = {{&S6325[31], 3}, {NULL}};
const Segment SGI64631[] = {{&S6325[80], 6}, {NULL}};
const Segment SGI64632[] = {{&S6327[44], 8}, {NULL}};
const Segment SGI64633[] = {{&S6327[44], 20}, {NULL}};
const Segment SGI64634[] = {{&S6327[30], 6}, {NULL}};
const Segment SGI64635[] = {{&S6327[31], 3}, {NULL}};
const Segment SGI64636[] = {{&S6327[78], 6}, {NULL}};
const Segment SGI64637[] = {{&S6329[44], 6}, {NULL}};
const Segment SGI64638[] = {{&S6329[44], 18}, {NULL}};
const Segment SGI64639[] = {{&S6329[30], 6}, {NULL}};
const Segment SGI64640[] = {{&S6329[31], 3}, {NULL}};
const Segment SGI64641[] = {{&S6329[76], 6}, {NULL}};
const Segment SGI64642[] = {{&S6330[44], 10}, {NULL}};
const Segment SGI64643[] = {{&S6330[44], 22}, {NULL}};
const Segment SGI64644[] = {{&S6330[30], 6}, {NULL}};
const Segment SGI64645[] = {{&S6330[31], 3}, {NULL}};
const Segment SGI64646[] = {{&S6330[80], 6}, {NULL}};
const Segment SGI64647[] = {{&S6331[44], 12}, {NULL}};
const Segment SGI64648[] = {{&S6331[44], 24}, {NULL}};
const Segment SGI64649[] = {{&S6331[30], 6}, {NULL}};
const Segment SGI64650[] = {{&S6331[31], 3}, {NULL}};
const Segment SGI64651[] = {{&S6331[82], 6}, {NULL}};
const Segment SGI64652[] = {{&S6332[44], 11}, {NULL}};
const Segment SGI64653[] = {{&S6332[44], 23}, {NULL}};
const Segment SGI64654[] = {{&S6332[30], 6}, {NULL}};
const Segment SGI64655[] = {{&S6332[31], 3}, {NULL}};
const Segment SGI64656[] = {{&S6332[81], 6}, {NULL}};
const Segment SGI64657[] = {{&S6334[44], 7}, {NULL}};
const Segment SGI64658[] = {{&S6334[44], 19}, {NULL}};
const Segment SGI64659[] = {{&S6334[30], 6}, {NULL}};
const Segment SGI64660[] = {{&S6334[31], 3}, {NULL}};
const Segment SGI64661[] = {{&S6334[77], 6}, {NULL}};
const Segment SGI64662[] = {{&S6337[44], 12}, {NULL}};
const Segment SGI64663[] = {{&S6337[44], 24}, {NULL}};
const Segment SGI64664[] = {{&S6337[30], 6}, {NULL}};
const Segment SGI64665[] = {{&S6337[31], 3}, {NULL}};
const Segment SGI64666[] = {{&S6337[82], 6}, {NULL}};
const Segment SGI64667[] = {{&S6339[44], 11}, {NULL}};
const Segment SGI64668[] = {{&S6339[44], 23}, {NULL}};
const Segment SGI64669[] = {{&S6339[30], 6}, {NULL}};
const Segment SGI64670[] = {{&S6339[31], 3}, {NULL}};
const Segment SGI64671[] = {{&S6339[81], 6}, {NULL}};
const Segment SGI64672[] = {{&S6341[44], 21}, {NULL}};
const Segment SGI64673[] = {{&S6341[44], 46}, {NULL}};
const Segment SGI64674[] = {{&S6341[30], 6}, {NULL}};
const Segment SGI64675[] = {{&S6341[31], 3}, {NULL}};
const Segment SGI64676[] = {{&S6341[104], 6}, {NULL}};
const Segment SGI64677[] = {{&S6344[44], 8}, {NULL}};
const Segment SGI64678[] = {{&S6344[44], 20}, {NULL}};
const Segment SGI64679[] = {{&S6344[30], 6}, {NULL}};
const Segment SGI64680[] = {{&S6344[31], 3}, {NULL}};
const Segment SGI64681[] = {{&S6344[78], 6}, {NULL}};
const Segment SGI64682[] = {{&S6347[44], 10}, {NULL}};
const Segment SGI64683[] = {{&S6347[44], 24}, {NULL}};
const Segment SGI64684[] = {{&S6347[30], 6}, {NULL}};
const Segment SGI64685[] = {{&S6347[31], 3}, {NULL}};
const Segment SGI64686[] = {{&S6347[82], 6}, {NULL}};
const Segment SGI64687[] = {{&S6348[44], 8}, {NULL}};
const Segment SGI64688[] = {{&S6348[44], 20}, {NULL}};
const Segment SGI64689[] = {{&S6348[30], 6}, {NULL}};
const Segment SGI64690[] = {{&S6348[31], 3}, {NULL}};
const Segment SGI64691[] = {{&S6348[78], 6}, {NULL}};
const Segment SGI64692[] = {{&S6353[44], 9}, {NULL}};
const Segment SGI64693[] = {{&S6353[44], 21}, {NULL}};
const Segment SGI64694[] = {{&S6353[30], 6}, {NULL}};
const Segment SGI64695[] = {{&S6353[31], 3}, {NULL}};
const Segment SGI64696[] = {{&S6353[79], 6}, {NULL}};
const Segment SGI64697[] = {{&S6354[44], 14}, {NULL}};
const Segment SGI64698[] = {{&S6354[44], 26}, {NULL}};
const Segment SGI64699[] = {{&S6354[30], 6}, {NULL}};
const Segment SGI64700[] = {{&S6354[31], 3}, {NULL}};
const Segment SGI64701[] = {{&S6354[84], 6}, {NULL}};
const Segment SGI64702[] = {{&S6364[58], 6}, {NULL}};
const Segment SGI64703[] = {{&S6364[41], 16}, {NULL}};
const Segment SGI64704[] = {{&S6364[29], 3}, {NULL}};
const Segment SGI64705[] = {{&S6364[87], 9}, {NULL}};
const Segment SGI64706[] = {{&S6366[44], 13}, {NULL}};
const Segment SGI64707[] = {{&S6366[44], 25}, {NULL}};
const Segment SGI64708[] = {{&S6366[30], 6}, {NULL}};
const Segment SGI64709[] = {{&S6366[31], 3}, {NULL}};
const Segment SGI64710[] = {{&S6366[83], 6}, {NULL}};
const Segment SGI64711[] = {{&S6369[44], 11}, {NULL}};
const Segment SGI64712[] = {{&S6369[44], 23}, {NULL}};
const Segment SGI64713[] = {{&S6369[30], 6}, {NULL}};
const Segment SGI64714[] = {{&S6369[31], 3}, {NULL}};
const Segment SGI64715[] = {{&S6369[81], 6}, {NULL}};
const Segment SGI64716[] = {{&S6375[44], 10}, {NULL}};
const Segment SGI64717[] = {{&S6375[44], 22}, {NULL}};
const Segment SGI64718[] = {{&S6375[30], 6}, {NULL}};
const Segment SGI64719[] = {{&S15607[44], 19}, {NULL}};
const Segment SGI64720[] = {{&S15607[44], 37}, {NULL}};
const Segment SGI64721[] = {{&S15607[30], 6}, {NULL}};
const Segment SGI64722[] = {{&S6375[31], 3}, {NULL}};
const Segment SGI64723[] = {{&S6375[80], 6}, {NULL}};
const Segment SGI64724[] = {{&S15607[69], 3}, {NULL}};
const Segment SGI64725[] = {{&S15607[95], 5}, {NULL}};
const Segment SGI64726[] = {{&S6377[44], 10}, {NULL}};
const Segment SGI64727[] = {{&S6377[44], 28}, {NULL}};
const Segment SGI64728[] = {{&S6377[30], 6}, {NULL}};
const Segment SGI64729[] = {{&S6377[31], 3}, {NULL}};
const Segment SGI64730[] = {{&S15609[44], 7}, {NULL}};
const Segment SGI64731[] = {{&S6377[86], 6}, {NULL}};
const Segment SGI64732[] = {{&S15609[44], 25}, {NULL}};
const Segment SGI64733[] = {{&S15609[30], 6}, {NULL}};
const Segment SGI64734[] = {{&S15609[57], 3}, {NULL}};
const Segment SGI64735[] = {{&S15609[83], 5}, {NULL}};
const Segment SGI64736[] = {{&S6379[44], 14}, {NULL}};
const Segment SGI64737[] = {{&S6379[44], 21}, {NULL}};
const Segment SGI64738[] = {{&S6379[30], 6}, {NULL}};
const Segment SGI64739[] = {{&S6379[31], 3}, {NULL}};
const Segment SGI64740[] = {{&S6379[79], 6}, {NULL}};
const Segment SGI64741[] = {{&S6380[44], 8}, {NULL}};
const Segment SGI64742[] = {{&S6380[44], 20}, {NULL}};
const Segment SGI64743[] = {{&S6380[30], 6}, {NULL}};
const Segment SGI64744[] = {{&S6380[31], 3}, {NULL}};
const Segment SGI64745[] = {{&S6380[78], 6}, {NULL}};
const Segment SGI64746[] = {{&S6389[44], 11}, {NULL}};
const Segment SGI64747[] = {{&S6389[44], 37}, {NULL}};
const Segment SGI64748[] = {{&S6389[30], 6}, {NULL}};
const Segment SGI64749[] = {{&S6389[31], 3}, {NULL}};
const Segment SGI64750[] = {{&S6389[95], 6}, {NULL}};
const Segment SGI64751[] = {{&S6390[44], 9}, {NULL}};
const Segment SGI64752[] = {{&S6390[44], 27}, {NULL}};
const Segment SGI64753[] = {{&S6390[30], 6}, {NULL}};
const Segment SGI64754[] = {{&S6390[31], 3}, {NULL}};
const Segment SGI64755[] = {{&S6390[85], 6}, {NULL}};
const Segment SGI64756[] = {{&S6394[44], 22}, {NULL}};
const Segment SGI64757[] = {{&S6394[44], 31}, {NULL}};
const Segment SGI64758[] = {{&S6394[30], 6}, {NULL}};
const Segment SGI64759[] = {{&S6394[31], 3}, {NULL}};
const Segment SGI64760[] = {{&S6394[89], 6}, {NULL}};
const Segment SGI64761[] = {{&S6395[44], 7}, {NULL}};
const Segment SGI64762[] = {{&S6395[44], 19}, {NULL}};
const Segment SGI64763[] = {{&S6395[30], 6}, {NULL}};
const Segment SGI64764[] = {{&S6395[31], 3}, {NULL}};
const Segment SGI64765[] = {{&S6395[77], 6}, {NULL}};
const Segment SGI64766[] = {{&S6397[44], 17}, {NULL}};
const Segment SGI64767[] = {{&S6397[44], 29}, {NULL}};
const Segment SGI64768[] = {{&S6397[30], 6}, {NULL}};
const Segment SGI64769[] = {{&S6397[31], 3}, {NULL}};
const Segment SGI64770[] = {{&S6397[87], 6}, {NULL}};
const Segment SGI64771[] = {{&S6398[44], 13}, {NULL}};
const Segment SGI64772[] = {{&S6398[44], 25}, {NULL}};
const Segment SGI64773[] = {{&S6398[30], 6}, {NULL}};
const Segment SGI64774[] = {{&S6398[31], 3}, {NULL}};
const Segment SGI64775[] = {{&S6398[83], 6}, {NULL}};
const Segment SGI64776[] = {{&S6399[44], 7}, {NULL}};
const Segment SGI64777[] = {{&S6399[44], 19}, {NULL}};
const Segment SGI64778[] = {{&S6399[30], 6}, {NULL}};
const Segment SGI64779[] = {{&S6399[31], 3}, {NULL}};
const Segment SGI64780[] = {{&S6399[77], 6}, {NULL}};
const Segment SGI64781[] = {{&S6401[44], 6}, {NULL}};
const Segment SGI64782[] = {{&S6401[44], 18}, {NULL}};
const Segment SGI64783[] = {{&S6401[30], 6}, {NULL}};
const Segment SGI64784[] = {{&S6401[31], 3}, {NULL}};
const Segment SGI64785[] = {{&S6401[76], 6}, {NULL}};
const Segment SGI64786[] = {{&S6402[44], 14}, {NULL}};
const Segment SGI64787[] = {{&S6402[44], 26}, {NULL}};
const Segment SGI64788[] = {{&S6402[30], 6}, {NULL}};
const Segment SGI64789[] = {{&S6402[31], 3}, {NULL}};
const Segment SGI64790[] = {{&S6402[84], 6}, {NULL}};
const Segment SGI64791[] = {{&S6403[44], 15}, {NULL}};
const Segment SGI64792[] = {{&S6403[44], 27}, {NULL}};
const Segment SGI64793[] = {{&S6403[30], 6}, {NULL}};
const Segment SGI64794[] = {{&S6403[31], 3}, {NULL}};
const Segment SGI64795[] = {{&S6403[85], 6}, {NULL}};
const Segment SGI64796[] = {{&S6407[44], 7}, {NULL}};
const Segment SGI64797[] = {{&S6407[44], 19}, {NULL}};
const Segment SGI64798[] = {{&S6407[30], 6}, {NULL}};
const Segment SGI64799[] = {{&S6407[31], 3}, {NULL}};
const Segment SGI64800[] = {{&S6407[77], 6}, {NULL}};
const Segment SGI64801[] = {{&S6408[44], 10}, {NULL}};
const Segment SGI64802[] = {{&S6408[44], 22}, {NULL}};
const Segment SGI64803[] = {{&S6408[30], 6}, {NULL}};
const Segment SGI64804[] = {{&S6408[31], 3}, {NULL}};
const Segment SGI64805[] = {{&S6408[80], 6}, {NULL}};
const Segment SGI64806[] = {{&S6409[44], 12}, {NULL}};
const Segment SGI64807[] = {{&S6409[44], 24}, {NULL}};
const Segment SGI64808[] = {{&S6409[30], 6}, {NULL}};
const Segment SGI64809[] = {{&S6409[31], 3}, {NULL}};
const Segment SGI64810[] = {{&S6409[82], 6}, {NULL}};
const Segment SGI64811[] = {{&S6410[44], 12}, {NULL}};
const Segment SGI64812[] = {{&S6410[44], 34}, {NULL}};
const Segment SGI64813[] = {{&S6410[30], 6}, {NULL}};
const Segment SGI64814[] = {{&S6410[31], 3}, {NULL}};
const Segment SGI64815[] = {{&S6410[92], 6}, {NULL}};
const Segment SGI64816[] = {{&S6411[44], 7}, {NULL}};
const Segment SGI64817[] = {{&S6411[44], 24}, {NULL}};
const Segment SGI64818[] = {{&S6411[30], 6}, {NULL}};
const Segment SGI64819[] = {{&S6411[31], 3}, {NULL}};
const Segment SGI64820[] = {{&S6411[82], 6}, {NULL}};
const Segment SGI64821[] = {{&S6412[44], 9}, {NULL}};
const Segment SGI64822[] = {{&S6412[44], 21}, {NULL}};
const Segment SGI64823[] = {{&S6412[30], 6}, {NULL}};
const Segment SGI64824[] = {{&S6412[31], 3}, {NULL}};
const Segment SGI64825[] = {{&S6412[79], 6}, {NULL}};
const Segment SGI64826[] = {{&S6418[44], 9}, {NULL}};
const Segment SGI64827[] = {{&S6418[44], 21}, {NULL}};
const Segment SGI64828[] = {{&S6418[30], 6}, {NULL}};
const Segment SGI64829[] = {{&S6418[31], 3}, {NULL}};
const Segment SGI64830[] = {{&S6418[79], 6}, {NULL}};
const Segment SGI64831[] = {{&S6420[44], 8}, {NULL}};
const Segment SGI64832[] = {{&S6420[44], 20}, {NULL}};
const Segment SGI64833[] = {{&S6420[30], 6}, {NULL}};
const Segment SGI64834[] = {{&S6420[31], 3}, {NULL}};
const Segment SGI64835[] = {{&S6420[78], 6}, {NULL}};
const Segment SGI64836[] = {{&S6423[44], 7}, {NULL}};
const Segment SGI64837[] = {{&S6423[44], 19}, {NULL}};
const Segment SGI64838[] = {{&S6423[30], 6}, {NULL}};
const Segment SGI64839[] = {{&S6423[31], 3}, {NULL}};
const Segment SGI64840[] = {{&S6423[77], 6}, {NULL}};
const Segment SGI64841[] = {{&S6425[44], 5}, {NULL}};
const Segment SGI64842[] = {{&S6425[44], 17}, {NULL}};
const Segment SGI64843[] = {{&S6425[30], 6}, {NULL}};
const Segment SGI64844[] = {{&S6425[31], 3}, {NULL}};
const Segment SGI64845[] = {{&S6425[75], 6}, {NULL}};
const Segment SGI64846[] = {{&S6427[44], 18}, {NULL}};
const Segment SGI64847[] = {{&S6427[44], 32}, {NULL}};
const Segment SGI64848[] = {{&S6427[30], 6}, {NULL}};
const Segment SGI64849[] = {{&S6427[31], 3}, {NULL}};
const Segment SGI64850[] = {{&S6427[90], 6}, {NULL}};
const Segment SGI64851[] = {{&S6430[44], 10}, {NULL}};
const Segment SGI64852[] = {{&S6430[44], 22}, {NULL}};
const Segment SGI64853[] = {{&S6430[30], 6}, {NULL}};
const Segment SGI64854[] = {{&S6430[31], 3}, {NULL}};
const Segment SGI64855[] = {{&S6430[80], 6}, {NULL}};
const Segment SGI64856[] = {{&S6431[44], 14}, {NULL}};
const Segment SGI64857[] = {{&S6431[44], 26}, {NULL}};
const Segment SGI64858[] = {{&S6431[30], 6}, {NULL}};
const Segment SGI64859[] = {{&S6431[31], 3}, {NULL}};
const Segment SGI64860[] = {{&S6431[84], 6}, {NULL}};
const Segment SGI64861[] = {{&S6443[44], 5}, {NULL}};
const Segment SGI64862[] = {{&S6443[44], 17}, {NULL}};
const Segment SGI64863[] = {{&S6443[30], 6}, {NULL}};
const Segment SGI64864[] = {{&S6443[31], 3}, {NULL}};
const Segment SGI64865[] = {{&S6443[75], 6}, {NULL}};
const Segment SGI64866[] = {{&S6445[44], 8}, {NULL}};
const Segment SGI64867[] = {{&S6445[44], 20}, {NULL}};
const Segment SGI64868[] = {{&S6445[30], 6}, {NULL}};
const Segment SGI64869[] = {{&S6445[31], 3}, {NULL}};
const Segment SGI64870[] = {{&S6445[78], 6}, {NULL}};
const Segment SGI64871[] = {{&S6450[44], 9}, {NULL}};
const Segment SGI64872[] = {{&S6450[44], 34}, {NULL}};
const Segment SGI64873[] = {{&S6450[30], 6}, {NULL}};
const Segment SGI64874[] = {{&S6450[31], 3}, {NULL}};
const Segment SGI64875[] = {{&S6450[92], 6}, {NULL}};
const Segment SGI64876[] = {{&S6456[44], 9}, {NULL}};
const Segment SGI64877[] = {{&S6456[44], 21}, {NULL}};
const Segment SGI64878[] = {{&S6456[30], 6}, {NULL}};
const Segment SGI64879[] = {{&S6456[31], 3}, {NULL}};
const Segment SGI64880[] = {{&S6456[79], 6}, {NULL}};
const Segment SGI64881[] = {{&S6457[44], 6}, {NULL}};
const Segment SGI64882[] = {{&S6457[44], 18}, {NULL}};
const Segment SGI64883[] = {{&S6457[30], 6}, {NULL}};
const Segment SGI64884[] = {{&S6457[31], 3}, {NULL}};
const Segment SGI64885[] = {{&S6457[76], 6}, {NULL}};
const Segment SGI64886[] = {{&S6458[44], 7}, {NULL}};
const Segment SGI64887[] = {{&S6458[44], 19}, {NULL}};
const Segment SGI64888[] = {{&S6458[30], 6}, {NULL}};
const Segment SGI64889[] = {{&S6458[31], 3}, {NULL}};
const Segment SGI64890[] = {{&S6458[77], 6}, {NULL}};
const Segment SGI64891[] = {{&S6460[44], 7}, {NULL}};
const Segment SGI64892[] = {{&S6460[44], 19}, {NULL}};
const Segment SGI64893[] = {{&S6460[30], 6}, {NULL}};
const Segment SGI64894[] = {{&S6460[80], 6}, {NULL}};
const Segment SGI64895[] = {{&S6461[44], 6}, {NULL}};
const Segment SGI64896[] = {{&S6461[44], 18}, {NULL}};
const Segment SGI64897[] = {{&S6461[30], 6}, {NULL}};
const Segment SGI64898[] = {{&S6461[31], 3}, {NULL}};
const Segment SGI64899[] = {{&S6461[76], 6}, {NULL}};
const Segment SGI64900[] = {{&S6463[44], 12}, {NULL}};
const Segment SGI64901[] = {{&S6463[44], 24}, {NULL}};
const Segment SGI64902[] = {{&S6463[30], 6}, {NULL}};
const Segment SGI64903[] = {{&S6463[31], 3}, {NULL}};
const Segment SGI64904[] = {{&S6463[82], 6}, {NULL}};
const Segment SGI64905[] = {{&S6464[44], 5}, {NULL}};
const Segment SGI64906[] = {{&S6464[44], 17}, {NULL}};
const Segment SGI64907[] = {{&S6464[30], 6}, {NULL}};
const Segment SGI64908[] = {{&S6464[31], 3}, {NULL}};
const Segment SGI64909[] = {{&S6464[75], 6}, {NULL}};
const Segment SGI64910[] = {{&S6465[44], 7}, {NULL}};
const Segment SGI64911[] = {{&S6465[44], 19}, {NULL}};
const Segment SGI64912[] = {{&S6465[30], 6}, {NULL}};
const Segment SGI64913[] = {{&S6465[31], 3}, {NULL}};
const Segment SGI64914[] = {{&S6465[77], 6}, {NULL}};
const Segment SGI64915[] = {{&S6467[44], 6}, {NULL}};
const Segment SGI64916[] = {{&S6467[44], 18}, {NULL}};
const Segment SGI64917[] = {{&S6467[30], 6}, {NULL}};
const Segment SGI64918[] = {{&S6467[31], 3}, {NULL}};
const Segment SGI64919[] = {{&S6467[76], 6}, {NULL}};
const Segment SGI64920[] = {{&S6469[44], 10}, {NULL}};
const Segment SGI64921[] = {{&S6469[44], 22}, {NULL}};
const Segment SGI64922[] = {{&S6469[30], 6}, {NULL}};
const Segment SGI64923[] = {{&S6469[31], 3}, {NULL}};
const Segment SGI64924[] = {{&S6469[80], 6}, {NULL}};
const Segment SGI64925[] = {{&S6472[44], 8}, {NULL}};
const Segment SGI64926[] = {{&S6472[44], 20}, {NULL}};
const Segment SGI64927[] = {{&S6472[30], 6}, {NULL}};
const Segment SGI64928[] = {{&S6472[31], 3}, {NULL}};
const Segment SGI64929[] = {{&S6472[78], 6}, {NULL}};
const Segment SGI64930[] = {{&S6473[44], 14}, {NULL}};
const Segment SGI64931[] = {{&S6473[44], 26}, {NULL}};
const Segment SGI64932[] = {{&S6473[30], 6}, {NULL}};
const Segment SGI64933[] = {{&S6473[31], 3}, {NULL}};
const Segment SGI64934[] = {{&S6473[84], 5}, {NULL}};
const Segment SGI64935[] = {{&S6475[44], 9}, {NULL}};
const Segment SGI64936[] = {{&S6475[44], 21}, {NULL}};
const Segment SGI64937[] = {{&S6475[30], 6}, {NULL}};
const Segment SGI64938[] = {{&S6475[31], 3}, {NULL}};
const Segment SGI64939[] = {{&S6475[79], 6}, {NULL}};
const Segment SGI64940[] = {{&S6479[44], 7}, {NULL}};
const Segment SGI64941[] = {{&S6479[44], 19}, {NULL}};
const Segment SGI64942[] = {{&S6479[30], 6}, {NULL}};
const Segment SGI64943[] = {{&S6479[31], 3}, {NULL}};
const Segment SGI64944[] = {{&S6479[77], 6}, {NULL}};
const Segment SGI64945[] = {{&S6482[44], 7}, {NULL}};
const Segment SGI64946[] = {{&S6482[44], 19}, {NULL}};
const Segment SGI64947[] = {{&S6482[30], 6}, {NULL}};
const Segment SGI64948[] = {{&S6482[31], 3}, {NULL}};
const Segment SGI64949[] = {{&S6482[77], 6}, {NULL}};
const Segment SGI64950[] = {{&S6485[44], 11}, {NULL}};
const Segment SGI64951[] = {{&S6485[44], 23}, {NULL}};
const Segment SGI64952[] = {{&S6485[30], 6}, {NULL}};
const Segment SGI64953[] = {{&S6485[31], 3}, {NULL}};
const Segment SGI64954[] = {{&S6485[81], 6}, {NULL}};
const Segment SGI64955[] = {{&S6490[44], 15}, {NULL}};
const Segment SGI64956[] = {{&S6490[44], 27}, {NULL}};
const Segment SGI64957[] = {{&S6490[30], 6}, {NULL}};
const Segment SGI64958[] = {{&S6490[31], 3}, {NULL}};
const Segment SGI64959[] = {{&S6490[85], 6}, {NULL}};
const Segment SGI64960[] = {{&S6492[44], 15}, {NULL}};
const Segment SGI64961[] = {{&S6492[44], 27}, {NULL}};
const Segment SGI64962[] = {{&S6492[30], 6}, {NULL}};
const Segment SGI64963[] = {{&S6492[31], 3}, {NULL}};
const Segment SGI64964[] = {{&S6492[85], 6}, {NULL}};
const Segment SGI64965[] = {{&S6495[44], 9}, {NULL}};
const Segment SGI64966[] = {{&S6495[44], 21}, {NULL}};
const Segment SGI64967[] = {{&S6495[30], 6}, {NULL}};
const Segment SGI64968[] = {{&S6495[31], 3}, {NULL}};
const Segment SGI64969[] = {{&S6495[79], 6}, {NULL}};
const Segment SGI64970[] = {{&S6499[44], 14}, {NULL}};
const Segment SGI64971[] = {{&S6499[44], 26}, {NULL}};
const Segment SGI64972[] = {{&S6499[30], 6}, {NULL}};
const Segment SGI64973[] = {{&S6499[31], 3}, {NULL}};
const Segment SGI64974[] = {{&S6499[84], 6}, {NULL}};
const Segment SGI64975[] = {{&S6501[44], 13}, {NULL}};
const Segment SGI64976[] = {{&S6501[44], 25}, {NULL}};
const Segment SGI64977[] = {{&S6501[30], 6}, {NULL}};
const Segment SGI64978[] = {{&S6501[31], 3}, {NULL}};
const Segment SGI64979[] = {{&S6501[83], 6}, {NULL}};
const Segment SGI64980[] = {{&S6502[44], 9}, {NULL}};
const Segment SGI64981[] = {{&S6502[44], 21}, {NULL}};
const Segment SGI64982[] = {{&S6502[30], 6}, {NULL}};
const Segment SGI64983[] = {{&S6502[31], 3}, {NULL}};
const Segment SGI64984[] = {{&S6502[79], 6}, {NULL}};
const Segment SGI64985[] = {{&S6506[44], 10}, {NULL}};
const Segment SGI64986[] = {{&S6506[44], 22}, {NULL}};
const Segment SGI64987[] = {{&S6506[30], 6}, {NULL}};
const Segment SGI64988[] = {{&S6506[31], 3}, {NULL}};
const Segment SGI64989[] = {{&S6506[80], 6}, {NULL}};
const Segment SGI64990[] = {{&S6516[44], 12}, {NULL}};
const Segment SGI64991[] = {{&S6516[44], 24}, {NULL}};
const Segment SGI64992[] = {{&S6516[30], 6}, {NULL}};
const Segment SGI64993[] = {{&S6516[31], 3}, {NULL}};
const Segment SGI64994[] = {{&S6516[82], 6}, {NULL}};
const Segment SGI64995[] = {{&S6519[44], 18}, {NULL}};
const Segment SGI64996[] = {{&S6519[44], 30}, {NULL}};
const Segment SGI64997[] = {{&S6519[30], 6}, {NULL}};
const Segment SGI64998[] = {{&S6519[31], 3}, {NULL}};
const Segment SGI64999[] = {{&S6519[88], 6}, {NULL}};
const Segment SGI65000[] = {{&S6521[44], 10}, {NULL}};
const Segment SGI65001[] = {{&S6521[44], 22}, {NULL}};
const Segment SGI65002[] = {{&S6521[30], 6}, {NULL}};
const Segment SGI65003[] = {{&S6521[31], 3}, {NULL}};
const Segment SGI65004[] = {{&S6521[80], 6}, {NULL}};
const Segment SGI65005[] = {{&S6524[44], 13}, {NULL}};
const Segment SGI65006[] = {{&S6524[44], 25}, {NULL}};
const Segment SGI65007[] = {{&S6524[30], 6}, {NULL}};
const Segment SGI65008[] = {{&S6524[31], 3}, {NULL}};
const Segment SGI65009[] = {{&S6524[83], 6}, {NULL}};
const Segment SGI65010[] = {{&S6529[44], 13}, {NULL}};
const Segment SGI65011[] = {{&S6529[44], 25}, {NULL}};
const Segment SGI65012[] = {{&S6529[30], 6}, {NULL}};
const Segment SGI65013[] = {{&S6529[31], 3}, {NULL}};
const Segment SGI65014[] = {{&S6529[83], 6}, {NULL}};
const Segment SGI65015[] = {{&S6531[44], 17}, {NULL}};
const Segment SGI65016[] = {{&S6531[44], 38}, {NULL}};
const Segment SGI65017[] = {{&S6531[30], 6}, {NULL}};
const Segment SGI65018[] = {{&S6531[31], 3}, {NULL}};
const Segment SGI65019[] = {{&S6531[96], 6}, {NULL}};
const Segment SGI65020[] = {{&S6532[44], 53}, {NULL}};
const Segment SGI65021[] = {{&S6532[44], 65}, {NULL}};
const Segment SGI65022[] = {{&S6532[30], 6}, {NULL}};
const Segment SGI65023[] = {{&S6532[31], 3}, {NULL}};
const Segment SGI65024[] = {{&S6532[123], 6}, {NULL}};
const Segment SGI65025[] = {{&S6539[44], 9}, {NULL}};
const Segment SGI65026[] = {{&S6539[44], 21}, {NULL}};
const Segment SGI65027[] = {{&S6539[30], 6}, {NULL}};
const Segment SGI65028[] = {{&S6539[31], 3}, {NULL}};
const Segment SGI65029[] = {{&S6539[79], 6}, {NULL}};
const Segment SGI65030[] = {{&S6540[44], 7}, {NULL}};
const Segment SGI65031[] = {{&S6540[44], 19}, {NULL}};
const Segment SGI65032[] = {{&S6540[30], 6}, {NULL}};
const Segment SGI65033[] = {{&S6540[31], 3}, {NULL}};
const Segment SGI65034[] = {{&S6540[77], 6}, {NULL}};
const Segment SGI65035[] = {{&S6541[44], 6}, {NULL}};
const Segment SGI65036[] = {{&S6541[44], 18}, {NULL}};
const Segment SGI65037[] = {{&S6541[30], 6}, {NULL}};
const Segment SGI65038[] = {{&S6541[31], 3}, {NULL}};
const Segment SGI65039[] = {{&S6541[76], 6}, {NULL}};
const Segment SGI65040[] = {{&S6542[44], 17}, {NULL}};
const Segment SGI65041[] = {{&S6542[44], 29}, {NULL}};
const Segment SGI65042[] = {{&S6542[30], 6}, {NULL}};
const Segment SGI65043[] = {{&S6542[31], 3}, {NULL}};
const Segment SGI65044[] = {{&S6542[87], 6}, {NULL}};
const Segment SGI65045[] = {{&S6544[44], 7}, {NULL}};
const Segment SGI65046[] = {{&S6544[44], 24}, {NULL}};
const Segment SGI65047[] = {{&S6544[30], 6}, {NULL}};
const Segment SGI65048[] = {{&S6544[31], 3}, {NULL}};
const Segment SGI65049[] = {{&S6544[82], 6}, {NULL}};
const Segment SGI65050[] = {{&S6545[44], 6}, {NULL}};
const Segment SGI65051[] = {{&S6545[44], 18}, {NULL}};
const Segment SGI65052[] = {{&S6545[30], 6}, {NULL}};
const Segment SGI65053[] = {{&S6545[31], 3}, {NULL}};
const Segment SGI65054[] = {{&S6545[76], 6}, {NULL}};
const Segment SGI65055[] = {{&S6546[44], 12}, {NULL}};
const Segment SGI65056[] = {{&S6546[44], 24}, {NULL}};
const Segment SGI65057[] = {{&S6546[30], 6}, {NULL}};
const Segment SGI65058[] = {{&S6546[31], 3}, {NULL}};
const Segment SGI65059[] = {{&S6546[82], 6}, {NULL}};
const Segment SGI65060[] = {{&S6549[44], 12}, {NULL}};
const Segment SGI65061[] = {{&S6549[44], 24}, {NULL}};
const Segment SGI65062[] = {{&S6549[30], 6}, {NULL}};
const Segment SGI65063[] = {{&S6549[31], 3}, {NULL}};
const Segment SGI65064[] = {{&S6549[82], 6}, {NULL}};
const Segment SGI65065[] = {{&S6550[44], 5}, {NULL}};
const Segment SGI65066[] = {{&S6550[44], 17}, {NULL}};
const Segment SGI65067[] = {{&S6550[30], 6}, {NULL}};
const Segment SGI65068[] = {{&S6550[31], 3}, {NULL}};
const Segment SGI65069[] = {{&S6550[75], 6}, {NULL}};
const Segment SGI65070[] = {{&S6556[44], 9}, {NULL}};
const Segment SGI65071[] = {{&S6556[44], 21}, {NULL}};
const Segment SGI65072[] = {{&S6556[30], 6}, {NULL}};
const Segment SGI65073[] = {{&S6556[31], 3}, {NULL}};
const Segment SGI65074[] = {{&S6556[79], 6}, {NULL}};
const Segment SGI65075[] = {{&S6558[44], 8}, {NULL}};
const Segment SGI65076[] = {{&S6558[44], 20}, {NULL}};
const Segment SGI65077[] = {{&S6558[30], 6}, {NULL}};
const Segment SGI65078[] = {{&S6558[31], 3}, {NULL}};
const Segment SGI65079[] = {{&S6558[78], 6}, {NULL}};
const Segment SGI65080[] = {{&S6560[44], 14}, {NULL}};
const Segment SGI65081[] = {{&S6560[44], 26}, {NULL}};
const Segment SGI65082[] = {{&S6560[30], 6}, {NULL}};
const Segment SGI65083[] = {{&S6560[31], 3}, {NULL}};
const Segment SGI65084[] = {{&S6560[84], 6}, {NULL}};
const Segment SGI65085[] = {{&S6562[44], 10}, {NULL}};
const Segment SGI65086[] = {{&S6562[44], 22}, {NULL}};
const Segment SGI65087[] = {{&S6562[30], 6}, {NULL}};
const Segment SGI65088[] = {{&S6562[31], 3}, {NULL}};
const Segment SGI65089[] = {{&S6562[80], 6}, {NULL}};
const Segment SGI65090[] = {{&S6563[44], 15}, {NULL}};
const Segment SGI65091[] = {{&S6563[44], 27}, {NULL}};
const Segment SGI65092[] = {{&S6563[30], 6}, {NULL}};
const Segment SGI65093[] = {{&S6563[31], 3}, {NULL}};
const Segment SGI65094[] = {{&S6563[85], 6}, {NULL}};
const Segment SGI65095[] = {{&S6564[44], 10}, {NULL}};
const Segment SGI65096[] = {{&S6564[44], 22}, {NULL}};
const Segment SGI65097[] = {{&S6564[30], 6}, {NULL}};
const Segment SGI65098[] = {{&S6564[31], 3}, {NULL}};
const Segment SGI65099[] = {{&S6564[80], 6}, {NULL}};
const Segment SGI65100[] = {{&S16013[44], 11}, {NULL}};
const Segment SGI65101[] = {{&S16013[44], 23}, {NULL}};
const Segment SGI65102[] = {{&S16013[30], 6}, {NULL}};
const Segment SGI65103[] = {{&S16013[125], 3}, {NULL}};
const Segment SGI65104[] = {{&S16013[91], 5}, {NULL}};
const Segment SGI65105[] = {{&S16020[44], 14}, {NULL}};
const Segment SGI65106[] = {{&S16020[44], 25}, {NULL}};
const Segment SGI65107[] = {{&S16020[30], 6}, {NULL}};
const Segment SGI65108[] = {{&S16020[117], 3}, {NULL}};
const Segment SGI65109[] = {{&S16020[83], 5}, {NULL}};
const Segment SGI65110[] = {{&S6566[44], 7}, {NULL}};
const Segment SGI65111[] = {{&S6566[44], 19}, {NULL}};
const Segment SGI65112[] = {{&S6566[30], 6}, {NULL}};
const Segment SGI65113[] = {{&S6566[31], 3}, {NULL}};
const Segment SGI65114[] = {{&S6566[77], 6}, {NULL}};
const Segment SGI65115[] = {{&S6567[44], 6}, {NULL}};
const Segment SGI65116[] = {{&S6567[44], 18}, {NULL}};
const Segment SGI65117[] = {{&S6567[30], 6}, {NULL}};
const Segment SGI65118[] = {{&S6567[31], 3}, {NULL}};
const Segment SGI65119[] = {{&S6567[76], 6}, {NULL}};
const Segment SGI65120[] = {{&S16043[44], 11}, {NULL}};
const Segment SGI65121[] = {{&S6569[44], 4}, {NULL}};
const Segment SGI65122[] = {{&S6569[44], 16}, {NULL}};
const Segment SGI65123[] = {{&S16043[44], 28}, {NULL}};
const Segment SGI65124[] = {{&S6569[30], 6}, {NULL}};
const Segment SGI65125[] = {{&S16043[30], 6}, {NULL}};
const Segment SGI65126[] = {{&S6569[31], 3}, {NULL}};
const Segment SGI65127[] = {{&S16043[120], 3}, {NULL}};
const Segment SGI65128[] = {{&S6569[74], 6}, {NULL}};
const Segment SGI65129[] = {{&S16043[86], 5}, {NULL}};
const Segment SGI65130[] = {{&S6573[44], 12}, {NULL}};
const Segment SGI65131[] = {{&S6573[44], 24}, {NULL}};
const Segment SGI65132[] = {{&S6573[30], 6}, {NULL}};
const Segment SGI65133[] = {{&S6573[31], 3}, {NULL}};
const Segment SGI65134[] = {{&S6573[82], 6}, {NULL}};
const Segment SGI65135[] = {{&S6576[44], 9}, {NULL}};
const Segment SGI65136[] = {{&S6576[44], 21}, {NULL}};
const Segment SGI65137[] = {{&S6576[30], 6}, {NULL}};
const Segment SGI65138[] = {{&S6576[31], 3}, {NULL}};
const Segment SGI65139[] = {{&S6576[79], 6}, {NULL}};
const Segment SGI65140[] = {{&S16057[44], 5}, {NULL}};
const Segment SGI65141[] = {{&S16057[44], 16}, {NULL}};
const Segment SGI65142[] = {{&S16057[30], 6}, {NULL}};
const Segment SGI65143[] = {{&S16057[108], 3}, {NULL}};
const Segment SGI65144[] = {{&S16057[74], 5}, {NULL}};
const Segment SGI65145[] = {{&S6585[44], 10}, {NULL}};
const Segment SGI65146[] = {{&S6585[44], 22}, {NULL}};
const Segment SGI65147[] = {{&S6585[30], 6}, {NULL}};
const Segment SGI65148[] = {{&S6585[31], 3}, {NULL}};
const Segment SGI65149[] = {{&S6585[80], 6}, {NULL}};
const Segment SGI65150[] = {{&S6586[44], 9}, {NULL}};
const Segment SGI65151[] = {{&S6586[44], 21}, {NULL}};
const Segment SGI65152[] = {{&S6586[30], 6}, {NULL}};
const Segment SGI65153[] = {{&S6586[31], 3}, {NULL}};
const Segment SGI65154[] = {{&S6586[79], 6}, {NULL}};
const Segment SGI65155[] = {{&S16062[44], 6}, {NULL}};
const Segment SGI65156[] = {{&S16062[44], 17}, {NULL}};
const Segment SGI65157[] = {{&S16062[30], 6}, {NULL}};
const Segment SGI65158[] = {{&S16062[109], 3}, {NULL}};
const Segment SGI65159[] = {{&S16062[75], 5}, {NULL}};
const Segment SGI65160[] = {{&S6591[44], 11}, {NULL}};
const Segment SGI65161[] = {{&S6591[44], 32}, {NULL}};
const Segment SGI65162[] = {{&S6591[30], 6}, {NULL}};
const Segment SGI65163[] = {{&S16066[44], 14}, {NULL}};
const Segment SGI65164[] = {{&S16066[44], 25}, {NULL}};
const Segment SGI65165[] = {{&S16066[30], 6}, {NULL}};
const Segment SGI65166[] = {{&S6591[31], 3}, {NULL}};
const Segment SGI65167[] = {{&S6591[90], 6}, {NULL}};
const Segment SGI65168[] = {{&S16066[117], 3}, {NULL}};
const Segment SGI65169[] = {{&S16066[83], 5}, {NULL}};
const Segment SGI65170[] = {{&S16073[44], 6}, {NULL}};
const Segment SGI65171[] = {{&S16073[44], 17}, {NULL}};
const Segment SGI65172[] = {{&S6599[44], 16}, {NULL}};
const Segment SGI65173[] = {{&S16073[30], 6}, {NULL}};
const Segment SGI65174[] = {{&S6599[44], 28}, {NULL}};
const Segment SGI65175[] = {{&S6599[30], 6}, {NULL}};
const Segment SGI65176[] = {{&S16073[109], 3}, {NULL}};
const Segment SGI65177[] = {{&S6599[31], 3}, {NULL}};
const Segment SGI65178[] = {{&S16073[75], 5}, {NULL}};
const Segment SGI65179[] = {{&S6599[86], 6}, {NULL}};
const Segment SGI65180[] = {{&S6600[44], 6}, {NULL}};
const Segment SGI65181[] = {{&S6600[44], 18}, {NULL}};
const Segment SGI65182[] = {{&S6600[30], 6}, {NULL}};
const Segment SGI65183[] = {{&S6600[31], 3}, {NULL}};
const Segment SGI65184[] = {{&S6600[76], 6}, {NULL}};
const Segment SGI65185[] = {{&S16081[44], 6}, {NULL}};
const Segment SGI65186[] = {{&S16081[44], 23}, {NULL}};
const Segment SGI65187[] = {{&S16081[30], 6}, {NULL}};
const Segment SGI65188[] = {{&S16081[115], 3}, {NULL}};
const Segment SGI65189[] = {{&S16081[81], 5}, {NULL}};
const Segment SGI65190[] = {{&S6608[44], 21}, {NULL}};
const Segment SGI65191[] = {{&S6608[44], 40}, {NULL}};
const Segment SGI65192[] = {{&S6608[30], 6}, {NULL}};
const Segment SGI65193[] = {{&S6608[31], 3}, {NULL}};
const Segment SGI65194[] = {{&S6608[98], 6}, {NULL}};
const Segment SGI65195[] = {{&S6611[44], 9}, {NULL}};
const Segment SGI65196[] = {{&S6611[44], 37}, {NULL}};
const Segment SGI65197[] = {{&S6611[30], 6}, {NULL}};
const Segment SGI65198[] = {{&S6611[31], 3}, {NULL}};
const Segment SGI65199[] = {{&S6611[95], 6}, {NULL}};
const Segment SGI65200[] = {{&S6613[44], 11}, {NULL}};
const Segment SGI65201[] = {{&S6613[44], 23}, {NULL}};
const Segment SGI65202[] = {{&S6613[30], 6}, {NULL}};
const Segment SGI65203[] = {{&S6613[31], 3}, {NULL}};
const Segment SGI65204[] = {{&S6613[81], 6}, {NULL}};
const Segment SGI65205[] = {{&S7306[44], 9}, {NULL}};
const Segment SGI65206[] = {{&S7306[44], 21}, {NULL}};
const Segment SGI65207[] = {{&S7306[30], 6}, {NULL}};
const Segment SGI65208[] = {{&S7306[31], 3}, {NULL}};
const Segment SGI65209[] = {{&S7306[79], 5}, {NULL}};
const Segment SGI65210[] = {{&S6618[44], 6}, {NULL}};
const Segment SGI65211[] = {{&S6618[44], 18}, {NULL}};
const Segment SGI65212[] = {{&S6618[30], 6}, {NULL}};
const Segment SGI65213[] = {{&S6618[31], 3}, {NULL}};
const Segment SGI65214[] = {{&S6618[76], 6}, {NULL}};
const Segment SGI65215[] = {{&S6622[44], 13}, {NULL}};
const Segment SGI65216[] = {{&S6622[44], 31}, {NULL}};
const Segment SGI65217[] = {{&S6622[30], 6}, {NULL}};
const Segment SGI65218[] = {{&S16099[44], 10}, {NULL}};
const Segment SGI65219[] = {{&S6622[31], 3}, {NULL}};
const Segment SGI65220[] = {{&S16099[44], 27}, {NULL}};
const Segment SGI65221[] = {{&S16099[30], 6}, {NULL}};
const Segment SGI65222[] = {{&S6622[89], 6}, {NULL}};
const Segment SGI65223[] = {{&S16099[119], 3}, {NULL}};
const Segment SGI65224[] = {{&S16099[85], 5}, {NULL}};
const Segment SGI65225[] = {{&S6623[44], 15}, {NULL}};
const Segment SGI65226[] = {{&S6623[44], 27}, {NULL}};
const Segment SGI65227[] = {{&S6623[30], 6}, {NULL}};
const Segment SGI65228[] = {{&S6623[31], 3}, {NULL}};
const Segment SGI65229[] = {{&S6623[85], 6}, {NULL}};
const Segment SGI65230[] = {{&S16101[44], 22}, {NULL}};
const Segment SGI65231[] = {{&S16101[44], 39}, {NULL}};
const Segment SGI65232[] = {{&S16101[30], 6}, {NULL}};
const Segment SGI65233[] = {{&S6624[44], 7}, {NULL}};
const Segment SGI65234[] = {{&S6624[44], 19}, {NULL}};
const Segment SGI65235[] = {{&S6624[30], 6}, {NULL}};
const Segment SGI65236[] = {{&S16101[131], 3}, {NULL}};
const Segment SGI65237[] = {{&S16101[97], 5}, {NULL}};
const Segment SGI65238[] = {{&S6624[31], 3}, {NULL}};
const Segment SGI65239[] = {{&S6624[77], 6}, {NULL}};
const Segment SGI65240[] = {{&S6625[44], 7}, {NULL}};
const Segment SGI65241[] = {{&S6625[44], 19}, {NULL}};
const Segment SGI65242[] = {{&S6625[30], 6}, {NULL}};
const Segment SGI65243[] = {{&S6625[31], 3}, {NULL}};
const Segment SGI65244[] = {{&S6625[77], 6}, {NULL}};
const Segment SGI65245[] = {{&S6626[44], 8}, {NULL}};
const Segment SGI65246[] = {{&S6626[44], 26}, {NULL}};
const Segment SGI65247[] = {{&S6626[30], 6}, {NULL}};
const Segment SGI65248[] = {{&S6626[31], 3}, {NULL}};
const Segment SGI65249[] = {{&S6626[84], 6}, {NULL}};
const Segment SGI65250[] = {{&S16107[44], 11}, {NULL}};
const Segment SGI65251[] = {{&S16107[44], 23}, {NULL}};
const Segment SGI65252[] = {{&S16107[30], 6}, {NULL}};
const Segment SGI65253[] = {{&S16107[137], 3}, {NULL}};
const Segment SGI65254[] = {{&S16107[103], 5}, {NULL}};
const Segment SGI65255[] = {{&S6632[44], 5}, {NULL}};
const Segment SGI65256[] = {{&S6632[44], 17}, {NULL}};
const Segment SGI65257[] = {{&S6632[30], 6}, {NULL}};
const Segment SGI65258[] = {{&S6632[31], 3}, {NULL}};
const Segment SGI65259[] = {{&S6632[75], 6}, {NULL}};
const Segment SGI65260[] = {{&S6635[44], 11}, {NULL}};
const Segment SGI65261[] = {{&S6635[44], 23}, {NULL}};
const Segment SGI65262[] = {{&S6635[30], 6}, {NULL}};
const Segment SGI65263[] = {{&S6635[31], 3}, {NULL}};
const Segment SGI65264[] = {{&S6635[81], 6}, {NULL}};
const Segment SGI65265[] = {{&S6636[44], 16}, {NULL}};
const Segment SGI65266[] = {{&S6636[44], 28}, {NULL}};
const Segment SGI65267[] = {{&S6636[30], 6}, {NULL}};
const Segment SGI65268[] = {{&S6636[31], 3}, {NULL}};
const Segment SGI65269[] = {{&S6636[86], 6}, {NULL}};
const Segment SGI65270[] = {{&S6637[44], 8}, {NULL}};
const Segment SGI65271[] = {{&S6637[44], 20}, {NULL}};
const Segment SGI65272[] = {{&S6637[30], 6}, {NULL}};
const Segment SGI65273[] = {{&S6637[31], 3}, {NULL}};
const Segment SGI65274[] = {{&S6637[78], 6}, {NULL}};
const Segment SGI65275[] = {{&S16120[44], 5}, {NULL}};
const Segment SGI65276[] = {{&S16120[44], 16}, {NULL}};
const Segment SGI65277[] = {{&S16120[30], 6}, {NULL}};
const Segment SGI65278[] = {{&S6639[44], 9}, {NULL}};
const Segment SGI65279[] = {{&S6639[44], 21}, {NULL}};
const Segment SGI65280[] = {{&S6639[30], 6}, {NULL}};
const Segment SGI65281[] = {{&S16120[119], 3}, {NULL}};
const Segment SGI65282[] = {{&S16120[85], 5}, {NULL}};
const Segment SGI65283[] = {{&S6639[31], 3}, {NULL}};
const Segment SGI65284[] = {{&S6639[79], 6}, {NULL}};
const Segment SGI65285[] = {{&S16123[44], 5}, {NULL}};
const Segment SGI65286[] = {{&S16123[44], 22}, {NULL}};
const Segment SGI65287[] = {{&S16123[30], 6}, {NULL}};
const Segment SGI65288[] = {{&S16123[114], 3}, {NULL}};
const Segment SGI65289[] = {{&S16123[80], 5}, {NULL}};
const Segment SGI65290[] = {{&S16135[44], 9}, {NULL}};
const Segment SGI65291[] = {{&S16135[44], 20}, {NULL}};
const Segment SGI65292[] = {{&S16135[30], 6}, {NULL}};
const Segment SGI65293[] = {{&S16135[112], 3}, {NULL}};
const Segment SGI65294[] = {{&S16135[78], 5}, {NULL}};
const Segment SGI65295[] = {{&S6660[44], 8}, {NULL}};
const Segment SGI65296[] = {{&S6660[44], 20}, {NULL}};
const Segment SGI65297[] = {{&S6660[30], 6}, {NULL}};
const Segment SGI65298[] = {{&S6660[31], 3}, {NULL}};
const Segment SGI65299[] = {{&S6660[78], 6}, {NULL}};
const Segment SGI65300[] = {{&S6665[44], 11}, {NULL}};
const Segment SGI65301[] = {{&S6665[44], 39}, {NULL}};
const Segment SGI65302[] = {{&S6665[30], 6}, {NULL}};
const Segment SGI65303[] = {{&S6665[31], 3}, {NULL}};
const Segment SGI65304[] = {{&S16166[44], 25}, {NULL}};
const Segment SGI65305[] = {{&S16166[44], 42}, {NULL}};
const Segment SGI65306[] = {{&S16166[30], 6}, {NULL}};
const Segment SGI65307[] = {{&S16166[134], 3}, {NULL}};
const Segment SGI65308[] = {{&S16166[100], 5}, {NULL}};
const Segment SGI65309[] = {{&S22598[44], 10}, {NULL}};
const Segment SGI65310[] = {{&S22598[44], 27}, {NULL}};
const Segment SGI65311[] = {{&S22598[30], 6}, {NULL}};
const Segment SGI65312[] = {{&S22598[119], 3}, {NULL}};
const Segment SGI65313[] = {{&S22598[85], 5}, {NULL}};
const Segment SGI65314[] = {{&S16186[44], 17}, {NULL}};
const Segment SGI65315[] = {{&S16186[44], 34}, {NULL}};
const Segment SGI65316[] = {{&S16186[30], 6}, {NULL}};
const Segment SGI65317[] = {{&S22601[38], 10}, {NULL}};
const Segment SGI65318[] = {{&S16186[126], 3}, {NULL}};
const Segment SGI65319[] = {{&S22601[31], 6}, {NULL}};
const Segment SGI65320[] = {{&S16186[92], 5}, {NULL}};
const Segment SGI65321[] = {{&S22604[44], 11}, {NULL}};
const Segment SGI65322[] = {{&S22604[44], 28}, {NULL}};
const Segment SGI65323[] = {{&S22604[30], 6}, {NULL}};
const Segment SGI65324[] = {{&S22604[120], 3}, {NULL}};
const Segment SGI65325[] = {{&S22604[86], 5}, {NULL}};
const Segment SGI65326[] = {{&S22605[44], 31}, {NULL}};
const Segment SGI65327[] = {{&S22605[44], 48}, {NULL}};
const Segment SGI65328[] = {{&S22605[30], 6}, {NULL}};
const Segment SGI65329[] = {{&S22605[140], 3}, {NULL}};
const Segment SGI65330[] = {{&S22605[106], 5}, {NULL}};
const Segment SGI65331[] = {{&S22610[44], 10}, {NULL}};
const Segment SGI65332[] = {{&S22610[44], 27}, {NULL}};
const Segment SGI65333[] = {{&S22610[30], 6}, {NULL}};
const Segment SGI65334[] = {{&S22610[88], 6}, {NULL}};
const Segment SGI65335[] = {{&S16199[44], 50}, {NULL}};
const Segment SGI65336[] = {{&S16199[30], 6}, {NULL}};
const Segment SGI65337[] = {{&S16199[161], 3}, {NULL}};
const Segment SGI65338[] = {{&S16199[127], 5}, {NULL}};
const Segment SGI65339[] = {{&S22612[42], 10}, {NULL}};
const Segment SGI65340[] = {{&S22612[30], 6}, {NULL}};
const Segment SGI65341[] = {{&S22612[117], 3}, {NULL}};
const Segment SGI65342[] = {{&S22612[83], 5}, {NULL}};
const Segment SGI65343[] = {{&S22615[44], 10}, {NULL}};
const Segment SGI65344[] = {{&S22615[44], 27}, {NULL}};
const Segment SGI65345[] = {{&S22615[30], 6}, {NULL}};
const Segment SGI65346[] = {{&S22615[88], 4}, {NULL}};
const Segment SGI65347[] = {{&S22615[88], 3}, {NULL}};
const Segment SGI65348[] = {{&S22616[44], 31}, {NULL}};
const Segment SGI65349[] = {{&S22616[44], 48}, {NULL}};
const Segment SGI65350[] = {{&S22616[30], 6}, {NULL}};
const Segment SGI65351[] = {{&S22616[140], 3}, {NULL}};
const Segment SGI65352[] = {{&S22616[106], 5}, {NULL}};
const Segment SGI65353[] = {{&S6665[97], 6}, {NULL}};
const Segment SGI65354[] = {{&S6668[44], 9}, {NULL}};
const Segment SGI65355[] = {{&S6668[44], 21}, {NULL}};
const Segment SGI65356[] = {{&S6668[30], 6}, {NULL}};
const Segment SGI65357[] = {{&S6668[31], 3}, {NULL}};
const Segment SGI65358[] = {{&S6668[79], 6}, {NULL}};
const Segment SGI65359[] = {{&S6670[38], 9}, {NULL}};
const Segment SGI65360[] = {{&S6670[31], 6}, {NULL}};
const Segment SGI65361[] = {{&S6671[44], 17}, {NULL}};
const Segment SGI65362[] = {{&S6671[44], 29}, {NULL}};
const Segment SGI65363[] = {{&S6671[30], 6}, {NULL}};
const Segment SGI65364[] = {{&S6671[31], 3}, {NULL}};
const Segment SGI65365[] = {{&S6671[87], 6}, {NULL}};
const Segment SGI65366[] = {{&S6672[44], 8}, {NULL}};
const Segment SGI65367[] = {{&S6672[44], 20}, {NULL}};
const Segment SGI65368[] = {{&S6672[30], 6}, {NULL}};
const Segment SGI65369[] = {{&S6672[31], 3}, {NULL}};
const Segment SGI65370[] = {{&S6672[78], 6}, {NULL}};
const Segment SGI65371[] = {{&S6673[44], 5}, {NULL}};
const Segment SGI65372[] = {{&S6673[44], 17}, {NULL}};
const Segment SGI65373[] = {{&S6673[30], 6}, {NULL}};
const Segment SGI65374[] = {{&S6673[31], 3}, {NULL}};
const Segment SGI65375[] = {{&S6673[75], 6}, {NULL}};
const Segment SGI65376[] = {{&S6674[44], 13}, {NULL}};
const Segment SGI65377[] = {{&S6674[44], 25}, {NULL}};
const Segment SGI65378[] = {{&S6674[30], 6}, {NULL}};
const Segment SGI65379[] = {{&S6674[31], 3}, {NULL}};
const Segment SGI65380[] = {{&S6674[83], 6}, {NULL}};
const Segment SGI65381[] = {{&S6675[44], 4}, {NULL}};
const Segment SGI65382[] = {{&S6675[44], 16}, {NULL}};
const Segment SGI65383[] = {{&S6675[30], 6}, {NULL}};
const Segment SGI65384[] = {{&S6675[31], 3}, {NULL}};
const Segment SGI65385[] = {{&S6675[74], 6}, {NULL}};
const Segment SGI65386[] = {{&S6678[44], 10}, {NULL}};
const Segment SGI65387[] = {{&S6678[44], 22}, {NULL}};
const Segment SGI65388[] = {{&S6678[30], 6}, {NULL}};
const Segment SGI65389[] = {{&S6678[31], 3}, {NULL}};
const Segment SGI65390[] = {{&S6678[80], 6}, {NULL}};
const Segment SGI65391[] = {{&S6679[44], 8}, {NULL}};
const Segment SGI65392[] = {{&S6679[44], 20}, {NULL}};
const Segment SGI65393[] = {{&S6679[30], 6}, {NULL}};
const Segment SGI65394[] = {{&S6679[31], 3}, {NULL}};
const Segment SGI65395[] = {{&S6679[78], 6}, {NULL}};
const Segment SGI65396[] = {{&S6680[44], 7}, {NULL}};
const Segment SGI65397[] = {{&S6680[44], 19}, {NULL}};
const Segment SGI65398[] = {{&S6680[30], 6}, {NULL}};
const Segment SGI65399[] = {{&S6680[31], 3}, {NULL}};
const Segment SGI65400[] = {{&S6680[77], 6}, {NULL}};
const Segment SGI65401[] = {{&S6681[44], 15}, {NULL}};
const Segment SGI65402[] = {{&S6681[44], 27}, {NULL}};
const Segment SGI65403[] = {{&S6681[30], 6}, {NULL}};
const Segment SGI65404[] = {{&S6681[31], 3}, {NULL}};
const Segment SGI65405[] = {{&S6681[85], 6}, {NULL}};
const Segment SGI65406[] = {{&S16249[44], 14}, {NULL}};
const Segment SGI65407[] = {{&S16249[44], 31}, {NULL}};
const Segment SGI65408[] = {{&S16249[30], 6}, {NULL}};
const Segment SGI65409[] = {{&S16249[92], 6}, {NULL}};
const Segment SGI65410[] = {{&S6682[44], 14}, {NULL}};
const Segment SGI65411[] = {{&S6682[44], 26}, {NULL}};
const Segment SGI65412[] = {{&S6682[30], 6}, {NULL}};
const Segment SGI65413[] = {{&S6682[31], 3}, {NULL}};
const Segment SGI65414[] = {{&S6682[84], 6}, {NULL}};
const Segment SGI65415[] = {{&S6685[44], 8}, {NULL}};
const Segment SGI65416[] = {{&S6685[44], 30}, {NULL}};
const Segment SGI65417[] = {{&S6685[30], 6}, {NULL}};
const Segment SGI65418[] = {{&S6685[31], 3}, {NULL}};
const Segment SGI65419[] = {{&S6685[88], 6}, {NULL}};
const Segment SGI65420[] = {{&S6690[44], 10}, {NULL}};
const Segment SGI65421[] = {{&S6690[44], 22}, {NULL}};
const Segment SGI65422[] = {{&S6690[30], 6}, {NULL}};
const Segment SGI65423[] = {{&S6690[31], 3}, {NULL}};
const Segment SGI65424[] = {{&S6690[80], 6}, {NULL}};
const Segment SGI65425[] = {{&S6696[44], 10}, {NULL}};
const Segment SGI65426[] = {{&S6696[44], 27}, {NULL}};
const Segment SGI65427[] = {{&S6696[30], 6}, {NULL}};
const Segment SGI65428[] = {{&S6696[31], 3}, {NULL}};
const Segment SGI65429[] = {{&S6696[85], 6}, {NULL}};
const Segment SGI65430[] = {{&S6700[44], 5}, {NULL}};
const Segment SGI65431[] = {{&S6700[44], 14}, {NULL}};
const Segment SGI65432[] = {{&S6700[30], 6}, {NULL}};
const Segment SGI65433[] = {{&S6700[31], 3}, {NULL}};
const Segment SGI65434[] = {{&S6700[72], 6}, {NULL}};
const Segment SGI65435[] = {{&S6703[44], 9}, {NULL}};
const Segment SGI65436[] = {{&S6703[44], 21}, {NULL}};
const Segment SGI65437[] = {{&S6703[30], 6}, {NULL}};
const Segment SGI65438[] = {{&S6703[31], 3}, {NULL}};
const Segment SGI65439[] = {{&S6703[79], 6}, {NULL}};
const Segment SGI65440[] = {{&S6710[44], 12}, {NULL}};
const Segment SGI65441[] = {{&S6710[44], 24}, {NULL}};
const Segment SGI65442[] = {{&S6710[30], 6}, {NULL}};
const Segment SGI65443[] = {{&S6710[31], 3}, {NULL}};
const Segment SGI65444[] = {{&S6710[82], 6}, {NULL}};
const Segment SGI65445[] = {{&S6716[44], 8}, {NULL}};
const Segment SGI65446[] = {{&S6716[44], 20}, {NULL}};
const Segment SGI65447[] = {{&S6716[30], 6}, {NULL}};
const Segment SGI65448[] = {{&S6716[31], 3}, {NULL}};
const Segment SGI65449[] = {{&S6716[78], 5}, {NULL}};
const Segment SGI65450[] = {{&S6726[44], 10}, {NULL}};
const Segment SGI65451[] = {{&S6726[44], 22}, {NULL}};
const Segment SGI65452[] = {{&S6726[30], 6}, {NULL}};
const Segment SGI65453[] = {{&S6726[31], 3}, {NULL}};
const Segment SGI65454[] = {{&S6726[80], 6}, {NULL}};
const Segment SGI65455[] = {{&S16306[44], 11}, {NULL}};
const Segment SGI65456[] = {{&S16306[44], 22}, {NULL}};
const Segment SGI65457[] = {{&S16306[30], 6}, {NULL}};
const Segment SGI65458[] = {{&S16306[114], 3}, {NULL}};
const Segment SGI65459[] = {{&S16306[80], 5}, {NULL}};
const Segment SGI65460[] = {{&S16308[44], 12}, {NULL}};
const Segment SGI65461[] = {{&S16308[44], 23}, {NULL}};
const Segment SGI65462[] = {{&S16308[30], 6}, {NULL}};
const Segment SGI65463[] = {{&S16308[115], 3}, {NULL}};
const Segment SGI65464[] = {{&S16308[81], 5}, {NULL}};
const Segment SGI65465[] = {{&S16313[44], 30}, {NULL}};
const Segment SGI65466[] = {{&S16313[44], 47}, {NULL}};
const Segment SGI65467[] = {{&S16313[30], 6}, {NULL}};
const Segment SGI65468[] = {{&S16313[139], 3}, {NULL}};
const Segment SGI65469[] = {{&S16313[105], 5}, {NULL}};
const Segment SGI65470[] = {{&S6740[44], 10}, {NULL}};
const Segment SGI65471[] = {{&S6740[44], 22}, {NULL}};
const Segment SGI65472[] = {{&S6740[30], 6}, {NULL}};
const Segment SGI65473[] = {{&S6740[31], 3}, {NULL}};
const Segment SGI65474[] = {{&S6740[80], 6}, {NULL}};
const Segment SGI65475[] = {{&S16323[44], 12}, {NULL}};
const Segment SGI65476[] = {{&S16323[44], 33}, {NULL}};
const Segment SGI65477[] = {{&S16323[30], 6}, {NULL}};
const Segment SGI65478[] = {{&S16323[142], 3}, {NULL}};
const Segment SGI65479[] = {{&S16323[108], 5}, {NULL}};
const Segment SGI65480[] = {{&S6748[44], 6}, {NULL}};
const Segment SGI65481[] = {{&S6748[44], 18}, {NULL}};
const Segment SGI65482[] = {{&S6748[30], 6}, {NULL}};
const Segment SGI65483[] = {{&S6748[31], 3}, {NULL}};
const Segment SGI65484[] = {{&S6748[76], 6}, {NULL}};
const Segment SGI65485[] = {{&S6749[44], 7}, {NULL}};
const Segment SGI65486[] = {{&S6749[44], 19}, {NULL}};
const Segment SGI65487[] = {{&S6749[30], 6}, {NULL}};
const Segment SGI65488[] = {{&S6749[31], 3}, {NULL}};
const Segment SGI65489[] = {{&S6749[77], 5}, {NULL}};
const Segment SGI65490[] = {{&S6750[44], 10}, {NULL}};
const Segment SGI65491[] = {{&S6750[44], 22}, {NULL}};
const Segment SGI65492[] = {{&S6750[30], 6}, {NULL}};
const Segment SGI65493[] = {{&S6750[31], 3}, {NULL}};
const Segment SGI65494[] = {{&S6750[80], 6}, {NULL}};
const Segment SGI65495[] = {{&S6752[44], 16}, {NULL}};
const Segment SGI65496[] = {{&S6752[44], 28}, {NULL}};
const Segment SGI65497[] = {{&S6752[30], 6}, {NULL}};
const Segment SGI65498[] = {{&S6752[31], 3}, {NULL}};
const Segment SGI65499[] = {{&S6752[86], 6}, {NULL}};
const Segment SGI65500[] = {{&S6753[44], 6}, {NULL}};
const Segment SGI65501[] = {{&S6753[44], 18}, {NULL}};
const Segment SGI65502[] = {{&S6753[30], 6}, {NULL}};
const Segment SGI65503[] = {{&S6753[31], 3}, {NULL}};
const Segment SGI65504[] = {{&S6753[76], 6}, {NULL}};
const Segment SGI65505[] = {{&S6754[44], 11}, {NULL}};
const Segment SGI65506[] = {{&S6754[44], 23}, {NULL}};
const Segment SGI65507[] = {{&S6754[30], 6}, {NULL}};
const Segment SGI65508[] = {{&S6754[31], 3}, {NULL}};
const Segment SGI65509[] = {{&S6754[81], 6}, {NULL}};
const Segment SGI65510[] = {{&S6757[44], 9}, {NULL}};
const Segment SGI65511[] = {{&S6757[44], 21}, {NULL}};
const Segment SGI65512[] = {{&S6757[30], 6}, {NULL}};
const Segment SGI65513[] = {{&S6757[31], 3}, {NULL}};
const Segment SGI65514[] = {{&S6757[79], 6}, {NULL}};
const Segment SGI65515[] = {{&S6758[44], 8}, {NULL}};
const Segment SGI65516[] = {{&S6758[44], 20}, {NULL}};
const Segment SGI65517[] = {{&S6758[30], 6}, {NULL}};
const Segment SGI65518[] = {{&S6758[31], 3}, {NULL}};
const Segment SGI65519[] = {{&S6758[78], 6}, {NULL}};
const Segment SGI65520[] = {{&S6759[44], 6}, {NULL}};
const Segment SGI65521[] = {{&S6759[44], 18}, {NULL}};
const Segment SGI65522[] = {{&S6759[30], 6}, {NULL}};
const Segment SGI65523[] = {{&S6759[31], 3}, {NULL}};
const Segment SGI65524[] = {{&S6759[76], 6}, {NULL}};
const Segment SGI65525[] = {{&S6760[44], 9}, {NULL}};
const Segment SGI65526[] = {{&S6760[44], 21}, {NULL}};
const Segment SGI65527[] = {{&S6760[30], 6}, {NULL}};
const Segment SGI65528[] = {{&S6760[31], 3}, {NULL}};
const Segment SGI65529[] = {{&S6760[79], 6}, {NULL}};
const Segment SGI65530[] = {{&S6763[44], 14}, {NULL}};
const Segment SGI65531[] = {{&S6763[44], 26}, {NULL}};
const Segment SGI65532[] = {{&S6763[30], 6}, {NULL}};
const Segment SGI65533[] = {{&S6763[31], 3}, {NULL}};
const Segment SGI65534[] = {{&S6763[84], 6}, {NULL}};
const Segment SGI65535[] = {{&S6764[44], 11}, {NULL}};
const Segment SGI65536[] = {{&S6764[44], 23}, {NULL}};
const Segment SGI65537[] = {{&S6764[30], 6}, {NULL}};
const Segment SGI65538[] = {{&S6764[31], 3}, {NULL}};
const Segment SGI65539[] = {{&S6764[81], 6}, {NULL}};
const Segment SGI65540[] = {{&S6765[44], 13}, {NULL}};
const Segment SGI65541[] = {{&S6765[44], 25}, {NULL}};
const Segment SGI65542[] = {{&S6765[30], 6}, {NULL}};
const Segment SGI65543[] = {{&S6765[31], 3}, {NULL}};
const Segment SGI65544[] = {{&S6765[83], 6}, {NULL}};
const Segment SGI65545[] = {{&S6766[44], 11}, {NULL}};
const Segment SGI65546[] = {{&S6766[44], 23}, {NULL}};
const Segment SGI65547[] = {{&S6766[30], 6}, {NULL}};
const Segment SGI65548[] = {{&S6766[31], 3}, {NULL}};
const Segment SGI65549[] = {{&S6766[81], 6}, {NULL}};
const Segment SGI65550[] = {{&S16361[44], 10}, {NULL}};
const Segment SGI65551[] = {{&S16361[44], 22}, {NULL}};
const Segment SGI65552[] = {{&S16361[30], 6}, {NULL}};
const Segment SGI65553[] = {{&S16361[131], 3}, {NULL}};
const Segment SGI65554[] = {{&S6767[44], 11}, {NULL}};
const Segment SGI65555[] = {{&S6767[44], 23}, {NULL}};
const Segment SGI65556[] = {{&S16361[97], 5}, {NULL}};
const Segment SGI65557[] = {{&S6767[30], 6}, {NULL}};
const Segment SGI65558[] = {{&S6767[31], 3}, {NULL}};
const Segment SGI65559[] = {{&S6767[81], 6}, {NULL}};
const Segment SGI65560[] = {{&S6769[44], 9}, {NULL}};
const Segment SGI65561[] = {{&S6769[44], 21}, {NULL}};
const Segment SGI65562[] = {{&S6769[30], 6}, {NULL}};
const Segment SGI65563[] = {{&S6769[31], 3}, {NULL}};
const Segment SGI65564[] = {{&S6769[79], 6}, {NULL}};
const Segment SGI65565[] = {{&S6773[44], 5}, {NULL}};
const Segment SGI65566[] = {{&S6773[44], 14}, {NULL}};
const Segment SGI65567[] = {{&S6773[30], 6}, {NULL}};
const Segment SGI65568[] = {{&S6773[31], 3}, {NULL}};
const Segment SGI65569[] = {{&S6773[72], 6}, {NULL}};
const Segment SGI65570[] = {{&S6774[44], 11}, {NULL}};
const Segment SGI65571[] = {{&S6774[44], 23}, {NULL}};
const Segment SGI65572[] = {{&S6774[30], 6}, {NULL}};
const Segment SGI65573[] = {{&S6774[31], 3}, {NULL}};
const Segment SGI65574[] = {{&S6774[81], 6}, {NULL}};
const Segment SGI65575[] = {{&S6776[44], 14}, {NULL}};
const Segment SGI65576[] = {{&S6776[44], 26}, {NULL}};
const Segment SGI65577[] = {{&S6776[30], 6}, {NULL}};
const Segment SGI65578[] = {{&S6776[31], 3}, {NULL}};
const Segment SGI65579[] = {{&S6776[84], 6}, {NULL}};
const Segment SGI65580[] = {{&S16383[44], 6}, {NULL}};
const Segment SGI65581[] = {{&S16383[44], 17}, {NULL}};
const Segment SGI65582[] = {{&S16383[30], 6}, {NULL}};
const Segment SGI65583[] = {{&S16383[78], 6}, {NULL}};
const Segment SGI65584[] = {{&S16384[44], 11}, {NULL}};
const Segment SGI65585[] = {{&S16384[44], 29}, {NULL}};
const Segment SGI65586[] = {{&S16384[30], 6}, {NULL}};
const Segment SGI65587[] = {{&S16384[121], 3}, {NULL}};
const Segment SGI65588[] = {{&S16384[87], 5}, {NULL}};
const Segment SGI65589[] = {{&S6786[44], 10}, {NULL}};
const Segment SGI65590[] = {{&S6786[44], 22}, {NULL}};
const Segment SGI65591[] = {{&S6786[30], 6}, {NULL}};
const Segment SGI65592[] = {{&S6786[31], 3}, {NULL}};
const Segment SGI65593[] = {{&S6786[80], 6}, {NULL}};
const Segment SGI65594[] = {{&S6790[44], 8}, {NULL}};
const Segment SGI65595[] = {{&S6790[44], 20}, {NULL}};
const Segment SGI65596[] = {{&S6790[30], 6}, {NULL}};
const Segment SGI65597[] = {{&S6790[31], 3}, {NULL}};
const Segment SGI65598[] = {{&S6790[78], 6}, {NULL}};
const Segment SGI65599[] = {{&S16390[46], 6}, {NULL}};
const Segment SGI65600[] = {{&S16390[101], 5}, {NULL}};
const Segment SGI65601[] = {{&S6791[44], 10}, {NULL}};
const Segment SGI65602[] = {{&S6791[44], 22}, {NULL}};
const Segment SGI65603[] = {{&S6791[30], 6}, {NULL}};
const Segment SGI65604[] = {{&S6791[31], 3}, {NULL}};
const Segment SGI65605[] = {{&S6791[80], 6}, {NULL}};
const Segment SGI65606[] = {{&S6796[44], 7}, {NULL}};
const Segment SGI65607[] = {{&S6796[44], 19}, {NULL}};
const Segment SGI65608[] = {{&S6796[30], 6}, {NULL}};
const Segment SGI65609[] = {{&S6796[31], 3}, {NULL}};
const Segment SGI65610[] = {{&S6796[77], 6}, {NULL}};
const Segment SGI65611[] = {{&S16400[44], 16}, {NULL}};
const Segment SGI65612[] = {{&S16400[44], 27}, {NULL}};
const Segment SGI65613[] = {{&S16400[30], 6}, {NULL}};
const Segment SGI65614[] = {{&S6799[44], 8}, {NULL}};
const Segment SGI65615[] = {{&S6799[44], 20}, {NULL}};
const Segment SGI65616[] = {{&S6799[30], 6}, {NULL}};
const Segment SGI65617[] = {{&S16400[119], 3}, {NULL}};
const Segment SGI65618[] = {{&S16400[85], 5}, {NULL}};
const Segment SGI65619[] = {{&S6799[31], 3}, {NULL}};
const Segment SGI65620[] = {{&S6799[78], 6}, {NULL}};
const Segment SGI65621[] = {{&S6801[44], 12}, {NULL}};
const Segment SGI65622[] = {{&S6801[44], 33}, {NULL}};
const Segment SGI65623[] = {{&S6801[30], 6}, {NULL}};
const Segment SGI65624[] = {{&S6801[31], 3}, {NULL}};
const Segment SGI65625[] = {{&S6801[91], 5}, {NULL}};
const Segment SGI65626[] = {{&S16405[44], 16}, {NULL}};
const Segment SGI65627[] = {{&S16405[44], 27}, {NULL}};
const Segment SGI65628[] = {{&S16405[30], 6}, {NULL}};
const Segment SGI65629[] = {{&S16405[119], 3}, {NULL}};
const Segment SGI65630[] = {{&S16405[85], 5}, {NULL}};
const Segment SGI65631[] = {{&S6807[44], 6}, {NULL}};
const Segment SGI65632[] = {{&S6807[44], 18}, {NULL}};
const Segment SGI65633[] = {{&S6807[30], 6}, {NULL}};
const Segment SGI65634[] = {{&S6807[31], 3}, {NULL}};
const Segment SGI65635[] = {{&S6807[76], 6}, {NULL}};
const Segment SGI65636[] = {{&S6808[44], 10}, {NULL}};
const Segment SGI65637[] = {{&S6808[44], 22}, {NULL}};
const Segment SGI65638[] = {{&S6808[30], 6}, {NULL}};
const Segment SGI65639[] = {{&S6808[31], 3}, {NULL}};
const Segment SGI65640[] = {{&S6808[80], 6}, {NULL}};
const Segment SGI65641[] = {{&S16418[44], 12}, {NULL}};
const Segment SGI65642[] = {{&S16418[44], 28}, {NULL}};
const Segment SGI65643[] = {{&S16418[30], 6}, {NULL}};
const Segment SGI65644[] = {{&S16418[120], 3}, {NULL}};
const Segment SGI65645[] = {{&S16418[86], 5}, {NULL}};
const Segment SGI65646[] = {{&S6818[44], 6}, {NULL}};
const Segment SGI65647[] = {{&S6818[44], 18}, {NULL}};
const Segment SGI65648[] = {{&S6818[30], 6}, {NULL}};
const Segment SGI65649[] = {{&S6818[31], 3}, {NULL}};
const Segment SGI65650[] = {{&S6818[76], 6}, {NULL}};
const Segment SGI65651[] = {{&S6825[44], 9}, {NULL}};
const Segment SGI65652[] = {{&S6825[44], 37}, {NULL}};
const Segment SGI65653[] = {{&S6825[30], 6}, {NULL}};
const Segment SGI65654[] = {{&S6825[31], 3}, {NULL}};
const Segment SGI65655[] = {{&S6825[95], 6}, {NULL}};
const Segment SGI65656[] = {{&S6828[44], 25}, {NULL}};
const Segment SGI65657[] = {{&S6828[44], 52}, {NULL}};
const Segment SGI65658[] = {{&S6828[30], 6}, {NULL}};
const Segment SGI65659[] = {{&S6828[31], 3}, {NULL}};
const Segment SGI65660[] = {{&S6828[110], 6}, {NULL}};
const Segment SGI65661[] = {{&S16435[44], 11}, {NULL}};
const Segment SGI65662[] = {{&S16435[44], 27}, {NULL}};
const Segment SGI65663[] = {{&S16435[30], 6}, {NULL}};
const Segment SGI65664[] = {{&S16435[119], 3}, {NULL}};
const Segment SGI65665[] = {{&S16435[85], 5}, {NULL}};
const Segment SGI65666[] = {{&S6830[44], 15}, {NULL}};
const Segment SGI65667[] = {{&S6830[44], 27}, {NULL}};
const Segment SGI65668[] = {{&S6830[30], 6}, {NULL}};
const Segment SGI65669[] = {{&S6830[31], 3}, {NULL}};
const Segment SGI65670[] = {{&S6830[85], 6}, {NULL}};
const Segment SGI65671[] = {{&S6833[44], 13}, {NULL}};
const Segment SGI65672[] = {{&S6833[44], 25}, {NULL}};
const Segment SGI65673[] = {{&S6833[30], 6}, {NULL}};
const Segment SGI65674[] = {{&S6833[31], 3}, {NULL}};
const Segment SGI65675[] = {{&S6833[83], 6}, {NULL}};
const Segment SGI65676[] = {{&S6839[44], 13}, {NULL}};
const Segment SGI65677[] = {{&S6839[44], 25}, {NULL}};
const Segment SGI65678[] = {{&S6839[30], 6}, {NULL}};
const Segment SGI65679[] = {{&S6839[31], 3}, {NULL}};
const Segment SGI65680[] = {{&S6839[83], 6}, {NULL}};
const Segment SGI65681[] = {{&S7605[44], 84}, {NULL}};
const Segment SGI65682[] = {{&S7605[44], 105}, {NULL}};
const Segment SGI65683[] = {{&S16447[44], 7}, {NULL}};
const Segment SGI65684[] = {{&S7605[30], 6}, {NULL}};
const Segment SGI65685[] = {{&S16447[44], 18}, {NULL}};
const Segment SGI65686[] = {{&S16447[30], 6}, {NULL}};
const Segment SGI65687[] = {{&S7605[31], 3}, {NULL}};
const Segment SGI65688[] = {{&S16447[79], 6}, {NULL}};
const Segment SGI65689[] = {{&S7605[122], 5}, {NULL}};
const Segment SGI65690[] = {{&S16450[44], 6}, {NULL}};
const Segment SGI65691[] = {{&S16450[44], 17}, {NULL}};
const Segment SGI65692[] = {{&S16450[30], 6}, {NULL}};
const Segment SGI65693[] = {{&S16450[78], 6}, {NULL}};
const Segment SGI65694[] = {{&S16454[44], 9}, {NULL}};
const Segment SGI65695[] = {{&S16454[44], 26}, {NULL}};
const Segment SGI65696[] = {{&S16454[30], 6}, {NULL}};
const Segment SGI65697[] = {{&S16454[118], 3}, {NULL}};
const Segment SGI65698[] = {{&S16454[84], 5}, {NULL}};
const Segment SGI65699[] = {{&S6852[44], 10}, {NULL}};
const Segment SGI65700[] = {{&S6852[44], 22}, {NULL}};
const Segment SGI65701[] = {{&S6852[30], 6}, {NULL}};
const Segment SGI65702[] = {{&S16457[44], 12}, {NULL}};
const Segment SGI65703[] = {{&S16457[44], 28}, {NULL}};
const Segment SGI65704[] = {{&S6852[31], 3}, {NULL}};
const Segment SGI65705[] = {{&S16457[30], 6}, {NULL}};
const Segment SGI65706[] = {{&S6852[80], 6}, {NULL}};
const Segment SGI65707[] = {{&S16457[132], 3}, {NULL}};
const Segment SGI65708[] = {{&S16457[98], 5}, {NULL}};
const Segment SGI65709[] = {{&S6854[44], 13}, {NULL}};
const Segment SGI65710[] = {{&S6854[44], 25}, {NULL}};
const Segment SGI65711[] = {{&S6854[30], 6}, {NULL}};
const Segment SGI65712[] = {{&S6854[31], 3}, {NULL}};
const Segment SGI65713[] = {{&S6854[83], 6}, {NULL}};
const Segment SGI65714[] = {{&S6861[44], 11}, {NULL}};
const Segment SGI65715[] = {{&S16466[44], 23}, {NULL}};
const Segment SGI65716[] = {{&S6861[44], 23}, {NULL}};
const Segment SGI65717[] = {{&S6861[30], 6}, {NULL}};
const Segment SGI65718[] = {{&S16466[44], 34}, {NULL}};
const Segment SGI65719[] = {{&S16466[30], 6}, {NULL}};
const Segment SGI65720[] = {{&S6861[31], 3}, {NULL}};
const Segment SGI65721[] = {{&S16466[126], 3}, {NULL}};
const Segment SGI65722[] = {{&S6861[81], 6}, {NULL}};
const Segment SGI65723[] = {{&S16466[92], 5}, {NULL}};
const Segment SGI65724[] = {{&S16468[44], 5}, {NULL}};
const Segment SGI65725[] = {{&S16468[44], 16}, {NULL}};
const Segment SGI65726[] = {{&S16468[30], 6}, {NULL}};
const Segment SGI65727[] = {{&S16468[108], 3}, {NULL}};
const Segment SGI65728[] = {{&S16468[74], 5}, {NULL}};
const Segment SGI65729[] = {{&S16477[44], 15}, {NULL}};
const Segment SGI65730[] = {{&S16477[44], 36}, {NULL}};
const Segment SGI65731[] = {{&S16477[30], 6}, {NULL}};
const Segment SGI65732[] = {{&S16477[128], 3}, {NULL}};
const Segment SGI65733[] = {{&S16477[94], 5}, {NULL}};
const Segment SGI65734[] = {{&S16478[44], 5}, {NULL}};
const Segment SGI65735[] = {{&S6875[44], 10}, {NULL}};
const Segment SGI65736[] = {{&S6875[44], 22}, {NULL}};
const Segment SGI65737[] = {{&S16478[44], 21}, {NULL}};
const Segment SGI65738[] = {{&S6875[30], 6}, {NULL}};
const Segment SGI65739[] = {{&S16478[30], 6}, {NULL}};
const Segment SGI65740[] = {{&S16478[113], 3}, {NULL}};
const Segment SGI65741[] = {{&S6875[31], 3}, {NULL}};
const Segment SGI65742[] = {{&S6875[80], 6}, {NULL}};
const Segment SGI65743[] = {{&S16478[79], 5}, {NULL}};
const Segment SGI65744[] = {{&S7664[44], 10}, {NULL}};
const Segment SGI65745[] = {{&S7664[44], 31}, {NULL}};
const Segment SGI65746[] = {{&S7664[30], 6}, {NULL}};
const Segment SGI65747[] = {{&S6896[44], 9}, {NULL}};
const Segment SGI65748[] = {{&S6896[44], 21}, {NULL}};
const Segment SGI65749[] = {{&S6896[30], 6}, {NULL}};
const Segment SGI65750[] = {{&S7664[31], 3}, {NULL}};
const Segment SGI65751[] = {{&S7664[89], 5}, {NULL}};
const Segment SGI65752[] = {{&S6896[31], 3}, {NULL}};
const Segment SGI65753[] = {{&S6896[79], 6}, {NULL}};
const Segment SGI65754[] = {{&S6897[44], 7}, {NULL}};
const Segment SGI65755[] = {{&S6897[44], 19}, {NULL}};
const Segment SGI65756[] = {{&S6897[30], 6}, {NULL}};
const Segment SGI65757[] = {{&S6897[31], 3}, {NULL}};
const Segment SGI65758[] = {{&S6897[77], 6}, {NULL}};
const Segment SGI65759[] = {{&S6898[44], 12}, {NULL}};
const Segment SGI65760[] = {{&S6898[44], 24}, {NULL}};
const Segment SGI65761[] = {{&S6898[30], 6}, {NULL}};
const Segment SGI65762[] = {{&S6898[31], 3}, {NULL}};
const Segment SGI65763[] = {{&S6898[82], 6}, {NULL}};
const Segment SGI65764[] = {{&S6899[44], 20}, {NULL}};
const Segment SGI65765[] = {{&S6899[44], 32}, {NULL}};
const Segment SGI65766[] = {{&S6899[30], 6}, {NULL}};
const Segment SGI65767[] = {{&S6899[31], 3}, {NULL}};
const Segment SGI65768[] = {{&S6899[91], 6}, {NULL}};
const Segment SGI65769[] = {{&S6900[44], 11}, {NULL}};
const Segment SGI65770[] = {{&S6900[44], 23}, {NULL}};
const Segment SGI65771[] = {{&S6900[30], 6}, {NULL}};
const Segment SGI65772[] = {{&S6900[31], 3}, {NULL}};
const Segment SGI65773[] = {{&S6900[81], 6}, {NULL}};
const Segment SGI65774[] = {{&S6901[44], 10}, {NULL}};
const Segment SGI65775[] = {{&S6901[44], 19}, {NULL}};
const Segment SGI65776[] = {{&S6901[30], 6}, {NULL}};
const Segment SGI65777[] = {{&S6901[31], 3}, {NULL}};
const Segment SGI65778[] = {{&S6901[77], 6}, {NULL}};
const Segment SGI65779[] = {{&S6902[44], 5}, {NULL}};
const Segment SGI65780[] = {{&S6902[44], 14}, {NULL}};
const Segment SGI65781[] = {{&S6902[30], 6}, {NULL}};
const Segment SGI65782[] = {{&S6902[31], 3}, {NULL}};
const Segment SGI65783[] = {{&S6902[72], 6}, {NULL}};
const Segment SGI65784[] = {{&S6903[44], 6}, {NULL}};
const Segment SGI65785[] = {{&S6903[44], 18}, {NULL}};
const Segment SGI65786[] = {{&S6903[30], 6}, {NULL}};
const Segment SGI65787[] = {{&S6903[31], 3}, {NULL}};
const Segment SGI65788[] = {{&S6903[76], 6}, {NULL}};
const Segment SGI65789[] = {{&S6905[44], 8}, {NULL}};
const Segment SGI65790[] = {{&S6905[44], 20}, {NULL}};
const Segment SGI65791[] = {{&S6905[30], 6}, {NULL}};
const Segment SGI65792[] = {{&S6905[31], 3}, {NULL}};
const Segment SGI65793[] = {{&S6905[78], 6}, {NULL}};
const Segment SGI65794[] = {{&S16497[44], 5}, {NULL}};
const Segment SGI65795[] = {{&S16497[44], 21}, {NULL}};
const Segment SGI65796[] = {{&S16497[30], 6}, {NULL}};
const Segment SGI65797[] = {{&S16497[113], 3}, {NULL}};
const Segment SGI65798[] = {{&S16497[79], 5}, {NULL}};
const Segment SGI65799[] = {{&S6908[27], 6}, {NULL}};
const Segment SGI65800[] = {{&S6908[28], 3}, {NULL}};
const Segment SGI65801[] = {{&S6908[47], 6}, {NULL}};
const Segment SGI65802[] = {{&S6908[100], 7}, {NULL}};
const Segment SGI65803[] = {{&S6910[44], 14}, {NULL}};
const Segment SGI65804[] = {{&S6910[44], 26}, {NULL}};
const Segment SGI65805[] = {{&S6910[30], 6}, {NULL}};
const Segment SGI65806[] = {{&S6910[31], 3}, {NULL}};
const Segment SGI65807[] = {{&S6910[84], 6}, {NULL}};
const Segment SGI65808[] = {{&S16504[44], 6}, {NULL}};
const Segment SGI65809[] = {{&S16504[44], 17}, {NULL}};
const Segment SGI65810[] = {{&S16504[30], 6}, {NULL}};
const Segment SGI65811[] = {{&S6912[44], 10}, {NULL}};
const Segment SGI65812[] = {{&S6912[44], 22}, {NULL}};
const Segment SGI65813[] = {{&S6912[30], 6}, {NULL}};
const Segment SGI65814[] = {{&S16504[78], 6}, {NULL}};
const Segment SGI65815[] = {{&S6912[31], 3}, {NULL}};
const Segment SGI65816[] = {{&S6912[80], 6}, {NULL}};
const Segment SGI65817[] = {{&S16505[44], 11}, {NULL}};
const Segment SGI65818[] = {{&S16505[44], 27}, {NULL}};
const Segment SGI65819[] = {{&S16505[30], 6}, {NULL}};
const Segment SGI65820[] = {{&S16505[119], 3}, {NULL}};
const Segment SGI65821[] = {{&S16505[85], 5}, {NULL}};
const Segment SGI65822[] = {{&S6919[44], 9}, {NULL}};
const Segment SGI65823[] = {{&S6919[44], 21}, {NULL}};
const Segment SGI65824[] = {{&S6919[30], 6}, {NULL}};
const Segment SGI65825[] = {{&S6919[31], 3}, {NULL}};
const Segment SGI65826[] = {{&S6919[79], 6}, {NULL}};
const Segment SGI65827[] = {{&S16510[44], 31}, {NULL}};
const Segment SGI65828[] = {{&S16510[30], 6}, {NULL}};
const Segment SGI65829[] = {{&S6920[44], 15}, {NULL}};
const Segment SGI65830[] = {{&S6920[44], 27}, {NULL}};
const Segment SGI65831[] = {{&S6920[30], 6}, {NULL}};
const Segment SGI65832[] = {{&S6920[31], 3}, {NULL}};
const Segment SGI65833[] = {{&S6920[85], 6}, {NULL}};
const Segment SGI65834[] = {{&S16531[44], 7}, {NULL}};
const Segment SGI65835[] = {{&S16531[44], 18}, {NULL}};
const Segment SGI65836[] = {{&S16531[30], 6}, {NULL}};
const Segment SGI65837[] = {{&S16531[79], 6}, {NULL}};
const Segment SGI65838[] = {{&S16536[44], 14}, {NULL}};
const Segment SGI65839[] = {{&S16536[44], 31}, {NULL}};
const Segment SGI65840[] = {{&S16536[30], 6}, {NULL}};
const Segment SGI65841[] = {{&S16536[123], 3}, {NULL}};
const Segment SGI65842[] = {{&S16536[89], 5}, {NULL}};
const Segment SGI65843[] = {{&S16540[44], 6}, {NULL}};
const Segment SGI65844[] = {{&S16540[44], 17}, {NULL}};
const Segment SGI65845[] = {{&S16540[30], 6}, {NULL}};
const Segment SGI65846[] = {{&S16540[109], 3}, {NULL}};
const Segment SGI65847[] = {{&S16540[75], 5}, {NULL}};
const Segment SGI65848[] = {{&S16547[44], 10}, {NULL}};
const Segment SGI65849[] = {{&S16547[44], 21}, {NULL}};
const Segment SGI65850[] = {{&S16547[30], 6}, {NULL}};
const Segment SGI65851[] = {{&S16547[82], 6}, {NULL}};
const Segment SGI65852[] = {{&S16591[115], 11}, {NULL}};
const Segment SGI65853[] = {{&S16591[115], 22}, {NULL}};
const Segment SGI65854[] = {{&S16591[101], 6}, {NULL}};
const Segment SGI65855[] = {{&S16591[185], 3}, {NULL}};
const Segment SGI65856[] = {{&S16591[65], 5}, {NULL}};
const Segment SGI65857[] = {{&S6922[44], 6}, {NULL}};
const Segment SGI65858[] = {{&S6922[44], 18}, {NULL}};
const Segment SGI65859[] = {{&S6922[30], 6}, {NULL}};
const Segment SGI65860[] = {{&S6922[31], 3}, {NULL}};
const Segment SGI65861[] = {{&S6922[76], 5}, {NULL}};
const Segment SGI65862[] = {{&S6923[44], 6}, {NULL}};
const Segment SGI65863[] = {{&S6923[44], 18}, {NULL}};
const Segment SGI65864[] = {{&S6923[30], 6}, {NULL}};
const Segment SGI65865[] = {{&S6923[31], 3}, {NULL}};
const Segment SGI65866[] = {{&S6923[76], 6}, {NULL}};
const Segment SGI65867[] = {{&S6929[44], 7}, {NULL}};
const Segment SGI65868[] = {{&S6929[44], 19}, {NULL}};
const Segment SGI65869[] = {{&S6929[30], 6}, {NULL}};
const Segment SGI65870[] = {{&S6929[31], 3}, {NULL}};
const Segment SGI65871[] = {{&S6929[77], 6}, {NULL}};
const Segment SGI65872[] = {{&S6930[44], 9}, {NULL}};
const Segment SGI65873[] = {{&S6930[44], 21}, {NULL}};
const Segment SGI65874[] = {{&S6930[30], 6}, {NULL}};
const Segment SGI65875[] = {{&S6930[31], 3}, {NULL}};
const Segment SGI65876[] = {{&S6930[79], 6}, {NULL}};
const Segment SGI65877[] = {{&S16608[44], 6}, {NULL}};
const Segment SGI65878[] = {{&S16608[44], 22}, {NULL}};
const Segment SGI65879[] = {{&S16608[30], 6}, {NULL}};
const Segment SGI65880[] = {{&S16608[114], 3}, {NULL}};
const Segment SGI65881[] = {{&S6932[44], 7}, {NULL}};
const Segment SGI65882[] = {{&S6932[44], 19}, {NULL}};
const Segment SGI65883[] = {{&S16608[80], 5}, {NULL}};
const Segment SGI65884[] = {{&S6932[30], 6}, {NULL}};
const Segment SGI65885[] = {{&S6932[31], 3}, {NULL}};
const Segment SGI65886[] = {{&S6932[77], 6}, {NULL}};
const Segment SGI65887[] = {{&S6934[44], 12}, {NULL}};
const Segment SGI65888[] = {{&S6934[44], 24}, {NULL}};
const Segment SGI65889[] = {{&S6934[30], 6}, {NULL}};
const Segment SGI65890[] = {{&S6934[31], 3}, {NULL}};
const Segment SGI65891[] = {{&S6934[82], 6}, {NULL}};
const Segment SGI65892[] = {{&S6935[44], 14}, {NULL}};
const Segment SGI65893[] = {{&S6935[44], 26}, {NULL}};
const Segment SGI65894[] = {{&S6935[30], 6}, {NULL}};
const Segment SGI65895[] = {{&S6935[31], 3}, {NULL}};
const Segment SGI65896[] = {{&S6935[84], 6}, {NULL}};
const Segment SGI65897[] = {{&S6936[44], 6}, {NULL}};
const Segment SGI65898[] = {{&S6936[44], 29}, {NULL}};
const Segment SGI65899[] = {{&S6936[30], 6}, {NULL}};
const Segment SGI65900[] = {{&S6936[31], 3}, {NULL}};
const Segment SGI65901[] = {{&S6936[87], 6}, {NULL}};
const Segment SGI65902[] = {{&S6937[44], 15}, {NULL}};
const Segment SGI65903[] = {{&S6937[44], 27}, {NULL}};
const Segment SGI65904[] = {{&S6937[30], 6}, {NULL}};
const Segment SGI65905[] = {{&S16617[44], 8}, {NULL}};
const Segment SGI65906[] = {{&S16617[44], 23}, {NULL}};
const Segment SGI65907[] = {{&S6937[31], 3}, {NULL}};
const Segment SGI65908[] = {{&S16617[30], 6}, {NULL}};
const Segment SGI65909[] = {{&S6937[85], 6}, {NULL}};
const Segment SGI65910[] = {{&S16617[115], 3}, {NULL}};
const Segment SGI65911[] = {{&S16617[81], 5}, {NULL}};
const Segment SGI65912[] = {{&S6939[44], 6}, {NULL}};
const Segment SGI65913[] = {{&S6939[44], 18}, {NULL}};
const Segment SGI65914[] = {{&S6939[30], 6}, {NULL}};
const Segment SGI65915[] = {{&S6939[31], 3}, {NULL}};
const Segment SGI65916[] = {{&S6939[76], 6}, {NULL}};
const Segment SGI65917[] = {{&S6940[44], 10}, {NULL}};
const Segment SGI65918[] = {{&S6940[44], 22}, {NULL}};
const Segment SGI65919[] = {{&S6940[30], 6}, {NULL}};
const Segment SGI65920[] = {{&S6940[31], 3}, {NULL}};
const Segment SGI65921[] = {{&S6940[80], 6}, {NULL}};
const Segment SGI65922[] = {{&S6941[44], 6}, {NULL}};
const Segment SGI65923[] = {{&S6941[44], 24}, {NULL}};
const Segment SGI65924[] = {{&S6941[30], 6}, {NULL}};
const Segment SGI65925[] = {{&S6941[31], 3}, {NULL}};
const Segment SGI65926[] = {{&S6941[82], 6}, {NULL}};
const Segment SGI65927[] = {{&S6945[44], 12}, {NULL}};
const Segment SGI65928[] = {{&S16628[44], 12}, {NULL}};
const Segment SGI65929[] = {{&S6945[44], 24}, {NULL}};
const Segment SGI65930[] = {{&S16628[44], 23}, {NULL}};
const Segment SGI65931[] = {{&S6945[30], 6}, {NULL}};
const Segment SGI65932[] = {{&S16628[30], 6}, {NULL}};
const Segment SGI65933[] = {{&S6945[31], 3}, {NULL}};
const Segment SGI65934[] = {{&S6945[82], 6}, {NULL}};
const Segment SGI65935[] = {{&S16628[84], 6}, {NULL}};
const Segment SGI65936[] = {{&S6947[44], 7}, {NULL}};
const Segment SGI65937[] = {{&S6947[44], 19}, {NULL}};
const Segment SGI65938[] = {{&S6947[30], 6}, {NULL}};
const Segment SGI65939[] = {{&S6947[31], 3}, {NULL}};
const Segment SGI65940[] = {{&S6947[77], 6}, {NULL}};
const Segment SGI65941[] = {{&S6950[44], 8}, {NULL}};
const Segment SGI65942[] = {{&S6950[44], 20}, {NULL}};
const Segment SGI65943[] = {{&S6950[30], 6}, {NULL}};
const Segment SGI65944[] = {{&S6950[31], 3}, {NULL}};
const Segment SGI65945[] = {{&S6950[78], 6}, {NULL}};
const Segment SGI65946[] = {{&S16640[44], 6}, {NULL}};
const Segment SGI65947[] = {{&S16640[44], 17}, {NULL}};
const Segment SGI65948[] = {{&S16640[30], 6}, {NULL}};
const Segment SGI65949[] = {{&S16640[130], 3}, {NULL}};
const Segment SGI65950[] = {{&S16640[96], 5}, {NULL}};
const Segment SGI65951[] = {{&S6957[44], 12}, {NULL}};
const Segment SGI65952[] = {{&S6957[44], 24}, {NULL}};
const Segment SGI65953[] = {{&S6957[30], 6}, {NULL}};
const Segment SGI65954[] = {{&S6957[31], 3}, {NULL}};
const Segment SGI65955[] = {{&S6957[82], 6}, {NULL}};
const Segment SGI65956[] = {{&S16642[44], 9}, {NULL}};
const Segment SGI65957[] = {{&S16642[44], 24}, {NULL}};
const Segment SGI65958[] = {{&S16642[30], 6}, {NULL}};
const Segment SGI65959[] = {{&S16642[116], 3}, {NULL}};
const Segment SGI65960[] = {{&S16642[82], 5}, {NULL}};
const Segment SGI65961[] = {{&S6964[44], 11}, {NULL}};
const Segment SGI65962[] = {{&S6964[44], 26}, {NULL}};
const Segment SGI65963[] = {{&S6964[30], 6}, {NULL}};
const Segment SGI65964[] = {{&S16648[44], 6}, {NULL}};
const Segment SGI65965[] = {{&S16648[44], 17}, {NULL}};
const Segment SGI65966[] = {{&S16648[30], 6}, {NULL}};
const Segment SGI65967[] = {{&S6964[31], 3}, {NULL}};
const Segment SGI65968[] = {{&S6964[84], 6}, {NULL}};
const Segment SGI65969[] = {{&S16648[78], 6}, {NULL}};
const Segment SGI65970[] = {{&S6967[44], 22}, {NULL}};
const Segment SGI65971[] = {{&S6967[44], 36}, {NULL}};
const Segment SGI65972[] = {{&S6967[30], 6}, {NULL}};
const Segment SGI65973[] = {{&S6967[31], 3}, {NULL}};
const Segment SGI65974[] = {{&S6967[94], 6}, {NULL}};
const Segment SGI65975[] = {{&S6968[44], 9}, {NULL}};
const Segment SGI65976[] = {{&S6968[44], 21}, {NULL}};
const Segment SGI65977[] = {{&S6968[30], 6}, {NULL}};
const Segment SGI65978[] = {{&S6968[31], 3}, {NULL}};
const Segment SGI65979[] = {{&S6968[79], 6}, {NULL}};
const Segment SGI65980[] = {{&S6969[44], 10}, {NULL}};
const Segment SGI65981[] = {{&S6969[44], 22}, {NULL}};
const Segment SGI65982[] = {{&S6969[30], 6}, {NULL}};
const Segment SGI65983[] = {{&S6969[31], 3}, {NULL}};
const Segment SGI65984[] = {{&S6969[80], 6}, {NULL}};
const Segment SGI65985[] = {{&S6972[44], 9}, {NULL}};
const Segment SGI65986[] = {{&S6972[44], 21}, {NULL}};
const Segment SGI65987[] = {{&S6972[30], 6}, {NULL}};
const Segment SGI65988[] = {{&S6972[31], 3}, {NULL}};
const Segment SGI65989[] = {{&S6972[79], 6}, {NULL}};
const Segment SGI65990[] = {{&S16660[44], 9}, {NULL}};
const Segment SGI65991[] = {{&S16660[44], 26}, {NULL}};
const Segment SGI65992[] = {{&S16660[30], 6}, {NULL}};
const Segment SGI65993[] = {{&S16660[118], 3}, {NULL}};
const Segment SGI65994[] = {{&S16660[84], 5}, {NULL}};
const Segment SGI65995[] = {{&S6973[44], 18}, {NULL}};
const Segment SGI65996[] = {{&S6973[44], 27}, {NULL}};
const Segment SGI65997[] = {{&S6973[30], 6}, {NULL}};
const Segment SGI65998[] = {{&S6973[31], 3}, {NULL}};
const Segment SGI65999[] = {{&S6973[85], 6}, {NULL}};
const Segment SGI66000[] = {{&S6974[44], 9}, {NULL}};
const Segment SGI66001[] = {{&S6974[44], 21}, {NULL}};
const Segment SGI66002[] = {{&S6974[30], 6}, {NULL}};
const Segment SGI66003[] = {{&S6974[31], 3}, {NULL}};
const Segment SGI66004[] = {{&S6974[79], 6}, {NULL}};
const Segment SGI66005[] = {{&S6975[44], 9}, {NULL}};
const Segment SGI66006[] = {{&S6975[44], 21}, {NULL}};
const Segment SGI66007[] = {{&S6975[30], 6}, {NULL}};
const Segment SGI66008[] = {{&S6975[31], 3}, {NULL}};
const Segment SGI66009[] = {{&S6975[79], 6}, {NULL}};
const Segment SGI66010[] = {{&S16668[44], 10}, {NULL}};
const Segment SGI66011[] = {{&S16668[44], 27}, {NULL}};
const Segment SGI66012[] = {{&S16668[30], 6}, {NULL}};
const Segment SGI66013[] = {{&S16668[119], 3}, {NULL}};
const Segment SGI66014[] = {{&S16668[85], 5}, {NULL}};
const Segment SGI66015[] = {{&S6982[44], 6}, {NULL}};
const Segment SGI66016[] = {{&S6982[44], 18}, {NULL}};
const Segment SGI66017[] = {{&S6982[30], 6}, {NULL}};
const Segment SGI66018[] = {{&S6982[79], 6}, {NULL}};
const Segment SGI66019[] = {{&S16674[44], 9}, {NULL}};
const Segment SGI66020[] = {{&S16674[44], 22}, {NULL}};
const Segment SGI66021[] = {{&S16674[30], 6}, {NULL}};
const Segment SGI66022[] = {{&S16674[114], 3}, {NULL}};
const Segment SGI66023[] = {{&S16674[80], 5}, {NULL}};
const Segment SGI66024[] = {{&S6986[44], 5}, {NULL}};
const Segment SGI66025[] = {{&S6986[44], 17}, {NULL}};
const Segment SGI66026[] = {{&S6986[30], 6}, {NULL}};
const Segment SGI66027[] = {{&S6986[31], 3}, {NULL}};
const Segment SGI66028[] = {{&S6986[75], 6}, {NULL}};
const Segment SGI66029[] = {{&S16684[44], 16}, {NULL}};
const Segment SGI66030[] = {{&S16684[44], 29}, {NULL}};
const Segment SGI66031[] = {{&S16684[30], 6}, {NULL}};
const Segment SGI66032[] = {{&S16684[121], 3}, {NULL}};
const Segment SGI66033[] = {{&S16684[87], 5}, {NULL}};
const Segment SGI66034[] = {{&S6999[44], 18}, {NULL}};
const Segment SGI66035[] = {{&S6999[44], 30}, {NULL}};
const Segment SGI66036[] = {{&S6999[30], 6}, {NULL}};
const Segment SGI66037[] = {{&S6999[31], 3}, {NULL}};
const Segment SGI66038[] = {{&S6999[88], 6}, {NULL}};
const Segment SGI66039[] = {{&S7001[44], 11}, {NULL}};
const Segment SGI66040[] = {{&S7001[44], 39}, {NULL}};
const Segment SGI66041[] = {{&S7001[30], 6}, {NULL}};
const Segment SGI66042[] = {{&S7001[31], 3}, {NULL}};
const Segment SGI66043[] = {{&S7001[97], 6}, {NULL}};
const Segment SGI66044[] = {{&S16699[44], 34}, {NULL}};
const Segment SGI66045[] = {{&S7010[44], 7}, {NULL}};
const Segment SGI66046[] = {{&S16699[44], 45}, {NULL}};
const Segment SGI66047[] = {{&S7010[44], 19}, {NULL}};
const Segment SGI66048[] = {{&S16699[30], 6}, {NULL}};
const Segment SGI66049[] = {{&S7010[30], 6}, {NULL}};
const Segment SGI66050[] = {{&S7010[31], 3}, {NULL}};
const Segment SGI66051[] = {{&S16699[137], 3}, {NULL}};
const Segment SGI66052[] = {{&S7010[77], 6}, {NULL}};
const Segment SGI66053[] = {{&S16699[103], 5}, {NULL}};
const Segment SGI66054[] = {{&S16714[44], 17}, {NULL}};
const Segment SGI66055[] = {{&S16714[30], 6}, {NULL}};
const Segment SGI66056[] = {{&S12680[42], 10}, {NULL}};
const Segment SGI66057[] = {{&S12680[42], 22}, {NULL}};
const Segment SGI66058[] = {{&S12680[30], 4}, {NULL}};
const Segment SGI66059[] = {{&S12680[31], 3}, {NULL}};
const Segment SGI66060[] = {{&S12680[78], 6}, {NULL}};
const Segment SGI66061[] = {{&S16728[44], 12}, {NULL}};
const Segment SGI66062[] = {{&S16728[44], 33}, {NULL}};
const Segment SGI66063[] = {{&S16728[30], 6}, {NULL}};
const Segment SGI66064[] = {{&S16728[125], 3}, {NULL}};
const Segment SGI66065[] = {{&S16728[91], 5}, {NULL}};
const Segment SGI66066[] = {{&S7046[44], 6}, {NULL}};
const Segment SGI66067[] = {{&S7046[44], 18}, {NULL}};
const Segment SGI66068[] = {{&S7046[30], 6}, {NULL}};
const Segment SGI66069[] = {{&S7046[31], 3}, {NULL}};
const Segment SGI66070[] = {{&S7046[76], 6}, {NULL}};
const Segment SGI66071[] = {{&S7047[44], 8}, {NULL}};
const Segment SGI66072[] = {{&S7047[44], 20}, {NULL}};
const Segment SGI66073[] = {{&S7047[30], 6}, {NULL}};
const Segment SGI66074[] = {{&S7047[31], 3}, {NULL}};
const Segment SGI66075[] = {{&S7047[78], 6}, {NULL}};
const Segment SGI66076[] = {{&S7048[44], 11}, {NULL}};
const Segment SGI66077[] = {{&S7048[44], 23}, {NULL}};
const Segment SGI66078[] = {{&S7048[30], 6}, {NULL}};
const Segment SGI66079[] = {{&S7048[31], 3}, {NULL}};
const Segment SGI66080[] = {{&S7048[81], 6}, {NULL}};
const Segment SGI66081[] = {{&S7049[44], 23}, {NULL}};
const Segment SGI66082[] = {{&S7049[44], 35}, {NULL}};
const Segment SGI66083[] = {{&S7049[30], 6}, {NULL}};
const Segment SGI66084[] = {{&S16743[44], 8}, {NULL}};
const Segment SGI66085[] = {{&S16743[44], 23}, {NULL}};
const Segment SGI66086[] = {{&S16743[30], 6}, {NULL}};
const Segment SGI66087[] = {{&S16743[115], 3}, {NULL}};
const Segment SGI66088[] = {{&S16743[81], 5}, {NULL}};
const Segment SGI66089[] = {{&S7049[31], 3}, {NULL}};
const Segment SGI66090[] = {{&S7049[93], 6}, {NULL}};
const Segment SGI66091[] = {{&S7050[44], 14}, {NULL}};
const Segment SGI66092[] = {{&S7050[44], 26}, {NULL}};
const Segment SGI66093[] = {{&S7050[30], 6}, {NULL}};
const Segment SGI66094[] = {{&S7050[31], 3}, {NULL}};
const Segment SGI66095[] = {{&S7050[84], 6}, {NULL}};
const Segment SGI66096[] = {{&S7051[44], 7}, {NULL}};
const Segment SGI66097[] = {{&S7051[44], 19}, {NULL}};
const Segment SGI66098[] = {{&S16752[44], 10}, {NULL}};
const Segment SGI66099[] = {{&S7051[30], 6}, {NULL}};
const Segment SGI66100[] = {{&S16752[44], 21}, {NULL}};
const Segment SGI66101[] = {{&S16752[30], 6}, {NULL}};
const Segment SGI66102[] = {{&S7051[31], 3}, {NULL}};
const Segment SGI66103[] = {{&S7051[77], 6}, {NULL}};
const Segment SGI66104[] = {{&S16752[113], 3}, {NULL}};
const Segment SGI66105[] = {{&S16752[79], 5}, {NULL}};
const Segment SGI66106[] = {{&S7052[44], 5}, {NULL}};
const Segment SGI66107[] = {{&S7052[44], 17}, {NULL}};
const Segment SGI66108[] = {{&S7052[30], 6}, {NULL}};
const Segment SGI66109[] = {{&S7052[31], 3}, {NULL}};
const Segment SGI66110[] = {{&S7052[75], 6}, {NULL}};
const Segment SGI66111[] = {{&S7054[44], 11}, {NULL}};
const Segment SGI66112[] = {{&S7054[44], 23}, {NULL}};
const Segment SGI66113[] = {{&S7054[30], 6}, {NULL}};
const Segment SGI66114[] = {{&S7054[31], 3}, {NULL}};
const Segment SGI66115[] = {{&S7054[81], 6}, {NULL}};
const Segment SGI66116[] = {{&S16757[44], 5}, {NULL}};
const Segment SGI66117[] = {{&S16757[44], 17}, {NULL}};
const Segment SGI66118[] = {{&S16757[30], 6}, {NULL}};
const Segment SGI66119[] = {{&S16757[109], 3}, {NULL}};
const Segment SGI66120[] = {{&S16757[75], 5}, {NULL}};
const Segment SGI66121[] = {{&S7058[44], 7}, {NULL}};
const Segment SGI66122[] = {{&S7058[44], 19}, {NULL}};
const Segment SGI66123[] = {{&S7058[30], 6}, {NULL}};
const Segment SGI66124[] = {{&S7058[31], 3}, {NULL}};
const Segment SGI66125[] = {{&S7058[77], 6}, {NULL}};
const Segment SGI66126[] = {{&S7059[44], 13}, {NULL}};
const Segment SGI66127[] = {{&S7059[44], 25}, {NULL}};
const Segment SGI66128[] = {{&S7059[30], 6}, {NULL}};
const Segment SGI66129[] = {{&S7059[31], 3}, {NULL}};
const Segment SGI66130[] = {{&S7059[83], 6}, {NULL}};
const Segment SGI66131[] = {{&S7063[44], 6}, {NULL}};
const Segment SGI66132[] = {{&S7063[44], 18}, {NULL}};
const Segment SGI66133[] = {{&S7063[30], 6}, {NULL}};
const Segment SGI66134[] = {{&S7063[31], 3}, {NULL}};
const Segment SGI66135[] = {{&S7063[76], 6}, {NULL}};
const Segment SGI66136[] = {{&S16769[44], 8}, {NULL}};
const Segment SGI66137[] = {{&S16769[44], 24}, {NULL}};
const Segment SGI66138[] = {{&S16769[30], 6}, {NULL}};
const Segment SGI66139[] = {{&S16769[116], 3}, {NULL}};
const Segment SGI66140[] = {{&S16769[82], 5}, {NULL}};
const Segment SGI66141[] = {{&S7065[44], 10}, {NULL}};
const Segment SGI66142[] = {{&S7065[44], 22}, {NULL}};
const Segment SGI66143[] = {{&S7065[30], 6}, {NULL}};
const Segment SGI66144[] = {{&S7065[31], 3}, {NULL}};
const Segment SGI66145[] = {{&S7065[80], 6}, {NULL}};
const Segment SGI66146[] = {{&S7072[44], 11}, {NULL}};
const Segment SGI66147[] = {{&S7072[44], 23}, {NULL}};
const Segment SGI66148[] = {{&S7072[30], 6}, {NULL}};
const Segment SGI66149[] = {{&S7072[31], 3}, {NULL}};
const Segment SGI66150[] = {{&S7072[81], 6}, {NULL}};
const Segment SGI66151[] = {{&S16778[44], 9}, {NULL}};
const Segment SGI66152[] = {{&S16778[44], 22}, {NULL}};
const Segment SGI66153[] = {{&S16778[30], 6}, {NULL}};
const Segment SGI66154[] = {{&S16778[114], 3}, {NULL}};
const Segment SGI66155[] = {{&S16778[80], 5}, {NULL}};
const Segment SGI66156[] = {{&S7076[44], 10}, {NULL}};
const Segment SGI66157[] = {{&S7076[44], 22}, {NULL}};
const Segment SGI66158[] = {{&S7076[30], 6}, {NULL}};
const Segment SGI66159[] = {{&S7076[31], 3}, {NULL}};
const Segment SGI66160[] = {{&S7076[80], 6}, {NULL}};
const Segment SGI66161[] = {{&S7082[44], 11}, {NULL}};
const Segment SGI66162[] = {{&S7082[44], 20}, {NULL}};
const Segment SGI66163[] = {{&S7082[30], 6}, {NULL}};
const Segment SGI66164[] = {{&S7082[31], 3}, {NULL}};
const Segment SGI66165[] = {{&S7082[78], 6}, {NULL}};
const Segment SGI66166[] = {{&S16791[44], 7}, {NULL}};
const Segment SGI66167[] = {{&S16791[44], 18}, {NULL}};
const Segment SGI66168[] = {{&S16791[30], 6}, {NULL}};
const Segment SGI66169[] = {{&S16791[110], 3}, {NULL}};
const Segment SGI66170[] = {{&S16791[76], 5}, {NULL}};
const Segment SGI66171[] = {{&S7087[44], 7}, {NULL}};
const Segment SGI66172[] = {{&S7087[44], 59}, {NULL}};
const Segment SGI66173[] = {{&S7087[30], 6}, {NULL}};
const Segment SGI66174[] = {{&S7087[31], 3}, {NULL}};
const Segment SGI66175[] = {{&S7087[117], 5}, {NULL}};
const Segment SGI66176[] = {{&S7091[44], 8}, {NULL}};
const Segment SGI66177[] = {{&S7091[44], 20}, {NULL}};
const Segment SGI66178[] = {{&S7091[30], 6}, {NULL}};
const Segment SGI66179[] = {{&S7091[31], 3}, {NULL}};
const Segment SGI66180[] = {{&S7091[78], 6}, {NULL}};
const Segment SGI66181[] = {{&S7100[44], 13}, {NULL}};
const Segment SGI66182[] = {{&S7100[44], 30}, {NULL}};
const Segment SGI66183[] = {{&S7100[30], 6}, {NULL}};
const Segment SGI66184[] = {{&S7100[31], 3}, {NULL}};
const Segment SGI66185[] = {{&S7100[88], 6}, {NULL}};
const Segment SGI66186[] = {{&S7104[44], 13}, {NULL}};
const Segment SGI66187[] = {{&S7104[44], 25}, {NULL}};
const Segment SGI66188[] = {{&S7104[30], 6}, {NULL}};
const Segment SGI66189[] = {{&S7104[31], 3}, {NULL}};
const Segment SGI66190[] = {{&S7104[83], 5}, {NULL}};
const Segment SGI66191[] = {{&S16817[44], 16}, {NULL}};
const Segment SGI66192[] = {{&S16817[44], 29}, {NULL}};
const Segment SGI66193[] = {{&S16817[30], 6}, {NULL}};
const Segment SGI66194[] = {{&S16817[121], 3}, {NULL}};
const Segment SGI66195[] = {{&S16817[87], 5}, {NULL}};
const Segment SGI66196[] = {{&S7110[44], 11}, {NULL}};
const Segment SGI66197[] = {{&S7110[44], 23}, {NULL}};
const Segment SGI66198[] = {{&S7110[30], 6}, {NULL}};
const Segment SGI66199[] = {{&S7110[31], 3}, {NULL}};
const Segment SGI66200[] = {{&S7110[81], 6}, {NULL}};
const Segment SGI66201[] = {{&S16823[44], 8}, {NULL}};
const Segment SGI66202[] = {{&S16823[44], 23}, {NULL}};
const Segment SGI66203[] = {{&S16823[30], 6}, {NULL}};
const Segment SGI66204[] = {{&S16823[115], 3}, {NULL}};
const Segment SGI66205[] = {{&S16823[81], 5}, {NULL}};
const Segment SGI66206[] = {{&S7982[44], 74}, {NULL}};
const Segment SGI66207[] = {{&S7982[44], 95}, {NULL}};
const Segment SGI66208[] = {{&S7982[30], 6}, {NULL}};
const Segment SGI66209[] = {{&S7982[31], 3}, {NULL}};
const Segment SGI66210[] = {{&S7982[112], 5}, {NULL}};
const Segment SGI66211[] = {{&S16828[44], 12}, {NULL}};
const Segment SGI66212[] = {{&S16828[44], 24}, {NULL}};
const Segment SGI66213[] = {{&S16828[30], 6}, {NULL}};
const Segment SGI66214[] = {{&S16828[116], 3}, {NULL}};
const Segment SGI66215[] = {{&S16828[82], 5}, {NULL}};
const Segment SGI66216[] = {{&S7119[44], 6}, {NULL}};
const Segment SGI66217[] = {{&S7119[44], 18}, {NULL}};
const Segment SGI66218[] = {{&S7119[30], 6}, {NULL}};
const Segment SGI66219[] = {{&S7119[31], 3}, {NULL}};
const Segment SGI66220[] = {{&S7119[76], 6}, {NULL}};
const Segment SGI66221[] = {{&S7127[44], 13}, {NULL}};
const Segment SGI66222[] = {{&S7127[44], 25}, {NULL}};
const Segment SGI66223[] = {{&S7127[30], 6}, {NULL}};
const Segment SGI66224[] = {{&S7127[31], 3}, {NULL}};
const Segment SGI66225[] = {{&S7127[83], 6}, {NULL}};
const Segment SGI66226[] = {{&S7133[44], 8}, {NULL}};
const Segment SGI66227[] = {{&S7133[44], 20}, {NULL}};
const Segment SGI66228[] = {{&S7133[30], 6}, {NULL}};
const Segment SGI66229[] = {{&S7133[31], 3}, {NULL}};
const Segment SGI66230[] = {{&S7133[78], 6}, {NULL}};
const Segment SGI66231[] = {{&S7137[44], 7}, {NULL}};
const Segment SGI66232[] = {{&S7137[44], 19}, {NULL}};
const Segment SGI66233[] = {{&S7137[30], 6}, {NULL}};
const Segment SGI66234[] = {{&S7137[31], 3}, {NULL}};
const Segment SGI66235[] = {{&S7137[77], 6}, {NULL}};
const Segment SGI66236[] = {{&S16851[44], 5}, {NULL}};
const Segment SGI66237[] = {{&S16851[44], 17}, {NULL}};
const Segment SGI66238[] = {{&S7139[44], 14}, {NULL}};
const Segment SGI66239[] = {{&S16851[30], 6}, {NULL}};
const Segment SGI66240[] = {{&S7139[44], 26}, {NULL}};
const Segment SGI66241[] = {{&S7139[30], 6}, {NULL}};
const Segment SGI66242[] = {{&S16851[109], 3}, {NULL}};
const Segment SGI66243[] = {{&S7139[31], 3}, {NULL}};
const Segment SGI66244[] = {{&S16851[75], 5}, {NULL}};
const Segment SGI66245[] = {{&S7139[84], 6}, {NULL}};
const Segment SGI66246[] = {{&S7140[44], 8}, {NULL}};
const Segment SGI66247[] = {{&S7140[44], 20}, {NULL}};
const Segment SGI66248[] = {{&S7140[30], 6}, {NULL}};
const Segment SGI66249[] = {{&S7140[31], 3}, {NULL}};
const Segment SGI66250[] = {{&S7140[78], 6}, {NULL}};
const Segment SGI66251[] = {{&S7141[44], 9}, {NULL}};
const Segment SGI66252[] = {{&S7141[44], 21}, {NULL}};
const Segment SGI66253[] = {{&S7141[30], 6}, {NULL}};
const Segment SGI66254[] = {{&S7141[31], 3}, {NULL}};
const Segment SGI66255[] = {{&S7141[79], 6}, {NULL}};
const Segment SGI66256[] = {{&S7142[44], 10}, {NULL}};
const Segment SGI66257[] = {{&S7142[44], 22}, {NULL}};
const Segment SGI66258[] = {{&S7142[30], 6}, {NULL}};
const Segment SGI66259[] = {{&S7142[31], 3}, {NULL}};
const Segment SGI66260[] = {{&S7142[80], 6}, {NULL}};
const Segment SGI66261[] = {{&S8018[44], 8}, {NULL}};
const Segment SGI66262[] = {{&S8018[44], 20}, {NULL}};
const Segment SGI66263[] = {{&S8018[30], 6}, {NULL}};
const Segment SGI66264[] = {{&S8018[31], 3}, {NULL}};
const Segment SGI66265[] = {{&S8018[78], 5}, {NULL}};
const Segment SGI66266[] = {{&S16865[44], 8}, {NULL}};
const Segment SGI66267[] = {{&S16865[44], 24}, {NULL}};
const Segment SGI66268[] = {{&S16865[30], 6}, {NULL}};
const Segment SGI66269[] = {{&S16865[116], 3}, {NULL}};
const Segment SGI66270[] = {{&S16865[82], 5}, {NULL}};
const Segment SGI66271[] = {{&S7152[44], 21}, {NULL}};
const Segment SGI66272[] = {{&S7152[44], 33}, {NULL}};
const Segment SGI66273[] = {{&S7152[30], 6}, {NULL}};
const Segment SGI66274[] = {{&S7152[31], 3}, {NULL}};
const Segment SGI66275[] = {{&S7152[91], 6}, {NULL}};
const Segment SGI66276[] = {{&S7153[44], 7}, {NULL}};
const Segment SGI66277[] = {{&S7153[44], 19}, {NULL}};
const Segment SGI66278[] = {{&S7153[30], 6}, {NULL}};
const Segment SGI66279[] = {{&S16869[44], 12}, {NULL}};
const Segment SGI66280[] = {{&S7153[31], 3}, {NULL}};
const Segment SGI66281[] = {{&S16869[44], 32}, {NULL}};
const Segment SGI66282[] = {{&S7153[77], 6}, {NULL}};
const Segment SGI66283[] = {{&S16869[30], 6}, {NULL}};
const Segment SGI66284[] = {{&S16869[124], 3}, {NULL}};
const Segment SGI66285[] = {{&S16869[90], 5}, {NULL}};
const Segment SGI66286[] = {{&S16870[44], 10}, {NULL}};
const Segment SGI66287[] = {{&S16870[44], 22}, {NULL}};
const Segment SGI66288[] = {{&S16870[30], 6}, {NULL}};
const Segment SGI66289[] = {{&S7155[44], 12}, {NULL}};
const Segment SGI66290[] = {{&S7155[44], 24}, {NULL}};
const Segment SGI66291[] = {{&S16870[114], 3}, {NULL}};
const Segment SGI66292[] = {{&S7155[30], 6}, {NULL}};
const Segment SGI66293[] = {{&S16870[80], 5}, {NULL}};
const Segment SGI66294[] = {{&S7155[31], 3}, {NULL}};
const Segment SGI66295[] = {{&S7155[82], 6}, {NULL}};
const Segment SGI66296[] = {{&S7157[44], 16}, {NULL}};
const Segment SGI66297[] = {{&S7157[30], 6}, {NULL}};
const Segment SGI66298[] = {{&S7157[31], 3}, {NULL}};
const Segment SGI66299[] = {{&S7157[88], 6}, {NULL}};
const Segment SGI66300[] = {{&S7160[44], 8}, {NULL}};
const Segment SGI66301[] = {{&S7160[44], 25}, {NULL}};
const Segment SGI66302[] = {{&S7160[30], 6}, {NULL}};
const Segment SGI66303[] = {{&S7160[31], 3}, {NULL}};
const Segment SGI66304[] = {{&S7160[83], 5}, {NULL}};
const Segment SGI66305[] = {{&S7167[44], 8}, {NULL}};
const Segment SGI66306[] = {{&S7167[44], 20}, {NULL}};
const Segment SGI66307[] = {{&S7167[30], 6}, {NULL}};
const Segment SGI66308[] = {{&S7167[31], 3}, {NULL}};
const Segment SGI66309[] = {{&S7167[78], 6}, {NULL}};
const Segment SGI66310[] = {{&S16897[44], 12}, {NULL}};
const Segment SGI66311[] = {{&S16897[44], 25}, {NULL}};
const Segment SGI66312[] = {{&S16897[30], 6}, {NULL}};
const Segment SGI66313[] = {{&S16897[117], 3}, {NULL}};
const Segment SGI66314[] = {{&S16897[83], 5}, {NULL}};
const Segment SGI66315[] = {{&S16901[44], 10}, {NULL}};
const Segment SGI66316[] = {{&S16901[44], 21}, {NULL}};
const Segment SGI66317[] = {{&S16901[30], 6}, {NULL}};
const Segment SGI66318[] = {{&S7185[44], 7}, {NULL}};
const Segment SGI66319[] = {{&S7185[44], 19}, {NULL}};
const Segment SGI66320[] = {{&S7185[30], 6}, {NULL}};
const Segment SGI66321[] = {{&S16901[82], 4}, {NULL}};
const Segment SGI66322[] = {{&S16901[82], 3}, {NULL}};
const Segment SGI66323[] = {{&S7185[31], 3}, {NULL}};
const Segment SGI66324[] = {{&S7185[77], 6}, {NULL}};
const Segment SGI66325[] = {{&S7186[44], 11}, {NULL}};
const Segment SGI66326[] = {{&S7186[44], 23}, {NULL}};
const Segment SGI66327[] = {{&S7186[30], 6}, {NULL}};
const Segment SGI66328[] = {{&S7186[31], 3}, {NULL}};
const Segment SGI66329[] = {{&S7186[81], 6}, {NULL}};
const Segment SGI66330[] = {{&S7187[44], 8}, {NULL}};
const Segment SGI66331[] = {{&S7187[44], 20}, {NULL}};
const Segment SGI66332[] = {{&S7187[30], 6}, {NULL}};
const Segment SGI66333[] = {{&S7187[31], 3}, {NULL}};
const Segment SGI66334[] = {{&S7187[78], 6}, {NULL}};
const Segment SGI66335[] = {{&S16906[115], 7}, {NULL}};
const Segment SGI66336[] = {{&S7188[38], 6}, {NULL}};
const Segment SGI66337[] = {{&S7188[31], 6}, {NULL}};
const Segment SGI66338[] = {{&S16906[115], 18}, {NULL}};
const Segment SGI66339[] = {{&S16906[101], 6}, {NULL}};
const Segment SGI66340[] = {{&S16906[181], 3}, {NULL}};
const Segment SGI66341[] = {{&S16906[65], 5}, {NULL}};
const Segment SGI66342[] = {{&S7189[44], 10}, {NULL}};
const Segment SGI66343[] = {{&S7189[44], 29}, {NULL}};
const Segment SGI66344[] = {{&S7189[30], 6}, {NULL}};
const Segment SGI66345[] = {{&S7189[31], 3}, {NULL}};
const Segment SGI66346[] = {{&S7189[87], 6}, {NULL}};
const Segment SGI66347[] = {{&S7190[44], 13}, {NULL}};
const Segment SGI66348[] = {{&S7190[44], 25}, {NULL}};
const Segment SGI66349[] = {{&S7190[30], 6}, {NULL}};
const Segment SGI66350[] = {{&S7190[31], 3}, {NULL}};
const Segment SGI66351[] = {{&S7190[83], 6}, {NULL}};
const Segment SGI66352[] = {{&S7192[44], 9}, {NULL}};
const Segment SGI66353[] = {{&S7192[44], 21}, {NULL}};
const Segment SGI66354[] = {{&S7192[30], 6}, {NULL}};
const Segment SGI66355[] = {{&S16912[44], 6}, {NULL}};
const Segment SGI66356[] = {{&S16912[44], 17}, {NULL}};
const Segment SGI66357[] = {{&S16912[30], 6}, {NULL}};
const Segment SGI66358[] = {{&S7192[82], 4}, {NULL}};
const Segment SGI66359[] = {{&S7192[82], 3}, {NULL}};
const Segment SGI66360[] = {{&S16912[109], 3}, {NULL}};
const Segment SGI66361[] = {{&S16912[75], 5}, {NULL}};
const Segment SGI66362[] = {{&S7194[44], 8}, {NULL}};
const Segment SGI66363[] = {{&S7194[44], 20}, {NULL}};
const Segment SGI66364[] = {{&S7194[30], 6}, {NULL}};
const Segment SGI66365[] = {{&S7194[31], 3}, {NULL}};
const Segment SGI66366[] = {{&S7194[78], 6}, {NULL}};
const Segment SGI66367[] = {{&S7197[44], 5}, {NULL}};
const Segment SGI66368[] = {{&S7197[44], 17}, {NULL}};
const Segment SGI66369[] = {{&S7197[30], 6}, {NULL}};
const Segment SGI66370[] = {{&S7197[31], 3}, {NULL}};
const Segment SGI66371[] = {{&S7197[75], 6}, {NULL}};
const Segment SGI66372[] = {{&S7198[44], 8}, {NULL}};
const Segment SGI66373[] = {{&S7198[44], 31}, {NULL}};
const Segment SGI66374[] = {{&S7198[30], 6}, {NULL}};
const Segment SGI66375[] = {{&S7198[31], 3}, {NULL}};
const Segment SGI66376[] = {{&S7198[89], 6}, {NULL}};
const Segment SGI66377[] = {{&S7200[44], 6}, {NULL}};
const Segment SGI66378[] = {{&S7200[44], 18}, {NULL}};
const Segment SGI66379[] = {{&S7200[30], 6}, {NULL}};
const Segment SGI66380[] = {{&S7200[79], 4}, {NULL}};
const Segment SGI66381[] = {{&S7200[79], 3}, {NULL}};
const Segment SGI66382[] = {{&S16926[44], 9}, {NULL}};
const Segment SGI66383[] = {{&S16926[44], 21}, {NULL}};
const Segment SGI66384[] = {{&S16926[30], 6}, {NULL}};
const Segment SGI66385[] = {{&S16926[131], 3}, {NULL}};
const Segment SGI66386[] = {{&S16926[97], 5}, {NULL}};
const Segment SGI66387[] = {{&S7203[44], 16}, {NULL}};
const Segment SGI66388[] = {{&S7203[44], 28}, {NULL}};
const Segment SGI66389[] = {{&S7203[30], 6}, {NULL}};
const Segment SGI66390[] = {{&S7203[31], 3}, {NULL}};
const Segment SGI66391[] = {{&S7203[86], 6}, {NULL}};
const Segment SGI66392[] = {{&S7204[44], 6}, {NULL}};
const Segment SGI66393[] = {{&S7204[44], 18}, {NULL}};
const Segment SGI66394[] = {{&S7204[30], 6}, {NULL}};
const Segment SGI66395[] = {{&S7204[31], 3}, {NULL}};
const Segment SGI66396[] = {{&S7204[76], 6}, {NULL}};
const Segment SGI66397[] = {{&S16937[44], 13}, {NULL}};
const Segment SGI66398[] = {{&S16937[44], 30}, {NULL}};
const Segment SGI66399[] = {{&S16937[30], 6}, {NULL}};
const Segment SGI66400[] = {{&S16937[142], 3}, {NULL}};
const Segment SGI66401[] = {{&S16937[108], 5}, {NULL}};
const Segment SGI66402[] = {{&S16940[44], 12}, {NULL}};
const Segment SGI66403[] = {{&S16940[44], 23}, {NULL}};
const Segment SGI66404[] = {{&S16940[30], 6}, {NULL}};
const Segment SGI66405[] = {{&S16940[84], 6}, {NULL}};
const Segment SGI66406[] = {{&S7220[44], 11}, {NULL}};
const Segment SGI66407[] = {{&S7220[44], 23}, {NULL}};
const Segment SGI66408[] = {{&S7220[30], 6}, {NULL}};
const Segment SGI66409[] = {{&S7220[31], 3}, {NULL}};
const Segment SGI66410[] = {{&S7220[81], 6}, {NULL}};
const Segment SGI66411[] = {{&S7221[44], 8}, {NULL}};
const Segment SGI66412[] = {{&S7221[44], 20}, {NULL}};
const Segment SGI66413[] = {{&S7221[30], 6}, {NULL}};
const Segment SGI66414[] = {{&S7221[31], 3}, {NULL}};
const Segment SGI66415[] = {{&S7221[78], 6}, {NULL}};
const Segment SGI66416[] = {{&S16950[44], 18}, {NULL}};
const Segment SGI66417[] = {{&S16950[44], 36}, {NULL}};
const Segment SGI66418[] = {{&S16950[30], 6}, {NULL}};
const Segment SGI66419[] = {{&S16950[128], 3}, {NULL}};
const Segment SGI66420[] = {{&S16950[94], 5}, {NULL}};
const Segment SGI66421[] = {{&S7237[44], 13}, {NULL}};
const Segment SGI66422[] = {{&S7237[44], 28}, {NULL}};
const Segment SGI66423[] = {{&S7237[30], 6}, {NULL}};
const Segment SGI66424[] = {{&S7237[31], 3}, {NULL}};
const Segment SGI66425[] = {{&S7237[86], 6}, {NULL}};
const Segment SGI66426[] = {{&S16967[44], 6}, {NULL}};
const Segment SGI66427[] = {{&S16967[44], 17}, {NULL}};
const Segment SGI66428[] = {{&S16967[30], 6}, {NULL}};
const Segment SGI66429[] = {{&S16967[109], 3}, {NULL}};
const Segment SGI66430[] = {{&S16967[75], 5}, {NULL}};
const Segment SGI66431[] = {{&S16969[44], 23}, {NULL}};
const Segment SGI66432[] = {{&S16969[44], 34}, {NULL}};
const Segment SGI66433[] = {{&S16969[30], 6}, {NULL}};
const Segment SGI66434[] = {{&S16969[126], 3}, {NULL}};
const Segment SGI66435[] = {{&S16969[92], 5}, {NULL}};
const Segment SGI66436[] = {{&S7246[44], 19}, {NULL}};
const Segment SGI66437[] = {{&S7246[44], 31}, {NULL}};
const Segment SGI66438[] = {{&S7246[30], 6}, {NULL}};
const Segment SGI66439[] = {{&S16972[44], 11}, {NULL}};
const Segment SGI66440[] = {{&S16972[44], 27}, {NULL}};
const Segment SGI66441[] = {{&S16972[30], 6}, {NULL}};
const Segment SGI66442[] = {{&S16972[119], 3}, {NULL}};
const Segment SGI66443[] = {{&S16972[85], 5}, {NULL}};
const Segment SGI66444[] = {{&S8193[44], 4}, {NULL}};
const Segment SGI66445[] = {{&S8193[44], 16}, {NULL}};
const Segment SGI66446[] = {{&S8193[30], 6}, {NULL}};
const Segment SGI66447[] = {{&S8193[31], 3}, {NULL}};
const Segment SGI66448[] = {{&S8193[74], 5}, {NULL}};
const Segment SGI66449[] = {{&S17018[44], 9}, {NULL}};
const Segment SGI66450[] = {{&S17018[44], 22}, {NULL}};
const Segment SGI66451[] = {{&S17018[30], 6}, {NULL}};
const Segment SGI66452[] = {{&S17018[114], 3}, {NULL}};
const Segment SGI66453[] = {{&S17018[80], 5}, {NULL}};
const Segment SGI66454[] = {{&S17020[44], 90}, {NULL}};
const Segment SGI66455[] = {{&S17020[44], 111}, {NULL}};
const Segment SGI66456[] = {{&S17020[30], 6}, {NULL}};
const Segment SGI66457[] = {{&S17020[99], 3}, {NULL}};
const Segment SGI66458[] = {{&S17020[128], 5}, {NULL}};
const Segment SGI66459[] = {{&S17027[44], 6}, {NULL}};
const Segment SGI66460[] = {{&S17027[44], 17}, {NULL}};
const Segment SGI66461[] = {{&S17027[30], 6}, {NULL}};
const Segment SGI66462[] = {{&S17027[109], 3}, {NULL}};
const Segment SGI66463[] = {{&S17027[75], 5}, {NULL}};
const Segment SGI66464[] = {{&S17051[44], 8}, {NULL}};
const Segment SGI66465[] = {{&S17051[44], 24}, {NULL}};
const Segment SGI66466[] = {{&S17051[30], 6}, {NULL}};
const Segment SGI66467[] = {{&S17051[116], 3}, {NULL}};
const Segment SGI66468[] = {{&S17051[82], 5}, {NULL}};
const Segment SGI66469[] = {{&S17052[44], 7}, {NULL}};
const Segment SGI66470[] = {{&S17052[44], 28}, {NULL}};
const Segment SGI66471[] = {{&S17052[30], 6}, {NULL}};
const Segment SGI66472[] = {{&S17052[120], 3}, {NULL}};
const Segment SGI66473[] = {{&S17052[86], 5}, {NULL}};
const Segment SGI66474[] = {{&S17061[44], 9}, {NULL}};
const Segment SGI66475[] = {{&S17061[44], 24}, {NULL}};
const Segment SGI66476[] = {{&S17061[30], 6}, {NULL}};
const Segment SGI66477[] = {{&S17061[85], 4}, {NULL}};
const Segment SGI66478[] = {{&S17061[85], 3}, {NULL}};
const Segment SGI66479[] = {{&S17064[44], 10}, {NULL}};
const Segment SGI66480[] = {{&S17064[44], 21}, {NULL}};
const Segment SGI66481[] = {{&S17064[30], 6}, {NULL}};
const Segment SGI66482[] = {{&S17064[113], 3}, {NULL}};
const Segment SGI66483[] = {{&S17064[79], 5}, {NULL}};
const Segment SGI66484[] = {{&S17068[44], 13}, {NULL}};
const Segment SGI66485[] = {{&S17068[44], 32}, {NULL}};
const Segment SGI66486[] = {{&S17068[30], 6}, {NULL}};
const Segment SGI66487[] = {{&S17068[142], 3}, {NULL}};
const Segment SGI66488[] = {{&S17068[108], 5}, {NULL}};
const Segment SGI66489[] = {{&S17097[44], 15}, {NULL}};
const Segment SGI66490[] = {{&S17097[44], 32}, {NULL}};
const Segment SGI66491[] = {{&S17097[30], 6}, {NULL}};
const Segment SGI66492[] = {{&S17097[55], 3}, {NULL}};
const Segment SGI66493[] = {{&S17097[90], 5}, {NULL}};
const Segment SGI66494[] = {{&S17111[44], 16}, {NULL}};
const Segment SGI66495[] = {{&S17111[44], 29}, {NULL}};
const Segment SGI66496[] = {{&S17111[30], 6}, {NULL}};
const Segment SGI66497[] = {{&S17111[121], 3}, {NULL}};
const Segment SGI66498[] = {{&S17111[87], 5}, {NULL}};
const Segment SGI66499[] = {{&S17114[44], 16}, {NULL}};
const Segment SGI66500[] = {{&S17114[44], 29}, {NULL}};
const Segment SGI66501[] = {{&S17114[30], 6}, {NULL}};
const Segment SGI66502[] = {{&S17114[121], 3}, {NULL}};
const Segment SGI66503[] = {{&S17114[87], 5}, {NULL}};
const Segment SGI66504[] = {{&S17122[44], 91}, {NULL}};
const Segment SGI66505[] = {{&S17122[44], 112}, {NULL}};
const Segment SGI66506[] = {{&S17122[30], 6}, {NULL}};
const Segment SGI66507[] = {{&S17122[203], 3}, {NULL}};
const Segment SGI66508[] = {{&S17122[129], 5}, {NULL}};
const Segment SGI66509[] = {{&S7246[31], 3}, {NULL}};
const Segment SGI66510[] = {{&S7246[89], 6}, {NULL}};
const Segment SGI66511[] = {{&S7250[57], 5}, {NULL}};
const Segment SGI66512[] = {{&S7250[40], 16}, {NULL}};
const Segment SGI66513[] = {{&S7250[28], 3}, {NULL}};
const Segment SGI66514[] = {{&S7250[85], 9}, {NULL}};
const Segment SGI66515[] = {{&S7253[44], 6}, {NULL}};
const Segment SGI66516[] = {{&S7253[44], 18}, {NULL}};
const Segment SGI66517[] = {{&S7253[30], 6}, {NULL}};
const Segment SGI66518[] = {{&S7253[31], 3}, {NULL}};
const Segment SGI66519[] = {{&S7253[76], 5}, {NULL}};
const Segment SGI66520[] = {{&S17134[44], 90}, {NULL}};
const Segment SGI66521[] = {{&S17134[44], 111}, {NULL}};
const Segment SGI66522[] = {{&S17134[30], 6}, {NULL}};
const Segment SGI66523[] = {{&S17134[202], 3}, {NULL}};
const Segment SGI66524[] = {{&S17134[128], 5}, {NULL}};
const Segment SGI66525[] = {{&S7257[44], 11}, {NULL}};
const Segment SGI66526[] = {{&S7257[44], 23}, {NULL}};
const Segment SGI66527[] = {{&S7257[30], 6}, {NULL}};
const Segment SGI66528[] = {{&S7257[31], 3}, {NULL}};
const Segment SGI66529[] = {{&S7257[81], 6}, {NULL}};
const Segment SGI66530[] = {{&S17137[44], 11}, {NULL}};
const Segment SGI66531[] = {{&S7259[44], 9}, {NULL}};
const Segment SGI66532[] = {{&S17137[44], 27}, {NULL}};
const Segment SGI66533[] = {{&S7259[44], 21}, {NULL}};
const Segment SGI66534[] = {{&S17137[30], 6}, {NULL}};
const Segment SGI66535[] = {{&S7259[30], 6}, {NULL}};
const Segment SGI66536[] = {{&S17137[119], 3}, {NULL}};
const Segment SGI66537[] = {{&S7259[31], 3}, {NULL}};
const Segment SGI66538[] = {{&S17137[85], 5}, {NULL}};
const Segment SGI66539[] = {{&S7259[79], 6}, {NULL}};
const Segment SGI66540[] = {{&S7260[44], 5}, {NULL}};
const Segment SGI66541[] = {{&S7260[44], 17}, {NULL}};
const Segment SGI66542[] = {{&S7260[30], 6}, {NULL}};
const Segment SGI66543[] = {{&S7260[31], 3}, {NULL}};
const Segment SGI66544[] = {{&S7260[75], 6}, {NULL}};
const Segment SGI66545[] = {{&S7269[44], 8}, {NULL}};
const Segment SGI66546[] = {{&S7269[44], 20}, {NULL}};
const Segment SGI66547[] = {{&S7269[30], 6}, {NULL}};
const Segment SGI66548[] = {{&S7269[31], 3}, {NULL}};
const Segment SGI66549[] = {{&S7269[78], 6}, {NULL}};
const Segment SGI66550[] = {{&S17150[44], 6}, {NULL}};
const Segment SGI66551[] = {{&S17150[44], 17}, {NULL}};
const Segment SGI66552[] = {{&S17150[30], 6}, {NULL}};
const Segment SGI66553[] = {{&S17150[109], 3}, {NULL}};
const Segment SGI66554[] = {{&S17150[75], 5}, {NULL}};
const Segment SGI66555[] = {{&S7276[44], 6}, {NULL}};
const Segment SGI66556[] = {{&S7276[44], 18}, {NULL}};
const Segment SGI66557[] = {{&S7276[30], 6}, {NULL}};
const Segment SGI66558[] = {{&S7276[31], 3}, {NULL}};
const Segment SGI66559[] = {{&S7276[76], 6}, {NULL}};
const Segment SGI66560[] = {{&S7285[44], 14}, {NULL}};
const Segment SGI66561[] = {{&S7285[44], 26}, {NULL}};
const Segment SGI66562[] = {{&S7285[30], 6}, {NULL}};
const Segment SGI66563[] = {{&S7285[31], 3}, {NULL}};
const Segment SGI66564[] = {{&S7285[84], 6}, {NULL}};
const Segment SGI66565[] = {{&S7290[44], 6}, {NULL}};
const Segment SGI66566[] = {{&S7290[44], 27}, {NULL}};
const Segment SGI66567[] = {{&S7290[30], 6}, {NULL}};
const Segment SGI66568[] = {{&S7290[31], 3}, {NULL}};
const Segment SGI66569[] = {{&S7290[85], 5}, {NULL}};
const Segment SGI66570[] = {{&S7292[44], 12}, {NULL}};
const Segment SGI66571[] = {{&S7292[44], 24}, {NULL}};
const Segment SGI66572[] = {{&S7292[30], 6}, {NULL}};
const Segment SGI66573[] = {{&S7292[31], 3}, {NULL}};
const Segment SGI66574[] = {{&S7292[82], 6}, {NULL}};
const Segment SGI66575[] = {{&S7295[44], 8}, {NULL}};
const Segment SGI66576[] = {{&S7295[44], 20}, {NULL}};
const Segment SGI66577[] = {{&S7295[30], 6}, {NULL}};
const Segment SGI66578[] = {{&S7295[31], 3}, {NULL}};
const Segment SGI66579[] = {{&S7295[78], 6}, {NULL}};
const Segment SGI66580[] = {{&S17183[44], 10}, {NULL}};
const Segment SGI66581[] = {{&S17183[44], 27}, {NULL}};
const Segment SGI66582[] = {{&S17183[30], 6}, {NULL}};
const Segment SGI66583[] = {{&S17183[119], 3}, {NULL}};
const Segment SGI66584[] = {{&S17183[85], 5}, {NULL}};
const Segment SGI66585[] = {{&S7310[44], 15}, {NULL}};
const Segment SGI66586[] = {{&S7310[44], 27}, {NULL}};
const Segment SGI66587[] = {{&S7310[30], 6}, {NULL}};
const Segment SGI66588[] = {{&S7310[31], 3}, {NULL}};
const Segment SGI66589[] = {{&S7310[85], 6}, {NULL}};
const Segment SGI66590[] = {{&S7311[44], 10}, {NULL}};
const Segment SGI66591[] = {{&S7311[44], 22}, {NULL}};
const Segment SGI66592[] = {{&S7311[30], 6}, {NULL}};
const Segment SGI66593[] = {{&S7311[31], 3}, {NULL}};
const Segment SGI66594[] = {{&S7311[80], 6}, {NULL}};
const Segment SGI66595[] = {{&S7312[44], 7}, {NULL}};
const Segment SGI66596[] = {{&S7312[44], 19}, {NULL}};
const Segment SGI66597[] = {{&S7312[30], 6}, {NULL}};
const Segment SGI66598[] = {{&S7312[31], 3}, {NULL}};
const Segment SGI66599[] = {{&S7312[77], 6}, {NULL}};
const Segment SGI66600[] = {{&S7313[44], 16}, {NULL}};
const Segment SGI66601[] = {{&S7313[44], 28}, {NULL}};
const Segment SGI66602[] = {{&S7313[30], 6}, {NULL}};
const Segment SGI66603[] = {{&S7313[31], 3}, {NULL}};
const Segment SGI66604[] = {{&S7313[86], 6}, {NULL}};
const Segment SGI66605[] = {{&S7315[44], 8}, {NULL}};
const Segment SGI66606[] = {{&S7315[44], 20}, {NULL}};
const Segment SGI66607[] = {{&S7315[30], 6}, {NULL}};
const Segment SGI66608[] = {{&S7315[31], 3}, {NULL}};
const Segment SGI66609[] = {{&S7315[78], 6}, {NULL}};
const Segment SGI66610[] = {{&S7319[44], 11}, {NULL}};
const Segment SGI66611[] = {{&S7319[44], 23}, {NULL}};
const Segment SGI66612[] = {{&S7319[30], 6}, {NULL}};
const Segment SGI66613[] = {{&S7319[31], 3}, {NULL}};
const Segment SGI66614[] = {{&S7319[81], 6}, {NULL}};
const Segment SGI66615[] = {{&S17207[44], 10}, {NULL}};
const Segment SGI66616[] = {{&S17207[44], 42}, {NULL}};
const Segment SGI66617[] = {{&S17207[30], 6}, {NULL}};
const Segment SGI66618[] = {{&S17207[134], 3}, {NULL}};
const Segment SGI66619[] = {{&S17207[100], 5}, {NULL}};
const Segment SGI66620[] = {{&S7321[38], 8}, {NULL}};
const Segment SGI66621[] = {{&S7321[31], 6}, {NULL}};
const Segment SGI66622[] = {{&S7322[44], 14}, {NULL}};
const Segment SGI66623[] = {{&S7322[44], 26}, {NULL}};
const Segment SGI66624[] = {{&S7322[30], 6}, {NULL}};
const Segment SGI66625[] = {{&S7322[31], 3}, {NULL}};
const Segment SGI66626[] = {{&S7322[84], 6}, {NULL}};
const Segment SGI66627[] = {{&S17209[44], 6}, {NULL}};
const Segment SGI66628[] = {{&S17209[44], 17}, {NULL}};
const Segment SGI66629[] = {{&S17209[30], 6}, {NULL}};
const Segment SGI66630[] = {{&S17209[78], 6}, {NULL}};
const Segment SGI66631[] = {{&S17211[44], 7}, {NULL}};
const Segment SGI66632[] = {{&S17211[44], 23}, {NULL}};
const Segment SGI66633[] = {{&S17211[30], 6}, {NULL}};
const Segment SGI66634[] = {{&S17211[115], 3}, {NULL}};
const Segment SGI66635[] = {{&S17211[81], 5}, {NULL}};
const Segment SGI66636[] = {{&S7333[44], 10}, {NULL}};
const Segment SGI66637[] = {{&S7333[44], 22}, {NULL}};
const Segment SGI66638[] = {{&S7333[30], 6}, {NULL}};
const Segment SGI66639[] = {{&S7333[31], 3}, {NULL}};
const Segment SGI66640[] = {{&S7333[80], 6}, {NULL}};
const Segment SGI66641[] = {{&S7336[44], 9}, {NULL}};
const Segment SGI66642[] = {{&S7336[44], 21}, {NULL}};
const Segment SGI66643[] = {{&S7336[30], 6}, {NULL}};
const Segment SGI66644[] = {{&S7336[31], 3}, {NULL}};
const Segment SGI66645[] = {{&S7336[79], 6}, {NULL}};
const Segment SGI66646[] = {{&S7337[44], 8}, {NULL}};
const Segment SGI66647[] = {{&S7337[44], 20}, {NULL}};
const Segment SGI66648[] = {{&S7337[30], 6}, {NULL}};
const Segment SGI66649[] = {{&S7337[31], 3}, {NULL}};
const Segment SGI66650[] = {{&S7337[78], 6}, {NULL}};
const Segment SGI66651[] = {{&S7340[44], 8}, {NULL}};
const Segment SGI66652[] = {{&S7340[44], 20}, {NULL}};
const Segment SGI66653[] = {{&S7340[30], 6}, {NULL}};
const Segment SGI66654[] = {{&S7340[31], 3}, {NULL}};
const Segment SGI66655[] = {{&S7340[78], 6}, {NULL}};
const Segment SGI66656[] = {{&S7342[44], 8}, {NULL}};
const Segment SGI66657[] = {{&S7342[44], 20}, {NULL}};
const Segment SGI66658[] = {{&S7342[30], 6}, {NULL}};
const Segment SGI66659[] = {{&S7342[31], 3}, {NULL}};
const Segment SGI66660[] = {{&S7342[78], 6}, {NULL}};
const Segment SGI66661[] = {{&S17232[44], 6}, {NULL}};
const Segment SGI66662[] = {{&S17232[44], 19}, {NULL}};
const Segment SGI66663[] = {{&S17232[30], 6}, {NULL}};
const Segment SGI66664[] = {{&S17232[111], 3}, {NULL}};
const Segment SGI66665[] = {{&S17232[77], 5}, {NULL}};
const Segment SGI66666[] = {{&S7343[44], 6}, {NULL}};
const Segment SGI66667[] = {{&S7343[44], 18}, {NULL}};
const Segment SGI66668[] = {{&S7343[30], 6}, {NULL}};
const Segment SGI66669[] = {{&S7343[31], 3}, {NULL}};
const Segment SGI66670[] = {{&S7343[76], 5}, {NULL}};
const Segment SGI66671[] = {{&S7344[44], 11}, {NULL}};
const Segment SGI66672[] = {{&S7344[44], 20}, {NULL}};
const Segment SGI66673[] = {{&S7344[30], 6}, {NULL}};
const Segment SGI66674[] = {{&S7344[31], 3}, {NULL}};
const Segment SGI66675[] = {{&S7344[78], 6}, {NULL}};
const Segment SGI66676[] = {{&S7348[44], 5}, {NULL}};
const Segment SGI66677[] = {{&S7348[44], 17}, {NULL}};
const Segment SGI66678[] = {{&S7348[30], 6}, {NULL}};
const Segment SGI66679[] = {{&S7348[31], 3}, {NULL}};
const Segment SGI66680[] = {{&S7348[75], 6}, {NULL}};
const Segment SGI66681[] = {{&S7350[44], 9}, {NULL}};
const Segment SGI66682[] = {{&S7350[44], 21}, {NULL}};
const Segment SGI66683[] = {{&S7350[30], 6}, {NULL}};
const Segment SGI66684[] = {{&S7350[31], 3}, {NULL}};
const Segment SGI66685[] = {{&S7350[79], 6}, {NULL}};
const Segment SGI66686[] = {{&S7356[44], 6}, {NULL}};
const Segment SGI66687[] = {{&S7356[44], 18}, {NULL}};
const Segment SGI66688[] = {{&S7356[30], 6}, {NULL}};
const Segment SGI66689[] = {{&S7356[79], 6}, {NULL}};
const Segment SGI66690[] = {{&S17253[44], 9}, {NULL}};
const Segment SGI66691[] = {{&S17253[44], 26}, {NULL}};
const Segment SGI66692[] = {{&S17253[30], 6}, {NULL}};
const Segment SGI66693[] = {{&S17253[118], 3}, {NULL}};
const Segment SGI66694[] = {{&S17253[84], 5}, {NULL}};
const Segment SGI66695[] = {{&S17256[44], 16}, {NULL}};
const Segment SGI66696[] = {{&S17256[30], 6}, {NULL}};
const Segment SGI66697[] = {{&S7374[44], 12}, {NULL}};
const Segment SGI66698[] = {{&S7374[44], 24}, {NULL}};
const Segment SGI66699[] = {{&S7374[30], 6}, {NULL}};
const Segment SGI66700[] = {{&S7374[31], 3}, {NULL}};
const Segment SGI66701[] = {{&S7374[82], 6}, {NULL}};
const Segment SGI66702[] = {{&S7376[44], 11}, {NULL}};
const Segment SGI66703[] = {{&S7376[44], 23}, {NULL}};
const Segment SGI66704[] = {{&S7376[30], 6}, {NULL}};
const Segment SGI66705[] = {{&S7376[31], 3}, {NULL}};
const Segment SGI66706[] = {{&S7376[81], 6}, {NULL}};
const Segment SGI66707[] = {{&S7378[44], 12}, {NULL}};
const Segment SGI66708[] = {{&S7378[44], 24}, {NULL}};
const Segment SGI66709[] = {{&S7378[30], 6}, {NULL}};
const Segment SGI66710[] = {{&S7378[31], 3}, {NULL}};
const Segment SGI66711[] = {{&S7378[82], 6}, {NULL}};
const Segment SGI66712[] = {{&S17274[44], 7}, {NULL}};
const Segment SGI66713[] = {{&S17274[44], 18}, {NULL}};
const Segment SGI66714[] = {{&S17274[30], 6}, {NULL}};
const Segment SGI66715[] = {{&S17274[110], 3}, {NULL}};
const Segment SGI66716[] = {{&S17274[76], 5}, {NULL}};
const Segment SGI66717[] = {{&S7379[44], 14}, {NULL}};
const Segment SGI66718[] = {{&S7379[44], 26}, {NULL}};
const Segment SGI66719[] = {{&S7379[30], 6}, {NULL}};
const Segment SGI66720[] = {{&S7379[31], 3}, {NULL}};
const Segment SGI66721[] = {{&S7379[84], 6}, {NULL}};
const Segment SGI66722[] = {{&S7380[44], 7}, {NULL}};
const Segment SGI66723[] = {{&S7380[44], 19}, {NULL}};
const Segment SGI66724[] = {{&S7380[30], 6}, {NULL}};
const Segment SGI66725[] = {{&S7380[31], 3}, {NULL}};
const Segment SGI66726[] = {{&S7380[77], 6}, {NULL}};
const Segment SGI66727[] = {{&S17277[44], 10}, {NULL}};
const Segment SGI66728[] = {{&S17277[44], 29}, {NULL}};
const Segment SGI66729[] = {{&S17277[30], 6}, {NULL}};
const Segment SGI66730[] = {{&S17277[121], 3}, {NULL}};
const Segment SGI66731[] = {{&S17277[87], 5}, {NULL}};
const Segment SGI66732[] = {{&S7382[44], 6}, {NULL}};
const Segment SGI66733[] = {{&S7382[44], 18}, {NULL}};
const Segment SGI66734[] = {{&S7382[30], 6}, {NULL}};
const Segment SGI66735[] = {{&S7382[31], 3}, {NULL}};
const Segment SGI66736[] = {{&S7382[76], 6}, {NULL}};
const Segment SGI66737[] = {{&S7383[44], 9}, {NULL}};
const Segment SGI66738[] = {{&S7383[44], 21}, {NULL}};
const Segment SGI66739[] = {{&S7383[30], 6}, {NULL}};
const Segment SGI66740[] = {{&S7383[31], 3}, {NULL}};
const Segment SGI66741[] = {{&S7383[79], 6}, {NULL}};
const Segment SGI66742[] = {{&S7384[44], 8}, {NULL}};
const Segment SGI66743[] = {{&S7384[44], 20}, {NULL}};
const Segment SGI66744[] = {{&S7384[30], 6}, {NULL}};
const Segment SGI66745[] = {{&S7384[31], 3}, {NULL}};
const Segment SGI66746[] = {{&S7384[78], 6}, {NULL}};
const Segment SGI66747[] = {{&S7385[44], 12}, {NULL}};
const Segment SGI66748[] = {{&S7385[44], 24}, {NULL}};
const Segment SGI66749[] = {{&S7385[30], 6}, {NULL}};
const Segment SGI66750[] = {{&S7385[31], 3}, {NULL}};
const Segment SGI66751[] = {{&S7385[82], 6}, {NULL}};
const Segment SGI66752[] = {{&S7386[44], 10}, {NULL}};
const Segment SGI66753[] = {{&S7386[44], 22}, {NULL}};
const Segment SGI66754[] = {{&S7386[30], 6}, {NULL}};
const Segment SGI66755[] = {{&S7386[31], 3}, {NULL}};
const Segment SGI66756[] = {{&S7386[80], 6}, {NULL}};
const Segment SGI66757[] = {{&S7387[44], 11}, {NULL}};
const Segment SGI66758[] = {{&S7387[44], 23}, {NULL}};
const Segment SGI66759[] = {{&S7387[30], 6}, {NULL}};
const Segment SGI66760[] = {{&S17288[44], 18}, {NULL}};
const Segment SGI66761[] = {{&S17288[44], 50}, {NULL}};
const Segment SGI66762[] = {{&S17288[30], 6}, {NULL}};
const Segment SGI66763[] = {{&S7387[84], 6}, {NULL}};
const Segment SGI66764[] = {{&S17288[142], 3}, {NULL}};
const Segment SGI66765[] = {{&S17288[108], 5}, {NULL}};
const Segment SGI66766[] = {{&S7388[44], 9}, {NULL}};
const Segment SGI66767[] = {{&S7388[44], 21}, {NULL}};
const Segment SGI66768[] = {{&S7388[30], 6}, {NULL}};
const Segment SGI66769[] = {{&S7388[31], 3}, {NULL}};
const Segment SGI66770[] = {{&S7388[79], 6}, {NULL}};
const Segment SGI66771[] = {{&S17290[44], 13}, {NULL}};
const Segment SGI66772[] = {{&S17290[44], 25}, {NULL}};
const Segment SGI66773[] = {{&S17290[30], 6}, {NULL}};
const Segment SGI66774[] = {{&S17290[117], 3}, {NULL}};
const Segment SGI66775[] = {{&S17290[83], 5}, {NULL}};
const Segment SGI66776[] = {{&S7390[44], 9}, {NULL}};
const Segment SGI66777[] = {{&S7390[44], 21}, {NULL}};
const Segment SGI66778[] = {{&S7390[30], 6}, {NULL}};
const Segment SGI66779[] = {{&S7390[31], 3}, {NULL}};
const Segment SGI66780[] = {{&S7390[79], 6}, {NULL}};
const Segment SGI66781[] = {{&S7391[44], 6}, {NULL}};
const Segment SGI66782[] = {{&S7391[44], 18}, {NULL}};
const Segment SGI66783[] = {{&S7391[30], 6}, {NULL}};
const Segment SGI66784[] = {{&S7391[79], 6}, {NULL}};
const Segment SGI66785[] = {{&S7392[44], 6}, {NULL}};
const Segment SGI66786[] = {{&S23563[44], 7}, {NULL}};
const Segment SGI66787[] = {{&S7392[44], 18}, {NULL}};
const Segment SGI66788[] = {{&S23563[44], 31}, {NULL}};
const Segment SGI66789[] = {{&S7392[30], 6}, {NULL}};
const Segment SGI66790[] = {{&S23563[30], 6}, {NULL}};
const Segment SGI66791[] = {{&S7392[31], 3}, {NULL}};
const Segment SGI66792[] = {{&S23563[123], 3}, {NULL}};
const Segment SGI66793[] = {{&S7392[76], 6}, {NULL}};
const Segment SGI66794[] = {{&S23563[89], 5}, {NULL}};
const Segment SGI66795[] = {{&S17296[44], 8}, {NULL}};
const Segment SGI66796[] = {{&S17296[44], 19}, {NULL}};
const Segment SGI66797[] = {{&S17296[30], 6}, {NULL}};
const Segment SGI66798[] = {{&S17296[111], 3}, {NULL}};
const Segment SGI66799[] = {{&S17296[77], 5}, {NULL}};
const Segment SGI66800[] = {{&S23564[44], 7}, {NULL}};
const Segment SGI66801[] = {{&S23564[44], 29}, {NULL}};
const Segment SGI66802[] = {{&S23564[30], 6}, {NULL}};
const Segment SGI66803[] = {{&S23564[121], 3}, {NULL}};
const Segment SGI66804[] = {{&S23564[87], 5}, {NULL}};
const Segment SGI66805[] = {{&S7395[44], 14}, {NULL}};
const Segment SGI66806[] = {{&S7395[44], 26}, {NULL}};
const Segment SGI66807[] = {{&S7395[30], 6}, {NULL}};
const Segment SGI66808[] = {{&S7395[31], 3}, {NULL}};
const Segment SGI66809[] = {{&S7395[84], 6}, {NULL}};
const Segment SGI66810[] = {{&S7396[44], 5}, {NULL}};
const Segment SGI66811[] = {{&S7396[44], 17}, {NULL}};
const Segment SGI66812[] = {{&S7396[30], 6}, {NULL}};
const Segment SGI66813[] = {{&S23567[44], 7}, {NULL}};
const Segment SGI66814[] = {{&S23567[44], 20}, {NULL}};
const Segment SGI66815[] = {{&S7396[31], 3}, {NULL}};
const Segment SGI66816[] = {{&S23567[30], 6}, {NULL}};
const Segment SGI66817[] = {{&S7396[75], 6}, {NULL}};
const Segment SGI66818[] = {{&S23567[112], 3}, {NULL}};
const Segment SGI66819[] = {{&S23567[78], 5}, {NULL}};
const Segment SGI66820[] = {{&S7399[44], 14}, {NULL}};
const Segment SGI66821[] = {{&S7399[44], 26}, {NULL}};
const Segment SGI66822[] = {{&S7399[30], 6}, {NULL}};
const Segment SGI66823[] = {{&S23570[44], 7}, {NULL}};
const Segment SGI66824[] = {{&S7399[31], 3}, {NULL}};
const Segment SGI66825[] = {{&S23570[44], 29}, {NULL}};
const Segment SGI66826[] = {{&S7399[84], 6}, {NULL}};
const Segment SGI66827[] = {{&S23570[30], 6}, {NULL}};
const Segment SGI66828[] = {{&S23570[121], 3}, {NULL}};
const Segment SGI66829[] = {{&S23570[87], 5}, {NULL}};
const Segment SGI66830[] = {{&S7400[44], 5}, {NULL}};
const Segment SGI66831[] = {{&S7400[44], 17}, {NULL}};
const Segment SGI66832[] = {{&S7400[30], 6}, {NULL}};
const Segment SGI66833[] = {{&S7400[31], 3}, {NULL}};
const Segment SGI66834[] = {{&S7400[75], 6}, {NULL}};
const Segment SGI66835[] = {{&S23571[44], 7}, {NULL}};
const Segment SGI66836[] = {{&S23571[44], 29}, {NULL}};
const Segment SGI66837[] = {{&S23571[30], 6}, {NULL}};
const Segment SGI66838[] = {{&S23571[121], 3}, {NULL}};
const Segment SGI66839[] = {{&S23571[87], 5}, {NULL}};
const Segment SGI66840[] = {{&S7401[44], 10}, {NULL}};
const Segment SGI66841[] = {{&S7401[44], 22}, {NULL}};
const Segment SGI66842[] = {{&S7401[30], 6}, {NULL}};
const Segment SGI66843[] = {{&S7401[31], 3}, {NULL}};
const Segment SGI66844[] = {{&S7401[80], 6}, {NULL}};
const Segment SGI66845[] = {{&S23572[38], 7}, {NULL}};
const Segment SGI66846[] = {{&S23572[31], 6}, {NULL}};
const Segment SGI66847[] = {{&S7402[44], 13}, {NULL}};
const Segment SGI66848[] = {{&S7402[44], 25}, {NULL}};
const Segment SGI66849[] = {{&S7402[30], 6}, {NULL}};
const Segment SGI66850[] = {{&S7402[31], 3}, {NULL}};
const Segment SGI66851[] = {{&S7402[83], 6}, {NULL}};
const Segment SGI66852[] = {{&S23574[44], 7}, {NULL}};
const Segment SGI66853[] = {{&S23574[44], 29}, {NULL}};
const Segment SGI66854[] = {{&S23574[30], 6}, {NULL}};
const Segment SGI66855[] = {{&S23574[121], 3}, {NULL}};
const Segment SGI66856[] = {{&S7403[44], 10}, {NULL}};
const Segment SGI66857[] = {{&S23574[87], 5}, {NULL}};
const Segment SGI66858[] = {{&S7403[44], 22}, {NULL}};
const Segment SGI66859[] = {{&S7403[30], 6}, {NULL}};
const Segment SGI66860[] = {{&S7403[31], 3}, {NULL}};
const Segment SGI66861[] = {{&S7403[80], 6}, {NULL}};
const Segment SGI66862[] = {{&S7404[44], 10}, {NULL}};
const Segment SGI66863[] = {{&S7404[44], 22}, {NULL}};
const Segment SGI66864[] = {{&S7404[30], 6}, {NULL}};
const Segment SGI66865[] = {{&S7404[31], 3}, {NULL}};
const Segment SGI66866[] = {{&S7404[80], 6}, {NULL}};
const Segment SGI66867[] = {{&S23577[44], 7}, {NULL}};
const Segment SGI66868[] = {{&S23577[44], 20}, {NULL}};
const Segment SGI66869[] = {{&S23577[30], 6}, {NULL}};
const Segment SGI66870[] = {{&S23577[112], 3}, {NULL}};
const Segment SGI66871[] = {{&S23577[78], 5}, {NULL}};
const Segment SGI66872[] = {{&S7407[44], 9}, {NULL}};
const Segment SGI66873[] = {{&S7407[44], 31}, {NULL}};
const Segment SGI66874[] = {{&S7407[30], 6}, {NULL}};
const Segment SGI66875[] = {{&S7407[31], 3}, {NULL}};
const Segment SGI66876[] = {{&S7407[89], 6}, {NULL}};
const Segment SGI66877[] = {{&S7409[44], 13}, {NULL}};
const Segment SGI66878[] = {{&S7409[44], 25}, {NULL}};
const Segment SGI66879[] = {{&S7409[30], 6}, {NULL}};
const Segment SGI66880[] = {{&S23584[44], 7}, {NULL}};
const Segment SGI66881[] = {{&S23584[44], 31}, {NULL}};
const Segment SGI66882[] = {{&S23584[30], 6}, {NULL}};
const Segment SGI66883[] = {{&S23584[123], 3}, {NULL}};
const Segment SGI66884[] = {{&S23584[89], 5}, {NULL}};
const Segment SGI66885[] = {{&S23588[42], 7}, {NULL}};
const Segment SGI66886[] = {{&S23588[30], 6}, {NULL}};
const Segment SGI66887[] = {{&S23588[110], 3}, {NULL}};
const Segment SGI66888[] = {{&S23588[76], 5}, {NULL}};
const Segment SGI66889[] = {{&S23591[44], 7}, {NULL}};
const Segment SGI66890[] = {{&S23591[44], 20}, {NULL}};
const Segment SGI66891[] = {{&S23591[30], 6}, {NULL}};
const Segment SGI66892[] = {{&S23591[78], 6}, {NULL}};
const Segment SGI66893[] = {{&S23592[44], 7}, {NULL}};
const Segment SGI66894[] = {{&S23592[44], 31}, {NULL}};
const Segment SGI66895[] = {{&S23592[30], 6}, {NULL}};
const Segment SGI66896[] = {{&S23592[123], 3}, {NULL}};
const Segment SGI66897[] = {{&S23592[89], 5}, {NULL}};
const Segment SGI66898[] = {{&S23594[44], 7}, {NULL}};
const Segment SGI66899[] = {{&S23594[44], 20}, {NULL}};
const Segment SGI66900[] = {{&S23594[30], 6}, {NULL}};
const Segment SGI66901[] = {{&S23594[112], 3}, {NULL}};
const Segment SGI66902[] = {{&S23594[78], 5}, {NULL}};
const Segment SGI66903[] = {{&S7409[86], 6}, {NULL}};
const Segment SGI66904[] = {{&S7412[44], 9}, {NULL}};
const Segment SGI66905[] = {{&S7412[44], 21}, {NULL}};
const Segment SGI66906[] = {{&S7412[30], 6}, {NULL}};
const Segment SGI66907[] = {{&S7412[31], 3}, {NULL}};
const Segment SGI66908[] = {{&S7412[79], 6}, {NULL}};
const Segment SGI66909[] = {{&S7413[44], 14}, {NULL}};
const Segment SGI66910[] = {{&S7413[44], 26}, {NULL}};
const Segment SGI66911[] = {{&S7413[30], 6}, {NULL}};
const Segment SGI66912[] = {{&S7413[31], 3}, {NULL}};
const Segment SGI66913[] = {{&S7413[84], 6}, {NULL}};
const Segment SGI66914[] = {{&S7414[44], 12}, {NULL}};
const Segment SGI66915[] = {{&S7414[44], 24}, {NULL}};
const Segment SGI66916[] = {{&S7414[30], 6}, {NULL}};
const Segment SGI66917[] = {{&S7414[31], 3}, {NULL}};
const Segment SGI66918[] = {{&S7414[82], 6}, {NULL}};
const Segment SGI66919[] = {{&S7416[44], 7}, {NULL}};
const Segment SGI66920[] = {{&S7416[44], 19}, {NULL}};
const Segment SGI66921[] = {{&S7416[30], 6}, {NULL}};
const Segment SGI66922[] = {{&S7416[31], 3}, {NULL}};
const Segment SGI66923[] = {{&S7416[77], 6}, {NULL}};
const Segment SGI66924[] = {{&S7424[44], 15}, {NULL}};
const Segment SGI66925[] = {{&S7424[44], 27}, {NULL}};
const Segment SGI66926[] = {{&S7424[30], 6}, {NULL}};
const Segment SGI66927[] = {{&S7424[31], 3}, {NULL}};
const Segment SGI66928[] = {{&S7424[85], 6}, {NULL}};
const Segment SGI66929[] = {{&S7425[44], 6}, {NULL}};
const Segment SGI66930[] = {{&S7425[44], 16}, {NULL}};
const Segment SGI66931[] = {{&S7425[30], 6}, {NULL}};
const Segment SGI66932[] = {{&S7425[31], 3}, {NULL}};
const Segment SGI66933[] = {{&S7425[74], 6}, {NULL}};
const Segment SGI66934[] = {{&S7426[44], 11}, {NULL}};
const Segment SGI66935[] = {{&S7426[44], 23}, {NULL}};
const Segment SGI66936[] = {{&S7426[30], 6}, {NULL}};
const Segment SGI66937[] = {{&S7426[31], 3}, {NULL}};
const Segment SGI66938[] = {{&S7426[81], 6}, {NULL}};
const Segment SGI66939[] = {{&S7427[44], 15}, {NULL}};
const Segment SGI66940[] = {{&S7427[44], 27}, {NULL}};
const Segment SGI66941[] = {{&S7427[30], 6}, {NULL}};
const Segment SGI66942[] = {{&S7427[31], 3}, {NULL}};
const Segment SGI66943[] = {{&S7427[85], 6}, {NULL}};
const Segment SGI66944[] = {{&S7434[44], 21}, {NULL}};
const Segment SGI66945[] = {{&S7434[44], 47}, {NULL}};
const Segment SGI66946[] = {{&S7434[30], 6}, {NULL}};
const Segment SGI66947[] = {{&S7434[31], 3}, {NULL}};
const Segment SGI66948[] = {{&S7434[105], 6}, {NULL}};
const Segment SGI66949[] = {{&S7435[44], 8}, {NULL}};
const Segment SGI66950[] = {{&S7435[44], 20}, {NULL}};
const Segment SGI66951[] = {{&S7435[30], 6}, {NULL}};
const Segment SGI66952[] = {{&S7435[31], 3}, {NULL}};
const Segment SGI66953[] = {{&S7435[78], 6}, {NULL}};
const Segment SGI66954[] = {{&S7436[44], 14}, {NULL}};
const Segment SGI66955[] = {{&S7436[44], 26}, {NULL}};
const Segment SGI66956[] = {{&S7436[30], 6}, {NULL}};
const Segment SGI66957[] = {{&S7436[31], 3}, {NULL}};
const Segment SGI66958[] = {{&S7436[84], 6}, {NULL}};
const Segment SGI66959[] = {{&S7437[44], 9}, {NULL}};
const Segment SGI66960[] = {{&S7437[44], 21}, {NULL}};
const Segment SGI66961[] = {{&S7437[30], 6}, {NULL}};
const Segment SGI66962[] = {{&S7437[31], 3}, {NULL}};
const Segment SGI66963[] = {{&S7437[79], 6}, {NULL}};
const Segment SGI66964[] = {{&S7438[44], 9}, {NULL}};
const Segment SGI66965[] = {{&S7438[44], 21}, {NULL}};
const Segment SGI66966[] = {{&S7438[30], 6}, {NULL}};
const Segment SGI66967[] = {{&S7438[31], 3}, {NULL}};
const Segment SGI66968[] = {{&S7438[79], 6}, {NULL}};
const Segment SGI66969[] = {{&S7439[44], 12}, {NULL}};
const Segment SGI66970[] = {{&S7439[44], 24}, {NULL}};
const Segment SGI66971[] = {{&S7439[30], 6}, {NULL}};
const Segment SGI66972[] = {{&S7439[31], 3}, {NULL}};
const Segment SGI66973[] = {{&S7439[82], 6}, {NULL}};
const Segment SGI66974[] = {{&S7445[44], 7}, {NULL}};
const Segment SGI66975[] = {{&S7445[44], 19}, {NULL}};
const Segment SGI66976[] = {{&S7445[30], 6}, {NULL}};
const Segment SGI66977[] = {{&S7445[31], 3}, {NULL}};
const Segment SGI66978[] = {{&S7445[77], 6}, {NULL}};
const Segment SGI66979[] = {{&S7448[44], 9}, {NULL}};
const Segment SGI66980[] = {{&S7448[44], 21}, {NULL}};
const Segment SGI66981[] = {{&S7448[30], 6}, {NULL}};
const Segment SGI66982[] = {{&S7448[31], 3}, {NULL}};
const Segment SGI66983[] = {{&S7448[79], 6}, {NULL}};
const Segment SGI66984[] = {{&S7458[44], 14}, {NULL}};
const Segment SGI66985[] = {{&S7458[44], 26}, {NULL}};
const Segment SGI66986[] = {{&S7458[30], 6}, {NULL}};
const Segment SGI66987[] = {{&S7458[31], 3}, {NULL}};
const Segment SGI66988[] = {{&S7458[84], 6}, {NULL}};
const Segment SGI66989[] = {{&S7465[44], 10}, {NULL}};
const Segment SGI66990[] = {{&S7465[44], 22}, {NULL}};
const Segment SGI66991[] = {{&S7465[30], 6}, {NULL}};
const Segment SGI66992[] = {{&S7465[31], 3}, {NULL}};
const Segment SGI66993[] = {{&S7465[80], 6}, {NULL}};
const Segment SGI66994[] = {{&S7477[44], 11}, {NULL}};
const Segment SGI66995[] = {{&S7477[44], 23}, {NULL}};
const Segment SGI66996[] = {{&S7477[30], 6}, {NULL}};
const Segment SGI66997[] = {{&S7477[31], 3}, {NULL}};
const Segment SGI66998[] = {{&S7477[81], 6}, {NULL}};
const Segment SGI66999[] = {{&S7486[44], 10}, {NULL}};
const Segment SGI67000[] = {{&S7486[44], 22}, {NULL}};
const Segment SGI67001[] = {{&S7486[30], 6}, {NULL}};
const Segment SGI67002[] = {{&S7486[31], 3}, {NULL}};
const Segment SGI67003[] = {{&S7486[80], 6}, {NULL}};
const Segment SGI67004[] = {{&S7487[44], 6}, {NULL}};
const Segment SGI67005[] = {{&S7487[44], 18}, {NULL}};
const Segment SGI67006[] = {{&S7487[30], 6}, {NULL}};
const Segment SGI67007[] = {{&S7487[31], 3}, {NULL}};
const Segment SGI67008[] = {{&S7487[76], 6}, {NULL}};
const Segment SGI67009[] = {{&S7504[44], 10}, {NULL}};
const Segment SGI67010[] = {{&S7504[44], 22}, {NULL}};
const Segment SGI67011[] = {{&S7504[30], 6}, {NULL}};
const Segment SGI67012[] = {{&S7504[31], 3}, {NULL}};
const Segment SGI67013[] = {{&S7504[80], 6}, {NULL}};
const Segment SGI67014[] = {{&S7505[44], 9}, {NULL}};
const Segment SGI67015[] = {{&S7505[44], 21}, {NULL}};
const Segment SGI67016[] = {{&S7505[30], 6}, {NULL}};
const Segment SGI67017[] = {{&S7505[31], 3}, {NULL}};
const Segment SGI67018[] = {{&S7505[79], 6}, {NULL}};
const Segment SGI67019[] = {{&S7507[44], 11}, {NULL}};
const Segment SGI67020[] = {{&S7507[44], 36}, {NULL}};
const Segment SGI67021[] = {{&S7507[30], 6}, {NULL}};
const Segment SGI67022[] = {{&S7507[31], 3}, {NULL}};
const Segment SGI67023[] = {{&S7507[94], 6}, {NULL}};
const Segment SGI67024[] = {{&S7511[44], 11}, {NULL}};
const Segment SGI67025[] = {{&S7511[44], 26}, {NULL}};
const Segment SGI67026[] = {{&S7511[30], 6}, {NULL}};
const Segment SGI67027[] = {{&S7511[31], 3}, {NULL}};
const Segment SGI67028[] = {{&S7511[84], 6}, {NULL}};
const Segment SGI67029[] = {{&S7516[44], 13}, {NULL}};
const Segment SGI67030[] = {{&S7516[44], 25}, {NULL}};
const Segment SGI67031[] = {{&S7516[30], 6}, {NULL}};
const Segment SGI67032[] = {{&S7516[83], 6}, {NULL}};
const Segment SGI67033[] = {{&S7536[44], 13}, {NULL}};
const Segment SGI67034[] = {{&S7536[30], 6}, {NULL}};
const Segment SGI67035[] = {{&S7536[31], 3}, {NULL}};
const Segment SGI67036[] = {{&S7536[71], 6}, {NULL}};
const Segment SGI67037[] = {{&S7543[44], 7}, {NULL}};
const Segment SGI67038[] = {{&S7543[44], 26}, {NULL}};
const Segment SGI67039[] = {{&S7543[30], 6}, {NULL}};
const Segment SGI67040[] = {{&S7543[31], 3}, {NULL}};
const Segment SGI67041[] = {{&S7543[84], 6}, {NULL}};
const Segment SGI67042[] = {{&S7548[44], 9}, {NULL}};
const Segment SGI67043[] = {{&S7548[44], 22}, {NULL}};
const Segment SGI67044[] = {{&S7548[30], 6}, {NULL}};
const Segment SGI67045[] = {{&S7548[31], 3}, {NULL}};
const Segment SGI67046[] = {{&S7548[80], 6}, {NULL}};
const Segment SGI67047[] = {{&S7550[44], 13}, {NULL}};
const Segment SGI67048[] = {{&S7550[30], 6}, {NULL}};
const Segment SGI67049[] = {{&S7550[31], 3}, {NULL}};
const Segment SGI67050[] = {{&S7550[71], 6}, {NULL}};
const Segment SGI67051[] = {{&S7555[44], 7}, {NULL}};
const Segment SGI67052[] = {{&S7555[44], 19}, {NULL}};
const Segment SGI67053[] = {{&S7555[30], 6}, {NULL}};
const Segment SGI67054[] = {{&S7555[31], 3}, {NULL}};
const Segment SGI67055[] = {{&S7555[77], 5}, {NULL}};
const Segment SGI67056[] = {{&S7558[44], 10}, {NULL}};
const Segment SGI67057[] = {{&S7558[44], 22}, {NULL}};
const Segment SGI67058[] = {{&S7558[30], 6}, {NULL}};
const Segment SGI67059[] = {{&S7558[31], 3}, {NULL}};
const Segment SGI67060[] = {{&S7558[80], 6}, {NULL}};
const Segment SGI67061[] = {{&S7566[44], 10}, {NULL}};
const Segment SGI67062[] = {{&S7566[30], 6}, {NULL}};
const Segment SGI67063[] = {{&S7566[31], 3}, {NULL}};
const Segment SGI67064[] = {{&S7566[82], 6}, {NULL}};
const Segment SGI67065[] = {{&S7568[44], 14}, {NULL}};
const Segment SGI67066[] = {{&S7568[44], 26}, {NULL}};
const Segment SGI67067[] = {{&S7568[30], 6}, {NULL}};
const Segment SGI67068[] = {{&S7568[31], 3}, {NULL}};
const Segment SGI67069[] = {{&S7568[84], 6}, {NULL}};
const Segment SGI67070[] = {{&S7570[44], 21}, {NULL}};
const Segment SGI67071[] = {{&S7570[44], 33}, {NULL}};
const Segment SGI67072[] = {{&S7570[30], 6}, {NULL}};
const Segment SGI67073[] = {{&S7570[31], 3}, {NULL}};
const Segment SGI67074[] = {{&S7570[91], 6}, {NULL}};
const Segment SGI67075[] = {{&S7572[44], 11}, {NULL}};
const Segment SGI67076[] = {{&S7572[44], 22}, {NULL}};
const Segment SGI67077[] = {{&S7572[30], 6}, {NULL}};
const Segment SGI67078[] = {{&S7572[31], 3}, {NULL}};
const Segment SGI67079[] = {{&S7572[80], 6}, {NULL}};
const Segment SGI67080[] = {{&S7600[44], 4}, {NULL}};
const Segment SGI67081[] = {{&S7600[44], 25}, {NULL}};
const Segment SGI67082[] = {{&S7600[30], 6}, {NULL}};
const Segment SGI67083[] = {{&S7600[31], 3}, {NULL}};
const Segment SGI67084[] = {{&S7600[83], 5}, {NULL}};
const Segment SGI67085[] = {{&S7606[38], 6}, {NULL}};
const Segment SGI67086[] = {{&S7606[31], 6}, {NULL}};
const Segment SGI67087[] = {{&S7608[44], 10}, {NULL}};
const Segment SGI67088[] = {{&S7608[44], 22}, {NULL}};
const Segment SGI67089[] = {{&S7608[30], 6}, {NULL}};
const Segment SGI67090[] = {{&S7608[31], 3}, {NULL}};
const Segment SGI67091[] = {{&S7608[80], 6}, {NULL}};
const Segment SGI67092[] = {{&S7618[44], 13}, {NULL}};
const Segment SGI67093[] = {{&S7618[44], 25}, {NULL}};
const Segment SGI67094[] = {{&S7618[30], 6}, {NULL}};
const Segment SGI67095[] = {{&S7618[31], 3}, {NULL}};
const Segment SGI67096[] = {{&S7618[83], 6}, {NULL}};
const Segment SGI67097[] = {{&S7623[44], 8}, {NULL}};
const Segment SGI67098[] = {{&S7623[44], 20}, {NULL}};
const Segment SGI67099[] = {{&S7623[30], 6}, {NULL}};
const Segment SGI67100[] = {{&S7623[31], 3}, {NULL}};
const Segment SGI67101[] = {{&S7623[78], 6}, {NULL}};
const Segment SGI67102[] = {{&S7633[44], 6}, {NULL}};
const Segment SGI67103[] = {{&S7633[44], 18}, {NULL}};
const Segment SGI67104[] = {{&S7633[30], 6}, {NULL}};
const Segment SGI67105[] = {{&S7633[31], 3}, {NULL}};
const Segment SGI67106[] = {{&S7633[76], 6}, {NULL}};
const Segment SGI67107[] = {{&S7651[44], 19}, {NULL}};
const Segment SGI67108[] = {{&S7651[44], 31}, {NULL}};
const Segment SGI67109[] = {{&S7651[30], 6}, {NULL}};
const Segment SGI67110[] = {{&S7651[31], 3}, {NULL}};
const Segment SGI67111[] = {{&S7651[89], 6}, {NULL}};
const Segment SGI67112[] = {{&S7652[44], 7}, {NULL}};
const Segment SGI67113[] = {{&S7652[44], 24}, {NULL}};
const Segment SGI67114[] = {{&S7652[30], 6}, {NULL}};
const Segment SGI67115[] = {{&S7652[31], 3}, {NULL}};
const Segment SGI67116[] = {{&S7652[82], 5}, {NULL}};
const Segment SGI67117[] = {{&S7666[44], 16}, {NULL}};
const Segment SGI67118[] = {{&S7666[44], 28}, {NULL}};
const Segment SGI67119[] = {{&S7666[30], 6}, {NULL}};
const Segment SGI67120[] = {{&S7666[31], 3}, {NULL}};
const Segment SGI67121[] = {{&S7666[86], 6}, {NULL}};
const Segment SGI67122[] = {{&S7667[44], 8}, {NULL}};
const Segment SGI67123[] = {{&S7667[44], 20}, {NULL}};
const Segment SGI67124[] = {{&S7667[30], 6}, {NULL}};
const Segment SGI67125[] = {{&S7667[31], 3}, {NULL}};
const Segment SGI67126[] = {{&S7667[78], 6}, {NULL}};
const Segment SGI67127[] = {{&S7675[44], 7}, {NULL}};
const Segment SGI67128[] = {{&S7675[44], 19}, {NULL}};
const Segment SGI67129[] = {{&S7675[30], 6}, {NULL}};
const Segment SGI67130[] = {{&S7675[31], 3}, {NULL}};
const Segment SGI67131[] = {{&S7675[77], 6}, {NULL}};
const Segment SGI67132[] = {{&S7679[44], 16}, {NULL}};
const Segment SGI67133[] = {{&S7679[44], 28}, {NULL}};
const Segment SGI67134[] = {{&S7679[30], 6}, {NULL}};
const Segment SGI67135[] = {{&S7679[31], 3}, {NULL}};
const Segment SGI67136[] = {{&S7679[86], 6}, {NULL}};
const Segment SGI67137[] = {{&S7711[44], 6}, {NULL}};
const Segment SGI67138[] = {{&S7711[44], 18}, {NULL}};
const Segment SGI67139[] = {{&S7711[30], 6}, {NULL}};
const Segment SGI67140[] = {{&S7711[31], 3}, {NULL}};
const Segment SGI67141[] = {{&S7711[76], 6}, {NULL}};
const Segment SGI67142[] = {{&S7712[44], 8}, {NULL}};
const Segment SGI67143[] = {{&S7712[44], 20}, {NULL}};
const Segment SGI67144[] = {{&S7712[30], 6}, {NULL}};
const Segment SGI67145[] = {{&S7712[31], 3}, {NULL}};
const Segment SGI67146[] = {{&S7712[78], 6}, {NULL}};
const Segment SGI67147[] = {{&S7714[44], 11}, {NULL}};
const Segment SGI67148[] = {{&S7714[44], 23}, {NULL}};
const Segment SGI67149[] = {{&S7714[30], 6}, {NULL}};
const Segment SGI67150[] = {{&S7714[31], 3}, {NULL}};
const Segment SGI67151[] = {{&S7714[81], 6}, {NULL}};
const Segment SGI67152[] = {{&S7715[44], 6}, {NULL}};
const Segment SGI67153[] = {{&S7715[44], 18}, {NULL}};
const Segment SGI67154[] = {{&S7715[30], 6}, {NULL}};
const Segment SGI67155[] = {{&S7715[31], 3}, {NULL}};
const Segment SGI67156[] = {{&S7715[76], 6}, {NULL}};
const Segment SGI67157[] = {{&S7717[44], 5}, {NULL}};
const Segment SGI67158[] = {{&S7717[44], 17}, {NULL}};
const Segment SGI67159[] = {{&S7717[30], 6}, {NULL}};
const Segment SGI67160[] = {{&S7717[31], 3}, {NULL}};
const Segment SGI67161[] = {{&S7717[75], 6}, {NULL}};
const Segment SGI67162[] = {{&S7718[44], 7}, {NULL}};
const Segment SGI67163[] = {{&S7718[44], 19}, {NULL}};
const Segment SGI67164[] = {{&S7718[30], 6}, {NULL}};
const Segment SGI67165[] = {{&S7718[31], 3}, {NULL}};
const Segment SGI67166[] = {{&S7718[77], 6}, {NULL}};
const Segment SGI67167[] = {{&S7720[44], 22}, {NULL}};
const Segment SGI67168[] = {{&S7720[44], 34}, {NULL}};
const Segment SGI67169[] = {{&S7720[30], 6}, {NULL}};
const Segment SGI67170[] = {{&S7720[31], 3}, {NULL}};
const Segment SGI67171[] = {{&S7720[92], 6}, {NULL}};
const Segment SGI67172[] = {{&S7725[44], 13}, {NULL}};
const Segment SGI67173[] = {{&S7725[44], 25}, {NULL}};
const Segment SGI67174[] = {{&S7725[30], 6}, {NULL}};
const Segment SGI67175[] = {{&S7725[31], 3}, {NULL}};
const Segment SGI67176[] = {{&S7725[83], 6}, {NULL}};
const Segment SGI67177[] = {{&S7727[44], 11}, {NULL}};
const Segment SGI67178[] = {{&S7727[44], 23}, {NULL}};
const Segment SGI67179[] = {{&S7727[30], 6}, {NULL}};
const Segment SGI67180[] = {{&S7727[31], 3}, {NULL}};
const Segment SGI67181[] = {{&S7727[81], 6}, {NULL}};
const Segment SGI67182[] = {{&S7735[44], 8}, {NULL}};
const Segment SGI67183[] = {{&S7735[44], 30}, {NULL}};
const Segment SGI67184[] = {{&S7735[30], 6}, {NULL}};
const Segment SGI67185[] = {{&S7735[31], 3}, {NULL}};
const Segment SGI67186[] = {{&S7735[88], 6}, {NULL}};
const Segment SGI67187[] = {{&S7749[44], 22}, {NULL}};
const Segment SGI67188[] = {{&S7749[44], 44}, {NULL}};
const Segment SGI67189[] = {{&S7749[30], 6}, {&S7749[52], 36}, {NULL}};
const Segment SGI67190[] = {{&S7749[31], 3}, {NULL}};
const Segment SGI67191[] = {{&S7749[102], 6}, {NULL}};
const Segment SGI67192[] = {{&S7753[44], 22}, {NULL}};
const Segment SGI67193[] = {{&S7753[30], 6}, {NULL}};
const Segment SGI67194[] = {{&S7753[31], 3}, {NULL}};
const Segment SGI67195[] = {{&S7753[82], 6}, {NULL}};
const Segment SGI67196[] = {{&S7768[44], 7}, {NULL}};
const Segment SGI67197[] = {{&S7768[44], 19}, {NULL}};
const Segment SGI67198[] = {{&S7768[30], 6}, {NULL}};
const Segment SGI67199[] = {{&S7768[31], 3}, {NULL}};
const Segment SGI67200[] = {{&S7768[77], 5}, {NULL}};
const Segment SGI67201[] = {{&S7771[44], 8}, {NULL}};
const Segment SGI67202[] = {{&S7771[44], 20}, {NULL}};
const Segment SGI67203[] = {{&S7771[30], 6}, {NULL}};
const Segment SGI67204[] = {{&S7771[31], 3}, {NULL}};
const Segment SGI67205[] = {{&S7771[78], 6}, {NULL}};
const Segment SGI67206[] = {{&S7777[44], 9}, {NULL}};
const Segment SGI67207[] = {{&S7777[44], 27}, {NULL}};
const Segment SGI67208[] = {{&S7777[30], 6}, {NULL}};
const Segment SGI67209[] = {{&S7777[31], 3}, {NULL}};
const Segment SGI67210[] = {{&S7777[85], 6}, {NULL}};
const Segment SGI67211[] = {{&S7797[44], 77}, {NULL}};
const Segment SGI67212[] = {{&S7797[44], 98}, {NULL}};
const Segment SGI67213[] = {{&S7797[30], 6}, {NULL}};
const Segment SGI67214[] = {{&S7797[31], 3}, {NULL}};
const Segment SGI67215[] = {{&S7797[115], 5}, {NULL}};
const Segment SGI67216[] = {{&S7803[44], 73}, {NULL}};
const Segment SGI67217[] = {{&S7803[44], 94}, {NULL}};
const Segment SGI67218[] = {{&S7803[30], 6}, {NULL}};
const Segment SGI67219[] = {{&S7803[31], 3}, {NULL}};
const Segment SGI67220[] = {{&S7803[111], 5}, {NULL}};
const Segment SGI67221[] = {{&S7822[44], 11}, {NULL}};
const Segment SGI67222[] = {{&S7822[44], 23}, {NULL}};
const Segment SGI67223[] = {{&S7822[30], 6}, {NULL}};
const Segment SGI67224[] = {{&S7822[31], 3}, {NULL}};
const Segment SGI67225[] = {{&S7822[81], 6}, {NULL}};
const Segment SGI67226[] = {{&S7823[44], 10}, {NULL}};
const Segment SGI67227[] = {{&S7823[44], 22}, {NULL}};
const Segment SGI67228[] = {{&S7823[30], 6}, {NULL}};
const Segment SGI67229[] = {{&S7823[31], 3}, {NULL}};
const Segment SGI67230[] = {{&S7823[80], 6}, {NULL}};
const Segment SGI67231[] = {{&S7824[44], 21}, {NULL}};
const Segment SGI67232[] = {{&S7824[44], 33}, {NULL}};
const Segment SGI67233[] = {{&S7824[30], 6}, {NULL}};
const Segment SGI67234[] = {{&S7824[31], 3}, {NULL}};
const Segment SGI67235[] = {{&S7824[91], 6}, {NULL}};
const Segment SGI67236[] = {{&S7825[44], 4}, {NULL}};
const Segment SGI67237[] = {{&S7825[44], 17}, {NULL}};
const Segment SGI67238[] = {{&S7825[30], 6}, {NULL}};
const Segment SGI67239[] = {{&S7825[31], 3}, {NULL}};
const Segment SGI67240[] = {{&S7825[75], 6}, {NULL}};
const Segment SGI67241[] = {{&S7827[44], 8}, {NULL}};
const Segment SGI67242[] = {{&S7827[44], 20}, {NULL}};
const Segment SGI67243[] = {{&S7827[30], 6}, {NULL}};
const Segment SGI67244[] = {{&S7827[31], 3}, {NULL}};
const Segment SGI67245[] = {{&S7827[78], 6}, {NULL}};
const Segment SGI67246[] = {{&S7828[44], 8}, {NULL}};
const Segment SGI67247[] = {{&S7828[44], 20}, {NULL}};
const Segment SGI67248[] = {{&S7828[30], 6}, {NULL}};
const Segment SGI67249[] = {{&S7828[31], 3}, {NULL}};
const Segment SGI67250[] = {{&S7828[78], 6}, {NULL}};
const Segment SGI67251[] = {{&S7834[44], 11}, {NULL}};
const Segment SGI67252[] = {{&S7834[44], 23}, {NULL}};
const Segment SGI67253[] = {{&S7834[30], 6}, {NULL}};
const Segment SGI67254[] = {{&S7834[31], 3}, {NULL}};
const Segment SGI67255[] = {{&S7834[81], 6}, {NULL}};
const Segment SGI67256[] = {{&S7837[44], 15}, {NULL}};
const Segment SGI67257[] = {{&S7837[44], 27}, {NULL}};
const Segment SGI67258[] = {{&S7837[30], 6}, {NULL}};
const Segment SGI67259[] = {{&S7837[31], 3}, {NULL}};
const Segment SGI67260[] = {{&S7837[85], 6}, {NULL}};
const Segment SGI67261[] = {{&S7846[44], 13}, {NULL}};
const Segment SGI67262[] = {{&S7846[44], 25}, {NULL}};
const Segment SGI67263[] = {{&S7846[30], 6}, {NULL}};
const Segment SGI67264[] = {{&S7846[31], 3}, {NULL}};
const Segment SGI67265[] = {{&S7846[83], 6}, {NULL}};
const Segment SGI67266[] = {{&S7849[44], 5}, {NULL}};
const Segment SGI67267[] = {{&S7849[44], 17}, {NULL}};
const Segment SGI67268[] = {{&S7849[30], 6}, {NULL}};
const Segment SGI67269[] = {{&S7849[31], 3}, {NULL}};
const Segment SGI67270[] = {{&S7849[75], 6}, {NULL}};
const Segment SGI67271[] = {{&S7852[44], 7}, {NULL}};
const Segment SGI67272[] = {{&S7852[44], 19}, {NULL}};
const Segment SGI67273[] = {{&S7852[30], 6}, {NULL}};
const Segment SGI67274[] = {{&S7852[31], 3}, {NULL}};
const Segment SGI67275[] = {{&S7852[77], 6}, {NULL}};
const Segment SGI67276[] = {{&S7866[44], 8}, {NULL}};
const Segment SGI67277[] = {{&S7866[44], 20}, {NULL}};
const Segment SGI67278[] = {{&S7866[30], 6}, {NULL}};
const Segment SGI67279[] = {{&S7866[31], 3}, {NULL}};
const Segment SGI67280[] = {{&S7866[78], 6}, {NULL}};
const Segment SGI67281[] = {{&S7888[38], 6}, {NULL}};
const Segment SGI67282[] = {{&S7888[31], 6}, {NULL}};
const Segment SGI67283[] = {{&S7906[44], 9}, {NULL}};
const Segment SGI67284[] = {{&S7906[44], 18}, {NULL}};
const Segment SGI67285[] = {{&S7906[30], 6}, {NULL}};
const Segment SGI67286[] = {{&S7906[31], 3}, {NULL}};
const Segment SGI67287[] = {{&S7906[76], 6}, {NULL}};
const Segment SGI67288[] = {{&S7920[44], 3}, {NULL}};
const Segment SGI67289[] = {{&S7920[44], 15}, {NULL}};
const Segment SGI67290[] = {{&S7920[30], 6}, {NULL}};
const Segment SGI67291[] = {{&S7920[31], 3}, {NULL}};
const Segment SGI67292[] = {{&S7920[73], 6}, {NULL}};
const Segment SGI67293[] = {{&S7929[44], 8}, {NULL}};
const Segment SGI67294[] = {{&S7929[44], 17}, {NULL}};
const Segment SGI67295[] = {{&S7929[30], 6}, {NULL}};
const Segment SGI67296[] = {{&S7929[31], 3}, {NULL}};
const Segment SGI67297[] = {{&S7929[75], 6}, {NULL}};
const Segment SGI67298[] = {{&S7930[44], 11}, {NULL}};
const Segment SGI67299[] = {{&S7930[44], 23}, {NULL}};
const Segment SGI67300[] = {{&S7930[30], 6}, {NULL}};
const Segment SGI67301[] = {{&S7930[31], 3}, {NULL}};
const Segment SGI67302[] = {{&S7930[81], 6}, {NULL}};
const Segment SGI67303[] = {{&S7932[44], 11}, {NULL}};
const Segment SGI67304[] = {{&S7932[44], 39}, {NULL}};
const Segment SGI67305[] = {{&S7932[30], 6}, {NULL}};
const Segment SGI67306[] = {{&S7932[31], 3}, {NULL}};
const Segment SGI67307[] = {{&S7932[97], 6}, {NULL}};
const Segment SGI67308[] = {{&S7940[44], 4}, {NULL}};
const Segment SGI67309[] = {{&S7940[44], 16}, {NULL}};
const Segment SGI67310[] = {{&S7940[30], 6}, {NULL}};
const Segment SGI67311[] = {{&S7940[31], 3}, {NULL}};
const Segment SGI67312[] = {{&S7940[74], 6}, {NULL}};
const Segment SGI67313[] = {{&S24433[44], 11}, {NULL}};
const Segment SGI67314[] = {{&S24433[44], 28}, {NULL}};
const Segment SGI67315[] = {{&S24433[30], 6}, {NULL}};
const Segment SGI67316[] = {{&S24433[120], 3}, {NULL}};
const Segment SGI67317[] = {{&S24433[86], 5}, {NULL}};
const Segment SGI67318[] = {{&S24434[44], 11}, {NULL}};
const Segment SGI67319[] = {{&S24434[44], 28}, {NULL}};
const Segment SGI67320[] = {{&S24434[30], 6}, {NULL}};
const Segment SGI67321[] = {{&S24434[120], 3}, {NULL}};
const Segment SGI67322[] = {{&S24434[86], 5}, {NULL}};
const Segment SGI67323[] = {{&S24435[38], 10}, {NULL}};
const Segment SGI67324[] = {{&S24435[31], 6}, {NULL}};
const Segment SGI67325[] = {{&S24436[44], 10}, {NULL}};
const Segment SGI67326[] = {{&S24436[44], 27}, {NULL}};
const Segment SGI67327[] = {{&S24436[30], 6}, {NULL}};
const Segment SGI67328[] = {{&S24436[119], 3}, {NULL}};
const Segment SGI67329[] = {{&S7971[44], 15}, {NULL}};
const Segment SGI67330[] = {{&S7971[44], 27}, {NULL}};
const Segment SGI67331[] = {{&S24436[85], 5}, {NULL}};
const Segment SGI67332[] = {{&S7971[30], 6}, {NULL}};
const Segment SGI67333[] = {{&S7971[31], 3}, {NULL}};
const Segment SGI67334[] = {{&S7971[85], 6}, {NULL}};
const Segment SGI67335[] = {{&S24453[44], 29}, {NULL}};
const Segment SGI67336[] = {{&S24453[44], 46}, {NULL}};
const Segment SGI67337[] = {{&S24453[30], 6}, {NULL}};
const Segment SGI67338[] = {{&S24453[138], 3}, {NULL}};
const Segment SGI67339[] = {{&S24453[104], 5}, {NULL}};
const Segment SGI67340[] = {{&S24456[44], 11}, {NULL}};
const Segment SGI67341[] = {{&S24456[44], 28}, {NULL}};
const Segment SGI67342[] = {{&S24456[30], 6}, {NULL}};
const Segment SGI67343[] = {{&S24456[120], 3}, {NULL}};
const Segment SGI67344[] = {{&S24456[86], 5}, {NULL}};
const Segment SGI67345[] = {{&S7994[44], 6}, {NULL}};
const Segment SGI67346[] = {{&S7994[44], 27}, {NULL}};
const Segment SGI67347[] = {{&S7994[30], 6}, {NULL}};
const Segment SGI67348[] = {{&S7994[31], 3}, {NULL}};
const Segment SGI67349[] = {{&S7994[85], 5}, {NULL}};
const Segment SGI67350[] = {{&S24463[42], 10}, {NULL}};
const Segment SGI67351[] = {{&S24463[30], 6}, {NULL}};
const Segment SGI67352[] = {{&S24463[117], 3}, {NULL}};
const Segment SGI67353[] = {{&S24463[83], 5}, {NULL}};
const Segment SGI67354[] = {{&S8004[44], 5}, {NULL}};
const Segment SGI67355[] = {{&S8004[44], 17}, {NULL}};
const Segment SGI67356[] = {{&S8004[30], 6}, {NULL}};
const Segment SGI67357[] = {{&S8004[31], 3}, {NULL}};
const Segment SGI67358[] = {{&S8004[75], 6}, {NULL}};
const Segment SGI67359[] = {{&S24472[44], 10}, {NULL}};
const Segment SGI67360[] = {{&S24472[44], 27}, {NULL}};
const Segment SGI67361[] = {{&S24472[30], 6}, {NULL}};
const Segment SGI67362[] = {{&S24472[88], 4}, {NULL}};
const Segment SGI67363[] = {{&S24472[88], 3}, {NULL}};
const Segment SGI67364[] = {{&S24478[44], 10}, {NULL}};
const Segment SGI67365[] = {{&S24478[44], 27}, {NULL}};
const Segment SGI67366[] = {{&S24478[30], 6}, {NULL}};
const Segment SGI67367[] = {{&S24478[121], 3}, {NULL}};
const Segment SGI67368[] = {{&S24478[85], 7}, {NULL}};
const Segment SGI67369[] = {{&S24478[141], 8}, {NULL}};
const Segment SGI67370[] = {{&S8015[44], 6}, {NULL}};
const Segment SGI67371[] = {{&S8015[44], 18}, {NULL}};
const Segment SGI67372[] = {{&S8015[30], 6}, {NULL}};
const Segment SGI67373[] = {{&S8015[31], 3}, {NULL}};
const Segment SGI67374[] = {{&S8015[76], 6}, {NULL}};
const Segment SGI67375[] = {{&S24481[38], 11}, {NULL}};
const Segment SGI67376[] = {{&S24481[31], 6}, {NULL}};
const Segment SGI67377[] = {{&S24482[38], 11}, {NULL}};
const Segment SGI67378[] = {{&S24482[31], 6}, {NULL}};
const Segment SGI67379[] = {{&S24487[44], 10}, {NULL}};
const Segment SGI67380[] = {{&S24487[44], 27}, {NULL}};
const Segment SGI67381[] = {{&S24487[30], 6}, {NULL}};
const Segment SGI67382[] = {{&S24487[88], 6}, {NULL}};
const Segment SGI67383[] = {{&S8027[44], 6}, {NULL}};
const Segment SGI67384[] = {{&S8027[44], 18}, {NULL}};
const Segment SGI67385[] = {{&S8027[30], 6}, {NULL}};
const Segment SGI67386[] = {{&S8027[79], 6}, {NULL}};
const Segment SGI67387[] = {{&S24508[44], 10}, {NULL}};
const Segment SGI67388[] = {{&S24508[44], 27}, {NULL}};
const Segment SGI67389[] = {{&S24508[30], 6}, {NULL}};
const Segment SGI67390[] = {{&S24508[85], 6}, {NULL}};
const Segment SGI67391[] = {{&S24519[44], 10}, {NULL}};
const Segment SGI67392[] = {{&S24519[44], 40}, {NULL}};
const Segment SGI67393[] = {{&S24519[30], 6}, {NULL}};
const Segment SGI67394[] = {{&S24520[44], 18}, {NULL}};
const Segment SGI67395[] = {{&S24520[44], 35}, {NULL}};
const Segment SGI67396[] = {{&S24520[30], 6}, {NULL}};
const Segment SGI67397[] = {{&S24520[127], 3}, {NULL}};
const Segment SGI67398[] = {{&S24520[93], 5}, {NULL}};
const Segment SGI67399[] = {{&S24521[45], 10}, {NULL}};
const Segment SGI67400[] = {{&S24521[45], 33}, {NULL}};
const Segment SGI67401[] = {{&S24521[31], 6}, {NULL}};
const Segment SGI67402[] = {{&S24529[44], 9}, {NULL}};
const Segment SGI67403[] = {{&S24529[30], 6}, {NULL}};
const Segment SGI67404[] = {{&S8075[44], 8}, {NULL}};
const Segment SGI67405[] = {{&S8075[44], 20}, {NULL}};
const Segment SGI67406[] = {{&S8075[30], 6}, {NULL}};
const Segment SGI67407[] = {{&S8075[31], 3}, {NULL}};
const Segment SGI67408[] = {{&S8075[78], 6}, {NULL}};
const Segment SGI67409[] = {{&S8079[44], 77}, {NULL}};
const Segment SGI67410[] = {{&S8079[44], 98}, {NULL}};
const Segment SGI67411[] = {{&S8079[30], 6}, {NULL}};
const Segment SGI67412[] = {{&S8079[31], 3}, {NULL}};
const Segment SGI67413[] = {{&S8079[115], 5}, {NULL}};
const Segment SGI67414[] = {{&S24536[45], 11}, {NULL}};
const Segment SGI67415[] = {{&S24536[45], 34}, {NULL}};
const Segment SGI67416[] = {{&S24536[31], 6}, {NULL}};
const Segment SGI67417[] = {{&S24537[44], 10}, {NULL}};
const Segment SGI67418[] = {{&S24537[44], 33}, {NULL}};
const Segment SGI67419[] = {{&S24537[30], 6}, {NULL}};
const Segment SGI67420[] = {{&S8085[44], 5}, {NULL}};
const Segment SGI67421[] = {{&S8085[44], 26}, {NULL}};
const Segment SGI67422[] = {{&S8085[30], 6}, {NULL}};
const Segment SGI67423[] = {{&S8085[31], 3}, {NULL}};
const Segment SGI67424[] = {{&S8085[84], 5}, {NULL}};
const Segment SGI67425[] = {{&S24540[44], 10}, {NULL}};
const Segment SGI67426[] = {{&S24540[44], 33}, {NULL}};
const Segment SGI67427[] = {{&S24540[30], 6}, {NULL}};
const Segment SGI67428[] = {{&S24540[125], 3}, {NULL}};
const Segment SGI67429[] = {{&S24540[91], 5}, {NULL}};
const Segment SGI67430[] = {{&S24541[52], 11}, {NULL}};
const Segment SGI67431[] = {{&S24541[52], 28}, {NULL}};
const Segment SGI67432[] = {{&S24541[38], 6}, {NULL}};
const Segment SGI67433[] = {{&S24545[38], 11}, {NULL}};
const Segment SGI67434[] = {{&S24545[31], 6}, {NULL}};
const Segment SGI67435[] = {{&S8093[44], 9}, {NULL}};
const Segment SGI67436[] = {{&S8093[44], 21}, {NULL}};
const Segment SGI67437[] = {{&S8093[30], 6}, {NULL}};
const Segment SGI67438[] = {{&S8093[31], 3}, {NULL}};
const Segment SGI67439[] = {{&S8093[79], 6}, {NULL}};
const Segment SGI67440[] = {{&S24548[46], 10}, {NULL}};
const Segment SGI67441[] = {{&S24548[46], 33}, {NULL}};
const Segment SGI67442[] = {{&S24548[32], 6}, {NULL}};
const Segment SGI67443[] = {{&S8096[44], 10}, {NULL}};
const Segment SGI67444[] = {{&S8096[44], 22}, {NULL}};
const Segment SGI67445[] = {{&S8096[30], 6}, {NULL}};
const Segment SGI67446[] = {{&S8096[31], 3}, {NULL}};
const Segment SGI67447[] = {{&S8096[80], 6}, {NULL}};
const Segment SGI67448[] = {{&S8097[44], 7}, {NULL}};
const Segment SGI67449[] = {{&S8097[44], 19}, {NULL}};
const Segment SGI67450[] = {{&S8097[30], 6}, {NULL}};
const Segment SGI67451[] = {{&S8097[31], 3}, {NULL}};
const Segment SGI67452[] = {{&S8097[77], 6}, {NULL}};
const Segment SGI67453[] = {{&S8103[44], 6}, {NULL}};
const Segment SGI67454[] = {{&S8103[44], 18}, {NULL}};
const Segment SGI67455[] = {{&S8103[30], 6}, {NULL}};
const Segment SGI67456[] = {{&S8103[31], 3}, {NULL}};
const Segment SGI67457[] = {{&S8103[76], 6}, {NULL}};
const Segment SGI67458[] = {{&S24559[42], 11}, {NULL}};
const Segment SGI67459[] = {{&S24559[30], 6}, {NULL}};
const Segment SGI67460[] = {{&S24559[118], 3}, {NULL}};
const Segment SGI67461[] = {{&S24559[84], 5}, {NULL}};
const Segment SGI67462[] = {{&S24560[44], 10}, {NULL}};
const Segment SGI67463[] = {{&S24560[44], 27}, {NULL}};
const Segment SGI67464[] = {{&S24560[30], 6}, {NULL}};
const Segment SGI67465[] = {{&S24572[44], 29}, {NULL}};
const Segment SGI67466[] = {{&S24572[44], 46}, {NULL}};
const Segment SGI67467[] = {{&S24572[30], 6}, {NULL}};
const Segment SGI67468[] = {{&S24572[138], 3}, {NULL}};
const Segment SGI67469[] = {{&S24572[104], 5}, {NULL}};
const Segment SGI67470[] = {{&S24577[44], 27}, {NULL}};
const Segment SGI67471[] = {{&S24577[30], 6}, {NULL}};
const Segment SGI67472[] = {{&S24584[56], 31}, {NULL}};
const Segment SGI67473[] = {{&S24584[56], 48}, {NULL}};
const Segment SGI67474[] = {{&S24584[30], 18}, {NULL}};
const Segment SGI67475[] = {{&S24584[152], 3}, {NULL}};
const Segment SGI67476[] = {{&S24584[118], 5}, {NULL}};
const Segment SGI67477[] = {{&S24585[42], 11}, {NULL}};
const Segment SGI67478[] = {{&S24585[30], 6}, {NULL}};
const Segment SGI67479[] = {{&S24585[118], 3}, {NULL}};
const Segment SGI67480[] = {{&S24585[84], 5}, {NULL}};
const Segment SGI67481[] = {{&S8148[44], 22}, {NULL}};
const Segment SGI67482[] = {{&S8148[44], 34}, {NULL}};
const Segment SGI67483[] = {{&S8148[30], 6}, {NULL}};
const Segment SGI67484[] = {{&S8148[31], 3}, {NULL}};
const Segment SGI67485[] = {{&S8148[92], 6}, {NULL}};
const Segment SGI67486[] = {{&S8149[44], 6}, {NULL}};
const Segment SGI67487[] = {{&S8149[44], 18}, {NULL}};
const Segment SGI67488[] = {{&S8149[30], 6}, {NULL}};
const Segment SGI67489[] = {{&S8149[31], 3}, {NULL}};
const Segment SGI67490[] = {{&S8149[76], 5}, {NULL}};
const Segment SGI67491[] = {{&S8176[44], 83}, {NULL}};
const Segment SGI67492[] = {{&S8176[44], 104}, {NULL}};
const Segment SGI67493[] = {{&S8176[30], 6}, {NULL}};
const Segment SGI67494[] = {{&S8176[31], 3}, {NULL}};
const Segment SGI67495[] = {{&S8176[121], 5}, {NULL}};
const Segment SGI67496[] = {{&S8190[44], 11}, {NULL}};
const Segment SGI67497[] = {{&S8190[44], 24}, {NULL}};
const Segment SGI67498[] = {{&S8190[30], 6}, {NULL}};
const Segment SGI67499[] = {{&S8190[31], 3}, {NULL}};
const Segment SGI67500[] = {{&S8190[82], 6}, {NULL}};
const Segment SGI67501[] = {{&S8199[44], 7}, {NULL}};
const Segment SGI67502[] = {{&S8199[44], 19}, {NULL}};
const Segment SGI67503[] = {{&S8199[30], 6}, {NULL}};
const Segment SGI67504[] = {{&S8199[31], 3}, {NULL}};
const Segment SGI67505[] = {{&S8199[77], 6}, {NULL}};
const Segment SGI67506[] = {{&S8202[44], 8}, {NULL}};
const Segment SGI67507[] = {{&S8202[44], 20}, {NULL}};
const Segment SGI67508[] = {{&S8202[30], 6}, {NULL}};
const Segment SGI67509[] = {{&S8202[31], 3}, {NULL}};
const Segment SGI67510[] = {{&S8202[78], 6}, {NULL}};
const Segment SGI67511[] = {{&S8205[44], 12}, {NULL}};
const Segment SGI67512[] = {{&S8205[44], 27}, {NULL}};
const Segment SGI67513[] = {{&S8205[30], 6}, {NULL}};
const Segment SGI67514[] = {{&S8205[31], 3}, {NULL}};
const Segment SGI67515[] = {{&S8205[85], 5}, {NULL}};
const Segment SGI67516[] = {{&S8206[44], 8}, {NULL}};
const Segment SGI67517[] = {{&S8206[44], 20}, {NULL}};
const Segment SGI67518[] = {{&S8206[30], 6}, {NULL}};
const Segment SGI67519[] = {{&S8206[31], 3}, {NULL}};
const Segment SGI67520[] = {{&S8206[78], 6}, {NULL}};
const Segment SGI67521[] = {{&S8207[44], 9}, {NULL}};
const Segment SGI67522[] = {{&S8207[44], 21}, {NULL}};
const Segment SGI67523[] = {{&S8207[30], 6}, {NULL}};
const Segment SGI67524[] = {{&S8207[31], 3}, {NULL}};
const Segment SGI67525[] = {{&S8207[79], 6}, {NULL}};
const Segment SGI67526[] = {{&S8209[44], 9}, {NULL}};
const Segment SGI67527[] = {{&S8209[44], 22}, {NULL}};
const Segment SGI67528[] = {{&S8209[30], 6}, {NULL}};
const Segment SGI67529[] = {{&S8209[31], 3}, {NULL}};
const Segment SGI67530[] = {{&S8209[80], 6}, {NULL}};
const Segment SGI67531[] = {{&S8221[30], 6}, {NULL}};
const Segment SGI67532[] = {{&S8221[31], 3}, {NULL}};
const Segment SGI67533[] = {{&S8221[80], 6}, {NULL}};
const Segment SGI67534[] = {{&S8228[44], 6}, {NULL}};
const Segment SGI67535[] = {{&S8228[44], 18}, {NULL}};
const Segment SGI67536[] = {{&S8228[30], 6}, {NULL}};
const Segment SGI67537[] = {{&S8228[31], 3}, {NULL}};
const Segment SGI67538[] = {{&S8228[76], 5}, {NULL}};
const Segment SGI67539[] = {{&S8246[44], 7}, {NULL}};
const Segment SGI67540[] = {{&S8246[44], 19}, {NULL}};
const Segment SGI67541[] = {{&S8246[30], 6}, {NULL}};
const Segment SGI67542[] = {{&S8246[31], 3}, {NULL}};
const Segment SGI67543[] = {{&S8246[77], 5}, {NULL}};
const Segment SGI67544[] = {{&S8250[44], 7}, {NULL}};
const Segment SGI67545[] = {{&S8250[44], 19}, {NULL}};
const Segment SGI67546[] = {{&S8250[30], 6}, {NULL}};
const Segment SGI67547[] = {{&S8250[31], 3}, {NULL}};
const Segment SGI67548[] = {{&S8250[77], 5}, {NULL}};
const Segment SGI67549[] = {{&S8255[46], 9}, {NULL}};
const Segment SGI67550[] = {{&S8255[46], 30}, {NULL}};
const Segment SGI67551[] = {{&S8255[30], 8}, {NULL}};
const Segment SGI67552[] = {{&S8255[31], 3}, {NULL}};
const Segment SGI67553[] = {{&S8255[90], 5}, {NULL}};
const Segment SGI67554[] = {{&S8262[44], 8}, {NULL}};
const Segment SGI67555[] = {{&S8262[44], 20}, {NULL}};
const Segment SGI67556[] = {{&S8262[30], 6}, {NULL}};
const Segment SGI67557[] = {{&S8262[31], 3}, {NULL}};
const Segment SGI67558[] = {{&S8262[78], 6}, {NULL}};
const Segment SGI67559[] = {{&S12379[75], 6}, {NULL}};
const Segment SGI67560[] = {{&S12379[0], 10}, {NULL}};
const Segment SGI67561[] = {{&S8278[44], 6}, {NULL}};
const Segment SGI67562[] = {{&S8278[44], 18}, {NULL}};
const Segment SGI67563[] = {{&S8278[30], 6}, {NULL}};
const Segment SGI67564[] = {{&S8278[31], 3}, {NULL}};
const Segment SGI67565[] = {{&S8278[76], 5}, {NULL}};
const Segment SGI67566[] = {{&S8279[44], 6}, {NULL}};
const Segment SGI67567[] = {{&S8279[44], 18}, {NULL}};
const Segment SGI67568[] = {{&S8279[30], 6}, {NULL}};
const Segment SGI67569[] = {{&S8279[31], 3}, {NULL}};
const Segment SGI67570[] = {{&S8279[76], 6}, {NULL}};
const Segment SGI67571[] = {{&S8280[44], 14}, {NULL}};
const Segment SGI67572[] = {{&S8280[44], 26}, {NULL}};
const Segment SGI67573[] = {{&S8280[30], 6}, {NULL}};
const Segment SGI67574[] = {{&S8280[31], 3}, {NULL}};
const Segment SGI67575[] = {{&S8280[84], 6}, {NULL}};
const Segment SGI67576[] = {{&S8281[44], 5}, {NULL}};
const Segment SGI67577[] = {{&S8281[44], 23}, {NULL}};
const Segment SGI67578[] = {{&S8281[30], 6}, {NULL}};
const Segment SGI67579[] = {{&S8281[31], 3}, {NULL}};
const Segment SGI67580[] = {{&S8281[81], 6}, {NULL}};
const Segment SGI67581[] = {{&S8306[44], 17}, {NULL}};
const Segment SGI67582[] = {{&S8306[44], 29}, {NULL}};
const Segment SGI67583[] = {{&S8306[30], 6}, {NULL}};
const Segment SGI67584[] = {{&S8306[31], 3}, {NULL}};
const Segment SGI67585[] = {{&S8306[87], 6}, {NULL}};
const Segment SGI67586[] = {{&S8307[44], 4}, {NULL}};
const Segment SGI67587[] = {{&S8307[44], 16}, {NULL}};
const Segment SGI67588[] = {{&S8307[30], 6}, {NULL}};
const Segment SGI67589[] = {{&S8307[31], 3}, {NULL}};
const Segment SGI67590[] = {{&S8307[74], 6}, {NULL}};
const Segment SGI67591[] = {{&S8309[44], 7}, {NULL}};
const Segment SGI67592[] = {{&S8309[44], 19}, {NULL}};
const Segment SGI67593[] = {{&S8309[30], 6}, {NULL}};
const Segment SGI67594[] = {{&S8309[31], 3}, {NULL}};
const Segment SGI67595[] = {{&S8309[77], 6}, {NULL}};
const Segment SGI67596[] = {{&S8314[44], 5}, {NULL}};
const Segment SGI67597[] = {{&S8314[44], 17}, {NULL}};
const Segment SGI67598[] = {{&S8314[30], 6}, {NULL}};
const Segment SGI67599[] = {{&S8314[31], 3}, {NULL}};
const Segment SGI67600[] = {{&S8314[75], 6}, {NULL}};
const Segment SGI67601[] = {{&S24959[44], 10}, {NULL}};
const Segment SGI67602[] = {{&S24959[44], 27}, {NULL}};
const Segment SGI67603[] = {{&S24959[30], 6}, {NULL}};
const Segment SGI67604[] = {{&S24959[119], 3}, {NULL}};
const Segment SGI67605[] = {{&S24959[85], 5}, {NULL}};
const Segment SGI67606[] = {{&S24960[44], 11}, {NULL}};
const Segment SGI67607[] = {{&S24960[44], 28}, {NULL}};
const Segment SGI67608[] = {{&S24960[30], 6}, {NULL}};
const Segment SGI67609[] = {{&S24960[120], 3}, {NULL}};
const Segment SGI67610[] = {{&S24960[86], 5}, {NULL}};
const Segment SGI67611[] = {{&S24961[38], 10}, {NULL}};
const Segment SGI67612[] = {{&S24961[31], 6}, {NULL}};
const Segment SGI67613[] = {{&S24963[44], 10}, {NULL}};
const Segment SGI67614[] = {{&S24963[44], 27}, {NULL}};
const Segment SGI67615[] = {{&S24963[30], 6}, {NULL}};
const Segment SGI67616[] = {{&S24963[88], 4}, {NULL}};
const Segment SGI67617[] = {{&S24963[88], 3}, {NULL}};
const Segment SGI67618[] = {{&S24964[44], 11}, {NULL}};
const Segment SGI67619[] = {{&S24964[44], 28}, {NULL}};
const Segment SGI67620[] = {{&S24964[30], 6}, {NULL}};
const Segment SGI67621[] = {{&S24964[120], 3}, {NULL}};
const Segment SGI67622[] = {{&S24964[86], 5}, {NULL}};
const Segment SGI67623[] = {{&S24965[44], 10}, {NULL}};
const Segment SGI67624[] = {{&S24965[44], 27}, {NULL}};
const Segment SGI67625[] = {{&S24965[30], 6}, {NULL}};
const Segment SGI67626[] = {{&S24965[88], 6}, {NULL}};
const Segment SGI67627[] = {{&S24966[38], 11}, {NULL}};
const Segment SGI67628[] = {{&S24966[31], 6}, {NULL}};
const Segment SGI67629[] = {{&S24968[42], 10}, {NULL}};
const Segment SGI67630[] = {{&S24968[30], 6}, {NULL}};
const Segment SGI67631[] = {{&S24968[117], 3}, {NULL}};
const Segment SGI67632[] = {{&S24968[83], 5}, {NULL}};
const Segment SGI67633[] = {{&S24969[44], 27}, {NULL}};
const Segment SGI67634[] = {{&S24969[30], 6}, {NULL}};
const Segment SGI67635[] = {{&S8329[38], 7}, {NULL}};
const Segment SGI67636[] = {{&S8329[31], 6}, {NULL}};
const Segment SGI67637[] = {{&S8332[44], 13}, {NULL}};
const Segment SGI67638[] = {{&S8332[44], 25}, {NULL}};
const Segment SGI67639[] = {{&S8332[30], 6}, {NULL}};
const Segment SGI67640[] = {{&S8332[31], 3}, {NULL}};
const Segment SGI67641[] = {{&S8332[83], 6}, {NULL}};
const Segment SGI67642[] = {{&S8337[44], 21}, {NULL}};
const Segment SGI67643[] = {{&S8337[44], 33}, {NULL}};
const Segment SGI67644[] = {{&S8337[30], 6}, {NULL}};
const Segment SGI67645[] = {{&S8337[31], 3}, {NULL}};
const Segment SGI67646[] = {{&S8337[91], 6}, {NULL}};
const Segment SGI67647[] = {{&S8342[44], 14}, {NULL}};
const Segment SGI67648[] = {{&S8342[44], 32}, {NULL}};
const Segment SGI67649[] = {{&S8342[30], 6}, {NULL}};
const Segment SGI67650[] = {{&S8342[31], 3}, {NULL}};
const Segment SGI67651[] = {{&S8342[90], 6}, {NULL}};
const Segment SGI67652[] = {{&S8349[44], 8}, {NULL}};
const Segment SGI67653[] = {{&S8349[44], 20}, {NULL}};
const Segment SGI67654[] = {{&S8349[30], 6}, {NULL}};
const Segment SGI67655[] = {{&S8349[31], 3}, {NULL}};
const Segment SGI67656[] = {{&S8349[78], 6}, {NULL}};
const Segment SGI67657[] = {{&S8358[44], 6}, {NULL}};
const Segment SGI67658[] = {{&S8358[44], 18}, {NULL}};
const Segment SGI67659[] = {{&S8358[30], 6}, {NULL}};
const Segment SGI67660[] = {{&S8358[31], 3}, {NULL}};
const Segment SGI67661[] = {{&S8358[76], 5}, {NULL}};
const Segment SGI67662[] = {{&S8359[44], 10}, {NULL}};
const Segment SGI67663[] = {{&S8359[44], 22}, {NULL}};
const Segment SGI67664[] = {{&S8359[30], 6}, {NULL}};
const Segment SGI67665[] = {{&S8359[31], 3}, {NULL}};
const Segment SGI67666[] = {{&S8359[80], 6}, {NULL}};
const Segment SGI67667[] = {{&S8363[44], 9}, {NULL}};
const Segment SGI67668[] = {{&S8363[44], 21}, {NULL}};
const Segment SGI67669[] = {{&S8363[30], 6}, {NULL}};
const Segment SGI67670[] = {{&S8363[31], 3}, {NULL}};
const Segment SGI67671[] = {{&S8363[79], 6}, {NULL}};
const Segment SGI67672[] = {{&S8364[44], 13}, {NULL}};
const Segment SGI67673[] = {{&S8364[44], 25}, {NULL}};
const Segment SGI67674[] = {{&S8364[30], 6}, {NULL}};
const Segment SGI67675[] = {{&S8364[31], 3}, {NULL}};
const Segment SGI67676[] = {{&S8364[83], 6}, {NULL}};
const Segment SGI67677[] = {{&S8394[44], 10}, {NULL}};
const Segment SGI67678[] = {{&S8394[44], 22}, {NULL}};
const Segment SGI67679[] = {{&S8394[30], 6}, {NULL}};
const Segment SGI67680[] = {{&S8394[31], 3}, {NULL}};
const Segment SGI67681[] = {{&S8394[80], 6}, {NULL}};
const Segment SGI67682[] = {{&S8397[44], 11}, {NULL}};
const Segment SGI67683[] = {{&S8397[44], 23}, {NULL}};
const Segment SGI67684[] = {{&S8397[30], 6}, {NULL}};
const Segment SGI67685[] = {{&S8397[31], 3}, {NULL}};
const Segment SGI67686[] = {{&S8397[81], 6}, {NULL}};
const Segment SGI67687[] = {{&S8406[44], 7}, {NULL}};
const Segment SGI67688[] = {{&S8406[44], 20}, {NULL}};
const Segment SGI67689[] = {{&S8406[30], 6}, {NULL}};
const Segment SGI67690[] = {{&S8406[31], 3}, {NULL}};
const Segment SGI67691[] = {{&S8406[78], 6}, {NULL}};
const Segment SGI67692[] = {{&S8407[44], 9}, {NULL}};
const Segment SGI67693[] = {{&S8407[44], 21}, {NULL}};
const Segment SGI67694[] = {{&S8407[30], 6}, {NULL}};
const Segment SGI67695[] = {{&S8407[31], 3}, {NULL}};
const Segment SGI67696[] = {{&S8407[79], 6}, {NULL}};
const Segment SGI67697[] = {{&S8412[44], 13}, {NULL}};
const Segment SGI67698[] = {{&S8412[44], 25}, {NULL}};
const Segment SGI67699[] = {{&S8412[30], 6}, {NULL}};
const Segment SGI67700[] = {{&S8412[31], 3}, {NULL}};
const Segment SGI67701[] = {{&S8412[83], 6}, {NULL}};
const Segment SGI67702[] = {{&S8422[38], 6}, {NULL}};
const Segment SGI67703[] = {{&S8422[31], 6}, {NULL}};
const Segment SGI67704[] = {{&S8425[44], 13}, {NULL}};
const Segment SGI67705[] = {{&S8425[44], 47}, {NULL}};
const Segment SGI67706[] = {{&S8425[30], 6}, {NULL}};
const Segment SGI67707[] = {{&S8425[31], 3}, {NULL}};
const Segment SGI67708[] = {{&S8425[105], 6}, {NULL}};
const Segment SGI67709[] = {{&S8428[44], 9}, {NULL}};
const Segment SGI67710[] = {{&S8428[44], 21}, {NULL}};
const Segment SGI67711[] = {{&S8428[30], 6}, {NULL}};
const Segment SGI67712[] = {{&S8428[31], 3}, {NULL}};
const Segment SGI67713[] = {{&S8428[79], 6}, {NULL}};
const Segment SGI67714[] = {{&S8436[44], 16}, {NULL}};
const Segment SGI67715[] = {{&S8436[44], 28}, {NULL}};
const Segment SGI67716[] = {{&S8436[30], 6}, {NULL}};
const Segment SGI67717[] = {{&S8436[31], 3}, {NULL}};
const Segment SGI67718[] = {{&S8436[86], 6}, {NULL}};
const Segment SGI67719[] = {{&S8437[44], 10}, {NULL}};
const Segment SGI67720[] = {{&S8437[44], 22}, {NULL}};
const Segment SGI67721[] = {{&S8437[30], 6}, {NULL}};
const Segment SGI67722[] = {{&S8437[83], 6}, {NULL}};
const Segment SGI67723[] = {{&S8451[44], 10}, {NULL}};
const Segment SGI67724[] = {{&S8451[44], 22}, {NULL}};
const Segment SGI67725[] = {{&S8451[30], 6}, {NULL}};
const Segment SGI67726[] = {{&S8451[31], 3}, {NULL}};
const Segment SGI67727[] = {{&S8451[80], 6}, {NULL}};
const Segment SGI67728[] = {{&S8499[44], 10}, {NULL}};
const Segment SGI67729[] = {{&S8499[44], 22}, {NULL}};
const Segment SGI67730[] = {{&S8499[30], 6}, {NULL}};
const Segment SGI67731[] = {{&S8499[31], 3}, {NULL}};
const Segment SGI67732[] = {{&S8499[80], 6}, {NULL}};
const Segment SGI67733[] = {{&S8509[44], 10}, {NULL}};
const Segment SGI67734[] = {{&S8509[44], 31}, {NULL}};
const Segment SGI67735[] = {{&S8509[30], 6}, {NULL}};
const Segment SGI67736[] = {{&S8509[31], 3}, {NULL}};
const Segment SGI67737[] = {{&S8509[89], 5}, {NULL}};
const Segment SGI67738[] = {{&S8519[38], 10}, {NULL}};
const Segment SGI67739[] = {{&S8519[31], 6}, {NULL}};
const Segment SGI67740[] = {{&S15126[42], 9}, {NULL}};
const Segment SGI67741[] = {{&S15126[42], 20}, {NULL}};
const Segment SGI67742[] = {{&S15126[30], 4}, {NULL}};
const Segment SGI67743[] = {{&S15126[111], 3}, {NULL}};
const Segment SGI67744[] = {{&S15126[76], 6}, {NULL}};
const Segment SGI67745[] = {{&S15126[131], 8}, {NULL}};
const Segment SGI67746[] = {{&S19073[44], 7}, {NULL}};
const Segment SGI67747[] = {{&S19073[44], 25}, {NULL}};
const Segment SGI67748[] = {{&S19073[30], 6}, {NULL}};
const Segment SGI67749[] = {{&S19073[57], 3}, {NULL}};
const Segment SGI67750[] = {{&S19073[83], 5}, {NULL}};
const Segment SGI67751[] = {{&S19084[44], 7}, {NULL}};
const Segment SGI67752[] = {{&S19084[44], 26}, {NULL}};
const Segment SGI67753[] = {{&S19084[30], 6}, {NULL}};
const Segment SGI67754[] = {{&S19084[118], 3}, {NULL}};
const Segment SGI67755[] = {{&S19084[84], 5}, {NULL}};
const Segment SGI67756[] = {{&S19086[44], 19}, {NULL}};
const Segment SGI67757[] = {{&S19086[44], 37}, {NULL}};
const Segment SGI67758[] = {{&S19086[30], 6}, {NULL}};
const Segment SGI67759[] = {{&S19086[69], 3}, {NULL}};
const Segment SGI67760[] = {{&S19086[95], 5}, {NULL}};
const Segment SGI67761[] = {{&S15378[42], 13}, {NULL}};
const Segment SGI67762[] = {{&S15378[42], 25}, {NULL}};
const Segment SGI67763[] = {{&S15378[30], 4}, {NULL}};
const Segment SGI67764[] = {{&S15378[31], 3}, {NULL}};
const Segment SGI67765[] = {{&S15378[81], 5}, {NULL}};
const Segment SGI67766[] = {{&S25547[44], 10}, {NULL}};
const Segment SGI67767[] = {{&S25547[44], 27}, {NULL}};
const Segment SGI67768[] = {{&S25547[30], 6}, {NULL}};
const Segment SGI67769[] = {{&S25547[119], 3}, {NULL}};
const Segment SGI67770[] = {{&S25547[85], 5}, {NULL}};
const Segment SGI67771[] = {{&S25548[44], 11}, {NULL}};
const Segment SGI67772[] = {{&S25548[44], 28}, {NULL}};
const Segment SGI67773[] = {{&S25548[30], 6}, {NULL}};
const Segment SGI67774[] = {{&S25548[120], 3}, {NULL}};
const Segment SGI67775[] = {{&S25548[86], 5}, {NULL}};
const Segment SGI67776[] = {{&S25549[38], 10}, {NULL}};
const Segment SGI67777[] = {{&S25549[31], 6}, {NULL}};
const Segment SGI67778[] = {{&S25550[44], 10}, {NULL}};
const Segment SGI67779[] = {{&S25550[44], 27}, {NULL}};
const Segment SGI67780[] = {{&S25550[30], 6}, {NULL}};
const Segment SGI67781[] = {{&S25550[88], 4}, {NULL}};
const Segment SGI67782[] = {{&S25550[49], 3}, {NULL}};
const Segment SGI67783[] = {{&S25551[44], 11}, {NULL}};
const Segment SGI67784[] = {{&S25551[44], 28}, {NULL}};
const Segment SGI67785[] = {{&S25551[30], 6}, {NULL}};
const Segment SGI67786[] = {{&S25551[120], 3}, {NULL}};
const Segment SGI67787[] = {{&S25551[86], 5}, {NULL}};
const Segment SGI67788[] = {{&S25552[42], 10}, {NULL}};
const Segment SGI67789[] = {{&S25552[30], 6}, {NULL}};
const Segment SGI67790[] = {{&S25552[117], 3}, {NULL}};
const Segment SGI67791[] = {{&S25552[83], 5}, {NULL}};
const Segment SGI67792[] = {{&S25553[38], 11}, {NULL}};
const Segment SGI67793[] = {{&S25553[31], 6}, {NULL}};
const Segment SGI67794[] = {{&S26036[44], 13}, {NULL}};
const Segment SGI67795[] = {{&S26036[44], 29}, {NULL}};
const Segment SGI67796[] = {{&S26036[30], 6}, {NULL}};
const Segment SGI67797[] = {{&S26036[121], 3}, {NULL}};
const Segment SGI67798[] = {{&S26036[87], 5}, {NULL}};
const Segment SGI67799[] = {{&S26040[44], 6}, {NULL}};
const Segment SGI67800[] = {{&S26040[44], 22}, {NULL}};
const Segment SGI67801[] = {{&S26040[30], 6}, {NULL}};
const Segment SGI67802[] = {{&S26040[114], 3}, {NULL}};
const Segment SGI67803[] = {{&S26040[80], 5}, {NULL}};
const Segment SGI67804[] = {{&S26042[38], 6}, {NULL}};
const Segment SGI67805[] = {{&S26042[31], 6}, {NULL}};
const Segment SGI67806[] = {{&S26043[44], 7}, {NULL}};
const Segment SGI67807[] = {{&S26043[44], 24}, {NULL}};
const Segment SGI67808[] = {{&S26043[30], 6}, {NULL}};
const Segment SGI67809[] = {{&S26043[116], 3}, {NULL}};
const Segment SGI67810[] = {{&S26043[82], 5}, {NULL}};
const Segment SGI67811[] = {{&S26044[38], 7}, {NULL}};
const Segment SGI67812[] = {{&S26044[31], 6}, {NULL}};
const Segment SGI67813[] = {{&S26046[38], 13}, {NULL}};
const Segment SGI67814[] = {{&S26046[31], 6}, {NULL}};
const Segment SGI67815[] = {{&S26198[38], 10}, {NULL}};
const Segment SGI67816[] = {{&S26198[31], 6}, {NULL}};
const Segment SGI67817[] = {{&S26199[44], 10}, {NULL}};
const Segment SGI67818[] = {{&S26199[44], 27}, {NULL}};
const Segment SGI67819[] = {{&S26199[30], 6}, {NULL}};
const Segment SGI67820[] = {{&S26199[88], 4}, {NULL}};
const Segment SGI67821[] = {{&S26199[88], 3}, {NULL}};
const Segment SGI67822[] = {{&S26200[44], 10}, {NULL}};
const Segment SGI67823[] = {{&S26200[44], 27}, {NULL}};
const Segment SGI67824[] = {{&S26200[30], 6}, {NULL}};
const Segment SGI67825[] = {{&S26200[119], 3}, {NULL}};
const Segment SGI67826[] = {{&S26200[85], 5}, {NULL}};
const Segment SGI67827[] = {{&S26201[44], 11}, {NULL}};
const Segment SGI67828[] = {{&S26201[44], 28}, {NULL}};
const Segment SGI67829[] = {{&S26201[30], 6}, {NULL}};
const Segment SGI67830[] = {{&S26201[120], 3}, {NULL}};
const Segment SGI67831[] = {{&S26201[86], 5}, {NULL}};
const Segment SGI67832[] = {{&S26202[44], 11}, {NULL}};
const Segment SGI67833[] = {{&S26202[44], 28}, {NULL}};
const Segment SGI67834[] = {{&S26202[30], 6}, {NULL}};
const Segment SGI67835[] = {{&S26202[120], 3}, {NULL}};
const Segment SGI67836[] = {{&S26202[86], 5}, {NULL}};
const Segment SGI67837[] = {{&S26203[38], 11}, {NULL}};
const Segment SGI67838[] = {{&S26203[31], 6}, {NULL}};
const Segment SGI67839[] = {{&S26204[42], 10}, {NULL}};
const Segment SGI67840[] = {{&S26204[30], 6}, {NULL}};
const Segment SGI67841[] = {{&S26204[117], 3}, {NULL}};
const Segment SGI67842[] = {{&S26204[83], 5}, {NULL}};
const Segment SGI67843[] = {{&S26205[44], 11}, {NULL}};
const Segment SGI67844[] = {{&S26205[44], 28}, {NULL}};
const Segment SGI67845[] = {{&S26205[30], 6}, {NULL}};
const Segment SGI67846[] = {{&S26205[89], 6}, {NULL}};
const Segment SGI67847[] = {{&S26206[44], 10}, {NULL}};
const Segment SGI67848[] = {{&S26206[44], 27}, {NULL}};
const Segment SGI67849[] = {{&S26206[30], 6}, {NULL}};
const Segment SGI67850[] = {{&S26206[88], 6}, {NULL}};
const Segment SGI67851[] = {{&S20249[44], 7}, {NULL}};
const Segment SGI67852[] = {{&S20249[44], 24}, {NULL}};
const Segment SGI67853[] = {{&S20249[30], 6}, {NULL}};
const Segment SGI67854[] = {{&S20249[57], 3}, {NULL}};
const Segment SGI67855[] = {{&S20249[82], 5}, {NULL}};
const Segment SGI67856[] = {{&S20498[30], 6}, {NULL}};
const Segment SGI67857[] = {{&S20498[104], 3}, {NULL}};
const Segment SGI67858[] = {{&S20498[57], 6}, {NULL}};
const Segment SGI67859[] = {{&S20523[44], 4}, {NULL}};
const Segment SGI67860[] = {{&S20523[44], 16}, {NULL}};
const Segment SGI67861[] = {{&S20523[30], 6}, {NULL}};
const Segment SGI67862[] = {{&S20523[108], 3}, {NULL}};
const Segment SGI67863[] = {{&S20523[74], 5}, {NULL}};
const Segment SGI67864[] = {{&S20551[44], 8}, {NULL}};
const Segment SGI67865[] = {{&S20551[44], 23}, {NULL}};
const Segment SGI67866[] = {{&S20551[30], 6}, {NULL}};
const Segment SGI67867[] = {{&S20551[115], 3}, {NULL}};
const Segment SGI67868[] = {{&S20551[81], 5}, {NULL}};
const Segment SGI67869[] = {{&S20555[44], 5}, {NULL}};
const Segment SGI67870[] = {{&S20555[44], 21}, {NULL}};
const Segment SGI67871[] = {{&S20555[30], 6}, {NULL}};
const Segment SGI67872[] = {{&S20555[113], 3}, {NULL}};
const Segment SGI67873[] = {{&S20555[79], 5}, {NULL}};
const Segment SGI67874[] = {{&S20577[22], 9}, {NULL}};
const Segment SGI67875[] = {{&S20577[44], 21}, {NULL}};
const Segment SGI67876[] = {{&S20577[30], 6}, {&S20577[52], 13}, {NULL}};
const Segment SGI67877[] = {{&S20577[113], 3}, {NULL}};
const Segment SGI67878[] = {{&S20577[79], 5}, {NULL}};
const Segment SGI67879[] = {{&S20597[11], 1}, {NULL}};
const Segment SGI67880[] = {{&S20597[44], 21}, {NULL}};
const Segment SGI67881[] = {{&S20597[30], 6}, {NULL}};
const Segment SGI67882[] = {{&S20597[113], 3}, {NULL}};
const Segment SGI67883[] = {{&S20597[79], 5}, {NULL}};
const Segment SGI67884[] = {{&S20605[46], 6}, {NULL}};
const Segment SGI67885[] = {{&S20605[154], 3}, {NULL}};
const Segment SGI67886[] = {{&S20605[120], 5}, {NULL}};
const Segment SGI67887[] = {{&S27026[44], 7}, {NULL}};
const Segment SGI67888[] = {{&S27026[44], 19}, {NULL}};
const Segment SGI67889[] = {{&S27026[30], 6}, {NULL}};
const Segment SGI67890[] = {{&S27026[111], 3}, {NULL}};
const Segment SGI67891[] = {{&S27026[77], 5}, {NULL}};
const Segment SGI67892[] = {{&S27155[44], 10}, {NULL}};
const Segment SGI67893[] = {{&S27155[44], 27}, {NULL}};
const Segment SGI67894[] = {{&S27155[30], 6}, {NULL}};
const Segment SGI67895[] = {{&S27155[119], 3}, {NULL}};
const Segment SGI67896[] = {{&S27155[85], 5}, {NULL}};
const Segment SGI67897[] = {{&S27157[38], 10}, {NULL}};
const Segment SGI67898[] = {{&S27157[31], 6}, {NULL}};
const Segment SGI67899[] = {{&S27215[38], 10}, {NULL}};
const Segment SGI67900[] = {{&S27215[31], 6}, {NULL}};
const Segment SGI67901[] = {{&S27216[44], 9}, {NULL}};
const Segment SGI67902[] = {{&S27216[44], 26}, {NULL}};
const Segment SGI67903[] = {{&S27216[30], 6}, {NULL}};
const Segment SGI67904[] = {{&S27216[118], 3}, {NULL}};
const Segment SGI67905[] = {{&S27216[84], 5}, {NULL}};
const Segment SGI67906[] = {{&S27217[44], 10}, {NULL}};
const Segment SGI67907[] = {{&S27217[44], 27}, {NULL}};
const Segment SGI67908[] = {{&S27217[30], 6}, {NULL}};
const Segment SGI67909[] = {{&S27217[119], 3}, {NULL}};
const Segment SGI67910[] = {{&S27217[85], 5}, {NULL}};
const Segment SGI67911[] = {{&S27219[64], 10}, {NULL}};
const Segment SGI67912[] = {{&S27219[64], 27}, {NULL}};
const Segment SGI67913[] = {{&S27219[50], 6}, {NULL}};
const Segment SGI67914[] = {{&S27220[44], 9}, {NULL}};
const Segment SGI67915[] = {{&S27220[44], 26}, {NULL}};
const Segment SGI67916[] = {{&S27220[30], 6}, {NULL}};
const Segment SGI67917[] = {{&S27220[118], 3}, {NULL}};
const Segment SGI67918[] = {{&S27220[84], 5}, {NULL}};
const Segment SGI67919[] = {{&S27221[44], 10}, {NULL}};
const Segment SGI67920[] = {{&S27221[44], 27}, {NULL}};
const Segment SGI67921[] = {{&S27221[30], 6}, {NULL}};
const Segment SGI67922[] = {{&S27221[88], 4}, {NULL}};
const Segment SGI67923[] = {{&S27221[88], 3}, {NULL}};
const Segment SGI67924[] = {{&S27223[44], 10}, {NULL}};
const Segment SGI67925[] = {{&S27223[44], 27}, {NULL}};
const Segment SGI67926[] = {{&S27223[30], 6}, {NULL}};
const Segment SGI67927[] = {{&S27223[119], 3}, {NULL}};
const Segment SGI67928[] = {{&S27223[85], 5}, {NULL}};
const Segment SGI67929[] = {{&S27225[43], 10}, {NULL}};
const Segment SGI67930[] = {{&S27225[43], 27}, {NULL}};
const Segment SGI67931[] = {{&S27225[30], 5}, {NULL}};
const Segment SGI67932[] = {{&S27225[118], 3}, {NULL}};
const Segment SGI67933[] = {{&S27225[84], 5}, {NULL}};
const Segment SGI67934[] = {{&S27226[44], 10}, {NULL}};
const Segment SGI67935[] = {{&S27226[44], 33}, {NULL}};
const Segment SGI67936[] = {{&S27226[30], 6}, {NULL}};
const Segment SGI67937[] = {{&S27226[125], 3}, {NULL}};
const Segment SGI67938[] = {{&S27226[91], 5}, {NULL}};
const Segment SGI67939[] = {{&S27227[43], 10}, {NULL}};
const Segment SGI67940[] = {{&S27227[43], 27}, {NULL}};
const Segment SGI67941[] = {{&S27227[118], 3}, {NULL}};
const Segment SGI67942[] = {{&S27227[84], 5}, {NULL}};
const Segment SGI67943[] = {{&S27228[42], 10}, {NULL}};
const Segment SGI67944[] = {{&S27228[30], 6}, {NULL}};
const Segment SGI67945[] = {{&S27228[117], 3}, {NULL}};
const Segment SGI67946[] = {{&S27228[83], 5}, {NULL}};
const Segment SGI67947[] = {{&S27229[44], 10}, {NULL}};
const Segment SGI67948[] = {{&S27229[44], 40}, {NULL}};
const Segment SGI67949[] = {{&S27229[30], 6}, {NULL}};
const Segment SGI67950[] = {{&S27229[132], 3}, {NULL}};
const Segment SGI67951[] = {{&S27229[98], 5}, {NULL}};
const Segment SGI67952[] = {{&S27231[44], 10}, {NULL}};
const Segment SGI67953[] = {{&S27231[44], 27}, {NULL}};
const Segment SGI67954[] = {{&S27231[30], 6}, {NULL}};
const Segment SGI67955[] = {{&S27231[121], 3}, {NULL}};
const Segment SGI67956[] = {{&S27231[85], 7}, {NULL}};
const Segment SGI67957[] = {{&S27231[141], 8}, {NULL}};
const Segment SGI67958[] = {{&S27233[44], 10}, {NULL}};
const Segment SGI67959[] = {{&S27233[44], 27}, {NULL}};
const Segment SGI67960[] = {{&S27233[30], 6}, {NULL}};
const Segment SGI67961[] = {{&S27233[85], 6}, {NULL}};
const Segment SGI67962[] = {{&S27234[44], 10}, {NULL}};
const Segment SGI67963[] = {{&S27234[44], 27}, {NULL}};
const Segment SGI67964[] = {{&S27234[30], 6}, {NULL}};
const Segment SGI67965[] = {{&S27234[88], 6}, {NULL}};
const Segment SGI67966[] = {{&S27262[44], 11}, {NULL}};
const Segment SGI67967[] = {{&S27262[44], 28}, {NULL}};
const Segment SGI67968[] = {{&S27262[30], 6}, {NULL}};
const Segment SGI67969[] = {{&S27262[120], 3}, {NULL}};
const Segment SGI67970[] = {{&S27262[86], 5}, {NULL}};
const Segment SGI67971[] = {{&S27263[44], 11}, {NULL}};
const Segment SGI67972[] = {{&S27263[44], 28}, {NULL}};
const Segment SGI67973[] = {{&S27263[30], 6}, {NULL}};
const Segment SGI67974[] = {{&S27263[120], 3}, {NULL}};
const Segment SGI67975[] = {{&S27263[86], 5}, {NULL}};
const Segment SGI67976[] = {{&S27264[38], 11}, {NULL}};
const Segment SGI67977[] = {{&S27264[31], 6}, {NULL}};
const Segment SGI67978[] = {{&S27269[44], 11}, {NULL}};
const Segment SGI67979[] = {{&S27269[44], 28}, {NULL}};
const Segment SGI67980[] = {{&S27269[30], 6}, {NULL}};
const Segment SGI67981[] = {{&S27269[120], 3}, {NULL}};
const Segment SGI67982[] = {{&S27269[86], 5}, {NULL}};
const Segment SGI67983[] = {{&S27270[38], 11}, {NULL}};
const Segment SGI67984[] = {{&S27270[31], 6}, {NULL}};
const Segment SGI67985[] = {{&S27278[38], 11}, {NULL}};
const Segment SGI67986[] = {{&S27278[31], 6}, {NULL}};
const Segment SGI67987[] = {{&S27279[44], 11}, {NULL}};
const Segment SGI67988[] = {{&S27279[44], 28}, {NULL}};
const Segment SGI67989[] = {{&S27279[30], 6}, {NULL}};
const Segment SGI67990[] = {{&S27279[120], 3}, {NULL}};
const Segment SGI67991[] = {{&S27279[86], 5}, {NULL}};
const Segment SGI67992[] = {{&S27280[36], 9}, {NULL}};
const Segment SGI67993[] = {{&S15083[0], 17}, {NULL}};
const Segment SGI67994[] = {{&S15180[68], 6}, {NULL}};
const Segment SGI67995[] = {{&S15180[0], 3}, {NULL}};
const Segment SGI67996[] = {{&S15198[44], 12}, {NULL}};
const Segment SGI67997[] = {{&S15198[44], 24}, {NULL}};
const Segment SGI67998[] = {{&S15198[30], 6}, {NULL}};
const Segment SGI67999[] = {{&S15198[31], 3}, {NULL}};
const Segment SGI68000[] = {{&S15198[82], 6}, {NULL}};
const Segment SGI68001[] = {{&S15198[0], 13}, {NULL}};
const Segment SGI68002[] = {{&S15412[44], 8}, {NULL}};
const Segment SGI68003[] = {{&S15412[44], 20}, {NULL}};
const Segment SGI68004[] = {{&S15412[30], 6}, {NULL}};
const Segment SGI68005[] = {{&S15412[31], 3}, {NULL}};
const Segment SGI68006[] = {{&S15412[78], 5}, {NULL}};
const Segment SGI68007[] = {{&S15412[0], 13}, {NULL}};
const Segment SGI68008[] = {{&S11291[44], 10}, {NULL}};
const Segment SGI68009[] = {{&S11291[44], 22}, {NULL}};
const Segment SGI68010[] = {{&S11291[30], 6}, {NULL}};
const Segment SGI68011[] = {{&S11291[80], 6}, {NULL}};
const Segment SGI68012[] = {{&S11292[44], 10}, {NULL}};
const Segment SGI68013[] = {{&S11292[44], 22}, {NULL}};
const Segment SGI68014[] = {{&S11292[30], 6}, {NULL}};
const Segment SGI68015[] = {{&S11292[31], 3}, {NULL}};
const Segment SGI68016[] = {{&S11292[80], 6}, {NULL}};
const Segment SGI68017[] = {{&S11293[38], 10}, {NULL}};
const Segment SGI68018[] = {{&S11293[31], 6}, {NULL}};
const Segment SGI68019[] = {{&S11294[44], 10}, {NULL}};
const Segment SGI68020[] = {{&S11294[44], 22}, {NULL}};
const Segment SGI68021[] = {{&S11294[30], 6}, {NULL}};
const Segment SGI68022[] = {{&S11294[31], 3}, {NULL}};
const Segment SGI68023[] = {{&S11294[80], 6}, {NULL}};
const Segment SGI68024[] = {{&S11296[44], 11}, {NULL}};
const Segment SGI68025[] = {{&S11296[44], 23}, {NULL}};
const Segment SGI68026[] = {{&S11296[30], 6}, {NULL}};
const Segment SGI68027[] = {{&S11296[31], 3}, {NULL}};
const Segment SGI68028[] = {{&S11296[81], 6}, {NULL}};
const Segment SGI68029[] = {{&S11297[44], 11}, {NULL}};
const Segment SGI68030[] = {{&S11297[44], 23}, {NULL}};
const Segment SGI68031[] = {{&S11297[30], 6}, {NULL}};
const Segment SGI68032[] = {{&S11297[31], 3}, {NULL}};
const Segment SGI68033[] = {{&S11297[81], 6}, {NULL}};
const Segment SGI68034[] = {{&S11298[44], 11}, {NULL}};
const Segment SGI68035[] = {{&S11298[44], 23}, {NULL}};
const Segment SGI68036[] = {{&S11298[30], 6}, {NULL}};
const Segment SGI68037[] = {{&S11298[31], 3}, {NULL}};
const Segment SGI68038[] = {{&S11298[81], 6}, {NULL}};
const Segment SGI68039[] = {{&S11300[44], 29}, {NULL}};
const Segment SGI68040[] = {{&S11300[44], 41}, {NULL}};
const Segment SGI68041[] = {{&S11300[30], 6}, {NULL}};
const Segment SGI68042[] = {{&S11300[31], 3}, {NULL}};
const Segment SGI68043[] = {{&S11300[99], 6}, {NULL}};
const Segment SGI68044[] = {{&S11301[44], 11}, {NULL}};
const Segment SGI68045[] = {{&S11301[44], 23}, {NULL}};
const Segment SGI68046[] = {{&S11301[30], 6}, {NULL}};
const Segment SGI68047[] = {{&S11301[31], 3}, {NULL}};
const Segment SGI68048[] = {{&S11301[81], 6}, {NULL}};
const Segment SGI68049[] = {{&S11303[44], 7}, {NULL}};
const Segment SGI68050[] = {{&S11303[44], 19}, {NULL}};
const Segment SGI68051[] = {{&S11303[30], 6}, {NULL}};
const Segment SGI68052[] = {{&S11303[31], 3}, {NULL}};
const Segment SGI68053[] = {{&S11303[77], 5}, {NULL}};
const Segment SGI68054[] = {{&S11305[44], 10}, {NULL}};
const Segment SGI68055[] = {{&S11305[44], 22}, {NULL}};
const Segment SGI68056[] = {{&S11305[30], 6}, {NULL}};
const Segment SGI68057[] = {{&S11305[83], 6}, {NULL}};
const Segment SGI68058[] = {{&S11306[38], 11}, {NULL}};
const Segment SGI68059[] = {{&S11306[31], 6}, {NULL}};
const Segment SGI68060[] = {{&S11307[38], 11}, {NULL}};
const Segment SGI68061[] = {{&S11307[31], 6}, {NULL}};
const Segment SGI68062[] = {{&S11308[38], 11}, {NULL}};
const Segment SGI68063[] = {{&S11308[31], 6}, {NULL}};
const Segment SGI68064[] = {{&S11310[44], 29}, {NULL}};
const Segment SGI68065[] = {{&S11310[44], 41}, {NULL}};
const Segment SGI68066[] = {{&S11310[30], 6}, {NULL}};
const Segment SGI68067[] = {{&S11310[31], 3}, {NULL}};
const Segment SGI68068[] = {{&S11310[99], 6}, {NULL}};
const Segment SGI68069[] = {{&S11314[44], 10}, {NULL}};
const Segment SGI68070[] = {{&S11314[44], 22}, {NULL}};
const Segment SGI68071[] = {{&S11314[30], 6}, {NULL}};
const Segment SGI68072[] = {{&S11314[83], 4}, {NULL}};
const Segment SGI68073[] = {{&S11314[83], 3}, {NULL}};
const Segment SGI68074[] = {{&S11315[42], 10}, {NULL}};
const Segment SGI68075[] = {{&S11315[30], 6}, {NULL}};
const Segment SGI68076[] = {{&S11315[31], 3}, {NULL}};
const Segment SGI68077[] = {{&S11315[78], 6}, {NULL}};
const Segment SGI68078[] = {{&S11319[44], 7}, {NULL}};
const Segment SGI68079[] = {{&S11319[44], 18}, {NULL}};
const Segment SGI68080[] = {{&S11319[30], 6}, {NULL}};
const Segment SGI68081[] = {{&S11319[31], 3}, {NULL}};
const Segment SGI68082[] = {{&S11319[76], 5}, {NULL}};
const Segment SGI68083[] = {{&S27977[38], 14}, {NULL}};
const Segment SGI68084[] = {{&S27977[31], 6}, {NULL}};
const Segment SGI68085[] = {{&S27978[44], 14}, {NULL}};
const Segment SGI68086[] = {{&S27978[44], 27}, {NULL}};
const Segment SGI68087[] = {{&S27978[30], 6}, {NULL}};
const Segment SGI68088[] = {{&S27978[119], 3}, {NULL}};
const Segment SGI68089[] = {{&S27978[85], 5}, {NULL}};
const Segment SGI68090[] = {{&S11342[44], 10}, {NULL}};
const Segment SGI68091[] = {{&S11342[44], 29}, {NULL}};
const Segment SGI68092[] = {{&S11342[30], 6}, {NULL}};
const Segment SGI68093[] = {{&S11342[31], 3}, {NULL}};
const Segment SGI68094[] = {{&S11342[87], 6}, {NULL}};
const Segment SGI68095[] = {{&S28212[44], 10}, {NULL}};
const Segment SGI68096[] = {{&S28212[44], 27}, {NULL}};
const Segment SGI68097[] = {{&S28212[30], 6}, {NULL}};
const Segment SGI68098[] = {{&S28212[119], 3}, {NULL}};
const Segment SGI68099[] = {{&S28212[85], 5}, {NULL}};
const Segment SGI68100[] = {{&S28221[38], 10}, {NULL}};
const Segment SGI68101[] = {{&S28221[31], 6}, {NULL}};
const Segment SGI68102[] = {{&S28223[43], 10}, {NULL}};
const Segment SGI68103[] = {{&S28223[43], 27}, {NULL}};
const Segment SGI68104[] = {{&S28223[30], 5}, {NULL}};
const Segment SGI68105[] = {{&S28223[118], 3}, {NULL}};
const Segment SGI68106[] = {{&S28223[84], 5}, {NULL}};
const Segment SGI68107[] = {{&S28224[44], 9}, {NULL}};
const Segment SGI68108[] = {{&S28224[44], 26}, {NULL}};
const Segment SGI68109[] = {{&S28224[30], 6}, {NULL}};
const Segment SGI68110[] = {{&S28224[118], 3}, {NULL}};
const Segment SGI68111[] = {{&S28224[84], 5}, {NULL}};
const Segment SGI68112[] = {{&S28225[44], 10}, {NULL}};
const Segment SGI68113[] = {{&S28225[44], 27}, {NULL}};
const Segment SGI68114[] = {{&S28225[30], 6}, {NULL}};
const Segment SGI68115[] = {{&S28225[119], 3}, {NULL}};
const Segment SGI68116[] = {{&S28225[85], 5}, {NULL}};
const Segment SGI68117[] = {{&S28226[44], 9}, {NULL}};
const Segment SGI68118[] = {{&S28226[44], 26}, {NULL}};
const Segment SGI68119[] = {{&S28226[30], 6}, {NULL}};
const Segment SGI68120[] = {{&S28226[118], 3}, {NULL}};
const Segment SGI68121[] = {{&S28226[84], 5}, {NULL}};
const Segment SGI68122[] = {{&S28227[43], 10}, {NULL}};
const Segment SGI68123[] = {{&S28227[43], 27}, {NULL}};
const Segment SGI68124[] = {{&S28227[118], 3}, {NULL}};
const Segment SGI68125[] = {{&S28227[84], 5}, {NULL}};
const Segment SGI68126[] = {{&S28228[44], 10}, {NULL}};
const Segment SGI68127[] = {{&S28228[44], 27}, {NULL}};
const Segment SGI68128[] = {{&S28228[30], 6}, {NULL}};
const Segment SGI68129[] = {{&S28228[85], 6}, {NULL}};
const Segment SGI68130[] = {{&S28229[44], 10}, {NULL}};
const Segment SGI68131[] = {{&S28229[44], 27}, {NULL}};
const Segment SGI68132[] = {{&S28229[30], 6}, {NULL}};
const Segment SGI68133[] = {{&S28229[121], 3}, {NULL}};
const Segment SGI68134[] = {{&S28229[85], 7}, {NULL}};
const Segment SGI68135[] = {{&S28229[141], 8}, {NULL}};
const Segment SGI68136[] = {{&S28231[44], 10}, {NULL}};
const Segment SGI68137[] = {{&S28231[44], 33}, {NULL}};
const Segment SGI68138[] = {{&S28231[30], 6}, {NULL}};
const Segment SGI68139[] = {{&S28231[125], 3}, {NULL}};
const Segment SGI68140[] = {{&S28231[91], 5}, {NULL}};
const Segment SGI68141[] = {{&S28232[42], 10}, {NULL}};
const Segment SGI68142[] = {{&S28232[30], 6}, {NULL}};
const Segment SGI68143[] = {{&S28232[117], 3}, {NULL}};
const Segment SGI68144[] = {{&S28232[83], 5}, {NULL}};
const Segment SGI68145[] = {{&S28423[44], 29}, {NULL}};
const Segment SGI68146[] = {{&S28423[44], 46}, {NULL}};
const Segment SGI68147[] = {{&S28423[30], 6}, {NULL}};
const Segment SGI68148[] = {{&S28423[138], 3}, {NULL}};
const Segment SGI68149[] = {{&S28423[104], 5}, {NULL}};
const Segment SGI68150[] = {{&S28424[44], 10}, {NULL}};
const Segment SGI68151[] = {{&S28424[44], 27}, {NULL}};
const Segment SGI68152[] = {{&S28424[30], 6}, {NULL}};
const Segment SGI68153[] = {{&S28424[119], 3}, {NULL}};
const Segment SGI68154[] = {{&S28424[85], 5}, {NULL}};
const Segment SGI68155[] = {{&S28425[44], 11}, {NULL}};
const Segment SGI68156[] = {{&S28425[44], 28}, {NULL}};
const Segment SGI68157[] = {{&S28425[30], 6}, {NULL}};
const Segment SGI68158[] = {{&S28425[120], 3}, {NULL}};
const Segment SGI68159[] = {{&S28425[86], 5}, {NULL}};
const Segment SGI68160[] = {{&S28426[44], 11}, {NULL}};
const Segment SGI68161[] = {{&S28426[44], 28}, {NULL}};
const Segment SGI68162[] = {{&S28426[30], 6}, {NULL}};
const Segment SGI68163[] = {{&S28426[120], 3}, {NULL}};
const Segment SGI68164[] = {{&S28426[86], 5}, {NULL}};
const Segment SGI68165[] = {{&S28427[38], 10}, {NULL}};
const Segment SGI68166[] = {{&S28427[31], 6}, {NULL}};
const Segment SGI68167[] = {{&S28428[44], 11}, {NULL}};
const Segment SGI68168[] = {{&S28428[44], 28}, {NULL}};
const Segment SGI68169[] = {{&S28428[30], 6}, {NULL}};
const Segment SGI68170[] = {{&S28428[120], 3}, {NULL}};
const Segment SGI68171[] = {{&S28428[86], 5}, {NULL}};
const Segment SGI68172[] = {{&S28429[44], 29}, {NULL}};
const Segment SGI68173[] = {{&S28429[44], 46}, {NULL}};
const Segment SGI68174[] = {{&S28429[30], 6}, {NULL}};
const Segment SGI68175[] = {{&S28429[138], 3}, {NULL}};
const Segment SGI68176[] = {{&S28429[104], 5}, {NULL}};
const Segment SGI68177[] = {{&S28430[44], 28}, {NULL}};
const Segment SGI68178[] = {{&S28430[44], 45}, {NULL}};
const Segment SGI68179[] = {{&S28430[30], 6}, {NULL}};
const Segment SGI68180[] = {{&S28430[137], 3}, {NULL}};
const Segment SGI68181[] = {{&S28430[103], 5}, {NULL}};
const Segment SGI68182[] = {{&S28431[44], 29}, {NULL}};
const Segment SGI68183[] = {{&S28431[44], 46}, {NULL}};
const Segment SGI68184[] = {{&S28431[30], 6}, {NULL}};
const Segment SGI68185[] = {{&S28431[138], 3}, {NULL}};
const Segment SGI68186[] = {{&S28431[104], 5}, {NULL}};
const Segment SGI68187[] = {{&S28432[44], 29}, {NULL}};
const Segment SGI68188[] = {{&S28432[44], 46}, {NULL}};
const Segment SGI68189[] = {{&S28432[30], 6}, {NULL}};
const Segment SGI68190[] = {{&S28432[138], 3}, {NULL}};
const Segment SGI68191[] = {{&S28432[104], 5}, {NULL}};
const Segment SGI68192[] = {{&S28433[44], 28}, {NULL}};
const Segment SGI68193[] = {{&S28433[44], 45}, {NULL}};
const Segment SGI68194[] = {{&S28433[30], 6}, {NULL}};
const Segment SGI68195[] = {{&S28433[137], 3}, {NULL}};
const Segment SGI68196[] = {{&S28433[103], 5}, {NULL}};
const Segment SGI68197[] = {{&S28434[44], 28}, {NULL}};
const Segment SGI68198[] = {{&S28434[44], 45}, {NULL}};
const Segment SGI68199[] = {{&S28434[30], 6}, {NULL}};
const Segment SGI68200[] = {{&S28434[137], 3}, {NULL}};
const Segment SGI68201[] = {{&S28434[103], 5}, {NULL}};
const Segment SGI68202[] = {{&S28435[44], 29}, {NULL}};
const Segment SGI68203[] = {{&S28435[44], 46}, {NULL}};
const Segment SGI68204[] = {{&S28435[30], 6}, {NULL}};
const Segment SGI68205[] = {{&S28435[138], 3}, {NULL}};
const Segment SGI68206[] = {{&S28435[104], 5}, {NULL}};
const Segment SGI68207[] = {{&S28436[44], 10}, {NULL}};
const Segment SGI68208[] = {{&S28436[44], 27}, {NULL}};
const Segment SGI68209[] = {{&S28436[30], 6}, {NULL}};
const Segment SGI68210[] = {{&S28436[88], 6}, {NULL}};
const Segment SGI68211[] = {{&S28437[38], 11}, {NULL}};
const Segment SGI68212[] = {{&S28437[31], 6}, {NULL}};
const Segment SGI68213[] = {{&S28438[42], 10}, {NULL}};
const Segment SGI68214[] = {{&S28438[30], 6}, {NULL}};
const Segment SGI68215[] = {{&S28438[117], 3}, {NULL}};
const Segment SGI68216[] = {{&S28438[83], 5}, {NULL}};
const Segment SGI68217[] = {{&S28439[38], 11}, {NULL}};
const Segment SGI68218[] = {{&S28439[31], 6}, {NULL}};
const Segment SGI68219[] = {{&S28440[44], 29}, {NULL}};
const Segment SGI68220[] = {{&S28440[44], 46}, {NULL}};
const Segment SGI68221[] = {{&S28440[30], 6}, {NULL}};
const Segment SGI68222[] = {{&S28440[138], 3}, {NULL}};
const Segment SGI68223[] = {{&S28440[104], 5}, {NULL}};
const Segment SGI68224[] = {{&S28441[56], 10}, {NULL}};
const Segment SGI68225[] = {{&S28441[56], 27}, {NULL}};
const Segment SGI68226[] = {{&S28441[30], 18}, {NULL}};
const Segment SGI68227[] = {{&S28441[131], 3}, {NULL}};
const Segment SGI68228[] = {{&S28441[97], 5}, {NULL}};
const Segment SGI68229[] = {{&S28442[44], 28}, {NULL}};
const Segment SGI68230[] = {{&S28442[44], 45}, {NULL}};
const Segment SGI68231[] = {{&S28442[30], 6}, {NULL}};
const Segment SGI68232[] = {{&S28442[137], 3}, {NULL}};
const Segment SGI68233[] = {{&S28442[103], 5}, {NULL}};
const Segment SGI68234[] = {{&S28587[44], 7}, {NULL}};
const Segment SGI68235[] = {{&S28587[44], 29}, {NULL}};
const Segment SGI68236[] = {{&S28587[30], 6}, {NULL}};
const Segment SGI68237[] = {{&S28587[121], 3}, {NULL}};
const Segment SGI68238[] = {{&S28587[87], 5}, {NULL}};
const Segment SGI68239[] = {{&S28588[44], 7}, {NULL}};
const Segment SGI68240[] = {{&S28588[44], 33}, {NULL}};
const Segment SGI68241[] = {{&S28588[30], 6}, {NULL}};
const Segment SGI68242[] = {{&S28588[125], 3}, {NULL}};
const Segment SGI68243[] = {{&S28588[91], 5}, {NULL}};
const Segment SGI68244[] = {{&S28591[44], 11}, {NULL}};
const Segment SGI68245[] = {{&S28591[44], 36}, {NULL}};
const Segment SGI68246[] = {{&S28591[30], 6}, {NULL}};
const Segment SGI68247[] = {{&S28591[128], 3}, {NULL}};
const Segment SGI68248[] = {{&S28591[94], 5}, {NULL}};
const Segment SGI68249[] = {{&S28592[44], 7}, {NULL}};
const Segment SGI68250[] = {{&S28592[44], 32}, {NULL}};
const Segment SGI68251[] = {{&S28592[30], 6}, {NULL}};
const Segment SGI68252[] = {{&S28592[124], 3}, {NULL}};
const Segment SGI68253[] = {{&S28592[90], 5}, {NULL}};
const Segment SGI68254[] = {{&S28593[44], 7}, {NULL}};
const Segment SGI68255[] = {{&S28593[44], 32}, {NULL}};
const Segment SGI68256[] = {{&S28593[30], 6}, {NULL}};
const Segment SGI68257[] = {{&S28593[90], 6}, {NULL}};
const Segment SGI68258[] = {{&S28595[44], 11}, {NULL}};
const Segment SGI68259[] = {{&S28595[44], 38}, {NULL}};
const Segment SGI68260[] = {{&S28595[30], 6}, {NULL}};
const Segment SGI68261[] = {{&S28595[130], 3}, {NULL}};
const Segment SGI68262[] = {{&S28595[96], 5}, {NULL}};
const Segment SGI68263[] = {{&S28596[44], 11}, {NULL}};
const Segment SGI68264[] = {{&S28596[44], 36}, {NULL}};
const Segment SGI68265[] = {{&S28596[30], 6}, {NULL}};
const Segment SGI68266[] = {{&S28596[128], 3}, {NULL}};
const Segment SGI68267[] = {{&S28596[94], 5}, {NULL}};
const Segment SGI68268[] = {{&S28601[44], 11}, {NULL}};
const Segment SGI68269[] = {{&S28601[44], 37}, {NULL}};
const Segment SGI68270[] = {{&S28601[30], 6}, {NULL}};
const Segment SGI68271[] = {{&S28601[129], 3}, {NULL}};
const Segment SGI68272[] = {{&S28601[95], 5}, {NULL}};
const Segment SGI68273[] = {{&S28603[44], 11}, {NULL}};
const Segment SGI68274[] = {{&S28603[44], 41}, {NULL}};
const Segment SGI68275[] = {{&S28603[30], 6}, {NULL}};
const Segment SGI68276[] = {{&S28603[133], 3}, {NULL}};
const Segment SGI68277[] = {{&S28603[99], 5}, {NULL}};
const Segment SGI68278[] = {{&S28604[44], 11}, {NULL}};
const Segment SGI68279[] = {{&S28604[44], 41}, {NULL}};
const Segment SGI68280[] = {{&S28604[30], 6}, {NULL}};
const Segment SGI68281[] = {{&S28604[133], 3}, {NULL}};
const Segment SGI68282[] = {{&S28604[99], 5}, {NULL}};
const Segment SGI68283[] = {{&S28606[44], 11}, {NULL}};
const Segment SGI68284[] = {{&S28606[44], 34}, {NULL}};
const Segment SGI68285[] = {{&S28606[30], 6}, {NULL}};
const Segment SGI68286[] = {{&S28606[126], 3}, {NULL}};
const Segment SGI68287[] = {{&S28606[92], 5}, {NULL}};
const Segment SGI68288[] = {{&S28607[44], 7}, {NULL}};
const Segment SGI68289[] = {{&S28607[44], 32}, {NULL}};
const Segment SGI68290[] = {{&S28607[30], 6}, {NULL}};
const Segment SGI68291[] = {{&S28607[124], 3}, {NULL}};
const Segment SGI68292[] = {{&S28607[90], 5}, {NULL}};
const Segment SGI68293[] = {{&S28608[44], 7}, {NULL}};
const Segment SGI68294[] = {{&S28608[44], 34}, {NULL}};
const Segment SGI68295[] = {{&S28608[30], 6}, {NULL}};
const Segment SGI68296[] = {{&S28608[126], 3}, {NULL}};
const Segment SGI68297[] = {{&S28608[92], 5}, {NULL}};
const Segment SGI68298[] = {{&S28615[44], 8}, {NULL}};
const Segment SGI68299[] = {{&S28615[44], 35}, {NULL}};
const Segment SGI68300[] = {{&S28615[30], 6}, {NULL}};
const Segment SGI68301[] = {{&S28615[127], 3}, {NULL}};
const Segment SGI68302[] = {{&S28615[93], 5}, {NULL}};
const Segment SGI68303[] = {{&S28616[38], 7}, {NULL}};
const Segment SGI68304[] = {{&S28616[31], 6}, {NULL}};
const Segment SGI68305[] = {{&S28618[44], 7}, {NULL}};
const Segment SGI68306[] = {{&S28618[44], 37}, {NULL}};
const Segment SGI68307[] = {{&S28618[30], 6}, {NULL}};
const Segment SGI68308[] = {{&S28618[129], 3}, {NULL}};
const Segment SGI68309[] = {{&S28618[95], 5}, {NULL}};
const Segment SGI68310[] = {{&S28625[44], 11}, {NULL}};
const Segment SGI68311[] = {{&S28625[44], 38}, {NULL}};
const Segment SGI68312[] = {{&S28625[30], 6}, {NULL}};
const Segment SGI68313[] = {{&S28625[130], 3}, {NULL}};
const Segment SGI68314[] = {{&S28625[96], 5}, {NULL}};
const Segment SGI68315[] = {{&S28630[44], 7}, {NULL}};
const Segment SGI68316[] = {{&S28630[44], 30}, {NULL}};
const Segment SGI68317[] = {{&S28630[30], 6}, {NULL}};
const Segment SGI68318[] = {{&S28630[122], 3}, {NULL}};
const Segment SGI68319[] = {{&S28630[88], 5}, {NULL}};
const Segment SGI68320[] = {{&S28631[44], 7}, {NULL}};
const Segment SGI68321[] = {{&S28631[44], 34}, {NULL}};
const Segment SGI68322[] = {{&S28631[30], 6}, {NULL}};
const Segment SGI68323[] = {{&S28631[126], 3}, {NULL}};
const Segment SGI68324[] = {{&S28631[92], 5}, {NULL}};
const Segment SGI68325[] = {{&S28632[44], 7}, {NULL}};
const Segment SGI68326[] = {{&S28632[44], 33}, {NULL}};
const Segment SGI68327[] = {{&S28632[30], 6}, {NULL}};
const Segment SGI68328[] = {{&S28632[125], 3}, {NULL}};
const Segment SGI68329[] = {{&S28632[91], 5}, {NULL}};
const Segment SGI68330[] = {{&S28633[44], 7}, {NULL}};
const Segment SGI68331[] = {{&S28633[44], 37}, {NULL}};
const Segment SGI68332[] = {{&S28633[30], 6}, {NULL}};
const Segment SGI68333[] = {{&S28633[129], 3}, {NULL}};
const Segment SGI68334[] = {{&S28633[95], 5}, {NULL}};
const Segment SGI68335[] = {{&S28634[44], 8}, {NULL}};
const Segment SGI68336[] = {{&S28634[44], 35}, {NULL}};
const Segment SGI68337[] = {{&S28634[30], 6}, {NULL}};
const Segment SGI68338[] = {{&S28634[127], 3}, {NULL}};
const Segment SGI68339[] = {{&S28634[93], 5}, {NULL}};
const Segment SGI68340[] = {{&S28635[44], 11}, {NULL}};
const Segment SGI68341[] = {{&S28635[44], 41}, {NULL}};
const Segment SGI68342[] = {{&S28635[30], 6}, {NULL}};
const Segment SGI68343[] = {{&S28635[133], 3}, {NULL}};
const Segment SGI68344[] = {{&S28635[99], 5}, {NULL}};
const Segment SGI68345[] = {{&S28636[44], 11}, {NULL}};
const Segment SGI68346[] = {{&S28636[44], 38}, {NULL}};
const Segment SGI68347[] = {{&S28636[30], 6}, {NULL}};
const Segment SGI68348[] = {{&S28636[130], 3}, {NULL}};
const Segment SGI68349[] = {{&S28636[96], 5}, {NULL}};
const Segment SGI68350[] = {{&S28637[44], 7}, {NULL}};
const Segment SGI68351[] = {{&S28637[44], 32}, {NULL}};
const Segment SGI68352[] = {{&S28637[30], 6}, {NULL}};
const Segment SGI68353[] = {{&S28637[124], 3}, {NULL}};
const Segment SGI68354[] = {{&S28637[90], 5}, {NULL}};
const Segment SGI68355[] = {{&S28639[44], 11}, {NULL}};
const Segment SGI68356[] = {{&S28639[44], 33}, {NULL}};
const Segment SGI68357[] = {{&S28639[30], 6}, {NULL}};
const Segment SGI68358[] = {{&S28639[125], 3}, {NULL}};
const Segment SGI68359[] = {{&S28639[91], 5}, {NULL}};
const Segment SGI68360[] = {{&S28644[44], 11}, {NULL}};
const Segment SGI68361[] = {{&S28644[44], 42}, {NULL}};
const Segment SGI68362[] = {{&S28644[30], 6}, {NULL}};
const Segment SGI68363[] = {{&S28644[134], 3}, {NULL}};
const Segment SGI68364[] = {{&S28644[100], 5}, {NULL}};
const Segment SGI68365[] = {{&S28645[44], 11}, {NULL}};
const Segment SGI68366[] = {{&S28645[44], 38}, {NULL}};
const Segment SGI68367[] = {{&S28645[30], 6}, {NULL}};
const Segment SGI68368[] = {{&S28645[130], 3}, {NULL}};
const Segment SGI68369[] = {{&S28645[96], 5}, {NULL}};
const Segment SGI68370[] = {{&S28646[44], 7}, {NULL}};
const Segment SGI68371[] = {{&S28646[44], 33}, {NULL}};
const Segment SGI68372[] = {{&S28646[30], 6}, {NULL}};
const Segment SGI68373[] = {{&S28646[94], 4}, {NULL}};
const Segment SGI68374[] = {{&S28646[94], 3}, {NULL}};
const Segment SGI68375[] = {{&S28648[44], 11}, {NULL}};
const Segment SGI68376[] = {{&S28648[44], 38}, {NULL}};
const Segment SGI68377[] = {{&S28648[30], 6}, {NULL}};
const Segment SGI68378[] = {{&S28648[130], 3}, {NULL}};
const Segment SGI68379[] = {{&S28648[96], 5}, {NULL}};
const Segment SGI68380[] = {{&S28649[44], 11}, {NULL}};
const Segment SGI68381[] = {{&S28649[44], 37}, {NULL}};
const Segment SGI68382[] = {{&S28649[30], 6}, {NULL}};
const Segment SGI68383[] = {{&S28649[129], 3}, {NULL}};
const Segment SGI68384[] = {{&S28649[95], 5}, {NULL}};
const Segment SGI68385[] = {{&S28650[44], 7}, {NULL}};
const Segment SGI68386[] = {{&S28650[44], 29}, {NULL}};
const Segment SGI68387[] = {{&S28650[30], 6}, {NULL}};
const Segment SGI68388[] = {{&S28650[121], 3}, {NULL}};
const Segment SGI68389[] = {{&S28650[87], 5}, {NULL}};
const Segment SGI68390[] = {{&S28651[44], 11}, {NULL}};
const Segment SGI68391[] = {{&S28651[44], 33}, {NULL}};
const Segment SGI68392[] = {{&S28651[30], 6}, {NULL}};
const Segment SGI68393[] = {{&S28651[125], 3}, {NULL}};
const Segment SGI68394[] = {{&S28651[91], 5}, {NULL}};
const Segment SGI68395[] = {{&S28652[44], 11}, {NULL}};
const Segment SGI68396[] = {{&S28652[44], 33}, {NULL}};
const Segment SGI68397[] = {{&S28652[30], 6}, {NULL}};
const Segment SGI68398[] = {{&S28652[125], 3}, {NULL}};
const Segment SGI68399[] = {{&S28652[91], 5}, {NULL}};
const Segment SGI68400[] = {{&S28655[44], 7}, {NULL}};
const Segment SGI68401[] = {{&S28655[44], 38}, {NULL}};
const Segment SGI68402[] = {{&S28655[30], 6}, {NULL}};
const Segment SGI68403[] = {{&S28655[130], 3}, {NULL}};
const Segment SGI68404[] = {{&S28655[96], 5}, {NULL}};
const Segment SGI68405[] = {{&S28656[44], 11}, {NULL}};
const Segment SGI68406[] = {{&S28656[44], 33}, {NULL}};
const Segment SGI68407[] = {{&S28656[30], 6}, {NULL}};
const Segment SGI68408[] = {{&S28656[125], 3}, {NULL}};
const Segment SGI68409[] = {{&S28656[91], 5}, {NULL}};
const Segment SGI68410[] = {{&S28657[44], 7}, {NULL}};
const Segment SGI68411[] = {{&S28657[44], 32}, {NULL}};
const Segment SGI68412[] = {{&S28657[30], 6}, {NULL}};
const Segment SGI68413[] = {{&S28657[124], 3}, {NULL}};
const Segment SGI68414[] = {{&S28657[90], 5}, {NULL}};
const Segment SGI68415[] = {{&S28660[44], 7}, {NULL}};
const Segment SGI68416[] = {{&S28660[44], 34}, {NULL}};
const Segment SGI68417[] = {{&S28660[30], 6}, {NULL}};
const Segment SGI68418[] = {{&S28660[126], 3}, {NULL}};
const Segment SGI68419[] = {{&S28660[92], 5}, {NULL}};
const Segment SGI68420[] = {{&S28661[44], 7}, {NULL}};
const Segment SGI68421[] = {{&S28661[44], 37}, {NULL}};
const Segment SGI68422[] = {{&S28661[30], 6}, {NULL}};
const Segment SGI68423[] = {{&S28661[129], 3}, {NULL}};
const Segment SGI68424[] = {{&S28661[95], 5}, {NULL}};
const Segment SGI68425[] = {{&S28662[44], 7}, {NULL}};
const Segment SGI68426[] = {{&S28662[44], 33}, {NULL}};
const Segment SGI68427[] = {{&S28662[30], 6}, {NULL}};
const Segment SGI68428[] = {{&S28662[94], 6}, {NULL}};
const Segment SGI68429[] = {{&S28664[44], 11}, {NULL}};
const Segment SGI68430[] = {{&S28664[44], 37}, {NULL}};
const Segment SGI68431[] = {{&S28664[30], 6}, {NULL}};
const Segment SGI68432[] = {{&S28664[129], 3}, {NULL}};
const Segment SGI68433[] = {{&S28664[95], 5}, {NULL}};
const Segment SGI68434[] = {{&S28665[44], 11}, {NULL}};
const Segment SGI68435[] = {{&S28665[44], 34}, {NULL}};
const Segment SGI68436[] = {{&S28665[30], 6}, {NULL}};
const Segment SGI68437[] = {{&S28665[126], 3}, {NULL}};
const Segment SGI68438[] = {{&S28665[92], 5}, {NULL}};
const Segment SGI68439[] = {{&S28666[44], 7}, {NULL}};
const Segment SGI68440[] = {{&S28666[44], 33}, {NULL}};
const Segment SGI68441[] = {{&S28666[30], 6}, {NULL}};
const Segment SGI68442[] = {{&S28666[94], 4}, {NULL}};
const Segment SGI68443[] = {{&S28666[94], 3}, {NULL}};
const Segment SGI68444[] = {{&S28669[44], 7}, {NULL}};
const Segment SGI68445[] = {{&S28669[44], 34}, {NULL}};
const Segment SGI68446[] = {{&S28669[30], 6}, {NULL}};
const Segment SGI68447[] = {{&S28669[126], 3}, {NULL}};
const Segment SGI68448[] = {{&S28669[92], 5}, {NULL}};
const Segment SGI68449[] = {{&S28670[44], 11}, {NULL}};
const Segment SGI68450[] = {{&S28670[44], 38}, {NULL}};
const Segment SGI68451[] = {{&S28670[30], 6}, {NULL}};
const Segment SGI68452[] = {{&S28670[130], 3}, {NULL}};
const Segment SGI68453[] = {{&S28670[96], 5}, {NULL}};
const Segment SGI68454[] = {{&S28671[44], 11}, {NULL}};
const Segment SGI68455[] = {{&S28671[44], 33}, {NULL}};
const Segment SGI68456[] = {{&S28671[30], 6}, {NULL}};
const Segment SGI68457[] = {{&S28671[125], 3}, {NULL}};
const Segment SGI68458[] = {{&S28671[91], 5}, {NULL}};
const Segment SGI68459[] = {{&S28679[44], 10}, {NULL}};
const Segment SGI68460[] = {{&S28679[44], 27}, {NULL}};
const Segment SGI68461[] = {{&S28679[30], 6}, {NULL}};
const Segment SGI68462[] = {{&S28679[119], 3}, {NULL}};
const Segment SGI68463[] = {{&S28679[85], 5}, {NULL}};
const Segment SGI68464[] = {{&S28680[44], 13}, {NULL}};
const Segment SGI68465[] = {{&S28680[44], 22}, {NULL}};
const Segment SGI68466[] = {{&S28680[30], 6}, {NULL}};
const Segment SGI68467[] = {{&S28680[114], 3}, {NULL}};
const Segment SGI68468[] = {{&S28680[80], 5}, {NULL}};
const Segment SGI68469[] = {{&S28681[44], 11}, {NULL}};
const Segment SGI68470[] = {{&S28681[44], 28}, {NULL}};
const Segment SGI68471[] = {{&S28681[30], 6}, {NULL}};
const Segment SGI68472[] = {{&S28681[120], 3}, {NULL}};
const Segment SGI68473[] = {{&S28681[86], 5}, {NULL}};
const Segment SGI68474[] = {{&S28682[44], 13}, {NULL}};
const Segment SGI68475[] = {{&S28682[44], 22}, {NULL}};
const Segment SGI68476[] = {{&S28682[30], 6}, {NULL}};
const Segment SGI68477[] = {{&S28682[114], 3}, {NULL}};
const Segment SGI68478[] = {{&S28682[80], 5}, {NULL}};
const Segment SGI68479[] = {{&S28684[38], 10}, {NULL}};
const Segment SGI68480[] = {{&S28684[31], 6}, {NULL}};
const Segment SGI68481[] = {{&S28685[38], 11}, {NULL}};
const Segment SGI68482[] = {{&S28685[31], 6}, {NULL}};
const Segment SGI68483[] = {{&S28686[44], 10}, {NULL}};
const Segment SGI68484[] = {{&S28686[44], 27}, {NULL}};
const Segment SGI68485[] = {{&S28686[30], 6}, {NULL}};
const Segment SGI68486[] = {{&S28686[121], 3}, {NULL}};
const Segment SGI68487[] = {{&S28686[85], 7}, {NULL}};
const Segment SGI68488[] = {{&S28686[141], 8}, {NULL}};
const Segment SGI68489[] = {{&S28687[44], 13}, {NULL}};
const Segment SGI68490[] = {{&S28687[44], 22}, {NULL}};
const Segment SGI68491[] = {{&S28687[30], 6}, {NULL}};
const Segment SGI68492[] = {{&S28687[114], 3}, {NULL}};
const Segment SGI68493[] = {{&S28687[80], 5}, {NULL}};
const Segment SGI68494[] = {{&S28688[42], 13}, {NULL}};
const Segment SGI68495[] = {{&S28688[30], 6}, {NULL}};
const Segment SGI68496[] = {{&S28688[112], 3}, {NULL}};
const Segment SGI68497[] = {{&S28688[78], 5}, {NULL}};
const Segment SGI68498[] = {{&S28771[44], 18}, {NULL}};
const Segment SGI68499[] = {{&S28771[44], 35}, {NULL}};
const Segment SGI68500[] = {{&S28771[30], 6}, {NULL}};
const Segment SGI68501[] = {{&S28771[127], 3}, {NULL}};
const Segment SGI68502[] = {{&S28771[93], 5}, {NULL}};
const Segment SGI68503[] = {{&S28772[38], 18}, {NULL}};
const Segment SGI68504[] = {{&S28772[31], 6}, {NULL}};
const Segment SGI68505[] = {{&S28773[44], 18}, {NULL}};
const Segment SGI68506[] = {{&S28773[44], 53}, {NULL}};
const Segment SGI68507[] = {{&S28773[30], 6}, {NULL}};
const Segment SGI68508[] = {{&S28773[145], 3}, {NULL}};
const Segment SGI68509[] = {{&S28773[111], 5}, {NULL}};
const Segment SGI68510[] = {{&S28774[42], 18}, {NULL}};
const Segment SGI68511[] = {{&S28774[30], 6}, {NULL}};
const Segment SGI68512[] = {{&S28774[125], 3}, {NULL}};
const Segment SGI68513[] = {{&S28774[91], 5}, {NULL}};
const Segment SGI68514[] = {{&S28775[44], 18}, {NULL}};
const Segment SGI68515[] = {{&S28775[44], 35}, {NULL}};
const Segment SGI68516[] = {{&S28775[30], 6}, {NULL}};
const Segment SGI68517[] = {{&S28775[127], 3}, {NULL}};
const Segment SGI68518[] = {{&S28775[93], 5}, {NULL}};
const Segment SGI68519[] = {{&S18034[42], 22}, {NULL}};
const Segment SGI68520[] = {{&S18034[42], 34}, {NULL}};
const Segment SGI68521[] = {{&S18034[30], 4}, {NULL}};
const Segment SGI68522[] = {{&S18034[31], 3}, {NULL}};
const Segment SGI68523[] = {{&S18034[90], 6}, {NULL}};
const Segment SGI68524[] = {{&S12302[38], 11}, {NULL}};
const Segment SGI68525[] = {{&S12302[31], 6}, {NULL}};
const Segment SGI68526[] = {{&S12303[44], 11}, {NULL}};
const Segment SGI68527[] = {{&S12303[44], 23}, {NULL}};
const Segment SGI68528[] = {{&S12303[30], 6}, {NULL}};
const Segment SGI68529[] = {{&S12303[31], 3}, {NULL}};
const Segment SGI68530[] = {{&S12303[81], 6}, {NULL}};
const Segment SGI68531[] = {{&S12304[44], 22}, {NULL}};
const Segment SGI68532[] = {{&S12304[44], 34}, {NULL}};
const Segment SGI68533[] = {{&S12304[30], 6}, {NULL}};
const Segment SGI68534[] = {{&S12304[31], 3}, {NULL}};
const Segment SGI68535[] = {{&S12304[92], 6}, {NULL}};
const Segment SGI68536[] = {{&S12305[44], 22}, {NULL}};
const Segment SGI68537[] = {{&S12305[44], 34}, {NULL}};
const Segment SGI68538[] = {{&S12305[30], 6}, {NULL}};
const Segment SGI68539[] = {{&S12305[31], 3}, {NULL}};
const Segment SGI68540[] = {{&S12305[92], 6}, {NULL}};
const Segment SGI68541[] = {{&S12306[44], 20}, {NULL}};
const Segment SGI68542[] = {{&S12306[44], 32}, {NULL}};
const Segment SGI68543[] = {{&S12306[30], 6}, {NULL}};
const Segment SGI68544[] = {{&S12306[31], 3}, {NULL}};
const Segment SGI68545[] = {{&S12306[90], 6}, {NULL}};
const Segment SGI68546[] = {{&S12307[44], 22}, {NULL}};
const Segment SGI68547[] = {{&S12307[44], 34}, {NULL}};
const Segment SGI68548[] = {{&S12307[30], 6}, {NULL}};
const Segment SGI68549[] = {{&S12307[31], 3}, {NULL}};
const Segment SGI68550[] = {{&S12307[92], 6}, {NULL}};
const Segment SGI68551[] = {{&S28918[44], 11}, {NULL}};
const Segment SGI68552[] = {{&S28918[44], 28}, {NULL}};
const Segment SGI68553[] = {{&S28918[30], 6}, {NULL}};
const Segment SGI68554[] = {{&S28918[120], 3}, {NULL}};
const Segment SGI68555[] = {{&S28918[86], 5}, {NULL}};
const Segment SGI68556[] = {{&S12308[34], 6}, {NULL}};
const Segment SGI68557[] = {{&S12309[44], 11}, {NULL}};
const Segment SGI68558[] = {{&S12309[44], 23}, {NULL}};
const Segment SGI68559[] = {{&S12309[30], 6}, {NULL}};
const Segment SGI68560[] = {{&S28920[38], 10}, {NULL}};
const Segment SGI68561[] = {{&S28920[31], 6}, {NULL}};
const Segment SGI68562[] = {{&S12309[31], 3}, {NULL}};
const Segment SGI68563[] = {{&S12309[81], 6}, {NULL}};
const Segment SGI68564[] = {{&S28921[44], 10}, {NULL}};
const Segment SGI68565[] = {{&S12310[44], 21}, {NULL}};
const Segment SGI68566[] = {{&S28921[44], 27}, {NULL}};
const Segment SGI68567[] = {{&S28921[30], 6}, {NULL}};
const Segment SGI68568[] = {{&S12310[44], 33}, {NULL}};
const Segment SGI68569[] = {{&S12310[30], 6}, {NULL}};
const Segment SGI68570[] = {{&S28921[119], 3}, {NULL}};
const Segment SGI68571[] = {{&S12310[31], 3}, {NULL}};
const Segment SGI68572[] = {{&S28921[85], 5}, {NULL}};
const Segment SGI68573[] = {{&S12310[91], 6}, {NULL}};
const Segment SGI68574[] = {{&S12311[44], 22}, {NULL}};
const Segment SGI68575[] = {{&S12311[44], 34}, {NULL}};
const Segment SGI68576[] = {{&S12311[30], 6}, {NULL}};
const Segment SGI68577[] = {{&S12311[31], 3}, {NULL}};
const Segment SGI68578[] = {{&S12311[92], 6}, {NULL}};
const Segment SGI68579[] = {{&S28924[44], 10}, {NULL}};
const Segment SGI68580[] = {{&S28924[44], 27}, {NULL}};
const Segment SGI68581[] = {{&S28924[30], 6}, {NULL}};
const Segment SGI68582[] = {{&S12312[44], 22}, {NULL}};
const Segment SGI68583[] = {{&S28924[88], 4}, {NULL}};
const Segment SGI68584[] = {{&S28924[88], 3}, {NULL}};
const Segment SGI68585[] = {{&S12312[44], 34}, {NULL}};
const Segment SGI68586[] = {{&S12312[30], 6}, {NULL}};
const Segment SGI68587[] = {{&S12312[31], 3}, {NULL}};
const Segment SGI68588[] = {{&S12312[92], 6}, {NULL}};
const Segment SGI68589[] = {{&S12314[44], 11}, {NULL}};
const Segment SGI68590[] = {{&S12314[44], 23}, {NULL}};
const Segment SGI68591[] = {{&S28927[44], 10}, {NULL}};
const Segment SGI68592[] = {{&S12314[30], 6}, {NULL}};
const Segment SGI68593[] = {{&S28927[44], 27}, {NULL}};
const Segment SGI68594[] = {{&S28927[30], 6}, {NULL}};
const Segment SGI68595[] = {{&S12314[31], 3}, {NULL}};
const Segment SGI68596[] = {{&S12314[81], 6}, {NULL}};
const Segment SGI68597[] = {{&S28927[88], 6}, {NULL}};
const Segment SGI68598[] = {{&S12315[44], 22}, {NULL}};
const Segment SGI68599[] = {{&S12315[44], 34}, {NULL}};
const Segment SGI68600[] = {{&S28928[42], 10}, {NULL}};
const Segment SGI68601[] = {{&S12315[30], 6}, {NULL}};
const Segment SGI68602[] = {{&S28928[30], 6}, {NULL}};
const Segment SGI68603[] = {{&S12315[31], 3}, {NULL}};
const Segment SGI68604[] = {{&S28928[117], 3}, {NULL}};
const Segment SGI68605[] = {{&S12315[92], 6}, {NULL}};
const Segment SGI68606[] = {{&S28928[83], 5}, {NULL}};
const Segment SGI68607[] = {{&S12316[44], 11}, {NULL}};
const Segment SGI68608[] = {{&S12316[44], 23}, {NULL}};
const Segment SGI68609[] = {{&S12316[30], 6}, {NULL}};
const Segment SGI68610[] = {{&S12316[84], 4}, {NULL}};
const Segment SGI68611[] = {{&S12316[84], 3}, {NULL}};
const Segment SGI68612[] = {{&S12317[44], 22}, {NULL}};
const Segment SGI68613[] = {{&S12317[44], 34}, {NULL}};
const Segment SGI68614[] = {{&S12317[30], 6}, {NULL}};
const Segment SGI68615[] = {{&S12317[31], 3}, {NULL}};
const Segment SGI68616[] = {{&S12317[92], 6}, {NULL}};
const Segment SGI68617[] = {{&S12318[44], 22}, {NULL}};
const Segment SGI68618[] = {{&S12318[44], 34}, {NULL}};
const Segment SGI68619[] = {{&S12318[30], 6}, {NULL}};
const Segment SGI68620[] = {{&S12318[31], 3}, {NULL}};
const Segment SGI68621[] = {{&S12318[92], 6}, {NULL}};
const Segment SGI68622[] = {{&S12319[44], 23}, {NULL}};
const Segment SGI68623[] = {{&S12319[44], 35}, {NULL}};
const Segment SGI68624[] = {{&S12319[30], 6}, {NULL}};
const Segment SGI68625[] = {{&S12319[31], 3}, {NULL}};
const Segment SGI68626[] = {{&S12319[93], 6}, {NULL}};
const Segment SGI68627[] = {{&S12320[44], 22}, {NULL}};
const Segment SGI68628[] = {{&S12320[44], 34}, {NULL}};
const Segment SGI68629[] = {{&S12320[30], 6}, {NULL}};
const Segment SGI68630[] = {{&S12320[31], 3}, {NULL}};
const Segment SGI68631[] = {{&S12320[92], 6}, {NULL}};
const Segment SGI68632[] = {{&S12321[44], 20}, {NULL}};
const Segment SGI68633[] = {{&S12321[44], 32}, {NULL}};
const Segment SGI68634[] = {{&S12321[30], 6}, {NULL}};
const Segment SGI68635[] = {{&S12321[31], 3}, {NULL}};
const Segment SGI68636[] = {{&S12321[90], 6}, {NULL}};
const Segment SGI68637[] = {{&S12322[44], 12}, {NULL}};
const Segment SGI68638[] = {{&S12322[44], 24}, {NULL}};
const Segment SGI68639[] = {{&S12322[30], 6}, {NULL}};
const Segment SGI68640[] = {{&S12322[31], 3}, {NULL}};
const Segment SGI68641[] = {{&S12322[82], 6}, {NULL}};
const Segment SGI68642[] = {{&S12323[44], 22}, {NULL}};
const Segment SGI68643[] = {{&S12323[44], 34}, {NULL}};
const Segment SGI68644[] = {{&S12323[30], 6}, {NULL}};
const Segment SGI68645[] = {{&S12323[31], 3}, {NULL}};
const Segment SGI68646[] = {{&S12323[92], 6}, {NULL}};
const Segment SGI68647[] = {{&S12324[44], 20}, {NULL}};
const Segment SGI68648[] = {{&S12324[44], 32}, {NULL}};
const Segment SGI68649[] = {{&S12324[30], 6}, {NULL}};
const Segment SGI68650[] = {{&S12324[31], 3}, {NULL}};
const Segment SGI68651[] = {{&S12324[90], 6}, {NULL}};
const Segment SGI68652[] = {{&S12325[44], 23}, {NULL}};
const Segment SGI68653[] = {{&S12325[44], 35}, {NULL}};
const Segment SGI68654[] = {{&S12325[30], 6}, {NULL}};
const Segment SGI68655[] = {{&S12325[31], 3}, {NULL}};
const Segment SGI68656[] = {{&S12325[93], 6}, {NULL}};
const Segment SGI68657[] = {{&S12327[44], 23}, {NULL}};
const Segment SGI68658[] = {{&S22613[44], 10}, {NULL}};
const Segment SGI68659[] = {{&S12327[44], 35}, {NULL}};
const Segment SGI68660[] = {{&S22613[44], 21}, {NULL}};
const Segment SGI68661[] = {{&S12327[30], 6}, {NULL}};
const Segment SGI68662[] = {{&S22613[30], 6}, {NULL}};
const Segment SGI68663[] = {{&S12327[31], 3}, {NULL}};
const Segment SGI68664[] = {{&S22613[113], 3}, {NULL}};
const Segment SGI68665[] = {{&S12327[93], 6}, {NULL}};
const Segment SGI68666[] = {{&S22613[79], 5}, {NULL}};
const Segment SGI68667[] = {{&S12328[44], 23}, {NULL}};
const Segment SGI68668[] = {{&S22614[38], 10}, {NULL}};
const Segment SGI68669[] = {{&S12328[44], 35}, {NULL}};
const Segment SGI68670[] = {{&S22614[31], 6}, {NULL}};
const Segment SGI68671[] = {{&S12328[30], 6}, {NULL}};
const Segment SGI68672[] = {{&S12328[31], 3}, {NULL}};
const Segment SGI68673[] = {{&S12328[93], 6}, {NULL}};
const Segment SGI68674[] = {{&S12329[44], 23}, {NULL}};
const Segment SGI68675[] = {{&S12329[44], 35}, {NULL}};
const Segment SGI68676[] = {{&S12329[30], 6}, {NULL}};
const Segment SGI68677[] = {{&S12329[31], 3}, {NULL}};
const Segment SGI68678[] = {{&S12329[93], 6}, {NULL}};
const Segment SGI68679[] = {{&S12331[44], 23}, {NULL}};
const Segment SGI68680[] = {{&S12331[44], 35}, {NULL}};
const Segment SGI68681[] = {{&S12331[30], 6}, {NULL}};
const Segment SGI68682[] = {{&S12331[31], 3}, {NULL}};
const Segment SGI68683[] = {{&S12331[93], 6}, {NULL}};
const Segment SGI68684[] = {{&S12332[44], 23}, {NULL}};
const Segment SGI68685[] = {{&S12332[44], 35}, {NULL}};
const Segment SGI68686[] = {{&S12332[30], 6}, {NULL}};
const Segment SGI68687[] = {{&S12332[31], 3}, {NULL}};
const Segment SGI68688[] = {{&S12332[93], 6}, {NULL}};
const Segment SGI68689[] = {{&S12333[44], 22}, {NULL}};
const Segment SGI68690[] = {{&S12333[44], 34}, {NULL}};
const Segment SGI68691[] = {{&S12333[30], 6}, {NULL}};
const Segment SGI68692[] = {{&S12333[31], 3}, {NULL}};
const Segment SGI68693[] = {{&S12333[92], 6}, {NULL}};
const Segment SGI68694[] = {{&S12337[44], 11}, {NULL}};
const Segment SGI68695[] = {{&S12337[44], 23}, {NULL}};
const Segment SGI68696[] = {{&S12337[30], 6}, {NULL}};
const Segment SGI68697[] = {{&S12337[84], 6}, {NULL}};
const Segment SGI68698[] = {{&S12339[44], 20}, {NULL}};
const Segment SGI68699[] = {{&S12339[44], 32}, {NULL}};
const Segment SGI68700[] = {{&S12339[30], 6}, {NULL}};
const Segment SGI68701[] = {{&S12339[31], 3}, {NULL}};
const Segment SGI68702[] = {{&S12339[90], 6}, {NULL}};
const Segment SGI68703[] = {{&S12341[44], 11}, {NULL}};
const Segment SGI68704[] = {{&S12341[44], 23}, {NULL}};
const Segment SGI68705[] = {{&S12341[30], 6}, {NULL}};
const Segment SGI68706[] = {{&S12341[84], 4}, {NULL}};
const Segment SGI68707[] = {{&S12341[84], 3}, {NULL}};
const Segment SGI68708[] = {{&S12343[44], 23}, {NULL}};
const Segment SGI68709[] = {{&S12343[44], 35}, {NULL}};
const Segment SGI68710[] = {{&S12343[30], 6}, {NULL}};
const Segment SGI68711[] = {{&S12343[31], 3}, {NULL}};
const Segment SGI68712[] = {{&S12343[93], 6}, {NULL}};
const Segment SGI68713[] = {{&S12344[42], 11}, {NULL}};
const Segment SGI68714[] = {{&S12344[30], 6}, {NULL}};
const Segment SGI68715[] = {{&S12344[31], 3}, {NULL}};
const Segment SGI68716[] = {{&S12344[79], 6}, {NULL}};
const Segment SGI68717[] = {{&S12347[44], 22}, {NULL}};
const Segment SGI68718[] = {{&S12347[44], 34}, {NULL}};
const Segment SGI68719[] = {{&S12347[30], 6}, {NULL}};
const Segment SGI68720[] = {{&S12347[31], 3}, {NULL}};
const Segment SGI68721[] = {{&S12347[92], 6}, {NULL}};
const Segment SGI68722[] = {{&S12364[42], 11}, {NULL}};
const Segment SGI68723[] = {{&S12364[30], 6}, {NULL}};
const Segment SGI68724[] = {{&S12364[31], 3}, {NULL}};
const Segment SGI68725[] = {{&S12364[79], 6}, {NULL}};
const Segment SGI68726[] = {{&S12365[44], 20}, {NULL}};
const Segment SGI68727[] = {{&S12365[44], 32}, {NULL}};
const Segment SGI68728[] = {{&S12365[30], 6}, {NULL}};
const Segment SGI68729[] = {{&S12365[31], 3}, {NULL}};
const Segment SGI68730[] = {{&S12365[90], 6}, {NULL}};
const Segment SGI68731[] = {{&S12366[44], 20}, {NULL}};
const Segment SGI68732[] = {{&S12366[44], 32}, {NULL}};
const Segment SGI68733[] = {{&S12366[30], 6}, {NULL}};
const Segment SGI68734[] = {{&S12366[31], 3}, {NULL}};
const Segment SGI68735[] = {{&S12366[90], 6}, {NULL}};
const Segment SGI68736[] = {{&S12367[44], 21}, {NULL}};
const Segment SGI68737[] = {{&S12367[44], 33}, {NULL}};
const Segment SGI68738[] = {{&S12367[30], 6}, {NULL}};
const Segment SGI68739[] = {{&S12367[31], 3}, {NULL}};
const Segment SGI68740[] = {{&S12367[91], 6}, {NULL}};
const Segment SGI68741[] = {{&S12368[44], 20}, {NULL}};
const Segment SGI68742[] = {{&S12368[44], 32}, {NULL}};
const Segment SGI68743[] = {{&S12368[30], 6}, {NULL}};
const Segment SGI68744[] = {{&S12368[31], 3}, {NULL}};
const Segment SGI68745[] = {{&S12368[90], 6}, {NULL}};
const Segment SGI68746[] = {{&S12369[44], 11}, {NULL}};
const Segment SGI68747[] = {{&S12369[44], 23}, {NULL}};
const Segment SGI68748[] = {{&S12369[30], 6}, {NULL}};
const Segment SGI68749[] = {{&S12369[84], 6}, {NULL}};
const Segment SGI68750[] = {{&S12370[44], 20}, {NULL}};
const Segment SGI68751[] = {{&S12370[44], 32}, {NULL}};
const Segment SGI68752[] = {{&S12370[30], 6}, {NULL}};
const Segment SGI68753[] = {{&S12370[31], 3}, {NULL}};
const Segment SGI68754[] = {{&S12370[90], 6}, {NULL}};
const Segment SGI68755[] = {{&S12372[44], 21}, {NULL}};
const Segment SGI68756[] = {{&S12372[44], 33}, {NULL}};
const Segment SGI68757[] = {{&S12372[30], 6}, {NULL}};
const Segment SGI68758[] = {{&S12372[31], 3}, {NULL}};
const Segment SGI68759[] = {{&S12372[91], 6}, {NULL}};
const Segment SGI68760[] = {{&S12373[44], 20}, {NULL}};
const Segment SGI68761[] = {{&S12373[44], 32}, {NULL}};
const Segment SGI68762[] = {{&S12373[30], 6}, {NULL}};
const Segment SGI68763[] = {{&S12373[31], 3}, {NULL}};
const Segment SGI68764[] = {{&S12373[90], 6}, {NULL}};
const Segment SGI68765[] = {{&S12376[44], 21}, {NULL}};
const Segment SGI68766[] = {{&S12376[44], 33}, {NULL}};
const Segment SGI68767[] = {{&S12376[30], 6}, {NULL}};
const Segment SGI68768[] = {{&S12376[31], 3}, {NULL}};
const Segment SGI68769[] = {{&S12376[91], 6}, {NULL}};
const Segment SGI68770[] = {{&S12380[44], 11}, {NULL}};
const Segment SGI68771[] = {{&S12380[44], 23}, {NULL}};
const Segment SGI68772[] = {{&S12380[30], 6}, {NULL}};
const Segment SGI68773[] = {{&S12380[31], 3}, {NULL}};
const Segment SGI68774[] = {{&S12380[81], 5}, {NULL}};
const Segment SGI68775[] = {{&S12382[44], 21}, {NULL}};
const Segment SGI68776[] = {{&S12382[44], 33}, {NULL}};
const Segment SGI68777[] = {{&S12382[30], 6}, {NULL}};
const Segment SGI68778[] = {{&S12382[31], 3}, {NULL}};
const Segment SGI68779[] = {{&S12382[91], 6}, {NULL}};
const Segment SGI68780[] = {{&S12383[44], 9}, {NULL}};
const Segment SGI68781[] = {{&S12383[44], 21}, {NULL}};
const Segment SGI68782[] = {{&S12383[30], 6}, {NULL}};
const Segment SGI68783[] = {{&S12383[31], 3}, {NULL}};
const Segment SGI68784[] = {{&S12383[79], 6}, {NULL}};
const Segment SGI68785[] = {{&S12384[44], 19}, {NULL}};
const Segment SGI68786[] = {{&S12384[44], 31}, {NULL}};
const Segment SGI68787[] = {{&S12384[30], 6}, {NULL}};
const Segment SGI68788[] = {{&S12384[31], 3}, {NULL}};
const Segment SGI68789[] = {{&S12384[89], 6}, {NULL}};
const Segment SGI68790[] = {{&S12658[44], 10}, {NULL}};
const Segment SGI68791[] = {{&S12658[44], 22}, {NULL}};
const Segment SGI68792[] = {{&S12658[30], 6}, {NULL}};
const Segment SGI68793[] = {{&S12658[31], 3}, {NULL}};
const Segment SGI68794[] = {{&S12658[80], 6}, {NULL}};
const Segment SGI68795[] = {{&S12659[44], 10}, {NULL}};
const Segment SGI68796[] = {{&S12659[44], 22}, {NULL}};
const Segment SGI68797[] = {{&S12659[30], 6}, {NULL}};
const Segment SGI68798[] = {{&S12659[31], 3}, {NULL}};
const Segment SGI68799[] = {{&S12659[80], 6}, {NULL}};
const Segment SGI68800[] = {{&S12660[38], 10}, {NULL}};
const Segment SGI68801[] = {{&S12660[31], 6}, {NULL}};
const Segment SGI68802[] = {{&S12661[44], 30}, {NULL}};
const Segment SGI68803[] = {{&S12661[44], 42}, {NULL}};
const Segment SGI68804[] = {{&S12661[30], 6}, {NULL}};
const Segment SGI68805[] = {{&S12661[31], 3}, {NULL}};
const Segment SGI68806[] = {{&S12661[100], 6}, {NULL}};
const Segment SGI68807[] = {{&S12666[42], 10}, {NULL}};
const Segment SGI68808[] = {{&S12666[30], 6}, {NULL}};
const Segment SGI68809[] = {{&S12666[31], 3}, {NULL}};
const Segment SGI68810[] = {{&S12666[78], 6}, {NULL}};
const Segment SGI68811[] = {{&S12672[44], 30}, {NULL}};
const Segment SGI68812[] = {{&S12672[44], 42}, {NULL}};
const Segment SGI68813[] = {{&S12672[30], 6}, {NULL}};
const Segment SGI68814[] = {{&S12672[31], 3}, {NULL}};
const Segment SGI68815[] = {{&S12672[100], 6}, {NULL}};
const Segment SGI68816[] = {{&S12673[44], 10}, {NULL}};
const Segment SGI68817[] = {{&S12673[44], 34}, {NULL}};
const Segment SGI68818[] = {{&S12673[30], 6}, {NULL}};
const Segment SGI68819[] = {{&S12674[44], 10}, {NULL}};
const Segment SGI68820[] = {{&S12674[44], 22}, {NULL}};
const Segment SGI68821[] = {{&S12674[30], 6}, {NULL}};
const Segment SGI68822[] = {{&S12674[83], 6}, {NULL}};
const Segment SGI68823[] = {{&S12675[64], 10}, {NULL}};
const Segment SGI68824[] = {{&S12675[64], 22}, {NULL}};
const Segment SGI68825[] = {{&S12675[50], 6}, {NULL}};
const Segment SGI68826[] = {{&S12677[44], 10}, {NULL}};
const Segment SGI68827[] = {{&S12677[44], 22}, {NULL}};
const Segment SGI68828[] = {{&S12677[30], 6}, {NULL}};
const Segment SGI68829[] = {{&S12677[125], 3}, {NULL}};
const Segment SGI68830[] = {{&S12677[90], 6}, {NULL}};
const Segment SGI68831[] = {{&S12677[145], 8}, {NULL}};
const Segment SGI68832[] = {{&S12679[44], 22}, {NULL}};
const Segment SGI68833[] = {{&S12679[30], 6}, {NULL}};
const Segment SGI68834[] = {{&S12682[44], 10}, {NULL}};
const Segment SGI68835[] = {{&S12682[44], 34}, {NULL}};
const Segment SGI68836[] = {{&S12682[30], 6}, {NULL}};
const Segment SGI68837[] = {{&S12687[42], 30}, {NULL}};
const Segment SGI68838[] = {{&S12687[30], 6}, {NULL}};
const Segment SGI68839[] = {{&S12687[31], 3}, {NULL}};
const Segment SGI68840[] = {{&S12687[98], 6}, {NULL}};
const Segment SGI68841[] = {{&S12688[59], 10}, {NULL}};
const Segment SGI68842[] = {{&S12688[59], 34}, {NULL}};
const Segment SGI68843[] = {{&S12688[45], 6}, {NULL}};
const Segment SGI68844[] = {{&S12690[44], 30}, {NULL}};
const Segment SGI68845[] = {{&S12690[44], 42}, {NULL}};
const Segment SGI68846[] = {{&S12690[30], 6}, {NULL}};
const Segment SGI68847[] = {{&S12690[31], 3}, {NULL}};
const Segment SGI68848[] = {{&S12690[100], 6}, {NULL}};
const Segment SGI68849[] = {{&S12696[58], 10}, {NULL}};
const Segment SGI68850[] = {{&S12696[58], 34}, {NULL}};
const Segment SGI68851[] = {{&S12696[44], 6}, {NULL}};
const Segment SGI68852[] = {{&S12698[44], 10}, {NULL}};
const Segment SGI68853[] = {{&S12698[44], 19}, {NULL}};
const Segment SGI68854[] = {{&S12698[30], 6}, {NULL}};
const Segment SGI68855[] = {{&S12698[31], 3}, {NULL}};
const Segment SGI68856[] = {{&S12698[77], 5}, {NULL}};
const Segment SGI68857[] = {{&S12699[55], 18}, {NULL}};
const Segment SGI68858[] = {{&S12699[48], 6}, {NULL}};
const Segment SGI68859[] = {{&S12700[44], 10}, {NULL}};
const Segment SGI68860[] = {{&S12700[44], 22}, {NULL}};
const Segment SGI68861[] = {{&S12700[30], 6}, {NULL}};
const Segment SGI68862[] = {{&S12700[116], 3}, {NULL}};
const Segment SGI68863[] = {{&S12700[80], 7}, {NULL}};
const Segment SGI68864[] = {{&S12700[136], 8}, {NULL}};
const Segment SGI68865[] = {{&S12704[44], 10}, {NULL}};
const Segment SGI68866[] = {{&S12704[44], 34}, {NULL}};
const Segment SGI68867[] = {{&S12704[30], 6}, {NULL}};
const Segment SGI68868[] = {{&S12706[45], 10}, {NULL}};
const Segment SGI68869[] = {{&S12706[45], 34}, {NULL}};
const Segment SGI68870[] = {{&S12706[31], 6}, {NULL}};
const Segment SGI68871[] = {{&S12707[44], 10}, {NULL}};
const Segment SGI68872[] = {{&S12707[44], 22}, {&S12707[208], 7}, {NULL}};
const Segment SGI68873[] = {{&S12707[30], 6}, {NULL}};
const Segment SGI68874[] = {{&S12707[31], 3}, {NULL}};
const Segment SGI68875[] = {{&S12707[80], 6}, {NULL}};
const Segment SGI68876[] = {{&S12710[45], 10}, {NULL}};
const Segment SGI68877[] = {{&S12710[45], 34}, {NULL}};
const Segment SGI68878[] = {{&S12710[31], 6}, {NULL}};
const Segment SGI68879[] = {{&S12713[44], 10}, {NULL}};
const Segment SGI68880[] = {{&S12713[44], 22}, {NULL}};
const Segment SGI68881[] = {{&S12713[30], 6}, {NULL}};
const Segment SGI68882[] = {{&S12713[31], 3}, {NULL}};
const Segment SGI68883[] = {{&S12713[80], 5}, {NULL}};
const Segment SGI68884[] = {{&S12714[44], 10}, {NULL}};
const Segment SGI68885[] = {{&S12714[44], 22}, {&S12714[271], 7}, {NULL}};
const Segment SGI68886[] = {{&S12714[30], 6}, {NULL}};
const Segment SGI68887[] = {{&S12714[31], 3}, {NULL}};
const Segment SGI68888[] = {{&S12714[80], 6}, {NULL}};
const Segment SGI68889[] = {{&S12715[46], 10}, {NULL}};
const Segment SGI68890[] = {{&S12715[46], 34}, {NULL}};
const Segment SGI68891[] = {{&S12715[32], 6}, {NULL}};
const Segment SGI68892[] = {{&S12718[44], 10}, {NULL}};
const Segment SGI68893[] = {{&S12718[44], 22}, {&S12718[45], 8}, {NULL}};
const Segment SGI68894[] = {{&S12718[30], 6}, {NULL}};
const Segment SGI68895[] = {{&S12718[31], 3}, {NULL}};
const Segment SGI68896[] = {{&S12718[80], 6}, {NULL}};
const Segment SGI68897[] = {{&S12722[44], 10}, {NULL}};
const Segment SGI68898[] = {{&S12722[44], 32}, {NULL}};
const Segment SGI68899[] = {{&S12722[30], 6}, {NULL}};
const Segment SGI68900[] = {{&S12722[31], 3}, {NULL}};
const Segment SGI68901[] = {{&S12722[90], 6}, {NULL}};
const Segment SGI68902[] = {{&S12725[46], 10}, {NULL}};
const Segment SGI68903[] = {{&S12725[46], 34}, {NULL}};
const Segment SGI68904[] = {{&S12725[32], 6}, {NULL}};
const Segment SGI68905[] = {{&S12730[46], 10}, {NULL}};
const Segment SGI68906[] = {{&S12730[30], 6}, {NULL}};
const Segment SGI68907[] = {{&S12730[118], 3}, {NULL}};
const Segment SGI68908[] = {{&S12730[82], 7}, {NULL}};
const Segment SGI68909[] = {{&S12730[138], 8}, {NULL}};
const Segment SGI68910[] = {{&S12731[103], 29}, {NULL}};
const Segment SGI68911[] = {{&S12732[46], 10}, {NULL}};
const Segment SGI68912[] = {{&S12732[46], 34}, {NULL}};
const Segment SGI68913[] = {{&S12732[32], 6}, {NULL}};
const Segment SGI68914[] = {{&S12733[35], 26}, {NULL}};
const Segment SGI68915[] = {{&S12734[28], 6}, {NULL}};
const Segment SGI68916[] = {{&S12735[44], 9}, {NULL}};
const Segment SGI68917[] = {{&S12735[30], 6}, {NULL}};
const Segment SGI68918[] = {{&S12736[44], 10}, {NULL}};
const Segment SGI68919[] = {{&S12736[44], 22}, {&S12736[154], 12}, {NULL}};
const Segment SGI68920[] = {{&S12736[30], 6}, {NULL}};
const Segment SGI68921[] = {{&S12736[31], 3}, {NULL}};
const Segment SGI68922[] = {{&S12736[80], 6}, {NULL}};
const Segment SGI68923[] = {{&S12737[44], 10}, {NULL}};
const Segment SGI68924[] = {{&S12737[44], 22}, {NULL}};
const Segment SGI68925[] = {{&S12737[30], 6}, {NULL}};
const Segment SGI68926[] = {{&S12737[80], 6}, {NULL}};
const Segment SGI68927[] = {{&S12739[45], 10}, {NULL}};
const Segment SGI68928[] = {{&S12739[45], 34}, {NULL}};
const Segment SGI68929[] = {{&S12739[31], 6}, {NULL}};
const Segment SGI68930[] = {{&S12740[44], 30}, {NULL}};
const Segment SGI68931[] = {{&S12740[44], 42}, {NULL}};
const Segment SGI68932[] = {{&S12740[30], 6}, {NULL}};
const Segment SGI68933[] = {{&S12740[31], 3}, {NULL}};
const Segment SGI68934[] = {{&S12740[100], 6}, {NULL}};
const Segment SGI68935[] = {{&S12741[44], 10}, {NULL}};
const Segment SGI68936[] = {{&S12741[44], 22}, {NULL}};
const Segment SGI68937[] = {{&S12741[30], 6}, {NULL}};
const Segment SGI68938[] = {{&S12741[31], 3}, {NULL}};
const Segment SGI68939[] = {{&S12741[80], 6}, {&S12741[80], 6}, {NULL}};
const Segment SGI68940[] = {{&S12743[44], 10}, {NULL}};
const Segment SGI68941[] = {{&S12743[44], 36}, {NULL}};
const Segment SGI68942[] = {{&S12743[30], 6}, {NULL}};
const Segment SGI68943[] = {{&S12743[129], 3}, {NULL}};
const Segment SGI68944[] = {{&S12743[94], 6}, {NULL}};
const Segment SGI68945[] = {{&S12743[149], 8}, {NULL}};
const Segment SGI68946[] = {{&S12744[44], 10}, {NULL}};
const Segment SGI68947[] = {{&S12744[44], 22}, {NULL}};
const Segment SGI68948[] = {{&S12744[30], 6}, {NULL}};
const Segment SGI68949[] = {{&S12744[80], 6}, {NULL}};
const Segment SGI68950[] = {{&S12745[44], 10}, {&S12745[175], 18}, {NULL}};
const Segment SGI68951[] = {{&S12745[44], 22}, {NULL}};
const Segment SGI68952[] = {{&S12745[30], 6}, {NULL}};
const Segment SGI68953[] = {{&S12745[31], 3}, {NULL}};
const Segment SGI68954[] = {{&S12745[80], 6}, {NULL}};
const Segment SGI68955[] = {{&S12747[44], 10}, {NULL}};
const Segment SGI68956[] = {{&S12747[44], 22}, {NULL}};
const Segment SGI68957[] = {{&S12747[30], 6}, {NULL}};
const Segment SGI68958[] = {{&S12747[31], 3}, {NULL}};
const Segment SGI68959[] = {{&S12747[125], 5}, {NULL}};
const Segment SGI68960[] = {{&S12748[44], 30}, {NULL}};
const Segment SGI68961[] = {{&S12748[44], 42}, {NULL}};
const Segment SGI68962[] = {{&S12748[30], 6}, {NULL}};
const Segment SGI68963[] = {{&S12748[31], 3}, {NULL}};
const Segment SGI68964[] = {{&S12748[100], 6}, {NULL}};
const Segment SGI68965[] = {{&S12749[44], 10}, {NULL}};
const Segment SGI68966[] = {{&S12749[44], 22}, {NULL}};
const Segment SGI68967[] = {{&S12749[30], 6}, {NULL}};
const Segment SGI68968[] = {{&S12749[83], 4}, {NULL}};
const Segment SGI68969[] = {{&S12749[83], 3}, {NULL}};
const Segment SGI68970[] = {{&S12754[44], 10}, {NULL}};
const Segment SGI68971[] = {{&S12754[44], 22}, {NULL}};
const Segment SGI68972[] = {{&S12754[30], 6}, {NULL}};
const Segment SGI68973[] = {{&S12754[31], 3}, {NULL}};
const Segment SGI68974[] = {{&S12754[80], 6}, {NULL}};
const Segment SGI68975[] = {{&S23156[44], 9}, {NULL}};
const Segment SGI68976[] = {{&S23156[44], 20}, {NULL}};
const Segment SGI68977[] = {{&S23156[30], 6}, {NULL}};
const Segment SGI68978[] = {{&S23156[112], 3}, {NULL}};
const Segment SGI68979[] = {{&S23156[78], 5}, {NULL}};
const Segment SGI68980[] = {{&S23166[38], 9}, {NULL}};
const Segment SGI68981[] = {{&S23166[31], 6}, {NULL}};
const Segment SGI68982[] = {{&S23174[44], 5}, {NULL}};
const Segment SGI68983[] = {{&S23174[44], 16}, {NULL}};
const Segment SGI68984[] = {{&S23174[30], 6}, {NULL}};
const Segment SGI68985[] = {{&S23174[118], 3}, {NULL}};
const Segment SGI68986[] = {{&S23174[84], 5}, {NULL}};
const Segment SGI68987[] = {{&S23180[44], 9}, {NULL}};
const Segment SGI68988[] = {{&S23180[44], 27}, {NULL}};
const Segment SGI68989[] = {{&S23180[30], 6}, {NULL}};
const Segment SGI68990[] = {{&S23180[137], 3}, {NULL}};
const Segment SGI68991[] = {{&S23180[103], 5}, {NULL}};
const Segment SGI68992[] = {{&S23181[44], 17}, {NULL}};
const Segment SGI68993[] = {{&S23181[44], 28}, {NULL}};
const Segment SGI68994[] = {{&S23181[30], 6}, {NULL}};
const Segment SGI68995[] = {{&S23181[120], 3}, {NULL}};
const Segment SGI68996[] = {{&S23181[86], 5}, {NULL}};
const Segment SGI68997[] = {{&S23182[44], 9}, {NULL}};
const Segment SGI68998[] = {{&S23182[44], 29}, {NULL}};
const Segment SGI68999[] = {{&S23182[30], 6}, {NULL}};
const Segment SGI69000[] = {{&S23182[137], 3}, {NULL}};
const Segment SGI69001[] = {{&S23182[103], 5}, {NULL}};
const Segment SGI69002[] = {{&S23183[44], 12}, {NULL}};
const Segment SGI69003[] = {{&S23183[44], 23}, {NULL}};
const Segment SGI69004[] = {{&S23183[30], 6}, {NULL}};
const Segment SGI69005[] = {{&S23183[115], 3}, {NULL}};
const Segment SGI69006[] = {{&S23183[81], 5}, {NULL}};
const Segment SGI69007[] = {{&S23185[44], 8}, {NULL}};
const Segment SGI69008[] = {{&S23185[44], 19}, {NULL}};
const Segment SGI69009[] = {{&S23185[30], 6}, {NULL}};
const Segment SGI69010[] = {{&S23185[143], 3}, {NULL}};
const Segment SGI69011[] = {{&S23185[109], 5}, {NULL}};
const Segment SGI69012[] = {{&S23186[44], 8}, {NULL}};
const Segment SGI69013[] = {{&S23186[44], 21}, {NULL}};
const Segment SGI69014[] = {{&S23186[30], 6}, {NULL}};
const Segment SGI69015[] = {{&S23186[82], 6}, {NULL}};
const Segment SGI69016[] = {{&S29690[44], 13}, {NULL}};
const Segment SGI69017[] = {{&S29690[44], 20}, {NULL}};
const Segment SGI69018[] = {{&S29690[30], 6}, {NULL}};
const Segment SGI69019[] = {{&S29690[112], 3}, {NULL}};
const Segment SGI69020[] = {{&S29690[78], 5}, {NULL}};
const Segment SGI69021[] = {{&S29692[38], 10}, {NULL}};
const Segment SGI69022[] = {{&S29692[31], 6}, {NULL}};
const Segment SGI69023[] = {{&S29695[44], 13}, {NULL}};
const Segment SGI69024[] = {{&S29695[44], 22}, {NULL}};
const Segment SGI69025[] = {{&S29695[30], 6}, {NULL}};
const Segment SGI69026[] = {{&S29695[114], 3}, {NULL}};
const Segment SGI69027[] = {{&S29695[80], 5}, {NULL}};
const Segment SGI69028[] = {{&S29696[44], 10}, {NULL}};
const Segment SGI69029[] = {{&S29696[44], 27}, {NULL}};
const Segment SGI69030[] = {{&S29696[30], 6}, {NULL}};
const Segment SGI69031[] = {{&S29696[119], 3}, {NULL}};
const Segment SGI69032[] = {{&S29696[85], 5}, {NULL}};
const Segment SGI69033[] = {{&S29697[44], 13}, {NULL}};
const Segment SGI69034[] = {{&S29697[44], 22}, {NULL}};
const Segment SGI69035[] = {{&S29697[30], 6}, {NULL}};
const Segment SGI69036[] = {{&S29697[114], 3}, {NULL}};
const Segment SGI69037[] = {{&S29697[80], 5}, {NULL}};
const Segment SGI69038[] = {{&S29699[44], 13}, {NULL}};
const Segment SGI69039[] = {{&S29699[44], 22}, {NULL}};
const Segment SGI69040[] = {{&S29699[30], 6}, {NULL}};
const Segment SGI69041[] = {{&S29699[114], 3}, {NULL}};
const Segment SGI69042[] = {{&S29699[80], 5}, {NULL}};
const Segment SGI69043[] = {{&S29700[42], 10}, {NULL}};
const Segment SGI69044[] = {{&S29700[30], 6}, {NULL}};
const Segment SGI69045[] = {{&S29700[117], 3}, {NULL}};
const Segment SGI69046[] = {{&S29700[83], 5}, {NULL}};
const Segment SGI69047[] = {{&S29701[44], 13}, {NULL}};
const Segment SGI69048[] = {{&S29701[44], 22}, {NULL}};
const Segment SGI69049[] = {{&S29701[30], 6}, {NULL}};
const Segment SGI69050[] = {{&S29701[114], 3}, {NULL}};
const Segment SGI69051[] = {{&S29701[80], 5}, {NULL}};
const Segment SGI69052[] = {{&S29703[44], 10}, {NULL}};
const Segment SGI69053[] = {{&S29703[44], 27}, {NULL}};
const Segment SGI69054[] = {{&S29703[30], 6}, {NULL}};
const Segment SGI69055[] = {{&S29703[85], 6}, {NULL}};
const Segment SGI69056[] = {{&S29704[44], 10}, {NULL}};
const Segment SGI69057[] = {{&S29704[44], 27}, {NULL}};
const Segment SGI69058[] = {{&S29704[30], 6}, {NULL}};
const Segment SGI69059[] = {{&S29704[121], 3}, {NULL}};
const Segment SGI69060[] = {{&S29704[85], 7}, {NULL}};
const Segment SGI69061[] = {{&S29704[141], 8}, {NULL}};
const Segment SGI69062[] = {{&S29705[44], 13}, {NULL}};
const Segment SGI69063[] = {{&S29705[44], 22}, {NULL}};
const Segment SGI69064[] = {{&S29705[30], 6}, {NULL}};
const Segment SGI69065[] = {{&S29705[114], 3}, {NULL}};
const Segment SGI69066[] = {{&S29705[80], 5}, {NULL}};
const Segment SGI69067[] = {{&S29706[44], 13}, {NULL}};
const Segment SGI69068[] = {{&S29706[44], 22}, {NULL}};
const Segment SGI69069[] = {{&S29706[30], 6}, {NULL}};
const Segment SGI69070[] = {{&S29706[114], 3}, {NULL}};
const Segment SGI69071[] = {{&S29706[80], 5}, {NULL}};
const Segment SGI69072[] = {{&S29707[64], 10}, {NULL}};
const Segment SGI69073[] = {{&S29707[64], 27}, {NULL}};
const Segment SGI69074[] = {{&S29707[50], 6}, {NULL}};
const Segment SGI69075[] = {{&S23562[44], 7}, {NULL}};
const Segment SGI69076[] = {{&S23562[44], 19}, {NULL}};
const Segment SGI69077[] = {{&S23562[30], 6}, {NULL}};
const Segment SGI69078[] = {{&S23562[130], 3}, {NULL}};
const Segment SGI69079[] = {{&S23562[96], 5}, {NULL}};
const Segment SGI69080[] = {{&S29708[42], 13}, {NULL}};
const Segment SGI69081[] = {{&S29708[30], 6}, {NULL}};
const Segment SGI69082[] = {{&S29708[112], 3}, {NULL}};
const Segment SGI69083[] = {{&S29708[78], 5}, {NULL}};
const Segment SGI69084[] = {{&S29722[44], 11}, {NULL}};
const Segment SGI69085[] = {{&S29722[44], 22}, {NULL}};
const Segment SGI69086[] = {{&S29722[30], 6}, {NULL}};
const Segment SGI69087[] = {{&S29722[114], 3}, {NULL}};
const Segment SGI69088[] = {{&S29722[80], 5}, {NULL}};
const Segment SGI69089[] = {{&S29727[44], 11}, {NULL}};
const Segment SGI69090[] = {{&S29727[44], 28}, {NULL}};
const Segment SGI69091[] = {{&S29727[30], 6}, {NULL}};
const Segment SGI69092[] = {{&S29727[120], 3}, {NULL}};
const Segment SGI69093[] = {{&S29727[86], 5}, {NULL}};
const Segment SGI69094[] = {{&S29729[44], 11}, {NULL}};
const Segment SGI69095[] = {{&S29729[44], 28}, {NULL}};
const Segment SGI69096[] = {{&S29729[30], 6}, {NULL}};
const Segment SGI69097[] = {{&S29729[120], 3}, {NULL}};
const Segment SGI69098[] = {{&S29729[86], 5}, {NULL}};
const Segment SGI69099[] = {{&S23589[38], 7}, {NULL}};
const Segment SGI69100[] = {{&S23589[31], 6}, {NULL}};
const Segment SGI69101[] = {{&S23590[44], 7}, {NULL}};
const Segment SGI69102[] = {{&S23590[44], 20}, {NULL}};
const Segment SGI69103[] = {{&S23590[30], 6}, {NULL}};
const Segment SGI69104[] = {{&S23590[120], 3}, {NULL}};
const Segment SGI69105[] = {{&S23590[86], 5}, {NULL}};
const Segment SGI69106[] = {{&S29734[44], 11}, {NULL}};
const Segment SGI69107[] = {{&S29734[44], 25}, {NULL}};
const Segment SGI69108[] = {{&S29734[30], 6}, {NULL}};
const Segment SGI69109[] = {{&S29734[117], 3}, {NULL}};
const Segment SGI69110[] = {{&S29734[83], 5}, {NULL}};
const Segment SGI69111[] = {{&S29738[44], 7}, {NULL}};
const Segment SGI69112[] = {{&S29738[44], 24}, {NULL}};
const Segment SGI69113[] = {{&S29738[30], 6}, {NULL}};
const Segment SGI69114[] = {{&S29738[116], 3}, {NULL}};
const Segment SGI69115[] = {{&S29738[82], 5}, {NULL}};
const Segment SGI69116[] = {{&S29739[44], 11}, {NULL}};
const Segment SGI69117[] = {{&S29739[44], 28}, {NULL}};
const Segment SGI69118[] = {{&S29739[30], 6}, {NULL}};
const Segment SGI69119[] = {{&S29739[120], 3}, {NULL}};
const Segment SGI69120[] = {{&S29739[86], 5}, {NULL}};
const Segment SGI69121[] = {{&S29740[44], 11}, {NULL}};
const Segment SGI69122[] = {{&S29740[44], 28}, {NULL}};
const Segment SGI69123[] = {{&S29740[30], 6}, {NULL}};
const Segment SGI69124[] = {{&S29740[120], 3}, {NULL}};
const Segment SGI69125[] = {{&S29740[86], 5}, {NULL}};
const Segment SGI69126[] = {{&S29741[44], 7}, {NULL}};
const Segment SGI69127[] = {{&S29741[44], 24}, {NULL}};
const Segment SGI69128[] = {{&S29741[30], 6}, {NULL}};
const Segment SGI69129[] = {{&S29741[116], 3}, {NULL}};
const Segment SGI69130[] = {{&S29741[82], 5}, {NULL}};
const Segment SGI69131[] = {{&S29744[44], 11}, {NULL}};
const Segment SGI69132[] = {{&S29744[44], 28}, {NULL}};
const Segment SGI69133[] = {{&S29744[30], 6}, {NULL}};
const Segment SGI69134[] = {{&S29744[120], 3}, {NULL}};
const Segment SGI69135[] = {{&S29744[86], 5}, {NULL}};
const Segment SGI69136[] = {{&S29745[44], 7}, {NULL}};
const Segment SGI69137[] = {{&S29745[44], 24}, {NULL}};
const Segment SGI69138[] = {{&S29745[30], 6}, {NULL}};
const Segment SGI69139[] = {{&S29745[116], 3}, {NULL}};
const Segment SGI69140[] = {{&S29745[82], 5}, {NULL}};
const Segment SGI69141[] = {{&S29747[44], 7}, {NULL}};
const Segment SGI69142[] = {{&S29747[44], 24}, {NULL}};
const Segment SGI69143[] = {{&S29747[30], 6}, {NULL}};
const Segment SGI69144[] = {{&S29747[116], 3}, {NULL}};
const Segment SGI69145[] = {{&S29747[82], 5}, {NULL}};
const Segment SGI69146[] = {{&S29748[44], 7}, {NULL}};
const Segment SGI69147[] = {{&S29748[44], 24}, {NULL}};
const Segment SGI69148[] = {{&S29748[30], 6}, {NULL}};
const Segment SGI69149[] = {{&S29748[116], 3}, {NULL}};
const Segment SGI69150[] = {{&S29748[82], 5}, {NULL}};
const Segment SGI69151[] = {{&S29754[44], 11}, {NULL}};
const Segment SGI69152[] = {{&S29754[44], 28}, {NULL}};
const Segment SGI69153[] = {{&S29754[30], 6}, {NULL}};
const Segment SGI69154[] = {{&S29754[120], 3}, {NULL}};
const Segment SGI69155[] = {{&S29754[86], 5}, {NULL}};
const Segment SGI69156[] = {{&S29756[44], 7}, {NULL}};
const Segment SGI69157[] = {{&S29756[44], 24}, {NULL}};
const Segment SGI69158[] = {{&S29756[30], 6}, {NULL}};
const Segment SGI69159[] = {{&S29756[116], 3}, {NULL}};
const Segment SGI69160[] = {{&S29756[82], 5}, {NULL}};
const Segment SGI69161[] = {{&S29757[44], 7}, {NULL}};
const Segment SGI69162[] = {{&S29757[44], 24}, {NULL}};
const Segment SGI69163[] = {{&S29757[30], 6}, {NULL}};
const Segment SGI69164[] = {{&S29757[116], 3}, {NULL}};
const Segment SGI69165[] = {{&S29757[82], 5}, {NULL}};
const Segment SGI69166[] = {{&S29758[44], 7}, {NULL}};
const Segment SGI69167[] = {{&S29758[44], 24}, {NULL}};
const Segment SGI69168[] = {{&S29758[30], 6}, {NULL}};
const Segment SGI69169[] = {{&S29758[116], 3}, {NULL}};
const Segment SGI69170[] = {{&S29758[82], 5}, {NULL}};
const Segment SGI69171[] = {{&S23632[44], 7}, {NULL}};
const Segment SGI69172[] = {{&S23632[44], 25}, {NULL}};
const Segment SGI69173[] = {{&S23632[30], 6}, {NULL}};
const Segment SGI69174[] = {{&S23632[57], 3}, {NULL}};
const Segment SGI69175[] = {{&S23632[83], 5}, {NULL}};
const Segment SGI69176[] = {{&S23421[42], 11}, {NULL}};
const Segment SGI69177[] = {{&S23421[42], 22}, {NULL}};
const Segment SGI69178[] = {{&S23421[30], 4}, {NULL}};
const Segment SGI69179[] = {{&S23421[113], 3}, {NULL}};
const Segment SGI69180[] = {{&S23421[78], 6}, {NULL}};
const Segment SGI69181[] = {{&S23422[42], 11}, {NULL}};
const Segment SGI69182[] = {{&S23422[42], 22}, {NULL}};
const Segment SGI69183[] = {{&S23422[30], 4}, {NULL}};
const Segment SGI69184[] = {{&S23422[113], 3}, {NULL}};
const Segment SGI69185[] = {{&S23422[78], 6}, {NULL}};
const Segment SGI69186[] = {{&S23423[42], 29}, {NULL}};
const Segment SGI69187[] = {{&S23423[42], 40}, {NULL}};
const Segment SGI69188[] = {{&S23423[30], 4}, {NULL}};
const Segment SGI69189[] = {{&S23423[131], 3}, {NULL}};
const Segment SGI69190[] = {{&S23423[96], 6}, {NULL}};
const Segment SGI69191[] = {{&S23424[42], 29}, {NULL}};
const Segment SGI69192[] = {{&S23424[42], 40}, {NULL}};
const Segment SGI69193[] = {{&S23424[30], 4}, {NULL}};
const Segment SGI69194[] = {{&S23424[131], 3}, {NULL}};
const Segment SGI69195[] = {{&S23424[96], 6}, {NULL}};
const Segment SGI69196[] = {{&S23425[42], 29}, {NULL}};
const Segment SGI69197[] = {{&S23425[42], 40}, {NULL}};
const Segment SGI69198[] = {{&S23425[30], 4}, {NULL}};
const Segment SGI69199[] = {{&S23425[131], 3}, {NULL}};
const Segment SGI69200[] = {{&S23425[96], 6}, {NULL}};
const Segment SGI69201[] = {{&S23426[42], 10}, {NULL}};
const Segment SGI69202[] = {{&S23426[42], 21}, {NULL}};
const Segment SGI69203[] = {{&S23426[30], 4}, {NULL}};
const Segment SGI69204[] = {{&S23426[112], 3}, {NULL}};
const Segment SGI69205[] = {{&S23426[77], 6}, {NULL}};
const Segment SGI69206[] = {{&S23427[42], 11}, {NULL}};
const Segment SGI69207[] = {{&S23427[42], 22}, {NULL}};
const Segment SGI69208[] = {{&S23427[30], 4}, {NULL}};
const Segment SGI69209[] = {{&S23427[113], 3}, {NULL}};
const Segment SGI69210[] = {{&S23427[78], 6}, {NULL}};
const Segment SGI69211[] = {{&S23428[42], 29}, {NULL}};
const Segment SGI69212[] = {{&S23428[42], 40}, {NULL}};
const Segment SGI69213[] = {{&S23428[30], 4}, {NULL}};
const Segment SGI69214[] = {{&S23428[131], 3}, {NULL}};
const Segment SGI69215[] = {{&S23428[96], 6}, {NULL}};
const Segment SGI69216[] = {{&S23429[36], 10}, {NULL}};
const Segment SGI69217[] = {{&S23429[31], 4}, {NULL}};
const Segment SGI69218[] = {{&S23430[42], 10}, {NULL}};
const Segment SGI69219[] = {{&S23430[42], 21}, {NULL}};
const Segment SGI69220[] = {{&S23430[30], 4}, {NULL}};
const Segment SGI69221[] = {{&S23430[112], 3}, {NULL}};
const Segment SGI69222[] = {{&S23430[77], 6}, {NULL}};
const Segment SGI69223[] = {{&S23431[36], 11}, {NULL}};
const Segment SGI69224[] = {{&S23431[31], 4}, {NULL}};
const Segment SGI69225[] = {{&S23432[36], 11}, {NULL}};
const Segment SGI69226[] = {{&S23432[31], 4}, {NULL}};
const Segment SGI69227[] = {{&S23433[42], 10}, {NULL}};
const Segment SGI69228[] = {{&S23433[42], 21}, {NULL}};
const Segment SGI69229[] = {{&S23433[30], 4}, {NULL}};
const Segment SGI69230[] = {{&S23433[31], 3}, {NULL}};
const Segment SGI69231[] = {{&S23433[87], 6}, {NULL}};
const Segment SGI69232[] = {{&S23433[142], 8}, {NULL}};
const Segment SGI69233[] = {{&S23434[40], 10}, {NULL}};
const Segment SGI69234[] = {{&S23434[30], 4}, {NULL}};
const Segment SGI69235[] = {{&S23434[110], 3}, {NULL}};
const Segment SGI69236[] = {{&S23434[75], 6}, {NULL}};
const Segment SGI69237[] = {{&S23435[42], 29}, {NULL}};
const Segment SGI69238[] = {{&S23435[42], 44}, {NULL}};
const Segment SGI69239[] = {{&S23435[30], 4}, {NULL}};
const Segment SGI69240[] = {{&S23435[135], 3}, {NULL}};
const Segment SGI69241[] = {{&S23435[100], 6}, {NULL}};
const Segment SGI69242[] = {{&S23436[40], 11}, {NULL}};
const Segment SGI69243[] = {{&S23436[30], 4}, {NULL}};
const Segment SGI69244[] = {{&S23436[111], 3}, {NULL}};
const Segment SGI69245[] = {{&S23436[76], 6}, {NULL}};
const Segment SGI69246[] = {{&S23437[36], 11}, {NULL}};
const Segment SGI69247[] = {{&S23437[31], 4}, {NULL}};
const Segment SGI69248[] = {{&S23438[42], 29}, {NULL}};
const Segment SGI69249[] = {{&S23438[42], 40}, {NULL}};
const Segment SGI69250[] = {{&S23438[30], 4}, {NULL}};
const Segment SGI69251[] = {{&S23438[131], 3}, {NULL}};
const Segment SGI69252[] = {{&S23438[96], 6}, {NULL}};
const Segment SGI69253[] = {{&S23439[42], 10}, {NULL}};
const Segment SGI69254[] = {{&S23439[42], 21}, {NULL}};
const Segment SGI69255[] = {{&S23439[30], 4}, {NULL}};
const Segment SGI69256[] = {{&S23439[111], 3}, {NULL}};
const Segment SGI69257[] = {{&S23439[77], 5}, {NULL}};
const Segment SGI69258[] = {{&S23440[42], 10}, {NULL}};
const Segment SGI69259[] = {{&S23440[42], 21}, {NULL}};
const Segment SGI69260[] = {{&S23440[30], 4}, {NULL}};
const Segment SGI69261[] = {{&S23440[80], 4}, {NULL}};
const Segment SGI69262[] = {{&S23440[80], 3}, {NULL}};
const Segment SGI69263[] = {{&S23441[42], 10}, {NULL}};
const Segment SGI69264[] = {{&S23441[42], 21}, {NULL}};
const Segment SGI69265[] = {{&S23441[30], 4}, {NULL}};
const Segment SGI69266[] = {{&S23441[113], 3}, {NULL}};
const Segment SGI69267[] = {{&S23441[77], 7}, {NULL}};
const Segment SGI69268[] = {{&S23441[133], 8}, {NULL}};
const Segment SGI69269[] = {{&S23442[42], 9}, {NULL}};
const Segment SGI69270[] = {{&S23442[42], 20}, {NULL}};
const Segment SGI69271[] = {{&S23442[30], 4}, {NULL}};
const Segment SGI69272[] = {{&S23442[111], 3}, {NULL}};
const Segment SGI69273[] = {{&S23442[76], 6}, {NULL}};
const Segment SGI69274[] = {{&S23443[42], 9}, {NULL}};
const Segment SGI69275[] = {{&S23443[30], 4}, {NULL}};
const Segment SGI69276[] = {{&S23444[42], 10}, {NULL}};
const Segment SGI69277[] = {{&S23444[42], 21}, {NULL}};
const Segment SGI69278[] = {{&S23444[30], 4}, {NULL}};
const Segment SGI69279[] = {{&S23444[80], 6}, {NULL}};
const Segment SGI69280[] = {{&S23445[44], 10}, {NULL}};
const Segment SGI69281[] = {{&S23445[32], 4}, {NULL}};
const Segment SGI69282[] = {{&S23446[43], 10}, {NULL}};
const Segment SGI69283[] = {{&S23446[43], 21}, {NULL}};
const Segment SGI69284[] = {{&S23446[31], 4}, {NULL}};
const Segment SGI69285[] = {{&S23448[42], 10}, {NULL}};
const Segment SGI69286[] = {{&S23448[42], 27}, {NULL}};
const Segment SGI69287[] = {{&S23448[30], 4}, {NULL}};
const Segment SGI69288[] = {{&S23449[42], 10}, {&S23449[170], 18}, {NULL}};
const Segment SGI69289[] = {{&S23449[42], 21}, {NULL}};
const Segment SGI69290[] = {{&S23449[30], 4}, {NULL}};
const Segment SGI69291[] = {{&S23449[112], 3}, {NULL}};
const Segment SGI69292[] = {{&S23449[77], 6}, {NULL}};
const Segment SGI69293[] = {{&S23450[42], 29}, {NULL}};
const Segment SGI69294[] = {{&S23450[42], 40}, {NULL}};
const Segment SGI69295[] = {{&S23450[30], 4}, {NULL}};
const Segment SGI69296[] = {{&S23450[131], 3}, {NULL}};
const Segment SGI69297[] = {{&S23450[96], 6}, {NULL}};
const Segment SGI69298[] = {{&S23451[42], 10}, {NULL}};
const Segment SGI69299[] = {{&S23451[42], 21}, {NULL}};
const Segment SGI69300[] = {{&S23451[30], 4}, {NULL}};
const Segment SGI69301[] = {{&S23451[112], 3}, {NULL}};
const Segment SGI69302[] = {{&S23451[77], 6}, {NULL}};
const Segment SGI69303[] = {{&S23452[42], 10}, {NULL}};
const Segment SGI69304[] = {{&S23452[42], 21}, {NULL}};
const Segment SGI69305[] = {{&S23452[30], 4}, {NULL}};
const Segment SGI69306[] = {{&S23452[77], 6}, {NULL}};
const Segment SGI69307[] = {{&S23453[53], 18}, {NULL}};
const Segment SGI69308[] = {{&S23453[48], 4}, {NULL}};
const Segment SGI69309[] = {{&S23454[42], 9}, {NULL}};
const Segment SGI69310[] = {{&S23454[30], 4}, {NULL}};
const Segment SGI69311[] = {{&S23454[101], 3}, {NULL}};
const Segment SGI69312[] = {{&S23454[65], 7}, {NULL}};
const Segment SGI69313[] = {{&S23454[121], 8}, {NULL}};
const Segment SGI69314[] = {{&S23455[42], 10}, {NULL}};
const Segment SGI69315[] = {{&S23455[42], 31}, {NULL}};
const Segment SGI69316[] = {{&S23455[30], 4}, {NULL}};
const Segment SGI69317[] = {{&S23455[122], 3}, {NULL}};
const Segment SGI69318[] = {{&S23455[87], 6}, {NULL}};
const Segment SGI69319[] = {{&S23456[50], 11}, {NULL}};
const Segment SGI69320[] = {{&S23456[50], 22}, {NULL}};
const Segment SGI69321[] = {{&S23456[38], 4}, {NULL}};
const Segment SGI69322[] = {{&S23457[42], 10}, {NULL}};
const Segment SGI69323[] = {{&S23457[30], 4}, {NULL}};
const Segment SGI69324[] = {{&S23870[44], 18}, {NULL}};
const Segment SGI69325[] = {{&S23870[44], 29}, {NULL}};
const Segment SGI69326[] = {{&S23870[30], 6}, {NULL}};
const Segment SGI69327[] = {{&S23459[42], 10}, {NULL}};
const Segment SGI69328[] = {{&S23459[42], 21}, {&S23459[266], 7}, {NULL}};
const Segment SGI69329[] = {{&S23870[138], 3}, {NULL}};
const Segment SGI69330[] = {{&S23459[30], 4}, {NULL}};
const Segment SGI69331[] = {{&S23870[104], 5}, {NULL}};
const Segment SGI69332[] = {{&S23459[112], 3}, {NULL}};
const Segment SGI69333[] = {{&S23459[77], 6}, {NULL}};
const Segment SGI69334[] = {{&S23461[42], 10}, {NULL}};
const Segment SGI69335[] = {{&S23461[42], 27}, {NULL}};
const Segment SGI69336[] = {{&S23461[30], 4}, {NULL}};
const Segment SGI69337[] = {{&S23463[43], 10}, {NULL}};
const Segment SGI69338[] = {{&S23463[43], 27}, {NULL}};
const Segment SGI69339[] = {{&S23463[31], 4}, {NULL}};
const Segment SGI69340[] = {{&S23464[42], 10}, {NULL}};
const Segment SGI69341[] = {{&S23464[42], 27}, {NULL}};
const Segment SGI69342[] = {{&S23464[30], 4}, {NULL}};
const Segment SGI69343[] = {{&S23465[97], 29}, {NULL}};
const Segment SGI69344[] = {{&S23466[42], 10}, {NULL}};
const Segment SGI69345[] = {{&S23466[42], 19}, {NULL}};
const Segment SGI69346[] = {{&S23466[30], 4}, {NULL}};
const Segment SGI69347[] = {{&S23466[109], 3}, {NULL}};
const Segment SGI69348[] = {{&S23466[75], 5}, {NULL}};
const Segment SGI69349[] = {{&S23467[42], 11}, {NULL}};
const Segment SGI69350[] = {{&S23467[42], 28}, {NULL}};
const Segment SGI69351[] = {{&S23467[30], 4}, {NULL}};
const Segment SGI69352[] = {{&S23468[42], 10}, {&S23468[42], 10}, {NULL}};
const Segment SGI69353[] = {{&S23468[42], 21}, {&S23468[42], 21}, {NULL}};
const Segment SGI69354[] = {{&S23468[30], 4}, {&S23468[30], 4}, {NULL}};
const Segment SGI69355[] = {{&S23468[112], 3}, {&S23468[112], 3}, {NULL}};
const Segment SGI69356[] = {{&S23468[77], 6}, {&S23468[77], 6}, {NULL}};
const Segment SGI69357[] = {{&S23469[30], 1}, {NULL}};
const Segment SGI69358[] = {{&S23469[61], 10}, {NULL}};
const Segment SGI69359[] = {{&S23469[49], 4}, {NULL}};
const Segment SGI69360[] = {{&S23471[42], 29}, {NULL}};
const Segment SGI69361[] = {{&S23471[42], 40}, {NULL}};
const Segment SGI69362[] = {{&S23471[30], 4}, {NULL}};
const Segment SGI69363[] = {{&S23471[131], 3}, {NULL}};
const Segment SGI69364[] = {{&S23471[96], 6}, {NULL}};
const Segment SGI69365[] = {{&S23472[30], 1}, {NULL}};
const Segment SGI69366[] = {{&S23472[61], 10}, {NULL}};
const Segment SGI69367[] = {{&S23472[49], 4}, {NULL}};
const Segment SGI69368[] = {{&S23473[42], 10}, {NULL}};
const Segment SGI69369[] = {{&S23473[42], 21}, {NULL}};
const Segment SGI69370[] = {{&S23473[30], 4}, {NULL}};
const Segment SGI69371[] = {{&S23473[112], 3}, {&S23473[192], 3}, {NULL}};
const Segment SGI69372[] = {{&S23473[77], 6}, {NULL}};
const Segment SGI69373[] = {{&S23474[42], 11}, {NULL}};
const Segment SGI69374[] = {{&S23474[42], 22}, {NULL}};
const Segment SGI69375[] = {{&S23474[30], 4}, {NULL}};
const Segment SGI69376[] = {{&S23474[114], 3}, {NULL}};
const Segment SGI69377[] = {{&S23474[78], 7}, {NULL}};
const Segment SGI69378[] = {{&S23474[134], 8}, {NULL}};
const Segment SGI69379[] = {{&S23475[42], 10}, {NULL}};
const Segment SGI69380[] = {{&S23475[42], 21}, {&S23475[205], 13}, {NULL}};
const Segment SGI69381[] = {{&S23475[30], 4}, {NULL}};
const Segment SGI69382[] = {{&S23475[112], 3}, {NULL}};
const Segment SGI69383[] = {{&S23475[77], 6}, {NULL}};
const Segment SGI69384[] = {{&S23476[42], 10}, {NULL}};
const Segment SGI69385[] = {{&S23476[42], 21}, {&S23476[202], 7}, {NULL}};
const Segment SGI69386[] = {{&S23476[30], 4}, {NULL}};
const Segment SGI69387[] = {{&S23476[112], 3}, {NULL}};
const Segment SGI69388[] = {{&S23476[77], 6}, {NULL}};
const Segment SGI69389[] = {{&S23477[42], 11}, {NULL}};
const Segment SGI69390[] = {{&S23477[42], 22}, {NULL}};
const Segment SGI69391[] = {{&S23477[30], 4}, {NULL}};
const Segment SGI69392[] = {{&S23478[42], 11}, {NULL}};
const Segment SGI69393[] = {{&S23478[42], 22}, {&S23478[203], 7}, {NULL}};
const Segment SGI69394[] = {{&S23478[30], 4}, {NULL}};
const Segment SGI69395[] = {{&S23478[113], 3}, {NULL}};
const Segment SGI69396[] = {{&S23478[78], 6}, {NULL}};
const Segment SGI69397[] = {{&S23479[32], 10}, {NULL}};
const Segment SGI69398[] = {{&S23479[32], 21}, {NULL}};
const Segment SGI69399[] = {{&S23480[42], 10}, {NULL}};
const Segment SGI69400[] = {{&S23480[42], 21}, {&S23480[43], 8}, {NULL}};
const Segment SGI69401[] = {{&S23480[30], 4}, {NULL}};
const Segment SGI69402[] = {{&S23480[112], 3}, {NULL}};
const Segment SGI69403[] = {{&S23480[77], 6}, {NULL}};
const Segment SGI69404[] = {{&S23481[42], 11}, {NULL}};
const Segment SGI69405[] = {{&S23481[42], 32}, {NULL}};
const Segment SGI69406[] = {{&S23481[30], 4}, {NULL}};
const Segment SGI69407[] = {{&S23481[123], 3}, {NULL}};
const Segment SGI69408[] = {{&S23481[88], 6}, {NULL}};
const Segment SGI69409[] = {{&S23482[53], 19}, {NULL}};
const Segment SGI69410[] = {{&S23482[48], 4}, {NULL}};
const Segment SGI69411[] = {{&S23483[42], 10}, {NULL}};
const Segment SGI69412[] = {{&S23483[42], 21}, {NULL}};
const Segment SGI69413[] = {{&S23483[30], 4}, {NULL}};
const Segment SGI69414[] = {{&S23483[82], 3}, {NULL}};
const Segment SGI69415[] = {{&S23485[42], 11}, {NULL}};
const Segment SGI69416[] = {{&S23485[42], 22}, {NULL}};
const Segment SGI69417[] = {{&S23485[30], 4}, {NULL}};
const Segment SGI69418[] = {{&S23485[113], 3}, {&S23485[149], 3}, {NULL}};
const Segment SGI69419[] = {{&S23485[78], 6}, {NULL}};
const Segment SGI69420[] = {{&S23486[42], 11}, {NULL}};
const Segment SGI69421[] = {{&S23486[42], 22}, {NULL}};
const Segment SGI69422[] = {{&S23486[30], 4}, {NULL}};
const Segment SGI69423[] = {{&S23486[114], 3}, {NULL}};
const Segment SGI69424[] = {{&S23486[78], 7}, {NULL}};
const Segment SGI69425[] = {{&S23486[134], 8}, {NULL}};
const Segment SGI69426[] = {{&S23487[42], 11}, {NULL}};
const Segment SGI69427[] = {{&S23487[42], 22}, {NULL}};
const Segment SGI69428[] = {{&S23487[30], 4}, {NULL}};
const Segment SGI69429[] = {{&S23487[114], 3}, {NULL}};
const Segment SGI69430[] = {{&S23487[78], 7}, {NULL}};
const Segment SGI69431[] = {{&S23487[134], 8}, {NULL}};
const Segment SGI69432[] = {{&S23488[42], 29}, {NULL}};
const Segment SGI69433[] = {{&S23488[42], 40}, {NULL}};
const Segment SGI69434[] = {{&S23488[30], 4}, {NULL}};
const Segment SGI69435[] = {{&S23488[131], 3}, {NULL}};
const Segment SGI69436[] = {{&S23488[96], 6}, {NULL}};
const Segment SGI69437[] = {{&S23489[44], 22}, {NULL}};
const Segment SGI69438[] = {{&S29965[44], 10}, {NULL}};
const Segment SGI69439[] = {{&S29965[44], 27}, {NULL}};
const Segment SGI69440[] = {{&S29965[30], 6}, {NULL}};
const Segment SGI69441[] = {{&S29965[119], 3}, {NULL}};
const Segment SGI69442[] = {{&S29965[85], 5}, {NULL}};
const Segment SGI69443[] = {{&S29968[38], 10}, {NULL}};
const Segment SGI69444[] = {{&S29968[31], 6}, {NULL}};
const Segment SGI69445[] = {{&S29972[44], 10}, {NULL}};
const Segment SGI69446[] = {{&S29972[44], 30}, {NULL}};
const Segment SGI69447[] = {{&S29972[30], 6}, {NULL}};
const Segment SGI69448[] = {{&S29972[122], 3}, {NULL}};
const Segment SGI69449[] = {{&S29972[88], 5}, {NULL}};
const Segment SGI69450[] = {{&S29974[42], 10}, {NULL}};
const Segment SGI69451[] = {{&S29974[30], 6}, {NULL}};
const Segment SGI69452[] = {{&S29974[117], 3}, {NULL}};
const Segment SGI69453[] = {{&S29974[83], 5}, {NULL}};
const Segment SGI69454[] = {{&S29975[44], 10}, {NULL}};
const Segment SGI69455[] = {{&S29975[44], 29}, {NULL}};
const Segment SGI69456[] = {{&S29975[30], 6}, {NULL}};
const Segment SGI69457[] = {{&S29975[121], 3}, {NULL}};
const Segment SGI69458[] = {{&S29975[87], 5}, {NULL}};
const Segment SGI69459[] = {{&S24464[44], 10}, {NULL}};
const Segment SGI69460[] = {{&S24464[44], 20}, {NULL}};
const Segment SGI69461[] = {{&S24464[30], 6}, {NULL}};
const Segment SGI69462[] = {{&S24464[112], 3}, {NULL}};
const Segment SGI69463[] = {{&S24464[78], 5}, {NULL}};
const Segment SGI69464[] = {{&S24469[38], 10}, {NULL}};
const Segment SGI69465[] = {{&S24469[31], 6}, {NULL}};
const Segment SGI69466[] = {{&S24474[44], 10}, {NULL}};
const Segment SGI69467[] = {{&S24474[44], 21}, {NULL}};
const Segment SGI69468[] = {{&S24474[30], 6}, {NULL}};
const Segment SGI69469[] = {{&S24474[113], 3}, {NULL}};
const Segment SGI69470[] = {{&S24474[79], 5}, {NULL}};
const Segment SGI69471[] = {{&S24484[44], 10}, {NULL}};
const Segment SGI69472[] = {{&S24484[44], 27}, {NULL}};
const Segment SGI69473[] = {{&S24484[30], 6}, {NULL}};
const Segment SGI69474[] = {{&S24484[119], 3}, {NULL}};
const Segment SGI69475[] = {{&S24484[85], 5}, {NULL}};
const Segment SGI69476[] = {{&S24491[44], 10}, {NULL}};
const Segment SGI69477[] = {{&S24491[44], 27}, {NULL}};
const Segment SGI69478[] = {{&S24491[30], 6}, {NULL}};
const Segment SGI69479[] = {{&S24491[88], 6}, {NULL}};
const Segment SGI69480[] = {{&S24532[44], 10}, {NULL}};
const Segment SGI69481[] = {{&S24532[44], 26}, {NULL}};
const Segment SGI69482[] = {{&S24532[30], 6}, {NULL}};
const Segment SGI69483[] = {{&S24532[87], 6}, {NULL}};
const Segment SGI69484[] = {{&S24539[44], 10}, {NULL}};
const Segment SGI69485[] = {{&S24539[44], 27}, {NULL}};
const Segment SGI69486[] = {{&S24539[30], 6}, {NULL}};
const Segment SGI69487[] = {{&S24539[88], 4}, {NULL}};
const Segment SGI69488[] = {{&S24539[88], 3}, {NULL}};
const Segment SGI69489[] = {{&S30450[44], 10}, {NULL}};
const Segment SGI69490[] = {{&S30450[44], 19}, {NULL}};
const Segment SGI69491[] = {{&S30450[30], 6}, {NULL}};
const Segment SGI69492[] = {{&S30450[111], 3}, {NULL}};
const Segment SGI69493[] = {{&S30450[77], 5}, {NULL}};
const Segment SGI69494[] = {{&S30451[44], 10}, {NULL}};
const Segment SGI69495[] = {{&S30451[44], 27}, {NULL}};
const Segment SGI69496[] = {{&S30451[30], 6}, {NULL}};
const Segment SGI69497[] = {{&S30451[119], 3}, {NULL}};
const Segment SGI69498[] = {{&S30451[85], 5}, {NULL}};
const Segment SGI69499[] = {{&S30452[38], 10}, {NULL}};
const Segment SGI69500[] = {{&S30452[31], 6}, {NULL}};
const Segment SGI69501[] = {{&S30453[44], 10}, {NULL}};
const Segment SGI69502[] = {{&S30453[44], 19}, {NULL}};
const Segment SGI69503[] = {{&S30453[30], 6}, {NULL}};
const Segment SGI69504[] = {{&S30453[111], 3}, {NULL}};
const Segment SGI69505[] = {{&S30453[77], 5}, {NULL}};
const Segment SGI69506[] = {{&S30454[44], 10}, {NULL}};
const Segment SGI69507[] = {{&S30454[44], 19}, {NULL}};
const Segment SGI69508[] = {{&S30454[30], 6}, {NULL}};
const Segment SGI69509[] = {{&S30454[111], 3}, {NULL}};
const Segment SGI69510[] = {{&S30454[77], 5}, {NULL}};
const Segment SGI69511[] = {{&S30455[44], 10}, {NULL}};
const Segment SGI69512[] = {{&S30455[44], 19}, {NULL}};
const Segment SGI69513[] = {{&S30455[30], 6}, {NULL}};
const Segment SGI69514[] = {{&S30455[111], 3}, {NULL}};
const Segment SGI69515[] = {{&S30455[77], 5}, {NULL}};
const Segment SGI69516[] = {{&S24555[44], 10}, {NULL}};
const Segment SGI69517[] = {{&S24555[44], 22}, {NULL}};
const Segment SGI69518[] = {{&S24555[30], 6}, {NULL}};
const Segment SGI69519[] = {{&S24555[83], 6}, {NULL}};
const Segment SGI69520[] = {{&S24570[44], 11}, {NULL}};
const Segment SGI69521[] = {{&S24570[44], 28}, {NULL}};
const Segment SGI69522[] = {{&S24570[30], 6}, {NULL}};
const Segment SGI69523[] = {{&S24570[120], 3}, {NULL}};
const Segment SGI69524[] = {{&S24570[86], 5}, {NULL}};
const Segment SGI69525[] = {{&S24586[44], 10}, {NULL}};
const Segment SGI69526[] = {{&S24586[44], 22}, {NULL}};
const Segment SGI69527[] = {{&S24586[30], 6}, {NULL}};
const Segment SGI69528[] = {{&S24586[83], 4}, {NULL}};
const Segment SGI69529[] = {{&S24586[83], 3}, {NULL}};
const Segment SGI69530[] = {{&S30606[44], 11}, {NULL}};
const Segment SGI69531[] = {{&S30606[44], 23}, {NULL}};
const Segment SGI69532[] = {{&S30606[30], 6}, {NULL}};
const Segment SGI69533[] = {{&S30606[115], 3}, {NULL}};
const Segment SGI69534[] = {{&S30606[81], 5}, {NULL}};
const Segment SGI69535[] = {{&S30611[38], 11}, {NULL}};
const Segment SGI69536[] = {{&S30611[31], 6}, {NULL}};
const Segment SGI69537[] = {{&S30617[44], 11}, {NULL}};
const Segment SGI69538[] = {{&S30617[44], 22}, {NULL}};
const Segment SGI69539[] = {{&S30617[30], 6}, {NULL}};
const Segment SGI69540[] = {{&S30617[114], 3}, {NULL}};
const Segment SGI69541[] = {{&S30617[80], 5}, {NULL}};
const Segment SGI69542[] = {{&S30624[42], 11}, {NULL}};
const Segment SGI69543[] = {{&S30624[30], 6}, {NULL}};
const Segment SGI69544[] = {{&S30624[113], 3}, {NULL}};
const Segment SGI69545[] = {{&S30624[79], 5}, {NULL}};
const Segment SGI69546[] = {{&S30626[44], 11}, {NULL}};
const Segment SGI69547[] = {{&S30626[44], 23}, {NULL}};
const Segment SGI69548[] = {{&S30626[30], 6}, {NULL}};
const Segment SGI69549[] = {{&S30626[115], 3}, {NULL}};
const Segment SGI69550[] = {{&S30626[81], 5}, {NULL}};
const Segment SGI69551[] = {{&S30627[44], 11}, {NULL}};
const Segment SGI69552[] = {{&S30627[44], 23}, {NULL}};
const Segment SGI69553[] = {{&S30627[30], 6}, {NULL}};
const Segment SGI69554[] = {{&S30627[84], 4}, {NULL}};
const Segment SGI69555[] = {{&S30627[84], 3}, {NULL}};
const Segment SGI69556[] = {{&S24354[42], 11}, {NULL}};
const Segment SGI69557[] = {{&S24354[42], 23}, {NULL}};
const Segment SGI69558[] = {{&S24354[30], 4}, {NULL}};
const Segment SGI69559[] = {{&S24354[114], 3}, {NULL}};
const Segment SGI69560[] = {{&S24354[79], 6}, {NULL}};
const Segment SGI69561[] = {{&S24355[36], 10}, {NULL}};
const Segment SGI69562[] = {{&S24355[31], 4}, {NULL}};
const Segment SGI69563[] = {{&S24356[42], 29}, {NULL}};
const Segment SGI69564[] = {{&S24356[42], 41}, {NULL}};
const Segment SGI69565[] = {{&S24356[30], 4}, {NULL}};
const Segment SGI69566[] = {{&S24356[132], 3}, {NULL}};
const Segment SGI69567[] = {{&S24356[97], 6}, {NULL}};
const Segment SGI69568[] = {{&S24357[42], 10}, {NULL}};
const Segment SGI69569[] = {{&S24357[42], 22}, {NULL}};
const Segment SGI69570[] = {{&S24357[30], 4}, {NULL}};
const Segment SGI69571[] = {{&S24357[113], 3}, {NULL}};
const Segment SGI69572[] = {{&S24357[78], 6}, {NULL}};
const Segment SGI69573[] = {{&S24358[42], 11}, {NULL}};
const Segment SGI69574[] = {{&S24358[42], 23}, {NULL}};
const Segment SGI69575[] = {{&S24358[30], 4}, {NULL}};
const Segment SGI69576[] = {{&S24358[114], 3}, {NULL}};
const Segment SGI69577[] = {{&S24358[79], 6}, {NULL}};
const Segment SGI69578[] = {{&S24359[36], 11}, {NULL}};
const Segment SGI69579[] = {{&S24359[31], 4}, {NULL}};
const Segment SGI69580[] = {{&S24360[42], 29}, {NULL}};
const Segment SGI69581[] = {{&S24360[42], 41}, {NULL}};
const Segment SGI69582[] = {{&S24360[30], 4}, {NULL}};
const Segment SGI69583[] = {{&S24360[132], 3}, {NULL}};
const Segment SGI69584[] = {{&S24360[97], 6}, {NULL}};
const Segment SGI69585[] = {{&S24361[42], 28}, {NULL}};
const Segment SGI69586[] = {{&S24361[42], 40}, {NULL}};
const Segment SGI69587[] = {{&S24361[30], 4}, {NULL}};
const Segment SGI69588[] = {{&S24361[131], 3}, {NULL}};
const Segment SGI69589[] = {{&S24361[96], 6}, {NULL}};
const Segment SGI69590[] = {{&S24362[42], 28}, {NULL}};
const Segment SGI69591[] = {{&S24362[42], 40}, {NULL}};
const Segment SGI69592[] = {{&S24362[30], 4}, {NULL}};
const Segment SGI69593[] = {{&S24362[131], 3}, {NULL}};
const Segment SGI69594[] = {{&S24362[96], 6}, {NULL}};
const Segment SGI69595[] = {{&S24363[40], 10}, {NULL}};
const Segment SGI69596[] = {{&S24363[30], 4}, {NULL}};
const Segment SGI69597[] = {{&S24363[111], 3}, {NULL}};
const Segment SGI69598[] = {{&S24363[76], 6}, {NULL}};
const Segment SGI69599[] = {{&S24364[42], 10}, {NULL}};
const Segment SGI69600[] = {{&S24364[42], 22}, {NULL}};
const Segment SGI69601[] = {{&S24364[30], 4}, {NULL}};
const Segment SGI69602[] = {{&S24364[113], 3}, {NULL}};
const Segment SGI69603[] = {{&S24364[78], 6}, {NULL}};
const Segment SGI69604[] = {{&S24365[42], 29}, {NULL}};
const Segment SGI69605[] = {{&S24365[42], 41}, {NULL}};
const Segment SGI69606[] = {{&S24365[30], 4}, {NULL}};
const Segment SGI69607[] = {{&S24365[132], 3}, {NULL}};
const Segment SGI69608[] = {{&S24365[97], 6}, {NULL}};
const Segment SGI69609[] = {{&S24366[42], 29}, {NULL}};
const Segment SGI69610[] = {{&S24366[42], 41}, {NULL}};
const Segment SGI69611[] = {{&S24366[30], 4}, {NULL}};
const Segment SGI69612[] = {{&S24366[132], 3}, {NULL}};
const Segment SGI69613[] = {{&S24366[97], 6}, {NULL}};
const Segment SGI69614[] = {{&S24367[42], 11}, {NULL}};
const Segment SGI69615[] = {{&S24367[42], 23}, {NULL}};
const Segment SGI69616[] = {{&S24367[30], 4}, {NULL}};
const Segment SGI69617[] = {{&S24367[114], 3}, {NULL}};
const Segment SGI69618[] = {{&S24367[79], 6}, {NULL}};
const Segment SGI69619[] = {{&S24368[42], 11}, {NULL}};
const Segment SGI69620[] = {{&S24368[42], 29}, {NULL}};
const Segment SGI69621[] = {{&S24368[30], 4}, {NULL}};
const Segment SGI69622[] = {{&S24369[42], 28}, {NULL}};
const Segment SGI69623[] = {{&S24369[42], 40}, {NULL}};
const Segment SGI69624[] = {{&S24369[30], 4}, {NULL}};
const Segment SGI69625[] = {{&S24369[131], 3}, {NULL}};
const Segment SGI69626[] = {{&S24369[96], 6}, {NULL}};
const Segment SGI69627[] = {{&S24370[36], 11}, {NULL}};
const Segment SGI69628[] = {{&S24370[31], 4}, {NULL}};
const Segment SGI69629[] = {{&S24371[42], 10}, {NULL}};
const Segment SGI69630[] = {{&S24371[42], 22}, {NULL}};
const Segment SGI69631[] = {{&S24371[30], 4}, {NULL}};
const Segment SGI69632[] = {{&S24371[81], 4}, {NULL}};
const Segment SGI69633[] = {{&S24371[81], 3}, {NULL}};
const Segment SGI69634[] = {{&S24372[42], 28}, {NULL}};
const Segment SGI69635[] = {{&S24372[42], 40}, {NULL}};
const Segment SGI69636[] = {{&S24372[30], 4}, {NULL}};
const Segment SGI69637[] = {{&S24372[131], 3}, {NULL}};
const Segment SGI69638[] = {{&S24372[96], 6}, {NULL}};
const Segment SGI69639[] = {{&S24373[42], 10}, {NULL}};
const Segment SGI69640[] = {{&S24373[42], 22}, {NULL}};
const Segment SGI69641[] = {{&S24373[30], 4}, {NULL}};
const Segment SGI69642[] = {{&S24373[81], 6}, {NULL}};
const Segment SGI69643[] = {{&S24374[42], 9}, {NULL}};
const Segment SGI69644[] = {{&S24374[30], 4}, {NULL}};
const Segment SGI69645[] = {{&S24375[42], 29}, {NULL}};
const Segment SGI69646[] = {{&S24375[42], 41}, {NULL}};
const Segment SGI69647[] = {{&S24375[30], 4}, {NULL}};
const Segment SGI69648[] = {{&S24375[132], 3}, {NULL}};
const Segment SGI69649[] = {{&S24375[97], 6}, {NULL}};
const Segment SGI69650[] = {{&S24376[53], 18}, {NULL}};
const Segment SGI69651[] = {{&S24376[48], 4}, {NULL}};
const Segment SGI69652[] = {{&S24377[40], 11}, {NULL}};
const Segment SGI69653[] = {{&S24377[30], 4}, {NULL}};
const Segment SGI69654[] = {{&S24377[112], 3}, {NULL}};
const Segment SGI69655[] = {{&S24377[77], 6}, {NULL}};
const Segment SGI69656[] = {{&S24378[42], 10}, {NULL}};
const Segment SGI69657[] = {{&S24378[42], 21}, {NULL}};
const Segment SGI69658[] = {{&S24378[30], 4}, {NULL}};
const Segment SGI69659[] = {{&S24378[130], 3}, {NULL}};
const Segment SGI69660[] = {{&S24378[96], 5}, {NULL}};
const Segment SGI69661[] = {{&S24379[32], 10}, {NULL}};
const Segment SGI69662[] = {{&S24379[32], 22}, {NULL}};
const Segment SGI69663[] = {{&S24380[63], 10}, {NULL}};
const Segment SGI69664[] = {{&S24380[63], 22}, {NULL}};
const Segment SGI69665[] = {{&S24380[51], 4}, {NULL}};
const Segment SGI69666[] = {{&S24381[42], 22}, {NULL}};
const Segment SGI69667[] = {{&S24381[30], 4}, {NULL}};
const Segment SGI69668[] = {{&S24382[40], 11}, {NULL}};
const Segment SGI69669[] = {{&S24382[30], 4}, {NULL}};
const Segment SGI69670[] = {{&S24382[112], 3}, {NULL}};
const Segment SGI69671[] = {{&S24382[77], 6}, {NULL}};
const Segment SGI69672[] = {{&S24383[40], 29}, {NULL}};
const Segment SGI69673[] = {{&S24383[30], 4}, {NULL}};
const Segment SGI69674[] = {{&S24383[130], 3}, {NULL}};
const Segment SGI69675[] = {{&S24383[95], 6}, {NULL}};
const Segment SGI69676[] = {{&S24384[43], 10}, {NULL}};
const Segment SGI69677[] = {{&S24384[30], 4}, {NULL}};
const Segment SGI69678[] = {{&S24384[114], 3}, {NULL}};
const Segment SGI69679[] = {{&S24384[79], 6}, {NULL}};
const Segment SGI69680[] = {{&S24385[42], 10}, {NULL}};
const Segment SGI69681[] = {{&S24385[42], 28}, {NULL}};
const Segment SGI69682[] = {{&S24385[30], 4}, {NULL}};
const Segment SGI69683[] = {{&S24387[42], 10}, {NULL}};
const Segment SGI69684[] = {{&S24387[42], 22}, {NULL}};
const Segment SGI69685[] = {{&S24387[30], 4}, {NULL}};
const Segment SGI69686[] = {{&S24387[113], 3}, {NULL}};
const Segment SGI69687[] = {{&S24387[78], 6}, {NULL}};
const Segment SGI69688[] = {{&S24388[42], 10}, {NULL}};
const Segment SGI69689[] = {{&S24388[42], 28}, {NULL}};
const Segment SGI69690[] = {{&S24388[30], 4}, {NULL}};
const Segment SGI69691[] = {{&S24389[42], 10}, {NULL}};
const Segment SGI69692[] = {{&S24389[42], 22}, {&S24389[206], 10}, {NULL}};
const Segment SGI69693[] = {{&S24389[30], 4}, {NULL}};
const Segment SGI69694[] = {{&S24389[113], 3}, {NULL}};
const Segment SGI69695[] = {{&S24389[78], 6}, {NULL}};
const Segment SGI69696[] = {{&S24390[42], 10}, {NULL}};
const Segment SGI69697[] = {{&S24390[42], 22}, {NULL}};
const Segment SGI69698[] = {{&S24390[30], 4}, {NULL}};
const Segment SGI69699[] = {{&S24390[113], 3}, {NULL}};
const Segment SGI69700[] = {{&S24390[78], 6}, {NULL}};
const Segment SGI69701[] = {{&S24390[133], 8}, {NULL}};
const Segment SGI69702[] = {{&S24391[43], 10}, {NULL}};
const Segment SGI69703[] = {{&S24391[43], 28}, {NULL}};
const Segment SGI69704[] = {{&S24391[31], 4}, {NULL}};
const Segment SGI69705[] = {{&S24392[42], 10}, {NULL}};
const Segment SGI69706[] = {{&S24392[42], 28}, {NULL}};
const Segment SGI69707[] = {{&S24392[30], 4}, {NULL}};
const Segment SGI69708[] = {{&S24393[42], 10}, {NULL}};
const Segment SGI69709[] = {{&S24393[42], 22}, {NULL}};
const Segment SGI69710[] = {{&S24393[30], 4}, {NULL}};
const Segment SGI69711[] = {{&S24393[114], 3}, {NULL}};
const Segment SGI69712[] = {{&S24393[78], 7}, {NULL}};
const Segment SGI69713[] = {{&S24393[134], 8}, {NULL}};
const Segment SGI69714[] = {{&S24394[42], 10}, {NULL}};
const Segment SGI69715[] = {{&S24394[42], 19}, {NULL}};
const Segment SGI69716[] = {{&S24394[30], 4}, {NULL}};
const Segment SGI69717[] = {{&S24394[109], 3}, {NULL}};
const Segment SGI69718[] = {{&S24394[75], 5}, {NULL}};
const Segment SGI69719[] = {{&S24757[44], 7}, {NULL}};
const Segment SGI69720[] = {{&S24757[44], 23}, {NULL}};
const Segment SGI69721[] = {{&S24757[30], 6}, {NULL}};
const Segment SGI69722[] = {{&S24757[115], 3}, {NULL}};
const Segment SGI69723[] = {{&S24757[81], 5}, {NULL}};
const Segment SGI69724[] = {{&S24758[44], 21}, {NULL}};
const Segment SGI69725[] = {{&S24758[44], 37}, {NULL}};
const Segment SGI69726[] = {{&S24758[30], 6}, {NULL}};
const Segment SGI69727[] = {{&S24758[129], 3}, {NULL}};
const Segment SGI69728[] = {{&S24758[95], 5}, {NULL}};
const Segment SGI69729[] = {{&S24759[44], 21}, {NULL}};
const Segment SGI69730[] = {{&S24759[44], 37}, {NULL}};
const Segment SGI69731[] = {{&S24759[30], 6}, {NULL}};
const Segment SGI69732[] = {{&S24759[129], 3}, {NULL}};
const Segment SGI69733[] = {{&S24759[95], 5}, {NULL}};
const Segment SGI69734[] = {{&S24760[44], 19}, {NULL}};
const Segment SGI69735[] = {{&S24760[44], 35}, {NULL}};
const Segment SGI69736[] = {{&S24760[30], 6}, {NULL}};
const Segment SGI69737[] = {{&S24760[127], 3}, {NULL}};
const Segment SGI69738[] = {{&S24760[93], 5}, {NULL}};
const Segment SGI69739[] = {{&S24761[44], 7}, {NULL}};
const Segment SGI69740[] = {{&S24761[44], 23}, {NULL}};
const Segment SGI69741[] = {{&S24761[30], 6}, {NULL}};
const Segment SGI69742[] = {{&S24761[115], 3}, {NULL}};
const Segment SGI69743[] = {{&S24761[81], 5}, {NULL}};
const Segment SGI69744[] = {{&S24762[44], 21}, {NULL}};
const Segment SGI69745[] = {{&S24762[44], 37}, {NULL}};
const Segment SGI69746[] = {{&S24762[30], 6}, {NULL}};
const Segment SGI69747[] = {{&S24762[129], 3}, {NULL}};
const Segment SGI69748[] = {{&S24762[95], 5}, {NULL}};
const Segment SGI69749[] = {{&S24763[38], 7}, {NULL}};
const Segment SGI69750[] = {{&S24763[31], 6}, {NULL}};
const Segment SGI69751[] = {{&S24764[44], 19}, {NULL}};
const Segment SGI69752[] = {{&S24764[44], 35}, {NULL}};
const Segment SGI69753[] = {{&S24764[30], 6}, {NULL}};
const Segment SGI69754[] = {{&S24764[127], 3}, {NULL}};
const Segment SGI69755[] = {{&S24764[93], 5}, {NULL}};
const Segment SGI69756[] = {{&S24765[44], 7}, {NULL}};
const Segment SGI69757[] = {{&S24765[44], 23}, {NULL}};
const Segment SGI69758[] = {{&S24765[30], 6}, {NULL}};
const Segment SGI69759[] = {{&S24765[84], 4}, {NULL}};
const Segment SGI69760[] = {{&S24765[84], 3}, {NULL}};
const Segment SGI69761[] = {{&S24766[44], 19}, {NULL}};
const Segment SGI69762[] = {{&S24766[44], 35}, {NULL}};
const Segment SGI69763[] = {{&S24766[30], 6}, {NULL}};
const Segment SGI69764[] = {{&S24766[127], 3}, {NULL}};
const Segment SGI69765[] = {{&S24766[93], 5}, {NULL}};
const Segment SGI69766[] = {{&S24767[44], 7}, {NULL}};
const Segment SGI69767[] = {{&S24767[44], 23}, {NULL}};
const Segment SGI69768[] = {{&S24767[30], 6}, {NULL}};
const Segment SGI69769[] = {{&S24767[115], 3}, {NULL}};
const Segment SGI69770[] = {{&S24767[81], 5}, {NULL}};
const Segment SGI69771[] = {{&S24768[44], 19}, {NULL}};
const Segment SGI69772[] = {{&S24768[44], 35}, {NULL}};
const Segment SGI69773[] = {{&S24768[30], 6}, {NULL}};
const Segment SGI69774[] = {{&S24768[127], 3}, {NULL}};
const Segment SGI69775[] = {{&S24768[93], 5}, {NULL}};
const Segment SGI69776[] = {{&S24769[44], 7}, {NULL}};
const Segment SGI69777[] = {{&S24769[44], 23}, {NULL}};
const Segment SGI69778[] = {{&S24769[30], 6}, {NULL}};
const Segment SGI69779[] = {{&S24769[115], 3}, {NULL}};
const Segment SGI69780[] = {{&S24769[81], 5}, {NULL}};
const Segment SGI69781[] = {{&S24770[44], 19}, {NULL}};
const Segment SGI69782[] = {{&S24770[44], 35}, {NULL}};
const Segment SGI69783[] = {{&S24770[30], 6}, {NULL}};
const Segment SGI69784[] = {{&S24770[127], 3}, {NULL}};
const Segment SGI69785[] = {{&S24770[93], 5}, {NULL}};
const Segment SGI69786[] = {{&S24771[44], 7}, {NULL}};
const Segment SGI69787[] = {{&S24771[44], 23}, {NULL}};
const Segment SGI69788[] = {{&S24771[30], 6}, {NULL}};
const Segment SGI69789[] = {{&S24771[115], 3}, {NULL}};
const Segment SGI69790[] = {{&S24771[81], 5}, {NULL}};
const Segment SGI69791[] = {{&S24772[44], 21}, {NULL}};
const Segment SGI69792[] = {{&S24772[44], 37}, {NULL}};
const Segment SGI69793[] = {{&S24772[30], 6}, {NULL}};
const Segment SGI69794[] = {{&S24772[129], 3}, {NULL}};
const Segment SGI69795[] = {{&S24772[95], 5}, {NULL}};
const Segment SGI69796[] = {{&S24773[44], 7}, {NULL}};
const Segment SGI69797[] = {{&S24773[44], 23}, {NULL}};
const Segment SGI69798[] = {{&S24773[30], 6}, {NULL}};
const Segment SGI69799[] = {{&S24773[84], 6}, {NULL}};
const Segment SGI69800[] = {{&S24774[44], 7}, {NULL}};
const Segment SGI69801[] = {{&S24774[44], 23}, {NULL}};
const Segment SGI69802[] = {{&S24774[30], 6}, {NULL}};
const Segment SGI69803[] = {{&S24774[84], 6}, {NULL}};
const Segment SGI69804[] = {{&S24775[44], 19}, {NULL}};
const Segment SGI69805[] = {{&S24775[44], 36}, {NULL}};
const Segment SGI69806[] = {{&S24775[30], 6}, {NULL}};
const Segment SGI69807[] = {{&S24775[128], 3}, {NULL}};
const Segment SGI69808[] = {{&S24775[94], 5}, {NULL}};
const Segment SGI69809[] = {{&S24776[44], 7}, {NULL}};
const Segment SGI69810[] = {{&S24776[44], 23}, {NULL}};
const Segment SGI69811[] = {{&S24776[30], 6}, {NULL}};
const Segment SGI69812[] = {{&S24776[84], 6}, {NULL}};
const Segment SGI69813[] = {{&S24777[44], 20}, {NULL}};
const Segment SGI69814[] = {{&S24777[44], 36}, {NULL}};
const Segment SGI69815[] = {{&S24777[30], 6}, {NULL}};
const Segment SGI69816[] = {{&S24777[128], 3}, {NULL}};
const Segment SGI69817[] = {{&S24777[94], 5}, {NULL}};
const Segment SGI69818[] = {{&S24778[44], 7}, {NULL}};
const Segment SGI69819[] = {{&S24778[44], 23}, {NULL}};
const Segment SGI69820[] = {{&S24778[30], 6}, {NULL}};
const Segment SGI69821[] = {{&S24778[84], 6}, {NULL}};
const Segment SGI69822[] = {{&S24395[42], 11}, {NULL}};
const Segment SGI69823[] = {{&S24395[42], 23}, {&S24395[207], 7}, {NULL}};
const Segment SGI69824[] = {{&S24779[44], 7}, {NULL}};
const Segment SGI69825[] = {{&S24395[30], 4}, {NULL}};
const Segment SGI69826[] = {{&S24779[44], 23}, {NULL}};
const Segment SGI69827[] = {{&S24779[30], 6}, {NULL}};
const Segment SGI69828[] = {{&S24779[84], 4}, {NULL}};
const Segment SGI69829[] = {{&S24395[114], 3}, {NULL}};
const Segment SGI69830[] = {{&S24779[84], 3}, {NULL}};
const Segment SGI69831[] = {{&S24395[79], 6}, {NULL}};
const Segment SGI69832[] = {{&S24780[44], 7}, {NULL}};
const Segment SGI69833[] = {{&S24780[44], 23}, {NULL}};
const Segment SGI69834[] = {{&S24780[30], 6}, {NULL}};
const Segment SGI69835[] = {{&S24396[53], 19}, {NULL}};
const Segment SGI69836[] = {{&S24396[48], 4}, {NULL}};
const Segment SGI69837[] = {{&S24780[84], 4}, {NULL}};
const Segment SGI69838[] = {{&S24780[84], 3}, {NULL}};
const Segment SGI69839[] = {{&S24781[44], 7}, {NULL}};
const Segment SGI69840[] = {{&S24781[44], 23}, {NULL}};
const Segment SGI69841[] = {{&S24781[30], 6}, {NULL}};
const Segment SGI69842[] = {{&S24781[84], 6}, {NULL}};
const Segment SGI69843[] = {{&S24397[42], 10}, {NULL}};
const Segment SGI69844[] = {{&S24397[42], 22}, {&S24397[206], 13}, {NULL}};
const Segment SGI69845[] = {{&S24397[30], 4}, {NULL}};
const Segment SGI69846[] = {{&S24397[113], 3}, {NULL}};
const Segment SGI69847[] = {{&S24397[78], 6}, {NULL}};
const Segment SGI69848[] = {{&S24782[44], 7}, {NULL}};
const Segment SGI69849[] = {{&S24782[44], 24}, {NULL}};
const Segment SGI69850[] = {{&S24782[30], 6}, {NULL}};
const Segment SGI69851[] = {{&S24782[116], 3}, {NULL}};
const Segment SGI69852[] = {{&S24782[82], 5}, {NULL}};
const Segment SGI69853[] = {{&S24399[42], 10}, {NULL}};
const Segment SGI69854[] = {{&S24399[42], 22}, {&S24399[267], 7}, {NULL}};
const Segment SGI69855[] = {{&S24399[30], 4}, {NULL}};
const Segment SGI69856[] = {{&S24399[113], 3}, {NULL}};
const Segment SGI69857[] = {{&S24399[78], 6}, {NULL}};
const Segment SGI69858[] = {{&S24400[50], 11}, {NULL}};
const Segment SGI69859[] = {{&S24400[50], 23}, {NULL}};
const Segment SGI69860[] = {{&S24400[38], 4}, {NULL}};
const Segment SGI69861[] = {{&S24401[42], 10}, {NULL}};
const Segment SGI69862[] = {{&S24401[42], 22}, {NULL}};
const Segment SGI69863[] = {{&S24401[30], 4}, {NULL}};
const Segment SGI69864[] = {{&S24401[113], 3}, {NULL}};
const Segment SGI69865[] = {{&S24401[78], 6}, {NULL}};
const Segment SGI69866[] = {{&S24402[42], 18}, {NULL}};
const Segment SGI69867[] = {{&S24402[42], 30}, {NULL}};
const Segment SGI69868[] = {{&S24403[42], 10}, {NULL}};
const Segment SGI69869[] = {{&S24403[42], 22}, {&S24403[203], 7}, {NULL}};
const Segment SGI69870[] = {{&S24403[30], 4}, {NULL}};
const Segment SGI69871[] = {{&S24403[113], 3}, {NULL}};
const Segment SGI69872[] = {{&S24403[78], 6}, {NULL}};
const Segment SGI69873[] = {{&S24405[38], 9}, {NULL}};
const Segment SGI69874[] = {{&S24405[26], 4}, {NULL}};
const Segment SGI69875[] = {{&S24405[84], 3}, {NULL}};
const Segment SGI69876[] = {{&S24405[61], 6}, {NULL}};
const Segment SGI69877[] = {{&S24406[44], 10}, {NULL}};
const Segment SGI69878[] = {{&S24406[44], 22}, {NULL}};
const Segment SGI69879[] = {{&S24406[30], 6}, {NULL}};
const Segment SGI69880[] = {{&S24406[115], 3}, {NULL}};
const Segment SGI69881[] = {{&S24406[80], 6}, {NULL}};
const Segment SGI69882[] = {{&S24407[42], 10}, {NULL}};
const Segment SGI69883[] = {{&S24407[42], 22}, {NULL}};
const Segment SGI69884[] = {{&S24407[30], 4}, {NULL}};
const Segment SGI69885[] = {{&S24407[112], 3}, {NULL}};
const Segment SGI69886[] = {{&S24407[78], 5}, {NULL}};
const Segment SGI69887[] = {{&S24409[37], 4}, {NULL}};
const Segment SGI69888[] = {{&S24410[42], 30}, {NULL}};
const Segment SGI69889[] = {{&S24410[42], 42}, {NULL}};
const Segment SGI69890[] = {{&S24410[30], 4}, {NULL}};
const Segment SGI69891[] = {{&S24410[133], 3}, {NULL}};
const Segment SGI69892[] = {{&S24410[98], 6}, {NULL}};
const Segment SGI69893[] = {{&S24411[42], 11}, {NULL}};
const Segment SGI69894[] = {{&S24411[42], 23}, {NULL}};
const Segment SGI69895[] = {{&S24411[30], 4}, {NULL}};
const Segment SGI69896[] = {{&S24411[115], 3}, {NULL}};
const Segment SGI69897[] = {{&S24411[79], 7}, {NULL}};
const Segment SGI69898[] = {{&S24411[135], 8}, {NULL}};
const Segment SGI69899[] = {{&S24414[42], 10}, {NULL}};
const Segment SGI69900[] = {{&S24414[30], 4}, {NULL}};
const Segment SGI69901[] = {{&S24415[105], 29}, {NULL}};
const Segment SGI69902[] = {{&S24416[42], 11}, {NULL}};
const Segment SGI69903[] = {{&S24416[42], 23}, {NULL}};
const Segment SGI69904[] = {{&S24416[30], 4}, {NULL}};
const Segment SGI69905[] = {{&S24417[70], 6}, {NULL}};
const Segment SGI69906[] = {{&S24417[30], 4}, {NULL}};
const Segment SGI69907[] = {{&S24417[126], 3}, {NULL}};
const Segment SGI69908[] = {{&S24417[90], 7}, {NULL}};
const Segment SGI69909[] = {{&S24417[146], 8}, {NULL}};
const Segment SGI69910[] = {{&S24419[36], 11}, {NULL}};
const Segment SGI69911[] = {{&S24419[110], 6}, {NULL}};
const Segment SGI69912[] = {{&S24419[31], 4}, {NULL}};
const Segment SGI69913[] = {{&S24420[42], 9}, {NULL}};
const Segment SGI69914[] = {{&S24420[30], 4}, {NULL}};
const Segment SGI69915[] = {{&S24420[99], 3}, {NULL}};
const Segment SGI69916[] = {{&S24420[65], 5}, {NULL}};
const Segment SGI69917[] = {{&S31208[44], 9}, {NULL}};
const Segment SGI69918[] = {{&S31208[44], 21}, {NULL}};
const Segment SGI69919[] = {{&S31208[30], 6}, {NULL}};
const Segment SGI69920[] = {{&S31208[113], 3}, {NULL}};
const Segment SGI69921[] = {{&S31208[79], 5}, {NULL}};
const Segment SGI69922[] = {{&S31209[38], 9}, {NULL}};
const Segment SGI69923[] = {{&S31209[31], 6}, {NULL}};
const Segment SGI69924[] = {{&S31210[43], 9}, {NULL}};
const Segment SGI69925[] = {{&S31210[43], 21}, {NULL}};
const Segment SGI69926[] = {{&S31210[112], 3}, {NULL}};
const Segment SGI69927[] = {{&S31210[78], 5}, {NULL}};
const Segment SGI69928[] = {{&S31211[44], 8}, {NULL}};
const Segment SGI69929[] = {{&S31211[44], 20}, {NULL}};
const Segment SGI69930[] = {{&S31211[30], 6}, {NULL}};
const Segment SGI69931[] = {{&S31211[112], 3}, {NULL}};
const Segment SGI69932[] = {{&S31211[78], 5}, {NULL}};
const Segment SGI69933[] = {{&S31212[43], 9}, {NULL}};
const Segment SGI69934[] = {{&S31212[43], 21}, {NULL}};
const Segment SGI69935[] = {{&S31212[30], 5}, {NULL}};
const Segment SGI69936[] = {{&S31212[112], 3}, {NULL}};
const Segment SGI69937[] = {{&S31212[78], 5}, {NULL}};
const Segment SGI69938[] = {{&S31213[44], 9}, {NULL}};
const Segment SGI69939[] = {{&S31213[44], 21}, {NULL}};
const Segment SGI69940[] = {{&S31213[30], 6}, {NULL}};
const Segment SGI69941[] = {{&S31213[113], 3}, {NULL}};
const Segment SGI69942[] = {{&S31213[79], 5}, {NULL}};
const Segment SGI69943[] = {{&S31214[44], 8}, {NULL}};
const Segment SGI69944[] = {{&S31214[44], 20}, {NULL}};
const Segment SGI69945[] = {{&S31214[30], 6}, {NULL}};
const Segment SGI69946[] = {{&S31214[112], 3}, {NULL}};
const Segment SGI69947[] = {{&S31214[78], 5}, {NULL}};
const Segment SGI69948[] = {{&S31216[44], 10}, {NULL}};
const Segment SGI69949[] = {{&S31216[44], 22}, {NULL}};
const Segment SGI69950[] = {{&S31216[30], 6}, {NULL}};
const Segment SGI69951[] = {{&S31216[114], 3}, {NULL}};
const Segment SGI69952[] = {{&S31216[80], 5}, {NULL}};
const Segment SGI69953[] = {{&S31217[38], 10}, {NULL}};
const Segment SGI69954[] = {{&S31217[31], 6}, {NULL}};
const Segment SGI69955[] = {{&S31234[44], 7}, {NULL}};
const Segment SGI69956[] = {{&S31234[44], 30}, {NULL}};
const Segment SGI69957[] = {{&S31234[30], 6}, {NULL}};
const Segment SGI69958[] = {{&S31234[122], 3}, {NULL}};
const Segment SGI69959[] = {{&S31234[88], 5}, {NULL}};
const Segment SGI69960[] = {{&S31235[44], 11}, {NULL}};
const Segment SGI69961[] = {{&S31235[44], 34}, {NULL}};
const Segment SGI69962[] = {{&S31235[30], 6}, {NULL}};
const Segment SGI69963[] = {{&S31235[126], 3}, {NULL}};
const Segment SGI69964[] = {{&S31235[92], 5}, {NULL}};
const Segment SGI69965[] = {{&S31236[44], 7}, {NULL}};
const Segment SGI69966[] = {{&S31236[44], 40}, {NULL}};
const Segment SGI69967[] = {{&S31236[30], 6}, {NULL}};
const Segment SGI69968[] = {{&S31236[132], 3}, {NULL}};
const Segment SGI69969[] = {{&S31236[98], 5}, {NULL}};
const Segment SGI69970[] = {{&S31237[44], 7}, {NULL}};
const Segment SGI69971[] = {{&S31237[44], 30}, {NULL}};
const Segment SGI69972[] = {{&S31237[30], 6}, {NULL}};
const Segment SGI69973[] = {{&S31237[122], 3}, {NULL}};
const Segment SGI69974[] = {{&S31237[88], 5}, {NULL}};
const Segment SGI69975[] = {{&S31238[44], 7}, {NULL}};
const Segment SGI69976[] = {{&S31238[44], 30}, {NULL}};
const Segment SGI69977[] = {{&S31238[30], 6}, {NULL}};
const Segment SGI69978[] = {{&S31238[122], 3}, {NULL}};
const Segment SGI69979[] = {{&S31238[88], 5}, {NULL}};
const Segment SGI69980[] = {{&S31240[44], 7}, {NULL}};
const Segment SGI69981[] = {{&S31240[44], 31}, {NULL}};
const Segment SGI69982[] = {{&S31240[30], 6}, {NULL}};
const Segment SGI69983[] = {{&S31240[123], 3}, {NULL}};
const Segment SGI69984[] = {{&S31240[89], 5}, {NULL}};
const Segment SGI69985[] = {{&S31241[44], 7}, {NULL}};
const Segment SGI69986[] = {{&S31241[44], 31}, {NULL}};
const Segment SGI69987[] = {{&S31241[30], 6}, {NULL}};
const Segment SGI69988[] = {{&S31241[123], 3}, {NULL}};
const Segment SGI69989[] = {{&S31241[89], 5}, {NULL}};
const Segment SGI69990[] = {{&S31243[44], 7}, {NULL}};
const Segment SGI69991[] = {{&S31243[44], 30}, {NULL}};
const Segment SGI69992[] = {{&S31243[30], 6}, {NULL}};
const Segment SGI69993[] = {{&S31243[122], 3}, {NULL}};
const Segment SGI69994[] = {{&S31243[88], 5}, {NULL}};
const Segment SGI69995[] = {{&S31244[38], 7}, {NULL}};
const Segment SGI69996[] = {{&S31244[31], 6}, {NULL}};
const Segment SGI69997[] = {{&S31251[44], 7}, {NULL}};
const Segment SGI69998[] = {{&S31251[44], 31}, {NULL}};
const Segment SGI69999[] = {{&S31251[30], 6}, {NULL}};
const Segment SGI70000[] = {{&S31251[92], 4}, {NULL}};
const Segment SGI70001[] = {{&S31251[92], 3}, {NULL}};
const Segment SGI70002[] = {{&S31253[44], 11}, {NULL}};
const Segment SGI70003[] = {{&S31253[44], 34}, {NULL}};
const Segment SGI70004[] = {{&S31253[30], 6}, {NULL}};
const Segment SGI70005[] = {{&S31253[126], 3}, {NULL}};
const Segment SGI70006[] = {{&S31253[92], 5}, {NULL}};
const Segment SGI70007[] = {{&S31254[44], 11}, {NULL}};
const Segment SGI70008[] = {{&S31254[44], 35}, {NULL}};
const Segment SGI70009[] = {{&S31254[30], 6}, {NULL}};
const Segment SGI70010[] = {{&S31254[127], 3}, {NULL}};
const Segment SGI70011[] = {{&S31254[93], 5}, {NULL}};
const Segment SGI70012[] = {{&S31256[44], 7}, {NULL}};
const Segment SGI70013[] = {{&S31256[44], 31}, {NULL}};
const Segment SGI70014[] = {{&S31256[30], 6}, {NULL}};
const Segment SGI70015[] = {{&S31256[123], 3}, {NULL}};
const Segment SGI70016[] = {{&S31256[89], 5}, {NULL}};
const Segment SGI70017[] = {{&S31257[44], 7}, {NULL}};
const Segment SGI70018[] = {{&S31257[44], 30}, {NULL}};
const Segment SGI70019[] = {{&S31257[30], 6}, {NULL}};
const Segment SGI70020[] = {{&S31257[122], 3}, {NULL}};
const Segment SGI70021[] = {{&S31257[88], 5}, {NULL}};
const Segment SGI70022[] = {{&S31258[44], 7}, {NULL}};
const Segment SGI70023[] = {{&S31258[44], 30}, {NULL}};
const Segment SGI70024[] = {{&S31258[30], 6}, {NULL}};
const Segment SGI70025[] = {{&S31258[122], 3}, {NULL}};
const Segment SGI70026[] = {{&S31258[88], 5}, {NULL}};
const Segment SGI70027[] = {{&S31261[44], 7}, {NULL}};
const Segment SGI70028[] = {{&S31261[44], 30}, {NULL}};
const Segment SGI70029[] = {{&S31261[30], 6}, {NULL}};
const Segment SGI70030[] = {{&S31261[91], 4}, {NULL}};
const Segment SGI70031[] = {{&S31261[91], 3}, {NULL}};
const Segment SGI70032[] = {{&S31264[44], 7}, {NULL}};
const Segment SGI70033[] = {{&S31264[44], 31}, {NULL}};
const Segment SGI70034[] = {{&S31264[30], 6}, {NULL}};
const Segment SGI70035[] = {{&S31264[92], 6}, {NULL}};
const Segment SGI70036[] = {{&S31265[44], 7}, {NULL}};
const Segment SGI70037[] = {{&S31265[44], 30}, {NULL}};
const Segment SGI70038[] = {{&S31265[30], 6}, {NULL}};
const Segment SGI70039[] = {{&S31265[91], 4}, {NULL}};
const Segment SGI70040[] = {{&S31265[91], 3}, {NULL}};
const Segment SGI70041[] = {{&S31266[44], 7}, {NULL}};
const Segment SGI70042[] = {{&S31266[44], 30}, {NULL}};
const Segment SGI70043[] = {{&S31266[30], 6}, {NULL}};
const Segment SGI70044[] = {{&S31266[91], 6}, {NULL}};
const Segment SGI70045[] = {{&S31267[44], 7}, {NULL}};
const Segment SGI70046[] = {{&S31267[44], 30}, {NULL}};
const Segment SGI70047[] = {{&S31267[30], 6}, {NULL}};
const Segment SGI70048[] = {{&S31267[122], 3}, {NULL}};
const Segment SGI70049[] = {{&S31267[88], 5}, {NULL}};
const Segment SGI70050[] = {{&S31268[44], 7}, {NULL}};
const Segment SGI70051[] = {{&S31268[44], 31}, {NULL}};
const Segment SGI70052[] = {{&S31268[30], 6}, {NULL}};
const Segment SGI70053[] = {{&S31268[123], 3}, {NULL}};
const Segment SGI70054[] = {{&S31268[89], 5}, {NULL}};
const Segment SGI70055[] = {{&S31269[44], 7}, {NULL}};
const Segment SGI70056[] = {{&S31269[44], 29}, {NULL}};
const Segment SGI70057[] = {{&S31269[30], 6}, {NULL}};
const Segment SGI70058[] = {{&S31269[121], 3}, {NULL}};
const Segment SGI70059[] = {{&S31269[87], 5}, {NULL}};
const Segment SGI70060[] = {{&S15055[11], 1}, {NULL}};
const Segment SGI70061[] = {{&S15055[54], 17}, {NULL}};
const Segment SGI70062[] = {{&S15055[30], 16}, {NULL}};
const Segment SGI70063[] = {{&S15055[31], 3}, {NULL}};
const Segment SGI70064[] = {{&S15055[85], 6}, {NULL}};
const Segment SGI70065[] = {{&S15056[11], 1}, {NULL}};
const Segment SGI70066[] = {{&S15056[44], 12}, {NULL}};
const Segment SGI70067[] = {{&S15056[30], 6}, {NULL}};
const Segment SGI70068[] = {{&S15056[31], 3}, {NULL}};
const Segment SGI70069[] = {{&S15056[70], 6}, {NULL}};
const Segment SGI70070[] = {{&S15057[11], 1}, {NULL}};
const Segment SGI70071[] = {{&S15057[44], 12}, {NULL}};
const Segment SGI70072[] = {{&S15057[30], 6}, {NULL}};
const Segment SGI70073[] = {{&S15057[31], 3}, {NULL}};
const Segment SGI70074[] = {{&S15057[70], 6}, {NULL}};
const Segment SGI70075[] = {{&S15058[11], 1}, {NULL}};
const Segment SGI70076[] = {{&S15058[44], 12}, {NULL}};
const Segment SGI70077[] = {{&S15058[30], 6}, {NULL}};
const Segment SGI70078[] = {{&S15058[31], 3}, {NULL}};
const Segment SGI70079[] = {{&S15058[70], 6}, {NULL}};
const Segment SGI70080[] = {{&S15059[11], 1}, {NULL}};
const Segment SGI70081[] = {{&S15059[44], 13}, {NULL}};
const Segment SGI70082[] = {{&S15059[30], 6}, {NULL}};
const Segment SGI70083[] = {{&S15059[31], 3}, {NULL}};
const Segment SGI70084[] = {{&S15059[71], 6}, {NULL}};
const Segment SGI70085[] = {{&S15064[11], 1}, {NULL}};
const Segment SGI70086[] = {{&S15064[44], 13}, {NULL}};
const Segment SGI70087[] = {{&S15064[30], 6}, {NULL}};
const Segment SGI70088[] = {{&S15064[31], 3}, {NULL}};
const Segment SGI70089[] = {{&S15064[71], 6}, {NULL}};
const Segment SGI70090[] = {{&S15065[44], 10}, {NULL}};
const Segment SGI70091[] = {{&S15065[44], 22}, {NULL}};
const Segment SGI70092[] = {{&S15065[30], 6}, {NULL}};
const Segment SGI70093[] = {{&S15065[31], 3}, {NULL}};
const Segment SGI70094[] = {{&S15065[80], 6}, {NULL}};
const Segment SGI70095[] = {{&S15066[44], 11}, {NULL}};
const Segment SGI70096[] = {{&S15066[44], 23}, {NULL}};
const Segment SGI70097[] = {{&S15066[30], 6}, {NULL}};
const Segment SGI70098[] = {{&S15066[31], 3}, {NULL}};
const Segment SGI70099[] = {{&S15066[81], 6}, {NULL}};
const Segment SGI70100[] = {{&S15071[11], 1}, {NULL}};
const Segment SGI70101[] = {{&S15071[44], 19}, {NULL}};
const Segment SGI70102[] = {{&S15071[30], 6}, {NULL}};
const Segment SGI70103[] = {{&S15071[31], 3}, {NULL}};
const Segment SGI70104[] = {{&S15071[77], 6}, {NULL}};
const Segment SGI70105[] = {{&S15073[11], 1}, {NULL}};
const Segment SGI70106[] = {{&S15073[54], 13}, {NULL}};
const Segment SGI70107[] = {{&S15073[30], 16}, {NULL}};
const Segment SGI70108[] = {{&S15073[31], 3}, {NULL}};
const Segment SGI70109[] = {{&S15073[81], 6}, {NULL}};
const Segment SGI70110[] = {{&S15074[38], 5}, {NULL}};
const Segment SGI70111[] = {{&S15074[31], 6}, {NULL}};
const Segment SGI70112[] = {{&S15076[44], 5}, {NULL}};
const Segment SGI70113[] = {{&S15076[44], 17}, {NULL}};
const Segment SGI70114[] = {{&S15076[30], 6}, {NULL}};
const Segment SGI70115[] = {{&S15076[31], 3}, {NULL}};
const Segment SGI70116[] = {{&S15076[75], 6}, {NULL}};
const Segment SGI70117[] = {{&S15077[11], 1}, {NULL}};
const Segment SGI70118[] = {{&S15077[44], 13}, {NULL}};
const Segment SGI70119[] = {{&S15077[30], 6}, {NULL}};
const Segment SGI70120[] = {{&S15077[31], 3}, {NULL}};
const Segment SGI70121[] = {{&S15077[71], 6}, {NULL}};
const Segment SGI70122[] = {{&S15078[44], 11}, {NULL}};
const Segment SGI70123[] = {{&S15078[44], 23}, {NULL}};
const Segment SGI70124[] = {{&S15078[30], 6}, {NULL}};
const Segment SGI70125[] = {{&S15078[31], 3}, {NULL}};
const Segment SGI70126[] = {{&S15078[81], 6}, {NULL}};
const Segment SGI70127[] = {{&S15079[11], 1}, {NULL}};
const Segment SGI70128[] = {{&S15079[54], 17}, {NULL}};
const Segment SGI70129[] = {{&S15079[30], 16}, {NULL}};
const Segment SGI70130[] = {{&S15079[31], 3}, {NULL}};
const Segment SGI70131[] = {{&S15079[85], 6}, {NULL}};
const Segment SGI70132[] = {{&S15081[44], 5}, {NULL}};
const Segment SGI70133[] = {{&S15081[44], 17}, {NULL}};
const Segment SGI70134[] = {{&S15081[30], 6}, {NULL}};
const Segment SGI70135[] = {{&S15081[31], 3}, {NULL}};
const Segment SGI70136[] = {{&S15081[75], 6}, {NULL}};
const Segment SGI70137[] = {{&S15082[44], 11}, {NULL}};
const Segment SGI70138[] = {{&S15082[44], 23}, {NULL}};
const Segment SGI70139[] = {{&S15082[30], 6}, {NULL}};
const Segment SGI70140[] = {{&S15082[31], 3}, {NULL}};
const Segment SGI70141[] = {{&S15082[81], 6}, {NULL}};
const Segment SGI70142[] = {{&S15084[44], 12}, {NULL}};
const Segment SGI70143[] = {{&S15084[44], 24}, {NULL}};
const Segment SGI70144[] = {{&S15084[30], 6}, {NULL}};
const Segment SGI70145[] = {{&S15084[31], 3}, {NULL}};
const Segment SGI70146[] = {{&S15084[82], 6}, {NULL}};
const Segment SGI70147[] = {{&S15085[11], 1}, {NULL}};
const Segment SGI70148[] = {{&S15085[44], 17}, {NULL}};
const Segment SGI70149[] = {{&S15085[30], 6}, {NULL}};
const Segment SGI70150[] = {{&S15085[31], 3}, {NULL}};
const Segment SGI70151[] = {{&S15085[75], 6}, {NULL}};
const Segment SGI70152[] = {{&S15087[44], 13}, {NULL}};
const Segment SGI70153[] = {{&S15087[44], 25}, {NULL}};
const Segment SGI70154[] = {{&S15087[30], 6}, {NULL}};
const Segment SGI70155[] = {{&S15087[31], 3}, {NULL}};
const Segment SGI70156[] = {{&S15087[83], 6}, {NULL}};
const Segment SGI70157[] = {{&S15088[44], 6}, {NULL}};
const Segment SGI70158[] = {{&S15088[44], 18}, {NULL}};
const Segment SGI70159[] = {{&S15088[30], 6}, {NULL}};
const Segment SGI70160[] = {{&S15088[31], 3}, {NULL}};
const Segment SGI70161[] = {{&S15088[76], 6}, {NULL}};
const Segment SGI70162[] = {{&S15089[11], 1}, {NULL}};
const Segment SGI70163[] = {{&S15089[54], 13}, {NULL}};
const Segment SGI70164[] = {{&S15089[30], 16}, {NULL}};
const Segment SGI70165[] = {{&S15089[31], 3}, {NULL}};
const Segment SGI70166[] = {{&S15089[81], 6}, {NULL}};
const Segment SGI70167[] = {{&S15090[44], 7}, {NULL}};
const Segment SGI70168[] = {{&S15090[44], 19}, {NULL}};
const Segment SGI70169[] = {{&S15090[30], 6}, {NULL}};
const Segment SGI70170[] = {{&S15090[31], 3}, {NULL}};
const Segment SGI70171[] = {{&S15090[77], 6}, {NULL}};
const Segment SGI70172[] = {{&S15092[11], 1}, {NULL}};
const Segment SGI70173[] = {{&S15092[44], 13}, {NULL}};
const Segment SGI70174[] = {{&S15092[30], 6}, {NULL}};
const Segment SGI70175[] = {{&S15092[31], 3}, {NULL}};
const Segment SGI70176[] = {{&S15092[71], 6}, {NULL}};
const Segment SGI70177[] = {{&S15093[44], 5}, {NULL}};
const Segment SGI70178[] = {{&S15093[44], 17}, {NULL}};
const Segment SGI70179[] = {{&S15093[30], 6}, {NULL}};
const Segment SGI70180[] = {{&S15093[31], 3}, {NULL}};
const Segment SGI70181[] = {{&S15093[75], 6}, {NULL}};
const Segment SGI70182[] = {{&S15095[44], 8}, {NULL}};
const Segment SGI70183[] = {{&S15095[44], 20}, {NULL}};
const Segment SGI70184[] = {{&S15095[30], 6}, {NULL}};
const Segment SGI70185[] = {{&S15095[31], 3}, {NULL}};
const Segment SGI70186[] = {{&S15095[78], 6}, {NULL}};
const Segment SGI70187[] = {{&S15096[44], 16}, {NULL}};
const Segment SGI70188[] = {{&S15096[44], 28}, {NULL}};
const Segment SGI70189[] = {{&S15096[30], 6}, {NULL}};
const Segment SGI70190[] = {{&S15096[31], 3}, {NULL}};
const Segment SGI70191[] = {{&S15096[86], 6}, {NULL}};
const Segment SGI70192[] = {{&S15099[44], 16}, {NULL}};
const Segment SGI70193[] = {{&S15099[44], 28}, {NULL}};
const Segment SGI70194[] = {{&S15099[30], 6}, {NULL}};
const Segment SGI70195[] = {{&S15099[31], 3}, {NULL}};
const Segment SGI70196[] = {{&S15099[86], 6}, {NULL}};
const Segment SGI70197[] = {{&S15100[11], 1}, {NULL}};
const Segment SGI70198[] = {{&S15100[44], 23}, {NULL}};
const Segment SGI70199[] = {{&S15100[30], 6}, {NULL}};
const Segment SGI70200[] = {{&S15100[31], 3}, {NULL}};
const Segment SGI70201[] = {{&S15100[81], 6}, {NULL}};
const Segment SGI70202[] = {{&S15101[11], 1}, {NULL}};
const Segment SGI70203[] = {{&S15101[44], 10}, {NULL}};
const Segment SGI70204[] = {{&S15101[30], 6}, {NULL}};
const Segment SGI70205[] = {{&S15101[31], 3}, {NULL}};
const Segment SGI70206[] = {{&S15101[68], 6}, {NULL}};
const Segment SGI70207[] = {{&S15104[44], 6}, {NULL}};
const Segment SGI70208[] = {{&S15104[44], 18}, {NULL}};
const Segment SGI70209[] = {{&S15104[30], 6}, {NULL}};
const Segment SGI70210[] = {{&S15104[31], 3}, {NULL}};
const Segment SGI70211[] = {{&S15104[76], 6}, {NULL}};
const Segment SGI70212[] = {{&S15106[23], 15}, {NULL}};
const Segment SGI70213[] = {{&S15106[31], 19}, {NULL}};
const Segment SGI70214[] = {{&S15107[44], 10}, {NULL}};
const Segment SGI70215[] = {{&S15107[44], 22}, {NULL}};
const Segment SGI70216[] = {{&S15107[30], 6}, {NULL}};
const Segment SGI70217[] = {{&S15107[31], 3}, {NULL}};
const Segment SGI70218[] = {{&S15107[80], 6}, {NULL}};
const Segment SGI70219[] = {{&S15108[44], 12}, {NULL}};
const Segment SGI70220[] = {{&S15108[44], 24}, {NULL}};
const Segment SGI70221[] = {{&S15108[30], 6}, {NULL}};
const Segment SGI70222[] = {{&S15108[31], 3}, {NULL}};
const Segment SGI70223[] = {{&S15108[82], 6}, {NULL}};
const Segment SGI70224[] = {{&S15109[44], 10}, {NULL}};
const Segment SGI70225[] = {{&S15109[44], 22}, {NULL}};
const Segment SGI70226[] = {{&S15109[30], 6}, {NULL}};
const Segment SGI70227[] = {{&S15109[31], 3}, {NULL}};
const Segment SGI70228[] = {{&S15109[80], 6}, {NULL}};
const Segment SGI70229[] = {{&S15110[44], 12}, {NULL}};
const Segment SGI70230[] = {{&S15110[44], 24}, {NULL}};
const Segment SGI70231[] = {{&S15110[30], 6}, {NULL}};
const Segment SGI70232[] = {{&S15110[31], 3}, {NULL}};
const Segment SGI70233[] = {{&S15110[82], 6}, {NULL}};
const Segment SGI70234[] = {{&S15111[44], 12}, {NULL}};
const Segment SGI70235[] = {{&S15111[44], 24}, {NULL}};
const Segment SGI70236[] = {{&S15111[30], 6}, {NULL}};
const Segment SGI70237[] = {{&S15111[31], 3}, {NULL}};
const Segment SGI70238[] = {{&S15111[82], 6}, {NULL}};
const Segment SGI70239[] = {{&S15113[11], 1}, {NULL}};
const Segment SGI70240[] = {{&S15113[44], 13}, {NULL}};
const Segment SGI70241[] = {{&S15113[30], 6}, {NULL}};
const Segment SGI70242[] = {{&S15113[31], 3}, {NULL}};
const Segment SGI70243[] = {{&S15113[71], 6}, {NULL}};
const Segment SGI70244[] = {{&S15114[44], 10}, {NULL}};
const Segment SGI70245[] = {{&S15114[44], 22}, {NULL}};
const Segment SGI70246[] = {{&S15114[30], 6}, {NULL}};
const Segment SGI70247[] = {{&S15114[31], 3}, {NULL}};
const Segment SGI70248[] = {{&S15114[80], 6}, {NULL}};
const Segment SGI70249[] = {{&S15115[44], 12}, {NULL}};
const Segment SGI70250[] = {{&S15115[44], 24}, {NULL}};
const Segment SGI70251[] = {{&S15115[30], 6}, {NULL}};
const Segment SGI70252[] = {{&S15115[31], 3}, {NULL}};
const Segment SGI70253[] = {{&S15115[82], 6}, {NULL}};
const Segment SGI70254[] = {{&S15116[44], 14}, {NULL}};
const Segment SGI70255[] = {{&S15116[44], 26}, {NULL}};
const Segment SGI70256[] = {{&S15116[30], 6}, {NULL}};
const Segment SGI70257[] = {{&S15116[31], 3}, {NULL}};
const Segment SGI70258[] = {{&S15116[84], 6}, {NULL}};
const Segment SGI70259[] = {{&S15117[44], 11}, {NULL}};
const Segment SGI70260[] = {{&S15117[44], 23}, {NULL}};
const Segment SGI70261[] = {{&S15117[30], 6}, {NULL}};
const Segment SGI70262[] = {{&S15117[31], 3}, {NULL}};
const Segment SGI70263[] = {{&S15117[81], 6}, {NULL}};
const Segment SGI70264[] = {{&S15118[11], 1}, {NULL}};
const Segment SGI70265[] = {{&S15118[44], 24}, {NULL}};
const Segment SGI70266[] = {{&S15118[30], 6}, {NULL}};
const Segment SGI70267[] = {{&S15118[31], 3}, {NULL}};
const Segment SGI70268[] = {{&S15118[82], 6}, {NULL}};
const Segment SGI70269[] = {{&S15120[44], 8}, {NULL}};
const Segment SGI70270[] = {{&S15120[44], 20}, {NULL}};
const Segment SGI70271[] = {{&S15120[30], 6}, {NULL}};
const Segment SGI70272[] = {{&S15120[31], 3}, {NULL}};
const Segment SGI70273[] = {{&S15120[78], 6}, {NULL}};
const Segment SGI70274[] = {{&S31693[44], 30}, {NULL}};
const Segment SGI70275[] = {{&S31693[44], 47}, {NULL}};
const Segment SGI70276[] = {{&S31693[30], 6}, {NULL}};
const Segment SGI70277[] = {{&S15121[44], 12}, {NULL}};
const Segment SGI70278[] = {{&S31693[139], 3}, {NULL}};
const Segment SGI70279[] = {{&S15121[44], 24}, {NULL}};
const Segment SGI70280[] = {{&S15121[30], 6}, {NULL}};
const Segment SGI70281[] = {{&S31693[105], 5}, {NULL}};
const Segment SGI70282[] = {{&S15121[31], 3}, {NULL}};
const Segment SGI70283[] = {{&S15121[82], 6}, {NULL}};
const Segment SGI70284[] = {{&S15122[44], 12}, {NULL}};
const Segment SGI70285[] = {{&S15122[44], 24}, {NULL}};
const Segment SGI70286[] = {{&S15122[30], 6}, {NULL}};
const Segment SGI70287[] = {{&S31695[44], 11}, {NULL}};
const Segment SGI70288[] = {{&S31695[44], 28}, {NULL}};
const Segment SGI70289[] = {{&S31695[30], 6}, {NULL}};
const Segment SGI70290[] = {{&S15122[31], 3}, {NULL}};
const Segment SGI70291[] = {{&S15122[82], 6}, {NULL}};
const Segment SGI70292[] = {{&S31695[120], 3}, {NULL}};
const Segment SGI70293[] = {{&S31695[86], 5}, {NULL}};
const Segment SGI70294[] = {{&S31696[38], 11}, {NULL}};
const Segment SGI70295[] = {{&S31696[31], 6}, {NULL}};
const Segment SGI70296[] = {{&S15125[44], 19}, {NULL}};
const Segment SGI70297[] = {{&S15125[44], 31}, {NULL}};
const Segment SGI70298[] = {{&S15125[30], 6}, {NULL}};
const Segment SGI70299[] = {{&S15125[31], 3}, {NULL}};
const Segment SGI70300[] = {{&S15125[89], 6}, {NULL}};
const Segment SGI70301[] = {{&S15127[30], 6}, {NULL}};
const Segment SGI70302[] = {{&S15127[31], 3}, {NULL}};
const Segment SGI70303[] = {{&S15127[57], 6}, {NULL}};
const Segment SGI70304[] = {{&S15128[23], 15}, {NULL}};
const Segment SGI70305[] = {{&S15128[31], 25}, {NULL}};
const Segment SGI70306[] = {{&S15130[11], 1}, {NULL}};
const Segment SGI70307[] = {{&S15130[44], 13}, {NULL}};
const Segment SGI70308[] = {{&S15130[30], 6}, {NULL}};
const Segment SGI70309[] = {{&S15130[31], 3}, {NULL}};
const Segment SGI70310[] = {{&S15130[71], 6}, {NULL}};
const Segment SGI70311[] = {{&S15131[38], 16}, {NULL}};
const Segment SGI70312[] = {{&S15131[31], 6}, {NULL}};
const Segment SGI70313[] = {{&S15133[44], 16}, {NULL}};
const Segment SGI70314[] = {{&S15133[44], 28}, {NULL}};
const Segment SGI70315[] = {{&S15133[30], 6}, {NULL}};
const Segment SGI70316[] = {{&S15133[31], 3}, {NULL}};
const Segment SGI70317[] = {{&S15133[86], 6}, {NULL}};
const Segment SGI70318[] = {{&S15134[44], 6}, {NULL}};
const Segment SGI70319[] = {{&S15134[44], 18}, {NULL}};
const Segment SGI70320[] = {{&S15134[30], 6}, {NULL}};
const Segment SGI70321[] = {{&S31712[44], 7}, {NULL}};
const Segment SGI70322[] = {{&S15134[31], 3}, {NULL}};
const Segment SGI70323[] = {{&S31712[44], 24}, {NULL}};
const Segment SGI70324[] = {{&S31712[30], 6}, {NULL}};
const Segment SGI70325[] = {{&S15134[76], 6}, {NULL}};
const Segment SGI70326[] = {{&S31712[116], 3}, {NULL}};
const Segment SGI70327[] = {{&S31712[82], 5}, {NULL}};
const Segment SGI70328[] = {{&S15135[44], 21}, {NULL}};
const Segment SGI70329[] = {{&S15135[44], 33}, {NULL}};
const Segment SGI70330[] = {{&S15135[30], 6}, {NULL}};
const Segment SGI70331[] = {{&S31713[44], 7}, {NULL}};
const Segment SGI70332[] = {{&S15135[31], 3}, {NULL}};
const Segment SGI70333[] = {{&S31713[44], 31}, {NULL}};
const Segment SGI70334[] = {{&S31713[30], 6}, {NULL}};
const Segment SGI70335[] = {{&S15135[91], 6}, {NULL}};
const Segment SGI70336[] = {{&S31713[123], 3}, {NULL}};
const Segment SGI70337[] = {{&S31713[89], 5}, {NULL}};
const Segment SGI70338[] = {{&S15136[44], 11}, {NULL}};
const Segment SGI70339[] = {{&S15136[44], 23}, {NULL}};
const Segment SGI70340[] = {{&S15136[30], 6}, {NULL}};
const Segment SGI70341[] = {{&S15136[31], 3}, {NULL}};
const Segment SGI70342[] = {{&S31714[44], 7}, {NULL}};
const Segment SGI70343[] = {{&S15136[81], 6}, {NULL}};
const Segment SGI70344[] = {{&S31714[44], 31}, {NULL}};
const Segment SGI70345[] = {{&S31714[30], 6}, {NULL}};
const Segment SGI70346[] = {{&S31714[123], 3}, {NULL}};
const Segment SGI70347[] = {{&S31714[89], 5}, {NULL}};
const Segment SGI70348[] = {{&S31715[44], 7}, {NULL}};
const Segment SGI70349[] = {{&S31715[44], 24}, {NULL}};
const Segment SGI70350[] = {{&S31715[30], 6}, {NULL}};
const Segment SGI70351[] = {{&S15138[44], 9}, {NULL}};
const Segment SGI70352[] = {{&S31715[116], 3}, {NULL}};
const Segment SGI70353[] = {{&S15138[44], 21}, {NULL}};
const Segment SGI70354[] = {{&S15138[30], 6}, {NULL}};
const Segment SGI70355[] = {{&S31715[82], 5}, {NULL}};
const Segment SGI70356[] = {{&S15138[31], 3}, {NULL}};
const Segment SGI70357[] = {{&S15138[79], 6}, {NULL}};
const Segment SGI70358[] = {{&S31716[44], 7}, {NULL}};
const Segment SGI70359[] = {{&S31716[44], 31}, {NULL}};
const Segment SGI70360[] = {{&S31716[30], 6}, {NULL}};
const Segment SGI70361[] = {{&S31716[123], 3}, {NULL}};
const Segment SGI70362[] = {{&S31716[89], 5}, {NULL}};
const Segment SGI70363[] = {{&S15140[44], 11}, {NULL}};
const Segment SGI70364[] = {{&S15140[44], 23}, {NULL}};
const Segment SGI70365[] = {{&S15140[30], 6}, {NULL}};
const Segment SGI70366[] = {{&S31717[44], 7}, {NULL}};
const Segment SGI70367[] = {{&S15140[31], 3}, {NULL}};
const Segment SGI70368[] = {{&S31717[44], 31}, {NULL}};
const Segment SGI70369[] = {{&S15140[81], 6}, {NULL}};
const Segment SGI70370[] = {{&S31717[30], 6}, {NULL}};
const Segment SGI70371[] = {{&S31717[123], 3}, {NULL}};
const Segment SGI70372[] = {{&S31717[89], 5}, {NULL}};
const Segment SGI70373[] = {{&S15142[23], 15}, {NULL}};
const Segment SGI70374[] = {{&S31718[44], 8}, {NULL}};
const Segment SGI70375[] = {{&S15142[31], 19}, {NULL}};
const Segment SGI70376[] = {{&S31718[44], 33}, {NULL}};
const Segment SGI70377[] = {{&S31718[30], 6}, {NULL}};
const Segment SGI70378[] = {{&S31718[125], 3}, {NULL}};
const Segment SGI70379[] = {{&S31718[91], 5}, {NULL}};
const Segment SGI70380[] = {{&S15143[44], 18}, {NULL}};
const Segment SGI70381[] = {{&S15143[44], 30}, {NULL}};
const Segment SGI70382[] = {{&S15143[30], 6}, {NULL}};
const Segment SGI70383[] = {{&S15143[31], 3}, {NULL}};
const Segment SGI70384[] = {{&S31719[44], 7}, {NULL}};
const Segment SGI70385[] = {{&S15143[88], 6}, {NULL}};
const Segment SGI70386[] = {{&S31719[44], 31}, {NULL}};
const Segment SGI70387[] = {{&S31719[30], 6}, {NULL}};
const Segment SGI70388[] = {{&S31719[123], 3}, {NULL}};
const Segment SGI70389[] = {{&S31719[89], 5}, {NULL}};
const Segment SGI70390[] = {{&S15144[11], 1}, {NULL}};
const Segment SGI70391[] = {{&S15144[44], 19}, {NULL}};
const Segment SGI70392[] = {{&S15144[30], 6}, {NULL}};
const Segment SGI70393[] = {{&S15144[31], 3}, {NULL}};
const Segment SGI70394[] = {{&S15144[77], 6}, {NULL}};
const Segment SGI70395[] = {{&S31720[44], 7}, {NULL}};
const Segment SGI70396[] = {{&S31720[44], 32}, {NULL}};
const Segment SGI70397[] = {{&S31720[30], 6}, {NULL}};
const Segment SGI70398[] = {{&S31720[124], 3}, {NULL}};
const Segment SGI70399[] = {{&S31720[90], 5}, {NULL}};
const Segment SGI70400[] = {{&S15145[44], 11}, {NULL}};
const Segment SGI70401[] = {{&S15145[44], 23}, {NULL}};
const Segment SGI70402[] = {{&S15145[30], 6}, {NULL}};
const Segment SGI70403[] = {{&S15145[31], 3}, {NULL}};
const Segment SGI70404[] = {{&S15145[81], 6}, {NULL}};
const Segment SGI70405[] = {{&S31721[44], 21}, {NULL}};
const Segment SGI70406[] = {{&S31721[44], 38}, {NULL}};
const Segment SGI70407[] = {{&S31721[30], 6}, {NULL}};
const Segment SGI70408[] = {{&S31721[130], 3}, {NULL}};
const Segment SGI70409[] = {{&S31721[96], 5}, {NULL}};
const Segment SGI70410[] = {{&S15146[11], 1}, {NULL}};
const Segment SGI70411[] = {{&S15146[44], 10}, {NULL}};
const Segment SGI70412[] = {{&S15146[30], 6}, {NULL}};
const Segment SGI70413[] = {{&S15146[31], 3}, {NULL}};
const Segment SGI70414[] = {{&S15146[68], 6}, {NULL}};
const Segment SGI70415[] = {{&S31722[44], 7}, {NULL}};
const Segment SGI70416[] = {{&S31722[44], 32}, {NULL}};
const Segment SGI70417[] = {{&S31722[30], 6}, {NULL}};
const Segment SGI70418[] = {{&S31722[124], 3}, {NULL}};
const Segment SGI70419[] = {{&S31722[90], 5}, {NULL}};
const Segment SGI70420[] = {{&S15147[44], 6}, {NULL}};
const Segment SGI70421[] = {{&S15147[44], 18}, {NULL}};
const Segment SGI70422[] = {{&S15147[30], 6}, {NULL}};
const Segment SGI70423[] = {{&S15147[31], 3}, {NULL}};
const Segment SGI70424[] = {{&S15147[76], 6}, {NULL}};
const Segment SGI70425[] = {{&S31723[44], 7}, {NULL}};
const Segment SGI70426[] = {{&S31723[44], 31}, {NULL}};
const Segment SGI70427[] = {{&S31723[30], 6}, {NULL}};
const Segment SGI70428[] = {{&S31723[123], 3}, {NULL}};
const Segment SGI70429[] = {{&S31723[89], 5}, {NULL}};
const Segment SGI70430[] = {{&S15148[44], 21}, {NULL}};
const Segment SGI70431[] = {{&S15148[44], 33}, {NULL}};
const Segment SGI70432[] = {{&S15148[30], 6}, {NULL}};
const Segment SGI70433[] = {{&S15148[31], 3}, {NULL}};
const Segment SGI70434[] = {{&S15148[91], 6}, {NULL}};
const Segment SGI70435[] = {{&S31724[44], 7}, {NULL}};
const Segment SGI70436[] = {{&S31724[44], 24}, {NULL}};
const Segment SGI70437[] = {{&S31724[30], 6}, {NULL}};
const Segment SGI70438[] = {{&S31724[85], 6}, {NULL}};
const Segment SGI70439[] = {{&S31725[38], 7}, {NULL}};
const Segment SGI70440[] = {{&S31725[31], 6}, {NULL}};
const Segment SGI70441[] = {{&S31726[44], 7}, {NULL}};
const Segment SGI70442[] = {{&S31726[44], 36}, {NULL}};
const Segment SGI70443[] = {{&S31726[30], 6}, {NULL}};
const Segment SGI70444[] = {{&S31726[129], 3}, {NULL}};
const Segment SGI70445[] = {{&S31726[95], 5}, {NULL}};
const Segment SGI70446[] = {{&S15153[44], 7}, {NULL}};
const Segment SGI70447[] = {{&S15153[44], 19}, {NULL}};
const Segment SGI70448[] = {{&S15153[30], 6}, {NULL}};
const Segment SGI70449[] = {{&S15153[31], 3}, {NULL}};
const Segment SGI70450[] = {{&S15153[77], 6}, {NULL}};
const Segment SGI70451[] = {{&S31727[44], 8}, {NULL}};
const Segment SGI70452[] = {{&S31727[44], 33}, {NULL}};
const Segment SGI70453[] = {{&S31727[30], 6}, {NULL}};
const Segment SGI70454[] = {{&S31727[125], 3}, {NULL}};
const Segment SGI70455[] = {{&S31727[91], 5}, {NULL}};
const Segment SGI70456[] = {{&S15154[44], 8}, {NULL}};
const Segment SGI70457[] = {{&S15154[44], 20}, {NULL}};
const Segment SGI70458[] = {{&S15154[30], 6}, {NULL}};
const Segment SGI70459[] = {{&S15154[31], 3}, {NULL}};
const Segment SGI70460[] = {{&S15154[78], 6}, {NULL}};
const Segment SGI70461[] = {{&S31728[44], 7}, {NULL}};
const Segment SGI70462[] = {{&S31728[44], 31}, {NULL}};
const Segment SGI70463[] = {{&S31728[30], 6}, {NULL}};
const Segment SGI70464[] = {{&S31728[123], 3}, {NULL}};
const Segment SGI70465[] = {{&S31728[89], 5}, {NULL}};
const Segment SGI70466[] = {{&S15155[44], 24}, {NULL}};
const Segment SGI70467[] = {{&S15155[44], 36}, {NULL}};
const Segment SGI70468[] = {{&S15155[30], 6}, {NULL}};
const Segment SGI70469[] = {{&S15155[31], 3}, {NULL}};
const Segment SGI70470[] = {{&S15155[94], 6}, {NULL}};
const Segment SGI70471[] = {{&S31729[44], 7}, {NULL}};
const Segment SGI70472[] = {{&S31729[44], 32}, {NULL}};
const Segment SGI70473[] = {{&S31729[30], 6}, {NULL}};
const Segment SGI70474[] = {{&S31729[124], 3}, {NULL}};
const Segment SGI70475[] = {{&S31729[90], 5}, {NULL}};
const Segment SGI70476[] = {{&S15157[30], 6}, {NULL}};
const Segment SGI70477[] = {{&S31730[44], 7}, {NULL}};
const Segment SGI70478[] = {{&S15157[31], 3}, {NULL}};
const Segment SGI70479[] = {{&S31730[44], 32}, {NULL}};
const Segment SGI70480[] = {{&S31730[30], 6}, {NULL}};
const Segment SGI70481[] = {{&S15157[57], 6}, {NULL}};
const Segment SGI70482[] = {{&S31730[124], 3}, {NULL}};
const Segment SGI70483[] = {{&S31730[90], 5}, {NULL}};
const Segment SGI70484[] = {{&S31731[38], 7}, {NULL}};
const Segment SGI70485[] = {{&S31731[31], 6}, {NULL}};
const Segment SGI70486[] = {{&S15162[44], 20}, {NULL}};
const Segment SGI70487[] = {{&S15162[44], 32}, {NULL}};
const Segment SGI70488[] = {{&S15162[30], 6}, {NULL}};
const Segment SGI70489[] = {{&S15162[31], 3}, {NULL}};
const Segment SGI70490[] = {{&S15162[90], 6}, {NULL}};
const Segment SGI70491[] = {{&S15163[44], 22}, {NULL}};
const Segment SGI70492[] = {{&S15163[44], 34}, {NULL}};
const Segment SGI70493[] = {{&S15163[30], 6}, {NULL}};
const Segment SGI70494[] = {{&S15163[31], 3}, {NULL}};
const Segment SGI70495[] = {{&S15163[92], 6}, {NULL}};
const Segment SGI70496[] = {{&S31737[44], 10}, {NULL}};
const Segment SGI70497[] = {{&S31737[44], 27}, {NULL}};
const Segment SGI70498[] = {{&S31737[30], 6}, {NULL}};
const Segment SGI70499[] = {{&S31737[119], 3}, {NULL}};
const Segment SGI70500[] = {{&S31737[85], 5}, {NULL}};
const Segment SGI70501[] = {{&S31738[38], 10}, {NULL}};
const Segment SGI70502[] = {{&S31738[31], 6}, {NULL}};
const Segment SGI70503[] = {{&S15166[37], 7}, {NULL}};
const Segment SGI70504[] = {{&S15166[30], 6}, {NULL}};
const Segment SGI70505[] = {{&S15166[73], 6}, {NULL}};
const Segment SGI70506[] = {{&S31741[44], 11}, {NULL}};
const Segment SGI70507[] = {{&S31741[44], 28}, {NULL}};
const Segment SGI70508[] = {{&S31741[30], 6}, {NULL}};
const Segment SGI70509[] = {{&S31741[120], 3}, {NULL}};
const Segment SGI70510[] = {{&S31741[86], 5}, {NULL}};
const Segment SGI70511[] = {{&S15169[11], 1}, {NULL}};
const Segment SGI70512[] = {{&S15169[44], 19}, {NULL}};
const Segment SGI70513[] = {{&S15169[30], 6}, {NULL}};
const Segment SGI70514[] = {{&S15169[31], 3}, {NULL}};
const Segment SGI70515[] = {{&S15169[77], 6}, {NULL}};
const Segment SGI70516[] = {{&S31743[44], 10}, {NULL}};
const Segment SGI70517[] = {{&S31743[44], 27}, {NULL}};
const Segment SGI70518[] = {{&S31743[30], 6}, {NULL}};
const Segment SGI70519[] = {{&S31743[88], 4}, {NULL}};
const Segment SGI70520[] = {{&S31743[88], 3}, {NULL}};
const Segment SGI70521[] = {{&S31744[44], 11}, {NULL}};
const Segment SGI70522[] = {{&S31744[44], 28}, {NULL}};
const Segment SGI70523[] = {{&S31744[30], 6}, {NULL}};
const Segment SGI70524[] = {{&S31744[120], 3}, {NULL}};
const Segment SGI70525[] = {{&S31744[86], 5}, {NULL}};
const Segment SGI70526[] = {{&S31745[55], 10}, {NULL}};
const Segment SGI70527[] = {{&S31745[55], 27}, {NULL}};
const Segment SGI70528[] = {{&S31745[30], 16}, {NULL}};
const Segment SGI70529[] = {{&S15174[44], 18}, {NULL}};
const Segment SGI70530[] = {{&S15174[44], 30}, {NULL}};
const Segment SGI70531[] = {{&S15174[30], 6}, {NULL}};
const Segment SGI70532[] = {{&S31745[130], 3}, {NULL}};
const Segment SGI70533[] = {{&S31745[96], 5}, {NULL}};
const Segment SGI70534[] = {{&S15174[31], 3}, {NULL}};
const Segment SGI70535[] = {{&S15174[88], 6}, {NULL}};
const Segment SGI70536[] = {{&S15175[44], 15}, {NULL}};
const Segment SGI70537[] = {{&S15175[44], 27}, {NULL}};
const Segment SGI70538[] = {{&S15175[30], 6}, {NULL}};
const Segment SGI70539[] = {{&S15175[31], 3}, {NULL}};
const Segment SGI70540[] = {{&S15175[85], 6}, {NULL}};
const Segment SGI70541[] = {{&S31749[42], 10}, {NULL}};
const Segment SGI70542[] = {{&S31749[30], 6}, {NULL}};
const Segment SGI70543[] = {{&S31749[117], 3}, {NULL}};
const Segment SGI70544[] = {{&S31749[83], 5}, {NULL}};
const Segment SGI70545[] = {{&S15177[11], 1}, {NULL}};
const Segment SGI70546[] = {{&S15177[44], 19}, {NULL}};
const Segment SGI70547[] = {{&S15177[30], 6}, {NULL}};
const Segment SGI70548[] = {{&S15177[31], 3}, {NULL}};
const Segment SGI70549[] = {{&S15177[77], 6}, {NULL}};
const Segment SGI70550[] = {{&S31750[44], 9}, {NULL}};
const Segment SGI70551[] = {{&S31750[44], 26}, {NULL}};
const Segment SGI70552[] = {{&S31750[30], 6}, {NULL}};
const Segment SGI70553[] = {{&S31750[118], 3}, {NULL}};
const Segment SGI70554[] = {{&S31750[84], 5}, {NULL}};
const Segment SGI70555[] = {{&S31751[38], 11}, {NULL}};
const Segment SGI70556[] = {{&S31751[31], 6}, {NULL}};
const Segment SGI70557[] = {{&S15187[44], 8}, {NULL}};
const Segment SGI70558[] = {{&S15187[44], 31}, {NULL}};
const Segment SGI70559[] = {{&S15187[30], 6}, {NULL}};
const Segment SGI70560[] = {{&S15187[31], 3}, {NULL}};
const Segment SGI70561[] = {{&S15187[89], 6}, {NULL}};
const Segment SGI70562[] = {{&S15189[44], 10}, {NULL}};
const Segment SGI70563[] = {{&S15189[44], 22}, {NULL}};
const Segment SGI70564[] = {{&S15189[30], 6}, {NULL}};
const Segment SGI70565[] = {{&S15189[31], 3}, {NULL}};
const Segment SGI70566[] = {{&S15189[80], 6}, {NULL}};
const Segment SGI70567[] = {{&S15191[44], 15}, {NULL}};
const Segment SGI70568[] = {{&S15191[44], 27}, {NULL}};
const Segment SGI70569[] = {{&S15191[30], 6}, {NULL}};
const Segment SGI70570[] = {{&S15191[31], 3}, {NULL}};
const Segment SGI70571[] = {{&S15191[85], 6}, {NULL}};
const Segment SGI70572[] = {{&S15199[44], 7}, {NULL}};
const Segment SGI70573[] = {{&S15199[44], 19}, {NULL}};
const Segment SGI70574[] = {{&S15199[30], 6}, {NULL}};
const Segment SGI70575[] = {{&S15199[31], 3}, {NULL}};
const Segment SGI70576[] = {{&S15199[77], 6}, {NULL}};
const Segment SGI70577[] = {{&S15200[23], 15}, {NULL}};
const Segment SGI70578[] = {{&S15200[31], 14}, {NULL}};
const Segment SGI70579[] = {{&S15201[44], 6}, {NULL}};
const Segment SGI70580[] = {{&S15201[44], 18}, {NULL}};
const Segment SGI70581[] = {{&S15201[30], 6}, {NULL}};
const Segment SGI70582[] = {{&S15201[31], 3}, {NULL}};
const Segment SGI70583[] = {{&S15201[76], 6}, {NULL}};
const Segment SGI70584[] = {{&S15202[44], 14}, {NULL}};
const Segment SGI70585[] = {{&S15202[44], 26}, {NULL}};
const Segment SGI70586[] = {{&S15202[30], 6}, {NULL}};
const Segment SGI70587[] = {{&S15202[31], 3}, {NULL}};
const Segment SGI70588[] = {{&S15202[84], 6}, {NULL}};
const Segment SGI70589[] = {{&S15203[44], 14}, {NULL}};
const Segment SGI70590[] = {{&S15203[44], 26}, {NULL}};
const Segment SGI70591[] = {{&S15203[30], 6}, {NULL}};
const Segment SGI70592[] = {{&S15203[31], 3}, {NULL}};
const Segment SGI70593[] = {{&S15203[84], 6}, {NULL}};
const Segment SGI70594[] = {{&S15204[44], 7}, {NULL}};
const Segment SGI70595[] = {{&S15204[44], 19}, {NULL}};
const Segment SGI70596[] = {{&S15204[30], 6}, {NULL}};
const Segment SGI70597[] = {{&S15204[31], 3}, {NULL}};
const Segment SGI70598[] = {{&S15204[77], 6}, {NULL}};
const Segment SGI70599[] = {{&S15205[44], 7}, {NULL}};
const Segment SGI70600[] = {{&S15205[44], 19}, {NULL}};
const Segment SGI70601[] = {{&S15205[30], 6}, {NULL}};
const Segment SGI70602[] = {{&S15205[31], 3}, {NULL}};
const Segment SGI70603[] = {{&S15205[77], 6}, {NULL}};
const Segment SGI70604[] = {{&S15207[44], 5}, {NULL}};
const Segment SGI70605[] = {{&S15207[44], 17}, {NULL}};
const Segment SGI70606[] = {{&S15207[30], 6}, {NULL}};
const Segment SGI70607[] = {{&S15207[31], 3}, {NULL}};
const Segment SGI70608[] = {{&S15207[75], 6}, {NULL}};
const Segment SGI70609[] = {{&S15208[44], 5}, {NULL}};
const Segment SGI70610[] = {{&S15208[44], 17}, {NULL}};
const Segment SGI70611[] = {{&S15208[30], 6}, {NULL}};
const Segment SGI70612[] = {{&S15208[31], 3}, {NULL}};
const Segment SGI70613[] = {{&S15208[75], 6}, {NULL}};
const Segment SGI70614[] = {{&S15209[44], 19}, {NULL}};
const Segment SGI70615[] = {{&S15209[44], 31}, {NULL}};
const Segment SGI70616[] = {{&S15209[30], 6}, {NULL}};
const Segment SGI70617[] = {{&S15209[31], 3}, {NULL}};
const Segment SGI70618[] = {{&S15209[89], 6}, {NULL}};
const Segment SGI70619[] = {{&S15210[44], 11}, {NULL}};
const Segment SGI70620[] = {{&S15210[44], 23}, {NULL}};
const Segment SGI70621[] = {{&S15210[30], 6}, {NULL}};
const Segment SGI70622[] = {{&S15210[31], 3}, {NULL}};
const Segment SGI70623[] = {{&S15210[81], 6}, {NULL}};
const Segment SGI70624[] = {{&S15211[44], 6}, {NULL}};
const Segment SGI70625[] = {{&S15211[44], 18}, {NULL}};
const Segment SGI70626[] = {{&S15211[30], 6}, {NULL}};
const Segment SGI70627[] = {{&S15211[31], 3}, {NULL}};
const Segment SGI70628[] = {{&S15211[76], 6}, {NULL}};
const Segment SGI70629[] = {{&S15214[44], 13}, {NULL}};
const Segment SGI70630[] = {{&S15214[44], 27}, {NULL}};
const Segment SGI70631[] = {{&S15214[30], 6}, {NULL}};
const Segment SGI70632[] = {{&S15214[31], 3}, {NULL}};
const Segment SGI70633[] = {{&S15214[85], 6}, {NULL}};
const Segment SGI70634[] = {{&S15215[11], 1}, {NULL}};
const Segment SGI70635[] = {{&S15215[44], 23}, {NULL}};
const Segment SGI70636[] = {{&S15215[30], 6}, {NULL}};
const Segment SGI70637[] = {{&S15215[31], 3}, {NULL}};
const Segment SGI70638[] = {{&S15215[81], 6}, {NULL}};
const Segment SGI70639[] = {{&S15216[44], 7}, {NULL}};
const Segment SGI70640[] = {{&S15216[44], 19}, {NULL}};
const Segment SGI70641[] = {{&S15216[30], 6}, {NULL}};
const Segment SGI70642[] = {{&S15216[31], 3}, {NULL}};
const Segment SGI70643[] = {{&S15216[77], 6}, {NULL}};
const Segment SGI70644[] = {{&S15218[44], 10}, {NULL}};
const Segment SGI70645[] = {{&S15218[44], 22}, {NULL}};
const Segment SGI70646[] = {{&S15218[30], 6}, {NULL}};
const Segment SGI70647[] = {{&S15218[31], 3}, {NULL}};
const Segment SGI70648[] = {{&S15218[80], 6}, {NULL}};
const Segment SGI70649[] = {{&S15220[44], 9}, {NULL}};
const Segment SGI70650[] = {{&S15220[44], 21}, {NULL}};
const Segment SGI70651[] = {{&S15220[30], 6}, {NULL}};
const Segment SGI70652[] = {{&S15220[31], 3}, {NULL}};
const Segment SGI70653[] = {{&S15220[79], 6}, {NULL}};
const Segment SGI70654[] = {{&S15221[44], 7}, {NULL}};
const Segment SGI70655[] = {{&S15221[44], 19}, {NULL}};
const Segment SGI70656[] = {{&S15221[30], 6}, {NULL}};
const Segment SGI70657[] = {{&S15221[31], 3}, {NULL}};
const Segment SGI70658[] = {{&S15221[77], 6}, {NULL}};
const Segment SGI70659[] = {{&S15227[44], 21}, {NULL}};
const Segment SGI70660[] = {{&S15227[44], 33}, {NULL}};
const Segment SGI70661[] = {{&S15227[30], 6}, {NULL}};
const Segment SGI70662[] = {{&S15227[31], 3}, {NULL}};
const Segment SGI70663[] = {{&S15227[91], 6}, {NULL}};
const Segment SGI70664[] = {{&S15228[44], 8}, {NULL}};
const Segment SGI70665[] = {{&S15228[44], 20}, {NULL}};
const Segment SGI70666[] = {{&S15228[30], 6}, {NULL}};
const Segment SGI70667[] = {{&S15228[31], 3}, {NULL}};
const Segment SGI70668[] = {{&S15228[78], 6}, {NULL}};
const Segment SGI70669[] = {{&S15230[44], 7}, {NULL}};
const Segment SGI70670[] = {{&S15230[44], 19}, {NULL}};
const Segment SGI70671[] = {{&S15230[30], 6}, {NULL}};
const Segment SGI70672[] = {{&S15230[31], 3}, {NULL}};
const Segment SGI70673[] = {{&S15230[77], 6}, {NULL}};
const Segment SGI70674[] = {{&S15231[44], 9}, {NULL}};
const Segment SGI70675[] = {{&S15231[44], 21}, {NULL}};
const Segment SGI70676[] = {{&S15231[30], 6}, {NULL}};
const Segment SGI70677[] = {{&S15231[31], 3}, {NULL}};
const Segment SGI70678[] = {{&S15231[79], 6}, {NULL}};
const Segment SGI70679[] = {{&S15232[44], 11}, {NULL}};
const Segment SGI70680[] = {{&S15232[44], 23}, {NULL}};
const Segment SGI70681[] = {{&S15232[30], 6}, {NULL}};
const Segment SGI70682[] = {{&S15232[31], 3}, {NULL}};
const Segment SGI70683[] = {{&S15232[81], 6}, {NULL}};
const Segment SGI70684[] = {{&S15239[44], 8}, {NULL}};
const Segment SGI70685[] = {{&S15239[44], 20}, {NULL}};
const Segment SGI70686[] = {{&S15239[30], 6}, {NULL}};
const Segment SGI70687[] = {{&S15239[31], 3}, {NULL}};
const Segment SGI70688[] = {{&S15239[78], 6}, {NULL}};
const Segment SGI70689[] = {{&S15240[44], 6}, {NULL}};
const Segment SGI70690[] = {{&S15240[44], 18}, {NULL}};
const Segment SGI70691[] = {{&S15240[30], 6}, {NULL}};
const Segment SGI70692[] = {{&S15240[31], 3}, {NULL}};
const Segment SGI70693[] = {{&S15240[76], 6}, {NULL}};
const Segment SGI70694[] = {{&S15242[44], 8}, {NULL}};
const Segment SGI70695[] = {{&S15242[44], 20}, {NULL}};
const Segment SGI70696[] = {{&S15242[30], 6}, {NULL}};
const Segment SGI70697[] = {{&S15242[31], 3}, {NULL}};
const Segment SGI70698[] = {{&S15242[78], 6}, {NULL}};
const Segment SGI70699[] = {{&S15245[44], 13}, {NULL}};
const Segment SGI70700[] = {{&S15245[44], 25}, {NULL}};
const Segment SGI70701[] = {{&S15245[30], 6}, {NULL}};
const Segment SGI70702[] = {{&S15245[31], 3}, {NULL}};
const Segment SGI70703[] = {{&S15245[83], 6}, {NULL}};
const Segment SGI70704[] = {{&S15246[44], 11}, {NULL}};
const Segment SGI70705[] = {{&S15246[44], 23}, {NULL}};
const Segment SGI70706[] = {{&S15246[30], 6}, {NULL}};
const Segment SGI70707[] = {{&S15246[31], 3}, {NULL}};
const Segment SGI70708[] = {{&S15246[81], 6}, {NULL}};
const Segment SGI70709[] = {{&S15249[44], 6}, {NULL}};
const Segment SGI70710[] = {{&S15249[44], 19}, {NULL}};
const Segment SGI70711[] = {{&S15249[30], 6}, {NULL}};
const Segment SGI70712[] = {{&S15249[31], 3}, {NULL}};
const Segment SGI70713[] = {{&S15249[77], 6}, {NULL}};
const Segment SGI70714[] = {{&S15254[44], 11}, {NULL}};
const Segment SGI70715[] = {{&S15254[44], 23}, {NULL}};
const Segment SGI70716[] = {{&S15254[30], 6}, {NULL}};
const Segment SGI70717[] = {{&S15254[31], 3}, {NULL}};
const Segment SGI70718[] = {{&S15254[81], 6}, {NULL}};
const Segment SGI70719[] = {{&S15264[44], 5}, {NULL}};
const Segment SGI70720[] = {{&S15264[44], 17}, {NULL}};
const Segment SGI70721[] = {{&S15264[30], 6}, {NULL}};
const Segment SGI70722[] = {{&S15264[31], 3}, {NULL}};
const Segment SGI70723[] = {{&S15264[75], 6}, {NULL}};
const Segment SGI70724[] = {{&S15265[44], 9}, {NULL}};
const Segment SGI70725[] = {{&S15265[44], 21}, {NULL}};
const Segment SGI70726[] = {{&S15265[30], 6}, {NULL}};
const Segment SGI70727[] = {{&S15265[31], 3}, {NULL}};
const Segment SGI70728[] = {{&S15265[79], 6}, {NULL}};
const Segment SGI70729[] = {{&S15267[44], 5}, {NULL}};
const Segment SGI70730[] = {{&S15267[44], 17}, {NULL}};
const Segment SGI70731[] = {{&S15267[30], 6}, {NULL}};
const Segment SGI70732[] = {{&S15267[31], 3}, {NULL}};
const Segment SGI70733[] = {{&S15267[75], 6}, {NULL}};
const Segment SGI70734[] = {{&S15268[44], 9}, {NULL}};
const Segment SGI70735[] = {{&S15268[44], 21}, {NULL}};
const Segment SGI70736[] = {{&S15268[30], 6}, {NULL}};
const Segment SGI70737[] = {{&S15268[31], 3}, {NULL}};
const Segment SGI70738[] = {{&S15268[79], 6}, {NULL}};
const Segment SGI70739[] = {{&S15269[44], 6}, {NULL}};
const Segment SGI70740[] = {{&S15269[44], 18}, {NULL}};
const Segment SGI70741[] = {{&S15269[30], 6}, {NULL}};
const Segment SGI70742[] = {{&S15269[31], 3}, {NULL}};
const Segment SGI70743[] = {{&S15269[76], 6}, {NULL}};
const Segment SGI70744[] = {{&S15272[44], 7}, {NULL}};
const Segment SGI70745[] = {{&S15272[44], 24}, {NULL}};
const Segment SGI70746[] = {{&S15272[30], 6}, {NULL}};
const Segment SGI70747[] = {{&S15272[85], 6}, {NULL}};
const Segment SGI70748[] = {{&S15273[44], 5}, {NULL}};
const Segment SGI70749[] = {{&S15273[44], 17}, {NULL}};
const Segment SGI70750[] = {{&S15273[30], 6}, {NULL}};
const Segment SGI70751[] = {{&S15273[31], 3}, {NULL}};
const Segment SGI70752[] = {{&S15273[75], 5}, {NULL}};
const Segment SGI70753[] = {{&S15275[44], 12}, {NULL}};
const Segment SGI70754[] = {{&S15275[44], 24}, {NULL}};
const Segment SGI70755[] = {{&S15275[30], 6}, {NULL}};
const Segment SGI70756[] = {{&S15275[31], 3}, {NULL}};
const Segment SGI70757[] = {{&S15275[82], 6}, {NULL}};
const Segment SGI70758[] = {{&S15276[44], 9}, {NULL}};
const Segment SGI70759[] = {{&S15276[44], 21}, {NULL}};
const Segment SGI70760[] = {{&S15276[30], 6}, {NULL}};
const Segment SGI70761[] = {{&S15276[31], 3}, {NULL}};
const Segment SGI70762[] = {{&S15276[79], 6}, {NULL}};
const Segment SGI70763[] = {{&S15278[44], 13}, {NULL}};
const Segment SGI70764[] = {{&S15278[44], 25}, {NULL}};
const Segment SGI70765[] = {{&S15278[30], 6}, {NULL}};
const Segment SGI70766[] = {{&S15278[31], 3}, {NULL}};
const Segment SGI70767[] = {{&S15278[83], 6}, {NULL}};
const Segment SGI70768[] = {{&S15279[44], 10}, {NULL}};
const Segment SGI70769[] = {{&S15279[44], 22}, {NULL}};
const Segment SGI70770[] = {{&S15279[30], 6}, {NULL}};
const Segment SGI70771[] = {{&S15279[31], 3}, {NULL}};
const Segment SGI70772[] = {{&S15279[80], 6}, {NULL}};
const Segment SGI70773[] = {{&S15282[44], 7}, {NULL}};
const Segment SGI70774[] = {{&S15282[44], 18}, {NULL}};
const Segment SGI70775[] = {{&S15282[30], 6}, {NULL}};
const Segment SGI70776[] = {{&S15282[31], 3}, {NULL}};
const Segment SGI70777[] = {{&S15282[76], 6}, {NULL}};
const Segment SGI70778[] = {{&S15283[44], 6}, {NULL}};
const Segment SGI70779[] = {{&S15283[44], 18}, {NULL}};
const Segment SGI70780[] = {{&S15283[30], 6}, {NULL}};
const Segment SGI70781[] = {{&S15283[31], 3}, {NULL}};
const Segment SGI70782[] = {{&S15283[76], 6}, {NULL}};
const Segment SGI70783[] = {{&S15284[44], 23}, {NULL}};
const Segment SGI70784[] = {{&S15284[44], 35}, {NULL}};
const Segment SGI70785[] = {{&S15284[30], 6}, {NULL}};
const Segment SGI70786[] = {{&S15284[31], 3}, {NULL}};
const Segment SGI70787[] = {{&S15284[93], 6}, {NULL}};
const Segment SGI70788[] = {{&S15287[44], 8}, {NULL}};
const Segment SGI70789[] = {{&S15287[44], 20}, {NULL}};
const Segment SGI70790[] = {{&S15287[30], 6}, {NULL}};
const Segment SGI70791[] = {{&S15287[31], 3}, {NULL}};
const Segment SGI70792[] = {{&S15287[78], 6}, {NULL}};
const Segment SGI70793[] = {{&S15288[52], 6}, {NULL}};
const Segment SGI70794[] = {{&S15288[114], 6}, {NULL}};
const Segment SGI70795[] = {{&S15290[44], 8}, {NULL}};
const Segment SGI70796[] = {{&S15290[44], 20}, {NULL}};
const Segment SGI70797[] = {{&S15290[30], 6}, {NULL}};
const Segment SGI70798[] = {{&S15290[31], 3}, {NULL}};
const Segment SGI70799[] = {{&S15290[78], 6}, {NULL}};
const Segment SGI70800[] = {{&S15291[44], 12}, {NULL}};
const Segment SGI70801[] = {{&S15291[44], 24}, {NULL}};
const Segment SGI70802[] = {{&S15291[30], 6}, {NULL}};
const Segment SGI70803[] = {{&S15291[31], 3}, {NULL}};
const Segment SGI70804[] = {{&S15291[82], 6}, {NULL}};
const Segment SGI70805[] = {{&S15292[44], 14}, {NULL}};
const Segment SGI70806[] = {{&S15292[44], 26}, {NULL}};
const Segment SGI70807[] = {{&S15292[30], 6}, {NULL}};
const Segment SGI70808[] = {{&S15292[31], 3}, {NULL}};
const Segment SGI70809[] = {{&S15292[84], 6}, {NULL}};
const Segment SGI70810[] = {{&S15293[44], 8}, {NULL}};
const Segment SGI70811[] = {{&S15293[44], 20}, {NULL}};
const Segment SGI70812[] = {{&S15293[30], 6}, {NULL}};
const Segment SGI70813[] = {{&S15293[31], 3}, {NULL}};
const Segment SGI70814[] = {{&S15293[78], 6}, {NULL}};
const Segment SGI70815[] = {{&S19637[44], 21}, {NULL}};
const Segment SGI70816[] = {{&S19637[44], 33}, {NULL}};
const Segment SGI70817[] = {{&S19637[30], 6}, {NULL}};
const Segment SGI70818[] = {{&S19637[31], 3}, {NULL}};
const Segment SGI70819[] = {{&S19637[91], 6}, {NULL}};
const Segment SGI70820[] = {{&S19637[0], 13}, {NULL}};
const Segment SGI70821[] = {{&S15296[44], 7}, {NULL}};
const Segment SGI70822[] = {{&S15296[44], 29}, {NULL}};
const Segment SGI70823[] = {{&S15296[30], 6}, {NULL}};
const Segment SGI70824[] = {{&S15296[31], 3}, {NULL}};
const Segment SGI70825[] = {{&S15296[87], 6}, {NULL}};
const Segment SGI70826[] = {{&S15299[52], 6}, {NULL}};
const Segment SGI70827[] = {{&S15299[114], 6}, {NULL}};
const Segment SGI70828[] = {{&S15304[44], 9}, {NULL}};
const Segment SGI70829[] = {{&S15304[44], 21}, {NULL}};
const Segment SGI70830[] = {{&S15304[30], 6}, {NULL}};
const Segment SGI70831[] = {{&S15304[31], 3}, {NULL}};
const Segment SGI70832[] = {{&S15304[79], 6}, {NULL}};
const Segment SGI70833[] = {{&S15305[44], 15}, {NULL}};
const Segment SGI70834[] = {{&S15305[44], 27}, {NULL}};
const Segment SGI70835[] = {{&S15305[30], 6}, {NULL}};
const Segment SGI70836[] = {{&S15305[31], 3}, {NULL}};
const Segment SGI70837[] = {{&S15305[85], 6}, {NULL}};
const Segment SGI70838[] = {{&S15308[11], 1}, {NULL}};
const Segment SGI70839[] = {{&S15308[44], 25}, {NULL}};
const Segment SGI70840[] = {{&S15308[30], 6}, {NULL}};
const Segment SGI70841[] = {{&S15308[31], 3}, {NULL}};
const Segment SGI70842[] = {{&S15308[83], 6}, {NULL}};
const Segment SGI70843[] = {{&S15312[44], 5}, {NULL}};
const Segment SGI70844[] = {{&S15312[44], 23}, {NULL}};
const Segment SGI70845[] = {{&S15312[30], 6}, {NULL}};
const Segment SGI70846[] = {{&S15312[31], 3}, {NULL}};
const Segment SGI70847[] = {{&S15312[81], 6}, {NULL}};
const Segment SGI70848[] = {{&S15321[42], 13}, {NULL}};
const Segment SGI70849[] = {{&S15321[30], 6}, {NULL}};
const Segment SGI70850[] = {{&S15321[31], 3}, {NULL}};
const Segment SGI70851[] = {{&S15321[79], 6}, {NULL}};
const Segment SGI70852[] = {{&S15324[22], 9}, {NULL}};
const Segment SGI70853[] = {{&S15324[44], 21}, {NULL}};
const Segment SGI70854[] = {{&S15324[30], 6}, {&S15324[52], 13}, {NULL}};
const Segment SGI70855[] = {{&S15324[31], 3}, {NULL}};
const Segment SGI70856[] = {{&S15324[79], 6}, {NULL}};
const Segment SGI70857[] = {{&S15325[44], 30}, {NULL}};
const Segment SGI70858[] = {{&S15325[44], 42}, {NULL}};
const Segment SGI70859[] = {{&S15325[30], 6}, {NULL}};
const Segment SGI70860[] = {{&S15325[31], 3}, {NULL}};
const Segment SGI70861[] = {{&S15325[100], 6}, {NULL}};
const Segment SGI70862[] = {{&S15328[11], 1}, {NULL}};
const Segment SGI70863[] = {{&S15328[44], 28}, {NULL}};
const Segment SGI70864[] = {{&S15328[30], 6}, {NULL}};
const Segment SGI70865[] = {{&S15328[31], 3}, {NULL}};
const Segment SGI70866[] = {{&S15328[86], 6}, {NULL}};
const Segment SGI70867[] = {{&S15332[11], 1}, {NULL}};
const Segment SGI70868[] = {{&S15332[44], 8}, {NULL}};
const Segment SGI70869[] = {{&S15332[30], 6}, {NULL}};
const Segment SGI70870[] = {{&S15332[31], 3}, {NULL}};
const Segment SGI70871[] = {{&S15332[66], 6}, {NULL}};
const Segment SGI70872[] = {{&S15340[11], 1}, {NULL}};
const Segment SGI70873[] = {{&S15340[30], 6}, {NULL}};
const Segment SGI70874[] = {{&S15340[31], 3}, {NULL}};
const Segment SGI70875[] = {{&S15340[64], 6}, {NULL}};
const Segment SGI70876[] = {{&S15344[44], 29}, {NULL}};
const Segment SGI70877[] = {{&S15344[44], 41}, {NULL}};
const Segment SGI70878[] = {{&S15344[30], 6}, {NULL}};
const Segment SGI70879[] = {{&S15344[31], 3}, {NULL}};
const Segment SGI70880[] = {{&S15344[99], 6}, {NULL}};
const Segment SGI70881[] = {{&S15349[37], 7}, {NULL}};
const Segment SGI70882[] = {{&S15349[30], 6}, {NULL}};
const Segment SGI70883[] = {{&S15349[73], 4}, {NULL}};
const Segment SGI70884[] = {{&S15349[73], 3}, {NULL}};
const Segment SGI70885[] = {{&S15350[11], 1}, {NULL}};
const Segment SGI70886[] = {{&S15350[44], 22}, {NULL}};
const Segment SGI70887[] = {{&S15350[30], 6}, {NULL}};
const Segment SGI70888[] = {{&S15350[31], 3}, {NULL}};
const Segment SGI70889[] = {{&S15350[80], 6}, {NULL}};
const Segment SGI70890[] = {{&S15358[11], 1}, {NULL}};
const Segment SGI70891[] = {{&S15358[44], 16}, {NULL}};
const Segment SGI70892[] = {{&S15358[30], 6}, {NULL}};
const Segment SGI70893[] = {{&S15358[31], 3}, {NULL}};
const Segment SGI70894[] = {{&S15358[74], 6}, {NULL}};
const Segment SGI70895[] = {{&S15359[44], 29}, {NULL}};
const Segment SGI70896[] = {{&S15359[44], 41}, {NULL}};
const Segment SGI70897[] = {{&S15359[30], 6}, {NULL}};
const Segment SGI70898[] = {{&S15359[31], 3}, {NULL}};
const Segment SGI70899[] = {{&S15359[99], 6}, {NULL}};
const Segment SGI70900[] = {{&S15362[44], 23}, {NULL}};
const Segment SGI70901[] = {{&S15362[44], 35}, {NULL}};
const Segment SGI70902[] = {{&S15362[30], 6}, {NULL}};
const Segment SGI70903[] = {{&S15362[31], 3}, {NULL}};
const Segment SGI70904[] = {{&S15362[93], 6}, {NULL}};
const Segment SGI70905[] = {{&S15379[44], 11}, {NULL}};
const Segment SGI70906[] = {{&S15379[44], 23}, {NULL}};
const Segment SGI70907[] = {{&S15379[30], 6}, {NULL}};
const Segment SGI70908[] = {{&S15379[31], 3}, {NULL}};
const Segment SGI70909[] = {{&S15379[81], 6}, {NULL}};
const Segment SGI70910[] = {{&S15380[11], 1}, {NULL}};
const Segment SGI70911[] = {{&S15380[44], 25}, {NULL}};
const Segment SGI70912[] = {{&S15380[30], 6}, {NULL}};
const Segment SGI70913[] = {{&S15380[31], 3}, {NULL}};
const Segment SGI70914[] = {{&S15380[83], 6}, {NULL}};
const Segment SGI70915[] = {{&S15381[11], 1}, {NULL}};
const Segment SGI70916[] = {{&S15381[44], 19}, {NULL}};
const Segment SGI70917[] = {{&S15381[30], 6}, {NULL}};
const Segment SGI70918[] = {{&S15381[31], 3}, {NULL}};
const Segment SGI70919[] = {{&S15381[77], 6}, {NULL}};
const Segment SGI70920[] = {{&S15385[11], 1}, {NULL}};
const Segment SGI70921[] = {{&S15385[44], 21}, {NULL}};
const Segment SGI70922[] = {{&S15385[30], 6}, {NULL}};
const Segment SGI70923[] = {{&S15385[31], 3}, {NULL}};
const Segment SGI70924[] = {{&S15385[79], 6}, {NULL}};
const Segment SGI70925[] = {{&S15386[11], 1}, {NULL}};
const Segment SGI70926[] = {{&S15386[44], 23}, {NULL}};
const Segment SGI70927[] = {{&S15386[30], 6}, {NULL}};
const Segment SGI70928[] = {{&S15386[31], 3}, {NULL}};
const Segment SGI70929[] = {{&S15386[81], 6}, {NULL}};
const Segment SGI70930[] = {{&S15391[44], 9}, {NULL}};
const Segment SGI70931[] = {{&S15391[44], 21}, {NULL}};
const Segment SGI70932[] = {{&S15391[30], 6}, {NULL}};
const Segment SGI70933[] = {{&S15391[31], 3}, {NULL}};
const Segment SGI70934[] = {{&S15391[79], 6}, {NULL}};
const Segment SGI70935[] = {{&S15394[44], 5}, {NULL}};
const Segment SGI70936[] = {{&S15394[44], 14}, {NULL}};
const Segment SGI70937[] = {{&S15394[30], 6}, {NULL}};
const Segment SGI70938[] = {{&S15394[31], 3}, {NULL}};
const Segment SGI70939[] = {{&S15394[72], 6}, {NULL}};
const Segment SGI70940[] = {{&S15398[11], 1}, {NULL}};
const Segment SGI70941[] = {{&S15398[44], 24}, {NULL}};
const Segment SGI70942[] = {{&S15398[30], 6}, {NULL}};
const Segment SGI70943[] = {{&S15398[31], 3}, {NULL}};
const Segment SGI70944[] = {{&S15398[82], 6}, {NULL}};
const Segment SGI70945[] = {{&S15400[44], 14}, {NULL}};
const Segment SGI70946[] = {{&S15400[44], 26}, {NULL}};
const Segment SGI70947[] = {{&S15400[30], 6}, {NULL}};
const Segment SGI70948[] = {{&S15400[31], 3}, {NULL}};
const Segment SGI70949[] = {{&S15400[84], 6}, {NULL}};
const Segment SGI70950[] = {{&S15401[44], 5}, {NULL}};
const Segment SGI70951[] = {{&S15401[44], 23}, {NULL}};
const Segment SGI70952[] = {{&S15401[30], 6}, {NULL}};
const Segment SGI70953[] = {{&S15401[31], 3}, {NULL}};
const Segment SGI70954[] = {{&S15401[81], 6}, {NULL}};
const Segment SGI70955[] = {{&S15402[44], 17}, {NULL}};
const Segment SGI70956[] = {{&S15402[44], 29}, {NULL}};
const Segment SGI70957[] = {{&S15402[30], 6}, {NULL}};
const Segment SGI70958[] = {{&S15402[31], 3}, {NULL}};
const Segment SGI70959[] = {{&S15402[87], 5}, {NULL}};
const Segment SGI70960[] = {{&S15403[11], 1}, {NULL}};
const Segment SGI70961[] = {{&S15403[44], 23}, {NULL}};
const Segment SGI70962[] = {{&S15403[30], 6}, {NULL}};
const Segment SGI70963[] = {{&S15403[31], 3}, {NULL}};
const Segment SGI70964[] = {{&S15403[81], 6}, {NULL}};
const Segment SGI70965[] = {{&S15405[11], 1}, {NULL}};
const Segment SGI70966[] = {{&S15405[44], 25}, {NULL}};
const Segment SGI70967[] = {{&S15405[30], 6}, {NULL}};
const Segment SGI70968[] = {{&S15405[31], 3}, {NULL}};
const Segment SGI70969[] = {{&S15405[83], 6}, {NULL}};
const Segment SGI70970[] = {{&S15407[11], 1}, {NULL}};
const Segment SGI70971[] = {{&S15407[44], 25}, {NULL}};
const Segment SGI70972[] = {{&S15407[30], 6}, {NULL}};
const Segment SGI70973[] = {{&S15407[31], 3}, {NULL}};
const Segment SGI70974[] = {{&S15407[83], 6}, {NULL}};
const Segment SGI70975[] = {{&S15410[44], 19}, {NULL}};
const Segment SGI70976[] = {{&S15410[44], 31}, {NULL}};
const Segment SGI70977[] = {{&S15410[30], 6}, {NULL}};
const Segment SGI70978[] = {{&S15410[31], 3}, {NULL}};
const Segment SGI70979[] = {{&S15410[89], 6}, {NULL}};
const Segment SGI70980[] = {{&S15411[37], 7}, {NULL}};
const Segment SGI70981[] = {{&S15411[30], 6}, {NULL}};
const Segment SGI70982[] = {{&S15411[31], 3}, {NULL}};
const Segment SGI70983[] = {{&S15411[99], 6}, {NULL}};
const Segment SGI70984[] = {{&S15421[44], 25}, {NULL}};
const Segment SGI70985[] = {{&S15421[44], 37}, {NULL}};
const Segment SGI70986[] = {{&S15421[30], 6}, {NULL}};
const Segment SGI70987[] = {{&S15421[31], 3}, {NULL}};
const Segment SGI70988[] = {{&S15421[95], 6}, {NULL}};
const Segment SGI70989[] = {{&S15424[38], 11}, {NULL}};
const Segment SGI70990[] = {{&S15424[31], 6}, {NULL}};
const Segment SGI70991[] = {{&S15425[23], 15}, {NULL}};
const Segment SGI70992[] = {{&S15425[31], 16}, {NULL}};
const Segment SGI70993[] = {{&S15426[23], 15}, {NULL}};
const Segment SGI70994[] = {{&S15426[31], 29}, {NULL}};
const Segment SGI70995[] = {{&S15429[11], 1}, {NULL}};
const Segment SGI70996[] = {{&S15429[44], 23}, {NULL}};
const Segment SGI70997[] = {{&S15429[30], 6}, {NULL}};
const Segment SGI70998[] = {{&S15429[31], 3}, {NULL}};
const Segment SGI70999[] = {{&S15429[81], 6}, {NULL}};
const Segment SGI71000[] = {{&S15431[11], 1}, {NULL}};
const Segment SGI71001[] = {{&S15431[44], 23}, {NULL}};
const Segment SGI71002[] = {{&S15431[30], 6}, {NULL}};
const Segment SGI71003[] = {{&S15431[31], 3}, {NULL}};
const Segment SGI71004[] = {{&S15431[81], 6}, {NULL}};
const Segment SGI71005[] = {{&S15435[11], 1}, {NULL}};
const Segment SGI71006[] = {{&S15435[44], 19}, {NULL}};
const Segment SGI71007[] = {{&S15435[30], 6}, {NULL}};
const Segment SGI71008[] = {{&S15435[31], 3}, {NULL}};
const Segment SGI71009[] = {{&S15435[77], 6}, {NULL}};
const Segment SGI71010[] = {{&S15443[42], 17}, {NULL}};
const Segment SGI71011[] = {{&S15443[30], 6}, {NULL}};
const Segment SGI71012[] = {{&S15443[31], 3}, {NULL}};
const Segment SGI71013[] = {{&S15443[85], 5}, {NULL}};
const Segment SGI71014[] = {{&S15445[38], 6}, {NULL}};
const Segment SGI71015[] = {{&S15445[31], 6}, {NULL}};
const Segment SGI71016[] = {{&S15447[11], 1}, {NULL}};
const Segment SGI71017[] = {{&S15447[44], 19}, {NULL}};
const Segment SGI71018[] = {{&S15447[30], 6}, {NULL}};
const Segment SGI71019[] = {{&S15447[31], 3}, {NULL}};
const Segment SGI71020[] = {{&S15447[77], 6}, {NULL}};
const Segment SGI71021[] = {{&S15448[38], 8}, {NULL}};
const Segment SGI71022[] = {{&S15448[31], 6}, {NULL}};
const Segment SGI71023[] = {{&S15449[23], 15}, {NULL}};
const Segment SGI71024[] = {{&S15449[31], 19}, {NULL}};
const Segment SGI71025[] = {{&S15450[44], 13}, {NULL}};
const Segment SGI71026[] = {{&S15450[44], 25}, {NULL}};
const Segment SGI71027[] = {{&S15450[30], 6}, {NULL}};
const Segment SGI71028[] = {{&S15450[31], 3}, {NULL}};
const Segment SGI71029[] = {{&S15450[83], 6}, {NULL}};
const Segment SGI71030[] = {{&S15451[44], 3}, {NULL}};
const Segment SGI71031[] = {{&S15451[44], 15}, {NULL}};
const Segment SGI71032[] = {{&S15451[30], 6}, {NULL}};
const Segment SGI71033[] = {{&S15451[31], 3}, {NULL}};
const Segment SGI71034[] = {{&S15451[73], 6}, {NULL}};
const Segment SGI71035[] = {{&S15452[44], 26}, {NULL}};
const Segment SGI71036[] = {{&S15452[44], 38}, {NULL}};
const Segment SGI71037[] = {{&S15452[30], 6}, {NULL}};
const Segment SGI71038[] = {{&S15452[31], 3}, {NULL}};
const Segment SGI71039[] = {{&S15452[96], 6}, {NULL}};
const Segment SGI71040[] = {{&S15461[11], 1}, {NULL}};
const Segment SGI71041[] = {{&S15461[44], 35}, {NULL}};
const Segment SGI71042[] = {{&S15461[30], 6}, {NULL}};
const Segment SGI71043[] = {{&S15461[31], 3}, {NULL}};
const Segment SGI71044[] = {{&S15461[93], 6}, {NULL}};
const Segment SGI71045[] = {{&S15465[30], 6}, {NULL}};
const Segment SGI71046[] = {{&S15465[31], 3}, {NULL}};
const Segment SGI71047[] = {{&S15465[57], 6}, {NULL}};
const Segment SGI71048[] = {{&S15466[11], 1}, {NULL}};
const Segment SGI71049[] = {{&S15466[44], 22}, {NULL}};
const Segment SGI71050[] = {{&S15466[30], 6}, {NULL}};
const Segment SGI71051[] = {{&S15466[31], 3}, {NULL}};
const Segment SGI71052[] = {{&S15466[80], 6}, {NULL}};
const Segment SGI71053[] = {{&S15471[44], 5}, {NULL}};
const Segment SGI71054[] = {{&S15471[44], 17}, {NULL}};
const Segment SGI71055[] = {{&S15471[30], 6}, {NULL}};
const Segment SGI71056[] = {{&S15471[31], 3}, {NULL}};
const Segment SGI71057[] = {{&S15471[75], 5}, {NULL}};
const Segment SGI71058[] = {{&S32285[44], 10}, {NULL}};
const Segment SGI71059[] = {{&S32285[44], 27}, {NULL}};
const Segment SGI71060[] = {{&S32285[30], 6}, {NULL}};
const Segment SGI71061[] = {{&S32285[119], 3}, {NULL}};
const Segment SGI71062[] = {{&S32285[85], 5}, {NULL}};
const Segment SGI71063[] = {{&S32286[44], 29}, {NULL}};
const Segment SGI71064[] = {{&S32286[44], 46}, {NULL}};
const Segment SGI71065[] = {{&S32286[30], 6}, {NULL}};
const Segment SGI71066[] = {{&S32286[138], 3}, {NULL}};
const Segment SGI71067[] = {{&S32286[104], 5}, {NULL}};
const Segment SGI71068[] = {{&S32288[44], 28}, {NULL}};
const Segment SGI71069[] = {{&S32288[44], 45}, {NULL}};
const Segment SGI71070[] = {{&S32288[30], 6}, {NULL}};
const Segment SGI71071[] = {{&S32288[137], 3}, {NULL}};
const Segment SGI71072[] = {{&S32288[103], 5}, {NULL}};
const Segment SGI71073[] = {{&S32289[44], 11}, {NULL}};
const Segment SGI71074[] = {{&S32289[44], 28}, {NULL}};
const Segment SGI71075[] = {{&S32289[30], 6}, {NULL}};
const Segment SGI71076[] = {{&S32289[120], 3}, {NULL}};
const Segment SGI71077[] = {{&S32289[86], 5}, {NULL}};
const Segment SGI71078[] = {{&S32291[44], 29}, {NULL}};
const Segment SGI71079[] = {{&S32291[44], 46}, {NULL}};
const Segment SGI71080[] = {{&S32291[30], 6}, {NULL}};
const Segment SGI71081[] = {{&S32291[138], 3}, {NULL}};
const Segment SGI71082[] = {{&S32291[104], 5}, {NULL}};
const Segment SGI71083[] = {{&S32292[44], 28}, {NULL}};
const Segment SGI71084[] = {{&S32292[44], 45}, {NULL}};
const Segment SGI71085[] = {{&S32292[30], 6}, {NULL}};
const Segment SGI71086[] = {{&S32292[137], 3}, {NULL}};
const Segment SGI71087[] = {{&S32292[103], 5}, {NULL}};
const Segment SGI71088[] = {{&S32293[38], 10}, {NULL}};
const Segment SGI71089[] = {{&S32293[31], 6}, {NULL}};
const Segment SGI71090[] = {{&S32294[44], 28}, {NULL}};
const Segment SGI71091[] = {{&S32294[44], 45}, {NULL}};
const Segment SGI71092[] = {{&S32294[30], 6}, {NULL}};
const Segment SGI71093[] = {{&S32294[137], 3}, {NULL}};
const Segment SGI71094[] = {{&S32294[103], 5}, {NULL}};
const Segment SGI71095[] = {{&S32295[44], 29}, {NULL}};
const Segment SGI71096[] = {{&S32295[44], 46}, {NULL}};
const Segment SGI71097[] = {{&S32295[30], 6}, {NULL}};
const Segment SGI71098[] = {{&S32295[138], 3}, {NULL}};
const Segment SGI71099[] = {{&S32295[104], 5}, {NULL}};
const Segment SGI71100[] = {{&S32296[44], 11}, {NULL}};
const Segment SGI71101[] = {{&S32296[44], 28}, {NULL}};
const Segment SGI71102[] = {{&S32296[30], 6}, {NULL}};
const Segment SGI71103[] = {{&S32296[120], 3}, {NULL}};
const Segment SGI71104[] = {{&S32296[86], 5}, {NULL}};
const Segment SGI71105[] = {{&S32299[44], 29}, {NULL}};
const Segment SGI71106[] = {{&S32299[44], 46}, {NULL}};
const Segment SGI71107[] = {{&S32299[30], 6}, {NULL}};
const Segment SGI71108[] = {{&S32299[138], 3}, {NULL}};
const Segment SGI71109[] = {{&S32299[104], 5}, {NULL}};
const Segment SGI71110[] = {{&S32300[44], 28}, {NULL}};
const Segment SGI71111[] = {{&S32300[44], 45}, {NULL}};
const Segment SGI71112[] = {{&S32300[30], 6}, {NULL}};
const Segment SGI71113[] = {{&S32300[137], 3}, {NULL}};
const Segment SGI71114[] = {{&S32300[103], 5}, {NULL}};
const Segment SGI71115[] = {{&S32301[44], 28}, {NULL}};
const Segment SGI71116[] = {{&S32301[44], 45}, {NULL}};
const Segment SGI71117[] = {{&S32301[30], 6}, {NULL}};
const Segment SGI71118[] = {{&S32301[137], 3}, {NULL}};
const Segment SGI71119[] = {{&S32301[103], 5}, {NULL}};
const Segment SGI71120[] = {{&S32302[44], 29}, {NULL}};
const Segment SGI71121[] = {{&S32302[44], 46}, {NULL}};
const Segment SGI71122[] = {{&S32302[30], 6}, {NULL}};
const Segment SGI71123[] = {{&S32302[138], 3}, {NULL}};
const Segment SGI71124[] = {{&S32302[104], 5}, {NULL}};
const Segment SGI71125[] = {{&S32303[38], 11}, {NULL}};
const Segment SGI71126[] = {{&S32303[31], 6}, {NULL}};
const Segment SGI71127[] = {{&S32304[42], 10}, {NULL}};
const Segment SGI71128[] = {{&S32304[30], 6}, {NULL}};
const Segment SGI71129[] = {{&S32304[117], 3}, {NULL}};
const Segment SGI71130[] = {{&S32304[83], 5}, {NULL}};
const Segment SGI71131[] = {{&S32305[44], 28}, {NULL}};
const Segment SGI71132[] = {{&S32305[44], 45}, {NULL}};
const Segment SGI71133[] = {{&S32305[30], 6}, {NULL}};
const Segment SGI71134[] = {{&S32305[137], 3}, {NULL}};
const Segment SGI71135[] = {{&S32305[103], 5}, {NULL}};
const Segment SGI71136[] = {{&S15785[44], 7}, {NULL}};
const Segment SGI71137[] = {{&S15785[44], 19}, {NULL}};
const Segment SGI71138[] = {{&S15785[30], 6}, {NULL}};
const Segment SGI71139[] = {{&S15785[31], 3}, {NULL}};
const Segment SGI71140[] = {{&S15785[77], 6}, {NULL}};
const Segment SGI71141[] = {{&S15802[38], 7}, {NULL}};
const Segment SGI71142[] = {{&S15802[31], 6}, {NULL}};
const Segment SGI71143[] = {{&S32629[44], 13}, {NULL}};
const Segment SGI71144[] = {{&S32629[44], 30}, {NULL}};
const Segment SGI71145[] = {{&S32629[30], 6}, {NULL}};
const Segment SGI71146[] = {{&S32629[122], 3}, {NULL}};
const Segment SGI71147[] = {{&S32629[88], 5}, {NULL}};
const Segment SGI71148[] = {{&S32631[38], 13}, {NULL}};
const Segment SGI71149[] = {{&S32631[31], 6}, {NULL}};
const Segment SGI71150[] = {{&S32633[44], 10}, {NULL}};
const Segment SGI71151[] = {{&S32633[44], 27}, {NULL}};
const Segment SGI71152[] = {{&S32633[30], 6}, {NULL}};
const Segment SGI71153[] = {{&S32633[119], 3}, {NULL}};
const Segment SGI71154[] = {{&S32633[85], 5}, {NULL}};
const Segment SGI71155[] = {{&S32634[38], 10}, {NULL}};
const Segment SGI71156[] = {{&S32634[31], 6}, {NULL}};
const Segment SGI71157[] = {{&S32636[44], 13}, {NULL}};
const Segment SGI71158[] = {{&S32636[44], 34}, {NULL}};
const Segment SGI71159[] = {{&S32636[30], 6}, {NULL}};
const Segment SGI71160[] = {{&S32636[126], 3}, {NULL}};
const Segment SGI71161[] = {{&S32636[92], 5}, {NULL}};
const Segment SGI71162[] = {{&S32637[44], 10}, {NULL}};
const Segment SGI71163[] = {{&S32637[44], 31}, {NULL}};
const Segment SGI71164[] = {{&S32637[30], 6}, {NULL}};
const Segment SGI71165[] = {{&S32637[123], 3}, {NULL}};
const Segment SGI71166[] = {{&S32637[89], 5}, {NULL}};
const Segment SGI71167[] = {{&S32638[42], 13}, {NULL}};
const Segment SGI71168[] = {{&S32638[30], 6}, {NULL}};
const Segment SGI71169[] = {{&S32638[120], 3}, {NULL}};
const Segment SGI71170[] = {{&S32638[86], 5}, {NULL}};
const Segment SGI71171[] = {{&S32676[44], 10}, {NULL}};
const Segment SGI71172[] = {{&S32676[44], 27}, {NULL}};
const Segment SGI71173[] = {{&S32676[30], 6}, {NULL}};
const Segment SGI71174[] = {{&S32676[119], 3}, {NULL}};
const Segment SGI71175[] = {{&S32676[85], 5}, {NULL}};
const Segment SGI71176[] = {{&S32677[38], 18}, {NULL}};
const Segment SGI71177[] = {{&S32677[31], 6}, {NULL}};
const Segment SGI71178[] = {{&S32678[44], 18}, {NULL}};
const Segment SGI71179[] = {{&S32678[44], 41}, {NULL}};
const Segment SGI71180[] = {{&S32678[30], 6}, {NULL}};
const Segment SGI71181[] = {{&S32680[44], 18}, {NULL}};
const Segment SGI71182[] = {{&S32680[44], 35}, {NULL}};
const Segment SGI71183[] = {{&S32680[30], 6}, {NULL}};
const Segment SGI71184[] = {{&S32680[127], 3}, {NULL}};
const Segment SGI71185[] = {{&S32680[93], 5}, {NULL}};
const Segment SGI71186[] = {{&S32681[42], 18}, {NULL}};
const Segment SGI71187[] = {{&S32681[30], 6}, {NULL}};
const Segment SGI71188[] = {{&S32681[125], 3}, {NULL}};
const Segment SGI71189[] = {{&S32681[91], 5}, {NULL}};
const Segment SGI71190[] = {{&S26775[44], 7}, {NULL}};
const Segment SGI71191[] = {{&S26775[44], 26}, {NULL}};
const Segment SGI71192[] = {{&S26775[30], 6}, {NULL}};
const Segment SGI71193[] = {{&S26775[118], 3}, {NULL}};
const Segment SGI71194[] = {{&S26775[84], 5}, {NULL}};
const Segment SGI71195[] = {{&S26776[44], 19}, {NULL}};
const Segment SGI71196[] = {{&S26776[44], 35}, {NULL}};
const Segment SGI71197[] = {{&S26776[30], 6}, {NULL}};
const Segment SGI71198[] = {{&S26776[127], 3}, {NULL}};
const Segment SGI71199[] = {{&S26776[93], 5}, {NULL}};
const Segment SGI71200[] = {{&S26778[44], 19}, {NULL}};
const Segment SGI71201[] = {{&S26778[44], 37}, {NULL}};
const Segment SGI71202[] = {{&S26778[30], 6}, {NULL}};
const Segment SGI71203[] = {{&S26778[69], 3}, {NULL}};
const Segment SGI71204[] = {{&S26778[95], 5}, {NULL}};
const Segment SGI71205[] = {{&S26782[44], 19}, {NULL}};
const Segment SGI71206[] = {{&S26782[44], 35}, {NULL}};
const Segment SGI71207[] = {{&S26782[30], 6}, {NULL}};
const Segment SGI71208[] = {{&S26782[127], 3}, {NULL}};
const Segment SGI71209[] = {{&S26782[93], 5}, {NULL}};
const Segment SGI71210[] = {{&S26783[44], 7}, {NULL}};
const Segment SGI71211[] = {{&S26783[44], 25}, {NULL}};
const Segment SGI71212[] = {{&S26783[30], 6}, {NULL}};
const Segment SGI71213[] = {{&S26783[57], 3}, {NULL}};
const Segment SGI71214[] = {{&S26783[83], 5}, {NULL}};
const Segment SGI71215[] = {{&S26785[44], 7}, {NULL}};
const Segment SGI71216[] = {{&S26785[44], 23}, {NULL}};
const Segment SGI71217[] = {{&S26785[30], 6}, {NULL}};
const Segment SGI71218[] = {{&S26785[115], 3}, {NULL}};
const Segment SGI71219[] = {{&S26785[81], 5}, {NULL}};
const Segment SGI71220[] = {{&S26786[44], 7}, {NULL}};
const Segment SGI71221[] = {{&S26786[44], 25}, {NULL}};
const Segment SGI71222[] = {{&S26786[30], 6}, {NULL}};
const Segment SGI71223[] = {{&S26786[57], 3}, {NULL}};
const Segment SGI71224[] = {{&S26786[83], 5}, {NULL}};
const Segment SGI71225[] = {{&S26787[44], 19}, {NULL}};
const Segment SGI71226[] = {{&S26787[44], 37}, {NULL}};
const Segment SGI71227[] = {{&S26787[30], 6}, {NULL}};
const Segment SGI71228[] = {{&S26787[69], 3}, {NULL}};
const Segment SGI71229[] = {{&S26787[95], 5}, {NULL}};
const Segment SGI71230[] = {{&S26788[38], 7}, {NULL}};
const Segment SGI71231[] = {{&S26788[31], 6}, {NULL}};
const Segment SGI71232[] = {{&S33002[44], 10}, {NULL}};
const Segment SGI71233[] = {{&S33002[44], 27}, {NULL}};
const Segment SGI71234[] = {{&S33002[30], 6}, {NULL}};
const Segment SGI71235[] = {{&S33002[119], 3}, {NULL}};
const Segment SGI71236[] = {{&S33002[85], 5}, {NULL}};
const Segment SGI71237[] = {{&S33003[44], 11}, {NULL}};
const Segment SGI71238[] = {{&S33003[44], 28}, {NULL}};
const Segment SGI71239[] = {{&S33003[30], 6}, {NULL}};
const Segment SGI71240[] = {{&S33003[120], 3}, {NULL}};
const Segment SGI71241[] = {{&S33003[86], 5}, {NULL}};
const Segment SGI71242[] = {{&S33004[38], 10}, {NULL}};
const Segment SGI71243[] = {{&S33004[31], 6}, {NULL}};
const Segment SGI71244[] = {{&S33005[42], 10}, {NULL}};
const Segment SGI71245[] = {{&S33005[30], 6}, {NULL}};
const Segment SGI71246[] = {{&S33005[117], 3}, {NULL}};
const Segment SGI71247[] = {{&S33005[83], 5}, {NULL}};
const Segment SGI71248[] = {{&S33006[44], 10}, {NULL}};
const Segment SGI71249[] = {{&S33006[44], 27}, {NULL}};
const Segment SGI71250[] = {{&S33006[30], 6}, {NULL}};
const Segment SGI71251[] = {{&S33006[119], 3}, {NULL}};
const Segment SGI71252[] = {{&S33006[85], 5}, {NULL}};
const Segment SGI71253[] = {{&S26894[44], 8}, {NULL}};
const Segment SGI71254[] = {{&S26894[44], 17}, {NULL}};
const Segment SGI71255[] = {{&S26894[30], 6}, {NULL}};
const Segment SGI71256[] = {{&S26894[109], 3}, {NULL}};
const Segment SGI71257[] = {{&S26894[75], 5}, {NULL}};
const Segment SGI71258[] = {{&S26908[44], 8}, {NULL}};
const Segment SGI71259[] = {{&S26908[44], 20}, {NULL}};
const Segment SGI71260[] = {{&S26908[30], 6}, {NULL}};
const Segment SGI71261[] = {{&S26908[118], 3}, {NULL}};
const Segment SGI71262[] = {{&S26908[84], 5}, {NULL}};
const Segment SGI71263[] = {{&S26909[38], 8}, {NULL}};
const Segment SGI71264[] = {{&S26909[31], 6}, {NULL}};
const Segment SGI71265[] = {{&S26913[44], 8}, {NULL}};
const Segment SGI71266[] = {{&S26913[44], 20}, {NULL}};
const Segment SGI71267[] = {{&S26913[30], 6}, {NULL}};
const Segment SGI71268[] = {{&S26913[112], 3}, {NULL}};
const Segment SGI71269[] = {{&S26913[78], 5}, {NULL}};
const Segment SGI71270[] = {{&S33007[44], 10}, {NULL}};
const Segment SGI71271[] = {{&S33007[44], 27}, {NULL}};
const Segment SGI71272[] = {{&S33007[30], 6}, {NULL}};
const Segment SGI71273[] = {{&S33007[88], 4}, {NULL}};
const Segment SGI71274[] = {{&S33007[88], 3}, {NULL}};
const Segment SGI71275[] = {{&S33008[38], 11}, {NULL}};
const Segment SGI71276[] = {{&S33008[31], 6}, {NULL}};
const Segment SGI71277[] = {{&S33009[44], 10}, {NULL}};
const Segment SGI71278[] = {{&S33009[44], 27}, {NULL}};
const Segment SGI71279[] = {{&S33009[30], 6}, {NULL}};
const Segment SGI71280[] = {{&S33009[88], 6}, {NULL}};
const Segment SGI71281[] = {{&S33010[38], 10}, {NULL}};
const Segment SGI71282[] = {{&S33010[31], 6}, {NULL}};
const Segment SGI71283[] = {{&S33101[42], 18}, {NULL}};
const Segment SGI71284[] = {{&S33101[30], 6}, {NULL}};
const Segment SGI71285[] = {{&S33101[125], 3}, {NULL}};
const Segment SGI71286[] = {{&S33101[91], 5}, {NULL}};
const Segment SGI71287[] = {{&S33102[44], 18}, {NULL}};
const Segment SGI71288[] = {{&S33102[44], 35}, {NULL}};
const Segment SGI71289[] = {{&S33102[30], 6}, {NULL}};
const Segment SGI71290[] = {{&S33102[127], 3}, {NULL}};
const Segment SGI71291[] = {{&S33102[93], 5}, {NULL}};
const Segment SGI71292[] = {{&S33103[38], 18}, {NULL}};
const Segment SGI71293[] = {{&S33103[31], 6}, {NULL}};
const Segment SGI71294[] = {{&S33104[44], 18}, {NULL}};
const Segment SGI71295[] = {{&S33104[44], 40}, {NULL}};
const Segment SGI71296[] = {{&S33104[30], 6}, {NULL}};
const Segment SGI71297[] = {{&S33104[132], 3}, {NULL}};
const Segment SGI71298[] = {{&S33104[98], 5}, {NULL}};
const Segment SGI71299[] = {{&S33105[44], 18}, {NULL}};
const Segment SGI71300[] = {{&S33105[44], 35}, {NULL}};
const Segment SGI71301[] = {{&S33105[30], 6}, {NULL}};
const Segment SGI71302[] = {{&S33105[96], 6}, {NULL}};
const Segment SGI71303[] = {{&S27017[44], 19}, {NULL}};
const Segment SGI71304[] = {{&S27017[44], 36}, {NULL}};
const Segment SGI71305[] = {{&S27017[30], 6}, {NULL}};
const Segment SGI71306[] = {{&S27017[128], 3}, {NULL}};
const Segment SGI71307[] = {{&S27017[94], 5}, {NULL}};
const Segment SGI71308[] = {{&S27018[44], 7}, {NULL}};
const Segment SGI71309[] = {{&S27018[44], 24}, {NULL}};
const Segment SGI71310[] = {{&S27018[30], 6}, {NULL}};
const Segment SGI71311[] = {{&S27018[116], 3}, {NULL}};
const Segment SGI71312[] = {{&S27018[82], 5}, {NULL}};
const Segment SGI71313[] = {{&S27019[44], 19}, {NULL}};
const Segment SGI71314[] = {{&S27019[44], 36}, {NULL}};
const Segment SGI71315[] = {{&S27019[30], 6}, {NULL}};
const Segment SGI71316[] = {{&S27019[128], 3}, {NULL}};
const Segment SGI71317[] = {{&S27019[94], 5}, {NULL}};
const Segment SGI71318[] = {{&S27020[44], 7}, {NULL}};
const Segment SGI71319[] = {{&S27020[44], 24}, {NULL}};
const Segment SGI71320[] = {{&S27020[30], 6}, {NULL}};
const Segment SGI71321[] = {{&S27020[116], 3}, {NULL}};
const Segment SGI71322[] = {{&S27020[82], 5}, {NULL}};
const Segment SGI71323[] = {{&S27021[38], 7}, {NULL}};
const Segment SGI71324[] = {{&S27021[31], 6}, {NULL}};
const Segment SGI71325[] = {{&S27022[44], 19}, {NULL}};
const Segment SGI71326[] = {{&S27022[44], 36}, {NULL}};
const Segment SGI71327[] = {{&S27022[30], 6}, {NULL}};
const Segment SGI71328[] = {{&S27022[128], 3}, {NULL}};
const Segment SGI71329[] = {{&S27022[94], 5}, {NULL}};
const Segment SGI71330[] = {{&S27023[44], 19}, {NULL}};
const Segment SGI71331[] = {{&S27023[44], 36}, {NULL}};
const Segment SGI71332[] = {{&S27023[30], 6}, {NULL}};
const Segment SGI71333[] = {{&S27023[128], 3}, {NULL}};
const Segment SGI71334[] = {{&S27023[94], 5}, {NULL}};
const Segment SGI71335[] = {{&S27024[44], 7}, {NULL}};
const Segment SGI71336[] = {{&S27024[44], 24}, {NULL}};
const Segment SGI71337[] = {{&S27024[30], 6}, {NULL}};
const Segment SGI71338[] = {{&S27024[116], 3}, {NULL}};
const Segment SGI71339[] = {{&S27024[82], 5}, {NULL}};
const Segment SGI71340[] = {{&S27025[44], 20}, {NULL}};
const Segment SGI71341[] = {{&S27025[44], 37}, {NULL}};
const Segment SGI71342[] = {{&S27025[30], 6}, {NULL}};
const Segment SGI71343[] = {{&S27025[129], 3}, {NULL}};
const Segment SGI71344[] = {{&S27025[95], 5}, {NULL}};
const Segment SGI71345[] = {{&S27027[38], 7}, {NULL}};
const Segment SGI71346[] = {{&S27027[31], 6}, {NULL}};
const Segment SGI71347[] = {{&S27028[44], 7}, {NULL}};
const Segment SGI71348[] = {{&S27028[44], 24}, {NULL}};
const Segment SGI71349[] = {{&S27028[30], 6}, {NULL}};
const Segment SGI71350[] = {{&S27028[85], 6}, {NULL}};
const Segment SGI71351[] = {{&S27029[42], 19}, {NULL}};
const Segment SGI71352[] = {{&S27029[30], 6}, {NULL}};
const Segment SGI71353[] = {{&S27029[126], 3}, {NULL}};
const Segment SGI71354[] = {{&S27029[92], 5}, {NULL}};
const Segment SGI71355[] = {{&S27030[44], 7}, {NULL}};
const Segment SGI71356[] = {{&S27030[44], 24}, {NULL}};
const Segment SGI71357[] = {{&S27030[30], 6}, {NULL}};
const Segment SGI71358[] = {{&S27030[116], 3}, {NULL}};
const Segment SGI71359[] = {{&S27030[82], 5}, {NULL}};
const Segment SGI71360[] = {{&S27141[44], 10}, {NULL}};
const Segment SGI71361[] = {{&S27141[44], 21}, {NULL}};
const Segment SGI71362[] = {{&S27141[30], 6}, {NULL}};
const Segment SGI71363[] = {{&S27141[113], 3}, {NULL}};
const Segment SGI71364[] = {{&S27141[79], 5}, {NULL}};
const Segment SGI71365[] = {{&S27150[44], 10}, {NULL}};
const Segment SGI71366[] = {{&S27150[44], 27}, {NULL}};
const Segment SGI71367[] = {{&S27150[30], 6}, {NULL}};
const Segment SGI71368[] = {{&S27150[119], 3}, {NULL}};
const Segment SGI71369[] = {{&S27150[85], 5}, {NULL}};
const Segment SGI71370[] = {{&S27734[44], 17}, {NULL}};
const Segment SGI71371[] = {{&S27734[44], 28}, {NULL}};
const Segment SGI71372[] = {{&S27734[30], 6}, {NULL}};
const Segment SGI71373[] = {{&S27734[120], 3}, {NULL}};
const Segment SGI71374[] = {{&S27734[86], 5}, {NULL}};
const Segment SGI71375[] = {{&S27735[44], 12}, {NULL}};
const Segment SGI71376[] = {{&S27735[44], 24}, {NULL}};
const Segment SGI71377[] = {{&S27735[30], 6}, {NULL}};
const Segment SGI71378[] = {{&S27735[116], 3}, {NULL}};
const Segment SGI71379[] = {{&S27735[82], 5}, {NULL}};
const Segment SGI71380[] = {{&S27736[44], 12}, {NULL}};
const Segment SGI71381[] = {{&S27736[44], 23}, {NULL}};
const Segment SGI71382[] = {{&S27736[30], 6}, {NULL}};
const Segment SGI71383[] = {{&S27736[115], 3}, {NULL}};
const Segment SGI71384[] = {{&S27736[81], 5}, {NULL}};
const Segment SGI71385[] = {{&S27738[44], 12}, {NULL}};
const Segment SGI71386[] = {{&S27738[44], 23}, {NULL}};
const Segment SGI71387[] = {{&S27738[30], 6}, {NULL}};
const Segment SGI71388[] = {{&S27738[115], 3}, {NULL}};
const Segment SGI71389[] = {{&S27738[81], 5}, {NULL}};
const Segment SGI71390[] = {{&S27744[44], 12}, {NULL}};
const Segment SGI71391[] = {{&S27744[44], 35}, {NULL}};
const Segment SGI71392[] = {{&S27744[30], 6}, {NULL}};
const Segment SGI71393[] = {{&S27744[127], 3}, {NULL}};
const Segment SGI71394[] = {{&S27744[93], 5}, {NULL}};
const Segment SGI71395[] = {{&S27745[38], 17}, {NULL}};
const Segment SGI71396[] = {{&S27745[31], 6}, {NULL}};
const Segment SGI71397[] = {{&S27746[44], 12}, {NULL}};
const Segment SGI71398[] = {{&S27746[44], 24}, {NULL}};
const Segment SGI71399[] = {{&S27746[30], 6}, {NULL}};
const Segment SGI71400[] = {{&S27746[116], 3}, {NULL}};
const Segment SGI71401[] = {{&S27746[82], 5}, {NULL}};
const Segment SGI71402[] = {{&S33947[38], 14}, {NULL}};
const Segment SGI71403[] = {{&S33947[31], 6}, {NULL}};
const Segment SGI71404[] = {{&S27747[44], 12}, {NULL}};
const Segment SGI71405[] = {{&S27747[44], 36}, {NULL}};
const Segment SGI71406[] = {{&S27747[30], 6}, {NULL}};
const Segment SGI71407[] = {{&S27747[128], 3}, {NULL}};
const Segment SGI71408[] = {{&S27747[94], 5}, {NULL}};
const Segment SGI71409[] = {{&S33948[44], 14}, {NULL}};
const Segment SGI71410[] = {{&S33948[44], 31}, {NULL}};
const Segment SGI71411[] = {{&S33948[30], 6}, {NULL}};
const Segment SGI71412[] = {{&S27748[44], 12}, {NULL}};
const Segment SGI71413[] = {{&S33948[123], 3}, {NULL}};
const Segment SGI71414[] = {{&S27748[44], 36}, {NULL}};
const Segment SGI71415[] = {{&S27748[30], 6}, {NULL}};
const Segment SGI71416[] = {{&S33948[89], 5}, {NULL}};
const Segment SGI71417[] = {{&S27748[128], 3}, {NULL}};
const Segment SGI71418[] = {{&S27748[94], 5}, {NULL}};
const Segment SGI71419[] = {{&S27751[38], 12}, {NULL}};
const Segment SGI71420[] = {{&S27751[31], 6}, {NULL}};
const Segment SGI71421[] = {{&S33951[42], 14}, {NULL}};
const Segment SGI71422[] = {{&S33951[30], 6}, {NULL}};
const Segment SGI71423[] = {{&S33951[121], 3}, {NULL}};
const Segment SGI71424[] = {{&S33951[87], 5}, {NULL}};
const Segment SGI71425[] = {{&S33952[44], 14}, {NULL}};
const Segment SGI71426[] = {{&S33952[44], 44}, {NULL}};
const Segment SGI71427[] = {{&S33952[30], 6}, {NULL}};
const Segment SGI71428[] = {{&S33952[136], 3}, {NULL}};
const Segment SGI71429[] = {{&S33952[102], 5}, {NULL}};
const Segment SGI71430[] = {{&S33954[44], 14}, {NULL}};
const Segment SGI71431[] = {{&S33954[44], 44}, {NULL}};
const Segment SGI71432[] = {{&S27756[44], 17}, {NULL}};
const Segment SGI71433[] = {{&S33954[30], 6}, {NULL}};
const Segment SGI71434[] = {{&S27756[44], 28}, {NULL}};
const Segment SGI71435[] = {{&S27756[30], 6}, {NULL}};
const Segment SGI71436[] = {{&S33954[136], 3}, {NULL}};
const Segment SGI71437[] = {{&S33954[102], 5}, {NULL}};
const Segment SGI71438[] = {{&S27756[86], 6}, {NULL}};
const Segment SGI71439[] = {{&S33955[44], 14}, {NULL}};
const Segment SGI71440[] = {{&S33955[44], 32}, {NULL}};
const Segment SGI71441[] = {{&S33955[30], 6}, {NULL}};
const Segment SGI71442[] = {{&S33955[124], 3}, {NULL}};
const Segment SGI71443[] = {{&S33955[90], 5}, {NULL}};
const Segment SGI71444[] = {{&S33957[44], 14}, {NULL}};
const Segment SGI71445[] = {{&S27760[44], 12}, {NULL}};
const Segment SGI71446[] = {{&S33957[44], 31}, {NULL}};
const Segment SGI71447[] = {{&S27760[44], 32}, {NULL}};
const Segment SGI71448[] = {{&S33957[30], 6}, {NULL}};
const Segment SGI71449[] = {{&S27760[30], 6}, {NULL}};
const Segment SGI71450[] = {{&S33957[125], 3}, {NULL}};
const Segment SGI71451[] = {{&S27760[124], 3}, {NULL}};
const Segment SGI71452[] = {{&S33957[89], 7}, {NULL}};
const Segment SGI71453[] = {{&S27760[90], 5}, {NULL}};
const Segment SGI71454[] = {{&S33957[145], 8}, {NULL}};
const Segment SGI71455[] = {{&S33958[44], 14}, {NULL}};
const Segment SGI71456[] = {{&S33958[44], 35}, {NULL}};
const Segment SGI71457[] = {{&S33958[30], 6}, {NULL}};
const Segment SGI71458[] = {{&S33958[127], 3}, {NULL}};
const Segment SGI71459[] = {{&S33958[93], 5}, {NULL}};
const Segment SGI71460[] = {{&S33959[44], 14}, {NULL}};
const Segment SGI71461[] = {{&S33959[44], 35}, {NULL}};
const Segment SGI71462[] = {{&S33959[30], 6}, {NULL}};
const Segment SGI71463[] = {{&S33959[127], 3}, {NULL}};
const Segment SGI71464[] = {{&S33959[93], 5}, {NULL}};
const Segment SGI71465[] = {{&S27764[44], 17}, {NULL}};
const Segment SGI71466[] = {{&S27764[44], 28}, {NULL}};
const Segment SGI71467[] = {{&S27764[30], 6}, {NULL}};
const Segment SGI71468[] = {{&S27764[120], 3}, {NULL}};
const Segment SGI71469[] = {{&S27764[86], 5}, {NULL}};
const Segment SGI71470[] = {{&S27766[44], 17}, {NULL}};
const Segment SGI71471[] = {{&S27766[44], 29}, {NULL}};
const Segment SGI71472[] = {{&S27766[30], 6}, {NULL}};
const Segment SGI71473[] = {{&S27766[121], 3}, {NULL}};
const Segment SGI71474[] = {{&S27766[87], 5}, {NULL}};
const Segment SGI71475[] = {{&S27767[44], 12}, {NULL}};
const Segment SGI71476[] = {{&S27767[44], 34}, {NULL}};
const Segment SGI71477[] = {{&S27767[30], 6}, {NULL}};
const Segment SGI71478[] = {{&S27767[126], 3}, {NULL}};
const Segment SGI71479[] = {{&S27767[92], 5}, {NULL}};
const Segment SGI71480[] = {{&S27768[41], 16}, {NULL}};
const Segment SGI71481[] = {{&S27768[104], 3}, {NULL}};
const Segment SGI71482[] = {{&S27768[71], 5}, {NULL}};
const Segment SGI71483[] = {{&S27782[44], 12}, {NULL}};
const Segment SGI71484[] = {{&S27782[44], 34}, {NULL}};
const Segment SGI71485[] = {{&S27782[30], 6}, {NULL}};
const Segment SGI71486[] = {{&S27782[126], 3}, {NULL}};
const Segment SGI71487[] = {{&S27782[92], 5}, {NULL}};
const Segment SGI71488[] = {{&S27783[44], 12}, {NULL}};
const Segment SGI71489[] = {{&S27783[44], 23}, {NULL}};
const Segment SGI71490[] = {{&S27783[30], 6}, {NULL}};
const Segment SGI71491[] = {{&S27783[84], 4}, {NULL}};
const Segment SGI71492[] = {{&S27783[84], 3}, {NULL}};
const Segment SGI71493[] = {{&S33988[44], 9}, {NULL}};
const Segment SGI71494[] = {{&S33988[44], 21}, {NULL}};
const Segment SGI71495[] = {{&S33988[30], 6}, {NULL}};
const Segment SGI71496[] = {{&S33988[113], 3}, {NULL}};
const Segment SGI71497[] = {{&S33988[79], 5}, {NULL}};
const Segment SGI71498[] = {{&S27787[44], 12}, {NULL}};
const Segment SGI71499[] = {{&S27787[44], 38}, {NULL}};
const Segment SGI71500[] = {{&S27787[30], 6}, {NULL}};
const Segment SGI71501[] = {{&S27787[148], 3}, {NULL}};
const Segment SGI71502[] = {{&S27787[114], 5}, {NULL}};
const Segment SGI71503[] = {{&S33989[38], 9}, {NULL}};
const Segment SGI71504[] = {{&S33989[31], 6}, {NULL}};
const Segment SGI71505[] = {{&S27789[44], 12}, {NULL}};
const Segment SGI71506[] = {{&S27789[44], 26}, {NULL}};
const Segment SGI71507[] = {{&S27789[30], 6}, {NULL}};
const Segment SGI71508[] = {{&S27789[118], 3}, {NULL}};
const Segment SGI71509[] = {{&S27789[84], 5}, {NULL}};
const Segment SGI71510[] = {{&S27790[44], 12}, {NULL}};
const Segment SGI71511[] = {{&S27790[44], 23}, {NULL}};
const Segment SGI71512[] = {{&S27790[30], 6}, {NULL}};
const Segment SGI71513[] = {{&S27790[115], 3}, {NULL}};
const Segment SGI71514[] = {{&S27790[81], 5}, {NULL}};
const Segment SGI71515[] = {{&S33994[44], 8}, {NULL}};
const Segment SGI71516[] = {{&S33994[44], 20}, {NULL}};
const Segment SGI71517[] = {{&S33994[30], 6}, {NULL}};
const Segment SGI71518[] = {{&S27791[44], 12}, {NULL}};
const Segment SGI71519[] = {{&S27791[44], 23}, {NULL}};
const Segment SGI71520[] = {{&S27791[30], 6}, {NULL}};
const Segment SGI71521[] = {{&S33994[112], 3}, {NULL}};
const Segment SGI71522[] = {{&S33994[78], 5}, {NULL}};
const Segment SGI71523[] = {{&S27791[84], 6}, {NULL}};
const Segment SGI71524[] = {{&S27792[44], 12}, {NULL}};
const Segment SGI71525[] = {{&S27792[44], 23}, {NULL}};
const Segment SGI71526[] = {{&S27792[30], 6}, {NULL}};
const Segment SGI71527[] = {{&S27792[115], 3}, {NULL}};
const Segment SGI71528[] = {{&S27792[81], 5}, {NULL}};
const Segment SGI71529[] = {{&S33996[44], 8}, {NULL}};
const Segment SGI71530[] = {{&S33996[44], 20}, {NULL}};
const Segment SGI71531[] = {{&S33996[30], 6}, {NULL}};
const Segment SGI71532[] = {{&S33996[112], 3}, {NULL}};
const Segment SGI71533[] = {{&S33996[78], 5}, {NULL}};
const Segment SGI71534[] = {{&S27794[44], 12}, {NULL}};
const Segment SGI71535[] = {{&S27794[44], 26}, {NULL}};
const Segment SGI71536[] = {{&S27794[30], 6}, {NULL}};
const Segment SGI71537[] = {{&S27794[118], 3}, {NULL}};
const Segment SGI71538[] = {{&S27794[84], 5}, {NULL}};
const Segment SGI71539[] = {{&S27795[44], 12}, {NULL}};
const Segment SGI71540[] = {{&S27795[44], 25}, {NULL}};
const Segment SGI71541[] = {{&S27795[30], 6}, {NULL}};
const Segment SGI71542[] = {{&S27795[117], 3}, {NULL}};
const Segment SGI71543[] = {{&S33999[43], 9}, {NULL}};
const Segment SGI71544[] = {{&S27795[83], 5}, {NULL}};
const Segment SGI71545[] = {{&S33999[43], 21}, {NULL}};
const Segment SGI71546[] = {{&S33999[30], 5}, {NULL}};
const Segment SGI71547[] = {{&S33999[112], 3}, {NULL}};
const Segment SGI71548[] = {{&S33999[78], 5}, {NULL}};
const Segment SGI71549[] = {{&S34000[43], 9}, {NULL}};
const Segment SGI71550[] = {{&S34000[43], 21}, {NULL}};
const Segment SGI71551[] = {{&S34000[112], 3}, {NULL}};
const Segment SGI71552[] = {{&S34000[78], 5}, {NULL}};
const Segment SGI71553[] = {{&S34001[44], 9}, {NULL}};
const Segment SGI71554[] = {{&S34001[44], 21}, {NULL}};
const Segment SGI71555[] = {{&S34001[30], 6}, {NULL}};
const Segment SGI71556[] = {{&S34001[113], 3}, {NULL}};
const Segment SGI71557[] = {{&S34001[79], 5}, {NULL}};
const Segment SGI71558[] = {{&S27799[44], 12}, {NULL}};
const Segment SGI71559[] = {{&S27799[44], 31}, {NULL}};
const Segment SGI71560[] = {{&S27799[30], 6}, {NULL}};
const Segment SGI71561[] = {{&S27799[123], 3}, {NULL}};
const Segment SGI71562[] = {{&S27799[89], 5}, {NULL}};
const Segment SGI71563[] = {{&S27800[44], 12}, {NULL}};
const Segment SGI71564[] = {{&S34003[44], 9}, {NULL}};
const Segment SGI71565[] = {{&S27800[44], 30}, {NULL}};
const Segment SGI71566[] = {{&S34003[44], 21}, {NULL}};
const Segment SGI71567[] = {{&S27800[30], 6}, {NULL}};
const Segment SGI71568[] = {{&S34003[30], 6}, {NULL}};
const Segment SGI71569[] = {{&S27800[122], 3}, {NULL}};
const Segment SGI71570[] = {{&S27800[88], 5}, {NULL}};
const Segment SGI71571[] = {{&S34003[79], 5}, {NULL}};
const Segment SGI71572[] = {{&S27396[42], 10}, {NULL}};
const Segment SGI71573[] = {{&S27396[42], 22}, {NULL}};
const Segment SGI71574[] = {{&S27396[30], 4}, {NULL}};
const Segment SGI71575[] = {{&S27396[113], 3}, {NULL}};
const Segment SGI71576[] = {{&S27396[78], 6}, {NULL}};
const Segment SGI71577[] = {{&S27397[36], 10}, {NULL}};
const Segment SGI71578[] = {{&S27397[31], 4}, {NULL}};
const Segment SGI71579[] = {{&S27398[42], 10}, {NULL}};
const Segment SGI71580[] = {{&S27398[42], 22}, {NULL}};
const Segment SGI71581[] = {{&S27398[30], 4}, {NULL}};
const Segment SGI71582[] = {{&S27398[113], 3}, {NULL}};
const Segment SGI71583[] = {{&S27398[78], 6}, {NULL}};
const Segment SGI71584[] = {{&S27399[42], 10}, {NULL}};
const Segment SGI71585[] = {{&S27399[42], 22}, {NULL}};
const Segment SGI71586[] = {{&S27399[30], 4}, {NULL}};
const Segment SGI71587[] = {{&S27399[113], 3}, {NULL}};
const Segment SGI71588[] = {{&S27399[78], 6}, {NULL}};
const Segment SGI71589[] = {{&S27400[36], 10}, {NULL}};
const Segment SGI71590[] = {{&S27400[31], 4}, {NULL}};
const Segment SGI71591[] = {{&S27401[42], 10}, {NULL}};
const Segment SGI71592[] = {{&S27401[42], 22}, {NULL}};
const Segment SGI71593[] = {{&S27401[30], 4}, {NULL}};
const Segment SGI71594[] = {{&S27401[113], 3}, {NULL}};
const Segment SGI71595[] = {{&S27401[78], 6}, {NULL}};
const Segment SGI71596[] = {{&S27402[42], 28}, {NULL}};
const Segment SGI71597[] = {{&S27402[42], 40}, {NULL}};
const Segment SGI71598[] = {{&S27402[30], 4}, {NULL}};
const Segment SGI71599[] = {{&S27402[131], 3}, {NULL}};
const Segment SGI71600[] = {{&S27402[96], 6}, {NULL}};
const Segment SGI71601[] = {{&S27403[40], 10}, {NULL}};
const Segment SGI71602[] = {{&S27403[30], 4}, {NULL}};
const Segment SGI71603[] = {{&S27403[111], 3}, {NULL}};
const Segment SGI71604[] = {{&S27403[76], 6}, {NULL}};
const Segment SGI71605[] = {{&S27404[42], 10}, {NULL}};
const Segment SGI71606[] = {{&S27404[42], 22}, {NULL}};
const Segment SGI71607[] = {{&S27404[30], 4}, {NULL}};
const Segment SGI71608[] = {{&S27404[123], 3}, {NULL}};
const Segment SGI71609[] = {{&S27404[88], 6}, {NULL}};
const Segment SGI71610[] = {{&S27404[143], 8}, {NULL}};
const Segment SGI71611[] = {{&S27405[42], 10}, {NULL}};
const Segment SGI71612[] = {{&S27405[42], 22}, {NULL}};
const Segment SGI71613[] = {{&S27405[30], 4}, {NULL}};
const Segment SGI71614[] = {{&S27405[114], 3}, {NULL}};
const Segment SGI71615[] = {{&S27405[78], 7}, {NULL}};
const Segment SGI71616[] = {{&S27405[134], 8}, {NULL}};
const Segment SGI71617[] = {{&S27406[42], 10}, {NULL}};
const Segment SGI71618[] = {{&S27406[42], 22}, {NULL}};
const Segment SGI71619[] = {{&S27406[30], 4}, {NULL}};
const Segment SGI71620[] = {{&S27406[81], 6}, {NULL}};
const Segment SGI71621[] = {{&S27407[42], 10}, {NULL}};
const Segment SGI71622[] = {{&S27407[42], 22}, {NULL}};
const Segment SGI71623[] = {{&S27407[30], 4}, {NULL}};
const Segment SGI71624[] = {{&S27407[81], 4}, {NULL}};
const Segment SGI71625[] = {{&S27407[81], 3}, {NULL}};
const Segment SGI71626[] = {{&S27408[42], 10}, {NULL}};
const Segment SGI71627[] = {{&S27408[42], 19}, {NULL}};
const Segment SGI71628[] = {{&S27408[30], 4}, {NULL}};
const Segment SGI71629[] = {{&S27408[109], 3}, {NULL}};
const Segment SGI71630[] = {{&S27408[75], 5}, {NULL}};
const Segment SGI71631[] = {{&S27409[40], 10}, {NULL}};
const Segment SGI71632[] = {{&S27409[30], 4}, {NULL}};
const Segment SGI71633[] = {{&S27409[111], 3}, {NULL}};
const Segment SGI71634[] = {{&S27409[76], 6}, {NULL}};
const Segment SGI71635[] = {{&S27411[43], 10}, {NULL}};
const Segment SGI71636[] = {{&S27411[43], 28}, {NULL}};
const Segment SGI71637[] = {{&S27411[31], 4}, {NULL}};
const Segment SGI71638[] = {{&S27412[32], 10}, {NULL}};
const Segment SGI71639[] = {{&S27412[32], 22}, {NULL}};
const Segment SGI71640[] = {{&S27413[42], 10}, {NULL}};
const Segment SGI71641[] = {{&S27413[42], 22}, {&S27413[259], 7}, {NULL}};
const Segment SGI71642[] = {{&S27413[30], 4}, {NULL}};
const Segment SGI71643[] = {{&S27413[113], 3}, {NULL}};
const Segment SGI71644[] = {{&S27413[78], 6}, {NULL}};
const Segment SGI71645[] = {{&S27414[42], 10}, {NULL}};
const Segment SGI71646[] = {{&S27414[42], 28}, {NULL}};
const Segment SGI71647[] = {{&S27414[30], 4}, {NULL}};
const Segment SGI71648[] = {{&S27415[53], 18}, {NULL}};
const Segment SGI71649[] = {{&S27415[48], 4}, {NULL}};
const Segment SGI71650[] = {{&S27416[43], 10}, {NULL}};
const Segment SGI71651[] = {{&S27416[43], 28}, {NULL}};
const Segment SGI71652[] = {{&S27416[31], 4}, {NULL}};
const Segment SGI71653[] = {{&S27417[42], 10}, {NULL}};
const Segment SGI71654[] = {{&S27417[42], 22}, {&S27417[206], 7}, {NULL}};
const Segment SGI71655[] = {{&S27417[30], 4}, {NULL}};
const Segment SGI71656[] = {{&S27417[113], 3}, {NULL}};
const Segment SGI71657[] = {{&S27417[78], 6}, {NULL}};
const Segment SGI71658[] = {{&S27418[97], 29}, {NULL}};
const Segment SGI71659[] = {{&S27932[52], 6}, {NULL}};
const Segment SGI71660[] = {{&S27932[52], 18}, {NULL}};
const Segment SGI71661[] = {{&S27932[30], 14}, {NULL}};
const Segment SGI71662[] = {{&S27932[118], 3}, {NULL}};
const Segment SGI71663[] = {{&S27932[84], 5}, {NULL}};
const Segment SGI71664[] = {{&S27933[44], 18}, {NULL}};
const Segment SGI71665[] = {{&S27933[44], 36}, {NULL}};
const Segment SGI71666[] = {{&S27933[30], 6}, {NULL}};
const Segment SGI71667[] = {{&S27933[128], 3}, {NULL}};
const Segment SGI71668[] = {{&S27933[94], 5}, {NULL}};
const Segment SGI71669[] = {{&S27934[44], 6}, {NULL}};
const Segment SGI71670[] = {{&S27934[44], 24}, {NULL}};
const Segment SGI71671[] = {{&S27934[30], 6}, {NULL}};
const Segment SGI71672[] = {{&S27934[116], 3}, {NULL}};
const Segment SGI71673[] = {{&S27934[82], 5}, {NULL}};
const Segment SGI71674[] = {{&S27935[44], 6}, {NULL}};
const Segment SGI71675[] = {{&S27935[44], 24}, {NULL}};
const Segment SGI71676[] = {{&S27935[30], 6}, {NULL}};
const Segment SGI71677[] = {{&S27935[116], 3}, {NULL}};
const Segment SGI71678[] = {{&S27935[82], 5}, {NULL}};
const Segment SGI71679[] = {{&S27938[38], 6}, {NULL}};
const Segment SGI71680[] = {{&S27938[31], 6}, {NULL}};
const Segment SGI71681[] = {{&S27940[52], 6}, {NULL}};
const Segment SGI71682[] = {{&S27940[52], 18}, {NULL}};
const Segment SGI71683[] = {{&S27940[30], 14}, {NULL}};
const Segment SGI71684[] = {{&S27940[118], 3}, {NULL}};
const Segment SGI71685[] = {{&S27940[84], 5}, {NULL}};
const Segment SGI71686[] = {{&S27942[57], 6}, {NULL}};
const Segment SGI71687[] = {{&S27942[57], 18}, {NULL}};
const Segment SGI71688[] = {{&S27942[30], 19}, {NULL}};
const Segment SGI71689[] = {{&S27942[123], 3}, {NULL}};
const Segment SGI71690[] = {{&S27942[89], 5}, {NULL}};
const Segment SGI71691[] = {{&S27944[55], 6}, {NULL}};
const Segment SGI71692[] = {{&S27944[55], 18}, {NULL}};
const Segment SGI71693[] = {{&S27944[30], 17}, {NULL}};
const Segment SGI71694[] = {{&S27944[121], 3}, {NULL}};
const Segment SGI71695[] = {{&S27944[87], 5}, {NULL}};
const Segment SGI71696[] = {{&S27419[42], 10}, {NULL}};
const Segment SGI71697[] = {{&S27419[42], 22}, {NULL}};
const Segment SGI71698[] = {{&S27419[30], 4}, {NULL}};
const Segment SGI71699[] = {{&S27419[112], 3}, {NULL}};
const Segment SGI71700[] = {{&S27419[78], 5}, {NULL}};
const Segment SGI71701[] = {{&S27420[44], 10}, {NULL}};
const Segment SGI71702[] = {{&S27420[44], 28}, {NULL}};
const Segment SGI71703[] = {{&S27420[32], 4}, {NULL}};
const Segment SGI71704[] = {{&S27421[42], 10}, {NULL}};
const Segment SGI71705[] = {{&S27421[42], 22}, {NULL}};
const Segment SGI71706[] = {{&S27421[30], 4}, {NULL}};
const Segment SGI71707[] = {{&S27421[78], 6}, {NULL}};
const Segment SGI71708[] = {{&S27422[63], 10}, {NULL}};
const Segment SGI71709[] = {{&S27422[63], 22}, {NULL}};
const Segment SGI71710[] = {{&S27422[51], 4}, {NULL}};
const Segment SGI71711[] = {{&S27423[42], 9}, {NULL}};
const Segment SGI71712[] = {{&S27423[30], 4}, {NULL}};
const Segment SGI71713[] = {{&S27426[42], 10}, {NULL}};
const Segment SGI71714[] = {{&S27426[42], 22}, {NULL}};
const Segment SGI71715[] = {{&S27426[30], 4}, {NULL}};
const Segment SGI71716[] = {{&S27426[113], 3}, {NULL}};
const Segment SGI71717[] = {{&S27426[78], 6}, {NULL}};
const Segment SGI71718[] = {{&S27992[38], 7}, {NULL}};
const Segment SGI71719[] = {{&S27992[31], 6}, {NULL}};
const Segment SGI71720[] = {{&S27993[44], 7}, {NULL}};
const Segment SGI71721[] = {{&S27993[44], 30}, {NULL}};
const Segment SGI71722[] = {{&S27993[30], 6}, {NULL}};
const Segment SGI71723[] = {{&S27993[122], 3}, {NULL}};
const Segment SGI71724[] = {{&S27993[88], 5}, {NULL}};
const Segment SGI71725[] = {{&S34413[44], 18}, {NULL}};
const Segment SGI71726[] = {{&S34413[44], 30}, {NULL}};
const Segment SGI71727[] = {{&S34413[30], 6}, {NULL}};
const Segment SGI71728[] = {{&S34413[122], 3}, {NULL}};
const Segment SGI71729[] = {{&S34413[88], 5}, {NULL}};
const Segment SGI71730[] = {{&S34549[44], 14}, {NULL}};
const Segment SGI71731[] = {{&S34549[44], 26}, {NULL}};
const Segment SGI71732[] = {{&S34549[30], 6}, {NULL}};
const Segment SGI71733[] = {{&S34549[118], 3}, {NULL}};
const Segment SGI71734[] = {{&S34549[84], 5}, {NULL}};
const Segment SGI71735[] = {{&S17978[44], 11}, {NULL}};
const Segment SGI71736[] = {{&S17978[44], 23}, {NULL}};
const Segment SGI71737[] = {{&S17978[30], 6}, {NULL}};
const Segment SGI71738[] = {{&S17978[31], 3}, {NULL}};
const Segment SGI71739[] = {{&S17978[81], 6}, {NULL}};
const Segment SGI71740[] = {{&S17979[44], 11}, {NULL}};
const Segment SGI71741[] = {{&S17979[44], 23}, {NULL}};
const Segment SGI71742[] = {{&S17979[30], 6}, {NULL}};
const Segment SGI71743[] = {{&S17979[31], 3}, {NULL}};
const Segment SGI71744[] = {{&S17979[81], 6}, {NULL}};
const Segment SGI71745[] = {{&S17980[44], 11}, {NULL}};
const Segment SGI71746[] = {{&S17980[44], 23}, {NULL}};
const Segment SGI71747[] = {{&S17980[30], 6}, {NULL}};
const Segment SGI71748[] = {{&S17980[31], 3}, {NULL}};
const Segment SGI71749[] = {{&S17980[81], 6}, {NULL}};
const Segment SGI71750[] = {{&S17981[44], 22}, {NULL}};
const Segment SGI71751[] = {{&S17981[44], 34}, {NULL}};
const Segment SGI71752[] = {{&S17981[30], 6}, {NULL}};
const Segment SGI71753[] = {{&S17981[31], 3}, {NULL}};
const Segment SGI71754[] = {{&S17981[92], 6}, {NULL}};
const Segment SGI71755[] = {{&S17982[44], 22}, {NULL}};
const Segment SGI71756[] = {{&S17982[44], 34}, {NULL}};
const Segment SGI71757[] = {{&S17982[30], 6}, {NULL}};
const Segment SGI71758[] = {{&S17982[31], 3}, {NULL}};
const Segment SGI71759[] = {{&S17982[92], 6}, {NULL}};
const Segment SGI71760[] = {{&S17983[44], 11}, {NULL}};
const Segment SGI71761[] = {{&S17983[44], 23}, {NULL}};
const Segment SGI71762[] = {{&S17983[30], 6}, {NULL}};
const Segment SGI71763[] = {{&S17983[81], 6}, {NULL}};
const Segment SGI71764[] = {{&S17984[44], 21}, {NULL}};
const Segment SGI71765[] = {{&S17984[44], 33}, {NULL}};
const Segment SGI71766[] = {{&S17984[30], 6}, {NULL}};
const Segment SGI71767[] = {{&S17984[31], 3}, {NULL}};
const Segment SGI71768[] = {{&S17984[91], 6}, {NULL}};
const Segment SGI71769[] = {{&S17985[38], 11}, {NULL}};
const Segment SGI71770[] = {{&S17985[31], 6}, {NULL}};
const Segment SGI71771[] = {{&S17986[44], 22}, {NULL}};
const Segment SGI71772[] = {{&S17986[44], 34}, {NULL}};
const Segment SGI71773[] = {{&S17986[30], 6}, {NULL}};
const Segment SGI71774[] = {{&S17986[31], 3}, {NULL}};
const Segment SGI71775[] = {{&S17986[92], 6}, {NULL}};
const Segment SGI71776[] = {{&S17987[44], 22}, {NULL}};
const Segment SGI71777[] = {{&S17987[44], 34}, {NULL}};
const Segment SGI71778[] = {{&S17987[30], 6}, {NULL}};
const Segment SGI71779[] = {{&S17987[31], 3}, {NULL}};
const Segment SGI71780[] = {{&S17987[92], 6}, {NULL}};
const Segment SGI71781[] = {{&S17988[38], 11}, {NULL}};
const Segment SGI71782[] = {{&S17988[31], 6}, {NULL}};
const Segment SGI71783[] = {{&S17989[44], 22}, {NULL}};
const Segment SGI71784[] = {{&S17989[44], 34}, {NULL}};
const Segment SGI71785[] = {{&S17989[30], 6}, {NULL}};
const Segment SGI71786[] = {{&S17989[31], 3}, {NULL}};
const Segment SGI71787[] = {{&S17989[92], 6}, {NULL}};
const Segment SGI71788[] = {{&S17990[44], 22}, {NULL}};
const Segment SGI71789[] = {{&S17990[44], 34}, {NULL}};
const Segment SGI71790[] = {{&S17990[30], 6}, {NULL}};
const Segment SGI71791[] = {{&S17990[31], 3}, {NULL}};
const Segment SGI71792[] = {{&S17990[92], 6}, {NULL}};
const Segment SGI71793[] = {{&S17991[44], 7}, {NULL}};
const Segment SGI71794[] = {{&S17991[44], 19}, {NULL}};
const Segment SGI71795[] = {{&S17991[30], 6}, {NULL}};
const Segment SGI71796[] = {{&S17991[31], 3}, {NULL}};
const Segment SGI71797[] = {{&S17991[77], 6}, {NULL}};
const Segment SGI71798[] = {{&S17992[44], 22}, {NULL}};
const Segment SGI71799[] = {{&S17992[44], 34}, {NULL}};
const Segment SGI71800[] = {{&S17992[30], 6}, {NULL}};
const Segment SGI71801[] = {{&S17992[31], 3}, {NULL}};
const Segment SGI71802[] = {{&S17992[92], 6}, {NULL}};
const Segment SGI71803[] = {{&S17993[44], 22}, {NULL}};
const Segment SGI71804[] = {{&S17993[44], 34}, {NULL}};
const Segment SGI71805[] = {{&S17993[30], 6}, {NULL}};
const Segment SGI71806[] = {{&S17993[31], 3}, {NULL}};
const Segment SGI71807[] = {{&S17993[92], 6}, {NULL}};
const Segment SGI71808[] = {{&S17994[44], 22}, {NULL}};
const Segment SGI71809[] = {{&S17994[44], 34}, {NULL}};
const Segment SGI71810[] = {{&S17994[30], 6}, {NULL}};
const Segment SGI71811[] = {{&S17994[31], 3}, {NULL}};
const Segment SGI71812[] = {{&S17994[92], 6}, {NULL}};
const Segment SGI71813[] = {{&S17995[44], 11}, {NULL}};
const Segment SGI71814[] = {{&S17995[44], 23}, {NULL}};
const Segment SGI71815[] = {{&S17995[30], 6}, {NULL}};
const Segment SGI71816[] = {{&S17995[31], 3}, {NULL}};
const Segment SGI71817[] = {{&S17995[81], 6}, {NULL}};
const Segment SGI71818[] = {{&S17997[44], 22}, {NULL}};
const Segment SGI71819[] = {{&S17997[44], 34}, {NULL}};
const Segment SGI71820[] = {{&S17997[30], 6}, {NULL}};
const Segment SGI71821[] = {{&S17997[31], 3}, {NULL}};
const Segment SGI71822[] = {{&S17997[92], 6}, {NULL}};
const Segment SGI71823[] = {{&S17998[44], 22}, {NULL}};
const Segment SGI71824[] = {{&S17998[44], 34}, {NULL}};
const Segment SGI71825[] = {{&S17998[30], 6}, {NULL}};
const Segment SGI71826[] = {{&S17998[31], 3}, {NULL}};
const Segment SGI71827[] = {{&S17998[92], 6}, {NULL}};
const Segment SGI71828[] = {{&S17999[44], 22}, {NULL}};
const Segment SGI71829[] = {{&S17999[44], 34}, {NULL}};
const Segment SGI71830[] = {{&S17999[30], 6}, {NULL}};
const Segment SGI71831[] = {{&S17999[31], 3}, {NULL}};
const Segment SGI71832[] = {{&S17999[92], 6}, {NULL}};
const Segment SGI71833[] = {{&S18000[44], 22}, {NULL}};
const Segment SGI71834[] = {{&S18000[44], 34}, {NULL}};
const Segment SGI71835[] = {{&S18000[30], 6}, {NULL}};
const Segment SGI71836[] = {{&S18000[31], 3}, {NULL}};
const Segment SGI71837[] = {{&S18000[92], 6}, {NULL}};
const Segment SGI71838[] = {{&S18001[44], 22}, {NULL}};
const Segment SGI71839[] = {{&S18001[44], 34}, {NULL}};
const Segment SGI71840[] = {{&S18001[30], 6}, {NULL}};
const Segment SGI71841[] = {{&S18001[31], 3}, {NULL}};
const Segment SGI71842[] = {{&S18001[92], 6}, {NULL}};
const Segment SGI71843[] = {{&S18002[44], 11}, {NULL}};
const Segment SGI71844[] = {{&S18002[44], 23}, {NULL}};
const Segment SGI71845[] = {{&S18002[30], 6}, {NULL}};
const Segment SGI71846[] = {{&S18002[84], 6}, {NULL}};
const Segment SGI71847[] = {{&S28545[44], 12}, {NULL}};
const Segment SGI71848[] = {{&S28545[44], 23}, {NULL}};
const Segment SGI71849[] = {{&S28545[30], 6}, {NULL}};
const Segment SGI71850[] = {{&S28545[132], 3}, {NULL}};
const Segment SGI71851[] = {{&S28545[98], 5}, {NULL}};
const Segment SGI71852[] = {{&S18014[44], 22}, {NULL}};
const Segment SGI71853[] = {{&S18014[44], 34}, {NULL}};
const Segment SGI71854[] = {{&S18014[30], 6}, {NULL}};
const Segment SGI71855[] = {{&S18014[31], 3}, {NULL}};
const Segment SGI71856[] = {{&S18014[92], 6}, {NULL}};
const Segment SGI71857[] = {{&S28546[44], 16}, {NULL}};
const Segment SGI71858[] = {{&S28546[44], 37}, {NULL}};
const Segment SGI71859[] = {{&S28546[30], 6}, {NULL}};
const Segment SGI71860[] = {{&S28546[129], 3}, {NULL}};
const Segment SGI71861[] = {{&S28546[95], 5}, {NULL}};
const Segment SGI71862[] = {{&S18015[44], 32}, {NULL}};
const Segment SGI71863[] = {{&S18015[44], 44}, {NULL}};
const Segment SGI71864[] = {{&S18015[30], 6}, {NULL}};
const Segment SGI71865[] = {{&S18015[31], 3}, {NULL}};
const Segment SGI71866[] = {{&S18015[102], 6}, {NULL}};
const Segment SGI71867[] = {{&S18016[44], 32}, {NULL}};
const Segment SGI71868[] = {{&S18016[44], 44}, {NULL}};
const Segment SGI71869[] = {{&S18016[30], 6}, {NULL}};
const Segment SGI71870[] = {{&S28548[44], 16}, {NULL}};
const Segment SGI71871[] = {{&S28548[44], 30}, {NULL}};
const Segment SGI71872[] = {{&S28548[30], 6}, {NULL}};
const Segment SGI71873[] = {{&S18016[31], 3}, {NULL}};
const Segment SGI71874[] = {{&S28548[122], 3}, {NULL}};
const Segment SGI71875[] = {{&S18016[102], 6}, {NULL}};
const Segment SGI71876[] = {{&S28548[88], 5}, {NULL}};
const Segment SGI71877[] = {{&S28551[44], 16}, {NULL}};
const Segment SGI71878[] = {{&S28551[44], 38}, {NULL}};
const Segment SGI71879[] = {{&S28551[30], 6}, {NULL}};
const Segment SGI71880[] = {{&S28551[130], 3}, {NULL}};
const Segment SGI71881[] = {{&S28551[96], 5}, {NULL}};
const Segment SGI71882[] = {{&S18021[44], 24}, {NULL}};
const Segment SGI71883[] = {{&S18021[44], 36}, {NULL}};
const Segment SGI71884[] = {{&S18021[30], 6}, {NULL}};
const Segment SGI71885[] = {{&S18021[31], 3}, {NULL}};
const Segment SGI71886[] = {{&S18021[94], 6}, {NULL}};
const Segment SGI71887[] = {{&S28554[44], 15}, {NULL}};
const Segment SGI71888[] = {{&S18023[44], 23}, {NULL}};
const Segment SGI71889[] = {{&S28554[44], 27}, {NULL}};
const Segment SGI71890[] = {{&S18023[30], 6}, {NULL}};
const Segment SGI71891[] = {{&S28554[30], 6}, {NULL}};
const Segment SGI71892[] = {{&S28554[119], 3}, {NULL}};
const Segment SGI71893[] = {{&S28554[85], 5}, {NULL}};
const Segment SGI71894[] = {{&S18024[44], 20}, {NULL}};
const Segment SGI71895[] = {{&S18024[44], 32}, {NULL}};
const Segment SGI71896[] = {{&S18024[30], 6}, {NULL}};
const Segment SGI71897[] = {{&S18024[31], 3}, {NULL}};
const Segment SGI71898[] = {{&S18024[90], 6}, {NULL}};
const Segment SGI71899[] = {{&S18026[44], 11}, {NULL}};
const Segment SGI71900[] = {{&S18026[44], 23}, {NULL}};
const Segment SGI71901[] = {{&S18026[30], 6}, {NULL}};
const Segment SGI71902[] = {{&S18026[31], 3}, {NULL}};
const Segment SGI71903[] = {{&S18026[81], 6}, {NULL}};
const Segment SGI71904[] = {{&S18027[44], 20}, {NULL}};
const Segment SGI71905[] = {{&S18027[44], 32}, {NULL}};
const Segment SGI71906[] = {{&S18027[30], 6}, {NULL}};
const Segment SGI71907[] = {{&S18027[31], 3}, {NULL}};
const Segment SGI71908[] = {{&S18027[90], 6}, {NULL}};
const Segment SGI71909[] = {{&S18029[44], 20}, {NULL}};
const Segment SGI71910[] = {{&S18029[44], 32}, {NULL}};
const Segment SGI71911[] = {{&S18029[30], 6}, {NULL}};
const Segment SGI71912[] = {{&S18029[31], 3}, {NULL}};
const Segment SGI71913[] = {{&S18029[90], 6}, {NULL}};
const Segment SGI71914[] = {{&S18030[44], 20}, {NULL}};
const Segment SGI71915[] = {{&S18030[44], 32}, {NULL}};
const Segment SGI71916[] = {{&S18030[30], 6}, {NULL}};
const Segment SGI71917[] = {{&S18030[31], 3}, {NULL}};
const Segment SGI71918[] = {{&S18030[90], 6}, {NULL}};
const Segment SGI71919[] = {{&S18031[44], 20}, {NULL}};
const Segment SGI71920[] = {{&S18031[44], 32}, {NULL}};
const Segment SGI71921[] = {{&S18031[30], 6}, {NULL}};
const Segment SGI71922[] = {{&S18031[31], 3}, {NULL}};
const Segment SGI71923[] = {{&S18031[90], 6}, {NULL}};
const Segment SGI71924[] = {{&S18035[44], 24}, {NULL}};
const Segment SGI71925[] = {{&S18035[44], 36}, {NULL}};
const Segment SGI71926[] = {{&S18035[30], 6}, {NULL}};
const Segment SGI71927[] = {{&S18035[31], 3}, {NULL}};
const Segment SGI71928[] = {{&S18035[94], 6}, {NULL}};
const Segment SGI71929[] = {{&S18036[42], 11}, {NULL}};
const Segment SGI71930[] = {{&S18036[30], 6}, {NULL}};
const Segment SGI71931[] = {{&S18036[31], 3}, {NULL}};
const Segment SGI71932[] = {{&S18036[79], 6}, {NULL}};
const Segment SGI71933[] = {{&S18039[44], 20}, {NULL}};
const Segment SGI71934[] = {{&S18039[44], 32}, {NULL}};
const Segment SGI71935[] = {{&S18039[30], 6}, {NULL}};
const Segment SGI71936[] = {{&S18039[31], 3}, {NULL}};
const Segment SGI71937[] = {{&S18039[90], 6}, {NULL}};
const Segment SGI71938[] = {{&S18040[44], 9}, {NULL}};
const Segment SGI71939[] = {{&S18040[44], 21}, {NULL}};
const Segment SGI71940[] = {{&S18040[30], 6}, {NULL}};
const Segment SGI71941[] = {{&S18040[31], 3}, {NULL}};
const Segment SGI71942[] = {{&S18040[79], 6}, {NULL}};
const Segment SGI71943[] = {{&S34782[44], 11}, {NULL}};
const Segment SGI71944[] = {{&S34782[44], 28}, {NULL}};
const Segment SGI71945[] = {{&S34782[30], 6}, {NULL}};
const Segment SGI71946[] = {{&S34782[122], 3}, {NULL}};
const Segment SGI71947[] = {{&S34782[86], 7}, {NULL}};
const Segment SGI71948[] = {{&S34782[142], 8}, {NULL}};
const Segment SGI71949[] = {{&S34784[44], 11}, {NULL}};
const Segment SGI71950[] = {{&S34784[44], 28}, {NULL}};
const Segment SGI71951[] = {{&S34784[30], 6}, {NULL}};
const Segment SGI71952[] = {{&S34784[120], 3}, {NULL}};
const Segment SGI71953[] = {{&S34784[86], 5}, {NULL}};
const Segment SGI71954[] = {{&S34785[38], 11}, {NULL}};
const Segment SGI71955[] = {{&S34785[31], 6}, {NULL}};
const Segment SGI71956[] = {{&S34786[43], 11}, {NULL}};
const Segment SGI71957[] = {{&S34786[43], 28}, {NULL}};
const Segment SGI71958[] = {{&S34786[30], 5}, {NULL}};
const Segment SGI71959[] = {{&S34786[119], 3}, {NULL}};
const Segment SGI71960[] = {{&S34786[85], 5}, {NULL}};
const Segment SGI71961[] = {{&S34787[43], 11}, {NULL}};
const Segment SGI71962[] = {{&S34787[43], 28}, {NULL}};
const Segment SGI71963[] = {{&S34787[119], 3}, {NULL}};
const Segment SGI71964[] = {{&S34787[85], 5}, {NULL}};
const Segment SGI71965[] = {{&S34788[44], 11}, {NULL}};
const Segment SGI71966[] = {{&S34788[44], 28}, {NULL}};
const Segment SGI71967[] = {{&S34788[30], 6}, {NULL}};
const Segment SGI71968[] = {{&S34788[120], 3}, {NULL}};
const Segment SGI71969[] = {{&S34788[86], 5}, {NULL}};
const Segment SGI71970[] = {{&S34789[44], 10}, {NULL}};
const Segment SGI71971[] = {{&S34789[44], 27}, {NULL}};
const Segment SGI71972[] = {{&S34789[30], 6}, {NULL}};
const Segment SGI71973[] = {{&S34789[119], 3}, {NULL}};
const Segment SGI71974[] = {{&S34789[85], 5}, {NULL}};
const Segment SGI71975[] = {{&S34790[44], 10}, {NULL}};
const Segment SGI71976[] = {{&S34790[44], 27}, {NULL}};
const Segment SGI71977[] = {{&S34790[30], 6}, {NULL}};
const Segment SGI71978[] = {{&S34790[119], 3}, {NULL}};
const Segment SGI71979[] = {{&S34790[85], 5}, {NULL}};
const Segment SGI71980[] = {{&S34804[44], 13}, {NULL}};
const Segment SGI71981[] = {{&S34804[44], 29}, {NULL}};
const Segment SGI71982[] = {{&S34804[30], 6}, {NULL}};
const Segment SGI71983[] = {{&S34804[121], 3}, {NULL}};
const Segment SGI71984[] = {{&S34804[87], 5}, {NULL}};
const Segment SGI71985[] = {{&S34805[44], 7}, {NULL}};
const Segment SGI71986[] = {{&S34805[44], 24}, {NULL}};
const Segment SGI71987[] = {{&S34805[30], 6}, {NULL}};
const Segment SGI71988[] = {{&S34805[116], 3}, {NULL}};
const Segment SGI71989[] = {{&S34805[82], 5}, {NULL}};
const Segment SGI71990[] = {{&S34806[38], 7}, {NULL}};
const Segment SGI71991[] = {{&S34806[31], 6}, {NULL}};
const Segment SGI71992[] = {{&S34807[44], 13}, {NULL}};
const Segment SGI71993[] = {{&S34807[44], 29}, {NULL}};
const Segment SGI71994[] = {{&S34807[30], 6}, {NULL}};
const Segment SGI71995[] = {{&S34807[121], 3}, {NULL}};
const Segment SGI71996[] = {{&S34807[87], 5}, {NULL}};
const Segment SGI71997[] = {{&S34808[44], 6}, {NULL}};
const Segment SGI71998[] = {{&S34808[44], 23}, {NULL}};
const Segment SGI71999[] = {{&S34808[30], 6}, {NULL}};
const Segment SGI72000[] = {{&S34808[115], 3}, {NULL}};
const Segment SGI72001[] = {{&S34808[81], 5}, {NULL}};
const Segment SGI72002[] = {{&S34809[44], 6}, {NULL}};
const Segment SGI72003[] = {{&S34809[44], 22}, {NULL}};
const Segment SGI72004[] = {{&S34809[30], 6}, {NULL}};
const Segment SGI72005[] = {{&S34809[114], 3}, {NULL}};
const Segment SGI72006[] = {{&S34809[80], 5}, {NULL}};
const Segment SGI72007[] = {{&S34810[38], 13}, {NULL}};
const Segment SGI72008[] = {{&S34810[31], 6}, {NULL}};
const Segment SGI72009[] = {{&S34811[43], 7}, {NULL}};
const Segment SGI72010[] = {{&S34811[43], 24}, {NULL}};
const Segment SGI72011[] = {{&S34811[30], 5}, {NULL}};
const Segment SGI72012[] = {{&S34811[115], 3}, {NULL}};
const Segment SGI72013[] = {{&S34811[81], 5}, {NULL}};
const Segment SGI72014[] = {{&S34812[44], 7}, {NULL}};
const Segment SGI72015[] = {{&S34812[44], 24}, {NULL}};
const Segment SGI72016[] = {{&S34812[30], 6}, {NULL}};
const Segment SGI72017[] = {{&S34812[116], 3}, {NULL}};
const Segment SGI72018[] = {{&S34812[82], 5}, {NULL}};
const Segment SGI72019[] = {{&S34813[42], 7}, {NULL}};
const Segment SGI72020[] = {{&S34813[30], 6}, {NULL}};
const Segment SGI72021[] = {{&S34813[114], 3}, {NULL}};
const Segment SGI72022[] = {{&S34813[80], 5}, {NULL}};
const Segment SGI72023[] = {{&S28780[38], 13}, {NULL}};
const Segment SGI72024[] = {{&S28780[31], 6}, {NULL}};
const Segment SGI72025[] = {{&S18298[44], 10}, {NULL}};
const Segment SGI72026[] = {{&S18298[44], 22}, {NULL}};
const Segment SGI72027[] = {{&S18298[30], 6}, {NULL}};
const Segment SGI72028[] = {{&S18298[31], 3}, {NULL}};
const Segment SGI72029[] = {{&S18298[80], 6}, {NULL}};
const Segment SGI72030[] = {{&S18301[38], 10}, {NULL}};
const Segment SGI72031[] = {{&S18301[31], 6}, {NULL}};
const Segment SGI72032[] = {{&S18302[42], 10}, {NULL}};
const Segment SGI72033[] = {{&S18302[30], 6}, {NULL}};
const Segment SGI72034[] = {{&S18302[31], 3}, {NULL}};
const Segment SGI72035[] = {{&S18302[78], 6}, {NULL}};
const Segment SGI72036[] = {{&S18304[44], 10}, {NULL}};
const Segment SGI72037[] = {{&S18304[44], 22}, {NULL}};
const Segment SGI72038[] = {{&S18304[30], 6}, {NULL}};
const Segment SGI72039[] = {{&S18304[31], 3}, {NULL}};
const Segment SGI72040[] = {{&S18304[80], 6}, {NULL}};
const Segment SGI72041[] = {{&S18305[44], 10}, {NULL}};
const Segment SGI72042[] = {{&S18305[44], 22}, {NULL}};
const Segment SGI72043[] = {{&S18305[30], 6}, {NULL}};
const Segment SGI72044[] = {{&S18305[125], 3}, {NULL}};
const Segment SGI72045[] = {{&S18305[90], 6}, {NULL}};
const Segment SGI72046[] = {{&S18305[145], 8}, {NULL}};
const Segment SGI72047[] = {{&S18306[44], 10}, {NULL}};
const Segment SGI72048[] = {{&S18306[44], 22}, {NULL}};
const Segment SGI72049[] = {{&S18306[30], 6}, {NULL}};
const Segment SGI72050[] = {{&S18306[83], 6}, {NULL}};
const Segment SGI72051[] = {{&S18308[45], 10}, {NULL}};
const Segment SGI72052[] = {{&S18308[45], 34}, {NULL}};
const Segment SGI72053[] = {{&S18308[31], 6}, {NULL}};
const Segment SGI72054[] = {{&S18311[65], 10}, {NULL}};
const Segment SGI72055[] = {{&S18311[65], 22}, {NULL}};
const Segment SGI72056[] = {{&S18311[51], 6}, {NULL}};
const Segment SGI72057[] = {{&S18312[45], 10}, {NULL}};
const Segment SGI72058[] = {{&S18312[45], 34}, {NULL}};
const Segment SGI72059[] = {{&S18312[31], 6}, {NULL}};
const Segment SGI72060[] = {{&S18315[44], 10}, {&S18315[175], 18}, {NULL}};
const Segment SGI72061[] = {{&S18315[44], 22}, {NULL}};
const Segment SGI72062[] = {{&S18315[30], 6}, {NULL}};
const Segment SGI72063[] = {{&S18315[31], 3}, {NULL}};
const Segment SGI72064[] = {{&S18315[80], 6}, {NULL}};
const Segment SGI72065[] = {{&S18318[44], 10}, {NULL}};
const Segment SGI72066[] = {{&S18318[44], 22}, {NULL}};
const Segment SGI72067[] = {{&S18318[30], 6}, {NULL}};
const Segment SGI72068[] = {{&S18318[80], 6}, {NULL}};
const Segment SGI72069[] = {{&S18320[44], 10}, {NULL}};
const Segment SGI72070[] = {{&S18320[44], 34}, {NULL}};
const Segment SGI72071[] = {{&S18320[30], 6}, {NULL}};
const Segment SGI72072[] = {{&S18321[44], 10}, {NULL}};
const Segment SGI72073[] = {{&S18321[44], 34}, {NULL}};
const Segment SGI72074[] = {{&S18321[30], 6}, {NULL}};
const Segment SGI72075[] = {{&S18322[44], 10}, {NULL}};
const Segment SGI72076[] = {{&S18322[44], 22}, {NULL}};
const Segment SGI72077[] = {{&S18322[30], 6}, {NULL}};
const Segment SGI72078[] = {{&S18322[116], 3}, {NULL}};
const Segment SGI72079[] = {{&S18322[80], 7}, {NULL}};
const Segment SGI72080[] = {{&S18322[136], 8}, {NULL}};
const Segment SGI72081[] = {{&S18323[44], 10}, {NULL}};
const Segment SGI72082[] = {{&S18323[44], 34}, {NULL}};
const Segment SGI72083[] = {{&S18323[30], 6}, {NULL}};
const Segment SGI72084[] = {{&S18326[105], 29}, {NULL}};
const Segment SGI72085[] = {{&S18327[30], 1}, {NULL}};
const Segment SGI72086[] = {{&S18327[63], 9}, {NULL}};
const Segment SGI72087[] = {{&S18327[49], 6}, {NULL}};
const Segment SGI72088[] = {{&S18329[44], 10}, {NULL}};
const Segment SGI72089[] = {{&S18329[44], 22}, {&S18329[207], 5}, {NULL}};
const Segment SGI72090[] = {{&S18329[30], 6}, {NULL}};
const Segment SGI72091[] = {{&S18329[31], 3}, {NULL}};
const Segment SGI72092[] = {{&S18329[80], 6}, {NULL}};
const Segment SGI72093[] = {{&S18332[44], 10}, {NULL}};
const Segment SGI72094[] = {{&S18332[44], 22}, {NULL}};
const Segment SGI72095[] = {{&S18332[30], 6}, {NULL}};
const Segment SGI72096[] = {{&S18332[31], 3}, {NULL}};
const Segment SGI72097[] = {{&S18332[80], 6}, {NULL}};
const Segment SGI72098[] = {{&S35063[44], 9}, {NULL}};
const Segment SGI72099[] = {{&S35063[44], 21}, {NULL}};
const Segment SGI72100[] = {{&S35063[30], 6}, {NULL}};
const Segment SGI72101[] = {{&S35063[113], 3}, {NULL}};
const Segment SGI72102[] = {{&S35063[79], 5}, {NULL}};
const Segment SGI72103[] = {{&S35064[38], 9}, {NULL}};
const Segment SGI72104[] = {{&S35064[31], 6}, {NULL}};
const Segment SGI72105[] = {{&S35065[44], 9}, {NULL}};
const Segment SGI72106[] = {{&S35065[44], 21}, {NULL}};
const Segment SGI72107[] = {{&S35065[30], 6}, {NULL}};
const Segment SGI72108[] = {{&S35065[113], 3}, {NULL}};
const Segment SGI72109[] = {{&S35065[79], 5}, {NULL}};
const Segment SGI72110[] = {{&S18339[44], 10}, {NULL}};
const Segment SGI72111[] = {{&S18339[44], 19}, {NULL}};
const Segment SGI72112[] = {{&S18339[30], 6}, {NULL}};
const Segment SGI72113[] = {{&S18339[31], 3}, {NULL}};
const Segment SGI72114[] = {{&S18339[77], 5}, {NULL}};
const Segment SGI72115[] = {{&S18346[44], 10}, {NULL}};
const Segment SGI72116[] = {{&S18346[44], 22}, {NULL}};
const Segment SGI72117[] = {{&S18346[30], 6}, {NULL}};
const Segment SGI72118[] = {{&S18346[31], 3}, {NULL}};
const Segment SGI72119[] = {{&S18346[80], 5}, {NULL}};
const Segment SGI72120[] = {{&S35078[44], 9}, {NULL}};
const Segment SGI72121[] = {{&S35078[44], 21}, {NULL}};
const Segment SGI72122[] = {{&S35078[30], 6}, {NULL}};
const Segment SGI72123[] = {{&S35078[82], 4}, {NULL}};
const Segment SGI72124[] = {{&S35078[82], 3}, {NULL}};
const Segment SGI72125[] = {{&S18347[44], 10}, {&S18347[44], 10}, {NULL}};
const Segment SGI72126[] = {{&S18347[44], 22}, {&S18347[44], 22}, {NULL}};
const Segment SGI72127[] = {{&S18347[30], 6}, {&S18347[30], 6}, {NULL}};
const Segment SGI72128[] = {{&S18347[31], 3}, {NULL}};
const Segment SGI72129[] = {{&S18347[80], 6}, {NULL}};
const Segment SGI72130[] = {{&S28926[44], 10}, {NULL}};
const Segment SGI72131[] = {{&S28926[44], 21}, {NULL}};
const Segment SGI72132[] = {{&S28926[30], 6}, {NULL}};
const Segment SGI72133[] = {{&S28926[113], 3}, {NULL}};
const Segment SGI72134[] = {{&S28926[79], 5}, {NULL}};
const Segment SGI72135[] = {{&S35090[44], 9}, {NULL}};
const Segment SGI72136[] = {{&S35090[44], 21}, {NULL}};
const Segment SGI72137[] = {{&S35090[30], 6}, {NULL}};
const Segment SGI72138[] = {{&S35090[82], 6}, {NULL}};
const Segment SGI72139[] = {{&S35092[44], 9}, {NULL}};
const Segment SGI72140[] = {{&S35092[44], 26}, {NULL}};
const Segment SGI72141[] = {{&S35092[30], 6}, {NULL}};
const Segment SGI72142[] = {{&S35092[118], 3}, {NULL}};
const Segment SGI72143[] = {{&S35092[84], 5}, {NULL}};
const Segment SGI72144[] = {{&S35108[38], 10}, {NULL}};
const Segment SGI72145[] = {{&S35108[31], 6}, {NULL}};
const Segment SGI72146[] = {{&S35113[44], 13}, {NULL}};
const Segment SGI72147[] = {{&S35113[44], 25}, {NULL}};
const Segment SGI72148[] = {{&S35113[30], 6}, {NULL}};
const Segment SGI72149[] = {{&S35113[117], 3}, {NULL}};
const Segment SGI72150[] = {{&S35113[83], 5}, {NULL}};
const Segment SGI72151[] = {{&S35121[44], 10}, {NULL}};
const Segment SGI72152[] = {{&S35121[44], 22}, {NULL}};
const Segment SGI72153[] = {{&S35121[30], 6}, {NULL}};
const Segment SGI72154[] = {{&S35121[114], 3}, {NULL}};
const Segment SGI72155[] = {{&S35121[80], 5}, {NULL}};
const Segment SGI72156[] = {{&S35304[44], 15}, {NULL}};
const Segment SGI72157[] = {{&S35304[44], 42}, {NULL}};
const Segment SGI72158[] = {{&S35304[30], 6}, {NULL}};
const Segment SGI72159[] = {{&S35304[134], 3}, {NULL}};
const Segment SGI72160[] = {{&S35304[100], 5}, {NULL}};
const Segment SGI72161[] = {{&S35305[44], 15}, {NULL}};
const Segment SGI72162[] = {{&S35305[44], 42}, {NULL}};
const Segment SGI72163[] = {{&S35305[30], 6}, {NULL}};
const Segment SGI72164[] = {{&S35305[134], 3}, {NULL}};
const Segment SGI72165[] = {{&S35305[100], 5}, {NULL}};
const Segment SGI72166[] = {{&S35306[44], 12}, {NULL}};
const Segment SGI72167[] = {{&S35306[44], 39}, {NULL}};
const Segment SGI72168[] = {{&S35306[30], 6}, {NULL}};
const Segment SGI72169[] = {{&S35306[131], 3}, {NULL}};
const Segment SGI72170[] = {{&S35306[97], 5}, {NULL}};
const Segment SGI72171[] = {{&S35309[44], 15}, {NULL}};
const Segment SGI72172[] = {{&S35309[44], 42}, {NULL}};
const Segment SGI72173[] = {{&S35309[30], 6}, {NULL}};
const Segment SGI72174[] = {{&S35309[134], 3}, {NULL}};
const Segment SGI72175[] = {{&S35309[100], 5}, {NULL}};
const Segment SGI72176[] = {{&S35310[44], 12}, {NULL}};
const Segment SGI72177[] = {{&S35310[44], 39}, {NULL}};
const Segment SGI72178[] = {{&S35310[30], 6}, {NULL}};
const Segment SGI72179[] = {{&S35310[97], 6}, {NULL}};
const Segment SGI72180[] = {{&S35312[44], 15}, {NULL}};
const Segment SGI72181[] = {{&S35312[44], 42}, {NULL}};
const Segment SGI72182[] = {{&S35312[30], 6}, {NULL}};
const Segment SGI72183[] = {{&S35312[134], 3}, {NULL}};
const Segment SGI72184[] = {{&S35312[100], 5}, {NULL}};
const Segment SGI72185[] = {{&S35313[44], 12}, {NULL}};
const Segment SGI72186[] = {{&S35313[44], 39}, {NULL}};
const Segment SGI72187[] = {{&S35313[30], 6}, {NULL}};
const Segment SGI72188[] = {{&S35313[131], 3}, {NULL}};
const Segment SGI72189[] = {{&S35313[97], 5}, {NULL}};
const Segment SGI72190[] = {{&S35316[38], 12}, {NULL}};
const Segment SGI72191[] = {{&S35316[31], 6}, {NULL}};
const Segment SGI72192[] = {{&S35321[44], 15}, {NULL}};
const Segment SGI72193[] = {{&S35321[44], 42}, {NULL}};
const Segment SGI72194[] = {{&S35321[30], 6}, {NULL}};
const Segment SGI72195[] = {{&S35321[134], 3}, {NULL}};
const Segment SGI72196[] = {{&S35321[100], 5}, {NULL}};
const Segment SGI72197[] = {{&S35322[44], 15}, {NULL}};
const Segment SGI72198[] = {{&S35322[44], 42}, {NULL}};
const Segment SGI72199[] = {{&S35322[30], 6}, {NULL}};
const Segment SGI72200[] = {{&S35322[134], 3}, {NULL}};
const Segment SGI72201[] = {{&S35322[100], 5}, {NULL}};
const Segment SGI72202[] = {{&S35323[44], 15}, {NULL}};
const Segment SGI72203[] = {{&S35323[44], 37}, {NULL}};
const Segment SGI72204[] = {{&S35323[30], 6}, {NULL}};
const Segment SGI72205[] = {{&S35323[129], 3}, {NULL}};
const Segment SGI72206[] = {{&S35323[95], 5}, {NULL}};
const Segment SGI72207[] = {{&S35325[44], 15}, {NULL}};
const Segment SGI72208[] = {{&S35325[44], 37}, {NULL}};
const Segment SGI72209[] = {{&S35325[30], 6}, {NULL}};
const Segment SGI72210[] = {{&S35325[129], 3}, {NULL}};
const Segment SGI72211[] = {{&S35325[95], 5}, {NULL}};
const Segment SGI72212[] = {{&S35328[44], 12}, {NULL}};
const Segment SGI72213[] = {{&S35328[44], 39}, {NULL}};
const Segment SGI72214[] = {{&S35328[30], 6}, {NULL}};
const Segment SGI72215[] = {{&S35328[133], 3}, {NULL}};
const Segment SGI72216[] = {{&S35328[97], 7}, {NULL}};
const Segment SGI72217[] = {{&S35328[153], 8}, {NULL}};
const Segment SGI72218[] = {{&S35329[44], 15}, {NULL}};
const Segment SGI72219[] = {{&S35329[44], 42}, {NULL}};
const Segment SGI72220[] = {{&S35329[30], 6}, {NULL}};
const Segment SGI72221[] = {{&S35329[134], 3}, {NULL}};
const Segment SGI72222[] = {{&S35329[100], 5}, {NULL}};
const Segment SGI72223[] = {{&S35330[44], 14}, {NULL}};
const Segment SGI72224[] = {{&S35330[44], 41}, {NULL}};
const Segment SGI72225[] = {{&S35330[30], 6}, {NULL}};
const Segment SGI72226[] = {{&S35330[133], 3}, {NULL}};
const Segment SGI72227[] = {{&S35330[99], 5}, {NULL}};
const Segment SGI72228[] = {{&S35331[43], 15}, {NULL}};
const Segment SGI72229[] = {{&S35331[43], 42}, {NULL}};
const Segment SGI72230[] = {{&S35331[30], 5}, {NULL}};
const Segment SGI72231[] = {{&S35331[133], 3}, {NULL}};
const Segment SGI72232[] = {{&S35331[99], 5}, {NULL}};
const Segment SGI72233[] = {{&S35332[44], 15}, {NULL}};
const Segment SGI72234[] = {{&S35332[44], 42}, {NULL}};
const Segment SGI72235[] = {{&S35332[30], 6}, {NULL}};
const Segment SGI72236[] = {{&S35332[134], 3}, {NULL}};
const Segment SGI72237[] = {{&S35332[100], 5}, {NULL}};
const Segment SGI72238[] = {{&S35333[43], 15}, {NULL}};
const Segment SGI72239[] = {{&S35333[43], 42}, {NULL}};
const Segment SGI72240[] = {{&S35333[133], 3}, {NULL}};
const Segment SGI72241[] = {{&S35333[99], 5}, {NULL}};
const Segment SGI72242[] = {{&S35334[44], 15}, {NULL}};
const Segment SGI72243[] = {{&S35334[44], 42}, {NULL}};
const Segment SGI72244[] = {{&S35334[30], 6}, {NULL}};
const Segment SGI72245[] = {{&S35334[134], 3}, {NULL}};
const Segment SGI72246[] = {{&S35334[100], 5}, {NULL}};
const Segment SGI72247[] = {{&S35335[43], 15}, {NULL}};
const Segment SGI72248[] = {{&S35335[43], 42}, {NULL}};
const Segment SGI72249[] = {{&S35335[30], 5}, {NULL}};
const Segment SGI72250[] = {{&S35335[133], 3}, {NULL}};
const Segment SGI72251[] = {{&S35335[99], 5}, {NULL}};
const Segment SGI72252[] = {{&S35336[43], 15}, {NULL}};
const Segment SGI72253[] = {{&S35336[43], 42}, {NULL}};
const Segment SGI72254[] = {{&S35336[133], 3}, {NULL}};
const Segment SGI72255[] = {{&S35336[99], 5}, {NULL}};
const Segment SGI72256[] = {{&S35337[44], 14}, {NULL}};
const Segment SGI72257[] = {{&S35337[44], 41}, {NULL}};
const Segment SGI72258[] = {{&S35337[30], 6}, {NULL}};
const Segment SGI72259[] = {{&S35337[133], 3}, {NULL}};
const Segment SGI72260[] = {{&S35337[99], 5}, {NULL}};
const Segment SGI72261[] = {{&S35338[44], 15}, {NULL}};
const Segment SGI72262[] = {{&S35338[44], 42}, {NULL}};
const Segment SGI72263[] = {{&S35338[30], 6}, {NULL}};
const Segment SGI72264[] = {{&S35338[134], 3}, {NULL}};
const Segment SGI72265[] = {{&S35338[100], 5}, {NULL}};
const Segment SGI72266[] = {{&S35339[44], 14}, {NULL}};
const Segment SGI72267[] = {{&S35339[44], 41}, {NULL}};
const Segment SGI72268[] = {{&S35339[30], 6}, {NULL}};
const Segment SGI72269[] = {{&S35339[133], 3}, {NULL}};
const Segment SGI72270[] = {{&S35339[99], 5}, {NULL}};
const Segment SGI72271[] = {{&S35340[44], 15}, {NULL}};
const Segment SGI72272[] = {{&S35340[44], 42}, {NULL}};
const Segment SGI72273[] = {{&S35340[30], 6}, {NULL}};
const Segment SGI72274[] = {{&S35340[134], 3}, {NULL}};
const Segment SGI72275[] = {{&S35340[100], 5}, {NULL}};
const Segment SGI72276[] = {{&S35341[44], 12}, {NULL}};
const Segment SGI72277[] = {{&S35341[44], 39}, {NULL}};
const Segment SGI72278[] = {{&S35341[30], 6}, {NULL}};
const Segment SGI72279[] = {{&S35341[131], 3}, {NULL}};
const Segment SGI72280[] = {{&S35341[97], 5}, {NULL}};
const Segment SGI72281[] = {{&S35406[44], 10}, {NULL}};
const Segment SGI72282[] = {{&S35406[44], 27}, {NULL}};
const Segment SGI72283[] = {{&S35406[30], 6}, {NULL}};
const Segment SGI72284[] = {{&S35406[119], 3}, {NULL}};
const Segment SGI72285[] = {{&S35406[85], 5}, {NULL}};
const Segment SGI72286[] = {{&S35407[38], 10}, {NULL}};
const Segment SGI72287[] = {{&S35407[31], 6}, {NULL}};
const Segment SGI72288[] = {{&S35408[44], 9}, {NULL}};
const Segment SGI72289[] = {{&S35408[44], 26}, {NULL}};
const Segment SGI72290[] = {{&S35408[30], 6}, {NULL}};
const Segment SGI72291[] = {{&S35408[118], 3}, {NULL}};
const Segment SGI72292[] = {{&S35408[84], 5}, {NULL}};
const Segment SGI72293[] = {{&S35409[43], 10}, {NULL}};
const Segment SGI72294[] = {{&S35409[43], 27}, {NULL}};
const Segment SGI72295[] = {{&S35409[30], 5}, {NULL}};
const Segment SGI72296[] = {{&S35409[118], 3}, {NULL}};
const Segment SGI72297[] = {{&S35409[84], 5}, {NULL}};
const Segment SGI72298[] = {{&S35410[44], 9}, {NULL}};
const Segment SGI72299[] = {{&S35410[44], 26}, {NULL}};
const Segment SGI72300[] = {{&S35410[30], 6}, {NULL}};
const Segment SGI72301[] = {{&S35410[118], 3}, {NULL}};
const Segment SGI72302[] = {{&S35410[84], 5}, {NULL}};
const Segment SGI72303[] = {{&S35411[43], 10}, {NULL}};
const Segment SGI72304[] = {{&S35411[43], 27}, {NULL}};
const Segment SGI72305[] = {{&S35411[118], 3}, {NULL}};
const Segment SGI72306[] = {{&S35411[84], 5}, {NULL}};
const Segment SGI72307[] = {{&S35412[44], 10}, {NULL}};
const Segment SGI72308[] = {{&S35412[44], 27}, {NULL}};
const Segment SGI72309[] = {{&S35412[30], 6}, {NULL}};
const Segment SGI72310[] = {{&S35412[119], 3}, {NULL}};
const Segment SGI72311[] = {{&S35412[85], 5}, {NULL}};
const Segment SGI72312[] = {{&S35413[44], 10}, {NULL}};
const Segment SGI72313[] = {{&S35413[44], 27}, {NULL}};
const Segment SGI72314[] = {{&S35413[30], 6}, {NULL}};
const Segment SGI72315[] = {{&S35413[121], 3}, {NULL}};
const Segment SGI72316[] = {{&S35413[85], 7}, {NULL}};
const Segment SGI72317[] = {{&S35413[141], 8}, {NULL}};
const Segment SGI72318[] = {{&S35414[42], 10}, {NULL}};
const Segment SGI72319[] = {{&S35414[30], 6}, {NULL}};
const Segment SGI72320[] = {{&S35414[117], 3}, {NULL}};
const Segment SGI72321[] = {{&S35414[83], 5}, {NULL}};
const Segment SGI72322[] = {{&S35451[44], 8}, {NULL}};
const Segment SGI72323[] = {{&S35451[44], 25}, {NULL}};
const Segment SGI72324[] = {{&S35451[30], 6}, {NULL}};
const Segment SGI72325[] = {{&S35451[117], 3}, {NULL}};
const Segment SGI72326[] = {{&S35451[83], 5}, {NULL}};
const Segment SGI72327[] = {{&S35452[38], 8}, {NULL}};
const Segment SGI72328[] = {{&S35452[31], 6}, {NULL}};
const Segment SGI72329[] = {{&S35529[44], 7}, {NULL}};
const Segment SGI72330[] = {{&S35529[44], 26}, {NULL}};
const Segment SGI72331[] = {{&S35529[30], 6}, {NULL}};
const Segment SGI72332[] = {{&S35529[118], 3}, {NULL}};
const Segment SGI72333[] = {{&S35529[84], 5}, {NULL}};
const Segment SGI72334[] = {{&S35530[44], 7}, {NULL}};
const Segment SGI72335[] = {{&S35530[44], 31}, {NULL}};
const Segment SGI72336[] = {{&S35530[30], 6}, {NULL}};
const Segment SGI72337[] = {{&S35531[38], 7}, {NULL}};
const Segment SGI72338[] = {{&S35531[31], 6}, {NULL}};
const Segment SGI72339[] = {{&S35532[44], 7}, {NULL}};
const Segment SGI72340[] = {{&S35532[44], 31}, {NULL}};
const Segment SGI72341[] = {{&S35532[30], 6}, {NULL}};
const Segment SGI72342[] = {{&S35532[89], 6}, {NULL}};
const Segment SGI72343[] = {{&S35533[44], 7}, {NULL}};
const Segment SGI72344[] = {{&S35533[44], 31}, {NULL}};
const Segment SGI72345[] = {{&S35533[30], 6}, {NULL}};
const Segment SGI72346[] = {{&S35533[123], 3}, {NULL}};
const Segment SGI72347[] = {{&S35533[89], 5}, {NULL}};
const Segment SGI72348[] = {{&S35535[44], 7}, {NULL}};
const Segment SGI72349[] = {{&S35535[44], 31}, {NULL}};
const Segment SGI72350[] = {{&S35535[30], 6}, {NULL}};
const Segment SGI72351[] = {{&S35535[123], 3}, {NULL}};
const Segment SGI72352[] = {{&S35535[89], 5}, {NULL}};
const Segment SGI72353[] = {{&S35536[44], 7}, {NULL}};
const Segment SGI72354[] = {{&S35536[44], 26}, {NULL}};
const Segment SGI72355[] = {{&S35536[30], 6}, {NULL}};
const Segment SGI72356[] = {{&S35536[118], 3}, {NULL}};
const Segment SGI72357[] = {{&S35536[84], 5}, {NULL}};
const Segment SGI72358[] = {{&S35537[44], 7}, {NULL}};
const Segment SGI72359[] = {{&S35537[44], 26}, {NULL}};
const Segment SGI72360[] = {{&S35537[30], 6}, {NULL}};
const Segment SGI72361[] = {{&S35537[118], 3}, {NULL}};
const Segment SGI72362[] = {{&S35537[84], 5}, {NULL}};
const Segment SGI72363[] = {{&S35538[44], 7}, {NULL}};
const Segment SGI72364[] = {{&S35538[44], 29}, {NULL}};
const Segment SGI72365[] = {{&S35538[30], 6}, {NULL}};
const Segment SGI72366[] = {{&S35538[121], 3}, {NULL}};
const Segment SGI72367[] = {{&S35538[87], 5}, {NULL}};
const Segment SGI72368[] = {{&S35539[44], 7}, {NULL}};
const Segment SGI72369[] = {{&S35539[44], 29}, {NULL}};
const Segment SGI72370[] = {{&S35539[30], 6}, {NULL}};
const Segment SGI72371[] = {{&S35539[121], 3}, {NULL}};
const Segment SGI72372[] = {{&S35539[87], 5}, {NULL}};
const Segment SGI72373[] = {{&S35540[44], 7}, {NULL}};
const Segment SGI72374[] = {{&S35540[44], 31}, {NULL}};
const Segment SGI72375[] = {{&S35540[30], 6}, {NULL}};
const Segment SGI72376[] = {{&S35540[123], 3}, {NULL}};
const Segment SGI72377[] = {{&S35540[89], 5}, {NULL}};
const Segment SGI72378[] = {{&S35541[44], 7}, {NULL}};
const Segment SGI72379[] = {{&S35541[44], 35}, {NULL}};
const Segment SGI72380[] = {{&S35541[30], 6}, {NULL}};
const Segment SGI72381[] = {{&S35541[127], 3}, {NULL}};
const Segment SGI72382[] = {{&S35541[93], 5}, {NULL}};
const Segment SGI72383[] = {{&S35542[44], 7}, {NULL}};
const Segment SGI72384[] = {{&S35542[44], 36}, {NULL}};
const Segment SGI72385[] = {{&S35542[30], 6}, {NULL}};
const Segment SGI72386[] = {{&S35542[97], 4}, {NULL}};
const Segment SGI72387[] = {{&S35542[97], 3}, {NULL}};
const Segment SGI72388[] = {{&S35543[44], 7}, {NULL}};
const Segment SGI72389[] = {{&S35543[44], 27}, {NULL}};
const Segment SGI72390[] = {{&S35543[30], 6}, {NULL}};
const Segment SGI72391[] = {{&S35543[119], 3}, {NULL}};
const Segment SGI72392[] = {{&S35543[85], 5}, {NULL}};
const Segment SGI72393[] = {{&S35544[44], 7}, {NULL}};
const Segment SGI72394[] = {{&S35544[44], 29}, {NULL}};
const Segment SGI72395[] = {{&S35544[30], 6}, {NULL}};
const Segment SGI72396[] = {{&S35544[121], 3}, {NULL}};
const Segment SGI72397[] = {{&S35544[87], 5}, {NULL}};
const Segment SGI72398[] = {{&S35545[44], 7}, {NULL}};
const Segment SGI72399[] = {{&S35545[44], 29}, {NULL}};
const Segment SGI72400[] = {{&S35545[30], 6}, {NULL}};
const Segment SGI72401[] = {{&S35545[121], 3}, {NULL}};
const Segment SGI72402[] = {{&S35545[87], 5}, {NULL}};
const Segment SGI72403[] = {{&S35548[44], 7}, {NULL}};
const Segment SGI72404[] = {{&S35548[44], 36}, {NULL}};
const Segment SGI72405[] = {{&S35548[30], 6}, {NULL}};
const Segment SGI72406[] = {{&S35548[128], 3}, {NULL}};
const Segment SGI72407[] = {{&S35548[94], 5}, {NULL}};
const Segment SGI72408[] = {{&S28807[42], 10}, {NULL}};
const Segment SGI72409[] = {{&S28807[42], 21}, {NULL}};
const Segment SGI72410[] = {{&S28807[30], 4}, {NULL}};
const Segment SGI72411[] = {{&S28807[112], 3}, {NULL}};
const Segment SGI72412[] = {{&S28807[77], 6}, {NULL}};
const Segment SGI72413[] = {{&S35549[44], 7}, {NULL}};
const Segment SGI72414[] = {{&S35549[44], 36}, {NULL}};
const Segment SGI72415[] = {{&S35549[30], 6}, {NULL}};
const Segment SGI72416[] = {{&S35549[128], 3}, {NULL}};
const Segment SGI72417[] = {{&S35549[94], 5}, {NULL}};
const Segment SGI72418[] = {{&S28808[42], 10}, {NULL}};
const Segment SGI72419[] = {{&S28808[42], 21}, {NULL}};
const Segment SGI72420[] = {{&S28808[30], 4}, {NULL}};
const Segment SGI72421[] = {{&S28808[112], 3}, {NULL}};
const Segment SGI72422[] = {{&S28808[77], 6}, {NULL}};
const Segment SGI72423[] = {{&S35550[44], 7}, {NULL}};
const Segment SGI72424[] = {{&S35550[44], 27}, {NULL}};
const Segment SGI72425[] = {{&S35550[30], 6}, {NULL}};
const Segment SGI72426[] = {{&S35550[88], 4}, {NULL}};
const Segment SGI72427[] = {{&S35550[88], 3}, {NULL}};
const Segment SGI72428[] = {{&S28809[42], 10}, {NULL}};
const Segment SGI72429[] = {{&S28809[42], 25}, {NULL}};
const Segment SGI72430[] = {{&S28809[30], 4}, {NULL}};
const Segment SGI72431[] = {{&S28809[116], 3}, {NULL}};
const Segment SGI72432[] = {{&S28809[81], 6}, {NULL}};
const Segment SGI72433[] = {{&S35551[44], 7}, {NULL}};
const Segment SGI72434[] = {{&S35551[44], 27}, {NULL}};
const Segment SGI72435[] = {{&S35551[30], 6}, {NULL}};
const Segment SGI72436[] = {{&S35551[88], 6}, {NULL}};
const Segment SGI72437[] = {{&S28810[36], 10}, {NULL}};
const Segment SGI72438[] = {{&S28810[31], 4}, {NULL}};
const Segment SGI72439[] = {{&S35552[44], 7}, {NULL}};
const Segment SGI72440[] = {{&S35552[44], 36}, {NULL}};
const Segment SGI72441[] = {{&S35552[30], 6}, {NULL}};
const Segment SGI72442[] = {{&S35552[97], 4}, {NULL}};
const Segment SGI72443[] = {{&S35552[97], 3}, {NULL}};
const Segment SGI72444[] = {{&S28811[42], 10}, {NULL}};
const Segment SGI72445[] = {{&S28811[42], 27}, {NULL}};
const Segment SGI72446[] = {{&S28811[30], 4}, {NULL}};
const Segment SGI72447[] = {{&S35553[44], 7}, {NULL}};
const Segment SGI72448[] = {{&S35553[44], 36}, {NULL}};
const Segment SGI72449[] = {{&S35553[30], 6}, {NULL}};
const Segment SGI72450[] = {{&S28812[42], 10}, {NULL}};
const Segment SGI72451[] = {{&S28812[42], 21}, {NULL}};
const Segment SGI72452[] = {{&S35553[97], 6}, {NULL}};
const Segment SGI72453[] = {{&S28812[30], 4}, {NULL}};
const Segment SGI72454[] = {{&S28812[112], 3}, {NULL}};
const Segment SGI72455[] = {{&S28812[77], 6}, {NULL}};
const Segment SGI72456[] = {{&S35554[44], 7}, {NULL}};
const Segment SGI72457[] = {{&S35554[44], 31}, {NULL}};
const Segment SGI72458[] = {{&S35554[30], 6}, {NULL}};
const Segment SGI72459[] = {{&S28813[42], 10}, {NULL}};
const Segment SGI72460[] = {{&S35554[123], 3}, {NULL}};
const Segment SGI72461[] = {{&S28813[42], 21}, {NULL}};
const Segment SGI72462[] = {{&S28813[30], 4}, {NULL}};
const Segment SGI72463[] = {{&S35554[89], 5}, {NULL}};
const Segment SGI72464[] = {{&S28813[31], 3}, {NULL}};
const Segment SGI72465[] = {{&S28813[87], 6}, {NULL}};
const Segment SGI72466[] = {{&S28813[142], 8}, {NULL}};
const Segment SGI72467[] = {{&S35555[44], 10}, {NULL}};
const Segment SGI72468[] = {{&S35555[44], 27}, {NULL}};
const Segment SGI72469[] = {{&S35555[30], 6}, {NULL}};
const Segment SGI72470[] = {{&S28814[40], 10}, {NULL}};
const Segment SGI72471[] = {{&S35555[119], 3}, {NULL}};
const Segment SGI72472[] = {{&S28814[30], 4}, {NULL}};
const Segment SGI72473[] = {{&S35555[85], 5}, {NULL}};
const Segment SGI72474[] = {{&S28814[110], 3}, {NULL}};
const Segment SGI72475[] = {{&S28814[75], 6}, {NULL}};
const Segment SGI72476[] = {{&S35556[44], 13}, {NULL}};
const Segment SGI72477[] = {{&S35556[44], 30}, {NULL}};
const Segment SGI72478[] = {{&S35556[30], 6}, {NULL}};
const Segment SGI72479[] = {{&S28815[42], 10}, {NULL}};
const Segment SGI72480[] = {{&S28815[42], 21}, {NULL}};
const Segment SGI72481[] = {{&S28815[30], 4}, {NULL}};
const Segment SGI72482[] = {{&S35556[124], 3}, {NULL}};
const Segment SGI72483[] = {{&S35556[88], 7}, {NULL}};
const Segment SGI72484[] = {{&S28815[113], 3}, {NULL}};
const Segment SGI72485[] = {{&S35556[144], 8}, {NULL}};
const Segment SGI72486[] = {{&S28815[77], 7}, {NULL}};
const Segment SGI72487[] = {{&S28815[133], 8}, {NULL}};
const Segment SGI72488[] = {{&S35557[44], 13}, {NULL}};
const Segment SGI72489[] = {{&S28816[42], 10}, {NULL}};
const Segment SGI72490[] = {{&S35557[44], 30}, {NULL}};
const Segment SGI72491[] = {{&S28816[42], 27}, {NULL}};
const Segment SGI72492[] = {{&S35557[30], 6}, {NULL}};
const Segment SGI72493[] = {{&S28816[30], 4}, {NULL}};
const Segment SGI72494[] = {{&S35557[122], 3}, {NULL}};
const Segment SGI72495[] = {{&S35557[88], 5}, {NULL}};
const Segment SGI72496[] = {{&S28817[42], 9}, {NULL}};
const Segment SGI72497[] = {{&S28817[30], 4}, {NULL}};
const Segment SGI72498[] = {{&S35558[38], 13}, {NULL}};
const Segment SGI72499[] = {{&S35558[31], 6}, {NULL}};
const Segment SGI72500[] = {{&S28818[42], 10}, {NULL}};
const Segment SGI72501[] = {{&S28818[42], 21}, {NULL}};
const Segment SGI72502[] = {{&S28818[30], 4}, {NULL}};
const Segment SGI72503[] = {{&S35559[38], 10}, {NULL}};
const Segment SGI72504[] = {{&S35559[31], 6}, {NULL}};
const Segment SGI72505[] = {{&S28818[80], 6}, {NULL}};
const Segment SGI72506[] = {{&S35560[42], 13}, {NULL}};
const Segment SGI72507[] = {{&S28819[42], 10}, {NULL}};
const Segment SGI72508[] = {{&S28819[42], 27}, {NULL}};
const Segment SGI72509[] = {{&S35560[30], 6}, {NULL}};
const Segment SGI72510[] = {{&S28819[30], 4}, {NULL}};
const Segment SGI72511[] = {{&S35560[120], 3}, {NULL}};
const Segment SGI72512[] = {{&S35560[86], 5}, {NULL}};
const Segment SGI72513[] = {{&S28820[42], 10}, {NULL}};
const Segment SGI72514[] = {{&S35561[44], 10}, {NULL}};
const Segment SGI72515[] = {{&S28820[42], 21}, {NULL}};
const Segment SGI72516[] = {{&S35561[44], 27}, {NULL}};
const Segment SGI72517[] = {{&S28820[30], 4}, {NULL}};
const Segment SGI72518[] = {{&S35561[30], 6}, {NULL}};
const Segment SGI72519[] = {{&S28820[112], 3}, {NULL}};
const Segment SGI72520[] = {{&S35561[121], 3}, {NULL}};
const Segment SGI72521[] = {{&S28820[77], 6}, {NULL}};
const Segment SGI72522[] = {{&S35561[85], 7}, {NULL}};
const Segment SGI72523[] = {{&S35561[141], 8}, {NULL}};
const Segment SGI72524[] = {{&S35562[42], 10}, {NULL}};
const Segment SGI72525[] = {{&S28821[42], 10}, {NULL}};
const Segment SGI72526[] = {{&S35562[30], 6}, {NULL}};
const Segment SGI72527[] = {{&S28821[42], 21}, {NULL}};
const Segment SGI72528[] = {{&S28821[30], 4}, {NULL}};
const Segment SGI72529[] = {{&S35562[117], 3}, {NULL}};
const Segment SGI72530[] = {{&S35562[83], 5}, {NULL}};
const Segment SGI72531[] = {{&S28821[111], 3}, {NULL}};
const Segment SGI72532[] = {{&S28821[77], 5}, {NULL}};
const Segment SGI72533[] = {{&S28822[42], 10}, {NULL}};
const Segment SGI72534[] = {{&S28822[42], 27}, {NULL}};
const Segment SGI72535[] = {{&S28822[30], 4}, {NULL}};
const Segment SGI72536[] = {{&S28823[42], 10}, {NULL}};
const Segment SGI72537[] = {{&S28823[42], 21}, {&S28823[205], 7}, {NULL}};
const Segment SGI72538[] = {{&S28823[30], 4}, {NULL}};
const Segment SGI72539[] = {{&S28823[112], 3}, {NULL}};
const Segment SGI72540[] = {{&S28823[77], 6}, {NULL}};
const Segment SGI72541[] = {{&S28824[32], 10}, {NULL}};
const Segment SGI72542[] = {{&S28824[32], 21}, {NULL}};
const Segment SGI72543[] = {{&S28826[42], 10}, {NULL}};
const Segment SGI72544[] = {{&S28826[42], 21}, {&S28826[204], 7}, {NULL}};
const Segment SGI72545[] = {{&S28826[30], 4}, {NULL}};
const Segment SGI72546[] = {{&S28826[112], 3}, {NULL}};
const Segment SGI72547[] = {{&S28826[77], 6}, {NULL}};
const Segment SGI72548[] = {{&S28827[42], 10}, {NULL}};
const Segment SGI72549[] = {{&S28827[42], 27}, {NULL}};
const Segment SGI72550[] = {{&S28827[30], 4}, {NULL}};
const Segment SGI72551[] = {{&S28828[51], 18}, {NULL}};
const Segment SGI72552[] = {{&S28828[46], 4}, {NULL}};
const Segment SGI72553[] = {{&S28829[63], 10}, {NULL}};
const Segment SGI72554[] = {{&S35578[44], 10}, {NULL}};
const Segment SGI72555[] = {{&S28829[63], 21}, {NULL}};
const Segment SGI72556[] = {{&S35578[44], 27}, {NULL}};
const Segment SGI72557[] = {{&S28829[51], 4}, {NULL}};
const Segment SGI72558[] = {{&S35578[30], 6}, {NULL}};
const Segment SGI72559[] = {{&S35578[121], 3}, {NULL}};
const Segment SGI72560[] = {{&S35578[85], 7}, {NULL}};
const Segment SGI72561[] = {{&S35578[141], 8}, {NULL}};
const Segment SGI72562[] = {{&S28830[43], 10}, {NULL}};
const Segment SGI72563[] = {{&S28830[43], 27}, {NULL}};
const Segment SGI72564[] = {{&S28830[31], 4}, {NULL}};
const Segment SGI72565[] = {{&S35579[38], 10}, {NULL}};
const Segment SGI72566[] = {{&S35579[31], 6}, {NULL}};
const Segment SGI72567[] = {{&S28831[42], 10}, {NULL}};
const Segment SGI72568[] = {{&S28831[42], 27}, {NULL}};
const Segment SGI72569[] = {{&S28831[30], 4}, {NULL}};
const Segment SGI72570[] = {{&S35581[44], 10}, {NULL}};
const Segment SGI72571[] = {{&S35581[44], 27}, {NULL}};
const Segment SGI72572[] = {{&S35581[30], 6}, {NULL}};
const Segment SGI72573[] = {{&S35581[119], 3}, {NULL}};
const Segment SGI72574[] = {{&S35581[85], 5}, {NULL}};
const Segment SGI72575[] = {{&S28832[42], 10}, {NULL}};
const Segment SGI72576[] = {{&S28832[42], 21}, {NULL}};
const Segment SGI72577[] = {{&S28832[30], 4}, {NULL}};
const Segment SGI72578[] = {{&S28832[112], 3}, {&S28832[153], 3}, {NULL}};
const Segment SGI72579[] = {{&S28832[77], 6}, {NULL}};
const Segment SGI72580[] = {{&S28834[97], 29}, {NULL}};
const Segment SGI72581[] = {{&S28836[41], 10}, {NULL}};
const Segment SGI72582[] = {{&S28836[41], 21}, {NULL}};
const Segment SGI72583[] = {{&S28836[111], 3}, {NULL}};
const Segment SGI72584[] = {{&S28836[76], 6}, {NULL}};
const Segment SGI72585[] = {{&S35589[42], 10}, {NULL}};
const Segment SGI72586[] = {{&S35589[30], 6}, {NULL}};
const Segment SGI72587[] = {{&S35589[117], 3}, {NULL}};
const Segment SGI72588[] = {{&S35589[83], 5}, {NULL}};
const Segment SGI72589[] = {{&S28837[40], 10}, {NULL}};
const Segment SGI72590[] = {{&S28837[30], 4}, {NULL}};
const Segment SGI72591[] = {{&S28837[111], 3}, {NULL}};
const Segment SGI72592[] = {{&S28837[75], 7}, {NULL}};
const Segment SGI72593[] = {{&S28837[131], 8}, {NULL}};
const Segment SGI72594[] = {{&S35590[44], 10}, {NULL}};
const Segment SGI72595[] = {{&S35590[44], 30}, {NULL}};
const Segment SGI72596[] = {{&S35590[30], 6}, {NULL}};
const Segment SGI72597[] = {{&S35590[122], 3}, {NULL}};
const Segment SGI72598[] = {{&S35590[88], 5}, {NULL}};
const Segment SGI72599[] = {{&S35719[44], 7}, {NULL}};
const Segment SGI72600[] = {{&S35719[44], 24}, {NULL}};
const Segment SGI72601[] = {{&S35719[30], 6}, {NULL}};
const Segment SGI72602[] = {{&S35719[116], 3}, {NULL}};
const Segment SGI72603[] = {{&S35719[82], 5}, {NULL}};
const Segment SGI72604[] = {{&S35723[44], 9}, {NULL}};
const Segment SGI72605[] = {{&S35723[44], 26}, {NULL}};
const Segment SGI72606[] = {{&S35723[30], 6}, {NULL}};
const Segment SGI72607[] = {{&S35723[118], 3}, {NULL}};
const Segment SGI72608[] = {{&S35723[84], 5}, {NULL}};
const Segment SGI72609[] = {{&S35731[44], 7}, {NULL}};
const Segment SGI72610[] = {{&S35731[44], 28}, {NULL}};
const Segment SGI72611[] = {{&S35731[30], 6}, {NULL}};
const Segment SGI72612[] = {{&S35731[120], 3}, {NULL}};
const Segment SGI72613[] = {{&S35731[86], 5}, {NULL}};
const Segment SGI72614[] = {{&S35741[44], 7}, {NULL}};
const Segment SGI72615[] = {{&S35741[44], 28}, {NULL}};
const Segment SGI72616[] = {{&S35741[30], 6}, {NULL}};
const Segment SGI72617[] = {{&S35741[120], 3}, {NULL}};
const Segment SGI72618[] = {{&S35741[86], 5}, {NULL}};
const Segment SGI72619[] = {{&S18964[44], 11}, {NULL}};
const Segment SGI72620[] = {{&S18964[44], 23}, {NULL}};
const Segment SGI72621[] = {{&S18964[30], 6}, {NULL}};
const Segment SGI72622[] = {{&S18964[31], 3}, {NULL}};
const Segment SGI72623[] = {{&S18964[81], 6}, {NULL}};
const Segment SGI72624[] = {{&S18965[38], 11}, {NULL}};
const Segment SGI72625[] = {{&S18965[31], 6}, {NULL}};
const Segment SGI72626[] = {{&S35745[38], 7}, {NULL}};
const Segment SGI72627[] = {{&S35745[31], 6}, {NULL}};
const Segment SGI72628[] = {{&S18966[44], 7}, {NULL}};
const Segment SGI72629[] = {{&S18966[44], 19}, {NULL}};
const Segment SGI72630[] = {{&S18966[30], 6}, {NULL}};
const Segment SGI72631[] = {{&S18966[31], 3}, {NULL}};
const Segment SGI72632[] = {{&S18966[77], 6}, {NULL}};
const Segment SGI72633[] = {{&S18967[44], 22}, {NULL}};
const Segment SGI72634[] = {{&S18967[44], 34}, {NULL}};
const Segment SGI72635[] = {{&S18967[30], 6}, {NULL}};
const Segment SGI72636[] = {{&S35747[44], 7}, {NULL}};
const Segment SGI72637[] = {{&S35747[44], 28}, {NULL}};
const Segment SGI72638[] = {{&S35747[30], 6}, {NULL}};
const Segment SGI72639[] = {{&S18967[31], 3}, {NULL}};
const Segment SGI72640[] = {{&S18967[92], 6}, {NULL}};
const Segment SGI72641[] = {{&S35747[120], 3}, {NULL}};
const Segment SGI72642[] = {{&S35747[86], 5}, {NULL}};
const Segment SGI72643[] = {{&S18968[44], 11}, {NULL}};
const Segment SGI72644[] = {{&S18968[44], 23}, {NULL}};
const Segment SGI72645[] = {{&S18968[30], 6}, {NULL}};
const Segment SGI72646[] = {{&S18968[84], 4}, {NULL}};
const Segment SGI72647[] = {{&S18968[84], 3}, {NULL}};
const Segment SGI72648[] = {{&S18969[44], 15}, {NULL}};
const Segment SGI72649[] = {{&S18969[44], 27}, {NULL}};
const Segment SGI72650[] = {{&S18969[30], 6}, {NULL}};
const Segment SGI72651[] = {{&S18969[31], 3}, {NULL}};
const Segment SGI72652[] = {{&S18969[85], 6}, {NULL}};
const Segment SGI72653[] = {{&S18970[44], 14}, {NULL}};
const Segment SGI72654[] = {{&S18970[44], 26}, {NULL}};
const Segment SGI72655[] = {{&S18970[30], 6}, {NULL}};
const Segment SGI72656[] = {{&S18970[31], 3}, {NULL}};
const Segment SGI72657[] = {{&S18970[84], 6}, {NULL}};
const Segment SGI72658[] = {{&S18971[44], 22}, {NULL}};
const Segment SGI72659[] = {{&S18971[44], 34}, {NULL}};
const Segment SGI72660[] = {{&S18971[30], 6}, {NULL}};
const Segment SGI72661[] = {{&S18971[31], 3}, {NULL}};
const Segment SGI72662[] = {{&S18971[92], 6}, {NULL}};
const Segment SGI72663[] = {{&S18972[44], 21}, {NULL}};
const Segment SGI72664[] = {{&S18972[44], 33}, {NULL}};
const Segment SGI72665[] = {{&S18972[30], 6}, {NULL}};
const Segment SGI72666[] = {{&S18972[31], 3}, {NULL}};
const Segment SGI72667[] = {{&S18972[91], 6}, {NULL}};
const Segment SGI72668[] = {{&S18973[44], 11}, {NULL}};
const Segment SGI72669[] = {{&S18973[44], 23}, {NULL}};
const Segment SGI72670[] = {{&S18973[30], 6}, {NULL}};
const Segment SGI72671[] = {{&S18973[31], 3}, {NULL}};
const Segment SGI72672[] = {{&S18973[81], 6}, {NULL}};
const Segment SGI72673[] = {{&S18974[44], 11}, {NULL}};
const Segment SGI72674[] = {{&S18974[44], 23}, {NULL}};
const Segment SGI72675[] = {{&S18974[30], 6}, {NULL}};
const Segment SGI72676[] = {{&S18974[31], 3}, {NULL}};
const Segment SGI72677[] = {{&S18974[81], 6}, {NULL}};
const Segment SGI72678[] = {{&S18975[44], 22}, {NULL}};
const Segment SGI72679[] = {{&S18975[44], 34}, {NULL}};
const Segment SGI72680[] = {{&S18975[30], 6}, {NULL}};
const Segment SGI72681[] = {{&S18975[31], 3}, {NULL}};
const Segment SGI72682[] = {{&S18975[92], 6}, {NULL}};
const Segment SGI72683[] = {{&S18976[44], 22}, {NULL}};
const Segment SGI72684[] = {{&S18976[44], 34}, {NULL}};
const Segment SGI72685[] = {{&S18976[30], 6}, {NULL}};
const Segment SGI72686[] = {{&S18976[31], 3}, {NULL}};
const Segment SGI72687[] = {{&S18976[92], 6}, {NULL}};
const Segment SGI72688[] = {{&S18977[44], 22}, {NULL}};
const Segment SGI72689[] = {{&S18977[44], 34}, {NULL}};
const Segment SGI72690[] = {{&S18977[30], 6}, {NULL}};
const Segment SGI72691[] = {{&S18977[31], 3}, {NULL}};
const Segment SGI72692[] = {{&S18977[92], 6}, {NULL}};
const Segment SGI72693[] = {{&S18978[44], 11}, {NULL}};
const Segment SGI72694[] = {{&S18978[44], 23}, {NULL}};
const Segment SGI72695[] = {{&S18978[30], 6}, {NULL}};
const Segment SGI72696[] = {{&S18978[84], 6}, {NULL}};
const Segment SGI72697[] = {{&S18979[44], 22}, {NULL}};
const Segment SGI72698[] = {{&S18979[44], 34}, {NULL}};
const Segment SGI72699[] = {{&S18979[30], 6}, {NULL}};
const Segment SGI72700[] = {{&S18979[31], 3}, {NULL}};
const Segment SGI72701[] = {{&S18979[92], 6}, {NULL}};
const Segment SGI72702[] = {{&S18980[44], 22}, {NULL}};
const Segment SGI72703[] = {{&S18980[44], 34}, {NULL}};
const Segment SGI72704[] = {{&S18980[30], 6}, {NULL}};
const Segment SGI72705[] = {{&S18980[31], 3}, {NULL}};
const Segment SGI72706[] = {{&S18980[92], 6}, {NULL}};
const Segment SGI72707[] = {{&S18981[44], 22}, {NULL}};
const Segment SGI72708[] = {{&S18981[44], 34}, {NULL}};
const Segment SGI72709[] = {{&S18981[30], 6}, {NULL}};
const Segment SGI72710[] = {{&S18981[31], 3}, {NULL}};
const Segment SGI72711[] = {{&S18981[92], 6}, {NULL}};
const Segment SGI72712[] = {{&S18982[44], 22}, {NULL}};
const Segment SGI72713[] = {{&S18982[44], 34}, {NULL}};
const Segment SGI72714[] = {{&S18982[30], 6}, {NULL}};
const Segment SGI72715[] = {{&S18982[31], 3}, {NULL}};
const Segment SGI72716[] = {{&S18982[92], 6}, {NULL}};
const Segment SGI72717[] = {{&S29712[44], 11}, {NULL}};
const Segment SGI72718[] = {{&S29712[44], 23}, {NULL}};
const Segment SGI72719[] = {{&S29712[30], 6}, {NULL}};
const Segment SGI72720[] = {{&S29712[115], 3}, {NULL}};
const Segment SGI72721[] = {{&S18983[44], 22}, {NULL}};
const Segment SGI72722[] = {{&S18983[44], 34}, {NULL}};
const Segment SGI72723[] = {{&S18983[30], 6}, {NULL}};
const Segment SGI72724[] = {{&S29712[81], 5}, {NULL}};
const Segment SGI72725[] = {{&S18983[31], 3}, {NULL}};
const Segment SGI72726[] = {{&S18983[92], 6}, {NULL}};
const Segment SGI72727[] = {{&S29713[44], 11}, {NULL}};
const Segment SGI72728[] = {{&S29713[44], 23}, {NULL}};
const Segment SGI72729[] = {{&S29713[30], 6}, {NULL}};
const Segment SGI72730[] = {{&S18984[44], 22}, {NULL}};
const Segment SGI72731[] = {{&S18984[44], 34}, {NULL}};
const Segment SGI72732[] = {{&S29713[115], 3}, {NULL}};
const Segment SGI72733[] = {{&S18984[30], 6}, {NULL}};
const Segment SGI72734[] = {{&S29713[81], 5}, {NULL}};
const Segment SGI72735[] = {{&S18984[31], 3}, {NULL}};
const Segment SGI72736[] = {{&S18984[92], 6}, {NULL}};
const Segment SGI72737[] = {{&S29714[44], 7}, {NULL}};
const Segment SGI72738[] = {{&S29714[44], 19}, {NULL}};
const Segment SGI72739[] = {{&S29714[30], 6}, {NULL}};
const Segment SGI72740[] = {{&S18985[44], 22}, {NULL}};
const Segment SGI72741[] = {{&S18985[44], 34}, {NULL}};
const Segment SGI72742[] = {{&S29714[111], 3}, {NULL}};
const Segment SGI72743[] = {{&S18985[30], 6}, {NULL}};
const Segment SGI72744[] = {{&S29714[77], 5}, {NULL}};
const Segment SGI72745[] = {{&S18985[31], 3}, {NULL}};
const Segment SGI72746[] = {{&S18985[92], 6}, {NULL}};
const Segment SGI72747[] = {{&S29715[44], 7}, {NULL}};
const Segment SGI72748[] = {{&S29715[44], 19}, {NULL}};
const Segment SGI72749[] = {{&S29715[30], 6}, {NULL}};
const Segment SGI72750[] = {{&S18986[44], 22}, {NULL}};
const Segment SGI72751[] = {{&S29715[111], 3}, {NULL}};
const Segment SGI72752[] = {{&S18986[44], 34}, {NULL}};
const Segment SGI72753[] = {{&S18986[30], 6}, {NULL}};
const Segment SGI72754[] = {{&S29715[77], 5}, {NULL}};
const Segment SGI72755[] = {{&S18986[31], 3}, {NULL}};
const Segment SGI72756[] = {{&S18986[92], 6}, {NULL}};
const Segment SGI72757[] = {{&S29716[54], 7}, {NULL}};
const Segment SGI72758[] = {{&S29716[54], 19}, {NULL}};
const Segment SGI72759[] = {{&S29716[40], 6}, {NULL}};
const Segment SGI72760[] = {{&S29716[121], 3}, {NULL}};
const Segment SGI72761[] = {{&S18987[44], 22}, {NULL}};
const Segment SGI72762[] = {{&S18987[44], 34}, {NULL}};
const Segment SGI72763[] = {{&S29716[87], 5}, {NULL}};
const Segment SGI72764[] = {{&S18987[30], 6}, {NULL}};
const Segment SGI72765[] = {{&S18987[31], 3}, {NULL}};
const Segment SGI72766[] = {{&S18987[92], 6}, {NULL}};
const Segment SGI72767[] = {{&S29717[44], 11}, {NULL}};
const Segment SGI72768[] = {{&S29717[44], 23}, {NULL}};
const Segment SGI72769[] = {{&S29717[30], 6}, {NULL}};
const Segment SGI72770[] = {{&S29717[115], 3}, {NULL}};
const Segment SGI72771[] = {{&S29717[81], 5}, {NULL}};
const Segment SGI72772[] = {{&S18989[44], 22}, {NULL}};
const Segment SGI72773[] = {{&S18989[44], 34}, {NULL}};
const Segment SGI72774[] = {{&S18989[30], 6}, {NULL}};
const Segment SGI72775[] = {{&S29718[44], 7}, {NULL}};
const Segment SGI72776[] = {{&S18989[31], 3}, {NULL}};
const Segment SGI72777[] = {{&S29718[44], 19}, {NULL}};
const Segment SGI72778[] = {{&S29718[30], 6}, {NULL}};
const Segment SGI72779[] = {{&S18989[92], 6}, {NULL}};
const Segment SGI72780[] = {{&S29718[111], 3}, {NULL}};
const Segment SGI72781[] = {{&S29718[77], 5}, {NULL}};
const Segment SGI72782[] = {{&S29719[44], 7}, {NULL}};
const Segment SGI72783[] = {{&S29719[44], 19}, {NULL}};
const Segment SGI72784[] = {{&S29719[30], 6}, {NULL}};
const Segment SGI72785[] = {{&S29719[111], 3}, {NULL}};
const Segment SGI72786[] = {{&S29719[77], 5}, {NULL}};
const Segment SGI72787[] = {{&S29720[44], 7}, {NULL}};
const Segment SGI72788[] = {{&S29720[44], 19}, {NULL}};
const Segment SGI72789[] = {{&S29720[30], 6}, {NULL}};
const Segment SGI72790[] = {{&S29720[111], 3}, {NULL}};
const Segment SGI72791[] = {{&S29720[77], 5}, {NULL}};
const Segment SGI72792[] = {{&S29721[44], 7}, {NULL}};
const Segment SGI72793[] = {{&S29721[44], 19}, {NULL}};
const Segment SGI72794[] = {{&S29721[30], 6}, {NULL}};
const Segment SGI72795[] = {{&S29721[111], 3}, {NULL}};
const Segment SGI72796[] = {{&S29721[77], 5}, {NULL}};
const Segment SGI72797[] = {{&S29723[44], 7}, {NULL}};
const Segment SGI72798[] = {{&S29723[44], 19}, {NULL}};
const Segment SGI72799[] = {{&S29723[30], 6}, {NULL}};
const Segment SGI72800[] = {{&S29723[111], 3}, {NULL}};
const Segment SGI72801[] = {{&S29723[77], 5}, {NULL}};
const Segment SGI72802[] = {{&S29724[44], 7}, {NULL}};
const Segment SGI72803[] = {{&S29724[44], 19}, {NULL}};
const Segment SGI72804[] = {{&S29724[30], 6}, {NULL}};
const Segment SGI72805[] = {{&S29724[111], 3}, {NULL}};
const Segment SGI72806[] = {{&S29724[77], 5}, {NULL}};
const Segment SGI72807[] = {{&S29725[44], 7}, {NULL}};
const Segment SGI72808[] = {{&S29725[44], 19}, {NULL}};
const Segment SGI72809[] = {{&S29725[30], 6}, {NULL}};
const Segment SGI72810[] = {{&S29725[111], 3}, {NULL}};
const Segment SGI72811[] = {{&S29725[77], 5}, {NULL}};
const Segment SGI72812[] = {{&S29726[44], 11}, {NULL}};
const Segment SGI72813[] = {{&S29726[44], 23}, {NULL}};
const Segment SGI72814[] = {{&S29726[30], 6}, {NULL}};
const Segment SGI72815[] = {{&S29726[115], 3}, {NULL}};
const Segment SGI72816[] = {{&S29726[81], 5}, {NULL}};
const Segment SGI72817[] = {{&S29728[44], 11}, {NULL}};
const Segment SGI72818[] = {{&S29728[44], 23}, {NULL}};
const Segment SGI72819[] = {{&S29728[30], 6}, {NULL}};
const Segment SGI72820[] = {{&S29728[115], 3}, {NULL}};
const Segment SGI72821[] = {{&S29728[81], 5}, {NULL}};
const Segment SGI72822[] = {{&S29730[44], 7}, {NULL}};
const Segment SGI72823[] = {{&S29730[44], 19}, {NULL}};
const Segment SGI72824[] = {{&S29730[30], 6}, {NULL}};
const Segment SGI72825[] = {{&S29730[111], 3}, {NULL}};
const Segment SGI72826[] = {{&S29730[77], 5}, {NULL}};
const Segment SGI72827[] = {{&S29731[44], 7}, {NULL}};
const Segment SGI72828[] = {{&S29731[44], 19}, {NULL}};
const Segment SGI72829[] = {{&S29731[30], 6}, {NULL}};
const Segment SGI72830[] = {{&S29731[80], 4}, {NULL}};
const Segment SGI72831[] = {{&S29731[80], 3}, {NULL}};
const Segment SGI72832[] = {{&S29732[44], 7}, {NULL}};
const Segment SGI72833[] = {{&S29732[44], 19}, {NULL}};
const Segment SGI72834[] = {{&S29732[30], 6}, {NULL}};
const Segment SGI72835[] = {{&S29732[111], 3}, {NULL}};
const Segment SGI72836[] = {{&S29732[77], 5}, {NULL}};
const Segment SGI72837[] = {{&S29733[44], 7}, {NULL}};
const Segment SGI72838[] = {{&S29733[44], 19}, {NULL}};
const Segment SGI72839[] = {{&S29733[30], 6}, {NULL}};
const Segment SGI72840[] = {{&S29733[80], 6}, {NULL}};
const Segment SGI72841[] = {{&S29735[44], 7}, {NULL}};
const Segment SGI72842[] = {{&S29735[44], 19}, {NULL}};
const Segment SGI72843[] = {{&S29735[30], 6}, {NULL}};
const Segment SGI72844[] = {{&S29735[111], 3}, {NULL}};
const Segment SGI72845[] = {{&S29735[77], 5}, {NULL}};
const Segment SGI72846[] = {{&S29736[44], 7}, {NULL}};
const Segment SGI72847[] = {{&S29736[44], 19}, {NULL}};
const Segment SGI72848[] = {{&S29736[30], 6}, {NULL}};
const Segment SGI72849[] = {{&S29736[111], 3}, {NULL}};
const Segment SGI72850[] = {{&S29736[77], 5}, {NULL}};
const Segment SGI72851[] = {{&S29737[38], 7}, {NULL}};
const Segment SGI72852[] = {{&S29737[31], 6}, {NULL}};
const Segment SGI72853[] = {{&S29742[54], 7}, {NULL}};
const Segment SGI72854[] = {{&S29742[54], 19}, {NULL}};
const Segment SGI72855[] = {{&S29742[40], 6}, {NULL}};
const Segment SGI72856[] = {{&S29742[121], 3}, {NULL}};
const Segment SGI72857[] = {{&S29742[87], 5}, {NULL}};
const Segment SGI72858[] = {{&S29743[44], 7}, {NULL}};
const Segment SGI72859[] = {{&S29743[44], 19}, {NULL}};
const Segment SGI72860[] = {{&S29743[30], 6}, {NULL}};
const Segment SGI72861[] = {{&S29743[80], 6}, {NULL}};
const Segment SGI72862[] = {{&S29746[44], 7}, {NULL}};
const Segment SGI72863[] = {{&S29746[44], 19}, {NULL}};
const Segment SGI72864[] = {{&S29746[30], 6}, {NULL}};
const Segment SGI72865[] = {{&S29746[111], 3}, {NULL}};
const Segment SGI72866[] = {{&S29746[77], 5}, {NULL}};
const Segment SGI72867[] = {{&S29749[44], 7}, {NULL}};
const Segment SGI72868[] = {{&S29749[44], 19}, {NULL}};
const Segment SGI72869[] = {{&S29749[30], 6}, {NULL}};
const Segment SGI72870[] = {{&S29749[111], 3}, {NULL}};
const Segment SGI72871[] = {{&S29749[77], 5}, {NULL}};
const Segment SGI72872[] = {{&S29750[54], 7}, {NULL}};
const Segment SGI72873[] = {{&S29750[54], 19}, {NULL}};
const Segment SGI72874[] = {{&S29750[40], 6}, {NULL}};
const Segment SGI72875[] = {{&S29750[121], 3}, {NULL}};
const Segment SGI72876[] = {{&S29750[87], 5}, {NULL}};
const Segment SGI72877[] = {{&S29751[44], 11}, {NULL}};
const Segment SGI72878[] = {{&S29751[44], 23}, {NULL}};
const Segment SGI72879[] = {{&S29751[30], 6}, {NULL}};
const Segment SGI72880[] = {{&S29751[115], 3}, {NULL}};
const Segment SGI72881[] = {{&S29751[81], 5}, {NULL}};
const Segment SGI72882[] = {{&S29752[44], 7}, {NULL}};
const Segment SGI72883[] = {{&S29752[44], 19}, {NULL}};
const Segment SGI72884[] = {{&S29752[30], 6}, {NULL}};
const Segment SGI72885[] = {{&S29752[80], 6}, {NULL}};
const Segment SGI72886[] = {{&S29753[44], 7}, {NULL}};
const Segment SGI72887[] = {{&S29753[44], 19}, {NULL}};
const Segment SGI72888[] = {{&S29753[30], 6}, {NULL}};
const Segment SGI72889[] = {{&S29753[111], 3}, {NULL}};
const Segment SGI72890[] = {{&S29753[77], 5}, {NULL}};
const Segment SGI72891[] = {{&S29755[44], 7}, {NULL}};
const Segment SGI72892[] = {{&S29755[44], 19}, {NULL}};
const Segment SGI72893[] = {{&S29755[30], 6}, {NULL}};
const Segment SGI72894[] = {{&S18990[44], 22}, {NULL}};
const Segment SGI72895[] = {{&S18990[44], 34}, {NULL}};
const Segment SGI72896[] = {{&S18990[30], 6}, {NULL}};
const Segment SGI72897[] = {{&S29755[80], 6}, {NULL}};
const Segment SGI72898[] = {{&S18990[31], 3}, {NULL}};
const Segment SGI72899[] = {{&S18990[92], 6}, {NULL}};
const Segment SGI72900[] = {{&S18991[42], 11}, {NULL}};
const Segment SGI72901[] = {{&S18991[30], 6}, {NULL}};
const Segment SGI72902[] = {{&S18991[31], 3}, {NULL}};
const Segment SGI72903[] = {{&S18991[79], 6}, {NULL}};
const Segment SGI72904[] = {{&S18992[44], 11}, {NULL}};
const Segment SGI72905[] = {{&S18992[44], 23}, {NULL}};
const Segment SGI72906[] = {{&S18992[30], 6}, {NULL}};
const Segment SGI72907[] = {{&S29759[44], 7}, {NULL}};
const Segment SGI72908[] = {{&S18992[31], 3}, {NULL}};
const Segment SGI72909[] = {{&S29759[44], 19}, {NULL}};
const Segment SGI72910[] = {{&S18992[81], 6}, {NULL}};
const Segment SGI72911[] = {{&S29759[30], 6}, {NULL}};
const Segment SGI72912[] = {{&S29759[111], 3}, {NULL}};
const Segment SGI72913[] = {{&S29759[77], 5}, {NULL}};
const Segment SGI72914[] = {{&S18994[42], 11}, {NULL}};
const Segment SGI72915[] = {{&S18994[30], 6}, {NULL}};
const Segment SGI72916[] = {{&S18994[31], 3}, {NULL}};
const Segment SGI72917[] = {{&S18994[79], 6}, {NULL}};
const Segment SGI72918[] = {{&S18996[44], 9}, {NULL}};
const Segment SGI72919[] = {{&S18996[44], 21}, {NULL}};
const Segment SGI72920[] = {{&S18996[30], 6}, {NULL}};
const Segment SGI72921[] = {{&S18996[31], 3}, {NULL}};
const Segment SGI72922[] = {{&S18996[79], 6}, {NULL}};
const Segment SGI72923[] = {{&S18997[44], 22}, {NULL}};
const Segment SGI72924[] = {{&S18997[44], 34}, {NULL}};
const Segment SGI72925[] = {{&S18997[30], 6}, {NULL}};
const Segment SGI72926[] = {{&S18997[31], 3}, {NULL}};
const Segment SGI72927[] = {{&S18997[92], 6}, {NULL}};
const Segment SGI72928[] = {{&S18998[44], 22}, {NULL}};
const Segment SGI72929[] = {{&S18998[44], 34}, {NULL}};
const Segment SGI72930[] = {{&S18998[30], 6}, {NULL}};
const Segment SGI72931[] = {{&S18998[31], 3}, {NULL}};
const Segment SGI72932[] = {{&S18998[92], 6}, {NULL}};
const Segment SGI72933[] = {{&S35967[57], 14}, {NULL}};
const Segment SGI72934[] = {{&S35967[57], 25}, {NULL}};
const Segment SGI72935[] = {{&S35967[43], 6}, {NULL}};
const Segment SGI72936[] = {{&S35967[130], 3}, {NULL}};
const Segment SGI72937[] = {{&S35967[96], 5}, {NULL}};
const Segment SGI72938[] = {{&S35970[44], 9}, {NULL}};
const Segment SGI72939[] = {{&S35970[44], 20}, {NULL}};
const Segment SGI72940[] = {{&S35970[30], 6}, {NULL}};
const Segment SGI72941[] = {{&S35970[112], 3}, {NULL}};
const Segment SGI72942[] = {{&S35970[78], 5}, {NULL}};
const Segment SGI72943[] = {{&S35971[44], 10}, {NULL}};
const Segment SGI72944[] = {{&S35971[44], 21}, {NULL}};
const Segment SGI72945[] = {{&S35971[30], 6}, {NULL}};
const Segment SGI72946[] = {{&S35971[113], 3}, {NULL}};
const Segment SGI72947[] = {{&S35971[79], 5}, {NULL}};
const Segment SGI72948[] = {{&S35973[38], 9}, {NULL}};
const Segment SGI72949[] = {{&S35973[31], 6}, {NULL}};
const Segment SGI72950[] = {{&S35975[44], 20}, {NULL}};
const Segment SGI72951[] = {{&S35975[44], 31}, {NULL}};
const Segment SGI72952[] = {{&S35975[30], 6}, {NULL}};
const Segment SGI72953[] = {{&S35975[123], 3}, {NULL}};
const Segment SGI72954[] = {{&S35975[89], 5}, {NULL}};
const Segment SGI72955[] = {{&S35976[57], 14}, {NULL}};
const Segment SGI72956[] = {{&S35976[57], 25}, {NULL}};
const Segment SGI72957[] = {{&S35976[43], 6}, {NULL}};
const Segment SGI72958[] = {{&S35976[130], 3}, {NULL}};
const Segment SGI72959[] = {{&S35976[96], 5}, {NULL}};
const Segment SGI72960[] = {{&S36008[44], 23}, {NULL}};
const Segment SGI72961[] = {{&S36008[44], 35}, {NULL}};
const Segment SGI72962[] = {{&S36008[30], 6}, {NULL}};
const Segment SGI72963[] = {{&S36008[127], 3}, {NULL}};
const Segment SGI72964[] = {{&S36008[93], 5}, {NULL}};
const Segment SGI72965[] = {{&S36009[44], 24}, {NULL}};
const Segment SGI72966[] = {{&S36009[44], 36}, {NULL}};
const Segment SGI72967[] = {{&S36009[30], 6}, {NULL}};
const Segment SGI72968[] = {{&S36009[128], 3}, {NULL}};
const Segment SGI72969[] = {{&S36009[94], 5}, {NULL}};
const Segment SGI72970[] = {{&S36010[44], 24}, {NULL}};
const Segment SGI72971[] = {{&S36010[44], 36}, {NULL}};
const Segment SGI72972[] = {{&S36010[30], 6}, {NULL}};
const Segment SGI72973[] = {{&S36010[128], 3}, {NULL}};
const Segment SGI72974[] = {{&S36010[94], 5}, {NULL}};
const Segment SGI72975[] = {{&S36011[44], 24}, {NULL}};
const Segment SGI72976[] = {{&S36011[44], 36}, {NULL}};
const Segment SGI72977[] = {{&S36011[30], 6}, {NULL}};
const Segment SGI72978[] = {{&S36011[128], 3}, {NULL}};
const Segment SGI72979[] = {{&S36011[94], 5}, {NULL}};
const Segment SGI72980[] = {{&S36012[44], 24}, {NULL}};
const Segment SGI72981[] = {{&S36012[44], 36}, {NULL}};
const Segment SGI72982[] = {{&S36012[30], 6}, {NULL}};
const Segment SGI72983[] = {{&S36012[128], 3}, {NULL}};
const Segment SGI72984[] = {{&S36012[94], 5}, {NULL}};
const Segment SGI72985[] = {{&S36014[44], 30}, {NULL}};
const Segment SGI72986[] = {{&S36014[44], 42}, {NULL}};
const Segment SGI72987[] = {{&S36014[30], 6}, {NULL}};
const Segment SGI72988[] = {{&S36014[134], 3}, {NULL}};
const Segment SGI72989[] = {{&S36014[100], 5}, {NULL}};
const Segment SGI72990[] = {{&S36015[44], 23}, {NULL}};
const Segment SGI72991[] = {{&S36015[44], 35}, {NULL}};
const Segment SGI72992[] = {{&S36015[30], 6}, {NULL}};
const Segment SGI72993[] = {{&S36015[127], 3}, {NULL}};
const Segment SGI72994[] = {{&S36015[93], 5}, {NULL}};
const Segment SGI72995[] = {{&S36017[42], 24}, {NULL}};
const Segment SGI72996[] = {{&S36017[30], 6}, {NULL}};
const Segment SGI72997[] = {{&S36017[126], 3}, {NULL}};
const Segment SGI72998[] = {{&S36017[92], 5}, {NULL}};
const Segment SGI72999[] = {{&S19204[44], 10}, {NULL}};
const Segment SGI73000[] = {{&S19204[44], 22}, {NULL}};
const Segment SGI73001[] = {{&S19204[30], 6}, {NULL}};
const Segment SGI73002[] = {{&S19204[31], 3}, {NULL}};
const Segment SGI73003[] = {{&S19204[80], 6}, {NULL}};
const Segment SGI73004[] = {{&S19205[44], 10}, {NULL}};
const Segment SGI73005[] = {{&S19205[44], 22}, {NULL}};
const Segment SGI73006[] = {{&S29966[44], 10}, {NULL}};
const Segment SGI73007[] = {{&S19205[30], 6}, {NULL}};
const Segment SGI73008[] = {{&S29966[44], 27}, {NULL}};
const Segment SGI73009[] = {{&S29966[30], 6}, {NULL}};
const Segment SGI73010[] = {{&S19205[31], 3}, {NULL}};
const Segment SGI73011[] = {{&S29966[119], 3}, {NULL}};
const Segment SGI73012[] = {{&S29966[85], 5}, {NULL}};
const Segment SGI73013[] = {{&S25477[43], 10}, {NULL}};
const Segment SGI73014[] = {{&S25477[43], 22}, {NULL}};
const Segment SGI73015[] = {{&S25477[30], 5}, {NULL}};
const Segment SGI73016[] = {{&S25477[31], 3}, {NULL}};
const Segment SGI73017[] = {{&S25477[79], 6}, {NULL}};
const Segment SGI73018[] = {{&S30011[44], 9}, {NULL}};
const Segment SGI73019[] = {{&S30011[44], 20}, {NULL}};
const Segment SGI73020[] = {{&S30011[30], 6}, {NULL}};
const Segment SGI73021[] = {{&S30011[112], 3}, {NULL}};
const Segment SGI73022[] = {{&S30011[78], 5}, {NULL}};
const Segment SGI73023[] = {{&S30012[44], 9}, {NULL}};
const Segment SGI73024[] = {{&S30012[44], 21}, {NULL}};
const Segment SGI73025[] = {{&S30012[30], 6}, {NULL}};
const Segment SGI73026[] = {{&S30012[113], 3}, {NULL}};
const Segment SGI73027[] = {{&S30012[79], 5}, {NULL}};
const Segment SGI73028[] = {{&S30014[44], 9}, {&S30014[66], 9}, {NULL}};
const Segment SGI73029[] = {{&S30014[44], 21}, {NULL}};
const Segment SGI73030[] = {{&S30014[30], 6}, {NULL}};
const Segment SGI73031[] = {{&S30014[134], 3}, {NULL}};
const Segment SGI73032[] = {{&S30014[100], 5}, {NULL}};
const Segment SGI73033[] = {{&S30019[38], 9}, {NULL}};
const Segment SGI73034[] = {{&S30019[31], 6}, {NULL}};
const Segment SGI73035[] = {{&S30020[44], 9}, {NULL}};
const Segment SGI73036[] = {{&S30020[44], 23}, {NULL}};
const Segment SGI73037[] = {{&S30020[30], 6}, {NULL}};
const Segment SGI73038[] = {{&S30020[115], 3}, {NULL}};
const Segment SGI73039[] = {{&S30020[81], 5}, {NULL}};
const Segment SGI73040[] = {{&S30028[44], 9}, {NULL}};
const Segment SGI73041[] = {{&S30028[44], 19}, {NULL}};
const Segment SGI73042[] = {{&S30028[30], 6}, {NULL}};
const Segment SGI73043[] = {{&S30028[111], 3}, {NULL}};
const Segment SGI73044[] = {{&S30028[77], 5}, {NULL}};
const Segment SGI73045[] = {{&S29516[42], 10}, {NULL}};
const Segment SGI73046[] = {{&S29516[42], 22}, {NULL}};
const Segment SGI73047[] = {{&S29516[30], 4}, {NULL}};
const Segment SGI73048[] = {{&S29516[113], 3}, {NULL}};
const Segment SGI73049[] = {{&S29516[78], 6}, {NULL}};
const Segment SGI73050[] = {{&S29517[42], 10}, {NULL}};
const Segment SGI73051[] = {{&S29517[42], 22}, {NULL}};
const Segment SGI73052[] = {{&S29517[30], 4}, {NULL}};
const Segment SGI73053[] = {{&S29517[81], 4}, {NULL}};
const Segment SGI73054[] = {{&S29517[81], 3}, {NULL}};
const Segment SGI73055[] = {{&S29518[36], 10}, {NULL}};
const Segment SGI73056[] = {{&S29518[31], 4}, {NULL}};
const Segment SGI73057[] = {{&S29519[42], 11}, {NULL}};
const Segment SGI73058[] = {{&S19205[80], 6}, {NULL}};
const Segment SGI73059[] = {{&S29519[42], 23}, {NULL}};
const Segment SGI73060[] = {{&S29519[30], 4}, {NULL}};
const Segment SGI73061[] = {{&S29519[114], 3}, {NULL}};
const Segment SGI73062[] = {{&S29519[79], 6}, {NULL}};
const Segment SGI73063[] = {{&S19206[44], 30}, {NULL}};
const Segment SGI73064[] = {{&S19206[44], 42}, {NULL}};
const Segment SGI73065[] = {{&S19206[30], 6}, {NULL}};
const Segment SGI73066[] = {{&S29520[36], 11}, {NULL}};
const Segment SGI73067[] = {{&S29520[31], 4}, {NULL}};
const Segment SGI73068[] = {{&S19206[31], 3}, {NULL}};
const Segment SGI73069[] = {{&S19206[100], 6}, {NULL}};
const Segment SGI73070[] = {{&S29521[42], 10}, {NULL}};
const Segment SGI73071[] = {{&S29521[42], 22}, {NULL}};
const Segment SGI73072[] = {{&S29521[30], 4}, {NULL}};
const Segment SGI73073[] = {{&S29521[123], 3}, {NULL}};
const Segment SGI73074[] = {{&S29521[88], 6}, {NULL}};
const Segment SGI73075[] = {{&S29521[143], 8}, {NULL}};
const Segment SGI73076[] = {{&S19208[44], 11}, {NULL}};
const Segment SGI73077[] = {{&S19208[44], 23}, {NULL}};
const Segment SGI73078[] = {{&S19208[30], 6}, {NULL}};
const Segment SGI73079[] = {{&S19208[31], 3}, {NULL}};
const Segment SGI73080[] = {{&S29522[42], 10}, {NULL}};
const Segment SGI73081[] = {{&S19208[81], 6}, {NULL}};
const Segment SGI73082[] = {{&S29522[42], 22}, {NULL}};
const Segment SGI73083[] = {{&S29522[30], 4}, {NULL}};
const Segment SGI73084[] = {{&S29522[81], 6}, {NULL}};
const Segment SGI73085[] = {{&S29523[42], 10}, {NULL}};
const Segment SGI73086[] = {{&S19210[38], 10}, {NULL}};
const Segment SGI73087[] = {{&S29523[42], 22}, {NULL}};
const Segment SGI73088[] = {{&S19210[31], 6}, {NULL}};
const Segment SGI73089[] = {{&S29523[30], 4}, {NULL}};
const Segment SGI73090[] = {{&S29523[113], 3}, {NULL}};
const Segment SGI73091[] = {{&S29523[78], 6}, {NULL}};
const Segment SGI73092[] = {{&S29524[40], 10}, {NULL}};
const Segment SGI73093[] = {{&S29524[30], 4}, {NULL}};
const Segment SGI73094[] = {{&S29524[111], 3}, {NULL}};
const Segment SGI73095[] = {{&S29524[76], 6}, {NULL}};
const Segment SGI73096[] = {{&S29525[42], 11}, {NULL}};
const Segment SGI73097[] = {{&S29525[42], 23}, {NULL}};
const Segment SGI73098[] = {{&S29525[30], 4}, {NULL}};
const Segment SGI73099[] = {{&S29525[123], 3}, {NULL}};
const Segment SGI73100[] = {{&S29525[88], 6}, {NULL}};
const Segment SGI73101[] = {{&S29525[143], 8}, {NULL}};
const Segment SGI73102[] = {{&S29526[42], 22}, {NULL}};
const Segment SGI73103[] = {{&S29526[30], 4}, {NULL}};
const Segment SGI73104[] = {{&S29527[42], 9}, {NULL}};
const Segment SGI73105[] = {{&S29527[30], 4}, {NULL}};
const Segment SGI73106[] = {{&S29528[42], 10}, {NULL}};
const Segment SGI73107[] = {{&S29528[42], 19}, {NULL}};
const Segment SGI73108[] = {{&S29528[30], 4}, {NULL}};
const Segment SGI73109[] = {{&S29528[109], 3}, {NULL}};
const Segment SGI73110[] = {{&S29528[75], 5}, {NULL}};
const Segment SGI73111[] = {{&S29529[42], 10}, {NULL}};
const Segment SGI73112[] = {{&S29529[42], 22}, {&S29529[204], 7}, {NULL}};
const Segment SGI73113[] = {{&S29529[30], 4}, {NULL}};
const Segment SGI73114[] = {{&S29529[113], 3}, {NULL}};
const Segment SGI73115[] = {{&S29529[78], 6}, {NULL}};
const Segment SGI73116[] = {{&S29531[42], 11}, {NULL}};
const Segment SGI73117[] = {{&S29531[42], 23}, {NULL}};
const Segment SGI73118[] = {{&S29531[30], 4}, {NULL}};
const Segment SGI73119[] = {{&S29531[114], 3}, {NULL}};
const Segment SGI73120[] = {{&S29531[79], 6}, {NULL}};
const Segment SGI73121[] = {{&S29532[53], 18}, {NULL}};
const Segment SGI73122[] = {{&S29532[48], 4}, {NULL}};
const Segment SGI73123[] = {{&S29533[42], 11}, {NULL}};
const Segment SGI73124[] = {{&S29533[42], 23}, {NULL}};
const Segment SGI73125[] = {{&S29533[30], 4}, {NULL}};
const Segment SGI73126[] = {{&S29533[82], 6}, {NULL}};
const Segment SGI73127[] = {{&S29534[42], 10}, {NULL}};
const Segment SGI73128[] = {{&S29534[42], 22}, {NULL}};
const Segment SGI73129[] = {{&S29534[30], 4}, {NULL}};
const Segment SGI73130[] = {{&S29534[114], 3}, {NULL}};
const Segment SGI73131[] = {{&S29534[78], 7}, {NULL}};
const Segment SGI73132[] = {{&S29534[134], 8}, {NULL}};
const Segment SGI73133[] = {{&S29535[42], 11}, {NULL}};
const Segment SGI73134[] = {{&S29535[42], 23}, {&S29535[206], 7}, {NULL}};
const Segment SGI73135[] = {{&S29535[30], 4}, {NULL}};
const Segment SGI73136[] = {{&S29535[114], 3}, {NULL}};
const Segment SGI73137[] = {{&S29535[79], 6}, {NULL}};
const Segment SGI73138[] = {{&S29536[40], 11}, {NULL}};
const Segment SGI73139[] = {{&S29536[30], 4}, {NULL}};
const Segment SGI73140[] = {{&S29536[112], 3}, {NULL}};
const Segment SGI73141[] = {{&S29536[77], 6}, {NULL}};
const Segment SGI73142[] = {{&S19213[42], 10}, {NULL}};
const Segment SGI73143[] = {{&S19213[30], 6}, {NULL}};
const Segment SGI73144[] = {{&S29537[42], 10}, {NULL}};
const Segment SGI73145[] = {{&S19213[31], 3}, {NULL}};
const Segment SGI73146[] = {{&S29537[42], 22}, {NULL}};
const Segment SGI73147[] = {{&S29537[30], 4}, {NULL}};
const Segment SGI73148[] = {{&S19213[78], 6}, {NULL}};
const Segment SGI73149[] = {{&S29537[112], 3}, {NULL}};
const Segment SGI73150[] = {{&S29537[78], 5}, {NULL}};
const Segment SGI73151[] = {{&S29538[42], 10}, {NULL}};
const Segment SGI73152[] = {{&S29538[42], 22}, {NULL}};
const Segment SGI73153[] = {{&S29538[30], 4}, {NULL}};
const Segment SGI73154[] = {{&S29538[113], 3}, {NULL}};
const Segment SGI73155[] = {{&S29538[78], 6}, {NULL}};
const Segment SGI73156[] = {{&S29539[42], 31}, {NULL}};
const Segment SGI73157[] = {{&S29539[30], 4}, {NULL}};
const Segment SGI73158[] = {{&S29539[83], 3}, {NULL}};
const Segment SGI73159[] = {{&S19218[44], 10}, {NULL}};
const Segment SGI73160[] = {{&S29540[42], 11}, {NULL}};
const Segment SGI73161[] = {{&S19218[44], 22}, {NULL}};
const Segment SGI73162[] = {{&S29540[42], 20}, {NULL}};
const Segment SGI73163[] = {{&S19218[30], 6}, {NULL}};
const Segment SGI73164[] = {{&S29540[30], 4}, {NULL}};
const Segment SGI73165[] = {{&S29540[110], 3}, {NULL}};
const Segment SGI73166[] = {{&S19218[83], 6}, {NULL}};
const Segment SGI73167[] = {{&S29540[76], 5}, {NULL}};
const Segment SGI73168[] = {{&S19219[38], 11}, {NULL}};
const Segment SGI73169[] = {{&S19219[31], 6}, {NULL}};
const Segment SGI73170[] = {{&S29541[42], 10}, {NULL}};
const Segment SGI73171[] = {{&S29541[42], 22}, {&S29541[205], 10}, {NULL}};
const Segment SGI73172[] = {{&S29541[30], 4}, {NULL}};
const Segment SGI73173[] = {{&S19220[44], 10}, {NULL}};
const Segment SGI73174[] = {{&S29541[113], 3}, {NULL}};
const Segment SGI73175[] = {{&S19220[44], 19}, {NULL}};
const Segment SGI73176[] = {{&S19220[30], 6}, {NULL}};
const Segment SGI73177[] = {{&S29541[78], 6}, {NULL}};
const Segment SGI73178[] = {{&S19220[31], 3}, {NULL}};
const Segment SGI73179[] = {{&S19220[77], 5}, {NULL}};
const Segment SGI73180[] = {{&S29542[42], 10}, {NULL}};
const Segment SGI73181[] = {{&S19221[44], 10}, {NULL}};
const Segment SGI73182[] = {{&S19221[44], 22}, {NULL}};
const Segment SGI73183[] = {{&S29542[42], 22}, {NULL}};
const Segment SGI73184[] = {{&S19221[30], 6}, {NULL}};
const Segment SGI73185[] = {{&S29542[30], 4}, {NULL}};
const Segment SGI73186[] = {{&S19221[83], 4}, {NULL}};
const Segment SGI73187[] = {{&S29542[113], 3}, {&S29542[193], 3}, {NULL}};
const Segment SGI73188[] = {{&S19221[83], 3}, {NULL}};
const Segment SGI73189[] = {{&S29542[78], 6}, {NULL}};
const Segment SGI73190[] = {{&S19222[64], 10}, {NULL}};
const Segment SGI73191[] = {{&S19222[64], 22}, {NULL}};
const Segment SGI73192[] = {{&S19222[50], 6}, {NULL}};
const Segment SGI73193[] = {{&S29543[42], 10}, {NULL}};
const Segment SGI73194[] = {{&S29543[42], 32}, {NULL}};
const Segment SGI73195[] = {{&S29543[30], 4}, {NULL}};
const Segment SGI73196[] = {{&S29543[123], 3}, {NULL}};
const Segment SGI73197[] = {{&S29543[88], 6}, {NULL}};
const Segment SGI73198[] = {{&S19223[44], 10}, {NULL}};
const Segment SGI73199[] = {{&S19223[44], 22}, {NULL}};
const Segment SGI73200[] = {{&S19223[30], 6}, {NULL}};
const Segment SGI73201[] = {{&S19223[124], 3}, {NULL}};
const Segment SGI73202[] = {{&S19223[89], 6}, {NULL}};
const Segment SGI73203[] = {{&S19223[144], 8}, {NULL}};
const Segment SGI73204[] = {{&S29545[42], 10}, {NULL}};
const Segment SGI73205[] = {{&S29545[42], 28}, {NULL}};
const Segment SGI73206[] = {{&S29545[30], 4}, {NULL}};
const Segment SGI73207[] = {{&S19225[44], 30}, {NULL}};
const Segment SGI73208[] = {{&S29546[42], 10}, {&S29546[42], 10}, {NULL}};
const Segment SGI73209[] = {{&S19225[44], 42}, {NULL}};
const Segment SGI73210[] = {{&S19225[30], 6}, {NULL}};
const Segment SGI73211[] = {{&S29546[42], 22}, {&S29546[42], 22}, {NULL}};
const Segment SGI73212[] = {{&S29546[30], 4}, {&S29546[30], 4}, {NULL}};
const Segment SGI73213[] = {{&S19225[31], 3}, {NULL}};
const Segment SGI73214[] = {{&S19225[100], 6}, {NULL}};
const Segment SGI73215[] = {{&S29546[113], 3}, {&S29546[113], 3}, {NULL}};
const Segment SGI73216[] = {{&S29546[78], 6}, {&S29546[78], 6}, {NULL}};
const Segment SGI73217[] = {{&S29547[42], 10}, {NULL}};
const Segment SGI73218[] = {{&S29547[42], 22}, {NULL}};
const Segment SGI73219[] = {{&S29547[30], 4}, {NULL}};
const Segment SGI73220[] = {{&S19227[44], 10}, {NULL}};
const Segment SGI73221[] = {{&S19227[44], 22}, {NULL}};
const Segment SGI73222[] = {{&S19227[30], 6}, {NULL}};
const Segment SGI73223[] = {{&S19227[80], 6}, {NULL}};
const Segment SGI73224[] = {{&S29548[42], 10}, {NULL}};
const Segment SGI73225[] = {{&S29548[42], 22}, {&S29548[206], 13}, {NULL}};
const Segment SGI73226[] = {{&S29548[30], 4}, {NULL}};
const Segment SGI73227[] = {{&S19228[44], 22}, {NULL}};
const Segment SGI73228[] = {{&S19228[30], 6}, {NULL}};
const Segment SGI73229[] = {{&S29548[113], 3}, {NULL}};
const Segment SGI73230[] = {{&S29548[78], 6}, {NULL}};
const Segment SGI73231[] = {{&S29549[42], 10}, {NULL}};
const Segment SGI73232[] = {{&S29549[42], 22}, {&S29549[206], 5}, {NULL}};
const Segment SGI73233[] = {{&S29549[30], 4}, {NULL}};
const Segment SGI73234[] = {{&S29549[113], 3}, {NULL}};
const Segment SGI73235[] = {{&S29549[78], 6}, {NULL}};
const Segment SGI73236[] = {{&S29551[42], 10}, {NULL}};
const Segment SGI73237[] = {{&S29551[42], 22}, {&S29551[205], 10}, {NULL}};
const Segment SGI73238[] = {{&S29551[30], 4}, {NULL}};
const Segment SGI73239[] = {{&S29551[113], 3}, {NULL}};
const Segment SGI73240[] = {{&S29551[78], 6}, {NULL}};
const Segment SGI73241[] = {{&S29552[42], 10}, {NULL}};
const Segment SGI73242[] = {{&S29552[42], 22}, {&S29552[205], 7}, {NULL}};
const Segment SGI73243[] = {{&S29552[30], 4}, {NULL}};
const Segment SGI73244[] = {{&S29552[113], 3}, {NULL}};
const Segment SGI73245[] = {{&S29552[78], 6}, {NULL}};
const Segment SGI73246[] = {{&S29553[44], 10}, {NULL}};
const Segment SGI73247[] = {{&S29553[44], 28}, {NULL}};
const Segment SGI73248[] = {{&S29553[32], 4}, {NULL}};
const Segment SGI73249[] = {{&S29554[102], 29}, {NULL}};
const Segment SGI73250[] = {{&S29555[42], 10}, {NULL}};
const Segment SGI73251[] = {{&S29555[42], 22}, {NULL}};
const Segment SGI73252[] = {{&S29555[30], 4}, {NULL}};
const Segment SGI73253[] = {{&S29555[113], 3}, {&S29555[149], 3}, {NULL}};
const Segment SGI73254[] = {{&S29555[78], 6}, {NULL}};
const Segment SGI73255[] = {{&S29556[42], 10}, {NULL}};
const Segment SGI73256[] = {{&S29556[42], 22}, {NULL}};
const Segment SGI73257[] = {{&S29556[30], 4}, {NULL}};
const Segment SGI73258[] = {{&S29556[113], 3}, {&S29556[193], 3}, {NULL}};
const Segment SGI73259[] = {{&S29556[78], 6}, {NULL}};
const Segment SGI73260[] = {{&S19247[44], 10}, {NULL}};
const Segment SGI73261[] = {{&S19247[44], 34}, {NULL}};
const Segment SGI73262[] = {{&S19247[30], 6}, {NULL}};
const Segment SGI73263[] = {{&S29557[43], 11}, {NULL}};
const Segment SGI73264[] = {{&S29557[43], 29}, {NULL}};
const Segment SGI73265[] = {{&S29557[31], 4}, {NULL}};
const Segment SGI73266[] = {{&S29558[63], 11}, {NULL}};
const Segment SGI73267[] = {{&S29558[63], 23}, {NULL}};
const Segment SGI73268[] = {{&S29558[51], 4}, {NULL}};
const Segment SGI73269[] = {{&S29559[42], 10}, {NULL}};
const Segment SGI73270[] = {{&S29559[42], 22}, {NULL}};
const Segment SGI73271[] = {{&S29559[30], 4}, {NULL}};
const Segment SGI73272[] = {{&S29559[78], 6}, {NULL}};
const Segment SGI73273[] = {{&S19252[44], 10}, {NULL}};
const Segment SGI73274[] = {{&S19252[44], 34}, {NULL}};
const Segment SGI73275[] = {{&S19252[30], 6}, {NULL}};
const Segment SGI73276[] = {{&S29560[42], 10}, {NULL}};
const Segment SGI73277[] = {{&S29560[42], 22}, {NULL}};
const Segment SGI73278[] = {{&S29560[30], 4}, {NULL}};
const Segment SGI73279[] = {{&S29560[113], 3}, {&S29560[193], 3}, {NULL}};
const Segment SGI73280[] = {{&S29560[78], 6}, {NULL}};
const Segment SGI73281[] = {{&S19253[55], 18}, {NULL}};
const Segment SGI73282[] = {{&S19253[48], 6}, {NULL}};
const Segment SGI73283[] = {{&S29563[42], 11}, {NULL}};
const Segment SGI73284[] = {{&S29563[42], 23}, {NULL}};
const Segment SGI73285[] = {{&S29563[30], 4}, {NULL}};
const Segment SGI73286[] = {{&S29563[113], 3}, {NULL}};
const Segment SGI73287[] = {{&S29563[79], 5}, {NULL}};
const Segment SGI73288[] = {{&S19257[44], 10}, {NULL}};
const Segment SGI73289[] = {{&S19257[44], 34}, {NULL}};
const Segment SGI73290[] = {{&S19257[30], 6}, {NULL}};
const Segment SGI73291[] = {{&S19259[44], 10}, {NULL}};
const Segment SGI73292[] = {{&S19259[44], 22}, {&S19259[270], 7}, {NULL}};
const Segment SGI73293[] = {{&S19259[30], 6}, {NULL}};
const Segment SGI73294[] = {{&S19259[31], 3}, {NULL}};
const Segment SGI73295[] = {{&S19259[80], 6}, {NULL}};
const Segment SGI73296[] = {{&S19274[44], 10}, {&S19274[44], 10}, {NULL}};
const Segment SGI73297[] = {{&S19274[44], 22}, {&S19274[44], 22}, {NULL}};
const Segment SGI73298[] = {{&S19274[30], 6}, {&S19274[30], 6}, {NULL}};
const Segment SGI73299[] = {{&S19274[31], 3}, {&S19274[31], 3}, {NULL}};
const Segment SGI73300[] = {{&S19274[80], 6}, {&S19274[80], 6}, {NULL}};
const Segment SGI73301[] = {{&S19277[47], 10}, {NULL}};
const Segment SGI73302[] = {{&S19277[47], 34}, {NULL}};
const Segment SGI73303[] = {{&S19277[33], 6}, {NULL}};
const Segment SGI73304[] = {{&S19279[46], 10}, {NULL}};
const Segment SGI73305[] = {{&S19279[46], 34}, {NULL}};
const Segment SGI73306[] = {{&S19279[32], 6}, {NULL}};
const Segment SGI73307[] = {{&S19284[44], 31}, {NULL}};
const Segment SGI73308[] = {{&S19284[44], 43}, {NULL}};
const Segment SGI73309[] = {{&S19284[30], 6}, {NULL}};
const Segment SGI73310[] = {{&S19284[31], 3}, {NULL}};
const Segment SGI73311[] = {{&S19284[101], 6}, {NULL}};
const Segment SGI73312[] = {{&S19290[44], 10}, {NULL}};
const Segment SGI73313[] = {{&S19290[44], 41}, {NULL}};
const Segment SGI73314[] = {{&S19290[30], 6}, {NULL}};
const Segment SGI73315[] = {{&S19290[31], 3}, {NULL}};
const Segment SGI73316[] = {{&S19290[99], 6}, {NULL}};
const Segment SGI73317[] = {{&S19293[42], 11}, {NULL}};
const Segment SGI73318[] = {{&S19293[30], 6}, {NULL}};
const Segment SGI73319[] = {{&S19293[31], 3}, {NULL}};
const Segment SGI73320[] = {{&S19293[79], 6}, {NULL}};
const Segment SGI73321[] = {{&S19294[42], 30}, {NULL}};
const Segment SGI73322[] = {{&S19294[30], 6}, {NULL}};
const Segment SGI73323[] = {{&S19294[31], 3}, {NULL}};
const Segment SGI73324[] = {{&S19294[98], 6}, {NULL}};
const Segment SGI73325[] = {{&S19295[44], 10}, {NULL}};
const Segment SGI73326[] = {{&S19295[44], 22}, {NULL}};
const Segment SGI73327[] = {{&S19295[30], 6}, {NULL}};
const Segment SGI73328[] = {{&S19295[31], 3}, {NULL}};
const Segment SGI73329[] = {{&S19295[80], 6}, {NULL}};
const Segment SGI73330[] = {{&S19299[44], 10}, {NULL}};
const Segment SGI73331[] = {{&S19299[44], 22}, {NULL}};
const Segment SGI73332[] = {{&S19299[30], 6}, {NULL}};
const Segment SGI73333[] = {{&S19299[31], 3}, {NULL}};
const Segment SGI73334[] = {{&S19299[80], 5}, {NULL}};
const Segment SGI73335[] = {{&S36354[44], 7}, {NULL}};
const Segment SGI73336[] = {{&S36354[44], 24}, {NULL}};
const Segment SGI73337[] = {{&S36354[30], 6}, {NULL}};
const Segment SGI73338[] = {{&S36354[116], 3}, {NULL}};
const Segment SGI73339[] = {{&S36354[82], 5}, {NULL}};
const Segment SGI73340[] = {{&S36355[44], 8}, {NULL}};
const Segment SGI73341[] = {{&S36355[44], 25}, {NULL}};
const Segment SGI73342[] = {{&S36355[30], 6}, {NULL}};
const Segment SGI73343[] = {{&S36355[117], 3}, {NULL}};
const Segment SGI73344[] = {{&S36355[83], 5}, {NULL}};
const Segment SGI73345[] = {{&S36356[38], 7}, {NULL}};
const Segment SGI73346[] = {{&S36356[31], 6}, {NULL}};
const Segment SGI73347[] = {{&S36357[44], 6}, {NULL}};
const Segment SGI73348[] = {{&S36357[44], 23}, {NULL}};
const Segment SGI73349[] = {{&S36357[30], 6}, {NULL}};
const Segment SGI73350[] = {{&S36357[115], 3}, {NULL}};
const Segment SGI73351[] = {{&S36357[81], 5}, {NULL}};
const Segment SGI73352[] = {{&S36358[43], 7}, {NULL}};
const Segment SGI73353[] = {{&S36358[43], 24}, {NULL}};
const Segment SGI73354[] = {{&S36358[30], 5}, {NULL}};
const Segment SGI73355[] = {{&S36358[115], 3}, {NULL}};
const Segment SGI73356[] = {{&S36358[81], 5}, {NULL}};
const Segment SGI73357[] = {{&S36359[44], 7}, {NULL}};
const Segment SGI73358[] = {{&S36359[44], 24}, {NULL}};
const Segment SGI73359[] = {{&S36359[30], 6}, {NULL}};
const Segment SGI73360[] = {{&S36359[116], 3}, {NULL}};
const Segment SGI73361[] = {{&S36359[82], 5}, {NULL}};
const Segment SGI73362[] = {{&S36360[43], 7}, {NULL}};
const Segment SGI73363[] = {{&S36360[43], 24}, {NULL}};
const Segment SGI73364[] = {{&S36360[115], 3}, {NULL}};
const Segment SGI73365[] = {{&S36360[81], 5}, {NULL}};
const Segment SGI73366[] = {{&S36361[44], 6}, {NULL}};
const Segment SGI73367[] = {{&S36361[44], 23}, {NULL}};
const Segment SGI73368[] = {{&S36361[30], 6}, {NULL}};
const Segment SGI73369[] = {{&S36361[115], 3}, {NULL}};
const Segment SGI73370[] = {{&S36361[81], 5}, {NULL}};
const Segment SGI73371[] = {{&S36362[38], 8}, {NULL}};
const Segment SGI73372[] = {{&S36362[31], 6}, {NULL}};
const Segment SGI73373[] = {{&S30607[44], 11}, {NULL}};
const Segment SGI73374[] = {{&S30607[44], 23}, {NULL}};
const Segment SGI73375[] = {{&S30607[30], 6}, {NULL}};
const Segment SGI73376[] = {{&S30607[115], 3}, {NULL}};
const Segment SGI73377[] = {{&S30607[81], 5}, {NULL}};
const Segment SGI73378[] = {{&S19635[44], 18}, {NULL}};
const Segment SGI73379[] = {{&S19635[44], 30}, {NULL}};
const Segment SGI73380[] = {{&S19635[30], 6}, {NULL}};
const Segment SGI73381[] = {{&S19635[31], 3}, {NULL}};
const Segment SGI73382[] = {{&S19635[88], 6}, {NULL}};
const Segment SGI73383[] = {{&S19636[44], 11}, {NULL}};
const Segment SGI73384[] = {{&S19636[44], 23}, {NULL}};
const Segment SGI73385[] = {{&S19636[30], 6}, {NULL}};
const Segment SGI73386[] = {{&S19636[31], 3}, {NULL}};
const Segment SGI73387[] = {{&S19636[81], 6}, {NULL}};
const Segment SGI73388[] = {{&S30618[44], 11}, {NULL}};
const Segment SGI73389[] = {{&S30618[44], 22}, {NULL}};
const Segment SGI73390[] = {{&S30618[30], 6}, {NULL}};
const Segment SGI73391[] = {{&S30618[114], 3}, {NULL}};
const Segment SGI73392[] = {{&S30618[80], 5}, {NULL}};
const Segment SGI73393[] = {{&S19638[38], 35}, {NULL}};
const Segment SGI73394[] = {{&S19638[31], 6}, {NULL}};
const Segment SGI73395[] = {{&S30619[38], 11}, {NULL}};
const Segment SGI73396[] = {{&S30619[31], 6}, {NULL}};
const Segment SGI73397[] = {{&S19639[44], 14}, {NULL}};
const Segment SGI73398[] = {{&S19639[44], 26}, {NULL}};
const Segment SGI73399[] = {{&S19639[30], 6}, {NULL}};
const Segment SGI73400[] = {{&S19639[31], 3}, {NULL}};
const Segment SGI73401[] = {{&S19639[84], 6}, {NULL}};
const Segment SGI73402[] = {{&S19640[44], 21}, {NULL}};
const Segment SGI73403[] = {{&S19640[44], 33}, {NULL}};
const Segment SGI73404[] = {{&S19640[30], 6}, {NULL}};
const Segment SGI73405[] = {{&S19640[31], 3}, {NULL}};
const Segment SGI73406[] = {{&S19640[91], 6}, {NULL}};
const Segment SGI73407[] = {{&S19641[44], 17}, {NULL}};
const Segment SGI73408[] = {{&S19641[44], 29}, {NULL}};
const Segment SGI73409[] = {{&S19641[30], 6}, {NULL}};
const Segment SGI73410[] = {{&S19641[31], 3}, {NULL}};
const Segment SGI73411[] = {{&S19641[87], 6}, {NULL}};
const Segment SGI73412[] = {{&S30625[44], 11}, {NULL}};
const Segment SGI73413[] = {{&S30625[44], 22}, {NULL}};
const Segment SGI73414[] = {{&S30625[30], 6}, {NULL}};
const Segment SGI73415[] = {{&S30625[114], 3}, {NULL}};
const Segment SGI73416[] = {{&S30625[80], 5}, {NULL}};
const Segment SGI73417[] = {{&S19642[44], 35}, {NULL}};
const Segment SGI73418[] = {{&S19642[44], 47}, {NULL}};
const Segment SGI73419[] = {{&S19642[30], 6}, {NULL}};
const Segment SGI73420[] = {{&S19642[31], 3}, {NULL}};
const Segment SGI73421[] = {{&S19642[105], 6}, {NULL}};
const Segment SGI73422[] = {{&S30628[46], 11}, {NULL}};
const Segment SGI73423[] = {{&S30628[46], 22}, {NULL}};
const Segment SGI73424[] = {{&S30628[30], 8}, {NULL}};
const Segment SGI73425[] = {{&S30628[116], 3}, {NULL}};
const Segment SGI73426[] = {{&S30628[82], 5}, {NULL}};
const Segment SGI73427[] = {{&S19644[44], 18}, {NULL}};
const Segment SGI73428[] = {{&S19644[44], 30}, {NULL}};
const Segment SGI73429[] = {{&S19644[30], 6}, {NULL}};
const Segment SGI73430[] = {{&S19644[31], 3}, {NULL}};
const Segment SGI73431[] = {{&S19644[88], 6}, {NULL}};
const Segment SGI73432[] = {{&S19645[44], 21}, {NULL}};
const Segment SGI73433[] = {{&S19645[44], 33}, {NULL}};
const Segment SGI73434[] = {{&S19645[30], 6}, {NULL}};
const Segment SGI73435[] = {{&S19645[31], 3}, {NULL}};
const Segment SGI73436[] = {{&S19645[91], 6}, {NULL}};
const Segment SGI73437[] = {{&S19646[38], 21}, {NULL}};
const Segment SGI73438[] = {{&S19646[31], 6}, {NULL}};
const Segment SGI73439[] = {{&S30634[44], 11}, {NULL}};
const Segment SGI73440[] = {{&S30634[44], 22}, {NULL}};
const Segment SGI73441[] = {{&S19647[44], 21}, {NULL}};
const Segment SGI73442[] = {{&S30634[30], 6}, {NULL}};
const Segment SGI73443[] = {{&S19647[44], 33}, {NULL}};
const Segment SGI73444[] = {{&S19647[30], 6}, {NULL}};
const Segment SGI73445[] = {{&S30634[113], 3}, {NULL}};
const Segment SGI73446[] = {{&S30634[80], 5}, {NULL}};
const Segment SGI73447[] = {{&S19647[94], 4}, {NULL}};
const Segment SGI73448[] = {{&S19647[94], 3}, {NULL}};
const Segment SGI73449[] = {{&S19649[38], 18}, {NULL}};
const Segment SGI73450[] = {{&S19649[31], 6}, {NULL}};
const Segment SGI73451[] = {{&S19650[44], 38}, {NULL}};
const Segment SGI73452[] = {{&S19650[44], 50}, {NULL}};
const Segment SGI73453[] = {{&S19650[30], 6}, {NULL}};
const Segment SGI73454[] = {{&S19650[31], 3}, {NULL}};
const Segment SGI73455[] = {{&S19650[108], 6}, {NULL}};
const Segment SGI73456[] = {{&S19651[44], 14}, {NULL}};
const Segment SGI73457[] = {{&S19651[44], 26}, {NULL}};
const Segment SGI73458[] = {{&S19651[30], 6}, {NULL}};
const Segment SGI73459[] = {{&S19651[31], 3}, {NULL}};
const Segment SGI73460[] = {{&S19651[84], 6}, {NULL}};
const Segment SGI73461[] = {{&S19652[44], 21}, {NULL}};
const Segment SGI73462[] = {{&S19652[44], 33}, {NULL}};
const Segment SGI73463[] = {{&S19652[30], 6}, {NULL}};
const Segment SGI73464[] = {{&S19652[94], 6}, {NULL}};
const Segment SGI73465[] = {{&S19653[44], 17}, {NULL}};
const Segment SGI73466[] = {{&S19653[44], 29}, {NULL}};
const Segment SGI73467[] = {{&S19653[30], 6}, {NULL}};
const Segment SGI73468[] = {{&S19653[31], 3}, {NULL}};
const Segment SGI73469[] = {{&S19653[87], 6}, {NULL}};
const Segment SGI73470[] = {{&S19654[44], 11}, {NULL}};
const Segment SGI73471[] = {{&S19654[44], 23}, {NULL}};
const Segment SGI73472[] = {{&S19654[30], 6}, {NULL}};
const Segment SGI73473[] = {{&S19654[84], 6}, {NULL}};
const Segment SGI73474[] = {{&S36816[38], 11}, {NULL}};
const Segment SGI73475[] = {{&S19655[42], 21}, {NULL}};
const Segment SGI73476[] = {{&S36816[31], 6}, {NULL}};
const Segment SGI73477[] = {{&S19655[30], 6}, {NULL}};
const Segment SGI73478[] = {{&S19655[31], 3}, {NULL}};
const Segment SGI73479[] = {{&S19655[89], 6}, {NULL}};
const Segment SGI73480[] = {{&S36817[44], 11}, {NULL}};
const Segment SGI73481[] = {{&S36817[44], 28}, {NULL}};
const Segment SGI73482[] = {{&S19656[38], 14}, {NULL}};
const Segment SGI73483[] = {{&S36817[30], 6}, {NULL}};
const Segment SGI73484[] = {{&S19656[31], 6}, {NULL}};
const Segment SGI73485[] = {{&S36817[120], 3}, {NULL}};
const Segment SGI73486[] = {{&S36817[86], 5}, {NULL}};
const Segment SGI73487[] = {{&S19657[44], 11}, {NULL}};
const Segment SGI73488[] = {{&S19657[44], 23}, {NULL}};
const Segment SGI73489[] = {{&S19657[30], 6}, {NULL}};
const Segment SGI73490[] = {{&S36818[44], 10}, {NULL}};
const Segment SGI73491[] = {{&S36818[44], 27}, {NULL}};
const Segment SGI73492[] = {{&S36818[30], 6}, {NULL}};
const Segment SGI73493[] = {{&S19657[84], 4}, {NULL}};
const Segment SGI73494[] = {{&S19657[84], 3}, {NULL}};
const Segment SGI73495[] = {{&S36818[119], 3}, {NULL}};
const Segment SGI73496[] = {{&S36818[85], 5}, {NULL}};
const Segment SGI73497[] = {{&S36821[38], 10}, {NULL}};
const Segment SGI73498[] = {{&S36821[31], 6}, {NULL}};
const Segment SGI73499[] = {{&S36822[44], 29}, {NULL}};
const Segment SGI73500[] = {{&S19662[42], 21}, {NULL}};
const Segment SGI73501[] = {{&S36822[44], 46}, {NULL}};
const Segment SGI73502[] = {{&S19662[30], 6}, {NULL}};
const Segment SGI73503[] = {{&S36822[30], 6}, {NULL}};
const Segment SGI73504[] = {{&S19662[31], 3}, {NULL}};
const Segment SGI73505[] = {{&S36822[138], 3}, {NULL}};
const Segment SGI73506[] = {{&S19662[89], 6}, {NULL}};
const Segment SGI73507[] = {{&S36822[104], 5}, {NULL}};
const Segment SGI73508[] = {{&S36823[44], 29}, {NULL}};
const Segment SGI73509[] = {{&S36823[44], 46}, {NULL}};
const Segment SGI73510[] = {{&S36823[30], 6}, {NULL}};
const Segment SGI73511[] = {{&S36823[138], 3}, {NULL}};
const Segment SGI73512[] = {{&S36823[104], 5}, {NULL}};
const Segment SGI73513[] = {{&S36824[44], 11}, {NULL}};
const Segment SGI73514[] = {{&S36824[44], 28}, {NULL}};
const Segment SGI73515[] = {{&S36824[30], 6}, {NULL}};
const Segment SGI73516[] = {{&S19670[42], 18}, {NULL}};
const Segment SGI73517[] = {{&S19670[30], 6}, {NULL}};
const Segment SGI73518[] = {{&S19670[31], 3}, {NULL}};
const Segment SGI73519[] = {{&S19670[86], 6}, {NULL}};
const Segment SGI73520[] = {{&S19671[42], 17}, {NULL}};
const Segment SGI73521[] = {{&S19671[30], 6}, {NULL}};
const Segment SGI73522[] = {{&S19671[31], 3}, {NULL}};
const Segment SGI73523[] = {{&S19671[85], 6}, {NULL}};
const Segment SGI73524[] = {{&S19672[38], 17}, {NULL}};
const Segment SGI73525[] = {{&S19672[31], 6}, {NULL}};
const Segment SGI73526[] = {{&S30612[42], 11}, {NULL}};
const Segment SGI73527[] = {{&S30612[42], 22}, {NULL}};
const Segment SGI73528[] = {{&S30612[30], 4}, {NULL}};
const Segment SGI73529[] = {{&S30612[112], 3}, {NULL}};
const Segment SGI73530[] = {{&S30612[78], 5}, {NULL}};
const Segment SGI73531[] = {{&S30613[42], 11}, {NULL}};
const Segment SGI73532[] = {{&S30613[42], 22}, {NULL}};
const Segment SGI73533[] = {{&S30613[30], 4}, {NULL}};
const Segment SGI73534[] = {{&S30613[112], 3}, {NULL}};
const Segment SGI73535[] = {{&S30613[78], 5}, {NULL}};
const Segment SGI73536[] = {{&S37073[44], 18}, {NULL}};
const Segment SGI73537[] = {{&S37073[44], 35}, {NULL}};
const Segment SGI73538[] = {{&S37073[30], 6}, {NULL}};
const Segment SGI73539[] = {{&S37073[127], 3}, {NULL}};
const Segment SGI73540[] = {{&S37073[93], 5}, {NULL}};
const Segment SGI73541[] = {{&S37074[42], 18}, {NULL}};
const Segment SGI73542[] = {{&S37074[30], 6}, {NULL}};
const Segment SGI73543[] = {{&S37074[125], 3}, {NULL}};
const Segment SGI73544[] = {{&S37074[91], 5}, {NULL}};
const Segment SGI73545[] = {{&S37075[38], 18}, {NULL}};
const Segment SGI73546[] = {{&S37075[31], 6}, {NULL}};
const Segment SGI73547[] = {{&S37088[38], 10}, {NULL}};
const Segment SGI73548[] = {{&S37088[31], 6}, {NULL}};
const Segment SGI73549[] = {{&S37089[44], 10}, {NULL}};
const Segment SGI73550[] = {{&S37089[44], 22}, {NULL}};
const Segment SGI73551[] = {{&S37089[30], 6}, {NULL}};
const Segment SGI73552[] = {{&S37089[114], 3}, {NULL}};
const Segment SGI73553[] = {{&S37089[80], 5}, {NULL}};
const Segment SGI73554[] = {{&S24587[44], 35}, {NULL}};
const Segment SGI73555[] = {{&S24587[44], 47}, {NULL}};
const Segment SGI73556[] = {{&S24587[30], 6}, {NULL}};
const Segment SGI73557[] = {{&S24587[31], 3}, {NULL}};
const Segment SGI73558[] = {{&S24587[105], 6}, {NULL}};
const Segment SGI73559[] = {{&S24587[0], 13}, {NULL}};
const Segment SGI73560[] = {{&S24588[44], 41}, {NULL}};
const Segment SGI73561[] = {{&S24588[44], 53}, {NULL}};
const Segment SGI73562[] = {{&S24588[30], 6}, {NULL}};
const Segment SGI73563[] = {{&S24588[31], 3}, {NULL}};
const Segment SGI73564[] = {{&S24588[111], 6}, {NULL}};
const Segment SGI73565[] = {{&S24588[0], 13}, {NULL}};
const Segment SGI73566[] = {{&S37311[44], 11}, {NULL}};
const Segment SGI73567[] = {{&S37311[44], 28}, {NULL}};
const Segment SGI73568[] = {{&S37311[30], 6}, {NULL}};
const Segment SGI73569[] = {{&S37311[120], 3}, {NULL}};
const Segment SGI73570[] = {{&S37311[86], 5}, {NULL}};
const Segment SGI73571[] = {{&S37312[38], 11}, {NULL}};
const Segment SGI73572[] = {{&S37312[31], 6}, {NULL}};
const Segment SGI73573[] = {{&S37313[44], 10}, {NULL}};
const Segment SGI73574[] = {{&S37313[44], 27}, {NULL}};
const Segment SGI73575[] = {{&S37313[30], 6}, {NULL}};
const Segment SGI73576[] = {{&S37313[119], 3}, {NULL}};
const Segment SGI73577[] = {{&S37313[85], 5}, {NULL}};
const Segment SGI73578[] = {{&S37314[44], 10}, {NULL}};
const Segment SGI73579[] = {{&S37314[44], 27}, {NULL}};
const Segment SGI73580[] = {{&S37314[30], 6}, {NULL}};
const Segment SGI73581[] = {{&S37314[119], 3}, {NULL}};
const Segment SGI73582[] = {{&S37314[85], 5}, {NULL}};
const Segment SGI73583[] = {{&S37315[43], 11}, {NULL}};
const Segment SGI73584[] = {{&S37315[43], 28}, {NULL}};
const Segment SGI73585[] = {{&S37315[30], 5}, {NULL}};
const Segment SGI73586[] = {{&S37315[119], 3}, {NULL}};
const Segment SGI73587[] = {{&S37315[85], 5}, {NULL}};
const Segment SGI73588[] = {{&S37316[44], 11}, {NULL}};
const Segment SGI73589[] = {{&S37316[44], 28}, {NULL}};
const Segment SGI73590[] = {{&S37316[30], 6}, {NULL}};
const Segment SGI73591[] = {{&S37316[120], 3}, {NULL}};
const Segment SGI73592[] = {{&S37316[86], 5}, {NULL}};
const Segment SGI73593[] = {{&S37317[43], 11}, {NULL}};
const Segment SGI73594[] = {{&S37317[43], 28}, {NULL}};
const Segment SGI73595[] = {{&S37317[119], 3}, {NULL}};
const Segment SGI73596[] = {{&S37317[85], 5}, {NULL}};
const Segment SGI73597[] = {{&S37318[42], 11}, {NULL}};
const Segment SGI73598[] = {{&S37318[30], 6}, {NULL}};
const Segment SGI73599[] = {{&S37318[118], 3}, {NULL}};
const Segment SGI73600[] = {{&S37318[84], 5}, {NULL}};
const Segment SGI73601[] = {{&S37391[44], 10}, {NULL}};
const Segment SGI73602[] = {{&S37391[44], 27}, {NULL}};
const Segment SGI73603[] = {{&S37391[30], 6}, {NULL}};
const Segment SGI73604[] = {{&S37391[119], 3}, {NULL}};
const Segment SGI73605[] = {{&S37391[85], 5}, {NULL}};
const Segment SGI73606[] = {{&S37392[38], 10}, {NULL}};
const Segment SGI73607[] = {{&S37392[31], 6}, {NULL}};
const Segment SGI73608[] = {{&S37393[44], 29}, {NULL}};
const Segment SGI73609[] = {{&S37393[44], 46}, {NULL}};
const Segment SGI73610[] = {{&S37393[30], 6}, {NULL}};
const Segment SGI73611[] = {{&S37393[138], 3}, {NULL}};
const Segment SGI73612[] = {{&S37393[104], 5}, {NULL}};
const Segment SGI73613[] = {{&S37394[44], 29}, {NULL}};
const Segment SGI73614[] = {{&S37394[44], 46}, {NULL}};
const Segment SGI73615[] = {{&S37394[30], 6}, {NULL}};
const Segment SGI73616[] = {{&S37394[138], 3}, {NULL}};
const Segment SGI73617[] = {{&S37394[104], 5}, {NULL}};
const Segment SGI73618[] = {{&S37395[42], 10}, {NULL}};
const Segment SGI73619[] = {{&S37395[30], 6}, {NULL}};
const Segment SGI73620[] = {{&S37395[117], 3}, {NULL}};
const Segment SGI73621[] = {{&S37395[83], 5}, {NULL}};
const Segment SGI73622[] = {{&S37396[44], 10}, {NULL}};
const Segment SGI73623[] = {{&S37396[44], 27}, {NULL}};
const Segment SGI73624[] = {{&S37396[30], 6}, {NULL}};
const Segment SGI73625[] = {{&S37396[88], 6}, {NULL}};
const Segment SGI73626[] = {{&S31270[44], 7}, {NULL}};
const Segment SGI73627[] = {{&S31270[44], 18}, {NULL}};
const Segment SGI73628[] = {{&S31270[30], 6}, {NULL}};
const Segment SGI73629[] = {{&S31270[79], 4}, {NULL}};
const Segment SGI73630[] = {{&S31270[79], 3}, {NULL}};
const Segment SGI73631[] = {{&S31272[44], 7}, {NULL}};
const Segment SGI73632[] = {{&S31272[44], 20}, {NULL}};
const Segment SGI73633[] = {{&S31272[30], 6}, {NULL}};
const Segment SGI73634[] = {{&S31272[112], 3}, {NULL}};
const Segment SGI73635[] = {{&S31272[78], 5}, {NULL}};
const Segment SGI73636[] = {{&S31273[44], 7}, {NULL}};
const Segment SGI73637[] = {{&S31273[44], 18}, {NULL}};
const Segment SGI73638[] = {{&S31273[30], 6}, {NULL}};
const Segment SGI73639[] = {{&S31273[79], 6}, {NULL}};
const Segment SGI73640[] = {{&S31276[38], 7}, {NULL}};
const Segment SGI73641[] = {{&S31276[31], 6}, {NULL}};
const Segment SGI73642[] = {{&S24930[44], 14}, {NULL}};
const Segment SGI73643[] = {{&S24930[44], 25}, {NULL}};
const Segment SGI73644[] = {{&S24930[30], 6}, {NULL}};
const Segment SGI73645[] = {{&S24930[31], 3}, {NULL}};
const Segment SGI73646[] = {{&S24930[83], 5}, {NULL}};
const Segment SGI73647[] = {{&S24930[0], 13}, {NULL}};
const Segment SGI73648[] = {{&S37593[44], 18}, {NULL}};
const Segment SGI73649[] = {{&S37593[44], 35}, {NULL}};
const Segment SGI73650[] = {{&S37593[30], 6}, {NULL}};
const Segment SGI73651[] = {{&S37593[127], 3}, {NULL}};
const Segment SGI73652[] = {{&S37593[93], 5}, {NULL}};
const Segment SGI73653[] = {{&S37594[38], 18}, {NULL}};
const Segment SGI73654[] = {{&S37594[31], 6}, {NULL}};
const Segment SGI73655[] = {{&S37595[42], 18}, {NULL}};
const Segment SGI73656[] = {{&S37595[30], 6}, {NULL}};
const Segment SGI73657[] = {{&S37595[125], 3}, {NULL}};
const Segment SGI73658[] = {{&S37595[91], 5}, {NULL}};
const Segment SGI73659[] = {{&S37626[44], 6}, {NULL}};
const Segment SGI73660[] = {{&S37626[44], 22}, {NULL}};
const Segment SGI73661[] = {{&S37626[30], 6}, {NULL}};
const Segment SGI73662[] = {{&S37626[114], 3}, {NULL}};
const Segment SGI73663[] = {{&S37626[80], 5}, {NULL}};
const Segment SGI73664[] = {{&S37627[38], 6}, {NULL}};
const Segment SGI73665[] = {{&S37627[31], 6}, {NULL}};
const Segment SGI73666[] = {{&S37628[53], 6}, {NULL}};
const Segment SGI73667[] = {{&S37628[53], 22}, {NULL}};
const Segment SGI73668[] = {{&S37628[30], 15}, {NULL}};
const Segment SGI73669[] = {{&S37628[123], 3}, {NULL}};
const Segment SGI73670[] = {{&S37628[89], 5}, {NULL}};
const Segment SGI73671[] = {{&S37629[64], 6}, {NULL}};
const Segment SGI73672[] = {{&S37629[64], 22}, {NULL}};
const Segment SGI73673[] = {{&S37629[30], 26}, {NULL}};
const Segment SGI73674[] = {{&S37629[134], 3}, {NULL}};
const Segment SGI73675[] = {{&S37629[100], 5}, {NULL}};
const Segment SGI73676[] = {{&S37630[63], 6}, {NULL}};
const Segment SGI73677[] = {{&S37630[63], 22}, {NULL}};
const Segment SGI73678[] = {{&S37630[30], 25}, {NULL}};
const Segment SGI73679[] = {{&S37630[133], 3}, {NULL}};
const Segment SGI73680[] = {{&S37630[99], 5}, {NULL}};
const Segment SGI73681[] = {{&S37652[44], 7}, {NULL}};
const Segment SGI73682[] = {{&S37652[44], 32}, {NULL}};
const Segment SGI73683[] = {{&S37652[30], 6}, {NULL}};
const Segment SGI73684[] = {{&S37652[124], 3}, {NULL}};
const Segment SGI73685[] = {{&S37652[90], 5}, {NULL}};
const Segment SGI73686[] = {{&S37688[44], 10}, {NULL}};
const Segment SGI73687[] = {{&S37688[44], 22}, {NULL}};
const Segment SGI73688[] = {{&S37688[30], 6}, {NULL}};
const Segment SGI73689[] = {{&S37688[114], 3}, {NULL}};
const Segment SGI73690[] = {{&S37688[80], 5}, {NULL}};
const Segment SGI73691[] = {{&S37692[61], 9}, {NULL}};
const Segment SGI73692[] = {{&S37692[61], 20}, {NULL}};
const Segment SGI73693[] = {{&S37692[30], 23}, {NULL}};
const Segment SGI73694[] = {{&S37692[129], 3}, {NULL}};
const Segment SGI73695[] = {{&S37692[95], 5}, {NULL}};
const Segment SGI73696[] = {{&S37693[54], 10}, {NULL}};
const Segment SGI73697[] = {{&S37693[54], 22}, {NULL}};
const Segment SGI73698[] = {{&S37693[30], 16}, {NULL}};
const Segment SGI73699[] = {{&S37693[124], 3}, {NULL}};
const Segment SGI73700[] = {{&S37693[90], 5}, {NULL}};
const Segment SGI73701[] = {{&S31451[44], 9}, {NULL}};
const Segment SGI73702[] = {{&S31451[44], 20}, {NULL}};
const Segment SGI73703[] = {{&S31451[30], 6}, {NULL}};
const Segment SGI73704[] = {{&S31451[112], 3}, {NULL}};
const Segment SGI73705[] = {{&S31451[78], 5}, {NULL}};
const Segment SGI73706[] = {{&S31453[44], 9}, {NULL}};
const Segment SGI73707[] = {{&S31453[44], 21}, {NULL}};
const Segment SGI73708[] = {{&S31453[30], 6}, {NULL}};
const Segment SGI73709[] = {{&S31453[127], 3}, {NULL}};
const Segment SGI73710[] = {{&S31453[93], 5}, {NULL}};
const Segment SGI73711[] = {{&S31459[38], 9}, {NULL}};
const Segment SGI73712[] = {{&S31459[31], 6}, {NULL}};
const Segment SGI73713[] = {{&S31467[44], 9}, {NULL}};
const Segment SGI73714[] = {{&S31467[44], 26}, {NULL}};
const Segment SGI73715[] = {{&S31467[30], 6}, {NULL}};
const Segment SGI73716[] = {{&S31467[132], 3}, {NULL}};
const Segment SGI73717[] = {{&S31467[98], 5}, {NULL}};
const Segment SGI73718[] = {{&S31841[44], 9}, {NULL}};
const Segment SGI73719[] = {{&S31841[44], 20}, {NULL}};
const Segment SGI73720[] = {{&S31841[30], 6}, {NULL}};
const Segment SGI73721[] = {{&S31841[123], 3}, {NULL}};
const Segment SGI73722[] = {{&S31841[89], 5}, {NULL}};
const Segment SGI73723[] = {{&S38072[44], 23}, {NULL}};
const Segment SGI73724[] = {{&S38072[44], 34}, {NULL}};
const Segment SGI73725[] = {{&S38072[30], 6}, {NULL}};
const Segment SGI73726[] = {{&S38072[126], 3}, {NULL}};
const Segment SGI73727[] = {{&S38072[92], 5}, {NULL}};
const Segment SGI73728[] = {{&S38073[44], 24}, {NULL}};
const Segment SGI73729[] = {{&S38073[44], 35}, {NULL}};
const Segment SGI73730[] = {{&S38073[30], 6}, {NULL}};
const Segment SGI73731[] = {{&S38073[127], 3}, {NULL}};
const Segment SGI73732[] = {{&S38073[93], 5}, {NULL}};
const Segment SGI73733[] = {{&S38074[44], 30}, {NULL}};
const Segment SGI73734[] = {{&S38074[44], 41}, {NULL}};
const Segment SGI73735[] = {{&S38074[30], 6}, {NULL}};
const Segment SGI73736[] = {{&S38074[133], 3}, {NULL}};
const Segment SGI73737[] = {{&S38074[99], 5}, {NULL}};
const Segment SGI73738[] = {{&S38075[44], 24}, {NULL}};
const Segment SGI73739[] = {{&S38075[44], 35}, {NULL}};
const Segment SGI73740[] = {{&S38075[30], 6}, {NULL}};
const Segment SGI73741[] = {{&S38075[127], 3}, {NULL}};
const Segment SGI73742[] = {{&S38075[93], 5}, {NULL}};
const Segment SGI73743[] = {{&S38076[44], 23}, {NULL}};
const Segment SGI73744[] = {{&S38076[44], 34}, {NULL}};
const Segment SGI73745[] = {{&S38076[30], 6}, {NULL}};
const Segment SGI73746[] = {{&S38076[126], 3}, {NULL}};
const Segment SGI73747[] = {{&S38076[92], 5}, {NULL}};
const Segment SGI73748[] = {{&S38077[44], 24}, {NULL}};
const Segment SGI73749[] = {{&S38077[44], 35}, {NULL}};
const Segment SGI73750[] = {{&S38077[30], 6}, {NULL}};
const Segment SGI73751[] = {{&S38077[127], 3}, {NULL}};
const Segment SGI73752[] = {{&S38077[93], 5}, {NULL}};
const Segment SGI73753[] = {{&S38078[44], 24}, {NULL}};
const Segment SGI73754[] = {{&S38078[44], 35}, {NULL}};
const Segment SGI73755[] = {{&S38078[30], 6}, {NULL}};
const Segment SGI73756[] = {{&S38078[127], 3}, {NULL}};
const Segment SGI73757[] = {{&S38078[93], 5}, {NULL}};
const Segment SGI73758[] = {{&S38079[44], 24}, {NULL}};
const Segment SGI73759[] = {{&S38079[44], 35}, {NULL}};
const Segment SGI73760[] = {{&S38079[30], 6}, {NULL}};
const Segment SGI73761[] = {{&S38079[127], 3}, {NULL}};
const Segment SGI73762[] = {{&S38079[93], 5}, {NULL}};
const Segment SGI73763[] = {{&S38080[44], 23}, {NULL}};
const Segment SGI73764[] = {{&S38080[44], 34}, {NULL}};
const Segment SGI73765[] = {{&S38080[30], 6}, {NULL}};
const Segment SGI73766[] = {{&S38080[126], 3}, {NULL}};
const Segment SGI73767[] = {{&S38080[92], 5}, {NULL}};
const Segment SGI73768[] = {{&S31945[44], 19}, {NULL}};
const Segment SGI73769[] = {{&S31945[44], 36}, {NULL}};
const Segment SGI73770[] = {{&S31945[30], 6}, {NULL}};
const Segment SGI73771[] = {{&S31945[128], 3}, {NULL}};
const Segment SGI73772[] = {{&S31945[94], 5}, {NULL}};
const Segment SGI73773[] = {{&S31946[44], 7}, {NULL}};
const Segment SGI73774[] = {{&S31946[44], 24}, {NULL}};
const Segment SGI73775[] = {{&S31946[30], 6}, {NULL}};
const Segment SGI73776[] = {{&S31946[116], 3}, {NULL}};
const Segment SGI73777[] = {{&S31946[82], 5}, {NULL}};
const Segment SGI73778[] = {{&S31947[44], 19}, {NULL}};
const Segment SGI73779[] = {{&S31947[44], 36}, {NULL}};
const Segment SGI73780[] = {{&S31947[30], 6}, {NULL}};
const Segment SGI73781[] = {{&S31947[128], 3}, {NULL}};
const Segment SGI73782[] = {{&S31947[94], 5}, {NULL}};
const Segment SGI73783[] = {{&S31948[44], 19}, {NULL}};
const Segment SGI73784[] = {{&S31948[44], 36}, {NULL}};
const Segment SGI73785[] = {{&S31948[30], 6}, {NULL}};
const Segment SGI73786[] = {{&S31948[128], 3}, {NULL}};
const Segment SGI73787[] = {{&S31948[94], 5}, {NULL}};
const Segment SGI73788[] = {{&S31949[44], 19}, {NULL}};
const Segment SGI73789[] = {{&S31949[44], 36}, {NULL}};
const Segment SGI73790[] = {{&S31949[30], 6}, {NULL}};
const Segment SGI73791[] = {{&S31949[128], 3}, {NULL}};
const Segment SGI73792[] = {{&S31949[94], 5}, {NULL}};
const Segment SGI73793[] = {{&S31950[44], 7}, {NULL}};
const Segment SGI73794[] = {{&S31950[44], 24}, {NULL}};
const Segment SGI73795[] = {{&S31950[30], 6}, {NULL}};
const Segment SGI73796[] = {{&S31950[85], 6}, {NULL}};
const Segment SGI73797[] = {{&S25475[44], 10}, {NULL}};
const Segment SGI73798[] = {{&S25475[44], 22}, {NULL}};
const Segment SGI73799[] = {{&S31951[44], 7}, {NULL}};
const Segment SGI73800[] = {{&S25475[30], 6}, {NULL}};
const Segment SGI73801[] = {{&S31951[44], 24}, {NULL}};
const Segment SGI73802[] = {{&S31951[30], 6}, {NULL}};
const Segment SGI73803[] = {{&S25475[31], 3}, {NULL}};
const Segment SGI73804[] = {{&S31951[116], 3}, {NULL}};
const Segment SGI73805[] = {{&S25475[80], 5}, {NULL}};
const Segment SGI73806[] = {{&S31951[82], 5}, {NULL}};
const Segment SGI73807[] = {{&S25475[0], 13}, {NULL}};
const Segment SGI73808[] = {{&S31952[38], 7}, {NULL}};
const Segment SGI73809[] = {{&S31952[31], 6}, {NULL}};
const Segment SGI73810[] = {{&S31953[44], 7}, {NULL}};
const Segment SGI73811[] = {{&S31953[44], 24}, {NULL}};
const Segment SGI73812[] = {{&S31953[30], 6}, {NULL}};
const Segment SGI73813[] = {{&S31953[116], 3}, {NULL}};
const Segment SGI73814[] = {{&S31953[82], 5}, {NULL}};
const Segment SGI73815[] = {{&S31954[44], 19}, {NULL}};
const Segment SGI73816[] = {{&S31954[44], 36}, {NULL}};
const Segment SGI73817[] = {{&S31954[30], 6}, {NULL}};
const Segment SGI73818[] = {{&S31954[128], 3}, {NULL}};
const Segment SGI73819[] = {{&S31954[94], 5}, {NULL}};
const Segment SGI73820[] = {{&S31955[44], 7}, {NULL}};
const Segment SGI73821[] = {{&S31955[44], 24}, {NULL}};
const Segment SGI73822[] = {{&S31955[30], 6}, {NULL}};
const Segment SGI73823[] = {{&S31955[85], 6}, {NULL}};
const Segment SGI73824[] = {{&S31956[42], 19}, {NULL}};
const Segment SGI73825[] = {{&S31956[30], 6}, {NULL}};
const Segment SGI73826[] = {{&S31956[126], 3}, {NULL}};
const Segment SGI73827[] = {{&S31956[92], 5}, {NULL}};
const Segment SGI73828[] = {{&S31957[44], 7}, {NULL}};
const Segment SGI73829[] = {{&S31957[44], 24}, {NULL}};
const Segment SGI73830[] = {{&S31957[30], 6}, {NULL}};
const Segment SGI73831[] = {{&S31957[116], 3}, {NULL}};
const Segment SGI73832[] = {{&S31957[82], 5}, {NULL}};
const Segment SGI73833[] = {{&S31958[44], 7}, {NULL}};
const Segment SGI73834[] = {{&S31958[44], 24}, {NULL}};
const Segment SGI73835[] = {{&S31958[30], 6}, {NULL}};
const Segment SGI73836[] = {{&S31958[116], 3}, {NULL}};
const Segment SGI73837[] = {{&S31958[82], 5}, {NULL}};
const Segment SGI73838[] = {{&S38196[44], 11}, {NULL}};
const Segment SGI73839[] = {{&S38196[44], 28}, {NULL}};
const Segment SGI73840[] = {{&S38196[30], 6}, {NULL}};
const Segment SGI73841[] = {{&S38196[120], 3}, {NULL}};
const Segment SGI73842[] = {{&S38196[86], 5}, {NULL}};
const Segment SGI73843[] = {{&S38197[38], 11}, {NULL}};
const Segment SGI73844[] = {{&S38197[31], 6}, {NULL}};
const Segment SGI73845[] = {{&S38201[44], 11}, {NULL}};
const Segment SGI73846[] = {{&S38201[44], 28}, {NULL}};
const Segment SGI73847[] = {{&S38201[30], 6}, {NULL}};
const Segment SGI73848[] = {{&S38201[121], 3}, {NULL}};
const Segment SGI73849[] = {{&S38201[86], 6}, {NULL}};
const Segment SGI73850[] = {{&S38209[42], 11}, {NULL}};
const Segment SGI73851[] = {{&S38209[30], 6}, {NULL}};
const Segment SGI73852[] = {{&S38209[118], 3}, {NULL}};
const Segment SGI73853[] = {{&S38209[84], 5}, {NULL}};
const Segment SGI73854[] = {{&S38210[44], 11}, {NULL}};
const Segment SGI73855[] = {{&S38210[44], 28}, {NULL}};
const Segment SGI73856[] = {{&S38210[30], 6}, {NULL}};
const Segment SGI73857[] = {{&S38210[89], 6}, {NULL}};
const Segment SGI73858[] = {{&S38211[44], 11}, {NULL}};
const Segment SGI73859[] = {{&S38211[44], 27}, {NULL}};
const Segment SGI73860[] = {{&S38211[30], 6}, {NULL}};
const Segment SGI73861[] = {{&S38211[119], 3}, {NULL}};
const Segment SGI73862[] = {{&S38211[85], 5}, {NULL}};
const Segment SGI73863[] = {{&S38222[44], 10}, {NULL}};
const Segment SGI73864[] = {{&S38222[44], 27}, {NULL}};
const Segment SGI73865[] = {{&S38222[30], 6}, {NULL}};
const Segment SGI73866[] = {{&S38222[119], 3}, {NULL}};
const Segment SGI73867[] = {{&S38222[85], 5}, {NULL}};
const Segment SGI73868[] = {{&S38224[38], 10}, {NULL}};
const Segment SGI73869[] = {{&S38224[31], 6}, {NULL}};
const Segment SGI73870[] = {{&S38273[44], 10}, {NULL}};
const Segment SGI73871[] = {{&S38273[44], 27}, {NULL}};
const Segment SGI73872[] = {{&S38273[30], 6}, {NULL}};
const Segment SGI73873[] = {{&S38273[119], 3}, {NULL}};
const Segment SGI73874[] = {{&S38273[85], 5}, {NULL}};
const Segment SGI73875[] = {{&S38274[44], 10}, {NULL}};
const Segment SGI73876[] = {{&S38274[44], 27}, {NULL}};
const Segment SGI73877[] = {{&S38274[30], 6}, {NULL}};
const Segment SGI73878[] = {{&S38274[88], 4}, {NULL}};
const Segment SGI73879[] = {{&S38274[88], 3}, {NULL}};
const Segment SGI73880[] = {{&S38275[38], 10}, {NULL}};
const Segment SGI73881[] = {{&S38275[31], 6}, {NULL}};
const Segment SGI73882[] = {{&S38277[44], 10}, {NULL}};
const Segment SGI73883[] = {{&S38277[44], 27}, {NULL}};
const Segment SGI73884[] = {{&S38277[30], 6}, {NULL}};
const Segment SGI73885[] = {{&S38277[88], 6}, {NULL}};
const Segment SGI73886[] = {{&S38530[44], 14}, {NULL}};
const Segment SGI73887[] = {{&S38530[44], 31}, {NULL}};
const Segment SGI73888[] = {{&S38530[30], 6}, {NULL}};
const Segment SGI73889[] = {{&S38530[123], 3}, {NULL}};
const Segment SGI73890[] = {{&S38530[89], 5}, {NULL}};
const Segment SGI73891[] = {{&S38531[44], 10}, {NULL}};
const Segment SGI73892[] = {{&S38531[44], 30}, {NULL}};
const Segment SGI73893[] = {{&S38531[30], 6}, {NULL}};
const Segment SGI73894[] = {{&S38531[122], 3}, {NULL}};
const Segment SGI73895[] = {{&S38531[88], 5}, {NULL}};
const Segment SGI73896[] = {{&S38532[38], 14}, {NULL}};
const Segment SGI73897[] = {{&S38532[31], 6}, {NULL}};
const Segment SGI73898[] = {{&S21246[44], 7}, {NULL}};
const Segment SGI73899[] = {{&S21246[44], 24}, {NULL}};
const Segment SGI73900[] = {{&S21246[30], 6}, {NULL}};
const Segment SGI73901[] = {{&S21246[31], 3}, {NULL}};
const Segment SGI73902[] = {{&S21246[82], 6}, {NULL}};
const Segment SGI73903[] = {{&S21248[38], 7}, {NULL}};
const Segment SGI73904[] = {{&S21248[31], 6}, {NULL}};
const Segment SGI73905[] = {{&S21249[44], 7}, {NULL}};
const Segment SGI73906[] = {{&S21249[44], 22}, {NULL}};
const Segment SGI73907[] = {{&S21249[30], 6}, {NULL}};
const Segment SGI73908[] = {{&S21249[31], 3}, {NULL}};
const Segment SGI73909[] = {{&S21249[80], 6}, {NULL}};
const Segment SGI73910[] = {{&S21255[44], 7}, {NULL}};
const Segment SGI73911[] = {{&S21255[44], 24}, {NULL}};
const Segment SGI73912[] = {{&S21255[30], 6}, {NULL}};
const Segment SGI73913[] = {{&S21255[85], 6}, {NULL}};
const Segment SGI73914[] = {{&S21256[44], 7}, {NULL}};
const Segment SGI73915[] = {{&S21256[44], 24}, {NULL}};
const Segment SGI73916[] = {{&S21256[30], 6}, {NULL}};
const Segment SGI73917[] = {{&S21256[31], 3}, {NULL}};
const Segment SGI73918[] = {{&S21256[82], 6}, {NULL}};
const Segment SGI73919[] = {{&S21257[44], 7}, {NULL}};
const Segment SGI73920[] = {{&S21257[44], 22}, {NULL}};
const Segment SGI73921[] = {{&S21257[30], 6}, {NULL}};
const Segment SGI73922[] = {{&S21257[31], 3}, {NULL}};
const Segment SGI73923[] = {{&S21257[80], 6}, {NULL}};
const Segment SGI73924[] = {{&S38576[44], 20}, {NULL}};
const Segment SGI73925[] = {{&S38576[44], 37}, {NULL}};
const Segment SGI73926[] = {{&S38576[30], 6}, {NULL}};
const Segment SGI73927[] = {{&S38576[129], 3}, {NULL}};
const Segment SGI73928[] = {{&S38576[95], 5}, {NULL}};
const Segment SGI73929[] = {{&S38577[38], 20}, {NULL}};
const Segment SGI73930[] = {{&S38577[31], 6}, {NULL}};
const Segment SGI73931[] = {{&S38578[43], 20}, {NULL}};
const Segment SGI73932[] = {{&S38578[43], 37}, {NULL}};
const Segment SGI73933[] = {{&S38578[128], 3}, {NULL}};
const Segment SGI73934[] = {{&S38578[94], 5}, {NULL}};
const Segment SGI73935[] = {{&S38579[43], 20}, {NULL}};
const Segment SGI73936[] = {{&S38579[43], 37}, {NULL}};
const Segment SGI73937[] = {{&S38579[30], 5}, {NULL}};
const Segment SGI73938[] = {{&S38579[128], 3}, {NULL}};
const Segment SGI73939[] = {{&S38579[94], 5}, {NULL}};
const Segment SGI73940[] = {{&S38580[44], 20}, {NULL}};
const Segment SGI73941[] = {{&S38580[44], 37}, {NULL}};
const Segment SGI73942[] = {{&S38580[30], 6}, {NULL}};
const Segment SGI73943[] = {{&S38580[129], 3}, {NULL}};
const Segment SGI73944[] = {{&S38580[95], 5}, {NULL}};
const Segment SGI73945[] = {{&S38581[44], 19}, {NULL}};
const Segment SGI73946[] = {{&S38581[44], 36}, {NULL}};
const Segment SGI73947[] = {{&S38581[30], 6}, {NULL}};
const Segment SGI73948[] = {{&S38581[128], 3}, {NULL}};
const Segment SGI73949[] = {{&S38581[94], 5}, {NULL}};
const Segment SGI73950[] = {{&S38582[44], 19}, {NULL}};
const Segment SGI73951[] = {{&S38582[44], 36}, {NULL}};
const Segment SGI73952[] = {{&S38582[30], 6}, {NULL}};
const Segment SGI73953[] = {{&S38582[128], 3}, {NULL}};
const Segment SGI73954[] = {{&S38582[94], 5}, {NULL}};
const Segment SGI73955[] = {{&S38583[44], 19}, {NULL}};
const Segment SGI73956[] = {{&S38583[44], 36}, {NULL}};
const Segment SGI73957[] = {{&S38583[30], 6}, {NULL}};
const Segment SGI73958[] = {{&S38583[128], 3}, {NULL}};
const Segment SGI73959[] = {{&S38583[94], 5}, {NULL}};
const Segment SGI73960[] = {{&S38584[44], 19}, {NULL}};
const Segment SGI73961[] = {{&S38584[44], 36}, {NULL}};
const Segment SGI73962[] = {{&S38584[30], 6}, {NULL}};
const Segment SGI73963[] = {{&S38584[128], 3}, {NULL}};
const Segment SGI73964[] = {{&S38584[94], 5}, {NULL}};
const Segment SGI73965[] = {{&S38595[44], 10}, {NULL}};
const Segment SGI73966[] = {{&S38595[44], 27}, {NULL}};
const Segment SGI73967[] = {{&S38595[30], 6}, {NULL}};
const Segment SGI73968[] = {{&S38595[119], 3}, {NULL}};
const Segment SGI73969[] = {{&S38595[85], 5}, {NULL}};
const Segment SGI73970[] = {{&S38603[44], 9}, {NULL}};
const Segment SGI73971[] = {{&S38603[44], 26}, {NULL}};
const Segment SGI73972[] = {{&S38603[30], 6}, {NULL}};
const Segment SGI73973[] = {{&S38603[118], 3}, {NULL}};
const Segment SGI73974[] = {{&S38603[84], 5}, {NULL}};
const Segment SGI73975[] = {{&S38702[44], 12}, {NULL}};
const Segment SGI73976[] = {{&S38702[30], 6}, {NULL}};
const Segment SGI73977[] = {{&S38702[104], 3}, {NULL}};
const Segment SGI73978[] = {{&S38702[70], 5}, {NULL}};
const Segment SGI73979[] = {{&S38703[44], 8}, {NULL}};
const Segment SGI73980[] = {{&S38703[44], 20}, {NULL}};
const Segment SGI73981[] = {{&S38703[30], 6}, {NULL}};
const Segment SGI73982[] = {{&S38703[112], 3}, {NULL}};
const Segment SGI73983[] = {{&S38703[78], 5}, {NULL}};
const Segment SGI73984[] = {{&S38704[38], 8}, {NULL}};
const Segment SGI73985[] = {{&S38704[31], 6}, {NULL}};
const Segment SGI73986[] = {{&S38705[30], 6}, {NULL}};
const Segment SGI73987[] = {{&S38705[101], 3}, {NULL}};
const Segment SGI73988[] = {{&S38705[67], 5}, {NULL}};
const Segment SGI73989[] = {{&S38742[44], 18}, {NULL}};
const Segment SGI73990[] = {{&S38742[44], 35}, {NULL}};
const Segment SGI73991[] = {{&S38742[30], 6}, {NULL}};
const Segment SGI73992[] = {{&S38742[127], 3}, {NULL}};
const Segment SGI73993[] = {{&S38742[93], 5}, {NULL}};
const Segment SGI73994[] = {{&S38743[38], 18}, {NULL}};
const Segment SGI73995[] = {{&S38743[31], 6}, {NULL}};
const Segment SGI73996[] = {{&S38744[50], 18}, {NULL}};
const Segment SGI73997[] = {{&S38744[50], 35}, {NULL}};
const Segment SGI73998[] = {{&S38744[30], 12}, {NULL}};
const Segment SGI73999[] = {{&S38744[133], 3}, {NULL}};
const Segment SGI74000[] = {{&S38744[99], 5}, {NULL}};
const Segment SGI74001[] = {{&S38745[44], 10}, {NULL}};
const Segment SGI74002[] = {{&S38745[44], 27}, {NULL}};
const Segment SGI74003[] = {{&S38745[30], 6}, {NULL}};
const Segment SGI74004[] = {{&S38745[119], 3}, {NULL}};
const Segment SGI74005[] = {{&S38745[85], 5}, {NULL}};
const Segment SGI74006[] = {{&S38751[44], 18}, {NULL}};
const Segment SGI74007[] = {{&S38751[44], 35}, {NULL}};
const Segment SGI74008[] = {{&S38751[30], 6}, {NULL}};
const Segment SGI74009[] = {{&S38751[96], 6}, {NULL}};
const Segment SGI74010[] = {{&S38752[42], 18}, {NULL}};
const Segment SGI74011[] = {{&S38752[30], 6}, {NULL}};
const Segment SGI74012[] = {{&S38752[125], 3}, {NULL}};
const Segment SGI74013[] = {{&S38752[91], 5}, {NULL}};
const Segment SGI74014[] = {{&S38753[44], 14}, {NULL}};
const Segment SGI74015[] = {{&S38753[44], 35}, {NULL}};
const Segment SGI74016[] = {{&S38753[30], 6}, {NULL}};
const Segment SGI74017[] = {{&S38753[127], 3}, {NULL}};
const Segment SGI74018[] = {{&S38753[93], 5}, {NULL}};
const Segment SGI74019[] = {{&S38754[38], 14}, {NULL}};
const Segment SGI74020[] = {{&S38754[31], 6}, {NULL}};
const Segment SGI74021[] = {{&S38755[42], 14}, {NULL}};
const Segment SGI74022[] = {{&S38755[30], 6}, {NULL}};
const Segment SGI74023[] = {{&S38755[125], 3}, {NULL}};
const Segment SGI74024[] = {{&S38755[91], 5}, {NULL}};
const Segment SGI74025[] = {{&S32675[44], 10}, {NULL}};
const Segment SGI74026[] = {{&S32675[44], 21}, {NULL}};
const Segment SGI74027[] = {{&S32675[30], 6}, {NULL}};
const Segment SGI74028[] = {{&S32675[113], 3}, {NULL}};
const Segment SGI74029[] = {{&S32675[79], 5}, {NULL}};
const Segment SGI74030[] = {{&S38829[44], 7}, {NULL}};
const Segment SGI74031[] = {{&S38829[44], 19}, {NULL}};
const Segment SGI74032[] = {{&S38829[30], 6}, {NULL}};
const Segment SGI74033[] = {{&S38829[111], 3}, {NULL}};
const Segment SGI74034[] = {{&S38829[77], 5}, {NULL}};
const Segment SGI74035[] = {{&S32682[38], 10}, {NULL}};
const Segment SGI74036[] = {{&S32682[31], 6}, {NULL}};
const Segment SGI74037[] = {{&S38870[44], 9}, {NULL}};
const Segment SGI74038[] = {{&S38870[44], 21}, {NULL}};
const Segment SGI74039[] = {{&S38870[30], 6}, {NULL}};
const Segment SGI74040[] = {{&S38870[113], 3}, {NULL}};
const Segment SGI74041[] = {{&S38870[79], 5}, {NULL}};
const Segment SGI74042[] = {{&S38871[38], 9}, {NULL}};
const Segment SGI74043[] = {{&S38871[31], 6}, {NULL}};
const Segment SGI74044[] = {{&S32885[44], 19}, {NULL}};
const Segment SGI74045[] = {{&S32885[44], 36}, {NULL}};
const Segment SGI74046[] = {{&S32885[30], 6}, {NULL}};
const Segment SGI74047[] = {{&S32885[128], 3}, {NULL}};
const Segment SGI74048[] = {{&S32885[94], 5}, {NULL}};
const Segment SGI74049[] = {{&S32886[44], 21}, {NULL}};
const Segment SGI74050[] = {{&S32886[44], 38}, {NULL}};
const Segment SGI74051[] = {{&S32886[30], 6}, {NULL}};
const Segment SGI74052[] = {{&S32886[130], 3}, {NULL}};
const Segment SGI74053[] = {{&S32886[96], 5}, {NULL}};
const Segment SGI74054[] = {{&S32887[44], 19}, {NULL}};
const Segment SGI74055[] = {{&S32887[44], 36}, {NULL}};
const Segment SGI74056[] = {{&S32887[30], 6}, {NULL}};
const Segment SGI74057[] = {{&S32887[128], 3}, {NULL}};
const Segment SGI74058[] = {{&S32887[94], 5}, {NULL}};
const Segment SGI74059[] = {{&S32888[44], 7}, {NULL}};
const Segment SGI74060[] = {{&S32888[44], 24}, {NULL}};
const Segment SGI74061[] = {{&S32888[30], 6}, {NULL}};
const Segment SGI74062[] = {{&S32888[85], 6}, {NULL}};
const Segment SGI74063[] = {{&S32889[44], 7}, {NULL}};
const Segment SGI74064[] = {{&S32889[44], 24}, {NULL}};
const Segment SGI74065[] = {{&S32889[30], 6}, {NULL}};
const Segment SGI74066[] = {{&S32889[116], 3}, {NULL}};
const Segment SGI74067[] = {{&S32889[82], 5}, {NULL}};
const Segment SGI74068[] = {{&S32890[38], 7}, {NULL}};
const Segment SGI74069[] = {{&S32890[31], 6}, {NULL}};
const Segment SGI74070[] = {{&S32891[44], 19}, {NULL}};
const Segment SGI74071[] = {{&S32891[44], 36}, {NULL}};
const Segment SGI74072[] = {{&S32891[30], 6}, {NULL}};
const Segment SGI74073[] = {{&S32891[128], 3}, {NULL}};
const Segment SGI74074[] = {{&S32891[94], 5}, {NULL}};
const Segment SGI74075[] = {{&S32892[44], 21}, {NULL}};
const Segment SGI74076[] = {{&S32892[44], 38}, {NULL}};
const Segment SGI74077[] = {{&S32892[30], 6}, {NULL}};
const Segment SGI74078[] = {{&S32892[130], 3}, {NULL}};
const Segment SGI74079[] = {{&S32892[96], 5}, {NULL}};
const Segment SGI74080[] = {{&S32893[44], 19}, {NULL}};
const Segment SGI74081[] = {{&S32893[44], 36}, {NULL}};
const Segment SGI74082[] = {{&S32893[30], 6}, {NULL}};
const Segment SGI74083[] = {{&S32893[128], 3}, {NULL}};
const Segment SGI74084[] = {{&S32893[94], 5}, {NULL}};
const Segment SGI74085[] = {{&S32894[44], 7}, {NULL}};
const Segment SGI74086[] = {{&S32894[44], 24}, {NULL}};
const Segment SGI74087[] = {{&S32894[30], 6}, {NULL}};
const Segment SGI74088[] = {{&S32894[116], 3}, {NULL}};
const Segment SGI74089[] = {{&S32894[82], 5}, {NULL}};
const Segment SGI74090[] = {{&S32895[44], 19}, {NULL}};
const Segment SGI74091[] = {{&S32895[44], 36}, {NULL}};
const Segment SGI74092[] = {{&S32895[30], 6}, {NULL}};
const Segment SGI74093[] = {{&S32895[128], 3}, {NULL}};
const Segment SGI74094[] = {{&S32895[94], 5}, {NULL}};
const Segment SGI74095[] = {{&S32896[44], 21}, {NULL}};
const Segment SGI74096[] = {{&S32896[44], 38}, {NULL}};
const Segment SGI74097[] = {{&S32896[30], 6}, {NULL}};
const Segment SGI74098[] = {{&S32896[130], 3}, {NULL}};
const Segment SGI74099[] = {{&S32896[96], 5}, {NULL}};
const Segment SGI74100[] = {{&S32897[44], 19}, {NULL}};
const Segment SGI74101[] = {{&S32897[44], 36}, {NULL}};
const Segment SGI74102[] = {{&S32897[30], 6}, {NULL}};
const Segment SGI74103[] = {{&S32897[128], 3}, {NULL}};
const Segment SGI74104[] = {{&S32897[94], 5}, {NULL}};
const Segment SGI74105[] = {{&S32898[44], 7}, {NULL}};
const Segment SGI74106[] = {{&S32898[44], 24}, {NULL}};
const Segment SGI74107[] = {{&S32898[30], 6}, {NULL}};
const Segment SGI74108[] = {{&S32898[85], 4}, {NULL}};
const Segment SGI74109[] = {{&S32898[85], 3}, {NULL}};
const Segment SGI74110[] = {{&S32899[44], 19}, {NULL}};
const Segment SGI74111[] = {{&S32899[44], 36}, {NULL}};
const Segment SGI74112[] = {{&S32899[30], 6}, {NULL}};
const Segment SGI74113[] = {{&S32899[128], 3}, {NULL}};
const Segment SGI74114[] = {{&S32899[94], 5}, {NULL}};
const Segment SGI74115[] = {{&S32900[44], 19}, {NULL}};
const Segment SGI74116[] = {{&S32900[44], 36}, {NULL}};
const Segment SGI74117[] = {{&S32900[30], 6}, {NULL}};
const Segment SGI74118[] = {{&S32900[128], 3}, {NULL}};
const Segment SGI74119[] = {{&S32900[94], 5}, {NULL}};
const Segment SGI74120[] = {{&S32901[44], 21}, {NULL}};
const Segment SGI74121[] = {{&S32901[44], 38}, {NULL}};
const Segment SGI74122[] = {{&S32901[30], 6}, {NULL}};
const Segment SGI74123[] = {{&S32901[130], 3}, {NULL}};
const Segment SGI74124[] = {{&S32901[96], 5}, {NULL}};
const Segment SGI74125[] = {{&S32902[44], 7}, {NULL}};
const Segment SGI74126[] = {{&S32902[44], 24}, {NULL}};
const Segment SGI74127[] = {{&S32902[30], 6}, {NULL}};
const Segment SGI74128[] = {{&S32902[116], 3}, {NULL}};
const Segment SGI74129[] = {{&S32902[82], 5}, {NULL}};
const Segment SGI74130[] = {{&S32903[44], 7}, {NULL}};
const Segment SGI74131[] = {{&S32903[44], 24}, {NULL}};
const Segment SGI74132[] = {{&S32903[30], 6}, {NULL}};
const Segment SGI74133[] = {{&S32903[116], 3}, {NULL}};
const Segment SGI74134[] = {{&S32903[82], 5}, {NULL}};
const Segment SGI74135[] = {{&S32904[44], 7}, {NULL}};
const Segment SGI74136[] = {{&S32904[44], 24}, {NULL}};
const Segment SGI74137[] = {{&S32904[30], 6}, {NULL}};
const Segment SGI74138[] = {{&S32904[85], 6}, {NULL}};
const Segment SGI74139[] = {{&S32905[44], 7}, {NULL}};
const Segment SGI74140[] = {{&S32905[44], 24}, {NULL}};
const Segment SGI74141[] = {{&S32905[30], 6}, {NULL}};
const Segment SGI74142[] = {{&S32905[116], 3}, {NULL}};
const Segment SGI74143[] = {{&S32905[82], 5}, {NULL}};
const Segment SGI74144[] = {{&S39217[44], 8}, {NULL}};
const Segment SGI74145[] = {{&S39217[44], 25}, {NULL}};
const Segment SGI74146[] = {{&S39217[30], 6}, {NULL}};
const Segment SGI74147[] = {{&S39217[117], 3}, {NULL}};
const Segment SGI74148[] = {{&S39217[83], 5}, {NULL}};
const Segment SGI74149[] = {{&S39218[38], 8}, {NULL}};
const Segment SGI74150[] = {{&S39218[31], 6}, {NULL}};
const Segment SGI74151[] = {{&S39219[42], 8}, {NULL}};
const Segment SGI74152[] = {{&S39219[30], 6}, {NULL}};
const Segment SGI74153[] = {{&S39219[115], 3}, {NULL}};
const Segment SGI74154[] = {{&S39219[81], 5}, {NULL}};
const Segment SGI74155[] = {{&S39346[44], 7}, {NULL}};
const Segment SGI74156[] = {{&S39346[44], 19}, {NULL}};
const Segment SGI74157[] = {{&S39346[30], 6}, {NULL}};
const Segment SGI74158[] = {{&S39346[111], 3}, {NULL}};
const Segment SGI74159[] = {{&S39346[77], 5}, {NULL}};
const Segment SGI74160[] = {{&S39347[44], 7}, {NULL}};
const Segment SGI74161[] = {{&S39347[44], 19}, {NULL}};
const Segment SGI74162[] = {{&S39347[30], 6}, {NULL}};
const Segment SGI74163[] = {{&S39347[111], 3}, {NULL}};
const Segment SGI74164[] = {{&S39347[77], 5}, {NULL}};
const Segment SGI74165[] = {{&S39348[44], 11}, {NULL}};
const Segment SGI74166[] = {{&S39348[44], 25}, {NULL}};
const Segment SGI74167[] = {{&S39348[30], 6}, {NULL}};
const Segment SGI74168[] = {{&S39348[117], 3}, {NULL}};
const Segment SGI74169[] = {{&S39348[83], 5}, {NULL}};
const Segment SGI74170[] = {{&S39349[44], 7}, {NULL}};
const Segment SGI74171[] = {{&S39349[44], 19}, {NULL}};
const Segment SGI74172[] = {{&S39349[30], 6}, {NULL}};
const Segment SGI74173[] = {{&S39349[80], 4}, {NULL}};
const Segment SGI74174[] = {{&S39349[80], 3}, {NULL}};
const Segment SGI74175[] = {{&S39350[38], 7}, {NULL}};
const Segment SGI74176[] = {{&S39350[31], 6}, {NULL}};
const Segment SGI74177[] = {{&S39351[44], 11}, {NULL}};
const Segment SGI74178[] = {{&S39351[44], 25}, {NULL}};
const Segment SGI74179[] = {{&S39351[30], 6}, {NULL}};
const Segment SGI74180[] = {{&S39351[117], 3}, {NULL}};
const Segment SGI74181[] = {{&S39351[83], 5}, {NULL}};
const Segment SGI74182[] = {{&S39352[44], 11}, {NULL}};
const Segment SGI74183[] = {{&S39352[44], 25}, {NULL}};
const Segment SGI74184[] = {{&S39352[30], 6}, {NULL}};
const Segment SGI74185[] = {{&S39352[117], 3}, {NULL}};
const Segment SGI74186[] = {{&S39352[83], 5}, {NULL}};
const Segment SGI74187[] = {{&S39353[44], 11}, {NULL}};
const Segment SGI74188[] = {{&S39353[44], 25}, {NULL}};
const Segment SGI74189[] = {{&S39353[30], 6}, {NULL}};
const Segment SGI74190[] = {{&S39353[117], 3}, {NULL}};
const Segment SGI74191[] = {{&S39353[83], 5}, {NULL}};
const Segment SGI74192[] = {{&S39354[44], 11}, {NULL}};
const Segment SGI74193[] = {{&S39354[44], 25}, {NULL}};
const Segment SGI74194[] = {{&S39354[30], 6}, {NULL}};
const Segment SGI74195[] = {{&S39354[117], 3}, {NULL}};
const Segment SGI74196[] = {{&S39354[83], 5}, {NULL}};
const Segment SGI74197[] = {{&S39355[44], 7}, {NULL}};
const Segment SGI74198[] = {{&S39355[44], 19}, {NULL}};
const Segment SGI74199[] = {{&S39355[30], 6}, {NULL}};
const Segment SGI74200[] = {{&S39355[77], 5}, {NULL}};
const Segment SGI74201[] = {{&S39356[44], 7}, {NULL}};
const Segment SGI74202[] = {{&S39356[44], 19}, {NULL}};
const Segment SGI74203[] = {{&S39356[30], 6}, {NULL}};
const Segment SGI74204[] = {{&S39356[77], 5}, {NULL}};
const Segment SGI74205[] = {{&S39357[44], 11}, {NULL}};
const Segment SGI74206[] = {{&S39357[44], 25}, {NULL}};
const Segment SGI74207[] = {{&S39357[30], 6}, {NULL}};
const Segment SGI74208[] = {{&S39357[117], 3}, {NULL}};
const Segment SGI74209[] = {{&S39357[83], 5}, {NULL}};
const Segment SGI74210[] = {{&S39358[44], 11}, {NULL}};
const Segment SGI74211[] = {{&S39358[44], 25}, {NULL}};
const Segment SGI74212[] = {{&S39358[30], 6}, {NULL}};
const Segment SGI74213[] = {{&S39358[117], 3}, {NULL}};
const Segment SGI74214[] = {{&S39358[83], 5}, {NULL}};
const Segment SGI74215[] = {{&S39359[44], 11}, {NULL}};
const Segment SGI74216[] = {{&S39359[44], 25}, {NULL}};
const Segment SGI74217[] = {{&S39359[30], 6}, {NULL}};
const Segment SGI74218[] = {{&S39359[117], 3}, {NULL}};
const Segment SGI74219[] = {{&S39359[83], 5}, {NULL}};
const Segment SGI74220[] = {{&S39360[44], 11}, {NULL}};
const Segment SGI74221[] = {{&S39360[44], 25}, {NULL}};
const Segment SGI74222[] = {{&S39360[30], 6}, {NULL}};
const Segment SGI74223[] = {{&S39360[117], 3}, {NULL}};
const Segment SGI74224[] = {{&S39360[83], 5}, {NULL}};
const Segment SGI74225[] = {{&S39361[44], 11}, {NULL}};
const Segment SGI74226[] = {{&S39361[44], 25}, {NULL}};
const Segment SGI74227[] = {{&S39361[30], 6}, {NULL}};
const Segment SGI74228[] = {{&S39361[117], 3}, {NULL}};
const Segment SGI74229[] = {{&S39361[83], 5}, {NULL}};
const Segment SGI74230[] = {{&S39362[44], 11}, {NULL}};
const Segment SGI74231[] = {{&S39362[44], 25}, {NULL}};
const Segment SGI74232[] = {{&S39362[30], 6}, {NULL}};
const Segment SGI74233[] = {{&S39362[83], 5}, {NULL}};
const Segment SGI74234[] = {{&S39363[38], 7}, {NULL}};
const Segment SGI74235[] = {{&S39363[31], 6}, {NULL}};
const Segment SGI74236[] = {{&S39450[44], 10}, {NULL}};
const Segment SGI74237[] = {{&S39450[44], 27}, {NULL}};
const Segment SGI74238[] = {{&S39450[30], 6}, {NULL}};
const Segment SGI74239[] = {{&S39450[119], 3}, {NULL}};
const Segment SGI74240[] = {{&S39450[85], 5}, {NULL}};
const Segment SGI74241[] = {{&S39451[38], 10}, {NULL}};
const Segment SGI74242[] = {{&S39451[31], 6}, {NULL}};
const Segment SGI74243[] = {{&S39452[42], 10}, {NULL}};
const Segment SGI74244[] = {{&S39452[30], 6}, {NULL}};
const Segment SGI74245[] = {{&S39452[117], 3}, {NULL}};
const Segment SGI74246[] = {{&S39452[83], 5}, {NULL}};
const Segment SGI74247[] = {{&S39470[44], 10}, {NULL}};
const Segment SGI74248[] = {{&S39470[44], 27}, {NULL}};
const Segment SGI74249[] = {{&S39470[30], 6}, {NULL}};
const Segment SGI74250[] = {{&S39470[119], 3}, {NULL}};
const Segment SGI74251[] = {{&S39470[85], 5}, {NULL}};
const Segment SGI74252[] = {{&S39471[38], 10}, {NULL}};
const Segment SGI74253[] = {{&S39471[31], 6}, {NULL}};
const Segment SGI74254[] = {{&S39472[44], 10}, {NULL}};
const Segment SGI74255[] = {{&S39472[44], 22}, {NULL}};
const Segment SGI74256[] = {{&S39472[30], 6}, {NULL}};
const Segment SGI74257[] = {{&S39472[114], 3}, {NULL}};
const Segment SGI74258[] = {{&S39472[80], 5}, {NULL}};
const Segment SGI74259[] = {{&S39474[44], 9}, {NULL}};
const Segment SGI74260[] = {{&S39474[44], 26}, {NULL}};
const Segment SGI74261[] = {{&S39474[30], 6}, {NULL}};
const Segment SGI74262[] = {{&S39474[118], 3}, {NULL}};
const Segment SGI74263[] = {{&S39474[84], 5}, {NULL}};
const Segment SGI74264[] = {{&S39475[44], 14}, {NULL}};
const Segment SGI74265[] = {{&S39475[44], 31}, {NULL}};
const Segment SGI74266[] = {{&S39475[30], 6}, {NULL}};
const Segment SGI74267[] = {{&S39475[123], 3}, {NULL}};
const Segment SGI74268[] = {{&S39475[89], 5}, {NULL}};
const Segment SGI74269[] = {{&S39476[44], 10}, {NULL}};
const Segment SGI74270[] = {{&S39476[44], 27}, {NULL}};
const Segment SGI74271[] = {{&S39476[30], 6}, {NULL}};
const Segment SGI74272[] = {{&S39476[119], 3}, {NULL}};
const Segment SGI74273[] = {{&S39476[85], 5}, {NULL}};
const Segment SGI74274[] = {{&S39477[38], 9}, {NULL}};
const Segment SGI74275[] = {{&S39477[31], 6}, {NULL}};
const Segment SGI74276[] = {{&S39478[44], 7}, {NULL}};
const Segment SGI74277[] = {{&S39478[44], 24}, {NULL}};
const Segment SGI74278[] = {{&S39478[30], 6}, {NULL}};
const Segment SGI74279[] = {{&S39478[116], 3}, {NULL}};
const Segment SGI74280[] = {{&S39478[82], 5}, {NULL}};
const Segment SGI74281[] = {{&S39479[44], 37}, {NULL}};
const Segment SGI74282[] = {{&S39479[30], 6}, {NULL}};
const Segment SGI74283[] = {{&S39479[113], 5}, {NULL}};
const Segment SGI74284[] = {{&S39480[44], 20}, {NULL}};
const Segment SGI74285[] = {{&S39480[44], 37}, {NULL}};
const Segment SGI74286[] = {{&S39480[30], 6}, {NULL}};
const Segment SGI74287[] = {{&S39480[129], 3}, {NULL}};
const Segment SGI74288[] = {{&S39480[95], 5}, {NULL}};
const Segment SGI74289[] = {{&S39481[44], 14}, {NULL}};
const Segment SGI74290[] = {{&S39481[44], 31}, {NULL}};
const Segment SGI74291[] = {{&S39481[30], 6}, {NULL}};
const Segment SGI74292[] = {{&S39481[123], 3}, {NULL}};
const Segment SGI74293[] = {{&S39481[89], 5}, {NULL}};
const Segment SGI74294[] = {{&S39482[44], 41}, {NULL}};
const Segment SGI74295[] = {{&S39482[30], 6}, {NULL}};
const Segment SGI74296[] = {{&S39482[117], 5}, {NULL}};
const Segment SGI74297[] = {{&S39483[44], 37}, {NULL}};
const Segment SGI74298[] = {{&S39483[30], 6}, {NULL}};
const Segment SGI74299[] = {{&S39483[113], 5}, {NULL}};
const Segment SGI74300[] = {{&S39484[44], 10}, {NULL}};
const Segment SGI74301[] = {{&S39484[44], 27}, {NULL}};
const Segment SGI74302[] = {{&S39484[30], 6}, {NULL}};
const Segment SGI74303[] = {{&S39484[119], 3}, {NULL}};
const Segment SGI74304[] = {{&S39484[85], 5}, {NULL}};
const Segment SGI74305[] = {{&S39485[44], 9}, {NULL}};
const Segment SGI74306[] = {{&S39485[44], 26}, {NULL}};
const Segment SGI74307[] = {{&S39485[30], 6}, {NULL}};
const Segment SGI74308[] = {{&S39485[118], 3}, {NULL}};
const Segment SGI74309[] = {{&S39485[84], 5}, {NULL}};
const Segment SGI74310[] = {{&S39486[38], 10}, {NULL}};
const Segment SGI74311[] = {{&S39486[31], 6}, {NULL}};
const Segment SGI74312[] = {{&S33127[44], 8}, {NULL}};
const Segment SGI74313[] = {{&S33127[44], 29}, {NULL}};
const Segment SGI74314[] = {{&S39528[44], 18}, {NULL}};
const Segment SGI74315[] = {{&S33127[30], 6}, {NULL}};
const Segment SGI74316[] = {{&S39528[44], 30}, {NULL}};
const Segment SGI74317[] = {{&S39528[30], 6}, {NULL}};
const Segment SGI74318[] = {{&S33127[121], 3}, {NULL}};
const Segment SGI74319[] = {{&S39528[122], 3}, {NULL}};
const Segment SGI74320[] = {{&S33127[87], 5}, {NULL}};
const Segment SGI74321[] = {{&S39528[88], 5}, {NULL}};
const Segment SGI74322[] = {{&S39529[38], 18}, {NULL}};
const Segment SGI74323[] = {{&S39529[31], 6}, {NULL}};
const Segment SGI74324[] = {{&S33129[44], 8}, {NULL}};
const Segment SGI74325[] = {{&S33129[44], 29}, {NULL}};
const Segment SGI74326[] = {{&S33129[30], 6}, {NULL}};
const Segment SGI74327[] = {{&S39530[44], 18}, {NULL}};
const Segment SGI74328[] = {{&S33129[87], 6}, {NULL}};
const Segment SGI74329[] = {{&S39530[44], 30}, {NULL}};
const Segment SGI74330[] = {{&S39530[30], 6}, {NULL}};
const Segment SGI74331[] = {{&S39530[122], 3}, {&S39530[164], 3}, {NULL}};
const Segment SGI74332[] = {{&S39530[88], 5}, {NULL}};
const Segment SGI74333[] = {{&S39531[44], 18}, {NULL}};
const Segment SGI74334[] = {{&S39531[44], 30}, {NULL}};
const Segment SGI74335[] = {{&S39531[30], 6}, {NULL}};
const Segment SGI74336[] = {{&S39531[122], 3}, {&S39531[164], 3}, {NULL}};
const Segment SGI74337[] = {{&S39531[88], 5}, {NULL}};
const Segment SGI74338[] = {{&S33234[44], 12}, {NULL}};
const Segment SGI74339[] = {{&S33234[44], 24}, {NULL}};
const Segment SGI74340[] = {{&S33234[30], 6}, {NULL}};
const Segment SGI74341[] = {{&S33234[116], 3}, {NULL}};
const Segment SGI74342[] = {{&S33234[82], 5}, {NULL}};
const Segment SGI74343[] = {{&S33242[38], 12}, {NULL}};
const Segment SGI74344[] = {{&S33242[31], 6}, {NULL}};
const Segment SGI74345[] = {{&S33245[44], 12}, {&S33245[69], 9}, {NULL}};
const Segment SGI74346[] = {{&S33245[44], 24}, {NULL}};
const Segment SGI74347[] = {{&S33245[30], 6}, {NULL}};
const Segment SGI74348[] = {{&S33245[137], 3}, {NULL}};
const Segment SGI74349[] = {{&S33245[103], 5}, {NULL}};
const Segment SGI74350[] = {{&S33247[44], 12}, {NULL}};
const Segment SGI74351[] = {{&S33247[44], 22}, {NULL}};
const Segment SGI74352[] = {{&S33247[30], 6}, {NULL}};
const Segment SGI74353[] = {{&S33247[114], 3}, {NULL}};
const Segment SGI74354[] = {{&S33247[80], 5}, {NULL}};
const Segment SGI74355[] = {{&S33251[44], 12}, {NULL}};
const Segment SGI74356[] = {{&S33251[44], 24}, {NULL}};
const Segment SGI74357[] = {{&S33251[30], 6}, {NULL}};
const Segment SGI74358[] = {{&S33251[85], 4}, {NULL}};
const Segment SGI74359[] = {{&S33251[85], 3}, {NULL}};
const Segment SGI74360[] = {{&S33254[44], 12}, {NULL}};
const Segment SGI74361[] = {{&S33254[44], 24}, {NULL}};
const Segment SGI74362[] = {{&S33254[30], 6}, {NULL}};
const Segment SGI74363[] = {{&S33254[85], 6}, {NULL}};
const Segment SGI74364[] = {{&S39784[44], 8}, {NULL}};
const Segment SGI74365[] = {{&S39784[44], 30}, {NULL}};
const Segment SGI74366[] = {{&S39784[30], 6}, {NULL}};
const Segment SGI74367[] = {{&S39784[122], 3}, {NULL}};
const Segment SGI74368[] = {{&S39784[88], 5}, {NULL}};
const Segment SGI74369[] = {{&S39789[44], 8}, {NULL}};
const Segment SGI74370[] = {{&S39789[44], 30}, {NULL}};
const Segment SGI74371[] = {{&S39789[30], 6}, {NULL}};
const Segment SGI74372[] = {{&S39789[91], 4}, {NULL}};
const Segment SGI74373[] = {{&S39789[91], 3}, {NULL}};
const Segment SGI74374[] = {{&S39790[44], 8}, {NULL}};
const Segment SGI74375[] = {{&S39790[44], 30}, {NULL}};
const Segment SGI74376[] = {{&S39790[30], 6}, {NULL}};
const Segment SGI74377[] = {{&S39790[122], 3}, {NULL}};
const Segment SGI74378[] = {{&S39790[88], 5}, {NULL}};
const Segment SGI74379[] = {{&S39791[44], 8}, {NULL}};
const Segment SGI74380[] = {{&S39791[44], 30}, {NULL}};
const Segment SGI74381[] = {{&S39791[30], 6}, {NULL}};
const Segment SGI74382[] = {{&S39791[91], 4}, {NULL}};
const Segment SGI74383[] = {{&S39791[91], 3}, {NULL}};
const Segment SGI74384[] = {{&S39792[44], 8}, {NULL}};
const Segment SGI74385[] = {{&S39792[44], 30}, {NULL}};
const Segment SGI74386[] = {{&S39792[30], 6}, {NULL}};
const Segment SGI74387[] = {{&S39792[122], 3}, {NULL}};
const Segment SGI74388[] = {{&S39792[88], 5}, {NULL}};
const Segment SGI74389[] = {{&S39795[44], 8}, {NULL}};
const Segment SGI74390[] = {{&S39795[44], 30}, {NULL}};
const Segment SGI74391[] = {{&S39795[30], 6}, {NULL}};
const Segment SGI74392[] = {{&S39795[122], 3}, {NULL}};
const Segment SGI74393[] = {{&S39795[88], 5}, {NULL}};
const Segment SGI74394[] = {{&S39798[44], 8}, {NULL}};
const Segment SGI74395[] = {{&S39798[44], 30}, {NULL}};
const Segment SGI74396[] = {{&S39798[30], 6}, {NULL}};
const Segment SGI74397[] = {{&S39798[91], 6}, {NULL}};
const Segment SGI74398[] = {{&S39799[44], 8}, {NULL}};
const Segment SGI74399[] = {{&S39799[44], 30}, {NULL}};
const Segment SGI74400[] = {{&S39799[30], 6}, {NULL}};
const Segment SGI74401[] = {{&S39799[122], 3}, {NULL}};
const Segment SGI74402[] = {{&S39799[88], 5}, {NULL}};
const Segment SGI74403[] = {{&S39800[44], 8}, {NULL}};
const Segment SGI74404[] = {{&S39800[44], 30}, {NULL}};
const Segment SGI74405[] = {{&S39800[30], 6}, {NULL}};
const Segment SGI74406[] = {{&S39800[91], 6}, {NULL}};
const Segment SGI74407[] = {{&S39810[38], 8}, {NULL}};
const Segment SGI74408[] = {{&S39810[31], 6}, {NULL}};
const Segment SGI74409[] = {{&S39811[44], 8}, {NULL}};
const Segment SGI74410[] = {{&S39811[44], 30}, {NULL}};
const Segment SGI74411[] = {{&S39811[30], 6}, {NULL}};
const Segment SGI74412[] = {{&S39811[91], 4}, {NULL}};
const Segment SGI74413[] = {{&S39811[91], 3}, {NULL}};
const Segment SGI74414[] = {{&S39812[44], 7}, {NULL}};
const Segment SGI74415[] = {{&S39812[44], 29}, {NULL}};
const Segment SGI74416[] = {{&S39812[30], 6}, {NULL}};
const Segment SGI74417[] = {{&S39812[121], 3}, {NULL}};
const Segment SGI74418[] = {{&S39812[87], 5}, {NULL}};
const Segment SGI74419[] = {{&S39813[44], 8}, {NULL}};
const Segment SGI74420[] = {{&S39813[44], 30}, {NULL}};
const Segment SGI74421[] = {{&S39813[30], 6}, {NULL}};
const Segment SGI74422[] = {{&S39813[91], 6}, {NULL}};
const Segment SGI74423[] = {{&S39815[44], 8}, {NULL}};
const Segment SGI74424[] = {{&S39815[44], 30}, {NULL}};
const Segment SGI74425[] = {{&S39815[30], 6}, {NULL}};
const Segment SGI74426[] = {{&S39815[91], 6}, {NULL}};
const Segment SGI74427[] = {{&S39819[44], 7}, {NULL}};
const Segment SGI74428[] = {{&S39819[44], 30}, {NULL}};
const Segment SGI74429[] = {{&S39819[30], 6}, {NULL}};
const Segment SGI74430[] = {{&S39819[122], 3}, {NULL}};
const Segment SGI74431[] = {{&S39819[88], 5}, {NULL}};
const Segment SGI74432[] = {{&S39850[44], 7}, {NULL}};
const Segment SGI74433[] = {{&S39850[44], 30}, {NULL}};
const Segment SGI74434[] = {{&S39850[30], 6}, {NULL}};
const Segment SGI74435[] = {{&S39850[122], 3}, {NULL}};
const Segment SGI74436[] = {{&S39850[88], 5}, {NULL}};
const Segment SGI74437[] = {{&S39851[44], 7}, {NULL}};
const Segment SGI74438[] = {{&S39851[44], 32}, {NULL}};
const Segment SGI74439[] = {{&S39851[30], 6}, {NULL}};
const Segment SGI74440[] = {{&S39851[124], 3}, {NULL}};
const Segment SGI74441[] = {{&S39851[90], 5}, {NULL}};
const Segment SGI74442[] = {{&S39852[44], 7}, {NULL}};
const Segment SGI74443[] = {{&S39852[44], 30}, {NULL}};
const Segment SGI74444[] = {{&S39852[30], 6}, {NULL}};
const Segment SGI74445[] = {{&S39852[122], 3}, {NULL}};
const Segment SGI74446[] = {{&S39852[88], 5}, {NULL}};
const Segment SGI74447[] = {{&S39853[44], 7}, {NULL}};
const Segment SGI74448[] = {{&S39853[44], 27}, {NULL}};
const Segment SGI74449[] = {{&S39853[30], 6}, {NULL}};
const Segment SGI74450[] = {{&S39853[119], 3}, {NULL}};
const Segment SGI74451[] = {{&S39853[85], 5}, {NULL}};
const Segment SGI74452[] = {{&S39862[38], 7}, {NULL}};
const Segment SGI74453[] = {{&S39862[31], 6}, {NULL}};
const Segment SGI74454[] = {{&S39866[44], 12}, {NULL}};
const Segment SGI74455[] = {{&S39866[44], 35}, {NULL}};
const Segment SGI74456[] = {{&S39866[30], 6}, {NULL}};
const Segment SGI74457[] = {{&S39866[127], 3}, {NULL}};
const Segment SGI74458[] = {{&S39866[93], 5}, {NULL}};
const Segment SGI74459[] = {{&S39867[38], 12}, {NULL}};
const Segment SGI74460[] = {{&S39867[31], 6}, {NULL}};
const Segment SGI74461[] = {{&S39868[38], 7}, {NULL}};
const Segment SGI74462[] = {{&S39868[31], 6}, {NULL}};
const Segment SGI74463[] = {{&S39871[44], 10}, {NULL}};
const Segment SGI74464[] = {{&S39871[44], 27}, {NULL}};
const Segment SGI74465[] = {{&S39871[30], 6}, {NULL}};
const Segment SGI74466[] = {{&S39871[119], 3}, {NULL}};
const Segment SGI74467[] = {{&S39871[85], 5}, {NULL}};
const Segment SGI74468[] = {{&S39873[38], 10}, {NULL}};
const Segment SGI74469[] = {{&S39873[31], 6}, {NULL}};
const Segment SGI74470[] = {{&S39875[56], 10}, {NULL}};
const Segment SGI74471[] = {{&S39875[56], 27}, {NULL}};
const Segment SGI74472[] = {{&S39875[30], 18}, {NULL}};
const Segment SGI74473[] = {{&S39875[131], 3}, {NULL}};
const Segment SGI74474[] = {{&S39875[97], 5}, {NULL}};
const Segment SGI74475[] = {{&S39876[42], 10}, {NULL}};
const Segment SGI74476[] = {{&S39876[30], 6}, {NULL}};
const Segment SGI74477[] = {{&S39876[117], 3}, {NULL}};
const Segment SGI74478[] = {{&S39876[83], 5}, {NULL}};
const Segment SGI74479[] = {{&S33157[42], 6}, {NULL}};
const Segment SGI74480[] = {{&S33157[42], 18}, {NULL}};
const Segment SGI74481[] = {{&S33157[30], 4}, {NULL}};
const Segment SGI74482[] = {{&S33157[109], 3}, {NULL}};
const Segment SGI74483[] = {{&S33157[74], 6}, {NULL}};
const Segment SGI74484[] = {{&S33158[42], 6}, {NULL}};
const Segment SGI74485[] = {{&S33158[42], 18}, {NULL}};
const Segment SGI74486[] = {{&S33158[30], 4}, {NULL}};
const Segment SGI74487[] = {{&S33158[109], 3}, {NULL}};
const Segment SGI74488[] = {{&S33158[74], 6}, {NULL}};
const Segment SGI74489[] = {{&S33159[44], 6}, {NULL}};
const Segment SGI74490[] = {{&S33159[44], 18}, {NULL}};
const Segment SGI74491[] = {{&S33159[30], 6}, {NULL}};
const Segment SGI74492[] = {{&S33159[111], 3}, {NULL}};
const Segment SGI74493[] = {{&S33159[76], 6}, {NULL}};
const Segment SGI74494[] = {{&S33160[36], 6}, {NULL}};
const Segment SGI74495[] = {{&S33160[31], 4}, {NULL}};
const Segment SGI74496[] = {{&S33161[44], 6}, {NULL}};
const Segment SGI74497[] = {{&S33161[44], 39}, {NULL}};
const Segment SGI74498[] = {{&S33161[30], 6}, {NULL}};
const Segment SGI74499[] = {{&S33162[42], 6}, {NULL}};
const Segment SGI74500[] = {{&S33162[30], 6}, {NULL}};
const Segment SGI74501[] = {{&S33162[109], 3}, {NULL}};
const Segment SGI74502[] = {{&S33162[74], 6}, {NULL}};
const Segment SGI74503[] = {{&S33163[38], 6}, {NULL}};
const Segment SGI74504[] = {{&S33163[31], 6}, {NULL}};
const Segment SGI74505[] = {{&S39931[44], 10}, {NULL}};
const Segment SGI74506[] = {{&S39931[44], 27}, {NULL}};
const Segment SGI74507[] = {{&S39931[30], 6}, {NULL}};
const Segment SGI74508[] = {{&S39931[119], 3}, {NULL}};
const Segment SGI74509[] = {{&S39931[85], 5}, {NULL}};
const Segment SGI74510[] = {{&S39932[38], 10}, {NULL}};
const Segment SGI74511[] = {{&S39932[31], 6}, {NULL}};
const Segment SGI74512[] = {{&S33164[44], 6}, {NULL}};
const Segment SGI74513[] = {{&S33164[44], 18}, {NULL}};
const Segment SGI74514[] = {{&S33164[30], 6}, {NULL}};
const Segment SGI74515[] = {{&S33164[111], 3}, {NULL}};
const Segment SGI74516[] = {{&S33164[76], 6}, {NULL}};
const Segment SGI74517[] = {{&S33165[44], 6}, {NULL}};
const Segment SGI74518[] = {{&S33165[44], 18}, {NULL}};
const Segment SGI74519[] = {{&S33165[30], 6}, {NULL}};
const Segment SGI74520[] = {{&S33165[111], 3}, {NULL}};
const Segment SGI74521[] = {{&S33165[76], 6}, {NULL}};
const Segment SGI74522[] = {{&S33166[44], 6}, {NULL}};
const Segment SGI74523[] = {{&S33166[44], 17}, {NULL}};
const Segment SGI74524[] = {{&S33166[30], 6}, {NULL}};
const Segment SGI74525[] = {{&S33166[110], 3}, {NULL}};
const Segment SGI74526[] = {{&S33166[75], 6}, {NULL}};
const Segment SGI74527[] = {{&S22837[44], 16}, {NULL}};
const Segment SGI74528[] = {{&S22837[44], 28}, {NULL}};
const Segment SGI74529[] = {{&S22837[30], 6}, {NULL}};
const Segment SGI74530[] = {{&S22837[31], 3}, {NULL}};
const Segment SGI74531[] = {{&S22837[86], 6}, {NULL}};
const Segment SGI74532[] = {{&S22838[44], 9}, {NULL}};
const Segment SGI74533[] = {{&S22838[44], 21}, {NULL}};
const Segment SGI74534[] = {{&S22838[30], 6}, {NULL}};
const Segment SGI74535[] = {{&S22838[31], 3}, {NULL}};
const Segment SGI74536[] = {{&S22838[79], 6}, {NULL}};
const Segment SGI74537[] = {{&S22839[44], 9}, {NULL}};
const Segment SGI74538[] = {{&S22839[44], 21}, {NULL}};
const Segment SGI74539[] = {{&S22839[30], 6}, {NULL}};
const Segment SGI74540[] = {{&S22839[31], 3}, {NULL}};
const Segment SGI74541[] = {{&S22839[79], 6}, {NULL}};
const Segment SGI74542[] = {{&S22840[44], 10}, {NULL}};
const Segment SGI74543[] = {{&S22840[44], 22}, {NULL}};
const Segment SGI74544[] = {{&S22840[30], 6}, {NULL}};
const Segment SGI74545[] = {{&S40000[44], 18}, {NULL}};
const Segment SGI74546[] = {{&S40000[44], 35}, {NULL}};
const Segment SGI74547[] = {{&S40000[30], 6}, {NULL}};
const Segment SGI74548[] = {{&S22840[31], 3}, {NULL}};
const Segment SGI74549[] = {{&S22840[80], 6}, {NULL}};
const Segment SGI74550[] = {{&S40000[127], 3}, {NULL}};
const Segment SGI74551[] = {{&S40000[93], 5}, {NULL}};
const Segment SGI74552[] = {{&S22841[44], 14}, {NULL}};
const Segment SGI74553[] = {{&S22841[44], 26}, {NULL}};
const Segment SGI74554[] = {{&S22841[30], 6}, {NULL}};
const Segment SGI74555[] = {{&S40001[38], 18}, {NULL}};
const Segment SGI74556[] = {{&S40001[31], 6}, {NULL}};
const Segment SGI74557[] = {{&S22841[31], 3}, {NULL}};
const Segment SGI74558[] = {{&S22841[84], 6}, {NULL}};
const Segment SGI74559[] = {{&S22842[44], 15}, {NULL}};
const Segment SGI74560[] = {{&S22842[44], 27}, {NULL}};
const Segment SGI74561[] = {{&S22842[30], 6}, {NULL}};
const Segment SGI74562[] = {{&S40002[44], 17}, {NULL}};
const Segment SGI74563[] = {{&S40002[44], 34}, {NULL}};
const Segment SGI74564[] = {{&S40002[30], 6}, {NULL}};
const Segment SGI74565[] = {{&S22842[31], 3}, {NULL}};
const Segment SGI74566[] = {{&S22842[85], 6}, {NULL}};
const Segment SGI74567[] = {{&S40002[126], 3}, {NULL}};
const Segment SGI74568[] = {{&S40002[92], 5}, {NULL}};
const Segment SGI74569[] = {{&S22843[44], 15}, {NULL}};
const Segment SGI74570[] = {{&S22843[44], 27}, {NULL}};
const Segment SGI74571[] = {{&S22843[30], 6}, {NULL}};
const Segment SGI74572[] = {{&S40003[43], 18}, {NULL}};
const Segment SGI74573[] = {{&S22843[31], 3}, {NULL}};
const Segment SGI74574[] = {{&S40003[43], 35}, {NULL}};
const Segment SGI74575[] = {{&S40003[30], 5}, {NULL}};
const Segment SGI74576[] = {{&S22843[85], 6}, {NULL}};
const Segment SGI74577[] = {{&S40003[126], 3}, {NULL}};
const Segment SGI74578[] = {{&S40003[92], 5}, {NULL}};
const Segment SGI74579[] = {{&S22844[44], 14}, {NULL}};
const Segment SGI74580[] = {{&S22844[44], 26}, {NULL}};
const Segment SGI74581[] = {{&S22844[30], 6}, {NULL}};
const Segment SGI74582[] = {{&S22844[31], 3}, {NULL}};
const Segment SGI74583[] = {{&S40004[44], 18}, {NULL}};
const Segment SGI74584[] = {{&S22844[84], 6}, {NULL}};
const Segment SGI74585[] = {{&S40004[44], 35}, {NULL}};
const Segment SGI74586[] = {{&S40004[30], 6}, {NULL}};
const Segment SGI74587[] = {{&S40004[127], 3}, {NULL}};
const Segment SGI74588[] = {{&S40004[93], 5}, {NULL}};
const Segment SGI74589[] = {{&S22845[44], 10}, {NULL}};
const Segment SGI74590[] = {{&S22845[44], 22}, {NULL}};
const Segment SGI74591[] = {{&S22845[30], 6}, {NULL}};
const Segment SGI74592[] = {{&S40005[43], 18}, {NULL}};
const Segment SGI74593[] = {{&S40005[43], 35}, {NULL}};
const Segment SGI74594[] = {{&S40005[126], 3}, {NULL}};
const Segment SGI74595[] = {{&S40005[92], 5}, {NULL}};
const Segment SGI74596[] = {{&S40006[44], 17}, {NULL}};
const Segment SGI74597[] = {{&S40006[44], 34}, {NULL}};
const Segment SGI74598[] = {{&S40006[30], 6}, {NULL}};
const Segment SGI74599[] = {{&S40006[126], 3}, {NULL}};
const Segment SGI74600[] = {{&S40006[92], 5}, {NULL}};
const Segment SGI74601[] = {{&S40007[44], 17}, {NULL}};
const Segment SGI74602[] = {{&S40007[44], 34}, {NULL}};
const Segment SGI74603[] = {{&S40007[30], 6}, {NULL}};
const Segment SGI74604[] = {{&S40007[126], 3}, {NULL}};
const Segment SGI74605[] = {{&S40007[92], 5}, {NULL}};
const Segment SGI74606[] = {{&S40027[38], 11}, {NULL}};
const Segment SGI74607[] = {{&S40027[31], 6}, {NULL}};
const Segment SGI74608[] = {{&S40028[44], 11}, {NULL}};
const Segment SGI74609[] = {{&S40028[44], 23}, {NULL}};
const Segment SGI74610[] = {{&S40028[30], 6}, {NULL}};
const Segment SGI74611[] = {{&S40028[115], 3}, {NULL}};
const Segment SGI74612[] = {{&S40028[81], 5}, {NULL}};
const Segment SGI74613[] = {{&S40029[44], 10}, {NULL}};
const Segment SGI74614[] = {{&S40029[30], 6}, {NULL}};
const Segment SGI74615[] = {{&S40029[104], 3}, {NULL}};
const Segment SGI74616[] = {{&S40029[70], 5}, {NULL}};
const Segment SGI74617[] = {{&S40040[44], 23}, {NULL}};
const Segment SGI74618[] = {{&S40040[30], 6}, {NULL}};
const Segment SGI74619[] = {{&S40040[117], 3}, {NULL}};
const Segment SGI74620[] = {{&S40040[83], 5}, {NULL}};
const Segment SGI74621[] = {{&S40045[44], 11}, {NULL}};
const Segment SGI74622[] = {{&S40045[44], 23}, {NULL}};
const Segment SGI74623[] = {{&S40045[30], 6}, {NULL}};
const Segment SGI74624[] = {{&S40045[84], 6}, {NULL}};
const Segment SGI74625[] = {{&S33748[44], 14}, {NULL}};
const Segment SGI74626[] = {{&S33748[44], 25}, {NULL}};
const Segment SGI74627[] = {{&S33748[30], 6}, {NULL}};
const Segment SGI74628[] = {{&S33748[117], 3}, {NULL}};
const Segment SGI74629[] = {{&S33748[83], 5}, {NULL}};
const Segment SGI74630[] = {{&S33755[38], 14}, {NULL}};
const Segment SGI74631[] = {{&S33755[31], 6}, {NULL}};
const Segment SGI74632[] = {{&S22845[31], 3}, {NULL}};
const Segment SGI74633[] = {{&S22845[80], 6}, {NULL}};
const Segment SGI74634[] = {{&S22846[38], 9}, {NULL}};
const Segment SGI74635[] = {{&S22846[31], 6}, {NULL}};
const Segment SGI74636[] = {{&S22847[41], 9}, {NULL}};
const Segment SGI74637[] = {{&S22847[30], 6}, {NULL}};
const Segment SGI74638[] = {{&S22847[31], 3}, {NULL}};
const Segment SGI74639[] = {{&S22847[76], 6}, {NULL}};
const Segment SGI74640[] = {{&S22848[44], 14}, {NULL}};
const Segment SGI74641[] = {{&S22848[44], 26}, {NULL}};
const Segment SGI74642[] = {{&S22848[30], 6}, {NULL}};
const Segment SGI74643[] = {{&S22848[31], 3}, {NULL}};
const Segment SGI74644[] = {{&S22848[84], 6}, {NULL}};
const Segment SGI74645[] = {{&S33802[44], 8}, {NULL}};
const Segment SGI74646[] = {{&S33802[44], 17}, {NULL}};
const Segment SGI74647[] = {{&S33802[30], 6}, {NULL}};
const Segment SGI74648[] = {{&S33802[109], 3}, {NULL}};
const Segment SGI74649[] = {{&S33802[75], 5}, {NULL}};
const Segment SGI74650[] = {{&S22849[38], 9}, {NULL}};
const Segment SGI74651[] = {{&S22849[31], 6}, {NULL}};
const Segment SGI74652[] = {{&S22850[44], 10}, {NULL}};
const Segment SGI74653[] = {{&S22850[44], 22}, {NULL}};
const Segment SGI74654[] = {{&S22850[30], 6}, {NULL}};
const Segment SGI74655[] = {{&S22850[31], 3}, {NULL}};
const Segment SGI74656[] = {{&S22850[80], 6}, {NULL}};
const Segment SGI74657[] = {{&S22851[44], 14}, {NULL}};
const Segment SGI74658[] = {{&S22851[44], 26}, {NULL}};
const Segment SGI74659[] = {{&S22851[30], 6}, {NULL}};
const Segment SGI74660[] = {{&S22851[31], 3}, {NULL}};
const Segment SGI74661[] = {{&S22851[84], 6}, {NULL}};
const Segment SGI74662[] = {{&S22852[38], 10}, {NULL}};
const Segment SGI74663[] = {{&S22852[31], 6}, {NULL}};
const Segment SGI74664[] = {{&S22853[44], 14}, {NULL}};
const Segment SGI74665[] = {{&S22853[44], 26}, {NULL}};
const Segment SGI74666[] = {{&S22853[30], 6}, {NULL}};
const Segment SGI74667[] = {{&S22853[31], 3}, {NULL}};
const Segment SGI74668[] = {{&S22853[84], 6}, {NULL}};
const Segment SGI74669[] = {{&S22854[44], 15}, {NULL}};
const Segment SGI74670[] = {{&S22854[44], 27}, {NULL}};
const Segment SGI74671[] = {{&S22854[30], 6}, {NULL}};
const Segment SGI74672[] = {{&S22854[31], 3}, {NULL}};
const Segment SGI74673[] = {{&S22854[85], 6}, {NULL}};
const Segment SGI74674[] = {{&S22855[38], 10}, {NULL}};
const Segment SGI74675[] = {{&S22855[31], 6}, {NULL}};
const Segment SGI74676[] = {{&S22856[38], 10}, {NULL}};
const Segment SGI74677[] = {{&S22856[31], 6}, {NULL}};
const Segment SGI74678[] = {{&S33816[38], 8}, {NULL}};
const Segment SGI74679[] = {{&S33816[31], 6}, {NULL}};
const Segment SGI74680[] = {{&S22857[44], 14}, {NULL}};
const Segment SGI74681[] = {{&S22857[44], 26}, {NULL}};
const Segment SGI74682[] = {{&S22857[30], 6}, {NULL}};
const Segment SGI74683[] = {{&S22857[31], 3}, {NULL}};
const Segment SGI74684[] = {{&S22857[84], 6}, {NULL}};
const Segment SGI74685[] = {{&S22858[44], 14}, {NULL}};
const Segment SGI74686[] = {{&S22858[44], 26}, {NULL}};
const Segment SGI74687[] = {{&S22858[30], 6}, {NULL}};
const Segment SGI74688[] = {{&S22858[31], 3}, {NULL}};
const Segment SGI74689[] = {{&S22858[84], 6}, {NULL}};
const Segment SGI74690[] = {{&S22859[44], 15}, {NULL}};
const Segment SGI74691[] = {{&S22859[44], 27}, {NULL}};
const Segment SGI74692[] = {{&S22859[30], 6}, {NULL}};
const Segment SGI74693[] = {{&S22859[31], 3}, {NULL}};
const Segment SGI74694[] = {{&S22859[85], 6}, {NULL}};
const Segment SGI74695[] = {{&S22860[44], 14}, {NULL}};
const Segment SGI74696[] = {{&S22860[44], 26}, {NULL}};
const Segment SGI74697[] = {{&S22860[30], 6}, {NULL}};
const Segment SGI74698[] = {{&S22860[31], 3}, {NULL}};
const Segment SGI74699[] = {{&S22860[84], 6}, {NULL}};
const Segment SGI74700[] = {{&S22861[44], 14}, {NULL}};
const Segment SGI74701[] = {{&S22861[44], 26}, {NULL}};
const Segment SGI74702[] = {{&S22861[30], 6}, {NULL}};
const Segment SGI74703[] = {{&S22861[31], 3}, {NULL}};
const Segment SGI74704[] = {{&S22861[84], 6}, {NULL}};
const Segment SGI74705[] = {{&S40242[44], 10}, {NULL}};
const Segment SGI74706[] = {{&S40242[44], 27}, {NULL}};
const Segment SGI74707[] = {{&S40242[30], 6}, {NULL}};
const Segment SGI74708[] = {{&S40242[119], 3}, {NULL}};
const Segment SGI74709[] = {{&S40242[85], 5}, {NULL}};
const Segment SGI74710[] = {{&S40243[38], 10}, {NULL}};
const Segment SGI74711[] = {{&S40243[31], 6}, {NULL}};
const Segment SGI74712[] = {{&S33956[44], 14}, {NULL}};
const Segment SGI74713[] = {{&S33956[44], 30}, {NULL}};
const Segment SGI74714[] = {{&S33956[30], 6}, {NULL}};
const Segment SGI74715[] = {{&S33956[122], 3}, {NULL}};
const Segment SGI74716[] = {{&S33956[88], 5}, {NULL}};
const Segment SGI74717[] = {{&S40274[44], 13}, {NULL}};
const Segment SGI74718[] = {{&S40274[44], 30}, {NULL}};
const Segment SGI74719[] = {{&S40274[30], 6}, {NULL}};
const Segment SGI74720[] = {{&S40274[122], 3}, {NULL}};
const Segment SGI74721[] = {{&S40274[88], 5}, {NULL}};
const Segment SGI74722[] = {{&S40275[38], 13}, {NULL}};
const Segment SGI74723[] = {{&S40275[31], 6}, {NULL}};
const Segment SGI74724[] = {{&S40276[42], 13}, {NULL}};
const Segment SGI74725[] = {{&S40276[30], 6}, {NULL}};
const Segment SGI74726[] = {{&S40276[120], 3}, {NULL}};
const Segment SGI74727[] = {{&S40276[86], 5}, {NULL}};
const Segment SGI74728[] = {{&S34004[44], 9}, {NULL}};
const Segment SGI74729[] = {{&S34004[44], 20}, {NULL}};
const Segment SGI74730[] = {{&S34004[30], 6}, {NULL}};
const Segment SGI74731[] = {{&S34004[128], 3}, {NULL}};
const Segment SGI74732[] = {{&S34004[94], 5}, {NULL}};
const Segment SGI74733[] = {{&S40355[44], 23}, {NULL}};
const Segment SGI74734[] = {{&S40355[44], 40}, {NULL}};
const Segment SGI74735[] = {{&S40355[30], 6}, {NULL}};
const Segment SGI74736[] = {{&S40355[132], 3}, {NULL}};
const Segment SGI74737[] = {{&S23107[44], 10}, {NULL}};
const Segment SGI74738[] = {{&S23107[44], 22}, {NULL}};
const Segment SGI74739[] = {{&S23107[30], 6}, {NULL}};
const Segment SGI74740[] = {{&S23107[31], 3}, {NULL}};
const Segment SGI74741[] = {{&S23107[80], 6}, {NULL}};
const Segment SGI74742[] = {{&S23109[38], 10}, {NULL}};
const Segment SGI74743[] = {{&S23109[31], 6}, {NULL}};
const Segment SGI74744[] = {{&S23112[44], 10}, {NULL}};
const Segment SGI74745[] = {{&S23112[44], 21}, {NULL}};
const Segment SGI74746[] = {{&S23112[30], 6}, {NULL}};
const Segment SGI74747[] = {{&S23112[31], 3}, {NULL}};
const Segment SGI74748[] = {{&S23112[79], 6}, {NULL}};
const Segment SGI74749[] = {{&S40355[98], 5}, {NULL}};
const Segment SGI74750[] = {{&S40356[44], 24}, {NULL}};
const Segment SGI74751[] = {{&S40356[44], 41}, {NULL}};
const Segment SGI74752[] = {{&S40356[30], 6}, {NULL}};
const Segment SGI74753[] = {{&S40356[133], 3}, {NULL}};
const Segment SGI74754[] = {{&S40356[99], 5}, {NULL}};
const Segment SGI74755[] = {{&S40357[44], 24}, {NULL}};
const Segment SGI74756[] = {{&S40357[44], 41}, {NULL}};
const Segment SGI74757[] = {{&S40357[30], 6}, {NULL}};
const Segment SGI74758[] = {{&S40357[133], 3}, {NULL}};
const Segment SGI74759[] = {{&S40357[99], 5}, {NULL}};
const Segment SGI74760[] = {{&S40382[44], 10}, {NULL}};
const Segment SGI74761[] = {{&S40382[44], 27}, {NULL}};
const Segment SGI74762[] = {{&S40382[30], 6}, {NULL}};
const Segment SGI74763[] = {{&S40382[119], 3}, {NULL}};
const Segment SGI74764[] = {{&S40382[85], 5}, {NULL}};
const Segment SGI74765[] = {{&S40383[44], 11}, {NULL}};
const Segment SGI74766[] = {{&S40383[44], 28}, {NULL}};
const Segment SGI74767[] = {{&S40383[30], 6}, {NULL}};
const Segment SGI74768[] = {{&S40383[120], 3}, {NULL}};
const Segment SGI74769[] = {{&S40383[86], 5}, {NULL}};
const Segment SGI74770[] = {{&S40384[42], 10}, {NULL}};
const Segment SGI74771[] = {{&S40384[30], 6}, {NULL}};
const Segment SGI74772[] = {{&S40384[117], 3}, {NULL}};
const Segment SGI74773[] = {{&S40384[83], 5}, {NULL}};
const Segment SGI74774[] = {{&S40385[38], 10}, {NULL}};
const Segment SGI74775[] = {{&S40385[31], 6}, {NULL}};
const Segment SGI74776[] = {{&S40386[44], 10}, {NULL}};
const Segment SGI74777[] = {{&S40386[44], 27}, {NULL}};
const Segment SGI74778[] = {{&S40386[30], 6}, {NULL}};
const Segment SGI74779[] = {{&S40386[88], 4}, {NULL}};
const Segment SGI74780[] = {{&S40386[49], 3}, {NULL}};
const Segment SGI74781[] = {{&S40387[38], 11}, {NULL}};
const Segment SGI74782[] = {{&S40387[31], 6}, {NULL}};
const Segment SGI74783[] = {{&S40432[44], 11}, {NULL}};
const Segment SGI74784[] = {{&S40432[44], 28}, {NULL}};
const Segment SGI74785[] = {{&S40432[30], 6}, {NULL}};
const Segment SGI74786[] = {{&S40432[120], 3}, {NULL}};
const Segment SGI74787[] = {{&S40432[86], 5}, {NULL}};
const Segment SGI74788[] = {{&S40433[44], 11}, {NULL}};
const Segment SGI74789[] = {{&S40433[44], 28}, {NULL}};
const Segment SGI74790[] = {{&S40433[30], 6}, {NULL}};
const Segment SGI74791[] = {{&S40433[120], 3}, {NULL}};
const Segment SGI74792[] = {{&S40433[86], 5}, {NULL}};
const Segment SGI74793[] = {{&S40434[38], 11}, {NULL}};
const Segment SGI74794[] = {{&S40434[31], 6}, {NULL}};
const Segment SGI74795[] = {{&S40435[38], 11}, {NULL}};
const Segment SGI74796[] = {{&S40435[31], 6}, {NULL}};
const Segment SGI74797[] = {{&S40436[44], 11}, {NULL}};
const Segment SGI74798[] = {{&S40436[44], 28}, {NULL}};
const Segment SGI74799[] = {{&S40436[30], 6}, {NULL}};
const Segment SGI74800[] = {{&S40436[120], 3}, {NULL}};
const Segment SGI74801[] = {{&S40436[86], 5}, {NULL}};
const Segment SGI74802[] = {{&S40437[44], 11}, {NULL}};
const Segment SGI74803[] = {{&S40437[44], 28}, {NULL}};
const Segment SGI74804[] = {{&S40437[30], 6}, {NULL}};
const Segment SGI74805[] = {{&S40437[89], 4}, {NULL}};
const Segment SGI74806[] = {{&S40437[89], 3}, {NULL}};
const Segment SGI74807[] = {{&S40438[38], 11}, {NULL}};
const Segment SGI74808[] = {{&S40438[31], 6}, {NULL}};
const Segment SGI74809[] = {{&S40439[44], 11}, {NULL}};
const Segment SGI74810[] = {{&S40439[44], 28}, {NULL}};
const Segment SGI74811[] = {{&S40439[30], 6}, {NULL}};
const Segment SGI74812[] = {{&S40439[89], 6}, {NULL}};
const Segment SGI74813[] = {{&S40440[44], 11}, {NULL}};
const Segment SGI74814[] = {{&S40440[44], 28}, {NULL}};
const Segment SGI74815[] = {{&S40440[30], 6}, {NULL}};
const Segment SGI74816[] = {{&S40440[89], 6}, {NULL}};
const Segment SGI74817[] = {{&S27639[44], 15}, {NULL}};
const Segment SGI74818[] = {{&S27639[44], 27}, {NULL}};
const Segment SGI74819[] = {{&S27639[30], 6}, {NULL}};
const Segment SGI74820[] = {{&S27639[31], 3}, {NULL}};
const Segment SGI74821[] = {{&S27639[85], 6}, {NULL}};
const Segment SGI74822[] = {{&S27639[0], 13}, {NULL}};
const Segment SGI74823[] = {{&S23322[44], 15}, {NULL}};
const Segment SGI74824[] = {{&S23322[44], 29}, {NULL}};
const Segment SGI74825[] = {{&S23322[30], 6}, {NULL}};
const Segment SGI74826[] = {{&S23322[31], 3}, {NULL}};
const Segment SGI74827[] = {{&S23322[87], 6}, {NULL}};
const Segment SGI74828[] = {{&S23323[44], 15}, {NULL}};
const Segment SGI74829[] = {{&S23323[44], 29}, {NULL}};
const Segment SGI74830[] = {{&S23323[30], 6}, {NULL}};
const Segment SGI74831[] = {{&S23323[31], 3}, {NULL}};
const Segment SGI74832[] = {{&S23323[87], 6}, {NULL}};
const Segment SGI74833[] = {{&S23324[44], 15}, {NULL}};
const Segment SGI74834[] = {{&S23324[44], 29}, {NULL}};
const Segment SGI74835[] = {{&S23324[30], 6}, {NULL}};
const Segment SGI74836[] = {{&S23324[31], 3}, {NULL}};
const Segment SGI74837[] = {{&S23324[87], 6}, {NULL}};
const Segment SGI74838[] = {{&S23325[44], 15}, {NULL}};
const Segment SGI74839[] = {{&S23325[44], 31}, {NULL}};
const Segment SGI74840[] = {{&S23325[30], 6}, {NULL}};
const Segment SGI74841[] = {{&S23325[31], 3}, {NULL}};
const Segment SGI74842[] = {{&S23325[89], 6}, {NULL}};
const Segment SGI74843[] = {{&S23326[44], 15}, {NULL}};
const Segment SGI74844[] = {{&S23326[44], 29}, {NULL}};
const Segment SGI74845[] = {{&S23326[30], 6}, {NULL}};
const Segment SGI74846[] = {{&S23326[31], 3}, {NULL}};
const Segment SGI74847[] = {{&S23326[87], 6}, {NULL}};
const Segment SGI74848[] = {{&S23327[44], 15}, {NULL}};
const Segment SGI74849[] = {{&S23327[44], 29}, {NULL}};
const Segment SGI74850[] = {{&S23327[30], 6}, {NULL}};
const Segment SGI74851[] = {{&S23327[31], 3}, {NULL}};
const Segment SGI74852[] = {{&S23327[87], 6}, {NULL}};
const Segment SGI74853[] = {{&S23328[44], 15}, {NULL}};
const Segment SGI74854[] = {{&S23328[44], 29}, {NULL}};
const Segment SGI74855[] = {{&S23328[30], 6}, {NULL}};
const Segment SGI74856[] = {{&S23328[31], 3}, {NULL}};
const Segment SGI74857[] = {{&S23328[87], 6}, {NULL}};
const Segment SGI74858[] = {{&S23330[44], 15}, {NULL}};
const Segment SGI74859[] = {{&S23330[44], 29}, {NULL}};
const Segment SGI74860[] = {{&S23330[30], 6}, {NULL}};
const Segment SGI74861[] = {{&S23330[31], 3}, {NULL}};
const Segment SGI74862[] = {{&S23330[87], 6}, {NULL}};
const Segment SGI74863[] = {{&S23331[38], 15}, {NULL}};
const Segment SGI74864[] = {{&S23331[31], 6}, {NULL}};
const Segment SGI74865[] = {{&S23333[44], 15}, {NULL}};
const Segment SGI74866[] = {{&S23333[44], 29}, {NULL}};
const Segment SGI74867[] = {{&S23333[30], 6}, {NULL}};
const Segment SGI74868[] = {{&S23333[31], 3}, {NULL}};
const Segment SGI74869[] = {{&S23333[87], 6}, {NULL}};
const Segment SGI74870[] = {{&S23334[42], 15}, {NULL}};
const Segment SGI74871[] = {{&S23334[30], 6}, {NULL}};
const Segment SGI74872[] = {{&S23334[31], 3}, {NULL}};
const Segment SGI74873[] = {{&S23334[85], 6}, {NULL}};
const Segment SGI74874[] = {{&S23357[44], 10}, {NULL}};
const Segment SGI74875[] = {{&S23357[44], 22}, {NULL}};
const Segment SGI74876[] = {{&S23357[30], 6}, {NULL}};
const Segment SGI74877[] = {{&S23357[31], 3}, {NULL}};
const Segment SGI74878[] = {{&S23357[80], 6}, {NULL}};
const Segment SGI74879[] = {{&S23358[44], 18}, {NULL}};
const Segment SGI74880[] = {{&S23358[44], 30}, {NULL}};
const Segment SGI74881[] = {{&S23358[30], 6}, {NULL}};
const Segment SGI74882[] = {{&S23358[31], 3}, {NULL}};
const Segment SGI74883[] = {{&S23358[88], 6}, {NULL}};
const Segment SGI74884[] = {{&S23360[44], 10}, {NULL}};
const Segment SGI74885[] = {{&S23360[44], 22}, {NULL}};
const Segment SGI74886[] = {{&S23360[30], 6}, {NULL}};
const Segment SGI74887[] = {{&S23360[31], 3}, {NULL}};
const Segment SGI74888[] = {{&S23360[80], 6}, {NULL}};
const Segment SGI74889[] = {{&S23361[38], 10}, {NULL}};
const Segment SGI74890[] = {{&S23361[31], 6}, {NULL}};
const Segment SGI74891[] = {{&S23362[44], 10}, {NULL}};
const Segment SGI74892[] = {{&S23362[44], 21}, {NULL}};
const Segment SGI74893[] = {{&S23362[30], 6}, {NULL}};
const Segment SGI74894[] = {{&S23362[31], 3}, {NULL}};
const Segment SGI74895[] = {{&S23362[79], 6}, {NULL}};
const Segment SGI74896[] = {{&S23366[44], 11}, {NULL}};
const Segment SGI74897[] = {{&S23366[44], 23}, {NULL}};
const Segment SGI74898[] = {{&S23366[30], 6}, {NULL}};
const Segment SGI74899[] = {{&S23366[31], 3}, {NULL}};
const Segment SGI74900[] = {{&S23366[81], 6}, {NULL}};
const Segment SGI74901[] = {{&S23367[38], 11}, {NULL}};
const Segment SGI74902[] = {{&S23367[31], 6}, {NULL}};
const Segment SGI74903[] = {{&S23372[44], 10}, {NULL}};
const Segment SGI74904[] = {{&S23372[44], 19}, {NULL}};
const Segment SGI74905[] = {{&S23372[30], 6}, {NULL}};
const Segment SGI74906[] = {{&S23372[31], 3}, {NULL}};
const Segment SGI74907[] = {{&S23372[77], 6}, {NULL}};
const Segment SGI74908[] = {{&S23373[44], 10}, {NULL}};
const Segment SGI74909[] = {{&S23373[44], 22}, {NULL}};
const Segment SGI74910[] = {{&S23373[30], 6}, {NULL}};
const Segment SGI74911[] = {{&S23373[80], 6}, {NULL}};
const Segment SGI74912[] = {{&S23377[42], 10}, {NULL}};
const Segment SGI74913[] = {{&S23377[30], 6}, {NULL}};
const Segment SGI74914[] = {{&S23377[31], 3}, {NULL}};
const Segment SGI74915[] = {{&S23377[78], 6}, {NULL}};
const Segment SGI74916[] = {{&S40720[44], 10}, {NULL}};
const Segment SGI74917[] = {{&S40720[44], 22}, {NULL}};
const Segment SGI74918[] = {{&S40720[30], 6}, {NULL}};
const Segment SGI74919[] = {{&S40720[114], 3}, {NULL}};
const Segment SGI74920[] = {{&S40720[80], 5}, {NULL}};
const Segment SGI74921[] = {{&S40721[44], 9}, {NULL}};
const Segment SGI74922[] = {{&S40721[44], 21}, {NULL}};
const Segment SGI74923[] = {{&S40721[30], 6}, {NULL}};
const Segment SGI74924[] = {{&S40721[113], 3}, {NULL}};
const Segment SGI74925[] = {{&S40721[79], 5}, {NULL}};
const Segment SGI74926[] = {{&S40722[38], 9}, {NULL}};
const Segment SGI74927[] = {{&S40722[31], 6}, {NULL}};
const Segment SGI74928[] = {{&S40723[38], 10}, {NULL}};
const Segment SGI74929[] = {{&S40723[31], 6}, {NULL}};
const Segment SGI74930[] = {{&S30312[42], 14}, {NULL}};
const Segment SGI74931[] = {{&S30312[42], 26}, {NULL}};
const Segment SGI74932[] = {{&S30312[30], 4}, {NULL}};
const Segment SGI74933[] = {{&S30312[31], 3}, {NULL}};
const Segment SGI74934[] = {{&S30312[82], 6}, {NULL}};
const Segment SGI74935[] = {{&S34547[44], 19}, {NULL}};
const Segment SGI74936[] = {{&S34547[44], 36}, {NULL}};
const Segment SGI74937[] = {{&S34547[30], 6}, {NULL}};
const Segment SGI74938[] = {{&S34547[128], 3}, {NULL}};
const Segment SGI74939[] = {{&S34547[94], 5}, {NULL}};
const Segment SGI74940[] = {{&S34548[38], 19}, {NULL}};
const Segment SGI74941[] = {{&S34548[31], 6}, {NULL}};
const Segment SGI74942[] = {{&S30339[42], 14}, {NULL}};
const Segment SGI74943[] = {{&S30339[42], 26}, {NULL}};
const Segment SGI74944[] = {{&S30339[30], 4}, {NULL}};
const Segment SGI74945[] = {{&S30339[31], 3}, {NULL}};
const Segment SGI74946[] = {{&S30339[82], 6}, {NULL}};
const Segment SGI74947[] = {{&S40814[44], 10}, {NULL}};
const Segment SGI74948[] = {{&S40814[44], 22}, {NULL}};
const Segment SGI74949[] = {{&S40814[30], 6}, {NULL}};
const Segment SGI74950[] = {{&S40814[114], 3}, {NULL}};
const Segment SGI74951[] = {{&S40814[80], 5}, {NULL}};
const Segment SGI74952[] = {{&S40815[38], 10}, {NULL}};
const Segment SGI74953[] = {{&S40815[31], 6}, {NULL}};
const Segment SGI74954[] = {{&S40817[42], 10}, {NULL}};
const Segment SGI74955[] = {{&S40817[30], 6}, {NULL}};
const Segment SGI74956[] = {{&S40817[112], 3}, {NULL}};
const Segment SGI74957[] = {{&S40817[78], 5}, {NULL}};
const Segment SGI74958[] = {{&S23492[44], 14}, {NULL}};
const Segment SGI74959[] = {{&S23492[44], 26}, {NULL}};
const Segment SGI74960[] = {{&S23492[30], 6}, {NULL}};
const Segment SGI74961[] = {{&S23492[31], 3}, {NULL}};
const Segment SGI74962[] = {{&S23492[84], 6}, {NULL}};
const Segment SGI74963[] = {{&S23493[44], 27}, {NULL}};
const Segment SGI74964[] = {{&S23493[44], 39}, {NULL}};
const Segment SGI74965[] = {{&S23493[30], 6}, {NULL}};
const Segment SGI74966[] = {{&S23493[31], 3}, {NULL}};
const Segment SGI74967[] = {{&S23493[97], 6}, {NULL}};
const Segment SGI74968[] = {{&S23494[44], 14}, {NULL}};
const Segment SGI74969[] = {{&S23494[44], 26}, {NULL}};
const Segment SGI74970[] = {{&S23494[30], 6}, {NULL}};
const Segment SGI74971[] = {{&S23494[31], 3}, {NULL}};
const Segment SGI74972[] = {{&S23494[84], 6}, {NULL}};
const Segment SGI74973[] = {{&S23495[44], 27}, {NULL}};
const Segment SGI74974[] = {{&S23495[44], 39}, {NULL}};
const Segment SGI74975[] = {{&S23495[30], 6}, {NULL}};
const Segment SGI74976[] = {{&S23495[31], 3}, {NULL}};
const Segment SGI74977[] = {{&S23495[97], 6}, {NULL}};
const Segment SGI74978[] = {{&S23496[44], 7}, {NULL}};
const Segment SGI74979[] = {{&S23496[44], 19}, {NULL}};
const Segment SGI74980[] = {{&S23496[30], 6}, {NULL}};
const Segment SGI74981[] = {{&S23496[31], 3}, {NULL}};
const Segment SGI74982[] = {{&S23496[77], 6}, {NULL}};
const Segment SGI74983[] = {{&S23497[38], 14}, {NULL}};
const Segment SGI74984[] = {{&S23497[31], 6}, {NULL}};
const Segment SGI74985[] = {{&S23498[44], 25}, {NULL}};
const Segment SGI74986[] = {{&S23498[44], 37}, {NULL}};
const Segment SGI74987[] = {{&S23498[30], 6}, {NULL}};
const Segment SGI74988[] = {{&S23498[31], 3}, {NULL}};
const Segment SGI74989[] = {{&S23498[95], 6}, {NULL}};
const Segment SGI74990[] = {{&S23499[44], 25}, {NULL}};
const Segment SGI74991[] = {{&S23499[44], 37}, {NULL}};
const Segment SGI74992[] = {{&S23499[30], 6}, {NULL}};
const Segment SGI74993[] = {{&S23499[31], 3}, {NULL}};
const Segment SGI74994[] = {{&S23499[95], 6}, {NULL}};
const Segment SGI74995[] = {{&S23500[44], 25}, {NULL}};
const Segment SGI74996[] = {{&S23500[44], 37}, {NULL}};
const Segment SGI74997[] = {{&S23500[30], 6}, {NULL}};
const Segment SGI74998[] = {{&S23500[31], 3}, {NULL}};
const Segment SGI74999[] = {{&S23500[95], 6}, {NULL}};
const Segment SGI75000[] = {{&S23501[44], 25}, {NULL}};
const Segment SGI75001[] = {{&S23501[44], 37}, {NULL}};
const Segment SGI75002[] = {{&S23501[30], 6}, {NULL}};
const Segment SGI75003[] = {{&S23501[31], 3}, {NULL}};
const Segment SGI75004[] = {{&S23501[95], 6}, {NULL}};
const Segment SGI75005[] = {{&S23502[44], 25}, {NULL}};
const Segment SGI75006[] = {{&S23502[44], 37}, {NULL}};
const Segment SGI75007[] = {{&S23502[30], 6}, {NULL}};
const Segment SGI75008[] = {{&S23502[31], 3}, {NULL}};
const Segment SGI75009[] = {{&S23502[95], 6}, {NULL}};
const Segment SGI75010[] = {{&S23503[44], 25}, {NULL}};
const Segment SGI75011[] = {{&S23503[44], 37}, {NULL}};
const Segment SGI75012[] = {{&S23503[30], 6}, {NULL}};
const Segment SGI75013[] = {{&S23503[31], 3}, {NULL}};
const Segment SGI75014[] = {{&S23503[95], 6}, {NULL}};
const Segment SGI75015[] = {{&S23504[44], 25}, {NULL}};
const Segment SGI75016[] = {{&S23504[44], 37}, {NULL}};
const Segment SGI75017[] = {{&S23504[30], 6}, {NULL}};
const Segment SGI75018[] = {{&S23504[31], 3}, {NULL}};
const Segment SGI75019[] = {{&S23504[95], 6}, {NULL}};
const Segment SGI75020[] = {{&S23505[44], 25}, {NULL}};
const Segment SGI75021[] = {{&S23505[44], 37}, {NULL}};
const Segment SGI75022[] = {{&S23505[30], 6}, {NULL}};
const Segment SGI75023[] = {{&S23505[31], 3}, {NULL}};
const Segment SGI75024[] = {{&S23505[95], 6}, {NULL}};
const Segment SGI75025[] = {{&S23506[44], 25}, {NULL}};
const Segment SGI75026[] = {{&S23506[44], 37}, {NULL}};
const Segment SGI75027[] = {{&S23506[30], 6}, {NULL}};
const Segment SGI75028[] = {{&S23506[31], 3}, {NULL}};
const Segment SGI75029[] = {{&S23506[95], 6}, {NULL}};
const Segment SGI75030[] = {{&S23507[44], 25}, {NULL}};
const Segment SGI75031[] = {{&S23507[44], 37}, {NULL}};
const Segment SGI75032[] = {{&S23507[30], 6}, {NULL}};
const Segment SGI75033[] = {{&S23507[31], 3}, {NULL}};
const Segment SGI75034[] = {{&S23507[95], 6}, {NULL}};
const Segment SGI75035[] = {{&S23508[44], 25}, {NULL}};
const Segment SGI75036[] = {{&S23508[44], 37}, {NULL}};
const Segment SGI75037[] = {{&S23508[30], 6}, {NULL}};
const Segment SGI75038[] = {{&S23508[31], 3}, {NULL}};
const Segment SGI75039[] = {{&S23508[95], 6}, {NULL}};
const Segment SGI75040[] = {{&S23509[44], 24}, {NULL}};
const Segment SGI75041[] = {{&S23509[44], 36}, {NULL}};
const Segment SGI75042[] = {{&S23509[30], 6}, {NULL}};
const Segment SGI75043[] = {{&S23509[31], 3}, {NULL}};
const Segment SGI75044[] = {{&S23509[94], 6}, {NULL}};
const Segment SGI75045[] = {{&S23511[44], 25}, {NULL}};
const Segment SGI75046[] = {{&S23511[44], 37}, {NULL}};
const Segment SGI75047[] = {{&S23511[30], 6}, {NULL}};
const Segment SGI75048[] = {{&S23511[31], 3}, {NULL}};
const Segment SGI75049[] = {{&S23511[95], 6}, {NULL}};
const Segment SGI75050[] = {{&S23512[44], 25}, {NULL}};
const Segment SGI75051[] = {{&S23512[44], 37}, {NULL}};
const Segment SGI75052[] = {{&S23512[30], 6}, {NULL}};
const Segment SGI75053[] = {{&S23512[31], 3}, {NULL}};
const Segment SGI75054[] = {{&S23512[95], 6}, {NULL}};
const Segment SGI75055[] = {{&S23513[44], 27}, {NULL}};
const Segment SGI75056[] = {{&S23513[44], 39}, {NULL}};
const Segment SGI75057[] = {{&S23513[30], 6}, {NULL}};
const Segment SGI75058[] = {{&S23513[31], 3}, {NULL}};
const Segment SGI75059[] = {{&S23513[97], 6}, {NULL}};
const Segment SGI75060[] = {{&S23514[44], 25}, {NULL}};
const Segment SGI75061[] = {{&S23514[44], 37}, {NULL}};
const Segment SGI75062[] = {{&S23514[30], 6}, {NULL}};
const Segment SGI75063[] = {{&S23514[31], 3}, {NULL}};
const Segment SGI75064[] = {{&S23514[95], 6}, {NULL}};
const Segment SGI75065[] = {{&S23516[44], 25}, {NULL}};
const Segment SGI75066[] = {{&S23516[44], 37}, {NULL}};
const Segment SGI75067[] = {{&S23516[30], 6}, {NULL}};
const Segment SGI75068[] = {{&S23516[31], 3}, {NULL}};
const Segment SGI75069[] = {{&S23516[95], 6}, {NULL}};
const Segment SGI75070[] = {{&S23517[44], 25}, {NULL}};
const Segment SGI75071[] = {{&S23517[44], 37}, {NULL}};
const Segment SGI75072[] = {{&S23517[30], 6}, {NULL}};
const Segment SGI75073[] = {{&S23517[31], 3}, {NULL}};
const Segment SGI75074[] = {{&S23517[95], 6}, {NULL}};
const Segment SGI75075[] = {{&S23518[44], 27}, {NULL}};
const Segment SGI75076[] = {{&S23518[44], 39}, {NULL}};
const Segment SGI75077[] = {{&S23518[30], 6}, {NULL}};
const Segment SGI75078[] = {{&S23518[31], 3}, {NULL}};
const Segment SGI75079[] = {{&S23518[97], 6}, {NULL}};
const Segment SGI75080[] = {{&S23519[44], 27}, {NULL}};
const Segment SGI75081[] = {{&S23519[44], 39}, {NULL}};
const Segment SGI75082[] = {{&S23519[30], 6}, {NULL}};
const Segment SGI75083[] = {{&S23519[31], 3}, {NULL}};
const Segment SGI75084[] = {{&S23519[97], 6}, {NULL}};
const Segment SGI75085[] = {{&S23520[44], 25}, {NULL}};
const Segment SGI75086[] = {{&S23520[44], 37}, {NULL}};
const Segment SGI75087[] = {{&S23520[30], 6}, {NULL}};
const Segment SGI75088[] = {{&S23520[31], 3}, {NULL}};
const Segment SGI75089[] = {{&S23520[95], 6}, {NULL}};
const Segment SGI75090[] = {{&S23521[44], 25}, {NULL}};
const Segment SGI75091[] = {{&S23521[44], 37}, {NULL}};
const Segment SGI75092[] = {{&S23521[30], 6}, {NULL}};
const Segment SGI75093[] = {{&S23521[31], 3}, {NULL}};
const Segment SGI75094[] = {{&S23521[95], 6}, {NULL}};
const Segment SGI75095[] = {{&S23522[44], 25}, {NULL}};
const Segment SGI75096[] = {{&S23522[44], 37}, {NULL}};
const Segment SGI75097[] = {{&S23522[30], 6}, {NULL}};
const Segment SGI75098[] = {{&S23522[31], 3}, {NULL}};
const Segment SGI75099[] = {{&S23522[95], 6}, {NULL}};
const Segment SGI75100[] = {{&S23524[44], 25}, {NULL}};
const Segment SGI75101[] = {{&S23524[44], 37}, {NULL}};
const Segment SGI75102[] = {{&S23524[30], 6}, {NULL}};
const Segment SGI75103[] = {{&S23524[31], 3}, {NULL}};
const Segment SGI75104[] = {{&S23524[95], 6}, {NULL}};
const Segment SGI75105[] = {{&S23525[44], 25}, {NULL}};
const Segment SGI75106[] = {{&S23525[44], 37}, {NULL}};
const Segment SGI75107[] = {{&S23525[30], 6}, {NULL}};
const Segment SGI75108[] = {{&S23525[31], 3}, {NULL}};
const Segment SGI75109[] = {{&S23525[95], 6}, {NULL}};
const Segment SGI75110[] = {{&S23526[42], 14}, {NULL}};
const Segment SGI75111[] = {{&S23526[30], 6}, {NULL}};
const Segment SGI75112[] = {{&S23526[31], 3}, {NULL}};
const Segment SGI75113[] = {{&S23526[82], 6}, {NULL}};
const Segment SGI75114[] = {{&S23527[44], 25}, {NULL}};
const Segment SGI75115[] = {{&S23527[44], 37}, {NULL}};
const Segment SGI75116[] = {{&S23527[30], 6}, {NULL}};
const Segment SGI75117[] = {{&S23527[31], 3}, {NULL}};
const Segment SGI75118[] = {{&S23527[95], 6}, {NULL}};
const Segment SGI75119[] = {{&S40895[44], 10}, {NULL}};
const Segment SGI75120[] = {{&S40895[44], 27}, {NULL}};
const Segment SGI75121[] = {{&S40895[30], 6}, {NULL}};
const Segment SGI75122[] = {{&S40895[119], 3}, {NULL}};
const Segment SGI75123[] = {{&S40895[85], 5}, {NULL}};
const Segment SGI75124[] = {{&S40897[38], 10}, {NULL}};
const Segment SGI75125[] = {{&S40897[31], 6}, {NULL}};
const Segment SGI75126[] = {{&S40898[42], 10}, {NULL}};
const Segment SGI75127[] = {{&S40898[30], 6}, {NULL}};
const Segment SGI75128[] = {{&S40898[117], 3}, {NULL}};
const Segment SGI75129[] = {{&S40898[83], 5}, {NULL}};
const Segment SGI75130[] = {{&S40928[44], 18}, {NULL}};
const Segment SGI75131[] = {{&S40928[44], 39}, {NULL}};
const Segment SGI75132[] = {{&S40928[30], 6}, {NULL}};
const Segment SGI75133[] = {{&S40928[131], 3}, {NULL}};
const Segment SGI75134[] = {{&S40928[97], 5}, {NULL}};
const Segment SGI75135[] = {{&S40929[44], 45}, {NULL}};
const Segment SGI75136[] = {{&S40929[30], 6}, {NULL}};
const Segment SGI75137[] = {{&S40929[121], 5}, {NULL}};
const Segment SGI75138[] = {{&S40930[44], 18}, {NULL}};
const Segment SGI75139[] = {{&S40930[44], 39}, {NULL}};
const Segment SGI75140[] = {{&S40930[30], 6}, {NULL}};
const Segment SGI75141[] = {{&S40930[131], 3}, {NULL}};
const Segment SGI75142[] = {{&S40930[97], 5}, {NULL}};
const Segment SGI75143[] = {{&S40931[44], 45}, {NULL}};
const Segment SGI75144[] = {{&S40931[30], 6}, {NULL}};
const Segment SGI75145[] = {{&S40931[121], 5}, {NULL}};
const Segment SGI75146[] = {{&S40932[44], 18}, {NULL}};
const Segment SGI75147[] = {{&S40932[44], 39}, {NULL}};
const Segment SGI75148[] = {{&S40932[30], 6}, {NULL}};
const Segment SGI75149[] = {{&S40932[131], 3}, {NULL}};
const Segment SGI75150[] = {{&S40932[97], 5}, {NULL}};
const Segment SGI75151[] = {{&S40933[44], 45}, {NULL}};
const Segment SGI75152[] = {{&S40933[30], 6}, {NULL}};
const Segment SGI75153[] = {{&S40933[121], 5}, {NULL}};
const Segment SGI75154[] = {{&S40934[44], 37}, {NULL}};
const Segment SGI75155[] = {{&S40934[30], 6}, {NULL}};
const Segment SGI75156[] = {{&S40934[113], 5}, {NULL}};
const Segment SGI75157[] = {{&S40935[44], 18}, {NULL}};
const Segment SGI75158[] = {{&S40935[44], 39}, {NULL}};
const Segment SGI75159[] = {{&S40935[30], 6}, {NULL}};
const Segment SGI75160[] = {{&S40935[131], 3}, {NULL}};
const Segment SGI75161[] = {{&S40935[97], 5}, {NULL}};
const Segment SGI75162[] = {{&S40936[44], 45}, {NULL}};
const Segment SGI75163[] = {{&S40936[30], 6}, {NULL}};
const Segment SGI75164[] = {{&S40936[121], 5}, {NULL}};
const Segment SGI75165[] = {{&S40937[44], 45}, {NULL}};
const Segment SGI75166[] = {{&S40937[30], 6}, {NULL}};
const Segment SGI75167[] = {{&S40937[121], 5}, {NULL}};
const Segment SGI75168[] = {{&S40938[44], 45}, {NULL}};
const Segment SGI75169[] = {{&S40938[30], 6}, {NULL}};
const Segment SGI75170[] = {{&S40938[121], 5}, {NULL}};
const Segment SGI75171[] = {{&S40939[44], 18}, {NULL}};
const Segment SGI75172[] = {{&S40939[44], 39}, {NULL}};
const Segment SGI75173[] = {{&S40939[30], 6}, {NULL}};
const Segment SGI75174[] = {{&S40939[131], 3}, {NULL}};
const Segment SGI75175[] = {{&S40939[97], 5}, {NULL}};
const Segment SGI75176[] = {{&S40940[44], 14}, {NULL}};
const Segment SGI75177[] = {{&S40940[44], 35}, {NULL}};
const Segment SGI75178[] = {{&S40940[30], 6}, {NULL}};
const Segment SGI75179[] = {{&S40940[127], 3}, {NULL}};
const Segment SGI75180[] = {{&S40940[93], 5}, {NULL}};
const Segment SGI75181[] = {{&S40941[44], 45}, {NULL}};
const Segment SGI75182[] = {{&S40941[30], 6}, {NULL}};
const Segment SGI75183[] = {{&S40941[121], 5}, {NULL}};
const Segment SGI75184[] = {{&S40942[44], 21}, {NULL}};
const Segment SGI75185[] = {{&S40942[44], 42}, {NULL}};
const Segment SGI75186[] = {{&S40942[30], 6}, {NULL}};
const Segment SGI75187[] = {{&S40942[134], 3}, {NULL}};
const Segment SGI75188[] = {{&S40942[100], 5}, {NULL}};
const Segment SGI75189[] = {{&S40943[38], 18}, {NULL}};
const Segment SGI75190[] = {{&S40943[31], 6}, {NULL}};
const Segment SGI75191[] = {{&S40944[44], 22}, {NULL}};
const Segment SGI75192[] = {{&S40944[44], 43}, {NULL}};
const Segment SGI75193[] = {{&S40944[30], 6}, {NULL}};
const Segment SGI75194[] = {{&S40944[135], 3}, {NULL}};
const Segment SGI75195[] = {{&S40944[101], 5}, {NULL}};
const Segment SGI75196[] = {{&S40945[44], 18}, {NULL}};
const Segment SGI75197[] = {{&S40945[44], 39}, {NULL}};
const Segment SGI75198[] = {{&S40945[30], 6}, {NULL}};
const Segment SGI75199[] = {{&S40945[131], 3}, {NULL}};
const Segment SGI75200[] = {{&S40945[97], 5}, {NULL}};
const Segment SGI75201[] = {{&S40972[44], 11}, {NULL}};
const Segment SGI75202[] = {{&S40972[44], 28}, {NULL}};
const Segment SGI75203[] = {{&S40972[30], 6}, {NULL}};
const Segment SGI75204[] = {{&S40972[120], 3}, {NULL}};
const Segment SGI75205[] = {{&S40972[86], 5}, {NULL}};
const Segment SGI75206[] = {{&S40973[44], 7}, {NULL}};
const Segment SGI75207[] = {{&S40973[44], 24}, {NULL}};
const Segment SGI75208[] = {{&S40973[30], 6}, {NULL}};
const Segment SGI75209[] = {{&S40973[116], 3}, {NULL}};
const Segment SGI75210[] = {{&S40973[82], 5}, {NULL}};
const Segment SGI75211[] = {{&S23676[44], 19}, {NULL}};
const Segment SGI75212[] = {{&S23676[44], 31}, {NULL}};
const Segment SGI75213[] = {{&S23676[30], 6}, {NULL}};
const Segment SGI75214[] = {{&S23676[31], 3}, {NULL}};
const Segment SGI75215[] = {{&S23676[89], 6}, {NULL}};
const Segment SGI75216[] = {{&S23677[38], 19}, {NULL}};
const Segment SGI75217[] = {{&S23677[31], 6}, {NULL}};
const Segment SGI75218[] = {{&S23678[44], 19}, {NULL}};
const Segment SGI75219[] = {{&S23678[44], 31}, {NULL}};
const Segment SGI75220[] = {{&S23678[30], 6}, {NULL}};
const Segment SGI75221[] = {{&S23678[31], 3}, {NULL}};
const Segment SGI75222[] = {{&S23678[89], 6}, {NULL}};
const Segment SGI75223[] = {{&S23679[61], 19}, {NULL}};
const Segment SGI75224[] = {{&S23679[61], 31}, {NULL}};
const Segment SGI75225[] = {{&S23679[30], 23}, {NULL}};
const Segment SGI75226[] = {{&S23679[31], 3}, {NULL}};
const Segment SGI75227[] = {{&S23679[106], 6}, {NULL}};
const Segment SGI75228[] = {{&S23681[44], 7}, {NULL}};
const Segment SGI75229[] = {{&S23681[44], 19}, {NULL}};
const Segment SGI75230[] = {{&S23681[30], 6}, {NULL}};
const Segment SGI75231[] = {{&S23681[31], 3}, {NULL}};
const Segment SGI75232[] = {{&S23681[77], 6}, {NULL}};
const Segment SGI75233[] = {{&S23683[42], 19}, {NULL}};
const Segment SGI75234[] = {{&S23683[30], 6}, {NULL}};
const Segment SGI75235[] = {{&S23683[31], 3}, {NULL}};
const Segment SGI75236[] = {{&S23683[87], 6}, {NULL}};
const Segment SGI75237[] = {{&S23684[44], 19}, {NULL}};
const Segment SGI75238[] = {{&S23684[44], 31}, {NULL}};
const Segment SGI75239[] = {{&S23684[30], 6}, {NULL}};
const Segment SGI75240[] = {{&S23684[134], 3}, {NULL}};
const Segment SGI75241[] = {{&S23684[99], 6}, {NULL}};
const Segment SGI75242[] = {{&S23684[154], 8}, {NULL}};
const Segment SGI75243[] = {{&S23685[44], 19}, {NULL}};
const Segment SGI75244[] = {{&S23685[44], 31}, {NULL}};
const Segment SGI75245[] = {{&S23685[30], 6}, {NULL}};
const Segment SGI75246[] = {{&S23685[89], 6}, {NULL}};
const Segment SGI75247[] = {{&S34783[44], 11}, {NULL}};
const Segment SGI75248[] = {{&S34783[44], 28}, {NULL}};
const Segment SGI75249[] = {{&S34783[30], 6}, {NULL}};
const Segment SGI75250[] = {{&S34783[120], 3}, {NULL}};
const Segment SGI75251[] = {{&S34783[86], 5}, {NULL}};
const Segment SGI75252[] = {{&S41052[44], 11}, {NULL}};
const Segment SGI75253[] = {{&S41052[44], 28}, {NULL}};
const Segment SGI75254[] = {{&S41052[30], 6}, {NULL}};
const Segment SGI75255[] = {{&S41052[120], 3}, {NULL}};
const Segment SGI75256[] = {{&S41052[86], 5}, {NULL}};
const Segment SGI75257[] = {{&S41054[44], 10}, {NULL}};
const Segment SGI75258[] = {{&S41054[44], 27}, {NULL}};
const Segment SGI75259[] = {{&S41054[30], 6}, {NULL}};
const Segment SGI75260[] = {{&S41054[119], 3}, {NULL}};
const Segment SGI75261[] = {{&S41054[85], 5}, {NULL}};
const Segment SGI75262[] = {{&S41060[38], 11}, {NULL}};
const Segment SGI75263[] = {{&S41060[31], 6}, {NULL}};
const Segment SGI75264[] = {{&S41061[44], 6}, {NULL}};
const Segment SGI75265[] = {{&S41061[44], 23}, {NULL}};
const Segment SGI75266[] = {{&S41061[30], 6}, {NULL}};
const Segment SGI75267[] = {{&S41061[115], 3}, {NULL}};
const Segment SGI75268[] = {{&S41061[81], 5}, {NULL}};
const Segment SGI75269[] = {{&S34439[42], 10}, {NULL}};
const Segment SGI75270[] = {{&S34439[42], 21}, {NULL}};
const Segment SGI75271[] = {{&S34439[30], 4}, {NULL}};
const Segment SGI75272[] = {{&S34439[112], 3}, {NULL}};
const Segment SGI75273[] = {{&S34439[77], 6}, {NULL}};
const Segment SGI75274[] = {{&S34440[42], 10}, {NULL}};
const Segment SGI75275[] = {{&S34440[42], 25}, {NULL}};
const Segment SGI75276[] = {{&S34440[30], 4}, {NULL}};
const Segment SGI75277[] = {{&S34440[116], 3}, {NULL}};
const Segment SGI75278[] = {{&S34440[81], 6}, {NULL}};
const Segment SGI75279[] = {{&S34441[42], 11}, {NULL}};
const Segment SGI75280[] = {{&S34441[42], 22}, {NULL}};
const Segment SGI75281[] = {{&S34441[30], 4}, {NULL}};
const Segment SGI75282[] = {{&S34441[113], 3}, {NULL}};
const Segment SGI75283[] = {{&S34441[78], 6}, {NULL}};
const Segment SGI75284[] = {{&S34442[36], 10}, {NULL}};
const Segment SGI75285[] = {{&S34442[31], 4}, {NULL}};
const Segment SGI75286[] = {{&S34443[42], 29}, {NULL}};
const Segment SGI75287[] = {{&S34443[42], 40}, {NULL}};
const Segment SGI75288[] = {{&S34443[30], 4}, {NULL}};
const Segment SGI75289[] = {{&S34443[131], 3}, {NULL}};
const Segment SGI75290[] = {{&S34443[96], 6}, {NULL}};
const Segment SGI75291[] = {{&S34444[42], 10}, {NULL}};
const Segment SGI75292[] = {{&S34444[42], 21}, {NULL}};
const Segment SGI75293[] = {{&S34444[30], 4}, {NULL}};
const Segment SGI75294[] = {{&S34444[112], 3}, {NULL}};
const Segment SGI75295[] = {{&S34444[77], 6}, {NULL}};
const Segment SGI75296[] = {{&S34445[40], 10}, {NULL}};
const Segment SGI75297[] = {{&S34445[30], 4}, {NULL}};
const Segment SGI75298[] = {{&S34445[110], 3}, {NULL}};
const Segment SGI75299[] = {{&S34445[75], 6}, {NULL}};
const Segment SGI75300[] = {{&S34446[42], 10}, {NULL}};
const Segment SGI75301[] = {{&S34446[42], 21}, {NULL}};
const Segment SGI75302[] = {{&S34446[30], 4}, {NULL}};
const Segment SGI75303[] = {{&S34446[80], 6}, {NULL}};
const Segment SGI75304[] = {{&S34447[36], 11}, {NULL}};
const Segment SGI75305[] = {{&S34447[31], 4}, {NULL}};
const Segment SGI75306[] = {{&S34448[42], 10}, {NULL}};
const Segment SGI75307[] = {{&S34448[42], 21}, {NULL}};
const Segment SGI75308[] = {{&S34448[30], 4}, {NULL}};
const Segment SGI75309[] = {{&S34448[80], 4}, {NULL}};
const Segment SGI75310[] = {{&S34448[80], 3}, {NULL}};
const Segment SGI75311[] = {{&S34449[42], 10}, {NULL}};
const Segment SGI75312[] = {{&S34449[42], 21}, {NULL}};
const Segment SGI75313[] = {{&S34449[30], 4}, {NULL}};
const Segment SGI75314[] = {{&S34449[31], 3}, {NULL}};
const Segment SGI75315[] = {{&S34449[87], 6}, {NULL}};
const Segment SGI75316[] = {{&S34449[142], 8}, {NULL}};
const Segment SGI75317[] = {{&S34450[42], 10}, {NULL}};
const Segment SGI75318[] = {{&S34450[42], 21}, {NULL}};
const Segment SGI75319[] = {{&S34450[30], 4}, {NULL}};
const Segment SGI75320[] = {{&S34450[111], 3}, {NULL}};
const Segment SGI75321[] = {{&S34450[77], 5}, {NULL}};
const Segment SGI75322[] = {{&S34451[42], 29}, {NULL}};
const Segment SGI75323[] = {{&S34451[42], 40}, {NULL}};
const Segment SGI75324[] = {{&S34451[30], 4}, {NULL}};
const Segment SGI75325[] = {{&S34451[131], 3}, {NULL}};
const Segment SGI75326[] = {{&S34451[96], 6}, {NULL}};
const Segment SGI75327[] = {{&S34452[42], 10}, {NULL}};
const Segment SGI75328[] = {{&S34452[42], 21}, {NULL}};
const Segment SGI75329[] = {{&S34452[30], 4}, {NULL}};
const Segment SGI75330[] = {{&S34453[42], 10}, {NULL}};
const Segment SGI75331[] = {{&S34453[42], 21}, {&S34453[205], 7}, {NULL}};
const Segment SGI75332[] = {{&S34453[30], 4}, {NULL}};
const Segment SGI75333[] = {{&S34453[112], 3}, {NULL}};
const Segment SGI75334[] = {{&S34453[77], 6}, {NULL}};
const Segment SGI75335[] = {{&S34454[42], 10}, {NULL}};
const Segment SGI75336[] = {{&S34454[42], 21}, {&S34454[224], 5}, {NULL}};
const Segment SGI75337[] = {{&S34454[30], 4}, {NULL}};
const Segment SGI75338[] = {{&S34950[44], 9}, {NULL}};
const Segment SGI75339[] = {{&S34950[44], 20}, {NULL}};
const Segment SGI75340[] = {{&S34950[30], 6}, {NULL}};
const Segment SGI75341[] = {{&S34454[112], 3}, {NULL}};
const Segment SGI75342[] = {{&S34950[112], 3}, {NULL}};
const Segment SGI75343[] = {{&S34454[77], 6}, {NULL}};
const Segment SGI75344[] = {{&S34950[78], 5}, {NULL}};
const Segment SGI75345[] = {{&S34455[42], 9}, {NULL}};
const Segment SGI75346[] = {{&S34455[30], 4}, {NULL}};
const Segment SGI75347[] = {{&S34456[43], 10}, {NULL}};
const Segment SGI75348[] = {{&S34456[43], 27}, {NULL}};
const Segment SGI75349[] = {{&S34456[31], 4}, {NULL}};
const Segment SGI75350[] = {{&S34457[42], 10}, {NULL}};
const Segment SGI75351[] = {{&S34457[42], 21}, {&S34457[205], 13}, {NULL}};
const Segment SGI75352[] = {{&S34457[30], 4}, {NULL}};
const Segment SGI75353[] = {{&S34956[38], 9}, {NULL}};
const Segment SGI75354[] = {{&S34956[31], 6}, {NULL}};
const Segment SGI75355[] = {{&S34457[112], 3}, {NULL}};
const Segment SGI75356[] = {{&S34457[77], 6}, {NULL}};
const Segment SGI75357[] = {{&S34458[42], 10}, {NULL}};
const Segment SGI75358[] = {{&S34458[42], 21}, {NULL}};
const Segment SGI75359[] = {{&S34458[30], 4}, {NULL}};
const Segment SGI75360[] = {{&S34458[113], 3}, {NULL}};
const Segment SGI75361[] = {{&S34458[77], 7}, {NULL}};
const Segment SGI75362[] = {{&S34458[133], 8}, {NULL}};
const Segment SGI75363[] = {{&S34459[43], 10}, {NULL}};
const Segment SGI75364[] = {{&S34459[43], 27}, {NULL}};
const Segment SGI75365[] = {{&S34459[31], 4}, {NULL}};
const Segment SGI75366[] = {{&S34460[53], 18}, {NULL}};
const Segment SGI75367[] = {{&S34460[48], 4}, {NULL}};
const Segment SGI75368[] = {{&S34463[44], 11}, {NULL}};
const Segment SGI75369[] = {{&S34463[44], 22}, {NULL}};
const Segment SGI75370[] = {{&S34463[32], 4}, {NULL}};
const Segment SGI75371[] = {{&S34464[42], 9}, {NULL}};
const Segment SGI75372[] = {{&S34464[42], 20}, {NULL}};
const Segment SGI75373[] = {{&S34464[30], 4}, {NULL}};
const Segment SGI75374[] = {{&S34464[111], 3}, {NULL}};
const Segment SGI75375[] = {{&S34464[76], 6}, {NULL}};
const Segment SGI75376[] = {{&S34465[41], 10}, {NULL}};
const Segment SGI75377[] = {{&S34465[41], 21}, {NULL}};
const Segment SGI75378[] = {{&S34465[111], 3}, {NULL}};
const Segment SGI75379[] = {{&S34465[76], 6}, {NULL}};
const Segment SGI75380[] = {{&S34466[42], 11}, {NULL}};
const Segment SGI75381[] = {{&S34466[42], 22}, {&S34466[210], 7}, {NULL}};
const Segment SGI75382[] = {{&S34466[30], 4}, {NULL}};
const Segment SGI75383[] = {{&S34466[113], 3}, {NULL}};
const Segment SGI75384[] = {{&S34466[78], 6}, {NULL}};
const Segment SGI75385[] = {{&S35070[42], 9}, {NULL}};
const Segment SGI75386[] = {{&S35070[42], 21}, {NULL}};
const Segment SGI75387[] = {{&S35070[30], 4}, {NULL}};
const Segment SGI75388[] = {{&S35070[111], 3}, {NULL}};
const Segment SGI75389[] = {{&S35070[77], 5}, {NULL}};
const Segment SGI75390[] = {{&S35071[42], 9}, {NULL}};
const Segment SGI75391[] = {{&S35071[42], 20}, {NULL}};
const Segment SGI75392[] = {{&S35071[30], 4}, {NULL}};
const Segment SGI75393[] = {{&S35071[110], 3}, {NULL}};
const Segment SGI75394[] = {{&S35071[76], 5}, {NULL}};
const Segment SGI75395[] = {{&S35072[42], 9}, {NULL}};
const Segment SGI75396[] = {{&S35072[42], 21}, {NULL}};
const Segment SGI75397[] = {{&S35072[30], 4}, {NULL}};
const Segment SGI75398[] = {{&S35072[111], 3}, {NULL}};
const Segment SGI75399[] = {{&S35072[77], 5}, {NULL}};
const Segment SGI75400[] = {{&S35073[42], 9}, {NULL}};
const Segment SGI75401[] = {{&S35073[42], 26}, {NULL}};
const Segment SGI75402[] = {{&S35073[30], 4}, {NULL}};
const Segment SGI75403[] = {{&S35073[116], 3}, {NULL}};
const Segment SGI75404[] = {{&S35073[82], 5}, {NULL}};
const Segment SGI75405[] = {{&S35074[42], 24}, {NULL}};
const Segment SGI75406[] = {{&S35074[42], 41}, {NULL}};
const Segment SGI75407[] = {{&S35074[30], 4}, {NULL}};
const Segment SGI75408[] = {{&S35074[131], 3}, {NULL}};
const Segment SGI75409[] = {{&S35074[97], 5}, {NULL}};
const Segment SGI75410[] = {{&S23875[44], 18}, {NULL}};
const Segment SGI75411[] = {{&S23875[44], 30}, {NULL}};
const Segment SGI75412[] = {{&S23875[30], 6}, {NULL}};
const Segment SGI75413[] = {{&S23875[31], 3}, {NULL}};
const Segment SGI75414[] = {{&S23875[88], 6}, {NULL}};
const Segment SGI75415[] = {{&S23876[44], 22}, {NULL}};
const Segment SGI75416[] = {{&S23876[44], 34}, {NULL}};
const Segment SGI75417[] = {{&S23876[30], 6}, {NULL}};
const Segment SGI75418[] = {{&S23876[31], 3}, {NULL}};
const Segment SGI75419[] = {{&S23876[92], 6}, {NULL}};
const Segment SGI75420[] = {{&S23877[38], 18}, {NULL}};
const Segment SGI75421[] = {{&S23877[31], 6}, {NULL}};
const Segment SGI75422[] = {{&S23878[44], 11}, {NULL}};
const Segment SGI75423[] = {{&S23878[44], 23}, {NULL}};
const Segment SGI75424[] = {{&S23878[30], 6}, {NULL}};
const Segment SGI75425[] = {{&S23878[31], 3}, {NULL}};
const Segment SGI75426[] = {{&S23878[81], 6}, {NULL}};
const Segment SGI75427[] = {{&S23879[44], 13}, {NULL}};
const Segment SGI75428[] = {{&S23879[44], 25}, {NULL}};
const Segment SGI75429[] = {{&S23879[30], 6}, {NULL}};
const Segment SGI75430[] = {{&S23879[31], 3}, {NULL}};
const Segment SGI75431[] = {{&S23879[83], 6}, {NULL}};
const Segment SGI75432[] = {{&S23880[44], 17}, {NULL}};
const Segment SGI75433[] = {{&S23880[44], 29}, {NULL}};
const Segment SGI75434[] = {{&S23880[30], 6}, {NULL}};
const Segment SGI75435[] = {{&S23880[31], 3}, {NULL}};
const Segment SGI75436[] = {{&S23880[87], 6}, {NULL}};
const Segment SGI75437[] = {{&S23881[44], 18}, {NULL}};
const Segment SGI75438[] = {{&S23881[44], 30}, {NULL}};
const Segment SGI75439[] = {{&S23881[30], 6}, {NULL}};
const Segment SGI75440[] = {{&S23881[31], 3}, {NULL}};
const Segment SGI75441[] = {{&S23881[88], 6}, {NULL}};
const Segment SGI75442[] = {{&S23882[44], 17}, {NULL}};
const Segment SGI75443[] = {{&S23882[44], 29}, {NULL}};
const Segment SGI75444[] = {{&S23882[30], 6}, {NULL}};
const Segment SGI75445[] = {{&S23882[31], 3}, {NULL}};
const Segment SGI75446[] = {{&S23882[87], 6}, {NULL}};
const Segment SGI75447[] = {{&S23883[44], 18}, {NULL}};
const Segment SGI75448[] = {{&S23883[44], 30}, {NULL}};
const Segment SGI75449[] = {{&S23883[30], 6}, {NULL}};
const Segment SGI75450[] = {{&S23883[91], 6}, {NULL}};
const Segment SGI75451[] = {{&S23884[44], 11}, {NULL}};
const Segment SGI75452[] = {{&S23884[44], 23}, {NULL}};
const Segment SGI75453[] = {{&S23884[30], 6}, {NULL}};
const Segment SGI75454[] = {{&S23884[31], 3}, {NULL}};
const Segment SGI75455[] = {{&S23884[81], 6}, {NULL}};
const Segment SGI75456[] = {{&S23885[44], 11}, {NULL}};
const Segment SGI75457[] = {{&S23885[44], 23}, {NULL}};
const Segment SGI75458[] = {{&S23885[30], 6}, {NULL}};
const Segment SGI75459[] = {{&S23885[84], 6}, {NULL}};
const Segment SGI75460[] = {{&S23886[44], 30}, {NULL}};
const Segment SGI75461[] = {{&S23886[30], 6}, {NULL}};
const Segment SGI75462[] = {{&S35066[44], 9}, {NULL}};
const Segment SGI75463[] = {{&S35066[44], 20}, {NULL}};
const Segment SGI75464[] = {{&S35066[30], 6}, {NULL}};
const Segment SGI75465[] = {{&S35066[112], 3}, {NULL}};
const Segment SGI75466[] = {{&S35066[78], 5}, {NULL}};
const Segment SGI75467[] = {{&S23887[38], 11}, {NULL}};
const Segment SGI75468[] = {{&S23887[31], 6}, {NULL}};
const Segment SGI75469[] = {{&S35067[44], 9}, {NULL}};
const Segment SGI75470[] = {{&S35067[44], 20}, {NULL}};
const Segment SGI75471[] = {{&S35067[30], 6}, {NULL}};
const Segment SGI75472[] = {{&S23888[38], 18}, {NULL}};
const Segment SGI75473[] = {{&S23888[31], 6}, {NULL}};
const Segment SGI75474[] = {{&S35067[112], 3}, {NULL}};
const Segment SGI75475[] = {{&S35067[78], 5}, {NULL}};
const Segment SGI75476[] = {{&S23889[44], 13}, {NULL}};
const Segment SGI75477[] = {{&S35068[44], 9}, {NULL}};
const Segment SGI75478[] = {{&S23889[44], 25}, {NULL}};
const Segment SGI75479[] = {{&S23889[30], 6}, {NULL}};
const Segment SGI75480[] = {{&S35068[44], 21}, {NULL}};
const Segment SGI75481[] = {{&S35068[30], 6}, {NULL}};
const Segment SGI75482[] = {{&S23889[31], 3}, {NULL}};
const Segment SGI75483[] = {{&S35068[113], 3}, {NULL}};
const Segment SGI75484[] = {{&S23889[83], 6}, {NULL}};
const Segment SGI75485[] = {{&S35068[79], 5}, {NULL}};
const Segment SGI75486[] = {{&S23890[44], 23}, {NULL}};
const Segment SGI75487[] = {{&S23890[44], 35}, {NULL}};
const Segment SGI75488[] = {{&S23890[30], 6}, {NULL}};
const Segment SGI75489[] = {{&S23890[31], 3}, {NULL}};
const Segment SGI75490[] = {{&S23890[93], 6}, {NULL}};
const Segment SGI75491[] = {{&S23891[44], 22}, {NULL}};
const Segment SGI75492[] = {{&S23891[30], 6}, {NULL}};
const Segment SGI75493[] = {{&S23891[31], 3}, {NULL}};
const Segment SGI75494[] = {{&S23891[82], 6}, {NULL}};
const Segment SGI75495[] = {{&S23892[44], 18}, {NULL}};
const Segment SGI75496[] = {{&S23892[44], 30}, {NULL}};
const Segment SGI75497[] = {{&S23892[30], 6}, {NULL}};
const Segment SGI75498[] = {{&S23892[31], 3}, {NULL}};
const Segment SGI75499[] = {{&S23892[88], 6}, {NULL}};
const Segment SGI75500[] = {{&S35076[38], 9}, {NULL}};
const Segment SGI75501[] = {{&S35076[31], 6}, {NULL}};
const Segment SGI75502[] = {{&S23893[44], 10}, {NULL}};
const Segment SGI75503[] = {{&S23893[30], 6}, {NULL}};
const Segment SGI75504[] = {{&S23893[31], 3}, {NULL}};
const Segment SGI75505[] = {{&S23893[70], 6}, {NULL}};
const Segment SGI75506[] = {{&S23896[44], 23}, {NULL}};
const Segment SGI75507[] = {{&S23896[44], 35}, {NULL}};
const Segment SGI75508[] = {{&S23896[30], 6}, {NULL}};
const Segment SGI75509[] = {{&S23896[31], 3}, {NULL}};
const Segment SGI75510[] = {{&S23896[93], 6}, {NULL}};
const Segment SGI75511[] = {{&S23897[42], 13}, {NULL}};
const Segment SGI75512[] = {{&S23897[30], 6}, {NULL}};
const Segment SGI75513[] = {{&S23897[31], 3}, {NULL}};
const Segment SGI75514[] = {{&S23897[81], 6}, {NULL}};
const Segment SGI75515[] = {{&S35085[44], 9}, {NULL}};
const Segment SGI75516[] = {{&S23898[44], 11}, {NULL}};
const Segment SGI75517[] = {{&S35085[44], 20}, {NULL}};
const Segment SGI75518[] = {{&S23898[44], 23}, {NULL}};
const Segment SGI75519[] = {{&S35085[30], 6}, {NULL}};
const Segment SGI75520[] = {{&S23898[30], 6}, {NULL}};
const Segment SGI75521[] = {{&S35085[112], 3}, {NULL}};
const Segment SGI75522[] = {{&S23898[31], 3}, {NULL}};
const Segment SGI75523[] = {{&S35085[78], 5}, {NULL}};
const Segment SGI75524[] = {{&S23898[81], 6}, {NULL}};
const Segment SGI75525[] = {{&S35088[46], 9}, {NULL}};
const Segment SGI75526[] = {{&S35088[46], 20}, {NULL}};
const Segment SGI75527[] = {{&S35088[30], 8}, {NULL}};
const Segment SGI75528[] = {{&S35088[114], 3}, {NULL}};
const Segment SGI75529[] = {{&S35088[80], 5}, {NULL}};
const Segment SGI75530[] = {{&S35105[44], 10}, {NULL}};
const Segment SGI75531[] = {{&S35105[44], 22}, {NULL}};
const Segment SGI75532[] = {{&S35105[30], 6}, {NULL}};
const Segment SGI75533[] = {{&S35105[114], 3}, {NULL}};
const Segment SGI75534[] = {{&S35105[80], 5}, {NULL}};
const Segment SGI75535[] = {{&S35106[44], 17}, {NULL}};
const Segment SGI75536[] = {{&S35106[44], 28}, {NULL}};
const Segment SGI75537[] = {{&S35106[30], 6}, {NULL}};
const Segment SGI75538[] = {{&S35106[120], 3}, {NULL}};
const Segment SGI75539[] = {{&S35106[86], 5}, {NULL}};
const Segment SGI75540[] = {{&S35111[44], 17}, {NULL}};
const Segment SGI75541[] = {{&S35111[44], 28}, {NULL}};
const Segment SGI75542[] = {{&S35111[30], 6}, {NULL}};
const Segment SGI75543[] = {{&S35111[138], 3}, {NULL}};
const Segment SGI75544[] = {{&S35111[104], 5}, {NULL}};
const Segment SGI75545[] = {{&S35123[38], 17}, {NULL}};
const Segment SGI75546[] = {{&S35123[31], 6}, {NULL}};
const Segment SGI75547[] = {{&S35124[44], 10}, {NULL}};
const Segment SGI75548[] = {{&S35124[44], 21}, {NULL}};
const Segment SGI75549[] = {{&S35124[30], 6}, {NULL}};
const Segment SGI75550[] = {{&S35124[129], 3}, {NULL}};
const Segment SGI75551[] = {{&S35124[95], 5}, {NULL}};
const Segment SGI75552[] = {{&S35130[38], 10}, {NULL}};
const Segment SGI75553[] = {{&S35130[31], 6}, {NULL}};
const Segment SGI75554[] = {{&S41272[44], 8}, {NULL}};
const Segment SGI75555[] = {{&S41272[44], 25}, {NULL}};
const Segment SGI75556[] = {{&S41272[30], 6}, {NULL}};
const Segment SGI75557[] = {{&S41272[117], 3}, {NULL}};
const Segment SGI75558[] = {{&S41272[83], 5}, {NULL}};
const Segment SGI75559[] = {{&S41273[38], 8}, {NULL}};
const Segment SGI75560[] = {{&S41273[31], 6}, {NULL}};
const Segment SGI75561[] = {{&S41274[42], 8}, {NULL}};
const Segment SGI75562[] = {{&S41274[30], 6}, {NULL}};
const Segment SGI75563[] = {{&S41274[115], 3}, {NULL}};
const Segment SGI75564[] = {{&S41274[81], 5}, {NULL}};
const Segment SGI75565[] = {{&S35588[38], 10}, {NULL}};
const Segment SGI75566[] = {{&S35588[31], 6}, {NULL}};
const Segment SGI75567[] = {{&S35179[42], 10}, {NULL}};
const Segment SGI75568[] = {{&S35179[42], 22}, {NULL}};
const Segment SGI75569[] = {{&S35179[30], 4}, {NULL}};
const Segment SGI75570[] = {{&S35179[113], 3}, {NULL}};
const Segment SGI75571[] = {{&S35179[78], 6}, {NULL}};
const Segment SGI75572[] = {{&S35180[36], 10}, {NULL}};
const Segment SGI75573[] = {{&S35180[31], 4}, {NULL}};
const Segment SGI75574[] = {{&S35181[42], 10}, {NULL}};
const Segment SGI75575[] = {{&S35181[42], 22}, {NULL}};
const Segment SGI75576[] = {{&S35181[30], 4}, {NULL}};
const Segment SGI75577[] = {{&S35181[81], 4}, {NULL}};
const Segment SGI75578[] = {{&S35181[81], 3}, {NULL}};
const Segment SGI75579[] = {{&S35182[42], 10}, {NULL}};
const Segment SGI75580[] = {{&S35182[42], 22}, {NULL}};
const Segment SGI75581[] = {{&S35182[30], 4}, {NULL}};
const Segment SGI75582[] = {{&S35182[113], 3}, {NULL}};
const Segment SGI75583[] = {{&S35182[78], 6}, {NULL}};
const Segment SGI75584[] = {{&S35183[42], 10}, {NULL}};
const Segment SGI75585[] = {{&S35183[42], 22}, {NULL}};
const Segment SGI75586[] = {{&S35183[30], 4}, {NULL}};
const Segment SGI75587[] = {{&S35183[81], 6}, {NULL}};
const Segment SGI75588[] = {{&S35184[42], 10}, {NULL}};
const Segment SGI75589[] = {{&S35184[42], 22}, {NULL}};
const Segment SGI75590[] = {{&S35184[30], 4}, {NULL}};
const Segment SGI75591[] = {{&S35184[123], 3}, {NULL}};
const Segment SGI75592[] = {{&S35184[88], 6}, {NULL}};
const Segment SGI75593[] = {{&S35184[143], 8}, {NULL}};
const Segment SGI75594[] = {{&S35185[40], 10}, {NULL}};
const Segment SGI75595[] = {{&S35185[30], 4}, {NULL}};
const Segment SGI75596[] = {{&S35185[111], 3}, {NULL}};
const Segment SGI75597[] = {{&S35185[76], 6}, {NULL}};
const Segment SGI75598[] = {{&S35186[42], 9}, {NULL}};
const Segment SGI75599[] = {{&S35186[30], 4}, {NULL}};
const Segment SGI75600[] = {{&S35187[42], 22}, {NULL}};
const Segment SGI75601[] = {{&S35187[30], 4}, {NULL}};
const Segment SGI75602[] = {{&S35188[42], 10}, {NULL}};
const Segment SGI75603[] = {{&S35188[42], 22}, {NULL}};
const Segment SGI75604[] = {{&S35188[30], 4}, {NULL}};
const Segment SGI75605[] = {{&S35188[112], 3}, {NULL}};
const Segment SGI75606[] = {{&S35188[78], 5}, {NULL}};
const Segment SGI75607[] = {{&S35189[42], 10}, {NULL}};
const Segment SGI75608[] = {{&S35189[42], 22}, {&S35189[237], 5}, {NULL}};
const Segment SGI75609[] = {{&S35189[30], 4}, {NULL}};
const Segment SGI75610[] = {{&S35189[113], 3}, {NULL}};
const Segment SGI75611[] = {{&S35189[78], 6}, {NULL}};
const Segment SGI75612[] = {{&S35190[42], 10}, {NULL}};
const Segment SGI75613[] = {{&S35190[42], 19}, {NULL}};
const Segment SGI75614[] = {{&S35190[30], 4}, {NULL}};
const Segment SGI75615[] = {{&S35190[109], 3}, {NULL}};
const Segment SGI75616[] = {{&S35190[75], 5}, {NULL}};
const Segment SGI75617[] = {{&S35191[53], 18}, {NULL}};
const Segment SGI75618[] = {{&S35191[48], 4}, {NULL}};
const Segment SGI75619[] = {{&S35192[42], 10}, {NULL}};
const Segment SGI75620[] = {{&S35192[42], 22}, {NULL}};
const Segment SGI75621[] = {{&S35192[30], 4}, {NULL}};
const Segment SGI75622[] = {{&S35192[114], 3}, {NULL}};
const Segment SGI75623[] = {{&S35192[78], 7}, {NULL}};
const Segment SGI75624[] = {{&S35192[134], 8}, {NULL}};
const Segment SGI75625[] = {{&S35193[42], 10}, {NULL}};
const Segment SGI75626[] = {{&S35193[42], 22}, {NULL}};
const Segment SGI75627[] = {{&S35193[30], 4}, {NULL}};
const Segment SGI75628[] = {{&S35195[42], 10}, {NULL}};
const Segment SGI75629[] = {{&S35195[42], 28}, {NULL}};
const Segment SGI75630[] = {{&S35195[30], 4}, {NULL}};
const Segment SGI75631[] = {{&S35196[30], 1}, {NULL}};
const Segment SGI75632[] = {{&S35196[61], 9}, {NULL}};
const Segment SGI75633[] = {{&S35196[49], 4}, {NULL}};
const Segment SGI75634[] = {{&S35197[42], 10}, {NULL}};
const Segment SGI75635[] = {{&S35197[42], 22}, {&S35197[211], 7}, {NULL}};
const Segment SGI75636[] = {{&S35197[30], 4}, {NULL}};
const Segment SGI75637[] = {{&S35197[113], 3}, {NULL}};
const Segment SGI75638[] = {{&S35197[78], 6}, {NULL}};
const Segment SGI75639[] = {{&S35198[42], 10}, {NULL}};
const Segment SGI75640[] = {{&S35198[42], 22}, {&S35198[203], 7}, {NULL}};
const Segment SGI75641[] = {{&S35198[30], 4}, {NULL}};
const Segment SGI75642[] = {{&S35198[113], 3}, {NULL}};
const Segment SGI75643[] = {{&S35198[78], 6}, {NULL}};
const Segment SGI75644[] = {{&S35199[28], 4}, {NULL}};
const Segment SGI75645[] = {{&S35200[44], 10}, {NULL}};
const Segment SGI75646[] = {{&S35200[44], 28}, {NULL}};
const Segment SGI75647[] = {{&S35200[32], 4}, {NULL}};
const Segment SGI75648[] = {{&S35201[42], 9}, {NULL}};
const Segment SGI75649[] = {{&S35201[30], 4}, {NULL}};
const Segment SGI75650[] = {{&S35201[101], 3}, {NULL}};
const Segment SGI75651[] = {{&S35201[65], 7}, {NULL}};
const Segment SGI75652[] = {{&S35201[121], 8}, {NULL}};
const Segment SGI75653[] = {{&S35203[42], 10}, {NULL}};
const Segment SGI75654[] = {{&S35203[42], 22}, {&S35203[206], 13}, {NULL}};
const Segment SGI75655[] = {{&S35203[30], 4}, {NULL}};
const Segment SGI75656[] = {{&S35203[113], 3}, {NULL}};
const Segment SGI75657[] = {{&S35203[78], 6}, {NULL}};
const Segment SGI75658[] = {{&S35204[42], 9}, {NULL}};
const Segment SGI75659[] = {{&S35204[42], 21}, {NULL}};
const Segment SGI75660[] = {{&S35204[30], 4}, {NULL}};
const Segment SGI75661[] = {{&S35204[112], 3}, {NULL}};
const Segment SGI75662[] = {{&S35204[77], 6}, {NULL}};
const Segment SGI75663[] = {{&S35205[96], 29}, {NULL}};
const Segment SGI75664[] = {{&S35206[43], 9}, {NULL}};
const Segment SGI75665[] = {{&S35206[24], 5}, {NULL}};
const Segment SGI75666[] = {{&S35737[44], 7}, {NULL}};
const Segment SGI75667[] = {{&S35737[44], 20}, {NULL}};
const Segment SGI75668[] = {{&S35737[30], 6}, {NULL}};
const Segment SGI75669[] = {{&S35737[112], 3}, {NULL}};
const Segment SGI75670[] = {{&S35737[78], 5}, {NULL}};
const Segment SGI75671[] = {{&S35744[44], 14}, {NULL}};
const Segment SGI75672[] = {{&S35744[44], 31}, {NULL}};
const Segment SGI75673[] = {{&S35744[30], 6}, {NULL}};
const Segment SGI75674[] = {{&S35744[136], 3}, {NULL}};
const Segment SGI75675[] = {{&S35744[102], 5}, {NULL}};
const Segment SGI75676[] = {{&S42021[44], 10}, {NULL}};
const Segment SGI75677[] = {{&S42021[44], 27}, {NULL}};
const Segment SGI75678[] = {{&S42021[30], 6}, {NULL}};
const Segment SGI75679[] = {{&S42021[119], 3}, {NULL}};
const Segment SGI75680[] = {{&S42021[85], 5}, {NULL}};
const Segment SGI75681[] = {{&S42022[38], 10}, {NULL}};
const Segment SGI75682[] = {{&S42022[31], 6}, {NULL}};
const Segment SGI75683[] = {{&S35977[38], 9}, {NULL}};
const Segment SGI75684[] = {{&S35977[31], 6}, {NULL}};
const Segment SGI75685[] = {{&S42145[44], 18}, {NULL}};
const Segment SGI75686[] = {{&S42145[44], 35}, {NULL}};
const Segment SGI75687[] = {{&S42145[30], 6}, {NULL}};
const Segment SGI75688[] = {{&S42145[127], 3}, {NULL}};
const Segment SGI75689[] = {{&S42145[93], 5}, {NULL}};
const Segment SGI75690[] = {{&S42146[38], 18}, {NULL}};
const Segment SGI75691[] = {{&S42146[31], 6}, {NULL}};
const Segment SGI75692[] = {{&S42147[42], 18}, {NULL}};
const Segment SGI75693[] = {{&S42147[30], 6}, {NULL}};
const Segment SGI75694[] = {{&S42147[125], 3}, {NULL}};
const Segment SGI75695[] = {{&S42147[91], 5}, {NULL}};
const Segment SGI75696[] = {{&S24589[44], 25}, {NULL}};
const Segment SGI75697[] = {{&S24589[44], 37}, {NULL}};
const Segment SGI75698[] = {{&S24589[30], 6}, {NULL}};
const Segment SGI75699[] = {{&S24589[31], 3}, {NULL}};
const Segment SGI75700[] = {{&S24589[95], 6}, {NULL}};
const Segment SGI75701[] = {{&S24590[38], 35}, {NULL}};
const Segment SGI75702[] = {{&S24590[31], 6}, {NULL}};
const Segment SGI75703[] = {{&S24591[44], 11}, {NULL}};
const Segment SGI75704[] = {{&S24591[44], 23}, {NULL}};
const Segment SGI75705[] = {{&S24591[30], 6}, {NULL}};
const Segment SGI75706[] = {{&S24591[31], 3}, {NULL}};
const Segment SGI75707[] = {{&S24591[81], 6}, {NULL}};
const Segment SGI75708[] = {{&S24592[44], 41}, {NULL}};
const Segment SGI75709[] = {{&S24592[44], 53}, {NULL}};
const Segment SGI75710[] = {{&S24592[30], 6}, {NULL}};
const Segment SGI75711[] = {{&S24592[31], 3}, {NULL}};
const Segment SGI75712[] = {{&S24592[111], 6}, {NULL}};
const Segment SGI75713[] = {{&S24593[44], 29}, {NULL}};
const Segment SGI75714[] = {{&S24593[44], 41}, {NULL}};
const Segment SGI75715[] = {{&S24593[30], 6}, {NULL}};
const Segment SGI75716[] = {{&S24593[31], 3}, {NULL}};
const Segment SGI75717[] = {{&S24593[99], 6}, {NULL}};
const Segment SGI75718[] = {{&S24594[38], 41}, {NULL}};
const Segment SGI75719[] = {{&S24594[31], 6}, {NULL}};
const Segment SGI75720[] = {{&S24595[44], 23}, {NULL}};
const Segment SGI75721[] = {{&S24595[44], 35}, {NULL}};
const Segment SGI75722[] = {{&S24595[30], 6}, {NULL}};
const Segment SGI75723[] = {{&S24595[31], 3}, {NULL}};
const Segment SGI75724[] = {{&S24595[93], 6}, {NULL}};
const Segment SGI75725[] = {{&S24596[44], 41}, {NULL}};
const Segment SGI75726[] = {{&S24596[44], 53}, {NULL}};
const Segment SGI75727[] = {{&S24596[30], 6}, {NULL}};
const Segment SGI75728[] = {{&S24596[114], 4}, {NULL}};
const Segment SGI75729[] = {{&S36242[44], 11}, {NULL}};
const Segment SGI75730[] = {{&S24596[114], 3}, {NULL}};
const Segment SGI75731[] = {{&S36242[44], 22}, {NULL}};
const Segment SGI75732[] = {{&S36242[30], 6}, {NULL}};
const Segment SGI75733[] = {{&S36242[114], 3}, {NULL}};
const Segment SGI75734[] = {{&S36242[80], 5}, {NULL}};
const Segment SGI75735[] = {{&S24597[44], 35}, {NULL}};
const Segment SGI75736[] = {{&S24597[44], 47}, {NULL}};
const Segment SGI75737[] = {{&S24597[30], 6}, {NULL}};
const Segment SGI75738[] = {{&S24597[31], 3}, {NULL}};
const Segment SGI75739[] = {{&S24597[105], 6}, {NULL}};
const Segment SGI75740[] = {{&S24598[44], 29}, {NULL}};
const Segment SGI75741[] = {{&S24598[44], 41}, {NULL}};
const Segment SGI75742[] = {{&S24598[30], 6}, {NULL}};
const Segment SGI75743[] = {{&S24598[31], 3}, {NULL}};
const Segment SGI75744[] = {{&S24598[99], 6}, {NULL}};
const Segment SGI75745[] = {{&S24599[44], 41}, {NULL}};
const Segment SGI75746[] = {{&S24599[44], 53}, {NULL}};
const Segment SGI75747[] = {{&S24599[30], 6}, {NULL}};
const Segment SGI75748[] = {{&S24599[31], 3}, {NULL}};
const Segment SGI75749[] = {{&S24599[111], 6}, {NULL}};
const Segment SGI75750[] = {{&S24600[44], 41}, {NULL}};
const Segment SGI75751[] = {{&S24600[44], 53}, {NULL}};
const Segment SGI75752[] = {{&S24600[30], 6}, {NULL}};
const Segment SGI75753[] = {{&S24600[114], 6}, {NULL}};
const Segment SGI75754[] = {{&S24601[44], 18}, {NULL}};
const Segment SGI75755[] = {{&S24601[44], 47}, {NULL}};
const Segment SGI75756[] = {{&S24601[30], 6}, {NULL}};
const Segment SGI75757[] = {{&S24601[31], 3}, {NULL}};
const Segment SGI75758[] = {{&S24601[105], 6}, {NULL}};
const Segment SGI75759[] = {{&S24602[44], 37}, {NULL}};
const Segment SGI75760[] = {{&S24602[44], 49}, {NULL}};
const Segment SGI75761[] = {{&S24602[30], 6}, {NULL}};
const Segment SGI75762[] = {{&S24602[31], 3}, {NULL}};
const Segment SGI75763[] = {{&S24602[107], 6}, {NULL}};
const Segment SGI75764[] = {{&S24603[42], 29}, {NULL}};
const Segment SGI75765[] = {{&S24603[30], 6}, {NULL}};
const Segment SGI75766[] = {{&S24603[31], 3}, {NULL}};
const Segment SGI75767[] = {{&S24603[97], 6}, {NULL}};
const Segment SGI75768[] = {{&S24604[38], 41}, {NULL}};
const Segment SGI75769[] = {{&S24604[31], 6}, {NULL}};
const Segment SGI75770[] = {{&S24605[44], 11}, {NULL}};
const Segment SGI75771[] = {{&S24605[44], 23}, {NULL}};
const Segment SGI75772[] = {{&S24605[30], 6}, {NULL}};
const Segment SGI75773[] = {{&S24605[31], 3}, {NULL}};
const Segment SGI75774[] = {{&S24605[81], 6}, {NULL}};
const Segment SGI75775[] = {{&S24606[44], 27}, {NULL}};
const Segment SGI75776[] = {{&S24606[44], 39}, {NULL}};
const Segment SGI75777[] = {{&S24606[30], 6}, {NULL}};
const Segment SGI75778[] = {{&S24606[31], 3}, {NULL}};
const Segment SGI75779[] = {{&S24606[97], 6}, {NULL}};
const Segment SGI75780[] = {{&S24607[44], 11}, {NULL}};
const Segment SGI75781[] = {{&S24607[44], 23}, {NULL}};
const Segment SGI75782[] = {{&S24607[30], 6}, {NULL}};
const Segment SGI75783[] = {{&S24607[84], 6}, {NULL}};
const Segment SGI75784[] = {{&S24608[44], 53}, {NULL}};
const Segment SGI75785[] = {{&S24608[30], 6}, {NULL}};
const Segment SGI75786[] = {{&S36267[38], 11}, {NULL}};
const Segment SGI75787[] = {{&S36267[31], 6}, {NULL}};
const Segment SGI75788[] = {{&S24614[42], 41}, {NULL}};
const Segment SGI75789[] = {{&S24614[30], 6}, {NULL}};
const Segment SGI75790[] = {{&S24614[31], 3}, {NULL}};
const Segment SGI75791[] = {{&S24614[109], 6}, {NULL}};
const Segment SGI75792[] = {{&S36288[44], 15}, {NULL}};
const Segment SGI75793[] = {{&S36288[44], 25}, {NULL}};
const Segment SGI75794[] = {{&S36288[30], 6}, {NULL}};
const Segment SGI75795[] = {{&S36288[117], 3}, {NULL}};
const Segment SGI75796[] = {{&S36288[83], 5}, {NULL}};
const Segment SGI75797[] = {{&S36302[44], 15}, {NULL}};
const Segment SGI75798[] = {{&S36302[44], 24}, {NULL}};
const Segment SGI75799[] = {{&S36302[30], 6}, {NULL}};
const Segment SGI75800[] = {{&S36302[137], 3}, {NULL}};
const Segment SGI75801[] = {{&S36302[103], 5}, {NULL}};
const Segment SGI75802[] = {{&S42257[38], 10}, {NULL}};
const Segment SGI75803[] = {{&S42257[31], 6}, {NULL}};
const Segment SGI75804[] = {{&S42258[44], 10}, {NULL}};
const Segment SGI75805[] = {{&S42258[44], 27}, {NULL}};
const Segment SGI75806[] = {{&S42258[30], 6}, {NULL}};
const Segment SGI75807[] = {{&S42258[119], 3}, {NULL}};
const Segment SGI75808[] = {{&S42258[85], 5}, {NULL}};
const Segment SGI75809[] = {{&S42259[42], 10}, {NULL}};
const Segment SGI75810[] = {{&S42259[30], 6}, {NULL}};
const Segment SGI75811[] = {{&S42259[117], 3}, {NULL}};
const Segment SGI75812[] = {{&S42259[83], 5}, {NULL}};
const Segment SGI75813[] = {{&S42261[44], 10}, {NULL}};
const Segment SGI75814[] = {{&S42261[44], 27}, {NULL}};
const Segment SGI75815[] = {{&S42261[30], 6}, {NULL}};
const Segment SGI75816[] = {{&S42261[121], 3}, {NULL}};
const Segment SGI75817[] = {{&S42261[85], 7}, {NULL}};
const Segment SGI75818[] = {{&S42261[141], 8}, {NULL}};
const Segment SGI75819[] = {{&S42263[44], 11}, {NULL}};
const Segment SGI75820[] = {{&S42263[44], 28}, {NULL}};
const Segment SGI75821[] = {{&S42263[30], 6}, {NULL}};
const Segment SGI75822[] = {{&S42263[120], 3}, {NULL}};
const Segment SGI75823[] = {{&S42263[86], 5}, {NULL}};
const Segment SGI75824[] = {{&S35930[42], 10}, {NULL}};
const Segment SGI75825[] = {{&S35930[42], 22}, {NULL}};
const Segment SGI75826[] = {{&S35930[30], 4}, {NULL}};
const Segment SGI75827[] = {{&S35930[113], 3}, {NULL}};
const Segment SGI75828[] = {{&S35930[78], 6}, {NULL}};
const Segment SGI75829[] = {{&S35931[42], 28}, {NULL}};
const Segment SGI75830[] = {{&S35931[42], 40}, {NULL}};
const Segment SGI75831[] = {{&S35931[30], 4}, {NULL}};
const Segment SGI75832[] = {{&S35931[131], 3}, {NULL}};
const Segment SGI75833[] = {{&S35931[96], 6}, {NULL}};
const Segment SGI75834[] = {{&S35932[42], 11}, {NULL}};
const Segment SGI75835[] = {{&S35932[42], 23}, {NULL}};
const Segment SGI75836[] = {{&S35932[30], 4}, {NULL}};
const Segment SGI75837[] = {{&S35932[114], 3}, {NULL}};
const Segment SGI75838[] = {{&S35932[79], 6}, {NULL}};
const Segment SGI75839[] = {{&S35933[36], 10}, {NULL}};
const Segment SGI75840[] = {{&S35933[31], 4}, {NULL}};
const Segment SGI75841[] = {{&S35934[42], 28}, {NULL}};
const Segment SGI75842[] = {{&S35934[42], 40}, {NULL}};
const Segment SGI75843[] = {{&S35934[30], 4}, {NULL}};
const Segment SGI75844[] = {{&S35934[131], 3}, {NULL}};
const Segment SGI75845[] = {{&S35934[96], 6}, {NULL}};
const Segment SGI75846[] = {{&S35935[40], 10}, {NULL}};
const Segment SGI75847[] = {{&S35935[30], 4}, {NULL}};
const Segment SGI75848[] = {{&S35935[111], 3}, {NULL}};
const Segment SGI75849[] = {{&S35935[76], 6}, {NULL}};
const Segment SGI75850[] = {{&S35936[42], 10}, {NULL}};
const Segment SGI75851[] = {{&S35936[42], 22}, {NULL}};
const Segment SGI75852[] = {{&S35936[30], 4}, {NULL}};
const Segment SGI75853[] = {{&S35936[113], 3}, {NULL}};
const Segment SGI75854[] = {{&S35936[78], 6}, {NULL}};
const Segment SGI75855[] = {{&S35937[42], 29}, {NULL}};
const Segment SGI75856[] = {{&S35937[42], 41}, {NULL}};
const Segment SGI75857[] = {{&S35937[30], 4}, {NULL}};
const Segment SGI75858[] = {{&S35937[132], 3}, {NULL}};
const Segment SGI75859[] = {{&S35937[97], 6}, {NULL}};
const Segment SGI75860[] = {{&S35938[36], 11}, {NULL}};
const Segment SGI75861[] = {{&S35938[31], 4}, {NULL}};
const Segment SGI75862[] = {{&S35939[42], 10}, {NULL}};
const Segment SGI75863[] = {{&S35939[42], 22}, {NULL}};
const Segment SGI75864[] = {{&S35939[30], 4}, {NULL}};
const Segment SGI75865[] = {{&S35939[123], 3}, {NULL}};
const Segment SGI75866[] = {{&S35939[88], 6}, {NULL}};
const Segment SGI75867[] = {{&S35939[143], 8}, {NULL}};
const Segment SGI75868[] = {{&S35940[42], 9}, {NULL}};
const Segment SGI75869[] = {{&S35940[30], 4}, {NULL}};
const Segment SGI75870[] = {{&S35941[42], 10}, {NULL}};
const Segment SGI75871[] = {{&S35941[42], 21}, {NULL}};
const Segment SGI75872[] = {{&S35941[30], 4}, {NULL}};
const Segment SGI75873[] = {{&S35941[112], 3}, {NULL}};
const Segment SGI75874[] = {{&S35941[77], 6}, {NULL}};
const Segment SGI75875[] = {{&S35942[42], 10}, {NULL}};
const Segment SGI75876[] = {{&S35942[42], 22}, {NULL}};
const Segment SGI75877[] = {{&S35942[30], 4}, {NULL}};
const Segment SGI75878[] = {{&S35942[81], 6}, {NULL}};
const Segment SGI75879[] = {{&S35943[42], 10}, {&S35943[171], 18}, {NULL}};
const Segment SGI75880[] = {{&S35943[42], 22}, {NULL}};
const Segment SGI75881[] = {{&S35943[30], 4}, {NULL}};
const Segment SGI75882[] = {{&S35943[113], 3}, {NULL}};
const Segment SGI75883[] = {{&S35943[78], 6}, {NULL}};
const Segment SGI75884[] = {{&S35944[53], 18}, {NULL}};
const Segment SGI75885[] = {{&S35944[48], 4}, {NULL}};
const Segment SGI75886[] = {{&S35945[42], 10}, {NULL}};
const Segment SGI75887[] = {{&S35945[42], 22}, {NULL}};
const Segment SGI75888[] = {{&S35945[30], 4}, {NULL}};
const Segment SGI75889[] = {{&S35946[42], 10}, {NULL}};
const Segment SGI75890[] = {{&S35946[42], 22}, {NULL}};
const Segment SGI75891[] = {{&S35946[30], 4}, {NULL}};
const Segment SGI75892[] = {{&S35946[114], 3}, {NULL}};
const Segment SGI75893[] = {{&S35946[78], 7}, {NULL}};
const Segment SGI75894[] = {{&S35946[134], 8}, {NULL}};
const Segment SGI75895[] = {{&S35947[42], 10}, {NULL}};
const Segment SGI75896[] = {{&S35947[42], 19}, {NULL}};
const Segment SGI75897[] = {{&S35947[30], 4}, {NULL}};
const Segment SGI75898[] = {{&S35947[109], 3}, {NULL}};
const Segment SGI75899[] = {{&S35947[75], 5}, {NULL}};
const Segment SGI75900[] = {{&S35948[42], 10}, {NULL}};
const Segment SGI75901[] = {{&S35948[42], 32}, {NULL}};
const Segment SGI75902[] = {{&S35948[30], 4}, {NULL}};
const Segment SGI75903[] = {{&S35948[123], 3}, {NULL}};
const Segment SGI75904[] = {{&S35948[88], 6}, {NULL}};
const Segment SGI75905[] = {{&S35949[42], 28}, {NULL}};
const Segment SGI75906[] = {{&S35949[42], 40}, {NULL}};
const Segment SGI75907[] = {{&S35949[30], 4}, {NULL}};
const Segment SGI75908[] = {{&S35949[131], 3}, {NULL}};
const Segment SGI75909[] = {{&S35949[96], 6}, {NULL}};
const Segment SGI75910[] = {{&S35950[42], 10}, {NULL}};
const Segment SGI75911[] = {{&S35950[42], 22}, {&S35950[203], 7}, {NULL}};
const Segment SGI75912[] = {{&S35950[30], 4}, {NULL}};
const Segment SGI75913[] = {{&S35950[113], 3}, {NULL}};
const Segment SGI75914[] = {{&S35950[78], 6}, {NULL}};
const Segment SGI75915[] = {{&S35951[97], 29}, {NULL}};
const Segment SGI75916[] = {{&S35953[42], 27}, {NULL}};
const Segment SGI75917[] = {{&S35953[42], 39}, {NULL}};
const Segment SGI75918[] = {{&S35953[30], 4}, {NULL}};
const Segment SGI75919[] = {{&S35953[130], 3}, {NULL}};
const Segment SGI75920[] = {{&S35953[95], 6}, {NULL}};
const Segment SGI75921[] = {{&S35954[42], 10}, {NULL}};
const Segment SGI75922[] = {{&S35954[42], 22}, {NULL}};
const Segment SGI75923[] = {{&S35954[30], 4}, {NULL}};
const Segment SGI75924[] = {{&S35954[81], 4}, {NULL}};
const Segment SGI75925[] = {{&S35954[81], 3}, {NULL}};
const Segment SGI75926[] = {{&S24918[44], 29}, {NULL}};
const Segment SGI75927[] = {{&S24918[44], 41}, {NULL}};
const Segment SGI75928[] = {{&S24918[30], 6}, {NULL}};
const Segment SGI75929[] = {{&S24918[31], 3}, {NULL}};
const Segment SGI75930[] = {{&S24918[99], 6}, {NULL}};
const Segment SGI75931[] = {{&S24919[38], 41}, {NULL}};
const Segment SGI75932[] = {{&S24919[31], 6}, {NULL}};
const Segment SGI75933[] = {{&S24920[44], 25}, {NULL}};
const Segment SGI75934[] = {{&S24920[44], 37}, {NULL}};
const Segment SGI75935[] = {{&S24920[30], 6}, {NULL}};
const Segment SGI75936[] = {{&S24920[31], 3}, {NULL}};
const Segment SGI75937[] = {{&S24920[95], 6}, {NULL}};
const Segment SGI75938[] = {{&S24921[44], 41}, {NULL}};
const Segment SGI75939[] = {{&S24921[44], 53}, {NULL}};
const Segment SGI75940[] = {{&S24921[30], 6}, {NULL}};
const Segment SGI75941[] = {{&S24921[31], 3}, {NULL}};
const Segment SGI75942[] = {{&S24921[111], 6}, {NULL}};
const Segment SGI75943[] = {{&S24922[44], 29}, {NULL}};
const Segment SGI75944[] = {{&S24922[44], 41}, {NULL}};
const Segment SGI75945[] = {{&S24922[30], 6}, {NULL}};
const Segment SGI75946[] = {{&S24922[31], 3}, {NULL}};
const Segment SGI75947[] = {{&S24922[99], 6}, {NULL}};
const Segment SGI75948[] = {{&S24923[44], 41}, {NULL}};
const Segment SGI75949[] = {{&S24923[44], 53}, {NULL}};
const Segment SGI75950[] = {{&S24923[30], 6}, {NULL}};
const Segment SGI75951[] = {{&S24923[31], 3}, {NULL}};
const Segment SGI75952[] = {{&S24923[111], 6}, {NULL}};
const Segment SGI75953[] = {{&S24924[44], 34}, {NULL}};
const Segment SGI75954[] = {{&S24924[44], 46}, {NULL}};
const Segment SGI75955[] = {{&S24924[30], 6}, {NULL}};
const Segment SGI75956[] = {{&S24924[31], 3}, {NULL}};
const Segment SGI75957[] = {{&S24924[104], 6}, {NULL}};
const Segment SGI75958[] = {{&S24925[44], 34}, {NULL}};
const Segment SGI75959[] = {{&S24925[44], 46}, {NULL}};
const Segment SGI75960[] = {{&S24925[30], 6}, {NULL}};
const Segment SGI75961[] = {{&S24925[31], 3}, {NULL}};
const Segment SGI75962[] = {{&S24925[104], 6}, {NULL}};
const Segment SGI75963[] = {{&S24926[44], 33}, {NULL}};
const Segment SGI75964[] = {{&S24926[44], 45}, {NULL}};
const Segment SGI75965[] = {{&S24926[30], 6}, {NULL}};
const Segment SGI75966[] = {{&S24926[31], 3}, {NULL}};
const Segment SGI75967[] = {{&S24926[103], 6}, {NULL}};
const Segment SGI75968[] = {{&S24927[44], 34}, {NULL}};
const Segment SGI75969[] = {{&S24927[44], 46}, {NULL}};
const Segment SGI75970[] = {{&S24927[30], 6}, {NULL}};
const Segment SGI75971[] = {{&S24927[31], 3}, {NULL}};
const Segment SGI75972[] = {{&S24927[104], 6}, {NULL}};
const Segment SGI75973[] = {{&S24928[44], 31}, {NULL}};
const Segment SGI75974[] = {{&S24928[44], 43}, {NULL}};
const Segment SGI75975[] = {{&S24928[30], 6}, {NULL}};
const Segment SGI75976[] = {{&S24928[31], 3}, {NULL}};
const Segment SGI75977[] = {{&S24928[101], 6}, {NULL}};
const Segment SGI75978[] = {{&S24929[44], 41}, {NULL}};
const Segment SGI75979[] = {{&S24929[44], 53}, {NULL}};
const Segment SGI75980[] = {{&S24929[30], 6}, {NULL}};
const Segment SGI75981[] = {{&S24929[114], 6}, {NULL}};
const Segment SGI75982[] = {{&S24931[44], 41}, {NULL}};
const Segment SGI75983[] = {{&S24931[44], 53}, {NULL}};
const Segment SGI75984[] = {{&S24931[30], 6}, {NULL}};
const Segment SGI75985[] = {{&S24931[114], 6}, {NULL}};
const Segment SGI75986[] = {{&S24932[38], 41}, {NULL}};
const Segment SGI75987[] = {{&S24932[31], 6}, {NULL}};
const Segment SGI75988[] = {{&S24933[44], 41}, {NULL}};
const Segment SGI75989[] = {{&S24933[44], 53}, {NULL}};
const Segment SGI75990[] = {{&S24933[30], 6}, {NULL}};
const Segment SGI75991[] = {{&S24933[114], 4}, {NULL}};
const Segment SGI75992[] = {{&S24933[114], 3}, {NULL}};
const Segment SGI75993[] = {{&S24934[44], 20}, {NULL}};
const Segment SGI75994[] = {{&S24934[44], 32}, {NULL}};
const Segment SGI75995[] = {{&S24934[30], 6}, {NULL}};
const Segment SGI75996[] = {{&S24934[31], 3}, {NULL}};
const Segment SGI75997[] = {{&S24934[90], 6}, {NULL}};
const Segment SGI75998[] = {{&S24935[44], 7}, {NULL}};
const Segment SGI75999[] = {{&S24935[44], 19}, {NULL}};
const Segment SGI76000[] = {{&S24935[30], 6}, {NULL}};
const Segment SGI76001[] = {{&S24935[31], 3}, {NULL}};
const Segment SGI76002[] = {{&S24935[77], 6}, {NULL}};
const Segment SGI76003[] = {{&S24936[44], 41}, {NULL}};
const Segment SGI76004[] = {{&S24936[44], 79}, {NULL}};
const Segment SGI76005[] = {{&S24936[30], 6}, {NULL}};
const Segment SGI76006[] = {{&S24936[31], 3}, {NULL}};
const Segment SGI76007[] = {{&S24936[137], 6}, {NULL}};
const Segment SGI76008[] = {{&S24938[44], 11}, {NULL}};
const Segment SGI76009[] = {{&S24938[44], 23}, {NULL}};
const Segment SGI76010[] = {{&S24938[30], 6}, {NULL}};
const Segment SGI76011[] = {{&S24938[31], 3}, {NULL}};
const Segment SGI76012[] = {{&S24938[81], 6}, {NULL}};
const Segment SGI76013[] = {{&S24939[42], 29}, {NULL}};
const Segment SGI76014[] = {{&S36613[44], 7}, {NULL}};
const Segment SGI76015[] = {{&S24939[30], 6}, {NULL}};
const Segment SGI76016[] = {{&S36613[44], 18}, {NULL}};
const Segment SGI76017[] = {{&S36613[30], 6}, {NULL}};
const Segment SGI76018[] = {{&S24939[31], 3}, {NULL}};
const Segment SGI76019[] = {{&S24939[97], 6}, {NULL}};
const Segment SGI76020[] = {{&S36613[110], 3}, {NULL}};
const Segment SGI76021[] = {{&S36613[76], 5}, {NULL}};
const Segment SGI76022[] = {{&S24940[42], 25}, {NULL}};
const Segment SGI76023[] = {{&S36614[44], 7}, {NULL}};
const Segment SGI76024[] = {{&S36614[44], 18}, {NULL}};
const Segment SGI76025[] = {{&S24940[30], 6}, {NULL}};
const Segment SGI76026[] = {{&S36614[30], 6}, {NULL}};
const Segment SGI76027[] = {{&S24940[31], 3}, {NULL}};
const Segment SGI76028[] = {{&S36614[110], 3}, {NULL}};
const Segment SGI76029[] = {{&S24940[93], 6}, {NULL}};
const Segment SGI76030[] = {{&S36614[76], 5}, {NULL}};
const Segment SGI76031[] = {{&S36615[38], 7}, {NULL}};
const Segment SGI76032[] = {{&S36615[31], 6}, {NULL}};
const Segment SGI76033[] = {{&S24942[44], 53}, {NULL}};
const Segment SGI76034[] = {{&S24942[30], 6}, {NULL}};
const Segment SGI76035[] = {{&S36616[44], 7}, {NULL}};
const Segment SGI76036[] = {{&S36616[44], 18}, {NULL}};
const Segment SGI76037[] = {{&S36616[30], 6}, {NULL}};
const Segment SGI76038[] = {{&S36616[110], 3}, {&S36616[169], 3}, {NULL}};
const Segment SGI76039[] = {{&S36616[76], 5}, {NULL}};
const Segment SGI76040[] = {{&S24943[42], 41}, {NULL}};
const Segment SGI76041[] = {{&S24943[30], 6}, {NULL}};
const Segment SGI76042[] = {{&S24943[31], 3}, {NULL}};
const Segment SGI76043[] = {{&S24943[109], 6}, {NULL}};
const Segment SGI76044[] = {{&S36617[42], 7}, {NULL}};
const Segment SGI76045[] = {{&S36617[30], 6}, {NULL}};
const Segment SGI76046[] = {{&S36617[108], 3}, {NULL}};
const Segment SGI76047[] = {{&S36617[74], 5}, {NULL}};
const Segment SGI76048[] = {{&S24944[44], 11}, {NULL}};
const Segment SGI76049[] = {{&S24944[44], 23}, {NULL}};
const Segment SGI76050[] = {{&S24944[30], 6}, {NULL}};
const Segment SGI76051[] = {{&S24944[31], 3}, {NULL}};
const Segment SGI76052[] = {{&S24944[81], 6}, {NULL}};
const Segment SGI76053[] = {{&S36618[44], 7}, {NULL}};
const Segment SGI76054[] = {{&S36618[44], 18}, {NULL}};
const Segment SGI76055[] = {{&S36618[30], 6}, {NULL}};
const Segment SGI76056[] = {{&S36618[110], 3}, {&S36618[152], 3}, {NULL}};
const Segment SGI76057[] = {{&S36618[76], 5}, {NULL}};
const Segment SGI76058[] = {{&S42609[44], 7}, {NULL}};
const Segment SGI76059[] = {{&S42609[44], 24}, {NULL}};
const Segment SGI76060[] = {{&S42609[30], 6}, {NULL}};
const Segment SGI76061[] = {{&S42609[116], 3}, {NULL}};
const Segment SGI76062[] = {{&S42609[82], 5}, {NULL}};
const Segment SGI76063[] = {{&S42610[38], 7}, {NULL}};
const Segment SGI76064[] = {{&S42610[31], 6}, {NULL}};
const Segment SGI76065[] = {{&S42611[44], 7}, {NULL}};
const Segment SGI76066[] = {{&S42611[44], 32}, {NULL}};
const Segment SGI76067[] = {{&S42611[30], 6}, {NULL}};
const Segment SGI76068[] = {{&S42611[124], 3}, {NULL}};
const Segment SGI76069[] = {{&S42611[90], 5}, {NULL}};
const Segment SGI76070[] = {{&S42612[44], 18}, {NULL}};
const Segment SGI76071[] = {{&S42612[44], 35}, {NULL}};
const Segment SGI76072[] = {{&S42612[30], 6}, {NULL}};
const Segment SGI76073[] = {{&S42612[127], 3}, {NULL}};
const Segment SGI76074[] = {{&S42612[93], 5}, {NULL}};
const Segment SGI76075[] = {{&S42613[44], 11}, {NULL}};
const Segment SGI76076[] = {{&S42613[44], 28}, {NULL}};
const Segment SGI76077[] = {{&S42613[30], 6}, {NULL}};
const Segment SGI76078[] = {{&S42613[120], 3}, {NULL}};
const Segment SGI76079[] = {{&S42613[86], 5}, {NULL}};
const Segment SGI76080[] = {{&S42614[38], 18}, {NULL}};
const Segment SGI76081[] = {{&S42614[31], 6}, {NULL}};
const Segment SGI76082[] = {{&S42615[38], 11}, {NULL}};
const Segment SGI76083[] = {{&S42615[31], 6}, {NULL}};
const Segment SGI76084[] = {{&S24998[44], 10}, {NULL}};
const Segment SGI76085[] = {{&S24998[44], 22}, {NULL}};
const Segment SGI76086[] = {{&S24998[30], 6}, {NULL}};
const Segment SGI76087[] = {{&S24998[31], 3}, {NULL}};
const Segment SGI76088[] = {{&S24998[80], 6}, {NULL}};
const Segment SGI76089[] = {{&S25000[44], 10}, {NULL}};
const Segment SGI76090[] = {{&S25000[44], 22}, {NULL}};
const Segment SGI76091[] = {{&S25000[30], 6}, {NULL}};
const Segment SGI76092[] = {{&S25000[31], 3}, {NULL}};
const Segment SGI76093[] = {{&S25000[80], 6}, {NULL}};
const Segment SGI76094[] = {{&S25010[38], 10}, {NULL}};
const Segment SGI76095[] = {{&S25010[31], 6}, {NULL}};
const Segment SGI76096[] = {{&S25011[44], 11}, {NULL}};
const Segment SGI76097[] = {{&S25011[44], 23}, {NULL}};
const Segment SGI76098[] = {{&S25011[30], 6}, {NULL}};
const Segment SGI76099[] = {{&S25011[31], 3}, {NULL}};
const Segment SGI76100[] = {{&S25011[81], 6}, {NULL}};
const Segment SGI76101[] = {{&S25012[44], 11}, {NULL}};
const Segment SGI76102[] = {{&S25012[44], 23}, {NULL}};
const Segment SGI76103[] = {{&S25012[30], 6}, {NULL}};
const Segment SGI76104[] = {{&S25012[31], 3}, {NULL}};
const Segment SGI76105[] = {{&S25012[81], 6}, {NULL}};
const Segment SGI76106[] = {{&S25013[44], 29}, {NULL}};
const Segment SGI76107[] = {{&S25013[44], 41}, {NULL}};
const Segment SGI76108[] = {{&S25013[30], 6}, {NULL}};
const Segment SGI76109[] = {{&S25013[31], 3}, {NULL}};
const Segment SGI76110[] = {{&S25013[99], 6}, {NULL}};
const Segment SGI76111[] = {{&S32412[42], 6}, {NULL}};
const Segment SGI76112[] = {{&S32412[42], 17}, {NULL}};
const Segment SGI76113[] = {{&S32412[30], 4}, {NULL}};
const Segment SGI76114[] = {{&S32412[31], 3}, {NULL}};
const Segment SGI76115[] = {{&S32412[73], 6}, {NULL}};
const Segment SGI76116[] = {{&S25026[44], 10}, {NULL}};
const Segment SGI76117[] = {{&S25026[44], 24}, {NULL}};
const Segment SGI76118[] = {{&S25026[30], 6}, {NULL}};
const Segment SGI76119[] = {{&S25026[31], 3}, {NULL}};
const Segment SGI76120[] = {{&S25026[82], 6}, {NULL}};
const Segment SGI76121[] = {{&S25056[44], 10}, {NULL}};
const Segment SGI76122[] = {{&S25056[44], 22}, {NULL}};
const Segment SGI76123[] = {{&S25056[30], 6}, {NULL}};
const Segment SGI76124[] = {{&S25056[31], 3}, {NULL}};
const Segment SGI76125[] = {{&S25056[80], 6}, {NULL}};
const Segment SGI76126[] = {{&S25062[44], 10}, {NULL}};
const Segment SGI76127[] = {{&S25062[44], 24}, {NULL}};
const Segment SGI76128[] = {{&S25062[30], 6}, {NULL}};
const Segment SGI76129[] = {{&S25062[31], 3}, {NULL}};
const Segment SGI76130[] = {{&S25062[82], 6}, {NULL}};
const Segment SGI76131[] = {{&S25087[44], 10}, {NULL}};
const Segment SGI76132[] = {{&S25087[44], 24}, {NULL}};
const Segment SGI76133[] = {{&S25087[30], 6}, {NULL}};
const Segment SGI76134[] = {{&S25087[31], 3}, {NULL}};
const Segment SGI76135[] = {{&S25087[82], 6}, {NULL}};
const Segment SGI76136[] = {{&S25088[44], 29}, {NULL}};
const Segment SGI76137[] = {{&S25088[44], 41}, {NULL}};
const Segment SGI76138[] = {{&S25088[30], 6}, {NULL}};
const Segment SGI76139[] = {{&S25088[31], 3}, {NULL}};
const Segment SGI76140[] = {{&S25088[99], 6}, {NULL}};
const Segment SGI76141[] = {{&S25097[44], 10}, {NULL}};
const Segment SGI76142[] = {{&S25097[44], 22}, {NULL}};
const Segment SGI76143[] = {{&S25097[30], 6}, {NULL}};
const Segment SGI76144[] = {{&S25097[31], 3}, {NULL}};
const Segment SGI76145[] = {{&S25097[80], 6}, {NULL}};
const Segment SGI76146[] = {{&S25098[38], 10}, {NULL}};
const Segment SGI76147[] = {{&S25098[31], 6}, {NULL}};
const Segment SGI76148[] = {{&S25101[42], 10}, {NULL}};
const Segment SGI76149[] = {{&S25101[30], 6}, {NULL}};
const Segment SGI76150[] = {{&S25101[31], 3}, {NULL}};
const Segment SGI76151[] = {{&S25101[78], 6}, {NULL}};
const Segment SGI76152[] = {{&S25103[44], 10}, {NULL}};
const Segment SGI76153[] = {{&S25103[44], 22}, {NULL}};
const Segment SGI76154[] = {{&S25103[30], 6}, {NULL}};
const Segment SGI76155[] = {{&S25103[83], 6}, {NULL}};
const Segment SGI76156[] = {{&S25104[44], 10}, {NULL}};
const Segment SGI76157[] = {{&S25104[44], 22}, {NULL}};
const Segment SGI76158[] = {{&S25104[30], 6}, {NULL}};
const Segment SGI76159[] = {{&S25104[124], 3}, {NULL}};
const Segment SGI76160[] = {{&S25104[89], 6}, {NULL}};
const Segment SGI76161[] = {{&S25104[144], 8}, {NULL}};
const Segment SGI76162[] = {{&S25109[44], 10}, {NULL}};
const Segment SGI76163[] = {{&S25109[44], 22}, {NULL}};
const Segment SGI76164[] = {{&S25109[30], 6}, {NULL}};
const Segment SGI76165[] = {{&S25109[31], 3}, {NULL}};
const Segment SGI76166[] = {{&S25109[80], 6}, {NULL}};
const Segment SGI76167[] = {{&S25111[44], 10}, {NULL}};
const Segment SGI76168[] = {{&S25111[44], 22}, {NULL}};
const Segment SGI76169[] = {{&S25111[30], 6}, {NULL}};
const Segment SGI76170[] = {{&S25111[80], 6}, {NULL}};
const Segment SGI76171[] = {{&S25112[58], 10}, {NULL}};
const Segment SGI76172[] = {{&S25112[58], 34}, {NULL}};
const Segment SGI76173[] = {{&S25112[44], 6}, {NULL}};
const Segment SGI76174[] = {{&S25114[44], 11}, {NULL}};
const Segment SGI76175[] = {{&S25114[44], 23}, {NULL}};
const Segment SGI76176[] = {{&S25114[30], 6}, {NULL}};
const Segment SGI76177[] = {{&S25114[31], 3}, {NULL}};
const Segment SGI76178[] = {{&S25114[81], 6}, {NULL}};
const Segment SGI76179[] = {{&S25115[44], 10}, {NULL}};
const Segment SGI76180[] = {{&S25115[44], 34}, {NULL}};
const Segment SGI76181[] = {{&S25115[30], 6}, {NULL}};
const Segment SGI76182[] = {{&S25116[44], 10}, {NULL}};
const Segment SGI76183[] = {{&S25116[44], 34}, {NULL}};
const Segment SGI76184[] = {{&S25116[30], 6}, {NULL}};
const Segment SGI76185[] = {{&S30287[44], 14}, {NULL}};
const Segment SGI76186[] = {{&S30287[44], 26}, {NULL}};
const Segment SGI76187[] = {{&S30287[30], 6}, {NULL}};
const Segment SGI76188[] = {{&S30287[31], 3}, {NULL}};
const Segment SGI76189[] = {{&S25118[44], 10}, {NULL}};
const Segment SGI76190[] = {{&S25118[44], 22}, {NULL}};
const Segment SGI76191[] = {{&S30287[84], 6}, {NULL}};
const Segment SGI76192[] = {{&S25118[30], 6}, {NULL}};
const Segment SGI76193[] = {{&S30287[0], 13}, {NULL}};
const Segment SGI76194[] = {{&S25118[116], 3}, {NULL}};
const Segment SGI76195[] = {{&S25118[80], 7}, {NULL}};
const Segment SGI76196[] = {{&S25118[136], 8}, {NULL}};
const Segment SGI76197[] = {{&S30289[44], 14}, {NULL}};
const Segment SGI76198[] = {{&S30289[44], 37}, {NULL}};
const Segment SGI76199[] = {{&S30289[30], 6}, {NULL}};
const Segment SGI76200[] = {{&S30289[31], 3}, {NULL}};
const Segment SGI76201[] = {{&S30289[95], 6}, {NULL}};
const Segment SGI76202[] = {{&S30289[0], 13}, {NULL}};
const Segment SGI76203[] = {{&S30290[38], 14}, {NULL}};
const Segment SGI76204[] = {{&S30290[31], 6}, {NULL}};
const Segment SGI76205[] = {{&S30290[0], 14}, {NULL}};
const Segment SGI76206[] = {{&S25125[64], 10}, {NULL}};
const Segment SGI76207[] = {{&S25125[64], 22}, {NULL}};
const Segment SGI76208[] = {{&S25125[50], 6}, {NULL}};
const Segment SGI76209[] = {{&S25126[44], 10}, {&S25126[175], 18}, {NULL}};
const Segment SGI76210[] = {{&S25126[44], 22}, {NULL}};
const Segment SGI76211[] = {{&S25126[30], 6}, {NULL}};
const Segment SGI76212[] = {{&S25126[31], 3}, {NULL}};
const Segment SGI76213[] = {{&S25126[80], 6}, {NULL}};
const Segment SGI76214[] = {{&S25127[44], 10}, {NULL}};
const Segment SGI76215[] = {{&S25127[44], 22}, {NULL}};
const Segment SGI76216[] = {{&S25127[30], 6}, {NULL}};
const Segment SGI76217[] = {{&S25127[31], 3}, {NULL}};
const Segment SGI76218[] = {{&S25127[80], 5}, {NULL}};
const Segment SGI76219[] = {{&S32439[42], 6}, {NULL}};
const Segment SGI76220[] = {{&S32439[42], 17}, {NULL}};
const Segment SGI76221[] = {{&S32439[30], 4}, {NULL}};
const Segment SGI76222[] = {{&S32439[31], 3}, {NULL}};
const Segment SGI76223[] = {{&S32439[73], 6}, {NULL}};
const Segment SGI76224[] = {{&S30317[44], 14}, {NULL}};
const Segment SGI76225[] = {{&S30317[44], 26}, {NULL}};
const Segment SGI76226[] = {{&S30317[30], 6}, {NULL}};
const Segment SGI76227[] = {{&S30317[31], 3}, {NULL}};
const Segment SGI76228[] = {{&S30317[84], 6}, {NULL}};
const Segment SGI76229[] = {{&S30317[0], 13}, {NULL}};
const Segment SGI76230[] = {{&S30322[44], 14}, {NULL}};
const Segment SGI76231[] = {{&S30322[44], 25}, {NULL}};
const Segment SGI76232[] = {{&S30322[30], 6}, {NULL}};
const Segment SGI76233[] = {{&S30322[31], 3}, {NULL}};
const Segment SGI76234[] = {{&S30322[83], 5}, {NULL}};
const Segment SGI76235[] = {{&S30322[0], 13}, {NULL}};
const Segment SGI76236[] = {{&S30323[44], 14}, {NULL}};
const Segment SGI76237[] = {{&S30323[44], 31}, {NULL}};
const Segment SGI76238[] = {{&S30323[30], 6}, {NULL}};
const Segment SGI76239[] = {{&S30323[31], 3}, {NULL}};
const Segment SGI76240[] = {{&S30323[89], 5}, {NULL}};
const Segment SGI76241[] = {{&S30323[0], 13}, {NULL}};
const Segment SGI76242[] = {{&S30333[44], 17}, {NULL}};
const Segment SGI76243[] = {{&S30333[44], 29}, {NULL}};
const Segment SGI76244[] = {{&S30333[30], 6}, {NULL}};
const Segment SGI76245[] = {{&S30333[31], 3}, {NULL}};
const Segment SGI76246[] = {{&S30333[87], 6}, {NULL}};
const Segment SGI76247[] = {{&S30333[0], 13}, {NULL}};
const Segment SGI76248[] = {{&S30334[44], 14}, {NULL}};
const Segment SGI76249[] = {{&S30334[44], 26}, {NULL}};
const Segment SGI76250[] = {{&S30334[30], 6}, {NULL}};
const Segment SGI76251[] = {{&S30334[87], 4}, {NULL}};
const Segment SGI76252[] = {{&S30334[87], 3}, {NULL}};
const Segment SGI76253[] = {{&S30334[0], 13}, {NULL}};
const Segment SGI76254[] = {{&S30340[44], 14}, {NULL}};
const Segment SGI76255[] = {{&S30340[44], 26}, {NULL}};
const Segment SGI76256[] = {{&S30340[30], 6}, {NULL}};
const Segment SGI76257[] = {{&S30340[31], 3}, {NULL}};
const Segment SGI76258[] = {{&S30340[84], 6}, {NULL}};
const Segment SGI76259[] = {{&S30340[0], 13}, {NULL}};
const Segment SGI76260[] = {{&S30341[44], 14}, {NULL}};
const Segment SGI76261[] = {{&S30341[44], 26}, {NULL}};
const Segment SGI76262[] = {{&S30341[30], 6}, {NULL}};
const Segment SGI76263[] = {{&S30341[31], 3}, {NULL}};
const Segment SGI76264[] = {{&S30341[84], 6}, {NULL}};
const Segment SGI76265[] = {{&S30341[0], 13}, {NULL}};
const Segment SGI76266[] = {{&S36484[42], 15}, {NULL}};
const Segment SGI76267[] = {{&S36484[42], 31}, {NULL}};
const Segment SGI76268[] = {{&S36484[30], 4}, {NULL}};
const Segment SGI76269[] = {{&S36484[122], 3}, {NULL}};
const Segment SGI76270[] = {{&S36484[87], 6}, {NULL}};
const Segment SGI76271[] = {{&S36485[42], 15}, {NULL}};
const Segment SGI76272[] = {{&S36485[42], 32}, {NULL}};
const Segment SGI76273[] = {{&S36485[30], 4}, {NULL}};
const Segment SGI76274[] = {{&S36485[123], 3}, {NULL}};
const Segment SGI76275[] = {{&S36485[88], 6}, {NULL}};
const Segment SGI76276[] = {{&S36486[38], 15}, {NULL}};
const Segment SGI76277[] = {{&S36486[31], 6}, {NULL}};
const Segment SGI76278[] = {{&S36487[42], 15}, {NULL}};
const Segment SGI76279[] = {{&S36487[42], 31}, {NULL}};
const Segment SGI76280[] = {{&S36487[30], 4}, {NULL}};
const Segment SGI76281[] = {{&S36487[122], 3}, {NULL}};
const Segment SGI76282[] = {{&S36487[87], 6}, {NULL}};
const Segment SGI76283[] = {{&S36488[36], 15}, {NULL}};
const Segment SGI76284[] = {{&S36488[31], 4}, {NULL}};
const Segment SGI76285[] = {{&S36489[44], 15}, {NULL}};
const Segment SGI76286[] = {{&S36489[44], 31}, {NULL}};
const Segment SGI76287[] = {{&S36489[30], 6}, {NULL}};
const Segment SGI76288[] = {{&S36489[124], 3}, {NULL}};
const Segment SGI76289[] = {{&S36489[89], 6}, {NULL}};
const Segment SGI76290[] = {{&S36490[44], 15}, {NULL}};
const Segment SGI76291[] = {{&S36490[44], 30}, {NULL}};
const Segment SGI76292[] = {{&S36490[30], 6}, {NULL}};
const Segment SGI76293[] = {{&S36490[123], 3}, {NULL}};
const Segment SGI76294[] = {{&S36490[88], 6}, {NULL}};
const Segment SGI76295[] = {{&S36953[44], 9}, {NULL}};
const Segment SGI76296[] = {{&S36953[44], 20}, {NULL}};
const Segment SGI76297[] = {{&S36953[30], 6}, {NULL}};
const Segment SGI76298[] = {{&S36491[44], 15}, {NULL}};
const Segment SGI76299[] = {{&S36491[44], 30}, {NULL}};
const Segment SGI76300[] = {{&S36491[30], 6}, {NULL}};
const Segment SGI76301[] = {{&S36953[112], 3}, {NULL}};
const Segment SGI76302[] = {{&S36953[78], 5}, {NULL}};
const Segment SGI76303[] = {{&S36491[123], 3}, {NULL}};
const Segment SGI76304[] = {{&S36491[88], 6}, {NULL}};
const Segment SGI76305[] = {{&S36954[44], 11}, {NULL}};
const Segment SGI76306[] = {{&S36954[44], 22}, {NULL}};
const Segment SGI76307[] = {{&S36492[42], 15}, {NULL}};
const Segment SGI76308[] = {{&S36954[30], 6}, {NULL}};
const Segment SGI76309[] = {{&S36492[42], 31}, {NULL}};
const Segment SGI76310[] = {{&S36492[30], 4}, {NULL}};
const Segment SGI76311[] = {{&S36954[114], 3}, {NULL}};
const Segment SGI76312[] = {{&S36492[122], 3}, {NULL}};
const Segment SGI76313[] = {{&S36954[80], 5}, {NULL}};
const Segment SGI76314[] = {{&S36492[87], 6}, {NULL}};
const Segment SGI76315[] = {{&S36955[44], 14}, {NULL}};
const Segment SGI76316[] = {{&S36955[44], 25}, {NULL}};
const Segment SGI76317[] = {{&S36955[30], 6}, {NULL}};
const Segment SGI76318[] = {{&S36493[42], 15}, {NULL}};
const Segment SGI76319[] = {{&S36493[42], 32}, {NULL}};
const Segment SGI76320[] = {{&S36493[30], 4}, {NULL}};
const Segment SGI76321[] = {{&S36955[117], 3}, {NULL}};
const Segment SGI76322[] = {{&S36493[123], 3}, {NULL}};
const Segment SGI76323[] = {{&S36955[83], 5}, {NULL}};
const Segment SGI76324[] = {{&S36493[88], 6}, {NULL}};
const Segment SGI76325[] = {{&S42921[44], 10}, {NULL}};
const Segment SGI76326[] = {{&S42921[44], 22}, {NULL}};
const Segment SGI76327[] = {{&S42921[30], 6}, {NULL}};
const Segment SGI76328[] = {{&S42921[114], 3}, {NULL}};
const Segment SGI76329[] = {{&S42921[80], 5}, {NULL}};
const Segment SGI76330[] = {{&S36956[44], 16}, {NULL}};
const Segment SGI76331[] = {{&S36494[44], 15}, {NULL}};
const Segment SGI76332[] = {{&S36956[44], 26}, {NULL}};
const Segment SGI76333[] = {{&S36494[44], 31}, {NULL}};
const Segment SGI76334[] = {{&S36956[30], 6}, {NULL}};
const Segment SGI76335[] = {{&S36494[30], 6}, {NULL}};
const Segment SGI76336[] = {{&S36956[118], 3}, {NULL}};
const Segment SGI76337[] = {{&S36494[124], 3}, {NULL}};
const Segment SGI76338[] = {{&S36956[84], 5}, {NULL}};
const Segment SGI76339[] = {{&S36494[89], 6}, {NULL}};
const Segment SGI76340[] = {{&S42922[38], 10}, {NULL}};
const Segment SGI76341[] = {{&S42922[31], 6}, {NULL}};
const Segment SGI76342[] = {{&S36957[38], 9}, {NULL}};
const Segment SGI76343[] = {{&S36957[31], 6}, {NULL}};
const Segment SGI76344[] = {{&S36495[42], 15}, {NULL}};
const Segment SGI76345[] = {{&S36495[42], 30}, {NULL}};
const Segment SGI76346[] = {{&S36495[30], 4}, {NULL}};
const Segment SGI76347[] = {{&S36495[121], 3}, {NULL}};
const Segment SGI76348[] = {{&S36495[86], 6}, {NULL}};
const Segment SGI76349[] = {{&S36958[44], 10}, {NULL}};
const Segment SGI76350[] = {{&S36958[44], 21}, {NULL}};
const Segment SGI76351[] = {{&S36958[30], 6}, {NULL}};
const Segment SGI76352[] = {{&S36958[113], 3}, {NULL}};
const Segment SGI76353[] = {{&S36496[44], 15}, {NULL}};
const Segment SGI76354[] = {{&S36496[44], 30}, {NULL}};
const Segment SGI76355[] = {{&S36958[79], 5}, {NULL}};
const Segment SGI76356[] = {{&S36496[30], 6}, {NULL}};
const Segment SGI76357[] = {{&S36496[123], 3}, {NULL}};
const Segment SGI76358[] = {{&S36496[88], 6}, {NULL}};
const Segment SGI76359[] = {{&S36959[44], 10}, {NULL}};
const Segment SGI76360[] = {{&S36959[44], 21}, {NULL}};
const Segment SGI76361[] = {{&S36959[30], 6}, {NULL}};
const Segment SGI76362[] = {{&S36497[44], 15}, {NULL}};
const Segment SGI76363[] = {{&S36959[113], 3}, {NULL}};
const Segment SGI76364[] = {{&S36497[44], 31}, {NULL}};
const Segment SGI76365[] = {{&S36497[30], 6}, {NULL}};
const Segment SGI76366[] = {{&S36959[79], 5}, {NULL}};
const Segment SGI76367[] = {{&S36497[124], 3}, {NULL}};
const Segment SGI76368[] = {{&S36497[89], 6}, {NULL}};
const Segment SGI76369[] = {{&S36960[44], 16}, {NULL}};
const Segment SGI76370[] = {{&S36960[44], 26}, {NULL}};
const Segment SGI76371[] = {{&S36960[30], 6}, {NULL}};
const Segment SGI76372[] = {{&S36960[118], 3}, {NULL}};
const Segment SGI76373[] = {{&S36960[84], 5}, {NULL}};
const Segment SGI76374[] = {{&S36961[44], 14}, {NULL}};
const Segment SGI76375[] = {{&S36961[44], 25}, {NULL}};
const Segment SGI76376[] = {{&S36961[30], 6}, {NULL}};
const Segment SGI76377[] = {{&S36961[117], 3}, {NULL}};
const Segment SGI76378[] = {{&S36961[83], 5}, {NULL}};
const Segment SGI76379[] = {{&S36963[38], 10}, {NULL}};
const Segment SGI76380[] = {{&S36963[31], 6}, {NULL}};
const Segment SGI76381[] = {{&S36964[38], 11}, {NULL}};
const Segment SGI76382[] = {{&S36964[31], 6}, {NULL}};
const Segment SGI76383[] = {{&S36965[41], 9}, {NULL}};
const Segment SGI76384[] = {{&S36965[30], 6}, {NULL}};
const Segment SGI76385[] = {{&S36965[109], 3}, {NULL}};
const Segment SGI76386[] = {{&S36965[75], 5}, {NULL}};
const Segment SGI76387[] = {{&S36966[44], 14}, {NULL}};
const Segment SGI76388[] = {{&S36966[44], 25}, {NULL}};
const Segment SGI76389[] = {{&S36966[30], 6}, {NULL}};
const Segment SGI76390[] = {{&S36966[117], 3}, {NULL}};
const Segment SGI76391[] = {{&S36966[83], 5}, {NULL}};
const Segment SGI76392[] = {{&S36967[44], 11}, {NULL}};
const Segment SGI76393[] = {{&S36967[44], 22}, {NULL}};
const Segment SGI76394[] = {{&S36967[30], 6}, {NULL}};
const Segment SGI76395[] = {{&S36967[114], 3}, {NULL}};
const Segment SGI76396[] = {{&S36967[80], 5}, {NULL}};
const Segment SGI76397[] = {{&S36968[44], 9}, {NULL}};
const Segment SGI76398[] = {{&S36968[44], 20}, {NULL}};
const Segment SGI76399[] = {{&S36968[30], 6}, {NULL}};
const Segment SGI76400[] = {{&S36968[81], 6}, {NULL}};
const Segment SGI76401[] = {{&S36969[44], 14}, {NULL}};
const Segment SGI76402[] = {{&S36969[44], 25}, {NULL}};
const Segment SGI76403[] = {{&S36969[30], 6}, {NULL}};
const Segment SGI76404[] = {{&S36969[117], 3}, {NULL}};
const Segment SGI76405[] = {{&S36969[83], 5}, {NULL}};
const Segment SGI76406[] = {{&S36970[44], 14}, {NULL}};
const Segment SGI76407[] = {{&S36970[44], 25}, {NULL}};
const Segment SGI76408[] = {{&S36970[30], 6}, {NULL}};
const Segment SGI76409[] = {{&S36970[117], 3}, {NULL}};
const Segment SGI76410[] = {{&S36970[83], 5}, {NULL}};
const Segment SGI76411[] = {{&S36990[44], 7}, {NULL}};
const Segment SGI76412[] = {{&S36990[44], 19}, {NULL}};
const Segment SGI76413[] = {{&S36990[30], 6}, {NULL}};
const Segment SGI76414[] = {{&S36990[111], 3}, {NULL}};
const Segment SGI76415[] = {{&S36990[77], 5}, {NULL}};
const Segment SGI76416[] = {{&S36991[44], 7}, {NULL}};
const Segment SGI76417[] = {{&S36991[44], 19}, {NULL}};
const Segment SGI76418[] = {{&S36991[30], 6}, {NULL}};
const Segment SGI76419[] = {{&S36991[111], 3}, {NULL}};
const Segment SGI76420[] = {{&S36991[77], 5}, {NULL}};
const Segment SGI76421[] = {{&S36992[44], 7}, {NULL}};
const Segment SGI76422[] = {{&S36992[44], 19}, {NULL}};
const Segment SGI76423[] = {{&S36992[30], 6}, {NULL}};
const Segment SGI76424[] = {{&S36992[111], 3}, {NULL}};
const Segment SGI76425[] = {{&S36992[77], 5}, {NULL}};
const Segment SGI76426[] = {{&S36993[44], 7}, {NULL}};
const Segment SGI76427[] = {{&S36993[44], 19}, {NULL}};
const Segment SGI76428[] = {{&S36993[30], 6}, {NULL}};
const Segment SGI76429[] = {{&S36993[111], 3}, {NULL}};
const Segment SGI76430[] = {{&S36993[77], 5}, {NULL}};
const Segment SGI76431[] = {{&S36994[44], 7}, {NULL}};
const Segment SGI76432[] = {{&S36994[44], 19}, {NULL}};
const Segment SGI76433[] = {{&S36994[30], 6}, {NULL}};
const Segment SGI76434[] = {{&S36994[111], 3}, {NULL}};
const Segment SGI76435[] = {{&S36994[77], 5}, {NULL}};
const Segment SGI76436[] = {{&S36995[38], 7}, {NULL}};
const Segment SGI76437[] = {{&S36995[31], 6}, {NULL}};
const Segment SGI76438[] = {{&S36996[44], 7}, {NULL}};
const Segment SGI76439[] = {{&S36996[44], 19}, {NULL}};
const Segment SGI76440[] = {{&S36996[30], 6}, {NULL}};
const Segment SGI76441[] = {{&S36996[111], 3}, {NULL}};
const Segment SGI76442[] = {{&S36996[77], 5}, {NULL}};
const Segment SGI76443[] = {{&S36997[38], 7}, {NULL}};
const Segment SGI76444[] = {{&S36997[31], 6}, {NULL}};
const Segment SGI76445[] = {{&S43129[44], 23}, {NULL}};
const Segment SGI76446[] = {{&S43129[44], 35}, {NULL}};
const Segment SGI76447[] = {{&S43129[30], 6}, {NULL}};
const Segment SGI76448[] = {{&S43129[127], 3}, {NULL}};
const Segment SGI76449[] = {{&S43129[93], 5}, {NULL}};
const Segment SGI76450[] = {{&S43130[44], 23}, {NULL}};
const Segment SGI76451[] = {{&S43130[44], 35}, {NULL}};
const Segment SGI76452[] = {{&S43130[30], 6}, {NULL}};
const Segment SGI76453[] = {{&S43130[127], 3}, {NULL}};
const Segment SGI76454[] = {{&S43130[93], 5}, {NULL}};
const Segment SGI76455[] = {{&S43131[44], 24}, {NULL}};
const Segment SGI76456[] = {{&S43131[44], 36}, {NULL}};
const Segment SGI76457[] = {{&S43131[30], 6}, {NULL}};
const Segment SGI76458[] = {{&S43131[128], 3}, {NULL}};
const Segment SGI76459[] = {{&S43131[94], 5}, {NULL}};
const Segment SGI76460[] = {{&S43132[44], 24}, {NULL}};
const Segment SGI76461[] = {{&S43132[44], 36}, {NULL}};
const Segment SGI76462[] = {{&S43132[30], 6}, {NULL}};
const Segment SGI76463[] = {{&S43132[128], 3}, {NULL}};
const Segment SGI76464[] = {{&S43132[94], 5}, {NULL}};
const Segment SGI76465[] = {{&S43162[44], 7}, {NULL}};
const Segment SGI76466[] = {{&S43162[44], 24}, {NULL}};
const Segment SGI76467[] = {{&S43162[30], 6}, {NULL}};
const Segment SGI76468[] = {{&S43162[116], 3}, {NULL}};
const Segment SGI76469[] = {{&S43162[82], 5}, {NULL}};
const Segment SGI76470[] = {{&S43163[38], 7}, {NULL}};
const Segment SGI76471[] = {{&S43163[31], 6}, {NULL}};
const Segment SGI76472[] = {{&S43164[44], 6}, {NULL}};
const Segment SGI76473[] = {{&S43164[44], 23}, {NULL}};
const Segment SGI76474[] = {{&S43164[30], 6}, {NULL}};
const Segment SGI76475[] = {{&S43164[115], 3}, {NULL}};
const Segment SGI76476[] = {{&S43164[81], 5}, {NULL}};
const Segment SGI76477[] = {{&S43165[44], 7}, {NULL}};
const Segment SGI76478[] = {{&S43165[44], 24}, {NULL}};
const Segment SGI76479[] = {{&S43165[30], 6}, {NULL}};
const Segment SGI76480[] = {{&S43165[85], 4}, {NULL}};
const Segment SGI76481[] = {{&S43165[85], 3}, {NULL}};
const Segment SGI76482[] = {{&S43166[44], 14}, {NULL}};
const Segment SGI76483[] = {{&S43166[44], 31}, {NULL}};
const Segment SGI76484[] = {{&S43166[30], 6}, {NULL}};
const Segment SGI76485[] = {{&S43166[123], 3}, {NULL}};
const Segment SGI76486[] = {{&S43166[89], 5}, {NULL}};
const Segment SGI76487[] = {{&S43167[44], 10}, {NULL}};
const Segment SGI76488[] = {{&S43167[44], 27}, {NULL}};
const Segment SGI76489[] = {{&S43167[30], 6}, {NULL}};
const Segment SGI76490[] = {{&S43167[119], 3}, {NULL}};
const Segment SGI76491[] = {{&S43167[85], 5}, {NULL}};
const Segment SGI76492[] = {{&S43170[44], 7}, {NULL}};
const Segment SGI76493[] = {{&S43170[44], 24}, {NULL}};
const Segment SGI76494[] = {{&S43170[30], 6}, {NULL}};
const Segment SGI76495[] = {{&S43170[85], 6}, {NULL}};
const Segment SGI76496[] = {{&S25432[44], 10}, {NULL}};
const Segment SGI76497[] = {{&S25432[44], 22}, {NULL}};
const Segment SGI76498[] = {{&S25432[30], 6}, {NULL}};
const Segment SGI76499[] = {{&S25432[31], 3}, {NULL}};
const Segment SGI76500[] = {{&S25432[80], 6}, {NULL}};
const Segment SGI76501[] = {{&S25433[44], 10}, {NULL}};
const Segment SGI76502[] = {{&S25433[44], 22}, {NULL}};
const Segment SGI76503[] = {{&S25433[30], 6}, {NULL}};
const Segment SGI76504[] = {{&S25433[31], 3}, {NULL}};
const Segment SGI76505[] = {{&S25433[80], 6}, {NULL}};
const Segment SGI76506[] = {{&S25440[38], 10}, {NULL}};
const Segment SGI76507[] = {{&S25440[31], 6}, {NULL}};
const Segment SGI76508[] = {{&S25467[44], 10}, {NULL}};
const Segment SGI76509[] = {{&S25467[44], 22}, {NULL}};
const Segment SGI76510[] = {{&S25467[30], 6}, {NULL}};
const Segment SGI76511[] = {{&S25467[31], 3}, {NULL}};
const Segment SGI76512[] = {{&S25467[80], 6}, {NULL}};
const Segment SGI76513[] = {{&S43265[44], 10}, {NULL}};
const Segment SGI76514[] = {{&S43265[44], 22}, {NULL}};
const Segment SGI76515[] = {{&S43265[30], 6}, {NULL}};
const Segment SGI76516[] = {{&S43265[114], 3}, {NULL}};
const Segment SGI76517[] = {{&S43265[80], 5}, {NULL}};
const Segment SGI76518[] = {{&S43266[38], 10}, {NULL}};
const Segment SGI76519[] = {{&S43266[31], 6}, {NULL}};
const Segment SGI76520[] = {{&S37319[44], 11}, {NULL}};
const Segment SGI76521[] = {{&S37319[44], 28}, {NULL}};
const Segment SGI76522[] = {{&S37319[30], 6}, {NULL}};
const Segment SGI76523[] = {{&S37319[120], 3}, {NULL}};
const Segment SGI76524[] = {{&S37319[86], 5}, {NULL}};
const Segment SGI76525[] = {{&S37357[38], 7}, {NULL}};
const Segment SGI76526[] = {{&S37357[31], 6}, {NULL}};
const Segment SGI76527[] = {{&S37367[44], 7}, {NULL}};
const Segment SGI76528[] = {{&S37367[44], 18}, {NULL}};
const Segment SGI76529[] = {{&S37367[30], 6}, {NULL}};
const Segment SGI76530[] = {{&S37367[132], 3}, {NULL}};
const Segment SGI76531[] = {{&S37367[98], 5}, {NULL}};
const Segment SGI76532[] = {{&S37488[44], 13}, {NULL}};
const Segment SGI76533[] = {{&S37488[44], 24}, {NULL}};
const Segment SGI76534[] = {{&S37488[30], 6}, {NULL}};
const Segment SGI76535[] = {{&S37488[116], 3}, {NULL}};
const Segment SGI76536[] = {{&S37488[82], 5}, {NULL}};
const Segment SGI76537[] = {{&S37489[44], 10}, {NULL}};
const Segment SGI76538[] = {{&S37489[44], 25}, {NULL}};
const Segment SGI76539[] = {{&S37489[30], 6}, {NULL}};
const Segment SGI76540[] = {{&S37489[117], 3}, {NULL}};
const Segment SGI76541[] = {{&S37489[83], 5}, {NULL}};
const Segment SGI76542[] = {{&S37490[38], 10}, {NULL}};
const Segment SGI76543[] = {{&S37490[31], 6}, {NULL}};
const Segment SGI76544[] = {{&S37491[44], 10}, {NULL}};
const Segment SGI76545[] = {{&S37491[44], 25}, {NULL}};
const Segment SGI76546[] = {{&S37491[30], 6}, {NULL}};
const Segment SGI76547[] = {{&S37491[117], 3}, {NULL}};
const Segment SGI76548[] = {{&S37491[83], 5}, {NULL}};
const Segment SGI76549[] = {{&S37492[44], 10}, {NULL}};
const Segment SGI76550[] = {{&S37492[44], 30}, {NULL}};
const Segment SGI76551[] = {{&S37492[30], 6}, {NULL}};
const Segment SGI76552[] = {{&S37492[122], 3}, {NULL}};
const Segment SGI76553[] = {{&S37492[88], 5}, {NULL}};
const Segment SGI76554[] = {{&S37493[44], 10}, {NULL}};
const Segment SGI76555[] = {{&S37493[44], 25}, {NULL}};
const Segment SGI76556[] = {{&S37493[30], 6}, {NULL}};
const Segment SGI76557[] = {{&S37493[117], 3}, {NULL}};
const Segment SGI76558[] = {{&S37493[83], 5}, {NULL}};
const Segment SGI76559[] = {{&S37494[42], 10}, {NULL}};
const Segment SGI76560[] = {{&S37494[30], 6}, {NULL}};
const Segment SGI76561[] = {{&S37494[115], 3}, {NULL}};
const Segment SGI76562[] = {{&S37494[81], 5}, {NULL}};
const Segment SGI76563[] = {{&S37495[44], 10}, {NULL}};
const Segment SGI76564[] = {{&S37495[44], 28}, {NULL}};
const Segment SGI76565[] = {{&S37495[30], 6}, {NULL}};
const Segment SGI76566[] = {{&S37495[120], 3}, {NULL}};
const Segment SGI76567[] = {{&S37495[86], 5}, {NULL}};
const Segment SGI76568[] = {{&S37496[44], 10}, {NULL}};
const Segment SGI76569[] = {{&S37496[44], 34}, {NULL}};
const Segment SGI76570[] = {{&S37496[30], 6}, {NULL}};
const Segment SGI76571[] = {{&S37496[127], 3}, {NULL}};
const Segment SGI76572[] = {{&S37496[93], 5}, {NULL}};
const Segment SGI76573[] = {{&S37497[44], 10}, {NULL}};
const Segment SGI76574[] = {{&S37497[44], 28}, {NULL}};
const Segment SGI76575[] = {{&S37497[30], 6}, {NULL}};
const Segment SGI76576[] = {{&S37497[120], 3}, {NULL}};
const Segment SGI76577[] = {{&S37497[86], 5}, {NULL}};
const Segment SGI76578[] = {{&S36927[42], 7}, {NULL}};
const Segment SGI76579[] = {{&S36927[42], 18}, {NULL}};
const Segment SGI76580[] = {{&S36927[30], 4}, {NULL}};
const Segment SGI76581[] = {{&S36927[109], 3}, {NULL}};
const Segment SGI76582[] = {{&S36927[74], 6}, {NULL}};
const Segment SGI76583[] = {{&S36928[36], 7}, {NULL}};
const Segment SGI76584[] = {{&S36928[31], 4}, {NULL}};
const Segment SGI76585[] = {{&S36929[42], 7}, {NULL}};
const Segment SGI76586[] = {{&S36929[42], 28}, {NULL}};
const Segment SGI76587[] = {{&S36929[30], 4}, {NULL}};
const Segment SGI76588[] = {{&S36930[42], 7}, {NULL}};
const Segment SGI76589[] = {{&S36930[42], 28}, {NULL}};
const Segment SGI76590[] = {{&S36930[30], 4}, {NULL}};
const Segment SGI76591[] = {{&S43604[44], 11}, {NULL}};
const Segment SGI76592[] = {{&S43604[44], 23}, {NULL}};
const Segment SGI76593[] = {{&S43604[30], 6}, {NULL}};
const Segment SGI76594[] = {{&S43604[115], 3}, {NULL}};
const Segment SGI76595[] = {{&S43604[81], 5}, {NULL}};
const Segment SGI76596[] = {{&S43606[44], 11}, {NULL}};
const Segment SGI76597[] = {{&S43606[44], 23}, {NULL}};
const Segment SGI76598[] = {{&S43606[30], 6}, {NULL}};
const Segment SGI76599[] = {{&S43606[115], 3}, {NULL}};
const Segment SGI76600[] = {{&S43606[81], 5}, {NULL}};
const Segment SGI76601[] = {{&S43608[38], 11}, {NULL}};
const Segment SGI76602[] = {{&S43608[31], 6}, {NULL}};
const Segment SGI76603[] = {{&S37020[44], 7}, {NULL}};
const Segment SGI76604[] = {{&S37020[44], 29}, {NULL}};
const Segment SGI76605[] = {{&S37020[30], 6}, {NULL}};
const Segment SGI76606[] = {{&S37021[38], 7}, {NULL}};
const Segment SGI76607[] = {{&S37021[31], 6}, {NULL}};
const Segment SGI76608[] = {{&S37022[44], 7}, {NULL}};
const Segment SGI76609[] = {{&S37022[44], 19}, {NULL}};
const Segment SGI76610[] = {{&S37022[30], 6}, {NULL}};
const Segment SGI76611[] = {{&S37022[112], 3}, {NULL}};
const Segment SGI76612[] = {{&S37022[77], 6}, {NULL}};
const Segment SGI76613[] = {{&S37023[44], 7}, {NULL}};
const Segment SGI76614[] = {{&S37023[44], 19}, {NULL}};
const Segment SGI76615[] = {{&S37023[30], 6}, {NULL}};
const Segment SGI76616[] = {{&S37023[112], 3}, {NULL}};
const Segment SGI76617[] = {{&S37023[77], 6}, {NULL}};
const Segment SGI76618[] = {{&S37024[44], 7}, {NULL}};
const Segment SGI76619[] = {{&S37024[44], 18}, {NULL}};
const Segment SGI76620[] = {{&S37024[30], 6}, {NULL}};
const Segment SGI76621[] = {{&S37024[111], 3}, {NULL}};
const Segment SGI76622[] = {{&S37024[76], 6}, {NULL}};
const Segment SGI76623[] = {{&S37025[44], 7}, {NULL}};
const Segment SGI76624[] = {{&S37025[44], 18}, {NULL}};
const Segment SGI76625[] = {{&S37025[30], 6}, {NULL}};
const Segment SGI76626[] = {{&S37025[111], 3}, {NULL}};
const Segment SGI76627[] = {{&S37025[76], 6}, {NULL}};
const Segment SGI76628[] = {{&S37026[44], 7}, {NULL}};
const Segment SGI76629[] = {{&S37026[44], 18}, {NULL}};
const Segment SGI76630[] = {{&S37026[30], 6}, {NULL}};
const Segment SGI76631[] = {{&S37026[111], 3}, {NULL}};
const Segment SGI76632[] = {{&S37026[76], 6}, {NULL}};
const Segment SGI76633[] = {{&S37027[42], 7}, {NULL}};
const Segment SGI76634[] = {{&S37027[30], 6}, {NULL}};
const Segment SGI76635[] = {{&S37027[110], 3}, {NULL}};
const Segment SGI76636[] = {{&S37027[75], 6}, {NULL}};
const Segment SGI76637[] = {{&S43656[44], 7}, {NULL}};
const Segment SGI76638[] = {{&S43656[44], 24}, {NULL}};
const Segment SGI76639[] = {{&S43656[30], 6}, {NULL}};
const Segment SGI76640[] = {{&S43656[116], 3}, {NULL}};
const Segment SGI76641[] = {{&S43656[82], 5}, {NULL}};
const Segment SGI76642[] = {{&S43657[44], 8}, {NULL}};
const Segment SGI76643[] = {{&S43657[44], 25}, {NULL}};
const Segment SGI76644[] = {{&S43657[30], 6}, {NULL}};
const Segment SGI76645[] = {{&S43657[117], 3}, {NULL}};
const Segment SGI76646[] = {{&S43657[83], 5}, {NULL}};
const Segment SGI76647[] = {{&S43730[44], 11}, {NULL}};
const Segment SGI76648[] = {{&S43730[44], 39}, {NULL}};
const Segment SGI76649[] = {{&S43730[30], 6}, {NULL}};
const Segment SGI76650[] = {{&S43730[131], 3}, {NULL}};
const Segment SGI76651[] = {{&S43730[97], 5}, {NULL}};
const Segment SGI76652[] = {{&S43731[44], 11}, {NULL}};
const Segment SGI76653[] = {{&S43731[44], 39}, {NULL}};
const Segment SGI76654[] = {{&S43731[30], 6}, {NULL}};
const Segment SGI76655[] = {{&S43731[131], 3}, {NULL}};
const Segment SGI76656[] = {{&S43731[97], 5}, {NULL}};
const Segment SGI76657[] = {{&S43732[44], 11}, {NULL}};
const Segment SGI76658[] = {{&S43732[44], 42}, {NULL}};
const Segment SGI76659[] = {{&S43732[30], 6}, {NULL}};
const Segment SGI76660[] = {{&S43732[134], 3}, {NULL}};
const Segment SGI76661[] = {{&S43732[100], 5}, {NULL}};
const Segment SGI76662[] = {{&S43733[38], 8}, {NULL}};
const Segment SGI76663[] = {{&S43733[31], 6}, {NULL}};
const Segment SGI76664[] = {{&S43734[44], 11}, {NULL}};
const Segment SGI76665[] = {{&S43734[44], 39}, {NULL}};
const Segment SGI76666[] = {{&S43734[30], 6}, {NULL}};
const Segment SGI76667[] = {{&S43734[131], 3}, {NULL}};
const Segment SGI76668[] = {{&S43734[97], 5}, {NULL}};
const Segment SGI76669[] = {{&S43735[44], 8}, {NULL}};
const Segment SGI76670[] = {{&S43735[44], 36}, {NULL}};
const Segment SGI76671[] = {{&S43735[30], 6}, {NULL}};
const Segment SGI76672[] = {{&S43735[128], 3}, {NULL}};
const Segment SGI76673[] = {{&S43735[94], 5}, {NULL}};
const Segment SGI76674[] = {{&S43736[44], 8}, {NULL}};
const Segment SGI76675[] = {{&S43736[44], 36}, {NULL}};
const Segment SGI76676[] = {{&S43736[30], 6}, {NULL}};
const Segment SGI76677[] = {{&S43736[128], 3}, {NULL}};
const Segment SGI76678[] = {{&S43736[94], 5}, {NULL}};
const Segment SGI76679[] = {{&S43739[44], 11}, {NULL}};
const Segment SGI76680[] = {{&S43739[44], 39}, {NULL}};
const Segment SGI76681[] = {{&S43739[30], 6}, {NULL}};
const Segment SGI76682[] = {{&S43739[131], 3}, {NULL}};
const Segment SGI76683[] = {{&S43739[97], 5}, {NULL}};
const Segment SGI76684[] = {{&S43740[43], 11}, {NULL}};
const Segment SGI76685[] = {{&S43740[43], 39}, {NULL}};
const Segment SGI76686[] = {{&S43740[30], 5}, {NULL}};
const Segment SGI76687[] = {{&S43740[130], 3}, {NULL}};
const Segment SGI76688[] = {{&S43740[96], 5}, {NULL}};
const Segment SGI76689[] = {{&S43983[44], 9}, {NULL}};
const Segment SGI76690[] = {{&S37397[36], 10}, {NULL}};
const Segment SGI76691[] = {{&S43983[44], 21}, {NULL}};
const Segment SGI76692[] = {{&S37397[31], 4}, {NULL}};
const Segment SGI76693[] = {{&S43983[30], 6}, {NULL}};
const Segment SGI76694[] = {{&S43983[113], 3}, {NULL}};
const Segment SGI76695[] = {{&S43983[79], 5}, {NULL}};
const Segment SGI76696[] = {{&S37398[42], 10}, {NULL}};
const Segment SGI76697[] = {{&S37398[42], 22}, {NULL}};
const Segment SGI76698[] = {{&S37398[30], 4}, {NULL}};
const Segment SGI76699[] = {{&S43984[38], 9}, {NULL}};
const Segment SGI76700[] = {{&S43984[31], 6}, {NULL}};
const Segment SGI76701[] = {{&S37398[113], 3}, {NULL}};
const Segment SGI76702[] = {{&S37398[78], 6}, {NULL}};
const Segment SGI76703[] = {{&S37399[42], 11}, {NULL}};
const Segment SGI76704[] = {{&S37399[42], 23}, {NULL}};
const Segment SGI76705[] = {{&S37399[30], 4}, {NULL}};
const Segment SGI76706[] = {{&S37399[114], 3}, {NULL}};
const Segment SGI76707[] = {{&S37399[79], 6}, {NULL}};
const Segment SGI76708[] = {{&S37400[42], 10}, {NULL}};
const Segment SGI76709[] = {{&S37400[42], 22}, {NULL}};
const Segment SGI76710[] = {{&S37400[30], 4}, {NULL}};
const Segment SGI76711[] = {{&S37400[113], 3}, {NULL}};
const Segment SGI76712[] = {{&S37400[78], 6}, {NULL}};
const Segment SGI76713[] = {{&S37401[42], 11}, {NULL}};
const Segment SGI76714[] = {{&S37401[42], 23}, {NULL}};
const Segment SGI76715[] = {{&S37401[30], 4}, {NULL}};
const Segment SGI76716[] = {{&S37401[114], 3}, {NULL}};
const Segment SGI76717[] = {{&S37401[79], 6}, {NULL}};
const Segment SGI76718[] = {{&S37402[42], 10}, {NULL}};
const Segment SGI76719[] = {{&S37402[42], 22}, {NULL}};
const Segment SGI76720[] = {{&S37402[30], 4}, {NULL}};
const Segment SGI76721[] = {{&S37402[113], 3}, {NULL}};
const Segment SGI76722[] = {{&S37402[78], 6}, {NULL}};
const Segment SGI76723[] = {{&S37403[42], 10}, {NULL}};
const Segment SGI76724[] = {{&S37403[42], 22}, {NULL}};
const Segment SGI76725[] = {{&S37403[30], 4}, {NULL}};
const Segment SGI76726[] = {{&S37403[113], 3}, {NULL}};
const Segment SGI76727[] = {{&S37403[78], 6}, {NULL}};
const Segment SGI76728[] = {{&S37404[42], 10}, {NULL}};
const Segment SGI76729[] = {{&S37404[42], 22}, {NULL}};
const Segment SGI76730[] = {{&S37404[30], 4}, {NULL}};
const Segment SGI76731[] = {{&S37404[114], 3}, {NULL}};
const Segment SGI76732[] = {{&S37404[78], 7}, {NULL}};
const Segment SGI76733[] = {{&S37404[134], 8}, {NULL}};
const Segment SGI76734[] = {{&S37405[40], 10}, {NULL}};
const Segment SGI76735[] = {{&S37405[30], 4}, {NULL}};
const Segment SGI76736[] = {{&S37405[111], 3}, {NULL}};
const Segment SGI76737[] = {{&S37405[76], 6}, {NULL}};
const Segment SGI76738[] = {{&S37406[43], 10}, {NULL}};
const Segment SGI76739[] = {{&S37406[43], 22}, {NULL}};
const Segment SGI76740[] = {{&S37406[31], 4}, {NULL}};
const Segment SGI76741[] = {{&S37407[42], 10}, {NULL}};
const Segment SGI76742[] = {{&S37407[42], 21}, {NULL}};
const Segment SGI76743[] = {{&S37407[30], 4}, {NULL}};
const Segment SGI76744[] = {{&S37407[112], 3}, {NULL}};
const Segment SGI76745[] = {{&S37407[77], 6}, {NULL}};
const Segment SGI76746[] = {{&S37408[42], 10}, {NULL}};
const Segment SGI76747[] = {{&S37408[42], 28}, {NULL}};
const Segment SGI76748[] = {{&S37408[30], 4}, {NULL}};
const Segment SGI76749[] = {{&S37409[42], 10}, {NULL}};
const Segment SGI76750[] = {{&S37409[42], 28}, {NULL}};
const Segment SGI76751[] = {{&S37409[30], 4}, {NULL}};
const Segment SGI76752[] = {{&S37410[44], 10}, {NULL}};
const Segment SGI76753[] = {{&S37410[44], 42}, {NULL}};
const Segment SGI76754[] = {{&S37410[32], 4}, {NULL}};
const Segment SGI76755[] = {{&S37411[42], 10}, {NULL}};
const Segment SGI76756[] = {{&S37411[42], 22}, {&S37411[206], 7}, {NULL}};
const Segment SGI76757[] = {{&S37411[30], 4}, {NULL}};
const Segment SGI76758[] = {{&S37411[113], 3}, {NULL}};
const Segment SGI76759[] = {{&S37411[78], 6}, {NULL}};
const Segment SGI76760[] = {{&S26245[44], 7}, {NULL}};
const Segment SGI76761[] = {{&S26245[44], 32}, {NULL}};
const Segment SGI76762[] = {{&S26245[30], 6}, {NULL}};
const Segment SGI76763[] = {{&S26245[31], 3}, {NULL}};
const Segment SGI76764[] = {{&S26245[90], 6}, {NULL}};
const Segment SGI76765[] = {{&S37412[42], 10}, {NULL}};
const Segment SGI76766[] = {{&S37412[42], 19}, {NULL}};
const Segment SGI76767[] = {{&S37412[30], 4}, {NULL}};
const Segment SGI76768[] = {{&S37412[109], 3}, {NULL}};
const Segment SGI76769[] = {{&S37412[75], 5}, {NULL}};
const Segment SGI76770[] = {{&S37413[42], 10}, {NULL}};
const Segment SGI76771[] = {{&S37413[42], 22}, {NULL}};
const Segment SGI76772[] = {{&S37413[30], 4}, {NULL}};
const Segment SGI76773[] = {{&S37413[81], 6}, {NULL}};
const Segment SGI76774[] = {{&S38212[44], 11}, {NULL}};
const Segment SGI76775[] = {{&S38212[44], 28}, {NULL}};
const Segment SGI76776[] = {{&S38212[30], 6}, {NULL}};
const Segment SGI76777[] = {{&S38212[120], 3}, {NULL}};
const Segment SGI76778[] = {{&S38212[86], 5}, {NULL}};
const Segment SGI76779[] = {{&S26297[44], 7}, {NULL}};
const Segment SGI76780[] = {{&S26297[44], 19}, {NULL}};
const Segment SGI76781[] = {{&S26297[30], 6}, {NULL}};
const Segment SGI76782[] = {{&S26297[31], 3}, {NULL}};
const Segment SGI76783[] = {{&S26297[77], 6}, {NULL}};
const Segment SGI76784[] = {{&S37552[42], 6}, {NULL}};
const Segment SGI76785[] = {{&S37552[42], 18}, {NULL}};
const Segment SGI76786[] = {{&S37552[30], 4}, {NULL}};
const Segment SGI76787[] = {{&S37552[109], 3}, {NULL}};
const Segment SGI76788[] = {{&S37552[74], 6}, {NULL}};
const Segment SGI76789[] = {{&S37553[44], 6}, {NULL}};
const Segment SGI76790[] = {{&S37553[44], 18}, {NULL}};
const Segment SGI76791[] = {{&S37553[30], 6}, {NULL}};
const Segment SGI76792[] = {{&S37553[111], 3}, {NULL}};
const Segment SGI76793[] = {{&S37553[76], 6}, {NULL}};
const Segment SGI76794[] = {{&S37554[36], 6}, {NULL}};
const Segment SGI76795[] = {{&S37554[31], 4}, {NULL}};
const Segment SGI76796[] = {{&S37555[38], 6}, {NULL}};
const Segment SGI76797[] = {{&S37555[31], 6}, {NULL}};
const Segment SGI76798[] = {{&S38395[44], 10}, {NULL}};
const Segment SGI76799[] = {{&S38395[44], 27}, {NULL}};
const Segment SGI76800[] = {{&S38395[30], 6}, {NULL}};
const Segment SGI76801[] = {{&S38395[119], 3}, {NULL}};
const Segment SGI76802[] = {{&S38395[85], 5}, {NULL}};
const Segment SGI76803[] = {{&S38396[38], 10}, {NULL}};
const Segment SGI76804[] = {{&S38396[31], 6}, {NULL}};
const Segment SGI76805[] = {{&S38397[44], 10}, {NULL}};
const Segment SGI76806[] = {{&S38397[44], 27}, {NULL}};
const Segment SGI76807[] = {{&S38397[30], 6}, {NULL}};
const Segment SGI76808[] = {{&S38397[119], 3}, {NULL}};
const Segment SGI76809[] = {{&S38397[85], 5}, {NULL}};
const Segment SGI76810[] = {{&S38398[44], 9}, {NULL}};
const Segment SGI76811[] = {{&S38398[44], 26}, {NULL}};
const Segment SGI76812[] = {{&S38398[30], 6}, {NULL}};
const Segment SGI76813[] = {{&S38398[118], 3}, {NULL}};
const Segment SGI76814[] = {{&S38398[84], 5}, {NULL}};
const Segment SGI76815[] = {{&S38399[43], 10}, {NULL}};
const Segment SGI76816[] = {{&S38399[43], 27}, {NULL}};
const Segment SGI76817[] = {{&S38399[30], 5}, {NULL}};
const Segment SGI76818[] = {{&S38399[118], 3}, {NULL}};
const Segment SGI76819[] = {{&S38399[84], 5}, {NULL}};
const Segment SGI76820[] = {{&S38400[44], 9}, {NULL}};
const Segment SGI76821[] = {{&S38400[44], 26}, {NULL}};
const Segment SGI76822[] = {{&S38400[30], 6}, {NULL}};
const Segment SGI76823[] = {{&S38400[118], 3}, {NULL}};
const Segment SGI76824[] = {{&S38400[84], 5}, {NULL}};
const Segment SGI76825[] = {{&S38401[43], 10}, {NULL}};
const Segment SGI76826[] = {{&S38401[43], 27}, {NULL}};
const Segment SGI76827[] = {{&S38401[118], 3}, {NULL}};
const Segment SGI76828[] = {{&S38401[84], 5}, {NULL}};
const Segment SGI76829[] = {{&S38402[44], 9}, {NULL}};
const Segment SGI76830[] = {{&S38402[44], 26}, {NULL}};
const Segment SGI76831[] = {{&S38402[30], 6}, {NULL}};
const Segment SGI76832[] = {{&S38402[118], 3}, {NULL}};
const Segment SGI76833[] = {{&S38402[84], 5}, {NULL}};
const Segment SGI76834[] = {{&S31770[44], 10}, {NULL}};
const Segment SGI76835[] = {{&S31770[44], 21}, {NULL}};
const Segment SGI76836[] = {{&S31770[30], 6}, {NULL}};
const Segment SGI76837[] = {{&S31770[31], 3}, {NULL}};
const Segment SGI76838[] = {{&S31770[79], 5}, {NULL}};
const Segment SGI76839[] = {{&S31770[0], 13}, {NULL}};
const Segment SGI76840[] = {{&S38446[38], 13}, {NULL}};
const Segment SGI76841[] = {{&S38446[31], 6}, {NULL}};
const Segment SGI76842[] = {{&S38447[44], 13}, {NULL}};
const Segment SGI76843[] = {{&S38447[44], 25}, {NULL}};
const Segment SGI76844[] = {{&S38447[30], 6}, {NULL}};
const Segment SGI76845[] = {{&S38447[117], 3}, {NULL}};
const Segment SGI76846[] = {{&S38447[83], 5}, {NULL}};
const Segment SGI76847[] = {{&S44428[44], 10}, {NULL}};
const Segment SGI76848[] = {{&S44428[44], 27}, {NULL}};
const Segment SGI76849[] = {{&S44428[30], 6}, {NULL}};
const Segment SGI76850[] = {{&S44428[149], 4}, {NULL}};
const Segment SGI76851[] = {{&S44428[175], 3}, {NULL}};
const Segment SGI76852[] = {{&S44428[119], 3}, {&S44428[224], 5}, {NULL}};
const Segment SGI76853[] = {{&S44428[85], 5}, {NULL}};
const Segment SGI76854[] = {{&S44429[42], 10}, {NULL}};
const Segment SGI76855[] = {{&S44429[30], 6}, {NULL}};
const Segment SGI76856[] = {{&S44429[147], 4}, {NULL}};
const Segment SGI76857[] = {{&S44429[173], 3}, {NULL}};
const Segment SGI76858[] = {{&S44429[117], 3}, {&S44429[222], 5}, {NULL}};
const Segment SGI76859[] = {{&S44429[83], 5}, {NULL}};
const Segment SGI76860[] = {{&S44500[44], 10}, {NULL}};
const Segment SGI76861[] = {{&S44500[44], 27}, {NULL}};
const Segment SGI76862[] = {{&S44500[30], 6}, {NULL}};
const Segment SGI76863[] = {{&S44500[119], 3}, {NULL}};
const Segment SGI76864[] = {{&S44500[85], 5}, {NULL}};
const Segment SGI76865[] = {{&S44501[44], 11}, {NULL}};
const Segment SGI76866[] = {{&S44501[44], 28}, {NULL}};
const Segment SGI76867[] = {{&S44501[30], 6}, {NULL}};
const Segment SGI76868[] = {{&S44501[120], 3}, {NULL}};
const Segment SGI76869[] = {{&S44501[86], 5}, {NULL}};
const Segment SGI76870[] = {{&S44502[44], 10}, {NULL}};
const Segment SGI76871[] = {{&S44502[44], 27}, {NULL}};
const Segment SGI76872[] = {{&S44502[30], 6}, {NULL}};
const Segment SGI76873[] = {{&S44502[88], 4}, {NULL}};
const Segment SGI76874[] = {{&S44502[88], 3}, {NULL}};
const Segment SGI76875[] = {{&S44503[38], 10}, {NULL}};
const Segment SGI76876[] = {{&S44503[31], 6}, {NULL}};
const Segment SGI76877[] = {{&S44504[42], 10}, {NULL}};
const Segment SGI76878[] = {{&S44504[30], 6}, {NULL}};
const Segment SGI76879[] = {{&S44504[117], 3}, {NULL}};
const Segment SGI76880[] = {{&S44504[83], 5}, {NULL}};
const Segment SGI76881[] = {{&S38592[44], 10}, {NULL}};
const Segment SGI76882[] = {{&S38592[44], 21}, {NULL}};
const Segment SGI76883[] = {{&S38592[30], 6}, {NULL}};
const Segment SGI76884[] = {{&S38592[113], 3}, {NULL}};
const Segment SGI76885[] = {{&S38592[79], 5}, {NULL}};
const Segment SGI76886[] = {{&S38600[38], 10}, {NULL}};
const Segment SGI76887[] = {{&S38600[31], 6}, {NULL}};
const Segment SGI76888[] = {{&S38609[44], 8}, {NULL}};
const Segment SGI76889[] = {{&S38609[44], 25}, {NULL}};
const Segment SGI76890[] = {{&S38609[30], 6}, {NULL}};
const Segment SGI76891[] = {{&S38609[117], 3}, {NULL}};
const Segment SGI76892[] = {{&S38609[83], 5}, {NULL}};
const Segment SGI76893[] = {{&S38610[44], 20}, {NULL}};
const Segment SGI76894[] = {{&S38610[44], 37}, {NULL}};
const Segment SGI76895[] = {{&S38610[30], 6}, {NULL}};
const Segment SGI76896[] = {{&S38610[129], 3}, {NULL}};
const Segment SGI76897[] = {{&S38610[95], 5}, {NULL}};
const Segment SGI76898[] = {{&S38611[42], 20}, {NULL}};
const Segment SGI76899[] = {{&S38611[30], 6}, {NULL}};
const Segment SGI76900[] = {{&S38611[127], 3}, {NULL}};
const Segment SGI76901[] = {{&S38611[93], 5}, {NULL}};
const Segment SGI76902[] = {{&S38612[42], 8}, {NULL}};
const Segment SGI76903[] = {{&S38612[30], 6}, {NULL}};
const Segment SGI76904[] = {{&S38612[115], 3}, {NULL}};
const Segment SGI76905[] = {{&S38612[81], 5}, {NULL}};
const Segment SGI76906[] = {{&S38613[38], 8}, {NULL}};
const Segment SGI76907[] = {{&S38613[31], 6}, {NULL}};
const Segment SGI76908[] = {{&S38614[44], 19}, {NULL}};
const Segment SGI76909[] = {{&S38614[44], 35}, {NULL}};
const Segment SGI76910[] = {{&S38614[30], 6}, {NULL}};
const Segment SGI76911[] = {{&S38614[127], 3}, {NULL}};
const Segment SGI76912[] = {{&S38614[93], 5}, {NULL}};
const Segment SGI76913[] = {{&S38615[44], 19}, {NULL}};
const Segment SGI76914[] = {{&S38615[44], 36}, {NULL}};
const Segment SGI76915[] = {{&S38615[30], 6}, {NULL}};
const Segment SGI76916[] = {{&S38615[128], 3}, {NULL}};
const Segment SGI76917[] = {{&S38615[94], 5}, {NULL}};
const Segment SGI76918[] = {{&S38616[38], 7}, {NULL}};
const Segment SGI76919[] = {{&S38616[31], 6}, {NULL}};
const Segment SGI76920[] = {{&S38617[44], 7}, {NULL}};
const Segment SGI76921[] = {{&S38617[44], 24}, {NULL}};
const Segment SGI76922[] = {{&S38617[30], 6}, {NULL}};
const Segment SGI76923[] = {{&S38617[116], 3}, {NULL}};
const Segment SGI76924[] = {{&S38617[82], 5}, {NULL}};
const Segment SGI76925[] = {{&S38618[44], 7}, {NULL}};
const Segment SGI76926[] = {{&S38618[44], 23}, {NULL}};
const Segment SGI76927[] = {{&S38618[30], 6}, {NULL}};
const Segment SGI76928[] = {{&S38618[115], 3}, {NULL}};
const Segment SGI76929[] = {{&S38618[81], 5}, {NULL}};
const Segment SGI76930[] = {{&S38619[44], 7}, {NULL}};
const Segment SGI76931[] = {{&S38619[44], 24}, {NULL}};
const Segment SGI76932[] = {{&S38619[30], 6}, {NULL}};
const Segment SGI76933[] = {{&S38619[85], 6}, {NULL}};
const Segment SGI76934[] = {{&S38699[44], 7}, {NULL}};
const Segment SGI76935[] = {{&S38699[44], 18}, {NULL}};
const Segment SGI76936[] = {{&S38699[30], 6}, {NULL}};
const Segment SGI76937[] = {{&S38699[110], 3}, {NULL}};
const Segment SGI76938[] = {{&S38699[76], 5}, {NULL}};
const Segment SGI76939[] = {{&S38701[38], 7}, {NULL}};
const Segment SGI76940[] = {{&S38701[31], 6}, {NULL}};
const Segment SGI76941[] = {{&S38750[38], 10}, {NULL}};
const Segment SGI76942[] = {{&S38750[31], 6}, {NULL}};
const Segment SGI76943[] = {{&S44713[44], 11}, {NULL}};
const Segment SGI76944[] = {{&S44713[44], 28}, {NULL}};
const Segment SGI76945[] = {{&S44713[30], 6}, {NULL}};
const Segment SGI76946[] = {{&S44713[120], 3}, {NULL}};
const Segment SGI76947[] = {{&S44713[86], 5}, {NULL}};
const Segment SGI76948[] = {{&S44716[44], 11}, {NULL}};
const Segment SGI76949[] = {{&S44716[44], 28}, {NULL}};
const Segment SGI76950[] = {{&S44716[30], 6}, {NULL}};
const Segment SGI76951[] = {{&S44716[120], 3}, {NULL}};
const Segment SGI76952[] = {{&S44716[86], 5}, {NULL}};
const Segment SGI76953[] = {{&S44717[44], 11}, {NULL}};
const Segment SGI76954[] = {{&S44717[44], 28}, {NULL}};
const Segment SGI76955[] = {{&S44717[30], 6}, {NULL}};
const Segment SGI76956[] = {{&S44717[120], 3}, {NULL}};
const Segment SGI76957[] = {{&S44717[86], 5}, {NULL}};
const Segment SGI76958[] = {{&S44718[38], 11}, {NULL}};
const Segment SGI76959[] = {{&S44718[31], 6}, {NULL}};
const Segment SGI76960[] = {{&S44719[44], 11}, {NULL}};
const Segment SGI76961[] = {{&S44719[44], 28}, {NULL}};
const Segment SGI76962[] = {{&S44719[30], 6}, {NULL}};
const Segment SGI76963[] = {{&S44719[120], 3}, {NULL}};
const Segment SGI76964[] = {{&S44719[86], 5}, {NULL}};
const Segment SGI76965[] = {{&S32413[44], 6}, {NULL}};
const Segment SGI76966[] = {{&S32413[44], 18}, {NULL}};
const Segment SGI76967[] = {{&S32413[30], 6}, {NULL}};
const Segment SGI76968[] = {{&S32413[31], 3}, {NULL}};
const Segment SGI76969[] = {{&S32413[76], 6}, {NULL}};
const Segment SGI76970[] = {{&S32413[0], 13}, {NULL}};
const Segment SGI76971[] = {{&S32414[44], 6}, {NULL}};
const Segment SGI76972[] = {{&S32414[44], 17}, {NULL}};
const Segment SGI76973[] = {{&S32414[30], 6}, {NULL}};
const Segment SGI76974[] = {{&S32414[31], 3}, {NULL}};
const Segment SGI76975[] = {{&S32414[75], 6}, {NULL}};
const Segment SGI76976[] = {{&S32414[0], 13}, {NULL}};
const Segment SGI76977[] = {{&S44821[44], 11}, {NULL}};
const Segment SGI76978[] = {{&S44821[44], 23}, {NULL}};
const Segment SGI76979[] = {{&S44821[30], 6}, {NULL}};
const Segment SGI76980[] = {{&S44821[115], 3}, {NULL}};
const Segment SGI76981[] = {{&S44821[81], 5}, {NULL}};
const Segment SGI76982[] = {{&S38941[44], 12}, {NULL}};
const Segment SGI76983[] = {{&S38941[44], 23}, {NULL}};
const Segment SGI76984[] = {{&S38941[30], 6}, {NULL}};
const Segment SGI76985[] = {{&S38941[115], 3}, {NULL}};
const Segment SGI76986[] = {{&S38941[81], 5}, {NULL}};
const Segment SGI76987[] = {{&S38946[38], 12}, {NULL}};
const Segment SGI76988[] = {{&S38946[31], 6}, {NULL}};
const Segment SGI76989[] = {{&S39168[44], 12}, {NULL}};
const Segment SGI76990[] = {{&S39168[44], 23}, {NULL}};
const Segment SGI76991[] = {{&S39168[30], 6}, {NULL}};
const Segment SGI76992[] = {{&S39168[131], 3}, {NULL}};
const Segment SGI76993[] = {{&S39168[97], 5}, {NULL}};
const Segment SGI76994[] = {{&S39168[78], 3}, {NULL}};
const Segment SGI76995[] = {{&S39169[44], 21}, {NULL}};
const Segment SGI76996[] = {{&S39169[44], 53}, {NULL}};
const Segment SGI76997[] = {{&S39169[30], 6}, {NULL}};
const Segment SGI76998[] = {{&S39169[145], 3}, {NULL}};
const Segment SGI76999[] = {{&S39169[111], 5}, {NULL}};
const Segment SGI77000[] = {{&S39170[44], 12}, {NULL}};
const Segment SGI77001[] = {{&S39170[44], 43}, {NULL}};
const Segment SGI77002[] = {{&S39170[30], 6}, {NULL}};
const Segment SGI77003[] = {{&S39170[135], 3}, {NULL}};
const Segment SGI77004[] = {{&S39170[101], 5}, {NULL}};
const Segment SGI77005[] = {{&S39171[44], 17}, {NULL}};
const Segment SGI77006[] = {{&S39171[44], 28}, {NULL}};
const Segment SGI77007[] = {{&S39171[30], 6}, {NULL}};
const Segment SGI77008[] = {{&S39171[139], 3}, {NULL}};
const Segment SGI77009[] = {{&S39171[105], 5}, {NULL}};
const Segment SGI77010[] = {{&S39171[35], 1}, {NULL}};
const Segment SGI77011[] = {{&S39171[74], 11}, {NULL}};
const Segment SGI77012[] = {{&S39172[44], 12}, {NULL}};
const Segment SGI77013[] = {{&S39172[44], 23}, {NULL}};
const Segment SGI77014[] = {{&S39172[30], 6}, {NULL}};
const Segment SGI77015[] = {{&S39172[132], 3}, {NULL}};
const Segment SGI77016[] = {{&S39172[98], 5}, {NULL}};
const Segment SGI77017[] = {{&S39173[44], 12}, {NULL}};
const Segment SGI77018[] = {{&S39173[44], 23}, {NULL}};
const Segment SGI77019[] = {{&S39173[30], 6}, {NULL}};
const Segment SGI77020[] = {{&S39173[136], 3}, {NULL}};
const Segment SGI77021[] = {{&S39173[102], 5}, {NULL}};
const Segment SGI77022[] = {{&S39173[35], 1}, {NULL}};
const Segment SGI77023[] = {{&S39173[69], 11}, {NULL}};
const Segment SGI77024[] = {{&S39174[44], 21}, {NULL}};
const Segment SGI77025[] = {{&S39174[44], 32}, {NULL}};
const Segment SGI77026[] = {{&S39174[30], 6}, {NULL}};
const Segment SGI77027[] = {{&S39174[124], 3}, {NULL}};
const Segment SGI77028[] = {{&S39174[90], 5}, {NULL}};
const Segment SGI77029[] = {{&S39175[38], 12}, {NULL}};
const Segment SGI77030[] = {{&S39175[31], 6}, {NULL}};
const Segment SGI77031[] = {{&S39190[44], 13}, {NULL}};
const Segment SGI77032[] = {{&S39190[44], 24}, {NULL}};
const Segment SGI77033[] = {{&S39190[30], 6}, {NULL}};
const Segment SGI77034[] = {{&S39190[116], 3}, {NULL}};
const Segment SGI77035[] = {{&S39190[82], 5}, {NULL}};
const Segment SGI77036[] = {{&S39198[38], 13}, {NULL}};
const Segment SGI77037[] = {{&S39198[31], 6}, {NULL}};
const Segment SGI77038[] = {{&S39201[44], 9}, {NULL}};
const Segment SGI77039[] = {{&S39201[44], 20}, {NULL}};
const Segment SGI77040[] = {{&S39201[30], 6}, {NULL}};
const Segment SGI77041[] = {{&S39201[129], 3}, {NULL}};
const Segment SGI77042[] = {{&S39201[95], 5}, {NULL}};
const Segment SGI77043[] = {{&S39203[44], 9}, {NULL}};
const Segment SGI77044[] = {{&S39203[44], 24}, {NULL}};
const Segment SGI77045[] = {{&S39203[30], 6}, {NULL}};
const Segment SGI77046[] = {{&S39203[116], 3}, {NULL}};
const Segment SGI77047[] = {{&S39203[82], 5}, {NULL}};
const Segment SGI77048[] = {{&S39205[44], 13}, {NULL}};
const Segment SGI77049[] = {{&S39205[44], 24}, {NULL}};
const Segment SGI77050[] = {{&S39205[30], 6}, {NULL}};
const Segment SGI77051[] = {{&S39205[129], 3}, {NULL}};
const Segment SGI77052[] = {{&S39205[95], 5}, {NULL}};
const Segment SGI77053[] = {{&S45190[44], 11}, {NULL}};
const Segment SGI77054[] = {{&S45190[44], 28}, {NULL}};
const Segment SGI77055[] = {{&S45190[30], 6}, {NULL}};
const Segment SGI77056[] = {{&S45190[120], 3}, {NULL}};
const Segment SGI77057[] = {{&S45190[86], 5}, {NULL}};
const Segment SGI77058[] = {{&S27620[44], 15}, {NULL}};
const Segment SGI77059[] = {{&S27620[44], 27}, {NULL}};
const Segment SGI77060[] = {{&S27620[30], 6}, {NULL}};
const Segment SGI77061[] = {{&S27620[31], 3}, {NULL}};
const Segment SGI77062[] = {{&S27620[85], 6}, {NULL}};
const Segment SGI77063[] = {{&S27621[44], 27}, {NULL}};
const Segment SGI77064[] = {{&S27621[44], 39}, {NULL}};
const Segment SGI77065[] = {{&S27621[30], 6}, {NULL}};
const Segment SGI77066[] = {{&S27621[31], 3}, {NULL}};
const Segment SGI77067[] = {{&S27621[97], 6}, {NULL}};
const Segment SGI77068[] = {{&S27622[44], 27}, {NULL}};
const Segment SGI77069[] = {{&S27622[44], 39}, {NULL}};
const Segment SGI77070[] = {{&S27622[30], 6}, {NULL}};
const Segment SGI77071[] = {{&S27622[31], 3}, {NULL}};
const Segment SGI77072[] = {{&S27622[97], 6}, {NULL}};
const Segment SGI77073[] = {{&S27624[44], 27}, {NULL}};
const Segment SGI77074[] = {{&S27624[44], 39}, {NULL}};
const Segment SGI77075[] = {{&S27624[30], 6}, {NULL}};
const Segment SGI77076[] = {{&S27624[31], 3}, {NULL}};
const Segment SGI77077[] = {{&S27624[97], 6}, {NULL}};
const Segment SGI77078[] = {{&S27625[44], 27}, {NULL}};
const Segment SGI77079[] = {{&S27625[44], 39}, {NULL}};
const Segment SGI77080[] = {{&S27625[30], 6}, {NULL}};
const Segment SGI77081[] = {{&S27625[31], 3}, {NULL}};
const Segment SGI77082[] = {{&S27625[97], 6}, {NULL}};
const Segment SGI77083[] = {{&S27626[44], 25}, {NULL}};
const Segment SGI77084[] = {{&S27626[44], 37}, {NULL}};
const Segment SGI77085[] = {{&S27626[30], 6}, {NULL}};
const Segment SGI77086[] = {{&S27626[31], 3}, {NULL}};
const Segment SGI77087[] = {{&S27626[95], 6}, {NULL}};
const Segment SGI77088[] = {{&S27627[44], 27}, {NULL}};
const Segment SGI77089[] = {{&S27627[44], 39}, {NULL}};
const Segment SGI77090[] = {{&S27627[30], 6}, {NULL}};
const Segment SGI77091[] = {{&S27627[31], 3}, {NULL}};
const Segment SGI77092[] = {{&S27627[97], 6}, {NULL}};
const Segment SGI77093[] = {{&S27628[38], 15}, {NULL}};
const Segment SGI77094[] = {{&S27628[31], 6}, {NULL}};
const Segment SGI77095[] = {{&S27631[44], 11}, {NULL}};
const Segment SGI77096[] = {{&S27631[44], 23}, {NULL}};
const Segment SGI77097[] = {{&S27631[30], 6}, {NULL}};
const Segment SGI77098[] = {{&S27631[31], 3}, {NULL}};
const Segment SGI77099[] = {{&S27631[81], 6}, {NULL}};
const Segment SGI77100[] = {{&S27632[44], 26}, {NULL}};
const Segment SGI77101[] = {{&S27632[44], 38}, {NULL}};
const Segment SGI77102[] = {{&S27632[30], 6}, {NULL}};
const Segment SGI77103[] = {{&S27632[31], 3}, {NULL}};
const Segment SGI77104[] = {{&S27632[96], 6}, {NULL}};
const Segment SGI77105[] = {{&S27633[44], 11}, {NULL}};
const Segment SGI77106[] = {{&S27633[44], 23}, {NULL}};
const Segment SGI77107[] = {{&S27633[30], 6}, {NULL}};
const Segment SGI77108[] = {{&S27633[31], 3}, {NULL}};
const Segment SGI77109[] = {{&S27633[81], 6}, {NULL}};
const Segment SGI77110[] = {{&S27634[44], 21}, {NULL}};
const Segment SGI77111[] = {{&S27634[44], 33}, {NULL}};
const Segment SGI77112[] = {{&S27634[30], 6}, {NULL}};
const Segment SGI77113[] = {{&S27634[31], 3}, {NULL}};
const Segment SGI77114[] = {{&S27634[91], 6}, {NULL}};
const Segment SGI77115[] = {{&S27635[44], 26}, {NULL}};
const Segment SGI77116[] = {{&S27635[44], 38}, {NULL}};
const Segment SGI77117[] = {{&S27635[30], 6}, {NULL}};
const Segment SGI77118[] = {{&S27635[31], 3}, {NULL}};
const Segment SGI77119[] = {{&S27635[96], 6}, {NULL}};
const Segment SGI77120[] = {{&S27636[44], 21}, {NULL}};
const Segment SGI77121[] = {{&S27636[44], 33}, {NULL}};
const Segment SGI77122[] = {{&S27636[30], 6}, {NULL}};
const Segment SGI77123[] = {{&S27636[31], 3}, {NULL}};
const Segment SGI77124[] = {{&S27636[91], 6}, {NULL}};
const Segment SGI77125[] = {{&S27637[44], 15}, {NULL}};
const Segment SGI77126[] = {{&S27637[44], 27}, {NULL}};
const Segment SGI77127[] = {{&S27637[30], 6}, {NULL}};
const Segment SGI77128[] = {{&S27637[31], 3}, {NULL}};
const Segment SGI77129[] = {{&S27637[85], 6}, {NULL}};
const Segment SGI77130[] = {{&S27638[44], 24}, {NULL}};
const Segment SGI77131[] = {{&S27638[44], 36}, {NULL}};
const Segment SGI77132[] = {{&S27638[30], 6}, {NULL}};
const Segment SGI77133[] = {{&S27638[31], 3}, {NULL}};
const Segment SGI77134[] = {{&S27638[94], 6}, {NULL}};
const Segment SGI77135[] = {{&S27640[44], 23}, {NULL}};
const Segment SGI77136[] = {{&S27640[44], 35}, {NULL}};
const Segment SGI77137[] = {{&S27640[30], 6}, {NULL}};
const Segment SGI77138[] = {{&S27640[31], 3}, {NULL}};
const Segment SGI77139[] = {{&S27640[93], 6}, {NULL}};
const Segment SGI77140[] = {{&S27642[44], 27}, {NULL}};
const Segment SGI77141[] = {{&S27642[44], 39}, {NULL}};
const Segment SGI77142[] = {{&S27642[30], 6}, {NULL}};
const Segment SGI77143[] = {{&S27642[31], 3}, {NULL}};
const Segment SGI77144[] = {{&S27642[97], 6}, {NULL}};
const Segment SGI77145[] = {{&S27643[44], 27}, {NULL}};
const Segment SGI77146[] = {{&S27643[44], 39}, {NULL}};
const Segment SGI77147[] = {{&S27643[30], 6}, {NULL}};
const Segment SGI77148[] = {{&S27643[31], 3}, {NULL}};
const Segment SGI77149[] = {{&S27643[97], 6}, {NULL}};
const Segment SGI77150[] = {{&S45341[44], 6}, {NULL}};
const Segment SGI77151[] = {{&S45341[44], 22}, {NULL}};
const Segment SGI77152[] = {{&S45341[30], 6}, {NULL}};
const Segment SGI77153[] = {{&S45341[114], 3}, {NULL}};
const Segment SGI77154[] = {{&S45341[80], 5}, {NULL}};
const Segment SGI77155[] = {{&S27645[38], 21}, {NULL}};
const Segment SGI77156[] = {{&S45342[38], 6}, {NULL}};
const Segment SGI77157[] = {{&S27645[31], 6}, {NULL}};
const Segment SGI77158[] = {{&S45342[31], 6}, {NULL}};
const Segment SGI77159[] = {{&S27646[44], 26}, {NULL}};
const Segment SGI77160[] = {{&S27646[44], 38}, {NULL}};
const Segment SGI77161[] = {{&S27646[30], 6}, {NULL}};
const Segment SGI77162[] = {{&S27646[31], 3}, {NULL}};
const Segment SGI77163[] = {{&S27646[96], 6}, {NULL}};
const Segment SGI77164[] = {{&S27647[44], 15}, {NULL}};
const Segment SGI77165[] = {{&S27647[44], 38}, {NULL}};
const Segment SGI77166[] = {{&S27647[30], 6}, {NULL}};
const Segment SGI77167[] = {{&S27647[31], 3}, {NULL}};
const Segment SGI77168[] = {{&S27647[96], 6}, {NULL}};
const Segment SGI77169[] = {{&S27649[38], 11}, {NULL}};
const Segment SGI77170[] = {{&S27649[31], 6}, {NULL}};
const Segment SGI77171[] = {{&S27650[44], 20}, {NULL}};
const Segment SGI77172[] = {{&S27650[44], 32}, {NULL}};
const Segment SGI77173[] = {{&S27650[30], 6}, {NULL}};
const Segment SGI77174[] = {{&S27650[31], 3}, {NULL}};
const Segment SGI77175[] = {{&S27650[90], 6}, {NULL}};
const Segment SGI77176[] = {{&S27651[44], 26}, {NULL}};
const Segment SGI77177[] = {{&S27651[44], 38}, {NULL}};
const Segment SGI77178[] = {{&S27651[30], 6}, {NULL}};
const Segment SGI77179[] = {{&S27651[31], 3}, {NULL}};
const Segment SGI77180[] = {{&S27651[96], 6}, {NULL}};
const Segment SGI77181[] = {{&S27653[44], 26}, {NULL}};
const Segment SGI77182[] = {{&S27653[44], 38}, {NULL}};
const Segment SGI77183[] = {{&S27653[30], 6}, {NULL}};
const Segment SGI77184[] = {{&S27653[31], 3}, {NULL}};
const Segment SGI77185[] = {{&S27653[96], 6}, {NULL}};
const Segment SGI77186[] = {{&S27654[44], 26}, {NULL}};
const Segment SGI77187[] = {{&S27654[44], 38}, {NULL}};
const Segment SGI77188[] = {{&S27654[30], 6}, {NULL}};
const Segment SGI77189[] = {{&S27654[31], 3}, {NULL}};
const Segment SGI77190[] = {{&S27654[96], 6}, {NULL}};
const Segment SGI77191[] = {{&S27655[44], 27}, {NULL}};
const Segment SGI77192[] = {{&S27655[44], 39}, {NULL}};
const Segment SGI77193[] = {{&S27655[30], 6}, {NULL}};
const Segment SGI77194[] = {{&S27655[31], 3}, {NULL}};
const Segment SGI77195[] = {{&S27655[97], 6}, {NULL}};
const Segment SGI77196[] = {{&S27656[44], 27}, {NULL}};
const Segment SGI77197[] = {{&S27656[44], 39}, {NULL}};
const Segment SGI77198[] = {{&S27656[30], 6}, {NULL}};
const Segment SGI77199[] = {{&S27656[31], 3}, {NULL}};
const Segment SGI77200[] = {{&S27656[97], 6}, {NULL}};
const Segment SGI77201[] = {{&S27657[44], 27}, {NULL}};
const Segment SGI77202[] = {{&S27657[44], 39}, {NULL}};
const Segment SGI77203[] = {{&S27657[30], 6}, {NULL}};
const Segment SGI77204[] = {{&S27657[31], 3}, {NULL}};
const Segment SGI77205[] = {{&S27657[97], 6}, {NULL}};
const Segment SGI77206[] = {{&S27658[44], 27}, {NULL}};
const Segment SGI77207[] = {{&S27658[44], 39}, {NULL}};
const Segment SGI77208[] = {{&S27658[30], 6}, {NULL}};
const Segment SGI77209[] = {{&S27658[31], 3}, {NULL}};
const Segment SGI77210[] = {{&S27658[97], 6}, {NULL}};
const Segment SGI77211[] = {{&S27659[44], 27}, {NULL}};
const Segment SGI77212[] = {{&S27659[44], 39}, {NULL}};
const Segment SGI77213[] = {{&S27659[30], 6}, {NULL}};
const Segment SGI77214[] = {{&S27659[31], 3}, {NULL}};
const Segment SGI77215[] = {{&S27659[97], 6}, {NULL}};
const Segment SGI77216[] = {{&S27660[44], 21}, {NULL}};
const Segment SGI77217[] = {{&S27660[44], 33}, {NULL}};
const Segment SGI77218[] = {{&S27660[30], 6}, {NULL}};
const Segment SGI77219[] = {{&S27660[94], 6}, {NULL}};
const Segment SGI77220[] = {{&S27661[44], 27}, {NULL}};
const Segment SGI77221[] = {{&S27661[44], 39}, {NULL}};
const Segment SGI77222[] = {{&S27661[30], 6}, {NULL}};
const Segment SGI77223[] = {{&S27661[31], 3}, {NULL}};
const Segment SGI77224[] = {{&S27661[97], 6}, {NULL}};
const Segment SGI77225[] = {{&S27662[44], 27}, {NULL}};
const Segment SGI77226[] = {{&S27662[44], 39}, {NULL}};
const Segment SGI77227[] = {{&S27662[30], 6}, {NULL}};
const Segment SGI77228[] = {{&S27662[31], 3}, {NULL}};
const Segment SGI77229[] = {{&S27662[97], 6}, {NULL}};
const Segment SGI77230[] = {{&S45548[44], 25}, {NULL}};
const Segment SGI77231[] = {{&S45548[44], 37}, {NULL}};
const Segment SGI77232[] = {{&S45548[30], 6}, {NULL}};
const Segment SGI77233[] = {{&S45548[129], 3}, {NULL}};
const Segment SGI77234[] = {{&S45548[95], 5}, {NULL}};
const Segment SGI77235[] = {{&S45549[44], 25}, {NULL}};
const Segment SGI77236[] = {{&S45549[44], 37}, {NULL}};
const Segment SGI77237[] = {{&S45549[30], 6}, {NULL}};
const Segment SGI77238[] = {{&S45549[129], 3}, {NULL}};
const Segment SGI77239[] = {{&S45549[95], 5}, {NULL}};
const Segment SGI77240[] = {{&S45550[44], 25}, {NULL}};
const Segment SGI77241[] = {{&S45550[44], 37}, {NULL}};
const Segment SGI77242[] = {{&S45550[30], 6}, {NULL}};
const Segment SGI77243[] = {{&S45550[129], 3}, {NULL}};
const Segment SGI77244[] = {{&S45550[95], 5}, {NULL}};
const Segment SGI77245[] = {{&S45551[44], 25}, {NULL}};
const Segment SGI77246[] = {{&S45551[44], 37}, {NULL}};
const Segment SGI77247[] = {{&S45551[30], 6}, {NULL}};
const Segment SGI77248[] = {{&S45551[129], 3}, {NULL}};
const Segment SGI77249[] = {{&S45551[95], 5}, {NULL}};
const Segment SGI77250[] = {{&S39782[44], 7}, {NULL}};
const Segment SGI77251[] = {{&S39782[44], 18}, {NULL}};
const Segment SGI77252[] = {{&S39782[30], 6}, {NULL}};
const Segment SGI77253[] = {{&S39782[110], 3}, {NULL}};
const Segment SGI77254[] = {{&S39782[76], 5}, {NULL}};
const Segment SGI77255[] = {{&S39785[44], 7}, {NULL}};
const Segment SGI77256[] = {{&S39785[44], 18}, {NULL}};
const Segment SGI77257[] = {{&S39785[30], 6}, {NULL}};
const Segment SGI77258[] = {{&S39785[76], 6}, {NULL}};
const Segment SGI77259[] = {{&S39786[38], 7}, {NULL}};
const Segment SGI77260[] = {{&S39786[31], 6}, {NULL}};
const Segment SGI77261[] = {{&S39787[44], 7}, {NULL}};
const Segment SGI77262[] = {{&S39787[44], 18}, {NULL}};
const Segment SGI77263[] = {{&S39787[30], 6}, {NULL}};
const Segment SGI77264[] = {{&S39787[79], 4}, {NULL}};
const Segment SGI77265[] = {{&S39787[79], 3}, {NULL}};
const Segment SGI77266[] = {{&S39788[44], 7}, {NULL}};
const Segment SGI77267[] = {{&S39788[44], 20}, {NULL}};
const Segment SGI77268[] = {{&S39788[30], 6}, {NULL}};
const Segment SGI77269[] = {{&S39788[112], 3}, {NULL}};
const Segment SGI77270[] = {{&S39788[78], 5}, {NULL}};
const Segment SGI77271[] = {{&S39797[44], 7}, {NULL}};
const Segment SGI77272[] = {{&S39797[44], 18}, {NULL}};
const Segment SGI77273[] = {{&S39797[30], 6}, {NULL}};
const Segment SGI77274[] = {{&S39797[79], 6}, {NULL}};
const Segment SGI77275[] = {{&S39808[44], 7}, {NULL}};
const Segment SGI77276[] = {{&S39808[44], 20}, {NULL}};
const Segment SGI77277[] = {{&S39808[30], 6}, {NULL}};
const Segment SGI77278[] = {{&S39808[112], 3}, {NULL}};
const Segment SGI77279[] = {{&S39808[78], 5}, {NULL}};
const Segment SGI77280[] = {{&S39814[44], 8}, {NULL}};
const Segment SGI77281[] = {{&S39814[44], 20}, {NULL}};
const Segment SGI77282[] = {{&S39814[30], 6}, {NULL}};
const Segment SGI77283[] = {{&S39814[112], 3}, {NULL}};
const Segment SGI77284[] = {{&S39814[78], 5}, {NULL}};
const Segment SGI77285[] = {{&S39816[38], 8}, {NULL}};
const Segment SGI77286[] = {{&S39816[31], 6}, {NULL}};
const Segment SGI77287[] = {{&S39817[44], 17}, {NULL}};
const Segment SGI77288[] = {{&S39817[30], 6}, {NULL}};
const Segment SGI77289[] = {{&S45729[44], 6}, {NULL}};
const Segment SGI77290[] = {{&S45729[44], 23}, {NULL}};
const Segment SGI77291[] = {{&S45729[30], 6}, {NULL}};
const Segment SGI77292[] = {{&S45729[115], 3}, {NULL}};
const Segment SGI77293[] = {{&S45729[81], 5}, {NULL}};
const Segment SGI77294[] = {{&S45730[38], 6}, {NULL}};
const Segment SGI77295[] = {{&S45730[31], 6}, {NULL}};
const Segment SGI77296[] = {{&S30288[44], 14}, {NULL}};
const Segment SGI77297[] = {{&S30288[44], 26}, {NULL}};
const Segment SGI77298[] = {{&S30288[30], 6}, {NULL}};
const Segment SGI77299[] = {{&S30288[31], 3}, {NULL}};
const Segment SGI77300[] = {{&S30288[84], 6}, {NULL}};
const Segment SGI77301[] = {{&S30291[44], 14}, {NULL}};
const Segment SGI77302[] = {{&S30291[44], 26}, {NULL}};
const Segment SGI77303[] = {{&S30291[30], 6}, {NULL}};
const Segment SGI77304[] = {{&S30291[31], 3}, {NULL}};
const Segment SGI77305[] = {{&S30291[84], 6}, {NULL}};
const Segment SGI77306[] = {{&S30292[38], 14}, {NULL}};
const Segment SGI77307[] = {{&S30292[31], 6}, {NULL}};
const Segment SGI77308[] = {{&S30302[64], 14}, {NULL}};
const Segment SGI77309[] = {{&S30302[64], 26}, {NULL}};
const Segment SGI77310[] = {{&S30302[50], 6}, {NULL}};
const Segment SGI77311[] = {{&S30320[44], 10}, {NULL}};
const Segment SGI77312[] = {{&S30320[44], 22}, {NULL}};
const Segment SGI77313[] = {{&S30320[30], 6}, {NULL}};
const Segment SGI77314[] = {{&S30320[31], 3}, {NULL}};
const Segment SGI77315[] = {{&S30320[80], 6}, {NULL}};
const Segment SGI77316[] = {{&S30336[44], 14}, {NULL}};
const Segment SGI77317[] = {{&S30336[44], 26}, {NULL}};
const Segment SGI77318[] = {{&S30336[30], 6}, {NULL}};
const Segment SGI77319[] = {{&S30336[87], 4}, {NULL}};
const Segment SGI77320[] = {{&S30336[87], 3}, {NULL}};
const Segment SGI77321[] = {{&S30346[44], 14}, {NULL}};
const Segment SGI77322[] = {{&S30346[44], 26}, {NULL}};
const Segment SGI77323[] = {{&S30346[30], 6}, {NULL}};
const Segment SGI77324[] = {{&S30346[87], 6}, {NULL}};
const Segment SGI77325[] = {{&S30351[44], 14}, {NULL}};
const Segment SGI77326[] = {{&S30351[44], 26}, {NULL}};
const Segment SGI77327[] = {{&S30351[30], 6}, {NULL}};
const Segment SGI77328[] = {{&S30351[31], 3}, {NULL}};
const Segment SGI77329[] = {{&S30351[84], 5}, {NULL}};
const Segment SGI77330[] = {{&S30354[44], 14}, {NULL}};
const Segment SGI77331[] = {{&S30354[44], 26}, {NULL}};
const Segment SGI77332[] = {{&S30354[30], 6}, {NULL}};
const Segment SGI77333[] = {{&S30354[31], 3}, {NULL}};
const Segment SGI77334[] = {{&S30354[84], 6}, {NULL}};
const Segment SGI77335[] = {{&S45917[44], 11}, {NULL}};
const Segment SGI77336[] = {{&S45917[44], 28}, {NULL}};
const Segment SGI77337[] = {{&S45917[30], 6}, {NULL}};
const Segment SGI77338[] = {{&S45917[120], 3}, {NULL}};
const Segment SGI77339[] = {{&S45917[86], 5}, {NULL}};
const Segment SGI77340[] = {{&S45918[38], 11}, {NULL}};
const Segment SGI77341[] = {{&S45918[31], 6}, {NULL}};
const Segment SGI77342[] = {{&S46044[44], 8}, {NULL}};
const Segment SGI77343[] = {{&S46044[44], 25}, {NULL}};
const Segment SGI77344[] = {{&S46044[30], 6}, {NULL}};
const Segment SGI77345[] = {{&S46044[117], 3}, {NULL}};
const Segment SGI77346[] = {{&S46044[83], 5}, {NULL}};
const Segment SGI77347[] = {{&S46047[42], 8}, {NULL}};
const Segment SGI77348[] = {{&S46047[30], 6}, {NULL}};
const Segment SGI77349[] = {{&S46047[115], 3}, {NULL}};
const Segment SGI77350[] = {{&S46047[81], 5}, {NULL}};
const Segment SGI77351[] = {{&S46048[38], 8}, {NULL}};
const Segment SGI77352[] = {{&S46048[31], 6}, {NULL}};
const Segment SGI77353[] = {{&S28352[44], 18}, {NULL}};
const Segment SGI77354[] = {{&S28352[44], 30}, {NULL}};
const Segment SGI77355[] = {{&S28352[30], 6}, {NULL}};
const Segment SGI77356[] = {{&S28352[31], 3}, {NULL}};
const Segment SGI77357[] = {{&S28352[88], 6}, {NULL}};
const Segment SGI77358[] = {{&S28356[44], 23}, {NULL}};
const Segment SGI77359[] = {{&S28356[44], 35}, {NULL}};
const Segment SGI77360[] = {{&S28356[30], 6}, {NULL}};
const Segment SGI77361[] = {{&S28356[31], 3}, {NULL}};
const Segment SGI77362[] = {{&S28356[93], 6}, {NULL}};
const Segment SGI77363[] = {{&S28360[44], 18}, {NULL}};
const Segment SGI77364[] = {{&S28360[44], 30}, {NULL}};
const Segment SGI77365[] = {{&S28360[30], 6}, {NULL}};
const Segment SGI77366[] = {{&S28360[91], 6}, {NULL}};
const Segment SGI77367[] = {{&S28361[38], 18}, {NULL}};
const Segment SGI77368[] = {{&S28361[31], 6}, {NULL}};
const Segment SGI77369[] = {{&S33962[44], 11}, {NULL}};
const Segment SGI77370[] = {{&S33962[44], 42}, {NULL}};
const Segment SGI77371[] = {{&S33962[30], 6}, {NULL}};
const Segment SGI77372[] = {{&S33962[31], 3}, {NULL}};
const Segment SGI77373[] = {{&S33962[100], 5}, {NULL}};
const Segment SGI77374[] = {{&S33962[0], 13}, {NULL}};
const Segment SGI77375[] = {{&S28380[44], 10}, {NULL}};
const Segment SGI77376[] = {{&S28380[44], 22}, {NULL}};
const Segment SGI77377[] = {{&S28380[30], 6}, {NULL}};
const Segment SGI77378[] = {{&S28380[31], 3}, {NULL}};
const Segment SGI77379[] = {{&S28380[80], 6}, {NULL}};
const Segment SGI77380[] = {{&S33969[44], 12}, {NULL}};
const Segment SGI77381[] = {{&S28381[38], 10}, {NULL}};
const Segment SGI77382[] = {{&S33969[44], 24}, {NULL}};
const Segment SGI77383[] = {{&S33969[30], 6}, {NULL}};
const Segment SGI77384[] = {{&S28381[31], 6}, {NULL}};
const Segment SGI77385[] = {{&S33969[31], 3}, {NULL}};
const Segment SGI77386[] = {{&S33969[82], 6}, {NULL}};
const Segment SGI77387[] = {{&S33969[0], 13}, {NULL}};
const Segment SGI77388[] = {{&S28385[44], 10}, {NULL}};
const Segment SGI77389[] = {{&S28385[44], 22}, {NULL}};
const Segment SGI77390[] = {{&S28385[30], 6}, {NULL}};
const Segment SGI77391[] = {{&S28385[31], 3}, {NULL}};
const Segment SGI77392[] = {{&S28385[80], 6}, {NULL}};
const Segment SGI77393[] = {{&S28387[42], 10}, {NULL}};
const Segment SGI77394[] = {{&S28387[30], 6}, {NULL}};
const Segment SGI77395[] = {{&S28387[31], 3}, {NULL}};
const Segment SGI77396[] = {{&S28387[78], 6}, {NULL}};
const Segment SGI77397[] = {{&S28389[64], 10}, {NULL}};
const Segment SGI77398[] = {{&S28389[64], 22}, {NULL}};
const Segment SGI77399[] = {{&S28389[50], 6}, {NULL}};
const Segment SGI77400[] = {{&S28392[34], 10}, {NULL}};
const Segment SGI77401[] = {{&S28392[34], 22}, {NULL}};
const Segment SGI77402[] = {{&S28396[44], 10}, {NULL}};
const Segment SGI77403[] = {{&S28396[44], 22}, {NULL}};
const Segment SGI77404[] = {{&S28396[30], 6}, {NULL}};
const Segment SGI77405[] = {{&S28396[80], 6}, {NULL}};
const Segment SGI77406[] = {{&S28401[44], 10}, {NULL}};
const Segment SGI77407[] = {{&S28401[44], 22}, {NULL}};
const Segment SGI77408[] = {{&S28401[30], 6}, {NULL}};
const Segment SGI77409[] = {{&S28401[83], 6}, {NULL}};
const Segment SGI77410[] = {{&S28402[44], 10}, {NULL}};
const Segment SGI77411[] = {{&S28402[44], 34}, {NULL}};
const Segment SGI77412[] = {{&S28402[30], 6}, {NULL}};
const Segment SGI77413[] = {{&S28406[46], 9}, {NULL}};
const Segment SGI77414[] = {{&S28406[32], 6}, {NULL}};
const Segment SGI77415[] = {{&S28415[44], 10}, {NULL}};
const Segment SGI77416[] = {{&S28415[44], 22}, {NULL}};
const Segment SGI77417[] = {{&S28415[30], 6}, {NULL}};
const Segment SGI77418[] = {{&S28415[83], 4}, {NULL}};
const Segment SGI77419[] = {{&S28415[83], 3}, {NULL}};
const Segment SGI77420[] = {{&S28419[44], 10}, {NULL}};
const Segment SGI77421[] = {{&S28419[44], 19}, {NULL}};
const Segment SGI77422[] = {{&S28419[30], 6}, {NULL}};
const Segment SGI77423[] = {{&S28419[31], 3}, {NULL}};
const Segment SGI77424[] = {{&S28419[77], 5}, {NULL}};
const Segment SGI77425[] = {{&S28422[44], 10}, {NULL}};
const Segment SGI77426[] = {{&S28422[44], 22}, {NULL}};
const Segment SGI77427[] = {{&S28422[30], 6}, {NULL}};
const Segment SGI77428[] = {{&S28422[116], 3}, {NULL}};
const Segment SGI77429[] = {{&S28422[80], 7}, {NULL}};
const Segment SGI77430[] = {{&S28422[136], 8}, {NULL}};
const Segment SGI77431[] = {{&S28454[44], 30}, {NULL}};
const Segment SGI77432[] = {{&S28454[44], 42}, {NULL}};
const Segment SGI77433[] = {{&S28454[30], 6}, {NULL}};
const Segment SGI77434[] = {{&S28454[31], 3}, {NULL}};
const Segment SGI77435[] = {{&S28454[100], 6}, {NULL}};
const Segment SGI77436[] = {{&S28455[44], 11}, {NULL}};
const Segment SGI77437[] = {{&S28455[44], 23}, {NULL}};
const Segment SGI77438[] = {{&S28455[30], 6}, {NULL}};
const Segment SGI77439[] = {{&S28455[31], 3}, {NULL}};
const Segment SGI77440[] = {{&S28455[81], 6}, {NULL}};
const Segment SGI77441[] = {{&S28456[44], 30}, {NULL}};
const Segment SGI77442[] = {{&S28456[44], 42}, {NULL}};
const Segment SGI77443[] = {{&S28456[30], 6}, {NULL}};
const Segment SGI77444[] = {{&S28456[31], 3}, {NULL}};
const Segment SGI77445[] = {{&S28456[100], 6}, {NULL}};
const Segment SGI77446[] = {{&S28457[44], 30}, {NULL}};
const Segment SGI77447[] = {{&S28457[44], 42}, {NULL}};
const Segment SGI77448[] = {{&S28457[30], 6}, {NULL}};
const Segment SGI77449[] = {{&S28457[31], 3}, {NULL}};
const Segment SGI77450[] = {{&S28457[100], 6}, {NULL}};
const Segment SGI77451[] = {{&S28458[44], 31}, {NULL}};
const Segment SGI77452[] = {{&S28458[44], 43}, {NULL}};
const Segment SGI77453[] = {{&S28458[30], 6}, {NULL}};
const Segment SGI77454[] = {{&S28458[31], 3}, {NULL}};
const Segment SGI77455[] = {{&S28458[101], 6}, {NULL}};
const Segment SGI77456[] = {{&S28459[38], 11}, {NULL}};
const Segment SGI77457[] = {{&S28459[31], 6}, {NULL}};
const Segment SGI77458[] = {{&S40378[44], 10}, {NULL}};
const Segment SGI77459[] = {{&S40378[44], 25}, {NULL}};
const Segment SGI77460[] = {{&S40378[30], 6}, {NULL}};
const Segment SGI77461[] = {{&S40378[117], 3}, {NULL}};
const Segment SGI77462[] = {{&S40378[83], 5}, {NULL}};
const Segment SGI77463[] = {{&S28460[44], 31}, {NULL}};
const Segment SGI77464[] = {{&S28460[44], 43}, {NULL}};
const Segment SGI77465[] = {{&S28460[30], 6}, {NULL}};
const Segment SGI77466[] = {{&S28460[31], 3}, {NULL}};
const Segment SGI77467[] = {{&S28460[101], 6}, {NULL}};
const Segment SGI77468[] = {{&S40379[44], 10}, {NULL}};
const Segment SGI77469[] = {{&S40379[44], 25}, {NULL}};
const Segment SGI77470[] = {{&S40379[30], 6}, {NULL}};
const Segment SGI77471[] = {{&S40379[117], 3}, {NULL}};
const Segment SGI77472[] = {{&S40379[83], 5}, {NULL}};
const Segment SGI77473[] = {{&S40380[38], 10}, {NULL}};
const Segment SGI77474[] = {{&S40380[31], 6}, {NULL}};
const Segment SGI77475[] = {{&S40381[44], 10}, {NULL}};
const Segment SGI77476[] = {{&S40381[44], 25}, {NULL}};
const Segment SGI77477[] = {{&S40381[30], 6}, {NULL}};
const Segment SGI77478[] = {{&S40381[117], 3}, {NULL}};
const Segment SGI77479[] = {{&S40381[83], 5}, {NULL}};
const Segment SGI77480[] = {{&S28466[42], 11}, {NULL}};
const Segment SGI77481[] = {{&S28466[30], 6}, {NULL}};
const Segment SGI77482[] = {{&S28466[31], 3}, {NULL}};
const Segment SGI77483[] = {{&S28466[79], 6}, {NULL}};
const Segment SGI77484[] = {{&S46507[6], 4}, {NULL}};
const Segment SGI77485[] = {{&S46507[83], 5}, {NULL}};
const Segment SGI77486[] = {{&S28853[44], 10}, {NULL}};
const Segment SGI77487[] = {{&S28853[44], 22}, {NULL}};
const Segment SGI77488[] = {{&S28853[30], 6}, {NULL}};
const Segment SGI77489[] = {{&S28853[31], 3}, {NULL}};
const Segment SGI77490[] = {{&S28853[80], 6}, {NULL}};
const Segment SGI77491[] = {{&S28854[38], 10}, {NULL}};
const Segment SGI77492[] = {{&S28854[31], 6}, {NULL}};
const Segment SGI77493[] = {{&S28859[44], 10}, {NULL}};
const Segment SGI77494[] = {{&S28859[44], 22}, {NULL}};
const Segment SGI77495[] = {{&S28859[30], 6}, {NULL}};
const Segment SGI77496[] = {{&S28859[31], 3}, {NULL}};
const Segment SGI77497[] = {{&S28859[80], 6}, {NULL}};
const Segment SGI77498[] = {{&S28860[42], 10}, {NULL}};
const Segment SGI77499[] = {{&S28860[30], 6}, {NULL}};
const Segment SGI77500[] = {{&S28860[31], 3}, {NULL}};
const Segment SGI77501[] = {{&S28860[78], 6}, {NULL}};
const Segment SGI77502[] = {{&S28861[44], 10}, {NULL}};
const Segment SGI77503[] = {{&S28861[44], 34}, {NULL}};
const Segment SGI77504[] = {{&S28861[30], 6}, {NULL}};
const Segment SGI77505[] = {{&S28864[64], 10}, {NULL}};
const Segment SGI77506[] = {{&S28864[64], 22}, {NULL}};
const Segment SGI77507[] = {{&S28864[50], 6}, {NULL}};
const Segment SGI77508[] = {{&S28867[34], 10}, {NULL}};
const Segment SGI77509[] = {{&S28867[34], 22}, {NULL}};
const Segment SGI77510[] = {{&S28868[44], 10}, {NULL}};
const Segment SGI77511[] = {{&S28868[44], 22}, {NULL}};
const Segment SGI77512[] = {{&S28868[30], 6}, {NULL}};
const Segment SGI77513[] = {{&S28868[83], 4}, {NULL}};
const Segment SGI77514[] = {{&S28868[83], 3}, {NULL}};
const Segment SGI77515[] = {{&S28875[44], 10}, {NULL}};
const Segment SGI77516[] = {{&S28875[44], 22}, {&S28875[206], 7}, {NULL}};
const Segment SGI77517[] = {{&S28875[30], 6}, {NULL}};
const Segment SGI77518[] = {{&S28875[31], 3}, {NULL}};
const Segment SGI77519[] = {{&S28875[80], 6}, {NULL}};
const Segment SGI77520[] = {{&S28880[44], 10}, {NULL}};
const Segment SGI77521[] = {{&S28880[44], 34}, {NULL}};
const Segment SGI77522[] = {{&S28880[30], 6}, {NULL}};
const Segment SGI77523[] = {{&S28893[103], 29}, {NULL}};
const Segment SGI77524[] = {{&S28894[44], 10}, {NULL}};
const Segment SGI77525[] = {{&S28894[44], 22}, {NULL}};
const Segment SGI77526[] = {{&S28894[30], 6}, {NULL}};
const Segment SGI77527[] = {{&S28894[116], 3}, {NULL}};
const Segment SGI77528[] = {{&S28894[80], 7}, {NULL}};
const Segment SGI77529[] = {{&S28894[136], 8}, {NULL}};
const Segment SGI77530[] = {{&S28895[44], 18}, {NULL}};
const Segment SGI77531[] = {{&S28895[44], 30}, {NULL}};
const Segment SGI77532[] = {{&S28897[44], 9}, {NULL}};
const Segment SGI77533[] = {{&S28897[44], 21}, {NULL}};
const Segment SGI77534[] = {{&S28897[30], 6}, {NULL}};
const Segment SGI77535[] = {{&S28897[31], 3}, {NULL}};
const Segment SGI77536[] = {{&S28897[79], 6}, {NULL}};
const Segment SGI77537[] = {{&S28902[44], 10}, {NULL}};
const Segment SGI77538[] = {{&S28902[44], 22}, {NULL}};
const Segment SGI77539[] = {{&S28902[30], 6}, {NULL}};
const Segment SGI77540[] = {{&S28902[80], 6}, {NULL}};
const Segment SGI77541[] = {{&S28931[44], 6}, {NULL}};
const Segment SGI77542[] = {{&S28931[44], 18}, {NULL}};
const Segment SGI77543[] = {{&S28931[30], 6}, {NULL}};
const Segment SGI77544[] = {{&S28931[31], 3}, {NULL}};
const Segment SGI77545[] = {{&S28931[76], 6}, {NULL}};
const Segment SGI77546[] = {{&S28932[44], 18}, {NULL}};
const Segment SGI77547[] = {{&S28932[44], 27}, {NULL}};
const Segment SGI77548[] = {{&S28932[30], 6}, {NULL}};
const Segment SGI77549[] = {{&S28932[31], 3}, {NULL}};
const Segment SGI77550[] = {{&S28932[85], 6}, {NULL}};
const Segment SGI77551[] = {{&S28933[38], 6}, {NULL}};
const Segment SGI77552[] = {{&S28933[31], 6}, {NULL}};
const Segment SGI77553[] = {{&S28934[44], 13}, {NULL}};
const Segment SGI77554[] = {{&S28934[44], 25}, {NULL}};
const Segment SGI77555[] = {{&S28934[30], 6}, {NULL}};
const Segment SGI77556[] = {{&S28934[31], 3}, {NULL}};
const Segment SGI77557[] = {{&S28934[83], 6}, {NULL}};
const Segment SGI77558[] = {{&S28935[44], 6}, {NULL}};
const Segment SGI77559[] = {{&S28935[44], 18}, {NULL}};
const Segment SGI77560[] = {{&S28935[30], 6}, {NULL}};
const Segment SGI77561[] = {{&S28935[31], 3}, {NULL}};
const Segment SGI77562[] = {{&S28935[76], 6}, {NULL}};
const Segment SGI77563[] = {{&S28939[42], 6}, {NULL}};
const Segment SGI77564[] = {{&S28939[30], 6}, {NULL}};
const Segment SGI77565[] = {{&S28939[31], 3}, {NULL}};
const Segment SGI77566[] = {{&S28939[74], 6}, {NULL}};
const Segment SGI77567[] = {{&S41050[44], 10}, {NULL}};
const Segment SGI77568[] = {{&S41050[44], 21}, {NULL}};
const Segment SGI77569[] = {{&S41050[30], 6}, {NULL}};
const Segment SGI77570[] = {{&S41050[113], 3}, {NULL}};
const Segment SGI77571[] = {{&S41050[79], 5}, {NULL}};
const Segment SGI77572[] = {{&S41057[38], 10}, {NULL}};
const Segment SGI77573[] = {{&S41057[31], 6}, {NULL}};
const Segment SGI77574[] = {{&S40404[42], 10}, {NULL}};
const Segment SGI77575[] = {{&S40404[42], 22}, {NULL}};
const Segment SGI77576[] = {{&S40404[30], 4}, {NULL}};
const Segment SGI77577[] = {{&S40404[113], 3}, {NULL}};
const Segment SGI77578[] = {{&S40404[78], 6}, {NULL}};
const Segment SGI77579[] = {{&S40405[36], 10}, {NULL}};
const Segment SGI77580[] = {{&S40405[31], 4}, {NULL}};
const Segment SGI77581[] = {{&S40406[42], 10}, {NULL}};
const Segment SGI77582[] = {{&S40406[42], 22}, {NULL}};
const Segment SGI77583[] = {{&S40406[30], 4}, {NULL}};
const Segment SGI77584[] = {{&S40406[113], 3}, {NULL}};
const Segment SGI77585[] = {{&S40406[78], 6}, {NULL}};
const Segment SGI77586[] = {{&S40407[42], 10}, {NULL}};
const Segment SGI77587[] = {{&S40407[42], 22}, {NULL}};
const Segment SGI77588[] = {{&S40407[30], 4}, {NULL}};
const Segment SGI77589[] = {{&S40407[81], 6}, {NULL}};
const Segment SGI77590[] = {{&S40409[40], 10}, {NULL}};
const Segment SGI77591[] = {{&S40409[30], 4}, {NULL}};
const Segment SGI77592[] = {{&S40409[111], 3}, {NULL}};
const Segment SGI77593[] = {{&S40409[76], 6}, {NULL}};
const Segment SGI77594[] = {{&S40410[42], 10}, {NULL}};
const Segment SGI77595[] = {{&S40410[42], 28}, {NULL}};
const Segment SGI77596[] = {{&S40410[30], 4}, {NULL}};
const Segment SGI77597[] = {{&S40411[42], 10}, {NULL}};
const Segment SGI77598[] = {{&S40411[42], 22}, {NULL}};
const Segment SGI77599[] = {{&S40411[30], 4}, {NULL}};
const Segment SGI77600[] = {{&S40412[42], 10}, {NULL}};
const Segment SGI77601[] = {{&S40412[42], 28}, {NULL}};
const Segment SGI77602[] = {{&S40412[30], 4}, {NULL}};
const Segment SGI77603[] = {{&S40413[42], 10}, {NULL}};
const Segment SGI77604[] = {{&S40413[42], 28}, {NULL}};
const Segment SGI77605[] = {{&S40413[30], 4}, {NULL}};
const Segment SGI77606[] = {{&S40414[43], 10}, {NULL}};
const Segment SGI77607[] = {{&S40414[43], 28}, {NULL}};
const Segment SGI77608[] = {{&S40414[31], 4}, {NULL}};
const Segment SGI77609[] = {{&S40415[42], 10}, {NULL}};
const Segment SGI77610[] = {{&S40415[42], 22}, {NULL}};
const Segment SGI77611[] = {{&S40415[30], 4}, {NULL}};
const Segment SGI77612[] = {{&S40415[123], 3}, {NULL}};
const Segment SGI77613[] = {{&S40415[88], 6}, {NULL}};
const Segment SGI77614[] = {{&S40415[143], 8}, {NULL}};
const Segment SGI77615[] = {{&S40416[42], 10}, {NULL}};
const Segment SGI77616[] = {{&S40416[42], 28}, {NULL}};
const Segment SGI77617[] = {{&S40416[30], 4}, {NULL}};
const Segment SGI77618[] = {{&S40417[42], 10}, {NULL}};
const Segment SGI77619[] = {{&S40417[42], 22}, {NULL}};
const Segment SGI77620[] = {{&S40417[30], 4}, {NULL}};
const Segment SGI77621[] = {{&S40417[112], 3}, {NULL}};
const Segment SGI77622[] = {{&S40417[78], 5}, {NULL}};
const Segment SGI77623[] = {{&S40418[53], 18}, {NULL}};
const Segment SGI77624[] = {{&S40418[48], 4}, {NULL}};
const Segment SGI77625[] = {{&S40419[30], 1}, {NULL}};
const Segment SGI77626[] = {{&S40419[61], 9}, {NULL}};
const Segment SGI77627[] = {{&S40419[49], 4}, {NULL}};
const Segment SGI77628[] = {{&S40420[96], 29}, {NULL}};
const Segment SGI77629[] = {{&S40421[42], 10}, {NULL}};
const Segment SGI77630[] = {{&S40421[42], 22}, {NULL}};
const Segment SGI77631[] = {{&S40421[30], 4}, {NULL}};
const Segment SGI77632[] = {{&S40421[113], 3}, {&S40421[193], 3}, {NULL}};
const Segment SGI77633[] = {{&S40421[78], 6}, {NULL}};
const Segment SGI77634[] = {{&S40423[42], 9}, {NULL}};
const Segment SGI77635[] = {{&S40423[30], 4}, {NULL}};
const Segment SGI77636[] = {{&S40424[42], 10}, {NULL}};
const Segment SGI77637[] = {{&S40424[42], 22}, {&S40424[205], 7}, {NULL}};
const Segment SGI77638[] = {{&S40424[30], 4}, {NULL}};
const Segment SGI77639[] = {{&S40424[113], 3}, {NULL}};
const Segment SGI77640[] = {{&S40424[78], 6}, {NULL}};
const Segment SGI77641[] = {{&S40426[42], 10}, {NULL}};
const Segment SGI77642[] = {{&S40426[42], 22}, {NULL}};
const Segment SGI77643[] = {{&S40426[30], 4}, {NULL}};
const Segment SGI77644[] = {{&S40426[113], 3}, {&S40426[193], 3}, {NULL}};
const Segment SGI77645[] = {{&S40426[78], 6}, {NULL}};
const Segment SGI77646[] = {{&S40536[42], 10}, {NULL}};
const Segment SGI77647[] = {{&S40536[42], 21}, {NULL}};
const Segment SGI77648[] = {{&S40536[30], 4}, {NULL}};
const Segment SGI77649[] = {{&S40536[112], 3}, {NULL}};
const Segment SGI77650[] = {{&S40536[77], 6}, {NULL}};
const Segment SGI77651[] = {{&S40537[42], 10}, {NULL}};
const Segment SGI77652[] = {{&S40537[42], 25}, {NULL}};
const Segment SGI77653[] = {{&S40537[30], 4}, {NULL}};
const Segment SGI77654[] = {{&S40537[116], 3}, {NULL}};
const Segment SGI77655[] = {{&S40537[81], 6}, {NULL}};
const Segment SGI77656[] = {{&S40538[42], 10}, {NULL}};
const Segment SGI77657[] = {{&S40538[42], 21}, {NULL}};
const Segment SGI77658[] = {{&S40538[30], 4}, {NULL}};
const Segment SGI77659[] = {{&S40538[112], 3}, {NULL}};
const Segment SGI77660[] = {{&S40538[77], 6}, {NULL}};
const Segment SGI77661[] = {{&S40539[36], 10}, {NULL}};
const Segment SGI77662[] = {{&S40539[31], 4}, {NULL}};
const Segment SGI77663[] = {{&S40540[42], 10}, {NULL}};
const Segment SGI77664[] = {{&S40540[42], 21}, {NULL}};
const Segment SGI77665[] = {{&S40540[30], 4}, {NULL}};
const Segment SGI77666[] = {{&S40540[113], 3}, {NULL}};
const Segment SGI77667[] = {{&S47202[44], 16}, {NULL}};
const Segment SGI77668[] = {{&S47202[44], 28}, {NULL}};
const Segment SGI77669[] = {{&S47202[30], 6}, {NULL}};
const Segment SGI77670[] = {{&S40540[77], 7}, {NULL}};
const Segment SGI77671[] = {{&S40540[133], 8}, {NULL}};
const Segment SGI77672[] = {{&S47202[120], 3}, {NULL}};
const Segment SGI77673[] = {{&S47202[86], 5}, {NULL}};
const Segment SGI77674[] = {{&S40541[42], 10}, {NULL}};
const Segment SGI77675[] = {{&S40541[42], 21}, {NULL}};
const Segment SGI77676[] = {{&S40541[30], 4}, {NULL}};
const Segment SGI77677[] = {{&S47203[38], 16}, {NULL}};
const Segment SGI77678[] = {{&S40541[31], 3}, {NULL}};
const Segment SGI77679[] = {{&S47203[31], 6}, {NULL}};
const Segment SGI77680[] = {{&S40541[87], 6}, {NULL}};
const Segment SGI77681[] = {{&S40541[142], 8}, {NULL}};
const Segment SGI77682[] = {{&S40542[42], 10}, {NULL}};
const Segment SGI77683[] = {{&S40542[42], 27}, {NULL}};
const Segment SGI77684[] = {{&S40542[30], 4}, {NULL}};
const Segment SGI77685[] = {{&S40543[42], 10}, {NULL}};
const Segment SGI77686[] = {{&S40543[42], 21}, {NULL}};
const Segment SGI77687[] = {{&S40543[30], 4}, {NULL}};
const Segment SGI77688[] = {{&S40543[80], 6}, {NULL}};
const Segment SGI77689[] = {{&S40544[32], 10}, {NULL}};
const Segment SGI77690[] = {{&S40544[32], 21}, {NULL}};
const Segment SGI77691[] = {{&S40545[42], 10}, {NULL}};
const Segment SGI77692[] = {{&S40545[42], 21}, {&S40545[205], 7}, {NULL}};
const Segment SGI77693[] = {{&S40545[30], 4}, {NULL}};
const Segment SGI77694[] = {{&S40545[112], 3}, {NULL}};
const Segment SGI77695[] = {{&S40545[77], 6}, {NULL}};
const Segment SGI77696[] = {{&S40546[44], 10}, {NULL}};
const Segment SGI77697[] = {{&S40546[44], 21}, {NULL}};
const Segment SGI77698[] = {{&S40546[32], 4}, {NULL}};
const Segment SGI77699[] = {{&S40547[40], 10}, {NULL}};
const Segment SGI77700[] = {{&S40547[30], 4}, {NULL}};
const Segment SGI77701[] = {{&S40547[110], 3}, {NULL}};
const Segment SGI77702[] = {{&S40547[75], 6}, {NULL}};
const Segment SGI77703[] = {{&S40548[37], 4}, {NULL}};
const Segment SGI77704[] = {{&S41008[42], 18}, {NULL}};
const Segment SGI77705[] = {{&S41008[42], 30}, {NULL}};
const Segment SGI77706[] = {{&S41008[30], 4}, {NULL}};
const Segment SGI77707[] = {{&S41008[121], 3}, {NULL}};
const Segment SGI77708[] = {{&S41008[86], 6}, {NULL}};
const Segment SGI77709[] = {{&S41009[42], 11}, {NULL}};
const Segment SGI77710[] = {{&S41009[42], 23}, {NULL}};
const Segment SGI77711[] = {{&S41009[30], 4}, {NULL}};
const Segment SGI77712[] = {{&S41009[114], 3}, {NULL}};
const Segment SGI77713[] = {{&S41009[79], 6}, {NULL}};
const Segment SGI77714[] = {{&S41010[42], 18}, {NULL}};
const Segment SGI77715[] = {{&S41010[42], 35}, {NULL}};
const Segment SGI77716[] = {{&S41010[30], 4}, {NULL}};
const Segment SGI77717[] = {{&S41011[36], 18}, {NULL}};
const Segment SGI77718[] = {{&S41011[31], 4}, {NULL}};
const Segment SGI77719[] = {{&S41012[42], 11}, {NULL}};
const Segment SGI77720[] = {{&S41012[42], 23}, {NULL}};
const Segment SGI77721[] = {{&S41012[30], 4}, {NULL}};
const Segment SGI77722[] = {{&S41012[114], 3}, {NULL}};
const Segment SGI77723[] = {{&S41012[79], 6}, {NULL}};
const Segment SGI77724[] = {{&S41013[42], 11}, {NULL}};
const Segment SGI77725[] = {{&S41013[42], 23}, {NULL}};
const Segment SGI77726[] = {{&S41013[30], 4}, {NULL}};
const Segment SGI77727[] = {{&S41013[114], 3}, {NULL}};
const Segment SGI77728[] = {{&S41013[79], 6}, {NULL}};
const Segment SGI77729[] = {{&S41014[36], 11}, {NULL}};
const Segment SGI77730[] = {{&S41014[31], 4}, {NULL}};
const Segment SGI77731[] = {{&S41015[36], 11}, {NULL}};
const Segment SGI77732[] = {{&S41015[31], 4}, {NULL}};
const Segment SGI77733[] = {{&S41016[42], 18}, {NULL}};
const Segment SGI77734[] = {{&S41016[42], 30}, {NULL}};
const Segment SGI77735[] = {{&S41016[30], 4}, {NULL}};
const Segment SGI77736[] = {{&S41016[121], 3}, {NULL}};
const Segment SGI77737[] = {{&S41016[86], 6}, {NULL}};
const Segment SGI77738[] = {{&S41023[42], 7}, {NULL}};
const Segment SGI77739[] = {{&S41023[42], 21}, {NULL}};
const Segment SGI77740[] = {{&S41023[30], 4}, {NULL}};
const Segment SGI77741[] = {{&S41023[112], 3}, {NULL}};
const Segment SGI77742[] = {{&S41023[77], 6}, {NULL}};
const Segment SGI77743[] = {{&S41024[42], 7}, {NULL}};
const Segment SGI77744[] = {{&S41024[42], 21}, {NULL}};
const Segment SGI77745[] = {{&S41024[30], 4}, {NULL}};
const Segment SGI77746[] = {{&S41024[112], 3}, {NULL}};
const Segment SGI77747[] = {{&S41024[77], 6}, {NULL}};
const Segment SGI77748[] = {{&S41025[42], 7}, {NULL}};
const Segment SGI77749[] = {{&S41025[42], 21}, {NULL}};
const Segment SGI77750[] = {{&S41025[30], 4}, {NULL}};
const Segment SGI77751[] = {{&S41025[112], 3}, {NULL}};
const Segment SGI77752[] = {{&S41025[77], 6}, {NULL}};
const Segment SGI77753[] = {{&S41026[36], 7}, {NULL}};
const Segment SGI77754[] = {{&S41026[31], 4}, {NULL}};
const Segment SGI77755[] = {{&S41027[42], 7}, {NULL}};
const Segment SGI77756[] = {{&S41027[42], 21}, {NULL}};
const Segment SGI77757[] = {{&S41027[30], 4}, {NULL}};
const Segment SGI77758[] = {{&S41027[112], 3}, {NULL}};
const Segment SGI77759[] = {{&S41027[77], 6}, {NULL}};
const Segment SGI77760[] = {{&S41969[44], 9}, {NULL}};
const Segment SGI77761[] = {{&S41969[44], 20}, {NULL}};
const Segment SGI77762[] = {{&S41969[30], 6}, {NULL}};
const Segment SGI77763[] = {{&S41969[112], 3}, {NULL}};
const Segment SGI77764[] = {{&S41969[78], 5}, {NULL}};
const Segment SGI77765[] = {{&S29861[44], 7}, {NULL}};
const Segment SGI77766[] = {{&S29861[44], 24}, {NULL}};
const Segment SGI77767[] = {{&S29861[30], 6}, {NULL}};
const Segment SGI77768[] = {{&S29861[31], 3}, {NULL}};
const Segment SGI77769[] = {{&S29861[82], 6}, {NULL}};
const Segment SGI77770[] = {{&S29862[44], 12}, {NULL}};
const Segment SGI77771[] = {{&S29862[44], 24}, {NULL}};
const Segment SGI77772[] = {{&S29862[30], 6}, {NULL}};
const Segment SGI77773[] = {{&S29862[31], 3}, {NULL}};
const Segment SGI77774[] = {{&S29862[82], 6}, {NULL}};
const Segment SGI77775[] = {{&S29863[44], 12}, {NULL}};
const Segment SGI77776[] = {{&S29863[44], 29}, {NULL}};
const Segment SGI77777[] = {{&S29863[30], 6}, {NULL}};
const Segment SGI77778[] = {{&S29863[31], 3}, {NULL}};
const Segment SGI77779[] = {{&S29863[87], 6}, {NULL}};
const Segment SGI77780[] = {{&S29864[38], 7}, {NULL}};
const Segment SGI77781[] = {{&S29864[31], 6}, {NULL}};
const Segment SGI77782[] = {{&S41155[38], 16}, {NULL}};
const Segment SGI77783[] = {{&S41155[31], 6}, {NULL}};
const Segment SGI77784[] = {{&S41156[44], 15}, {NULL}};
const Segment SGI77785[] = {{&S41156[44], 27}, {NULL}};
const Segment SGI77786[] = {{&S41156[30], 6}, {NULL}};
const Segment SGI77787[] = {{&S41156[120], 3}, {NULL}};
const Segment SGI77788[] = {{&S41156[85], 6}, {NULL}};
const Segment SGI77789[] = {{&S41157[44], 19}, {NULL}};
const Segment SGI77790[] = {{&S41157[44], 31}, {NULL}};
const Segment SGI77791[] = {{&S41157[30], 6}, {NULL}};
const Segment SGI77792[] = {{&S41157[124], 3}, {NULL}};
const Segment SGI77793[] = {{&S41157[89], 6}, {NULL}};
const Segment SGI77794[] = {{&S41158[42], 17}, {NULL}};
const Segment SGI77795[] = {{&S41158[42], 29}, {NULL}};
const Segment SGI77796[] = {{&S41158[30], 4}, {NULL}};
const Segment SGI77797[] = {{&S41158[120], 3}, {NULL}};
const Segment SGI77798[] = {{&S41158[85], 6}, {NULL}};
const Segment SGI77799[] = {{&S41159[44], 11}, {NULL}};
const Segment SGI77800[] = {{&S41159[44], 23}, {NULL}};
const Segment SGI77801[] = {{&S41159[30], 6}, {NULL}};
const Segment SGI77802[] = {{&S41159[116], 3}, {NULL}};
const Segment SGI77803[] = {{&S41160[42], 17}, {NULL}};
const Segment SGI77804[] = {{&S41160[30], 6}, {NULL}};
const Segment SGI77805[] = {{&S41160[120], 3}, {NULL}};
const Segment SGI77806[] = {{&S41160[85], 6}, {NULL}};
const Segment SGI77807[] = {{&S41161[44], 11}, {NULL}};
const Segment SGI77808[] = {{&S41161[44], 23}, {NULL}};
const Segment SGI77809[] = {{&S41161[30], 6}, {NULL}};
const Segment SGI77810[] = {{&S41161[116], 3}, {NULL}};
const Segment SGI77811[] = {{&S41161[81], 6}, {NULL}};
const Segment SGI77812[] = {{&S41162[44], 10}, {NULL}};
const Segment SGI77813[] = {{&S41162[44], 22}, {NULL}};
const Segment SGI77814[] = {{&S41162[30], 6}, {NULL}};
const Segment SGI77815[] = {{&S41162[115], 3}, {NULL}};
const Segment SGI77816[] = {{&S41162[80], 6}, {NULL}};
const Segment SGI77817[] = {{&S41163[38], 19}, {NULL}};
const Segment SGI77818[] = {{&S41163[31], 6}, {NULL}};
const Segment SGI77819[] = {{&S41164[44], 17}, {NULL}};
const Segment SGI77820[] = {{&S41164[44], 29}, {NULL}};
const Segment SGI77821[] = {{&S41164[30], 6}, {NULL}};
const Segment SGI77822[] = {{&S41164[122], 3}, {NULL}};
const Segment SGI77823[] = {{&S41164[87], 6}, {NULL}};
const Segment SGI77824[] = {{&S41165[44], 16}, {NULL}};
const Segment SGI77825[] = {{&S41165[44], 28}, {NULL}};
const Segment SGI77826[] = {{&S41165[30], 6}, {NULL}};
const Segment SGI77827[] = {{&S41165[121], 3}, {NULL}};
const Segment SGI77828[] = {{&S41165[86], 6}, {NULL}};
const Segment SGI77829[] = {{&S41166[44], 16}, {NULL}};
const Segment SGI77830[] = {{&S41166[44], 28}, {NULL}};
const Segment SGI77831[] = {{&S41166[30], 6}, {NULL}};
const Segment SGI77832[] = {{&S41166[121], 3}, {NULL}};
const Segment SGI77833[] = {{&S41166[86], 6}, {NULL}};
const Segment SGI77834[] = {{&S41167[38], 17}, {NULL}};
const Segment SGI77835[] = {{&S41167[31], 6}, {NULL}};
const Segment SGI77836[] = {{&S41168[44], 18}, {NULL}};
const Segment SGI77837[] = {{&S41168[44], 30}, {NULL}};
const Segment SGI77838[] = {{&S41168[30], 6}, {NULL}};
const Segment SGI77839[] = {{&S41168[123], 3}, {NULL}};
const Segment SGI77840[] = {{&S41168[88], 6}, {NULL}};
const Segment SGI77841[] = {{&S48036[44], 10}, {NULL}};
const Segment SGI77842[] = {{&S48036[44], 22}, {NULL}};
const Segment SGI77843[] = {{&S48036[30], 6}, {NULL}};
const Segment SGI77844[] = {{&S48036[114], 3}, {NULL}};
const Segment SGI77845[] = {{&S48036[80], 5}, {NULL}};
const Segment SGI77846[] = {{&S48037[44], 9}, {NULL}};
const Segment SGI77847[] = {{&S48037[44], 21}, {NULL}};
const Segment SGI77848[] = {{&S48037[30], 6}, {NULL}};
const Segment SGI77849[] = {{&S48037[113], 3}, {NULL}};
const Segment SGI77850[] = {{&S48037[79], 5}, {NULL}};
const Segment SGI77851[] = {{&S48038[44], 9}, {NULL}};
const Segment SGI77852[] = {{&S48038[44], 21}, {NULL}};
const Segment SGI77853[] = {{&S48038[30], 6}, {NULL}};
const Segment SGI77854[] = {{&S48038[113], 3}, {NULL}};
const Segment SGI77855[] = {{&S48038[79], 5}, {NULL}};
const Segment SGI77856[] = {{&S48039[44], 14}, {NULL}};
const Segment SGI77857[] = {{&S48039[44], 26}, {NULL}};
const Segment SGI77858[] = {{&S48039[30], 6}, {NULL}};
const Segment SGI77859[] = {{&S48039[118], 3}, {NULL}};
const Segment SGI77860[] = {{&S48039[84], 5}, {NULL}};
const Segment SGI77861[] = {{&S48040[38], 9}, {NULL}};
const Segment SGI77862[] = {{&S48040[31], 6}, {NULL}};
const Segment SGI77863[] = {{&S48041[44], 14}, {NULL}};
const Segment SGI77864[] = {{&S48041[44], 26}, {NULL}};
const Segment SGI77865[] = {{&S48041[30], 6}, {NULL}};
const Segment SGI77866[] = {{&S48041[118], 3}, {NULL}};
const Segment SGI77867[] = {{&S48041[84], 5}, {NULL}};
const Segment SGI77868[] = {{&S48042[44], 14}, {NULL}};
const Segment SGI77869[] = {{&S48042[44], 26}, {NULL}};
const Segment SGI77870[] = {{&S48042[30], 6}, {NULL}};
const Segment SGI77871[] = {{&S48042[118], 3}, {NULL}};
const Segment SGI77872[] = {{&S48042[84], 5}, {NULL}};
const Segment SGI77873[] = {{&S48043[44], 14}, {NULL}};
const Segment SGI77874[] = {{&S48043[44], 26}, {NULL}};
const Segment SGI77875[] = {{&S48043[30], 6}, {NULL}};
const Segment SGI77876[] = {{&S48043[118], 3}, {NULL}};
const Segment SGI77877[] = {{&S48043[84], 5}, {NULL}};
const Segment SGI77878[] = {{&S41195[44], 4}, {NULL}};
const Segment SGI77879[] = {{&S41195[44], 16}, {NULL}};
const Segment SGI77880[] = {{&S41195[30], 6}, {NULL}};
const Segment SGI77881[] = {{&S41195[109], 3}, {NULL}};
const Segment SGI77882[] = {{&S41195[74], 6}, {NULL}};
const Segment SGI77883[] = {{&S41196[38], 4}, {NULL}};
const Segment SGI77884[] = {{&S41196[31], 6}, {NULL}};
const Segment SGI77885[] = {{&S30044[44], 12}, {NULL}};
const Segment SGI77886[] = {{&S30044[44], 38}, {NULL}};
const Segment SGI77887[] = {{&S30044[30], 6}, {NULL}};
const Segment SGI77888[] = {{&S30044[31], 3}, {NULL}};
const Segment SGI77889[] = {{&S30044[96], 6}, {NULL}};
const Segment SGI77890[] = {{&S30046[44], 15}, {NULL}};
const Segment SGI77891[] = {{&S30046[44], 42}, {NULL}};
const Segment SGI77892[] = {{&S30046[30], 6}, {NULL}};
const Segment SGI77893[] = {{&S30046[31], 3}, {NULL}};
const Segment SGI77894[] = {{&S30046[100], 6}, {NULL}};
const Segment SGI77895[] = {{&S30047[44], 15}, {NULL}};
const Segment SGI77896[] = {{&S30047[44], 41}, {NULL}};
const Segment SGI77897[] = {{&S30047[30], 6}, {NULL}};
const Segment SGI77898[] = {{&S30047[31], 3}, {NULL}};
const Segment SGI77899[] = {{&S30047[99], 6}, {NULL}};
const Segment SGI77900[] = {{&S30048[44], 15}, {NULL}};
const Segment SGI77901[] = {{&S30048[44], 42}, {NULL}};
const Segment SGI77902[] = {{&S30048[30], 6}, {NULL}};
const Segment SGI77903[] = {{&S30048[31], 3}, {NULL}};
const Segment SGI77904[] = {{&S30048[100], 6}, {NULL}};
const Segment SGI77905[] = {{&S30049[44], 12}, {NULL}};
const Segment SGI77906[] = {{&S30049[44], 39}, {NULL}};
const Segment SGI77907[] = {{&S30049[30], 6}, {NULL}};
const Segment SGI77908[] = {{&S30049[31], 3}, {NULL}};
const Segment SGI77909[] = {{&S30049[97], 6}, {NULL}};
const Segment SGI77910[] = {{&S30051[44], 15}, {NULL}};
const Segment SGI77911[] = {{&S30051[44], 42}, {NULL}};
const Segment SGI77912[] = {{&S30051[30], 6}, {NULL}};
const Segment SGI77913[] = {{&S30051[31], 3}, {NULL}};
const Segment SGI77914[] = {{&S30051[100], 6}, {NULL}};
const Segment SGI77915[] = {{&S30053[44], 12}, {NULL}};
const Segment SGI77916[] = {{&S30053[44], 24}, {NULL}};
const Segment SGI77917[] = {{&S30053[30], 6}, {NULL}};
const Segment SGI77918[] = {{&S30053[31], 3}, {NULL}};
const Segment SGI77919[] = {{&S30053[82], 6}, {NULL}};
const Segment SGI77920[] = {{&S30054[44], 12}, {NULL}};
const Segment SGI77921[] = {{&S30054[44], 39}, {NULL}};
const Segment SGI77922[] = {{&S30054[30], 6}, {NULL}};
const Segment SGI77923[] = {{&S30054[31], 3}, {NULL}};
const Segment SGI77924[] = {{&S30054[97], 6}, {NULL}};
const Segment SGI77925[] = {{&S30069[38], 12}, {NULL}};
const Segment SGI77926[] = {{&S30069[31], 6}, {NULL}};
const Segment SGI77927[] = {{&S48056[44], 23}, {NULL}};
const Segment SGI77928[] = {{&S48056[44], 35}, {NULL}};
const Segment SGI77929[] = {{&S48056[30], 6}, {NULL}};
const Segment SGI77930[] = {{&S48056[127], 3}, {NULL}};
const Segment SGI77931[] = {{&S48056[93], 5}, {NULL}};
const Segment SGI77932[] = {{&S48057[44], 24}, {NULL}};
const Segment SGI77933[] = {{&S48057[44], 36}, {NULL}};
const Segment SGI77934[] = {{&S48057[30], 6}, {NULL}};
const Segment SGI77935[] = {{&S48057[128], 3}, {NULL}};
const Segment SGI77936[] = {{&S48057[94], 5}, {NULL}};
const Segment SGI77937[] = {{&S48058[44], 24}, {NULL}};
const Segment SGI77938[] = {{&S48058[44], 36}, {NULL}};
const Segment SGI77939[] = {{&S48058[30], 6}, {NULL}};
const Segment SGI77940[] = {{&S48058[128], 3}, {NULL}};
const Segment SGI77941[] = {{&S48058[94], 5}, {NULL}};
const Segment SGI77942[] = {{&S48059[44], 23}, {NULL}};
const Segment SGI77943[] = {{&S48059[44], 35}, {NULL}};
const Segment SGI77944[] = {{&S48059[30], 6}, {NULL}};
const Segment SGI77945[] = {{&S48059[127], 3}, {NULL}};
const Segment SGI77946[] = {{&S48059[93], 5}, {NULL}};
const Segment SGI77947[] = {{&S42191[44], 14}, {NULL}};
const Segment SGI77948[] = {{&S42191[44], 25}, {NULL}};
const Segment SGI77949[] = {{&S42191[30], 6}, {NULL}};
const Segment SGI77950[] = {{&S42191[117], 3}, {NULL}};
const Segment SGI77951[] = {{&S42191[83], 5}, {NULL}};
const Segment SGI77952[] = {{&S48243[44], 24}, {NULL}};
const Segment SGI77953[] = {{&S48243[44], 36}, {NULL}};
const Segment SGI77954[] = {{&S48243[30], 6}, {NULL}};
const Segment SGI77955[] = {{&S48243[128], 3}, {NULL}};
const Segment SGI77956[] = {{&S48243[94], 5}, {NULL}};
const Segment SGI77957[] = {{&S41513[44], 19}, {NULL}};
const Segment SGI77958[] = {{&S41513[44], 30}, {NULL}};
const Segment SGI77959[] = {{&S41513[30], 6}, {NULL}};
const Segment SGI77960[] = {{&S41513[123], 3}, {NULL}};
const Segment SGI77961[] = {{&S41513[88], 6}, {NULL}};
const Segment SGI77962[] = {{&S41514[42], 19}, {NULL}};
const Segment SGI77963[] = {{&S41514[42], 31}, {NULL}};
const Segment SGI77964[] = {{&S41514[30], 4}, {NULL}};
const Segment SGI77965[] = {{&S41514[122], 3}, {NULL}};
const Segment SGI77966[] = {{&S41514[87], 6}, {NULL}};
const Segment SGI77967[] = {{&S41515[44], 19}, {NULL}};
const Segment SGI77968[] = {{&S41515[44], 30}, {NULL}};
const Segment SGI77969[] = {{&S41515[30], 6}, {NULL}};
const Segment SGI77970[] = {{&S41515[123], 3}, {NULL}};
const Segment SGI77971[] = {{&S41515[88], 6}, {NULL}};
const Segment SGI77972[] = {{&S41516[38], 19}, {NULL}};
const Segment SGI77973[] = {{&S41516[31], 6}, {NULL}};
const Segment SGI77974[] = {{&S41517[36], 19}, {NULL}};
const Segment SGI77975[] = {{&S41517[31], 4}, {NULL}};
const Segment SGI77976[] = {{&S41518[61], 19}, {NULL}};
const Segment SGI77977[] = {{&S41518[61], 30}, {NULL}};
const Segment SGI77978[] = {{&S41518[30], 23}, {NULL}};
const Segment SGI77979[] = {{&S41518[140], 3}, {NULL}};
const Segment SGI77980[] = {{&S41518[105], 6}, {NULL}};
const Segment SGI77981[] = {{&S41519[44], 13}, {NULL}};
const Segment SGI77982[] = {{&S41519[44], 24}, {NULL}};
const Segment SGI77983[] = {{&S41519[30], 6}, {NULL}};
const Segment SGI77984[] = {{&S41519[117], 3}, {NULL}};
const Segment SGI77985[] = {{&S41519[82], 6}, {NULL}};
const Segment SGI77986[] = {{&S48326[6], 4}, {NULL}};
const Segment SGI77987[] = {{&S48326[76], 5}, {NULL}};
const Segment SGI77988[] = {{&S30260[44], 12}, {NULL}};
const Segment SGI77989[] = {{&S30260[44], 24}, {NULL}};
const Segment SGI77990[] = {{&S30260[30], 6}, {NULL}};
const Segment SGI77991[] = {{&S30260[31], 3}, {NULL}};
const Segment SGI77992[] = {{&S30260[82], 6}, {NULL}};
const Segment SGI77993[] = {{&S41715[42], 6}, {NULL}};
const Segment SGI77994[] = {{&S41715[42], 17}, {NULL}};
const Segment SGI77995[] = {{&S41715[30], 4}, {NULL}};
const Segment SGI77996[] = {{&S41715[108], 3}, {NULL}};
const Segment SGI77997[] = {{&S41715[73], 6}, {NULL}};
const Segment SGI77998[] = {{&S30293[44], 14}, {NULL}};
const Segment SGI77999[] = {{&S30293[44], 26}, {NULL}};
const Segment SGI78000[] = {{&S30293[30], 6}, {NULL}};
const Segment SGI78001[] = {{&S30293[31], 3}, {NULL}};
const Segment SGI78002[] = {{&S30293[84], 6}, {NULL}};
const Segment SGI78003[] = {{&S41716[42], 7}, {NULL}};
const Segment SGI78004[] = {{&S41716[42], 23}, {NULL}};
const Segment SGI78005[] = {{&S41716[30], 4}, {NULL}};
const Segment SGI78006[] = {{&S41716[114], 3}, {NULL}};
const Segment SGI78007[] = {{&S41716[79], 6}, {NULL}};
const Segment SGI78008[] = {{&S30294[38], 14}, {NULL}};
const Segment SGI78009[] = {{&S30294[31], 6}, {NULL}};
const Segment SGI78010[] = {{&S41717[42], 6}, {NULL}};
const Segment SGI78011[] = {{&S41717[42], 18}, {NULL}};
const Segment SGI78012[] = {{&S41717[30], 4}, {NULL}};
const Segment SGI78013[] = {{&S30295[44], 14}, {NULL}};
const Segment SGI78014[] = {{&S41717[109], 3}, {NULL}};
const Segment SGI78015[] = {{&S30295[44], 25}, {NULL}};
const Segment SGI78016[] = {{&S30295[30], 6}, {NULL}};
const Segment SGI78017[] = {{&S41717[74], 6}, {NULL}};
const Segment SGI78018[] = {{&S30295[31], 3}, {NULL}};
const Segment SGI78019[] = {{&S30295[83], 6}, {NULL}};
const Segment SGI78020[] = {{&S41718[42], 6}, {NULL}};
const Segment SGI78021[] = {{&S41718[42], 17}, {NULL}};
const Segment SGI78022[] = {{&S41718[30], 4}, {NULL}};
const Segment SGI78023[] = {{&S30296[44], 14}, {NULL}};
const Segment SGI78024[] = {{&S30296[44], 25}, {NULL}};
const Segment SGI78025[] = {{&S30296[30], 6}, {NULL}};
const Segment SGI78026[] = {{&S41718[108], 3}, {NULL}};
const Segment SGI78027[] = {{&S41718[73], 6}, {NULL}};
const Segment SGI78028[] = {{&S30296[31], 3}, {NULL}};
const Segment SGI78029[] = {{&S30296[83], 6}, {NULL}};
const Segment SGI78030[] = {{&S41719[42], 6}, {NULL}};
const Segment SGI78031[] = {{&S41719[42], 17}, {NULL}};
const Segment SGI78032[] = {{&S41719[30], 4}, {NULL}};
const Segment SGI78033[] = {{&S30297[44], 14}, {NULL}};
const Segment SGI78034[] = {{&S30297[44], 26}, {NULL}};
const Segment SGI78035[] = {{&S30297[30], 6}, {NULL}};
const Segment SGI78036[] = {{&S41719[108], 3}, {NULL}};
const Segment SGI78037[] = {{&S41719[73], 6}, {NULL}};
const Segment SGI78038[] = {{&S30297[84], 5}, {NULL}};
const Segment SGI78039[] = {{&S41720[42], 6}, {NULL}};
const Segment SGI78040[] = {{&S41720[42], 17}, {NULL}};
const Segment SGI78041[] = {{&S30298[44], 14}, {NULL}};
const Segment SGI78042[] = {{&S41720[30], 4}, {NULL}};
const Segment SGI78043[] = {{&S30298[44], 26}, {NULL}};
const Segment SGI78044[] = {{&S30298[30], 6}, {NULL}};
const Segment SGI78045[] = {{&S41720[108], 3}, {NULL}};
const Segment SGI78046[] = {{&S30298[31], 3}, {NULL}};
const Segment SGI78047[] = {{&S41720[73], 6}, {NULL}};
const Segment SGI78048[] = {{&S30298[84], 6}, {NULL}};
const Segment SGI78049[] = {{&S41721[42], 6}, {NULL}};
const Segment SGI78050[] = {{&S41721[42], 17}, {NULL}};
const Segment SGI78051[] = {{&S41721[30], 4}, {NULL}};
const Segment SGI78052[] = {{&S30299[44], 13}, {NULL}};
const Segment SGI78053[] = {{&S30299[44], 25}, {NULL}};
const Segment SGI78054[] = {{&S30299[30], 6}, {NULL}};
const Segment SGI78055[] = {{&S41721[108], 3}, {NULL}};
const Segment SGI78056[] = {{&S41721[73], 6}, {NULL}};
const Segment SGI78057[] = {{&S30299[31], 3}, {NULL}};
const Segment SGI78058[] = {{&S30299[83], 6}, {NULL}};
const Segment SGI78059[] = {{&S41722[42], 6}, {NULL}};
const Segment SGI78060[] = {{&S41722[42], 18}, {NULL}};
const Segment SGI78061[] = {{&S41722[30], 4}, {NULL}};
const Segment SGI78062[] = {{&S41722[109], 3}, {NULL}};
const Segment SGI78063[] = {{&S41722[74], 6}, {NULL}};
const Segment SGI78064[] = {{&S41723[42], 6}, {NULL}};
const Segment SGI78065[] = {{&S41723[42], 17}, {NULL}};
const Segment SGI78066[] = {{&S41723[30], 4}, {NULL}};
const Segment SGI78067[] = {{&S41723[108], 3}, {NULL}};
const Segment SGI78068[] = {{&S41723[73], 6}, {NULL}};
const Segment SGI78069[] = {{&S30303[44], 13}, {NULL}};
const Segment SGI78070[] = {{&S30303[44], 25}, {NULL}};
const Segment SGI78071[] = {{&S30303[30], 6}, {NULL}};
const Segment SGI78072[] = {{&S30303[31], 3}, {NULL}};
const Segment SGI78073[] = {{&S30303[83], 6}, {NULL}};
const Segment SGI78074[] = {{&S41724[42], 6}, {NULL}};
const Segment SGI78075[] = {{&S41724[42], 17}, {NULL}};
const Segment SGI78076[] = {{&S41724[30], 4}, {NULL}};
const Segment SGI78077[] = {{&S41724[108], 3}, {NULL}};
const Segment SGI78078[] = {{&S41724[73], 6}, {NULL}};
const Segment SGI78079[] = {{&S41725[42], 6}, {NULL}};
const Segment SGI78080[] = {{&S41725[42], 18}, {NULL}};
const Segment SGI78081[] = {{&S41725[30], 4}, {NULL}};
const Segment SGI78082[] = {{&S41725[109], 3}, {NULL}};
const Segment SGI78083[] = {{&S41725[74], 6}, {NULL}};
const Segment SGI78084[] = {{&S41726[42], 6}, {NULL}};
const Segment SGI78085[] = {{&S41726[42], 18}, {NULL}};
const Segment SGI78086[] = {{&S41726[30], 4}, {NULL}};
const Segment SGI78087[] = {{&S41726[109], 3}, {NULL}};
const Segment SGI78088[] = {{&S41726[74], 6}, {NULL}};
const Segment SGI78089[] = {{&S41727[42], 6}, {NULL}};
const Segment SGI78090[] = {{&S41727[42], 18}, {NULL}};
const Segment SGI78091[] = {{&S41727[30], 4}, {NULL}};
const Segment SGI78092[] = {{&S41727[109], 3}, {NULL}};
const Segment SGI78093[] = {{&S41727[74], 6}, {NULL}};
const Segment SGI78094[] = {{&S41728[42], 6}, {NULL}};
const Segment SGI78095[] = {{&S41728[42], 18}, {NULL}};
const Segment SGI78096[] = {{&S41728[30], 4}, {NULL}};
const Segment SGI78097[] = {{&S41728[109], 3}, {NULL}};
const Segment SGI78098[] = {{&S41728[74], 6}, {NULL}};
const Segment SGI78099[] = {{&S41729[40], 6}, {NULL}};
const Segment SGI78100[] = {{&S41729[30], 4}, {NULL}};
const Segment SGI78101[] = {{&S41729[106], 3}, {NULL}};
const Segment SGI78102[] = {{&S41729[71], 6}, {NULL}};
const Segment SGI78103[] = {{&S41730[42], 6}, {NULL}};
const Segment SGI78104[] = {{&S41730[42], 17}, {NULL}};
const Segment SGI78105[] = {{&S41730[30], 4}, {NULL}};
const Segment SGI78106[] = {{&S41730[108], 3}, {NULL}};
const Segment SGI78107[] = {{&S30318[44], 14}, {NULL}};
const Segment SGI78108[] = {{&S30318[44], 26}, {NULL}};
const Segment SGI78109[] = {{&S41730[73], 6}, {NULL}};
const Segment SGI78110[] = {{&S30318[30], 6}, {NULL}};
const Segment SGI78111[] = {{&S30318[31], 3}, {NULL}};
const Segment SGI78112[] = {{&S30318[84], 6}, {NULL}};
const Segment SGI78113[] = {{&S41731[42], 6}, {NULL}};
const Segment SGI78114[] = {{&S41731[42], 17}, {NULL}};
const Segment SGI78115[] = {{&S41731[30], 4}, {NULL}};
const Segment SGI78116[] = {{&S41731[108], 3}, {NULL}};
const Segment SGI78117[] = {{&S41731[73], 6}, {NULL}};
const Segment SGI78118[] = {{&S41732[42], 6}, {NULL}};
const Segment SGI78119[] = {{&S41732[42], 17}, {NULL}};
const Segment SGI78120[] = {{&S41732[30], 4}, {NULL}};
const Segment SGI78121[] = {{&S41732[108], 3}, {NULL}};
const Segment SGI78122[] = {{&S41732[73], 6}, {NULL}};
const Segment SGI78123[] = {{&S41733[36], 7}, {NULL}};
const Segment SGI78124[] = {{&S41733[31], 4}, {NULL}};
const Segment SGI78125[] = {{&S30327[44], 14}, {NULL}};
const Segment SGI78126[] = {{&S30327[44], 26}, {NULL}};
const Segment SGI78127[] = {{&S30327[30], 6}, {NULL}};
const Segment SGI78128[] = {{&S30327[31], 3}, {NULL}};
const Segment SGI78129[] = {{&S30327[84], 6}, {NULL}};
const Segment SGI78130[] = {{&S30342[44], 14}, {NULL}};
const Segment SGI78131[] = {{&S30342[44], 26}, {NULL}};
const Segment SGI78132[] = {{&S30342[30], 6}, {NULL}};
const Segment SGI78133[] = {{&S30342[31], 3}, {NULL}};
const Segment SGI78134[] = {{&S30342[84], 6}, {NULL}};
const Segment SGI78135[] = {{&S30348[44], 22}, {NULL}};
const Segment SGI78136[] = {{&S30348[44], 34}, {NULL}};
const Segment SGI78137[] = {{&S30348[30], 6}, {NULL}};
const Segment SGI78138[] = {{&S30348[31], 3}, {NULL}};
const Segment SGI78139[] = {{&S30348[92], 6}, {NULL}};
const Segment SGI78140[] = {{&S30349[44], 14}, {NULL}};
const Segment SGI78141[] = {{&S30349[44], 25}, {NULL}};
const Segment SGI78142[] = {{&S30349[30], 6}, {NULL}};
const Segment SGI78143[] = {{&S30349[31], 3}, {NULL}};
const Segment SGI78144[] = {{&S30349[83], 6}, {NULL}};
const Segment SGI78145[] = {{&S30353[44], 14}, {NULL}};
const Segment SGI78146[] = {{&S30353[44], 23}, {NULL}};
const Segment SGI78147[] = {{&S30353[30], 6}, {NULL}};
const Segment SGI78148[] = {{&S30353[31], 3}, {NULL}};
const Segment SGI78149[] = {{&S30353[81], 6}, {NULL}};
const Segment SGI78150[] = {{&S30356[44], 14}, {NULL}};
const Segment SGI78151[] = {{&S30356[44], 25}, {NULL}};
const Segment SGI78152[] = {{&S30356[30], 6}, {NULL}};
const Segment SGI78153[] = {{&S30356[31], 3}, {NULL}};
const Segment SGI78154[] = {{&S30356[83], 6}, {NULL}};
const Segment SGI78155[] = {{&S30358[44], 14}, {NULL}};
const Segment SGI78156[] = {{&S30358[44], 26}, {NULL}};
const Segment SGI78157[] = {{&S30358[30], 6}, {NULL}};
const Segment SGI78158[] = {{&S30358[31], 3}, {NULL}};
const Segment SGI78159[] = {{&S30358[84], 6}, {NULL}};
const Segment SGI78160[] = {{&S30359[44], 14}, {NULL}};
const Segment SGI78161[] = {{&S30359[44], 28}, {NULL}};
const Segment SGI78162[] = {{&S30359[30], 6}, {NULL}};
const Segment SGI78163[] = {{&S30359[31], 3}, {NULL}};
const Segment SGI78164[] = {{&S30359[86], 6}, {NULL}};
const Segment SGI78165[] = {{&S35602[44], 9}, {NULL}};
const Segment SGI78166[] = {{&S35602[44], 20}, {NULL}};
const Segment SGI78167[] = {{&S35602[30], 6}, {NULL}};
const Segment SGI78168[] = {{&S35602[31], 3}, {NULL}};
const Segment SGI78169[] = {{&S35602[78], 6}, {NULL}};
const Segment SGI78170[] = {{&S35602[0], 13}, {NULL}};
const Segment SGI78171[] = {{&S30418[44], 11}, {NULL}};
const Segment SGI78172[] = {{&S30418[44], 23}, {NULL}};
const Segment SGI78173[] = {{&S30418[30], 6}, {NULL}};
const Segment SGI78174[] = {{&S30418[31], 3}, {NULL}};
const Segment SGI78175[] = {{&S30418[81], 6}, {NULL}};
const Segment SGI78176[] = {{&S30419[44], 16}, {NULL}};
const Segment SGI78177[] = {{&S30419[44], 28}, {NULL}};
const Segment SGI78178[] = {{&S30419[30], 6}, {NULL}};
const Segment SGI78179[] = {{&S30419[31], 3}, {NULL}};
const Segment SGI78180[] = {{&S30419[86], 6}, {NULL}};
const Segment SGI78181[] = {{&S30420[38], 16}, {NULL}};
const Segment SGI78182[] = {{&S30420[31], 6}, {NULL}};
const Segment SGI78183[] = {{&S30421[44], 18}, {NULL}};
const Segment SGI78184[] = {{&S30421[44], 30}, {NULL}};
const Segment SGI78185[] = {{&S30421[30], 6}, {NULL}};
const Segment SGI78186[] = {{&S30421[31], 3}, {NULL}};
const Segment SGI78187[] = {{&S30421[88], 6}, {NULL}};
const Segment SGI78188[] = {{&S30422[44], 9}, {NULL}};
const Segment SGI78189[] = {{&S30422[44], 21}, {NULL}};
const Segment SGI78190[] = {{&S30422[30], 6}, {NULL}};
const Segment SGI78191[] = {{&S30422[31], 3}, {NULL}};
const Segment SGI78192[] = {{&S30422[79], 6}, {NULL}};
const Segment SGI78193[] = {{&S30423[38], 9}, {NULL}};
const Segment SGI78194[] = {{&S30423[31], 6}, {NULL}};
const Segment SGI78195[] = {{&S30485[44], 6}, {NULL}};
const Segment SGI78196[] = {{&S30485[44], 18}, {NULL}};
const Segment SGI78197[] = {{&S30485[30], 6}, {NULL}};
const Segment SGI78198[] = {{&S30485[31], 3}, {NULL}};
const Segment SGI78199[] = {{&S30485[76], 6}, {NULL}};
const Segment SGI78200[] = {{&S30486[38], 6}, {NULL}};
const Segment SGI78201[] = {{&S30486[31], 6}, {NULL}};
const Segment SGI78202[] = {{&S30487[44], 6}, {NULL}};
const Segment SGI78203[] = {{&S30487[44], 18}, {NULL}};
const Segment SGI78204[] = {{&S30487[30], 6}, {NULL}};
const Segment SGI78205[] = {{&S30487[31], 3}, {NULL}};
const Segment SGI78206[] = {{&S30487[76], 6}, {NULL}};
const Segment SGI78207[] = {{&S30488[42], 6}, {NULL}};
const Segment SGI78208[] = {{&S30488[30], 6}, {NULL}};
const Segment SGI78209[] = {{&S30488[31], 3}, {NULL}};
const Segment SGI78210[] = {{&S30488[74], 6}, {NULL}};
const Segment SGI78211[] = {{&S48757[44], 10}, {NULL}};
const Segment SGI78212[] = {{&S48757[44], 22}, {NULL}};
const Segment SGI78213[] = {{&S48757[30], 6}, {NULL}};
const Segment SGI78214[] = {{&S48757[114], 3}, {NULL}};
const Segment SGI78215[] = {{&S48757[80], 5}, {NULL}};
const Segment SGI78216[] = {{&S48758[38], 10}, {NULL}};
const Segment SGI78217[] = {{&S48758[31], 6}, {NULL}};
const Segment SGI78218[] = {{&S48992[44], 11}, {NULL}};
const Segment SGI78219[] = {{&S48992[44], 28}, {NULL}};
const Segment SGI78220[] = {{&S48992[30], 6}, {NULL}};
const Segment SGI78221[] = {{&S48992[120], 3}, {NULL}};
const Segment SGI78222[] = {{&S48992[86], 5}, {NULL}};
const Segment SGI78223[] = {{&S48993[38], 11}, {NULL}};
const Segment SGI78224[] = {{&S48993[31], 6}, {NULL}};
const Segment SGI78225[] = {{&S49042[44], 7}, {NULL}};
const Segment SGI78226[] = {{&S49042[44], 33}, {NULL}};
const Segment SGI78227[] = {{&S49042[30], 6}, {NULL}};
const Segment SGI78228[] = {{&S49042[125], 3}, {NULL}};
const Segment SGI78229[] = {{&S49042[91], 5}, {NULL}};
const Segment SGI78230[] = {{&S49047[44], 7}, {NULL}};
const Segment SGI78231[] = {{&S49047[44], 33}, {NULL}};
const Segment SGI78232[] = {{&S49047[30], 6}, {NULL}};
const Segment SGI78233[] = {{&S49047[125], 3}, {NULL}};
const Segment SGI78234[] = {{&S49047[91], 5}, {NULL}};
const Segment SGI78235[] = {{&S49048[44], 7}, {NULL}};
const Segment SGI78236[] = {{&S49048[44], 33}, {NULL}};
const Segment SGI78237[] = {{&S49048[30], 6}, {NULL}};
const Segment SGI78238[] = {{&S49048[125], 3}, {NULL}};
const Segment SGI78239[] = {{&S49048[91], 5}, {NULL}};
const Segment SGI78240[] = {{&S49049[44], 7}, {NULL}};
const Segment SGI78241[] = {{&S49049[44], 33}, {NULL}};
const Segment SGI78242[] = {{&S49049[30], 6}, {NULL}};
const Segment SGI78243[] = {{&S49049[125], 3}, {NULL}};
const Segment SGI78244[] = {{&S49049[91], 5}, {NULL}};
const Segment SGI78245[] = {{&S49052[44], 7}, {NULL}};
const Segment SGI78246[] = {{&S49052[44], 36}, {NULL}};
const Segment SGI78247[] = {{&S49052[30], 6}, {NULL}};
const Segment SGI78248[] = {{&S49052[128], 3}, {NULL}};
const Segment SGI78249[] = {{&S49052[94], 5}, {NULL}};
const Segment SGI78250[] = {{&S49262[44], 11}, {NULL}};
const Segment SGI78251[] = {{&S49262[44], 28}, {NULL}};
const Segment SGI78252[] = {{&S49262[30], 6}, {NULL}};
const Segment SGI78253[] = {{&S49262[120], 3}, {NULL}};
const Segment SGI78254[] = {{&S49262[86], 5}, {NULL}};
const Segment SGI78255[] = {{&S49263[38], 11}, {NULL}};
const Segment SGI78256[] = {{&S49263[31], 6}, {NULL}};
const Segment SGI78257[] = {{&S49282[44], 11}, {NULL}};
const Segment SGI78258[] = {{&S49282[44], 23}, {NULL}};
const Segment SGI78259[] = {{&S49282[30], 6}, {NULL}};
const Segment SGI78260[] = {{&S49282[115], 3}, {NULL}};
const Segment SGI78261[] = {{&S49282[81], 5}, {NULL}};
const Segment SGI78262[] = {{&S49283[38], 11}, {NULL}};
const Segment SGI78263[] = {{&S49283[31], 6}, {NULL}};
const Segment SGI78264[] = {{&S49284[44], 11}, {NULL}};
const Segment SGI78265[] = {{&S49284[44], 23}, {NULL}};
const Segment SGI78266[] = {{&S49284[30], 6}, {NULL}};
const Segment SGI78267[] = {{&S49284[115], 3}, {NULL}};
const Segment SGI78268[] = {{&S49284[81], 5}, {NULL}};
const Segment SGI78269[] = {{&S49386[44], 12}, {NULL}};
const Segment SGI78270[] = {{&S49386[44], 24}, {NULL}};
const Segment SGI78271[] = {{&S49386[30], 6}, {NULL}};
const Segment SGI78272[] = {{&S49386[116], 3}, {NULL}};
const Segment SGI78273[] = {{&S49386[82], 5}, {NULL}};
const Segment SGI78274[] = {{&S49387[38], 12}, {NULL}};
const Segment SGI78275[] = {{&S49387[31], 6}, {NULL}};
const Segment SGI78276[] = {{&S43427[44], 24}, {NULL}};
const Segment SGI78277[] = {{&S43427[44], 35}, {NULL}};
const Segment SGI78278[] = {{&S43427[30], 6}, {NULL}};
const Segment SGI78279[] = {{&S43427[127], 3}, {NULL}};
const Segment SGI78280[] = {{&S43427[93], 5}, {NULL}};
const Segment SGI78281[] = {{&S43430[44], 13}, {NULL}};
const Segment SGI78282[] = {{&S43430[44], 24}, {NULL}};
const Segment SGI78283[] = {{&S43430[30], 6}, {NULL}};
const Segment SGI78284[] = {{&S43430[116], 3}, {NULL}};
const Segment SGI78285[] = {{&S43430[82], 5}, {NULL}};
const Segment SGI78286[] = {{&S43431[44], 13}, {NULL}};
const Segment SGI78287[] = {{&S43431[44], 24}, {NULL}};
const Segment SGI78288[] = {{&S43431[30], 6}, {NULL}};
const Segment SGI78289[] = {{&S43431[116], 3}, {NULL}};
const Segment SGI78290[] = {{&S43431[82], 5}, {NULL}};
const Segment SGI78291[] = {{&S43609[44], 11}, {NULL}};
const Segment SGI78292[] = {{&S43609[44], 42}, {NULL}};
const Segment SGI78293[] = {{&S43609[30], 6}, {NULL}};
const Segment SGI78294[] = {{&S43609[134], 3}, {NULL}};
const Segment SGI78295[] = {{&S43609[100], 5}, {NULL}};
const Segment SGI78296[] = {{&S43611[44], 11}, {NULL}};
const Segment SGI78297[] = {{&S43611[44], 33}, {NULL}};
const Segment SGI78298[] = {{&S43611[30], 6}, {NULL}};
const Segment SGI78299[] = {{&S43611[125], 3}, {NULL}};
const Segment SGI78300[] = {{&S43611[91], 5}, {NULL}};
const Segment SGI78301[] = {{&S49717[44], 5}, {NULL}};
const Segment SGI78302[] = {{&S49717[44], 17}, {NULL}};
const Segment SGI78303[] = {{&S49717[30], 6}, {NULL}};
const Segment SGI78304[] = {{&S49717[109], 3}, {NULL}};
const Segment SGI78305[] = {{&S49717[75], 5}, {NULL}};
const Segment SGI78306[] = {{&S49718[44], 10}, {NULL}};
const Segment SGI78307[] = {{&S49718[44], 22}, {NULL}};
const Segment SGI78308[] = {{&S49718[30], 6}, {NULL}};
const Segment SGI78309[] = {{&S49718[114], 3}, {NULL}};
const Segment SGI78310[] = {{&S49718[80], 5}, {NULL}};
const Segment SGI78311[] = {{&S49719[38], 5}, {NULL}};
const Segment SGI78312[] = {{&S49719[31], 6}, {NULL}};
const Segment SGI78313[] = {{&S37070[44], 13}, {NULL}};
const Segment SGI78314[] = {{&S37070[44], 34}, {NULL}};
const Segment SGI78315[] = {{&S37070[30], 6}, {NULL}};
const Segment SGI78316[] = {{&S37070[31], 3}, {NULL}};
const Segment SGI78317[] = {{&S37070[92], 5}, {NULL}};
const Segment SGI78318[] = {{&S37070[0], 13}, {NULL}};
const Segment SGI78319[] = {{&S31752[44], 10}, {NULL}};
const Segment SGI78320[] = {{&S31752[44], 22}, {NULL}};
const Segment SGI78321[] = {{&S31752[30], 6}, {NULL}};
const Segment SGI78322[] = {{&S31752[31], 3}, {NULL}};
const Segment SGI78323[] = {{&S31752[80], 6}, {NULL}};
const Segment SGI78324[] = {{&S31753[44], 14}, {NULL}};
const Segment SGI78325[] = {{&S31753[44], 26}, {NULL}};
const Segment SGI78326[] = {{&S31753[30], 6}, {NULL}};
const Segment SGI78327[] = {{&S31753[31], 3}, {NULL}};
const Segment SGI78328[] = {{&S31753[84], 6}, {NULL}};
const Segment SGI78329[] = {{&S31754[38], 14}, {NULL}};
const Segment SGI78330[] = {{&S31754[31], 6}, {NULL}};
const Segment SGI78331[] = {{&S31755[44], 10}, {NULL}};
const Segment SGI78332[] = {{&S31755[44], 22}, {NULL}};
const Segment SGI78333[] = {{&S31755[30], 6}, {NULL}};
const Segment SGI78334[] = {{&S31755[31], 3}, {NULL}};
const Segment SGI78335[] = {{&S31755[80], 6}, {NULL}};
const Segment SGI78336[] = {{&S31756[44], 10}, {NULL}};
const Segment SGI78337[] = {{&S31756[44], 22}, {NULL}};
const Segment SGI78338[] = {{&S31756[30], 6}, {NULL}};
const Segment SGI78339[] = {{&S31756[31], 3}, {NULL}};
const Segment SGI78340[] = {{&S31756[80], 6}, {NULL}};
const Segment SGI78341[] = {{&S31757[44], 14}, {NULL}};
const Segment SGI78342[] = {{&S31757[44], 26}, {NULL}};
const Segment SGI78343[] = {{&S31757[30], 6}, {NULL}};
const Segment SGI78344[] = {{&S31757[31], 3}, {NULL}};
const Segment SGI78345[] = {{&S31757[84], 6}, {NULL}};
const Segment SGI78346[] = {{&S31758[38], 10}, {NULL}};
const Segment SGI78347[] = {{&S31758[31], 6}, {NULL}};
const Segment SGI78348[] = {{&S31759[38], 14}, {NULL}};
const Segment SGI78349[] = {{&S31759[31], 6}, {NULL}};
const Segment SGI78350[] = {{&S31762[44], 14}, {NULL}};
const Segment SGI78351[] = {{&S31762[44], 35}, {NULL}};
const Segment SGI78352[] = {{&S31762[30], 6}, {NULL}};
const Segment SGI78353[] = {{&S31762[31], 3}, {NULL}};
const Segment SGI78354[] = {{&S31762[93], 6}, {NULL}};
const Segment SGI78355[] = {{&S31764[44], 14}, {NULL}};
const Segment SGI78356[] = {{&S31764[44], 40}, {NULL}};
const Segment SGI78357[] = {{&S31764[30], 6}, {NULL}};
const Segment SGI78358[] = {{&S31764[31], 3}, {NULL}};
const Segment SGI78359[] = {{&S31764[98], 6}, {NULL}};
const Segment SGI78360[] = {{&S31766[44], 7}, {NULL}};
const Segment SGI78361[] = {{&S31766[44], 19}, {NULL}};
const Segment SGI78362[] = {{&S31766[30], 6}, {NULL}};
const Segment SGI78363[] = {{&S31766[31], 3}, {NULL}};
const Segment SGI78364[] = {{&S31766[77], 6}, {NULL}};
const Segment SGI78365[] = {{&S31771[44], 12}, {NULL}};
const Segment SGI78366[] = {{&S31771[44], 33}, {NULL}};
const Segment SGI78367[] = {{&S31771[30], 6}, {NULL}};
const Segment SGI78368[] = {{&S31771[31], 3}, {NULL}};
const Segment SGI78369[] = {{&S31771[91], 5}, {NULL}};
const Segment SGI78370[] = {{&S43433[42], 7}, {NULL}};
const Segment SGI78371[] = {{&S43433[42], 21}, {NULL}};
const Segment SGI78372[] = {{&S43433[30], 4}, {NULL}};
const Segment SGI78373[] = {{&S43433[112], 3}, {NULL}};
const Segment SGI78374[] = {{&S43433[77], 6}, {NULL}};
const Segment SGI78375[] = {{&S43434[42], 7}, {NULL}};
const Segment SGI78376[] = {{&S43434[42], 21}, {NULL}};
const Segment SGI78377[] = {{&S43434[30], 4}, {NULL}};
const Segment SGI78378[] = {{&S43434[112], 3}, {NULL}};
const Segment SGI78379[] = {{&S43434[77], 6}, {NULL}};
const Segment SGI78380[] = {{&S43435[42], 7}, {NULL}};
const Segment SGI78381[] = {{&S43435[42], 23}, {NULL}};
const Segment SGI78382[] = {{&S43435[30], 4}, {NULL}};
const Segment SGI78383[] = {{&S43435[114], 3}, {NULL}};
const Segment SGI78384[] = {{&S43435[79], 6}, {NULL}};
const Segment SGI78385[] = {{&S43436[36], 7}, {NULL}};
const Segment SGI78386[] = {{&S43436[31], 4}, {NULL}};
const Segment SGI78387[] = {{&S43437[42], 7}, {NULL}};
const Segment SGI78388[] = {{&S43437[42], 23}, {NULL}};
const Segment SGI78389[] = {{&S43437[30], 4}, {NULL}};
const Segment SGI78390[] = {{&S43437[114], 3}, {NULL}};
const Segment SGI78391[] = {{&S43437[79], 6}, {NULL}};
const Segment SGI78392[] = {{&S43438[42], 7}, {NULL}};
const Segment SGI78393[] = {{&S43438[42], 27}, {NULL}};
const Segment SGI78394[] = {{&S43438[30], 4}, {NULL}};
const Segment SGI78395[] = {{&S43438[118], 3}, {NULL}};
const Segment SGI78396[] = {{&S43438[83], 6}, {NULL}};
const Segment SGI78397[] = {{&S31933[44], 15}, {NULL}};
const Segment SGI78398[] = {{&S31933[44], 26}, {NULL}};
const Segment SGI78399[] = {{&S31933[30], 6}, {NULL}};
const Segment SGI78400[] = {{&S31933[31], 3}, {NULL}};
const Segment SGI78401[] = {{&S31933[84], 6}, {NULL}};
const Segment SGI78402[] = {{&S31934[44], 15}, {NULL}};
const Segment SGI78403[] = {{&S31934[44], 27}, {NULL}};
const Segment SGI78404[] = {{&S31934[30], 6}, {NULL}};
const Segment SGI78405[] = {{&S31934[31], 3}, {NULL}};
const Segment SGI78406[] = {{&S31934[85], 6}, {NULL}};
const Segment SGI78407[] = {{&S50238[44], 13}, {NULL}};
const Segment SGI78408[] = {{&S50238[44], 32}, {NULL}};
const Segment SGI78409[] = {{&S50238[30], 6}, {NULL}};
const Segment SGI78410[] = {{&S50238[124], 3}, {NULL}};
const Segment SGI78411[] = {{&S50238[90], 5}, {NULL}};
const Segment SGI78412[] = {{&S31939[38], 15}, {NULL}};
const Segment SGI78413[] = {{&S31939[31], 6}, {NULL}};
const Segment SGI78414[] = {{&S50240[38], 13}, {NULL}};
const Segment SGI78415[] = {{&S50240[31], 6}, {NULL}};
const Segment SGI78416[] = {{&S50241[44], 13}, {NULL}};
const Segment SGI78417[] = {{&S50241[44], 32}, {NULL}};
const Segment SGI78418[] = {{&S50241[30], 6}, {NULL}};
const Segment SGI78419[] = {{&S50241[93], 6}, {NULL}};
const Segment SGI78420[] = {{&S31943[44], 15}, {NULL}};
const Segment SGI78421[] = {{&S31943[44], 26}, {NULL}};
const Segment SGI78422[] = {{&S31943[30], 6}, {NULL}};
const Segment SGI78423[] = {{&S31943[31], 3}, {NULL}};
const Segment SGI78424[] = {{&S31943[84], 6}, {NULL}};
const Segment SGI78425[] = {{&S43614[42], 11}, {NULL}};
const Segment SGI78426[] = {{&S43614[42], 23}, {NULL}};
const Segment SGI78427[] = {{&S43614[30], 4}, {NULL}};
const Segment SGI78428[] = {{&S43614[114], 3}, {NULL}};
const Segment SGI78429[] = {{&S43614[79], 6}, {NULL}};
const Segment SGI78430[] = {{&S43615[42], 10}, {NULL}};
const Segment SGI78431[] = {{&S43615[42], 22}, {NULL}};
const Segment SGI78432[] = {{&S43615[30], 4}, {NULL}};
const Segment SGI78433[] = {{&S43615[113], 3}, {NULL}};
const Segment SGI78434[] = {{&S43615[78], 6}, {NULL}};
const Segment SGI78435[] = {{&S43616[36], 10}, {NULL}};
const Segment SGI78436[] = {{&S43616[31], 4}, {NULL}};
const Segment SGI78437[] = {{&S43617[36], 11}, {NULL}};
const Segment SGI78438[] = {{&S43617[31], 4}, {NULL}};
const Segment SGI78439[] = {{&S43618[42], 29}, {NULL}};
const Segment SGI78440[] = {{&S43618[42], 41}, {NULL}};
const Segment SGI78441[] = {{&S43618[30], 4}, {NULL}};
const Segment SGI78442[] = {{&S43618[132], 3}, {NULL}};
const Segment SGI78443[] = {{&S43618[97], 6}, {NULL}};
const Segment SGI78444[] = {{&S43619[42], 29}, {NULL}};
const Segment SGI78445[] = {{&S43619[42], 41}, {NULL}};
const Segment SGI78446[] = {{&S43619[30], 4}, {NULL}};
const Segment SGI78447[] = {{&S43619[132], 3}, {NULL}};
const Segment SGI78448[] = {{&S43619[97], 6}, {NULL}};
const Segment SGI78449[] = {{&S43620[42], 28}, {NULL}};
const Segment SGI78450[] = {{&S43620[42], 40}, {NULL}};
const Segment SGI78451[] = {{&S43620[30], 4}, {NULL}};
const Segment SGI78452[] = {{&S43620[131], 3}, {NULL}};
const Segment SGI78453[] = {{&S43620[96], 6}, {NULL}};
const Segment SGI78454[] = {{&S43621[42], 29}, {NULL}};
const Segment SGI78455[] = {{&S43621[42], 41}, {NULL}};
const Segment SGI78456[] = {{&S43621[30], 4}, {NULL}};
const Segment SGI78457[] = {{&S43621[132], 3}, {NULL}};
const Segment SGI78458[] = {{&S43621[97], 6}, {NULL}};
const Segment SGI78459[] = {{&S43622[42], 29}, {NULL}};
const Segment SGI78460[] = {{&S43622[42], 41}, {NULL}};
const Segment SGI78461[] = {{&S43622[30], 4}, {NULL}};
const Segment SGI78462[] = {{&S43622[132], 3}, {NULL}};
const Segment SGI78463[] = {{&S43622[97], 6}, {NULL}};
const Segment SGI78464[] = {{&S43623[40], 11}, {NULL}};
const Segment SGI78465[] = {{&S43623[30], 4}, {NULL}};
const Segment SGI78466[] = {{&S43623[112], 3}, {NULL}};
const Segment SGI78467[] = {{&S43623[77], 6}, {NULL}};
const Segment SGI78468[] = {{&S43624[42], 29}, {NULL}};
const Segment SGI78469[] = {{&S43624[42], 41}, {NULL}};
const Segment SGI78470[] = {{&S43624[30], 4}, {NULL}};
const Segment SGI78471[] = {{&S43624[132], 3}, {NULL}};
const Segment SGI78472[] = {{&S43624[97], 6}, {NULL}};
const Segment SGI78473[] = {{&S43625[42], 29}, {NULL}};
const Segment SGI78474[] = {{&S43625[42], 41}, {NULL}};
const Segment SGI78475[] = {{&S43625[30], 4}, {NULL}};
const Segment SGI78476[] = {{&S43625[132], 3}, {NULL}};
const Segment SGI78477[] = {{&S43625[97], 6}, {NULL}};
const Segment SGI78478[] = {{&S43626[42], 11}, {NULL}};
const Segment SGI78479[] = {{&S43626[42], 23}, {NULL}};
const Segment SGI78480[] = {{&S43626[30], 4}, {NULL}};
const Segment SGI78481[] = {{&S43626[114], 3}, {NULL}};
const Segment SGI78482[] = {{&S43626[79], 6}, {NULL}};
const Segment SGI78483[] = {{&S43627[42], 29}, {NULL}};
const Segment SGI78484[] = {{&S43627[42], 41}, {NULL}};
const Segment SGI78485[] = {{&S43627[30], 4}, {NULL}};
const Segment SGI78486[] = {{&S43627[132], 3}, {NULL}};
const Segment SGI78487[] = {{&S43627[97], 6}, {NULL}};
const Segment SGI78488[] = {{&S43628[42], 11}, {NULL}};
const Segment SGI78489[] = {{&S43628[42], 23}, {&S43628[207], 7}, {NULL}};
const Segment SGI78490[] = {{&S43628[30], 4}, {NULL}};
const Segment SGI78491[] = {{&S43628[114], 3}, {NULL}};
const Segment SGI78492[] = {{&S43628[79], 6}, {NULL}};
const Segment SGI78493[] = {{&S43629[42], 28}, {NULL}};
const Segment SGI78494[] = {{&S43629[42], 40}, {NULL}};
const Segment SGI78495[] = {{&S43629[30], 4}, {NULL}};
const Segment SGI78496[] = {{&S43629[131], 3}, {NULL}};
const Segment SGI78497[] = {{&S43629[96], 6}, {NULL}};
const Segment SGI78498[] = {{&S43630[42], 29}, {NULL}};
const Segment SGI78499[] = {{&S43630[42], 41}, {NULL}};
const Segment SGI78500[] = {{&S43630[30], 4}, {NULL}};
const Segment SGI78501[] = {{&S43630[132], 3}, {NULL}};
const Segment SGI78502[] = {{&S43630[97], 6}, {NULL}};
const Segment SGI78503[] = {{&S43631[42], 28}, {NULL}};
const Segment SGI78504[] = {{&S43631[42], 40}, {NULL}};
const Segment SGI78505[] = {{&S43631[30], 4}, {NULL}};
const Segment SGI78506[] = {{&S43631[131], 3}, {NULL}};
const Segment SGI78507[] = {{&S43631[96], 6}, {NULL}};
const Segment SGI78508[] = {{&S43632[42], 29}, {NULL}};
const Segment SGI78509[] = {{&S43632[42], 41}, {NULL}};
const Segment SGI78510[] = {{&S43632[30], 4}, {NULL}};
const Segment SGI78511[] = {{&S43632[132], 3}, {NULL}};
const Segment SGI78512[] = {{&S43632[97], 6}, {NULL}};
const Segment SGI78513[] = {{&S43633[42], 29}, {NULL}};
const Segment SGI78514[] = {{&S43633[42], 41}, {NULL}};
const Segment SGI78515[] = {{&S43633[30], 4}, {NULL}};
const Segment SGI78516[] = {{&S43633[132], 3}, {NULL}};
const Segment SGI78517[] = {{&S43633[97], 6}, {NULL}};
const Segment SGI78518[] = {{&S43634[42], 10}, {NULL}};
const Segment SGI78519[] = {{&S43634[42], 22}, {NULL}};
const Segment SGI78520[] = {{&S43634[30], 4}, {NULL}};
const Segment SGI78521[] = {{&S43634[113], 3}, {NULL}};
const Segment SGI78522[] = {{&S43634[78], 6}, {NULL}};
const Segment SGI78523[] = {{&S43635[44], 10}, {NULL}};
const Segment SGI78524[] = {{&S43635[44], 21}, {NULL}};
const Segment SGI78525[] = {{&S43635[30], 6}, {NULL}};
const Segment SGI78526[] = {{&S43635[114], 3}, {NULL}};
const Segment SGI78527[] = {{&S43635[79], 6}, {NULL}};
const Segment SGI78528[] = {{&S43636[38], 10}, {NULL}};
const Segment SGI78529[] = {{&S43636[31], 6}, {NULL}};
const Segment SGI78530[] = {{&S43637[44], 10}, {NULL}};
const Segment SGI78531[] = {{&S43637[44], 21}, {NULL}};
const Segment SGI78532[] = {{&S43637[30], 6}, {NULL}};
const Segment SGI78533[] = {{&S43637[114], 3}, {NULL}};
const Segment SGI78534[] = {{&S43637[79], 6}, {NULL}};
const Segment SGI78535[] = {{&S32035[44], 10}, {NULL}};
const Segment SGI78536[] = {{&S32035[44], 22}, {NULL}};
const Segment SGI78537[] = {{&S32035[30], 6}, {NULL}};
const Segment SGI78538[] = {{&S32035[31], 3}, {NULL}};
const Segment SGI78539[] = {{&S32035[80], 6}, {NULL}};
const Segment SGI78540[] = {{&S32036[44], 9}, {NULL}};
const Segment SGI78541[] = {{&S32036[44], 21}, {NULL}};
const Segment SGI78542[] = {{&S32036[30], 6}, {NULL}};
const Segment SGI78543[] = {{&S32036[31], 3}, {NULL}};
const Segment SGI78544[] = {{&S32036[79], 6}, {NULL}};
const Segment SGI78545[] = {{&S32037[44], 10}, {NULL}};
const Segment SGI78546[] = {{&S32037[44], 22}, {NULL}};
const Segment SGI78547[] = {{&S32037[30], 6}, {NULL}};
const Segment SGI78548[] = {{&S32037[31], 3}, {NULL}};
const Segment SGI78549[] = {{&S32037[80], 6}, {NULL}};
const Segment SGI78550[] = {{&S32038[44], 10}, {NULL}};
const Segment SGI78551[] = {{&S32038[44], 22}, {NULL}};
const Segment SGI78552[] = {{&S32038[30], 6}, {NULL}};
const Segment SGI78553[] = {{&S32038[31], 3}, {NULL}};
const Segment SGI78554[] = {{&S32038[80], 6}, {NULL}};
const Segment SGI78555[] = {{&S32039[38], 9}, {NULL}};
const Segment SGI78556[] = {{&S32039[31], 6}, {NULL}};
const Segment SGI78557[] = {{&S32040[44], 9}, {NULL}};
const Segment SGI78558[] = {{&S32040[44], 21}, {NULL}};
const Segment SGI78559[] = {{&S32040[30], 6}, {NULL}};
const Segment SGI78560[] = {{&S32040[82], 6}, {NULL}};
const Segment SGI78561[] = {{&S32041[38], 10}, {NULL}};
const Segment SGI78562[] = {{&S32041[31], 6}, {NULL}};
const Segment SGI78563[] = {{&S32042[38], 10}, {NULL}};
const Segment SGI78564[] = {{&S32042[31], 6}, {NULL}};
const Segment SGI78565[] = {{&S32043[44], 14}, {NULL}};
const Segment SGI78566[] = {{&S32043[44], 26}, {NULL}};
const Segment SGI78567[] = {{&S32043[30], 6}, {NULL}};
const Segment SGI78568[] = {{&S32043[31], 3}, {NULL}};
const Segment SGI78569[] = {{&S32043[84], 6}, {NULL}};
const Segment SGI78570[] = {{&S32044[44], 14}, {NULL}};
const Segment SGI78571[] = {{&S32044[44], 26}, {NULL}};
const Segment SGI78572[] = {{&S32044[30], 6}, {NULL}};
const Segment SGI78573[] = {{&S32044[31], 3}, {NULL}};
const Segment SGI78574[] = {{&S32044[84], 6}, {NULL}};
const Segment SGI78575[] = {{&S32045[44], 14}, {NULL}};
const Segment SGI78576[] = {{&S32045[44], 26}, {NULL}};
const Segment SGI78577[] = {{&S32045[30], 6}, {NULL}};
const Segment SGI78578[] = {{&S32045[31], 3}, {NULL}};
const Segment SGI78579[] = {{&S32045[84], 6}, {NULL}};
const Segment SGI78580[] = {{&S32046[38], 10}, {NULL}};
const Segment SGI78581[] = {{&S32046[31], 6}, {NULL}};
const Segment SGI78582[] = {{&S32047[44], 9}, {NULL}};
const Segment SGI78583[] = {{&S32047[44], 21}, {NULL}};
const Segment SGI78584[] = {{&S32047[30], 6}, {NULL}};
const Segment SGI78585[] = {{&S32047[82], 4}, {NULL}};
const Segment SGI78586[] = {{&S32047[82], 3}, {NULL}};
const Segment SGI78587[] = {{&S43690[42], 9}, {NULL}};
const Segment SGI78588[] = {{&S43690[42], 21}, {NULL}};
const Segment SGI78589[] = {{&S43690[30], 4}, {NULL}};
const Segment SGI78590[] = {{&S43690[112], 3}, {NULL}};
const Segment SGI78591[] = {{&S43690[77], 6}, {NULL}};
const Segment SGI78592[] = {{&S43691[42], 15}, {NULL}};
const Segment SGI78593[] = {{&S43691[42], 27}, {NULL}};
const Segment SGI78594[] = {{&S43691[30], 4}, {NULL}};
const Segment SGI78595[] = {{&S43691[118], 3}, {NULL}};
const Segment SGI78596[] = {{&S43691[83], 6}, {NULL}};
const Segment SGI78597[] = {{&S43692[36], 15}, {NULL}};
const Segment SGI78598[] = {{&S43692[31], 4}, {NULL}};
const Segment SGI78599[] = {{&S32057[44], 9}, {NULL}};
const Segment SGI78600[] = {{&S32057[44], 21}, {NULL}};
const Segment SGI78601[] = {{&S32057[30], 6}, {NULL}};
const Segment SGI78602[] = {{&S32057[31], 3}, {NULL}};
const Segment SGI78603[] = {{&S32057[79], 6}, {NULL}};
const Segment SGI78604[] = {{&S32058[44], 18}, {NULL}};
const Segment SGI78605[] = {{&S32058[44], 30}, {NULL}};
const Segment SGI78606[] = {{&S32058[30], 6}, {NULL}};
const Segment SGI78607[] = {{&S32058[31], 3}, {NULL}};
const Segment SGI78608[] = {{&S32058[88], 6}, {NULL}};
const Segment SGI78609[] = {{&S32059[44], 23}, {NULL}};
const Segment SGI78610[] = {{&S32059[44], 35}, {NULL}};
const Segment SGI78611[] = {{&S32059[30], 6}, {NULL}};
const Segment SGI78612[] = {{&S32059[31], 3}, {NULL}};
const Segment SGI78613[] = {{&S32059[93], 6}, {NULL}};
const Segment SGI78614[] = {{&S32060[44], 12}, {NULL}};
const Segment SGI78615[] = {{&S32060[44], 24}, {NULL}};
const Segment SGI78616[] = {{&S32060[30], 6}, {NULL}};
const Segment SGI78617[] = {{&S32060[31], 3}, {NULL}};
const Segment SGI78618[] = {{&S32060[82], 6}, {NULL}};
const Segment SGI78619[] = {{&S32061[38], 9}, {NULL}};
const Segment SGI78620[] = {{&S32061[31], 6}, {NULL}};
const Segment SGI78621[] = {{&S32062[44], 23}, {NULL}};
const Segment SGI78622[] = {{&S32062[44], 35}, {NULL}};
const Segment SGI78623[] = {{&S32062[30], 6}, {NULL}};
const Segment SGI78624[] = {{&S32062[31], 3}, {NULL}};
const Segment SGI78625[] = {{&S32062[93], 6}, {NULL}};
const Segment SGI78626[] = {{&S32063[44], 23}, {NULL}};
const Segment SGI78627[] = {{&S32063[44], 35}, {NULL}};
const Segment SGI78628[] = {{&S32063[30], 6}, {NULL}};
const Segment SGI78629[] = {{&S32063[31], 3}, {NULL}};
const Segment SGI78630[] = {{&S32063[93], 6}, {NULL}};
const Segment SGI78631[] = {{&S32064[44], 9}, {NULL}};
const Segment SGI78632[] = {{&S32064[44], 20}, {NULL}};
const Segment SGI78633[] = {{&S32064[30], 6}, {NULL}};
const Segment SGI78634[] = {{&S32064[31], 3}, {NULL}};
const Segment SGI78635[] = {{&S32064[78], 6}, {NULL}};
const Segment SGI78636[] = {{&S32065[44], 23}, {NULL}};
const Segment SGI78637[] = {{&S32065[44], 35}, {NULL}};
const Segment SGI78638[] = {{&S32065[30], 6}, {NULL}};
const Segment SGI78639[] = {{&S32065[31], 3}, {NULL}};
const Segment SGI78640[] = {{&S32065[93], 6}, {NULL}};
const Segment SGI78641[] = {{&S32066[44], 9}, {NULL}};
const Segment SGI78642[] = {{&S32066[44], 21}, {NULL}};
const Segment SGI78643[] = {{&S32066[30], 6}, {NULL}};
const Segment SGI78644[] = {{&S32066[31], 3}, {NULL}};
const Segment SGI78645[] = {{&S32066[79], 6}, {NULL}};
const Segment SGI78646[] = {{&S32068[44], 9}, {NULL}};
const Segment SGI78647[] = {{&S32068[44], 21}, {NULL}};
const Segment SGI78648[] = {{&S32068[30], 6}, {NULL}};
const Segment SGI78649[] = {{&S32068[79], 6}, {NULL}};
const Segment SGI78650[] = {{&S32071[38], 9}, {NULL}};
const Segment SGI78651[] = {{&S32071[31], 6}, {NULL}};
const Segment SGI78652[] = {{&S32073[42], 9}, {NULL}};
const Segment SGI78653[] = {{&S32073[30], 6}, {NULL}};
const Segment SGI78654[] = {{&S32073[31], 3}, {NULL}};
const Segment SGI78655[] = {{&S32073[77], 6}, {NULL}};
const Segment SGI78656[] = {{&S32074[44], 23}, {NULL}};
const Segment SGI78657[] = {{&S32074[44], 35}, {NULL}};
const Segment SGI78658[] = {{&S32074[30], 6}, {NULL}};
const Segment SGI78659[] = {{&S32074[31], 3}, {NULL}};
const Segment SGI78660[] = {{&S32074[93], 6}, {NULL}};
const Segment SGI78661[] = {{&S32075[42], 18}, {NULL}};
const Segment SGI78662[] = {{&S32075[30], 6}, {NULL}};
const Segment SGI78663[] = {{&S32075[31], 3}, {NULL}};
const Segment SGI78664[] = {{&S32075[86], 6}, {NULL}};
const Segment SGI78665[] = {{&S32076[44], 18}, {NULL}};
const Segment SGI78666[] = {{&S32076[44], 28}, {NULL}};
const Segment SGI78667[] = {{&S32076[30], 6}, {NULL}};
const Segment SGI78668[] = {{&S32076[31], 3}, {NULL}};
const Segment SGI78669[] = {{&S32076[86], 6}, {NULL}};
const Segment SGI78670[] = {{&S32088[44], 9}, {NULL}};
const Segment SGI78671[] = {{&S32088[44], 21}, {NULL}};
const Segment SGI78672[] = {{&S32088[30], 6}, {NULL}};
const Segment SGI78673[] = {{&S32088[31], 3}, {NULL}};
const Segment SGI78674[] = {{&S32088[79], 6}, {NULL}};
const Segment SGI78675[] = {{&S32089[44], 14}, {NULL}};
const Segment SGI78676[] = {{&S32089[44], 26}, {NULL}};
const Segment SGI78677[] = {{&S32089[30], 6}, {NULL}};
const Segment SGI78678[] = {{&S32089[31], 3}, {NULL}};
const Segment SGI78679[] = {{&S32089[84], 6}, {NULL}};
const Segment SGI78680[] = {{&S32090[38], 9}, {NULL}};
const Segment SGI78681[] = {{&S32090[31], 6}, {NULL}};
const Segment SGI78682[] = {{&S32091[44], 16}, {NULL}};
const Segment SGI78683[] = {{&S32091[44], 28}, {NULL}};
const Segment SGI78684[] = {{&S32091[30], 6}, {NULL}};
const Segment SGI78685[] = {{&S32091[31], 3}, {NULL}};
const Segment SGI78686[] = {{&S32091[86], 6}, {NULL}};
const Segment SGI78687[] = {{&S32092[44], 14}, {NULL}};
const Segment SGI78688[] = {{&S32092[44], 26}, {NULL}};
const Segment SGI78689[] = {{&S32092[30], 6}, {NULL}};
const Segment SGI78690[] = {{&S32092[31], 3}, {NULL}};
const Segment SGI78691[] = {{&S32092[84], 6}, {NULL}};
const Segment SGI78692[] = {{&S32093[44], 14}, {NULL}};
const Segment SGI78693[] = {{&S32093[44], 26}, {NULL}};
const Segment SGI78694[] = {{&S32093[30], 6}, {NULL}};
const Segment SGI78695[] = {{&S32093[31], 3}, {NULL}};
const Segment SGI78696[] = {{&S32093[84], 6}, {NULL}};
const Segment SGI78697[] = {{&S32094[44], 14}, {NULL}};
const Segment SGI78698[] = {{&S32094[44], 26}, {NULL}};
const Segment SGI78699[] = {{&S32094[30], 6}, {NULL}};
const Segment SGI78700[] = {{&S32094[31], 3}, {NULL}};
const Segment SGI78701[] = {{&S32094[84], 6}, {NULL}};
const Segment SGI78702[] = {{&S32095[41], 9}, {NULL}};
const Segment SGI78703[] = {{&S32095[30], 6}, {NULL}};
const Segment SGI78704[] = {{&S32095[31], 3}, {NULL}};
const Segment SGI78705[] = {{&S32095[76], 6}, {NULL}};
const Segment SGI78706[] = {{&S32096[44], 14}, {NULL}};
const Segment SGI78707[] = {{&S32096[44], 26}, {NULL}};
const Segment SGI78708[] = {{&S32096[30], 6}, {NULL}};
const Segment SGI78709[] = {{&S32096[31], 3}, {NULL}};
const Segment SGI78710[] = {{&S32096[84], 6}, {NULL}};
const Segment SGI78711[] = {{&S32097[44], 14}, {NULL}};
const Segment SGI78712[] = {{&S32097[44], 26}, {NULL}};
const Segment SGI78713[] = {{&S32097[30], 6}, {NULL}};
const Segment SGI78714[] = {{&S32097[31], 3}, {NULL}};
const Segment SGI78715[] = {{&S32097[84], 6}, {NULL}};
const Segment SGI78716[] = {{&S32098[44], 14}, {NULL}};
const Segment SGI78717[] = {{&S32098[44], 26}, {NULL}};
const Segment SGI78718[] = {{&S32098[30], 6}, {NULL}};
const Segment SGI78719[] = {{&S32098[31], 3}, {NULL}};
const Segment SGI78720[] = {{&S32098[84], 6}, {NULL}};
const Segment SGI78721[] = {{&S32099[44], 14}, {NULL}};
const Segment SGI78722[] = {{&S32099[44], 26}, {NULL}};
const Segment SGI78723[] = {{&S32099[30], 6}, {NULL}};
const Segment SGI78724[] = {{&S32099[31], 3}, {NULL}};
const Segment SGI78725[] = {{&S32099[84], 6}, {NULL}};
const Segment SGI78726[] = {{&S32100[44], 14}, {NULL}};
const Segment SGI78727[] = {{&S32100[44], 26}, {NULL}};
const Segment SGI78728[] = {{&S32100[30], 6}, {NULL}};
const Segment SGI78729[] = {{&S32100[31], 3}, {NULL}};
const Segment SGI78730[] = {{&S32100[84], 6}, {NULL}};
const Segment SGI78731[] = {{&S32102[44], 7}, {NULL}};
const Segment SGI78732[] = {{&S32102[44], 24}, {NULL}};
const Segment SGI78733[] = {{&S32102[30], 6}, {NULL}};
const Segment SGI78734[] = {{&S32102[31], 3}, {NULL}};
const Segment SGI78735[] = {{&S32102[82], 6}, {NULL}};
const Segment SGI78736[] = {{&S32103[38], 7}, {NULL}};
const Segment SGI78737[] = {{&S32103[31], 6}, {NULL}};
const Segment SGI78738[] = {{&S50486[44], 10}, {NULL}};
const Segment SGI78739[] = {{&S50486[44], 27}, {NULL}};
const Segment SGI78740[] = {{&S50486[30], 6}, {NULL}};
const Segment SGI78741[] = {{&S50486[119], 3}, {NULL}};
const Segment SGI78742[] = {{&S50486[85], 5}, {NULL}};
const Segment SGI78743[] = {{&S50487[38], 10}, {NULL}};
const Segment SGI78744[] = {{&S50487[31], 6}, {NULL}};
const Segment SGI78745[] = {{&S50488[44], 10}, {NULL}};
const Segment SGI78746[] = {{&S50488[44], 27}, {NULL}};
const Segment SGI78747[] = {{&S50488[30], 6}, {NULL}};
const Segment SGI78748[] = {{&S50488[85], 6}, {NULL}};
const Segment SGI78749[] = {{&S50489[44], 10}, {NULL}};
const Segment SGI78750[] = {{&S50489[44], 33}, {NULL}};
const Segment SGI78751[] = {{&S50489[30], 6}, {NULL}};
const Segment SGI78752[] = {{&S50490[44], 10}, {NULL}};
const Segment SGI78753[] = {{&S50490[44], 33}, {NULL}};
const Segment SGI78754[] = {{&S50490[30], 6}, {NULL}};
const Segment SGI78755[] = {{&S50491[44], 10}, {NULL}};
const Segment SGI78756[] = {{&S50491[44], 33}, {NULL}};
const Segment SGI78757[] = {{&S50491[30], 6}, {NULL}};
const Segment SGI78758[] = {{&S50494[44], 10}, {NULL}};
const Segment SGI78759[] = {{&S50494[44], 27}, {NULL}};
const Segment SGI78760[] = {{&S50494[30], 6}, {NULL}};
const Segment SGI78761[] = {{&S50494[88], 6}, {NULL}};
const Segment SGI78762[] = {{&S50495[42], 10}, {NULL}};
const Segment SGI78763[] = {{&S50495[30], 6}, {NULL}};
const Segment SGI78764[] = {{&S50495[117], 3}, {NULL}};
const Segment SGI78765[] = {{&S50495[83], 5}, {NULL}};
const Segment SGI78766[] = {{&S50496[44], 10}, {NULL}};
const Segment SGI78767[] = {{&S50496[44], 27}, {NULL}};
const Segment SGI78768[] = {{&S50496[30], 6}, {NULL}};
const Segment SGI78769[] = {{&S50496[120], 3}, {NULL}};
const Segment SGI78770[] = {{&S50496[85], 6}, {NULL}};
const Segment SGI78771[] = {{&S50496[140], 8}, {NULL}};
const Segment SGI78772[] = {{&S44390[44], 9}, {NULL}};
const Segment SGI78773[] = {{&S44390[44], 20}, {NULL}};
const Segment SGI78774[] = {{&S44390[30], 6}, {NULL}};
const Segment SGI78775[] = {{&S44390[112], 3}, {NULL}};
const Segment SGI78776[] = {{&S44390[78], 5}, {NULL}};
const Segment SGI78777[] = {{&S44391[44], 5}, {NULL}};
const Segment SGI78778[] = {{&S44391[44], 17}, {NULL}};
const Segment SGI78779[] = {{&S44391[30], 6}, {NULL}};
const Segment SGI78780[] = {{&S44391[109], 3}, {NULL}};
const Segment SGI78781[] = {{&S44391[75], 5}, {NULL}};
const Segment SGI78782[] = {{&S44399[38], 9}, {NULL}};
const Segment SGI78783[] = {{&S44399[31], 6}, {NULL}};
const Segment SGI78784[] = {{&S44401[44], 5}, {NULL}};
const Segment SGI78785[] = {{&S44401[44], 16}, {NULL}};
const Segment SGI78786[] = {{&S44401[30], 6}, {NULL}};
const Segment SGI78787[] = {{&S44401[108], 3}, {NULL}};
const Segment SGI78788[] = {{&S44401[74], 5}, {NULL}};
const Segment SGI78789[] = {{&S44407[44], 5}, {NULL}};
const Segment SGI78790[] = {{&S44407[44], 15}, {NULL}};
const Segment SGI78791[] = {{&S44407[30], 6}, {NULL}};
const Segment SGI78792[] = {{&S44407[107], 3}, {NULL}};
const Segment SGI78793[] = {{&S44407[73], 5}, {NULL}};
const Segment SGI78794[] = {{&S44408[44], 9}, {NULL}};
const Segment SGI78795[] = {{&S44408[44], 29}, {NULL}};
const Segment SGI78796[] = {{&S44408[30], 6}, {NULL}};
const Segment SGI78797[] = {{&S44408[121], 3}, {NULL}};
const Segment SGI78798[] = {{&S44408[87], 5}, {NULL}};
const Segment SGI78799[] = {{&S44414[44], 9}, {NULL}};
const Segment SGI78800[] = {{&S44414[44], 20}, {NULL}};
const Segment SGI78801[] = {{&S44414[30], 6}, {NULL}};
const Segment SGI78802[] = {{&S44414[81], 4}, {NULL}};
const Segment SGI78803[] = {{&S44414[81], 3}, {NULL}};
const Segment SGI78804[] = {{&S44416[38], 5}, {NULL}};
const Segment SGI78805[] = {{&S44416[31], 6}, {NULL}};
const Segment SGI78806[] = {{&S44417[44], 9}, {NULL}};
const Segment SGI78807[] = {{&S44417[44], 29}, {NULL}};
const Segment SGI78808[] = {{&S44417[30], 6}, {NULL}};
const Segment SGI78809[] = {{&S44417[121], 3}, {NULL}};
const Segment SGI78810[] = {{&S44417[87], 5}, {NULL}};
const Segment SGI78811[] = {{&S44419[44], 9}, {NULL}};
const Segment SGI78812[] = {{&S44419[44], 20}, {NULL}};
const Segment SGI78813[] = {{&S44419[30], 6}, {NULL}};
const Segment SGI78814[] = {{&S44419[121], 3}, {NULL}};
const Segment SGI78815[] = {{&S44419[87], 5}, {NULL}};
const Segment SGI78816[] = {{&S44420[44], 10}, {NULL}};
const Segment SGI78817[] = {{&S44420[44], 21}, {NULL}};
const Segment SGI78818[] = {{&S44420[30], 6}, {NULL}};
const Segment SGI78819[] = {{&S44420[113], 3}, {NULL}};
const Segment SGI78820[] = {{&S44420[79], 5}, {NULL}};
const Segment SGI78821[] = {{&S44421[44], 9}, {NULL}};
const Segment SGI78822[] = {{&S44421[44], 20}, {NULL}};
const Segment SGI78823[] = {{&S44421[30], 6}, {NULL}};
const Segment SGI78824[] = {{&S44421[112], 3}, {NULL}};
const Segment SGI78825[] = {{&S44421[78], 5}, {NULL}};
const Segment SGI78826[] = {{&S37808[44], 5}, {NULL}};
const Segment SGI78827[] = {{&S37808[44], 18}, {NULL}};
const Segment SGI78828[] = {{&S37808[30], 6}, {NULL}};
const Segment SGI78829[] = {{&S37808[31], 3}, {NULL}};
const Segment SGI78830[] = {{&S37808[76], 6}, {NULL}};
const Segment SGI78831[] = {{&S37808[0], 13}, {NULL}};
const Segment SGI78832[] = {{&S32405[44], 6}, {NULL}};
const Segment SGI78833[] = {{&S32405[44], 17}, {NULL}};
const Segment SGI78834[] = {{&S32405[30], 6}, {NULL}};
const Segment SGI78835[] = {{&S32405[31], 3}, {NULL}};
const Segment SGI78836[] = {{&S32405[75], 6}, {NULL}};
const Segment SGI78837[] = {{&S32406[44], 6}, {NULL}};
const Segment SGI78838[] = {{&S32406[44], 18}, {NULL}};
const Segment SGI78839[] = {{&S32406[30], 6}, {NULL}};
const Segment SGI78840[] = {{&S32406[31], 3}, {NULL}};
const Segment SGI78841[] = {{&S32406[76], 6}, {NULL}};
const Segment SGI78842[] = {{&S32409[44], 6}, {NULL}};
const Segment SGI78843[] = {{&S32409[44], 17}, {NULL}};
const Segment SGI78844[] = {{&S32409[30], 6}, {NULL}};
const Segment SGI78845[] = {{&S50665[44], 8}, {NULL}};
const Segment SGI78846[] = {{&S50665[44], 25}, {NULL}};
const Segment SGI78847[] = {{&S50665[30], 6}, {NULL}};
const Segment SGI78848[] = {{&S50665[117], 3}, {NULL}};
const Segment SGI78849[] = {{&S50665[83], 5}, {NULL}};
const Segment SGI78850[] = {{&S50666[42], 8}, {NULL}};
const Segment SGI78851[] = {{&S32415[44], 6}, {NULL}};
const Segment SGI78852[] = {{&S50666[30], 6}, {NULL}};
const Segment SGI78853[] = {{&S32415[44], 17}, {NULL}};
const Segment SGI78854[] = {{&S32415[30], 6}, {NULL}};
const Segment SGI78855[] = {{&S50666[115], 3}, {NULL}};
const Segment SGI78856[] = {{&S32415[31], 3}, {NULL}};
const Segment SGI78857[] = {{&S50666[81], 5}, {NULL}};
const Segment SGI78858[] = {{&S32415[75], 6}, {NULL}};
const Segment SGI78859[] = {{&S50667[38], 8}, {NULL}};
const Segment SGI78860[] = {{&S50667[31], 6}, {NULL}};
const Segment SGI78861[] = {{&S32416[44], 6}, {NULL}};
const Segment SGI78862[] = {{&S32416[44], 18}, {NULL}};
const Segment SGI78863[] = {{&S32416[30], 6}, {NULL}};
const Segment SGI78864[] = {{&S32416[31], 3}, {NULL}};
const Segment SGI78865[] = {{&S32416[76], 6}, {NULL}};
const Segment SGI78866[] = {{&S50668[44], 8}, {NULL}};
const Segment SGI78867[] = {{&S50668[44], 25}, {NULL}};
const Segment SGI78868[] = {{&S50668[30], 6}, {NULL}};
const Segment SGI78869[] = {{&S32417[44], 6}, {NULL}};
const Segment SGI78870[] = {{&S32417[44], 18}, {NULL}};
const Segment SGI78871[] = {{&S50668[117], 3}, {NULL}};
const Segment SGI78872[] = {{&S32417[30], 6}, {NULL}};
const Segment SGI78873[] = {{&S50668[83], 5}, {NULL}};
const Segment SGI78874[] = {{&S32417[31], 3}, {NULL}};
const Segment SGI78875[] = {{&S32417[76], 6}, {NULL}};
const Segment SGI78876[] = {{&S32418[44], 6}, {NULL}};
const Segment SGI78877[] = {{&S32418[44], 17}, {NULL}};
const Segment SGI78878[] = {{&S32418[30], 6}, {NULL}};
const Segment SGI78879[] = {{&S32418[31], 3}, {NULL}};
const Segment SGI78880[] = {{&S32418[75], 6}, {NULL}};
const Segment SGI78881[] = {{&S32420[44], 6}, {NULL}};
const Segment SGI78882[] = {{&S32420[44], 17}, {NULL}};
const Segment SGI78883[] = {{&S32420[30], 6}, {NULL}};
const Segment SGI78884[] = {{&S32420[75], 5}, {NULL}};
const Segment SGI78885[] = {{&S32421[44], 6}, {NULL}};
const Segment SGI78886[] = {{&S32421[44], 17}, {NULL}};
const Segment SGI78887[] = {{&S32421[30], 6}, {NULL}};
const Segment SGI78888[] = {{&S32421[31], 3}, {NULL}};
const Segment SGI78889[] = {{&S32421[75], 6}, {NULL}};
const Segment SGI78890[] = {{&S32423[44], 6}, {NULL}};
const Segment SGI78891[] = {{&S32423[44], 18}, {NULL}};
const Segment SGI78892[] = {{&S32423[30], 6}, {NULL}};
const Segment SGI78893[] = {{&S32423[31], 3}, {NULL}};
const Segment SGI78894[] = {{&S32423[76], 6}, {NULL}};
const Segment SGI78895[] = {{&S32425[38], 6}, {NULL}};
const Segment SGI78896[] = {{&S32425[31], 6}, {NULL}};
const Segment SGI78897[] = {{&S32428[44], 6}, {NULL}};
const Segment SGI78898[] = {{&S32428[44], 18}, {NULL}};
const Segment SGI78899[] = {{&S32428[30], 6}, {NULL}};
const Segment SGI78900[] = {{&S32428[31], 3}, {NULL}};
const Segment SGI78901[] = {{&S32428[76], 6}, {NULL}};
const Segment SGI78902[] = {{&S32429[44], 6}, {NULL}};
const Segment SGI78903[] = {{&S32429[44], 17}, {NULL}};
const Segment SGI78904[] = {{&S32429[30], 6}, {NULL}};
const Segment SGI78905[] = {{&S32429[31], 3}, {NULL}};
const Segment SGI78906[] = {{&S32429[75], 6}, {NULL}};
const Segment SGI78907[] = {{&S32493[44], 7}, {NULL}};
const Segment SGI78908[] = {{&S32493[44], 19}, {NULL}};
const Segment SGI78909[] = {{&S32493[30], 6}, {NULL}};
const Segment SGI78910[] = {{&S32493[31], 3}, {NULL}};
const Segment SGI78911[] = {{&S32493[77], 6}, {NULL}};
const Segment SGI78912[] = {{&S32494[44], 14}, {NULL}};
const Segment SGI78913[] = {{&S32494[44], 26}, {NULL}};
const Segment SGI78914[] = {{&S32494[30], 6}, {NULL}};
const Segment SGI78915[] = {{&S32494[31], 3}, {NULL}};
const Segment SGI78916[] = {{&S32494[84], 6}, {NULL}};
const Segment SGI78917[] = {{&S32495[44], 7}, {NULL}};
const Segment SGI78918[] = {{&S32495[44], 28}, {NULL}};
const Segment SGI78919[] = {{&S32495[30], 6}, {NULL}};
const Segment SGI78920[] = {{&S32495[31], 3}, {NULL}};
const Segment SGI78921[] = {{&S32495[86], 6}, {NULL}};
const Segment SGI78922[] = {{&S32503[44], 7}, {NULL}};
const Segment SGI78923[] = {{&S32503[44], 28}, {NULL}};
const Segment SGI78924[] = {{&S32503[30], 6}, {NULL}};
const Segment SGI78925[] = {{&S32503[31], 3}, {NULL}};
const Segment SGI78926[] = {{&S32503[86], 6}, {NULL}};
const Segment SGI78927[] = {{&S44818[44], 5}, {NULL}};
const Segment SGI78928[] = {{&S44818[44], 17}, {NULL}};
const Segment SGI78929[] = {{&S44818[30], 6}, {NULL}};
const Segment SGI78930[] = {{&S44818[109], 3}, {NULL}};
const Segment SGI78931[] = {{&S44818[75], 5}, {NULL}};
const Segment SGI78932[] = {{&S44819[44], 5}, {NULL}};
const Segment SGI78933[] = {{&S44819[44], 17}, {NULL}};
const Segment SGI78934[] = {{&S44819[30], 6}, {NULL}};
const Segment SGI78935[] = {{&S44819[109], 3}, {NULL}};
const Segment SGI78936[] = {{&S44819[75], 5}, {NULL}};
const Segment SGI78937[] = {{&S44262[42], 17}, {NULL}};
const Segment SGI78938[] = {{&S44262[42], 37}, {NULL}};
const Segment SGI78939[] = {{&S44262[30], 4}, {NULL}};
const Segment SGI78940[] = {{&S44262[128], 3}, {NULL}};
const Segment SGI78941[] = {{&S44262[93], 6}, {NULL}};
const Segment SGI78942[] = {{&S44263[42], 10}, {NULL}};
const Segment SGI78943[] = {{&S44263[42], 30}, {NULL}};
const Segment SGI78944[] = {{&S44263[30], 4}, {NULL}};
const Segment SGI78945[] = {{&S44263[121], 3}, {NULL}};
const Segment SGI78946[] = {{&S44263[86], 6}, {NULL}};
const Segment SGI78947[] = {{&S44264[42], 13}, {NULL}};
const Segment SGI78948[] = {{&S44264[42], 33}, {NULL}};
const Segment SGI78949[] = {{&S44264[30], 4}, {NULL}};
const Segment SGI78950[] = {{&S44264[124], 3}, {NULL}};
const Segment SGI78951[] = {{&S44264[89], 6}, {NULL}};
const Segment SGI78952[] = {{&S44265[36], 17}, {NULL}};
const Segment SGI78953[] = {{&S44265[31], 4}, {NULL}};
const Segment SGI78954[] = {{&S44266[36], 13}, {NULL}};
const Segment SGI78955[] = {{&S44266[31], 4}, {NULL}};
const Segment SGI78956[] = {{&S35612[44], 24}, {NULL}};
const Segment SGI78957[] = {{&S35612[44], 35}, {NULL}};
const Segment SGI78958[] = {{&S35612[30], 6}, {NULL}};
const Segment SGI78959[] = {{&S35612[96], 6}, {NULL}};
const Segment SGI78960[] = {{&S32826[34], 11}, {NULL}};
const Segment SGI78961[] = {{&S32826[27], 6}, {NULL}};
const Segment SGI78962[] = {{&S32826[98], 9}, {NULL}};
const Segment SGI78963[] = {{&S32826[71], 6}, {NULL}};
const Segment SGI78964[] = {{&S32828[34], 11}, {NULL}};
const Segment SGI78965[] = {{&S32828[27], 6}, {NULL}};
const Segment SGI78966[] = {{&S32828[98], 9}, {NULL}};
const Segment SGI78967[] = {{&S32828[71], 6}, {NULL}};
const Segment SGI78968[] = {{&S32829[34], 12}, {NULL}};
const Segment SGI78969[] = {{&S32829[27], 6}, {NULL}};
const Segment SGI78970[] = {{&S32829[99], 9}, {NULL}};
const Segment SGI78971[] = {{&S32829[72], 6}, {NULL}};
const Segment SGI78972[] = {{&S32830[34], 12}, {NULL}};
const Segment SGI78973[] = {{&S32830[27], 6}, {NULL}};
const Segment SGI78974[] = {{&S32830[99], 9}, {NULL}};
const Segment SGI78975[] = {{&S32830[72], 6}, {NULL}};
const Segment SGI78976[] = {{&S32831[34], 10}, {NULL}};
const Segment SGI78977[] = {{&S32831[27], 6}, {NULL}};
const Segment SGI78978[] = {{&S32831[97], 9}, {NULL}};
const Segment SGI78979[] = {{&S32831[70], 6}, {NULL}};
const Segment SGI78980[] = {{&S32833[34], 11}, {NULL}};
const Segment SGI78981[] = {{&S32833[27], 6}, {NULL}};
const Segment SGI78982[] = {{&S32833[98], 9}, {NULL}};
const Segment SGI78983[] = {{&S32833[71], 6}, {NULL}};
const Segment SGI78984[] = {{&S32834[34], 11}, {NULL}};
const Segment SGI78985[] = {{&S32834[27], 6}, {NULL}};
const Segment SGI78986[] = {{&S32834[98], 9}, {NULL}};
const Segment SGI78987[] = {{&S32834[71], 6}, {NULL}};
const Segment SGI78988[] = {{&S32835[34], 11}, {NULL}};
const Segment SGI78989[] = {{&S32835[27], 6}, {NULL}};
const Segment SGI78990[] = {{&S32835[98], 9}, {NULL}};
const Segment SGI78991[] = {{&S32835[71], 6}, {NULL}};
const Segment SGI78992[] = {{&S32836[34], 10}, {NULL}};
const Segment SGI78993[] = {{&S32836[27], 6}, {NULL}};
const Segment SGI78994[] = {{&S32836[97], 9}, {NULL}};
const Segment SGI78995[] = {{&S32836[70], 6}, {NULL}};
const Segment SGI78996[] = {{&S32837[34], 12}, {NULL}};
const Segment SGI78997[] = {{&S32837[27], 6}, {NULL}};
const Segment SGI78998[] = {{&S32837[99], 9}, {NULL}};
const Segment SGI78999[] = {{&S32837[72], 6}, {NULL}};
const Segment SGI79000[] = {{&S45138[44], 10}, {NULL}};
const Segment SGI79001[] = {{&S45138[44], 22}, {NULL}};
const Segment SGI79002[] = {{&S45138[30], 6}, {NULL}};
const Segment SGI79003[] = {{&S45138[133], 3}, {NULL}};
const Segment SGI79004[] = {{&S45138[99], 5}, {NULL}};
const Segment SGI79005[] = {{&S45138[35], 1}, {NULL}};
const Segment SGI79006[] = {{&S45138[68], 11}, {NULL}};
const Segment SGI79007[] = {{&S45139[44], 10}, {NULL}};
const Segment SGI79008[] = {{&S45139[44], 22}, {NULL}};
const Segment SGI79009[] = {{&S45139[30], 6}, {NULL}};
const Segment SGI79010[] = {{&S45139[139], 3}, {NULL}};
const Segment SGI79011[] = {{&S45139[105], 5}, {NULL}};
const Segment SGI79012[] = {{&S45139[35], 1}, {NULL}};
const Segment SGI79013[] = {{&S45139[68], 11}, {NULL}};
const Segment SGI79014[] = {{&S45140[44], 10}, {NULL}};
const Segment SGI79015[] = {{&S45140[44], 22}, {NULL}};
const Segment SGI79016[] = {{&S45140[30], 6}, {NULL}};
const Segment SGI79017[] = {{&S45140[133], 3}, {NULL}};
const Segment SGI79018[] = {{&S45140[99], 5}, {NULL}};
const Segment SGI79019[] = {{&S45140[74], 11}, {NULL}};
const Segment SGI79020[] = {{&S45141[44], 10}, {NULL}};
const Segment SGI79021[] = {{&S45141[44], 22}, {NULL}};
const Segment SGI79022[] = {{&S45141[30], 6}, {NULL}};
const Segment SGI79023[] = {{&S45141[125], 3}, {NULL}};
const Segment SGI79024[] = {{&S45141[91], 5}, {NULL}};
const Segment SGI79025[] = {{&S45141[68], 3}, {NULL}};
const Segment SGI79026[] = {{&S51315[44], 11}, {NULL}};
const Segment SGI79027[] = {{&S51315[44], 22}, {NULL}};
const Segment SGI79028[] = {{&S51315[30], 6}, {NULL}};
const Segment SGI79029[] = {{&S51315[114], 3}, {NULL}};
const Segment SGI79030[] = {{&S51315[80], 5}, {NULL}};
const Segment SGI79031[] = {{&S51316[44], 7}, {NULL}};
const Segment SGI79032[] = {{&S51316[44], 24}, {NULL}};
const Segment SGI79033[] = {{&S51316[30], 6}, {NULL}};
const Segment SGI79034[] = {{&S51316[116], 3}, {NULL}};
const Segment SGI79035[] = {{&S51316[82], 5}, {NULL}};
const Segment SGI79036[] = {{&S51317[44], 7}, {NULL}};
const Segment SGI79037[] = {{&S51317[44], 24}, {NULL}};
const Segment SGI79038[] = {{&S51317[30], 6}, {NULL}};
const Segment SGI79039[] = {{&S51317[116], 3}, {NULL}};
const Segment SGI79040[] = {{&S51317[82], 5}, {NULL}};
const Segment SGI79041[] = {{&S51318[44], 7}, {NULL}};
const Segment SGI79042[] = {{&S51318[44], 24}, {NULL}};
const Segment SGI79043[] = {{&S51318[30], 6}, {NULL}};
const Segment SGI79044[] = {{&S51318[116], 3}, {NULL}};
const Segment SGI79045[] = {{&S51318[82], 5}, {NULL}};
const Segment SGI79046[] = {{&S51319[44], 7}, {NULL}};
const Segment SGI79047[] = {{&S51319[44], 24}, {NULL}};
const Segment SGI79048[] = {{&S51319[30], 6}, {NULL}};
const Segment SGI79049[] = {{&S51319[116], 3}, {NULL}};
const Segment SGI79050[] = {{&S51319[82], 5}, {NULL}};
const Segment SGI79051[] = {{&S51320[38], 7}, {NULL}};
const Segment SGI79052[] = {{&S51320[31], 6}, {NULL}};
const Segment SGI79053[] = {{&S51321[44], 7}, {NULL}};
const Segment SGI79054[] = {{&S51321[44], 24}, {NULL}};
const Segment SGI79055[] = {{&S51321[30], 6}, {NULL}};
const Segment SGI79056[] = {{&S51321[116], 3}, {NULL}};
const Segment SGI79057[] = {{&S51321[82], 5}, {NULL}};
const Segment SGI79058[] = {{&S51322[44], 7}, {NULL}};
const Segment SGI79059[] = {{&S51322[44], 24}, {NULL}};
const Segment SGI79060[] = {{&S51322[30], 6}, {NULL}};
const Segment SGI79061[] = {{&S51322[116], 3}, {NULL}};
const Segment SGI79062[] = {{&S51322[82], 5}, {NULL}};
const Segment SGI79063[] = {{&S32956[44], 18}, {NULL}};
const Segment SGI79064[] = {{&S32956[44], 30}, {NULL}};
const Segment SGI79065[] = {{&S32956[30], 6}, {NULL}};
const Segment SGI79066[] = {{&S32956[31], 3}, {NULL}};
const Segment SGI79067[] = {{&S32956[88], 6}, {NULL}};
const Segment SGI79068[] = {{&S32957[42], 18}, {NULL}};
const Segment SGI79069[] = {{&S32957[30], 6}, {NULL}};
const Segment SGI79070[] = {{&S32957[31], 3}, {NULL}};
const Segment SGI79071[] = {{&S32957[86], 6}, {NULL}};
const Segment SGI79072[] = {{&S32958[38], 18}, {NULL}};
const Segment SGI79073[] = {{&S32958[31], 6}, {NULL}};
const Segment SGI79074[] = {{&S32963[44], 18}, {NULL}};
const Segment SGI79075[] = {{&S32963[44], 38}, {NULL}};
const Segment SGI79076[] = {{&S32963[30], 6}, {NULL}};
const Segment SGI79077[] = {{&S32963[31], 3}, {NULL}};
const Segment SGI79078[] = {{&S32963[96], 6}, {NULL}};
const Segment SGI79079[] = {{&S51464[44], 4}, {NULL}};
const Segment SGI79080[] = {{&S51464[44], 16}, {NULL}};
const Segment SGI79081[] = {{&S51464[30], 6}, {NULL}};
const Segment SGI79082[] = {{&S51464[108], 3}, {NULL}};
const Segment SGI79083[] = {{&S51464[74], 5}, {NULL}};
const Segment SGI79084[] = {{&S51465[44], 12}, {NULL}};
const Segment SGI79085[] = {{&S51465[44], 24}, {NULL}};
const Segment SGI79086[] = {{&S51465[30], 6}, {NULL}};
const Segment SGI79087[] = {{&S51465[116], 3}, {NULL}};
const Segment SGI79088[] = {{&S51465[82], 5}, {NULL}};
const Segment SGI79089[] = {{&S51466[38], 4}, {NULL}};
const Segment SGI79090[] = {{&S51466[31], 6}, {NULL}};
const Segment SGI79091[] = {{&S51467[38], 12}, {NULL}};
const Segment SGI79092[] = {{&S51467[31], 6}, {NULL}};
const Segment SGI79093[] = {{&S33167[44], 15}, {NULL}};
const Segment SGI79094[] = {{&S33167[44], 27}, {NULL}};
const Segment SGI79095[] = {{&S33167[30], 6}, {NULL}};
const Segment SGI79096[] = {{&S33167[31], 3}, {NULL}};
const Segment SGI79097[] = {{&S33167[85], 6}, {NULL}};
const Segment SGI79098[] = {{&S33168[44], 12}, {NULL}};
const Segment SGI79099[] = {{&S33168[44], 24}, {NULL}};
const Segment SGI79100[] = {{&S33168[30], 6}, {NULL}};
const Segment SGI79101[] = {{&S33168[31], 3}, {NULL}};
const Segment SGI79102[] = {{&S33168[82], 6}, {NULL}};
const Segment SGI79103[] = {{&S33169[38], 12}, {NULL}};
const Segment SGI79104[] = {{&S33169[31], 6}, {NULL}};
const Segment SGI79105[] = {{&S33171[44], 12}, {NULL}};
const Segment SGI79106[] = {{&S33171[44], 24}, {NULL}};
const Segment SGI79107[] = {{&S33171[30], 6}, {NULL}};
const Segment SGI79108[] = {{&S33171[31], 3}, {NULL}};
const Segment SGI79109[] = {{&S33171[82], 6}, {NULL}};
const Segment SGI79110[] = {{&S33174[44], 11}, {NULL}};
const Segment SGI79111[] = {{&S33174[44], 23}, {NULL}};
const Segment SGI79112[] = {{&S33174[30], 6}, {NULL}};
const Segment SGI79113[] = {{&S33174[31], 3}, {NULL}};
const Segment SGI79114[] = {{&S33174[81], 6}, {NULL}};
const Segment SGI79115[] = {{&S44832[42], 10}, {NULL}};
const Segment SGI79116[] = {{&S44832[42], 22}, {NULL}};
const Segment SGI79117[] = {{&S44832[30], 4}, {NULL}};
const Segment SGI79118[] = {{&S44832[113], 3}, {NULL}};
const Segment SGI79119[] = {{&S44832[78], 6}, {NULL}};
const Segment SGI79120[] = {{&S44833[42], 29}, {NULL}};
const Segment SGI79121[] = {{&S44833[42], 41}, {NULL}};
const Segment SGI79122[] = {{&S44833[30], 4}, {NULL}};
const Segment SGI79123[] = {{&S44833[132], 3}, {NULL}};
const Segment SGI79124[] = {{&S44833[97], 6}, {NULL}};
const Segment SGI79125[] = {{&S44834[42], 29}, {NULL}};
const Segment SGI79126[] = {{&S44834[42], 41}, {NULL}};
const Segment SGI79127[] = {{&S44834[30], 4}, {NULL}};
const Segment SGI79128[] = {{&S44834[132], 3}, {NULL}};
const Segment SGI79129[] = {{&S44834[97], 6}, {NULL}};
const Segment SGI79130[] = {{&S44835[36], 10}, {NULL}};
const Segment SGI79131[] = {{&S44835[31], 4}, {NULL}};
const Segment SGI79132[] = {{&S44836[42], 29}, {NULL}};
const Segment SGI79133[] = {{&S44836[42], 41}, {NULL}};
const Segment SGI79134[] = {{&S44836[30], 4}, {NULL}};
const Segment SGI79135[] = {{&S44836[132], 3}, {NULL}};
const Segment SGI79136[] = {{&S44836[97], 6}, {NULL}};
const Segment SGI79137[] = {{&S44837[40], 10}, {NULL}};
const Segment SGI79138[] = {{&S44837[30], 4}, {NULL}};
const Segment SGI79139[] = {{&S44837[111], 3}, {NULL}};
const Segment SGI79140[] = {{&S44837[76], 6}, {NULL}};
const Segment SGI79141[] = {{&S44838[42], 10}, {NULL}};
const Segment SGI79142[] = {{&S44838[42], 22}, {NULL}};
const Segment SGI79143[] = {{&S44838[30], 4}, {NULL}};
const Segment SGI79144[] = {{&S44838[113], 3}, {NULL}};
const Segment SGI79145[] = {{&S44838[78], 6}, {NULL}};
const Segment SGI79146[] = {{&S44839[42], 10}, {NULL}};
const Segment SGI79147[] = {{&S44839[42], 28}, {NULL}};
const Segment SGI79148[] = {{&S44839[30], 4}, {NULL}};
const Segment SGI79149[] = {{&S44840[42], 10}, {NULL}};
const Segment SGI79150[] = {{&S44840[42], 28}, {NULL}};
const Segment SGI79151[] = {{&S44840[30], 4}, {NULL}};
const Segment SGI79152[] = {{&S44890[42], 6}, {NULL}};
const Segment SGI79153[] = {{&S44890[42], 18}, {NULL}};
const Segment SGI79154[] = {{&S44890[30], 4}, {NULL}};
const Segment SGI79155[] = {{&S44890[109], 3}, {NULL}};
const Segment SGI79156[] = {{&S44890[74], 6}, {NULL}};
const Segment SGI79157[] = {{&S44891[44], 6}, {NULL}};
const Segment SGI79158[] = {{&S44891[44], 18}, {NULL}};
const Segment SGI79159[] = {{&S44891[30], 6}, {NULL}};
const Segment SGI79160[] = {{&S44891[111], 3}, {NULL}};
const Segment SGI79161[] = {{&S44891[76], 6}, {NULL}};
const Segment SGI79162[] = {{&S44892[44], 6}, {NULL}};
const Segment SGI79163[] = {{&S44892[44], 18}, {NULL}};
const Segment SGI79164[] = {{&S44892[30], 6}, {NULL}};
const Segment SGI79165[] = {{&S51689[44], 30}, {NULL}};
const Segment SGI79166[] = {{&S51689[44], 47}, {NULL}};
const Segment SGI79167[] = {{&S51689[30], 6}, {NULL}};
const Segment SGI79168[] = {{&S44892[111], 3}, {NULL}};
const Segment SGI79169[] = {{&S44892[76], 6}, {NULL}};
const Segment SGI79170[] = {{&S51689[139], 3}, {NULL}};
const Segment SGI79171[] = {{&S51689[105], 5}, {NULL}};
const Segment SGI79172[] = {{&S44893[44], 6}, {NULL}};
const Segment SGI79173[] = {{&S44893[44], 18}, {NULL}};
const Segment SGI79174[] = {{&S44893[30], 6}, {NULL}};
const Segment SGI79175[] = {{&S44893[111], 3}, {NULL}};
const Segment SGI79176[] = {{&S51690[44], 11}, {NULL}};
const Segment SGI79177[] = {{&S51690[44], 28}, {NULL}};
const Segment SGI79178[] = {{&S44893[76], 6}, {NULL}};
const Segment SGI79179[] = {{&S51690[30], 6}, {NULL}};
const Segment SGI79180[] = {{&S51690[120], 3}, {NULL}};
const Segment SGI79181[] = {{&S51690[86], 5}, {NULL}};
const Segment SGI79182[] = {{&S44894[36], 6}, {NULL}};
const Segment SGI79183[] = {{&S44894[31], 4}, {NULL}};
const Segment SGI79184[] = {{&S51691[44], 18}, {NULL}};
const Segment SGI79185[] = {{&S51691[44], 35}, {NULL}};
const Segment SGI79186[] = {{&S51691[30], 6}, {NULL}};
const Segment SGI79187[] = {{&S44895[44], 6}, {NULL}};
const Segment SGI79188[] = {{&S51691[127], 3}, {NULL}};
const Segment SGI79189[] = {{&S44895[44], 18}, {NULL}};
const Segment SGI79190[] = {{&S44895[30], 6}, {NULL}};
const Segment SGI79191[] = {{&S51691[93], 5}, {NULL}};
const Segment SGI79192[] = {{&S44895[111], 3}, {NULL}};
const Segment SGI79193[] = {{&S44895[76], 6}, {NULL}};
const Segment SGI79194[] = {{&S51692[38], 11}, {NULL}};
const Segment SGI79195[] = {{&S51692[31], 6}, {NULL}};
const Segment SGI79196[] = {{&S44896[44], 6}, {NULL}};
const Segment SGI79197[] = {{&S44896[44], 18}, {NULL}};
const Segment SGI79198[] = {{&S44896[30], 6}, {NULL}};
const Segment SGI79199[] = {{&S44896[111], 3}, {NULL}};
const Segment SGI79200[] = {{&S44896[76], 6}, {NULL}};
const Segment SGI79201[] = {{&S44897[38], 6}, {NULL}};
const Segment SGI79202[] = {{&S44897[31], 6}, {NULL}};
const Segment SGI79203[] = {{&S44898[38], 6}, {NULL}};
const Segment SGI79204[] = {{&S44898[31], 6}, {NULL}};
const Segment SGI79205[] = {{&S44899[42], 6}, {NULL}};
const Segment SGI79206[] = {{&S44899[42], 17}, {NULL}};
const Segment SGI79207[] = {{&S44899[30], 4}, {NULL}};
const Segment SGI79208[] = {{&S44899[108], 3}, {NULL}};
const Segment SGI79209[] = {{&S44899[73], 6}, {NULL}};
const Segment SGI79210[] = {{&S44900[42], 6}, {NULL}};
const Segment SGI79211[] = {{&S44900[42], 17}, {NULL}};
const Segment SGI79212[] = {{&S44900[30], 4}, {NULL}};
const Segment SGI79213[] = {{&S44900[108], 3}, {NULL}};
const Segment SGI79214[] = {{&S44900[73], 6}, {NULL}};
const Segment SGI79215[] = {{&S44901[42], 6}, {NULL}};
const Segment SGI79216[] = {{&S44901[42], 17}, {NULL}};
const Segment SGI79217[] = {{&S44901[30], 4}, {NULL}};
const Segment SGI79218[] = {{&S44901[108], 3}, {NULL}};
const Segment SGI79219[] = {{&S44901[73], 6}, {NULL}};
const Segment SGI79220[] = {{&S44902[42], 6}, {NULL}};
const Segment SGI79221[] = {{&S44902[42], 17}, {NULL}};
const Segment SGI79222[] = {{&S44902[30], 4}, {NULL}};
const Segment SGI79223[] = {{&S44902[108], 3}, {NULL}};
const Segment SGI79224[] = {{&S44902[73], 6}, {NULL}};
const Segment SGI79225[] = {{&S44903[42], 6}, {NULL}};
const Segment SGI79226[] = {{&S44903[42], 17}, {NULL}};
const Segment SGI79227[] = {{&S44903[30], 4}, {NULL}};
const Segment SGI79228[] = {{&S44903[108], 3}, {NULL}};
const Segment SGI79229[] = {{&S44903[73], 6}, {NULL}};
const Segment SGI79230[] = {{&S44904[42], 6}, {NULL}};
const Segment SGI79231[] = {{&S44904[42], 17}, {NULL}};
const Segment SGI79232[] = {{&S44904[30], 4}, {NULL}};
const Segment SGI79233[] = {{&S44904[108], 3}, {NULL}};
const Segment SGI79234[] = {{&S44904[73], 6}, {NULL}};
const Segment SGI79235[] = {{&S44905[42], 6}, {NULL}};
const Segment SGI79236[] = {{&S44905[42], 18}, {NULL}};
const Segment SGI79237[] = {{&S44905[30], 4}, {NULL}};
const Segment SGI79238[] = {{&S44905[109], 3}, {NULL}};
const Segment SGI79239[] = {{&S44905[74], 6}, {NULL}};
const Segment SGI79240[] = {{&S44906[42], 6}, {NULL}};
const Segment SGI79241[] = {{&S44906[42], 18}, {NULL}};
const Segment SGI79242[] = {{&S44906[30], 4}, {NULL}};
const Segment SGI79243[] = {{&S44906[109], 3}, {NULL}};
const Segment SGI79244[] = {{&S44906[74], 6}, {NULL}};
const Segment SGI79245[] = {{&S44907[42], 6}, {NULL}};
const Segment SGI79246[] = {{&S44907[42], 17}, {NULL}};
const Segment SGI79247[] = {{&S44907[30], 4}, {NULL}};
const Segment SGI79248[] = {{&S44907[108], 3}, {NULL}};
const Segment SGI79249[] = {{&S44907[73], 6}, {NULL}};
const Segment SGI79250[] = {{&S44908[44], 6}, {NULL}};
const Segment SGI79251[] = {{&S44908[44], 18}, {NULL}};
const Segment SGI79252[] = {{&S44908[30], 6}, {NULL}};
const Segment SGI79253[] = {{&S44908[111], 3}, {NULL}};
const Segment SGI79254[] = {{&S44908[76], 6}, {NULL}};
const Segment SGI79255[] = {{&S33333[44], 18}, {NULL}};
const Segment SGI79256[] = {{&S33333[44], 30}, {NULL}};
const Segment SGI79257[] = {{&S33333[30], 6}, {NULL}};
const Segment SGI79258[] = {{&S33333[91], 6}, {NULL}};
const Segment SGI79259[] = {{&S44909[42], 6}, {NULL}};
const Segment SGI79260[] = {{&S44909[42], 18}, {NULL}};
const Segment SGI79261[] = {{&S44909[30], 4}, {NULL}};
const Segment SGI79262[] = {{&S44909[109], 3}, {NULL}};
const Segment SGI79263[] = {{&S44909[74], 6}, {NULL}};
const Segment SGI79264[] = {{&S33334[44], 18}, {NULL}};
const Segment SGI79265[] = {{&S33334[44], 30}, {NULL}};
const Segment SGI79266[] = {{&S33334[30], 6}, {NULL}};
const Segment SGI79267[] = {{&S33334[31], 3}, {NULL}};
const Segment SGI79268[] = {{&S33334[88], 6}, {NULL}};
const Segment SGI79269[] = {{&S44910[42], 6}, {NULL}};
const Segment SGI79270[] = {{&S44910[42], 17}, {NULL}};
const Segment SGI79271[] = {{&S44910[30], 4}, {NULL}};
const Segment SGI79272[] = {{&S44910[108], 3}, {NULL}};
const Segment SGI79273[] = {{&S44910[73], 6}, {NULL}};
const Segment SGI79274[] = {{&S33335[44], 18}, {NULL}};
const Segment SGI79275[] = {{&S33335[44], 30}, {NULL}};
const Segment SGI79276[] = {{&S33335[30], 6}, {NULL}};
const Segment SGI79277[] = {{&S33335[31], 3}, {NULL}};
const Segment SGI79278[] = {{&S44911[42], 6}, {NULL}};
const Segment SGI79279[] = {{&S33335[88], 6}, {NULL}};
const Segment SGI79280[] = {{&S44911[42], 18}, {NULL}};
const Segment SGI79281[] = {{&S44911[30], 4}, {NULL}};
const Segment SGI79282[] = {{&S44911[109], 3}, {NULL}};
const Segment SGI79283[] = {{&S44911[74], 6}, {NULL}};
const Segment SGI79284[] = {{&S44912[42], 6}, {NULL}};
const Segment SGI79285[] = {{&S44912[42], 17}, {NULL}};
const Segment SGI79286[] = {{&S44912[30], 4}, {NULL}};
const Segment SGI79287[] = {{&S44912[108], 3}, {NULL}};
const Segment SGI79288[] = {{&S44912[73], 6}, {NULL}};
const Segment SGI79289[] = {{&S33338[38], 18}, {NULL}};
const Segment SGI79290[] = {{&S33338[31], 6}, {NULL}};
const Segment SGI79291[] = {{&S44913[42], 6}, {NULL}};
const Segment SGI79292[] = {{&S44913[42], 17}, {NULL}};
const Segment SGI79293[] = {{&S44913[30], 4}, {NULL}};
const Segment SGI79294[] = {{&S44913[108], 3}, {NULL}};
const Segment SGI79295[] = {{&S44913[73], 6}, {NULL}};
const Segment SGI79296[] = {{&S44914[44], 6}, {NULL}};
const Segment SGI79297[] = {{&S33341[38], 18}, {NULL}};
const Segment SGI79298[] = {{&S44914[44], 18}, {NULL}};
const Segment SGI79299[] = {{&S44914[30], 6}, {NULL}};
const Segment SGI79300[] = {{&S33341[31], 6}, {NULL}};
const Segment SGI79301[] = {{&S44914[111], 3}, {NULL}};
const Segment SGI79302[] = {{&S44914[76], 6}, {NULL}};
const Segment SGI79303[] = {{&S44915[38], 6}, {NULL}};
const Segment SGI79304[] = {{&S44915[31], 6}, {NULL}};
const Segment SGI79305[] = {{&S44916[44], 6}, {NULL}};
const Segment SGI79306[] = {{&S44916[44], 18}, {NULL}};
const Segment SGI79307[] = {{&S44916[30], 6}, {NULL}};
const Segment SGI79308[] = {{&S44916[111], 3}, {NULL}};
const Segment SGI79309[] = {{&S44916[76], 6}, {NULL}};
const Segment SGI79310[] = {{&S44917[44], 6}, {NULL}};
const Segment SGI79311[] = {{&S44917[44], 17}, {NULL}};
const Segment SGI79312[] = {{&S44917[30], 6}, {NULL}};
const Segment SGI79313[] = {{&S44917[110], 3}, {NULL}};
const Segment SGI79314[] = {{&S44917[75], 6}, {NULL}};
const Segment SGI79315[] = {{&S44918[44], 6}, {NULL}};
const Segment SGI79316[] = {{&S44918[44], 17}, {NULL}};
const Segment SGI79317[] = {{&S44918[30], 6}, {NULL}};
const Segment SGI79318[] = {{&S44918[110], 3}, {NULL}};
const Segment SGI79319[] = {{&S44918[75], 6}, {NULL}};
const Segment SGI79320[] = {{&S44919[44], 6}, {NULL}};
const Segment SGI79321[] = {{&S44919[44], 18}, {NULL}};
const Segment SGI79322[] = {{&S44919[30], 6}, {NULL}};
const Segment SGI79323[] = {{&S44919[111], 3}, {NULL}};
const Segment SGI79324[] = {{&S44919[76], 6}, {NULL}};
const Segment SGI79325[] = {{&S44920[42], 6}, {NULL}};
const Segment SGI79326[] = {{&S44920[42], 17}, {NULL}};
const Segment SGI79327[] = {{&S44920[30], 4}, {NULL}};
const Segment SGI79328[] = {{&S44920[108], 3}, {NULL}};
const Segment SGI79329[] = {{&S44920[73], 6}, {NULL}};
const Segment SGI79330[] = {{&S44921[44], 6}, {NULL}};
const Segment SGI79331[] = {{&S44921[44], 18}, {NULL}};
const Segment SGI79332[] = {{&S44921[30], 6}, {NULL}};
const Segment SGI79333[] = {{&S44921[111], 3}, {NULL}};
const Segment SGI79334[] = {{&S44921[76], 6}, {NULL}};
const Segment SGI79335[] = {{&S44922[44], 6}, {NULL}};
const Segment SGI79336[] = {{&S44922[44], 18}, {NULL}};
const Segment SGI79337[] = {{&S44922[30], 6}, {NULL}};
const Segment SGI79338[] = {{&S44922[111], 3}, {NULL}};
const Segment SGI79339[] = {{&S44922[76], 6}, {NULL}};
const Segment SGI79340[] = {{&S44923[44], 6}, {NULL}};
const Segment SGI79341[] = {{&S44923[44], 18}, {NULL}};
const Segment SGI79342[] = {{&S44923[30], 6}, {NULL}};
const Segment SGI79343[] = {{&S44923[111], 3}, {NULL}};
const Segment SGI79344[] = {{&S44923[76], 6}, {NULL}};
const Segment SGI79345[] = {{&S44924[42], 6}, {NULL}};
const Segment SGI79346[] = {{&S44924[42], 18}, {NULL}};
const Segment SGI79347[] = {{&S44924[30], 4}, {NULL}};
const Segment SGI79348[] = {{&S44924[109], 3}, {NULL}};
const Segment SGI79349[] = {{&S44924[74], 6}, {NULL}};
const Segment SGI79350[] = {{&S44925[44], 6}, {NULL}};
const Segment SGI79351[] = {{&S44925[44], 18}, {NULL}};
const Segment SGI79352[] = {{&S44925[30], 6}, {NULL}};
const Segment SGI79353[] = {{&S44925[111], 3}, {NULL}};
const Segment SGI79354[] = {{&S44925[76], 6}, {NULL}};
const Segment SGI79355[] = {{&S44926[44], 6}, {NULL}};
const Segment SGI79356[] = {{&S44926[44], 18}, {NULL}};
const Segment SGI79357[] = {{&S44926[30], 6}, {NULL}};
const Segment SGI79358[] = {{&S44926[111], 3}, {NULL}};
const Segment SGI79359[] = {{&S44926[76], 6}, {NULL}};
const Segment SGI79360[] = {{&S44927[42], 6}, {NULL}};
const Segment SGI79361[] = {{&S44927[30], 6}, {NULL}};
const Segment SGI79362[] = {{&S44927[109], 3}, {NULL}};
const Segment SGI79363[] = {{&S44927[74], 6}, {NULL}};
const Segment SGI79364[] = {{&S44928[44], 6}, {NULL}};
const Segment SGI79365[] = {{&S44928[44], 18}, {NULL}};
const Segment SGI79366[] = {{&S44928[30], 6}, {NULL}};
const Segment SGI79367[] = {{&S44928[111], 3}, {NULL}};
const Segment SGI79368[] = {{&S44928[76], 6}, {NULL}};
const Segment SGI79369[] = {{&S44929[44], 6}, {NULL}};
const Segment SGI79370[] = {{&S44929[44], 18}, {NULL}};
const Segment SGI79371[] = {{&S44929[30], 6}, {NULL}};
const Segment SGI79372[] = {{&S44929[111], 3}, {NULL}};
const Segment SGI79373[] = {{&S44929[76], 6}, {NULL}};
const Segment SGI79374[] = {{&S44930[38], 6}, {NULL}};
const Segment SGI79375[] = {{&S44930[31], 6}, {NULL}};
const Segment SGI79376[] = {{&S44931[44], 7}, {NULL}};
const Segment SGI79377[] = {{&S44931[44], 23}, {NULL}};
const Segment SGI79378[] = {{&S44931[30], 6}, {NULL}};
const Segment SGI79379[] = {{&S44931[116], 3}, {NULL}};
const Segment SGI79380[] = {{&S44931[81], 6}, {NULL}};
const Segment SGI79381[] = {{&S51909[44], 10}, {NULL}};
const Segment SGI79382[] = {{&S51909[44], 27}, {NULL}};
const Segment SGI79383[] = {{&S51909[30], 6}, {NULL}};
const Segment SGI79384[] = {{&S51909[119], 3}, {NULL}};
const Segment SGI79385[] = {{&S51909[85], 5}, {NULL}};
const Segment SGI79386[] = {{&S51910[38], 10}, {NULL}};
const Segment SGI79387[] = {{&S51910[31], 6}, {NULL}};
const Segment SGI79388[] = {{&S45127[42], 7}, {NULL}};
const Segment SGI79389[] = {{&S45127[42], 23}, {NULL}};
const Segment SGI79390[] = {{&S45127[30], 4}, {NULL}};
const Segment SGI79391[] = {{&S45127[114], 3}, {NULL}};
const Segment SGI79392[] = {{&S45127[79], 6}, {NULL}};
const Segment SGI79393[] = {{&S45128[42], 7}, {NULL}};
const Segment SGI79394[] = {{&S45128[42], 27}, {NULL}};
const Segment SGI79395[] = {{&S45128[30], 4}, {NULL}};
const Segment SGI79396[] = {{&S45128[118], 3}, {NULL}};
const Segment SGI79397[] = {{&S45128[83], 6}, {NULL}};
const Segment SGI79398[] = {{&S45129[42], 7}, {NULL}};
const Segment SGI79399[] = {{&S45129[42], 21}, {NULL}};
const Segment SGI79400[] = {{&S45129[30], 4}, {NULL}};
const Segment SGI79401[] = {{&S45129[112], 3}, {NULL}};
const Segment SGI79402[] = {{&S45129[77], 6}, {NULL}};
const Segment SGI79403[] = {{&S45130[42], 7}, {NULL}};
const Segment SGI79404[] = {{&S45130[42], 21}, {NULL}};
const Segment SGI79405[] = {{&S45130[30], 4}, {NULL}};
const Segment SGI79406[] = {{&S45130[112], 3}, {NULL}};
const Segment SGI79407[] = {{&S45130[77], 6}, {NULL}};
const Segment SGI79408[] = {{&S45131[42], 7}, {NULL}};
const Segment SGI79409[] = {{&S45131[42], 23}, {NULL}};
const Segment SGI79410[] = {{&S45131[30], 4}, {NULL}};
const Segment SGI79411[] = {{&S45131[114], 3}, {NULL}};
const Segment SGI79412[] = {{&S45131[79], 6}, {NULL}};
const Segment SGI79413[] = {{&S45132[42], 7}, {NULL}};
const Segment SGI79414[] = {{&S45132[42], 21}, {NULL}};
const Segment SGI79415[] = {{&S45132[30], 4}, {NULL}};
const Segment SGI79416[] = {{&S45132[112], 3}, {NULL}};
const Segment SGI79417[] = {{&S45132[77], 6}, {NULL}};
const Segment SGI79418[] = {{&S45133[42], 7}, {NULL}};
const Segment SGI79419[] = {{&S45133[42], 21}, {NULL}};
const Segment SGI79420[] = {{&S45133[30], 4}, {NULL}};
const Segment SGI79421[] = {{&S45133[112], 3}, {NULL}};
const Segment SGI79422[] = {{&S45133[77], 6}, {NULL}};
const Segment SGI79423[] = {{&S45134[42], 7}, {NULL}};
const Segment SGI79424[] = {{&S45134[42], 21}, {NULL}};
const Segment SGI79425[] = {{&S45134[30], 4}, {NULL}};
const Segment SGI79426[] = {{&S45134[112], 3}, {NULL}};
const Segment SGI79427[] = {{&S45134[77], 6}, {NULL}};
const Segment SGI79428[] = {{&S45135[42], 7}, {NULL}};
const Segment SGI79429[] = {{&S45135[42], 23}, {NULL}};
const Segment SGI79430[] = {{&S45135[30], 4}, {NULL}};
const Segment SGI79431[] = {{&S45135[114], 3}, {NULL}};
const Segment SGI79432[] = {{&S45135[79], 6}, {NULL}};
const Segment SGI79433[] = {{&S45136[36], 7}, {NULL}};
const Segment SGI79434[] = {{&S45136[31], 4}, {NULL}};
const Segment SGI79435[] = {{&S45137[42], 7}, {NULL}};
const Segment SGI79436[] = {{&S45137[42], 23}, {NULL}};
const Segment SGI79437[] = {{&S45137[30], 4}, {NULL}};
const Segment SGI79438[] = {{&S45137[114], 3}, {NULL}};
const Segment SGI79439[] = {{&S45137[79], 6}, {NULL}};
const Segment SGI79440[] = {{&S33618[44], 7}, {NULL}};
const Segment SGI79441[] = {{&S33618[44], 21}, {NULL}};
const Segment SGI79442[] = {{&S33618[30], 6}, {NULL}};
const Segment SGI79443[] = {{&S33618[31], 3}, {NULL}};
const Segment SGI79444[] = {{&S33618[79], 6}, {NULL}};
const Segment SGI79445[] = {{&S33619[44], 7}, {NULL}};
const Segment SGI79446[] = {{&S33619[44], 21}, {NULL}};
const Segment SGI79447[] = {{&S33619[30], 6}, {NULL}};
const Segment SGI79448[] = {{&S33619[31], 3}, {NULL}};
const Segment SGI79449[] = {{&S33619[79], 6}, {NULL}};
const Segment SGI79450[] = {{&S33620[44], 7}, {NULL}};
const Segment SGI79451[] = {{&S33620[44], 28}, {NULL}};
const Segment SGI79452[] = {{&S33620[30], 6}, {NULL}};
const Segment SGI79453[] = {{&S33620[31], 3}, {NULL}};
const Segment SGI79454[] = {{&S33620[86], 6}, {NULL}};
const Segment SGI79455[] = {{&S33622[38], 7}, {NULL}};
const Segment SGI79456[] = {{&S33622[31], 6}, {NULL}};
const Segment SGI79457[] = {{&S33626[44], 22}, {NULL}};
const Segment SGI79458[] = {{&S33626[44], 34}, {NULL}};
const Segment SGI79459[] = {{&S33626[30], 6}, {NULL}};
const Segment SGI79460[] = {{&S33626[31], 3}, {NULL}};
const Segment SGI79461[] = {{&S33626[92], 6}, {NULL}};
const Segment SGI79462[] = {{&S33627[44], 13}, {NULL}};
const Segment SGI79463[] = {{&S33627[44], 25}, {NULL}};
const Segment SGI79464[] = {{&S33627[30], 6}, {NULL}};
const Segment SGI79465[] = {{&S33627[31], 3}, {NULL}};
const Segment SGI79466[] = {{&S33627[83], 6}, {NULL}};
const Segment SGI79467[] = {{&S33628[44], 25}, {NULL}};
const Segment SGI79468[] = {{&S33628[44], 37}, {NULL}};
const Segment SGI79469[] = {{&S33628[30], 6}, {NULL}};
const Segment SGI79470[] = {{&S33628[31], 3}, {NULL}};
const Segment SGI79471[] = {{&S33628[95], 6}, {NULL}};
const Segment SGI79472[] = {{&S33629[38], 13}, {NULL}};
const Segment SGI79473[] = {{&S33629[31], 6}, {NULL}};
const Segment SGI79474[] = {{&S33630[44], 25}, {NULL}};
const Segment SGI79475[] = {{&S33630[44], 37}, {NULL}};
const Segment SGI79476[] = {{&S33630[30], 6}, {NULL}};
const Segment SGI79477[] = {{&S33630[31], 3}, {NULL}};
const Segment SGI79478[] = {{&S33630[95], 6}, {NULL}};
const Segment SGI79479[] = {{&S33631[44], 25}, {NULL}};
const Segment SGI79480[] = {{&S33631[44], 37}, {NULL}};
const Segment SGI79481[] = {{&S33631[30], 6}, {NULL}};
const Segment SGI79482[] = {{&S33631[31], 3}, {NULL}};
const Segment SGI79483[] = {{&S33631[95], 6}, {NULL}};
const Segment SGI79484[] = {{&S33632[44], 24}, {NULL}};
const Segment SGI79485[] = {{&S33632[44], 36}, {NULL}};
const Segment SGI79486[] = {{&S33632[30], 6}, {NULL}};
const Segment SGI79487[] = {{&S33632[31], 3}, {NULL}};
const Segment SGI79488[] = {{&S33632[94], 6}, {NULL}};
const Segment SGI79489[] = {{&S33633[42], 25}, {NULL}};
const Segment SGI79490[] = {{&S33633[30], 6}, {NULL}};
const Segment SGI79491[] = {{&S33633[31], 3}, {NULL}};
const Segment SGI79492[] = {{&S33633[93], 6}, {NULL}};
const Segment SGI79493[] = {{&S33634[42], 13}, {NULL}};
const Segment SGI79494[] = {{&S33634[30], 6}, {NULL}};
const Segment SGI79495[] = {{&S33634[31], 3}, {NULL}};
const Segment SGI79496[] = {{&S33634[81], 6}, {NULL}};
const Segment SGI79497[] = {{&S33635[44], 24}, {NULL}};
const Segment SGI79498[] = {{&S33635[44], 36}, {NULL}};
const Segment SGI79499[] = {{&S33635[30], 6}, {NULL}};
const Segment SGI79500[] = {{&S33635[31], 3}, {NULL}};
const Segment SGI79501[] = {{&S33635[94], 6}, {NULL}};
const Segment SGI79502[] = {{&S52162[44], 10}, {NULL}};
const Segment SGI79503[] = {{&S52162[44], 22}, {NULL}};
const Segment SGI79504[] = {{&S52162[30], 6}, {NULL}};
const Segment SGI79505[] = {{&S52162[114], 3}, {NULL}};
const Segment SGI79506[] = {{&S52162[80], 5}, {NULL}};
const Segment SGI79507[] = {{&S52163[44], 10}, {NULL}};
const Segment SGI79508[] = {{&S52163[44], 22}, {NULL}};
const Segment SGI79509[] = {{&S52163[30], 6}, {NULL}};
const Segment SGI79510[] = {{&S52163[114], 3}, {NULL}};
const Segment SGI79511[] = {{&S52163[80], 5}, {NULL}};
const Segment SGI79512[] = {{&S52164[44], 7}, {NULL}};
const Segment SGI79513[] = {{&S52164[44], 19}, {NULL}};
const Segment SGI79514[] = {{&S52164[30], 6}, {NULL}};
const Segment SGI79515[] = {{&S52164[111], 3}, {NULL}};
const Segment SGI79516[] = {{&S52164[77], 5}, {NULL}};
const Segment SGI79517[] = {{&S52165[38], 10}, {NULL}};
const Segment SGI79518[] = {{&S52165[31], 6}, {NULL}};
const Segment SGI79519[] = {{&S52230[34], 10}, {NULL}};
const Segment SGI79520[] = {{&S52230[27], 6}, {NULL}};
const Segment SGI79521[] = {{&S52230[109], 2}, {NULL}};
const Segment SGI79522[] = {{&S52230[75], 6}, {NULL}};
const Segment SGI79523[] = {{&S52230[143], 8}, {NULL}};
const Segment SGI79524[] = {{&S52231[34], 11}, {NULL}};
const Segment SGI79525[] = {{&S52231[27], 6}, {NULL}};
const Segment SGI79526[] = {{&S52231[110], 2}, {NULL}};
const Segment SGI79527[] = {{&S52231[76], 6}, {NULL}};
const Segment SGI79528[] = {{&S52231[144], 8}, {NULL}};
const Segment SGI79529[] = {{&S52233[34], 11}, {NULL}};
const Segment SGI79530[] = {{&S52233[27], 6}, {NULL}};
const Segment SGI79531[] = {{&S52233[110], 2}, {NULL}};
const Segment SGI79532[] = {{&S52233[76], 6}, {NULL}};
const Segment SGI79533[] = {{&S52233[144], 8}, {NULL}};
const Segment SGI79534[] = {{&S45352[42], 13}, {NULL}};
const Segment SGI79535[] = {{&S45352[42], 25}, {NULL}};
const Segment SGI79536[] = {{&S45352[30], 4}, {NULL}};
const Segment SGI79537[] = {{&S45352[116], 3}, {NULL}};
const Segment SGI79538[] = {{&S45352[81], 6}, {NULL}};
const Segment SGI79539[] = {{&S45353[44], 13}, {NULL}};
const Segment SGI79540[] = {{&S45353[44], 25}, {NULL}};
const Segment SGI79541[] = {{&S45353[30], 6}, {NULL}};
const Segment SGI79542[] = {{&S45353[118], 3}, {NULL}};
const Segment SGI79543[] = {{&S45353[83], 6}, {NULL}};
const Segment SGI79544[] = {{&S45354[38], 13}, {NULL}};
const Segment SGI79545[] = {{&S45354[31], 6}, {NULL}};
const Segment SGI79546[] = {{&S45355[36], 13}, {NULL}};
const Segment SGI79547[] = {{&S45355[31], 4}, {NULL}};
const Segment SGI79548[] = {{&S52322[44], 11}, {NULL}};
const Segment SGI79549[] = {{&S52322[44], 28}, {NULL}};
const Segment SGI79550[] = {{&S52322[30], 6}, {NULL}};
const Segment SGI79551[] = {{&S52322[120], 3}, {NULL}};
const Segment SGI79552[] = {{&S52322[86], 5}, {NULL}};
const Segment SGI79553[] = {{&S52325[44], 11}, {NULL}};
const Segment SGI79554[] = {{&S52325[44], 28}, {NULL}};
const Segment SGI79555[] = {{&S52325[30], 6}, {NULL}};
const Segment SGI79556[] = {{&S52325[120], 3}, {NULL}};
const Segment SGI79557[] = {{&S52325[86], 5}, {NULL}};
const Segment SGI79558[] = {{&S52329[38], 11}, {NULL}};
const Segment SGI79559[] = {{&S52329[31], 6}, {NULL}};
const Segment SGI79560[] = {{&S52336[44], 10}, {NULL}};
const Segment SGI79561[] = {{&S52336[44], 27}, {NULL}};
const Segment SGI79562[] = {{&S52336[30], 6}, {NULL}};
const Segment SGI79563[] = {{&S52336[119], 3}, {NULL}};
const Segment SGI79564[] = {{&S52336[85], 5}, {NULL}};
const Segment SGI79565[] = {{&S52337[44], 11}, {NULL}};
const Segment SGI79566[] = {{&S52337[44], 28}, {NULL}};
const Segment SGI79567[] = {{&S52337[30], 6}, {NULL}};
const Segment SGI79568[] = {{&S52337[120], 3}, {NULL}};
const Segment SGI79569[] = {{&S52337[86], 5}, {NULL}};
const Segment SGI79570[] = {{&S52338[38], 10}, {NULL}};
const Segment SGI79571[] = {{&S52338[31], 6}, {NULL}};
const Segment SGI79572[] = {{&S52339[44], 11}, {NULL}};
const Segment SGI79573[] = {{&S52339[44], 28}, {NULL}};
const Segment SGI79574[] = {{&S52339[30], 6}, {NULL}};
const Segment SGI79575[] = {{&S52339[120], 3}, {NULL}};
const Segment SGI79576[] = {{&S52339[86], 5}, {NULL}};
const Segment SGI79577[] = {{&S52389[44], 5}, {NULL}};
const Segment SGI79578[] = {{&S52389[44], 16}, {NULL}};
const Segment SGI79579[] = {{&S52389[30], 6}, {NULL}};
const Segment SGI79580[] = {{&S52389[108], 3}, {NULL}};
const Segment SGI79581[] = {{&S52389[74], 5}, {NULL}};
const Segment SGI79582[] = {{&S33827[42], 18}, {NULL}};
const Segment SGI79583[] = {{&S33827[30], 6}, {NULL}};
const Segment SGI79584[] = {{&S33827[31], 3}, {NULL}};
const Segment SGI79585[] = {{&S33827[86], 6}, {NULL}};
const Segment SGI79586[] = {{&S33828[44], 18}, {NULL}};
const Segment SGI79587[] = {{&S33828[44], 30}, {NULL}};
const Segment SGI79588[] = {{&S33828[30], 6}, {NULL}};
const Segment SGI79589[] = {{&S33828[31], 3}, {NULL}};
const Segment SGI79590[] = {{&S33828[88], 6}, {NULL}};
const Segment SGI79591[] = {{&S33833[38], 18}, {NULL}};
const Segment SGI79592[] = {{&S33833[31], 6}, {NULL}};
const Segment SGI79593[] = {{&S52481[44], 21}, {NULL}};
const Segment SGI79594[] = {{&S52481[44], 33}, {NULL}};
const Segment SGI79595[] = {{&S52481[30], 6}, {NULL}};
const Segment SGI79596[] = {{&S52481[125], 3}, {NULL}};
const Segment SGI79597[] = {{&S52481[91], 5}, {NULL}};
const Segment SGI79598[] = {{&S52514[44], 9}, {NULL}};
const Segment SGI79599[] = {{&S52514[44], 26}, {NULL}};
const Segment SGI79600[] = {{&S52514[30], 6}, {NULL}};
const Segment SGI79601[] = {{&S52514[118], 3}, {NULL}};
const Segment SGI79602[] = {{&S52514[84], 5}, {NULL}};
const Segment SGI79603[] = {{&S52515[44], 10}, {NULL}};
const Segment SGI79604[] = {{&S52515[44], 27}, {NULL}};
const Segment SGI79605[] = {{&S52515[30], 6}, {NULL}};
const Segment SGI79606[] = {{&S52515[119], 3}, {NULL}};
const Segment SGI79607[] = {{&S52515[85], 5}, {NULL}};
const Segment SGI79608[] = {{&S52516[38], 9}, {NULL}};
const Segment SGI79609[] = {{&S52516[31], 6}, {NULL}};
const Segment SGI79610[] = {{&S33964[44], 15}, {NULL}};
const Segment SGI79611[] = {{&S33964[44], 27}, {NULL}};
const Segment SGI79612[] = {{&S33964[30], 6}, {NULL}};
const Segment SGI79613[] = {{&S33964[31], 3}, {NULL}};
const Segment SGI79614[] = {{&S33964[85], 6}, {NULL}};
const Segment SGI79615[] = {{&S33965[44], 12}, {NULL}};
const Segment SGI79616[] = {{&S33965[44], 24}, {NULL}};
const Segment SGI79617[] = {{&S33965[30], 6}, {NULL}};
const Segment SGI79618[] = {{&S33965[31], 3}, {NULL}};
const Segment SGI79619[] = {{&S33965[82], 6}, {NULL}};
const Segment SGI79620[] = {{&S33967[44], 12}, {NULL}};
const Segment SGI79621[] = {{&S33967[44], 24}, {NULL}};
const Segment SGI79622[] = {{&S33967[30], 6}, {NULL}};
const Segment SGI79623[] = {{&S33967[31], 3}, {NULL}};
const Segment SGI79624[] = {{&S33967[82], 6}, {NULL}};
const Segment SGI79625[] = {{&S33968[44], 15}, {NULL}};
const Segment SGI79626[] = {{&S33968[44], 27}, {NULL}};
const Segment SGI79627[] = {{&S33968[30], 6}, {NULL}};
const Segment SGI79628[] = {{&S33968[31], 3}, {NULL}};
const Segment SGI79629[] = {{&S33968[85], 6}, {NULL}};
const Segment SGI79630[] = {{&S33970[38], 12}, {NULL}};
const Segment SGI79631[] = {{&S33970[31], 6}, {NULL}};
const Segment SGI79632[] = {{&S33973[44], 15}, {NULL}};
const Segment SGI79633[] = {{&S33973[44], 28}, {NULL}};
const Segment SGI79634[] = {{&S33973[30], 6}, {NULL}};
const Segment SGI79635[] = {{&S33973[31], 3}, {NULL}};
const Segment SGI79636[] = {{&S33973[86], 6}, {NULL}};
const Segment SGI79637[] = {{&S33974[44], 12}, {NULL}};
const Segment SGI79638[] = {{&S33974[44], 24}, {NULL}};
const Segment SGI79639[] = {{&S33974[30], 6}, {NULL}};
const Segment SGI79640[] = {{&S33974[82], 6}, {NULL}};
const Segment SGI79641[] = {{&S33975[44], 12}, {NULL}};
const Segment SGI79642[] = {{&S33975[44], 25}, {NULL}};
const Segment SGI79643[] = {{&S33975[30], 6}, {NULL}};
const Segment SGI79644[] = {{&S33975[31], 3}, {NULL}};
const Segment SGI79645[] = {{&S33975[83], 6}, {NULL}};
const Segment SGI79646[] = {{&S33977[44], 12}, {NULL}};
const Segment SGI79647[] = {{&S33977[44], 38}, {NULL}};
const Segment SGI79648[] = {{&S33977[30], 6}, {NULL}};
const Segment SGI79649[] = {{&S33977[31], 3}, {NULL}};
const Segment SGI79650[] = {{&S33977[96], 6}, {NULL}};
const Segment SGI79651[] = {{&S33978[44], 15}, {NULL}};
const Segment SGI79652[] = {{&S33978[44], 41}, {NULL}};
const Segment SGI79653[] = {{&S33978[30], 6}, {NULL}};
const Segment SGI79654[] = {{&S33978[31], 3}, {NULL}};
const Segment SGI79655[] = {{&S33978[99], 6}, {NULL}};
const Segment SGI79656[] = {{&S33979[42], 12}, {NULL}};
const Segment SGI79657[] = {{&S33979[30], 6}, {NULL}};
const Segment SGI79658[] = {{&S33979[31], 3}, {NULL}};
const Segment SGI79659[] = {{&S33979[80], 6}, {NULL}};
const Segment SGI79660[] = {{&S33980[42], 15}, {NULL}};
const Segment SGI79661[] = {{&S33980[30], 6}, {NULL}};
const Segment SGI79662[] = {{&S33980[31], 3}, {NULL}};
const Segment SGI79663[] = {{&S33980[83], 6}, {NULL}};
const Segment SGI79664[] = {{&S34020[44], 11}, {NULL}};
const Segment SGI79665[] = {{&S34020[44], 23}, {NULL}};
const Segment SGI79666[] = {{&S34020[30], 6}, {NULL}};
const Segment SGI79667[] = {{&S34020[31], 3}, {NULL}};
const Segment SGI79668[] = {{&S34020[81], 6}, {NULL}};
const Segment SGI79669[] = {{&S34021[44], 11}, {NULL}};
const Segment SGI79670[] = {{&S34021[44], 23}, {NULL}};
const Segment SGI79671[] = {{&S34021[30], 6}, {NULL}};
const Segment SGI79672[] = {{&S34021[31], 3}, {NULL}};
const Segment SGI79673[] = {{&S34021[81], 6}, {NULL}};
const Segment SGI79674[] = {{&S34022[38], 11}, {NULL}};
const Segment SGI79675[] = {{&S34022[31], 6}, {NULL}};
const Segment SGI79676[] = {{&S34023[44], 11}, {NULL}};
const Segment SGI79677[] = {{&S34023[44], 23}, {NULL}};
const Segment SGI79678[] = {{&S34023[30], 6}, {NULL}};
const Segment SGI79679[] = {{&S34023[31], 3}, {NULL}};
const Segment SGI79680[] = {{&S34023[81], 6}, {NULL}};
const Segment SGI79681[] = {{&S34024[38], 11}, {NULL}};
const Segment SGI79682[] = {{&S34024[31], 6}, {NULL}};
const Segment SGI79683[] = {{&S34025[38], 11}, {NULL}};
const Segment SGI79684[] = {{&S34025[31], 6}, {NULL}};
const Segment SGI79685[] = {{&S46506[6], 4}, {NULL}};
const Segment SGI79686[] = {{&S46506[86], 5}, {NULL}};
const Segment SGI79687[] = {{&S34032[44], 11}, {NULL}};
const Segment SGI79688[] = {{&S34032[44], 28}, {NULL}};
const Segment SGI79689[] = {{&S34032[30], 6}, {NULL}};
const Segment SGI79690[] = {{&S34032[31], 3}, {NULL}};
const Segment SGI79691[] = {{&S34032[86], 6}, {NULL}};
const Segment SGI79692[] = {{&S46520[44], 14}, {NULL}};
const Segment SGI79693[] = {{&S46520[44], 31}, {NULL}};
const Segment SGI79694[] = {{&S46520[30], 6}, {NULL}};
const Segment SGI79695[] = {{&S46520[123], 3}, {NULL}};
const Segment SGI79696[] = {{&S46520[89], 5}, {NULL}};
const Segment SGI79697[] = {{&S46522[44], 7}, {NULL}};
const Segment SGI79698[] = {{&S46522[44], 24}, {NULL}};
const Segment SGI79699[] = {{&S46522[30], 6}, {NULL}};
const Segment SGI79700[] = {{&S46522[116], 3}, {NULL}};
const Segment SGI79701[] = {{&S46522[82], 5}, {NULL}};
const Segment SGI79702[] = {{&S46523[38], 14}, {NULL}};
const Segment SGI79703[] = {{&S46523[31], 6}, {NULL}};
const Segment SGI79704[] = {{&S34044[34], 7}, {NULL}};
const Segment SGI79705[] = {{&S34044[27], 6}, {NULL}};
const Segment SGI79706[] = {{&S34044[94], 9}, {NULL}};
const Segment SGI79707[] = {{&S34044[67], 6}, {NULL}};
const Segment SGI79708[] = {{&S34045[34], 12}, {NULL}};
const Segment SGI79709[] = {{&S34045[27], 6}, {NULL}};
const Segment SGI79710[] = {{&S34045[99], 9}, {NULL}};
const Segment SGI79711[] = {{&S34045[72], 6}, {NULL}};
const Segment SGI79712[] = {{&S34046[34], 13}, {NULL}};
const Segment SGI79713[] = {{&S34046[27], 6}, {NULL}};
const Segment SGI79714[] = {{&S34046[102], 9}, {NULL}};
const Segment SGI79715[] = {{&S34046[75], 6}, {NULL}};
const Segment SGI79716[] = {{&S34047[34], 9}, {NULL}};
const Segment SGI79717[] = {{&S34047[27], 6}, {NULL}};
const Segment SGI79718[] = {{&S34047[96], 9}, {NULL}};
const Segment SGI79719[] = {{&S34047[69], 6}, {NULL}};
const Segment SGI79720[] = {{&S34050[34], 18}, {NULL}};
const Segment SGI79721[] = {{&S34050[27], 6}, {NULL}};
const Segment SGI79722[] = {{&S34050[105], 9}, {NULL}};
const Segment SGI79723[] = {{&S34050[78], 6}, {NULL}};
const Segment SGI79724[] = {{&S34051[34], 6}, {NULL}};
const Segment SGI79725[] = {{&S34051[27], 6}, {NULL}};
const Segment SGI79726[] = {{&S34051[93], 9}, {NULL}};
const Segment SGI79727[] = {{&S34051[66], 6}, {NULL}};
const Segment SGI79728[] = {{&S34052[34], 6}, {NULL}};
const Segment SGI79729[] = {{&S34052[27], 6}, {NULL}};
const Segment SGI79730[] = {{&S34052[93], 9}, {NULL}};
const Segment SGI79731[] = {{&S34052[66], 6}, {NULL}};
const Segment SGI79732[] = {{&S34053[34], 7}, {NULL}};
const Segment SGI79733[] = {{&S34053[27], 6}, {NULL}};
const Segment SGI79734[] = {{&S34053[94], 9}, {NULL}};
const Segment SGI79735[] = {{&S34053[67], 6}, {NULL}};
const Segment SGI79736[] = {{&S34054[34], 9}, {NULL}};
const Segment SGI79737[] = {{&S34054[27], 6}, {NULL}};
const Segment SGI79738[] = {{&S34054[96], 9}, {NULL}};
const Segment SGI79739[] = {{&S34054[69], 6}, {NULL}};
const Segment SGI79740[] = {{&S34056[34], 6}, {NULL}};
const Segment SGI79741[] = {{&S34056[27], 6}, {NULL}};
const Segment SGI79742[] = {{&S34056[93], 9}, {NULL}};
const Segment SGI79743[] = {{&S34056[66], 6}, {NULL}};
const Segment SGI79744[] = {{&S34057[34], 12}, {NULL}};
const Segment SGI79745[] = {{&S34057[27], 6}, {&S34057[42], 16}, {NULL}};
const Segment SGI79746[] = {{&S34057[99], 9}, {NULL}};
const Segment SGI79747[] = {{&S34057[72], 6}, {NULL}};
const Segment SGI79748[] = {{&S34059[34], 6}, {NULL}};
const Segment SGI79749[] = {{&S34059[27], 6}, {NULL}};
const Segment SGI79750[] = {{&S34059[93], 9}, {NULL}};
const Segment SGI79751[] = {{&S34059[66], 6}, {NULL}};
const Segment SGI79752[] = {{&S34060[34], 7}, {NULL}};
const Segment SGI79753[] = {{&S34060[27], 6}, {NULL}};
const Segment SGI79754[] = {{&S34060[94], 9}, {NULL}};
const Segment SGI79755[] = {{&S34060[67], 6}, {NULL}};
const Segment SGI79756[] = {{&S34064[34], 8}, {NULL}};
const Segment SGI79757[] = {{&S34064[27], 6}, {NULL}};
const Segment SGI79758[] = {{&S34064[95], 9}, {NULL}};
const Segment SGI79759[] = {{&S34064[68], 6}, {NULL}};
const Segment SGI79760[] = {{&S34065[34], 8}, {NULL}};
const Segment SGI79761[] = {{&S34065[27], 6}, {NULL}};
const Segment SGI79762[] = {{&S34065[95], 9}, {NULL}};
const Segment SGI79763[] = {{&S34065[68], 6}, {NULL}};
const Segment SGI79764[] = {{&S34066[34], 12}, {NULL}};
const Segment SGI79765[] = {{&S34066[27], 6}, {NULL}};
const Segment SGI79766[] = {{&S34066[99], 9}, {NULL}};
const Segment SGI79767[] = {{&S34066[72], 6}, {NULL}};
const Segment SGI79768[] = {{&S34067[34], 10}, {NULL}};
const Segment SGI79769[] = {{&S34067[27], 6}, {NULL}};
const Segment SGI79770[] = {{&S34067[97], 9}, {NULL}};
const Segment SGI79771[] = {{&S34067[70], 6}, {NULL}};
const Segment SGI79772[] = {{&S34068[34], 7}, {NULL}};
const Segment SGI79773[] = {{&S34068[27], 6}, {NULL}};
const Segment SGI79774[] = {{&S34068[94], 9}, {NULL}};
const Segment SGI79775[] = {{&S34068[67], 6}, {NULL}};
const Segment SGI79776[] = {{&S34072[34], 9}, {NULL}};
const Segment SGI79777[] = {{&S34072[27], 6}, {NULL}};
const Segment SGI79778[] = {{&S34072[96], 9}, {NULL}};
const Segment SGI79779[] = {{&S34072[69], 6}, {NULL}};
const Segment SGI79780[] = {{&S34074[34], 14}, {NULL}};
const Segment SGI79781[] = {{&S34074[27], 6}, {NULL}};
const Segment SGI79782[] = {{&S34074[101], 9}, {NULL}};
const Segment SGI79783[] = {{&S34074[74], 6}, {NULL}};
const Segment SGI79784[] = {{&S34079[34], 11}, {NULL}};
const Segment SGI79785[] = {{&S34079[27], 6}, {NULL}};
const Segment SGI79786[] = {{&S34079[98], 9}, {NULL}};
const Segment SGI79787[] = {{&S34079[71], 6}, {NULL}};
const Segment SGI79788[] = {{&S34081[34], 5}, {NULL}};
const Segment SGI79789[] = {{&S34081[27], 6}, {NULL}};
const Segment SGI79790[] = {{&S34081[92], 9}, {NULL}};
const Segment SGI79791[] = {{&S34081[65], 6}, {NULL}};
const Segment SGI79792[] = {{&S34083[34], 11}, {NULL}};
const Segment SGI79793[] = {{&S34083[27], 6}, {NULL}};
const Segment SGI79794[] = {{&S34083[98], 9}, {NULL}};
const Segment SGI79795[] = {{&S34083[71], 6}, {NULL}};
const Segment SGI79796[] = {{&S34084[34], 6}, {NULL}};
const Segment SGI79797[] = {{&S34084[27], 6}, {NULL}};
const Segment SGI79798[] = {{&S34084[93], 9}, {NULL}};
const Segment SGI79799[] = {{&S34084[66], 6}, {NULL}};
const Segment SGI79800[] = {{&S34089[34], 10}, {NULL}};
const Segment SGI79801[] = {{&S34089[27], 6}, {NULL}};
const Segment SGI79802[] = {{&S34089[97], 9}, {NULL}};
const Segment SGI79803[] = {{&S34089[70], 6}, {NULL}};
const Segment SGI79804[] = {{&S34091[34], 12}, {NULL}};
const Segment SGI79805[] = {{&S34091[27], 6}, {NULL}};
const Segment SGI79806[] = {{&S34091[99], 9}, {NULL}};
const Segment SGI79807[] = {{&S34091[72], 6}, {NULL}};
const Segment SGI79808[] = {{&S34092[34], 10}, {NULL}};
const Segment SGI79809[] = {{&S34092[27], 6}, {NULL}};
const Segment SGI79810[] = {{&S34092[97], 9}, {NULL}};
const Segment SGI79811[] = {{&S34092[70], 6}, {NULL}};
const Segment SGI79812[] = {{&S34093[34], 11}, {NULL}};
const Segment SGI79813[] = {{&S34093[27], 6}, {NULL}};
const Segment SGI79814[] = {{&S34093[98], 9}, {NULL}};
const Segment SGI79815[] = {{&S34093[71], 6}, {NULL}};
const Segment SGI79816[] = {{&S34102[34], 7}, {NULL}};
const Segment SGI79817[] = {{&S34102[27], 6}, {NULL}};
const Segment SGI79818[] = {{&S34102[94], 9}, {NULL}};
const Segment SGI79819[] = {{&S34102[67], 6}, {NULL}};
const Segment SGI79820[] = {{&S34109[34], 6}, {NULL}};
const Segment SGI79821[] = {{&S34109[27], 6}, {NULL}};
const Segment SGI79822[] = {{&S34109[93], 9}, {NULL}};
const Segment SGI79823[] = {{&S34109[66], 6}, {NULL}};
const Segment SGI79824[] = {{&S34110[34], 8}, {NULL}};
const Segment SGI79825[] = {{&S34110[27], 6}, {NULL}};
const Segment SGI79826[] = {{&S34110[95], 9}, {NULL}};
const Segment SGI79827[] = {{&S34110[68], 6}, {NULL}};
const Segment SGI79828[] = {{&S34112[34], 10}, {NULL}};
const Segment SGI79829[] = {{&S34112[27], 6}, {NULL}};
const Segment SGI79830[] = {{&S34112[97], 9}, {NULL}};
const Segment SGI79831[] = {{&S34112[70], 6}, {NULL}};
const Segment SGI79832[] = {{&S34119[34], 18}, {NULL}};
const Segment SGI79833[] = {{&S34119[27], 6}, {NULL}};
const Segment SGI79834[] = {{&S34119[105], 9}, {NULL}};
const Segment SGI79835[] = {{&S34119[78], 6}, {NULL}};
const Segment SGI79836[] = {{&S34136[34], 14}, {NULL}};
const Segment SGI79837[] = {{&S34136[27], 6}, {NULL}};
const Segment SGI79838[] = {{&S34136[101], 9}, {NULL}};
const Segment SGI79839[] = {{&S34136[74], 6}, {NULL}};
const Segment SGI79840[] = {{&S34138[34], 9}, {NULL}};
const Segment SGI79841[] = {{&S34138[27], 6}, {NULL}};
const Segment SGI79842[] = {{&S34138[96], 9}, {NULL}};
const Segment SGI79843[] = {{&S34138[69], 6}, {NULL}};
const Segment SGI79844[] = {{&S34140[34], 6}, {NULL}};
const Segment SGI79845[] = {{&S34140[27], 6}, {NULL}};
const Segment SGI79846[] = {{&S34140[93], 9}, {NULL}};
const Segment SGI79847[] = {{&S34140[66], 6}, {NULL}};
const Segment SGI79848[] = {{&S34143[34], 7}, {NULL}};
const Segment SGI79849[] = {{&S34143[27], 6}, {NULL}};
const Segment SGI79850[] = {{&S34143[94], 9}, {NULL}};
const Segment SGI79851[] = {{&S34143[67], 6}, {NULL}};
const Segment SGI79852[] = {{&S34147[34], 13}, {NULL}};
const Segment SGI79853[] = {{&S34147[27], 6}, {NULL}};
const Segment SGI79854[] = {{&S34147[100], 9}, {NULL}};
const Segment SGI79855[] = {{&S34147[73], 6}, {NULL}};
const Segment SGI79856[] = {{&S34148[34], 10}, {NULL}};
const Segment SGI79857[] = {{&S34148[27], 6}, {NULL}};
const Segment SGI79858[] = {{&S34148[102], 9}, {NULL}};
const Segment SGI79859[] = {{&S34148[75], 6}, {NULL}};
const Segment SGI79860[] = {{&S34149[34], 30}, {NULL}};
const Segment SGI79861[] = {{&S34149[27], 6}, {NULL}};
const Segment SGI79862[] = {{&S34149[132], 9}, {NULL}};
const Segment SGI79863[] = {{&S34149[105], 6}, {NULL}};
const Segment SGI79864[] = {{&S34155[34], 7}, {NULL}};
const Segment SGI79865[] = {{&S34155[27], 6}, {NULL}};
const Segment SGI79866[] = {{&S34155[94], 9}, {NULL}};
const Segment SGI79867[] = {{&S34155[67], 6}, {NULL}};
const Segment SGI79868[] = {{&S34160[34], 8}, {NULL}};
const Segment SGI79869[] = {{&S34160[27], 6}, {NULL}};
const Segment SGI79870[] = {{&S34160[95], 9}, {NULL}};
const Segment SGI79871[] = {{&S34160[68], 6}, {NULL}};
const Segment SGI79872[] = {{&S34161[34], 7}, {NULL}};
const Segment SGI79873[] = {{&S34161[27], 6}, {NULL}};
const Segment SGI79874[] = {{&S34161[94], 9}, {NULL}};
const Segment SGI79875[] = {{&S34161[67], 6}, {NULL}};
const Segment SGI79876[] = {{&S34162[34], 15}, {NULL}};
const Segment SGI79877[] = {{&S34162[27], 6}, {NULL}};
const Segment SGI79878[] = {{&S34162[102], 9}, {NULL}};
const Segment SGI79879[] = {{&S34162[75], 6}, {NULL}};
const Segment SGI79880[] = {{&S34190[34], 18}, {NULL}};
const Segment SGI79881[] = {{&S34190[27], 6}, {NULL}};
const Segment SGI79882[] = {{&S34190[105], 9}, {NULL}};
const Segment SGI79883[] = {{&S34190[78], 6}, {NULL}};
const Segment SGI79884[] = {{&S45921[42], 41}, {NULL}};
const Segment SGI79885[] = {{&S45921[30], 4}, {NULL}};
const Segment SGI79886[] = {{&S45921[132], 3}, {NULL}};
const Segment SGI79887[] = {{&S45921[97], 6}, {NULL}};
const Segment SGI79888[] = {{&S34194[34], 7}, {NULL}};
const Segment SGI79889[] = {{&S34194[27], 6}, {NULL}};
const Segment SGI79890[] = {{&S34194[94], 9}, {NULL}};
const Segment SGI79891[] = {{&S34194[67], 6}, {NULL}};
const Segment SGI79892[] = {{&S45922[42], 41}, {NULL}};
const Segment SGI79893[] = {{&S45922[30], 4}, {NULL}};
const Segment SGI79894[] = {{&S45922[132], 3}, {NULL}};
const Segment SGI79895[] = {{&S34195[34], 7}, {NULL}};
const Segment SGI79896[] = {{&S45922[97], 6}, {NULL}};
const Segment SGI79897[] = {{&S34195[27], 6}, {NULL}};
const Segment SGI79898[] = {{&S34195[94], 9}, {NULL}};
const Segment SGI79899[] = {{&S34195[67], 6}, {NULL}};
const Segment SGI79900[] = {{&S45923[42], 41}, {NULL}};
const Segment SGI79901[] = {{&S45923[30], 4}, {NULL}};
const Segment SGI79902[] = {{&S45923[132], 3}, {NULL}};
const Segment SGI79903[] = {{&S45923[97], 6}, {NULL}};
const Segment SGI79904[] = {{&S45924[42], 41}, {NULL}};
const Segment SGI79905[] = {{&S45924[30], 4}, {NULL}};
const Segment SGI79906[] = {{&S34198[34], 8}, {NULL}};
const Segment SGI79907[] = {{&S45924[132], 3}, {NULL}};
const Segment SGI79908[] = {{&S34198[27], 6}, {NULL}};
const Segment SGI79909[] = {{&S45924[97], 6}, {NULL}};
const Segment SGI79910[] = {{&S34198[95], 9}, {NULL}};
const Segment SGI79911[] = {{&S34198[68], 6}, {NULL}};
const Segment SGI79912[] = {{&S34199[34], 5}, {NULL}};
const Segment SGI79913[] = {{&S45925[42], 41}, {NULL}};
const Segment SGI79914[] = {{&S45925[30], 4}, {NULL}};
const Segment SGI79915[] = {{&S34199[27], 6}, {NULL}};
const Segment SGI79916[] = {{&S45925[132], 3}, {NULL}};
const Segment SGI79917[] = {{&S34199[92], 9}, {NULL}};
const Segment SGI79918[] = {{&S34199[65], 6}, {NULL}};
const Segment SGI79919[] = {{&S45925[97], 6}, {NULL}};
const Segment SGI79920[] = {{&S45926[42], 41}, {NULL}};
const Segment SGI79921[] = {{&S45926[30], 4}, {NULL}};
const Segment SGI79922[] = {{&S45926[132], 3}, {NULL}};
const Segment SGI79923[] = {{&S34201[34], 4}, {NULL}};
const Segment SGI79924[] = {{&S45926[97], 6}, {NULL}};
const Segment SGI79925[] = {{&S34201[27], 6}, {NULL}};
const Segment SGI79926[] = {{&S34201[91], 9}, {NULL}};
const Segment SGI79927[] = {{&S34201[64], 6}, {NULL}};
const Segment SGI79928[] = {{&S45927[42], 41}, {NULL}};
const Segment SGI79929[] = {{&S45927[30], 4}, {NULL}};
const Segment SGI79930[] = {{&S45927[132], 3}, {NULL}};
const Segment SGI79931[] = {{&S45927[97], 6}, {NULL}};
const Segment SGI79932[] = {{&S34203[34], 13}, {NULL}};
const Segment SGI79933[] = {{&S34203[27], 6}, {NULL}};
const Segment SGI79934[] = {{&S45928[42], 41}, {NULL}};
const Segment SGI79935[] = {{&S34203[115], 9}, {NULL}};
const Segment SGI79936[] = {{&S45928[30], 4}, {NULL}};
const Segment SGI79937[] = {{&S34203[88], 6}, {NULL}};
const Segment SGI79938[] = {{&S45928[132], 3}, {NULL}};
const Segment SGI79939[] = {{&S45928[97], 6}, {NULL}};
const Segment SGI79940[] = {{&S34204[34], 12}, {NULL}};
const Segment SGI79941[] = {{&S34204[27], 6}, {NULL}};
const Segment SGI79942[] = {{&S34204[99], 9}, {NULL}};
const Segment SGI79943[] = {{&S34204[72], 6}, {NULL}};
const Segment SGI79944[] = {{&S45929[42], 41}, {NULL}};
const Segment SGI79945[] = {{&S45929[30], 4}, {NULL}};
const Segment SGI79946[] = {{&S45929[132], 3}, {NULL}};
const Segment SGI79947[] = {{&S45929[97], 6}, {NULL}};
const Segment SGI79948[] = {{&S45930[42], 41}, {NULL}};
const Segment SGI79949[] = {{&S45930[30], 4}, {NULL}};
const Segment SGI79950[] = {{&S45930[132], 3}, {NULL}};
const Segment SGI79951[] = {{&S45930[97], 6}, {NULL}};
const Segment SGI79952[] = {{&S34207[34], 9}, {NULL}};
const Segment SGI79953[] = {{&S34207[27], 6}, {NULL}};
const Segment SGI79954[] = {{&S34207[93], 9}, {NULL}};
const Segment SGI79955[] = {{&S34207[66], 6}, {NULL}};
const Segment SGI79956[] = {{&S45931[42], 41}, {NULL}};
const Segment SGI79957[] = {{&S45931[30], 4}, {NULL}};
const Segment SGI79958[] = {{&S45931[132], 3}, {NULL}};
const Segment SGI79959[] = {{&S45931[97], 6}, {NULL}};
const Segment SGI79960[] = {{&S45932[42], 41}, {NULL}};
const Segment SGI79961[] = {{&S45932[30], 4}, {NULL}};
const Segment SGI79962[] = {{&S45932[132], 3}, {NULL}};
const Segment SGI79963[] = {{&S45932[97], 6}, {NULL}};
const Segment SGI79964[] = {{&S45933[42], 41}, {NULL}};
const Segment SGI79965[] = {{&S45933[30], 4}, {NULL}};
const Segment SGI79966[] = {{&S45933[132], 3}, {NULL}};
const Segment SGI79967[] = {{&S45933[97], 6}, {NULL}};
const Segment SGI79968[] = {{&S34213[27], 6}, {NULL}};
const Segment SGI79969[] = {{&S34213[108], 9}, {NULL}};
const Segment SGI79970[] = {{&S34213[81], 6}, {NULL}};
const Segment SGI79971[] = {{&S45934[42], 41}, {NULL}};
const Segment SGI79972[] = {{&S45934[30], 4}, {NULL}};
const Segment SGI79973[] = {{&S45934[132], 3}, {NULL}};
const Segment SGI79974[] = {{&S45934[97], 6}, {NULL}};
const Segment SGI79975[] = {{&S34214[34], 11}, {NULL}};
const Segment SGI79976[] = {{&S34214[27], 6}, {NULL}};
const Segment SGI79977[] = {{&S34214[98], 9}, {NULL}};
const Segment SGI79978[] = {{&S34214[71], 6}, {NULL}};
const Segment SGI79979[] = {{&S45935[42], 50}, {NULL}};
const Segment SGI79980[] = {{&S45935[30], 4}, {NULL}};
const Segment SGI79981[] = {{&S45935[141], 3}, {NULL}};
const Segment SGI79982[] = {{&S45935[106], 6}, {NULL}};
const Segment SGI79983[] = {{&S45936[42], 41}, {NULL}};
const Segment SGI79984[] = {{&S45936[30], 4}, {NULL}};
const Segment SGI79985[] = {{&S45936[132], 3}, {NULL}};
const Segment SGI79986[] = {{&S45936[97], 6}, {NULL}};
const Segment SGI79987[] = {{&S45937[42], 41}, {NULL}};
const Segment SGI79988[] = {{&S45937[30], 4}, {NULL}};
const Segment SGI79989[] = {{&S45937[132], 3}, {NULL}};
const Segment SGI79990[] = {{&S45937[97], 6}, {NULL}};
const Segment SGI79991[] = {{&S45938[42], 41}, {NULL}};
const Segment SGI79992[] = {{&S45938[30], 4}, {NULL}};
const Segment SGI79993[] = {{&S45938[132], 3}, {NULL}};
const Segment SGI79994[] = {{&S45938[97], 6}, {NULL}};
const Segment SGI79995[] = {{&S34222[34], 9}, {NULL}};
const Segment SGI79996[] = {{&S45939[42], 41}, {NULL}};
const Segment SGI79997[] = {{&S45939[30], 4}, {NULL}};
const Segment SGI79998[] = {{&S45939[132], 3}, {NULL}};
const Segment SGI79999[] = {{&S45939[97], 6}, {NULL}};
const Segment SGI80000[] = {{&S45940[42], 41}, {NULL}};
const Segment SGI80001[] = {{&S45940[30], 4}, {NULL}};
const Segment SGI80002[] = {{&S45940[132], 3}, {NULL}};
const Segment SGI80003[] = {{&S45940[97], 6}, {NULL}};
const Segment SGI80004[] = {{&S45941[42], 41}, {NULL}};
const Segment SGI80005[] = {{&S45941[30], 4}, {NULL}};
const Segment SGI80006[] = {{&S45941[132], 3}, {NULL}};
const Segment SGI80007[] = {{&S45941[97], 6}, {NULL}};
const Segment SGI80008[] = {{&S45942[42], 41}, {NULL}};
const Segment SGI80009[] = {{&S45942[30], 4}, {NULL}};
const Segment SGI80010[] = {{&S45942[132], 3}, {NULL}};
const Segment SGI80011[] = {{&S45942[97], 6}, {NULL}};
const Segment SGI80012[] = {{&S45943[42], 41}, {NULL}};
const Segment SGI80013[] = {{&S45943[30], 4}, {NULL}};
const Segment SGI80014[] = {{&S45943[132], 3}, {NULL}};
const Segment SGI80015[] = {{&S45943[97], 6}, {NULL}};
const Segment SGI80016[] = {{&S45944[42], 41}, {NULL}};
const Segment SGI80017[] = {{&S45944[30], 4}, {NULL}};
const Segment SGI80018[] = {{&S45944[132], 3}, {NULL}};
const Segment SGI80019[] = {{&S45944[97], 6}, {NULL}};
const Segment SGI80020[] = {{&S45945[42], 41}, {NULL}};
const Segment SGI80021[] = {{&S45945[30], 4}, {NULL}};
const Segment SGI80022[] = {{&S45945[132], 3}, {NULL}};
const Segment SGI80023[] = {{&S45945[97], 6}, {NULL}};
const Segment SGI80024[] = {{&S45946[42], 41}, {NULL}};
const Segment SGI80025[] = {{&S45946[30], 4}, {NULL}};
const Segment SGI80026[] = {{&S45946[131], 3}, {NULL}};
const Segment SGI80027[] = {{&S45946[97], 5}, {NULL}};
const Segment SGI80028[] = {{&S45947[42], 41}, {NULL}};
const Segment SGI80029[] = {{&S45947[30], 4}, {NULL}};
const Segment SGI80030[] = {{&S45947[132], 3}, {NULL}};
const Segment SGI80031[] = {{&S45947[97], 6}, {NULL}};
const Segment SGI80032[] = {{&S52799[44], 7}, {NULL}};
const Segment SGI80033[] = {{&S52799[44], 37}, {NULL}};
const Segment SGI80034[] = {{&S52799[30], 6}, {NULL}};
const Segment SGI80035[] = {{&S52799[129], 3}, {NULL}};
const Segment SGI80036[] = {{&S52799[95], 5}, {NULL}};
const Segment SGI80037[] = {{&S52800[44], 7}, {NULL}};
const Segment SGI80038[] = {{&S52800[44], 37}, {NULL}};
const Segment SGI80039[] = {{&S52800[30], 6}, {NULL}};
const Segment SGI80040[] = {{&S52800[129], 3}, {NULL}};
const Segment SGI80041[] = {{&S52800[95], 5}, {NULL}};
const Segment SGI80042[] = {{&S52801[44], 7}, {NULL}};
const Segment SGI80043[] = {{&S52801[44], 37}, {NULL}};
const Segment SGI80044[] = {{&S52801[30], 6}, {NULL}};
const Segment SGI80045[] = {{&S52801[129], 3}, {NULL}};
const Segment SGI80046[] = {{&S52801[95], 5}, {NULL}};
const Segment SGI80047[] = {{&S34222[27], 6}, {NULL}};
const Segment SGI80048[] = {{&S34222[96], 9}, {NULL}};
const Segment SGI80049[] = {{&S34222[69], 6}, {NULL}};
const Segment SGI80050[] = {{&S34223[34], 6}, {NULL}};
const Segment SGI80051[] = {{&S34223[27], 6}, {NULL}};
const Segment SGI80052[] = {{&S34223[93], 9}, {NULL}};
const Segment SGI80053[] = {{&S34223[66], 6}, {NULL}};
const Segment SGI80054[] = {{&S46830[44], 9}, {NULL}};
const Segment SGI80055[] = {{&S46830[44], 20}, {NULL}};
const Segment SGI80056[] = {{&S46830[30], 6}, {NULL}};
const Segment SGI80057[] = {{&S46830[131], 3}, {NULL}};
const Segment SGI80058[] = {{&S46830[97], 5}, {NULL}};
const Segment SGI80059[] = {{&S46830[35], 1}, {NULL}};
const Segment SGI80060[] = {{&S46830[66], 11}, {NULL}};
const Segment SGI80061[] = {{&S46831[44], 9}, {NULL}};
const Segment SGI80062[] = {{&S34236[34], 11}, {NULL}};
const Segment SGI80063[] = {{&S46831[44], 20}, {NULL}};
const Segment SGI80064[] = {{&S46831[30], 6}, {NULL}};
const Segment SGI80065[] = {{&S34236[27], 6}, {NULL}};
const Segment SGI80066[] = {{&S46831[143], 3}, {NULL}};
const Segment SGI80067[] = {{&S34236[98], 9}, {NULL}};
const Segment SGI80068[] = {{&S34236[71], 6}, {NULL}};
const Segment SGI80069[] = {{&S46831[109], 5}, {NULL}};
const Segment SGI80070[] = {{&S46831[66], 11}, {NULL}};
const Segment SGI80071[] = {{&S46832[44], 9}, {NULL}};
const Segment SGI80072[] = {{&S46832[44], 20}, {NULL}};
const Segment SGI80073[] = {{&S46832[30], 6}, {NULL}};
const Segment SGI80074[] = {{&S46832[146], 3}, {NULL}};
const Segment SGI80075[] = {{&S46832[112], 5}, {NULL}};
const Segment SGI80076[] = {{&S46832[66], 11}, {NULL}};
const Segment SGI80077[] = {{&S34240[34], 8}, {NULL}};
const Segment SGI80078[] = {{&S46833[44], 9}, {&S46833[84], 9}, {NULL}};
const Segment SGI80079[] = {{&S34240[27], 6}, {NULL}};
const Segment SGI80080[] = {{&S46833[44], 20}, {NULL}};
const Segment SGI80081[] = {{&S46833[30], 6}, {NULL}};
const Segment SGI80082[] = {{&S34240[95], 9}, {NULL}};
const Segment SGI80083[] = {{&S46833[152], 3}, {NULL}};
const Segment SGI80084[] = {{&S34240[68], 6}, {NULL}};
const Segment SGI80085[] = {{&S46833[118], 5}, {NULL}};
const Segment SGI80086[] = {{&S46833[35], 1}, {NULL}};
const Segment SGI80087[] = {{&S46833[66], 11}, {NULL}};
const Segment SGI80088[] = {{&S34242[34], 17}, {NULL}};
const Segment SGI80089[] = {{&S34242[27], 6}, {NULL}};
const Segment SGI80090[] = {{&S34242[104], 9}, {NULL}};
const Segment SGI80091[] = {{&S34242[77], 6}, {NULL}};
const Segment SGI80092[] = {{&S34245[34], 6}, {NULL}};
const Segment SGI80093[] = {{&S34245[27], 6}, {NULL}};
const Segment SGI80094[] = {{&S34245[93], 9}, {NULL}};
const Segment SGI80095[] = {{&S34245[66], 6}, {NULL}};
const Segment SGI80096[] = {{&S34246[34], 9}, {NULL}};
const Segment SGI80097[] = {{&S34246[27], 6}, {NULL}};
const Segment SGI80098[] = {{&S34246[96], 9}, {NULL}};
const Segment SGI80099[] = {{&S34246[69], 6}, {NULL}};
const Segment SGI80100[] = {{&S34256[34], 8}, {NULL}};
const Segment SGI80101[] = {{&S34256[27], 6}, {NULL}};
const Segment SGI80102[] = {{&S34256[95], 9}, {NULL}};
const Segment SGI80103[] = {{&S34256[68], 6}, {NULL}};
const Segment SGI80104[] = {{&S34257[34], 4}, {NULL}};
const Segment SGI80105[] = {{&S34257[27], 6}, {NULL}};
const Segment SGI80106[] = {{&S34257[91], 9}, {NULL}};
const Segment SGI80107[] = {{&S34257[64], 6}, {NULL}};
const Segment SGI80108[] = {{&S34268[34], 14}, {NULL}};
const Segment SGI80109[] = {{&S34268[27], 6}, {NULL}};
const Segment SGI80110[] = {{&S34268[101], 9}, {NULL}};
const Segment SGI80111[] = {{&S34268[74], 6}, {NULL}};
const Segment SGI80112[] = {{&S34274[34], 15}, {NULL}};
const Segment SGI80113[] = {{&S34274[27], 6}, {NULL}};
const Segment SGI80114[] = {{&S34274[102], 9}, {NULL}};
const Segment SGI80115[] = {{&S34274[75], 6}, {NULL}};
const Segment SGI80116[] = {{&S34282[34], 16}, {NULL}};
const Segment SGI80117[] = {{&S34282[27], 6}, {NULL}};
const Segment SGI80118[] = {{&S34282[103], 9}, {NULL}};
const Segment SGI80119[] = {{&S34282[76], 6}, {NULL}};
const Segment SGI80120[] = {{&S34295[34], 7}, {NULL}};
const Segment SGI80121[] = {{&S34295[27], 6}, {NULL}};
const Segment SGI80122[] = {{&S34295[94], 9}, {NULL}};
const Segment SGI80123[] = {{&S34295[67], 6}, {NULL}};
const Segment SGI80124[] = {{&S34297[34], 12}, {NULL}};
const Segment SGI80125[] = {{&S34297[27], 6}, {NULL}};
const Segment SGI80126[] = {{&S34297[99], 9}, {NULL}};
const Segment SGI80127[] = {{&S34297[72], 6}, {NULL}};
const Segment SGI80128[] = {{&S34308[34], 11}, {NULL}};
const Segment SGI80129[] = {{&S34308[27], 6}, {NULL}};
const Segment SGI80130[] = {{&S34308[98], 9}, {NULL}};
const Segment SGI80131[] = {{&S34308[71], 6}, {NULL}};
const Segment SGI80132[] = {{&S34311[34], 8}, {NULL}};
const Segment SGI80133[] = {{&S34311[27], 6}, {NULL}};
const Segment SGI80134[] = {{&S34311[95], 9}, {NULL}};
const Segment SGI80135[] = {{&S34311[68], 6}, {NULL}};
const Segment SGI80136[] = {{&S34312[34], 7}, {NULL}};
const Segment SGI80137[] = {{&S34312[27], 6}, {NULL}};
const Segment SGI80138[] = {{&S34312[93], 9}, {NULL}};
const Segment SGI80139[] = {{&S34312[66], 6}, {NULL}};
const Segment SGI80140[] = {{&S34313[34], 17}, {NULL}};
const Segment SGI80141[] = {{&S34313[27], 6}, {NULL}};
const Segment SGI80142[] = {{&S34313[104], 9}, {NULL}};
const Segment SGI80143[] = {{&S34313[77], 6}, {NULL}};
const Segment SGI80144[] = {{&S34314[34], 9}, {NULL}};
const Segment SGI80145[] = {{&S34314[27], 6}, {NULL}};
const Segment SGI80146[] = {{&S34314[96], 9}, {NULL}};
const Segment SGI80147[] = {{&S34314[69], 6}, {NULL}};
const Segment SGI80148[] = {{&S34315[34], 10}, {NULL}};
const Segment SGI80149[] = {{&S34315[27], 6}, {NULL}};
const Segment SGI80150[] = {{&S34315[97], 9}, {NULL}};
const Segment SGI80151[] = {{&S34315[70], 6}, {NULL}};
const Segment SGI80152[] = {{&S34317[34], 4}, {NULL}};
const Segment SGI80153[] = {{&S34317[27], 6}, {NULL}};
const Segment SGI80154[] = {{&S34317[91], 9}, {NULL}};
const Segment SGI80155[] = {{&S34317[64], 6}, {NULL}};
const Segment SGI80156[] = {{&S34318[34], 14}, {NULL}};
const Segment SGI80157[] = {{&S34318[27], 6}, {NULL}};
const Segment SGI80158[] = {{&S34318[101], 9}, {NULL}};
const Segment SGI80159[] = {{&S34318[74], 6}, {NULL}};
const Segment SGI80160[] = {{&S34319[34], 11}, {NULL}};
const Segment SGI80161[] = {{&S34319[27], 6}, {NULL}};
const Segment SGI80162[] = {{&S34319[98], 9}, {NULL}};
const Segment SGI80163[] = {{&S34319[71], 6}, {NULL}};
const Segment SGI80164[] = {{&S34321[34], 7}, {NULL}};
const Segment SGI80165[] = {{&S34321[27], 6}, {NULL}};
const Segment SGI80166[] = {{&S34321[94], 9}, {NULL}};
const Segment SGI80167[] = {{&S34321[67], 6}, {NULL}};
const Segment SGI80168[] = {{&S34322[34], 8}, {NULL}};
const Segment SGI80169[] = {{&S34322[27], 6}, {NULL}};
const Segment SGI80170[] = {{&S34322[95], 9}, {NULL}};
const Segment SGI80171[] = {{&S34322[68], 6}, {NULL}};
const Segment SGI80172[] = {{&S34325[34], 4}, {NULL}};
const Segment SGI80173[] = {{&S34325[27], 6}, {NULL}};
const Segment SGI80174[] = {{&S34325[91], 9}, {NULL}};
const Segment SGI80175[] = {{&S34325[64], 6}, {NULL}};
const Segment SGI80176[] = {{&S34330[34], 7}, {NULL}};
const Segment SGI80177[] = {{&S34330[27], 6}, {NULL}};
const Segment SGI80178[] = {{&S34330[94], 9}, {NULL}};
const Segment SGI80179[] = {{&S34330[67], 6}, {NULL}};
const Segment SGI80180[] = {{&S34331[34], 8}, {NULL}};
const Segment SGI80181[] = {{&S34331[27], 6}, {NULL}};
const Segment SGI80182[] = {{&S34331[95], 9}, {NULL}};
const Segment SGI80183[] = {{&S34331[68], 6}, {NULL}};
const Segment SGI80184[] = {{&S34334[34], 17}, {NULL}};
const Segment SGI80185[] = {{&S34334[27], 6}, {NULL}};
const Segment SGI80186[] = {{&S34334[104], 9}, {NULL}};
const Segment SGI80187[] = {{&S34334[77], 6}, {NULL}};
const Segment SGI80188[] = {{&S34336[34], 7}, {NULL}};
const Segment SGI80189[] = {{&S34336[27], 6}, {NULL}};
const Segment SGI80190[] = {{&S34336[94], 9}, {NULL}};
const Segment SGI80191[] = {{&S34336[67], 6}, {NULL}};
const Segment SGI80192[] = {{&S34339[34], 13}, {NULL}};
const Segment SGI80193[] = {{&S34339[27], 6}, {NULL}};
const Segment SGI80194[] = {{&S34339[100], 9}, {NULL}};
const Segment SGI80195[] = {{&S34339[73], 6}, {NULL}};
const Segment SGI80196[] = {{&S34342[34], 12}, {NULL}};
const Segment SGI80197[] = {{&S34342[27], 6}, {NULL}};
const Segment SGI80198[] = {{&S34342[122], 9}, {NULL}};
const Segment SGI80199[] = {{&S34342[95], 6}, {NULL}};
const Segment SGI80200[] = {{&S34347[34], 13}, {NULL}};
const Segment SGI80201[] = {{&S34347[27], 6}, {NULL}};
const Segment SGI80202[] = {{&S34347[100], 9}, {NULL}};
const Segment SGI80203[] = {{&S34347[73], 6}, {NULL}};
const Segment SGI80204[] = {{&S34354[34], 6}, {NULL}};
const Segment SGI80205[] = {{&S34354[27], 6}, {NULL}};
const Segment SGI80206[] = {{&S34354[93], 9}, {NULL}};
const Segment SGI80207[] = {{&S34354[66], 6}, {NULL}};
const Segment SGI80208[] = {{&S34359[34], 6}, {NULL}};
const Segment SGI80209[] = {{&S34359[27], 6}, {NULL}};
const Segment SGI80210[] = {{&S34359[99], 9}, {NULL}};
const Segment SGI80211[] = {{&S34359[72], 6}, {NULL}};
const Segment SGI80212[] = {{&S34371[34], 8}, {NULL}};
const Segment SGI80213[] = {{&S34371[27], 6}, {NULL}};
const Segment SGI80214[] = {{&S34371[95], 9}, {NULL}};
const Segment SGI80215[] = {{&S34371[68], 6}, {NULL}};
const Segment SGI80216[] = {{&S34373[34], 9}, {NULL}};
const Segment SGI80217[] = {{&S34373[27], 6}, {NULL}};
const Segment SGI80218[] = {{&S34373[93], 9}, {NULL}};
const Segment SGI80219[] = {{&S34373[66], 6}, {NULL}};
const Segment SGI80220[] = {{&S34377[34], 8}, {NULL}};
const Segment SGI80221[] = {{&S34377[27], 6}, {NULL}};
const Segment SGI80222[] = {{&S34377[103], 9}, {NULL}};
const Segment SGI80223[] = {{&S34377[76], 6}, {NULL}};
const Segment SGI80224[] = {{&S47076[44], 9}, {NULL}};
const Segment SGI80225[] = {{&S47076[44], 20}, {NULL}};
const Segment SGI80226[] = {{&S47076[30], 6}, {NULL}};
const Segment SGI80227[] = {{&S47076[135], 3}, {NULL}};
const Segment SGI80228[] = {{&S47076[101], 5}, {NULL}};
const Segment SGI80229[] = {{&S34391[34], 5}, {NULL}};
const Segment SGI80230[] = {{&S34391[27], 6}, {NULL}};
const Segment SGI80231[] = {{&S34391[92], 9}, {NULL}};
const Segment SGI80232[] = {{&S34391[65], 6}, {NULL}};
const Segment SGI80233[] = {{&S34394[34], 19}, {NULL}};
const Segment SGI80234[] = {{&S34394[27], 6}, {NULL}};
const Segment SGI80235[] = {{&S34394[108], 9}, {NULL}};
const Segment SGI80236[] = {{&S34394[81], 6}, {NULL}};
const Segment SGI80237[] = {{&S34400[34], 8}, {NULL}};
const Segment SGI80238[] = {{&S34400[27], 6}, {NULL}};
const Segment SGI80239[] = {{&S34400[95], 9}, {NULL}};
const Segment SGI80240[] = {{&S34400[68], 6}, {NULL}};
const Segment SGI80241[] = {{&S53191[44], 13}, {NULL}};
const Segment SGI80242[] = {{&S53191[44], 30}, {NULL}};
const Segment SGI80243[] = {{&S53191[30], 6}, {NULL}};
const Segment SGI80244[] = {{&S53191[122], 3}, {NULL}};
const Segment SGI80245[] = {{&S53191[88], 5}, {NULL}};
const Segment SGI80246[] = {{&S53192[44], 13}, {NULL}};
const Segment SGI80247[] = {{&S53192[44], 30}, {NULL}};
const Segment SGI80248[] = {{&S53192[30], 6}, {NULL}};
const Segment SGI80249[] = {{&S53192[91], 6}, {NULL}};
const Segment SGI80250[] = {{&S53251[44], 7}, {NULL}};
const Segment SGI80251[] = {{&S53251[44], 32}, {NULL}};
const Segment SGI80252[] = {{&S53251[30], 6}, {NULL}};
const Segment SGI80253[] = {{&S53251[124], 3}, {NULL}};
const Segment SGI80254[] = {{&S53251[90], 5}, {NULL}};
const Segment SGI80255[] = {{&S53252[44], 7}, {NULL}};
const Segment SGI80256[] = {{&S53252[44], 32}, {NULL}};
const Segment SGI80257[] = {{&S53252[30], 6}, {NULL}};
const Segment SGI80258[] = {{&S53252[124], 3}, {NULL}};
const Segment SGI80259[] = {{&S53252[90], 5}, {NULL}};
const Segment SGI80260[] = {{&S53253[44], 7}, {NULL}};
const Segment SGI80261[] = {{&S53253[44], 32}, {NULL}};
const Segment SGI80262[] = {{&S53253[30], 6}, {NULL}};
const Segment SGI80263[] = {{&S53253[124], 3}, {NULL}};
const Segment SGI80264[] = {{&S53253[90], 5}, {NULL}};
const Segment SGI80265[] = {{&S53254[44], 7}, {NULL}};
const Segment SGI80266[] = {{&S53254[44], 32}, {NULL}};
const Segment SGI80267[] = {{&S53254[30], 6}, {NULL}};
const Segment SGI80268[] = {{&S53254[124], 3}, {NULL}};
const Segment SGI80269[] = {{&S53254[90], 5}, {NULL}};
const Segment SGI80270[] = {{&S53255[44], 7}, {NULL}};
const Segment SGI80271[] = {{&S53255[44], 32}, {NULL}};
const Segment SGI80272[] = {{&S53255[30], 6}, {NULL}};
const Segment SGI80273[] = {{&S53255[124], 3}, {NULL}};
const Segment SGI80274[] = {{&S53255[90], 5}, {NULL}};
const Segment SGI80275[] = {{&S37809[44], 5}, {NULL}};
const Segment SGI80276[] = {{&S37809[44], 18}, {NULL}};
const Segment SGI80277[] = {{&S37809[30], 6}, {NULL}};
const Segment SGI80278[] = {{&S37809[31], 3}, {NULL}};
const Segment SGI80279[] = {{&S37809[76], 6}, {NULL}};
const Segment SGI80280[] = {{&S37817[44], 12}, {NULL}};
const Segment SGI80281[] = {{&S37817[44], 25}, {NULL}};
const Segment SGI80282[] = {{&S37817[30], 6}, {NULL}};
const Segment SGI80283[] = {{&S37817[31], 3}, {NULL}};
const Segment SGI80284[] = {{&S37817[83], 6}, {NULL}};
const Segment SGI80285[] = {{&S37826[38], 5}, {NULL}};
const Segment SGI80286[] = {{&S37826[31], 6}, {NULL}};
const Segment SGI80287[] = {{&S37830[44], 5}, {NULL}};
const Segment SGI80288[] = {{&S37830[44], 18}, {NULL}};
const Segment SGI80289[] = {{&S37830[30], 6}, {NULL}};
const Segment SGI80290[] = {{&S37830[31], 3}, {NULL}};
const Segment SGI80291[] = {{&S37830[76], 6}, {NULL}};
const Segment SGI80292[] = {{&S53294[44], 5}, {NULL}};
const Segment SGI80293[] = {{&S53294[44], 17}, {NULL}};
const Segment SGI80294[] = {{&S53294[30], 6}, {NULL}};
const Segment SGI80295[] = {{&S53294[109], 3}, {NULL}};
const Segment SGI80296[] = {{&S53294[75], 5}, {NULL}};
const Segment SGI80297[] = {{&S53295[38], 5}, {NULL}};
const Segment SGI80298[] = {{&S53295[31], 6}, {NULL}};
const Segment SGI80299[] = {{&S53296[44], 5}, {NULL}};
const Segment SGI80300[] = {{&S53296[44], 17}, {NULL}};
const Segment SGI80301[] = {{&S53296[30], 6}, {NULL}};
const Segment SGI80302[] = {{&S53296[78], 4}, {NULL}};
const Segment SGI80303[] = {{&S53296[78], 3}, {NULL}};
const Segment SGI80304[] = {{&S34551[44], 29}, {NULL}};
const Segment SGI80305[] = {{&S34551[44], 41}, {NULL}};
const Segment SGI80306[] = {{&S34551[30], 6}, {NULL}};
const Segment SGI80307[] = {{&S34551[31], 3}, {NULL}};
const Segment SGI80308[] = {{&S34551[99], 6}, {NULL}};
const Segment SGI80309[] = {{&S34552[38], 29}, {NULL}};
const Segment SGI80310[] = {{&S34552[31], 6}, {NULL}};
const Segment SGI80311[] = {{&S34553[44], 30}, {NULL}};
const Segment SGI80312[] = {{&S34553[44], 42}, {NULL}};
const Segment SGI80313[] = {{&S34553[30], 6}, {NULL}};
const Segment SGI80314[] = {{&S34553[31], 3}, {NULL}};
const Segment SGI80315[] = {{&S34553[100], 6}, {NULL}};
const Segment SGI80316[] = {{&S34554[44], 17}, {NULL}};
const Segment SGI80317[] = {{&S34554[44], 29}, {NULL}};
const Segment SGI80318[] = {{&S34554[30], 6}, {NULL}};
const Segment SGI80319[] = {{&S34554[31], 3}, {NULL}};
const Segment SGI80320[] = {{&S34554[87], 6}, {NULL}};
const Segment SGI80321[] = {{&S34555[38], 30}, {NULL}};
const Segment SGI80322[] = {{&S34555[31], 6}, {NULL}};
const Segment SGI80323[] = {{&S47388[44], 11}, {NULL}};
const Segment SGI80324[] = {{&S47388[44], 23}, {NULL}};
const Segment SGI80325[] = {{&S47388[30], 6}, {NULL}};
const Segment SGI80326[] = {{&S47388[134], 3}, {NULL}};
const Segment SGI80327[] = {{&S47388[100], 5}, {NULL}};
const Segment SGI80328[] = {{&S47388[35], 1}, {NULL}};
const Segment SGI80329[] = {{&S34557[44], 30}, {NULL}};
const Segment SGI80330[] = {{&S47388[69], 11}, {NULL}};
const Segment SGI80331[] = {{&S34557[44], 42}, {NULL}};
const Segment SGI80332[] = {{&S34557[30], 6}, {NULL}};
const Segment SGI80333[] = {{&S34557[31], 3}, {NULL}};
const Segment SGI80334[] = {{&S34557[100], 6}, {NULL}};
const Segment SGI80335[] = {{&S47389[44], 11}, {NULL}};
const Segment SGI80336[] = {{&S47389[44], 23}, {NULL}};
const Segment SGI80337[] = {{&S47389[30], 6}, {NULL}};
const Segment SGI80338[] = {{&S47389[137], 3}, {NULL}};
const Segment SGI80339[] = {{&S47389[103], 5}, {NULL}};
const Segment SGI80340[] = {{&S34558[44], 29}, {NULL}};
const Segment SGI80341[] = {{&S34558[44], 41}, {NULL}};
const Segment SGI80342[] = {{&S47389[78], 11}, {NULL}};
const Segment SGI80343[] = {{&S34558[30], 6}, {NULL}};
const Segment SGI80344[] = {{&S34558[31], 3}, {NULL}};
const Segment SGI80345[] = {{&S34558[99], 6}, {NULL}};
const Segment SGI80346[] = {{&S46396[44], 14}, {NULL}};
const Segment SGI80347[] = {{&S46396[44], 25}, {NULL}};
const Segment SGI80348[] = {{&S46396[30], 6}, {NULL}};
const Segment SGI80349[] = {{&S46396[118], 3}, {NULL}};
const Segment SGI80350[] = {{&S46396[83], 6}, {NULL}};
const Segment SGI80351[] = {{&S46397[38], 14}, {NULL}};
const Segment SGI80352[] = {{&S46397[31], 6}, {NULL}};
const Segment SGI80353[] = {{&S53442[44], 10}, {NULL}};
const Segment SGI80354[] = {{&S53442[44], 22}, {NULL}};
const Segment SGI80355[] = {{&S53442[30], 6}, {NULL}};
const Segment SGI80356[] = {{&S53442[114], 3}, {NULL}};
const Segment SGI80357[] = {{&S53442[80], 5}, {NULL}};
const Segment SGI80358[] = {{&S53443[44], 23}, {NULL}};
const Segment SGI80359[] = {{&S53443[44], 35}, {NULL}};
const Segment SGI80360[] = {{&S53443[30], 6}, {NULL}};
const Segment SGI80361[] = {{&S53443[127], 3}, {NULL}};
const Segment SGI80362[] = {{&S53443[93], 5}, {NULL}};
const Segment SGI80363[] = {{&S53444[38], 10}, {NULL}};
const Segment SGI80364[] = {{&S53444[31], 6}, {NULL}};
const Segment SGI80365[] = {{&S53447[44], 11}, {NULL}};
const Segment SGI80366[] = {{&S53447[44], 23}, {NULL}};
const Segment SGI80367[] = {{&S53447[30], 6}, {NULL}};
const Segment SGI80368[] = {{&S53447[115], 3}, {NULL}};
const Segment SGI80369[] = {{&S53447[81], 5}, {NULL}};
const Segment SGI80370[] = {{&S53448[44], 6}, {NULL}};
const Segment SGI80371[] = {{&S53448[44], 18}, {NULL}};
const Segment SGI80372[] = {{&S53448[30], 6}, {NULL}};
const Segment SGI80373[] = {{&S53448[110], 3}, {NULL}};
const Segment SGI80374[] = {{&S53448[76], 5}, {NULL}};
const Segment SGI80375[] = {{&S53449[38], 6}, {NULL}};
const Segment SGI80376[] = {{&S53449[31], 6}, {NULL}};
const Segment SGI80377[] = {{&S53453[44], 10}, {NULL}};
const Segment SGI80378[] = {{&S53453[44], 21}, {NULL}};
const Segment SGI80379[] = {{&S53453[30], 6}, {NULL}};
const Segment SGI80380[] = {{&S53453[113], 3}, {NULL}};
const Segment SGI80381[] = {{&S53453[79], 5}, {NULL}};
const Segment SGI80382[] = {{&S53455[44], 10}, {NULL}};
const Segment SGI80383[] = {{&S53455[44], 22}, {NULL}};
const Segment SGI80384[] = {{&S53455[30], 6}, {NULL}};
const Segment SGI80385[] = {{&S53455[114], 3}, {NULL}};
const Segment SGI80386[] = {{&S53455[80], 5}, {NULL}};
const Segment SGI80387[] = {{&S53456[38], 10}, {NULL}};
const Segment SGI80388[] = {{&S53456[31], 6}, {NULL}};
const Segment SGI80389[] = {{&S53457[44], 10}, {NULL}};
const Segment SGI80390[] = {{&S53457[44], 21}, {NULL}};
const Segment SGI80391[] = {{&S53457[30], 6}, {NULL}};
const Segment SGI80392[] = {{&S53457[113], 3}, {NULL}};
const Segment SGI80393[] = {{&S53457[79], 5}, {NULL}};
const Segment SGI80394[] = {{&S47480[44], 13}, {NULL}};
const Segment SGI80395[] = {{&S47480[44], 25}, {NULL}};
const Segment SGI80396[] = {{&S47480[30], 6}, {NULL}};
const Segment SGI80397[] = {{&S47480[136], 3}, {NULL}};
const Segment SGI80398[] = {{&S47480[102], 5}, {NULL}};
const Segment SGI80399[] = {{&S47480[35], 1}, {NULL}};
const Segment SGI80400[] = {{&S47480[71], 11}, {NULL}};
const Segment SGI80401[] = {{&S47482[44], 9}, {NULL}};
const Segment SGI80402[] = {{&S47482[44], 21}, {NULL}};
const Segment SGI80403[] = {{&S47482[30], 6}, {NULL}};
const Segment SGI80404[] = {{&S47482[128], 3}, {NULL}};
const Segment SGI80405[] = {{&S47482[94], 5}, {NULL}};
const Segment SGI80406[] = {{&S47482[35], 1}, {NULL}};
const Segment SGI80407[] = {{&S47482[67], 11}, {NULL}};
const Segment SGI80408[] = {{&S53521[44], 7}, {NULL}};
const Segment SGI80409[] = {{&S53521[44], 33}, {NULL}};
const Segment SGI80410[] = {{&S53521[30], 6}, {NULL}};
const Segment SGI80411[] = {{&S53521[125], 3}, {NULL}};
const Segment SGI80412[] = {{&S53521[91], 5}, {NULL}};
const Segment SGI80413[] = {{&S53522[44], 7}, {NULL}};
const Segment SGI80414[] = {{&S53522[44], 33}, {NULL}};
const Segment SGI80415[] = {{&S53522[30], 6}, {NULL}};
const Segment SGI80416[] = {{&S53522[125], 3}, {NULL}};
const Segment SGI80417[] = {{&S53522[91], 5}, {NULL}};
const Segment SGI80418[] = {{&S53523[44], 7}, {NULL}};
const Segment SGI80419[] = {{&S53523[44], 33}, {NULL}};
const Segment SGI80420[] = {{&S53523[30], 6}, {NULL}};
const Segment SGI80421[] = {{&S53523[125], 3}, {NULL}};
const Segment SGI80422[] = {{&S53523[91], 5}, {NULL}};
const Segment SGI80423[] = {{&S53524[44], 7}, {NULL}};
const Segment SGI80424[] = {{&S53524[44], 33}, {NULL}};
const Segment SGI80425[] = {{&S53524[30], 6}, {NULL}};
const Segment SGI80426[] = {{&S53524[125], 3}, {NULL}};
const Segment SGI80427[] = {{&S53524[91], 5}, {NULL}};
const Segment SGI80428[] = {{&S53525[44], 7}, {NULL}};
const Segment SGI80429[] = {{&S53525[44], 33}, {NULL}};
const Segment SGI80430[] = {{&S53525[30], 6}, {NULL}};
const Segment SGI80431[] = {{&S53525[125], 3}, {NULL}};
const Segment SGI80432[] = {{&S53525[91], 5}, {NULL}};
const Segment SGI80433[] = {{&S46562[42], 7}, {NULL}};
const Segment SGI80434[] = {{&S46562[42], 19}, {NULL}};
const Segment SGI80435[] = {{&S46562[30], 4}, {NULL}};
const Segment SGI80436[] = {{&S46562[110], 3}, {NULL}};
const Segment SGI80437[] = {{&S46562[75], 6}, {NULL}};
const Segment SGI80438[] = {{&S46563[44], 7}, {NULL}};
const Segment SGI80439[] = {{&S46563[44], 19}, {NULL}};
const Segment SGI80440[] = {{&S46563[30], 6}, {NULL}};
const Segment SGI80441[] = {{&S46563[112], 3}, {NULL}};
const Segment SGI80442[] = {{&S46563[77], 6}, {NULL}};
const Segment SGI80443[] = {{&S46564[38], 7}, {NULL}};
const Segment SGI80444[] = {{&S46564[31], 6}, {NULL}};
const Segment SGI80445[] = {{&S47634[44], 4}, {NULL}};
const Segment SGI80446[] = {{&S47634[44], 18}, {NULL}};
const Segment SGI80447[] = {{&S47634[30], 6}, {NULL}};
const Segment SGI80448[] = {{&S47634[120], 3}, {NULL}};
const Segment SGI80449[] = {{&S47634[86], 5}, {NULL}};
const Segment SGI80450[] = {{&S53606[44], 5}, {NULL}};
const Segment SGI80451[] = {{&S53606[44], 26}, {NULL}};
const Segment SGI80452[] = {{&S53606[30], 6}, {NULL}};
const Segment SGI80453[] = {{&S53606[118], 3}, {NULL}};
const Segment SGI80454[] = {{&S47635[44], 11}, {NULL}};
const Segment SGI80455[] = {{&S53606[84], 5}, {NULL}};
const Segment SGI80456[] = {{&S47635[44], 20}, {NULL}};
const Segment SGI80457[] = {{&S47635[30], 6}, {NULL}};
const Segment SGI80458[] = {{&S47635[112], 3}, {NULL}};
const Segment SGI80459[] = {{&S47635[78], 5}, {NULL}};
const Segment SGI80460[] = {{&S47636[44], 6}, {NULL}};
const Segment SGI80461[] = {{&S47636[44], 25}, {NULL}};
const Segment SGI80462[] = {{&S47636[30], 6}, {NULL}};
const Segment SGI80463[] = {{&S47636[56], 3}, {NULL}};
const Segment SGI80464[] = {{&S47636[97], 5}, {NULL}};
const Segment SGI80465[] = {{&S47637[44], 6}, {NULL}};
const Segment SGI80466[] = {{&S47637[44], 25}, {NULL}};
const Segment SGI80467[] = {{&S47637[30], 6}, {NULL}};
const Segment SGI80468[] = {{&S47637[131], 3}, {NULL}};
const Segment SGI80469[] = {{&S46591[44], 6}, {NULL}};
const Segment SGI80470[] = {{&S46591[44], 17}, {NULL}};
const Segment SGI80471[] = {{&S47637[97], 5}, {NULL}};
const Segment SGI80472[] = {{&S46591[30], 6}, {NULL}};
const Segment SGI80473[] = {{&S46591[110], 3}, {NULL}};
const Segment SGI80474[] = {{&S46591[75], 6}, {NULL}};
const Segment SGI80475[] = {{&S47638[44], 4}, {NULL}};
const Segment SGI80476[] = {{&S47638[44], 13}, {NULL}};
const Segment SGI80477[] = {{&S47638[30], 6}, {NULL}};
const Segment SGI80478[] = {{&S46592[38], 6}, {NULL}};
const Segment SGI80479[] = {{&S46592[31], 6}, {NULL}};
const Segment SGI80480[] = {{&S47638[105], 3}, {NULL}};
const Segment SGI80481[] = {{&S47638[71], 5}, {NULL}};
const Segment SGI80482[] = {{&S47639[38], 4}, {NULL}};
const Segment SGI80483[] = {{&S46593[44], 6}, {NULL}};
const Segment SGI80484[] = {{&S47639[31], 6}, {NULL}};
const Segment SGI80485[] = {{&S46593[44], 17}, {NULL}};
const Segment SGI80486[] = {{&S46593[30], 6}, {NULL}};
const Segment SGI80487[] = {{&S46593[110], 3}, {NULL}};
const Segment SGI80488[] = {{&S46593[75], 6}, {NULL}};
const Segment SGI80489[] = {{&S47640[44], 4}, {NULL}};
const Segment SGI80490[] = {{&S47640[44], 22}, {NULL}};
const Segment SGI80491[] = {{&S47640[30], 6}, {NULL}};
const Segment SGI80492[] = {{&S47640[128], 3}, {NULL}};
const Segment SGI80493[] = {{&S47640[94], 5}, {NULL}};
const Segment SGI80494[] = {{&S47641[44], 6}, {NULL}};
const Segment SGI80495[] = {{&S47641[44], 24}, {NULL}};
const Segment SGI80496[] = {{&S47641[30], 6}, {NULL}};
const Segment SGI80497[] = {{&S47641[56], 3}, {NULL}};
const Segment SGI80498[] = {{&S47641[96], 5}, {NULL}};
const Segment SGI80499[] = {{&S47646[44], 4}, {NULL}};
const Segment SGI80500[] = {{&S47646[44], 13}, {NULL}};
const Segment SGI80501[] = {{&S47646[30], 6}, {NULL}};
const Segment SGI80502[] = {{&S47646[105], 3}, {NULL}};
const Segment SGI80503[] = {{&S47646[71], 5}, {NULL}};
const Segment SGI80504[] = {{&S47648[38], 6}, {NULL}};
const Segment SGI80505[] = {{&S47648[31], 6}, {NULL}};
const Segment SGI80506[] = {{&S47649[44], 6}, {NULL}};
const Segment SGI80507[] = {{&S47649[44], 17}, {NULL}};
const Segment SGI80508[] = {{&S47649[30], 6}, {NULL}};
const Segment SGI80509[] = {{&S47649[123], 3}, {NULL}};
const Segment SGI80510[] = {{&S47649[89], 5}, {NULL}};
const Segment SGI80511[] = {{&S47650[44], 6}, {NULL}};
const Segment SGI80512[] = {{&S47650[44], 27}, {NULL}};
const Segment SGI80513[] = {{&S47650[30], 6}, {NULL}};
const Segment SGI80514[] = {{&S47650[119], 3}, {NULL}};
const Segment SGI80515[] = {{&S47650[85], 5}, {NULL}};
const Segment SGI80516[] = {{&S47651[44], 6}, {NULL}};
const Segment SGI80517[] = {{&S47651[44], 25}, {NULL}};
const Segment SGI80518[] = {{&S47651[30], 6}, {NULL}};
const Segment SGI80519[] = {{&S47651[56], 3}, {NULL}};
const Segment SGI80520[] = {{&S47651[99], 5}, {NULL}};
const Segment SGI80521[] = {{&S47653[48], 6}, {NULL}};
const Segment SGI80522[] = {{&S47653[48], 25}, {NULL}};
const Segment SGI80523[] = {{&S47653[30], 10}, {NULL}};
const Segment SGI80524[] = {{&S47653[60], 3}, {NULL}};
const Segment SGI80525[] = {{&S47653[103], 5}, {NULL}};
const Segment SGI80526[] = {{&S47654[44], 4}, {NULL}};
const Segment SGI80527[] = {{&S47654[44], 23}, {NULL}};
const Segment SGI80528[] = {{&S47654[30], 6}, {NULL}};
const Segment SGI80529[] = {{&S47654[130], 3}, {NULL}};
const Segment SGI80530[] = {{&S47654[96], 5}, {NULL}};
const Segment SGI80531[] = {{&S47656[44], 6}, {NULL}};
const Segment SGI80532[] = {{&S47656[44], 24}, {NULL}};
const Segment SGI80533[] = {{&S47656[30], 6}, {NULL}};
const Segment SGI80534[] = {{&S47656[130], 3}, {NULL}};
const Segment SGI80535[] = {{&S47656[96], 5}, {NULL}};
const Segment SGI80536[] = {{&S47657[44], 6}, {NULL}};
const Segment SGI80537[] = {{&S47657[44], 22}, {NULL}};
const Segment SGI80538[] = {{&S47657[30], 6}, {NULL}};
const Segment SGI80539[] = {{&S47657[126], 3}, {NULL}};
const Segment SGI80540[] = {{&S47657[92], 5}, {NULL}};
const Segment SGI80541[] = {{&S47658[44], 6}, {NULL}};
const Segment SGI80542[] = {{&S47658[44], 21}, {NULL}};
const Segment SGI80543[] = {{&S47658[30], 6}, {NULL}};
const Segment SGI80544[] = {{&S47658[127], 3}, {NULL}};
const Segment SGI80545[] = {{&S47658[93], 5}, {NULL}};
const Segment SGI80546[] = {{&S47659[44], 6}, {NULL}};
const Segment SGI80547[] = {{&S47659[44], 16}, {NULL}};
const Segment SGI80548[] = {{&S47659[30], 6}, {NULL}};
const Segment SGI80549[] = {{&S47659[117], 3}, {NULL}};
const Segment SGI80550[] = {{&S47659[83], 5}, {NULL}};
const Segment SGI80551[] = {{&S53650[44], 8}, {NULL}};
const Segment SGI80552[] = {{&S47660[44], 6}, {NULL}};
const Segment SGI80553[] = {{&S53650[44], 25}, {NULL}};
const Segment SGI80554[] = {{&S47660[44], 40}, {NULL}};
const Segment SGI80555[] = {{&S53650[30], 6}, {NULL}};
const Segment SGI80556[] = {{&S47660[30], 6}, {NULL}};
const Segment SGI80557[] = {{&S53650[117], 3}, {NULL}};
const Segment SGI80558[] = {{&S47660[161], 3}, {NULL}};
const Segment SGI80559[] = {{&S53650[83], 5}, {NULL}};
const Segment SGI80560[] = {{&S47660[127], 5}, {NULL}};
const Segment SGI80561[] = {{&S47661[44], 5}, {NULL}};
const Segment SGI80562[] = {{&S47661[44], 16}, {NULL}};
const Segment SGI80563[] = {{&S47661[30], 6}, {NULL}};
const Segment SGI80564[] = {{&S53652[38], 8}, {NULL}};
const Segment SGI80565[] = {{&S53652[31], 6}, {NULL}};
const Segment SGI80566[] = {{&S47661[120], 3}, {NULL}};
const Segment SGI80567[] = {{&S47661[86], 5}, {NULL}};
const Segment SGI80568[] = {{&S47662[44], 6}, {NULL}};
const Segment SGI80569[] = {{&S47662[44], 24}, {NULL}};
const Segment SGI80570[] = {{&S47662[30], 6}, {NULL}};
const Segment SGI80571[] = {{&S47662[56], 3}, {NULL}};
const Segment SGI80572[] = {{&S47662[101], 5}, {NULL}};
const Segment SGI80573[] = {{&S53709[44], 6}, {NULL}};
const Segment SGI80574[] = {{&S53709[44], 22}, {NULL}};
const Segment SGI80575[] = {{&S53709[30], 6}, {NULL}};
const Segment SGI80576[] = {{&S53709[114], 3}, {NULL}};
const Segment SGI80577[] = {{&S53709[80], 5}, {NULL}};
const Segment SGI80578[] = {{&S40777[44], 11}, {NULL}};
const Segment SGI80579[] = {{&S40777[44], 23}, {NULL}};
const Segment SGI80580[] = {{&S40777[30], 6}, {NULL}};
const Segment SGI80581[] = {{&S40777[31], 3}, {NULL}};
const Segment SGI80582[] = {{&S40777[81], 5}, {NULL}};
const Segment SGI80583[] = {{&S40777[0], 13}, {NULL}};
const Segment SGI80584[] = {{&S34815[44], 9}, {NULL}};
const Segment SGI80585[] = {{&S34815[44], 21}, {NULL}};
const Segment SGI80586[] = {{&S34815[30], 6}, {NULL}};
const Segment SGI80587[] = {{&S34815[31], 3}, {NULL}};
const Segment SGI80588[] = {{&S34815[79], 6}, {NULL}};
const Segment SGI80589[] = {{&S34816[38], 9}, {NULL}};
const Segment SGI80590[] = {{&S34816[31], 6}, {NULL}};
const Segment SGI80591[] = {{&S34817[44], 14}, {NULL}};
const Segment SGI80592[] = {{&S34817[44], 26}, {NULL}};
const Segment SGI80593[] = {{&S34817[30], 6}, {NULL}};
const Segment SGI80594[] = {{&S34817[31], 3}, {NULL}};
const Segment SGI80595[] = {{&S34817[84], 6}, {NULL}};
const Segment SGI80596[] = {{&S34818[44], 14}, {NULL}};
const Segment SGI80597[] = {{&S34818[44], 26}, {NULL}};
const Segment SGI80598[] = {{&S34818[30], 6}, {NULL}};
const Segment SGI80599[] = {{&S34818[31], 3}, {NULL}};
const Segment SGI80600[] = {{&S34818[84], 6}, {NULL}};
const Segment SGI80601[] = {{&S34819[44], 14}, {NULL}};
const Segment SGI80602[] = {{&S34819[44], 26}, {NULL}};
const Segment SGI80603[] = {{&S34819[30], 6}, {NULL}};
const Segment SGI80604[] = {{&S34819[31], 3}, {NULL}};
const Segment SGI80605[] = {{&S34819[84], 6}, {NULL}};
const Segment SGI80606[] = {{&S34820[44], 14}, {NULL}};
const Segment SGI80607[] = {{&S34820[44], 26}, {NULL}};
const Segment SGI80608[] = {{&S34820[30], 6}, {NULL}};
const Segment SGI80609[] = {{&S34820[31], 3}, {NULL}};
const Segment SGI80610[] = {{&S34820[84], 6}, {NULL}};
const Segment SGI80611[] = {{&S34821[44], 10}, {NULL}};
const Segment SGI80612[] = {{&S34821[44], 22}, {NULL}};
const Segment SGI80613[] = {{&S34821[30], 6}, {NULL}};
const Segment SGI80614[] = {{&S34821[31], 3}, {NULL}};
const Segment SGI80615[] = {{&S34821[80], 6}, {NULL}};
const Segment SGI80616[] = {{&S34822[44], 9}, {NULL}};
const Segment SGI80617[] = {{&S34822[44], 21}, {NULL}};
const Segment SGI80618[] = {{&S34822[30], 6}, {NULL}};
const Segment SGI80619[] = {{&S34822[82], 6}, {NULL}};
const Segment SGI80620[] = {{&S34823[44], 14}, {NULL}};
const Segment SGI80621[] = {{&S34823[44], 26}, {NULL}};
const Segment SGI80622[] = {{&S34823[30], 6}, {NULL}};
const Segment SGI80623[] = {{&S34823[31], 3}, {NULL}};
const Segment SGI80624[] = {{&S34823[84], 6}, {NULL}};
const Segment SGI80625[] = {{&S34824[41], 9}, {NULL}};
const Segment SGI80626[] = {{&S34824[30], 6}, {NULL}};
const Segment SGI80627[] = {{&S34824[31], 3}, {NULL}};
const Segment SGI80628[] = {{&S34824[76], 6}, {NULL}};
const Segment SGI80629[] = {{&S34825[38], 10}, {NULL}};
const Segment SGI80630[] = {{&S34825[31], 6}, {NULL}};
const Segment SGI80631[] = {{&S47753[44], 11}, {NULL}};
const Segment SGI80632[] = {{&S47753[44], 23}, {NULL}};
const Segment SGI80633[] = {{&S47753[30], 6}, {NULL}};
const Segment SGI80634[] = {{&S47753[115], 3}, {NULL}};
const Segment SGI80635[] = {{&S47753[81], 5}, {NULL}};
const Segment SGI80636[] = {{&S47755[44], 11}, {NULL}};
const Segment SGI80637[] = {{&S47755[44], 22}, {NULL}};
const Segment SGI80638[] = {{&S47755[30], 6}, {NULL}};
const Segment SGI80639[] = {{&S47755[114], 3}, {NULL}};
const Segment SGI80640[] = {{&S47755[80], 5}, {NULL}};
const Segment SGI80641[] = {{&S47756[44], 11}, {NULL}};
const Segment SGI80642[] = {{&S47756[44], 22}, {NULL}};
const Segment SGI80643[] = {{&S47756[30], 6}, {NULL}};
const Segment SGI80644[] = {{&S47757[44], 11}, {NULL}};
const Segment SGI80645[] = {{&S47757[44], 23}, {NULL}};
const Segment SGI80646[] = {{&S47757[30], 6}, {NULL}};
const Segment SGI80647[] = {{&S47757[81], 6}, {NULL}};
const Segment SGI80648[] = {{&S47758[38], 11}, {NULL}};
const Segment SGI80649[] = {{&S47758[31], 6}, {NULL}};
const Segment SGI80650[] = {{&S53756[44], 10}, {NULL}};
const Segment SGI80651[] = {{&S53756[44], 27}, {NULL}};
const Segment SGI80652[] = {{&S53756[30], 6}, {NULL}};
const Segment SGI80653[] = {{&S53756[149], 4}, {NULL}};
const Segment SGI80654[] = {{&S53756[175], 3}, {NULL}};
const Segment SGI80655[] = {{&S53756[119], 3}, {&S53756[224], 5}, {NULL}};
const Segment SGI80656[] = {{&S53756[85], 5}, {NULL}};
const Segment SGI80657[] = {{&S53837[44], 6}, {NULL}};
const Segment SGI80658[] = {{&S53837[44], 27}, {NULL}};
const Segment SGI80659[] = {{&S53837[30], 6}, {NULL}};
const Segment SGI80660[] = {{&S53837[119], 3}, {NULL}};
const Segment SGI80661[] = {{&S53837[85], 5}, {NULL}};
const Segment SGI80662[] = {{&S53838[44], 6}, {NULL}};
const Segment SGI80663[] = {{&S53838[44], 31}, {NULL}};
const Segment SGI80664[] = {{&S53838[30], 6}, {NULL}};
const Segment SGI80665[] = {{&S53838[123], 3}, {NULL}};
const Segment SGI80666[] = {{&S53838[89], 5}, {NULL}};
const Segment SGI80667[] = {{&S53842[44], 6}, {NULL}};
const Segment SGI80668[] = {{&S53842[44], 31}, {NULL}};
const Segment SGI80669[] = {{&S53842[30], 6}, {NULL}};
const Segment SGI80670[] = {{&S53842[123], 3}, {NULL}};
const Segment SGI80671[] = {{&S53842[89], 5}, {NULL}};
const Segment SGI80672[] = {{&S53843[44], 6}, {NULL}};
const Segment SGI80673[] = {{&S53843[44], 31}, {NULL}};
const Segment SGI80674[] = {{&S53843[30], 6}, {NULL}};
const Segment SGI80675[] = {{&S53843[123], 3}, {NULL}};
const Segment SGI80676[] = {{&S53843[89], 5}, {NULL}};
const Segment SGI80677[] = {{&S47863[44], 20}, {NULL}};
const Segment SGI80678[] = {{&S47863[44], 38}, {NULL}};
const Segment SGI80679[] = {{&S47863[30], 6}, {NULL}};
const Segment SGI80680[] = {{&S47863[70], 3}, {NULL}};
const Segment SGI80681[] = {{&S47863[96], 5}, {NULL}};
const Segment SGI80682[] = {{&S46792[42], 7}, {NULL}};
const Segment SGI80683[] = {{&S46792[42], 21}, {NULL}};
const Segment SGI80684[] = {{&S46792[30], 4}, {NULL}};
const Segment SGI80685[] = {{&S46792[112], 3}, {NULL}};
const Segment SGI80686[] = {{&S46792[77], 6}, {NULL}};
const Segment SGI80687[] = {{&S46793[42], 7}, {NULL}};
const Segment SGI80688[] = {{&S46793[42], 21}, {NULL}};
const Segment SGI80689[] = {{&S46793[30], 4}, {NULL}};
const Segment SGI80690[] = {{&S46793[112], 3}, {NULL}};
const Segment SGI80691[] = {{&S46793[77], 6}, {NULL}};
const Segment SGI80692[] = {{&S46794[36], 7}, {NULL}};
const Segment SGI80693[] = {{&S46794[31], 4}, {NULL}};
const Segment SGI80694[] = {{&S46795[42], 7}, {NULL}};
const Segment SGI80695[] = {{&S46795[42], 23}, {NULL}};
const Segment SGI80696[] = {{&S46795[30], 4}, {NULL}};
const Segment SGI80697[] = {{&S46795[114], 3}, {NULL}};
const Segment SGI80698[] = {{&S46795[79], 6}, {NULL}};
const Segment SGI80699[] = {{&S46796[42], 7}, {NULL}};
const Segment SGI80700[] = {{&S46796[42], 21}, {NULL}};
const Segment SGI80701[] = {{&S46796[30], 4}, {NULL}};
const Segment SGI80702[] = {{&S46796[112], 3}, {NULL}};
const Segment SGI80703[] = {{&S46796[77], 6}, {NULL}};
const Segment SGI80704[] = {{&S48005[44], 15}, {NULL}};
const Segment SGI80705[] = {{&S48005[44], 32}, {NULL}};
const Segment SGI80706[] = {{&S48005[30], 6}, {NULL}};
const Segment SGI80707[] = {{&S48005[124], 3}, {NULL}};
const Segment SGI80708[] = {{&S48005[90], 5}, {NULL}};
const Segment SGI80709[] = {{&S48006[44], 23}, {NULL}};
const Segment SGI80710[] = {{&S48006[44], 40}, {NULL}};
const Segment SGI80711[] = {{&S48006[30], 6}, {NULL}};
const Segment SGI80712[] = {{&S48006[132], 3}, {NULL}};
const Segment SGI80713[] = {{&S48006[98], 5}, {NULL}};
const Segment SGI80714[] = {{&S48007[38], 15}, {NULL}};
const Segment SGI80715[] = {{&S48007[31], 6}, {NULL}};
const Segment SGI80716[] = {{&S35030[44], 10}, {NULL}};
const Segment SGI80717[] = {{&S35030[44], 22}, {NULL}};
const Segment SGI80718[] = {{&S35030[30], 6}, {NULL}};
const Segment SGI80719[] = {{&S35030[31], 3}, {NULL}};
const Segment SGI80720[] = {{&S35030[80], 6}, {NULL}};
const Segment SGI80721[] = {{&S35031[44], 18}, {NULL}};
const Segment SGI80722[] = {{&S35031[44], 30}, {NULL}};
const Segment SGI80723[] = {{&S35031[30], 6}, {NULL}};
const Segment SGI80724[] = {{&S35031[31], 3}, {NULL}};
const Segment SGI80725[] = {{&S35031[88], 6}, {NULL}};
const Segment SGI80726[] = {{&S48023[44], 9}, {NULL}};
const Segment SGI80727[] = {{&S48023[44], 20}, {NULL}};
const Segment SGI80728[] = {{&S48023[30], 6}, {NULL}};
const Segment SGI80729[] = {{&S48023[127], 3}, {NULL}};
const Segment SGI80730[] = {{&S48023[93], 5}, {NULL}};
const Segment SGI80731[] = {{&S48023[35], 1}, {NULL}};
const Segment SGI80732[] = {{&S48023[66], 11}, {NULL}};
const Segment SGI80733[] = {{&S35033[38], 18}, {NULL}};
const Segment SGI80734[] = {{&S35033[31], 6}, {NULL}};
const Segment SGI80735[] = {{&S48024[44], 9}, {NULL}};
const Segment SGI80736[] = {{&S48024[44], 20}, {NULL}};
const Segment SGI80737[] = {{&S48024[30], 6}, {NULL}};
const Segment SGI80738[] = {{&S48024[128], 3}, {NULL}};
const Segment SGI80739[] = {{&S48024[94], 5}, {NULL}};
const Segment SGI80740[] = {{&S48024[77], 3}, {NULL}};
const Segment SGI80741[] = {{&S35034[44], 13}, {NULL}};
const Segment SGI80742[] = {{&S35034[44], 24}, {NULL}};
const Segment SGI80743[] = {{&S35034[30], 6}, {NULL}};
const Segment SGI80744[] = {{&S35034[31], 3}, {NULL}};
const Segment SGI80745[] = {{&S48025[44], 5}, {NULL}};
const Segment SGI80746[] = {{&S35034[82], 6}, {NULL}};
const Segment SGI80747[] = {{&S48025[44], 17}, {NULL}};
const Segment SGI80748[] = {{&S48025[30], 6}, {NULL}};
const Segment SGI80749[] = {{&S48025[125], 3}, {NULL}};
const Segment SGI80750[] = {{&S48025[91], 5}, {NULL}};
const Segment SGI80751[] = {{&S48025[74], 3}, {NULL}};
const Segment SGI80752[] = {{&S35035[38], 10}, {NULL}};
const Segment SGI80753[] = {{&S35035[31], 6}, {NULL}};
const Segment SGI80754[] = {{&S48026[44], 9}, {NULL}};
const Segment SGI80755[] = {{&S48026[44], 20}, {NULL}};
const Segment SGI80756[] = {{&S48026[30], 6}, {NULL}};
const Segment SGI80757[] = {{&S48026[184], 3}, {NULL}};
const Segment SGI80758[] = {{&S48026[150], 5}, {NULL}};
const Segment SGI80759[] = {{&S48026[35], 1}, {NULL}};
const Segment SGI80760[] = {{&S48026[112], 11}, {NULL}};
const Segment SGI80761[] = {{&S48027[44], 5}, {NULL}};
const Segment SGI80762[] = {{&S48027[44], 16}, {NULL}};
const Segment SGI80763[] = {{&S48027[30], 6}, {NULL}};
const Segment SGI80764[] = {{&S48027[124], 3}, {NULL}};
const Segment SGI80765[] = {{&S48027[90], 5}, {NULL}};
const Segment SGI80766[] = {{&S48027[73], 3}, {NULL}};
const Segment SGI80767[] = {{&S48028[44], 5}, {NULL}};
const Segment SGI80768[] = {{&S48028[44], 16}, {NULL}};
const Segment SGI80769[] = {{&S48028[30], 6}, {NULL}};
const Segment SGI80770[] = {{&S48028[123], 3}, {NULL}};
const Segment SGI80771[] = {{&S48028[89], 5}, {NULL}};
const Segment SGI80772[] = {{&S48028[35], 1}, {NULL}};
const Segment SGI80773[] = {{&S48028[62], 11}, {NULL}};
const Segment SGI80774[] = {{&S48029[44], 9}, {NULL}};
const Segment SGI80775[] = {{&S48029[44], 20}, {NULL}};
const Segment SGI80776[] = {{&S48029[30], 6}, {NULL}};
const Segment SGI80777[] = {{&S48029[149], 3}, {NULL}};
const Segment SGI80778[] = {{&S48029[115], 5}, {NULL}};
const Segment SGI80779[] = {{&S48029[79], 11}, {NULL}};
const Segment SGI80780[] = {{&S48030[44], 9}, {NULL}};
const Segment SGI80781[] = {{&S48030[44], 20}, {NULL}};
const Segment SGI80782[] = {{&S48030[30], 6}, {NULL}};
const Segment SGI80783[] = {{&S48030[127], 3}, {NULL}};
const Segment SGI80784[] = {{&S48030[93], 5}, {NULL}};
const Segment SGI80785[] = {{&S48030[35], 1}, {NULL}};
const Segment SGI80786[] = {{&S48030[66], 11}, {NULL}};
const Segment SGI80787[] = {{&S48251[44], 11}, {NULL}};
const Segment SGI80788[] = {{&S48251[44], 23}, {NULL}};
const Segment SGI80789[] = {{&S48251[30], 6}, {NULL}};
const Segment SGI80790[] = {{&S48251[115], 3}, {NULL}};
const Segment SGI80791[] = {{&S48251[81], 5}, {NULL}};
const Segment SGI80792[] = {{&S48252[44], 11}, {NULL}};
const Segment SGI80793[] = {{&S48252[44], 23}, {NULL}};
const Segment SGI80794[] = {{&S48252[30], 6}, {NULL}};
const Segment SGI80795[] = {{&S48252[115], 3}, {NULL}};
const Segment SGI80796[] = {{&S48252[81], 5}, {NULL}};
const Segment SGI80797[] = {{&S48253[44], 7}, {NULL}};
const Segment SGI80798[] = {{&S48253[44], 19}, {NULL}};
const Segment SGI80799[] = {{&S48253[30], 6}, {NULL}};
const Segment SGI80800[] = {{&S48253[111], 3}, {NULL}};
const Segment SGI80801[] = {{&S48253[77], 5}, {NULL}};
const Segment SGI80802[] = {{&S48254[44], 7}, {NULL}};
const Segment SGI80803[] = {{&S48254[44], 19}, {NULL}};
const Segment SGI80804[] = {{&S48254[30], 6}, {NULL}};
const Segment SGI80805[] = {{&S48254[80], 6}, {NULL}};
const Segment SGI80806[] = {{&S48255[44], 7}, {NULL}};
const Segment SGI80807[] = {{&S48255[44], 19}, {NULL}};
const Segment SGI80808[] = {{&S48255[30], 6}, {NULL}};
const Segment SGI80809[] = {{&S48255[111], 3}, {NULL}};
const Segment SGI80810[] = {{&S48255[77], 5}, {NULL}};
const Segment SGI80811[] = {{&S48256[44], 11}, {NULL}};
const Segment SGI80812[] = {{&S48256[44], 23}, {NULL}};
const Segment SGI80813[] = {{&S48256[30], 6}, {NULL}};
const Segment SGI80814[] = {{&S48256[115], 3}, {NULL}};
const Segment SGI80815[] = {{&S48256[81], 5}, {NULL}};
const Segment SGI80816[] = {{&S48257[44], 11}, {NULL}};
const Segment SGI80817[] = {{&S48257[44], 23}, {NULL}};
const Segment SGI80818[] = {{&S48257[30], 6}, {NULL}};
const Segment SGI80819[] = {{&S48257[115], 3}, {NULL}};
const Segment SGI80820[] = {{&S48257[81], 5}, {NULL}};
const Segment SGI80821[] = {{&S48258[44], 11}, {NULL}};
const Segment SGI80822[] = {{&S48258[44], 23}, {NULL}};
const Segment SGI80823[] = {{&S48258[30], 6}, {NULL}};
const Segment SGI80824[] = {{&S48258[115], 3}, {NULL}};
const Segment SGI80825[] = {{&S48258[81], 5}, {NULL}};
const Segment SGI80826[] = {{&S48259[44], 11}, {NULL}};
const Segment SGI80827[] = {{&S48259[44], 23}, {NULL}};
const Segment SGI80828[] = {{&S48259[30], 6}, {NULL}};
const Segment SGI80829[] = {{&S48259[115], 3}, {NULL}};
const Segment SGI80830[] = {{&S48259[81], 5}, {NULL}};
const Segment SGI80831[] = {{&S48260[44], 7}, {NULL}};
const Segment SGI80832[] = {{&S48260[44], 19}, {NULL}};
const Segment SGI80833[] = {{&S48260[30], 6}, {NULL}};
const Segment SGI80834[] = {{&S48260[111], 3}, {NULL}};
const Segment SGI80835[] = {{&S48260[77], 5}, {NULL}};
const Segment SGI80836[] = {{&S48261[44], 11}, {NULL}};
const Segment SGI80837[] = {{&S48261[44], 23}, {NULL}};
const Segment SGI80838[] = {{&S48261[30], 6}, {NULL}};
const Segment SGI80839[] = {{&S48261[115], 3}, {NULL}};
const Segment SGI80840[] = {{&S48261[81], 5}, {NULL}};
const Segment SGI80841[] = {{&S48262[44], 7}, {NULL}};
const Segment SGI80842[] = {{&S48262[44], 19}, {NULL}};
const Segment SGI80843[] = {{&S48262[30], 6}, {NULL}};
const Segment SGI80844[] = {{&S48262[111], 3}, {NULL}};
const Segment SGI80845[] = {{&S48262[77], 5}, {NULL}};
const Segment SGI80846[] = {{&S48263[44], 7}, {NULL}};
const Segment SGI80847[] = {{&S48263[44], 19}, {NULL}};
const Segment SGI80848[] = {{&S48263[30], 6}, {NULL}};
const Segment SGI80849[] = {{&S48263[111], 3}, {NULL}};
const Segment SGI80850[] = {{&S48263[77], 5}, {NULL}};
const Segment SGI80851[] = {{&S48264[38], 7}, {NULL}};
const Segment SGI80852[] = {{&S48264[31], 6}, {NULL}};
const Segment SGI80853[] = {{&S48265[44], 11}, {NULL}};
const Segment SGI80854[] = {{&S48265[44], 23}, {NULL}};
const Segment SGI80855[] = {{&S48265[30], 6}, {NULL}};
const Segment SGI80856[] = {{&S48265[115], 3}, {NULL}};
const Segment SGI80857[] = {{&S48265[81], 5}, {NULL}};
const Segment SGI80858[] = {{&S48266[44], 7}, {NULL}};
const Segment SGI80859[] = {{&S48266[44], 19}, {NULL}};
const Segment SGI80860[] = {{&S48266[30], 6}, {NULL}};
const Segment SGI80861[] = {{&S48266[111], 3}, {NULL}};
const Segment SGI80862[] = {{&S48266[77], 5}, {NULL}};
const Segment SGI80863[] = {{&S48267[44], 11}, {NULL}};
const Segment SGI80864[] = {{&S48267[44], 23}, {NULL}};
const Segment SGI80865[] = {{&S48267[30], 6}, {NULL}};
const Segment SGI80866[] = {{&S48267[115], 3}, {NULL}};
const Segment SGI80867[] = {{&S48267[81], 5}, {NULL}};
const Segment SGI80868[] = {{&S54376[44], 13}, {NULL}};
const Segment SGI80869[] = {{&S54376[44], 30}, {NULL}};
const Segment SGI80870[] = {{&S54376[30], 6}, {NULL}};
const Segment SGI80871[] = {{&S54376[122], 3}, {NULL}};
const Segment SGI80872[] = {{&S54376[88], 5}, {NULL}};
const Segment SGI80873[] = {{&S48328[6], 4}, {NULL}};
const Segment SGI80874[] = {{&S48328[76], 5}, {NULL}};
const Segment SGI80875[] = {{&S54430[44], 15}, {NULL}};
const Segment SGI80876[] = {{&S54430[44], 36}, {NULL}};
const Segment SGI80877[] = {{&S54430[30], 6}, {NULL}};
const Segment SGI80878[] = {{&S54430[128], 3}, {NULL}};
const Segment SGI80879[] = {{&S54430[94], 5}, {NULL}};
const Segment SGI80880[] = {{&S54431[44], 15}, {NULL}};
const Segment SGI80881[] = {{&S54431[44], 36}, {NULL}};
const Segment SGI80882[] = {{&S54431[30], 6}, {NULL}};
const Segment SGI80883[] = {{&S54431[97], 4}, {NULL}};
const Segment SGI80884[] = {{&S54431[97], 3}, {NULL}};
const Segment SGI80885[] = {{&S54438[44], 15}, {NULL}};
const Segment SGI80886[] = {{&S54438[44], 36}, {NULL}};
const Segment SGI80887[] = {{&S54438[30], 6}, {NULL}};
const Segment SGI80888[] = {{&S54438[97], 6}, {NULL}};
const Segment SGI80889[] = {{&S54441[44], 7}, {NULL}};
const Segment SGI80890[] = {{&S54441[44], 24}, {NULL}};
const Segment SGI80891[] = {{&S54441[30], 6}, {NULL}};
const Segment SGI80892[] = {{&S54441[116], 3}, {NULL}};
const Segment SGI80893[] = {{&S54441[82], 5}, {NULL}};
const Segment SGI80894[] = {{&S54442[38], 15}, {NULL}};
const Segment SGI80895[] = {{&S54442[31], 6}, {NULL}};
const Segment SGI80896[] = {{&S54443[44], 36}, {NULL}};
const Segment SGI80897[] = {{&S54443[30], 6}, {NULL}};
const Segment SGI80898[] = {{&S48578[44], 7}, {NULL}};
const Segment SGI80899[] = {{&S48578[44], 18}, {NULL}};
const Segment SGI80900[] = {{&S48578[30], 6}, {NULL}};
const Segment SGI80901[] = {{&S48578[129], 3}, {NULL}};
const Segment SGI80902[] = {{&S48578[95], 5}, {NULL}};
const Segment SGI80903[] = {{&S48578[35], 1}, {NULL}};
const Segment SGI80904[] = {{&S48578[64], 11}, {NULL}};
const Segment SGI80905[] = {{&S48579[44], 7}, {NULL}};
const Segment SGI80906[] = {{&S48579[44], 37}, {NULL}};
const Segment SGI80907[] = {{&S48579[30], 6}, {NULL}};
const Segment SGI80908[] = {{&S48579[144], 3}, {NULL}};
const Segment SGI80909[] = {{&S48579[110], 5}, {NULL}};
const Segment SGI80910[] = {{&S48579[35], 1}, {NULL}};
const Segment SGI80911[] = {{&S48579[83], 11}, {NULL}};
const Segment SGI80912[] = {{&S48580[44], 7}, {NULL}};
const Segment SGI80913[] = {{&S48580[44], 37}, {NULL}};
const Segment SGI80914[] = {{&S48580[30], 6}, {NULL}};
const Segment SGI80915[] = {{&S48580[144], 3}, {NULL}};
const Segment SGI80916[] = {{&S48580[110], 5}, {NULL}};
const Segment SGI80917[] = {{&S48580[35], 1}, {NULL}};
const Segment SGI80918[] = {{&S48580[83], 11}, {NULL}};
const Segment SGI80919[] = {{&S41752[44], 10}, {NULL}};
const Segment SGI80920[] = {{&S41752[44], 27}, {NULL}};
const Segment SGI80921[] = {{&S41752[30], 6}, {NULL}};
const Segment SGI80922[] = {{&S41752[31], 3}, {NULL}};
const Segment SGI80923[] = {{&S41752[85], 6}, {NULL}};
const Segment SGI80924[] = {{&S41752[0], 13}, {NULL}};
const Segment SGI80925[] = {{&S35591[44], 9}, {NULL}};
const Segment SGI80926[] = {{&S35591[44], 20}, {NULL}};
const Segment SGI80927[] = {{&S35591[30], 6}, {NULL}};
const Segment SGI80928[] = {{&S35591[31], 3}, {NULL}};
const Segment SGI80929[] = {{&S35591[78], 6}, {NULL}};
const Segment SGI80930[] = {{&S35592[57], 9}, {NULL}};
const Segment SGI80931[] = {{&S35592[57], 20}, {NULL}};
const Segment SGI80932[] = {{&S35592[30], 19}, {NULL}};
const Segment SGI80933[] = {{&S35592[31], 3}, {NULL}};
const Segment SGI80934[] = {{&S35592[91], 6}, {NULL}};
const Segment SGI80935[] = {{&S35594[44], 9}, {NULL}};
const Segment SGI80936[] = {{&S35594[44], 21}, {NULL}};
const Segment SGI80937[] = {{&S35594[30], 6}, {NULL}};
const Segment SGI80938[] = {{&S35594[31], 3}, {NULL}};
const Segment SGI80939[] = {{&S35594[79], 6}, {NULL}};
const Segment SGI80940[] = {{&S35596[44], 5}, {NULL}};
const Segment SGI80941[] = {{&S35596[44], 17}, {NULL}};
const Segment SGI80942[] = {{&S35596[30], 6}, {NULL}};
const Segment SGI80943[] = {{&S35596[31], 3}, {NULL}};
const Segment SGI80944[] = {{&S35596[75], 6}, {NULL}};
const Segment SGI80945[] = {{&S35597[44], 9}, {NULL}};
const Segment SGI80946[] = {{&S35597[44], 20}, {NULL}};
const Segment SGI80947[] = {{&S35597[30], 6}, {NULL}};
const Segment SGI80948[] = {{&S35597[31], 3}, {NULL}};
const Segment SGI80949[] = {{&S35597[78], 6}, {NULL}};
const Segment SGI80950[] = {{&S35599[44], 12}, {NULL}};
const Segment SGI80951[] = {{&S35599[44], 23}, {NULL}};
const Segment SGI80952[] = {{&S35599[30], 6}, {NULL}};
const Segment SGI80953[] = {{&S35599[31], 3}, {NULL}};
const Segment SGI80954[] = {{&S35599[81], 6}, {NULL}};
const Segment SGI80955[] = {{&S35600[38], 5}, {NULL}};
const Segment SGI80956[] = {{&S35600[31], 6}, {NULL}};
const Segment SGI80957[] = {{&S54676[44], 11}, {NULL}};
const Segment SGI80958[] = {{&S54676[44], 28}, {NULL}};
const Segment SGI80959[] = {{&S54676[30], 6}, {NULL}};
const Segment SGI80960[] = {{&S54676[120], 3}, {NULL}};
const Segment SGI80961[] = {{&S54676[86], 5}, {NULL}};
const Segment SGI80962[] = {{&S54677[38], 11}, {NULL}};
const Segment SGI80963[] = {{&S54677[31], 6}, {NULL}};
const Segment SGI80964[] = {{&S35605[44], 18}, {NULL}};
const Segment SGI80965[] = {{&S35605[44], 30}, {NULL}};
const Segment SGI80966[] = {{&S35605[30], 6}, {NULL}};
const Segment SGI80967[] = {{&S35605[31], 3}, {NULL}};
const Segment SGI80968[] = {{&S35605[88], 6}, {NULL}};
const Segment SGI80969[] = {{&S35609[44], 6}, {NULL}};
const Segment SGI80970[] = {{&S35609[44], 18}, {NULL}};
const Segment SGI80971[] = {{&S35609[30], 6}, {NULL}};
const Segment SGI80972[] = {{&S35609[31], 3}, {NULL}};
const Segment SGI80973[] = {{&S35609[76], 6}, {NULL}};
const Segment SGI80974[] = {{&S35610[44], 10}, {NULL}};
const Segment SGI80975[] = {{&S35610[44], 24}, {NULL}};
const Segment SGI80976[] = {{&S35610[30], 6}, {NULL}};
const Segment SGI80977[] = {{&S35610[31], 3}, {NULL}};
const Segment SGI80978[] = {{&S35610[82], 6}, {NULL}};
const Segment SGI80979[] = {{&S35611[44], 5}, {NULL}};
const Segment SGI80980[] = {{&S35611[44], 23}, {NULL}};
const Segment SGI80981[] = {{&S35611[30], 6}, {NULL}};
const Segment SGI80982[] = {{&S35611[31], 3}, {NULL}};
const Segment SGI80983[] = {{&S35611[81], 6}, {NULL}};
const Segment SGI80984[] = {{&S47442[42], 6}, {NULL}};
const Segment SGI80985[] = {{&S47442[42], 17}, {NULL}};
const Segment SGI80986[] = {{&S47442[30], 4}, {NULL}};
const Segment SGI80987[] = {{&S47442[108], 3}, {NULL}};
const Segment SGI80988[] = {{&S47442[73], 6}, {NULL}};
const Segment SGI80989[] = {{&S47443[36], 6}, {NULL}};
const Segment SGI80990[] = {{&S47443[31], 4}, {NULL}};
const Segment SGI80991[] = {{&S47444[42], 6}, {NULL}};
const Segment SGI80992[] = {{&S47444[42], 17}, {NULL}};
const Segment SGI80993[] = {{&S47444[30], 4}, {NULL}};
const Segment SGI80994[] = {{&S47444[108], 3}, {NULL}};
const Segment SGI80995[] = {{&S47444[73], 6}, {NULL}};
const Segment SGI80996[] = {{&S47445[42], 22}, {NULL}};
const Segment SGI80997[] = {{&S47445[42], 33}, {NULL}};
const Segment SGI80998[] = {{&S47445[30], 4}, {NULL}};
const Segment SGI80999[] = {{&S47445[124], 3}, {NULL}};
const Segment SGI81000[] = {{&S47445[89], 6}, {NULL}};
const Segment SGI81001[] = {{&S54808[45], 10}, {NULL}};
const Segment SGI81002[] = {{&S54808[30], 6}, {NULL}};
const Segment SGI81003[] = {{&S54808[120], 3}, {NULL}};
const Segment SGI81004[] = {{&S54808[86], 5}, {NULL}};
const Segment SGI81005[] = {{&S54808[144], 13}, {NULL}};
const Segment SGI81006[] = {{&S54810[45], 10}, {NULL}};
const Segment SGI81007[] = {{&S54810[30], 6}, {NULL}};
const Segment SGI81008[] = {{&S54810[120], 3}, {NULL}};
const Segment SGI81009[] = {{&S54810[86], 5}, {NULL}};
const Segment SGI81010[] = {{&S54810[144], 13}, {NULL}};
const Segment SGI81011[] = {{&S54859[44], 23}, {NULL}};
const Segment SGI81012[] = {{&S54859[44], 35}, {NULL}};
const Segment SGI81013[] = {{&S54859[30], 6}, {NULL}};
const Segment SGI81014[] = {{&S54859[127], 3}, {NULL}};
const Segment SGI81015[] = {{&S54859[93], 5}, {NULL}};
const Segment SGI81016[] = {{&S54860[44], 24}, {NULL}};
const Segment SGI81017[] = {{&S54860[44], 36}, {NULL}};
const Segment SGI81018[] = {{&S54860[30], 6}, {NULL}};
const Segment SGI81019[] = {{&S54860[128], 3}, {NULL}};
const Segment SGI81020[] = {{&S54860[94], 5}, {NULL}};
const Segment SGI81021[] = {{&S54861[44], 23}, {NULL}};
const Segment SGI81022[] = {{&S54861[44], 35}, {NULL}};
const Segment SGI81023[] = {{&S54861[30], 6}, {NULL}};
const Segment SGI81024[] = {{&S54861[127], 3}, {NULL}};
const Segment SGI81025[] = {{&S54861[93], 5}, {NULL}};
const Segment SGI81026[] = {{&S54862[44], 32}, {NULL}};
const Segment SGI81027[] = {{&S54862[44], 44}, {NULL}};
const Segment SGI81028[] = {{&S54862[30], 6}, {NULL}};
const Segment SGI81029[] = {{&S54862[136], 3}, {NULL}};
const Segment SGI81030[] = {{&S54862[102], 5}, {NULL}};
const Segment SGI81031[] = {{&S54863[44], 24}, {NULL}};
const Segment SGI81032[] = {{&S54863[44], 36}, {NULL}};
const Segment SGI81033[] = {{&S54863[30], 6}, {NULL}};
const Segment SGI81034[] = {{&S54863[128], 3}, {NULL}};
const Segment SGI81035[] = {{&S54863[94], 5}, {NULL}};
const Segment SGI81036[] = {{&S35848[44], 8}, {NULL}};
const Segment SGI81037[] = {{&S35848[44], 22}, {NULL}};
const Segment SGI81038[] = {{&S35848[30], 6}, {NULL}};
const Segment SGI81039[] = {{&S35848[31], 3}, {NULL}};
const Segment SGI81040[] = {{&S35848[80], 6}, {NULL}};
const Segment SGI81041[] = {{&S35849[44], 8}, {NULL}};
const Segment SGI81042[] = {{&S35849[44], 22}, {NULL}};
const Segment SGI81043[] = {{&S35849[30], 6}, {NULL}};
const Segment SGI81044[] = {{&S35849[31], 3}, {NULL}};
const Segment SGI81045[] = {{&S35849[80], 6}, {NULL}};
const Segment SGI81046[] = {{&S35850[44], 8}, {NULL}};
const Segment SGI81047[] = {{&S35850[44], 21}, {NULL}};
const Segment SGI81048[] = {{&S35850[30], 6}, {NULL}};
const Segment SGI81049[] = {{&S35850[31], 3}, {NULL}};
const Segment SGI81050[] = {{&S35850[79], 6}, {NULL}};
const Segment SGI81051[] = {{&S35851[44], 8}, {NULL}};
const Segment SGI81052[] = {{&S35851[44], 21}, {NULL}};
const Segment SGI81053[] = {{&S35851[30], 6}, {NULL}};
const Segment SGI81054[] = {{&S35851[31], 3}, {NULL}};
const Segment SGI81055[] = {{&S35851[79], 6}, {NULL}};
const Segment SGI81056[] = {{&S35852[44], 8}, {NULL}};
const Segment SGI81057[] = {{&S35852[44], 21}, {NULL}};
const Segment SGI81058[] = {{&S35852[30], 6}, {NULL}};
const Segment SGI81059[] = {{&S35852[31], 3}, {NULL}};
const Segment SGI81060[] = {{&S35852[79], 6}, {NULL}};
const Segment SGI81061[] = {{&S35853[44], 8}, {NULL}};
const Segment SGI81062[] = {{&S35853[44], 21}, {NULL}};
const Segment SGI81063[] = {{&S35853[30], 6}, {NULL}};
const Segment SGI81064[] = {{&S35853[31], 3}, {NULL}};
const Segment SGI81065[] = {{&S35853[79], 6}, {NULL}};
const Segment SGI81066[] = {{&S35854[44], 8}, {NULL}};
const Segment SGI81067[] = {{&S35854[44], 21}, {NULL}};
const Segment SGI81068[] = {{&S35854[30], 6}, {NULL}};
const Segment SGI81069[] = {{&S35854[31], 3}, {NULL}};
const Segment SGI81070[] = {{&S35854[79], 6}, {NULL}};
const Segment SGI81071[] = {{&S35855[44], 8}, {NULL}};
const Segment SGI81072[] = {{&S35855[44], 22}, {NULL}};
const Segment SGI81073[] = {{&S35855[30], 6}, {NULL}};
const Segment SGI81074[] = {{&S35855[31], 3}, {NULL}};
const Segment SGI81075[] = {{&S35855[80], 6}, {NULL}};
const Segment SGI81076[] = {{&S35856[38], 8}, {NULL}};
const Segment SGI81077[] = {{&S35856[31], 6}, {NULL}};
const Segment SGI81078[] = {{&S35857[38], 8}, {NULL}};
const Segment SGI81079[] = {{&S35857[31], 6}, {NULL}};
const Segment SGI81080[] = {{&S35858[44], 8}, {NULL}};
const Segment SGI81081[] = {{&S35858[44], 21}, {NULL}};
const Segment SGI81082[] = {{&S35858[30], 6}, {NULL}};
const Segment SGI81083[] = {{&S35858[31], 3}, {NULL}};
const Segment SGI81084[] = {{&S35858[79], 6}, {NULL}};
const Segment SGI81085[] = {{&S48814[44], 8}, {NULL}};
const Segment SGI81086[] = {{&S48814[44], 24}, {NULL}};
const Segment SGI81087[] = {{&S48814[30], 6}, {NULL}};
const Segment SGI81088[] = {{&S48814[116], 3}, {NULL}};
const Segment SGI81089[] = {{&S48814[82], 5}, {NULL}};
const Segment SGI81090[] = {{&S48815[44], 20}, {NULL}};
const Segment SGI81091[] = {{&S48815[44], 36}, {NULL}};
const Segment SGI81092[] = {{&S48815[30], 6}, {NULL}};
const Segment SGI81093[] = {{&S48815[128], 3}, {NULL}};
const Segment SGI81094[] = {{&S48815[94], 5}, {NULL}};
const Segment SGI81095[] = {{&S48816[44], 20}, {NULL}};
const Segment SGI81096[] = {{&S48816[44], 35}, {NULL}};
const Segment SGI81097[] = {{&S48816[30], 6}, {NULL}};
const Segment SGI81098[] = {{&S48816[127], 3}, {NULL}};
const Segment SGI81099[] = {{&S48816[93], 5}, {NULL}};
const Segment SGI81100[] = {{&S48817[44], 8}, {NULL}};
const Segment SGI81101[] = {{&S48817[44], 23}, {NULL}};
const Segment SGI81102[] = {{&S48817[30], 6}, {NULL}};
const Segment SGI81103[] = {{&S48817[115], 3}, {NULL}};
const Segment SGI81104[] = {{&S48817[81], 5}, {NULL}};
const Segment SGI81105[] = {{&S48818[44], 20}, {NULL}};
const Segment SGI81106[] = {{&S48818[44], 36}, {NULL}};
const Segment SGI81107[] = {{&S48818[30], 6}, {NULL}};
const Segment SGI81108[] = {{&S48818[128], 3}, {NULL}};
const Segment SGI81109[] = {{&S48818[94], 5}, {NULL}};
const Segment SGI81110[] = {{&S48819[44], 8}, {NULL}};
const Segment SGI81111[] = {{&S48819[44], 24}, {NULL}};
const Segment SGI81112[] = {{&S48819[30], 6}, {NULL}};
const Segment SGI81113[] = {{&S35877[34], 10}, {NULL}};
const Segment SGI81114[] = {{&S48819[116], 3}, {NULL}};
const Segment SGI81115[] = {{&S35877[27], 6}, {NULL}};
const Segment SGI81116[] = {{&S48819[82], 5}, {NULL}};
const Segment SGI81117[] = {{&S35877[97], 9}, {NULL}};
const Segment SGI81118[] = {{&S35877[70], 6}, {NULL}};
const Segment SGI81119[] = {{&S48820[44], 20}, {NULL}};
const Segment SGI81120[] = {{&S48820[44], 36}, {NULL}};
const Segment SGI81121[] = {{&S48820[30], 6}, {NULL}};
const Segment SGI81122[] = {{&S48820[128], 3}, {NULL}};
const Segment SGI81123[] = {{&S48820[94], 5}, {NULL}};
const Segment SGI81124[] = {{&S48821[44], 8}, {NULL}};
const Segment SGI81125[] = {{&S48821[44], 24}, {NULL}};
const Segment SGI81126[] = {{&S48821[30], 6}, {NULL}};
const Segment SGI81127[] = {{&S48821[116], 3}, {NULL}};
const Segment SGI81128[] = {{&S48821[82], 5}, {NULL}};
const Segment SGI81129[] = {{&S48822[38], 8}, {NULL}};
const Segment SGI81130[] = {{&S48822[31], 6}, {NULL}};
const Segment SGI81131[] = {{&S35884[34], 10}, {NULL}};
const Segment SGI81132[] = {{&S35884[27], 6}, {NULL}};
const Segment SGI81133[] = {{&S35884[97], 9}, {NULL}};
const Segment SGI81134[] = {{&S35884[70], 6}, {NULL}};
const Segment SGI81135[] = {{&S35909[44], 7}, {NULL}};
const Segment SGI81136[] = {{&S35909[44], 19}, {NULL}};
const Segment SGI81137[] = {{&S35909[30], 6}, {NULL}};
const Segment SGI81138[] = {{&S35909[31], 3}, {NULL}};
const Segment SGI81139[] = {{&S35909[77], 6}, {NULL}};
const Segment SGI81140[] = {{&S35910[44], 7}, {NULL}};
const Segment SGI81141[] = {{&S35910[44], 24}, {NULL}};
const Segment SGI81142[] = {{&S35910[30], 6}, {NULL}};
const Segment SGI81143[] = {{&S35910[31], 3}, {NULL}};
const Segment SGI81144[] = {{&S35910[82], 6}, {NULL}};
const Segment SGI81145[] = {{&S35911[44], 6}, {NULL}};
const Segment SGI81146[] = {{&S35911[44], 26}, {NULL}};
const Segment SGI81147[] = {{&S35911[30], 6}, {NULL}};
const Segment SGI81148[] = {{&S35911[31], 3}, {NULL}};
const Segment SGI81149[] = {{&S35911[84], 6}, {NULL}};
const Segment SGI81150[] = {{&S35912[44], 7}, {NULL}};
const Segment SGI81151[] = {{&S35912[44], 24}, {NULL}};
const Segment SGI81152[] = {{&S35912[30], 6}, {NULL}};
const Segment SGI81153[] = {{&S35912[85], 6}, {NULL}};
const Segment SGI81154[] = {{&S35913[38], 7}, {NULL}};
const Segment SGI81155[] = {{&S35913[31], 6}, {NULL}};
const Segment SGI81156[] = {{&S35914[44], 7}, {NULL}};
const Segment SGI81157[] = {{&S35914[44], 31}, {NULL}};
const Segment SGI81158[] = {{&S35914[30], 6}, {NULL}};
const Segment SGI81159[] = {{&S35914[31], 3}, {NULL}};
const Segment SGI81160[] = {{&S35914[89], 6}, {NULL}};
const Segment SGI81161[] = {{&S35915[44], 7}, {NULL}};
const Segment SGI81162[] = {{&S35915[44], 31}, {NULL}};
const Segment SGI81163[] = {{&S35915[30], 6}, {NULL}};
const Segment SGI81164[] = {{&S35915[31], 3}, {NULL}};
const Segment SGI81165[] = {{&S35915[89], 6}, {NULL}};
const Segment SGI81166[] = {{&S35917[44], 8}, {NULL}};
const Segment SGI81167[] = {{&S35917[44], 26}, {NULL}};
const Segment SGI81168[] = {{&S35917[30], 6}, {NULL}};
const Segment SGI81169[] = {{&S35917[31], 3}, {NULL}};
const Segment SGI81170[] = {{&S35917[84], 6}, {NULL}};
const Segment SGI81171[] = {{&S35918[44], 7}, {NULL}};
const Segment SGI81172[] = {{&S35918[44], 31}, {NULL}};
const Segment SGI81173[] = {{&S35918[30], 6}, {NULL}};
const Segment SGI81174[] = {{&S35918[31], 3}, {NULL}};
const Segment SGI81175[] = {{&S35918[89], 6}, {NULL}};
const Segment SGI81176[] = {{&S35919[44], 7}, {NULL}};
const Segment SGI81177[] = {{&S35919[44], 31}, {NULL}};
const Segment SGI81178[] = {{&S35919[30], 6}, {NULL}};
const Segment SGI81179[] = {{&S35919[31], 3}, {NULL}};
const Segment SGI81180[] = {{&S35919[89], 6}, {NULL}};
const Segment SGI81181[] = {{&S35920[44], 7}, {NULL}};
const Segment SGI81182[] = {{&S35920[44], 34}, {NULL}};
const Segment SGI81183[] = {{&S35920[30], 6}, {NULL}};
const Segment SGI81184[] = {{&S35920[31], 3}, {NULL}};
const Segment SGI81185[] = {{&S35920[92], 6}, {NULL}};
const Segment SGI81186[] = {{&S35922[38], 8}, {NULL}};
const Segment SGI81187[] = {{&S35922[31], 6}, {NULL}};
const Segment SGI81188[] = {{&S35923[44], 8}, {NULL}};
const Segment SGI81189[] = {{&S35923[44], 26}, {NULL}};
const Segment SGI81190[] = {{&S35923[30], 6}, {NULL}};
const Segment SGI81191[] = {{&S35923[84], 6}, {NULL}};
const Segment SGI81192[] = {{&S55145[44], 10}, {NULL}};
const Segment SGI81193[] = {{&S55145[44], 27}, {NULL}};
const Segment SGI81194[] = {{&S55145[30], 6}, {NULL}};
const Segment SGI81195[] = {{&S55145[119], 3}, {NULL}};
const Segment SGI81196[] = {{&S55145[85], 5}, {NULL}};
const Segment SGI81197[] = {{&S55146[38], 10}, {NULL}};
const Segment SGI81198[] = {{&S55146[31], 6}, {NULL}};
const Segment SGI81199[] = {{&S36025[44], 10}, {NULL}};
const Segment SGI81200[] = {{&S36025[44], 22}, {NULL}};
const Segment SGI81201[] = {{&S36025[30], 6}, {NULL}};
const Segment SGI81202[] = {{&S36025[31], 3}, {NULL}};
const Segment SGI81203[] = {{&S36025[80], 6}, {NULL}};
const Segment SGI81204[] = {{&S36026[44], 13}, {NULL}};
const Segment SGI81205[] = {{&S36026[44], 25}, {NULL}};
const Segment SGI81206[] = {{&S36026[30], 6}, {NULL}};
const Segment SGI81207[] = {{&S36026[31], 3}, {NULL}};
const Segment SGI81208[] = {{&S36026[83], 6}, {NULL}};
const Segment SGI81209[] = {{&S36035[38], 10}, {NULL}};
const Segment SGI81210[] = {{&S36035[31], 6}, {NULL}};
const Segment SGI81211[] = {{&S49055[44], 10}, {NULL}};
const Segment SGI81212[] = {{&S49055[44], 25}, {NULL}};
const Segment SGI81213[] = {{&S49055[30], 6}, {NULL}};
const Segment SGI81214[] = {{&S49055[117], 3}, {NULL}};
const Segment SGI81215[] = {{&S49055[83], 5}, {NULL}};
const Segment SGI81216[] = {{&S49056[44], 10}, {NULL}};
const Segment SGI81217[] = {{&S49056[44], 25}, {NULL}};
const Segment SGI81218[] = {{&S49056[30], 6}, {NULL}};
const Segment SGI81219[] = {{&S49056[117], 3}, {NULL}};
const Segment SGI81220[] = {{&S49056[83], 5}, {NULL}};
const Segment SGI81221[] = {{&S49057[38], 10}, {NULL}};
const Segment SGI81222[] = {{&S49057[31], 6}, {NULL}};
const Segment SGI81223[] = {{&S55256[45], 10}, {NULL}};
const Segment SGI81224[] = {{&S55256[30], 6}, {NULL}};
const Segment SGI81225[] = {{&S55256[120], 3}, {NULL}};
const Segment SGI81226[] = {{&S55256[86], 5}, {NULL}};
const Segment SGI81227[] = {{&S55256[144], 13}, {NULL}};
const Segment SGI81228[] = {{&S55258[45], 10}, {NULL}};
const Segment SGI81229[] = {{&S55258[30], 6}, {NULL}};
const Segment SGI81230[] = {{&S55258[120], 3}, {NULL}};
const Segment SGI81231[] = {{&S55258[86], 5}, {NULL}};
const Segment SGI81232[] = {{&S55258[144], 13}, {NULL}};
const Segment SGI81233[] = {{&S55259[45], 11}, {NULL}};
const Segment SGI81234[] = {{&S55259[30], 6}, {NULL}};
const Segment SGI81235[] = {{&S55259[121], 3}, {NULL}};
const Segment SGI81236[] = {{&S55259[87], 5}, {NULL}};
const Segment SGI81237[] = {{&S55259[145], 13}, {NULL}};
const Segment SGI81238[] = {{&S55260[45], 11}, {NULL}};
const Segment SGI81239[] = {{&S55260[30], 6}, {NULL}};
const Segment SGI81240[] = {{&S55260[121], 3}, {NULL}};
const Segment SGI81241[] = {{&S55260[87], 5}, {NULL}};
const Segment SGI81242[] = {{&S55260[145], 13}, {NULL}};
const Segment SGI81243[] = {{&S55261[44], 10}, {NULL}};
const Segment SGI81244[] = {{&S55261[44], 27}, {NULL}};
const Segment SGI81245[] = {{&S55261[30], 6}, {NULL}};
const Segment SGI81246[] = {{&S55261[120], 3}, {NULL}};
const Segment SGI81247[] = {{&S55261[85], 6}, {NULL}};
const Segment SGI81248[] = {{&S55261[140], 8}, {NULL}};
const Segment SGI81249[] = {{&S55261[149], 13}, {NULL}};
const Segment SGI81250[] = {{&S49202[44], 9}, {NULL}};
const Segment SGI81251[] = {{&S49202[44], 19}, {NULL}};
const Segment SGI81252[] = {{&S49202[30], 6}, {NULL}};
const Segment SGI81253[] = {{&S49202[111], 3}, {NULL}};
const Segment SGI81254[] = {{&S49202[77], 5}, {NULL}};
const Segment SGI81255[] = {{&S49204[44], 5}, {NULL}};
const Segment SGI81256[] = {{&S49204[44], 15}, {NULL}};
const Segment SGI81257[] = {{&S49204[30], 6}, {NULL}};
const Segment SGI81258[] = {{&S49204[107], 3}, {NULL}};
const Segment SGI81259[] = {{&S49204[73], 5}, {NULL}};
const Segment SGI81260[] = {{&S49206[44], 9}, {NULL}};
const Segment SGI81261[] = {{&S49206[44], 19}, {NULL}};
const Segment SGI81262[] = {{&S49206[30], 6}, {NULL}};
const Segment SGI81263[] = {{&S49206[80], 4}, {NULL}};
const Segment SGI81264[] = {{&S49206[80], 3}, {NULL}};
const Segment SGI81265[] = {{&S49208[38], 5}, {NULL}};
const Segment SGI81266[] = {{&S49208[31], 6}, {NULL}};
const Segment SGI81267[] = {{&S49209[44], 9}, {NULL}};
const Segment SGI81268[] = {{&S49209[44], 19}, {NULL}};
const Segment SGI81269[] = {{&S49209[30], 6}, {NULL}};
const Segment SGI81270[] = {{&S49209[80], 6}, {NULL}};
const Segment SGI81271[] = {{&S49212[38], 9}, {NULL}};
const Segment SGI81272[] = {{&S49212[31], 6}, {NULL}};
const Segment SGI81273[] = {{&S49213[44], 9}, {NULL}};
const Segment SGI81274[] = {{&S49213[44], 22}, {NULL}};
const Segment SGI81275[] = {{&S49213[30], 6}, {NULL}};
const Segment SGI81276[] = {{&S49213[114], 3}, {NULL}};
const Segment SGI81277[] = {{&S49213[80], 5}, {NULL}};
const Segment SGI81278[] = {{&S55497[44], 9}, {NULL}};
const Segment SGI81279[] = {{&S55497[44], 26}, {NULL}};
const Segment SGI81280[] = {{&S55497[30], 6}, {NULL}};
const Segment SGI81281[] = {{&S55497[118], 3}, {NULL}};
const Segment SGI81282[] = {{&S55497[84], 5}, {NULL}};
const Segment SGI81283[] = {{&S55498[44], 9}, {NULL}};
const Segment SGI81284[] = {{&S55498[44], 26}, {NULL}};
const Segment SGI81285[] = {{&S55498[30], 6}, {NULL}};
const Segment SGI81286[] = {{&S55498[118], 3}, {NULL}};
const Segment SGI81287[] = {{&S55498[84], 5}, {NULL}};
const Segment SGI81288[] = {{&S55499[44], 10}, {NULL}};
const Segment SGI81289[] = {{&S55499[44], 30}, {NULL}};
const Segment SGI81290[] = {{&S55499[30], 6}, {NULL}};
const Segment SGI81291[] = {{&S55499[122], 3}, {NULL}};
const Segment SGI81292[] = {{&S55499[88], 5}, {NULL}};
const Segment SGI81293[] = {{&S55500[44], 10}, {NULL}};
const Segment SGI81294[] = {{&S55500[44], 27}, {NULL}};
const Segment SGI81295[] = {{&S55500[30], 6}, {NULL}};
const Segment SGI81296[] = {{&S55500[119], 3}, {NULL}};
const Segment SGI81297[] = {{&S55500[85], 5}, {NULL}};
const Segment SGI81298[] = {{&S55641[44], 14}, {NULL}};
const Segment SGI81299[] = {{&S55641[44], 26}, {NULL}};
const Segment SGI81300[] = {{&S55641[30], 6}, {NULL}};
const Segment SGI81301[] = {{&S55641[118], 3}, {NULL}};
const Segment SGI81302[] = {{&S55641[84], 5}, {NULL}};
const Segment SGI81303[] = {{&S55642[38], 14}, {NULL}};
const Segment SGI81304[] = {{&S55642[31], 6}, {NULL}};
const Segment SGI81305[] = {{&S55750[44], 41}, {NULL}};
const Segment SGI81306[] = {{&S55750[44], 58}, {NULL}};
const Segment SGI81307[] = {{&S55750[30], 6}, {NULL}};
const Segment SGI81308[] = {{&S55750[150], 3}, {NULL}};
const Segment SGI81309[] = {{&S55750[116], 5}, {NULL}};
const Segment SGI81310[] = {{&S55792[44], 10}, {NULL}};
const Segment SGI81311[] = {{&S55792[44], 30}, {NULL}};
const Segment SGI81312[] = {{&S55792[30], 6}, {NULL}};
const Segment SGI81313[] = {{&S55792[122], 3}, {NULL}};
const Segment SGI81314[] = {{&S55792[88], 5}, {NULL}};
const Segment SGI81315[] = {{&S48376[42], 9}, {NULL}};
const Segment SGI81316[] = {{&S48376[42], 20}, {NULL}};
const Segment SGI81317[] = {{&S48376[30], 4}, {NULL}};
const Segment SGI81318[] = {{&S48376[111], 3}, {NULL}};
const Segment SGI81319[] = {{&S48376[76], 6}, {NULL}};
const Segment SGI81320[] = {{&S48377[36], 9}, {NULL}};
const Segment SGI81321[] = {{&S48377[31], 4}, {NULL}};
const Segment SGI81322[] = {{&S55986[44], 11}, {NULL}};
const Segment SGI81323[] = {{&S55986[44], 28}, {NULL}};
const Segment SGI81324[] = {{&S55986[30], 6}, {NULL}};
const Segment SGI81325[] = {{&S55986[120], 3}, {NULL}};
const Segment SGI81326[] = {{&S55986[86], 5}, {NULL}};
const Segment SGI81327[] = {{&S55987[38], 11}, {NULL}};
const Segment SGI81328[] = {{&S55987[31], 6}, {NULL}};
const Segment SGI81329[] = {{&S55988[42], 11}, {NULL}};
const Segment SGI81330[] = {{&S55988[30], 6}, {NULL}};
const Segment SGI81331[] = {{&S55988[118], 3}, {NULL}};
const Segment SGI81332[] = {{&S55988[84], 5}, {NULL}};
const Segment SGI81333[] = {{&S56095[44], 7}, {NULL}};
const Segment SGI81334[] = {{&S56095[44], 24}, {NULL}};
const Segment SGI81335[] = {{&S56095[30], 6}, {NULL}};
const Segment SGI81336[] = {{&S56095[116], 3}, {NULL}};
const Segment SGI81337[] = {{&S56095[82], 5}, {NULL}};
const Segment SGI81338[] = {{&S56096[38], 7}, {NULL}};
const Segment SGI81339[] = {{&S56096[31], 6}, {NULL}};
const Segment SGI81340[] = {{&S56125[44], 10}, {NULL}};
const Segment SGI81341[] = {{&S56125[44], 27}, {NULL}};
const Segment SGI81342[] = {{&S56125[30], 6}, {NULL}};
const Segment SGI81343[] = {{&S56125[85], 6}, {NULL}};
const Segment SGI81344[] = {{&S56125[112], 13}, {NULL}};
const Segment SGI81345[] = {{&S56126[44], 11}, {NULL}};
const Segment SGI81346[] = {{&S56126[44], 28}, {NULL}};
const Segment SGI81347[] = {{&S56126[30], 6}, {NULL}};
const Segment SGI81348[] = {{&S56126[86], 6}, {NULL}};
const Segment SGI81349[] = {{&S56126[113], 13}, {NULL}};
const Segment SGI81350[] = {{&S56127[44], 10}, {NULL}};
const Segment SGI81351[] = {{&S56127[44], 27}, {NULL}};
const Segment SGI81352[] = {{&S56127[30], 6}, {NULL}};
const Segment SGI81353[] = {{&S56127[85], 6}, {NULL}};
const Segment SGI81354[] = {{&S56127[112], 13}, {NULL}};
const Segment SGI81355[] = {{&S36736[44], 7}, {NULL}};
const Segment SGI81356[] = {{&S36736[44], 21}, {NULL}};
const Segment SGI81357[] = {{&S36736[30], 6}, {NULL}};
const Segment SGI81358[] = {{&S36736[31], 3}, {NULL}};
const Segment SGI81359[] = {{&S36736[79], 6}, {NULL}};
const Segment SGI81360[] = {{&S36738[44], 7}, {NULL}};
const Segment SGI81361[] = {{&S36738[44], 19}, {NULL}};
const Segment SGI81362[] = {{&S36738[30], 6}, {NULL}};
const Segment SGI81363[] = {{&S36738[31], 3}, {NULL}};
const Segment SGI81364[] = {{&S36738[77], 6}, {NULL}};
const Segment SGI81365[] = {{&S36741[44], 7}, {NULL}};
const Segment SGI81366[] = {{&S36741[44], 21}, {NULL}};
const Segment SGI81367[] = {{&S36741[30], 6}, {NULL}};
const Segment SGI81368[] = {{&S36741[31], 3}, {NULL}};
const Segment SGI81369[] = {{&S36741[79], 6}, {NULL}};
const Segment SGI81370[] = {{&S36742[44], 7}, {NULL}};
const Segment SGI81371[] = {{&S36742[44], 21}, {NULL}};
const Segment SGI81372[] = {{&S36742[30], 6}, {NULL}};
const Segment SGI81373[] = {{&S36742[31], 3}, {NULL}};
const Segment SGI81374[] = {{&S36742[79], 6}, {NULL}};
const Segment SGI81375[] = {{&S36743[44], 7}, {NULL}};
const Segment SGI81376[] = {{&S36743[44], 21}, {NULL}};
const Segment SGI81377[] = {{&S36743[30], 6}, {NULL}};
const Segment SGI81378[] = {{&S36743[31], 3}, {NULL}};
const Segment SGI81379[] = {{&S36743[79], 6}, {NULL}};
const Segment SGI81380[] = {{&S49859[44], 12}, {NULL}};
const Segment SGI81381[] = {{&S49859[44], 22}, {NULL}};
const Segment SGI81382[] = {{&S49859[30], 6}, {NULL}};
const Segment SGI81383[] = {{&S49859[114], 3}, {NULL}};
const Segment SGI81384[] = {{&S49859[80], 5}, {NULL}};
const Segment SGI81385[] = {{&S49861[44], 12}, {NULL}};
const Segment SGI81386[] = {{&S49861[44], 25}, {NULL}};
const Segment SGI81387[] = {{&S49861[30], 6}, {NULL}};
const Segment SGI81388[] = {{&S49861[117], 3}, {NULL}};
const Segment SGI81389[] = {{&S49861[83], 5}, {NULL}};
const Segment SGI81390[] = {{&S49866[38], 12}, {NULL}};
const Segment SGI81391[] = {{&S49866[31], 6}, {NULL}};
const Segment SGI81392[] = {{&S49867[44], 12}, {NULL}};
const Segment SGI81393[] = {{&S49867[44], 22}, {NULL}};
const Segment SGI81394[] = {{&S49867[30], 6}, {NULL}};
const Segment SGI81395[] = {{&S49867[83], 4}, {NULL}};
const Segment SGI81396[] = {{&S49867[83], 3}, {NULL}};
const Segment SGI81397[] = {{&S49870[44], 17}, {NULL}};
const Segment SGI81398[] = {{&S49870[44], 27}, {NULL}};
const Segment SGI81399[] = {{&S49870[30], 6}, {NULL}};
const Segment SGI81400[] = {{&S49870[119], 3}, {NULL}};
const Segment SGI81401[] = {{&S49870[85], 5}, {NULL}};
const Segment SGI81402[] = {{&S56194[45], 10}, {NULL}};
const Segment SGI81403[] = {{&S56194[30], 6}, {NULL}};
const Segment SGI81404[] = {{&S56194[120], 3}, {NULL}};
const Segment SGI81405[] = {{&S49871[44], 12}, {NULL}};
const Segment SGI81406[] = {{&S49871[44], 22}, {NULL}};
const Segment SGI81407[] = {{&S56194[86], 5}, {NULL}};
const Segment SGI81408[] = {{&S49871[30], 6}, {NULL}};
const Segment SGI81409[] = {{&S56194[144], 13}, {NULL}};
const Segment SGI81410[] = {{&S49871[83], 6}, {NULL}};
const Segment SGI81411[] = {{&S49874[44], 12}, {NULL}};
const Segment SGI81412[] = {{&S49874[44], 25}, {NULL}};
const Segment SGI81413[] = {{&S49874[30], 6}, {NULL}};
const Segment SGI81414[] = {{&S49874[117], 3}, {NULL}};
const Segment SGI81415[] = {{&S49874[83], 5}, {NULL}};
const Segment SGI81416[] = {{&S43213[44], 9}, {NULL}};
const Segment SGI81417[] = {{&S43213[44], 21}, {NULL}};
const Segment SGI81418[] = {{&S43213[30], 6}, {NULL}};
const Segment SGI81419[] = {{&S43213[31], 3}, {NULL}};
const Segment SGI81420[] = {{&S43213[79], 6}, {NULL}};
const Segment SGI81421[] = {{&S43213[0], 13}, {NULL}};
const Segment SGI81422[] = {{&S43214[44], 9}, {NULL}};
const Segment SGI81423[] = {{&S43214[44], 20}, {NULL}};
const Segment SGI81424[] = {{&S43214[30], 6}, {NULL}};
const Segment SGI81425[] = {{&S43214[31], 3}, {NULL}};
const Segment SGI81426[] = {{&S43214[78], 6}, {NULL}};
const Segment SGI81427[] = {{&S43214[0], 13}, {NULL}};
const Segment SGI81428[] = {{&S43233[44], 9}, {NULL}};
const Segment SGI81429[] = {{&S43233[44], 22}, {NULL}};
const Segment SGI81430[] = {{&S43233[30], 6}, {NULL}};
const Segment SGI81431[] = {{&S43233[31], 3}, {NULL}};
const Segment SGI81432[] = {{&S43233[80], 6}, {NULL}};
const Segment SGI81433[] = {{&S43233[0], 13}, {NULL}};
const Segment SGI81434[] = {{&S48690[44], 7}, {NULL}};
const Segment SGI81435[] = {{&S48690[44], 21}, {NULL}};
const Segment SGI81436[] = {{&S48690[30], 6}, {NULL}};
const Segment SGI81437[] = {{&S48690[114], 3}, {NULL}};
const Segment SGI81438[] = {{&S48690[79], 6}, {NULL}};
const Segment SGI81439[] = {{&S48691[44], 7}, {NULL}};
const Segment SGI81440[] = {{&S48691[44], 23}, {NULL}};
const Segment SGI81441[] = {{&S48691[30], 6}, {NULL}};
const Segment SGI81442[] = {{&S48691[116], 3}, {NULL}};
const Segment SGI81443[] = {{&S48691[81], 6}, {NULL}};
const Segment SGI81444[] = {{&S48692[44], 7}, {NULL}};
const Segment SGI81445[] = {{&S48692[44], 21}, {NULL}};
const Segment SGI81446[] = {{&S48692[30], 6}, {NULL}};
const Segment SGI81447[] = {{&S48692[114], 3}, {NULL}};
const Segment SGI81448[] = {{&S48692[79], 6}, {NULL}};
const Segment SGI81449[] = {{&S48693[44], 7}, {NULL}};
const Segment SGI81450[] = {{&S48693[44], 24}, {NULL}};
const Segment SGI81451[] = {{&S48693[30], 6}, {NULL}};
const Segment SGI81452[] = {{&S48693[117], 3}, {NULL}};
const Segment SGI81453[] = {{&S48693[82], 6}, {NULL}};
const Segment SGI81454[] = {{&S48694[44], 7}, {NULL}};
const Segment SGI81455[] = {{&S48694[44], 21}, {NULL}};
const Segment SGI81456[] = {{&S48694[30], 6}, {NULL}};
const Segment SGI81457[] = {{&S48694[114], 3}, {NULL}};
const Segment SGI81458[] = {{&S48694[79], 6}, {NULL}};
const Segment SGI81459[] = {{&S48695[44], 7}, {NULL}};
const Segment SGI81460[] = {{&S48695[44], 23}, {NULL}};
const Segment SGI81461[] = {{&S48695[30], 6}, {NULL}};
const Segment SGI81462[] = {{&S48695[116], 3}, {NULL}};
const Segment SGI81463[] = {{&S48695[81], 6}, {NULL}};
const Segment SGI81464[] = {{&S48696[44], 7}, {NULL}};
const Segment SGI81465[] = {{&S48696[44], 21}, {NULL}};
const Segment SGI81466[] = {{&S48696[30], 6}, {NULL}};
const Segment SGI81467[] = {{&S48696[114], 3}, {NULL}};
const Segment SGI81468[] = {{&S48696[79], 6}, {NULL}};
const Segment SGI81469[] = {{&S48697[44], 7}, {NULL}};
const Segment SGI81470[] = {{&S48697[44], 23}, {NULL}};
const Segment SGI81471[] = {{&S48697[30], 6}, {NULL}};
const Segment SGI81472[] = {{&S48697[116], 3}, {NULL}};
const Segment SGI81473[] = {{&S48697[81], 6}, {NULL}};
const Segment SGI81474[] = {{&S48698[44], 7}, {NULL}};
const Segment SGI81475[] = {{&S48698[44], 23}, {NULL}};
const Segment SGI81476[] = {{&S48698[30], 6}, {NULL}};
const Segment SGI81477[] = {{&S48698[116], 3}, {NULL}};
const Segment SGI81478[] = {{&S48698[81], 6}, {NULL}};
const Segment SGI81479[] = {{&S48699[44], 7}, {NULL}};
const Segment SGI81480[] = {{&S48699[44], 21}, {NULL}};
const Segment SGI81481[] = {{&S48699[30], 6}, {NULL}};
const Segment SGI81482[] = {{&S48699[114], 3}, {NULL}};
const Segment SGI81483[] = {{&S48699[79], 6}, {NULL}};
const Segment SGI81484[] = {{&S48700[44], 7}, {NULL}};
const Segment SGI81485[] = {{&S48700[44], 24}, {NULL}};
const Segment SGI81486[] = {{&S48700[30], 6}, {NULL}};
const Segment SGI81487[] = {{&S48700[117], 3}, {NULL}};
const Segment SGI81488[] = {{&S48700[82], 6}, {NULL}};
const Segment SGI81489[] = {{&S48701[42], 7}, {NULL}};
const Segment SGI81490[] = {{&S48701[42], 21}, {NULL}};
const Segment SGI81491[] = {{&S48701[30], 4}, {NULL}};
const Segment SGI81492[] = {{&S48701[112], 3}, {NULL}};
const Segment SGI81493[] = {{&S48701[77], 6}, {NULL}};
const Segment SGI81494[] = {{&S48702[44], 7}, {NULL}};
const Segment SGI81495[] = {{&S48702[44], 23}, {NULL}};
const Segment SGI81496[] = {{&S48702[30], 6}, {NULL}};
const Segment SGI81497[] = {{&S48702[116], 3}, {NULL}};
const Segment SGI81498[] = {{&S48702[81], 6}, {NULL}};
const Segment SGI81499[] = {{&S48703[38], 7}, {NULL}};
const Segment SGI81500[] = {{&S48703[31], 6}, {NULL}};
const Segment SGI81501[] = {{&S37063[44], 6}, {NULL}};
const Segment SGI81502[] = {{&S37063[44], 18}, {NULL}};
const Segment SGI81503[] = {{&S37063[30], 6}, {NULL}};
const Segment SGI81504[] = {{&S37063[31], 3}, {NULL}};
const Segment SGI81505[] = {{&S37063[76], 6}, {NULL}};
const Segment SGI81506[] = {{&S37064[38], 6}, {NULL}};
const Segment SGI81507[] = {{&S37064[31], 6}, {NULL}};
const Segment SGI81508[] = {{&S37067[44], 6}, {NULL}};
const Segment SGI81509[] = {{&S37067[44], 18}, {NULL}};
const Segment SGI81510[] = {{&S37067[30], 6}, {NULL}};
const Segment SGI81511[] = {{&S37067[31], 3}, {NULL}};
const Segment SGI81512[] = {{&S37067[76], 6}, {NULL}};
const Segment SGI81513[] = {{&S37068[42], 6}, {NULL}};
const Segment SGI81514[] = {{&S37068[30], 6}, {NULL}};
const Segment SGI81515[] = {{&S37068[31], 3}, {NULL}};
const Segment SGI81516[] = {{&S37068[74], 6}, {NULL}};
const Segment SGI81517[] = {{&S48866[42], 6}, {NULL}};
const Segment SGI81518[] = {{&S48866[42], 18}, {NULL}};
const Segment SGI81519[] = {{&S48866[30], 4}, {NULL}};
const Segment SGI81520[] = {{&S48866[109], 3}, {NULL}};
const Segment SGI81521[] = {{&S48866[74], 6}, {NULL}};
const Segment SGI81522[] = {{&S48867[44], 6}, {NULL}};
const Segment SGI81523[] = {{&S48867[44], 18}, {NULL}};
const Segment SGI81524[] = {{&S48867[30], 6}, {NULL}};
const Segment SGI81525[] = {{&S48867[111], 3}, {NULL}};
const Segment SGI81526[] = {{&S48867[76], 6}, {NULL}};
const Segment SGI81527[] = {{&S48868[38], 6}, {NULL}};
const Segment SGI81528[] = {{&S48868[31], 6}, {NULL}};
const Segment SGI81529[] = {{&S48869[36], 6}, {NULL}};
const Segment SGI81530[] = {{&S48869[31], 4}, {NULL}};
const Segment SGI81531[] = {{&S37149[44], 17}, {NULL}};
const Segment SGI81532[] = {{&S37149[44], 37}, {NULL}};
const Segment SGI81533[] = {{&S37149[30], 6}, {NULL}};
const Segment SGI81534[] = {{&S37149[31], 3}, {NULL}};
const Segment SGI81535[] = {{&S37149[95], 6}, {NULL}};
const Segment SGI81536[] = {{&S37150[44], 13}, {NULL}};
const Segment SGI81537[] = {{&S37150[44], 33}, {NULL}};
const Segment SGI81538[] = {{&S37150[30], 6}, {NULL}};
const Segment SGI81539[] = {{&S37150[31], 3}, {NULL}};
const Segment SGI81540[] = {{&S37150[91], 6}, {NULL}};
const Segment SGI81541[] = {{&S37151[38], 17}, {NULL}};
const Segment SGI81542[] = {{&S37151[31], 6}, {NULL}};
const Segment SGI81543[] = {{&S37152[44], 17}, {NULL}};
const Segment SGI81544[] = {{&S37152[44], 37}, {NULL}};
const Segment SGI81545[] = {{&S37152[30], 6}, {NULL}};
const Segment SGI81546[] = {{&S37152[31], 3}, {NULL}};
const Segment SGI81547[] = {{&S37152[95], 6}, {NULL}};
const Segment SGI81548[] = {{&S37153[38], 17}, {NULL}};
const Segment SGI81549[] = {{&S37153[31], 6}, {NULL}};
const Segment SGI81550[] = {{&S37155[44], 25}, {NULL}};
const Segment SGI81551[] = {{&S37155[44], 45}, {NULL}};
const Segment SGI81552[] = {{&S37155[30], 6}, {NULL}};
const Segment SGI81553[] = {{&S37155[31], 3}, {NULL}};
const Segment SGI81554[] = {{&S37155[103], 6}, {NULL}};
const Segment SGI81555[] = {{&S37157[38], 13}, {NULL}};
const Segment SGI81556[] = {{&S37157[31], 6}, {NULL}};
const Segment SGI81557[] = {{&S48940[44], 8}, {NULL}};
const Segment SGI81558[] = {{&S48940[44], 29}, {NULL}};
const Segment SGI81559[] = {{&S48940[30], 6}, {NULL}};
const Segment SGI81560[] = {{&S48940[121], 3}, {NULL}};
const Segment SGI81561[] = {{&S48940[87], 5}, {NULL}};
const Segment SGI81562[] = {{&S48941[44], 8}, {NULL}};
const Segment SGI81563[] = {{&S48941[44], 31}, {NULL}};
const Segment SGI81564[] = {{&S48941[30], 6}, {NULL}};
const Segment SGI81565[] = {{&S48941[123], 3}, {NULL}};
const Segment SGI81566[] = {{&S48941[89], 5}, {NULL}};
const Segment SGI81567[] = {{&S48942[44], 8}, {NULL}};
const Segment SGI81568[] = {{&S48942[44], 31}, {NULL}};
const Segment SGI81569[] = {{&S48942[30], 6}, {NULL}};
const Segment SGI81570[] = {{&S48942[123], 3}, {NULL}};
const Segment SGI81571[] = {{&S48942[89], 5}, {NULL}};
const Segment SGI81572[] = {{&S48943[44], 8}, {NULL}};
const Segment SGI81573[] = {{&S48943[44], 31}, {NULL}};
const Segment SGI81574[] = {{&S48943[30], 6}, {NULL}};
const Segment SGI81575[] = {{&S48943[123], 3}, {NULL}};
const Segment SGI81576[] = {{&S48943[89], 5}, {NULL}};
const Segment SGI81577[] = {{&S48944[44], 8}, {NULL}};
const Segment SGI81578[] = {{&S48944[44], 29}, {NULL}};
const Segment SGI81579[] = {{&S48944[30], 6}, {NULL}};
const Segment SGI81580[] = {{&S48944[121], 3}, {NULL}};
const Segment SGI81581[] = {{&S48944[87], 5}, {NULL}};
const Segment SGI81582[] = {{&S48945[44], 8}, {NULL}};
const Segment SGI81583[] = {{&S48945[44], 31}, {NULL}};
const Segment SGI81584[] = {{&S48945[30], 6}, {NULL}};
const Segment SGI81585[] = {{&S48945[123], 3}, {NULL}};
const Segment SGI81586[] = {{&S48945[89], 5}, {NULL}};
const Segment SGI81587[] = {{&S48946[44], 8}, {NULL}};
const Segment SGI81588[] = {{&S48946[44], 31}, {NULL}};
const Segment SGI81589[] = {{&S48946[30], 6}, {NULL}};
const Segment SGI81590[] = {{&S48946[123], 3}, {NULL}};
const Segment SGI81591[] = {{&S48946[89], 5}, {NULL}};
const Segment SGI81592[] = {{&S50336[44], 11}, {NULL}};
const Segment SGI81593[] = {{&S48947[44], 8}, {NULL}};
const Segment SGI81594[] = {{&S50336[44], 22}, {NULL}};
const Segment SGI81595[] = {{&S48947[44], 31}, {NULL}};
const Segment SGI81596[] = {{&S50336[30], 6}, {NULL}};
const Segment SGI81597[] = {{&S48947[30], 6}, {NULL}};
const Segment SGI81598[] = {{&S50336[133], 3}, {NULL}};
const Segment SGI81599[] = {{&S48947[123], 3}, {NULL}};
const Segment SGI81600[] = {{&S50336[99], 5}, {NULL}};
const Segment SGI81601[] = {{&S48947[89], 5}, {NULL}};
const Segment SGI81602[] = {{&S50336[35], 1}, {NULL}};
const Segment SGI81603[] = {{&S50336[68], 11}, {NULL}};
const Segment SGI81604[] = {{&S48948[44], 8}, {NULL}};
const Segment SGI81605[] = {{&S48948[44], 31}, {NULL}};
const Segment SGI81606[] = {{&S48948[30], 6}, {NULL}};
const Segment SGI81607[] = {{&S48948[123], 3}, {NULL}};
const Segment SGI81608[] = {{&S48948[89], 5}, {NULL}};
const Segment SGI81609[] = {{&S48949[44], 7}, {NULL}};
const Segment SGI81610[] = {{&S48949[44], 32}, {NULL}};
const Segment SGI81611[] = {{&S48949[30], 6}, {NULL}};
const Segment SGI81612[] = {{&S48949[124], 3}, {NULL}};
const Segment SGI81613[] = {{&S48949[90], 5}, {NULL}};
const Segment SGI81614[] = {{&S48950[44], 8}, {NULL}};
const Segment SGI81615[] = {{&S48950[44], 31}, {NULL}};
const Segment SGI81616[] = {{&S48950[30], 6}, {NULL}};
const Segment SGI81617[] = {{&S48950[124], 3}, {NULL}};
const Segment SGI81618[] = {{&S48950[89], 6}, {NULL}};
const Segment SGI81619[] = {{&S48951[44], 7}, {NULL}};
const Segment SGI81620[] = {{&S48951[44], 32}, {NULL}};
const Segment SGI81621[] = {{&S48951[30], 6}, {NULL}};
const Segment SGI81622[] = {{&S48951[125], 3}, {NULL}};
const Segment SGI81623[] = {{&S48951[90], 6}, {NULL}};
const Segment SGI81624[] = {{&S48952[44], 8}, {NULL}};
const Segment SGI81625[] = {{&S48952[44], 31}, {NULL}};
const Segment SGI81626[] = {{&S48952[30], 6}, {NULL}};
const Segment SGI81627[] = {{&S48952[124], 3}, {NULL}};
const Segment SGI81628[] = {{&S48952[89], 6}, {NULL}};
const Segment SGI81629[] = {{&S48953[44], 8}, {NULL}};
const Segment SGI81630[] = {{&S48953[44], 31}, {NULL}};
const Segment SGI81631[] = {{&S48953[30], 6}, {NULL}};
const Segment SGI81632[] = {{&S48953[124], 3}, {NULL}};
const Segment SGI81633[] = {{&S48953[89], 6}, {NULL}};
const Segment SGI81634[] = {{&S48954[38], 8}, {NULL}};
const Segment SGI81635[] = {{&S48954[31], 6}, {NULL}};
const Segment SGI81636[] = {{&S50648[44], 7}, {NULL}};
const Segment SGI81637[] = {{&S50648[44], 19}, {NULL}};
const Segment SGI81638[] = {{&S50648[30], 6}, {NULL}};
const Segment SGI81639[] = {{&S50648[130], 3}, {NULL}};
const Segment SGI81640[] = {{&S50648[96], 5}, {NULL}};
const Segment SGI81641[] = {{&S50648[35], 1}, {NULL}};
const Segment SGI81642[] = {{&S50648[65], 11}, {NULL}};
const Segment SGI81643[] = {{&S50649[44], 7}, {NULL}};
const Segment SGI81644[] = {{&S50649[44], 19}, {NULL}};
const Segment SGI81645[] = {{&S50649[30], 6}, {NULL}};
const Segment SGI81646[] = {{&S50649[127], 3}, {NULL}};
const Segment SGI81647[] = {{&S50649[93], 5}, {NULL}};
const Segment SGI81648[] = {{&S50649[65], 3}, {NULL}};
const Segment SGI81649[] = {{&S50650[44], 7}, {NULL}};
const Segment SGI81650[] = {{&S50650[44], 20}, {NULL}};
const Segment SGI81651[] = {{&S50650[30], 6}, {NULL}};
const Segment SGI81652[] = {{&S50650[131], 3}, {NULL}};
const Segment SGI81653[] = {{&S50650[97], 5}, {NULL}};
const Segment SGI81654[] = {{&S50650[35], 1}, {NULL}};
const Segment SGI81655[] = {{&S50650[66], 11}, {NULL}};
const Segment SGI81656[] = {{&S50651[44], 7}, {NULL}};
const Segment SGI81657[] = {{&S50651[44], 20}, {NULL}};
const Segment SGI81658[] = {{&S50651[30], 6}, {NULL}};
const Segment SGI81659[] = {{&S50651[130], 3}, {NULL}};
const Segment SGI81660[] = {{&S50651[96], 5}, {NULL}};
const Segment SGI81661[] = {{&S50651[66], 3}, {NULL}};
const Segment SGI81662[] = {{&S56900[44], 11}, {NULL}};
const Segment SGI81663[] = {{&S56900[44], 28}, {NULL}};
const Segment SGI81664[] = {{&S56900[30], 6}, {NULL}};
const Segment SGI81665[] = {{&S56900[120], 3}, {NULL}};
const Segment SGI81666[] = {{&S56900[86], 5}, {NULL}};
const Segment SGI81667[] = {{&S56901[38], 11}, {NULL}};
const Segment SGI81668[] = {{&S56901[31], 6}, {NULL}};
const Segment SGI81669[] = {{&S56903[44], 10}, {NULL}};
const Segment SGI81670[] = {{&S56903[44], 22}, {NULL}};
const Segment SGI81671[] = {{&S56903[30], 6}, {NULL}};
const Segment SGI81672[] = {{&S56903[114], 3}, {NULL}};
const Segment SGI81673[] = {{&S56903[80], 5}, {NULL}};
const Segment SGI81674[] = {{&S56904[38], 10}, {NULL}};
const Segment SGI81675[] = {{&S56904[31], 6}, {NULL}};
const Segment SGI81676[] = {{&S50748[44], 11}, {NULL}};
const Segment SGI81677[] = {{&S50748[44], 23}, {NULL}};
const Segment SGI81678[] = {{&S50748[30], 6}, {NULL}};
const Segment SGI81679[] = {{&S50748[134], 3}, {NULL}};
const Segment SGI81680[] = {{&S50748[100], 5}, {NULL}};
const Segment SGI81681[] = {{&S50748[35], 1}, {NULL}};
const Segment SGI81682[] = {{&S50748[69], 11}, {NULL}};
const Segment SGI81683[] = {{&S49392[44], 7}, {NULL}};
const Segment SGI81684[] = {{&S49392[44], 26}, {NULL}};
const Segment SGI81685[] = {{&S49392[30], 6}, {NULL}};
const Segment SGI81686[] = {{&S49392[119], 3}, {NULL}};
const Segment SGI81687[] = {{&S49392[84], 6}, {NULL}};
const Segment SGI81688[] = {{&S49393[44], 7}, {NULL}};
const Segment SGI81689[] = {{&S49393[44], 21}, {NULL}};
const Segment SGI81690[] = {{&S49393[30], 6}, {NULL}};
const Segment SGI81691[] = {{&S49393[114], 3}, {NULL}};
const Segment SGI81692[] = {{&S49393[79], 6}, {NULL}};
const Segment SGI81693[] = {{&S49394[44], 7}, {NULL}};
const Segment SGI81694[] = {{&S49394[44], 21}, {NULL}};
const Segment SGI81695[] = {{&S49394[30], 6}, {NULL}};
const Segment SGI81696[] = {{&S49394[114], 3}, {NULL}};
const Segment SGI81697[] = {{&S49394[79], 6}, {NULL}};
const Segment SGI81698[] = {{&S49395[44], 7}, {NULL}};
const Segment SGI81699[] = {{&S49395[44], 21}, {NULL}};
const Segment SGI81700[] = {{&S49395[30], 6}, {NULL}};
const Segment SGI81701[] = {{&S49395[114], 3}, {NULL}};
const Segment SGI81702[] = {{&S49395[79], 6}, {NULL}};
const Segment SGI81703[] = {{&S49396[44], 7}, {NULL}};
const Segment SGI81704[] = {{&S49396[44], 21}, {NULL}};
const Segment SGI81705[] = {{&S49396[30], 6}, {NULL}};
const Segment SGI81706[] = {{&S49396[114], 3}, {NULL}};
const Segment SGI81707[] = {{&S49396[79], 6}, {NULL}};
const Segment SGI81708[] = {{&S49397[44], 7}, {NULL}};
const Segment SGI81709[] = {{&S49397[44], 21}, {NULL}};
const Segment SGI81710[] = {{&S49397[30], 6}, {NULL}};
const Segment SGI81711[] = {{&S49397[114], 3}, {NULL}};
const Segment SGI81712[] = {{&S49397[79], 6}, {NULL}};
const Segment SGI81713[] = {{&S49398[44], 7}, {NULL}};
const Segment SGI81714[] = {{&S49398[44], 23}, {NULL}};
const Segment SGI81715[] = {{&S49398[30], 6}, {NULL}};
const Segment SGI81716[] = {{&S49398[116], 3}, {NULL}};
const Segment SGI81717[] = {{&S49398[81], 6}, {NULL}};
const Segment SGI81718[] = {{&S49399[44], 7}, {NULL}};
const Segment SGI81719[] = {{&S49399[44], 21}, {NULL}};
const Segment SGI81720[] = {{&S49399[30], 6}, {NULL}};
const Segment SGI81721[] = {{&S49399[114], 3}, {NULL}};
const Segment SGI81722[] = {{&S49399[79], 6}, {NULL}};
const Segment SGI81723[] = {{&S49400[38], 7}, {NULL}};
const Segment SGI81724[] = {{&S49400[31], 6}, {NULL}};
const Segment SGI81725[] = {{&S44015[34], 15}, {NULL}};
const Segment SGI81726[] = {{&S44015[27], 6}, {NULL}};
const Segment SGI81727[] = {{&S44015[102], 9}, {NULL}};
const Segment SGI81728[] = {{&S44015[75], 6}, {NULL}};
const Segment SGI81729[] = {{&S44015[0], 13}, {NULL}};
const Segment SGI81730[] = {{&S57075[44], 5}, {NULL}};
const Segment SGI81731[] = {{&S57075[44], 26}, {NULL}};
const Segment SGI81732[] = {{&S57075[30], 6}, {NULL}};
const Segment SGI81733[] = {{&S57075[118], 3}, {NULL}};
const Segment SGI81734[] = {{&S57075[84], 5}, {NULL}};
const Segment SGI81735[] = {{&S57076[38], 5}, {NULL}};
const Segment SGI81736[] = {{&S57076[31], 6}, {NULL}};
const Segment SGI81737[] = {{&S44020[34], 14}, {NULL}};
const Segment SGI81738[] = {{&S44020[27], 6}, {NULL}};
const Segment SGI81739[] = {{&S44020[101], 9}, {NULL}};
const Segment SGI81740[] = {{&S44020[74], 6}, {NULL}};
const Segment SGI81741[] = {{&S44020[0], 13}, {NULL}};
const Segment SGI81742[] = {{&S44028[34], 17}, {NULL}};
const Segment SGI81743[] = {{&S44028[27], 6}, {NULL}};
const Segment SGI81744[] = {{&S44028[104], 9}, {NULL}};
const Segment SGI81745[] = {{&S44028[77], 6}, {NULL}};
const Segment SGI81746[] = {{&S44028[0], 13}, {NULL}};
const Segment SGI81747[] = {{&S37610[44], 6}, {NULL}};
const Segment SGI81748[] = {{&S37610[44], 18}, {NULL}};
const Segment SGI81749[] = {{&S37610[30], 6}, {NULL}};
const Segment SGI81750[] = {{&S37610[31], 3}, {NULL}};
const Segment SGI81751[] = {{&S37610[76], 6}, {NULL}};
const Segment SGI81752[] = {{&S37611[38], 6}, {NULL}};
const Segment SGI81753[] = {{&S37611[31], 6}, {NULL}};
const Segment SGI81754[] = {{&S37612[44], 6}, {NULL}};
const Segment SGI81755[] = {{&S37612[44], 18}, {NULL}};
const Segment SGI81756[] = {{&S37612[30], 6}, {NULL}};
const Segment SGI81757[] = {{&S37612[31], 3}, {NULL}};
const Segment SGI81758[] = {{&S37612[76], 6}, {NULL}};
const Segment SGI81759[] = {{&S37613[42], 6}, {NULL}};
const Segment SGI81760[] = {{&S37613[30], 6}, {NULL}};
const Segment SGI81761[] = {{&S37613[31], 3}, {NULL}};
const Segment SGI81762[] = {{&S37613[74], 6}, {NULL}};
const Segment SGI81763[] = {{&S57181[44], 10}, {NULL}};
const Segment SGI81764[] = {{&S57181[44], 27}, {NULL}};
const Segment SGI81765[] = {{&S57181[30], 6}, {NULL}};
const Segment SGI81766[] = {{&S57181[119], 3}, {NULL}};
const Segment SGI81767[] = {{&S57181[85], 5}, {NULL}};
const Segment SGI81768[] = {{&S57182[44], 11}, {NULL}};
const Segment SGI81769[] = {{&S57182[44], 28}, {NULL}};
const Segment SGI81770[] = {{&S57182[30], 6}, {NULL}};
const Segment SGI81771[] = {{&S57182[120], 3}, {NULL}};
const Segment SGI81772[] = {{&S57182[86], 5}, {NULL}};
const Segment SGI81773[] = {{&S57183[38], 10}, {NULL}};
const Segment SGI81774[] = {{&S57183[31], 6}, {NULL}};
const Segment SGI81775[] = {{&S37703[44], 7}, {NULL}};
const Segment SGI81776[] = {{&S37703[44], 19}, {NULL}};
const Segment SGI81777[] = {{&S37703[30], 6}, {NULL}};
const Segment SGI81778[] = {{&S37703[31], 3}, {NULL}};
const Segment SGI81779[] = {{&S37703[77], 6}, {NULL}};
const Segment SGI81780[] = {{&S37704[38], 7}, {NULL}};
const Segment SGI81781[] = {{&S37704[31], 6}, {NULL}};
const Segment SGI81782[] = {{&S37705[44], 7}, {NULL}};
const Segment SGI81783[] = {{&S37705[44], 19}, {NULL}};
const Segment SGI81784[] = {{&S37705[30], 6}, {NULL}};
const Segment SGI81785[] = {{&S37705[31], 3}, {NULL}};
const Segment SGI81786[] = {{&S37705[77], 6}, {NULL}};
const Segment SGI81787[] = {{&S57209[44], 10}, {NULL}};
const Segment SGI81788[] = {{&S57209[44], 27}, {NULL}};
const Segment SGI81789[] = {{&S57209[30], 6}, {NULL}};
const Segment SGI81790[] = {{&S57209[149], 4}, {NULL}};
const Segment SGI81791[] = {{&S57209[175], 3}, {NULL}};
const Segment SGI81792[] = {{&S57209[119], 3}, {&S57209[224], 5}, {NULL}};
const Segment SGI81793[] = {{&S57209[85], 5}, {NULL}};
const Segment SGI81794[] = {{&S37754[44], 12}, {NULL}};
const Segment SGI81795[] = {{&S37754[44], 23}, {NULL}};
const Segment SGI81796[] = {{&S37754[30], 6}, {NULL}};
const Segment SGI81797[] = {{&S37754[31], 3}, {NULL}};
const Segment SGI81798[] = {{&S37754[81], 6}, {NULL}};
const Segment SGI81799[] = {{&S49629[42], 10}, {NULL}};
const Segment SGI81800[] = {{&S49629[42], 22}, {NULL}};
const Segment SGI81801[] = {{&S49629[30], 4}, {NULL}};
const Segment SGI81802[] = {{&S49629[113], 3}, {NULL}};
const Segment SGI81803[] = {{&S49629[78], 6}, {NULL}};
const Segment SGI81804[] = {{&S37757[44], 12}, {NULL}};
const Segment SGI81805[] = {{&S37757[44], 24}, {NULL}};
const Segment SGI81806[] = {{&S37757[30], 6}, {NULL}};
const Segment SGI81807[] = {{&S37757[31], 3}, {NULL}};
const Segment SGI81808[] = {{&S37757[82], 6}, {NULL}};
const Segment SGI81809[] = {{&S49630[42], 13}, {NULL}};
const Segment SGI81810[] = {{&S49630[42], 25}, {NULL}};
const Segment SGI81811[] = {{&S49630[30], 4}, {NULL}};
const Segment SGI81812[] = {{&S49630[116], 3}, {NULL}};
const Segment SGI81813[] = {{&S49630[81], 6}, {NULL}};
const Segment SGI81814[] = {{&S49631[36], 10}, {NULL}};
const Segment SGI81815[] = {{&S49631[31], 4}, {NULL}};
const Segment SGI81816[] = {{&S37759[44], 14}, {NULL}};
const Segment SGI81817[] = {{&S37759[44], 26}, {NULL}};
const Segment SGI81818[] = {{&S37759[30], 6}, {NULL}};
const Segment SGI81819[] = {{&S37759[31], 3}, {NULL}};
const Segment SGI81820[] = {{&S37759[84], 6}, {NULL}};
const Segment SGI81821[] = {{&S49632[42], 10}, {NULL}};
const Segment SGI81822[] = {{&S49632[42], 22}, {NULL}};
const Segment SGI81823[] = {{&S49632[30], 4}, {NULL}};
const Segment SGI81824[] = {{&S37760[44], 14}, {NULL}};
const Segment SGI81825[] = {{&S37760[44], 26}, {NULL}};
const Segment SGI81826[] = {{&S37760[30], 6}, {NULL}};
const Segment SGI81827[] = {{&S49632[113], 3}, {NULL}};
const Segment SGI81828[] = {{&S49632[78], 6}, {NULL}};
const Segment SGI81829[] = {{&S37760[31], 3}, {NULL}};
const Segment SGI81830[] = {{&S37760[84], 6}, {NULL}};
const Segment SGI81831[] = {{&S37761[44], 21}, {NULL}};
const Segment SGI81832[] = {{&S37761[44], 33}, {NULL}};
const Segment SGI81833[] = {{&S37761[30], 6}, {NULL}};
const Segment SGI81834[] = {{&S37761[31], 3}, {NULL}};
const Segment SGI81835[] = {{&S37761[91], 6}, {NULL}};
const Segment SGI81836[] = {{&S37762[44], 8}, {NULL}};
const Segment SGI81837[] = {{&S37762[44], 17}, {NULL}};
const Segment SGI81838[] = {{&S37762[30], 6}, {NULL}};
const Segment SGI81839[] = {{&S37762[31], 3}, {NULL}};
const Segment SGI81840[] = {{&S37762[75], 6}, {NULL}};
const Segment SGI81841[] = {{&S37764[44], 15}, {NULL}};
const Segment SGI81842[] = {{&S37764[44], 27}, {NULL}};
const Segment SGI81843[] = {{&S37764[30], 6}, {NULL}};
const Segment SGI81844[] = {{&S37764[31], 3}, {NULL}};
const Segment SGI81845[] = {{&S37764[85], 6}, {NULL}};
const Segment SGI81846[] = {{&S37765[44], 15}, {NULL}};
const Segment SGI81847[] = {{&S37765[44], 27}, {NULL}};
const Segment SGI81848[] = {{&S37765[30], 6}, {NULL}};
const Segment SGI81849[] = {{&S37765[31], 3}, {NULL}};
const Segment SGI81850[] = {{&S37765[85], 6}, {NULL}};
const Segment SGI81851[] = {{&S37767[38], 14}, {NULL}};
const Segment SGI81852[] = {{&S37767[31], 6}, {NULL}};
const Segment SGI81853[] = {{&S37768[44], 21}, {NULL}};
const Segment SGI81854[] = {{&S37768[44], 33}, {NULL}};
const Segment SGI81855[] = {{&S37768[30], 6}, {NULL}};
const Segment SGI81856[] = {{&S37768[31], 3}, {NULL}};
const Segment SGI81857[] = {{&S37768[91], 6}, {NULL}};
const Segment SGI81858[] = {{&S37769[42], 14}, {NULL}};
const Segment SGI81859[] = {{&S37769[30], 6}, {NULL}};
const Segment SGI81860[] = {{&S37769[31], 3}, {NULL}};
const Segment SGI81861[] = {{&S37769[82], 6}, {NULL}};
const Segment SGI81862[] = {{&S37770[38], 15}, {NULL}};
const Segment SGI81863[] = {{&S37770[31], 6}, {NULL}};
const Segment SGI81864[] = {{&S37774[44], 14}, {NULL}};
const Segment SGI81865[] = {{&S37774[44], 26}, {NULL}};
const Segment SGI81866[] = {{&S37774[30], 6}, {NULL}};
const Segment SGI81867[] = {{&S37774[31], 3}, {NULL}};
const Segment SGI81868[] = {{&S37774[84], 6}, {NULL}};
const Segment SGI81869[] = {{&S37775[44], 16}, {NULL}};
const Segment SGI81870[] = {{&S37775[44], 28}, {NULL}};
const Segment SGI81871[] = {{&S37775[30], 6}, {NULL}};
const Segment SGI81872[] = {{&S37775[31], 3}, {NULL}};
const Segment SGI81873[] = {{&S37775[86], 6}, {NULL}};
const Segment SGI81874[] = {{&S37776[38], 21}, {NULL}};
const Segment SGI81875[] = {{&S37776[31], 6}, {NULL}};
const Segment SGI81876[] = {{&S57319[44], 4}, {NULL}};
const Segment SGI81877[] = {{&S57319[44], 16}, {NULL}};
const Segment SGI81878[] = {{&S57319[30], 6}, {NULL}};
const Segment SGI81879[] = {{&S57319[108], 3}, {NULL}};
const Segment SGI81880[] = {{&S57319[74], 5}, {NULL}};
const Segment SGI81881[] = {{&S57320[44], 12}, {NULL}};
const Segment SGI81882[] = {{&S57320[44], 24}, {NULL}};
const Segment SGI81883[] = {{&S57320[30], 6}, {NULL}};
const Segment SGI81884[] = {{&S57320[116], 3}, {NULL}};
const Segment SGI81885[] = {{&S57320[82], 5}, {NULL}};
const Segment SGI81886[] = {{&S57321[44], 5}, {NULL}};
const Segment SGI81887[] = {{&S57321[44], 17}, {NULL}};
const Segment SGI81888[] = {{&S57321[30], 6}, {NULL}};
const Segment SGI81889[] = {{&S57321[109], 3}, {NULL}};
const Segment SGI81890[] = {{&S57321[75], 5}, {NULL}};
const Segment SGI81891[] = {{&S57322[38], 4}, {NULL}};
const Segment SGI81892[] = {{&S57322[31], 6}, {NULL}};
const Segment SGI81893[] = {{&S37810[44], 5}, {NULL}};
const Segment SGI81894[] = {{&S37810[44], 14}, {NULL}};
const Segment SGI81895[] = {{&S37810[30], 6}, {NULL}};
const Segment SGI81896[] = {{&S37810[31], 3}, {NULL}};
const Segment SGI81897[] = {{&S37810[72], 6}, {NULL}};
const Segment SGI81898[] = {{&S37812[44], 22}, {NULL}};
const Segment SGI81899[] = {{&S49796[44], 14}, {NULL}};
const Segment SGI81900[] = {{&S49796[44], 26}, {NULL}};
const Segment SGI81901[] = {{&S49796[30], 6}, {NULL}};
const Segment SGI81902[] = {{&S49796[119], 3}, {NULL}};
const Segment SGI81903[] = {{&S49796[84], 6}, {NULL}};
const Segment SGI81904[] = {{&S49797[42], 14}, {NULL}};
const Segment SGI81905[] = {{&S49797[42], 26}, {NULL}};
const Segment SGI81906[] = {{&S49797[30], 4}, {NULL}};
const Segment SGI81907[] = {{&S49797[117], 3}, {NULL}};
const Segment SGI81908[] = {{&S49797[82], 6}, {NULL}};
const Segment SGI81909[] = {{&S49798[38], 14}, {NULL}};
const Segment SGI81910[] = {{&S49798[31], 6}, {NULL}};
const Segment SGI81911[] = {{&S37812[44], 34}, {NULL}};
const Segment SGI81912[] = {{&S37812[30], 6}, {NULL}};
const Segment SGI81913[] = {{&S37812[31], 3}, {NULL}};
const Segment SGI81914[] = {{&S37812[92], 6}, {NULL}};
const Segment SGI81915[] = {{&S37813[44], 22}, {NULL}};
const Segment SGI81916[] = {{&S37813[44], 34}, {NULL}};
const Segment SGI81917[] = {{&S37813[30], 6}, {NULL}};
const Segment SGI81918[] = {{&S37813[31], 3}, {NULL}};
const Segment SGI81919[] = {{&S37813[92], 6}, {NULL}};
const Segment SGI81920[] = {{&S37816[44], 5}, {NULL}};
const Segment SGI81921[] = {{&S37816[44], 14}, {NULL}};
const Segment SGI81922[] = {{&S37816[30], 6}, {NULL}};
const Segment SGI81923[] = {{&S37816[31], 3}, {NULL}};
const Segment SGI81924[] = {{&S37816[72], 6}, {NULL}};
const Segment SGI81925[] = {{&S37822[38], 22}, {NULL}};
const Segment SGI81926[] = {{&S37822[31], 6}, {NULL}};
const Segment SGI81927[] = {{&S37823[44], 12}, {NULL}};
const Segment SGI81928[] = {{&S37823[44], 21}, {NULL}};
const Segment SGI81929[] = {{&S37823[30], 6}, {NULL}};
const Segment SGI81930[] = {{&S37823[31], 3}, {NULL}};
const Segment SGI81931[] = {{&S37823[79], 6}, {NULL}};
const Segment SGI81932[] = {{&S37825[44], 5}, {NULL}};
const Segment SGI81933[] = {{&S37825[44], 16}, {NULL}};
const Segment SGI81934[] = {{&S37825[30], 6}, {NULL}};
const Segment SGI81935[] = {{&S37825[31], 3}, {NULL}};
const Segment SGI81936[] = {{&S37825[74], 5}, {NULL}};
const Segment SGI81937[] = {{&S37828[44], 14}, {NULL}};
const Segment SGI81938[] = {{&S37828[44], 28}, {NULL}};
const Segment SGI81939[] = {{&S37828[30], 6}, {NULL}};
const Segment SGI81940[] = {{&S37828[31], 3}, {NULL}};
const Segment SGI81941[] = {{&S37828[86], 6}, {NULL}};
const Segment SGI81942[] = {{&S37829[44], 9}, {NULL}};
const Segment SGI81943[] = {{&S37829[44], 21}, {NULL}};
const Segment SGI81944[] = {{&S37829[30], 6}, {NULL}};
const Segment SGI81945[] = {{&S37829[31], 3}, {NULL}};
const Segment SGI81946[] = {{&S37829[79], 6}, {NULL}};
const Segment SGI81947[] = {{&S57498[44], 10}, {NULL}};
const Segment SGI81948[] = {{&S57498[44], 27}, {NULL}};
const Segment SGI81949[] = {{&S57498[30], 6}, {NULL}};
const Segment SGI81950[] = {{&S57498[119], 3}, {NULL}};
const Segment SGI81951[] = {{&S57498[85], 5}, {NULL}};
const Segment SGI81952[] = {{&S57499[44], 10}, {NULL}};
const Segment SGI81953[] = {{&S57499[44], 27}, {NULL}};
const Segment SGI81954[] = {{&S57499[30], 6}, {NULL}};
const Segment SGI81955[] = {{&S57499[88], 4}, {NULL}};
const Segment SGI81956[] = {{&S57499[88], 3}, {NULL}};
const Segment SGI81957[] = {{&S57500[44], 10}, {NULL}};
const Segment SGI81958[] = {{&S57500[44], 27}, {NULL}};
const Segment SGI81959[] = {{&S57500[30], 6}, {NULL}};
const Segment SGI81960[] = {{&S57500[88], 6}, {NULL}};
const Segment SGI81961[] = {{&S57501[38], 10}, {NULL}};
const Segment SGI81962[] = {{&S57501[31], 6}, {NULL}};
const Segment SGI81963[] = {{&S57502[44], 7}, {NULL}};
const Segment SGI81964[] = {{&S57502[44], 19}, {NULL}};
const Segment SGI81965[] = {{&S57502[30], 6}, {NULL}};
const Segment SGI81966[] = {{&S57502[111], 3}, {NULL}};
const Segment SGI81967[] = {{&S57502[77], 5}, {NULL}};
const Segment SGI81968[] = {{&S37883[44], 25}, {NULL}};
const Segment SGI81969[] = {{&S37883[44], 37}, {NULL}};
const Segment SGI81970[] = {{&S37883[30], 6}, {NULL}};
const Segment SGI81971[] = {{&S37883[31], 3}, {NULL}};
const Segment SGI81972[] = {{&S37883[95], 6}, {NULL}};
const Segment SGI81973[] = {{&S37884[44], 20}, {NULL}};
const Segment SGI81974[] = {{&S37884[44], 32}, {NULL}};
const Segment SGI81975[] = {{&S37884[30], 6}, {NULL}};
const Segment SGI81976[] = {{&S37884[31], 3}, {NULL}};
const Segment SGI81977[] = {{&S37884[90], 6}, {NULL}};
const Segment SGI81978[] = {{&S37885[44], 26}, {NULL}};
const Segment SGI81979[] = {{&S37885[44], 38}, {NULL}};
const Segment SGI81980[] = {{&S37885[30], 6}, {NULL}};
const Segment SGI81981[] = {{&S37885[31], 3}, {NULL}};
const Segment SGI81982[] = {{&S37885[96], 6}, {NULL}};
const Segment SGI81983[] = {{&S37886[44], 33}, {NULL}};
const Segment SGI81984[] = {{&S37886[44], 45}, {NULL}};
const Segment SGI81985[] = {{&S37886[30], 6}, {NULL}};
const Segment SGI81986[] = {{&S37886[31], 3}, {NULL}};
const Segment SGI81987[] = {{&S37886[103], 6}, {NULL}};
const Segment SGI81988[] = {{&S37888[38], 25}, {NULL}};
const Segment SGI81989[] = {{&S37888[31], 6}, {NULL}};
const Segment SGI81990[] = {{&S37889[44], 21}, {NULL}};
const Segment SGI81991[] = {{&S37889[44], 33}, {NULL}};
const Segment SGI81992[] = {{&S37889[30], 6}, {NULL}};
const Segment SGI81993[] = {{&S37889[31], 3}, {NULL}};
const Segment SGI81994[] = {{&S37889[91], 6}, {NULL}};
const Segment SGI81995[] = {{&S37892[44], 24}, {NULL}};
const Segment SGI81996[] = {{&S37892[44], 36}, {NULL}};
const Segment SGI81997[] = {{&S37892[30], 6}, {NULL}};
const Segment SGI81998[] = {{&S37892[31], 3}, {NULL}};
const Segment SGI81999[] = {{&S37892[94], 6}, {NULL}};
const Segment SGI82000[] = {{&S37893[44], 30}, {NULL}};
const Segment SGI82001[] = {{&S37893[44], 42}, {NULL}};
const Segment SGI82002[] = {{&S37893[30], 6}, {NULL}};
const Segment SGI82003[] = {{&S37893[31], 3}, {NULL}};
const Segment SGI82004[] = {{&S37893[100], 6}, {NULL}};
const Segment SGI82005[] = {{&S37894[38], 26}, {NULL}};
const Segment SGI82006[] = {{&S37894[31], 6}, {NULL}};
const Segment SGI82007[] = {{&S37896[44], 17}, {NULL}};
const Segment SGI82008[] = {{&S37896[44], 29}, {NULL}};
const Segment SGI82009[] = {{&S37896[30], 6}, {NULL}};
const Segment SGI82010[] = {{&S37896[31], 3}, {NULL}};
const Segment SGI82011[] = {{&S37896[87], 6}, {NULL}};
const Segment SGI82012[] = {{&S37900[44], 21}, {NULL}};
const Segment SGI82013[] = {{&S37900[44], 33}, {NULL}};
const Segment SGI82014[] = {{&S37900[30], 6}, {NULL}};
const Segment SGI82015[] = {{&S37900[31], 3}, {NULL}};
const Segment SGI82016[] = {{&S37900[91], 6}, {NULL}};
const Segment SGI82017[] = {{&S37902[44], 34}, {NULL}};
const Segment SGI82018[] = {{&S37902[44], 46}, {NULL}};
const Segment SGI82019[] = {{&S37902[30], 6}, {NULL}};
const Segment SGI82020[] = {{&S37902[31], 3}, {NULL}};
const Segment SGI82021[] = {{&S37902[104], 6}, {NULL}};
const Segment SGI82022[] = {{&S37906[44], 23}, {NULL}};
const Segment SGI82023[] = {{&S37906[44], 35}, {NULL}};
const Segment SGI82024[] = {{&S37906[30], 6}, {NULL}};
const Segment SGI82025[] = {{&S37906[31], 3}, {NULL}};
const Segment SGI82026[] = {{&S37906[93], 6}, {NULL}};
const Segment SGI82027[] = {{&S37909[44], 23}, {NULL}};
const Segment SGI82028[] = {{&S37909[44], 35}, {NULL}};
const Segment SGI82029[] = {{&S37909[30], 6}, {NULL}};
const Segment SGI82030[] = {{&S37909[31], 3}, {NULL}};
const Segment SGI82031[] = {{&S37909[93], 6}, {NULL}};
const Segment SGI82032[] = {{&S37910[44], 22}, {NULL}};
const Segment SGI82033[] = {{&S37910[44], 34}, {NULL}};
const Segment SGI82034[] = {{&S37910[30], 6}, {NULL}};
const Segment SGI82035[] = {{&S37910[31], 3}, {NULL}};
const Segment SGI82036[] = {{&S37910[92], 6}, {NULL}};
const Segment SGI82037[] = {{&S37911[44], 17}, {NULL}};
const Segment SGI82038[] = {{&S37911[44], 29}, {NULL}};
const Segment SGI82039[] = {{&S37911[30], 6}, {NULL}};
const Segment SGI82040[] = {{&S37911[31], 3}, {NULL}};
const Segment SGI82041[] = {{&S37911[87], 6}, {NULL}};
const Segment SGI82042[] = {{&S37912[38], 24}, {NULL}};
const Segment SGI82043[] = {{&S37912[31], 6}, {NULL}};
const Segment SGI82044[] = {{&S37913[44], 17}, {NULL}};
const Segment SGI82045[] = {{&S37913[44], 29}, {NULL}};
const Segment SGI82046[] = {{&S37913[30], 6}, {NULL}};
const Segment SGI82047[] = {{&S37913[31], 3}, {NULL}};
const Segment SGI82048[] = {{&S37913[87], 6}, {NULL}};
const Segment SGI82049[] = {{&S57625[44], 56}, {NULL}};
const Segment SGI82050[] = {{&S57625[30], 6}, {NULL}};
const Segment SGI82051[] = {{&S57625[148], 3}, {NULL}};
const Segment SGI82052[] = {{&S57625[114], 5}, {NULL}};
const Segment SGI82053[] = {{&S57626[44], 56}, {NULL}};
const Segment SGI82054[] = {{&S57626[30], 6}, {NULL}};
const Segment SGI82055[] = {{&S57626[148], 3}, {NULL}};
const Segment SGI82056[] = {{&S57626[114], 5}, {NULL}};
const Segment SGI82057[] = {{&S57627[44], 56}, {NULL}};
const Segment SGI82058[] = {{&S57627[30], 6}, {NULL}};
const Segment SGI82059[] = {{&S57627[148], 3}, {NULL}};
const Segment SGI82060[] = {{&S57627[114], 5}, {NULL}};
const Segment SGI82061[] = {{&S57628[44], 56}, {NULL}};
const Segment SGI82062[] = {{&S57628[30], 6}, {NULL}};
const Segment SGI82063[] = {{&S57628[148], 3}, {NULL}};
const Segment SGI82064[] = {{&S57628[114], 5}, {NULL}};
const Segment SGI82065[] = {{&S57629[44], 56}, {NULL}};
const Segment SGI82066[] = {{&S57629[30], 6}, {NULL}};
const Segment SGI82067[] = {{&S57629[148], 3}, {NULL}};
const Segment SGI82068[] = {{&S57629[114], 5}, {NULL}};
const Segment SGI82069[] = {{&S57630[44], 56}, {NULL}};
const Segment SGI82070[] = {{&S57630[30], 6}, {NULL}};
const Segment SGI82071[] = {{&S57630[148], 3}, {NULL}};
const Segment SGI82072[] = {{&S57630[114], 5}, {NULL}};
const Segment SGI82073[] = {{&S57631[44], 56}, {NULL}};
const Segment SGI82074[] = {{&S57631[30], 6}, {NULL}};
const Segment SGI82075[] = {{&S50011[42], 10}, {NULL}};
const Segment SGI82076[] = {{&S50011[42], 22}, {NULL}};
const Segment SGI82077[] = {{&S50011[30], 4}, {NULL}};
const Segment SGI82078[] = {{&S57631[148], 3}, {NULL}};
const Segment SGI82079[] = {{&S57631[114], 5}, {NULL}};
const Segment SGI82080[] = {{&S50011[113], 3}, {NULL}};
const Segment SGI82081[] = {{&S50011[78], 6}, {NULL}};
const Segment SGI82082[] = {{&S50012[36], 10}, {NULL}};
const Segment SGI82083[] = {{&S50012[31], 4}, {NULL}};
const Segment SGI82084[] = {{&S57632[44], 56}, {NULL}};
const Segment SGI82085[] = {{&S57632[30], 6}, {NULL}};
const Segment SGI82086[] = {{&S57632[148], 3}, {NULL}};
const Segment SGI82087[] = {{&S57632[114], 5}, {NULL}};
const Segment SGI82088[] = {{&S50013[42], 10}, {NULL}};
const Segment SGI82089[] = {{&S50013[42], 22}, {NULL}};
const Segment SGI82090[] = {{&S50013[30], 4}, {NULL}};
const Segment SGI82091[] = {{&S50013[113], 3}, {NULL}};
const Segment SGI82092[] = {{&S50013[78], 6}, {NULL}};
const Segment SGI82093[] = {{&S57633[44], 56}, {NULL}};
const Segment SGI82094[] = {{&S57633[30], 6}, {NULL}};
const Segment SGI82095[] = {{&S57633[148], 3}, {NULL}};
const Segment SGI82096[] = {{&S57633[114], 5}, {NULL}};
const Segment SGI82097[] = {{&S50014[42], 10}, {NULL}};
const Segment SGI82098[] = {{&S50014[42], 22}, {NULL}};
const Segment SGI82099[] = {{&S50014[30], 4}, {NULL}};
const Segment SGI82100[] = {{&S50014[81], 6}, {NULL}};
const Segment SGI82101[] = {{&S57634[44], 57}, {NULL}};
const Segment SGI82102[] = {{&S57634[30], 6}, {NULL}};
const Segment SGI82103[] = {{&S57634[149], 3}, {NULL}};
const Segment SGI82104[] = {{&S57634[115], 5}, {NULL}};
const Segment SGI82105[] = {{&S50015[40], 10}, {NULL}};
const Segment SGI82106[] = {{&S50015[30], 4}, {NULL}};
const Segment SGI82107[] = {{&S50015[111], 3}, {NULL}};
const Segment SGI82108[] = {{&S50015[76], 6}, {NULL}};
const Segment SGI82109[] = {{&S57635[44], 56}, {NULL}};
const Segment SGI82110[] = {{&S57635[30], 6}, {NULL}};
const Segment SGI82111[] = {{&S50016[42], 10}, {NULL}};
const Segment SGI82112[] = {{&S50016[42], 28}, {NULL}};
const Segment SGI82113[] = {{&S50016[30], 4}, {NULL}};
const Segment SGI82114[] = {{&S57635[148], 3}, {NULL}};
const Segment SGI82115[] = {{&S57635[114], 5}, {NULL}};
const Segment SGI82116[] = {{&S57636[44], 56}, {NULL}};
const Segment SGI82117[] = {{&S57636[30], 6}, {NULL}};
const Segment SGI82118[] = {{&S50017[42], 10}, {NULL}};
const Segment SGI82119[] = {{&S50017[42], 22}, {&S50017[206], 7}, {NULL}};
const Segment SGI82120[] = {{&S57636[148], 3}, {NULL}};
const Segment SGI82121[] = {{&S50017[30], 4}, {NULL}};
const Segment SGI82122[] = {{&S57636[114], 5}, {NULL}};
const Segment SGI82123[] = {{&S50017[113], 3}, {NULL}};
const Segment SGI82124[] = {{&S50017[78], 6}, {NULL}};
const Segment SGI82125[] = {{&S57637[44], 56}, {NULL}};
const Segment SGI82126[] = {{&S57637[30], 6}, {NULL}};
const Segment SGI82127[] = {{&S50018[42], 10}, {NULL}};
const Segment SGI82128[] = {{&S50018[42], 22}, {NULL}};
const Segment SGI82129[] = {{&S50018[30], 4}, {NULL}};
const Segment SGI82130[] = {{&S57637[148], 3}, {NULL}};
const Segment SGI82131[] = {{&S57637[114], 5}, {NULL}};
const Segment SGI82132[] = {{&S38005[44], 10}, {NULL}};
const Segment SGI82133[] = {{&S38005[44], 22}, {NULL}};
const Segment SGI82134[] = {{&S38005[30], 6}, {NULL}};
const Segment SGI82135[] = {{&S38005[31], 3}, {NULL}};
const Segment SGI82136[] = {{&S38005[80], 6}, {NULL}};
const Segment SGI82137[] = {{&S38006[44], 10}, {NULL}};
const Segment SGI82138[] = {{&S38006[44], 22}, {NULL}};
const Segment SGI82139[] = {{&S38006[30], 6}, {NULL}};
const Segment SGI82140[] = {{&S38006[31], 3}, {NULL}};
const Segment SGI82141[] = {{&S38006[80], 6}, {NULL}};
const Segment SGI82142[] = {{&S38007[54], 10}, {NULL}};
const Segment SGI82143[] = {{&S38007[54], 83}, {NULL}};
const Segment SGI82144[] = {{&S38007[30], 16}, {NULL}};
const Segment SGI82145[] = {{&S57653[6], 4}, {NULL}};
const Segment SGI82146[] = {{&S57653[85], 5}, {NULL}};
const Segment SGI82147[] = {{&S50055[42], 15}, {NULL}};
const Segment SGI82148[] = {{&S50055[42], 31}, {NULL}};
const Segment SGI82149[] = {{&S50055[30], 4}, {NULL}};
const Segment SGI82150[] = {{&S50055[122], 3}, {NULL}};
const Segment SGI82151[] = {{&S50055[87], 6}, {NULL}};
const Segment SGI82152[] = {{&S50056[42], 15}, {NULL}};
const Segment SGI82153[] = {{&S50056[42], 32}, {NULL}};
const Segment SGI82154[] = {{&S50056[30], 4}, {NULL}};
const Segment SGI82155[] = {{&S50056[123], 3}, {NULL}};
const Segment SGI82156[] = {{&S50056[88], 6}, {NULL}};
const Segment SGI82157[] = {{&S50057[36], 15}, {NULL}};
const Segment SGI82158[] = {{&S50057[31], 4}, {NULL}};
const Segment SGI82159[] = {{&S50069[42], 6}, {NULL}};
const Segment SGI82160[] = {{&S50069[42], 17}, {NULL}};
const Segment SGI82161[] = {{&S50069[30], 4}, {NULL}};
const Segment SGI82162[] = {{&S50069[31], 3}, {NULL}};
const Segment SGI82163[] = {{&S50069[73], 7}, {NULL}};
const Segment SGI82164[] = {{&S50069[129], 8}, {NULL}};
const Segment SGI82165[] = {{&S50070[44], 6}, {NULL}};
const Segment SGI82166[] = {{&S50070[44], 17}, {NULL}};
const Segment SGI82167[] = {{&S50070[30], 6}, {NULL}};
const Segment SGI82168[] = {{&S50070[110], 3}, {NULL}};
const Segment SGI82169[] = {{&S50070[75], 6}, {NULL}};
const Segment SGI82170[] = {{&S50071[44], 6}, {NULL}};
const Segment SGI82171[] = {{&S50071[44], 17}, {NULL}};
const Segment SGI82172[] = {{&S50071[30], 6}, {NULL}};
const Segment SGI82173[] = {{&S50071[110], 3}, {NULL}};
const Segment SGI82174[] = {{&S50071[75], 6}, {NULL}};
const Segment SGI82175[] = {{&S50072[42], 6}, {NULL}};
const Segment SGI82176[] = {{&S50072[42], 17}, {NULL}};
const Segment SGI82177[] = {{&S50072[30], 4}, {NULL}};
const Segment SGI82178[] = {{&S50072[108], 3}, {NULL}};
const Segment SGI82179[] = {{&S50072[73], 6}, {NULL}};
const Segment SGI82180[] = {{&S50073[44], 6}, {NULL}};
const Segment SGI82181[] = {{&S50073[44], 17}, {NULL}};
const Segment SGI82182[] = {{&S50073[30], 6}, {NULL}};
const Segment SGI82183[] = {{&S50073[110], 3}, {NULL}};
const Segment SGI82184[] = {{&S50073[75], 6}, {NULL}};
const Segment SGI82185[] = {{&S50074[42], 6}, {NULL}};
const Segment SGI82186[] = {{&S50074[42], 18}, {NULL}};
const Segment SGI82187[] = {{&S50074[30], 4}, {NULL}};
const Segment SGI82188[] = {{&S50074[109], 3}, {NULL}};
const Segment SGI82189[] = {{&S50074[74], 6}, {NULL}};
const Segment SGI82190[] = {{&S50075[44], 6}, {NULL}};
const Segment SGI82191[] = {{&S50075[44], 18}, {NULL}};
const Segment SGI82192[] = {{&S50075[30], 6}, {NULL}};
const Segment SGI82193[] = {{&S50075[111], 3}, {NULL}};
const Segment SGI82194[] = {{&S50075[76], 6}, {NULL}};
const Segment SGI82195[] = {{&S50076[44], 6}, {NULL}};
const Segment SGI82196[] = {{&S50076[44], 18}, {NULL}};
const Segment SGI82197[] = {{&S50076[30], 6}, {NULL}};
const Segment SGI82198[] = {{&S50076[111], 3}, {NULL}};
const Segment SGI82199[] = {{&S50076[76], 6}, {NULL}};
const Segment SGI82200[] = {{&S50077[38], 6}, {NULL}};
const Segment SGI82201[] = {{&S50077[31], 6}, {NULL}};
const Segment SGI82202[] = {{&S50078[44], 6}, {NULL}};
const Segment SGI82203[] = {{&S50078[44], 18}, {NULL}};
const Segment SGI82204[] = {{&S50078[30], 6}, {NULL}};
const Segment SGI82205[] = {{&S50078[111], 3}, {NULL}};
const Segment SGI82206[] = {{&S50078[76], 6}, {NULL}};
const Segment SGI82207[] = {{&S50079[42], 6}, {NULL}};
const Segment SGI82208[] = {{&S50079[42], 17}, {NULL}};
const Segment SGI82209[] = {{&S50079[30], 4}, {NULL}};
const Segment SGI82210[] = {{&S50079[108], 3}, {NULL}};
const Segment SGI82211[] = {{&S50079[73], 6}, {NULL}};
const Segment SGI82212[] = {{&S57710[6], 4}, {NULL}};
const Segment SGI82213[] = {{&S57710[78], 5}, {NULL}};
const Segment SGI82214[] = {{&S50080[44], 6}, {NULL}};
const Segment SGI82215[] = {{&S50080[44], 17}, {NULL}};
const Segment SGI82216[] = {{&S50080[30], 6}, {NULL}};
const Segment SGI82217[] = {{&S50080[110], 3}, {NULL}};
const Segment SGI82218[] = {{&S50080[75], 6}, {NULL}};
const Segment SGI82219[] = {{&S50081[44], 6}, {NULL}};
const Segment SGI82220[] = {{&S50081[44], 21}, {NULL}};
const Segment SGI82221[] = {{&S50081[30], 6}, {NULL}};
const Segment SGI82222[] = {{&S50081[114], 3}, {NULL}};
const Segment SGI82223[] = {{&S50081[79], 6}, {NULL}};
const Segment SGI82224[] = {{&S50082[44], 6}, {NULL}};
const Segment SGI82225[] = {{&S50082[44], 18}, {NULL}};
const Segment SGI82226[] = {{&S50082[30], 6}, {NULL}};
const Segment SGI82227[] = {{&S50082[111], 3}, {NULL}};
const Segment SGI82228[] = {{&S50082[76], 6}, {NULL}};
const Segment SGI82229[] = {{&S50083[42], 6}, {NULL}};
const Segment SGI82230[] = {{&S50083[30], 6}, {NULL}};
const Segment SGI82231[] = {{&S50083[108], 3}, {NULL}};
const Segment SGI82232[] = {{&S50083[73], 6}, {NULL}};
const Segment SGI82233[] = {{&S50084[42], 6}, {NULL}};
const Segment SGI82234[] = {{&S50084[42], 17}, {NULL}};
const Segment SGI82235[] = {{&S50084[30], 4}, {NULL}};
const Segment SGI82236[] = {{&S50084[108], 3}, {NULL}};
const Segment SGI82237[] = {{&S50084[73], 6}, {NULL}};
const Segment SGI82238[] = {{&S50085[42], 6}, {NULL}};
const Segment SGI82239[] = {{&S50085[42], 18}, {NULL}};
const Segment SGI82240[] = {{&S50085[30], 4}, {NULL}};
const Segment SGI82241[] = {{&S50085[109], 3}, {NULL}};
const Segment SGI82242[] = {{&S50085[74], 6}, {NULL}};
const Segment SGI82243[] = {{&S50086[42], 6}, {NULL}};
const Segment SGI82244[] = {{&S50086[42], 17}, {NULL}};
const Segment SGI82245[] = {{&S50086[30], 4}, {NULL}};
const Segment SGI82246[] = {{&S50086[31], 3}, {NULL}};
const Segment SGI82247[] = {{&S50086[73], 6}, {NULL}};
const Segment SGI82248[] = {{&S50086[129], 8}, {NULL}};
const Segment SGI82249[] = {{&S57765[44], 9}, {NULL}};
const Segment SGI82250[] = {{&S57765[44], 20}, {NULL}};
const Segment SGI82251[] = {{&S57765[30], 6}, {NULL}};
const Segment SGI82252[] = {{&S57765[112], 3}, {NULL}};
const Segment SGI82253[] = {{&S57765[78], 5}, {NULL}};
const Segment SGI82254[] = {{&S57766[44], 14}, {NULL}};
const Segment SGI82255[] = {{&S57766[44], 25}, {NULL}};
const Segment SGI82256[] = {{&S57766[30], 6}, {NULL}};
const Segment SGI82257[] = {{&S57766[117], 3}, {NULL}};
const Segment SGI82258[] = {{&S57766[83], 5}, {NULL}};
const Segment SGI82259[] = {{&S57767[44], 14}, {NULL}};
const Segment SGI82260[] = {{&S57767[44], 25}, {NULL}};
const Segment SGI82261[] = {{&S57767[30], 6}, {NULL}};
const Segment SGI82262[] = {{&S57767[117], 3}, {NULL}};
const Segment SGI82263[] = {{&S57767[83], 5}, {NULL}};
const Segment SGI82264[] = {{&S57768[41], 9}, {NULL}};
const Segment SGI82265[] = {{&S57768[30], 6}, {NULL}};
const Segment SGI82266[] = {{&S57768[109], 3}, {NULL}};
const Segment SGI82267[] = {{&S57768[75], 5}, {NULL}};
const Segment SGI82268[] = {{&S57769[38], 9}, {NULL}};
const Segment SGI82269[] = {{&S57769[31], 6}, {NULL}};
const Segment SGI82270[] = {{&S57770[44], 14}, {NULL}};
const Segment SGI82271[] = {{&S57770[44], 25}, {NULL}};
const Segment SGI82272[] = {{&S57770[30], 6}, {NULL}};
const Segment SGI82273[] = {{&S57770[117], 3}, {NULL}};
const Segment SGI82274[] = {{&S57770[83], 5}, {NULL}};
const Segment SGI82275[] = {{&S38289[44], 10}, {NULL}};
const Segment SGI82276[] = {{&S38289[44], 22}, {NULL}};
const Segment SGI82277[] = {{&S38289[30], 6}, {NULL}};
const Segment SGI82278[] = {{&S38289[31], 3}, {NULL}};
const Segment SGI82279[] = {{&S38289[80], 6}, {NULL}};
const Segment SGI82280[] = {{&S51768[44], 9}, {NULL}};
const Segment SGI82281[] = {{&S51768[44], 20}, {NULL}};
const Segment SGI82282[] = {{&S51768[30], 6}, {NULL}};
const Segment SGI82283[] = {{&S51768[131], 3}, {NULL}};
const Segment SGI82284[] = {{&S51768[97], 5}, {NULL}};
const Segment SGI82285[] = {{&S51768[35], 1}, {NULL}};
const Segment SGI82286[] = {{&S51768[66], 11}, {NULL}};
const Segment SGI82287[] = {{&S51946[44], 12}, {&S51946[87], 9}, {NULL}};
const Segment SGI82288[] = {{&S51946[44], 23}, {NULL}};
const Segment SGI82289[] = {{&S51946[30], 6}, {NULL}};
const Segment SGI82290[] = {{&S51946[155], 3}, {NULL}};
const Segment SGI82291[] = {{&S51946[121], 5}, {NULL}};
const Segment SGI82292[] = {{&S51946[35], 1}, {NULL}};
const Segment SGI82293[] = {{&S51946[69], 11}, {NULL}};
const Segment SGI82294[] = {{&S51947[44], 12}, {NULL}};
const Segment SGI82295[] = {{&S51947[44], 23}, {NULL}};
const Segment SGI82296[] = {{&S51947[30], 6}, {NULL}};
const Segment SGI82297[] = {{&S51947[130], 3}, {NULL}};
const Segment SGI82298[] = {{&S51947[96], 5}, {NULL}};
const Segment SGI82299[] = {{&S51947[35], 1}, {NULL}};
const Segment SGI82300[] = {{&S51947[69], 11}, {NULL}};
const Segment SGI82301[] = {{&S58123[31], 8}, {NULL}};
const Segment SGI82302[] = {{&S58123[54], 3}, {NULL}};
const Segment SGI82303[] = {{&S58124[39], 8}, {NULL}};
const Segment SGI82304[] = {{&S58124[103], 3}, {NULL}};
const Segment SGI82305[] = {{&S58124[92], 3}, {NULL}};
const Segment SGI82306[] = {{&S58124[69], 6}, {NULL}};
const Segment SGI82307[] = {{&S50450[42], 19}, {NULL}};
const Segment SGI82308[] = {{&S50450[42], 30}, {NULL}};
const Segment SGI82309[] = {{&S50450[30], 4}, {NULL}};
const Segment SGI82310[] = {{&S50450[121], 3}, {NULL}};
const Segment SGI82311[] = {{&S50450[86], 6}, {NULL}};
const Segment SGI82312[] = {{&S50451[42], 19}, {NULL}};
const Segment SGI82313[] = {{&S50451[42], 30}, {NULL}};
const Segment SGI82314[] = {{&S50451[30], 4}, {NULL}};
const Segment SGI82315[] = {{&S50451[121], 3}, {NULL}};
const Segment SGI82316[] = {{&S50451[86], 6}, {NULL}};
const Segment SGI82317[] = {{&S50452[36], 19}, {NULL}};
const Segment SGI82318[] = {{&S50452[31], 4}, {NULL}};
const Segment SGI82319[] = {{&S50453[42], 19}, {NULL}};
const Segment SGI82320[] = {{&S50453[42], 30}, {NULL}};
const Segment SGI82321[] = {{&S50453[30], 4}, {NULL}};
const Segment SGI82322[] = {{&S50453[122], 3}, {NULL}};
const Segment SGI82323[] = {{&S50453[86], 7}, {NULL}};
const Segment SGI82324[] = {{&S38620[44], 14}, {NULL}};
const Segment SGI82325[] = {{&S38620[44], 31}, {NULL}};
const Segment SGI82326[] = {{&S38620[30], 6}, {NULL}};
const Segment SGI82327[] = {{&S38620[31], 3}, {NULL}};
const Segment SGI82328[] = {{&S38620[89], 6}, {NULL}};
const Segment SGI82329[] = {{&S38621[44], 14}, {NULL}};
const Segment SGI82330[] = {{&S38621[44], 31}, {NULL}};
const Segment SGI82331[] = {{&S38621[30], 6}, {NULL}};
const Segment SGI82332[] = {{&S38621[31], 3}, {NULL}};
const Segment SGI82333[] = {{&S38621[89], 6}, {NULL}};
const Segment SGI82334[] = {{&S38622[44], 14}, {NULL}};
const Segment SGI82335[] = {{&S38622[44], 38}, {NULL}};
const Segment SGI82336[] = {{&S38622[30], 6}, {NULL}};
const Segment SGI82337[] = {{&S38622[31], 3}, {NULL}};
const Segment SGI82338[] = {{&S38622[96], 6}, {NULL}};
const Segment SGI82339[] = {{&S38623[44], 15}, {NULL}};
const Segment SGI82340[] = {{&S38623[44], 40}, {NULL}};
const Segment SGI82341[] = {{&S38623[30], 6}, {NULL}};
const Segment SGI82342[] = {{&S38623[31], 3}, {NULL}};
const Segment SGI82343[] = {{&S38623[98], 6}, {NULL}};
const Segment SGI82344[] = {{&S38624[44], 14}, {NULL}};
const Segment SGI82345[] = {{&S38624[44], 38}, {NULL}};
const Segment SGI82346[] = {{&S38624[30], 6}, {NULL}};
const Segment SGI82347[] = {{&S38624[31], 3}, {NULL}};
const Segment SGI82348[] = {{&S38624[96], 6}, {NULL}};
const Segment SGI82349[] = {{&S38625[44], 14}, {NULL}};
const Segment SGI82350[] = {{&S38625[44], 38}, {NULL}};
const Segment SGI82351[] = {{&S38625[30], 6}, {NULL}};
const Segment SGI82352[] = {{&S38625[31], 3}, {NULL}};
const Segment SGI82353[] = {{&S38625[96], 6}, {NULL}};
const Segment SGI82354[] = {{&S38626[38], 14}, {NULL}};
const Segment SGI82355[] = {{&S38626[31], 6}, {NULL}};
const Segment SGI82356[] = {{&S38627[44], 15}, {NULL}};
const Segment SGI82357[] = {{&S38627[44], 40}, {NULL}};
const Segment SGI82358[] = {{&S38627[30], 6}, {NULL}};
const Segment SGI82359[] = {{&S38627[31], 3}, {NULL}};
const Segment SGI82360[] = {{&S38627[98], 6}, {NULL}};
const Segment SGI82361[] = {{&S38628[38], 14}, {NULL}};
const Segment SGI82362[] = {{&S38628[31], 6}, {NULL}};
const Segment SGI82363[] = {{&S38629[44], 21}, {NULL}};
const Segment SGI82364[] = {{&S38629[44], 38}, {NULL}};
const Segment SGI82365[] = {{&S38629[30], 6}, {NULL}};
const Segment SGI82366[] = {{&S38629[31], 3}, {NULL}};
const Segment SGI82367[] = {{&S38629[96], 6}, {NULL}};
const Segment SGI82368[] = {{&S38630[44], 14}, {NULL}};
const Segment SGI82369[] = {{&S38630[44], 31}, {NULL}};
const Segment SGI82370[] = {{&S38630[30], 6}, {NULL}};
const Segment SGI82371[] = {{&S38630[92], 6}, {NULL}};
const Segment SGI82372[] = {{&S38631[44], 14}, {NULL}};
const Segment SGI82373[] = {{&S38631[44], 38}, {NULL}};
const Segment SGI82374[] = {{&S38631[30], 6}, {NULL}};
const Segment SGI82375[] = {{&S38631[31], 3}, {NULL}};
const Segment SGI82376[] = {{&S38631[96], 6}, {NULL}};
const Segment SGI82377[] = {{&S38632[44], 15}, {NULL}};
const Segment SGI82378[] = {{&S38632[44], 40}, {NULL}};
const Segment SGI82379[] = {{&S38632[30], 6}, {NULL}};
const Segment SGI82380[] = {{&S38632[31], 3}, {NULL}};
const Segment SGI82381[] = {{&S38632[98], 6}, {NULL}};
const Segment SGI82382[] = {{&S38633[44], 13}, {NULL}};
const Segment SGI82383[] = {{&S38633[44], 30}, {NULL}};
const Segment SGI82384[] = {{&S38633[30], 6}, {NULL}};
const Segment SGI82385[] = {{&S38633[31], 3}, {NULL}};
const Segment SGI82386[] = {{&S38633[88], 6}, {NULL}};
const Segment SGI82387[] = {{&S38634[44], 13}, {NULL}};
const Segment SGI82388[] = {{&S38634[44], 37}, {NULL}};
const Segment SGI82389[] = {{&S38634[30], 6}, {NULL}};
const Segment SGI82390[] = {{&S38634[31], 3}, {NULL}};
const Segment SGI82391[] = {{&S38634[95], 6}, {NULL}};
const Segment SGI82392[] = {{&S38635[38], 15}, {NULL}};
const Segment SGI82393[] = {{&S38635[31], 6}, {NULL}};
const Segment SGI82394[] = {{&S38636[44], 10}, {NULL}};
const Segment SGI82395[] = {{&S38636[44], 27}, {NULL}};
const Segment SGI82396[] = {{&S38636[30], 6}, {NULL}};
const Segment SGI82397[] = {{&S38636[31], 3}, {NULL}};
const Segment SGI82398[] = {{&S38636[85], 6}, {NULL}};
const Segment SGI82399[] = {{&S50582[42], 4}, {NULL}};
const Segment SGI82400[] = {{&S50582[42], 15}, {NULL}};
const Segment SGI82401[] = {{&S50582[30], 4}, {NULL}};
const Segment SGI82402[] = {{&S50582[105], 3}, {NULL}};
const Segment SGI82403[] = {{&S50582[71], 6}, {NULL}};
const Segment SGI82404[] = {{&S50583[42], 4}, {NULL}};
const Segment SGI82405[] = {{&S50583[42], 28}, {NULL}};
const Segment SGI82406[] = {{&S50583[30], 4}, {NULL}};
const Segment SGI82407[] = {{&S50583[119], 3}, {NULL}};
const Segment SGI82408[] = {{&S50583[84], 6}, {NULL}};
const Segment SGI82409[] = {{&S50584[42], 4}, {NULL}};
const Segment SGI82410[] = {{&S50584[42], 28}, {NULL}};
const Segment SGI82411[] = {{&S50584[30], 4}, {NULL}};
const Segment SGI82412[] = {{&S50584[119], 3}, {NULL}};
const Segment SGI82413[] = {{&S50584[84], 6}, {NULL}};
const Segment SGI82414[] = {{&S50585[42], 4}, {NULL}};
const Segment SGI82415[] = {{&S50585[42], 28}, {NULL}};
const Segment SGI82416[] = {{&S50585[30], 4}, {NULL}};
const Segment SGI82417[] = {{&S50585[119], 3}, {NULL}};
const Segment SGI82418[] = {{&S50585[84], 6}, {NULL}};
const Segment SGI82419[] = {{&S52135[44], 7}, {NULL}};
const Segment SGI82420[] = {{&S52135[44], 18}, {NULL}};
const Segment SGI82421[] = {{&S52135[30], 6}, {NULL}};
const Segment SGI82422[] = {{&S52135[110], 3}, {NULL}};
const Segment SGI82423[] = {{&S52135[76], 5}, {NULL}};
const Segment SGI82424[] = {{&S52137[38], 7}, {NULL}};
const Segment SGI82425[] = {{&S52137[31], 6}, {NULL}};
const Segment SGI82426[] = {{&S52138[44], 7}, {NULL}};
const Segment SGI82427[] = {{&S52138[44], 18}, {NULL}};
const Segment SGI82428[] = {{&S52138[30], 6}, {NULL}};
const Segment SGI82429[] = {{&S52138[79], 4}, {NULL}};
const Segment SGI82430[] = {{&S52138[79], 3}, {NULL}};
const Segment SGI82431[] = {{&S52141[44], 7}, {NULL}};
const Segment SGI82432[] = {{&S52141[44], 18}, {NULL}};
const Segment SGI82433[] = {{&S52141[30], 6}, {NULL}};
const Segment SGI82434[] = {{&S52141[79], 6}, {NULL}};
const Segment SGI82435[] = {{&S50586[42], 4}, {NULL}};
const Segment SGI82436[] = {{&S50586[42], 28}, {NULL}};
const Segment SGI82437[] = {{&S50586[30], 4}, {NULL}};
const Segment SGI82438[] = {{&S50586[119], 3}, {NULL}};
const Segment SGI82439[] = {{&S50586[84], 6}, {NULL}};
const Segment SGI82440[] = {{&S50587[42], 4}, {NULL}};
const Segment SGI82441[] = {{&S50587[42], 28}, {NULL}};
const Segment SGI82442[] = {{&S50587[30], 4}, {NULL}};
const Segment SGI82443[] = {{&S50587[119], 3}, {NULL}};
const Segment SGI82444[] = {{&S50587[84], 6}, {NULL}};
const Segment SGI82445[] = {{&S50588[42], 4}, {NULL}};
const Segment SGI82446[] = {{&S50588[42], 28}, {NULL}};
const Segment SGI82447[] = {{&S50588[30], 4}, {NULL}};
const Segment SGI82448[] = {{&S50588[119], 3}, {NULL}};
const Segment SGI82449[] = {{&S50588[84], 6}, {NULL}};
const Segment SGI82450[] = {{&S50589[42], 4}, {NULL}};
const Segment SGI82451[] = {{&S50589[42], 28}, {NULL}};
const Segment SGI82452[] = {{&S50589[30], 4}, {NULL}};
const Segment SGI82453[] = {{&S50589[119], 3}, {NULL}};
const Segment SGI82454[] = {{&S50589[84], 6}, {NULL}};
const Segment SGI82455[] = {{&S50590[36], 4}, {NULL}};
const Segment SGI82456[] = {{&S50590[31], 4}, {NULL}};
const Segment SGI82457[] = {{&S50591[42], 4}, {NULL}};
const Segment SGI82458[] = {{&S50591[42], 28}, {NULL}};
const Segment SGI82459[] = {{&S50591[30], 4}, {NULL}};
const Segment SGI82460[] = {{&S50591[119], 3}, {NULL}};
const Segment SGI82461[] = {{&S50591[84], 6}, {NULL}};
const Segment SGI82462[] = {{&S38772[34], 10}, {NULL}};
const Segment SGI82463[] = {{&S38772[27], 6}, {NULL}};
const Segment SGI82464[] = {{&S38772[97], 9}, {NULL}};
const Segment SGI82465[] = {{&S38772[70], 6}, {NULL}};
const Segment SGI82466[] = {{&S38774[34], 11}, {NULL}};
const Segment SGI82467[] = {{&S38774[27], 6}, {NULL}};
const Segment SGI82468[] = {{&S38774[98], 9}, {NULL}};
const Segment SGI82469[] = {{&S38774[71], 6}, {NULL}};
const Segment SGI82470[] = {{&S38776[34], 11}, {NULL}};
const Segment SGI82471[] = {{&S38776[27], 6}, {NULL}};
const Segment SGI82472[] = {{&S38776[98], 9}, {NULL}};
const Segment SGI82473[] = {{&S38776[71], 6}, {NULL}};
const Segment SGI82474[] = {{&S38777[34], 11}, {NULL}};
const Segment SGI82475[] = {{&S38777[27], 6}, {NULL}};
const Segment SGI82476[] = {{&S38777[98], 9}, {NULL}};
const Segment SGI82477[] = {{&S38777[71], 6}, {NULL}};
const Segment SGI82478[] = {{&S38780[34], 10}, {NULL}};
const Segment SGI82479[] = {{&S38780[27], 6}, {NULL}};
const Segment SGI82480[] = {{&S38780[97], 9}, {NULL}};
const Segment SGI82481[] = {{&S38780[70], 6}, {NULL}};
const Segment SGI82482[] = {{&S52270[38], 12}, {NULL}};
const Segment SGI82483[] = {{&S52270[31], 6}, {NULL}};
const Segment SGI82484[] = {{&S52271[44], 12}, {NULL}};
const Segment SGI82485[] = {{&S52271[44], 23}, {NULL}};
const Segment SGI82486[] = {{&S52271[30], 6}, {NULL}};
const Segment SGI82487[] = {{&S52271[115], 3}, {NULL}};
const Segment SGI82488[] = {{&S52271[81], 5}, {NULL}};
const Segment SGI82489[] = {{&S38830[34], 19}, {NULL}};
const Segment SGI82490[] = {{&S38830[27], 6}, {NULL}};
const Segment SGI82491[] = {{&S38830[106], 9}, {NULL}};
const Segment SGI82492[] = {{&S38830[79], 6}, {NULL}};
const Segment SGI82493[] = {{&S38831[34], 11}, {NULL}};
const Segment SGI82494[] = {{&S38831[27], 6}, {NULL}};
const Segment SGI82495[] = {{&S38831[98], 9}, {NULL}};
const Segment SGI82496[] = {{&S38831[71], 6}, {NULL}};
const Segment SGI82497[] = {{&S38834[34], 10}, {NULL}};
const Segment SGI82498[] = {{&S38834[27], 6}, {NULL}};
const Segment SGI82499[] = {{&S38834[97], 9}, {NULL}};
const Segment SGI82500[] = {{&S38834[70], 6}, {NULL}};
const Segment SGI82501[] = {{&S38835[34], 18}, {NULL}};
const Segment SGI82502[] = {{&S38835[27], 6}, {NULL}};
const Segment SGI82503[] = {{&S38835[105], 9}, {NULL}};
const Segment SGI82504[] = {{&S38835[78], 6}, {NULL}};
const Segment SGI82505[] = {{&S38837[34], 11}, {NULL}};
const Segment SGI82506[] = {{&S38837[27], 6}, {NULL}};
const Segment SGI82507[] = {{&S38837[98], 9}, {NULL}};
const Segment SGI82508[] = {{&S38837[71], 6}, {NULL}};
const Segment SGI82509[] = {{&S38840[34], 10}, {NULL}};
const Segment SGI82510[] = {{&S38840[27], 6}, {NULL}};
const Segment SGI82511[] = {{&S38840[97], 9}, {NULL}};
const Segment SGI82512[] = {{&S38840[70], 6}, {NULL}};
const Segment SGI82513[] = {{&S52272[44], 12}, {NULL}};
const Segment SGI82514[] = {{&S52272[44], 23}, {NULL}};
const Segment SGI82515[] = {{&S52272[30], 6}, {NULL}};
const Segment SGI82516[] = {{&S52272[115], 3}, {NULL}};
const Segment SGI82517[] = {{&S52272[81], 5}, {NULL}};
const Segment SGI82518[] = {{&S52375[44], 12}, {NULL}};
const Segment SGI82519[] = {{&S52375[44], 23}, {NULL}};
const Segment SGI82520[] = {{&S52375[30], 6}, {NULL}};
const Segment SGI82521[] = {{&S52375[134], 3}, {NULL}};
const Segment SGI82522[] = {{&S52375[100], 5}, {NULL}};
const Segment SGI82523[] = {{&S52375[35], 1}, {NULL}};
const Segment SGI82524[] = {{&S52375[69], 11}, {NULL}};
const Segment SGI82525[] = {{&S52376[44], 12}, {NULL}};
const Segment SGI82526[] = {{&S52376[44], 23}, {NULL}};
const Segment SGI82527[] = {{&S52376[30], 6}, {NULL}};
const Segment SGI82528[] = {{&S52376[125], 3}, {NULL}};
const Segment SGI82529[] = {{&S52376[91], 5}, {NULL}};
const Segment SGI82530[] = {{&S52376[74], 3}, {NULL}};
const Segment SGI82531[] = {{&S38907[34], 11}, {NULL}};
const Segment SGI82532[] = {{&S38907[27], 6}, {NULL}};
const Segment SGI82533[] = {{&S38907[98], 9}, {NULL}};
const Segment SGI82534[] = {{&S38907[71], 6}, {NULL}};
const Segment SGI82535[] = {{&S38909[34], 10}, {NULL}};
const Segment SGI82536[] = {{&S38909[27], 6}, {NULL}};
const Segment SGI82537[] = {{&S38909[97], 9}, {NULL}};
const Segment SGI82538[] = {{&S38909[70], 6}, {NULL}};
const Segment SGI82539[] = {{&S38910[34], 11}, {NULL}};
const Segment SGI82540[] = {{&S38910[27], 6}, {NULL}};
const Segment SGI82541[] = {{&S38910[98], 9}, {NULL}};
const Segment SGI82542[] = {{&S38910[71], 6}, {NULL}};
const Segment SGI82543[] = {{&S38912[34], 11}, {NULL}};
const Segment SGI82544[] = {{&S38912[27], 6}, {NULL}};
const Segment SGI82545[] = {{&S38912[98], 9}, {NULL}};
const Segment SGI82546[] = {{&S38912[71], 6}, {NULL}};
const Segment SGI82547[] = {{&S38913[34], 11}, {NULL}};
const Segment SGI82548[] = {{&S38913[27], 6}, {NULL}};
const Segment SGI82549[] = {{&S38913[98], 9}, {NULL}};
const Segment SGI82550[] = {{&S38913[71], 6}, {NULL}};
const Segment SGI82551[] = {{&S38915[34], 11}, {NULL}};
const Segment SGI82552[] = {{&S38915[27], 6}, {NULL}};
const Segment SGI82553[] = {{&S38915[98], 9}, {NULL}};
const Segment SGI82554[] = {{&S38915[71], 6}, {NULL}};
const Segment SGI82555[] = {{&S58716[44], 10}, {NULL}};
const Segment SGI82556[] = {{&S58716[44], 27}, {NULL}};
const Segment SGI82557[] = {{&S58716[30], 6}, {NULL}};
const Segment SGI82558[] = {{&S58716[119], 3}, {NULL}};
const Segment SGI82559[] = {{&S58716[85], 5}, {NULL}};
const Segment SGI82560[] = {{&S58717[42], 10}, {NULL}};
const Segment SGI82561[] = {{&S58717[30], 6}, {NULL}};
const Segment SGI82562[] = {{&S58717[117], 3}, {NULL}};
const Segment SGI82563[] = {{&S58717[83], 5}, {NULL}};
const Segment SGI82564[] = {{&S58718[44], 10}, {NULL}};
const Segment SGI82565[] = {{&S58718[44], 27}, {NULL}};
const Segment SGI82566[] = {{&S58718[30], 6}, {NULL}};
const Segment SGI82567[] = {{&S58718[119], 3}, {NULL}};
const Segment SGI82568[] = {{&S58718[85], 5}, {NULL}};
const Segment SGI82569[] = {{&S58779[44], 9}, {NULL}};
const Segment SGI82570[] = {{&S58779[44], 21}, {NULL}};
const Segment SGI82571[] = {{&S58779[30], 6}, {NULL}};
const Segment SGI82572[] = {{&S58779[113], 3}, {NULL}};
const Segment SGI82573[] = {{&S58779[79], 5}, {NULL}};
const Segment SGI82574[] = {{&S52392[44], 5}, {NULL}};
const Segment SGI82575[] = {{&S52392[44], 16}, {NULL}};
const Segment SGI82576[] = {{&S52392[30], 6}, {NULL}};
const Segment SGI82577[] = {{&S52392[108], 3}, {NULL}};
const Segment SGI82578[] = {{&S52392[74], 5}, {NULL}};
const Segment SGI82579[] = {{&S39055[34], 2}, {NULL}};
const Segment SGI82580[] = {{&S39055[27], 6}, {NULL}};
const Segment SGI82581[] = {{&S39055[89], 9}, {NULL}};
const Segment SGI82582[] = {{&S39055[62], 6}, {NULL}};
const Segment SGI82583[] = {{&S39058[34], 5}, {NULL}};
const Segment SGI82584[] = {{&S39058[27], 6}, {NULL}};
const Segment SGI82585[] = {{&S39058[92], 9}, {NULL}};
const Segment SGI82586[] = {{&S39058[65], 6}, {NULL}};
const Segment SGI82587[] = {{&S39059[11], 1}, {NULL}};
const Segment SGI82588[] = {{&S39059[27], 6}, {NULL}};
const Segment SGI82589[] = {{&S39059[88], 9}, {NULL}};
const Segment SGI82590[] = {{&S39059[61], 6}, {NULL}};
const Segment SGI82591[] = {{&S39062[34], 5}, {NULL}};
const Segment SGI82592[] = {{&S39062[27], 6}, {NULL}};
const Segment SGI82593[] = {{&S39062[92], 9}, {NULL}};
const Segment SGI82594[] = {{&S39062[65], 6}, {NULL}};
const Segment SGI82595[] = {{&S39064[11], 1}, {NULL}};
const Segment SGI82596[] = {{&S39064[27], 16}, {NULL}};
const Segment SGI82597[] = {{&S39064[98], 9}, {NULL}};
const Segment SGI82598[] = {{&S39064[71], 6}, {NULL}};
const Segment SGI82599[] = {{&S39065[34], 16}, {NULL}};
const Segment SGI82600[] = {{&S39065[27], 6}, {NULL}};
const Segment SGI82601[] = {{&S39065[103], 9}, {NULL}};
const Segment SGI82602[] = {{&S39065[76], 6}, {NULL}};
const Segment SGI82603[] = {{&S39066[34], 11}, {NULL}};
const Segment SGI82604[] = {{&S39066[27], 6}, {NULL}};
const Segment SGI82605[] = {{&S39066[98], 9}, {NULL}};
const Segment SGI82606[] = {{&S39066[71], 6}, {NULL}};
const Segment SGI82607[] = {{&S39067[34], 19}, {NULL}};
const Segment SGI82608[] = {{&S39067[27], 6}, {NULL}};
const Segment SGI82609[] = {{&S39067[106], 9}, {NULL}};
const Segment SGI82610[] = {{&S39067[79], 6}, {NULL}};
const Segment SGI82611[] = {{&S39068[34], 16}, {NULL}};
const Segment SGI82612[] = {{&S39068[27], 6}, {NULL}};
const Segment SGI82613[] = {{&S39068[103], 9}, {NULL}};
const Segment SGI82614[] = {{&S39068[76], 6}, {NULL}};
const Segment SGI82615[] = {{&S39069[34], 11}, {NULL}};
const Segment SGI82616[] = {{&S39069[27], 6}, {NULL}};
const Segment SGI82617[] = {{&S39069[98], 9}, {NULL}};
const Segment SGI82618[] = {{&S39069[71], 6}, {NULL}};
const Segment SGI82619[] = {{&S39070[34], 18}, {NULL}};
const Segment SGI82620[] = {{&S39070[27], 6}, {NULL}};
const Segment SGI82621[] = {{&S39070[105], 9}, {NULL}};
const Segment SGI82622[] = {{&S39070[78], 6}, {NULL}};
const Segment SGI82623[] = {{&S39071[34], 21}, {NULL}};
const Segment SGI82624[] = {{&S39071[27], 6}, {NULL}};
const Segment SGI82625[] = {{&S39071[108], 9}, {NULL}};
const Segment SGI82626[] = {{&S39071[81], 6}, {NULL}};
const Segment SGI82627[] = {{&S39074[34], 7}, {NULL}};
const Segment SGI82628[] = {{&S39074[27], 6}, {NULL}};
const Segment SGI82629[] = {{&S39074[94], 9}, {NULL}};
const Segment SGI82630[] = {{&S39074[67], 6}, {NULL}};
const Segment SGI82631[] = {{&S39076[34], 16}, {NULL}};
const Segment SGI82632[] = {{&S39076[27], 6}, {NULL}};
const Segment SGI82633[] = {{&S39076[103], 9}, {NULL}};
const Segment SGI82634[] = {{&S39076[76], 6}, {NULL}};
const Segment SGI82635[] = {{&S39078[34], 20}, {NULL}};
const Segment SGI82636[] = {{&S39078[27], 6}, {NULL}};
const Segment SGI82637[] = {{&S39078[107], 9}, {NULL}};
const Segment SGI82638[] = {{&S39078[80], 6}, {NULL}};
const Segment SGI82639[] = {{&S39079[11], 1}, {NULL}};
const Segment SGI82640[] = {{&S39079[27], 6}, {NULL}};
const Segment SGI82641[] = {{&S39079[94], 9}, {NULL}};
const Segment SGI82642[] = {{&S39079[67], 6}, {NULL}};
const Segment SGI82643[] = {{&S39080[34], 11}, {NULL}};
const Segment SGI82644[] = {{&S39080[27], 6}, {NULL}};
const Segment SGI82645[] = {{&S39080[98], 9}, {NULL}};
const Segment SGI82646[] = {{&S39080[71], 6}, {NULL}};
const Segment SGI82647[] = {{&S39081[34], 21}, {NULL}};
const Segment SGI82648[] = {{&S39081[27], 6}, {NULL}};
const Segment SGI82649[] = {{&S39081[108], 9}, {NULL}};
const Segment SGI82650[] = {{&S39081[81], 6}, {NULL}};
const Segment SGI82651[] = {{&S39082[34], 14}, {NULL}};
const Segment SGI82652[] = {{&S39082[27], 6}, {NULL}};
const Segment SGI82653[] = {{&S39082[101], 9}, {NULL}};
const Segment SGI82654[] = {{&S39082[74], 6}, {NULL}};
const Segment SGI82655[] = {{&S39083[34], 6}, {NULL}};
const Segment SGI82656[] = {{&S39083[27], 6}, {NULL}};
const Segment SGI82657[] = {{&S39083[93], 9}, {NULL}};
const Segment SGI82658[] = {{&S39083[66], 6}, {NULL}};
const Segment SGI82659[] = {{&S39084[34], 6}, {NULL}};
const Segment SGI82660[] = {{&S39084[27], 6}, {NULL}};
const Segment SGI82661[] = {{&S39084[93], 9}, {NULL}};
const Segment SGI82662[] = {{&S39084[66], 6}, {NULL}};
const Segment SGI82663[] = {{&S39085[34], 11}, {NULL}};
const Segment SGI82664[] = {{&S39085[27], 6}, {NULL}};
const Segment SGI82665[] = {{&S39085[98], 9}, {NULL}};
const Segment SGI82666[] = {{&S39085[71], 6}, {NULL}};
const Segment SGI82667[] = {{&S52470[44], 7}, {NULL}};
const Segment SGI82668[] = {{&S39087[34], 10}, {NULL}};
const Segment SGI82669[] = {{&S52470[44], 19}, {NULL}};
const Segment SGI82670[] = {{&S52470[30], 6}, {NULL}};
const Segment SGI82671[] = {{&S39087[27], 6}, {NULL}};
const Segment SGI82672[] = {{&S52470[111], 3}, {NULL}};
const Segment SGI82673[] = {{&S39087[97], 9}, {NULL}};
const Segment SGI82674[] = {{&S52470[77], 5}, {NULL}};
const Segment SGI82675[] = {{&S39087[70], 6}, {NULL}};
const Segment SGI82676[] = {{&S52471[44], 7}, {NULL}};
const Segment SGI82677[] = {{&S39088[34], 6}, {NULL}};
const Segment SGI82678[] = {{&S52471[44], 18}, {NULL}};
const Segment SGI82679[] = {{&S39088[27], 6}, {NULL}};
const Segment SGI82680[] = {{&S52471[30], 6}, {NULL}};
const Segment SGI82681[] = {{&S52471[110], 3}, {NULL}};
const Segment SGI82682[] = {{&S39088[93], 9}, {NULL}};
const Segment SGI82683[] = {{&S39088[66], 6}, {NULL}};
const Segment SGI82684[] = {{&S52471[76], 5}, {NULL}};
const Segment SGI82685[] = {{&S39089[34], 9}, {NULL}};
const Segment SGI82686[] = {{&S39089[27], 6}, {NULL}};
const Segment SGI82687[] = {{&S39089[96], 9}, {NULL}};
const Segment SGI82688[] = {{&S39089[69], 6}, {NULL}};
const Segment SGI82689[] = {{&S39091[34], 8}, {NULL}};
const Segment SGI82690[] = {{&S39091[27], 6}, {NULL}};
const Segment SGI82691[] = {{&S39091[95], 9}, {NULL}};
const Segment SGI82692[] = {{&S39091[68], 6}, {NULL}};
const Segment SGI82693[] = {{&S52478[44], 7}, {NULL}};
const Segment SGI82694[] = {{&S52478[44], 19}, {NULL}};
const Segment SGI82695[] = {{&S39093[34], 7}, {NULL}};
const Segment SGI82696[] = {{&S52478[30], 6}, {NULL}};
const Segment SGI82697[] = {{&S39093[27], 6}, {NULL}};
const Segment SGI82698[] = {{&S52478[130], 3}, {NULL}};
const Segment SGI82699[] = {{&S39093[94], 9}, {NULL}};
const Segment SGI82700[] = {{&S52478[96], 5}, {NULL}};
const Segment SGI82701[] = {{&S39093[67], 6}, {NULL}};
const Segment SGI82702[] = {{&S52479[38], 7}, {NULL}};
const Segment SGI82703[] = {{&S39094[34], 12}, {NULL}};
const Segment SGI82704[] = {{&S52479[31], 6}, {NULL}};
const Segment SGI82705[] = {{&S39094[27], 6}, {NULL}};
const Segment SGI82706[] = {{&S39094[99], 9}, {NULL}};
const Segment SGI82707[] = {{&S39094[72], 6}, {NULL}};
const Segment SGI82708[] = {{&S39095[34], 14}, {NULL}};
const Segment SGI82709[] = {{&S39095[27], 6}, {NULL}};
const Segment SGI82710[] = {{&S39095[101], 9}, {NULL}};
const Segment SGI82711[] = {{&S39095[74], 6}, {NULL}};
const Segment SGI82712[] = {{&S52483[44], 7}, {NULL}};
const Segment SGI82713[] = {{&S52483[44], 19}, {NULL}};
const Segment SGI82714[] = {{&S52483[30], 6}, {NULL}};
const Segment SGI82715[] = {{&S52483[118], 3}, {NULL}};
const Segment SGI82716[] = {{&S52483[84], 5}, {NULL}};
const Segment SGI82717[] = {{&S39098[34], 16}, {NULL}};
const Segment SGI82718[] = {{&S39098[27], 6}, {NULL}};
const Segment SGI82719[] = {{&S39098[103], 9}, {NULL}};
const Segment SGI82720[] = {{&S39098[76], 6}, {NULL}};
const Segment SGI82721[] = {{&S39099[34], 6}, {NULL}};
const Segment SGI82722[] = {{&S39099[27], 6}, {NULL}};
const Segment SGI82723[] = {{&S39099[93], 9}, {NULL}};
const Segment SGI82724[] = {{&S39099[66], 6}, {NULL}};
const Segment SGI82725[] = {{&S39104[34], 7}, {NULL}};
const Segment SGI82726[] = {{&S39104[27], 6}, {NULL}};
const Segment SGI82727[] = {{&S39104[94], 9}, {NULL}};
const Segment SGI82728[] = {{&S39104[67], 6}, {NULL}};
const Segment SGI82729[] = {{&S39107[34], 12}, {NULL}};
const Segment SGI82730[] = {{&S39107[27], 6}, {NULL}};
const Segment SGI82731[] = {{&S39107[99], 9}, {NULL}};
const Segment SGI82732[] = {{&S39107[72], 6}, {NULL}};
const Segment SGI82733[] = {{&S39109[34], 24}, {NULL}};
const Segment SGI82734[] = {{&S39109[27], 6}, {NULL}};
const Segment SGI82735[] = {{&S39109[111], 9}, {NULL}};
const Segment SGI82736[] = {{&S39109[84], 6}, {NULL}};
const Segment SGI82737[] = {{&S39115[34], 6}, {NULL}};
const Segment SGI82738[] = {{&S39115[27], 6}, {NULL}};
const Segment SGI82739[] = {{&S39115[93], 9}, {NULL}};
const Segment SGI82740[] = {{&S39115[66], 6}, {NULL}};
const Segment SGI82741[] = {{&S39117[34], 9}, {NULL}};
const Segment SGI82742[] = {{&S39117[27], 6}, {NULL}};
const Segment SGI82743[] = {{&S39117[96], 9}, {NULL}};
const Segment SGI82744[] = {{&S39117[69], 6}, {NULL}};
const Segment SGI82745[] = {{&S52509[44], 9}, {NULL}};
const Segment SGI82746[] = {{&S52509[44], 20}, {NULL}};
const Segment SGI82747[] = {{&S52509[30], 6}, {NULL}};
const Segment SGI82748[] = {{&S52509[127], 3}, {NULL}};
const Segment SGI82749[] = {{&S52509[93], 5}, {NULL}};
const Segment SGI82750[] = {{&S52509[35], 1}, {NULL}};
const Segment SGI82751[] = {{&S52509[66], 11}, {NULL}};
const Segment SGI82752[] = {{&S39124[34], 8}, {NULL}};
const Segment SGI82753[] = {{&S39124[27], 6}, {NULL}};
const Segment SGI82754[] = {{&S39124[95], 9}, {NULL}};
const Segment SGI82755[] = {{&S39124[68], 6}, {NULL}};
const Segment SGI82756[] = {{&S39126[34], 25}, {NULL}};
const Segment SGI82757[] = {{&S39126[27], 6}, {NULL}};
const Segment SGI82758[] = {{&S39126[112], 9}, {NULL}};
const Segment SGI82759[] = {{&S39126[85], 6}, {NULL}};
const Segment SGI82760[] = {{&S39128[34], 6}, {NULL}};
const Segment SGI82761[] = {{&S39128[27], 6}, {NULL}};
const Segment SGI82762[] = {{&S39128[94], 9}, {NULL}};
const Segment SGI82763[] = {{&S39128[67], 6}, {NULL}};
const Segment SGI82764[] = {{&S39129[30], 6}, {NULL}};
const Segment SGI82765[] = {{&S39129[84], 9}, {NULL}};
const Segment SGI82766[] = {{&S39129[57], 6}, {NULL}};
const Segment SGI82767[] = {{&S39130[34], 8}, {NULL}};
const Segment SGI82768[] = {{&S39130[27], 6}, {NULL}};
const Segment SGI82769[] = {{&S39130[95], 9}, {NULL}};
const Segment SGI82770[] = {{&S39130[68], 6}, {NULL}};
const Segment SGI82771[] = {{&S39131[34], 22}, {NULL}};
const Segment SGI82772[] = {{&S39131[27], 6}, {NULL}};
const Segment SGI82773[] = {{&S39131[109], 9}, {NULL}};
const Segment SGI82774[] = {{&S39131[82], 6}, {NULL}};
const Segment SGI82775[] = {{&S39134[34], 23}, {NULL}};
const Segment SGI82776[] = {{&S39134[27], 6}, {NULL}};
const Segment SGI82777[] = {{&S52510[44], 9}, {NULL}};
const Segment SGI82778[] = {{&S52510[44], 20}, {NULL}};
const Segment SGI82779[] = {{&S52510[30], 6}, {NULL}};
const Segment SGI82780[] = {{&S52510[131], 3}, {NULL}};
const Segment SGI82781[] = {{&S52510[97], 5}, {NULL}};
const Segment SGI82782[] = {{&S52510[35], 1}, {NULL}};
const Segment SGI82783[] = {{&S52510[66], 11}, {NULL}};
const Segment SGI82784[] = {{&S52511[44], 9}, {NULL}};
const Segment SGI82785[] = {{&S52511[44], 20}, {NULL}};
const Segment SGI82786[] = {{&S52511[30], 6}, {NULL}};
const Segment SGI82787[] = {{&S52511[131], 3}, {NULL}};
const Segment SGI82788[] = {{&S52511[97], 5}, {NULL}};
const Segment SGI82789[] = {{&S52511[76], 3}, {NULL}};
const Segment SGI82790[] = {{&S39134[110], 9}, {NULL}};
const Segment SGI82791[] = {{&S39134[83], 6}, {NULL}};
const Segment SGI82792[] = {{&S39136[34], 3}, {NULL}};
const Segment SGI82793[] = {{&S39136[27], 6}, {NULL}};
const Segment SGI82794[] = {{&S39136[90], 9}, {NULL}};
const Segment SGI82795[] = {{&S39136[63], 6}, {NULL}};
const Segment SGI82796[] = {{&S52568[44], 10}, {NULL}};
const Segment SGI82797[] = {{&S52568[44], 21}, {NULL}};
const Segment SGI82798[] = {{&S52568[30], 6}, {NULL}};
const Segment SGI82799[] = {{&S52568[132], 3}, {NULL}};
const Segment SGI82800[] = {{&S52568[98], 5}, {NULL}};
const Segment SGI82801[] = {{&S52568[35], 1}, {NULL}};
const Segment SGI82802[] = {{&S52568[67], 11}, {NULL}};
const Segment SGI82803[] = {{&S52569[44], 10}, {NULL}};
const Segment SGI82804[] = {{&S52569[44], 35}, {NULL}};
const Segment SGI82805[] = {{&S52569[30], 6}, {NULL}};
const Segment SGI82806[] = {{&S52569[146], 3}, {NULL}};
const Segment SGI82807[] = {{&S52569[112], 5}, {NULL}};
const Segment SGI82808[] = {{&S52569[35], 1}, {NULL}};
const Segment SGI82809[] = {{&S52569[81], 11}, {NULL}};
const Segment SGI82810[] = {{&S52570[44], 10}, {NULL}};
const Segment SGI82811[] = {{&S52570[44], 21}, {NULL}};
const Segment SGI82812[] = {{&S52570[30], 6}, {NULL}};
const Segment SGI82813[] = {{&S52570[132], 3}, {NULL}};
const Segment SGI82814[] = {{&S52570[98], 5}, {NULL}};
const Segment SGI82815[] = {{&S52570[35], 1}, {NULL}};
const Segment SGI82816[] = {{&S52570[67], 11}, {NULL}};
const Segment SGI82817[] = {{&S52571[44], 7}, {NULL}};
const Segment SGI82818[] = {{&S52571[44], 29}, {NULL}};
const Segment SGI82819[] = {{&S52571[30], 6}, {NULL}};
const Segment SGI82820[] = {{&S52571[140], 3}, {NULL}};
const Segment SGI82821[] = {{&S52571[106], 5}, {NULL}};
const Segment SGI82822[] = {{&S52571[35], 1}, {NULL}};
const Segment SGI82823[] = {{&S52571[75], 11}, {NULL}};
const Segment SGI82824[] = {{&S52572[44], 7}, {NULL}};
const Segment SGI82825[] = {{&S52572[44], 18}, {NULL}};
const Segment SGI82826[] = {{&S52572[30], 6}, {NULL}};
const Segment SGI82827[] = {{&S52572[129], 3}, {NULL}};
const Segment SGI82828[] = {{&S52572[95], 5}, {NULL}};
const Segment SGI82829[] = {{&S52572[35], 1}, {NULL}};
const Segment SGI82830[] = {{&S52572[64], 11}, {NULL}};
const Segment SGI82831[] = {{&S52649[44], 8}, {NULL}};
const Segment SGI82832[] = {{&S52649[44], 26}, {NULL}};
const Segment SGI82833[] = {{&S52649[30], 6}, {NULL}};
const Segment SGI82834[] = {{&S52649[118], 3}, {NULL}};
const Segment SGI82835[] = {{&S52649[84], 5}, {NULL}};
const Segment SGI82836[] = {{&S52651[44], 8}, {NULL}};
const Segment SGI82837[] = {{&S52651[44], 33}, {NULL}};
const Segment SGI82838[] = {{&S52651[30], 6}, {NULL}};
const Segment SGI82839[] = {{&S52651[125], 3}, {NULL}};
const Segment SGI82840[] = {{&S52651[91], 5}, {NULL}};
const Segment SGI82841[] = {{&S52652[38], 8}, {NULL}};
const Segment SGI82842[] = {{&S52652[31], 6}, {NULL}};
const Segment SGI82843[] = {{&S46160[74], 6}, {NULL}};
const Segment SGI82844[] = {{&S46160[0], 9}, {NULL}};
const Segment SGI82845[] = {{&S51230[44], 9}, {NULL}};
const Segment SGI82846[] = {{&S51230[44], 20}, {NULL}};
const Segment SGI82847[] = {{&S51230[30], 6}, {NULL}};
const Segment SGI82848[] = {{&S51230[113], 3}, {NULL}};
const Segment SGI82849[] = {{&S51230[78], 6}, {NULL}};
const Segment SGI82850[] = {{&S51231[36], 9}, {NULL}};
const Segment SGI82851[] = {{&S51231[29], 6}, {NULL}};
const Segment SGI82852[] = {{&S59238[44], 10}, {NULL}};
const Segment SGI82853[] = {{&S59238[44], 22}, {NULL}};
const Segment SGI82854[] = {{&S59238[30], 6}, {NULL}};
const Segment SGI82855[] = {{&S59238[114], 3}, {NULL}};
const Segment SGI82856[] = {{&S59238[80], 5}, {NULL}};
const Segment SGI82857[] = {{&S59316[44], 14}, {NULL}};
const Segment SGI82858[] = {{&S59316[44], 31}, {NULL}};
const Segment SGI82859[] = {{&S59316[30], 6}, {NULL}};
const Segment SGI82860[] = {{&S39534[34], 11}, {NULL}};
const Segment SGI82861[] = {{&S39534[27], 6}, {NULL}};
const Segment SGI82862[] = {{&S39534[98], 9}, {NULL}};
const Segment SGI82863[] = {{&S39534[71], 6}, {NULL}};
const Segment SGI82864[] = {{&S39537[34], 9}, {NULL}};
const Segment SGI82865[] = {{&S39537[27], 6}, {NULL}};
const Segment SGI82866[] = {{&S39537[96], 9}, {NULL}};
const Segment SGI82867[] = {{&S39537[69], 6}, {NULL}};
const Segment SGI82868[] = {{&S59316[123], 3}, {NULL}};
const Segment SGI82869[] = {{&S59316[89], 5}, {NULL}};
const Segment SGI82870[] = {{&S39540[34], 12}, {NULL}};
const Segment SGI82871[] = {{&S39540[27], 6}, {NULL}};
const Segment SGI82872[] = {{&S39540[98], 9}, {NULL}};
const Segment SGI82873[] = {{&S39540[72], 6}, {NULL}};
const Segment SGI82874[] = {{&S51363[42], 10}, {NULL}};
const Segment SGI82875[] = {{&S51363[42], 41}, {NULL}};
const Segment SGI82876[] = {{&S51363[30], 4}, {NULL}};
const Segment SGI82877[] = {{&S51363[131], 3}, {NULL}};
const Segment SGI82878[] = {{&S51363[97], 5}, {NULL}};
const Segment SGI82879[] = {{&S51364[42], 7}, {NULL}};
const Segment SGI82880[] = {{&S51364[42], 32}, {NULL}};
const Segment SGI82881[] = {{&S51364[30], 4}, {NULL}};
const Segment SGI82882[] = {{&S51364[123], 3}, {NULL}};
const Segment SGI82883[] = {{&S51364[88], 6}, {NULL}};
const Segment SGI82884[] = {{&S51365[42], 10}, {NULL}};
const Segment SGI82885[] = {{&S51365[42], 34}, {NULL}};
const Segment SGI82886[] = {{&S51365[30], 4}, {NULL}};
const Segment SGI82887[] = {{&S51365[124], 3}, {NULL}};
const Segment SGI82888[] = {{&S51365[90], 5}, {NULL}};
const Segment SGI82889[] = {{&S59363[57], 5}, {NULL}};
const Segment SGI82890[] = {{&S59363[57], 26}, {NULL}};
const Segment SGI82891[] = {{&S59363[43], 6}, {NULL}};
const Segment SGI82892[] = {{&S59363[131], 3}, {NULL}};
const Segment SGI82893[] = {{&S59363[97], 5}, {NULL}};
const Segment SGI82894[] = {{&S59363[0], 12}, {NULL}};
const Segment SGI82895[] = {{&S51366[44], 10}, {NULL}};
const Segment SGI82896[] = {{&S51366[44], 32}, {NULL}};
const Segment SGI82897[] = {{&S51366[30], 6}, {NULL}};
const Segment SGI82898[] = {{&S51366[124], 3}, {NULL}};
const Segment SGI82899[] = {{&S51366[90], 5}, {NULL}};
const Segment SGI82900[] = {{&S51367[42], 10}, {NULL}};
const Segment SGI82901[] = {{&S51367[42], 35}, {NULL}};
const Segment SGI82902[] = {{&S51367[30], 4}, {NULL}};
const Segment SGI82903[] = {{&S51367[125], 3}, {NULL}};
const Segment SGI82904[] = {{&S51367[91], 5}, {NULL}};
const Segment SGI82905[] = {{&S51368[44], 7}, {NULL}};
const Segment SGI82906[] = {{&S51368[44], 29}, {NULL}};
const Segment SGI82907[] = {{&S51368[30], 6}, {NULL}};
const Segment SGI82908[] = {{&S51368[122], 3}, {NULL}};
const Segment SGI82909[] = {{&S51368[87], 6}, {NULL}};
const Segment SGI82910[] = {{&S51369[38], 7}, {NULL}};
const Segment SGI82911[] = {{&S51369[31], 6}, {NULL}};
const Segment SGI82912[] = {{&S51370[44], 10}, {NULL}};
const Segment SGI82913[] = {{&S51370[44], 32}, {NULL}};
const Segment SGI82914[] = {{&S51370[30], 6}, {NULL}};
const Segment SGI82915[] = {{&S51370[124], 3}, {NULL}};
const Segment SGI82916[] = {{&S51370[90], 5}, {NULL}};
const Segment SGI82917[] = {{&S39667[34], 12}, {NULL}};
const Segment SGI82918[] = {{&S39667[27], 6}, {NULL}};
const Segment SGI82919[] = {{&S39667[99], 9}, {NULL}};
const Segment SGI82920[] = {{&S39667[72], 6}, {NULL}};
const Segment SGI82921[] = {{&S39668[34], 12}, {NULL}};
const Segment SGI82922[] = {{&S39668[27], 6}, {NULL}};
const Segment SGI82923[] = {{&S39668[99], 9}, {NULL}};
const Segment SGI82924[] = {{&S39668[72], 6}, {NULL}};
const Segment SGI82925[] = {{&S39669[34], 11}, {NULL}};
const Segment SGI82926[] = {{&S39669[27], 6}, {NULL}};
const Segment SGI82927[] = {{&S39669[98], 9}, {NULL}};
const Segment SGI82928[] = {{&S39669[71], 6}, {NULL}};
const Segment SGI82929[] = {{&S39670[34], 11}, {NULL}};
const Segment SGI82930[] = {{&S39670[27], 6}, {NULL}};
const Segment SGI82931[] = {{&S39670[98], 9}, {NULL}};
const Segment SGI82932[] = {{&S39670[71], 6}, {NULL}};
const Segment SGI82933[] = {{&S59473[44], 5}, {NULL}};
const Segment SGI82934[] = {{&S59473[44], 17}, {NULL}};
const Segment SGI82935[] = {{&S59473[30], 6}, {NULL}};
const Segment SGI82936[] = {{&S59473[109], 3}, {NULL}};
const Segment SGI82937[] = {{&S59473[75], 5}, {NULL}};
const Segment SGI82938[] = {{&S59475[38], 5}, {NULL}};
const Segment SGI82939[] = {{&S59475[31], 6}, {NULL}};
const Segment SGI82940[] = {{&S44014[34], 14}, {NULL}};
const Segment SGI82941[] = {{&S44014[27], 6}, {NULL}};
const Segment SGI82942[] = {{&S44014[101], 9}, {NULL}};
const Segment SGI82943[] = {{&S44014[74], 6}, {NULL}};
const Segment SGI82944[] = {{&S44017[34], 15}, {NULL}};
const Segment SGI82945[] = {{&S44017[27], 6}, {NULL}};
const Segment SGI82946[] = {{&S44017[102], 9}, {NULL}};
const Segment SGI82947[] = {{&S44017[75], 6}, {NULL}};
const Segment SGI82948[] = {{&S44021[34], 10}, {NULL}};
const Segment SGI82949[] = {{&S44021[27], 6}, {NULL}};
const Segment SGI82950[] = {{&S44021[97], 9}, {NULL}};
const Segment SGI82951[] = {{&S44021[70], 6}, {NULL}};
const Segment SGI82952[] = {{&S59577[44], 10}, {NULL}};
const Segment SGI82953[] = {{&S59577[44], 27}, {NULL}};
const Segment SGI82954[] = {{&S59577[30], 6}, {NULL}};
const Segment SGI82955[] = {{&S59577[149], 4}, {NULL}};
const Segment SGI82956[] = {{&S46736[44], 10}, {NULL}};
const Segment SGI82957[] = {{&S46736[44], 32}, {NULL}};
const Segment SGI82958[] = {{&S46736[30], 6}, {NULL}};
const Segment SGI82959[] = {{&S46736[31], 3}, {NULL}};
const Segment SGI82960[] = {{&S46736[90], 5}, {NULL}};
const Segment SGI82961[] = {{&S46736[0], 13}, {NULL}};
const Segment SGI82962[] = {{&S46738[38], 10}, {NULL}};
const Segment SGI82963[] = {{&S46738[31], 6}, {NULL}};
const Segment SGI82964[] = {{&S46738[0], 14}, {NULL}};
const Segment SGI82965[] = {{&S46740[44], 10}, {NULL}};
const Segment SGI82966[] = {{&S46740[44], 27}, {NULL}};
const Segment SGI82967[] = {{&S46740[30], 6}, {NULL}};
const Segment SGI82968[] = {{&S46740[31], 3}, {NULL}};
const Segment SGI82969[] = {{&S46740[85], 5}, {NULL}};
const Segment SGI82970[] = {{&S46740[0], 13}, {NULL}};
const Segment SGI82971[] = {{&S46741[42], 10}, {NULL}};
const Segment SGI82972[] = {{&S46741[30], 6}, {NULL}};
const Segment SGI82973[] = {{&S46741[31], 3}, {NULL}};
const Segment SGI82974[] = {{&S46741[88], 5}, {NULL}};
const Segment SGI82975[] = {{&S46741[0], 13}, {NULL}};
const Segment SGI82976[] = {{&S39915[44], 14}, {NULL}};
const Segment SGI82977[] = {{&S39915[44], 26}, {NULL}};
const Segment SGI82978[] = {{&S39915[30], 6}, {NULL}};
const Segment SGI82979[] = {{&S39915[31], 3}, {NULL}};
const Segment SGI82980[] = {{&S39915[84], 6}, {NULL}};
const Segment SGI82981[] = {{&S39916[38], 14}, {NULL}};
const Segment SGI82982[] = {{&S39916[31], 6}, {NULL}};
const Segment SGI82983[] = {{&S59577[175], 3}, {NULL}};
const Segment SGI82984[] = {{&S59577[119], 3}, {&S59577[224], 5}, {NULL}};
const Segment SGI82985[] = {{&S59577[85], 5}, {NULL}};
const Segment SGI82986[] = {{&S39951[44], 10}, {NULL}};
const Segment SGI82987[] = {{&S39951[44], 22}, {NULL}};
const Segment SGI82988[] = {{&S39951[30], 6}, {NULL}};
const Segment SGI82989[] = {{&S39951[31], 3}, {NULL}};
const Segment SGI82990[] = {{&S39951[80], 6}, {NULL}};
const Segment SGI82991[] = {{&S39952[38], 10}, {NULL}};
const Segment SGI82992[] = {{&S39952[31], 6}, {NULL}};
const Segment SGI82993[] = {{&S39953[44], 8}, {NULL}};
const Segment SGI82994[] = {{&S39953[44], 20}, {NULL}};
const Segment SGI82995[] = {{&S39953[30], 6}, {NULL}};
const Segment SGI82996[] = {{&S39953[31], 3}, {NULL}};
const Segment SGI82997[] = {{&S39953[78], 6}, {NULL}};
const Segment SGI82998[] = {{&S51813[42], 7}, {NULL}};
const Segment SGI82999[] = {{&S51813[42], 19}, {NULL}};
const Segment SGI83000[] = {{&S51813[30], 4}, {NULL}};
const Segment SGI83001[] = {{&S51813[110], 3}, {NULL}};
const Segment SGI83002[] = {{&S51813[75], 6}, {NULL}};
const Segment SGI83003[] = {{&S51814[44], 7}, {NULL}};
const Segment SGI83004[] = {{&S51814[44], 19}, {NULL}};
const Segment SGI83005[] = {{&S51814[30], 6}, {NULL}};
const Segment SGI83006[] = {{&S51814[112], 3}, {NULL}};
const Segment SGI83007[] = {{&S51814[77], 6}, {NULL}};
const Segment SGI83008[] = {{&S51815[36], 7}, {NULL}};
const Segment SGI83009[] = {{&S51815[31], 4}, {NULL}};
const Segment SGI83010[] = {{&S51816[38], 7}, {NULL}};
const Segment SGI83011[] = {{&S51816[31], 6}, {NULL}};
const Segment SGI83012[] = {{&S53072[44], 15}, {NULL}};
const Segment SGI83013[] = {{&S53072[44], 37}, {NULL}};
const Segment SGI83014[] = {{&S53072[30], 6}, {NULL}};
const Segment SGI83015[] = {{&S53072[129], 3}, {NULL}};
const Segment SGI83016[] = {{&S53072[95], 5}, {NULL}};
const Segment SGI83017[] = {{&S53073[44], 8}, {NULL}};
const Segment SGI83018[] = {{&S53073[44], 30}, {NULL}};
const Segment SGI83019[] = {{&S53073[30], 6}, {NULL}};
const Segment SGI83020[] = {{&S53073[122], 3}, {NULL}};
const Segment SGI83021[] = {{&S53073[88], 5}, {NULL}};
const Segment SGI83022[] = {{&S53074[44], 15}, {NULL}};
const Segment SGI83023[] = {{&S53074[44], 37}, {NULL}};
const Segment SGI83024[] = {{&S53074[30], 6}, {NULL}};
const Segment SGI83025[] = {{&S53074[129], 3}, {NULL}};
const Segment SGI83026[] = {{&S53074[95], 5}, {NULL}};
const Segment SGI83027[] = {{&S53075[44], 8}, {NULL}};
const Segment SGI83028[] = {{&S53075[44], 30}, {NULL}};
const Segment SGI83029[] = {{&S53075[30], 6}, {NULL}};
const Segment SGI83030[] = {{&S53075[122], 3}, {NULL}};
const Segment SGI83031[] = {{&S53075[88], 5}, {NULL}};
const Segment SGI83032[] = {{&S53076[44], 15}, {NULL}};
const Segment SGI83033[] = {{&S53076[44], 37}, {NULL}};
const Segment SGI83034[] = {{&S53076[30], 6}, {NULL}};
const Segment SGI83035[] = {{&S53076[129], 3}, {NULL}};
const Segment SGI83036[] = {{&S53076[95], 5}, {NULL}};
const Segment SGI83037[] = {{&S53077[44], 15}, {NULL}};
const Segment SGI83038[] = {{&S53077[44], 37}, {NULL}};
const Segment SGI83039[] = {{&S53077[30], 6}, {NULL}};
const Segment SGI83040[] = {{&S53077[129], 3}, {NULL}};
const Segment SGI83041[] = {{&S53077[95], 5}, {NULL}};
const Segment SGI83042[] = {{&S53078[44], 8}, {NULL}};
const Segment SGI83043[] = {{&S53078[44], 30}, {NULL}};
const Segment SGI83044[] = {{&S53078[30], 6}, {NULL}};
const Segment SGI83045[] = {{&S53078[122], 3}, {NULL}};
const Segment SGI83046[] = {{&S53078[88], 5}, {NULL}};
const Segment SGI83047[] = {{&S53079[38], 8}, {NULL}};
const Segment SGI83048[] = {{&S53079[31], 6}, {NULL}};
const Segment SGI83049[] = {{&S59773[45], 10}, {NULL}};
const Segment SGI83050[] = {{&S59773[30], 6}, {NULL}};
const Segment SGI83051[] = {{&S59773[120], 3}, {NULL}};
const Segment SGI83052[] = {{&S59773[86], 5}, {NULL}};
const Segment SGI83053[] = {{&S59773[144], 13}, {NULL}};
const Segment SGI83054[] = {{&S59774[44], 10}, {NULL}};
const Segment SGI83055[] = {{&S59774[44], 27}, {NULL}};
const Segment SGI83056[] = {{&S59774[30], 6}, {NULL}};
const Segment SGI83057[] = {{&S59774[120], 3}, {NULL}};
const Segment SGI83058[] = {{&S59774[85], 6}, {NULL}};
const Segment SGI83059[] = {{&S59774[140], 8}, {NULL}};
const Segment SGI83060[] = {{&S59774[149], 13}, {NULL}};
const Segment SGI83061[] = {{&S40207[44], 17}, {NULL}};
const Segment SGI83062[] = {{&S40207[44], 45}, {NULL}};
const Segment SGI83063[] = {{&S40207[30], 6}, {NULL}};
const Segment SGI83064[] = {{&S40207[31], 3}, {NULL}};
const Segment SGI83065[] = {{&S40207[103], 6}, {NULL}};
const Segment SGI83066[] = {{&S40208[38], 17}, {NULL}};
const Segment SGI83067[] = {{&S40208[31], 6}, {NULL}};
const Segment SGI83068[] = {{&S40209[44], 17}, {NULL}};
const Segment SGI83069[] = {{&S40209[44], 43}, {NULL}};
const Segment SGI83070[] = {{&S40209[30], 6}, {NULL}};
const Segment SGI83071[] = {{&S40209[31], 3}, {NULL}};
const Segment SGI83072[] = {{&S40209[101], 6}, {NULL}};
const Segment SGI83073[] = {{&S40210[44], 17}, {NULL}};
const Segment SGI83074[] = {{&S40210[44], 39}, {NULL}};
const Segment SGI83075[] = {{&S40210[30], 6}, {NULL}};
const Segment SGI83076[] = {{&S40210[31], 3}, {NULL}};
const Segment SGI83077[] = {{&S40210[97], 6}, {NULL}};
const Segment SGI83078[] = {{&S40368[44], 10}, {NULL}};
const Segment SGI83079[] = {{&S40368[44], 22}, {NULL}};
const Segment SGI83080[] = {{&S40368[30], 6}, {NULL}};
const Segment SGI83081[] = {{&S40368[31], 3}, {NULL}};
const Segment SGI83082[] = {{&S40368[80], 6}, {NULL}};
const Segment SGI83083[] = {{&S40373[44], 20}, {NULL}};
const Segment SGI83084[] = {{&S40373[44], 32}, {NULL}};
const Segment SGI83085[] = {{&S40373[30], 6}, {NULL}};
const Segment SGI83086[] = {{&S40373[31], 3}, {NULL}};
const Segment SGI83087[] = {{&S40373[90], 6}, {NULL}};
const Segment SGI83088[] = {{&S40374[38], 20}, {NULL}};
const Segment SGI83089[] = {{&S40374[31], 6}, {NULL}};
const Segment SGI83090[] = {{&S59905[44], 10}, {NULL}};
const Segment SGI83091[] = {{&S59905[44], 22}, {NULL}};
const Segment SGI83092[] = {{&S59905[30], 6}, {NULL}};
const Segment SGI83093[] = {{&S59905[114], 3}, {NULL}};
const Segment SGI83094[] = {{&S59905[80], 5}, {NULL}};
const Segment SGI83095[] = {{&S59906[44], 7}, {NULL}};
const Segment SGI83096[] = {{&S59906[44], 19}, {NULL}};
const Segment SGI83097[] = {{&S59906[30], 6}, {NULL}};
const Segment SGI83098[] = {{&S59906[111], 3}, {NULL}};
const Segment SGI83099[] = {{&S59906[77], 5}, {NULL}};
const Segment SGI83100[] = {{&S59907[44], 6}, {NULL}};
const Segment SGI83101[] = {{&S59907[44], 18}, {NULL}};
const Segment SGI83102[] = {{&S59907[30], 6}, {NULL}};
const Segment SGI83103[] = {{&S59907[79], 4}, {NULL}};
const Segment SGI83104[] = {{&S59907[79], 3}, {NULL}};
const Segment SGI83105[] = {{&S59908[44], 6}, {NULL}};
const Segment SGI83106[] = {{&S59908[44], 18}, {NULL}};
const Segment SGI83107[] = {{&S59908[30], 6}, {NULL}};
const Segment SGI83108[] = {{&S59908[79], 6}, {NULL}};
const Segment SGI83109[] = {{&S59909[44], 11}, {NULL}};
const Segment SGI83110[] = {{&S59909[44], 23}, {NULL}};
const Segment SGI83111[] = {{&S59909[30], 6}, {NULL}};
const Segment SGI83112[] = {{&S59909[115], 3}, {NULL}};
const Segment SGI83113[] = {{&S59909[81], 5}, {NULL}};
const Segment SGI83114[] = {{&S59910[38], 6}, {NULL}};
const Segment SGI83115[] = {{&S59910[31], 6}, {NULL}};
const Segment SGI83116[] = {{&S59911[38], 7}, {NULL}};
const Segment SGI83117[] = {{&S59911[31], 6}, {NULL}};
const Segment SGI83118[] = {{&S59924[45], 10}, {NULL}};
const Segment SGI83119[] = {{&S59924[30], 6}, {NULL}};
const Segment SGI83120[] = {{&S59924[120], 3}, {NULL}};
const Segment SGI83121[] = {{&S59924[86], 5}, {NULL}};
const Segment SGI83122[] = {{&S59924[144], 13}, {NULL}};
const Segment SGI83123[] = {{&S59961[44], 10}, {NULL}};
const Segment SGI83124[] = {{&S59961[44], 22}, {NULL}};
const Segment SGI83125[] = {{&S59961[30], 6}, {NULL}};
const Segment SGI83126[] = {{&S59961[114], 3}, {NULL}};
const Segment SGI83127[] = {{&S59961[80], 5}, {NULL}};
const Segment SGI83128[] = {{&S59962[38], 10}, {NULL}};
const Segment SGI83129[] = {{&S59962[31], 6}, {NULL}};
const Segment SGI83130[] = {{&S40497[44], 19}, {NULL}};
const Segment SGI83131[] = {{&S40497[44], 31}, {NULL}};
const Segment SGI83132[] = {{&S40497[30], 6}, {NULL}};
const Segment SGI83133[] = {{&S40497[31], 3}, {NULL}};
const Segment SGI83134[] = {{&S40497[89], 6}, {NULL}};
const Segment SGI83135[] = {{&S40498[44], 14}, {NULL}};
const Segment SGI83136[] = {{&S40498[44], 26}, {NULL}};
const Segment SGI83137[] = {{&S40498[30], 6}, {NULL}};
const Segment SGI83138[] = {{&S40498[31], 3}, {NULL}};
const Segment SGI83139[] = {{&S40498[84], 6}, {NULL}};
const Segment SGI83140[] = {{&S40499[44], 14}, {NULL}};
const Segment SGI83141[] = {{&S40499[44], 26}, {NULL}};
const Segment SGI83142[] = {{&S40499[30], 6}, {NULL}};
const Segment SGI83143[] = {{&S40499[87], 6}, {NULL}};
const Segment SGI83144[] = {{&S59993[44], 7}, {NULL}};
const Segment SGI83145[] = {{&S59993[44], 33}, {NULL}};
const Segment SGI83146[] = {{&S59993[30], 6}, {NULL}};
const Segment SGI83147[] = {{&S59993[125], 3}, {NULL}};
const Segment SGI83148[] = {{&S59993[91], 5}, {NULL}};
const Segment SGI83149[] = {{&S40513[38], 14}, {NULL}};
const Segment SGI83150[] = {{&S40513[31], 6}, {NULL}};
const Segment SGI83151[] = {{&S40549[44], 13}, {NULL}};
const Segment SGI83152[] = {{&S40549[44], 22}, {NULL}};
const Segment SGI83153[] = {{&S40549[30], 6}, {NULL}};
const Segment SGI83154[] = {{&S40549[31], 3}, {NULL}};
const Segment SGI83155[] = {{&S40549[80], 6}, {NULL}};
const Segment SGI83156[] = {{&S40550[38], 13}, {NULL}};
const Segment SGI83157[] = {{&S40550[31], 6}, {NULL}};
const Segment SGI83158[] = {{&S40558[44], 12}, {NULL}};
const Segment SGI83159[] = {{&S40558[44], 24}, {NULL}};
const Segment SGI83160[] = {{&S40558[30], 6}, {NULL}};
const Segment SGI83161[] = {{&S40558[31], 3}, {NULL}};
const Segment SGI83162[] = {{&S40558[82], 6}, {NULL}};
const Segment SGI83163[] = {{&S40559[44], 9}, {NULL}};
const Segment SGI83164[] = {{&S40559[44], 21}, {NULL}};
const Segment SGI83165[] = {{&S40559[30], 6}, {NULL}};
const Segment SGI83166[] = {{&S40559[31], 3}, {NULL}};
const Segment SGI83167[] = {{&S40559[79], 6}, {NULL}};
const Segment SGI83168[] = {{&S40560[38], 9}, {NULL}};
const Segment SGI83169[] = {{&S40560[31], 6}, {NULL}};
const Segment SGI83170[] = {{&S52421[42], 11}, {NULL}};
const Segment SGI83171[] = {{&S52421[42], 23}, {NULL}};
const Segment SGI83172[] = {{&S52421[30], 4}, {NULL}};
const Segment SGI83173[] = {{&S52421[114], 3}, {NULL}};
const Segment SGI83174[] = {{&S52421[79], 6}, {NULL}};
const Segment SGI83175[] = {{&S52422[42], 11}, {NULL}};
const Segment SGI83176[] = {{&S52422[42], 23}, {NULL}};
const Segment SGI83177[] = {{&S52422[30], 4}, {NULL}};
const Segment SGI83178[] = {{&S52422[114], 3}, {NULL}};
const Segment SGI83179[] = {{&S52422[79], 6}, {NULL}};
const Segment SGI83180[] = {{&S52423[42], 11}, {NULL}};
const Segment SGI83181[] = {{&S52423[42], 23}, {NULL}};
const Segment SGI83182[] = {{&S52423[30], 4}, {NULL}};
const Segment SGI83183[] = {{&S52423[114], 3}, {NULL}};
const Segment SGI83184[] = {{&S52423[79], 6}, {NULL}};
const Segment SGI83185[] = {{&S52424[42], 11}, {NULL}};
const Segment SGI83186[] = {{&S52424[42], 23}, {NULL}};
const Segment SGI83187[] = {{&S52424[30], 4}, {NULL}};
const Segment SGI83188[] = {{&S52424[114], 3}, {NULL}};
const Segment SGI83189[] = {{&S52424[79], 6}, {NULL}};
const Segment SGI83190[] = {{&S52425[42], 11}, {NULL}};
const Segment SGI83191[] = {{&S52425[42], 23}, {NULL}};
const Segment SGI83192[] = {{&S52425[30], 4}, {NULL}};
const Segment SGI83193[] = {{&S52425[114], 3}, {NULL}};
const Segment SGI83194[] = {{&S52425[79], 6}, {NULL}};
const Segment SGI83195[] = {{&S60013[44], 11}, {NULL}};
const Segment SGI83196[] = {{&S60013[44], 28}, {NULL}};
const Segment SGI83197[] = {{&S60013[30], 6}, {NULL}};
const Segment SGI83198[] = {{&S60013[120], 3}, {NULL}};
const Segment SGI83199[] = {{&S60013[86], 5}, {NULL}};
const Segment SGI83200[] = {{&S60014[38], 11}, {NULL}};
const Segment SGI83201[] = {{&S60014[31], 6}, {NULL}};
const Segment SGI83202[] = {{&S60058[44], 11}, {NULL}};
const Segment SGI83203[] = {{&S60058[44], 28}, {NULL}};
const Segment SGI83204[] = {{&S60058[30], 6}, {NULL}};
const Segment SGI83205[] = {{&S60058[120], 3}, {NULL}};
const Segment SGI83206[] = {{&S60058[86], 5}, {NULL}};
const Segment SGI83207[] = {{&S52426[42], 11}, {NULL}};
const Segment SGI83208[] = {{&S52426[42], 23}, {NULL}};
const Segment SGI83209[] = {{&S52426[30], 4}, {NULL}};
const Segment SGI83210[] = {{&S52426[114], 3}, {NULL}};
const Segment SGI83211[] = {{&S52426[79], 6}, {NULL}};
const Segment SGI83212[] = {{&S52427[36], 11}, {NULL}};
const Segment SGI83213[] = {{&S52427[31], 4}, {NULL}};
const Segment SGI83214[] = {{&S52507[44], 6}, {NULL}};
const Segment SGI83215[] = {{&S52507[44], 17}, {NULL}};
const Segment SGI83216[] = {{&S52507[30], 6}, {NULL}};
const Segment SGI83217[] = {{&S52507[110], 3}, {NULL}};
const Segment SGI83218[] = {{&S52507[75], 6}, {NULL}};
const Segment SGI83219[] = {{&S52508[38], 6}, {NULL}};
const Segment SGI83220[] = {{&S52508[31], 6}, {NULL}};
const Segment SGI83221[] = {{&S53563[44], 10}, {NULL}};
const Segment SGI83222[] = {{&S53563[44], 21}, {NULL}};
const Segment SGI83223[] = {{&S53563[30], 6}, {NULL}};
const Segment SGI83224[] = {{&S53563[132], 3}, {NULL}};
const Segment SGI83225[] = {{&S53563[98], 5}, {NULL}};
const Segment SGI83226[] = {{&S53563[35], 1}, {NULL}};
const Segment SGI83227[] = {{&S53563[67], 11}, {NULL}};
const Segment SGI83228[] = {{&S53564[44], 10}, {NULL}};
const Segment SGI83229[] = {{&S53564[44], 21}, {NULL}};
const Segment SGI83230[] = {{&S53564[30], 6}, {NULL}};
const Segment SGI83231[] = {{&S53564[148], 3}, {NULL}};
const Segment SGI83232[] = {{&S53564[114], 5}, {NULL}};
const Segment SGI83233[] = {{&S53564[72], 3}, {NULL}};
const Segment SGI83234[] = {{&S52573[42], 17}, {NULL}};
const Segment SGI83235[] = {{&S52573[42], 45}, {NULL}};
const Segment SGI83236[] = {{&S52573[30], 4}, {NULL}};
const Segment SGI83237[] = {{&S52573[136], 3}, {NULL}};
const Segment SGI83238[] = {{&S52573[101], 6}, {NULL}};
const Segment SGI83239[] = {{&S52574[36], 17}, {NULL}};
const Segment SGI83240[] = {{&S52574[31], 4}, {NULL}};
const Segment SGI83241[] = {{&S52575[42], 17}, {NULL}};
const Segment SGI83242[] = {{&S52575[42], 39}, {NULL}};
const Segment SGI83243[] = {{&S52575[30], 4}, {NULL}};
const Segment SGI83244[] = {{&S52575[130], 3}, {NULL}};
const Segment SGI83245[] = {{&S52575[95], 6}, {NULL}};
const Segment SGI83246[] = {{&S52576[42], 17}, {NULL}};
const Segment SGI83247[] = {{&S52576[42], 39}, {NULL}};
const Segment SGI83248[] = {{&S52576[30], 4}, {NULL}};
const Segment SGI83249[] = {{&S52576[130], 3}, {NULL}};
const Segment SGI83250[] = {{&S52576[95], 6}, {NULL}};
const Segment SGI83251[] = {{&S52577[42], 17}, {NULL}};
const Segment SGI83252[] = {{&S52577[42], 39}, {NULL}};
const Segment SGI83253[] = {{&S52577[30], 4}, {NULL}};
const Segment SGI83254[] = {{&S52577[130], 3}, {NULL}};
const Segment SGI83255[] = {{&S52577[95], 6}, {NULL}};
const Segment SGI83256[] = {{&S52578[42], 17}, {NULL}};
const Segment SGI83257[] = {{&S52578[42], 39}, {NULL}};
const Segment SGI83258[] = {{&S52578[30], 4}, {NULL}};
const Segment SGI83259[] = {{&S52578[130], 3}, {NULL}};
const Segment SGI83260[] = {{&S52578[95], 6}, {NULL}};
const Segment SGI83261[] = {{&S40765[44], 11}, {NULL}};
const Segment SGI83262[] = {{&S40765[44], 23}, {NULL}};
const Segment SGI83263[] = {{&S40765[30], 6}, {NULL}};
const Segment SGI83264[] = {{&S40765[31], 3}, {NULL}};
const Segment SGI83265[] = {{&S40765[81], 6}, {NULL}};
const Segment SGI83266[] = {{&S60264[44], 9}, {NULL}};
const Segment SGI83267[] = {{&S60264[44], 26}, {NULL}};
const Segment SGI83268[] = {{&S60264[30], 6}, {NULL}};
const Segment SGI83269[] = {{&S60264[118], 3}, {NULL}};
const Segment SGI83270[] = {{&S60264[84], 5}, {NULL}};
const Segment SGI83271[] = {{&S60265[42], 9}, {NULL}};
const Segment SGI83272[] = {{&S60265[30], 6}, {NULL}};
const Segment SGI83273[] = {{&S60265[116], 3}, {NULL}};
const Segment SGI83274[] = {{&S60265[82], 5}, {NULL}};
const Segment SGI83275[] = {{&S60266[38], 9}, {NULL}};
const Segment SGI83276[] = {{&S60266[31], 6}, {NULL}};
const Segment SGI83277[] = {{&S40766[44], 11}, {NULL}};
const Segment SGI83278[] = {{&S40766[44], 23}, {NULL}};
const Segment SGI83279[] = {{&S40766[30], 6}, {NULL}};
const Segment SGI83280[] = {{&S40766[31], 3}, {NULL}};
const Segment SGI83281[] = {{&S40766[81], 6}, {NULL}};
const Segment SGI83282[] = {{&S40767[44], 11}, {NULL}};
const Segment SGI83283[] = {{&S40767[44], 23}, {NULL}};
const Segment SGI83284[] = {{&S40767[30], 6}, {NULL}};
const Segment SGI83285[] = {{&S40767[31], 3}, {NULL}};
const Segment SGI83286[] = {{&S40767[81], 6}, {NULL}};
const Segment SGI83287[] = {{&S40772[38], 11}, {NULL}};
const Segment SGI83288[] = {{&S40772[31], 6}, {NULL}};
const Segment SGI83289[] = {{&S40773[44], 11}, {NULL}};
const Segment SGI83290[] = {{&S40773[44], 23}, {NULL}};
const Segment SGI83291[] = {{&S40773[30], 6}, {NULL}};
const Segment SGI83292[] = {{&S40773[31], 3}, {NULL}};
const Segment SGI83293[] = {{&S40773[81], 6}, {NULL}};
const Segment SGI83294[] = {{&S60306[44], 5}, {NULL}};
const Segment SGI83295[] = {{&S60306[44], 17}, {NULL}};
const Segment SGI83296[] = {{&S60306[30], 6}, {NULL}};
const Segment SGI83297[] = {{&S60306[109], 3}, {NULL}};
const Segment SGI83298[] = {{&S60306[75], 5}, {NULL}};
const Segment SGI83299[] = {{&S60307[38], 5}, {NULL}};
const Segment SGI83300[] = {{&S60307[31], 6}, {NULL}};
const Segment SGI83301[] = {{&S40828[44], 14}, {NULL}};
const Segment SGI83302[] = {{&S40828[44], 26}, {NULL}};
const Segment SGI83303[] = {{&S40828[30], 6}, {NULL}};
const Segment SGI83304[] = {{&S40828[31], 3}, {NULL}};
const Segment SGI83305[] = {{&S40828[84], 6}, {NULL}};
const Segment SGI83306[] = {{&S40829[44], 9}, {NULL}};
const Segment SGI83307[] = {{&S40829[44], 21}, {NULL}};
const Segment SGI83308[] = {{&S40829[30], 6}, {NULL}};
const Segment SGI83309[] = {{&S40829[31], 3}, {NULL}};
const Segment SGI83310[] = {{&S40829[79], 6}, {NULL}};
const Segment SGI83311[] = {{&S52763[42], 8}, {NULL}};
const Segment SGI83312[] = {{&S52763[42], 26}, {NULL}};
const Segment SGI83313[] = {{&S52763[30], 4}, {NULL}};
const Segment SGI83314[] = {{&S52763[116], 3}, {NULL}};
const Segment SGI83315[] = {{&S52763[82], 5}, {NULL}};
const Segment SGI83316[] = {{&S52764[42], 8}, {NULL}};
const Segment SGI83317[] = {{&S52764[42], 31}, {NULL}};
const Segment SGI83318[] = {{&S52764[30], 4}, {NULL}};
const Segment SGI83319[] = {{&S52764[121], 3}, {NULL}};
const Segment SGI83320[] = {{&S52764[87], 5}, {NULL}};
const Segment SGI83321[] = {{&S52765[42], 8}, {NULL}};
const Segment SGI83322[] = {{&S52765[42], 31}, {NULL}};
const Segment SGI83323[] = {{&S52765[30], 4}, {NULL}};
const Segment SGI83324[] = {{&S52765[122], 3}, {NULL}};
const Segment SGI83325[] = {{&S52765[87], 6}, {NULL}};
const Segment SGI83326[] = {{&S52766[36], 8}, {NULL}};
const Segment SGI83327[] = {{&S52766[31], 4}, {NULL}};
const Segment SGI83328[] = {{&S60473[45], 10}, {NULL}};
const Segment SGI83329[] = {{&S60473[30], 6}, {NULL}};
const Segment SGI83330[] = {{&S60473[120], 3}, {NULL}};
const Segment SGI83331[] = {{&S52767[44], 8}, {NULL}};
const Segment SGI83332[] = {{&S60473[86], 5}, {NULL}};
const Segment SGI83333[] = {{&S52767[44], 30}, {NULL}};
const Segment SGI83334[] = {{&S52767[30], 6}, {NULL}};
const Segment SGI83335[] = {{&S60473[144], 13}, {NULL}};
const Segment SGI83336[] = {{&S52767[122], 3}, {NULL}};
const Segment SGI83337[] = {{&S52767[88], 5}, {NULL}};
const Segment SGI83338[] = {{&S52768[42], 8}, {NULL}};
const Segment SGI83339[] = {{&S52768[42], 23}, {NULL}};
const Segment SGI83340[] = {{&S52768[30], 4}, {NULL}};
const Segment SGI83341[] = {{&S52768[113], 3}, {NULL}};
const Segment SGI83342[] = {{&S52768[79], 5}, {NULL}};
const Segment SGI83343[] = {{&S52769[44], 8}, {NULL}};
const Segment SGI83344[] = {{&S52769[44], 31}, {NULL}};
const Segment SGI83345[] = {{&S52769[30], 6}, {NULL}};
const Segment SGI83346[] = {{&S52769[123], 3}, {NULL}};
const Segment SGI83347[] = {{&S52769[89], 5}, {NULL}};
const Segment SGI83348[] = {{&S52770[44], 8}, {NULL}};
const Segment SGI83349[] = {{&S52770[44], 31}, {NULL}};
const Segment SGI83350[] = {{&S52770[30], 6}, {NULL}};
const Segment SGI83351[] = {{&S52770[123], 3}, {NULL}};
const Segment SGI83352[] = {{&S52770[89], 5}, {NULL}};
const Segment SGI83353[] = {{&S52771[44], 8}, {NULL}};
const Segment SGI83354[] = {{&S52771[44], 30}, {NULL}};
const Segment SGI83355[] = {{&S52771[30], 6}, {NULL}};
const Segment SGI83356[] = {{&S52771[122], 3}, {NULL}};
const Segment SGI83357[] = {{&S52771[88], 5}, {NULL}};
const Segment SGI83358[] = {{&S52772[44], 8}, {NULL}};
const Segment SGI83359[] = {{&S52772[44], 30}, {NULL}};
const Segment SGI83360[] = {{&S52772[30], 6}, {NULL}};
const Segment SGI83361[] = {{&S52772[122], 3}, {NULL}};
const Segment SGI83362[] = {{&S52772[88], 5}, {NULL}};
const Segment SGI83363[] = {{&S52773[44], 8}, {NULL}};
const Segment SGI83364[] = {{&S52773[44], 30}, {NULL}};
const Segment SGI83365[] = {{&S52773[30], 6}, {NULL}};
const Segment SGI83366[] = {{&S52773[122], 3}, {NULL}};
const Segment SGI83367[] = {{&S52773[88], 5}, {NULL}};
const Segment SGI83368[] = {{&S52774[44], 8}, {NULL}};
const Segment SGI83369[] = {{&S52774[44], 30}, {NULL}};
const Segment SGI83370[] = {{&S52774[30], 6}, {NULL}};
const Segment SGI83371[] = {{&S52774[123], 3}, {NULL}};
const Segment SGI83372[] = {{&S52774[88], 6}, {NULL}};
const Segment SGI83373[] = {{&S52775[44], 8}, {NULL}};
const Segment SGI83374[] = {{&S52775[44], 31}, {NULL}};
const Segment SGI83375[] = {{&S52775[30], 6}, {NULL}};
const Segment SGI83376[] = {{&S52775[124], 3}, {NULL}};
const Segment SGI83377[] = {{&S52775[89], 6}, {NULL}};
const Segment SGI83378[] = {{&S52776[38], 8}, {NULL}};
const Segment SGI83379[] = {{&S52776[31], 6}, {NULL}};
const Segment SGI83380[] = {{&S60528[44], 14}, {NULL}};
const Segment SGI83381[] = {{&S60528[44], 31}, {NULL}};
const Segment SGI83382[] = {{&S60528[30], 6}, {NULL}};
const Segment SGI83383[] = {{&S60528[123], 3}, {NULL}};
const Segment SGI83384[] = {{&S60528[89], 5}, {NULL}};
const Segment SGI83385[] = {{&S60529[44], 14}, {NULL}};
const Segment SGI83386[] = {{&S60529[44], 31}, {NULL}};
const Segment SGI83387[] = {{&S60529[30], 6}, {NULL}};
const Segment SGI83388[] = {{&S60529[92], 6}, {NULL}};
const Segment SGI83389[] = {{&S60530[38], 14}, {NULL}};
const Segment SGI83390[] = {{&S60530[31], 6}, {NULL}};
const Segment SGI83391[] = {{&S53839[44], 10}, {NULL}};
const Segment SGI83392[] = {{&S53839[44], 21}, {NULL}};
const Segment SGI83393[] = {{&S53839[30], 6}, {NULL}};
const Segment SGI83394[] = {{&S53839[113], 3}, {NULL}};
const Segment SGI83395[] = {{&S53839[79], 5}, {NULL}};
const Segment SGI83396[] = {{&S53841[44], 6}, {NULL}};
const Segment SGI83397[] = {{&S53841[44], 19}, {NULL}};
const Segment SGI83398[] = {{&S53841[30], 6}, {NULL}};
const Segment SGI83399[] = {{&S53841[111], 3}, {NULL}};
const Segment SGI83400[] = {{&S53841[77], 5}, {NULL}};
const Segment SGI83401[] = {{&S40985[44], 14}, {NULL}};
const Segment SGI83402[] = {{&S40985[44], 23}, {NULL}};
const Segment SGI83403[] = {{&S40985[30], 6}, {NULL}};
const Segment SGI83404[] = {{&S40985[31], 3}, {NULL}};
const Segment SGI83405[] = {{&S40985[81], 6}, {NULL}};
const Segment SGI83406[] = {{&S40986[38], 14}, {NULL}};
const Segment SGI83407[] = {{&S40986[31], 6}, {NULL}};
const Segment SGI83408[] = {{&S60698[44], 10}, {NULL}};
const Segment SGI83409[] = {{&S41114[44], 11}, {NULL}};
const Segment SGI83410[] = {{&S41114[44], 23}, {NULL}};
const Segment SGI83411[] = {{&S41114[30], 6}, {NULL}};
const Segment SGI83412[] = {{&S41114[31], 3}, {NULL}};
const Segment SGI83413[] = {{&S41114[81], 6}, {NULL}};
const Segment SGI83414[] = {{&S41123[38], 11}, {NULL}};
const Segment SGI83415[] = {{&S41123[31], 6}, {NULL}};
const Segment SGI83416[] = {{&S60698[44], 22}, {NULL}};
const Segment SGI83417[] = {{&S60698[30], 6}, {NULL}};
const Segment SGI83418[] = {{&S60698[114], 3}, {NULL}};
const Segment SGI83419[] = {{&S60698[80], 5}, {NULL}};
const Segment SGI83420[] = {{&S60699[38], 10}, {NULL}};
const Segment SGI83421[] = {{&S60699[31], 6}, {NULL}};
const Segment SGI83422[] = {{&S41204[34], 10}, {NULL}};
const Segment SGI83423[] = {{&S41204[27], 6}, {NULL}};
const Segment SGI83424[] = {{&S41204[97], 9}, {NULL}};
const Segment SGI83425[] = {{&S41204[70], 6}, {NULL}};
const Segment SGI83426[] = {{&S48100[44], 13}, {NULL}};
const Segment SGI83427[] = {{&S48100[44], 25}, {NULL}};
const Segment SGI83428[] = {{&S48100[30], 6}, {NULL}};
const Segment SGI83429[] = {{&S48100[31], 3}, {NULL}};
const Segment SGI83430[] = {{&S48100[83], 6}, {NULL}};
const Segment SGI83431[] = {{&S48100[0], 13}, {NULL}};
const Segment SGI83432[] = {{&S41218[34], 10}, {NULL}};
const Segment SGI83433[] = {{&S41218[27], 6}, {NULL}};
const Segment SGI83434[] = {{&S41218[97], 9}, {NULL}};
const Segment SGI83435[] = {{&S41218[70], 6}, {NULL}};
const Segment SGI83436[] = {{&S41220[34], 11}, {NULL}};
const Segment SGI83437[] = {{&S41220[27], 6}, {NULL}};
const Segment SGI83438[] = {{&S41220[98], 9}, {NULL}};
const Segment SGI83439[] = {{&S41220[71], 6}, {NULL}};
const Segment SGI83440[] = {{&S52936[42], 17}, {NULL}};
const Segment SGI83441[] = {{&S52936[42], 29}, {NULL}};
const Segment SGI83442[] = {{&S52936[30], 4}, {NULL}};
const Segment SGI83443[] = {{&S52936[120], 3}, {NULL}};
const Segment SGI83444[] = {{&S52936[85], 6}, {NULL}};
const Segment SGI83445[] = {{&S52937[42], 13}, {NULL}};
const Segment SGI83446[] = {{&S52937[42], 25}, {NULL}};
const Segment SGI83447[] = {{&S52937[30], 4}, {NULL}};
const Segment SGI83448[] = {{&S52937[116], 3}, {NULL}};
const Segment SGI83449[] = {{&S52937[81], 6}, {NULL}};
const Segment SGI83450[] = {{&S52938[44], 6}, {NULL}};
const Segment SGI83451[] = {{&S52938[44], 17}, {NULL}};
const Segment SGI83452[] = {{&S52938[30], 6}, {NULL}};
const Segment SGI83453[] = {{&S52938[110], 3}, {NULL}};
const Segment SGI83454[] = {{&S52938[75], 6}, {NULL}};
const Segment SGI83455[] = {{&S52939[44], 11}, {NULL}};
const Segment SGI83456[] = {{&S52939[44], 23}, {NULL}};
const Segment SGI83457[] = {{&S52939[30], 6}, {NULL}};
const Segment SGI83458[] = {{&S52939[115], 3}, {NULL}};
const Segment SGI83459[] = {{&S52939[81], 5}, {NULL}};
const Segment SGI83460[] = {{&S52940[44], 8}, {NULL}};
const Segment SGI83461[] = {{&S52940[44], 19}, {NULL}};
const Segment SGI83462[] = {{&S52940[30], 6}, {NULL}};
const Segment SGI83463[] = {{&S52940[112], 3}, {NULL}};
const Segment SGI83464[] = {{&S52940[77], 6}, {NULL}};
const Segment SGI83465[] = {{&S52941[42], 7}, {NULL}};
const Segment SGI83466[] = {{&S52941[42], 19}, {NULL}};
const Segment SGI83467[] = {{&S52941[30], 4}, {NULL}};
const Segment SGI83468[] = {{&S52941[110], 3}, {NULL}};
const Segment SGI83469[] = {{&S52941[75], 6}, {NULL}};
const Segment SGI83470[] = {{&S52942[42], 18}, {NULL}};
const Segment SGI83471[] = {{&S52942[42], 30}, {NULL}};
const Segment SGI83472[] = {{&S52942[30], 4}, {NULL}};
const Segment SGI83473[] = {{&S52942[121], 3}, {NULL}};
const Segment SGI83474[] = {{&S52942[86], 6}, {NULL}};
const Segment SGI83475[] = {{&S52943[42], 13}, {NULL}};
const Segment SGI83476[] = {{&S52943[42], 30}, {NULL}};
const Segment SGI83477[] = {{&S52943[30], 4}, {NULL}};
const Segment SGI83478[] = {{&S52943[120], 3}, {NULL}};
const Segment SGI83479[] = {{&S52943[86], 5}, {NULL}};
const Segment SGI83480[] = {{&S52944[42], 16}, {NULL}};
const Segment SGI83481[] = {{&S52944[42], 28}, {NULL}};
const Segment SGI83482[] = {{&S52944[30], 4}, {NULL}};
const Segment SGI83483[] = {{&S52944[119], 3}, {NULL}};
const Segment SGI83484[] = {{&S52944[84], 6}, {NULL}};
const Segment SGI83485[] = {{&S52945[42], 8}, {NULL}};
const Segment SGI83486[] = {{&S52945[42], 20}, {NULL}};
const Segment SGI83487[] = {{&S52945[30], 4}, {NULL}};
const Segment SGI83488[] = {{&S52945[111], 3}, {NULL}};
const Segment SGI83489[] = {{&S52945[76], 6}, {NULL}};
const Segment SGI83490[] = {{&S52946[44], 10}, {NULL}};
const Segment SGI83491[] = {{&S52946[44], 22}, {NULL}};
const Segment SGI83492[] = {{&S52946[30], 6}, {NULL}};
const Segment SGI83493[] = {{&S52946[114], 3}, {NULL}};
const Segment SGI83494[] = {{&S52946[80], 5}, {NULL}};
const Segment SGI83495[] = {{&S52947[42], 8}, {NULL}};
const Segment SGI83496[] = {{&S52947[42], 20}, {NULL}};
const Segment SGI83497[] = {{&S52947[30], 4}, {NULL}};
const Segment SGI83498[] = {{&S52947[111], 3}, {NULL}};
const Segment SGI83499[] = {{&S52947[76], 6}, {NULL}};
const Segment SGI83500[] = {{&S52948[42], 16}, {NULL}};
const Segment SGI83501[] = {{&S52948[42], 39}, {NULL}};
const Segment SGI83502[] = {{&S52948[30], 4}, {NULL}};
const Segment SGI83503[] = {{&S52948[130], 3}, {NULL}};
const Segment SGI83504[] = {{&S52948[95], 6}, {NULL}};
const Segment SGI83505[] = {{&S52949[42], 8}, {NULL}};
const Segment SGI83506[] = {{&S52949[42], 24}, {NULL}};
const Segment SGI83507[] = {{&S52949[30], 4}, {NULL}};
const Segment SGI83508[] = {{&S60765[44], 14}, {NULL}};
const Segment SGI83509[] = {{&S60765[44], 26}, {NULL}};
const Segment SGI83510[] = {{&S60765[30], 6}, {NULL}};
const Segment SGI83511[] = {{&S60765[118], 3}, {NULL}};
const Segment SGI83512[] = {{&S60765[84], 5}, {NULL}};
const Segment SGI83513[] = {{&S60766[38], 14}, {NULL}};
const Segment SGI83514[] = {{&S60766[31], 6}, {NULL}};
const Segment SGI83515[] = {{&S54114[44], 7}, {NULL}};
const Segment SGI83516[] = {{&S54114[44], 18}, {NULL}};
const Segment SGI83517[] = {{&S54114[30], 6}, {NULL}};
const Segment SGI83518[] = {{&S54114[129], 3}, {NULL}};
const Segment SGI83519[] = {{&S54114[95], 5}, {NULL}};
const Segment SGI83520[] = {{&S54114[35], 1}, {NULL}};
const Segment SGI83521[] = {{&S54114[64], 11}, {NULL}};
const Segment SGI83522[] = {{&S54115[44], 7}, {NULL}};
const Segment SGI83523[] = {{&S54115[44], 18}, {NULL}};
const Segment SGI83524[] = {{&S54115[30], 6}, {NULL}};
const Segment SGI83525[] = {{&S54115[119], 3}, {NULL}};
const Segment SGI83526[] = {{&S54115[85], 5}, {NULL}};
const Segment SGI83527[] = {{&S54115[68], 3}, {NULL}};
const Segment SGI83528[] = {{&S52949[115], 3}, {NULL}};
const Segment SGI83529[] = {{&S52949[80], 6}, {NULL}};
const Segment SGI83530[] = {{&S52950[44], 9}, {NULL}};
const Segment SGI83531[] = {{&S52950[44], 21}, {NULL}};
const Segment SGI83532[] = {{&S52950[30], 6}, {NULL}};
const Segment SGI83533[] = {{&S52950[114], 3}, {NULL}};
const Segment SGI83534[] = {{&S52950[79], 6}, {NULL}};
const Segment SGI83535[] = {{&S52951[42], 9}, {NULL}};
const Segment SGI83536[] = {{&S52951[42], 21}, {NULL}};
const Segment SGI83537[] = {{&S52951[30], 4}, {NULL}};
const Segment SGI83538[] = {{&S52951[112], 3}, {NULL}};
const Segment SGI83539[] = {{&S52951[77], 6}, {NULL}};
const Segment SGI83540[] = {{&S52952[44], 10}, {NULL}};
const Segment SGI83541[] = {{&S52952[44], 22}, {NULL}};
const Segment SGI83542[] = {{&S52952[30], 6}, {NULL}};
const Segment SGI83543[] = {{&S52952[115], 3}, {NULL}};
const Segment SGI83544[] = {{&S52952[80], 6}, {NULL}};
const Segment SGI83545[] = {{&S52953[44], 5}, {NULL}};
const Segment SGI83546[] = {{&S52953[44], 16}, {NULL}};
const Segment SGI83547[] = {{&S52953[30], 6}, {NULL}};
const Segment SGI83548[] = {{&S52953[109], 3}, {NULL}};
const Segment SGI83549[] = {{&S52953[74], 6}, {NULL}};
const Segment SGI83550[] = {{&S52954[42], 7}, {NULL}};
const Segment SGI83551[] = {{&S52954[42], 21}, {NULL}};
const Segment SGI83552[] = {{&S52954[30], 4}, {NULL}};
const Segment SGI83553[] = {{&S52954[112], 3}, {NULL}};
const Segment SGI83554[] = {{&S52954[77], 6}, {NULL}};
const Segment SGI83555[] = {{&S52955[42], 19}, {NULL}};
const Segment SGI83556[] = {{&S52955[42], 31}, {NULL}};
const Segment SGI83557[] = {{&S52955[30], 4}, {NULL}};
const Segment SGI83558[] = {{&S52955[121], 3}, {NULL}};
const Segment SGI83559[] = {{&S52955[87], 5}, {NULL}};
const Segment SGI83560[] = {{&S52956[42], 11}, {NULL}};
const Segment SGI83561[] = {{&S52956[42], 28}, {NULL}};
const Segment SGI83562[] = {{&S52956[30], 4}, {NULL}};
const Segment SGI83563[] = {{&S52956[118], 3}, {NULL}};
const Segment SGI83564[] = {{&S52956[84], 5}, {NULL}};
const Segment SGI83565[] = {{&S52957[44], 11}, {NULL}};
const Segment SGI83566[] = {{&S52957[44], 23}, {NULL}};
const Segment SGI83567[] = {{&S52957[30], 6}, {NULL}};
const Segment SGI83568[] = {{&S52957[116], 3}, {NULL}};
const Segment SGI83569[] = {{&S52957[81], 6}, {NULL}};
const Segment SGI83570[] = {{&S52958[42], 11}, {NULL}};
const Segment SGI83571[] = {{&S52958[42], 23}, {NULL}};
const Segment SGI83572[] = {{&S52958[30], 4}, {NULL}};
const Segment SGI83573[] = {{&S52958[114], 3}, {NULL}};
const Segment SGI83574[] = {{&S52958[79], 6}, {NULL}};
const Segment SGI83575[] = {{&S52959[42], 9}, {NULL}};
const Segment SGI83576[] = {{&S52959[42], 21}, {NULL}};
const Segment SGI83577[] = {{&S52959[30], 4}, {NULL}};
const Segment SGI83578[] = {{&S52959[112], 3}, {NULL}};
const Segment SGI83579[] = {{&S52959[77], 6}, {NULL}};
const Segment SGI83580[] = {{&S52960[42], 14}, {NULL}};
const Segment SGI83581[] = {{&S52960[42], 26}, {NULL}};
const Segment SGI83582[] = {{&S52960[30], 4}, {NULL}};
const Segment SGI83583[] = {{&S52960[117], 3}, {NULL}};
const Segment SGI83584[] = {{&S52960[82], 6}, {NULL}};
const Segment SGI83585[] = {{&S52961[44], 6}, {NULL}};
const Segment SGI83586[] = {{&S52961[44], 17}, {NULL}};
const Segment SGI83587[] = {{&S52961[30], 6}, {NULL}};
const Segment SGI83588[] = {{&S52961[110], 3}, {NULL}};
const Segment SGI83589[] = {{&S52961[75], 6}, {NULL}};
const Segment SGI83590[] = {{&S52962[44], 8}, {NULL}};
const Segment SGI83591[] = {{&S52962[44], 19}, {NULL}};
const Segment SGI83592[] = {{&S52962[30], 6}, {NULL}};
const Segment SGI83593[] = {{&S52962[112], 3}, {NULL}};
const Segment SGI83594[] = {{&S52962[77], 6}, {NULL}};
const Segment SGI83595[] = {{&S52963[42], 10}, {NULL}};
const Segment SGI83596[] = {{&S52963[42], 25}, {NULL}};
const Segment SGI83597[] = {{&S52963[30], 4}, {NULL}};
const Segment SGI83598[] = {{&S52963[116], 3}, {NULL}};
const Segment SGI83599[] = {{&S52963[81], 6}, {NULL}};
const Segment SGI83600[] = {{&S52964[44], 8}, {NULL}};
const Segment SGI83601[] = {{&S52964[44], 20}, {NULL}};
const Segment SGI83602[] = {{&S52964[30], 6}, {NULL}};
const Segment SGI83603[] = {{&S52964[113], 3}, {NULL}};
const Segment SGI83604[] = {{&S52964[78], 6}, {NULL}};
const Segment SGI83605[] = {{&S52965[42], 9}, {NULL}};
const Segment SGI83606[] = {{&S52965[42], 21}, {NULL}};
const Segment SGI83607[] = {{&S52965[30], 4}, {NULL}};
const Segment SGI83608[] = {{&S52965[112], 3}, {NULL}};
const Segment SGI83609[] = {{&S52965[77], 6}, {NULL}};
const Segment SGI83610[] = {{&S52966[44], 12}, {NULL}};
const Segment SGI83611[] = {{&S52966[44], 24}, {NULL}};
const Segment SGI83612[] = {{&S52966[30], 6}, {NULL}};
const Segment SGI83613[] = {{&S52966[117], 3}, {NULL}};
const Segment SGI83614[] = {{&S52966[82], 6}, {NULL}};
const Segment SGI83615[] = {{&S52967[44], 5}, {NULL}};
const Segment SGI83616[] = {{&S52967[44], 17}, {NULL}};
const Segment SGI83617[] = {{&S52967[30], 6}, {NULL}};
const Segment SGI83618[] = {{&S52967[109], 3}, {NULL}};
const Segment SGI83619[] = {{&S52967[75], 5}, {NULL}};
const Segment SGI83620[] = {{&S52968[44], 15}, {NULL}};
const Segment SGI83621[] = {{&S52968[44], 26}, {NULL}};
const Segment SGI83622[] = {{&S52968[30], 6}, {NULL}};
const Segment SGI83623[] = {{&S52968[119], 3}, {NULL}};
const Segment SGI83624[] = {{&S52968[84], 6}, {NULL}};
const Segment SGI83625[] = {{&S52969[44], 9}, {NULL}};
const Segment SGI83626[] = {{&S52969[44], 21}, {NULL}};
const Segment SGI83627[] = {{&S52969[30], 6}, {NULL}};
const Segment SGI83628[] = {{&S52969[114], 3}, {NULL}};
const Segment SGI83629[] = {{&S52969[79], 6}, {NULL}};
const Segment SGI83630[] = {{&S52970[42], 8}, {NULL}};
const Segment SGI83631[] = {{&S52970[42], 20}, {NULL}};
const Segment SGI83632[] = {{&S52970[30], 4}, {NULL}};
const Segment SGI83633[] = {{&S52970[111], 3}, {NULL}};
const Segment SGI83634[] = {{&S52970[76], 6}, {NULL}};
const Segment SGI83635[] = {{&S41491[34], 7}, {NULL}};
const Segment SGI83636[] = {{&S41491[27], 6}, {NULL}};
const Segment SGI83637[] = {{&S41491[94], 9}, {NULL}};
const Segment SGI83638[] = {{&S41491[67], 6}, {NULL}};
const Segment SGI83639[] = {{&S61043[44], 17}, {NULL}};
const Segment SGI83640[] = {{&S61043[30], 6}, {NULL}};
const Segment SGI83641[] = {{&S61043[109], 3}, {NULL}};
const Segment SGI83642[] = {{&S61043[75], 5}, {NULL}};
const Segment SGI83643[] = {{&S61044[44], 13}, {NULL}};
const Segment SGI83644[] = {{&S61044[44], 25}, {NULL}};
const Segment SGI83645[] = {{&S61044[30], 6}, {NULL}};
const Segment SGI83646[] = {{&S53057[42], 10}, {NULL}};
const Segment SGI83647[] = {{&S53057[42], 41}, {NULL}};
const Segment SGI83648[] = {{&S61044[86], 6}, {NULL}};
const Segment SGI83649[] = {{&S53057[30], 4}, {NULL}};
const Segment SGI83650[] = {{&S53057[131], 3}, {NULL}};
const Segment SGI83651[] = {{&S53057[97], 5}, {NULL}};
const Segment SGI83652[] = {{&S61045[44], 13}, {NULL}};
const Segment SGI83653[] = {{&S61045[44], 25}, {NULL}};
const Segment SGI83654[] = {{&S61045[30], 6}, {NULL}};
const Segment SGI83655[] = {{&S61045[117], 3}, {NULL}};
const Segment SGI83656[] = {{&S61045[83], 5}, {NULL}};
const Segment SGI83657[] = {{&S53058[42], 10}, {NULL}};
const Segment SGI83658[] = {{&S53058[42], 35}, {NULL}};
const Segment SGI83659[] = {{&S53058[30], 4}, {NULL}};
const Segment SGI83660[] = {{&S53058[125], 3}, {NULL}};
const Segment SGI83661[] = {{&S53058[91], 5}, {NULL}};
const Segment SGI83662[] = {{&S61046[38], 13}, {NULL}};
const Segment SGI83663[] = {{&S61046[31], 6}, {NULL}};
const Segment SGI83664[] = {{&S53059[42], 7}, {NULL}};
const Segment SGI83665[] = {{&S53059[42], 32}, {NULL}};
const Segment SGI83666[] = {{&S53059[30], 4}, {NULL}};
const Segment SGI83667[] = {{&S53059[123], 3}, {NULL}};
const Segment SGI83668[] = {{&S53059[88], 6}, {NULL}};
const Segment SGI83669[] = {{&S53060[44], 10}, {NULL}};
const Segment SGI83670[] = {{&S53060[44], 32}, {NULL}};
const Segment SGI83671[] = {{&S53060[30], 6}, {NULL}};
const Segment SGI83672[] = {{&S53060[124], 3}, {NULL}};
const Segment SGI83673[] = {{&S53060[90], 5}, {NULL}};
const Segment SGI83674[] = {{&S53061[44], 7}, {NULL}};
const Segment SGI83675[] = {{&S53061[44], 29}, {NULL}};
const Segment SGI83676[] = {{&S53061[30], 6}, {NULL}};
const Segment SGI83677[] = {{&S53061[122], 3}, {NULL}};
const Segment SGI83678[] = {{&S53061[87], 6}, {NULL}};
const Segment SGI83679[] = {{&S53062[38], 7}, {NULL}};
const Segment SGI83680[] = {{&S53062[31], 6}, {NULL}};
const Segment SGI83681[] = {{&S53063[36], 7}, {NULL}};
const Segment SGI83682[] = {{&S53063[31], 4}, {NULL}};
const Segment SGI83683[] = {{&S61158[44], 11}, {NULL}};
const Segment SGI83684[] = {{&S61158[44], 28}, {NULL}};
const Segment SGI83685[] = {{&S61158[30], 6}, {NULL}};
const Segment SGI83686[] = {{&S61158[150], 4}, {NULL}};
const Segment SGI83687[] = {{&S61158[176], 3}, {NULL}};
const Segment SGI83688[] = {{&S61158[120], 3}, {&S61158[225], 5}, {NULL}};
const Segment SGI83689[] = {{&S61158[86], 5}, {NULL}};
const Segment SGI83690[] = {{&S41632[44], 25}, {NULL}};
const Segment SGI83691[] = {{&S41632[44], 37}, {NULL}};
const Segment SGI83692[] = {{&S41632[30], 6}, {NULL}};
const Segment SGI83693[] = {{&S41632[31], 3}, {NULL}};
const Segment SGI83694[] = {{&S41632[95], 6}, {NULL}};
const Segment SGI83695[] = {{&S41633[38], 25}, {NULL}};
const Segment SGI83696[] = {{&S41633[31], 6}, {NULL}};
const Segment SGI83697[] = {{&S41634[44], 7}, {NULL}};
const Segment SGI83698[] = {{&S41634[44], 19}, {NULL}};
const Segment SGI83699[] = {{&S41634[30], 6}, {NULL}};
const Segment SGI83700[] = {{&S41634[31], 3}, {NULL}};
const Segment SGI83701[] = {{&S41634[77], 6}, {NULL}};
const Segment SGI83702[] = {{&S41636[44], 25}, {NULL}};
const Segment SGI83703[] = {{&S41636[44], 37}, {NULL}};
const Segment SGI83704[] = {{&S41636[30], 6}, {NULL}};
const Segment SGI83705[] = {{&S41636[31], 3}, {NULL}};
const Segment SGI83706[] = {{&S41636[95], 6}, {NULL}};
const Segment SGI83707[] = {{&S41637[42], 25}, {NULL}};
const Segment SGI83708[] = {{&S41637[30], 6}, {NULL}};
const Segment SGI83709[] = {{&S41637[31], 3}, {NULL}};
const Segment SGI83710[] = {{&S41637[93], 6}, {NULL}};
const Segment SGI83711[] = {{&S53275[44], 7}, {NULL}};
const Segment SGI83712[] = {{&S53275[44], 19}, {NULL}};
const Segment SGI83713[] = {{&S53275[30], 6}, {NULL}};
const Segment SGI83714[] = {{&S53275[112], 3}, {NULL}};
const Segment SGI83715[] = {{&S53275[77], 6}, {NULL}};
const Segment SGI83716[] = {{&S53276[42], 7}, {NULL}};
const Segment SGI83717[] = {{&S53276[42], 19}, {NULL}};
const Segment SGI83718[] = {{&S53276[30], 4}, {NULL}};
const Segment SGI83719[] = {{&S53276[110], 3}, {NULL}};
const Segment SGI83720[] = {{&S53276[75], 6}, {NULL}};
const Segment SGI83721[] = {{&S53277[38], 7}, {NULL}};
const Segment SGI83722[] = {{&S53277[31], 6}, {NULL}};
const Segment SGI83723[] = {{&S61298[44], 10}, {NULL}};
const Segment SGI83724[] = {{&S61298[44], 22}, {NULL}};
const Segment SGI83725[] = {{&S61298[30], 6}, {NULL}};
const Segment SGI83726[] = {{&S61298[114], 3}, {NULL}};
const Segment SGI83727[] = {{&S61298[80], 5}, {NULL}};
const Segment SGI83728[] = {{&S61299[38], 10}, {NULL}};
const Segment SGI83729[] = {{&S61299[31], 6}, {NULL}};
const Segment SGI83730[] = {{&S61325[44], 4}, {NULL}};
const Segment SGI83731[] = {{&S61325[44], 16}, {NULL}};
const Segment SGI83732[] = {{&S61325[30], 6}, {NULL}};
const Segment SGI83733[] = {{&S61325[108], 3}, {NULL}};
const Segment SGI83734[] = {{&S61325[74], 5}, {NULL}};
const Segment SGI83735[] = {{&S61327[44], 9}, {NULL}};
const Segment SGI83736[] = {{&S61327[44], 21}, {NULL}};
const Segment SGI83737[] = {{&S61327[30], 6}, {NULL}};
const Segment SGI83738[] = {{&S61327[113], 3}, {NULL}};
const Segment SGI83739[] = {{&S61327[79], 5}, {NULL}};
const Segment SGI83740[] = {{&S54435[44], 7}, {NULL}};
const Segment SGI83741[] = {{&S54435[44], 18}, {NULL}};
const Segment SGI83742[] = {{&S54435[30], 6}, {NULL}};
const Segment SGI83743[] = {{&S54435[79], 4}, {NULL}};
const Segment SGI83744[] = {{&S54435[79], 3}, {NULL}};
const Segment SGI83745[] = {{&S54436[44], 7}, {NULL}};
const Segment SGI83746[] = {{&S54436[44], 18}, {NULL}};
const Segment SGI83747[] = {{&S54436[30], 6}, {NULL}};
const Segment SGI83748[] = {{&S54436[110], 3}, {NULL}};
const Segment SGI83749[] = {{&S54436[76], 5}, {NULL}};
const Segment SGI83750[] = {{&S54444[44], 14}, {NULL}};
const Segment SGI83751[] = {{&S54444[44], 25}, {NULL}};
const Segment SGI83752[] = {{&S54444[30], 6}, {NULL}};
const Segment SGI83753[] = {{&S54444[86], 6}, {NULL}};
const Segment SGI83754[] = {{&S41742[44], 10}, {NULL}};
const Segment SGI83755[] = {{&S41742[44], 27}, {NULL}};
const Segment SGI83756[] = {{&S41742[30], 6}, {NULL}};
const Segment SGI83757[] = {{&S41742[31], 3}, {NULL}};
const Segment SGI83758[] = {{&S41742[85], 6}, {NULL}};
const Segment SGI83759[] = {{&S41743[44], 10}, {NULL}};
const Segment SGI83760[] = {{&S41743[44], 22}, {NULL}};
const Segment SGI83761[] = {{&S41743[30], 6}, {NULL}};
const Segment SGI83762[] = {{&S41743[31], 3}, {NULL}};
const Segment SGI83763[] = {{&S41743[80], 6}, {NULL}};
const Segment SGI83764[] = {{&S41748[38], 10}, {NULL}};
const Segment SGI83765[] = {{&S41748[31], 6}, {NULL}};
const Segment SGI83766[] = {{&S41753[44], 10}, {NULL}};
const Segment SGI83767[] = {{&S41753[44], 30}, {NULL}};
const Segment SGI83768[] = {{&S41753[30], 6}, {NULL}};
const Segment SGI83769[] = {{&S41753[31], 3}, {NULL}};
const Segment SGI83770[] = {{&S41753[88], 6}, {NULL}};
const Segment SGI83771[] = {{&S41755[44], 11}, {NULL}};
const Segment SGI83772[] = {{&S41755[44], 28}, {NULL}};
const Segment SGI83773[] = {{&S41755[30], 6}, {NULL}};
const Segment SGI83774[] = {{&S41755[31], 3}, {NULL}};
const Segment SGI83775[] = {{&S41755[86], 6}, {NULL}};
const Segment SGI83776[] = {{&S41760[44], 11}, {NULL}};
const Segment SGI83777[] = {{&S41760[44], 23}, {NULL}};
const Segment SGI83778[] = {{&S41760[30], 6}, {NULL}};
const Segment SGI83779[] = {{&S41760[31], 3}, {NULL}};
const Segment SGI83780[] = {{&S41760[81], 6}, {NULL}};
const Segment SGI83781[] = {{&S41780[34], 20}, {NULL}};
const Segment SGI83782[] = {{&S41780[27], 6}, {NULL}};
const Segment SGI83783[] = {{&S41780[107], 9}, {NULL}};
const Segment SGI83784[] = {{&S41780[80], 6}, {NULL}};
const Segment SGI83785[] = {{&S41782[34], 19}, {NULL}};
const Segment SGI83786[] = {{&S41782[27], 6}, {NULL}};
const Segment SGI83787[] = {{&S41782[106], 9}, {NULL}};
const Segment SGI83788[] = {{&S41782[79], 6}, {NULL}};
const Segment SGI83789[] = {{&S61514[44], 8}, {NULL}};
const Segment SGI83790[] = {{&S61514[44], 25}, {NULL}};
const Segment SGI83791[] = {{&S61514[30], 6}, {NULL}};
const Segment SGI83792[] = {{&S61514[117], 3}, {NULL}};
const Segment SGI83793[] = {{&S61514[83], 5}, {NULL}};
const Segment SGI83794[] = {{&S41785[34], 7}, {NULL}};
const Segment SGI83795[] = {{&S41785[27], 6}, {NULL}};
const Segment SGI83796[] = {{&S41785[94], 9}, {NULL}};
const Segment SGI83797[] = {{&S41785[67], 6}, {NULL}};
const Segment SGI83798[] = {{&S61557[44], 11}, {NULL}};
const Segment SGI83799[] = {{&S61557[44], 28}, {NULL}};
const Segment SGI83800[] = {{&S61557[30], 6}, {NULL}};
const Segment SGI83801[] = {{&S61557[120], 3}, {NULL}};
const Segment SGI83802[] = {{&S61557[86], 5}, {NULL}};
const Segment SGI83803[] = {{&S61558[38], 11}, {NULL}};
const Segment SGI83804[] = {{&S61558[31], 6}, {NULL}};
const Segment SGI83805[] = {{&S50888[42], 14}, {NULL}};
const Segment SGI83806[] = {{&S50888[42], 25}, {NULL}};
const Segment SGI83807[] = {{&S50888[30], 4}, {NULL}};
const Segment SGI83808[] = {{&S50888[31], 3}, {NULL}};
const Segment SGI83809[] = {{&S50888[81], 6}, {NULL}};
const Segment SGI83810[] = {{&S41990[34], 10}, {NULL}};
const Segment SGI83811[] = {{&S41990[27], 6}, {NULL}};
const Segment SGI83812[] = {{&S41990[97], 9}, {NULL}};
const Segment SGI83813[] = {{&S41990[70], 6}, {NULL}};
const Segment SGI83814[] = {{&S61893[44], 14}, {NULL}};
const Segment SGI83815[] = {{&S61893[44], 26}, {NULL}};
const Segment SGI83816[] = {{&S61893[30], 6}, {NULL}};
const Segment SGI83817[] = {{&S61893[118], 3}, {NULL}};
const Segment SGI83818[] = {{&S61893[84], 5}, {NULL}};
const Segment SGI83819[] = {{&S61894[38], 14}, {NULL}};
const Segment SGI83820[] = {{&S61894[31], 6}, {NULL}};
const Segment SGI83821[] = {{&S53846[42], 10}, {NULL}};
const Segment SGI83822[] = {{&S53846[42], 22}, {NULL}};
const Segment SGI83823[] = {{&S53846[30], 4}, {NULL}};
const Segment SGI83824[] = {{&S53846[113], 3}, {NULL}};
const Segment SGI83825[] = {{&S53846[78], 6}, {NULL}};
const Segment SGI83826[] = {{&S53847[36], 10}, {NULL}};
const Segment SGI83827[] = {{&S53847[31], 4}, {NULL}};
const Segment SGI83828[] = {{&S42154[34], 10}, {NULL}};
const Segment SGI83829[] = {{&S42154[27], 6}, {NULL}};
const Segment SGI83830[] = {{&S53848[42], 10}, {NULL}};
const Segment SGI83831[] = {{&S42154[97], 9}, {NULL}};
const Segment SGI83832[] = {{&S53848[42], 22}, {NULL}};
const Segment SGI83833[] = {{&S42154[70], 6}, {NULL}};
const Segment SGI83834[] = {{&S53848[30], 4}, {NULL}};
const Segment SGI83835[] = {{&S53848[113], 3}, {NULL}};
const Segment SGI83836[] = {{&S53848[78], 6}, {NULL}};
const Segment SGI83837[] = {{&S53849[42], 10}, {NULL}};
const Segment SGI83838[] = {{&S53849[42], 28}, {NULL}};
const Segment SGI83839[] = {{&S53849[30], 4}, {NULL}};
const Segment SGI83840[] = {{&S53849[118], 3}, {NULL}};
const Segment SGI83841[] = {{&S42156[34], 11}, {NULL}};
const Segment SGI83842[] = {{&S53849[84], 5}, {NULL}};
const Segment SGI83843[] = {{&S42156[27], 6}, {NULL}};
const Segment SGI83844[] = {{&S42156[98], 9}, {NULL}};
const Segment SGI83845[] = {{&S42156[71], 6}, {NULL}};
const Segment SGI83846[] = {{&S53878[44], 20}, {NULL}};
const Segment SGI83847[] = {{&S53878[44], 31}, {NULL}};
const Segment SGI83848[] = {{&S53878[30], 6}, {NULL}};
const Segment SGI83849[] = {{&S53878[124], 3}, {NULL}};
const Segment SGI83850[] = {{&S53878[89], 6}, {NULL}};
const Segment SGI83851[] = {{&S53879[42], 20}, {NULL}};
const Segment SGI83852[] = {{&S53879[42], 32}, {NULL}};
const Segment SGI83853[] = {{&S53879[30], 4}, {NULL}};
const Segment SGI83854[] = {{&S53879[123], 3}, {NULL}};
const Segment SGI83855[] = {{&S53879[88], 6}, {NULL}};
const Segment SGI83856[] = {{&S53880[38], 20}, {NULL}};
const Segment SGI83857[] = {{&S53880[31], 6}, {NULL}};
const Segment SGI83858[] = {{&S53881[61], 20}, {NULL}};
const Segment SGI83859[] = {{&S53881[61], 31}, {NULL}};
const Segment SGI83860[] = {{&S53881[30], 23}, {NULL}};
const Segment SGI83861[] = {{&S53881[141], 3}, {NULL}};
const Segment SGI83862[] = {{&S53881[106], 6}, {NULL}};
const Segment SGI83863[] = {{&S53944[44], 14}, {NULL}};
const Segment SGI83864[] = {{&S53944[44], 26}, {NULL}};
const Segment SGI83865[] = {{&S53944[30], 6}, {NULL}};
const Segment SGI83866[] = {{&S53944[119], 3}, {NULL}};
const Segment SGI83867[] = {{&S53944[84], 6}, {NULL}};
const Segment SGI83868[] = {{&S42238[44], 13}, {NULL}};
const Segment SGI83869[] = {{&S42238[44], 25}, {NULL}};
const Segment SGI83870[] = {{&S42238[30], 6}, {NULL}};
const Segment SGI83871[] = {{&S42238[31], 3}, {NULL}};
const Segment SGI83872[] = {{&S42238[83], 6}, {NULL}};
const Segment SGI83873[] = {{&S42239[44], 20}, {NULL}};
const Segment SGI83874[] = {{&S42239[44], 29}, {NULL}};
const Segment SGI83875[] = {{&S42239[30], 6}, {NULL}};
const Segment SGI83876[] = {{&S42239[31], 3}, {NULL}};
const Segment SGI83877[] = {{&S42239[87], 6}, {NULL}};
const Segment SGI83878[] = {{&S42240[44], 13}, {NULL}};
const Segment SGI83879[] = {{&S42240[44], 25}, {NULL}};
const Segment SGI83880[] = {{&S42240[30], 6}, {NULL}};
const Segment SGI83881[] = {{&S42240[31], 3}, {NULL}};
const Segment SGI83882[] = {{&S42240[83], 6}, {NULL}};
const Segment SGI83883[] = {{&S42243[38], 20}, {NULL}};
const Segment SGI83884[] = {{&S42243[31], 6}, {NULL}};
const Segment SGI83885[] = {{&S42245[44], 20}, {NULL}};
const Segment SGI83886[] = {{&S42245[44], 34}, {NULL}};
const Segment SGI83887[] = {{&S42245[30], 6}, {NULL}};
const Segment SGI83888[] = {{&S42245[31], 3}, {NULL}};
const Segment SGI83889[] = {{&S42245[92], 6}, {NULL}};
const Segment SGI83890[] = {{&S42247[44], 20}, {NULL}};
const Segment SGI83891[] = {{&S42247[44], 32}, {NULL}};
const Segment SGI83892[] = {{&S42247[30], 6}, {NULL}};
const Segment SGI83893[] = {{&S42247[31], 3}, {NULL}};
const Segment SGI83894[] = {{&S42247[90], 6}, {NULL}};
const Segment SGI83895[] = {{&S42249[38], 13}, {NULL}};
const Segment SGI83896[] = {{&S42249[31], 6}, {NULL}};
const Segment SGI83897[] = {{&S42250[44], 6}, {NULL}};
const Segment SGI83898[] = {{&S42250[44], 18}, {NULL}};
const Segment SGI83899[] = {{&S42250[30], 6}, {NULL}};
const Segment SGI83900[] = {{&S42250[31], 3}, {NULL}};
const Segment SGI83901[] = {{&S42250[76], 6}, {NULL}};
const Segment SGI83902[] = {{&S42251[38], 6}, {NULL}};
const Segment SGI83903[] = {{&S42251[31], 6}, {NULL}};
const Segment SGI83904[] = {{&S42252[44], 6}, {NULL}};
const Segment SGI83905[] = {{&S42252[44], 18}, {NULL}};
const Segment SGI83906[] = {{&S42252[30], 6}, {NULL}};
const Segment SGI83907[] = {{&S42252[31], 3}, {NULL}};
const Segment SGI83908[] = {{&S42252[76], 6}, {NULL}};
const Segment SGI83909[] = {{&S42254[42], 6}, {NULL}};
const Segment SGI83910[] = {{&S42254[30], 6}, {NULL}};
const Segment SGI83911[] = {{&S42254[31], 3}, {NULL}};
const Segment SGI83912[] = {{&S42254[74], 6}, {NULL}};
const Segment SGI83913[] = {{&S42291[44], 12}, {NULL}};
const Segment SGI83914[] = {{&S42291[44], 24}, {NULL}};
const Segment SGI83915[] = {{&S42291[30], 6}, {NULL}};
const Segment SGI83916[] = {{&S42291[31], 3}, {NULL}};
const Segment SGI83917[] = {{&S42291[82], 6}, {NULL}};
const Segment SGI83918[] = {{&S42292[44], 12}, {NULL}};
const Segment SGI83919[] = {{&S42292[44], 24}, {NULL}};
const Segment SGI83920[] = {{&S42292[30], 6}, {NULL}};
const Segment SGI83921[] = {{&S42292[31], 3}, {NULL}};
const Segment SGI83922[] = {{&S42292[82], 6}, {NULL}};
const Segment SGI83923[] = {{&S42316[44], 10}, {NULL}};
const Segment SGI83924[] = {{&S42316[44], 22}, {NULL}};
const Segment SGI83925[] = {{&S42316[30], 6}, {NULL}};
const Segment SGI83926[] = {{&S42316[31], 3}, {NULL}};
const Segment SGI83927[] = {{&S42316[80], 6}, {NULL}};
const Segment SGI83928[] = {{&S42317[44], 10}, {NULL}};
const Segment SGI83929[] = {{&S42317[44], 17}, {NULL}};
const Segment SGI83930[] = {{&S42317[30], 6}, {NULL}};
const Segment SGI83931[] = {{&S62081[6], 4}, {NULL}};
const Segment SGI83932[] = {{&S62081[82], 5}, {NULL}};
const Segment SGI83933[] = {{&S42338[34], 11}, {NULL}};
const Segment SGI83934[] = {{&S42338[27], 6}, {NULL}};
const Segment SGI83935[] = {{&S42338[98], 9}, {NULL}};
const Segment SGI83936[] = {{&S42338[71], 6}, {NULL}};
const Segment SGI83937[] = {{&S42339[34], 10}, {NULL}};
const Segment SGI83938[] = {{&S42339[27], 6}, {NULL}};
const Segment SGI83939[] = {{&S42339[97], 9}, {NULL}};
const Segment SGI83940[] = {{&S42339[70], 6}, {NULL}};
const Segment SGI83941[] = {{&S42341[34], 11}, {NULL}};
const Segment SGI83942[] = {{&S42341[27], 6}, {NULL}};
const Segment SGI83943[] = {{&S42341[98], 9}, {NULL}};
const Segment SGI83944[] = {{&S42341[71], 6}, {NULL}};
const Segment SGI83945[] = {{&S42342[34], 11}, {NULL}};
const Segment SGI83946[] = {{&S42342[27], 6}, {NULL}};
const Segment SGI83947[] = {{&S42342[98], 9}, {NULL}};
const Segment SGI83948[] = {{&S42342[71], 6}, {NULL}};
const Segment SGI83949[] = {{&S62184[44], 10}, {NULL}};
const Segment SGI83950[] = {{&S62184[44], 27}, {NULL}};
const Segment SGI83951[] = {{&S62184[30], 6}, {NULL}};
const Segment SGI83952[] = {{&S62184[119], 3}, {NULL}};
const Segment SGI83953[] = {{&S62184[85], 5}, {NULL}};
const Segment SGI83954[] = {{&S62185[38], 10}, {NULL}};
const Segment SGI83955[] = {{&S62185[31], 6}, {NULL}};
const Segment SGI83956[] = {{&S62189[44], 10}, {NULL}};
const Segment SGI83957[] = {{&S62189[44], 27}, {NULL}};
const Segment SGI83958[] = {{&S62189[30], 6}, {NULL}};
const Segment SGI83959[] = {{&S62189[85], 6}, {NULL}};
const Segment SGI83960[] = {{&S62189[112], 13}, {NULL}};
const Segment SGI83961[] = {{&S62191[44], 11}, {NULL}};
const Segment SGI83962[] = {{&S62191[44], 28}, {NULL}};
const Segment SGI83963[] = {{&S62191[30], 6}, {NULL}};
const Segment SGI83964[] = {{&S62191[86], 6}, {NULL}};
const Segment SGI83965[] = {{&S62191[113], 13}, {NULL}};
const Segment SGI83966[] = {{&S62192[44], 10}, {NULL}};
const Segment SGI83967[] = {{&S62192[44], 27}, {NULL}};
const Segment SGI83968[] = {{&S62192[30], 6}, {NULL}};
const Segment SGI83969[] = {{&S62192[85], 6}, {NULL}};
const Segment SGI83970[] = {{&S62192[112], 13}, {NULL}};
const Segment SGI83971[] = {{&S54187[42], 8}, {NULL}};
const Segment SGI83972[] = {{&S54187[42], 20}, {NULL}};
const Segment SGI83973[] = {{&S54187[30], 4}, {NULL}};
const Segment SGI83974[] = {{&S62205[30], 6}, {NULL}};
const Segment SGI83975[] = {{&S54187[111], 3}, {NULL}};
const Segment SGI83976[] = {{&S54187[76], 6}, {NULL}};
const Segment SGI83977[] = {{&S62205[92], 2}, {NULL}};
const Segment SGI83978[] = {{&S62205[57], 6}, {NULL}};
const Segment SGI83979[] = {{&S54188[42], 10}, {NULL}};
const Segment SGI83980[] = {{&S54188[42], 22}, {NULL}};
const Segment SGI83981[] = {{&S54188[30], 4}, {NULL}};
const Segment SGI83982[] = {{&S54188[113], 3}, {NULL}};
const Segment SGI83983[] = {{&S54188[78], 6}, {NULL}};
const Segment SGI83984[] = {{&S54189[36], 8}, {NULL}};
const Segment SGI83985[] = {{&S54189[31], 4}, {NULL}};
const Segment SGI83986[] = {{&S54190[36], 10}, {NULL}};
const Segment SGI83987[] = {{&S54190[31], 4}, {NULL}};
const Segment SGI83988[] = {{&S54191[42], 10}, {NULL}};
const Segment SGI83989[] = {{&S54191[42], 22}, {NULL}};
const Segment SGI83990[] = {{&S54191[30], 4}, {NULL}};
const Segment SGI83991[] = {{&S54191[81], 6}, {NULL}};
const Segment SGI83992[] = {{&S54192[42], 10}, {NULL}};
const Segment SGI83993[] = {{&S54192[42], 22}, {NULL}};
const Segment SGI83994[] = {{&S54192[30], 4}, {NULL}};
const Segment SGI83995[] = {{&S54192[113], 3}, {NULL}};
const Segment SGI83996[] = {{&S54192[78], 6}, {NULL}};
const Segment SGI83997[] = {{&S54193[42], 10}, {NULL}};
const Segment SGI83998[] = {{&S54193[42], 22}, {&S54193[205], 10}, {NULL}};
const Segment SGI83999[] = {{&S54193[30], 4}, {NULL}};
const Segment SGI84000[] = {{&S54193[113], 3}, {NULL}};
const Segment SGI84001[] = {{&S54193[78], 6}, {NULL}};
const Segment SGI84002[] = {{&S54194[40], 10}, {NULL}};
const Segment SGI84003[] = {{&S54194[30], 4}, {NULL}};
const Segment SGI84004[] = {{&S54194[111], 3}, {NULL}};
const Segment SGI84005[] = {{&S54194[76], 6}, {NULL}};
const Segment SGI84006[] = {{&S54195[62], 10}, {NULL}};
const Segment SGI84007[] = {{&S54195[62], 22}, {NULL}};
const Segment SGI84008[] = {{&S54195[50], 4}, {NULL}};
const Segment SGI84009[] = {{&S49598[6], 4}, {NULL}};
const Segment SGI84010[] = {{&S49598[86], 5}, {NULL}};
const Segment SGI84011[] = {{&S49598[0], 27}, {NULL}};
const Segment SGI84012[] = {{&S62261[44], 16}, {NULL}};
const Segment SGI84013[] = {{&S62261[44], 28}, {NULL}};
const Segment SGI84014[] = {{&S62261[30], 6}, {NULL}};
const Segment SGI84015[] = {{&S62261[120], 3}, {NULL}};
const Segment SGI84016[] = {{&S62261[86], 5}, {NULL}};
const Segment SGI84017[] = {{&S62262[44], 11}, {NULL}};
const Segment SGI84018[] = {{&S62262[44], 28}, {NULL}};
const Segment SGI84019[] = {{&S62262[30], 6}, {NULL}};
const Segment SGI84020[] = {{&S62262[86], 6}, {NULL}};
const Segment SGI84021[] = {{&S62262[113], 13}, {NULL}};
const Segment SGI84022[] = {{&S62334[44], 11}, {NULL}};
const Segment SGI84023[] = {{&S62334[44], 28}, {NULL}};
const Segment SGI84024[] = {{&S62334[30], 6}, {NULL}};
const Segment SGI84025[] = {{&S62334[150], 4}, {NULL}};
const Segment SGI84026[] = {{&S62334[176], 3}, {NULL}};
const Segment SGI84027[] = {{&S62334[120], 3}, {&S62334[225], 5}, {NULL}};
const Segment SGI84028[] = {{&S62334[86], 5}, {NULL}};
const Segment SGI84029[] = {{&S62337[45], 11}, {NULL}};
const Segment SGI84030[] = {{&S62337[30], 6}, {NULL}};
const Segment SGI84031[] = {{&S62337[121], 3}, {NULL}};
const Segment SGI84032[] = {{&S62337[87], 5}, {NULL}};
const Segment SGI84033[] = {{&S62337[145], 13}, {NULL}};
const Segment SGI84034[] = {{&S62338[45], 11}, {NULL}};
const Segment SGI84035[] = {{&S62338[30], 6}, {NULL}};
const Segment SGI84036[] = {{&S62338[121], 3}, {NULL}};
const Segment SGI84037[] = {{&S62338[87], 5}, {NULL}};
const Segment SGI84038[] = {{&S62338[145], 13}, {NULL}};
const Segment SGI84039[] = {{&S62339[44], 11}, {NULL}};
const Segment SGI84040[] = {{&S62339[44], 28}, {NULL}};
const Segment SGI84041[] = {{&S62339[30], 6}, {NULL}};
const Segment SGI84042[] = {{&S62339[121], 3}, {NULL}};
const Segment SGI84043[] = {{&S62339[86], 6}, {NULL}};
const Segment SGI84044[] = {{&S62339[141], 8}, {NULL}};
const Segment SGI84045[] = {{&S62339[150], 13}, {NULL}};
const Segment SGI84046[] = {{&S55392[44], 13}, {NULL}};
const Segment SGI84047[] = {{&S55392[44], 30}, {NULL}};
const Segment SGI84048[] = {{&S55392[30], 6}, {NULL}};
const Segment SGI84049[] = {{&S55392[122], 3}, {NULL}};
const Segment SGI84050[] = {{&S55392[88], 5}, {NULL}};
const Segment SGI84051[] = {{&S54338[44], 11}, {NULL}};
const Segment SGI84052[] = {{&S54338[44], 33}, {NULL}};
const Segment SGI84053[] = {{&S54338[30], 6}, {NULL}};
const Segment SGI84054[] = {{&S54338[125], 3}, {NULL}};
const Segment SGI84055[] = {{&S54338[91], 5}, {NULL}};
const Segment SGI84056[] = {{&S54339[42], 11}, {NULL}};
const Segment SGI84057[] = {{&S54339[42], 30}, {NULL}};
const Segment SGI84058[] = {{&S54339[30], 4}, {NULL}};
const Segment SGI84059[] = {{&S54339[120], 3}, {NULL}};
const Segment SGI84060[] = {{&S54339[86], 5}, {NULL}};
const Segment SGI84061[] = {{&S54340[42], 11}, {NULL}};
const Segment SGI84062[] = {{&S54340[42], 34}, {NULL}};
const Segment SGI84063[] = {{&S54340[30], 4}, {NULL}};
const Segment SGI84064[] = {{&S54340[124], 3}, {NULL}};
const Segment SGI84065[] = {{&S54340[90], 5}, {NULL}};
const Segment SGI84066[] = {{&S54341[42], 11}, {NULL}};
const Segment SGI84067[] = {{&S54341[42], 34}, {NULL}};
const Segment SGI84068[] = {{&S54341[30], 4}, {NULL}};
const Segment SGI84069[] = {{&S54341[125], 3}, {NULL}};
const Segment SGI84070[] = {{&S54341[90], 6}, {NULL}};
const Segment SGI84071[] = {{&S54342[44], 11}, {NULL}};
const Segment SGI84072[] = {{&S54342[44], 34}, {NULL}};
const Segment SGI84073[] = {{&S54342[30], 6}, {NULL}};
const Segment SGI84074[] = {{&S54342[126], 3}, {NULL}};
const Segment SGI84075[] = {{&S54342[92], 5}, {NULL}};
const Segment SGI84076[] = {{&S54343[36], 11}, {NULL}};
const Segment SGI84077[] = {{&S54343[31], 4}, {NULL}};
const Segment SGI84078[] = {{&S62436[44], 11}, {NULL}};
const Segment SGI84079[] = {{&S62436[44], 22}, {NULL}};
const Segment SGI84080[] = {{&S62436[30], 6}, {NULL}};
const Segment SGI84081[] = {{&S54344[42], 11}, {NULL}};
const Segment SGI84082[] = {{&S54344[42], 30}, {NULL}};
const Segment SGI84083[] = {{&S62436[114], 3}, {NULL}};
const Segment SGI84084[] = {{&S54344[30], 4}, {NULL}};
const Segment SGI84085[] = {{&S62436[80], 5}, {NULL}};
const Segment SGI84086[] = {{&S54344[121], 3}, {NULL}};
const Segment SGI84087[] = {{&S54344[86], 6}, {NULL}};
const Segment SGI84088[] = {{&S62437[44], 7}, {NULL}};
const Segment SGI84089[] = {{&S62437[44], 19}, {NULL}};
const Segment SGI84090[] = {{&S62437[30], 6}, {NULL}};
const Segment SGI84091[] = {{&S54345[42], 11}, {NULL}};
const Segment SGI84092[] = {{&S54345[42], 34}, {NULL}};
const Segment SGI84093[] = {{&S54345[30], 4}, {NULL}};
const Segment SGI84094[] = {{&S62437[80], 6}, {NULL}};
const Segment SGI84095[] = {{&S54345[125], 3}, {NULL}};
const Segment SGI84096[] = {{&S54345[90], 6}, {NULL}};
const Segment SGI84097[] = {{&S62438[44], 7}, {NULL}};
const Segment SGI84098[] = {{&S62438[44], 24}, {NULL}};
const Segment SGI84099[] = {{&S62438[30], 6}, {NULL}};
const Segment SGI84100[] = {{&S54346[44], 11}, {NULL}};
const Segment SGI84101[] = {{&S54346[44], 34}, {NULL}};
const Segment SGI84102[] = {{&S62438[116], 3}, {NULL}};
const Segment SGI84103[] = {{&S54346[30], 6}, {NULL}};
const Segment SGI84104[] = {{&S62438[82], 5}, {NULL}};
const Segment SGI84105[] = {{&S54346[126], 3}, {NULL}};
const Segment SGI84106[] = {{&S54346[92], 5}, {NULL}};
const Segment SGI84107[] = {{&S62439[44], 7}, {NULL}};
const Segment SGI84108[] = {{&S62439[44], 19}, {NULL}};
const Segment SGI84109[] = {{&S62439[30], 6}, {NULL}};
const Segment SGI84110[] = {{&S62439[111], 3}, {NULL}};
const Segment SGI84111[] = {{&S54347[44], 11}, {NULL}};
const Segment SGI84112[] = {{&S54347[44], 34}, {NULL}};
const Segment SGI84113[] = {{&S62439[77], 5}, {NULL}};
const Segment SGI84114[] = {{&S54347[30], 6}, {NULL}};
const Segment SGI84115[] = {{&S54347[126], 3}, {NULL}};
const Segment SGI84116[] = {{&S54347[92], 5}, {NULL}};
const Segment SGI84117[] = {{&S54348[42], 11}, {NULL}};
const Segment SGI84118[] = {{&S54348[42], 35}, {NULL}};
const Segment SGI84119[] = {{&S54348[30], 4}, {NULL}};
const Segment SGI84120[] = {{&S54348[126], 3}, {NULL}};
const Segment SGI84121[] = {{&S54348[91], 6}, {NULL}};
const Segment SGI84122[] = {{&S54349[44], 11}, {NULL}};
const Segment SGI84123[] = {{&S54349[44], 33}, {NULL}};
const Segment SGI84124[] = {{&S54349[30], 6}, {NULL}};
const Segment SGI84125[] = {{&S54349[126], 3}, {NULL}};
const Segment SGI84126[] = {{&S54349[91], 6}, {NULL}};
const Segment SGI84127[] = {{&S54350[44], 11}, {NULL}};
const Segment SGI84128[] = {{&S54350[44], 33}, {NULL}};
const Segment SGI84129[] = {{&S54350[30], 6}, {NULL}};
const Segment SGI84130[] = {{&S54350[125], 3}, {NULL}};
const Segment SGI84131[] = {{&S54350[91], 5}, {NULL}};
const Segment SGI84132[] = {{&S54351[44], 11}, {NULL}};
const Segment SGI84133[] = {{&S54351[44], 34}, {NULL}};
const Segment SGI84134[] = {{&S54351[30], 6}, {NULL}};
const Segment SGI84135[] = {{&S54351[127], 3}, {NULL}};
const Segment SGI84136[] = {{&S54351[92], 6}, {NULL}};
const Segment SGI84137[] = {{&S54352[44], 11}, {NULL}};
const Segment SGI84138[] = {{&S54352[44], 33}, {NULL}};
const Segment SGI84139[] = {{&S54352[30], 6}, {NULL}};
const Segment SGI84140[] = {{&S54352[125], 3}, {NULL}};
const Segment SGI84141[] = {{&S54352[91], 5}, {NULL}};
const Segment SGI84142[] = {{&S55569[44], 9}, {NULL}};
const Segment SGI84143[] = {{&S55569[44], 20}, {NULL}};
const Segment SGI84144[] = {{&S55569[30], 6}, {NULL}};
const Segment SGI84145[] = {{&S55569[131], 3}, {NULL}};
const Segment SGI84146[] = {{&S55569[97], 5}, {NULL}};
const Segment SGI84147[] = {{&S55569[35], 1}, {NULL}};
const Segment SGI84148[] = {{&S55569[66], 11}, {NULL}};
const Segment SGI84149[] = {{&S55570[44], 9}, {NULL}};
const Segment SGI84150[] = {{&S55570[44], 20}, {NULL}};
const Segment SGI84151[] = {{&S55570[30], 6}, {NULL}};
const Segment SGI84152[] = {{&S55570[134], 3}, {NULL}};
const Segment SGI84153[] = {{&S55570[100], 5}, {NULL}};
const Segment SGI84154[] = {{&S55570[66], 3}, {NULL}};
const Segment SGI84155[] = {{&S62542[44], 4}, {NULL}};
const Segment SGI84156[] = {{&S62542[44], 16}, {NULL}};
const Segment SGI84157[] = {{&S62542[30], 6}, {NULL}};
const Segment SGI84158[] = {{&S62542[108], 3}, {NULL}};
const Segment SGI84159[] = {{&S62542[74], 5}, {NULL}};
const Segment SGI84160[] = {{&S42804[44], 10}, {NULL}};
const Segment SGI84161[] = {{&S42804[44], 22}, {NULL}};
const Segment SGI84162[] = {{&S42804[30], 6}, {NULL}};
const Segment SGI84163[] = {{&S42804[31], 3}, {NULL}};
const Segment SGI84164[] = {{&S42804[80], 6}, {NULL}};
const Segment SGI84165[] = {{&S42806[38], 10}, {NULL}};
const Segment SGI84166[] = {{&S42806[31], 6}, {NULL}};
const Segment SGI84167[] = {{&S42808[51], 10}, {NULL}};
const Segment SGI84168[] = {{&S42808[51], 22}, {NULL}};
const Segment SGI84169[] = {{&S42808[30], 13}, {NULL}};
const Segment SGI84170[] = {{&S42808[31], 3}, {NULL}};
const Segment SGI84171[] = {{&S42808[87], 6}, {NULL}};
const Segment SGI84172[] = {{&S42809[44], 10}, {NULL}};
const Segment SGI84173[] = {{&S42809[44], 19}, {NULL}};
const Segment SGI84174[] = {{&S42809[30], 6}, {NULL}};
const Segment SGI84175[] = {{&S42810[44], 10}, {NULL}};
const Segment SGI84176[] = {{&S42810[44], 23}, {NULL}};
const Segment SGI84177[] = {{&S42810[30], 6}, {NULL}};
const Segment SGI84178[] = {{&S42810[31], 3}, {NULL}};
const Segment SGI84179[] = {{&S42810[81], 6}, {NULL}};
const Segment SGI84180[] = {{&S42811[44], 10}, {NULL}};
const Segment SGI84181[] = {{&S42811[44], 45}, {NULL}};
const Segment SGI84182[] = {{&S42811[30], 6}, {NULL}};
const Segment SGI84183[] = {{&S42817[44], 10}, {NULL}};
const Segment SGI84184[] = {{&S42817[44], 22}, {NULL}};
const Segment SGI84185[] = {{&S42817[30], 6}, {NULL}};
const Segment SGI84186[] = {{&S42817[80], 6}, {NULL}};
const Segment SGI84187[] = {{&S42886[34], 18}, {NULL}};
const Segment SGI84188[] = {{&S42886[27], 6}, {NULL}};
const Segment SGI84189[] = {{&S42886[105], 9}, {NULL}};
const Segment SGI84190[] = {{&S42886[78], 6}, {NULL}};
const Segment SGI84191[] = {{&S42889[34], 18}, {NULL}};
const Segment SGI84192[] = {{&S42889[27], 6}, {NULL}};
const Segment SGI84193[] = {{&S42889[105], 9}, {NULL}};
const Segment SGI84194[] = {{&S42889[78], 6}, {NULL}};
const Segment SGI84195[] = {{&S62638[44], 10}, {NULL}};
const Segment SGI84196[] = {{&S62638[44], 31}, {NULL}};
const Segment SGI84197[] = {{&S62638[30], 6}, {NULL}};
const Segment SGI84198[] = {{&S62638[123], 3}, {NULL}};
const Segment SGI84199[] = {{&S62638[89], 5}, {NULL}};
const Segment SGI84200[] = {{&S62639[44], 11}, {NULL}};
const Segment SGI84201[] = {{&S62639[44], 32}, {NULL}};
const Segment SGI84202[] = {{&S62639[30], 6}, {NULL}};
const Segment SGI84203[] = {{&S62639[124], 3}, {NULL}};
const Segment SGI84204[] = {{&S62639[90], 5}, {NULL}};
const Segment SGI84205[] = {{&S42901[44], 14}, {NULL}};
const Segment SGI84206[] = {{&S42901[44], 26}, {NULL}};
const Segment SGI84207[] = {{&S42901[30], 6}, {NULL}};
const Segment SGI84208[] = {{&S42901[31], 3}, {NULL}};
const Segment SGI84209[] = {{&S42901[84], 6}, {NULL}};
const Segment SGI84210[] = {{&S42902[44], 14}, {NULL}};
const Segment SGI84211[] = {{&S42902[44], 26}, {NULL}};
const Segment SGI84212[] = {{&S42902[30], 6}, {NULL}};
const Segment SGI84213[] = {{&S42902[31], 3}, {NULL}};
const Segment SGI84214[] = {{&S42902[84], 6}, {NULL}};
const Segment SGI84215[] = {{&S42903[44], 17}, {NULL}};
const Segment SGI84216[] = {{&S42903[44], 29}, {NULL}};
const Segment SGI84217[] = {{&S42903[30], 6}, {NULL}};
const Segment SGI84218[] = {{&S42903[31], 3}, {NULL}};
const Segment SGI84219[] = {{&S42903[87], 6}, {NULL}};
const Segment SGI84220[] = {{&S42904[38], 14}, {NULL}};
const Segment SGI84221[] = {{&S42904[31], 6}, {NULL}};
const Segment SGI84222[] = {{&S55731[44], 13}, {NULL}};
const Segment SGI84223[] = {{&S55731[44], 24}, {NULL}};
const Segment SGI84224[] = {{&S55731[30], 6}, {NULL}};
const Segment SGI84225[] = {{&S55731[135], 3}, {NULL}};
const Segment SGI84226[] = {{&S55731[101], 5}, {NULL}};
const Segment SGI84227[] = {{&S55731[35], 1}, {NULL}};
const Segment SGI84228[] = {{&S55731[70], 11}, {NULL}};
const Segment SGI84229[] = {{&S55732[44], 13}, {NULL}};
const Segment SGI84230[] = {{&S55732[44], 24}, {NULL}};
const Segment SGI84231[] = {{&S55732[30], 6}, {NULL}};
const Segment SGI84232[] = {{&S55732[135], 3}, {NULL}};
const Segment SGI84233[] = {{&S55732[101], 5}, {NULL}};
const Segment SGI84234[] = {{&S55732[35], 1}, {NULL}};
const Segment SGI84235[] = {{&S55732[70], 11}, {NULL}};
const Segment SGI84236[] = {{&S55733[44], 9}, {NULL}};
const Segment SGI84237[] = {{&S55733[44], 20}, {NULL}};
const Segment SGI84238[] = {{&S55733[30], 6}, {NULL}};
const Segment SGI84239[] = {{&S55733[131], 3}, {NULL}};
const Segment SGI84240[] = {{&S55733[97], 5}, {NULL}};
const Segment SGI84241[] = {{&S55733[35], 1}, {NULL}};
const Segment SGI84242[] = {{&S55733[66], 11}, {NULL}};
const Segment SGI84243[] = {{&S49996[44], 10}, {NULL}};
const Segment SGI84244[] = {{&S49996[44], 22}, {NULL}};
const Segment SGI84245[] = {{&S49996[30], 6}, {NULL}};
const Segment SGI84246[] = {{&S49996[31], 3}, {NULL}};
const Segment SGI84247[] = {{&S49996[80], 5}, {NULL}};
const Segment SGI84248[] = {{&S49996[0], 13}, {NULL}};
const Segment SGI84249[] = {{&S42923[34], 22}, {NULL}};
const Segment SGI84250[] = {{&S42923[27], 6}, {NULL}};
const Segment SGI84251[] = {{&S42923[109], 9}, {NULL}};
const Segment SGI84252[] = {{&S42923[82], 6}, {NULL}};
const Segment SGI84253[] = {{&S62689[44], 10}, {NULL}};
const Segment SGI84254[] = {{&S42924[34], 19}, {NULL}};
const Segment SGI84255[] = {{&S62689[44], 27}, {NULL}};
const Segment SGI84256[] = {{&S62689[30], 6}, {NULL}};
const Segment SGI84257[] = {{&S42924[27], 6}, {NULL}};
const Segment SGI84258[] = {{&S42924[106], 9}, {NULL}};
const Segment SGI84259[] = {{&S42924[79], 6}, {NULL}};
const Segment SGI84260[] = {{&S62689[85], 6}, {NULL}};
const Segment SGI84261[] = {{&S62689[112], 13}, {NULL}};
const Segment SGI84262[] = {{&S42925[34], 36}, {NULL}};
const Segment SGI84263[] = {{&S42925[27], 6}, {NULL}};
const Segment SGI84264[] = {{&S42925[123], 9}, {NULL}};
const Segment SGI84265[] = {{&S42925[96], 6}, {NULL}};
const Segment SGI84266[] = {{&S42926[34], 21}, {NULL}};
const Segment SGI84267[] = {{&S42926[27], 6}, {NULL}};
const Segment SGI84268[] = {{&S42926[108], 9}, {NULL}};
const Segment SGI84269[] = {{&S42926[81], 6}, {NULL}};
const Segment SGI84270[] = {{&S42927[34], 18}, {NULL}};
const Segment SGI84271[] = {{&S42927[27], 6}, {NULL}};
const Segment SGI84272[] = {{&S42927[105], 9}, {NULL}};
const Segment SGI84273[] = {{&S42927[78], 6}, {NULL}};
const Segment SGI84274[] = {{&S42928[34], 35}, {NULL}};
const Segment SGI84275[] = {{&S42928[27], 6}, {NULL}};
const Segment SGI84276[] = {{&S42928[122], 9}, {NULL}};
const Segment SGI84277[] = {{&S42928[95], 6}, {NULL}};
const Segment SGI84278[] = {{&S42929[34], 17}, {NULL}};
const Segment SGI84279[] = {{&S42929[27], 6}, {NULL}};
const Segment SGI84280[] = {{&S42929[104], 9}, {NULL}};
const Segment SGI84281[] = {{&S42929[77], 6}, {NULL}};
const Segment SGI84282[] = {{&S42930[34], 38}, {NULL}};
const Segment SGI84283[] = {{&S42930[27], 6}, {NULL}};
const Segment SGI84284[] = {{&S42930[125], 9}, {NULL}};
const Segment SGI84285[] = {{&S42930[98], 6}, {NULL}};
const Segment SGI84286[] = {{&S42931[34], 14}, {NULL}};
const Segment SGI84287[] = {{&S42931[27], 6}, {NULL}};
const Segment SGI84288[] = {{&S42931[101], 9}, {NULL}};
const Segment SGI84289[] = {{&S42931[74], 6}, {NULL}};
const Segment SGI84290[] = {{&S62714[44], 10}, {NULL}};
const Segment SGI84291[] = {{&S62714[44], 27}, {NULL}};
const Segment SGI84292[] = {{&S62714[30], 6}, {NULL}};
const Segment SGI84293[] = {{&S62714[119], 3}, {NULL}};
const Segment SGI84294[] = {{&S62714[85], 5}, {NULL}};
const Segment SGI84295[] = {{&S62715[44], 10}, {NULL}};
const Segment SGI84296[] = {{&S62715[44], 27}, {NULL}};
const Segment SGI84297[] = {{&S62715[30], 6}, {NULL}};
const Segment SGI84298[] = {{&S62715[88], 4}, {NULL}};
const Segment SGI84299[] = {{&S62715[88], 3}, {NULL}};
const Segment SGI84300[] = {{&S62716[44], 10}, {NULL}};
const Segment SGI84301[] = {{&S62716[44], 27}, {NULL}};
const Segment SGI84302[] = {{&S62716[30], 6}, {NULL}};
const Segment SGI84303[] = {{&S62716[88], 6}, {NULL}};
const Segment SGI84304[] = {{&S62717[38], 10}, {NULL}};
const Segment SGI84305[] = {{&S62717[31], 6}, {NULL}};
const Segment SGI84306[] = {{&S42955[34], 10}, {NULL}};
const Segment SGI84307[] = {{&S42955[27], 6}, {NULL}};
const Segment SGI84308[] = {{&S42955[97], 9}, {NULL}};
const Segment SGI84309[] = {{&S42955[70], 6}, {NULL}};
const Segment SGI84310[] = {{&S42957[34], 11}, {NULL}};
const Segment SGI84311[] = {{&S42957[27], 6}, {NULL}};
const Segment SGI84312[] = {{&S42957[98], 9}, {NULL}};
const Segment SGI84313[] = {{&S42957[71], 6}, {NULL}};
const Segment SGI84314[] = {{&S42958[34], 10}, {NULL}};
const Segment SGI84315[] = {{&S42958[27], 6}, {NULL}};
const Segment SGI84316[] = {{&S42958[97], 9}, {NULL}};
const Segment SGI84317[] = {{&S42958[70], 6}, {NULL}};
const Segment SGI84318[] = {{&S42960[34], 11}, {NULL}};
const Segment SGI84319[] = {{&S42960[27], 6}, {NULL}};
const Segment SGI84320[] = {{&S42960[98], 9}, {NULL}};
const Segment SGI84321[] = {{&S42960[71], 6}, {NULL}};
const Segment SGI84322[] = {{&S42961[34], 11}, {NULL}};
const Segment SGI84323[] = {{&S42961[27], 6}, {NULL}};
const Segment SGI84324[] = {{&S42961[98], 9}, {NULL}};
const Segment SGI84325[] = {{&S42961[71], 6}, {NULL}};
const Segment SGI84326[] = {{&S55787[44], 7}, {NULL}};
const Segment SGI84327[] = {{&S55787[44], 18}, {NULL}};
const Segment SGI84328[] = {{&S55787[30], 6}, {NULL}};
const Segment SGI84329[] = {{&S55787[110], 3}, {NULL}};
const Segment SGI84330[] = {{&S55787[76], 5}, {NULL}};
const Segment SGI84331[] = {{&S54600[42], 7}, {NULL}};
const Segment SGI84332[] = {{&S54600[42], 18}, {NULL}};
const Segment SGI84333[] = {{&S54600[30], 4}, {NULL}};
const Segment SGI84334[] = {{&S54600[109], 3}, {NULL}};
const Segment SGI84335[] = {{&S54600[74], 6}, {NULL}};
const Segment SGI84336[] = {{&S54601[42], 7}, {NULL}};
const Segment SGI84337[] = {{&S54601[42], 20}, {NULL}};
const Segment SGI84338[] = {{&S54601[30], 4}, {NULL}};
const Segment SGI84339[] = {{&S54601[111], 3}, {NULL}};
const Segment SGI84340[] = {{&S54601[76], 6}, {NULL}};
const Segment SGI84341[] = {{&S54602[42], 7}, {NULL}};
const Segment SGI84342[] = {{&S54602[42], 20}, {NULL}};
const Segment SGI84343[] = {{&S54602[30], 4}, {NULL}};
const Segment SGI84344[] = {{&S54602[111], 3}, {NULL}};
const Segment SGI84345[] = {{&S54602[76], 6}, {NULL}};
const Segment SGI84346[] = {{&S54603[36], 7}, {NULL}};
const Segment SGI84347[] = {{&S54603[31], 4}, {NULL}};
const Segment SGI84348[] = {{&S62937[44], 9}, {NULL}};
const Segment SGI84349[] = {{&S62937[44], 21}, {NULL}};
const Segment SGI84350[] = {{&S62937[30], 6}, {NULL}};
const Segment SGI84351[] = {{&S62937[113], 3}, {NULL}};
const Segment SGI84352[] = {{&S62937[79], 5}, {NULL}};
const Segment SGI84353[] = {{&S62938[44], 6}, {NULL}};
const Segment SGI84354[] = {{&S62938[44], 18}, {NULL}};
const Segment SGI84355[] = {{&S62938[30], 6}, {NULL}};
const Segment SGI84356[] = {{&S62938[110], 3}, {NULL}};
const Segment SGI84357[] = {{&S62938[76], 5}, {NULL}};
const Segment SGI84358[] = {{&S62939[44], 9}, {NULL}};
const Segment SGI84359[] = {{&S62939[44], 21}, {NULL}};
const Segment SGI84360[] = {{&S62939[30], 6}, {NULL}};
const Segment SGI84361[] = {{&S62939[82], 6}, {NULL}};
const Segment SGI84362[] = {{&S63084[44], 8}, {NULL}};
const Segment SGI84363[] = {{&S63084[44], 20}, {NULL}};
const Segment SGI84364[] = {{&S63084[30], 6}, {NULL}};
const Segment SGI84365[] = {{&S63084[112], 3}, {NULL}};
const Segment SGI84366[] = {{&S63084[78], 5}, {NULL}};
const Segment SGI84367[] = {{&S43215[44], 9}, {NULL}};
const Segment SGI84368[] = {{&S43215[44], 21}, {NULL}};
const Segment SGI84369[] = {{&S43215[30], 6}, {NULL}};
const Segment SGI84370[] = {{&S43215[31], 3}, {NULL}};
const Segment SGI84371[] = {{&S43215[79], 6}, {NULL}};
const Segment SGI84372[] = {{&S43216[44], 9}, {NULL}};
const Segment SGI84373[] = {{&S43216[44], 21}, {NULL}};
const Segment SGI84374[] = {{&S43216[30], 6}, {NULL}};
const Segment SGI84375[] = {{&S43216[31], 3}, {NULL}};
const Segment SGI84376[] = {{&S43216[79], 6}, {NULL}};
const Segment SGI84377[] = {{&S43217[44], 9}, {NULL}};
const Segment SGI84378[] = {{&S43217[44], 21}, {NULL}};
const Segment SGI84379[] = {{&S43217[30], 6}, {NULL}};
const Segment SGI84380[] = {{&S43217[31], 3}, {NULL}};
const Segment SGI84381[] = {{&S43217[79], 6}, {NULL}};
const Segment SGI84382[] = {{&S43218[44], 9}, {NULL}};
const Segment SGI84383[] = {{&S43218[44], 21}, {NULL}};
const Segment SGI84384[] = {{&S43218[30], 6}, {NULL}};
const Segment SGI84385[] = {{&S43218[31], 3}, {NULL}};
const Segment SGI84386[] = {{&S43218[79], 6}, {NULL}};
const Segment SGI84387[] = {{&S43219[38], 9}, {NULL}};
const Segment SGI84388[] = {{&S43219[31], 6}, {NULL}};
const Segment SGI84389[] = {{&S43220[44], 24}, {NULL}};
const Segment SGI84390[] = {{&S43220[44], 36}, {NULL}};
const Segment SGI84391[] = {{&S43220[30], 6}, {NULL}};
const Segment SGI84392[] = {{&S43220[31], 3}, {NULL}};
const Segment SGI84393[] = {{&S43220[94], 6}, {NULL}};
const Segment SGI84394[] = {{&S43234[44], 8}, {NULL}};
const Segment SGI84395[] = {{&S43234[44], 20}, {NULL}};
const Segment SGI84396[] = {{&S43234[30], 6}, {NULL}};
const Segment SGI84397[] = {{&S43234[31], 3}, {NULL}};
const Segment SGI84398[] = {{&S43234[78], 6}, {NULL}};
const Segment SGI84399[] = {{&S43237[44], 9}, {NULL}};
const Segment SGI84400[] = {{&S43237[44], 18}, {NULL}};
const Segment SGI84401[] = {{&S43237[30], 6}, {NULL}};
const Segment SGI84402[] = {{&S43237[31], 3}, {NULL}};
const Segment SGI84403[] = {{&S43237[76], 6}, {NULL}};
const Segment SGI84404[] = {{&S43240[44], 9}, {NULL}};
const Segment SGI84405[] = {{&S43240[44], 21}, {NULL}};
const Segment SGI84406[] = {{&S43240[30], 6}, {NULL}};
const Segment SGI84407[] = {{&S43240[31], 3}, {NULL}};
const Segment SGI84408[] = {{&S43240[79], 6}, {NULL}};
const Segment SGI84409[] = {{&S43249[34], 11}, {NULL}};
const Segment SGI84410[] = {{&S43249[27], 6}, {NULL}};
const Segment SGI84411[] = {{&S43249[98], 9}, {NULL}};
const Segment SGI84412[] = {{&S43249[71], 6}, {NULL}};
const Segment SGI84413[] = {{&S43250[34], 10}, {NULL}};
const Segment SGI84414[] = {{&S43250[27], 6}, {NULL}};
const Segment SGI84415[] = {{&S43250[97], 9}, {NULL}};
const Segment SGI84416[] = {{&S43250[70], 6}, {NULL}};
const Segment SGI84417[] = {{&S43281[34], 12}, {NULL}};
const Segment SGI84418[] = {{&S43281[27], 6}, {NULL}};
const Segment SGI84419[] = {{&S43281[113], 9}, {NULL}};
const Segment SGI84420[] = {{&S43281[86], 6}, {NULL}};
const Segment SGI84421[] = {{&S43299[34], 10}, {NULL}};
const Segment SGI84422[] = {{&S43299[27], 6}, {NULL}};
const Segment SGI84423[] = {{&S43299[97], 9}, {NULL}};
const Segment SGI84424[] = {{&S43299[70], 6}, {NULL}};
const Segment SGI84425[] = {{&S43301[34], 11}, {NULL}};
const Segment SGI84426[] = {{&S43301[27], 6}, {NULL}};
const Segment SGI84427[] = {{&S43301[98], 9}, {NULL}};
const Segment SGI84428[] = {{&S43301[71], 6}, {NULL}};
const Segment SGI84429[] = {{&S43302[34], 11}, {NULL}};
const Segment SGI84430[] = {{&S43302[27], 6}, {NULL}};
const Segment SGI84431[] = {{&S43302[98], 9}, {NULL}};
const Segment SGI84432[] = {{&S43302[71], 6}, {NULL}};
const Segment SGI84433[] = {{&S43303[34], 11}, {NULL}};
const Segment SGI84434[] = {{&S43303[27], 6}, {NULL}};
const Segment SGI84435[] = {{&S43303[98], 9}, {NULL}};
const Segment SGI84436[] = {{&S43303[71], 6}, {NULL}};
const Segment SGI84437[] = {{&S55118[44], 8}, {NULL}};
const Segment SGI84438[] = {{&S55118[44], 20}, {NULL}};
const Segment SGI84439[] = {{&S55118[30], 6}, {NULL}};
const Segment SGI84440[] = {{&S55118[113], 3}, {NULL}};
const Segment SGI84441[] = {{&S55118[78], 6}, {NULL}};
const Segment SGI84442[] = {{&S55119[42], 8}, {NULL}};
const Segment SGI84443[] = {{&S55119[42], 20}, {NULL}};
const Segment SGI84444[] = {{&S55119[30], 4}, {NULL}};
const Segment SGI84445[] = {{&S55119[111], 3}, {NULL}};
const Segment SGI84446[] = {{&S55119[76], 6}, {NULL}};
const Segment SGI84447[] = {{&S55120[38], 8}, {NULL}};
const Segment SGI84448[] = {{&S55120[31], 6}, {NULL}};
const Segment SGI84449[] = {{&S43372[34], 16}, {NULL}};
const Segment SGI84450[] = {{&S43372[27], 6}, {NULL}};
const Segment SGI84451[] = {{&S43372[102], 9}, {NULL}};
const Segment SGI84452[] = {{&S43372[75], 6}, {NULL}};
const Segment SGI84453[] = {{&S43373[34], 15}, {NULL}};
const Segment SGI84454[] = {{&S43373[27], 6}, {NULL}};
const Segment SGI84455[] = {{&S43373[101], 9}, {NULL}};
const Segment SGI84456[] = {{&S43373[74], 6}, {NULL}};
const Segment SGI84457[] = {{&S43405[44], 6}, {NULL}};
const Segment SGI84458[] = {{&S43405[44], 18}, {NULL}};
const Segment SGI84459[] = {{&S43405[30], 6}, {NULL}};
const Segment SGI84460[] = {{&S43405[31], 3}, {NULL}};
const Segment SGI84461[] = {{&S43405[76], 6}, {NULL}};
const Segment SGI84462[] = {{&S43406[38], 6}, {NULL}};
const Segment SGI84463[] = {{&S43406[31], 6}, {NULL}};
const Segment SGI84464[] = {{&S43411[44], 29}, {NULL}};
const Segment SGI84465[] = {{&S43411[44], 41}, {NULL}};
const Segment SGI84466[] = {{&S43411[30], 6}, {NULL}};
const Segment SGI84467[] = {{&S43411[31], 3}, {NULL}};
const Segment SGI84468[] = {{&S43411[99], 6}, {NULL}};
const Segment SGI84469[] = {{&S43412[38], 29}, {NULL}};
const Segment SGI84470[] = {{&S43412[31], 6}, {NULL}};
const Segment SGI84471[] = {{&S43414[44], 17}, {NULL}};
const Segment SGI84472[] = {{&S43414[44], 29}, {NULL}};
const Segment SGI84473[] = {{&S43414[30], 6}, {NULL}};
const Segment SGI84474[] = {{&S43414[31], 3}, {NULL}};
const Segment SGI84475[] = {{&S43414[87], 6}, {NULL}};
const Segment SGI84476[] = {{&S50622[74], 6}, {NULL}};
const Segment SGI84477[] = {{&S50622[0], 9}, {NULL}};
const Segment SGI84478[] = {{&S43486[34], 11}, {NULL}};
const Segment SGI84479[] = {{&S43486[27], 6}, {NULL}};
const Segment SGI84480[] = {{&S43486[98], 9}, {NULL}};
const Segment SGI84481[] = {{&S43486[71], 6}, {NULL}};
const Segment SGI84482[] = {{&S43487[34], 11}, {NULL}};
const Segment SGI84483[] = {{&S43487[27], 6}, {NULL}};
const Segment SGI84484[] = {{&S43487[98], 9}, {NULL}};
const Segment SGI84485[] = {{&S43487[71], 6}, {NULL}};
const Segment SGI84486[] = {{&S43489[34], 10}, {NULL}};
const Segment SGI84487[] = {{&S43489[27], 6}, {NULL}};
const Segment SGI84488[] = {{&S43489[97], 9}, {NULL}};
const Segment SGI84489[] = {{&S43489[70], 6}, {NULL}};
const Segment SGI84490[] = {{&S43490[34], 11}, {NULL}};
const Segment SGI84491[] = {{&S43490[27], 6}, {NULL}};
const Segment SGI84492[] = {{&S43490[98], 9}, {NULL}};
const Segment SGI84493[] = {{&S43490[71], 6}, {NULL}};
const Segment SGI84494[] = {{&S43493[34], 10}, {NULL}};
const Segment SGI84495[] = {{&S43493[27], 6}, {NULL}};
const Segment SGI84496[] = {{&S43493[97], 9}, {NULL}};
const Segment SGI84497[] = {{&S43493[70], 6}, {NULL}};
const Segment SGI84498[] = {{&S63495[44], 10}, {NULL}};
const Segment SGI84499[] = {{&S63495[44], 27}, {NULL}};
const Segment SGI84500[] = {{&S63495[30], 6}, {NULL}};
const Segment SGI84501[] = {{&S63495[119], 3}, {NULL}};
const Segment SGI84502[] = {{&S63495[85], 5}, {NULL}};
const Segment SGI84503[] = {{&S63496[44], 10}, {NULL}};
const Segment SGI84504[] = {{&S63496[44], 27}, {NULL}};
const Segment SGI84505[] = {{&S63496[30], 6}, {NULL}};
const Segment SGI84506[] = {{&S63496[88], 4}, {NULL}};
const Segment SGI84507[] = {{&S63496[88], 3}, {NULL}};
const Segment SGI84508[] = {{&S63497[44], 10}, {NULL}};
const Segment SGI84509[] = {{&S63497[44], 27}, {NULL}};
const Segment SGI84510[] = {{&S63497[30], 6}, {NULL}};
const Segment SGI84511[] = {{&S63497[88], 6}, {NULL}};
const Segment SGI84512[] = {{&S63498[44], 27}, {NULL}};
const Segment SGI84513[] = {{&S63498[30], 6}, {NULL}};
const Segment SGI84514[] = {{&S63499[38], 10}, {NULL}};
const Segment SGI84515[] = {{&S63499[31], 6}, {NULL}};
const Segment SGI84516[] = {{&S63554[44], 10}, {NULL}};
const Segment SGI84517[] = {{&S63554[44], 27}, {NULL}};
const Segment SGI84518[] = {{&S63554[30], 6}, {NULL}};
const Segment SGI84519[] = {{&S63554[119], 3}, {NULL}};
const Segment SGI84520[] = {{&S63554[85], 5}, {NULL}};
const Segment SGI84521[] = {{&S63555[44], 10}, {NULL}};
const Segment SGI84522[] = {{&S63555[44], 27}, {NULL}};
const Segment SGI84523[] = {{&S63555[30], 6}, {NULL}};
const Segment SGI84524[] = {{&S63555[88], 6}, {NULL}};
const Segment SGI84525[] = {{&S63556[38], 10}, {NULL}};
const Segment SGI84526[] = {{&S63556[31], 6}, {NULL}};
const Segment SGI84527[] = {{&S63557[44], 27}, {NULL}};
const Segment SGI84528[] = {{&S63557[30], 6}, {NULL}};
const Segment SGI84529[] = {{&S43682[34], 10}, {NULL}};
const Segment SGI84530[] = {{&S43682[27], 6}, {NULL}};
const Segment SGI84531[] = {{&S43682[97], 9}, {NULL}};
const Segment SGI84532[] = {{&S43682[70], 6}, {NULL}};
const Segment SGI84533[] = {{&S43684[34], 11}, {NULL}};
const Segment SGI84534[] = {{&S43684[27], 6}, {NULL}};
const Segment SGI84535[] = {{&S43684[98], 9}, {NULL}};
const Segment SGI84536[] = {{&S43684[71], 6}, {NULL}};
const Segment SGI84537[] = {{&S43685[34], 11}, {NULL}};
const Segment SGI84538[] = {{&S43685[27], 6}, {NULL}};
const Segment SGI84539[] = {{&S43685[98], 9}, {NULL}};
const Segment SGI84540[] = {{&S43685[71], 6}, {NULL}};
const Segment SGI84541[] = {{&S43688[34], 12}, {NULL}};
const Segment SGI84542[] = {{&S43688[27], 6}, {NULL}};
const Segment SGI84543[] = {{&S43688[99], 9}, {NULL}};
const Segment SGI84544[] = {{&S43688[72], 6}, {NULL}};
const Segment SGI84545[] = {{&S55606[42], 10}, {NULL}};
const Segment SGI84546[] = {{&S55606[42], 22}, {NULL}};
const Segment SGI84547[] = {{&S55606[30], 4}, {NULL}};
const Segment SGI84548[] = {{&S55606[113], 3}, {NULL}};
const Segment SGI84549[] = {{&S55606[78], 6}, {NULL}};
const Segment SGI84550[] = {{&S55607[36], 10}, {NULL}};
const Segment SGI84551[] = {{&S55607[31], 4}, {NULL}};
const Segment SGI84552[] = {{&S63601[44], 4}, {NULL}};
const Segment SGI84553[] = {{&S63601[44], 16}, {NULL}};
const Segment SGI84554[] = {{&S63601[30], 6}, {NULL}};
const Segment SGI84555[] = {{&S63601[108], 3}, {NULL}};
const Segment SGI84556[] = {{&S63601[74], 5}, {NULL}};
const Segment SGI84557[] = {{&S63602[38], 4}, {NULL}};
const Segment SGI84558[] = {{&S63602[31], 6}, {NULL}};
const Segment SGI84559[] = {{&S63665[44], 5}, {NULL}};
const Segment SGI84560[] = {{&S63665[44], 27}, {NULL}};
const Segment SGI84561[] = {{&S63665[30], 6}, {NULL}};
const Segment SGI84562[] = {{&S63665[119], 3}, {NULL}};
const Segment SGI84563[] = {{&S63665[85], 5}, {NULL}};
const Segment SGI84564[] = {{&S63666[44], 16}, {NULL}};
const Segment SGI84565[] = {{&S63666[44], 38}, {NULL}};
const Segment SGI84566[] = {{&S63666[30], 6}, {NULL}};
const Segment SGI84567[] = {{&S63666[130], 3}, {NULL}};
const Segment SGI84568[] = {{&S63666[96], 5}, {NULL}};
const Segment SGI84569[] = {{&S55638[42], 6}, {NULL}};
const Segment SGI84570[] = {{&S55638[42], 17}, {NULL}};
const Segment SGI84571[] = {{&S55638[30], 4}, {NULL}};
const Segment SGI84572[] = {{&S55638[108], 3}, {NULL}};
const Segment SGI84573[] = {{&S55638[73], 6}, {NULL}};
const Segment SGI84574[] = {{&S55639[44], 6}, {NULL}};
const Segment SGI84575[] = {{&S55639[44], 18}, {NULL}};
const Segment SGI84576[] = {{&S55639[30], 6}, {NULL}};
const Segment SGI84577[] = {{&S55639[111], 3}, {NULL}};
const Segment SGI84578[] = {{&S55639[76], 6}, {NULL}};
const Segment SGI84579[] = {{&S55640[38], 6}, {NULL}};
const Segment SGI84580[] = {{&S55640[31], 6}, {NULL}};
const Segment SGI84581[] = {{&S43853[44], 12}, {NULL}};
const Segment SGI84582[] = {{&S43853[44], 24}, {NULL}};
const Segment SGI84583[] = {{&S43853[30], 6}, {NULL}};
const Segment SGI84584[] = {{&S43853[31], 3}, {NULL}};
const Segment SGI84585[] = {{&S43853[82], 6}, {NULL}};
const Segment SGI84586[] = {{&S43854[44], 12}, {NULL}};
const Segment SGI84587[] = {{&S43854[44], 24}, {NULL}};
const Segment SGI84588[] = {{&S43854[30], 6}, {NULL}};
const Segment SGI84589[] = {{&S43854[31], 3}, {NULL}};
const Segment SGI84590[] = {{&S43854[82], 6}, {NULL}};
const Segment SGI84591[] = {{&S43855[38], 12}, {NULL}};
const Segment SGI84592[] = {{&S43855[31], 6}, {NULL}};
const Segment SGI84593[] = {{&S43856[44], 23}, {NULL}};
const Segment SGI84594[] = {{&S43856[44], 35}, {NULL}};
const Segment SGI84595[] = {{&S43856[30], 6}, {NULL}};
const Segment SGI84596[] = {{&S43856[31], 3}, {NULL}};
const Segment SGI84597[] = {{&S43856[93], 6}, {NULL}};
const Segment SGI84598[] = {{&S43861[44], 23}, {NULL}};
const Segment SGI84599[] = {{&S43861[44], 35}, {NULL}};
const Segment SGI84600[] = {{&S43861[30], 6}, {NULL}};
const Segment SGI84601[] = {{&S43861[31], 3}, {NULL}};
const Segment SGI84602[] = {{&S43861[93], 6}, {NULL}};
const Segment SGI84603[] = {{&S43892[44], 9}, {NULL}};
const Segment SGI84604[] = {{&S43892[44], 21}, {NULL}};
const Segment SGI84605[] = {{&S43892[30], 6}, {NULL}};
const Segment SGI84606[] = {{&S43892[31], 3}, {NULL}};
const Segment SGI84607[] = {{&S43892[79], 6}, {NULL}};
const Segment SGI84608[] = {{&S43893[44], 12}, {NULL}};
const Segment SGI84609[] = {{&S43893[44], 24}, {NULL}};
const Segment SGI84610[] = {{&S43893[30], 6}, {NULL}};
const Segment SGI84611[] = {{&S43893[31], 3}, {NULL}};
const Segment SGI84612[] = {{&S43893[82], 6}, {NULL}};
const Segment SGI84613[] = {{&S43894[44], 11}, {NULL}};
const Segment SGI84614[] = {{&S43894[44], 23}, {NULL}};
const Segment SGI84615[] = {{&S43894[30], 6}, {NULL}};
const Segment SGI84616[] = {{&S43894[31], 3}, {NULL}};
const Segment SGI84617[] = {{&S43894[81], 6}, {NULL}};
const Segment SGI84618[] = {{&S43895[38], 9}, {NULL}};
const Segment SGI84619[] = {{&S43895[31], 6}, {NULL}};
const Segment SGI84620[] = {{&S63808[44], 11}, {NULL}};
const Segment SGI84621[] = {{&S63808[44], 28}, {NULL}};
const Segment SGI84622[] = {{&S63808[30], 6}, {NULL}};
const Segment SGI84623[] = {{&S63808[120], 3}, {NULL}};
const Segment SGI84624[] = {{&S63808[86], 5}, {NULL}};
const Segment SGI84625[] = {{&S50867[44], 12}, {NULL}};
const Segment SGI84626[] = {{&S50867[44], 23}, {NULL}};
const Segment SGI84627[] = {{&S50867[30], 6}, {NULL}};
const Segment SGI84628[] = {{&S50867[31], 3}, {NULL}};
const Segment SGI84629[] = {{&S50867[81], 6}, {NULL}};
const Segment SGI84630[] = {{&S50867[0], 13}, {NULL}};
const Segment SGI84631[] = {{&S64003[45], 11}, {NULL}};
const Segment SGI84632[] = {{&S64003[30], 6}, {NULL}};
const Segment SGI84633[] = {{&S64003[121], 3}, {NULL}};
const Segment SGI84634[] = {{&S64003[87], 5}, {NULL}};
const Segment SGI84635[] = {{&S64003[145], 13}, {NULL}};
const Segment SGI84636[] = {{&S44016[34], 15}, {NULL}};
const Segment SGI84637[] = {{&S44016[27], 6}, {NULL}};
const Segment SGI84638[] = {{&S44016[102], 9}, {NULL}};
const Segment SGI84639[] = {{&S44016[75], 6}, {NULL}};
const Segment SGI84640[] = {{&S44023[34], 14}, {NULL}};
const Segment SGI84641[] = {{&S44023[27], 6}, {NULL}};
const Segment SGI84642[] = {{&S44023[101], 9}, {NULL}};
const Segment SGI84643[] = {{&S44023[74], 6}, {NULL}};
const Segment SGI84644[] = {{&S56742[44], 14}, {NULL}};
const Segment SGI84645[] = {{&S56742[44], 25}, {NULL}};
const Segment SGI84646[] = {{&S56742[30], 6}, {NULL}};
const Segment SGI84647[] = {{&S44067[34], 10}, {NULL}};
const Segment SGI84648[] = {{&S44067[27], 6}, {NULL}};
const Segment SGI84649[] = {{&S44067[97], 9}, {NULL}};
const Segment SGI84650[] = {{&S44067[70], 6}, {NULL}};
const Segment SGI84651[] = {{&S56742[136], 3}, {NULL}};
const Segment SGI84652[] = {{&S56742[102], 5}, {NULL}};
const Segment SGI84653[] = {{&S56742[35], 1}, {NULL}};
const Segment SGI84654[] = {{&S56742[71], 11}, {NULL}};
const Segment SGI84655[] = {{&S53699[42], 6}, {NULL}};
const Segment SGI84656[] = {{&S53699[42], 18}, {NULL}};
const Segment SGI84657[] = {{&S53699[30], 4}, {NULL}};
const Segment SGI84658[] = {{&S53699[31], 3}, {NULL}};
const Segment SGI84659[] = {{&S53699[74], 6}, {NULL}};
const Segment SGI84660[] = {{&S53699[127], 6}, {NULL}};
const Segment SGI84661[] = {{&S56851[44], 9}, {NULL}};
const Segment SGI84662[] = {{&S56851[44], 20}, {NULL}};
const Segment SGI84663[] = {{&S56851[30], 6}, {NULL}};
const Segment SGI84664[] = {{&S64181[45], 11}, {NULL}};
const Segment SGI84665[] = {{&S64181[30], 6}, {NULL}};
const Segment SGI84666[] = {{&S64181[121], 3}, {NULL}};
const Segment SGI84667[] = {{&S64181[87], 5}, {NULL}};
const Segment SGI84668[] = {{&S64181[145], 13}, {NULL}};
const Segment SGI84669[] = {{&S56177[44], 14}, {NULL}};
const Segment SGI84670[] = {{&S56177[44], 25}, {NULL}};
const Segment SGI84671[] = {{&S56177[30], 6}, {NULL}};
const Segment SGI84672[] = {{&S56177[118], 3}, {NULL}};
const Segment SGI84673[] = {{&S56177[83], 6}, {NULL}};
const Segment SGI84674[] = {{&S56178[42], 14}, {NULL}};
const Segment SGI84675[] = {{&S56178[42], 26}, {NULL}};
const Segment SGI84676[] = {{&S56178[30], 4}, {NULL}};
const Segment SGI84677[] = {{&S56178[117], 3}, {NULL}};
const Segment SGI84678[] = {{&S56178[82], 6}, {NULL}};
const Segment SGI84679[] = {{&S56179[38], 14}, {NULL}};
const Segment SGI84680[] = {{&S56179[31], 6}, {NULL}};
const Segment SGI84681[] = {{&S56851[131], 3}, {NULL}};
const Segment SGI84682[] = {{&S56851[97], 5}, {NULL}};
const Segment SGI84683[] = {{&S56851[35], 1}, {NULL}};
const Segment SGI84684[] = {{&S56851[66], 11}, {NULL}};
const Segment SGI84685[] = {{&S56852[44], 9}, {NULL}};
const Segment SGI84686[] = {{&S56852[44], 20}, {NULL}};
const Segment SGI84687[] = {{&S56852[30], 6}, {NULL}};
const Segment SGI84688[] = {{&S56852[154], 3}, {NULL}};
const Segment SGI84689[] = {{&S56852[120], 5}, {NULL}};
const Segment SGI84690[] = {{&S56852[85], 11}, {NULL}};
const Segment SGI84691[] = {{&S44220[44], 10}, {NULL}};
const Segment SGI84692[] = {{&S44220[44], 22}, {NULL}};
const Segment SGI84693[] = {{&S44220[30], 6}, {NULL}};
const Segment SGI84694[] = {{&S44220[31], 3}, {NULL}};
const Segment SGI84695[] = {{&S44220[80], 6}, {NULL}};
const Segment SGI84696[] = {{&S44221[44], 10}, {NULL}};
const Segment SGI84697[] = {{&S44221[44], 22}, {NULL}};
const Segment SGI84698[] = {{&S44221[30], 6}, {NULL}};
const Segment SGI84699[] = {{&S44221[31], 3}, {NULL}};
const Segment SGI84700[] = {{&S44221[80], 6}, {NULL}};
const Segment SGI84701[] = {{&S44222[44], 10}, {NULL}};
const Segment SGI84702[] = {{&S44222[44], 22}, {NULL}};
const Segment SGI84703[] = {{&S44222[30], 6}, {NULL}};
const Segment SGI84704[] = {{&S44222[31], 3}, {NULL}};
const Segment SGI84705[] = {{&S44222[80], 6}, {NULL}};
const Segment SGI84706[] = {{&S44223[38], 10}, {NULL}};
const Segment SGI84707[] = {{&S44223[31], 6}, {NULL}};
const Segment SGI84708[] = {{&S44225[38], 10}, {NULL}};
const Segment SGI84709[] = {{&S44225[31], 6}, {NULL}};
const Segment SGI84710[] = {{&S44226[44], 17}, {NULL}};
const Segment SGI84711[] = {{&S44226[44], 29}, {NULL}};
const Segment SGI84712[] = {{&S44226[30], 6}, {NULL}};
const Segment SGI84713[] = {{&S44226[90], 6}, {NULL}};
const Segment SGI84714[] = {{&S44229[38], 10}, {NULL}};
const Segment SGI84715[] = {{&S44229[31], 6}, {NULL}};
const Segment SGI84716[] = {{&S44230[44], 9}, {NULL}};
const Segment SGI84717[] = {{&S44230[44], 21}, {NULL}};
const Segment SGI84718[] = {{&S44230[30], 6}, {NULL}};
const Segment SGI84719[] = {{&S44230[31], 3}, {NULL}};
const Segment SGI84720[] = {{&S44230[79], 6}, {NULL}};
const Segment SGI84721[] = {{&S44233[44], 9}, {NULL}};
const Segment SGI84722[] = {{&S44233[44], 21}, {NULL}};
const Segment SGI84723[] = {{&S44233[30], 6}, {NULL}};
const Segment SGI84724[] = {{&S44233[82], 6}, {NULL}};
const Segment SGI84725[] = {{&S44248[34], 10}, {NULL}};
const Segment SGI84726[] = {{&S44248[27], 6}, {NULL}};
const Segment SGI84727[] = {{&S44248[97], 9}, {NULL}};
const Segment SGI84728[] = {{&S44248[70], 6}, {NULL}};
const Segment SGI84729[] = {{&S44251[34], 11}, {NULL}};
const Segment SGI84730[] = {{&S44251[27], 6}, {NULL}};
const Segment SGI84731[] = {{&S44251[98], 9}, {NULL}};
const Segment SGI84732[] = {{&S44251[71], 6}, {NULL}};
const Segment SGI84733[] = {{&S44253[34], 10}, {NULL}};
const Segment SGI84734[] = {{&S44253[27], 6}, {NULL}};
const Segment SGI84735[] = {{&S44253[97], 9}, {NULL}};
const Segment SGI84736[] = {{&S44253[70], 6}, {NULL}};
const Segment SGI84737[] = {{&S44281[34], 7}, {NULL}};
const Segment SGI84738[] = {{&S44281[27], 6}, {NULL}};
const Segment SGI84739[] = {{&S44281[100], 9}, {NULL}};
const Segment SGI84740[] = {{&S44281[73], 6}, {NULL}};
const Segment SGI84741[] = {{&S56877[44], 7}, {NULL}};
const Segment SGI84742[] = {{&S56877[44], 17}, {NULL}};
const Segment SGI84743[] = {{&S56877[30], 6}, {NULL}};
const Segment SGI84744[] = {{&S56877[109], 3}, {NULL}};
const Segment SGI84745[] = {{&S56877[75], 5}, {NULL}};
const Segment SGI84746[] = {{&S56878[44], 7}, {NULL}};
const Segment SGI84747[] = {{&S56878[44], 17}, {NULL}};
const Segment SGI84748[] = {{&S56878[30], 6}, {NULL}};
const Segment SGI84749[] = {{&S56878[78], 4}, {NULL}};
const Segment SGI84750[] = {{&S56878[78], 3}, {NULL}};
const Segment SGI84751[] = {{&S56880[44], 7}, {NULL}};
const Segment SGI84752[] = {{&S56880[44], 20}, {NULL}};
const Segment SGI84753[] = {{&S56880[30], 6}, {NULL}};
const Segment SGI84754[] = {{&S56880[112], 3}, {NULL}};
const Segment SGI84755[] = {{&S56880[78], 5}, {NULL}};
const Segment SGI84756[] = {{&S56881[38], 7}, {NULL}};
const Segment SGI84757[] = {{&S56881[31], 6}, {NULL}};
const Segment SGI84758[] = {{&S56882[44], 7}, {NULL}};
const Segment SGI84759[] = {{&S56882[44], 17}, {NULL}};
const Segment SGI84760[] = {{&S56882[30], 6}, {NULL}};
const Segment SGI84761[] = {{&S56882[78], 6}, {NULL}};
const Segment SGI84762[] = {{&S56885[44], 7}, {NULL}};
const Segment SGI84763[] = {{&S56885[44], 17}, {NULL}};
const Segment SGI84764[] = {{&S56885[30], 6}, {NULL}};
const Segment SGI84765[] = {{&S56885[109], 3}, {NULL}};
const Segment SGI84766[] = {{&S56885[75], 5}, {NULL}};
const Segment SGI84767[] = {{&S56886[44], 8}, {NULL}};
const Segment SGI84768[] = {{&S56886[44], 18}, {NULL}};
const Segment SGI84769[] = {{&S56886[30], 6}, {NULL}};
const Segment SGI84770[] = {{&S56886[110], 3}, {NULL}};
const Segment SGI84771[] = {{&S56886[76], 5}, {NULL}};
const Segment SGI84772[] = {{&S56887[44], 7}, {NULL}};
const Segment SGI84773[] = {{&S56887[44], 17}, {NULL}};
const Segment SGI84774[] = {{&S56887[30], 6}, {NULL}};
const Segment SGI84775[] = {{&S56887[78], 6}, {NULL}};
const Segment SGI84776[] = {{&S64583[34], 10}, {NULL}};
const Segment SGI84777[] = {{&S64583[27], 6}, {NULL}};
const Segment SGI84778[] = {{&S64583[109], 2}, {NULL}};
const Segment SGI84779[] = {{&S64583[75], 6}, {NULL}};
const Segment SGI84780[] = {{&S64583[143], 8}, {NULL}};
const Segment SGI84781[] = {{&S64584[34], 11}, {NULL}};
const Segment SGI84782[] = {{&S64584[27], 6}, {NULL}};
const Segment SGI84783[] = {{&S64584[110], 2}, {NULL}};
const Segment SGI84784[] = {{&S64584[76], 6}, {NULL}};
const Segment SGI84785[] = {{&S64584[144], 8}, {NULL}};
const Segment SGI84786[] = {{&S64600[44], 4}, {NULL}};
const Segment SGI84787[] = {{&S64600[44], 16}, {NULL}};
const Segment SGI84788[] = {{&S64600[30], 6}, {NULL}};
const Segment SGI84789[] = {{&S64600[108], 3}, {NULL}};
const Segment SGI84790[] = {{&S64600[74], 5}, {NULL}};
const Segment SGI84791[] = {{&S64601[44], 4}, {NULL}};
const Segment SGI84792[] = {{&S64601[44], 25}, {NULL}};
const Segment SGI84793[] = {{&S64601[30], 6}, {NULL}};
const Segment SGI84794[] = {{&S64601[117], 3}, {NULL}};
const Segment SGI84795[] = {{&S64601[83], 5}, {NULL}};
const Segment SGI84796[] = {{&S64639[44], 5}, {NULL}};
const Segment SGI84797[] = {{&S64639[44], 17}, {NULL}};
const Segment SGI84798[] = {{&S64639[30], 6}, {NULL}};
const Segment SGI84799[] = {{&S64639[109], 3}, {NULL}};
const Segment SGI84800[] = {{&S64639[75], 5}, {NULL}};
const Segment SGI84801[] = {{&S64674[44], 11}, {NULL}};
const Segment SGI84802[] = {{&S64674[44], 23}, {NULL}};
const Segment SGI84803[] = {{&S64674[30], 6}, {NULL}};
const Segment SGI84804[] = {{&S64674[115], 3}, {NULL}};
const Segment SGI84805[] = {{&S64674[81], 5}, {NULL}};
const Segment SGI84806[] = {{&S64675[44], 10}, {NULL}};
const Segment SGI84807[] = {{&S64675[44], 22}, {NULL}};
const Segment SGI84808[] = {{&S64675[30], 6}, {NULL}};
const Segment SGI84809[] = {{&S64675[114], 3}, {NULL}};
const Segment SGI84810[] = {{&S64675[80], 5}, {NULL}};
const Segment SGI84811[] = {{&S64676[44], 6}, {NULL}};
const Segment SGI84812[] = {{&S64676[44], 18}, {NULL}};
const Segment SGI84813[] = {{&S64676[30], 6}, {NULL}};
const Segment SGI84814[] = {{&S64676[79], 6}, {NULL}};
const Segment SGI84815[] = {{&S64677[38], 6}, {NULL}};
const Segment SGI84816[] = {{&S64677[31], 6}, {NULL}};
const Segment SGI84817[] = {{&S64746[44], 7}, {NULL}};
const Segment SGI84818[] = {{&S64746[44], 19}, {NULL}};
const Segment SGI84819[] = {{&S64746[30], 6}, {NULL}};
const Segment SGI84820[] = {{&S64746[111], 3}, {NULL}};
const Segment SGI84821[] = {{&S64746[77], 5}, {NULL}};
const Segment SGI84822[] = {{&S64747[38], 7}, {NULL}};
const Segment SGI84823[] = {{&S64747[31], 6}, {NULL}};
const Segment SGI84824[] = {{&S44612[44], 15}, {NULL}};
const Segment SGI84825[] = {{&S44612[44], 27}, {NULL}};
const Segment SGI84826[] = {{&S44612[30], 6}, {NULL}};
const Segment SGI84827[] = {{&S44612[31], 3}, {NULL}};
const Segment SGI84828[] = {{&S44612[85], 6}, {NULL}};
const Segment SGI84829[] = {{&S44613[38], 15}, {NULL}};
const Segment SGI84830[] = {{&S44613[31], 6}, {NULL}};
const Segment SGI84831[] = {{&S64888[44], 8}, {NULL}};
const Segment SGI84832[] = {{&S64888[44], 26}, {NULL}};
const Segment SGI84833[] = {{&S64888[30], 6}, {NULL}};
const Segment SGI84834[] = {{&S64888[118], 3}, {NULL}};
const Segment SGI84835[] = {{&S64888[84], 5}, {NULL}};
const Segment SGI84836[] = {{&S57326[44], 10}, {NULL}};
const Segment SGI84837[] = {{&S57326[44], 22}, {NULL}};
const Segment SGI84838[] = {{&S57326[30], 6}, {NULL}};
const Segment SGI84839[] = {{&S57326[129], 3}, {NULL}};
const Segment SGI84840[] = {{&S57326[95], 5}, {NULL}};
const Segment SGI84841[] = {{&S57326[35], 1}, {NULL}};
const Segment SGI84842[] = {{&S57326[68], 11}, {NULL}};
const Segment SGI84843[] = {{&S57327[44], 10}, {NULL}};
const Segment SGI84844[] = {{&S57327[44], 27}, {NULL}};
const Segment SGI84845[] = {{&S57327[30], 6}, {NULL}};
const Segment SGI84846[] = {{&S57327[138], 3}, {NULL}};
const Segment SGI84847[] = {{&S57327[104], 5}, {NULL}};
const Segment SGI84848[] = {{&S57327[35], 1}, {NULL}};
const Segment SGI84849[] = {{&S57327[73], 11}, {NULL}};
const Segment SGI84850[] = {{&S49597[6], 4}, {NULL}};
const Segment SGI84851[] = {{&S49597[83], 5}, {NULL}};
const Segment SGI84852[] = {{&S64929[45], 10}, {NULL}};
const Segment SGI84853[] = {{&S64929[30], 6}, {NULL}};
const Segment SGI84854[] = {{&S64929[120], 3}, {NULL}};
const Segment SGI84855[] = {{&S64929[86], 5}, {NULL}};
const Segment SGI84856[] = {{&S64929[144], 13}, {NULL}};
const Segment SGI84857[] = {{&S64930[44], 10}, {NULL}};
const Segment SGI84858[] = {{&S64930[44], 27}, {NULL}};
const Segment SGI84859[] = {{&S64930[30], 6}, {NULL}};
const Segment SGI84860[] = {{&S64930[120], 3}, {NULL}};
const Segment SGI84861[] = {{&S64930[85], 6}, {NULL}};
const Segment SGI84862[] = {{&S64930[140], 8}, {NULL}};
const Segment SGI84863[] = {{&S64930[149], 13}, {NULL}};
const Segment SGI84864[] = {{&S44825[34], 15}, {NULL}};
const Segment SGI84865[] = {{&S44825[27], 6}, {NULL}};
const Segment SGI84866[] = {{&S44825[104], 9}, {NULL}};
const Segment SGI84867[] = {{&S44825[77], 6}, {NULL}};
const Segment SGI84868[] = {{&S44826[34], 16}, {NULL}};
const Segment SGI84869[] = {{&S44826[27], 6}, {NULL}};
const Segment SGI84870[] = {{&S44826[105], 9}, {NULL}};
const Segment SGI84871[] = {{&S44826[78], 6}, {NULL}};
const Segment SGI84872[] = {{&S65036[44], 10}, {NULL}};
const Segment SGI84873[] = {{&S65036[44], 27}, {NULL}};
const Segment SGI84874[] = {{&S65036[30], 6}, {NULL}};
const Segment SGI84875[] = {{&S65036[85], 6}, {NULL}};
const Segment SGI84876[] = {{&S65036[112], 13}, {NULL}};
const Segment SGI84877[] = {{&S65038[44], 11}, {NULL}};
const Segment SGI84878[] = {{&S65038[44], 28}, {NULL}};
const Segment SGI84879[] = {{&S65038[30], 6}, {NULL}};
const Segment SGI84880[] = {{&S65038[86], 6}, {NULL}};
const Segment SGI84881[] = {{&S65038[113], 13}, {NULL}};
const Segment SGI84882[] = {{&S44852[44], 9}, {NULL}};
const Segment SGI84883[] = {{&S44852[44], 20}, {NULL}};
const Segment SGI84884[] = {{&S44852[30], 6}, {NULL}};
const Segment SGI84885[] = {{&S44852[31], 3}, {NULL}};
const Segment SGI84886[] = {{&S44852[78], 6}, {NULL}};
const Segment SGI84887[] = {{&S65198[44], 7}, {NULL}};
const Segment SGI84888[] = {{&S65198[44], 19}, {NULL}};
const Segment SGI84889[] = {{&S65198[30], 6}, {NULL}};
const Segment SGI84890[] = {{&S65198[111], 3}, {NULL}};
const Segment SGI84891[] = {{&S65198[77], 5}, {NULL}};
const Segment SGI84892[] = {{&S57655[6], 4}, {NULL}};
const Segment SGI84893[] = {{&S57655[85], 5}, {NULL}};
const Segment SGI84894[] = {{&S52287[44], 10}, {NULL}};
const Segment SGI84895[] = {{&S52287[44], 27}, {NULL}};
const Segment SGI84896[] = {{&S52287[30], 6}, {NULL}};
const Segment SGI84897[] = {{&S52287[31], 3}, {NULL}};
const Segment SGI84898[] = {{&S52287[85], 6}, {NULL}};
const Segment SGI84899[] = {{&S52287[0], 13}, {NULL}};
const Segment SGI84900[] = {{&S65229[59], 10}, {NULL}};
const Segment SGI84901[] = {{&S65229[59], 27}, {NULL}};
const Segment SGI84902[] = {{&S65229[45], 6}, {NULL}};
const Segment SGI84903[] = {{&S65229[134], 3}, {NULL}};
const Segment SGI84904[] = {{&S65229[100], 5}, {NULL}};
const Segment SGI84905[] = {{&S65229[0], 14}, {NULL}};
const Segment SGI84906[] = {{&S65231[57], 10}, {NULL}};
const Segment SGI84907[] = {{&S65231[45], 6}, {NULL}};
const Segment SGI84908[] = {{&S65231[132], 3}, {NULL}};
const Segment SGI84909[] = {{&S65231[98], 5}, {NULL}};
const Segment SGI84910[] = {{&S65231[0], 14}, {NULL}};
const Segment SGI84911[] = {{&S45037[44], 7}, {NULL}};
const Segment SGI84912[] = {{&S45037[44], 24}, {NULL}};
const Segment SGI84913[] = {{&S45037[30], 6}, {NULL}};
const Segment SGI84914[] = {{&S45037[31], 3}, {NULL}};
const Segment SGI84915[] = {{&S45037[82], 6}, {NULL}};
const Segment SGI84916[] = {{&S45038[44], 7}, {NULL}};
const Segment SGI84917[] = {{&S45038[44], 24}, {NULL}};
const Segment SGI84918[] = {{&S45038[30], 6}, {NULL}};
const Segment SGI84919[] = {{&S45038[85], 4}, {NULL}};
const Segment SGI84920[] = {{&S45038[85], 3}, {NULL}};
const Segment SGI84921[] = {{&S45039[38], 7}, {NULL}};
const Segment SGI84922[] = {{&S45039[31], 6}, {NULL}};
const Segment SGI84923[] = {{&S45040[44], 7}, {NULL}};
const Segment SGI84924[] = {{&S45040[44], 24}, {NULL}};
const Segment SGI84925[] = {{&S45040[30], 6}, {NULL}};
const Segment SGI84926[] = {{&S45040[85], 6}, {NULL}};
const Segment SGI84927[] = {{&S45041[44], 7}, {NULL}};
const Segment SGI84928[] = {{&S45041[44], 24}, {NULL}};
const Segment SGI84929[] = {{&S45041[30], 6}, {NULL}};
const Segment SGI84930[] = {{&S45041[82], 6}, {NULL}};
const Segment SGI84931[] = {{&S65262[45], 10}, {NULL}};
const Segment SGI84932[] = {{&S65262[30], 6}, {NULL}};
const Segment SGI84933[] = {{&S65262[120], 3}, {NULL}};
const Segment SGI84934[] = {{&S65262[86], 5}, {NULL}};
const Segment SGI84935[] = {{&S65262[144], 13}, {NULL}};
const Segment SGI84936[] = {{&S65263[45], 11}, {NULL}};
const Segment SGI84937[] = {{&S65263[30], 6}, {NULL}};
const Segment SGI84938[] = {{&S65263[121], 3}, {NULL}};
const Segment SGI84939[] = {{&S65263[87], 5}, {NULL}};
const Segment SGI84940[] = {{&S65263[145], 13}, {NULL}};
const Segment SGI84941[] = {{&S65274[44], 12}, {NULL}};
const Segment SGI84942[] = {{&S65274[44], 24}, {NULL}};
const Segment SGI84943[] = {{&S65274[30], 6}, {NULL}};
const Segment SGI84944[] = {{&S57713[6], 4}, {NULL}};
const Segment SGI84945[] = {{&S57713[78], 5}, {NULL}};
const Segment SGI84946[] = {{&S65274[116], 3}, {NULL}};
const Segment SGI84947[] = {{&S65274[82], 5}, {NULL}};
const Segment SGI84948[] = {{&S65275[38], 12}, {NULL}};
const Segment SGI84949[] = {{&S65275[31], 6}, {NULL}};
const Segment SGI84950[] = {{&S65358[44], 10}, {NULL}};
const Segment SGI84951[] = {{&S65358[44], 27}, {NULL}};
const Segment SGI84952[] = {{&S65358[30], 6}, {NULL}};
const Segment SGI84953[] = {{&S65358[119], 3}, {NULL}};
const Segment SGI84954[] = {{&S65358[85], 5}, {NULL}};
const Segment SGI84955[] = {{&S65359[44], 10}, {NULL}};
const Segment SGI84956[] = {{&S65359[44], 27}, {NULL}};
const Segment SGI84957[] = {{&S65359[30], 6}, {NULL}};
const Segment SGI84958[] = {{&S65359[88], 6}, {NULL}};
const Segment SGI84959[] = {{&S65360[38], 10}, {NULL}};
const Segment SGI84960[] = {{&S65360[31], 6}, {NULL}};
const Segment SGI84961[] = {{&S65368[44], 19}, {NULL}};
const Segment SGI84962[] = {{&S65368[44], 31}, {NULL}};
const Segment SGI84963[] = {{&S65368[30], 6}, {NULL}};
const Segment SGI84964[] = {{&S65368[123], 3}, {NULL}};
const Segment SGI84965[] = {{&S65368[89], 5}, {NULL}};
const Segment SGI84966[] = {{&S65369[44], 20}, {NULL}};
const Segment SGI84967[] = {{&S65369[44], 32}, {NULL}};
const Segment SGI84968[] = {{&S65369[30], 6}, {NULL}};
const Segment SGI84969[] = {{&S65369[124], 3}, {NULL}};
const Segment SGI84970[] = {{&S65369[90], 5}, {NULL}};
const Segment SGI84971[] = {{&S65395[44], 13}, {NULL}};
const Segment SGI84972[] = {{&S65395[44], 25}, {NULL}};
const Segment SGI84973[] = {{&S65395[30], 6}, {NULL}};
const Segment SGI84974[] = {{&S65395[86], 4}, {NULL}};
const Segment SGI84975[] = {{&S65395[86], 3}, {NULL}};
const Segment SGI84976[] = {{&S65396[44], 13}, {NULL}};
const Segment SGI84977[] = {{&S65396[44], 25}, {NULL}};
const Segment SGI84978[] = {{&S65396[30], 6}, {NULL}};
const Segment SGI84979[] = {{&S65396[117], 3}, {NULL}};
const Segment SGI84980[] = {{&S65396[83], 5}, {NULL}};
const Segment SGI84981[] = {{&S65397[44], 13}, {NULL}};
const Segment SGI84982[] = {{&S65397[44], 25}, {NULL}};
const Segment SGI84983[] = {{&S65397[30], 6}, {NULL}};
const Segment SGI84984[] = {{&S65397[86], 6}, {NULL}};
const Segment SGI84985[] = {{&S65399[38], 13}, {NULL}};
const Segment SGI84986[] = {{&S65399[31], 6}, {NULL}};
const Segment SGI84987[] = {{&S65400[44], 25}, {NULL}};
const Segment SGI84988[] = {{&S65400[30], 6}, {NULL}};
const Segment SGI84989[] = {{&S57917[44], 19}, {NULL}};
const Segment SGI84990[] = {{&S57917[44], 36}, {NULL}};
const Segment SGI84991[] = {{&S57917[30], 6}, {NULL}};
const Segment SGI84992[] = {{&S57917[128], 3}, {NULL}};
const Segment SGI84993[] = {{&S57917[94], 5}, {NULL}};
const Segment SGI84994[] = {{&S57918[44], 19}, {NULL}};
const Segment SGI84995[] = {{&S57918[44], 36}, {NULL}};
const Segment SGI84996[] = {{&S57918[30], 6}, {NULL}};
const Segment SGI84997[] = {{&S57918[128], 3}, {NULL}};
const Segment SGI84998[] = {{&S57918[94], 5}, {NULL}};
const Segment SGI84999[] = {{&S57919[44], 19}, {NULL}};
const Segment SGI85000[] = {{&S57919[44], 36}, {NULL}};
const Segment SGI85001[] = {{&S57919[30], 6}, {NULL}};
const Segment SGI85002[] = {{&S57919[128], 3}, {NULL}};
const Segment SGI85003[] = {{&S57919[94], 5}, {NULL}};
const Segment SGI85004[] = {{&S57920[44], 7}, {NULL}};
const Segment SGI85005[] = {{&S57920[44], 24}, {NULL}};
const Segment SGI85006[] = {{&S57920[30], 6}, {NULL}};
const Segment SGI85007[] = {{&S57920[116], 3}, {NULL}};
const Segment SGI85008[] = {{&S57920[82], 5}, {NULL}};
const Segment SGI85009[] = {{&S57921[44], 7}, {NULL}};
const Segment SGI85010[] = {{&S57921[44], 24}, {NULL}};
const Segment SGI85011[] = {{&S57921[30], 6}, {NULL}};
const Segment SGI85012[] = {{&S57921[116], 3}, {NULL}};
const Segment SGI85013[] = {{&S57921[82], 5}, {NULL}};
const Segment SGI85014[] = {{&S57922[44], 19}, {NULL}};
const Segment SGI85015[] = {{&S57922[44], 36}, {NULL}};
const Segment SGI85016[] = {{&S57922[30], 6}, {NULL}};
const Segment SGI85017[] = {{&S57922[128], 3}, {NULL}};
const Segment SGI85018[] = {{&S57922[94], 5}, {NULL}};
const Segment SGI85019[] = {{&S57923[44], 7}, {NULL}};
const Segment SGI85020[] = {{&S57923[44], 24}, {NULL}};
const Segment SGI85021[] = {{&S57923[30], 6}, {NULL}};
const Segment SGI85022[] = {{&S57923[116], 3}, {NULL}};
const Segment SGI85023[] = {{&S57923[82], 5}, {NULL}};
const Segment SGI85024[] = {{&S57924[44], 19}, {NULL}};
const Segment SGI85025[] = {{&S57924[44], 36}, {NULL}};
const Segment SGI85026[] = {{&S57924[30], 6}, {NULL}};
const Segment SGI85027[] = {{&S57924[128], 3}, {NULL}};
const Segment SGI85028[] = {{&S57924[94], 5}, {NULL}};
const Segment SGI85029[] = {{&S57925[44], 7}, {NULL}};
const Segment SGI85030[] = {{&S57925[44], 24}, {NULL}};
const Segment SGI85031[] = {{&S57925[30], 6}, {NULL}};
const Segment SGI85032[] = {{&S57925[116], 3}, {NULL}};
const Segment SGI85033[] = {{&S57925[82], 5}, {NULL}};
const Segment SGI85034[] = {{&S57926[38], 7}, {NULL}};
const Segment SGI85035[] = {{&S57926[31], 6}, {NULL}};
const Segment SGI85036[] = {{&S65509[44], 10}, {NULL}};
const Segment SGI85037[] = {{&S65509[44], 27}, {NULL}};
const Segment SGI85038[] = {{&S65509[30], 6}, {NULL}};
const Segment SGI85039[] = {{&S65509[85], 6}, {NULL}};
const Segment SGI85040[] = {{&S65509[112], 13}, {NULL}};
const Segment SGI85041[] = {{&S65513[44], 11}, {NULL}};
const Segment SGI85042[] = {{&S65513[44], 28}, {NULL}};
const Segment SGI85043[] = {{&S65513[30], 6}, {NULL}};
const Segment SGI85044[] = {{&S65513[86], 6}, {NULL}};
const Segment SGI85045[] = {{&S65513[113], 13}, {NULL}};
const Segment SGI85046[] = {{&S65564[44], 13}, {NULL}};
const Segment SGI85047[] = {{&S65564[44], 25}, {NULL}};
const Segment SGI85048[] = {{&S65564[30], 6}, {NULL}};
const Segment SGI85049[] = {{&S65564[117], 3}, {NULL}};
const Segment SGI85050[] = {{&S65564[83], 5}, {NULL}};
const Segment SGI85051[] = {{&S57479[42], 9}, {NULL}};
const Segment SGI85052[] = {{&S57479[42], 20}, {NULL}};
const Segment SGI85053[] = {{&S57479[30], 4}, {NULL}};
const Segment SGI85054[] = {{&S57479[111], 3}, {NULL}};
const Segment SGI85055[] = {{&S57479[76], 6}, {NULL}};
const Segment SGI85056[] = {{&S57480[42], 14}, {NULL}};
const Segment SGI85057[] = {{&S57480[42], 25}, {NULL}};
const Segment SGI85058[] = {{&S57480[30], 4}, {NULL}};
const Segment SGI85059[] = {{&S57480[116], 3}, {NULL}};
const Segment SGI85060[] = {{&S57480[81], 6}, {NULL}};
const Segment SGI85061[] = {{&S57481[36], 9}, {NULL}};
const Segment SGI85062[] = {{&S57481[31], 4}, {NULL}};
const Segment SGI85063[] = {{&S57491[44], 15}, {NULL}};
const Segment SGI85064[] = {{&S57491[44], 31}, {NULL}};
const Segment SGI85065[] = {{&S57491[30], 6}, {NULL}};
const Segment SGI85066[] = {{&S57491[124], 3}, {NULL}};
const Segment SGI85067[] = {{&S57491[89], 6}, {NULL}};
const Segment SGI85068[] = {{&S57492[44], 15}, {NULL}};
const Segment SGI85069[] = {{&S57492[44], 30}, {NULL}};
const Segment SGI85070[] = {{&S57492[30], 6}, {NULL}};
const Segment SGI85071[] = {{&S57492[123], 3}, {NULL}};
const Segment SGI85072[] = {{&S57492[88], 6}, {NULL}};
const Segment SGI85073[] = {{&S57493[42], 15}, {NULL}};
const Segment SGI85074[] = {{&S57493[42], 31}, {NULL}};
const Segment SGI85075[] = {{&S57493[30], 4}, {NULL}};
const Segment SGI85076[] = {{&S57493[122], 3}, {NULL}};
const Segment SGI85077[] = {{&S57493[87], 6}, {NULL}};
const Segment SGI85078[] = {{&S57494[44], 15}, {NULL}};
const Segment SGI85079[] = {{&S57494[44], 31}, {NULL}};
const Segment SGI85080[] = {{&S57494[30], 6}, {NULL}};
const Segment SGI85081[] = {{&S57494[124], 3}, {NULL}};
const Segment SGI85082[] = {{&S57494[89], 6}, {NULL}};
const Segment SGI85083[] = {{&S65603[45], 10}, {NULL}};
const Segment SGI85084[] = {{&S65603[30], 6}, {NULL}};
const Segment SGI85085[] = {{&S65603[120], 3}, {NULL}};
const Segment SGI85086[] = {{&S65603[86], 5}, {NULL}};
const Segment SGI85087[] = {{&S57495[42], 15}, {NULL}};
const Segment SGI85088[] = {{&S57495[42], 30}, {NULL}};
const Segment SGI85089[] = {{&S65603[144], 13}, {NULL}};
const Segment SGI85090[] = {{&S57495[30], 4}, {NULL}};
const Segment SGI85091[] = {{&S57495[121], 3}, {NULL}};
const Segment SGI85092[] = {{&S57495[86], 6}, {NULL}};
const Segment SGI85093[] = {{&S57496[42], 15}, {NULL}};
const Segment SGI85094[] = {{&S57496[42], 31}, {NULL}};
const Segment SGI85095[] = {{&S57496[30], 4}, {NULL}};
const Segment SGI85096[] = {{&S57496[122], 3}, {NULL}};
const Segment SGI85097[] = {{&S57496[87], 6}, {NULL}};
const Segment SGI85098[] = {{&S65622[44], 17}, {NULL}};
const Segment SGI85099[] = {{&S65622[44], 43}, {NULL}};
const Segment SGI85100[] = {{&S65622[30], 6}, {NULL}};
const Segment SGI85101[] = {{&S65622[135], 3}, {NULL}};
const Segment SGI85102[] = {{&S65622[101], 5}, {NULL}};
const Segment SGI85103[] = {{&S65623[44], 14}, {NULL}};
const Segment SGI85104[] = {{&S65623[44], 40}, {NULL}};
const Segment SGI85105[] = {{&S65623[30], 6}, {NULL}};
const Segment SGI85106[] = {{&S65623[101], 4}, {NULL}};
const Segment SGI85107[] = {{&S65623[101], 3}, {NULL}};
const Segment SGI85108[] = {{&S65624[44], 14}, {NULL}};
const Segment SGI85109[] = {{&S65624[44], 40}, {NULL}};
const Segment SGI85110[] = {{&S65624[30], 6}, {NULL}};
const Segment SGI85111[] = {{&S65624[101], 6}, {NULL}};
const Segment SGI85112[] = {{&S65626[44], 40}, {NULL}};
const Segment SGI85113[] = {{&S65626[30], 6}, {NULL}};
const Segment SGI85114[] = {{&S45356[44], 7}, {NULL}};
const Segment SGI85115[] = {{&S45356[44], 19}, {NULL}};
const Segment SGI85116[] = {{&S45356[30], 6}, {NULL}};
const Segment SGI85117[] = {{&S45356[31], 3}, {NULL}};
const Segment SGI85118[] = {{&S45356[77], 6}, {NULL}};
const Segment SGI85119[] = {{&S45357[38], 7}, {NULL}};
const Segment SGI85120[] = {{&S45357[31], 6}, {NULL}};
const Segment SGI85121[] = {{&S45362[44], 10}, {NULL}};
const Segment SGI85122[] = {{&S45362[44], 22}, {NULL}};
const Segment SGI85123[] = {{&S45362[30], 6}, {NULL}};
const Segment SGI85124[] = {{&S45362[31], 3}, {NULL}};
const Segment SGI85125[] = {{&S45362[80], 6}, {NULL}};
const Segment SGI85126[] = {{&S45363[44], 10}, {NULL}};
const Segment SGI85127[] = {{&S45363[44], 22}, {NULL}};
const Segment SGI85128[] = {{&S45363[30], 6}, {NULL}};
const Segment SGI85129[] = {{&S45363[31], 3}, {NULL}};
const Segment SGI85130[] = {{&S45363[80], 6}, {NULL}};
const Segment SGI85131[] = {{&S58101[44], 10}, {NULL}};
const Segment SGI85132[] = {{&S58101[44], 21}, {NULL}};
const Segment SGI85133[] = {{&S58101[30], 6}, {NULL}};
const Segment SGI85134[] = {{&S58101[160], 3}, {NULL}};
const Segment SGI85135[] = {{&S58101[126], 5}, {NULL}};
const Segment SGI85136[] = {{&S58102[44], 10}, {NULL}};
const Segment SGI85137[] = {{&S58102[44], 21}, {NULL}};
const Segment SGI85138[] = {{&S58102[30], 6}, {NULL}};
const Segment SGI85139[] = {{&S58102[132], 3}, {NULL}};
const Segment SGI85140[] = {{&S58102[98], 5}, {NULL}};
const Segment SGI85141[] = {{&S58102[35], 1}, {NULL}};
const Segment SGI85142[] = {{&S58102[67], 11}, {NULL}};
const Segment SGI85143[] = {{&S58103[44], 10}, {NULL}};
const Segment SGI85144[] = {{&S58103[44], 21}, {NULL}};
const Segment SGI85145[] = {{&S58103[30], 6}, {NULL}};
const Segment SGI85146[] = {{&S58103[132], 3}, {NULL}};
const Segment SGI85147[] = {{&S58103[98], 5}, {NULL}};
const Segment SGI85148[] = {{&S58103[35], 1}, {NULL}};
const Segment SGI85149[] = {{&S58103[67], 11}, {NULL}};
const Segment SGI85150[] = {{&S65695[6], 4}, {NULL}};
const Segment SGI85151[] = {{&S65695[82], 5}, {NULL}};
const Segment SGI85152[] = {{&S58160[44], 14}, {NULL}};
const Segment SGI85153[] = {{&S58160[44], 25}, {NULL}};
const Segment SGI85154[] = {{&S58160[30], 6}, {NULL}};
const Segment SGI85155[] = {{&S58160[136], 3}, {NULL}};
const Segment SGI85156[] = {{&S58160[102], 5}, {NULL}};
const Segment SGI85157[] = {{&S58160[35], 1}, {NULL}};
const Segment SGI85158[] = {{&S58160[71], 11}, {NULL}};
const Segment SGI85159[] = {{&S58161[44], 14}, {NULL}};
const Segment SGI85160[] = {{&S58161[44], 25}, {NULL}};
const Segment SGI85161[] = {{&S58161[30], 6}, {NULL}};
const Segment SGI85162[] = {{&S58161[137], 3}, {NULL}};
const Segment SGI85163[] = {{&S58161[103], 5}, {NULL}};
const Segment SGI85164[] = {{&S58161[81], 3}, {NULL}};
const Segment SGI85165[] = {{&S65757[44], 6}, {NULL}};
const Segment SGI85166[] = {{&S65757[44], 18}, {NULL}};
const Segment SGI85167[] = {{&S65757[30], 6}, {NULL}};
const Segment SGI85168[] = {{&S65757[110], 3}, {NULL}};
const Segment SGI85169[] = {{&S65757[76], 5}, {NULL}};
const Segment SGI85170[] = {{&S45432[34], 7}, {NULL}};
const Segment SGI85171[] = {{&S45432[27], 6}, {NULL}};
const Segment SGI85172[] = {{&S45432[94], 9}, {NULL}};
const Segment SGI85173[] = {{&S45432[67], 6}, {NULL}};
const Segment SGI85174[] = {{&S65791[44], 4}, {NULL}};
const Segment SGI85175[] = {{&S65791[44], 16}, {NULL}};
const Segment SGI85176[] = {{&S65791[30], 6}, {NULL}};
const Segment SGI85177[] = {{&S65791[108], 3}, {NULL}};
const Segment SGI85178[] = {{&S65791[74], 5}, {NULL}};
const Segment SGI85179[] = {{&S65792[38], 4}, {NULL}};
const Segment SGI85180[] = {{&S65792[31], 6}, {NULL}};
const Segment SGI85181[] = {{&S57682[42], 40}, {NULL}};
const Segment SGI85182[] = {{&S57682[30], 4}, {NULL}};
const Segment SGI85183[] = {{&S57682[131], 3}, {NULL}};
const Segment SGI85184[] = {{&S57682[96], 6}, {NULL}};
const Segment SGI85185[] = {{&S57683[42], 40}, {NULL}};
const Segment SGI85186[] = {{&S57683[30], 4}, {NULL}};
const Segment SGI85187[] = {{&S57683[131], 3}, {NULL}};
const Segment SGI85188[] = {{&S57683[96], 6}, {NULL}};
const Segment SGI85189[] = {{&S57684[42], 40}, {NULL}};
const Segment SGI85190[] = {{&S57684[30], 4}, {NULL}};
const Segment SGI85191[] = {{&S57684[131], 3}, {NULL}};
const Segment SGI85192[] = {{&S57684[96], 6}, {NULL}};
const Segment SGI85193[] = {{&S57685[42], 40}, {NULL}};
const Segment SGI85194[] = {{&S57685[30], 4}, {NULL}};
const Segment SGI85195[] = {{&S57685[131], 3}, {NULL}};
const Segment SGI85196[] = {{&S57685[96], 6}, {NULL}};
const Segment SGI85197[] = {{&S57686[42], 40}, {NULL}};
const Segment SGI85198[] = {{&S57686[30], 4}, {NULL}};
const Segment SGI85199[] = {{&S57686[131], 3}, {NULL}};
const Segment SGI85200[] = {{&S57686[96], 6}, {NULL}};
const Segment SGI85201[] = {{&S57687[42], 40}, {NULL}};
const Segment SGI85202[] = {{&S57687[30], 4}, {NULL}};
const Segment SGI85203[] = {{&S57687[131], 3}, {NULL}};
const Segment SGI85204[] = {{&S57687[96], 6}, {NULL}};
const Segment SGI85205[] = {{&S57688[42], 40}, {NULL}};
const Segment SGI85206[] = {{&S57688[30], 4}, {NULL}};
const Segment SGI85207[] = {{&S57688[131], 3}, {NULL}};
const Segment SGI85208[] = {{&S57688[96], 6}, {NULL}};
const Segment SGI85209[] = {{&S57689[42], 40}, {NULL}};
const Segment SGI85210[] = {{&S57689[30], 4}, {NULL}};
const Segment SGI85211[] = {{&S57689[131], 3}, {NULL}};
const Segment SGI85212[] = {{&S57689[96], 6}, {NULL}};
const Segment SGI85213[] = {{&S57690[42], 40}, {NULL}};
const Segment SGI85214[] = {{&S57690[30], 4}, {NULL}};
const Segment SGI85215[] = {{&S57690[131], 3}, {NULL}};
const Segment SGI85216[] = {{&S57690[96], 6}, {NULL}};
const Segment SGI85217[] = {{&S57691[42], 40}, {NULL}};
const Segment SGI85218[] = {{&S57691[30], 4}, {NULL}};
const Segment SGI85219[] = {{&S57691[131], 3}, {NULL}};
const Segment SGI85220[] = {{&S57691[96], 6}, {NULL}};
const Segment SGI85221[] = {{&S57692[42], 40}, {NULL}};
const Segment SGI85222[] = {{&S57692[30], 4}, {NULL}};
const Segment SGI85223[] = {{&S57692[131], 3}, {NULL}};
const Segment SGI85224[] = {{&S57692[96], 6}, {NULL}};
const Segment SGI85225[] = {{&S57693[42], 40}, {NULL}};
const Segment SGI85226[] = {{&S57693[30], 4}, {NULL}};
const Segment SGI85227[] = {{&S57693[131], 3}, {NULL}};
const Segment SGI85228[] = {{&S57693[96], 6}, {NULL}};
const Segment SGI85229[] = {{&S57694[42], 40}, {NULL}};
const Segment SGI85230[] = {{&S57694[30], 4}, {NULL}};
const Segment SGI85231[] = {{&S57694[131], 3}, {NULL}};
const Segment SGI85232[] = {{&S57694[96], 6}, {NULL}};
const Segment SGI85233[] = {{&S57695[44], 11}, {NULL}};
const Segment SGI85234[] = {{&S57695[44], 22}, {NULL}};
const Segment SGI85235[] = {{&S57695[30], 6}, {NULL}};
const Segment SGI85236[] = {{&S57695[115], 3}, {NULL}};
const Segment SGI85237[] = {{&S57695[80], 6}, {NULL}};
const Segment SGI85238[] = {{&S57696[42], 40}, {NULL}};
const Segment SGI85239[] = {{&S57696[30], 4}, {NULL}};
const Segment SGI85240[] = {{&S57696[131], 3}, {NULL}};
const Segment SGI85241[] = {{&S57696[96], 6}, {NULL}};
const Segment SGI85242[] = {{&S57698[42], 11}, {NULL}};
const Segment SGI85243[] = {{&S57698[42], 22}, {NULL}};
const Segment SGI85244[] = {{&S57698[30], 4}, {NULL}};
const Segment SGI85245[] = {{&S57698[113], 3}, {NULL}};
const Segment SGI85246[] = {{&S57698[78], 6}, {NULL}};
const Segment SGI85247[] = {{&S57699[42], 11}, {NULL}};
const Segment SGI85248[] = {{&S57699[42], 22}, {NULL}};
const Segment SGI85249[] = {{&S57699[30], 4}, {NULL}};
const Segment SGI85250[] = {{&S57699[113], 3}, {NULL}};
const Segment SGI85251[] = {{&S57699[78], 6}, {NULL}};
const Segment SGI85252[] = {{&S57700[42], 11}, {NULL}};
const Segment SGI85253[] = {{&S57700[42], 22}, {NULL}};
const Segment SGI85254[] = {{&S57700[30], 4}, {NULL}};
const Segment SGI85255[] = {{&S57700[113], 3}, {NULL}};
const Segment SGI85256[] = {{&S57700[78], 6}, {NULL}};
const Segment SGI85257[] = {{&S57701[36], 11}, {NULL}};
const Segment SGI85258[] = {{&S57701[31], 4}, {NULL}};
const Segment SGI85259[] = {{&S57702[36], 11}, {NULL}};
const Segment SGI85260[] = {{&S57702[31], 4}, {NULL}};
const Segment SGI85261[] = {{&S45507[34], 10}, {NULL}};
const Segment SGI85262[] = {{&S45507[27], 6}, {NULL}};
const Segment SGI85263[] = {{&S45507[97], 9}, {NULL}};
const Segment SGI85264[] = {{&S45507[70], 6}, {NULL}};
const Segment SGI85265[] = {{&S45516[34], 11}, {NULL}};
const Segment SGI85266[] = {{&S45516[27], 6}, {NULL}};
const Segment SGI85267[] = {{&S45516[98], 9}, {NULL}};
const Segment SGI85268[] = {{&S45516[71], 6}, {NULL}};
const Segment SGI85269[] = {{&S58316[44], 14}, {NULL}};
const Segment SGI85270[] = {{&S58316[44], 25}, {NULL}};
const Segment SGI85271[] = {{&S58316[30], 6}, {NULL}};
const Segment SGI85272[] = {{&S58316[117], 3}, {NULL}};
const Segment SGI85273[] = {{&S58316[83], 5}, {NULL}};
const Segment SGI85274[] = {{&S65876[44], 21}, {NULL}};
const Segment SGI85275[] = {{&S65876[44], 38}, {NULL}};
const Segment SGI85276[] = {{&S65876[30], 6}, {NULL}};
const Segment SGI85277[] = {{&S65876[130], 3}, {NULL}};
const Segment SGI85278[] = {{&S65876[96], 5}, {NULL}};
const Segment SGI85279[] = {{&S58319[38], 14}, {NULL}};
const Segment SGI85280[] = {{&S58319[31], 6}, {NULL}};
const Segment SGI85281[] = {{&S58322[44], 14}, {NULL}};
const Segment SGI85282[] = {{&S58322[44], 25}, {NULL}};
const Segment SGI85283[] = {{&S58322[30], 6}, {NULL}};
const Segment SGI85284[] = {{&S58322[86], 4}, {NULL}};
const Segment SGI85285[] = {{&S58322[86], 3}, {NULL}};
const Segment SGI85286[] = {{&S58323[44], 14}, {NULL}};
const Segment SGI85287[] = {{&S58323[44], 25}, {NULL}};
const Segment SGI85288[] = {{&S58323[30], 6}, {NULL}};
const Segment SGI85289[] = {{&S58323[117], 3}, {NULL}};
const Segment SGI85290[] = {{&S58323[83], 5}, {NULL}};
const Segment SGI85291[] = {{&S45540[34], 10}, {NULL}};
const Segment SGI85292[] = {{&S45540[27], 6}, {NULL}};
const Segment SGI85293[] = {{&S45540[97], 9}, {NULL}};
const Segment SGI85294[] = {{&S45540[70], 6}, {NULL}};
const Segment SGI85295[] = {{&S65904[44], 18}, {NULL}};
const Segment SGI85296[] = {{&S65904[44], 35}, {NULL}};
const Segment SGI85297[] = {{&S65904[30], 6}, {NULL}};
const Segment SGI85298[] = {{&S65904[127], 3}, {NULL}};
const Segment SGI85299[] = {{&S65904[93], 5}, {NULL}};
const Segment SGI85300[] = {{&S45571[34], 10}, {NULL}};
const Segment SGI85301[] = {{&S45571[27], 6}, {NULL}};
const Segment SGI85302[] = {{&S45571[97], 9}, {NULL}};
const Segment SGI85303[] = {{&S45571[70], 6}, {NULL}};
const Segment SGI85304[] = {{&S65953[44], 11}, {NULL}};
const Segment SGI85305[] = {{&S65953[44], 28}, {NULL}};
const Segment SGI85306[] = {{&S65953[30], 6}, {NULL}};
const Segment SGI85307[] = {{&S65953[86], 6}, {NULL}};
const Segment SGI85308[] = {{&S65953[113], 13}, {NULL}};
const Segment SGI85309[] = {{&S65954[44], 10}, {NULL}};
const Segment SGI85310[] = {{&S65954[44], 27}, {NULL}};
const Segment SGI85311[] = {{&S65954[30], 6}, {NULL}};
const Segment SGI85312[] = {{&S65954[85], 6}, {NULL}};
const Segment SGI85313[] = {{&S65954[112], 13}, {NULL}};
const Segment SGI85314[] = {{&S45647[34], 11}, {NULL}};
const Segment SGI85315[] = {{&S45647[27], 6}, {NULL}};
const Segment SGI85316[] = {{&S45647[98], 9}, {NULL}};
const Segment SGI85317[] = {{&S45647[71], 6}, {NULL}};
const Segment SGI85318[] = {{&S45649[34], 10}, {NULL}};
const Segment SGI85319[] = {{&S45649[27], 6}, {NULL}};
const Segment SGI85320[] = {{&S45649[97], 9}, {NULL}};
const Segment SGI85321[] = {{&S45649[70], 6}, {NULL}};
const Segment SGI85322[] = {{&S45667[44], 10}, {NULL}};
const Segment SGI85323[] = {{&S45667[44], 22}, {NULL}};
const Segment SGI85324[] = {{&S45667[30], 6}, {NULL}};
const Segment SGI85325[] = {{&S45667[31], 3}, {NULL}};
const Segment SGI85326[] = {{&S45667[80], 6}, {NULL}};
const Segment SGI85327[] = {{&S45668[44], 10}, {NULL}};
const Segment SGI85328[] = {{&S45668[44], 22}, {NULL}};
const Segment SGI85329[] = {{&S45668[30], 6}, {NULL}};
const Segment SGI85330[] = {{&S45668[31], 3}, {NULL}};
const Segment SGI85331[] = {{&S45668[80], 6}, {NULL}};
const Segment SGI85332[] = {{&S45732[44], 15}, {NULL}};
const Segment SGI85333[] = {{&S45732[44], 27}, {NULL}};
const Segment SGI85334[] = {{&S45732[30], 6}, {NULL}};
const Segment SGI85335[] = {{&S45732[31], 3}, {NULL}};
const Segment SGI85336[] = {{&S45732[85], 6}, {NULL}};
const Segment SGI85337[] = {{&S45733[44], 6}, {NULL}};
const Segment SGI85338[] = {{&S45733[44], 18}, {NULL}};
const Segment SGI85339[] = {{&S45733[30], 6}, {NULL}};
const Segment SGI85340[] = {{&S45733[31], 3}, {NULL}};
const Segment SGI85341[] = {{&S45733[76], 6}, {NULL}};
const Segment SGI85342[] = {{&S45735[38], 15}, {NULL}};
const Segment SGI85343[] = {{&S45735[31], 6}, {NULL}};
const Segment SGI85344[] = {{&S45738[34], 10}, {NULL}};
const Segment SGI85345[] = {{&S45738[27], 6}, {NULL}};
const Segment SGI85346[] = {{&S45738[97], 9}, {NULL}};
const Segment SGI85347[] = {{&S45738[70], 6}, {NULL}};
const Segment SGI85348[] = {{&S66096[44], 8}, {NULL}};
const Segment SGI85349[] = {{&S66096[44], 20}, {NULL}};
const Segment SGI85350[] = {{&S66096[30], 6}, {NULL}};
const Segment SGI85351[] = {{&S66096[112], 3}, {NULL}};
const Segment SGI85352[] = {{&S66096[78], 5}, {NULL}};
const Segment SGI85353[] = {{&S45781[44], 10}, {NULL}};
const Segment SGI85354[] = {{&S45781[44], 22}, {NULL}};
const Segment SGI85355[] = {{&S45781[30], 6}, {NULL}};
const Segment SGI85356[] = {{&S45781[31], 3}, {NULL}};
const Segment SGI85357[] = {{&S45781[80], 6}, {NULL}};
const Segment SGI85358[] = {{&S45782[44], 13}, {NULL}};
const Segment SGI85359[] = {{&S45782[44], 25}, {NULL}};
const Segment SGI85360[] = {{&S45782[30], 6}, {NULL}};
const Segment SGI85361[] = {{&S45782[31], 3}, {NULL}};
const Segment SGI85362[] = {{&S45782[83], 6}, {NULL}};
const Segment SGI85363[] = {{&S45783[38], 13}, {NULL}};
const Segment SGI85364[] = {{&S45783[31], 6}, {NULL}};
const Segment SGI85365[] = {{&S45789[44], 14}, {NULL}};
const Segment SGI85366[] = {{&S45789[44], 26}, {NULL}};
const Segment SGI85367[] = {{&S45789[30], 6}, {NULL}};
const Segment SGI85368[] = {{&S45789[31], 3}, {NULL}};
const Segment SGI85369[] = {{&S45789[84], 6}, {NULL}};
const Segment SGI85370[] = {{&S45790[44], 9}, {NULL}};
const Segment SGI85371[] = {{&S45790[44], 21}, {NULL}};
const Segment SGI85372[] = {{&S45790[30], 6}, {NULL}};
const Segment SGI85373[] = {{&S45790[31], 3}, {NULL}};
const Segment SGI85374[] = {{&S45790[79], 6}, {NULL}};
const Segment SGI85375[] = {{&S45791[38], 9}, {NULL}};
const Segment SGI85376[] = {{&S45791[31], 6}, {NULL}};
const Segment SGI85377[] = {{&S66140[44], 10}, {NULL}};
const Segment SGI85378[] = {{&S66140[44], 27}, {NULL}};
const Segment SGI85379[] = {{&S66140[30], 6}, {NULL}};
const Segment SGI85380[] = {{&S66140[149], 4}, {NULL}};
const Segment SGI85381[] = {{&S66140[175], 3}, {NULL}};
const Segment SGI85382[] = {{&S66140[119], 3}, {&S66140[224], 5}, {NULL}};
const Segment SGI85383[] = {{&S66140[85], 5}, {NULL}};
const Segment SGI85384[] = {{&S55563[42], 12}, {NULL}};
const Segment SGI85385[] = {{&S55563[42], 24}, {NULL}};
const Segment SGI85386[] = {{&S55563[30], 4}, {NULL}};
const Segment SGI85387[] = {{&S55563[31], 3}, {NULL}};
const Segment SGI85388[] = {{&S55563[109], 6}, {NULL}};
const Segment SGI85389[] = {{&S58657[44], 15}, {NULL}};
const Segment SGI85390[] = {{&S58657[44], 25}, {NULL}};
const Segment SGI85391[] = {{&S58657[30], 6}, {NULL}};
const Segment SGI85392[] = {{&S58657[117], 3}, {NULL}};
const Segment SGI85393[] = {{&S58657[83], 5}, {NULL}};
const Segment SGI85394[] = {{&S58660[44], 15}, {NULL}};
const Segment SGI85395[] = {{&S58660[44], 25}, {NULL}};
const Segment SGI85396[] = {{&S58660[30], 6}, {NULL}};
const Segment SGI85397[] = {{&S58660[86], 4}, {NULL}};
const Segment SGI85398[] = {{&S58660[86], 3}, {NULL}};
const Segment SGI85399[] = {{&S58661[38], 15}, {NULL}};
const Segment SGI85400[] = {{&S58661[31], 6}, {NULL}};
const Segment SGI85401[] = {{&S58662[44], 11}, {NULL}};
const Segment SGI85402[] = {{&S58662[44], 21}, {NULL}};
const Segment SGI85403[] = {{&S58662[30], 6}, {NULL}};
const Segment SGI85404[] = {{&S45851[44], 10}, {NULL}};
const Segment SGI85405[] = {{&S58662[113], 3}, {NULL}};
const Segment SGI85406[] = {{&S45851[44], 22}, {NULL}};
const Segment SGI85407[] = {{&S45851[30], 6}, {NULL}};
const Segment SGI85408[] = {{&S58662[79], 5}, {NULL}};
const Segment SGI85409[] = {{&S45851[31], 3}, {NULL}};
const Segment SGI85410[] = {{&S45851[80], 6}, {NULL}};
const Segment SGI85411[] = {{&S58663[44], 15}, {NULL}};
const Segment SGI85412[] = {{&S58663[44], 25}, {NULL}};
const Segment SGI85413[] = {{&S58663[30], 6}, {NULL}};
const Segment SGI85414[] = {{&S45852[44], 10}, {NULL}};
const Segment SGI85415[] = {{&S45852[44], 22}, {NULL}};
const Segment SGI85416[] = {{&S45852[30], 6}, {NULL}};
const Segment SGI85417[] = {{&S58663[86], 6}, {NULL}};
const Segment SGI85418[] = {{&S45852[31], 3}, {NULL}};
const Segment SGI85419[] = {{&S45852[80], 6}, {NULL}};
const Segment SGI85420[] = {{&S45853[44], 10}, {NULL}};
const Segment SGI85421[] = {{&S45853[44], 22}, {NULL}};
const Segment SGI85422[] = {{&S45853[30], 6}, {NULL}};
const Segment SGI85423[] = {{&S45853[31], 3}, {NULL}};
const Segment SGI85424[] = {{&S66186[45], 10}, {NULL}};
const Segment SGI85425[] = {{&S45853[80], 6}, {NULL}};
const Segment SGI85426[] = {{&S66186[30], 6}, {NULL}};
const Segment SGI85427[] = {{&S66186[120], 3}, {NULL}};
const Segment SGI85428[] = {{&S66186[86], 5}, {NULL}};
const Segment SGI85429[] = {{&S66186[144], 13}, {NULL}};
const Segment SGI85430[] = {{&S45854[38], 10}, {NULL}};
const Segment SGI85431[] = {{&S45854[31], 6}, {NULL}};
const Segment SGI85432[] = {{&S45855[38], 10}, {NULL}};
const Segment SGI85433[] = {{&S45855[31], 6}, {NULL}};
const Segment SGI85434[] = {{&S45856[38], 10}, {NULL}};
const Segment SGI85435[] = {{&S45856[31], 6}, {NULL}};
const Segment SGI85436[] = {{&S66190[45], 11}, {NULL}};
const Segment SGI85437[] = {{&S66190[30], 6}, {NULL}};
const Segment SGI85438[] = {{&S66190[121], 3}, {NULL}};
const Segment SGI85439[] = {{&S45857[42], 10}, {NULL}};
const Segment SGI85440[] = {{&S66190[87], 5}, {NULL}};
const Segment SGI85441[] = {{&S45857[30], 6}, {NULL}};
const Segment SGI85442[] = {{&S66190[145], 13}, {NULL}};
const Segment SGI85443[] = {{&S45857[31], 3}, {NULL}};
const Segment SGI85444[] = {{&S45857[78], 6}, {NULL}};
const Segment SGI85445[] = {{&S45858[42], 10}, {NULL}};
const Segment SGI85446[] = {{&S45858[30], 6}, {NULL}};
const Segment SGI85447[] = {{&S45858[31], 3}, {NULL}};
const Segment SGI85448[] = {{&S45858[78], 6}, {NULL}};
const Segment SGI85449[] = {{&S66193[44], 10}, {NULL}};
const Segment SGI85450[] = {{&S66193[44], 27}, {NULL}};
const Segment SGI85451[] = {{&S66193[30], 6}, {NULL}};
const Segment SGI85452[] = {{&S45859[42], 10}, {NULL}};
const Segment SGI85453[] = {{&S45859[30], 6}, {NULL}};
const Segment SGI85454[] = {{&S66193[120], 3}, {NULL}};
const Segment SGI85455[] = {{&S66193[85], 6}, {NULL}};
const Segment SGI85456[] = {{&S45859[31], 3}, {NULL}};
const Segment SGI85457[] = {{&S66193[140], 8}, {NULL}};
const Segment SGI85458[] = {{&S66193[149], 13}, {NULL}};
const Segment SGI85459[] = {{&S45859[78], 6}, {NULL}};
const Segment SGI85460[] = {{&S45860[44], 10}, {NULL}};
const Segment SGI85461[] = {{&S45860[44], 19}, {NULL}};
const Segment SGI85462[] = {{&S45860[30], 6}, {NULL}};
const Segment SGI85463[] = {{&S45860[31], 3}, {NULL}};
const Segment SGI85464[] = {{&S45860[77], 6}, {NULL}};
const Segment SGI85465[] = {{&S45861[44], 15}, {NULL}};
const Segment SGI85466[] = {{&S45861[44], 24}, {NULL}};
const Segment SGI85467[] = {{&S45861[30], 6}, {NULL}};
const Segment SGI85468[] = {{&S45861[31], 3}, {NULL}};
const Segment SGI85469[] = {{&S45861[82], 6}, {NULL}};
const Segment SGI85470[] = {{&S45862[44], 10}, {NULL}};
const Segment SGI85471[] = {{&S45862[44], 22}, {NULL}};
const Segment SGI85472[] = {{&S45862[30], 6}, {NULL}};
const Segment SGI85473[] = {{&S45862[31], 3}, {NULL}};
const Segment SGI85474[] = {{&S45862[80], 6}, {NULL}};
const Segment SGI85475[] = {{&S45863[44], 9}, {NULL}};
const Segment SGI85476[] = {{&S45863[44], 25}, {NULL}};
const Segment SGI85477[] = {{&S45863[30], 6}, {NULL}};
const Segment SGI85478[] = {{&S45863[31], 3}, {NULL}};
const Segment SGI85479[] = {{&S45863[83], 6}, {NULL}};
const Segment SGI85480[] = {{&S45864[44], 10}, {NULL}};
const Segment SGI85481[] = {{&S45864[44], 19}, {NULL}};
const Segment SGI85482[] = {{&S45864[30], 6}, {NULL}};
const Segment SGI85483[] = {{&S45864[31], 3}, {NULL}};
const Segment SGI85484[] = {{&S45864[77], 6}, {NULL}};
const Segment SGI85485[] = {{&S45865[44], 10}, {NULL}};
const Segment SGI85486[] = {{&S45865[44], 22}, {NULL}};
const Segment SGI85487[] = {{&S45865[30], 6}, {NULL}};
const Segment SGI85488[] = {{&S45865[31], 3}, {NULL}};
const Segment SGI85489[] = {{&S45865[80], 6}, {NULL}};
const Segment SGI85490[] = {{&S45866[44], 15}, {NULL}};
const Segment SGI85491[] = {{&S45866[44], 24}, {NULL}};
const Segment SGI85492[] = {{&S45866[30], 6}, {NULL}};
const Segment SGI85493[] = {{&S45866[31], 3}, {NULL}};
const Segment SGI85494[] = {{&S45866[82], 6}, {NULL}};
const Segment SGI85495[] = {{&S45919[44], 9}, {NULL}};
const Segment SGI85496[] = {{&S45919[44], 21}, {NULL}};
const Segment SGI85497[] = {{&S45919[30], 6}, {NULL}};
const Segment SGI85498[] = {{&S45919[31], 3}, {NULL}};
const Segment SGI85499[] = {{&S45919[79], 6}, {NULL}};
const Segment SGI85500[] = {{&S45920[38], 9}, {NULL}};
const Segment SGI85501[] = {{&S45920[31], 6}, {NULL}};
const Segment SGI85502[] = {{&S66284[44], 10}, {NULL}};
const Segment SGI85503[] = {{&S66284[44], 22}, {NULL}};
const Segment SGI85504[] = {{&S66284[30], 6}, {NULL}};
const Segment SGI85505[] = {{&S66284[114], 3}, {NULL}};
const Segment SGI85506[] = {{&S66284[80], 5}, {NULL}};
const Segment SGI85507[] = {{&S66301[45], 11}, {NULL}};
const Segment SGI85508[] = {{&S66301[30], 6}, {NULL}};
const Segment SGI85509[] = {{&S66301[121], 3}, {NULL}};
const Segment SGI85510[] = {{&S66301[87], 5}, {NULL}};
const Segment SGI85511[] = {{&S66301[145], 13}, {NULL}};
const Segment SGI85512[] = {{&S66303[45], 10}, {NULL}};
const Segment SGI85513[] = {{&S66303[30], 6}, {NULL}};
const Segment SGI85514[] = {{&S66303[120], 3}, {NULL}};
const Segment SGI85515[] = {{&S66303[86], 5}, {NULL}};
const Segment SGI85516[] = {{&S66303[144], 13}, {NULL}};
const Segment SGI85517[] = {{&S66305[44], 10}, {NULL}};
const Segment SGI85518[] = {{&S66305[44], 27}, {NULL}};
const Segment SGI85519[] = {{&S66305[30], 6}, {NULL}};
const Segment SGI85520[] = {{&S66305[120], 3}, {NULL}};
const Segment SGI85521[] = {{&S66305[85], 6}, {NULL}};
const Segment SGI85522[] = {{&S66305[140], 8}, {NULL}};
const Segment SGI85523[] = {{&S66305[149], 13}, {NULL}};
const Segment SGI85524[] = {{&S46038[44], 9}, {NULL}};
const Segment SGI85525[] = {{&S46038[44], 21}, {NULL}};
const Segment SGI85526[] = {{&S46038[30], 6}, {NULL}};
const Segment SGI85527[] = {{&S46038[31], 3}, {NULL}};
const Segment SGI85528[] = {{&S46038[79], 6}, {NULL}};
const Segment SGI85529[] = {{&S46039[38], 9}, {NULL}};
const Segment SGI85530[] = {{&S46039[31], 6}, {NULL}};
const Segment SGI85531[] = {{&S58157[44], 6}, {NULL}};
const Segment SGI85532[] = {{&S58157[44], 18}, {NULL}};
const Segment SGI85533[] = {{&S58157[30], 6}, {NULL}};
const Segment SGI85534[] = {{&S58157[111], 3}, {NULL}};
const Segment SGI85535[] = {{&S58157[76], 6}, {NULL}};
const Segment SGI85536[] = {{&S58158[38], 6}, {NULL}};
const Segment SGI85537[] = {{&S58158[31], 6}, {NULL}};
const Segment SGI85538[] = {{&S46090[34], 8}, {NULL}};
const Segment SGI85539[] = {{&S46090[27], 6}, {NULL}};
const Segment SGI85540[] = {{&S46090[96], 9}, {NULL}};
const Segment SGI85541[] = {{&S46090[69], 6}, {NULL}};
const Segment SGI85542[] = {{&S46091[34], 8}, {NULL}};
const Segment SGI85543[] = {{&S46091[27], 6}, {NULL}};
const Segment SGI85544[] = {{&S46091[96], 9}, {NULL}};
const Segment SGI85545[] = {{&S46091[69], 6}, {NULL}};
const Segment SGI85546[] = {{&S46112[34], 10}, {NULL}};
const Segment SGI85547[] = {{&S46112[27], 6}, {NULL}};
const Segment SGI85548[] = {{&S46112[97], 9}, {NULL}};
const Segment SGI85549[] = {{&S46112[70], 6}, {NULL}};
const Segment SGI85550[] = {{&S46122[34], 10}, {NULL}};
const Segment SGI85551[] = {{&S46122[27], 6}, {NULL}};
const Segment SGI85552[] = {{&S46122[97], 9}, {NULL}};
const Segment SGI85553[] = {{&S46122[70], 6}, {NULL}};
const Segment SGI85554[] = {{&S46127[34], 11}, {NULL}};
const Segment SGI85555[] = {{&S46127[27], 6}, {NULL}};
const Segment SGI85556[] = {{&S46127[98], 9}, {NULL}};
const Segment SGI85557[] = {{&S46127[71], 6}, {NULL}};
const Segment SGI85558[] = {{&S46139[44], 10}, {NULL}};
const Segment SGI85559[] = {{&S46139[44], 22}, {NULL}};
const Segment SGI85560[] = {{&S46139[30], 6}, {NULL}};
const Segment SGI85561[] = {{&S46139[31], 3}, {NULL}};
const Segment SGI85562[] = {{&S46139[80], 6}, {NULL}};
const Segment SGI85563[] = {{&S46140[44], 7}, {NULL}};
const Segment SGI85564[] = {{&S46140[44], 19}, {NULL}};
const Segment SGI85565[] = {{&S46140[30], 6}, {NULL}};
const Segment SGI85566[] = {{&S46140[31], 3}, {NULL}};
const Segment SGI85567[] = {{&S46140[77], 6}, {NULL}};
const Segment SGI85568[] = {{&S46142[44], 7}, {NULL}};
const Segment SGI85569[] = {{&S46142[44], 19}, {NULL}};
const Segment SGI85570[] = {{&S46142[30], 6}, {NULL}};
const Segment SGI85571[] = {{&S46142[31], 3}, {NULL}};
const Segment SGI85572[] = {{&S46142[77], 5}, {NULL}};
const Segment SGI85573[] = {{&S46150[44], 10}, {NULL}};
const Segment SGI85574[] = {{&S46150[44], 22}, {NULL}};
const Segment SGI85575[] = {{&S46150[30], 6}, {NULL}};
const Segment SGI85576[] = {{&S46150[83], 6}, {NULL}};
const Segment SGI85577[] = {{&S46152[44], 10}, {NULL}};
const Segment SGI85578[] = {{&S46152[44], 22}, {NULL}};
const Segment SGI85579[] = {{&S46152[30], 6}, {NULL}};
const Segment SGI85580[] = {{&S46152[31], 3}, {NULL}};
const Segment SGI85581[] = {{&S46152[80], 5}, {NULL}};
const Segment SGI85582[] = {{&S46154[44], 10}, {NULL}};
const Segment SGI85583[] = {{&S46154[44], 21}, {NULL}};
const Segment SGI85584[] = {{&S46154[30], 6}, {NULL}};
const Segment SGI85585[] = {{&S46154[31], 3}, {NULL}};
const Segment SGI85586[] = {{&S46154[79], 5}, {NULL}};
const Segment SGI85587[] = {{&S46158[38], 7}, {NULL}};
const Segment SGI85588[] = {{&S46158[31], 6}, {NULL}};
const Segment SGI85589[] = {{&S66572[44], 14}, {NULL}};
const Segment SGI85590[] = {{&S66572[44], 44}, {NULL}};
const Segment SGI85591[] = {{&S66572[30], 6}, {NULL}};
const Segment SGI85592[] = {{&S66572[143], 3}, {NULL}};
const Segment SGI85593[] = {{&S66572[109], 5}, {NULL}};
const Segment SGI85594[] = {{&S66572[88], 8}, {NULL}};
const Segment SGI85595[] = {{&S66573[44], 14}, {NULL}};
const Segment SGI85596[] = {{&S66573[44], 44}, {NULL}};
const Segment SGI85597[] = {{&S66573[30], 6}, {NULL}};
const Segment SGI85598[] = {{&S66573[143], 3}, {NULL}};
const Segment SGI85599[] = {{&S66573[109], 5}, {NULL}};
const Segment SGI85600[] = {{&S66573[88], 8}, {NULL}};
const Segment SGI85601[] = {{&S66574[44], 14}, {NULL}};
const Segment SGI85602[] = {{&S66574[44], 31}, {NULL}};
const Segment SGI85603[] = {{&S66574[30], 6}, {NULL}};
const Segment SGI85604[] = {{&S66574[130], 3}, {NULL}};
const Segment SGI85605[] = {{&S66574[96], 5}, {NULL}};
const Segment SGI85606[] = {{&S66574[75], 8}, {NULL}};
const Segment SGI85607[] = {{&S66575[44], 14}, {NULL}};
const Segment SGI85608[] = {{&S66575[44], 31}, {NULL}};
const Segment SGI85609[] = {{&S66575[30], 6}, {NULL}};
const Segment SGI85610[] = {{&S66575[123], 3}, {NULL}};
const Segment SGI85611[] = {{&S66575[89], 5}, {NULL}};
const Segment SGI85612[] = {{&S66575[147], 7}, {NULL}};
const Segment SGI85613[] = {{&S66662[44], 14}, {NULL}};
const Segment SGI85614[] = {{&S66662[44], 26}, {NULL}};
const Segment SGI85615[] = {{&S66662[30], 6}, {NULL}};
const Segment SGI85616[] = {{&S66662[87], 6}, {NULL}};
const Segment SGI85617[] = {{&S66663[44], 14}, {NULL}};
const Segment SGI85618[] = {{&S66663[44], 26}, {NULL}};
const Segment SGI85619[] = {{&S66663[30], 6}, {NULL}};
const Segment SGI85620[] = {{&S66663[118], 3}, {NULL}};
const Segment SGI85621[] = {{&S66663[84], 5}, {NULL}};
const Segment SGI85622[] = {{&S66665[44], 6}, {NULL}};
const Segment SGI85623[] = {{&S66665[44], 18}, {NULL}};
const Segment SGI85624[] = {{&S66665[30], 6}, {NULL}};
const Segment SGI85625[] = {{&S66665[110], 3}, {NULL}};
const Segment SGI85626[] = {{&S66665[76], 5}, {NULL}};
const Segment SGI85627[] = {{&S66666[38], 14}, {NULL}};
const Segment SGI85628[] = {{&S66666[31], 6}, {NULL}};
const Segment SGI85629[] = {{&S66667[44], 14}, {NULL}};
const Segment SGI85630[] = {{&S66667[44], 26}, {NULL}};
const Segment SGI85631[] = {{&S66667[30], 6}, {NULL}};
const Segment SGI85632[] = {{&S66667[118], 3}, {NULL}};
const Segment SGI85633[] = {{&S66667[84], 5}, {NULL}};
const Segment SGI85634[] = {{&S46314[44], 18}, {NULL}};
const Segment SGI85635[] = {{&S46314[44], 30}, {NULL}};
const Segment SGI85636[] = {{&S46314[30], 6}, {NULL}};
const Segment SGI85637[] = {{&S46314[31], 3}, {NULL}};
const Segment SGI85638[] = {{&S46314[88], 6}, {NULL}};
const Segment SGI85639[] = {{&S46315[44], 11}, {NULL}};
const Segment SGI85640[] = {{&S46315[44], 20}, {NULL}};
const Segment SGI85641[] = {{&S46315[30], 6}, {NULL}};
const Segment SGI85642[] = {{&S46315[31], 3}, {NULL}};
const Segment SGI85643[] = {{&S46315[78], 6}, {NULL}};
const Segment SGI85644[] = {{&S46316[44], 15}, {NULL}};
const Segment SGI85645[] = {{&S46316[44], 27}, {NULL}};
const Segment SGI85646[] = {{&S46316[30], 6}, {NULL}};
const Segment SGI85647[] = {{&S46316[31], 3}, {NULL}};
const Segment SGI85648[] = {{&S46316[85], 6}, {NULL}};
const Segment SGI85649[] = {{&S46317[44], 20}, {NULL}};
const Segment SGI85650[] = {{&S46317[44], 32}, {NULL}};
const Segment SGI85651[] = {{&S46317[30], 6}, {NULL}};
const Segment SGI85652[] = {{&S46317[31], 3}, {NULL}};
const Segment SGI85653[] = {{&S46317[90], 6}, {NULL}};
const Segment SGI85654[] = {{&S46318[44], 11}, {NULL}};
const Segment SGI85655[] = {{&S46318[44], 23}, {NULL}};
const Segment SGI85656[] = {{&S46318[30], 6}, {NULL}};
const Segment SGI85657[] = {{&S46318[31], 3}, {NULL}};
const Segment SGI85658[] = {{&S46318[81], 6}, {NULL}};
const Segment SGI85659[] = {{&S46320[38], 15}, {NULL}};
const Segment SGI85660[] = {{&S46320[31], 6}, {NULL}};
const Segment SGI85661[] = {{&S46324[44], 11}, {NULL}};
const Segment SGI85662[] = {{&S46324[44], 28}, {NULL}};
const Segment SGI85663[] = {{&S46324[30], 6}, {NULL}};
const Segment SGI85664[] = {{&S46324[31], 3}, {NULL}};
const Segment SGI85665[] = {{&S46324[86], 6}, {NULL}};
const Segment SGI85666[] = {{&S46328[44], 15}, {NULL}};
const Segment SGI85667[] = {{&S46328[44], 27}, {NULL}};
const Segment SGI85668[] = {{&S46328[30], 6}, {NULL}};
const Segment SGI85669[] = {{&S46328[31], 3}, {NULL}};
const Segment SGI85670[] = {{&S46328[85], 6}, {NULL}};
const Segment SGI85671[] = {{&S46330[51], 11}, {NULL}};
const Segment SGI85672[] = {{&S46330[51], 28}, {NULL}};
const Segment SGI85673[] = {{&S46330[30], 12}, {NULL}};
const Segment SGI85674[] = {{&S46330[31], 3}, {NULL}};
const Segment SGI85675[] = {{&S46330[93], 6}, {NULL}};
const Segment SGI85676[] = {{&S46334[44], 20}, {NULL}};
const Segment SGI85677[] = {{&S46334[44], 32}, {NULL}};
const Segment SGI85678[] = {{&S46334[30], 6}, {NULL}};
const Segment SGI85679[] = {{&S46334[31], 3}, {NULL}};
const Segment SGI85680[] = {{&S46334[90], 6}, {NULL}};
const Segment SGI85681[] = {{&S46339[34], 10}, {NULL}};
const Segment SGI85682[] = {{&S46339[27], 6}, {NULL}};
const Segment SGI85683[] = {{&S46339[97], 9}, {NULL}};
const Segment SGI85684[] = {{&S46339[70], 6}, {NULL}};
const Segment SGI85685[] = {{&S46378[34], 10}, {NULL}};
const Segment SGI85686[] = {{&S46378[27], 6}, {NULL}};
const Segment SGI85687[] = {{&S46378[97], 9}, {NULL}};
const Segment SGI85688[] = {{&S46378[70], 6}, {NULL}};
const Segment SGI85689[] = {{&S66772[44], 6}, {NULL}};
const Segment SGI85690[] = {{&S66772[44], 18}, {NULL}};
const Segment SGI85691[] = {{&S66772[30], 6}, {NULL}};
const Segment SGI85692[] = {{&S66772[110], 3}, {NULL}};
const Segment SGI85693[] = {{&S66772[76], 5}, {NULL}};
const Segment SGI85694[] = {{&S46393[44], 17}, {NULL}};
const Segment SGI85695[] = {{&S46393[44], 29}, {NULL}};
const Segment SGI85696[] = {{&S46393[30], 6}, {NULL}};
const Segment SGI85697[] = {{&S46393[31], 3}, {NULL}};
const Segment SGI85698[] = {{&S46393[87], 5}, {NULL}};
const Segment SGI85699[] = {{&S46402[44], 11}, {NULL}};
const Segment SGI85700[] = {{&S46402[44], 23}, {NULL}};
const Segment SGI85701[] = {{&S46402[30], 6}, {NULL}};
const Segment SGI85702[] = {{&S46402[31], 3}, {NULL}};
const Segment SGI85703[] = {{&S46402[81], 6}, {NULL}};
const Segment SGI85704[] = {{&S46403[38], 11}, {NULL}};
const Segment SGI85705[] = {{&S46403[31], 6}, {NULL}};
const Segment SGI85706[] = {{&S46404[44], 11}, {NULL}};
const Segment SGI85707[] = {{&S46404[44], 23}, {NULL}};
const Segment SGI85708[] = {{&S46404[30], 6}, {NULL}};
const Segment SGI85709[] = {{&S46404[81], 6}, {NULL}};
const Segment SGI85710[] = {{&S46405[42], 11}, {NULL}};
const Segment SGI85711[] = {{&S46405[30], 6}, {NULL}};
const Segment SGI85712[] = {{&S46405[31], 3}, {NULL}};
const Segment SGI85713[] = {{&S46405[79], 6}, {NULL}};
const Segment SGI85714[] = {{&S66820[44], 10}, {NULL}};
const Segment SGI85715[] = {{&S66820[44], 27}, {NULL}};
const Segment SGI85716[] = {{&S66820[30], 6}, {NULL}};
const Segment SGI85717[] = {{&S66820[85], 6}, {NULL}};
const Segment SGI85718[] = {{&S66820[112], 13}, {NULL}};
const Segment SGI85719[] = {{&S66837[45], 11}, {NULL}};
const Segment SGI85720[] = {{&S66837[30], 6}, {NULL}};
const Segment SGI85721[] = {{&S66837[121], 3}, {NULL}};
const Segment SGI85722[] = {{&S66837[87], 5}, {NULL}};
const Segment SGI85723[] = {{&S66837[145], 13}, {NULL}};
const Segment SGI85724[] = {{&S66840[45], 10}, {NULL}};
const Segment SGI85725[] = {{&S66840[30], 6}, {NULL}};
const Segment SGI85726[] = {{&S66840[120], 3}, {NULL}};
const Segment SGI85727[] = {{&S66840[86], 5}, {NULL}};
const Segment SGI85728[] = {{&S66840[144], 13}, {NULL}};
const Segment SGI85729[] = {{&S46575[44], 11}, {NULL}};
const Segment SGI85730[] = {{&S46575[44], 23}, {NULL}};
const Segment SGI85731[] = {{&S46575[30], 6}, {NULL}};
const Segment SGI85732[] = {{&S46575[31], 3}, {NULL}};
const Segment SGI85733[] = {{&S46575[81], 6}, {NULL}};
const Segment SGI85734[] = {{&S46576[38], 11}, {NULL}};
const Segment SGI85735[] = {{&S46576[31], 6}, {NULL}};
const Segment SGI85736[] = {{&S46594[44], 10}, {NULL}};
const Segment SGI85737[] = {{&S46594[44], 22}, {NULL}};
const Segment SGI85738[] = {{&S46594[30], 6}, {NULL}};
const Segment SGI85739[] = {{&S46594[31], 3}, {NULL}};
const Segment SGI85740[] = {{&S46594[80], 6}, {NULL}};
const Segment SGI85741[] = {{&S46596[44], 12}, {NULL}};
const Segment SGI85742[] = {{&S46596[44], 24}, {NULL}};
const Segment SGI85743[] = {{&S46596[30], 6}, {NULL}};
const Segment SGI85744[] = {{&S46596[31], 3}, {NULL}};
const Segment SGI85745[] = {{&S46596[82], 6}, {NULL}};
const Segment SGI85746[] = {{&S46597[44], 16}, {NULL}};
const Segment SGI85747[] = {{&S46597[44], 28}, {NULL}};
const Segment SGI85748[] = {{&S46597[30], 6}, {NULL}};
const Segment SGI85749[] = {{&S46597[31], 3}, {NULL}};
const Segment SGI85750[] = {{&S46597[86], 6}, {NULL}};
const Segment SGI85751[] = {{&S46599[44], 11}, {NULL}};
const Segment SGI85752[] = {{&S46599[44], 23}, {NULL}};
const Segment SGI85753[] = {{&S46599[30], 6}, {NULL}};
const Segment SGI85754[] = {{&S46599[31], 3}, {NULL}};
const Segment SGI85755[] = {{&S46599[81], 6}, {NULL}};
const Segment SGI85756[] = {{&S46600[44], 15}, {NULL}};
const Segment SGI85757[] = {{&S46600[44], 27}, {NULL}};
const Segment SGI85758[] = {{&S46600[30], 6}, {NULL}};
const Segment SGI85759[] = {{&S46600[31], 3}, {NULL}};
const Segment SGI85760[] = {{&S46600[85], 6}, {NULL}};
const Segment SGI85761[] = {{&S46607[38], 16}, {NULL}};
const Segment SGI85762[] = {{&S46607[31], 6}, {NULL}};
const Segment SGI85763[] = {{&S58772[42], 13}, {NULL}};
const Segment SGI85764[] = {{&S58772[42], 30}, {NULL}};
const Segment SGI85765[] = {{&S58772[30], 4}, {NULL}};
const Segment SGI85766[] = {{&S58772[120], 3}, {NULL}};
const Segment SGI85767[] = {{&S58772[86], 5}, {NULL}};
const Segment SGI85768[] = {{&S58773[44], 13}, {NULL}};
const Segment SGI85769[] = {{&S58773[44], 30}, {NULL}};
const Segment SGI85770[] = {{&S58773[30], 6}, {NULL}};
const Segment SGI85771[] = {{&S58773[122], 3}, {NULL}};
const Segment SGI85772[] = {{&S58773[88], 5}, {NULL}};
const Segment SGI85773[] = {{&S58774[36], 13}, {NULL}};
const Segment SGI85774[] = {{&S58774[31], 4}, {NULL}};
const Segment SGI85775[] = {{&S67028[44], 10}, {NULL}};
const Segment SGI85776[] = {{&S67028[44], 27}, {NULL}};
const Segment SGI85777[] = {{&S67028[30], 6}, {NULL}};
const Segment SGI85778[] = {{&S67028[85], 6}, {NULL}};
const Segment SGI85779[] = {{&S67028[112], 13}, {NULL}};
const Segment SGI85780[] = {{&S46655[34], 10}, {NULL}};
const Segment SGI85781[] = {{&S46655[27], 6}, {NULL}};
const Segment SGI85782[] = {{&S46655[97], 9}, {NULL}};
const Segment SGI85783[] = {{&S46655[70], 6}, {NULL}};
const Segment SGI85784[] = {{&S67054[44], 11}, {NULL}};
const Segment SGI85785[] = {{&S67054[44], 22}, {NULL}};
const Segment SGI85786[] = {{&S67054[30], 6}, {NULL}};
const Segment SGI85787[] = {{&S67054[114], 3}, {NULL}};
const Segment SGI85788[] = {{&S67054[80], 5}, {NULL}};
const Segment SGI85789[] = {{&S67055[44], 7}, {NULL}};
const Segment SGI85790[] = {{&S67055[44], 24}, {NULL}};
const Segment SGI85791[] = {{&S67055[30], 6}, {NULL}};
const Segment SGI85792[] = {{&S67055[116], 3}, {NULL}};
const Segment SGI85793[] = {{&S67055[82], 5}, {NULL}};
const Segment SGI85794[] = {{&S67056[44], 7}, {NULL}};
const Segment SGI85795[] = {{&S67056[44], 24}, {NULL}};
const Segment SGI85796[] = {{&S67056[30], 6}, {NULL}};
const Segment SGI85797[] = {{&S67056[116], 3}, {NULL}};
const Segment SGI85798[] = {{&S67056[82], 5}, {NULL}};
const Segment SGI85799[] = {{&S58859[44], 10}, {NULL}};
const Segment SGI85800[] = {{&S58859[44], 21}, {NULL}};
const Segment SGI85801[] = {{&S58859[30], 6}, {NULL}};
const Segment SGI85802[] = {{&S58859[114], 3}, {NULL}};
const Segment SGI85803[] = {{&S58859[79], 6}, {NULL}};
const Segment SGI85804[] = {{&S46734[44], 10}, {NULL}};
const Segment SGI85805[] = {{&S46734[44], 27}, {NULL}};
const Segment SGI85806[] = {{&S46734[30], 6}, {NULL}};
const Segment SGI85807[] = {{&S46734[31], 3}, {NULL}};
const Segment SGI85808[] = {{&S46734[85], 5}, {NULL}};
const Segment SGI85809[] = {{&S58860[44], 15}, {NULL}};
const Segment SGI85810[] = {{&S58860[44], 26}, {NULL}};
const Segment SGI85811[] = {{&S58860[30], 6}, {NULL}};
const Segment SGI85812[] = {{&S58860[119], 3}, {NULL}};
const Segment SGI85813[] = {{&S58860[84], 6}, {NULL}};
const Segment SGI85814[] = {{&S58861[44], 10}, {NULL}};
const Segment SGI85815[] = {{&S58861[44], 21}, {NULL}};
const Segment SGI85816[] = {{&S58861[30], 6}, {NULL}};
const Segment SGI85817[] = {{&S58861[114], 3}, {NULL}};
const Segment SGI85818[] = {{&S46737[44], 10}, {NULL}};
const Segment SGI85819[] = {{&S58861[79], 6}, {NULL}};
const Segment SGI85820[] = {{&S46737[44], 32}, {NULL}};
const Segment SGI85821[] = {{&S46737[30], 6}, {NULL}};
const Segment SGI85822[] = {{&S46737[31], 3}, {NULL}};
const Segment SGI85823[] = {{&S46737[90], 5}, {NULL}};
const Segment SGI85824[] = {{&S58862[44], 16}, {NULL}};
const Segment SGI85825[] = {{&S58862[44], 27}, {NULL}};
const Segment SGI85826[] = {{&S58862[30], 6}, {NULL}};
const Segment SGI85827[] = {{&S58862[120], 3}, {NULL}};
const Segment SGI85828[] = {{&S58862[85], 6}, {NULL}};
const Segment SGI85829[] = {{&S58863[44], 15}, {NULL}};
const Segment SGI85830[] = {{&S58863[44], 26}, {NULL}};
const Segment SGI85831[] = {{&S58863[30], 6}, {NULL}};
const Segment SGI85832[] = {{&S58863[119], 3}, {NULL}};
const Segment SGI85833[] = {{&S58863[84], 6}, {NULL}};
const Segment SGI85834[] = {{&S58864[44], 9}, {NULL}};
const Segment SGI85835[] = {{&S58864[44], 20}, {NULL}};
const Segment SGI85836[] = {{&S58864[30], 6}, {NULL}};
const Segment SGI85837[] = {{&S58864[113], 3}, {NULL}};
const Segment SGI85838[] = {{&S58864[78], 6}, {NULL}};
const Segment SGI85839[] = {{&S59542[44], 16}, {NULL}};
const Segment SGI85840[] = {{&S59542[44], 27}, {NULL}};
const Segment SGI85841[] = {{&S59542[30], 6}, {NULL}};
const Segment SGI85842[] = {{&S58865[44], 20}, {NULL}};
const Segment SGI85843[] = {{&S58865[44], 31}, {NULL}};
const Segment SGI85844[] = {{&S58865[30], 6}, {&S58865[52], 23}, {NULL}};
const Segment SGI85845[] = {{&S59542[138], 3}, {NULL}};
const Segment SGI85846[] = {{&S58865[124], 3}, {NULL}};
const Segment SGI85847[] = {{&S59542[104], 5}, {NULL}};
const Segment SGI85848[] = {{&S59542[35], 1}, {NULL}};
const Segment SGI85849[] = {{&S58865[89], 6}, {NULL}};
const Segment SGI85850[] = {{&S59542[73], 11}, {NULL}};
const Segment SGI85851[] = {{&S58866[44], 15}, {NULL}};
const Segment SGI85852[] = {{&S58866[44], 26}, {NULL}};
const Segment SGI85853[] = {{&S58866[30], 6}, {NULL}};
const Segment SGI85854[] = {{&S59543[44], 12}, {NULL}};
const Segment SGI85855[] = {{&S59543[44], 23}, {NULL}};
const Segment SGI85856[] = {{&S59543[30], 6}, {NULL}};
const Segment SGI85857[] = {{&S58866[119], 3}, {NULL}};
const Segment SGI85858[] = {{&S58866[84], 6}, {NULL}};
const Segment SGI85859[] = {{&S59543[134], 3}, {NULL}};
const Segment SGI85860[] = {{&S59543[100], 5}, {NULL}};
const Segment SGI85861[] = {{&S59543[35], 1}, {NULL}};
const Segment SGI85862[] = {{&S59543[69], 11}, {NULL}};
const Segment SGI85863[] = {{&S58867[44], 15}, {NULL}};
const Segment SGI85864[] = {{&S58867[44], 26}, {NULL}};
const Segment SGI85865[] = {{&S58867[30], 6}, {NULL}};
const Segment SGI85866[] = {{&S58867[119], 3}, {NULL}};
const Segment SGI85867[] = {{&S59544[44], 12}, {NULL}};
const Segment SGI85868[] = {{&S58867[84], 6}, {NULL}};
const Segment SGI85869[] = {{&S59544[44], 24}, {NULL}};
const Segment SGI85870[] = {{&S59544[30], 6}, {NULL}};
const Segment SGI85871[] = {{&S59544[131], 3}, {NULL}};
const Segment SGI85872[] = {{&S59544[97], 5}, {NULL}};
const Segment SGI85873[] = {{&S59544[35], 1}, {NULL}};
const Segment SGI85874[] = {{&S59544[70], 11}, {NULL}};
const Segment SGI85875[] = {{&S58868[44], 14}, {NULL}};
const Segment SGI85876[] = {{&S58868[44], 31}, {NULL}};
const Segment SGI85877[] = {{&S58868[30], 6}, {NULL}};
const Segment SGI85878[] = {{&S58868[123], 3}, {NULL}};
const Segment SGI85879[] = {{&S58868[89], 5}, {NULL}};
const Segment SGI85880[] = {{&S59545[44], 18}, {NULL}};
const Segment SGI85881[] = {{&S59545[44], 30}, {NULL}};
const Segment SGI85882[] = {{&S59545[30], 6}, {NULL}};
const Segment SGI85883[] = {{&S59545[137], 3}, {NULL}};
const Segment SGI85884[] = {{&S59545[103], 5}, {NULL}};
const Segment SGI85885[] = {{&S59545[35], 1}, {NULL}};
const Segment SGI85886[] = {{&S59545[76], 11}, {NULL}};
const Segment SGI85887[] = {{&S58869[44], 10}, {NULL}};
const Segment SGI85888[] = {{&S58869[44], 22}, {NULL}};
const Segment SGI85889[] = {{&S58869[30], 6}, {NULL}};
const Segment SGI85890[] = {{&S58869[115], 3}, {NULL}};
const Segment SGI85891[] = {{&S58869[80], 6}, {NULL}};
const Segment SGI85892[] = {{&S58870[44], 11}, {NULL}};
const Segment SGI85893[] = {{&S58870[44], 28}, {NULL}};
const Segment SGI85894[] = {{&S58870[30], 6}, {NULL}};
const Segment SGI85895[] = {{&S58870[120], 3}, {NULL}};
const Segment SGI85896[] = {{&S58870[86], 5}, {NULL}};
const Segment SGI85897[] = {{&S58871[44], 5}, {NULL}};
const Segment SGI85898[] = {{&S58871[44], 22}, {NULL}};
const Segment SGI85899[] = {{&S58871[30], 6}, {NULL}};
const Segment SGI85900[] = {{&S58871[114], 3}, {NULL}};
const Segment SGI85901[] = {{&S58871[80], 5}, {NULL}};
const Segment SGI85902[] = {{&S58872[44], 11}, {NULL}};
const Segment SGI85903[] = {{&S58872[44], 28}, {NULL}};
const Segment SGI85904[] = {{&S58872[30], 6}, {NULL}};
const Segment SGI85905[] = {{&S58872[119], 3}, {NULL}};
const Segment SGI85906[] = {{&S58872[86], 5}, {NULL}};
const Segment SGI85907[] = {{&S58873[42], 11}, {NULL}};
const Segment SGI85908[] = {{&S58873[42], 26}, {NULL}};
const Segment SGI85909[] = {{&S58873[30], 4}, {NULL}};
const Segment SGI85910[] = {{&S58873[137], 3}, {NULL}};
const Segment SGI85911[] = {{&S58873[103], 5}, {NULL}};
const Segment SGI85912[] = {{&S67143[45], 10}, {NULL}};
const Segment SGI85913[] = {{&S67143[30], 6}, {NULL}};
const Segment SGI85914[] = {{&S67143[120], 3}, {NULL}};
const Segment SGI85915[] = {{&S67143[86], 5}, {NULL}};
const Segment SGI85916[] = {{&S67143[144], 13}, {NULL}};
const Segment SGI85917[] = {{&S67144[44], 10}, {NULL}};
const Segment SGI85918[] = {{&S67144[44], 27}, {NULL}};
const Segment SGI85919[] = {{&S67144[30], 6}, {NULL}};
const Segment SGI85920[] = {{&S67144[120], 3}, {NULL}};
const Segment SGI85921[] = {{&S67144[85], 6}, {NULL}};
const Segment SGI85922[] = {{&S67144[140], 8}, {NULL}};
const Segment SGI85923[] = {{&S67144[149], 13}, {NULL}};
const Segment SGI85924[] = {{&S67156[45], 10}, {NULL}};
const Segment SGI85925[] = {{&S67156[30], 6}, {NULL}};
const Segment SGI85926[] = {{&S67156[120], 3}, {NULL}};
const Segment SGI85927[] = {{&S67156[86], 5}, {NULL}};
const Segment SGI85928[] = {{&S67156[144], 13}, {NULL}};
const Segment SGI85929[] = {{&S67165[44], 7}, {NULL}};
const Segment SGI85930[] = {{&S67165[44], 28}, {NULL}};
const Segment SGI85931[] = {{&S67165[30], 6}, {NULL}};
const Segment SGI85932[] = {{&S67165[89], 4}, {NULL}};
const Segment SGI85933[] = {{&S67165[89], 3}, {NULL}};
const Segment SGI85934[] = {{&S67166[44], 26}, {NULL}};
const Segment SGI85935[] = {{&S67166[30], 6}, {NULL}};
const Segment SGI85936[] = {{&S67166[136], 3}, {NULL}};
const Segment SGI85937[] = {{&S67166[102], 5}, {NULL}};
const Segment SGI85938[] = {{&S67169[44], 7}, {NULL}};
const Segment SGI85939[] = {{&S67169[44], 28}, {NULL}};
const Segment SGI85940[] = {{&S67169[30], 6}, {NULL}};
const Segment SGI85941[] = {{&S67169[120], 3}, {NULL}};
const Segment SGI85942[] = {{&S67169[86], 5}, {NULL}};
const Segment SGI85943[] = {{&S67170[44], 7}, {NULL}};
const Segment SGI85944[] = {{&S67170[44], 28}, {NULL}};
const Segment SGI85945[] = {{&S67170[30], 6}, {NULL}};
const Segment SGI85946[] = {{&S67170[89], 6}, {NULL}};
const Segment SGI85947[] = {{&S46826[34], 10}, {NULL}};
const Segment SGI85948[] = {{&S46826[27], 6}, {NULL}};
const Segment SGI85949[] = {{&S46826[97], 9}, {NULL}};
const Segment SGI85950[] = {{&S46826[70], 6}, {NULL}};
const Segment SGI85951[] = {{&S46828[34], 11}, {NULL}};
const Segment SGI85952[] = {{&S46828[27], 6}, {NULL}};
const Segment SGI85953[] = {{&S46828[98], 9}, {NULL}};
const Segment SGI85954[] = {{&S46828[71], 6}, {NULL}};
const Segment SGI85955[] = {{&S46856[34], 11}, {NULL}};
const Segment SGI85956[] = {{&S46856[27], 6}, {NULL}};
const Segment SGI85957[] = {{&S46856[98], 9}, {NULL}};
const Segment SGI85958[] = {{&S46856[71], 6}, {NULL}};
const Segment SGI85959[] = {{&S46857[34], 10}, {NULL}};
const Segment SGI85960[] = {{&S46857[27], 6}, {NULL}};
const Segment SGI85961[] = {{&S67226[44], 10}, {NULL}};
const Segment SGI85962[] = {{&S67226[44], 22}, {NULL}};
const Segment SGI85963[] = {{&S67226[30], 6}, {NULL}};
const Segment SGI85964[] = {{&S46857[97], 9}, {NULL}};
const Segment SGI85965[] = {{&S67226[114], 3}, {NULL}};
const Segment SGI85966[] = {{&S46857[70], 6}, {NULL}};
const Segment SGI85967[] = {{&S67226[80], 5}, {NULL}};
const Segment SGI85968[] = {{&S67284[45], 10}, {NULL}};
const Segment SGI85969[] = {{&S67284[30], 6}, {NULL}};
const Segment SGI85970[] = {{&S67284[120], 3}, {NULL}};
const Segment SGI85971[] = {{&S67284[86], 5}, {NULL}};
const Segment SGI85972[] = {{&S67284[144], 13}, {NULL}};
const Segment SGI85973[] = {{&S67378[44], 10}, {NULL}};
const Segment SGI85974[] = {{&S67378[44], 22}, {NULL}};
const Segment SGI85975[] = {{&S67378[30], 6}, {NULL}};
const Segment SGI85976[] = {{&S67378[114], 3}, {NULL}};
const Segment SGI85977[] = {{&S67378[80], 5}, {NULL}};
const Segment SGI85978[] = {{&S67379[44], 11}, {NULL}};
const Segment SGI85979[] = {{&S67379[44], 23}, {NULL}};
const Segment SGI85980[] = {{&S67379[30], 6}, {NULL}};
const Segment SGI85981[] = {{&S67379[115], 3}, {NULL}};
const Segment SGI85982[] = {{&S67379[81], 5}, {NULL}};
const Segment SGI85983[] = {{&S59844[44], 18}, {NULL}};
const Segment SGI85984[] = {{&S59844[44], 29}, {NULL}};
const Segment SGI85985[] = {{&S59844[30], 6}, {NULL}};
const Segment SGI85986[] = {{&S59844[121], 3}, {NULL}};
const Segment SGI85987[] = {{&S59844[87], 5}, {NULL}};
const Segment SGI85988[] = {{&S59845[44], 13}, {NULL}};
const Segment SGI85989[] = {{&S59845[44], 24}, {NULL}};
const Segment SGI85990[] = {{&S59845[30], 6}, {NULL}};
const Segment SGI85991[] = {{&S59845[116], 3}, {NULL}};
const Segment SGI85992[] = {{&S59845[82], 5}, {NULL}};
const Segment SGI85993[] = {{&S59846[44], 18}, {NULL}};
const Segment SGI85994[] = {{&S59846[44], 29}, {NULL}};
const Segment SGI85995[] = {{&S59846[30], 6}, {NULL}};
const Segment SGI85996[] = {{&S59846[121], 3}, {NULL}};
const Segment SGI85997[] = {{&S59846[87], 5}, {NULL}};
const Segment SGI85998[] = {{&S59848[44], 15}, {NULL}};
const Segment SGI85999[] = {{&S59848[44], 26}, {NULL}};
const Segment SGI86000[] = {{&S59848[30], 6}, {NULL}};
const Segment SGI86001[] = {{&S59848[118], 3}, {NULL}};
const Segment SGI86002[] = {{&S59848[84], 5}, {NULL}};
const Segment SGI86003[] = {{&S59849[38], 18}, {NULL}};
const Segment SGI86004[] = {{&S59849[31], 6}, {NULL}};
const Segment SGI86005[] = {{&S52350[44], 10}, {NULL}};
const Segment SGI86006[] = {{&S52350[44], 22}, {NULL}};
const Segment SGI86007[] = {{&S52350[30], 6}, {NULL}};
const Segment SGI86008[] = {{&S67502[44], 10}, {NULL}};
const Segment SGI86009[] = {{&S52350[31], 3}, {NULL}};
const Segment SGI86010[] = {{&S67502[44], 21}, {NULL}};
const Segment SGI86011[] = {{&S67502[30], 6}, {NULL}};
const Segment SGI86012[] = {{&S52350[80], 6}, {NULL}};
const Segment SGI86013[] = {{&S67502[113], 3}, {NULL}};
const Segment SGI86014[] = {{&S67502[79], 5}, {NULL}};
const Segment SGI86015[] = {{&S67503[44], 10}, {NULL}};
const Segment SGI86016[] = {{&S67503[44], 22}, {NULL}};
const Segment SGI86017[] = {{&S67503[30], 6}, {NULL}};
const Segment SGI86018[] = {{&S67503[114], 3}, {NULL}};
const Segment SGI86019[] = {{&S67503[80], 5}, {NULL}};
const Segment SGI86020[] = {{&S67540[44], 10}, {NULL}};
const Segment SGI86021[] = {{&S67540[44], 22}, {NULL}};
const Segment SGI86022[] = {{&S67540[30], 6}, {NULL}};
const Segment SGI86023[] = {{&S67540[114], 3}, {NULL}};
const Segment SGI86024[] = {{&S67540[80], 5}, {NULL}};
const Segment SGI86025[] = {{&S47064[34], 19}, {NULL}};
const Segment SGI86026[] = {{&S47064[27], 6}, {NULL}};
const Segment SGI86027[] = {{&S47064[106], 9}, {NULL}};
const Segment SGI86028[] = {{&S47064[79], 6}, {NULL}};
const Segment SGI86029[] = {{&S47067[34], 18}, {NULL}};
const Segment SGI86030[] = {{&S47067[27], 6}, {NULL}};
const Segment SGI86031[] = {{&S47067[105], 9}, {NULL}};
const Segment SGI86032[] = {{&S47067[78], 6}, {NULL}};
const Segment SGI86033[] = {{&S67581[44], 15}, {NULL}};
const Segment SGI86034[] = {{&S67581[44], 27}, {NULL}};
const Segment SGI86035[] = {{&S67581[30], 6}, {NULL}};
const Segment SGI86036[] = {{&S67581[119], 3}, {NULL}};
const Segment SGI86037[] = {{&S67581[85], 5}, {NULL}};
const Segment SGI86038[] = {{&S47090[44], 10}, {NULL}};
const Segment SGI86039[] = {{&S47090[44], 27}, {NULL}};
const Segment SGI86040[] = {{&S47090[30], 6}, {NULL}};
const Segment SGI86041[] = {{&S47090[31], 3}, {NULL}};
const Segment SGI86042[] = {{&S47090[85], 5}, {NULL}};
const Segment SGI86043[] = {{&S47093[34], 10}, {NULL}};
const Segment SGI86044[] = {{&S47093[27], 6}, {NULL}};
const Segment SGI86045[] = {{&S47093[97], 9}, {NULL}};
const Segment SGI86046[] = {{&S47093[70], 6}, {NULL}};
const Segment SGI86047[] = {{&S47094[34], 11}, {NULL}};
const Segment SGI86048[] = {{&S47094[27], 6}, {NULL}};
const Segment SGI86049[] = {{&S60027[44], 10}, {NULL}};
const Segment SGI86050[] = {{&S60027[44], 27}, {NULL}};
const Segment SGI86051[] = {{&S60027[30], 6}, {NULL}};
const Segment SGI86052[] = {{&S47094[98], 9}, {NULL}};
const Segment SGI86053[] = {{&S47094[71], 6}, {NULL}};
const Segment SGI86054[] = {{&S60027[119], 3}, {NULL}};
const Segment SGI86055[] = {{&S60027[85], 5}, {NULL}};
const Segment SGI86056[] = {{&S47097[44], 7}, {NULL}};
const Segment SGI86057[] = {{&S67602[45], 10}, {NULL}};
const Segment SGI86058[] = {{&S47097[44], 16}, {NULL}};
const Segment SGI86059[] = {{&S47097[30], 6}, {NULL}};
const Segment SGI86060[] = {{&S67602[30], 6}, {NULL}};
const Segment SGI86061[] = {{&S47097[31], 3}, {NULL}};
const Segment SGI86062[] = {{&S67602[120], 3}, {NULL}};
const Segment SGI86063[] = {{&S47097[74], 6}, {NULL}};
const Segment SGI86064[] = {{&S67602[86], 5}, {NULL}};
const Segment SGI86065[] = {{&S67602[144], 13}, {NULL}};
const Segment SGI86066[] = {{&S67635[44], 6}, {NULL}};
const Segment SGI86067[] = {{&S67635[44], 25}, {NULL}};
const Segment SGI86068[] = {{&S67635[30], 6}, {NULL}};
const Segment SGI86069[] = {{&S67635[117], 3}, {NULL}};
const Segment SGI86070[] = {{&S67635[83], 5}, {NULL}};
const Segment SGI86071[] = {{&S67639[44], 14}, {NULL}};
const Segment SGI86072[] = {{&S67639[44], 26}, {NULL}};
const Segment SGI86073[] = {{&S67639[30], 6}, {NULL}};
const Segment SGI86074[] = {{&S67639[118], 3}, {NULL}};
const Segment SGI86075[] = {{&S67639[84], 5}, {NULL}};
const Segment SGI86076[] = {{&S47161[44], 9}, {NULL}};
const Segment SGI86077[] = {{&S47161[44], 21}, {NULL}};
const Segment SGI86078[] = {{&S47161[30], 6}, {NULL}};
const Segment SGI86079[] = {{&S47161[31], 3}, {NULL}};
const Segment SGI86080[] = {{&S47161[79], 6}, {NULL}};
const Segment SGI86081[] = {{&S47164[38], 9}, {NULL}};
const Segment SGI86082[] = {{&S47164[31], 6}, {NULL}};
const Segment SGI86083[] = {{&S47165[44], 9}, {NULL}};
const Segment SGI86084[] = {{&S47165[44], 21}, {NULL}};
const Segment SGI86085[] = {{&S47165[30], 6}, {NULL}};
const Segment SGI86086[] = {{&S47165[31], 3}, {NULL}};
const Segment SGI86087[] = {{&S47165[79], 6}, {NULL}};
const Segment SGI86088[] = {{&S47166[44], 9}, {NULL}};
const Segment SGI86089[] = {{&S47166[44], 21}, {NULL}};
const Segment SGI86090[] = {{&S47166[30], 6}, {NULL}};
const Segment SGI86091[] = {{&S47166[31], 3}, {NULL}};
const Segment SGI86092[] = {{&S47166[79], 6}, {NULL}};
const Segment SGI86093[] = {{&S67685[45], 11}, {NULL}};
const Segment SGI86094[] = {{&S67685[30], 6}, {NULL}};
const Segment SGI86095[] = {{&S67685[121], 3}, {NULL}};
const Segment SGI86096[] = {{&S67685[87], 5}, {NULL}};
const Segment SGI86097[] = {{&S67685[145], 13}, {NULL}};
const Segment SGI86098[] = {{&S67687[45], 10}, {NULL}};
const Segment SGI86099[] = {{&S67687[30], 6}, {NULL}};
const Segment SGI86100[] = {{&S67687[120], 3}, {NULL}};
const Segment SGI86101[] = {{&S67687[86], 5}, {NULL}};
const Segment SGI86102[] = {{&S67687[144], 13}, {NULL}};
const Segment SGI86103[] = {{&S47181[44], 10}, {NULL}};
const Segment SGI86104[] = {{&S47181[44], 27}, {NULL}};
const Segment SGI86105[] = {{&S47181[30], 6}, {NULL}};
const Segment SGI86106[] = {{&S47181[31], 3}, {NULL}};
const Segment SGI86107[] = {{&S47181[85], 5}, {NULL}};
const Segment SGI86108[] = {{&S67706[44], 5}, {NULL}};
const Segment SGI86109[] = {{&S67706[44], 17}, {NULL}};
const Segment SGI86110[] = {{&S67706[30], 6}, {NULL}};
const Segment SGI86111[] = {{&S67706[109], 3}, {NULL}};
const Segment SGI86112[] = {{&S67706[75], 5}, {NULL}};
const Segment SGI86113[] = {{&S60068[44], 10}, {NULL}};
const Segment SGI86114[] = {{&S60068[44], 21}, {NULL}};
const Segment SGI86115[] = {{&S60068[30], 6}, {NULL}};
const Segment SGI86116[] = {{&S60068[113], 3}, {NULL}};
const Segment SGI86117[] = {{&S60068[79], 5}, {NULL}};
const Segment SGI86118[] = {{&S60070[44], 10}, {NULL}};
const Segment SGI86119[] = {{&S60070[44], 21}, {NULL}};
const Segment SGI86120[] = {{&S60070[30], 6}, {NULL}};
const Segment SGI86121[] = {{&S60070[113], 3}, {NULL}};
const Segment SGI86122[] = {{&S60070[79], 5}, {NULL}};
const Segment SGI86123[] = {{&S60071[44], 10}, {NULL}};
const Segment SGI86124[] = {{&S60071[44], 21}, {NULL}};
const Segment SGI86125[] = {{&S60071[30], 6}, {NULL}};
const Segment SGI86126[] = {{&S60071[146], 3}, {NULL}};
const Segment SGI86127[] = {{&S60071[112], 5}, {NULL}};
const Segment SGI86128[] = {{&S67759[45], 10}, {NULL}};
const Segment SGI86129[] = {{&S67759[30], 6}, {NULL}};
const Segment SGI86130[] = {{&S67759[120], 3}, {NULL}};
const Segment SGI86131[] = {{&S67759[86], 5}, {NULL}};
const Segment SGI86132[] = {{&S67759[144], 13}, {NULL}};
const Segment SGI86133[] = {{&S60073[44], 10}, {NULL}};
const Segment SGI86134[] = {{&S60073[44], 21}, {NULL}};
const Segment SGI86135[] = {{&S60073[30], 6}, {NULL}};
const Segment SGI86136[] = {{&S60073[136], 3}, {NULL}};
const Segment SGI86137[] = {{&S60073[102], 5}, {NULL}};
const Segment SGI86138[] = {{&S60074[44], 10}, {NULL}};
const Segment SGI86139[] = {{&S60074[44], 21}, {NULL}};
const Segment SGI86140[] = {{&S60074[30], 6}, {NULL}};
const Segment SGI86141[] = {{&S60074[128], 3}, {NULL}};
const Segment SGI86142[] = {{&S60074[94], 5}, {NULL}};
const Segment SGI86143[] = {{&S60075[44], 10}, {NULL}};
const Segment SGI86144[] = {{&S60075[44], 21}, {NULL}};
const Segment SGI86145[] = {{&S60075[30], 6}, {NULL}};
const Segment SGI86146[] = {{&S60075[128], 3}, {NULL}};
const Segment SGI86147[] = {{&S60075[94], 5}, {NULL}};
const Segment SGI86148[] = {{&S60076[38], 10}, {NULL}};
const Segment SGI86149[] = {{&S60076[31], 6}, {NULL}};
const Segment SGI86150[] = {{&S47254[44], 6}, {NULL}};
const Segment SGI86151[] = {{&S47254[44], 18}, {NULL}};
const Segment SGI86152[] = {{&S47254[30], 6}, {NULL}};
const Segment SGI86153[] = {{&S47254[31], 3}, {NULL}};
const Segment SGI86154[] = {{&S47254[76], 6}, {NULL}};
const Segment SGI86155[] = {{&S47255[38], 6}, {NULL}};
const Segment SGI86156[] = {{&S47255[31], 6}, {NULL}};
const Segment SGI86157[] = {{&S47315[34], 11}, {NULL}};
const Segment SGI86158[] = {{&S47315[27], 6}, {NULL}};
const Segment SGI86159[] = {{&S47315[98], 9}, {NULL}};
const Segment SGI86160[] = {{&S47315[71], 6}, {NULL}};
const Segment SGI86161[] = {{&S47316[34], 10}, {NULL}};
const Segment SGI86162[] = {{&S47316[27], 6}, {NULL}};
const Segment SGI86163[] = {{&S47316[97], 9}, {NULL}};
const Segment SGI86164[] = {{&S47316[70], 6}, {NULL}};
const Segment SGI86165[] = {{&S67840[44], 30}, {NULL}};
const Segment SGI86166[] = {{&S67840[44], 42}, {NULL}};
const Segment SGI86167[] = {{&S67840[30], 6}, {NULL}};
const Segment SGI86168[] = {{&S67840[134], 3}, {NULL}};
const Segment SGI86169[] = {{&S67840[100], 5}, {NULL}};
const Segment SGI86170[] = {{&S67841[44], 30}, {NULL}};
const Segment SGI86171[] = {{&S67841[44], 41}, {NULL}};
const Segment SGI86172[] = {{&S67841[30], 6}, {NULL}};
const Segment SGI86173[] = {{&S67841[133], 3}, {NULL}};
const Segment SGI86174[] = {{&S67841[99], 5}, {NULL}};
const Segment SGI86175[] = {{&S67855[44], 21}, {NULL}};
const Segment SGI86176[] = {{&S67855[44], 33}, {NULL}};
const Segment SGI86177[] = {{&S67855[30], 6}, {NULL}};
const Segment SGI86178[] = {{&S67855[125], 3}, {NULL}};
const Segment SGI86179[] = {{&S67855[91], 5}, {NULL}};
const Segment SGI86180[] = {{&S47353[34], 19}, {NULL}};
const Segment SGI86181[] = {{&S47353[27], 6}, {NULL}};
const Segment SGI86182[] = {{&S47353[106], 9}, {NULL}};
const Segment SGI86183[] = {{&S47353[79], 6}, {NULL}};
const Segment SGI86184[] = {{&S47355[34], 7}, {NULL}};
const Segment SGI86185[] = {{&S47355[27], 6}, {NULL}};
const Segment SGI86186[] = {{&S47355[94], 9}, {NULL}};
const Segment SGI86187[] = {{&S47355[67], 6}, {NULL}};
const Segment SGI86188[] = {{&S47370[44], 9}, {NULL}};
const Segment SGI86189[] = {{&S47370[44], 21}, {NULL}};
const Segment SGI86190[] = {{&S47370[30], 6}, {NULL}};
const Segment SGI86191[] = {{&S47370[31], 3}, {NULL}};
const Segment SGI86192[] = {{&S47370[79], 6}, {NULL}};
const Segment SGI86193[] = {{&S47371[38], 9}, {NULL}};
const Segment SGI86194[] = {{&S47371[31], 6}, {NULL}};
const Segment SGI86195[] = {{&S47426[34], 7}, {NULL}};
const Segment SGI86196[] = {{&S47426[27], 6}, {NULL}};
const Segment SGI86197[] = {{&S47426[94], 9}, {NULL}};
const Segment SGI86198[] = {{&S47426[67], 6}, {NULL}};
const Segment SGI86199[] = {{&S47428[34], 6}, {NULL}};
const Segment SGI86200[] = {{&S47428[27], 6}, {NULL}};
const Segment SGI86201[] = {{&S47428[93], 9}, {NULL}};
const Segment SGI86202[] = {{&S47428[66], 6}, {NULL}};
const Segment SGI86203[] = {{&S57245[6], 4}, {NULL}};
const Segment SGI86204[] = {{&S57245[74], 5}, {NULL}};
const Segment SGI86205[] = {{&S47431[34], 18}, {NULL}};
const Segment SGI86206[] = {{&S47431[27], 6}, {NULL}};
const Segment SGI86207[] = {{&S47431[102], 9}, {NULL}};
const Segment SGI86208[] = {{&S47431[75], 6}, {NULL}};
const Segment SGI86209[] = {{&S59758[44], 11}, {NULL}};
const Segment SGI86210[] = {{&S59758[44], 34}, {NULL}};
const Segment SGI86211[] = {{&S59758[30], 6}, {NULL}};
const Segment SGI86212[] = {{&S59758[126], 3}, {NULL}};
const Segment SGI86213[] = {{&S59758[92], 5}, {NULL}};
const Segment SGI86214[] = {{&S59759[44], 11}, {NULL}};
const Segment SGI86215[] = {{&S59759[44], 34}, {NULL}};
const Segment SGI86216[] = {{&S59759[30], 6}, {NULL}};
const Segment SGI86217[] = {{&S59759[126], 3}, {NULL}};
const Segment SGI86218[] = {{&S59759[92], 5}, {NULL}};
const Segment SGI86219[] = {{&S59760[44], 11}, {NULL}};
const Segment SGI86220[] = {{&S59760[44], 34}, {NULL}};
const Segment SGI86221[] = {{&S59760[30], 6}, {NULL}};
const Segment SGI86222[] = {{&S59760[126], 3}, {NULL}};
const Segment SGI86223[] = {{&S59760[92], 5}, {NULL}};
const Segment SGI86224[] = {{&S59761[44], 11}, {NULL}};
const Segment SGI86225[] = {{&S59761[44], 34}, {NULL}};
const Segment SGI86226[] = {{&S59761[30], 6}, {NULL}};
const Segment SGI86227[] = {{&S59761[126], 3}, {NULL}};
const Segment SGI86228[] = {{&S59761[92], 5}, {NULL}};
const Segment SGI86229[] = {{&S59762[44], 7}, {NULL}};
const Segment SGI86230[] = {{&S59762[44], 31}, {NULL}};
const Segment SGI86231[] = {{&S59762[30], 6}, {NULL}};
const Segment SGI86232[] = {{&S59762[123], 3}, {NULL}};
const Segment SGI86233[] = {{&S59762[89], 5}, {NULL}};
const Segment SGI86234[] = {{&S59763[44], 7}, {NULL}};
const Segment SGI86235[] = {{&S59763[44], 32}, {NULL}};
const Segment SGI86236[] = {{&S59763[30], 6}, {NULL}};
const Segment SGI86237[] = {{&S59763[124], 3}, {NULL}};
const Segment SGI86238[] = {{&S59763[90], 5}, {NULL}};
const Segment SGI86239[] = {{&S59764[44], 7}, {NULL}};
const Segment SGI86240[] = {{&S59764[44], 32}, {NULL}};
const Segment SGI86241[] = {{&S59764[30], 6}, {NULL}};
const Segment SGI86242[] = {{&S59764[124], 3}, {NULL}};
const Segment SGI86243[] = {{&S59764[90], 5}, {NULL}};
const Segment SGI86244[] = {{&S59765[44], 7}, {NULL}};
const Segment SGI86245[] = {{&S59765[44], 32}, {NULL}};
const Segment SGI86246[] = {{&S59765[30], 6}, {NULL}};
const Segment SGI86247[] = {{&S59765[125], 3}, {NULL}};
const Segment SGI86248[] = {{&S59765[90], 6}, {NULL}};
const Segment SGI86249[] = {{&S59766[44], 11}, {NULL}};
const Segment SGI86250[] = {{&S59766[44], 34}, {NULL}};
const Segment SGI86251[] = {{&S59766[30], 6}, {NULL}};
const Segment SGI86252[] = {{&S59766[126], 3}, {NULL}};
const Segment SGI86253[] = {{&S59766[92], 5}, {NULL}};
const Segment SGI86254[] = {{&S59767[44], 7}, {NULL}};
const Segment SGI86255[] = {{&S59767[44], 32}, {NULL}};
const Segment SGI86256[] = {{&S59767[30], 6}, {NULL}};
const Segment SGI86257[] = {{&S59767[124], 3}, {NULL}};
const Segment SGI86258[] = {{&S59767[90], 5}, {NULL}};
const Segment SGI86259[] = {{&S54945[44], 10}, {NULL}};
const Segment SGI86260[] = {{&S54945[44], 27}, {NULL}};
const Segment SGI86261[] = {{&S54945[30], 6}, {NULL}};
const Segment SGI86262[] = {{&S54945[31], 3}, {NULL}};
const Segment SGI86263[] = {{&S54945[85], 6}, {NULL}};
const Segment SGI86264[] = {{&S54945[0], 13}, {NULL}};
const Segment SGI86265[] = {{&S68019[31], 9}, {NULL}};
const Segment SGI86266[] = {{&S68019[55], 3}, {NULL}};
const Segment SGI86267[] = {{&S68020[31], 8}, {NULL}};
const Segment SGI86268[] = {{&S68020[54], 3}, {NULL}};
const Segment SGI86269[] = {{&S68095[44], 17}, {NULL}};
const Segment SGI86270[] = {{&S68095[44], 38}, {NULL}};
const Segment SGI86271[] = {{&S68095[30], 6}, {NULL}};
const Segment SGI86272[] = {{&S68095[130], 3}, {NULL}};
const Segment SGI86273[] = {{&S68095[96], 5}, {NULL}};
const Segment SGI86274[] = {{&S68096[44], 17}, {NULL}};
const Segment SGI86275[] = {{&S68096[44], 38}, {NULL}};
const Segment SGI86276[] = {{&S68096[30], 6}, {NULL}};
const Segment SGI86277[] = {{&S68096[130], 3}, {NULL}};
const Segment SGI86278[] = {{&S68096[96], 5}, {NULL}};
const Segment SGI86279[] = {{&S68148[44], 14}, {NULL}};
const Segment SGI86280[] = {{&S68148[44], 26}, {NULL}};
const Segment SGI86281[] = {{&S68148[30], 6}, {NULL}};
const Segment SGI86282[] = {{&S68148[118], 3}, {NULL}};
const Segment SGI86283[] = {{&S68148[84], 5}, {NULL}};
const Segment SGI86284[] = {{&S68157[44], 10}, {NULL}};
const Segment SGI86285[] = {{&S68157[44], 27}, {NULL}};
const Segment SGI86286[] = {{&S68157[30], 6}, {NULL}};
const Segment SGI86287[] = {{&S68157[85], 6}, {NULL}};
const Segment SGI86288[] = {{&S68157[112], 13}, {NULL}};
const Segment SGI86289[] = {{&S68158[44], 10}, {NULL}};
const Segment SGI86290[] = {{&S68158[44], 27}, {NULL}};
const Segment SGI86291[] = {{&S68158[30], 6}, {NULL}};
const Segment SGI86292[] = {{&S68158[85], 6}, {NULL}};
const Segment SGI86293[] = {{&S68158[112], 13}, {NULL}};
const Segment SGI86294[] = {{&S68159[44], 11}, {NULL}};
const Segment SGI86295[] = {{&S68159[44], 28}, {NULL}};
const Segment SGI86296[] = {{&S68159[30], 6}, {NULL}};
const Segment SGI86297[] = {{&S68159[86], 6}, {NULL}};
const Segment SGI86298[] = {{&S68159[113], 13}, {NULL}};
const Segment SGI86299[] = {{&S60432[44], 10}, {NULL}};
const Segment SGI86300[] = {{&S60432[44], 22}, {NULL}};
const Segment SGI86301[] = {{&S60432[30], 6}, {NULL}};
const Segment SGI86302[] = {{&S60432[138], 3}, {NULL}};
const Segment SGI86303[] = {{&S60432[104], 5}, {NULL}};
const Segment SGI86304[] = {{&S60432[85], 3}, {NULL}};
const Segment SGI86305[] = {{&S60433[44], 10}, {NULL}};
const Segment SGI86306[] = {{&S60433[44], 22}, {NULL}};
const Segment SGI86307[] = {{&S60433[30], 6}, {NULL}};
const Segment SGI86308[] = {{&S60433[133], 3}, {NULL}};
const Segment SGI86309[] = {{&S60433[99], 5}, {NULL}};
const Segment SGI86310[] = {{&S60433[35], 1}, {NULL}};
const Segment SGI86311[] = {{&S60433[68], 11}, {NULL}};
const Segment SGI86312[] = {{&S60434[44], 10}, {NULL}};
const Segment SGI86313[] = {{&S60434[44], 21}, {NULL}};
const Segment SGI86314[] = {{&S60434[30], 6}, {NULL}};
const Segment SGI86315[] = {{&S60434[132], 3}, {NULL}};
const Segment SGI86316[] = {{&S60434[98], 5}, {NULL}};
const Segment SGI86317[] = {{&S60434[35], 1}, {NULL}};
const Segment SGI86318[] = {{&S60434[67], 11}, {NULL}};
const Segment SGI86319[] = {{&S60435[44], 10}, {NULL}};
const Segment SGI86320[] = {{&S60435[44], 21}, {NULL}};
const Segment SGI86321[] = {{&S60435[30], 6}, {NULL}};
const Segment SGI86322[] = {{&S60435[137], 3}, {NULL}};
const Segment SGI86323[] = {{&S60435[103], 5}, {NULL}};
const Segment SGI86324[] = {{&S60435[84], 3}, {NULL}};
const Segment SGI86325[] = {{&S60439[54], 10}, {NULL}};
const Segment SGI86326[] = {{&S60439[54], 28}, {NULL}};
const Segment SGI86327[] = {{&S60439[30], 15}, {NULL}};
const Segment SGI86328[] = {{&S60439[130], 3}, {NULL}};
const Segment SGI86329[] = {{&S60439[96], 5}, {NULL}};
const Segment SGI86330[] = {{&S60440[52], 10}, {NULL}};
const Segment SGI86331[] = {{&S60440[30], 15}, {NULL}};
const Segment SGI86332[] = {{&S60440[128], 3}, {NULL}};
const Segment SGI86333[] = {{&S60440[94], 5}, {NULL}};
const Segment SGI86334[] = {{&S47694[34], 7}, {NULL}};
const Segment SGI86335[] = {{&S47694[27], 6}, {NULL}};
const Segment SGI86336[] = {{&S47694[99], 9}, {NULL}};
const Segment SGI86337[] = {{&S47694[72], 6}, {NULL}};
const Segment SGI86338[] = {{&S47696[34], 8}, {NULL}};
const Segment SGI86339[] = {{&S47696[27], 6}, {NULL}};
const Segment SGI86340[] = {{&S47696[101], 9}, {NULL}};
const Segment SGI86341[] = {{&S47696[74], 6}, {NULL}};
const Segment SGI86342[] = {{&S47697[34], 6}, {NULL}};
const Segment SGI86343[] = {{&S47697[27], 6}, {NULL}};
const Segment SGI86344[] = {{&S47697[101], 9}, {NULL}};
const Segment SGI86345[] = {{&S47697[74], 6}, {NULL}};
const Segment SGI86346[] = {{&S47741[34], 10}, {NULL}};
const Segment SGI86347[] = {{&S47741[27], 6}, {NULL}};
const Segment SGI86348[] = {{&S47741[97], 9}, {NULL}};
const Segment SGI86349[] = {{&S47741[70], 6}, {NULL}};
const Segment SGI86350[] = {{&S68261[31], 8}, {NULL}};
const Segment SGI86351[] = {{&S68261[54], 3}, {NULL}};
const Segment SGI86352[] = {{&S68262[39], 8}, {NULL}};
const Segment SGI86353[] = {{&S68262[103], 3}, {NULL}};
const Segment SGI86354[] = {{&S68262[92], 3}, {NULL}};
const Segment SGI86355[] = {{&S68262[69], 6}, {NULL}};
const Segment SGI86356[] = {{&S68307[45], 10}, {NULL}};
const Segment SGI86357[] = {{&S68307[30], 6}, {NULL}};
const Segment SGI86358[] = {{&S68307[120], 3}, {NULL}};
const Segment SGI86359[] = {{&S68307[86], 5}, {NULL}};
const Segment SGI86360[] = {{&S68307[144], 13}, {NULL}};
const Segment SGI86361[] = {{&S68308[44], 10}, {NULL}};
const Segment SGI86362[] = {{&S68308[44], 27}, {NULL}};
const Segment SGI86363[] = {{&S68308[30], 6}, {NULL}};
const Segment SGI86364[] = {{&S68308[120], 3}, {NULL}};
const Segment SGI86365[] = {{&S68308[85], 6}, {NULL}};
const Segment SGI86366[] = {{&S68308[140], 8}, {NULL}};
const Segment SGI86367[] = {{&S68308[149], 13}, {NULL}};
const Segment SGI86368[] = {{&S47822[34], 10}, {NULL}};
const Segment SGI86369[] = {{&S47822[27], 6}, {NULL}};
const Segment SGI86370[] = {{&S47822[97], 9}, {NULL}};
const Segment SGI86371[] = {{&S47822[70], 6}, {NULL}};
const Segment SGI86372[] = {{&S47823[34], 18}, {NULL}};
const Segment SGI86373[] = {{&S47823[27], 6}, {NULL}};
const Segment SGI86374[] = {{&S47823[105], 9}, {NULL}};
const Segment SGI86375[] = {{&S47823[78], 6}, {NULL}};
const Segment SGI86376[] = {{&S68322[44], 11}, {NULL}};
const Segment SGI86377[] = {{&S68322[44], 23}, {NULL}};
const Segment SGI86378[] = {{&S68322[30], 6}, {NULL}};
const Segment SGI86379[] = {{&S68322[115], 3}, {NULL}};
const Segment SGI86380[] = {{&S68322[81], 5}, {NULL}};
const Segment SGI86381[] = {{&S68323[44], 23}, {NULL}};
const Segment SGI86382[] = {{&S68323[30], 6}, {NULL}};
const Segment SGI86383[] = {{&S68324[44], 11}, {NULL}};
const Segment SGI86384[] = {{&S68324[44], 23}, {NULL}};
const Segment SGI86385[] = {{&S68324[30], 6}, {NULL}};
const Segment SGI86386[] = {{&S68324[84], 6}, {NULL}};
const Segment SGI86387[] = {{&S68325[44], 11}, {NULL}};
const Segment SGI86388[] = {{&S68325[44], 23}, {NULL}};
const Segment SGI86389[] = {{&S68325[30], 6}, {NULL}};
const Segment SGI86390[] = {{&S68325[115], 3}, {NULL}};
const Segment SGI86391[] = {{&S47831[44], 11}, {NULL}};
const Segment SGI86392[] = {{&S47831[44], 23}, {NULL}};
const Segment SGI86393[] = {{&S47831[30], 6}, {NULL}};
const Segment SGI86394[] = {{&S68325[81], 5}, {NULL}};
const Segment SGI86395[] = {{&S47831[31], 3}, {NULL}};
const Segment SGI86396[] = {{&S47831[81], 6}, {NULL}};
const Segment SGI86397[] = {{&S68326[38], 11}, {NULL}};
const Segment SGI86398[] = {{&S68326[31], 6}, {NULL}};
const Segment SGI86399[] = {{&S47832[38], 11}, {NULL}};
const Segment SGI86400[] = {{&S47832[31], 6}, {NULL}};
const Segment SGI86401[] = {{&S68329[45], 10}, {NULL}};
const Segment SGI86402[] = {{&S68329[30], 6}, {NULL}};
const Segment SGI86403[] = {{&S68329[120], 3}, {NULL}};
const Segment SGI86404[] = {{&S68329[86], 5}, {NULL}};
const Segment SGI86405[] = {{&S68329[144], 13}, {NULL}};
const Segment SGI86406[] = {{&S47844[34], 19}, {NULL}};
const Segment SGI86407[] = {{&S47844[27], 6}, {NULL}};
const Segment SGI86408[] = {{&S47844[106], 9}, {NULL}};
const Segment SGI86409[] = {{&S47844[79], 6}, {NULL}};
const Segment SGI86410[] = {{&S47845[34], 7}, {NULL}};
const Segment SGI86411[] = {{&S47845[27], 6}, {NULL}};
const Segment SGI86412[] = {{&S47845[94], 9}, {NULL}};
const Segment SGI86413[] = {{&S47845[67], 6}, {NULL}};
const Segment SGI86414[] = {{&S60549[44], 12}, {NULL}};
const Segment SGI86415[] = {{&S60549[44], 23}, {NULL}};
const Segment SGI86416[] = {{&S60549[30], 6}, {NULL}};
const Segment SGI86417[] = {{&S60549[134], 3}, {NULL}};
const Segment SGI86418[] = {{&S60549[100], 5}, {NULL}};
const Segment SGI86419[] = {{&S60052[44], 10}, {NULL}};
const Segment SGI86420[] = {{&S60052[44], 21}, {NULL}};
const Segment SGI86421[] = {{&S60549[35], 1}, {NULL}};
const Segment SGI86422[] = {{&S60052[30], 6}, {NULL}};
const Segment SGI86423[] = {{&S60549[69], 11}, {NULL}};
const Segment SGI86424[] = {{&S60052[114], 3}, {NULL}};
const Segment SGI86425[] = {{&S60052[79], 6}, {NULL}};
const Segment SGI86426[] = {{&S60550[44], 11}, {NULL}};
const Segment SGI86427[] = {{&S60550[44], 22}, {NULL}};
const Segment SGI86428[] = {{&S60550[30], 6}, {NULL}};
const Segment SGI86429[] = {{&S60550[133], 3}, {NULL}};
const Segment SGI86430[] = {{&S60053[38], 10}, {NULL}};
const Segment SGI86431[] = {{&S60053[31], 6}, {NULL}};
const Segment SGI86432[] = {{&S60550[99], 5}, {NULL}};
const Segment SGI86433[] = {{&S60550[35], 1}, {NULL}};
const Segment SGI86434[] = {{&S60550[68], 11}, {NULL}};
const Segment SGI86435[] = {{&S60054[42], 10}, {NULL}};
const Segment SGI86436[] = {{&S60054[30], 6}, {NULL}};
const Segment SGI86437[] = {{&S60551[44], 11}, {NULL}};
const Segment SGI86438[] = {{&S60551[44], 28}, {NULL}};
const Segment SGI86439[] = {{&S60551[30], 6}, {NULL}};
const Segment SGI86440[] = {{&S60054[112], 3}, {NULL}};
const Segment SGI86441[] = {{&S60054[77], 6}, {NULL}};
const Segment SGI86442[] = {{&S60551[141], 3}, {NULL}};
const Segment SGI86443[] = {{&S60551[107], 5}, {NULL}};
const Segment SGI86444[] = {{&S60551[74], 3}, {NULL}};
const Segment SGI86445[] = {{&S60055[42], 10}, {NULL}};
const Segment SGI86446[] = {{&S60055[42], 21}, {NULL}};
const Segment SGI86447[] = {{&S60055[30], 4}, {NULL}};
const Segment SGI86448[] = {{&S60055[112], 3}, {NULL}};
const Segment SGI86449[] = {{&S60552[68], 7}, {NULL}};
const Segment SGI86450[] = {{&S60552[61], 6}, {&S60552[68], 19}, {NULL}};
const Segment SGI86451[] = {{&S60552[30], 30}, {NULL}};
const Segment SGI86452[] = {{&S60552[154], 3}, {NULL}};
const Segment SGI86453[] = {{&S60552[120], 5}, {NULL}};
const Segment SGI86454[] = {{&S60552[35], 1}, {NULL}};
const Segment SGI86455[] = {{&S60552[89], 11}, {NULL}};
const Segment SGI86456[] = {{&S60056[42], 10}, {NULL}};
const Segment SGI86457[] = {{&S60056[42], 21}, {NULL}};
const Segment SGI86458[] = {{&S60056[30], 4}, {NULL}};
const Segment SGI86459[] = {{&S60056[112], 3}, {NULL}};
const Segment SGI86460[] = {{&S60056[77], 6}, {NULL}};
const Segment SGI86461[] = {{&S60554[44], 11}, {NULL}};
const Segment SGI86462[] = {{&S60554[44], 22}, {NULL}};
const Segment SGI86463[] = {{&S60554[30], 6}, {NULL}};
const Segment SGI86464[] = {{&S60554[133], 3}, {NULL}};
const Segment SGI86465[] = {{&S60554[99], 5}, {NULL}};
const Segment SGI86466[] = {{&S60554[35], 1}, {NULL}};
const Segment SGI86467[] = {{&S60554[68], 11}, {NULL}};
const Segment SGI86468[] = {{&S60555[44], 10}, {NULL}};
const Segment SGI86469[] = {{&S60555[44], 21}, {NULL}};
const Segment SGI86470[] = {{&S60555[30], 6}, {NULL}};
const Segment SGI86471[] = {{&S60555[128], 3}, {NULL}};
const Segment SGI86472[] = {{&S60555[94], 5}, {NULL}};
const Segment SGI86473[] = {{&S60555[35], 1}, {NULL}};
const Segment SGI86474[] = {{&S60555[67], 11}, {NULL}};
const Segment SGI86475[] = {{&S60556[44], 7}, {NULL}};
const Segment SGI86476[] = {{&S60556[44], 31}, {NULL}};
const Segment SGI86477[] = {{&S60556[30], 6}, {NULL}};
const Segment SGI86478[] = {{&S60556[123], 3}, {NULL}};
const Segment SGI86479[] = {{&S60556[89], 5}, {NULL}};
const Segment SGI86480[] = {{&S60556[61], 3}, {NULL}};
const Segment SGI86481[] = {{&S60557[44], 6}, {NULL}};
const Segment SGI86482[] = {{&S60557[44], 23}, {NULL}};
const Segment SGI86483[] = {{&S60557[30], 6}, {NULL}};
const Segment SGI86484[] = {{&S60557[127], 3}, {NULL}};
const Segment SGI86485[] = {{&S60557[93], 5}, {NULL}};
const Segment SGI86486[] = {{&S60557[69], 3}, {NULL}};
const Segment SGI86487[] = {{&S60558[44], 10}, {NULL}};
const Segment SGI86488[] = {{&S60558[44], 21}, {NULL}};
const Segment SGI86489[] = {{&S60558[30], 6}, {NULL}};
const Segment SGI86490[] = {{&S60558[132], 3}, {NULL}};
const Segment SGI86491[] = {{&S60558[98], 5}, {NULL}};
const Segment SGI86492[] = {{&S60558[35], 1}, {NULL}};
const Segment SGI86493[] = {{&S60558[67], 11}, {NULL}};
const Segment SGI86494[] = {{&S60559[68], 7}, {NULL}};
const Segment SGI86495[] = {{&S60559[61], 6}, {&S60559[68], 19}, {NULL}};
const Segment SGI86496[] = {{&S60559[30], 30}, {NULL}};
const Segment SGI86497[] = {{&S60559[154], 3}, {NULL}};
const Segment SGI86498[] = {{&S60559[120], 5}, {NULL}};
const Segment SGI86499[] = {{&S60559[35], 1}, {NULL}};
const Segment SGI86500[] = {{&S60559[89], 11}, {NULL}};
const Segment SGI86501[] = {{&S60560[44], 7}, {NULL}};
const Segment SGI86502[] = {{&S60560[44], 19}, {NULL}};
const Segment SGI86503[] = {{&S60560[30], 6}, {NULL}};
const Segment SGI86504[] = {{&S60560[126], 3}, {NULL}};
const Segment SGI86505[] = {{&S60560[92], 5}, {NULL}};
const Segment SGI86506[] = {{&S60560[35], 1}, {NULL}};
const Segment SGI86507[] = {{&S60560[65], 11}, {NULL}};
const Segment SGI86508[] = {{&S47889[34], 7}, {NULL}};
const Segment SGI86509[] = {{&S47889[27], 6}, {NULL}};
const Segment SGI86510[] = {{&S60561[44], 6}, {NULL}};
const Segment SGI86511[] = {{&S60561[44], 23}, {NULL}};
const Segment SGI86512[] = {{&S47889[93], 9}, {NULL}};
const Segment SGI86513[] = {{&S60561[30], 6}, {NULL}};
const Segment SGI86514[] = {{&S47889[66], 6}, {NULL}};
const Segment SGI86515[] = {{&S60561[130], 3}, {NULL}};
const Segment SGI86516[] = {{&S60561[96], 5}, {NULL}};
const Segment SGI86517[] = {{&S60561[35], 1}, {NULL}};
const Segment SGI86518[] = {{&S60561[69], 11}, {NULL}};
const Segment SGI86519[] = {{&S60562[44], 16}, {NULL}};
const Segment SGI86520[] = {{&S60562[44], 27}, {NULL}};
const Segment SGI86521[] = {{&S60562[30], 6}, {NULL}};
const Segment SGI86522[] = {{&S47891[34], 6}, {NULL}};
const Segment SGI86523[] = {{&S47891[27], 6}, {NULL}};
const Segment SGI86524[] = {{&S60562[138], 3}, {NULL}};
const Segment SGI86525[] = {{&S60562[104], 5}, {NULL}};
const Segment SGI86526[] = {{&S60562[35], 1}, {NULL}};
const Segment SGI86527[] = {{&S47891[92], 9}, {NULL}};
const Segment SGI86528[] = {{&S47891[65], 6}, {NULL}};
const Segment SGI86529[] = {{&S60562[73], 11}, {NULL}};
const Segment SGI86530[] = {{&S60563[63], 12}, {NULL}};
const Segment SGI86531[] = {{&S60563[63], 23}, {NULL}};
const Segment SGI86532[] = {{&S60563[30], 25}, {NULL}};
const Segment SGI86533[] = {{&S60563[153], 3}, {NULL}};
const Segment SGI86534[] = {{&S60563[119], 5}, {NULL}};
const Segment SGI86535[] = {{&S60563[35], 1}, {NULL}};
const Segment SGI86536[] = {{&S60563[88], 11}, {NULL}};
const Segment SGI86537[] = {{&S60564[44], 12}, {NULL}};
const Segment SGI86538[] = {{&S60564[44], 24}, {NULL}};
const Segment SGI86539[] = {{&S60564[30], 6}, {NULL}};
const Segment SGI86540[] = {{&S60564[131], 3}, {NULL}};
const Segment SGI86541[] = {{&S60564[97], 5}, {NULL}};
const Segment SGI86542[] = {{&S60564[35], 1}, {NULL}};
const Segment SGI86543[] = {{&S60564[70], 11}, {NULL}};
const Segment SGI86544[] = {{&S60565[44], 17}, {NULL}};
const Segment SGI86545[] = {{&S60565[44], 28}, {NULL}};
const Segment SGI86546[] = {{&S60565[30], 6}, {NULL}};
const Segment SGI86547[] = {{&S60565[145], 3}, {NULL}};
const Segment SGI86548[] = {{&S60565[111], 5}, {NULL}};
const Segment SGI86549[] = {{&S60565[86], 11}, {NULL}};
const Segment SGI86550[] = {{&S60566[45], 12}, {NULL}};
const Segment SGI86551[] = {{&S60566[45], 24}, {NULL}};
const Segment SGI86552[] = {{&S60566[30], 7}, {NULL}};
const Segment SGI86553[] = {{&S60566[134], 3}, {NULL}};
const Segment SGI86554[] = {{&S60566[100], 5}, {NULL}};
const Segment SGI86555[] = {{&S60566[35], 1}, {NULL}};
const Segment SGI86556[] = {{&S60566[71], 11}, {NULL}};
const Segment SGI86557[] = {{&S60567[44], 10}, {&S60567[81], 9}, {NULL}};
const Segment SGI86558[] = {{&S60567[44], 21}, {NULL}};
const Segment SGI86559[] = {{&S60567[30], 6}, {NULL}};
const Segment SGI86560[] = {{&S60567[149], 3}, {NULL}};
const Segment SGI86561[] = {{&S60567[115], 5}, {NULL}};
const Segment SGI86562[] = {{&S60567[35], 1}, {NULL}};
const Segment SGI86563[] = {{&S60567[67], 11}, {NULL}};
const Segment SGI86564[] = {{&S60568[44], 6}, {NULL}};
const Segment SGI86565[] = {{&S60568[44], 27}, {NULL}};
const Segment SGI86566[] = {{&S60568[30], 6}, {NULL}};
const Segment SGI86567[] = {{&S60568[133], 3}, {NULL}};
const Segment SGI86568[] = {{&S60568[99], 5}, {NULL}};
const Segment SGI86569[] = {{&S60568[65], 3}, {&S60568[65], 3}, {NULL}};
const Segment SGI86570[] = {{&S60591[44], 10}, {NULL}};
const Segment SGI86571[] = {{&S60591[44], 20}, {NULL}};
const Segment SGI86572[] = {{&S60591[30], 6}, {NULL}};
const Segment SGI86573[] = {{&S60591[112], 3}, {NULL}};
const Segment SGI86574[] = {{&S60591[78], 5}, {NULL}};
const Segment SGI86575[] = {{&S60593[44], 10}, {NULL}};
const Segment SGI86576[] = {{&S60593[44], 22}, {NULL}};
const Segment SGI86577[] = {{&S60593[30], 6}, {NULL}};
const Segment SGI86578[] = {{&S60593[114], 3}, {NULL}};
const Segment SGI86579[] = {{&S60593[80], 5}, {NULL}};
const Segment SGI86580[] = {{&S60594[38], 10}, {NULL}};
const Segment SGI86581[] = {{&S60594[31], 6}, {NULL}};
const Segment SGI86582[] = {{&S60596[44], 10}, {NULL}};
const Segment SGI86583[] = {{&S60596[44], 20}, {NULL}};
const Segment SGI86584[] = {{&S60596[30], 6}, {NULL}};
const Segment SGI86585[] = {{&S60596[81], 4}, {NULL}};
const Segment SGI86586[] = {{&S60596[81], 3}, {NULL}};
const Segment SGI86587[] = {{&S60598[44], 10}, {NULL}};
const Segment SGI86588[] = {{&S60598[44], 20}, {NULL}};
const Segment SGI86589[] = {{&S60598[30], 6}, {NULL}};
const Segment SGI86590[] = {{&S60598[81], 6}, {NULL}};
const Segment SGI86591[] = {{&S47972[34], 9}, {NULL}};
const Segment SGI86592[] = {{&S47972[27], 6}, {NULL}};
const Segment SGI86593[] = {{&S47972[96], 9}, {NULL}};
const Segment SGI86594[] = {{&S47972[69], 6}, {NULL}};
const Segment SGI86595[] = {{&S47973[34], 10}, {NULL}};
const Segment SGI86596[] = {{&S47973[27], 6}, {NULL}};
const Segment SGI86597[] = {{&S47973[97], 9}, {NULL}};
const Segment SGI86598[] = {{&S47973[70], 6}, {NULL}};
const Segment SGI86599[] = {{&S60660[44], 13}, {NULL}};
const Segment SGI86600[] = {{&S60660[44], 24}, {NULL}};
const Segment SGI86601[] = {{&S60660[30], 6}, {NULL}};
const Segment SGI86602[] = {{&S60660[116], 3}, {NULL}};
const Segment SGI86603[] = {{&S60660[82], 5}, {NULL}};
const Segment SGI86604[] = {{&S60661[44], 10}, {NULL}};
const Segment SGI86605[] = {{&S60661[44], 21}, {NULL}};
const Segment SGI86606[] = {{&S60661[30], 6}, {NULL}};
const Segment SGI86607[] = {{&S60661[131], 3}, {NULL}};
const Segment SGI86608[] = {{&S60661[97], 5}, {NULL}};
const Segment SGI86609[] = {{&S60662[38], 10}, {NULL}};
const Segment SGI86610[] = {{&S60662[31], 6}, {NULL}};
const Segment SGI86611[] = {{&S60663[38], 13}, {NULL}};
const Segment SGI86612[] = {{&S60663[31], 6}, {NULL}};
const Segment SGI86613[] = {{&S48031[34], 19}, {NULL}};
const Segment SGI86614[] = {{&S48031[27], 6}, {NULL}};
const Segment SGI86615[] = {{&S48031[106], 9}, {NULL}};
const Segment SGI86616[] = {{&S48031[79], 6}, {NULL}};
const Segment SGI86617[] = {{&S48033[34], 7}, {NULL}};
const Segment SGI86618[] = {{&S48033[27], 6}, {NULL}};
const Segment SGI86619[] = {{&S48033[94], 9}, {NULL}};
const Segment SGI86620[] = {{&S48033[67], 6}, {NULL}};
const Segment SGI86621[] = {{&S60227[44], 7}, {NULL}};
const Segment SGI86622[] = {{&S60227[44], 18}, {NULL}};
const Segment SGI86623[] = {{&S60227[30], 6}, {NULL}};
const Segment SGI86624[] = {{&S60227[111], 3}, {NULL}};
const Segment SGI86625[] = {{&S60227[76], 6}, {NULL}};
const Segment SGI86626[] = {{&S48051[44], 9}, {NULL}};
const Segment SGI86627[] = {{&S48051[44], 21}, {NULL}};
const Segment SGI86628[] = {{&S48051[30], 6}, {NULL}};
const Segment SGI86629[] = {{&S48051[31], 3}, {NULL}};
const Segment SGI86630[] = {{&S48051[79], 6}, {NULL}};
const Segment SGI86631[] = {{&S48052[38], 9}, {NULL}};
const Segment SGI86632[] = {{&S48052[31], 6}, {NULL}};
const Segment SGI86633[] = {{&S68622[44], 15}, {NULL}};
const Segment SGI86634[] = {{&S68622[44], 27}, {NULL}};
const Segment SGI86635[] = {{&S68622[30], 6}, {NULL}};
const Segment SGI86636[] = {{&S68622[119], 3}, {NULL}};
const Segment SGI86637[] = {{&S68622[85], 5}, {NULL}};
const Segment SGI86638[] = {{&S68623[44], 10}, {NULL}};
const Segment SGI86639[] = {{&S68623[44], 22}, {NULL}};
const Segment SGI86640[] = {{&S68623[30], 6}, {NULL}};
const Segment SGI86641[] = {{&S68623[114], 3}, {NULL}};
const Segment SGI86642[] = {{&S68623[80], 5}, {NULL}};
const Segment SGI86643[] = {{&S60262[44], 18}, {NULL}};
const Segment SGI86644[] = {{&S60262[44], 30}, {NULL}};
const Segment SGI86645[] = {{&S60262[30], 6}, {NULL}};
const Segment SGI86646[] = {{&S60262[123], 3}, {NULL}};
const Segment SGI86647[] = {{&S60262[88], 6}, {NULL}};
const Segment SGI86648[] = {{&S60730[44], 10}, {NULL}};
const Segment SGI86649[] = {{&S60730[44], 27}, {NULL}};
const Segment SGI86650[] = {{&S60730[30], 6}, {NULL}};
const Segment SGI86651[] = {{&S60730[138], 3}, {NULL}};
const Segment SGI86652[] = {{&S60730[104], 5}, {NULL}};
const Segment SGI86653[] = {{&S60730[35], 1}, {NULL}};
const Segment SGI86654[] = {{&S60730[73], 11}, {NULL}};
const Segment SGI86655[] = {{&S48093[44], 13}, {NULL}};
const Segment SGI86656[] = {{&S48093[44], 25}, {NULL}};
const Segment SGI86657[] = {{&S48093[30], 6}, {NULL}};
const Segment SGI86658[] = {{&S48093[31], 3}, {NULL}};
const Segment SGI86659[] = {{&S48093[83], 6}, {NULL}};
const Segment SGI86660[] = {{&S48095[44], 8}, {NULL}};
const Segment SGI86661[] = {{&S48095[44], 20}, {NULL}};
const Segment SGI86662[] = {{&S48095[30], 6}, {NULL}};
const Segment SGI86663[] = {{&S48095[31], 3}, {NULL}};
const Segment SGI86664[] = {{&S48095[78], 6}, {NULL}};
const Segment SGI86665[] = {{&S48098[44], 8}, {NULL}};
const Segment SGI86666[] = {{&S48098[44], 20}, {NULL}};
const Segment SGI86667[] = {{&S48098[30], 6}, {NULL}};
const Segment SGI86668[] = {{&S48098[81], 4}, {NULL}};
const Segment SGI86669[] = {{&S48098[81], 3}, {NULL}};
const Segment SGI86670[] = {{&S48101[44], 13}, {NULL}};
const Segment SGI86671[] = {{&S48101[44], 25}, {NULL}};
const Segment SGI86672[] = {{&S48101[30], 6}, {NULL}};
const Segment SGI86673[] = {{&S48101[86], 4}, {NULL}};
const Segment SGI86674[] = {{&S48101[86], 3}, {NULL}};
const Segment SGI86675[] = {{&S48104[44], 8}, {NULL}};
const Segment SGI86676[] = {{&S48104[44], 20}, {NULL}};
const Segment SGI86677[] = {{&S48104[30], 6}, {NULL}};
const Segment SGI86678[] = {{&S48104[81], 6}, {NULL}};
const Segment SGI86679[] = {{&S68655[44], 7}, {NULL}};
const Segment SGI86680[] = {{&S68655[44], 19}, {NULL}};
const Segment SGI86681[] = {{&S68655[30], 6}, {NULL}};
const Segment SGI86682[] = {{&S68655[111], 3}, {NULL}};
const Segment SGI86683[] = {{&S68655[77], 5}, {NULL}};
const Segment SGI86684[] = {{&S68656[44], 7}, {NULL}};
const Segment SGI86685[] = {{&S68656[44], 19}, {NULL}};
const Segment SGI86686[] = {{&S68656[30], 6}, {NULL}};
const Segment SGI86687[] = {{&S68656[80], 4}, {NULL}};
const Segment SGI86688[] = {{&S68656[80], 3}, {NULL}};
const Segment SGI86689[] = {{&S68658[44], 7}, {NULL}};
const Segment SGI86690[] = {{&S68658[44], 19}, {NULL}};
const Segment SGI86691[] = {{&S68658[30], 6}, {NULL}};
const Segment SGI86692[] = {{&S68658[80], 6}, {NULL}};
const Segment SGI86693[] = {{&S68660[44], 19}, {NULL}};
const Segment SGI86694[] = {{&S68660[30], 6}, {NULL}};
const Segment SGI86695[] = {{&S68661[44], 9}, {NULL}};
const Segment SGI86696[] = {{&S68661[44], 21}, {NULL}};
const Segment SGI86697[] = {{&S68661[30], 6}, {NULL}};
const Segment SGI86698[] = {{&S68661[113], 3}, {NULL}};
const Segment SGI86699[] = {{&S68661[79], 5}, {NULL}};
const Segment SGI86700[] = {{&S48137[34], 13}, {NULL}};
const Segment SGI86701[] = {{&S48137[27], 6}, {NULL}};
const Segment SGI86702[] = {{&S48137[100], 9}, {NULL}};
const Segment SGI86703[] = {{&S48137[73], 6}, {NULL}};
const Segment SGI86704[] = {{&S48138[34], 12}, {NULL}};
const Segment SGI86705[] = {{&S48138[27], 6}, {NULL}};
const Segment SGI86706[] = {{&S48138[99], 9}, {NULL}};
const Segment SGI86707[] = {{&S48138[72], 6}, {NULL}};
const Segment SGI86708[] = {{&S68701[44], 7}, {NULL}};
const Segment SGI86709[] = {{&S68701[44], 19}, {NULL}};
const Segment SGI86710[] = {{&S68701[30], 6}, {NULL}};
const Segment SGI86711[] = {{&S68701[111], 3}, {NULL}};
const Segment SGI86712[] = {{&S68701[77], 5}, {NULL}};
const Segment SGI86713[] = {{&S48161[34], 10}, {NULL}};
const Segment SGI86714[] = {{&S48161[27], 6}, {NULL}};
const Segment SGI86715[] = {{&S48161[97], 9}, {NULL}};
const Segment SGI86716[] = {{&S48161[70], 6}, {NULL}};
const Segment SGI86717[] = {{&S48163[34], 11}, {NULL}};
const Segment SGI86718[] = {{&S48163[27], 6}, {NULL}};
const Segment SGI86719[] = {{&S48163[98], 9}, {NULL}};
const Segment SGI86720[] = {{&S48163[71], 6}, {NULL}};
const Segment SGI86721[] = {{&S48164[44], 15}, {NULL}};
const Segment SGI86722[] = {{&S48164[44], 27}, {NULL}};
const Segment SGI86723[] = {{&S48164[30], 6}, {NULL}};
const Segment SGI86724[] = {{&S48164[31], 3}, {NULL}};
const Segment SGI86725[] = {{&S48164[85], 5}, {NULL}};
const Segment SGI86726[] = {{&S68716[44], 10}, {NULL}};
const Segment SGI86727[] = {{&S68716[44], 22}, {NULL}};
const Segment SGI86728[] = {{&S68716[30], 6}, {NULL}};
const Segment SGI86729[] = {{&S68716[114], 3}, {NULL}};
const Segment SGI86730[] = {{&S68716[80], 5}, {NULL}};
const Segment SGI86731[] = {{&S68721[44], 10}, {NULL}};
const Segment SGI86732[] = {{&S68721[44], 27}, {NULL}};
const Segment SGI86733[] = {{&S68721[30], 6}, {NULL}};
const Segment SGI86734[] = {{&S68721[85], 6}, {NULL}};
const Segment SGI86735[] = {{&S68721[112], 13}, {NULL}};
const Segment SGI86736[] = {{&S68735[44], 5}, {NULL}};
const Segment SGI86737[] = {{&S68735[44], 26}, {NULL}};
const Segment SGI86738[] = {{&S68735[30], 6}, {NULL}};
const Segment SGI86739[] = {{&S68735[118], 3}, {NULL}};
const Segment SGI86740[] = {{&S68735[84], 5}, {NULL}};
const Segment SGI86741[] = {{&S48225[34], 11}, {NULL}};
const Segment SGI86742[] = {{&S48225[27], 6}, {NULL}};
const Segment SGI86743[] = {{&S48225[98], 9}, {NULL}};
const Segment SGI86744[] = {{&S48225[71], 6}, {NULL}};
const Segment SGI86745[] = {{&S60466[44], 9}, {NULL}};
const Segment SGI86746[] = {{&S60466[44], 20}, {NULL}};
const Segment SGI86747[] = {{&S60466[30], 6}, {NULL}};
const Segment SGI86748[] = {{&S60466[113], 3}, {NULL}};
const Segment SGI86749[] = {{&S60466[78], 6}, {NULL}};
const Segment SGI86750[] = {{&S60467[44], 14}, {NULL}};
const Segment SGI86751[] = {{&S60467[44], 25}, {NULL}};
const Segment SGI86752[] = {{&S60467[30], 6}, {NULL}};
const Segment SGI86753[] = {{&S60467[118], 3}, {NULL}};
const Segment SGI86754[] = {{&S60467[83], 6}, {NULL}};
const Segment SGI86755[] = {{&S60468[44], 9}, {NULL}};
const Segment SGI86756[] = {{&S60468[44], 20}, {NULL}};
const Segment SGI86757[] = {{&S60468[30], 6}, {NULL}};
const Segment SGI86758[] = {{&S60468[113], 3}, {NULL}};
const Segment SGI86759[] = {{&S60468[78], 6}, {NULL}};
const Segment SGI86760[] = {{&S60469[38], 9}, {NULL}};
const Segment SGI86761[] = {{&S60469[31], 6}, {NULL}};
const Segment SGI86762[] = {{&S60475[42], 13}, {NULL}};
const Segment SGI86763[] = {{&S60475[42], 24}, {NULL}};
const Segment SGI86764[] = {{&S60475[30], 4}, {NULL}};
const Segment SGI86765[] = {{&S60475[115], 3}, {NULL}};
const Segment SGI86766[] = {{&S60475[80], 6}, {NULL}};
const Segment SGI86767[] = {{&S60476[42], 23}, {NULL}};
const Segment SGI86768[] = {{&S60476[42], 34}, {NULL}};
const Segment SGI86769[] = {{&S60476[30], 4}, {NULL}};
const Segment SGI86770[] = {{&S60476[125], 3}, {NULL}};
const Segment SGI86771[] = {{&S60476[90], 6}, {NULL}};
const Segment SGI86772[] = {{&S60477[42], 23}, {NULL}};
const Segment SGI86773[] = {{&S60477[42], 34}, {NULL}};
const Segment SGI86774[] = {{&S60477[30], 4}, {NULL}};
const Segment SGI86775[] = {{&S60477[125], 3}, {NULL}};
const Segment SGI86776[] = {{&S60477[90], 6}, {NULL}};
const Segment SGI86777[] = {{&S60478[42], 11}, {NULL}};
const Segment SGI86778[] = {{&S60478[42], 22}, {NULL}};
const Segment SGI86779[] = {{&S60478[30], 4}, {NULL}};
const Segment SGI86780[] = {{&S60478[113], 3}, {NULL}};
const Segment SGI86781[] = {{&S60478[78], 6}, {NULL}};
const Segment SGI86782[] = {{&S60479[42], 10}, {NULL}};
const Segment SGI86783[] = {{&S60479[42], 21}, {NULL}};
const Segment SGI86784[] = {{&S60479[30], 4}, {NULL}};
const Segment SGI86785[] = {{&S60479[112], 3}, {NULL}};
const Segment SGI86786[] = {{&S60479[77], 6}, {NULL}};
const Segment SGI86787[] = {{&S60480[36], 11}, {NULL}};
const Segment SGI86788[] = {{&S60480[31], 4}, {NULL}};
const Segment SGI86789[] = {{&S60481[44], 10}, {NULL}};
const Segment SGI86790[] = {{&S60481[32], 4}, {NULL}};
const Segment SGI86791[] = {{&S60482[36], 13}, {NULL}};
const Segment SGI86792[] = {{&S60482[31], 4}, {NULL}};
const Segment SGI86793[] = {{&S48304[34], 11}, {NULL}};
const Segment SGI86794[] = {{&S48304[27], 6}, {NULL}};
const Segment SGI86795[] = {{&S48304[98], 9}, {NULL}};
const Segment SGI86796[] = {{&S48304[71], 6}, {NULL}};
const Segment SGI86797[] = {{&S48306[34], 11}, {NULL}};
const Segment SGI86798[] = {{&S48306[27], 6}, {NULL}};
const Segment SGI86799[] = {{&S48306[98], 9}, {NULL}};
const Segment SGI86800[] = {{&S48306[71], 6}, {NULL}};
const Segment SGI86801[] = {{&S68875[44], 13}, {NULL}};
const Segment SGI86802[] = {{&S68875[44], 25}, {NULL}};
const Segment SGI86803[] = {{&S68875[30], 6}, {NULL}};
const Segment SGI86804[] = {{&S68875[117], 3}, {NULL}};
const Segment SGI86805[] = {{&S68875[83], 5}, {NULL}};
const Segment SGI86806[] = {{&S48359[34], 12}, {NULL}};
const Segment SGI86807[] = {{&S48359[27], 6}, {NULL}};
const Segment SGI86808[] = {{&S48359[99], 9}, {NULL}};
const Segment SGI86809[] = {{&S48359[72], 6}, {NULL}};
const Segment SGI86810[] = {{&S48360[34], 11}, {NULL}};
const Segment SGI86811[] = {{&S48360[27], 6}, {NULL}};
const Segment SGI86812[] = {{&S48360[98], 9}, {NULL}};
const Segment SGI86813[] = {{&S48360[71], 6}, {NULL}};
const Segment SGI86814[] = {{&S48363[34], 14}, {NULL}};
const Segment SGI86815[] = {{&S48363[27], 6}, {NULL}};
const Segment SGI86816[] = {{&S48363[101], 9}, {NULL}};
const Segment SGI86817[] = {{&S48363[74], 6}, {NULL}};
const Segment SGI86818[] = {{&S48364[34], 13}, {NULL}};
const Segment SGI86819[] = {{&S48364[27], 6}, {NULL}};
const Segment SGI86820[] = {{&S48364[100], 9}, {NULL}};
const Segment SGI86821[] = {{&S48364[73], 6}, {NULL}};
const Segment SGI86822[] = {{&S68887[45], 10}, {NULL}};
const Segment SGI86823[] = {{&S68887[30], 6}, {NULL}};
const Segment SGI86824[] = {{&S68887[120], 3}, {NULL}};
const Segment SGI86825[] = {{&S68887[86], 5}, {NULL}};
const Segment SGI86826[] = {{&S68887[144], 13}, {NULL}};
const Segment SGI86827[] = {{&S68891[45], 11}, {NULL}};
const Segment SGI86828[] = {{&S68891[30], 6}, {NULL}};
const Segment SGI86829[] = {{&S68891[121], 3}, {NULL}};
const Segment SGI86830[] = {{&S68891[87], 5}, {NULL}};
const Segment SGI86831[] = {{&S68891[145], 13}, {NULL}};
const Segment SGI86832[] = {{&S48553[34], 30}, {NULL}};
const Segment SGI86833[] = {{&S48553[27], 6}, {NULL}};
const Segment SGI86834[] = {{&S48553[117], 9}, {NULL}};
const Segment SGI86835[] = {{&S48553[90], 6}, {NULL}};
const Segment SGI86836[] = {{&S48554[34], 29}, {NULL}};
const Segment SGI86837[] = {{&S48554[27], 6}, {NULL}};
const Segment SGI86838[] = {{&S48554[116], 9}, {NULL}};
const Segment SGI86839[] = {{&S48554[89], 6}, {NULL}};
const Segment SGI86840[] = {{&S48555[34], 30}, {NULL}};
const Segment SGI86841[] = {{&S48555[27], 6}, {NULL}};
const Segment SGI86842[] = {{&S48555[117], 9}, {NULL}};
const Segment SGI86843[] = {{&S48555[90], 6}, {NULL}};
const Segment SGI86844[] = {{&S48558[34], 30}, {NULL}};
const Segment SGI86845[] = {{&S48558[27], 6}, {NULL}};
const Segment SGI86846[] = {{&S48558[117], 9}, {NULL}};
const Segment SGI86847[] = {{&S48558[90], 6}, {NULL}};
const Segment SGI86848[] = {{&S48559[34], 17}, {NULL}};
const Segment SGI86849[] = {{&S48559[27], 6}, {NULL}};
const Segment SGI86850[] = {{&S48559[104], 9}, {NULL}};
const Segment SGI86851[] = {{&S48559[77], 6}, {NULL}};
const Segment SGI86852[] = {{&S48576[34], 18}, {NULL}};
const Segment SGI86853[] = {{&S48576[27], 6}, {NULL}};
const Segment SGI86854[] = {{&S48576[105], 9}, {NULL}};
const Segment SGI86855[] = {{&S48576[78], 6}, {NULL}};
const Segment SGI86856[] = {{&S69112[45], 10}, {NULL}};
const Segment SGI86857[] = {{&S69112[30], 6}, {NULL}};
const Segment SGI86858[] = {{&S69112[120], 3}, {NULL}};
const Segment SGI86859[] = {{&S69112[86], 5}, {NULL}};
const Segment SGI86860[] = {{&S69112[144], 13}, {NULL}};
const Segment SGI86861[] = {{&S69113[45], 11}, {NULL}};
const Segment SGI86862[] = {{&S69113[30], 6}, {NULL}};
const Segment SGI86863[] = {{&S69113[121], 3}, {NULL}};
const Segment SGI86864[] = {{&S69113[87], 5}, {NULL}};
const Segment SGI86865[] = {{&S69113[145], 13}, {NULL}};
const Segment SGI86866[] = {{&S48636[34], 10}, {NULL}};
const Segment SGI86867[] = {{&S48636[27], 6}, {NULL}};
const Segment SGI86868[] = {{&S48636[97], 9}, {NULL}};
const Segment SGI86869[] = {{&S48636[70], 6}, {NULL}};
const Segment SGI86870[] = {{&S69115[44], 5}, {NULL}};
const Segment SGI86871[] = {{&S69115[44], 25}, {NULL}};
const Segment SGI86872[] = {{&S69115[30], 6}, {NULL}};
const Segment SGI86873[] = {{&S69115[117], 3}, {NULL}};
const Segment SGI86874[] = {{&S69115[83], 5}, {NULL}};
const Segment SGI86875[] = {{&S48637[44], 41}, {NULL}};
const Segment SGI86876[] = {{&S48637[30], 6}, {NULL}};
const Segment SGI86877[] = {{&S48637[117], 6}, {NULL}};
const Segment SGI86878[] = {{&S69116[38], 5}, {NULL}};
const Segment SGI86879[] = {{&S69116[31], 6}, {NULL}};
const Segment SGI86880[] = {{&S48638[44], 14}, {NULL}};
const Segment SGI86881[] = {{&S48638[44], 31}, {NULL}};
const Segment SGI86882[] = {{&S48638[30], 6}, {NULL}};
const Segment SGI86883[] = {{&S48638[92], 4}, {NULL}};
const Segment SGI86884[] = {{&S48638[92], 3}, {NULL}};
const Segment SGI86885[] = {{&S48639[44], 14}, {NULL}};
const Segment SGI86886[] = {{&S48639[44], 31}, {NULL}};
const Segment SGI86887[] = {{&S48639[30], 6}, {NULL}};
const Segment SGI86888[] = {{&S48639[31], 3}, {NULL}};
const Segment SGI86889[] = {{&S48639[89], 6}, {NULL}};
const Segment SGI86890[] = {{&S48640[44], 14}, {NULL}};
const Segment SGI86891[] = {{&S48640[44], 31}, {NULL}};
const Segment SGI86892[] = {{&S48640[30], 6}, {NULL}};
const Segment SGI86893[] = {{&S48640[92], 6}, {NULL}};
const Segment SGI86894[] = {{&S48641[38], 14}, {NULL}};
const Segment SGI86895[] = {{&S48641[31], 6}, {NULL}};
const Segment SGI86896[] = {{&S48642[44], 31}, {NULL}};
const Segment SGI86897[] = {{&S48642[30], 6}, {NULL}};
const Segment SGI86898[] = {{&S69182[44], 23}, {NULL}};
const Segment SGI86899[] = {{&S69182[44], 35}, {NULL}};
const Segment SGI86900[] = {{&S69182[30], 6}, {NULL}};
const Segment SGI86901[] = {{&S69182[127], 3}, {NULL}};
const Segment SGI86902[] = {{&S69182[93], 5}, {NULL}};
const Segment SGI86903[] = {{&S48712[34], 11}, {NULL}};
const Segment SGI86904[] = {{&S48712[27], 6}, {NULL}};
const Segment SGI86905[] = {{&S48712[98], 9}, {NULL}};
const Segment SGI86906[] = {{&S48712[71], 6}, {NULL}};
const Segment SGI86907[] = {{&S48715[34], 10}, {NULL}};
const Segment SGI86908[] = {{&S48715[27], 6}, {NULL}};
const Segment SGI86909[] = {{&S48715[97], 9}, {NULL}};
const Segment SGI86910[] = {{&S48715[70], 6}, {NULL}};
const Segment SGI86911[] = {{&S48717[34], 10}, {NULL}};
const Segment SGI86912[] = {{&S48717[27], 6}, {NULL}};
const Segment SGI86913[] = {{&S48717[97], 9}, {NULL}};
const Segment SGI86914[] = {{&S48717[70], 6}, {NULL}};
const Segment SGI86915[] = {{&S48729[34], 10}, {NULL}};
const Segment SGI86916[] = {{&S48729[27], 6}, {NULL}};
const Segment SGI86917[] = {{&S48729[97], 9}, {NULL}};
const Segment SGI86918[] = {{&S48729[70], 6}, {NULL}};
const Segment SGI86919[] = {{&S48738[34], 42}, {NULL}};
const Segment SGI86920[] = {{&S48738[27], 6}, {NULL}};
const Segment SGI86921[] = {{&S48738[129], 9}, {NULL}};
const Segment SGI86922[] = {{&S48738[102], 6}, {NULL}};
const Segment SGI86923[] = {{&S48739[34], 36}, {NULL}};
const Segment SGI86924[] = {{&S48739[27], 6}, {NULL}};
const Segment SGI86925[] = {{&S48739[123], 9}, {NULL}};
const Segment SGI86926[] = {{&S48739[96], 6}, {NULL}};
const Segment SGI86927[] = {{&S48740[34], 41}, {NULL}};
const Segment SGI86928[] = {{&S48740[27], 6}, {NULL}};
const Segment SGI86929[] = {{&S48740[128], 9}, {NULL}};
const Segment SGI86930[] = {{&S48740[101], 6}, {NULL}};
const Segment SGI86931[] = {{&S48741[34], 35}, {NULL}};
const Segment SGI86932[] = {{&S48741[27], 6}, {NULL}};
const Segment SGI86933[] = {{&S48741[122], 9}, {NULL}};
const Segment SGI86934[] = {{&S48741[95], 6}, {NULL}};
const Segment SGI86935[] = {{&S48742[34], 41}, {NULL}};
const Segment SGI86936[] = {{&S48742[27], 6}, {NULL}};
const Segment SGI86937[] = {{&S48742[128], 9}, {NULL}};
const Segment SGI86938[] = {{&S48742[101], 6}, {NULL}};
const Segment SGI86939[] = {{&S48743[34], 37}, {NULL}};
const Segment SGI86940[] = {{&S48743[27], 6}, {NULL}};
const Segment SGI86941[] = {{&S48743[124], 9}, {NULL}};
const Segment SGI86942[] = {{&S48743[97], 6}, {NULL}};
const Segment SGI86943[] = {{&S60925[42], 15}, {NULL}};
const Segment SGI86944[] = {{&S60925[42], 27}, {NULL}};
const Segment SGI86945[] = {{&S60925[30], 4}, {NULL}};
const Segment SGI86946[] = {{&S60925[118], 3}, {NULL}};
const Segment SGI86947[] = {{&S60925[83], 6}, {NULL}};
const Segment SGI86948[] = {{&S60926[36], 15}, {NULL}};
const Segment SGI86949[] = {{&S60926[31], 4}, {NULL}};
const Segment SGI86950[] = {{&S69278[44], 14}, {NULL}};
const Segment SGI86951[] = {{&S69278[44], 26}, {NULL}};
const Segment SGI86952[] = {{&S69278[30], 6}, {NULL}};
const Segment SGI86953[] = {{&S69278[118], 3}, {NULL}};
const Segment SGI86954[] = {{&S69278[84], 5}, {NULL}};
const Segment SGI86955[] = {{&S69313[31], 9}, {NULL}};
const Segment SGI86956[] = {{&S69313[55], 3}, {NULL}};
const Segment SGI86957[] = {{&S69314[39], 9}, {NULL}};
const Segment SGI86958[] = {{&S69314[104], 3}, {NULL}};
const Segment SGI86959[] = {{&S69314[93], 3}, {NULL}};
const Segment SGI86960[] = {{&S69314[70], 6}, {NULL}};
const Segment SGI86961[] = {{&S69331[45], 10}, {NULL}};
const Segment SGI86962[] = {{&S69331[30], 6}, {NULL}};
const Segment SGI86963[] = {{&S69331[120], 3}, {NULL}};
const Segment SGI86964[] = {{&S69331[86], 5}, {NULL}};
const Segment SGI86965[] = {{&S69331[144], 13}, {NULL}};
const Segment SGI86966[] = {{&S58826[40], 12}, {NULL}};
const Segment SGI86967[] = {{&S58826[40], 38}, {NULL}};
const Segment SGI86968[] = {{&S58826[29], 4}, {NULL}};
const Segment SGI86969[] = {{&S58826[30], 3}, {NULL}};
const Segment SGI86970[] = {{&S58826[92], 5}, {NULL}};
const Segment SGI86971[] = {{&S69383[44], 16}, {NULL}};
const Segment SGI86972[] = {{&S69383[44], 28}, {NULL}};
const Segment SGI86973[] = {{&S69383[30], 6}, {NULL}};
const Segment SGI86974[] = {{&S69383[120], 3}, {NULL}};
const Segment SGI86975[] = {{&S69383[86], 5}, {NULL}};
const Segment SGI86976[] = {{&S69384[44], 15}, {NULL}};
const Segment SGI86977[] = {{&S69384[44], 27}, {NULL}};
const Segment SGI86978[] = {{&S69384[30], 6}, {NULL}};
const Segment SGI86979[] = {{&S69384[119], 3}, {NULL}};
const Segment SGI86980[] = {{&S69384[85], 5}, {NULL}};
const Segment SGI86981[] = {{&S69460[44], 10}, {NULL}};
const Segment SGI86982[] = {{&S69460[44], 27}, {NULL}};
const Segment SGI86983[] = {{&S69460[30], 6}, {NULL}};
const Segment SGI86984[] = {{&S69460[119], 3}, {NULL}};
const Segment SGI86985[] = {{&S69460[85], 5}, {NULL}};
const Segment SGI86986[] = {{&S69461[44], 10}, {NULL}};
const Segment SGI86987[] = {{&S69461[44], 27}, {NULL}};
const Segment SGI86988[] = {{&S69461[30], 6}, {NULL}};
const Segment SGI86989[] = {{&S69461[88], 4}, {NULL}};
const Segment SGI86990[] = {{&S69461[88], 3}, {NULL}};
const Segment SGI86991[] = {{&S69462[44], 10}, {NULL}};
const Segment SGI86992[] = {{&S69462[44], 27}, {NULL}};
const Segment SGI86993[] = {{&S69462[30], 6}, {NULL}};
const Segment SGI86994[] = {{&S69462[88], 6}, {NULL}};
const Segment SGI86995[] = {{&S69463[44], 11}, {NULL}};
const Segment SGI86996[] = {{&S69463[44], 28}, {NULL}};
const Segment SGI86997[] = {{&S69463[30], 6}, {NULL}};
const Segment SGI86998[] = {{&S69463[120], 3}, {NULL}};
const Segment SGI86999[] = {{&S69463[86], 5}, {NULL}};
const Segment SGI87000[] = {{&S69464[38], 11}, {NULL}};
const Segment SGI87001[] = {{&S69464[31], 6}, {NULL}};
const Segment SGI87002[] = {{&S69465[44], 11}, {NULL}};
const Segment SGI87003[] = {{&S69465[44], 28}, {NULL}};
const Segment SGI87004[] = {{&S69465[30], 6}, {NULL}};
const Segment SGI87005[] = {{&S69465[89], 6}, {NULL}};
const Segment SGI87006[] = {{&S69466[44], 10}, {NULL}};
const Segment SGI87007[] = {{&S69466[44], 27}, {NULL}};
const Segment SGI87008[] = {{&S69466[30], 6}, {NULL}};
const Segment SGI87009[] = {{&S69466[119], 3}, {NULL}};
const Segment SGI87010[] = {{&S69466[85], 5}, {NULL}};
const Segment SGI87011[] = {{&S69467[44], 10}, {NULL}};
const Segment SGI87012[] = {{&S69467[44], 27}, {NULL}};
const Segment SGI87013[] = {{&S69467[30], 6}, {NULL}};
const Segment SGI87014[] = {{&S69467[88], 6}, {NULL}};
const Segment SGI87015[] = {{&S69468[38], 10}, {NULL}};
const Segment SGI87016[] = {{&S69468[31], 6}, {NULL}};
const Segment SGI87017[] = {{&S69500[44], 11}, {NULL}};
const Segment SGI87018[] = {{&S69500[44], 28}, {NULL}};
const Segment SGI87019[] = {{&S69500[30], 6}, {NULL}};
const Segment SGI87020[] = {{&S69500[120], 3}, {NULL}};
const Segment SGI87021[] = {{&S69500[86], 5}, {NULL}};
const Segment SGI87022[] = {{&S48833[44], 13}, {NULL}};
const Segment SGI87023[] = {{&S48833[44], 25}, {NULL}};
const Segment SGI87024[] = {{&S48833[30], 6}, {NULL}};
const Segment SGI87025[] = {{&S48833[31], 3}, {NULL}};
const Segment SGI87026[] = {{&S48833[83], 6}, {NULL}};
const Segment SGI87027[] = {{&S48834[38], 13}, {NULL}};
const Segment SGI87028[] = {{&S48834[31], 6}, {NULL}};
const Segment SGI87029[] = {{&S48848[34], 18}, {NULL}};
const Segment SGI87030[] = {{&S48848[27], 6}, {NULL}};
const Segment SGI87031[] = {{&S48848[113], 9}, {NULL}};
const Segment SGI87032[] = {{&S48848[86], 6}, {NULL}};
const Segment SGI87033[] = {{&S48849[34], 17}, {NULL}};
const Segment SGI87034[] = {{&S48849[27], 6}, {NULL}};
const Segment SGI87035[] = {{&S48849[112], 9}, {NULL}};
const Segment SGI87036[] = {{&S48849[85], 6}, {NULL}};
const Segment SGI87037[] = {{&S48850[34], 13}, {NULL}};
const Segment SGI87038[] = {{&S69539[44], 4}, {NULL}};
const Segment SGI87039[] = {{&S48850[27], 6}, {NULL}};
const Segment SGI87040[] = {{&S69539[44], 16}, {NULL}};
const Segment SGI87041[] = {{&S69539[30], 6}, {NULL}};
const Segment SGI87042[] = {{&S48850[108], 9}, {NULL}};
const Segment SGI87043[] = {{&S69539[108], 3}, {NULL}};
const Segment SGI87044[] = {{&S48850[81], 6}, {NULL}};
const Segment SGI87045[] = {{&S69539[74], 5}, {NULL}};
const Segment SGI87046[] = {{&S48851[34], 17}, {NULL}};
const Segment SGI87047[] = {{&S48851[27], 6}, {NULL}};
const Segment SGI87048[] = {{&S48851[112], 9}, {NULL}};
const Segment SGI87049[] = {{&S48851[85], 6}, {NULL}};
const Segment SGI87050[] = {{&S69543[44], 9}, {NULL}};
const Segment SGI87051[] = {{&S69543[44], 21}, {NULL}};
const Segment SGI87052[] = {{&S69543[30], 6}, {NULL}};
const Segment SGI87053[] = {{&S69543[113], 3}, {NULL}};
const Segment SGI87054[] = {{&S69543[79], 5}, {NULL}};
const Segment SGI87055[] = {{&S48856[34], 21}, {NULL}};
const Segment SGI87056[] = {{&S48856[27], 6}, {NULL}};
const Segment SGI87057[] = {{&S48856[108], 9}, {NULL}};
const Segment SGI87058[] = {{&S48856[81], 6}, {NULL}};
const Segment SGI87059[] = {{&S48857[34], 18}, {NULL}};
const Segment SGI87060[] = {{&S48857[27], 6}, {NULL}};
const Segment SGI87061[] = {{&S48857[105], 9}, {NULL}};
const Segment SGI87062[] = {{&S48857[78], 6}, {NULL}};
const Segment SGI87063[] = {{&S48859[34], 35}, {NULL}};
const Segment SGI87064[] = {{&S48859[27], 6}, {NULL}};
const Segment SGI87065[] = {{&S48859[122], 9}, {NULL}};
const Segment SGI87066[] = {{&S48859[95], 6}, {NULL}};
const Segment SGI87067[] = {{&S69561[44], 11}, {NULL}};
const Segment SGI87068[] = {{&S69561[44], 38}, {NULL}};
const Segment SGI87069[] = {{&S69561[30], 6}, {NULL}};
const Segment SGI87070[] = {{&S69561[130], 3}, {NULL}};
const Segment SGI87071[] = {{&S69561[96], 5}, {NULL}};
const Segment SGI87072[] = {{&S69562[44], 11}, {NULL}};
const Segment SGI87073[] = {{&S69562[44], 38}, {NULL}};
const Segment SGI87074[] = {{&S69562[30], 6}, {NULL}};
const Segment SGI87075[] = {{&S48871[34], 10}, {NULL}};
const Segment SGI87076[] = {{&S69562[130], 3}, {NULL}};
const Segment SGI87077[] = {{&S48871[27], 6}, {NULL}};
const Segment SGI87078[] = {{&S69562[96], 5}, {NULL}};
const Segment SGI87079[] = {{&S48871[97], 9}, {NULL}};
const Segment SGI87080[] = {{&S48871[70], 6}, {NULL}};
const Segment SGI87081[] = {{&S69563[44], 11}, {NULL}};
const Segment SGI87082[] = {{&S69563[44], 38}, {NULL}};
const Segment SGI87083[] = {{&S69563[30], 6}, {NULL}};
const Segment SGI87084[] = {{&S69563[130], 3}, {NULL}};
const Segment SGI87085[] = {{&S69563[96], 5}, {NULL}};
const Segment SGI87086[] = {{&S48873[34], 11}, {NULL}};
const Segment SGI87087[] = {{&S48873[27], 6}, {NULL}};
const Segment SGI87088[] = {{&S48873[98], 9}, {NULL}};
const Segment SGI87089[] = {{&S48873[71], 6}, {NULL}};
const Segment SGI87090[] = {{&S69564[44], 11}, {NULL}};
const Segment SGI87091[] = {{&S69564[44], 44}, {NULL}};
const Segment SGI87092[] = {{&S69564[30], 6}, {NULL}};
const Segment SGI87093[] = {{&S69564[136], 3}, {NULL}};
const Segment SGI87094[] = {{&S69564[102], 5}, {NULL}};
const Segment SGI87095[] = {{&S48958[44], 11}, {NULL}};
const Segment SGI87096[] = {{&S48958[44], 39}, {NULL}};
const Segment SGI87097[] = {{&S48958[30], 6}, {NULL}};
const Segment SGI87098[] = {{&S48958[31], 3}, {NULL}};
const Segment SGI87099[] = {{&S48958[97], 6}, {NULL}};
const Segment SGI87100[] = {{&S48967[44], 11}, {NULL}};
const Segment SGI87101[] = {{&S48967[44], 39}, {NULL}};
const Segment SGI87102[] = {{&S48967[30], 6}, {NULL}};
const Segment SGI87103[] = {{&S48967[31], 3}, {NULL}};
const Segment SGI87104[] = {{&S48967[97], 6}, {NULL}};
const Segment SGI87105[] = {{&S49088[44], 9}, {NULL}};
const Segment SGI87106[] = {{&S49088[44], 21}, {NULL}};
const Segment SGI87107[] = {{&S49088[30], 6}, {NULL}};
const Segment SGI87108[] = {{&S49088[31], 3}, {NULL}};
const Segment SGI87109[] = {{&S49088[79], 6}, {NULL}};
const Segment SGI87110[] = {{&S49104[34], 10}, {NULL}};
const Segment SGI87111[] = {{&S49104[27], 6}, {NULL}};
const Segment SGI87112[] = {{&S49104[97], 9}, {NULL}};
const Segment SGI87113[] = {{&S49104[70], 6}, {NULL}};
const Segment SGI87114[] = {{&S69806[44], 10}, {NULL}};
const Segment SGI87115[] = {{&S69806[44], 27}, {NULL}};
const Segment SGI87116[] = {{&S69806[30], 6}, {NULL}};
const Segment SGI87117[] = {{&S69806[119], 3}, {NULL}};
const Segment SGI87118[] = {{&S69806[85], 5}, {NULL}};
const Segment SGI87119[] = {{&S69809[44], 14}, {NULL}};
const Segment SGI87120[] = {{&S69809[44], 26}, {NULL}};
const Segment SGI87121[] = {{&S69809[30], 6}, {NULL}};
const Segment SGI87122[] = {{&S69809[87], 6}, {NULL}};
const Segment SGI87123[] = {{&S69810[44], 14}, {NULL}};
const Segment SGI87124[] = {{&S69810[44], 26}, {NULL}};
const Segment SGI87125[] = {{&S69810[30], 6}, {NULL}};
const Segment SGI87126[] = {{&S69810[87], 4}, {NULL}};
const Segment SGI87127[] = {{&S69810[87], 3}, {NULL}};
const Segment SGI87128[] = {{&S69811[44], 14}, {NULL}};
const Segment SGI87129[] = {{&S69811[44], 26}, {NULL}};
const Segment SGI87130[] = {{&S69811[30], 6}, {NULL}};
const Segment SGI87131[] = {{&S69811[118], 3}, {NULL}};
const Segment SGI87132[] = {{&S69811[84], 5}, {NULL}};
const Segment SGI87133[] = {{&S69812[44], 26}, {NULL}};
const Segment SGI87134[] = {{&S69812[30], 6}, {NULL}};
const Segment SGI87135[] = {{&S69813[38], 14}, {NULL}};
const Segment SGI87136[] = {{&S69813[31], 6}, {NULL}};
const Segment SGI87137[] = {{&S49180[44], 10}, {NULL}};
const Segment SGI87138[] = {{&S49180[44], 22}, {NULL}};
const Segment SGI87139[] = {{&S49180[30], 6}, {NULL}};
const Segment SGI87140[] = {{&S49180[31], 3}, {NULL}};
const Segment SGI87141[] = {{&S49180[80], 6}, {NULL}};
const Segment SGI87142[] = {{&S49181[38], 10}, {NULL}};
const Segment SGI87143[] = {{&S49181[31], 6}, {NULL}};
const Segment SGI87144[] = {{&S49216[44], 9}, {NULL}};
const Segment SGI87145[] = {{&S49216[44], 21}, {NULL}};
const Segment SGI87146[] = {{&S49216[30], 6}, {NULL}};
const Segment SGI87147[] = {{&S49216[31], 3}, {NULL}};
const Segment SGI87148[] = {{&S49216[79], 6}, {NULL}};
const Segment SGI87149[] = {{&S49219[38], 9}, {NULL}};
const Segment SGI87150[] = {{&S49219[31], 6}, {NULL}};
const Segment SGI87151[] = {{&S61504[44], 10}, {NULL}};
const Segment SGI87152[] = {{&S61504[44], 21}, {NULL}};
const Segment SGI87153[] = {{&S61504[30], 6}, {NULL}};
const Segment SGI87154[] = {{&S61504[114], 3}, {NULL}};
const Segment SGI87155[] = {{&S61504[79], 6}, {NULL}};
const Segment SGI87156[] = {{&S61505[38], 10}, {NULL}};
const Segment SGI87157[] = {{&S61505[31], 6}, {NULL}};
const Segment SGI87158[] = {{&S62076[6], 4}, {NULL}};
const Segment SGI87159[] = {{&S62076[85], 5}, {NULL}};
const Segment SGI87160[] = {{&S69932[44], 7}, {NULL}};
const Segment SGI87161[] = {{&S69932[44], 19}, {NULL}};
const Segment SGI87162[] = {{&S69932[30], 6}, {NULL}};
const Segment SGI87163[] = {{&S69932[111], 3}, {NULL}};
const Segment SGI87164[] = {{&S69932[77], 5}, {NULL}};
const Segment SGI87165[] = {{&S49308[44], 5}, {NULL}};
const Segment SGI87166[] = {{&S49308[44], 17}, {NULL}};
const Segment SGI87167[] = {{&S49308[30], 6}, {NULL}};
const Segment SGI87168[] = {{&S49308[31], 3}, {NULL}};
const Segment SGI87169[] = {{&S49308[75], 6}, {NULL}};
const Segment SGI87170[] = {{&S49309[44], 5}, {NULL}};
const Segment SGI87171[] = {{&S49309[44], 18}, {NULL}};
const Segment SGI87172[] = {{&S49309[30], 6}, {NULL}};
const Segment SGI87173[] = {{&S49309[31], 3}, {NULL}};
const Segment SGI87174[] = {{&S49309[76], 6}, {NULL}};
const Segment SGI87175[] = {{&S69994[44], 25}, {NULL}};
const Segment SGI87176[] = {{&S69994[30], 6}, {NULL}};
const Segment SGI87177[] = {{&S69994[135], 3}, {NULL}};
const Segment SGI87178[] = {{&S69994[101], 5}, {NULL}};
const Segment SGI87179[] = {{&S69999[44], 7}, {NULL}};
const Segment SGI87180[] = {{&S69999[44], 27}, {NULL}};
const Segment SGI87181[] = {{&S69999[30], 6}, {NULL}};
const Segment SGI87182[] = {{&S69999[88], 6}, {NULL}};
const Segment SGI87183[] = {{&S70000[44], 7}, {NULL}};
const Segment SGI87184[] = {{&S70000[44], 27}, {NULL}};
const Segment SGI87185[] = {{&S70000[30], 6}, {NULL}};
const Segment SGI87186[] = {{&S70000[119], 3}, {NULL}};
const Segment SGI87187[] = {{&S70000[85], 5}, {NULL}};
const Segment SGI87188[] = {{&S70017[44], 10}, {NULL}};
const Segment SGI87189[] = {{&S70017[44], 22}, {NULL}};
const Segment SGI87190[] = {{&S70017[30], 6}, {NULL}};
const Segment SGI87191[] = {{&S70017[114], 3}, {NULL}};
const Segment SGI87192[] = {{&S70017[80], 5}, {NULL}};
const Segment SGI87193[] = {{&S70018[44], 6}, {NULL}};
const Segment SGI87194[] = {{&S70018[44], 18}, {NULL}};
const Segment SGI87195[] = {{&S70018[30], 6}, {NULL}};
const Segment SGI87196[] = {{&S70018[79], 4}, {NULL}};
const Segment SGI87197[] = {{&S70018[79], 3}, {NULL}};
const Segment SGI87198[] = {{&S70019[44], 6}, {NULL}};
const Segment SGI87199[] = {{&S70019[44], 18}, {NULL}};
const Segment SGI87200[] = {{&S70019[30], 6}, {NULL}};
const Segment SGI87201[] = {{&S70019[79], 6}, {NULL}};
const Segment SGI87202[] = {{&S70020[38], 6}, {NULL}};
const Segment SGI87203[] = {{&S70020[31], 6}, {NULL}};
const Segment SGI87204[] = {{&S62203[30], 6}, {NULL}};
const Segment SGI87205[] = {{&S62203[92], 2}, {NULL}};
const Segment SGI87206[] = {{&S62203[57], 6}, {NULL}};
const Segment SGI87207[] = {{&S70055[44], 14}, {NULL}};
const Segment SGI87208[] = {{&S70055[44], 26}, {NULL}};
const Segment SGI87209[] = {{&S70055[30], 6}, {NULL}};
const Segment SGI87210[] = {{&S70055[118], 3}, {NULL}};
const Segment SGI87211[] = {{&S70055[84], 5}, {NULL}};
const Segment SGI87212[] = {{&S70056[44], 14}, {NULL}};
const Segment SGI87213[] = {{&S70056[44], 26}, {NULL}};
const Segment SGI87214[] = {{&S70056[30], 6}, {NULL}};
const Segment SGI87215[] = {{&S70056[118], 3}, {NULL}};
const Segment SGI87216[] = {{&S70056[84], 5}, {NULL}};
const Segment SGI87217[] = {{&S70060[44], 10}, {NULL}};
const Segment SGI87218[] = {{&S70060[44], 27}, {NULL}};
const Segment SGI87219[] = {{&S70060[30], 6}, {NULL}};
const Segment SGI87220[] = {{&S70060[85], 6}, {NULL}};
const Segment SGI87221[] = {{&S70060[112], 13}, {NULL}};
const Segment SGI87222[] = {{&S62377[44], 7}, {NULL}};
const Segment SGI87223[] = {{&S62377[44], 30}, {&S62377[75], 18}, {NULL}};
const Segment SGI87224[] = {{&S62377[30], 6}, {NULL}};
const Segment SGI87225[] = {{&S62377[141], 3}, {NULL}};
const Segment SGI87226[] = {{&S62377[107], 5}, {NULL}};
const Segment SGI87227[] = {{&S62377[35], 1}, {NULL}};
const Segment SGI87228[] = {{&S62377[76], 11}, {NULL}};
const Segment SGI87229[] = {{&S56969[74], 6}, {NULL}};
const Segment SGI87230[] = {{&S56969[0], 9}, {NULL}};
const Segment SGI87231[] = {{&S70085[44], 11}, {NULL}};
const Segment SGI87232[] = {{&S70085[44], 23}, {NULL}};
const Segment SGI87233[] = {{&S70085[30], 6}, {NULL}};
const Segment SGI87234[] = {{&S70085[115], 3}, {NULL}};
const Segment SGI87235[] = {{&S70085[81], 5}, {NULL}};
const Segment SGI87236[] = {{&S49590[34], 6}, {NULL}};
const Segment SGI87237[] = {{&S49590[27], 6}, {NULL}};
const Segment SGI87238[] = {{&S49590[93], 9}, {NULL}};
const Segment SGI87239[] = {{&S49590[66], 6}, {NULL}};
const Segment SGI87240[] = {{&S49591[34], 5}, {NULL}};
const Segment SGI87241[] = {{&S49591[27], 6}, {NULL}};
const Segment SGI87242[] = {{&S49591[92], 9}, {NULL}};
const Segment SGI87243[] = {{&S49591[65], 6}, {NULL}};
const Segment SGI87244[] = {{&S49595[6], 4}, {NULL}};
const Segment SGI87245[] = {{&S49595[83], 5}, {NULL}};
const Segment SGI87246[] = {{&S49621[44], 13}, {NULL}};
const Segment SGI87247[] = {{&S49621[44], 25}, {NULL}};
const Segment SGI87248[] = {{&S49621[30], 6}, {NULL}};
const Segment SGI87249[] = {{&S49621[31], 3}, {NULL}};
const Segment SGI87250[] = {{&S49621[83], 6}, {NULL}};
const Segment SGI87251[] = {{&S49622[44], 18}, {NULL}};
const Segment SGI87252[] = {{&S49622[44], 30}, {NULL}};
const Segment SGI87253[] = {{&S49622[30], 6}, {NULL}};
const Segment SGI87254[] = {{&S49622[31], 3}, {NULL}};
const Segment SGI87255[] = {{&S49622[88], 6}, {NULL}};
const Segment SGI87256[] = {{&S49623[44], 13}, {NULL}};
const Segment SGI87257[] = {{&S49623[44], 25}, {NULL}};
const Segment SGI87258[] = {{&S49623[30], 6}, {NULL}};
const Segment SGI87259[] = {{&S49623[31], 3}, {NULL}};
const Segment SGI87260[] = {{&S49623[83], 6}, {NULL}};
const Segment SGI87261[] = {{&S70164[44], 14}, {NULL}};
const Segment SGI87262[] = {{&S70164[44], 26}, {NULL}};
const Segment SGI87263[] = {{&S70164[30], 6}, {NULL}};
const Segment SGI87264[] = {{&S49624[42], 13}, {NULL}};
const Segment SGI87265[] = {{&S49624[30], 6}, {NULL}};
const Segment SGI87266[] = {{&S70164[118], 3}, {NULL}};
const Segment SGI87267[] = {{&S70164[84], 5}, {NULL}};
const Segment SGI87268[] = {{&S49624[31], 3}, {NULL}};
const Segment SGI87269[] = {{&S49624[81], 6}, {NULL}};
const Segment SGI87270[] = {{&S70165[44], 7}, {NULL}};
const Segment SGI87271[] = {{&S70165[44], 19}, {NULL}};
const Segment SGI87272[] = {{&S49625[38], 18}, {NULL}};
const Segment SGI87273[] = {{&S70165[30], 6}, {NULL}};
const Segment SGI87274[] = {{&S49625[31], 6}, {NULL}};
const Segment SGI87275[] = {{&S70165[111], 3}, {NULL}};
const Segment SGI87276[] = {{&S70165[77], 5}, {NULL}};
const Segment SGI87277[] = {{&S49626[44], 18}, {NULL}};
const Segment SGI87278[] = {{&S49626[44], 30}, {NULL}};
const Segment SGI87279[] = {{&S49626[30], 6}, {NULL}};
const Segment SGI87280[] = {{&S49626[31], 3}, {NULL}};
const Segment SGI87281[] = {{&S49626[88], 6}, {NULL}};
const Segment SGI87282[] = {{&S70192[45], 10}, {NULL}};
const Segment SGI87283[] = {{&S70192[30], 6}, {NULL}};
const Segment SGI87284[] = {{&S70192[120], 3}, {NULL}};
const Segment SGI87285[] = {{&S70192[86], 5}, {NULL}};
const Segment SGI87286[] = {{&S70192[144], 13}, {NULL}};
const Segment SGI87287[] = {{&S70200[30], 6}, {NULL}};
const Segment SGI87288[] = {{&S70200[92], 2}, {NULL}};
const Segment SGI87289[] = {{&S70200[57], 6}, {NULL}};
const Segment SGI87290[] = {{&S70215[44], 10}, {NULL}};
const Segment SGI87291[] = {{&S70215[44], 27}, {NULL}};
const Segment SGI87292[] = {{&S70215[30], 6}, {NULL}};
const Segment SGI87293[] = {{&S70215[85], 6}, {NULL}};
const Segment SGI87294[] = {{&S70215[112], 13}, {NULL}};
const Segment SGI87295[] = {{&S70248[44], 11}, {NULL}};
const Segment SGI87296[] = {{&S70248[44], 32}, {NULL}};
const Segment SGI87297[] = {{&S70248[30], 6}, {NULL}};
const Segment SGI87298[] = {{&S70248[124], 3}, {NULL}};
const Segment SGI87299[] = {{&S70248[90], 5}, {NULL}};
const Segment SGI87300[] = {{&S62496[44], 16}, {NULL}};
const Segment SGI87301[] = {{&S62496[44], 27}, {NULL}};
const Segment SGI87302[] = {{&S62496[30], 6}, {NULL}};
const Segment SGI87303[] = {{&S49729[34], 10}, {NULL}};
const Segment SGI87304[] = {{&S49729[27], 6}, {NULL}};
const Segment SGI87305[] = {{&S62496[118], 3}, {NULL}};
const Segment SGI87306[] = {{&S62496[85], 5}, {NULL}};
const Segment SGI87307[] = {{&S49729[97], 9}, {NULL}};
const Segment SGI87308[] = {{&S49729[70], 6}, {NULL}};
const Segment SGI87309[] = {{&S70277[44], 11}, {NULL}};
const Segment SGI87310[] = {{&S70277[44], 28}, {NULL}};
const Segment SGI87311[] = {{&S70277[30], 6}, {NULL}};
const Segment SGI87312[] = {{&S70277[150], 4}, {NULL}};
const Segment SGI87313[] = {{&S70277[176], 3}, {NULL}};
const Segment SGI87314[] = {{&S70277[120], 3}, {&S70277[225], 5}, {NULL}};
const Segment SGI87315[] = {{&S70277[86], 5}, {NULL}};
const Segment SGI87316[] = {{&S62557[44], 7}, {NULL}};
const Segment SGI87317[] = {{&S62557[44], 18}, {NULL}};
const Segment SGI87318[] = {{&S62557[30], 6}, {NULL}};
const Segment SGI87319[] = {{&S62557[129], 3}, {NULL}};
const Segment SGI87320[] = {{&S62557[95], 5}, {NULL}};
const Segment SGI87321[] = {{&S62557[35], 1}, {NULL}};
const Segment SGI87322[] = {{&S62557[64], 11}, {NULL}};
const Segment SGI87323[] = {{&S57243[6], 4}, {NULL}};
const Segment SGI87324[] = {{&S57243[76], 5}, {NULL}};
const Segment SGI87325[] = {{&S57243[0], 27}, {NULL}};
const Segment SGI87326[] = {{&S49820[34], 10}, {NULL}};
const Segment SGI87327[] = {{&S49820[27], 6}, {NULL}};
const Segment SGI87328[] = {{&S49820[97], 9}, {NULL}};
const Segment SGI87329[] = {{&S49820[70], 6}, {NULL}};
const Segment SGI87330[] = {{&S49824[34], 11}, {NULL}};
const Segment SGI87331[] = {{&S49824[27], 6}, {NULL}};
const Segment SGI87332[] = {{&S49824[98], 9}, {NULL}};
const Segment SGI87333[] = {{&S70366[44], 11}, {NULL}};
const Segment SGI87334[] = {{&S49824[71], 6}, {NULL}};
const Segment SGI87335[] = {{&S70366[44], 23}, {NULL}};
const Segment SGI87336[] = {{&S70366[30], 6}, {NULL}};
const Segment SGI87337[] = {{&S70366[115], 3}, {NULL}};
const Segment SGI87338[] = {{&S70366[81], 5}, {NULL}};
const Segment SGI87339[] = {{&S70367[44], 11}, {NULL}};
const Segment SGI87340[] = {{&S70367[44], 23}, {NULL}};
const Segment SGI87341[] = {{&S70367[30], 6}, {NULL}};
const Segment SGI87342[] = {{&S70367[84], 4}, {NULL}};
const Segment SGI87343[] = {{&S70367[84], 3}, {NULL}};
const Segment SGI87344[] = {{&S70369[44], 11}, {NULL}};
const Segment SGI87345[] = {{&S70369[44], 23}, {NULL}};
const Segment SGI87346[] = {{&S70369[30], 6}, {NULL}};
const Segment SGI87347[] = {{&S70369[84], 6}, {NULL}};
const Segment SGI87348[] = {{&S70370[38], 11}, {NULL}};
const Segment SGI87349[] = {{&S70370[31], 6}, {NULL}};
const Segment SGI87350[] = {{&S70389[44], 10}, {NULL}};
const Segment SGI87351[] = {{&S70389[44], 27}, {NULL}};
const Segment SGI87352[] = {{&S70389[30], 6}, {NULL}};
const Segment SGI87353[] = {{&S70389[85], 6}, {NULL}};
const Segment SGI87354[] = {{&S70389[112], 13}, {NULL}};
const Segment SGI87355[] = {{&S70411[44], 15}, {NULL}};
const Segment SGI87356[] = {{&S70411[44], 36}, {NULL}};
const Segment SGI87357[] = {{&S70411[30], 6}, {NULL}};
const Segment SGI87358[] = {{&S70411[128], 3}, {NULL}};
const Segment SGI87359[] = {{&S70411[94], 5}, {NULL}};
const Segment SGI87360[] = {{&S70412[44], 22}, {NULL}};
const Segment SGI87361[] = {{&S70412[44], 43}, {NULL}};
const Segment SGI87362[] = {{&S70412[30], 6}, {NULL}};
const Segment SGI87363[] = {{&S70412[135], 3}, {NULL}};
const Segment SGI87364[] = {{&S70412[101], 5}, {NULL}};
const Segment SGI87365[] = {{&S70413[44], 22}, {NULL}};
const Segment SGI87366[] = {{&S70413[44], 43}, {NULL}};
const Segment SGI87367[] = {{&S70413[30], 6}, {NULL}};
const Segment SGI87368[] = {{&S70413[135], 3}, {NULL}};
const Segment SGI87369[] = {{&S70413[101], 5}, {NULL}};
const Segment SGI87370[] = {{&S49918[44], 11}, {NULL}};
const Segment SGI87371[] = {{&S49918[44], 23}, {NULL}};
const Segment SGI87372[] = {{&S49918[30], 6}, {NULL}};
const Segment SGI87373[] = {{&S49918[31], 3}, {NULL}};
const Segment SGI87374[] = {{&S49918[81], 6}, {NULL}};
const Segment SGI87375[] = {{&S70462[44], 10}, {NULL}};
const Segment SGI87376[] = {{&S70462[44], 22}, {NULL}};
const Segment SGI87377[] = {{&S70462[30], 6}, {NULL}};
const Segment SGI87378[] = {{&S70462[114], 3}, {NULL}};
const Segment SGI87379[] = {{&S70462[80], 5}, {NULL}};
const Segment SGI87380[] = {{&S70463[44], 6}, {NULL}};
const Segment SGI87381[] = {{&S70463[44], 18}, {NULL}};
const Segment SGI87382[] = {{&S70463[30], 6}, {NULL}};
const Segment SGI87383[] = {{&S70463[79], 6}, {NULL}};
const Segment SGI87384[] = {{&S70464[44], 6}, {NULL}};
const Segment SGI87385[] = {{&S70464[44], 18}, {NULL}};
const Segment SGI87386[] = {{&S70464[30], 6}, {NULL}};
const Segment SGI87387[] = {{&S70464[79], 4}, {NULL}};
const Segment SGI87388[] = {{&S70464[79], 3}, {NULL}};
const Segment SGI87389[] = {{&S70465[38], 6}, {NULL}};
const Segment SGI87390[] = {{&S70465[31], 6}, {NULL}};
const Segment SGI87391[] = {{&S49943[44], 18}, {NULL}};
const Segment SGI87392[] = {{&S49943[44], 30}, {NULL}};
const Segment SGI87393[] = {{&S49943[30], 6}, {NULL}};
const Segment SGI87394[] = {{&S49943[31], 3}, {NULL}};
const Segment SGI87395[] = {{&S49943[88], 6}, {NULL}};
const Segment SGI87396[] = {{&S49944[38], 18}, {NULL}};
const Segment SGI87397[] = {{&S49944[31], 6}, {NULL}};
const Segment SGI87398[] = {{&S49960[44], 8}, {NULL}};
const Segment SGI87399[] = {{&S49960[44], 17}, {NULL}};
const Segment SGI87400[] = {{&S49960[30], 6}, {NULL}};
const Segment SGI87401[] = {{&S49960[31], 3}, {NULL}};
const Segment SGI87402[] = {{&S49960[75], 6}, {NULL}};
const Segment SGI87403[] = {{&S49961[42], 8}, {NULL}};
const Segment SGI87404[] = {{&S49961[30], 6}, {NULL}};
const Segment SGI87405[] = {{&S49961[31], 3}, {NULL}};
const Segment SGI87406[] = {{&S49961[73], 6}, {NULL}};
const Segment SGI87407[] = {{&S49963[34], 7}, {NULL}};
const Segment SGI87408[] = {{&S49963[27], 6}, {NULL}};
const Segment SGI87409[] = {{&S49963[94], 9}, {NULL}};
const Segment SGI87410[] = {{&S49963[67], 6}, {NULL}};
const Segment SGI87411[] = {{&S49964[34], 6}, {NULL}};
const Segment SGI87412[] = {{&S49964[27], 6}, {NULL}};
const Segment SGI87413[] = {{&S49964[93], 9}, {NULL}};
const Segment SGI87414[] = {{&S49964[66], 6}, {NULL}};
const Segment SGI87415[] = {{&S49986[44], 4}, {NULL}};
const Segment SGI87416[] = {{&S49986[44], 34}, {NULL}};
const Segment SGI87417[] = {{&S49986[30], 6}, {NULL}};
const Segment SGI87418[] = {{&S49986[31], 3}, {NULL}};
const Segment SGI87419[] = {{&S49986[92], 6}, {NULL}};
const Segment SGI87420[] = {{&S49987[44], 4}, {NULL}};
const Segment SGI87421[] = {{&S49987[44], 41}, {NULL}};
const Segment SGI87422[] = {{&S49987[30], 6}, {NULL}};
const Segment SGI87423[] = {{&S49987[31], 3}, {NULL}};
const Segment SGI87424[] = {{&S49987[99], 6}, {NULL}};
const Segment SGI87425[] = {{&S49988[44], 4}, {NULL}};
const Segment SGI87426[] = {{&S49988[44], 34}, {NULL}};
const Segment SGI87427[] = {{&S49988[30], 6}, {NULL}};
const Segment SGI87428[] = {{&S49988[31], 3}, {NULL}};
const Segment SGI87429[] = {{&S49988[92], 6}, {NULL}};
const Segment SGI87430[] = {{&S49994[44], 11}, {NULL}};
const Segment SGI87431[] = {{&S49994[44], 23}, {NULL}};
const Segment SGI87432[] = {{&S49994[30], 6}, {NULL}};
const Segment SGI87433[] = {{&S49994[31], 3}, {NULL}};
const Segment SGI87434[] = {{&S49994[81], 6}, {NULL}};
const Segment SGI87435[] = {{&S70562[44], 10}, {NULL}};
const Segment SGI87436[] = {{&S70562[44], 27}, {NULL}};
const Segment SGI87437[] = {{&S70562[30], 6}, {NULL}};
const Segment SGI87438[] = {{&S70562[88], 6}, {NULL}};
const Segment SGI87439[] = {{&S70563[44], 10}, {NULL}};
const Segment SGI87440[] = {{&S70563[44], 27}, {NULL}};
const Segment SGI87441[] = {{&S70563[30], 6}, {NULL}};
const Segment SGI87442[] = {{&S70563[119], 3}, {NULL}};
const Segment SGI87443[] = {{&S70563[85], 5}, {NULL}};
const Segment SGI87444[] = {{&S70564[38], 10}, {NULL}};
const Segment SGI87445[] = {{&S70564[31], 6}, {NULL}};
const Segment SGI87446[] = {{&S70565[44], 7}, {NULL}};
const Segment SGI87447[] = {{&S70565[44], 19}, {NULL}};
const Segment SGI87448[] = {{&S70565[30], 6}, {NULL}};
const Segment SGI87449[] = {{&S70565[111], 3}, {NULL}};
const Segment SGI87450[] = {{&S70565[77], 5}, {NULL}};
const Segment SGI87451[] = {{&S50049[34], 9}, {NULL}};
const Segment SGI87452[] = {{&S50049[27], 6}, {NULL}};
const Segment SGI87453[] = {{&S50049[96], 9}, {NULL}};
const Segment SGI87454[] = {{&S50049[69], 6}, {NULL}};
const Segment SGI87455[] = {{&S50050[34], 10}, {NULL}};
const Segment SGI87456[] = {{&S50050[27], 6}, {NULL}};
const Segment SGI87457[] = {{&S50050[97], 9}, {NULL}};
const Segment SGI87458[] = {{&S50050[70], 6}, {NULL}};
const Segment SGI87459[] = {{&S50051[34], 9}, {NULL}};
const Segment SGI87460[] = {{&S50051[27], 6}, {NULL}};
const Segment SGI87461[] = {{&S50051[96], 9}, {NULL}};
const Segment SGI87462[] = {{&S50051[69], 6}, {NULL}};
const Segment SGI87463[] = {{&S50052[34], 10}, {NULL}};
const Segment SGI87464[] = {{&S50052[27], 6}, {NULL}};
const Segment SGI87465[] = {{&S50052[97], 9}, {NULL}};
const Segment SGI87466[] = {{&S50052[70], 6}, {NULL}};
const Segment SGI87467[] = {{&S50053[34], 10}, {NULL}};
const Segment SGI87468[] = {{&S50053[27], 6}, {NULL}};
const Segment SGI87469[] = {{&S50053[97], 9}, {NULL}};
const Segment SGI87470[] = {{&S50053[70], 6}, {NULL}};
const Segment SGI87471[] = {{&S50054[34], 10}, {NULL}};
const Segment SGI87472[] = {{&S50054[27], 6}, {NULL}};
const Segment SGI87473[] = {{&S50054[97], 9}, {NULL}};
const Segment SGI87474[] = {{&S50054[70], 6}, {NULL}};
const Segment SGI87475[] = {{&S50061[34], 10}, {NULL}};
const Segment SGI87476[] = {{&S50061[27], 6}, {NULL}};
const Segment SGI87477[] = {{&S50061[97], 9}, {NULL}};
const Segment SGI87478[] = {{&S50061[70], 6}, {NULL}};
const Segment SGI87479[] = {{&S62883[44], 7}, {NULL}};
const Segment SGI87480[] = {{&S62883[44], 19}, {NULL}};
const Segment SGI87481[] = {{&S62883[30], 6}, {NULL}};
const Segment SGI87482[] = {{&S62883[111], 3}, {NULL}};
const Segment SGI87483[] = {{&S62883[77], 5}, {NULL}};
const Segment SGI87484[] = {{&S62884[44], 7}, {NULL}};
const Segment SGI87485[] = {{&S62884[44], 19}, {NULL}};
const Segment SGI87486[] = {{&S62884[30], 6}, {NULL}};
const Segment SGI87487[] = {{&S62884[111], 3}, {NULL}};
const Segment SGI87488[] = {{&S62884[77], 5}, {NULL}};
const Segment SGI87489[] = {{&S62885[44], 7}, {NULL}};
const Segment SGI87490[] = {{&S62885[44], 19}, {NULL}};
const Segment SGI87491[] = {{&S62885[30], 6}, {NULL}};
const Segment SGI87492[] = {{&S62885[111], 3}, {NULL}};
const Segment SGI87493[] = {{&S62885[77], 5}, {NULL}};
const Segment SGI87494[] = {{&S50106[34], 29}, {NULL}};
const Segment SGI87495[] = {{&S50106[27], 6}, {NULL}};
const Segment SGI87496[] = {{&S50106[116], 9}, {NULL}};
const Segment SGI87497[] = {{&S50106[89], 6}, {NULL}};
const Segment SGI87498[] = {{&S50108[34], 17}, {NULL}};
const Segment SGI87499[] = {{&S50108[27], 6}, {NULL}};
const Segment SGI87500[] = {{&S50108[104], 9}, {NULL}};
const Segment SGI87501[] = {{&S50108[77], 6}, {NULL}};
const Segment SGI87502[] = {{&S70666[44], 21}, {NULL}};
const Segment SGI87503[] = {{&S70666[44], 33}, {NULL}};
const Segment SGI87504[] = {{&S70666[30], 6}, {NULL}};
const Segment SGI87505[] = {{&S70666[125], 3}, {NULL}};
const Segment SGI87506[] = {{&S70666[91], 5}, {NULL}};
const Segment SGI87507[] = {{&S70667[38], 21}, {NULL}};
const Segment SGI87508[] = {{&S70667[31], 6}, {NULL}};
const Segment SGI87509[] = {{&S50189[34], 10}, {NULL}};
const Segment SGI87510[] = {{&S50189[27], 6}, {NULL}};
const Segment SGI87511[] = {{&S50189[97], 9}, {NULL}};
const Segment SGI87512[] = {{&S50189[70], 6}, {NULL}};
const Segment SGI87513[] = {{&S50194[34], 11}, {NULL}};
const Segment SGI87514[] = {{&S50194[27], 6}, {NULL}};
const Segment SGI87515[] = {{&S50194[98], 9}, {NULL}};
const Segment SGI87516[] = {{&S50194[71], 6}, {NULL}};
const Segment SGI87517[] = {{&S50273[44], 9}, {NULL}};
const Segment SGI87518[] = {{&S50273[44], 21}, {NULL}};
const Segment SGI87519[] = {{&S50273[30], 6}, {NULL}};
const Segment SGI87520[] = {{&S50273[31], 3}, {NULL}};
const Segment SGI87521[] = {{&S50273[79], 6}, {NULL}};
const Segment SGI87522[] = {{&S50274[38], 9}, {NULL}};
const Segment SGI87523[] = {{&S50274[31], 6}, {NULL}};
const Segment SGI87524[] = {{&S62985[44], 13}, {NULL}};
const Segment SGI87525[] = {{&S62985[44], 25}, {NULL}};
const Segment SGI87526[] = {{&S62985[30], 6}, {NULL}};
const Segment SGI87527[] = {{&S62985[136], 3}, {NULL}};
const Segment SGI87528[] = {{&S62985[102], 5}, {NULL}};
const Segment SGI87529[] = {{&S62985[35], 1}, {NULL}};
const Segment SGI87530[] = {{&S62985[71], 11}, {NULL}};
const Segment SGI87531[] = {{&S62986[44], 13}, {NULL}};
const Segment SGI87532[] = {{&S62986[44], 25}, {NULL}};
const Segment SGI87533[] = {{&S62986[30], 6}, {NULL}};
const Segment SGI87534[] = {{&S62986[150], 3}, {NULL}};
const Segment SGI87535[] = {{&S62986[116], 5}, {NULL}};
const Segment SGI87536[] = {{&S62986[71], 11}, {NULL}};
const Segment SGI87537[] = {{&S62987[44], 20}, {NULL}};
const Segment SGI87538[] = {{&S62987[44], 32}, {NULL}};
const Segment SGI87539[] = {{&S62987[30], 6}, {NULL}};
const Segment SGI87540[] = {{&S62987[139], 3}, {NULL}};
const Segment SGI87541[] = {{&S62987[105], 5}, {NULL}};
const Segment SGI87542[] = {{&S62987[35], 1}, {NULL}};
const Segment SGI87543[] = {{&S62987[78], 11}, {NULL}};
const Segment SGI87544[] = {{&S62572[42], 8}, {NULL}};
const Segment SGI87545[] = {{&S62572[42], 20}, {NULL}};
const Segment SGI87546[] = {{&S62572[30], 4}, {NULL}};
const Segment SGI87547[] = {{&S62572[111], 3}, {NULL}};
const Segment SGI87548[] = {{&S62572[76], 6}, {NULL}};
const Segment SGI87549[] = {{&S62573[36], 8}, {NULL}};
const Segment SGI87550[] = {{&S62573[31], 4}, {NULL}};
const Segment SGI87551[] = {{&S50364[44], 6}, {NULL}};
const Segment SGI87552[] = {{&S50364[44], 18}, {NULL}};
const Segment SGI87553[] = {{&S50364[30], 6}, {NULL}};
const Segment SGI87554[] = {{&S50364[31], 3}, {NULL}};
const Segment SGI87555[] = {{&S50364[76], 6}, {NULL}};
const Segment SGI87556[] = {{&S50365[38], 6}, {NULL}};
const Segment SGI87557[] = {{&S50365[31], 6}, {NULL}};
const Segment SGI87558[] = {{&S50374[34], 10}, {NULL}};
const Segment SGI87559[] = {{&S50374[27], 6}, {NULL}};
const Segment SGI87560[] = {{&S50374[97], 9}, {NULL}};
const Segment SGI87561[] = {{&S50374[70], 6}, {NULL}};
const Segment SGI87562[] = {{&S63082[44], 4}, {NULL}};
const Segment SGI87563[] = {{&S63082[44], 15}, {NULL}};
const Segment SGI87564[] = {{&S63082[30], 6}, {NULL}};
const Segment SGI87565[] = {{&S63082[107], 3}, {NULL}};
const Segment SGI87566[] = {{&S63082[73], 5}, {NULL}};
const Segment SGI87567[] = {{&S70796[31], 8}, {NULL}};
const Segment SGI87568[] = {{&S70796[54], 3}, {NULL}};
const Segment SGI87569[] = {{&S70850[44], 10}, {NULL}};
const Segment SGI87570[] = {{&S70850[44], 27}, {NULL}};
const Segment SGI87571[] = {{&S70850[30], 6}, {NULL}};
const Segment SGI87572[] = {{&S70850[126], 3}, {NULL}};
const Segment SGI87573[] = {{&S70850[92], 5}, {NULL}};
const Segment SGI87574[] = {{&S70850[71], 8}, {NULL}};
const Segment SGI87575[] = {{&S70851[44], 10}, {NULL}};
const Segment SGI87576[] = {{&S70851[44], 27}, {NULL}};
const Segment SGI87577[] = {{&S70851[30], 6}, {NULL}};
const Segment SGI87578[] = {{&S70851[119], 3}, {NULL}};
const Segment SGI87579[] = {{&S70851[85], 5}, {NULL}};
const Segment SGI87580[] = {{&S70851[143], 7}, {NULL}};
const Segment SGI87581[] = {{&S50467[34], 19}, {NULL}};
const Segment SGI87582[] = {{&S50467[27], 6}, {NULL}};
const Segment SGI87583[] = {{&S50467[106], 9}, {NULL}};
const Segment SGI87584[] = {{&S50467[79], 6}, {NULL}};
const Segment SGI87585[] = {{&S50469[34], 7}, {NULL}};
const Segment SGI87586[] = {{&S50469[27], 6}, {NULL}};
const Segment SGI87587[] = {{&S50469[94], 9}, {NULL}};
const Segment SGI87588[] = {{&S50469[67], 6}, {NULL}};
const Segment SGI87589[] = {{&S50473[34], 8}, {NULL}};
const Segment SGI87590[] = {{&S50473[27], 6}, {NULL}};
const Segment SGI87591[] = {{&S50473[97], 9}, {NULL}};
const Segment SGI87592[] = {{&S50473[70], 6}, {NULL}};
const Segment SGI87593[] = {{&S50474[34], 7}, {NULL}};
const Segment SGI87594[] = {{&S50474[27], 6}, {NULL}};
const Segment SGI87595[] = {{&S50474[96], 9}, {NULL}};
const Segment SGI87596[] = {{&S50474[69], 6}, {NULL}};
const Segment SGI87597[] = {{&S70872[45], 10}, {NULL}};
const Segment SGI87598[] = {{&S70872[30], 6}, {NULL}};
const Segment SGI87599[] = {{&S70872[120], 3}, {NULL}};
const Segment SGI87600[] = {{&S70872[86], 5}, {NULL}};
const Segment SGI87601[] = {{&S63083[38], 4}, {NULL}};
const Segment SGI87602[] = {{&S70872[144], 13}, {NULL}};
const Segment SGI87603[] = {{&S63083[31], 6}, {NULL}};
const Segment SGI87604[] = {{&S63087[44], 4}, {NULL}};
const Segment SGI87605[] = {{&S63087[44], 15}, {NULL}};
const Segment SGI87606[] = {{&S63087[30], 6}, {NULL}};
const Segment SGI87607[] = {{&S63087[107], 3}, {NULL}};
const Segment SGI87608[] = {{&S63087[73], 5}, {NULL}};
const Segment SGI87609[] = {{&S63098[44], 8}, {NULL}};
const Segment SGI87610[] = {{&S63098[44], 18}, {NULL}};
const Segment SGI87611[] = {{&S63098[30], 6}, {NULL}};
const Segment SGI87612[] = {{&S63098[110], 3}, {NULL}};
const Segment SGI87613[] = {{&S63098[76], 5}, {NULL}};
const Segment SGI87614[] = {{&S63099[38], 8}, {NULL}};
const Segment SGI87615[] = {{&S63099[31], 6}, {NULL}};
const Segment SGI87616[] = {{&S57981[77], 6}, {NULL}};
const Segment SGI87617[] = {{&S57981[0], 12}, {NULL}};
const Segment SGI87618[] = {{&S57988[44], 10}, {NULL}};
const Segment SGI87619[] = {{&S57988[44], 22}, {NULL}};
const Segment SGI87620[] = {{&S57988[30], 6}, {NULL}};
const Segment SGI87621[] = {{&S57988[31], 3}, {NULL}};
const Segment SGI87622[] = {{&S57988[80], 6}, {NULL}};
const Segment SGI87623[] = {{&S57988[0], 13}, {NULL}};
const Segment SGI87624[] = {{&S70922[44], 7}, {NULL}};
const Segment SGI87625[] = {{&S70922[44], 15}, {NULL}};
const Segment SGI87626[] = {{&S70922[30], 6}, {NULL}};
const Segment SGI87627[] = {{&S70922[107], 3}, {NULL}};
const Segment SGI87628[] = {{&S70922[73], 5}, {NULL}};
const Segment SGI87629[] = {{&S70923[38], 7}, {NULL}};
const Segment SGI87630[] = {{&S70923[31], 6}, {NULL}};
const Segment SGI87631[] = {{&S70969[44], 11}, {NULL}};
const Segment SGI87632[] = {{&S70969[44], 23}, {NULL}};
const Segment SGI87633[] = {{&S70969[30], 6}, {NULL}};
const Segment SGI87634[] = {{&S70969[115], 3}, {NULL}};
const Segment SGI87635[] = {{&S70969[81], 5}, {NULL}};
const Segment SGI87636[] = {{&S70971[44], 13}, {NULL}};
const Segment SGI87637[] = {{&S70971[44], 25}, {NULL}};
const Segment SGI87638[] = {{&S70971[30], 6}, {NULL}};
const Segment SGI87639[] = {{&S70971[117], 3}, {NULL}};
const Segment SGI87640[] = {{&S70971[83], 5}, {NULL}};
const Segment SGI87641[] = {{&S70972[44], 13}, {NULL}};
const Segment SGI87642[] = {{&S70972[44], 25}, {NULL}};
const Segment SGI87643[] = {{&S70972[30], 6}, {NULL}};
const Segment SGI87644[] = {{&S64382[45], 12}, {NULL}};
const Segment SGI87645[] = {{&S64382[45], 22}, {NULL}};
const Segment SGI87646[] = {{&S70972[86], 4}, {NULL}};
const Segment SGI87647[] = {{&S64382[30], 7}, {NULL}};
const Segment SGI87648[] = {{&S70972[86], 3}, {NULL}};
const Segment SGI87649[] = {{&S64382[115], 3}, {NULL}};
const Segment SGI87650[] = {{&S64382[81], 5}, {NULL}};
const Segment SGI87651[] = {{&S70973[44], 11}, {NULL}};
const Segment SGI87652[] = {{&S70973[44], 23}, {NULL}};
const Segment SGI87653[] = {{&S70973[30], 6}, {NULL}};
const Segment SGI87654[] = {{&S70973[115], 3}, {NULL}};
const Segment SGI87655[] = {{&S70973[81], 5}, {NULL}};
const Segment SGI87656[] = {{&S70974[44], 13}, {NULL}};
const Segment SGI87657[] = {{&S70974[44], 25}, {NULL}};
const Segment SGI87658[] = {{&S70974[30], 6}, {NULL}};
const Segment SGI87659[] = {{&S70974[86], 6}, {NULL}};
const Segment SGI87660[] = {{&S70975[44], 13}, {NULL}};
const Segment SGI87661[] = {{&S70975[44], 25}, {NULL}};
const Segment SGI87662[] = {{&S70975[30], 6}, {NULL}};
const Segment SGI87663[] = {{&S70975[117], 3}, {NULL}};
const Segment SGI87664[] = {{&S70975[83], 5}, {NULL}};
const Segment SGI87665[] = {{&S70976[44], 11}, {NULL}};
const Segment SGI87666[] = {{&S70976[44], 23}, {NULL}};
const Segment SGI87667[] = {{&S70976[30], 6}, {NULL}};
const Segment SGI87668[] = {{&S70976[115], 3}, {NULL}};
const Segment SGI87669[] = {{&S70976[81], 5}, {NULL}};
const Segment SGI87670[] = {{&S50632[44], 4}, {NULL}};
const Segment SGI87671[] = {{&S50632[44], 26}, {NULL}};
const Segment SGI87672[] = {{&S50632[30], 6}, {NULL}};
const Segment SGI87673[] = {{&S50632[31], 3}, {NULL}};
const Segment SGI87674[] = {{&S50632[84], 6}, {NULL}};
const Segment SGI87675[] = {{&S50633[42], 4}, {NULL}};
const Segment SGI87676[] = {{&S50633[30], 6}, {NULL}};
const Segment SGI87677[] = {{&S50633[31], 3}, {NULL}};
const Segment SGI87678[] = {{&S50633[82], 6}, {NULL}};
const Segment SGI87679[] = {{&S50634[38], 4}, {NULL}};
const Segment SGI87680[] = {{&S50634[31], 6}, {NULL}};
const Segment SGI87681[] = {{&S71031[44], 10}, {NULL}};
const Segment SGI87682[] = {{&S71031[44], 27}, {NULL}};
const Segment SGI87683[] = {{&S71031[30], 6}, {NULL}};
const Segment SGI87684[] = {{&S71031[88], 4}, {NULL}};
const Segment SGI87685[] = {{&S71031[88], 3}, {NULL}};
const Segment SGI87686[] = {{&S71033[44], 10}, {NULL}};
const Segment SGI87687[] = {{&S71033[44], 27}, {NULL}};
const Segment SGI87688[] = {{&S71033[30], 6}, {NULL}};
const Segment SGI87689[] = {{&S71033[119], 3}, {NULL}};
const Segment SGI87690[] = {{&S71033[85], 5}, {NULL}};
const Segment SGI87691[] = {{&S71034[44], 10}, {NULL}};
const Segment SGI87692[] = {{&S71034[44], 27}, {NULL}};
const Segment SGI87693[] = {{&S71034[30], 6}, {NULL}};
const Segment SGI87694[] = {{&S71034[88], 6}, {NULL}};
const Segment SGI87695[] = {{&S50732[44], 9}, {NULL}};
const Segment SGI87696[] = {{&S50732[44], 21}, {NULL}};
const Segment SGI87697[] = {{&S50732[30], 6}, {NULL}};
const Segment SGI87698[] = {{&S50732[31], 3}, {NULL}};
const Segment SGI87699[] = {{&S50732[79], 6}, {NULL}};
const Segment SGI87700[] = {{&S50733[38], 9}, {NULL}};
const Segment SGI87701[] = {{&S50733[31], 6}, {NULL}};
const Segment SGI87702[] = {{&S50736[34], 21}, {NULL}};
const Segment SGI87703[] = {{&S50736[27], 6}, {NULL}};
const Segment SGI87704[] = {{&S50736[108], 9}, {NULL}};
const Segment SGI87705[] = {{&S50736[81], 6}, {NULL}};
const Segment SGI87706[] = {{&S50737[34], 18}, {NULL}};
const Segment SGI87707[] = {{&S71112[44], 10}, {NULL}};
const Segment SGI87708[] = {{&S71112[44], 22}, {NULL}};
const Segment SGI87709[] = {{&S50737[27], 6}, {NULL}};
const Segment SGI87710[] = {{&S71112[30], 6}, {NULL}};
const Segment SGI87711[] = {{&S71112[114], 3}, {NULL}};
const Segment SGI87712[] = {{&S50737[105], 9}, {NULL}};
const Segment SGI87713[] = {{&S50737[78], 6}, {NULL}};
const Segment SGI87714[] = {{&S71112[80], 5}, {NULL}};
const Segment SGI87715[] = {{&S50738[34], 35}, {NULL}};
const Segment SGI87716[] = {{&S50738[27], 6}, {NULL}};
const Segment SGI87717[] = {{&S71113[44], 10}, {NULL}};
const Segment SGI87718[] = {{&S71113[44], 22}, {NULL}};
const Segment SGI87719[] = {{&S71113[30], 6}, {NULL}};
const Segment SGI87720[] = {{&S50738[122], 9}, {NULL}};
const Segment SGI87721[] = {{&S50738[95], 6}, {NULL}};
const Segment SGI87722[] = {{&S71113[83], 6}, {NULL}};
const Segment SGI87723[] = {{&S71114[38], 10}, {NULL}};
const Segment SGI87724[] = {{&S71114[31], 6}, {NULL}};
const Segment SGI87725[] = {{&S63343[44], 11}, {NULL}};
const Segment SGI87726[] = {{&S63343[44], 21}, {NULL}};
const Segment SGI87727[] = {{&S63343[30], 6}, {NULL}};
const Segment SGI87728[] = {{&S63343[113], 3}, {NULL}};
const Segment SGI87729[] = {{&S63343[79], 5}, {NULL}};
const Segment SGI87730[] = {{&S63346[38], 11}, {NULL}};
const Segment SGI87731[] = {{&S63346[31], 6}, {NULL}};
const Segment SGI87732[] = {{&S63347[44], 11}, {NULL}};
const Segment SGI87733[] = {{&S63347[44], 21}, {NULL}};
const Segment SGI87734[] = {{&S63347[30], 6}, {NULL}};
const Segment SGI87735[] = {{&S63347[82], 4}, {NULL}};
const Segment SGI87736[] = {{&S63347[82], 3}, {NULL}};
const Segment SGI87737[] = {{&S50818[44], 9}, {NULL}};
const Segment SGI87738[] = {{&S50818[44], 21}, {NULL}};
const Segment SGI87739[] = {{&S50818[30], 6}, {NULL}};
const Segment SGI87740[] = {{&S50818[31], 3}, {NULL}};
const Segment SGI87741[] = {{&S50818[79], 6}, {NULL}};
const Segment SGI87742[] = {{&S50824[38], 9}, {NULL}};
const Segment SGI87743[] = {{&S50824[31], 6}, {NULL}};
const Segment SGI87744[] = {{&S71207[44], 10}, {NULL}};
const Segment SGI87745[] = {{&S71207[44], 27}, {NULL}};
const Segment SGI87746[] = {{&S71207[30], 6}, {NULL}};
const Segment SGI87747[] = {{&S71207[85], 6}, {NULL}};
const Segment SGI87748[] = {{&S71207[112], 13}, {NULL}};
const Segment SGI87749[] = {{&S50868[44], 26}, {NULL}};
const Segment SGI87750[] = {{&S50868[44], 38}, {NULL}};
const Segment SGI87751[] = {{&S50868[30], 6}, {NULL}};
const Segment SGI87752[] = {{&S50868[31], 3}, {NULL}};
const Segment SGI87753[] = {{&S50868[96], 6}, {NULL}};
const Segment SGI87754[] = {{&S50869[44], 12}, {NULL}};
const Segment SGI87755[] = {{&S50869[44], 24}, {NULL}};
const Segment SGI87756[] = {{&S50869[30], 6}, {NULL}};
const Segment SGI87757[] = {{&S50869[31], 3}, {NULL}};
const Segment SGI87758[] = {{&S50869[82], 6}, {NULL}};
const Segment SGI87759[] = {{&S50870[44], 12}, {NULL}};
const Segment SGI87760[] = {{&S50870[44], 23}, {NULL}};
const Segment SGI87761[] = {{&S50870[30], 6}, {NULL}};
const Segment SGI87762[] = {{&S50870[31], 3}, {NULL}};
const Segment SGI87763[] = {{&S50870[81], 6}, {NULL}};
const Segment SGI87764[] = {{&S50872[44], 12}, {NULL}};
const Segment SGI87765[] = {{&S50872[44], 23}, {NULL}};
const Segment SGI87766[] = {{&S50872[30], 6}, {NULL}};
const Segment SGI87767[] = {{&S50872[31], 3}, {NULL}};
const Segment SGI87768[] = {{&S50872[81], 6}, {NULL}};
const Segment SGI87769[] = {{&S50875[44], 12}, {NULL}};
const Segment SGI87770[] = {{&S50875[44], 23}, {NULL}};
const Segment SGI87771[] = {{&S50875[30], 6}, {NULL}};
const Segment SGI87772[] = {{&S50875[117], 3}, {NULL}};
const Segment SGI87773[] = {{&S50875[81], 7}, {NULL}};
const Segment SGI87774[] = {{&S50875[137], 8}, {NULL}};
const Segment SGI87775[] = {{&S50881[38], 12}, {NULL}};
const Segment SGI87776[] = {{&S50881[31], 6}, {NULL}};
const Segment SGI87777[] = {{&S63477[44], 8}, {NULL}};
const Segment SGI87778[] = {{&S63477[44], 17}, {NULL}};
const Segment SGI87779[] = {{&S63477[30], 6}, {NULL}};
const Segment SGI87780[] = {{&S63477[128], 3}, {NULL}};
const Segment SGI87781[] = {{&S63477[94], 5}, {NULL}};
const Segment SGI87782[] = {{&S63477[35], 1}, {NULL}};
const Segment SGI87783[] = {{&S63477[63], 11}, {NULL}};
const Segment SGI87784[] = {{&S50899[44], 10}, {NULL}};
const Segment SGI87785[] = {{&S50899[44], 22}, {NULL}};
const Segment SGI87786[] = {{&S50899[30], 6}, {NULL}};
const Segment SGI87787[] = {{&S50899[31], 3}, {NULL}};
const Segment SGI87788[] = {{&S50899[80], 6}, {NULL}};
const Segment SGI87789[] = {{&S50900[44], 10}, {NULL}};
const Segment SGI87790[] = {{&S50900[44], 22}, {NULL}};
const Segment SGI87791[] = {{&S50900[30], 6}, {NULL}};
const Segment SGI87792[] = {{&S50900[124], 3}, {NULL}};
const Segment SGI87793[] = {{&S50900[89], 6}, {NULL}};
const Segment SGI87794[] = {{&S50900[144], 8}, {NULL}};
const Segment SGI87795[] = {{&S50901[38], 10}, {NULL}};
const Segment SGI87796[] = {{&S50901[31], 6}, {NULL}};
const Segment SGI87797[] = {{&S50902[44], 10}, {NULL}};
const Segment SGI87798[] = {{&S50902[44], 22}, {NULL}};
const Segment SGI87799[] = {{&S50902[30], 6}, {NULL}};
const Segment SGI87800[] = {{&S50902[83], 6}, {NULL}};
const Segment SGI87801[] = {{&S50904[44], 10}, {NULL}};
const Segment SGI87802[] = {{&S50904[44], 33}, {NULL}};
const Segment SGI87803[] = {{&S50904[30], 6}, {NULL}};
const Segment SGI87804[] = {{&S50904[31], 3}, {NULL}};
const Segment SGI87805[] = {{&S50904[91], 6}, {NULL}};
const Segment SGI87806[] = {{&S50905[44], 10}, {NULL}};
const Segment SGI87807[] = {{&S50905[44], 34}, {NULL}};
const Segment SGI87808[] = {{&S50905[30], 6}, {NULL}};
const Segment SGI87809[] = {{&S50905[31], 3}, {NULL}};
const Segment SGI87810[] = {{&S50905[92], 6}, {NULL}};
const Segment SGI87811[] = {{&S50906[44], 10}, {NULL}};
const Segment SGI87812[] = {{&S50906[44], 34}, {NULL}};
const Segment SGI87813[] = {{&S50906[30], 6}, {NULL}};
const Segment SGI87814[] = {{&S50906[31], 3}, {NULL}};
const Segment SGI87815[] = {{&S50906[92], 6}, {NULL}};
const Segment SGI87816[] = {{&S50907[44], 10}, {NULL}};
const Segment SGI87817[] = {{&S50907[44], 34}, {NULL}};
const Segment SGI87818[] = {{&S50907[30], 6}, {NULL}};
const Segment SGI87819[] = {{&S50907[31], 3}, {NULL}};
const Segment SGI87820[] = {{&S50907[92], 6}, {NULL}};
const Segment SGI87821[] = {{&S50908[44], 10}, {NULL}};
const Segment SGI87822[] = {{&S50908[44], 33}, {NULL}};
const Segment SGI87823[] = {{&S50908[30], 6}, {NULL}};
const Segment SGI87824[] = {{&S50908[31], 3}, {NULL}};
const Segment SGI87825[] = {{&S50908[91], 6}, {NULL}};
const Segment SGI87826[] = {{&S50909[44], 10}, {NULL}};
const Segment SGI87827[] = {{&S50909[44], 37}, {NULL}};
const Segment SGI87828[] = {{&S50909[30], 6}, {NULL}};
const Segment SGI87829[] = {{&S50909[31], 3}, {NULL}};
const Segment SGI87830[] = {{&S50909[95], 6}, {NULL}};
const Segment SGI87831[] = {{&S50910[44], 10}, {NULL}};
const Segment SGI87832[] = {{&S50910[44], 34}, {NULL}};
const Segment SGI87833[] = {{&S50910[30], 6}, {NULL}};
const Segment SGI87834[] = {{&S50910[31], 3}, {NULL}};
const Segment SGI87835[] = {{&S50910[92], 6}, {NULL}};
const Segment SGI87836[] = {{&S50912[44], 10}, {NULL}};
const Segment SGI87837[] = {{&S50912[44], 22}, {NULL}};
const Segment SGI87838[] = {{&S50912[30], 6}, {NULL}};
const Segment SGI87839[] = {{&S50912[31], 3}, {NULL}};
const Segment SGI87840[] = {{&S50912[80], 6}, {NULL}};
const Segment SGI87841[] = {{&S50913[44], 10}, {NULL}};
const Segment SGI87842[] = {{&S50913[44], 32}, {NULL}};
const Segment SGI87843[] = {{&S50913[30], 6}, {NULL}};
const Segment SGI87844[] = {{&S50913[31], 3}, {NULL}};
const Segment SGI87845[] = {{&S50913[90], 6}, {NULL}};
const Segment SGI87846[] = {{&S50914[44], 10}, {NULL}};
const Segment SGI87847[] = {{&S50914[44], 34}, {NULL}};
const Segment SGI87848[] = {{&S50914[30], 6}, {NULL}};
const Segment SGI87849[] = {{&S50914[31], 3}, {NULL}};
const Segment SGI87850[] = {{&S50914[92], 6}, {NULL}};
const Segment SGI87851[] = {{&S50926[44], 13}, {NULL}};
const Segment SGI87852[] = {{&S50926[44], 25}, {NULL}};
const Segment SGI87853[] = {{&S50926[30], 6}, {NULL}};
const Segment SGI87854[] = {{&S50926[31], 3}, {NULL}};
const Segment SGI87855[] = {{&S50926[83], 6}, {NULL}};
const Segment SGI87856[] = {{&S50927[44], 10}, {NULL}};
const Segment SGI87857[] = {{&S50927[44], 22}, {NULL}};
const Segment SGI87858[] = {{&S50927[30], 6}, {NULL}};
const Segment SGI87859[] = {{&S50927[83], 6}, {NULL}};
const Segment SGI87860[] = {{&S50931[44], 10}, {NULL}};
const Segment SGI87861[] = {{&S50931[44], 22}, {NULL}};
const Segment SGI87862[] = {{&S50931[30], 6}, {NULL}};
const Segment SGI87863[] = {{&S50931[31], 3}, {NULL}};
const Segment SGI87864[] = {{&S50931[80], 6}, {NULL}};
const Segment SGI87865[] = {{&S71369[44], 15}, {NULL}};
const Segment SGI87866[] = {{&S71369[44], 27}, {NULL}};
const Segment SGI87867[] = {{&S71369[30], 6}, {NULL}};
const Segment SGI87868[] = {{&S71369[119], 3}, {NULL}};
const Segment SGI87869[] = {{&S71369[85], 5}, {NULL}};
const Segment SGI87870[] = {{&S51005[34], 10}, {NULL}};
const Segment SGI87871[] = {{&S51005[27], 6}, {NULL}};
const Segment SGI87872[] = {{&S51005[97], 9}, {NULL}};
const Segment SGI87873[] = {{&S51005[70], 6}, {NULL}};
const Segment SGI87874[] = {{&S71410[44], 12}, {NULL}};
const Segment SGI87875[] = {{&S71410[44], 24}, {NULL}};
const Segment SGI87876[] = {{&S71410[30], 6}, {NULL}};
const Segment SGI87877[] = {{&S71410[116], 3}, {NULL}};
const Segment SGI87878[] = {{&S71410[82], 5}, {NULL}};
const Segment SGI87879[] = {{&S71411[38], 12}, {NULL}};
const Segment SGI87880[] = {{&S71411[31], 6}, {NULL}};
const Segment SGI87881[] = {{&S63660[44], 7}, {NULL}};
const Segment SGI87882[] = {{&S63660[44], 20}, {NULL}};
const Segment SGI87883[] = {{&S63660[30], 6}, {NULL}};
const Segment SGI87884[] = {{&S63660[129], 3}, {NULL}};
const Segment SGI87885[] = {{&S63660[95], 5}, {NULL}};
const Segment SGI87886[] = {{&S63660[66], 3}, {NULL}};
const Segment SGI87887[] = {{&S63661[44], 14}, {NULL}};
const Segment SGI87888[] = {{&S63661[44], 26}, {NULL}};
const Segment SGI87889[] = {{&S63661[30], 6}, {NULL}};
const Segment SGI87890[] = {{&S63661[137], 3}, {NULL}};
const Segment SGI87891[] = {{&S63661[103], 5}, {NULL}};
const Segment SGI87892[] = {{&S63661[35], 1}, {NULL}};
const Segment SGI87893[] = {{&S63661[72], 11}, {NULL}};
const Segment SGI87894[] = {{&S63662[44], 7}, {NULL}};
const Segment SGI87895[] = {{&S63662[44], 24}, {NULL}};
const Segment SGI87896[] = {{&S63662[30], 6}, {NULL}};
const Segment SGI87897[] = {{&S63662[140], 3}, {NULL}};
const Segment SGI87898[] = {{&S63662[106], 5}, {NULL}};
const Segment SGI87899[] = {{&S63662[35], 1}, {NULL}};
const Segment SGI87900[] = {{&S63662[70], 11}, {NULL}};
const Segment SGI87901[] = {{&S63663[44], 7}, {NULL}};
const Segment SGI87902[] = {{&S63663[44], 28}, {NULL}};
const Segment SGI87903[] = {{&S63663[30], 6}, {NULL}};
const Segment SGI87904[] = {{&S63663[137], 3}, {NULL}};
const Segment SGI87905[] = {{&S63663[103], 5}, {NULL}};
const Segment SGI87906[] = {{&S63663[57], 11}, {&S63663[57], 11}, {NULL}};
const Segment SGI87907[] = {{&S63664[44], 7}, {NULL}};
const Segment SGI87908[] = {{&S63664[44], 30}, {NULL}};
const Segment SGI87909[] = {{&S63664[30], 6}, {NULL}};
const Segment SGI87910[] = {{&S63664[139], 3}, {NULL}};
const Segment SGI87911[] = {{&S63664[105], 5}, {NULL}};
const Segment SGI87912[] = {{&S63664[57], 11}, {&S63664[57], 11}, {NULL}};
const Segment SGI87913[] = {{&S71451[44], 5}, {NULL}};
const Segment SGI87914[] = {{&S71451[44], 27}, {NULL}};
const Segment SGI87915[] = {{&S71451[30], 6}, {NULL}};
const Segment SGI87916[] = {{&S71451[119], 3}, {NULL}};
const Segment SGI87917[] = {{&S71451[85], 5}, {NULL}};
const Segment SGI87918[] = {{&S51077[44], 13}, {NULL}};
const Segment SGI87919[] = {{&S51077[44], 25}, {NULL}};
const Segment SGI87920[] = {{&S51077[30], 6}, {NULL}};
const Segment SGI87921[] = {{&S51077[31], 3}, {NULL}};
const Segment SGI87922[] = {{&S51077[83], 6}, {NULL}};
const Segment SGI87923[] = {{&S51078[44], 11}, {NULL}};
const Segment SGI87924[] = {{&S51078[44], 23}, {NULL}};
const Segment SGI87925[] = {{&S51078[30], 6}, {NULL}};
const Segment SGI87926[] = {{&S51078[31], 3}, {NULL}};
const Segment SGI87927[] = {{&S51078[81], 6}, {NULL}};
const Segment SGI87928[] = {{&S51079[44], 11}, {NULL}};
const Segment SGI87929[] = {{&S51079[44], 23}, {NULL}};
const Segment SGI87930[] = {{&S51079[30], 6}, {NULL}};
const Segment SGI87931[] = {{&S51079[31], 3}, {NULL}};
const Segment SGI87932[] = {{&S51079[81], 6}, {NULL}};
const Segment SGI87933[] = {{&S51080[44], 8}, {NULL}};
const Segment SGI87934[] = {{&S51080[44], 20}, {NULL}};
const Segment SGI87935[] = {{&S51080[30], 6}, {NULL}};
const Segment SGI87936[] = {{&S51080[31], 3}, {NULL}};
const Segment SGI87937[] = {{&S51080[78], 6}, {NULL}};
const Segment SGI87938[] = {{&S51081[44], 9}, {NULL}};
const Segment SGI87939[] = {{&S51081[44], 21}, {NULL}};
const Segment SGI87940[] = {{&S51081[30], 6}, {NULL}};
const Segment SGI87941[] = {{&S51081[31], 3}, {NULL}};
const Segment SGI87942[] = {{&S51081[79], 6}, {NULL}};
const Segment SGI87943[] = {{&S71492[44], 21}, {NULL}};
const Segment SGI87944[] = {{&S71492[44], 48}, {NULL}};
const Segment SGI87945[] = {{&S71492[30], 6}, {NULL}};
const Segment SGI87946[] = {{&S71492[140], 3}, {NULL}};
const Segment SGI87947[] = {{&S71492[106], 5}, {NULL}};
const Segment SGI87948[] = {{&S71493[44], 21}, {NULL}};
const Segment SGI87949[] = {{&S71493[44], 49}, {NULL}};
const Segment SGI87950[] = {{&S71493[30], 6}, {NULL}};
const Segment SGI87951[] = {{&S71493[141], 3}, {NULL}};
const Segment SGI87952[] = {{&S71493[107], 5}, {NULL}};
const Segment SGI87953[] = {{&S71494[44], 21}, {NULL}};
const Segment SGI87954[] = {{&S71494[44], 49}, {NULL}};
const Segment SGI87955[] = {{&S71494[30], 6}, {NULL}};
const Segment SGI87956[] = {{&S71494[141], 3}, {NULL}};
const Segment SGI87957[] = {{&S71494[107], 5}, {NULL}};
const Segment SGI87958[] = {{&S51113[34], 10}, {NULL}};
const Segment SGI87959[] = {{&S51113[27], 6}, {NULL}};
const Segment SGI87960[] = {{&S51113[97], 9}, {NULL}};
const Segment SGI87961[] = {{&S51113[70], 6}, {NULL}};
const Segment SGI87962[] = {{&S51125[44], 18}, {NULL}};
const Segment SGI87963[] = {{&S51125[44], 30}, {NULL}};
const Segment SGI87964[] = {{&S51125[30], 6}, {NULL}};
const Segment SGI87965[] = {{&S51125[31], 3}, {NULL}};
const Segment SGI87966[] = {{&S51125[88], 6}, {NULL}};
const Segment SGI87967[] = {{&S51126[38], 18}, {NULL}};
const Segment SGI87968[] = {{&S51126[31], 6}, {NULL}};
const Segment SGI87969[] = {{&S71579[44], 8}, {NULL}};
const Segment SGI87970[] = {{&S71579[44], 19}, {NULL}};
const Segment SGI87971[] = {{&S71579[30], 6}, {NULL}};
const Segment SGI87972[] = {{&S71579[111], 3}, {NULL}};
const Segment SGI87973[] = {{&S71579[77], 5}, {NULL}};
const Segment SGI87974[] = {{&S71581[44], 11}, {NULL}};
const Segment SGI87975[] = {{&S71581[44], 23}, {NULL}};
const Segment SGI87976[] = {{&S71581[30], 6}, {NULL}};
const Segment SGI87977[] = {{&S71581[115], 3}, {NULL}};
const Segment SGI87978[] = {{&S71581[81], 5}, {NULL}};
const Segment SGI87979[] = {{&S71596[44], 12}, {NULL}};
const Segment SGI87980[] = {{&S71596[44], 24}, {NULL}};
const Segment SGI87981[] = {{&S71596[30], 6}, {NULL}};
const Segment SGI87982[] = {{&S71596[116], 3}, {NULL}};
const Segment SGI87983[] = {{&S71596[82], 5}, {NULL}};
const Segment SGI87984[] = {{&S71597[44], 12}, {NULL}};
const Segment SGI87985[] = {{&S71597[44], 24}, {NULL}};
const Segment SGI87986[] = {{&S71597[30], 6}, {NULL}};
const Segment SGI87987[] = {{&S71597[116], 3}, {NULL}};
const Segment SGI87988[] = {{&S71597[82], 5}, {NULL}};
const Segment SGI87989[] = {{&S51221[34], 18}, {NULL}};
const Segment SGI87990[] = {{&S51221[27], 6}, {NULL}};
const Segment SGI87991[] = {{&S51221[105], 9}, {NULL}};
const Segment SGI87992[] = {{&S51221[78], 6}, {NULL}};
const Segment SGI87993[] = {{&S51222[34], 21}, {NULL}};
const Segment SGI87994[] = {{&S51222[27], 6}, {NULL}};
const Segment SGI87995[] = {{&S51222[108], 9}, {NULL}};
const Segment SGI87996[] = {{&S51222[81], 6}, {NULL}};
const Segment SGI87997[] = {{&S51224[34], 35}, {NULL}};
const Segment SGI87998[] = {{&S51224[27], 6}, {NULL}};
const Segment SGI87999[] = {{&S51224[122], 9}, {NULL}};
const Segment SGI88000[] = {{&S51224[95], 6}, {NULL}};
const Segment SGI88001[] = {{&S71634[44], 10}, {NULL}};
const Segment SGI88002[] = {{&S71634[44], 27}, {NULL}};
const Segment SGI88003[] = {{&S71634[30], 6}, {NULL}};
const Segment SGI88004[] = {{&S71634[85], 6}, {NULL}};
const Segment SGI88005[] = {{&S71634[112], 13}, {NULL}};
const Segment SGI88006[] = {{&S51268[44], 6}, {NULL}};
const Segment SGI88007[] = {{&S51268[44], 18}, {NULL}};
const Segment SGI88008[] = {{&S51268[30], 6}, {NULL}};
const Segment SGI88009[] = {{&S51268[31], 3}, {NULL}};
const Segment SGI88010[] = {{&S51268[76], 6}, {NULL}};
const Segment SGI88011[] = {{&S51271[44], 7}, {NULL}};
const Segment SGI88012[] = {{&S51271[44], 19}, {NULL}};
const Segment SGI88013[] = {{&S51271[30], 6}, {NULL}};
const Segment SGI88014[] = {{&S51271[31], 3}, {NULL}};
const Segment SGI88015[] = {{&S51271[77], 6}, {NULL}};
const Segment SGI88016[] = {{&S71659[45], 10}, {NULL}};
const Segment SGI88017[] = {{&S71659[30], 6}, {NULL}};
const Segment SGI88018[] = {{&S71659[120], 3}, {NULL}};
const Segment SGI88019[] = {{&S71659[86], 5}, {NULL}};
const Segment SGI88020[] = {{&S71659[144], 13}, {NULL}};
const Segment SGI88021[] = {{&S71660[45], 11}, {NULL}};
const Segment SGI88022[] = {{&S71660[30], 6}, {NULL}};
const Segment SGI88023[] = {{&S71660[121], 3}, {NULL}};
const Segment SGI88024[] = {{&S71660[87], 5}, {NULL}};
const Segment SGI88025[] = {{&S71660[145], 13}, {NULL}};
const Segment SGI88026[] = {{&S51292[34], 16}, {NULL}};
const Segment SGI88027[] = {{&S51292[27], 6}, {NULL}};
const Segment SGI88028[] = {{&S51292[103], 9}, {NULL}};
const Segment SGI88029[] = {{&S51292[76], 6}, {NULL}};
const Segment SGI88030[] = {{&S51293[34], 11}, {NULL}};
const Segment SGI88031[] = {{&S51293[27], 6}, {NULL}};
const Segment SGI88032[] = {{&S51293[98], 9}, {NULL}};
const Segment SGI88033[] = {{&S51293[71], 6}, {NULL}};
const Segment SGI88034[] = {{&S51294[34], 11}, {NULL}};
const Segment SGI88035[] = {{&S51294[27], 6}, {NULL}};
const Segment SGI88036[] = {{&S51294[98], 9}, {NULL}};
const Segment SGI88037[] = {{&S51294[71], 6}, {NULL}};
const Segment SGI88038[] = {{&S51295[34], 21}, {NULL}};
const Segment SGI88039[] = {{&S51295[27], 6}, {NULL}};
const Segment SGI88040[] = {{&S51295[108], 9}, {NULL}};
const Segment SGI88041[] = {{&S51295[81], 6}, {NULL}};
const Segment SGI88042[] = {{&S51296[34], 15}, {NULL}};
const Segment SGI88043[] = {{&S51296[27], 6}, {NULL}};
const Segment SGI88044[] = {{&S51296[102], 9}, {NULL}};
const Segment SGI88045[] = {{&S51296[75], 6}, {NULL}};
const Segment SGI88046[] = {{&S51299[34], 20}, {NULL}};
const Segment SGI88047[] = {{&S51299[27], 6}, {NULL}};
const Segment SGI88048[] = {{&S51299[107], 9}, {NULL}};
const Segment SGI88049[] = {{&S51299[80], 6}, {NULL}};
const Segment SGI88050[] = {{&S71689[59], 5}, {NULL}};
const Segment SGI88051[] = {{&S71689[59], 23}, {NULL}};
const Segment SGI88052[] = {{&S71689[45], 6}, {NULL}};
const Segment SGI88053[] = {{&S71689[130], 3}, {NULL}};
const Segment SGI88054[] = {{&S71689[96], 5}, {NULL}};
const Segment SGI88055[] = {{&S71689[0], 14}, {NULL}};
const Segment SGI88056[] = {{&S51478[34], 8}, {NULL}};
const Segment SGI88057[] = {{&S51478[27], 6}, {NULL}};
const Segment SGI88058[] = {{&S51478[100], 9}, {NULL}};
const Segment SGI88059[] = {{&S51478[73], 6}, {NULL}};
const Segment SGI88060[] = {{&S51479[34], 7}, {NULL}};
const Segment SGI88061[] = {{&S51479[27], 6}, {NULL}};
const Segment SGI88062[] = {{&S51479[99], 9}, {NULL}};
const Segment SGI88063[] = {{&S51479[72], 6}, {NULL}};
const Segment SGI88064[] = {{&S51481[34], 7}, {NULL}};
const Segment SGI88065[] = {{&S51481[27], 6}, {NULL}};
const Segment SGI88066[] = {{&S51481[99], 9}, {NULL}};
const Segment SGI88067[] = {{&S51481[72], 6}, {NULL}};
const Segment SGI88068[] = {{&S51510[34], 10}, {NULL}};
const Segment SGI88069[] = {{&S51510[27], 6}, {NULL}};
const Segment SGI88070[] = {{&S51510[97], 9}, {NULL}};
const Segment SGI88071[] = {{&S51510[70], 6}, {NULL}};
const Segment SGI88072[] = {{&S51513[44], 6}, {NULL}};
const Segment SGI88073[] = {{&S51513[44], 18}, {NULL}};
const Segment SGI88074[] = {{&S51513[30], 6}, {NULL}};
const Segment SGI88075[] = {{&S51513[31], 3}, {NULL}};
const Segment SGI88076[] = {{&S51513[76], 6}, {NULL}};
const Segment SGI88077[] = {{&S71916[44], 15}, {NULL}};
const Segment SGI88078[] = {{&S71916[44], 26}, {NULL}};
const Segment SGI88079[] = {{&S71916[30], 6}, {NULL}};
const Segment SGI88080[] = {{&S71916[118], 3}, {NULL}};
const Segment SGI88081[] = {{&S71916[84], 5}, {NULL}};
const Segment SGI88082[] = {{&S51514[38], 6}, {NULL}};
const Segment SGI88083[] = {{&S51514[31], 6}, {NULL}};
const Segment SGI88084[] = {{&S51516[42], 6}, {NULL}};
const Segment SGI88085[] = {{&S51516[30], 6}, {NULL}};
const Segment SGI88086[] = {{&S51516[31], 3}, {NULL}};
const Segment SGI88087[] = {{&S51516[74], 6}, {NULL}};
const Segment SGI88088[] = {{&S71961[44], 6}, {NULL}};
const Segment SGI88089[] = {{&S71961[44], 18}, {NULL}};
const Segment SGI88090[] = {{&S71961[30], 6}, {NULL}};
const Segment SGI88091[] = {{&S71961[110], 3}, {NULL}};
const Segment SGI88092[] = {{&S71961[76], 5}, {NULL}};
const Segment SGI88093[] = {{&S71962[44], 13}, {NULL}};
const Segment SGI88094[] = {{&S71962[44], 25}, {NULL}};
const Segment SGI88095[] = {{&S71962[30], 6}, {NULL}};
const Segment SGI88096[] = {{&S71962[117], 3}, {NULL}};
const Segment SGI88097[] = {{&S71962[83], 5}, {NULL}};
const Segment SGI88098[] = {{&S71963[44], 12}, {NULL}};
const Segment SGI88099[] = {{&S71963[44], 24}, {NULL}};
const Segment SGI88100[] = {{&S71963[30], 6}, {NULL}};
const Segment SGI88101[] = {{&S71963[116], 3}, {NULL}};
const Segment SGI88102[] = {{&S71963[82], 5}, {NULL}};
const Segment SGI88103[] = {{&S71964[44], 14}, {NULL}};
const Segment SGI88104[] = {{&S71964[44], 26}, {NULL}};
const Segment SGI88105[] = {{&S71964[30], 6}, {NULL}};
const Segment SGI88106[] = {{&S71964[118], 3}, {NULL}};
const Segment SGI88107[] = {{&S71964[84], 5}, {NULL}};
const Segment SGI88108[] = {{&S51569[34], 14}, {NULL}};
const Segment SGI88109[] = {{&S51569[27], 6}, {NULL}};
const Segment SGI88110[] = {{&S51569[106], 9}, {NULL}};
const Segment SGI88111[] = {{&S51569[79], 6}, {NULL}};
const Segment SGI88112[] = {{&S51570[34], 14}, {NULL}};
const Segment SGI88113[] = {{&S51570[27], 6}, {NULL}};
const Segment SGI88114[] = {{&S51570[106], 9}, {NULL}};
const Segment SGI88115[] = {{&S51570[79], 6}, {NULL}};
const Segment SGI88116[] = {{&S51572[34], 21}, {NULL}};
const Segment SGI88117[] = {{&S51572[27], 6}, {NULL}};
const Segment SGI88118[] = {{&S51572[113], 9}, {NULL}};
const Segment SGI88119[] = {{&S51572[86], 6}, {NULL}};
const Segment SGI88120[] = {{&S51573[34], 15}, {NULL}};
const Segment SGI88121[] = {{&S51573[27], 6}, {NULL}};
const Segment SGI88122[] = {{&S51573[115], 9}, {NULL}};
const Segment SGI88123[] = {{&S51573[88], 6}, {NULL}};
const Segment SGI88124[] = {{&S51574[34], 10}, {NULL}};
const Segment SGI88125[] = {{&S51574[27], 6}, {NULL}};
const Segment SGI88126[] = {{&S51574[102], 9}, {NULL}};
const Segment SGI88127[] = {{&S51574[75], 6}, {NULL}};
const Segment SGI88128[] = {{&S64274[44], 14}, {NULL}};
const Segment SGI88129[] = {{&S64274[44], 25}, {NULL}};
const Segment SGI88130[] = {{&S64274[30], 6}, {NULL}};
const Segment SGI88131[] = {{&S64274[117], 3}, {NULL}};
const Segment SGI88132[] = {{&S64274[83], 5}, {NULL}};
const Segment SGI88133[] = {{&S64278[44], 14}, {NULL}};
const Segment SGI88134[] = {{&S64278[44], 25}, {NULL}};
const Segment SGI88135[] = {{&S64278[30], 6}, {NULL}};
const Segment SGI88136[] = {{&S64278[117], 3}, {NULL}};
const Segment SGI88137[] = {{&S64278[83], 5}, {NULL}};
const Segment SGI88138[] = {{&S64279[38], 14}, {NULL}};
const Segment SGI88139[] = {{&S64279[31], 6}, {NULL}};
const Segment SGI88140[] = {{&S51593[34], 11}, {NULL}};
const Segment SGI88141[] = {{&S51593[27], 6}, {NULL}};
const Segment SGI88142[] = {{&S51593[98], 9}, {NULL}};
const Segment SGI88143[] = {{&S51593[71], 6}, {NULL}};
const Segment SGI88144[] = {{&S64280[44], 14}, {NULL}};
const Segment SGI88145[] = {{&S64280[44], 25}, {NULL}};
const Segment SGI88146[] = {{&S64280[30], 6}, {NULL}};
const Segment SGI88147[] = {{&S64280[117], 3}, {NULL}};
const Segment SGI88148[] = {{&S64280[83], 5}, {NULL}};
const Segment SGI88149[] = {{&S72006[44], 15}, {NULL}};
const Segment SGI88150[] = {{&S72006[44], 27}, {NULL}};
const Segment SGI88151[] = {{&S72006[30], 6}, {NULL}};
const Segment SGI88152[] = {{&S72006[119], 3}, {NULL}};
const Segment SGI88153[] = {{&S72006[85], 5}, {NULL}};
const Segment SGI88154[] = {{&S72012[44], 20}, {NULL}};
const Segment SGI88155[] = {{&S72012[44], 32}, {NULL}};
const Segment SGI88156[] = {{&S72012[30], 6}, {NULL}};
const Segment SGI88157[] = {{&S72012[124], 3}, {NULL}};
const Segment SGI88158[] = {{&S72012[90], 5}, {NULL}};
const Segment SGI88159[] = {{&S72013[44], 21}, {NULL}};
const Segment SGI88160[] = {{&S72013[44], 33}, {NULL}};
const Segment SGI88161[] = {{&S72013[30], 6}, {NULL}};
const Segment SGI88162[] = {{&S72013[125], 3}, {NULL}};
const Segment SGI88163[] = {{&S72013[91], 5}, {NULL}};
const Segment SGI88164[] = {{&S72033[31], 8}, {NULL}};
const Segment SGI88165[] = {{&S72033[54], 3}, {NULL}};
const Segment SGI88166[] = {{&S51647[34], 10}, {NULL}};
const Segment SGI88167[] = {{&S51647[27], 6}, {NULL}};
const Segment SGI88168[] = {{&S51647[97], 9}, {NULL}};
const Segment SGI88169[] = {{&S51647[70], 6}, {NULL}};
const Segment SGI88170[] = {{&S72057[44], 11}, {NULL}};
const Segment SGI88171[] = {{&S72057[44], 23}, {NULL}};
const Segment SGI88172[] = {{&S72057[30], 6}, {NULL}};
const Segment SGI88173[] = {{&S72057[115], 3}, {NULL}};
const Segment SGI88174[] = {{&S72057[81], 5}, {NULL}};
const Segment SGI88175[] = {{&S72078[44], 10}, {NULL}};
const Segment SGI88176[] = {{&S72078[44], 27}, {NULL}};
const Segment SGI88177[] = {{&S72078[30], 6}, {NULL}};
const Segment SGI88178[] = {{&S72078[85], 6}, {NULL}};
const Segment SGI88179[] = {{&S72078[112], 13}, {NULL}};
const Segment SGI88180[] = {{&S51722[34], 7}, {NULL}};
const Segment SGI88181[] = {{&S51722[27], 6}, {NULL}};
const Segment SGI88182[] = {{&S51722[94], 9}, {NULL}};
const Segment SGI88183[] = {{&S51722[67], 6}, {NULL}};
const Segment SGI88184[] = {{&S72128[44], 8}, {NULL}};
const Segment SGI88185[] = {{&S72128[44], 31}, {NULL}};
const Segment SGI88186[] = {{&S72128[30], 6}, {NULL}};
const Segment SGI88187[] = {{&S72128[123], 3}, {NULL}};
const Segment SGI88188[] = {{&S72128[89], 5}, {NULL}};
const Segment SGI88189[] = {{&S72140[34], 10}, {NULL}};
const Segment SGI88190[] = {{&S72140[27], 6}, {NULL}};
const Segment SGI88191[] = {{&S72140[109], 2}, {NULL}};
const Segment SGI88192[] = {{&S72140[75], 6}, {NULL}};
const Segment SGI88193[] = {{&S72140[143], 8}, {NULL}};
const Segment SGI88194[] = {{&S64383[44], 12}, {NULL}};
const Segment SGI88195[] = {{&S64383[44], 23}, {NULL}};
const Segment SGI88196[] = {{&S64383[30], 6}, {NULL}};
const Segment SGI88197[] = {{&S64383[123], 3}, {NULL}};
const Segment SGI88198[] = {{&S64383[89], 5}, {NULL}};
const Segment SGI88199[] = {{&S51760[44], 12}, {NULL}};
const Segment SGI88200[] = {{&S51760[44], 24}, {NULL}};
const Segment SGI88201[] = {{&S51760[30], 6}, {NULL}};
const Segment SGI88202[] = {{&S51760[31], 3}, {NULL}};
const Segment SGI88203[] = {{&S51760[82], 6}, {NULL}};
const Segment SGI88204[] = {{&S51761[38], 12}, {NULL}};
const Segment SGI88205[] = {{&S51761[31], 6}, {NULL}};
const Segment SGI88206[] = {{&S51762[44], 14}, {NULL}};
const Segment SGI88207[] = {{&S51762[44], 26}, {NULL}};
const Segment SGI88208[] = {{&S51762[30], 6}, {NULL}};
const Segment SGI88209[] = {{&S51762[31], 3}, {NULL}};
const Segment SGI88210[] = {{&S51762[84], 6}, {NULL}};
const Segment SGI88211[] = {{&S51763[44], 12}, {NULL}};
const Segment SGI88212[] = {{&S51763[44], 24}, {NULL}};
const Segment SGI88213[] = {{&S51763[30], 6}, {NULL}};
const Segment SGI88214[] = {{&S51763[31], 3}, {NULL}};
const Segment SGI88215[] = {{&S51763[82], 6}, {NULL}};
const Segment SGI88216[] = {{&S64085[44], 19}, {NULL}};
const Segment SGI88217[] = {{&S64085[44], 30}, {NULL}};
const Segment SGI88218[] = {{&S64085[30], 6}, {NULL}};
const Segment SGI88219[] = {{&S64085[123], 3}, {NULL}};
const Segment SGI88220[] = {{&S64085[88], 6}, {NULL}};
const Segment SGI88221[] = {{&S51792[44], 7}, {NULL}};
const Segment SGI88222[] = {{&S51792[44], 20}, {NULL}};
const Segment SGI88223[] = {{&S51792[30], 6}, {NULL}};
const Segment SGI88224[] = {{&S51792[31], 3}, {NULL}};
const Segment SGI88225[] = {{&S51792[78], 6}, {NULL}};
const Segment SGI88226[] = {{&S51793[44], 7}, {NULL}};
const Segment SGI88227[] = {{&S51793[44], 20}, {NULL}};
const Segment SGI88228[] = {{&S51793[30], 6}, {NULL}};
const Segment SGI88229[] = {{&S51793[31], 3}, {NULL}};
const Segment SGI88230[] = {{&S51793[78], 6}, {NULL}};
const Segment SGI88231[] = {{&S51794[44], 7}, {NULL}};
const Segment SGI88232[] = {{&S51794[44], 20}, {NULL}};
const Segment SGI88233[] = {{&S51794[30], 6}, {NULL}};
const Segment SGI88234[] = {{&S51794[31], 3}, {NULL}};
const Segment SGI88235[] = {{&S51794[78], 6}, {NULL}};
const Segment SGI88236[] = {{&S51795[44], 7}, {NULL}};
const Segment SGI88237[] = {{&S51795[44], 20}, {NULL}};
const Segment SGI88238[] = {{&S51795[30], 6}, {NULL}};
const Segment SGI88239[] = {{&S51795[31], 3}, {NULL}};
const Segment SGI88240[] = {{&S51795[78], 6}, {NULL}};
const Segment SGI88241[] = {{&S51796[44], 7}, {NULL}};
const Segment SGI88242[] = {{&S51796[44], 20}, {NULL}};
const Segment SGI88243[] = {{&S51796[30], 6}, {NULL}};
const Segment SGI88244[] = {{&S51796[31], 3}, {NULL}};
const Segment SGI88245[] = {{&S51796[78], 6}, {NULL}};
const Segment SGI88246[] = {{&S51797[38], 7}, {NULL}};
const Segment SGI88247[] = {{&S51797[31], 6}, {NULL}};
const Segment SGI88248[] = {{&S51798[44], 7}, {NULL}};
const Segment SGI88249[] = {{&S51798[44], 20}, {NULL}};
const Segment SGI88250[] = {{&S51798[30], 6}, {NULL}};
const Segment SGI88251[] = {{&S51798[31], 3}, {NULL}};
const Segment SGI88252[] = {{&S51798[78], 6}, {NULL}};
const Segment SGI88253[] = {{&S72277[44], 12}, {NULL}};
const Segment SGI88254[] = {{&S72277[44], 24}, {NULL}};
const Segment SGI88255[] = {{&S72277[30], 6}, {NULL}};
const Segment SGI88256[] = {{&S72277[116], 3}, {NULL}};
const Segment SGI88257[] = {{&S72277[82], 5}, {NULL}};
const Segment SGI88258[] = {{&S51809[34], 10}, {NULL}};
const Segment SGI88259[] = {{&S51809[27], 6}, {NULL}};
const Segment SGI88260[] = {{&S51809[97], 9}, {NULL}};
const Segment SGI88261[] = {{&S51809[70], 6}, {NULL}};
const Segment SGI88262[] = {{&S64465[44], 7}, {NULL}};
const Segment SGI88263[] = {{&S64465[44], 19}, {NULL}};
const Segment SGI88264[] = {{&S64465[30], 6}, {NULL}};
const Segment SGI88265[] = {{&S64465[111], 3}, {NULL}};
const Segment SGI88266[] = {{&S64465[77], 5}, {NULL}};
const Segment SGI88267[] = {{&S72299[31], 9}, {NULL}};
const Segment SGI88268[] = {{&S72299[55], 3}, {NULL}};
const Segment SGI88269[] = {{&S64466[44], 7}, {NULL}};
const Segment SGI88270[] = {{&S64466[44], 19}, {NULL}};
const Segment SGI88271[] = {{&S64466[30], 6}, {NULL}};
const Segment SGI88272[] = {{&S64466[111], 3}, {NULL}};
const Segment SGI88273[] = {{&S64466[77], 5}, {NULL}};
const Segment SGI88274[] = {{&S72300[31], 8}, {NULL}};
const Segment SGI88275[] = {{&S72300[54], 3}, {NULL}};
const Segment SGI88276[] = {{&S64467[44], 7}, {NULL}};
const Segment SGI88277[] = {{&S64467[44], 19}, {NULL}};
const Segment SGI88278[] = {{&S64467[30], 6}, {NULL}};
const Segment SGI88279[] = {{&S64467[111], 3}, {NULL}};
const Segment SGI88280[] = {{&S64467[77], 5}, {NULL}};
const Segment SGI88281[] = {{&S64468[44], 7}, {NULL}};
const Segment SGI88282[] = {{&S64468[44], 19}, {NULL}};
const Segment SGI88283[] = {{&S64468[30], 6}, {NULL}};
const Segment SGI88284[] = {{&S64468[111], 3}, {NULL}};
const Segment SGI88285[] = {{&S64468[77], 5}, {NULL}};
const Segment SGI88286[] = {{&S72304[44], 12}, {NULL}};
const Segment SGI88287[] = {{&S72304[44], 32}, {NULL}};
const Segment SGI88288[] = {{&S72304[30], 6}, {NULL}};
const Segment SGI88289[] = {{&S72304[124], 3}, {NULL}};
const Segment SGI88290[] = {{&S72304[90], 5}, {NULL}};
const Segment SGI88291[] = {{&S72305[44], 12}, {NULL}};
const Segment SGI88292[] = {{&S72305[44], 34}, {NULL}};
const Segment SGI88293[] = {{&S72305[30], 6}, {NULL}};
const Segment SGI88294[] = {{&S72305[126], 3}, {NULL}};
const Segment SGI88295[] = {{&S72305[92], 5}, {NULL}};
const Segment SGI88296[] = {{&S64477[44], 14}, {NULL}};
const Segment SGI88297[] = {{&S64477[44], 25}, {NULL}};
const Segment SGI88298[] = {{&S64477[30], 6}, {NULL}};
const Segment SGI88299[] = {{&S64477[117], 3}, {NULL}};
const Segment SGI88300[] = {{&S64477[83], 5}, {NULL}};
const Segment SGI88301[] = {{&S64478[44], 14}, {NULL}};
const Segment SGI88302[] = {{&S64478[44], 30}, {NULL}};
const Segment SGI88303[] = {{&S64478[30], 6}, {NULL}};
const Segment SGI88304[] = {{&S64478[122], 3}, {NULL}};
const Segment SGI88305[] = {{&S64478[88], 5}, {NULL}};
const Segment SGI88306[] = {{&S72325[59], 10}, {NULL}};
const Segment SGI88307[] = {{&S72325[59], 27}, {NULL}};
const Segment SGI88308[] = {{&S72325[45], 6}, {NULL}};
const Segment SGI88309[] = {{&S72325[134], 3}, {NULL}};
const Segment SGI88310[] = {{&S72325[100], 5}, {NULL}};
const Segment SGI88311[] = {{&S72325[0], 14}, {NULL}};
const Segment SGI88312[] = {{&S72326[57], 10}, {NULL}};
const Segment SGI88313[] = {{&S72326[45], 6}, {NULL}};
const Segment SGI88314[] = {{&S72326[132], 3}, {NULL}};
const Segment SGI88315[] = {{&S72326[98], 5}, {NULL}};
const Segment SGI88316[] = {{&S72326[0], 14}, {NULL}};
const Segment SGI88317[] = {{&S51866[44], 11}, {NULL}};
const Segment SGI88318[] = {{&S51866[44], 23}, {NULL}};
const Segment SGI88319[] = {{&S51866[30], 6}, {NULL}};
const Segment SGI88320[] = {{&S51866[31], 3}, {NULL}};
const Segment SGI88321[] = {{&S51866[81], 6}, {NULL}};
const Segment SGI88322[] = {{&S51867[44], 17}, {NULL}};
const Segment SGI88323[] = {{&S51867[44], 29}, {NULL}};
const Segment SGI88324[] = {{&S51867[30], 6}, {NULL}};
const Segment SGI88325[] = {{&S51867[31], 3}, {NULL}};
const Segment SGI88326[] = {{&S51867[87], 6}, {NULL}};
const Segment SGI88327[] = {{&S51868[44], 11}, {NULL}};
const Segment SGI88328[] = {{&S51868[44], 23}, {NULL}};
const Segment SGI88329[] = {{&S51868[30], 6}, {NULL}};
const Segment SGI88330[] = {{&S51868[84], 6}, {NULL}};
const Segment SGI88331[] = {{&S51869[44], 11}, {NULL}};
const Segment SGI88332[] = {{&S51869[44], 23}, {NULL}};
const Segment SGI88333[] = {{&S51869[30], 6}, {NULL}};
const Segment SGI88334[] = {{&S51869[84], 4}, {NULL}};
const Segment SGI88335[] = {{&S51869[84], 3}, {NULL}};
const Segment SGI88336[] = {{&S51870[44], 17}, {NULL}};
const Segment SGI88337[] = {{&S51870[44], 29}, {NULL}};
const Segment SGI88338[] = {{&S51870[30], 6}, {NULL}};
const Segment SGI88339[] = {{&S51870[31], 3}, {NULL}};
const Segment SGI88340[] = {{&S51870[87], 6}, {NULL}};
const Segment SGI88341[] = {{&S51871[44], 11}, {NULL}};
const Segment SGI88342[] = {{&S51871[44], 23}, {NULL}};
const Segment SGI88343[] = {{&S51871[30], 6}, {NULL}};
const Segment SGI88344[] = {{&S51871[31], 3}, {NULL}};
const Segment SGI88345[] = {{&S51871[81], 6}, {NULL}};
const Segment SGI88346[] = {{&S51872[38], 17}, {NULL}};
const Segment SGI88347[] = {{&S51872[31], 6}, {NULL}};
const Segment SGI88348[] = {{&S51873[44], 17}, {NULL}};
const Segment SGI88349[] = {{&S51873[44], 29}, {NULL}};
const Segment SGI88350[] = {{&S51873[30], 6}, {NULL}};
const Segment SGI88351[] = {{&S51873[90], 6}, {NULL}};
const Segment SGI88352[] = {{&S64229[44], 10}, {NULL}};
const Segment SGI88353[] = {{&S64229[44], 22}, {NULL}};
const Segment SGI88354[] = {{&S64229[30], 6}, {NULL}};
const Segment SGI88355[] = {{&S64229[115], 3}, {NULL}};
const Segment SGI88356[] = {{&S64229[80], 6}, {NULL}};
const Segment SGI88357[] = {{&S64230[42], 10}, {NULL}};
const Segment SGI88358[] = {{&S64230[30], 6}, {NULL}};
const Segment SGI88359[] = {{&S64230[113], 3}, {NULL}};
const Segment SGI88360[] = {{&S64230[78], 6}, {NULL}};
const Segment SGI88361[] = {{&S64231[44], 10}, {NULL}};
const Segment SGI88362[] = {{&S64231[44], 22}, {NULL}};
const Segment SGI88363[] = {{&S64231[30], 6}, {NULL}};
const Segment SGI88364[] = {{&S64231[115], 3}, {NULL}};
const Segment SGI88365[] = {{&S64231[80], 6}, {NULL}};
const Segment SGI88366[] = {{&S51904[34], 18}, {NULL}};
const Segment SGI88367[] = {{&S51904[27], 6}, {NULL}};
const Segment SGI88368[] = {{&S51904[105], 9}, {NULL}};
const Segment SGI88369[] = {{&S51904[78], 6}, {NULL}};
const Segment SGI88370[] = {{&S51905[34], 21}, {NULL}};
const Segment SGI88371[] = {{&S51905[27], 6}, {NULL}};
const Segment SGI88372[] = {{&S51905[108], 9}, {NULL}};
const Segment SGI88373[] = {{&S51905[81], 6}, {NULL}};
const Segment SGI88374[] = {{&S51907[34], 35}, {NULL}};
const Segment SGI88375[] = {{&S51907[27], 6}, {NULL}};
const Segment SGI88376[] = {{&S51907[122], 9}, {NULL}};
const Segment SGI88377[] = {{&S51907[95], 6}, {NULL}};
const Segment SGI88378[] = {{&S51953[34], 21}, {NULL}};
const Segment SGI88379[] = {{&S72456[6], 4}, {NULL}};
const Segment SGI88380[] = {{&S51953[27], 6}, {NULL}};
const Segment SGI88381[] = {{&S72456[84], 5}, {NULL}};
const Segment SGI88382[] = {{&S51953[108], 9}, {NULL}};
const Segment SGI88383[] = {{&S51953[81], 6}, {NULL}};
const Segment SGI88384[] = {{&S51954[34], 18}, {NULL}};
const Segment SGI88385[] = {{&S51954[27], 6}, {NULL}};
const Segment SGI88386[] = {{&S51954[105], 9}, {NULL}};
const Segment SGI88387[] = {{&S51954[78], 6}, {NULL}};
const Segment SGI88388[] = {{&S51956[34], 35}, {NULL}};
const Segment SGI88389[] = {{&S51956[27], 6}, {NULL}};
const Segment SGI88390[] = {{&S51956[122], 9}, {NULL}};
const Segment SGI88391[] = {{&S51956[95], 6}, {NULL}};
const Segment SGI88392[] = {{&S72520[44], 10}, {NULL}};
const Segment SGI88393[] = {{&S72520[44], 27}, {NULL}};
const Segment SGI88394[] = {{&S72520[30], 6}, {NULL}};
const Segment SGI88395[] = {{&S72520[85], 6}, {NULL}};
const Segment SGI88396[] = {{&S72520[112], 13}, {NULL}};
const Segment SGI88397[] = {{&S52043[34], 14}, {NULL}};
const Segment SGI88398[] = {{&S52043[27], 6}, {NULL}};
const Segment SGI88399[] = {{&S52043[101], 9}, {NULL}};
const Segment SGI88400[] = {{&S52043[74], 6}, {NULL}};
const Segment SGI88401[] = {{&S52045[34], 15}, {NULL}};
const Segment SGI88402[] = {{&S52045[27], 6}, {NULL}};
const Segment SGI88403[] = {{&S52045[102], 9}, {NULL}};
const Segment SGI88404[] = {{&S52045[75], 6}, {NULL}};
const Segment SGI88405[] = {{&S52052[44], 13}, {NULL}};
const Segment SGI88406[] = {{&S52052[44], 25}, {NULL}};
const Segment SGI88407[] = {{&S52052[30], 6}, {NULL}};
const Segment SGI88408[] = {{&S52052[31], 3}, {NULL}};
const Segment SGI88409[] = {{&S52052[83], 6}, {NULL}};
const Segment SGI88410[] = {{&S52053[44], 13}, {NULL}};
const Segment SGI88411[] = {{&S52053[44], 25}, {NULL}};
const Segment SGI88412[] = {{&S52053[30], 6}, {NULL}};
const Segment SGI88413[] = {{&S52053[86], 6}, {NULL}};
const Segment SGI88414[] = {{&S52054[38], 13}, {NULL}};
const Segment SGI88415[] = {{&S52054[31], 6}, {NULL}};
const Segment SGI88416[] = {{&S52055[44], 13}, {NULL}};
const Segment SGI88417[] = {{&S52055[44], 25}, {NULL}};
const Segment SGI88418[] = {{&S52055[30], 6}, {NULL}};
const Segment SGI88419[] = {{&S52055[86], 4}, {NULL}};
const Segment SGI88420[] = {{&S52055[86], 3}, {NULL}};
const Segment SGI88421[] = {{&S52084[44], 15}, {NULL}};
const Segment SGI88422[] = {{&S52084[44], 29}, {NULL}};
const Segment SGI88423[] = {{&S52084[30], 6}, {NULL}};
const Segment SGI88424[] = {{&S52084[31], 3}, {NULL}};
const Segment SGI88425[] = {{&S52084[87], 6}, {NULL}};
const Segment SGI88426[] = {{&S52085[44], 15}, {NULL}};
const Segment SGI88427[] = {{&S52085[44], 29}, {NULL}};
const Segment SGI88428[] = {{&S72588[59], 11}, {NULL}};
const Segment SGI88429[] = {{&S52085[30], 6}, {NULL}};
const Segment SGI88430[] = {{&S72588[59], 28}, {NULL}};
const Segment SGI88431[] = {{&S72588[45], 6}, {NULL}};
const Segment SGI88432[] = {{&S52085[31], 3}, {NULL}};
const Segment SGI88433[] = {{&S52085[87], 6}, {NULL}};
const Segment SGI88434[] = {{&S72588[135], 3}, {NULL}};
const Segment SGI88435[] = {{&S72588[101], 5}, {NULL}};
const Segment SGI88436[] = {{&S72588[0], 14}, {NULL}};
const Segment SGI88437[] = {{&S52086[44], 15}, {NULL}};
const Segment SGI88438[] = {{&S52086[44], 29}, {NULL}};
const Segment SGI88439[] = {{&S52086[30], 6}, {NULL}};
const Segment SGI88440[] = {{&S52086[31], 3}, {NULL}};
const Segment SGI88441[] = {{&S52086[87], 6}, {NULL}};
const Segment SGI88442[] = {{&S52087[44], 15}, {NULL}};
const Segment SGI88443[] = {{&S52087[44], 29}, {NULL}};
const Segment SGI88444[] = {{&S52087[30], 6}, {NULL}};
const Segment SGI88445[] = {{&S52087[31], 3}, {NULL}};
const Segment SGI88446[] = {{&S52087[87], 6}, {NULL}};
const Segment SGI88447[] = {{&S52088[38], 15}, {NULL}};
const Segment SGI88448[] = {{&S52088[31], 6}, {NULL}};
const Segment SGI88449[] = {{&S72594[59], 10}, {NULL}};
const Segment SGI88450[] = {{&S72594[59], 27}, {NULL}};
const Segment SGI88451[] = {{&S72594[45], 6}, {NULL}};
const Segment SGI88452[] = {{&S72594[134], 3}, {NULL}};
const Segment SGI88453[] = {{&S72594[100], 5}, {NULL}};
const Segment SGI88454[] = {{&S72594[0], 14}, {NULL}};
const Segment SGI88455[] = {{&S72596[59], 10}, {NULL}};
const Segment SGI88456[] = {{&S72596[59], 27}, {NULL}};
const Segment SGI88457[] = {{&S72596[45], 6}, {NULL}};
const Segment SGI88458[] = {{&S72596[134], 3}, {NULL}};
const Segment SGI88459[] = {{&S72596[100], 5}, {NULL}};
const Segment SGI88460[] = {{&S72596[0], 14}, {NULL}};
const Segment SGI88461[] = {{&S72599[45], 10}, {NULL}};
const Segment SGI88462[] = {{&S72599[30], 6}, {NULL}};
const Segment SGI88463[] = {{&S72599[120], 3}, {NULL}};
const Segment SGI88464[] = {{&S72599[86], 5}, {NULL}};
const Segment SGI88465[] = {{&S72599[144], 13}, {NULL}};
const Segment SGI88466[] = {{&S72600[44], 10}, {NULL}};
const Segment SGI88467[] = {{&S72600[44], 27}, {NULL}};
const Segment SGI88468[] = {{&S72600[30], 6}, {NULL}};
const Segment SGI88469[] = {{&S72600[120], 3}, {NULL}};
const Segment SGI88470[] = {{&S72600[85], 6}, {NULL}};
const Segment SGI88471[] = {{&S72600[140], 8}, {NULL}};
const Segment SGI88472[] = {{&S72600[149], 13}, {NULL}};
const Segment SGI88473[] = {{&S52104[34], 11}, {NULL}};
const Segment SGI88474[] = {{&S52104[27], 6}, {NULL}};
const Segment SGI88475[] = {{&S52104[98], 9}, {NULL}};
const Segment SGI88476[] = {{&S52104[71], 6}, {NULL}};
const Segment SGI88477[] = {{&S52106[34], 11}, {NULL}};
const Segment SGI88478[] = {{&S52106[27], 6}, {NULL}};
const Segment SGI88479[] = {{&S52106[98], 9}, {NULL}};
const Segment SGI88480[] = {{&S52106[71], 6}, {NULL}};
const Segment SGI88481[] = {{&S52111[34], 19}, {NULL}};
const Segment SGI88482[] = {{&S52111[27], 6}, {NULL}};
const Segment SGI88483[] = {{&S52111[106], 9}, {NULL}};
const Segment SGI88484[] = {{&S52111[79], 6}, {NULL}};
const Segment SGI88485[] = {{&S52113[34], 7}, {NULL}};
const Segment SGI88486[] = {{&S52113[27], 6}, {NULL}};
const Segment SGI88487[] = {{&S52113[94], 9}, {NULL}};
const Segment SGI88488[] = {{&S52113[67], 6}, {NULL}};
const Segment SGI88489[] = {{&S57246[6], 4}, {NULL}};
const Segment SGI88490[] = {{&S57246[76], 5}, {NULL}};
const Segment SGI88491[] = {{&S52148[44], 9}, {NULL}};
const Segment SGI88492[] = {{&S52148[44], 21}, {NULL}};
const Segment SGI88493[] = {{&S52148[30], 6}, {NULL}};
const Segment SGI88494[] = {{&S52148[31], 3}, {NULL}};
const Segment SGI88495[] = {{&S52148[79], 6}, {NULL}};
const Segment SGI88496[] = {{&S72659[45], 10}, {NULL}};
const Segment SGI88497[] = {{&S72659[30], 6}, {NULL}};
const Segment SGI88498[] = {{&S72659[120], 3}, {NULL}};
const Segment SGI88499[] = {{&S72659[86], 5}, {NULL}};
const Segment SGI88500[] = {{&S72659[144], 13}, {NULL}};
const Segment SGI88501[] = {{&S72663[44], 12}, {NULL}};
const Segment SGI88502[] = {{&S72663[44], 24}, {NULL}};
const Segment SGI88503[] = {{&S72663[30], 6}, {NULL}};
const Segment SGI88504[] = {{&S72663[116], 3}, {NULL}};
const Segment SGI88505[] = {{&S72663[82], 5}, {NULL}};
const Segment SGI88506[] = {{&S72677[44], 11}, {NULL}};
const Segment SGI88507[] = {{&S72677[44], 23}, {NULL}};
const Segment SGI88508[] = {{&S72677[30], 6}, {NULL}};
const Segment SGI88509[] = {{&S72677[115], 3}, {NULL}};
const Segment SGI88510[] = {{&S72677[81], 5}, {NULL}};
const Segment SGI88511[] = {{&S72692[44], 14}, {NULL}};
const Segment SGI88512[] = {{&S72692[44], 26}, {NULL}};
const Segment SGI88513[] = {{&S72692[30], 6}, {NULL}};
const Segment SGI88514[] = {{&S72692[118], 3}, {NULL}};
const Segment SGI88515[] = {{&S72692[84], 5}, {NULL}};
const Segment SGI88516[] = {{&S72702[45], 10}, {NULL}};
const Segment SGI88517[] = {{&S72702[30], 6}, {NULL}};
const Segment SGI88518[] = {{&S72702[120], 3}, {NULL}};
const Segment SGI88519[] = {{&S72702[86], 5}, {NULL}};
const Segment SGI88520[] = {{&S72702[144], 13}, {NULL}};
const Segment SGI88521[] = {{&S72735[45], 10}, {NULL}};
const Segment SGI88522[] = {{&S72735[30], 6}, {NULL}};
const Segment SGI88523[] = {{&S72735[120], 3}, {NULL}};
const Segment SGI88524[] = {{&S72735[86], 5}, {NULL}};
const Segment SGI88525[] = {{&S72735[144], 13}, {NULL}};
const Segment SGI88526[] = {{&S72737[44], 10}, {NULL}};
const Segment SGI88527[] = {{&S72737[44], 27}, {NULL}};
const Segment SGI88528[] = {{&S72737[30], 6}, {NULL}};
const Segment SGI88529[] = {{&S72737[120], 3}, {NULL}};
const Segment SGI88530[] = {{&S72737[85], 6}, {NULL}};
const Segment SGI88531[] = {{&S72737[140], 8}, {NULL}};
const Segment SGI88532[] = {{&S72737[149], 13}, {NULL}};
const Segment SGI88533[] = {{&S72744[44], 10}, {NULL}};
const Segment SGI88534[] = {{&S72744[44], 27}, {NULL}};
const Segment SGI88535[] = {{&S72744[30], 6}, {NULL}};
const Segment SGI88536[] = {{&S72744[149], 4}, {NULL}};
const Segment SGI88537[] = {{&S72744[175], 3}, {NULL}};
const Segment SGI88538[] = {{&S72744[119], 3}, {&S72744[224], 5}, {NULL}};
const Segment SGI88539[] = {{&S72744[85], 5}, {NULL}};
const Segment SGI88540[] = {{&S52283[44], 10}, {NULL}};
const Segment SGI88541[] = {{&S52283[44], 27}, {NULL}};
const Segment SGI88542[] = {{&S52283[30], 6}, {NULL}};
const Segment SGI88543[] = {{&S52283[31], 3}, {NULL}};
const Segment SGI88544[] = {{&S52283[85], 6}, {NULL}};
const Segment SGI88545[] = {{&S52284[44], 10}, {NULL}};
const Segment SGI88546[] = {{&S52284[44], 22}, {NULL}};
const Segment SGI88547[] = {{&S52284[30], 6}, {NULL}};
const Segment SGI88548[] = {{&S52284[31], 3}, {NULL}};
const Segment SGI88549[] = {{&S52284[80], 6}, {NULL}};
const Segment SGI88550[] = {{&S52286[38], 10}, {NULL}};
const Segment SGI88551[] = {{&S52286[31], 6}, {NULL}};
const Segment SGI88552[] = {{&S64655[42], 7}, {NULL}};
const Segment SGI88553[] = {{&S64655[42], 18}, {NULL}};
const Segment SGI88554[] = {{&S64655[30], 4}, {NULL}};
const Segment SGI88555[] = {{&S64655[109], 3}, {NULL}};
const Segment SGI88556[] = {{&S64655[74], 6}, {NULL}};
const Segment SGI88557[] = {{&S72798[34], 7}, {NULL}};
const Segment SGI88558[] = {{&S72798[27], 6}, {NULL}};
const Segment SGI88559[] = {{&S72798[113], 2}, {NULL}};
const Segment SGI88560[] = {{&S72798[79], 6}, {NULL}};
const Segment SGI88561[] = {{&S52317[44], 9}, {NULL}};
const Segment SGI88562[] = {{&S52317[44], 21}, {NULL}};
const Segment SGI88563[] = {{&S72798[147], 8}, {NULL}};
const Segment SGI88564[] = {{&S52317[30], 6}, {NULL}};
const Segment SGI88565[] = {{&S52317[31], 3}, {NULL}};
const Segment SGI88566[] = {{&S52317[79], 6}, {NULL}};
const Segment SGI88567[] = {{&S52344[34], 15}, {NULL}};
const Segment SGI88568[] = {{&S52344[27], 6}, {NULL}};
const Segment SGI88569[] = {{&S52344[101], 9}, {NULL}};
const Segment SGI88570[] = {{&S52344[74], 6}, {NULL}};
const Segment SGI88571[] = {{&S52351[44], 10}, {NULL}};
const Segment SGI88572[] = {{&S52351[44], 22}, {NULL}};
const Segment SGI88573[] = {{&S52351[30], 6}, {NULL}};
const Segment SGI88574[] = {{&S52351[31], 3}, {NULL}};
const Segment SGI88575[] = {{&S52351[80], 6}, {NULL}};
const Segment SGI88576[] = {{&S52352[44], 9}, {NULL}};
const Segment SGI88577[] = {{&S52352[44], 21}, {NULL}};
const Segment SGI88578[] = {{&S52352[30], 6}, {NULL}};
const Segment SGI88579[] = {{&S52352[31], 3}, {NULL}};
const Segment SGI88580[] = {{&S52352[79], 6}, {NULL}};
const Segment SGI88581[] = {{&S52354[38], 10}, {NULL}};
const Segment SGI88582[] = {{&S52354[31], 6}, {NULL}};
const Segment SGI88583[] = {{&S52357[44], 9}, {NULL}};
const Segment SGI88584[] = {{&S52357[44], 21}, {NULL}};
const Segment SGI88585[] = {{&S52357[30], 6}, {NULL}};
const Segment SGI88586[] = {{&S52357[31], 3}, {NULL}};
const Segment SGI88587[] = {{&S52357[79], 6}, {NULL}};
const Segment SGI88588[] = {{&S52361[44], 10}, {NULL}};
const Segment SGI88589[] = {{&S52361[44], 21}, {NULL}};
const Segment SGI88590[] = {{&S52361[30], 6}, {NULL}};
const Segment SGI88591[] = {{&S52361[31], 3}, {NULL}};
const Segment SGI88592[] = {{&S52361[79], 6}, {NULL}};
const Segment SGI88593[] = {{&S52363[44], 10}, {NULL}};
const Segment SGI88594[] = {{&S52363[44], 22}, {NULL}};
const Segment SGI88595[] = {{&S52363[30], 6}, {NULL}};
const Segment SGI88596[] = {{&S52363[31], 3}, {NULL}};
const Segment SGI88597[] = {{&S52363[80], 6}, {NULL}};
const Segment SGI88598[] = {{&S72928[44], 7}, {NULL}};
const Segment SGI88599[] = {{&S72928[44], 19}, {NULL}};
const Segment SGI88600[] = {{&S72928[30], 6}, {NULL}};
const Segment SGI88601[] = {{&S72928[111], 3}, {NULL}};
const Segment SGI88602[] = {{&S72928[77], 5}, {NULL}};
const Segment SGI88603[] = {{&S72929[44], 7}, {NULL}};
const Segment SGI88604[] = {{&S72929[44], 19}, {NULL}};
const Segment SGI88605[] = {{&S72929[30], 6}, {NULL}};
const Segment SGI88606[] = {{&S72929[77], 5}, {NULL}};
const Segment SGI88607[] = {{&S72930[38], 7}, {NULL}};
const Segment SGI88608[] = {{&S72930[31], 6}, {NULL}};
const Segment SGI88609[] = {{&S52500[34], 6}, {NULL}};
const Segment SGI88610[] = {{&S52500[27], 6}, {NULL}};
const Segment SGI88611[] = {{&S52500[93], 9}, {NULL}};
const Segment SGI88612[] = {{&S52500[66], 6}, {NULL}};
const Segment SGI88613[] = {{&S64887[44], 7}, {NULL}};
const Segment SGI88614[] = {{&S64887[44], 18}, {NULL}};
const Segment SGI88615[] = {{&S64887[30], 6}, {NULL}};
const Segment SGI88616[] = {{&S64887[111], 3}, {NULL}};
const Segment SGI88617[] = {{&S64887[76], 6}, {NULL}};
const Segment SGI88618[] = {{&S52525[34], 11}, {NULL}};
const Segment SGI88619[] = {{&S52525[27], 6}, {NULL}};
const Segment SGI88620[] = {{&S52525[98], 9}, {NULL}};
const Segment SGI88621[] = {{&S52525[71], 6}, {NULL}};
const Segment SGI88622[] = {{&S52553[34], 6}, {NULL}};
const Segment SGI88623[] = {{&S52553[27], 6}, {NULL}};
const Segment SGI88624[] = {{&S52553[93], 9}, {NULL}};
const Segment SGI88625[] = {{&S52553[66], 6}, {NULL}};
const Segment SGI88626[] = {{&S73055[44], 10}, {NULL}};
const Segment SGI88627[] = {{&S73055[44], 27}, {NULL}};
const Segment SGI88628[] = {{&S73055[30], 6}, {NULL}};
const Segment SGI88629[] = {{&S73055[119], 3}, {NULL}};
const Segment SGI88630[] = {{&S73055[85], 5}, {NULL}};
const Segment SGI88631[] = {{&S73057[44], 11}, {NULL}};
const Segment SGI88632[] = {{&S73057[44], 28}, {NULL}};
const Segment SGI88633[] = {{&S73057[30], 6}, {NULL}};
const Segment SGI88634[] = {{&S73057[120], 3}, {NULL}};
const Segment SGI88635[] = {{&S73057[86], 5}, {NULL}};
const Segment SGI88636[] = {{&S73058[44], 13}, {NULL}};
const Segment SGI88637[] = {{&S73058[44], 30}, {NULL}};
const Segment SGI88638[] = {{&S73058[30], 6}, {NULL}};
const Segment SGI88639[] = {{&S73058[122], 3}, {NULL}};
const Segment SGI88640[] = {{&S73058[88], 5}, {NULL}};
const Segment SGI88641[] = {{&S52609[58], 5}, {NULL}};
const Segment SGI88642[] = {{&S52609[41], 16}, {NULL}};
const Segment SGI88643[] = {{&S52609[29], 3}, {NULL}};
const Segment SGI88644[] = {{&S52609[86], 9}, {NULL}};
const Segment SGI88645[] = {{&S52613[62], 5}, {NULL}};
const Segment SGI88646[] = {{&S52613[44], 17}, {NULL}};
const Segment SGI88647[] = {{&S52613[30], 6}, {NULL}};
const Segment SGI88648[] = {{&S52613[31], 3}, {NULL}};
const Segment SGI88649[] = {{&S52613[90], 6}, {NULL}};
const Segment SGI88650[] = {{&S52625[34], 10}, {NULL}};
const Segment SGI88651[] = {{&S52625[27], 6}, {NULL}};
const Segment SGI88652[] = {{&S52625[97], 9}, {NULL}};
const Segment SGI88653[] = {{&S52625[70], 6}, {NULL}};
const Segment SGI88654[] = {{&S64996[42], 10}, {NULL}};
const Segment SGI88655[] = {{&S64996[42], 32}, {NULL}};
const Segment SGI88656[] = {{&S64996[30], 4}, {NULL}};
const Segment SGI88657[] = {{&S64996[123], 3}, {NULL}};
const Segment SGI88658[] = {{&S64996[88], 6}, {NULL}};
const Segment SGI88659[] = {{&S64997[42], 17}, {NULL}};
const Segment SGI88660[] = {{&S64997[42], 39}, {NULL}};
const Segment SGI88661[] = {{&S64997[30], 4}, {NULL}};
const Segment SGI88662[] = {{&S64997[130], 3}, {NULL}};
const Segment SGI88663[] = {{&S64997[95], 6}, {NULL}};
const Segment SGI88664[] = {{&S52632[44], 9}, {NULL}};
const Segment SGI88665[] = {{&S52632[44], 21}, {NULL}};
const Segment SGI88666[] = {{&S52632[30], 6}, {NULL}};
const Segment SGI88667[] = {{&S52632[31], 3}, {NULL}};
const Segment SGI88668[] = {{&S52632[79], 6}, {NULL}};
const Segment SGI88669[] = {{&S52653[34], 6}, {NULL}};
const Segment SGI88670[] = {{&S52653[27], 6}, {NULL}};
const Segment SGI88671[] = {{&S52653[93], 9}, {NULL}};
const Segment SGI88672[] = {{&S52653[66], 6}, {NULL}};
const Segment SGI88673[] = {{&S52657[34], 12}, {NULL}};
const Segment SGI88674[] = {{&S52657[27], 6}, {NULL}};
const Segment SGI88675[] = {{&S52657[99], 9}, {NULL}};
const Segment SGI88676[] = {{&S52657[72], 6}, {NULL}};
const Segment SGI88677[] = {{&S52659[34], 12}, {NULL}};
const Segment SGI88678[] = {{&S52659[27], 6}, {NULL}};
const Segment SGI88679[] = {{&S52659[99], 9}, {NULL}};
const Segment SGI88680[] = {{&S52659[72], 6}, {NULL}};
const Segment SGI88681[] = {{&S52689[34], 15}, {NULL}};
const Segment SGI88682[] = {{&S52689[27], 6}, {NULL}};
const Segment SGI88683[] = {{&S52689[104], 9}, {NULL}};
const Segment SGI88684[] = {{&S52689[77], 6}, {NULL}};
const Segment SGI88685[] = {{&S65095[42], 6}, {NULL}};
const Segment SGI88686[] = {{&S65095[42], 17}, {NULL}};
const Segment SGI88687[] = {{&S65095[30], 4}, {NULL}};
const Segment SGI88688[] = {{&S65095[108], 3}, {NULL}};
const Segment SGI88689[] = {{&S65095[73], 6}, {NULL}};
const Segment SGI88690[] = {{&S65096[44], 7}, {NULL}};
const Segment SGI88691[] = {{&S65096[44], 19}, {NULL}};
const Segment SGI88692[] = {{&S65096[30], 6}, {NULL}};
const Segment SGI88693[] = {{&S65096[112], 3}, {NULL}};
const Segment SGI88694[] = {{&S65096[77], 6}, {NULL}};
const Segment SGI88695[] = {{&S65098[42], 15}, {NULL}};
const Segment SGI88696[] = {{&S65098[42], 32}, {NULL}};
const Segment SGI88697[] = {{&S65098[30], 4}, {NULL}};
const Segment SGI88698[] = {{&S65098[122], 3}, {NULL}};
const Segment SGI88699[] = {{&S65098[88], 5}, {NULL}};
const Segment SGI88700[] = {{&S65100[42], 6}, {NULL}};
const Segment SGI88701[] = {{&S65100[42], 19}, {NULL}};
const Segment SGI88702[] = {{&S65100[30], 4}, {NULL}};
const Segment SGI88703[] = {{&S65100[110], 3}, {NULL}};
const Segment SGI88704[] = {{&S65100[75], 6}, {NULL}};
const Segment SGI88705[] = {{&S65101[42], 6}, {NULL}};
const Segment SGI88706[] = {{&S65101[42], 19}, {NULL}};
const Segment SGI88707[] = {{&S65101[30], 4}, {NULL}};
const Segment SGI88708[] = {{&S65101[110], 3}, {NULL}};
const Segment SGI88709[] = {{&S65101[75], 6}, {NULL}};
const Segment SGI88710[] = {{&S65102[77], 3}, {NULL}};
const Segment SGI88711[] = {{&S65102[45], 3}, {NULL}};
const Segment SGI88712[] = {{&S65102[95], 5}, {NULL}};
const Segment SGI88713[] = {{&S52761[44], 10}, {NULL}};
const Segment SGI88714[] = {{&S52761[44], 22}, {NULL}};
const Segment SGI88715[] = {{&S52761[30], 6}, {NULL}};
const Segment SGI88716[] = {{&S52761[31], 3}, {NULL}};
const Segment SGI88717[] = {{&S52761[80], 6}, {NULL}};
const Segment SGI88718[] = {{&S52762[38], 10}, {NULL}};
const Segment SGI88719[] = {{&S52762[31], 6}, {NULL}};
const Segment SGI88720[] = {{&S52796[34], 15}, {NULL}};
const Segment SGI88721[] = {{&S52796[27], 6}, {NULL}};
const Segment SGI88722[] = {{&S52796[101], 9}, {NULL}};
const Segment SGI88723[] = {{&S52796[74], 6}, {NULL}};
const Segment SGI88724[] = {{&S65194[42], 10}, {NULL}};
const Segment SGI88725[] = {{&S65194[42], 22}, {NULL}};
const Segment SGI88726[] = {{&S65194[30], 4}, {NULL}};
const Segment SGI88727[] = {{&S65194[113], 3}, {NULL}};
const Segment SGI88728[] = {{&S65194[78], 6}, {NULL}};
const Segment SGI88729[] = {{&S65195[42], 13}, {NULL}};
const Segment SGI88730[] = {{&S65195[42], 25}, {NULL}};
const Segment SGI88731[] = {{&S65195[30], 4}, {NULL}};
const Segment SGI88732[] = {{&S65195[116], 3}, {NULL}};
const Segment SGI88733[] = {{&S65195[81], 6}, {NULL}};
const Segment SGI88734[] = {{&S65196[36], 10}, {NULL}};
const Segment SGI88735[] = {{&S65196[31], 4}, {NULL}};
const Segment SGI88736[] = {{&S73316[45], 13}, {NULL}};
const Segment SGI88737[] = {{&S73316[30], 6}, {NULL}};
const Segment SGI88738[] = {{&S73316[118], 3}, {NULL}};
const Segment SGI88739[] = {{&S73316[84], 5}, {NULL}};
const Segment SGI88740[] = {{&S73316[142], 13}, {NULL}};
const Segment SGI88741[] = {{&S65290[44], 11}, {NULL}};
const Segment SGI88742[] = {{&S65290[44], 33}, {NULL}};
const Segment SGI88743[] = {{&S65290[30], 6}, {NULL}};
const Segment SGI88744[] = {{&S65290[144], 3}, {NULL}};
const Segment SGI88745[] = {{&S65290[110], 5}, {NULL}};
const Segment SGI88746[] = {{&S65290[35], 1}, {NULL}};
const Segment SGI88747[] = {{&S65290[79], 11}, {NULL}};
const Segment SGI88748[] = {{&S65291[44], 11}, {NULL}};
const Segment SGI88749[] = {{&S65291[44], 33}, {NULL}};
const Segment SGI88750[] = {{&S65291[30], 6}, {NULL}};
const Segment SGI88751[] = {{&S65291[144], 3}, {NULL}};
const Segment SGI88752[] = {{&S65291[110], 5}, {NULL}};
const Segment SGI88753[] = {{&S65291[85], 3}, {NULL}};
const Segment SGI88754[] = {{&S52864[34], 11}, {NULL}};
const Segment SGI88755[] = {{&S52864[27], 6}, {NULL}};
const Segment SGI88756[] = {{&S52864[98], 9}, {NULL}};
const Segment SGI88757[] = {{&S52864[71], 6}, {NULL}};
const Segment SGI88758[] = {{&S52865[34], 11}, {NULL}};
const Segment SGI88759[] = {{&S52865[27], 6}, {NULL}};
const Segment SGI88760[] = {{&S52865[98], 9}, {NULL}};
const Segment SGI88761[] = {{&S52865[71], 6}, {NULL}};
const Segment SGI88762[] = {{&S65326[34], 11}, {NULL}};
const Segment SGI88763[] = {{&S65326[27], 6}, {NULL}};
const Segment SGI88764[] = {{&S65326[104], 2}, {NULL}};
const Segment SGI88765[] = {{&S65326[70], 6}, {NULL}};
const Segment SGI88766[] = {{&S65326[138], 8}, {NULL}};
const Segment SGI88767[] = {{&S73382[44], 16}, {NULL}};
const Segment SGI88768[] = {{&S73382[44], 33}, {NULL}};
const Segment SGI88769[] = {{&S73382[30], 6}, {NULL}};
const Segment SGI88770[] = {{&S73382[125], 3}, {NULL}};
const Segment SGI88771[] = {{&S73382[91], 5}, {NULL}};
const Segment SGI88772[] = {{&S52923[34], 12}, {NULL}};
const Segment SGI88773[] = {{&S52923[27], 6}, {NULL}};
const Segment SGI88774[] = {{&S52923[99], 9}, {NULL}};
const Segment SGI88775[] = {{&S52923[72], 6}, {NULL}};
const Segment SGI88776[] = {{&S52924[34], 11}, {NULL}};
const Segment SGI88777[] = {{&S52924[27], 6}, {NULL}};
const Segment SGI88778[] = {{&S52924[98], 9}, {NULL}};
const Segment SGI88779[] = {{&S52924[71], 6}, {NULL}};
const Segment SGI88780[] = {{&S73417[52], 10}, {NULL}};
const Segment SGI88781[] = {{&S73417[52], 27}, {NULL}};
const Segment SGI88782[] = {{&S73417[38], 6}, {NULL}};
const Segment SGI88783[] = {{&S73417[93], 5}, {NULL}};
const Segment SGI88784[] = {{&S73417[0], 14}, {NULL}};
const Segment SGI88785[] = {{&S73434[45], 10}, {NULL}};
const Segment SGI88786[] = {{&S73434[30], 6}, {NULL}};
const Segment SGI88787[] = {{&S73434[120], 3}, {NULL}};
const Segment SGI88788[] = {{&S73434[86], 5}, {NULL}};
const Segment SGI88789[] = {{&S73434[144], 13}, {NULL}};
const Segment SGI88790[] = {{&S73438[44], 13}, {NULL}};
const Segment SGI88791[] = {{&S73438[44], 25}, {NULL}};
const Segment SGI88792[] = {{&S73438[30], 6}, {NULL}};
const Segment SGI88793[] = {{&S73438[117], 3}, {NULL}};
const Segment SGI88794[] = {{&S73438[83], 5}, {NULL}};
const Segment SGI88795[] = {{&S73482[44], 20}, {NULL}};
const Segment SGI88796[] = {{&S73482[44], 32}, {NULL}};
const Segment SGI88797[] = {{&S73482[30], 6}, {NULL}};
const Segment SGI88798[] = {{&S73482[124], 3}, {NULL}};
const Segment SGI88799[] = {{&S73482[90], 5}, {NULL}};
const Segment SGI88800[] = {{&S73483[44], 21}, {NULL}};
const Segment SGI88801[] = {{&S73483[44], 33}, {NULL}};
const Segment SGI88802[] = {{&S73483[30], 6}, {NULL}};
const Segment SGI88803[] = {{&S73483[125], 3}, {NULL}};
const Segment SGI88804[] = {{&S73483[91], 5}, {NULL}};
const Segment SGI88805[] = {{&S65459[44], 8}, {NULL}};
const Segment SGI88806[] = {{&S65459[44], 23}, {NULL}};
const Segment SGI88807[] = {{&S65459[30], 6}, {NULL}};
const Segment SGI88808[] = {{&S65459[115], 3}, {NULL}};
const Segment SGI88809[] = {{&S65459[81], 5}, {NULL}};
const Segment SGI88810[] = {{&S53015[34], 10}, {NULL}};
const Segment SGI88811[] = {{&S53015[27], 6}, {NULL}};
const Segment SGI88812[] = {{&S53015[97], 9}, {NULL}};
const Segment SGI88813[] = {{&S53015[70], 6}, {NULL}};
const Segment SGI88814[] = {{&S65422[42], 7}, {NULL}};
const Segment SGI88815[] = {{&S65422[42], 18}, {NULL}};
const Segment SGI88816[] = {{&S65422[30], 4}, {NULL}};
const Segment SGI88817[] = {{&S65422[109], 3}, {NULL}};
const Segment SGI88818[] = {{&S65422[74], 6}, {NULL}};
const Segment SGI88819[] = {{&S73514[44], 6}, {NULL}};
const Segment SGI88820[] = {{&S73514[44], 18}, {NULL}};
const Segment SGI88821[] = {{&S73514[30], 6}, {NULL}};
const Segment SGI88822[] = {{&S73514[110], 3}, {NULL}};
const Segment SGI88823[] = {{&S73514[76], 5}, {NULL}};
const Segment SGI88824[] = {{&S53051[44], 7}, {NULL}};
const Segment SGI88825[] = {{&S53051[44], 19}, {NULL}};
const Segment SGI88826[] = {{&S53051[30], 6}, {NULL}};
const Segment SGI88827[] = {{&S53051[31], 3}, {NULL}};
const Segment SGI88828[] = {{&S53051[77], 6}, {NULL}};
const Segment SGI88829[] = {{&S53052[38], 7}, {NULL}};
const Segment SGI88830[] = {{&S53052[31], 6}, {NULL}};
const Segment SGI88831[] = {{&S53053[44], 7}, {NULL}};
const Segment SGI88832[] = {{&S53053[44], 19}, {NULL}};
const Segment SGI88833[] = {{&S53053[30], 6}, {NULL}};
const Segment SGI88834[] = {{&S53053[31], 3}, {NULL}};
const Segment SGI88835[] = {{&S53053[77], 6}, {NULL}};
const Segment SGI88836[] = {{&S60258[70], 6}, {NULL}};
const Segment SGI88837[] = {{&S60258[0], 5}, {NULL}};
const Segment SGI88838[] = {{&S73567[44], 23}, {NULL}};
const Segment SGI88839[] = {{&S73567[44], 35}, {NULL}};
const Segment SGI88840[] = {{&S73567[30], 6}, {NULL}};
const Segment SGI88841[] = {{&S73567[127], 3}, {NULL}};
const Segment SGI88842[] = {{&S73567[93], 5}, {NULL}};
const Segment SGI88843[] = {{&S73569[44], 23}, {NULL}};
const Segment SGI88844[] = {{&S73569[44], 35}, {NULL}};
const Segment SGI88845[] = {{&S73569[30], 6}, {NULL}};
const Segment SGI88846[] = {{&S73569[127], 3}, {NULL}};
const Segment SGI88847[] = {{&S73569[93], 5}, {NULL}};
const Segment SGI88848[] = {{&S73573[44], 9}, {NULL}};
const Segment SGI88849[] = {{&S73573[44], 33}, {NULL}};
const Segment SGI88850[] = {{&S73573[30], 6}, {NULL}};
const Segment SGI88851[] = {{&S73573[125], 3}, {NULL}};
const Segment SGI88852[] = {{&S73573[91], 5}, {NULL}};
const Segment SGI88853[] = {{&S73576[44], 14}, {NULL}};
const Segment SGI88854[] = {{&S73576[44], 31}, {NULL}};
const Segment SGI88855[] = {{&S73576[30], 6}, {NULL}};
const Segment SGI88856[] = {{&S73576[135], 3}, {NULL}};
const Segment SGI88857[] = {{&S73576[89], 5}, {NULL}};
const Segment SGI88858[] = {{&S73580[44], 17}, {NULL}};
const Segment SGI88859[] = {{&S73580[44], 38}, {NULL}};
const Segment SGI88860[] = {{&S73580[30], 6}, {NULL}};
const Segment SGI88861[] = {{&S73580[130], 3}, {NULL}};
const Segment SGI88862[] = {{&S73580[96], 5}, {NULL}};
const Segment SGI88863[] = {{&S53172[34], 11}, {NULL}};
const Segment SGI88864[] = {{&S73645[44], 20}, {NULL}};
const Segment SGI88865[] = {{&S53172[27], 6}, {NULL}};
const Segment SGI88866[] = {{&S73645[44], 46}, {NULL}};
const Segment SGI88867[] = {{&S73645[30], 6}, {NULL}};
const Segment SGI88868[] = {{&S53172[98], 9}, {NULL}};
const Segment SGI88869[] = {{&S53172[71], 6}, {NULL}};
const Segment SGI88870[] = {{&S73645[138], 3}, {NULL}};
const Segment SGI88871[] = {{&S73645[104], 5}, {NULL}};
const Segment SGI88872[] = {{&S53173[34], 11}, {NULL}};
const Segment SGI88873[] = {{&S53173[27], 6}, {NULL}};
const Segment SGI88874[] = {{&S73646[44], 20}, {NULL}};
const Segment SGI88875[] = {{&S73646[44], 46}, {NULL}};
const Segment SGI88876[] = {{&S73646[30], 6}, {NULL}};
const Segment SGI88877[] = {{&S53173[98], 9}, {NULL}};
const Segment SGI88878[] = {{&S53173[71], 6}, {NULL}};
const Segment SGI88879[] = {{&S73646[107], 4}, {NULL}};
const Segment SGI88880[] = {{&S73646[107], 3}, {NULL}};
const Segment SGI88881[] = {{&S73647[44], 20}, {NULL}};
const Segment SGI88882[] = {{&S73647[44], 32}, {NULL}};
const Segment SGI88883[] = {{&S73647[30], 6}, {NULL}};
const Segment SGI88884[] = {{&S73647[124], 3}, {NULL}};
const Segment SGI88885[] = {{&S73647[90], 5}, {NULL}};
const Segment SGI88886[] = {{&S73648[44], 20}, {NULL}};
const Segment SGI88887[] = {{&S73648[44], 46}, {NULL}};
const Segment SGI88888[] = {{&S73648[30], 6}, {NULL}};
const Segment SGI88889[] = {{&S73648[107], 6}, {NULL}};
const Segment SGI88890[] = {{&S73649[44], 20}, {NULL}};
const Segment SGI88891[] = {{&S73649[44], 46}, {NULL}};
const Segment SGI88892[] = {{&S73649[30], 6}, {NULL}};
const Segment SGI88893[] = {{&S73649[138], 3}, {NULL}};
const Segment SGI88894[] = {{&S73649[104], 5}, {NULL}};
const Segment SGI88895[] = {{&S73650[44], 20}, {NULL}};
const Segment SGI88896[] = {{&S73650[44], 46}, {NULL}};
const Segment SGI88897[] = {{&S73650[30], 6}, {NULL}};
const Segment SGI88898[] = {{&S73650[83], 3}, {NULL}};
const Segment SGI88899[] = {{&S73650[104], 5}, {NULL}};
const Segment SGI88900[] = {{&S73651[44], 20}, {NULL}};
const Segment SGI88901[] = {{&S73651[44], 46}, {NULL}};
const Segment SGI88902[] = {{&S73651[30], 6}, {NULL}};
const Segment SGI88903[] = {{&S73651[107], 6}, {NULL}};
const Segment SGI88904[] = {{&S73652[44], 20}, {NULL}};
const Segment SGI88905[] = {{&S73652[44], 46}, {NULL}};
const Segment SGI88906[] = {{&S73652[30], 6}, {NULL}};
const Segment SGI88907[] = {{&S73652[107], 4}, {NULL}};
const Segment SGI88908[] = {{&S73652[107], 3}, {NULL}};
const Segment SGI88909[] = {{&S65670[44], 17}, {NULL}};
const Segment SGI88910[] = {{&S65670[44], 33}, {NULL}};
const Segment SGI88911[] = {{&S65670[30], 6}, {NULL}};
const Segment SGI88912[] = {{&S65670[135], 3}, {NULL}};
const Segment SGI88913[] = {{&S65670[101], 5}, {NULL}};
const Segment SGI88914[] = {{&S65689[6], 4}, {NULL}};
const Segment SGI88915[] = {{&S65689[85], 5}, {NULL}};
const Segment SGI88916[] = {{&S53256[44], 9}, {NULL}};
const Segment SGI88917[] = {{&S53256[44], 21}, {NULL}};
const Segment SGI88918[] = {{&S53256[30], 6}, {NULL}};
const Segment SGI88919[] = {{&S53256[31], 3}, {NULL}};
const Segment SGI88920[] = {{&S53256[79], 6}, {NULL}};
const Segment SGI88921[] = {{&S53257[42], 9}, {NULL}};
const Segment SGI88922[] = {{&S53257[30], 6}, {NULL}};
const Segment SGI88923[] = {{&S53257[31], 3}, {NULL}};
const Segment SGI88924[] = {{&S53257[77], 6}, {NULL}};
const Segment SGI88925[] = {{&S65682[44], 9}, {NULL}};
const Segment SGI88926[] = {{&S65682[44], 21}, {NULL}};
const Segment SGI88927[] = {{&S65682[30], 6}, {NULL}};
const Segment SGI88928[] = {{&S65682[114], 3}, {NULL}};
const Segment SGI88929[] = {{&S65682[79], 6}, {NULL}};
const Segment SGI88930[] = {{&S65683[38], 9}, {NULL}};
const Segment SGI88931[] = {{&S65683[31], 6}, {NULL}};
const Segment SGI88932[] = {{&S65782[44], 17}, {NULL}};
const Segment SGI88933[] = {{&S65782[44], 28}, {NULL}};
const Segment SGI88934[] = {{&S65782[30], 6}, {NULL}};
const Segment SGI88935[] = {{&S65782[120], 3}, {NULL}};
const Segment SGI88936[] = {{&S65782[86], 5}, {NULL}};
const Segment SGI88937[] = {{&S73656[44], 10}, {NULL}};
const Segment SGI88938[] = {{&S73656[44], 22}, {NULL}};
const Segment SGI88939[] = {{&S73656[30], 6}, {NULL}};
const Segment SGI88940[] = {{&S65783[44], 23}, {NULL}};
const Segment SGI88941[] = {{&S65783[44], 34}, {NULL}};
const Segment SGI88942[] = {{&S65783[30], 6}, {NULL}};
const Segment SGI88943[] = {{&S73656[114], 3}, {NULL}};
const Segment SGI88944[] = {{&S73656[80], 5}, {NULL}};
const Segment SGI88945[] = {{&S65783[126], 3}, {NULL}};
const Segment SGI88946[] = {{&S65783[92], 5}, {NULL}};
const Segment SGI88947[] = {{&S65784[44], 28}, {NULL}};
const Segment SGI88948[] = {{&S65784[44], 39}, {NULL}};
const Segment SGI88949[] = {{&S65784[30], 6}, {NULL}};
const Segment SGI88950[] = {{&S65784[131], 3}, {NULL}};
const Segment SGI88951[] = {{&S65784[97], 5}, {NULL}};
const Segment SGI88952[] = {{&S65785[38], 17}, {NULL}};
const Segment SGI88953[] = {{&S65785[31], 6}, {NULL}};
const Segment SGI88954[] = {{&S65786[44], 23}, {NULL}};
const Segment SGI88955[] = {{&S65786[44], 34}, {NULL}};
const Segment SGI88956[] = {{&S65786[30], 6}, {NULL}};
const Segment SGI88957[] = {{&S65786[126], 3}, {NULL}};
const Segment SGI88958[] = {{&S65786[92], 5}, {NULL}};
const Segment SGI88959[] = {{&S65787[44], 28}, {NULL}};
const Segment SGI88960[] = {{&S65787[44], 39}, {NULL}};
const Segment SGI88961[] = {{&S65787[30], 6}, {NULL}};
const Segment SGI88962[] = {{&S65787[131], 3}, {NULL}};
const Segment SGI88963[] = {{&S65787[97], 5}, {NULL}};
const Segment SGI88964[] = {{&S65788[44], 28}, {NULL}};
const Segment SGI88965[] = {{&S65788[44], 39}, {NULL}};
const Segment SGI88966[] = {{&S65788[30], 6}, {NULL}};
const Segment SGI88967[] = {{&S65788[131], 3}, {NULL}};
const Segment SGI88968[] = {{&S65788[97], 5}, {NULL}};
const Segment SGI88969[] = {{&S65789[44], 28}, {NULL}};
const Segment SGI88970[] = {{&S65789[44], 39}, {NULL}};
const Segment SGI88971[] = {{&S65789[30], 6}, {NULL}};
const Segment SGI88972[] = {{&S73669[44], 11}, {NULL}};
const Segment SGI88973[] = {{&S65789[131], 3}, {NULL}};
const Segment SGI88974[] = {{&S73669[44], 28}, {NULL}};
const Segment SGI88975[] = {{&S73669[30], 6}, {NULL}};
const Segment SGI88976[] = {{&S65789[97], 5}, {NULL}};
const Segment SGI88977[] = {{&S73669[119], 3}, {NULL}};
const Segment SGI88978[] = {{&S73669[86], 5}, {NULL}};
const Segment SGI88979[] = {{&S73669[123], 14}, {NULL}};
const Segment SGI88980[] = {{&S73671[44], 10}, {NULL}};
const Segment SGI88981[] = {{&S73671[44], 27}, {NULL}};
const Segment SGI88982[] = {{&S73671[30], 6}, {NULL}};
const Segment SGI88983[] = {{&S73671[118], 3}, {NULL}};
const Segment SGI88984[] = {{&S73671[85], 5}, {NULL}};
const Segment SGI88985[] = {{&S73671[122], 14}, {NULL}};
const Segment SGI88986[] = {{&S73674[44], 10}, {NULL}};
const Segment SGI88987[] = {{&S73674[44], 27}, {NULL}};
const Segment SGI88988[] = {{&S73674[30], 6}, {NULL}};
const Segment SGI88989[] = {{&S73674[118], 3}, {NULL}};
const Segment SGI88990[] = {{&S73674[85], 5}, {NULL}};
const Segment SGI88991[] = {{&S73674[122], 14}, {NULL}};
const Segment SGI88992[] = {{&S53411[44], 13}, {NULL}};
const Segment SGI88993[] = {{&S53411[44], 25}, {NULL}};
const Segment SGI88994[] = {{&S53411[30], 6}, {NULL}};
const Segment SGI88995[] = {{&S53411[31], 3}, {NULL}};
const Segment SGI88996[] = {{&S53411[83], 6}, {NULL}};
const Segment SGI88997[] = {{&S53412[44], 13}, {NULL}};
const Segment SGI88998[] = {{&S53412[44], 25}, {NULL}};
const Segment SGI88999[] = {{&S53412[30], 6}, {NULL}};
const Segment SGI89000[] = {{&S53412[86], 6}, {NULL}};
const Segment SGI89001[] = {{&S53413[38], 13}, {NULL}};
const Segment SGI89002[] = {{&S53413[31], 6}, {NULL}};
const Segment SGI89003[] = {{&S53418[34], 11}, {NULL}};
const Segment SGI89004[] = {{&S53418[27], 6}, {NULL}};
const Segment SGI89005[] = {{&S53418[98], 9}, {NULL}};
const Segment SGI89006[] = {{&S53418[71], 6}, {NULL}};
const Segment SGI89007[] = {{&S73815[57], 4}, {NULL}};
const Segment SGI89008[] = {{&S73815[57], 16}, {NULL}};
const Segment SGI89009[] = {{&S73815[43], 6}, {NULL}};
const Segment SGI89010[] = {{&S73815[121], 3}, {NULL}};
const Segment SGI89011[] = {{&S73815[87], 5}, {NULL}};
const Segment SGI89012[] = {{&S73815[0], 12}, {NULL}};
const Segment SGI89013[] = {{&S73819[44], 22}, {NULL}};
const Segment SGI89014[] = {{&S73819[44], 34}, {NULL}};
const Segment SGI89015[] = {{&S73819[30], 6}, {NULL}};
const Segment SGI89016[] = {{&S73819[126], 3}, {NULL}};
const Segment SGI89017[] = {{&S73819[92], 5}, {NULL}};
const Segment SGI89018[] = {{&S53432[34], 18}, {NULL}};
const Segment SGI89019[] = {{&S73820[44], 20}, {NULL}};
const Segment SGI89020[] = {{&S53432[27], 6}, {NULL}};
const Segment SGI89021[] = {{&S73820[44], 32}, {NULL}};
const Segment SGI89022[] = {{&S73820[30], 6}, {NULL}};
const Segment SGI89023[] = {{&S53432[105], 9}, {NULL}};
const Segment SGI89024[] = {{&S73820[124], 3}, {NULL}};
const Segment SGI89025[] = {{&S53432[78], 6}, {NULL}};
const Segment SGI89026[] = {{&S73820[90], 5}, {NULL}};
const Segment SGI89027[] = {{&S53439[34], 15}, {NULL}};
const Segment SGI89028[] = {{&S53439[27], 6}, {NULL}};
const Segment SGI89029[] = {{&S53439[104], 9}, {NULL}};
const Segment SGI89030[] = {{&S53439[77], 6}, {NULL}};
const Segment SGI89031[] = {{&S60710[34], 9}, {NULL}};
const Segment SGI89032[] = {{&S60710[27], 6}, {NULL}};
const Segment SGI89033[] = {{&S60710[95], 9}, {NULL}};
const Segment SGI89034[] = {{&S60710[68], 6}, {NULL}};
const Segment SGI89035[] = {{&S60710[0], 13}, {NULL}};
const Segment SGI89036[] = {{&S53468[44], 12}, {NULL}};
const Segment SGI89037[] = {{&S53468[44], 24}, {NULL}};
const Segment SGI89038[] = {{&S53468[30], 6}, {NULL}};
const Segment SGI89039[] = {{&S53468[31], 3}, {NULL}};
const Segment SGI89040[] = {{&S53468[82], 6}, {NULL}};
const Segment SGI89041[] = {{&S53470[44], 12}, {NULL}};
const Segment SGI89042[] = {{&S53470[44], 24}, {NULL}};
const Segment SGI89043[] = {{&S53470[30], 6}, {NULL}};
const Segment SGI89044[] = {{&S53470[85], 4}, {NULL}};
const Segment SGI89045[] = {{&S53470[85], 3}, {NULL}};
const Segment SGI89046[] = {{&S53472[44], 12}, {NULL}};
const Segment SGI89047[] = {{&S53472[44], 33}, {NULL}};
const Segment SGI89048[] = {{&S53472[30], 6}, {NULL}};
const Segment SGI89049[] = {{&S53472[31], 3}, {NULL}};
const Segment SGI89050[] = {{&S53472[91], 5}, {NULL}};
const Segment SGI89051[] = {{&S53510[34], 11}, {NULL}};
const Segment SGI89052[] = {{&S53510[27], 6}, {NULL}};
const Segment SGI89053[] = {{&S53510[98], 9}, {NULL}};
const Segment SGI89054[] = {{&S53510[71], 6}, {NULL}};
const Segment SGI89055[] = {{&S53553[34], 10}, {NULL}};
const Segment SGI89056[] = {{&S53553[27], 6}, {NULL}};
const Segment SGI89057[] = {{&S53553[97], 9}, {NULL}};
const Segment SGI89058[] = {{&S53553[70], 6}, {NULL}};
const Segment SGI89059[] = {{&S66004[42], 7}, {NULL}};
const Segment SGI89060[] = {{&S66004[42], 21}, {NULL}};
const Segment SGI89061[] = {{&S66004[30], 4}, {NULL}};
const Segment SGI89062[] = {{&S66004[112], 3}, {NULL}};
const Segment SGI89063[] = {{&S66004[77], 6}, {NULL}};
const Segment SGI89064[] = {{&S66005[42], 7}, {NULL}};
const Segment SGI89065[] = {{&S66005[42], 21}, {NULL}};
const Segment SGI89066[] = {{&S66005[30], 4}, {NULL}};
const Segment SGI89067[] = {{&S66005[112], 3}, {NULL}};
const Segment SGI89068[] = {{&S66005[77], 6}, {NULL}};
const Segment SGI89069[] = {{&S66006[42], 7}, {NULL}};
const Segment SGI89070[] = {{&S66006[42], 20}, {NULL}};
const Segment SGI89071[] = {{&S66006[30], 4}, {NULL}};
const Segment SGI89072[] = {{&S66006[111], 3}, {NULL}};
const Segment SGI89073[] = {{&S66006[76], 6}, {NULL}};
const Segment SGI89074[] = {{&S66007[42], 7}, {NULL}};
const Segment SGI89075[] = {{&S66007[42], 23}, {NULL}};
const Segment SGI89076[] = {{&S66007[30], 4}, {NULL}};
const Segment SGI89077[] = {{&S66007[114], 3}, {NULL}};
const Segment SGI89078[] = {{&S66007[79], 6}, {NULL}};
const Segment SGI89079[] = {{&S66008[42], 7}, {NULL}};
const Segment SGI89080[] = {{&S66008[42], 23}, {NULL}};
const Segment SGI89081[] = {{&S66008[30], 4}, {NULL}};
const Segment SGI89082[] = {{&S66008[114], 3}, {NULL}};
const Segment SGI89083[] = {{&S66008[79], 6}, {NULL}};
const Segment SGI89084[] = {{&S53587[44], 8}, {NULL}};
const Segment SGI89085[] = {{&S53587[44], 20}, {NULL}};
const Segment SGI89086[] = {{&S53587[30], 6}, {NULL}};
const Segment SGI89087[] = {{&S53587[31], 3}, {NULL}};
const Segment SGI89088[] = {{&S53587[78], 6}, {NULL}};
const Segment SGI89089[] = {{&S53588[42], 8}, {NULL}};
const Segment SGI89090[] = {{&S53588[30], 6}, {NULL}};
const Segment SGI89091[] = {{&S53588[31], 3}, {NULL}};
const Segment SGI89092[] = {{&S53588[76], 6}, {NULL}};
const Segment SGI89093[] = {{&S53589[38], 8}, {NULL}};
const Segment SGI89094[] = {{&S53589[31], 6}, {NULL}};
const Segment SGI89095[] = {{&S53593[34], 15}, {NULL}};
const Segment SGI89096[] = {{&S53593[27], 6}, {NULL}};
const Segment SGI89097[] = {{&S53593[99], 9}, {NULL}};
const Segment SGI89098[] = {{&S53593[72], 6}, {NULL}};
const Segment SGI89099[] = {{&S53594[34], 14}, {NULL}};
const Segment SGI89100[] = {{&S53594[27], 6}, {NULL}};
const Segment SGI89101[] = {{&S53594[98], 9}, {NULL}};
const Segment SGI89102[] = {{&S53594[71], 6}, {NULL}};
const Segment SGI89103[] = {{&S53598[44], 11}, {NULL}};
const Segment SGI89104[] = {{&S53598[44], 23}, {NULL}};
const Segment SGI89105[] = {{&S53598[30], 6}, {NULL}};
const Segment SGI89106[] = {{&S53598[31], 3}, {NULL}};
const Segment SGI89107[] = {{&S53598[81], 6}, {NULL}};
const Segment SGI89108[] = {{&S73997[44], 10}, {NULL}};
const Segment SGI89109[] = {{&S73997[44], 27}, {NULL}};
const Segment SGI89110[] = {{&S73997[30], 6}, {NULL}};
const Segment SGI89111[] = {{&S73997[119], 3}, {NULL}};
const Segment SGI89112[] = {{&S73997[85], 5}, {NULL}};
const Segment SGI89113[] = {{&S73998[44], 11}, {NULL}};
const Segment SGI89114[] = {{&S73998[44], 28}, {NULL}};
const Segment SGI89115[] = {{&S73998[30], 6}, {NULL}};
const Segment SGI89116[] = {{&S73998[120], 3}, {NULL}};
const Segment SGI89117[] = {{&S73998[86], 5}, {NULL}};
const Segment SGI89118[] = {{&S73999[44], 10}, {NULL}};
const Segment SGI89119[] = {{&S73999[44], 27}, {NULL}};
const Segment SGI89120[] = {{&S73999[30], 6}, {NULL}};
const Segment SGI89121[] = {{&S73999[88], 6}, {NULL}};
const Segment SGI89122[] = {{&S60886[74], 6}, {NULL}};
const Segment SGI89123[] = {{&S60886[0], 9}, {NULL}};
const Segment SGI89124[] = {{&S74001[44], 11}, {NULL}};
const Segment SGI89125[] = {{&S74001[44], 23}, {NULL}};
const Segment SGI89126[] = {{&S74001[30], 6}, {NULL}};
const Segment SGI89127[] = {{&S74001[115], 3}, {NULL}};
const Segment SGI89128[] = {{&S74001[81], 5}, {NULL}};
const Segment SGI89129[] = {{&S74012[44], 12}, {NULL}};
const Segment SGI89130[] = {{&S74012[44], 33}, {NULL}};
const Segment SGI89131[] = {{&S74012[30], 6}, {NULL}};
const Segment SGI89132[] = {{&S74012[125], 3}, {NULL}};
const Segment SGI89133[] = {{&S74012[91], 5}, {NULL}};
const Segment SGI89134[] = {{&S53647[34], 15}, {NULL}};
const Segment SGI89135[] = {{&S53647[27], 6}, {NULL}};
const Segment SGI89136[] = {{&S53647[104], 9}, {NULL}};
const Segment SGI89137[] = {{&S53647[77], 6}, {NULL}};
const Segment SGI89138[] = {{&S66192[45], 10}, {NULL}};
const Segment SGI89139[] = {{&S66192[30], 6}, {NULL}};
const Segment SGI89140[] = {{&S66192[115], 3}, {NULL}};
const Segment SGI89141[] = {{&S66192[81], 5}, {NULL}};
const Segment SGI89142[] = {{&S66192[139], 13}, {NULL}};
const Segment SGI89143[] = {{&S53674[34], 6}, {NULL}};
const Segment SGI89144[] = {{&S53674[27], 6}, {NULL}};
const Segment SGI89145[] = {{&S53674[93], 9}, {NULL}};
const Segment SGI89146[] = {{&S53674[66], 6}, {NULL}};
const Segment SGI89147[] = {{&S53683[34], 10}, {NULL}};
const Segment SGI89148[] = {{&S53683[27], 6}, {NULL}};
const Segment SGI89149[] = {{&S53683[97], 9}, {NULL}};
const Segment SGI89150[] = {{&S53683[70], 6}, {NULL}};
const Segment SGI89151[] = {{&S53683[139], 8}, {NULL}};
const Segment SGI89152[] = {{&S74063[44], 7}, {NULL}};
const Segment SGI89153[] = {{&S74063[44], 19}, {NULL}};
const Segment SGI89154[] = {{&S74063[30], 6}, {NULL}};
const Segment SGI89155[] = {{&S53685[34], 10}, {NULL}};
const Segment SGI89156[] = {{&S74063[111], 3}, {NULL}};
const Segment SGI89157[] = {{&S53685[27], 6}, {NULL}};
const Segment SGI89158[] = {{&S74063[77], 5}, {NULL}};
const Segment SGI89159[] = {{&S53685[97], 9}, {NULL}};
const Segment SGI89160[] = {{&S53685[70], 6}, {NULL}};
const Segment SGI89161[] = {{&S53685[138], 8}, {NULL}};
const Segment SGI89162[] = {{&S66224[44], 8}, {NULL}};
const Segment SGI89163[] = {{&S66224[44], 19}, {NULL}};
const Segment SGI89164[] = {{&S66224[30], 6}, {NULL}};
const Segment SGI89165[] = {{&S66224[111], 3}, {NULL}};
const Segment SGI89166[] = {{&S53687[34], 11}, {NULL}};
const Segment SGI89167[] = {{&S66224[77], 5}, {NULL}};
const Segment SGI89168[] = {{&S53687[27], 6}, {NULL}};
const Segment SGI89169[] = {{&S53687[98], 9}, {NULL}};
const Segment SGI89170[] = {{&S53687[71], 6}, {NULL}};
const Segment SGI89171[] = {{&S53687[139], 8}, {NULL}};
const Segment SGI89172[] = {{&S53688[34], 11}, {NULL}};
const Segment SGI89173[] = {{&S53688[27], 6}, {NULL}};
const Segment SGI89174[] = {{&S53688[98], 9}, {NULL}};
const Segment SGI89175[] = {{&S53688[71], 6}, {NULL}};
const Segment SGI89176[] = {{&S53688[139], 8}, {NULL}};
const Segment SGI89177[] = {{&S66228[38], 8}, {NULL}};
const Segment SGI89178[] = {{&S66228[31], 6}, {NULL}};
const Segment SGI89179[] = {{&S53689[34], 11}, {NULL}};
const Segment SGI89180[] = {{&S53689[27], 6}, {NULL}};
const Segment SGI89181[] = {{&S53689[98], 9}, {NULL}};
const Segment SGI89182[] = {{&S53689[71], 6}, {NULL}};
const Segment SGI89183[] = {{&S53689[139], 8}, {NULL}};
const Segment SGI89184[] = {{&S53700[44], 7}, {NULL}};
const Segment SGI89185[] = {{&S53700[44], 19}, {NULL}};
const Segment SGI89186[] = {{&S53700[30], 6}, {NULL}};
const Segment SGI89187[] = {{&S53700[31], 3}, {NULL}};
const Segment SGI89188[] = {{&S53700[77], 6}, {NULL}};
const Segment SGI89189[] = {{&S53717[34], 14}, {NULL}};
const Segment SGI89190[] = {{&S53717[27], 6}, {NULL}};
const Segment SGI89191[] = {{&S53717[101], 9}, {NULL}};
const Segment SGI89192[] = {{&S53717[74], 6}, {NULL}};
const Segment SGI89193[] = {{&S53718[34], 10}, {NULL}};
const Segment SGI89194[] = {{&S53718[27], 6}, {NULL}};
const Segment SGI89195[] = {{&S53718[97], 9}, {NULL}};
const Segment SGI89196[] = {{&S53718[70], 6}, {NULL}};
const Segment SGI89197[] = {{&S53719[34], 14}, {NULL}};
const Segment SGI89198[] = {{&S53719[27], 6}, {NULL}};
const Segment SGI89199[] = {{&S53719[101], 9}, {NULL}};
const Segment SGI89200[] = {{&S53719[74], 6}, {NULL}};
const Segment SGI89201[] = {{&S53720[34], 7}, {NULL}};
const Segment SGI89202[] = {{&S53720[27], 6}, {NULL}};
const Segment SGI89203[] = {{&S53720[94], 9}, {NULL}};
const Segment SGI89204[] = {{&S53720[67], 6}, {NULL}};
const Segment SGI89205[] = {{&S66274[44], 27}, {NULL}};
const Segment SGI89206[] = {{&S66274[44], 38}, {NULL}};
const Segment SGI89207[] = {{&S66274[30], 6}, {NULL}};
const Segment SGI89208[] = {{&S66274[130], 3}, {NULL}};
const Segment SGI89209[] = {{&S66274[96], 5}, {NULL}};
const Segment SGI89210[] = {{&S74145[45], 11}, {NULL}};
const Segment SGI89211[] = {{&S74145[30], 6}, {NULL}};
const Segment SGI89212[] = {{&S74145[121], 3}, {NULL}};
const Segment SGI89213[] = {{&S74145[87], 5}, {NULL}};
const Segment SGI89214[] = {{&S74145[145], 13}, {NULL}};
const Segment SGI89215[] = {{&S74154[44], 9}, {NULL}};
const Segment SGI89216[] = {{&S74154[44], 22}, {NULL}};
const Segment SGI89217[] = {{&S74154[30], 6}, {NULL}};
const Segment SGI89218[] = {{&S74154[114], 3}, {NULL}};
const Segment SGI89219[] = {{&S74154[80], 5}, {NULL}};
const Segment SGI89220[] = {{&S74162[44], 11}, {NULL}};
const Segment SGI89221[] = {{&S74162[44], 32}, {NULL}};
const Segment SGI89222[] = {{&S74162[30], 6}, {NULL}};
const Segment SGI89223[] = {{&S74162[124], 3}, {NULL}};
const Segment SGI89224[] = {{&S74162[90], 5}, {NULL}};
const Segment SGI89225[] = {{&S66137[42], 10}, {NULL}};
const Segment SGI89226[] = {{&S66137[42], 22}, {NULL}};
const Segment SGI89227[] = {{&S66137[30], 4}, {NULL}};
const Segment SGI89228[] = {{&S66137[113], 3}, {NULL}};
const Segment SGI89229[] = {{&S66137[78], 6}, {NULL}};
const Segment SGI89230[] = {{&S66138[36], 10}, {NULL}};
const Segment SGI89231[] = {{&S66138[31], 4}, {NULL}};
const Segment SGI89232[] = {{&S66139[40], 10}, {NULL}};
const Segment SGI89233[] = {{&S66139[30], 4}, {NULL}};
const Segment SGI89234[] = {{&S66139[111], 3}, {NULL}};
const Segment SGI89235[] = {{&S66139[76], 6}, {NULL}};
const Segment SGI89236[] = {{&S61036[77], 6}, {NULL}};
const Segment SGI89237[] = {{&S61036[0], 12}, {NULL}};
const Segment SGI89238[] = {{&S74207[34], 10}, {NULL}};
const Segment SGI89239[] = {{&S74207[27], 6}, {NULL}};
const Segment SGI89240[] = {{&S74207[109], 2}, {NULL}};
const Segment SGI89241[] = {{&S74207[75], 6}, {NULL}};
const Segment SGI89242[] = {{&S53808[34], 11}, {NULL}};
const Segment SGI89243[] = {{&S53808[27], 6}, {NULL}};
const Segment SGI89244[] = {{&S74207[143], 8}, {NULL}};
const Segment SGI89245[] = {{&S53808[98], 9}, {NULL}};
const Segment SGI89246[] = {{&S53808[71], 6}, {NULL}};
const Segment SGI89247[] = {{&S74300[45], 10}, {NULL}};
const Segment SGI89248[] = {{&S74300[30], 6}, {NULL}};
const Segment SGI89249[] = {{&S74300[120], 3}, {NULL}};
const Segment SGI89250[] = {{&S74300[86], 5}, {NULL}};
const Segment SGI89251[] = {{&S74300[144], 13}, {NULL}};
const Segment SGI89252[] = {{&S53900[44], 8}, {NULL}};
const Segment SGI89253[] = {{&S53900[44], 20}, {NULL}};
const Segment SGI89254[] = {{&S53900[30], 6}, {NULL}};
const Segment SGI89255[] = {{&S53900[31], 3}, {NULL}};
const Segment SGI89256[] = {{&S53900[78], 6}, {NULL}};
const Segment SGI89257[] = {{&S53901[44], 7}, {NULL}};
const Segment SGI89258[] = {{&S53901[44], 19}, {NULL}};
const Segment SGI89259[] = {{&S53901[30], 6}, {NULL}};
const Segment SGI89260[] = {{&S53901[31], 3}, {NULL}};
const Segment SGI89261[] = {{&S53901[77], 6}, {NULL}};
const Segment SGI89262[] = {{&S53902[38], 8}, {NULL}};
const Segment SGI89263[] = {{&S53902[31], 6}, {NULL}};
const Segment SGI89264[] = {{&S74352[45], 24}, {NULL}};
const Segment SGI89265[] = {{&S74352[30], 6}, {NULL}};
const Segment SGI89266[] = {{&S74352[129], 3}, {NULL}};
const Segment SGI89267[] = {{&S74352[95], 5}, {NULL}};
const Segment SGI89268[] = {{&S74352[153], 13}, {NULL}};
const Segment SGI89269[] = {{&S53951[34], 18}, {NULL}};
const Segment SGI89270[] = {{&S53951[27], 6}, {NULL}};
const Segment SGI89271[] = {{&S53951[105], 9}, {NULL}};
const Segment SGI89272[] = {{&S53951[78], 6}, {NULL}};
const Segment SGI89273[] = {{&S74368[44], 10}, {NULL}};
const Segment SGI89274[] = {{&S74368[44], 27}, {NULL}};
const Segment SGI89275[] = {{&S74368[30], 6}, {NULL}};
const Segment SGI89276[] = {{&S74368[85], 6}, {NULL}};
const Segment SGI89277[] = {{&S74368[112], 13}, {NULL}};
const Segment SGI89278[] = {{&S53953[34], 14}, {NULL}};
const Segment SGI89279[] = {{&S53953[27], 6}, {NULL}};
const Segment SGI89280[] = {{&S53953[101], 9}, {NULL}};
const Segment SGI89281[] = {{&S53953[74], 6}, {NULL}};
const Segment SGI89282[] = {{&S74394[44], 10}, {NULL}};
const Segment SGI89283[] = {{&S74394[44], 27}, {NULL}};
const Segment SGI89284[] = {{&S74394[30], 6}, {NULL}};
const Segment SGI89285[] = {{&S74394[85], 6}, {NULL}};
const Segment SGI89286[] = {{&S74394[112], 13}, {NULL}};
const Segment SGI89287[] = {{&S54009[34], 11}, {NULL}};
const Segment SGI89288[] = {{&S54009[27], 6}, {NULL}};
const Segment SGI89289[] = {{&S54009[98], 9}, {NULL}};
const Segment SGI89290[] = {{&S54009[71], 6}, {NULL}};
const Segment SGI89291[] = {{&S74446[44], 10}, {NULL}};
const Segment SGI89292[] = {{&S74446[44], 27}, {NULL}};
const Segment SGI89293[] = {{&S74446[30], 6}, {NULL}};
const Segment SGI89294[] = {{&S74446[119], 3}, {NULL}};
const Segment SGI89295[] = {{&S74446[85], 5}, {NULL}};
const Segment SGI89296[] = {{&S74449[44], 20}, {NULL}};
const Segment SGI89297[] = {{&S74449[44], 32}, {NULL}};
const Segment SGI89298[] = {{&S74449[30], 6}, {NULL}};
const Segment SGI89299[] = {{&S74449[124], 3}, {NULL}};
const Segment SGI89300[] = {{&S74449[90], 5}, {NULL}};
const Segment SGI89301[] = {{&S74450[44], 21}, {NULL}};
const Segment SGI89302[] = {{&S74450[44], 33}, {NULL}};
const Segment SGI89303[] = {{&S74450[30], 6}, {NULL}};
const Segment SGI89304[] = {{&S74450[125], 3}, {NULL}};
const Segment SGI89305[] = {{&S74450[91], 5}, {NULL}};
const Segment SGI89306[] = {{&S66628[44], 7}, {NULL}};
const Segment SGI89307[] = {{&S66628[44], 18}, {NULL}};
const Segment SGI89308[] = {{&S66628[30], 6}, {NULL}};
const Segment SGI89309[] = {{&S66628[110], 3}, {NULL}};
const Segment SGI89310[] = {{&S66628[76], 5}, {NULL}};
const Segment SGI89311[] = {{&S66629[38], 7}, {NULL}};
const Segment SGI89312[] = {{&S66629[31], 6}, {NULL}};
const Segment SGI89313[] = {{&S74472[44], 10}, {NULL}};
const Segment SGI89314[] = {{&S74472[44], 27}, {NULL}};
const Segment SGI89315[] = {{&S74472[30], 6}, {NULL}};
const Segment SGI89316[] = {{&S74472[88], 4}, {NULL}};
const Segment SGI89317[] = {{&S74472[88], 3}, {NULL}};
const Segment SGI89318[] = {{&S74474[44], 10}, {NULL}};
const Segment SGI89319[] = {{&S74474[44], 27}, {NULL}};
const Segment SGI89320[] = {{&S74474[30], 6}, {NULL}};
const Segment SGI89321[] = {{&S74474[119], 3}, {NULL}};
const Segment SGI89322[] = {{&S74474[85], 5}, {NULL}};
const Segment SGI89323[] = {{&S74475[44], 10}, {NULL}};
const Segment SGI89324[] = {{&S74475[44], 27}, {NULL}};
const Segment SGI89325[] = {{&S74475[30], 6}, {NULL}};
const Segment SGI89326[] = {{&S74475[88], 6}, {NULL}};
const Segment SGI89327[] = {{&S74478[44], 9}, {NULL}};
const Segment SGI89328[] = {{&S74478[44], 21}, {NULL}};
const Segment SGI89329[] = {{&S74478[30], 6}, {NULL}};
const Segment SGI89330[] = {{&S74478[113], 3}, {NULL}};
const Segment SGI89331[] = {{&S54078[34], 10}, {NULL}};
const Segment SGI89332[] = {{&S74478[79], 5}, {NULL}};
const Segment SGI89333[] = {{&S54078[27], 6}, {NULL}};
const Segment SGI89334[] = {{&S54078[97], 9}, {NULL}};
const Segment SGI89335[] = {{&S54078[70], 6}, {NULL}};
const Segment SGI89336[] = {{&S74479[44], 9}, {NULL}};
const Segment SGI89337[] = {{&S74479[44], 21}, {NULL}};
const Segment SGI89338[] = {{&S74479[30], 6}, {NULL}};
const Segment SGI89339[] = {{&S74479[82], 6}, {NULL}};
const Segment SGI89340[] = {{&S54079[34], 9}, {NULL}};
const Segment SGI89341[] = {{&S54079[27], 6}, {NULL}};
const Segment SGI89342[] = {{&S54079[96], 9}, {NULL}};
const Segment SGI89343[] = {{&S54079[69], 6}, {NULL}};
const Segment SGI89344[] = {{&S74481[44], 15}, {NULL}};
const Segment SGI89345[] = {{&S74481[44], 27}, {NULL}};
const Segment SGI89346[] = {{&S74481[30], 6}, {NULL}};
const Segment SGI89347[] = {{&S74481[119], 3}, {NULL}};
const Segment SGI89348[] = {{&S74481[85], 5}, {NULL}};
const Segment SGI89349[] = {{&S66677[44], 10}, {NULL}};
const Segment SGI89350[] = {{&S66677[44], 27}, {NULL}};
const Segment SGI89351[] = {{&S66677[30], 6}, {NULL}};
const Segment SGI89352[] = {{&S66677[138], 3}, {NULL}};
const Segment SGI89353[] = {{&S66677[104], 5}, {NULL}};
const Segment SGI89354[] = {{&S66677[35], 1}, {NULL}};
const Segment SGI89355[] = {{&S66677[73], 11}, {NULL}};
const Segment SGI89356[] = {{&S54101[34], 7}, {NULL}};
const Segment SGI89357[] = {{&S54101[27], 6}, {NULL}};
const Segment SGI89358[] = {{&S54101[96], 9}, {NULL}};
const Segment SGI89359[] = {{&S54101[69], 6}, {NULL}};
const Segment SGI89360[] = {{&S54117[34], 10}, {NULL}};
const Segment SGI89361[] = {{&S54117[27], 6}, {NULL}};
const Segment SGI89362[] = {{&S54117[97], 9}, {NULL}};
const Segment SGI89363[] = {{&S54117[70], 6}, {NULL}};
const Segment SGI89364[] = {{&S54118[34], 9}, {NULL}};
const Segment SGI89365[] = {{&S54118[27], 6}, {NULL}};
const Segment SGI89366[] = {{&S54118[96], 9}, {NULL}};
const Segment SGI89367[] = {{&S54118[69], 6}, {NULL}};
const Segment SGI89368[] = {{&S54119[34], 10}, {NULL}};
const Segment SGI89369[] = {{&S54119[27], 6}, {NULL}};
const Segment SGI89370[] = {{&S54119[97], 9}, {NULL}};
const Segment SGI89371[] = {{&S54119[70], 6}, {NULL}};
const Segment SGI89372[] = {{&S54120[34], 10}, {NULL}};
const Segment SGI89373[] = {{&S54120[27], 6}, {NULL}};
const Segment SGI89374[] = {{&S54120[97], 9}, {NULL}};
const Segment SGI89375[] = {{&S54120[70], 6}, {NULL}};
const Segment SGI89376[] = {{&S54122[44], 8}, {NULL}};
const Segment SGI89377[] = {{&S54122[44], 20}, {NULL}};
const Segment SGI89378[] = {{&S54122[30], 6}, {NULL}};
const Segment SGI89379[] = {{&S54122[31], 3}, {NULL}};
const Segment SGI89380[] = {{&S54122[78], 6}, {NULL}};
const Segment SGI89381[] = {{&S54123[38], 8}, {NULL}};
const Segment SGI89382[] = {{&S54123[31], 6}, {NULL}};
const Segment SGI89383[] = {{&S74535[57], 10}, {NULL}};
const Segment SGI89384[] = {{&S74535[57], 27}, {NULL}};
const Segment SGI89385[] = {{&S74535[43], 6}, {NULL}};
const Segment SGI89386[] = {{&S74535[132], 3}, {NULL}};
const Segment SGI89387[] = {{&S74535[98], 5}, {NULL}};
const Segment SGI89388[] = {{&S74535[0], 12}, {NULL}};
const Segment SGI89389[] = {{&S66769[44], 6}, {NULL}};
const Segment SGI89390[] = {{&S66769[44], 23}, {NULL}};
const Segment SGI89391[] = {{&S66769[30], 6}, {NULL}};
const Segment SGI89392[] = {{&S66769[56], 3}, {NULL}};
const Segment SGI89393[] = {{&S66769[81], 5}, {NULL}};
const Segment SGI89394[] = {{&S66773[44], 6}, {NULL}};
const Segment SGI89395[] = {{&S66773[44], 24}, {NULL}};
const Segment SGI89396[] = {{&S66773[30], 6}, {NULL}};
const Segment SGI89397[] = {{&S66773[116], 3}, {NULL}};
const Segment SGI89398[] = {{&S66773[82], 5}, {NULL}};
const Segment SGI89399[] = {{&S66774[44], 6}, {NULL}};
const Segment SGI89400[] = {{&S66774[44], 25}, {NULL}};
const Segment SGI89401[] = {{&S66774[30], 6}, {NULL}};
const Segment SGI89402[] = {{&S66774[117], 3}, {NULL}};
const Segment SGI89403[] = {{&S66774[83], 5}, {NULL}};
const Segment SGI89404[] = {{&S74584[44], 7}, {NULL}};
const Segment SGI89405[] = {{&S74584[44], 21}, {NULL}};
const Segment SGI89406[] = {{&S74584[30], 6}, {NULL}};
const Segment SGI89407[] = {{&S74584[82], 6}, {NULL}};
const Segment SGI89408[] = {{&S66775[44], 6}, {NULL}};
const Segment SGI89409[] = {{&S66775[44], 25}, {NULL}};
const Segment SGI89410[] = {{&S66775[30], 6}, {NULL}};
const Segment SGI89411[] = {{&S66775[56], 3}, {NULL}};
const Segment SGI89412[] = {{&S66775[83], 5}, {NULL}};
const Segment SGI89413[] = {{&S74585[44], 7}, {NULL}};
const Segment SGI89414[] = {{&S74585[44], 21}, {NULL}};
const Segment SGI89415[] = {{&S74585[30], 6}, {NULL}};
const Segment SGI89416[] = {{&S74585[113], 3}, {NULL}};
const Segment SGI89417[] = {{&S66776[38], 6}, {NULL}};
const Segment SGI89418[] = {{&S66776[31], 6}, {NULL}};
const Segment SGI89419[] = {{&S74585[79], 5}, {NULL}};
const Segment SGI89420[] = {{&S66791[44], 10}, {NULL}};
const Segment SGI89421[] = {{&S66791[44], 22}, {NULL}};
const Segment SGI89422[] = {{&S66791[30], 6}, {NULL}};
const Segment SGI89423[] = {{&S66791[131], 3}, {NULL}};
const Segment SGI89424[] = {{&S66791[97], 5}, {NULL}};
const Segment SGI89425[] = {{&S66791[35], 1}, {NULL}};
const Segment SGI89426[] = {{&S66791[68], 11}, {NULL}};
const Segment SGI89427[] = {{&S66792[44], 10}, {NULL}};
const Segment SGI89428[] = {{&S66792[44], 21}, {NULL}};
const Segment SGI89429[] = {{&S66792[30], 6}, {NULL}};
const Segment SGI89430[] = {{&S66792[132], 3}, {NULL}};
const Segment SGI89431[] = {{&S66792[98], 5}, {NULL}};
const Segment SGI89432[] = {{&S66792[35], 1}, {NULL}};
const Segment SGI89433[] = {{&S66792[67], 11}, {NULL}};
const Segment SGI89434[] = {{&S66793[44], 10}, {NULL}};
const Segment SGI89435[] = {{&S66793[44], 27}, {NULL}};
const Segment SGI89436[] = {{&S66793[30], 6}, {NULL}};
const Segment SGI89437[] = {{&S66793[134], 3}, {NULL}};
const Segment SGI89438[] = {{&S66793[100], 5}, {NULL}};
const Segment SGI89439[] = {{&S66793[35], 1}, {NULL}};
const Segment SGI89440[] = {{&S66793[73], 11}, {NULL}};
const Segment SGI89441[] = {{&S66794[44], 10}, {NULL}};
const Segment SGI89442[] = {{&S66794[44], 21}, {NULL}};
const Segment SGI89443[] = {{&S66794[30], 6}, {NULL}};
const Segment SGI89444[] = {{&S66794[132], 3}, {NULL}};
const Segment SGI89445[] = {{&S66794[98], 5}, {NULL}};
const Segment SGI89446[] = {{&S66794[35], 1}, {NULL}};
const Segment SGI89447[] = {{&S66794[67], 11}, {NULL}};
const Segment SGI89448[] = {{&S54158[34], 11}, {NULL}};
const Segment SGI89449[] = {{&S54158[27], 6}, {NULL}};
const Segment SGI89450[] = {{&S74608[44], 10}, {NULL}};
const Segment SGI89451[] = {{&S74608[44], 27}, {NULL}};
const Segment SGI89452[] = {{&S66795[44], 7}, {NULL}};
const Segment SGI89453[] = {{&S74608[30], 6}, {NULL}};
const Segment SGI89454[] = {{&S66795[44], 19}, {NULL}};
const Segment SGI89455[] = {{&S54158[98], 9}, {NULL}};
const Segment SGI89456[] = {{&S66795[30], 6}, {NULL}};
const Segment SGI89457[] = {{&S54158[71], 6}, {NULL}};
const Segment SGI89458[] = {{&S66795[111], 3}, {NULL}};
const Segment SGI89459[] = {{&S74608[85], 6}, {NULL}};
const Segment SGI89460[] = {{&S66795[77], 5}, {NULL}};
const Segment SGI89461[] = {{&S74608[112], 13}, {NULL}};
const Segment SGI89462[] = {{&S66796[44], 7}, {NULL}};
const Segment SGI89463[] = {{&S66796[44], 19}, {NULL}};
const Segment SGI89464[] = {{&S66796[30], 6}, {NULL}};
const Segment SGI89465[] = {{&S66796[111], 3}, {NULL}};
const Segment SGI89466[] = {{&S66796[77], 5}, {NULL}};
const Segment SGI89467[] = {{&S54161[34], 11}, {NULL}};
const Segment SGI89468[] = {{&S54161[27], 6}, {NULL}};
const Segment SGI89469[] = {{&S54161[98], 9}, {NULL}};
const Segment SGI89470[] = {{&S54161[71], 6}, {NULL}};
const Segment SGI89471[] = {{&S66797[44], 7}, {NULL}};
const Segment SGI89472[] = {{&S66797[44], 19}, {NULL}};
const Segment SGI89473[] = {{&S66797[30], 6}, {NULL}};
const Segment SGI89474[] = {{&S66797[111], 3}, {NULL}};
const Segment SGI89475[] = {{&S66797[77], 5}, {NULL}};
const Segment SGI89476[] = {{&S66798[44], 7}, {NULL}};
const Segment SGI89477[] = {{&S66798[44], 19}, {NULL}};
const Segment SGI89478[] = {{&S66798[30], 6}, {NULL}};
const Segment SGI89479[] = {{&S66798[111], 3}, {NULL}};
const Segment SGI89480[] = {{&S66798[77], 5}, {NULL}};
const Segment SGI89481[] = {{&S66799[44], 7}, {NULL}};
const Segment SGI89482[] = {{&S66799[44], 19}, {NULL}};
const Segment SGI89483[] = {{&S66799[30], 6}, {NULL}};
const Segment SGI89484[] = {{&S66799[111], 3}, {NULL}};
const Segment SGI89485[] = {{&S66799[77], 5}, {NULL}};
const Segment SGI89486[] = {{&S66800[44], 7}, {NULL}};
const Segment SGI89487[] = {{&S66800[44], 19}, {NULL}};
const Segment SGI89488[] = {{&S66800[30], 6}, {NULL}};
const Segment SGI89489[] = {{&S66800[111], 3}, {NULL}};
const Segment SGI89490[] = {{&S66800[77], 5}, {NULL}};
const Segment SGI89491[] = {{&S74632[45], 10}, {NULL}};
const Segment SGI89492[] = {{&S74632[30], 6}, {NULL}};
const Segment SGI89493[] = {{&S74632[120], 3}, {NULL}};
const Segment SGI89494[] = {{&S74632[86], 5}, {NULL}};
const Segment SGI89495[] = {{&S74632[144], 13}, {NULL}};
const Segment SGI89496[] = {{&S54183[34], 12}, {NULL}};
const Segment SGI89497[] = {{&S54183[27], 6}, {NULL}};
const Segment SGI89498[] = {{&S54183[99], 9}, {NULL}};
const Segment SGI89499[] = {{&S54183[72], 6}, {NULL}};
const Segment SGI89500[] = {{&S54263[34], 11}, {NULL}};
const Segment SGI89501[] = {{&S54263[27], 6}, {NULL}};
const Segment SGI89502[] = {{&S54263[98], 9}, {NULL}};
const Segment SGI89503[] = {{&S54263[71], 6}, {NULL}};
const Segment SGI89504[] = {{&S54264[34], 5}, {NULL}};
const Segment SGI89505[] = {{&S54264[27], 6}, {NULL}};
const Segment SGI89506[] = {{&S54264[98], 9}, {NULL}};
const Segment SGI89507[] = {{&S54264[71], 6}, {NULL}};
const Segment SGI89508[] = {{&S74806[45], 10}, {NULL}};
const Segment SGI89509[] = {{&S74806[30], 6}, {NULL}};
const Segment SGI89510[] = {{&S74806[120], 3}, {NULL}};
const Segment SGI89511[] = {{&S74806[86], 5}, {NULL}};
const Segment SGI89512[] = {{&S74806[144], 13}, {NULL}};
const Segment SGI89513[] = {{&S54279[34], 10}, {NULL}};
const Segment SGI89514[] = {{&S54279[27], 6}, {NULL}};
const Segment SGI89515[] = {{&S54279[97], 9}, {NULL}};
const Segment SGI89516[] = {{&S54279[70], 6}, {NULL}};
const Segment SGI89517[] = {{&S74807[44], 10}, {NULL}};
const Segment SGI89518[] = {{&S74807[44], 27}, {NULL}};
const Segment SGI89519[] = {{&S74807[30], 6}, {NULL}};
const Segment SGI89520[] = {{&S74807[120], 3}, {NULL}};
const Segment SGI89521[] = {{&S74807[85], 6}, {NULL}};
const Segment SGI89522[] = {{&S74807[140], 8}, {NULL}};
const Segment SGI89523[] = {{&S74807[149], 13}, {NULL}};
const Segment SGI89524[] = {{&S54280[34], 10}, {NULL}};
const Segment SGI89525[] = {{&S54280[27], 6}, {NULL}};
const Segment SGI89526[] = {{&S54280[97], 9}, {NULL}};
const Segment SGI89527[] = {{&S54280[70], 6}, {NULL}};
const Segment SGI89528[] = {{&S54281[34], 8}, {NULL}};
const Segment SGI89529[] = {{&S54281[27], 6}, {NULL}};
const Segment SGI89530[] = {{&S54281[95], 9}, {NULL}};
const Segment SGI89531[] = {{&S54281[68], 6}, {NULL}};
const Segment SGI89532[] = {{&S54286[34], 7}, {NULL}};
const Segment SGI89533[] = {{&S54286[27], 6}, {NULL}};
const Segment SGI89534[] = {{&S54286[94], 9}, {NULL}};
const Segment SGI89535[] = {{&S54286[67], 6}, {NULL}};
const Segment SGI89536[] = {{&S54288[34], 6}, {NULL}};
const Segment SGI89537[] = {{&S54288[27], 6}, {NULL}};
const Segment SGI89538[] = {{&S54288[93], 9}, {NULL}};
const Segment SGI89539[] = {{&S54288[66], 6}, {NULL}};
const Segment SGI89540[] = {{&S54289[34], 11}, {NULL}};
const Segment SGI89541[] = {{&S54289[27], 6}, {NULL}};
const Segment SGI89542[] = {{&S54289[98], 9}, {NULL}};
const Segment SGI89543[] = {{&S54289[71], 6}, {NULL}};
const Segment SGI89544[] = {{&S54303[34], 7}, {NULL}};
const Segment SGI89545[] = {{&S54303[27], 6}, {NULL}};
const Segment SGI89546[] = {{&S54303[94], 9}, {NULL}};
const Segment SGI89547[] = {{&S54303[67], 6}, {NULL}};
const Segment SGI89548[] = {{&S66738[42], 10}, {NULL}};
const Segment SGI89549[] = {{&S66738[42], 22}, {NULL}};
const Segment SGI89550[] = {{&S66738[30], 4}, {NULL}};
const Segment SGI89551[] = {{&S66738[137], 4}, {NULL}};
const Segment SGI89552[] = {{&S66738[163], 3}, {NULL}};
const Segment SGI89553[] = {{&S66738[113], 3}, {&S66738[212], 5}, {NULL}};
const Segment SGI89554[] = {{&S66738[78], 6}, {NULL}};
const Segment SGI89555[] = {{&S66739[40], 10}, {NULL}};
const Segment SGI89556[] = {{&S66739[30], 4}, {NULL}};
const Segment SGI89557[] = {{&S66739[135], 4}, {NULL}};
const Segment SGI89558[] = {{&S66739[161], 3}, {NULL}};
const Segment SGI89559[] = {{&S66739[111], 3}, {&S66739[210], 5}, {NULL}};
const Segment SGI89560[] = {{&S66739[76], 6}, {NULL}};
const Segment SGI89561[] = {{&S54309[34], 16}, {NULL}};
const Segment SGI89562[] = {{&S54309[27], 6}, {NULL}};
const Segment SGI89563[] = {{&S54309[103], 9}, {NULL}};
const Segment SGI89564[] = {{&S54309[76], 6}, {NULL}};
const Segment SGI89565[] = {{&S54310[34], 18}, {NULL}};
const Segment SGI89566[] = {{&S54310[27], 6}, {NULL}};
const Segment SGI89567[] = {{&S54310[105], 9}, {NULL}};
const Segment SGI89568[] = {{&S54310[78], 6}, {NULL}};
const Segment SGI89569[] = {{&S54318[34], 14}, {NULL}};
const Segment SGI89570[] = {{&S54318[27], 6}, {NULL}};
const Segment SGI89571[] = {{&S54318[98], 9}, {NULL}};
const Segment SGI89572[] = {{&S54318[71], 6}, {NULL}};
const Segment SGI89573[] = {{&S54319[34], 13}, {NULL}};
const Segment SGI89574[] = {{&S54319[27], 6}, {NULL}};
const Segment SGI89575[] = {{&S54319[97], 9}, {NULL}};
const Segment SGI89576[] = {{&S54319[70], 6}, {NULL}};
const Segment SGI89577[] = {{&S54323[34], 11}, {NULL}};
const Segment SGI89578[] = {{&S54323[27], 6}, {NULL}};
const Segment SGI89579[] = {{&S54323[98], 9}, {NULL}};
const Segment SGI89580[] = {{&S54323[71], 6}, {NULL}};
const Segment SGI89581[] = {{&S54336[34], 13}, {NULL}};
const Segment SGI89582[] = {{&S54336[27], 6}, {NULL}};
const Segment SGI89583[] = {{&S54336[100], 9}, {NULL}};
const Segment SGI89584[] = {{&S54336[73], 6}, {NULL}};
const Segment SGI89585[] = {{&S61656[44], 9}, {NULL}};
const Segment SGI89586[] = {{&S61656[44], 21}, {NULL}};
const Segment SGI89587[] = {{&S61656[30], 6}, {NULL}};
const Segment SGI89588[] = {{&S61656[31], 3}, {NULL}};
const Segment SGI89589[] = {{&S61656[79], 6}, {NULL}};
const Segment SGI89590[] = {{&S61656[0], 13}, {NULL}};
const Segment SGI89591[] = {{&S54386[34], 13}, {NULL}};
const Segment SGI89592[] = {{&S54386[27], 6}, {NULL}};
const Segment SGI89593[] = {{&S54386[100], 9}, {NULL}};
const Segment SGI89594[] = {{&S54386[73], 6}, {NULL}};
const Segment SGI89595[] = {{&S54387[34], 12}, {NULL}};
const Segment SGI89596[] = {{&S54387[27], 6}, {NULL}};
const Segment SGI89597[] = {{&S54387[99], 9}, {NULL}};
const Segment SGI89598[] = {{&S54387[72], 6}, {NULL}};
const Segment SGI89599[] = {{&S54445[34], 13}, {NULL}};
const Segment SGI89600[] = {{&S54445[27], 6}, {NULL}};
const Segment SGI89601[] = {{&S54445[100], 9}, {NULL}};
const Segment SGI89602[] = {{&S54445[73], 6}, {NULL}};
const Segment SGI89603[] = {{&S54446[34], 14}, {NULL}};
const Segment SGI89604[] = {{&S54446[27], 6}, {NULL}};
const Segment SGI89605[] = {{&S54446[101], 9}, {NULL}};
const Segment SGI89606[] = {{&S54446[74], 6}, {NULL}};
const Segment SGI89607[] = {{&S74925[44], 4}, {NULL}};
const Segment SGI89608[] = {{&S74925[44], 16}, {NULL}};
const Segment SGI89609[] = {{&S74925[30], 6}, {NULL}};
const Segment SGI89610[] = {{&S74925[108], 3}, {NULL}};
const Segment SGI89611[] = {{&S74925[74], 5}, {NULL}};
const Segment SGI89612[] = {{&S54454[44], 9}, {NULL}};
const Segment SGI89613[] = {{&S54454[44], 21}, {NULL}};
const Segment SGI89614[] = {{&S54454[30], 6}, {NULL}};
const Segment SGI89615[] = {{&S54454[31], 3}, {NULL}};
const Segment SGI89616[] = {{&S54454[79], 6}, {NULL}};
const Segment SGI89617[] = {{&S75019[44], 11}, {NULL}};
const Segment SGI89618[] = {{&S75019[44], 23}, {NULL}};
const Segment SGI89619[] = {{&S75019[30], 6}, {NULL}};
const Segment SGI89620[] = {{&S75019[115], 3}, {NULL}};
const Segment SGI89621[] = {{&S75019[81], 5}, {NULL}};
const Segment SGI89622[] = {{&S75020[44], 7}, {NULL}};
const Segment SGI89623[] = {{&S75020[44], 19}, {NULL}};
const Segment SGI89624[] = {{&S75020[30], 6}, {NULL}};
const Segment SGI89625[] = {{&S75020[111], 3}, {NULL}};
const Segment SGI89626[] = {{&S75020[77], 5}, {NULL}};
const Segment SGI89627[] = {{&S75021[44], 11}, {NULL}};
const Segment SGI89628[] = {{&S75021[44], 23}, {NULL}};
const Segment SGI89629[] = {{&S75021[30], 6}, {NULL}};
const Segment SGI89630[] = {{&S75021[84], 6}, {NULL}};
const Segment SGI89631[] = {{&S54485[34], 12}, {NULL}};
const Segment SGI89632[] = {{&S54485[27], 6}, {NULL}};
const Segment SGI89633[] = {{&S54485[99], 9}, {NULL}};
const Segment SGI89634[] = {{&S54485[72], 6}, {NULL}};
const Segment SGI89635[] = {{&S75052[44], 11}, {NULL}};
const Segment SGI89636[] = {{&S75052[44], 28}, {NULL}};
const Segment SGI89637[] = {{&S75052[30], 6}, {NULL}};
const Segment SGI89638[] = {{&S75052[120], 3}, {NULL}};
const Segment SGI89639[] = {{&S75052[86], 5}, {NULL}};
const Segment SGI89640[] = {{&S67325[44], 8}, {NULL}};
const Segment SGI89641[] = {{&S67325[44], 19}, {NULL}};
const Segment SGI89642[] = {{&S67325[30], 6}, {NULL}};
const Segment SGI89643[] = {{&S67325[130], 3}, {NULL}};
const Segment SGI89644[] = {{&S67325[96], 5}, {NULL}};
const Segment SGI89645[] = {{&S67325[35], 1}, {NULL}};
const Segment SGI89646[] = {{&S67325[65], 11}, {NULL}};
const Segment SGI89647[] = {{&S54518[34], 15}, {NULL}};
const Segment SGI89648[] = {{&S54518[27], 6}, {NULL}};
const Segment SGI89649[] = {{&S54518[101], 9}, {NULL}};
const Segment SGI89650[] = {{&S54518[74], 6}, {NULL}};
const Segment SGI89651[] = {{&S75069[44], 17}, {NULL}};
const Segment SGI89652[] = {{&S75069[44], 29}, {NULL}};
const Segment SGI89653[] = {{&S75069[30], 6}, {NULL}};
const Segment SGI89654[] = {{&S75069[121], 3}, {NULL}};
const Segment SGI89655[] = {{&S75069[87], 5}, {NULL}};
const Segment SGI89656[] = {{&S54525[34], 18}, {NULL}};
const Segment SGI89657[] = {{&S54525[27], 6}, {NULL}};
const Segment SGI89658[] = {{&S54525[105], 9}, {NULL}};
const Segment SGI89659[] = {{&S54525[78], 6}, {NULL}};
const Segment SGI89660[] = {{&S54527[34], 18}, {NULL}};
const Segment SGI89661[] = {{&S54527[27], 6}, {NULL}};
const Segment SGI89662[] = {{&S54527[105], 9}, {NULL}};
const Segment SGI89663[] = {{&S54527[78], 6}, {NULL}};
const Segment SGI89664[] = {{&S54531[44], 5}, {NULL}};
const Segment SGI89665[] = {{&S54531[44], 17}, {NULL}};
const Segment SGI89666[] = {{&S54531[30], 6}, {NULL}};
const Segment SGI89667[] = {{&S54531[31], 3}, {NULL}};
const Segment SGI89668[] = {{&S54531[75], 6}, {NULL}};
const Segment SGI89669[] = {{&S54535[38], 5}, {NULL}};
const Segment SGI89670[] = {{&S54535[31], 6}, {NULL}};
const Segment SGI89671[] = {{&S54536[44], 5}, {NULL}};
const Segment SGI89672[] = {{&S54536[44], 17}, {NULL}};
const Segment SGI89673[] = {{&S54536[30], 6}, {NULL}};
const Segment SGI89674[] = {{&S54536[119], 3}, {NULL}};
const Segment SGI89675[] = {{&S54536[84], 6}, {NULL}};
const Segment SGI89676[] = {{&S54536[139], 8}, {NULL}};
const Segment SGI89677[] = {{&S61874[74], 6}, {NULL}};
const Segment SGI89678[] = {{&S61874[0], 9}, {NULL}};
const Segment SGI89679[] = {{&S75140[31], 8}, {NULL}};
const Segment SGI89680[] = {{&S75140[54], 3}, {NULL}};
const Segment SGI89681[] = {{&S75141[44], 20}, {NULL}};
const Segment SGI89682[] = {{&S75141[44], 32}, {NULL}};
const Segment SGI89683[] = {{&S75141[30], 6}, {NULL}};
const Segment SGI89684[] = {{&S75141[124], 3}, {NULL}};
const Segment SGI89685[] = {{&S75141[90], 5}, {NULL}};
const Segment SGI89686[] = {{&S75142[44], 21}, {NULL}};
const Segment SGI89687[] = {{&S75142[44], 33}, {NULL}};
const Segment SGI89688[] = {{&S75142[30], 6}, {NULL}};
const Segment SGI89689[] = {{&S75142[125], 3}, {NULL}};
const Segment SGI89690[] = {{&S75142[91], 5}, {NULL}};
const Segment SGI89691[] = {{&S75181[44], 10}, {NULL}};
const Segment SGI89692[] = {{&S75181[44], 27}, {NULL}};
const Segment SGI89693[] = {{&S75181[30], 6}, {NULL}};
const Segment SGI89694[] = {{&S75181[88], 6}, {NULL}};
const Segment SGI89695[] = {{&S75183[44], 10}, {NULL}};
const Segment SGI89696[] = {{&S75183[44], 27}, {NULL}};
const Segment SGI89697[] = {{&S75183[30], 6}, {NULL}};
const Segment SGI89698[] = {{&S75183[119], 3}, {NULL}};
const Segment SGI89699[] = {{&S75183[85], 5}, {NULL}};
const Segment SGI89700[] = {{&S54645[44], 6}, {NULL}};
const Segment SGI89701[] = {{&S54645[44], 18}, {NULL}};
const Segment SGI89702[] = {{&S54645[30], 6}, {NULL}};
const Segment SGI89703[] = {{&S54645[107], 2}, {NULL}};
const Segment SGI89704[] = {{&S54645[76], 5}, {NULL}};
const Segment SGI89705[] = {{&S67466[44], 7}, {NULL}};
const Segment SGI89706[] = {{&S67466[44], 19}, {NULL}};
const Segment SGI89707[] = {{&S67466[30], 6}, {NULL}};
const Segment SGI89708[] = {{&S67466[130], 3}, {NULL}};
const Segment SGI89709[] = {{&S67466[96], 5}, {NULL}};
const Segment SGI89710[] = {{&S67466[35], 1}, {NULL}};
const Segment SGI89711[] = {{&S67466[65], 11}, {NULL}};
const Segment SGI89712[] = {{&S75194[45], 10}, {NULL}};
const Segment SGI89713[] = {{&S75194[30], 6}, {NULL}};
const Segment SGI89714[] = {{&S75194[120], 3}, {NULL}};
const Segment SGI89715[] = {{&S75194[86], 5}, {NULL}};
const Segment SGI89716[] = {{&S75194[144], 13}, {NULL}};
const Segment SGI89717[] = {{&S75220[6], 4}, {NULL}};
const Segment SGI89718[] = {{&S75220[87], 5}, {NULL}};
const Segment SGI89719[] = {{&S75245[44], 7}, {NULL}};
const Segment SGI89720[] = {{&S75245[44], 19}, {NULL}};
const Segment SGI89721[] = {{&S75245[30], 6}, {NULL}};
const Segment SGI89722[] = {{&S75245[111], 3}, {NULL}};
const Segment SGI89723[] = {{&S75245[77], 5}, {NULL}};
const Segment SGI89724[] = {{&S75246[44], 7}, {NULL}};
const Segment SGI89725[] = {{&S75246[44], 19}, {NULL}};
const Segment SGI89726[] = {{&S75246[30], 6}, {NULL}};
const Segment SGI89727[] = {{&S75246[111], 3}, {NULL}};
const Segment SGI89728[] = {{&S75246[77], 5}, {NULL}};
const Segment SGI89729[] = {{&S67551[44], 9}, {NULL}};
const Segment SGI89730[] = {{&S67551[44], 20}, {NULL}};
const Segment SGI89731[] = {{&S67551[30], 6}, {NULL}};
const Segment SGI89732[] = {{&S67551[131], 3}, {NULL}};
const Segment SGI89733[] = {{&S67551[97], 5}, {NULL}};
const Segment SGI89734[] = {{&S67551[35], 1}, {NULL}};
const Segment SGI89735[] = {{&S67551[66], 11}, {NULL}};
const Segment SGI89736[] = {{&S67560[44], 13}, {NULL}};
const Segment SGI89737[] = {{&S67560[44], 23}, {NULL}};
const Segment SGI89738[] = {{&S67560[30], 6}, {NULL}};
const Segment SGI89739[] = {{&S67560[115], 3}, {NULL}};
const Segment SGI89740[] = {{&S67560[81], 5}, {NULL}};
const Segment SGI89741[] = {{&S75342[44], 13}, {NULL}};
const Segment SGI89742[] = {{&S75342[44], 25}, {NULL}};
const Segment SGI89743[] = {{&S75342[30], 6}, {NULL}};
const Segment SGI89744[] = {{&S75342[117], 3}, {NULL}};
const Segment SGI89745[] = {{&S75342[83], 5}, {NULL}};
const Segment SGI89746[] = {{&S54812[44], 6}, {NULL}};
const Segment SGI89747[] = {{&S54812[44], 18}, {NULL}};
const Segment SGI89748[] = {{&S54812[30], 6}, {NULL}};
const Segment SGI89749[] = {{&S54812[31], 3}, {NULL}};
const Segment SGI89750[] = {{&S54812[76], 6}, {NULL}};
const Segment SGI89751[] = {{&S75364[44], 10}, {NULL}};
const Segment SGI89752[] = {{&S75364[44], 27}, {NULL}};
const Segment SGI89753[] = {{&S75364[30], 6}, {NULL}};
const Segment SGI89754[] = {{&S75364[85], 6}, {NULL}};
const Segment SGI89755[] = {{&S75364[112], 13}, {NULL}};
const Segment SGI89756[] = {{&S75365[44], 10}, {NULL}};
const Segment SGI89757[] = {{&S75365[44], 27}, {NULL}};
const Segment SGI89758[] = {{&S75365[30], 6}, {NULL}};
const Segment SGI89759[] = {{&S75365[85], 6}, {NULL}};
const Segment SGI89760[] = {{&S75365[112], 13}, {NULL}};
const Segment SGI89761[] = {{&S75366[44], 11}, {NULL}};
const Segment SGI89762[] = {{&S75366[44], 28}, {NULL}};
const Segment SGI89763[] = {{&S75366[30], 6}, {NULL}};
const Segment SGI89764[] = {{&S75366[86], 6}, {NULL}};
const Segment SGI89765[] = {{&S75366[113], 13}, {NULL}};
const Segment SGI89766[] = {{&S75423[45], 9}, {NULL}};
const Segment SGI89767[] = {{&S75423[30], 6}, {NULL}};
const Segment SGI89768[] = {{&S75423[114], 3}, {NULL}};
const Segment SGI89769[] = {{&S75423[80], 5}, {NULL}};
const Segment SGI89770[] = {{&S75423[138], 13}, {NULL}};
const Segment SGI89771[] = {{&S75424[44], 9}, {NULL}};
const Segment SGI89772[] = {{&S75424[44], 21}, {NULL}};
const Segment SGI89773[] = {{&S75424[30], 6}, {NULL}};
const Segment SGI89774[] = {{&S75424[114], 3}, {NULL}};
const Segment SGI89775[] = {{&S75424[79], 6}, {NULL}};
const Segment SGI89776[] = {{&S75424[134], 8}, {NULL}};
const Segment SGI89777[] = {{&S75424[143], 13}, {NULL}};
const Segment SGI89778[] = {{&S67415[42], 7}, {NULL}};
const Segment SGI89779[] = {{&S67415[42], 20}, {NULL}};
const Segment SGI89780[] = {{&S67415[30], 4}, {NULL}};
const Segment SGI89781[] = {{&S67415[111], 3}, {NULL}};
const Segment SGI89782[] = {{&S67415[76], 6}, {NULL}};
const Segment SGI89783[] = {{&S67416[42], 7}, {NULL}};
const Segment SGI89784[] = {{&S67416[42], 23}, {NULL}};
const Segment SGI89785[] = {{&S67416[30], 4}, {NULL}};
const Segment SGI89786[] = {{&S67416[114], 3}, {NULL}};
const Segment SGI89787[] = {{&S67416[79], 6}, {NULL}};
const Segment SGI89788[] = {{&S67417[42], 7}, {NULL}};
const Segment SGI89789[] = {{&S67417[42], 21}, {NULL}};
const Segment SGI89790[] = {{&S67417[30], 4}, {NULL}};
const Segment SGI89791[] = {{&S67417[112], 3}, {NULL}};
const Segment SGI89792[] = {{&S67417[77], 6}, {NULL}};
const Segment SGI89793[] = {{&S67418[42], 7}, {NULL}};
const Segment SGI89794[] = {{&S67418[42], 21}, {NULL}};
const Segment SGI89795[] = {{&S67418[30], 4}, {NULL}};
const Segment SGI89796[] = {{&S67418[112], 3}, {NULL}};
const Segment SGI89797[] = {{&S67418[77], 6}, {NULL}};
const Segment SGI89798[] = {{&S67419[42], 7}, {NULL}};
const Segment SGI89799[] = {{&S67419[42], 23}, {NULL}};
const Segment SGI89800[] = {{&S67419[30], 4}, {NULL}};
const Segment SGI89801[] = {{&S67419[114], 3}, {NULL}};
const Segment SGI89802[] = {{&S67419[79], 6}, {NULL}};
const Segment SGI89803[] = {{&S54913[34], 15}, {NULL}};
const Segment SGI89804[] = {{&S67722[44], 4}, {NULL}};
const Segment SGI89805[] = {{&S67722[44], 15}, {NULL}};
const Segment SGI89806[] = {{&S54913[27], 6}, {NULL}};
const Segment SGI89807[] = {{&S67722[30], 6}, {NULL}};
const Segment SGI89808[] = {{&S67420[42], 7}, {NULL}};
const Segment SGI89809[] = {{&S67420[42], 21}, {NULL}};
const Segment SGI89810[] = {{&S67420[30], 4}, {NULL}};
const Segment SGI89811[] = {{&S67722[126], 3}, {NULL}};
const Segment SGI89812[] = {{&S54913[108], 9}, {NULL}};
const Segment SGI89813[] = {{&S54913[81], 6}, {NULL}};
const Segment SGI89814[] = {{&S67722[92], 5}, {NULL}};
const Segment SGI89815[] = {{&S67420[112], 3}, {NULL}};
const Segment SGI89816[] = {{&S67722[35], 1}, {NULL}};
const Segment SGI89817[] = {{&S67722[61], 11}, {NULL}};
const Segment SGI89818[] = {{&S67420[77], 6}, {NULL}};
const Segment SGI89819[] = {{&S54914[34], 15}, {NULL}};
const Segment SGI89820[] = {{&S54914[27], 6}, {NULL}};
const Segment SGI89821[] = {{&S67421[42], 7}, {NULL}};
const Segment SGI89822[] = {{&S67421[42], 21}, {NULL}};
const Segment SGI89823[] = {{&S67421[30], 4}, {NULL}};
const Segment SGI89824[] = {{&S54914[108], 9}, {NULL}};
const Segment SGI89825[] = {{&S54914[81], 6}, {NULL}};
const Segment SGI89826[] = {{&S67421[112], 3}, {NULL}};
const Segment SGI89827[] = {{&S67421[77], 6}, {NULL}};
const Segment SGI89828[] = {{&S54915[34], 18}, {NULL}};
const Segment SGI89829[] = {{&S54915[27], 6}, {NULL}};
const Segment SGI89830[] = {{&S67422[42], 7}, {NULL}};
const Segment SGI89831[] = {{&S67422[42], 21}, {NULL}};
const Segment SGI89832[] = {{&S54915[114], 9}, {NULL}};
const Segment SGI89833[] = {{&S67422[30], 4}, {NULL}};
const Segment SGI89834[] = {{&S54915[87], 6}, {NULL}};
const Segment SGI89835[] = {{&S67422[112], 3}, {NULL}};
const Segment SGI89836[] = {{&S67422[77], 6}, {NULL}};
const Segment SGI89837[] = {{&S54916[34], 18}, {NULL}};
const Segment SGI89838[] = {{&S54916[27], 6}, {NULL}};
const Segment SGI89839[] = {{&S67423[42], 7}, {NULL}};
const Segment SGI89840[] = {{&S54916[105], 9}, {NULL}};
const Segment SGI89841[] = {{&S67423[42], 21}, {NULL}};
const Segment SGI89842[] = {{&S54916[78], 6}, {NULL}};
const Segment SGI89843[] = {{&S67423[30], 4}, {NULL}};
const Segment SGI89844[] = {{&S67423[112], 3}, {NULL}};
const Segment SGI89845[] = {{&S67423[77], 6}, {NULL}};
const Segment SGI89846[] = {{&S54917[34], 18}, {NULL}};
const Segment SGI89847[] = {{&S54917[27], 6}, {NULL}};
const Segment SGI89848[] = {{&S67424[42], 7}, {NULL}};
const Segment SGI89849[] = {{&S67424[42], 21}, {NULL}};
const Segment SGI89850[] = {{&S54917[105], 9}, {NULL}};
const Segment SGI89851[] = {{&S67424[30], 4}, {NULL}};
const Segment SGI89852[] = {{&S54917[78], 6}, {NULL}};
const Segment SGI89853[] = {{&S67424[112], 3}, {NULL}};
const Segment SGI89854[] = {{&S67424[77], 6}, {NULL}};
const Segment SGI89855[] = {{&S67425[42], 7}, {NULL}};
const Segment SGI89856[] = {{&S67425[42], 21}, {NULL}};
const Segment SGI89857[] = {{&S67425[30], 4}, {NULL}};
const Segment SGI89858[] = {{&S67425[112], 3}, {NULL}};
const Segment SGI89859[] = {{&S67425[77], 6}, {NULL}};
const Segment SGI89860[] = {{&S67426[42], 7}, {NULL}};
const Segment SGI89861[] = {{&S67426[42], 21}, {NULL}};
const Segment SGI89862[] = {{&S67426[30], 4}, {NULL}};
const Segment SGI89863[] = {{&S67426[112], 3}, {NULL}};
const Segment SGI89864[] = {{&S67426[77], 6}, {NULL}};
const Segment SGI89865[] = {{&S54927[44], 9}, {NULL}};
const Segment SGI89866[] = {{&S54927[44], 21}, {NULL}};
const Segment SGI89867[] = {{&S54927[30], 6}, {NULL}};
const Segment SGI89868[] = {{&S54927[31], 3}, {NULL}};
const Segment SGI89869[] = {{&S54927[79], 6}, {NULL}};
const Segment SGI89870[] = {{&S54946[44], 10}, {NULL}};
const Segment SGI89871[] = {{&S54946[44], 27}, {NULL}};
const Segment SGI89872[] = {{&S54946[30], 6}, {NULL}};
const Segment SGI89873[] = {{&S54946[31], 3}, {NULL}};
const Segment SGI89874[] = {{&S54946[85], 6}, {NULL}};
const Segment SGI89875[] = {{&S54947[44], 10}, {NULL}};
const Segment SGI89876[] = {{&S54947[44], 22}, {NULL}};
const Segment SGI89877[] = {{&S54947[30], 6}, {NULL}};
const Segment SGI89878[] = {{&S54947[31], 3}, {NULL}};
const Segment SGI89879[] = {{&S54947[80], 6}, {NULL}};
const Segment SGI89880[] = {{&S54951[38], 10}, {NULL}};
const Segment SGI89881[] = {{&S54951[31], 6}, {NULL}};
const Segment SGI89882[] = {{&S67785[44], 7}, {NULL}};
const Segment SGI89883[] = {{&S67785[44], 19}, {NULL}};
const Segment SGI89884[] = {{&S67785[30], 6}, {NULL}};
const Segment SGI89885[] = {{&S67785[111], 3}, {NULL}};
const Segment SGI89886[] = {{&S67785[77], 5}, {NULL}};
const Segment SGI89887[] = {{&S67786[44], 7}, {NULL}};
const Segment SGI89888[] = {{&S67786[44], 19}, {NULL}};
const Segment SGI89889[] = {{&S67786[30], 6}, {NULL}};
const Segment SGI89890[] = {{&S67786[111], 3}, {NULL}};
const Segment SGI89891[] = {{&S67786[77], 5}, {NULL}};
const Segment SGI89892[] = {{&S67787[44], 7}, {NULL}};
const Segment SGI89893[] = {{&S67787[44], 19}, {NULL}};
const Segment SGI89894[] = {{&S67787[30], 6}, {NULL}};
const Segment SGI89895[] = {{&S67787[111], 3}, {NULL}};
const Segment SGI89896[] = {{&S67787[77], 5}, {NULL}};
const Segment SGI89897[] = {{&S67788[44], 7}, {NULL}};
const Segment SGI89898[] = {{&S67788[44], 19}, {NULL}};
const Segment SGI89899[] = {{&S67788[30], 6}, {NULL}};
const Segment SGI89900[] = {{&S67788[111], 3}, {NULL}};
const Segment SGI89901[] = {{&S67788[77], 5}, {NULL}};
const Segment SGI89902[] = {{&S67789[44], 7}, {NULL}};
const Segment SGI89903[] = {{&S67789[44], 19}, {NULL}};
const Segment SGI89904[] = {{&S67789[30], 6}, {NULL}};
const Segment SGI89905[] = {{&S67789[111], 3}, {NULL}};
const Segment SGI89906[] = {{&S67789[77], 5}, {NULL}};
const Segment SGI89907[] = {{&S54990[34], 7}, {NULL}};
const Segment SGI89908[] = {{&S54990[27], 6}, {NULL}};
const Segment SGI89909[] = {{&S54990[94], 9}, {NULL}};
const Segment SGI89910[] = {{&S54990[67], 6}, {NULL}};
const Segment SGI89911[] = {{&S75520[44], 10}, {NULL}};
const Segment SGI89912[] = {{&S75520[44], 27}, {NULL}};
const Segment SGI89913[] = {{&S75520[30], 6}, {NULL}};
const Segment SGI89914[] = {{&S75520[88], 6}, {NULL}};
const Segment SGI89915[] = {{&S75521[44], 10}, {NULL}};
const Segment SGI89916[] = {{&S75521[44], 27}, {NULL}};
const Segment SGI89917[] = {{&S75521[30], 6}, {NULL}};
const Segment SGI89918[] = {{&S75521[119], 3}, {NULL}};
const Segment SGI89919[] = {{&S75521[85], 5}, {NULL}};
const Segment SGI89920[] = {{&S75525[44], 9}, {NULL}};
const Segment SGI89921[] = {{&S75525[44], 21}, {NULL}};
const Segment SGI89922[] = {{&S75525[30], 6}, {NULL}};
const Segment SGI89923[] = {{&S75525[113], 3}, {NULL}};
const Segment SGI89924[] = {{&S75525[79], 5}, {NULL}};
const Segment SGI89925[] = {{&S75555[44], 10}, {NULL}};
const Segment SGI89926[] = {{&S75555[44], 27}, {NULL}};
const Segment SGI89927[] = {{&S75555[30], 6}, {NULL}};
const Segment SGI89928[] = {{&S75555[149], 4}, {NULL}};
const Segment SGI89929[] = {{&S75555[175], 3}, {NULL}};
const Segment SGI89930[] = {{&S75555[119], 3}, {&S75555[224], 5}, {NULL}};
const Segment SGI89931[] = {{&S75555[85], 5}, {NULL}};
const Segment SGI89932[] = {{&S67862[44], 10}, {NULL}};
const Segment SGI89933[] = {{&S67862[44], 20}, {NULL}};
const Segment SGI89934[] = {{&S67862[30], 6}, {NULL}};
const Segment SGI89935[] = {{&S67862[112], 3}, {NULL}};
const Segment SGI89936[] = {{&S67862[78], 5}, {NULL}};
const Segment SGI89937[] = {{&S55052[34], 6}, {NULL}};
const Segment SGI89938[] = {{&S55052[27], 6}, {NULL}};
const Segment SGI89939[] = {{&S55052[93], 9}, {NULL}};
const Segment SGI89940[] = {{&S55052[66], 6}, {NULL}};
const Segment SGI89941[] = {{&S67865[38], 10}, {NULL}};
const Segment SGI89942[] = {{&S67865[31], 6}, {NULL}};
const Segment SGI89943[] = {{&S67869[44], 10}, {NULL}};
const Segment SGI89944[] = {{&S67869[44], 20}, {NULL}};
const Segment SGI89945[] = {{&S67869[30], 6}, {NULL}};
const Segment SGI89946[] = {{&S67869[81], 4}, {NULL}};
const Segment SGI89947[] = {{&S67869[81], 3}, {NULL}};
const Segment SGI89948[] = {{&S75593[59], 10}, {NULL}};
const Segment SGI89949[] = {{&S75593[59], 27}, {NULL}};
const Segment SGI89950[] = {{&S75593[45], 6}, {NULL}};
const Segment SGI89951[] = {{&S75593[134], 3}, {NULL}};
const Segment SGI89952[] = {{&S75593[100], 5}, {NULL}};
const Segment SGI89953[] = {{&S75593[0], 14}, {NULL}};
const Segment SGI89954[] = {{&S55075[44], 15}, {NULL}};
const Segment SGI89955[] = {{&S55075[44], 33}, {NULL}};
const Segment SGI89956[] = {{&S55075[30], 6}, {NULL}};
const Segment SGI89957[] = {{&S55075[31], 3}, {NULL}};
const Segment SGI89958[] = {{&S55075[91], 6}, {NULL}};
const Segment SGI89959[] = {{&S55076[44], 15}, {NULL}};
const Segment SGI89960[] = {{&S55076[44], 33}, {NULL}};
const Segment SGI89961[] = {{&S55076[30], 6}, {NULL}};
const Segment SGI89962[] = {{&S55076[31], 3}, {NULL}};
const Segment SGI89963[] = {{&S55076[91], 6}, {NULL}};
const Segment SGI89964[] = {{&S55077[44], 18}, {NULL}};
const Segment SGI89965[] = {{&S55077[44], 39}, {NULL}};
const Segment SGI89966[] = {{&S55077[30], 6}, {NULL}};
const Segment SGI89967[] = {{&S55077[31], 3}, {NULL}};
const Segment SGI89968[] = {{&S55077[97], 6}, {NULL}};
const Segment SGI89969[] = {{&S55078[38], 15}, {NULL}};
const Segment SGI89970[] = {{&S55078[31], 6}, {NULL}};
const Segment SGI89971[] = {{&S55079[38], 18}, {NULL}};
const Segment SGI89972[] = {{&S55079[31], 6}, {NULL}};
const Segment SGI89973[] = {{&S55125[34], 9}, {NULL}};
const Segment SGI89974[] = {{&S55125[27], 6}, {NULL}};
const Segment SGI89975[] = {{&S55125[96], 9}, {NULL}};
const Segment SGI89976[] = {{&S55125[69], 6}, {NULL}};
const Segment SGI89977[] = {{&S55126[34], 10}, {NULL}};
const Segment SGI89978[] = {{&S55126[27], 6}, {NULL}};
const Segment SGI89979[] = {{&S55126[97], 9}, {NULL}};
const Segment SGI89980[] = {{&S55126[70], 6}, {NULL}};
const Segment SGI89981[] = {{&S55127[34], 9}, {NULL}};
const Segment SGI89982[] = {{&S55127[27], 6}, {NULL}};
const Segment SGI89983[] = {{&S55127[96], 9}, {NULL}};
const Segment SGI89984[] = {{&S55127[69], 6}, {NULL}};
const Segment SGI89985[] = {{&S55132[44], 16}, {NULL}};
const Segment SGI89986[] = {{&S55132[44], 28}, {NULL}};
const Segment SGI89987[] = {{&S55132[30], 6}, {NULL}};
const Segment SGI89988[] = {{&S55132[31], 3}, {NULL}};
const Segment SGI89989[] = {{&S55132[86], 6}, {NULL}};
const Segment SGI89990[] = {{&S55133[38], 16}, {NULL}};
const Segment SGI89991[] = {{&S55133[31], 6}, {NULL}};
const Segment SGI89992[] = {{&S55134[44], 11}, {NULL}};
const Segment SGI89993[] = {{&S55134[44], 23}, {NULL}};
const Segment SGI89994[] = {{&S55134[30], 6}, {NULL}};
const Segment SGI89995[] = {{&S55134[31], 3}, {NULL}};
const Segment SGI89996[] = {{&S55134[81], 6}, {NULL}};
const Segment SGI89997[] = {{&S55135[44], 12}, {NULL}};
const Segment SGI89998[] = {{&S55135[44], 24}, {NULL}};
const Segment SGI89999[] = {{&S55135[30], 6}, {NULL}};
const Segment SGI90000[] = {{&S55135[31], 3}, {NULL}};
const Segment SGI90001[] = {{&S55135[82], 6}, {NULL}};
const Segment SGI90002[] = {{&S55140[44], 16}, {NULL}};
const Segment SGI90003[] = {{&S55140[44], 28}, {NULL}};
const Segment SGI90004[] = {{&S55140[30], 6}, {NULL}};
const Segment SGI90005[] = {{&S55140[31], 3}, {NULL}};
const Segment SGI90006[] = {{&S55140[86], 6}, {NULL}};
const Segment SGI90007[] = {{&S75693[44], 10}, {NULL}};
const Segment SGI90008[] = {{&S75693[44], 27}, {NULL}};
const Segment SGI90009[] = {{&S75693[30], 6}, {NULL}};
const Segment SGI90010[] = {{&S75693[119], 3}, {NULL}};
const Segment SGI90011[] = {{&S75693[85], 5}, {NULL}};
const Segment SGI90012[] = {{&S75694[44], 10}, {NULL}};
const Segment SGI90013[] = {{&S75694[44], 27}, {NULL}};
const Segment SGI90014[] = {{&S75694[30], 6}, {NULL}};
const Segment SGI90015[] = {{&S75694[88], 6}, {NULL}};
const Segment SGI90016[] = {{&S55189[34], 10}, {NULL}};
const Segment SGI90017[] = {{&S55189[27], 6}, {NULL}};
const Segment SGI90018[] = {{&S55189[97], 9}, {NULL}};
const Segment SGI90019[] = {{&S55189[70], 6}, {NULL}};
const Segment SGI90020[] = {{&S55190[34], 25}, {NULL}};
const Segment SGI90021[] = {{&S55190[27], 6}, {NULL}};
const Segment SGI90022[] = {{&S55190[112], 9}, {NULL}};
const Segment SGI90023[] = {{&S55190[85], 6}, {NULL}};
const Segment SGI90024[] = {{&S55191[34], 20}, {NULL}};
const Segment SGI90025[] = {{&S55191[27], 6}, {NULL}};
const Segment SGI90026[] = {{&S55191[107], 9}, {NULL}};
const Segment SGI90027[] = {{&S55191[80], 6}, {NULL}};
const Segment SGI90028[] = {{&S55192[34], 26}, {NULL}};
const Segment SGI90029[] = {{&S55192[27], 6}, {NULL}};
const Segment SGI90030[] = {{&S55192[113], 9}, {NULL}};
const Segment SGI90031[] = {{&S55192[86], 6}, {NULL}};
const Segment SGI90032[] = {{&S55195[34], 24}, {NULL}};
const Segment SGI90033[] = {{&S55195[27], 6}, {NULL}};
const Segment SGI90034[] = {{&S55195[111], 9}, {NULL}};
const Segment SGI90035[] = {{&S55195[84], 6}, {NULL}};
const Segment SGI90036[] = {{&S55196[34], 21}, {NULL}};
const Segment SGI90037[] = {{&S55196[27], 6}, {NULL}};
const Segment SGI90038[] = {{&S55196[108], 9}, {NULL}};
const Segment SGI90039[] = {{&S55196[81], 6}, {NULL}};
const Segment SGI90040[] = {{&S55203[34], 30}, {NULL}};
const Segment SGI90041[] = {{&S55203[27], 6}, {NULL}};
const Segment SGI90042[] = {{&S55203[117], 9}, {NULL}};
const Segment SGI90043[] = {{&S55203[90], 6}, {NULL}};
const Segment SGI90044[] = {{&S55205[34], 17}, {NULL}};
const Segment SGI90045[] = {{&S55205[27], 6}, {NULL}};
const Segment SGI90046[] = {{&S55205[104], 9}, {NULL}};
const Segment SGI90047[] = {{&S55205[77], 6}, {NULL}};
const Segment SGI90048[] = {{&S75742[45], 11}, {NULL}};
const Segment SGI90049[] = {{&S75742[30], 6}, {NULL}};
const Segment SGI90050[] = {{&S75742[121], 3}, {NULL}};
const Segment SGI90051[] = {{&S75742[87], 5}, {NULL}};
const Segment SGI90052[] = {{&S75742[145], 13}, {NULL}};
const Segment SGI90053[] = {{&S75743[45], 10}, {NULL}};
const Segment SGI90054[] = {{&S75743[30], 6}, {NULL}};
const Segment SGI90055[] = {{&S75743[120], 3}, {NULL}};
const Segment SGI90056[] = {{&S75743[86], 5}, {NULL}};
const Segment SGI90057[] = {{&S75743[144], 13}, {NULL}};
const Segment SGI90058[] = {{&S75754[44], 9}, {NULL}};
const Segment SGI90059[] = {{&S75754[44], 21}, {NULL}};
const Segment SGI90060[] = {{&S75754[30], 6}, {NULL}};
const Segment SGI90061[] = {{&S75754[79], 6}, {NULL}};
const Segment SGI90062[] = {{&S75754[106], 13}, {NULL}};
const Segment SGI90063[] = {{&S75785[44], 11}, {NULL}};
const Segment SGI90064[] = {{&S75785[44], 23}, {NULL}};
const Segment SGI90065[] = {{&S75785[30], 6}, {NULL}};
const Segment SGI90066[] = {{&S75785[115], 3}, {NULL}};
const Segment SGI90067[] = {{&S75785[81], 5}, {NULL}};
const Segment SGI90068[] = {{&S75786[44], 6}, {NULL}};
const Segment SGI90069[] = {{&S75786[44], 18}, {NULL}};
const Segment SGI90070[] = {{&S75786[30], 6}, {NULL}};
const Segment SGI90071[] = {{&S75786[110], 3}, {NULL}};
const Segment SGI90072[] = {{&S75786[76], 5}, {NULL}};
const Segment SGI90073[] = {{&S55269[34], 41}, {NULL}};
const Segment SGI90074[] = {{&S55269[27], 6}, {NULL}};
const Segment SGI90075[] = {{&S55269[128], 9}, {NULL}};
const Segment SGI90076[] = {{&S55269[101], 6}, {NULL}};
const Segment SGI90077[] = {{&S55270[34], 35}, {NULL}};
const Segment SGI90078[] = {{&S55270[27], 6}, {NULL}};
const Segment SGI90079[] = {{&S55270[122], 9}, {NULL}};
const Segment SGI90080[] = {{&S55270[95], 6}, {NULL}};
const Segment SGI90081[] = {{&S55271[34], 41}, {NULL}};
const Segment SGI90082[] = {{&S55271[27], 6}, {NULL}};
const Segment SGI90083[] = {{&S55271[128], 9}, {NULL}};
const Segment SGI90084[] = {{&S55271[101], 6}, {NULL}};
const Segment SGI90085[] = {{&S55273[34], 11}, {NULL}};
const Segment SGI90086[] = {{&S55273[27], 6}, {NULL}};
const Segment SGI90087[] = {{&S55273[98], 9}, {NULL}};
const Segment SGI90088[] = {{&S55273[71], 6}, {NULL}};
const Segment SGI90089[] = {{&S75815[44], 10}, {NULL}};
const Segment SGI90090[] = {{&S75815[44], 27}, {NULL}};
const Segment SGI90091[] = {{&S75815[30], 6}, {NULL}};
const Segment SGI90092[] = {{&S75815[85], 6}, {NULL}};
const Segment SGI90093[] = {{&S75815[112], 13}, {NULL}};
const Segment SGI90094[] = {{&S68129[44], 14}, {NULL}};
const Segment SGI90095[] = {{&S68129[44], 24}, {NULL}};
const Segment SGI90096[] = {{&S68129[30], 6}, {NULL}};
const Segment SGI90097[] = {{&S68129[116], 3}, {NULL}};
const Segment SGI90098[] = {{&S68129[82], 5}, {NULL}};
const Segment SGI90099[] = {{&S68130[38], 14}, {NULL}};
const Segment SGI90100[] = {{&S68130[31], 6}, {NULL}};
const Segment SGI90101[] = {{&S68131[44], 14}, {NULL}};
const Segment SGI90102[] = {{&S68131[44], 24}, {NULL}};
const Segment SGI90103[] = {{&S68131[30], 6}, {NULL}};
const Segment SGI90104[] = {{&S68131[85], 4}, {NULL}};
const Segment SGI90105[] = {{&S68131[85], 3}, {NULL}};
const Segment SGI90106[] = {{&S68132[44], 14}, {NULL}};
const Segment SGI90107[] = {{&S68132[44], 24}, {NULL}};
const Segment SGI90108[] = {{&S68132[30], 6}, {NULL}};
const Segment SGI90109[] = {{&S68132[85], 6}, {NULL}};
const Segment SGI90110[] = {{&S75830[44], 11}, {NULL}};
const Segment SGI90111[] = {{&S75830[44], 23}, {NULL}};
const Segment SGI90112[] = {{&S75830[30], 6}, {NULL}};
const Segment SGI90113[] = {{&S75830[115], 3}, {NULL}};
const Segment SGI90114[] = {{&S75830[81], 5}, {NULL}};
const Segment SGI90115[] = {{&S75837[44], 11}, {NULL}};
const Segment SGI90116[] = {{&S75837[44], 28}, {NULL}};
const Segment SGI90117[] = {{&S75837[30], 6}, {NULL}};
const Segment SGI90118[] = {{&S75837[150], 4}, {NULL}};
const Segment SGI90119[] = {{&S75837[176], 3}, {NULL}};
const Segment SGI90120[] = {{&S75837[120], 3}, {&S75837[225], 5}, {NULL}};
const Segment SGI90121[] = {{&S75837[86], 5}, {NULL}};
const Segment SGI90122[] = {{&S55336[34], 6}, {NULL}};
const Segment SGI90123[] = {{&S55336[27], 6}, {NULL}};
const Segment SGI90124[] = {{&S55336[93], 9}, {NULL}};
const Segment SGI90125[] = {{&S55336[66], 6}, {NULL}};
const Segment SGI90126[] = {{&S55337[44], 13}, {NULL}};
const Segment SGI90127[] = {{&S55337[44], 25}, {NULL}};
const Segment SGI90128[] = {{&S55337[30], 6}, {NULL}};
const Segment SGI90129[] = {{&S55337[31], 3}, {NULL}};
const Segment SGI90130[] = {{&S55337[83], 6}, {NULL}};
const Segment SGI90131[] = {{&S55338[44], 13}, {NULL}};
const Segment SGI90132[] = {{&S55338[44], 25}, {NULL}};
const Segment SGI90133[] = {{&S55338[30], 6}, {NULL}};
const Segment SGI90134[] = {{&S55338[31], 3}, {NULL}};
const Segment SGI90135[] = {{&S55338[83], 6}, {NULL}};
const Segment SGI90136[] = {{&S55339[44], 11}, {NULL}};
const Segment SGI90137[] = {{&S55339[44], 23}, {NULL}};
const Segment SGI90138[] = {{&S55339[30], 6}, {NULL}};
const Segment SGI90139[] = {{&S55339[31], 3}, {NULL}};
const Segment SGI90140[] = {{&S55339[81], 6}, {NULL}};
const Segment SGI90141[] = {{&S55340[44], 8}, {NULL}};
const Segment SGI90142[] = {{&S55340[44], 20}, {NULL}};
const Segment SGI90143[] = {{&S55340[30], 6}, {NULL}};
const Segment SGI90144[] = {{&S55340[31], 3}, {NULL}};
const Segment SGI90145[] = {{&S55340[78], 6}, {NULL}};
const Segment SGI90146[] = {{&S55345[38], 8}, {NULL}};
const Segment SGI90147[] = {{&S55345[31], 6}, {NULL}};
const Segment SGI90148[] = {{&S75896[44], 10}, {NULL}};
const Segment SGI90149[] = {{&S75896[44], 27}, {NULL}};
const Segment SGI90150[] = {{&S75896[30], 6}, {NULL}};
const Segment SGI90151[] = {{&S75896[85], 6}, {NULL}};
const Segment SGI90152[] = {{&S75896[112], 13}, {NULL}};
const Segment SGI90153[] = {{&S55382[34], 7}, {NULL}};
const Segment SGI90154[] = {{&S55382[27], 6}, {NULL}};
const Segment SGI90155[] = {{&S55382[99], 9}, {NULL}};
const Segment SGI90156[] = {{&S55382[72], 6}, {NULL}};
const Segment SGI90157[] = {{&S55405[44], 9}, {NULL}};
const Segment SGI90158[] = {{&S55405[44], 21}, {NULL}};
const Segment SGI90159[] = {{&S55405[30], 6}, {NULL}};
const Segment SGI90160[] = {{&S55405[31], 3}, {NULL}};
const Segment SGI90161[] = {{&S55405[79], 6}, {NULL}};
const Segment SGI90162[] = {{&S55406[44], 9}, {NULL}};
const Segment SGI90163[] = {{&S55406[44], 21}, {NULL}};
const Segment SGI90164[] = {{&S55406[30], 6}, {NULL}};
const Segment SGI90165[] = {{&S55406[31], 3}, {NULL}};
const Segment SGI90166[] = {{&S55406[79], 6}, {NULL}};
const Segment SGI90167[] = {{&S55451[44], 6}, {NULL}};
const Segment SGI90168[] = {{&S55451[44], 18}, {NULL}};
const Segment SGI90169[] = {{&S55451[30], 6}, {NULL}};
const Segment SGI90170[] = {{&S55451[31], 3}, {NULL}};
const Segment SGI90171[] = {{&S55451[76], 6}, {NULL}};
const Segment SGI90172[] = {{&S55454[38], 6}, {NULL}};
const Segment SGI90173[] = {{&S55454[31], 6}, {NULL}};
const Segment SGI90174[] = {{&S55462[34], 12}, {NULL}};
const Segment SGI90175[] = {{&S55462[27], 6}, {NULL}};
const Segment SGI90176[] = {{&S55462[99], 9}, {NULL}};
const Segment SGI90177[] = {{&S55462[72], 6}, {NULL}};
const Segment SGI90178[] = {{&S76020[34], 10}, {NULL}};
const Segment SGI90179[] = {{&S76020[27], 6}, {NULL}};
const Segment SGI90180[] = {{&S76020[109], 2}, {NULL}};
const Segment SGI90181[] = {{&S76020[75], 6}, {NULL}};
const Segment SGI90182[] = {{&S76020[143], 8}, {NULL}};
const Segment SGI90183[] = {{&S76036[44], 14}, {NULL}};
const Segment SGI90184[] = {{&S76036[44], 26}, {NULL}};
const Segment SGI90185[] = {{&S76036[30], 6}, {NULL}};
const Segment SGI90186[] = {{&S76036[118], 3}, {NULL}};
const Segment SGI90187[] = {{&S76036[84], 5}, {NULL}};
const Segment SGI90188[] = {{&S55600[34], 7}, {NULL}};
const Segment SGI90189[] = {{&S55600[27], 6}, {NULL}};
const Segment SGI90190[] = {{&S55600[99], 9}, {NULL}};
const Segment SGI90191[] = {{&S55600[72], 6}, {NULL}};
const Segment SGI90192[] = {{&S68345[44], 14}, {NULL}};
const Segment SGI90193[] = {{&S68345[44], 25}, {NULL}};
const Segment SGI90194[] = {{&S68345[30], 6}, {NULL}};
const Segment SGI90195[] = {{&S68345[136], 3}, {NULL}};
const Segment SGI90196[] = {{&S68345[102], 5}, {NULL}};
const Segment SGI90197[] = {{&S68345[35], 1}, {NULL}};
const Segment SGI90198[] = {{&S68345[71], 11}, {NULL}};
const Segment SGI90199[] = {{&S68112[44], 5}, {NULL}};
const Segment SGI90200[] = {{&S68112[44], 16}, {NULL}};
const Segment SGI90201[] = {{&S68112[30], 6}, {NULL}};
const Segment SGI90202[] = {{&S68112[109], 3}, {NULL}};
const Segment SGI90203[] = {{&S68112[74], 6}, {NULL}};
const Segment SGI90204[] = {{&S76163[44], 31}, {NULL}};
const Segment SGI90205[] = {{&S76163[30], 6}, {NULL}};
const Segment SGI90206[] = {{&S76163[142], 3}, {NULL}};
const Segment SGI90207[] = {{&S76163[108], 5}, {NULL}};
const Segment SGI90208[] = {{&S76164[44], 7}, {NULL}};
const Segment SGI90209[] = {{&S76164[44], 33}, {NULL}};
const Segment SGI90210[] = {{&S76164[30], 6}, {NULL}};
const Segment SGI90211[] = {{&S76164[125], 3}, {NULL}};
const Segment SGI90212[] = {{&S76164[91], 5}, {NULL}};
const Segment SGI90213[] = {{&S76165[44], 32}, {&S76165[77], 9}, {NULL}};
const Segment SGI90214[] = {{&S76165[30], 6}, {NULL}};
const Segment SGI90215[] = {{&S76165[143], 3}, {NULL}};
const Segment SGI90216[] = {{&S76165[109], 5}, {NULL}};
const Segment SGI90217[] = {{&S76166[44], 31}, {NULL}};
const Segment SGI90218[] = {{&S76166[30], 6}, {NULL}};
const Segment SGI90219[] = {{&S76166[142], 3}, {NULL}};
const Segment SGI90220[] = {{&S76166[108], 5}, {NULL}};
const Segment SGI90221[] = {{&S76167[44], 32}, {&S76167[77], 9}, {NULL}};
const Segment SGI90222[] = {{&S76167[30], 6}, {NULL}};
const Segment SGI90223[] = {{&S76167[143], 3}, {NULL}};
const Segment SGI90224[] = {{&S76167[109], 5}, {NULL}};
const Segment SGI90225[] = {{&S76168[44], 7}, {NULL}};
const Segment SGI90226[] = {{&S76168[44], 33}, {NULL}};
const Segment SGI90227[] = {{&S76168[30], 6}, {NULL}};
const Segment SGI90228[] = {{&S76168[94], 4}, {NULL}};
const Segment SGI90229[] = {{&S76168[94], 3}, {NULL}};
const Segment SGI90230[] = {{&S76169[44], 7}, {NULL}};
const Segment SGI90231[] = {{&S76169[44], 34}, {NULL}};
const Segment SGI90232[] = {{&S76169[30], 6}, {NULL}};
const Segment SGI90233[] = {{&S76169[126], 3}, {NULL}};
const Segment SGI90234[] = {{&S76169[92], 5}, {NULL}};
const Segment SGI90235[] = {{&S76170[44], 7}, {NULL}};
const Segment SGI90236[] = {{&S76170[44], 34}, {NULL}};
const Segment SGI90237[] = {{&S76170[30], 6}, {NULL}};
const Segment SGI90238[] = {{&S76170[95], 4}, {NULL}};
const Segment SGI90239[] = {{&S76170[95], 3}, {NULL}};
const Segment SGI90240[] = {{&S68385[44], 7}, {NULL}};
const Segment SGI90241[] = {{&S68385[44], 19}, {NULL}};
const Segment SGI90242[] = {{&S68385[30], 6}, {NULL}};
const Segment SGI90243[] = {{&S68385[130], 3}, {NULL}};
const Segment SGI90244[] = {{&S68385[96], 5}, {NULL}};
const Segment SGI90245[] = {{&S68385[35], 1}, {NULL}};
const Segment SGI90246[] = {{&S68385[65], 11}, {NULL}};
const Segment SGI90247[] = {{&S55704[44], 10}, {NULL}};
const Segment SGI90248[] = {{&S55704[44], 22}, {NULL}};
const Segment SGI90249[] = {{&S55704[30], 6}, {NULL}};
const Segment SGI90250[] = {{&S55704[139], 4}, {NULL}};
const Segment SGI90251[] = {{&S55704[165], 3}, {NULL}};
const Segment SGI90252[] = {{&S55704[31], 3}, {&S55704[214], 5}, {NULL}};
const Segment SGI90253[] = {{&S55704[80], 6}, {NULL}};
const Segment SGI90254[] = {{&S55706[42], 10}, {NULL}};
const Segment SGI90255[] = {{&S55706[30], 6}, {NULL}};
const Segment SGI90256[] = {{&S55706[137], 4}, {NULL}};
const Segment SGI90257[] = {{&S55706[163], 3}, {NULL}};
const Segment SGI90258[] = {{&S55706[31], 3}, {&S55706[212], 5}, {NULL}};
const Segment SGI90259[] = {{&S55706[78], 6}, {NULL}};
const Segment SGI90260[] = {{&S68438[44], 7}, {NULL}};
const Segment SGI90261[] = {{&S68438[44], 19}, {NULL}};
const Segment SGI90262[] = {{&S68438[30], 6}, {NULL}};
const Segment SGI90263[] = {{&S68438[111], 3}, {NULL}};
const Segment SGI90264[] = {{&S68438[77], 5}, {NULL}};
const Segment SGI90265[] = {{&S68439[44], 7}, {NULL}};
const Segment SGI90266[] = {{&S68439[44], 18}, {NULL}};
const Segment SGI90267[] = {{&S68439[30], 6}, {NULL}};
const Segment SGI90268[] = {{&S68439[126], 3}, {NULL}};
const Segment SGI90269[] = {{&S68439[92], 5}, {NULL}};
const Segment SGI90270[] = {{&S68439[35], 1}, {NULL}};
const Segment SGI90271[] = {{&S68439[64], 11}, {NULL}};
const Segment SGI90272[] = {{&S68440[44], 7}, {NULL}};
const Segment SGI90273[] = {{&S68440[44], 42}, {NULL}};
const Segment SGI90274[] = {{&S68440[30], 6}, {NULL}};
const Segment SGI90275[] = {{&S68440[153], 3}, {NULL}};
const Segment SGI90276[] = {{&S68440[119], 5}, {NULL}};
const Segment SGI90277[] = {{&S68440[35], 1}, {NULL}};
const Segment SGI90278[] = {{&S68440[88], 11}, {NULL}};
const Segment SGI90279[] = {{&S68113[42], 5}, {NULL}};
const Segment SGI90280[] = {{&S68113[42], 17}, {NULL}};
const Segment SGI90281[] = {{&S68113[30], 4}, {NULL}};
const Segment SGI90282[] = {{&S68113[108], 3}, {NULL}};
const Segment SGI90283[] = {{&S68113[73], 6}, {NULL}};
const Segment SGI90284[] = {{&S68114[38], 5}, {NULL}};
const Segment SGI90285[] = {{&S68114[31], 6}, {NULL}};
const Segment SGI90286[] = {{&S76277[44], 11}, {NULL}};
const Segment SGI90287[] = {{&S76277[44], 34}, {NULL}};
const Segment SGI90288[] = {{&S76277[30], 6}, {NULL}};
const Segment SGI90289[] = {{&S76277[126], 3}, {NULL}};
const Segment SGI90290[] = {{&S76277[92], 5}, {NULL}};
const Segment SGI90291[] = {{&S76278[44], 12}, {NULL}};
const Segment SGI90292[] = {{&S76278[44], 35}, {NULL}};
const Segment SGI90293[] = {{&S76278[30], 6}, {NULL}};
const Segment SGI90294[] = {{&S76278[127], 3}, {NULL}};
const Segment SGI90295[] = {{&S76278[93], 5}, {NULL}};
const Segment SGI90296[] = {{&S76279[44], 11}, {NULL}};
const Segment SGI90297[] = {{&S76279[44], 23}, {NULL}};
const Segment SGI90298[] = {{&S76279[30], 6}, {NULL}};
const Segment SGI90299[] = {{&S76279[115], 3}, {NULL}};
const Segment SGI90300[] = {{&S76279[81], 5}, {NULL}};
const Segment SGI90301[] = {{&S76280[44], 11}, {NULL}};
const Segment SGI90302[] = {{&S76280[44], 23}, {NULL}};
const Segment SGI90303[] = {{&S76280[30], 6}, {NULL}};
const Segment SGI90304[] = {{&S76280[84], 6}, {NULL}};
const Segment SGI90305[] = {{&S76303[44], 10}, {NULL}};
const Segment SGI90306[] = {{&S76303[44], 27}, {NULL}};
const Segment SGI90307[] = {{&S76303[30], 6}, {NULL}};
const Segment SGI90308[] = {{&S76303[85], 6}, {NULL}};
const Segment SGI90309[] = {{&S76303[112], 13}, {NULL}};
const Segment SGI90310[] = {{&S55829[44], 25}, {NULL}};
const Segment SGI90311[] = {{&S76329[59], 11}, {NULL}};
const Segment SGI90312[] = {{&S55829[44], 54}, {NULL}};
const Segment SGI90313[] = {{&S55829[30], 6}, {NULL}};
const Segment SGI90314[] = {{&S76329[59], 28}, {NULL}};
const Segment SGI90315[] = {{&S76329[45], 6}, {NULL}};
const Segment SGI90316[] = {{&S55829[31], 3}, {NULL}};
const Segment SGI90317[] = {{&S76329[135], 3}, {NULL}};
const Segment SGI90318[] = {{&S55829[112], 6}, {NULL}};
const Segment SGI90319[] = {{&S76329[101], 5}, {NULL}};
const Segment SGI90320[] = {{&S76329[0], 14}, {NULL}};
const Segment SGI90321[] = {{&S55830[44], 25}, {NULL}};
const Segment SGI90322[] = {{&S55830[44], 54}, {NULL}};
const Segment SGI90323[] = {{&S55830[30], 6}, {NULL}};
const Segment SGI90324[] = {{&S55830[31], 3}, {NULL}};
const Segment SGI90325[] = {{&S55830[112], 6}, {NULL}};
const Segment SGI90326[] = {{&S55831[44], 25}, {NULL}};
const Segment SGI90327[] = {{&S55831[44], 54}, {NULL}};
const Segment SGI90328[] = {{&S55831[30], 6}, {NULL}};
const Segment SGI90329[] = {{&S55831[31], 3}, {NULL}};
const Segment SGI90330[] = {{&S55831[112], 6}, {NULL}};
const Segment SGI90331[] = {{&S55832[44], 14}, {NULL}};
const Segment SGI90332[] = {{&S55832[44], 26}, {NULL}};
const Segment SGI90333[] = {{&S55832[30], 6}, {NULL}};
const Segment SGI90334[] = {{&S55832[31], 3}, {NULL}};
const Segment SGI90335[] = {{&S55832[84], 6}, {NULL}};
const Segment SGI90336[] = {{&S55833[44], 25}, {NULL}};
const Segment SGI90337[] = {{&S55833[44], 54}, {NULL}};
const Segment SGI90338[] = {{&S76337[59], 10}, {NULL}};
const Segment SGI90339[] = {{&S55833[30], 6}, {NULL}};
const Segment SGI90340[] = {{&S76337[59], 27}, {NULL}};
const Segment SGI90341[] = {{&S76337[45], 6}, {NULL}};
const Segment SGI90342[] = {{&S55833[31], 3}, {NULL}};
const Segment SGI90343[] = {{&S76337[134], 3}, {NULL}};
const Segment SGI90344[] = {{&S55833[112], 6}, {NULL}};
const Segment SGI90345[] = {{&S76337[100], 5}, {NULL}};
const Segment SGI90346[] = {{&S76337[0], 14}, {NULL}};
const Segment SGI90347[] = {{&S76339[44], 11}, {NULL}};
const Segment SGI90348[] = {{&S76339[44], 23}, {NULL}};
const Segment SGI90349[] = {{&S76339[30], 6}, {NULL}};
const Segment SGI90350[] = {{&S76339[115], 3}, {NULL}};
const Segment SGI90351[] = {{&S76339[81], 5}, {NULL}};
const Segment SGI90352[] = {{&S76340[44], 11}, {NULL}};
const Segment SGI90353[] = {{&S76340[44], 23}, {NULL}};
const Segment SGI90354[] = {{&S76340[30], 6}, {NULL}};
const Segment SGI90355[] = {{&S76340[84], 6}, {NULL}};
const Segment SGI90356[] = {{&S76357[44], 7}, {NULL}};
const Segment SGI90357[] = {{&S76357[44], 19}, {NULL}};
const Segment SGI90358[] = {{&S76357[30], 6}, {NULL}};
const Segment SGI90359[] = {{&S55860[34], 7}, {NULL}};
const Segment SGI90360[] = {{&S55860[27], 6}, {NULL}};
const Segment SGI90361[] = {{&S76357[111], 3}, {NULL}};
const Segment SGI90362[] = {{&S76357[77], 5}, {NULL}};
const Segment SGI90363[] = {{&S55860[94], 9}, {NULL}};
const Segment SGI90364[] = {{&S55860[67], 6}, {NULL}};
const Segment SGI90365[] = {{&S55861[34], 6}, {NULL}};
const Segment SGI90366[] = {{&S55861[27], 6}, {NULL}};
const Segment SGI90367[] = {{&S55861[93], 9}, {NULL}};
const Segment SGI90368[] = {{&S55861[66], 6}, {NULL}};
const Segment SGI90369[] = {{&S68595[44], 7}, {NULL}};
const Segment SGI90370[] = {{&S68595[44], 18}, {NULL}};
const Segment SGI90371[] = {{&S68595[30], 6}, {NULL}};
const Segment SGI90372[] = {{&S68595[161], 3}, {NULL}};
const Segment SGI90373[] = {{&S68595[127], 5}, {NULL}};
const Segment SGI90374[] = {{&S68595[35], 1}, {NULL}};
const Segment SGI90375[] = {{&S68595[95], 11}, {NULL}};
const Segment SGI90376[] = {{&S55880[34], 11}, {NULL}};
const Segment SGI90377[] = {{&S55880[27], 6}, {NULL}};
const Segment SGI90378[] = {{&S68596[44], 7}, {NULL}};
const Segment SGI90379[] = {{&S55880[98], 9}, {NULL}};
const Segment SGI90380[] = {{&S68596[44], 18}, {NULL}};
const Segment SGI90381[] = {{&S55880[71], 6}, {NULL}};
const Segment SGI90382[] = {{&S68596[30], 6}, {NULL}};
const Segment SGI90383[] = {{&S68596[129], 3}, {NULL}};
const Segment SGI90384[] = {{&S68596[95], 5}, {NULL}};
const Segment SGI90385[] = {{&S68596[35], 1}, {NULL}};
const Segment SGI90386[] = {{&S68596[64], 11}, {NULL}};
const Segment SGI90387[] = {{&S68597[44], 16}, {NULL}};
const Segment SGI90388[] = {{&S68597[44], 27}, {NULL}};
const Segment SGI90389[] = {{&S68597[30], 6}, {NULL}};
const Segment SGI90390[] = {{&S68597[138], 3}, {NULL}};
const Segment SGI90391[] = {{&S68597[104], 5}, {NULL}};
const Segment SGI90392[] = {{&S68597[35], 1}, {NULL}};
const Segment SGI90393[] = {{&S68597[73], 11}, {NULL}};
const Segment SGI90394[] = {{&S68598[44], 7}, {NULL}};
const Segment SGI90395[] = {{&S68598[44], 18}, {NULL}};
const Segment SGI90396[] = {{&S68598[30], 6}, {NULL}};
const Segment SGI90397[] = {{&S68598[129], 3}, {NULL}};
const Segment SGI90398[] = {{&S68598[95], 5}, {NULL}};
const Segment SGI90399[] = {{&S68598[35], 1}, {NULL}};
const Segment SGI90400[] = {{&S68598[64], 11}, {NULL}};
const Segment SGI90401[] = {{&S55887[34], 5}, {NULL}};
const Segment SGI90402[] = {{&S55887[27], 6}, {NULL}};
const Segment SGI90403[] = {{&S68599[44], 10}, {NULL}};
const Segment SGI90404[] = {{&S68599[44], 27}, {NULL}};
const Segment SGI90405[] = {{&S68599[30], 6}, {NULL}};
const Segment SGI90406[] = {{&S55887[92], 9}, {NULL}};
const Segment SGI90407[] = {{&S55887[65], 6}, {NULL}};
const Segment SGI90408[] = {{&S68599[138], 3}, {NULL}};
const Segment SGI90409[] = {{&S68599[104], 5}, {NULL}};
const Segment SGI90410[] = {{&S68599[35], 1}, {NULL}};
const Segment SGI90411[] = {{&S68599[73], 11}, {NULL}};
const Segment SGI90412[] = {{&S76387[44], 5}, {NULL}};
const Segment SGI90413[] = {{&S76387[44], 17}, {NULL}};
const Segment SGI90414[] = {{&S76387[30], 6}, {NULL}};
const Segment SGI90415[] = {{&S55889[11], 1}, {NULL}};
const Segment SGI90416[] = {{&S55889[27], 6}, {NULL}};
const Segment SGI90417[] = {{&S76387[109], 3}, {NULL}};
const Segment SGI90418[] = {{&S76387[75], 5}, {NULL}};
const Segment SGI90419[] = {{&S55889[83], 9}, {NULL}};
const Segment SGI90420[] = {{&S55889[56], 6}, {NULL}};
const Segment SGI90421[] = {{&S55890[34], 16}, {NULL}};
const Segment SGI90422[] = {{&S55890[27], 6}, {NULL}};
const Segment SGI90423[] = {{&S55890[103], 9}, {NULL}};
const Segment SGI90424[] = {{&S55890[76], 6}, {NULL}};
const Segment SGI90425[] = {{&S55898[34], 7}, {NULL}};
const Segment SGI90426[] = {{&S55898[27], 6}, {NULL}};
const Segment SGI90427[] = {{&S55898[94], 9}, {NULL}};
const Segment SGI90428[] = {{&S55898[67], 6}, {NULL}};
const Segment SGI90429[] = {{&S55900[34], 5}, {NULL}};
const Segment SGI90430[] = {{&S55900[27], 6}, {NULL}};
const Segment SGI90431[] = {{&S55900[92], 9}, {NULL}};
const Segment SGI90432[] = {{&S55900[65], 6}, {NULL}};
const Segment SGI90433[] = {{&S55904[34], 6}, {NULL}};
const Segment SGI90434[] = {{&S55904[27], 6}, {NULL}};
const Segment SGI90435[] = {{&S55904[93], 9}, {NULL}};
const Segment SGI90436[] = {{&S55904[66], 6}, {NULL}};
const Segment SGI90437[] = {{&S55955[34], 16}, {NULL}};
const Segment SGI90438[] = {{&S55955[27], 6}, {NULL}};
const Segment SGI90439[] = {{&S55955[103], 9}, {NULL}};
const Segment SGI90440[] = {{&S55955[76], 6}, {NULL}};
const Segment SGI90441[] = {{&S55956[34], 9}, {NULL}};
const Segment SGI90442[] = {{&S55956[27], 6}, {NULL}};
const Segment SGI90443[] = {{&S55956[96], 9}, {NULL}};
const Segment SGI90444[] = {{&S55956[69], 6}, {NULL}};
const Segment SGI90445[] = {{&S55985[34], 6}, {NULL}};
const Segment SGI90446[] = {{&S55985[27], 6}, {NULL}};
const Segment SGI90447[] = {{&S55985[92], 9}, {NULL}};
const Segment SGI90448[] = {{&S55985[65], 6}, {NULL}};
const Segment SGI90449[] = {{&S76462[44], 11}, {NULL}};
const Segment SGI90450[] = {{&S76462[44], 28}, {NULL}};
const Segment SGI90451[] = {{&S76462[30], 6}, {NULL}};
const Segment SGI90452[] = {{&S76462[120], 3}, {NULL}};
const Segment SGI90453[] = {{&S76462[86], 5}, {NULL}};
const Segment SGI90454[] = {{&S76537[44], 10}, {NULL}};
const Segment SGI90455[] = {{&S76537[44], 27}, {NULL}};
const Segment SGI90456[] = {{&S76537[30], 6}, {NULL}};
const Segment SGI90457[] = {{&S76537[85], 6}, {NULL}};
const Segment SGI90458[] = {{&S76537[112], 13}, {NULL}};
const Segment SGI90459[] = {{&S76538[44], 10}, {NULL}};
const Segment SGI90460[] = {{&S76538[44], 27}, {NULL}};
const Segment SGI90461[] = {{&S76538[30], 6}, {NULL}};
const Segment SGI90462[] = {{&S76538[85], 6}, {NULL}};
const Segment SGI90463[] = {{&S76538[112], 13}, {NULL}};
const Segment SGI90464[] = {{&S56069[34], 10}, {NULL}};
const Segment SGI90465[] = {{&S56069[27], 6}, {NULL}};
const Segment SGI90466[] = {{&S56069[97], 9}, {NULL}};
const Segment SGI90467[] = {{&S56069[70], 6}, {NULL}};
const Segment SGI90468[] = {{&S56070[34], 9}, {NULL}};
const Segment SGI90469[] = {{&S56070[27], 6}, {NULL}};
const Segment SGI90470[] = {{&S56070[96], 9}, {NULL}};
const Segment SGI90471[] = {{&S56070[69], 6}, {NULL}};
const Segment SGI90472[] = {{&S56073[34], 9}, {NULL}};
const Segment SGI90473[] = {{&S56073[27], 6}, {NULL}};
const Segment SGI90474[] = {{&S56073[96], 9}, {NULL}};
const Segment SGI90475[] = {{&S56073[69], 6}, {NULL}};
const Segment SGI90476[] = {{&S76560[44], 11}, {NULL}};
const Segment SGI90477[] = {{&S76560[44], 23}, {NULL}};
const Segment SGI90478[] = {{&S76560[30], 6}, {NULL}};
const Segment SGI90479[] = {{&S76560[115], 3}, {NULL}};
const Segment SGI90480[] = {{&S76560[81], 5}, {NULL}};
const Segment SGI90481[] = {{&S56081[34], 10}, {NULL}};
const Segment SGI90482[] = {{&S56081[27], 6}, {NULL}};
const Segment SGI90483[] = {{&S56081[97], 9}, {NULL}};
const Segment SGI90484[] = {{&S56081[70], 6}, {NULL}};
const Segment SGI90485[] = {{&S56110[34], 13}, {NULL}};
const Segment SGI90486[] = {{&S56110[27], 6}, {NULL}};
const Segment SGI90487[] = {{&S56110[100], 9}, {NULL}};
const Segment SGI90488[] = {{&S56110[73], 6}, {NULL}};
const Segment SGI90489[] = {{&S56120[34], 15}, {NULL}};
const Segment SGI90490[] = {{&S56120[27], 6}, {NULL}};
const Segment SGI90491[] = {{&S56120[101], 9}, {NULL}};
const Segment SGI90492[] = {{&S56120[74], 6}, {NULL}};
const Segment SGI90493[] = {{&S56138[34], 7}, {NULL}};
const Segment SGI90494[] = {{&S56138[27], 6}, {NULL}};
const Segment SGI90495[] = {{&S56138[99], 9}, {NULL}};
const Segment SGI90496[] = {{&S56138[72], 6}, {NULL}};
const Segment SGI90497[] = {{&S76711[44], 14}, {NULL}};
const Segment SGI90498[] = {{&S76711[44], 26}, {NULL}};
const Segment SGI90499[] = {{&S76711[30], 6}, {NULL}};
const Segment SGI90500[] = {{&S76711[118], 3}, {NULL}};
const Segment SGI90501[] = {{&S76711[84], 5}, {NULL}};
const Segment SGI90502[] = {{&S56243[34], 13}, {NULL}};
const Segment SGI90503[] = {{&S56243[27], 6}, {NULL}};
const Segment SGI90504[] = {{&S56243[100], 9}, {NULL}};
const Segment SGI90505[] = {{&S56243[73], 6}, {NULL}};
const Segment SGI90506[] = {{&S56265[34], 10}, {NULL}};
const Segment SGI90507[] = {{&S56265[27], 6}, {NULL}};
const Segment SGI90508[] = {{&S56265[97], 9}, {NULL}};
const Segment SGI90509[] = {{&S56265[70], 6}, {NULL}};
const Segment SGI90510[] = {{&S76739[59], 11}, {NULL}};
const Segment SGI90511[] = {{&S76739[59], 28}, {NULL}};
const Segment SGI90512[] = {{&S76739[45], 6}, {NULL}};
const Segment SGI90513[] = {{&S76739[135], 3}, {NULL}};
const Segment SGI90514[] = {{&S76739[101], 5}, {NULL}};
const Segment SGI90515[] = {{&S76739[0], 14}, {NULL}};
const Segment SGI90516[] = {{&S56270[34], 10}, {NULL}};
const Segment SGI90517[] = {{&S56270[27], 6}, {NULL}};
const Segment SGI90518[] = {{&S56270[97], 9}, {NULL}};
const Segment SGI90519[] = {{&S56270[70], 6}, {NULL}};
const Segment SGI90520[] = {{&S56270[138], 8}, {NULL}};
const Segment SGI90521[] = {{&S56273[34], 11}, {NULL}};
const Segment SGI90522[] = {{&S56273[27], 6}, {NULL}};
const Segment SGI90523[] = {{&S56273[98], 9}, {NULL}};
const Segment SGI90524[] = {{&S56273[71], 6}, {NULL}};
const Segment SGI90525[] = {{&S56273[139], 8}, {NULL}};
const Segment SGI90526[] = {{&S56274[34], 11}, {NULL}};
const Segment SGI90527[] = {{&S56274[27], 6}, {NULL}};
const Segment SGI90528[] = {{&S56274[98], 9}, {NULL}};
const Segment SGI90529[] = {{&S56274[71], 6}, {NULL}};
const Segment SGI90530[] = {{&S56274[139], 8}, {NULL}};
const Segment SGI90531[] = {{&S56275[34], 11}, {NULL}};
const Segment SGI90532[] = {{&S56275[27], 6}, {NULL}};
const Segment SGI90533[] = {{&S56275[98], 9}, {NULL}};
const Segment SGI90534[] = {{&S56275[71], 6}, {NULL}};
const Segment SGI90535[] = {{&S56275[139], 8}, {NULL}};
const Segment SGI90536[] = {{&S56309[34], 13}, {NULL}};
const Segment SGI90537[] = {{&S56309[27], 6}, {NULL}};
const Segment SGI90538[] = {{&S56309[100], 9}, {NULL}};
const Segment SGI90539[] = {{&S56309[73], 6}, {NULL}};
const Segment SGI90540[] = {{&S56312[34], 10}, {NULL}};
const Segment SGI90541[] = {{&S56312[27], 6}, {NULL}};
const Segment SGI90542[] = {{&S56312[97], 9}, {NULL}};
const Segment SGI90543[] = {{&S56312[70], 6}, {NULL}};
const Segment SGI90544[] = {{&S76880[44], 10}, {NULL}};
const Segment SGI90545[] = {{&S76880[44], 27}, {NULL}};
const Segment SGI90546[] = {{&S76880[30], 6}, {NULL}};
const Segment SGI90547[] = {{&S76880[163], 3}, {NULL}};
const Segment SGI90548[] = {{&S76880[129], 3}, {NULL}};
const Segment SGI90549[] = {{&S76880[94], 6}, {NULL}};
const Segment SGI90550[] = {{&S76880[149], 8}, {NULL}};
const Segment SGI90551[] = {{&S76920[44], 6}, {NULL}};
const Segment SGI90552[] = {{&S76920[44], 18}, {NULL}};
const Segment SGI90553[] = {{&S76920[30], 6}, {NULL}};
const Segment SGI90554[] = {{&S76920[110], 3}, {NULL}};
const Segment SGI90555[] = {{&S76920[76], 5}, {NULL}};
const Segment SGI90556[] = {{&S66328[42], 14}, {NULL}};
const Segment SGI90557[] = {{&S66328[42], 25}, {NULL}};
const Segment SGI90558[] = {{&S66328[30], 4}, {NULL}};
const Segment SGI90559[] = {{&S76928[44], 11}, {NULL}};
const Segment SGI90560[] = {{&S76928[44], 23}, {NULL}};
const Segment SGI90561[] = {{&S76928[30], 6}, {NULL}};
const Segment SGI90562[] = {{&S66328[31], 3}, {NULL}};
const Segment SGI90563[] = {{&S66328[81], 6}, {NULL}};
const Segment SGI90564[] = {{&S76928[115], 3}, {NULL}};
const Segment SGI90565[] = {{&S76928[81], 5}, {NULL}};
const Segment SGI90566[] = {{&S76946[44], 10}, {NULL}};
const Segment SGI90567[] = {{&S76946[44], 27}, {NULL}};
const Segment SGI90568[] = {{&S76946[30], 6}, {NULL}};
const Segment SGI90569[] = {{&S76946[85], 6}, {NULL}};
const Segment SGI90570[] = {{&S76946[112], 13}, {NULL}};
const Segment SGI90571[] = {{&S66370[42], 17}, {NULL}};
const Segment SGI90572[] = {{&S66370[42], 38}, {NULL}};
const Segment SGI90573[] = {{&S66370[30], 4}, {NULL}};
const Segment SGI90574[] = {{&S66370[31], 3}, {NULL}};
const Segment SGI90575[] = {{&S66370[94], 5}, {NULL}};
const Segment SGI90576[] = {{&S76974[44], 16}, {NULL}};
const Segment SGI90577[] = {{&S76974[44], 27}, {NULL}};
const Segment SGI90578[] = {{&S76974[30], 6}, {NULL}};
const Segment SGI90579[] = {{&S76974[119], 3}, {NULL}};
const Segment SGI90580[] = {{&S76974[85], 5}, {NULL}};
const Segment SGI90581[] = {{&S66372[42], 17}, {NULL}};
const Segment SGI90582[] = {{&S66372[42], 38}, {NULL}};
const Segment SGI90583[] = {{&S66372[30], 4}, {NULL}};
const Segment SGI90584[] = {{&S66372[31], 3}, {NULL}};
const Segment SGI90585[] = {{&S66372[94], 5}, {NULL}};
const Segment SGI90586[] = {{&S69099[44], 10}, {NULL}};
const Segment SGI90587[] = {{&S69099[44], 27}, {NULL}};
const Segment SGI90588[] = {{&S69099[30], 6}, {NULL}};
const Segment SGI90589[] = {{&S69099[119], 3}, {NULL}};
const Segment SGI90590[] = {{&S69099[85], 5}, {NULL}};
const Segment SGI90591[] = {{&S69102[38], 10}, {NULL}};
const Segment SGI90592[] = {{&S69102[31], 6}, {NULL}};
const Segment SGI90593[] = {{&S69103[44], 10}, {NULL}};
const Segment SGI90594[] = {{&S69103[44], 22}, {NULL}};
const Segment SGI90595[] = {{&S69103[30], 6}, {NULL}};
const Segment SGI90596[] = {{&S69103[121], 3}, {NULL}};
const Segment SGI90597[] = {{&S69103[87], 5}, {NULL}};
const Segment SGI90598[] = {{&S69128[44], 9}, {NULL}};
const Segment SGI90599[] = {{&S69128[44], 21}, {NULL}};
const Segment SGI90600[] = {{&S69128[30], 6}, {NULL}};
const Segment SGI90601[] = {{&S69128[113], 3}, {NULL}};
const Segment SGI90602[] = {{&S69128[79], 5}, {NULL}};
const Segment SGI90603[] = {{&S69129[44], 9}, {NULL}};
const Segment SGI90604[] = {{&S69129[44], 20}, {NULL}};
const Segment SGI90605[] = {{&S69129[30], 6}, {NULL}};
const Segment SGI90606[] = {{&S69129[112], 3}, {NULL}};
const Segment SGI90607[] = {{&S69129[78], 5}, {NULL}};
const Segment SGI90608[] = {{&S77032[44], 5}, {NULL}};
const Segment SGI90609[] = {{&S77032[44], 17}, {NULL}};
const Segment SGI90610[] = {{&S77032[30], 6}, {NULL}};
const Segment SGI90611[] = {{&S77032[109], 3}, {NULL}};
const Segment SGI90612[] = {{&S77032[75], 5}, {NULL}};
const Segment SGI90613[] = {{&S77041[44], 10}, {NULL}};
const Segment SGI90614[] = {{&S77041[44], 27}, {NULL}};
const Segment SGI90615[] = {{&S77041[30], 6}, {NULL}};
const Segment SGI90616[] = {{&S77041[85], 6}, {NULL}};
const Segment SGI90617[] = {{&S77041[112], 13}, {NULL}};
const Segment SGI90618[] = {{&S69260[44], 7}, {NULL}};
const Segment SGI90619[] = {{&S69260[44], 24}, {NULL}};
const Segment SGI90620[] = {{&S69260[30], 6}, {NULL}};
const Segment SGI90621[] = {{&S69260[82], 6}, {NULL}};
const Segment SGI90622[] = {{&S69260[109], 13}, {NULL}};
const Segment SGI90623[] = {{&S69261[44], 7}, {NULL}};
const Segment SGI90624[] = {{&S69261[44], 24}, {NULL}};
const Segment SGI90625[] = {{&S69261[30], 6}, {NULL}};
const Segment SGI90626[] = {{&S69261[82], 6}, {NULL}};
const Segment SGI90627[] = {{&S69261[109], 13}, {NULL}};
const Segment SGI90628[] = {{&S69262[44], 7}, {NULL}};
const Segment SGI90629[] = {{&S69262[44], 24}, {NULL}};
const Segment SGI90630[] = {{&S69262[30], 6}, {NULL}};
const Segment SGI90631[] = {{&S69262[82], 6}, {NULL}};
const Segment SGI90632[] = {{&S69262[109], 13}, {NULL}};
const Segment SGI90633[] = {{&S77073[44], 9}, {NULL}};
const Segment SGI90634[] = {{&S77073[44], 21}, {NULL}};
const Segment SGI90635[] = {{&S77073[30], 6}, {NULL}};
const Segment SGI90636[] = {{&S77073[113], 3}, {NULL}};
const Segment SGI90637[] = {{&S77073[79], 5}, {NULL}};
const Segment SGI90638[] = {{&S77077[44], 5}, {NULL}};
const Segment SGI90639[] = {{&S77077[44], 26}, {NULL}};
const Segment SGI90640[] = {{&S77077[30], 6}, {NULL}};
const Segment SGI90641[] = {{&S77077[84], 6}, {NULL}};
const Segment SGI90642[] = {{&S77077[111], 13}, {NULL}};
const Segment SGI90643[] = {{&S69235[44], 12}, {NULL}};
const Segment SGI90644[] = {{&S69235[44], 26}, {NULL}};
const Segment SGI90645[] = {{&S69235[30], 6}, {NULL}};
const Segment SGI90646[] = {{&S69235[119], 3}, {NULL}};
const Segment SGI90647[] = {{&S69235[84], 6}, {NULL}};
const Segment SGI90648[] = {{&S69236[44], 7}, {NULL}};
const Segment SGI90649[] = {{&S69236[44], 21}, {NULL}};
const Segment SGI90650[] = {{&S69236[30], 6}, {NULL}};
const Segment SGI90651[] = {{&S69236[114], 3}, {NULL}};
const Segment SGI90652[] = {{&S69236[79], 6}, {NULL}};
const Segment SGI90653[] = {{&S69237[44], 7}, {NULL}};
const Segment SGI90654[] = {{&S69237[44], 23}, {NULL}};
const Segment SGI90655[] = {{&S69237[30], 6}, {NULL}};
const Segment SGI90656[] = {{&S69237[116], 3}, {NULL}};
const Segment SGI90657[] = {{&S69237[81], 6}, {NULL}};
const Segment SGI90658[] = {{&S69238[44], 7}, {NULL}};
const Segment SGI90659[] = {{&S69238[44], 21}, {NULL}};
const Segment SGI90660[] = {{&S69238[30], 6}, {NULL}};
const Segment SGI90661[] = {{&S69238[114], 3}, {NULL}};
const Segment SGI90662[] = {{&S69238[79], 6}, {NULL}};
const Segment SGI90663[] = {{&S69239[44], 7}, {NULL}};
const Segment SGI90664[] = {{&S69239[44], 24}, {NULL}};
const Segment SGI90665[] = {{&S69239[30], 6}, {NULL}};
const Segment SGI90666[] = {{&S69239[117], 3}, {NULL}};
const Segment SGI90667[] = {{&S69239[82], 6}, {NULL}};
const Segment SGI90668[] = {{&S69240[44], 7}, {NULL}};
const Segment SGI90669[] = {{&S69240[44], 21}, {NULL}};
const Segment SGI90670[] = {{&S69240[30], 6}, {NULL}};
const Segment SGI90671[] = {{&S69240[114], 3}, {NULL}};
const Segment SGI90672[] = {{&S69240[79], 6}, {NULL}};
const Segment SGI90673[] = {{&S69241[44], 7}, {NULL}};
const Segment SGI90674[] = {{&S69241[44], 23}, {NULL}};
const Segment SGI90675[] = {{&S69241[30], 6}, {NULL}};
const Segment SGI90676[] = {{&S69241[116], 3}, {NULL}};
const Segment SGI90677[] = {{&S69241[81], 6}, {NULL}};
const Segment SGI90678[] = {{&S69242[44], 7}, {NULL}};
const Segment SGI90679[] = {{&S69242[44], 21}, {NULL}};
const Segment SGI90680[] = {{&S69242[30], 6}, {NULL}};
const Segment SGI90681[] = {{&S69242[114], 3}, {NULL}};
const Segment SGI90682[] = {{&S69242[79], 6}, {NULL}};
const Segment SGI90683[] = {{&S69243[44], 7}, {NULL}};
const Segment SGI90684[] = {{&S69243[44], 23}, {NULL}};
const Segment SGI90685[] = {{&S69243[30], 6}, {NULL}};
const Segment SGI90686[] = {{&S69243[116], 3}, {NULL}};
const Segment SGI90687[] = {{&S69366[44], 9}, {NULL}};
const Segment SGI90688[] = {{&S69243[81], 6}, {NULL}};
const Segment SGI90689[] = {{&S69366[44], 19}, {NULL}};
const Segment SGI90690[] = {{&S69366[30], 6}, {NULL}};
const Segment SGI90691[] = {{&S69366[111], 3}, {NULL}};
const Segment SGI90692[] = {{&S69366[77], 5}, {NULL}};
const Segment SGI90693[] = {{&S69367[44], 8}, {NULL}};
const Segment SGI90694[] = {{&S69367[44], 18}, {NULL}};
const Segment SGI90695[] = {{&S69367[30], 6}, {NULL}};
const Segment SGI90696[] = {{&S69367[110], 3}, {NULL}};
const Segment SGI90697[] = {{&S69367[76], 5}, {NULL}};
const Segment SGI90698[] = {{&S69368[38], 9}, {NULL}};
const Segment SGI90699[] = {{&S69368[31], 6}, {NULL}};
const Segment SGI90700[] = {{&S69369[44], 8}, {NULL}};
const Segment SGI90701[] = {{&S69369[44], 18}, {NULL}};
const Segment SGI90702[] = {{&S69369[30], 6}, {NULL}};
const Segment SGI90703[] = {{&S69369[79], 6}, {NULL}};
const Segment SGI90704[] = {{&S77197[44], 15}, {NULL}};
const Segment SGI90705[] = {{&S77197[44], 27}, {NULL}};
const Segment SGI90706[] = {{&S77197[30], 6}, {NULL}};
const Segment SGI90707[] = {{&S77197[119], 3}, {NULL}};
const Segment SGI90708[] = {{&S77197[85], 5}, {NULL}};
const Segment SGI90709[] = {{&S77198[44], 10}, {NULL}};
const Segment SGI90710[] = {{&S77198[44], 22}, {NULL}};
const Segment SGI90711[] = {{&S77198[30], 6}, {NULL}};
const Segment SGI90712[] = {{&S77198[83], 6}, {NULL}};
const Segment SGI90713[] = {{&S77199[44], 15}, {NULL}};
const Segment SGI90714[] = {{&S77199[44], 27}, {NULL}};
const Segment SGI90715[] = {{&S77199[30], 6}, {NULL}};
const Segment SGI90716[] = {{&S77199[119], 3}, {NULL}};
const Segment SGI90717[] = {{&S77199[85], 5}, {NULL}};
const Segment SGI90718[] = {{&S77200[44], 10}, {NULL}};
const Segment SGI90719[] = {{&S77200[44], 22}, {NULL}};
const Segment SGI90720[] = {{&S77200[30], 6}, {NULL}};
const Segment SGI90721[] = {{&S77200[114], 3}, {NULL}};
const Segment SGI90722[] = {{&S77200[80], 5}, {NULL}};
const Segment SGI90723[] = {{&S69412[44], 10}, {NULL}};
const Segment SGI90724[] = {{&S69412[44], 21}, {NULL}};
const Segment SGI90725[] = {{&S69412[30], 6}, {NULL}};
const Segment SGI90726[] = {{&S69412[132], 3}, {NULL}};
const Segment SGI90727[] = {{&S69412[98], 5}, {NULL}};
const Segment SGI90728[] = {{&S69412[35], 1}, {NULL}};
const Segment SGI90729[] = {{&S69412[67], 11}, {NULL}};
const Segment SGI90730[] = {{&S77227[45], 10}, {NULL}};
const Segment SGI90731[] = {{&S77227[30], 6}, {NULL}};
const Segment SGI90732[] = {{&S77227[120], 3}, {NULL}};
const Segment SGI90733[] = {{&S77227[86], 5}, {NULL}};
const Segment SGI90734[] = {{&S77227[144], 13}, {NULL}};
const Segment SGI90735[] = {{&S77232[59], 5}, {NULL}};
const Segment SGI90736[] = {{&S77232[59], 17}, {NULL}};
const Segment SGI90737[] = {{&S77232[45], 6}, {NULL}};
const Segment SGI90738[] = {{&S77232[124], 3}, {NULL}};
const Segment SGI90739[] = {{&S77232[90], 5}, {NULL}};
const Segment SGI90740[] = {{&S77232[0], 14}, {NULL}};
const Segment SGI90741[] = {{&S77245[44], 10}, {NULL}};
const Segment SGI90742[] = {{&S77245[44], 27}, {NULL}};
const Segment SGI90743[] = {{&S77245[30], 6}, {NULL}};
const Segment SGI90744[] = {{&S77245[85], 6}, {NULL}};
const Segment SGI90745[] = {{&S77245[112], 13}, {NULL}};
const Segment SGI90746[] = {{&S77247[44], 15}, {NULL}};
const Segment SGI90747[] = {{&S77247[44], 27}, {NULL}};
const Segment SGI90748[] = {{&S77247[30], 6}, {NULL}};
const Segment SGI90749[] = {{&S77247[119], 3}, {NULL}};
const Segment SGI90750[] = {{&S77247[85], 5}, {NULL}};
const Segment SGI90751[] = {{&S56700[34], 26}, {NULL}};
const Segment SGI90752[] = {{&S56700[27], 6}, {NULL}};
const Segment SGI90753[] = {{&S56700[113], 9}, {NULL}};
const Segment SGI90754[] = {{&S56700[86], 6}, {NULL}};
const Segment SGI90755[] = {{&S56701[34], 25}, {NULL}};
const Segment SGI90756[] = {{&S56701[27], 6}, {NULL}};
const Segment SGI90757[] = {{&S56701[112], 9}, {NULL}};
const Segment SGI90758[] = {{&S56701[85], 6}, {NULL}};
const Segment SGI90759[] = {{&S56703[34], 7}, {NULL}};
const Segment SGI90760[] = {{&S56703[27], 6}, {NULL}};
const Segment SGI90761[] = {{&S56703[94], 9}, {NULL}};
const Segment SGI90762[] = {{&S56703[67], 6}, {NULL}};
const Segment SGI90763[] = {{&S77290[44], 9}, {NULL}};
const Segment SGI90764[] = {{&S77290[44], 21}, {NULL}};
const Segment SGI90765[] = {{&S77290[30], 6}, {NULL}};
const Segment SGI90766[] = {{&S77290[113], 3}, {NULL}};
const Segment SGI90767[] = {{&S77290[79], 5}, {NULL}};
const Segment SGI90768[] = {{&S77310[44], 11}, {NULL}};
const Segment SGI90769[] = {{&S77310[44], 28}, {NULL}};
const Segment SGI90770[] = {{&S77310[30], 6}, {NULL}};
const Segment SGI90771[] = {{&S77310[86], 6}, {NULL}};
const Segment SGI90772[] = {{&S77310[113], 13}, {NULL}};
const Segment SGI90773[] = {{&S77339[45], 24}, {NULL}};
const Segment SGI90774[] = {{&S77339[30], 6}, {NULL}};
const Segment SGI90775[] = {{&S77339[129], 3}, {NULL}};
const Segment SGI90776[] = {{&S56777[44], 7}, {NULL}};
const Segment SGI90777[] = {{&S56777[44], 19}, {NULL}};
const Segment SGI90778[] = {{&S56777[30], 6}, {NULL}};
const Segment SGI90779[] = {{&S77339[95], 5}, {NULL}};
const Segment SGI90780[] = {{&S77339[153], 13}, {NULL}};
const Segment SGI90781[] = {{&S56777[31], 3}, {NULL}};
const Segment SGI90782[] = {{&S56777[77], 6}, {NULL}};
const Segment SGI90783[] = {{&S56778[38], 7}, {NULL}};
const Segment SGI90784[] = {{&S77340[44], 24}, {NULL}};
const Segment SGI90785[] = {{&S56778[31], 6}, {NULL}};
const Segment SGI90786[] = {{&S77340[44], 36}, {NULL}};
const Segment SGI90787[] = {{&S77340[30], 6}, {NULL}};
const Segment SGI90788[] = {{&S77340[129], 3}, {NULL}};
const Segment SGI90789[] = {{&S77340[94], 6}, {NULL}};
const Segment SGI90790[] = {{&S77340[149], 8}, {NULL}};
const Segment SGI90791[] = {{&S77340[158], 13}, {NULL}};
const Segment SGI90792[] = {{&S77341[44], 13}, {NULL}};
const Segment SGI90793[] = {{&S77341[44], 25}, {NULL}};
const Segment SGI90794[] = {{&S77341[30], 6}, {NULL}};
const Segment SGI90795[] = {{&S77341[117], 3}, {NULL}};
const Segment SGI90796[] = {{&S77341[83], 5}, {NULL}};
const Segment SGI90797[] = {{&S56792[34], 15}, {NULL}};
const Segment SGI90798[] = {{&S56792[27], 6}, {NULL}};
const Segment SGI90799[] = {{&S56792[104], 9}, {NULL}};
const Segment SGI90800[] = {{&S56792[77], 6}, {NULL}};
const Segment SGI90801[] = {{&S77384[31], 8}, {NULL}};
const Segment SGI90802[] = {{&S77384[54], 3}, {NULL}};
const Segment SGI90803[] = {{&S56825[34], 42}, {NULL}};
const Segment SGI90804[] = {{&S56825[27], 6}, {NULL}};
const Segment SGI90805[] = {{&S56825[129], 9}, {NULL}};
const Segment SGI90806[] = {{&S56825[102], 6}, {NULL}};
const Segment SGI90807[] = {{&S56826[34], 41}, {NULL}};
const Segment SGI90808[] = {{&S56826[27], 6}, {NULL}};
const Segment SGI90809[] = {{&S56826[128], 9}, {NULL}};
const Segment SGI90810[] = {{&S56826[101], 6}, {NULL}};
const Segment SGI90811[] = {{&S56827[34], 41}, {NULL}};
const Segment SGI90812[] = {{&S56827[27], 6}, {NULL}};
const Segment SGI90813[] = {{&S56827[128], 9}, {NULL}};
const Segment SGI90814[] = {{&S56827[101], 6}, {NULL}};
const Segment SGI90815[] = {{&S56828[34], 31}, {NULL}};
const Segment SGI90816[] = {{&S56828[27], 6}, {NULL}};
const Segment SGI90817[] = {{&S56828[118], 9}, {NULL}};
const Segment SGI90818[] = {{&S56828[91], 6}, {NULL}};
const Segment SGI90819[] = {{&S56837[34], 18}, {NULL}};
const Segment SGI90820[] = {{&S56837[27], 6}, {NULL}};
const Segment SGI90821[] = {{&S56837[105], 9}, {NULL}};
const Segment SGI90822[] = {{&S56837[78], 6}, {NULL}};
const Segment SGI90823[] = {{&S69621[44], 8}, {NULL}};
const Segment SGI90824[] = {{&S69621[44], 29}, {NULL}};
const Segment SGI90825[] = {{&S69621[30], 6}, {NULL}};
const Segment SGI90826[] = {{&S69621[121], 3}, {NULL}};
const Segment SGI90827[] = {{&S69621[87], 5}, {NULL}};
const Segment SGI90828[] = {{&S77429[44], 10}, {NULL}};
const Segment SGI90829[] = {{&S77429[44], 27}, {NULL}};
const Segment SGI90830[] = {{&S77429[30], 6}, {NULL}};
const Segment SGI90831[] = {{&S77429[149], 4}, {NULL}};
const Segment SGI90832[] = {{&S77429[175], 3}, {NULL}};
const Segment SGI90833[] = {{&S77429[119], 3}, {&S77429[224], 5}, {NULL}};
const Segment SGI90834[] = {{&S77429[85], 5}, {NULL}};
const Segment SGI90835[] = {{&S56866[34], 6}, {NULL}};
const Segment SGI90836[] = {{&S56866[27], 6}, {NULL}};
const Segment SGI90837[] = {{&S56866[93], 9}, {NULL}};
const Segment SGI90838[] = {{&S56866[66], 6}, {NULL}};
const Segment SGI90839[] = {{&S56872[34], 11}, {NULL}};
const Segment SGI90840[] = {{&S56872[27], 6}, {NULL}};
const Segment SGI90841[] = {{&S56872[98], 9}, {NULL}};
const Segment SGI90842[] = {{&S56872[71], 6}, {NULL}};
const Segment SGI90843[] = {{&S56892[34], 18}, {NULL}};
const Segment SGI90844[] = {{&S56892[27], 6}, {NULL}};
const Segment SGI90845[] = {{&S56892[105], 9}, {NULL}};
const Segment SGI90846[] = {{&S56892[78], 6}, {NULL}};
const Segment SGI90847[] = {{&S56894[34], 10}, {NULL}};
const Segment SGI90848[] = {{&S56894[27], 6}, {NULL}};
const Segment SGI90849[] = {{&S56894[97], 9}, {NULL}};
const Segment SGI90850[] = {{&S56894[70], 6}, {NULL}};
const Segment SGI90851[] = {{&S56917[34], 16}, {NULL}};
const Segment SGI90852[] = {{&S56917[27], 6}, {NULL}};
const Segment SGI90853[] = {{&S56917[103], 9}, {NULL}};
const Segment SGI90854[] = {{&S56917[76], 6}, {NULL}};
const Segment SGI90855[] = {{&S56919[34], 15}, {NULL}};
const Segment SGI90856[] = {{&S56919[27], 6}, {NULL}};
const Segment SGI90857[] = {{&S56919[102], 9}, {NULL}};
const Segment SGI90858[] = {{&S56919[75], 6}, {NULL}};
const Segment SGI90859[] = {{&S56921[34], 11}, {NULL}};
const Segment SGI90860[] = {{&S56921[27], 6}, {NULL}};
const Segment SGI90861[] = {{&S56921[103], 9}, {NULL}};
const Segment SGI90862[] = {{&S56921[76], 6}, {NULL}};
const Segment SGI90863[] = {{&S56963[34], 12}, {NULL}};
const Segment SGI90864[] = {{&S56963[27], 6}, {NULL}};
const Segment SGI90865[] = {{&S56963[99], 9}, {NULL}};
const Segment SGI90866[] = {{&S56963[72], 6}, {NULL}};
const Segment SGI90867[] = {{&S56974[44], 8}, {NULL}};
const Segment SGI90868[] = {{&S56974[44], 20}, {NULL}};
const Segment SGI90869[] = {{&S56974[30], 6}, {NULL}};
const Segment SGI90870[] = {{&S56974[31], 3}, {NULL}};
const Segment SGI90871[] = {{&S56974[78], 6}, {NULL}};
const Segment SGI90872[] = {{&S69804[44], 8}, {NULL}};
const Segment SGI90873[] = {{&S69804[44], 17}, {NULL}};
const Segment SGI90874[] = {{&S69804[30], 6}, {NULL}};
const Segment SGI90875[] = {{&S69804[124], 3}, {NULL}};
const Segment SGI90876[] = {{&S69804[90], 5}, {NULL}};
const Segment SGI90877[] = {{&S69804[35], 1}, {NULL}};
const Segment SGI90878[] = {{&S69804[63], 11}, {NULL}};
const Segment SGI90879[] = {{&S69805[44], 10}, {NULL}};
const Segment SGI90880[] = {{&S69805[44], 27}, {NULL}};
const Segment SGI90881[] = {{&S69805[30], 6}, {NULL}};
const Segment SGI90882[] = {{&S69805[138], 3}, {NULL}};
const Segment SGI90883[] = {{&S69805[104], 5}, {NULL}};
const Segment SGI90884[] = {{&S69805[35], 1}, {NULL}};
const Segment SGI90885[] = {{&S69805[73], 11}, {NULL}};
const Segment SGI90886[] = {{&S77620[44], 10}, {NULL}};
const Segment SGI90887[] = {{&S77620[44], 27}, {NULL}};
const Segment SGI90888[] = {{&S77620[30], 6}, {NULL}};
const Segment SGI90889[] = {{&S77620[85], 6}, {NULL}};
const Segment SGI90890[] = {{&S77620[112], 13}, {NULL}};
const Segment SGI90891[] = {{&S77622[44], 11}, {NULL}};
const Segment SGI90892[] = {{&S77622[44], 28}, {NULL}};
const Segment SGI90893[] = {{&S77622[30], 6}, {NULL}};
const Segment SGI90894[] = {{&S77622[86], 6}, {NULL}};
const Segment SGI90895[] = {{&S77622[113], 13}, {NULL}};
const Segment SGI90896[] = {{&S69742[44], 7}, {NULL}};
const Segment SGI90897[] = {{&S69742[44], 19}, {NULL}};
const Segment SGI90898[] = {{&S69742[30], 6}, {NULL}};
const Segment SGI90899[] = {{&S69742[112], 3}, {NULL}};
const Segment SGI90900[] = {{&S69742[77], 6}, {NULL}};
const Segment SGI90901[] = {{&S77641[45], 10}, {NULL}};
const Segment SGI90902[] = {{&S77641[30], 6}, {NULL}};
const Segment SGI90903[] = {{&S77641[120], 3}, {NULL}};
const Segment SGI90904[] = {{&S77641[86], 5}, {NULL}};
const Segment SGI90905[] = {{&S77641[144], 13}, {NULL}};
const Segment SGI90906[] = {{&S77661[34], 10}, {NULL}};
const Segment SGI90907[] = {{&S77661[27], 6}, {NULL}};
const Segment SGI90908[] = {{&S77661[109], 2}, {NULL}};
const Segment SGI90909[] = {{&S77661[75], 6}, {NULL}};
const Segment SGI90910[] = {{&S77661[143], 8}, {NULL}};
const Segment SGI90911[] = {{&S77688[59], 14}, {NULL}};
const Segment SGI90912[] = {{&S77688[59], 31}, {NULL}};
const Segment SGI90913[] = {{&S77688[45], 6}, {NULL}};
const Segment SGI90914[] = {{&S77688[138], 3}, {NULL}};
const Segment SGI90915[] = {{&S77688[104], 5}, {NULL}};
const Segment SGI90916[] = {{&S77688[0], 14}, {NULL}};
const Segment SGI90917[] = {{&S77689[45], 10}, {NULL}};
const Segment SGI90918[] = {{&S77689[30], 6}, {NULL}};
const Segment SGI90919[] = {{&S77689[120], 3}, {NULL}};
const Segment SGI90920[] = {{&S77689[86], 5}, {NULL}};
const Segment SGI90921[] = {{&S77689[144], 13}, {NULL}};
const Segment SGI90922[] = {{&S77710[44], 5}, {NULL}};
const Segment SGI90923[] = {{&S77710[44], 17}, {NULL}};
const Segment SGI90924[] = {{&S77710[30], 6}, {NULL}};
const Segment SGI90925[] = {{&S77710[75], 6}, {NULL}};
const Segment SGI90926[] = {{&S77710[102], 13}, {NULL}};
const Segment SGI90927[] = {{&S57042[11], 1}, {NULL}};
const Segment SGI90928[] = {{&S57042[27], 6}, {NULL}};
const Segment SGI90929[] = {{&S57042[88], 9}, {NULL}};
const Segment SGI90930[] = {{&S57042[61], 6}, {NULL}};
const Segment SGI90931[] = {{&S57050[34], 16}, {NULL}};
const Segment SGI90932[] = {{&S57050[27], 6}, {NULL}};
const Segment SGI90933[] = {{&S57050[103], 9}, {NULL}};
const Segment SGI90934[] = {{&S57050[76], 6}, {NULL}};
const Segment SGI90935[] = {{&S57051[11], 1}, {NULL}};
const Segment SGI90936[] = {{&S57051[27], 6}, {NULL}};
const Segment SGI90937[] = {{&S57051[83], 9}, {NULL}};
const Segment SGI90938[] = {{&S57051[56], 6}, {NULL}};
const Segment SGI90939[] = {{&S57056[34], 5}, {NULL}};
const Segment SGI90940[] = {{&S57056[27], 6}, {NULL}};
const Segment SGI90941[] = {{&S57056[92], 9}, {NULL}};
const Segment SGI90942[] = {{&S57056[65], 6}, {NULL}};
const Segment SGI90943[] = {{&S57060[34], 7}, {NULL}};
const Segment SGI90944[] = {{&S57060[27], 6}, {NULL}};
const Segment SGI90945[] = {{&S57060[94], 9}, {NULL}};
const Segment SGI90946[] = {{&S57060[67], 6}, {NULL}};
const Segment SGI90947[] = {{&S57062[34], 5}, {NULL}};
const Segment SGI90948[] = {{&S57062[27], 6}, {NULL}};
const Segment SGI90949[] = {{&S57062[92], 9}, {NULL}};
const Segment SGI90950[] = {{&S57062[65], 6}, {NULL}};
const Segment SGI90951[] = {{&S57063[34], 18}, {NULL}};
const Segment SGI90952[] = {{&S57063[27], 6}, {NULL}};
const Segment SGI90953[] = {{&S77769[44], 10}, {NULL}};
const Segment SGI90954[] = {{&S77769[44], 22}, {NULL}};
const Segment SGI90955[] = {{&S77769[30], 6}, {NULL}};
const Segment SGI90956[] = {{&S57063[105], 9}, {NULL}};
const Segment SGI90957[] = {{&S77769[114], 3}, {NULL}};
const Segment SGI90958[] = {{&S57063[78], 6}, {NULL}};
const Segment SGI90959[] = {{&S77769[80], 5}, {NULL}};
const Segment SGI90960[] = {{&S77773[45], 10}, {NULL}};
const Segment SGI90961[] = {{&S77773[30], 6}, {NULL}};
const Segment SGI90962[] = {{&S77773[120], 3}, {NULL}};
const Segment SGI90963[] = {{&S77773[86], 5}, {NULL}};
const Segment SGI90964[] = {{&S77773[144], 13}, {NULL}};
const Segment SGI90965[] = {{&S57080[34], 11}, {NULL}};
const Segment SGI90966[] = {{&S57080[27], 6}, {NULL}};
const Segment SGI90967[] = {{&S57080[98], 9}, {NULL}};
const Segment SGI90968[] = {{&S57080[71], 6}, {NULL}};
const Segment SGI90969[] = {{&S70037[44], 6}, {NULL}};
const Segment SGI90970[] = {{&S70037[44], 16}, {NULL}};
const Segment SGI90971[] = {{&S70037[30], 6}, {NULL}};
const Segment SGI90972[] = {{&S70037[108], 3}, {NULL}};
const Segment SGI90973[] = {{&S70037[74], 5}, {NULL}};
const Segment SGI90974[] = {{&S70039[38], 6}, {NULL}};
const Segment SGI90975[] = {{&S70039[31], 6}, {NULL}};
const Segment SGI90976[] = {{&S70040[44], 6}, {NULL}};
const Segment SGI90977[] = {{&S70040[44], 16}, {NULL}};
const Segment SGI90978[] = {{&S70040[30], 6}, {NULL}};
const Segment SGI90979[] = {{&S70040[77], 6}, {NULL}};
const Segment SGI90980[] = {{&S57088[11], 1}, {NULL}};
const Segment SGI90981[] = {{&S57088[27], 6}, {NULL}};
const Segment SGI90982[] = {{&S57088[88], 9}, {NULL}};
const Segment SGI90983[] = {{&S57088[61], 6}, {NULL}};
const Segment SGI90984[] = {{&S57092[34], 5}, {NULL}};
const Segment SGI90985[] = {{&S57092[27], 6}, {NULL}};
const Segment SGI90986[] = {{&S57092[92], 9}, {NULL}};
const Segment SGI90987[] = {{&S57092[65], 6}, {NULL}};
const Segment SGI90988[] = {{&S57094[34], 16}, {NULL}};
const Segment SGI90989[] = {{&S57094[27], 6}, {NULL}};
const Segment SGI90990[] = {{&S57094[103], 9}, {NULL}};
const Segment SGI90991[] = {{&S57094[76], 6}, {NULL}};
const Segment SGI90992[] = {{&S57099[11], 1}, {NULL}};
const Segment SGI90993[] = {{&S57099[27], 6}, {NULL}};
const Segment SGI90994[] = {{&S57099[94], 9}, {NULL}};
const Segment SGI90995[] = {{&S57099[67], 6}, {NULL}};
const Segment SGI90996[] = {{&S57103[34], 19}, {NULL}};
const Segment SGI90997[] = {{&S57103[27], 6}, {NULL}};
const Segment SGI90998[] = {{&S57103[106], 9}, {NULL}};
const Segment SGI90999[] = {{&S57103[79], 6}, {NULL}};
const Segment SGI91000[] = {{&S57107[34], 20}, {NULL}};
const Segment SGI91001[] = {{&S57107[27], 6}, {NULL}};
const Segment SGI91002[] = {{&S57107[107], 9}, {NULL}};
const Segment SGI91003[] = {{&S57107[80], 6}, {NULL}};
const Segment SGI91004[] = {{&S57108[34], 5}, {NULL}};
const Segment SGI91005[] = {{&S57108[27], 6}, {NULL}};
const Segment SGI91006[] = {{&S57108[92], 9}, {NULL}};
const Segment SGI91007[] = {{&S57108[65], 6}, {NULL}};
const Segment SGI91008[] = {{&S57110[34], 18}, {NULL}};
const Segment SGI91009[] = {{&S57110[27], 6}, {NULL}};
const Segment SGI91010[] = {{&S57110[105], 9}, {NULL}};
const Segment SGI91011[] = {{&S57110[78], 6}, {NULL}};
const Segment SGI91012[] = {{&S77838[44], 5}, {NULL}};
const Segment SGI91013[] = {{&S77838[44], 17}, {NULL}};
const Segment SGI91014[] = {{&S77838[30], 6}, {NULL}};
const Segment SGI91015[] = {{&S77838[109], 3}, {NULL}};
const Segment SGI91016[] = {{&S77838[75], 5}, {NULL}};
const Segment SGI91017[] = {{&S77845[34], 10}, {NULL}};
const Segment SGI91018[] = {{&S77845[27], 6}, {NULL}};
const Segment SGI91019[] = {{&S77845[109], 2}, {NULL}};
const Segment SGI91020[] = {{&S77845[75], 6}, {NULL}};
const Segment SGI91021[] = {{&S77845[143], 8}, {NULL}};
const Segment SGI91022[] = {{&S57150[44], 5}, {NULL}};
const Segment SGI91023[] = {{&S57150[44], 17}, {NULL}};
const Segment SGI91024[] = {{&S57150[30], 6}, {NULL}};
const Segment SGI91025[] = {{&S57150[31], 3}, {NULL}};
const Segment SGI91026[] = {{&S57150[75], 6}, {NULL}};
const Segment SGI91027[] = {{&S70002[44], 25}, {NULL}};
const Segment SGI91028[] = {{&S70002[44], 36}, {NULL}};
const Segment SGI91029[] = {{&S70002[30], 6}, {NULL}};
const Segment SGI91030[] = {{&S70002[129], 3}, {NULL}};
const Segment SGI91031[] = {{&S70002[94], 6}, {NULL}};
const Segment SGI91032[] = {{&S70003[44], 7}, {NULL}};
const Segment SGI91033[] = {{&S70003[44], 18}, {NULL}};
const Segment SGI91034[] = {{&S70003[30], 6}, {NULL}};
const Segment SGI91035[] = {{&S70003[111], 3}, {NULL}};
const Segment SGI91036[] = {{&S70003[76], 6}, {NULL}};
const Segment SGI91037[] = {{&S57185[34], 6}, {NULL}};
const Segment SGI91038[] = {{&S57185[27], 6}, {NULL}};
const Segment SGI91039[] = {{&S57185[93], 9}, {NULL}};
const Segment SGI91040[] = {{&S57185[66], 6}, {NULL}};
const Segment SGI91041[] = {{&S57190[44], 17}, {NULL}};
const Segment SGI91042[] = {{&S57190[44], 29}, {NULL}};
const Segment SGI91043[] = {{&S57190[30], 6}, {NULL}};
const Segment SGI91044[] = {{&S57190[31], 3}, {NULL}};
const Segment SGI91045[] = {{&S57190[87], 6}, {NULL}};
const Segment SGI91046[] = {{&S57191[38], 17}, {NULL}};
const Segment SGI91047[] = {{&S57191[31], 6}, {NULL}};
const Segment SGI91048[] = {{&S57192[44], 23}, {NULL}};
const Segment SGI91049[] = {{&S57192[44], 35}, {NULL}};
const Segment SGI91050[] = {{&S57192[30], 6}, {NULL}};
const Segment SGI91051[] = {{&S57192[31], 3}, {NULL}};
const Segment SGI91052[] = {{&S57192[93], 6}, {NULL}};
const Segment SGI91053[] = {{&S57193[44], 23}, {NULL}};
const Segment SGI91054[] = {{&S57193[44], 35}, {NULL}};
const Segment SGI91055[] = {{&S57193[30], 6}, {NULL}};
const Segment SGI91056[] = {{&S57193[31], 3}, {NULL}};
const Segment SGI91057[] = {{&S57193[93], 6}, {NULL}};
const Segment SGI91058[] = {{&S57194[38], 23}, {NULL}};
const Segment SGI91059[] = {{&S57194[31], 6}, {NULL}};
const Segment SGI91060[] = {{&S70202[30], 6}, {NULL}};
const Segment SGI91061[] = {{&S70202[92], 2}, {NULL}};
const Segment SGI91062[] = {{&S70202[57], 6}, {NULL}};
const Segment SGI91063[] = {{&S57231[34], 10}, {NULL}};
const Segment SGI91064[] = {{&S57231[27], 6}, {NULL}};
const Segment SGI91065[] = {{&S57231[97], 9}, {NULL}};
const Segment SGI91066[] = {{&S57231[70], 6}, {NULL}};
const Segment SGI91067[] = {{&S57241[6], 4}, {NULL}};
const Segment SGI91068[] = {{&S57241[76], 5}, {NULL}};
const Segment SGI91069[] = {{&S64603[77], 6}, {NULL}};
const Segment SGI91070[] = {{&S64603[0], 12}, {NULL}};
const Segment SGI91071[] = {{&S77997[44], 10}, {NULL}};
const Segment SGI91072[] = {{&S77997[44], 27}, {NULL}};
const Segment SGI91073[] = {{&S77997[30], 6}, {NULL}};
const Segment SGI91074[] = {{&S77997[85], 6}, {NULL}};
const Segment SGI91075[] = {{&S77997[112], 13}, {NULL}};
const Segment SGI91076[] = {{&S78015[44], 7}, {NULL}};
const Segment SGI91077[] = {{&S78015[44], 33}, {NULL}};
const Segment SGI91078[] = {{&S78015[30], 6}, {NULL}};
const Segment SGI91079[] = {{&S78015[125], 3}, {NULL}};
const Segment SGI91080[] = {{&S78015[91], 5}, {NULL}};
const Segment SGI91081[] = {{&S78017[44], 7}, {NULL}};
const Segment SGI91082[] = {{&S78017[44], 33}, {NULL}};
const Segment SGI91083[] = {{&S78017[30], 6}, {NULL}};
const Segment SGI91084[] = {{&S78017[125], 3}, {NULL}};
const Segment SGI91085[] = {{&S78017[91], 5}, {NULL}};
const Segment SGI91086[] = {{&S78018[44], 7}, {NULL}};
const Segment SGI91087[] = {{&S78018[44], 35}, {NULL}};
const Segment SGI91088[] = {{&S78018[30], 6}, {NULL}};
const Segment SGI91089[] = {{&S78018[127], 3}, {NULL}};
const Segment SGI91090[] = {{&S78018[93], 5}, {NULL}};
const Segment SGI91091[] = {{&S70337[44], 8}, {NULL}};
const Segment SGI91092[] = {{&S70337[44], 19}, {NULL}};
const Segment SGI91093[] = {{&S70337[30], 6}, {NULL}};
const Segment SGI91094[] = {{&S70337[111], 3}, {NULL}};
const Segment SGI91095[] = {{&S70337[77], 5}, {NULL}};
const Segment SGI91096[] = {{&S70368[44], 11}, {NULL}};
const Segment SGI91097[] = {{&S70368[44], 23}, {NULL}};
const Segment SGI91098[] = {{&S70368[30], 6}, {NULL}};
const Segment SGI91099[] = {{&S70368[115], 3}, {NULL}};
const Segment SGI91100[] = {{&S70368[81], 5}, {NULL}};
const Segment SGI91101[] = {{&S70267[42], 7}, {NULL}};
const Segment SGI91102[] = {{&S70267[42], 21}, {NULL}};
const Segment SGI91103[] = {{&S70267[30], 4}, {NULL}};
const Segment SGI91104[] = {{&S70267[112], 3}, {NULL}};
const Segment SGI91105[] = {{&S70267[77], 6}, {NULL}};
const Segment SGI91106[] = {{&S70268[42], 10}, {NULL}};
const Segment SGI91107[] = {{&S70268[42], 21}, {NULL}};
const Segment SGI91108[] = {{&S70268[30], 4}, {NULL}};
const Segment SGI91109[] = {{&S70268[112], 3}, {NULL}};
const Segment SGI91110[] = {{&S70268[77], 6}, {NULL}};
const Segment SGI91111[] = {{&S70269[42], 8}, {NULL}};
const Segment SGI91112[] = {{&S70269[42], 19}, {NULL}};
const Segment SGI91113[] = {{&S70269[30], 4}, {NULL}};
const Segment SGI91114[] = {{&S70269[110], 3}, {NULL}};
const Segment SGI91115[] = {{&S70269[75], 6}, {NULL}};
const Segment SGI91116[] = {{&S70270[42], 9}, {NULL}};
const Segment SGI91117[] = {{&S70270[42], 20}, {NULL}};
const Segment SGI91118[] = {{&S70270[30], 4}, {NULL}};
const Segment SGI91119[] = {{&S70270[111], 3}, {NULL}};
const Segment SGI91120[] = {{&S70270[76], 6}, {NULL}};
const Segment SGI91121[] = {{&S70271[42], 20}, {NULL}};
const Segment SGI91122[] = {{&S70271[42], 32}, {NULL}};
const Segment SGI91123[] = {{&S70271[30], 4}, {NULL}};
const Segment SGI91124[] = {{&S70271[123], 3}, {NULL}};
const Segment SGI91125[] = {{&S70271[88], 6}, {NULL}};
const Segment SGI91126[] = {{&S70272[42], 100}, {NULL}};
const Segment SGI91127[] = {{&S70272[42], 111}, {NULL}};
const Segment SGI91128[] = {{&S70272[30], 4}, {NULL}};
const Segment SGI91129[] = {{&S70272[202], 3}, {NULL}};
const Segment SGI91130[] = {{&S70272[167], 6}, {NULL}};
const Segment SGI91131[] = {{&S70273[42], 5}, {NULL}};
const Segment SGI91132[] = {{&S70273[42], 17}, {NULL}};
const Segment SGI91133[] = {{&S70273[30], 4}, {NULL}};
const Segment SGI91134[] = {{&S70273[108], 3}, {NULL}};
const Segment SGI91135[] = {{&S70273[73], 6}, {NULL}};
const Segment SGI91136[] = {{&S70274[42], 14}, {NULL}};
const Segment SGI91137[] = {{&S70274[42], 25}, {NULL}};
const Segment SGI91138[] = {{&S70274[30], 4}, {NULL}};
const Segment SGI91139[] = {{&S70274[116], 3}, {NULL}};
const Segment SGI91140[] = {{&S70274[81], 6}, {NULL}};
const Segment SGI91141[] = {{&S70275[42], 13}, {NULL}};
const Segment SGI91142[] = {{&S70275[42], 25}, {NULL}};
const Segment SGI91143[] = {{&S70275[30], 4}, {NULL}};
const Segment SGI91144[] = {{&S70275[116], 3}, {NULL}};
const Segment SGI91145[] = {{&S70275[81], 6}, {NULL}};
const Segment SGI91146[] = {{&S70424[44], 20}, {NULL}};
const Segment SGI91147[] = {{&S70424[44], 35}, {NULL}};
const Segment SGI91148[] = {{&S57377[34], 35}, {NULL}};
const Segment SGI91149[] = {{&S70424[30], 6}, {NULL}};
const Segment SGI91150[] = {{&S57377[27], 6}, {NULL}};
const Segment SGI91151[] = {{&S70424[127], 3}, {NULL}};
const Segment SGI91152[] = {{&S70424[93], 5}, {NULL}};
const Segment SGI91153[] = {{&S57377[122], 9}, {NULL}};
const Segment SGI91154[] = {{&S57377[95], 6}, {NULL}};
const Segment SGI91155[] = {{&S70425[44], 20}, {NULL}};
const Segment SGI91156[] = {{&S57378[34], 41}, {NULL}};
const Segment SGI91157[] = {{&S70425[44], 35}, {NULL}};
const Segment SGI91158[] = {{&S70425[30], 6}, {NULL}};
const Segment SGI91159[] = {{&S57378[27], 6}, {NULL}};
const Segment SGI91160[] = {{&S70425[127], 3}, {NULL}};
const Segment SGI91161[] = {{&S57378[128], 9}, {NULL}};
const Segment SGI91162[] = {{&S70425[93], 5}, {NULL}};
const Segment SGI91163[] = {{&S57378[101], 6}, {NULL}};
const Segment SGI91164[] = {{&S70426[44], 8}, {NULL}};
const Segment SGI91165[] = {{&S70426[44], 23}, {NULL}};
const Segment SGI91166[] = {{&S70426[30], 6}, {NULL}};
const Segment SGI91167[] = {{&S70426[115], 3}, {NULL}};
const Segment SGI91168[] = {{&S70426[81], 5}, {NULL}};
const Segment SGI91169[] = {{&S57380[34], 41}, {NULL}};
const Segment SGI91170[] = {{&S57380[27], 6}, {NULL}};
const Segment SGI91171[] = {{&S57380[128], 9}, {NULL}};
const Segment SGI91172[] = {{&S57380[101], 6}, {NULL}};
const Segment SGI91173[] = {{&S70427[44], 8}, {NULL}};
const Segment SGI91174[] = {{&S70427[44], 23}, {NULL}};
const Segment SGI91175[] = {{&S70427[30], 6}, {NULL}};
const Segment SGI91176[] = {{&S70427[115], 3}, {NULL}};
const Segment SGI91177[] = {{&S70427[81], 5}, {NULL}};
const Segment SGI91178[] = {{&S70428[44], 20}, {NULL}};
const Segment SGI91179[] = {{&S70428[44], 36}, {NULL}};
const Segment SGI91180[] = {{&S70428[30], 6}, {NULL}};
const Segment SGI91181[] = {{&S70428[128], 3}, {NULL}};
const Segment SGI91182[] = {{&S70428[94], 5}, {NULL}};
const Segment SGI91183[] = {{&S57392[44], 21}, {NULL}};
const Segment SGI91184[] = {{&S57392[44], 33}, {NULL}};
const Segment SGI91185[] = {{&S57392[30], 6}, {NULL}};
const Segment SGI91186[] = {{&S57392[31], 3}, {NULL}};
const Segment SGI91187[] = {{&S57392[91], 6}, {NULL}};
const Segment SGI91188[] = {{&S57394[44], 19}, {NULL}};
const Segment SGI91189[] = {{&S57394[44], 33}, {NULL}};
const Segment SGI91190[] = {{&S57394[30], 6}, {NULL}};
const Segment SGI91191[] = {{&S57394[31], 3}, {NULL}};
const Segment SGI91192[] = {{&S57394[91], 6}, {NULL}};
const Segment SGI91193[] = {{&S57435[34], 20}, {NULL}};
const Segment SGI91194[] = {{&S57435[27], 6}, {NULL}};
const Segment SGI91195[] = {{&S57435[104], 9}, {NULL}};
const Segment SGI91196[] = {{&S57435[77], 6}, {NULL}};
const Segment SGI91197[] = {{&S57438[34], 6}, {NULL}};
const Segment SGI91198[] = {{&S57438[27], 6}, {NULL}};
const Segment SGI91199[] = {{&S57438[93], 9}, {NULL}};
const Segment SGI91200[] = {{&S57438[66], 6}, {NULL}};
const Segment SGI91201[] = {{&S70429[44], 8}, {NULL}};
const Segment SGI91202[] = {{&S70429[44], 24}, {NULL}};
const Segment SGI91203[] = {{&S70429[30], 6}, {NULL}};
const Segment SGI91204[] = {{&S70429[116], 3}, {NULL}};
const Segment SGI91205[] = {{&S70429[82], 5}, {NULL}};
const Segment SGI91206[] = {{&S70430[38], 8}, {NULL}};
const Segment SGI91207[] = {{&S70430[31], 6}, {NULL}};
const Segment SGI91208[] = {{&S70431[44], 20}, {NULL}};
const Segment SGI91209[] = {{&S70431[44], 36}, {NULL}};
const Segment SGI91210[] = {{&S70431[30], 6}, {NULL}};
const Segment SGI91211[] = {{&S70431[128], 3}, {NULL}};
const Segment SGI91212[] = {{&S70431[94], 5}, {NULL}};
const Segment SGI91213[] = {{&S70432[44], 8}, {NULL}};
const Segment SGI91214[] = {{&S70432[44], 24}, {NULL}};
const Segment SGI91215[] = {{&S70432[30], 6}, {NULL}};
const Segment SGI91216[] = {{&S70432[116], 3}, {NULL}};
const Segment SGI91217[] = {{&S70432[82], 5}, {NULL}};
const Segment SGI91218[] = {{&S78133[44], 11}, {NULL}};
const Segment SGI91219[] = {{&S78133[44], 23}, {NULL}};
const Segment SGI91220[] = {{&S78133[30], 6}, {NULL}};
const Segment SGI91221[] = {{&S78133[115], 3}, {NULL}};
const Segment SGI91222[] = {{&S78133[81], 5}, {NULL}};
const Segment SGI91223[] = {{&S57473[34], 12}, {NULL}};
const Segment SGI91224[] = {{&S57473[27], 6}, {NULL}};
const Segment SGI91225[] = {{&S57473[99], 9}, {NULL}};
const Segment SGI91226[] = {{&S57473[72], 6}, {NULL}};
const Segment SGI91227[] = {{&S57474[34], 12}, {NULL}};
const Segment SGI91228[] = {{&S57474[27], 6}, {NULL}};
const Segment SGI91229[] = {{&S57474[99], 9}, {NULL}};
const Segment SGI91230[] = {{&S57474[72], 6}, {NULL}};
const Segment SGI91231[] = {{&S70478[44], 6}, {NULL}};
const Segment SGI91232[] = {{&S70478[44], 17}, {NULL}};
const Segment SGI91233[] = {{&S70478[30], 6}, {NULL}};
const Segment SGI91234[] = {{&S70478[78], 4}, {NULL}};
const Segment SGI91235[] = {{&S70478[78], 3}, {NULL}};
const Segment SGI91236[] = {{&S70479[44], 6}, {NULL}};
const Segment SGI91237[] = {{&S70479[44], 17}, {NULL}};
const Segment SGI91238[] = {{&S70479[30], 6}, {NULL}};
const Segment SGI91239[] = {{&S70479[109], 3}, {NULL}};
const Segment SGI91240[] = {{&S70479[75], 5}, {NULL}};
const Segment SGI91241[] = {{&S70480[44], 6}, {NULL}};
const Segment SGI91242[] = {{&S70480[44], 17}, {NULL}};
const Segment SGI91243[] = {{&S70480[30], 6}, {NULL}};
const Segment SGI91244[] = {{&S70480[78], 6}, {NULL}};
const Segment SGI91245[] = {{&S70481[44], 17}, {NULL}};
const Segment SGI91246[] = {{&S70481[30], 6}, {NULL}};
const Segment SGI91247[] = {{&S57514[34], 7}, {NULL}};
const Segment SGI91248[] = {{&S57514[27], 6}, {NULL}};
const Segment SGI91249[] = {{&S57514[96], 9}, {NULL}};
const Segment SGI91250[] = {{&S57514[69], 6}, {NULL}};
const Segment SGI91251[] = {{&S70503[44], 9}, {NULL}};
const Segment SGI91252[] = {{&S70503[44], 20}, {NULL}};
const Segment SGI91253[] = {{&S70503[30], 6}, {NULL}};
const Segment SGI91254[] = {{&S70503[127], 3}, {NULL}};
const Segment SGI91255[] = {{&S70503[93], 5}, {NULL}};
const Segment SGI91256[] = {{&S70503[35], 1}, {NULL}};
const Segment SGI91257[] = {{&S70503[66], 11}, {NULL}};
const Segment SGI91258[] = {{&S70504[44], 9}, {NULL}};
const Segment SGI91259[] = {{&S70504[44], 21}, {NULL}};
const Segment SGI91260[] = {{&S70504[30], 6}, {NULL}};
const Segment SGI91261[] = {{&S70504[128], 3}, {NULL}};
const Segment SGI91262[] = {{&S70504[94], 5}, {NULL}};
const Segment SGI91263[] = {{&S70504[35], 1}, {NULL}};
const Segment SGI91264[] = {{&S70504[67], 11}, {NULL}};
const Segment SGI91265[] = {{&S70505[44], 9}, {NULL}};
const Segment SGI91266[] = {{&S70505[44], 20}, {NULL}};
const Segment SGI91267[] = {{&S70505[30], 6}, {NULL}};
const Segment SGI91268[] = {{&S70505[131], 3}, {NULL}};
const Segment SGI91269[] = {{&S70505[97], 5}, {NULL}};
const Segment SGI91270[] = {{&S70505[35], 1}, {NULL}};
const Segment SGI91271[] = {{&S70505[66], 11}, {NULL}};
const Segment SGI91272[] = {{&S70506[44], 12}, {NULL}};
const Segment SGI91273[] = {{&S70506[44], 23}, {NULL}};
const Segment SGI91274[] = {{&S70506[30], 6}, {NULL}};
const Segment SGI91275[] = {{&S70506[134], 3}, {NULL}};
const Segment SGI91276[] = {{&S70506[100], 5}, {NULL}};
const Segment SGI91277[] = {{&S70506[35], 1}, {NULL}};
const Segment SGI91278[] = {{&S70506[69], 11}, {NULL}};
const Segment SGI91279[] = {{&S57526[44], 13}, {NULL}};
const Segment SGI91280[] = {{&S57526[44], 25}, {NULL}};
const Segment SGI91281[] = {{&S57526[30], 6}, {NULL}};
const Segment SGI91282[] = {{&S57526[31], 3}, {NULL}};
const Segment SGI91283[] = {{&S57526[83], 6}, {NULL}};
const Segment SGI91284[] = {{&S57527[34], 12}, {NULL}};
const Segment SGI91285[] = {{&S57527[27], 6}, {NULL}};
const Segment SGI91286[] = {{&S57527[99], 9}, {NULL}};
const Segment SGI91287[] = {{&S57527[72], 6}, {NULL}};
const Segment SGI91288[] = {{&S57534[34], 9}, {NULL}};
const Segment SGI91289[] = {{&S57534[27], 6}, {NULL}};
const Segment SGI91290[] = {{&S57534[96], 9}, {NULL}};
const Segment SGI91291[] = {{&S57534[69], 6}, {NULL}};
const Segment SGI91292[] = {{&S57535[34], 10}, {NULL}};
const Segment SGI91293[] = {{&S57535[27], 6}, {NULL}};
const Segment SGI91294[] = {{&S57535[97], 9}, {NULL}};
const Segment SGI91295[] = {{&S57535[70], 6}, {NULL}};
const Segment SGI91296[] = {{&S57536[34], 9}, {NULL}};
const Segment SGI91297[] = {{&S57536[27], 6}, {NULL}};
const Segment SGI91298[] = {{&S57536[96], 9}, {NULL}};
const Segment SGI91299[] = {{&S57536[69], 6}, {NULL}};
const Segment SGI91300[] = {{&S57537[34], 10}, {NULL}};
const Segment SGI91301[] = {{&S57537[27], 6}, {NULL}};
const Segment SGI91302[] = {{&S57537[97], 9}, {NULL}};
const Segment SGI91303[] = {{&S57537[70], 6}, {NULL}};
const Segment SGI91304[] = {{&S57538[34], 10}, {NULL}};
const Segment SGI91305[] = {{&S57538[27], 6}, {NULL}};
const Segment SGI91306[] = {{&S57538[97], 9}, {NULL}};
const Segment SGI91307[] = {{&S57538[70], 6}, {NULL}};
const Segment SGI91308[] = {{&S78238[45], 10}, {NULL}};
const Segment SGI91309[] = {{&S78238[30], 6}, {NULL}};
const Segment SGI91310[] = {{&S78238[120], 3}, {NULL}};
const Segment SGI91311[] = {{&S78238[86], 5}, {NULL}};
const Segment SGI91312[] = {{&S78238[144], 13}, {NULL}};
const Segment SGI91313[] = {{&S78240[44], 8}, {NULL}};
const Segment SGI91314[] = {{&S78240[44], 20}, {NULL}};
const Segment SGI91315[] = {{&S78240[30], 6}, {NULL}};
const Segment SGI91316[] = {{&S78240[112], 3}, {NULL}};
const Segment SGI91317[] = {{&S57566[44], 8}, {NULL}};
const Segment SGI91318[] = {{&S57566[44], 20}, {NULL}};
const Segment SGI91319[] = {{&S78240[78], 5}, {NULL}};
const Segment SGI91320[] = {{&S57566[30], 6}, {NULL}};
const Segment SGI91321[] = {{&S57566[31], 3}, {NULL}};
const Segment SGI91322[] = {{&S57566[78], 6}, {NULL}};
const Segment SGI91323[] = {{&S78247[44], 10}, {NULL}};
const Segment SGI91324[] = {{&S78247[44], 27}, {NULL}};
const Segment SGI91325[] = {{&S78247[30], 6}, {NULL}};
const Segment SGI91326[] = {{&S78247[119], 3}, {NULL}};
const Segment SGI91327[] = {{&S78247[85], 5}, {NULL}};
const Segment SGI91328[] = {{&S78247[143], 11}, {NULL}};
const Segment SGI91329[] = {{&S57576[34], 9}, {NULL}};
const Segment SGI91330[] = {{&S57576[27], 6}, {NULL}};
const Segment SGI91331[] = {{&S57576[96], 9}, {NULL}};
const Segment SGI91332[] = {{&S57576[69], 6}, {NULL}};
const Segment SGI91333[] = {{&S57577[34], 10}, {NULL}};
const Segment SGI91334[] = {{&S57577[27], 6}, {NULL}};
const Segment SGI91335[] = {{&S57577[97], 9}, {NULL}};
const Segment SGI91336[] = {{&S57577[70], 6}, {NULL}};
const Segment SGI91337[] = {{&S57578[34], 10}, {NULL}};
const Segment SGI91338[] = {{&S57578[27], 6}, {NULL}};
const Segment SGI91339[] = {{&S57578[97], 9}, {NULL}};
const Segment SGI91340[] = {{&S57578[70], 6}, {NULL}};
const Segment SGI91341[] = {{&S65016[75], 6}, {NULL}};
const Segment SGI91342[] = {{&S57579[34], 9}, {NULL}};
const Segment SGI91343[] = {{&S57579[27], 6}, {NULL}};
const Segment SGI91344[] = {{&S65016[0], 10}, {NULL}};
const Segment SGI91345[] = {{&S57579[96], 9}, {NULL}};
const Segment SGI91346[] = {{&S57579[69], 6}, {NULL}};
const Segment SGI91347[] = {{&S57580[34], 10}, {NULL}};
const Segment SGI91348[] = {{&S57580[27], 6}, {NULL}};
const Segment SGI91349[] = {{&S57580[97], 9}, {NULL}};
const Segment SGI91350[] = {{&S57580[70], 6}, {NULL}};
const Segment SGI91351[] = {{&S57595[34], 8}, {NULL}};
const Segment SGI91352[] = {{&S57595[27], 6}, {NULL}};
const Segment SGI91353[] = {{&S57595[96], 9}, {NULL}};
const Segment SGI91354[] = {{&S57595[69], 6}, {NULL}};
const Segment SGI91355[] = {{&S57596[34], 8}, {NULL}};
const Segment SGI91356[] = {{&S57596[27], 6}, {NULL}};
const Segment SGI91357[] = {{&S57596[96], 9}, {NULL}};
const Segment SGI91358[] = {{&S57596[69], 6}, {NULL}};
const Segment SGI91359[] = {{&S57622[34], 41}, {NULL}};
const Segment SGI91360[] = {{&S57622[27], 6}, {NULL}};
const Segment SGI91361[] = {{&S57622[128], 9}, {NULL}};
const Segment SGI91362[] = {{&S57622[101], 6}, {NULL}};
const Segment SGI91363[] = {{&S57623[34], 35}, {NULL}};
const Segment SGI91364[] = {{&S57623[27], 6}, {NULL}};
const Segment SGI91365[] = {{&S57623[122], 9}, {NULL}};
const Segment SGI91366[] = {{&S57623[95], 6}, {NULL}};
const Segment SGI91367[] = {{&S57624[34], 41}, {NULL}};
const Segment SGI91368[] = {{&S57624[27], 6}, {NULL}};
const Segment SGI91369[] = {{&S57624[128], 9}, {NULL}};
const Segment SGI91370[] = {{&S57624[101], 6}, {NULL}};
const Segment SGI91371[] = {{&S78336[44], 10}, {NULL}};
const Segment SGI91372[] = {{&S78336[44], 27}, {NULL}};
const Segment SGI91373[] = {{&S78336[30], 6}, {NULL}};
const Segment SGI91374[] = {{&S78336[126], 3}, {NULL}};
const Segment SGI91375[] = {{&S78336[92], 5}, {NULL}};
const Segment SGI91376[] = {{&S78336[71], 8}, {NULL}};
const Segment SGI91377[] = {{&S78339[44], 11}, {NULL}};
const Segment SGI91378[] = {{&S78339[44], 28}, {NULL}};
const Segment SGI91379[] = {{&S78339[30], 6}, {NULL}};
const Segment SGI91380[] = {{&S78339[120], 3}, {NULL}};
const Segment SGI91381[] = {{&S78339[86], 5}, {NULL}};
const Segment SGI91382[] = {{&S78339[144], 7}, {NULL}};
const Segment SGI91383[] = {{&S78385[45], 10}, {NULL}};
const Segment SGI91384[] = {{&S78385[30], 6}, {NULL}};
const Segment SGI91385[] = {{&S78385[120], 3}, {NULL}};
const Segment SGI91386[] = {{&S78385[86], 5}, {NULL}};
const Segment SGI91387[] = {{&S78385[144], 13}, {NULL}};
const Segment SGI91388[] = {{&S78390[44], 18}, {NULL}};
const Segment SGI91389[] = {{&S78390[44], 35}, {NULL}};
const Segment SGI91390[] = {{&S78390[30], 6}, {NULL}};
const Segment SGI91391[] = {{&S78390[127], 3}, {NULL}};
const Segment SGI91392[] = {{&S78390[93], 5}, {NULL}};
const Segment SGI91393[] = {{&S78391[44], 18}, {NULL}};
const Segment SGI91394[] = {{&S78391[44], 35}, {NULL}};
const Segment SGI91395[] = {{&S78391[30], 6}, {NULL}};
const Segment SGI91396[] = {{&S78391[96], 6}, {NULL}};
const Segment SGI91397[] = {{&S78392[38], 18}, {NULL}};
const Segment SGI91398[] = {{&S78392[31], 6}, {NULL}};
const Segment SGI91399[] = {{&S78400[61], 10}, {NULL}};
const Segment SGI91400[] = {{&S78400[61], 27}, {NULL}};
const Segment SGI91401[] = {{&S78400[47], 6}, {NULL}};
const Segment SGI91402[] = {{&S78400[136], 3}, {NULL}};
const Segment SGI91403[] = {{&S78400[102], 5}, {NULL}};
const Segment SGI91404[] = {{&S78400[0], 14}, {NULL}};
const Segment SGI91405[] = {{&S78417[44], 10}, {NULL}};
const Segment SGI91406[] = {{&S78417[44], 27}, {NULL}};
const Segment SGI91407[] = {{&S78417[30], 6}, {NULL}};
const Segment SGI91408[] = {{&S78417[85], 6}, {NULL}};
const Segment SGI91409[] = {{&S78417[112], 13}, {NULL}};
const Segment SGI91410[] = {{&S57758[34], 11}, {NULL}};
const Segment SGI91411[] = {{&S57758[27], 6}, {NULL}};
const Segment SGI91412[] = {{&S57758[98], 9}, {NULL}};
const Segment SGI91413[] = {{&S57758[71], 6}, {NULL}};
const Segment SGI91414[] = {{&S65223[44], 10}, {NULL}};
const Segment SGI91415[] = {{&S65223[44], 27}, {NULL}};
const Segment SGI91416[] = {{&S65223[30], 6}, {NULL}};
const Segment SGI91417[] = {{&S65223[31], 3}, {NULL}};
const Segment SGI91418[] = {{&S65223[85], 6}, {NULL}};
const Segment SGI91419[] = {{&S65223[0], 13}, {NULL}};
const Segment SGI91420[] = {{&S57761[34], 10}, {NULL}};
const Segment SGI91421[] = {{&S57761[27], 6}, {NULL}};
const Segment SGI91422[] = {{&S57761[97], 9}, {NULL}};
const Segment SGI91423[] = {{&S57761[70], 6}, {NULL}};
const Segment SGI91424[] = {{&S57762[34], 14}, {NULL}};
const Segment SGI91425[] = {{&S57762[27], 6}, {NULL}};
const Segment SGI91426[] = {{&S57762[101], 9}, {NULL}};
const Segment SGI91427[] = {{&S57762[74], 6}, {NULL}};
const Segment SGI91428[] = {{&S57763[34], 10}, {NULL}};
const Segment SGI91429[] = {{&S57763[27], 6}, {NULL}};
const Segment SGI91430[] = {{&S57763[97], 9}, {NULL}};
const Segment SGI91431[] = {{&S57763[70], 6}, {NULL}};
const Segment SGI91432[] = {{&S57764[34], 14}, {NULL}};
const Segment SGI91433[] = {{&S57764[27], 6}, {NULL}};
const Segment SGI91434[] = {{&S57764[101], 9}, {NULL}};
const Segment SGI91435[] = {{&S57764[74], 6}, {NULL}};
const Segment SGI91436[] = {{&S57773[34], 35}, {NULL}};
const Segment SGI91437[] = {{&S57773[27], 6}, {NULL}};
const Segment SGI91438[] = {{&S57773[122], 9}, {NULL}};
const Segment SGI91439[] = {{&S57773[95], 6}, {NULL}};
const Segment SGI91440[] = {{&S57774[34], 41}, {NULL}};
const Segment SGI91441[] = {{&S57774[27], 6}, {NULL}};
const Segment SGI91442[] = {{&S57774[128], 9}, {NULL}};
const Segment SGI91443[] = {{&S57774[101], 6}, {NULL}};
const Segment SGI91444[] = {{&S57776[34], 41}, {NULL}};
const Segment SGI91445[] = {{&S57776[27], 6}, {NULL}};
const Segment SGI91446[] = {{&S57776[128], 9}, {NULL}};
const Segment SGI91447[] = {{&S57776[101], 6}, {NULL}};
const Segment SGI91448[] = {{&S57787[34], 12}, {NULL}};
const Segment SGI91449[] = {{&S57787[27], 6}, {NULL}};
const Segment SGI91450[] = {{&S57787[99], 9}, {NULL}};
const Segment SGI91451[] = {{&S57787[72], 6}, {NULL}};
const Segment SGI91452[] = {{&S70776[42], 10}, {NULL}};
const Segment SGI91453[] = {{&S70776[42], 21}, {NULL}};
const Segment SGI91454[] = {{&S70776[30], 4}, {NULL}};
const Segment SGI91455[] = {{&S70776[112], 3}, {NULL}};
const Segment SGI91456[] = {{&S70776[77], 6}, {NULL}};
const Segment SGI91457[] = {{&S70777[42], 13}, {NULL}};
const Segment SGI91458[] = {{&S70777[42], 24}, {NULL}};
const Segment SGI91459[] = {{&S70777[30], 4}, {NULL}};
const Segment SGI91460[] = {{&S70777[115], 3}, {NULL}};
const Segment SGI91461[] = {{&S70777[80], 6}, {NULL}};
const Segment SGI91462[] = {{&S70778[36], 10}, {NULL}};
const Segment SGI91463[] = {{&S70778[31], 4}, {NULL}};
const Segment SGI91464[] = {{&S78496[44], 28}, {NULL}};
const Segment SGI91465[] = {{&S78496[44], 45}, {NULL}};
const Segment SGI91466[] = {{&S78496[30], 6}, {NULL}};
const Segment SGI91467[] = {{&S78496[144], 3}, {NULL}};
const Segment SGI91468[] = {{&S78496[110], 5}, {NULL}};
const Segment SGI91469[] = {{&S78496[89], 8}, {NULL}};
const Segment SGI91470[] = {{&S70822[44], 9}, {NULL}};
const Segment SGI91471[] = {{&S70822[44], 30}, {NULL}};
const Segment SGI91472[] = {{&S70822[30], 6}, {NULL}};
const Segment SGI91473[] = {{&S70822[122], 3}, {NULL}};
const Segment SGI91474[] = {{&S70822[88], 5}, {NULL}};
const Segment SGI91475[] = {{&S70823[44], 76}, {NULL}};
const Segment SGI91476[] = {{&S70823[44], 97}, {NULL}};
const Segment SGI91477[] = {{&S70823[30], 6}, {NULL}};
const Segment SGI91478[] = {{&S70823[188], 3}, {NULL}};
const Segment SGI91479[] = {{&S70823[114], 5}, {NULL}};
const Segment SGI91480[] = {{&S78540[45], 9}, {NULL}};
const Segment SGI91481[] = {{&S78540[30], 6}, {NULL}};
const Segment SGI91482[] = {{&S78540[114], 3}, {NULL}};
const Segment SGI91483[] = {{&S78540[80], 5}, {NULL}};
const Segment SGI91484[] = {{&S78540[138], 13}, {NULL}};
const Segment SGI91485[] = {{&S70852[44], 15}, {NULL}};
const Segment SGI91486[] = {{&S70852[44], 26}, {NULL}};
const Segment SGI91487[] = {{&S70852[30], 6}, {NULL}};
const Segment SGI91488[] = {{&S70852[118], 3}, {NULL}};
const Segment SGI91489[] = {{&S70852[84], 5}, {NULL}};
const Segment SGI91490[] = {{&S57889[34], 9}, {NULL}};
const Segment SGI91491[] = {{&S57889[27], 6}, {NULL}};
const Segment SGI91492[] = {{&S57889[96], 9}, {NULL}};
const Segment SGI91493[] = {{&S57889[69], 6}, {NULL}};
const Segment SGI91494[] = {{&S57933[34], 11}, {NULL}};
const Segment SGI91495[] = {{&S57933[27], 6}, {NULL}};
const Segment SGI91496[] = {{&S57933[98], 9}, {NULL}};
const Segment SGI91497[] = {{&S57933[71], 6}, {NULL}};
const Segment SGI91498[] = {{&S57936[44], 6}, {NULL}};
const Segment SGI91499[] = {{&S57936[44], 18}, {NULL}};
const Segment SGI91500[] = {{&S57936[30], 6}, {NULL}};
const Segment SGI91501[] = {{&S57936[31], 3}, {NULL}};
const Segment SGI91502[] = {{&S57936[76], 6}, {NULL}};
const Segment SGI91503[] = {{&S57938[34], 9}, {NULL}};
const Segment SGI91504[] = {{&S57938[27], 6}, {NULL}};
const Segment SGI91505[] = {{&S57938[96], 9}, {NULL}};
const Segment SGI91506[] = {{&S57938[69], 6}, {NULL}};
const Segment SGI91507[] = {{&S57939[34], 10}, {NULL}};
const Segment SGI91508[] = {{&S57939[27], 6}, {NULL}};
const Segment SGI91509[] = {{&S57939[97], 9}, {NULL}};
const Segment SGI91510[] = {{&S57939[70], 6}, {NULL}};
const Segment SGI91511[] = {{&S57943[34], 10}, {NULL}};
const Segment SGI91512[] = {{&S57943[27], 6}, {NULL}};
const Segment SGI91513[] = {{&S57943[97], 9}, {NULL}};
const Segment SGI91514[] = {{&S57943[70], 6}, {NULL}};
const Segment SGI91515[] = {{&S57966[34], 19}, {NULL}};
const Segment SGI91516[] = {{&S57966[27], 6}, {NULL}};
const Segment SGI91517[] = {{&S57966[106], 9}, {NULL}};
const Segment SGI91518[] = {{&S57966[79], 6}, {NULL}};
const Segment SGI91519[] = {{&S57967[34], 18}, {NULL}};
const Segment SGI91520[] = {{&S57967[27], 6}, {NULL}};
const Segment SGI91521[] = {{&S57967[105], 9}, {NULL}};
const Segment SGI91522[] = {{&S57967[78], 6}, {NULL}};
const Segment SGI91523[] = {{&S57968[34], 18}, {NULL}};
const Segment SGI91524[] = {{&S57968[27], 6}, {NULL}};
const Segment SGI91525[] = {{&S57968[105], 9}, {NULL}};
const Segment SGI91526[] = {{&S57968[78], 6}, {NULL}};
const Segment SGI91527[] = {{&S57969[34], 11}, {NULL}};
const Segment SGI91528[] = {{&S57969[27], 6}, {NULL}};
const Segment SGI91529[] = {{&S57969[98], 9}, {NULL}};
const Segment SGI91530[] = {{&S57969[71], 6}, {NULL}};
const Segment SGI91531[] = {{&S57975[67], 6}, {NULL}};
const Segment SGI91532[] = {{&S57975[68], 3}, {NULL}};
const Segment SGI91533[] = {{&S57975[87], 6}, {NULL}};
const Segment SGI91534[] = {{&S57976[44], 14}, {NULL}};
const Segment SGI91535[] = {{&S57976[44], 26}, {NULL}};
const Segment SGI91536[] = {{&S57976[30], 6}, {NULL}};
const Segment SGI91537[] = {{&S57976[31], 3}, {NULL}};
const Segment SGI91538[] = {{&S57976[84], 6}, {NULL}};
const Segment SGI91539[] = {{&S57980[44], 14}, {NULL}};
const Segment SGI91540[] = {{&S57980[44], 26}, {NULL}};
const Segment SGI91541[] = {{&S57980[30], 6}, {NULL}};
const Segment SGI91542[] = {{&S57980[87], 6}, {NULL}};
const Segment SGI91543[] = {{&S57983[44], 26}, {NULL}};
const Segment SGI91544[] = {{&S57983[30], 6}, {NULL}};
const Segment SGI91545[] = {{&S78680[59], 13}, {NULL}};
const Segment SGI91546[] = {{&S78680[59], 32}, {NULL}};
const Segment SGI91547[] = {{&S78680[45], 6}, {NULL}};
const Segment SGI91548[] = {{&S78680[139], 3}, {NULL}};
const Segment SGI91549[] = {{&S78680[105], 5}, {NULL}};
const Segment SGI91550[] = {{&S78680[0], 14}, {NULL}};
const Segment SGI91551[] = {{&S57990[44], 10}, {NULL}};
const Segment SGI91552[] = {{&S57990[44], 27}, {NULL}};
const Segment SGI91553[] = {{&S57990[30], 6}, {NULL}};
const Segment SGI91554[] = {{&S57990[31], 3}, {NULL}};
const Segment SGI91555[] = {{&S57990[85], 6}, {NULL}};
const Segment SGI91556[] = {{&S78687[59], 10}, {NULL}};
const Segment SGI91557[] = {{&S78687[59], 27}, {NULL}};
const Segment SGI91558[] = {{&S78687[45], 6}, {NULL}};
const Segment SGI91559[] = {{&S78687[134], 3}, {NULL}};
const Segment SGI91560[] = {{&S78687[100], 5}, {NULL}};
const Segment SGI91561[] = {{&S78687[0], 14}, {NULL}};
const Segment SGI91562[] = {{&S57993[44], 10}, {NULL}};
const Segment SGI91563[] = {{&S57993[44], 22}, {NULL}};
const Segment SGI91564[] = {{&S57993[30], 6}, {NULL}};
const Segment SGI91565[] = {{&S57993[31], 3}, {NULL}};
const Segment SGI91566[] = {{&S57993[80], 6}, {NULL}};
const Segment SGI91567[] = {{&S57995[44], 10}, {NULL}};
const Segment SGI91568[] = {{&S57995[44], 27}, {NULL}};
const Segment SGI91569[] = {{&S57995[30], 6}, {NULL}};
const Segment SGI91570[] = {{&S57995[31], 3}, {NULL}};
const Segment SGI91571[] = {{&S57995[85], 5}, {NULL}};
const Segment SGI91572[] = {{&S57996[38], 10}, {NULL}};
const Segment SGI91573[] = {{&S57996[31], 6}, {NULL}};
const Segment SGI91574[] = {{&S57998[44], 10}, {NULL}};
const Segment SGI91575[] = {{&S57998[44], 19}, {NULL}};
const Segment SGI91576[] = {{&S57998[30], 6}, {NULL}};
const Segment SGI91577[] = {{&S57998[31], 3}, {NULL}};
const Segment SGI91578[] = {{&S57998[77], 6}, {NULL}};
const Segment SGI91579[] = {{&S57999[44], 10}, {NULL}};
const Segment SGI91580[] = {{&S57999[44], 24}, {NULL}};
const Segment SGI91581[] = {{&S57999[30], 6}, {NULL}};
const Segment SGI91582[] = {{&S57999[31], 3}, {NULL}};
const Segment SGI91583[] = {{&S57999[82], 6}, {NULL}};
const Segment SGI91584[] = {{&S58001[44], 10}, {NULL}};
const Segment SGI91585[] = {{&S58001[44], 47}, {NULL}};
const Segment SGI91586[] = {{&S58001[30], 6}, {NULL}};
const Segment SGI91587[] = {{&S58001[31], 3}, {NULL}};
const Segment SGI91588[] = {{&S58001[105], 6}, {NULL}};
const Segment SGI91589[] = {{&S58009[44], 10}, {NULL}};
const Segment SGI91590[] = {{&S58009[44], 19}, {NULL}};
const Segment SGI91591[] = {{&S58009[30], 6}, {NULL}};
const Segment SGI91592[] = {{&S58009[31], 3}, {NULL}};
const Segment SGI91593[] = {{&S58009[77], 6}, {NULL}};
const Segment SGI91594[] = {{&S58014[44], 10}, {NULL}};
const Segment SGI91595[] = {{&S58014[44], 40}, {NULL}};
const Segment SGI91596[] = {{&S58014[30], 6}, {NULL}};
const Segment SGI91597[] = {{&S58014[31], 3}, {NULL}};
const Segment SGI91598[] = {{&S58014[98], 6}, {NULL}};
const Segment SGI91599[] = {{&S58015[44], 10}, {NULL}};
const Segment SGI91600[] = {{&S58015[44], 22}, {NULL}};
const Segment SGI91601[] = {{&S58015[30], 6}, {NULL}};
const Segment SGI91602[] = {{&S58015[31], 3}, {NULL}};
const Segment SGI91603[] = {{&S58015[80], 6}, {NULL}};
const Segment SGI91604[] = {{&S65519[6], 4}, {NULL}};
const Segment SGI91605[] = {{&S65519[85], 5}, {NULL}};
const Segment SGI91606[] = {{&S65519[0], 27}, {NULL}};
const Segment SGI91607[] = {{&S58020[44], 10}, {NULL}};
const Segment SGI91608[] = {{&S58020[44], 32}, {NULL}};
const Segment SGI91609[] = {{&S58020[30], 6}, {NULL}};
const Segment SGI91610[] = {{&S58020[31], 3}, {NULL}};
const Segment SGI91611[] = {{&S58020[90], 6}, {NULL}};
const Segment SGI91612[] = {{&S71039[44], 9}, {NULL}};
const Segment SGI91613[] = {{&S71039[44], 21}, {NULL}};
const Segment SGI91614[] = {{&S71039[30], 6}, {NULL}};
const Segment SGI91615[] = {{&S71039[132], 3}, {NULL}};
const Segment SGI91616[] = {{&S71039[98], 5}, {NULL}};
const Segment SGI91617[] = {{&S71039[35], 1}, {NULL}};
const Segment SGI91618[] = {{&S71039[67], 11}, {NULL}};
const Segment SGI91619[] = {{&S68193[54], 10}, {NULL}};
const Segment SGI91620[] = {{&S68193[61], 3}, {NULL}};
const Segment SGI91621[] = {{&S68193[97], 9}, {NULL}};
const Segment SGI91622[] = {{&S71069[42], 20}, {NULL}};
const Segment SGI91623[] = {{&S71069[42], 32}, {NULL}};
const Segment SGI91624[] = {{&S71069[30], 4}, {NULL}};
const Segment SGI91625[] = {{&S71069[123], 3}, {NULL}};
const Segment SGI91626[] = {{&S71069[88], 6}, {NULL}};
const Segment SGI91627[] = {{&S71070[42], 20}, {NULL}};
const Segment SGI91628[] = {{&S71070[42], 32}, {NULL}};
const Segment SGI91629[] = {{&S71070[30], 4}, {NULL}};
const Segment SGI91630[] = {{&S71070[133], 3}, {NULL}};
const Segment SGI91631[] = {{&S71070[98], 6}, {NULL}};
const Segment SGI91632[] = {{&S71070[153], 8}, {NULL}};
const Segment SGI91633[] = {{&S65584[44], 8}, {NULL}};
const Segment SGI91634[] = {{&S65584[44], 19}, {NULL}};
const Segment SGI91635[] = {{&S65584[30], 6}, {NULL}};
const Segment SGI91636[] = {{&S65584[31], 3}, {NULL}};
const Segment SGI91637[] = {{&S65584[77], 6}, {NULL}};
const Segment SGI91638[] = {{&S65584[0], 13}, {NULL}};
const Segment SGI91639[] = {{&S78815[45], 11}, {NULL}};
const Segment SGI91640[] = {{&S78815[30], 6}, {NULL}};
const Segment SGI91641[] = {{&S78815[121], 3}, {NULL}};
const Segment SGI91642[] = {{&S78815[87], 5}, {NULL}};
const Segment SGI91643[] = {{&S78815[145], 13}, {NULL}};
const Segment SGI91644[] = {{&S78816[45], 10}, {NULL}};
const Segment SGI91645[] = {{&S78816[30], 6}, {NULL}};
const Segment SGI91646[] = {{&S78816[120], 3}, {NULL}};
const Segment SGI91647[] = {{&S78816[86], 5}, {NULL}};
const Segment SGI91648[] = {{&S78816[144], 13}, {NULL}};
const Segment SGI91649[] = {{&S78826[45], 10}, {NULL}};
const Segment SGI91650[] = {{&S78826[30], 6}, {NULL}};
const Segment SGI91651[] = {{&S78826[120], 3}, {NULL}};
const Segment SGI91652[] = {{&S78826[86], 5}, {NULL}};
const Segment SGI91653[] = {{&S78826[144], 13}, {NULL}};
const Segment SGI91654[] = {{&S68265[42], 9}, {NULL}};
const Segment SGI91655[] = {{&S68265[42], 30}, {NULL}};
const Segment SGI91656[] = {{&S68265[30], 4}, {NULL}};
const Segment SGI91657[] = {{&S68265[31], 3}, {NULL}};
const Segment SGI91658[] = {{&S68265[86], 5}, {NULL}};
const Segment SGI91659[] = {{&S71141[44], 9}, {NULL}};
const Segment SGI91660[] = {{&S71141[44], 19}, {NULL}};
const Segment SGI91661[] = {{&S71141[30], 6}, {NULL}};
const Segment SGI91662[] = {{&S71141[111], 3}, {NULL}};
const Segment SGI91663[] = {{&S71141[77], 5}, {NULL}};
const Segment SGI91664[] = {{&S58136[34], 6}, {NULL}};
const Segment SGI91665[] = {{&S58136[27], 6}, {NULL}};
const Segment SGI91666[] = {{&S58136[92], 9}, {NULL}};
const Segment SGI91667[] = {{&S58136[65], 6}, {NULL}};
const Segment SGI91668[] = {{&S58143[34], 15}, {NULL}};
const Segment SGI91669[] = {{&S58143[27], 6}, {NULL}};
const Segment SGI91670[] = {{&S58143[102], 9}, {NULL}};
const Segment SGI91671[] = {{&S58143[75], 6}, {NULL}};
const Segment SGI91672[] = {{&S58144[34], 14}, {NULL}};
const Segment SGI91673[] = {{&S58144[27], 6}, {NULL}};
const Segment SGI91674[] = {{&S58144[101], 9}, {NULL}};
const Segment SGI91675[] = {{&S58144[74], 6}, {NULL}};
const Segment SGI91676[] = {{&S58145[34], 18}, {NULL}};
const Segment SGI91677[] = {{&S58145[27], 6}, {NULL}};
const Segment SGI91678[] = {{&S58145[105], 9}, {NULL}};
const Segment SGI91679[] = {{&S58145[78], 6}, {NULL}};
const Segment SGI91680[] = {{&S58146[34], 18}, {NULL}};
const Segment SGI91681[] = {{&S58146[27], 6}, {NULL}};
const Segment SGI91682[] = {{&S58146[105], 9}, {NULL}};
const Segment SGI91683[] = {{&S58146[78], 6}, {NULL}};
const Segment SGI91684[] = {{&S78881[45], 10}, {NULL}};
const Segment SGI91685[] = {{&S78881[30], 6}, {NULL}};
const Segment SGI91686[] = {{&S78881[120], 3}, {NULL}};
const Segment SGI91687[] = {{&S78881[86], 5}, {NULL}};
const Segment SGI91688[] = {{&S78881[144], 13}, {NULL}};
const Segment SGI91689[] = {{&S78882[59], 10}, {NULL}};
const Segment SGI91690[] = {{&S78882[59], 27}, {NULL}};
const Segment SGI91691[] = {{&S78882[45], 6}, {NULL}};
const Segment SGI91692[] = {{&S78882[134], 3}, {NULL}};
const Segment SGI91693[] = {{&S78882[100], 5}, {NULL}};
const Segment SGI91694[] = {{&S78882[0], 14}, {NULL}};
const Segment SGI91695[] = {{&S78883[57], 10}, {NULL}};
const Segment SGI91696[] = {{&S78883[45], 6}, {NULL}};
const Segment SGI91697[] = {{&S78883[132], 3}, {NULL}};
const Segment SGI91698[] = {{&S78883[98], 5}, {NULL}};
const Segment SGI91699[] = {{&S78883[0], 14}, {NULL}};
const Segment SGI91700[] = {{&S58186[34], 7}, {NULL}};
const Segment SGI91701[] = {{&S58186[27], 6}, {NULL}};
const Segment SGI91702[] = {{&S58186[94], 9}, {NULL}};
const Segment SGI91703[] = {{&S58186[67], 6}, {NULL}};
const Segment SGI91704[] = {{&S78895[59], 11}, {NULL}};
const Segment SGI91705[] = {{&S78895[59], 28}, {NULL}};
const Segment SGI91706[] = {{&S78895[45], 6}, {NULL}};
const Segment SGI91707[] = {{&S78895[135], 3}, {NULL}};
const Segment SGI91708[] = {{&S78895[101], 5}, {NULL}};
const Segment SGI91709[] = {{&S78895[0], 14}, {NULL}};
const Segment SGI91710[] = {{&S58198[34], 11}, {NULL}};
const Segment SGI91711[] = {{&S58198[27], 6}, {NULL}};
const Segment SGI91712[] = {{&S58198[98], 9}, {NULL}};
const Segment SGI91713[] = {{&S58198[71], 6}, {NULL}};
const Segment SGI91714[] = {{&S78901[44], 10}, {NULL}};
const Segment SGI91715[] = {{&S58199[34], 7}, {NULL}};
const Segment SGI91716[] = {{&S78901[44], 27}, {NULL}};
const Segment SGI91717[] = {{&S78901[30], 6}, {NULL}};
const Segment SGI91718[] = {{&S58199[27], 6}, {NULL}};
const Segment SGI91719[] = {{&S78901[119], 3}, {NULL}};
const Segment SGI91720[] = {{&S58199[94], 9}, {NULL}};
const Segment SGI91721[] = {{&S78901[85], 5}, {NULL}};
const Segment SGI91722[] = {{&S58199[67], 6}, {NULL}};
const Segment SGI91723[] = {{&S78902[44], 11}, {NULL}};
const Segment SGI91724[] = {{&S78902[44], 28}, {NULL}};
const Segment SGI91725[] = {{&S78902[30], 6}, {NULL}};
const Segment SGI91726[] = {{&S78902[120], 3}, {NULL}};
const Segment SGI91727[] = {{&S78902[86], 5}, {NULL}};
const Segment SGI91728[] = {{&S58203[34], 10}, {NULL}};
const Segment SGI91729[] = {{&S58203[27], 6}, {NULL}};
const Segment SGI91730[] = {{&S58203[97], 9}, {NULL}};
const Segment SGI91731[] = {{&S58203[70], 6}, {NULL}};
const Segment SGI91732[] = {{&S58204[34], 5}, {NULL}};
const Segment SGI91733[] = {{&S58204[27], 6}, {NULL}};
const Segment SGI91734[] = {{&S58204[98], 9}, {NULL}};
const Segment SGI91735[] = {{&S58204[71], 6}, {NULL}};
const Segment SGI91736[] = {{&S58205[34], 8}, {NULL}};
const Segment SGI91737[] = {{&S58205[27], 6}, {NULL}};
const Segment SGI91738[] = {{&S58205[95], 9}, {NULL}};
const Segment SGI91739[] = {{&S58205[68], 6}, {NULL}};
const Segment SGI91740[] = {{&S58208[34], 11}, {NULL}};
const Segment SGI91741[] = {{&S58208[27], 6}, {NULL}};
const Segment SGI91742[] = {{&S58208[98], 9}, {NULL}};
const Segment SGI91743[] = {{&S58208[71], 6}, {NULL}};
const Segment SGI91744[] = {{&S58212[34], 13}, {NULL}};
const Segment SGI91745[] = {{&S58212[27], 6}, {NULL}};
const Segment SGI91746[] = {{&S58212[100], 9}, {NULL}};
const Segment SGI91747[] = {{&S58212[73], 6}, {NULL}};
const Segment SGI91748[] = {{&S58217[34], 12}, {NULL}};
const Segment SGI91749[] = {{&S58217[27], 6}, {NULL}};
const Segment SGI91750[] = {{&S58217[99], 9}, {NULL}};
const Segment SGI91751[] = {{&S58217[72], 6}, {NULL}};
const Segment SGI91752[] = {{&S78944[44], 4}, {NULL}};
const Segment SGI91753[] = {{&S78944[44], 16}, {NULL}};
const Segment SGI91754[] = {{&S78944[30], 6}, {NULL}};
const Segment SGI91755[] = {{&S78944[108], 3}, {NULL}};
const Segment SGI91756[] = {{&S78944[74], 5}, {NULL}};
const Segment SGI91757[] = {{&S58245[44], 9}, {NULL}};
const Segment SGI91758[] = {{&S58245[44], 21}, {NULL}};
const Segment SGI91759[] = {{&S58245[30], 6}, {NULL}};
const Segment SGI91760[] = {{&S58245[31], 3}, {NULL}};
const Segment SGI91761[] = {{&S58245[79], 6}, {NULL}};
const Segment SGI91762[] = {{&S58246[38], 9}, {NULL}};
const Segment SGI91763[] = {{&S58246[31], 6}, {NULL}};
const Segment SGI91764[] = {{&S78959[44], 10}, {NULL}};
const Segment SGI91765[] = {{&S78959[44], 27}, {NULL}};
const Segment SGI91766[] = {{&S78959[30], 6}, {NULL}};
const Segment SGI91767[] = {{&S78959[85], 6}, {NULL}};
const Segment SGI91768[] = {{&S78959[112], 13}, {NULL}};
const Segment SGI91769[] = {{&S58249[34], 11}, {NULL}};
const Segment SGI91770[] = {{&S58249[27], 6}, {NULL}};
const Segment SGI91771[] = {{&S58249[98], 9}, {NULL}};
const Segment SGI91772[] = {{&S58249[71], 6}, {NULL}};
const Segment SGI91773[] = {{&S58272[44], 7}, {NULL}};
const Segment SGI91774[] = {{&S58272[44], 20}, {NULL}};
const Segment SGI91775[] = {{&S58272[30], 6}, {NULL}};
const Segment SGI91776[] = {{&S58272[31], 3}, {NULL}};
const Segment SGI91777[] = {{&S58272[78], 6}, {NULL}};
const Segment SGI91778[] = {{&S58273[44], 7}, {NULL}};
const Segment SGI91779[] = {{&S58273[44], 20}, {NULL}};
const Segment SGI91780[] = {{&S58273[30], 6}, {NULL}};
const Segment SGI91781[] = {{&S58273[31], 3}, {NULL}};
const Segment SGI91782[] = {{&S58273[78], 6}, {NULL}};
const Segment SGI91783[] = {{&S58274[38], 7}, {NULL}};
const Segment SGI91784[] = {{&S58274[31], 6}, {NULL}};
const Segment SGI91785[] = {{&S58275[44], 7}, {NULL}};
const Segment SGI91786[] = {{&S58275[44], 20}, {NULL}};
const Segment SGI91787[] = {{&S58275[30], 6}, {NULL}};
const Segment SGI91788[] = {{&S58275[31], 3}, {NULL}};
const Segment SGI91789[] = {{&S58275[78], 6}, {NULL}};
const Segment SGI91790[] = {{&S58276[44], 7}, {NULL}};
const Segment SGI91791[] = {{&S58276[44], 20}, {NULL}};
const Segment SGI91792[] = {{&S58276[30], 6}, {NULL}};
const Segment SGI91793[] = {{&S58276[31], 3}, {NULL}};
const Segment SGI91794[] = {{&S58276[78], 6}, {NULL}};
const Segment SGI91795[] = {{&S58277[34], 6}, {NULL}};
const Segment SGI91796[] = {{&S58277[27], 6}, {NULL}};
const Segment SGI91797[] = {{&S58277[93], 9}, {NULL}};
const Segment SGI91798[] = {{&S58277[66], 6}, {NULL}};
const Segment SGI91799[] = {{&S58281[34], 7}, {NULL}};
const Segment SGI91800[] = {{&S58281[27], 6}, {NULL}};
const Segment SGI91801[] = {{&S58281[96], 9}, {NULL}};
const Segment SGI91802[] = {{&S58281[69], 6}, {NULL}};
const Segment SGI91803[] = {{&S58303[34], 7}, {NULL}};
const Segment SGI91804[] = {{&S58303[27], 6}, {NULL}};
const Segment SGI91805[] = {{&S58303[96], 9}, {NULL}};
const Segment SGI91806[] = {{&S58303[69], 6}, {NULL}};
const Segment SGI91807[] = {{&S58310[44], 10}, {NULL}};
const Segment SGI91808[] = {{&S58310[44], 22}, {NULL}};
const Segment SGI91809[] = {{&S58310[30], 6}, {NULL}};
const Segment SGI91810[] = {{&S58310[31], 3}, {NULL}};
const Segment SGI91811[] = {{&S58310[80], 6}, {NULL}};
const Segment SGI91812[] = {{&S58311[44], 11}, {NULL}};
const Segment SGI91813[] = {{&S58311[44], 23}, {NULL}};
const Segment SGI91814[] = {{&S58311[30], 6}, {NULL}};
const Segment SGI91815[] = {{&S58311[31], 3}, {NULL}};
const Segment SGI91816[] = {{&S58311[81], 6}, {NULL}};
const Segment SGI91817[] = {{&S58312[44], 6}, {NULL}};
const Segment SGI91818[] = {{&S58312[44], 18}, {NULL}};
const Segment SGI91819[] = {{&S58312[30], 6}, {NULL}};
const Segment SGI91820[] = {{&S58312[79], 6}, {NULL}};
const Segment SGI91821[] = {{&S58313[38], 6}, {NULL}};
const Segment SGI91822[] = {{&S58313[31], 6}, {NULL}};
const Segment SGI91823[] = {{&S58314[38], 7}, {NULL}};
const Segment SGI91824[] = {{&S58314[31], 6}, {NULL}};
const Segment SGI91825[] = {{&S58315[41], 10}, {NULL}};
const Segment SGI91826[] = {{&S58315[30], 6}, {NULL}};
const Segment SGI91827[] = {{&S58315[31], 3}, {NULL}};
const Segment SGI91828[] = {{&S58315[77], 6}, {NULL}};
const Segment SGI91829[] = {{&S78991[44], 7}, {NULL}};
const Segment SGI91830[] = {{&S78991[44], 19}, {NULL}};
const Segment SGI91831[] = {{&S78991[30], 6}, {NULL}};
const Segment SGI91832[] = {{&S78991[111], 3}, {NULL}};
const Segment SGI91833[] = {{&S78991[77], 5}, {NULL}};
const Segment SGI91834[] = {{&S78992[44], 7}, {NULL}};
const Segment SGI91835[] = {{&S78992[44], 19}, {NULL}};
const Segment SGI91836[] = {{&S78992[30], 6}, {NULL}};
const Segment SGI91837[] = {{&S78992[80], 6}, {NULL}};
const Segment SGI91838[] = {{&S58327[34], 6}, {NULL}};
const Segment SGI91839[] = {{&S58327[27], 6}, {NULL}};
const Segment SGI91840[] = {{&S58327[92], 9}, {NULL}};
const Segment SGI91841[] = {{&S58327[65], 6}, {NULL}};
const Segment SGI91842[] = {{&S78993[38], 7}, {NULL}};
const Segment SGI91843[] = {{&S78993[31], 6}, {NULL}};
const Segment SGI91844[] = {{&S58330[34], 7}, {NULL}};
const Segment SGI91845[] = {{&S58330[27], 6}, {NULL}};
const Segment SGI91846[] = {{&S78996[44], 16}, {NULL}};
const Segment SGI91847[] = {{&S78996[44], 38}, {NULL}};
const Segment SGI91848[] = {{&S78996[30], 6}, {NULL}};
const Segment SGI91849[] = {{&S58330[99], 9}, {NULL}};
const Segment SGI91850[] = {{&S58330[72], 6}, {NULL}};
const Segment SGI91851[] = {{&S78996[130], 3}, {NULL}};
const Segment SGI91852[] = {{&S78996[96], 5}, {NULL}};
const Segment SGI91853[] = {{&S78997[44], 16}, {NULL}};
const Segment SGI91854[] = {{&S78997[44], 38}, {NULL}};
const Segment SGI91855[] = {{&S78997[30], 6}, {NULL}};
const Segment SGI91856[] = {{&S78997[130], 3}, {NULL}};
const Segment SGI91857[] = {{&S78997[96], 5}, {NULL}};
const Segment SGI91858[] = {{&S79030[59], 8}, {NULL}};
const Segment SGI91859[] = {{&S79030[59], 20}, {NULL}};
const Segment SGI91860[] = {{&S79030[45], 6}, {NULL}};
const Segment SGI91861[] = {{&S79030[127], 3}, {NULL}};
const Segment SGI91862[] = {{&S79030[93], 5}, {NULL}};
const Segment SGI91863[] = {{&S79030[0], 14}, {NULL}};
const Segment SGI91864[] = {{&S79031[59], 5}, {NULL}};
const Segment SGI91865[] = {{&S79031[59], 17}, {NULL}};
const Segment SGI91866[] = {{&S79031[45], 6}, {NULL}};
const Segment SGI91867[] = {{&S79031[124], 3}, {NULL}};
const Segment SGI91868[] = {{&S79031[90], 5}, {NULL}};
const Segment SGI91869[] = {{&S79031[0], 14}, {NULL}};
const Segment SGI91870[] = {{&S79032[59], 13}, {NULL}};
const Segment SGI91871[] = {{&S79032[59], 25}, {NULL}};
const Segment SGI91872[] = {{&S79032[45], 6}, {NULL}};
const Segment SGI91873[] = {{&S79032[132], 3}, {NULL}};
const Segment SGI91874[] = {{&S79032[98], 5}, {NULL}};
const Segment SGI91875[] = {{&S79032[0], 14}, {NULL}};
const Segment SGI91876[] = {{&S79112[52], 10}, {NULL}};
const Segment SGI91877[] = {{&S79112[52], 27}, {NULL}};
const Segment SGI91878[] = {{&S79112[38], 6}, {NULL}};
const Segment SGI91879[] = {{&S79112[93], 5}, {NULL}};
const Segment SGI91880[] = {{&S79112[0], 14}, {NULL}};
const Segment SGI91881[] = {{&S65836[74], 6}, {NULL}};
const Segment SGI91882[] = {{&S65836[0], 9}, {NULL}};
const Segment SGI91883[] = {{&S71576[44], 16}, {NULL}};
const Segment SGI91884[] = {{&S71576[44], 27}, {NULL}};
const Segment SGI91885[] = {{&S71576[30], 6}, {NULL}};
const Segment SGI91886[] = {{&S71576[119], 3}, {NULL}};
const Segment SGI91887[] = {{&S71576[85], 5}, {NULL}};
const Segment SGI91888[] = {{&S79235[44], 21}, {NULL}};
const Segment SGI91889[] = {{&S79235[44], 33}, {NULL}};
const Segment SGI91890[] = {{&S79235[30], 6}, {NULL}};
const Segment SGI91891[] = {{&S79235[125], 3}, {NULL}};
const Segment SGI91892[] = {{&S79235[91], 5}, {NULL}};
const Segment SGI91893[] = {{&S79238[45], 9}, {NULL}};
const Segment SGI91894[] = {{&S79238[30], 6}, {NULL}};
const Segment SGI91895[] = {{&S79238[114], 3}, {NULL}};
const Segment SGI91896[] = {{&S79238[80], 5}, {NULL}};
const Segment SGI91897[] = {{&S79238[138], 13}, {NULL}};
const Segment SGI91898[] = {{&S79241[44], 13}, {NULL}};
const Segment SGI91899[] = {{&S79241[44], 25}, {NULL}};
const Segment SGI91900[] = {{&S79241[30], 6}, {NULL}};
const Segment SGI91901[] = {{&S79241[117], 3}, {NULL}};
const Segment SGI91902[] = {{&S79241[83], 5}, {NULL}};
const Segment SGI91903[] = {{&S79247[44], 10}, {NULL}};
const Segment SGI91904[] = {{&S79247[44], 24}, {NULL}};
const Segment SGI91905[] = {{&S79247[30], 6}, {NULL}};
const Segment SGI91906[] = {{&S79247[116], 3}, {NULL}};
const Segment SGI91907[] = {{&S79247[82], 5}, {NULL}};
const Segment SGI91908[] = {{&S58445[34], 11}, {NULL}};
const Segment SGI91909[] = {{&S58445[27], 6}, {NULL}};
const Segment SGI91910[] = {{&S58445[98], 9}, {NULL}};
const Segment SGI91911[] = {{&S58445[71], 6}, {NULL}};
const Segment SGI91912[] = {{&S71645[44], 10}, {NULL}};
const Segment SGI91913[] = {{&S71645[44], 27}, {NULL}};
const Segment SGI91914[] = {{&S71645[30], 6}, {NULL}};
const Segment SGI91915[] = {{&S71645[138], 3}, {NULL}};
const Segment SGI91916[] = {{&S71645[104], 5}, {NULL}};
const Segment SGI91917[] = {{&S71645[35], 1}, {NULL}};
const Segment SGI91918[] = {{&S71645[73], 11}, {NULL}};
const Segment SGI91919[] = {{&S71646[44], 10}, {NULL}};
const Segment SGI91920[] = {{&S71646[44], 27}, {NULL}};
const Segment SGI91921[] = {{&S71646[30], 6}, {NULL}};
const Segment SGI91922[] = {{&S71646[132], 3}, {NULL}};
const Segment SGI91923[] = {{&S71646[98], 5}, {NULL}};
const Segment SGI91924[] = {{&S71646[78], 3}, {NULL}};
const Segment SGI91925[] = {{&S58466[34], 15}, {NULL}};
const Segment SGI91926[] = {{&S58466[27], 6}, {NULL}};
const Segment SGI91927[] = {{&S58466[102], 9}, {NULL}};
const Segment SGI91928[] = {{&S58466[75], 6}, {NULL}};
const Segment SGI91929[] = {{&S79286[44], 15}, {NULL}};
const Segment SGI91930[] = {{&S79286[44], 27}, {NULL}};
const Segment SGI91931[] = {{&S79286[30], 6}, {NULL}};
const Segment SGI91932[] = {{&S79286[119], 3}, {NULL}};
const Segment SGI91933[] = {{&S79286[85], 5}, {NULL}};
const Segment SGI91934[] = {{&S58483[44], 11}, {NULL}};
const Segment SGI91935[] = {{&S58483[44], 23}, {NULL}};
const Segment SGI91936[] = {{&S58483[30], 6}, {NULL}};
const Segment SGI91937[] = {{&S58483[31], 3}, {NULL}};
const Segment SGI91938[] = {{&S58483[81], 6}, {NULL}};
const Segment SGI91939[] = {{&S58484[44], 11}, {NULL}};
const Segment SGI91940[] = {{&S58484[44], 23}, {NULL}};
const Segment SGI91941[] = {{&S58484[30], 6}, {NULL}};
const Segment SGI91942[] = {{&S71575[42], 7}, {NULL}};
const Segment SGI91943[] = {{&S71575[42], 19}, {NULL}};
const Segment SGI91944[] = {{&S71575[30], 4}, {NULL}};
const Segment SGI91945[] = {{&S58484[84], 6}, {NULL}};
const Segment SGI91946[] = {{&S71575[110], 3}, {NULL}};
const Segment SGI91947[] = {{&S71575[75], 6}, {NULL}};
const Segment SGI91948[] = {{&S58485[44], 11}, {NULL}};
const Segment SGI91949[] = {{&S58485[44], 23}, {NULL}};
const Segment SGI91950[] = {{&S58485[30], 6}, {NULL}};
const Segment SGI91951[] = {{&S58485[84], 4}, {NULL}};
const Segment SGI91952[] = {{&S58485[84], 3}, {NULL}};
const Segment SGI91953[] = {{&S58486[38], 11}, {NULL}};
const Segment SGI91954[] = {{&S58486[31], 6}, {NULL}};
const Segment SGI91955[] = {{&S58487[44], 23}, {NULL}};
const Segment SGI91956[] = {{&S58487[30], 6}, {NULL}};
const Segment SGI91957[] = {{&S79323[44], 10}, {NULL}};
const Segment SGI91958[] = {{&S79323[44], 27}, {NULL}};
const Segment SGI91959[] = {{&S79323[30], 6}, {NULL}};
const Segment SGI91960[] = {{&S79323[149], 4}, {NULL}};
const Segment SGI91961[] = {{&S79323[175], 3}, {NULL}};
const Segment SGI91962[] = {{&S79323[119], 3}, {&S79323[224], 5}, {NULL}};
const Segment SGI91963[] = {{&S79323[85], 5}, {NULL}};
const Segment SGI91964[] = {{&S79342[34], 11}, {NULL}};
const Segment SGI91965[] = {{&S79342[27], 6}, {NULL}};
const Segment SGI91966[] = {{&S79342[110], 2}, {NULL}};
const Segment SGI91967[] = {{&S79342[76], 6}, {NULL}};
const Segment SGI91968[] = {{&S79342[144], 8}, {NULL}};
const Segment SGI91969[] = {{&S58523[34], 9}, {NULL}};
const Segment SGI91970[] = {{&S58523[27], 6}, {NULL}};
const Segment SGI91971[] = {{&S58523[96], 9}, {NULL}};
const Segment SGI91972[] = {{&S58523[69], 6}, {NULL}};
const Segment SGI91973[] = {{&S58524[34], 10}, {NULL}};
const Segment SGI91974[] = {{&S58524[27], 6}, {NULL}};
const Segment SGI91975[] = {{&S58524[97], 9}, {NULL}};
const Segment SGI91976[] = {{&S58524[70], 6}, {NULL}};
const Segment SGI91977[] = {{&S58525[34], 9}, {NULL}};
const Segment SGI91978[] = {{&S58525[27], 6}, {NULL}};
const Segment SGI91979[] = {{&S58525[96], 9}, {NULL}};
const Segment SGI91980[] = {{&S58525[69], 6}, {NULL}};
const Segment SGI91981[] = {{&S58526[34], 10}, {NULL}};
const Segment SGI91982[] = {{&S58526[27], 6}, {NULL}};
const Segment SGI91983[] = {{&S58526[97], 9}, {NULL}};
const Segment SGI91984[] = {{&S58526[70], 6}, {NULL}};
const Segment SGI91985[] = {{&S58527[34], 10}, {NULL}};
const Segment SGI91986[] = {{&S58527[27], 6}, {NULL}};
const Segment SGI91987[] = {{&S58527[97], 9}, {NULL}};
const Segment SGI91988[] = {{&S58527[70], 6}, {NULL}};
const Segment SGI91989[] = {{&S58538[34], 18}, {NULL}};
const Segment SGI91990[] = {{&S58538[27], 6}, {NULL}};
const Segment SGI91991[] = {{&S58538[105], 9}, {NULL}};
const Segment SGI91992[] = {{&S58538[78], 6}, {NULL}};
const Segment SGI91993[] = {{&S79420[44], 10}, {NULL}};
const Segment SGI91994[] = {{&S79420[44], 22}, {NULL}};
const Segment SGI91995[] = {{&S79420[30], 6}, {NULL}};
const Segment SGI91996[] = {{&S79420[114], 3}, {NULL}};
const Segment SGI91997[] = {{&S79420[80], 5}, {NULL}};
const Segment SGI91998[] = {{&S79421[44], 6}, {NULL}};
const Segment SGI91999[] = {{&S79421[44], 18}, {NULL}};
const Segment SGI92000[] = {{&S79421[30], 6}, {NULL}};
const Segment SGI92001[] = {{&S79421[79], 4}, {NULL}};
const Segment SGI92002[] = {{&S79421[79], 3}, {NULL}};
const Segment SGI92003[] = {{&S79422[44], 6}, {NULL}};
const Segment SGI92004[] = {{&S79422[44], 18}, {NULL}};
const Segment SGI92005[] = {{&S79422[30], 6}, {NULL}};
const Segment SGI92006[] = {{&S79422[79], 6}, {NULL}};
const Segment SGI92007[] = {{&S58548[34], 15}, {NULL}};
const Segment SGI92008[] = {{&S58548[27], 6}, {NULL}};
const Segment SGI92009[] = {{&S58548[104], 9}, {NULL}};
const Segment SGI92010[] = {{&S58548[77], 6}, {NULL}};
const Segment SGI92011[] = {{&S79536[45], 10}, {NULL}};
const Segment SGI92012[] = {{&S79536[30], 6}, {NULL}};
const Segment SGI92013[] = {{&S79536[120], 3}, {NULL}};
const Segment SGI92014[] = {{&S79536[86], 5}, {NULL}};
const Segment SGI92015[] = {{&S79536[144], 13}, {NULL}};
const Segment SGI92016[] = {{&S79553[44], 17}, {NULL}};
const Segment SGI92017[] = {{&S79553[44], 38}, {NULL}};
const Segment SGI92018[] = {{&S79553[30], 6}, {NULL}};
const Segment SGI92019[] = {{&S79553[130], 3}, {NULL}};
const Segment SGI92020[] = {{&S79553[96], 5}, {NULL}};
const Segment SGI92021[] = {{&S79554[44], 17}, {NULL}};
const Segment SGI92022[] = {{&S79554[44], 38}, {NULL}};
const Segment SGI92023[] = {{&S79554[30], 6}, {NULL}};
const Segment SGI92024[] = {{&S79554[130], 3}, {NULL}};
const Segment SGI92025[] = {{&S79554[96], 5}, {NULL}};
const Segment SGI92026[] = {{&S79563[34], 10}, {NULL}};
const Segment SGI92027[] = {{&S79563[27], 6}, {NULL}};
const Segment SGI92028[] = {{&S79563[109], 2}, {NULL}};
const Segment SGI92029[] = {{&S79563[75], 6}, {NULL}};
const Segment SGI92030[] = {{&S79563[143], 8}, {NULL}};
const Segment SGI92031[] = {{&S58610[34], 7}, {NULL}};
const Segment SGI92032[] = {{&S58610[27], 6}, {NULL}};
const Segment SGI92033[] = {{&S58610[99], 9}, {NULL}};
const Segment SGI92034[] = {{&S58610[72], 6}, {NULL}};
const Segment SGI92035[] = {{&S58653[34], 10}, {NULL}};
const Segment SGI92036[] = {{&S58653[27], 6}, {NULL}};
const Segment SGI92037[] = {{&S58653[97], 9}, {NULL}};
const Segment SGI92038[] = {{&S58653[70], 6}, {NULL}};
const Segment SGI92039[] = {{&S58654[41], 10}, {NULL}};
const Segment SGI92040[] = {{&S58654[27], 13}, {NULL}};
const Segment SGI92041[] = {{&S58654[104], 9}, {NULL}};
const Segment SGI92042[] = {{&S58654[77], 6}, {NULL}};
const Segment SGI92043[] = {{&S79641[44], 5}, {NULL}};
const Segment SGI92044[] = {{&S79641[44], 17}, {NULL}};
const Segment SGI92045[] = {{&S79641[30], 6}, {NULL}};
const Segment SGI92046[] = {{&S79641[109], 3}, {NULL}};
const Segment SGI92047[] = {{&S79641[75], 5}, {NULL}};
const Segment SGI92048[] = {{&S58697[34], 18}, {NULL}};
const Segment SGI92049[] = {{&S58697[27], 6}, {NULL}};
const Segment SGI92050[] = {{&S72030[44], 10}, {NULL}};
const Segment SGI92051[] = {{&S72030[44], 21}, {NULL}};
const Segment SGI92052[] = {{&S72030[30], 6}, {NULL}};
const Segment SGI92053[] = {{&S58697[105], 9}, {NULL}};
const Segment SGI92054[] = {{&S58697[78], 6}, {NULL}};
const Segment SGI92055[] = {{&S72030[132], 3}, {NULL}};
const Segment SGI92056[] = {{&S72030[98], 5}, {NULL}};
const Segment SGI92057[] = {{&S72030[35], 1}, {NULL}};
const Segment SGI92058[] = {{&S72030[67], 11}, {NULL}};
const Segment SGI92059[] = {{&S72031[44], 10}, {NULL}};
const Segment SGI92060[] = {{&S58699[34], 10}, {NULL}};
const Segment SGI92061[] = {{&S72031[44], 29}, {&S72031[74], 18}, {NULL}};
const Segment SGI92062[] = {{&S72031[30], 6}, {NULL}};
const Segment SGI92063[] = {{&S58699[27], 6}, {NULL}};
const Segment SGI92064[] = {{&S72031[140], 3}, {NULL}};
const Segment SGI92065[] = {{&S58699[97], 9}, {NULL}};
const Segment SGI92066[] = {{&S58699[70], 6}, {NULL}};
const Segment SGI92067[] = {{&S72031[106], 5}, {NULL}};
const Segment SGI92068[] = {{&S72031[35], 1}, {NULL}};
const Segment SGI92069[] = {{&S72031[75], 11}, {NULL}};
const Segment SGI92070[] = {{&S72031[63], 3}, {NULL}};
const Segment SGI92071[] = {{&S69137[6], 4}, {NULL}};
const Segment SGI92072[] = {{&S69137[80], 5}, {NULL}};
const Segment SGI92073[] = {{&S66373[44], 17}, {NULL}};
const Segment SGI92074[] = {{&S66373[44], 38}, {NULL}};
const Segment SGI92075[] = {{&S66373[30], 6}, {NULL}};
const Segment SGI92076[] = {{&S66373[31], 3}, {NULL}};
const Segment SGI92077[] = {{&S66373[96], 5}, {NULL}};
const Segment SGI92078[] = {{&S66373[0], 13}, {NULL}};
const Segment SGI92079[] = {{&S79731[44], 10}, {NULL}};
const Segment SGI92080[] = {{&S79731[44], 27}, {NULL}};
const Segment SGI92081[] = {{&S79731[30], 6}, {NULL}};
const Segment SGI92082[] = {{&S79731[118], 3}, {NULL}};
const Segment SGI92083[] = {{&S79731[85], 5}, {NULL}};
const Segment SGI92084[] = {{&S79731[122], 14}, {NULL}};
const Segment SGI92085[] = {{&S72156[44], 6}, {NULL}};
const Segment SGI92086[] = {{&S72156[44], 27}, {NULL}};
const Segment SGI92087[] = {{&S72156[30], 6}, {NULL}};
const Segment SGI92088[] = {{&S72156[119], 3}, {NULL}};
const Segment SGI92089[] = {{&S72156[85], 5}, {NULL}};
const Segment SGI92090[] = {{&S72157[44], 6}, {NULL}};
const Segment SGI92091[] = {{&S72157[44], 27}, {NULL}};
const Segment SGI92092[] = {{&S72157[30], 6}, {NULL}};
const Segment SGI92093[] = {{&S72157[119], 3}, {NULL}};
const Segment SGI92094[] = {{&S72157[85], 5}, {NULL}};
const Segment SGI92095[] = {{&S72158[44], 6}, {NULL}};
const Segment SGI92096[] = {{&S72158[44], 27}, {NULL}};
const Segment SGI92097[] = {{&S72158[30], 6}, {NULL}};
const Segment SGI92098[] = {{&S72158[119], 3}, {NULL}};
const Segment SGI92099[] = {{&S72158[85], 5}, {NULL}};
const Segment SGI92100[] = {{&S79772[45], 10}, {NULL}};
const Segment SGI92101[] = {{&S79772[30], 6}, {NULL}};
const Segment SGI92102[] = {{&S79772[120], 3}, {NULL}};
const Segment SGI92103[] = {{&S79772[86], 5}, {NULL}};
const Segment SGI92104[] = {{&S79772[144], 13}, {NULL}};
const Segment SGI92105[] = {{&S58732[34], 6}, {NULL}};
const Segment SGI92106[] = {{&S58732[27], 6}, {NULL}};
const Segment SGI92107[] = {{&S58732[93], 9}, {NULL}};
const Segment SGI92108[] = {{&S58732[66], 6}, {NULL}};
const Segment SGI92109[] = {{&S58733[34], 6}, {NULL}};
const Segment SGI92110[] = {{&S58733[27], 6}, {NULL}};
const Segment SGI92111[] = {{&S58733[93], 9}, {NULL}};
const Segment SGI92112[] = {{&S58733[66], 6}, {NULL}};
const Segment SGI92113[] = {{&S58740[44], 6}, {NULL}};
const Segment SGI92114[] = {{&S58740[44], 18}, {NULL}};
const Segment SGI92115[] = {{&S58740[30], 6}, {NULL}};
const Segment SGI92116[] = {{&S58740[31], 3}, {NULL}};
const Segment SGI92117[] = {{&S58740[76], 6}, {NULL}};
const Segment SGI92118[] = {{&S72025[44], 10}, {NULL}};
const Segment SGI92119[] = {{&S72025[44], 21}, {NULL}};
const Segment SGI92120[] = {{&S72025[30], 6}, {NULL}};
const Segment SGI92121[] = {{&S72025[114], 3}, {NULL}};
const Segment SGI92122[] = {{&S72025[79], 6}, {NULL}};
const Segment SGI92123[] = {{&S72026[44], 10}, {NULL}};
const Segment SGI92124[] = {{&S72026[44], 25}, {NULL}};
const Segment SGI92125[] = {{&S72026[30], 6}, {NULL}};
const Segment SGI92126[] = {{&S72026[118], 3}, {NULL}};
const Segment SGI92127[] = {{&S72026[83], 6}, {NULL}};
const Segment SGI92128[] = {{&S72027[41], 10}, {NULL}};
const Segment SGI92129[] = {{&S72027[41], 21}, {NULL}};
const Segment SGI92130[] = {{&S72027[76], 6}, {NULL}};
const Segment SGI92131[] = {{&S72028[38], 10}, {NULL}};
const Segment SGI92132[] = {{&S72028[31], 6}, {NULL}};
const Segment SGI92133[] = {{&S58792[34], 12}, {NULL}};
const Segment SGI92134[] = {{&S58792[27], 6}, {NULL}};
const Segment SGI92135[] = {{&S58792[99], 9}, {NULL}};
const Segment SGI92136[] = {{&S58792[72], 6}, {NULL}};
const Segment SGI92137[] = {{&S58804[44], 11}, {NULL}};
const Segment SGI92138[] = {{&S58804[44], 23}, {NULL}};
const Segment SGI92139[] = {{&S58804[30], 6}, {NULL}};
const Segment SGI92140[] = {{&S58804[31], 3}, {NULL}};
const Segment SGI92141[] = {{&S58804[81], 6}, {NULL}};
const Segment SGI92142[] = {{&S58810[44], 11}, {NULL}};
const Segment SGI92143[] = {{&S58810[44], 23}, {NULL}};
const Segment SGI92144[] = {{&S58810[30], 6}, {NULL}};
const Segment SGI92145[] = {{&S79867[44], 10}, {NULL}};
const Segment SGI92146[] = {{&S79867[44], 27}, {NULL}};
const Segment SGI92147[] = {{&S58810[31], 3}, {NULL}};
const Segment SGI92148[] = {{&S79867[30], 6}, {NULL}};
const Segment SGI92149[] = {{&S58810[81], 5}, {NULL}};
const Segment SGI92150[] = {{&S79867[85], 6}, {NULL}};
const Segment SGI92151[] = {{&S79867[112], 13}, {NULL}};
const Segment SGI92152[] = {{&S58811[44], 11}, {NULL}};
const Segment SGI92153[] = {{&S58811[44], 22}, {NULL}};
const Segment SGI92154[] = {{&S58811[30], 6}, {NULL}};
const Segment SGI92155[] = {{&S58811[31], 3}, {NULL}};
const Segment SGI92156[] = {{&S58811[80], 5}, {NULL}};
const Segment SGI92157[] = {{&S58812[44], 11}, {NULL}};
const Segment SGI92158[] = {{&S58812[44], 22}, {NULL}};
const Segment SGI92159[] = {{&S58812[30], 6}, {NULL}};
const Segment SGI92160[] = {{&S58812[31], 3}, {NULL}};
const Segment SGI92161[] = {{&S58812[80], 5}, {NULL}};
const Segment SGI92162[] = {{&S58814[34], 11}, {NULL}};
const Segment SGI92163[] = {{&S58814[27], 6}, {NULL}};
const Segment SGI92164[] = {{&S58814[98], 9}, {NULL}};
const Segment SGI92165[] = {{&S58814[71], 6}, {NULL}};
const Segment SGI92166[] = {{&S58828[34], 14}, {NULL}};
const Segment SGI92167[] = {{&S58828[27], 6}, {NULL}};
const Segment SGI92168[] = {{&S58828[101], 9}, {NULL}};
const Segment SGI92169[] = {{&S58828[74], 6}, {NULL}};
const Segment SGI92170[] = {{&S79906[45], 10}, {NULL}};
const Segment SGI92171[] = {{&S72301[44], 6}, {NULL}};
const Segment SGI92172[] = {{&S79906[30], 6}, {NULL}};
const Segment SGI92173[] = {{&S72301[44], 25}, {NULL}};
const Segment SGI92174[] = {{&S72301[30], 6}, {NULL}};
const Segment SGI92175[] = {{&S79906[120], 3}, {NULL}};
const Segment SGI92176[] = {{&S72301[132], 3}, {NULL}};
const Segment SGI92177[] = {{&S79906[86], 5}, {NULL}};
const Segment SGI92178[] = {{&S72301[98], 5}, {NULL}};
const Segment SGI92179[] = {{&S79906[144], 13}, {NULL}};
const Segment SGI92180[] = {{&S72301[75], 3}, {NULL}};
const Segment SGI92181[] = {{&S72302[44], 6}, {NULL}};
const Segment SGI92182[] = {{&S72302[44], 17}, {NULL}};
const Segment SGI92183[] = {{&S72302[30], 6}, {NULL}};
const Segment SGI92184[] = {{&S72302[124], 3}, {NULL}};
const Segment SGI92185[] = {{&S72302[90], 5}, {NULL}};
const Segment SGI92186[] = {{&S72302[63], 11}, {NULL}};
const Segment SGI92187[] = {{&S79909[45], 5}, {NULL}};
const Segment SGI92188[] = {{&S79909[30], 6}, {NULL}};
const Segment SGI92189[] = {{&S79909[119], 3}, {NULL}};
const Segment SGI92190[] = {{&S79909[85], 5}, {NULL}};
const Segment SGI92191[] = {{&S79909[143], 13}, {NULL}};
const Segment SGI92192[] = {{&S79910[44], 5}, {NULL}};
const Segment SGI92193[] = {{&S79910[44], 26}, {NULL}};
const Segment SGI92194[] = {{&S79910[30], 6}, {NULL}};
const Segment SGI92195[] = {{&S79910[119], 3}, {NULL}};
const Segment SGI92196[] = {{&S79910[84], 6}, {NULL}};
const Segment SGI92197[] = {{&S79910[139], 8}, {NULL}};
const Segment SGI92198[] = {{&S79910[148], 13}, {NULL}};
const Segment SGI92199[] = {{&S79936[44], 10}, {NULL}};
const Segment SGI92200[] = {{&S79936[44], 27}, {NULL}};
const Segment SGI92201[] = {{&S79936[30], 6}, {NULL}};
const Segment SGI92202[] = {{&S79936[85], 6}, {NULL}};
const Segment SGI92203[] = {{&S79936[112], 13}, {NULL}};
const Segment SGI92204[] = {{&S79986[44], 10}, {NULL}};
const Segment SGI92205[] = {{&S79986[44], 27}, {NULL}};
const Segment SGI92206[] = {{&S79986[30], 6}, {NULL}};
const Segment SGI92207[] = {{&S79986[85], 6}, {NULL}};
const Segment SGI92208[] = {{&S79986[112], 13}, {NULL}};
const Segment SGI92209[] = {{&S79995[44], 12}, {NULL}};
const Segment SGI92210[] = {{&S79995[44], 38}, {NULL}};
const Segment SGI92211[] = {{&S79995[30], 6}, {NULL}};
const Segment SGI92212[] = {{&S79995[130], 3}, {NULL}};
const Segment SGI92213[] = {{&S79995[96], 5}, {NULL}};
const Segment SGI92214[] = {{&S58930[34], 15}, {NULL}};
const Segment SGI92215[] = {{&S58930[27], 6}, {NULL}};
const Segment SGI92216[] = {{&S58930[102], 9}, {NULL}};
const Segment SGI92217[] = {{&S58930[75], 6}, {NULL}};
const Segment SGI92218[] = {{&S80004[59], 10}, {NULL}};
const Segment SGI92219[] = {{&S80004[59], 27}, {NULL}};
const Segment SGI92220[] = {{&S80004[45], 6}, {NULL}};
const Segment SGI92221[] = {{&S80004[134], 3}, {NULL}};
const Segment SGI92222[] = {{&S80004[100], 5}, {NULL}};
const Segment SGI92223[] = {{&S80004[0], 14}, {NULL}};
const Segment SGI92224[] = {{&S58937[34], 14}, {NULL}};
const Segment SGI92225[] = {{&S58937[27], 6}, {NULL}};
const Segment SGI92226[] = {{&S58937[101], 9}, {NULL}};
const Segment SGI92227[] = {{&S58937[74], 6}, {NULL}};
const Segment SGI92228[] = {{&S58939[44], 6}, {NULL}};
const Segment SGI92229[] = {{&S58939[44], 16}, {NULL}};
const Segment SGI92230[] = {{&S58939[30], 6}, {NULL}};
const Segment SGI92231[] = {{&S58939[31], 3}, {NULL}};
const Segment SGI92232[] = {{&S58939[74], 6}, {NULL}};
const Segment SGI92233[] = {{&S58940[44], 7}, {NULL}};
const Segment SGI92234[] = {{&S58940[44], 17}, {NULL}};
const Segment SGI92235[] = {{&S58940[30], 6}, {NULL}};
const Segment SGI92236[] = {{&S58940[31], 3}, {NULL}};
const Segment SGI92237[] = {{&S58940[75], 6}, {NULL}};
const Segment SGI92238[] = {{&S58941[44], 8}, {NULL}};
const Segment SGI92239[] = {{&S58941[44], 18}, {NULL}};
const Segment SGI92240[] = {{&S58941[30], 6}, {NULL}};
const Segment SGI92241[] = {{&S58941[31], 3}, {NULL}};
const Segment SGI92242[] = {{&S58941[76], 6}, {NULL}};
const Segment SGI92243[] = {{&S58942[44], 11}, {NULL}};
const Segment SGI92244[] = {{&S58942[44], 21}, {NULL}};
const Segment SGI92245[] = {{&S58942[30], 6}, {NULL}};
const Segment SGI92246[] = {{&S58942[31], 3}, {NULL}};
const Segment SGI92247[] = {{&S58942[79], 6}, {NULL}};
const Segment SGI92248[] = {{&S58943[44], 18}, {NULL}};
const Segment SGI92249[] = {{&S58943[44], 28}, {NULL}};
const Segment SGI92250[] = {{&S58943[30], 6}, {NULL}};
const Segment SGI92251[] = {{&S58943[31], 3}, {NULL}};
const Segment SGI92252[] = {{&S58943[86], 6}, {NULL}};
const Segment SGI92253[] = {{&S80017[44], 20}, {NULL}};
const Segment SGI92254[] = {{&S80017[44], 32}, {NULL}};
const Segment SGI92255[] = {{&S80017[30], 6}, {NULL}};
const Segment SGI92256[] = {{&S80017[124], 3}, {NULL}};
const Segment SGI92257[] = {{&S80017[90], 5}, {NULL}};
const Segment SGI92258[] = {{&S58944[44], 14}, {NULL}};
const Segment SGI92259[] = {{&S58944[44], 24}, {NULL}};
const Segment SGI92260[] = {{&S58944[30], 6}, {NULL}};
const Segment SGI92261[] = {{&S58944[31], 3}, {NULL}};
const Segment SGI92262[] = {{&S80018[44], 8}, {NULL}};
const Segment SGI92263[] = {{&S58944[82], 6}, {NULL}};
const Segment SGI92264[] = {{&S80018[44], 29}, {NULL}};
const Segment SGI92265[] = {{&S80018[30], 6}, {NULL}};
const Segment SGI92266[] = {{&S80018[121], 3}, {NULL}};
const Segment SGI92267[] = {{&S80018[87], 5}, {NULL}};
const Segment SGI92268[] = {{&S58945[44], 20}, {NULL}};
const Segment SGI92269[] = {{&S58945[44], 29}, {NULL}};
const Segment SGI92270[] = {{&S58945[30], 6}, {NULL}};
const Segment SGI92271[] = {{&S58945[31], 3}, {NULL}};
const Segment SGI92272[] = {{&S58945[87], 6}, {NULL}};
const Segment SGI92273[] = {{&S58946[44], 6}, {NULL}};
const Segment SGI92274[] = {{&S58946[44], 16}, {NULL}};
const Segment SGI92275[] = {{&S58946[30], 6}, {NULL}};
const Segment SGI92276[] = {{&S58946[31], 3}, {NULL}};
const Segment SGI92277[] = {{&S58946[74], 6}, {NULL}};
const Segment SGI92278[] = {{&S58947[44], 16}, {NULL}};
const Segment SGI92279[] = {{&S58947[44], 26}, {NULL}};
const Segment SGI92280[] = {{&S58947[30], 6}, {NULL}};
const Segment SGI92281[] = {{&S58947[31], 3}, {NULL}};
const Segment SGI92282[] = {{&S58947[84], 6}, {NULL}};
const Segment SGI92283[] = {{&S58948[44], 17}, {NULL}};
const Segment SGI92284[] = {{&S58948[44], 27}, {NULL}};
const Segment SGI92285[] = {{&S58948[30], 6}, {NULL}};
const Segment SGI92286[] = {{&S58948[31], 3}, {NULL}};
const Segment SGI92287[] = {{&S58948[85], 6}, {NULL}};
const Segment SGI92288[] = {{&S58949[44], 23}, {NULL}};
const Segment SGI92289[] = {{&S58949[44], 33}, {NULL}};
const Segment SGI92290[] = {{&S58949[30], 6}, {NULL}};
const Segment SGI92291[] = {{&S58949[31], 3}, {NULL}};
const Segment SGI92292[] = {{&S58949[91], 6}, {NULL}};
const Segment SGI92293[] = {{&S58950[44], 10}, {NULL}};
const Segment SGI92294[] = {{&S58950[44], 20}, {NULL}};
const Segment SGI92295[] = {{&S58950[30], 6}, {NULL}};
const Segment SGI92296[] = {{&S58950[31], 3}, {NULL}};
const Segment SGI92297[] = {{&S58950[78], 6}, {NULL}};
const Segment SGI92298[] = {{&S58951[44], 14}, {NULL}};
const Segment SGI92299[] = {{&S58951[44], 24}, {NULL}};
const Segment SGI92300[] = {{&S58951[30], 6}, {NULL}};
const Segment SGI92301[] = {{&S58951[31], 3}, {NULL}};
const Segment SGI92302[] = {{&S58951[82], 6}, {NULL}};
const Segment SGI92303[] = {{&S72459[6], 4}, {NULL}};
const Segment SGI92304[] = {{&S72459[81], 5}, {NULL}};
const Segment SGI92305[] = {{&S80060[44], 9}, {NULL}};
const Segment SGI92306[] = {{&S80060[44], 21}, {NULL}};
const Segment SGI92307[] = {{&S80060[30], 6}, {NULL}};
const Segment SGI92308[] = {{&S80060[113], 3}, {NULL}};
const Segment SGI92309[] = {{&S80060[79], 5}, {NULL}};
const Segment SGI92310[] = {{&S80061[44], 9}, {NULL}};
const Segment SGI92311[] = {{&S80061[44], 21}, {NULL}};
const Segment SGI92312[] = {{&S80061[30], 6}, {NULL}};
const Segment SGI92313[] = {{&S80061[82], 6}, {NULL}};
const Segment SGI92314[] = {{&S58987[34], 11}, {NULL}};
const Segment SGI92315[] = {{&S58987[27], 6}, {NULL}};
const Segment SGI92316[] = {{&S58987[98], 9}, {NULL}};
const Segment SGI92317[] = {{&S58987[71], 6}, {NULL}};
const Segment SGI92318[] = {{&S80080[45], 24}, {NULL}};
const Segment SGI92319[] = {{&S80080[30], 6}, {NULL}};
const Segment SGI92320[] = {{&S80080[129], 3}, {NULL}};
const Segment SGI92321[] = {{&S80080[95], 5}, {NULL}};
const Segment SGI92322[] = {{&S80080[153], 13}, {NULL}};
const Segment SGI92323[] = {{&S80081[44], 20}, {NULL}};
const Segment SGI92324[] = {{&S80081[44], 37}, {NULL}};
const Segment SGI92325[] = {{&S80081[30], 6}, {NULL}};
const Segment SGI92326[] = {{&S80081[129], 3}, {NULL}};
const Segment SGI92327[] = {{&S80081[95], 5}, {NULL}};
const Segment SGI92328[] = {{&S80107[44], 10}, {NULL}};
const Segment SGI92329[] = {{&S80107[44], 27}, {NULL}};
const Segment SGI92330[] = {{&S80107[30], 6}, {NULL}};
const Segment SGI92331[] = {{&S80107[85], 6}, {NULL}};
const Segment SGI92332[] = {{&S80107[112], 13}, {NULL}};
const Segment SGI92333[] = {{&S72562[44], 9}, {NULL}};
const Segment SGI92334[] = {{&S72562[44], 21}, {NULL}};
const Segment SGI92335[] = {{&S72562[30], 6}, {NULL}};
const Segment SGI92336[] = {{&S72562[113], 3}, {NULL}};
const Segment SGI92337[] = {{&S72562[79], 5}, {NULL}};
const Segment SGI92338[] = {{&S72566[38], 9}, {NULL}};
const Segment SGI92339[] = {{&S72566[31], 6}, {NULL}};
const Segment SGI92340[] = {{&S80163[44], 11}, {NULL}};
const Segment SGI92341[] = {{&S80163[44], 28}, {NULL}};
const Segment SGI92342[] = {{&S80163[30], 6}, {NULL}};
const Segment SGI92343[] = {{&S80163[120], 3}, {NULL}};
const Segment SGI92344[] = {{&S80163[86], 5}, {NULL}};
const Segment SGI92345[] = {{&S80163[144], 11}, {NULL}};
const Segment SGI92346[] = {{&S80175[44], 12}, {NULL}};
const Segment SGI92347[] = {{&S80175[44], 24}, {NULL}};
const Segment SGI92348[] = {{&S80175[30], 6}, {NULL}};
const Segment SGI92349[] = {{&S80175[82], 6}, {NULL}};
const Segment SGI92350[] = {{&S80175[109], 13}, {NULL}};
const Segment SGI92351[] = {{&S80176[44], 4}, {NULL}};
const Segment SGI92352[] = {{&S80176[44], 16}, {NULL}};
const Segment SGI92353[] = {{&S80176[30], 6}, {NULL}};
const Segment SGI92354[] = {{&S80176[74], 6}, {NULL}};
const Segment SGI92355[] = {{&S80176[101], 13}, {NULL}};
const Segment SGI92356[] = {{&S72601[45], 7}, {NULL}};
const Segment SGI92357[] = {{&S72601[30], 6}, {NULL}};
const Segment SGI92358[] = {{&S72601[117], 3}, {NULL}};
const Segment SGI92359[] = {{&S72601[83], 5}, {NULL}};
const Segment SGI92360[] = {{&S72601[141], 13}, {NULL}};
const Segment SGI92361[] = {{&S80206[45], 11}, {NULL}};
const Segment SGI92362[] = {{&S80206[30], 6}, {NULL}};
const Segment SGI92363[] = {{&S80206[121], 3}, {NULL}};
const Segment SGI92364[] = {{&S80206[87], 5}, {NULL}};
const Segment SGI92365[] = {{&S80206[145], 13}, {NULL}};
const Segment SGI92366[] = {{&S80207[44], 11}, {NULL}};
const Segment SGI92367[] = {{&S80207[44], 28}, {NULL}};
const Segment SGI92368[] = {{&S80207[30], 6}, {NULL}};
const Segment SGI92369[] = {{&S72614[44], 11}, {NULL}};
const Segment SGI92370[] = {{&S80207[121], 3}, {NULL}};
const Segment SGI92371[] = {{&S72614[44], 21}, {NULL}};
const Segment SGI92372[] = {{&S72614[30], 6}, {NULL}};
const Segment SGI92373[] = {{&S80207[86], 6}, {NULL}};
const Segment SGI92374[] = {{&S80207[141], 8}, {NULL}};
const Segment SGI92375[] = {{&S72614[113], 3}, {NULL}};
const Segment SGI92376[] = {{&S80207[150], 13}, {NULL}};
const Segment SGI92377[] = {{&S72614[79], 5}, {NULL}};
const Segment SGI92378[] = {{&S80225[44], 13}, {NULL}};
const Segment SGI92379[] = {{&S80225[44], 25}, {NULL}};
const Segment SGI92380[] = {{&S80225[30], 6}, {NULL}};
const Segment SGI92381[] = {{&S80225[86], 6}, {NULL}};
const Segment SGI92382[] = {{&S80226[44], 13}, {NULL}};
const Segment SGI92383[] = {{&S80226[44], 25}, {NULL}};
const Segment SGI92384[] = {{&S80226[30], 6}, {NULL}};
const Segment SGI92385[] = {{&S80226[117], 3}, {NULL}};
const Segment SGI92386[] = {{&S80226[83], 5}, {NULL}};
const Segment SGI92387[] = {{&S59137[34], 11}, {NULL}};
const Segment SGI92388[] = {{&S59137[27], 6}, {NULL}};
const Segment SGI92389[] = {{&S59137[98], 9}, {NULL}};
const Segment SGI92390[] = {{&S59137[71], 6}, {NULL}};
const Segment SGI92391[] = {{&S59146[44], 14}, {NULL}};
const Segment SGI92392[] = {{&S59146[44], 31}, {NULL}};
const Segment SGI92393[] = {{&S59146[30], 6}, {NULL}};
const Segment SGI92394[] = {{&S59146[31], 3}, {NULL}};
const Segment SGI92395[] = {{&S59146[89], 6}, {NULL}};
const Segment SGI92396[] = {{&S59147[38], 14}, {NULL}};
const Segment SGI92397[] = {{&S59147[31], 6}, {NULL}};
const Segment SGI92398[] = {{&S59150[34], 18}, {NULL}};
const Segment SGI92399[] = {{&S59150[27], 6}, {NULL}};
const Segment SGI92400[] = {{&S59150[105], 9}, {NULL}};
const Segment SGI92401[] = {{&S59150[78], 6}, {NULL}};
const Segment SGI92402[] = {{&S59151[34], 18}, {NULL}};
const Segment SGI92403[] = {{&S59151[27], 6}, {NULL}};
const Segment SGI92404[] = {{&S59151[105], 9}, {NULL}};
const Segment SGI92405[] = {{&S59151[78], 6}, {NULL}};
const Segment SGI92406[] = {{&S59159[34], 6}, {NULL}};
const Segment SGI92407[] = {{&S59159[27], 6}, {NULL}};
const Segment SGI92408[] = {{&S59159[93], 9}, {NULL}};
const Segment SGI92409[] = {{&S59159[66], 6}, {NULL}};
const Segment SGI92410[] = {{&S80298[44], 9}, {NULL}};
const Segment SGI92411[] = {{&S80298[44], 21}, {NULL}};
const Segment SGI92412[] = {{&S80298[30], 6}, {NULL}};
const Segment SGI92413[] = {{&S80298[113], 3}, {NULL}};
const Segment SGI92414[] = {{&S80298[79], 5}, {NULL}};
const Segment SGI92415[] = {{&S80314[44], 13}, {NULL}};
const Segment SGI92416[] = {{&S80314[44], 25}, {NULL}};
const Segment SGI92417[] = {{&S80314[30], 6}, {NULL}};
const Segment SGI92418[] = {{&S80314[117], 3}, {NULL}};
const Segment SGI92419[] = {{&S80314[83], 5}, {NULL}};
const Segment SGI92420[] = {{&S72257[44], 12}, {NULL}};
const Segment SGI92421[] = {{&S72257[44], 23}, {NULL}};
const Segment SGI92422[] = {{&S72257[30], 6}, {NULL}};
const Segment SGI92423[] = {{&S72257[116], 3}, {NULL}};
const Segment SGI92424[] = {{&S72257[81], 6}, {NULL}};
const Segment SGI92425[] = {{&S72258[42], 12}, {NULL}};
const Segment SGI92426[] = {{&S72258[42], 24}, {NULL}};
const Segment SGI92427[] = {{&S72258[30], 4}, {NULL}};
const Segment SGI92428[] = {{&S72258[115], 3}, {NULL}};
const Segment SGI92429[] = {{&S72258[80], 6}, {NULL}};
const Segment SGI92430[] = {{&S72259[38], 12}, {NULL}};
const Segment SGI92431[] = {{&S72259[31], 6}, {NULL}};
const Segment SGI92432[] = {{&S80332[44], 10}, {NULL}};
const Segment SGI92433[] = {{&S80332[44], 27}, {NULL}};
const Segment SGI92434[] = {{&S80332[30], 6}, {NULL}};
const Segment SGI92435[] = {{&S80332[120], 3}, {NULL}};
const Segment SGI92436[] = {{&S80332[85], 6}, {NULL}};
const Segment SGI92437[] = {{&S80332[140], 8}, {NULL}};
const Segment SGI92438[] = {{&S80332[149], 13}, {NULL}};
const Segment SGI92439[] = {{&S80333[41], 10}, {NULL}};
const Segment SGI92440[] = {{&S80333[30], 6}, {NULL}};
const Segment SGI92441[] = {{&S80333[117], 3}, {NULL}};
const Segment SGI92442[] = {{&S80333[82], 6}, {NULL}};
const Segment SGI92443[] = {{&S80333[137], 8}, {NULL}};
const Segment SGI92444[] = {{&S80333[146], 13}, {NULL}};
const Segment SGI92445[] = {{&S59233[44], 15}, {NULL}};
const Segment SGI92446[] = {{&S59233[44], 27}, {NULL}};
const Segment SGI92447[] = {{&S59233[30], 6}, {NULL}};
const Segment SGI92448[] = {{&S59233[31], 3}, {NULL}};
const Segment SGI92449[] = {{&S59233[85], 6}, {NULL}};
const Segment SGI92450[] = {{&S80367[44], 12}, {NULL}};
const Segment SGI92451[] = {{&S80367[44], 24}, {NULL}};
const Segment SGI92452[] = {{&S80367[30], 6}, {NULL}};
const Segment SGI92453[] = {{&S80367[116], 3}, {NULL}};
const Segment SGI92454[] = {{&S80367[82], 5}, {NULL}};
const Segment SGI92455[] = {{&S59247[44], 8}, {NULL}};
const Segment SGI92456[] = {{&S59247[44], 20}, {NULL}};
const Segment SGI92457[] = {{&S59247[30], 6}, {NULL}};
const Segment SGI92458[] = {{&S59247[31], 3}, {NULL}};
const Segment SGI92459[] = {{&S59247[78], 6}, {NULL}};
const Segment SGI92460[] = {{&S59248[44], 14}, {NULL}};
const Segment SGI92461[] = {{&S59248[44], 26}, {NULL}};
const Segment SGI92462[] = {{&S59248[30], 6}, {NULL}};
const Segment SGI92463[] = {{&S59248[31], 3}, {NULL}};
const Segment SGI92464[] = {{&S59248[84], 6}, {NULL}};
const Segment SGI92465[] = {{&S80371[44], 10}, {NULL}};
const Segment SGI92466[] = {{&S80371[44], 22}, {NULL}};
const Segment SGI92467[] = {{&S80371[30], 6}, {NULL}};
const Segment SGI92468[] = {{&S80371[114], 3}, {NULL}};
const Segment SGI92469[] = {{&S80371[80], 5}, {NULL}};
const Segment SGI92470[] = {{&S80372[38], 10}, {NULL}};
const Segment SGI92471[] = {{&S80372[31], 6}, {NULL}};
const Segment SGI92472[] = {{&S80413[45], 10}, {NULL}};
const Segment SGI92473[] = {{&S80413[30], 6}, {NULL}};
const Segment SGI92474[] = {{&S80413[120], 3}, {NULL}};
const Segment SGI92475[] = {{&S80413[86], 5}, {NULL}};
const Segment SGI92476[] = {{&S80413[144], 13}, {NULL}};
const Segment SGI92477[] = {{&S59311[34], 13}, {NULL}};
const Segment SGI92478[] = {{&S59311[27], 6}, {NULL}};
const Segment SGI92479[] = {{&S59311[100], 9}, {NULL}};
const Segment SGI92480[] = {{&S59311[73], 6}, {NULL}};
const Segment SGI92481[] = {{&S59333[62], 5}, {NULL}};
const Segment SGI92482[] = {{&S59333[44], 17}, {NULL}};
const Segment SGI92483[] = {{&S59333[30], 6}, {NULL}};
const Segment SGI92484[] = {{&S59333[31], 3}, {NULL}};
const Segment SGI92485[] = {{&S59333[90], 5}, {NULL}};
const Segment SGI92486[] = {{&S59334[60], 5}, {NULL}};
const Segment SGI92487[] = {{&S59334[43], 16}, {NULL}};
const Segment SGI92488[] = {{&S59334[29], 6}, {NULL}};
const Segment SGI92489[] = {{&S59334[30], 3}, {NULL}};
const Segment SGI92490[] = {{&S59334[88], 6}, {NULL}};
const Segment SGI92491[] = {{&S59342[44], 12}, {NULL}};
const Segment SGI92492[] = {{&S59342[44], 24}, {NULL}};
const Segment SGI92493[] = {{&S59342[30], 6}, {NULL}};
const Segment SGI92494[] = {{&S59342[31], 3}, {NULL}};
const Segment SGI92495[] = {{&S59342[82], 6}, {NULL}};
const Segment SGI92496[] = {{&S59357[44], 8}, {NULL}};
const Segment SGI92497[] = {{&S59357[44], 20}, {NULL}};
const Segment SGI92498[] = {{&S59357[30], 6}, {NULL}};
const Segment SGI92499[] = {{&S59357[31], 3}, {NULL}};
const Segment SGI92500[] = {{&S59357[78], 6}, {NULL}};
const Segment SGI92501[] = {{&S72800[34], 7}, {NULL}};
const Segment SGI92502[] = {{&S72800[27], 6}, {NULL}};
const Segment SGI92503[] = {{&S72800[102], 2}, {NULL}};
const Segment SGI92504[] = {{&S72800[68], 6}, {NULL}};
const Segment SGI92505[] = {{&S72800[136], 8}, {NULL}};
const Segment SGI92506[] = {{&S59414[34], 5}, {NULL}};
const Segment SGI92507[] = {{&S59414[27], 6}, {NULL}};
const Segment SGI92508[] = {{&S59414[98], 9}, {NULL}};
const Segment SGI92509[] = {{&S59414[71], 6}, {NULL}};
const Segment SGI92510[] = {{&S59419[34], 7}, {NULL}};
const Segment SGI92511[] = {{&S59419[27], 6}, {NULL}};
const Segment SGI92512[] = {{&S59419[94], 9}, {NULL}};
const Segment SGI92513[] = {{&S59419[67], 6}, {NULL}};
const Segment SGI92514[] = {{&S72862[44], 14}, {NULL}};
const Segment SGI92515[] = {{&S72862[44], 25}, {NULL}};
const Segment SGI92516[] = {{&S72862[30], 6}, {NULL}};
const Segment SGI92517[] = {{&S72862[136], 3}, {NULL}};
const Segment SGI92518[] = {{&S72862[102], 5}, {NULL}};
const Segment SGI92519[] = {{&S72862[35], 1}, {NULL}};
const Segment SGI92520[] = {{&S72862[71], 11}, {NULL}};
const Segment SGI92521[] = {{&S72864[44], 6}, {NULL}};
const Segment SGI92522[] = {{&S72864[44], 25}, {NULL}};
const Segment SGI92523[] = {{&S72864[30], 6}, {NULL}};
const Segment SGI92524[] = {{&S72864[132], 3}, {NULL}};
const Segment SGI92525[] = {{&S72864[98], 5}, {NULL}};
const Segment SGI92526[] = {{&S72864[75], 3}, {NULL}};
const Segment SGI92527[] = {{&S72886[44], 6}, {NULL}};
const Segment SGI92528[] = {{&S72886[44], 17}, {NULL}};
const Segment SGI92529[] = {{&S72886[30], 6}, {NULL}};
const Segment SGI92530[] = {{&S72886[109], 3}, {NULL}};
const Segment SGI92531[] = {{&S72886[75], 5}, {NULL}};
const Segment SGI92532[] = {{&S72891[38], 6}, {NULL}};
const Segment SGI92533[] = {{&S72891[31], 6}, {NULL}};
const Segment SGI92534[] = {{&S72892[44], 6}, {NULL}};
const Segment SGI92535[] = {{&S72892[44], 17}, {NULL}};
const Segment SGI92536[] = {{&S72892[30], 6}, {NULL}};
const Segment SGI92537[] = {{&S72892[122], 3}, {NULL}};
const Segment SGI92538[] = {{&S72892[88], 5}, {NULL}};
const Segment SGI92539[] = {{&S72893[44], 6}, {NULL}};
const Segment SGI92540[] = {{&S72893[44], 17}, {NULL}};
const Segment SGI92541[] = {{&S72893[30], 6}, {NULL}};
const Segment SGI92542[] = {{&S72893[78], 4}, {NULL}};
const Segment SGI92543[] = {{&S72893[78], 3}, {NULL}};
const Segment SGI92544[] = {{&S72895[44], 18}, {NULL}};
const Segment SGI92545[] = {{&S72895[44], 28}, {NULL}};
const Segment SGI92546[] = {{&S72895[30], 6}, {NULL}};
const Segment SGI92547[] = {{&S72895[120], 3}, {NULL}};
const Segment SGI92548[] = {{&S72895[86], 5}, {NULL}};
const Segment SGI92549[] = {{&S72897[44], 19}, {NULL}};
const Segment SGI92550[] = {{&S72897[44], 30}, {NULL}};
const Segment SGI92551[] = {{&S72897[30], 6}, {NULL}};
const Segment SGI92552[] = {{&S72897[121], 3}, {NULL}};
const Segment SGI92553[] = {{&S72897[88], 5}, {NULL}};
const Segment SGI92554[] = {{&S80599[44], 18}, {NULL}};
const Segment SGI92555[] = {{&S80599[44], 30}, {NULL}};
const Segment SGI92556[] = {{&S80599[30], 6}, {NULL}};
const Segment SGI92557[] = {{&S80599[122], 3}, {NULL}};
const Segment SGI92558[] = {{&S80599[88], 5}, {NULL}};
const Segment SGI92559[] = {{&S59445[34], 10}, {NULL}};
const Segment SGI92560[] = {{&S59445[27], 6}, {NULL}};
const Segment SGI92561[] = {{&S59445[97], 9}, {NULL}};
const Segment SGI92562[] = {{&S59445[70], 6}, {NULL}};
const Segment SGI92563[] = {{&S59477[44], 4}, {NULL}};
const Segment SGI92564[] = {{&S59477[44], 16}, {NULL}};
const Segment SGI92565[] = {{&S59477[30], 6}, {NULL}};
const Segment SGI92566[] = {{&S59477[31], 3}, {NULL}};
const Segment SGI92567[] = {{&S59477[74], 6}, {NULL}};
const Segment SGI92568[] = {{&S59482[34], 11}, {NULL}};
const Segment SGI92569[] = {{&S59482[27], 6}, {NULL}};
const Segment SGI92570[] = {{&S59482[98], 9}, {NULL}};
const Segment SGI92571[] = {{&S59482[71], 6}, {NULL}};
const Segment SGI92572[] = {{&S59483[34], 11}, {NULL}};
const Segment SGI92573[] = {{&S59483[27], 6}, {NULL}};
const Segment SGI92574[] = {{&S59483[98], 9}, {NULL}};
const Segment SGI92575[] = {{&S59483[71], 6}, {NULL}};
const Segment SGI92576[] = {{&S59499[34], 5}, {NULL}};
const Segment SGI92577[] = {{&S59499[27], 6}, {NULL}};
const Segment SGI92578[] = {{&S59499[98], 9}, {NULL}};
const Segment SGI92579[] = {{&S59499[71], 6}, {NULL}};
const Segment SGI92580[] = {{&S59502[34], 10}, {NULL}};
const Segment SGI92581[] = {{&S59502[27], 6}, {NULL}};
const Segment SGI92582[] = {{&S59502[97], 9}, {NULL}};
const Segment SGI92583[] = {{&S59502[70], 6}, {NULL}};
const Segment SGI92584[] = {{&S59503[34], 11}, {NULL}};
const Segment SGI92585[] = {{&S59503[27], 6}, {NULL}};
const Segment SGI92586[] = {{&S59503[98], 9}, {NULL}};
const Segment SGI92587[] = {{&S59503[71], 6}, {NULL}};
const Segment SGI92588[] = {{&S59509[34], 7}, {NULL}};
const Segment SGI92589[] = {{&S59509[27], 6}, {NULL}};
const Segment SGI92590[] = {{&S59509[94], 9}, {NULL}};
const Segment SGI92591[] = {{&S59509[67], 6}, {NULL}};
const Segment SGI92592[] = {{&S80740[45], 10}, {NULL}};
const Segment SGI92593[] = {{&S80740[30], 6}, {NULL}};
const Segment SGI92594[] = {{&S80740[120], 3}, {NULL}};
const Segment SGI92595[] = {{&S80740[86], 5}, {NULL}};
const Segment SGI92596[] = {{&S80740[144], 13}, {NULL}};
const Segment SGI92597[] = {{&S80741[44], 10}, {NULL}};
const Segment SGI92598[] = {{&S80741[44], 27}, {NULL}};
const Segment SGI92599[] = {{&S80741[30], 6}, {NULL}};
const Segment SGI92600[] = {{&S80741[120], 3}, {NULL}};
const Segment SGI92601[] = {{&S80741[85], 6}, {NULL}};
const Segment SGI92602[] = {{&S80741[140], 8}, {NULL}};
const Segment SGI92603[] = {{&S80741[149], 13}, {NULL}};
const Segment SGI92604[] = {{&S59522[34], 7}, {NULL}};
const Segment SGI92605[] = {{&S59522[27], 6}, {NULL}};
const Segment SGI92606[] = {{&S59522[94], 9}, {NULL}};
const Segment SGI92607[] = {{&S59522[67], 6}, {NULL}};
const Segment SGI92608[] = {{&S80756[44], 10}, {NULL}};
const Segment SGI92609[] = {{&S80756[44], 27}, {NULL}};
const Segment SGI92610[] = {{&S80756[30], 6}, {NULL}};
const Segment SGI92611[] = {{&S80756[88], 6}, {NULL}};
const Segment SGI92612[] = {{&S80757[44], 10}, {NULL}};
const Segment SGI92613[] = {{&S80757[44], 27}, {NULL}};
const Segment SGI92614[] = {{&S80757[30], 6}, {NULL}};
const Segment SGI92615[] = {{&S80757[119], 3}, {NULL}};
const Segment SGI92616[] = {{&S80757[85], 5}, {NULL}};
const Segment SGI92617[] = {{&S73026[44], 10}, {NULL}};
const Segment SGI92618[] = {{&S73026[44], 21}, {NULL}};
const Segment SGI92619[] = {{&S73026[30], 6}, {NULL}};
const Segment SGI92620[] = {{&S73026[132], 3}, {NULL}};
const Segment SGI92621[] = {{&S73026[98], 5}, {NULL}};
const Segment SGI92622[] = {{&S73026[35], 1}, {NULL}};
const Segment SGI92623[] = {{&S73026[67], 11}, {NULL}};
const Segment SGI92624[] = {{&S73039[44], 14}, {NULL}};
const Segment SGI92625[] = {{&S73039[44], 25}, {NULL}};
const Segment SGI92626[] = {{&S73039[30], 6}, {NULL}};
const Segment SGI92627[] = {{&S73039[117], 3}, {NULL}};
const Segment SGI92628[] = {{&S73039[83], 5}, {NULL}};
const Segment SGI92629[] = {{&S73040[44], 14}, {NULL}};
const Segment SGI92630[] = {{&S73040[44], 30}, {NULL}};
const Segment SGI92631[] = {{&S73040[30], 6}, {NULL}};
const Segment SGI92632[] = {{&S73040[122], 3}, {NULL}};
const Segment SGI92633[] = {{&S73040[88], 5}, {NULL}};
const Segment SGI92634[] = {{&S73051[44], 7}, {NULL}};
const Segment SGI92635[] = {{&S73051[44], 18}, {NULL}};
const Segment SGI92636[] = {{&S73051[30], 6}, {NULL}};
const Segment SGI92637[] = {{&S73051[129], 3}, {NULL}};
const Segment SGI92638[] = {{&S73051[95], 5}, {NULL}};
const Segment SGI92639[] = {{&S73051[35], 1}, {NULL}};
const Segment SGI92640[] = {{&S73051[64], 11}, {NULL}};
const Segment SGI92641[] = {{&S73052[44], 7}, {NULL}};
const Segment SGI92642[] = {{&S73052[44], 18}, {NULL}};
const Segment SGI92643[] = {{&S73052[30], 6}, {NULL}};
const Segment SGI92644[] = {{&S73052[122], 3}, {NULL}};
const Segment SGI92645[] = {{&S73052[88], 5}, {NULL}};
const Segment SGI92646[] = {{&S73052[66], 3}, {NULL}};
const Segment SGI92647[] = {{&S73053[44], 7}, {NULL}};
const Segment SGI92648[] = {{&S73053[44], 18}, {NULL}};
const Segment SGI92649[] = {{&S73053[30], 6}, {NULL}};
const Segment SGI92650[] = {{&S73053[129], 3}, {NULL}};
const Segment SGI92651[] = {{&S73053[95], 5}, {NULL}};
const Segment SGI92652[] = {{&S73053[35], 1}, {NULL}};
const Segment SGI92653[] = {{&S73053[64], 11}, {NULL}};
const Segment SGI92654[] = {{&S73054[44], 10}, {NULL}};
const Segment SGI92655[] = {{&S73054[44], 21}, {NULL}};
const Segment SGI92656[] = {{&S73054[30], 6}, {NULL}};
const Segment SGI92657[] = {{&S73054[125], 3}, {NULL}};
const Segment SGI92658[] = {{&S73054[91], 5}, {NULL}};
const Segment SGI92659[] = {{&S73054[69], 3}, {NULL}};
const Segment SGI92660[] = {{&S80900[44], 5}, {NULL}};
const Segment SGI92661[] = {{&S80900[44], 17}, {NULL}};
const Segment SGI92662[] = {{&S80900[30], 6}, {NULL}};
const Segment SGI92663[] = {{&S80900[109], 3}, {NULL}};
const Segment SGI92664[] = {{&S80900[75], 5}, {NULL}};
const Segment SGI92665[] = {{&S59589[34], 18}, {NULL}};
const Segment SGI92666[] = {{&S59589[27], 6}, {NULL}};
const Segment SGI92667[] = {{&S59589[105], 9}, {NULL}};
const Segment SGI92668[] = {{&S59589[78], 6}, {NULL}};
const Segment SGI92669[] = {{&S73179[38], 9}, {NULL}};
const Segment SGI92670[] = {{&S73179[31], 6}, {NULL}};
const Segment SGI92671[] = {{&S73180[44], 9}, {NULL}};
const Segment SGI92672[] = {{&S73180[44], 20}, {NULL}};
const Segment SGI92673[] = {{&S73180[30], 6}, {NULL}};
const Segment SGI92674[] = {{&S73180[130], 3}, {NULL}};
const Segment SGI92675[] = {{&S73180[96], 5}, {NULL}};
const Segment SGI92676[] = {{&S80948[44], 25}, {NULL}};
const Segment SGI92677[] = {{&S80948[44], 37}, {NULL}};
const Segment SGI92678[] = {{&S80948[30], 6}, {NULL}};
const Segment SGI92679[] = {{&S80948[129], 3}, {NULL}};
const Segment SGI92680[] = {{&S80948[95], 5}, {NULL}};
const Segment SGI92681[] = {{&S73222[44], 9}, {NULL}};
const Segment SGI92682[] = {{&S73222[44], 20}, {NULL}};
const Segment SGI92683[] = {{&S73222[30], 6}, {NULL}};
const Segment SGI92684[] = {{&S73222[124], 3}, {NULL}};
const Segment SGI92685[] = {{&S73222[90], 5}, {NULL}};
const Segment SGI92686[] = {{&S73222[66], 3}, {NULL}};
const Segment SGI92687[] = {{&S73223[44], 9}, {NULL}};
const Segment SGI92688[] = {{&S80974[44], 13}, {NULL}};
const Segment SGI92689[] = {{&S80974[44], 34}, {NULL}};
const Segment SGI92690[] = {{&S73223[44], 20}, {NULL}};
const Segment SGI92691[] = {{&S80974[30], 6}, {NULL}};
const Segment SGI92692[] = {{&S73223[30], 6}, {NULL}};
const Segment SGI92693[] = {{&S80974[126], 3}, {NULL}};
const Segment SGI92694[] = {{&S73223[127], 3}, {NULL}};
const Segment SGI92695[] = {{&S80974[92], 5}, {NULL}};
const Segment SGI92696[] = {{&S73223[93], 5}, {NULL}};
const Segment SGI92697[] = {{&S73223[35], 1}, {NULL}};
const Segment SGI92698[] = {{&S73223[66], 11}, {NULL}};
const Segment SGI92699[] = {{&S73224[44], 9}, {NULL}};
const Segment SGI92700[] = {{&S73224[44], 20}, {NULL}};
const Segment SGI92701[] = {{&S73224[30], 6}, {NULL}};
const Segment SGI92702[] = {{&S73224[131], 3}, {NULL}};
const Segment SGI92703[] = {{&S73224[97], 5}, {NULL}};
const Segment SGI92704[] = {{&S73224[35], 1}, {NULL}};
const Segment SGI92705[] = {{&S73224[66], 11}, {NULL}};
const Segment SGI92706[] = {{&S80978[44], 10}, {NULL}};
const Segment SGI92707[] = {{&S80978[44], 27}, {NULL}};
const Segment SGI92708[] = {{&S80978[30], 6}, {NULL}};
const Segment SGI92709[] = {{&S80978[85], 6}, {NULL}};
const Segment SGI92710[] = {{&S80978[112], 13}, {NULL}};
const Segment SGI92711[] = {{&S81028[34], 11}, {NULL}};
const Segment SGI92712[] = {{&S81028[27], 6}, {NULL}};
const Segment SGI92713[] = {{&S81028[110], 2}, {NULL}};
const Segment SGI92714[] = {{&S81028[76], 6}, {NULL}};
const Segment SGI92715[] = {{&S81028[144], 8}, {NULL}};
const Segment SGI92716[] = {{&S81029[34], 10}, {NULL}};
const Segment SGI92717[] = {{&S81029[27], 6}, {NULL}};
const Segment SGI92718[] = {{&S81029[109], 2}, {NULL}};
const Segment SGI92719[] = {{&S81029[75], 6}, {NULL}};
const Segment SGI92720[] = {{&S81029[143], 8}, {NULL}};
const Segment SGI92721[] = {{&S81034[52], 11}, {NULL}};
const Segment SGI92722[] = {{&S81034[52], 28}, {NULL}};
const Segment SGI92723[] = {{&S81034[38], 6}, {NULL}};
const Segment SGI92724[] = {{&S81034[94], 5}, {NULL}};
const Segment SGI92725[] = {{&S81034[0], 14}, {NULL}};
const Segment SGI92726[] = {{&S59609[44], 7}, {NULL}};
const Segment SGI92727[] = {{&S59609[44], 19}, {NULL}};
const Segment SGI92728[] = {{&S59609[30], 6}, {NULL}};
const Segment SGI92729[] = {{&S59609[31], 3}, {NULL}};
const Segment SGI92730[] = {{&S59609[77], 6}, {NULL}};
const Segment SGI92731[] = {{&S59610[44], 9}, {NULL}};
const Segment SGI92732[] = {{&S59610[44], 21}, {NULL}};
const Segment SGI92733[] = {{&S59610[30], 6}, {NULL}};
const Segment SGI92734[] = {{&S59610[31], 3}, {NULL}};
const Segment SGI92735[] = {{&S59610[79], 6}, {NULL}};
const Segment SGI92736[] = {{&S59613[34], 14}, {NULL}};
const Segment SGI92737[] = {{&S59613[27], 6}, {NULL}};
const Segment SGI92738[] = {{&S59613[101], 9}, {NULL}};
const Segment SGI92739[] = {{&S59613[74], 6}, {NULL}};
const Segment SGI92740[] = {{&S67451[34], 14}, {NULL}};
const Segment SGI92741[] = {{&S67451[27], 6}, {NULL}};
const Segment SGI92742[] = {{&S67451[101], 9}, {NULL}};
const Segment SGI92743[] = {{&S67451[74], 6}, {NULL}};
const Segment SGI92744[] = {{&S67451[0], 13}, {NULL}};
const Segment SGI92745[] = {{&S81073[45], 10}, {NULL}};
const Segment SGI92746[] = {{&S81073[30], 6}, {NULL}};
const Segment SGI92747[] = {{&S81073[120], 3}, {NULL}};
const Segment SGI92748[] = {{&S81073[86], 5}, {NULL}};
const Segment SGI92749[] = {{&S81073[144], 13}, {NULL}};
const Segment SGI92750[] = {{&S59636[34], 12}, {NULL}};
const Segment SGI92751[] = {{&S59636[27], 6}, {NULL}};
const Segment SGI92752[] = {{&S59636[99], 9}, {NULL}};
const Segment SGI92753[] = {{&S59636[72], 6}, {NULL}};
const Segment SGI92754[] = {{&S59637[34], 16}, {NULL}};
const Segment SGI92755[] = {{&S59637[27], 6}, {NULL}};
const Segment SGI92756[] = {{&S59637[103], 9}, {NULL}};
const Segment SGI92757[] = {{&S59637[76], 6}, {NULL}};
const Segment SGI92758[] = {{&S59639[34], 11}, {NULL}};
const Segment SGI92759[] = {{&S59639[27], 6}, {NULL}};
const Segment SGI92760[] = {{&S59639[98], 9}, {NULL}};
const Segment SGI92761[] = {{&S59639[71], 6}, {NULL}};
const Segment SGI92762[] = {{&S59640[34], 15}, {NULL}};
const Segment SGI92763[] = {{&S59640[27], 6}, {NULL}};
const Segment SGI92764[] = {{&S59640[102], 9}, {NULL}};
const Segment SGI92765[] = {{&S59640[75], 6}, {NULL}};
const Segment SGI92766[] = {{&S81141[44], 14}, {NULL}};
const Segment SGI92767[] = {{&S81141[44], 26}, {NULL}};
const Segment SGI92768[] = {{&S81141[30], 6}, {NULL}};
const Segment SGI92769[] = {{&S73398[44], 27}, {NULL}};
const Segment SGI92770[] = {{&S73398[44], 38}, {NULL}};
const Segment SGI92771[] = {{&S81141[118], 3}, {NULL}};
const Segment SGI92772[] = {{&S73398[30], 6}, {NULL}};
const Segment SGI92773[] = {{&S81141[84], 5}, {NULL}};
const Segment SGI92774[] = {{&S73398[149], 3}, {NULL}};
const Segment SGI92775[] = {{&S73398[115], 5}, {NULL}};
const Segment SGI92776[] = {{&S73398[35], 1}, {NULL}};
const Segment SGI92777[] = {{&S73398[84], 11}, {NULL}};
const Segment SGI92778[] = {{&S59711[44], 7}, {NULL}};
const Segment SGI92779[] = {{&S59711[44], 19}, {NULL}};
const Segment SGI92780[] = {{&S59711[30], 6}, {NULL}};
const Segment SGI92781[] = {{&S59711[80], 6}, {NULL}};
const Segment SGI92782[] = {{&S59712[44], 7}, {NULL}};
const Segment SGI92783[] = {{&S59712[44], 19}, {NULL}};
const Segment SGI92784[] = {{&S59712[30], 6}, {NULL}};
const Segment SGI92785[] = {{&S59712[31], 3}, {NULL}};
const Segment SGI92786[] = {{&S59712[77], 6}, {NULL}};
const Segment SGI92787[] = {{&S81159[44], 14}, {NULL}};
const Segment SGI92788[] = {{&S81159[44], 26}, {NULL}};
const Segment SGI92789[] = {{&S81159[30], 6}, {NULL}};
const Segment SGI92790[] = {{&S81159[84], 6}, {NULL}};
const Segment SGI92791[] = {{&S81159[111], 13}, {NULL}};
const Segment SGI92792[] = {{&S81178[44], 27}, {NULL}};
const Segment SGI92793[] = {{&S81178[30], 6}, {NULL}};
const Segment SGI92794[] = {{&S81178[137], 3}, {NULL}};
const Segment SGI92795[] = {{&S81178[103], 5}, {NULL}};
const Segment SGI92796[] = {{&S81179[44], 8}, {NULL}};
const Segment SGI92797[] = {{&S81179[44], 29}, {NULL}};
const Segment SGI92798[] = {{&S81179[30], 6}, {NULL}};
const Segment SGI92799[] = {{&S81179[90], 6}, {NULL}};
const Segment SGI92800[] = {{&S81180[44], 27}, {NULL}};
const Segment SGI92801[] = {{&S81180[30], 6}, {NULL}};
const Segment SGI92802[] = {{&S81180[137], 3}, {NULL}};
const Segment SGI92803[] = {{&S81180[103], 5}, {NULL}};
const Segment SGI92804[] = {{&S81181[44], 8}, {NULL}};
const Segment SGI92805[] = {{&S81181[44], 29}, {NULL}};
const Segment SGI92806[] = {{&S81181[30], 6}, {NULL}};
const Segment SGI92807[] = {{&S81181[121], 3}, {NULL}};
const Segment SGI92808[] = {{&S81181[87], 5}, {NULL}};
const Segment SGI92809[] = {{&S81187[44], 13}, {NULL}};
const Segment SGI92810[] = {{&S81187[44], 25}, {NULL}};
const Segment SGI92811[] = {{&S81187[30], 6}, {NULL}};
const Segment SGI92812[] = {{&S81187[117], 3}, {NULL}};
const Segment SGI92813[] = {{&S81187[83], 5}, {NULL}};
const Segment SGI92814[] = {{&S59772[44], 11}, {NULL}};
const Segment SGI92815[] = {{&S59772[44], 23}, {NULL}};
const Segment SGI92816[] = {{&S59772[30], 6}, {NULL}};
const Segment SGI92817[] = {{&S59772[31], 3}, {NULL}};
const Segment SGI92818[] = {{&S59772[81], 6}, {NULL}};
const Segment SGI92819[] = {{&S59821[34], 14}, {NULL}};
const Segment SGI92820[] = {{&S59821[27], 6}, {NULL}};
const Segment SGI92821[] = {{&S59821[101], 9}, {NULL}};
const Segment SGI92822[] = {{&S59821[74], 6}, {NULL}};
const Segment SGI92823[] = {{&S59822[34], 14}, {NULL}};
const Segment SGI92824[] = {{&S59822[27], 6}, {NULL}};
const Segment SGI92825[] = {{&S59822[101], 9}, {NULL}};
const Segment SGI92826[] = {{&S59822[74], 6}, {NULL}};
const Segment SGI92827[] = {{&S59823[34], 10}, {NULL}};
const Segment SGI92828[] = {{&S59823[27], 6}, {NULL}};
const Segment SGI92829[] = {{&S59823[97], 9}, {NULL}};
const Segment SGI92830[] = {{&S59823[70], 6}, {NULL}};
const Segment SGI92831[] = {{&S59824[34], 35}, {NULL}};
const Segment SGI92832[] = {{&S59824[27], 6}, {NULL}};
const Segment SGI92833[] = {{&S59824[122], 9}, {NULL}};
const Segment SGI92834[] = {{&S59824[95], 6}, {NULL}};
const Segment SGI92835[] = {{&S59826[34], 41}, {NULL}};
const Segment SGI92836[] = {{&S59826[27], 6}, {NULL}};
const Segment SGI92837[] = {{&S59826[128], 9}, {NULL}};
const Segment SGI92838[] = {{&S59826[101], 6}, {NULL}};
const Segment SGI92839[] = {{&S73557[44], 8}, {NULL}};
const Segment SGI92840[] = {{&S73557[44], 19}, {NULL}};
const Segment SGI92841[] = {{&S73557[30], 6}, {NULL}};
const Segment SGI92842[] = {{&S59827[34], 41}, {NULL}};
const Segment SGI92843[] = {{&S59827[27], 6}, {NULL}};
const Segment SGI92844[] = {{&S73557[111], 3}, {NULL}};
const Segment SGI92845[] = {{&S73557[77], 5}, {NULL}};
const Segment SGI92846[] = {{&S59827[128], 9}, {NULL}};
const Segment SGI92847[] = {{&S59827[101], 6}, {NULL}};
const Segment SGI92848[] = {{&S73558[44], 8}, {NULL}};
const Segment SGI92849[] = {{&S73558[44], 25}, {NULL}};
const Segment SGI92850[] = {{&S73558[30], 6}, {NULL}};
const Segment SGI92851[] = {{&S73558[117], 3}, {NULL}};
const Segment SGI92852[] = {{&S73558[83], 5}, {NULL}};
const Segment SGI92853[] = {{&S59881[34], 9}, {NULL}};
const Segment SGI92854[] = {{&S59881[27], 6}, {NULL}};
const Segment SGI92855[] = {{&S59881[96], 9}, {NULL}};
const Segment SGI92856[] = {{&S59881[69], 6}, {NULL}};
const Segment SGI92857[] = {{&S59882[34], 9}, {NULL}};
const Segment SGI92858[] = {{&S59882[27], 6}, {NULL}};
const Segment SGI92859[] = {{&S59882[96], 9}, {NULL}};
const Segment SGI92860[] = {{&S59882[69], 6}, {NULL}};
const Segment SGI92861[] = {{&S59887[34], 17}, {NULL}};
const Segment SGI92862[] = {{&S59887[27], 6}, {NULL}};
const Segment SGI92863[] = {{&S59887[120], 9}, {NULL}};
const Segment SGI92864[] = {{&S59887[93], 6}, {NULL}};
const Segment SGI92865[] = {{&S59890[34], 18}, {NULL}};
const Segment SGI92866[] = {{&S59890[27], 6}, {NULL}};
const Segment SGI92867[] = {{&S59890[105], 9}, {NULL}};
const Segment SGI92868[] = {{&S59890[78], 6}, {NULL}};
const Segment SGI92869[] = {{&S81248[44], 10}, {NULL}};
const Segment SGI92870[] = {{&S81248[44], 27}, {NULL}};
const Segment SGI92871[] = {{&S81248[30], 6}, {NULL}};
const Segment SGI92872[] = {{&S81248[119], 3}, {NULL}};
const Segment SGI92873[] = {{&S81248[85], 5}, {NULL}};
const Segment SGI92874[] = {{&S59899[34], 6}, {NULL}};
const Segment SGI92875[] = {{&S59899[27], 6}, {NULL}};
const Segment SGI92876[] = {{&S59899[93], 9}, {NULL}};
const Segment SGI92877[] = {{&S59899[66], 6}, {NULL}};
const Segment SGI92878[] = {{&S81260[59], 10}, {NULL}};
const Segment SGI92879[] = {{&S81260[59], 27}, {NULL}};
const Segment SGI92880[] = {{&S81260[45], 6}, {NULL}};
const Segment SGI92881[] = {{&S81260[134], 3}, {NULL}};
const Segment SGI92882[] = {{&S81260[100], 5}, {NULL}};
const Segment SGI92883[] = {{&S81260[0], 14}, {NULL}};
const Segment SGI92884[] = {{&S81261[44], 10}, {NULL}};
const Segment SGI92885[] = {{&S81261[44], 22}, {NULL}};
const Segment SGI92886[] = {{&S81261[30], 6}, {NULL}};
const Segment SGI92887[] = {{&S81261[114], 3}, {NULL}};
const Segment SGI92888[] = {{&S81261[80], 5}, {NULL}};
const Segment SGI92889[] = {{&S81262[44], 10}, {NULL}};
const Segment SGI92890[] = {{&S81262[44], 22}, {NULL}};
const Segment SGI92891[] = {{&S81262[30], 6}, {NULL}};
const Segment SGI92892[] = {{&S81262[83], 6}, {NULL}};
const Segment SGI92893[] = {{&S73657[44], 22}, {NULL}};
const Segment SGI92894[] = {{&S73657[30], 6}, {NULL}};
const Segment SGI92895[] = {{&S73657[114], 3}, {NULL}};
const Segment SGI92896[] = {{&S73657[80], 5}, {NULL}};
const Segment SGI92897[] = {{&S59929[34], 18}, {NULL}};
const Segment SGI92898[] = {{&S81277[44], 11}, {NULL}};
const Segment SGI92899[] = {{&S59929[27], 6}, {NULL}};
const Segment SGI92900[] = {{&S81277[44], 23}, {NULL}};
const Segment SGI92901[] = {{&S81277[30], 6}, {NULL}};
const Segment SGI92902[] = {{&S81277[115], 3}, {NULL}};
const Segment SGI92903[] = {{&S59929[105], 9}, {NULL}};
const Segment SGI92904[] = {{&S59929[78], 6}, {NULL}};
const Segment SGI92905[] = {{&S81277[81], 5}, {NULL}};
const Segment SGI92906[] = {{&S59930[34], 10}, {NULL}};
const Segment SGI92907[] = {{&S59930[27], 6}, {NULL}};
const Segment SGI92908[] = {{&S59930[97], 9}, {NULL}};
const Segment SGI92909[] = {{&S59930[70], 6}, {NULL}};
const Segment SGI92910[] = {{&S73679[44], 9}, {NULL}};
const Segment SGI92911[] = {{&S73679[44], 21}, {NULL}};
const Segment SGI92912[] = {{&S73679[30], 6}, {NULL}};
const Segment SGI92913[] = {{&S73679[132], 3}, {NULL}};
const Segment SGI92914[] = {{&S73679[98], 5}, {NULL}};
const Segment SGI92915[] = {{&S73679[35], 1}, {NULL}};
const Segment SGI92916[] = {{&S73679[67], 11}, {NULL}};
const Segment SGI92917[] = {{&S73680[44], 9}, {NULL}};
const Segment SGI92918[] = {{&S73680[44], 20}, {NULL}};
const Segment SGI92919[] = {{&S73680[30], 6}, {NULL}};
const Segment SGI92920[] = {{&S73680[131], 3}, {NULL}};
const Segment SGI92921[] = {{&S73680[97], 5}, {NULL}};
const Segment SGI92922[] = {{&S73680[35], 1}, {NULL}};
const Segment SGI92923[] = {{&S73680[66], 11}, {NULL}};
const Segment SGI92924[] = {{&S59948[44], 10}, {NULL}};
const Segment SGI92925[] = {{&S59948[44], 24}, {NULL}};
const Segment SGI92926[] = {{&S59948[30], 6}, {NULL}};
const Segment SGI92927[] = {{&S59948[31], 3}, {NULL}};
const Segment SGI92928[] = {{&S59948[82], 6}, {NULL}};
const Segment SGI92929[] = {{&S59949[44], 12}, {NULL}};
const Segment SGI92930[] = {{&S59949[44], 28}, {NULL}};
const Segment SGI92931[] = {{&S59949[30], 6}, {NULL}};
const Segment SGI92932[] = {{&S59949[31], 3}, {NULL}};
const Segment SGI92933[] = {{&S59949[86], 6}, {NULL}};
const Segment SGI92934[] = {{&S59958[34], 14}, {NULL}};
const Segment SGI92935[] = {{&S59958[27], 6}, {NULL}};
const Segment SGI92936[] = {{&S59958[101], 9}, {NULL}};
const Segment SGI92937[] = {{&S59958[74], 6}, {NULL}};
const Segment SGI92938[] = {{&S59959[34], 14}, {NULL}};
const Segment SGI92939[] = {{&S59959[27], 6}, {NULL}};
const Segment SGI92940[] = {{&S59959[101], 9}, {NULL}};
const Segment SGI92941[] = {{&S59959[74], 6}, {NULL}};
const Segment SGI92942[] = {{&S59960[34], 10}, {NULL}};
const Segment SGI92943[] = {{&S59960[27], 6}, {NULL}};
const Segment SGI92944[] = {{&S59960[97], 9}, {NULL}};
const Segment SGI92945[] = {{&S59960[70], 6}, {NULL}};
const Segment SGI92946[] = {{&S81324[44], 26}, {NULL}};
const Segment SGI92947[] = {{&S81324[30], 6}, {NULL}};
const Segment SGI92948[] = {{&S81324[136], 3}, {NULL}};
const Segment SGI92949[] = {{&S81324[102], 5}, {NULL}};
const Segment SGI92950[] = {{&S81325[44], 26}, {NULL}};
const Segment SGI92951[] = {{&S81325[30], 6}, {NULL}};
const Segment SGI92952[] = {{&S81325[67], 3}, {NULL}};
const Segment SGI92953[] = {{&S81325[102], 5}, {NULL}};
const Segment SGI92954[] = {{&S59982[44], 18}, {NULL}};
const Segment SGI92955[] = {{&S59982[44], 34}, {NULL}};
const Segment SGI92956[] = {{&S59982[30], 6}, {NULL}};
const Segment SGI92957[] = {{&S59982[31], 3}, {NULL}};
const Segment SGI92958[] = {{&S59982[92], 6}, {NULL}};
const Segment SGI92959[] = {{&S59983[44], 18}, {NULL}};
const Segment SGI92960[] = {{&S59983[44], 34}, {NULL}};
const Segment SGI92961[] = {{&S59983[30], 6}, {NULL}};
const Segment SGI92962[] = {{&S59983[31], 3}, {NULL}};
const Segment SGI92963[] = {{&S59983[92], 6}, {NULL}};
const Segment SGI92964[] = {{&S73735[44], 7}, {NULL}};
const Segment SGI92965[] = {{&S73735[44], 24}, {NULL}};
const Segment SGI92966[] = {{&S73735[30], 6}, {NULL}};
const Segment SGI92967[] = {{&S59984[44], 18}, {NULL}};
const Segment SGI92968[] = {{&S59984[44], 34}, {NULL}};
const Segment SGI92969[] = {{&S59984[30], 6}, {NULL}};
const Segment SGI92970[] = {{&S73735[82], 6}, {NULL}};
const Segment SGI92971[] = {{&S59984[31], 3}, {NULL}};
const Segment SGI92972[] = {{&S73735[109], 13}, {NULL}};
const Segment SGI92973[] = {{&S59984[92], 6}, {NULL}};
const Segment SGI92974[] = {{&S73736[44], 7}, {NULL}};
const Segment SGI92975[] = {{&S59985[44], 18}, {NULL}};
const Segment SGI92976[] = {{&S73736[44], 24}, {NULL}};
const Segment SGI92977[] = {{&S59985[44], 34}, {NULL}};
const Segment SGI92978[] = {{&S73736[30], 6}, {NULL}};
const Segment SGI92979[] = {{&S59985[30], 6}, {NULL}};
const Segment SGI92980[] = {{&S59985[31], 3}, {NULL}};
const Segment SGI92981[] = {{&S73736[82], 6}, {NULL}};
const Segment SGI92982[] = {{&S59985[92], 6}, {NULL}};
const Segment SGI92983[] = {{&S73736[109], 13}, {NULL}};
const Segment SGI92984[] = {{&S73752[44], 9}, {NULL}};
const Segment SGI92985[] = {{&S73752[44], 20}, {NULL}};
const Segment SGI92986[] = {{&S73752[30], 6}, {NULL}};
const Segment SGI92987[] = {{&S73752[112], 3}, {NULL}};
const Segment SGI92988[] = {{&S81357[44], 16}, {NULL}};
const Segment SGI92989[] = {{&S73752[78], 5}, {NULL}};
const Segment SGI92990[] = {{&S81357[44], 28}, {NULL}};
const Segment SGI92991[] = {{&S81357[30], 6}, {NULL}};
const Segment SGI92992[] = {{&S81357[120], 3}, {NULL}};
const Segment SGI92993[] = {{&S81357[86], 5}, {NULL}};
const Segment SGI92994[] = {{&S73754[38], 9}, {NULL}};
const Segment SGI92995[] = {{&S73754[31], 6}, {NULL}};
const Segment SGI92996[] = {{&S60009[34], 14}, {NULL}};
const Segment SGI92997[] = {{&S60009[27], 6}, {NULL}};
const Segment SGI92998[] = {{&S60009[106], 9}, {NULL}};
const Segment SGI92999[] = {{&S60009[79], 6}, {NULL}};
const Segment SGI93000[] = {{&S81407[59], 4}, {NULL}};
const Segment SGI93001[] = {{&S81407[59], 16}, {NULL}};
const Segment SGI93002[] = {{&S81407[45], 6}, {NULL}};
const Segment SGI93003[] = {{&S81407[123], 3}, {NULL}};
const Segment SGI93004[] = {{&S81407[89], 5}, {NULL}};
const Segment SGI93005[] = {{&S81407[0], 14}, {NULL}};
const Segment SGI93006[] = {{&S73817[44], 5}, {NULL}};
const Segment SGI93007[] = {{&S73817[44], 21}, {NULL}};
const Segment SGI93008[] = {{&S73817[30], 6}, {NULL}};
const Segment SGI93009[] = {{&S73817[113], 3}, {NULL}};
const Segment SGI93010[] = {{&S73817[79], 5}, {NULL}};
const Segment SGI93011[] = {{&S73825[44], 10}, {NULL}};
const Segment SGI93012[] = {{&S73825[44], 30}, {NULL}};
const Segment SGI93013[] = {{&S73825[30], 6}, {NULL}};
const Segment SGI93014[] = {{&S73825[71], 3}, {NULL}};
const Segment SGI93015[] = {{&S73825[88], 5}, {NULL}};
const Segment SGI93016[] = {{&S73826[44], 10}, {NULL}};
const Segment SGI93017[] = {{&S73826[44], 32}, {NULL}};
const Segment SGI93018[] = {{&S73826[30], 6}, {NULL}};
const Segment SGI93019[] = {{&S73826[71], 3}, {NULL}};
const Segment SGI93020[] = {{&S73826[90], 5}, {NULL}};
const Segment SGI93021[] = {{&S73827[44], 10}, {NULL}};
const Segment SGI93022[] = {{&S73827[44], 32}, {NULL}};
const Segment SGI93023[] = {{&S73827[30], 6}, {NULL}};
const Segment SGI93024[] = {{&S73827[71], 3}, {NULL}};
const Segment SGI93025[] = {{&S73827[90], 5}, {NULL}};
const Segment SGI93026[] = {{&S73828[44], 10}, {NULL}};
const Segment SGI93027[] = {{&S73828[44], 32}, {NULL}};
const Segment SGI93028[] = {{&S73828[30], 6}, {NULL}};
const Segment SGI93029[] = {{&S73828[71], 3}, {NULL}};
const Segment SGI93030[] = {{&S73828[90], 5}, {NULL}};
const Segment SGI93031[] = {{&S73413[32], 10}, {NULL}};
const Segment SGI93032[] = {{&S73413[27], 4}, {NULL}};
const Segment SGI93033[] = {{&S73413[102], 2}, {NULL}};
const Segment SGI93034[] = {{&S73413[68], 6}, {NULL}};
const Segment SGI93035[] = {{&S60109[44], 15}, {NULL}};
const Segment SGI93036[] = {{&S60109[44], 27}, {NULL}};
const Segment SGI93037[] = {{&S60109[30], 6}, {NULL}};
const Segment SGI93038[] = {{&S73413[130], 8}, {NULL}};
const Segment SGI93039[] = {{&S60109[31], 3}, {NULL}};
const Segment SGI93040[] = {{&S60109[85], 6}, {NULL}};
const Segment SGI93041[] = {{&S73414[32], 10}, {NULL}};
const Segment SGI93042[] = {{&S73414[27], 4}, {NULL}};
const Segment SGI93043[] = {{&S73414[102], 2}, {NULL}};
const Segment SGI93044[] = {{&S73414[68], 6}, {NULL}};
const Segment SGI93045[] = {{&S73414[129], 8}, {NULL}};
const Segment SGI93046[] = {{&S73855[44], 10}, {NULL}};
const Segment SGI93047[] = {{&S73855[44], 21}, {NULL}};
const Segment SGI93048[] = {{&S73855[30], 6}, {NULL}};
const Segment SGI93049[] = {{&S73855[132], 3}, {NULL}};
const Segment SGI93050[] = {{&S73855[98], 5}, {NULL}};
const Segment SGI93051[] = {{&S73855[35], 1}, {NULL}};
const Segment SGI93052[] = {{&S73855[67], 11}, {NULL}};
const Segment SGI93053[] = {{&S73856[44], 10}, {NULL}};
const Segment SGI93054[] = {{&S73856[44], 27}, {NULL}};
const Segment SGI93055[] = {{&S73856[30], 6}, {NULL}};
const Segment SGI93056[] = {{&S73856[138], 3}, {NULL}};
const Segment SGI93057[] = {{&S73856[104], 5}, {NULL}};
const Segment SGI93058[] = {{&S73856[35], 1}, {NULL}};
const Segment SGI93059[] = {{&S73856[73], 11}, {NULL}};
const Segment SGI93060[] = {{&S60134[34], 10}, {NULL}};
const Segment SGI93061[] = {{&S60134[27], 6}, {NULL}};
const Segment SGI93062[] = {{&S60134[97], 9}, {NULL}};
const Segment SGI93063[] = {{&S60134[70], 6}, {NULL}};
const Segment SGI93064[] = {{&S60153[34], 9}, {NULL}};
const Segment SGI93065[] = {{&S60153[27], 6}, {NULL}};
const Segment SGI93066[] = {{&S60153[96], 9}, {NULL}};
const Segment SGI93067[] = {{&S60153[69], 6}, {NULL}};
const Segment SGI93068[] = {{&S60154[34], 9}, {NULL}};
const Segment SGI93069[] = {{&S60154[27], 6}, {NULL}};
const Segment SGI93070[] = {{&S60154[96], 9}, {NULL}};
const Segment SGI93071[] = {{&S60154[69], 6}, {NULL}};
const Segment SGI93072[] = {{&S60167[34], 9}, {NULL}};
const Segment SGI93073[] = {{&S60167[27], 6}, {NULL}};
const Segment SGI93074[] = {{&S60167[96], 9}, {NULL}};
const Segment SGI93075[] = {{&S60167[69], 6}, {NULL}};
const Segment SGI93076[] = {{&S60168[34], 9}, {NULL}};
const Segment SGI93077[] = {{&S60168[27], 6}, {NULL}};
const Segment SGI93078[] = {{&S60168[96], 9}, {NULL}};
const Segment SGI93079[] = {{&S60168[69], 6}, {NULL}};
const Segment SGI93080[] = {{&S60173[34], 6}, {NULL}};
const Segment SGI93081[] = {{&S60173[27], 6}, {NULL}};
const Segment SGI93082[] = {{&S60173[93], 9}, {NULL}};
const Segment SGI93083[] = {{&S60173[66], 6}, {NULL}};
const Segment SGI93084[] = {{&S60174[34], 6}, {NULL}};
const Segment SGI93085[] = {{&S60174[27], 6}, {NULL}};
const Segment SGI93086[] = {{&S60174[93], 9}, {NULL}};
const Segment SGI93087[] = {{&S60174[66], 6}, {NULL}};
const Segment SGI93088[] = {{&S81533[59], 10}, {NULL}};
const Segment SGI93089[] = {{&S81533[59], 27}, {NULL}};
const Segment SGI93090[] = {{&S81533[45], 6}, {NULL}};
const Segment SGI93091[] = {{&S81533[134], 3}, {NULL}};
const Segment SGI93092[] = {{&S81533[100], 5}, {NULL}};
const Segment SGI93093[] = {{&S81533[0], 14}, {NULL}};
const Segment SGI93094[] = {{&S81534[59], 11}, {NULL}};
const Segment SGI93095[] = {{&S81534[59], 28}, {NULL}};
const Segment SGI93096[] = {{&S81534[45], 6}, {NULL}};
const Segment SGI93097[] = {{&S81534[135], 3}, {NULL}};
const Segment SGI93098[] = {{&S81534[101], 5}, {NULL}};
const Segment SGI93099[] = {{&S81534[0], 14}, {NULL}};
const Segment SGI93100[] = {{&S81538[44], 14}, {NULL}};
const Segment SGI93101[] = {{&S81538[44], 26}, {NULL}};
const Segment SGI93102[] = {{&S81538[30], 6}, {NULL}};
const Segment SGI93103[] = {{&S81538[118], 3}, {NULL}};
const Segment SGI93104[] = {{&S81538[84], 5}, {NULL}};
const Segment SGI93105[] = {{&S60199[44], 12}, {NULL}};
const Segment SGI93106[] = {{&S60199[44], 24}, {NULL}};
const Segment SGI93107[] = {{&S60199[30], 6}, {NULL}};
const Segment SGI93108[] = {{&S60199[31], 3}, {NULL}};
const Segment SGI93109[] = {{&S60199[82], 6}, {NULL}};
const Segment SGI93110[] = {{&S73946[44], 11}, {NULL}};
const Segment SGI93111[] = {{&S73946[44], 22}, {NULL}};
const Segment SGI93112[] = {{&S73946[30], 6}, {NULL}};
const Segment SGI93113[] = {{&S73946[83], 6}, {NULL}};
const Segment SGI93114[] = {{&S60203[44], 8}, {NULL}};
const Segment SGI93115[] = {{&S60203[44], 20}, {NULL}};
const Segment SGI93116[] = {{&S60203[30], 6}, {NULL}};
const Segment SGI93117[] = {{&S60203[31], 3}, {NULL}};
const Segment SGI93118[] = {{&S60203[78], 6}, {NULL}};
const Segment SGI93119[] = {{&S73947[44], 22}, {NULL}};
const Segment SGI93120[] = {{&S73947[30], 6}, {NULL}};
const Segment SGI93121[] = {{&S60213[34], 18}, {NULL}};
const Segment SGI93122[] = {{&S60213[27], 6}, {NULL}};
const Segment SGI93123[] = {{&S60213[105], 9}, {NULL}};
const Segment SGI93124[] = {{&S81561[44], 12}, {NULL}};
const Segment SGI93125[] = {{&S60213[78], 6}, {NULL}};
const Segment SGI93126[] = {{&S81561[44], 24}, {NULL}};
const Segment SGI93127[] = {{&S81561[30], 6}, {NULL}};
const Segment SGI93128[] = {{&S81561[116], 3}, {NULL}};
const Segment SGI93129[] = {{&S81561[82], 5}, {NULL}};
const Segment SGI93130[] = {{&S60220[34], 18}, {NULL}};
const Segment SGI93131[] = {{&S60220[27], 6}, {NULL}};
const Segment SGI93132[] = {{&S60220[105], 9}, {NULL}};
const Segment SGI93133[] = {{&S60220[78], 6}, {NULL}};
const Segment SGI93134[] = {{&S60229[34], 9}, {NULL}};
const Segment SGI93135[] = {{&S60229[27], 6}, {NULL}};
const Segment SGI93136[] = {{&S60229[96], 9}, {NULL}};
const Segment SGI93137[] = {{&S60229[69], 6}, {NULL}};
const Segment SGI93138[] = {{&S60230[34], 10}, {NULL}};
const Segment SGI93139[] = {{&S60230[27], 6}, {NULL}};
const Segment SGI93140[] = {{&S60230[97], 9}, {NULL}};
const Segment SGI93141[] = {{&S60230[70], 6}, {NULL}};
const Segment SGI93142[] = {{&S60231[34], 9}, {NULL}};
const Segment SGI93143[] = {{&S60231[27], 6}, {NULL}};
const Segment SGI93144[] = {{&S60231[96], 9}, {NULL}};
const Segment SGI93145[] = {{&S60231[69], 6}, {NULL}};
const Segment SGI93146[] = {{&S81622[45], 10}, {NULL}};
const Segment SGI93147[] = {{&S81622[30], 6}, {NULL}};
const Segment SGI93148[] = {{&S81622[120], 3}, {NULL}};
const Segment SGI93149[] = {{&S81622[86], 5}, {NULL}};
const Segment SGI93150[] = {{&S81622[144], 13}, {NULL}};
const Segment SGI93151[] = {{&S74025[38], 10}, {NULL}};
const Segment SGI93152[] = {{&S74025[31], 6}, {NULL}};
const Segment SGI93153[] = {{&S68175[76], 6}, {NULL}};
const Segment SGI93154[] = {{&S74026[44], 10}, {NULL}};
const Segment SGI93155[] = {{&S68175[0], 11}, {NULL}};
const Segment SGI93156[] = {{&S74026[44], 20}, {NULL}};
const Segment SGI93157[] = {{&S74026[30], 6}, {NULL}};
const Segment SGI93158[] = {{&S74026[112], 3}, {NULL}};
const Segment SGI93159[] = {{&S74026[78], 5}, {NULL}};
const Segment SGI93160[] = {{&S81632[44], 4}, {NULL}};
const Segment SGI93161[] = {{&S81632[44], 16}, {NULL}};
const Segment SGI93162[] = {{&S81632[30], 6}, {NULL}};
const Segment SGI93163[] = {{&S81632[108], 3}, {NULL}};
const Segment SGI93164[] = {{&S81632[74], 5}, {NULL}};
const Segment SGI93165[] = {{&S74035[44], 75}, {NULL}};
const Segment SGI93166[] = {{&S74035[44], 96}, {NULL}};
const Segment SGI93167[] = {{&S74035[30], 6}, {NULL}};
const Segment SGI93168[] = {{&S74035[187], 3}, {NULL}};
const Segment SGI93169[] = {{&S74035[113], 5}, {NULL}};
const Segment SGI93170[] = {{&S60346[34], 12}, {NULL}};
const Segment SGI93171[] = {{&S60346[27], 6}, {NULL}};
const Segment SGI93172[] = {{&S60346[99], 9}, {NULL}};
const Segment SGI93173[] = {{&S60346[72], 6}, {NULL}};
const Segment SGI93174[] = {{&S81691[45], 10}, {NULL}};
const Segment SGI93175[] = {{&S81691[30], 6}, {NULL}};
const Segment SGI93176[] = {{&S81691[120], 3}, {NULL}};
const Segment SGI93177[] = {{&S81691[86], 5}, {NULL}};
const Segment SGI93178[] = {{&S81691[144], 13}, {NULL}};
const Segment SGI93179[] = {{&S60380[44], 13}, {NULL}};
const Segment SGI93180[] = {{&S60380[44], 25}, {NULL}};
const Segment SGI93181[] = {{&S60380[30], 6}, {NULL}};
const Segment SGI93182[] = {{&S60380[31], 3}, {NULL}};
const Segment SGI93183[] = {{&S60380[83], 6}, {NULL}};
const Segment SGI93184[] = {{&S60382[38], 13}, {NULL}};
const Segment SGI93185[] = {{&S60382[31], 6}, {NULL}};
const Segment SGI93186[] = {{&S60383[44], 13}, {NULL}};
const Segment SGI93187[] = {{&S60383[44], 25}, {NULL}};
const Segment SGI93188[] = {{&S60383[30], 6}, {NULL}};
const Segment SGI93189[] = {{&S81728[44], 10}, {NULL}};
const Segment SGI93190[] = {{&S81728[44], 27}, {NULL}};
const Segment SGI93191[] = {{&S81728[30], 6}, {NULL}};
const Segment SGI93192[] = {{&S60383[127], 3}, {NULL}};
const Segment SGI93193[] = {{&S60383[92], 6}, {NULL}};
const Segment SGI93194[] = {{&S60383[147], 8}, {NULL}};
const Segment SGI93195[] = {{&S81728[85], 6}, {NULL}};
const Segment SGI93196[] = {{&S81728[112], 13}, {NULL}};
const Segment SGI93197[] = {{&S81751[44], 10}, {NULL}};
const Segment SGI93198[] = {{&S81751[44], 27}, {NULL}};
const Segment SGI93199[] = {{&S81751[30], 6}, {NULL}};
const Segment SGI93200[] = {{&S81751[149], 4}, {NULL}};
const Segment SGI93201[] = {{&S81751[175], 3}, {NULL}};
const Segment SGI93202[] = {{&S81751[119], 3}, {&S81751[224], 5}, {NULL}};
const Segment SGI93203[] = {{&S81751[85], 5}, {NULL}};
const Segment SGI93204[] = {{&S60411[34], 14}, {NULL}};
const Segment SGI93205[] = {{&S60411[27], 6}, {NULL}};
const Segment SGI93206[] = {{&S60411[101], 9}, {NULL}};
const Segment SGI93207[] = {{&S60411[74], 6}, {NULL}};
const Segment SGI93208[] = {{&S81755[57], 7}, {NULL}};
const Segment SGI93209[] = {{&S81755[57], 23}, {NULL}};
const Segment SGI93210[] = {{&S81755[43], 6}, {NULL}};
const Segment SGI93211[] = {{&S81755[128], 3}, {NULL}};
const Segment SGI93212[] = {{&S60412[34], 14}, {NULL}};
const Segment SGI93213[] = {{&S60412[27], 6}, {NULL}};
const Segment SGI93214[] = {{&S81755[94], 5}, {NULL}};
const Segment SGI93215[] = {{&S60412[101], 9}, {NULL}};
const Segment SGI93216[] = {{&S60412[74], 6}, {NULL}};
const Segment SGI93217[] = {{&S60413[34], 10}, {NULL}};
const Segment SGI93218[] = {{&S60413[27], 6}, {NULL}};
const Segment SGI93219[] = {{&S60413[97], 9}, {NULL}};
const Segment SGI93220[] = {{&S60413[70], 6}, {NULL}};
const Segment SGI93221[] = {{&S81813[45], 5}, {NULL}};
const Segment SGI93222[] = {{&S81813[30], 6}, {NULL}};
const Segment SGI93223[] = {{&S81813[110], 3}, {NULL}};
const Segment SGI93224[] = {{&S81813[76], 5}, {NULL}};
const Segment SGI93225[] = {{&S81813[134], 13}, {NULL}};
const Segment SGI93226[] = {{&S81814[44], 5}, {NULL}};
const Segment SGI93227[] = {{&S81814[44], 17}, {NULL}};
const Segment SGI93228[] = {{&S81814[30], 6}, {NULL}};
const Segment SGI93229[] = {{&S81814[110], 3}, {NULL}};
const Segment SGI93230[] = {{&S81814[75], 6}, {NULL}};
const Segment SGI93231[] = {{&S81814[130], 8}, {NULL}};
const Segment SGI93232[] = {{&S81814[139], 13}, {NULL}};
const Segment SGI93233[] = {{&S81833[44], 14}, {NULL}};
const Segment SGI93234[] = {{&S81833[44], 26}, {NULL}};
const Segment SGI93235[] = {{&S81833[30], 6}, {NULL}};
const Segment SGI93236[] = {{&S81833[84], 6}, {NULL}};
const Segment SGI93237[] = {{&S81833[111], 13}, {NULL}};
const Segment SGI93238[] = {{&S60487[34], 9}, {NULL}};
const Segment SGI93239[] = {{&S60487[27], 6}, {NULL}};
const Segment SGI93240[] = {{&S60487[96], 9}, {NULL}};
const Segment SGI93241[] = {{&S60487[69], 6}, {NULL}};
const Segment SGI93242[] = {{&S60488[34], 9}, {NULL}};
const Segment SGI93243[] = {{&S60488[27], 6}, {NULL}};
const Segment SGI93244[] = {{&S60488[96], 9}, {NULL}};
const Segment SGI93245[] = {{&S60488[69], 6}, {NULL}};
const Segment SGI93246[] = {{&S60489[34], 10}, {NULL}};
const Segment SGI93247[] = {{&S60489[27], 6}, {NULL}};
const Segment SGI93248[] = {{&S60489[97], 9}, {NULL}};
const Segment SGI93249[] = {{&S60489[70], 6}, {NULL}};
const Segment SGI93250[] = {{&S60490[34], 10}, {NULL}};
const Segment SGI93251[] = {{&S60490[27], 6}, {NULL}};
const Segment SGI93252[] = {{&S60490[97], 9}, {NULL}};
const Segment SGI93253[] = {{&S60490[70], 6}, {NULL}};
const Segment SGI93254[] = {{&S60491[34], 10}, {NULL}};
const Segment SGI93255[] = {{&S60491[27], 6}, {NULL}};
const Segment SGI93256[] = {{&S60491[97], 9}, {NULL}};
const Segment SGI93257[] = {{&S60491[70], 6}, {NULL}};
const Segment SGI93258[] = {{&S81889[31], 8}, {NULL}};
const Segment SGI93259[] = {{&S81889[54], 3}, {NULL}};
const Segment SGI93260[] = {{&S81894[45], 10}, {NULL}};
const Segment SGI93261[] = {{&S81894[30], 6}, {NULL}};
const Segment SGI93262[] = {{&S81894[120], 3}, {NULL}};
const Segment SGI93263[] = {{&S81894[86], 5}, {NULL}};
const Segment SGI93264[] = {{&S81894[144], 13}, {NULL}};
const Segment SGI93265[] = {{&S81902[44], 10}, {NULL}};
const Segment SGI93266[] = {{&S81902[44], 27}, {NULL}};
const Segment SGI93267[] = {{&S81902[30], 6}, {NULL}};
const Segment SGI93268[] = {{&S81902[119], 3}, {NULL}};
const Segment SGI93269[] = {{&S81902[85], 5}, {NULL}};
const Segment SGI93270[] = {{&S81902[142], 10}, {NULL}};
const Segment SGI93271[] = {{&S60514[44], 11}, {NULL}};
const Segment SGI93272[] = {{&S60514[44], 23}, {NULL}};
const Segment SGI93273[] = {{&S60514[30], 6}, {NULL}};
const Segment SGI93274[] = {{&S60514[31], 3}, {NULL}};
const Segment SGI93275[] = {{&S60514[81], 6}, {NULL}};
const Segment SGI93276[] = {{&S60515[45], 11}, {NULL}};
const Segment SGI93277[] = {{&S60515[30], 6}, {NULL}};
const Segment SGI93278[] = {{&S60515[31], 3}, {NULL}};
const Segment SGI93279[] = {{&S60515[82], 6}, {NULL}};
const Segment SGI93280[] = {{&S74346[44], 12}, {NULL}};
const Segment SGI93281[] = {{&S74346[44], 23}, {NULL}};
const Segment SGI93282[] = {{&S74346[30], 6}, {NULL}};
const Segment SGI93283[] = {{&S74346[115], 3}, {NULL}};
const Segment SGI93284[] = {{&S74346[81], 5}, {NULL}};
const Segment SGI93285[] = {{&S60569[44], 10}, {NULL}};
const Segment SGI93286[] = {{&S60569[44], 21}, {NULL}};
const Segment SGI93287[] = {{&S60569[30], 6}, {NULL}};
const Segment SGI93288[] = {{&S60569[31], 3}, {NULL}};
const Segment SGI93289[] = {{&S60569[79], 6}, {NULL}};
const Segment SGI93290[] = {{&S81962[44], 10}, {NULL}};
const Segment SGI93291[] = {{&S81962[44], 22}, {NULL}};
const Segment SGI93292[] = {{&S81962[30], 6}, {NULL}};
const Segment SGI93293[] = {{&S81962[114], 3}, {NULL}};
const Segment SGI93294[] = {{&S81962[80], 5}, {NULL}};
const Segment SGI93295[] = {{&S60570[44], 10}, {NULL}};
const Segment SGI93296[] = {{&S60570[44], 21}, {NULL}};
const Segment SGI93297[] = {{&S60570[30], 6}, {NULL}};
const Segment SGI93298[] = {{&S60570[31], 3}, {NULL}};
const Segment SGI93299[] = {{&S60570[79], 6}, {NULL}};
const Segment SGI93300[] = {{&S81963[44], 6}, {NULL}};
const Segment SGI93301[] = {{&S81963[44], 18}, {NULL}};
const Segment SGI93302[] = {{&S81963[30], 6}, {NULL}};
const Segment SGI93303[] = {{&S81963[79], 6}, {NULL}};
const Segment SGI93304[] = {{&S60571[44], 10}, {NULL}};
const Segment SGI93305[] = {{&S60571[44], 21}, {NULL}};
const Segment SGI93306[] = {{&S60571[30], 6}, {NULL}};
const Segment SGI93307[] = {{&S60571[31], 3}, {NULL}};
const Segment SGI93308[] = {{&S60571[79], 6}, {NULL}};
const Segment SGI93309[] = {{&S60572[44], 10}, {NULL}};
const Segment SGI93310[] = {{&S60572[44], 21}, {NULL}};
const Segment SGI93311[] = {{&S60572[30], 6}, {NULL}};
const Segment SGI93312[] = {{&S60572[31], 3}, {NULL}};
const Segment SGI93313[] = {{&S60572[79], 6}, {NULL}};
const Segment SGI93314[] = {{&S60573[44], 10}, {NULL}};
const Segment SGI93315[] = {{&S60573[44], 21}, {NULL}};
const Segment SGI93316[] = {{&S60573[30], 6}, {NULL}};
const Segment SGI93317[] = {{&S60573[31], 3}, {NULL}};
const Segment SGI93318[] = {{&S60573[79], 6}, {NULL}};
const Segment SGI93319[] = {{&S60574[38], 10}, {NULL}};
const Segment SGI93320[] = {{&S60574[31], 6}, {NULL}};
const Segment SGI93321[] = {{&S81970[38], 7}, {NULL}};
const Segment SGI93322[] = {{&S81970[31], 6}, {NULL}};
const Segment SGI93323[] = {{&S60575[44], 10}, {NULL}};
const Segment SGI93324[] = {{&S60575[44], 21}, {NULL}};
const Segment SGI93325[] = {{&S60575[30], 6}, {NULL}};
const Segment SGI93326[] = {{&S60575[31], 3}, {NULL}};
const Segment SGI93327[] = {{&S60575[79], 6}, {NULL}};
const Segment SGI93328[] = {{&S81971[44], 7}, {NULL}};
const Segment SGI93329[] = {{&S81971[44], 15}, {NULL}};
const Segment SGI93330[] = {{&S81971[30], 6}, {NULL}};
const Segment SGI93331[] = {{&S81971[107], 3}, {NULL}};
const Segment SGI93332[] = {{&S81971[73], 5}, {NULL}};
const Segment SGI93333[] = {{&S60576[44], 10}, {NULL}};
const Segment SGI93334[] = {{&S60576[44], 21}, {NULL}};
const Segment SGI93335[] = {{&S60576[30], 6}, {NULL}};
const Segment SGI93336[] = {{&S60576[31], 3}, {NULL}};
const Segment SGI93337[] = {{&S60576[79], 6}, {NULL}};
const Segment SGI93338[] = {{&S60577[44], 10}, {NULL}};
const Segment SGI93339[] = {{&S60577[44], 21}, {NULL}};
const Segment SGI93340[] = {{&S60577[30], 6}, {NULL}};
const Segment SGI93341[] = {{&S60577[31], 3}, {NULL}};
const Segment SGI93342[] = {{&S60577[79], 6}, {NULL}};
const Segment SGI93343[] = {{&S60601[34], 6}, {NULL}};
const Segment SGI93344[] = {{&S60601[27], 6}, {NULL}};
const Segment SGI93345[] = {{&S60601[93], 9}, {NULL}};
const Segment SGI93346[] = {{&S60601[66], 6}, {NULL}};
const Segment SGI93347[] = {{&S60602[34], 6}, {NULL}};
const Segment SGI93348[] = {{&S60602[27], 6}, {NULL}};
const Segment SGI93349[] = {{&S60602[93], 9}, {NULL}};
const Segment SGI93350[] = {{&S60602[66], 6}, {NULL}};
const Segment SGI93351[] = {{&S82005[44], 10}, {NULL}};
const Segment SGI93352[] = {{&S82005[44], 27}, {NULL}};
const Segment SGI93353[] = {{&S82005[30], 6}, {NULL}};
const Segment SGI93354[] = {{&S82005[85], 6}, {NULL}};
const Segment SGI93355[] = {{&S82005[112], 13}, {NULL}};
const Segment SGI93356[] = {{&S60615[44], 7}, {NULL}};
const Segment SGI93357[] = {{&S60615[44], 19}, {NULL}};
const Segment SGI93358[] = {{&S60615[30], 6}, {NULL}};
const Segment SGI93359[] = {{&S60615[31], 3}, {NULL}};
const Segment SGI93360[] = {{&S60615[77], 6}, {NULL}};
const Segment SGI93361[] = {{&S60643[34], 9}, {NULL}};
const Segment SGI93362[] = {{&S60643[27], 6}, {NULL}};
const Segment SGI93363[] = {{&S60643[96], 9}, {NULL}};
const Segment SGI93364[] = {{&S60643[69], 6}, {NULL}};
const Segment SGI93365[] = {{&S60644[34], 9}, {NULL}};
const Segment SGI93366[] = {{&S60644[27], 6}, {NULL}};
const Segment SGI93367[] = {{&S60644[96], 9}, {NULL}};
const Segment SGI93368[] = {{&S60644[69], 6}, {NULL}};
const Segment SGI93369[] = {{&S60649[34], 41}, {NULL}};
const Segment SGI93370[] = {{&S60649[27], 6}, {NULL}};
const Segment SGI93371[] = {{&S60649[128], 9}, {NULL}};
const Segment SGI93372[] = {{&S60649[101], 6}, {NULL}};
const Segment SGI93373[] = {{&S60650[34], 41}, {NULL}};
const Segment SGI93374[] = {{&S60650[27], 6}, {NULL}};
const Segment SGI93375[] = {{&S60650[128], 9}, {NULL}};
const Segment SGI93376[] = {{&S60650[101], 6}, {NULL}};
const Segment SGI93377[] = {{&S60658[34], 41}, {NULL}};
const Segment SGI93378[] = {{&S60658[27], 6}, {NULL}};
const Segment SGI93379[] = {{&S60658[128], 9}, {NULL}};
const Segment SGI93380[] = {{&S60658[101], 6}, {NULL}};
const Segment SGI93381[] = {{&S60659[34], 41}, {NULL}};
const Segment SGI93382[] = {{&S60659[27], 6}, {NULL}};
const Segment SGI93383[] = {{&S60659[128], 9}, {NULL}};
const Segment SGI93384[] = {{&S60659[101], 6}, {NULL}};
const Segment SGI93385[] = {{&S74476[44], 9}, {NULL}};
const Segment SGI93386[] = {{&S74476[44], 20}, {NULL}};
const Segment SGI93387[] = {{&S74476[30], 6}, {NULL}};
const Segment SGI93388[] = {{&S74476[131], 3}, {NULL}};
const Segment SGI93389[] = {{&S74476[97], 5}, {NULL}};
const Segment SGI93390[] = {{&S74476[35], 1}, {NULL}};
const Segment SGI93391[] = {{&S74476[66], 11}, {NULL}};
const Segment SGI93392[] = {{&S74477[44], 11}, {NULL}};
const Segment SGI93393[] = {{&S74477[44], 22}, {NULL}};
const Segment SGI93394[] = {{&S74477[30], 6}, {NULL}};
const Segment SGI93395[] = {{&S74477[114], 3}, {NULL}};
const Segment SGI93396[] = {{&S74477[80], 5}, {NULL}};
const Segment SGI93397[] = {{&S82095[44], 10}, {NULL}};
const Segment SGI93398[] = {{&S82095[44], 22}, {NULL}};
const Segment SGI93399[] = {{&S82095[30], 6}, {NULL}};
const Segment SGI93400[] = {{&S82095[114], 3}, {NULL}};
const Segment SGI93401[] = {{&S82095[80], 5}, {NULL}};
const Segment SGI93402[] = {{&S82140[44], 4}, {NULL}};
const Segment SGI93403[] = {{&S82140[44], 25}, {NULL}};
const Segment SGI93404[] = {{&S82140[30], 6}, {NULL}};
const Segment SGI93405[] = {{&S82140[117], 3}, {NULL}};
const Segment SGI93406[] = {{&S82140[83], 5}, {NULL}};
const Segment SGI93407[] = {{&S74624[44], 10}, {NULL}};
const Segment SGI93408[] = {{&S74624[44], 20}, {NULL}};
const Segment SGI93409[] = {{&S74624[30], 6}, {NULL}};
const Segment SGI93410[] = {{&S74624[112], 3}, {NULL}};
const Segment SGI93411[] = {{&S74624[78], 5}, {NULL}};
const Segment SGI93412[] = {{&S82205[45], 10}, {NULL}};
const Segment SGI93413[] = {{&S74625[44], 9}, {NULL}};
const Segment SGI93414[] = {{&S74625[44], 19}, {NULL}};
const Segment SGI93415[] = {{&S82205[30], 6}, {NULL}};
const Segment SGI93416[] = {{&S74625[30], 6}, {NULL}};
const Segment SGI93417[] = {{&S74625[111], 3}, {NULL}};
const Segment SGI93418[] = {{&S82205[120], 3}, {NULL}};
const Segment SGI93419[] = {{&S74625[77], 5}, {NULL}};
const Segment SGI93420[] = {{&S82205[86], 5}, {NULL}};
const Segment SGI93421[] = {{&S82205[144], 13}, {NULL}};
const Segment SGI93422[] = {{&S60703[34], 10}, {NULL}};
const Segment SGI93423[] = {{&S60703[27], 6}, {NULL}};
const Segment SGI93424[] = {{&S74626[44], 10}, {NULL}};
const Segment SGI93425[] = {{&S74626[44], 20}, {NULL}};
const Segment SGI93426[] = {{&S74626[30], 6}, {NULL}};
const Segment SGI93427[] = {{&S60703[97], 9}, {NULL}};
const Segment SGI93428[] = {{&S60703[70], 6}, {NULL}};
const Segment SGI93429[] = {{&S74626[81], 4}, {NULL}};
const Segment SGI93430[] = {{&S74626[81], 3}, {NULL}};
const Segment SGI93431[] = {{&S74628[44], 10}, {NULL}};
const Segment SGI93432[] = {{&S74628[44], 20}, {NULL}};
const Segment SGI93433[] = {{&S74628[30], 6}, {NULL}};
const Segment SGI93434[] = {{&S74628[81], 6}, {NULL}};
const Segment SGI93435[] = {{&S60706[34], 10}, {NULL}};
const Segment SGI93436[] = {{&S60706[27], 6}, {NULL}};
const Segment SGI93437[] = {{&S60706[97], 9}, {NULL}};
const Segment SGI93438[] = {{&S60706[70], 6}, {NULL}};
const Segment SGI93439[] = {{&S60708[34], 9}, {NULL}};
const Segment SGI93440[] = {{&S60708[27], 6}, {NULL}};
const Segment SGI93441[] = {{&S60708[96], 9}, {NULL}};
const Segment SGI93442[] = {{&S60708[69], 6}, {NULL}};
const Segment SGI93443[] = {{&S60711[34], 24}, {NULL}};
const Segment SGI93444[] = {{&S60711[27], 6}, {NULL}};
const Segment SGI93445[] = {{&S60711[111], 9}, {NULL}};
const Segment SGI93446[] = {{&S60711[84], 6}, {NULL}};
const Segment SGI93447[] = {{&S82218[44], 14}, {NULL}};
const Segment SGI93448[] = {{&S82218[44], 26}, {NULL}};
const Segment SGI93449[] = {{&S82218[30], 6}, {NULL}};
const Segment SGI93450[] = {{&S82218[118], 3}, {NULL}};
const Segment SGI93451[] = {{&S82218[84], 5}, {NULL}};
const Segment SGI93452[] = {{&S82244[44], 14}, {NULL}};
const Segment SGI93453[] = {{&S82244[44], 26}, {NULL}};
const Segment SGI93454[] = {{&S82244[30], 6}, {NULL}};
const Segment SGI93455[] = {{&S82244[119], 3}, {NULL}};
const Segment SGI93456[] = {{&S82244[84], 6}, {NULL}};
const Segment SGI93457[] = {{&S82244[139], 8}, {NULL}};
const Segment SGI93458[] = {{&S82244[148], 13}, {NULL}};
const Segment SGI93459[] = {{&S60787[34], 18}, {NULL}};
const Segment SGI93460[] = {{&S60787[27], 6}, {NULL}};
const Segment SGI93461[] = {{&S60787[105], 9}, {NULL}};
const Segment SGI93462[] = {{&S60787[78], 6}, {NULL}};
const Segment SGI93463[] = {{&S60788[34], 18}, {NULL}};
const Segment SGI93464[] = {{&S60788[27], 6}, {NULL}};
const Segment SGI93465[] = {{&S60788[105], 9}, {NULL}};
const Segment SGI93466[] = {{&S60788[78], 6}, {NULL}};
const Segment SGI93467[] = {{&S60789[34], 11}, {NULL}};
const Segment SGI93468[] = {{&S60789[27], 6}, {NULL}};
const Segment SGI93469[] = {{&S60789[98], 9}, {NULL}};
const Segment SGI93470[] = {{&S60789[71], 6}, {NULL}};
const Segment SGI93471[] = {{&S82290[44], 9}, {NULL}};
const Segment SGI93472[] = {{&S82290[44], 21}, {NULL}};
const Segment SGI93473[] = {{&S82290[30], 6}, {NULL}};
const Segment SGI93474[] = {{&S82290[113], 3}, {NULL}};
const Segment SGI93475[] = {{&S82290[79], 5}, {NULL}};
const Segment SGI93476[] = {{&S74727[44], 10}, {NULL}};
const Segment SGI93477[] = {{&S74727[44], 30}, {NULL}};
const Segment SGI93478[] = {{&S74727[30], 6}, {NULL}};
const Segment SGI93479[] = {{&S74727[122], 3}, {NULL}};
const Segment SGI93480[] = {{&S74727[88], 5}, {NULL}};
const Segment SGI93481[] = {{&S74730[44], 10}, {NULL}};
const Segment SGI93482[] = {{&S74730[44], 21}, {NULL}};
const Segment SGI93483[] = {{&S74730[30], 6}, {NULL}};
const Segment SGI93484[] = {{&S74730[113], 3}, {NULL}};
const Segment SGI93485[] = {{&S74730[79], 5}, {NULL}};
const Segment SGI93486[] = {{&S74734[44], 31}, {NULL}};
const Segment SGI93487[] = {{&S74734[30], 6}, {NULL}};
const Segment SGI93488[] = {{&S74735[44], 6}, {NULL}};
const Segment SGI93489[] = {{&S74735[44], 17}, {NULL}};
const Segment SGI93490[] = {{&S74735[30], 6}, {NULL}};
const Segment SGI93491[] = {{&S74735[109], 3}, {NULL}};
const Segment SGI93492[] = {{&S74735[75], 5}, {NULL}};
const Segment SGI93493[] = {{&S74736[44], 6}, {NULL}};
const Segment SGI93494[] = {{&S74736[44], 17}, {NULL}};
const Segment SGI93495[] = {{&S74736[30], 6}, {NULL}};
const Segment SGI93496[] = {{&S74736[78], 6}, {NULL}};
const Segment SGI93497[] = {{&S74737[44], 6}, {NULL}};
const Segment SGI93498[] = {{&S74737[44], 17}, {NULL}};
const Segment SGI93499[] = {{&S74737[30], 6}, {NULL}};
const Segment SGI93500[] = {{&S74737[78], 4}, {NULL}};
const Segment SGI93501[] = {{&S74737[78], 3}, {NULL}};
const Segment SGI93502[] = {{&S82314[45], 24}, {NULL}};
const Segment SGI93503[] = {{&S82314[30], 6}, {NULL}};
const Segment SGI93504[] = {{&S82314[129], 3}, {NULL}};
const Segment SGI93505[] = {{&S82314[95], 5}, {NULL}};
const Segment SGI93506[] = {{&S82314[153], 13}, {NULL}};
const Segment SGI93507[] = {{&S82315[44], 24}, {NULL}};
const Segment SGI93508[] = {{&S82315[44], 36}, {NULL}};
const Segment SGI93509[] = {{&S82315[30], 6}, {NULL}};
const Segment SGI93510[] = {{&S82315[129], 3}, {NULL}};
const Segment SGI93511[] = {{&S82315[94], 6}, {NULL}};
const Segment SGI93512[] = {{&S82315[149], 8}, {NULL}};
const Segment SGI93513[] = {{&S82315[158], 13}, {NULL}};
const Segment SGI93514[] = {{&S82318[45], 10}, {NULL}};
const Segment SGI93515[] = {{&S82318[30], 6}, {NULL}};
const Segment SGI93516[] = {{&S82318[120], 3}, {NULL}};
const Segment SGI93517[] = {{&S82318[86], 5}, {NULL}};
const Segment SGI93518[] = {{&S82318[144], 13}, {NULL}};
const Segment SGI93519[] = {{&S74280[42], 7}, {NULL}};
const Segment SGI93520[] = {{&S74280[42], 31}, {NULL}};
const Segment SGI93521[] = {{&S74280[30], 4}, {NULL}};
const Segment SGI93522[] = {{&S74280[121], 3}, {NULL}};
const Segment SGI93523[] = {{&S74280[87], 5}, {NULL}};
const Segment SGI93524[] = {{&S74281[44], 7}, {NULL}};
const Segment SGI93525[] = {{&S74281[44], 32}, {NULL}};
const Segment SGI93526[] = {{&S74281[30], 6}, {NULL}};
const Segment SGI93527[] = {{&S74281[124], 3}, {NULL}};
const Segment SGI93528[] = {{&S74281[90], 5}, {NULL}};
const Segment SGI93529[] = {{&S74282[44], 7}, {NULL}};
const Segment SGI93530[] = {{&S74282[44], 32}, {NULL}};
const Segment SGI93531[] = {{&S74282[30], 6}, {NULL}};
const Segment SGI93532[] = {{&S74282[125], 3}, {NULL}};
const Segment SGI93533[] = {{&S74282[90], 6}, {NULL}};
const Segment SGI93534[] = {{&S68812[34], 14}, {NULL}};
const Segment SGI93535[] = {{&S68812[27], 6}, {NULL}};
const Segment SGI93536[] = {{&S68812[101], 9}, {NULL}};
const Segment SGI93537[] = {{&S68812[74], 6}, {NULL}};
const Segment SGI93538[] = {{&S68812[0], 13}, {NULL}};
const Segment SGI93539[] = {{&S74355[42], 10}, {NULL}};
const Segment SGI93540[] = {{&S74355[42], 22}, {NULL}};
const Segment SGI93541[] = {{&S74355[30], 4}, {NULL}};
const Segment SGI93542[] = {{&S74355[113], 3}, {NULL}};
const Segment SGI93543[] = {{&S74355[78], 6}, {NULL}};
const Segment SGI93544[] = {{&S60807[34], 7}, {NULL}};
const Segment SGI93545[] = {{&S60807[27], 6}, {NULL}};
const Segment SGI93546[] = {{&S60807[94], 9}, {NULL}};
const Segment SGI93547[] = {{&S60807[67], 6}, {NULL}};
const Segment SGI93548[] = {{&S74860[44], 7}, {NULL}};
const Segment SGI93549[] = {{&S74860[44], 18}, {NULL}};
const Segment SGI93550[] = {{&S74860[30], 6}, {NULL}};
const Segment SGI93551[] = {{&S74860[110], 3}, {NULL}};
const Segment SGI93552[] = {{&S74860[76], 5}, {NULL}};
const Segment SGI93553[] = {{&S74862[38], 7}, {NULL}};
const Segment SGI93554[] = {{&S74862[31], 6}, {NULL}};
const Segment SGI93555[] = {{&S60828[44], 9}, {NULL}};
const Segment SGI93556[] = {{&S60828[44], 20}, {NULL}};
const Segment SGI93557[] = {{&S60828[30], 6}, {NULL}};
const Segment SGI93558[] = {{&S60828[31], 3}, {NULL}};
const Segment SGI93559[] = {{&S60828[78], 6}, {NULL}};
const Segment SGI93560[] = {{&S60832[44], 9}, {NULL}};
const Segment SGI93561[] = {{&S60832[44], 21}, {NULL}};
const Segment SGI93562[] = {{&S60832[30], 6}, {NULL}};
const Segment SGI93563[] = {{&S60832[31], 3}, {NULL}};
const Segment SGI93564[] = {{&S60832[79], 6}, {NULL}};
const Segment SGI93565[] = {{&S60835[38], 9}, {NULL}};
const Segment SGI93566[] = {{&S60835[31], 6}, {NULL}};
const Segment SGI93567[] = {{&S60901[34], 7}, {NULL}};
const Segment SGI93568[] = {{&S60901[27], 6}, {NULL}};
const Segment SGI93569[] = {{&S60901[96], 9}, {NULL}};
const Segment SGI93570[] = {{&S60901[69], 6}, {NULL}};
const Segment SGI93571[] = {{&S60928[44], 22}, {NULL}};
const Segment SGI93572[] = {{&S60928[30], 6}, {NULL}};
const Segment SGI93573[] = {{&S60928[31], 3}, {NULL}};
const Segment SGI93574[] = {{&S60928[82], 6}, {NULL}};
const Segment SGI93575[] = {{&S60929[44], 11}, {NULL}};
const Segment SGI93576[] = {{&S60929[44], 23}, {NULL}};
const Segment SGI93577[] = {{&S60929[30], 6}, {NULL}};
const Segment SGI93578[] = {{&S60929[84], 4}, {NULL}};
const Segment SGI93579[] = {{&S60929[84], 3}, {NULL}};
const Segment SGI93580[] = {{&S60930[44], 11}, {NULL}};
const Segment SGI93581[] = {{&S60930[44], 23}, {NULL}};
const Segment SGI93582[] = {{&S60930[30], 6}, {NULL}};
const Segment SGI93583[] = {{&S60930[84], 6}, {NULL}};
const Segment SGI93584[] = {{&S60931[44], 22}, {NULL}};
const Segment SGI93585[] = {{&S60931[30], 6}, {NULL}};
const Segment SGI93586[] = {{&S60931[31], 3}, {NULL}};
const Segment SGI93587[] = {{&S60931[82], 6}, {NULL}};
const Segment SGI93588[] = {{&S60932[44], 22}, {NULL}};
const Segment SGI93589[] = {{&S60932[30], 6}, {NULL}};
const Segment SGI93590[] = {{&S60932[31], 3}, {NULL}};
const Segment SGI93591[] = {{&S60932[82], 6}, {NULL}};
const Segment SGI93592[] = {{&S60933[44], 22}, {NULL}};
const Segment SGI93593[] = {{&S60933[30], 6}, {NULL}};
const Segment SGI93594[] = {{&S60933[31], 3}, {NULL}};
const Segment SGI93595[] = {{&S60933[82], 6}, {NULL}};
const Segment SGI93596[] = {{&S60934[44], 10}, {NULL}};
const Segment SGI93597[] = {{&S60934[30], 6}, {NULL}};
const Segment SGI93598[] = {{&S60934[31], 3}, {NULL}};
const Segment SGI93599[] = {{&S60934[70], 6}, {NULL}};
const Segment SGI93600[] = {{&S60935[44], 23}, {NULL}};
const Segment SGI93601[] = {{&S60935[30], 6}, {NULL}};
const Segment SGI93602[] = {{&S82475[45], 10}, {NULL}};
const Segment SGI93603[] = {{&S82475[30], 6}, {NULL}};
const Segment SGI93604[] = {{&S82475[120], 3}, {NULL}};
const Segment SGI93605[] = {{&S82475[86], 5}, {NULL}};
const Segment SGI93606[] = {{&S82475[144], 13}, {NULL}};
const Segment SGI93607[] = {{&S60936[44], 22}, {NULL}};
const Segment SGI93608[] = {{&S60936[30], 6}, {NULL}};
const Segment SGI93609[] = {{&S60936[31], 3}, {NULL}};
const Segment SGI93610[] = {{&S60936[82], 6}, {NULL}};
const Segment SGI93611[] = {{&S60937[44], 22}, {NULL}};
const Segment SGI93612[] = {{&S60937[30], 6}, {NULL}};
const Segment SGI93613[] = {{&S60937[31], 3}, {NULL}};
const Segment SGI93614[] = {{&S60937[82], 6}, {NULL}};
const Segment SGI93615[] = {{&S60938[38], 11}, {NULL}};
const Segment SGI93616[] = {{&S60938[31], 6}, {NULL}};
const Segment SGI93617[] = {{&S60939[44], 22}, {NULL}};
const Segment SGI93618[] = {{&S60939[30], 6}, {NULL}};
const Segment SGI93619[] = {{&S60939[31], 3}, {NULL}};
const Segment SGI93620[] = {{&S60939[82], 6}, {NULL}};
const Segment SGI93621[] = {{&S60940[44], 14}, {NULL}};
const Segment SGI93622[] = {{&S60940[44], 26}, {NULL}};
const Segment SGI93623[] = {{&S60940[30], 6}, {NULL}};
const Segment SGI93624[] = {{&S60940[31], 3}, {NULL}};
const Segment SGI93625[] = {{&S60940[84], 6}, {NULL}};
const Segment SGI93626[] = {{&S60941[44], 14}, {NULL}};
const Segment SGI93627[] = {{&S60941[44], 26}, {NULL}};
const Segment SGI93628[] = {{&S60941[30], 6}, {NULL}};
const Segment SGI93629[] = {{&S60941[31], 3}, {NULL}};
const Segment SGI93630[] = {{&S60941[84], 6}, {NULL}};
const Segment SGI93631[] = {{&S60942[44], 14}, {NULL}};
const Segment SGI93632[] = {{&S60942[44], 26}, {NULL}};
const Segment SGI93633[] = {{&S60942[30], 6}, {NULL}};
const Segment SGI93634[] = {{&S60942[31], 3}, {NULL}};
const Segment SGI93635[] = {{&S60942[84], 6}, {NULL}};
const Segment SGI93636[] = {{&S60944[44], 8}, {NULL}};
const Segment SGI93637[] = {{&S60944[44], 20}, {NULL}};
const Segment SGI93638[] = {{&S60944[30], 6}, {NULL}};
const Segment SGI93639[] = {{&S60944[31], 3}, {NULL}};
const Segment SGI93640[] = {{&S60944[78], 6}, {NULL}};
const Segment SGI93641[] = {{&S60961[44], 8}, {NULL}};
const Segment SGI93642[] = {{&S60961[44], 21}, {NULL}};
const Segment SGI93643[] = {{&S60961[30], 6}, {NULL}};
const Segment SGI93644[] = {{&S60961[31], 3}, {NULL}};
const Segment SGI93645[] = {{&S60961[79], 6}, {NULL}};
const Segment SGI93646[] = {{&S60962[44], 8}, {NULL}};
const Segment SGI93647[] = {{&S60962[44], 21}, {NULL}};
const Segment SGI93648[] = {{&S60962[30], 6}, {NULL}};
const Segment SGI93649[] = {{&S60962[31], 3}, {NULL}};
const Segment SGI93650[] = {{&S60962[79], 6}, {NULL}};
const Segment SGI93651[] = {{&S60963[44], 8}, {NULL}};
const Segment SGI93652[] = {{&S60963[44], 21}, {NULL}};
const Segment SGI93653[] = {{&S60963[30], 6}, {NULL}};
const Segment SGI93654[] = {{&S60963[31], 3}, {NULL}};
const Segment SGI93655[] = {{&S60963[79], 6}, {NULL}};
const Segment SGI93656[] = {{&S60964[44], 8}, {NULL}};
const Segment SGI93657[] = {{&S60964[44], 21}, {NULL}};
const Segment SGI93658[] = {{&S60964[30], 6}, {NULL}};
const Segment SGI93659[] = {{&S60964[31], 3}, {NULL}};
const Segment SGI93660[] = {{&S60964[79], 6}, {NULL}};
const Segment SGI93661[] = {{&S60965[44], 8}, {NULL}};
const Segment SGI93662[] = {{&S60965[44], 21}, {NULL}};
const Segment SGI93663[] = {{&S60965[30], 6}, {NULL}};
const Segment SGI93664[] = {{&S60965[31], 3}, {NULL}};
const Segment SGI93665[] = {{&S60965[79], 6}, {NULL}};
const Segment SGI93666[] = {{&S60966[44], 8}, {NULL}};
const Segment SGI93667[] = {{&S60966[44], 21}, {NULL}};
const Segment SGI93668[] = {{&S60966[30], 6}, {NULL}};
const Segment SGI93669[] = {{&S60966[31], 3}, {NULL}};
const Segment SGI93670[] = {{&S60966[79], 6}, {NULL}};
const Segment SGI93671[] = {{&S60967[44], 8}, {NULL}};
const Segment SGI93672[] = {{&S60967[44], 21}, {NULL}};
const Segment SGI93673[] = {{&S60967[30], 6}, {NULL}};
const Segment SGI93674[] = {{&S60967[31], 3}, {NULL}};
const Segment SGI93675[] = {{&S60967[79], 6}, {NULL}};
const Segment SGI93676[] = {{&S60968[44], 8}, {NULL}};
const Segment SGI93677[] = {{&S60968[44], 21}, {NULL}};
const Segment SGI93678[] = {{&S60968[30], 6}, {NULL}};
const Segment SGI93679[] = {{&S60968[31], 3}, {NULL}};
const Segment SGI93680[] = {{&S60968[79], 6}, {NULL}};
const Segment SGI93681[] = {{&S82550[44], 25}, {NULL}};
const Segment SGI93682[] = {{&S82550[44], 37}, {NULL}};
const Segment SGI93683[] = {{&S82550[30], 6}, {NULL}};
const Segment SGI93684[] = {{&S82550[129], 3}, {NULL}};
const Segment SGI93685[] = {{&S82550[95], 5}, {NULL}};
const Segment SGI93686[] = {{&S82551[44], 17}, {NULL}};
const Segment SGI93687[] = {{&S82551[44], 29}, {NULL}};
const Segment SGI93688[] = {{&S82551[30], 6}, {NULL}};
const Segment SGI93689[] = {{&S82551[121], 3}, {NULL}};
const Segment SGI93690[] = {{&S82551[87], 5}, {NULL}};
const Segment SGI93691[] = {{&S82552[57], 7}, {NULL}};
const Segment SGI93692[] = {{&S82552[57], 33}, {NULL}};
const Segment SGI93693[] = {{&S82552[43], 6}, {NULL}};
const Segment SGI93694[] = {{&S82552[138], 3}, {NULL}};
const Segment SGI93695[] = {{&S82552[104], 5}, {NULL}};
const Segment SGI93696[] = {{&S82553[59], 10}, {NULL}};
const Segment SGI93697[] = {{&S82553[59], 27}, {NULL}};
const Segment SGI93698[] = {{&S82553[45], 6}, {NULL}};
const Segment SGI93699[] = {{&S82553[134], 3}, {NULL}};
const Segment SGI93700[] = {{&S82553[100], 5}, {NULL}};
const Segment SGI93701[] = {{&S82553[0], 14}, {NULL}};
const Segment SGI93702[] = {{&S82554[59], 11}, {NULL}};
const Segment SGI93703[] = {{&S82554[59], 28}, {NULL}};
const Segment SGI93704[] = {{&S82554[45], 6}, {NULL}};
const Segment SGI93705[] = {{&S82554[135], 3}, {NULL}};
const Segment SGI93706[] = {{&S82554[101], 5}, {NULL}};
const Segment SGI93707[] = {{&S82554[0], 14}, {NULL}};
const Segment SGI93708[] = {{&S82569[44], 10}, {NULL}};
const Segment SGI93709[] = {{&S82569[44], 27}, {NULL}};
const Segment SGI93710[] = {{&S82569[30], 6}, {NULL}};
const Segment SGI93711[] = {{&S82569[119], 3}, {NULL}};
const Segment SGI93712[] = {{&S82569[85], 5}, {NULL}};
const Segment SGI93713[] = {{&S82569[143], 11}, {NULL}};
const Segment SGI93714[] = {{&S74663[42], 11}, {NULL}};
const Segment SGI93715[] = {{&S74663[42], 23}, {NULL}};
const Segment SGI93716[] = {{&S74663[30], 4}, {NULL}};
const Segment SGI93717[] = {{&S74663[114], 3}, {NULL}};
const Segment SGI93718[] = {{&S74663[79], 6}, {NULL}};
const Segment SGI93719[] = {{&S61021[44], 8}, {NULL}};
const Segment SGI93720[] = {{&S61021[44], 20}, {NULL}};
const Segment SGI93721[] = {{&S61021[30], 6}, {NULL}};
const Segment SGI93722[] = {{&S61021[31], 3}, {NULL}};
const Segment SGI93723[] = {{&S61021[78], 6}, {NULL}};
const Segment SGI93724[] = {{&S61022[44], 14}, {NULL}};
const Segment SGI93725[] = {{&S61022[44], 26}, {NULL}};
const Segment SGI93726[] = {{&S61022[30], 6}, {NULL}};
const Segment SGI93727[] = {{&S61022[31], 3}, {NULL}};
const Segment SGI93728[] = {{&S61022[84], 6}, {NULL}};
const Segment SGI93729[] = {{&S82692[45], 10}, {NULL}};
const Segment SGI93730[] = {{&S82692[30], 6}, {NULL}};
const Segment SGI93731[] = {{&S82692[120], 3}, {NULL}};
const Segment SGI93732[] = {{&S82692[86], 5}, {NULL}};
const Segment SGI93733[] = {{&S82692[144], 13}, {NULL}};
const Segment SGI93734[] = {{&S61081[44], 6}, {NULL}};
const Segment SGI93735[] = {{&S61081[44], 18}, {NULL}};
const Segment SGI93736[] = {{&S61081[30], 6}, {NULL}};
const Segment SGI93737[] = {{&S61081[31], 3}, {NULL}};
const Segment SGI93738[] = {{&S61081[76], 6}, {NULL}};
const Segment SGI93739[] = {{&S61085[44], 11}, {NULL}};
const Segment SGI93740[] = {{&S61085[44], 23}, {NULL}};
const Segment SGI93741[] = {{&S61085[30], 6}, {NULL}};
const Segment SGI93742[] = {{&S61085[31], 3}, {NULL}};
const Segment SGI93743[] = {{&S61085[81], 6}, {NULL}};
const Segment SGI93744[] = {{&S61104[34], 11}, {NULL}};
const Segment SGI93745[] = {{&S61104[27], 6}, {NULL}};
const Segment SGI93746[] = {{&S61104[98], 9}, {NULL}};
const Segment SGI93747[] = {{&S61104[71], 6}, {NULL}};
const Segment SGI93748[] = {{&S82723[44], 10}, {NULL}};
const Segment SGI93749[] = {{&S61105[34], 18}, {NULL}};
const Segment SGI93750[] = {{&S82723[44], 27}, {NULL}};
const Segment SGI93751[] = {{&S82723[30], 6}, {NULL}};
const Segment SGI93752[] = {{&S61105[27], 6}, {NULL}};
const Segment SGI93753[] = {{&S61105[105], 9}, {NULL}};
const Segment SGI93754[] = {{&S61105[78], 6}, {NULL}};
const Segment SGI93755[] = {{&S82723[85], 6}, {NULL}};
const Segment SGI93756[] = {{&S82723[112], 13}, {NULL}};
const Segment SGI93757[] = {{&S61106[34], 18}, {NULL}};
const Segment SGI93758[] = {{&S61106[27], 6}, {NULL}};
const Segment SGI93759[] = {{&S61106[105], 9}, {NULL}};
const Segment SGI93760[] = {{&S61106[78], 6}, {NULL}};
const Segment SGI93761[] = {{&S61117[34], 7}, {NULL}};
const Segment SGI93762[] = {{&S61117[27], 6}, {NULL}};
const Segment SGI93763[] = {{&S61117[99], 9}, {NULL}};
const Segment SGI93764[] = {{&S61117[72], 6}, {NULL}};
const Segment SGI93765[] = {{&S61119[34], 12}, {NULL}};
const Segment SGI93766[] = {{&S61119[27], 6}, {NULL}};
const Segment SGI93767[] = {{&S61119[99], 9}, {NULL}};
const Segment SGI93768[] = {{&S61119[72], 6}, {NULL}};
const Segment SGI93769[] = {{&S61147[34], 14}, {NULL}};
const Segment SGI93770[] = {{&S61147[27], 6}, {NULL}};
const Segment SGI93771[] = {{&S61147[101], 9}, {NULL}};
const Segment SGI93772[] = {{&S61147[74], 6}, {NULL}};
const Segment SGI93773[] = {{&S61167[44], 4}, {NULL}};
const Segment SGI93774[] = {{&S61167[44], 18}, {NULL}};
const Segment SGI93775[] = {{&S61167[30], 6}, {NULL}};
const Segment SGI93776[] = {{&S61167[31], 3}, {NULL}};
const Segment SGI93777[] = {{&S61167[76], 6}, {NULL}};
const Segment SGI93778[] = {{&S82794[44], 10}, {NULL}};
const Segment SGI93779[] = {{&S82794[44], 27}, {NULL}};
const Segment SGI93780[] = {{&S82794[30], 6}, {NULL}};
const Segment SGI93781[] = {{&S82794[118], 3}, {NULL}};
const Segment SGI93782[] = {{&S82794[85], 5}, {NULL}};
const Segment SGI93783[] = {{&S82794[122], 14}, {NULL}};
const Segment SGI93784[] = {{&S82807[44], 11}, {NULL}};
const Segment SGI93785[] = {{&S82807[44], 28}, {NULL}};
const Segment SGI93786[] = {{&S82807[30], 6}, {NULL}};
const Segment SGI93787[] = {{&S82807[119], 3}, {NULL}};
const Segment SGI93788[] = {{&S82807[86], 5}, {NULL}};
const Segment SGI93789[] = {{&S82807[123], 14}, {NULL}};
const Segment SGI93790[] = {{&S69392[34], 14}, {NULL}};
const Segment SGI93791[] = {{&S69392[27], 6}, {NULL}};
const Segment SGI93792[] = {{&S69392[101], 9}, {NULL}};
const Segment SGI93793[] = {{&S69392[74], 6}, {NULL}};
const Segment SGI93794[] = {{&S69392[0], 13}, {NULL}};
const Segment SGI93795[] = {{&S82810[44], 10}, {NULL}};
const Segment SGI93796[] = {{&S82810[44], 27}, {NULL}};
const Segment SGI93797[] = {{&S82810[30], 6}, {NULL}};
const Segment SGI93798[] = {{&S82810[118], 3}, {NULL}};
const Segment SGI93799[] = {{&S82810[85], 5}, {NULL}};
const Segment SGI93800[] = {{&S82810[122], 14}, {NULL}};
const Segment SGI93801[] = {{&S82839[44], 8}, {NULL}};
const Segment SGI93802[] = {{&S82839[44], 20}, {NULL}};
const Segment SGI93803[] = {{&S82839[30], 6}, {NULL}};
const Segment SGI93804[] = {{&S82839[112], 3}, {NULL}};
const Segment SGI93805[] = {{&S82839[78], 5}, {NULL}};
const Segment SGI93806[] = {{&S82840[45], 10}, {NULL}};
const Segment SGI93807[] = {{&S82840[30], 6}, {NULL}};
const Segment SGI93808[] = {{&S82840[120], 3}, {NULL}};
const Segment SGI93809[] = {{&S82840[86], 5}, {NULL}};
const Segment SGI93810[] = {{&S75399[44], 11}, {NULL}};
const Segment SGI93811[] = {{&S82840[144], 13}, {NULL}};
const Segment SGI93812[] = {{&S75399[44], 27}, {NULL}};
const Segment SGI93813[] = {{&S75399[30], 6}, {NULL}};
const Segment SGI93814[] = {{&S61234[44], 9}, {NULL}};
const Segment SGI93815[] = {{&S61234[44], 21}, {NULL}};
const Segment SGI93816[] = {{&S75399[119], 3}, {NULL}};
const Segment SGI93817[] = {{&S61234[30], 6}, {NULL}};
const Segment SGI93818[] = {{&S75399[85], 5}, {NULL}};
const Segment SGI93819[] = {{&S61234[31], 3}, {NULL}};
const Segment SGI93820[] = {{&S61234[79], 6}, {NULL}};
const Segment SGI93821[] = {{&S75403[38], 11}, {NULL}};
const Segment SGI93822[] = {{&S75403[31], 6}, {NULL}};
const Segment SGI93823[] = {{&S75406[44], 11}, {NULL}};
const Segment SGI93824[] = {{&S75406[44], 33}, {NULL}};
const Segment SGI93825[] = {{&S75406[30], 6}, {NULL}};
const Segment SGI93826[] = {{&S75406[94], 4}, {NULL}};
const Segment SGI93827[] = {{&S75406[94], 3}, {NULL}};
const Segment SGI93828[] = {{&S82855[44], 10}, {NULL}};
const Segment SGI93829[] = {{&S82855[44], 27}, {NULL}};
const Segment SGI93830[] = {{&S82855[30], 6}, {NULL}};
const Segment SGI93831[] = {{&S82855[85], 6}, {NULL}};
const Segment SGI93832[] = {{&S82855[112], 13}, {NULL}};
const Segment SGI93833[] = {{&S82856[44], 24}, {NULL}};
const Segment SGI93834[] = {{&S82856[44], 36}, {NULL}};
const Segment SGI93835[] = {{&S82856[30], 6}, {NULL}};
const Segment SGI93836[] = {{&S82856[94], 6}, {NULL}};
const Segment SGI93837[] = {{&S82856[121], 13}, {NULL}};
const Segment SGI93838[] = {{&S61301[34], 18}, {NULL}};
const Segment SGI93839[] = {{&S61301[27], 6}, {NULL}};
const Segment SGI93840[] = {{&S61301[105], 9}, {NULL}};
const Segment SGI93841[] = {{&S61301[78], 6}, {NULL}};
const Segment SGI93842[] = {{&S61303[34], 18}, {NULL}};
const Segment SGI93843[] = {{&S61303[27], 6}, {NULL}};
const Segment SGI93844[] = {{&S61303[105], 9}, {NULL}};
const Segment SGI93845[] = {{&S61303[78], 6}, {NULL}};
const Segment SGI93846[] = {{&S82906[45], 10}, {NULL}};
const Segment SGI93847[] = {{&S82906[30], 6}, {NULL}};
const Segment SGI93848[] = {{&S82906[120], 3}, {NULL}};
const Segment SGI93849[] = {{&S82906[86], 5}, {NULL}};
const Segment SGI93850[] = {{&S82906[144], 13}, {NULL}};
const Segment SGI93851[] = {{&S61329[44], 11}, {NULL}};
const Segment SGI93852[] = {{&S61329[44], 23}, {NULL}};
const Segment SGI93853[] = {{&S61329[30], 6}, {NULL}};
const Segment SGI93854[] = {{&S61329[31], 3}, {NULL}};
const Segment SGI93855[] = {{&S61329[81], 6}, {NULL}};
const Segment SGI93856[] = {{&S82934[44], 11}, {NULL}};
const Segment SGI93857[] = {{&S82934[44], 28}, {NULL}};
const Segment SGI93858[] = {{&S82934[30], 6}, {NULL}};
const Segment SGI93859[] = {{&S82934[119], 3}, {NULL}};
const Segment SGI93860[] = {{&S82934[86], 5}, {NULL}};
const Segment SGI93861[] = {{&S82934[123], 14}, {NULL}};
const Segment SGI93862[] = {{&S61359[44], 10}, {NULL}};
const Segment SGI93863[] = {{&S61359[44], 22}, {NULL}};
const Segment SGI93864[] = {{&S61359[30], 6}, {NULL}};
const Segment SGI93865[] = {{&S61359[80], 6}, {NULL}};
const Segment SGI93866[] = {{&S61359[107], 13}, {NULL}};
const Segment SGI93867[] = {{&S61360[44], 11}, {NULL}};
const Segment SGI93868[] = {{&S61360[44], 23}, {NULL}};
const Segment SGI93869[] = {{&S61360[30], 6}, {NULL}};
const Segment SGI93870[] = {{&S61360[81], 6}, {NULL}};
const Segment SGI93871[] = {{&S61360[108], 13}, {NULL}};
const Segment SGI93872[] = {{&S61377[34], 8}, {NULL}};
const Segment SGI93873[] = {{&S61377[27], 6}, {NULL}};
const Segment SGI93874[] = {{&S61377[100], 9}, {NULL}};
const Segment SGI93875[] = {{&S61377[73], 6}, {NULL}};
const Segment SGI93876[] = {{&S61378[34], 7}, {NULL}};
const Segment SGI93877[] = {{&S61378[27], 6}, {NULL}};
const Segment SGI93878[] = {{&S61378[99], 9}, {NULL}};
const Segment SGI93879[] = {{&S61378[72], 6}, {NULL}};
const Segment SGI93880[] = {{&S82975[45], 10}, {NULL}};
const Segment SGI93881[] = {{&S82975[30], 6}, {NULL}};
const Segment SGI93882[] = {{&S82975[120], 3}, {NULL}};
const Segment SGI93883[] = {{&S82975[86], 5}, {NULL}};
const Segment SGI93884[] = {{&S82975[144], 13}, {NULL}};
const Segment SGI93885[] = {{&S75547[44], 14}, {NULL}};
const Segment SGI93886[] = {{&S75547[44], 24}, {NULL}};
const Segment SGI93887[] = {{&S75547[30], 6}, {NULL}};
const Segment SGI93888[] = {{&S75547[85], 4}, {NULL}};
const Segment SGI93889[] = {{&S75547[85], 3}, {NULL}};
const Segment SGI93890[] = {{&S75548[44], 14}, {NULL}};
const Segment SGI93891[] = {{&S82987[44], 21}, {NULL}};
const Segment SGI93892[] = {{&S75548[44], 24}, {NULL}};
const Segment SGI93893[] = {{&S82987[44], 33}, {NULL}};
const Segment SGI93894[] = {{&S75548[30], 6}, {NULL}};
const Segment SGI93895[] = {{&S82987[30], 6}, {NULL}};
const Segment SGI93896[] = {{&S75548[116], 3}, {NULL}};
const Segment SGI93897[] = {{&S82987[125], 3}, {NULL}};
const Segment SGI93898[] = {{&S75548[82], 5}, {NULL}};
const Segment SGI93899[] = {{&S82987[91], 5}, {NULL}};
const Segment SGI93900[] = {{&S75549[44], 14}, {NULL}};
const Segment SGI93901[] = {{&S75549[44], 24}, {NULL}};
const Segment SGI93902[] = {{&S82988[44], 19}, {NULL}};
const Segment SGI93903[] = {{&S75549[30], 6}, {NULL}};
const Segment SGI93904[] = {{&S82988[44], 31}, {NULL}};
const Segment SGI93905[] = {{&S82988[30], 6}, {NULL}};
const Segment SGI93906[] = {{&S82988[123], 3}, {NULL}};
const Segment SGI93907[] = {{&S75549[85], 6}, {NULL}};
const Segment SGI93908[] = {{&S82988[89], 5}, {NULL}};
const Segment SGI93909[] = {{&S82991[45], 10}, {NULL}};
const Segment SGI93910[] = {{&S82991[30], 6}, {NULL}};
const Segment SGI93911[] = {{&S82991[120], 3}, {NULL}};
const Segment SGI93912[] = {{&S82991[86], 5}, {NULL}};
const Segment SGI93913[] = {{&S82991[144], 13}, {NULL}};
const Segment SGI93914[] = {{&S83011[44], 9}, {NULL}};
const Segment SGI93915[] = {{&S83011[44], 21}, {NULL}};
const Segment SGI93916[] = {{&S83011[30], 6}, {NULL}};
const Segment SGI93917[] = {{&S83011[113], 3}, {NULL}};
const Segment SGI93918[] = {{&S83011[79], 5}, {NULL}};
const Segment SGI93919[] = {{&S83023[34], 18}, {NULL}};
const Segment SGI93920[] = {{&S83023[27], 6}, {NULL}};
const Segment SGI93921[] = {{&S83023[117], 2}, {NULL}};
const Segment SGI93922[] = {{&S83023[83], 6}, {NULL}};
const Segment SGI93923[] = {{&S83023[151], 8}, {NULL}};
const Segment SGI93924[] = {{&S75590[44], 10}, {NULL}};
const Segment SGI93925[] = {{&S75590[44], 21}, {NULL}};
const Segment SGI93926[] = {{&S75590[30], 6}, {NULL}};
const Segment SGI93927[] = {{&S75590[128], 3}, {NULL}};
const Segment SGI93928[] = {{&S75590[94], 5}, {NULL}};
const Segment SGI93929[] = {{&S75590[35], 1}, {NULL}};
const Segment SGI93930[] = {{&S75590[67], 11}, {NULL}};
const Segment SGI93931[] = {{&S75591[44], 10}, {NULL}};
const Segment SGI93932[] = {{&S75591[44], 21}, {NULL}};
const Segment SGI93933[] = {{&S75591[30], 6}, {NULL}};
const Segment SGI93934[] = {{&S75591[139], 3}, {NULL}};
const Segment SGI93935[] = {{&S75591[105], 5}, {NULL}};
const Segment SGI93936[] = {{&S75591[67], 11}, {NULL}};
const Segment SGI93937[] = {{&S83032[44], 11}, {NULL}};
const Segment SGI93938[] = {{&S83032[44], 28}, {NULL}};
const Segment SGI93939[] = {{&S83032[30], 6}, {NULL}};
const Segment SGI93940[] = {{&S83032[86], 6}, {NULL}};
const Segment SGI93941[] = {{&S83032[113], 13}, {NULL}};
const Segment SGI93942[] = {{&S83033[44], 10}, {NULL}};
const Segment SGI93943[] = {{&S83033[44], 27}, {NULL}};
const Segment SGI93944[] = {{&S83033[30], 6}, {NULL}};
const Segment SGI93945[] = {{&S83033[85], 6}, {NULL}};
const Segment SGI93946[] = {{&S83033[112], 13}, {NULL}};
const Segment SGI93947[] = {{&S61465[34], 10}, {NULL}};
const Segment SGI93948[] = {{&S61465[27], 6}, {NULL}};
const Segment SGI93949[] = {{&S61465[97], 9}, {NULL}};
const Segment SGI93950[] = {{&S61465[70], 6}, {NULL}};
const Segment SGI93951[] = {{&S61466[34], 9}, {NULL}};
const Segment SGI93952[] = {{&S61466[27], 6}, {NULL}};
const Segment SGI93953[] = {{&S61466[96], 9}, {NULL}};
const Segment SGI93954[] = {{&S61466[69], 6}, {NULL}};
const Segment SGI93955[] = {{&S61467[34], 16}, {NULL}};
const Segment SGI93956[] = {{&S61467[27], 6}, {NULL}};
const Segment SGI93957[] = {{&S61467[103], 9}, {NULL}};
const Segment SGI93958[] = {{&S61467[76], 6}, {NULL}};
const Segment SGI93959[] = {{&S61472[34], 7}, {NULL}};
const Segment SGI93960[] = {{&S61472[27], 6}, {NULL}};
const Segment SGI93961[] = {{&S61472[99], 9}, {NULL}};
const Segment SGI93962[] = {{&S61472[72], 6}, {NULL}};
const Segment SGI93963[] = {{&S83057[44], 7}, {NULL}};
const Segment SGI93964[] = {{&S83057[44], 19}, {NULL}};
const Segment SGI93965[] = {{&S83057[30], 6}, {NULL}};
const Segment SGI93966[] = {{&S83057[111], 3}, {NULL}};
const Segment SGI93967[] = {{&S83057[77], 5}, {NULL}};
const Segment SGI93968[] = {{&S61515[34], 9}, {NULL}};
const Segment SGI93969[] = {{&S61515[27], 6}, {NULL}};
const Segment SGI93970[] = {{&S61515[96], 9}, {NULL}};
const Segment SGI93971[] = {{&S61515[69], 6}, {NULL}};
const Segment SGI93972[] = {{&S83106[44], 10}, {NULL}};
const Segment SGI93973[] = {{&S83106[44], 27}, {NULL}};
const Segment SGI93974[] = {{&S83106[30], 6}, {NULL}};
const Segment SGI93975[] = {{&S83106[85], 6}, {NULL}};
const Segment SGI93976[] = {{&S83106[112], 13}, {NULL}};
const Segment SGI93977[] = {{&S61552[44], 7}, {NULL}};
const Segment SGI93978[] = {{&S61552[44], 19}, {NULL}};
const Segment SGI93979[] = {{&S61552[30], 6}, {NULL}};
const Segment SGI93980[] = {{&S61552[31], 3}, {NULL}};
const Segment SGI93981[] = {{&S61552[77], 6}, {NULL}};
const Segment SGI93982[] = {{&S61553[44], 7}, {NULL}};
const Segment SGI93983[] = {{&S61553[44], 24}, {NULL}};
const Segment SGI93984[] = {{&S61553[30], 6}, {NULL}};
const Segment SGI93985[] = {{&S61553[31], 3}, {NULL}};
const Segment SGI93986[] = {{&S61553[82], 6}, {NULL}};
const Segment SGI93987[] = {{&S61554[44], 7}, {NULL}};
const Segment SGI93988[] = {{&S61554[44], 24}, {NULL}};
const Segment SGI93989[] = {{&S61554[30], 6}, {NULL}};
const Segment SGI93990[] = {{&S61554[85], 6}, {NULL}};
const Segment SGI93991[] = {{&S61555[38], 7}, {NULL}};
const Segment SGI93992[] = {{&S61555[31], 6}, {NULL}};
const Segment SGI93993[] = {{&S75695[45], 7}, {NULL}};
const Segment SGI93994[] = {{&S75695[30], 6}, {NULL}};
const Segment SGI93995[] = {{&S61556[44], 8}, {NULL}};
const Segment SGI93996[] = {{&S61556[44], 26}, {NULL}};
const Segment SGI93997[] = {{&S61556[30], 6}, {NULL}};
const Segment SGI93998[] = {{&S75695[117], 3}, {NULL}};
const Segment SGI93999[] = {{&S75695[83], 5}, {NULL}};
const Segment SGI94000[] = {{&S61556[31], 3}, {NULL}};
const Segment SGI94001[] = {{&S61556[84], 6}, {NULL}};
const Segment SGI94002[] = {{&S75695[141], 13}, {NULL}};
const Segment SGI94003[] = {{&S83154[44], 15}, {NULL}};
const Segment SGI94004[] = {{&S83154[44], 27}, {NULL}};
const Segment SGI94005[] = {{&S83154[30], 6}, {NULL}};
const Segment SGI94006[] = {{&S83154[119], 3}, {NULL}};
const Segment SGI94007[] = {{&S83154[85], 5}, {NULL}};
const Segment SGI94008[] = {{&S67449[34], 14}, {NULL}};
const Segment SGI94009[] = {{&S67449[27], 6}, {NULL}};
const Segment SGI94010[] = {{&S67449[101], 9}, {NULL}};
const Segment SGI94011[] = {{&S67449[74], 6}, {NULL}};
const Segment SGI94012[] = {{&S83215[59], 10}, {NULL}};
const Segment SGI94013[] = {{&S83215[59], 27}, {NULL}};
const Segment SGI94014[] = {{&S83215[45], 6}, {NULL}};
const Segment SGI94015[] = {{&S83215[134], 3}, {NULL}};
const Segment SGI94016[] = {{&S83215[100], 5}, {NULL}};
const Segment SGI94017[] = {{&S83215[0], 14}, {NULL}};
const Segment SGI94018[] = {{&S61647[34], 29}, {NULL}};
const Segment SGI94019[] = {{&S61647[27], 6}, {NULL}};
const Segment SGI94020[] = {{&S61647[116], 9}, {NULL}};
const Segment SGI94021[] = {{&S61647[89], 6}, {NULL}};
const Segment SGI94022[] = {{&S61648[34], 30}, {NULL}};
const Segment SGI94023[] = {{&S61648[27], 6}, {NULL}};
const Segment SGI94024[] = {{&S61648[117], 9}, {NULL}};
const Segment SGI94025[] = {{&S61648[90], 6}, {NULL}};
const Segment SGI94026[] = {{&S61649[34], 30}, {NULL}};
const Segment SGI94027[] = {{&S61649[27], 6}, {NULL}};
const Segment SGI94028[] = {{&S61649[117], 9}, {NULL}};
const Segment SGI94029[] = {{&S61649[90], 6}, {NULL}};
const Segment SGI94030[] = {{&S61655[44], 9}, {NULL}};
const Segment SGI94031[] = {{&S61655[44], 21}, {NULL}};
const Segment SGI94032[] = {{&S61655[30], 6}, {NULL}};
const Segment SGI94033[] = {{&S61655[31], 3}, {NULL}};
const Segment SGI94034[] = {{&S61655[79], 6}, {NULL}};
const Segment SGI94035[] = {{&S61659[34], 6}, {NULL}};
const Segment SGI94036[] = {{&S61659[27], 6}, {NULL}};
const Segment SGI94037[] = {{&S61659[92], 9}, {NULL}};
const Segment SGI94038[] = {{&S61659[65], 6}, {NULL}};
const Segment SGI94039[] = {{&S75866[44], 7}, {NULL}};
const Segment SGI94040[] = {{&S75866[44], 31}, {NULL}};
const Segment SGI94041[] = {{&S75866[30], 6}, {NULL}};
const Segment SGI94042[] = {{&S75866[138], 3}, {NULL}};
const Segment SGI94043[] = {{&S75866[104], 5}, {NULL}};
const Segment SGI94044[] = {{&S75866[35], 1}, {NULL}};
const Segment SGI94045[] = {{&S75866[77], 11}, {NULL}};
const Segment SGI94046[] = {{&S75867[44], 10}, {NULL}};
const Segment SGI94047[] = {{&S75867[44], 22}, {NULL}};
const Segment SGI94048[] = {{&S75867[30], 6}, {NULL}};
const Segment SGI94049[] = {{&S75867[129], 3}, {NULL}};
const Segment SGI94050[] = {{&S75867[95], 5}, {NULL}};
const Segment SGI94051[] = {{&S75867[35], 1}, {NULL}};
const Segment SGI94052[] = {{&S75867[68], 11}, {NULL}};
const Segment SGI94053[] = {{&S83317[45], 10}, {NULL}};
const Segment SGI94054[] = {{&S83317[30], 6}, {NULL}};
const Segment SGI94055[] = {{&S83317[120], 3}, {NULL}};
const Segment SGI94056[] = {{&S83317[86], 5}, {NULL}};
const Segment SGI94057[] = {{&S83317[144], 13}, {NULL}};
const Segment SGI94058[] = {{&S83357[44], 9}, {NULL}};
const Segment SGI94059[] = {{&S83357[44], 21}, {NULL}};
const Segment SGI94060[] = {{&S83357[30], 6}, {NULL}};
const Segment SGI94061[] = {{&S83357[79], 6}, {NULL}};
const Segment SGI94062[] = {{&S83357[106], 13}, {NULL}};
const Segment SGI94063[] = {{&S75939[44], 11}, {NULL}};
const Segment SGI94064[] = {{&S75939[44], 22}, {NULL}};
const Segment SGI94065[] = {{&S75939[30], 6}, {NULL}};
const Segment SGI94066[] = {{&S75939[83], 6}, {NULL}};
const Segment SGI94067[] = {{&S75940[44], 11}, {NULL}};
const Segment SGI94068[] = {{&S75940[44], 22}, {NULL}};
const Segment SGI94069[] = {{&S75940[30], 6}, {NULL}};
const Segment SGI94070[] = {{&S75940[83], 4}, {NULL}};
const Segment SGI94071[] = {{&S75940[83], 3}, {NULL}};
const Segment SGI94072[] = {{&S75942[44], 22}, {NULL}};
const Segment SGI94073[] = {{&S75942[30], 6}, {NULL}};
const Segment SGI94074[] = {{&S75175[44], 5}, {NULL}};
const Segment SGI94075[] = {{&S75175[44], 22}, {NULL}};
const Segment SGI94076[] = {{&S75175[30], 6}, {NULL}};
const Segment SGI94077[] = {{&S75175[114], 3}, {NULL}};
const Segment SGI94078[] = {{&S75175[80], 5}, {NULL}};
const Segment SGI94079[] = {{&S83429[44], 4}, {NULL}};
const Segment SGI94080[] = {{&S83429[44], 16}, {NULL}};
const Segment SGI94081[] = {{&S83429[30], 6}, {NULL}};
const Segment SGI94082[] = {{&S83429[108], 3}, {NULL}};
const Segment SGI94083[] = {{&S83429[74], 5}, {NULL}};
const Segment SGI94084[] = {{&S75176[44], 29}, {NULL}};
const Segment SGI94085[] = {{&S75176[44], 46}, {NULL}};
const Segment SGI94086[] = {{&S75176[30], 6}, {NULL}};
const Segment SGI94087[] = {{&S75176[138], 3}, {NULL}};
const Segment SGI94088[] = {{&S75176[104], 5}, {NULL}};
const Segment SGI94089[] = {{&S75177[44], 15}, {NULL}};
const Segment SGI94090[] = {{&S75177[44], 32}, {NULL}};
const Segment SGI94091[] = {{&S75177[30], 6}, {NULL}};
const Segment SGI94092[] = {{&S75177[124], 3}, {NULL}};
const Segment SGI94093[] = {{&S75177[90], 5}, {NULL}};
const Segment SGI94094[] = {{&S75178[30], 4}, {NULL}};
const Segment SGI94095[] = {{&S75178[90], 3}, {NULL}};
const Segment SGI94096[] = {{&S75178[55], 6}, {NULL}};
const Segment SGI94097[] = {{&S83435[44], 14}, {NULL}};
const Segment SGI94098[] = {{&S83435[44], 26}, {NULL}};
const Segment SGI94099[] = {{&S83435[30], 6}, {NULL}};
const Segment SGI94100[] = {{&S83435[118], 3}, {NULL}};
const Segment SGI94101[] = {{&S83435[84], 5}, {NULL}};
const Segment SGI94102[] = {{&S61861[34], 18}, {NULL}};
const Segment SGI94103[] = {{&S61861[27], 6}, {NULL}};
const Segment SGI94104[] = {{&S61861[105], 9}, {NULL}};
const Segment SGI94105[] = {{&S61861[78], 6}, {NULL}};
const Segment SGI94106[] = {{&S61863[34], 10}, {NULL}};
const Segment SGI94107[] = {{&S61863[27], 6}, {NULL}};
const Segment SGI94108[] = {{&S61863[97], 9}, {NULL}};
const Segment SGI94109[] = {{&S61863[70], 6}, {NULL}};
const Segment SGI94110[] = {{&S61887[44], 9}, {NULL}};
const Segment SGI94111[] = {{&S61887[44], 21}, {NULL}};
const Segment SGI94112[] = {{&S61887[30], 6}, {NULL}};
const Segment SGI94113[] = {{&S61887[31], 3}, {NULL}};
const Segment SGI94114[] = {{&S61887[79], 6}, {NULL}};
const Segment SGI94115[] = {{&S83472[44], 10}, {NULL}};
const Segment SGI94116[] = {{&S83472[44], 27}, {NULL}};
const Segment SGI94117[] = {{&S83472[30], 6}, {NULL}};
const Segment SGI94118[] = {{&S83472[85], 6}, {NULL}};
const Segment SGI94119[] = {{&S83472[112], 13}, {NULL}};
const Segment SGI94120[] = {{&S61913[44], 8}, {NULL}};
const Segment SGI94121[] = {{&S61913[44], 20}, {NULL}};
const Segment SGI94122[] = {{&S61913[30], 6}, {NULL}};
const Segment SGI94123[] = {{&S61913[31], 3}, {NULL}};
const Segment SGI94124[] = {{&S61913[78], 6}, {NULL}};
const Segment SGI94125[] = {{&S61941[34], 14}, {NULL}};
const Segment SGI94126[] = {{&S61941[27], 6}, {NULL}};
const Segment SGI94127[] = {{&S61941[101], 9}, {NULL}};
const Segment SGI94128[] = {{&S61941[74], 6}, {NULL}};
const Segment SGI94129[] = {{&S61944[34], 14}, {NULL}};
const Segment SGI94130[] = {{&S61944[27], 6}, {NULL}};
const Segment SGI94131[] = {{&S61944[101], 9}, {NULL}};
const Segment SGI94132[] = {{&S61944[74], 6}, {NULL}};
const Segment SGI94133[] = {{&S61948[34], 15}, {NULL}};
const Segment SGI94134[] = {{&S61948[27], 6}, {NULL}};
const Segment SGI94135[] = {{&S61948[102], 9}, {NULL}};
const Segment SGI94136[] = {{&S61948[75], 6}, {NULL}};
const Segment SGI94137[] = {{&S61957[34], 11}, {NULL}};
const Segment SGI94138[] = {{&S61957[27], 6}, {NULL}};
const Segment SGI94139[] = {{&S61957[98], 9}, {NULL}};
const Segment SGI94140[] = {{&S61957[71], 6}, {NULL}};
const Segment SGI94141[] = {{&S83547[44], 10}, {NULL}};
const Segment SGI94142[] = {{&S83547[44], 27}, {NULL}};
const Segment SGI94143[] = {{&S83547[30], 6}, {NULL}};
const Segment SGI94144[] = {{&S83547[85], 6}, {NULL}};
const Segment SGI94145[] = {{&S83547[112], 13}, {NULL}};
const Segment SGI94146[] = {{&S83559[44], 10}, {NULL}};
const Segment SGI94147[] = {{&S83559[44], 27}, {NULL}};
const Segment SGI94148[] = {{&S83559[30], 6}, {NULL}};
const Segment SGI94149[] = {{&S83559[85], 6}, {NULL}};
const Segment SGI94150[] = {{&S83559[112], 13}, {NULL}};
const Segment SGI94151[] = {{&S61997[34], 6}, {NULL}};
const Segment SGI94152[] = {{&S61997[27], 6}, {NULL}};
const Segment SGI94153[] = {{&S61997[93], 9}, {NULL}};
const Segment SGI94154[] = {{&S61997[66], 6}, {NULL}};
const Segment SGI94155[] = {{&S83639[44], 9}, {NULL}};
const Segment SGI94156[] = {{&S83639[44], 21}, {NULL}};
const Segment SGI94157[] = {{&S83639[30], 6}, {NULL}};
const Segment SGI94158[] = {{&S83639[79], 6}, {NULL}};
const Segment SGI94159[] = {{&S83639[106], 13}, {NULL}};
const Segment SGI94160[] = {{&S62058[34], 41}, {NULL}};
const Segment SGI94161[] = {{&S62058[27], 6}, {NULL}};
const Segment SGI94162[] = {{&S62058[128], 9}, {NULL}};
const Segment SGI94163[] = {{&S62058[101], 6}, {NULL}};
const Segment SGI94164[] = {{&S62059[34], 41}, {NULL}};
const Segment SGI94165[] = {{&S62059[27], 6}, {NULL}};
const Segment SGI94166[] = {{&S62059[128], 9}, {NULL}};
const Segment SGI94167[] = {{&S62059[101], 6}, {NULL}};
const Segment SGI94168[] = {{&S76270[44], 9}, {NULL}};
const Segment SGI94169[] = {{&S76270[44], 20}, {NULL}};
const Segment SGI94170[] = {{&S76270[30], 6}, {NULL}};
const Segment SGI94171[] = {{&S76270[131], 3}, {NULL}};
const Segment SGI94172[] = {{&S76270[97], 5}, {NULL}};
const Segment SGI94173[] = {{&S76270[35], 1}, {NULL}};
const Segment SGI94174[] = {{&S76270[66], 11}, {NULL}};
const Segment SGI94175[] = {{&S76284[45], 7}, {NULL}};
const Segment SGI94176[] = {{&S76284[30], 6}, {NULL}};
const Segment SGI94177[] = {{&S76284[117], 3}, {NULL}};
const Segment SGI94178[] = {{&S76284[83], 5}, {NULL}};
const Segment SGI94179[] = {{&S76284[141], 13}, {NULL}};
const Segment SGI94180[] = {{&S83756[45], 10}, {NULL}};
const Segment SGI94181[] = {{&S83756[30], 6}, {NULL}};
const Segment SGI94182[] = {{&S83756[120], 3}, {NULL}};
const Segment SGI94183[] = {{&S83756[86], 5}, {NULL}};
const Segment SGI94184[] = {{&S83756[144], 13}, {NULL}};
const Segment SGI94185[] = {{&S83764[59], 10}, {NULL}};
const Segment SGI94186[] = {{&S83764[59], 27}, {NULL}};
const Segment SGI94187[] = {{&S83764[45], 6}, {NULL}};
const Segment SGI94188[] = {{&S83764[134], 3}, {NULL}};
const Segment SGI94189[] = {{&S83764[100], 5}, {NULL}};
const Segment SGI94190[] = {{&S83764[0], 14}, {NULL}};
const Segment SGI94191[] = {{&S62120[44], 18}, {NULL}};
const Segment SGI94192[] = {{&S62120[44], 27}, {NULL}};
const Segment SGI94193[] = {{&S62120[30], 6}, {NULL}};
const Segment SGI94194[] = {{&S62120[31], 3}, {NULL}};
const Segment SGI94195[] = {{&S62120[85], 6}, {NULL}};
const Segment SGI94196[] = {{&S75537[32], 10}, {NULL}};
const Segment SGI94197[] = {{&S75537[27], 4}, {NULL}};
const Segment SGI94198[] = {{&S75537[102], 2}, {NULL}};
const Segment SGI94199[] = {{&S75537[68], 6}, {NULL}};
const Segment SGI94200[] = {{&S75537[129], 8}, {NULL}};
const Segment SGI94201[] = {{&S76380[44], 12}, {NULL}};
const Segment SGI94202[] = {{&S76380[44], 22}, {NULL}};
const Segment SGI94203[] = {{&S76380[30], 6}, {NULL}};
const Segment SGI94204[] = {{&S76380[114], 3}, {NULL}};
const Segment SGI94205[] = {{&S76380[80], 5}, {NULL}};
const Segment SGI94206[] = {{&S76381[38], 12}, {NULL}};
const Segment SGI94207[] = {{&S76381[31], 6}, {NULL}};
const Segment SGI94208[] = {{&S76382[44], 15}, {NULL}};
const Segment SGI94209[] = {{&S76382[44], 25}, {NULL}};
const Segment SGI94210[] = {{&S76382[30], 6}, {NULL}};
const Segment SGI94211[] = {{&S76382[117], 3}, {NULL}};
const Segment SGI94212[] = {{&S76382[83], 5}, {NULL}};
const Segment SGI94213[] = {{&S76383[44], 7}, {&S76383[82], 9}, {NULL}};
const Segment SGI94214[] = {{&S76383[44], 37}, {&S76383[82], 20}, {NULL}};
const Segment SGI94215[] = {{&S76383[30], 6}, {NULL}};
const Segment SGI94216[] = {{&S76383[150], 3}, {NULL}};
const Segment SGI94217[] = {{&S76383[116], 5}, {NULL}};
const Segment SGI94218[] = {{&S83834[45], 10}, {NULL}};
const Segment SGI94219[] = {{&S83834[30], 6}, {NULL}};
const Segment SGI94220[] = {{&S83834[120], 3}, {NULL}};
const Segment SGI94221[] = {{&S83834[86], 5}, {NULL}};
const Segment SGI94222[] = {{&S83834[144], 13}, {NULL}};
const Segment SGI94223[] = {{&S83837[45], 10}, {NULL}};
const Segment SGI94224[] = {{&S83837[30], 6}, {NULL}};
const Segment SGI94225[] = {{&S83837[120], 3}, {NULL}};
const Segment SGI94226[] = {{&S83837[86], 5}, {NULL}};
const Segment SGI94227[] = {{&S83837[144], 13}, {NULL}};
const Segment SGI94228[] = {{&S83850[59], 10}, {NULL}};
const Segment SGI94229[] = {{&S83850[59], 27}, {NULL}};
const Segment SGI94230[] = {{&S83850[45], 6}, {NULL}};
const Segment SGI94231[] = {{&S83850[134], 3}, {NULL}};
const Segment SGI94232[] = {{&S83850[100], 5}, {NULL}};
const Segment SGI94233[] = {{&S83850[0], 14}, {NULL}};
const Segment SGI94234[] = {{&S76435[44], 12}, {NULL}};
const Segment SGI94235[] = {{&S76435[44], 23}, {NULL}};
const Segment SGI94236[] = {{&S76435[30], 6}, {NULL}};
const Segment SGI94237[] = {{&S76435[115], 3}, {NULL}};
const Segment SGI94238[] = {{&S76435[81], 5}, {NULL}};
const Segment SGI94239[] = {{&S76436[44], 12}, {NULL}};
const Segment SGI94240[] = {{&S76436[44], 23}, {NULL}};
const Segment SGI94241[] = {{&S76436[30], 6}, {NULL}};
const Segment SGI94242[] = {{&S76436[115], 3}, {NULL}};
const Segment SGI94243[] = {{&S76436[81], 5}, {NULL}};
const Segment SGI94244[] = {{&S76440[44], 12}, {NULL}};
const Segment SGI94245[] = {{&S76440[44], 23}, {NULL}};
const Segment SGI94246[] = {{&S76440[30], 6}, {NULL}};
const Segment SGI94247[] = {{&S76440[134], 3}, {NULL}};
const Segment SGI94248[] = {{&S76440[100], 5}, {NULL}};
const Segment SGI94249[] = {{&S76444[44], 6}, {NULL}};
const Segment SGI94250[] = {{&S76444[44], 16}, {NULL}};
const Segment SGI94251[] = {{&S76444[30], 6}, {NULL}};
const Segment SGI94252[] = {{&S76444[108], 3}, {NULL}};
const Segment SGI94253[] = {{&S76444[74], 5}, {NULL}};
const Segment SGI94254[] = {{&S83873[44], 14}, {NULL}};
const Segment SGI94255[] = {{&S83873[44], 26}, {NULL}};
const Segment SGI94256[] = {{&S83873[30], 6}, {NULL}};
const Segment SGI94257[] = {{&S83873[84], 6}, {NULL}};
const Segment SGI94258[] = {{&S83873[111], 13}, {NULL}};
const Segment SGI94259[] = {{&S76448[44], 6}, {NULL}};
const Segment SGI94260[] = {{&S76448[44], 16}, {NULL}};
const Segment SGI94261[] = {{&S76448[30], 6}, {NULL}};
const Segment SGI94262[] = {{&S76448[77], 4}, {NULL}};
const Segment SGI94263[] = {{&S76448[77], 3}, {NULL}};
const Segment SGI94264[] = {{&S76450[44], 9}, {NULL}};
const Segment SGI94265[] = {{&S76450[44], 19}, {NULL}};
const Segment SGI94266[] = {{&S76450[30], 6}, {NULL}};
const Segment SGI94267[] = {{&S76450[111], 3}, {NULL}};
const Segment SGI94268[] = {{&S76450[77], 5}, {NULL}};
const Segment SGI94269[] = {{&S76451[44], 9}, {&S76451[64], 9}, {NULL}};
const Segment SGI94270[] = {{&S76451[44], 19}, {NULL}};
const Segment SGI94271[] = {{&S76451[30], 6}, {NULL}};
const Segment SGI94272[] = {{&S76451[132], 3}, {NULL}};
const Segment SGI94273[] = {{&S76451[98], 5}, {NULL}};
const Segment SGI94274[] = {{&S75692[44], 18}, {NULL}};
const Segment SGI94275[] = {{&S75692[44], 30}, {NULL}};
const Segment SGI94276[] = {{&S75692[30], 6}, {NULL}};
const Segment SGI94277[] = {{&S75692[123], 3}, {NULL}};
const Segment SGI94278[] = {{&S75692[88], 6}, {NULL}};
const Segment SGI94279[] = {{&S62306[44], 10}, {NULL}};
const Segment SGI94280[] = {{&S62306[44], 22}, {NULL}};
const Segment SGI94281[] = {{&S62306[30], 6}, {NULL}};
const Segment SGI94282[] = {{&S62306[146], 4}, {NULL}};
const Segment SGI94283[] = {{&S62306[172], 3}, {NULL}};
const Segment SGI94284[] = {{&S62306[31], 3}, {&S62306[221], 5}, {NULL}};
const Segment SGI94285[] = {{&S62306[80], 6}, {NULL}};
const Segment SGI94286[] = {{&S83947[34], 10}, {NULL}};
const Segment SGI94287[] = {{&S83947[27], 6}, {NULL}};
const Segment SGI94288[] = {{&S83947[109], 2}, {NULL}};
const Segment SGI94289[] = {{&S83947[75], 6}, {NULL}};
const Segment SGI94290[] = {{&S83947[144], 8}, {NULL}};
const Segment SGI94291[] = {{&S83948[34], 10}, {NULL}};
const Segment SGI94292[] = {{&S83948[27], 6}, {NULL}};
const Segment SGI94293[] = {{&S83948[109], 2}, {NULL}};
const Segment SGI94294[] = {{&S83948[75], 6}, {NULL}};
const Segment SGI94295[] = {{&S83948[143], 8}, {NULL}};
const Segment SGI94296[] = {{&S62317[44], 14}, {NULL}};
const Segment SGI94297[] = {{&S62317[44], 26}, {NULL}};
const Segment SGI94298[] = {{&S62317[30], 6}, {NULL}};
const Segment SGI94299[] = {{&S62317[31], 3}, {NULL}};
const Segment SGI94300[] = {{&S62317[84], 6}, {NULL}};
const Segment SGI94301[] = {{&S62319[44], 19}, {NULL}};
const Segment SGI94302[] = {{&S62319[44], 31}, {NULL}};
const Segment SGI94303[] = {{&S62319[30], 6}, {NULL}};
const Segment SGI94304[] = {{&S62319[31], 3}, {NULL}};
const Segment SGI94305[] = {{&S62319[89], 6}, {NULL}};
const Segment SGI94306[] = {{&S62320[44], 8}, {NULL}};
const Segment SGI94307[] = {{&S62320[44], 20}, {NULL}};
const Segment SGI94308[] = {{&S62320[30], 6}, {NULL}};
const Segment SGI94309[] = {{&S62320[31], 3}, {NULL}};
const Segment SGI94310[] = {{&S62320[78], 6}, {NULL}};
const Segment SGI94311[] = {{&S62324[44], 14}, {NULL}};
const Segment SGI94312[] = {{&S62324[44], 23}, {NULL}};
const Segment SGI94313[] = {{&S62324[30], 6}, {NULL}};
const Segment SGI94314[] = {{&S62324[31], 3}, {NULL}};
const Segment SGI94315[] = {{&S62324[81], 6}, {NULL}};
const Segment SGI94316[] = {{&S62326[44], 7}, {NULL}};
const Segment SGI94317[] = {{&S62326[44], 19}, {NULL}};
const Segment SGI94318[] = {{&S62326[30], 6}, {NULL}};
const Segment SGI94319[] = {{&S83966[44], 11}, {NULL}};
const Segment SGI94320[] = {{&S83966[44], 28}, {NULL}};
const Segment SGI94321[] = {{&S62326[31], 3}, {NULL}};
const Segment SGI94322[] = {{&S83966[30], 6}, {NULL}};
const Segment SGI94323[] = {{&S62326[77], 6}, {NULL}};
const Segment SGI94324[] = {{&S83966[119], 3}, {NULL}};
const Segment SGI94325[] = {{&S83966[86], 5}, {NULL}};
const Segment SGI94326[] = {{&S83966[123], 14}, {NULL}};
const Segment SGI94327[] = {{&S83971[34], 10}, {NULL}};
const Segment SGI94328[] = {{&S83971[27], 6}, {NULL}};
const Segment SGI94329[] = {{&S83971[109], 2}, {NULL}};
const Segment SGI94330[] = {{&S83971[75], 6}, {NULL}};
const Segment SGI94331[] = {{&S83971[143], 8}, {NULL}};
const Segment SGI94332[] = {{&S83973[52], 10}, {NULL}};
const Segment SGI94333[] = {{&S83973[52], 27}, {NULL}};
const Segment SGI94334[] = {{&S83973[38], 6}, {NULL}};
const Segment SGI94335[] = {{&S83973[93], 5}, {NULL}};
const Segment SGI94336[] = {{&S83973[0], 14}, {NULL}};
const Segment SGI94337[] = {{&S70317[45], 8}, {NULL}};
const Segment SGI94338[] = {{&S70317[45], 20}, {NULL}};
const Segment SGI94339[] = {{&S70317[30], 7}, {NULL}};
const Segment SGI94340[] = {{&S70317[31], 3}, {NULL}};
const Segment SGI94341[] = {{&S70317[79], 6}, {NULL}};
const Segment SGI94342[] = {{&S70317[0], 13}, {NULL}};
const Segment SGI94343[] = {{&S83998[57], 10}, {NULL}};
const Segment SGI94344[] = {{&S83998[57], 27}, {NULL}};
const Segment SGI94345[] = {{&S83998[43], 6}, {NULL}};
const Segment SGI94346[] = {{&S83998[132], 3}, {NULL}};
const Segment SGI94347[] = {{&S83998[98], 5}, {NULL}};
const Segment SGI94348[] = {{&S83998[0], 12}, {NULL}};
const Segment SGI94349[] = {{&S84000[58], 7}, {NULL}};
const Segment SGI94350[] = {{&S84000[58], 24}, {NULL}};
const Segment SGI94351[] = {{&S84000[44], 6}, {NULL}};
const Segment SGI94352[] = {{&S84000[130], 3}, {NULL}};
const Segment SGI94353[] = {{&S84000[96], 5}, {NULL}};
const Segment SGI94354[] = {{&S84000[0], 14}, {NULL}};
const Segment SGI94355[] = {{&S76613[44], 6}, {NULL}};
const Segment SGI94356[] = {{&S76613[44], 25}, {NULL}};
const Segment SGI94357[] = {{&S76613[30], 6}, {NULL}};
const Segment SGI94358[] = {{&S76613[132], 3}, {NULL}};
const Segment SGI94359[] = {{&S76613[98], 5}, {NULL}};
const Segment SGI94360[] = {{&S76613[75], 3}, {NULL}};
const Segment SGI94361[] = {{&S84029[44], 10}, {NULL}};
const Segment SGI94362[] = {{&S84029[44], 27}, {NULL}};
const Segment SGI94363[] = {{&S84029[30], 6}, {NULL}};
const Segment SGI94364[] = {{&S84029[119], 3}, {NULL}};
const Segment SGI94365[] = {{&S84029[85], 5}, {NULL}};
const Segment SGI94366[] = {{&S84029[143], 7}, {NULL}};
const Segment SGI94367[] = {{&S84038[44], 10}, {NULL}};
const Segment SGI94368[] = {{&S84038[44], 27}, {NULL}};
const Segment SGI94369[] = {{&S84038[30], 6}, {NULL}};
const Segment SGI94370[] = {{&S84038[85], 6}, {NULL}};
const Segment SGI94371[] = {{&S84038[112], 13}, {NULL}};
const Segment SGI94372[] = {{&S62351[44], 9}, {NULL}};
const Segment SGI94373[] = {{&S62351[44], 21}, {NULL}};
const Segment SGI94374[] = {{&S62351[30], 6}, {NULL}};
const Segment SGI94375[] = {{&S62351[31], 3}, {NULL}};
const Segment SGI94376[] = {{&S62351[79], 6}, {NULL}};
const Segment SGI94377[] = {{&S62363[34], 21}, {NULL}};
const Segment SGI94378[] = {{&S62363[27], 6}, {NULL}};
const Segment SGI94379[] = {{&S62363[108], 9}, {NULL}};
const Segment SGI94380[] = {{&S62363[81], 6}, {NULL}};
const Segment SGI94381[] = {{&S62364[34], 20}, {NULL}};
const Segment SGI94382[] = {{&S62364[27], 6}, {NULL}};
const Segment SGI94383[] = {{&S62364[107], 9}, {NULL}};
const Segment SGI94384[] = {{&S62364[80], 6}, {NULL}};
const Segment SGI94385[] = {{&S62398[34], 9}, {NULL}};
const Segment SGI94386[] = {{&S62398[27], 6}, {NULL}};
const Segment SGI94387[] = {{&S62398[96], 9}, {NULL}};
const Segment SGI94388[] = {{&S62398[69], 6}, {NULL}};
const Segment SGI94389[] = {{&S62444[34], 10}, {NULL}};
const Segment SGI94390[] = {{&S62444[27], 6}, {NULL}};
const Segment SGI94391[] = {{&S62444[97], 9}, {NULL}};
const Segment SGI94392[] = {{&S62444[70], 6}, {NULL}};
const Segment SGI94393[] = {{&S84147[52], 10}, {NULL}};
const Segment SGI94394[] = {{&S84147[52], 27}, {NULL}};
const Segment SGI94395[] = {{&S84147[38], 6}, {NULL}};
const Segment SGI94396[] = {{&S84147[93], 5}, {NULL}};
const Segment SGI94397[] = {{&S84147[0], 14}, {NULL}};
const Segment SGI94398[] = {{&S62445[34], 10}, {NULL}};
const Segment SGI94399[] = {{&S62445[27], 6}, {NULL}};
const Segment SGI94400[] = {{&S62445[97], 9}, {NULL}};
const Segment SGI94401[] = {{&S62445[70], 6}, {NULL}};
const Segment SGI94402[] = {{&S62446[34], 10}, {NULL}};
const Segment SGI94403[] = {{&S62446[27], 6}, {NULL}};
const Segment SGI94404[] = {{&S62446[97], 9}, {NULL}};
const Segment SGI94405[] = {{&S62446[70], 6}, {NULL}};
const Segment SGI94406[] = {{&S62447[34], 9}, {NULL}};
const Segment SGI94407[] = {{&S62447[27], 6}, {NULL}};
const Segment SGI94408[] = {{&S62447[96], 9}, {NULL}};
const Segment SGI94409[] = {{&S62447[69], 6}, {NULL}};
const Segment SGI94410[] = {{&S62448[34], 9}, {NULL}};
const Segment SGI94411[] = {{&S62448[27], 6}, {NULL}};
const Segment SGI94412[] = {{&S62448[96], 9}, {NULL}};
const Segment SGI94413[] = {{&S62448[69], 6}, {NULL}};
const Segment SGI94414[] = {{&S62471[44], 6}, {NULL}};
const Segment SGI94415[] = {{&S62471[44], 18}, {NULL}};
const Segment SGI94416[] = {{&S62471[30], 6}, {NULL}};
const Segment SGI94417[] = {{&S62471[31], 3}, {NULL}};
const Segment SGI94418[] = {{&S62471[76], 6}, {NULL}};
const Segment SGI94419[] = {{&S62472[38], 6}, {NULL}};
const Segment SGI94420[] = {{&S62472[31], 6}, {NULL}};
const Segment SGI94421[] = {{&S84207[45], 10}, {NULL}};
const Segment SGI94422[] = {{&S84207[30], 6}, {NULL}};
const Segment SGI94423[] = {{&S84207[120], 3}, {NULL}};
const Segment SGI94424[] = {{&S84207[86], 5}, {NULL}};
const Segment SGI94425[] = {{&S84207[144], 13}, {NULL}};
const Segment SGI94426[] = {{&S75988[42], 10}, {NULL}};
const Segment SGI94427[] = {{&S75988[42], 21}, {NULL}};
const Segment SGI94428[] = {{&S75988[30], 4}, {NULL}};
const Segment SGI94429[] = {{&S75988[136], 4}, {NULL}};
const Segment SGI94430[] = {{&S75988[162], 3}, {NULL}};
const Segment SGI94431[] = {{&S75988[112], 3}, {&S75988[211], 5}, {NULL}};
const Segment SGI94432[] = {{&S75988[77], 6}, {NULL}};
const Segment SGI94433[] = {{&S75989[40], 10}, {NULL}};
const Segment SGI94434[] = {{&S75989[30], 4}, {NULL}};
const Segment SGI94435[] = {{&S75989[134], 4}, {NULL}};
const Segment SGI94436[] = {{&S75989[160], 3}, {NULL}};
const Segment SGI94437[] = {{&S75989[110], 3}, {&S75989[209], 5}, {NULL}};
const Segment SGI94438[] = {{&S75989[75], 6}, {NULL}};
const Segment SGI94439[] = {{&S62515[34], 7}, {NULL}};
const Segment SGI94440[] = {{&S62515[27], 6}, {NULL}};
const Segment SGI94441[] = {{&S62515[99], 9}, {NULL}};
const Segment SGI94442[] = {{&S62515[72], 6}, {NULL}};
const Segment SGI94443[] = {{&S84241[44], 10}, {NULL}};
const Segment SGI94444[] = {{&S84241[44], 27}, {NULL}};
const Segment SGI94445[] = {{&S84241[30], 6}, {NULL}};
const Segment SGI94446[] = {{&S84241[118], 3}, {NULL}};
const Segment SGI94447[] = {{&S84241[85], 5}, {NULL}};
const Segment SGI94448[] = {{&S84241[122], 14}, {NULL}};
const Segment SGI94449[] = {{&S62533[44], 14}, {NULL}};
const Segment SGI94450[] = {{&S62533[44], 26}, {NULL}};
const Segment SGI94451[] = {{&S62533[30], 6}, {NULL}};
const Segment SGI94452[] = {{&S62533[31], 3}, {NULL}};
const Segment SGI94453[] = {{&S62533[84], 6}, {NULL}};
const Segment SGI94454[] = {{&S84249[52], 10}, {NULL}};
const Segment SGI94455[] = {{&S84249[52], 27}, {NULL}};
const Segment SGI94456[] = {{&S84249[38], 6}, {NULL}};
const Segment SGI94457[] = {{&S84249[93], 5}, {NULL}};
const Segment SGI94458[] = {{&S84249[0], 14}, {NULL}};
const Segment SGI94459[] = {{&S62539[34], 16}, {NULL}};
const Segment SGI94460[] = {{&S62539[27], 6}, {NULL}};
const Segment SGI94461[] = {{&S62539[103], 9}, {NULL}};
const Segment SGI94462[] = {{&S62539[76], 6}, {NULL}};
const Segment SGI94463[] = {{&S62540[34], 9}, {NULL}};
const Segment SGI94464[] = {{&S62540[27], 6}, {NULL}};
const Segment SGI94465[] = {{&S62540[96], 9}, {NULL}};
const Segment SGI94466[] = {{&S62540[69], 6}, {NULL}};
const Segment SGI94467[] = {{&S70595[70], 6}, {NULL}};
const Segment SGI94468[] = {{&S70595[0], 5}, {NULL}};
const Segment SGI94469[] = {{&S62543[34], 13}, {NULL}};
const Segment SGI94470[] = {{&S62543[27], 6}, {NULL}};
const Segment SGI94471[] = {{&S62543[100], 9}, {NULL}};
const Segment SGI94472[] = {{&S62543[73], 6}, {NULL}};
const Segment SGI94473[] = {{&S84288[59], 24}, {NULL}};
const Segment SGI94474[] = {{&S84288[59], 36}, {NULL}};
const Segment SGI94475[] = {{&S84288[45], 6}, {NULL}};
const Segment SGI94476[] = {{&S84288[143], 3}, {NULL}};
const Segment SGI94477[] = {{&S84288[109], 5}, {NULL}};
const Segment SGI94478[] = {{&S84288[0], 14}, {NULL}};
const Segment SGI94479[] = {{&S62563[34], 10}, {NULL}};
const Segment SGI94480[] = {{&S62563[27], 6}, {NULL}};
const Segment SGI94481[] = {{&S62563[102], 9}, {NULL}};
const Segment SGI94482[] = {{&S62563[75], 6}, {NULL}};
const Segment SGI94483[] = {{&S62564[34], 11}, {NULL}};
const Segment SGI94484[] = {{&S62564[27], 6}, {NULL}};
const Segment SGI94485[] = {{&S62564[103], 9}, {NULL}};
const Segment SGI94486[] = {{&S62564[76], 6}, {NULL}};
const Segment SGI94487[] = {{&S62566[34], 10}, {NULL}};
const Segment SGI94488[] = {{&S62566[27], 6}, {NULL}};
const Segment SGI94489[] = {{&S62566[99], 9}, {NULL}};
const Segment SGI94490[] = {{&S62566[72], 6}, {NULL}};
const Segment SGI94491[] = {{&S62574[44], 9}, {NULL}};
const Segment SGI94492[] = {{&S62574[44], 29}, {NULL}};
const Segment SGI94493[] = {{&S62574[30], 6}, {NULL}};
const Segment SGI94494[] = {{&S62574[31], 3}, {NULL}};
const Segment SGI94495[] = {{&S62574[87], 6}, {NULL}};
const Segment SGI94496[] = {{&S62579[34], 7}, {NULL}};
const Segment SGI94497[] = {{&S62579[27], 6}, {NULL}};
const Segment SGI94498[] = {{&S62579[96], 9}, {NULL}};
const Segment SGI94499[] = {{&S62579[69], 6}, {NULL}};
const Segment SGI94500[] = {{&S62624[34], 18}, {NULL}};
const Segment SGI94501[] = {{&S62624[27], 6}, {NULL}};
const Segment SGI94502[] = {{&S62624[105], 9}, {NULL}};
const Segment SGI94503[] = {{&S62624[78], 6}, {NULL}};
const Segment SGI94504[] = {{&S62625[34], 18}, {NULL}};
const Segment SGI94505[] = {{&S62625[27], 6}, {NULL}};
const Segment SGI94506[] = {{&S62625[105], 9}, {NULL}};
const Segment SGI94507[] = {{&S62625[78], 6}, {NULL}};
const Segment SGI94508[] = {{&S62626[34], 11}, {NULL}};
const Segment SGI94509[] = {{&S62626[27], 6}, {NULL}};
const Segment SGI94510[] = {{&S62626[98], 9}, {NULL}};
const Segment SGI94511[] = {{&S62626[71], 6}, {NULL}};
const Segment SGI94512[] = {{&S84421[44], 10}, {NULL}};
const Segment SGI94513[] = {{&S84421[44], 27}, {NULL}};
const Segment SGI94514[] = {{&S84421[30], 6}, {NULL}};
const Segment SGI94515[] = {{&S84421[88], 6}, {NULL}};
const Segment SGI94516[] = {{&S62670[34], 14}, {NULL}};
const Segment SGI94517[] = {{&S62670[27], 6}, {NULL}};
const Segment SGI94518[] = {{&S62670[101], 9}, {NULL}};
const Segment SGI94519[] = {{&S62670[74], 6}, {NULL}};
const Segment SGI94520[] = {{&S62676[34], 14}, {NULL}};
const Segment SGI94521[] = {{&S62676[27], 6}, {NULL}};
const Segment SGI94522[] = {{&S62676[101], 9}, {NULL}};
const Segment SGI94523[] = {{&S62676[74], 6}, {NULL}};
const Segment SGI94524[] = {{&S62700[34], 6}, {NULL}};
const Segment SGI94525[] = {{&S62700[27], 6}, {NULL}};
const Segment SGI94526[] = {{&S62700[93], 9}, {NULL}};
const Segment SGI94527[] = {{&S62700[66], 6}, {NULL}};
const Segment SGI94528[] = {{&S62701[34], 6}, {NULL}};
const Segment SGI94529[] = {{&S62701[27], 6}, {NULL}};
const Segment SGI94530[] = {{&S62701[93], 9}, {NULL}};
const Segment SGI94531[] = {{&S62701[66], 6}, {NULL}};
const Segment SGI94532[] = {{&S62706[34], 9}, {NULL}};
const Segment SGI94533[] = {{&S62706[27], 6}, {NULL}};
const Segment SGI94534[] = {{&S62706[96], 9}, {NULL}};
const Segment SGI94535[] = {{&S62706[69], 6}, {NULL}};
const Segment SGI94536[] = {{&S62707[34], 16}, {NULL}};
const Segment SGI94537[] = {{&S62707[27], 6}, {NULL}};
const Segment SGI94538[] = {{&S62707[103], 9}, {NULL}};
const Segment SGI94539[] = {{&S62707[76], 6}, {NULL}};
const Segment SGI94540[] = {{&S62719[34], 14}, {NULL}};
const Segment SGI94541[] = {{&S62719[27], 6}, {NULL}};
const Segment SGI94542[] = {{&S62719[101], 9}, {NULL}};
const Segment SGI94543[] = {{&S62719[74], 6}, {NULL}};
const Segment SGI94544[] = {{&S84524[44], 11}, {NULL}};
const Segment SGI94545[] = {{&S84524[44], 23}, {NULL}};
const Segment SGI94546[] = {{&S84524[30], 6}, {NULL}};
const Segment SGI94547[] = {{&S84524[115], 3}, {NULL}};
const Segment SGI94548[] = {{&S84524[81], 5}, {NULL}};
const Segment SGI94549[] = {{&S84532[44], 11}, {NULL}};
const Segment SGI94550[] = {{&S84532[44], 28}, {NULL}};
const Segment SGI94551[] = {{&S84532[30], 6}, {NULL}};
const Segment SGI94552[] = {{&S84532[119], 3}, {NULL}};
const Segment SGI94553[] = {{&S84532[86], 5}, {NULL}};
const Segment SGI94554[] = {{&S84532[123], 14}, {NULL}};
const Segment SGI94555[] = {{&S84533[44], 10}, {NULL}};
const Segment SGI94556[] = {{&S84533[44], 27}, {NULL}};
const Segment SGI94557[] = {{&S84533[30], 6}, {NULL}};
const Segment SGI94558[] = {{&S84533[118], 3}, {NULL}};
const Segment SGI94559[] = {{&S84533[85], 5}, {NULL}};
const Segment SGI94560[] = {{&S84533[122], 14}, {NULL}};
const Segment SGI94561[] = {{&S70896[44], 14}, {NULL}};
const Segment SGI94562[] = {{&S70896[44], 27}, {NULL}};
const Segment SGI94563[] = {{&S70896[30], 6}, {NULL}};
const Segment SGI94564[] = {{&S70896[31], 3}, {NULL}};
const Segment SGI94565[] = {{&S70896[85], 6}, {NULL}};
const Segment SGI94566[] = {{&S70896[0], 13}, {NULL}};
const Segment SGI94567[] = {{&S84539[44], 10}, {NULL}};
const Segment SGI94568[] = {{&S84539[44], 27}, {NULL}};
const Segment SGI94569[] = {{&S84539[30], 6}, {NULL}};
const Segment SGI94570[] = {{&S84539[85], 6}, {NULL}};
const Segment SGI94571[] = {{&S84539[112], 13}, {NULL}};
const Segment SGI94572[] = {{&S84562[44], 10}, {NULL}};
const Segment SGI94573[] = {{&S84562[44], 27}, {NULL}};
const Segment SGI94574[] = {{&S84562[30], 6}, {NULL}};
const Segment SGI94575[] = {{&S84562[85], 6}, {NULL}};
const Segment SGI94576[] = {{&S84562[112], 13}, {NULL}};
const Segment SGI94577[] = {{&S77221[44], 7}, {NULL}};
const Segment SGI94578[] = {{&S77221[44], 18}, {NULL}};
const Segment SGI94579[] = {{&S77221[30], 6}, {NULL}};
const Segment SGI94580[] = {{&S77221[129], 3}, {NULL}};
const Segment SGI94581[] = {{&S77221[95], 5}, {NULL}};
const Segment SGI94582[] = {{&S77221[35], 1}, {NULL}};
const Segment SGI94583[] = {{&S77221[64], 11}, {NULL}};
const Segment SGI94584[] = {{&S84621[44], 11}, {NULL}};
const Segment SGI94585[] = {{&S84621[44], 28}, {NULL}};
const Segment SGI94586[] = {{&S84621[30], 6}, {NULL}};
const Segment SGI94587[] = {{&S84621[120], 3}, {NULL}};
const Segment SGI94588[] = {{&S84621[86], 5}, {NULL}};
const Segment SGI94589[] = {{&S84621[154], 3}, {NULL}};
const Segment SGI94590[] = {{&S84622[44], 10}, {NULL}};
const Segment SGI94591[] = {{&S84622[44], 27}, {NULL}};
const Segment SGI94592[] = {{&S84622[30], 6}, {NULL}};
const Segment SGI94593[] = {{&S84622[119], 3}, {NULL}};
const Segment SGI94594[] = {{&S84622[85], 5}, {NULL}};
const Segment SGI94595[] = {{&S84622[153], 3}, {NULL}};
const Segment SGI94596[] = {{&S62886[34], 13}, {NULL}};
const Segment SGI94597[] = {{&S62886[27], 6}, {NULL}};
const Segment SGI94598[] = {{&S62886[100], 9}, {NULL}};
const Segment SGI94599[] = {{&S62886[73], 6}, {NULL}};
const Segment SGI94600[] = {{&S62891[34], 15}, {NULL}};
const Segment SGI94601[] = {{&S62891[27], 6}, {NULL}};
const Segment SGI94602[] = {{&S62891[102], 9}, {NULL}};
const Segment SGI94603[] = {{&S62891[75], 6}, {NULL}};
const Segment SGI94604[] = {{&S62892[34], 21}, {NULL}};
const Segment SGI94605[] = {{&S62892[27], 6}, {NULL}};
const Segment SGI94606[] = {{&S62892[108], 9}, {NULL}};
const Segment SGI94607[] = {{&S62892[81], 6}, {NULL}};
const Segment SGI94608[] = {{&S62893[34], 11}, {NULL}};
const Segment SGI94609[] = {{&S62893[27], 6}, {NULL}};
const Segment SGI94610[] = {{&S62893[98], 9}, {NULL}};
const Segment SGI94611[] = {{&S62893[71], 6}, {NULL}};
const Segment SGI94612[] = {{&S62905[44], 13}, {NULL}};
const Segment SGI94613[] = {{&S62905[44], 25}, {NULL}};
const Segment SGI94614[] = {{&S62905[30], 6}, {NULL}};
const Segment SGI94615[] = {{&S62905[31], 3}, {NULL}};
const Segment SGI94616[] = {{&S62905[83], 6}, {NULL}};
const Segment SGI94617[] = {{&S62906[44], 15}, {NULL}};
const Segment SGI94618[] = {{&S62906[44], 27}, {NULL}};
const Segment SGI94619[] = {{&S62906[30], 6}, {NULL}};
const Segment SGI94620[] = {{&S62906[31], 3}, {NULL}};
const Segment SGI94621[] = {{&S62906[85], 6}, {NULL}};
const Segment SGI94622[] = {{&S62907[61], 13}, {NULL}};
const Segment SGI94623[] = {{&S62907[61], 25}, {NULL}};
const Segment SGI94624[] = {{&S62907[30], 23}, {NULL}};
const Segment SGI94625[] = {{&S62907[31], 3}, {NULL}};
const Segment SGI94626[] = {{&S62907[100], 6}, {NULL}};
const Segment SGI94627[] = {{&S62978[34], 11}, {NULL}};
const Segment SGI94628[] = {{&S62978[27], 6}, {NULL}};
const Segment SGI94629[] = {{&S62978[98], 9}, {NULL}};
const Segment SGI94630[] = {{&S62978[71], 6}, {NULL}};
const Segment SGI94631[] = {{&S77372[44], 9}, {NULL}};
const Segment SGI94632[] = {{&S77372[44], 19}, {NULL}};
const Segment SGI94633[] = {{&S77372[30], 6}, {NULL}};
const Segment SGI94634[] = {{&S77372[111], 3}, {NULL}};
const Segment SGI94635[] = {{&S77372[77], 5}, {NULL}};
const Segment SGI94636[] = {{&S77375[44], 16}, {NULL}};
const Segment SGI94637[] = {{&S77375[44], 26}, {NULL}};
const Segment SGI94638[] = {{&S77375[30], 6}, {NULL}};
const Segment SGI94639[] = {{&S77375[118], 3}, {NULL}};
const Segment SGI94640[] = {{&S77375[84], 5}, {NULL}};
const Segment SGI94641[] = {{&S77376[44], 16}, {NULL}};
const Segment SGI94642[] = {{&S77376[44], 26}, {NULL}};
const Segment SGI94643[] = {{&S77376[30], 6}, {NULL}};
const Segment SGI94644[] = {{&S77376[87], 4}, {NULL}};
const Segment SGI94645[] = {{&S77376[87], 3}, {NULL}};
const Segment SGI94646[] = {{&S62995[34], 14}, {NULL}};
const Segment SGI94647[] = {{&S62995[27], 6}, {NULL}};
const Segment SGI94648[] = {{&S62995[101], 9}, {NULL}};
const Segment SGI94649[] = {{&S62995[74], 6}, {NULL}};
const Segment SGI94650[] = {{&S62996[34], 10}, {NULL}};
const Segment SGI94651[] = {{&S62996[27], 6}, {NULL}};
const Segment SGI94652[] = {{&S62996[97], 9}, {NULL}};
const Segment SGI94653[] = {{&S62996[70], 6}, {NULL}};
const Segment SGI94654[] = {{&S62997[34], 14}, {NULL}};
const Segment SGI94655[] = {{&S62997[27], 6}, {NULL}};
const Segment SGI94656[] = {{&S62997[101], 9}, {NULL}};
const Segment SGI94657[] = {{&S62997[74], 6}, {NULL}};
const Segment SGI94658[] = {{&S71136[34], 14}, {NULL}};
const Segment SGI94659[] = {{&S71136[27], 6}, {NULL}};
const Segment SGI94660[] = {{&S71136[101], 9}, {NULL}};
const Segment SGI94661[] = {{&S71136[74], 6}, {NULL}};
const Segment SGI94662[] = {{&S71136[0], 13}, {NULL}};
const Segment SGI94663[] = {{&S63011[34], 14}, {NULL}};
const Segment SGI94664[] = {{&S63011[27], 6}, {NULL}};
const Segment SGI94665[] = {{&S63011[101], 9}, {NULL}};
const Segment SGI94666[] = {{&S63011[74], 6}, {NULL}};
const Segment SGI94667[] = {{&S63012[34], 15}, {NULL}};
const Segment SGI94668[] = {{&S63012[27], 6}, {NULL}};
const Segment SGI94669[] = {{&S63012[102], 9}, {NULL}};
const Segment SGI94670[] = {{&S63012[75], 6}, {NULL}};
const Segment SGI94671[] = {{&S63013[34], 21}, {NULL}};
const Segment SGI94672[] = {{&S63013[27], 6}, {NULL}};
const Segment SGI94673[] = {{&S63013[108], 9}, {NULL}};
const Segment SGI94674[] = {{&S63013[81], 6}, {NULL}};
const Segment SGI94675[] = {{&S63015[34], 14}, {NULL}};
const Segment SGI94676[] = {{&S63015[27], 6}, {NULL}};
const Segment SGI94677[] = {{&S63015[101], 9}, {NULL}};
const Segment SGI94678[] = {{&S63015[74], 6}, {NULL}};
const Segment SGI94679[] = {{&S63016[34], 16}, {NULL}};
const Segment SGI94680[] = {{&S63016[27], 6}, {NULL}};
const Segment SGI94681[] = {{&S63016[103], 9}, {NULL}};
const Segment SGI94682[] = {{&S63016[76], 6}, {NULL}};
const Segment SGI94683[] = {{&S84685[31], 8}, {NULL}};
const Segment SGI94684[] = {{&S84685[55], 3}, {NULL}};
const Segment SGI94685[] = {{&S84686[39], 8}, {NULL}};
const Segment SGI94686[] = {{&S84686[104], 3}, {NULL}};
const Segment SGI94687[] = {{&S84686[93], 3}, {NULL}};
const Segment SGI94688[] = {{&S84686[70], 6}, {NULL}};
const Segment SGI94689[] = {{&S63025[34], 9}, {NULL}};
const Segment SGI94690[] = {{&S63025[27], 6}, {NULL}};
const Segment SGI94691[] = {{&S63025[96], 9}, {NULL}};
const Segment SGI94692[] = {{&S63025[69], 6}, {NULL}};
const Segment SGI94693[] = {{&S63026[34], 9}, {NULL}};
const Segment SGI94694[] = {{&S63026[27], 6}, {NULL}};
const Segment SGI94695[] = {{&S63026[96], 9}, {NULL}};
const Segment SGI94696[] = {{&S63026[69], 6}, {NULL}};
const Segment SGI94697[] = {{&S84701[59], 11}, {NULL}};
const Segment SGI94698[] = {{&S84701[59], 28}, {NULL}};
const Segment SGI94699[] = {{&S84701[45], 6}, {NULL}};
const Segment SGI94700[] = {{&S84701[135], 3}, {NULL}};
const Segment SGI94701[] = {{&S84701[101], 5}, {NULL}};
const Segment SGI94702[] = {{&S84701[0], 14}, {NULL}};
const Segment SGI94703[] = {{&S63092[44], 9}, {NULL}};
const Segment SGI94704[] = {{&S63092[44], 21}, {NULL}};
const Segment SGI94705[] = {{&S63092[30], 6}, {NULL}};
const Segment SGI94706[] = {{&S63092[31], 3}, {NULL}};
const Segment SGI94707[] = {{&S63092[79], 6}, {NULL}};
const Segment SGI94708[] = {{&S77504[44], 7}, {NULL}};
const Segment SGI94709[] = {{&S77504[44], 24}, {NULL}};
const Segment SGI94710[] = {{&S77504[30], 6}, {NULL}};
const Segment SGI94711[] = {{&S77504[82], 6}, {NULL}};
const Segment SGI94712[] = {{&S77504[109], 13}, {NULL}};
const Segment SGI94713[] = {{&S77505[44], 7}, {NULL}};
const Segment SGI94714[] = {{&S77505[44], 24}, {NULL}};
const Segment SGI94715[] = {{&S77505[30], 6}, {NULL}};
const Segment SGI94716[] = {{&S77505[82], 6}, {NULL}};
const Segment SGI94717[] = {{&S77505[109], 13}, {NULL}};
const Segment SGI94718[] = {{&S63122[44], 8}, {NULL}};
const Segment SGI94719[] = {{&S63122[44], 20}, {NULL}};
const Segment SGI94720[] = {{&S63122[30], 6}, {NULL}};
const Segment SGI94721[] = {{&S63122[31], 3}, {NULL}};
const Segment SGI94722[] = {{&S63122[78], 6}, {NULL}};
const Segment SGI94723[] = {{&S63123[38], 8}, {NULL}};
const Segment SGI94724[] = {{&S63123[31], 6}, {NULL}};
const Segment SGI94725[] = {{&S63140[56], 4}, {NULL}};
const Segment SGI94726[] = {{&S63140[56], 26}, {NULL}};
const Segment SGI94727[] = {{&S63140[42], 6}, {NULL}};
const Segment SGI94728[] = {{&S63140[43], 3}, {NULL}};
const Segment SGI94729[] = {{&S63140[96], 6}, {NULL}};
const Segment SGI94730[] = {{&S63141[54], 4}, {NULL}};
const Segment SGI94731[] = {{&S63141[42], 6}, {NULL}};
const Segment SGI94732[] = {{&S63141[43], 3}, {NULL}};
const Segment SGI94733[] = {{&S63141[94], 6}, {NULL}};
const Segment SGI94734[] = {{&S63142[52], 6}, {NULL}};
const Segment SGI94735[] = {{&S63142[114], 6}, {NULL}};
const Segment SGI94736[] = {{&S63143[44], 7}, {NULL}};
const Segment SGI94737[] = {{&S63143[44], 24}, {NULL}};
const Segment SGI94738[] = {{&S63143[30], 6}, {NULL}};
const Segment SGI94739[] = {{&S63143[85], 6}, {NULL}};
const Segment SGI94740[] = {{&S63144[44], 7}, {NULL}};
const Segment SGI94741[] = {{&S63144[44], 24}, {NULL}};
const Segment SGI94742[] = {{&S63144[30], 6}, {NULL}};
const Segment SGI94743[] = {{&S63144[31], 3}, {NULL}};
const Segment SGI94744[] = {{&S63144[82], 6}, {NULL}};
const Segment SGI94745[] = {{&S63145[38], 7}, {NULL}};
const Segment SGI94746[] = {{&S63145[31], 6}, {NULL}};
const Segment SGI94747[] = {{&S63148[34], 41}, {NULL}};
const Segment SGI94748[] = {{&S63148[27], 6}, {NULL}};
const Segment SGI94749[] = {{&S63148[128], 9}, {NULL}};
const Segment SGI94750[] = {{&S63148[101], 6}, {NULL}};
const Segment SGI94751[] = {{&S63149[34], 41}, {NULL}};
const Segment SGI94752[] = {{&S63149[27], 6}, {NULL}};
const Segment SGI94753[] = {{&S63149[128], 9}, {NULL}};
const Segment SGI94754[] = {{&S63149[101], 6}, {NULL}};
const Segment SGI94755[] = {{&S84825[44], 12}, {NULL}};
const Segment SGI94756[] = {{&S84825[44], 33}, {NULL}};
const Segment SGI94757[] = {{&S84825[30], 6}, {NULL}};
const Segment SGI94758[] = {{&S84825[125], 3}, {NULL}};
const Segment SGI94759[] = {{&S84825[91], 5}, {NULL}};
const Segment SGI94760[] = {{&S77577[44], 6}, {NULL}};
const Segment SGI94761[] = {{&S77577[44], 17}, {NULL}};
const Segment SGI94762[] = {{&S77577[30], 6}, {NULL}};
const Segment SGI94763[] = {{&S77577[138], 3}, {NULL}};
const Segment SGI94764[] = {{&S77577[104], 5}, {NULL}};
const Segment SGI94765[] = {{&S77577[85], 3}, {NULL}};
const Segment SGI94766[] = {{&S77578[44], 6}, {NULL}};
const Segment SGI94767[] = {{&S77578[44], 17}, {NULL}};
const Segment SGI94768[] = {{&S77578[30], 6}, {NULL}};
const Segment SGI94769[] = {{&S77578[128], 3}, {NULL}};
const Segment SGI94770[] = {{&S77578[94], 5}, {NULL}};
const Segment SGI94771[] = {{&S77578[35], 1}, {NULL}};
const Segment SGI94772[] = {{&S77578[63], 11}, {NULL}};
const Segment SGI94773[] = {{&S63188[34], 18}, {NULL}};
const Segment SGI94774[] = {{&S63188[27], 6}, {NULL}};
const Segment SGI94775[] = {{&S63188[105], 9}, {NULL}};
const Segment SGI94776[] = {{&S63188[78], 6}, {NULL}};
const Segment SGI94777[] = {{&S63189[34], 18}, {NULL}};
const Segment SGI94778[] = {{&S63189[27], 6}, {NULL}};
const Segment SGI94779[] = {{&S63189[105], 9}, {NULL}};
const Segment SGI94780[] = {{&S63189[78], 6}, {NULL}};
const Segment SGI94781[] = {{&S63190[34], 11}, {NULL}};
const Segment SGI94782[] = {{&S63190[27], 6}, {NULL}};
const Segment SGI94783[] = {{&S63190[98], 9}, {NULL}};
const Segment SGI94784[] = {{&S63190[71], 6}, {NULL}};
const Segment SGI94785[] = {{&S84870[44], 19}, {NULL}};
const Segment SGI94786[] = {{&S84870[44], 31}, {NULL}};
const Segment SGI94787[] = {{&S84870[30], 6}, {NULL}};
const Segment SGI94788[] = {{&S84870[123], 3}, {NULL}};
const Segment SGI94789[] = {{&S84870[89], 5}, {NULL}};
const Segment SGI94790[] = {{&S84886[45], 10}, {NULL}};
const Segment SGI94791[] = {{&S84886[30], 6}, {NULL}};
const Segment SGI94792[] = {{&S84886[120], 3}, {NULL}};
const Segment SGI94793[] = {{&S84886[86], 5}, {NULL}};
const Segment SGI94794[] = {{&S84886[144], 13}, {NULL}};
const Segment SGI94795[] = {{&S63224[34], 9}, {NULL}};
const Segment SGI94796[] = {{&S63224[27], 6}, {NULL}};
const Segment SGI94797[] = {{&S63224[96], 9}, {NULL}};
const Segment SGI94798[] = {{&S63224[69], 6}, {NULL}};
const Segment SGI94799[] = {{&S63225[34], 10}, {NULL}};
const Segment SGI94800[] = {{&S63225[27], 6}, {NULL}};
const Segment SGI94801[] = {{&S63225[97], 9}, {NULL}};
const Segment SGI94802[] = {{&S63225[70], 6}, {NULL}};
const Segment SGI94803[] = {{&S63226[34], 10}, {NULL}};
const Segment SGI94804[] = {{&S63226[27], 6}, {NULL}};
const Segment SGI94805[] = {{&S63226[97], 9}, {NULL}};
const Segment SGI94806[] = {{&S63226[70], 6}, {NULL}};
const Segment SGI94807[] = {{&S63227[34], 10}, {NULL}};
const Segment SGI94808[] = {{&S63227[27], 6}, {NULL}};
const Segment SGI94809[] = {{&S63227[97], 9}, {NULL}};
const Segment SGI94810[] = {{&S63227[70], 6}, {NULL}};
const Segment SGI94811[] = {{&S77628[44], 7}, {NULL}};
const Segment SGI94812[] = {{&S77628[44], 18}, {NULL}};
const Segment SGI94813[] = {{&S77628[30], 6}, {NULL}};
const Segment SGI94814[] = {{&S77628[110], 3}, {NULL}};
const Segment SGI94815[] = {{&S77628[76], 5}, {NULL}};
const Segment SGI94816[] = {{&S63236[34], 12}, {NULL}};
const Segment SGI94817[] = {{&S63236[27], 6}, {NULL}};
const Segment SGI94818[] = {{&S63236[99], 9}, {NULL}};
const Segment SGI94819[] = {{&S63236[72], 6}, {NULL}};
const Segment SGI94820[] = {{&S63237[34], 11}, {NULL}};
const Segment SGI94821[] = {{&S63237[27], 6}, {NULL}};
const Segment SGI94822[] = {{&S63237[98], 9}, {NULL}};
const Segment SGI94823[] = {{&S63237[71], 6}, {NULL}};
const Segment SGI94824[] = {{&S63238[34], 11}, {NULL}};
const Segment SGI94825[] = {{&S63238[27], 6}, {NULL}};
const Segment SGI94826[] = {{&S63238[98], 9}, {NULL}};
const Segment SGI94827[] = {{&S63238[71], 6}, {NULL}};
const Segment SGI94828[] = {{&S63239[34], 11}, {NULL}};
const Segment SGI94829[] = {{&S63239[27], 6}, {NULL}};
const Segment SGI94830[] = {{&S63239[98], 9}, {NULL}};
const Segment SGI94831[] = {{&S63239[71], 6}, {NULL}};
const Segment SGI94832[] = {{&S63264[34], 10}, {NULL}};
const Segment SGI94833[] = {{&S63264[27], 6}, {NULL}};
const Segment SGI94834[] = {{&S63264[97], 9}, {NULL}};
const Segment SGI94835[] = {{&S63264[70], 6}, {NULL}};
const Segment SGI94836[] = {{&S63313[34], 9}, {NULL}};
const Segment SGI94837[] = {{&S63313[27], 6}, {NULL}};
const Segment SGI94838[] = {{&S63313[96], 9}, {NULL}};
const Segment SGI94839[] = {{&S63313[69], 6}, {NULL}};
const Segment SGI94840[] = {{&S63314[34], 9}, {NULL}};
const Segment SGI94841[] = {{&S63314[27], 6}, {NULL}};
const Segment SGI94842[] = {{&S63314[96], 9}, {NULL}};
const Segment SGI94843[] = {{&S63314[69], 6}, {NULL}};
const Segment SGI94844[] = {{&S63318[44], 7}, {NULL}};
const Segment SGI94845[] = {{&S63318[44], 19}, {NULL}};
const Segment SGI94846[] = {{&S63318[30], 6}, {NULL}};
const Segment SGI94847[] = {{&S63318[31], 3}, {NULL}};
const Segment SGI94848[] = {{&S63318[77], 6}, {NULL}};
const Segment SGI94849[] = {{&S63321[44], 9}, {NULL}};
const Segment SGI94850[] = {{&S63321[44], 21}, {NULL}};
const Segment SGI94851[] = {{&S63321[30], 6}, {NULL}};
const Segment SGI94852[] = {{&S63321[31], 3}, {NULL}};
const Segment SGI94853[] = {{&S63321[79], 6}, {NULL}};
const Segment SGI94854[] = {{&S84998[44], 12}, {NULL}};
const Segment SGI94855[] = {{&S84998[44], 28}, {NULL}};
const Segment SGI94856[] = {{&S84998[30], 6}, {NULL}};
const Segment SGI94857[] = {{&S84998[120], 3}, {NULL}};
const Segment SGI94858[] = {{&S84998[86], 5}, {NULL}};
const Segment SGI94859[] = {{&S63326[34], 12}, {NULL}};
const Segment SGI94860[] = {{&S63326[27], 6}, {NULL}};
const Segment SGI94861[] = {{&S63326[99], 9}, {NULL}};
const Segment SGI94862[] = {{&S63326[72], 6}, {NULL}};
const Segment SGI94863[] = {{&S63327[34], 12}, {NULL}};
const Segment SGI94864[] = {{&S63327[27], 6}, {NULL}};
const Segment SGI94865[] = {{&S63327[99], 9}, {NULL}};
const Segment SGI94866[] = {{&S63327[72], 6}, {NULL}};
const Segment SGI94867[] = {{&S85006[44], 10}, {NULL}};
const Segment SGI94868[] = {{&S85006[44], 27}, {NULL}};
const Segment SGI94869[] = {{&S85006[30], 6}, {NULL}};
const Segment SGI94870[] = {{&S85006[118], 3}, {NULL}};
const Segment SGI94871[] = {{&S85006[85], 5}, {NULL}};
const Segment SGI94872[] = {{&S85006[122], 14}, {NULL}};
const Segment SGI94873[] = {{&S63376[44], 9}, {NULL}};
const Segment SGI94874[] = {{&S63376[44], 21}, {NULL}};
const Segment SGI94875[] = {{&S63376[30], 6}, {NULL}};
const Segment SGI94876[] = {{&S63376[31], 3}, {NULL}};
const Segment SGI94877[] = {{&S63376[79], 6}, {NULL}};
const Segment SGI94878[] = {{&S63377[44], 15}, {NULL}};
const Segment SGI94879[] = {{&S63377[44], 27}, {NULL}};
const Segment SGI94880[] = {{&S63377[30], 6}, {NULL}};
const Segment SGI94881[] = {{&S63377[31], 3}, {NULL}};
const Segment SGI94882[] = {{&S63377[85], 6}, {NULL}};
const Segment SGI94883[] = {{&S63378[38], 15}, {NULL}};
const Segment SGI94884[] = {{&S63378[31], 6}, {NULL}};
const Segment SGI94885[] = {{&S85067[44], 9}, {NULL}};
const Segment SGI94886[] = {{&S85067[44], 21}, {NULL}};
const Segment SGI94887[] = {{&S85067[30], 6}, {NULL}};
const Segment SGI94888[] = {{&S85067[113], 3}, {NULL}};
const Segment SGI94889[] = {{&S85067[79], 5}, {NULL}};
const Segment SGI94890[] = {{&S77770[44], 7}, {NULL}};
const Segment SGI94891[] = {{&S77770[44], 18}, {NULL}};
const Segment SGI94892[] = {{&S77770[30], 6}, {NULL}};
const Segment SGI94893[] = {{&S77770[129], 3}, {NULL}};
const Segment SGI94894[] = {{&S77770[95], 5}, {NULL}};
const Segment SGI94895[] = {{&S77770[35], 1}, {NULL}};
const Segment SGI94896[] = {{&S77770[64], 11}, {NULL}};
const Segment SGI94897[] = {{&S63404[44], 12}, {NULL}};
const Segment SGI94898[] = {{&S63404[44], 21}, {NULL}};
const Segment SGI94899[] = {{&S63404[30], 6}, {NULL}};
const Segment SGI94900[] = {{&S63404[31], 3}, {NULL}};
const Segment SGI94901[] = {{&S63404[79], 6}, {NULL}};
const Segment SGI94902[] = {{&S63405[44], 8}, {NULL}};
const Segment SGI94903[] = {{&S63405[44], 26}, {NULL}};
const Segment SGI94904[] = {{&S63405[30], 6}, {NULL}};
const Segment SGI94905[] = {{&S63405[31], 3}, {NULL}};
const Segment SGI94906[] = {{&S85074[31], 8}, {NULL}};
const Segment SGI94907[] = {{&S85074[54], 3}, {NULL}};
const Segment SGI94908[] = {{&S63405[84], 6}, {NULL}};
const Segment SGI94909[] = {{&S63406[44], 8}, {NULL}};
const Segment SGI94910[] = {{&S63406[44], 24}, {NULL}};
const Segment SGI94911[] = {{&S63406[30], 6}, {NULL}};
const Segment SGI94912[] = {{&S63406[31], 3}, {NULL}};
const Segment SGI94913[] = {{&S63406[82], 6}, {NULL}};
const Segment SGI94914[] = {{&S85079[59], 8}, {NULL}};
const Segment SGI94915[] = {{&S85079[59], 20}, {NULL}};
const Segment SGI94916[] = {{&S85079[45], 6}, {NULL}};
const Segment SGI94917[] = {{&S85079[127], 3}, {NULL}};
const Segment SGI94918[] = {{&S85079[93], 5}, {NULL}};
const Segment SGI94919[] = {{&S85079[0], 14}, {NULL}};
const Segment SGI94920[] = {{&S85080[59], 5}, {NULL}};
const Segment SGI94921[] = {{&S85080[59], 17}, {NULL}};
const Segment SGI94922[] = {{&S85080[45], 6}, {NULL}};
const Segment SGI94923[] = {{&S85080[124], 3}, {NULL}};
const Segment SGI94924[] = {{&S85080[90], 5}, {NULL}};
const Segment SGI94925[] = {{&S85080[0], 14}, {NULL}};
const Segment SGI94926[] = {{&S85084[44], 10}, {NULL}};
const Segment SGI94927[] = {{&S85084[44], 22}, {NULL}};
const Segment SGI94928[] = {{&S85084[30], 6}, {NULL}};
const Segment SGI94929[] = {{&S85084[114], 3}, {NULL}};
const Segment SGI94930[] = {{&S85084[80], 5}, {NULL}};
const Segment SGI94931[] = {{&S63451[44], 12}, {NULL}};
const Segment SGI94932[] = {{&S63451[44], 24}, {NULL}};
const Segment SGI94933[] = {{&S63451[30], 6}, {NULL}};
const Segment SGI94934[] = {{&S63451[31], 3}, {NULL}};
const Segment SGI94935[] = {{&S63451[82], 6}, {NULL}};
const Segment SGI94936[] = {{&S71516[75], 6}, {NULL}};
const Segment SGI94937[] = {{&S71516[0], 10}, {NULL}};
const Segment SGI94938[] = {{&S71518[89], 6}, {NULL}};
const Segment SGI94939[] = {{&S71518[0], 24}, {NULL}};
const Segment SGI94940[] = {{&S63500[44], 6}, {NULL}};
const Segment SGI94941[] = {{&S63500[44], 18}, {NULL}};
const Segment SGI94942[] = {{&S63500[30], 6}, {NULL}};
const Segment SGI94943[] = {{&S63500[31], 3}, {NULL}};
const Segment SGI94944[] = {{&S63500[76], 6}, {NULL}};
const Segment SGI94945[] = {{&S63501[44], 6}, {NULL}};
const Segment SGI94946[] = {{&S63501[44], 18}, {NULL}};
const Segment SGI94947[] = {{&S63501[30], 6}, {NULL}};
const Segment SGI94948[] = {{&S63501[79], 6}, {NULL}};
const Segment SGI94949[] = {{&S63502[38], 6}, {NULL}};
const Segment SGI94950[] = {{&S63502[31], 6}, {NULL}};
const Segment SGI94951[] = {{&S85196[44], 11}, {NULL}};
const Segment SGI94952[] = {{&S85196[44], 28}, {NULL}};
const Segment SGI94953[] = {{&S85196[30], 6}, {NULL}};
const Segment SGI94954[] = {{&S85196[86], 6}, {NULL}};
const Segment SGI94955[] = {{&S85196[113], 13}, {NULL}};
const Segment SGI94956[] = {{&S77871[44], 10}, {NULL}};
const Segment SGI94957[] = {{&S77871[44], 20}, {NULL}};
const Segment SGI94958[] = {{&S77871[30], 6}, {NULL}};
const Segment SGI94959[] = {{&S77871[112], 3}, {NULL}};
const Segment SGI94960[] = {{&S77871[78], 5}, {NULL}};
const Segment SGI94961[] = {{&S77872[44], 10}, {NULL}};
const Segment SGI94962[] = {{&S77872[44], 23}, {NULL}};
const Segment SGI94963[] = {{&S77872[30], 6}, {NULL}};
const Segment SGI94964[] = {{&S77872[115], 3}, {NULL}};
const Segment SGI94965[] = {{&S77872[81], 5}, {NULL}};
const Segment SGI94966[] = {{&S77873[38], 10}, {NULL}};
const Segment SGI94967[] = {{&S77873[31], 6}, {NULL}};
const Segment SGI94968[] = {{&S77875[44], 10}, {NULL}};
const Segment SGI94969[] = {{&S77875[44], 23}, {NULL}};
const Segment SGI94970[] = {{&S77875[30], 6}, {NULL}};
const Segment SGI94971[] = {{&S77875[84], 6}, {NULL}};
const Segment SGI94972[] = {{&S77876[44], 10}, {NULL}};
const Segment SGI94973[] = {{&S77876[44], 23}, {NULL}};
const Segment SGI94974[] = {{&S77876[30], 6}, {NULL}};
const Segment SGI94975[] = {{&S77876[81], 6}, {NULL}};
const Segment SGI94976[] = {{&S63549[44], 14}, {NULL}};
const Segment SGI94977[] = {{&S63549[44], 26}, {NULL}};
const Segment SGI94978[] = {{&S63549[30], 6}, {NULL}};
const Segment SGI94979[] = {{&S63549[31], 3}, {NULL}};
const Segment SGI94980[] = {{&S63549[84], 6}, {NULL}};
const Segment SGI94981[] = {{&S63550[44], 14}, {NULL}};
const Segment SGI94982[] = {{&S63550[44], 26}, {NULL}};
const Segment SGI94983[] = {{&S63550[30], 6}, {NULL}};
const Segment SGI94984[] = {{&S63550[31], 3}, {NULL}};
const Segment SGI94985[] = {{&S63550[84], 6}, {NULL}};
const Segment SGI94986[] = {{&S63551[38], 14}, {NULL}};
const Segment SGI94987[] = {{&S63551[31], 6}, {NULL}};
const Segment SGI94988[] = {{&S63553[44], 14}, {NULL}};
const Segment SGI94989[] = {{&S63553[44], 26}, {NULL}};
const Segment SGI94990[] = {{&S63553[30], 6}, {NULL}};
const Segment SGI94991[] = {{&S63553[129], 3}, {NULL}};
const Segment SGI94992[] = {{&S63553[94], 6}, {NULL}};
const Segment SGI94993[] = {{&S63553[149], 8}, {NULL}};
const Segment SGI94994[] = {{&S68811[34], 14}, {NULL}};
const Segment SGI94995[] = {{&S68811[27], 6}, {NULL}};
const Segment SGI94996[] = {{&S68811[101], 9}, {NULL}};
const Segment SGI94997[] = {{&S68811[74], 6}, {NULL}};
const Segment SGI94998[] = {{&S85265[45], 10}, {NULL}};
const Segment SGI94999[] = {{&S85265[30], 6}, {NULL}};
const Segment SGI95000[] = {{&S85265[120], 3}, {NULL}};
const Segment SGI95001[] = {{&S85265[86], 5}, {NULL}};
const Segment SGI95002[] = {{&S85265[144], 13}, {NULL}};
const Segment SGI95003[] = {{&S85266[45], 11}, {NULL}};
const Segment SGI95004[] = {{&S85266[30], 6}, {NULL}};
const Segment SGI95005[] = {{&S85266[121], 3}, {NULL}};
const Segment SGI95006[] = {{&S85266[87], 5}, {NULL}};
const Segment SGI95007[] = {{&S85266[145], 13}, {NULL}};
const Segment SGI95008[] = {{&S85267[59], 4}, {NULL}};
const Segment SGI95009[] = {{&S85267[59], 16}, {NULL}};
const Segment SGI95010[] = {{&S85267[45], 6}, {NULL}};
const Segment SGI95011[] = {{&S85267[123], 3}, {NULL}};
const Segment SGI95012[] = {{&S85267[89], 5}, {NULL}};
const Segment SGI95013[] = {{&S85267[0], 14}, {NULL}};
const Segment SGI95014[] = {{&S85271[45], 24}, {NULL}};
const Segment SGI95015[] = {{&S85271[30], 6}, {NULL}};
const Segment SGI95016[] = {{&S85271[129], 3}, {NULL}};
const Segment SGI95017[] = {{&S63605[34], 21}, {NULL}};
const Segment SGI95018[] = {{&S85271[95], 5}, {NULL}};
const Segment SGI95019[] = {{&S63605[27], 6}, {NULL}};
const Segment SGI95020[] = {{&S85271[153], 13}, {NULL}};
const Segment SGI95021[] = {{&S63605[108], 9}, {NULL}};
const Segment SGI95022[] = {{&S63605[81], 6}, {NULL}};
const Segment SGI95023[] = {{&S63606[34], 15}, {NULL}};
const Segment SGI95024[] = {{&S63606[27], 6}, {NULL}};
const Segment SGI95025[] = {{&S63606[102], 9}, {NULL}};
const Segment SGI95026[] = {{&S63606[75], 6}, {NULL}};
const Segment SGI95027[] = {{&S63608[34], 11}, {NULL}};
const Segment SGI95028[] = {{&S63608[27], 6}, {NULL}};
const Segment SGI95029[] = {{&S63608[98], 9}, {NULL}};
const Segment SGI95030[] = {{&S63608[71], 6}, {NULL}};
const Segment SGI95031[] = {{&S63612[34], 6}, {NULL}};
const Segment SGI95032[] = {{&S63612[27], 6}, {NULL}};
const Segment SGI95033[] = {{&S63612[93], 9}, {NULL}};
const Segment SGI95034[] = {{&S63612[66], 6}, {NULL}};
const Segment SGI95035[] = {{&S63614[34], 16}, {NULL}};
const Segment SGI95036[] = {{&S63614[27], 6}, {NULL}};
const Segment SGI95037[] = {{&S63614[103], 9}, {NULL}};
const Segment SGI95038[] = {{&S63614[76], 6}, {NULL}};
const Segment SGI95039[] = {{&S63615[34], 9}, {NULL}};
const Segment SGI95040[] = {{&S63615[27], 6}, {NULL}};
const Segment SGI95041[] = {{&S63615[96], 9}, {NULL}};
const Segment SGI95042[] = {{&S63615[69], 6}, {NULL}};
const Segment SGI95043[] = {{&S85344[31], 8}, {NULL}};
const Segment SGI95044[] = {{&S85344[54], 3}, {NULL}};
const Segment SGI95045[] = {{&S63678[44], 7}, {NULL}};
const Segment SGI95046[] = {{&S63678[44], 19}, {NULL}};
const Segment SGI95047[] = {{&S63678[30], 6}, {NULL}};
const Segment SGI95048[] = {{&S63678[31], 3}, {NULL}};
const Segment SGI95049[] = {{&S63678[77], 6}, {NULL}};
const Segment SGI95050[] = {{&S63680[44], 8}, {NULL}};
const Segment SGI95051[] = {{&S63680[44], 20}, {NULL}};
const Segment SGI95052[] = {{&S63680[30], 6}, {NULL}};
const Segment SGI95053[] = {{&S63680[31], 3}, {NULL}};
const Segment SGI95054[] = {{&S63680[78], 6}, {NULL}};
const Segment SGI95055[] = {{&S63681[44], 9}, {NULL}};
const Segment SGI95056[] = {{&S63681[44], 21}, {NULL}};
const Segment SGI95057[] = {{&S63681[30], 6}, {NULL}};
const Segment SGI95058[] = {{&S63681[31], 3}, {NULL}};
const Segment SGI95059[] = {{&S63681[79], 6}, {NULL}};
const Segment SGI95060[] = {{&S85362[59], 15}, {NULL}};
const Segment SGI95061[] = {{&S85362[59], 36}, {NULL}};
const Segment SGI95062[] = {{&S85362[45], 6}, {NULL}};
const Segment SGI95063[] = {{&S85362[143], 3}, {NULL}};
const Segment SGI95064[] = {{&S85362[109], 5}, {NULL}};
const Segment SGI95065[] = {{&S85362[0], 14}, {NULL}};
const Segment SGI95066[] = {{&S63706[44], 13}, {NULL}};
const Segment SGI95067[] = {{&S63706[44], 25}, {NULL}};
const Segment SGI95068[] = {{&S63706[30], 6}, {NULL}};
const Segment SGI95069[] = {{&S63706[31], 3}, {NULL}};
const Segment SGI95070[] = {{&S63706[83], 6}, {NULL}};
const Segment SGI95071[] = {{&S85377[45], 4}, {NULL}};
const Segment SGI95072[] = {{&S85377[30], 6}, {NULL}};
const Segment SGI95073[] = {{&S85377[109], 3}, {NULL}};
const Segment SGI95074[] = {{&S85377[75], 5}, {NULL}};
const Segment SGI95075[] = {{&S85377[133], 13}, {NULL}};
const Segment SGI95076[] = {{&S77329[44], 7}, {NULL}};
const Segment SGI95077[] = {{&S77329[44], 24}, {NULL}};
const Segment SGI95078[] = {{&S77329[30], 6}, {NULL}};
const Segment SGI95079[] = {{&S85378[45], 12}, {NULL}};
const Segment SGI95080[] = {{&S77329[116], 3}, {NULL}};
const Segment SGI95081[] = {{&S85378[30], 6}, {NULL}};
const Segment SGI95082[] = {{&S77329[82], 5}, {NULL}};
const Segment SGI95083[] = {{&S85378[117], 3}, {NULL}};
const Segment SGI95084[] = {{&S85378[83], 5}, {NULL}};
const Segment SGI95085[] = {{&S85378[141], 13}, {NULL}};
const Segment SGI95086[] = {{&S85379[44], 12}, {NULL}};
const Segment SGI95087[] = {{&S85379[44], 24}, {NULL}};
const Segment SGI95088[] = {{&S85379[30], 6}, {NULL}};
const Segment SGI95089[] = {{&S85379[117], 3}, {NULL}};
const Segment SGI95090[] = {{&S85379[82], 6}, {NULL}};
const Segment SGI95091[] = {{&S85379[137], 8}, {NULL}};
const Segment SGI95092[] = {{&S85379[146], 13}, {NULL}};
const Segment SGI95093[] = {{&S63721[44], 6}, {NULL}};
const Segment SGI95094[] = {{&S63721[44], 18}, {NULL}};
const Segment SGI95095[] = {{&S63721[30], 6}, {NULL}};
const Segment SGI95096[] = {{&S63721[31], 3}, {NULL}};
const Segment SGI95097[] = {{&S63721[76], 6}, {NULL}};
const Segment SGI95098[] = {{&S63722[44], 6}, {NULL}};
const Segment SGI95099[] = {{&S63722[44], 18}, {NULL}};
const Segment SGI95100[] = {{&S63722[30], 6}, {NULL}};
const Segment SGI95101[] = {{&S63722[31], 3}, {NULL}};
const Segment SGI95102[] = {{&S63722[76], 6}, {NULL}};
const Segment SGI95103[] = {{&S63733[34], 10}, {NULL}};
const Segment SGI95104[] = {{&S63733[27], 6}, {NULL}};
const Segment SGI95105[] = {{&S63733[97], 9}, {NULL}};
const Segment SGI95106[] = {{&S63733[70], 6}, {NULL}};
const Segment SGI95107[] = {{&S85397[52], 4}, {NULL}};
const Segment SGI95108[] = {{&S85397[52], 16}, {NULL}};
const Segment SGI95109[] = {{&S85397[38], 6}, {NULL}};
const Segment SGI95110[] = {{&S85397[82], 5}, {NULL}};
const Segment SGI95111[] = {{&S85397[0], 14}, {NULL}};
const Segment SGI95112[] = {{&S63734[34], 9}, {NULL}};
const Segment SGI95113[] = {{&S63734[27], 6}, {NULL}};
const Segment SGI95114[] = {{&S63734[96], 9}, {NULL}};
const Segment SGI95115[] = {{&S63734[69], 6}, {NULL}};
const Segment SGI95116[] = {{&S63735[34], 10}, {NULL}};
const Segment SGI95117[] = {{&S63735[27], 6}, {NULL}};
const Segment SGI95118[] = {{&S63735[97], 9}, {NULL}};
const Segment SGI95119[] = {{&S63735[70], 6}, {NULL}};
const Segment SGI95120[] = {{&S63736[34], 10}, {NULL}};
const Segment SGI95121[] = {{&S63736[27], 6}, {NULL}};
const Segment SGI95122[] = {{&S63736[97], 9}, {NULL}};
const Segment SGI95123[] = {{&S63736[70], 6}, {NULL}};
const Segment SGI95124[] = {{&S63745[34], 10}, {NULL}};
const Segment SGI95125[] = {{&S63745[27], 6}, {NULL}};
const Segment SGI95126[] = {{&S63745[97], 9}, {NULL}};
const Segment SGI95127[] = {{&S63745[70], 6}, {NULL}};
const Segment SGI95128[] = {{&S63745[138], 8}, {NULL}};
const Segment SGI95129[] = {{&S85473[45], 11}, {NULL}};
const Segment SGI95130[] = {{&S85473[30], 6}, {NULL}};
const Segment SGI95131[] = {{&S85473[121], 3}, {NULL}};
const Segment SGI95132[] = {{&S85473[87], 5}, {NULL}};
const Segment SGI95133[] = {{&S85473[145], 13}, {NULL}};
const Segment SGI95134[] = {{&S63810[34], 17}, {NULL}};
const Segment SGI95135[] = {{&S63810[27], 6}, {NULL}};
const Segment SGI95136[] = {{&S63810[104], 9}, {NULL}};
const Segment SGI95137[] = {{&S63810[77], 6}, {NULL}};
const Segment SGI95138[] = {{&S85474[45], 10}, {NULL}};
const Segment SGI95139[] = {{&S85474[30], 6}, {NULL}};
const Segment SGI95140[] = {{&S85474[120], 3}, {NULL}};
const Segment SGI95141[] = {{&S85474[86], 5}, {NULL}};
const Segment SGI95142[] = {{&S85474[144], 13}, {NULL}};
const Segment SGI95143[] = {{&S63819[34], 7}, {NULL}};
const Segment SGI95144[] = {{&S63819[27], 6}, {NULL}};
const Segment SGI95145[] = {{&S63819[94], 9}, {NULL}};
const Segment SGI95146[] = {{&S63819[67], 6}, {NULL}};
const Segment SGI95147[] = {{&S85483[61], 10}, {NULL}};
const Segment SGI95148[] = {{&S85483[61], 27}, {NULL}};
const Segment SGI95149[] = {{&S85483[47], 6}, {NULL}};
const Segment SGI95150[] = {{&S85483[136], 3}, {NULL}};
const Segment SGI95151[] = {{&S85483[102], 5}, {NULL}};
const Segment SGI95152[] = {{&S85483[0], 14}, {NULL}};
const Segment SGI95153[] = {{&S63834[45], 10}, {NULL}};
const Segment SGI95154[] = {{&S63834[30], 6}, {NULL}};
const Segment SGI95155[] = {{&S63834[31], 3}, {NULL}};
const Segment SGI95156[] = {{&S63834[81], 5}, {NULL}};
const Segment SGI95157[] = {{&S63834[139], 13}, {NULL}};
const Segment SGI95158[] = {{&S63837[45], 11}, {NULL}};
const Segment SGI95159[] = {{&S63837[30], 6}, {NULL}};
const Segment SGI95160[] = {{&S63837[31], 3}, {NULL}};
const Segment SGI95161[] = {{&S63837[82], 5}, {NULL}};
const Segment SGI95162[] = {{&S63837[140], 13}, {NULL}};
const Segment SGI95163[] = {{&S63838[45], 11}, {NULL}};
const Segment SGI95164[] = {{&S63838[30], 6}, {NULL}};
const Segment SGI95165[] = {{&S63838[31], 3}, {NULL}};
const Segment SGI95166[] = {{&S63838[82], 5}, {NULL}};
const Segment SGI95167[] = {{&S63838[140], 13}, {NULL}};
const Segment SGI95168[] = {{&S85505[44], 21}, {NULL}};
const Segment SGI95169[] = {{&S85505[44], 33}, {NULL}};
const Segment SGI95170[] = {{&S85505[30], 6}, {NULL}};
const Segment SGI95171[] = {{&S85505[125], 3}, {NULL}};
const Segment SGI95172[] = {{&S85505[91], 5}, {NULL}};
const Segment SGI95173[] = {{&S85506[42], 21}, {NULL}};
const Segment SGI95174[] = {{&S85506[30], 6}, {NULL}};
const Segment SGI95175[] = {{&S85506[123], 3}, {NULL}};
const Segment SGI95176[] = {{&S85506[89], 5}, {NULL}};
const Segment SGI95177[] = {{&S63866[34], 7}, {NULL}};
const Segment SGI95178[] = {{&S63866[27], 6}, {NULL}};
const Segment SGI95179[] = {{&S63866[94], 9}, {NULL}};
const Segment SGI95180[] = {{&S63866[67], 6}, {NULL}};
const Segment SGI95181[] = {{&S63907[44], 8}, {NULL}};
const Segment SGI95182[] = {{&S63907[44], 20}, {NULL}};
const Segment SGI95183[] = {{&S63907[30], 6}, {NULL}};
const Segment SGI95184[] = {{&S63907[31], 3}, {NULL}};
const Segment SGI95185[] = {{&S63907[78], 6}, {NULL}};
const Segment SGI95186[] = {{&S85570[45], 10}, {NULL}};
const Segment SGI95187[] = {{&S85570[30], 6}, {NULL}};
const Segment SGI95188[] = {{&S85570[120], 3}, {NULL}};
const Segment SGI95189[] = {{&S85570[86], 5}, {NULL}};
const Segment SGI95190[] = {{&S85570[144], 13}, {NULL}};
const Segment SGI95191[] = {{&S85576[59], 10}, {NULL}};
const Segment SGI95192[] = {{&S85576[59], 27}, {NULL}};
const Segment SGI95193[] = {{&S85576[45], 6}, {NULL}};
const Segment SGI95194[] = {{&S85576[134], 3}, {NULL}};
const Segment SGI95195[] = {{&S85576[100], 5}, {NULL}};
const Segment SGI95196[] = {{&S85576[0], 14}, {NULL}};
const Segment SGI95197[] = {{&S85586[44], 10}, {NULL}};
const Segment SGI95198[] = {{&S85586[44], 27}, {NULL}};
const Segment SGI95199[] = {{&S85586[30], 6}, {NULL}};
const Segment SGI95200[] = {{&S85586[163], 3}, {NULL}};
const Segment SGI95201[] = {{&S85586[129], 3}, {NULL}};
const Segment SGI95202[] = {{&S85586[94], 6}, {NULL}};
const Segment SGI95203[] = {{&S85586[149], 8}, {NULL}};
const Segment SGI95204[] = {{&S85595[44], 16}, {NULL}};
const Segment SGI95205[] = {{&S85595[44], 28}, {NULL}};
const Segment SGI95206[] = {{&S85595[30], 6}, {NULL}};
const Segment SGI95207[] = {{&S85595[120], 3}, {NULL}};
const Segment SGI95208[] = {{&S85595[86], 5}, {NULL}};
const Segment SGI95209[] = {{&S63964[34], 14}, {NULL}};
const Segment SGI95210[] = {{&S63964[27], 6}, {NULL}};
const Segment SGI95211[] = {{&S63964[101], 9}, {NULL}};
const Segment SGI95212[] = {{&S63964[74], 6}, {NULL}};
const Segment SGI95213[] = {{&S63966[34], 8}, {NULL}};
const Segment SGI95214[] = {{&S63966[27], 6}, {NULL}};
const Segment SGI95215[] = {{&S63966[95], 9}, {NULL}};
const Segment SGI95216[] = {{&S63966[68], 6}, {NULL}};
const Segment SGI95217[] = {{&S85624[44], 10}, {NULL}};
const Segment SGI95218[] = {{&S85624[44], 27}, {NULL}};
const Segment SGI95219[] = {{&S85624[30], 6}, {NULL}};
const Segment SGI95220[] = {{&S85624[85], 6}, {NULL}};
const Segment SGI95221[] = {{&S85624[112], 13}, {NULL}};
const Segment SGI95222[] = {{&S85625[44], 11}, {NULL}};
const Segment SGI95223[] = {{&S85625[44], 28}, {NULL}};
const Segment SGI95224[] = {{&S85625[30], 6}, {NULL}};
const Segment SGI95225[] = {{&S85625[86], 6}, {NULL}};
const Segment SGI95226[] = {{&S85625[113], 13}, {NULL}};
const Segment SGI95227[] = {{&S85626[44], 10}, {NULL}};
const Segment SGI95228[] = {{&S85626[44], 27}, {NULL}};
const Segment SGI95229[] = {{&S85626[30], 6}, {NULL}};
const Segment SGI95230[] = {{&S85626[85], 6}, {NULL}};
const Segment SGI95231[] = {{&S85626[112], 13}, {NULL}};
const Segment SGI95232[] = {{&S85673[44], 11}, {NULL}};
const Segment SGI95233[] = {{&S85673[44], 28}, {NULL}};
const Segment SGI95234[] = {{&S85673[30], 6}, {NULL}};
const Segment SGI95235[] = {{&S85673[120], 3}, {NULL}};
const Segment SGI95236[] = {{&S85673[86], 5}, {NULL}};
const Segment SGI95237[] = {{&S78334[44], 9}, {NULL}};
const Segment SGI95238[] = {{&S78334[44], 20}, {NULL}};
const Segment SGI95239[] = {{&S78334[30], 6}, {NULL}};
const Segment SGI95240[] = {{&S78334[131], 3}, {NULL}};
const Segment SGI95241[] = {{&S78334[97], 5}, {NULL}};
const Segment SGI95242[] = {{&S78334[35], 1}, {NULL}};
const Segment SGI95243[] = {{&S78334[66], 11}, {NULL}};
const Segment SGI95244[] = {{&S85683[44], 14}, {NULL}};
const Segment SGI95245[] = {{&S85683[44], 26}, {NULL}};
const Segment SGI95246[] = {{&S85683[30], 6}, {NULL}};
const Segment SGI95247[] = {{&S85683[118], 3}, {NULL}};
const Segment SGI95248[] = {{&S85683[84], 5}, {NULL}};
const Segment SGI95249[] = {{&S64060[34], 14}, {NULL}};
const Segment SGI95250[] = {{&S64060[27], 6}, {NULL}};
const Segment SGI95251[] = {{&S64060[101], 9}, {NULL}};
const Segment SGI95252[] = {{&S64060[74], 6}, {NULL}};
const Segment SGI95253[] = {{&S85698[44], 9}, {NULL}};
const Segment SGI95254[] = {{&S85698[44], 21}, {NULL}};
const Segment SGI95255[] = {{&S85698[30], 6}, {NULL}};
const Segment SGI95256[] = {{&S64069[34], 18}, {NULL}};
const Segment SGI95257[] = {{&S64069[27], 6}, {NULL}};
const Segment SGI95258[] = {{&S85698[113], 3}, {NULL}};
const Segment SGI95259[] = {{&S85698[79], 5}, {NULL}};
const Segment SGI95260[] = {{&S64069[105], 9}, {NULL}};
const Segment SGI95261[] = {{&S64069[78], 6}, {NULL}};
const Segment SGI95262[] = {{&S85704[45], 10}, {NULL}};
const Segment SGI95263[] = {{&S85704[30], 6}, {NULL}};
const Segment SGI95264[] = {{&S85704[120], 3}, {NULL}};
const Segment SGI95265[] = {{&S85704[86], 5}, {NULL}};
const Segment SGI95266[] = {{&S85704[144], 13}, {NULL}};
const Segment SGI95267[] = {{&S64092[34], 11}, {NULL}};
const Segment SGI95268[] = {{&S64092[27], 6}, {NULL}};
const Segment SGI95269[] = {{&S64092[98], 9}, {NULL}};
const Segment SGI95270[] = {{&S64092[71], 6}, {NULL}};
const Segment SGI95271[] = {{&S85726[44], 24}, {NULL}};
const Segment SGI95272[] = {{&S85726[44], 36}, {NULL}};
const Segment SGI95273[] = {{&S85726[30], 6}, {NULL}};
const Segment SGI95274[] = {{&S85726[94], 6}, {NULL}};
const Segment SGI95275[] = {{&S85726[121], 13}, {NULL}};
const Segment SGI95276[] = {{&S64115[44], 22}, {NULL}};
const Segment SGI95277[] = {{&S64115[44], 34}, {NULL}};
const Segment SGI95278[] = {{&S64115[30], 6}, {NULL}};
const Segment SGI95279[] = {{&S85739[44], 25}, {NULL}};
const Segment SGI95280[] = {{&S85739[44], 37}, {NULL}};
const Segment SGI95281[] = {{&S85739[30], 6}, {NULL}};
const Segment SGI95282[] = {{&S64115[31], 3}, {NULL}};
const Segment SGI95283[] = {{&S64115[92], 6}, {NULL}};
const Segment SGI95284[] = {{&S85739[129], 3}, {NULL}};
const Segment SGI95285[] = {{&S85739[95], 5}, {NULL}};
const Segment SGI95286[] = {{&S64117[38], 22}, {NULL}};
const Segment SGI95287[] = {{&S64117[31], 6}, {NULL}};
const Segment SGI95288[] = {{&S64118[44], 8}, {NULL}};
const Segment SGI95289[] = {{&S64118[44], 20}, {NULL}};
const Segment SGI95290[] = {{&S64118[30], 6}, {NULL}};
const Segment SGI95291[] = {{&S64118[31], 3}, {NULL}};
const Segment SGI95292[] = {{&S64118[78], 6}, {NULL}};
const Segment SGI95293[] = {{&S64119[34], 15}, {NULL}};
const Segment SGI95294[] = {{&S64119[27], 6}, {NULL}};
const Segment SGI95295[] = {{&S64119[102], 9}, {NULL}};
const Segment SGI95296[] = {{&S64119[75], 6}, {NULL}};
const Segment SGI95297[] = {{&S64120[34], 21}, {NULL}};
const Segment SGI95298[] = {{&S64120[27], 6}, {NULL}};
const Segment SGI95299[] = {{&S64120[108], 9}, {NULL}};
const Segment SGI95300[] = {{&S64120[81], 6}, {NULL}};
const Segment SGI95301[] = {{&S64121[34], 11}, {NULL}};
const Segment SGI95302[] = {{&S64121[27], 6}, {NULL}};
const Segment SGI95303[] = {{&S64121[98], 9}, {NULL}};
const Segment SGI95304[] = {{&S64121[71], 6}, {NULL}};
const Segment SGI95305[] = {{&S85756[44], 10}, {NULL}};
const Segment SGI95306[] = {{&S85756[44], 27}, {NULL}};
const Segment SGI95307[] = {{&S85756[30], 6}, {NULL}};
const Segment SGI95308[] = {{&S85756[85], 6}, {NULL}};
const Segment SGI95309[] = {{&S85756[112], 13}, {NULL}};
const Segment SGI95310[] = {{&S64131[34], 10}, {NULL}};
const Segment SGI95311[] = {{&S64131[27], 6}, {NULL}};
const Segment SGI95312[] = {{&S64131[97], 9}, {NULL}};
const Segment SGI95313[] = {{&S64131[70], 6}, {NULL}};
const Segment SGI95314[] = {{&S64132[34], 9}, {NULL}};
const Segment SGI95315[] = {{&S64132[27], 6}, {NULL}};
const Segment SGI95316[] = {{&S64132[96], 9}, {NULL}};
const Segment SGI95317[] = {{&S64132[69], 6}, {NULL}};
const Segment SGI95318[] = {{&S64133[34], 10}, {NULL}};
const Segment SGI95319[] = {{&S64133[27], 6}, {NULL}};
const Segment SGI95320[] = {{&S64133[97], 9}, {NULL}};
const Segment SGI95321[] = {{&S64133[70], 6}, {NULL}};
const Segment SGI95322[] = {{&S64134[34], 10}, {NULL}};
const Segment SGI95323[] = {{&S64134[27], 6}, {NULL}};
const Segment SGI95324[] = {{&S64134[97], 9}, {NULL}};
const Segment SGI95325[] = {{&S64134[70], 6}, {NULL}};
const Segment SGI95326[] = {{&S85803[45], 24}, {NULL}};
const Segment SGI95327[] = {{&S85803[30], 6}, {NULL}};
const Segment SGI95328[] = {{&S85803[129], 3}, {NULL}};
const Segment SGI95329[] = {{&S85803[95], 5}, {NULL}};
const Segment SGI95330[] = {{&S85803[153], 13}, {NULL}};
const Segment SGI95331[] = {{&S85807[44], 7}, {NULL}};
const Segment SGI95332[] = {{&S85807[44], 19}, {NULL}};
const Segment SGI95333[] = {{&S85807[30], 6}, {NULL}};
const Segment SGI95334[] = {{&S85807[111], 3}, {NULL}};
const Segment SGI95335[] = {{&S85807[77], 5}, {NULL}};
const Segment SGI95336[] = {{&S78486[44], 7}, {NULL}};
const Segment SGI95337[] = {{&S78486[44], 18}, {NULL}};
const Segment SGI95338[] = {{&S78486[30], 6}, {NULL}};
const Segment SGI95339[] = {{&S78486[110], 3}, {NULL}};
const Segment SGI95340[] = {{&S78486[76], 5}, {NULL}};
const Segment SGI95341[] = {{&S78494[44], 11}, {NULL}};
const Segment SGI95342[] = {{&S78494[44], 29}, {NULL}};
const Segment SGI95343[] = {{&S78494[30], 6}, {NULL}};
const Segment SGI95344[] = {{&S78494[128], 3}, {NULL}};
const Segment SGI95345[] = {{&S85818[45], 10}, {NULL}};
const Segment SGI95346[] = {{&S85818[30], 6}, {NULL}};
const Segment SGI95347[] = {{&S78494[94], 5}, {NULL}};
const Segment SGI95348[] = {{&S78494[73], 8}, {NULL}};
const Segment SGI95349[] = {{&S85818[120], 3}, {NULL}};
const Segment SGI95350[] = {{&S85818[86], 5}, {NULL}};
const Segment SGI95351[] = {{&S85818[144], 13}, {NULL}};
const Segment SGI95352[] = {{&S78495[44], 11}, {NULL}};
const Segment SGI95353[] = {{&S78495[44], 28}, {NULL}};
const Segment SGI95354[] = {{&S78495[30], 6}, {NULL}};
const Segment SGI95355[] = {{&S78495[127], 3}, {NULL}};
const Segment SGI95356[] = {{&S78495[93], 5}, {NULL}};
const Segment SGI95357[] = {{&S78495[72], 8}, {NULL}};
const Segment SGI95358[] = {{&S64209[44], 8}, {NULL}};
const Segment SGI95359[] = {{&S64209[44], 26}, {NULL}};
const Segment SGI95360[] = {{&S64209[30], 6}, {NULL}};
const Segment SGI95361[] = {{&S64209[31], 3}, {NULL}};
const Segment SGI95362[] = {{&S64209[84], 6}, {NULL}};
const Segment SGI95363[] = {{&S64210[38], 8}, {NULL}};
const Segment SGI95364[] = {{&S64210[31], 6}, {NULL}};
const Segment SGI95365[] = {{&S78510[34], 7}, {NULL}};
const Segment SGI95366[] = {{&S78510[27], 6}, {NULL}};
const Segment SGI95367[] = {{&S78510[106], 2}, {NULL}};
const Segment SGI95368[] = {{&S78510[72], 6}, {NULL}};
const Segment SGI95369[] = {{&S78510[140], 8}, {NULL}};
const Segment SGI95370[] = {{&S64217[34], 7}, {NULL}};
const Segment SGI95371[] = {{&S64217[27], 6}, {NULL}};
const Segment SGI95372[] = {{&S64217[94], 9}, {NULL}};
const Segment SGI95373[] = {{&S64217[67], 6}, {NULL}};
const Segment SGI95374[] = {{&S64236[34], 10}, {NULL}};
const Segment SGI95375[] = {{&S64236[27], 6}, {NULL}};
const Segment SGI95376[] = {{&S64236[97], 9}, {NULL}};
const Segment SGI95377[] = {{&S64236[70], 6}, {NULL}};
const Segment SGI95378[] = {{&S85875[44], 10}, {NULL}};
const Segment SGI95379[] = {{&S85875[44], 27}, {NULL}};
const Segment SGI95380[] = {{&S85875[30], 6}, {NULL}};
const Segment SGI95381[] = {{&S85875[119], 3}, {NULL}};
const Segment SGI95382[] = {{&S85875[85], 5}, {NULL}};
const Segment SGI95383[] = {{&S85875[143], 13}, {NULL}};
const Segment SGI95384[] = {{&S64286[44], 15}, {NULL}};
const Segment SGI95385[] = {{&S64286[44], 27}, {NULL}};
const Segment SGI95386[] = {{&S64286[30], 6}, {NULL}};
const Segment SGI95387[] = {{&S64286[31], 3}, {NULL}};
const Segment SGI95388[] = {{&S64286[85], 6}, {NULL}};
const Segment SGI95389[] = {{&S77684[42], 11}, {NULL}};
const Segment SGI95390[] = {{&S77684[42], 23}, {NULL}};
const Segment SGI95391[] = {{&S77684[30], 4}, {NULL}};
const Segment SGI95392[] = {{&S77684[113], 3}, {NULL}};
const Segment SGI95393[] = {{&S77684[79], 5}, {NULL}};
const Segment SGI95394[] = {{&S77685[44], 11}, {NULL}};
const Segment SGI95395[] = {{&S77685[44], 23}, {NULL}};
const Segment SGI95396[] = {{&S77685[30], 6}, {NULL}};
const Segment SGI95397[] = {{&S77685[115], 3}, {NULL}};
const Segment SGI95398[] = {{&S77685[81], 5}, {NULL}};
const Segment SGI95399[] = {{&S77686[42], 11}, {NULL}};
const Segment SGI95400[] = {{&S77686[42], 23}, {NULL}};
const Segment SGI95401[] = {{&S77686[30], 4}, {NULL}};
const Segment SGI95402[] = {{&S77686[113], 3}, {NULL}};
const Segment SGI95403[] = {{&S77686[79], 5}, {NULL}};
const Segment SGI95404[] = {{&S85929[44], 11}, {NULL}};
const Segment SGI95405[] = {{&S85929[44], 34}, {NULL}};
const Segment SGI95406[] = {{&S85929[30], 6}, {NULL}};
const Segment SGI95407[] = {{&S85929[126], 3}, {NULL}};
const Segment SGI95408[] = {{&S85929[92], 5}, {NULL}};
const Segment SGI95409[] = {{&S85979[44], 14}, {NULL}};
const Segment SGI95410[] = {{&S85979[44], 26}, {NULL}};
const Segment SGI95411[] = {{&S85979[30], 6}, {NULL}};
const Segment SGI95412[] = {{&S85979[118], 3}, {NULL}};
const Segment SGI95413[] = {{&S85979[84], 5}, {NULL}};
const Segment SGI95414[] = {{&S78677[44], 10}, {NULL}};
const Segment SGI95415[] = {{&S78677[44], 21}, {NULL}};
const Segment SGI95416[] = {{&S78677[30], 6}, {NULL}};
const Segment SGI95417[] = {{&S78677[132], 3}, {NULL}};
const Segment SGI95418[] = {{&S78677[98], 5}, {NULL}};
const Segment SGI95419[] = {{&S78677[35], 1}, {NULL}};
const Segment SGI95420[] = {{&S78677[67], 11}, {NULL}};
const Segment SGI95421[] = {{&S78678[44], 10}, {NULL}};
const Segment SGI95422[] = {{&S78678[44], 21}, {NULL}};
const Segment SGI95423[] = {{&S78678[30], 6}, {NULL}};
const Segment SGI95424[] = {{&S78678[159], 3}, {NULL}};
const Segment SGI95425[] = {{&S78678[125], 5}, {NULL}};
const Segment SGI95426[] = {{&S78678[67], 11}, {NULL}};
const Segment SGI95427[] = {{&S69393[34], 14}, {NULL}};
const Segment SGI95428[] = {{&S69393[27], 6}, {NULL}};
const Segment SGI95429[] = {{&S69393[101], 9}, {NULL}};
const Segment SGI95430[] = {{&S69393[74], 6}, {NULL}};
const Segment SGI95431[] = {{&S64420[44], 10}, {NULL}};
const Segment SGI95432[] = {{&S64420[44], 22}, {NULL}};
const Segment SGI95433[] = {{&S64420[30], 6}, {NULL}};
const Segment SGI95434[] = {{&S64420[139], 4}, {NULL}};
const Segment SGI95435[] = {{&S64420[165], 3}, {NULL}};
const Segment SGI95436[] = {{&S64420[31], 3}, {&S64420[214], 5}, {NULL}};
const Segment SGI95437[] = {{&S64420[80], 6}, {NULL}};
const Segment SGI95438[] = {{&S86045[44], 39}, {NULL}};
const Segment SGI95439[] = {{&S86045[30], 6}, {NULL}};
const Segment SGI95440[] = {{&S86045[115], 5}, {NULL}};
const Segment SGI95441[] = {{&S86046[44], 15}, {NULL}};
const Segment SGI95442[] = {{&S86046[44], 33}, {NULL}};
const Segment SGI95443[] = {{&S86046[30], 6}, {NULL}};
const Segment SGI95444[] = {{&S86046[94], 6}, {NULL}};
const Segment SGI95445[] = {{&S86047[44], 39}, {NULL}};
const Segment SGI95446[] = {{&S86047[30], 6}, {NULL}};
const Segment SGI95447[] = {{&S86047[115], 5}, {NULL}};
const Segment SGI95448[] = {{&S86048[44], 39}, {NULL}};
const Segment SGI95449[] = {{&S86048[30], 6}, {NULL}};
const Segment SGI95450[] = {{&S86048[115], 5}, {NULL}};
const Segment SGI95451[] = {{&S78728[44], 10}, {NULL}};
const Segment SGI95452[] = {{&S78728[44], 22}, {NULL}};
const Segment SGI95453[] = {{&S78728[30], 6}, {NULL}};
const Segment SGI95454[] = {{&S78728[133], 3}, {NULL}};
const Segment SGI95455[] = {{&S78728[99], 5}, {NULL}};
const Segment SGI95456[] = {{&S78728[35], 1}, {NULL}};
const Segment SGI95457[] = {{&S78728[68], 11}, {NULL}};
const Segment SGI95458[] = {{&S78729[44], 10}, {NULL}};
const Segment SGI95459[] = {{&S78729[44], 27}, {NULL}};
const Segment SGI95460[] = {{&S78729[30], 6}, {NULL}};
const Segment SGI95461[] = {{&S78729[138], 3}, {NULL}};
const Segment SGI95462[] = {{&S78729[104], 5}, {NULL}};
const Segment SGI95463[] = {{&S78729[35], 1}, {NULL}};
const Segment SGI95464[] = {{&S78729[73], 11}, {NULL}};
const Segment SGI95465[] = {{&S86082[34], 10}, {NULL}};
const Segment SGI95466[] = {{&S86082[27], 6}, {NULL}};
const Segment SGI95467[] = {{&S64473[45], 10}, {NULL}};
const Segment SGI95468[] = {{&S64473[30], 6}, {NULL}};
const Segment SGI95469[] = {{&S86082[109], 2}, {NULL}};
const Segment SGI95470[] = {{&S86082[75], 6}, {NULL}};
const Segment SGI95471[] = {{&S64473[31], 3}, {NULL}};
const Segment SGI95472[] = {{&S86082[143], 8}, {NULL}};
const Segment SGI95473[] = {{&S64473[81], 5}, {NULL}};
const Segment SGI95474[] = {{&S64473[139], 13}, {NULL}};
const Segment SGI95475[] = {{&S64476[45], 11}, {NULL}};
const Segment SGI95476[] = {{&S64476[30], 6}, {NULL}};
const Segment SGI95477[] = {{&S64476[31], 3}, {NULL}};
const Segment SGI95478[] = {{&S64476[82], 5}, {NULL}};
const Segment SGI95479[] = {{&S64476[140], 13}, {NULL}};
const Segment SGI95480[] = {{&S86138[45], 5}, {NULL}};
const Segment SGI95481[] = {{&S86138[30], 6}, {NULL}};
const Segment SGI95482[] = {{&S86138[110], 3}, {NULL}};
const Segment SGI95483[] = {{&S86138[76], 5}, {NULL}};
const Segment SGI95484[] = {{&S86138[134], 13}, {NULL}};
const Segment SGI95485[] = {{&S78776[44], 13}, {NULL}};
const Segment SGI95486[] = {{&S78776[44], 24}, {NULL}};
const Segment SGI95487[] = {{&S78776[30], 6}, {NULL}};
const Segment SGI95488[] = {{&S78776[135], 3}, {NULL}};
const Segment SGI95489[] = {{&S78776[101], 5}, {NULL}};
const Segment SGI95490[] = {{&S78776[35], 1}, {NULL}};
const Segment SGI95491[] = {{&S78776[70], 11}, {NULL}};
const Segment SGI95492[] = {{&S78777[44], 13}, {NULL}};
const Segment SGI95493[] = {{&S78777[44], 24}, {NULL}};
const Segment SGI95494[] = {{&S78777[30], 6}, {NULL}};
const Segment SGI95495[] = {{&S78777[135], 3}, {NULL}};
const Segment SGI95496[] = {{&S78777[101], 5}, {NULL}};
const Segment SGI95497[] = {{&S78777[35], 1}, {NULL}};
const Segment SGI95498[] = {{&S78777[74], 11}, {NULL}};
const Segment SGI95499[] = {{&S64608[44], 6}, {NULL}};
const Segment SGI95500[] = {{&S64608[44], 18}, {NULL}};
const Segment SGI95501[] = {{&S64608[30], 6}, {NULL}};
const Segment SGI95502[] = {{&S64608[31], 3}, {NULL}};
const Segment SGI95503[] = {{&S64608[76], 6}, {NULL}};
const Segment SGI95504[] = {{&S64610[38], 6}, {NULL}};
const Segment SGI95505[] = {{&S64610[31], 6}, {NULL}};
const Segment SGI95506[] = {{&S86242[31], 12}, {NULL}};
const Segment SGI95507[] = {{&S86242[58], 3}, {NULL}};
const Segment SGI95508[] = {{&S64652[34], 7}, {NULL}};
const Segment SGI95509[] = {{&S64652[27], 6}, {NULL}};
const Segment SGI95510[] = {{&S64652[94], 9}, {NULL}};
const Segment SGI95511[] = {{&S64652[67], 6}, {NULL}};
const Segment SGI95512[] = {{&S78942[44], 7}, {NULL}};
const Segment SGI95513[] = {{&S78942[44], 18}, {NULL}};
const Segment SGI95514[] = {{&S78942[30], 6}, {NULL}};
const Segment SGI95515[] = {{&S78942[125], 3}, {NULL}};
const Segment SGI95516[] = {{&S78942[91], 5}, {NULL}};
const Segment SGI95517[] = {{&S78942[35], 1}, {NULL}};
const Segment SGI95518[] = {{&S78942[64], 11}, {NULL}};
const Segment SGI95519[] = {{&S64737[44], 9}, {NULL}};
const Segment SGI95520[] = {{&S64737[44], 21}, {NULL}};
const Segment SGI95521[] = {{&S64737[30], 6}, {NULL}};
const Segment SGI95522[] = {{&S64737[31], 3}, {NULL}};
const Segment SGI95523[] = {{&S64737[79], 6}, {NULL}};
const Segment SGI95524[] = {{&S86347[44], 18}, {NULL}};
const Segment SGI95525[] = {{&S86347[44], 45}, {NULL}};
const Segment SGI95526[] = {{&S86347[30], 6}, {NULL}};
const Segment SGI95527[] = {{&S86347[137], 3}, {NULL}};
const Segment SGI95528[] = {{&S86347[103], 5}, {NULL}};
const Segment SGI95529[] = {{&S86361[44], 11}, {NULL}};
const Segment SGI95530[] = {{&S86361[44], 23}, {NULL}};
const Segment SGI95531[] = {{&S86361[30], 6}, {NULL}};
const Segment SGI95532[] = {{&S86361[115], 3}, {NULL}};
const Segment SGI95533[] = {{&S86361[81], 5}, {NULL}};
const Segment SGI95534[] = {{&S86368[44], 10}, {NULL}};
const Segment SGI95535[] = {{&S86368[44], 27}, {NULL}};
const Segment SGI95536[] = {{&S86368[30], 6}, {NULL}};
const Segment SGI95537[] = {{&S86368[85], 6}, {NULL}};
const Segment SGI95538[] = {{&S86368[112], 13}, {NULL}};
const Segment SGI95539[] = {{&S64777[44], 17}, {NULL}};
const Segment SGI95540[] = {{&S64777[44], 29}, {NULL}};
const Segment SGI95541[] = {{&S64777[30], 6}, {NULL}};
const Segment SGI95542[] = {{&S64777[31], 3}, {NULL}};
const Segment SGI95543[] = {{&S64777[87], 6}, {NULL}};
const Segment SGI95544[] = {{&S64778[38], 17}, {NULL}};
const Segment SGI95545[] = {{&S64778[31], 6}, {NULL}};
const Segment SGI95546[] = {{&S64779[44], 4}, {NULL}};
const Segment SGI95547[] = {{&S64779[44], 16}, {NULL}};
const Segment SGI95548[] = {{&S64779[30], 6}, {NULL}};
const Segment SGI95549[] = {{&S64779[31], 3}, {NULL}};
const Segment SGI95550[] = {{&S64779[74], 6}, {NULL}};
const Segment SGI95551[] = {{&S64780[44], 4}, {NULL}};
const Segment SGI95552[] = {{&S64780[44], 16}, {NULL}};
const Segment SGI95553[] = {{&S64780[30], 6}, {NULL}};
const Segment SGI95554[] = {{&S64780[31], 3}, {NULL}};
const Segment SGI95555[] = {{&S64780[74], 6}, {NULL}};
const Segment SGI95556[] = {{&S86385[34], 18}, {NULL}};
const Segment SGI95557[] = {{&S86385[27], 6}, {NULL}};
const Segment SGI95558[] = {{&S86385[117], 2}, {NULL}};
const Segment SGI95559[] = {{&S86385[83], 6}, {NULL}};
const Segment SGI95560[] = {{&S86385[151], 8}, {NULL}};
const Segment SGI95561[] = {{&S78190[42], 11}, {NULL}};
const Segment SGI95562[] = {{&S78190[42], 23}, {NULL}};
const Segment SGI95563[] = {{&S78190[30], 4}, {NULL}};
const Segment SGI95564[] = {{&S78190[114], 3}, {NULL}};
const Segment SGI95565[] = {{&S78190[79], 6}, {NULL}};
const Segment SGI95566[] = {{&S72721[75], 6}, {NULL}};
const Segment SGI95567[] = {{&S72721[0], 10}, {NULL}};
const Segment SGI95568[] = {{&S86438[45], 10}, {NULL}};
const Segment SGI95569[] = {{&S86438[30], 6}, {NULL}};
const Segment SGI95570[] = {{&S86438[120], 3}, {NULL}};
const Segment SGI95571[] = {{&S86438[86], 5}, {NULL}};
const Segment SGI95572[] = {{&S86438[144], 13}, {NULL}};
const Segment SGI95573[] = {{&S86448[44], 21}, {NULL}};
const Segment SGI95574[] = {{&S86448[44], 38}, {NULL}};
const Segment SGI95575[] = {{&S86448[30], 6}, {NULL}};
const Segment SGI95576[] = {{&S86448[130], 3}, {NULL}};
const Segment SGI95577[] = {{&S86448[96], 5}, {NULL}};
const Segment SGI95578[] = {{&S64860[34], 16}, {NULL}};
const Segment SGI95579[] = {{&S64860[27], 6}, {NULL}};
const Segment SGI95580[] = {{&S64860[103], 9}, {NULL}};
const Segment SGI95581[] = {{&S64860[76], 6}, {NULL}};
const Segment SGI95582[] = {{&S64861[34], 16}, {NULL}};
const Segment SGI95583[] = {{&S64861[27], 6}, {NULL}};
const Segment SGI95584[] = {{&S64861[103], 9}, {NULL}};
const Segment SGI95585[] = {{&S64861[76], 6}, {NULL}};
const Segment SGI95586[] = {{&S86467[59], 10}, {NULL}};
const Segment SGI95587[] = {{&S86467[59], 27}, {NULL}};
const Segment SGI95588[] = {{&S86467[45], 6}, {NULL}};
const Segment SGI95589[] = {{&S86467[134], 3}, {NULL}};
const Segment SGI95590[] = {{&S86467[100], 5}, {NULL}};
const Segment SGI95591[] = {{&S86467[0], 14}, {NULL}};
const Segment SGI95592[] = {{&S86470[44], 13}, {NULL}};
const Segment SGI95593[] = {{&S86470[44], 25}, {NULL}};
const Segment SGI95594[] = {{&S86470[30], 6}, {NULL}};
const Segment SGI95595[] = {{&S86470[117], 3}, {NULL}};
const Segment SGI95596[] = {{&S86470[83], 5}, {NULL}};
const Segment SGI95597[] = {{&S64883[34], 17}, {NULL}};
const Segment SGI95598[] = {{&S64883[27], 6}, {NULL}};
const Segment SGI95599[] = {{&S64883[112], 9}, {NULL}};
const Segment SGI95600[] = {{&S64883[85], 6}, {NULL}};
const Segment SGI95601[] = {{&S64884[34], 17}, {NULL}};
const Segment SGI95602[] = {{&S64884[27], 6}, {NULL}};
const Segment SGI95603[] = {{&S64884[112], 9}, {NULL}};
const Segment SGI95604[] = {{&S64884[85], 6}, {NULL}};
const Segment SGI95605[] = {{&S64886[34], 13}, {NULL}};
const Segment SGI95606[] = {{&S64886[27], 6}, {NULL}};
const Segment SGI95607[] = {{&S64886[108], 9}, {NULL}};
const Segment SGI95608[] = {{&S64886[81], 6}, {NULL}};
const Segment SGI95609[] = {{&S79145[44], 12}, {NULL}};
const Segment SGI95610[] = {{&S79145[44], 23}, {NULL}};
const Segment SGI95611[] = {{&S79145[30], 6}, {NULL}};
const Segment SGI95612[] = {{&S79145[134], 3}, {NULL}};
const Segment SGI95613[] = {{&S79145[100], 5}, {NULL}};
const Segment SGI95614[] = {{&S79145[35], 1}, {NULL}};
const Segment SGI95615[] = {{&S79145[69], 11}, {NULL}};
const Segment SGI95616[] = {{&S86503[44], 10}, {NULL}};
const Segment SGI95617[] = {{&S86503[44], 27}, {NULL}};
const Segment SGI95618[] = {{&S86503[30], 6}, {NULL}};
const Segment SGI95619[] = {{&S86503[119], 3}, {NULL}};
const Segment SGI95620[] = {{&S86503[85], 5}, {NULL}};
const Segment SGI95621[] = {{&S86503[143], 7}, {NULL}};
const Segment SGI95622[] = {{&S86560[45], 11}, {NULL}};
const Segment SGI95623[] = {{&S86560[30], 6}, {NULL}};
const Segment SGI95624[] = {{&S86560[121], 3}, {NULL}};
const Segment SGI95625[] = {{&S86560[87], 5}, {NULL}};
const Segment SGI95626[] = {{&S86560[145], 13}, {NULL}};
const Segment SGI95627[] = {{&S64911[34], 16}, {NULL}};
const Segment SGI95628[] = {{&S64911[27], 6}, {NULL}};
const Segment SGI95629[] = {{&S64911[103], 9}, {NULL}};
const Segment SGI95630[] = {{&S64911[76], 6}, {NULL}};
const Segment SGI95631[] = {{&S86562[45], 10}, {NULL}};
const Segment SGI95632[] = {{&S86562[30], 6}, {NULL}};
const Segment SGI95633[] = {{&S86562[120], 3}, {NULL}};
const Segment SGI95634[] = {{&S86562[86], 5}, {NULL}};
const Segment SGI95635[] = {{&S86562[144], 13}, {NULL}};
const Segment SGI95636[] = {{&S64912[34], 9}, {NULL}};
const Segment SGI95637[] = {{&S64912[27], 6}, {NULL}};
const Segment SGI95638[] = {{&S64912[96], 9}, {NULL}};
const Segment SGI95639[] = {{&S64912[69], 6}, {NULL}};
const Segment SGI95640[] = {{&S86586[45], 10}, {NULL}};
const Segment SGI95641[] = {{&S86586[30], 6}, {NULL}};
const Segment SGI95642[] = {{&S86586[120], 3}, {NULL}};
const Segment SGI95643[] = {{&S86586[86], 5}, {NULL}};
const Segment SGI95644[] = {{&S86586[144], 13}, {NULL}};
const Segment SGI95645[] = {{&S86589[45], 24}, {NULL}};
const Segment SGI95646[] = {{&S86589[30], 6}, {NULL}};
const Segment SGI95647[] = {{&S86589[129], 3}, {NULL}};
const Segment SGI95648[] = {{&S86589[95], 5}, {NULL}};
const Segment SGI95649[] = {{&S86589[153], 13}, {NULL}};
const Segment SGI95650[] = {{&S86591[44], 13}, {NULL}};
const Segment SGI95651[] = {{&S86591[44], 25}, {NULL}};
const Segment SGI95652[] = {{&S86591[30], 6}, {NULL}};
const Segment SGI95653[] = {{&S86591[117], 3}, {NULL}};
const Segment SGI95654[] = {{&S86591[83], 5}, {NULL}};
const Segment SGI95655[] = {{&S64962[44], 11}, {NULL}};
const Segment SGI95656[] = {{&S64962[44], 23}, {NULL}};
const Segment SGI95657[] = {{&S64962[30], 6}, {NULL}};
const Segment SGI95658[] = {{&S64962[31], 3}, {NULL}};
const Segment SGI95659[] = {{&S64962[81], 6}, {NULL}};
const Segment SGI95660[] = {{&S64963[44], 4}, {NULL}};
const Segment SGI95661[] = {{&S64963[44], 16}, {NULL}};
const Segment SGI95662[] = {{&S64963[30], 6}, {NULL}};
const Segment SGI95663[] = {{&S64963[31], 3}, {NULL}};
const Segment SGI95664[] = {{&S64963[74], 6}, {NULL}};
const Segment SGI95665[] = {{&S75986[32], 17}, {NULL}};
const Segment SGI95666[] = {{&S75986[27], 4}, {NULL}};
const Segment SGI95667[] = {{&S75986[102], 9}, {NULL}};
const Segment SGI95668[] = {{&S75986[75], 6}, {NULL}};
const Segment SGI95669[] = {{&S86644[44], 10}, {NULL}};
const Segment SGI95670[] = {{&S86644[44], 27}, {NULL}};
const Segment SGI95671[] = {{&S86644[30], 6}, {NULL}};
const Segment SGI95672[] = {{&S86644[120], 3}, {NULL}};
const Segment SGI95673[] = {{&S86644[85], 6}, {NULL}};
const Segment SGI95674[] = {{&S86644[140], 8}, {NULL}};
const Segment SGI95675[] = {{&S86644[149], 13}, {NULL}};
const Segment SGI95676[] = {{&S86674[58], 15}, {NULL}};
const Segment SGI95677[] = {{&S86674[58], 36}, {NULL}};
const Segment SGI95678[] = {{&S86674[44], 6}, {NULL}};
const Segment SGI95679[] = {{&S86674[142], 3}, {NULL}};
const Segment SGI95680[] = {{&S86674[108], 5}, {NULL}};
const Segment SGI95681[] = {{&S86674[0], 14}, {NULL}};
const Segment SGI95682[] = {{&S73031[44], 11}, {NULL}};
const Segment SGI95683[] = {{&S73031[44], 35}, {NULL}};
const Segment SGI95684[] = {{&S73031[30], 6}, {NULL}};
const Segment SGI95685[] = {{&S73031[31], 3}, {NULL}};
const Segment SGI95686[] = {{&S73031[93], 5}, {NULL}};
const Segment SGI95687[] = {{&S73031[0], 13}, {NULL}};
const Segment SGI95688[] = {{&S86783[39], 8}, {NULL}};
const Segment SGI95689[] = {{&S86783[104], 3}, {NULL}};
const Segment SGI95690[] = {{&S86783[93], 3}, {NULL}};
const Segment SGI95691[] = {{&S86783[70], 6}, {NULL}};
const Segment SGI95692[] = {{&S86784[31], 8}, {NULL}};
const Segment SGI95693[] = {{&S86784[55], 3}, {NULL}};
const Segment SGI95694[] = {{&S65045[44], 6}, {NULL}};
const Segment SGI95695[] = {{&S65045[44], 18}, {NULL}};
const Segment SGI95696[] = {{&S65045[30], 6}, {NULL}};
const Segment SGI95697[] = {{&S65045[31], 3}, {NULL}};
const Segment SGI95698[] = {{&S65045[76], 6}, {NULL}};
const Segment SGI95699[] = {{&S86817[44], 11}, {NULL}};
const Segment SGI95700[] = {{&S86817[44], 23}, {NULL}};
const Segment SGI95701[] = {{&S86817[30], 6}, {NULL}};
const Segment SGI95702[] = {{&S86817[115], 3}, {NULL}};
const Segment SGI95703[] = {{&S86817[81], 5}, {NULL}};
const Segment SGI95704[] = {{&S65167[30], 6}, {NULL}};
const Segment SGI95705[] = {{&S65167[85], 9}, {NULL}};
const Segment SGI95706[] = {{&S65167[57], 6}, {NULL}};
const Segment SGI95707[] = {{&S86947[45], 5}, {NULL}};
const Segment SGI95708[] = {{&S86947[30], 6}, {NULL}};
const Segment SGI95709[] = {{&S86947[110], 3}, {NULL}};
const Segment SGI95710[] = {{&S86947[76], 5}, {NULL}};
const Segment SGI95711[] = {{&S86947[134], 13}, {NULL}};
const Segment SGI95712[] = {{&S86954[59], 10}, {NULL}};
const Segment SGI95713[] = {{&S86954[59], 27}, {NULL}};
const Segment SGI95714[] = {{&S86954[45], 6}, {NULL}};
const Segment SGI95715[] = {{&S86954[134], 3}, {NULL}};
const Segment SGI95716[] = {{&S86954[100], 5}, {NULL}};
const Segment SGI95717[] = {{&S86954[0], 14}, {NULL}};
const Segment SGI95718[] = {{&S86957[44], 13}, {NULL}};
const Segment SGI95719[] = {{&S86957[44], 34}, {NULL}};
const Segment SGI95720[] = {{&S86957[30], 6}, {NULL}};
const Segment SGI95721[] = {{&S86957[126], 3}, {NULL}};
const Segment SGI95722[] = {{&S86957[92], 5}, {NULL}};
const Segment SGI95723[] = {{&S86958[44], 13}, {NULL}};
const Segment SGI95724[] = {{&S86958[44], 30}, {NULL}};
const Segment SGI95725[] = {{&S86958[30], 6}, {NULL}};
const Segment SGI95726[] = {{&S86958[122], 3}, {NULL}};
const Segment SGI95727[] = {{&S86958[88], 5}, {NULL}};
const Segment SGI95728[] = {{&S79637[44], 14}, {NULL}};
const Segment SGI95729[] = {{&S79637[44], 25}, {NULL}};
const Segment SGI95730[] = {{&S79637[30], 6}, {NULL}};
const Segment SGI95731[] = {{&S79637[86], 4}, {NULL}};
const Segment SGI95732[] = {{&S79637[86], 3}, {NULL}};
const Segment SGI95733[] = {{&S79639[44], 14}, {NULL}};
const Segment SGI95734[] = {{&S79639[44], 25}, {NULL}};
const Segment SGI95735[] = {{&S79639[30], 6}, {NULL}};
const Segment SGI95736[] = {{&S79639[86], 6}, {NULL}};
const Segment SGI95737[] = {{&S79640[44], 14}, {NULL}};
const Segment SGI95738[] = {{&S79640[44], 25}, {NULL}};
const Segment SGI95739[] = {{&S79640[30], 6}, {NULL}};
const Segment SGI95740[] = {{&S79640[117], 3}, {NULL}};
const Segment SGI95741[] = {{&S79640[83], 5}, {NULL}};
const Segment SGI95742[] = {{&S65221[34], 10}, {NULL}};
const Segment SGI95743[] = {{&S65221[27], 6}, {NULL}};
const Segment SGI95744[] = {{&S65221[97], 9}, {NULL}};
const Segment SGI95745[] = {{&S65221[70], 6}, {NULL}};
const Segment SGI95746[] = {{&S65224[44], 10}, {NULL}};
const Segment SGI95747[] = {{&S65224[44], 27}, {NULL}};
const Segment SGI95748[] = {{&S65224[30], 6}, {NULL}};
const Segment SGI95749[] = {{&S65224[31], 3}, {NULL}};
const Segment SGI95750[] = {{&S65224[85], 6}, {NULL}};
const Segment SGI95751[] = {{&S65227[44], 10}, {NULL}};
const Segment SGI95752[] = {{&S65227[44], 22}, {NULL}};
const Segment SGI95753[] = {{&S65227[30], 6}, {NULL}};
const Segment SGI95754[] = {{&S87076[44], 10}, {NULL}};
const Segment SGI95755[] = {{&S65227[31], 3}, {NULL}};
const Segment SGI95756[] = {{&S87076[44], 27}, {NULL}};
const Segment SGI95757[] = {{&S87076[30], 6}, {NULL}};
const Segment SGI95758[] = {{&S65227[80], 6}, {NULL}};
const Segment SGI95759[] = {{&S87076[126], 3}, {NULL}};
const Segment SGI95760[] = {{&S87076[92], 5}, {NULL}};
const Segment SGI95761[] = {{&S87076[71], 8}, {NULL}};
const Segment SGI95762[] = {{&S87077[44], 10}, {NULL}};
const Segment SGI95763[] = {{&S87077[44], 27}, {NULL}};
const Segment SGI95764[] = {{&S87077[30], 6}, {NULL}};
const Segment SGI95765[] = {{&S87077[119], 3}, {NULL}};
const Segment SGI95766[] = {{&S87077[85], 5}, {NULL}};
const Segment SGI95767[] = {{&S87077[143], 7}, {NULL}};
const Segment SGI95768[] = {{&S79827[44], 10}, {NULL}};
const Segment SGI95769[] = {{&S79827[44], 27}, {NULL}};
const Segment SGI95770[] = {{&S79827[30], 6}, {NULL}};
const Segment SGI95771[] = {{&S79827[138], 3}, {NULL}};
const Segment SGI95772[] = {{&S79827[104], 5}, {NULL}};
const Segment SGI95773[] = {{&S79827[35], 1}, {NULL}};
const Segment SGI95774[] = {{&S79827[73], 11}, {NULL}};
const Segment SGI95775[] = {{&S65303[34], 6}, {NULL}};
const Segment SGI95776[] = {{&S65303[27], 6}, {NULL}};
const Segment SGI95777[] = {{&S65303[93], 9}, {NULL}};
const Segment SGI95778[] = {{&S65303[66], 6}, {NULL}};
const Segment SGI95779[] = {{&S65343[44], 9}, {NULL}};
const Segment SGI95780[] = {{&S65343[44], 21}, {NULL}};
const Segment SGI95781[] = {{&S65343[30], 6}, {NULL}};
const Segment SGI95782[] = {{&S65343[31], 3}, {NULL}};
const Segment SGI95783[] = {{&S65343[79], 6}, {NULL}};
const Segment SGI95784[] = {{&S65344[38], 9}, {NULL}};
const Segment SGI95785[] = {{&S65344[31], 6}, {NULL}};
const Segment SGI95786[] = {{&S87208[31], 3}, {NULL}};
const Segment SGI95787[] = {{&S87208[49], 3}, {NULL}};
const Segment SGI95788[] = {{&S65391[34], 18}, {NULL}};
const Segment SGI95789[] = {{&S65391[27], 6}, {NULL}};
const Segment SGI95790[] = {{&S65391[105], 9}, {NULL}};
const Segment SGI95791[] = {{&S65391[78], 6}, {NULL}};
const Segment SGI95792[] = {{&S65392[34], 17}, {NULL}};
const Segment SGI95793[] = {{&S65392[27], 6}, {NULL}};
const Segment SGI95794[] = {{&S65392[104], 9}, {NULL}};
const Segment SGI95795[] = {{&S65392[77], 6}, {NULL}};
const Segment SGI95796[] = {{&S65393[34], 23}, {NULL}};
const Segment SGI95797[] = {{&S65393[27], 6}, {NULL}};
const Segment SGI95798[] = {{&S65393[110], 9}, {NULL}};
const Segment SGI95799[] = {{&S87241[44], 4}, {NULL}};
const Segment SGI95800[] = {{&S65393[83], 6}, {NULL}};
const Segment SGI95801[] = {{&S87241[44], 16}, {NULL}};
const Segment SGI95802[] = {{&S87241[30], 6}, {NULL}};
const Segment SGI95803[] = {{&S87241[74], 6}, {NULL}};
const Segment SGI95804[] = {{&S87241[101], 13}, {NULL}};
const Segment SGI95805[] = {{&S65418[34], 7}, {NULL}};
const Segment SGI95806[] = {{&S65418[27], 6}, {NULL}};
const Segment SGI95807[] = {{&S65418[99], 9}, {NULL}};
const Segment SGI95808[] = {{&S65418[72], 6}, {NULL}};
const Segment SGI95809[] = {{&S78988[42], 11}, {NULL}};
const Segment SGI95810[] = {{&S78988[42], 23}, {NULL}};
const Segment SGI95811[] = {{&S78988[30], 4}, {NULL}};
const Segment SGI95812[] = {{&S78988[114], 3}, {NULL}};
const Segment SGI95813[] = {{&S78988[79], 6}, {NULL}};
const Segment SGI95814[] = {{&S87320[44], 21}, {NULL}};
const Segment SGI95815[] = {{&S65473[45], 10}, {NULL}};
const Segment SGI95816[] = {{&S87320[44], 33}, {NULL}};
const Segment SGI95817[] = {{&S87320[30], 6}, {NULL}};
const Segment SGI95818[] = {{&S65473[30], 6}, {NULL}};
const Segment SGI95819[] = {{&S87320[125], 3}, {NULL}};
const Segment SGI95820[] = {{&S65473[31], 3}, {NULL}};
const Segment SGI95821[] = {{&S87320[91], 5}, {NULL}};
const Segment SGI95822[] = {{&S65473[81], 5}, {NULL}};
const Segment SGI95823[] = {{&S65473[139], 13}, {NULL}};
const Segment SGI95824[] = {{&S65475[45], 11}, {NULL}};
const Segment SGI95825[] = {{&S65475[30], 6}, {NULL}};
const Segment SGI95826[] = {{&S65475[31], 3}, {NULL}};
const Segment SGI95827[] = {{&S65475[82], 5}, {NULL}};
const Segment SGI95828[] = {{&S65475[140], 13}, {NULL}};
const Segment SGI95829[] = {{&S87324[44], 11}, {NULL}};
const Segment SGI95830[] = {{&S87324[44], 28}, {NULL}};
const Segment SGI95831[] = {{&S87324[30], 6}, {NULL}};
const Segment SGI95832[] = {{&S87324[120], 3}, {NULL}};
const Segment SGI95833[] = {{&S87324[86], 5}, {NULL}};
const Segment SGI95834[] = {{&S87324[144], 11}, {NULL}};
const Segment SGI95835[] = {{&S87349[44], 5}, {NULL}};
const Segment SGI95836[] = {{&S87349[44], 17}, {NULL}};
const Segment SGI95837[] = {{&S87349[30], 6}, {NULL}};
const Segment SGI95838[] = {{&S87349[75], 6}, {NULL}};
const Segment SGI95839[] = {{&S87349[102], 13}, {NULL}};
const Segment SGI95840[] = {{&S87350[44], 5}, {NULL}};
const Segment SGI95841[] = {{&S87350[44], 17}, {NULL}};
const Segment SGI95842[] = {{&S87350[30], 6}, {NULL}};
const Segment SGI95843[] = {{&S87350[75], 6}, {NULL}};
const Segment SGI95844[] = {{&S87350[102], 13}, {NULL}};
const Segment SGI95845[] = {{&S87353[44], 4}, {NULL}};
const Segment SGI95846[] = {{&S87353[44], 16}, {NULL}};
const Segment SGI95847[] = {{&S87353[30], 6}, {NULL}};
const Segment SGI95848[] = {{&S87353[74], 6}, {NULL}};
const Segment SGI95849[] = {{&S87353[101], 13}, {NULL}};
const Segment SGI95850[] = {{&S65520[6], 4}, {NULL}};
const Segment SGI95851[] = {{&S65520[82], 5}, {NULL}};
const Segment SGI95852[] = {{&S87364[44], 12}, {NULL}};
const Segment SGI95853[] = {{&S87364[44], 24}, {NULL}};
const Segment SGI95854[] = {{&S87364[30], 6}, {NULL}};
const Segment SGI95855[] = {{&S87364[116], 3}, {NULL}};
const Segment SGI95856[] = {{&S87364[82], 5}, {NULL}};
const Segment SGI95857[] = {{&S87365[34], 10}, {NULL}};
const Segment SGI95858[] = {{&S87365[27], 6}, {NULL}};
const Segment SGI95859[] = {{&S65526[34], 14}, {NULL}};
const Segment SGI95860[] = {{&S65526[27], 6}, {NULL}};
const Segment SGI95861[] = {{&S87365[104], 2}, {NULL}};
const Segment SGI95862[] = {{&S87365[70], 6}, {NULL}};
const Segment SGI95863[] = {{&S65526[106], 9}, {NULL}};
const Segment SGI95864[] = {{&S65526[79], 6}, {NULL}};
const Segment SGI95865[] = {{&S87365[138], 8}, {NULL}};
const Segment SGI95866[] = {{&S65527[34], 15}, {NULL}};
const Segment SGI95867[] = {{&S65527[27], 6}, {NULL}};
const Segment SGI95868[] = {{&S65527[115], 9}, {NULL}};
const Segment SGI95869[] = {{&S65527[88], 6}, {NULL}};
const Segment SGI95870[] = {{&S65528[34], 5}, {NULL}};
const Segment SGI95871[] = {{&S65528[27], 6}, {NULL}};
const Segment SGI95872[] = {{&S65528[92], 9}, {NULL}};
const Segment SGI95873[] = {{&S65528[65], 6}, {NULL}};
const Segment SGI95874[] = {{&S87369[52], 10}, {NULL}};
const Segment SGI95875[] = {{&S87369[52], 27}, {NULL}};
const Segment SGI95876[] = {{&S87369[38], 6}, {NULL}};
const Segment SGI95877[] = {{&S87369[93], 5}, {NULL}};
const Segment SGI95878[] = {{&S87369[0], 14}, {NULL}};
const Segment SGI95879[] = {{&S87370[52], 5}, {NULL}};
const Segment SGI95880[] = {{&S87370[52], 26}, {NULL}};
const Segment SGI95881[] = {{&S87370[38], 6}, {NULL}};
const Segment SGI95882[] = {{&S87370[92], 5}, {NULL}};
const Segment SGI95883[] = {{&S87370[0], 14}, {NULL}};
const Segment SGI95884[] = {{&S65557[44], 24}, {NULL}};
const Segment SGI95885[] = {{&S65557[44], 36}, {NULL}};
const Segment SGI95886[] = {{&S65557[30], 6}, {NULL}};
const Segment SGI95887[] = {{&S65557[31], 3}, {NULL}};
const Segment SGI95888[] = {{&S65557[94], 6}, {NULL}};
const Segment SGI95889[] = {{&S65565[44], 15}, {NULL}};
const Segment SGI95890[] = {{&S65565[44], 27}, {NULL}};
const Segment SGI95891[] = {{&S65565[30], 6}, {NULL}};
const Segment SGI95892[] = {{&S65565[31], 3}, {NULL}};
const Segment SGI95893[] = {{&S65565[85], 6}, {NULL}};
const Segment SGI95894[] = {{&S65566[44], 15}, {NULL}};
const Segment SGI95895[] = {{&S65566[44], 27}, {NULL}};
const Segment SGI95896[] = {{&S65566[30], 6}, {NULL}};
const Segment SGI95897[] = {{&S65566[31], 3}, {NULL}};
const Segment SGI95898[] = {{&S65566[85], 6}, {NULL}};
const Segment SGI95899[] = {{&S65567[44], 15}, {NULL}};
const Segment SGI95900[] = {{&S65567[44], 27}, {NULL}};
const Segment SGI95901[] = {{&S65567[30], 6}, {NULL}};
const Segment SGI95902[] = {{&S65567[31], 3}, {NULL}};
const Segment SGI95903[] = {{&S65567[85], 6}, {NULL}};
const Segment SGI95904[] = {{&S65568[44], 15}, {NULL}};
const Segment SGI95905[] = {{&S65568[44], 27}, {NULL}};
const Segment SGI95906[] = {{&S65568[30], 6}, {NULL}};
const Segment SGI95907[] = {{&S65568[31], 3}, {NULL}};
const Segment SGI95908[] = {{&S65568[85], 6}, {NULL}};
const Segment SGI95909[] = {{&S65569[44], 15}, {NULL}};
const Segment SGI95910[] = {{&S65569[44], 27}, {NULL}};
const Segment SGI95911[] = {{&S65569[30], 6}, {NULL}};
const Segment SGI95912[] = {{&S65569[31], 3}, {NULL}};
const Segment SGI95913[] = {{&S65569[85], 6}, {NULL}};
const Segment SGI95914[] = {{&S65570[44], 15}, {NULL}};
const Segment SGI95915[] = {{&S65570[44], 27}, {NULL}};
const Segment SGI95916[] = {{&S65570[30], 6}, {NULL}};
const Segment SGI95917[] = {{&S65570[31], 3}, {NULL}};
const Segment SGI95918[] = {{&S65570[85], 6}, {NULL}};
const Segment SGI95919[] = {{&S65586[44], 8}, {NULL}};
const Segment SGI95920[] = {{&S65586[44], 20}, {NULL}};
const Segment SGI95921[] = {{&S65586[30], 6}, {NULL}};
const Segment SGI95922[] = {{&S65586[31], 3}, {NULL}};
const Segment SGI95923[] = {{&S65586[78], 6}, {NULL}};
const Segment SGI95924[] = {{&S65588[44], 9}, {NULL}};
const Segment SGI95925[] = {{&S65588[44], 18}, {NULL}};
const Segment SGI95926[] = {{&S65588[30], 6}, {NULL}};
const Segment SGI95927[] = {{&S65588[31], 3}, {NULL}};
const Segment SGI95928[] = {{&S65588[76], 6}, {NULL}};
const Segment SGI95929[] = {{&S65591[44], 9}, {NULL}};
const Segment SGI95930[] = {{&S65591[44], 21}, {NULL}};
const Segment SGI95931[] = {{&S65591[30], 6}, {NULL}};
const Segment SGI95932[] = {{&S65591[31], 3}, {NULL}};
const Segment SGI95933[] = {{&S65591[79], 6}, {NULL}};
const Segment SGI95934[] = {{&S65593[34], 10}, {NULL}};
const Segment SGI95935[] = {{&S65593[27], 6}, {NULL}};
const Segment SGI95936[] = {{&S65593[97], 9}, {NULL}};
const Segment SGI95937[] = {{&S65593[70], 6}, {NULL}};
const Segment SGI95938[] = {{&S65594[44], 7}, {NULL}};
const Segment SGI95939[] = {{&S65594[44], 20}, {NULL}};
const Segment SGI95940[] = {{&S65594[30], 6}, {NULL}};
const Segment SGI95941[] = {{&S65594[31], 3}, {NULL}};
const Segment SGI95942[] = {{&S65594[78], 6}, {NULL}};
const Segment SGI95943[] = {{&S65595[44], 7}, {NULL}};
const Segment SGI95944[] = {{&S65595[44], 20}, {NULL}};
const Segment SGI95945[] = {{&S65595[30], 6}, {NULL}};
const Segment SGI95946[] = {{&S65595[31], 3}, {NULL}};
const Segment SGI95947[] = {{&S65595[78], 6}, {NULL}};
const Segment SGI95948[] = {{&S79184[11], 1}, {NULL}};
const Segment SGI95949[] = {{&S79184[42], 12}, {NULL}};
const Segment SGI95950[] = {{&S79184[30], 4}, {NULL}};
const Segment SGI95951[] = {{&S79184[103], 3}, {NULL}};
const Segment SGI95952[] = {{&S79184[68], 6}, {NULL}};
const Segment SGI95953[] = {{&S79185[11], 1}, {NULL}};
const Segment SGI95954[] = {{&S79185[42], 13}, {NULL}};
const Segment SGI95955[] = {{&S79185[30], 4}, {NULL}};
const Segment SGI95956[] = {{&S79185[104], 3}, {NULL}};
const Segment SGI95957[] = {{&S79185[69], 6}, {NULL}};
const Segment SGI95958[] = {{&S79187[11], 1}, {NULL}};
const Segment SGI95959[] = {{&S79187[42], 13}, {NULL}};
const Segment SGI95960[] = {{&S79187[30], 4}, {NULL}};
const Segment SGI95961[] = {{&S79187[104], 3}, {NULL}};
const Segment SGI95962[] = {{&S79187[69], 6}, {NULL}};
const Segment SGI95963[] = {{&S79239[42], 6}, {NULL}};
const Segment SGI95964[] = {{&S79239[42], 18}, {NULL}};
const Segment SGI95965[] = {{&S79239[30], 4}, {NULL}};
const Segment SGI95966[] = {{&S79239[109], 3}, {NULL}};
const Segment SGI95967[] = {{&S79239[74], 6}, {NULL}};
const Segment SGI95968[] = {{&S80217[44], 9}, {NULL}};
const Segment SGI95969[] = {{&S80217[44], 20}, {NULL}};
const Segment SGI95970[] = {{&S80217[30], 6}, {NULL}};
const Segment SGI95971[] = {{&S80217[131], 3}, {NULL}};
const Segment SGI95972[] = {{&S80217[97], 5}, {NULL}};
const Segment SGI95973[] = {{&S80217[35], 1}, {NULL}};
const Segment SGI95974[] = {{&S80217[66], 11}, {NULL}};
const Segment SGI95975[] = {{&S65615[44], 12}, {NULL}};
const Segment SGI95976[] = {{&S65615[44], 21}, {NULL}};
const Segment SGI95977[] = {{&S65615[30], 6}, {NULL}};
const Segment SGI95978[] = {{&S65615[31], 3}, {NULL}};
const Segment SGI95979[] = {{&S65615[79], 6}, {NULL}};
const Segment SGI95980[] = {{&S65616[44], 11}, {NULL}};
const Segment SGI95981[] = {{&S65616[44], 23}, {NULL}};
const Segment SGI95982[] = {{&S65616[30], 6}, {NULL}};
const Segment SGI95983[] = {{&S65616[31], 3}, {NULL}};
const Segment SGI95984[] = {{&S65616[81], 6}, {NULL}};
const Segment SGI95985[] = {{&S65617[44], 22}, {NULL}};
const Segment SGI95986[] = {{&S65617[44], 34}, {NULL}};
const Segment SGI95987[] = {{&S65617[30], 6}, {NULL}};
const Segment SGI95988[] = {{&S65617[31], 3}, {NULL}};
const Segment SGI95989[] = {{&S65617[92], 6}, {NULL}};
const Segment SGI95990[] = {{&S87580[59], 4}, {NULL}};
const Segment SGI95991[] = {{&S87580[59], 16}, {NULL}};
const Segment SGI95992[] = {{&S87580[45], 6}, {NULL}};
const Segment SGI95993[] = {{&S87580[123], 3}, {NULL}};
const Segment SGI95994[] = {{&S87580[89], 5}, {NULL}};
const Segment SGI95995[] = {{&S87580[0], 14}, {NULL}};
const Segment SGI95996[] = {{&S65643[34], 14}, {NULL}};
const Segment SGI95997[] = {{&S65643[27], 6}, {NULL}};
const Segment SGI95998[] = {{&S65643[101], 9}, {NULL}};
const Segment SGI95999[] = {{&S65643[74], 6}, {NULL}};
const Segment SGI96000[] = {{&S80278[44], 6}, {NULL}};
const Segment SGI96001[] = {{&S80278[44], 25}, {NULL}};
const Segment SGI96002[] = {{&S80278[30], 6}, {NULL}};
const Segment SGI96003[] = {{&S80278[56], 3}, {NULL}};
const Segment SGI96004[] = {{&S80278[96], 5}, {NULL}};
const Segment SGI96005[] = {{&S80279[44], 6}, {NULL}};
const Segment SGI96006[] = {{&S80279[44], 25}, {NULL}};
const Segment SGI96007[] = {{&S80279[30], 6}, {NULL}};
const Segment SGI96008[] = {{&S80279[134], 3}, {NULL}};
const Segment SGI96009[] = {{&S80279[100], 5}, {NULL}};
const Segment SGI96010[] = {{&S80280[44], 6}, {NULL}};
const Segment SGI96011[] = {{&S80280[44], 13}, {NULL}};
const Segment SGI96012[] = {{&S80280[30], 6}, {NULL}};
const Segment SGI96013[] = {{&S80280[121], 3}, {NULL}};
const Segment SGI96014[] = {{&S80280[87], 5}, {NULL}};
const Segment SGI96015[] = {{&S65671[44], 23}, {NULL}};
const Segment SGI96016[] = {{&S65671[44], 35}, {NULL}};
const Segment SGI96017[] = {{&S65671[30], 6}, {NULL}};
const Segment SGI96018[] = {{&S65671[31], 3}, {NULL}};
const Segment SGI96019[] = {{&S65671[93], 6}, {NULL}};
const Segment SGI96020[] = {{&S65672[44], 12}, {NULL}};
const Segment SGI96021[] = {{&S65672[44], 24}, {NULL}};
const Segment SGI96022[] = {{&S65672[30], 6}, {NULL}};
const Segment SGI96023[] = {{&S65672[31], 3}, {NULL}};
const Segment SGI96024[] = {{&S65672[82], 6}, {NULL}};
const Segment SGI96025[] = {{&S65675[44], 7}, {NULL}};
const Segment SGI96026[] = {{&S65675[44], 19}, {NULL}};
const Segment SGI96027[] = {{&S65675[30], 6}, {NULL}};
const Segment SGI96028[] = {{&S65675[31], 3}, {NULL}};
const Segment SGI96029[] = {{&S65675[77], 6}, {NULL}};
const Segment SGI96030[] = {{&S65679[34], 41}, {NULL}};
const Segment SGI96031[] = {{&S65679[27], 6}, {NULL}};
const Segment SGI96032[] = {{&S65679[128], 9}, {NULL}};
const Segment SGI96033[] = {{&S65679[101], 6}, {NULL}};
const Segment SGI96034[] = {{&S65680[34], 41}, {NULL}};
const Segment SGI96035[] = {{&S65680[27], 6}, {NULL}};
const Segment SGI96036[] = {{&S65680[128], 9}, {NULL}};
const Segment SGI96037[] = {{&S65680[101], 6}, {NULL}};
const Segment SGI96038[] = {{&S80346[44], 9}, {NULL}};
const Segment SGI96039[] = {{&S80346[44], 20}, {NULL}};
const Segment SGI96040[] = {{&S80346[30], 6}, {NULL}};
const Segment SGI96041[] = {{&S80346[127], 3}, {NULL}};
const Segment SGI96042[] = {{&S80346[93], 5}, {NULL}};
const Segment SGI96043[] = {{&S80346[35], 1}, {NULL}};
const Segment SGI96044[] = {{&S80346[66], 11}, {NULL}};
const Segment SGI96045[] = {{&S80347[44], 9}, {NULL}};
const Segment SGI96046[] = {{&S80347[44], 20}, {NULL}};
const Segment SGI96047[] = {{&S80347[30], 6}, {NULL}};
const Segment SGI96048[] = {{&S80347[142], 3}, {NULL}};
const Segment SGI96049[] = {{&S80347[108], 5}, {NULL}};
const Segment SGI96050[] = {{&S80347[66], 11}, {NULL}};
const Segment SGI96051[] = {{&S80348[44], 13}, {NULL}};
const Segment SGI96052[] = {{&S80348[44], 23}, {NULL}};
const Segment SGI96053[] = {{&S80348[30], 6}, {NULL}};
const Segment SGI96054[] = {{&S80348[115], 3}, {NULL}};
const Segment SGI96055[] = {{&S80348[81], 5}, {NULL}};
const Segment SGI96056[] = {{&S87691[38], 7}, {NULL}};
const Segment SGI96057[] = {{&S87691[31], 6}, {NULL}};
const Segment SGI96058[] = {{&S87692[44], 7}, {NULL}};
const Segment SGI96059[] = {{&S87692[44], 15}, {NULL}};
const Segment SGI96060[] = {{&S87692[30], 6}, {NULL}};
const Segment SGI96061[] = {{&S87692[107], 3}, {NULL}};
const Segment SGI96062[] = {{&S87692[73], 5}, {NULL}};
const Segment SGI96063[] = {{&S87697[44], 10}, {NULL}};
const Segment SGI96064[] = {{&S87697[44], 22}, {NULL}};
const Segment SGI96065[] = {{&S87697[30], 6}, {NULL}};
const Segment SGI96066[] = {{&S87697[114], 3}, {NULL}};
const Segment SGI96067[] = {{&S87697[80], 5}, {NULL}};
const Segment SGI96068[] = {{&S80403[44], 5}, {NULL}};
const Segment SGI96069[] = {{&S80403[44], 16}, {NULL}};
const Segment SGI96070[] = {{&S80403[30], 6}, {NULL}};
const Segment SGI96071[] = {{&S80403[108], 3}, {NULL}};
const Segment SGI96072[] = {{&S80403[74], 5}, {NULL}};
const Segment SGI96073[] = {{&S87729[44], 16}, {NULL}};
const Segment SGI96074[] = {{&S87729[44], 28}, {NULL}};
const Segment SGI96075[] = {{&S87729[30], 6}, {NULL}};
const Segment SGI96076[] = {{&S87729[86], 6}, {NULL}};
const Segment SGI96077[] = {{&S87729[113], 13}, {NULL}};
const Segment SGI96078[] = {{&S74128[79], 6}, {NULL}};
const Segment SGI96079[] = {{&S74128[0], 14}, {NULL}};
const Segment SGI96080[] = {{&S65884[34], 18}, {NULL}};
const Segment SGI96081[] = {{&S65884[27], 6}, {NULL}};
const Segment SGI96082[] = {{&S65884[105], 9}, {NULL}};
const Segment SGI96083[] = {{&S65884[78], 6}, {NULL}};
const Segment SGI96084[] = {{&S65885[34], 18}, {NULL}};
const Segment SGI96085[] = {{&S65885[27], 6}, {NULL}};
const Segment SGI96086[] = {{&S65885[105], 9}, {NULL}};
const Segment SGI96087[] = {{&S65885[78], 6}, {NULL}};
const Segment SGI96088[] = {{&S65886[34], 11}, {NULL}};
const Segment SGI96089[] = {{&S65886[27], 6}, {NULL}};
const Segment SGI96090[] = {{&S65886[98], 9}, {NULL}};
const Segment SGI96091[] = {{&S65886[71], 6}, {NULL}};
const Segment SGI96092[] = {{&S74181[77], 6}, {NULL}};
const Segment SGI96093[] = {{&S74181[0], 12}, {NULL}};
const Segment SGI96094[] = {{&S79589[42], 12}, {NULL}};
const Segment SGI96095[] = {{&S79589[42], 24}, {NULL}};
const Segment SGI96096[] = {{&S79589[30], 4}, {NULL}};
const Segment SGI96097[] = {{&S79589[115], 3}, {NULL}};
const Segment SGI96098[] = {{&S79589[80], 6}, {NULL}};
const Segment SGI96099[] = {{&S80585[44], 16}, {NULL}};
const Segment SGI96100[] = {{&S80585[44], 26}, {NULL}};
const Segment SGI96101[] = {{&S80585[30], 6}, {NULL}};
const Segment SGI96102[] = {{&S80585[118], 3}, {NULL}};
const Segment SGI96103[] = {{&S80585[84], 5}, {NULL}};
const Segment SGI96104[] = {{&S80588[44], 11}, {NULL}};
const Segment SGI96105[] = {{&S80588[44], 22}, {NULL}};
const Segment SGI96106[] = {{&S80588[30], 6}, {NULL}};
const Segment SGI96107[] = {{&S80588[133], 3}, {NULL}};
const Segment SGI96108[] = {{&S80588[99], 5}, {NULL}};
const Segment SGI96109[] = {{&S80588[35], 1}, {NULL}};
const Segment SGI96110[] = {{&S80588[68], 11}, {NULL}};
const Segment SGI96111[] = {{&S65990[44], 22}, {NULL}};
const Segment SGI96112[] = {{&S65990[44], 34}, {NULL}};
const Segment SGI96113[] = {{&S65990[30], 6}, {NULL}};
const Segment SGI96114[] = {{&S65990[31], 3}, {NULL}};
const Segment SGI96115[] = {{&S65990[92], 6}, {NULL}};
const Segment SGI96116[] = {{&S65991[44], 22}, {NULL}};
const Segment SGI96117[] = {{&S65991[44], 34}, {NULL}};
const Segment SGI96118[] = {{&S65991[30], 6}, {NULL}};
const Segment SGI96119[] = {{&S65991[31], 3}, {NULL}};
const Segment SGI96120[] = {{&S65991[92], 6}, {NULL}};
const Segment SGI96121[] = {{&S80635[44], 10}, {NULL}};
const Segment SGI96122[] = {{&S80635[44], 21}, {NULL}};
const Segment SGI96123[] = {{&S80635[30], 6}, {NULL}};
const Segment SGI96124[] = {{&S65992[44], 16}, {NULL}};
const Segment SGI96125[] = {{&S80635[128], 3}, {NULL}};
const Segment SGI96126[] = {{&S65992[44], 28}, {NULL}};
const Segment SGI96127[] = {{&S65992[30], 6}, {NULL}};
const Segment SGI96128[] = {{&S80635[94], 5}, {NULL}};
const Segment SGI96129[] = {{&S80635[35], 1}, {NULL}};
const Segment SGI96130[] = {{&S80635[67], 11}, {NULL}};
const Segment SGI96131[] = {{&S65992[31], 3}, {NULL}};
const Segment SGI96132[] = {{&S65992[86], 6}, {NULL}};
const Segment SGI96133[] = {{&S65993[44], 19}, {NULL}};
const Segment SGI96134[] = {{&S65993[44], 31}, {NULL}};
const Segment SGI96135[] = {{&S65993[30], 6}, {NULL}};
const Segment SGI96136[] = {{&S65993[31], 3}, {NULL}};
const Segment SGI96137[] = {{&S65993[89], 6}, {NULL}};
const Segment SGI96138[] = {{&S65994[44], 21}, {NULL}};
const Segment SGI96139[] = {{&S65994[44], 33}, {NULL}};
const Segment SGI96140[] = {{&S65994[30], 6}, {NULL}};
const Segment SGI96141[] = {{&S65994[31], 3}, {NULL}};
const Segment SGI96142[] = {{&S65994[91], 6}, {NULL}};
const Segment SGI96143[] = {{&S66014[44], 8}, {NULL}};
const Segment SGI96144[] = {{&S66014[44], 20}, {NULL}};
const Segment SGI96145[] = {{&S66014[30], 6}, {NULL}};
const Segment SGI96146[] = {{&S66014[31], 3}, {NULL}};
const Segment SGI96147[] = {{&S66014[78], 6}, {NULL}};
const Segment SGI96148[] = {{&S66019[34], 21}, {NULL}};
const Segment SGI96149[] = {{&S66019[27], 6}, {NULL}};
const Segment SGI96150[] = {{&S66019[108], 9}, {NULL}};
const Segment SGI96151[] = {{&S66019[81], 6}, {NULL}};
const Segment SGI96152[] = {{&S66021[34], 11}, {NULL}};
const Segment SGI96153[] = {{&S66021[27], 6}, {NULL}};
const Segment SGI96154[] = {{&S66021[98], 9}, {NULL}};
const Segment SGI96155[] = {{&S66021[71], 6}, {NULL}};
const Segment SGI96156[] = {{&S66022[34], 15}, {NULL}};
const Segment SGI96157[] = {{&S66022[27], 6}, {NULL}};
const Segment SGI96158[] = {{&S66022[102], 9}, {NULL}};
const Segment SGI96159[] = {{&S66022[75], 6}, {NULL}};
const Segment SGI96160[] = {{&S66094[44], 10}, {NULL}};
const Segment SGI96161[] = {{&S66094[44], 22}, {NULL}};
const Segment SGI96162[] = {{&S66094[30], 6}, {NULL}};
const Segment SGI96163[] = {{&S66094[31], 3}, {NULL}};
const Segment SGI96164[] = {{&S66094[80], 6}, {NULL}};
const Segment SGI96165[] = {{&S79806[32], 10}, {NULL}};
const Segment SGI96166[] = {{&S79806[27], 4}, {NULL}};
const Segment SGI96167[] = {{&S79806[101], 2}, {NULL}};
const Segment SGI96168[] = {{&S79806[67], 6}, {NULL}};
const Segment SGI96169[] = {{&S79806[128], 8}, {NULL}};
const Segment SGI96170[] = {{&S80778[44], 7}, {NULL}};
const Segment SGI96171[] = {{&S80778[44], 24}, {NULL}};
const Segment SGI96172[] = {{&S80778[30], 6}, {NULL}};
const Segment SGI96173[] = {{&S80778[82], 6}, {NULL}};
const Segment SGI96174[] = {{&S80778[109], 13}, {NULL}};
const Segment SGI96175[] = {{&S66141[44], 9}, {NULL}};
const Segment SGI96176[] = {{&S66141[44], 21}, {NULL}};
const Segment SGI96177[] = {{&S66141[30], 6}, {NULL}};
const Segment SGI96178[] = {{&S66141[31], 3}, {NULL}};
const Segment SGI96179[] = {{&S66141[79], 6}, {NULL}};
const Segment SGI96180[] = {{&S66166[44], 7}, {NULL}};
const Segment SGI96181[] = {{&S66166[44], 19}, {NULL}};
const Segment SGI96182[] = {{&S66166[30], 6}, {NULL}};
const Segment SGI96183[] = {{&S66166[31], 3}, {NULL}};
const Segment SGI96184[] = {{&S66166[77], 6}, {NULL}};
const Segment SGI96185[] = {{&S66175[34], 10}, {NULL}};
const Segment SGI96186[] = {{&S66175[27], 6}, {NULL}};
const Segment SGI96187[] = {{&S66175[97], 9}, {NULL}};
const Segment SGI96188[] = {{&S66175[70], 6}, {NULL}};
const Segment SGI96189[] = {{&S66195[34], 14}, {NULL}};
const Segment SGI96190[] = {{&S66195[27], 6}, {NULL}};
const Segment SGI96191[] = {{&S66195[98], 9}, {NULL}};
const Segment SGI96192[] = {{&S66195[71], 6}, {NULL}};
const Segment SGI96193[] = {{&S87842[44], 6}, {NULL}};
const Segment SGI96194[] = {{&S87842[44], 18}, {NULL}};
const Segment SGI96195[] = {{&S87842[30], 6}, {NULL}};
const Segment SGI96196[] = {{&S87842[110], 3}, {NULL}};
const Segment SGI96197[] = {{&S87842[76], 5}, {NULL}};
const Segment SGI96198[] = {{&S87844[59], 13}, {NULL}};
const Segment SGI96199[] = {{&S87844[59], 32}, {NULL}};
const Segment SGI96200[] = {{&S87844[45], 6}, {NULL}};
const Segment SGI96201[] = {{&S87844[139], 3}, {NULL}};
const Segment SGI96202[] = {{&S87844[105], 5}, {NULL}};
const Segment SGI96203[] = {{&S87844[0], 14}, {NULL}};
const Segment SGI96204[] = {{&S87859[61], 10}, {NULL}};
const Segment SGI96205[] = {{&S87859[61], 27}, {NULL}};
const Segment SGI96206[] = {{&S87859[47], 6}, {NULL}};
const Segment SGI96207[] = {{&S87859[136], 3}, {NULL}};
const Segment SGI96208[] = {{&S87859[102], 5}, {NULL}};
const Segment SGI96209[] = {{&S87859[0], 14}, {NULL}};
const Segment SGI96210[] = {{&S77488[42], 15}, {NULL}};
const Segment SGI96211[] = {{&S77488[42], 27}, {NULL}};
const Segment SGI96212[] = {{&S77488[30], 4}, {NULL}};
const Segment SGI96213[] = {{&S77488[31], 3}, {NULL}};
const Segment SGI96214[] = {{&S77488[83], 6}, {NULL}};
const Segment SGI96215[] = {{&S66277[44], 6}, {NULL}};
const Segment SGI96216[] = {{&S66277[44], 18}, {NULL}};
const Segment SGI96217[] = {{&S66277[30], 6}, {NULL}};
const Segment SGI96218[] = {{&S66277[31], 3}, {NULL}};
const Segment SGI96219[] = {{&S66277[76], 6}, {NULL}};
const Segment SGI96220[] = {{&S66278[38], 6}, {NULL}};
const Segment SGI96221[] = {{&S66278[31], 6}, {NULL}};
const Segment SGI96222[] = {{&S66282[34], 10}, {NULL}};
const Segment SGI96223[] = {{&S66282[27], 6}, {NULL}};
const Segment SGI96224[] = {{&S66282[97], 9}, {NULL}};
const Segment SGI96225[] = {{&S66282[70], 6}, {NULL}};
const Segment SGI96226[] = {{&S74537[44], 11}, {NULL}};
const Segment SGI96227[] = {{&S74537[44], 22}, {NULL}};
const Segment SGI96228[] = {{&S74537[30], 6}, {NULL}};
const Segment SGI96229[] = {{&S74537[31], 3}, {NULL}};
const Segment SGI96230[] = {{&S74537[80], 5}, {NULL}};
const Segment SGI96231[] = {{&S74537[0], 13}, {NULL}};
const Segment SGI96232[] = {{&S87975[44], 6}, {NULL}};
const Segment SGI96233[] = {{&S87975[44], 18}, {NULL}};
const Segment SGI96234[] = {{&S87975[30], 6}, {NULL}};
const Segment SGI96235[] = {{&S87975[110], 3}, {NULL}};
const Segment SGI96236[] = {{&S87975[76], 5}, {NULL}};
const Segment SGI96237[] = {{&S66289[34], 6}, {NULL}};
const Segment SGI96238[] = {{&S66289[27], 6}, {NULL}};
const Segment SGI96239[] = {{&S66289[93], 9}, {NULL}};
const Segment SGI96240[] = {{&S66289[66], 6}, {NULL}};
const Segment SGI96241[] = {{&S66299[44], 12}, {NULL}};
const Segment SGI96242[] = {{&S66299[44], 24}, {NULL}};
const Segment SGI96243[] = {{&S66299[30], 6}, {NULL}};
const Segment SGI96244[] = {{&S66299[31], 3}, {NULL}};
const Segment SGI96245[] = {{&S66299[82], 6}, {NULL}};
const Segment SGI96246[] = {{&S87995[44], 11}, {NULL}};
const Segment SGI96247[] = {{&S87995[44], 28}, {NULL}};
const Segment SGI96248[] = {{&S87995[30], 6}, {NULL}};
const Segment SGI96249[] = {{&S87995[119], 3}, {NULL}};
const Segment SGI96250[] = {{&S87995[86], 5}, {NULL}};
const Segment SGI96251[] = {{&S87995[123], 14}, {NULL}};
const Segment SGI96252[] = {{&S66314[44], 6}, {NULL}};
const Segment SGI96253[] = {{&S66314[44], 18}, {NULL}};
const Segment SGI96254[] = {{&S66314[30], 6}, {NULL}};
const Segment SGI96255[] = {{&S66314[31], 3}, {NULL}};
const Segment SGI96256[] = {{&S66314[76], 6}, {NULL}};
const Segment SGI96257[] = {{&S66316[34], 12}, {NULL}};
const Segment SGI96258[] = {{&S66316[27], 6}, {NULL}};
const Segment SGI96259[] = {{&S66316[99], 9}, {NULL}};
const Segment SGI96260[] = {{&S66316[72], 6}, {NULL}};
const Segment SGI96261[] = {{&S66317[34], 12}, {NULL}};
const Segment SGI96262[] = {{&S66317[27], 6}, {NULL}};
const Segment SGI96263[] = {{&S66317[99], 9}, {NULL}};
const Segment SGI96264[] = {{&S66317[72], 6}, {NULL}};
const Segment SGI96265[] = {{&S66331[44], 14}, {NULL}};
const Segment SGI96266[] = {{&S66331[44], 40}, {NULL}};
const Segment SGI96267[] = {{&S66331[30], 6}, {NULL}};
const Segment SGI96268[] = {{&S66331[31], 3}, {NULL}};
const Segment SGI96269[] = {{&S66331[98], 6}, {NULL}};
const Segment SGI96270[] = {{&S66332[44], 14}, {NULL}};
const Segment SGI96271[] = {{&S66332[44], 34}, {NULL}};
const Segment SGI96272[] = {{&S66332[30], 6}, {NULL}};
const Segment SGI96273[] = {{&S66332[31], 3}, {NULL}};
const Segment SGI96274[] = {{&S66332[92], 6}, {NULL}};
const Segment SGI96275[] = {{&S66333[44], 14}, {NULL}};
const Segment SGI96276[] = {{&S66333[44], 33}, {NULL}};
const Segment SGI96277[] = {{&S66333[30], 6}, {NULL}};
const Segment SGI96278[] = {{&S66333[31], 3}, {NULL}};
const Segment SGI96279[] = {{&S66333[91], 6}, {NULL}};
const Segment SGI96280[] = {{&S66334[44], 14}, {NULL}};
const Segment SGI96281[] = {{&S66334[44], 33}, {NULL}};
const Segment SGI96282[] = {{&S66334[30], 6}, {NULL}};
const Segment SGI96283[] = {{&S66334[31], 3}, {NULL}};
const Segment SGI96284[] = {{&S66334[91], 6}, {NULL}};
const Segment SGI96285[] = {{&S66338[34], 29}, {NULL}};
const Segment SGI96286[] = {{&S66338[27], 6}, {NULL}};
const Segment SGI96287[] = {{&S66338[116], 9}, {NULL}};
const Segment SGI96288[] = {{&S66338[89], 6}, {NULL}};
const Segment SGI96289[] = {{&S66339[34], 30}, {NULL}};
const Segment SGI96290[] = {{&S66339[27], 6}, {NULL}};
const Segment SGI96291[] = {{&S66339[117], 9}, {NULL}};
const Segment SGI96292[] = {{&S66339[90], 6}, {NULL}};
const Segment SGI96293[] = {{&S66340[34], 30}, {NULL}};
const Segment SGI96294[] = {{&S66340[27], 6}, {NULL}};
const Segment SGI96295[] = {{&S66340[117], 9}, {NULL}};
const Segment SGI96296[] = {{&S66340[90], 6}, {NULL}};
const Segment SGI96297[] = {{&S66345[34], 6}, {NULL}};
const Segment SGI96298[] = {{&S66345[27], 6}, {NULL}};
const Segment SGI96299[] = {{&S66345[93], 9}, {NULL}};
const Segment SGI96300[] = {{&S66345[66], 6}, {NULL}};
const Segment SGI96301[] = {{&S66358[34], 15}, {NULL}};
const Segment SGI96302[] = {{&S66358[27], 6}, {NULL}};
const Segment SGI96303[] = {{&S66358[102], 9}, {NULL}};
const Segment SGI96304[] = {{&S66358[75], 6}, {NULL}};
const Segment SGI96305[] = {{&S66369[44], 17}, {NULL}};
const Segment SGI96306[] = {{&S66369[44], 38}, {NULL}};
const Segment SGI96307[] = {{&S66369[30], 6}, {NULL}};
const Segment SGI96308[] = {{&S66369[31], 3}, {NULL}};
const Segment SGI96309[] = {{&S66369[96], 5}, {NULL}};
const Segment SGI96310[] = {{&S88085[45], 10}, {NULL}};
const Segment SGI96311[] = {{&S88085[30], 6}, {NULL}};
const Segment SGI96312[] = {{&S88085[120], 3}, {NULL}};
const Segment SGI96313[] = {{&S88085[86], 5}, {NULL}};
const Segment SGI96314[] = {{&S88085[144], 13}, {NULL}};
const Segment SGI96315[] = {{&S88103[44], 11}, {NULL}};
const Segment SGI96316[] = {{&S88103[44], 23}, {NULL}};
const Segment SGI96317[] = {{&S88103[30], 6}, {NULL}};
const Segment SGI96318[] = {{&S88103[115], 3}, {NULL}};
const Segment SGI96319[] = {{&S88103[81], 5}, {NULL}};
const Segment SGI96320[] = {{&S88113[44], 10}, {NULL}};
const Segment SGI96321[] = {{&S88113[44], 27}, {NULL}};
const Segment SGI96322[] = {{&S88113[30], 6}, {NULL}};
const Segment SGI96323[] = {{&S88113[118], 3}, {NULL}};
const Segment SGI96324[] = {{&S88113[85], 5}, {NULL}};
const Segment SGI96325[] = {{&S88113[122], 14}, {NULL}};
const Segment SGI96326[] = {{&S88120[34], 13}, {NULL}};
const Segment SGI96327[] = {{&S88120[27], 6}, {NULL}};
const Segment SGI96328[] = {{&S88120[112], 2}, {NULL}};
const Segment SGI96329[] = {{&S88120[78], 6}, {NULL}};
const Segment SGI96330[] = {{&S88120[146], 8}, {NULL}};
const Segment SGI96331[] = {{&S88121[44], 10}, {NULL}};
const Segment SGI96332[] = {{&S88121[44], 22}, {NULL}};
const Segment SGI96333[] = {{&S88121[30], 6}, {NULL}};
const Segment SGI96334[] = {{&S88121[80], 6}, {NULL}};
const Segment SGI96335[] = {{&S88121[107], 13}, {NULL}};
const Segment SGI96336[] = {{&S88127[44], 10}, {NULL}};
const Segment SGI96337[] = {{&S88127[44], 27}, {NULL}};
const Segment SGI96338[] = {{&S88127[30], 6}, {NULL}};
const Segment SGI96339[] = {{&S88127[118], 3}, {NULL}};
const Segment SGI96340[] = {{&S88127[85], 5}, {NULL}};
const Segment SGI96341[] = {{&S88127[122], 14}, {NULL}};
const Segment SGI96342[] = {{&S66454[44], 7}, {NULL}};
const Segment SGI96343[] = {{&S66454[44], 19}, {NULL}};
const Segment SGI96344[] = {{&S66454[30], 6}, {NULL}};
const Segment SGI96345[] = {{&S66454[31], 3}, {NULL}};
const Segment SGI96346[] = {{&S66454[77], 6}, {NULL}};
const Segment SGI96347[] = {{&S71135[34], 14}, {NULL}};
const Segment SGI96348[] = {{&S71135[27], 6}, {NULL}};
const Segment SGI96349[] = {{&S71135[101], 9}, {NULL}};
const Segment SGI96350[] = {{&S71135[74], 6}, {NULL}};
const Segment SGI96351[] = {{&S80995[44], 6}, {NULL}};
const Segment SGI96352[] = {{&S80995[44], 24}, {NULL}};
const Segment SGI96353[] = {{&S80995[30], 6}, {NULL}};
const Segment SGI96354[] = {{&S80995[131], 3}, {NULL}};
const Segment SGI96355[] = {{&S80995[97], 5}, {NULL}};
const Segment SGI96356[] = {{&S80995[35], 1}, {NULL}};
const Segment SGI96357[] = {{&S80995[70], 11}, {NULL}};
const Segment SGI96358[] = {{&S66538[44], 10}, {NULL}};
const Segment SGI96359[] = {{&S66538[44], 22}, {NULL}};
const Segment SGI96360[] = {{&S66538[30], 6}, {NULL}};
const Segment SGI96361[] = {{&S66538[80], 6}, {NULL}};
const Segment SGI96362[] = {{&S66538[107], 13}, {NULL}};
const Segment SGI96363[] = {{&S88245[34], 10}, {NULL}};
const Segment SGI96364[] = {{&S88245[27], 6}, {NULL}};
const Segment SGI96365[] = {{&S88245[109], 2}, {NULL}};
const Segment SGI96366[] = {{&S88245[75], 6}, {NULL}};
const Segment SGI96367[] = {{&S88245[143], 8}, {NULL}};
const Segment SGI96368[] = {{&S66559[34], 10}, {NULL}};
const Segment SGI96369[] = {{&S66559[27], 6}, {NULL}};
const Segment SGI96370[] = {{&S66559[97], 9}, {NULL}};
const Segment SGI96371[] = {{&S66559[70], 6}, {NULL}};
const Segment SGI96372[] = {{&S66560[34], 9}, {NULL}};
const Segment SGI96373[] = {{&S66560[27], 6}, {NULL}};
const Segment SGI96374[] = {{&S66560[96], 9}, {NULL}};
const Segment SGI96375[] = {{&S66560[69], 6}, {NULL}};
const Segment SGI96376[] = {{&S66561[34], 10}, {NULL}};
const Segment SGI96377[] = {{&S66561[27], 6}, {NULL}};
const Segment SGI96378[] = {{&S66561[97], 9}, {NULL}};
const Segment SGI96379[] = {{&S66561[70], 6}, {NULL}};
const Segment SGI96380[] = {{&S66562[34], 10}, {NULL}};
const Segment SGI96381[] = {{&S66562[27], 6}, {NULL}};
const Segment SGI96382[] = {{&S66562[97], 9}, {NULL}};
const Segment SGI96383[] = {{&S66562[70], 6}, {NULL}};
const Segment SGI96384[] = {{&S80323[32], 10}, {NULL}};
const Segment SGI96385[] = {{&S80323[27], 4}, {NULL}};
const Segment SGI96386[] = {{&S81064[44], 11}, {NULL}};
const Segment SGI96387[] = {{&S80323[102], 2}, {NULL}};
const Segment SGI96388[] = {{&S81064[44], 20}, {NULL}};
const Segment SGI96389[] = {{&S80323[68], 6}, {NULL}};
const Segment SGI96390[] = {{&S81064[30], 6}, {NULL}};
const Segment SGI96391[] = {{&S80323[129], 8}, {NULL}};
const Segment SGI96392[] = {{&S81064[131], 3}, {NULL}};
const Segment SGI96393[] = {{&S81064[97], 5}, {NULL}};
const Segment SGI96394[] = {{&S81064[35], 1}, {NULL}};
const Segment SGI96395[] = {{&S81064[66], 11}, {NULL}};
const Segment SGI96396[] = {{&S81065[44], 11}, {NULL}};
const Segment SGI96397[] = {{&S81065[44], 23}, {NULL}};
const Segment SGI96398[] = {{&S81065[30], 6}, {NULL}};
const Segment SGI96399[] = {{&S81065[134], 3}, {NULL}};
const Segment SGI96400[] = {{&S81065[100], 5}, {NULL}};
const Segment SGI96401[] = {{&S81065[69], 3}, {NULL}};
const Segment SGI96402[] = {{&S88264[44], 5}, {NULL}};
const Segment SGI96403[] = {{&S88264[44], 26}, {NULL}};
const Segment SGI96404[] = {{&S88264[30], 6}, {NULL}};
const Segment SGI96405[] = {{&S88264[84], 6}, {NULL}};
const Segment SGI96406[] = {{&S88264[111], 13}, {NULL}};
const Segment SGI96407[] = {{&S66581[44], 6}, {NULL}};
const Segment SGI96408[] = {{&S66581[44], 18}, {NULL}};
const Segment SGI96409[] = {{&S66581[30], 6}, {NULL}};
const Segment SGI96410[] = {{&S66581[31], 3}, {NULL}};
const Segment SGI96411[] = {{&S66581[76], 6}, {NULL}};
const Segment SGI96412[] = {{&S81075[45], 10}, {NULL}};
const Segment SGI96413[] = {{&S81075[30], 6}, {NULL}};
const Segment SGI96414[] = {{&S66582[38], 6}, {NULL}};
const Segment SGI96415[] = {{&S66582[31], 6}, {NULL}};
const Segment SGI96416[] = {{&S81075[115], 3}, {NULL}};
const Segment SGI96417[] = {{&S81075[81], 5}, {NULL}};
const Segment SGI96418[] = {{&S81075[139], 13}, {NULL}};
const Segment SGI96419[] = {{&S66617[44], 7}, {NULL}};
const Segment SGI96420[] = {{&S66617[44], 19}, {NULL}};
const Segment SGI96421[] = {{&S66617[30], 6}, {NULL}};
const Segment SGI96422[] = {{&S66617[31], 3}, {NULL}};
const Segment SGI96423[] = {{&S66617[77], 6}, {NULL}};
const Segment SGI96424[] = {{&S66619[44], 8}, {NULL}};
const Segment SGI96425[] = {{&S66619[44], 20}, {NULL}};
const Segment SGI96426[] = {{&S66619[30], 6}, {NULL}};
const Segment SGI96427[] = {{&S66619[31], 3}, {NULL}};
const Segment SGI96428[] = {{&S66619[78], 6}, {NULL}};
const Segment SGI96429[] = {{&S66621[38], 8}, {NULL}};
const Segment SGI96430[] = {{&S66621[31], 6}, {NULL}};
const Segment SGI96431[] = {{&S66639[44], 9}, {NULL}};
const Segment SGI96432[] = {{&S66639[44], 27}, {NULL}};
const Segment SGI96433[] = {{&S66639[30], 6}, {NULL}};
const Segment SGI96434[] = {{&S66639[31], 3}, {NULL}};
const Segment SGI96435[] = {{&S66639[85], 6}, {NULL}};
const Segment SGI96436[] = {{&S66641[44], 9}, {NULL}};
const Segment SGI96437[] = {{&S66641[44], 22}, {NULL}};
const Segment SGI96438[] = {{&S66641[30], 6}, {NULL}};
const Segment SGI96439[] = {{&S66641[31], 3}, {NULL}};
const Segment SGI96440[] = {{&S66641[80], 6}, {NULL}};
const Segment SGI96441[] = {{&S66642[44], 9}, {NULL}};
const Segment SGI96442[] = {{&S66642[44], 21}, {NULL}};
const Segment SGI96443[] = {{&S66642[30], 6}, {NULL}};
const Segment SGI96444[] = {{&S66642[31], 3}, {NULL}};
const Segment SGI96445[] = {{&S66642[79], 6}, {NULL}};
const Segment SGI96446[] = {{&S88340[45], 10}, {NULL}};
const Segment SGI96447[] = {{&S88340[30], 6}, {NULL}};
const Segment SGI96448[] = {{&S88340[120], 3}, {NULL}};
const Segment SGI96449[] = {{&S88340[86], 5}, {NULL}};
const Segment SGI96450[] = {{&S88340[144], 13}, {NULL}};
const Segment SGI96451[] = {{&S66648[34], 11}, {NULL}};
const Segment SGI96452[] = {{&S66648[27], 6}, {NULL}};
const Segment SGI96453[] = {{&S66648[98], 9}, {NULL}};
const Segment SGI96454[] = {{&S66648[71], 6}, {NULL}};
const Segment SGI96455[] = {{&S66650[34], 15}, {NULL}};
const Segment SGI96456[] = {{&S66650[27], 6}, {NULL}};
const Segment SGI96457[] = {{&S66650[102], 9}, {NULL}};
const Segment SGI96458[] = {{&S66650[75], 6}, {NULL}};
const Segment SGI96459[] = {{&S66651[34], 21}, {NULL}};
const Segment SGI96460[] = {{&S66651[27], 6}, {NULL}};
const Segment SGI96461[] = {{&S66651[108], 9}, {NULL}};
const Segment SGI96462[] = {{&S66651[81], 6}, {NULL}};
const Segment SGI96463[] = {{&S88354[44], 9}, {NULL}};
const Segment SGI96464[] = {{&S88354[44], 21}, {NULL}};
const Segment SGI96465[] = {{&S88354[30], 6}, {NULL}};
const Segment SGI96466[] = {{&S88354[79], 6}, {NULL}};
const Segment SGI96467[] = {{&S88354[106], 13}, {NULL}};
const Segment SGI96468[] = {{&S66668[34], 17}, {NULL}};
const Segment SGI96469[] = {{&S66668[27], 6}, {NULL}};
const Segment SGI96470[] = {{&S66668[104], 9}, {NULL}};
const Segment SGI96471[] = {{&S66668[77], 6}, {NULL}};
const Segment SGI96472[] = {{&S88367[44], 6}, {NULL}};
const Segment SGI96473[] = {{&S88367[44], 25}, {NULL}};
const Segment SGI96474[] = {{&S88367[30], 6}, {NULL}};
const Segment SGI96475[] = {{&S88367[83], 6}, {NULL}};
const Segment SGI96476[] = {{&S88367[110], 13}, {NULL}};
const Segment SGI96477[] = {{&S66722[34], 14}, {NULL}};
const Segment SGI96478[] = {{&S66722[27], 6}, {NULL}};
const Segment SGI96479[] = {{&S66722[101], 9}, {NULL}};
const Segment SGI96480[] = {{&S66722[74], 6}, {NULL}};
const Segment SGI96481[] = {{&S66727[34], 10}, {NULL}};
const Segment SGI96482[] = {{&S66727[27], 6}, {NULL}};
const Segment SGI96483[] = {{&S66727[97], 9}, {NULL}};
const Segment SGI96484[] = {{&S66727[70], 6}, {NULL}};
const Segment SGI96485[] = {{&S66727[138], 8}, {NULL}};
const Segment SGI96486[] = {{&S88438[58], 11}, {NULL}};
const Segment SGI96487[] = {{&S88438[58], 23}, {NULL}};
const Segment SGI96488[] = {{&S88438[44], 6}, {NULL}};
const Segment SGI96489[] = {{&S88438[129], 3}, {NULL}};
const Segment SGI96490[] = {{&S88438[95], 5}, {NULL}};
const Segment SGI96491[] = {{&S88438[0], 14}, {NULL}};
const Segment SGI96492[] = {{&S66751[34], 10}, {NULL}};
const Segment SGI96493[] = {{&S66751[27], 6}, {NULL}};
const Segment SGI96494[] = {{&S66751[97], 9}, {NULL}};
const Segment SGI96495[] = {{&S66751[70], 6}, {NULL}};
const Segment SGI96496[] = {{&S66751[132], 8}, {NULL}};
const Segment SGI96497[] = {{&S66755[34], 10}, {NULL}};
const Segment SGI96498[] = {{&S66755[27], 6}, {NULL}};
const Segment SGI96499[] = {{&S66755[97], 9}, {NULL}};
const Segment SGI96500[] = {{&S66755[70], 6}, {NULL}};
const Segment SGI96501[] = {{&S66755[131], 8}, {NULL}};
const Segment SGI96502[] = {{&S88474[44], 9}, {NULL}};
const Segment SGI96503[] = {{&S88474[44], 21}, {NULL}};
const Segment SGI96504[] = {{&S88474[30], 6}, {NULL}};
const Segment SGI96505[] = {{&S88474[113], 3}, {NULL}};
const Segment SGI96506[] = {{&S88474[79], 5}, {NULL}};
const Segment SGI96507[] = {{&S88495[44], 11}, {NULL}};
const Segment SGI96508[] = {{&S88495[44], 28}, {NULL}};
const Segment SGI96509[] = {{&S88495[30], 6}, {NULL}};
const Segment SGI96510[] = {{&S88495[86], 6}, {NULL}};
const Segment SGI96511[] = {{&S88495[113], 13}, {NULL}};
const Segment SGI96512[] = {{&S66804[34], 16}, {NULL}};
const Segment SGI96513[] = {{&S66804[27], 6}, {NULL}};
const Segment SGI96514[] = {{&S66804[103], 9}, {NULL}};
const Segment SGI96515[] = {{&S66804[76], 6}, {NULL}};
const Segment SGI96516[] = {{&S66816[34], 17}, {NULL}};
const Segment SGI96517[] = {{&S66816[27], 6}, {NULL}};
const Segment SGI96518[] = {{&S66816[112], 9}, {NULL}};
const Segment SGI96519[] = {{&S66816[85], 6}, {NULL}};
const Segment SGI96520[] = {{&S66817[34], 17}, {NULL}};
const Segment SGI96521[] = {{&S66817[27], 6}, {NULL}};
const Segment SGI96522[] = {{&S66817[112], 9}, {NULL}};
const Segment SGI96523[] = {{&S66817[85], 6}, {NULL}};
const Segment SGI96524[] = {{&S66819[34], 13}, {NULL}};
const Segment SGI96525[] = {{&S66819[27], 6}, {NULL}};
const Segment SGI96526[] = {{&S66819[108], 9}, {NULL}};
const Segment SGI96527[] = {{&S66819[81], 6}, {NULL}};
const Segment SGI96528[] = {{&S88528[44], 11}, {NULL}};
const Segment SGI96529[] = {{&S88528[44], 28}, {NULL}};
const Segment SGI96530[] = {{&S88528[30], 6}, {NULL}};
const Segment SGI96531[] = {{&S88528[119], 3}, {NULL}};
const Segment SGI96532[] = {{&S88528[86], 5}, {NULL}};
const Segment SGI96533[] = {{&S88528[123], 14}, {NULL}};
const Segment SGI96534[] = {{&S66833[34], 7}, {NULL}};
const Segment SGI96535[] = {{&S66833[27], 6}, {NULL}};
const Segment SGI96536[] = {{&S66833[94], 9}, {NULL}};
const Segment SGI96537[] = {{&S66833[67], 6}, {NULL}};
const Segment SGI96538[] = {{&S66849[34], 10}, {NULL}};
const Segment SGI96539[] = {{&S66849[27], 6}, {NULL}};
const Segment SGI96540[] = {{&S66849[97], 9}, {NULL}};
const Segment SGI96541[] = {{&S66849[70], 6}, {NULL}};
const Segment SGI96542[] = {{&S88601[44], 9}, {NULL}};
const Segment SGI96543[] = {{&S88601[44], 21}, {NULL}};
const Segment SGI96544[] = {{&S88601[30], 6}, {NULL}};
const Segment SGI96545[] = {{&S88601[113], 3}, {NULL}};
const Segment SGI96546[] = {{&S88601[79], 5}, {NULL}};
const Segment SGI96547[] = {{&S88613[59], 10}, {NULL}};
const Segment SGI96548[] = {{&S88613[59], 27}, {NULL}};
const Segment SGI96549[] = {{&S88613[45], 6}, {NULL}};
const Segment SGI96550[] = {{&S88613[134], 3}, {NULL}};
const Segment SGI96551[] = {{&S88613[100], 5}, {NULL}};
const Segment SGI96552[] = {{&S88613[0], 14}, {NULL}};
const Segment SGI96553[] = {{&S88614[59], 10}, {NULL}};
const Segment SGI96554[] = {{&S88614[59], 27}, {NULL}};
const Segment SGI96555[] = {{&S88614[45], 6}, {NULL}};
const Segment SGI96556[] = {{&S88614[134], 3}, {NULL}};
const Segment SGI96557[] = {{&S88614[100], 5}, {NULL}};
const Segment SGI96558[] = {{&S88614[0], 14}, {NULL}};
const Segment SGI96559[] = {{&S88637[44], 10}, {NULL}};
const Segment SGI96560[] = {{&S88637[44], 27}, {NULL}};
const Segment SGI96561[] = {{&S88637[30], 6}, {NULL}};
const Segment SGI96562[] = {{&S88637[85], 6}, {NULL}};
const Segment SGI96563[] = {{&S88637[112], 13}, {NULL}};
const Segment SGI96564[] = {{&S88639[44], 5}, {NULL}};
const Segment SGI96565[] = {{&S88639[44], 26}, {NULL}};
const Segment SGI96566[] = {{&S88639[30], 6}, {NULL}};
const Segment SGI96567[] = {{&S88639[117], 3}, {NULL}};
const Segment SGI96568[] = {{&S88639[84], 5}, {NULL}};
const Segment SGI96569[] = {{&S88639[121], 14}, {NULL}};
const Segment SGI96570[] = {{&S81469[44], 9}, {NULL}};
const Segment SGI96571[] = {{&S81469[44], 20}, {NULL}};
const Segment SGI96572[] = {{&S81469[30], 6}, {NULL}};
const Segment SGI96573[] = {{&S81469[131], 3}, {NULL}};
const Segment SGI96574[] = {{&S81469[97], 5}, {NULL}};
const Segment SGI96575[] = {{&S81469[35], 1}, {NULL}};
const Segment SGI96576[] = {{&S81469[66], 11}, {NULL}};
const Segment SGI96577[] = {{&S66937[34], 7}, {NULL}};
const Segment SGI96578[] = {{&S66937[27], 6}, {NULL}};
const Segment SGI96579[] = {{&S66937[94], 9}, {NULL}};
const Segment SGI96580[] = {{&S66937[67], 6}, {NULL}};
const Segment SGI96581[] = {{&S66938[34], 10}, {NULL}};
const Segment SGI96582[] = {{&S66938[27], 6}, {NULL}};
const Segment SGI96583[] = {{&S66938[97], 9}, {NULL}};
const Segment SGI96584[] = {{&S66938[70], 6}, {NULL}};
const Segment SGI96585[] = {{&S88743[44], 16}, {NULL}};
const Segment SGI96586[] = {{&S88743[44], 28}, {NULL}};
const Segment SGI96587[] = {{&S88743[30], 6}, {NULL}};
const Segment SGI96588[] = {{&S88743[120], 3}, {NULL}};
const Segment SGI96589[] = {{&S88743[86], 5}, {NULL}};
const Segment SGI96590[] = {{&S88748[44], 13}, {NULL}};
const Segment SGI96591[] = {{&S88748[44], 30}, {NULL}};
const Segment SGI96592[] = {{&S88748[30], 6}, {NULL}};
const Segment SGI96593[] = {{&S88748[129], 3}, {NULL}};
const Segment SGI96594[] = {{&S88748[95], 5}, {NULL}};
const Segment SGI96595[] = {{&S88748[74], 8}, {NULL}};
const Segment SGI96596[] = {{&S88749[44], 13}, {NULL}};
const Segment SGI96597[] = {{&S88749[44], 30}, {NULL}};
const Segment SGI96598[] = {{&S88749[30], 6}, {NULL}};
const Segment SGI96599[] = {{&S88749[122], 3}, {NULL}};
const Segment SGI96600[] = {{&S88749[88], 5}, {NULL}};
const Segment SGI96601[] = {{&S88749[146], 7}, {NULL}};
const Segment SGI96602[] = {{&S88759[44], 5}, {NULL}};
const Segment SGI96603[] = {{&S88759[44], 17}, {NULL}};
const Segment SGI96604[] = {{&S88759[30], 6}, {NULL}};
const Segment SGI96605[] = {{&S88759[75], 6}, {NULL}};
const Segment SGI96606[] = {{&S88759[102], 13}, {NULL}};
const Segment SGI96607[] = {{&S66998[44], 8}, {NULL}};
const Segment SGI96608[] = {{&S66998[44], 20}, {NULL}};
const Segment SGI96609[] = {{&S66998[30], 6}, {NULL}};
const Segment SGI96610[] = {{&S66998[31], 3}, {NULL}};
const Segment SGI96611[] = {{&S66998[78], 6}, {NULL}};
const Segment SGI96612[] = {{&S66999[44], 18}, {NULL}};
const Segment SGI96613[] = {{&S66999[44], 30}, {NULL}};
const Segment SGI96614[] = {{&S66999[30], 6}, {NULL}};
const Segment SGI96615[] = {{&S66999[31], 3}, {NULL}};
const Segment SGI96616[] = {{&S66999[88], 6}, {NULL}};
const Segment SGI96617[] = {{&S67007[34], 17}, {NULL}};
const Segment SGI96618[] = {{&S67007[27], 6}, {NULL}};
const Segment SGI96619[] = {{&S67007[120], 9}, {NULL}};
const Segment SGI96620[] = {{&S67007[93], 6}, {NULL}};
const Segment SGI96621[] = {{&S67008[44], 9}, {NULL}};
const Segment SGI96622[] = {{&S67008[44], 21}, {NULL}};
const Segment SGI96623[] = {{&S67008[30], 6}, {NULL}};
const Segment SGI96624[] = {{&S67008[31], 3}, {NULL}};
const Segment SGI96625[] = {{&S67008[79], 6}, {NULL}};
const Segment SGI96626[] = {{&S67011[44], 6}, {NULL}};
const Segment SGI96627[] = {{&S67011[44], 18}, {NULL}};
const Segment SGI96628[] = {{&S67011[30], 6}, {NULL}};
const Segment SGI96629[] = {{&S67011[31], 3}, {NULL}};
const Segment SGI96630[] = {{&S67011[76], 6}, {NULL}};
const Segment SGI96631[] = {{&S88851[45], 10}, {NULL}};
const Segment SGI96632[] = {{&S88851[30], 6}, {NULL}};
const Segment SGI96633[] = {{&S88851[120], 3}, {NULL}};
const Segment SGI96634[] = {{&S88851[86], 5}, {NULL}};
const Segment SGI96635[] = {{&S88851[144], 13}, {NULL}};
const Segment SGI96636[] = {{&S88873[45], 5}, {NULL}};
const Segment SGI96637[] = {{&S88873[30], 6}, {NULL}};
const Segment SGI96638[] = {{&S88873[110], 3}, {NULL}};
const Segment SGI96639[] = {{&S88873[76], 5}, {NULL}};
const Segment SGI96640[] = {{&S88873[134], 13}, {NULL}};
const Segment SGI96641[] = {{&S80831[42], 13}, {NULL}};
const Segment SGI96642[] = {{&S80831[30], 4}, {NULL}};
const Segment SGI96643[] = {{&S80831[104], 3}, {NULL}};
const Segment SGI96644[] = {{&S80831[69], 6}, {NULL}};
const Segment SGI96645[] = {{&S80832[42], 13}, {NULL}};
const Segment SGI96646[] = {{&S80832[30], 4}, {NULL}};
const Segment SGI96647[] = {{&S80832[104], 3}, {NULL}};
const Segment SGI96648[] = {{&S80832[69], 6}, {NULL}};
const Segment SGI96649[] = {{&S80833[44], 9}, {NULL}};
const Segment SGI96650[] = {{&S80833[44], 20}, {NULL}};
const Segment SGI96651[] = {{&S80833[30], 6}, {NULL}};
const Segment SGI96652[] = {{&S80833[113], 3}, {NULL}};
const Segment SGI96653[] = {{&S80833[78], 6}, {NULL}};
const Segment SGI96654[] = {{&S88911[44], 10}, {NULL}};
const Segment SGI96655[] = {{&S88911[44], 27}, {NULL}};
const Segment SGI96656[] = {{&S88911[30], 6}, {NULL}};
const Segment SGI96657[] = {{&S88911[120], 3}, {NULL}};
const Segment SGI96658[] = {{&S88911[85], 6}, {NULL}};
const Segment SGI96659[] = {{&S88911[140], 8}, {NULL}};
const Segment SGI96660[] = {{&S88911[149], 13}, {NULL}};
const Segment SGI96661[] = {{&S80867[44], 8}, {NULL}};
const Segment SGI96662[] = {{&S80867[44], 19}, {NULL}};
const Segment SGI96663[] = {{&S80867[30], 6}, {NULL}};
const Segment SGI96664[] = {{&S80867[112], 3}, {NULL}};
const Segment SGI96665[] = {{&S80867[77], 6}, {NULL}};
const Segment SGI96666[] = {{&S81693[45], 10}, {NULL}};
const Segment SGI96667[] = {{&S81693[30], 6}, {NULL}};
const Segment SGI96668[] = {{&S81693[115], 3}, {NULL}};
const Segment SGI96669[] = {{&S81693[81], 5}, {NULL}};
const Segment SGI96670[] = {{&S81693[139], 13}, {NULL}};
const Segment SGI96671[] = {{&S88931[44], 13}, {NULL}};
const Segment SGI96672[] = {{&S88931[44], 25}, {NULL}};
const Segment SGI96673[] = {{&S88931[30], 6}, {NULL}};
const Segment SGI96674[] = {{&S88931[83], 6}, {NULL}};
const Segment SGI96675[] = {{&S88931[110], 13}, {NULL}};
const Segment SGI96676[] = {{&S88935[52], 10}, {NULL}};
const Segment SGI96677[] = {{&S88935[52], 27}, {NULL}};
const Segment SGI96678[] = {{&S88935[38], 6}, {NULL}};
const Segment SGI96679[] = {{&S88935[93], 5}, {NULL}};
const Segment SGI96680[] = {{&S88935[0], 14}, {NULL}};
const Segment SGI96681[] = {{&S67067[34], 29}, {NULL}};
const Segment SGI96682[] = {{&S67067[27], 6}, {NULL}};
const Segment SGI96683[] = {{&S67067[116], 9}, {NULL}};
const Segment SGI96684[] = {{&S67067[89], 6}, {NULL}};
const Segment SGI96685[] = {{&S81746[44], 11}, {NULL}};
const Segment SGI96686[] = {{&S81746[44], 23}, {NULL}};
const Segment SGI96687[] = {{&S81746[30], 6}, {NULL}};
const Segment SGI96688[] = {{&S81746[130], 3}, {NULL}};
const Segment SGI96689[] = {{&S81746[96], 5}, {NULL}};
const Segment SGI96690[] = {{&S81746[35], 1}, {NULL}};
const Segment SGI96691[] = {{&S81746[69], 11}, {NULL}};
const Segment SGI96692[] = {{&S67102[44], 10}, {NULL}};
const Segment SGI96693[] = {{&S67102[44], 22}, {NULL}};
const Segment SGI96694[] = {{&S67102[30], 6}, {NULL}};
const Segment SGI96695[] = {{&S67102[31], 3}, {NULL}};
const Segment SGI96696[] = {{&S67102[80], 6}, {NULL}};
const Segment SGI96697[] = {{&S89001[59], 11}, {NULL}};
const Segment SGI96698[] = {{&S89001[59], 28}, {NULL}};
const Segment SGI96699[] = {{&S89001[45], 6}, {NULL}};
const Segment SGI96700[] = {{&S89001[135], 3}, {NULL}};
const Segment SGI96701[] = {{&S89001[101], 5}, {NULL}};
const Segment SGI96702[] = {{&S89001[0], 14}, {NULL}};
const Segment SGI96703[] = {{&S89013[59], 4}, {NULL}};
const Segment SGI96704[] = {{&S89013[59], 16}, {NULL}};
const Segment SGI96705[] = {{&S89013[45], 6}, {NULL}};
const Segment SGI96706[] = {{&S78465[42], 4}, {NULL}};
const Segment SGI96707[] = {{&S78465[42], 25}, {NULL}};
const Segment SGI96708[] = {{&S78465[30], 4}, {NULL}};
const Segment SGI96709[] = {{&S89013[123], 3}, {NULL}};
const Segment SGI96710[] = {{&S89013[89], 5}, {NULL}};
const Segment SGI96711[] = {{&S78465[31], 3}, {NULL}};
const Segment SGI96712[] = {{&S78465[81], 5}, {NULL}};
const Segment SGI96713[] = {{&S89013[0], 14}, {NULL}};
const Segment SGI96714[] = {{&S75539[34], 14}, {NULL}};
const Segment SGI96715[] = {{&S75539[27], 6}, {NULL}};
const Segment SGI96716[] = {{&S75539[101], 9}, {NULL}};
const Segment SGI96717[] = {{&S75539[74], 6}, {NULL}};
const Segment SGI96718[] = {{&S75539[0], 13}, {NULL}};
const Segment SGI96719[] = {{&S89036[45], 10}, {NULL}};
const Segment SGI96720[] = {{&S89036[30], 6}, {NULL}};
const Segment SGI96721[] = {{&S89036[120], 3}, {NULL}};
const Segment SGI96722[] = {{&S89036[86], 5}, {NULL}};
const Segment SGI96723[] = {{&S89036[144], 13}, {NULL}};
const Segment SGI96724[] = {{&S67140[44], 11}, {NULL}};
const Segment SGI96725[] = {{&S67140[44], 23}, {NULL}};
const Segment SGI96726[] = {{&S67140[30], 6}, {NULL}};
const Segment SGI96727[] = {{&S67140[31], 3}, {NULL}};
const Segment SGI96728[] = {{&S67140[81], 6}, {NULL}};
const Segment SGI96729[] = {{&S67140[141], 11}, {NULL}};
const Segment SGI96730[] = {{&S67141[44], 10}, {NULL}};
const Segment SGI96731[] = {{&S67141[44], 22}, {NULL}};
const Segment SGI96732[] = {{&S67141[30], 6}, {NULL}};
const Segment SGI96733[] = {{&S67141[31], 3}, {NULL}};
const Segment SGI96734[] = {{&S67141[80], 6}, {NULL}};
const Segment SGI96735[] = {{&S67141[140], 11}, {NULL}};
const Segment SGI96736[] = {{&S67145[34], 14}, {NULL}};
const Segment SGI96737[] = {{&S67145[27], 6}, {NULL}};
const Segment SGI96738[] = {{&S67145[101], 9}, {NULL}};
const Segment SGI96739[] = {{&S67145[74], 6}, {NULL}};
const Segment SGI96740[] = {{&S75600[44], 13}, {NULL}};
const Segment SGI96741[] = {{&S75600[44], 25}, {NULL}};
const Segment SGI96742[] = {{&S75600[30], 6}, {NULL}};
const Segment SGI96743[] = {{&S89072[44], 10}, {NULL}};
const Segment SGI96744[] = {{&S75600[31], 3}, {NULL}};
const Segment SGI96745[] = {{&S89072[44], 27}, {NULL}};
const Segment SGI96746[] = {{&S75600[83], 6}, {NULL}};
const Segment SGI96747[] = {{&S89072[30], 6}, {NULL}};
const Segment SGI96748[] = {{&S75600[0], 13}, {NULL}};
const Segment SGI96749[] = {{&S89072[119], 3}, {NULL}};
const Segment SGI96750[] = {{&S89072[85], 5}, {NULL}};
const Segment SGI96751[] = {{&S89072[143], 7}, {NULL}};
const Segment SGI96752[] = {{&S89080[44], 10}, {NULL}};
const Segment SGI96753[] = {{&S89080[44], 27}, {NULL}};
const Segment SGI96754[] = {{&S89080[30], 6}, {NULL}};
const Segment SGI96755[] = {{&S89080[85], 6}, {NULL}};
const Segment SGI96756[] = {{&S89080[112], 13}, {NULL}};
const Segment SGI96757[] = {{&S67183[44], 17}, {NULL}};
const Segment SGI96758[] = {{&S67183[30], 6}, {NULL}};
const Segment SGI96759[] = {{&S67183[31], 3}, {NULL}};
const Segment SGI96760[] = {{&S67183[89], 6}, {NULL}};
const Segment SGI96761[] = {{&S67184[44], 16}, {NULL}};
const Segment SGI96762[] = {{&S67184[44], 28}, {NULL}};
const Segment SGI96763[] = {{&S67184[30], 6}, {NULL}};
const Segment SGI96764[] = {{&S67184[31], 3}, {NULL}};
const Segment SGI96765[] = {{&S67184[86], 6}, {NULL}};
const Segment SGI96766[] = {{&S67187[44], 5}, {NULL}};
const Segment SGI96767[] = {{&S67187[44], 17}, {NULL}};
const Segment SGI96768[] = {{&S67187[30], 6}, {NULL}};
const Segment SGI96769[] = {{&S67187[31], 3}, {NULL}};
const Segment SGI96770[] = {{&S67187[75], 6}, {NULL}};
const Segment SGI96771[] = {{&S67188[38], 5}, {NULL}};
const Segment SGI96772[] = {{&S67188[31], 6}, {NULL}};
const Segment SGI96773[] = {{&S67200[34], 10}, {NULL}};
const Segment SGI96774[] = {{&S67200[27], 6}, {NULL}};
const Segment SGI96775[] = {{&S67200[97], 9}, {NULL}};
const Segment SGI96776[] = {{&S67200[70], 6}, {NULL}};
const Segment SGI96777[] = {{&S67201[34], 9}, {NULL}};
const Segment SGI96778[] = {{&S67201[27], 6}, {NULL}};
const Segment SGI96779[] = {{&S67201[96], 9}, {NULL}};
const Segment SGI96780[] = {{&S67201[69], 6}, {NULL}};
const Segment SGI96781[] = {{&S67202[34], 10}, {NULL}};
const Segment SGI96782[] = {{&S67202[27], 6}, {NULL}};
const Segment SGI96783[] = {{&S67202[97], 9}, {NULL}};
const Segment SGI96784[] = {{&S67202[70], 6}, {NULL}};
const Segment SGI96785[] = {{&S67203[34], 10}, {NULL}};
const Segment SGI96786[] = {{&S67203[27], 6}, {NULL}};
const Segment SGI96787[] = {{&S67203[97], 9}, {NULL}};
const Segment SGI96788[] = {{&S67203[70], 6}, {NULL}};
const Segment SGI96789[] = {{&S89154[44], 10}, {NULL}};
const Segment SGI96790[] = {{&S89154[44], 27}, {NULL}};
const Segment SGI96791[] = {{&S89154[30], 6}, {NULL}};
const Segment SGI96792[] = {{&S89154[119], 3}, {NULL}};
const Segment SGI96793[] = {{&S89154[85], 5}, {NULL}};
const Segment SGI96794[] = {{&S67228[34], 8}, {NULL}};
const Segment SGI96795[] = {{&S67228[27], 6}, {NULL}};
const Segment SGI96796[] = {{&S67228[96], 9}, {NULL}};
const Segment SGI96797[] = {{&S67228[69], 6}, {NULL}};
const Segment SGI96798[] = {{&S67229[34], 8}, {NULL}};
const Segment SGI96799[] = {{&S67229[27], 6}, {NULL}};
const Segment SGI96800[] = {{&S67229[96], 9}, {NULL}};
const Segment SGI96801[] = {{&S67229[69], 6}, {NULL}};
const Segment SGI96802[] = {{&S89162[45], 9}, {NULL}};
const Segment SGI96803[] = {{&S89162[30], 6}, {NULL}};
const Segment SGI96804[] = {{&S89162[114], 3}, {NULL}};
const Segment SGI96805[] = {{&S89162[80], 5}, {NULL}};
const Segment SGI96806[] = {{&S89162[138], 13}, {NULL}};
const Segment SGI96807[] = {{&S89164[59], 11}, {NULL}};
const Segment SGI96808[] = {{&S89164[59], 23}, {NULL}};
const Segment SGI96809[] = {{&S89164[45], 6}, {NULL}};
const Segment SGI96810[] = {{&S89164[130], 3}, {NULL}};
const Segment SGI96811[] = {{&S89164[96], 5}, {NULL}};
const Segment SGI96812[] = {{&S89164[0], 14}, {NULL}};
const Segment SGI96813[] = {{&S89179[44], 4}, {NULL}};
const Segment SGI96814[] = {{&S89179[44], 16}, {NULL}};
const Segment SGI96815[] = {{&S89179[30], 6}, {NULL}};
const Segment SGI96816[] = {{&S89179[107], 3}, {NULL}};
const Segment SGI96817[] = {{&S89179[74], 5}, {NULL}};
const Segment SGI96818[] = {{&S89179[111], 14}, {NULL}};
const Segment SGI96819[] = {{&S67253[34], 7}, {NULL}};
const Segment SGI96820[] = {{&S67253[27], 6}, {NULL}};
const Segment SGI96821[] = {{&S67253[99], 9}, {NULL}};
const Segment SGI96822[] = {{&S67253[72], 6}, {NULL}};
const Segment SGI96823[] = {{&S67265[34], 10}, {NULL}};
const Segment SGI96824[] = {{&S67265[27], 6}, {NULL}};
const Segment SGI96825[] = {{&S67265[97], 9}, {NULL}};
const Segment SGI96826[] = {{&S67265[70], 6}, {NULL}};
const Segment SGI96827[] = {{&S89196[44], 5}, {NULL}};
const Segment SGI96828[] = {{&S89196[44], 26}, {NULL}};
const Segment SGI96829[] = {{&S89196[30], 6}, {NULL}};
const Segment SGI96830[] = {{&S89196[84], 6}, {NULL}};
const Segment SGI96831[] = {{&S89196[111], 13}, {NULL}};
const Segment SGI96832[] = {{&S67287[44], 10}, {NULL}};
const Segment SGI96833[] = {{&S67287[44], 22}, {NULL}};
const Segment SGI96834[] = {{&S67287[30], 6}, {NULL}};
const Segment SGI96835[] = {{&S67287[31], 3}, {NULL}};
const Segment SGI96836[] = {{&S67287[80], 6}, {NULL}};
const Segment SGI96837[] = {{&S67288[38], 10}, {NULL}};
const Segment SGI96838[] = {{&S67288[31], 6}, {NULL}};
const Segment SGI96839[] = {{&S67300[44], 4}, {NULL}};
const Segment SGI96840[] = {{&S67300[44], 16}, {NULL}};
const Segment SGI96841[] = {{&S67300[30], 6}, {NULL}};
const Segment SGI96842[] = {{&S67300[31], 3}, {NULL}};
const Segment SGI96843[] = {{&S67300[74], 6}, {NULL}};
const Segment SGI96844[] = {{&S67302[44], 4}, {NULL}};
const Segment SGI96845[] = {{&S67302[44], 15}, {NULL}};
const Segment SGI96846[] = {{&S67302[30], 6}, {NULL}};
const Segment SGI96847[] = {{&S67302[31], 3}, {NULL}};
const Segment SGI96848[] = {{&S67302[73], 6}, {NULL}};
const Segment SGI96849[] = {{&S67326[44], 12}, {NULL}};
const Segment SGI96850[] = {{&S67326[44], 24}, {NULL}};
const Segment SGI96851[] = {{&S67326[30], 6}, {NULL}};
const Segment SGI96852[] = {{&S67326[31], 3}, {NULL}};
const Segment SGI96853[] = {{&S67326[82], 6}, {NULL}};
const Segment SGI96854[] = {{&S67328[44], 9}, {NULL}};
const Segment SGI96855[] = {{&S67328[44], 21}, {NULL}};
const Segment SGI96856[] = {{&S67328[30], 6}, {NULL}};
const Segment SGI96857[] = {{&S67328[31], 3}, {NULL}};
const Segment SGI96858[] = {{&S67328[79], 6}, {NULL}};
const Segment SGI96859[] = {{&S89259[31], 8}, {NULL}};
const Segment SGI96860[] = {{&S89259[54], 3}, {NULL}};
const Segment SGI96861[] = {{&S67331[34], 11}, {NULL}};
const Segment SGI96862[] = {{&S67331[27], 6}, {NULL}};
const Segment SGI96863[] = {{&S67331[98], 9}, {NULL}};
const Segment SGI96864[] = {{&S67331[71], 6}, {NULL}};
const Segment SGI96865[] = {{&S67335[34], 11}, {NULL}};
const Segment SGI96866[] = {{&S67335[27], 6}, {NULL}};
const Segment SGI96867[] = {{&S67335[98], 9}, {NULL}};
const Segment SGI96868[] = {{&S67335[71], 6}, {NULL}};
const Segment SGI96869[] = {{&S89279[59], 5}, {NULL}};
const Segment SGI96870[] = {{&S89279[59], 17}, {NULL}};
const Segment SGI96871[] = {{&S89279[45], 6}, {NULL}};
const Segment SGI96872[] = {{&S89279[124], 3}, {NULL}};
const Segment SGI96873[] = {{&S89279[90], 5}, {NULL}};
const Segment SGI96874[] = {{&S89279[0], 14}, {NULL}};
const Segment SGI96875[] = {{&S67349[44], 15}, {NULL}};
const Segment SGI96876[] = {{&S67349[44], 27}, {NULL}};
const Segment SGI96877[] = {{&S67349[30], 6}, {NULL}};
const Segment SGI96878[] = {{&S67349[31], 3}, {NULL}};
const Segment SGI96879[] = {{&S67349[85], 6}, {NULL}};
const Segment SGI96880[] = {{&S67361[44], 11}, {NULL}};
const Segment SGI96881[] = {{&S67361[30], 6}, {NULL}};
const Segment SGI96882[] = {{&S67361[31], 3}, {NULL}};
const Segment SGI96883[] = {{&S67361[86], 6}, {NULL}};
const Segment SGI96884[] = {{&S67362[44], 9}, {NULL}};
const Segment SGI96885[] = {{&S67362[44], 21}, {NULL}};
const Segment SGI96886[] = {{&S67362[30], 6}, {NULL}};
const Segment SGI96887[] = {{&S67362[31], 3}, {NULL}};
const Segment SGI96888[] = {{&S67362[79], 6}, {NULL}};
const Segment SGI96889[] = {{&S89303[45], 9}, {NULL}};
const Segment SGI96890[] = {{&S89303[30], 6}, {NULL}};
const Segment SGI96891[] = {{&S89303[113], 3}, {NULL}};
const Segment SGI96892[] = {{&S89303[79], 5}, {NULL}};
const Segment SGI96893[] = {{&S89303[137], 13}, {NULL}};
const Segment SGI96894[] = {{&S89319[44], 14}, {NULL}};
const Segment SGI96895[] = {{&S89319[44], 26}, {NULL}};
const Segment SGI96896[] = {{&S89319[30], 6}, {NULL}};
const Segment SGI96897[] = {{&S89319[84], 6}, {NULL}};
const Segment SGI96898[] = {{&S89319[111], 13}, {NULL}};
const Segment SGI96899[] = {{&S67390[44], 5}, {NULL}};
const Segment SGI96900[] = {{&S67390[44], 17}, {NULL}};
const Segment SGI96901[] = {{&S67390[30], 6}, {NULL}};
const Segment SGI96902[] = {{&S67390[31], 3}, {NULL}};
const Segment SGI96903[] = {{&S67390[75], 6}, {NULL}};
const Segment SGI96904[] = {{&S75850[73], 6}, {NULL}};
const Segment SGI96905[] = {{&S75850[0], 8}, {NULL}};
const Segment SGI96906[] = {{&S82027[44], 6}, {NULL}};
const Segment SGI96907[] = {{&S82027[44], 17}, {NULL}};
const Segment SGI96908[] = {{&S82027[30], 6}, {NULL}};
const Segment SGI96909[] = {{&S82027[109], 3}, {NULL}};
const Segment SGI96910[] = {{&S82027[75], 5}, {NULL}};
const Segment SGI96911[] = {{&S89339[44], 16}, {NULL}};
const Segment SGI96912[] = {{&S89339[30], 6}, {NULL}};
const Segment SGI96913[] = {{&S89339[108], 3}, {NULL}};
const Segment SGI96914[] = {{&S89339[74], 5}, {NULL}};
const Segment SGI96915[] = {{&S89361[44], 4}, {NULL}};
const Segment SGI96916[] = {{&S89361[44], 16}, {NULL}};
const Segment SGI96917[] = {{&S89361[30], 6}, {NULL}};
const Segment SGI96918[] = {{&S89361[74], 6}, {NULL}};
const Segment SGI96919[] = {{&S89361[101], 13}, {NULL}};
const Segment SGI96920[] = {{&S67437[44], 10}, {NULL}};
const Segment SGI96921[] = {{&S67437[44], 22}, {NULL}};
const Segment SGI96922[] = {{&S67437[30], 6}, {NULL}};
const Segment SGI96923[] = {{&S67437[139], 4}, {NULL}};
const Segment SGI96924[] = {{&S67437[165], 3}, {NULL}};
const Segment SGI96925[] = {{&S67437[31], 3}, {&S67437[214], 5}, {NULL}};
const Segment SGI96926[] = {{&S67437[80], 6}, {NULL}};
const Segment SGI96927[] = {{&S67438[42], 10}, {NULL}};
const Segment SGI96928[] = {{&S67438[30], 6}, {NULL}};
const Segment SGI96929[] = {{&S67438[137], 4}, {NULL}};
const Segment SGI96930[] = {{&S67438[163], 3}, {NULL}};
const Segment SGI96931[] = {{&S67438[31], 3}, {&S67438[212], 5}, {NULL}};
const Segment SGI96932[] = {{&S67438[78], 6}, {NULL}};
const Segment SGI96933[] = {{&S67476[34], 14}, {NULL}};
const Segment SGI96934[] = {{&S67476[27], 6}, {NULL}};
const Segment SGI96935[] = {{&S67476[101], 9}, {NULL}};
const Segment SGI96936[] = {{&S67476[74], 6}, {NULL}};
const Segment SGI96937[] = {{&S67492[44], 8}, {NULL}};
const Segment SGI96938[] = {{&S67492[44], 20}, {NULL}};
const Segment SGI96939[] = {{&S67492[30], 6}, {NULL}};
const Segment SGI96940[] = {{&S67492[31], 3}, {NULL}};
const Segment SGI96941[] = {{&S67492[78], 6}, {NULL}};
const Segment SGI96942[] = {{&S67496[34], 10}, {NULL}};
const Segment SGI96943[] = {{&S67496[27], 6}, {NULL}};
const Segment SGI96944[] = {{&S67496[97], 9}, {NULL}};
const Segment SGI96945[] = {{&S67496[70], 6}, {NULL}};
const Segment SGI96946[] = {{&S89441[59], 15}, {NULL}};
const Segment SGI96947[] = {{&S89441[59], 36}, {NULL}};
const Segment SGI96948[] = {{&S89441[45], 6}, {NULL}};
const Segment SGI96949[] = {{&S89441[143], 3}, {NULL}};
const Segment SGI96950[] = {{&S89441[109], 5}, {NULL}};
const Segment SGI96951[] = {{&S89441[0], 14}, {NULL}};
const Segment SGI96952[] = {{&S89442[44], 9}, {NULL}};
const Segment SGI96953[] = {{&S89442[44], 21}, {NULL}};
const Segment SGI96954[] = {{&S89442[30], 6}, {NULL}};
const Segment SGI96955[] = {{&S89442[113], 3}, {NULL}};
const Segment SGI96956[] = {{&S89442[79], 5}, {NULL}};
const Segment SGI96957[] = {{&S89461[59], 5}, {NULL}};
const Segment SGI96958[] = {{&S89461[59], 26}, {NULL}};
const Segment SGI96959[] = {{&S89461[45], 6}, {NULL}};
const Segment SGI96960[] = {{&S89461[133], 3}, {NULL}};
const Segment SGI96961[] = {{&S89461[99], 5}, {NULL}};
const Segment SGI96962[] = {{&S89461[0], 14}, {NULL}};
const Segment SGI96963[] = {{&S89482[45], 11}, {NULL}};
const Segment SGI96964[] = {{&S89482[30], 6}, {NULL}};
const Segment SGI96965[] = {{&S89482[121], 3}, {NULL}};
const Segment SGI96966[] = {{&S89482[87], 5}, {NULL}};
const Segment SGI96967[] = {{&S89482[145], 13}, {NULL}};
const Segment SGI96968[] = {{&S89487[44], 10}, {NULL}};
const Segment SGI96969[] = {{&S89487[44], 27}, {NULL}};
const Segment SGI96970[] = {{&S89487[30], 6}, {NULL}};
const Segment SGI96971[] = {{&S89487[118], 3}, {NULL}};
const Segment SGI96972[] = {{&S89487[85], 5}, {NULL}};
const Segment SGI96973[] = {{&S89487[122], 14}, {NULL}};
const Segment SGI96974[] = {{&S89488[44], 13}, {NULL}};
const Segment SGI96975[] = {{&S89488[44], 30}, {NULL}};
const Segment SGI96976[] = {{&S89488[30], 6}, {NULL}};
const Segment SGI96977[] = {{&S89488[88], 6}, {NULL}};
const Segment SGI96978[] = {{&S89488[115], 13}, {NULL}};
const Segment SGI96979[] = {{&S89510[44], 11}, {NULL}};
const Segment SGI96980[] = {{&S89510[44], 28}, {NULL}};
const Segment SGI96981[] = {{&S89510[30], 6}, {NULL}};
const Segment SGI96982[] = {{&S89510[120], 3}, {NULL}};
const Segment SGI96983[] = {{&S89510[86], 5}, {NULL}};
const Segment SGI96984[] = {{&S89510[144], 7}, {NULL}};
const Segment SGI96985[] = {{&S67579[44], 10}, {NULL}};
const Segment SGI96986[] = {{&S67579[44], 22}, {NULL}};
const Segment SGI96987[] = {{&S67579[30], 6}, {NULL}};
const Segment SGI96988[] = {{&S67579[31], 3}, {NULL}};
const Segment SGI96989[] = {{&S67579[80], 6}, {NULL}};
const Segment SGI96990[] = {{&S67580[44], 7}, {NULL}};
const Segment SGI96991[] = {{&S67580[44], 19}, {NULL}};
const Segment SGI96992[] = {{&S67580[30], 6}, {NULL}};
const Segment SGI96993[] = {{&S67580[31], 3}, {NULL}};
const Segment SGI96994[] = {{&S67580[77], 6}, {NULL}};
const Segment SGI96995[] = {{&S67595[44], 7}, {NULL}};
const Segment SGI96996[] = {{&S67595[44], 28}, {NULL}};
const Segment SGI96997[] = {{&S67595[30], 6}, {NULL}};
const Segment SGI96998[] = {{&S67595[31], 3}, {NULL}};
const Segment SGI96999[] = {{&S67595[86], 6}, {NULL}};
const Segment SGI97000[] = {{&S67596[44], 7}, {NULL}};
const Segment SGI97001[] = {{&S67596[44], 19}, {NULL}};
const Segment SGI97002[] = {{&S67596[30], 6}, {NULL}};
const Segment SGI97003[] = {{&S67596[31], 3}, {NULL}};
const Segment SGI97004[] = {{&S67596[77], 6}, {NULL}};
const Segment SGI97005[] = {{&S67599[44], 8}, {NULL}};
const Segment SGI97006[] = {{&S67599[44], 20}, {NULL}};
const Segment SGI97007[] = {{&S67599[30], 6}, {NULL}};
const Segment SGI97008[] = {{&S67599[31], 3}, {NULL}};
const Segment SGI97009[] = {{&S67599[78], 6}, {NULL}};
const Segment SGI97010[] = {{&S67616[44], 17}, {NULL}};
const Segment SGI97011[] = {{&S67616[44], 29}, {NULL}};
const Segment SGI97012[] = {{&S67616[30], 6}, {NULL}};
const Segment SGI97013[] = {{&S67616[31], 3}, {NULL}};
const Segment SGI97014[] = {{&S67616[87], 6}, {NULL}};
const Segment SGI97015[] = {{&S67617[38], 17}, {NULL}};
const Segment SGI97016[] = {{&S67617[31], 6}, {NULL}};
const Segment SGI97017[] = {{&S82274[44], 7}, {NULL}};
const Segment SGI97018[] = {{&S82274[44], 17}, {NULL}};
const Segment SGI97019[] = {{&S82274[30], 6}, {NULL}};
const Segment SGI97020[] = {{&S82274[109], 3}, {NULL}};
const Segment SGI97021[] = {{&S82274[75], 5}, {NULL}};
const Segment SGI97022[] = {{&S82275[44], 30}, {NULL}};
const Segment SGI97023[] = {{&S82275[30], 6}, {NULL}};
const Segment SGI97024[] = {{&S82275[134], 3}, {NULL}};
const Segment SGI97025[] = {{&S82275[100], 5}, {NULL}};
const Segment SGI97026[] = {{&S82276[44], 17}, {NULL}};
const Segment SGI97027[] = {{&S82276[44], 27}, {NULL}};
const Segment SGI97028[] = {{&S82276[30], 6}, {NULL}};
const Segment SGI97029[] = {{&S82276[119], 3}, {NULL}};
const Segment SGI97030[] = {{&S82276[85], 5}, {NULL}};
const Segment SGI97031[] = {{&S76078[77], 6}, {NULL}};
const Segment SGI97032[] = {{&S76078[0], 12}, {NULL}};
const Segment SGI97033[] = {{&S89608[44], 12}, {NULL}};
const Segment SGI97034[] = {{&S89608[44], 24}, {NULL}};
const Segment SGI97035[] = {{&S89608[30], 6}, {NULL}};
const Segment SGI97036[] = {{&S89608[116], 3}, {NULL}};
const Segment SGI97037[] = {{&S89608[82], 5}, {NULL}};
const Segment SGI97038[] = {{&S67671[34], 21}, {NULL}};
const Segment SGI97039[] = {{&S67671[27], 6}, {NULL}};
const Segment SGI97040[] = {{&S67671[108], 9}, {NULL}};
const Segment SGI97041[] = {{&S67671[81], 6}, {NULL}};
const Segment SGI97042[] = {{&S67671[149], 8}, {NULL}};
const Segment SGI97043[] = {{&S67672[34], 18}, {NULL}};
const Segment SGI97044[] = {{&S67672[27], 6}, {NULL}};
const Segment SGI97045[] = {{&S67672[105], 9}, {NULL}};
const Segment SGI97046[] = {{&S67672[78], 6}, {NULL}};
const Segment SGI97047[] = {{&S67672[146], 8}, {NULL}};
const Segment SGI97048[] = {{&S67674[44], 5}, {NULL}};
const Segment SGI97049[] = {{&S67674[44], 17}, {NULL}};
const Segment SGI97050[] = {{&S67674[30], 6}, {NULL}};
const Segment SGI97051[] = {{&S67674[31], 3}, {NULL}};
const Segment SGI97052[] = {{&S67674[75], 6}, {NULL}};
const Segment SGI97053[] = {{&S67676[38], 5}, {NULL}};
const Segment SGI97054[] = {{&S67676[31], 6}, {NULL}};
const Segment SGI97055[] = {{&S67704[44], 11}, {NULL}};
const Segment SGI97056[] = {{&S67704[44], 23}, {NULL}};
const Segment SGI97057[] = {{&S67704[30], 6}, {NULL}};
const Segment SGI97058[] = {{&S67704[31], 3}, {NULL}};
const Segment SGI97059[] = {{&S67704[81], 6}, {NULL}};
const Segment SGI97060[] = {{&S67705[44], 6}, {NULL}};
const Segment SGI97061[] = {{&S67705[44], 18}, {NULL}};
const Segment SGI97062[] = {{&S67705[30], 6}, {NULL}};
const Segment SGI97063[] = {{&S67705[31], 3}, {NULL}};
const Segment SGI97064[] = {{&S67705[76], 6}, {NULL}};
const Segment SGI97065[] = {{&S89651[44], 10}, {NULL}};
const Segment SGI97066[] = {{&S89651[44], 27}, {NULL}};
const Segment SGI97067[] = {{&S89651[30], 6}, {NULL}};
const Segment SGI97068[] = {{&S89651[118], 3}, {NULL}};
const Segment SGI97069[] = {{&S89651[85], 5}, {NULL}};
const Segment SGI97070[] = {{&S89651[122], 14}, {NULL}};
const Segment SGI97071[] = {{&S89652[44], 11}, {NULL}};
const Segment SGI97072[] = {{&S89652[44], 28}, {NULL}};
const Segment SGI97073[] = {{&S89652[30], 6}, {NULL}};
const Segment SGI97074[] = {{&S89652[120], 3}, {NULL}};
const Segment SGI97075[] = {{&S89652[86], 5}, {NULL}};
const Segment SGI97076[] = {{&S89652[144], 7}, {NULL}};
const Segment SGI97077[] = {{&S67730[44], 10}, {NULL}};
const Segment SGI97078[] = {{&S67730[44], 22}, {NULL}};
const Segment SGI97079[] = {{&S67730[30], 6}, {NULL}};
const Segment SGI97080[] = {{&S67730[31], 3}, {NULL}};
const Segment SGI97081[] = {{&S67730[80], 6}, {NULL}};
const Segment SGI97082[] = {{&S67791[44], 9}, {NULL}};
const Segment SGI97083[] = {{&S67791[44], 21}, {NULL}};
const Segment SGI97084[] = {{&S67791[30], 6}, {NULL}};
const Segment SGI97085[] = {{&S67791[31], 3}, {NULL}};
const Segment SGI97086[] = {{&S67791[79], 6}, {NULL}};
const Segment SGI97087[] = {{&S67792[44], 10}, {NULL}};
const Segment SGI97088[] = {{&S67792[44], 22}, {NULL}};
const Segment SGI97089[] = {{&S67792[30], 6}, {NULL}};
const Segment SGI97090[] = {{&S67792[31], 3}, {NULL}};
const Segment SGI97091[] = {{&S67792[80], 6}, {NULL}};
const Segment SGI97092[] = {{&S67800[34], 30}, {NULL}};
const Segment SGI97093[] = {{&S67800[27], 6}, {NULL}};
const Segment SGI97094[] = {{&S67800[117], 9}, {NULL}};
const Segment SGI97095[] = {{&S89738[59], 4}, {NULL}};
const Segment SGI97096[] = {{&S67800[90], 6}, {NULL}};
const Segment SGI97097[] = {{&S89738[59], 16}, {NULL}};
const Segment SGI97098[] = {{&S89738[45], 6}, {NULL}};
const Segment SGI97099[] = {{&S89738[123], 3}, {NULL}};
const Segment SGI97100[] = {{&S89738[89], 5}, {NULL}};
const Segment SGI97101[] = {{&S89738[0], 14}, {NULL}};
const Segment SGI97102[] = {{&S67801[34], 29}, {NULL}};
const Segment SGI97103[] = {{&S67801[27], 6}, {NULL}};
const Segment SGI97104[] = {{&S67801[116], 9}, {NULL}};
const Segment SGI97105[] = {{&S89739[59], 5}, {NULL}};
const Segment SGI97106[] = {{&S67801[89], 6}, {NULL}};
const Segment SGI97107[] = {{&S89739[59], 17}, {NULL}};
const Segment SGI97108[] = {{&S89739[45], 6}, {NULL}};
const Segment SGI97109[] = {{&S89739[124], 3}, {NULL}};
const Segment SGI97110[] = {{&S89739[90], 5}, {NULL}};
const Segment SGI97111[] = {{&S89739[0], 14}, {NULL}};
const Segment SGI97112[] = {{&S67802[34], 30}, {NULL}};
const Segment SGI97113[] = {{&S67802[27], 6}, {NULL}};
const Segment SGI97114[] = {{&S67802[117], 9}, {NULL}};
const Segment SGI97115[] = {{&S67802[90], 6}, {NULL}};
const Segment SGI97116[] = {{&S67805[34], 21}, {NULL}};
const Segment SGI97117[] = {{&S67805[27], 6}, {NULL}};
const Segment SGI97118[] = {{&S67805[108], 9}, {NULL}};
const Segment SGI97119[] = {{&S67805[81], 6}, {NULL}};
const Segment SGI97120[] = {{&S67805[149], 8}, {NULL}};
const Segment SGI97121[] = {{&S67806[34], 18}, {NULL}};
const Segment SGI97122[] = {{&S67806[27], 6}, {NULL}};
const Segment SGI97123[] = {{&S67806[105], 9}, {NULL}};
const Segment SGI97124[] = {{&S67806[78], 6}, {NULL}};
const Segment SGI97125[] = {{&S67806[146], 8}, {NULL}};
const Segment SGI97126[] = {{&S82472[44], 18}, {NULL}};
const Segment SGI97127[] = {{&S82472[44], 36}, {NULL}};
const Segment SGI97128[] = {{&S82472[30], 6}, {NULL}};
const Segment SGI97129[] = {{&S82472[135], 3}, {NULL}};
const Segment SGI97130[] = {{&S82472[101], 5}, {NULL}};
const Segment SGI97131[] = {{&S82472[80], 8}, {NULL}};
const Segment SGI97132[] = {{&S67828[34], 10}, {NULL}};
const Segment SGI97133[] = {{&S67828[27], 6}, {NULL}};
const Segment SGI97134[] = {{&S67828[97], 9}, {NULL}};
const Segment SGI97135[] = {{&S67828[70], 6}, {NULL}};
const Segment SGI97136[] = {{&S81635[42], 14}, {NULL}};
const Segment SGI97137[] = {{&S81635[42], 25}, {NULL}};
const Segment SGI97138[] = {{&S81635[30], 4}, {NULL}};
const Segment SGI97139[] = {{&S81635[116], 3}, {NULL}};
const Segment SGI97140[] = {{&S81635[81], 6}, {NULL}};
const Segment SGI97141[] = {{&S67854[44], 14}, {NULL}};
const Segment SGI97142[] = {{&S67854[44], 26}, {NULL}};
const Segment SGI97143[] = {{&S67854[30], 6}, {NULL}};
const Segment SGI97144[] = {{&S67854[31], 3}, {NULL}};
const Segment SGI97145[] = {{&S67854[84], 6}, {NULL}};
const Segment SGI97146[] = {{&S89823[44], 9}, {NULL}};
const Segment SGI97147[] = {{&S89823[44], 21}, {NULL}};
const Segment SGI97148[] = {{&S89823[30], 6}, {NULL}};
const Segment SGI97149[] = {{&S89823[113], 3}, {NULL}};
const Segment SGI97150[] = {{&S89823[79], 5}, {NULL}};
const Segment SGI97151[] = {{&S89841[44], 14}, {NULL}};
const Segment SGI97152[] = {{&S89841[44], 26}, {NULL}};
const Segment SGI97153[] = {{&S89841[30], 6}, {NULL}};
const Segment SGI97154[] = {{&S89841[119], 3}, {NULL}};
const Segment SGI97155[] = {{&S89841[84], 6}, {NULL}};
const Segment SGI97156[] = {{&S89841[139], 8}, {NULL}};
const Segment SGI97157[] = {{&S89841[148], 13}, {NULL}};
const Segment SGI97158[] = {{&S67946[34], 14}, {NULL}};
const Segment SGI97159[] = {{&S67946[27], 6}, {NULL}};
const Segment SGI97160[] = {{&S67946[101], 9}, {NULL}};
const Segment SGI97161[] = {{&S67946[74], 6}, {NULL}};
const Segment SGI97162[] = {{&S67951[34], 14}, {NULL}};
const Segment SGI97163[] = {{&S67951[27], 6}, {NULL}};
const Segment SGI97164[] = {{&S67951[98], 9}, {NULL}};
const Segment SGI97165[] = {{&S67951[71], 6}, {NULL}};
const Segment SGI97166[] = {{&S67970[44], 5}, {NULL}};
const Segment SGI97167[] = {{&S67970[44], 17}, {NULL}};
const Segment SGI97168[] = {{&S67970[30], 6}, {NULL}};
const Segment SGI97169[] = {{&S67970[31], 3}, {NULL}};
const Segment SGI97170[] = {{&S67970[75], 6}, {NULL}};
const Segment SGI97171[] = {{&S89906[59], 10}, {NULL}};
const Segment SGI97172[] = {{&S89906[59], 27}, {NULL}};
const Segment SGI97173[] = {{&S89906[45], 6}, {NULL}};
const Segment SGI97174[] = {{&S89906[134], 3}, {NULL}};
const Segment SGI97175[] = {{&S89906[100], 5}, {NULL}};
const Segment SGI97176[] = {{&S89906[0], 14}, {NULL}};
const Segment SGI97177[] = {{&S89931[45], 11}, {NULL}};
const Segment SGI97178[] = {{&S89931[30], 6}, {NULL}};
const Segment SGI97179[] = {{&S89931[121], 3}, {NULL}};
const Segment SGI97180[] = {{&S89931[87], 5}, {NULL}};
const Segment SGI97181[] = {{&S89931[145], 13}, {NULL}};
const Segment SGI97182[] = {{&S89972[44], 5}, {NULL}};
const Segment SGI97183[] = {{&S89972[44], 17}, {NULL}};
const Segment SGI97184[] = {{&S89972[30], 6}, {NULL}};
const Segment SGI97185[] = {{&S89972[75], 6}, {NULL}};
const Segment SGI97186[] = {{&S89972[102], 13}, {NULL}};
const Segment SGI97187[] = {{&S89978[44], 6}, {NULL}};
const Segment SGI97188[] = {{&S89978[44], 18}, {NULL}};
const Segment SGI97189[] = {{&S89978[30], 6}, {NULL}};
const Segment SGI97190[] = {{&S89978[110], 3}, {NULL}};
const Segment SGI97191[] = {{&S89978[76], 5}, {NULL}};
const Segment SGI97192[] = {{&S68063[34], 6}, {NULL}};
const Segment SGI97193[] = {{&S68063[27], 6}, {NULL}};
const Segment SGI97194[] = {{&S68063[93], 9}, {NULL}};
const Segment SGI97195[] = {{&S68063[66], 6}, {NULL}};
const Segment SGI97196[] = {{&S68089[34], 10}, {NULL}};
const Segment SGI97197[] = {{&S68089[27], 6}, {NULL}};
const Segment SGI97198[] = {{&S68089[97], 9}, {NULL}};
const Segment SGI97199[] = {{&S68089[70], 6}, {NULL}};
const Segment SGI97200[] = {{&S68090[34], 10}, {NULL}};
const Segment SGI97201[] = {{&S68090[27], 6}, {NULL}};
const Segment SGI97202[] = {{&S68090[97], 9}, {NULL}};
const Segment SGI97203[] = {{&S68090[70], 6}, {NULL}};
const Segment SGI97204[] = {{&S68092[34], 10}, {NULL}};
const Segment SGI97205[] = {{&S68092[27], 6}, {NULL}};
const Segment SGI97206[] = {{&S68092[97], 9}, {NULL}};
const Segment SGI97207[] = {{&S68092[70], 6}, {NULL}};
const Segment SGI97208[] = {{&S68094[34], 9}, {NULL}};
const Segment SGI97209[] = {{&S68094[27], 6}, {NULL}};
const Segment SGI97210[] = {{&S68094[96], 9}, {NULL}};
const Segment SGI97211[] = {{&S68094[69], 6}, {NULL}};
const Segment SGI97212[] = {{&S68115[44], 8}, {NULL}};
const Segment SGI97213[] = {{&S68115[44], 20}, {NULL}};
const Segment SGI97214[] = {{&S68115[30], 6}, {NULL}};
const Segment SGI97215[] = {{&S68115[31], 3}, {NULL}};
const Segment SGI97216[] = {{&S68115[78], 6}, {NULL}};
const Segment SGI97217[] = {{&S90051[31], 22}, {NULL}};
const Segment SGI97218[] = {{&S90051[68], 3}, {NULL}};
const Segment SGI97219[] = {{&S90073[44], 10}, {NULL}};
const Segment SGI97220[] = {{&S90073[44], 27}, {NULL}};
const Segment SGI97221[] = {{&S90073[30], 6}, {NULL}};
const Segment SGI97222[] = {{&S90073[119], 3}, {NULL}};
const Segment SGI97223[] = {{&S90073[85], 5}, {NULL}};
const Segment SGI97224[] = {{&S90073[143], 11}, {NULL}};
const Segment SGI97225[] = {{&S90075[44], 10}, {NULL}};
const Segment SGI97226[] = {{&S90075[44], 27}, {NULL}};
const Segment SGI97227[] = {{&S90075[30], 6}, {NULL}};
const Segment SGI97228[] = {{&S90075[119], 3}, {NULL}};
const Segment SGI97229[] = {{&S90075[85], 5}, {NULL}};
const Segment SGI97230[] = {{&S90075[143], 11}, {NULL}};
const Segment SGI97231[] = {{&S68154[34], 9}, {NULL}};
const Segment SGI97232[] = {{&S68154[27], 6}, {NULL}};
const Segment SGI97233[] = {{&S68154[96], 9}, {NULL}};
const Segment SGI97234[] = {{&S68154[69], 6}, {NULL}};
const Segment SGI97235[] = {{&S68183[34], 12}, {NULL}};
const Segment SGI97236[] = {{&S68183[27], 6}, {NULL}};
const Segment SGI97237[] = {{&S68183[99], 9}, {NULL}};
const Segment SGI97238[] = {{&S68183[72], 6}, {NULL}};
const Segment SGI97239[] = {{&S68184[34], 12}, {NULL}};
const Segment SGI97240[] = {{&S68184[27], 6}, {NULL}};
const Segment SGI97241[] = {{&S68184[99], 9}, {NULL}};
const Segment SGI97242[] = {{&S68184[72], 6}, {NULL}};
const Segment SGI97243[] = {{&S90109[44], 10}, {NULL}};
const Segment SGI97244[] = {{&S90109[44], 27}, {NULL}};
const Segment SGI97245[] = {{&S90109[30], 6}, {NULL}};
const Segment SGI97246[] = {{&S90109[118], 3}, {NULL}};
const Segment SGI97247[] = {{&S90109[85], 5}, {NULL}};
const Segment SGI97248[] = {{&S90109[122], 14}, {NULL}};
const Segment SGI97249[] = {{&S90128[44], 5}, {NULL}};
const Segment SGI97250[] = {{&S90128[44], 27}, {NULL}};
const Segment SGI97251[] = {{&S90128[30], 6}, {NULL}};
const Segment SGI97252[] = {{&S90128[85], 6}, {NULL}};
const Segment SGI97253[] = {{&S90128[112], 13}, {NULL}};
const Segment SGI97254[] = {{&S68224[44], 11}, {NULL}};
const Segment SGI97255[] = {{&S68224[44], 23}, {NULL}};
const Segment SGI97256[] = {{&S68224[30], 6}, {NULL}};
const Segment SGI97257[] = {{&S68224[31], 3}, {NULL}};
const Segment SGI97258[] = {{&S68224[81], 6}, {NULL}};
const Segment SGI97259[] = {{&S68225[44], 11}, {NULL}};
const Segment SGI97260[] = {{&S68225[44], 23}, {NULL}};
const Segment SGI97261[] = {{&S68225[30], 6}, {NULL}};
const Segment SGI97262[] = {{&S68225[31], 3}, {NULL}};
const Segment SGI97263[] = {{&S68225[81], 6}, {NULL}};
const Segment SGI97264[] = {{&S68226[44], 11}, {NULL}};
const Segment SGI97265[] = {{&S68226[44], 23}, {NULL}};
const Segment SGI97266[] = {{&S68226[30], 6}, {NULL}};
const Segment SGI97267[] = {{&S68226[31], 3}, {NULL}};
const Segment SGI97268[] = {{&S68226[81], 6}, {NULL}};
const Segment SGI97269[] = {{&S90195[56], 10}, {NULL}};
const Segment SGI97270[] = {{&S90195[56], 27}, {NULL}};
const Segment SGI97271[] = {{&S90195[42], 6}, {NULL}};
const Segment SGI97272[] = {{&S68270[44], 82}, {NULL}};
const Segment SGI97273[] = {{&S68270[44], 103}, {NULL}};
const Segment SGI97274[] = {{&S68270[30], 6}, {NULL}};
const Segment SGI97275[] = {{&S90195[131], 3}, {NULL}};
const Segment SGI97276[] = {{&S90195[97], 5}, {NULL}};
const Segment SGI97277[] = {{&S90195[0], 12}, {NULL}};
const Segment SGI97278[] = {{&S68270[31], 3}, {NULL}};
const Segment SGI97279[] = {{&S68270[120], 5}, {NULL}};
const Segment SGI97280[] = {{&S68271[44], 82}, {NULL}};
const Segment SGI97281[] = {{&S68271[44], 103}, {NULL}};
const Segment SGI97282[] = {{&S68271[30], 6}, {NULL}};
const Segment SGI97283[] = {{&S68271[31], 3}, {NULL}};
const Segment SGI97284[] = {{&S68271[120], 5}, {NULL}};
const Segment SGI97285[] = {{&S68272[44], 85}, {NULL}};
const Segment SGI97286[] = {{&S68272[44], 106}, {NULL}};
const Segment SGI97287[] = {{&S68272[30], 6}, {NULL}};
const Segment SGI97288[] = {{&S68272[31], 3}, {NULL}};
const Segment SGI97289[] = {{&S68272[123], 5}, {NULL}};
const Segment SGI97290[] = {{&S68291[44], 14}, {NULL}};
const Segment SGI97291[] = {{&S68291[44], 26}, {NULL}};
const Segment SGI97292[] = {{&S68291[30], 6}, {NULL}};
const Segment SGI97293[] = {{&S68291[31], 3}, {NULL}};
const Segment SGI97294[] = {{&S68291[84], 6}, {NULL}};
const Segment SGI97295[] = {{&S68292[44], 5}, {NULL}};
const Segment SGI97296[] = {{&S68292[44], 17}, {NULL}};
const Segment SGI97297[] = {{&S68292[30], 6}, {NULL}};
const Segment SGI97298[] = {{&S68292[31], 3}, {NULL}};
const Segment SGI97299[] = {{&S68292[75], 6}, {NULL}};
const Segment SGI97300[] = {{&S68293[38], 5}, {NULL}};
const Segment SGI97301[] = {{&S68293[31], 6}, {NULL}};
const Segment SGI97302[] = {{&S90242[59], 10}, {NULL}};
const Segment SGI97303[] = {{&S90242[59], 27}, {NULL}};
const Segment SGI97304[] = {{&S90242[45], 6}, {NULL}};
const Segment SGI97305[] = {{&S90242[134], 3}, {NULL}};
const Segment SGI97306[] = {{&S90242[100], 5}, {NULL}};
const Segment SGI97307[] = {{&S90242[0], 14}, {NULL}};
const Segment SGI97308[] = {{&S82985[31], 5}, {NULL}};
const Segment SGI97309[] = {{&S82985[51], 3}, {NULL}};
const Segment SGI97310[] = {{&S83044[44], 7}, {NULL}};
const Segment SGI97311[] = {{&S83044[44], 19}, {NULL}};
const Segment SGI97312[] = {{&S83044[30], 6}, {NULL}};
const Segment SGI97313[] = {{&S83044[126], 3}, {NULL}};
const Segment SGI97314[] = {{&S83044[92], 5}, {NULL}};
const Segment SGI97315[] = {{&S83044[35], 1}, {NULL}};
const Segment SGI97316[] = {{&S83044[65], 11}, {NULL}};
const Segment SGI97317[] = {{&S90334[52], 10}, {NULL}};
const Segment SGI97318[] = {{&S90334[52], 27}, {NULL}};
const Segment SGI97319[] = {{&S90334[38], 6}, {NULL}};
const Segment SGI97320[] = {{&S90334[93], 5}, {NULL}};
const Segment SGI97321[] = {{&S90334[0], 14}, {NULL}};
const Segment SGI97322[] = {{&S90335[52], 11}, {NULL}};
const Segment SGI97323[] = {{&S90335[52], 28}, {NULL}};
const Segment SGI97324[] = {{&S90335[38], 6}, {NULL}};
const Segment SGI97325[] = {{&S90335[94], 5}, {NULL}};
const Segment SGI97326[] = {{&S90335[0], 14}, {NULL}};
const Segment SGI97327[] = {{&S83058[44], 6}, {NULL}};
const Segment SGI97328[] = {{&S83058[44], 23}, {NULL}};
const Segment SGI97329[] = {{&S83058[30], 6}, {NULL}};
const Segment SGI97330[] = {{&S83058[128], 3}, {NULL}};
const Segment SGI97331[] = {{&S83058[94], 5}, {NULL}};
const Segment SGI97332[] = {{&S90343[44], 5}, {NULL}};
const Segment SGI97333[] = {{&S90343[44], 26}, {NULL}};
const Segment SGI97334[] = {{&S90343[30], 6}, {NULL}};
const Segment SGI97335[] = {{&S90343[119], 3}, {NULL}};
const Segment SGI97336[] = {{&S90343[84], 6}, {NULL}};
const Segment SGI97337[] = {{&S90343[139], 8}, {NULL}};
const Segment SGI97338[] = {{&S90343[148], 13}, {NULL}};
const Segment SGI97339[] = {{&S90394[57], 11}, {NULL}};
const Segment SGI97340[] = {{&S90394[57], 28}, {NULL}};
const Segment SGI97341[] = {{&S90394[43], 6}, {NULL}};
const Segment SGI97342[] = {{&S90394[133], 3}, {NULL}};
const Segment SGI97343[] = {{&S90394[99], 5}, {NULL}};
const Segment SGI97344[] = {{&S90394[0], 12}, {NULL}};
const Segment SGI97345[] = {{&S90408[59], 14}, {NULL}};
const Segment SGI97346[] = {{&S90408[59], 26}, {NULL}};
const Segment SGI97347[] = {{&S90408[45], 6}, {NULL}};
const Segment SGI97348[] = {{&S90408[133], 3}, {NULL}};
const Segment SGI97349[] = {{&S90408[99], 5}, {NULL}};
const Segment SGI97350[] = {{&S90408[0], 14}, {NULL}};
const Segment SGI97351[] = {{&S82196[44], 9}, {NULL}};
const Segment SGI97352[] = {{&S82196[44], 20}, {NULL}};
const Segment SGI97353[] = {{&S82196[30], 6}, {NULL}};
const Segment SGI97354[] = {{&S82196[113], 3}, {NULL}};
const Segment SGI97355[] = {{&S82196[78], 6}, {NULL}};
const Segment SGI97356[] = {{&S90422[59], 13}, {NULL}};
const Segment SGI97357[] = {{&S90422[59], 25}, {NULL}};
const Segment SGI97358[] = {{&S90422[45], 6}, {NULL}};
const Segment SGI97359[] = {{&S90422[132], 3}, {NULL}};
const Segment SGI97360[] = {{&S90422[98], 5}, {NULL}};
const Segment SGI97361[] = {{&S90422[0], 14}, {NULL}};
const Segment SGI97362[] = {{&S76760[74], 6}, {NULL}};
const Segment SGI97363[] = {{&S76760[0], 9}, {NULL}};
const Segment SGI97364[] = {{&S76773[44], 12}, {NULL}};
const Segment SGI97365[] = {{&S76773[44], 24}, {NULL}};
const Segment SGI97366[] = {{&S76773[30], 6}, {NULL}};
const Segment SGI97367[] = {{&S76773[31], 3}, {NULL}};
const Segment SGI97368[] = {{&S76773[82], 6}, {NULL}};
const Segment SGI97369[] = {{&S76773[0], 13}, {NULL}};
const Segment SGI97370[] = {{&S90454[61], 10}, {NULL}};
const Segment SGI97371[] = {{&S90454[61], 27}, {NULL}};
const Segment SGI97372[] = {{&S90454[47], 6}, {NULL}};
const Segment SGI97373[] = {{&S90454[136], 3}, {NULL}};
const Segment SGI97374[] = {{&S90454[102], 5}, {NULL}};
const Segment SGI97375[] = {{&S90454[0], 14}, {NULL}};
const Segment SGI97376[] = {{&S90483[34], 11}, {NULL}};
const Segment SGI97377[] = {{&S90483[27], 6}, {NULL}};
const Segment SGI97378[] = {{&S90483[110], 2}, {NULL}};
const Segment SGI97379[] = {{&S90483[76], 6}, {NULL}};
const Segment SGI97380[] = {{&S90483[144], 8}, {NULL}};
const Segment SGI97381[] = {{&S90494[44], 10}, {NULL}};
const Segment SGI97382[] = {{&S90494[44], 27}, {NULL}};
const Segment SGI97383[] = {{&S90494[30], 6}, {NULL}};
const Segment SGI97384[] = {{&S90494[119], 3}, {NULL}};
const Segment SGI97385[] = {{&S90494[85], 5}, {NULL}};
const Segment SGI97386[] = {{&S90494[143], 7}, {NULL}};
const Segment SGI97387[] = {{&S90520[44], 9}, {NULL}};
const Segment SGI97388[] = {{&S90520[44], 21}, {NULL}};
const Segment SGI97389[] = {{&S90520[30], 6}, {NULL}};
const Segment SGI97390[] = {{&S90520[79], 6}, {NULL}};
const Segment SGI97391[] = {{&S90520[106], 13}, {NULL}};
const Segment SGI97392[] = {{&S76929[74], 6}, {NULL}};
const Segment SGI97393[] = {{&S76929[0], 9}, {NULL}};
const Segment SGI97394[] = {{&S90604[45], 5}, {NULL}};
const Segment SGI97395[] = {{&S90604[30], 6}, {NULL}};
const Segment SGI97396[] = {{&S90604[110], 3}, {NULL}};
const Segment SGI97397[] = {{&S90604[76], 5}, {NULL}};
const Segment SGI97398[] = {{&S90604[134], 13}, {NULL}};
const Segment SGI97399[] = {{&S90605[44], 5}, {NULL}};
const Segment SGI97400[] = {{&S90605[44], 17}, {NULL}};
const Segment SGI97401[] = {{&S90605[30], 6}, {NULL}};
const Segment SGI97402[] = {{&S90605[110], 3}, {NULL}};
const Segment SGI97403[] = {{&S90605[75], 6}, {NULL}};
const Segment SGI97404[] = {{&S90605[130], 8}, {NULL}};
const Segment SGI97405[] = {{&S90605[139], 13}, {NULL}};
const Segment SGI97406[] = {{&S90608[45], 14}, {NULL}};
const Segment SGI97407[] = {{&S90608[30], 6}, {NULL}};
const Segment SGI97408[] = {{&S90608[119], 3}, {NULL}};
const Segment SGI97409[] = {{&S90608[85], 5}, {NULL}};
const Segment SGI97410[] = {{&S90608[143], 13}, {NULL}};
const Segment SGI97411[] = {{&S90609[44], 14}, {NULL}};
const Segment SGI97412[] = {{&S90609[44], 26}, {NULL}};
const Segment SGI97413[] = {{&S90609[30], 6}, {NULL}};
const Segment SGI97414[] = {{&S90609[119], 3}, {NULL}};
const Segment SGI97415[] = {{&S90609[84], 6}, {NULL}};
const Segment SGI97416[] = {{&S90609[139], 8}, {NULL}};
const Segment SGI97417[] = {{&S90609[148], 13}, {NULL}};
const Segment SGI97418[] = {{&S68515[34], 7}, {NULL}};
const Segment SGI97419[] = {{&S68515[27], 6}, {NULL}};
const Segment SGI97420[] = {{&S68515[99], 9}, {NULL}};
const Segment SGI97421[] = {{&S68515[72], 6}, {NULL}};
const Segment SGI97422[] = {{&S90626[44], 10}, {NULL}};
const Segment SGI97423[] = {{&S90626[44], 27}, {NULL}};
const Segment SGI97424[] = {{&S90626[30], 6}, {NULL}};
const Segment SGI97425[] = {{&S90626[120], 3}, {NULL}};
const Segment SGI97426[] = {{&S90626[85], 6}, {NULL}};
const Segment SGI97427[] = {{&S90626[140], 8}, {NULL}};
const Segment SGI97428[] = {{&S90626[149], 13}, {NULL}};
const Segment SGI97429[] = {{&S68531[34], 6}, {NULL}};
const Segment SGI97430[] = {{&S68531[27], 6}, {NULL}};
const Segment SGI97431[] = {{&S90627[45], 10}, {NULL}};
const Segment SGI97432[] = {{&S90627[30], 6}, {NULL}};
const Segment SGI97433[] = {{&S68531[92], 9}, {NULL}};
const Segment SGI97434[] = {{&S68531[65], 6}, {NULL}};
const Segment SGI97435[] = {{&S90627[120], 3}, {NULL}};
const Segment SGI97436[] = {{&S90627[86], 5}, {NULL}};
const Segment SGI97437[] = {{&S90627[144], 13}, {NULL}};
const Segment SGI97438[] = {{&S90629[44], 10}, {NULL}};
const Segment SGI97439[] = {{&S90629[44], 27}, {NULL}};
const Segment SGI97440[] = {{&S90629[30], 6}, {NULL}};
const Segment SGI97441[] = {{&S90629[119], 3}, {NULL}};
const Segment SGI97442[] = {{&S90629[85], 5}, {NULL}};
const Segment SGI97443[] = {{&S90629[143], 11}, {NULL}};
const Segment SGI97444[] = {{&S90646[44], 5}, {NULL}};
const Segment SGI97445[] = {{&S90646[44], 23}, {NULL}};
const Segment SGI97446[] = {{&S90646[30], 6}, {NULL}};
const Segment SGI97447[] = {{&S90646[115], 3}, {NULL}};
const Segment SGI97448[] = {{&S90646[81], 5}, {NULL}};
const Segment SGI97449[] = {{&S90647[38], 5}, {NULL}};
const Segment SGI97450[] = {{&S90647[31], 6}, {NULL}};
const Segment SGI97451[] = {{&S90649[44], 8}, {NULL}};
const Segment SGI97452[] = {{&S90649[44], 20}, {NULL}};
const Segment SGI97453[] = {{&S90649[30], 6}, {NULL}};
const Segment SGI97454[] = {{&S90649[112], 3}, {NULL}};
const Segment SGI97455[] = {{&S90649[78], 5}, {NULL}};
const Segment SGI97456[] = {{&S90649[136], 7}, {NULL}};
const Segment SGI97457[] = {{&S82495[32], 10}, {NULL}};
const Segment SGI97458[] = {{&S82495[27], 4}, {NULL}};
const Segment SGI97459[] = {{&S82495[101], 2}, {NULL}};
const Segment SGI97460[] = {{&S82495[67], 6}, {NULL}};
const Segment SGI97461[] = {{&S82495[128], 8}, {NULL}};
const Segment SGI97462[] = {{&S68605[34], 13}, {NULL}};
const Segment SGI97463[] = {{&S68605[27], 6}, {NULL}};
const Segment SGI97464[] = {{&S68605[97], 9}, {NULL}};
const Segment SGI97465[] = {{&S68605[70], 6}, {NULL}};
const Segment SGI97466[] = {{&S90733[44], 10}, {NULL}};
const Segment SGI97467[] = {{&S90733[44], 27}, {NULL}};
const Segment SGI97468[] = {{&S90733[30], 6}, {NULL}};
const Segment SGI97469[] = {{&S90733[85], 6}, {NULL}};
const Segment SGI97470[] = {{&S90733[112], 13}, {NULL}};
const Segment SGI97471[] = {{&S82542[44], 8}, {NULL}};
const Segment SGI97472[] = {{&S82542[44], 25}, {NULL}};
const Segment SGI97473[] = {{&S82542[30], 6}, {NULL}};
const Segment SGI97474[] = {{&S82542[117], 3}, {NULL}};
const Segment SGI97475[] = {{&S82542[83], 5}, {NULL}};
const Segment SGI97476[] = {{&S90788[45], 10}, {NULL}};
const Segment SGI97477[] = {{&S90788[30], 6}, {NULL}};
const Segment SGI97478[] = {{&S90788[120], 3}, {NULL}};
const Segment SGI97479[] = {{&S90788[86], 5}, {NULL}};
const Segment SGI97480[] = {{&S90788[144], 13}, {NULL}};
const Segment SGI97481[] = {{&S90799[59], 10}, {NULL}};
const Segment SGI97482[] = {{&S90799[59], 27}, {NULL}};
const Segment SGI97483[] = {{&S90799[45], 6}, {NULL}};
const Segment SGI97484[] = {{&S90799[134], 3}, {NULL}};
const Segment SGI97485[] = {{&S90799[100], 5}, {NULL}};
const Segment SGI97486[] = {{&S90799[0], 14}, {NULL}};
const Segment SGI97487[] = {{&S90800[45], 4}, {NULL}};
const Segment SGI97488[] = {{&S90800[30], 6}, {NULL}};
const Segment SGI97489[] = {{&S90800[109], 3}, {NULL}};
const Segment SGI97490[] = {{&S90800[75], 5}, {NULL}};
const Segment SGI97491[] = {{&S90800[133], 13}, {NULL}};
const Segment SGI97492[] = {{&S90837[59], 10}, {NULL}};
const Segment SGI97493[] = {{&S90837[59], 27}, {NULL}};
const Segment SGI97494[] = {{&S90837[45], 6}, {NULL}};
const Segment SGI97495[] = {{&S90837[134], 3}, {NULL}};
const Segment SGI97496[] = {{&S90837[100], 5}, {NULL}};
const Segment SGI97497[] = {{&S90837[0], 14}, {NULL}};
const Segment SGI97498[] = {{&S68630[34], 14}, {NULL}};
const Segment SGI97499[] = {{&S68630[27], 6}, {NULL}};
const Segment SGI97500[] = {{&S68630[98], 9}, {NULL}};
const Segment SGI97501[] = {{&S68630[71], 6}, {NULL}};
const Segment SGI97502[] = {{&S90849[44], 10}, {NULL}};
const Segment SGI97503[] = {{&S90849[44], 27}, {NULL}};
const Segment SGI97504[] = {{&S90849[30], 6}, {NULL}};
const Segment SGI97505[] = {{&S90849[119], 3}, {NULL}};
const Segment SGI97506[] = {{&S90849[85], 5}, {NULL}};
const Segment SGI97507[] = {{&S90849[143], 11}, {NULL}};
const Segment SGI97508[] = {{&S68685[34], 10}, {NULL}};
const Segment SGI97509[] = {{&S68685[27], 6}, {NULL}};
const Segment SGI97510[] = {{&S68685[97], 9}, {NULL}};
const Segment SGI97511[] = {{&S68685[70], 6}, {NULL}};
const Segment SGI97512[] = {{&S90902[44], 11}, {NULL}};
const Segment SGI97513[] = {{&S90902[44], 23}, {NULL}};
const Segment SGI97514[] = {{&S90902[30], 6}, {NULL}};
const Segment SGI97515[] = {{&S90902[115], 3}, {NULL}};
const Segment SGI97516[] = {{&S90902[81], 5}, {NULL}};
const Segment SGI97517[] = {{&S68692[44], 7}, {NULL}};
const Segment SGI97518[] = {{&S68692[44], 19}, {NULL}};
const Segment SGI97519[] = {{&S68692[30], 6}, {NULL}};
const Segment SGI97520[] = {{&S68692[31], 3}, {NULL}};
const Segment SGI97521[] = {{&S68692[77], 6}, {NULL}};
const Segment SGI97522[] = {{&S90916[44], 5}, {NULL}};
const Segment SGI97523[] = {{&S90916[44], 17}, {NULL}};
const Segment SGI97524[] = {{&S90916[30], 6}, {NULL}};
const Segment SGI97525[] = {{&S90916[75], 6}, {NULL}};
const Segment SGI97526[] = {{&S90916[102], 13}, {NULL}};
const Segment SGI97527[] = {{&S68728[44], 10}, {NULL}};
const Segment SGI97528[] = {{&S68728[44], 22}, {NULL}};
const Segment SGI97529[] = {{&S68728[30], 6}, {NULL}};
const Segment SGI97530[] = {{&S68728[31], 3}, {NULL}};
const Segment SGI97531[] = {{&S68728[80], 6}, {NULL}};
const Segment SGI97532[] = {{&S68729[38], 10}, {NULL}};
const Segment SGI97533[] = {{&S68729[31], 6}, {NULL}};
const Segment SGI97534[] = {{&S68745[34], 15}, {NULL}};
const Segment SGI97535[] = {{&S68745[27], 6}, {NULL}};
const Segment SGI97536[] = {{&S68745[102], 9}, {NULL}};
const Segment SGI97537[] = {{&S68745[75], 6}, {NULL}};
const Segment SGI97538[] = {{&S90983[44], 23}, {NULL}};
const Segment SGI97539[] = {{&S90983[44], 50}, {NULL}};
const Segment SGI97540[] = {{&S90983[30], 6}, {NULL}};
const Segment SGI97541[] = {{&S90983[142], 3}, {NULL}};
const Segment SGI97542[] = {{&S90983[108], 5}, {NULL}};
const Segment SGI97543[] = {{&S91007[44], 9}, {NULL}};
const Segment SGI97544[] = {{&S91007[44], 21}, {NULL}};
const Segment SGI97545[] = {{&S91007[30], 6}, {NULL}};
const Segment SGI97546[] = {{&S91007[113], 3}, {NULL}};
const Segment SGI97547[] = {{&S91007[79], 5}, {NULL}};
const Segment SGI97548[] = {{&S91057[44], 10}, {NULL}};
const Segment SGI97549[] = {{&S68838[44], 6}, {NULL}};
const Segment SGI97550[] = {{&S91057[44], 27}, {NULL}};
const Segment SGI97551[] = {{&S68838[44], 18}, {NULL}};
const Segment SGI97552[] = {{&S91057[30], 6}, {NULL}};
const Segment SGI97553[] = {{&S68838[30], 6}, {NULL}};
const Segment SGI97554[] = {{&S91057[119], 3}, {NULL}};
const Segment SGI97555[] = {{&S68838[31], 3}, {NULL}};
const Segment SGI97556[] = {{&S91057[85], 5}, {NULL}};
const Segment SGI97557[] = {{&S91057[143], 7}, {NULL}};
const Segment SGI97558[] = {{&S68838[76], 6}, {NULL}};
const Segment SGI97559[] = {{&S68838[146], 3}, {NULL}};
const Segment SGI97560[] = {{&S68839[44], 6}, {NULL}};
const Segment SGI97561[] = {{&S68839[44], 18}, {NULL}};
const Segment SGI97562[] = {{&S68839[30], 6}, {NULL}};
const Segment SGI97563[] = {{&S68839[31], 3}, {NULL}};
const Segment SGI97564[] = {{&S68839[76], 6}, {NULL}};
const Segment SGI97565[] = {{&S68839[139], 3}, {NULL}};
const Segment SGI97566[] = {{&S91079[44], 10}, {NULL}};
const Segment SGI97567[] = {{&S91079[44], 27}, {NULL}};
const Segment SGI97568[] = {{&S91079[30], 6}, {NULL}};
const Segment SGI97569[] = {{&S91079[120], 3}, {NULL}};
const Segment SGI97570[] = {{&S91079[85], 6}, {NULL}};
const Segment SGI97571[] = {{&S91079[140], 8}, {NULL}};
const Segment SGI97572[] = {{&S91079[149], 13}, {NULL}};
const Segment SGI97573[] = {{&S83818[44], 12}, {NULL}};
const Segment SGI97574[] = {{&S83818[44], 23}, {NULL}};
const Segment SGI97575[] = {{&S83818[30], 6}, {NULL}};
const Segment SGI97576[] = {{&S83818[84], 4}, {NULL}};
const Segment SGI97577[] = {{&S83818[84], 3}, {NULL}};
const Segment SGI97578[] = {{&S83819[44], 12}, {NULL}};
const Segment SGI97579[] = {{&S83819[44], 23}, {NULL}};
const Segment SGI97580[] = {{&S83819[30], 6}, {NULL}};
const Segment SGI97581[] = {{&S83819[84], 6}, {NULL}};
const Segment SGI97582[] = {{&S83820[44], 23}, {NULL}};
const Segment SGI97583[] = {{&S83820[30], 6}, {NULL}};
const Segment SGI97584[] = {{&S91087[44], 10}, {NULL}};
const Segment SGI97585[] = {{&S91087[44], 27}, {NULL}};
const Segment SGI97586[] = {{&S91087[30], 6}, {NULL}};
const Segment SGI97587[] = {{&S91087[120], 3}, {NULL}};
const Segment SGI97588[] = {{&S91087[85], 6}, {NULL}};
const Segment SGI97589[] = {{&S91087[140], 8}, {NULL}};
const Segment SGI97590[] = {{&S91087[149], 13}, {NULL}};
const Segment SGI97591[] = {{&S91088[44], 11}, {NULL}};
const Segment SGI97592[] = {{&S91088[44], 28}, {NULL}};
const Segment SGI97593[] = {{&S91088[30], 6}, {NULL}};
const Segment SGI97594[] = {{&S91088[121], 3}, {NULL}};
const Segment SGI97595[] = {{&S91088[86], 6}, {NULL}};
const Segment SGI97596[] = {{&S91088[141], 8}, {NULL}};
const Segment SGI97597[] = {{&S91088[150], 13}, {NULL}};
const Segment SGI97598[] = {{&S91089[44], 10}, {NULL}};
const Segment SGI97599[] = {{&S91089[44], 27}, {NULL}};
const Segment SGI97600[] = {{&S91089[30], 6}, {NULL}};
const Segment SGI97601[] = {{&S91089[85], 6}, {NULL}};
const Segment SGI97602[] = {{&S91089[112], 13}, {NULL}};
const Segment SGI97603[] = {{&S68883[44], 13}, {NULL}};
const Segment SGI97604[] = {{&S68883[44], 25}, {NULL}};
const Segment SGI97605[] = {{&S68883[30], 6}, {NULL}};
const Segment SGI97606[] = {{&S68883[31], 3}, {NULL}};
const Segment SGI97607[] = {{&S68883[83], 6}, {NULL}};
const Segment SGI97608[] = {{&S68884[44], 24}, {NULL}};
const Segment SGI97609[] = {{&S68884[44], 36}, {NULL}};
const Segment SGI97610[] = {{&S68884[30], 6}, {NULL}};
const Segment SGI97611[] = {{&S68884[31], 3}, {NULL}};
const Segment SGI97612[] = {{&S68884[94], 6}, {NULL}};
const Segment SGI97613[] = {{&S77345[44], 12}, {NULL}};
const Segment SGI97614[] = {{&S77345[44], 33}, {NULL}};
const Segment SGI97615[] = {{&S77345[30], 6}, {NULL}};
const Segment SGI97616[] = {{&S77345[31], 3}, {NULL}};
const Segment SGI97617[] = {{&S77345[91], 5}, {NULL}};
const Segment SGI97618[] = {{&S77345[0], 13}, {NULL}};
const Segment SGI97619[] = {{&S68920[34], 9}, {NULL}};
const Segment SGI97620[] = {{&S68920[27], 6}, {NULL}};
const Segment SGI97621[] = {{&S68920[96], 9}, {NULL}};
const Segment SGI97622[] = {{&S68920[69], 6}, {NULL}};
const Segment SGI97623[] = {{&S68925[42], 9}, {NULL}};
const Segment SGI97624[] = {{&S68925[30], 6}, {NULL}};
const Segment SGI97625[] = {{&S68925[31], 3}, {NULL}};
const Segment SGI97626[] = {{&S68925[77], 6}, {NULL}};
const Segment SGI97627[] = {{&S91148[58], 11}, {NULL}};
const Segment SGI97628[] = {{&S68926[44], 9}, {NULL}};
const Segment SGI97629[] = {{&S91148[58], 28}, {NULL}};
const Segment SGI97630[] = {{&S68926[44], 21}, {NULL}};
const Segment SGI97631[] = {{&S91148[44], 6}, {NULL}};
const Segment SGI97632[] = {{&S68926[30], 6}, {NULL}};
const Segment SGI97633[] = {{&S91148[134], 3}, {NULL}};
const Segment SGI97634[] = {{&S68926[31], 3}, {NULL}};
const Segment SGI97635[] = {{&S68926[79], 6}, {NULL}};
const Segment SGI97636[] = {{&S91148[100], 5}, {NULL}};
const Segment SGI97637[] = {{&S91148[0], 14}, {NULL}};
const Segment SGI97638[] = {{&S82845[42], 10}, {NULL}};
const Segment SGI97639[] = {{&S82845[42], 27}, {NULL}};
const Segment SGI97640[] = {{&S82845[30], 4}, {NULL}};
const Segment SGI97641[] = {{&S82845[117], 3}, {NULL}};
const Segment SGI97642[] = {{&S82845[83], 5}, {NULL}};
const Segment SGI97643[] = {{&S91158[31], 8}, {NULL}};
const Segment SGI97644[] = {{&S91158[54], 3}, {NULL}};
const Segment SGI97645[] = {{&S91161[59], 10}, {NULL}};
const Segment SGI97646[] = {{&S91161[59], 27}, {NULL}};
const Segment SGI97647[] = {{&S91161[45], 6}, {NULL}};
const Segment SGI97648[] = {{&S91161[134], 3}, {NULL}};
const Segment SGI97649[] = {{&S91161[100], 5}, {NULL}};
const Segment SGI97650[] = {{&S91161[0], 14}, {NULL}};
const Segment SGI97651[] = {{&S91170[59], 9}, {NULL}};
const Segment SGI97652[] = {{&S91170[59], 21}, {NULL}};
const Segment SGI97653[] = {{&S91170[45], 6}, {NULL}};
const Segment SGI97654[] = {{&S91170[128], 3}, {NULL}};
const Segment SGI97655[] = {{&S91170[94], 5}, {NULL}};
const Segment SGI97656[] = {{&S91170[0], 14}, {NULL}};
const Segment SGI97657[] = {{&S68951[34], 17}, {NULL}};
const Segment SGI97658[] = {{&S68951[27], 6}, {NULL}};
const Segment SGI97659[] = {{&S68951[120], 9}, {NULL}};
const Segment SGI97660[] = {{&S68951[93], 6}, {NULL}};
const Segment SGI97661[] = {{&S83925[44], 15}, {NULL}};
const Segment SGI97662[] = {{&S83925[44], 25}, {NULL}};
const Segment SGI97663[] = {{&S83925[30], 6}, {NULL}};
const Segment SGI97664[] = {{&S83925[136], 3}, {NULL}};
const Segment SGI97665[] = {{&S83925[102], 5}, {NULL}};
const Segment SGI97666[] = {{&S83925[35], 1}, {NULL}};
const Segment SGI97667[] = {{&S83925[71], 11}, {NULL}};
const Segment SGI97668[] = {{&S83930[44], 13}, {NULL}};
const Segment SGI97669[] = {{&S83930[44], 24}, {NULL}};
const Segment SGI97670[] = {{&S83930[30], 6}, {NULL}};
const Segment SGI97671[] = {{&S83930[85], 4}, {NULL}};
const Segment SGI97672[] = {{&S83930[85], 3}, {NULL}};
const Segment SGI97673[] = {{&S83931[44], 13}, {NULL}};
const Segment SGI97674[] = {{&S83931[44], 24}, {NULL}};
const Segment SGI97675[] = {{&S83931[30], 6}, {NULL}};
const Segment SGI97676[] = {{&S83931[85], 6}, {NULL}};
const Segment SGI97677[] = {{&S83932[44], 24}, {NULL}};
const Segment SGI97678[] = {{&S83932[30], 6}, {NULL}};
const Segment SGI97679[] = {{&S91198[44], 11}, {NULL}};
const Segment SGI97680[] = {{&S91198[44], 28}, {NULL}};
const Segment SGI97681[] = {{&S91198[30], 6}, {NULL}};
const Segment SGI97682[] = {{&S91198[164], 3}, {NULL}};
const Segment SGI97683[] = {{&S91198[130], 3}, {NULL}};
const Segment SGI97684[] = {{&S91198[95], 6}, {NULL}};
const Segment SGI97685[] = {{&S91198[150], 8}, {NULL}};
const Segment SGI97686[] = {{&S91212[44], 4}, {NULL}};
const Segment SGI97687[] = {{&S91212[44], 16}, {NULL}};
const Segment SGI97688[] = {{&S91212[30], 6}, {NULL}};
const Segment SGI97689[] = {{&S91212[74], 6}, {NULL}};
const Segment SGI97690[] = {{&S91212[101], 13}, {NULL}};
const Segment SGI97691[] = {{&S69010[44], 8}, {NULL}};
const Segment SGI97692[] = {{&S69010[44], 20}, {NULL}};
const Segment SGI97693[] = {{&S69010[30], 6}, {NULL}};
const Segment SGI97694[] = {{&S69010[31], 3}, {NULL}};
const Segment SGI97695[] = {{&S69010[78], 6}, {NULL}};
const Segment SGI97696[] = {{&S82935[32], 10}, {NULL}};
const Segment SGI97697[] = {{&S82935[27], 4}, {NULL}};
const Segment SGI97698[] = {{&S82935[102], 2}, {NULL}};
const Segment SGI97699[] = {{&S82935[68], 6}, {NULL}};
const Segment SGI97700[] = {{&S82935[129], 8}, {NULL}};
const Segment SGI97701[] = {{&S91239[44], 10}, {NULL}};
const Segment SGI97702[] = {{&S91239[44], 27}, {NULL}};
const Segment SGI97703[] = {{&S91239[30], 6}, {NULL}};
const Segment SGI97704[] = {{&S91239[119], 3}, {NULL}};
const Segment SGI97705[] = {{&S91239[85], 5}, {NULL}};
const Segment SGI97706[] = {{&S91239[143], 7}, {NULL}};
const Segment SGI97707[] = {{&S91309[45], 11}, {NULL}};
const Segment SGI97708[] = {{&S91309[30], 6}, {NULL}};
const Segment SGI97709[] = {{&S91309[121], 3}, {NULL}};
const Segment SGI97710[] = {{&S91309[87], 5}, {NULL}};
const Segment SGI97711[] = {{&S91309[145], 13}, {NULL}};
const Segment SGI97712[] = {{&S91310[44], 11}, {NULL}};
const Segment SGI97713[] = {{&S91310[44], 28}, {NULL}};
const Segment SGI97714[] = {{&S91310[30], 6}, {NULL}};
const Segment SGI97715[] = {{&S91310[121], 3}, {NULL}};
const Segment SGI97716[] = {{&S91310[86], 6}, {NULL}};
const Segment SGI97717[] = {{&S91310[141], 8}, {NULL}};
const Segment SGI97718[] = {{&S91310[150], 13}, {NULL}};
const Segment SGI97719[] = {{&S91312[44], 7}, {NULL}};
const Segment SGI97720[] = {{&S91312[44], 19}, {NULL}};
const Segment SGI97721[] = {{&S91312[30], 6}, {NULL}};
const Segment SGI97722[] = {{&S91312[111], 3}, {NULL}};
const Segment SGI97723[] = {{&S91312[77], 5}, {NULL}};
const Segment SGI97724[] = {{&S91329[45], 4}, {NULL}};
const Segment SGI97725[] = {{&S91329[30], 6}, {NULL}};
const Segment SGI97726[] = {{&S91329[109], 3}, {NULL}};
const Segment SGI97727[] = {{&S91329[75], 5}, {NULL}};
const Segment SGI97728[] = {{&S91329[133], 13}, {NULL}};
const Segment SGI97729[] = {{&S69136[6], 4}, {NULL}};
const Segment SGI97730[] = {{&S69136[82], 5}, {NULL}};
const Segment SGI97731[] = {{&S91356[34], 10}, {NULL}};
const Segment SGI97732[] = {{&S91356[27], 6}, {NULL}};
const Segment SGI97733[] = {{&S91356[109], 2}, {NULL}};
const Segment SGI97734[] = {{&S91356[75], 6}, {NULL}};
const Segment SGI97735[] = {{&S91356[144], 8}, {NULL}};
const Segment SGI97736[] = {{&S91357[34], 10}, {NULL}};
const Segment SGI97737[] = {{&S91357[27], 6}, {NULL}};
const Segment SGI97738[] = {{&S91357[109], 2}, {NULL}};
const Segment SGI97739[] = {{&S91357[75], 6}, {NULL}};
const Segment SGI97740[] = {{&S91357[143], 8}, {NULL}};
const Segment SGI97741[] = {{&S91358[45], 5}, {NULL}};
const Segment SGI97742[] = {{&S91358[30], 6}, {NULL}};
const Segment SGI97743[] = {{&S91358[110], 3}, {NULL}};
const Segment SGI97744[] = {{&S91358[76], 5}, {NULL}};
const Segment SGI97745[] = {{&S91358[134], 13}, {NULL}};
const Segment SGI97746[] = {{&S69166[34], 7}, {NULL}};
const Segment SGI97747[] = {{&S69166[27], 6}, {NULL}};
const Segment SGI97748[] = {{&S69166[94], 9}, {NULL}};
const Segment SGI97749[] = {{&S69166[67], 6}, {NULL}};
const Segment SGI97750[] = {{&S69167[34], 6}, {NULL}};
const Segment SGI97751[] = {{&S69167[27], 6}, {NULL}};
const Segment SGI97752[] = {{&S69167[93], 9}, {NULL}};
const Segment SGI97753[] = {{&S69167[66], 6}, {NULL}};
const Segment SGI97754[] = {{&S69169[44], 14}, {NULL}};
const Segment SGI97755[] = {{&S69169[44], 26}, {NULL}};
const Segment SGI97756[] = {{&S69169[30], 6}, {NULL}};
const Segment SGI97757[] = {{&S69169[31], 3}, {NULL}};
const Segment SGI97758[] = {{&S69169[84], 6}, {NULL}};
const Segment SGI97759[] = {{&S69173[34], 20}, {NULL}};
const Segment SGI97760[] = {{&S69173[27], 6}, {NULL}};
const Segment SGI97761[] = {{&S69173[107], 9}, {NULL}};
const Segment SGI97762[] = {{&S69173[80], 6}, {NULL}};
const Segment SGI97763[] = {{&S91397[44], 9}, {NULL}};
const Segment SGI97764[] = {{&S91397[44], 21}, {NULL}};
const Segment SGI97765[] = {{&S91397[30], 6}, {NULL}};
const Segment SGI97766[] = {{&S91397[113], 3}, {NULL}};
const Segment SGI97767[] = {{&S91397[79], 5}, {NULL}};
const Segment SGI97768[] = {{&S91400[44], 19}, {NULL}};
const Segment SGI97769[] = {{&S91400[44], 31}, {NULL}};
const Segment SGI97770[] = {{&S91400[30], 6}, {NULL}};
const Segment SGI97771[] = {{&S91400[89], 6}, {NULL}};
const Segment SGI97772[] = {{&S91400[116], 13}, {NULL}};
const Segment SGI97773[] = {{&S69244[44], 8}, {NULL}};
const Segment SGI97774[] = {{&S69244[44], 20}, {NULL}};
const Segment SGI97775[] = {{&S69244[30], 6}, {NULL}};
const Segment SGI97776[] = {{&S69244[31], 3}, {NULL}};
const Segment SGI97777[] = {{&S69244[78], 6}, {NULL}};
const Segment SGI97778[] = {{&S69251[34], 6}, {NULL}};
const Segment SGI97779[] = {{&S69251[27], 6}, {NULL}};
const Segment SGI97780[] = {{&S69251[93], 9}, {NULL}};
const Segment SGI97781[] = {{&S69251[66], 6}, {NULL}};
const Segment SGI97782[] = {{&S69265[34], 17}, {NULL}};
const Segment SGI97783[] = {{&S69265[27], 6}, {NULL}};
const Segment SGI97784[] = {{&S69265[120], 9}, {NULL}};
const Segment SGI97785[] = {{&S69265[93], 6}, {NULL}};
const Segment SGI97786[] = {{&S69281[34], 20}, {NULL}};
const Segment SGI97787[] = {{&S69281[27], 6}, {NULL}};
const Segment SGI97788[] = {{&S69281[107], 9}, {NULL}};
const Segment SGI97789[] = {{&S69281[80], 6}, {NULL}};
const Segment SGI97790[] = {{&S69297[44], 6}, {NULL}};
const Segment SGI97791[] = {{&S69297[44], 18}, {NULL}};
const Segment SGI97792[] = {{&S69297[30], 6}, {NULL}};
const Segment SGI97793[] = {{&S69297[31], 3}, {NULL}};
const Segment SGI97794[] = {{&S69297[76], 6}, {NULL}};
const Segment SGI97795[] = {{&S69298[44], 6}, {NULL}};
const Segment SGI97796[] = {{&S69298[44], 18}, {NULL}};
const Segment SGI97797[] = {{&S69298[30], 6}, {NULL}};
const Segment SGI97798[] = {{&S69298[79], 6}, {NULL}};
const Segment SGI97799[] = {{&S69343[44], 9}, {NULL}};
const Segment SGI97800[] = {{&S69343[44], 18}, {NULL}};
const Segment SGI97801[] = {{&S69343[30], 6}, {NULL}};
const Segment SGI97802[] = {{&S69343[31], 3}, {NULL}};
const Segment SGI97803[] = {{&S69343[76], 6}, {NULL}};
const Segment SGI97804[] = {{&S69345[44], 9}, {NULL}};
const Segment SGI97805[] = {{&S69345[44], 21}, {NULL}};
const Segment SGI97806[] = {{&S69345[30], 6}, {NULL}};
const Segment SGI97807[] = {{&S69345[31], 3}, {NULL}};
const Segment SGI97808[] = {{&S69345[79], 6}, {NULL}};
const Segment SGI97809[] = {{&S84175[44], 9}, {NULL}};
const Segment SGI97810[] = {{&S84175[44], 20}, {NULL}};
const Segment SGI97811[] = {{&S84175[30], 6}, {NULL}};
const Segment SGI97812[] = {{&S84175[131], 3}, {NULL}};
const Segment SGI97813[] = {{&S84175[97], 5}, {NULL}};
const Segment SGI97814[] = {{&S84175[35], 1}, {NULL}};
const Segment SGI97815[] = {{&S84175[66], 11}, {NULL}};
const Segment SGI97816[] = {{&S91555[44], 12}, {NULL}};
const Segment SGI97817[] = {{&S91555[44], 24}, {NULL}};
const Segment SGI97818[] = {{&S91555[30], 6}, {NULL}};
const Segment SGI97819[] = {{&S91555[116], 3}, {NULL}};
const Segment SGI97820[] = {{&S91555[82], 5}, {NULL}};
const Segment SGI97821[] = {{&S91580[45], 13}, {NULL}};
const Segment SGI97822[] = {{&S91580[30], 6}, {NULL}};
const Segment SGI97823[] = {{&S91580[118], 3}, {NULL}};
const Segment SGI97824[] = {{&S91580[84], 5}, {NULL}};
const Segment SGI97825[] = {{&S91580[142], 13}, {NULL}};
const Segment SGI97826[] = {{&S91598[44], 10}, {NULL}};
const Segment SGI97827[] = {{&S91598[44], 27}, {NULL}};
const Segment SGI97828[] = {{&S91598[30], 6}, {NULL}};
const Segment SGI97829[] = {{&S91598[118], 3}, {NULL}};
const Segment SGI97830[] = {{&S91598[85], 5}, {NULL}};
const Segment SGI97831[] = {{&S91598[122], 14}, {NULL}};
const Segment SGI97832[] = {{&S91611[44], 14}, {NULL}};
const Segment SGI97833[] = {{&S91611[44], 35}, {NULL}};
const Segment SGI97834[] = {{&S91611[30], 6}, {NULL}};
const Segment SGI97835[] = {{&S91611[127], 3}, {NULL}};
const Segment SGI97836[] = {{&S91611[93], 5}, {NULL}};
const Segment SGI97837[] = {{&S91611[151], 7}, {NULL}};
const Segment SGI97838[] = {{&S69442[34], 10}, {NULL}};
const Segment SGI97839[] = {{&S69442[27], 6}, {NULL}};
const Segment SGI97840[] = {{&S69442[97], 9}, {NULL}};
const Segment SGI97841[] = {{&S69442[70], 6}, {NULL}};
const Segment SGI97842[] = {{&S69445[44], 10}, {NULL}};
const Segment SGI97843[] = {{&S69445[44], 22}, {NULL}};
const Segment SGI97844[] = {{&S69445[30], 6}, {NULL}};
const Segment SGI97845[] = {{&S69445[31], 3}, {NULL}};
const Segment SGI97846[] = {{&S69445[80], 6}, {NULL}};
const Segment SGI97847[] = {{&S69446[44], 11}, {NULL}};
const Segment SGI97848[] = {{&S69446[44], 23}, {NULL}};
const Segment SGI97849[] = {{&S69446[30], 6}, {NULL}};
const Segment SGI97850[] = {{&S69446[31], 3}, {NULL}};
const Segment SGI97851[] = {{&S69446[81], 6}, {NULL}};
const Segment SGI97852[] = {{&S69455[34], 10}, {NULL}};
const Segment SGI97853[] = {{&S69455[27], 6}, {NULL}};
const Segment SGI97854[] = {{&S69455[97], 9}, {NULL}};
const Segment SGI97855[] = {{&S69455[70], 6}, {NULL}};
const Segment SGI97856[] = {{&S69456[34], 29}, {NULL}};
const Segment SGI97857[] = {{&S69456[27], 6}, {NULL}};
const Segment SGI97858[] = {{&S69456[116], 9}, {NULL}};
const Segment SGI97859[] = {{&S69456[89], 6}, {NULL}};
const Segment SGI97860[] = {{&S91649[44], 10}, {NULL}};
const Segment SGI97861[] = {{&S91649[44], 27}, {NULL}};
const Segment SGI97862[] = {{&S91649[30], 6}, {NULL}};
const Segment SGI97863[] = {{&S91649[120], 3}, {NULL}};
const Segment SGI97864[] = {{&S91649[85], 6}, {NULL}};
const Segment SGI97865[] = {{&S91649[140], 8}, {NULL}};
const Segment SGI97866[] = {{&S91649[149], 13}, {NULL}};
const Segment SGI97867[] = {{&S91650[44], 5}, {NULL}};
const Segment SGI97868[] = {{&S91650[44], 17}, {NULL}};
const Segment SGI97869[] = {{&S91650[30], 6}, {NULL}};
const Segment SGI97870[] = {{&S91650[110], 3}, {NULL}};
const Segment SGI97871[] = {{&S91650[75], 6}, {NULL}};
const Segment SGI97872[] = {{&S91650[130], 8}, {NULL}};
const Segment SGI97873[] = {{&S91650[139], 13}, {NULL}};
const Segment SGI97874[] = {{&S91654[44], 5}, {NULL}};
const Segment SGI97875[] = {{&S91654[44], 17}, {NULL}};
const Segment SGI97876[] = {{&S91654[30], 6}, {NULL}};
const Segment SGI97877[] = {{&S91654[75], 6}, {NULL}};
const Segment SGI97878[] = {{&S91654[102], 13}, {NULL}};
const Segment SGI97879[] = {{&S91656[44], 10}, {NULL}};
const Segment SGI97880[] = {{&S91656[44], 27}, {NULL}};
const Segment SGI97881[] = {{&S91656[30], 6}, {NULL}};
const Segment SGI97882[] = {{&S91656[85], 6}, {NULL}};
const Segment SGI97883[] = {{&S91656[112], 13}, {NULL}};
const Segment SGI97884[] = {{&S84314[44], 7}, {NULL}};
const Segment SGI97885[] = {{&S84314[44], 24}, {NULL}};
const Segment SGI97886[] = {{&S84314[30], 6}, {NULL}};
const Segment SGI97887[] = {{&S91658[44], 10}, {NULL}};
const Segment SGI97888[] = {{&S91658[44], 27}, {NULL}};
const Segment SGI97889[] = {{&S91658[30], 6}, {NULL}};
const Segment SGI97890[] = {{&S91658[120], 3}, {NULL}};
const Segment SGI97891[] = {{&S84314[82], 6}, {NULL}};
const Segment SGI97892[] = {{&S91658[85], 6}, {NULL}};
const Segment SGI97893[] = {{&S91658[140], 8}, {NULL}};
const Segment SGI97894[] = {{&S91658[149], 13}, {NULL}};
const Segment SGI97895[] = {{&S84314[109], 13}, {NULL}};
const Segment SGI97896[] = {{&S69516[44], 9}, {NULL}};
const Segment SGI97897[] = {{&S69516[44], 21}, {NULL}};
const Segment SGI97898[] = {{&S69516[30], 6}, {NULL}};
const Segment SGI97899[] = {{&S69516[31], 3}, {NULL}};
const Segment SGI97900[] = {{&S69516[79], 6}, {NULL}};
const Segment SGI97901[] = {{&S69528[34], 10}, {NULL}};
const Segment SGI97902[] = {{&S69528[27], 6}, {NULL}};
const Segment SGI97903[] = {{&S83459[42], 15}, {NULL}};
const Segment SGI97904[] = {{&S83459[42], 26}, {NULL}};
const Segment SGI97905[] = {{&S83459[30], 4}, {NULL}};
const Segment SGI97906[] = {{&S69528[97], 9}, {NULL}};
const Segment SGI97907[] = {{&S69528[70], 6}, {NULL}};
const Segment SGI97908[] = {{&S83459[117], 3}, {NULL}};
const Segment SGI97909[] = {{&S83459[82], 6}, {NULL}};
const Segment SGI97910[] = {{&S91719[44], 7}, {NULL}};
const Segment SGI97911[] = {{&S91719[44], 19}, {NULL}};
const Segment SGI97912[] = {{&S91719[30], 6}, {NULL}};
const Segment SGI97913[] = {{&S91719[111], 3}, {NULL}};
const Segment SGI97914[] = {{&S91719[77], 5}, {NULL}};
const Segment SGI97915[] = {{&S91734[59], 10}, {NULL}};
const Segment SGI97916[] = {{&S91734[59], 27}, {NULL}};
const Segment SGI97917[] = {{&S91734[45], 6}, {NULL}};
const Segment SGI97918[] = {{&S91734[134], 3}, {NULL}};
const Segment SGI97919[] = {{&S91734[100], 5}, {NULL}};
const Segment SGI97920[] = {{&S91734[0], 14}, {NULL}};
const Segment SGI97921[] = {{&S81140[42], 14}, {NULL}};
const Segment SGI97922[] = {{&S81140[42], 35}, {NULL}};
const Segment SGI97923[] = {{&S81140[30], 4}, {NULL}};
const Segment SGI97924[] = {{&S81140[31], 3}, {NULL}};
const Segment SGI97925[] = {{&S91773[61], 11}, {NULL}};
const Segment SGI97926[] = {{&S91773[61], 28}, {NULL}};
const Segment SGI97927[] = {{&S91773[47], 6}, {NULL}};
const Segment SGI97928[] = {{&S91773[137], 3}, {NULL}};
const Segment SGI97929[] = {{&S91773[103], 5}, {NULL}};
const Segment SGI97930[] = {{&S91773[0], 14}, {NULL}};
const Segment SGI97931[] = {{&S83525[43], 10}, {NULL}};
const Segment SGI97932[] = {{&S83525[30], 4}, {NULL}};
const Segment SGI97933[] = {{&S83525[113], 3}, {NULL}};
const Segment SGI97934[] = {{&S83525[79], 5}, {NULL}};
const Segment SGI97935[] = {{&S83525[137], 13}, {NULL}};
const Segment SGI97936[] = {{&S69629[34], 13}, {NULL}};
const Segment SGI97937[] = {{&S69629[27], 6}, {NULL}};
const Segment SGI97938[] = {{&S69629[97], 9}, {NULL}};
const Segment SGI97939[] = {{&S69629[70], 6}, {NULL}};
const Segment SGI97940[] = {{&S69639[44], 9}, {NULL}};
const Segment SGI97941[] = {{&S69639[44], 21}, {NULL}};
const Segment SGI97942[] = {{&S69639[30], 6}, {NULL}};
const Segment SGI97943[] = {{&S69639[31], 3}, {NULL}};
const Segment SGI97944[] = {{&S69639[79], 6}, {NULL}};
const Segment SGI97945[] = {{&S69640[44], 11}, {NULL}};
const Segment SGI97946[] = {{&S69640[44], 23}, {NULL}};
const Segment SGI97947[] = {{&S81140[91], 5}, {NULL}};
const Segment SGI97948[] = {{&S91819[44], 5}, {NULL}};
const Segment SGI97949[] = {{&S91819[44], 17}, {NULL}};
const Segment SGI97950[] = {{&S91819[30], 6}, {NULL}};
const Segment SGI97951[] = {{&S91819[75], 6}, {NULL}};
const Segment SGI97952[] = {{&S91819[102], 13}, {NULL}};
const Segment SGI97953[] = {{&S77977[44], 4}, {NULL}};
const Segment SGI97954[] = {{&S77977[44], 16}, {NULL}};
const Segment SGI97955[] = {{&S77977[30], 6}, {NULL}};
const Segment SGI97956[] = {{&S77977[31], 3}, {NULL}};
const Segment SGI97957[] = {{&S77977[74], 5}, {NULL}};
const Segment SGI97958[] = {{&S77977[0], 13}, {NULL}};
const Segment SGI97959[] = {{&S91848[44], 4}, {NULL}};
const Segment SGI97960[] = {{&S91848[44], 16}, {NULL}};
const Segment SGI97961[] = {{&S91848[30], 6}, {NULL}};
const Segment SGI97962[] = {{&S91848[74], 6}, {NULL}};
const Segment SGI97963[] = {{&S91848[101], 13}, {NULL}};
const Segment SGI97964[] = {{&S78019[44], 10}, {NULL}};
const Segment SGI97965[] = {{&S78019[44], 19}, {NULL}};
const Segment SGI97966[] = {{&S78019[30], 6}, {NULL}};
const Segment SGI97967[] = {{&S78019[31], 3}, {NULL}};
const Segment SGI97968[] = {{&S78019[77], 6}, {NULL}};
const Segment SGI97969[] = {{&S78019[0], 13}, {NULL}};
const Segment SGI97970[] = {{&S91911[44], 10}, {NULL}};
const Segment SGI97971[] = {{&S91911[44], 27}, {NULL}};
const Segment SGI97972[] = {{&S91911[30], 6}, {NULL}};
const Segment SGI97973[] = {{&S91911[119], 3}, {NULL}};
const Segment SGI97974[] = {{&S91911[85], 5}, {NULL}};
const Segment SGI97975[] = {{&S91911[143], 7}, {NULL}};
const Segment SGI97976[] = {{&S91912[44], 11}, {NULL}};
const Segment SGI97977[] = {{&S91912[44], 28}, {NULL}};
const Segment SGI97978[] = {{&S91912[30], 6}, {NULL}};
const Segment SGI97979[] = {{&S91912[120], 3}, {NULL}};
const Segment SGI97980[] = {{&S91912[86], 5}, {NULL}};
const Segment SGI97981[] = {{&S91912[144], 7}, {NULL}};
const Segment SGI97982[] = {{&S91914[59], 11}, {NULL}};
const Segment SGI97983[] = {{&S91914[59], 23}, {NULL}};
const Segment SGI97984[] = {{&S91914[45], 6}, {NULL}};
const Segment SGI97985[] = {{&S91914[130], 3}, {NULL}};
const Segment SGI97986[] = {{&S91914[96], 5}, {NULL}};
const Segment SGI97987[] = {{&S91914[0], 14}, {NULL}};
const Segment SGI97988[] = {{&S84590[44], 11}, {NULL}};
const Segment SGI97989[] = {{&S84590[44], 21}, {NULL}};
const Segment SGI97990[] = {{&S84590[30], 6}, {NULL}};
const Segment SGI97991[] = {{&S84590[113], 3}, {NULL}};
const Segment SGI97992[] = {{&S84590[79], 5}, {NULL}};
const Segment SGI97993[] = {{&S84591[44], 8}, {NULL}};
const Segment SGI97994[] = {{&S84591[44], 18}, {NULL}};
const Segment SGI97995[] = {{&S84591[30], 6}, {NULL}};
const Segment SGI97996[] = {{&S84591[110], 3}, {NULL}};
const Segment SGI97997[] = {{&S84591[76], 5}, {NULL}};
const Segment SGI97998[] = {{&S84592[44], 8}, {NULL}};
const Segment SGI97999[] = {{&S84592[44], 18}, {NULL}};
const Segment SGI98000[] = {{&S84592[30], 6}, {NULL}};
const Segment SGI98001[] = {{&S84592[110], 3}, {NULL}};
const Segment SGI98002[] = {{&S84592[76], 5}, {NULL}};
const Segment SGI98003[] = {{&S84593[44], 5}, {NULL}};
const Segment SGI98004[] = {{&S84593[44], 15}, {NULL}};
const Segment SGI98005[] = {{&S84593[30], 6}, {NULL}};
const Segment SGI98006[] = {{&S84593[107], 3}, {NULL}};
const Segment SGI98007[] = {{&S84593[73], 5}, {NULL}};
const Segment SGI98008[] = {{&S91944[59], 10}, {NULL}};
const Segment SGI98009[] = {{&S91944[59], 27}, {NULL}};
const Segment SGI98010[] = {{&S91944[45], 6}, {NULL}};
const Segment SGI98011[] = {{&S91944[134], 3}, {NULL}};
const Segment SGI98012[] = {{&S91944[100], 5}, {NULL}};
const Segment SGI98013[] = {{&S91944[0], 14}, {NULL}};
const Segment SGI98014[] = {{&S84613[45], 7}, {NULL}};
const Segment SGI98015[] = {{&S84613[30], 6}, {NULL}};
const Segment SGI98016[] = {{&S84613[117], 3}, {NULL}};
const Segment SGI98017[] = {{&S84613[83], 5}, {NULL}};
const Segment SGI98018[] = {{&S84613[141], 13}, {NULL}};
const Segment SGI98019[] = {{&S91948[59], 10}, {NULL}};
const Segment SGI98020[] = {{&S91948[59], 27}, {NULL}};
const Segment SGI98021[] = {{&S91948[45], 6}, {NULL}};
const Segment SGI98022[] = {{&S91948[134], 3}, {NULL}};
const Segment SGI98023[] = {{&S91948[100], 5}, {NULL}};
const Segment SGI98024[] = {{&S91948[0], 14}, {NULL}};
const Segment SGI98025[] = {{&S69640[30], 6}, {NULL}};
const Segment SGI98026[] = {{&S69640[31], 3}, {NULL}};
const Segment SGI98027[] = {{&S69640[81], 6}, {NULL}};
const Segment SGI98028[] = {{&S69654[34], 12}, {NULL}};
const Segment SGI98029[] = {{&S69654[27], 6}, {NULL}};
const Segment SGI98030[] = {{&S69654[99], 9}, {NULL}};
const Segment SGI98031[] = {{&S69654[72], 6}, {NULL}};
const Segment SGI98032[] = {{&S69655[34], 12}, {NULL}};
const Segment SGI98033[] = {{&S69655[27], 6}, {NULL}};
const Segment SGI98034[] = {{&S69655[99], 9}, {NULL}};
const Segment SGI98035[] = {{&S69655[72], 6}, {NULL}};
const Segment SGI98036[] = {{&S91969[45], 13}, {NULL}};
const Segment SGI98037[] = {{&S91969[30], 6}, {NULL}};
const Segment SGI98038[] = {{&S91969[123], 3}, {NULL}};
const Segment SGI98039[] = {{&S91969[89], 5}, {NULL}};
const Segment SGI98040[] = {{&S91969[147], 13}, {NULL}};
const Segment SGI98041[] = {{&S91970[45], 10}, {NULL}};
const Segment SGI98042[] = {{&S91970[30], 6}, {NULL}};
const Segment SGI98043[] = {{&S91970[120], 3}, {NULL}};
const Segment SGI98044[] = {{&S91970[86], 5}, {NULL}};
const Segment SGI98045[] = {{&S91970[144], 13}, {NULL}};
const Segment SGI98046[] = {{&S91974[44], 18}, {NULL}};
const Segment SGI98047[] = {{&S91974[44], 35}, {NULL}};
const Segment SGI98048[] = {{&S91974[30], 6}, {NULL}};
const Segment SGI98049[] = {{&S91974[127], 3}, {NULL}};
const Segment SGI98050[] = {{&S91974[93], 5}, {NULL}};
const Segment SGI98051[] = {{&S91974[151], 11}, {NULL}};
const Segment SGI98052[] = {{&S69678[44], 10}, {NULL}};
const Segment SGI98053[] = {{&S69678[44], 22}, {NULL}};
const Segment SGI98054[] = {{&S69678[30], 6}, {NULL}};
const Segment SGI98055[] = {{&S69678[31], 3}, {NULL}};
const Segment SGI98056[] = {{&S69678[80], 6}, {NULL}};
const Segment SGI98057[] = {{&S92009[44], 10}, {NULL}};
const Segment SGI98058[] = {{&S92009[44], 27}, {NULL}};
const Segment SGI98059[] = {{&S92009[30], 6}, {NULL}};
const Segment SGI98060[] = {{&S92009[85], 6}, {NULL}};
const Segment SGI98061[] = {{&S92009[112], 13}, {NULL}};
const Segment SGI98062[] = {{&S69707[44], 8}, {NULL}};
const Segment SGI98063[] = {{&S69707[44], 20}, {NULL}};
const Segment SGI98064[] = {{&S69707[30], 6}, {NULL}};
const Segment SGI98065[] = {{&S69707[31], 3}, {NULL}};
const Segment SGI98066[] = {{&S69707[78], 6}, {NULL}};
const Segment SGI98067[] = {{&S92047[44], 10}, {NULL}};
const Segment SGI98068[] = {{&S92047[44], 27}, {NULL}};
const Segment SGI98069[] = {{&S92047[30], 6}, {NULL}};
const Segment SGI98070[] = {{&S92047[118], 3}, {NULL}};
const Segment SGI98071[] = {{&S92047[85], 5}, {NULL}};
const Segment SGI98072[] = {{&S92047[122], 14}, {NULL}};
const Segment SGI98073[] = {{&S69796[44], 13}, {NULL}};
const Segment SGI98074[] = {{&S69796[44], 25}, {NULL}};
const Segment SGI98075[] = {{&S69796[30], 6}, {NULL}};
const Segment SGI98076[] = {{&S69796[31], 3}, {NULL}};
const Segment SGI98077[] = {{&S69796[83], 6}, {NULL}};
const Segment SGI98078[] = {{&S69797[44], 6}, {NULL}};
const Segment SGI98079[] = {{&S69797[44], 18}, {NULL}};
const Segment SGI98080[] = {{&S69797[30], 6}, {NULL}};
const Segment SGI98081[] = {{&S69797[31], 3}, {NULL}};
const Segment SGI98082[] = {{&S69797[76], 6}, {NULL}};
const Segment SGI98083[] = {{&S83841[43], 10}, {NULL}};
const Segment SGI98084[] = {{&S83841[30], 4}, {NULL}};
const Segment SGI98085[] = {{&S83841[113], 3}, {NULL}};
const Segment SGI98086[] = {{&S83841[79], 5}, {NULL}};
const Segment SGI98087[] = {{&S83841[137], 13}, {NULL}};
const Segment SGI98088[] = {{&S83842[43], 11}, {NULL}};
const Segment SGI98089[] = {{&S83842[30], 4}, {NULL}};
const Segment SGI98090[] = {{&S69800[44], 17}, {NULL}};
const Segment SGI98091[] = {{&S69800[44], 29}, {NULL}};
const Segment SGI98092[] = {{&S69800[30], 6}, {NULL}};
const Segment SGI98093[] = {{&S83842[114], 3}, {NULL}};
const Segment SGI98094[] = {{&S83842[80], 5}, {NULL}};
const Segment SGI98095[] = {{&S69800[31], 3}, {NULL}};
const Segment SGI98096[] = {{&S83842[138], 13}, {NULL}};
const Segment SGI98097[] = {{&S69800[87], 6}, {NULL}};
const Segment SGI98098[] = {{&S69801[44], 17}, {NULL}};
const Segment SGI98099[] = {{&S69801[44], 28}, {NULL}};
const Segment SGI98100[] = {{&S69801[30], 6}, {NULL}};
const Segment SGI98101[] = {{&S69801[31], 3}, {NULL}};
const Segment SGI98102[] = {{&S69801[86], 6}, {NULL}};
const Segment SGI98103[] = {{&S92138[59], 5}, {NULL}};
const Segment SGI98104[] = {{&S92138[59], 17}, {NULL}};
const Segment SGI98105[] = {{&S92138[45], 6}, {NULL}};
const Segment SGI98106[] = {{&S92138[124], 3}, {NULL}};
const Segment SGI98107[] = {{&S92138[90], 5}, {NULL}};
const Segment SGI98108[] = {{&S92138[0], 14}, {NULL}};
const Segment SGI98109[] = {{&S92142[59], 10}, {NULL}};
const Segment SGI98110[] = {{&S92142[59], 27}, {NULL}};
const Segment SGI98111[] = {{&S92142[45], 6}, {NULL}};
const Segment SGI98112[] = {{&S92142[134], 3}, {NULL}};
const Segment SGI98113[] = {{&S92142[100], 5}, {NULL}};
const Segment SGI98114[] = {{&S92142[0], 14}, {NULL}};
const Segment SGI98115[] = {{&S92165[45], 9}, {NULL}};
const Segment SGI98116[] = {{&S92165[30], 6}, {NULL}};
const Segment SGI98117[] = {{&S92165[114], 3}, {NULL}};
const Segment SGI98118[] = {{&S92165[80], 5}, {NULL}};
const Segment SGI98119[] = {{&S92165[138], 13}, {NULL}};
const Segment SGI98120[] = {{&S92184[45], 11}, {NULL}};
const Segment SGI98121[] = {{&S92184[30], 6}, {NULL}};
const Segment SGI98122[] = {{&S92184[121], 3}, {NULL}};
const Segment SGI98123[] = {{&S92184[87], 5}, {NULL}};
const Segment SGI98124[] = {{&S92184[145], 13}, {NULL}};
const Segment SGI98125[] = {{&S69860[34], 14}, {NULL}};
const Segment SGI98126[] = {{&S69860[27], 6}, {NULL}};
const Segment SGI98127[] = {{&S69860[101], 9}, {NULL}};
const Segment SGI98128[] = {{&S69860[74], 6}, {NULL}};
const Segment SGI98129[] = {{&S69882[34], 9}, {NULL}};
const Segment SGI98130[] = {{&S69882[27], 6}, {NULL}};
const Segment SGI98131[] = {{&S69882[96], 9}, {NULL}};
const Segment SGI98132[] = {{&S69882[69], 6}, {NULL}};
const Segment SGI98133[] = {{&S92250[44], 10}, {NULL}};
const Segment SGI98134[] = {{&S92250[44], 27}, {NULL}};
const Segment SGI98135[] = {{&S92250[30], 6}, {NULL}};
const Segment SGI98136[] = {{&S92250[85], 6}, {NULL}};
const Segment SGI98137[] = {{&S92250[112], 13}, {NULL}};
const Segment SGI98138[] = {{&S69911[44], 13}, {NULL}};
const Segment SGI98139[] = {{&S69911[44], 25}, {NULL}};
const Segment SGI98140[] = {{&S69911[30], 6}, {NULL}};
const Segment SGI98141[] = {{&S69911[31], 3}, {NULL}};
const Segment SGI98142[] = {{&S69911[83], 6}, {NULL}};
const Segment SGI98143[] = {{&S69912[44], 23}, {NULL}};
const Segment SGI98144[] = {{&S69912[44], 35}, {NULL}};
const Segment SGI98145[] = {{&S69912[30], 6}, {NULL}};
const Segment SGI98146[] = {{&S69912[31], 3}, {NULL}};
const Segment SGI98147[] = {{&S69912[93], 6}, {NULL}};
const Segment SGI98148[] = {{&S69913[44], 23}, {NULL}};
const Segment SGI98149[] = {{&S69913[44], 35}, {NULL}};
const Segment SGI98150[] = {{&S69913[30], 6}, {NULL}};
const Segment SGI98151[] = {{&S69913[31], 3}, {NULL}};
const Segment SGI98152[] = {{&S69913[93], 6}, {NULL}};
const Segment SGI98153[] = {{&S69914[44], 14}, {NULL}};
const Segment SGI98154[] = {{&S69914[44], 26}, {NULL}};
const Segment SGI98155[] = {{&S69914[30], 6}, {NULL}};
const Segment SGI98156[] = {{&S69914[31], 3}, {NULL}};
const Segment SGI98157[] = {{&S69914[84], 6}, {NULL}};
const Segment SGI98158[] = {{&S69954[34], 14}, {NULL}};
const Segment SGI98159[] = {{&S69954[27], 6}, {NULL}};
const Segment SGI98160[] = {{&S69954[106], 9}, {NULL}};
const Segment SGI98161[] = {{&S69954[79], 6}, {NULL}};
const Segment SGI98162[] = {{&S69955[34], 15}, {NULL}};
const Segment SGI98163[] = {{&S69955[27], 6}, {NULL}};
const Segment SGI98164[] = {{&S69955[115], 9}, {NULL}};
const Segment SGI98165[] = {{&S69955[88], 6}, {NULL}};
const Segment SGI98166[] = {{&S92331[45], 5}, {NULL}};
const Segment SGI98167[] = {{&S92331[30], 6}, {NULL}};
const Segment SGI98168[] = {{&S92331[110], 3}, {NULL}};
const Segment SGI98169[] = {{&S92331[76], 5}, {NULL}};
const Segment SGI98170[] = {{&S92331[134], 13}, {NULL}};
const Segment SGI98171[] = {{&S92344[44], 10}, {NULL}};
const Segment SGI98172[] = {{&S92344[44], 27}, {NULL}};
const Segment SGI98173[] = {{&S92344[30], 6}, {NULL}};
const Segment SGI98174[] = {{&S92344[149], 4}, {NULL}};
const Segment SGI98175[] = {{&S92344[175], 3}, {NULL}};
const Segment SGI98176[] = {{&S92344[119], 3}, {&S92344[224], 5}, {NULL}};
const Segment SGI98177[] = {{&S92344[85], 5}, {NULL}};
const Segment SGI98178[] = {{&S92358[45], 10}, {NULL}};
const Segment SGI98179[] = {{&S92358[30], 6}, {NULL}};
const Segment SGI98180[] = {{&S92358[120], 3}, {NULL}};
const Segment SGI98181[] = {{&S92358[86], 5}, {NULL}};
const Segment SGI98182[] = {{&S92358[144], 13}, {NULL}};
const Segment SGI98183[] = {{&S92365[44], 10}, {NULL}};
const Segment SGI98184[] = {{&S92365[44], 27}, {NULL}};
const Segment SGI98185[] = {{&S92365[30], 6}, {NULL}};
const Segment SGI98186[] = {{&S92365[119], 3}, {NULL}};
const Segment SGI98187[] = {{&S92365[85], 5}, {NULL}};
const Segment SGI98188[] = {{&S92365[143], 11}, {NULL}};
const Segment SGI98189[] = {{&S70023[34], 8}, {NULL}};
const Segment SGI98190[] = {{&S70023[27], 6}, {NULL}};
const Segment SGI98191[] = {{&S70023[95], 9}, {NULL}};
const Segment SGI98192[] = {{&S70023[68], 6}, {NULL}};
const Segment SGI98193[] = {{&S70029[34], 15}, {NULL}};
const Segment SGI98194[] = {{&S70029[27], 6}, {NULL}};
const Segment SGI98195[] = {{&S70029[102], 9}, {NULL}};
const Segment SGI98196[] = {{&S70029[75], 6}, {NULL}};
const Segment SGI98197[] = {{&S92388[44], 11}, {NULL}};
const Segment SGI98198[] = {{&S92388[44], 28}, {NULL}};
const Segment SGI98199[] = {{&S92388[30], 6}, {NULL}};
const Segment SGI98200[] = {{&S92388[120], 3}, {NULL}};
const Segment SGI98201[] = {{&S92388[86], 5}, {NULL}};
const Segment SGI98202[] = {{&S92388[144], 11}, {NULL}};
const Segment SGI98203[] = {{&S92391[44], 10}, {NULL}};
const Segment SGI98204[] = {{&S92391[44], 27}, {NULL}};
const Segment SGI98205[] = {{&S92391[30], 6}, {NULL}};
const Segment SGI98206[] = {{&S92391[118], 3}, {NULL}};
const Segment SGI98207[] = {{&S92391[85], 5}, {NULL}};
const Segment SGI98208[] = {{&S92391[122], 14}, {NULL}};
const Segment SGI98209[] = {{&S70053[44], 7}, {NULL}};
const Segment SGI98210[] = {{&S70053[44], 19}, {NULL}};
const Segment SGI98211[] = {{&S70053[30], 6}, {NULL}};
const Segment SGI98212[] = {{&S70053[31], 3}, {NULL}};
const Segment SGI98213[] = {{&S70053[77], 6}, {NULL}};
const Segment SGI98214[] = {{&S92424[52], 14}, {NULL}};
const Segment SGI98215[] = {{&S92424[52], 26}, {NULL}};
const Segment SGI98216[] = {{&S92424[38], 6}, {NULL}};
const Segment SGI98217[] = {{&S92424[92], 5}, {NULL}};
const Segment SGI98218[] = {{&S92424[0], 14}, {NULL}};
const Segment SGI98219[] = {{&S84169[44], 10}, {NULL}};
const Segment SGI98220[] = {{&S84169[44], 25}, {NULL}};
const Segment SGI98221[] = {{&S84169[30], 6}, {NULL}};
const Segment SGI98222[] = {{&S84169[118], 3}, {NULL}};
const Segment SGI98223[] = {{&S84169[83], 6}, {NULL}};
const Segment SGI98224[] = {{&S84170[44], 10}, {NULL}};
const Segment SGI98225[] = {{&S84170[44], 21}, {NULL}};
const Segment SGI98226[] = {{&S84170[30], 6}, {NULL}};
const Segment SGI98227[] = {{&S84170[114], 3}, {NULL}};
const Segment SGI98228[] = {{&S84170[79], 6}, {NULL}};
const Segment SGI98229[] = {{&S70092[34], 10}, {NULL}};
const Segment SGI98230[] = {{&S70092[27], 6}, {NULL}};
const Segment SGI98231[] = {{&S70092[97], 9}, {NULL}};
const Segment SGI98232[] = {{&S70092[70], 6}, {NULL}};
const Segment SGI98233[] = {{&S85033[44], 7}, {NULL}};
const Segment SGI98234[] = {{&S85033[44], 24}, {NULL}};
const Segment SGI98235[] = {{&S85033[30], 6}, {NULL}};
const Segment SGI98236[] = {{&S85033[82], 6}, {NULL}};
const Segment SGI98237[] = {{&S85033[109], 13}, {NULL}};
const Segment SGI98238[] = {{&S85034[44], 7}, {NULL}};
const Segment SGI98239[] = {{&S85034[44], 24}, {NULL}};
const Segment SGI98240[] = {{&S85034[30], 6}, {NULL}};
const Segment SGI98241[] = {{&S85034[82], 6}, {NULL}};
const Segment SGI98242[] = {{&S85034[109], 13}, {NULL}};
const Segment SGI98243[] = {{&S70137[34], 14}, {NULL}};
const Segment SGI98244[] = {{&S70137[27], 6}, {NULL}};
const Segment SGI98245[] = {{&S70137[106], 9}, {NULL}};
const Segment SGI98246[] = {{&S70137[79], 6}, {NULL}};
const Segment SGI98247[] = {{&S85043[44], 11}, {NULL}};
const Segment SGI98248[] = {{&S85043[44], 21}, {NULL}};
const Segment SGI98249[] = {{&S85043[30], 6}, {NULL}};
const Segment SGI98250[] = {{&S85043[113], 3}, {NULL}};
const Segment SGI98251[] = {{&S85043[79], 5}, {NULL}};
const Segment SGI98252[] = {{&S85046[44], 11}, {NULL}};
const Segment SGI98253[] = {{&S85046[44], 21}, {NULL}};
const Segment SGI98254[] = {{&S85046[30], 6}, {NULL}};
const Segment SGI98255[] = {{&S85046[82], 4}, {NULL}};
const Segment SGI98256[] = {{&S85046[82], 3}, {NULL}};
const Segment SGI98257[] = {{&S70144[34], 18}, {NULL}};
const Segment SGI98258[] = {{&S70144[27], 6}, {NULL}};
const Segment SGI98259[] = {{&S70144[114], 9}, {NULL}};
const Segment SGI98260[] = {{&S70144[87], 6}, {NULL}};
const Segment SGI98261[] = {{&S70145[34], 15}, {NULL}};
const Segment SGI98262[] = {{&S70145[27], 6}, {NULL}};
const Segment SGI98263[] = {{&S70145[108], 9}, {NULL}};
const Segment SGI98264[] = {{&S70145[81], 6}, {NULL}};
const Segment SGI98265[] = {{&S70146[34], 15}, {NULL}};
const Segment SGI98266[] = {{&S70146[27], 6}, {NULL}};
const Segment SGI98267[] = {{&S70146[108], 9}, {NULL}};
const Segment SGI98268[] = {{&S70146[81], 6}, {NULL}};
const Segment SGI98269[] = {{&S85055[44], 10}, {NULL}};
const Segment SGI98270[] = {{&S85055[44], 21}, {NULL}};
const Segment SGI98271[] = {{&S85055[30], 6}, {NULL}};
const Segment SGI98272[] = {{&S85055[129], 3}, {NULL}};
const Segment SGI98273[] = {{&S85055[95], 5}, {NULL}};
const Segment SGI98274[] = {{&S85055[35], 1}, {NULL}};
const Segment SGI98275[] = {{&S85055[67], 11}, {NULL}};
const Segment SGI98276[] = {{&S85056[44], 10}, {NULL}};
const Segment SGI98277[] = {{&S85056[44], 31}, {NULL}};
const Segment SGI98278[] = {{&S85056[30], 6}, {NULL}};
const Segment SGI98279[] = {{&S85056[138], 3}, {NULL}};
const Segment SGI98280[] = {{&S85056[104], 5}, {NULL}};
const Segment SGI98281[] = {{&S85056[35], 1}, {NULL}};
const Segment SGI98282[] = {{&S85056[77], 11}, {NULL}};
const Segment SGI98283[] = {{&S70182[34], 13}, {NULL}};
const Segment SGI98284[] = {{&S70182[27], 6}, {NULL}};
const Segment SGI98285[] = {{&S70182[100], 9}, {NULL}};
const Segment SGI98286[] = {{&S70182[73], 6}, {NULL}};
const Segment SGI98287[] = {{&S70188[44], 7}, {NULL}};
const Segment SGI98288[] = {{&S70188[44], 19}, {NULL}};
const Segment SGI98289[] = {{&S70188[30], 6}, {NULL}};
const Segment SGI98290[] = {{&S70188[31], 3}, {NULL}};
const Segment SGI98291[] = {{&S70188[77], 6}, {NULL}};
const Segment SGI98292[] = {{&S84304[42], 11}, {NULL}};
const Segment SGI98293[] = {{&S84304[42], 23}, {NULL}};
const Segment SGI98294[] = {{&S84304[30], 4}, {NULL}};
const Segment SGI98295[] = {{&S84304[114], 3}, {NULL}};
const Segment SGI98296[] = {{&S84304[79], 6}, {NULL}};
const Segment SGI98297[] = {{&S84304[132], 13}, {NULL}};
const Segment SGI98298[] = {{&S84305[42], 10}, {NULL}};
const Segment SGI98299[] = {{&S84305[42], 22}, {NULL}};
const Segment SGI98300[] = {{&S84305[30], 4}, {NULL}};
const Segment SGI98301[] = {{&S84305[113], 3}, {NULL}};
const Segment SGI98302[] = {{&S84305[78], 6}, {NULL}};
const Segment SGI98303[] = {{&S84305[131], 13}, {NULL}};
const Segment SGI98304[] = {{&S70217[34], 14}, {NULL}};
const Segment SGI98305[] = {{&S70217[27], 6}, {NULL}};
const Segment SGI98306[] = {{&S70217[98], 9}, {NULL}};
const Segment SGI98307[] = {{&S70217[71], 6}, {NULL}};
const Segment SGI98308[] = {{&S92594[61], 10}, {NULL}};
const Segment SGI98309[] = {{&S92594[61], 27}, {NULL}};
const Segment SGI98310[] = {{&S92594[47], 6}, {NULL}};
const Segment SGI98311[] = {{&S92594[136], 3}, {NULL}};
const Segment SGI98312[] = {{&S92594[102], 5}, {NULL}};
const Segment SGI98313[] = {{&S92594[0], 14}, {NULL}};
const Segment SGI98314[] = {{&S85150[44], 13}, {NULL}};
const Segment SGI98315[] = {{&S85150[44], 23}, {NULL}};
const Segment SGI98316[] = {{&S85150[30], 6}, {NULL}};
const Segment SGI98317[] = {{&S85150[115], 3}, {NULL}};
const Segment SGI98318[] = {{&S85150[81], 5}, {NULL}};
const Segment SGI98319[] = {{&S92632[44], 13}, {NULL}};
const Segment SGI98320[] = {{&S92632[44], 30}, {NULL}};
const Segment SGI98321[] = {{&S92632[30], 6}, {NULL}};
const Segment SGI98322[] = {{&S92632[88], 6}, {NULL}};
const Segment SGI98323[] = {{&S92632[115], 13}, {NULL}};
const Segment SGI98324[] = {{&S85209[34], 7}, {NULL}};
const Segment SGI98325[] = {{&S85209[27], 6}, {NULL}};
const Segment SGI98326[] = {{&S85209[106], 2}, {NULL}};
const Segment SGI98327[] = {{&S85209[72], 6}, {NULL}};
const Segment SGI98328[] = {{&S85209[140], 8}, {NULL}};
const Segment SGI98329[] = {{&S70245[34], 6}, {NULL}};
const Segment SGI98330[] = {{&S70245[27], 6}, {NULL}};
const Segment SGI98331[] = {{&S70245[93], 9}, {NULL}};
const Segment SGI98332[] = {{&S70245[66], 6}, {NULL}};
const Segment SGI98333[] = {{&S85230[44], 7}, {NULL}};
const Segment SGI98334[] = {{&S85230[44], 24}, {NULL}};
const Segment SGI98335[] = {{&S85230[30], 6}, {NULL}};
const Segment SGI98336[] = {{&S85230[82], 6}, {NULL}};
const Segment SGI98337[] = {{&S85230[109], 13}, {NULL}};
const Segment SGI98338[] = {{&S85256[44], 7}, {NULL}};
const Segment SGI98339[] = {{&S85256[44], 25}, {NULL}};
const Segment SGI98340[] = {{&S85256[30], 6}, {NULL}};
const Segment SGI98341[] = {{&S85256[57], 3}, {NULL}};
const Segment SGI98342[] = {{&S85256[102], 5}, {NULL}};
const Segment SGI98343[] = {{&S85256[35], 1}, {NULL}};
const Segment SGI98344[] = {{&S85256[71], 11}, {NULL}};
const Segment SGI98345[] = {{&S92737[44], 10}, {NULL}};
const Segment SGI98346[] = {{&S92737[44], 27}, {NULL}};
const Segment SGI98347[] = {{&S92737[30], 6}, {NULL}};
const Segment SGI98348[] = {{&S92737[119], 3}, {NULL}};
const Segment SGI98349[] = {{&S92737[85], 5}, {NULL}};
const Segment SGI98350[] = {{&S92737[143], 11}, {NULL}};
const Segment SGI98351[] = {{&S92774[44], 10}, {NULL}};
const Segment SGI98352[] = {{&S92774[44], 27}, {NULL}};
const Segment SGI98353[] = {{&S92774[30], 6}, {NULL}};
const Segment SGI98354[] = {{&S92774[85], 6}, {NULL}};
const Segment SGI98355[] = {{&S92774[112], 13}, {NULL}};
const Segment SGI98356[] = {{&S84494[44], 7}, {NULL}};
const Segment SGI98357[] = {{&S84494[44], 19}, {NULL}};
const Segment SGI98358[] = {{&S84494[30], 6}, {NULL}};
const Segment SGI98359[] = {{&S84494[111], 3}, {NULL}};
const Segment SGI98360[] = {{&S84494[77], 5}, {NULL}};
const Segment SGI98361[] = {{&S84495[42], 7}, {NULL}};
const Segment SGI98362[] = {{&S84495[42], 19}, {NULL}};
const Segment SGI98363[] = {{&S84495[30], 4}, {NULL}};
const Segment SGI98364[] = {{&S84495[109], 3}, {NULL}};
const Segment SGI98365[] = {{&S84495[75], 5}, {NULL}};
const Segment SGI98366[] = {{&S92792[34], 7}, {NULL}};
const Segment SGI98367[] = {{&S92792[27], 6}, {NULL}};
const Segment SGI98368[] = {{&S92792[106], 2}, {NULL}};
const Segment SGI98369[] = {{&S92792[72], 6}, {NULL}};
const Segment SGI98370[] = {{&S92792[140], 8}, {NULL}};
const Segment SGI98371[] = {{&S92806[44], 9}, {NULL}};
const Segment SGI98372[] = {{&S92806[44], 21}, {NULL}};
const Segment SGI98373[] = {{&S92806[30], 6}, {NULL}};
const Segment SGI98374[] = {{&S92806[79], 6}, {NULL}};
const Segment SGI98375[] = {{&S81984[42], 14}, {NULL}};
const Segment SGI98376[] = {{&S81984[42], 26}, {NULL}};
const Segment SGI98377[] = {{&S92806[106], 13}, {NULL}};
const Segment SGI98378[] = {{&S81984[30], 4}, {NULL}};
const Segment SGI98379[] = {{&S81984[31], 3}, {NULL}};
const Segment SGI98380[] = {{&S81984[82], 6}, {NULL}};
const Segment SGI98381[] = {{&S70316[44], 17}, {NULL}};
const Segment SGI98382[] = {{&S70316[44], 29}, {NULL}};
const Segment SGI98383[] = {{&S70316[30], 6}, {NULL}};
const Segment SGI98384[] = {{&S70316[31], 3}, {NULL}};
const Segment SGI98385[] = {{&S70316[87], 6}, {NULL}};
const Segment SGI98386[] = {{&S92872[45], 10}, {NULL}};
const Segment SGI98387[] = {{&S92872[30], 6}, {NULL}};
const Segment SGI98388[] = {{&S92872[120], 3}, {NULL}};
const Segment SGI98389[] = {{&S92872[86], 5}, {NULL}};
const Segment SGI98390[] = {{&S92872[144], 13}, {NULL}};
const Segment SGI98391[] = {{&S70349[44], 7}, {NULL}};
const Segment SGI98392[] = {{&S70349[44], 19}, {NULL}};
const Segment SGI98393[] = {{&S70349[30], 6}, {NULL}};
const Segment SGI98394[] = {{&S70349[31], 3}, {NULL}};
const Segment SGI98395[] = {{&S70349[77], 6}, {NULL}};
const Segment SGI98396[] = {{&S92891[59], 10}, {NULL}};
const Segment SGI98397[] = {{&S92891[59], 27}, {NULL}};
const Segment SGI98398[] = {{&S92891[45], 6}, {NULL}};
const Segment SGI98399[] = {{&S92891[134], 3}, {NULL}};
const Segment SGI98400[] = {{&S92891[100], 5}, {NULL}};
const Segment SGI98401[] = {{&S92891[0], 14}, {NULL}};
const Segment SGI98402[] = {{&S84625[42], 10}, {NULL}};
const Segment SGI98403[] = {{&S84625[42], 21}, {NULL}};
const Segment SGI98404[] = {{&S84625[30], 4}, {NULL}};
const Segment SGI98405[] = {{&S84625[136], 4}, {NULL}};
const Segment SGI98406[] = {{&S84625[162], 3}, {NULL}};
const Segment SGI98407[] = {{&S84625[112], 3}, {&S84625[211], 5}, {NULL}};
const Segment SGI98408[] = {{&S84625[77], 6}, {NULL}};
const Segment SGI98409[] = {{&S84626[40], 10}, {NULL}};
const Segment SGI98410[] = {{&S84626[30], 4}, {NULL}};
const Segment SGI98411[] = {{&S84626[134], 4}, {NULL}};
const Segment SGI98412[] = {{&S84626[160], 3}, {NULL}};
const Segment SGI98413[] = {{&S84626[110], 3}, {&S84626[209], 5}, {NULL}};
const Segment SGI98414[] = {{&S84626[75], 6}, {NULL}};
const Segment SGI98415[] = {{&S70387[34], 12}, {NULL}};
const Segment SGI98416[] = {{&S70387[27], 6}, {NULL}};
const Segment SGI98417[] = {{&S70387[99], 9}, {NULL}};
const Segment SGI98418[] = {{&S70387[72], 6}, {NULL}};
const Segment SGI98419[] = {{&S70388[34], 12}, {NULL}};
const Segment SGI98420[] = {{&S70388[27], 6}, {NULL}};
const Segment SGI98421[] = {{&S70388[99], 9}, {NULL}};
const Segment SGI98422[] = {{&S70388[72], 6}, {NULL}};
const Segment SGI98423[] = {{&S70392[34], 23}, {NULL}};
const Segment SGI98424[] = {{&S70392[27], 6}, {NULL}};
const Segment SGI98425[] = {{&S70392[110], 9}, {NULL}};
const Segment SGI98426[] = {{&S70392[83], 6}, {NULL}};
const Segment SGI98427[] = {{&S70393[34], 17}, {NULL}};
const Segment SGI98428[] = {{&S70393[27], 6}, {NULL}};
const Segment SGI98429[] = {{&S70393[104], 9}, {NULL}};
const Segment SGI98430[] = {{&S70393[77], 6}, {NULL}};
const Segment SGI98431[] = {{&S92927[44], 10}, {NULL}};
const Segment SGI98432[] = {{&S92927[44], 27}, {NULL}};
const Segment SGI98433[] = {{&S92927[30], 6}, {NULL}};
const Segment SGI98434[] = {{&S92927[85], 6}, {NULL}};
const Segment SGI98435[] = {{&S92927[112], 13}, {NULL}};
const Segment SGI98436[] = {{&S92931[44], 11}, {NULL}};
const Segment SGI98437[] = {{&S92931[44], 28}, {NULL}};
const Segment SGI98438[] = {{&S92931[30], 6}, {NULL}};
const Segment SGI98439[] = {{&S92931[120], 3}, {NULL}};
const Segment SGI98440[] = {{&S92931[86], 5}, {NULL}};
const Segment SGI98441[] = {{&S92931[131], 14}, {NULL}};
const Segment SGI98442[] = {{&S70418[34], 11}, {NULL}};
const Segment SGI98443[] = {{&S70418[27], 6}, {NULL}};
const Segment SGI98444[] = {{&S70418[98], 9}, {NULL}};
const Segment SGI98445[] = {{&S70418[71], 6}, {NULL}};
const Segment SGI98446[] = {{&S92952[44], 10}, {NULL}};
const Segment SGI98447[] = {{&S92952[44], 27}, {NULL}};
const Segment SGI98448[] = {{&S92952[30], 6}, {NULL}};
const Segment SGI98449[] = {{&S92952[118], 3}, {NULL}};
const Segment SGI98450[] = {{&S92952[85], 5}, {NULL}};
const Segment SGI98451[] = {{&S92952[122], 14}, {NULL}};
const Segment SGI98452[] = {{&S93002[44], 13}, {NULL}};
const Segment SGI98453[] = {{&S93002[44], 30}, {NULL}};
const Segment SGI98454[] = {{&S93002[30], 6}, {NULL}};
const Segment SGI98455[] = {{&S93002[88], 6}, {NULL}};
const Segment SGI98456[] = {{&S93002[115], 13}, {NULL}};
const Segment SGI98457[] = {{&S75540[34], 14}, {NULL}};
const Segment SGI98458[] = {{&S75540[27], 6}, {NULL}};
const Segment SGI98459[] = {{&S75540[101], 9}, {NULL}};
const Segment SGI98460[] = {{&S75540[74], 6}, {NULL}};
const Segment SGI98461[] = {{&S70484[44], 9}, {NULL}};
const Segment SGI98462[] = {{&S70484[44], 21}, {NULL}};
const Segment SGI98463[] = {{&S70484[30], 6}, {NULL}};
const Segment SGI98464[] = {{&S70484[31], 3}, {NULL}};
const Segment SGI98465[] = {{&S70484[79], 6}, {NULL}};
const Segment SGI98466[] = {{&S70486[34], 6}, {NULL}};
const Segment SGI98467[] = {{&S70486[27], 6}, {NULL}};
const Segment SGI98468[] = {{&S93010[44], 10}, {NULL}};
const Segment SGI98469[] = {{&S93010[44], 27}, {NULL}};
const Segment SGI98470[] = {{&S93010[30], 6}, {NULL}};
const Segment SGI98471[] = {{&S70486[93], 9}, {NULL}};
const Segment SGI98472[] = {{&S70486[66], 6}, {NULL}};
const Segment SGI98473[] = {{&S93010[85], 6}, {NULL}};
const Segment SGI98474[] = {{&S93010[112], 13}, {NULL}};
const Segment SGI98475[] = {{&S70489[44], 10}, {NULL}};
const Segment SGI98476[] = {{&S70489[44], 22}, {NULL}};
const Segment SGI98477[] = {{&S70489[30], 6}, {NULL}};
const Segment SGI98478[] = {{&S70489[31], 3}, {NULL}};
const Segment SGI98479[] = {{&S70489[80], 6}, {NULL}};
const Segment SGI98480[] = {{&S70489[143], 3}, {NULL}};
const Segment SGI98481[] = {{&S70494[34], 18}, {NULL}};
const Segment SGI98482[] = {{&S70494[27], 6}, {NULL}};
const Segment SGI98483[] = {{&S70494[114], 9}, {NULL}};
const Segment SGI98484[] = {{&S70494[87], 6}, {NULL}};
const Segment SGI98485[] = {{&S70495[34], 15}, {NULL}};
const Segment SGI98486[] = {{&S70495[27], 6}, {NULL}};
const Segment SGI98487[] = {{&S70495[108], 9}, {NULL}};
const Segment SGI98488[] = {{&S70495[81], 6}, {NULL}};
const Segment SGI98489[] = {{&S70496[34], 15}, {NULL}};
const Segment SGI98490[] = {{&S70496[27], 6}, {NULL}};
const Segment SGI98491[] = {{&S70496[108], 9}, {NULL}};
const Segment SGI98492[] = {{&S70496[81], 6}, {NULL}};
const Segment SGI98493[] = {{&S93027[44], 10}, {NULL}};
const Segment SGI98494[] = {{&S93027[44], 27}, {NULL}};
const Segment SGI98495[] = {{&S93027[30], 6}, {NULL}};
const Segment SGI98496[] = {{&S93027[118], 3}, {NULL}};
const Segment SGI98497[] = {{&S93027[85], 5}, {NULL}};
const Segment SGI98498[] = {{&S93027[122], 14}, {NULL}};
const Segment SGI98499[] = {{&S93030[44], 10}, {NULL}};
const Segment SGI98500[] = {{&S93030[44], 27}, {NULL}};
const Segment SGI98501[] = {{&S93030[30], 6}, {NULL}};
const Segment SGI98502[] = {{&S93030[85], 6}, {NULL}};
const Segment SGI98503[] = {{&S93030[112], 13}, {NULL}};
const Segment SGI98504[] = {{&S93032[44], 10}, {NULL}};
const Segment SGI98505[] = {{&S93032[44], 27}, {NULL}};
const Segment SGI98506[] = {{&S93032[30], 6}, {NULL}};
const Segment SGI98507[] = {{&S93032[118], 3}, {NULL}};
const Segment SGI98508[] = {{&S93032[85], 5}, {NULL}};
const Segment SGI98509[] = {{&S93032[122], 14}, {NULL}};
const Segment SGI98510[] = {{&S79206[81], 6}, {NULL}};
const Segment SGI98511[] = {{&S79206[0], 16}, {NULL}};
const Segment SGI98512[] = {{&S84801[43], 10}, {NULL}};
const Segment SGI98513[] = {{&S84801[30], 4}, {NULL}};
const Segment SGI98514[] = {{&S84801[113], 3}, {NULL}};
const Segment SGI98515[] = {{&S84801[79], 5}, {NULL}};
const Segment SGI98516[] = {{&S84801[137], 13}, {NULL}};
const Segment SGI98517[] = {{&S84802[43], 11}, {NULL}};
const Segment SGI98518[] = {{&S84802[30], 4}, {NULL}};
const Segment SGI98519[] = {{&S84802[114], 3}, {NULL}};
const Segment SGI98520[] = {{&S84802[80], 5}, {NULL}};
const Segment SGI98521[] = {{&S84802[138], 13}, {NULL}};
const Segment SGI98522[] = {{&S70548[34], 10}, {NULL}};
const Segment SGI98523[] = {{&S70548[27], 6}, {NULL}};
const Segment SGI98524[] = {{&S70548[97], 9}, {NULL}};
const Segment SGI98525[] = {{&S70548[70], 6}, {NULL}};
const Segment SGI98526[] = {{&S70548[138], 8}, {NULL}};
const Segment SGI98527[] = {{&S70550[34], 11}, {NULL}};
const Segment SGI98528[] = {{&S70550[27], 6}, {NULL}};
const Segment SGI98529[] = {{&S70550[98], 9}, {NULL}};
const Segment SGI98530[] = {{&S70550[71], 6}, {NULL}};
const Segment SGI98531[] = {{&S70550[139], 8}, {NULL}};
const Segment SGI98532[] = {{&S93084[31], 8}, {NULL}};
const Segment SGI98533[] = {{&S93084[54], 3}, {NULL}};
const Segment SGI98534[] = {{&S93088[57], 10}, {NULL}};
const Segment SGI98535[] = {{&S93088[57], 27}, {NULL}};
const Segment SGI98536[] = {{&S93088[43], 6}, {NULL}};
const Segment SGI98537[] = {{&S93088[132], 3}, {NULL}};
const Segment SGI98538[] = {{&S93088[98], 5}, {NULL}};
const Segment SGI98539[] = {{&S93088[0], 12}, {NULL}};
const Segment SGI98540[] = {{&S93089[58], 17}, {NULL}};
const Segment SGI98541[] = {{&S93089[58], 43}, {NULL}};
const Segment SGI98542[] = {{&S93089[44], 6}, {NULL}};
const Segment SGI98543[] = {{&S93089[149], 3}, {NULL}};
const Segment SGI98544[] = {{&S93089[115], 5}, {NULL}};
const Segment SGI98545[] = {{&S93089[0], 14}, {NULL}};
const Segment SGI98546[] = {{&S93090[45], 10}, {NULL}};
const Segment SGI98547[] = {{&S93090[30], 6}, {NULL}};
const Segment SGI98548[] = {{&S93090[120], 3}, {NULL}};
const Segment SGI98549[] = {{&S93090[86], 5}, {NULL}};
const Segment SGI98550[] = {{&S93090[144], 13}, {NULL}};
const Segment SGI98551[] = {{&S70574[34], 14}, {NULL}};
const Segment SGI98552[] = {{&S70574[27], 6}, {NULL}};
const Segment SGI98553[] = {{&S70574[101], 9}, {NULL}};
const Segment SGI98554[] = {{&S70574[74], 6}, {NULL}};
const Segment SGI98555[] = {{&S93109[59], 10}, {NULL}};
const Segment SGI98556[] = {{&S93109[59], 27}, {NULL}};
const Segment SGI98557[] = {{&S93109[45], 6}, {NULL}};
const Segment SGI98558[] = {{&S93109[134], 3}, {NULL}};
const Segment SGI98559[] = {{&S93109[100], 5}, {NULL}};
const Segment SGI98560[] = {{&S93109[0], 14}, {NULL}};
const Segment SGI98561[] = {{&S85684[59], 7}, {NULL}};
const Segment SGI98562[] = {{&S85684[59], 24}, {NULL}};
const Segment SGI98563[] = {{&S85684[45], 6}, {NULL}};
const Segment SGI98564[] = {{&S85684[131], 3}, {NULL}};
const Segment SGI98565[] = {{&S85684[97], 5}, {NULL}};
const Segment SGI98566[] = {{&S85684[0], 14}, {NULL}};
const Segment SGI98567[] = {{&S93119[44], 6}, {NULL}};
const Segment SGI98568[] = {{&S93119[44], 18}, {NULL}};
const Segment SGI98569[] = {{&S93119[30], 6}, {NULL}};
const Segment SGI98570[] = {{&S85685[59], 7}, {NULL}};
const Segment SGI98571[] = {{&S85685[59], 24}, {NULL}};
const Segment SGI98572[] = {{&S85685[45], 6}, {NULL}};
const Segment SGI98573[] = {{&S93119[110], 3}, {NULL}};
const Segment SGI98574[] = {{&S93119[76], 5}, {NULL}};
const Segment SGI98575[] = {{&S85685[131], 3}, {NULL}};
const Segment SGI98576[] = {{&S85685[97], 5}, {NULL}};
const Segment SGI98577[] = {{&S85685[0], 14}, {NULL}};
const Segment SGI98578[] = {{&S70651[34], 6}, {NULL}};
const Segment SGI98579[] = {{&S70651[27], 6}, {NULL}};
const Segment SGI98580[] = {{&S70651[93], 9}, {NULL}};
const Segment SGI98581[] = {{&S70651[66], 6}, {NULL}};
const Segment SGI98582[] = {{&S93174[44], 10}, {NULL}};
const Segment SGI98583[] = {{&S93174[44], 27}, {NULL}};
const Segment SGI98584[] = {{&S93174[30], 6}, {NULL}};
const Segment SGI98585[] = {{&S93174[85], 6}, {NULL}};
const Segment SGI98586[] = {{&S93174[112], 13}, {NULL}};
const Segment SGI98587[] = {{&S93188[44], 10}, {NULL}};
const Segment SGI98588[] = {{&S93188[44], 27}, {NULL}};
const Segment SGI98589[] = {{&S93188[30], 6}, {NULL}};
const Segment SGI98590[] = {{&S93188[118], 3}, {NULL}};
const Segment SGI98591[] = {{&S93188[85], 5}, {NULL}};
const Segment SGI98592[] = {{&S93188[122], 14}, {NULL}};
const Segment SGI98593[] = {{&S70678[44], 8}, {NULL}};
const Segment SGI98594[] = {{&S70678[44], 20}, {NULL}};
const Segment SGI98595[] = {{&S70678[30], 6}, {NULL}};
const Segment SGI98596[] = {{&S70678[31], 3}, {NULL}};
const Segment SGI98597[] = {{&S70678[78], 6}, {NULL}};
const Segment SGI98598[] = {{&S93202[44], 9}, {NULL}};
const Segment SGI98599[] = {{&S93202[44], 21}, {NULL}};
const Segment SGI98600[] = {{&S93202[30], 6}, {NULL}};
const Segment SGI98601[] = {{&S93202[79], 6}, {NULL}};
const Segment SGI98602[] = {{&S93202[106], 13}, {NULL}};
const Segment SGI98603[] = {{&S93234[44], 10}, {NULL}};
const Segment SGI98604[] = {{&S93234[44], 27}, {NULL}};
const Segment SGI98605[] = {{&S93234[30], 6}, {NULL}};
const Segment SGI98606[] = {{&S93234[119], 3}, {NULL}};
const Segment SGI98607[] = {{&S93234[85], 5}, {NULL}};
const Segment SGI98608[] = {{&S93234[143], 7}, {NULL}};
const Segment SGI98609[] = {{&S93235[44], 11}, {NULL}};
const Segment SGI98610[] = {{&S93235[44], 28}, {NULL}};
const Segment SGI98611[] = {{&S93235[30], 6}, {NULL}};
const Segment SGI98612[] = {{&S93235[120], 3}, {NULL}};
const Segment SGI98613[] = {{&S93235[86], 5}, {NULL}};
const Segment SGI98614[] = {{&S93235[144], 7}, {NULL}};
const Segment SGI98615[] = {{&S70745[44], 8}, {NULL}};
const Segment SGI98616[] = {{&S70745[44], 20}, {NULL}};
const Segment SGI98617[] = {{&S70745[30], 6}, {NULL}};
const Segment SGI98618[] = {{&S70745[31], 3}, {NULL}};
const Segment SGI98619[] = {{&S70745[78], 6}, {NULL}};
const Segment SGI98620[] = {{&S70754[45], 10}, {NULL}};
const Segment SGI98621[] = {{&S70754[30], 6}, {NULL}};
const Segment SGI98622[] = {{&S70754[31], 3}, {NULL}};
const Segment SGI98623[] = {{&S70754[81], 5}, {NULL}};
const Segment SGI98624[] = {{&S70754[139], 13}, {NULL}};
const Segment SGI98625[] = {{&S70755[45], 11}, {NULL}};
const Segment SGI98626[] = {{&S70755[30], 6}, {NULL}};
const Segment SGI98627[] = {{&S70755[31], 3}, {NULL}};
const Segment SGI98628[] = {{&S70755[82], 5}, {NULL}};
const Segment SGI98629[] = {{&S70755[140], 13}, {NULL}};
const Segment SGI98630[] = {{&S93328[57], 4}, {NULL}};
const Segment SGI98631[] = {{&S93328[57], 16}, {NULL}};
const Segment SGI98632[] = {{&S93328[43], 6}, {NULL}};
const Segment SGI98633[] = {{&S93328[121], 3}, {NULL}};
const Segment SGI98634[] = {{&S93328[87], 5}, {NULL}};
const Segment SGI98635[] = {{&S93328[0], 12}, {NULL}};
const Segment SGI98636[] = {{&S70827[44], 10}, {NULL}};
const Segment SGI98637[] = {{&S70827[44], 22}, {NULL}};
const Segment SGI98638[] = {{&S70827[30], 6}, {NULL}};
const Segment SGI98639[] = {{&S70827[31], 3}, {NULL}};
const Segment SGI98640[] = {{&S70827[80], 6}, {NULL}};
const Segment SGI98641[] = {{&S70827[143], 3}, {NULL}};
const Segment SGI98642[] = {{&S70831[44], 10}, {NULL}};
const Segment SGI98643[] = {{&S70831[44], 22}, {NULL}};
const Segment SGI98644[] = {{&S70831[30], 6}, {NULL}};
const Segment SGI98645[] = {{&S70831[80], 6}, {NULL}};
const Segment SGI98646[] = {{&S70831[107], 13}, {NULL}};
const Segment SGI98647[] = {{&S85909[44], 13}, {NULL}};
const Segment SGI98648[] = {{&S85909[44], 23}, {NULL}};
const Segment SGI98649[] = {{&S85909[30], 6}, {NULL}};
const Segment SGI98650[] = {{&S85909[115], 3}, {NULL}};
const Segment SGI98651[] = {{&S85909[81], 5}, {NULL}};
const Segment SGI98652[] = {{&S85921[44], 11}, {NULL}};
const Segment SGI98653[] = {{&S85921[44], 36}, {NULL}};
const Segment SGI98654[] = {{&S85921[30], 6}, {NULL}};
const Segment SGI98655[] = {{&S85921[97], 4}, {NULL}};
const Segment SGI98656[] = {{&S85921[97], 3}, {NULL}};
const Segment SGI98657[] = {{&S85923[44], 11}, {NULL}};
const Segment SGI98658[] = {{&S85923[44], 36}, {NULL}};
const Segment SGI98659[] = {{&S85923[30], 6}, {NULL}};
const Segment SGI98660[] = {{&S85923[128], 3}, {NULL}};
const Segment SGI98661[] = {{&S85923[94], 5}, {NULL}};
const Segment SGI98662[] = {{&S70875[44], 10}, {NULL}};
const Segment SGI98663[] = {{&S70875[44], 21}, {NULL}};
const Segment SGI98664[] = {{&S70875[30], 6}, {NULL}};
const Segment SGI98665[] = {{&S70875[31], 3}, {NULL}};
const Segment SGI98666[] = {{&S70875[98], 6}, {NULL}};
const Segment SGI98667[] = {{&S79503[75], 6}, {NULL}};
const Segment SGI98668[] = {{&S79503[0], 10}, {NULL}};
const Segment SGI98669[] = {{&S93390[44], 13}, {NULL}};
const Segment SGI98670[] = {{&S93390[44], 30}, {NULL}};
const Segment SGI98671[] = {{&S93390[30], 6}, {NULL}};
const Segment SGI98672[] = {{&S93390[122], 3}, {NULL}};
const Segment SGI98673[] = {{&S93390[88], 5}, {NULL}};
const Segment SGI98674[] = {{&S93390[146], 11}, {NULL}};
const Segment SGI98675[] = {{&S70884[44], 13}, {NULL}};
const Segment SGI98676[] = {{&S70884[44], 25}, {NULL}};
const Segment SGI98677[] = {{&S70884[30], 6}, {NULL}};
const Segment SGI98678[] = {{&S93391[44], 14}, {NULL}};
const Segment SGI98679[] = {{&S93391[44], 26}, {NULL}};
const Segment SGI98680[] = {{&S70884[31], 3}, {NULL}};
const Segment SGI98681[] = {{&S93391[30], 6}, {NULL}};
const Segment SGI98682[] = {{&S70884[83], 6}, {NULL}};
const Segment SGI98683[] = {{&S93391[118], 3}, {NULL}};
const Segment SGI98684[] = {{&S93391[84], 5}, {NULL}};
const Segment SGI98685[] = {{&S93398[44], 5}, {NULL}};
const Segment SGI98686[] = {{&S93398[44], 26}, {NULL}};
const Segment SGI98687[] = {{&S93398[30], 6}, {NULL}};
const Segment SGI98688[] = {{&S93398[119], 3}, {NULL}};
const Segment SGI98689[] = {{&S93398[84], 6}, {NULL}};
const Segment SGI98690[] = {{&S93398[139], 8}, {NULL}};
const Segment SGI98691[] = {{&S93398[148], 13}, {NULL}};
const Segment SGI98692[] = {{&S70895[38], 14}, {NULL}};
const Segment SGI98693[] = {{&S70895[31], 6}, {NULL}};
const Segment SGI98694[] = {{&S70897[44], 14}, {NULL}};
const Segment SGI98695[] = {{&S70897[44], 26}, {NULL}};
const Segment SGI98696[] = {{&S70897[30], 6}, {NULL}};
const Segment SGI98697[] = {{&S70897[31], 3}, {NULL}};
const Segment SGI98698[] = {{&S70897[84], 6}, {NULL}};
const Segment SGI98699[] = {{&S93407[44], 11}, {NULL}};
const Segment SGI98700[] = {{&S93407[44], 28}, {NULL}};
const Segment SGI98701[] = {{&S93407[30], 6}, {NULL}};
const Segment SGI98702[] = {{&S93407[120], 3}, {NULL}};
const Segment SGI98703[] = {{&S93407[86], 5}, {NULL}};
const Segment SGI98704[] = {{&S93407[144], 11}, {NULL}};
const Segment SGI98705[] = {{&S70901[44], 11}, {NULL}};
const Segment SGI98706[] = {{&S70901[44], 23}, {NULL}};
const Segment SGI98707[] = {{&S70901[30], 6}, {NULL}};
const Segment SGI98708[] = {{&S70901[31], 3}, {NULL}};
const Segment SGI98709[] = {{&S70901[81], 6}, {NULL}};
const Segment SGI98710[] = {{&S93429[44], 7}, {NULL}};
const Segment SGI98711[] = {{&S93429[44], 31}, {NULL}};
const Segment SGI98712[] = {{&S93429[30], 6}, {NULL}};
const Segment SGI98713[] = {{&S93429[123], 3}, {NULL}};
const Segment SGI98714[] = {{&S93429[89], 5}, {NULL}};
const Segment SGI98715[] = {{&S93429[147], 7}, {NULL}};
const Segment SGI98716[] = {{&S70941[44], 9}, {NULL}};
const Segment SGI98717[] = {{&S70941[44], 21}, {NULL}};
const Segment SGI98718[] = {{&S70941[30], 6}, {NULL}};
const Segment SGI98719[] = {{&S70941[31], 3}, {NULL}};
const Segment SGI98720[] = {{&S70941[79], 6}, {NULL}};
const Segment SGI98721[] = {{&S70980[34], 20}, {NULL}};
const Segment SGI98722[] = {{&S70980[27], 6}, {NULL}};
const Segment SGI98723[] = {{&S70980[107], 9}, {NULL}};
const Segment SGI98724[] = {{&S70980[80], 6}, {NULL}};
const Segment SGI98725[] = {{&S70985[34], 15}, {NULL}};
const Segment SGI98726[] = {{&S70985[27], 6}, {NULL}};
const Segment SGI98727[] = {{&S70985[102], 9}, {NULL}};
const Segment SGI98728[] = {{&S70985[75], 6}, {NULL}};
const Segment SGI98729[] = {{&S86088[34], 10}, {NULL}};
const Segment SGI98730[] = {{&S86088[27], 6}, {NULL}};
const Segment SGI98731[] = {{&S86088[107], 2}, {NULL}};
const Segment SGI98732[] = {{&S86088[73], 6}, {NULL}};
const Segment SGI98733[] = {{&S86088[141], 8}, {NULL}};
const Segment SGI98734[] = {{&S71026[34], 6}, {NULL}};
const Segment SGI98735[] = {{&S71026[27], 6}, {NULL}};
const Segment SGI98736[] = {{&S71026[93], 9}, {NULL}};
const Segment SGI98737[] = {{&S71026[66], 6}, {NULL}};
const Segment SGI98738[] = {{&S93540[44], 10}, {NULL}};
const Segment SGI98739[] = {{&S93540[44], 22}, {NULL}};
const Segment SGI98740[] = {{&S93540[30], 6}, {NULL}};
const Segment SGI98741[] = {{&S93540[114], 3}, {NULL}};
const Segment SGI98742[] = {{&S93540[80], 5}, {NULL}};
const Segment SGI98743[] = {{&S93545[30], 6}, {NULL}};
const Segment SGI98744[] = {{&S93545[91], 3}, {NULL}};
const Segment SGI98745[] = {{&S93545[125], 3}, {NULL}};
const Segment SGI98746[] = {{&S93545[57], 5}, {NULL}};
const Segment SGI98747[] = {{&S71048[34], 17}, {NULL}};
const Segment SGI98748[] = {{&S71048[27], 6}, {NULL}};
const Segment SGI98749[] = {{&S71048[120], 9}, {NULL}};
const Segment SGI98750[] = {{&S71048[93], 6}, {NULL}};
const Segment SGI98751[] = {{&S71076[34], 9}, {NULL}};
const Segment SGI98752[] = {{&S71076[27], 6}, {NULL}};
const Segment SGI98753[] = {{&S71076[96], 9}, {NULL}};
const Segment SGI98754[] = {{&S71076[69], 6}, {NULL}};
const Segment SGI98755[] = {{&S93589[38], 6}, {NULL}};
const Segment SGI98756[] = {{&S93589[31], 6}, {NULL}};
const Segment SGI98757[] = {{&S93590[44], 6}, {NULL}};
const Segment SGI98758[] = {{&S93590[44], 18}, {NULL}};
const Segment SGI98759[] = {{&S93590[30], 6}, {NULL}};
const Segment SGI98760[] = {{&S93590[110], 3}, {NULL}};
const Segment SGI98761[] = {{&S93590[76], 5}, {NULL}};
const Segment SGI98762[] = {{&S71086[34], 12}, {NULL}};
const Segment SGI98763[] = {{&S71086[27], 6}, {NULL}};
const Segment SGI98764[] = {{&S71086[99], 9}, {NULL}};
const Segment SGI98765[] = {{&S71086[72], 6}, {NULL}};
const Segment SGI98766[] = {{&S71087[34], 12}, {NULL}};
const Segment SGI98767[] = {{&S71087[27], 6}, {NULL}};
const Segment SGI98768[] = {{&S71087[99], 9}, {NULL}};
const Segment SGI98769[] = {{&S71087[72], 6}, {NULL}};
const Segment SGI98770[] = {{&S71088[34], 6}, {NULL}};
const Segment SGI98771[] = {{&S71088[27], 6}, {NULL}};
const Segment SGI98772[] = {{&S71088[93], 9}, {NULL}};
const Segment SGI98773[] = {{&S71088[66], 6}, {NULL}};
const Segment SGI98774[] = {{&S93607[59], 10}, {NULL}};
const Segment SGI98775[] = {{&S93607[59], 27}, {NULL}};
const Segment SGI98776[] = {{&S93607[45], 6}, {NULL}};
const Segment SGI98777[] = {{&S93607[134], 3}, {NULL}};
const Segment SGI98778[] = {{&S93607[100], 5}, {NULL}};
const Segment SGI98779[] = {{&S93607[0], 14}, {NULL}};
const Segment SGI98780[] = {{&S93610[44], 10}, {NULL}};
const Segment SGI98781[] = {{&S93610[44], 27}, {NULL}};
const Segment SGI98782[] = {{&S93610[30], 6}, {NULL}};
const Segment SGI98783[] = {{&S93610[119], 3}, {NULL}};
const Segment SGI98784[] = {{&S93610[85], 5}, {NULL}};
const Segment SGI98785[] = {{&S93616[44], 11}, {NULL}};
const Segment SGI98786[] = {{&S93616[44], 28}, {NULL}};
const Segment SGI98787[] = {{&S93616[30], 6}, {NULL}};
const Segment SGI98788[] = {{&S93616[119], 3}, {NULL}};
const Segment SGI98789[] = {{&S93616[86], 5}, {NULL}};
const Segment SGI98790[] = {{&S93616[123], 14}, {NULL}};
const Segment SGI98791[] = {{&S71118[34], 7}, {NULL}};
const Segment SGI98792[] = {{&S71118[27], 6}, {NULL}};
const Segment SGI98793[] = {{&S71118[94], 9}, {NULL}};
const Segment SGI98794[] = {{&S71118[67], 6}, {NULL}};
const Segment SGI98795[] = {{&S71119[34], 29}, {NULL}};
const Segment SGI98796[] = {{&S71119[27], 6}, {NULL}};
const Segment SGI98797[] = {{&S71119[116], 9}, {NULL}};
const Segment SGI98798[] = {{&S71119[89], 6}, {NULL}};
const Segment SGI98799[] = {{&S93633[44], 10}, {NULL}};
const Segment SGI98800[] = {{&S93633[44], 27}, {NULL}};
const Segment SGI98801[] = {{&S93633[30], 6}, {NULL}};
const Segment SGI98802[] = {{&S93633[85], 6}, {NULL}};
const Segment SGI98803[] = {{&S93633[112], 13}, {NULL}};
const Segment SGI98804[] = {{&S93636[61], 4}, {NULL}};
const Segment SGI98805[] = {{&S93636[61], 16}, {NULL}};
const Segment SGI98806[] = {{&S93636[47], 6}, {NULL}};
const Segment SGI98807[] = {{&S93636[125], 3}, {NULL}};
const Segment SGI98808[] = {{&S93636[91], 5}, {NULL}};
const Segment SGI98809[] = {{&S93636[0], 14}, {NULL}};
const Segment SGI98810[] = {{&S93673[44], 10}, {NULL}};
const Segment SGI98811[] = {{&S93673[44], 27}, {NULL}};
const Segment SGI98812[] = {{&S93673[30], 6}, {NULL}};
const Segment SGI98813[] = {{&S93673[118], 3}, {NULL}};
const Segment SGI98814[] = {{&S93673[85], 5}, {NULL}};
const Segment SGI98815[] = {{&S93673[122], 14}, {NULL}};
const Segment SGI98816[] = {{&S71176[44], 6}, {NULL}};
const Segment SGI98817[] = {{&S71176[44], 18}, {NULL}};
const Segment SGI98818[] = {{&S71176[30], 6}, {NULL}};
const Segment SGI98819[] = {{&S71176[31], 3}, {NULL}};
const Segment SGI98820[] = {{&S71176[76], 6}, {NULL}};
const Segment SGI98821[] = {{&S93677[44], 10}, {NULL}};
const Segment SGI98822[] = {{&S93677[44], 27}, {NULL}};
const Segment SGI98823[] = {{&S93677[30], 6}, {NULL}};
const Segment SGI98824[] = {{&S93677[118], 3}, {NULL}};
const Segment SGI98825[] = {{&S93677[85], 5}, {NULL}};
const Segment SGI98826[] = {{&S93677[122], 14}, {NULL}};
const Segment SGI98827[] = {{&S71185[44], 10}, {NULL}};
const Segment SGI98828[] = {{&S71185[44], 22}, {NULL}};
const Segment SGI98829[] = {{&S71185[30], 6}, {NULL}};
const Segment SGI98830[] = {{&S71185[31], 3}, {NULL}};
const Segment SGI98831[] = {{&S71185[80], 6}, {NULL}};
const Segment SGI98832[] = {{&S71185[143], 3}, {NULL}};
const Segment SGI98833[] = {{&S93680[44], 10}, {NULL}};
const Segment SGI98834[] = {{&S93680[44], 27}, {NULL}};
const Segment SGI98835[] = {{&S93680[30], 6}, {NULL}};
const Segment SGI98836[] = {{&S93680[119], 3}, {NULL}};
const Segment SGI98837[] = {{&S93680[85], 5}, {NULL}};
const Segment SGI98838[] = {{&S93680[143], 7}, {NULL}};
const Segment SGI98839[] = {{&S71241[34], 14}, {NULL}};
const Segment SGI98840[] = {{&S71241[27], 6}, {NULL}};
const Segment SGI98841[] = {{&S71241[98], 9}, {NULL}};
const Segment SGI98842[] = {{&S71241[71], 6}, {NULL}};
const Segment SGI98843[] = {{&S71254[44], 6}, {NULL}};
const Segment SGI98844[] = {{&S71254[44], 29}, {NULL}};
const Segment SGI98845[] = {{&S71254[30], 6}, {NULL}};
const Segment SGI98846[] = {{&S71254[31], 3}, {NULL}};
const Segment SGI98847[] = {{&S71254[87], 6}, {NULL}};
const Segment SGI98848[] = {{&S86357[45], 7}, {NULL}};
const Segment SGI98849[] = {{&S86357[30], 6}, {NULL}};
const Segment SGI98850[] = {{&S86357[117], 3}, {NULL}};
const Segment SGI98851[] = {{&S86357[83], 5}, {NULL}};
const Segment SGI98852[] = {{&S86357[141], 13}, {NULL}};
const Segment SGI98853[] = {{&S86388[44], 14}, {NULL}};
const Segment SGI98854[] = {{&S86388[44], 30}, {NULL}};
const Segment SGI98855[] = {{&S71301[44], 14}, {NULL}};
const Segment SGI98856[] = {{&S86388[30], 6}, {NULL}};
const Segment SGI98857[] = {{&S71301[44], 26}, {NULL}};
const Segment SGI98858[] = {{&S71301[30], 6}, {NULL}};
const Segment SGI98859[] = {{&S86388[122], 3}, {NULL}};
const Segment SGI98860[] = {{&S71301[31], 3}, {NULL}};
const Segment SGI98861[] = {{&S86388[88], 5}, {NULL}};
const Segment SGI98862[] = {{&S71301[84], 6}, {NULL}};
const Segment SGI98863[] = {{&S71310[44], 13}, {NULL}};
const Segment SGI98864[] = {{&S71310[44], 29}, {NULL}};
const Segment SGI98865[] = {{&S71310[30], 6}, {NULL}};
const Segment SGI98866[] = {{&S71310[31], 3}, {NULL}};
const Segment SGI98867[] = {{&S71310[87], 6}, {NULL}};
const Segment SGI98868[] = {{&S71318[34], 6}, {NULL}};
const Segment SGI98869[] = {{&S71318[27], 6}, {NULL}};
const Segment SGI98870[] = {{&S71318[93], 9}, {NULL}};
const Segment SGI98871[] = {{&S71318[66], 6}, {NULL}};
const Segment SGI98872[] = {{&S71320[34], 17}, {NULL}};
const Segment SGI98873[] = {{&S71320[27], 6}, {NULL}};
const Segment SGI98874[] = {{&S71320[104], 9}, {NULL}};
const Segment SGI98875[] = {{&S71320[77], 6}, {NULL}};
const Segment SGI98876[] = {{&S71321[34], 23}, {NULL}};
const Segment SGI98877[] = {{&S71321[27], 6}, {NULL}};
const Segment SGI98878[] = {{&S71321[110], 9}, {NULL}};
const Segment SGI98879[] = {{&S71321[83], 6}, {NULL}};
const Segment SGI98880[] = {{&S71326[44], 11}, {NULL}};
const Segment SGI98881[] = {{&S71326[44], 18}, {NULL}};
const Segment SGI98882[] = {{&S71326[30], 6}, {NULL}};
const Segment SGI98883[] = {{&S71326[31], 3}, {NULL}};
const Segment SGI98884[] = {{&S71326[76], 6}, {NULL}};
const Segment SGI98885[] = {{&S86446[44], 15}, {NULL}};
const Segment SGI98886[] = {{&S86446[44], 26}, {NULL}};
const Segment SGI98887[] = {{&S86446[30], 6}, {NULL}};
const Segment SGI98888[] = {{&S86446[118], 3}, {NULL}};
const Segment SGI98889[] = {{&S86446[84], 5}, {NULL}};
const Segment SGI98890[] = {{&S71362[34], 29}, {NULL}};
const Segment SGI98891[] = {{&S71362[27], 6}, {NULL}};
const Segment SGI98892[] = {{&S71362[116], 9}, {NULL}};
const Segment SGI98893[] = {{&S71362[89], 6}, {NULL}};
const Segment SGI98894[] = {{&S93827[31], 3}, {NULL}};
const Segment SGI98895[] = {{&S93827[49], 3}, {NULL}};
const Segment SGI98896[] = {{&S93836[59], 11}, {NULL}};
const Segment SGI98897[] = {{&S93836[59], 23}, {NULL}};
const Segment SGI98898[] = {{&S93836[45], 6}, {NULL}};
const Segment SGI98899[] = {{&S93836[130], 3}, {NULL}};
const Segment SGI98900[] = {{&S93836[96], 5}, {NULL}};
const Segment SGI98901[] = {{&S93836[0], 14}, {NULL}};
const Segment SGI98902[] = {{&S93837[45], 4}, {NULL}};
const Segment SGI98903[] = {{&S93837[30], 6}, {NULL}};
const Segment SGI98904[] = {{&S93837[109], 3}, {NULL}};
const Segment SGI98905[] = {{&S93837[75], 5}, {NULL}};
const Segment SGI98906[] = {{&S93837[133], 13}, {NULL}};
const Segment SGI98907[] = {{&S93839[45], 5}, {NULL}};
const Segment SGI98908[] = {{&S93839[30], 6}, {NULL}};
const Segment SGI98909[] = {{&S93839[110], 3}, {NULL}};
const Segment SGI98910[] = {{&S93839[76], 5}, {NULL}};
const Segment SGI98911[] = {{&S93839[134], 13}, {NULL}};
const Segment SGI98912[] = {{&S86483[44], 9}, {NULL}};
const Segment SGI98913[] = {{&S86483[44], 20}, {NULL}};
const Segment SGI98914[] = {{&S86483[30], 6}, {NULL}};
const Segment SGI98915[] = {{&S86483[131], 3}, {NULL}};
const Segment SGI98916[] = {{&S86483[97], 5}, {NULL}};
const Segment SGI98917[] = {{&S86483[35], 1}, {NULL}};
const Segment SGI98918[] = {{&S86483[66], 11}, {NULL}};
const Segment SGI98919[] = {{&S86484[44], 9}, {NULL}};
const Segment SGI98920[] = {{&S86484[44], 20}, {NULL}};
const Segment SGI98921[] = {{&S86484[30], 6}, {NULL}};
const Segment SGI98922[] = {{&S86484[125], 3}, {NULL}};
const Segment SGI98923[] = {{&S86484[91], 5}, {NULL}};
const Segment SGI98924[] = {{&S86484[66], 3}, {NULL}};
const Segment SGI98925[] = {{&S86504[44], 7}, {NULL}};
const Segment SGI98926[] = {{&S86504[44], 24}, {NULL}};
const Segment SGI98927[] = {{&S86504[30], 6}, {NULL}};
const Segment SGI98928[] = {{&S86504[82], 6}, {NULL}};
const Segment SGI98929[] = {{&S86504[109], 13}, {NULL}};
const Segment SGI98930[] = {{&S71415[34], 18}, {NULL}};
const Segment SGI98931[] = {{&S71415[27], 6}, {NULL}};
const Segment SGI98932[] = {{&S71415[105], 9}, {NULL}};
const Segment SGI98933[] = {{&S71415[78], 6}, {NULL}};
const Segment SGI98934[] = {{&S85674[42], 19}, {NULL}};
const Segment SGI98935[] = {{&S85674[42], 31}, {NULL}};
const Segment SGI98936[] = {{&S85674[30], 4}, {&S85674[50], 23}, {NULL}};
const Segment SGI98937[] = {{&S85674[122], 3}, {NULL}};
const Segment SGI98938[] = {{&S85674[87], 6}, {NULL}};
const Segment SGI98939[] = {{&S71439[44], 10}, {NULL}};
const Segment SGI98940[] = {{&S71439[44], 27}, {NULL}};
const Segment SGI98941[] = {{&S71439[30], 6}, {NULL}};
const Segment SGI98942[] = {{&S71439[31], 3}, {NULL}};
const Segment SGI98943[] = {{&S71439[85], 6}, {NULL}};
const Segment SGI98944[] = {{&S71440[44], 10}, {NULL}};
const Segment SGI98945[] = {{&S71440[44], 22}, {NULL}};
const Segment SGI98946[] = {{&S71440[30], 6}, {NULL}};
const Segment SGI98947[] = {{&S71440[31], 3}, {NULL}};
const Segment SGI98948[] = {{&S71440[80], 6}, {NULL}};
const Segment SGI98949[] = {{&S93909[44], 10}, {NULL}};
const Segment SGI98950[] = {{&S93909[44], 27}, {NULL}};
const Segment SGI98951[] = {{&S93909[30], 6}, {NULL}};
const Segment SGI98952[] = {{&S93909[118], 3}, {NULL}};
const Segment SGI98953[] = {{&S93909[85], 5}, {NULL}};
const Segment SGI98954[] = {{&S93909[122], 14}, {NULL}};
const Segment SGI98955[] = {{&S71444[38], 10}, {NULL}};
const Segment SGI98956[] = {{&S71444[31], 6}, {NULL}};
const Segment SGI98957[] = {{&S93914[59], 12}, {NULL}};
const Segment SGI98958[] = {{&S93914[59], 24}, {NULL}};
const Segment SGI98959[] = {{&S93914[45], 6}, {NULL}};
const Segment SGI98960[] = {{&S93914[131], 3}, {NULL}};
const Segment SGI98961[] = {{&S93914[97], 5}, {NULL}};
const Segment SGI98962[] = {{&S93914[0], 14}, {NULL}};
const Segment SGI98963[] = {{&S93916[44], 4}, {NULL}};
const Segment SGI98964[] = {{&S93916[44], 16}, {NULL}};
const Segment SGI98965[] = {{&S93916[30], 6}, {NULL}};
const Segment SGI98966[] = {{&S93916[74], 6}, {NULL}};
const Segment SGI98967[] = {{&S93916[101], 13}, {NULL}};
const Segment SGI98968[] = {{&S71465[34], 5}, {NULL}};
const Segment SGI98969[] = {{&S71465[27], 6}, {NULL}};
const Segment SGI98970[] = {{&S71465[92], 9}, {NULL}};
const Segment SGI98971[] = {{&S71465[65], 6}, {NULL}};
const Segment SGI98972[] = {{&S71489[44], 13}, {NULL}};
const Segment SGI98973[] = {{&S71489[30], 6}, {NULL}};
const Segment SGI98974[] = {{&S71489[31], 3}, {NULL}};
const Segment SGI98975[] = {{&S71489[85], 6}, {NULL}};
const Segment SGI98976[] = {{&S71490[31], 6}, {NULL}};
const Segment SGI98977[] = {{&S80047[44], 10}, {NULL}};
const Segment SGI98978[] = {{&S80047[44], 20}, {NULL}};
const Segment SGI98979[] = {{&S80047[30], 6}, {NULL}};
const Segment SGI98980[] = {{&S80047[31], 3}, {NULL}};
const Segment SGI98981[] = {{&S80047[78], 5}, {NULL}};
const Segment SGI98982[] = {{&S80047[0], 13}, {NULL}};
const Segment SGI98983[] = {{&S71529[34], 13}, {NULL}};
const Segment SGI98984[] = {{&S71529[27], 6}, {NULL}};
const Segment SGI98985[] = {{&S71529[97], 9}, {NULL}};
const Segment SGI98986[] = {{&S71529[70], 6}, {NULL}};
const Segment SGI98987[] = {{&S94022[45], 10}, {NULL}};
const Segment SGI98988[] = {{&S94022[30], 6}, {NULL}};
const Segment SGI98989[] = {{&S94022[120], 3}, {NULL}};
const Segment SGI98990[] = {{&S94022[86], 5}, {NULL}};
const Segment SGI98991[] = {{&S94022[144], 13}, {NULL}};
const Segment SGI98992[] = {{&S86663[44], 10}, {NULL}};
const Segment SGI98993[] = {{&S86663[44], 21}, {NULL}};
const Segment SGI98994[] = {{&S86663[30], 6}, {NULL}};
const Segment SGI98995[] = {{&S86663[132], 3}, {NULL}};
const Segment SGI98996[] = {{&S94033[57], 10}, {NULL}};
const Segment SGI98997[] = {{&S86663[98], 5}, {NULL}};
const Segment SGI98998[] = {{&S94033[57], 27}, {NULL}};
const Segment SGI98999[] = {{&S94033[43], 6}, {NULL}};
const Segment SGI99000[] = {{&S86663[35], 1}, {NULL}};
const Segment SGI99001[] = {{&S86663[67], 11}, {NULL}};
const Segment SGI99002[] = {{&S94033[132], 3}, {NULL}};
const Segment SGI99003[] = {{&S94033[98], 5}, {NULL}};
const Segment SGI99004[] = {{&S94033[0], 12}, {NULL}};
const Segment SGI99005[] = {{&S94034[57], 11}, {NULL}};
const Segment SGI99006[] = {{&S94034[57], 28}, {NULL}};
const Segment SGI99007[] = {{&S94034[43], 6}, {NULL}};
const Segment SGI99008[] = {{&S94034[133], 3}, {NULL}};
const Segment SGI99009[] = {{&S94034[99], 5}, {NULL}};
const Segment SGI99010[] = {{&S94034[0], 12}, {NULL}};
const Segment SGI99011[] = {{&S71566[34], 13}, {NULL}};
const Segment SGI99012[] = {{&S86672[44], 10}, {NULL}};
const Segment SGI99013[] = {{&S71566[27], 6}, {NULL}};
const Segment SGI99014[] = {{&S86672[44], 30}, {NULL}};
const Segment SGI99015[] = {{&S86672[30], 6}, {NULL}};
const Segment SGI99016[] = {{&S71566[97], 9}, {NULL}};
const Segment SGI99017[] = {{&S86672[122], 3}, {NULL}};
const Segment SGI99018[] = {{&S94040[59], 10}, {NULL}};
const Segment SGI99019[] = {{&S71566[70], 6}, {NULL}};
const Segment SGI99020[] = {{&S94040[59], 27}, {NULL}};
const Segment SGI99021[] = {{&S94040[45], 6}, {NULL}};
const Segment SGI99022[] = {{&S86672[88], 5}, {NULL}};
const Segment SGI99023[] = {{&S94040[134], 3}, {NULL}};
const Segment SGI99024[] = {{&S94040[100], 5}, {NULL}};
const Segment SGI99025[] = {{&S94040[0], 14}, {NULL}};
const Segment SGI99026[] = {{&S94052[45], 4}, {NULL}};
const Segment SGI99027[] = {{&S94052[30], 6}, {NULL}};
const Segment SGI99028[] = {{&S94052[109], 3}, {NULL}};
const Segment SGI99029[] = {{&S94052[75], 5}, {NULL}};
const Segment SGI99030[] = {{&S94052[133], 13}, {NULL}};
const Segment SGI99031[] = {{&S94058[58], 11}, {NULL}};
const Segment SGI99032[] = {{&S94058[58], 23}, {NULL}};
const Segment SGI99033[] = {{&S94058[44], 6}, {NULL}};
const Segment SGI99034[] = {{&S94058[129], 3}, {NULL}};
const Segment SGI99035[] = {{&S94058[95], 5}, {NULL}};
const Segment SGI99036[] = {{&S94058[0], 14}, {NULL}};
const Segment SGI99037[] = {{&S85846[42], 10}, {NULL}};
const Segment SGI99038[] = {{&S85846[42], 22}, {NULL}};
const Segment SGI99039[] = {{&S85846[30], 4}, {NULL}};
const Segment SGI99040[] = {{&S85846[113], 3}, {NULL}};
const Segment SGI99041[] = {{&S85846[78], 6}, {NULL}};
const Segment SGI99042[] = {{&S85847[42], 10}, {NULL}};
const Segment SGI99043[] = {{&S85847[42], 22}, {NULL}};
const Segment SGI99044[] = {{&S85847[30], 4}, {NULL}};
const Segment SGI99045[] = {{&S85847[113], 3}, {NULL}};
const Segment SGI99046[] = {{&S85847[78], 6}, {NULL}};
const Segment SGI99047[] = {{&S80174[44], 7}, {NULL}};
const Segment SGI99048[] = {{&S80174[44], 19}, {NULL}};
const Segment SGI99049[] = {{&S80174[30], 6}, {NULL}};
const Segment SGI99050[] = {{&S80174[31], 3}, {NULL}};
const Segment SGI99051[] = {{&S80174[77], 5}, {NULL}};
const Segment SGI99052[] = {{&S80174[0], 13}, {NULL}};
const Segment SGI99053[] = {{&S94107[44], 5}, {NULL}};
const Segment SGI99054[] = {{&S94107[44], 17}, {NULL}};
const Segment SGI99055[] = {{&S94107[30], 6}, {NULL}};
const Segment SGI99056[] = {{&S94107[110], 3}, {NULL}};
const Segment SGI99057[] = {{&S94107[75], 6}, {NULL}};
const Segment SGI99058[] = {{&S94107[130], 8}, {NULL}};
const Segment SGI99059[] = {{&S94107[139], 13}, {NULL}};
const Segment SGI99060[] = {{&S94135[44], 10}, {NULL}};
const Segment SGI99061[] = {{&S94135[44], 27}, {NULL}};
const Segment SGI99062[] = {{&S94135[30], 6}, {NULL}};
const Segment SGI99063[] = {{&S94135[118], 3}, {NULL}};
const Segment SGI99064[] = {{&S94135[85], 5}, {NULL}};
const Segment SGI99065[] = {{&S94135[122], 14}, {NULL}};
const Segment SGI99066[] = {{&S94137[44], 10}, {NULL}};
const Segment SGI99067[] = {{&S94137[44], 27}, {NULL}};
const Segment SGI99068[] = {{&S94137[30], 6}, {NULL}};
const Segment SGI99069[] = {{&S94137[118], 3}, {NULL}};
const Segment SGI99070[] = {{&S94137[85], 5}, {NULL}};
const Segment SGI99071[] = {{&S94137[122], 14}, {NULL}};
const Segment SGI99072[] = {{&S71672[44], 14}, {NULL}};
const Segment SGI99073[] = {{&S71672[44], 26}, {NULL}};
const Segment SGI99074[] = {{&S71672[30], 6}, {NULL}};
const Segment SGI99075[] = {{&S71672[31], 3}, {NULL}};
const Segment SGI99076[] = {{&S71672[84], 6}, {NULL}};
const Segment SGI99077[] = {{&S94143[44], 11}, {NULL}};
const Segment SGI99078[] = {{&S94143[44], 28}, {NULL}};
const Segment SGI99079[] = {{&S94143[30], 6}, {NULL}};
const Segment SGI99080[] = {{&S94143[120], 3}, {NULL}};
const Segment SGI99081[] = {{&S94143[86], 5}, {NULL}};
const Segment SGI99082[] = {{&S94143[144], 7}, {NULL}};
const Segment SGI99083[] = {{&S86800[44], 13}, {NULL}};
const Segment SGI99084[] = {{&S86800[44], 24}, {NULL}};
const Segment SGI99085[] = {{&S86800[30], 6}, {NULL}};
const Segment SGI99086[] = {{&S86800[135], 3}, {NULL}};
const Segment SGI99087[] = {{&S86800[101], 5}, {NULL}};
const Segment SGI99088[] = {{&S86800[35], 1}, {NULL}};
const Segment SGI99089[] = {{&S86800[70], 11}, {NULL}};
const Segment SGI99090[] = {{&S71692[34], 20}, {NULL}};
const Segment SGI99091[] = {{&S71692[27], 6}, {NULL}};
const Segment SGI99092[] = {{&S71692[107], 9}, {NULL}};
const Segment SGI99093[] = {{&S71692[80], 6}, {NULL}};
const Segment SGI99094[] = {{&S71725[34], 17}, {NULL}};
const Segment SGI99095[] = {{&S71725[27], 6}, {NULL}};
const Segment SGI99096[] = {{&S71725[112], 9}, {NULL}};
const Segment SGI99097[] = {{&S71725[85], 6}, {NULL}};
const Segment SGI99098[] = {{&S71726[34], 13}, {NULL}};
const Segment SGI99099[] = {{&S71726[27], 6}, {NULL}};
const Segment SGI99100[] = {{&S71726[108], 9}, {NULL}};
const Segment SGI99101[] = {{&S71726[81], 6}, {NULL}};
const Segment SGI99102[] = {{&S71727[34], 17}, {NULL}};
const Segment SGI99103[] = {{&S71727[27], 6}, {NULL}};
const Segment SGI99104[] = {{&S71727[112], 9}, {NULL}};
const Segment SGI99105[] = {{&S71727[85], 6}, {NULL}};
const Segment SGI99106[] = {{&S94224[59], 11}, {NULL}};
const Segment SGI99107[] = {{&S94224[59], 28}, {NULL}};
const Segment SGI99108[] = {{&S94224[45], 6}, {NULL}};
const Segment SGI99109[] = {{&S94224[135], 3}, {NULL}};
const Segment SGI99110[] = {{&S94224[101], 5}, {NULL}};
const Segment SGI99111[] = {{&S94224[0], 14}, {NULL}};
const Segment SGI99112[] = {{&S94229[45], 5}, {NULL}};
const Segment SGI99113[] = {{&S94229[30], 6}, {NULL}};
const Segment SGI99114[] = {{&S94229[110], 3}, {NULL}};
const Segment SGI99115[] = {{&S94229[76], 5}, {NULL}};
const Segment SGI99116[] = {{&S94229[134], 13}, {NULL}};
const Segment SGI99117[] = {{&S94230[59], 10}, {NULL}};
const Segment SGI99118[] = {{&S94230[59], 27}, {NULL}};
const Segment SGI99119[] = {{&S94230[45], 6}, {NULL}};
const Segment SGI99120[] = {{&S94230[134], 3}, {NULL}};
const Segment SGI99121[] = {{&S94230[100], 5}, {NULL}};
const Segment SGI99122[] = {{&S94230[0], 14}, {NULL}};
const Segment SGI99123[] = {{&S94237[59], 11}, {NULL}};
const Segment SGI99124[] = {{&S94237[59], 28}, {NULL}};
const Segment SGI99125[] = {{&S94237[45], 6}, {NULL}};
const Segment SGI99126[] = {{&S94237[101], 5}, {NULL}};
const Segment SGI99127[] = {{&S94237[0], 14}, {NULL}};
const Segment SGI99128[] = {{&S94238[44], 11}, {NULL}};
const Segment SGI99129[] = {{&S94238[44], 28}, {NULL}};
const Segment SGI99130[] = {{&S94238[30], 6}, {NULL}};
const Segment SGI99131[] = {{&S94238[120], 3}, {NULL}};
const Segment SGI99132[] = {{&S94238[86], 5}, {NULL}};
const Segment SGI99133[] = {{&S94238[131], 14}, {NULL}};
const Segment SGI99134[] = {{&S71774[34], 5}, {NULL}};
const Segment SGI99135[] = {{&S71774[27], 6}, {NULL}};
const Segment SGI99136[] = {{&S71774[92], 9}, {NULL}};
const Segment SGI99137[] = {{&S71774[65], 6}, {NULL}};
const Segment SGI99138[] = {{&S71779[34], 14}, {NULL}};
const Segment SGI99139[] = {{&S71779[27], 6}, {NULL}};
const Segment SGI99140[] = {{&S71779[101], 9}, {NULL}};
const Segment SGI99141[] = {{&S71779[74], 6}, {NULL}};
const Segment SGI99142[] = {{&S71781[34], 15}, {NULL}};
const Segment SGI99143[] = {{&S71781[27], 6}, {NULL}};
const Segment SGI99144[] = {{&S71781[102], 9}, {NULL}};
const Segment SGI99145[] = {{&S71781[75], 6}, {NULL}};
const Segment SGI99146[] = {{&S86043[42], 10}, {NULL}};
const Segment SGI99147[] = {{&S86043[42], 22}, {NULL}};
const Segment SGI99148[] = {{&S86043[30], 4}, {NULL}};
const Segment SGI99149[] = {{&S86043[113], 3}, {NULL}};
const Segment SGI99150[] = {{&S86043[78], 6}, {NULL}};
const Segment SGI99151[] = {{&S86044[42], 10}, {NULL}};
const Segment SGI99152[] = {{&S86044[42], 22}, {NULL}};
const Segment SGI99153[] = {{&S86044[30], 4}, {NULL}};
const Segment SGI99154[] = {{&S86044[113], 3}, {NULL}};
const Segment SGI99155[] = {{&S86044[78], 6}, {NULL}};
const Segment SGI99156[] = {{&S94261[44], 10}, {NULL}};
const Segment SGI99157[] = {{&S94261[44], 27}, {NULL}};
const Segment SGI99158[] = {{&S94261[30], 6}, {NULL}};
const Segment SGI99159[] = {{&S94261[163], 3}, {NULL}};
const Segment SGI99160[] = {{&S94261[129], 3}, {NULL}};
const Segment SGI99161[] = {{&S94261[94], 6}, {NULL}};
const Segment SGI99162[] = {{&S94261[149], 8}, {NULL}};
const Segment SGI99163[] = {{&S86929[44], 16}, {NULL}};
const Segment SGI99164[] = {{&S86929[44], 26}, {NULL}};
const Segment SGI99165[] = {{&S86929[30], 6}, {NULL}};
const Segment SGI99166[] = {{&S86929[118], 3}, {NULL}};
const Segment SGI99167[] = {{&S86929[84], 5}, {NULL}};
const Segment SGI99168[] = {{&S86930[44], 15}, {NULL}};
const Segment SGI99169[] = {{&S86930[44], 25}, {NULL}};
const Segment SGI99170[] = {{&S86930[30], 6}, {NULL}};
const Segment SGI99171[] = {{&S86930[117], 3}, {NULL}};
const Segment SGI99172[] = {{&S86930[83], 5}, {NULL}};
const Segment SGI99173[] = {{&S94280[44], 10}, {NULL}};
const Segment SGI99174[] = {{&S94280[44], 27}, {NULL}};
const Segment SGI99175[] = {{&S94280[30], 6}, {NULL}};
const Segment SGI99176[] = {{&S94280[85], 6}, {NULL}};
const Segment SGI99177[] = {{&S94280[112], 13}, {NULL}};
const Segment SGI99178[] = {{&S71831[44], 7}, {NULL}};
const Segment SGI99179[] = {{&S71831[44], 19}, {NULL}};
const Segment SGI99180[] = {{&S71831[30], 6}, {NULL}};
const Segment SGI99181[] = {{&S71831[31], 3}, {NULL}};
const Segment SGI99182[] = {{&S71831[77], 6}, {NULL}};
const Segment SGI99183[] = {{&S86959[45], 7}, {NULL}};
const Segment SGI99184[] = {{&S86959[30], 6}, {NULL}};
const Segment SGI99185[] = {{&S86959[117], 3}, {NULL}};
const Segment SGI99186[] = {{&S86959[83], 5}, {NULL}};
const Segment SGI99187[] = {{&S86959[141], 13}, {NULL}};
const Segment SGI99188[] = {{&S94299[44], 10}, {NULL}};
const Segment SGI99189[] = {{&S94299[44], 27}, {NULL}};
const Segment SGI99190[] = {{&S94299[30], 6}, {NULL}};
const Segment SGI99191[] = {{&S94299[120], 3}, {NULL}};
const Segment SGI99192[] = {{&S94299[85], 6}, {NULL}};
const Segment SGI99193[] = {{&S94299[140], 8}, {NULL}};
const Segment SGI99194[] = {{&S94299[149], 13}, {NULL}};
const Segment SGI99195[] = {{&S94310[34], 9}, {NULL}};
const Segment SGI99196[] = {{&S94310[27], 6}, {NULL}};
const Segment SGI99197[] = {{&S94310[103], 2}, {NULL}};
const Segment SGI99198[] = {{&S94310[69], 6}, {NULL}};
const Segment SGI99199[] = {{&S94310[137], 8}, {NULL}};
const Segment SGI99200[] = {{&S94313[59], 10}, {NULL}};
const Segment SGI99201[] = {{&S94313[59], 27}, {NULL}};
const Segment SGI99202[] = {{&S94313[45], 6}, {NULL}};
const Segment SGI99203[] = {{&S94313[134], 3}, {NULL}};
const Segment SGI99204[] = {{&S94313[100], 5}, {NULL}};
const Segment SGI99205[] = {{&S94313[0], 14}, {NULL}};
const Segment SGI99206[] = {{&S71875[34], 10}, {NULL}};
const Segment SGI99207[] = {{&S71875[27], 6}, {NULL}};
const Segment SGI99208[] = {{&S94319[34], 10}, {NULL}};
const Segment SGI99209[] = {{&S94319[27], 6}, {NULL}};
const Segment SGI99210[] = {{&S71875[97], 9}, {NULL}};
const Segment SGI99211[] = {{&S71875[70], 6}, {NULL}};
const Segment SGI99212[] = {{&S94319[109], 2}, {NULL}};
const Segment SGI99213[] = {{&S94319[75], 6}, {NULL}};
const Segment SGI99214[] = {{&S94319[143], 8}, {NULL}};
const Segment SGI99215[] = {{&S71878[34], 11}, {NULL}};
const Segment SGI99216[] = {{&S71878[27], 6}, {NULL}};
const Segment SGI99217[] = {{&S71878[98], 9}, {NULL}};
const Segment SGI99218[] = {{&S71878[71], 6}, {NULL}};
const Segment SGI99219[] = {{&S71878[139], 8}, {NULL}};
const Segment SGI99220[] = {{&S94326[44], 10}, {NULL}};
const Segment SGI99221[] = {{&S94326[44], 27}, {NULL}};
const Segment SGI99222[] = {{&S94326[30], 6}, {NULL}};
const Segment SGI99223[] = {{&S94326[118], 3}, {NULL}};
const Segment SGI99224[] = {{&S94326[85], 5}, {NULL}};
const Segment SGI99225[] = {{&S94326[122], 14}, {NULL}};
const Segment SGI99226[] = {{&S94368[45], 13}, {NULL}};
const Segment SGI99227[] = {{&S94368[30], 6}, {NULL}};
const Segment SGI99228[] = {{&S94368[127], 3}, {NULL}};
const Segment SGI99229[] = {{&S94368[93], 5}, {NULL}};
const Segment SGI99230[] = {{&S94368[151], 13}, {NULL}};
const Segment SGI99231[] = {{&S94385[44], 10}, {NULL}};
const Segment SGI99232[] = {{&S94385[44], 27}, {NULL}};
const Segment SGI99233[] = {{&S94385[30], 6}, {NULL}};
const Segment SGI99234[] = {{&S94385[85], 6}, {NULL}};
const Segment SGI99235[] = {{&S94385[112], 13}, {NULL}};
const Segment SGI99236[] = {{&S87069[44], 9}, {NULL}};
const Segment SGI99237[] = {{&S87069[44], 20}, {NULL}};
const Segment SGI99238[] = {{&S87069[30], 6}, {NULL}};
const Segment SGI99239[] = {{&S87069[112], 3}, {NULL}};
const Segment SGI99240[] = {{&S87069[78], 5}, {NULL}};
const Segment SGI99241[] = {{&S71989[34], 6}, {NULL}};
const Segment SGI99242[] = {{&S71989[27], 6}, {NULL}};
const Segment SGI99243[] = {{&S71989[93], 9}, {NULL}};
const Segment SGI99244[] = {{&S71989[66], 6}, {NULL}};
const Segment SGI99245[] = {{&S71999[44], 11}, {NULL}};
const Segment SGI99246[] = {{&S71999[44], 23}, {NULL}};
const Segment SGI99247[] = {{&S71999[30], 6}, {NULL}};
const Segment SGI99248[] = {{&S71999[84], 6}, {NULL}};
const Segment SGI99249[] = {{&S72000[44], 11}, {NULL}};
const Segment SGI99250[] = {{&S72000[44], 23}, {NULL}};
const Segment SGI99251[] = {{&S72000[30], 6}, {NULL}};
const Segment SGI99252[] = {{&S72000[31], 3}, {NULL}};
const Segment SGI99253[] = {{&S72000[81], 6}, {NULL}};
const Segment SGI99254[] = {{&S72001[44], 11}, {NULL}};
const Segment SGI99255[] = {{&S72001[44], 23}, {NULL}};
const Segment SGI99256[] = {{&S72001[30], 6}, {NULL}};
const Segment SGI99257[] = {{&S72001[84], 4}, {NULL}};
const Segment SGI99258[] = {{&S72001[84], 3}, {NULL}};
const Segment SGI99259[] = {{&S94449[57], 10}, {NULL}};
const Segment SGI99260[] = {{&S94449[57], 27}, {NULL}};
const Segment SGI99261[] = {{&S94449[43], 6}, {NULL}};
const Segment SGI99262[] = {{&S72002[38], 11}, {NULL}};
const Segment SGI99263[] = {{&S72002[31], 6}, {NULL}};
const Segment SGI99264[] = {{&S94449[132], 3}, {NULL}};
const Segment SGI99265[] = {{&S94449[98], 5}, {NULL}};
const Segment SGI99266[] = {{&S94449[0], 12}, {NULL}};
const Segment SGI99267[] = {{&S72003[44], 23}, {NULL}};
const Segment SGI99268[] = {{&S72003[30], 6}, {NULL}};
const Segment SGI99269[] = {{&S94450[59], 10}, {NULL}};
const Segment SGI99270[] = {{&S94450[59], 27}, {NULL}};
const Segment SGI99271[] = {{&S94450[45], 6}, {NULL}};
const Segment SGI99272[] = {{&S94450[134], 3}, {NULL}};
const Segment SGI99273[] = {{&S94450[100], 5}, {NULL}};
const Segment SGI99274[] = {{&S94450[0], 14}, {NULL}};
const Segment SGI99275[] = {{&S72040[34], 9}, {NULL}};
const Segment SGI99276[] = {{&S72040[27], 6}, {NULL}};
const Segment SGI99277[] = {{&S72040[96], 9}, {NULL}};
const Segment SGI99278[] = {{&S72040[69], 6}, {NULL}};
const Segment SGI99279[] = {{&S94487[59], 7}, {NULL}};
const Segment SGI99280[] = {{&S94487[59], 19}, {NULL}};
const Segment SGI99281[] = {{&S94487[45], 6}, {NULL}};
const Segment SGI99282[] = {{&S94487[126], 3}, {NULL}};
const Segment SGI99283[] = {{&S94487[92], 5}, {NULL}};
const Segment SGI99284[] = {{&S94487[0], 14}, {NULL}};
const Segment SGI99285[] = {{&S72069[34], 5}, {NULL}};
const Segment SGI99286[] = {{&S72069[27], 6}, {NULL}};
const Segment SGI99287[] = {{&S72069[92], 9}, {NULL}};
const Segment SGI99288[] = {{&S72069[65], 6}, {NULL}};
const Segment SGI99289[] = {{&S72070[34], 9}, {NULL}};
const Segment SGI99290[] = {{&S72070[27], 6}, {NULL}};
const Segment SGI99291[] = {{&S72070[96], 9}, {NULL}};
const Segment SGI99292[] = {{&S72070[69], 6}, {NULL}};
const Segment SGI99293[] = {{&S94529[45], 5}, {NULL}};
const Segment SGI99294[] = {{&S94529[30], 6}, {NULL}};
const Segment SGI99295[] = {{&S94529[119], 3}, {NULL}};
const Segment SGI99296[] = {{&S94529[85], 5}, {NULL}};
const Segment SGI99297[] = {{&S94529[143], 13}, {NULL}};
const Segment SGI99298[] = {{&S72087[44], 19}, {NULL}};
const Segment SGI99299[] = {{&S72087[44], 31}, {NULL}};
const Segment SGI99300[] = {{&S72087[30], 6}, {NULL}};
const Segment SGI99301[] = {{&S72087[120], 2}, {NULL}};
const Segment SGI99302[] = {{&S72087[89], 5}, {NULL}};
const Segment SGI99303[] = {{&S72104[34], 11}, {NULL}};
const Segment SGI99304[] = {{&S72104[27], 6}, {NULL}};
const Segment SGI99305[] = {{&S72104[98], 9}, {NULL}};
const Segment SGI99306[] = {{&S72104[71], 6}, {NULL}};
const Segment SGI99307[] = {{&S72129[44], 11}, {NULL}};
const Segment SGI99308[] = {{&S72129[44], 23}, {NULL}};
const Segment SGI99309[] = {{&S72129[30], 6}, {NULL}};
const Segment SGI99310[] = {{&S72129[31], 3}, {NULL}};
const Segment SGI99311[] = {{&S72129[81], 6}, {NULL}};
const Segment SGI99312[] = {{&S72130[38], 11}, {NULL}};
const Segment SGI99313[] = {{&S94571[44], 10}, {NULL}};
const Segment SGI99314[] = {{&S72130[31], 6}, {NULL}};
const Segment SGI99315[] = {{&S94571[44], 27}, {NULL}};
const Segment SGI99316[] = {{&S94571[30], 6}, {NULL}};
const Segment SGI99317[] = {{&S94571[119], 3}, {NULL}};
const Segment SGI99318[] = {{&S94571[85], 5}, {NULL}};
const Segment SGI99319[] = {{&S94571[143], 7}, {NULL}};
const Segment SGI99320[] = {{&S72145[34], 10}, {NULL}};
const Segment SGI99321[] = {{&S72145[27], 6}, {NULL}};
const Segment SGI99322[] = {{&S72145[97], 9}, {NULL}};
const Segment SGI99323[] = {{&S72145[70], 6}, {NULL}};
const Segment SGI99324[] = {{&S72145[138], 8}, {NULL}};
const Segment SGI99325[] = {{&S72159[45], 11}, {NULL}};
const Segment SGI99326[] = {{&S72159[30], 6}, {NULL}};
const Segment SGI99327[] = {{&S72159[31], 3}, {NULL}};
const Segment SGI99328[] = {{&S72159[82], 5}, {NULL}};
const Segment SGI99329[] = {{&S72159[139], 13}, {NULL}};
const Segment SGI99330[] = {{&S94604[44], 10}, {NULL}};
const Segment SGI99331[] = {{&S94604[44], 22}, {NULL}};
const Segment SGI99332[] = {{&S94604[30], 6}, {NULL}};
const Segment SGI99333[] = {{&S94604[80], 6}, {NULL}};
const Segment SGI99334[] = {{&S94604[107], 13}, {NULL}};
const Segment SGI99335[] = {{&S72167[34], 13}, {NULL}};
const Segment SGI99336[] = {{&S72167[27], 6}, {NULL}};
const Segment SGI99337[] = {{&S72167[100], 9}, {NULL}};
const Segment SGI99338[] = {{&S94606[44], 10}, {NULL}};
const Segment SGI99339[] = {{&S72167[73], 6}, {NULL}};
const Segment SGI99340[] = {{&S94606[44], 27}, {NULL}};
const Segment SGI99341[] = {{&S94606[30], 6}, {NULL}};
const Segment SGI99342[] = {{&S94606[118], 3}, {NULL}};
const Segment SGI99343[] = {{&S94606[85], 5}, {NULL}};
const Segment SGI99344[] = {{&S94606[122], 14}, {NULL}};
const Segment SGI99345[] = {{&S94618[44], 14}, {NULL}};
const Segment SGI99346[] = {{&S94618[44], 26}, {NULL}};
const Segment SGI99347[] = {{&S94618[30], 6}, {NULL}};
const Segment SGI99348[] = {{&S94618[84], 6}, {NULL}};
const Segment SGI99349[] = {{&S94618[111], 13}, {NULL}};
const Segment SGI99350[] = {{&S86285[42], 10}, {NULL}};
const Segment SGI99351[] = {{&S86285[42], 22}, {NULL}};
const Segment SGI99352[] = {{&S86285[30], 4}, {NULL}};
const Segment SGI99353[] = {{&S86285[113], 3}, {NULL}};
const Segment SGI99354[] = {{&S86285[78], 6}, {NULL}};
const Segment SGI99355[] = {{&S94674[30], 6}, {NULL}};
const Segment SGI99356[] = {{&S94674[91], 3}, {NULL}};
const Segment SGI99357[] = {{&S94674[125], 3}, {NULL}};
const Segment SGI99358[] = {{&S94674[57], 5}, {NULL}};
const Segment SGI99359[] = {{&S86337[29], 8}, {NULL}};
const Segment SGI99360[] = {{&S86337[52], 3}, {NULL}};
const Segment SGI99361[] = {{&S86338[37], 8}, {NULL}};
const Segment SGI99362[] = {{&S86338[101], 3}, {NULL}};
const Segment SGI99363[] = {{&S86338[90], 3}, {NULL}};
const Segment SGI99364[] = {{&S86338[67], 6}, {NULL}};
const Segment SGI99365[] = {{&S94690[59], 10}, {NULL}};
const Segment SGI99366[] = {{&S94690[59], 27}, {NULL}};
const Segment SGI99367[] = {{&S94690[45], 6}, {NULL}};
const Segment SGI99368[] = {{&S94690[134], 3}, {NULL}};
const Segment SGI99369[] = {{&S94690[100], 5}, {NULL}};
const Segment SGI99370[] = {{&S94690[0], 14}, {NULL}};
const Segment SGI99371[] = {{&S94693[45], 9}, {NULL}};
const Segment SGI99372[] = {{&S94693[30], 6}, {NULL}};
const Segment SGI99373[] = {{&S94693[114], 3}, {NULL}};
const Segment SGI99374[] = {{&S94693[80], 5}, {NULL}};
const Segment SGI99375[] = {{&S94693[138], 13}, {NULL}};
const Segment SGI99376[] = {{&S94697[31], 7}, {NULL}};
const Segment SGI99377[] = {{&S94697[53], 3}, {NULL}};
const Segment SGI99378[] = {{&S72288[34], 7}, {NULL}};
const Segment SGI99379[] = {{&S72288[27], 6}, {NULL}};
const Segment SGI99380[] = {{&S72288[99], 9}, {NULL}};
const Segment SGI99381[] = {{&S72288[72], 6}, {NULL}};
const Segment SGI99382[] = {{&S72307[34], 6}, {NULL}};
const Segment SGI99383[] = {{&S72307[27], 6}, {NULL}};
const Segment SGI99384[] = {{&S72307[93], 9}, {NULL}};
const Segment SGI99385[] = {{&S72307[66], 6}, {NULL}};
const Segment SGI99386[] = {{&S86398[34], 17}, {NULL}};
const Segment SGI99387[] = {{&S86398[27], 6}, {NULL}};
const Segment SGI99388[] = {{&S86398[111], 2}, {NULL}};
const Segment SGI99389[] = {{&S86398[77], 6}, {NULL}};
const Segment SGI99390[] = {{&S86398[138], 8}, {NULL}};
const Segment SGI99391[] = {{&S72315[34], 7}, {NULL}};
const Segment SGI99392[] = {{&S72315[27], 6}, {NULL}};
const Segment SGI99393[] = {{&S72315[94], 9}, {NULL}};
const Segment SGI99394[] = {{&S72315[67], 6}, {NULL}};
const Segment SGI99395[] = {{&S94756[59], 5}, {NULL}};
const Segment SGI99396[] = {{&S94756[59], 17}, {NULL}};
const Segment SGI99397[] = {{&S94756[45], 6}, {NULL}};
const Segment SGI99398[] = {{&S94756[124], 3}, {NULL}};
const Segment SGI99399[] = {{&S94756[90], 5}, {NULL}};
const Segment SGI99400[] = {{&S94756[0], 14}, {NULL}};
const Segment SGI99401[] = {{&S72336[34], 7}, {NULL}};
const Segment SGI99402[] = {{&S72336[27], 6}, {NULL}};
const Segment SGI99403[] = {{&S72336[99], 9}, {NULL}};
const Segment SGI99404[] = {{&S72336[72], 6}, {NULL}};
const Segment SGI99405[] = {{&S72348[44], 6}, {NULL}};
const Segment SGI99406[] = {{&S72348[44], 18}, {NULL}};
const Segment SGI99407[] = {{&S72348[30], 6}, {NULL}};
const Segment SGI99408[] = {{&S72348[31], 3}, {NULL}};
const Segment SGI99409[] = {{&S72348[76], 6}, {NULL}};
const Segment SGI99410[] = {{&S72349[38], 6}, {NULL}};
const Segment SGI99411[] = {{&S72349[31], 6}, {NULL}};
const Segment SGI99412[] = {{&S72355[34], 13}, {NULL}};
const Segment SGI99413[] = {{&S72355[27], 6}, {NULL}};
const Segment SGI99414[] = {{&S72355[97], 9}, {NULL}};
const Segment SGI99415[] = {{&S72355[70], 6}, {NULL}};
const Segment SGI99416[] = {{&S72357[44], 14}, {NULL}};
const Segment SGI99417[] = {{&S72357[44], 26}, {NULL}};
const Segment SGI99418[] = {{&S72357[30], 6}, {NULL}};
const Segment SGI99419[] = {{&S72357[84], 6}, {NULL}};
const Segment SGI99420[] = {{&S72357[111], 13}, {NULL}};
const Segment SGI99421[] = {{&S94817[34], 10}, {NULL}};
const Segment SGI99422[] = {{&S94817[27], 6}, {NULL}};
const Segment SGI99423[] = {{&S94817[109], 2}, {NULL}};
const Segment SGI99424[] = {{&S94817[75], 6}, {NULL}};
const Segment SGI99425[] = {{&S94817[143], 8}, {NULL}};
const Segment SGI99426[] = {{&S94822[44], 10}, {NULL}};
const Segment SGI99427[] = {{&S94822[44], 27}, {NULL}};
const Segment SGI99428[] = {{&S94822[30], 6}, {NULL}};
const Segment SGI99429[] = {{&S94822[119], 3}, {NULL}};
const Segment SGI99430[] = {{&S94822[85], 5}, {NULL}};
const Segment SGI99431[] = {{&S94822[143], 7}, {NULL}};
const Segment SGI99432[] = {{&S72386[34], 17}, {NULL}};
const Segment SGI99433[] = {{&S72386[27], 6}, {NULL}};
const Segment SGI99434[] = {{&S72386[112], 9}, {NULL}};
const Segment SGI99435[] = {{&S72386[85], 6}, {NULL}};
const Segment SGI99436[] = {{&S72387[34], 13}, {NULL}};
const Segment SGI99437[] = {{&S72387[27], 6}, {NULL}};
const Segment SGI99438[] = {{&S72387[108], 9}, {NULL}};
const Segment SGI99439[] = {{&S72387[81], 6}, {NULL}};
const Segment SGI99440[] = {{&S72388[34], 17}, {NULL}};
const Segment SGI99441[] = {{&S72388[27], 6}, {NULL}};
const Segment SGI99442[] = {{&S72388[112], 9}, {NULL}};
const Segment SGI99443[] = {{&S72388[85], 6}, {NULL}};
const Segment SGI99444[] = {{&S87436[44], 6}, {NULL}};
const Segment SGI99445[] = {{&S87436[44], 17}, {NULL}};
const Segment SGI99446[] = {{&S87436[30], 6}, {NULL}};
const Segment SGI99447[] = {{&S87436[128], 3}, {NULL}};
const Segment SGI99448[] = {{&S87436[94], 5}, {NULL}};
const Segment SGI99449[] = {{&S87436[35], 1}, {NULL}};
const Segment SGI99450[] = {{&S87436[63], 11}, {NULL}};
const Segment SGI99451[] = {{&S87438[44], 10}, {NULL}};
const Segment SGI99452[] = {{&S87438[44], 20}, {NULL}};
const Segment SGI99453[] = {{&S87438[30], 6}, {NULL}};
const Segment SGI99454[] = {{&S87438[131], 3}, {NULL}};
const Segment SGI99455[] = {{&S87438[97], 5}, {NULL}};
const Segment SGI99456[] = {{&S87438[35], 1}, {NULL}};
const Segment SGI99457[] = {{&S87438[66], 11}, {NULL}};
const Segment SGI99458[] = {{&S87439[44], 10}, {NULL}};
const Segment SGI99459[] = {{&S87439[44], 20}, {NULL}};
const Segment SGI99460[] = {{&S87439[30], 6}, {NULL}};
const Segment SGI99461[] = {{&S87439[112], 3}, {NULL}};
const Segment SGI99462[] = {{&S87439[78], 5}, {NULL}};
const Segment SGI99463[] = {{&S72389[34], 29}, {NULL}};
const Segment SGI99464[] = {{&S72389[27], 6}, {NULL}};
const Segment SGI99465[] = {{&S72389[116], 9}, {NULL}};
const Segment SGI99466[] = {{&S72389[89], 6}, {NULL}};
const Segment SGI99467[] = {{&S72408[44], 10}, {NULL}};
const Segment SGI99468[] = {{&S72408[44], 30}, {NULL}};
const Segment SGI99469[] = {{&S72408[30], 6}, {NULL}};
const Segment SGI99470[] = {{&S72408[31], 3}, {NULL}};
const Segment SGI99471[] = {{&S72408[88], 6}, {NULL}};
const Segment SGI99472[] = {{&S72432[44], 10}, {NULL}};
const Segment SGI99473[] = {{&S72432[44], 22}, {NULL}};
const Segment SGI99474[] = {{&S72432[30], 6}, {NULL}};
const Segment SGI99475[] = {{&S72432[80], 6}, {NULL}};
const Segment SGI99476[] = {{&S72432[107], 13}, {NULL}};
const Segment SGI99477[] = {{&S80963[73], 6}, {NULL}};
const Segment SGI99478[] = {{&S80963[0], 8}, {NULL}};
const Segment SGI99479[] = {{&S72462[45], 10}, {NULL}};
const Segment SGI99480[] = {{&S72462[30], 6}, {NULL}};
const Segment SGI99481[] = {{&S72462[31], 3}, {NULL}};
const Segment SGI99482[] = {{&S72462[81], 5}, {NULL}};
const Segment SGI99483[] = {{&S72462[139], 13}, {NULL}};
const Segment SGI99484[] = {{&S84018[42], 14}, {NULL}};
const Segment SGI99485[] = {{&S84018[42], 26}, {NULL}};
const Segment SGI99486[] = {{&S84018[30], 4}, {NULL}};
const Segment SGI99487[] = {{&S84018[31], 3}, {NULL}};
const Segment SGI99488[] = {{&S84018[82], 6}, {NULL}};
const Segment SGI99489[] = {{&S72480[44], 8}, {NULL}};
const Segment SGI99490[] = {{&S72480[44], 20}, {NULL}};
const Segment SGI99491[] = {{&S72480[30], 6}, {NULL}};
const Segment SGI99492[] = {{&S72480[31], 3}, {NULL}};
const Segment SGI99493[] = {{&S72480[78], 6}, {NULL}};
const Segment SGI99494[] = {{&S94946[59], 5}, {NULL}};
const Segment SGI99495[] = {{&S94946[59], 17}, {NULL}};
const Segment SGI99496[] = {{&S94946[45], 6}, {NULL}};
const Segment SGI99497[] = {{&S94946[124], 3}, {NULL}};
const Segment SGI99498[] = {{&S94946[90], 5}, {NULL}};
const Segment SGI99499[] = {{&S94946[0], 14}, {NULL}};
const Segment SGI99500[] = {{&S94960[59], 5}, {NULL}};
const Segment SGI99501[] = {{&S94960[59], 17}, {NULL}};
const Segment SGI99502[] = {{&S87562[44], 7}, {NULL}};
const Segment SGI99503[] = {{&S94960[45], 6}, {NULL}};
const Segment SGI99504[] = {{&S87562[44], 18}, {NULL}};
const Segment SGI99505[] = {{&S87562[30], 6}, {NULL}};
const Segment SGI99506[] = {{&S94960[124], 3}, {NULL}};
const Segment SGI99507[] = {{&S87562[129], 3}, {NULL}};
const Segment SGI99508[] = {{&S94960[90], 5}, {NULL}};
const Segment SGI99509[] = {{&S87562[95], 5}, {NULL}};
const Segment SGI99510[] = {{&S94960[0], 14}, {NULL}};
const Segment SGI99511[] = {{&S87562[35], 1}, {NULL}};
const Segment SGI99512[] = {{&S87562[64], 11}, {NULL}};
const Segment SGI99513[] = {{&S87563[44], 7}, {NULL}};
const Segment SGI99514[] = {{&S87563[44], 19}, {NULL}};
const Segment SGI99515[] = {{&S87563[30], 6}, {NULL}};
const Segment SGI99516[] = {{&S87563[126], 3}, {NULL}};
const Segment SGI99517[] = {{&S87563[92], 5}, {NULL}};
const Segment SGI99518[] = {{&S87563[35], 1}, {NULL}};
const Segment SGI99519[] = {{&S87563[65], 11}, {NULL}};
const Segment SGI99520[] = {{&S94978[59], 10}, {NULL}};
const Segment SGI99521[] = {{&S94978[59], 27}, {NULL}};
const Segment SGI99522[] = {{&S94978[45], 6}, {NULL}};
const Segment SGI99523[] = {{&S94978[134], 3}, {NULL}};
const Segment SGI99524[] = {{&S94978[100], 5}, {NULL}};
const Segment SGI99525[] = {{&S94978[0], 14}, {NULL}};
const Segment SGI99526[] = {{&S72542[45], 10}, {NULL}};
const Segment SGI99527[] = {{&S72542[30], 6}, {NULL}};
const Segment SGI99528[] = {{&S72542[31], 3}, {NULL}};
const Segment SGI99529[] = {{&S72542[81], 5}, {NULL}};
const Segment SGI99530[] = {{&S72542[139], 13}, {NULL}};
const Segment SGI99531[] = {{&S72547[34], 17}, {NULL}};
const Segment SGI99532[] = {{&S72547[27], 6}, {NULL}};
const Segment SGI99533[] = {{&S72547[120], 9}, {NULL}};
const Segment SGI99534[] = {{&S72547[93], 6}, {NULL}};
const Segment SGI99535[] = {{&S72598[44], 6}, {NULL}};
const Segment SGI99536[] = {{&S72598[44], 18}, {NULL}};
const Segment SGI99537[] = {{&S72598[30], 6}, {NULL}};
const Segment SGI99538[] = {{&S72598[107], 2}, {NULL}};
const Segment SGI99539[] = {{&S72598[76], 5}, {NULL}};
const Segment SGI99540[] = {{&S72615[44], 11}, {NULL}};
const Segment SGI99541[] = {{&S72615[44], 23}, {NULL}};
const Segment SGI99542[] = {{&S72615[30], 6}, {NULL}};
const Segment SGI99543[] = {{&S72615[31], 3}, {NULL}};
const Segment SGI99544[] = {{&S72615[81], 6}, {NULL}};
const Segment SGI99545[] = {{&S72616[38], 11}, {NULL}};
const Segment SGI99546[] = {{&S72616[31], 6}, {NULL}};
const Segment SGI99547[] = {{&S95007[44], 10}, {NULL}};
const Segment SGI99548[] = {{&S95007[44], 27}, {NULL}};
const Segment SGI99549[] = {{&S95007[30], 6}, {NULL}};
const Segment SGI99550[] = {{&S95007[118], 3}, {NULL}};
const Segment SGI99551[] = {{&S72630[34], 9}, {NULL}};
const Segment SGI99552[] = {{&S95007[85], 5}, {NULL}};
const Segment SGI99553[] = {{&S72630[27], 6}, {NULL}};
const Segment SGI99554[] = {{&S95007[122], 14}, {NULL}};
const Segment SGI99555[] = {{&S72630[96], 9}, {NULL}};
const Segment SGI99556[] = {{&S72630[69], 6}, {NULL}};
const Segment SGI99557[] = {{&S95048[44], 11}, {NULL}};
const Segment SGI99558[] = {{&S95048[44], 23}, {NULL}};
const Segment SGI99559[] = {{&S95048[30], 6}, {NULL}};
const Segment SGI99560[] = {{&S95048[114], 3}, {NULL}};
const Segment SGI99561[] = {{&S95048[81], 5}, {NULL}};
const Segment SGI99562[] = {{&S95048[118], 14}, {NULL}};
const Segment SGI99563[] = {{&S95049[44], 10}, {NULL}};
const Segment SGI99564[] = {{&S95049[44], 27}, {NULL}};
const Segment SGI99565[] = {{&S95049[30], 6}, {NULL}};
const Segment SGI99566[] = {{&S95049[118], 3}, {NULL}};
const Segment SGI99567[] = {{&S95049[85], 5}, {NULL}};
const Segment SGI99568[] = {{&S95049[122], 14}, {NULL}};
const Segment SGI99569[] = {{&S95052[44], 5}, {NULL}};
const Segment SGI99570[] = {{&S95052[44], 17}, {NULL}};
const Segment SGI99571[] = {{&S95052[30], 6}, {NULL}};
const Segment SGI99572[] = {{&S95052[75], 6}, {NULL}};
const Segment SGI99573[] = {{&S95052[102], 13}, {NULL}};
const Segment SGI99574[] = {{&S84280[42], 15}, {NULL}};
const Segment SGI99575[] = {{&S84280[42], 26}, {NULL}};
const Segment SGI99576[] = {{&S84280[30], 4}, {NULL}};
const Segment SGI99577[] = {{&S84280[31], 3}, {NULL}};
const Segment SGI99578[] = {{&S84280[82], 5}, {NULL}};
const Segment SGI99579[] = {{&S95064[44], 10}, {NULL}};
const Segment SGI99580[] = {{&S95064[44], 27}, {NULL}};
const Segment SGI99581[] = {{&S95064[30], 6}, {NULL}};
const Segment SGI99582[] = {{&S95064[118], 3}, {NULL}};
const Segment SGI99583[] = {{&S95064[85], 5}, {NULL}};
const Segment SGI99584[] = {{&S95064[122], 14}, {NULL}};
const Segment SGI99585[] = {{&S95067[44], 10}, {NULL}};
const Segment SGI99586[] = {{&S95067[44], 27}, {NULL}};
const Segment SGI99587[] = {{&S95067[30], 6}, {NULL}};
const Segment SGI99588[] = {{&S95067[85], 6}, {NULL}};
const Segment SGI99589[] = {{&S95067[112], 13}, {NULL}};
const Segment SGI99590[] = {{&S72699[44], 10}, {NULL}};
const Segment SGI99591[] = {{&S72699[44], 22}, {NULL}};
const Segment SGI99592[] = {{&S72699[30], 6}, {NULL}};
const Segment SGI99593[] = {{&S72699[31], 3}, {NULL}};
const Segment SGI99594[] = {{&S72699[80], 6}, {NULL}};
const Segment SGI99595[] = {{&S72700[44], 11}, {NULL}};
const Segment SGI99596[] = {{&S72700[44], 23}, {NULL}};
const Segment SGI99597[] = {{&S72700[30], 6}, {NULL}};
const Segment SGI99598[] = {{&S72700[31], 3}, {NULL}};
const Segment SGI99599[] = {{&S72700[81], 6}, {NULL}};
const Segment SGI99600[] = {{&S72701[44], 10}, {NULL}};
const Segment SGI99601[] = {{&S72701[44], 22}, {NULL}};
const Segment SGI99602[] = {{&S72701[30], 6}, {NULL}};
const Segment SGI99603[] = {{&S72701[31], 3}, {NULL}};
const Segment SGI99604[] = {{&S72701[80], 6}, {NULL}};
const Segment SGI99605[] = {{&S72703[44], 14}, {NULL}};
const Segment SGI99606[] = {{&S72703[44], 26}, {NULL}};
const Segment SGI99607[] = {{&S72703[30], 6}, {NULL}};
const Segment SGI99608[] = {{&S72703[31], 3}, {NULL}};
const Segment SGI99609[] = {{&S72703[84], 6}, {NULL}};
const Segment SGI99610[] = {{&S95078[34], 10}, {NULL}};
const Segment SGI99611[] = {{&S95078[27], 6}, {NULL}};
const Segment SGI99612[] = {{&S95078[109], 2}, {NULL}};
const Segment SGI99613[] = {{&S95078[75], 6}, {NULL}};
const Segment SGI99614[] = {{&S95078[143], 8}, {NULL}};
const Segment SGI99615[] = {{&S72738[44], 16}, {NULL}};
const Segment SGI99616[] = {{&S72738[44], 28}, {NULL}};
const Segment SGI99617[] = {{&S72738[30], 6}, {NULL}};
const Segment SGI99618[] = {{&S72738[31], 3}, {NULL}};
const Segment SGI99619[] = {{&S72738[86], 6}, {NULL}};
const Segment SGI99620[] = {{&S72747[44], 9}, {NULL}};
const Segment SGI99621[] = {{&S72747[44], 21}, {NULL}};
const Segment SGI99622[] = {{&S72747[30], 6}, {NULL}};
const Segment SGI99623[] = {{&S72747[31], 3}, {NULL}};
const Segment SGI99624[] = {{&S72747[79], 6}, {NULL}};
const Segment SGI99625[] = {{&S87798[44], 8}, {NULL}};
const Segment SGI99626[] = {{&S87798[44], 18}, {NULL}};
const Segment SGI99627[] = {{&S87798[30], 6}, {NULL}};
const Segment SGI99628[] = {{&S87798[110], 3}, {NULL}};
const Segment SGI99629[] = {{&S87798[76], 5}, {NULL}};
const Segment SGI99630[] = {{&S87801[44], 12}, {NULL}};
const Segment SGI99631[] = {{&S87801[44], 22}, {NULL}};
const Segment SGI99632[] = {{&S87801[30], 6}, {NULL}};
const Segment SGI99633[] = {{&S87801[114], 3}, {NULL}};
const Segment SGI99634[] = {{&S87801[80], 5}, {NULL}};
const Segment SGI99635[] = {{&S72756[34], 18}, {NULL}};
const Segment SGI99636[] = {{&S72756[27], 6}, {NULL}};
const Segment SGI99637[] = {{&S72756[105], 9}, {NULL}};
const Segment SGI99638[] = {{&S72756[78], 6}, {NULL}};
const Segment SGI99639[] = {{&S72756[146], 8}, {NULL}};
const Segment SGI99640[] = {{&S72757[34], 18}, {NULL}};
const Segment SGI99641[] = {{&S72757[27], 6}, {NULL}};
const Segment SGI99642[] = {{&S72757[105], 9}, {NULL}};
const Segment SGI99643[] = {{&S72757[78], 6}, {NULL}};
const Segment SGI99644[] = {{&S72757[146], 8}, {NULL}};
const Segment SGI99645[] = {{&S95164[45], 10}, {NULL}};
const Segment SGI99646[] = {{&S95164[30], 6}, {NULL}};
const Segment SGI99647[] = {{&S95164[120], 3}, {NULL}};
const Segment SGI99648[] = {{&S95164[86], 5}, {NULL}};
const Segment SGI99649[] = {{&S95164[144], 13}, {NULL}};
const Segment SGI99650[] = {{&S72784[44], 8}, {NULL}};
const Segment SGI99651[] = {{&S72784[44], 20}, {NULL}};
const Segment SGI99652[] = {{&S72784[30], 6}, {NULL}};
const Segment SGI99653[] = {{&S72784[31], 3}, {NULL}};
const Segment SGI99654[] = {{&S72784[78], 6}, {NULL}};
const Segment SGI99655[] = {{&S81365[74], 6}, {NULL}};
const Segment SGI99656[] = {{&S81365[0], 9}, {NULL}};
const Segment SGI99657[] = {{&S72797[34], 11}, {NULL}};
const Segment SGI99658[] = {{&S72797[27], 6}, {NULL}};
const Segment SGI99659[] = {{&S72797[98], 9}, {NULL}};
const Segment SGI99660[] = {{&S72797[71], 6}, {NULL}};
const Segment SGI99661[] = {{&S95190[45], 10}, {NULL}};
const Segment SGI99662[] = {{&S95190[30], 6}, {NULL}};
const Segment SGI99663[] = {{&S95190[120], 3}, {NULL}};
const Segment SGI99664[] = {{&S95190[86], 5}, {NULL}};
const Segment SGI99665[] = {{&S95190[144], 13}, {NULL}};
const Segment SGI99666[] = {{&S95195[44], 14}, {NULL}};
const Segment SGI99667[] = {{&S95195[44], 26}, {NULL}};
const Segment SGI99668[] = {{&S95195[30], 6}, {NULL}};
const Segment SGI99669[] = {{&S95195[118], 3}, {NULL}};
const Segment SGI99670[] = {{&S95195[84], 5}, {NULL}};
const Segment SGI99671[] = {{&S95244[45], 7}, {NULL}};
const Segment SGI99672[] = {{&S95244[30], 6}, {NULL}};
const Segment SGI99673[] = {{&S95244[117], 3}, {NULL}};
const Segment SGI99674[] = {{&S95244[83], 5}, {NULL}};
const Segment SGI99675[] = {{&S95244[141], 13}, {NULL}};
const Segment SGI99676[] = {{&S95253[34], 10}, {NULL}};
const Segment SGI99677[] = {{&S95253[27], 6}, {NULL}};
const Segment SGI99678[] = {{&S95253[109], 2}, {NULL}};
const Segment SGI99679[] = {{&S95253[75], 6}, {NULL}};
const Segment SGI99680[] = {{&S95253[143], 8}, {NULL}};
const Segment SGI99681[] = {{&S95272[44], 5}, {NULL}};
const Segment SGI99682[] = {{&S95272[44], 17}, {NULL}};
const Segment SGI99683[] = {{&S95272[30], 6}, {NULL}};
const Segment SGI99684[] = {{&S95272[109], 3}, {NULL}};
const Segment SGI99685[] = {{&S95272[75], 5}, {NULL}};
const Segment SGI99686[] = {{&S87914[44], 10}, {NULL}};
const Segment SGI99687[] = {{&S87914[44], 21}, {NULL}};
const Segment SGI99688[] = {{&S87914[30], 6}, {NULL}};
const Segment SGI99689[] = {{&S87914[132], 3}, {NULL}};
const Segment SGI99690[] = {{&S87914[98], 5}, {NULL}};
const Segment SGI99691[] = {{&S87914[35], 1}, {NULL}};
const Segment SGI99692[] = {{&S87914[67], 11}, {NULL}};
const Segment SGI99693[] = {{&S87915[44], 7}, {NULL}};
const Segment SGI99694[] = {{&S87915[44], 37}, {NULL}};
const Segment SGI99695[] = {{&S87915[30], 6}, {NULL}};
const Segment SGI99696[] = {{&S87915[129], 3}, {NULL}};
const Segment SGI99697[] = {{&S87915[95], 5}, {NULL}};
const Segment SGI99698[] = {{&S95286[34], 18}, {NULL}};
const Segment SGI99699[] = {{&S95286[27], 6}, {NULL}};
const Segment SGI99700[] = {{&S87917[44], 7}, {NULL}};
const Segment SGI99701[] = {{&S95286[117], 2}, {NULL}};
const Segment SGI99702[] = {{&S87917[44], 37}, {NULL}};
const Segment SGI99703[] = {{&S95286[83], 6}, {NULL}};
const Segment SGI99704[] = {{&S87917[30], 6}, {NULL}};
const Segment SGI99705[] = {{&S95286[151], 8}, {NULL}};
const Segment SGI99706[] = {{&S87917[129], 3}, {NULL}};
const Segment SGI99707[] = {{&S87917[95], 5}, {NULL}};
const Segment SGI99708[] = {{&S87918[44], 7}, {NULL}};
const Segment SGI99709[] = {{&S87918[44], 36}, {NULL}};
const Segment SGI99710[] = {{&S87918[30], 6}, {NULL}};
const Segment SGI99711[] = {{&S87918[128], 3}, {NULL}};
const Segment SGI99712[] = {{&S87918[94], 5}, {NULL}};
const Segment SGI99713[] = {{&S95306[44], 10}, {NULL}};
const Segment SGI99714[] = {{&S95306[44], 27}, {NULL}};
const Segment SGI99715[] = {{&S95306[30], 6}, {NULL}};
const Segment SGI99716[] = {{&S95306[119], 3}, {NULL}};
const Segment SGI99717[] = {{&S95306[85], 5}, {NULL}};
const Segment SGI99718[] = {{&S95306[143], 7}, {NULL}};
const Segment SGI99719[] = {{&S87945[45], 7}, {NULL}};
const Segment SGI99720[] = {{&S87945[30], 6}, {NULL}};
const Segment SGI99721[] = {{&S87945[117], 3}, {NULL}};
const Segment SGI99722[] = {{&S87945[83], 5}, {NULL}};
const Segment SGI99723[] = {{&S87945[141], 13}, {NULL}};
const Segment SGI99724[] = {{&S72826[45], 10}, {NULL}};
const Segment SGI99725[] = {{&S72826[30], 6}, {NULL}};
const Segment SGI99726[] = {{&S72826[31], 3}, {NULL}};
const Segment SGI99727[] = {{&S72826[81], 5}, {NULL}};
const Segment SGI99728[] = {{&S72826[139], 13}, {NULL}};
const Segment SGI99729[] = {{&S95340[52], 10}, {NULL}};
const Segment SGI99730[] = {{&S95340[52], 27}, {NULL}};
const Segment SGI99731[] = {{&S95340[38], 6}, {NULL}};
const Segment SGI99732[] = {{&S95340[93], 5}, {NULL}};
const Segment SGI99733[] = {{&S95340[0], 14}, {NULL}};
const Segment SGI99734[] = {{&S72848[34], 11}, {NULL}};
const Segment SGI99735[] = {{&S72848[27], 6}, {NULL}};
const Segment SGI99736[] = {{&S72848[98], 9}, {NULL}};
const Segment SGI99737[] = {{&S72848[71], 6}, {NULL}};
const Segment SGI99738[] = {{&S72848[139], 8}, {NULL}};
const Segment SGI99739[] = {{&S72850[34], 10}, {NULL}};
const Segment SGI99740[] = {{&S72850[27], 6}, {NULL}};
const Segment SGI99741[] = {{&S72850[97], 9}, {NULL}};
const Segment SGI99742[] = {{&S72850[70], 6}, {NULL}};
const Segment SGI99743[] = {{&S72850[138], 8}, {NULL}};
const Segment SGI99744[] = {{&S95352[44], 9}, {NULL}};
const Segment SGI99745[] = {{&S95352[44], 21}, {NULL}};
const Segment SGI99746[] = {{&S95352[30], 6}, {NULL}};
const Segment SGI99747[] = {{&S95352[114], 3}, {NULL}};
const Segment SGI99748[] = {{&S95352[79], 6}, {NULL}};
const Segment SGI99749[] = {{&S95352[134], 8}, {NULL}};
const Segment SGI99750[] = {{&S95352[143], 13}, {NULL}};
const Segment SGI99751[] = {{&S88011[52], 7}, {NULL}};
const Segment SGI99752[] = {{&S88011[52], 24}, {NULL}};
const Segment SGI99753[] = {{&S88011[38], 6}, {NULL}};
const Segment SGI99754[] = {{&S88011[90], 5}, {NULL}};
const Segment SGI99755[] = {{&S88011[0], 14}, {NULL}};
const Segment SGI99756[] = {{&S88012[52], 7}, {NULL}};
const Segment SGI99757[] = {{&S88012[52], 24}, {NULL}};
const Segment SGI99758[] = {{&S88012[38], 6}, {NULL}};
const Segment SGI99759[] = {{&S88012[90], 5}, {NULL}};
const Segment SGI99760[] = {{&S88012[0], 14}, {NULL}};
const Segment SGI99761[] = {{&S72898[34], 6}, {NULL}};
const Segment SGI99762[] = {{&S88024[44], 7}, {NULL}};
const Segment SGI99763[] = {{&S72898[27], 6}, {NULL}};
const Segment SGI99764[] = {{&S88024[44], 24}, {NULL}};
const Segment SGI99765[] = {{&S88024[30], 6}, {NULL}};
const Segment SGI99766[] = {{&S72898[93], 9}, {NULL}};
const Segment SGI99767[] = {{&S72898[66], 6}, {NULL}};
const Segment SGI99768[] = {{&S88024[82], 6}, {NULL}};
const Segment SGI99769[] = {{&S88024[109], 13}, {NULL}};
const Segment SGI99770[] = {{&S72909[34], 7}, {NULL}};
const Segment SGI99771[] = {{&S72909[27], 6}, {NULL}};
const Segment SGI99772[] = {{&S72909[94], 9}, {NULL}};
const Segment SGI99773[] = {{&S72909[67], 6}, {NULL}};
const Segment SGI99774[] = {{&S72911[34], 5}, {NULL}};
const Segment SGI99775[] = {{&S72911[27], 6}, {NULL}};
const Segment SGI99776[] = {{&S72911[92], 9}, {NULL}};
const Segment SGI99777[] = {{&S95413[45], 10}, {NULL}};
const Segment SGI99778[] = {{&S72911[65], 6}, {NULL}};
const Segment SGI99779[] = {{&S95413[30], 6}, {NULL}};
const Segment SGI99780[] = {{&S95413[120], 3}, {NULL}};
const Segment SGI99781[] = {{&S95413[86], 5}, {NULL}};
const Segment SGI99782[] = {{&S95413[144], 13}, {NULL}};
const Segment SGI99783[] = {{&S72912[34], 25}, {NULL}};
const Segment SGI99784[] = {{&S72912[27], 6}, {NULL}};
const Segment SGI99785[] = {{&S72912[112], 9}, {NULL}};
const Segment SGI99786[] = {{&S72912[85], 6}, {NULL}};
const Segment SGI99787[] = {{&S95434[57], 10}, {NULL}};
const Segment SGI99788[] = {{&S95434[57], 27}, {NULL}};
const Segment SGI99789[] = {{&S95434[43], 6}, {NULL}};
const Segment SGI99790[] = {{&S95434[132], 3}, {NULL}};
const Segment SGI99791[] = {{&S95434[98], 5}, {NULL}};
const Segment SGI99792[] = {{&S95434[0], 12}, {NULL}};
const Segment SGI99793[] = {{&S95435[59], 10}, {NULL}};
const Segment SGI99794[] = {{&S95435[59], 27}, {NULL}};
const Segment SGI99795[] = {{&S95435[45], 6}, {NULL}};
const Segment SGI99796[] = {{&S95435[134], 3}, {NULL}};
const Segment SGI99797[] = {{&S95435[100], 5}, {NULL}};
const Segment SGI99798[] = {{&S95435[0], 14}, {NULL}};
const Segment SGI99799[] = {{&S95439[59], 10}, {NULL}};
const Segment SGI99800[] = {{&S95439[59], 27}, {NULL}};
const Segment SGI99801[] = {{&S95439[45], 6}, {NULL}};
const Segment SGI99802[] = {{&S95439[134], 3}, {NULL}};
const Segment SGI99803[] = {{&S95439[100], 5}, {NULL}};
const Segment SGI99804[] = {{&S95439[0], 14}, {NULL}};
const Segment SGI99805[] = {{&S95440[44], 10}, {NULL}};
const Segment SGI99806[] = {{&S95440[44], 27}, {NULL}};
const Segment SGI99807[] = {{&S95440[30], 6}, {NULL}};
const Segment SGI99808[] = {{&S95440[126], 3}, {NULL}};
const Segment SGI99809[] = {{&S72940[34], 25}, {NULL}};
const Segment SGI99810[] = {{&S72940[27], 6}, {NULL}};
const Segment SGI99811[] = {{&S95440[92], 5}, {NULL}};
const Segment SGI99812[] = {{&S95440[71], 8}, {NULL}};
const Segment SGI99813[] = {{&S72940[112], 9}, {NULL}};
const Segment SGI99814[] = {{&S72940[85], 6}, {NULL}};
const Segment SGI99815[] = {{&S95441[44], 10}, {NULL}};
const Segment SGI99816[] = {{&S95441[44], 27}, {NULL}};
const Segment SGI99817[] = {{&S95441[30], 6}, {NULL}};
const Segment SGI99818[] = {{&S95441[119], 3}, {NULL}};
const Segment SGI99819[] = {{&S95441[85], 5}, {NULL}};
const Segment SGI99820[] = {{&S95441[143], 7}, {NULL}};
const Segment SGI99821[] = {{&S72950[44], 7}, {NULL}};
const Segment SGI99822[] = {{&S72950[44], 19}, {NULL}};
const Segment SGI99823[] = {{&S72950[30], 6}, {NULL}};
const Segment SGI99824[] = {{&S72950[31], 3}, {NULL}};
const Segment SGI99825[] = {{&S72950[77], 6}, {NULL}};
const Segment SGI99826[] = {{&S95453[59], 10}, {NULL}};
const Segment SGI99827[] = {{&S95453[59], 27}, {NULL}};
const Segment SGI99828[] = {{&S95453[45], 6}, {NULL}};
const Segment SGI99829[] = {{&S95453[134], 3}, {NULL}};
const Segment SGI99830[] = {{&S95453[100], 5}, {NULL}};
const Segment SGI99831[] = {{&S95453[0], 14}, {NULL}};
const Segment SGI99832[] = {{&S95459[59], 4}, {NULL}};
const Segment SGI99833[] = {{&S95459[59], 16}, {NULL}};
const Segment SGI99834[] = {{&S95459[45], 6}, {NULL}};
const Segment SGI99835[] = {{&S95459[123], 3}, {NULL}};
const Segment SGI99836[] = {{&S95459[89], 5}, {NULL}};
const Segment SGI99837[] = {{&S95459[0], 14}, {NULL}};
const Segment SGI99838[] = {{&S72976[34], 18}, {NULL}};
const Segment SGI99839[] = {{&S72976[27], 6}, {NULL}};
const Segment SGI99840[] = {{&S72976[114], 9}, {NULL}};
const Segment SGI99841[] = {{&S72976[87], 6}, {NULL}};
const Segment SGI99842[] = {{&S72977[34], 15}, {NULL}};
const Segment SGI99843[] = {{&S72977[27], 6}, {NULL}};
const Segment SGI99844[] = {{&S72977[108], 9}, {NULL}};
const Segment SGI99845[] = {{&S72977[81], 6}, {NULL}};
const Segment SGI99846[] = {{&S72978[34], 15}, {NULL}};
const Segment SGI99847[] = {{&S72978[27], 6}, {NULL}};
const Segment SGI99848[] = {{&S72978[108], 9}, {NULL}};
const Segment SGI99849[] = {{&S72978[81], 6}, {NULL}};
const Segment SGI99850[] = {{&S95491[45], 5}, {NULL}};
const Segment SGI99851[] = {{&S95491[30], 6}, {NULL}};
const Segment SGI99852[] = {{&S95491[120], 3}, {NULL}};
const Segment SGI99853[] = {{&S95491[86], 5}, {NULL}};
const Segment SGI99854[] = {{&S95491[144], 13}, {NULL}};
const Segment SGI99855[] = {{&S95496[45], 10}, {NULL}};
const Segment SGI99856[] = {{&S95496[30], 6}, {NULL}};
const Segment SGI99857[] = {{&S95496[120], 3}, {NULL}};
const Segment SGI99858[] = {{&S95496[86], 5}, {NULL}};
const Segment SGI99859[] = {{&S95496[144], 13}, {NULL}};
const Segment SGI99860[] = {{&S73003[44], 7}, {NULL}};
const Segment SGI99861[] = {{&S73003[44], 19}, {NULL}};
const Segment SGI99862[] = {{&S73003[30], 6}, {NULL}};
const Segment SGI99863[] = {{&S73003[31], 3}, {NULL}};
const Segment SGI99864[] = {{&S73003[77], 6}, {NULL}};
const Segment SGI99865[] = {{&S95506[44], 10}, {NULL}};
const Segment SGI99866[] = {{&S95506[44], 27}, {NULL}};
const Segment SGI99867[] = {{&S95506[30], 6}, {NULL}};
const Segment SGI99868[] = {{&S95506[85], 6}, {NULL}};
const Segment SGI99869[] = {{&S95506[112], 13}, {NULL}};
const Segment SGI99870[] = {{&S95513[44], 10}, {NULL}};
const Segment SGI99871[] = {{&S95513[44], 27}, {NULL}};
const Segment SGI99872[] = {{&S95513[30], 6}, {NULL}};
const Segment SGI99873[] = {{&S95513[119], 3}, {NULL}};
const Segment SGI99874[] = {{&S95513[85], 5}, {NULL}};
const Segment SGI99875[] = {{&S95513[143], 11}, {NULL}};
const Segment SGI99876[] = {{&S95518[44], 10}, {NULL}};
const Segment SGI99877[] = {{&S95518[44], 27}, {NULL}};
const Segment SGI99878[] = {{&S95518[30], 6}, {NULL}};
const Segment SGI99879[] = {{&S95518[163], 3}, {NULL}};
const Segment SGI99880[] = {{&S95518[129], 3}, {NULL}};
const Segment SGI99881[] = {{&S95518[94], 6}, {NULL}};
const Segment SGI99882[] = {{&S95518[149], 8}, {NULL}};
const Segment SGI99883[] = {{&S95522[44], 11}, {NULL}};
const Segment SGI99884[] = {{&S95522[44], 28}, {NULL}};
const Segment SGI99885[] = {{&S95522[30], 6}, {NULL}};
const Segment SGI99886[] = {{&S95522[120], 3}, {NULL}};
const Segment SGI99887[] = {{&S95522[86], 5}, {NULL}};
const Segment SGI99888[] = {{&S95522[144], 11}, {NULL}};
const Segment SGI99889[] = {{&S88191[44], 15}, {NULL}};
const Segment SGI99890[] = {{&S88191[44], 25}, {NULL}};
const Segment SGI99891[] = {{&S88191[30], 6}, {NULL}};
const Segment SGI99892[] = {{&S88191[117], 3}, {NULL}};
const Segment SGI99893[] = {{&S88191[83], 5}, {NULL}};
const Segment SGI99894[] = {{&S81682[74], 6}, {NULL}};
const Segment SGI99895[] = {{&S81682[0], 9}, {NULL}};
const Segment SGI99896[] = {{&S73059[34], 10}, {NULL}};
const Segment SGI99897[] = {{&S73059[27], 6}, {NULL}};
const Segment SGI99898[] = {{&S73059[97], 9}, {NULL}};
const Segment SGI99899[] = {{&S73059[70], 6}, {NULL}};
const Segment SGI99900[] = {{&S73060[34], 10}, {NULL}};
const Segment SGI99901[] = {{&S73060[27], 6}, {NULL}};
const Segment SGI99902[] = {{&S73060[97], 9}, {NULL}};
const Segment SGI99903[] = {{&S73060[70], 6}, {NULL}};
const Segment SGI99904[] = {{&S73061[34], 10}, {NULL}};
const Segment SGI99905[] = {{&S73061[27], 6}, {NULL}};
const Segment SGI99906[] = {{&S73061[97], 9}, {NULL}};
const Segment SGI99907[] = {{&S73061[70], 6}, {NULL}};
const Segment SGI99908[] = {{&S73062[34], 10}, {NULL}};
const Segment SGI99909[] = {{&S73062[27], 6}, {NULL}};
const Segment SGI99910[] = {{&S73062[94], 9}, {NULL}};
const Segment SGI99911[] = {{&S73062[67], 6}, {NULL}};
const Segment SGI99912[] = {{&S73073[44], 14}, {NULL}};
const Segment SGI99913[] = {{&S73073[44], 26}, {NULL}};
const Segment SGI99914[] = {{&S73073[30], 6}, {NULL}};
const Segment SGI99915[] = {{&S73073[31], 3}, {NULL}};
const Segment SGI99916[] = {{&S73073[84], 6}, {NULL}};
const Segment SGI99917[] = {{&S73075[44], 14}, {NULL}};
const Segment SGI99918[] = {{&S73075[44], 26}, {NULL}};
const Segment SGI99919[] = {{&S73075[30], 6}, {NULL}};
const Segment SGI99920[] = {{&S73075[31], 3}, {NULL}};
const Segment SGI99921[] = {{&S73075[84], 6}, {NULL}};
const Segment SGI99922[] = {{&S73078[34], 15}, {NULL}};
const Segment SGI99923[] = {{&S73078[27], 6}, {NULL}};
const Segment SGI99924[] = {{&S73078[102], 9}, {NULL}};
const Segment SGI99925[] = {{&S73078[75], 6}, {NULL}};
const Segment SGI99926[] = {{&S95598[44], 24}, {NULL}};
const Segment SGI99927[] = {{&S95598[44], 36}, {NULL}};
const Segment SGI99928[] = {{&S95598[30], 6}, {NULL}};
const Segment SGI99929[] = {{&S95598[94], 6}, {NULL}};
const Segment SGI99930[] = {{&S95598[121], 13}, {NULL}};
const Segment SGI99931[] = {{&S73119[44], 10}, {NULL}};
const Segment SGI99932[] = {{&S73119[44], 22}, {NULL}};
const Segment SGI99933[] = {{&S73119[30], 6}, {NULL}};
const Segment SGI99934[] = {{&S73119[31], 3}, {NULL}};
const Segment SGI99935[] = {{&S73119[80], 6}, {NULL}};
const Segment SGI99936[] = {{&S73120[38], 10}, {NULL}};
const Segment SGI99937[] = {{&S73120[31], 6}, {NULL}};
const Segment SGI99938[] = {{&S88297[44], 12}, {NULL}};
const Segment SGI99939[] = {{&S88297[44], 22}, {NULL}};
const Segment SGI99940[] = {{&S88297[30], 6}, {NULL}};
const Segment SGI99941[] = {{&S88297[114], 3}, {NULL}};
const Segment SGI99942[] = {{&S88297[80], 5}, {NULL}};
const Segment SGI99943[] = {{&S95647[44], 9}, {NULL}};
const Segment SGI99944[] = {{&S95647[44], 21}, {NULL}};
const Segment SGI99945[] = {{&S95647[30], 6}, {NULL}};
const Segment SGI99946[] = {{&S95647[79], 6}, {NULL}};
const Segment SGI99947[] = {{&S95647[106], 13}, {NULL}};
const Segment SGI99948[] = {{&S73158[34], 17}, {NULL}};
const Segment SGI99949[] = {{&S73158[27], 6}, {NULL}};
const Segment SGI99950[] = {{&S73158[104], 9}, {NULL}};
const Segment SGI99951[] = {{&S73158[77], 6}, {NULL}};
const Segment SGI99952[] = {{&S73159[34], 23}, {NULL}};
const Segment SGI99953[] = {{&S73159[27], 6}, {NULL}};
const Segment SGI99954[] = {{&S73159[110], 9}, {NULL}};
const Segment SGI99955[] = {{&S73159[83], 6}, {NULL}};
const Segment SGI99956[] = {{&S81805[77], 6}, {NULL}};
const Segment SGI99957[] = {{&S81805[0], 12}, {NULL}};
const Segment SGI99958[] = {{&S88322[44], 7}, {NULL}};
const Segment SGI99959[] = {{&S88322[44], 23}, {NULL}};
const Segment SGI99960[] = {{&S88322[30], 6}, {NULL}};
const Segment SGI99961[] = {{&S88322[130], 3}, {NULL}};
const Segment SGI99962[] = {{&S88322[96], 5}, {NULL}};
const Segment SGI99963[] = {{&S88322[35], 1}, {NULL}};
const Segment SGI99964[] = {{&S88322[69], 11}, {NULL}};
const Segment SGI99965[] = {{&S73182[44], 7}, {NULL}};
const Segment SGI99966[] = {{&S88328[45], 7}, {NULL}};
const Segment SGI99967[] = {{&S73182[44], 19}, {NULL}};
const Segment SGI99968[] = {{&S73182[30], 6}, {NULL}};
const Segment SGI99969[] = {{&S88328[30], 6}, {NULL}};
const Segment SGI99970[] = {{&S73182[31], 3}, {NULL}};
const Segment SGI99971[] = {{&S88328[117], 3}, {NULL}};
const Segment SGI99972[] = {{&S73182[77], 6}, {NULL}};
const Segment SGI99973[] = {{&S88328[83], 5}, {NULL}};
const Segment SGI99974[] = {{&S88328[141], 13}, {NULL}};
const Segment SGI99975[] = {{&S73183[44], 7}, {NULL}};
const Segment SGI99976[] = {{&S73183[44], 19}, {NULL}};
const Segment SGI99977[] = {{&S73183[30], 6}, {NULL}};
const Segment SGI99978[] = {{&S73183[77], 6}, {NULL}};
const Segment SGI99979[] = {{&S73184[44], 7}, {NULL}};
const Segment SGI99980[] = {{&S73184[44], 19}, {NULL}};
const Segment SGI99981[] = {{&S73184[30], 6}, {NULL}};
const Segment SGI99982[] = {{&S73184[31], 3}, {NULL}};
const Segment SGI99983[] = {{&S73184[77], 6}, {NULL}};
const Segment SGI99984[] = {{&S95731[58], 10}, {NULL}};
const Segment SGI99985[] = {{&S95731[58], 27}, {NULL}};
const Segment SGI99986[] = {{&S95731[44], 6}, {NULL}};
const Segment SGI99987[] = {{&S95731[133], 3}, {NULL}};
const Segment SGI99988[] = {{&S95731[99], 5}, {NULL}};
const Segment SGI99989[] = {{&S95731[0], 14}, {NULL}};
const Segment SGI99990[] = {{&S95735[57], 10}, {NULL}};
const Segment SGI99991[] = {{&S95735[57], 27}, {NULL}};
const Segment SGI99992[] = {{&S95735[43], 6}, {NULL}};
const Segment SGI99993[] = {{&S95735[132], 3}, {NULL}};
const Segment SGI99994[] = {{&S95735[98], 5}, {NULL}};
const Segment SGI99995[] = {{&S73234[44], 4}, {NULL}};
const Segment SGI99996[] = {{&S73234[44], 16}, {NULL}};
const Segment SGI99997[] = {{&S95735[0], 12}, {NULL}};
const Segment SGI99998[] = {{&S73234[30], 6}, {NULL}};
const Segment SGI99999[] = {{&S73234[31], 3}, {NULL}};
const Segment SGI100000[] = {{&S73234[74], 6}, {NULL}};
const Segment SGI100001[] = {{&S73235[44], 4}, {NULL}};
const Segment SGI100002[] = {{&S73235[44], 16}, {NULL}};
const Segment SGI100003[] = {{&S73235[30], 6}, {NULL}};
const Segment SGI100004[] = {{&S73235[77], 6}, {NULL}};
const Segment SGI100005[] = {{&S73236[44], 9}, {NULL}};
const Segment SGI100006[] = {{&S73236[44], 21}, {NULL}};
const Segment SGI100007[] = {{&S73236[30], 6}, {NULL}};
const Segment SGI100008[] = {{&S73236[31], 3}, {NULL}};
const Segment SGI100009[] = {{&S73236[79], 6}, {NULL}};
const Segment SGI100010[] = {{&S95741[59], 11}, {NULL}};
const Segment SGI100011[] = {{&S95741[59], 28}, {NULL}};
const Segment SGI100012[] = {{&S95741[45], 6}, {NULL}};
const Segment SGI100013[] = {{&S95741[135], 3}, {NULL}};
const Segment SGI100014[] = {{&S95741[101], 5}, {NULL}};
const Segment SGI100015[] = {{&S95741[0], 14}, {NULL}};
const Segment SGI100016[] = {{&S73253[45], 10}, {NULL}};
const Segment SGI100017[] = {{&S73253[30], 6}, {NULL}};
const Segment SGI100018[] = {{&S73253[31], 3}, {NULL}};
const Segment SGI100019[] = {{&S73253[81], 5}, {NULL}};
const Segment SGI100020[] = {{&S73253[139], 13}, {NULL}};
const Segment SGI100021[] = {{&S73254[45], 11}, {NULL}};
const Segment SGI100022[] = {{&S73254[30], 6}, {NULL}};
const Segment SGI100023[] = {{&S73254[31], 3}, {NULL}};
const Segment SGI100024[] = {{&S73254[82], 5}, {NULL}};
const Segment SGI100025[] = {{&S73254[140], 13}, {NULL}};
const Segment SGI100026[] = {{&S88423[44], 16}, {NULL}};
const Segment SGI100027[] = {{&S88423[44], 27}, {NULL}};
const Segment SGI100028[] = {{&S88423[30], 6}, {NULL}};
const Segment SGI100029[] = {{&S88423[119], 3}, {NULL}};
const Segment SGI100030[] = {{&S88423[85], 5}, {NULL}};
const Segment SGI100031[] = {{&S73273[44], 21}, {NULL}};
const Segment SGI100032[] = {{&S73273[44], 33}, {NULL}};
const Segment SGI100033[] = {{&S73273[30], 6}, {NULL}};
const Segment SGI100034[] = {{&S73273[31], 3}, {NULL}};
const Segment SGI100035[] = {{&S73273[91], 6}, {NULL}};
const Segment SGI100036[] = {{&S73274[44], 10}, {NULL}};
const Segment SGI100037[] = {{&S73274[44], 22}, {NULL}};
const Segment SGI100038[] = {{&S73274[30], 6}, {NULL}};
const Segment SGI100039[] = {{&S73274[31], 3}, {NULL}};
const Segment SGI100040[] = {{&S73274[80], 6}, {NULL}};
const Segment SGI100041[] = {{&S73275[44], 9}, {NULL}};
const Segment SGI100042[] = {{&S73275[44], 21}, {NULL}};
const Segment SGI100043[] = {{&S73275[30], 6}, {NULL}};
const Segment SGI100044[] = {{&S73275[31], 3}, {NULL}};
const Segment SGI100045[] = {{&S73275[79], 6}, {NULL}};
const Segment SGI100046[] = {{&S73276[44], 9}, {NULL}};
const Segment SGI100047[] = {{&S73276[44], 21}, {NULL}};
const Segment SGI100048[] = {{&S73276[30], 6}, {NULL}};
const Segment SGI100049[] = {{&S73276[31], 3}, {NULL}};
const Segment SGI100050[] = {{&S73276[79], 6}, {NULL}};
const Segment SGI100051[] = {{&S95786[44], 10}, {NULL}};
const Segment SGI100052[] = {{&S95786[44], 27}, {NULL}};
const Segment SGI100053[] = {{&S95786[30], 6}, {NULL}};
const Segment SGI100054[] = {{&S95786[118], 3}, {NULL}};
const Segment SGI100055[] = {{&S95786[85], 5}, {NULL}};
const Segment SGI100056[] = {{&S95786[122], 14}, {NULL}};
const Segment SGI100057[] = {{&S73313[34], 10}, {NULL}};
const Segment SGI100058[] = {{&S73313[27], 6}, {NULL}};
const Segment SGI100059[] = {{&S73313[97], 9}, {NULL}};
const Segment SGI100060[] = {{&S73313[70], 6}, {NULL}};
const Segment SGI100061[] = {{&S73321[34], 13}, {NULL}};
const Segment SGI100062[] = {{&S73321[27], 6}, {NULL}};
const Segment SGI100063[] = {{&S73321[100], 9}, {NULL}};
const Segment SGI100064[] = {{&S73321[73], 6}, {NULL}};
const Segment SGI100065[] = {{&S73322[34], 10}, {NULL}};
const Segment SGI100066[] = {{&S73322[27], 6}, {NULL}};
const Segment SGI100067[] = {{&S73322[97], 9}, {NULL}};
const Segment SGI100068[] = {{&S73322[70], 6}, {NULL}};
const Segment SGI100069[] = {{&S95833[44], 10}, {NULL}};
const Segment SGI100070[] = {{&S95833[44], 27}, {NULL}};
const Segment SGI100071[] = {{&S95833[30], 6}, {NULL}};
const Segment SGI100072[] = {{&S95833[118], 3}, {NULL}};
const Segment SGI100073[] = {{&S95833[85], 5}, {NULL}};
const Segment SGI100074[] = {{&S95833[122], 14}, {NULL}};
const Segment SGI100075[] = {{&S73329[34], 18}, {NULL}};
const Segment SGI100076[] = {{&S73329[27], 6}, {NULL}};
const Segment SGI100077[] = {{&S73329[105], 9}, {NULL}};
const Segment SGI100078[] = {{&S73329[78], 6}, {NULL}};
const Segment SGI100079[] = {{&S73329[146], 8}, {NULL}};
const Segment SGI100080[] = {{&S95843[44], 10}, {NULL}};
const Segment SGI100081[] = {{&S95843[44], 27}, {NULL}};
const Segment SGI100082[] = {{&S95843[30], 6}, {NULL}};
const Segment SGI100083[] = {{&S95843[120], 3}, {NULL}};
const Segment SGI100084[] = {{&S95843[85], 6}, {NULL}};
const Segment SGI100085[] = {{&S95843[140], 8}, {NULL}};
const Segment SGI100086[] = {{&S95843[149], 13}, {NULL}};
const Segment SGI100087[] = {{&S95844[44], 10}, {NULL}};
const Segment SGI100088[] = {{&S95844[44], 27}, {NULL}};
const Segment SGI100089[] = {{&S95844[30], 6}, {NULL}};
const Segment SGI100090[] = {{&S95844[118], 3}, {NULL}};
const Segment SGI100091[] = {{&S95844[85], 5}, {NULL}};
const Segment SGI100092[] = {{&S95844[122], 14}, {NULL}};
const Segment SGI100093[] = {{&S88509[44], 7}, {NULL}};
const Segment SGI100094[] = {{&S88509[44], 24}, {NULL}};
const Segment SGI100095[] = {{&S88509[30], 6}, {NULL}};
const Segment SGI100096[] = {{&S88509[115], 3}, {NULL}};
const Segment SGI100097[] = {{&S88509[82], 5}, {NULL}};
const Segment SGI100098[] = {{&S88509[119], 14}, {NULL}};
const Segment SGI100099[] = {{&S88510[44], 7}, {NULL}};
const Segment SGI100100[] = {{&S88510[44], 24}, {NULL}};
const Segment SGI100101[] = {{&S88510[30], 6}, {NULL}};
const Segment SGI100102[] = {{&S88510[115], 3}, {NULL}};
const Segment SGI100103[] = {{&S88510[82], 5}, {NULL}};
const Segment SGI100104[] = {{&S88510[119], 14}, {NULL}};
const Segment SGI100105[] = {{&S73357[34], 6}, {NULL}};
const Segment SGI100106[] = {{&S73357[27], 6}, {NULL}};
const Segment SGI100107[] = {{&S73357[93], 9}, {NULL}};
const Segment SGI100108[] = {{&S73357[66], 6}, {NULL}};
const Segment SGI100109[] = {{&S95863[44], 6}, {NULL}};
const Segment SGI100110[] = {{&S95863[44], 22}, {NULL}};
const Segment SGI100111[] = {{&S95863[30], 6}, {NULL}};
const Segment SGI100112[] = {{&S87596[42], 10}, {NULL}};
const Segment SGI100113[] = {{&S87596[42], 21}, {NULL}};
const Segment SGI100114[] = {{&S87596[30], 4}, {NULL}};
const Segment SGI100115[] = {{&S95863[114], 3}, {NULL}};
const Segment SGI100116[] = {{&S95863[80], 5}, {NULL}};
const Segment SGI100117[] = {{&S87596[112], 3}, {NULL}};
const Segment SGI100118[] = {{&S87596[77], 6}, {NULL}};
const Segment SGI100119[] = {{&S87596[130], 13}, {NULL}};
const Segment SGI100120[] = {{&S87597[42], 11}, {NULL}};
const Segment SGI100121[] = {{&S87597[42], 22}, {NULL}};
const Segment SGI100122[] = {{&S87597[30], 4}, {NULL}};
const Segment SGI100123[] = {{&S87597[113], 3}, {NULL}};
const Segment SGI100124[] = {{&S87597[78], 6}, {NULL}};
const Segment SGI100125[] = {{&S87597[131], 13}, {NULL}};
const Segment SGI100126[] = {{&S73364[34], 11}, {NULL}};
const Segment SGI100127[] = {{&S73364[27], 6}, {NULL}};
const Segment SGI100128[] = {{&S73364[98], 9}, {NULL}};
const Segment SGI100129[] = {{&S73364[71], 6}, {NULL}};
const Segment SGI100130[] = {{&S95882[44], 9}, {NULL}};
const Segment SGI100131[] = {{&S95882[44], 21}, {NULL}};
const Segment SGI100132[] = {{&S95882[30], 6}, {NULL}};
const Segment SGI100133[] = {{&S95882[113], 3}, {NULL}};
const Segment SGI100134[] = {{&S95882[79], 5}, {NULL}};
const Segment SGI100135[] = {{&S95882[137], 7}, {NULL}};
const Segment SGI100136[] = {{&S73384[44], 6}, {NULL}};
const Segment SGI100137[] = {{&S73384[44], 18}, {NULL}};
const Segment SGI100138[] = {{&S73384[30], 6}, {NULL}};
const Segment SGI100139[] = {{&S73384[31], 3}, {NULL}};
const Segment SGI100140[] = {{&S73384[76], 6}, {NULL}};
const Segment SGI100141[] = {{&S95891[44], 10}, {NULL}};
const Segment SGI100142[] = {{&S95891[44], 27}, {NULL}};
const Segment SGI100143[] = {{&S95891[30], 6}, {NULL}};
const Segment SGI100144[] = {{&S95891[85], 6}, {NULL}};
const Segment SGI100145[] = {{&S95891[112], 13}, {NULL}};
const Segment SGI100146[] = {{&S73392[34], 6}, {NULL}};
const Segment SGI100147[] = {{&S73392[27], 6}, {NULL}};
const Segment SGI100148[] = {{&S73392[93], 9}, {NULL}};
const Segment SGI100149[] = {{&S73392[66], 6}, {NULL}};
const Segment SGI100150[] = {{&S73399[44], 7}, {NULL}};
const Segment SGI100151[] = {{&S73399[44], 20}, {NULL}};
const Segment SGI100152[] = {{&S73399[30], 6}, {NULL}};
const Segment SGI100153[] = {{&S73399[31], 3}, {NULL}};
const Segment SGI100154[] = {{&S73399[78], 6}, {NULL}};
const Segment SGI100155[] = {{&S73400[44], 7}, {NULL}};
const Segment SGI100156[] = {{&S73400[44], 20}, {NULL}};
const Segment SGI100157[] = {{&S73400[30], 6}, {NULL}};
const Segment SGI100158[] = {{&S73400[31], 3}, {NULL}};
const Segment SGI100159[] = {{&S73400[78], 6}, {NULL}};
const Segment SGI100160[] = {{&S95905[52], 10}, {NULL}};
const Segment SGI100161[] = {{&S95905[52], 27}, {NULL}};
const Segment SGI100162[] = {{&S95905[38], 6}, {NULL}};
const Segment SGI100163[] = {{&S95905[93], 5}, {NULL}};
const Segment SGI100164[] = {{&S95905[0], 14}, {NULL}};
const Segment SGI100165[] = {{&S88589[44], 15}, {NULL}};
const Segment SGI100166[] = {{&S88589[44], 25}, {NULL}};
const Segment SGI100167[] = {{&S88589[30], 6}, {NULL}};
const Segment SGI100168[] = {{&S88589[117], 3}, {NULL}};
const Segment SGI100169[] = {{&S88589[83], 5}, {NULL}};
const Segment SGI100170[] = {{&S88590[44], 11}, {NULL}};
const Segment SGI100171[] = {{&S88590[44], 21}, {NULL}};
const Segment SGI100172[] = {{&S88590[30], 6}, {NULL}};
const Segment SGI100173[] = {{&S88590[113], 3}, {NULL}};
const Segment SGI100174[] = {{&S88590[79], 5}, {NULL}};
const Segment SGI100175[] = {{&S95926[44], 10}, {NULL}};
const Segment SGI100176[] = {{&S95926[44], 27}, {NULL}};
const Segment SGI100177[] = {{&S95926[30], 6}, {NULL}};
const Segment SGI100178[] = {{&S95926[120], 3}, {NULL}};
const Segment SGI100179[] = {{&S95926[85], 6}, {NULL}};
const Segment SGI100180[] = {{&S95926[140], 8}, {NULL}};
const Segment SGI100181[] = {{&S95926[149], 13}, {NULL}};
const Segment SGI100182[] = {{&S73449[34], 7}, {NULL}};
const Segment SGI100183[] = {{&S73449[27], 6}, {NULL}};
const Segment SGI100184[] = {{&S73449[94], 9}, {NULL}};
const Segment SGI100185[] = {{&S73449[67], 6}, {NULL}};
const Segment SGI100186[] = {{&S73474[44], 7}, {NULL}};
const Segment SGI100187[] = {{&S73474[44], 20}, {NULL}};
const Segment SGI100188[] = {{&S73474[30], 6}, {NULL}};
const Segment SGI100189[] = {{&S73474[31], 3}, {NULL}};
const Segment SGI100190[] = {{&S73474[78], 6}, {NULL}};
const Segment SGI100191[] = {{&S73475[44], 7}, {NULL}};
const Segment SGI100192[] = {{&S73475[44], 20}, {NULL}};
const Segment SGI100193[] = {{&S73475[30], 6}, {NULL}};
const Segment SGI100194[] = {{&S73475[31], 3}, {NULL}};
const Segment SGI100195[] = {{&S73475[78], 6}, {NULL}};
const Segment SGI100196[] = {{&S73476[44], 7}, {NULL}};
const Segment SGI100197[] = {{&S73476[44], 20}, {NULL}};
const Segment SGI100198[] = {{&S73476[30], 6}, {NULL}};
const Segment SGI100199[] = {{&S73476[31], 3}, {NULL}};
const Segment SGI100200[] = {{&S73476[78], 6}, {NULL}};
const Segment SGI100201[] = {{&S73477[44], 7}, {NULL}};
const Segment SGI100202[] = {{&S73477[44], 20}, {NULL}};
const Segment SGI100203[] = {{&S73477[30], 6}, {NULL}};
const Segment SGI100204[] = {{&S73477[31], 3}, {NULL}};
const Segment SGI100205[] = {{&S73477[78], 6}, {NULL}};
const Segment SGI100206[] = {{&S73478[44], 7}, {NULL}};
const Segment SGI100207[] = {{&S73478[44], 20}, {NULL}};
const Segment SGI100208[] = {{&S73478[30], 6}, {NULL}};
const Segment SGI100209[] = {{&S73478[31], 3}, {NULL}};
const Segment SGI100210[] = {{&S73478[78], 6}, {NULL}};
const Segment SGI100211[] = {{&S73488[44], 8}, {NULL}};
const Segment SGI100212[] = {{&S73488[44], 20}, {NULL}};
const Segment SGI100213[] = {{&S73488[30], 6}, {NULL}};
const Segment SGI100214[] = {{&S73488[31], 3}, {NULL}};
const Segment SGI100215[] = {{&S73488[78], 6}, {NULL}};
const Segment SGI100216[] = {{&S73489[44], 9}, {NULL}};
const Segment SGI100217[] = {{&S73489[44], 35}, {NULL}};
const Segment SGI100218[] = {{&S73489[30], 6}, {NULL}};
const Segment SGI100219[] = {{&S73489[31], 3}, {NULL}};
const Segment SGI100220[] = {{&S73489[93], 6}, {NULL}};
const Segment SGI100221[] = {{&S73490[44], 9}, {NULL}};
const Segment SGI100222[] = {{&S73490[44], 29}, {NULL}};
const Segment SGI100223[] = {{&S73490[30], 6}, {NULL}};
const Segment SGI100224[] = {{&S73490[31], 3}, {NULL}};
const Segment SGI100225[] = {{&S73490[87], 6}, {NULL}};
const Segment SGI100226[] = {{&S73491[44], 9}, {NULL}};
const Segment SGI100227[] = {{&S73491[44], 28}, {NULL}};
const Segment SGI100228[] = {{&S73491[30], 6}, {NULL}};
const Segment SGI100229[] = {{&S73491[31], 3}, {NULL}};
const Segment SGI100230[] = {{&S73491[86], 6}, {NULL}};
const Segment SGI100231[] = {{&S96021[57], 10}, {NULL}};
const Segment SGI100232[] = {{&S96021[57], 27}, {NULL}};
const Segment SGI100233[] = {{&S96021[43], 6}, {NULL}};
const Segment SGI100234[] = {{&S96021[132], 3}, {NULL}};
const Segment SGI100235[] = {{&S96021[98], 5}, {NULL}};
const Segment SGI100236[] = {{&S96021[0], 12}, {NULL}};
const Segment SGI100237[] = {{&S73515[44], 7}, {NULL}};
const Segment SGI100238[] = {{&S73515[44], 20}, {NULL}};
const Segment SGI100239[] = {{&S73515[30], 6}, {NULL}};
const Segment SGI100240[] = {{&S73515[31], 3}, {NULL}};
const Segment SGI100241[] = {{&S73515[78], 6}, {NULL}};
const Segment SGI100242[] = {{&S96034[45], 4}, {NULL}};
const Segment SGI100243[] = {{&S96034[30], 6}, {NULL}};
const Segment SGI100244[] = {{&S73516[44], 7}, {NULL}};
const Segment SGI100245[] = {{&S73516[44], 20}, {NULL}};
const Segment SGI100246[] = {{&S73516[30], 6}, {NULL}};
const Segment SGI100247[] = {{&S96034[109], 3}, {NULL}};
const Segment SGI100248[] = {{&S73516[31], 3}, {NULL}};
const Segment SGI100249[] = {{&S96034[75], 5}, {NULL}};
const Segment SGI100250[] = {{&S73516[78], 6}, {NULL}};
const Segment SGI100251[] = {{&S96034[133], 13}, {NULL}};
const Segment SGI100252[] = {{&S96039[44], 10}, {NULL}};
const Segment SGI100253[] = {{&S73521[34], 41}, {NULL}};
const Segment SGI100254[] = {{&S96039[44], 27}, {NULL}};
const Segment SGI100255[] = {{&S96039[30], 6}, {NULL}};
const Segment SGI100256[] = {{&S73521[27], 6}, {NULL}};
const Segment SGI100257[] = {{&S96039[149], 4}, {NULL}};
const Segment SGI100258[] = {{&S96039[175], 3}, {NULL}};
const Segment SGI100259[] = {{&S96039[119], 3}, {&S96039[224], 5}, {NULL}};
const Segment SGI100260[] = {{&S73521[128], 9}, {NULL}};
const Segment SGI100261[] = {{&S73521[101], 6}, {NULL}};
const Segment SGI100262[] = {{&S96039[85], 5}, {NULL}};
const Segment SGI100263[] = {{&S73521[169], 8}, {NULL}};
const Segment SGI100264[] = {{&S73522[34], 35}, {NULL}};
const Segment SGI100265[] = {{&S73522[27], 6}, {NULL}};
const Segment SGI100266[] = {{&S73522[122], 9}, {NULL}};
const Segment SGI100267[] = {{&S73522[95], 6}, {NULL}};
const Segment SGI100268[] = {{&S73522[163], 8}, {NULL}};
const Segment SGI100269[] = {{&S88714[44], 10}, {NULL}};
const Segment SGI100270[] = {{&S88714[44], 21}, {NULL}};
const Segment SGI100271[] = {{&S88714[30], 6}, {NULL}};
const Segment SGI100272[] = {{&S88714[132], 3}, {NULL}};
const Segment SGI100273[] = {{&S88714[98], 5}, {NULL}};
const Segment SGI100274[] = {{&S88714[35], 1}, {NULL}};
const Segment SGI100275[] = {{&S88714[67], 11}, {NULL}};
const Segment SGI100276[] = {{&S88715[44], 10}, {NULL}};
const Segment SGI100277[] = {{&S88715[44], 21}, {NULL}};
const Segment SGI100278[] = {{&S88715[30], 6}, {NULL}};
const Segment SGI100279[] = {{&S88715[132], 3}, {NULL}};
const Segment SGI100280[] = {{&S88715[98], 5}, {NULL}};
const Segment SGI100281[] = {{&S88715[35], 1}, {NULL}};
const Segment SGI100282[] = {{&S88715[67], 11}, {NULL}};
const Segment SGI100283[] = {{&S88716[44], 10}, {NULL}};
const Segment SGI100284[] = {{&S88716[44], 21}, {NULL}};
const Segment SGI100285[] = {{&S88716[30], 6}, {NULL}};
const Segment SGI100286[] = {{&S88716[135], 3}, {NULL}};
const Segment SGI100287[] = {{&S88716[101], 5}, {NULL}};
const Segment SGI100288[] = {{&S88716[67], 11}, {NULL}};
const Segment SGI100289[] = {{&S96056[59], 24}, {NULL}};
const Segment SGI100290[] = {{&S96056[59], 36}, {NULL}};
const Segment SGI100291[] = {{&S96056[45], 6}, {NULL}};
const Segment SGI100292[] = {{&S96056[143], 3}, {NULL}};
const Segment SGI100293[] = {{&S96056[109], 5}, {NULL}};
const Segment SGI100294[] = {{&S96056[0], 14}, {NULL}};
const Segment SGI100295[] = {{&S73552[34], 11}, {NULL}};
const Segment SGI100296[] = {{&S73552[27], 6}, {NULL}};
const Segment SGI100297[] = {{&S73552[98], 9}, {NULL}};
const Segment SGI100298[] = {{&S73552[71], 6}, {NULL}};
const Segment SGI100299[] = {{&S73552[139], 8}, {NULL}};
const Segment SGI100300[] = {{&S96072[58], 10}, {NULL}};
const Segment SGI100301[] = {{&S96072[58], 22}, {NULL}};
const Segment SGI100302[] = {{&S96072[44], 6}, {NULL}};
const Segment SGI100303[] = {{&S96072[128], 3}, {NULL}};
const Segment SGI100304[] = {{&S96072[94], 5}, {NULL}};
const Segment SGI100305[] = {{&S96072[0], 14}, {NULL}};
const Segment SGI100306[] = {{&S96073[59], 20}, {NULL}};
const Segment SGI100307[] = {{&S96073[59], 32}, {NULL}};
const Segment SGI100308[] = {{&S96073[45], 6}, {NULL}};
const Segment SGI100309[] = {{&S96073[139], 3}, {NULL}};
const Segment SGI100310[] = {{&S96073[105], 5}, {NULL}};
const Segment SGI100311[] = {{&S96073[0], 14}, {NULL}};
const Segment SGI100312[] = {{&S96079[44], 10}, {NULL}};
const Segment SGI100313[] = {{&S96079[44], 27}, {NULL}};
const Segment SGI100314[] = {{&S96079[30], 6}, {NULL}};
const Segment SGI100315[] = {{&S96079[119], 3}, {NULL}};
const Segment SGI100316[] = {{&S96079[85], 5}, {NULL}};
const Segment SGI100317[] = {{&S96079[143], 13}, {NULL}};
const Segment SGI100318[] = {{&S96081[45], 4}, {NULL}};
const Segment SGI100319[] = {{&S96081[30], 6}, {NULL}};
const Segment SGI100320[] = {{&S96081[109], 3}, {NULL}};
const Segment SGI100321[] = {{&S96081[75], 5}, {NULL}};
const Segment SGI100322[] = {{&S96081[133], 13}, {NULL}};
const Segment SGI100323[] = {{&S96094[44], 10}, {NULL}};
const Segment SGI100324[] = {{&S96094[44], 27}, {NULL}};
const Segment SGI100325[] = {{&S96094[30], 6}, {NULL}};
const Segment SGI100326[] = {{&S96094[85], 6}, {NULL}};
const Segment SGI100327[] = {{&S96094[112], 13}, {NULL}};
const Segment SGI100328[] = {{&S96120[44], 10}, {NULL}};
const Segment SGI100329[] = {{&S96120[44], 27}, {NULL}};
const Segment SGI100330[] = {{&S96120[30], 6}, {NULL}};
const Segment SGI100331[] = {{&S96120[118], 3}, {NULL}};
const Segment SGI100332[] = {{&S96120[85], 5}, {NULL}};
const Segment SGI100333[] = {{&S96120[122], 14}, {NULL}};
const Segment SGI100334[] = {{&S73614[44], 13}, {NULL}};
const Segment SGI100335[] = {{&S73614[44], 25}, {NULL}};
const Segment SGI100336[] = {{&S73614[30], 6}, {NULL}};
const Segment SGI100337[] = {{&S73614[31], 3}, {NULL}};
const Segment SGI100338[] = {{&S73614[83], 6}, {NULL}};
const Segment SGI100339[] = {{&S73616[44], 11}, {NULL}};
const Segment SGI100340[] = {{&S73616[44], 23}, {NULL}};
const Segment SGI100341[] = {{&S73616[30], 6}, {NULL}};
const Segment SGI100342[] = {{&S73616[31], 3}, {NULL}};
const Segment SGI100343[] = {{&S73616[81], 6}, {NULL}};
const Segment SGI100344[] = {{&S73623[34], 15}, {NULL}};
const Segment SGI100345[] = {{&S73623[27], 6}, {NULL}};
const Segment SGI100346[] = {{&S73623[102], 9}, {NULL}};
const Segment SGI100347[] = {{&S73623[75], 6}, {NULL}};
const Segment SGI100348[] = {{&S96155[34], 10}, {NULL}};
const Segment SGI100349[] = {{&S96155[27], 6}, {NULL}};
const Segment SGI100350[] = {{&S96155[109], 2}, {NULL}};
const Segment SGI100351[] = {{&S96155[75], 6}, {NULL}};
const Segment SGI100352[] = {{&S96155[143], 8}, {NULL}};
const Segment SGI100353[] = {{&S96161[44], 9}, {NULL}};
const Segment SGI100354[] = {{&S96161[44], 21}, {NULL}};
const Segment SGI100355[] = {{&S96161[30], 6}, {NULL}};
const Segment SGI100356[] = {{&S96161[114], 3}, {NULL}};
const Segment SGI100357[] = {{&S96161[79], 6}, {NULL}};
const Segment SGI100358[] = {{&S96161[134], 8}, {NULL}};
const Segment SGI100359[] = {{&S96161[143], 13}, {NULL}};
const Segment SGI100360[] = {{&S87926[42], 20}, {NULL}};
const Segment SGI100361[] = {{&S87926[42], 31}, {NULL}};
const Segment SGI100362[] = {{&S87926[30], 4}, {NULL}};
const Segment SGI100363[] = {{&S87926[122], 3}, {NULL}};
const Segment SGI100364[] = {{&S87926[87], 6}, {NULL}};
const Segment SGI100365[] = {{&S96213[44], 6}, {NULL}};
const Segment SGI100366[] = {{&S96213[44], 22}, {NULL}};
const Segment SGI100367[] = {{&S96213[30], 6}, {NULL}};
const Segment SGI100368[] = {{&S96213[114], 3}, {NULL}};
const Segment SGI100369[] = {{&S96213[80], 5}, {NULL}};
const Segment SGI100370[] = {{&S96213[138], 7}, {NULL}};
const Segment SGI100371[] = {{&S96243[52], 11}, {NULL}};
const Segment SGI100372[] = {{&S96243[52], 28}, {NULL}};
const Segment SGI100373[] = {{&S96243[38], 6}, {NULL}};
const Segment SGI100374[] = {{&S96243[94], 5}, {NULL}};
const Segment SGI100375[] = {{&S96243[0], 14}, {NULL}};
const Segment SGI100376[] = {{&S96245[52], 10}, {NULL}};
const Segment SGI100377[] = {{&S96245[52], 27}, {NULL}};
const Segment SGI100378[] = {{&S96245[38], 6}, {NULL}};
const Segment SGI100379[] = {{&S96245[93], 5}, {NULL}};
const Segment SGI100380[] = {{&S96245[0], 14}, {NULL}};
const Segment SGI100381[] = {{&S73682[44], 7}, {NULL}};
const Segment SGI100382[] = {{&S73682[44], 19}, {NULL}};
const Segment SGI100383[] = {{&S73682[30], 6}, {NULL}};
const Segment SGI100384[] = {{&S73682[31], 3}, {NULL}};
const Segment SGI100385[] = {{&S73682[77], 6}, {NULL}};
const Segment SGI100386[] = {{&S73697[34], 11}, {NULL}};
const Segment SGI100387[] = {{&S73697[27], 6}, {NULL}};
const Segment SGI100388[] = {{&S73697[98], 9}, {NULL}};
const Segment SGI100389[] = {{&S73697[71], 6}, {NULL}};
const Segment SGI100390[] = {{&S96343[45], 10}, {NULL}};
const Segment SGI100391[] = {{&S96343[30], 6}, {NULL}};
const Segment SGI100392[] = {{&S96343[120], 3}, {NULL}};
const Segment SGI100393[] = {{&S96343[86], 5}, {NULL}};
const Segment SGI100394[] = {{&S96343[144], 13}, {NULL}};
const Segment SGI100395[] = {{&S96347[57], 10}, {NULL}};
const Segment SGI100396[] = {{&S96347[57], 27}, {NULL}};
const Segment SGI100397[] = {{&S96347[43], 6}, {NULL}};
const Segment SGI100398[] = {{&S96347[132], 3}, {NULL}};
const Segment SGI100399[] = {{&S96347[98], 5}, {NULL}};
const Segment SGI100400[] = {{&S96347[0], 12}, {NULL}};
const Segment SGI100401[] = {{&S96358[31], 8}, {NULL}};
const Segment SGI100402[] = {{&S96358[54], 3}, {NULL}};
const Segment SGI100403[] = {{&S96365[59], 10}, {NULL}};
const Segment SGI100404[] = {{&S96365[59], 27}, {NULL}};
const Segment SGI100405[] = {{&S96365[45], 6}, {NULL}};
const Segment SGI100406[] = {{&S96365[134], 3}, {NULL}};
const Segment SGI100407[] = {{&S96365[100], 5}, {NULL}};
const Segment SGI100408[] = {{&S96365[0], 14}, {NULL}};
const Segment SGI100409[] = {{&S96379[59], 5}, {NULL}};
const Segment SGI100410[] = {{&S96379[59], 27}, {NULL}};
const Segment SGI100411[] = {{&S96379[45], 6}, {NULL}};
const Segment SGI100412[] = {{&S96379[134], 3}, {NULL}};
const Segment SGI100413[] = {{&S96379[100], 5}, {NULL}};
const Segment SGI100414[] = {{&S96379[0], 14}, {NULL}};
const Segment SGI100415[] = {{&S96396[59], 5}, {NULL}};
const Segment SGI100416[] = {{&S96396[59], 17}, {NULL}};
const Segment SGI100417[] = {{&S96396[45], 6}, {NULL}};
const Segment SGI100418[] = {{&S96396[124], 3}, {NULL}};
const Segment SGI100419[] = {{&S96396[90], 5}, {NULL}};
const Segment SGI100420[] = {{&S96396[0], 14}, {NULL}};
const Segment SGI100421[] = {{&S96412[44], 19}, {NULL}};
const Segment SGI100422[] = {{&S96412[44], 30}, {NULL}};
const Segment SGI100423[] = {{&S96412[30], 6}, {NULL}};
const Segment SGI100424[] = {{&S96412[122], 3}, {NULL}};
const Segment SGI100425[] = {{&S96412[88], 5}, {NULL}};
const Segment SGI100426[] = {{&S96418[44], 10}, {NULL}};
const Segment SGI100427[] = {{&S96418[44], 27}, {NULL}};
const Segment SGI100428[] = {{&S96418[30], 6}, {NULL}};
const Segment SGI100429[] = {{&S73832[44], 10}, {NULL}};
const Segment SGI100430[] = {{&S96418[119], 3}, {NULL}};
const Segment SGI100431[] = {{&S73832[44], 22}, {NULL}};
const Segment SGI100432[] = {{&S73832[30], 6}, {NULL}};
const Segment SGI100433[] = {{&S96418[85], 5}, {NULL}};
const Segment SGI100434[] = {{&S96418[143], 13}, {NULL}};
const Segment SGI100435[] = {{&S73832[80], 6}, {NULL}};
const Segment SGI100436[] = {{&S73832[107], 13}, {NULL}};
const Segment SGI100437[] = {{&S73836[34], 7}, {NULL}};
const Segment SGI100438[] = {{&S73836[27], 6}, {NULL}};
const Segment SGI100439[] = {{&S73836[94], 9}, {NULL}};
const Segment SGI100440[] = {{&S73836[67], 6}, {NULL}};
const Segment SGI100441[] = {{&S96432[44], 10}, {NULL}};
const Segment SGI100442[] = {{&S96432[44], 22}, {NULL}};
const Segment SGI100443[] = {{&S96432[30], 6}, {NULL}};
const Segment SGI100444[] = {{&S96432[113], 3}, {NULL}};
const Segment SGI100445[] = {{&S73845[34], 11}, {NULL}};
const Segment SGI100446[] = {{&S96432[80], 5}, {NULL}};
const Segment SGI100447[] = {{&S73845[27], 6}, {NULL}};
const Segment SGI100448[] = {{&S96432[117], 14}, {NULL}};
const Segment SGI100449[] = {{&S73845[98], 9}, {NULL}};
const Segment SGI100450[] = {{&S73845[71], 6}, {NULL}};
const Segment SGI100451[] = {{&S73845[139], 8}, {NULL}};
const Segment SGI100452[] = {{&S96434[45], 10}, {NULL}};
const Segment SGI100453[] = {{&S96434[30], 6}, {NULL}};
const Segment SGI100454[] = {{&S96434[115], 3}, {NULL}};
const Segment SGI100455[] = {{&S73847[34], 19}, {NULL}};
const Segment SGI100456[] = {{&S96434[81], 5}, {NULL}};
const Segment SGI100457[] = {{&S73847[27], 6}, {NULL}};
const Segment SGI100458[] = {{&S96434[139], 13}, {NULL}};
const Segment SGI100459[] = {{&S73847[106], 9}, {NULL}};
const Segment SGI100460[] = {{&S73847[79], 6}, {NULL}};
const Segment SGI100461[] = {{&S96437[34], 11}, {NULL}};
const Segment SGI100462[] = {{&S96437[27], 6}, {NULL}};
const Segment SGI100463[] = {{&S96437[110], 2}, {NULL}};
const Segment SGI100464[] = {{&S96437[76], 6}, {NULL}};
const Segment SGI100465[] = {{&S96437[144], 8}, {NULL}};
const Segment SGI100466[] = {{&S89135[45], 7}, {NULL}};
const Segment SGI100467[] = {{&S89135[30], 6}, {NULL}};
const Segment SGI100468[] = {{&S89135[117], 3}, {NULL}};
const Segment SGI100469[] = {{&S89135[83], 5}, {NULL}};
const Segment SGI100470[] = {{&S89135[141], 13}, {NULL}};
const Segment SGI100471[] = {{&S73860[44], 9}, {NULL}};
const Segment SGI100472[] = {{&S73860[44], 21}, {NULL}};
const Segment SGI100473[] = {{&S73860[30], 6}, {NULL}};
const Segment SGI100474[] = {{&S73860[31], 3}, {NULL}};
const Segment SGI100475[] = {{&S73860[79], 6}, {NULL}};
const Segment SGI100476[] = {{&S96454[44], 10}, {NULL}};
const Segment SGI100477[] = {{&S96454[44], 27}, {NULL}};
const Segment SGI100478[] = {{&S96454[30], 6}, {NULL}};
const Segment SGI100479[] = {{&S96454[119], 3}, {NULL}};
const Segment SGI100480[] = {{&S96454[85], 5}, {NULL}};
const Segment SGI100481[] = {{&S96454[143], 11}, {NULL}};
const Segment SGI100482[] = {{&S96463[34], 18}, {NULL}};
const Segment SGI100483[] = {{&S96463[27], 6}, {NULL}};
const Segment SGI100484[] = {{&S96463[117], 2}, {NULL}};
const Segment SGI100485[] = {{&S96463[83], 6}, {NULL}};
const Segment SGI100486[] = {{&S96463[151], 8}, {NULL}};
const Segment SGI100487[] = {{&S73891[34], 13}, {NULL}};
const Segment SGI100488[] = {{&S73891[27], 6}, {NULL}};
const Segment SGI100489[] = {{&S73891[100], 9}, {NULL}};
const Segment SGI100490[] = {{&S73891[73], 6}, {NULL}};
const Segment SGI100491[] = {{&S96493[34], 11}, {NULL}};
const Segment SGI100492[] = {{&S96493[27], 6}, {NULL}};
const Segment SGI100493[] = {{&S96493[110], 2}, {NULL}};
const Segment SGI100494[] = {{&S96493[76], 6}, {NULL}};
const Segment SGI100495[] = {{&S73915[44], 11}, {NULL}};
const Segment SGI100496[] = {{&S73915[44], 23}, {NULL}};
const Segment SGI100497[] = {{&S96493[144], 8}, {NULL}};
const Segment SGI100498[] = {{&S73915[30], 6}, {NULL}};
const Segment SGI100499[] = {{&S73915[31], 3}, {NULL}};
const Segment SGI100500[] = {{&S73915[81], 6}, {NULL}};
const Segment SGI100501[] = {{&S96498[44], 14}, {NULL}};
const Segment SGI100502[] = {{&S96498[44], 26}, {NULL}};
const Segment SGI100503[] = {{&S96498[30], 6}, {NULL}};
const Segment SGI100504[] = {{&S96498[119], 3}, {NULL}};
const Segment SGI100505[] = {{&S96498[84], 6}, {NULL}};
const Segment SGI100506[] = {{&S96498[139], 8}, {NULL}};
const Segment SGI100507[] = {{&S96498[148], 13}, {NULL}};
const Segment SGI100508[] = {{&S96502[44], 4}, {NULL}};
const Segment SGI100509[] = {{&S96502[44], 16}, {NULL}};
const Segment SGI100510[] = {{&S96502[30], 6}, {NULL}};
const Segment SGI100511[] = {{&S96502[74], 6}, {NULL}};
const Segment SGI100512[] = {{&S96502[101], 13}, {NULL}};
const Segment SGI100513[] = {{&S73931[34], 7}, {NULL}};
const Segment SGI100514[] = {{&S73931[27], 6}, {NULL}};
const Segment SGI100515[] = {{&S73931[99], 9}, {NULL}};
const Segment SGI100516[] = {{&S73931[72], 6}, {NULL}};
const Segment SGI100517[] = {{&S96514[44], 10}, {NULL}};
const Segment SGI100518[] = {{&S96514[44], 27}, {NULL}};
const Segment SGI100519[] = {{&S96514[30], 6}, {NULL}};
const Segment SGI100520[] = {{&S96514[118], 3}, {NULL}};
const Segment SGI100521[] = {{&S96514[85], 5}, {NULL}};
const Segment SGI100522[] = {{&S96514[122], 14}, {NULL}};
const Segment SGI100523[] = {{&S73985[44], 8}, {NULL}};
const Segment SGI100524[] = {{&S73985[44], 20}, {NULL}};
const Segment SGI100525[] = {{&S73985[30], 6}, {NULL}};
const Segment SGI100526[] = {{&S73985[31], 3}, {NULL}};
const Segment SGI100527[] = {{&S73985[78], 6}, {NULL}};
const Segment SGI100528[] = {{&S73987[44], 14}, {NULL}};
const Segment SGI100529[] = {{&S73987[44], 26}, {NULL}};
const Segment SGI100530[] = {{&S73987[30], 6}, {NULL}};
const Segment SGI100531[] = {{&S73987[31], 3}, {NULL}};
const Segment SGI100532[] = {{&S73987[84], 6}, {NULL}};
const Segment SGI100533[] = {{&S73990[44], 14}, {NULL}};
const Segment SGI100534[] = {{&S73990[44], 25}, {NULL}};
const Segment SGI100535[] = {{&S73990[30], 6}, {NULL}};
const Segment SGI100536[] = {{&S73990[31], 3}, {NULL}};
const Segment SGI100537[] = {{&S73990[83], 6}, {NULL}};
const Segment SGI100538[] = {{&S73994[44], 24}, {NULL}};
const Segment SGI100539[] = {{&S73994[44], 36}, {NULL}};
const Segment SGI100540[] = {{&S73994[30], 6}, {NULL}};
const Segment SGI100541[] = {{&S73994[31], 3}, {NULL}};
const Segment SGI100542[] = {{&S73994[94], 6}, {NULL}};
const Segment SGI100543[] = {{&S96573[44], 13}, {NULL}};
const Segment SGI100544[] = {{&S96573[44], 30}, {NULL}};
const Segment SGI100545[] = {{&S96573[30], 6}, {NULL}};
const Segment SGI100546[] = {{&S96573[88], 6}, {NULL}};
const Segment SGI100547[] = {{&S96573[115], 13}, {NULL}};
const Segment SGI100548[] = {{&S74036[34], 25}, {NULL}};
const Segment SGI100549[] = {{&S74036[27], 6}, {NULL}};
const Segment SGI100550[] = {{&S74036[112], 9}, {NULL}};
const Segment SGI100551[] = {{&S74036[85], 6}, {NULL}};
const Segment SGI100552[] = {{&S74072[44], 7}, {NULL}};
const Segment SGI100553[] = {{&S74072[44], 20}, {NULL}};
const Segment SGI100554[] = {{&S74072[30], 6}, {NULL}};
const Segment SGI100555[] = {{&S74072[31], 3}, {NULL}};
const Segment SGI100556[] = {{&S74072[78], 6}, {NULL}};
const Segment SGI100557[] = {{&S74073[44], 7}, {NULL}};
const Segment SGI100558[] = {{&S74073[44], 20}, {NULL}};
const Segment SGI100559[] = {{&S74073[30], 6}, {NULL}};
const Segment SGI100560[] = {{&S74073[31], 3}, {NULL}};
const Segment SGI100561[] = {{&S74073[78], 6}, {NULL}};
const Segment SGI100562[] = {{&S74074[44], 7}, {NULL}};
const Segment SGI100563[] = {{&S74074[44], 20}, {NULL}};
const Segment SGI100564[] = {{&S74074[30], 6}, {NULL}};
const Segment SGI100565[] = {{&S74074[31], 3}, {NULL}};
const Segment SGI100566[] = {{&S74074[78], 6}, {NULL}};
const Segment SGI100567[] = {{&S88220[42], 10}, {NULL}};
const Segment SGI100568[] = {{&S88220[42], 22}, {NULL}};
const Segment SGI100569[] = {{&S88220[30], 4}, {NULL}};
const Segment SGI100570[] = {{&S88220[113], 3}, {NULL}};
const Segment SGI100571[] = {{&S88220[78], 6}, {NULL}};
const Segment SGI100572[] = {{&S88220[131], 13}, {NULL}};
const Segment SGI100573[] = {{&S96683[45], 5}, {NULL}};
const Segment SGI100574[] = {{&S96683[30], 6}, {NULL}};
const Segment SGI100575[] = {{&S96683[110], 3}, {NULL}};
const Segment SGI100576[] = {{&S96683[76], 5}, {NULL}};
const Segment SGI100577[] = {{&S96683[134], 13}, {NULL}};
const Segment SGI100578[] = {{&S74103[44], 8}, {NULL}};
const Segment SGI100579[] = {{&S74103[44], 20}, {NULL}};
const Segment SGI100580[] = {{&S74103[30], 6}, {NULL}};
const Segment SGI100581[] = {{&S74103[31], 3}, {NULL}};
const Segment SGI100582[] = {{&S74103[78], 6}, {NULL}};
const Segment SGI100583[] = {{&S89420[44], 12}, {NULL}};
const Segment SGI100584[] = {{&S89420[44], 23}, {NULL}};
const Segment SGI100585[] = {{&S89420[30], 6}, {NULL}};
const Segment SGI100586[] = {{&S89420[134], 3}, {NULL}};
const Segment SGI100587[] = {{&S89420[100], 5}, {NULL}};
const Segment SGI100588[] = {{&S89420[35], 1}, {NULL}};
const Segment SGI100589[] = {{&S89420[69], 11}, {NULL}};
const Segment SGI100590[] = {{&S74130[34], 9}, {NULL}};
const Segment SGI100591[] = {{&S74130[27], 6}, {NULL}};
const Segment SGI100592[] = {{&S74130[96], 9}, {NULL}};
const Segment SGI100593[] = {{&S74130[69], 6}, {NULL}};
const Segment SGI100594[] = {{&S74130[137], 8}, {NULL}};
const Segment SGI100595[] = {{&S74131[34], 9}, {NULL}};
const Segment SGI100596[] = {{&S74131[27], 6}, {NULL}};
const Segment SGI100597[] = {{&S74131[96], 9}, {NULL}};
const Segment SGI100598[] = {{&S74131[69], 6}, {NULL}};
const Segment SGI100599[] = {{&S74131[137], 8}, {NULL}};
const Segment SGI100600[] = {{&S74132[34], 10}, {NULL}};
const Segment SGI100601[] = {{&S74132[27], 6}, {NULL}};
const Segment SGI100602[] = {{&S74132[97], 9}, {NULL}};
const Segment SGI100603[] = {{&S74132[70], 6}, {NULL}};
const Segment SGI100604[] = {{&S74132[138], 8}, {NULL}};
const Segment SGI100605[] = {{&S74133[34], 10}, {NULL}};
const Segment SGI100606[] = {{&S74133[27], 6}, {NULL}};
const Segment SGI100607[] = {{&S74133[97], 9}, {NULL}};
const Segment SGI100608[] = {{&S74133[70], 6}, {NULL}};
const Segment SGI100609[] = {{&S74133[138], 8}, {NULL}};
const Segment SGI100610[] = {{&S74144[34], 10}, {NULL}};
const Segment SGI100611[] = {{&S74144[27], 6}, {NULL}};
const Segment SGI100612[] = {{&S74144[97], 9}, {NULL}};
const Segment SGI100613[] = {{&S74144[70], 6}, {NULL}};
const Segment SGI100614[] = {{&S74146[44], 7}, {NULL}};
const Segment SGI100615[] = {{&S74146[44], 19}, {NULL}};
const Segment SGI100616[] = {{&S74146[30], 6}, {NULL}};
const Segment SGI100617[] = {{&S74146[31], 3}, {NULL}};
const Segment SGI100618[] = {{&S74146[77], 6}, {NULL}};
const Segment SGI100619[] = {{&S74147[44], 8}, {NULL}};
const Segment SGI100620[] = {{&S74147[44], 20}, {NULL}};
const Segment SGI100621[] = {{&S74147[30], 6}, {NULL}};
const Segment SGI100622[] = {{&S74147[31], 3}, {NULL}};
const Segment SGI100623[] = {{&S74147[78], 6}, {NULL}};
const Segment SGI100624[] = {{&S74150[44], 8}, {NULL}};
const Segment SGI100625[] = {{&S74150[44], 20}, {NULL}};
const Segment SGI100626[] = {{&S74150[30], 6}, {NULL}};
const Segment SGI100627[] = {{&S74150[31], 3}, {NULL}};
const Segment SGI100628[] = {{&S74150[78], 6}, {NULL}};
const Segment SGI100629[] = {{&S74158[34], 10}, {NULL}};
const Segment SGI100630[] = {{&S74158[27], 6}, {NULL}};
const Segment SGI100631[] = {{&S74158[97], 9}, {NULL}};
const Segment SGI100632[] = {{&S74158[70], 6}, {NULL}};
const Segment SGI100633[] = {{&S88304[42], 10}, {NULL}};
const Segment SGI100634[] = {{&S88304[42], 22}, {NULL}};
const Segment SGI100635[] = {{&S88304[30], 4}, {NULL}};
const Segment SGI100636[] = {{&S88304[113], 3}, {NULL}};
const Segment SGI100637[] = {{&S88304[78], 6}, {NULL}};
const Segment SGI100638[] = {{&S88305[42], 10}, {NULL}};
const Segment SGI100639[] = {{&S88305[42], 22}, {NULL}};
const Segment SGI100640[] = {{&S88305[30], 4}, {NULL}};
const Segment SGI100641[] = {{&S88305[113], 3}, {NULL}};
const Segment SGI100642[] = {{&S88305[78], 6}, {NULL}};
const Segment SGI100643[] = {{&S88306[42], 10}, {NULL}};
const Segment SGI100644[] = {{&S88306[42], 25}, {NULL}};
const Segment SGI100645[] = {{&S88306[30], 4}, {NULL}};
const Segment SGI100646[] = {{&S88306[116], 3}, {NULL}};
const Segment SGI100647[] = {{&S88306[81], 6}, {NULL}};
const Segment SGI100648[] = {{&S96765[44], 11}, {NULL}};
const Segment SGI100649[] = {{&S96765[44], 28}, {NULL}};
const Segment SGI100650[] = {{&S96765[30], 6}, {NULL}};
const Segment SGI100651[] = {{&S96765[119], 3}, {NULL}};
const Segment SGI100652[] = {{&S96765[86], 5}, {NULL}};
const Segment SGI100653[] = {{&S96765[123], 14}, {NULL}};
const Segment SGI100654[] = {{&S96769[59], 20}, {NULL}};
const Segment SGI100655[] = {{&S96769[59], 32}, {NULL}};
const Segment SGI100656[] = {{&S96769[45], 6}, {NULL}};
const Segment SGI100657[] = {{&S96769[139], 3}, {NULL}};
const Segment SGI100658[] = {{&S74180[34], 10}, {NULL}};
const Segment SGI100659[] = {{&S74180[27], 6}, {NULL}};
const Segment SGI100660[] = {{&S96769[105], 5}, {NULL}};
const Segment SGI100661[] = {{&S96769[0], 14}, {NULL}};
const Segment SGI100662[] = {{&S74180[97], 9}, {NULL}};
const Segment SGI100663[] = {{&S74180[70], 6}, {NULL}};
const Segment SGI100664[] = {{&S96772[34], 9}, {NULL}};
const Segment SGI100665[] = {{&S96772[27], 6}, {NULL}};
const Segment SGI100666[] = {{&S96772[103], 2}, {NULL}};
const Segment SGI100667[] = {{&S96772[69], 6}, {NULL}};
const Segment SGI100668[] = {{&S96772[137], 8}, {NULL}};
const Segment SGI100669[] = {{&S96783[34], 10}, {NULL}};
const Segment SGI100670[] = {{&S96783[27], 6}, {NULL}};
const Segment SGI100671[] = {{&S96783[109], 2}, {NULL}};
const Segment SGI100672[] = {{&S96783[75], 6}, {NULL}};
const Segment SGI100673[] = {{&S96783[143], 8}, {NULL}};
const Segment SGI100674[] = {{&S74197[44], 7}, {NULL}};
const Segment SGI100675[] = {{&S74197[44], 19}, {NULL}};
const Segment SGI100676[] = {{&S74197[30], 6}, {NULL}};
const Segment SGI100677[] = {{&S74197[31], 3}, {NULL}};
const Segment SGI100678[] = {{&S74197[77], 6}, {NULL}};
const Segment SGI100679[] = {{&S89498[55], 10}, {NULL}};
const Segment SGI100680[] = {{&S89498[55], 28}, {NULL}};
const Segment SGI100681[] = {{&S89498[30], 16}, {NULL}};
const Segment SGI100682[] = {{&S89498[131], 3}, {NULL}};
const Segment SGI100683[] = {{&S96786[44], 14}, {NULL}};
const Segment SGI100684[] = {{&S96786[44], 26}, {NULL}};
const Segment SGI100685[] = {{&S96786[30], 6}, {NULL}};
const Segment SGI100686[] = {{&S89498[97], 5}, {NULL}};
const Segment SGI100687[] = {{&S96786[84], 6}, {NULL}};
const Segment SGI100688[] = {{&S96786[111], 13}, {NULL}};
const Segment SGI100689[] = {{&S96790[44], 10}, {NULL}};
const Segment SGI100690[] = {{&S96790[44], 24}, {NULL}};
const Segment SGI100691[] = {{&S96790[30], 6}, {NULL}};
const Segment SGI100692[] = {{&S96790[116], 3}, {NULL}};
const Segment SGI100693[] = {{&S96790[82], 5}, {NULL}};
const Segment SGI100694[] = {{&S74206[34], 11}, {NULL}};
const Segment SGI100695[] = {{&S74206[27], 6}, {NULL}};
const Segment SGI100696[] = {{&S74206[98], 9}, {NULL}};
const Segment SGI100697[] = {{&S74206[71], 6}, {NULL}};
const Segment SGI100698[] = {{&S74206[139], 8}, {NULL}};
const Segment SGI100699[] = {{&S96813[44], 10}, {NULL}};
const Segment SGI100700[] = {{&S96813[44], 27}, {NULL}};
const Segment SGI100701[] = {{&S96813[30], 6}, {NULL}};
const Segment SGI100702[] = {{&S96813[85], 6}, {NULL}};
const Segment SGI100703[] = {{&S96813[112], 13}, {NULL}};
const Segment SGI100704[] = {{&S96820[44], 12}, {NULL}};
const Segment SGI100705[] = {{&S96820[44], 24}, {NULL}};
const Segment SGI100706[] = {{&S96820[30], 6}, {NULL}};
const Segment SGI100707[] = {{&S96820[82], 6}, {NULL}};
const Segment SGI100708[] = {{&S96820[109], 13}, {NULL}};
const Segment SGI100709[] = {{&S96828[44], 10}, {NULL}};
const Segment SGI100710[] = {{&S96828[44], 27}, {NULL}};
const Segment SGI100711[] = {{&S96828[30], 6}, {NULL}};
const Segment SGI100712[] = {{&S96828[85], 6}, {NULL}};
const Segment SGI100713[] = {{&S96828[112], 13}, {NULL}};
const Segment SGI100714[] = {{&S74259[34], 9}, {NULL}};
const Segment SGI100715[] = {{&S74259[27], 6}, {NULL}};
const Segment SGI100716[] = {{&S74259[96], 9}, {NULL}};
const Segment SGI100717[] = {{&S74259[69], 6}, {NULL}};
const Segment SGI100718[] = {{&S96861[44], 14}, {NULL}};
const Segment SGI100719[] = {{&S96861[44], 26}, {NULL}};
const Segment SGI100720[] = {{&S96861[30], 6}, {NULL}};
const Segment SGI100721[] = {{&S96861[118], 3}, {NULL}};
const Segment SGI100722[] = {{&S96861[84], 5}, {NULL}};
const Segment SGI100723[] = {{&S96861[142], 13}, {NULL}};
const Segment SGI100724[] = {{&S74294[44], 10}, {NULL}};
const Segment SGI100725[] = {{&S74294[44], 22}, {NULL}};
const Segment SGI100726[] = {{&S74294[30], 6}, {NULL}};
const Segment SGI100727[] = {{&S74294[31], 3}, {NULL}};
const Segment SGI100728[] = {{&S74294[80], 6}, {NULL}};
const Segment SGI100729[] = {{&S88428[42], 7}, {NULL}};
const Segment SGI100730[] = {{&S88428[42], 26}, {NULL}};
const Segment SGI100731[] = {{&S88428[30], 4}, {NULL}};
const Segment SGI100732[] = {{&S88428[117], 3}, {NULL}};
const Segment SGI100733[] = {{&S88428[82], 6}, {NULL}};
const Segment SGI100734[] = {{&S74298[44], 10}, {NULL}};
const Segment SGI100735[] = {{&S74298[44], 22}, {NULL}};
const Segment SGI100736[] = {{&S74298[30], 6}, {NULL}};
const Segment SGI100737[] = {{&S74298[31], 3}, {NULL}};
const Segment SGI100738[] = {{&S74298[80], 6}, {NULL}};
const Segment SGI100739[] = {{&S74298[150], 3}, {NULL}};
const Segment SGI100740[] = {{&S89598[44], 7}, {NULL}};
const Segment SGI100741[] = {{&S89598[44], 19}, {NULL}};
const Segment SGI100742[] = {{&S89598[30], 6}, {NULL}};
const Segment SGI100743[] = {{&S89598[126], 3}, {NULL}};
const Segment SGI100744[] = {{&S89598[92], 5}, {NULL}};
const Segment SGI100745[] = {{&S89598[35], 1}, {NULL}};
const Segment SGI100746[] = {{&S89598[65], 11}, {NULL}};
const Segment SGI100747[] = {{&S74353[44], 7}, {NULL}};
const Segment SGI100748[] = {{&S74353[44], 19}, {NULL}};
const Segment SGI100749[] = {{&S74353[30], 6}, {NULL}};
const Segment SGI100750[] = {{&S74353[31], 3}, {NULL}};
const Segment SGI100751[] = {{&S74353[77], 6}, {NULL}};
const Segment SGI100752[] = {{&S74357[44], 8}, {NULL}};
const Segment SGI100753[] = {{&S74357[44], 20}, {NULL}};
const Segment SGI100754[] = {{&S74357[30], 6}, {NULL}};
const Segment SGI100755[] = {{&S74357[31], 3}, {NULL}};
const Segment SGI100756[] = {{&S74357[78], 6}, {NULL}};
const Segment SGI100757[] = {{&S74369[34], 6}, {NULL}};
const Segment SGI100758[] = {{&S74369[27], 6}, {NULL}};
const Segment SGI100759[] = {{&S74369[93], 9}, {NULL}};
const Segment SGI100760[] = {{&S74369[66], 6}, {NULL}};
const Segment SGI100761[] = {{&S74423[34], 9}, {NULL}};
const Segment SGI100762[] = {{&S74423[27], 6}, {NULL}};
const Segment SGI100763[] = {{&S74423[96], 9}, {NULL}};
const Segment SGI100764[] = {{&S74423[69], 6}, {NULL}};
const Segment SGI100765[] = {{&S74425[44], 41}, {NULL}};
const Segment SGI100766[] = {{&S74425[44], 53}, {NULL}};
const Segment SGI100767[] = {{&S74425[30], 6}, {NULL}};
const Segment SGI100768[] = {{&S97026[44], 12}, {NULL}};
const Segment SGI100769[] = {{&S97026[44], 24}, {NULL}};
const Segment SGI100770[] = {{&S97026[30], 6}, {NULL}};
const Segment SGI100771[] = {{&S97026[116], 3}, {NULL}};
const Segment SGI100772[] = {{&S97026[82], 5}, {NULL}};
const Segment SGI100773[] = {{&S97050[45], 11}, {NULL}};
const Segment SGI100774[] = {{&S97050[30], 6}, {NULL}};
const Segment SGI100775[] = {{&S97050[121], 3}, {NULL}};
const Segment SGI100776[] = {{&S97050[87], 5}, {NULL}};
const Segment SGI100777[] = {{&S97050[145], 13}, {NULL}};
const Segment SGI100778[] = {{&S97057[44], 11}, {NULL}};
const Segment SGI100779[] = {{&S97057[44], 23}, {NULL}};
const Segment SGI100780[] = {{&S97057[30], 6}, {NULL}};
const Segment SGI100781[] = {{&S97057[114], 3}, {NULL}};
const Segment SGI100782[] = {{&S97057[81], 5}, {NULL}};
const Segment SGI100783[] = {{&S97057[118], 14}, {NULL}};
const Segment SGI100784[] = {{&S97058[44], 10}, {NULL}};
const Segment SGI100785[] = {{&S97058[44], 27}, {NULL}};
const Segment SGI100786[] = {{&S97058[30], 6}, {NULL}};
const Segment SGI100787[] = {{&S97058[118], 3}, {NULL}};
const Segment SGI100788[] = {{&S97058[85], 5}, {NULL}};
const Segment SGI100789[] = {{&S97058[122], 14}, {NULL}};
const Segment SGI100790[] = {{&S97059[45], 13}, {NULL}};
const Segment SGI100791[] = {{&S97059[30], 6}, {NULL}};
const Segment SGI100792[] = {{&S97059[125], 3}, {NULL}};
const Segment SGI100793[] = {{&S97059[91], 5}, {NULL}};
const Segment SGI100794[] = {{&S97059[149], 13}, {NULL}};
const Segment SGI100795[] = {{&S97060[44], 10}, {NULL}};
const Segment SGI100796[] = {{&S97060[44], 27}, {NULL}};
const Segment SGI100797[] = {{&S97060[30], 6}, {NULL}};
const Segment SGI100798[] = {{&S97060[85], 6}, {NULL}};
const Segment SGI100799[] = {{&S97060[112], 13}, {NULL}};
const Segment SGI100800[] = {{&S97089[44], 10}, {NULL}};
const Segment SGI100801[] = {{&S97089[44], 27}, {NULL}};
const Segment SGI100802[] = {{&S97089[30], 6}, {NULL}};
const Segment SGI100803[] = {{&S97089[118], 3}, {NULL}};
const Segment SGI100804[] = {{&S97089[85], 5}, {NULL}};
const Segment SGI100805[] = {{&S97089[122], 14}, {NULL}};
const Segment SGI100806[] = {{&S74425[111], 6}, {NULL}};
const Segment SGI100807[] = {{&S74425[138], 13}, {NULL}};
const Segment SGI100808[] = {{&S97097[44], 11}, {NULL}};
const Segment SGI100809[] = {{&S97097[44], 28}, {NULL}};
const Segment SGI100810[] = {{&S97097[30], 6}, {NULL}};
const Segment SGI100811[] = {{&S97097[86], 6}, {NULL}};
const Segment SGI100812[] = {{&S97097[113], 13}, {NULL}};
const Segment SGI100813[] = {{&S74429[34], 17}, {NULL}};
const Segment SGI100814[] = {{&S74429[27], 6}, {NULL}};
const Segment SGI100815[] = {{&S74429[104], 9}, {NULL}};
const Segment SGI100816[] = {{&S74429[77], 6}, {NULL}};
const Segment SGI100817[] = {{&S74430[34], 23}, {NULL}};
const Segment SGI100818[] = {{&S74430[27], 6}, {NULL}};
const Segment SGI100819[] = {{&S74430[110], 9}, {NULL}};
const Segment SGI100820[] = {{&S74430[83], 6}, {NULL}};
const Segment SGI100821[] = {{&S74432[34], 7}, {NULL}};
const Segment SGI100822[] = {{&S74432[27], 6}, {NULL}};
const Segment SGI100823[] = {{&S74432[99], 9}, {NULL}};
const Segment SGI100824[] = {{&S74432[72], 6}, {NULL}};
const Segment SGI100825[] = {{&S74432[140], 8}, {NULL}};
const Segment SGI100826[] = {{&S74452[34], 6}, {NULL}};
const Segment SGI100827[] = {{&S74452[27], 6}, {NULL}};
const Segment SGI100828[] = {{&S74452[93], 9}, {NULL}};
const Segment SGI100829[] = {{&S74452[66], 6}, {NULL}};
const Segment SGI100830[] = {{&S97127[44], 6}, {NULL}};
const Segment SGI100831[] = {{&S97127[44], 18}, {NULL}};
const Segment SGI100832[] = {{&S97127[30], 6}, {NULL}};
const Segment SGI100833[] = {{&S97127[76], 6}, {NULL}};
const Segment SGI100834[] = {{&S97127[103], 13}, {NULL}};
const Segment SGI100835[] = {{&S74483[34], 10}, {NULL}};
const Segment SGI100836[] = {{&S74483[27], 6}, {NULL}};
const Segment SGI100837[] = {{&S74483[108], 9}, {NULL}};
const Segment SGI100838[] = {{&S74483[81], 6}, {NULL}};
const Segment SGI100839[] = {{&S97137[31], 3}, {NULL}};
const Segment SGI100840[] = {{&S97137[49], 3}, {NULL}};
const Segment SGI100841[] = {{&S97140[44], 12}, {NULL}};
const Segment SGI100842[] = {{&S97140[44], 24}, {NULL}};
const Segment SGI100843[] = {{&S97140[30], 6}, {NULL}};
const Segment SGI100844[] = {{&S97140[117], 3}, {NULL}};
const Segment SGI100845[] = {{&S97140[82], 6}, {NULL}};
const Segment SGI100846[] = {{&S97140[137], 8}, {NULL}};
const Segment SGI100847[] = {{&S97140[146], 13}, {NULL}};
const Segment SGI100848[] = {{&S74516[34], 11}, {NULL}};
const Segment SGI100849[] = {{&S74516[27], 6}, {NULL}};
const Segment SGI100850[] = {{&S74516[98], 9}, {NULL}};
const Segment SGI100851[] = {{&S74516[71], 6}, {NULL}};
const Segment SGI100852[] = {{&S97151[44], 10}, {NULL}};
const Segment SGI100853[] = {{&S97151[44], 27}, {NULL}};
const Segment SGI100854[] = {{&S97151[30], 6}, {NULL}};
const Segment SGI100855[] = {{&S97151[119], 3}, {NULL}};
const Segment SGI100856[] = {{&S97151[85], 5}, {NULL}};
const Segment SGI100857[] = {{&S97151[143], 7}, {NULL}};
const Segment SGI100858[] = {{&S97185[44], 10}, {NULL}};
const Segment SGI100859[] = {{&S97185[44], 27}, {NULL}};
const Segment SGI100860[] = {{&S97185[30], 6}, {NULL}};
const Segment SGI100861[] = {{&S97185[119], 3}, {NULL}};
const Segment SGI100862[] = {{&S97185[85], 5}, {NULL}};
const Segment SGI100863[] = {{&S97185[143], 7}, {NULL}};
const Segment SGI100864[] = {{&S97208[52], 24}, {NULL}};
const Segment SGI100865[] = {{&S97208[52], 36}, {NULL}};
const Segment SGI100866[] = {{&S89887[44], 9}, {NULL}};
const Segment SGI100867[] = {{&S97208[38], 6}, {NULL}};
const Segment SGI100868[] = {{&S89887[44], 20}, {NULL}};
const Segment SGI100869[] = {{&S89887[30], 6}, {NULL}};
const Segment SGI100870[] = {{&S97208[102], 5}, {NULL}};
const Segment SGI100871[] = {{&S89887[127], 3}, {NULL}};
const Segment SGI100872[] = {{&S89887[93], 5}, {NULL}};
const Segment SGI100873[] = {{&S97208[0], 14}, {NULL}};
const Segment SGI100874[] = {{&S89888[44], 12}, {NULL}};
const Segment SGI100875[] = {{&S89888[44], 23}, {NULL}};
const Segment SGI100876[] = {{&S89888[30], 6}, {NULL}};
const Segment SGI100877[] = {{&S89888[115], 3}, {NULL}};
const Segment SGI100878[] = {{&S89888[81], 5}, {NULL}};
const Segment SGI100879[] = {{&S97220[44], 10}, {NULL}};
const Segment SGI100880[] = {{&S97220[44], 27}, {NULL}};
const Segment SGI100881[] = {{&S97220[30], 6}, {NULL}};
const Segment SGI100882[] = {{&S97220[85], 6}, {NULL}};
const Segment SGI100883[] = {{&S97220[112], 13}, {NULL}};
const Segment SGI100884[] = {{&S74542[34], 15}, {NULL}};
const Segment SGI100885[] = {{&S74542[27], 6}, {NULL}};
const Segment SGI100886[] = {{&S74542[102], 9}, {NULL}};
const Segment SGI100887[] = {{&S74542[75], 6}, {NULL}};
const Segment SGI100888[] = {{&S97334[59], 5}, {NULL}};
const Segment SGI100889[] = {{&S97334[59], 17}, {NULL}};
const Segment SGI100890[] = {{&S97334[45], 6}, {NULL}};
const Segment SGI100891[] = {{&S97334[124], 3}, {NULL}};
const Segment SGI100892[] = {{&S97334[90], 5}, {NULL}};
const Segment SGI100893[] = {{&S97334[0], 14}, {NULL}};
const Segment SGI100894[] = {{&S97359[59], 10}, {NULL}};
const Segment SGI100895[] = {{&S97359[59], 27}, {NULL}};
const Segment SGI100896[] = {{&S97359[45], 6}, {NULL}};
const Segment SGI100897[] = {{&S97359[134], 3}, {NULL}};
const Segment SGI100898[] = {{&S97359[100], 5}, {NULL}};
const Segment SGI100899[] = {{&S97359[0], 14}, {NULL}};
const Segment SGI100900[] = {{&S97362[45], 7}, {NULL}};
const Segment SGI100901[] = {{&S97362[30], 6}, {NULL}};
const Segment SGI100902[] = {{&S97362[123], 3}, {NULL}};
const Segment SGI100903[] = {{&S97362[89], 5}, {NULL}};
const Segment SGI100904[] = {{&S97362[147], 13}, {NULL}};
const Segment SGI100905[] = {{&S97370[59], 20}, {NULL}};
const Segment SGI100906[] = {{&S97370[59], 32}, {NULL}};
const Segment SGI100907[] = {{&S97370[45], 6}, {NULL}};
const Segment SGI100908[] = {{&S97370[139], 3}, {NULL}};
const Segment SGI100909[] = {{&S97370[105], 5}, {NULL}};
const Segment SGI100910[] = {{&S97370[0], 14}, {NULL}};
const Segment SGI100911[] = {{&S97375[45], 5}, {NULL}};
const Segment SGI100912[] = {{&S97375[30], 6}, {NULL}};
const Segment SGI100913[] = {{&S97375[120], 3}, {NULL}};
const Segment SGI100914[] = {{&S97375[86], 5}, {NULL}};
const Segment SGI100915[] = {{&S97375[144], 13}, {NULL}};
const Segment SGI100916[] = {{&S74666[44], 6}, {NULL}};
const Segment SGI100917[] = {{&S74666[44], 22}, {NULL}};
const Segment SGI100918[] = {{&S74666[30], 6}, {NULL}};
const Segment SGI100919[] = {{&S74666[31], 3}, {NULL}};
const Segment SGI100920[] = {{&S74666[80], 6}, {NULL}};
const Segment SGI100921[] = {{&S97388[44], 7}, {NULL}};
const Segment SGI100922[] = {{&S97388[44], 31}, {NULL}};
const Segment SGI100923[] = {{&S97388[30], 6}, {NULL}};
const Segment SGI100924[] = {{&S74669[34], 12}, {NULL}};
const Segment SGI100925[] = {{&S74669[27], 6}, {NULL}};
const Segment SGI100926[] = {{&S97388[123], 3}, {NULL}};
const Segment SGI100927[] = {{&S97388[89], 5}, {NULL}};
const Segment SGI100928[] = {{&S74669[99], 9}, {NULL}};
const Segment SGI100929[] = {{&S74669[72], 6}, {NULL}};
const Segment SGI100930[] = {{&S97388[147], 11}, {NULL}};
const Segment SGI100931[] = {{&S97397[44], 6}, {NULL}};
const Segment SGI100932[] = {{&S97397[44], 18}, {NULL}};
const Segment SGI100933[] = {{&S97397[30], 6}, {NULL}};
const Segment SGI100934[] = {{&S97397[109], 3}, {NULL}};
const Segment SGI100935[] = {{&S97397[76], 5}, {NULL}};
const Segment SGI100936[] = {{&S97397[113], 14}, {NULL}};
const Segment SGI100937[] = {{&S74688[34], 7}, {NULL}};
const Segment SGI100938[] = {{&S74688[27], 6}, {NULL}};
const Segment SGI100939[] = {{&S74688[99], 9}, {NULL}};
const Segment SGI100940[] = {{&S74688[72], 6}, {NULL}};
const Segment SGI100941[] = {{&S97413[45], 11}, {NULL}};
const Segment SGI100942[] = {{&S97413[30], 6}, {NULL}};
const Segment SGI100943[] = {{&S97413[121], 3}, {NULL}};
const Segment SGI100944[] = {{&S97413[87], 5}, {NULL}};
const Segment SGI100945[] = {{&S97413[145], 13}, {NULL}};
const Segment SGI100946[] = {{&S97416[61], 5}, {NULL}};
const Segment SGI100947[] = {{&S97416[61], 26}, {NULL}};
const Segment SGI100948[] = {{&S97416[47], 6}, {NULL}};
const Segment SGI100949[] = {{&S97416[135], 3}, {NULL}};
const Segment SGI100950[] = {{&S97416[101], 5}, {NULL}};
const Segment SGI100951[] = {{&S97416[0], 14}, {NULL}};
const Segment SGI100952[] = {{&S97437[44], 16}, {NULL}};
const Segment SGI100953[] = {{&S97437[44], 28}, {NULL}};
const Segment SGI100954[] = {{&S97437[30], 6}, {NULL}};
const Segment SGI100955[] = {{&S97437[121], 3}, {NULL}};
const Segment SGI100956[] = {{&S97437[86], 6}, {NULL}};
const Segment SGI100957[] = {{&S97437[141], 8}, {NULL}};
const Segment SGI100958[] = {{&S97437[150], 13}, {NULL}};
const Segment SGI100959[] = {{&S74733[44], 15}, {NULL}};
const Segment SGI100960[] = {{&S74733[44], 27}, {NULL}};
const Segment SGI100961[] = {{&S74733[30], 6}, {NULL}};
const Segment SGI100962[] = {{&S74733[31], 3}, {NULL}};
const Segment SGI100963[] = {{&S74733[85], 6}, {NULL}};
const Segment SGI100964[] = {{&S74756[34], 14}, {NULL}};
const Segment SGI100965[] = {{&S74756[27], 6}, {NULL}};
const Segment SGI100966[] = {{&S74756[101], 9}, {NULL}};
const Segment SGI100967[] = {{&S74756[74], 6}, {NULL}};
const Segment SGI100968[] = {{&S74757[34], 15}, {NULL}};
const Segment SGI100969[] = {{&S74757[27], 6}, {NULL}};
const Segment SGI100970[] = {{&S74757[102], 9}, {NULL}};
const Segment SGI100971[] = {{&S74757[75], 6}, {NULL}};
const Segment SGI100972[] = {{&S90213[44], 9}, {NULL}};
const Segment SGI100973[] = {{&S90213[44], 20}, {NULL}};
const Segment SGI100974[] = {{&S90213[30], 6}, {NULL}};
const Segment SGI100975[] = {{&S90213[131], 3}, {NULL}};
const Segment SGI100976[] = {{&S90213[97], 5}, {NULL}};
const Segment SGI100977[] = {{&S90213[35], 1}, {NULL}};
const Segment SGI100978[] = {{&S90213[66], 11}, {NULL}};
const Segment SGI100979[] = {{&S90276[44], 10}, {NULL}};
const Segment SGI100980[] = {{&S90276[44], 30}, {NULL}};
const Segment SGI100981[] = {{&S90276[30], 6}, {NULL}};
const Segment SGI100982[] = {{&S90276[122], 3}, {NULL}};
const Segment SGI100983[] = {{&S90276[88], 5}, {NULL}};
const Segment SGI100984[] = {{&S90277[44], 10}, {NULL}};
const Segment SGI100985[] = {{&S90277[44], 30}, {NULL}};
const Segment SGI100986[] = {{&S90277[30], 6}, {NULL}};
const Segment SGI100987[] = {{&S90277[122], 3}, {NULL}};
const Segment SGI100988[] = {{&S90277[88], 5}, {NULL}};
const Segment SGI100989[] = {{&S97474[34], 18}, {NULL}};
const Segment SGI100990[] = {{&S97474[27], 6}, {NULL}};
const Segment SGI100991[] = {{&S74866[44], 14}, {NULL}};
const Segment SGI100992[] = {{&S74866[44], 38}, {NULL}};
const Segment SGI100993[] = {{&S74866[30], 6}, {NULL}};
const Segment SGI100994[] = {{&S97474[117], 2}, {NULL}};
const Segment SGI100995[] = {{&S97474[83], 6}, {NULL}};
const Segment SGI100996[] = {{&S97474[151], 8}, {NULL}};
const Segment SGI100997[] = {{&S74866[31], 3}, {NULL}};
const Segment SGI100998[] = {{&S74866[96], 6}, {NULL}};
const Segment SGI100999[] = {{&S74867[44], 14}, {NULL}};
const Segment SGI101000[] = {{&S74867[44], 38}, {NULL}};
const Segment SGI101001[] = {{&S74867[30], 6}, {NULL}};
const Segment SGI101002[] = {{&S74867[31], 3}, {NULL}};
const Segment SGI101003[] = {{&S74867[96], 6}, {NULL}};
const Segment SGI101004[] = {{&S74868[44], 7}, {NULL}};
const Segment SGI101005[] = {{&S74868[44], 20}, {NULL}};
const Segment SGI101006[] = {{&S74868[30], 6}, {NULL}};
const Segment SGI101007[] = {{&S74868[31], 3}, {NULL}};
const Segment SGI101008[] = {{&S74868[78], 6}, {NULL}};
const Segment SGI101009[] = {{&S74869[44], 7}, {NULL}};
const Segment SGI101010[] = {{&S74869[44], 20}, {NULL}};
const Segment SGI101011[] = {{&S74869[30], 6}, {NULL}};
const Segment SGI101012[] = {{&S74869[31], 3}, {NULL}};
const Segment SGI101013[] = {{&S74869[78], 6}, {NULL}};
const Segment SGI101014[] = {{&S74870[44], 7}, {NULL}};
const Segment SGI101015[] = {{&S74870[44], 20}, {NULL}};
const Segment SGI101016[] = {{&S74870[30], 6}, {NULL}};
const Segment SGI101017[] = {{&S74870[31], 3}, {NULL}};
const Segment SGI101018[] = {{&S74870[78], 6}, {NULL}};
const Segment SGI101019[] = {{&S97494[44], 9}, {NULL}};
const Segment SGI101020[] = {{&S97494[44], 21}, {NULL}};
const Segment SGI101021[] = {{&S97494[30], 6}, {NULL}};
const Segment SGI101022[] = {{&S97494[79], 6}, {NULL}};
const Segment SGI101023[] = {{&S97494[106], 13}, {NULL}};
const Segment SGI101024[] = {{&S74881[44], 11}, {NULL}};
const Segment SGI101025[] = {{&S74881[44], 23}, {NULL}};
const Segment SGI101026[] = {{&S74881[30], 6}, {NULL}};
const Segment SGI101027[] = {{&S74881[31], 3}, {NULL}};
const Segment SGI101028[] = {{&S74881[81], 6}, {NULL}};
const Segment SGI101029[] = {{&S90302[34], 7}, {NULL}};
const Segment SGI101030[] = {{&S90302[27], 6}, {NULL}};
const Segment SGI101031[] = {{&S90302[37], 2}, {NULL}};
const Segment SGI101032[] = {{&S90302[71], 6}, {NULL}};
const Segment SGI101033[] = {{&S90302[139], 8}, {NULL}};
const Segment SGI101034[] = {{&S97501[44], 6}, {NULL}};
const Segment SGI101035[] = {{&S97501[44], 18}, {NULL}};
const Segment SGI101036[] = {{&S97501[30], 6}, {NULL}};
const Segment SGI101037[] = {{&S97501[109], 3}, {NULL}};
const Segment SGI101038[] = {{&S97501[76], 5}, {NULL}};
const Segment SGI101039[] = {{&S97501[113], 14}, {NULL}};
const Segment SGI101040[] = {{&S97533[44], 7}, {NULL}};
const Segment SGI101041[] = {{&S97533[44], 31}, {NULL}};
const Segment SGI101042[] = {{&S97533[30], 6}, {NULL}};
const Segment SGI101043[] = {{&S97533[89], 6}, {NULL}};
const Segment SGI101044[] = {{&S97533[116], 13}, {NULL}};
const Segment SGI101045[] = {{&S97553[44], 18}, {NULL}};
const Segment SGI101046[] = {{&S97553[44], 35}, {NULL}};
const Segment SGI101047[] = {{&S97553[30], 6}, {NULL}};
const Segment SGI101048[] = {{&S97553[127], 3}, {NULL}};
const Segment SGI101049[] = {{&S74948[34], 11}, {NULL}};
const Segment SGI101050[] = {{&S74948[27], 6}, {NULL}};
const Segment SGI101051[] = {{&S97553[93], 5}, {NULL}};
const Segment SGI101052[] = {{&S97553[151], 7}, {NULL}};
const Segment SGI101053[] = {{&S74948[98], 9}, {NULL}};
const Segment SGI101054[] = {{&S74948[71], 6}, {NULL}};
const Segment SGI101055[] = {{&S97555[44], 10}, {NULL}};
const Segment SGI101056[] = {{&S97555[44], 27}, {NULL}};
const Segment SGI101057[] = {{&S97555[30], 6}, {NULL}};
const Segment SGI101058[] = {{&S97555[119], 3}, {NULL}};
const Segment SGI101059[] = {{&S97555[85], 5}, {NULL}};
const Segment SGI101060[] = {{&S97555[143], 7}, {NULL}};
const Segment SGI101061[] = {{&S97557[52], 10}, {NULL}};
const Segment SGI101062[] = {{&S97557[52], 27}, {NULL}};
const Segment SGI101063[] = {{&S97557[38], 6}, {NULL}};
const Segment SGI101064[] = {{&S97557[93], 5}, {NULL}};
const Segment SGI101065[] = {{&S97557[0], 14}, {NULL}};
const Segment SGI101066[] = {{&S74956[34], 11}, {NULL}};
const Segment SGI101067[] = {{&S74956[27], 6}, {NULL}};
const Segment SGI101068[] = {{&S74956[98], 9}, {NULL}};
const Segment SGI101069[] = {{&S74956[71], 6}, {NULL}};
const Segment SGI101070[] = {{&S97563[44], 11}, {NULL}};
const Segment SGI101071[] = {{&S97563[44], 23}, {NULL}};
const Segment SGI101072[] = {{&S97563[30], 6}, {NULL}};
const Segment SGI101073[] = {{&S97563[114], 3}, {NULL}};
const Segment SGI101074[] = {{&S97563[81], 5}, {NULL}};
const Segment SGI101075[] = {{&S97563[118], 14}, {NULL}};
const Segment SGI101076[] = {{&S90378[44], 22}, {NULL}};
const Segment SGI101077[] = {{&S90378[44], 32}, {NULL}};
const Segment SGI101078[] = {{&S90378[30], 6}, {NULL}};
const Segment SGI101079[] = {{&S90378[124], 3}, {NULL}};
const Segment SGI101080[] = {{&S90378[90], 5}, {NULL}};
const Segment SGI101081[] = {{&S74984[44], 13}, {NULL}};
const Segment SGI101082[] = {{&S74984[44], 25}, {NULL}};
const Segment SGI101083[] = {{&S74984[30], 6}, {NULL}};
const Segment SGI101084[] = {{&S74984[31], 3}, {NULL}};
const Segment SGI101085[] = {{&S74984[83], 6}, {NULL}};
const Segment SGI101086[] = {{&S74993[34], 18}, {NULL}};
const Segment SGI101087[] = {{&S74993[27], 6}, {NULL}};
const Segment SGI101088[] = {{&S74993[114], 9}, {NULL}};
const Segment SGI101089[] = {{&S74993[87], 6}, {NULL}};
const Segment SGI101090[] = {{&S74994[34], 15}, {NULL}};
const Segment SGI101091[] = {{&S74994[27], 6}, {NULL}};
const Segment SGI101092[] = {{&S74994[108], 9}, {NULL}};
const Segment SGI101093[] = {{&S74994[81], 6}, {NULL}};
const Segment SGI101094[] = {{&S75014[34], 13}, {NULL}};
const Segment SGI101095[] = {{&S75014[27], 6}, {NULL}};
const Segment SGI101096[] = {{&S75014[100], 9}, {NULL}};
const Segment SGI101097[] = {{&S75014[73], 6}, {NULL}};
const Segment SGI101098[] = {{&S75015[34], 12}, {NULL}};
const Segment SGI101099[] = {{&S75015[27], 6}, {NULL}};
const Segment SGI101100[] = {{&S75015[99], 9}, {NULL}};
const Segment SGI101101[] = {{&S75015[72], 6}, {NULL}};
const Segment SGI101102[] = {{&S75037[34], 25}, {NULL}};
const Segment SGI101103[] = {{&S75037[27], 6}, {NULL}};
const Segment SGI101104[] = {{&S97712[57], 10}, {NULL}};
const Segment SGI101105[] = {{&S97712[57], 27}, {NULL}};
const Segment SGI101106[] = {{&S97712[43], 6}, {NULL}};
const Segment SGI101107[] = {{&S97712[132], 3}, {NULL}};
const Segment SGI101108[] = {{&S97712[98], 5}, {NULL}};
const Segment SGI101109[] = {{&S97712[0], 12}, {NULL}};
const Segment SGI101110[] = {{&S97721[59], 4}, {NULL}};
const Segment SGI101111[] = {{&S97721[59], 16}, {NULL}};
const Segment SGI101112[] = {{&S97721[45], 6}, {NULL}};
const Segment SGI101113[] = {{&S97721[123], 3}, {NULL}};
const Segment SGI101114[] = {{&S97721[89], 5}, {NULL}};
const Segment SGI101115[] = {{&S97721[0], 14}, {NULL}};
const Segment SGI101116[] = {{&S90545[44], 8}, {NULL}};
const Segment SGI101117[] = {{&S90545[44], 18}, {NULL}};
const Segment SGI101118[] = {{&S90545[30], 6}, {NULL}};
const Segment SGI101119[] = {{&S90545[110], 3}, {NULL}};
const Segment SGI101120[] = {{&S90545[76], 5}, {NULL}};
const Segment SGI101121[] = {{&S75037[112], 9}, {NULL}};
const Segment SGI101122[] = {{&S75037[85], 6}, {NULL}};
const Segment SGI101123[] = {{&S97758[45], 4}, {NULL}};
const Segment SGI101124[] = {{&S97758[30], 6}, {NULL}};
const Segment SGI101125[] = {{&S97758[109], 3}, {NULL}};
const Segment SGI101126[] = {{&S97758[75], 5}, {NULL}};
const Segment SGI101127[] = {{&S97758[133], 13}, {NULL}};
const Segment SGI101128[] = {{&S97760[45], 5}, {NULL}};
const Segment SGI101129[] = {{&S97760[30], 6}, {NULL}};
const Segment SGI101130[] = {{&S97760[110], 3}, {NULL}};
const Segment SGI101131[] = {{&S97760[76], 5}, {NULL}};
const Segment SGI101132[] = {{&S97760[134], 13}, {NULL}};
const Segment SGI101133[] = {{&S75051[44], 8}, {NULL}};
const Segment SGI101134[] = {{&S75051[44], 20}, {NULL}};
const Segment SGI101135[] = {{&S75051[30], 6}, {NULL}};
const Segment SGI101136[] = {{&S75051[31], 3}, {NULL}};
const Segment SGI101137[] = {{&S75051[78], 6}, {NULL}};
const Segment SGI101138[] = {{&S97767[59], 11}, {NULL}};
const Segment SGI101139[] = {{&S97767[59], 23}, {NULL}};
const Segment SGI101140[] = {{&S97767[45], 6}, {NULL}};
const Segment SGI101141[] = {{&S97767[130], 3}, {NULL}};
const Segment SGI101142[] = {{&S97767[96], 5}, {NULL}};
const Segment SGI101143[] = {{&S97767[0], 14}, {NULL}};
const Segment SGI101144[] = {{&S75064[44], 10}, {NULL}};
const Segment SGI101145[] = {{&S75064[44], 22}, {NULL}};
const Segment SGI101146[] = {{&S75064[30], 6}, {NULL}};
const Segment SGI101147[] = {{&S75064[31], 3}, {NULL}};
const Segment SGI101148[] = {{&S75064[80], 9}, {NULL}};
const Segment SGI101149[] = {{&S75065[44], 11}, {NULL}};
const Segment SGI101150[] = {{&S75065[44], 23}, {NULL}};
const Segment SGI101151[] = {{&S75065[30], 6}, {NULL}};
const Segment SGI101152[] = {{&S75065[31], 3}, {NULL}};
const Segment SGI101153[] = {{&S75065[81], 9}, {NULL}};
const Segment SGI101154[] = {{&S75066[44], 10}, {NULL}};
const Segment SGI101155[] = {{&S75066[44], 22}, {NULL}};
const Segment SGI101156[] = {{&S75066[30], 6}, {NULL}};
const Segment SGI101157[] = {{&S75066[31], 3}, {NULL}};
const Segment SGI101158[] = {{&S75066[80], 6}, {NULL}};
const Segment SGI101159[] = {{&S97794[34], 10}, {NULL}};
const Segment SGI101160[] = {{&S97794[27], 6}, {NULL}};
const Segment SGI101161[] = {{&S97794[109], 2}, {NULL}};
const Segment SGI101162[] = {{&S97794[75], 6}, {NULL}};
const Segment SGI101163[] = {{&S97794[143], 8}, {NULL}};
const Segment SGI101164[] = {{&S97795[59], 10}, {NULL}};
const Segment SGI101165[] = {{&S97795[59], 27}, {NULL}};
const Segment SGI101166[] = {{&S97795[45], 6}, {NULL}};
const Segment SGI101167[] = {{&S97795[134], 3}, {NULL}};
const Segment SGI101168[] = {{&S97795[100], 5}, {NULL}};
const Segment SGI101169[] = {{&S97795[0], 14}, {NULL}};
const Segment SGI101170[] = {{&S97807[44], 11}, {NULL}};
const Segment SGI101171[] = {{&S97807[44], 23}, {NULL}};
const Segment SGI101172[] = {{&S97807[30], 6}, {NULL}};
const Segment SGI101173[] = {{&S97807[114], 3}, {NULL}};
const Segment SGI101174[] = {{&S97807[81], 5}, {NULL}};
const Segment SGI101175[] = {{&S97807[118], 14}, {NULL}};
const Segment SGI101176[] = {{&S90654[44], 7}, {NULL}};
const Segment SGI101177[] = {{&S90654[44], 24}, {NULL}};
const Segment SGI101178[] = {{&S90654[30], 6}, {NULL}};
const Segment SGI101179[] = {{&S90654[82], 6}, {NULL}};
const Segment SGI101180[] = {{&S90654[109], 13}, {NULL}};
const Segment SGI101181[] = {{&S97904[44], 10}, {NULL}};
const Segment SGI101182[] = {{&S97904[44], 27}, {NULL}};
const Segment SGI101183[] = {{&S97904[30], 6}, {NULL}};
const Segment SGI101184[] = {{&S97904[118], 3}, {NULL}};
const Segment SGI101185[] = {{&S97904[85], 5}, {NULL}};
const Segment SGI101186[] = {{&S97904[122], 14}, {NULL}};
const Segment SGI101187[] = {{&S97917[44], 10}, {NULL}};
const Segment SGI101188[] = {{&S97917[44], 27}, {NULL}};
const Segment SGI101189[] = {{&S97917[30], 6}, {NULL}};
const Segment SGI101190[] = {{&S97917[85], 6}, {NULL}};
const Segment SGI101191[] = {{&S97917[112], 13}, {NULL}};
const Segment SGI101192[] = {{&S90756[44], 10}, {NULL}};
const Segment SGI101193[] = {{&S90756[44], 20}, {NULL}};
const Segment SGI101194[] = {{&S90756[30], 6}, {NULL}};
const Segment SGI101195[] = {{&S90756[112], 3}, {NULL}};
const Segment SGI101196[] = {{&S90756[78], 5}, {NULL}};
const Segment SGI101197[] = {{&S97920[44], 10}, {NULL}};
const Segment SGI101198[] = {{&S97920[44], 27}, {NULL}};
const Segment SGI101199[] = {{&S97920[30], 6}, {NULL}};
const Segment SGI101200[] = {{&S90757[44], 10}, {NULL}};
const Segment SGI101201[] = {{&S90757[44], 27}, {NULL}};
const Segment SGI101202[] = {{&S90757[30], 6}, {NULL}};
const Segment SGI101203[] = {{&S97920[85], 6}, {NULL}};
const Segment SGI101204[] = {{&S97920[112], 13}, {NULL}};
const Segment SGI101205[] = {{&S90757[119], 3}, {NULL}};
const Segment SGI101206[] = {{&S90757[85], 5}, {NULL}};
const Segment SGI101207[] = {{&S97928[44], 10}, {NULL}};
const Segment SGI101208[] = {{&S97928[44], 31}, {NULL}};
const Segment SGI101209[] = {{&S97928[30], 6}, {NULL}};
const Segment SGI101210[] = {{&S97928[124], 3}, {NULL}};
const Segment SGI101211[] = {{&S97928[89], 6}, {NULL}};
const Segment SGI101212[] = {{&S97928[144], 8}, {NULL}};
const Segment SGI101213[] = {{&S97928[153], 13}, {NULL}};
const Segment SGI101214[] = {{&S90770[6], 4}, {NULL}};
const Segment SGI101215[] = {{&S90770[84], 5}, {NULL}};
const Segment SGI101216[] = {{&S97939[44], 10}, {NULL}};
const Segment SGI101217[] = {{&S97939[44], 27}, {NULL}};
const Segment SGI101218[] = {{&S97939[30], 6}, {NULL}};
const Segment SGI101219[] = {{&S97939[85], 6}, {NULL}};
const Segment SGI101220[] = {{&S97939[112], 13}, {NULL}};
const Segment SGI101221[] = {{&S75150[34], 6}, {NULL}};
const Segment SGI101222[] = {{&S97956[52], 10}, {NULL}};
const Segment SGI101223[] = {{&S75150[27], 6}, {NULL}};
const Segment SGI101224[] = {{&S97956[52], 27}, {NULL}};
const Segment SGI101225[] = {{&S97956[38], 6}, {NULL}};
const Segment SGI101226[] = {{&S75150[93], 9}, {NULL}};
const Segment SGI101227[] = {{&S75150[66], 6}, {NULL}};
const Segment SGI101228[] = {{&S97956[93], 5}, {NULL}};
const Segment SGI101229[] = {{&S97956[0], 14}, {NULL}};
const Segment SGI101230[] = {{&S97957[52], 5}, {NULL}};
const Segment SGI101231[] = {{&S97957[52], 17}, {NULL}};
const Segment SGI101232[] = {{&S75151[34], 20}, {NULL}};
const Segment SGI101233[] = {{&S97957[38], 6}, {NULL}};
const Segment SGI101234[] = {{&S75151[27], 6}, {NULL}};
const Segment SGI101235[] = {{&S97957[83], 5}, {NULL}};
const Segment SGI101236[] = {{&S75151[107], 9}, {NULL}};
const Segment SGI101237[] = {{&S75151[80], 6}, {NULL}};
const Segment SGI101238[] = {{&S97957[0], 14}, {NULL}};
const Segment SGI101239[] = {{&S97981[52], 9}, {NULL}};
const Segment SGI101240[] = {{&S97981[52], 21}, {NULL}};
const Segment SGI101241[] = {{&S97981[38], 6}, {NULL}};
const Segment SGI101242[] = {{&S97981[87], 5}, {NULL}};
const Segment SGI101243[] = {{&S97981[0], 14}, {NULL}};
const Segment SGI101244[] = {{&S97991[44], 10}, {NULL}};
const Segment SGI101245[] = {{&S97991[44], 27}, {NULL}};
const Segment SGI101246[] = {{&S97991[30], 6}, {NULL}};
const Segment SGI101247[] = {{&S97991[118], 3}, {NULL}};
const Segment SGI101248[] = {{&S97991[85], 5}, {NULL}};
const Segment SGI101249[] = {{&S97991[122], 14}, {NULL}};
const Segment SGI101250[] = {{&S75191[44], 11}, {NULL}};
const Segment SGI101251[] = {{&S75191[44], 20}, {NULL}};
const Segment SGI101252[] = {{&S75191[30], 6}, {NULL}};
const Segment SGI101253[] = {{&S75191[31], 3}, {NULL}};
const Segment SGI101254[] = {{&S75191[78], 6}, {NULL}};
const Segment SGI101255[] = {{&S90861[44], 7}, {NULL}};
const Segment SGI101256[] = {{&S90861[44], 24}, {NULL}};
const Segment SGI101257[] = {{&S90861[30], 6}, {NULL}};
const Segment SGI101258[] = {{&S90861[144], 3}, {NULL}};
const Segment SGI101259[] = {{&S90861[110], 5}, {NULL}};
const Segment SGI101260[] = {{&S90861[70], 11}, {NULL}};
const Segment SGI101261[] = {{&S90875[44], 7}, {NULL}};
const Segment SGI101262[] = {{&S90875[44], 18}, {NULL}};
const Segment SGI101263[] = {{&S90875[30], 6}, {NULL}};
const Segment SGI101264[] = {{&S90875[110], 3}, {NULL}};
const Segment SGI101265[] = {{&S90875[76], 5}, {NULL}};
const Segment SGI101266[] = {{&S90875[134], 7}, {NULL}};
const Segment SGI101267[] = {{&S90877[34], 7}, {NULL}};
const Segment SGI101268[] = {{&S90877[27], 6}, {NULL}};
const Segment SGI101269[] = {{&S90877[106], 2}, {NULL}};
const Segment SGI101270[] = {{&S90877[72], 6}, {NULL}};
const Segment SGI101271[] = {{&S90877[140], 8}, {NULL}};
const Segment SGI101272[] = {{&S75248[44], 11}, {NULL}};
const Segment SGI101273[] = {{&S75248[44], 23}, {NULL}};
const Segment SGI101274[] = {{&S75248[30], 6}, {NULL}};
const Segment SGI101275[] = {{&S75248[31], 3}, {NULL}};
const Segment SGI101276[] = {{&S75248[81], 6}, {NULL}};
const Segment SGI101277[] = {{&S75250[44], 11}, {NULL}};
const Segment SGI101278[] = {{&S75250[44], 23}, {NULL}};
const Segment SGI101279[] = {{&S75250[30], 6}, {NULL}};
const Segment SGI101280[] = {{&S75250[84], 6}, {NULL}};
const Segment SGI101281[] = {{&S75289[44], 5}, {NULL}};
const Segment SGI101282[] = {{&S75289[44], 17}, {NULL}};
const Segment SGI101283[] = {{&S75289[30], 6}, {NULL}};
const Segment SGI101284[] = {{&S75289[31], 3}, {NULL}};
const Segment SGI101285[] = {{&S75289[75], 6}, {NULL}};
const Segment SGI101286[] = {{&S98124[57], 24}, {NULL}};
const Segment SGI101287[] = {{&S98124[57], 36}, {NULL}};
const Segment SGI101288[] = {{&S98124[43], 6}, {NULL}};
const Segment SGI101289[] = {{&S98124[141], 3}, {NULL}};
const Segment SGI101290[] = {{&S98124[107], 5}, {NULL}};
const Segment SGI101291[] = {{&S98124[0], 12}, {NULL}};
const Segment SGI101292[] = {{&S90973[31], 5}, {NULL}};
const Segment SGI101293[] = {{&S90973[51], 3}, {NULL}};
const Segment SGI101294[] = {{&S98153[44], 10}, {NULL}};
const Segment SGI101295[] = {{&S98153[44], 27}, {NULL}};
const Segment SGI101296[] = {{&S98153[30], 6}, {NULL}};
const Segment SGI101297[] = {{&S98153[119], 3}, {NULL}};
const Segment SGI101298[] = {{&S98153[85], 5}, {NULL}};
const Segment SGI101299[] = {{&S98153[130], 14}, {NULL}};
const Segment SGI101300[] = {{&S98170[45], 5}, {NULL}};
const Segment SGI101301[] = {{&S98170[30], 6}, {NULL}};
const Segment SGI101302[] = {{&S98170[110], 3}, {NULL}};
const Segment SGI101303[] = {{&S98170[76], 5}, {NULL}};
const Segment SGI101304[] = {{&S98170[134], 13}, {NULL}};
const Segment SGI101305[] = {{&S75343[44], 19}, {NULL}};
const Segment SGI101306[] = {{&S75343[44], 36}, {NULL}};
const Segment SGI101307[] = {{&S75343[30], 6}, {NULL}};
const Segment SGI101308[] = {{&S75343[31], 3}, {NULL}};
const Segment SGI101309[] = {{&S75343[94], 6}, {NULL}};
const Segment SGI101310[] = {{&S75354[44], 9}, {NULL}};
const Segment SGI101311[] = {{&S75354[44], 21}, {NULL}};
const Segment SGI101312[] = {{&S75354[30], 6}, {NULL}};
const Segment SGI101313[] = {{&S75354[31], 3}, {NULL}};
const Segment SGI101314[] = {{&S75354[79], 6}, {NULL}};
const Segment SGI101315[] = {{&S75359[34], 14}, {NULL}};
const Segment SGI101316[] = {{&S75359[27], 6}, {NULL}};
const Segment SGI101317[] = {{&S75359[101], 9}, {NULL}};
const Segment SGI101318[] = {{&S75359[74], 6}, {NULL}};
const Segment SGI101319[] = {{&S98193[59], 5}, {NULL}};
const Segment SGI101320[] = {{&S98193[59], 17}, {NULL}};
const Segment SGI101321[] = {{&S98193[45], 6}, {NULL}};
const Segment SGI101322[] = {{&S98193[124], 3}, {NULL}};
const Segment SGI101323[] = {{&S98193[90], 5}, {NULL}};
const Segment SGI101324[] = {{&S98193[0], 14}, {NULL}};
const Segment SGI101325[] = {{&S98194[59], 16}, {NULL}};
const Segment SGI101326[] = {{&S98194[59], 28}, {NULL}};
const Segment SGI101327[] = {{&S98194[45], 6}, {NULL}};
const Segment SGI101328[] = {{&S98194[135], 3}, {NULL}};
const Segment SGI101329[] = {{&S98194[101], 5}, {NULL}};
const Segment SGI101330[] = {{&S98194[0], 14}, {NULL}};
const Segment SGI101331[] = {{&S98195[59], 17}, {NULL}};
const Segment SGI101332[] = {{&S98195[59], 43}, {NULL}};
const Segment SGI101333[] = {{&S98195[45], 6}, {NULL}};
const Segment SGI101334[] = {{&S98195[150], 3}, {NULL}};
const Segment SGI101335[] = {{&S98195[116], 5}, {NULL}};
const Segment SGI101336[] = {{&S98195[0], 14}, {NULL}};
const Segment SGI101337[] = {{&S75371[34], 14}, {NULL}};
const Segment SGI101338[] = {{&S75371[27], 6}, {NULL}};
const Segment SGI101339[] = {{&S75371[101], 9}, {NULL}};
const Segment SGI101340[] = {{&S75371[74], 6}, {NULL}};
const Segment SGI101341[] = {{&S98199[44], 10}, {NULL}};
const Segment SGI101342[] = {{&S98199[44], 27}, {NULL}};
const Segment SGI101343[] = {{&S98199[30], 6}, {NULL}};
const Segment SGI101344[] = {{&S75372[34], 15}, {NULL}};
const Segment SGI101345[] = {{&S75372[27], 6}, {NULL}};
const Segment SGI101346[] = {{&S98199[118], 3}, {NULL}};
const Segment SGI101347[] = {{&S98199[85], 5}, {NULL}};
const Segment SGI101348[] = {{&S98199[122], 14}, {NULL}};
const Segment SGI101349[] = {{&S75372[102], 9}, {NULL}};
const Segment SGI101350[] = {{&S75372[75], 6}, {NULL}};
const Segment SGI101351[] = {{&S98229[59], 14}, {NULL}};
const Segment SGI101352[] = {{&S98229[59], 31}, {NULL}};
const Segment SGI101353[] = {{&S98229[45], 6}, {NULL}};
const Segment SGI101354[] = {{&S98229[138], 3}, {NULL}};
const Segment SGI101355[] = {{&S98229[104], 5}, {NULL}};
const Segment SGI101356[] = {{&S98229[0], 14}, {NULL}};
const Segment SGI101357[] = {{&S98236[44], 11}, {NULL}};
const Segment SGI101358[] = {{&S98236[44], 28}, {NULL}};
const Segment SGI101359[] = {{&S98236[30], 6}, {NULL}};
const Segment SGI101360[] = {{&S98236[86], 6}, {NULL}};
const Segment SGI101361[] = {{&S98236[113], 13}, {NULL}};
const Segment SGI101362[] = {{&S75412[34], 10}, {NULL}};
const Segment SGI101363[] = {{&S75412[27], 6}, {NULL}};
const Segment SGI101364[] = {{&S75412[97], 9}, {NULL}};
const Segment SGI101365[] = {{&S75412[70], 6}, {NULL}};
const Segment SGI101366[] = {{&S98237[44], 10}, {NULL}};
const Segment SGI101367[] = {{&S98237[44], 27}, {NULL}};
const Segment SGI101368[] = {{&S98237[30], 6}, {NULL}};
const Segment SGI101369[] = {{&S98237[119], 3}, {NULL}};
const Segment SGI101370[] = {{&S98237[85], 5}, {NULL}};
const Segment SGI101371[] = {{&S98237[143], 11}, {NULL}};
const Segment SGI101372[] = {{&S75417[44], 9}, {NULL}};
const Segment SGI101373[] = {{&S75417[44], 21}, {NULL}};
const Segment SGI101374[] = {{&S75417[30], 6}, {NULL}};
const Segment SGI101375[] = {{&S75417[31], 3}, {NULL}};
const Segment SGI101376[] = {{&S75417[79], 6}, {NULL}};
const Segment SGI101377[] = {{&S98255[44], 5}, {NULL}};
const Segment SGI101378[] = {{&S98255[44], 26}, {NULL}};
const Segment SGI101379[] = {{&S98255[30], 6}, {NULL}};
const Segment SGI101380[] = {{&S98255[162], 3}, {NULL}};
const Segment SGI101381[] = {{&S83674[44], 12}, {NULL}};
const Segment SGI101382[] = {{&S98255[128], 3}, {NULL}};
const Segment SGI101383[] = {{&S83674[44], 25}, {NULL}};
const Segment SGI101384[] = {{&S83674[30], 6}, {NULL}};
const Segment SGI101385[] = {{&S98255[93], 6}, {NULL}};
const Segment SGI101386[] = {{&S98255[148], 8}, {NULL}};
const Segment SGI101387[] = {{&S83674[31], 3}, {NULL}};
const Segment SGI101388[] = {{&S83674[83], 6}, {NULL}};
const Segment SGI101389[] = {{&S83674[0], 13}, {NULL}};
const Segment SGI101390[] = {{&S98256[59], 11}, {NULL}};
const Segment SGI101391[] = {{&S98256[59], 28}, {NULL}};
const Segment SGI101392[] = {{&S98256[45], 6}, {NULL}};
const Segment SGI101393[] = {{&S98256[135], 3}, {NULL}};
const Segment SGI101394[] = {{&S98256[101], 5}, {NULL}};
const Segment SGI101395[] = {{&S98256[0], 14}, {NULL}};
const Segment SGI101396[] = {{&S98263[44], 10}, {NULL}};
const Segment SGI101397[] = {{&S98263[44], 27}, {NULL}};
const Segment SGI101398[] = {{&S98263[30], 6}, {NULL}};
const Segment SGI101399[] = {{&S98263[118], 3}, {NULL}};
const Segment SGI101400[] = {{&S98263[85], 5}, {NULL}};
const Segment SGI101401[] = {{&S98263[122], 14}, {NULL}};
const Segment SGI101402[] = {{&S98270[44], 10}, {NULL}};
const Segment SGI101403[] = {{&S98270[44], 27}, {NULL}};
const Segment SGI101404[] = {{&S98270[30], 6}, {NULL}};
const Segment SGI101405[] = {{&S98270[85], 6}, {NULL}};
const Segment SGI101406[] = {{&S98270[112], 13}, {NULL}};
const Segment SGI101407[] = {{&S98273[44], 4}, {NULL}};
const Segment SGI101408[] = {{&S98273[44], 25}, {NULL}};
const Segment SGI101409[] = {{&S98273[30], 6}, {NULL}};
const Segment SGI101410[] = {{&S98273[83], 6}, {NULL}};
const Segment SGI101411[] = {{&S98273[110], 13}, {NULL}};
const Segment SGI101412[] = {{&S75454[44], 10}, {NULL}};
const Segment SGI101413[] = {{&S75454[44], 22}, {NULL}};
const Segment SGI101414[] = {{&S75454[30], 6}, {NULL}};
const Segment SGI101415[] = {{&S75454[31], 3}, {NULL}};
const Segment SGI101416[] = {{&S75454[80], 6}, {NULL}};
const Segment SGI101417[] = {{&S75466[34], 13}, {NULL}};
const Segment SGI101418[] = {{&S75466[27], 6}, {NULL}};
const Segment SGI101419[] = {{&S75466[100], 9}, {NULL}};
const Segment SGI101420[] = {{&S75466[73], 6}, {NULL}};
const Segment SGI101421[] = {{&S91153[44], 7}, {NULL}};
const Segment SGI101422[] = {{&S91153[44], 17}, {NULL}};
const Segment SGI101423[] = {{&S91153[30], 6}, {NULL}};
const Segment SGI101424[] = {{&S91153[109], 3}, {NULL}};
const Segment SGI101425[] = {{&S91153[75], 5}, {NULL}};
const Segment SGI101426[] = {{&S91154[44], 7}, {NULL}};
const Segment SGI101427[] = {{&S91154[44], 17}, {NULL}};
const Segment SGI101428[] = {{&S91154[30], 6}, {NULL}};
const Segment SGI101429[] = {{&S91154[78], 6}, {NULL}};
const Segment SGI101430[] = {{&S75484[34], 11}, {NULL}};
const Segment SGI101431[] = {{&S75484[27], 6}, {NULL}};
const Segment SGI101432[] = {{&S75484[98], 9}, {NULL}};
const Segment SGI101433[] = {{&S75484[71], 6}, {NULL}};
const Segment SGI101434[] = {{&S75494[44], 10}, {NULL}};
const Segment SGI101435[] = {{&S75494[44], 22}, {NULL}};
const Segment SGI101436[] = {{&S75494[30], 6}, {NULL}};
const Segment SGI101437[] = {{&S75494[31], 3}, {NULL}};
const Segment SGI101438[] = {{&S75494[80], 6}, {NULL}};
const Segment SGI101439[] = {{&S75497[34], 10}, {NULL}};
const Segment SGI101440[] = {{&S75497[27], 6}, {NULL}};
const Segment SGI101441[] = {{&S75497[97], 9}, {NULL}};
const Segment SGI101442[] = {{&S75497[70], 6}, {NULL}};
const Segment SGI101443[] = {{&S91233[44], 10}, {NULL}};
const Segment SGI101444[] = {{&S91233[44], 25}, {NULL}};
const Segment SGI101445[] = {{&S91233[30], 6}, {NULL}};
const Segment SGI101446[] = {{&S91233[117], 3}, {NULL}};
const Segment SGI101447[] = {{&S91233[83], 5}, {NULL}};
const Segment SGI101448[] = {{&S91233[141], 7}, {NULL}};
const Segment SGI101449[] = {{&S75553[44], 11}, {NULL}};
const Segment SGI101450[] = {{&S75553[44], 23}, {NULL}};
const Segment SGI101451[] = {{&S75553[30], 6}, {NULL}};
const Segment SGI101452[] = {{&S75553[31], 3}, {NULL}};
const Segment SGI101453[] = {{&S75553[81], 6}, {NULL}};
const Segment SGI101454[] = {{&S98423[44], 10}, {NULL}};
const Segment SGI101455[] = {{&S98423[44], 27}, {NULL}};
const Segment SGI101456[] = {{&S98423[30], 6}, {NULL}};
const Segment SGI101457[] = {{&S98423[85], 6}, {NULL}};
const Segment SGI101458[] = {{&S98423[112], 13}, {NULL}};
const Segment SGI101459[] = {{&S75592[44], 10}, {NULL}};
const Segment SGI101460[] = {{&S75592[44], 22}, {NULL}};
const Segment SGI101461[] = {{&S75592[30], 6}, {NULL}};
const Segment SGI101462[] = {{&S75592[31], 3}, {NULL}};
const Segment SGI101463[] = {{&S75592[80], 6}, {NULL}};
const Segment SGI101464[] = {{&S98441[44], 12}, {NULL}};
const Segment SGI101465[] = {{&S98441[44], 24}, {NULL}};
const Segment SGI101466[] = {{&S98441[30], 6}, {NULL}};
const Segment SGI101467[] = {{&S98441[82], 6}, {NULL}};
const Segment SGI101468[] = {{&S98441[109], 13}, {NULL}};
const Segment SGI101469[] = {{&S91301[44], 12}, {NULL}};
const Segment SGI101470[] = {{&S91301[44], 23}, {NULL}};
const Segment SGI101471[] = {{&S91301[30], 6}, {NULL}};
const Segment SGI101472[] = {{&S91301[130], 3}, {NULL}};
const Segment SGI101473[] = {{&S91301[96], 5}, {NULL}};
const Segment SGI101474[] = {{&S91301[35], 1}, {NULL}};
const Segment SGI101475[] = {{&S91301[69], 11}, {NULL}};
const Segment SGI101476[] = {{&S98460[44], 11}, {NULL}};
const Segment SGI101477[] = {{&S98460[44], 32}, {NULL}};
const Segment SGI101478[] = {{&S98460[30], 6}, {NULL}};
const Segment SGI101479[] = {{&S98460[125], 3}, {NULL}};
const Segment SGI101480[] = {{&S98460[90], 6}, {NULL}};
const Segment SGI101481[] = {{&S98460[145], 8}, {NULL}};
const Segment SGI101482[] = {{&S98460[154], 13}, {NULL}};
const Segment SGI101483[] = {{&S98465[44], 24}, {NULL}};
const Segment SGI101484[] = {{&S98465[44], 36}, {NULL}};
const Segment SGI101485[] = {{&S98465[30], 6}, {NULL}};
const Segment SGI101486[] = {{&S98465[129], 3}, {NULL}};
const Segment SGI101487[] = {{&S98465[94], 6}, {NULL}};
const Segment SGI101488[] = {{&S98465[149], 8}, {NULL}};
const Segment SGI101489[] = {{&S98465[158], 13}, {NULL}};
const Segment SGI101490[] = {{&S98469[44], 10}, {NULL}};
const Segment SGI101491[] = {{&S98469[44], 27}, {NULL}};
const Segment SGI101492[] = {{&S98469[30], 6}, {NULL}};
const Segment SGI101493[] = {{&S98469[118], 3}, {NULL}};
const Segment SGI101494[] = {{&S98469[85], 5}, {NULL}};
const Segment SGI101495[] = {{&S98469[122], 14}, {NULL}};
const Segment SGI101496[] = {{&S98470[44], 10}, {NULL}};
const Segment SGI101497[] = {{&S98470[44], 27}, {NULL}};
const Segment SGI101498[] = {{&S98470[30], 6}, {NULL}};
const Segment SGI101499[] = {{&S98470[85], 6}, {NULL}};
const Segment SGI101500[] = {{&S75642[34], 7}, {NULL}};
const Segment SGI101501[] = {{&S98470[112], 13}, {NULL}};
const Segment SGI101502[] = {{&S75642[27], 6}, {NULL}};
const Segment SGI101503[] = {{&S75642[99], 9}, {NULL}};
const Segment SGI101504[] = {{&S75642[72], 6}, {NULL}};
const Segment SGI101505[] = {{&S75642[140], 8}, {NULL}};
const Segment SGI101506[] = {{&S91341[44], 6}, {NULL}};
const Segment SGI101507[] = {{&S91341[44], 24}, {NULL}};
const Segment SGI101508[] = {{&S91341[30], 6}, {NULL}};
const Segment SGI101509[] = {{&S91341[85], 6}, {NULL}};
const Segment SGI101510[] = {{&S89668[32], 10}, {NULL}};
const Segment SGI101511[] = {{&S89668[27], 4}, {NULL}};
const Segment SGI101512[] = {{&S89668[102], 2}, {NULL}};
const Segment SGI101513[] = {{&S89668[68], 6}, {NULL}};
const Segment SGI101514[] = {{&S89668[129], 8}, {NULL}};
const Segment SGI101515[] = {{&S75651[34], 7}, {NULL}};
const Segment SGI101516[] = {{&S75651[27], 6}, {NULL}};
const Segment SGI101517[] = {{&S75651[99], 9}, {NULL}};
const Segment SGI101518[] = {{&S75651[72], 6}, {NULL}};
const Segment SGI101519[] = {{&S98481[44], 9}, {NULL}};
const Segment SGI101520[] = {{&S98481[44], 21}, {NULL}};
const Segment SGI101521[] = {{&S98481[30], 6}, {NULL}};
const Segment SGI101522[] = {{&S98481[113], 3}, {NULL}};
const Segment SGI101523[] = {{&S98481[79], 5}, {NULL}};
const Segment SGI101524[] = {{&S98481[137], 7}, {NULL}};
const Segment SGI101525[] = {{&S75670[34], 11}, {NULL}};
const Segment SGI101526[] = {{&S75670[27], 6}, {NULL}};
const Segment SGI101527[] = {{&S75670[98], 9}, {NULL}};
const Segment SGI101528[] = {{&S75670[71], 6}, {NULL}};
const Segment SGI101529[] = {{&S75671[34], 15}, {NULL}};
const Segment SGI101530[] = {{&S75671[27], 6}, {NULL}};
const Segment SGI101531[] = {{&S75671[102], 9}, {NULL}};
const Segment SGI101532[] = {{&S75671[75], 6}, {NULL}};
const Segment SGI101533[] = {{&S75677[34], 7}, {NULL}};
const Segment SGI101534[] = {{&S75677[27], 6}, {NULL}};
const Segment SGI101535[] = {{&S75677[99], 9}, {NULL}};
const Segment SGI101536[] = {{&S75677[72], 6}, {NULL}};
const Segment SGI101537[] = {{&S75720[34], 13}, {NULL}};
const Segment SGI101538[] = {{&S75720[27], 6}, {NULL}};
const Segment SGI101539[] = {{&S75720[100], 9}, {NULL}};
const Segment SGI101540[] = {{&S75720[73], 6}, {NULL}};
const Segment SGI101541[] = {{&S75750[34], 17}, {NULL}};
const Segment SGI101542[] = {{&S75750[27], 6}, {NULL}};
const Segment SGI101543[] = {{&S75750[104], 9}, {NULL}};
const Segment SGI101544[] = {{&S75750[77], 6}, {NULL}};
const Segment SGI101545[] = {{&S75751[34], 23}, {NULL}};
const Segment SGI101546[] = {{&S75751[27], 6}, {NULL}};
const Segment SGI101547[] = {{&S75751[110], 9}, {NULL}};
const Segment SGI101548[] = {{&S75751[83], 6}, {NULL}};
const Segment SGI101549[] = {{&S98597[59], 10}, {NULL}};
const Segment SGI101550[] = {{&S98597[59], 27}, {NULL}};
const Segment SGI101551[] = {{&S98597[45], 6}, {NULL}};
const Segment SGI101552[] = {{&S98597[134], 3}, {NULL}};
const Segment SGI101553[] = {{&S98597[100], 5}, {NULL}};
const Segment SGI101554[] = {{&S98597[0], 14}, {NULL}};
const Segment SGI101555[] = {{&S98617[31], 8}, {NULL}};
const Segment SGI101556[] = {{&S98617[54], 3}, {NULL}};
const Segment SGI101557[] = {{&S98623[44], 10}, {NULL}};
const Segment SGI101558[] = {{&S98623[44], 27}, {NULL}};
const Segment SGI101559[] = {{&S98623[30], 6}, {NULL}};
const Segment SGI101560[] = {{&S98623[149], 4}, {NULL}};
const Segment SGI101561[] = {{&S98623[175], 3}, {NULL}};
const Segment SGI101562[] = {{&S98623[119], 3}, {&S98623[224], 5}, {NULL}};
const Segment SGI101563[] = {{&S98623[85], 5}, {NULL}};
const Segment SGI101564[] = {{&S75790[45], 10}, {NULL}};
const Segment SGI101565[] = {{&S75790[30], 6}, {NULL}};
const Segment SGI101566[] = {{&S75790[31], 3}, {NULL}};
const Segment SGI101567[] = {{&S75790[81], 5}, {NULL}};
const Segment SGI101568[] = {{&S75790[139], 13}, {NULL}};
const Segment SGI101569[] = {{&S98627[45], 4}, {NULL}};
const Segment SGI101570[] = {{&S98627[30], 6}, {NULL}};
const Segment SGI101571[] = {{&S98627[109], 3}, {NULL}};
const Segment SGI101572[] = {{&S98627[75], 5}, {NULL}};
const Segment SGI101573[] = {{&S98627[133], 13}, {NULL}};
const Segment SGI101574[] = {{&S98628[59], 14}, {NULL}};
const Segment SGI101575[] = {{&S98628[59], 26}, {NULL}};
const Segment SGI101576[] = {{&S98628[45], 6}, {NULL}};
const Segment SGI101577[] = {{&S98628[133], 3}, {NULL}};
const Segment SGI101578[] = {{&S98628[99], 5}, {NULL}};
const Segment SGI101579[] = {{&S98628[0], 14}, {NULL}};
const Segment SGI101580[] = {{&S98639[57], 10}, {NULL}};
const Segment SGI101581[] = {{&S98639[57], 27}, {NULL}};
const Segment SGI101582[] = {{&S98639[43], 6}, {NULL}};
const Segment SGI101583[] = {{&S98639[132], 3}, {NULL}};
const Segment SGI101584[] = {{&S98639[98], 5}, {NULL}};
const Segment SGI101585[] = {{&S98639[0], 12}, {NULL}};
const Segment SGI101586[] = {{&S75811[34], 15}, {NULL}};
const Segment SGI101587[] = {{&S75811[27], 6}, {NULL}};
const Segment SGI101588[] = {{&S75811[102], 9}, {NULL}};
const Segment SGI101589[] = {{&S75811[75], 6}, {NULL}};
const Segment SGI101590[] = {{&S98648[44], 10}, {NULL}};
const Segment SGI101591[] = {{&S98648[44], 27}, {NULL}};
const Segment SGI101592[] = {{&S98648[30], 6}, {NULL}};
const Segment SGI101593[] = {{&S91513[44], 8}, {NULL}};
const Segment SGI101594[] = {{&S91513[44], 19}, {NULL}};
const Segment SGI101595[] = {{&S91513[30], 6}, {NULL}};
const Segment SGI101596[] = {{&S98648[85], 6}, {NULL}};
const Segment SGI101597[] = {{&S91513[125], 3}, {NULL}};
const Segment SGI101598[] = {{&S98648[112], 13}, {NULL}};
const Segment SGI101599[] = {{&S91513[91], 5}, {NULL}};
const Segment SGI101600[] = {{&S91513[65], 3}, {NULL}};
const Segment SGI101601[] = {{&S98653[59], 10}, {NULL}};
const Segment SGI101602[] = {{&S98653[59], 27}, {NULL}};
const Segment SGI101603[] = {{&S98653[45], 6}, {NULL}};
const Segment SGI101604[] = {{&S98653[134], 3}, {NULL}};
const Segment SGI101605[] = {{&S98653[100], 5}, {NULL}};
const Segment SGI101606[] = {{&S98653[0], 14}, {NULL}};
const Segment SGI101607[] = {{&S98658[59], 10}, {NULL}};
const Segment SGI101608[] = {{&S98658[59], 27}, {NULL}};
const Segment SGI101609[] = {{&S98658[45], 6}, {NULL}};
const Segment SGI101610[] = {{&S98658[134], 3}, {NULL}};
const Segment SGI101611[] = {{&S98658[100], 5}, {NULL}};
const Segment SGI101612[] = {{&S98658[0], 14}, {NULL}};
const Segment SGI101613[] = {{&S98660[45], 11}, {NULL}};
const Segment SGI101614[] = {{&S98660[30], 6}, {NULL}};
const Segment SGI101615[] = {{&S98660[121], 3}, {NULL}};
const Segment SGI101616[] = {{&S98660[87], 5}, {NULL}};
const Segment SGI101617[] = {{&S98660[145], 13}, {NULL}};
const Segment SGI101618[] = {{&S75835[34], 7}, {NULL}};
const Segment SGI101619[] = {{&S75835[27], 6}, {NULL}};
const Segment SGI101620[] = {{&S75835[99], 9}, {NULL}};
const Segment SGI101621[] = {{&S75835[72], 6}, {NULL}};
const Segment SGI101622[] = {{&S98671[44], 10}, {NULL}};
const Segment SGI101623[] = {{&S98671[44], 27}, {NULL}};
const Segment SGI101624[] = {{&S98671[30], 6}, {NULL}};
const Segment SGI101625[] = {{&S98671[85], 6}, {NULL}};
const Segment SGI101626[] = {{&S98671[112], 13}, {NULL}};
const Segment SGI101627[] = {{&S98688[44], 10}, {NULL}};
const Segment SGI101628[] = {{&S98688[44], 22}, {NULL}};
const Segment SGI101629[] = {{&S98688[30], 6}, {NULL}};
const Segment SGI101630[] = {{&S98688[80], 6}, {NULL}};
const Segment SGI101631[] = {{&S98688[107], 13}, {NULL}};
const Segment SGI101632[] = {{&S98689[44], 10}, {NULL}};
const Segment SGI101633[] = {{&S98689[44], 27}, {NULL}};
const Segment SGI101634[] = {{&S98689[30], 6}, {NULL}};
const Segment SGI101635[] = {{&S98689[85], 6}, {NULL}};
const Segment SGI101636[] = {{&S98689[112], 13}, {NULL}};
const Segment SGI101637[] = {{&S98692[45], 9}, {NULL}};
const Segment SGI101638[] = {{&S98692[30], 6}, {NULL}};
const Segment SGI101639[] = {{&S98692[114], 3}, {NULL}};
const Segment SGI101640[] = {{&S98692[80], 5}, {NULL}};
const Segment SGI101641[] = {{&S98692[138], 13}, {NULL}};
const Segment SGI101642[] = {{&S98695[44], 9}, {NULL}};
const Segment SGI101643[] = {{&S98695[44], 21}, {NULL}};
const Segment SGI101644[] = {{&S98695[30], 6}, {NULL}};
const Segment SGI101645[] = {{&S98695[79], 6}, {NULL}};
const Segment SGI101646[] = {{&S98695[106], 13}, {NULL}};
const Segment SGI101647[] = {{&S98705[44], 10}, {NULL}};
const Segment SGI101648[] = {{&S98705[44], 27}, {NULL}};
const Segment SGI101649[] = {{&S98705[30], 6}, {NULL}};
const Segment SGI101650[] = {{&S98705[163], 3}, {NULL}};
const Segment SGI101651[] = {{&S98705[129], 3}, {NULL}};
const Segment SGI101652[] = {{&S98705[94], 6}, {NULL}};
const Segment SGI101653[] = {{&S98705[149], 8}, {NULL}};
const Segment SGI101654[] = {{&S75894[34], 41}, {NULL}};
const Segment SGI101655[] = {{&S75894[27], 6}, {NULL}};
const Segment SGI101656[] = {{&S75894[128], 9}, {NULL}};
const Segment SGI101657[] = {{&S75894[101], 6}, {NULL}};
const Segment SGI101658[] = {{&S75894[169], 8}, {NULL}};
const Segment SGI101659[] = {{&S98712[59], 10}, {NULL}};
const Segment SGI101660[] = {{&S98712[59], 22}, {NULL}};
const Segment SGI101661[] = {{&S98712[45], 6}, {NULL}};
const Segment SGI101662[] = {{&S98712[129], 3}, {NULL}};
const Segment SGI101663[] = {{&S98712[95], 5}, {NULL}};
const Segment SGI101664[] = {{&S98712[0], 14}, {NULL}};
const Segment SGI101665[] = {{&S98726[45], 10}, {NULL}};
const Segment SGI101666[] = {{&S98726[30], 6}, {NULL}};
const Segment SGI101667[] = {{&S98726[117], 3}, {NULL}};
const Segment SGI101668[] = {{&S98726[83], 5}, {NULL}};
const Segment SGI101669[] = {{&S98726[141], 13}, {NULL}};
const Segment SGI101670[] = {{&S75934[44], 7}, {NULL}};
const Segment SGI101671[] = {{&S75934[44], 19}, {NULL}};
const Segment SGI101672[] = {{&S75934[30], 6}, {NULL}};
const Segment SGI101673[] = {{&S75934[31], 3}, {NULL}};
const Segment SGI101674[] = {{&S75934[77], 6}, {NULL}};
const Segment SGI101675[] = {{&S75935[38], 7}, {NULL}};
const Segment SGI101676[] = {{&S75935[31], 6}, {NULL}};
const Segment SGI101677[] = {{&S75936[44], 9}, {NULL}};
const Segment SGI101678[] = {{&S75936[44], 21}, {NULL}};
const Segment SGI101679[] = {{&S75936[30], 6}, {NULL}};
const Segment SGI101680[] = {{&S75936[31], 3}, {NULL}};
const Segment SGI101681[] = {{&S75936[79], 6}, {NULL}};
const Segment SGI101682[] = {{&S75938[44], 8}, {NULL}};
const Segment SGI101683[] = {{&S75938[44], 20}, {NULL}};
const Segment SGI101684[] = {{&S75938[30], 6}, {NULL}};
const Segment SGI101685[] = {{&S75938[31], 3}, {NULL}};
const Segment SGI101686[] = {{&S75938[78], 6}, {NULL}};
const Segment SGI101687[] = {{&S98751[44], 24}, {NULL}};
const Segment SGI101688[] = {{&S98751[44], 36}, {NULL}};
const Segment SGI101689[] = {{&S98751[30], 6}, {NULL}};
const Segment SGI101690[] = {{&S98751[129], 3}, {NULL}};
const Segment SGI101691[] = {{&S98751[94], 6}, {NULL}};
const Segment SGI101692[] = {{&S98751[149], 8}, {NULL}};
const Segment SGI101693[] = {{&S98751[158], 13}, {NULL}};
const Segment SGI101694[] = {{&S75979[44], 6}, {NULL}};
const Segment SGI101695[] = {{&S75979[44], 18}, {NULL}};
const Segment SGI101696[] = {{&S75979[30], 6}, {NULL}};
const Segment SGI101697[] = {{&S75979[31], 3}, {NULL}};
const Segment SGI101698[] = {{&S75979[76], 6}, {NULL}};
const Segment SGI101699[] = {{&S75980[44], 6}, {NULL}};
const Segment SGI101700[] = {{&S75980[44], 18}, {NULL}};
const Segment SGI101701[] = {{&S75980[30], 6}, {NULL}};
const Segment SGI101702[] = {{&S75980[79], 6}, {NULL}};
const Segment SGI101703[] = {{&S75981[44], 6}, {NULL}};
const Segment SGI101704[] = {{&S75981[44], 18}, {NULL}};
const Segment SGI101705[] = {{&S75981[30], 6}, {NULL}};
const Segment SGI101706[] = {{&S75981[31], 3}, {NULL}};
const Segment SGI101707[] = {{&S75981[76], 6}, {NULL}};
const Segment SGI101708[] = {{&S75982[44], 18}, {NULL}};
const Segment SGI101709[] = {{&S75982[30], 6}, {NULL}};
const Segment SGI101710[] = {{&S91696[44], 7}, {NULL}};
const Segment SGI101711[] = {{&S91696[44], 25}, {NULL}};
const Segment SGI101712[] = {{&S91696[30], 6}, {NULL}};
const Segment SGI101713[] = {{&S91696[57], 3}, {NULL}};
const Segment SGI101714[] = {{&S91696[102], 5}, {NULL}};
const Segment SGI101715[] = {{&S91696[35], 1}, {NULL}};
const Segment SGI101716[] = {{&S91696[71], 11}, {NULL}};
const Segment SGI101717[] = {{&S98804[44], 18}, {NULL}};
const Segment SGI101718[] = {{&S98804[44], 35}, {NULL}};
const Segment SGI101719[] = {{&S98804[30], 6}, {NULL}};
const Segment SGI101720[] = {{&S98804[127], 3}, {NULL}};
const Segment SGI101721[] = {{&S98804[93], 5}, {NULL}};
const Segment SGI101722[] = {{&S98804[151], 11}, {NULL}};
const Segment SGI101723[] = {{&S75999[34], 13}, {NULL}};
const Segment SGI101724[] = {{&S75999[27], 6}, {NULL}};
const Segment SGI101725[] = {{&S75999[108], 9}, {NULL}};
const Segment SGI101726[] = {{&S75999[81], 6}, {NULL}};
const Segment SGI101727[] = {{&S76000[34], 17}, {NULL}};
const Segment SGI101728[] = {{&S76000[27], 6}, {NULL}};
const Segment SGI101729[] = {{&S76000[112], 9}, {NULL}};
const Segment SGI101730[] = {{&S76000[85], 6}, {NULL}};
const Segment SGI101731[] = {{&S98813[44], 10}, {NULL}};
const Segment SGI101732[] = {{&S98813[44], 27}, {NULL}};
const Segment SGI101733[] = {{&S98813[30], 6}, {NULL}};
const Segment SGI101734[] = {{&S98813[85], 6}, {NULL}};
const Segment SGI101735[] = {{&S98813[112], 13}, {NULL}};
const Segment SGI101736[] = {{&S76001[34], 17}, {NULL}};
const Segment SGI101737[] = {{&S76001[27], 6}, {NULL}};
const Segment SGI101738[] = {{&S76001[112], 9}, {NULL}};
const Segment SGI101739[] = {{&S76001[85], 6}, {NULL}};
const Segment SGI101740[] = {{&S91717[44], 17}, {NULL}};
const Segment SGI101741[] = {{&S91717[44], 28}, {NULL}};
const Segment SGI101742[] = {{&S91717[30], 6}, {NULL}};
const Segment SGI101743[] = {{&S91717[120], 3}, {NULL}};
const Segment SGI101744[] = {{&S91717[86], 5}, {NULL}};
const Segment SGI101745[] = {{&S98820[44], 6}, {NULL}};
const Segment SGI101746[] = {{&S98820[44], 18}, {NULL}};
const Segment SGI101747[] = {{&S98820[30], 6}, {NULL}};
const Segment SGI101748[] = {{&S98820[109], 3}, {NULL}};
const Segment SGI101749[] = {{&S98820[76], 5}, {NULL}};
const Segment SGI101750[] = {{&S98820[113], 14}, {NULL}};
const Segment SGI101751[] = {{&S91718[44], 7}, {NULL}};
const Segment SGI101752[] = {{&S91718[44], 26}, {NULL}};
const Segment SGI101753[] = {{&S91718[30], 6}, {NULL}};
const Segment SGI101754[] = {{&S91718[57], 3}, {NULL}};
const Segment SGI101755[] = {{&S91718[103], 5}, {NULL}};
const Segment SGI101756[] = {{&S91718[35], 1}, {NULL}};
const Segment SGI101757[] = {{&S91718[72], 11}, {NULL}};
const Segment SGI101758[] = {{&S98826[44], 11}, {NULL}};
const Segment SGI101759[] = {{&S98826[44], 23}, {NULL}};
const Segment SGI101760[] = {{&S98826[30], 6}, {NULL}};
const Segment SGI101761[] = {{&S98826[114], 3}, {NULL}};
const Segment SGI101762[] = {{&S98826[81], 5}, {NULL}};
const Segment SGI101763[] = {{&S98826[118], 14}, {NULL}};
const Segment SGI101764[] = {{&S98845[44], 11}, {NULL}};
const Segment SGI101765[] = {{&S98845[44], 28}, {NULL}};
const Segment SGI101766[] = {{&S98845[30], 6}, {NULL}};
const Segment SGI101767[] = {{&S98845[86], 6}, {NULL}};
const Segment SGI101768[] = {{&S98845[113], 13}, {NULL}};
const Segment SGI101769[] = {{&S76037[44], 15}, {NULL}};
const Segment SGI101770[] = {{&S76037[44], 27}, {NULL}};
const Segment SGI101771[] = {{&S76037[30], 6}, {NULL}};
const Segment SGI101772[] = {{&S91748[44], 7}, {NULL}};
const Segment SGI101773[] = {{&S76037[31], 3}, {NULL}};
const Segment SGI101774[] = {{&S91748[44], 18}, {NULL}};
const Segment SGI101775[] = {{&S98847[44], 9}, {NULL}};
const Segment SGI101776[] = {{&S91748[30], 6}, {NULL}};
const Segment SGI101777[] = {{&S76037[85], 6}, {NULL}};
const Segment SGI101778[] = {{&S98847[44], 21}, {NULL}};
const Segment SGI101779[] = {{&S98847[30], 6}, {NULL}};
const Segment SGI101780[] = {{&S91748[110], 3}, {NULL}};
const Segment SGI101781[] = {{&S91748[76], 5}, {NULL}};
const Segment SGI101782[] = {{&S98847[79], 6}, {NULL}};
const Segment SGI101783[] = {{&S91748[134], 11}, {NULL}};
const Segment SGI101784[] = {{&S98847[106], 13}, {NULL}};
const Segment SGI101785[] = {{&S76043[44], 9}, {NULL}};
const Segment SGI101786[] = {{&S76043[44], 21}, {NULL}};
const Segment SGI101787[] = {{&S76043[30], 6}, {NULL}};
const Segment SGI101788[] = {{&S76043[31], 3}, {NULL}};
const Segment SGI101789[] = {{&S76043[79], 6}, {NULL}};
const Segment SGI101790[] = {{&S98859[44], 5}, {NULL}};
const Segment SGI101791[] = {{&S98859[44], 26}, {NULL}};
const Segment SGI101792[] = {{&S98859[30], 6}, {NULL}};
const Segment SGI101793[] = {{&S98859[119], 3}, {NULL}};
const Segment SGI101794[] = {{&S98859[84], 6}, {NULL}};
const Segment SGI101795[] = {{&S98859[139], 8}, {NULL}};
const Segment SGI101796[] = {{&S98859[148], 13}, {NULL}};
const Segment SGI101797[] = {{&S89982[42], 10}, {NULL}};
const Segment SGI101798[] = {{&S89982[42], 22}, {NULL}};
const Segment SGI101799[] = {{&S89982[30], 4}, {NULL}};
const Segment SGI101800[] = {{&S89982[113], 3}, {NULL}};
const Segment SGI101801[] = {{&S89982[78], 6}, {NULL}};
const Segment SGI101802[] = {{&S89982[131], 13}, {NULL}};
const Segment SGI101803[] = {{&S76069[44], 19}, {NULL}};
const Segment SGI101804[] = {{&S76069[44], 31}, {NULL}};
const Segment SGI101805[] = {{&S76069[30], 6}, {NULL}};
const Segment SGI101806[] = {{&S76069[31], 3}, {NULL}};
const Segment SGI101807[] = {{&S76069[89], 6}, {NULL}};
const Segment SGI101808[] = {{&S76070[38], 19}, {NULL}};
const Segment SGI101809[] = {{&S76070[31], 6}, {NULL}};
const Segment SGI101810[] = {{&S98902[44], 10}, {NULL}};
const Segment SGI101811[] = {{&S98902[44], 27}, {NULL}};
const Segment SGI101812[] = {{&S98902[30], 6}, {NULL}};
const Segment SGI101813[] = {{&S98902[118], 3}, {NULL}};
const Segment SGI101814[] = {{&S98902[85], 5}, {NULL}};
const Segment SGI101815[] = {{&S98902[122], 14}, {NULL}};
const Segment SGI101816[] = {{&S76090[34], 16}, {NULL}};
const Segment SGI101817[] = {{&S76090[27], 6}, {NULL}};
const Segment SGI101818[] = {{&S76090[103], 9}, {NULL}};
const Segment SGI101819[] = {{&S76090[76], 6}, {NULL}};
const Segment SGI101820[] = {{&S76091[34], 16}, {NULL}};
const Segment SGI101821[] = {{&S76091[27], 6}, {NULL}};
const Segment SGI101822[] = {{&S98906[44], 4}, {NULL}};
const Segment SGI101823[] = {{&S76091[103], 9}, {NULL}};
const Segment SGI101824[] = {{&S98906[44], 16}, {NULL}};
const Segment SGI101825[] = {{&S76091[76], 6}, {NULL}};
const Segment SGI101826[] = {{&S98906[30], 6}, {NULL}};
const Segment SGI101827[] = {{&S98906[74], 6}, {NULL}};
const Segment SGI101828[] = {{&S98906[101], 13}, {NULL}};
const Segment SGI101829[] = {{&S76097[44], 10}, {NULL}};
const Segment SGI101830[] = {{&S76097[44], 22}, {NULL}};
const Segment SGI101831[] = {{&S76097[30], 6}, {NULL}};
const Segment SGI101832[] = {{&S76097[80], 6}, {NULL}};
const Segment SGI101833[] = {{&S76097[107], 13}, {NULL}};
const Segment SGI101834[] = {{&S98947[44], 10}, {NULL}};
const Segment SGI101835[] = {{&S98947[44], 27}, {NULL}};
const Segment SGI101836[] = {{&S98947[30], 6}, {NULL}};
const Segment SGI101837[] = {{&S98947[85], 6}, {NULL}};
const Segment SGI101838[] = {{&S98947[112], 13}, {NULL}};
const Segment SGI101839[] = {{&S76145[34], 9}, {NULL}};
const Segment SGI101840[] = {{&S76145[27], 6}, {NULL}};
const Segment SGI101841[] = {{&S76145[96], 9}, {NULL}};
const Segment SGI101842[] = {{&S76145[69], 6}, {NULL}};
const Segment SGI101843[] = {{&S98957[44], 10}, {NULL}};
const Segment SGI101844[] = {{&S98957[44], 27}, {NULL}};
const Segment SGI101845[] = {{&S98957[30], 6}, {NULL}};
const Segment SGI101846[] = {{&S98957[85], 6}, {NULL}};
const Segment SGI101847[] = {{&S98957[112], 13}, {NULL}};
const Segment SGI101848[] = {{&S76155[34], 11}, {NULL}};
const Segment SGI101849[] = {{&S76155[27], 6}, {NULL}};
const Segment SGI101850[] = {{&S76155[98], 9}, {NULL}};
const Segment SGI101851[] = {{&S76155[71], 6}, {NULL}};
const Segment SGI101852[] = {{&S98962[44], 9}, {NULL}};
const Segment SGI101853[] = {{&S98962[44], 21}, {NULL}};
const Segment SGI101854[] = {{&S98962[30], 6}, {NULL}};
const Segment SGI101855[] = {{&S98962[79], 6}, {NULL}};
const Segment SGI101856[] = {{&S98962[106], 13}, {NULL}};
const Segment SGI101857[] = {{&S91868[44], 12}, {NULL}};
const Segment SGI101858[] = {{&S91868[44], 23}, {NULL}};
const Segment SGI101859[] = {{&S91868[30], 6}, {NULL}};
const Segment SGI101860[] = {{&S91868[115], 3}, {NULL}};
const Segment SGI101861[] = {{&S91868[81], 5}, {NULL}};
const Segment SGI101862[] = {{&S76194[34], 13}, {NULL}};
const Segment SGI101863[] = {{&S76194[27], 6}, {NULL}};
const Segment SGI101864[] = {{&S76194[100], 9}, {NULL}};
const Segment SGI101865[] = {{&S76194[73], 6}, {NULL}};
const Segment SGI101866[] = {{&S76197[44], 8}, {NULL}};
const Segment SGI101867[] = {{&S76197[44], 20}, {NULL}};
const Segment SGI101868[] = {{&S76197[30], 6}, {NULL}};
const Segment SGI101869[] = {{&S76197[31], 3}, {NULL}};
const Segment SGI101870[] = {{&S76197[78], 6}, {NULL}};
const Segment SGI101871[] = {{&S76206[34], 13}, {NULL}};
const Segment SGI101872[] = {{&S76206[27], 6}, {NULL}};
const Segment SGI101873[] = {{&S76206[100], 9}, {NULL}};
const Segment SGI101874[] = {{&S76206[73], 6}, {NULL}};
const Segment SGI101875[] = {{&S76211[44], 14}, {NULL}};
const Segment SGI101876[] = {{&S76211[44], 26}, {NULL}};
const Segment SGI101877[] = {{&S76211[30], 6}, {NULL}};
const Segment SGI101878[] = {{&S76211[84], 6}, {NULL}};
const Segment SGI101879[] = {{&S76211[111], 13}, {NULL}};
const Segment SGI101880[] = {{&S76212[44], 15}, {NULL}};
const Segment SGI101881[] = {{&S76212[44], 27}, {NULL}};
const Segment SGI101882[] = {{&S76212[30], 6}, {NULL}};
const Segment SGI101883[] = {{&S76212[85], 6}, {NULL}};
const Segment SGI101884[] = {{&S76212[112], 13}, {NULL}};
const Segment SGI101885[] = {{&S76214[34], 11}, {NULL}};
const Segment SGI101886[] = {{&S76214[27], 6}, {NULL}};
const Segment SGI101887[] = {{&S76214[98], 9}, {NULL}};
const Segment SGI101888[] = {{&S76214[71], 6}, {NULL}};
const Segment SGI101889[] = {{&S76217[34], 14}, {NULL}};
const Segment SGI101890[] = {{&S76217[27], 6}, {NULL}};
const Segment SGI101891[] = {{&S76217[101], 9}, {NULL}};
const Segment SGI101892[] = {{&S76217[74], 6}, {NULL}};
const Segment SGI101893[] = {{&S76218[34], 23}, {NULL}};
const Segment SGI101894[] = {{&S76218[27], 6}, {NULL}};
const Segment SGI101895[] = {{&S76218[110], 9}, {NULL}};
const Segment SGI101896[] = {{&S76218[83], 6}, {NULL}};
const Segment SGI101897[] = {{&S91949[45], 7}, {NULL}};
const Segment SGI101898[] = {{&S91949[30], 6}, {NULL}};
const Segment SGI101899[] = {{&S91949[117], 3}, {NULL}};
const Segment SGI101900[] = {{&S91949[83], 5}, {NULL}};
const Segment SGI101901[] = {{&S91949[141], 13}, {NULL}};
const Segment SGI101902[] = {{&S99097[57], 14}, {NULL}};
const Segment SGI101903[] = {{&S99097[57], 26}, {NULL}};
const Segment SGI101904[] = {{&S99097[43], 6}, {NULL}};
const Segment SGI101905[] = {{&S99097[131], 3}, {NULL}};
const Segment SGI101906[] = {{&S99097[97], 5}, {NULL}};
const Segment SGI101907[] = {{&S99097[0], 12}, {NULL}};
const Segment SGI101908[] = {{&S76241[34], 25}, {NULL}};
const Segment SGI101909[] = {{&S76241[27], 6}, {NULL}};
const Segment SGI101910[] = {{&S76241[112], 9}, {NULL}};
const Segment SGI101911[] = {{&S76241[85], 6}, {NULL}};
const Segment SGI101912[] = {{&S76242[34], 26}, {NULL}};
const Segment SGI101913[] = {{&S76242[27], 6}, {NULL}};
const Segment SGI101914[] = {{&S76242[113], 9}, {NULL}};
const Segment SGI101915[] = {{&S76242[86], 6}, {NULL}};
const Segment SGI101916[] = {{&S76245[34], 24}, {NULL}};
const Segment SGI101917[] = {{&S76245[27], 6}, {NULL}};
const Segment SGI101918[] = {{&S76245[111], 9}, {NULL}};
const Segment SGI101919[] = {{&S76245[84], 6}, {NULL}};
const Segment SGI101920[] = {{&S99128[44], 10}, {NULL}};
const Segment SGI101921[] = {{&S99128[44], 27}, {NULL}};
const Segment SGI101922[] = {{&S99128[30], 6}, {NULL}};
const Segment SGI101923[] = {{&S99128[149], 4}, {NULL}};
const Segment SGI101924[] = {{&S99128[175], 3}, {NULL}};
const Segment SGI101925[] = {{&S99128[119], 3}, {&S99128[224], 5}, {NULL}};
const Segment SGI101926[] = {{&S99128[85], 5}, {NULL}};
const Segment SGI101927[] = {{&S76276[34], 24}, {NULL}};
const Segment SGI101928[] = {{&S76276[27], 6}, {NULL}};
const Segment SGI101929[] = {{&S76276[111], 9}, {NULL}};
const Segment SGI101930[] = {{&S76276[84], 6}, {NULL}};
const Segment SGI101931[] = {{&S76295[34], 11}, {NULL}};
const Segment SGI101932[] = {{&S76295[27], 6}, {NULL}};
const Segment SGI101933[] = {{&S76295[98], 9}, {NULL}};
const Segment SGI101934[] = {{&S76295[71], 6}, {NULL}};
const Segment SGI101935[] = {{&S99167[59], 11}, {NULL}};
const Segment SGI101936[] = {{&S99167[59], 23}, {NULL}};
const Segment SGI101937[] = {{&S99167[45], 6}, {NULL}};
const Segment SGI101938[] = {{&S99167[130], 3}, {NULL}};
const Segment SGI101939[] = {{&S99167[96], 5}, {NULL}};
const Segment SGI101940[] = {{&S99167[0], 14}, {NULL}};
const Segment SGI101941[] = {{&S99169[45], 9}, {NULL}};
const Segment SGI101942[] = {{&S99169[30], 6}, {NULL}};
const Segment SGI101943[] = {{&S99169[114], 3}, {NULL}};
const Segment SGI101944[] = {{&S99169[80], 5}, {NULL}};
const Segment SGI101945[] = {{&S99169[138], 13}, {NULL}};
const Segment SGI101946[] = {{&S99171[45], 18}, {NULL}};
const Segment SGI101947[] = {{&S99171[30], 6}, {NULL}};
const Segment SGI101948[] = {{&S99171[128], 3}, {NULL}};
const Segment SGI101949[] = {{&S99171[94], 5}, {NULL}};
const Segment SGI101950[] = {{&S99171[152], 13}, {NULL}};
const Segment SGI101951[] = {{&S99192[44], 10}, {NULL}};
const Segment SGI101952[] = {{&S99192[44], 27}, {NULL}};
const Segment SGI101953[] = {{&S99192[30], 6}, {NULL}};
const Segment SGI101954[] = {{&S99192[85], 6}, {NULL}};
const Segment SGI101955[] = {{&S99192[112], 13}, {NULL}};
const Segment SGI101956[] = {{&S90322[34], 7}, {NULL}};
const Segment SGI101957[] = {{&S90322[27], 6}, {NULL}};
const Segment SGI101958[] = {{&S90322[101], 2}, {NULL}};
const Segment SGI101959[] = {{&S90322[67], 6}, {NULL}};
const Segment SGI101960[] = {{&S90322[128], 8}, {NULL}};
const Segment SGI101961[] = {{&S99212[31], 9}, {NULL}};
const Segment SGI101962[] = {{&S99212[55], 3}, {NULL}};
const Segment SGI101963[] = {{&S92100[44], 7}, {NULL}};
const Segment SGI101964[] = {{&S92100[44], 18}, {NULL}};
const Segment SGI101965[] = {{&S92100[30], 6}, {NULL}};
const Segment SGI101966[] = {{&S92100[110], 3}, {NULL}};
const Segment SGI101967[] = {{&S92100[76], 5}, {NULL}};
const Segment SGI101968[] = {{&S99224[45], 10}, {NULL}};
const Segment SGI101969[] = {{&S99224[30], 6}, {NULL}};
const Segment SGI101970[] = {{&S99224[120], 3}, {NULL}};
const Segment SGI101971[] = {{&S99224[86], 5}, {NULL}};
const Segment SGI101972[] = {{&S99224[144], 13}, {NULL}};
const Segment SGI101973[] = {{&S92115[44], 9}, {NULL}};
const Segment SGI101974[] = {{&S92115[44], 19}, {NULL}};
const Segment SGI101975[] = {{&S92115[30], 6}, {NULL}};
const Segment SGI101976[] = {{&S92115[111], 3}, {NULL}};
const Segment SGI101977[] = {{&S92115[77], 5}, {NULL}};
const Segment SGI101978[] = {{&S99241[44], 10}, {NULL}};
const Segment SGI101979[] = {{&S99241[44], 27}, {NULL}};
const Segment SGI101980[] = {{&S99241[30], 6}, {NULL}};
const Segment SGI101981[] = {{&S99241[118], 3}, {NULL}};
const Segment SGI101982[] = {{&S99241[85], 5}, {NULL}};
const Segment SGI101983[] = {{&S99241[122], 14}, {NULL}};
const Segment SGI101984[] = {{&S76390[34], 12}, {NULL}};
const Segment SGI101985[] = {{&S76390[27], 6}, {NULL}};
const Segment SGI101986[] = {{&S76390[99], 9}, {NULL}};
const Segment SGI101987[] = {{&S76390[72], 6}, {NULL}};
const Segment SGI101988[] = {{&S76394[44], 12}, {NULL}};
const Segment SGI101989[] = {{&S76394[44], 24}, {NULL}};
const Segment SGI101990[] = {{&S76394[30], 6}, {NULL}};
const Segment SGI101991[] = {{&S76394[31], 3}, {NULL}};
const Segment SGI101992[] = {{&S76394[82], 6}, {NULL}};
const Segment SGI101993[] = {{&S99263[44], 5}, {NULL}};
const Segment SGI101994[] = {{&S99263[44], 17}, {NULL}};
const Segment SGI101995[] = {{&S99263[30], 6}, {NULL}};
const Segment SGI101996[] = {{&S99263[153], 3}, {NULL}};
const Segment SGI101997[] = {{&S99263[119], 3}, {NULL}};
const Segment SGI101998[] = {{&S99263[84], 6}, {NULL}};
const Segment SGI101999[] = {{&S99263[139], 8}, {NULL}};
const Segment SGI102000[] = {{&S90390[44], 7}, {NULL}};
const Segment SGI102001[] = {{&S90390[44], 26}, {NULL}};
const Segment SGI102002[] = {{&S90390[30], 6}, {NULL}};
const Segment SGI102003[] = {{&S99264[44], 13}, {NULL}};
const Segment SGI102004[] = {{&S90390[119], 3}, {NULL}};
const Segment SGI102005[] = {{&S99264[44], 25}, {NULL}};
const Segment SGI102006[] = {{&S99264[30], 6}, {NULL}};
const Segment SGI102007[] = {{&S90390[84], 6}, {NULL}};
const Segment SGI102008[] = {{&S99264[117], 3}, {NULL}};
const Segment SGI102009[] = {{&S99264[83], 5}, {NULL}};
const Segment SGI102010[] = {{&S90391[44], 7}, {NULL}};
const Segment SGI102011[] = {{&S90391[44], 26}, {NULL}};
const Segment SGI102012[] = {{&S90391[30], 6}, {NULL}};
const Segment SGI102013[] = {{&S90391[119], 3}, {NULL}};
const Segment SGI102014[] = {{&S90391[84], 6}, {NULL}};
const Segment SGI102015[] = {{&S99269[44], 14}, {NULL}};
const Segment SGI102016[] = {{&S99269[44], 26}, {NULL}};
const Segment SGI102017[] = {{&S99269[30], 6}, {NULL}};
const Segment SGI102018[] = {{&S99269[84], 6}, {NULL}};
const Segment SGI102019[] = {{&S99269[111], 13}, {NULL}};
const Segment SGI102020[] = {{&S92162[44], 7}, {NULL}};
const Segment SGI102021[] = {{&S92162[44], 18}, {NULL}};
const Segment SGI102022[] = {{&S92162[30], 6}, {NULL}};
const Segment SGI102023[] = {{&S92162[126], 3}, {NULL}};
const Segment SGI102024[] = {{&S92162[92], 5}, {NULL}};
const Segment SGI102025[] = {{&S92162[35], 1}, {NULL}};
const Segment SGI102026[] = {{&S92162[64], 11}, {NULL}};
const Segment SGI102027[] = {{&S99285[59], 10}, {NULL}};
const Segment SGI102028[] = {{&S99285[59], 22}, {NULL}};
const Segment SGI102029[] = {{&S99285[45], 6}, {NULL}};
const Segment SGI102030[] = {{&S99285[129], 3}, {NULL}};
const Segment SGI102031[] = {{&S99285[95], 5}, {NULL}};
const Segment SGI102032[] = {{&S99285[0], 14}, {NULL}};
const Segment SGI102033[] = {{&S99289[59], 14}, {NULL}};
const Segment SGI102034[] = {{&S99289[59], 26}, {NULL}};
const Segment SGI102035[] = {{&S99289[45], 6}, {NULL}};
const Segment SGI102036[] = {{&S99289[133], 3}, {NULL}};
const Segment SGI102037[] = {{&S99289[99], 5}, {NULL}};
const Segment SGI102038[] = {{&S99289[0], 14}, {NULL}};
const Segment SGI102039[] = {{&S99304[44], 10}, {NULL}};
const Segment SGI102040[] = {{&S99304[44], 27}, {NULL}};
const Segment SGI102041[] = {{&S99304[30], 6}, {NULL}};
const Segment SGI102042[] = {{&S99304[119], 3}, {NULL}};
const Segment SGI102043[] = {{&S99304[85], 5}, {NULL}};
const Segment SGI102044[] = {{&S99304[142], 10}, {NULL}};
const Segment SGI102045[] = {{&S99323[44], 10}, {NULL}};
const Segment SGI102046[] = {{&S99323[44], 31}, {NULL}};
const Segment SGI102047[] = {{&S99323[30], 6}, {NULL}};
const Segment SGI102048[] = {{&S99323[89], 6}, {NULL}};
const Segment SGI102049[] = {{&S99323[116], 13}, {NULL}};
const Segment SGI102050[] = {{&S99330[44], 5}, {NULL}};
const Segment SGI102051[] = {{&S99330[44], 17}, {NULL}};
const Segment SGI102052[] = {{&S99330[30], 6}, {NULL}};
const Segment SGI102053[] = {{&S99330[75], 6}, {NULL}};
const Segment SGI102054[] = {{&S99330[102], 13}, {NULL}};
const Segment SGI102055[] = {{&S99356[44], 14}, {NULL}};
const Segment SGI102056[] = {{&S99356[44], 26}, {NULL}};
const Segment SGI102057[] = {{&S99356[30], 6}, {NULL}};
const Segment SGI102058[] = {{&S99356[84], 6}, {NULL}};
const Segment SGI102059[] = {{&S99356[111], 13}, {NULL}};
const Segment SGI102060[] = {{&S76408[34], 15}, {NULL}};
const Segment SGI102061[] = {{&S76408[27], 6}, {NULL}};
const Segment SGI102062[] = {{&S76408[102], 9}, {NULL}};
const Segment SGI102063[] = {{&S76408[75], 6}, {NULL}};
const Segment SGI102064[] = {{&S76412[34], 41}, {NULL}};
const Segment SGI102065[] = {{&S76412[27], 6}, {NULL}};
const Segment SGI102066[] = {{&S76412[128], 9}, {NULL}};
const Segment SGI102067[] = {{&S76412[101], 6}, {NULL}};
const Segment SGI102068[] = {{&S76412[169], 8}, {NULL}};
const Segment SGI102069[] = {{&S76413[34], 13}, {NULL}};
const Segment SGI102070[] = {{&S76413[27], 6}, {NULL}};
const Segment SGI102071[] = {{&S76413[100], 9}, {NULL}};
const Segment SGI102072[] = {{&S76413[73], 6}, {NULL}};
const Segment SGI102073[] = {{&S76417[34], 9}, {NULL}};
const Segment SGI102074[] = {{&S76417[27], 6}, {NULL}};
const Segment SGI102075[] = {{&S76417[96], 9}, {NULL}};
const Segment SGI102076[] = {{&S76417[69], 6}, {NULL}};
const Segment SGI102077[] = {{&S99361[44], 6}, {NULL}};
const Segment SGI102078[] = {{&S99361[44], 18}, {NULL}};
const Segment SGI102079[] = {{&S99361[30], 6}, {NULL}};
const Segment SGI102080[] = {{&S99361[76], 6}, {NULL}};
const Segment SGI102081[] = {{&S99361[103], 13}, {NULL}};
const Segment SGI102082[] = {{&S76456[44], 22}, {NULL}};
const Segment SGI102083[] = {{&S76456[44], 34}, {NULL}};
const Segment SGI102084[] = {{&S76456[30], 6}, {NULL}};
const Segment SGI102085[] = {{&S99363[44], 10}, {NULL}};
const Segment SGI102086[] = {{&S99363[44], 27}, {NULL}};
const Segment SGI102087[] = {{&S99363[30], 6}, {NULL}};
const Segment SGI102088[] = {{&S76456[31], 3}, {NULL}};
const Segment SGI102089[] = {{&S76456[92], 6}, {NULL}};
const Segment SGI102090[] = {{&S99363[119], 3}, {NULL}};
const Segment SGI102091[] = {{&S99363[85], 5}, {NULL}};
const Segment SGI102092[] = {{&S99363[143], 7}, {NULL}};
const Segment SGI102093[] = {{&S99370[44], 10}, {NULL}};
const Segment SGI102094[] = {{&S99370[44], 22}, {NULL}};
const Segment SGI102095[] = {{&S99370[30], 6}, {NULL}};
const Segment SGI102096[] = {{&S99370[80], 6}, {NULL}};
const Segment SGI102097[] = {{&S99370[107], 13}, {NULL}};
const Segment SGI102098[] = {{&S76499[44], 12}, {NULL}};
const Segment SGI102099[] = {{&S76499[44], 24}, {NULL}};
const Segment SGI102100[] = {{&S76499[30], 6}, {NULL}};
const Segment SGI102101[] = {{&S76499[31], 3}, {NULL}};
const Segment SGI102102[] = {{&S76499[82], 6}, {NULL}};
const Segment SGI102103[] = {{&S76500[44], 7}, {NULL}};
const Segment SGI102104[] = {{&S76500[44], 19}, {NULL}};
const Segment SGI102105[] = {{&S76500[30], 6}, {NULL}};
const Segment SGI102106[] = {{&S76500[31], 3}, {NULL}};
const Segment SGI102107[] = {{&S76500[77], 6}, {NULL}};
const Segment SGI102108[] = {{&S99414[44], 18}, {NULL}};
const Segment SGI102109[] = {{&S99414[44], 30}, {NULL}};
const Segment SGI102110[] = {{&S99414[30], 6}, {NULL}};
const Segment SGI102111[] = {{&S99414[122], 3}, {NULL}};
const Segment SGI102112[] = {{&S99414[88], 5}, {NULL}};
const Segment SGI102113[] = {{&S99414[146], 7}, {NULL}};
const Segment SGI102114[] = {{&S76528[34], 6}, {NULL}};
const Segment SGI102115[] = {{&S76528[27], 6}, {NULL}};
const Segment SGI102116[] = {{&S99427[45], 8}, {NULL}};
const Segment SGI102117[] = {{&S99427[30], 6}, {NULL}};
const Segment SGI102118[] = {{&S76528[93], 9}, {NULL}};
const Segment SGI102119[] = {{&S76528[66], 6}, {NULL}};
const Segment SGI102120[] = {{&S99427[113], 3}, {NULL}};
const Segment SGI102121[] = {{&S99427[79], 5}, {NULL}};
const Segment SGI102122[] = {{&S99427[137], 13}, {NULL}};
const Segment SGI102123[] = {{&S76530[34], 14}, {NULL}};
const Segment SGI102124[] = {{&S76530[27], 6}, {NULL}};
const Segment SGI102125[] = {{&S76530[106], 9}, {NULL}};
const Segment SGI102126[] = {{&S76530[79], 6}, {NULL}};
const Segment SGI102127[] = {{&S99435[44], 10}, {NULL}};
const Segment SGI102128[] = {{&S99435[44], 27}, {NULL}};
const Segment SGI102129[] = {{&S99435[30], 6}, {NULL}};
const Segment SGI102130[] = {{&S99435[119], 3}, {NULL}};
const Segment SGI102131[] = {{&S99435[85], 5}, {NULL}};
const Segment SGI102132[] = {{&S99435[143], 13}, {NULL}};
const Segment SGI102133[] = {{&S99438[44], 10}, {NULL}};
const Segment SGI102134[] = {{&S99438[44], 27}, {NULL}};
const Segment SGI102135[] = {{&S99438[30], 6}, {NULL}};
const Segment SGI102136[] = {{&S76541[34], 9}, {NULL}};
const Segment SGI102137[] = {{&S99438[119], 3}, {NULL}};
const Segment SGI102138[] = {{&S76541[27], 6}, {NULL}};
const Segment SGI102139[] = {{&S99438[85], 5}, {NULL}};
const Segment SGI102140[] = {{&S99438[143], 7}, {NULL}};
const Segment SGI102141[] = {{&S76541[96], 9}, {NULL}};
const Segment SGI102142[] = {{&S76541[69], 6}, {NULL}};
const Segment SGI102143[] = {{&S99448[44], 20}, {NULL}};
const Segment SGI102144[] = {{&S99448[44], 37}, {NULL}};
const Segment SGI102145[] = {{&S99448[30], 6}, {NULL}};
const Segment SGI102146[] = {{&S99448[129], 3}, {NULL}};
const Segment SGI102147[] = {{&S99448[95], 5}, {NULL}};
const Segment SGI102148[] = {{&S99448[153], 7}, {NULL}};
const Segment SGI102149[] = {{&S99449[52], 4}, {NULL}};
const Segment SGI102150[] = {{&S99449[52], 16}, {NULL}};
const Segment SGI102151[] = {{&S99449[38], 6}, {NULL}};
const Segment SGI102152[] = {{&S99449[82], 5}, {NULL}};
const Segment SGI102153[] = {{&S99449[0], 14}, {NULL}};
const Segment SGI102154[] = {{&S84761[44], 12}, {NULL}};
const Segment SGI102155[] = {{&S76572[44], 11}, {NULL}};
const Segment SGI102156[] = {{&S84761[44], 33}, {NULL}};
const Segment SGI102157[] = {{&S76572[44], 23}, {NULL}};
const Segment SGI102158[] = {{&S84761[30], 6}, {NULL}};
const Segment SGI102159[] = {{&S76572[30], 6}, {NULL}};
const Segment SGI102160[] = {{&S84761[31], 3}, {NULL}};
const Segment SGI102161[] = {{&S76572[31], 3}, {NULL}};
const Segment SGI102162[] = {{&S84761[91], 5}, {NULL}};
const Segment SGI102163[] = {{&S76572[81], 6}, {NULL}};
const Segment SGI102164[] = {{&S84761[0], 13}, {NULL}};
const Segment SGI102165[] = {{&S76573[44], 11}, {NULL}};
const Segment SGI102166[] = {{&S76573[44], 23}, {NULL}};
const Segment SGI102167[] = {{&S76573[30], 6}, {NULL}};
const Segment SGI102168[] = {{&S76573[84], 6}, {NULL}};
const Segment SGI102169[] = {{&S76583[45], 11}, {NULL}};
const Segment SGI102170[] = {{&S76583[30], 6}, {NULL}};
const Segment SGI102171[] = {{&S76583[31], 3}, {NULL}};
const Segment SGI102172[] = {{&S76583[82], 5}, {NULL}};
const Segment SGI102173[] = {{&S76583[140], 13}, {NULL}};
const Segment SGI102174[] = {{&S76590[34], 15}, {NULL}};
const Segment SGI102175[] = {{&S76590[27], 6}, {NULL}};
const Segment SGI102176[] = {{&S99487[44], 12}, {NULL}};
const Segment SGI102177[] = {{&S76590[102], 9}, {NULL}};
const Segment SGI102178[] = {{&S99487[44], 24}, {NULL}};
const Segment SGI102179[] = {{&S76590[75], 6}, {NULL}};
const Segment SGI102180[] = {{&S99487[30], 6}, {NULL}};
const Segment SGI102181[] = {{&S99487[117], 3}, {NULL}};
const Segment SGI102182[] = {{&S99487[82], 6}, {NULL}};
const Segment SGI102183[] = {{&S99487[137], 8}, {NULL}};
const Segment SGI102184[] = {{&S99487[146], 13}, {NULL}};
const Segment SGI102185[] = {{&S76626[34], 6}, {NULL}};
const Segment SGI102186[] = {{&S76626[27], 6}, {NULL}};
const Segment SGI102187[] = {{&S76626[93], 9}, {NULL}};
const Segment SGI102188[] = {{&S76626[66], 6}, {NULL}};
const Segment SGI102189[] = {{&S76656[44], 13}, {NULL}};
const Segment SGI102190[] = {{&S76656[44], 25}, {NULL}};
const Segment SGI102191[] = {{&S76656[30], 6}, {NULL}};
const Segment SGI102192[] = {{&S76656[31], 3}, {NULL}};
const Segment SGI102193[] = {{&S76656[83], 6}, {NULL}};
const Segment SGI102194[] = {{&S76669[34], 41}, {NULL}};
const Segment SGI102195[] = {{&S76669[27], 6}, {NULL}};
const Segment SGI102196[] = {{&S76669[128], 9}, {NULL}};
const Segment SGI102197[] = {{&S76669[101], 6}, {NULL}};
const Segment SGI102198[] = {{&S76669[169], 8}, {NULL}};
const Segment SGI102199[] = {{&S76673[34], 14}, {NULL}};
const Segment SGI102200[] = {{&S76673[27], 6}, {NULL}};
const Segment SGI102201[] = {{&S76673[101], 9}, {NULL}};
const Segment SGI102202[] = {{&S76673[74], 6}, {NULL}};
const Segment SGI102203[] = {{&S76673[142], 8}, {NULL}};
const Segment SGI102204[] = {{&S76697[44], 22}, {NULL}};
const Segment SGI102205[] = {{&S76697[30], 6}, {NULL}};
const Segment SGI102206[] = {{&S76697[31], 3}, {NULL}};
const Segment SGI102207[] = {{&S76697[82], 6}, {NULL}};
const Segment SGI102208[] = {{&S76698[44], 22}, {NULL}};
const Segment SGI102209[] = {{&S76698[30], 6}, {NULL}};
const Segment SGI102210[] = {{&S76698[31], 3}, {NULL}};
const Segment SGI102211[] = {{&S76698[82], 6}, {NULL}};
const Segment SGI102212[] = {{&S76699[44], 10}, {NULL}};
const Segment SGI102213[] = {{&S76699[30], 6}, {NULL}};
const Segment SGI102214[] = {{&S76699[31], 3}, {NULL}};
const Segment SGI102215[] = {{&S76699[70], 6}, {NULL}};
const Segment SGI102216[] = {{&S76700[44], 11}, {NULL}};
const Segment SGI102217[] = {{&S76700[44], 23}, {NULL}};
const Segment SGI102218[] = {{&S76700[30], 6}, {NULL}};
const Segment SGI102219[] = {{&S76700[84], 6}, {NULL}};
const Segment SGI102220[] = {{&S76701[44], 11}, {NULL}};
const Segment SGI102221[] = {{&S76701[44], 23}, {NULL}};
const Segment SGI102222[] = {{&S76701[30], 6}, {NULL}};
const Segment SGI102223[] = {{&S76701[84], 4}, {NULL}};
const Segment SGI102224[] = {{&S76701[84], 3}, {NULL}};
const Segment SGI102225[] = {{&S76702[44], 22}, {NULL}};
const Segment SGI102226[] = {{&S76702[30], 6}, {NULL}};
const Segment SGI102227[] = {{&S76702[31], 3}, {NULL}};
const Segment SGI102228[] = {{&S76702[82], 6}, {NULL}};
const Segment SGI102229[] = {{&S90827[44], 5}, {NULL}};
const Segment SGI102230[] = {{&S90827[44], 17}, {NULL}};
const Segment SGI102231[] = {{&S90827[30], 6}, {NULL}};
const Segment SGI102232[] = {{&S90827[110], 3}, {NULL}};
const Segment SGI102233[] = {{&S90827[75], 6}, {NULL}};
const Segment SGI102234[] = {{&S76744[44], 12}, {NULL}};
const Segment SGI102235[] = {{&S76744[44], 24}, {NULL}};
const Segment SGI102236[] = {{&S76744[30], 6}, {NULL}};
const Segment SGI102237[] = {{&S76744[31], 3}, {NULL}};
const Segment SGI102238[] = {{&S76744[82], 6}, {NULL}};
const Segment SGI102239[] = {{&S99606[45], 10}, {NULL}};
const Segment SGI102240[] = {{&S99606[30], 6}, {NULL}};
const Segment SGI102241[] = {{&S99606[120], 3}, {NULL}};
const Segment SGI102242[] = {{&S99606[86], 5}, {NULL}};
const Segment SGI102243[] = {{&S99606[144], 13}, {NULL}};
const Segment SGI102244[] = {{&S76775[44], 12}, {NULL}};
const Segment SGI102245[] = {{&S76775[44], 23}, {NULL}};
const Segment SGI102246[] = {{&S76775[30], 6}, {NULL}};
const Segment SGI102247[] = {{&S76775[31], 3}, {NULL}};
const Segment SGI102248[] = {{&S76775[81], 6}, {NULL}};
const Segment SGI102249[] = {{&S76776[44], 12}, {NULL}};
const Segment SGI102250[] = {{&S76776[44], 24}, {NULL}};
const Segment SGI102251[] = {{&S76776[30], 6}, {NULL}};
const Segment SGI102252[] = {{&S76776[31], 3}, {NULL}};
const Segment SGI102253[] = {{&S76776[82], 6}, {NULL}};
const Segment SGI102254[] = {{&S76814[44], 11}, {NULL}};
const Segment SGI102255[] = {{&S76814[44], 23}, {NULL}};
const Segment SGI102256[] = {{&S76814[30], 6}, {NULL}};
const Segment SGI102257[] = {{&S76814[31], 3}, {NULL}};
const Segment SGI102258[] = {{&S76814[81], 6}, {NULL}};
const Segment SGI102259[] = {{&S76815[44], 11}, {NULL}};
const Segment SGI102260[] = {{&S76815[44], 23}, {NULL}};
const Segment SGI102261[] = {{&S76815[30], 6}, {NULL}};
const Segment SGI102262[] = {{&S76815[84], 6}, {NULL}};
const Segment SGI102263[] = {{&S99660[45], 10}, {NULL}};
const Segment SGI102264[] = {{&S99660[30], 6}, {NULL}};
const Segment SGI102265[] = {{&S99660[120], 3}, {NULL}};
const Segment SGI102266[] = {{&S99660[86], 5}, {NULL}};
const Segment SGI102267[] = {{&S99660[144], 13}, {NULL}};
const Segment SGI102268[] = {{&S99662[45], 14}, {NULL}};
const Segment SGI102269[] = {{&S99662[30], 6}, {NULL}};
const Segment SGI102270[] = {{&S99662[124], 3}, {NULL}};
const Segment SGI102271[] = {{&S99662[90], 5}, {NULL}};
const Segment SGI102272[] = {{&S99662[148], 13}, {NULL}};
const Segment SGI102273[] = {{&S99676[44], 10}, {NULL}};
const Segment SGI102274[] = {{&S99676[44], 27}, {NULL}};
const Segment SGI102275[] = {{&S99676[30], 6}, {NULL}};
const Segment SGI102276[] = {{&S99676[149], 4}, {NULL}};
const Segment SGI102277[] = {{&S99676[119], 3}, {&S99676[202], 5}, {NULL}};
const Segment SGI102278[] = {{&S99676[85], 5}, {NULL}};
const Segment SGI102279[] = {{&S76840[34], 15}, {NULL}};
const Segment SGI102280[] = {{&S76840[27], 6}, {NULL}};
const Segment SGI102281[] = {{&S76840[102], 9}, {NULL}};
const Segment SGI102282[] = {{&S76840[75], 6}, {NULL}};
const Segment SGI102283[] = {{&S99714[59], 10}, {NULL}};
const Segment SGI102284[] = {{&S99714[59], 27}, {NULL}};
const Segment SGI102285[] = {{&S99714[45], 6}, {NULL}};
const Segment SGI102286[] = {{&S76878[44], 13}, {NULL}};
const Segment SGI102287[] = {{&S76878[44], 25}, {NULL}};
const Segment SGI102288[] = {{&S76878[30], 6}, {NULL}};
const Segment SGI102289[] = {{&S99714[134], 3}, {NULL}};
const Segment SGI102290[] = {{&S99714[100], 5}, {NULL}};
const Segment SGI102291[] = {{&S76878[31], 3}, {NULL}};
const Segment SGI102292[] = {{&S99714[0], 14}, {NULL}};
const Segment SGI102293[] = {{&S76878[83], 6}, {NULL}};
const Segment SGI102294[] = {{&S99735[59], 13}, {NULL}};
const Segment SGI102295[] = {{&S99735[59], 25}, {NULL}};
const Segment SGI102296[] = {{&S99735[45], 6}, {NULL}};
const Segment SGI102297[] = {{&S99735[132], 3}, {NULL}};
const Segment SGI102298[] = {{&S99735[98], 5}, {NULL}};
const Segment SGI102299[] = {{&S99735[0], 14}, {NULL}};
const Segment SGI102300[] = {{&S99742[44], 10}, {NULL}};
const Segment SGI102301[] = {{&S99742[44], 27}, {NULL}};
const Segment SGI102302[] = {{&S99742[30], 6}, {NULL}};
const Segment SGI102303[] = {{&S99742[85], 6}, {NULL}};
const Segment SGI102304[] = {{&S99742[112], 13}, {NULL}};
const Segment SGI102305[] = {{&S76919[34], 4}, {NULL}};
const Segment SGI102306[] = {{&S76919[27], 6}, {NULL}};
const Segment SGI102307[] = {{&S76919[116], 9}, {NULL}};
const Segment SGI102308[] = {{&S76919[89], 6}, {NULL}};
const Segment SGI102309[] = {{&S76923[44], 10}, {NULL}};
const Segment SGI102310[] = {{&S76923[44], 31}, {NULL}};
const Segment SGI102311[] = {{&S76923[30], 6}, {NULL}};
const Segment SGI102312[] = {{&S76923[31], 3}, {NULL}};
const Segment SGI102313[] = {{&S76923[89], 6}, {NULL}};
const Segment SGI102314[] = {{&S99765[59], 9}, {NULL}};
const Segment SGI102315[] = {{&S99765[59], 21}, {NULL}};
const Segment SGI102316[] = {{&S99765[45], 6}, {NULL}};
const Segment SGI102317[] = {{&S99765[128], 3}, {NULL}};
const Segment SGI102318[] = {{&S99765[94], 5}, {NULL}};
const Segment SGI102319[] = {{&S99765[0], 14}, {NULL}};
const Segment SGI102320[] = {{&S76939[34], 26}, {NULL}};
const Segment SGI102321[] = {{&S76939[27], 6}, {NULL}};
const Segment SGI102322[] = {{&S76939[113], 9}, {NULL}};
const Segment SGI102323[] = {{&S76939[86], 6}, {NULL}};
const Segment SGI102324[] = {{&S76940[34], 25}, {NULL}};
const Segment SGI102325[] = {{&S76940[27], 6}, {NULL}};
const Segment SGI102326[] = {{&S76940[112], 9}, {NULL}};
const Segment SGI102327[] = {{&S76940[85], 6}, {NULL}};
const Segment SGI102328[] = {{&S76943[34], 24}, {NULL}};
const Segment SGI102329[] = {{&S99777[59], 10}, {NULL}};
const Segment SGI102330[] = {{&S99777[59], 27}, {NULL}};
const Segment SGI102331[] = {{&S76943[27], 6}, {NULL}};
const Segment SGI102332[] = {{&S99777[45], 6}, {NULL}};
const Segment SGI102333[] = {{&S99777[134], 3}, {NULL}};
const Segment SGI102334[] = {{&S76943[111], 9}, {NULL}};
const Segment SGI102335[] = {{&S76943[84], 6}, {NULL}};
const Segment SGI102336[] = {{&S99777[100], 5}, {NULL}};
const Segment SGI102337[] = {{&S99777[0], 14}, {NULL}};
const Segment SGI102338[] = {{&S76947[34], 9}, {NULL}};
const Segment SGI102339[] = {{&S76947[27], 6}, {NULL}};
const Segment SGI102340[] = {{&S76947[96], 9}, {NULL}};
const Segment SGI102341[] = {{&S76947[69], 6}, {NULL}};
const Segment SGI102342[] = {{&S90994[42], 10}, {NULL}};
const Segment SGI102343[] = {{&S90994[42], 22}, {NULL}};
const Segment SGI102344[] = {{&S90994[30], 4}, {NULL}};
const Segment SGI102345[] = {{&S99788[44], 10}, {NULL}};
const Segment SGI102346[] = {{&S99788[44], 27}, {NULL}};
const Segment SGI102347[] = {{&S99788[30], 6}, {NULL}};
const Segment SGI102348[] = {{&S90994[113], 3}, {NULL}};
const Segment SGI102349[] = {{&S99788[118], 3}, {NULL}};
const Segment SGI102350[] = {{&S90994[78], 6}, {NULL}};
const Segment SGI102351[] = {{&S99788[85], 5}, {NULL}};
const Segment SGI102352[] = {{&S90994[131], 13}, {NULL}};
const Segment SGI102353[] = {{&S99788[122], 14}, {NULL}};
const Segment SGI102354[] = {{&S99794[45], 14}, {NULL}};
const Segment SGI102355[] = {{&S99794[30], 6}, {NULL}};
const Segment SGI102356[] = {{&S99794[119], 3}, {NULL}};
const Segment SGI102357[] = {{&S99794[85], 5}, {NULL}};
const Segment SGI102358[] = {{&S99794[143], 13}, {NULL}};
const Segment SGI102359[] = {{&S99796[45], 7}, {NULL}};
const Segment SGI102360[] = {{&S99796[30], 6}, {NULL}};
const Segment SGI102361[] = {{&S99796[124], 3}, {NULL}};
const Segment SGI102362[] = {{&S99796[90], 5}, {NULL}};
const Segment SGI102363[] = {{&S99796[148], 13}, {NULL}};
const Segment SGI102364[] = {{&S76969[34], 6}, {NULL}};
const Segment SGI102365[] = {{&S76969[27], 6}, {NULL}};
const Segment SGI102366[] = {{&S76969[93], 9}, {NULL}};
const Segment SGI102367[] = {{&S76969[66], 6}, {NULL}};
const Segment SGI102368[] = {{&S99798[44], 6}, {NULL}};
const Segment SGI102369[] = {{&S99798[44], 18}, {NULL}};
const Segment SGI102370[] = {{&S99798[30], 6}, {NULL}};
const Segment SGI102371[] = {{&S99798[76], 6}, {NULL}};
const Segment SGI102372[] = {{&S99798[103], 13}, {NULL}};
const Segment SGI102373[] = {{&S76976[34], 8}, {NULL}};
const Segment SGI102374[] = {{&S76976[27], 6}, {NULL}};
const Segment SGI102375[] = {{&S76976[95], 9}, {NULL}};
const Segment SGI102376[] = {{&S76976[68], 6}, {NULL}};
const Segment SGI102377[] = {{&S99805[44], 6}, {NULL}};
const Segment SGI102378[] = {{&S99805[44], 18}, {NULL}};
const Segment SGI102379[] = {{&S99805[30], 6}, {NULL}};
const Segment SGI102380[] = {{&S99805[76], 6}, {NULL}};
const Segment SGI102381[] = {{&S99805[103], 13}, {NULL}};
const Segment SGI102382[] = {{&S76979[44], 14}, {NULL}};
const Segment SGI102383[] = {{&S76979[44], 26}, {NULL}};
const Segment SGI102384[] = {{&S76979[30], 6}, {NULL}};
const Segment SGI102385[] = {{&S76979[31], 3}, {NULL}};
const Segment SGI102386[] = {{&S76979[84], 6}, {NULL}};
const Segment SGI102387[] = {{&S99809[44], 10}, {NULL}};
const Segment SGI102388[] = {{&S76980[44], 15}, {NULL}};
const Segment SGI102389[] = {{&S99809[44], 27}, {NULL}};
const Segment SGI102390[] = {{&S76980[44], 27}, {NULL}};
const Segment SGI102391[] = {{&S99809[30], 6}, {NULL}};
const Segment SGI102392[] = {{&S76980[30], 6}, {NULL}};
const Segment SGI102393[] = {{&S76980[31], 3}, {NULL}};
const Segment SGI102394[] = {{&S76980[85], 6}, {NULL}};
const Segment SGI102395[] = {{&S99809[85], 6}, {NULL}};
const Segment SGI102396[] = {{&S99809[112], 13}, {NULL}};
const Segment SGI102397[] = {{&S76981[44], 11}, {NULL}};
const Segment SGI102398[] = {{&S76981[44], 23}, {NULL}};
const Segment SGI102399[] = {{&S76981[30], 6}, {NULL}};
const Segment SGI102400[] = {{&S76981[31], 3}, {NULL}};
const Segment SGI102401[] = {{&S76981[81], 6}, {NULL}};
const Segment SGI102402[] = {{&S99812[44], 13}, {NULL}};
const Segment SGI102403[] = {{&S76982[44], 13}, {NULL}};
const Segment SGI102404[] = {{&S76982[44], 25}, {NULL}};
const Segment SGI102405[] = {{&S99812[44], 25}, {NULL}};
const Segment SGI102406[] = {{&S76982[30], 6}, {NULL}};
const Segment SGI102407[] = {{&S99812[30], 6}, {NULL}};
const Segment SGI102408[] = {{&S76982[31], 3}, {NULL}};
const Segment SGI102409[] = {{&S76982[83], 6}, {NULL}};
const Segment SGI102410[] = {{&S99812[83], 6}, {NULL}};
const Segment SGI102411[] = {{&S99812[110], 13}, {NULL}};
const Segment SGI102412[] = {{&S76983[44], 11}, {NULL}};
const Segment SGI102413[] = {{&S76983[44], 23}, {NULL}};
const Segment SGI102414[] = {{&S76983[30], 6}, {NULL}};
const Segment SGI102415[] = {{&S76983[31], 3}, {NULL}};
const Segment SGI102416[] = {{&S76983[81], 6}, {NULL}};
const Segment SGI102417[] = {{&S99820[44], 10}, {NULL}};
const Segment SGI102418[] = {{&S99820[44], 27}, {NULL}};
const Segment SGI102419[] = {{&S99820[30], 6}, {NULL}};
const Segment SGI102420[] = {{&S99820[119], 3}, {NULL}};
const Segment SGI102421[] = {{&S99820[85], 5}, {NULL}};
const Segment SGI102422[] = {{&S99820[143], 11}, {NULL}};
const Segment SGI102423[] = {{&S99837[44], 11}, {NULL}};
const Segment SGI102424[] = {{&S99837[44], 32}, {NULL}};
const Segment SGI102425[] = {{&S99837[30], 6}, {NULL}};
const Segment SGI102426[] = {{&S99837[90], 6}, {NULL}};
const Segment SGI102427[] = {{&S99837[117], 13}, {NULL}};
const Segment SGI102428[] = {{&S99843[44], 10}, {NULL}};
const Segment SGI102429[] = {{&S99843[44], 27}, {NULL}};
const Segment SGI102430[] = {{&S99843[30], 6}, {NULL}};
const Segment SGI102431[] = {{&S99843[118], 3}, {NULL}};
const Segment SGI102432[] = {{&S99843[85], 5}, {NULL}};
const Segment SGI102433[] = {{&S99843[122], 14}, {NULL}};
const Segment SGI102434[] = {{&S77021[44], 24}, {NULL}};
const Segment SGI102435[] = {{&S77021[30], 6}, {NULL}};
const Segment SGI102436[] = {{&S91069[34], 4}, {NULL}};
const Segment SGI102437[] = {{&S77021[31], 3}, {NULL}};
const Segment SGI102438[] = {{&S91069[27], 6}, {NULL}};
const Segment SGI102439[] = {{&S77021[100], 6}, {NULL}};
const Segment SGI102440[] = {{&S91069[98], 2}, {NULL}};
const Segment SGI102441[] = {{&S91069[64], 6}, {NULL}};
const Segment SGI102442[] = {{&S91069[125], 8}, {NULL}};
const Segment SGI102443[] = {{&S77022[44], 14}, {NULL}};
const Segment SGI102444[] = {{&S77022[44], 26}, {NULL}};
const Segment SGI102445[] = {{&S77022[30], 6}, {NULL}};
const Segment SGI102446[] = {{&S77022[87], 6}, {NULL}};
const Segment SGI102447[] = {{&S99852[44], 10}, {NULL}};
const Segment SGI102448[] = {{&S99852[44], 27}, {NULL}};
const Segment SGI102449[] = {{&S99852[30], 6}, {NULL}};
const Segment SGI102450[] = {{&S99852[119], 3}, {NULL}};
const Segment SGI102451[] = {{&S99852[85], 5}, {NULL}};
const Segment SGI102452[] = {{&S77023[44], 14}, {NULL}};
const Segment SGI102453[] = {{&S77023[44], 26}, {NULL}};
const Segment SGI102454[] = {{&S77023[30], 6}, {NULL}};
const Segment SGI102455[] = {{&S77023[31], 3}, {NULL}};
const Segment SGI102456[] = {{&S77023[84], 6}, {NULL}};
const Segment SGI102457[] = {{&S99854[59], 5}, {NULL}};
const Segment SGI102458[] = {{&S77024[44], 14}, {NULL}};
const Segment SGI102459[] = {{&S99854[59], 17}, {NULL}};
const Segment SGI102460[] = {{&S77024[44], 26}, {NULL}};
const Segment SGI102461[] = {{&S99854[45], 6}, {NULL}};
const Segment SGI102462[] = {{&S77024[30], 6}, {NULL}};
const Segment SGI102463[] = {{&S99854[124], 3}, {NULL}};
const Segment SGI102464[] = {{&S77024[87], 4}, {NULL}};
const Segment SGI102465[] = {{&S99854[90], 5}, {NULL}};
const Segment SGI102466[] = {{&S77024[87], 3}, {NULL}};
const Segment SGI102467[] = {{&S99854[0], 14}, {NULL}};
const Segment SGI102468[] = {{&S99867[44], 14}, {NULL}};
const Segment SGI102469[] = {{&S99867[44], 26}, {NULL}};
const Segment SGI102470[] = {{&S99867[30], 6}, {NULL}};
const Segment SGI102471[] = {{&S99867[84], 6}, {NULL}};
const Segment SGI102472[] = {{&S99867[111], 13}, {NULL}};
const Segment SGI102473[] = {{&S77076[45], 14}, {NULL}};
const Segment SGI102474[] = {{&S77076[30], 6}, {NULL}};
const Segment SGI102475[] = {{&S77076[31], 3}, {NULL}};
const Segment SGI102476[] = {{&S77076[85], 5}, {NULL}};
const Segment SGI102477[] = {{&S77076[143], 13}, {NULL}};
const Segment SGI102478[] = {{&S77081[34], 11}, {NULL}};
const Segment SGI102479[] = {{&S77081[27], 6}, {NULL}};
const Segment SGI102480[] = {{&S77081[98], 9}, {NULL}};
const Segment SGI102481[] = {{&S77081[71], 6}, {NULL}};
const Segment SGI102482[] = {{&S77081[139], 8}, {NULL}};
const Segment SGI102483[] = {{&S85205[44], 7}, {NULL}};
const Segment SGI102484[] = {{&S85205[44], 28}, {NULL}};
const Segment SGI102485[] = {{&S85205[30], 6}, {NULL}};
const Segment SGI102486[] = {{&S85205[31], 3}, {NULL}};
const Segment SGI102487[] = {{&S85205[86], 5}, {NULL}};
const Segment SGI102488[] = {{&S85205[0], 13}, {NULL}};
const Segment SGI102489[] = {{&S92768[44], 7}, {NULL}};
const Segment SGI102490[] = {{&S92768[44], 18}, {NULL}};
const Segment SGI102491[] = {{&S92768[30], 6}, {NULL}};
const Segment SGI102492[] = {{&S92768[76], 6}, {NULL}};
const Segment SGI102493[] = {{&S92768[103], 13}, {NULL}};
const Segment SGI102494[] = {{&S77112[34], 25}, {NULL}};
const Segment SGI102495[] = {{&S77112[27], 6}, {NULL}};
const Segment SGI102496[] = {{&S77112[112], 9}, {NULL}};
const Segment SGI102497[] = {{&S77112[85], 6}, {NULL}};
const Segment SGI102498[] = {{&S77117[34], 8}, {NULL}};
const Segment SGI102499[] = {{&S77117[27], 6}, {NULL}};
const Segment SGI102500[] = {{&S77117[97], 9}, {NULL}};
const Segment SGI102501[] = {{&S77117[70], 6}, {NULL}};
const Segment SGI102502[] = {{&S77118[34], 8}, {NULL}};
const Segment SGI102503[] = {{&S77118[27], 6}, {NULL}};
const Segment SGI102504[] = {{&S77118[96], 9}, {NULL}};
const Segment SGI102505[] = {{&S77118[69], 6}, {NULL}};
const Segment SGI102506[] = {{&S77127[34], 9}, {NULL}};
const Segment SGI102507[] = {{&S77127[27], 6}, {NULL}};
const Segment SGI102508[] = {{&S77127[96], 9}, {NULL}};
const Segment SGI102509[] = {{&S77127[69], 6}, {NULL}};
const Segment SGI102510[] = {{&S77136[44], 12}, {NULL}};
const Segment SGI102511[] = {{&S77136[44], 24}, {NULL}};
const Segment SGI102512[] = {{&S77136[30], 6}, {NULL}};
const Segment SGI102513[] = {{&S77136[31], 3}, {NULL}};
const Segment SGI102514[] = {{&S77136[82], 6}, {NULL}};
const Segment SGI102515[] = {{&S77137[34], 7}, {NULL}};
const Segment SGI102516[] = {{&S77137[27], 6}, {NULL}};
const Segment SGI102517[] = {{&S77137[94], 9}, {NULL}};
const Segment SGI102518[] = {{&S77137[67], 6}, {NULL}};
const Segment SGI102519[] = {{&S77140[34], 15}, {NULL}};
const Segment SGI102520[] = {{&S77140[27], 6}, {NULL}};
const Segment SGI102521[] = {{&S77140[104], 9}, {NULL}};
const Segment SGI102522[] = {{&S77140[77], 6}, {NULL}};
const Segment SGI102523[] = {{&S77148[44], 7}, {NULL}};
const Segment SGI102524[] = {{&S77148[44], 24}, {NULL}};
const Segment SGI102525[] = {{&S77148[30], 6}, {NULL}};
const Segment SGI102526[] = {{&S77148[82], 6}, {NULL}};
const Segment SGI102527[] = {{&S77148[109], 13}, {NULL}};
const Segment SGI102528[] = {{&S77149[34], 14}, {NULL}};
const Segment SGI102529[] = {{&S77149[27], 6}, {NULL}};
const Segment SGI102530[] = {{&S77149[101], 9}, {NULL}};
const Segment SGI102531[] = {{&S77149[74], 6}, {NULL}};
const Segment SGI102532[] = {{&S77149[142], 8}, {NULL}};
const Segment SGI102533[] = {{&S99873[61], 10}, {NULL}};
const Segment SGI102534[] = {{&S99873[61], 27}, {NULL}};
const Segment SGI102535[] = {{&S99873[47], 6}, {NULL}};
const Segment SGI102536[] = {{&S99873[136], 3}, {NULL}};
const Segment SGI102537[] = {{&S99873[102], 5}, {NULL}};
const Segment SGI102538[] = {{&S99873[0], 14}, {NULL}};
const Segment SGI102539[] = {{&S77203[34], 9}, {NULL}};
const Segment SGI102540[] = {{&S77203[27], 6}, {NULL}};
const Segment SGI102541[] = {{&S77203[96], 9}, {NULL}};
const Segment SGI102542[] = {{&S77203[69], 6}, {NULL}};
const Segment SGI102543[] = {{&S99882[44], 10}, {NULL}};
const Segment SGI102544[] = {{&S99882[44], 27}, {NULL}};
const Segment SGI102545[] = {{&S99882[30], 6}, {NULL}};
const Segment SGI102546[] = {{&S99882[118], 3}, {NULL}};
const Segment SGI102547[] = {{&S99882[85], 5}, {NULL}};
const Segment SGI102548[] = {{&S99882[122], 14}, {NULL}};
const Segment SGI102549[] = {{&S77219[44], 11}, {NULL}};
const Segment SGI102550[] = {{&S77219[44], 23}, {NULL}};
const Segment SGI102551[] = {{&S77219[30], 6}, {NULL}};
const Segment SGI102552[] = {{&S77219[81], 6}, {NULL}};
const Segment SGI102553[] = {{&S77219[108], 13}, {NULL}};
const Segment SGI102554[] = {{&S99907[59], 10}, {NULL}};
const Segment SGI102555[] = {{&S99907[59], 22}, {NULL}};
const Segment SGI102556[] = {{&S99907[45], 6}, {NULL}};
const Segment SGI102557[] = {{&S99907[129], 3}, {NULL}};
const Segment SGI102558[] = {{&S99907[95], 5}, {NULL}};
const Segment SGI102559[] = {{&S99907[0], 14}, {NULL}};
const Segment SGI102560[] = {{&S99908[59], 12}, {NULL}};
const Segment SGI102561[] = {{&S99908[59], 24}, {NULL}};
const Segment SGI102562[] = {{&S99908[45], 6}, {NULL}};
const Segment SGI102563[] = {{&S99908[131], 3}, {NULL}};
const Segment SGI102564[] = {{&S99908[97], 5}, {NULL}};
const Segment SGI102565[] = {{&S99908[0], 14}, {NULL}};
const Segment SGI102566[] = {{&S99912[44], 11}, {NULL}};
const Segment SGI102567[] = {{&S99912[44], 23}, {NULL}};
const Segment SGI102568[] = {{&S99912[30], 6}, {NULL}};
const Segment SGI102569[] = {{&S99912[81], 6}, {NULL}};
const Segment SGI102570[] = {{&S99912[108], 13}, {NULL}};
const Segment SGI102571[] = {{&S99916[44], 12}, {NULL}};
const Segment SGI102572[] = {{&S99916[44], 24}, {NULL}};
const Segment SGI102573[] = {{&S99916[30], 6}, {NULL}};
const Segment SGI102574[] = {{&S99916[115], 3}, {NULL}};
const Segment SGI102575[] = {{&S99916[82], 5}, {NULL}};
const Segment SGI102576[] = {{&S99916[119], 14}, {NULL}};
const Segment SGI102577[] = {{&S77244[34], 7}, {NULL}};
const Segment SGI102578[] = {{&S77244[27], 6}, {NULL}};
const Segment SGI102579[] = {{&S77244[99], 9}, {NULL}};
const Segment SGI102580[] = {{&S77244[72], 6}, {NULL}};
const Segment SGI102581[] = {{&S99920[34], 10}, {NULL}};
const Segment SGI102582[] = {{&S99920[27], 6}, {NULL}};
const Segment SGI102583[] = {{&S99920[109], 2}, {NULL}};
const Segment SGI102584[] = {{&S99920[75], 6}, {NULL}};
const Segment SGI102585[] = {{&S99920[143], 8}, {NULL}};
const Segment SGI102586[] = {{&S99927[44], 13}, {NULL}};
const Segment SGI102587[] = {{&S99927[44], 25}, {NULL}};
const Segment SGI102588[] = {{&S99927[30], 6}, {NULL}};
const Segment SGI102589[] = {{&S99927[83], 6}, {NULL}};
const Segment SGI102590[] = {{&S99927[110], 13}, {NULL}};
const Segment SGI102591[] = {{&S92957[44], 9}, {NULL}};
const Segment SGI102592[] = {{&S92957[44], 21}, {NULL}};
const Segment SGI102593[] = {{&S92957[30], 6}, {NULL}};
const Segment SGI102594[] = {{&S92957[130], 3}, {NULL}};
const Segment SGI102595[] = {{&S92957[96], 5}, {NULL}};
const Segment SGI102596[] = {{&S92957[35], 1}, {NULL}};
const Segment SGI102597[] = {{&S92957[67], 11}, {NULL}};
const Segment SGI102598[] = {{&S99948[44], 10}, {NULL}};
const Segment SGI102599[] = {{&S99948[44], 27}, {NULL}};
const Segment SGI102600[] = {{&S99948[30], 6}, {NULL}};
const Segment SGI102601[] = {{&S99948[118], 3}, {NULL}};
const Segment SGI102602[] = {{&S99948[85], 5}, {NULL}};
const Segment SGI102603[] = {{&S99948[122], 14}, {NULL}};
const Segment SGI102604[] = {{&S77288[44], 9}, {NULL}};
const Segment SGI102605[] = {{&S77288[44], 18}, {NULL}};
const Segment SGI102606[] = {{&S77288[30], 6}, {NULL}};
const Segment SGI102607[] = {{&S77288[31], 3}, {NULL}};
const Segment SGI102608[] = {{&S77288[76], 6}, {NULL}};
const Segment SGI102609[] = {{&S99963[44], 10}, {NULL}};
const Segment SGI102610[] = {{&S99963[44], 27}, {NULL}};
const Segment SGI102611[] = {{&S99963[30], 6}, {NULL}};
const Segment SGI102612[] = {{&S99963[118], 3}, {NULL}};
const Segment SGI102613[] = {{&S99963[85], 5}, {NULL}};
const Segment SGI102614[] = {{&S99963[122], 14}, {NULL}};
const Segment SGI102615[] = {{&S77308[34], 9}, {NULL}};
const Segment SGI102616[] = {{&S77308[27], 6}, {NULL}};
const Segment SGI102617[] = {{&S77308[96], 9}, {NULL}};
const Segment SGI102618[] = {{&S77308[69], 6}, {NULL}};
const Segment SGI102619[] = {{&S100001[52], 12}, {NULL}};
const Segment SGI102620[] = {{&S100001[52], 24}, {NULL}};
const Segment SGI102621[] = {{&S100001[38], 6}, {NULL}};
const Segment SGI102622[] = {{&S100001[90], 5}, {NULL}};
const Segment SGI102623[] = {{&S100001[0], 14}, {NULL}};
const Segment SGI102624[] = {{&S100003[52], 9}, {NULL}};
const Segment SGI102625[] = {{&S100003[52], 21}, {NULL}};
const Segment SGI102626[] = {{&S100003[38], 6}, {NULL}};
const Segment SGI102627[] = {{&S100003[87], 5}, {NULL}};
const Segment SGI102628[] = {{&S100003[0], 14}, {NULL}};
const Segment SGI102629[] = {{&S100016[44], 6}, {NULL}};
const Segment SGI102630[] = {{&S100016[44], 25}, {NULL}};
const Segment SGI102631[] = {{&S100016[30], 6}, {NULL}};
const Segment SGI102632[] = {{&S100016[118], 3}, {NULL}};
const Segment SGI102633[] = {{&S100016[83], 6}, {NULL}};
const Segment SGI102634[] = {{&S100016[138], 8}, {NULL}};
const Segment SGI102635[] = {{&S100016[147], 13}, {NULL}};
const Segment SGI102636[] = {{&S77389[44], 11}, {NULL}};
const Segment SGI102637[] = {{&S77389[44], 23}, {NULL}};
const Segment SGI102638[] = {{&S77389[30], 6}, {NULL}};
const Segment SGI102639[] = {{&S77389[31], 3}, {NULL}};
const Segment SGI102640[] = {{&S77389[81], 6}, {NULL}};
const Segment SGI102641[] = {{&S93079[44], 10}, {NULL}};
const Segment SGI102642[] = {{&S93079[44], 21}, {NULL}};
const Segment SGI102643[] = {{&S93079[30], 6}, {NULL}};
const Segment SGI102644[] = {{&S93079[132], 3}, {NULL}};
const Segment SGI102645[] = {{&S93079[98], 5}, {NULL}};
const Segment SGI102646[] = {{&S93079[35], 1}, {NULL}};
const Segment SGI102647[] = {{&S93079[67], 11}, {NULL}};
const Segment SGI102648[] = {{&S77402[44], 17}, {NULL}};
const Segment SGI102649[] = {{&S77402[44], 29}, {NULL}};
const Segment SGI102650[] = {{&S77402[30], 6}, {NULL}};
const Segment SGI102651[] = {{&S77402[31], 3}, {NULL}};
const Segment SGI102652[] = {{&S77402[87], 6}, {NULL}};
const Segment SGI102653[] = {{&S77416[34], 6}, {NULL}};
const Segment SGI102654[] = {{&S77416[27], 6}, {NULL}};
const Segment SGI102655[] = {{&S77416[93], 9}, {NULL}};
const Segment SGI102656[] = {{&S77416[66], 6}, {NULL}};
const Segment SGI102657[] = {{&S77447[34], 11}, {NULL}};
const Segment SGI102658[] = {{&S77447[27], 6}, {NULL}};
const Segment SGI102659[] = {{&S77447[98], 9}, {NULL}};
const Segment SGI102660[] = {{&S77447[71], 6}, {NULL}};
const Segment SGI102661[] = {{&S77447[139], 8}, {NULL}};
const Segment SGI102662[] = {{&S77468[44], 18}, {NULL}};
const Segment SGI102663[] = {{&S77468[44], 30}, {NULL}};
const Segment SGI102664[] = {{&S77468[30], 6}, {NULL}};
const Segment SGI102665[] = {{&S77468[31], 3}, {NULL}};
const Segment SGI102666[] = {{&S77468[88], 6}, {NULL}};
const Segment SGI102667[] = {{&S77469[44], 22}, {NULL}};
const Segment SGI102668[] = {{&S77469[44], 34}, {NULL}};
const Segment SGI102669[] = {{&S77469[30], 6}, {NULL}};
const Segment SGI102670[] = {{&S77469[31], 3}, {NULL}};
const Segment SGI102671[] = {{&S77469[92], 6}, {NULL}};
const Segment SGI102672[] = {{&S77470[44], 18}, {NULL}};
const Segment SGI102673[] = {{&S77470[44], 30}, {NULL}};
const Segment SGI102674[] = {{&S77470[30], 6}, {NULL}};
const Segment SGI102675[] = {{&S77470[31], 3}, {NULL}};
const Segment SGI102676[] = {{&S77470[88], 6}, {NULL}};
const Segment SGI102677[] = {{&S77475[44], 10}, {NULL}};
const Segment SGI102678[] = {{&S77475[44], 22}, {NULL}};
const Segment SGI102679[] = {{&S77475[30], 6}, {NULL}};
const Segment SGI102680[] = {{&S77475[31], 3}, {NULL}};
const Segment SGI102681[] = {{&S77475[87], 6}, {NULL}};
const Segment SGI102682[] = {{&S77475[66], 8}, {NULL}};
const Segment SGI102683[] = {{&S100205[57], 10}, {NULL}};
const Segment SGI102684[] = {{&S100205[57], 27}, {NULL}};
const Segment SGI102685[] = {{&S100205[43], 6}, {NULL}};
const Segment SGI102686[] = {{&S100205[132], 3}, {NULL}};
const Segment SGI102687[] = {{&S100205[98], 5}, {NULL}};
const Segment SGI102688[] = {{&S100205[0], 12}, {NULL}};
const Segment SGI102689[] = {{&S100208[59], 10}, {NULL}};
const Segment SGI102690[] = {{&S100208[59], 27}, {NULL}};
const Segment SGI102691[] = {{&S100208[45], 6}, {NULL}};
const Segment SGI102692[] = {{&S100208[134], 3}, {NULL}};
const Segment SGI102693[] = {{&S100208[100], 5}, {NULL}};
const Segment SGI102694[] = {{&S100208[0], 14}, {NULL}};
const Segment SGI102695[] = {{&S91653[32], 10}, {NULL}};
const Segment SGI102696[] = {{&S91653[27], 4}, {NULL}};
const Segment SGI102697[] = {{&S91653[102], 2}, {NULL}};
const Segment SGI102698[] = {{&S91653[68], 6}, {NULL}};
const Segment SGI102699[] = {{&S91653[129], 8}, {NULL}};
const Segment SGI102700[] = {{&S100241[59], 11}, {NULL}};
const Segment SGI102701[] = {{&S100241[59], 23}, {NULL}};
const Segment SGI102702[] = {{&S100241[45], 6}, {NULL}};
const Segment SGI102703[] = {{&S100241[130], 3}, {NULL}};
const Segment SGI102704[] = {{&S100241[96], 5}, {NULL}};
const Segment SGI102705[] = {{&S100241[0], 14}, {NULL}};
const Segment SGI102706[] = {{&S100253[59], 10}, {NULL}};
const Segment SGI102707[] = {{&S100253[59], 22}, {NULL}};
const Segment SGI102708[] = {{&S100253[45], 6}, {NULL}};
const Segment SGI102709[] = {{&S100253[129], 3}, {NULL}};
const Segment SGI102710[] = {{&S100253[95], 5}, {NULL}};
const Segment SGI102711[] = {{&S100253[0], 14}, {NULL}};
const Segment SGI102712[] = {{&S77476[44], 10}, {NULL}};
const Segment SGI102713[] = {{&S77476[44], 22}, {NULL}};
const Segment SGI102714[] = {{&S77476[30], 6}, {NULL}};
const Segment SGI102715[] = {{&S77476[31], 3}, {NULL}};
const Segment SGI102716[] = {{&S77476[80], 6}, {NULL}};
const Segment SGI102717[] = {{&S77476[140], 7}, {NULL}};
const Segment SGI102718[] = {{&S77486[44], 16}, {NULL}};
const Segment SGI102719[] = {{&S77486[44], 28}, {NULL}};
const Segment SGI102720[] = {{&S77486[30], 6}, {NULL}};
const Segment SGI102721[] = {{&S77486[31], 3}, {NULL}};
const Segment SGI102722[] = {{&S77486[86], 6}, {NULL}};
const Segment SGI102723[] = {{&S100286[59], 10}, {NULL}};
const Segment SGI102724[] = {{&S100286[59], 27}, {NULL}};
const Segment SGI102725[] = {{&S100286[45], 6}, {NULL}};
const Segment SGI102726[] = {{&S100286[134], 3}, {NULL}};
const Segment SGI102727[] = {{&S100286[100], 5}, {NULL}};
const Segment SGI102728[] = {{&S100286[0], 14}, {NULL}};
const Segment SGI102729[] = {{&S77500[34], 7}, {NULL}};
const Segment SGI102730[] = {{&S77500[27], 6}, {NULL}};
const Segment SGI102731[] = {{&S77500[99], 9}, {NULL}};
const Segment SGI102732[] = {{&S77500[72], 6}, {NULL}};
const Segment SGI102733[] = {{&S100312[59], 13}, {NULL}};
const Segment SGI102734[] = {{&S100312[59], 25}, {NULL}};
const Segment SGI102735[] = {{&S100312[45], 6}, {NULL}};
const Segment SGI102736[] = {{&S100312[132], 3}, {NULL}};
const Segment SGI102737[] = {{&S100312[98], 5}, {NULL}};
const Segment SGI102738[] = {{&S100312[0], 14}, {NULL}};
const Segment SGI102739[] = {{&S77532[44], 11}, {NULL}};
const Segment SGI102740[] = {{&S77532[44], 23}, {NULL}};
const Segment SGI102741[] = {{&S77532[30], 6}, {NULL}};
const Segment SGI102742[] = {{&S77532[31], 3}, {NULL}};
const Segment SGI102743[] = {{&S77532[81], 6}, {NULL}};
const Segment SGI102744[] = {{&S100335[59], 7}, {NULL}};
const Segment SGI102745[] = {{&S100335[59], 19}, {NULL}};
const Segment SGI102746[] = {{&S100335[45], 6}, {NULL}};
const Segment SGI102747[] = {{&S100335[126], 3}, {NULL}};
const Segment SGI102748[] = {{&S100335[92], 5}, {NULL}};
const Segment SGI102749[] = {{&S100335[0], 14}, {NULL}};
const Segment SGI102750[] = {{&S100337[59], 10}, {NULL}};
const Segment SGI102751[] = {{&S100337[59], 22}, {NULL}};
const Segment SGI102752[] = {{&S100337[45], 6}, {NULL}};
const Segment SGI102753[] = {{&S100337[129], 3}, {NULL}};
const Segment SGI102754[] = {{&S100337[95], 5}, {NULL}};
const Segment SGI102755[] = {{&S100337[0], 14}, {NULL}};
const Segment SGI102756[] = {{&S100344[45], 5}, {NULL}};
const Segment SGI102757[] = {{&S100344[30], 6}, {NULL}};
const Segment SGI102758[] = {{&S100344[119], 3}, {NULL}};
const Segment SGI102759[] = {{&S100344[85], 5}, {NULL}};
const Segment SGI102760[] = {{&S100344[143], 13}, {NULL}};
const Segment SGI102761[] = {{&S100349[45], 16}, {NULL}};
const Segment SGI102762[] = {{&S100349[30], 6}, {NULL}};
const Segment SGI102763[] = {{&S100349[121], 3}, {NULL}};
const Segment SGI102764[] = {{&S100349[87], 5}, {NULL}};
const Segment SGI102765[] = {{&S77557[34], 9}, {NULL}};
const Segment SGI102766[] = {{&S100349[145], 13}, {NULL}};
const Segment SGI102767[] = {{&S77557[27], 6}, {NULL}};
const Segment SGI102768[] = {{&S77557[96], 9}, {NULL}};
const Segment SGI102769[] = {{&S77557[69], 6}, {NULL}};
const Segment SGI102770[] = {{&S77557[137], 8}, {NULL}};
const Segment SGI102771[] = {{&S100352[44], 18}, {NULL}};
const Segment SGI102772[] = {{&S100352[44], 35}, {NULL}};
const Segment SGI102773[] = {{&S100352[30], 6}, {NULL}};
const Segment SGI102774[] = {{&S100352[127], 3}, {NULL}};
const Segment SGI102775[] = {{&S100352[93], 5}, {NULL}};
const Segment SGI102776[] = {{&S100352[150], 10}, {NULL}};
const Segment SGI102777[] = {{&S100355[45], 10}, {NULL}};
const Segment SGI102778[] = {{&S100355[30], 6}, {NULL}};
const Segment SGI102779[] = {{&S100355[120], 3}, {NULL}};
const Segment SGI102780[] = {{&S100355[86], 5}, {NULL}};
const Segment SGI102781[] = {{&S100355[144], 13}, {NULL}};
const Segment SGI102782[] = {{&S77597[34], 11}, {NULL}};
const Segment SGI102783[] = {{&S77597[27], 6}, {NULL}};
const Segment SGI102784[] = {{&S77597[98], 9}, {NULL}};
const Segment SGI102785[] = {{&S77597[71], 6}, {NULL}};
const Segment SGI102786[] = {{&S100391[44], 10}, {NULL}};
const Segment SGI102787[] = {{&S100391[44], 27}, {NULL}};
const Segment SGI102788[] = {{&S100391[30], 6}, {NULL}};
const Segment SGI102789[] = {{&S100391[119], 3}, {NULL}};
const Segment SGI102790[] = {{&S100391[85], 5}, {NULL}};
const Segment SGI102791[] = {{&S100391[130], 14}, {NULL}};
const Segment SGI102792[] = {{&S100392[44], 11}, {NULL}};
const Segment SGI102793[] = {{&S100392[44], 28}, {NULL}};
const Segment SGI102794[] = {{&S100392[30], 6}, {NULL}};
const Segment SGI102795[] = {{&S100392[120], 3}, {NULL}};
const Segment SGI102796[] = {{&S100392[86], 5}, {NULL}};
const Segment SGI102797[] = {{&S100392[144], 13}, {NULL}};
const Segment SGI102798[] = {{&S100401[44], 13}, {NULL}};
const Segment SGI102799[] = {{&S100401[44], 25}, {NULL}};
const Segment SGI102800[] = {{&S100401[30], 6}, {NULL}};
const Segment SGI102801[] = {{&S100401[117], 3}, {NULL}};
const Segment SGI102802[] = {{&S100401[83], 5}, {NULL}};
const Segment SGI102803[] = {{&S100403[44], 11}, {NULL}};
const Segment SGI102804[] = {{&S100403[44], 23}, {NULL}};
const Segment SGI102805[] = {{&S100403[30], 6}, {NULL}};
const Segment SGI102806[] = {{&S100403[115], 3}, {NULL}};
const Segment SGI102807[] = {{&S100403[81], 5}, {NULL}};
const Segment SGI102808[] = {{&S100414[44], 10}, {NULL}};
const Segment SGI102809[] = {{&S100414[44], 27}, {NULL}};
const Segment SGI102810[] = {{&S100414[30], 6}, {NULL}};
const Segment SGI102811[] = {{&S100414[119], 3}, {NULL}};
const Segment SGI102812[] = {{&S100414[85], 5}, {NULL}};
const Segment SGI102813[] = {{&S100414[143], 13}, {NULL}};
const Segment SGI102814[] = {{&S100423[44], 5}, {NULL}};
const Segment SGI102815[] = {{&S77635[44], 9}, {NULL}};
const Segment SGI102816[] = {{&S100423[44], 27}, {NULL}};
const Segment SGI102817[] = {{&S77635[44], 21}, {NULL}};
const Segment SGI102818[] = {{&S100423[30], 6}, {NULL}};
const Segment SGI102819[] = {{&S77635[30], 6}, {NULL}};
const Segment SGI102820[] = {{&S77635[31], 3}, {NULL}};
const Segment SGI102821[] = {{&S100423[85], 6}, {NULL}};
const Segment SGI102822[] = {{&S77635[79], 6}, {NULL}};
const Segment SGI102823[] = {{&S100423[112], 13}, {NULL}};
const Segment SGI102824[] = {{&S77636[44], 7}, {NULL}};
const Segment SGI102825[] = {{&S77636[44], 19}, {NULL}};
const Segment SGI102826[] = {{&S77636[30], 6}, {NULL}};
const Segment SGI102827[] = {{&S77636[31], 3}, {NULL}};
const Segment SGI102828[] = {{&S77636[77], 6}, {NULL}};
const Segment SGI102829[] = {{&S77638[44], 9}, {NULL}};
const Segment SGI102830[] = {{&S77638[44], 21}, {NULL}};
const Segment SGI102831[] = {{&S77638[30], 6}, {NULL}};
const Segment SGI102832[] = {{&S77638[31], 3}, {NULL}};
const Segment SGI102833[] = {{&S77638[79], 6}, {NULL}};
const Segment SGI102834[] = {{&S100441[44], 18}, {NULL}};
const Segment SGI102835[] = {{&S100441[44], 35}, {NULL}};
const Segment SGI102836[] = {{&S100441[30], 6}, {NULL}};
const Segment SGI102837[] = {{&S100441[93], 6}, {NULL}};
const Segment SGI102838[] = {{&S100441[120], 13}, {NULL}};
const Segment SGI102839[] = {{&S91885[29], 8}, {NULL}};
const Segment SGI102840[] = {{&S91885[53], 3}, {NULL}};
const Segment SGI102841[] = {{&S91886[37], 8}, {NULL}};
const Segment SGI102842[] = {{&S91886[102], 3}, {NULL}};
const Segment SGI102843[] = {{&S91886[27], 3}, {NULL}};
const Segment SGI102844[] = {{&S91886[68], 6}, {NULL}};
const Segment SGI102845[] = {{&S100467[44], 11}, {NULL}};
const Segment SGI102846[] = {{&S100467[44], 23}, {NULL}};
const Segment SGI102847[] = {{&S100467[30], 6}, {NULL}};
const Segment SGI102848[] = {{&S100467[114], 3}, {NULL}};
const Segment SGI102849[] = {{&S100467[81], 5}, {NULL}};
const Segment SGI102850[] = {{&S100467[118], 14}, {NULL}};
const Segment SGI102851[] = {{&S100469[44], 11}, {NULL}};
const Segment SGI102852[] = {{&S100469[44], 23}, {NULL}};
const Segment SGI102853[] = {{&S100469[30], 6}, {NULL}};
const Segment SGI102854[] = {{&S100469[81], 6}, {NULL}};
const Segment SGI102855[] = {{&S100469[108], 13}, {NULL}};
const Segment SGI102856[] = {{&S100470[44], 10}, {NULL}};
const Segment SGI102857[] = {{&S100470[44], 27}, {NULL}};
const Segment SGI102858[] = {{&S100470[30], 6}, {NULL}};
const Segment SGI102859[] = {{&S100470[119], 3}, {NULL}};
const Segment SGI102860[] = {{&S100470[85], 5}, {NULL}};
const Segment SGI102861[] = {{&S100470[142], 10}, {NULL}};
const Segment SGI102862[] = {{&S93560[44], 10}, {NULL}};
const Segment SGI102863[] = {{&S93560[44], 20}, {NULL}};
const Segment SGI102864[] = {{&S93560[30], 6}, {NULL}};
const Segment SGI102865[] = {{&S93560[112], 3}, {NULL}};
const Segment SGI102866[] = {{&S93560[78], 5}, {NULL}};
const Segment SGI102867[] = {{&S100494[44], 8}, {NULL}};
const Segment SGI102868[] = {{&S100494[44], 20}, {NULL}};
const Segment SGI102869[] = {{&S100494[30], 6}, {NULL}};
const Segment SGI102870[] = {{&S100494[78], 6}, {NULL}};
const Segment SGI102871[] = {{&S100494[105], 13}, {NULL}};
const Segment SGI102872[] = {{&S100503[44], 10}, {NULL}};
const Segment SGI102873[] = {{&S100503[44], 27}, {NULL}};
const Segment SGI102874[] = {{&S100503[30], 6}, {NULL}};
const Segment SGI102875[] = {{&S100503[118], 3}, {NULL}};
const Segment SGI102876[] = {{&S100503[85], 5}, {NULL}};
const Segment SGI102877[] = {{&S100503[122], 14}, {NULL}};
const Segment SGI102878[] = {{&S91955[43], 10}, {NULL}};
const Segment SGI102879[] = {{&S91955[30], 4}, {NULL}};
const Segment SGI102880[] = {{&S91955[112], 3}, {NULL}};
const Segment SGI102881[] = {{&S91955[78], 5}, {NULL}};
const Segment SGI102882[] = {{&S91955[136], 13}, {NULL}};
const Segment SGI102883[] = {{&S100518[44], 10}, {NULL}};
const Segment SGI102884[] = {{&S100518[44], 27}, {NULL}};
const Segment SGI102885[] = {{&S100518[30], 6}, {NULL}};
const Segment SGI102886[] = {{&S100518[119], 3}, {NULL}};
const Segment SGI102887[] = {{&S100518[85], 5}, {NULL}};
const Segment SGI102888[] = {{&S100518[142], 10}, {NULL}};
const Segment SGI102889[] = {{&S100519[44], 10}, {NULL}};
const Segment SGI102890[] = {{&S100519[44], 27}, {NULL}};
const Segment SGI102891[] = {{&S100519[30], 6}, {NULL}};
const Segment SGI102892[] = {{&S100519[119], 3}, {NULL}};
const Segment SGI102893[] = {{&S100519[85], 5}, {NULL}};
const Segment SGI102894[] = {{&S100519[142], 10}, {NULL}};
const Segment SGI102895[] = {{&S100529[44], 12}, {NULL}};
const Segment SGI102896[] = {{&S100529[44], 33}, {NULL}};
const Segment SGI102897[] = {{&S100529[30], 6}, {NULL}};
const Segment SGI102898[] = {{&S100529[126], 3}, {NULL}};
const Segment SGI102899[] = {{&S77639[44], 18}, {NULL}};
const Segment SGI102900[] = {{&S77639[44], 30}, {NULL}};
const Segment SGI102901[] = {{&S100529[91], 6}, {NULL}};
const Segment SGI102902[] = {{&S77639[30], 6}, {NULL}};
const Segment SGI102903[] = {{&S100529[146], 8}, {NULL}};
const Segment SGI102904[] = {{&S100529[155], 13}, {NULL}};
const Segment SGI102905[] = {{&S77639[31], 3}, {NULL}};
const Segment SGI102906[] = {{&S77639[88], 6}, {NULL}};
const Segment SGI102907[] = {{&S100530[44], 9}, {NULL}};
const Segment SGI102908[] = {{&S100530[44], 22}, {NULL}};
const Segment SGI102909[] = {{&S100530[30], 6}, {NULL}};
const Segment SGI102910[] = {{&S100530[115], 3}, {NULL}};
const Segment SGI102911[] = {{&S100530[80], 6}, {NULL}};
const Segment SGI102912[] = {{&S100530[135], 8}, {NULL}};
const Segment SGI102913[] = {{&S100530[144], 13}, {NULL}};
const Segment SGI102914[] = {{&S91984[34], 17}, {NULL}};
const Segment SGI102915[] = {{&S91984[27], 6}, {NULL}};
const Segment SGI102916[] = {{&S91984[111], 2}, {NULL}};
const Segment SGI102917[] = {{&S91984[77], 6}, {NULL}};
const Segment SGI102918[] = {{&S91984[138], 8}, {NULL}};
const Segment SGI102919[] = {{&S91985[32], 10}, {NULL}};
const Segment SGI102920[] = {{&S91985[27], 4}, {NULL}};
const Segment SGI102921[] = {{&S91985[102], 2}, {NULL}};
const Segment SGI102922[] = {{&S91985[68], 6}, {NULL}};
const Segment SGI102923[] = {{&S91985[129], 8}, {NULL}};
const Segment SGI102924[] = {{&S100540[59], 10}, {NULL}};
const Segment SGI102925[] = {{&S100540[59], 27}, {NULL}};
const Segment SGI102926[] = {{&S100540[45], 6}, {NULL}};
const Segment SGI102927[] = {{&S77652[34], 14}, {NULL}};
const Segment SGI102928[] = {{&S77652[27], 6}, {NULL}};
const Segment SGI102929[] = {{&S100540[134], 3}, {NULL}};
const Segment SGI102930[] = {{&S100540[100], 5}, {NULL}};
const Segment SGI102931[] = {{&S77652[101], 9}, {NULL}};
const Segment SGI102932[] = {{&S77652[74], 6}, {NULL}};
const Segment SGI102933[] = {{&S100540[0], 14}, {NULL}};
const Segment SGI102934[] = {{&S77653[34], 15}, {NULL}};
const Segment SGI102935[] = {{&S77653[27], 6}, {NULL}};
const Segment SGI102936[] = {{&S77653[102], 9}, {NULL}};
const Segment SGI102937[] = {{&S77653[75], 6}, {NULL}};
const Segment SGI102938[] = {{&S91989[42], 10}, {NULL}};
const Segment SGI102939[] = {{&S91989[42], 22}, {NULL}};
const Segment SGI102940[] = {{&S91989[30], 4}, {NULL}};
const Segment SGI102941[] = {{&S91989[78], 6}, {NULL}};
const Segment SGI102942[] = {{&S91989[105], 13}, {NULL}};
const Segment SGI102943[] = {{&S77660[34], 13}, {NULL}};
const Segment SGI102944[] = {{&S77660[27], 6}, {NULL}};
const Segment SGI102945[] = {{&S77660[100], 9}, {NULL}};
const Segment SGI102946[] = {{&S77660[73], 6}, {NULL}};
const Segment SGI102947[] = {{&S100551[44], 6}, {NULL}};
const Segment SGI102948[] = {{&S100551[44], 18}, {NULL}};
const Segment SGI102949[] = {{&S100551[30], 6}, {NULL}};
const Segment SGI102950[] = {{&S100551[76], 6}, {NULL}};
const Segment SGI102951[] = {{&S100551[103], 13}, {NULL}};
const Segment SGI102952[] = {{&S100563[61], 10}, {NULL}};
const Segment SGI102953[] = {{&S100563[61], 27}, {NULL}};
const Segment SGI102954[] = {{&S100563[47], 6}, {NULL}};
const Segment SGI102955[] = {{&S100563[136], 3}, {NULL}};
const Segment SGI102956[] = {{&S100563[102], 5}, {NULL}};
const Segment SGI102957[] = {{&S100563[0], 14}, {NULL}};
const Segment SGI102958[] = {{&S100580[44], 10}, {NULL}};
const Segment SGI102959[] = {{&S100580[44], 31}, {NULL}};
const Segment SGI102960[] = {{&S100580[30], 6}, {NULL}};
const Segment SGI102961[] = {{&S100580[89], 6}, {NULL}};
const Segment SGI102962[] = {{&S100580[116], 13}, {NULL}};
const Segment SGI102963[] = {{&S77699[44], 10}, {NULL}};
const Segment SGI102964[] = {{&S77699[44], 22}, {NULL}};
const Segment SGI102965[] = {{&S77699[30], 6}, {NULL}};
const Segment SGI102966[] = {{&S77699[139], 4}, {NULL}};
const Segment SGI102967[] = {{&S77699[165], 3}, {NULL}};
const Segment SGI102968[] = {{&S77699[31], 3}, {&S77699[214], 5}, {NULL}};
const Segment SGI102969[] = {{&S77699[80], 6}, {NULL}};
const Segment SGI102970[] = {{&S77700[42], 10}, {NULL}};
const Segment SGI102971[] = {{&S77700[30], 6}, {NULL}};
const Segment SGI102972[] = {{&S77700[137], 4}, {NULL}};
const Segment SGI102973[] = {{&S77700[163], 3}, {NULL}};
const Segment SGI102974[] = {{&S77700[31], 3}, {&S77700[212], 5}, {NULL}};
const Segment SGI102975[] = {{&S77700[78], 6}, {NULL}};
const Segment SGI102976[] = {{&S100602[22], 9}, {NULL}};
const Segment SGI102977[] = {{&S100602[44], 21}, {NULL}};
const Segment SGI102978[] = {{&S100602[30], 6}, {&S100602[52], 13}, {NULL}};
const Segment SGI102979[] = {{&S100602[157], 3}, {NULL}};
const Segment SGI102980[] = {{&S100602[123], 3}, {NULL}};
const Segment SGI102981[] = {{&S100602[88], 6}, {NULL}};
const Segment SGI102982[] = {{&S100602[143], 8}, {NULL}};
const Segment SGI102983[] = {{&S100615[44], 13}, {NULL}};
const Segment SGI102984[] = {{&S100615[44], 25}, {NULL}};
const Segment SGI102985[] = {{&S100615[30], 6}, {NULL}};
const Segment SGI102986[] = {{&S100615[83], 6}, {NULL}};
const Segment SGI102987[] = {{&S100615[110], 13}, {NULL}};
const Segment SGI102988[] = {{&S100624[44], 10}, {NULL}};
const Segment SGI102989[] = {{&S100624[44], 22}, {NULL}};
const Segment SGI102990[] = {{&S100624[30], 6}, {NULL}};
const Segment SGI102991[] = {{&S77741[44], 8}, {NULL}};
const Segment SGI102992[] = {{&S77741[44], 20}, {NULL}};
const Segment SGI102993[] = {{&S100624[114], 3}, {NULL}};
const Segment SGI102994[] = {{&S77741[30], 6}, {NULL}};
const Segment SGI102995[] = {{&S100624[80], 5}, {NULL}};
const Segment SGI102996[] = {{&S77741[31], 3}, {NULL}};
const Segment SGI102997[] = {{&S100624[138], 7}, {NULL}};
const Segment SGI102998[] = {{&S77741[78], 6}, {NULL}};
const Segment SGI102999[] = {{&S77751[44], 14}, {NULL}};
const Segment SGI103000[] = {{&S77751[44], 31}, {NULL}};
const Segment SGI103001[] = {{&S77751[30], 6}, {NULL}};
const Segment SGI103002[] = {{&S77751[89], 6}, {NULL}};
const Segment SGI103003[] = {{&S77751[116], 13}, {NULL}};
const Segment SGI103004[] = {{&S100637[44], 6}, {NULL}};
const Segment SGI103005[] = {{&S100637[44], 18}, {NULL}};
const Segment SGI103006[] = {{&S100637[30], 6}, {NULL}};
const Segment SGI103007[] = {{&S100637[76], 6}, {NULL}};
const Segment SGI103008[] = {{&S100637[103], 13}, {NULL}};
const Segment SGI103009[] = {{&S77759[34], 15}, {NULL}};
const Segment SGI103010[] = {{&S77759[27], 6}, {NULL}};
const Segment SGI103011[] = {{&S77759[102], 9}, {NULL}};
const Segment SGI103012[] = {{&S77759[75], 6}, {NULL}};
const Segment SGI103013[] = {{&S92024[34], 7}, {NULL}};
const Segment SGI103014[] = {{&S92024[27], 6}, {NULL}};
const Segment SGI103015[] = {{&S92024[101], 2}, {NULL}};
const Segment SGI103016[] = {{&S92024[67], 6}, {NULL}};
const Segment SGI103017[] = {{&S92024[128], 8}, {NULL}};
const Segment SGI103018[] = {{&S100663[44], 10}, {NULL}};
const Segment SGI103019[] = {{&S100663[44], 27}, {NULL}};
const Segment SGI103020[] = {{&S100663[30], 6}, {NULL}};
const Segment SGI103021[] = {{&S100663[85], 6}, {NULL}};
const Segment SGI103022[] = {{&S100663[112], 13}, {NULL}};
const Segment SGI103023[] = {{&S100673[44], 24}, {NULL}};
const Segment SGI103024[] = {{&S100673[44], 36}, {NULL}};
const Segment SGI103025[] = {{&S100673[30], 6}, {NULL}};
const Segment SGI103026[] = {{&S100673[129], 3}, {NULL}};
const Segment SGI103027[] = {{&S100673[94], 6}, {NULL}};
const Segment SGI103028[] = {{&S100673[149], 8}, {NULL}};
const Segment SGI103029[] = {{&S100673[158], 13}, {NULL}};
const Segment SGI103030[] = {{&S93761[44], 12}, {NULL}};
const Segment SGI103031[] = {{&S93761[44], 22}, {NULL}};
const Segment SGI103032[] = {{&S93761[30], 6}, {NULL}};
const Segment SGI103033[] = {{&S93761[114], 3}, {NULL}};
const Segment SGI103034[] = {{&S93761[80], 5}, {NULL}};
const Segment SGI103035[] = {{&S100698[44], 10}, {NULL}};
const Segment SGI103036[] = {{&S100698[44], 27}, {NULL}};
const Segment SGI103037[] = {{&S100698[30], 6}, {NULL}};
const Segment SGI103038[] = {{&S100698[85], 6}, {NULL}};
const Segment SGI103039[] = {{&S100698[112], 13}, {NULL}};
const Segment SGI103040[] = {{&S100703[44], 5}, {NULL}};
const Segment SGI103041[] = {{&S100703[44], 26}, {NULL}};
const Segment SGI103042[] = {{&S100703[30], 6}, {NULL}};
const Segment SGI103043[] = {{&S100703[84], 6}, {NULL}};
const Segment SGI103044[] = {{&S100703[111], 13}, {NULL}};
const Segment SGI103045[] = {{&S77855[44], 14}, {NULL}};
const Segment SGI103046[] = {{&S77855[44], 26}, {NULL}};
const Segment SGI103047[] = {{&S77855[30], 6}, {NULL}};
const Segment SGI103048[] = {{&S77855[84], 6}, {NULL}};
const Segment SGI103049[] = {{&S77855[111], 13}, {NULL}};
const Segment SGI103050[] = {{&S93852[44], 13}, {NULL}};
const Segment SGI103051[] = {{&S93852[44], 30}, {NULL}};
const Segment SGI103052[] = {{&S93852[30], 6}, {NULL}};
const Segment SGI103053[] = {{&S93852[53], 3}, {NULL}};
const Segment SGI103054[] = {{&S93852[88], 5}, {NULL}};
const Segment SGI103055[] = {{&S77902[44], 10}, {NULL}};
const Segment SGI103056[] = {{&S77902[44], 22}, {NULL}};
const Segment SGI103057[] = {{&S77902[30], 6}, {NULL}};
const Segment SGI103058[] = {{&S77902[31], 3}, {NULL}};
const Segment SGI103059[] = {{&S77902[80], 6}, {NULL}};
const Segment SGI103060[] = {{&S77902[143], 3}, {NULL}};
const Segment SGI103061[] = {{&S77905[34], 7}, {NULL}};
const Segment SGI103062[] = {{&S77905[27], 6}, {NULL}};
const Segment SGI103063[] = {{&S77905[99], 9}, {NULL}};
const Segment SGI103064[] = {{&S77905[72], 6}, {NULL}};
const Segment SGI103065[] = {{&S100842[31], 3}, {NULL}};
const Segment SGI103066[] = {{&S100842[49], 3}, {NULL}};
const Segment SGI103067[] = {{&S77965[34], 8}, {NULL}};
const Segment SGI103068[] = {{&S77965[27], 6}, {NULL}};
const Segment SGI103069[] = {{&S77965[95], 9}, {NULL}};
const Segment SGI103070[] = {{&S77965[68], 6}, {NULL}};
const Segment SGI103071[] = {{&S93930[52], 10}, {NULL}};
const Segment SGI103072[] = {{&S93930[52], 25}, {NULL}};
const Segment SGI103073[] = {{&S93930[38], 6}, {NULL}};
const Segment SGI103074[] = {{&S93930[91], 5}, {NULL}};
const Segment SGI103075[] = {{&S93930[0], 14}, {NULL}};
const Segment SGI103076[] = {{&S77967[44], 14}, {NULL}};
const Segment SGI103077[] = {{&S77967[44], 26}, {NULL}};
const Segment SGI103078[] = {{&S77967[30], 6}, {NULL}};
const Segment SGI103079[] = {{&S77967[87], 6}, {NULL}};
const Segment SGI103080[] = {{&S77968[44], 14}, {NULL}};
const Segment SGI103081[] = {{&S77968[44], 26}, {NULL}};
const Segment SGI103082[] = {{&S77968[30], 6}, {NULL}};
const Segment SGI103083[] = {{&S77968[87], 4}, {NULL}};
const Segment SGI103084[] = {{&S77968[87], 3}, {NULL}};
const Segment SGI103085[] = {{&S77969[44], 14}, {NULL}};
const Segment SGI103086[] = {{&S77969[44], 26}, {NULL}};
const Segment SGI103087[] = {{&S77969[30], 6}, {NULL}};
const Segment SGI103088[] = {{&S77969[31], 3}, {NULL}};
const Segment SGI103089[] = {{&S77969[84], 6}, {NULL}};
const Segment SGI103090[] = {{&S77970[44], 26}, {NULL}};
const Segment SGI103091[] = {{&S77970[30], 6}, {NULL}};
const Segment SGI103092[] = {{&S77975[34], 9}, {NULL}};
const Segment SGI103093[] = {{&S77975[27], 6}, {NULL}};
const Segment SGI103094[] = {{&S77975[96], 9}, {NULL}};
const Segment SGI103095[] = {{&S77975[69], 6}, {NULL}};
const Segment SGI103096[] = {{&S77978[44], 8}, {NULL}};
const Segment SGI103097[] = {{&S77978[44], 20}, {NULL}};
const Segment SGI103098[] = {{&S77978[30], 6}, {NULL}};
const Segment SGI103099[] = {{&S77978[31], 3}, {NULL}};
const Segment SGI103100[] = {{&S77978[78], 6}, {NULL}};
const Segment SGI103101[] = {{&S100873[58], 8}, {NULL}};
const Segment SGI103102[] = {{&S100873[58], 26}, {NULL}};
const Segment SGI103103[] = {{&S100873[44], 6}, {NULL}};
const Segment SGI103104[] = {{&S100873[132], 3}, {NULL}};
const Segment SGI103105[] = {{&S100873[98], 5}, {NULL}};
const Segment SGI103106[] = {{&S100873[0], 14}, {NULL}};
const Segment SGI103107[] = {{&S78005[34], 13}, {NULL}};
const Segment SGI103108[] = {{&S78005[27], 6}, {NULL}};
const Segment SGI103109[] = {{&S78005[100], 9}, {NULL}};
const Segment SGI103110[] = {{&S78005[73], 6}, {NULL}};
const Segment SGI103111[] = {{&S100898[59], 14}, {NULL}};
const Segment SGI103112[] = {{&S100898[59], 31}, {NULL}};
const Segment SGI103113[] = {{&S100898[45], 6}, {NULL}};
const Segment SGI103114[] = {{&S100898[138], 3}, {NULL}};
const Segment SGI103115[] = {{&S100898[104], 5}, {NULL}};
const Segment SGI103116[] = {{&S100898[0], 14}, {NULL}};
const Segment SGI103117[] = {{&S78020[44], 10}, {NULL}};
const Segment SGI103118[] = {{&S78020[44], 22}, {NULL}};
const Segment SGI103119[] = {{&S78020[30], 6}, {NULL}};
const Segment SGI103120[] = {{&S78020[83], 6}, {NULL}};
const Segment SGI103121[] = {{&S78021[44], 10}, {NULL}};
const Segment SGI103122[] = {{&S78021[44], 22}, {NULL}};
const Segment SGI103123[] = {{&S78021[30], 6}, {NULL}};
const Segment SGI103124[] = {{&S78021[31], 3}, {NULL}};
const Segment SGI103125[] = {{&S78021[80], 6}, {NULL}};
const Segment SGI103126[] = {{&S78022[34], 7}, {NULL}};
const Segment SGI103127[] = {{&S78022[27], 6}, {NULL}};
const Segment SGI103128[] = {{&S78022[94], 9}, {NULL}};
const Segment SGI103129[] = {{&S78022[67], 6}, {NULL}};
const Segment SGI103130[] = {{&S100937[44], 10}, {NULL}};
const Segment SGI103131[] = {{&S100937[44], 27}, {NULL}};
const Segment SGI103132[] = {{&S100937[30], 6}, {NULL}};
const Segment SGI103133[] = {{&S100937[118], 3}, {NULL}};
const Segment SGI103134[] = {{&S100937[85], 5}, {NULL}};
const Segment SGI103135[] = {{&S100937[122], 14}, {NULL}};
const Segment SGI103136[] = {{&S78052[34], 6}, {NULL}};
const Segment SGI103137[] = {{&S78052[27], 6}, {NULL}};
const Segment SGI103138[] = {{&S78052[93], 9}, {NULL}};
const Segment SGI103139[] = {{&S78052[66], 6}, {NULL}};
const Segment SGI103140[] = {{&S94027[44], 12}, {NULL}};
const Segment SGI103141[] = {{&S94027[44], 22}, {NULL}};
const Segment SGI103142[] = {{&S94027[30], 6}, {NULL}};
const Segment SGI103143[] = {{&S94027[114], 3}, {NULL}};
const Segment SGI103144[] = {{&S94027[80], 5}, {NULL}};
const Segment SGI103145[] = {{&S94029[44], 8}, {NULL}};
const Segment SGI103146[] = {{&S94029[44], 18}, {NULL}};
const Segment SGI103147[] = {{&S94029[30], 6}, {NULL}};
const Segment SGI103148[] = {{&S94029[110], 3}, {NULL}};
const Segment SGI103149[] = {{&S94029[76], 5}, {NULL}};
const Segment SGI103150[] = {{&S94030[44], 8}, {NULL}};
const Segment SGI103151[] = {{&S94030[44], 18}, {NULL}};
const Segment SGI103152[] = {{&S94030[30], 6}, {NULL}};
const Segment SGI103153[] = {{&S94030[79], 6}, {NULL}};
const Segment SGI103154[] = {{&S78077[44], 5}, {NULL}};
const Segment SGI103155[] = {{&S78077[44], 17}, {NULL}};
const Segment SGI103156[] = {{&S78077[30], 6}, {NULL}};
const Segment SGI103157[] = {{&S78077[31], 3}, {NULL}};
const Segment SGI103158[] = {{&S78077[75], 6}, {NULL}};
const Segment SGI103159[] = {{&S78079[6], 4}, {NULL}};
const Segment SGI103160[] = {{&S78079[81], 5}, {NULL}};
const Segment SGI103161[] = {{&S86261[44], 11}, {NULL}};
const Segment SGI103162[] = {{&S86261[44], 28}, {NULL}};
const Segment SGI103163[] = {{&S86261[30], 6}, {NULL}};
const Segment SGI103164[] = {{&S86261[31], 3}, {NULL}};
const Segment SGI103165[] = {{&S86261[86], 5}, {NULL}};
const Segment SGI103166[] = {{&S86261[0], 13}, {NULL}};
const Segment SGI103167[] = {{&S78106[34], 11}, {NULL}};
const Segment SGI103168[] = {{&S78106[27], 6}, {NULL}};
const Segment SGI103169[] = {{&S78106[98], 9}, {NULL}};
const Segment SGI103170[] = {{&S78106[71], 6}, {NULL}};
const Segment SGI103171[] = {{&S78106[139], 8}, {NULL}};
const Segment SGI103172[] = {{&S92148[57], 10}, {NULL}};
const Segment SGI103173[] = {{&S92148[57], 22}, {NULL}};
const Segment SGI103174[] = {{&S92148[45], 4}, {NULL}};
const Segment SGI103175[] = {{&S92148[128], 3}, {NULL}};
const Segment SGI103176[] = {{&S92148[93], 6}, {NULL}};
const Segment SGI103177[] = {{&S92148[0], 14}, {NULL}};
const Segment SGI103178[] = {{&S92149[57], 11}, {NULL}};
const Segment SGI103179[] = {{&S92149[57], 23}, {NULL}};
const Segment SGI103180[] = {{&S92149[45], 4}, {NULL}};
const Segment SGI103181[] = {{&S92149[129], 3}, {NULL}};
const Segment SGI103182[] = {{&S92149[94], 6}, {NULL}};
const Segment SGI103183[] = {{&S92149[0], 14}, {NULL}};
const Segment SGI103184[] = {{&S92150[55], 10}, {NULL}};
const Segment SGI103185[] = {{&S92150[45], 4}, {NULL}};
const Segment SGI103186[] = {{&S92150[126], 3}, {NULL}};
const Segment SGI103187[] = {{&S92150[91], 6}, {NULL}};
const Segment SGI103188[] = {{&S92150[0], 14}, {NULL}};
const Segment SGI103189[] = {{&S101028[59], 15}, {NULL}};
const Segment SGI103190[] = {{&S101028[59], 36}, {NULL}};
const Segment SGI103191[] = {{&S101028[45], 6}, {NULL}};
const Segment SGI103192[] = {{&S101028[143], 3}, {NULL}};
const Segment SGI103193[] = {{&S101028[109], 5}, {NULL}};
const Segment SGI103194[] = {{&S101028[0], 14}, {NULL}};
const Segment SGI103195[] = {{&S78135[44], 9}, {NULL}};
const Segment SGI103196[] = {{&S78135[44], 21}, {NULL}};
const Segment SGI103197[] = {{&S78135[30], 6}, {NULL}};
const Segment SGI103198[] = {{&S78135[31], 3}, {NULL}};
const Segment SGI103199[] = {{&S78135[79], 6}, {NULL}};
const Segment SGI103200[] = {{&S101033[58], 9}, {NULL}};
const Segment SGI103201[] = {{&S101033[58], 21}, {NULL}};
const Segment SGI103202[] = {{&S101033[44], 6}, {NULL}};
const Segment SGI103203[] = {{&S101033[127], 3}, {NULL}};
const Segment SGI103204[] = {{&S101033[93], 5}, {NULL}};
const Segment SGI103205[] = {{&S101033[0], 14}, {NULL}};
const Segment SGI103206[] = {{&S101056[57], 5}, {NULL}};
const Segment SGI103207[] = {{&S101056[57], 17}, {NULL}};
const Segment SGI103208[] = {{&S101056[43], 6}, {NULL}};
const Segment SGI103209[] = {{&S101056[122], 3}, {NULL}};
const Segment SGI103210[] = {{&S101056[88], 5}, {NULL}};
const Segment SGI103211[] = {{&S101056[0], 12}, {NULL}};
const Segment SGI103212[] = {{&S78179[44], 12}, {NULL}};
const Segment SGI103213[] = {{&S78179[44], 23}, {NULL}};
const Segment SGI103214[] = {{&S78179[30], 6}, {NULL}};
const Segment SGI103215[] = {{&S78179[31], 3}, {NULL}};
const Segment SGI103216[] = {{&S78179[81], 6}, {NULL}};
const Segment SGI103217[] = {{&S78182[44], 12}, {NULL}};
const Segment SGI103218[] = {{&S78182[44], 24}, {NULL}};
const Segment SGI103219[] = {{&S78182[30], 6}, {NULL}};
const Segment SGI103220[] = {{&S78182[31], 3}, {NULL}};
const Segment SGI103221[] = {{&S78182[82], 6}, {NULL}};
const Segment SGI103222[] = {{&S78183[44], 12}, {NULL}};
const Segment SGI103223[] = {{&S78183[44], 24}, {NULL}};
const Segment SGI103224[] = {{&S78183[30], 6}, {NULL}};
const Segment SGI103225[] = {{&S78183[31], 3}, {NULL}};
const Segment SGI103226[] = {{&S78183[82], 6}, {NULL}};
const Segment SGI103227[] = {{&S78185[44], 18}, {NULL}};
const Segment SGI103228[] = {{&S78185[44], 30}, {NULL}};
const Segment SGI103229[] = {{&S78185[30], 6}, {NULL}};
const Segment SGI103230[] = {{&S78185[31], 3}, {NULL}};
const Segment SGI103231[] = {{&S78185[88], 6}, {NULL}};
const Segment SGI103232[] = {{&S101078[59], 10}, {NULL}};
const Segment SGI103233[] = {{&S101078[59], 27}, {NULL}};
const Segment SGI103234[] = {{&S101078[45], 6}, {NULL}};
const Segment SGI103235[] = {{&S101078[134], 3}, {NULL}};
const Segment SGI103236[] = {{&S101078[100], 5}, {NULL}};
const Segment SGI103237[] = {{&S101078[0], 14}, {NULL}};
const Segment SGI103238[] = {{&S101091[45], 7}, {NULL}};
const Segment SGI103239[] = {{&S101091[30], 6}, {NULL}};
const Segment SGI103240[] = {{&S101091[117], 3}, {NULL}};
const Segment SGI103241[] = {{&S101091[83], 5}, {NULL}};
const Segment SGI103242[] = {{&S101091[141], 13}, {NULL}};
const Segment SGI103243[] = {{&S101098[45], 14}, {NULL}};
const Segment SGI103244[] = {{&S101098[30], 6}, {NULL}};
const Segment SGI103245[] = {{&S101098[119], 3}, {NULL}};
const Segment SGI103246[] = {{&S101098[85], 5}, {NULL}};
const Segment SGI103247[] = {{&S101098[143], 13}, {NULL}};
const Segment SGI103248[] = {{&S78212[44], 9}, {NULL}};
const Segment SGI103249[] = {{&S78212[44], 21}, {NULL}};
const Segment SGI103250[] = {{&S78212[30], 6}, {NULL}};
const Segment SGI103251[] = {{&S78212[79], 6}, {NULL}};
const Segment SGI103252[] = {{&S78212[106], 13}, {NULL}};
const Segment SGI103253[] = {{&S78236[44], 9}, {NULL}};
const Segment SGI103254[] = {{&S78236[44], 21}, {NULL}};
const Segment SGI103255[] = {{&S78236[30], 6}, {NULL}};
const Segment SGI103256[] = {{&S78236[31], 3}, {NULL}};
const Segment SGI103257[] = {{&S78236[79], 6}, {NULL}};
const Segment SGI103258[] = {{&S78252[44], 11}, {NULL}};
const Segment SGI103259[] = {{&S78252[44], 23}, {NULL}};
const Segment SGI103260[] = {{&S78252[30], 6}, {NULL}};
const Segment SGI103261[] = {{&S78252[31], 3}, {NULL}};
const Segment SGI103262[] = {{&S78252[81], 6}, {NULL}};
const Segment SGI103263[] = {{&S78261[44], 10}, {NULL}};
const Segment SGI103264[] = {{&S78261[44], 22}, {NULL}};
const Segment SGI103265[] = {{&S78261[30], 6}, {NULL}};
const Segment SGI103266[] = {{&S78261[80], 6}, {NULL}};
const Segment SGI103267[] = {{&S78261[107], 13}, {NULL}};
const Segment SGI103268[] = {{&S78266[44], 10}, {NULL}};
const Segment SGI103269[] = {{&S78266[44], 22}, {NULL}};
const Segment SGI103270[] = {{&S78266[30], 6}, {NULL}};
const Segment SGI103271[] = {{&S78266[80], 6}, {NULL}};
const Segment SGI103272[] = {{&S78266[107], 13}, {NULL}};
const Segment SGI103273[] = {{&S92333[29], 8}, {NULL}};
const Segment SGI103274[] = {{&S92333[53], 3}, {NULL}};
const Segment SGI103275[] = {{&S78295[34], 8}, {NULL}};
const Segment SGI103276[] = {{&S78295[27], 6}, {NULL}};
const Segment SGI103277[] = {{&S92334[37], 8}, {NULL}};
const Segment SGI103278[] = {{&S92334[102], 3}, {NULL}};
const Segment SGI103279[] = {{&S78295[95], 9}, {NULL}};
const Segment SGI103280[] = {{&S92334[91], 3}, {NULL}};
const Segment SGI103281[] = {{&S78295[68], 6}, {NULL}};
const Segment SGI103282[] = {{&S92334[68], 6}, {NULL}};
const Segment SGI103283[] = {{&S78305[44], 9}, {NULL}};
const Segment SGI103284[] = {{&S78305[44], 21}, {NULL}};
const Segment SGI103285[] = {{&S78305[30], 6}, {NULL}};
const Segment SGI103286[] = {{&S78305[31], 3}, {NULL}};
const Segment SGI103287[] = {{&S78305[79], 6}, {NULL}};
const Segment SGI103288[] = {{&S78313[34], 15}, {NULL}};
const Segment SGI103289[] = {{&S78313[27], 6}, {NULL}};
const Segment SGI103290[] = {{&S78313[108], 9}, {NULL}};
const Segment SGI103291[] = {{&S78313[81], 6}, {NULL}};
const Segment SGI103292[] = {{&S78314[34], 18}, {NULL}};
const Segment SGI103293[] = {{&S78314[27], 6}, {NULL}};
const Segment SGI103294[] = {{&S78314[114], 9}, {NULL}};
const Segment SGI103295[] = {{&S78314[87], 6}, {NULL}};
const Segment SGI103296[] = {{&S101230[59], 14}, {NULL}};
const Segment SGI103297[] = {{&S101230[59], 31}, {NULL}};
const Segment SGI103298[] = {{&S101230[45], 6}, {NULL}};
const Segment SGI103299[] = {{&S101230[138], 3}, {NULL}};
const Segment SGI103300[] = {{&S101230[104], 5}, {NULL}};
const Segment SGI103301[] = {{&S101230[0], 14}, {NULL}};
const Segment SGI103302[] = {{&S78341[44], 12}, {NULL}};
const Segment SGI103303[] = {{&S78341[44], 24}, {NULL}};
const Segment SGI103304[] = {{&S78341[30], 6}, {NULL}};
const Segment SGI103305[] = {{&S78341[31], 3}, {NULL}};
const Segment SGI103306[] = {{&S78341[82], 6}, {NULL}};
const Segment SGI103307[] = {{&S78343[44], 12}, {NULL}};
const Segment SGI103308[] = {{&S78343[44], 34}, {NULL}};
const Segment SGI103309[] = {{&S78343[30], 6}, {NULL}};
const Segment SGI103310[] = {{&S78343[31], 3}, {NULL}};
const Segment SGI103311[] = {{&S78343[92], 6}, {NULL}};
const Segment SGI103312[] = {{&S78386[44], 7}, {NULL}};
const Segment SGI103313[] = {{&S78386[44], 19}, {NULL}};
const Segment SGI103314[] = {{&S78386[30], 6}, {NULL}};
const Segment SGI103315[] = {{&S78386[31], 3}, {NULL}};
const Segment SGI103316[] = {{&S78386[77], 6}, {NULL}};
const Segment SGI103317[] = {{&S78387[34], 24}, {NULL}};
const Segment SGI103318[] = {{&S78387[27], 6}, {NULL}};
const Segment SGI103319[] = {{&S78387[111], 9}, {NULL}};
const Segment SGI103320[] = {{&S78387[84], 6}, {NULL}};
const Segment SGI103321[] = {{&S78396[45], 9}, {NULL}};
const Segment SGI103322[] = {{&S78396[30], 6}, {NULL}};
const Segment SGI103323[] = {{&S78396[31], 3}, {NULL}};
const Segment SGI103324[] = {{&S78396[80], 5}, {NULL}};
const Segment SGI103325[] = {{&S78396[138], 13}, {NULL}};
const Segment SGI103326[] = {{&S78397[45], 9}, {NULL}};
const Segment SGI103327[] = {{&S78397[30], 6}, {NULL}};
const Segment SGI103328[] = {{&S78397[31], 3}, {NULL}};
const Segment SGI103329[] = {{&S78397[80], 5}, {NULL}};
const Segment SGI103330[] = {{&S78397[138], 13}, {NULL}};
const Segment SGI103331[] = {{&S78398[45], 10}, {NULL}};
const Segment SGI103332[] = {{&S78398[30], 6}, {NULL}};
const Segment SGI103333[] = {{&S78398[31], 3}, {NULL}};
const Segment SGI103334[] = {{&S78398[81], 5}, {NULL}};
const Segment SGI103335[] = {{&S78398[139], 13}, {NULL}};
const Segment SGI103336[] = {{&S78401[44], 18}, {NULL}};
const Segment SGI103337[] = {{&S78401[44], 30}, {NULL}};
const Segment SGI103338[] = {{&S78401[30], 6}, {NULL}};
const Segment SGI103339[] = {{&S78401[88], 6}, {NULL}};
const Segment SGI103340[] = {{&S78401[115], 13}, {NULL}};
const Segment SGI103341[] = {{&S78402[44], 21}, {NULL}};
const Segment SGI103342[] = {{&S78402[44], 33}, {NULL}};
const Segment SGI103343[] = {{&S78402[30], 6}, {NULL}};
const Segment SGI103344[] = {{&S78402[91], 6}, {NULL}};
const Segment SGI103345[] = {{&S78402[118], 13}, {NULL}};
const Segment SGI103346[] = {{&S78444[34], 18}, {NULL}};
const Segment SGI103347[] = {{&S78444[27], 6}, {NULL}};
const Segment SGI103348[] = {{&S78444[105], 9}, {NULL}};
const Segment SGI103349[] = {{&S78444[78], 6}, {NULL}};
const Segment SGI103350[] = {{&S78445[34], 13}, {NULL}};
const Segment SGI103351[] = {{&S78445[27], 6}, {NULL}};
const Segment SGI103352[] = {{&S78445[100], 9}, {NULL}};
const Segment SGI103353[] = {{&S78445[73], 6}, {NULL}};
const Segment SGI103354[] = {{&S78457[44], 8}, {NULL}};
const Segment SGI103355[] = {{&S78457[44], 20}, {NULL}};
const Segment SGI103356[] = {{&S78457[30], 6}, {NULL}};
const Segment SGI103357[] = {{&S78457[31], 3}, {NULL}};
const Segment SGI103358[] = {{&S78457[78], 6}, {NULL}};
const Segment SGI103359[] = {{&S101375[59], 11}, {NULL}};
const Segment SGI103360[] = {{&S101375[59], 23}, {NULL}};
const Segment SGI103361[] = {{&S101375[45], 6}, {NULL}};
const Segment SGI103362[] = {{&S101375[130], 3}, {NULL}};
const Segment SGI103363[] = {{&S101375[96], 5}, {NULL}};
const Segment SGI103364[] = {{&S101375[0], 14}, {NULL}};
const Segment SGI103365[] = {{&S101376[59], 17}, {NULL}};
const Segment SGI103366[] = {{&S101376[59], 43}, {NULL}};
const Segment SGI103367[] = {{&S101376[45], 6}, {NULL}};
const Segment SGI103368[] = {{&S101376[150], 3}, {NULL}};
const Segment SGI103369[] = {{&S101376[116], 5}, {NULL}};
const Segment SGI103370[] = {{&S101376[0], 14}, {NULL}};
const Segment SGI103371[] = {{&S94454[31], 5}, {NULL}};
const Segment SGI103372[] = {{&S94454[52], 3}, {NULL}};
const Segment SGI103373[] = {{&S78493[44], 9}, {NULL}};
const Segment SGI103374[] = {{&S78493[44], 21}, {NULL}};
const Segment SGI103375[] = {{&S78493[30], 6}, {NULL}};
const Segment SGI103376[] = {{&S78493[31], 3}, {NULL}};
const Segment SGI103377[] = {{&S78493[79], 6}, {NULL}};
const Segment SGI103378[] = {{&S92543[42], 9}, {NULL}};
const Segment SGI103379[] = {{&S92543[30], 4}, {NULL}};
const Segment SGI103380[] = {{&S92543[100], 3}, {NULL}};
const Segment SGI103381[] = {{&S92543[65], 6}, {NULL}};
const Segment SGI103382[] = {{&S78498[44], 14}, {NULL}};
const Segment SGI103383[] = {{&S78498[44], 26}, {NULL}};
const Segment SGI103384[] = {{&S78498[30], 6}, {NULL}};
const Segment SGI103385[] = {{&S78498[31], 3}, {NULL}};
const Segment SGI103386[] = {{&S78498[84], 6}, {NULL}};
const Segment SGI103387[] = {{&S78500[44], 14}, {NULL}};
const Segment SGI103388[] = {{&S78500[44], 26}, {NULL}};
const Segment SGI103389[] = {{&S78500[30], 6}, {NULL}};
const Segment SGI103390[] = {{&S78500[31], 3}, {NULL}};
const Segment SGI103391[] = {{&S78500[84], 6}, {NULL}};
const Segment SGI103392[] = {{&S101416[45], 10}, {NULL}};
const Segment SGI103393[] = {{&S101416[30], 6}, {NULL}};
const Segment SGI103394[] = {{&S101416[120], 3}, {NULL}};
const Segment SGI103395[] = {{&S101416[86], 5}, {NULL}};
const Segment SGI103396[] = {{&S101416[144], 13}, {NULL}};
const Segment SGI103397[] = {{&S92693[44], 9}, {NULL}};
const Segment SGI103398[] = {{&S92693[44], 21}, {NULL}};
const Segment SGI103399[] = {{&S92693[30], 6}, {NULL}};
const Segment SGI103400[] = {{&S92693[114], 3}, {NULL}};
const Segment SGI103401[] = {{&S92693[79], 6}, {NULL}};
const Segment SGI103402[] = {{&S101571[44], 10}, {NULL}};
const Segment SGI103403[] = {{&S101571[44], 27}, {NULL}};
const Segment SGI103404[] = {{&S101571[30], 6}, {NULL}};
const Segment SGI103405[] = {{&S101571[119], 3}, {NULL}};
const Segment SGI103406[] = {{&S101571[85], 5}, {NULL}};
const Segment SGI103407[] = {{&S101571[142], 10}, {NULL}};
const Segment SGI103408[] = {{&S101584[59], 10}, {NULL}};
const Segment SGI103409[] = {{&S101584[59], 22}, {NULL}};
const Segment SGI103410[] = {{&S101584[45], 6}, {NULL}};
const Segment SGI103411[] = {{&S101584[129], 3}, {NULL}};
const Segment SGI103412[] = {{&S101584[95], 5}, {NULL}};
const Segment SGI103413[] = {{&S101584[0], 14}, {NULL}};
const Segment SGI103414[] = {{&S101591[59], 10}, {NULL}};
const Segment SGI103415[] = {{&S101591[59], 22}, {NULL}};
const Segment SGI103416[] = {{&S101591[45], 6}, {NULL}};
const Segment SGI103417[] = {{&S101591[129], 3}, {NULL}};
const Segment SGI103418[] = {{&S101591[95], 5}, {NULL}};
const Segment SGI103419[] = {{&S101591[0], 14}, {NULL}};
const Segment SGI103420[] = {{&S86902[44], 14}, {NULL}};
const Segment SGI103421[] = {{&S86902[44], 26}, {NULL}};
const Segment SGI103422[] = {{&S86902[30], 6}, {NULL}};
const Segment SGI103423[] = {{&S86902[115], 2}, {NULL}};
const Segment SGI103424[] = {{&S86902[84], 5}, {NULL}};
const Segment SGI103425[] = {{&S86902[0], 13}, {NULL}};
const Segment SGI103426[] = {{&S92729[42], 10}, {NULL}};
const Segment SGI103427[] = {{&S92729[42], 22}, {NULL}};
const Segment SGI103428[] = {{&S92729[30], 4}, {NULL}};
const Segment SGI103429[] = {{&S92729[113], 3}, {NULL}};
const Segment SGI103430[] = {{&S92729[78], 6}, {NULL}};
const Segment SGI103431[] = {{&S92729[131], 13}, {NULL}};
const Segment SGI103432[] = {{&S94696[44], 18}, {NULL}};
const Segment SGI103433[] = {{&S94696[44], 29}, {NULL}};
const Segment SGI103434[] = {{&S94696[30], 6}, {NULL}};
const Segment SGI103435[] = {{&S94696[136], 3}, {NULL}};
const Segment SGI103436[] = {{&S94696[102], 5}, {NULL}};
const Segment SGI103437[] = {{&S94696[35], 1}, {NULL}};
const Segment SGI103438[] = {{&S94696[75], 11}, {NULL}};
const Segment SGI103439[] = {{&S78515[34], 9}, {NULL}};
const Segment SGI103440[] = {{&S78515[27], 6}, {NULL}};
const Segment SGI103441[] = {{&S78515[96], 9}, {NULL}};
const Segment SGI103442[] = {{&S78515[69], 6}, {NULL}};
const Segment SGI103443[] = {{&S86963[74], 6}, {NULL}};
const Segment SGI103444[] = {{&S86963[0], 11}, {NULL}};
const Segment SGI103445[] = {{&S78545[34], 9}, {NULL}};
const Segment SGI103446[] = {{&S78545[27], 6}, {NULL}};
const Segment SGI103447[] = {{&S78545[104], 9}, {NULL}};
const Segment SGI103448[] = {{&S78545[77], 6}, {NULL}};
const Segment SGI103449[] = {{&S78583[44], 11}, {NULL}};
const Segment SGI103450[] = {{&S78583[44], 23}, {NULL}};
const Segment SGI103451[] = {{&S78583[30], 6}, {NULL}};
const Segment SGI103452[] = {{&S78583[31], 3}, {NULL}};
const Segment SGI103453[] = {{&S78583[81], 6}, {NULL}};
const Segment SGI103454[] = {{&S78584[44], 11}, {NULL}};
const Segment SGI103455[] = {{&S78584[44], 23}, {NULL}};
const Segment SGI103456[] = {{&S78584[30], 6}, {NULL}};
const Segment SGI103457[] = {{&S78584[84], 6}, {NULL}};
const Segment SGI103458[] = {{&S78585[44], 11}, {NULL}};
const Segment SGI103459[] = {{&S78585[44], 23}, {NULL}};
const Segment SGI103460[] = {{&S78585[30], 6}, {NULL}};
const Segment SGI103461[] = {{&S78585[31], 3}, {NULL}};
const Segment SGI103462[] = {{&S78585[81], 6}, {NULL}};
const Segment SGI103463[] = {{&S78638[44], 8}, {NULL}};
const Segment SGI103464[] = {{&S78638[44], 20}, {NULL}};
const Segment SGI103465[] = {{&S78638[30], 6}, {NULL}};
const Segment SGI103466[] = {{&S78638[31], 3}, {NULL}};
const Segment SGI103467[] = {{&S78638[78], 6}, {NULL}};
const Segment SGI103468[] = {{&S78658[34], 11}, {NULL}};
const Segment SGI103469[] = {{&S78658[27], 6}, {NULL}};
const Segment SGI103470[] = {{&S78658[98], 9}, {NULL}};
const Segment SGI103471[] = {{&S78658[71], 6}, {NULL}};
const Segment SGI103472[] = {{&S92920[42], 10}, {NULL}};
const Segment SGI103473[] = {{&S92920[42], 22}, {NULL}};
const Segment SGI103474[] = {{&S92920[30], 4}, {NULL}};
const Segment SGI103475[] = {{&S92920[137], 4}, {NULL}};
const Segment SGI103476[] = {{&S92920[163], 3}, {NULL}};
const Segment SGI103477[] = {{&S92920[113], 3}, {&S92920[212], 5}, {NULL}};
const Segment SGI103478[] = {{&S92920[78], 6}, {NULL}};
const Segment SGI103479[] = {{&S92921[42], 10}, {NULL}};
const Segment SGI103480[] = {{&S92921[42], 22}, {NULL}};
const Segment SGI103481[] = {{&S92921[30], 4}, {NULL}};
const Segment SGI103482[] = {{&S92921[137], 4}, {NULL}};
const Segment SGI103483[] = {{&S92921[163], 3}, {NULL}};
const Segment SGI103484[] = {{&S92921[113], 3}, {&S92921[212], 5}, {NULL}};
const Segment SGI103485[] = {{&S92921[78], 6}, {NULL}};
const Segment SGI103486[] = {{&S78683[59], 10}, {NULL}};
const Segment SGI103487[] = {{&S78683[59], 22}, {NULL}};
const Segment SGI103488[] = {{&S78683[45], 6}, {NULL}};
const Segment SGI103489[] = {{&S78683[46], 3}, {NULL}};
const Segment SGI103490[] = {{&S78683[95], 6}, {NULL}};
const Segment SGI103491[] = {{&S78683[0], 14}, {NULL}};
const Segment SGI103492[] = {{&S78685[57], 10}, {NULL}};
const Segment SGI103493[] = {{&S78685[45], 6}, {NULL}};
const Segment SGI103494[] = {{&S78685[46], 3}, {NULL}};
const Segment SGI103495[] = {{&S78685[93], 6}, {NULL}};
const Segment SGI103496[] = {{&S78685[0], 14}, {NULL}};
const Segment SGI103497[] = {{&S78688[45], 11}, {NULL}};
const Segment SGI103498[] = {{&S78688[30], 6}, {NULL}};
const Segment SGI103499[] = {{&S78688[31], 3}, {NULL}};
const Segment SGI103500[] = {{&S78688[82], 5}, {NULL}};
const Segment SGI103501[] = {{&S78688[140], 13}, {NULL}};
const Segment SGI103502[] = {{&S78691[44], 12}, {NULL}};
const Segment SGI103503[] = {{&S78691[44], 24}, {NULL}};
const Segment SGI103504[] = {{&S78691[30], 6}, {NULL}};
const Segment SGI103505[] = {{&S78691[31], 3}, {NULL}};
const Segment SGI103506[] = {{&S78691[82], 6}, {NULL}};
const Segment SGI103507[] = {{&S78701[34], 18}, {NULL}};
const Segment SGI103508[] = {{&S78701[27], 6}, {NULL}};
const Segment SGI103509[] = {{&S78701[105], 9}, {NULL}};
const Segment SGI103510[] = {{&S78701[78], 6}, {NULL}};
const Segment SGI103511[] = {{&S78707[34], 7}, {NULL}};
const Segment SGI103512[] = {{&S78707[27], 6}, {NULL}};
const Segment SGI103513[] = {{&S78707[94], 9}, {NULL}};
const Segment SGI103514[] = {{&S78707[67], 6}, {NULL}};
const Segment SGI103515[] = {{&S78717[34], 5}, {NULL}};
const Segment SGI103516[] = {{&S78717[27], 6}, {NULL}};
const Segment SGI103517[] = {{&S78717[92], 9}, {NULL}};
const Segment SGI103518[] = {{&S78717[65], 6}, {NULL}};
const Segment SGI103519[] = {{&S101709[59], 15}, {NULL}};
const Segment SGI103520[] = {{&S101709[59], 36}, {NULL}};
const Segment SGI103521[] = {{&S101709[45], 6}, {NULL}};
const Segment SGI103522[] = {{&S101709[143], 3}, {NULL}};
const Segment SGI103523[] = {{&S101709[109], 5}, {NULL}};
const Segment SGI103524[] = {{&S101709[0], 14}, {NULL}};
const Segment SGI103525[] = {{&S101710[57], 10}, {NULL}};
const Segment SGI103526[] = {{&S101710[57], 27}, {NULL}};
const Segment SGI103527[] = {{&S101710[43], 6}, {NULL}};
const Segment SGI103528[] = {{&S101710[132], 3}, {NULL}};
const Segment SGI103529[] = {{&S101710[98], 5}, {NULL}};
const Segment SGI103530[] = {{&S101710[0], 12}, {NULL}};
const Segment SGI103531[] = {{&S101729[57], 10}, {NULL}};
const Segment SGI103532[] = {{&S101729[57], 27}, {NULL}};
const Segment SGI103533[] = {{&S101729[43], 6}, {NULL}};
const Segment SGI103534[] = {{&S101729[132], 3}, {NULL}};
const Segment SGI103535[] = {{&S101729[98], 5}, {NULL}};
const Segment SGI103536[] = {{&S101729[0], 12}, {NULL}};
const Segment SGI103537[] = {{&S94952[44], 10}, {NULL}};
const Segment SGI103538[] = {{&S94952[44], 20}, {NULL}};
const Segment SGI103539[] = {{&S94952[30], 6}, {NULL}};
const Segment SGI103540[] = {{&S94952[112], 3}, {NULL}};
const Segment SGI103541[] = {{&S94952[78], 5}, {NULL}};
const Segment SGI103542[] = {{&S101768[44], 10}, {NULL}};
const Segment SGI103543[] = {{&S101768[44], 27}, {NULL}};
const Segment SGI103544[] = {{&S101768[30], 6}, {NULL}};
const Segment SGI103545[] = {{&S101768[118], 3}, {NULL}};
const Segment SGI103546[] = {{&S101768[85], 5}, {NULL}};
const Segment SGI103547[] = {{&S101768[122], 14}, {NULL}};
const Segment SGI103548[] = {{&S78733[34], 6}, {NULL}};
const Segment SGI103549[] = {{&S78733[27], 6}, {NULL}};
const Segment SGI103550[] = {{&S78733[93], 9}, {NULL}};
const Segment SGI103551[] = {{&S78733[66], 6}, {NULL}};
const Segment SGI103552[] = {{&S93096[44], 19}, {NULL}};
const Segment SGI103553[] = {{&S93096[44], 36}, {NULL}};
const Segment SGI103554[] = {{&S93096[30], 6}, {NULL}};
const Segment SGI103555[] = {{&S93096[128], 3}, {NULL}};
const Segment SGI103556[] = {{&S93096[94], 5}, {NULL}};
const Segment SGI103557[] = {{&S78778[34], 13}, {NULL}};
const Segment SGI103558[] = {{&S78778[27], 6}, {NULL}};
const Segment SGI103559[] = {{&S78778[100], 9}, {NULL}};
const Segment SGI103560[] = {{&S78778[73], 6}, {NULL}};
const Segment SGI103561[] = {{&S78779[44], 10}, {NULL}};
const Segment SGI103562[] = {{&S78779[44], 35}, {NULL}};
const Segment SGI103563[] = {{&S78779[30], 6}, {NULL}};
const Segment SGI103564[] = {{&S78779[31], 3}, {NULL}};
const Segment SGI103565[] = {{&S78779[93], 6}, {NULL}};
const Segment SGI103566[] = {{&S78781[44], 10}, {NULL}};
const Segment SGI103567[] = {{&S78781[44], 19}, {NULL}};
const Segment SGI103568[] = {{&S78781[30], 6}, {NULL}};
const Segment SGI103569[] = {{&S78781[31], 3}, {NULL}};
const Segment SGI103570[] = {{&S78781[77], 6}, {NULL}};
const Segment SGI103571[] = {{&S78790[44], 9}, {NULL}};
const Segment SGI103572[] = {{&S78790[44], 21}, {NULL}};
const Segment SGI103573[] = {{&S78790[30], 6}, {NULL}};
const Segment SGI103574[] = {{&S78790[31], 3}, {NULL}};
const Segment SGI103575[] = {{&S78790[79], 6}, {NULL}};
const Segment SGI103576[] = {{&S101858[57], 10}, {NULL}};
const Segment SGI103577[] = {{&S101858[57], 27}, {NULL}};
const Segment SGI103578[] = {{&S101858[43], 6}, {NULL}};
const Segment SGI103579[] = {{&S101858[132], 3}, {NULL}};
const Segment SGI103580[] = {{&S101858[98], 5}, {NULL}};
const Segment SGI103581[] = {{&S101858[0], 12}, {NULL}};
const Segment SGI103582[] = {{&S101866[44], 20}, {NULL}};
const Segment SGI103583[] = {{&S101866[44], 47}, {NULL}};
const Segment SGI103584[] = {{&S101866[30], 6}, {NULL}};
const Segment SGI103585[] = {{&S101866[139], 3}, {NULL}};
const Segment SGI103586[] = {{&S101866[105], 5}, {NULL}};
const Segment SGI103587[] = {{&S101867[44], 17}, {NULL}};
const Segment SGI103588[] = {{&S101867[44], 44}, {NULL}};
const Segment SGI103589[] = {{&S101867[30], 6}, {NULL}};
const Segment SGI103590[] = {{&S101867[136], 3}, {NULL}};
const Segment SGI103591[] = {{&S101867[102], 5}, {NULL}};
const Segment SGI103592[] = {{&S101868[44], 17}, {NULL}};
const Segment SGI103593[] = {{&S101868[44], 44}, {NULL}};
const Segment SGI103594[] = {{&S101868[30], 6}, {NULL}};
const Segment SGI103595[] = {{&S101868[105], 6}, {NULL}};
const Segment SGI103596[] = {{&S78814[34], 7}, {NULL}};
const Segment SGI103597[] = {{&S78814[27], 6}, {NULL}};
const Segment SGI103598[] = {{&S78814[94], 9}, {NULL}};
const Segment SGI103599[] = {{&S78814[67], 6}, {NULL}};
const Segment SGI103600[] = {{&S101904[44], 10}, {NULL}};
const Segment SGI103601[] = {{&S101904[44], 27}, {NULL}};
const Segment SGI103602[] = {{&S101904[30], 6}, {NULL}};
const Segment SGI103603[] = {{&S101904[119], 3}, {NULL}};
const Segment SGI103604[] = {{&S101904[85], 5}, {NULL}};
const Segment SGI103605[] = {{&S101904[143], 11}, {NULL}};
const Segment SGI103606[] = {{&S78853[34], 12}, {NULL}};
const Segment SGI103607[] = {{&S78853[27], 6}, {NULL}};
const Segment SGI103608[] = {{&S78853[99], 9}, {NULL}};
const Segment SGI103609[] = {{&S78853[72], 6}, {NULL}};
const Segment SGI103610[] = {{&S91373[42], 17}, {NULL}};
const Segment SGI103611[] = {{&S91373[42], 29}, {NULL}};
const Segment SGI103612[] = {{&S91373[30], 4}, {NULL}};
const Segment SGI103613[] = {{&S91373[31], 3}, {NULL}};
const Segment SGI103614[] = {{&S91373[85], 6}, {NULL}};
const Segment SGI103615[] = {{&S78885[59], 10}, {NULL}};
const Segment SGI103616[] = {{&S78885[59], 22}, {NULL}};
const Segment SGI103617[] = {{&S78885[45], 6}, {NULL}};
const Segment SGI103618[] = {{&S78885[46], 3}, {NULL}};
const Segment SGI103619[] = {{&S78885[95], 6}, {NULL}};
const Segment SGI103620[] = {{&S78885[0], 14}, {NULL}};
const Segment SGI103621[] = {{&S78903[44], 10}, {NULL}};
const Segment SGI103622[] = {{&S78903[44], 22}, {NULL}};
const Segment SGI103623[] = {{&S78903[30], 6}, {NULL}};
const Segment SGI103624[] = {{&S78903[95], 5}, {NULL}};
const Segment SGI103625[] = {{&S78903[128], 14}, {NULL}};
const Segment SGI103626[] = {{&S78907[44], 10}, {NULL}};
const Segment SGI103627[] = {{&S78907[44], 22}, {NULL}};
const Segment SGI103628[] = {{&S78907[30], 6}, {NULL}};
const Segment SGI103629[] = {{&S78907[111], 2}, {NULL}};
const Segment SGI103630[] = {{&S78907[80], 5}, {NULL}};
const Segment SGI103631[] = {{&S78945[34], 6}, {NULL}};
const Segment SGI103632[] = {{&S78945[27], 6}, {NULL}};
const Segment SGI103633[] = {{&S78945[93], 9}, {NULL}};
const Segment SGI103634[] = {{&S78945[66], 6}, {NULL}};
const Segment SGI103635[] = {{&S78964[34], 18}, {NULL}};
const Segment SGI103636[] = {{&S78964[27], 6}, {NULL}};
const Segment SGI103637[] = {{&S78964[105], 9}, {NULL}};
const Segment SGI103638[] = {{&S78964[78], 6}, {NULL}};
const Segment SGI103639[] = {{&S93325[43], 11}, {NULL}};
const Segment SGI103640[] = {{&S93325[30], 4}, {NULL}};
const Segment SGI103641[] = {{&S93325[114], 3}, {NULL}};
const Segment SGI103642[] = {{&S93325[80], 5}, {NULL}};
const Segment SGI103643[] = {{&S93325[138], 13}, {NULL}};
const Segment SGI103644[] = {{&S93326[43], 10}, {NULL}};
const Segment SGI103645[] = {{&S93326[30], 4}, {NULL}};
const Segment SGI103646[] = {{&S93326[113], 3}, {NULL}};
const Segment SGI103647[] = {{&S93326[79], 5}, {NULL}};
const Segment SGI103648[] = {{&S93326[137], 13}, {NULL}};
const Segment SGI103649[] = {{&S78994[34], 7}, {NULL}};
const Segment SGI103650[] = {{&S78994[27], 6}, {NULL}};
const Segment SGI103651[] = {{&S78994[94], 9}, {NULL}};
const Segment SGI103652[] = {{&S78994[67], 6}, {NULL}};
const Segment SGI103653[] = {{&S78995[34], 6}, {NULL}};
const Segment SGI103654[] = {{&S78995[27], 6}, {NULL}};
const Segment SGI103655[] = {{&S78995[93], 9}, {NULL}};
const Segment SGI103656[] = {{&S78995[66], 6}, {NULL}};
const Segment SGI103657[] = {{&S79006[44], 11}, {NULL}};
const Segment SGI103658[] = {{&S79006[44], 23}, {NULL}};
const Segment SGI103659[] = {{&S79006[30], 6}, {NULL}};
const Segment SGI103660[] = {{&S79006[81], 6}, {NULL}};
const Segment SGI103661[] = {{&S79006[108], 13}, {NULL}};
const Segment SGI103662[] = {{&S79014[34], 18}, {NULL}};
const Segment SGI103663[] = {{&S79014[27], 6}, {NULL}};
const Segment SGI103664[] = {{&S79014[105], 9}, {NULL}};
const Segment SGI103665[] = {{&S79014[78], 6}, {NULL}};
const Segment SGI103666[] = {{&S95157[44], 20}, {NULL}};
const Segment SGI103667[] = {{&S95157[44], 31}, {NULL}};
const Segment SGI103668[] = {{&S95157[30], 6}, {NULL}};
const Segment SGI103669[] = {{&S95157[123], 3}, {NULL}};
const Segment SGI103670[] = {{&S95157[89], 5}, {NULL}};
const Segment SGI103671[] = {{&S79025[44], 13}, {NULL}};
const Segment SGI103672[] = {{&S79025[44], 25}, {NULL}};
const Segment SGI103673[] = {{&S79025[30], 6}, {NULL}};
const Segment SGI103674[] = {{&S79025[31], 3}, {NULL}};
const Segment SGI103675[] = {{&S79025[83], 6}, {NULL}};
const Segment SGI103676[] = {{&S79036[44], 10}, {NULL}};
const Segment SGI103677[] = {{&S102058[59], 11}, {NULL}};
const Segment SGI103678[] = {{&S79036[44], 22}, {NULL}};
const Segment SGI103679[] = {{&S102058[59], 28}, {NULL}};
const Segment SGI103680[] = {{&S79036[30], 6}, {NULL}};
const Segment SGI103681[] = {{&S102058[45], 6}, {NULL}};
const Segment SGI103682[] = {{&S79036[31], 3}, {NULL}};
const Segment SGI103683[] = {{&S102058[135], 3}, {NULL}};
const Segment SGI103684[] = {{&S79036[80], 6}, {NULL}};
const Segment SGI103685[] = {{&S102058[101], 5}, {NULL}};
const Segment SGI103686[] = {{&S79036[140], 11}, {NULL}};
const Segment SGI103687[] = {{&S102058[0], 14}, {NULL}};
const Segment SGI103688[] = {{&S79042[34], 13}, {NULL}};
const Segment SGI103689[] = {{&S79042[27], 6}, {NULL}};
const Segment SGI103690[] = {{&S79042[100], 9}, {NULL}};
const Segment SGI103691[] = {{&S79042[73], 6}, {NULL}};
const Segment SGI103692[] = {{&S79047[34], 9}, {NULL}};
const Segment SGI103693[] = {{&S79047[27], 6}, {NULL}};
const Segment SGI103694[] = {{&S79047[96], 9}, {NULL}};
const Segment SGI103695[] = {{&S79047[69], 6}, {NULL}};
const Segment SGI103696[] = {{&S95194[44], 12}, {NULL}};
const Segment SGI103697[] = {{&S95194[44], 25}, {NULL}};
const Segment SGI103698[] = {{&S95194[30], 6}, {NULL}};
const Segment SGI103699[] = {{&S95194[117], 3}, {NULL}};
const Segment SGI103700[] = {{&S95194[83], 5}, {NULL}};
const Segment SGI103701[] = {{&S95194[148], 7}, {NULL}};
const Segment SGI103702[] = {{&S102075[44], 10}, {NULL}};
const Segment SGI103703[] = {{&S102075[44], 27}, {NULL}};
const Segment SGI103704[] = {{&S102075[30], 6}, {NULL}};
const Segment SGI103705[] = {{&S102075[126], 3}, {NULL}};
const Segment SGI103706[] = {{&S102075[92], 5}, {NULL}};
const Segment SGI103707[] = {{&S102075[71], 8}, {NULL}};
const Segment SGI103708[] = {{&S79067[44], 41}, {NULL}};
const Segment SGI103709[] = {{&S79067[44], 53}, {NULL}};
const Segment SGI103710[] = {{&S79067[30], 6}, {NULL}};
const Segment SGI103711[] = {{&S79067[111], 6}, {NULL}};
const Segment SGI103712[] = {{&S79067[138], 13}, {NULL}};
const Segment SGI103713[] = {{&S102124[45], 11}, {NULL}};
const Segment SGI103714[] = {{&S102124[30], 6}, {NULL}};
const Segment SGI103715[] = {{&S102124[121], 3}, {NULL}};
const Segment SGI103716[] = {{&S102124[87], 5}, {NULL}};
const Segment SGI103717[] = {{&S102124[145], 13}, {NULL}};
const Segment SGI103718[] = {{&S95259[44], 7}, {NULL}};
const Segment SGI103719[] = {{&S95259[44], 33}, {NULL}};
const Segment SGI103720[] = {{&S95259[30], 6}, {NULL}};
const Segment SGI103721[] = {{&S95259[125], 3}, {NULL}};
const Segment SGI103722[] = {{&S95259[91], 5}, {NULL}};
const Segment SGI103723[] = {{&S79121[34], 12}, {NULL}};
const Segment SGI103724[] = {{&S79121[27], 6}, {NULL}};
const Segment SGI103725[] = {{&S79121[99], 9}, {NULL}};
const Segment SGI103726[] = {{&S79121[72], 6}, {NULL}};
const Segment SGI103727[] = {{&S95337[44], 10}, {NULL}};
const Segment SGI103728[] = {{&S102224[57], 10}, {NULL}};
const Segment SGI103729[] = {{&S95337[44], 27}, {NULL}};
const Segment SGI103730[] = {{&S102224[57], 27}, {NULL}};
const Segment SGI103731[] = {{&S95337[30], 6}, {NULL}};
const Segment SGI103732[] = {{&S102224[43], 6}, {NULL}};
const Segment SGI103733[] = {{&S95337[119], 3}, {NULL}};
const Segment SGI103734[] = {{&S102224[132], 3}, {NULL}};
const Segment SGI103735[] = {{&S95337[85], 5}, {NULL}};
const Segment SGI103736[] = {{&S102224[98], 5}, {NULL}};
const Segment SGI103737[] = {{&S95337[143], 7}, {NULL}};
const Segment SGI103738[] = {{&S102224[0], 12}, {NULL}};
const Segment SGI103739[] = {{&S79218[34], 7}, {NULL}};
const Segment SGI103740[] = {{&S79218[27], 6}, {NULL}};
const Segment SGI103741[] = {{&S79218[99], 9}, {NULL}};
const Segment SGI103742[] = {{&S79218[72], 6}, {NULL}};
const Segment SGI103743[] = {{&S102255[59], 10}, {NULL}};
const Segment SGI103744[] = {{&S102255[59], 27}, {NULL}};
const Segment SGI103745[] = {{&S102255[45], 6}, {NULL}};
const Segment SGI103746[] = {{&S102255[134], 3}, {NULL}};
const Segment SGI103747[] = {{&S102255[100], 5}, {NULL}};
const Segment SGI103748[] = {{&S102255[0], 14}, {NULL}};
const Segment SGI103749[] = {{&S102256[59], 10}, {NULL}};
const Segment SGI103750[] = {{&S102256[59], 27}, {NULL}};
const Segment SGI103751[] = {{&S102256[45], 6}, {NULL}};
const Segment SGI103752[] = {{&S102256[134], 3}, {NULL}};
const Segment SGI103753[] = {{&S102256[100], 5}, {NULL}};
const Segment SGI103754[] = {{&S102256[0], 14}, {NULL}};
const Segment SGI103755[] = {{&S79260[34], 16}, {NULL}};
const Segment SGI103756[] = {{&S79260[27], 6}, {NULL}};
const Segment SGI103757[] = {{&S79260[103], 9}, {NULL}};
const Segment SGI103758[] = {{&S79260[76], 6}, {NULL}};
const Segment SGI103759[] = {{&S79261[34], 16}, {NULL}};
const Segment SGI103760[] = {{&S79261[27], 6}, {NULL}};
const Segment SGI103761[] = {{&S79261[103], 9}, {NULL}};
const Segment SGI103762[] = {{&S79261[76], 6}, {NULL}};
const Segment SGI103763[] = {{&S79292[34], 15}, {NULL}};
const Segment SGI103764[] = {{&S79292[27], 6}, {NULL}};
const Segment SGI103765[] = {{&S79292[102], 9}, {NULL}};
const Segment SGI103766[] = {{&S79292[75], 6}, {NULL}};
const Segment SGI103767[] = {{&S102321[34], 10}, {NULL}};
const Segment SGI103768[] = {{&S102321[27], 6}, {NULL}};
const Segment SGI103769[] = {{&S102321[109], 2}, {NULL}};
const Segment SGI103770[] = {{&S102321[75], 6}, {NULL}};
const Segment SGI103771[] = {{&S102321[143], 8}, {NULL}};
const Segment SGI103772[] = {{&S79329[34], 11}, {NULL}};
const Segment SGI103773[] = {{&S79329[27], 6}, {NULL}};
const Segment SGI103774[] = {{&S79329[98], 9}, {NULL}};
const Segment SGI103775[] = {{&S79329[71], 6}, {NULL}};
const Segment SGI103776[] = {{&S95500[44], 7}, {NULL}};
const Segment SGI103777[] = {{&S95500[44], 17}, {NULL}};
const Segment SGI103778[] = {{&S95500[30], 6}, {NULL}};
const Segment SGI103779[] = {{&S95500[75], 6}, {NULL}};
const Segment SGI103780[] = {{&S95500[102], 13}, {NULL}};
const Segment SGI103781[] = {{&S79372[34], 7}, {NULL}};
const Segment SGI103782[] = {{&S79372[27], 6}, {NULL}};
const Segment SGI103783[] = {{&S95502[59], 10}, {NULL}};
const Segment SGI103784[] = {{&S95502[59], 25}, {NULL}};
const Segment SGI103785[] = {{&S95502[45], 6}, {NULL}};
const Segment SGI103786[] = {{&S79372[94], 9}, {NULL}};
const Segment SGI103787[] = {{&S79372[67], 6}, {NULL}};
const Segment SGI103788[] = {{&S95502[132], 3}, {NULL}};
const Segment SGI103789[] = {{&S95502[98], 5}, {NULL}};
const Segment SGI103790[] = {{&S95502[0], 14}, {NULL}};
const Segment SGI103791[] = {{&S93614[44], 12}, {NULL}};
const Segment SGI103792[] = {{&S93614[44], 24}, {NULL}};
const Segment SGI103793[] = {{&S93614[30], 6}, {NULL}};
const Segment SGI103794[] = {{&S93614[117], 3}, {NULL}};
const Segment SGI103795[] = {{&S93614[82], 6}, {NULL}};
const Segment SGI103796[] = {{&S79398[44], 41}, {NULL}};
const Segment SGI103797[] = {{&S79398[44], 53}, {NULL}};
const Segment SGI103798[] = {{&S79398[30], 6}, {NULL}};
const Segment SGI103799[] = {{&S79398[111], 6}, {NULL}};
const Segment SGI103800[] = {{&S79398[138], 13}, {NULL}};
const Segment SGI103801[] = {{&S102451[57], 10}, {NULL}};
const Segment SGI103802[] = {{&S102451[57], 27}, {NULL}};
const Segment SGI103803[] = {{&S102451[43], 6}, {NULL}};
const Segment SGI103804[] = {{&S102451[132], 3}, {NULL}};
const Segment SGI103805[] = {{&S102451[98], 5}, {NULL}};
const Segment SGI103806[] = {{&S102451[0], 12}, {NULL}};
const Segment SGI103807[] = {{&S79427[44], 10}, {NULL}};
const Segment SGI103808[] = {{&S79427[44], 22}, {NULL}};
const Segment SGI103809[] = {{&S79427[30], 6}, {NULL}};
const Segment SGI103810[] = {{&S79427[83], 6}, {NULL}};
const Segment SGI103811[] = {{&S79428[38], 10}, {NULL}};
const Segment SGI103812[] = {{&S79428[31], 6}, {NULL}};
const Segment SGI103813[] = {{&S79430[44], 12}, {NULL}};
const Segment SGI103814[] = {{&S79430[44], 24}, {NULL}};
const Segment SGI103815[] = {{&S79430[30], 6}, {NULL}};
const Segment SGI103816[] = {{&S79430[85], 6}, {NULL}};
const Segment SGI103817[] = {{&S79437[44], 10}, {NULL}};
const Segment SGI103818[] = {{&S79437[44], 19}, {NULL}};
const Segment SGI103819[] = {{&S79437[30], 6}, {NULL}};
const Segment SGI103820[] = {{&S79437[31], 3}, {NULL}};
const Segment SGI103821[] = {{&S79437[77], 6}, {NULL}};
const Segment SGI103822[] = {{&S102481[59], 10}, {NULL}};
const Segment SGI103823[] = {{&S102481[59], 27}, {NULL}};
const Segment SGI103824[] = {{&S102481[45], 6}, {NULL}};
const Segment SGI103825[] = {{&S102481[134], 3}, {NULL}};
const Segment SGI103826[] = {{&S102481[100], 5}, {NULL}};
const Segment SGI103827[] = {{&S102481[0], 14}, {NULL}};
const Segment SGI103828[] = {{&S79455[34], 14}, {NULL}};
const Segment SGI103829[] = {{&S79455[27], 6}, {NULL}};
const Segment SGI103830[] = {{&S79455[101], 9}, {NULL}};
const Segment SGI103831[] = {{&S79455[74], 6}, {NULL}};
const Segment SGI103832[] = {{&S79485[44], 11}, {NULL}};
const Segment SGI103833[] = {{&S79485[44], 23}, {NULL}};
const Segment SGI103834[] = {{&S79485[30], 6}, {NULL}};
const Segment SGI103835[] = {{&S79485[84], 6}, {NULL}};
const Segment SGI103836[] = {{&S79486[44], 22}, {NULL}};
const Segment SGI103837[] = {{&S79486[30], 6}, {NULL}};
const Segment SGI103838[] = {{&S79486[31], 3}, {NULL}};
const Segment SGI103839[] = {{&S79486[82], 6}, {NULL}};
const Segment SGI103840[] = {{&S79487[44], 22}, {NULL}};
const Segment SGI103841[] = {{&S79487[30], 6}, {NULL}};
const Segment SGI103842[] = {{&S79487[31], 3}, {NULL}};
const Segment SGI103843[] = {{&S79487[82], 6}, {NULL}};
const Segment SGI103844[] = {{&S79488[44], 22}, {NULL}};
const Segment SGI103845[] = {{&S79488[30], 6}, {NULL}};
const Segment SGI103846[] = {{&S79488[31], 3}, {NULL}};
const Segment SGI103847[] = {{&S79488[82], 6}, {NULL}};
const Segment SGI103848[] = {{&S79489[44], 22}, {NULL}};
const Segment SGI103849[] = {{&S79489[30], 6}, {NULL}};
const Segment SGI103850[] = {{&S79489[31], 3}, {NULL}};
const Segment SGI103851[] = {{&S79489[82], 6}, {NULL}};
const Segment SGI103852[] = {{&S79539[44], 17}, {NULL}};
const Segment SGI103853[] = {{&S79539[44], 29}, {NULL}};
const Segment SGI103854[] = {{&S79539[30], 6}, {NULL}};
const Segment SGI103855[] = {{&S79539[31], 3}, {NULL}};
const Segment SGI103856[] = {{&S79539[87], 6}, {NULL}};
const Segment SGI103857[] = {{&S79540[44], 10}, {NULL}};
const Segment SGI103858[] = {{&S79540[44], 22}, {NULL}};
const Segment SGI103859[] = {{&S79540[30], 6}, {NULL}};
const Segment SGI103860[] = {{&S79540[31], 3}, {NULL}};
const Segment SGI103861[] = {{&S79540[80], 6}, {NULL}};
const Segment SGI103862[] = {{&S102633[58], 10}, {NULL}};
const Segment SGI103863[] = {{&S102633[58], 27}, {NULL}};
const Segment SGI103864[] = {{&S102633[44], 6}, {NULL}};
const Segment SGI103865[] = {{&S102633[133], 3}, {NULL}};
const Segment SGI103866[] = {{&S102633[99], 5}, {NULL}};
const Segment SGI103867[] = {{&S102633[0], 14}, {NULL}};
const Segment SGI103868[] = {{&S102647[44], 10}, {NULL}};
const Segment SGI103869[] = {{&S102647[44], 27}, {NULL}};
const Segment SGI103870[] = {{&S102647[30], 6}, {NULL}};
const Segment SGI103871[] = {{&S102647[126], 3}, {NULL}};
const Segment SGI103872[] = {{&S102647[92], 5}, {NULL}};
const Segment SGI103873[] = {{&S102647[71], 8}, {NULL}};
const Segment SGI103874[] = {{&S93851[42], 11}, {NULL}};
const Segment SGI103875[] = {{&S93851[42], 23}, {NULL}};
const Segment SGI103876[] = {{&S93851[30], 4}, {NULL}};
const Segment SGI103877[] = {{&S93851[114], 3}, {NULL}};
const Segment SGI103878[] = {{&S93851[79], 6}, {NULL}};
const Segment SGI103879[] = {{&S102670[58], 10}, {NULL}};
const Segment SGI103880[] = {{&S102670[58], 27}, {NULL}};
const Segment SGI103881[] = {{&S102670[44], 6}, {NULL}};
const Segment SGI103882[] = {{&S102670[133], 3}, {NULL}};
const Segment SGI103883[] = {{&S102670[99], 5}, {NULL}};
const Segment SGI103884[] = {{&S102670[0], 14}, {NULL}};
const Segment SGI103885[] = {{&S102672[59], 8}, {NULL}};
const Segment SGI103886[] = {{&S102672[59], 30}, {NULL}};
const Segment SGI103887[] = {{&S102672[45], 6}, {NULL}};
const Segment SGI103888[] = {{&S102672[137], 3}, {NULL}};
const Segment SGI103889[] = {{&S102672[103], 5}, {NULL}};
const Segment SGI103890[] = {{&S102672[0], 14}, {NULL}};
const Segment SGI103891[] = {{&S102673[59], 10}, {NULL}};
const Segment SGI103892[] = {{&S102673[59], 27}, {NULL}};
const Segment SGI103893[] = {{&S102673[45], 6}, {NULL}};
const Segment SGI103894[] = {{&S102673[134], 3}, {NULL}};
const Segment SGI103895[] = {{&S102673[100], 5}, {NULL}};
const Segment SGI103896[] = {{&S102673[0], 14}, {NULL}};
const Segment SGI103897[] = {{&S79597[44], 10}, {NULL}};
const Segment SGI103898[] = {{&S79597[44], 22}, {NULL}};
const Segment SGI103899[] = {{&S79597[30], 6}, {NULL}};
const Segment SGI103900[] = {{&S79597[31], 3}, {NULL}};
const Segment SGI103901[] = {{&S79597[80], 6}, {NULL}};
const Segment SGI103902[] = {{&S102725[59], 7}, {NULL}};
const Segment SGI103903[] = {{&S102725[59], 19}, {NULL}};
const Segment SGI103904[] = {{&S102725[45], 6}, {NULL}};
const Segment SGI103905[] = {{&S102725[126], 3}, {NULL}};
const Segment SGI103906[] = {{&S102725[92], 5}, {NULL}};
const Segment SGI103907[] = {{&S102725[0], 14}, {NULL}};
const Segment SGI103908[] = {{&S95727[44], 8}, {NULL}};
const Segment SGI103909[] = {{&S95727[44], 19}, {NULL}};
const Segment SGI103910[] = {{&S95727[30], 6}, {NULL}};
const Segment SGI103911[] = {{&S95727[111], 3}, {NULL}};
const Segment SGI103912[] = {{&S95727[77], 5}, {NULL}};
const Segment SGI103913[] = {{&S95729[44], 5}, {NULL}};
const Segment SGI103914[] = {{&S95729[44], 16}, {NULL}};
const Segment SGI103915[] = {{&S95729[30], 6}, {NULL}};
const Segment SGI103916[] = {{&S95729[123], 3}, {NULL}};
const Segment SGI103917[] = {{&S95729[89], 5}, {NULL}};
const Segment SGI103918[] = {{&S95729[35], 1}, {NULL}};
const Segment SGI103919[] = {{&S95729[62], 11}, {NULL}};
const Segment SGI103920[] = {{&S79630[34], 16}, {NULL}};
const Segment SGI103921[] = {{&S79630[27], 6}, {NULL}};
const Segment SGI103922[] = {{&S79630[103], 9}, {NULL}};
const Segment SGI103923[] = {{&S79630[76], 6}, {NULL}};
const Segment SGI103924[] = {{&S79631[34], 17}, {NULL}};
const Segment SGI103925[] = {{&S79631[27], 6}, {NULL}};
const Segment SGI103926[] = {{&S79631[104], 9}, {NULL}};
const Segment SGI103927[] = {{&S79631[77], 6}, {NULL}};
const Segment SGI103928[] = {{&S102867[45], 10}, {NULL}};
const Segment SGI103929[] = {{&S102867[30], 6}, {NULL}};
const Segment SGI103930[] = {{&S102867[120], 3}, {NULL}};
const Segment SGI103931[] = {{&S102867[86], 5}, {NULL}};
const Segment SGI103932[] = {{&S102867[144], 13}, {NULL}};
const Segment SGI103933[] = {{&S79642[44], 9}, {NULL}};
const Segment SGI103934[] = {{&S79642[44], 21}, {NULL}};
const Segment SGI103935[] = {{&S79642[30], 6}, {NULL}};
const Segment SGI103936[] = {{&S79642[31], 3}, {NULL}};
const Segment SGI103937[] = {{&S79642[79], 6}, {NULL}};
const Segment SGI103938[] = {{&S102876[59], 10}, {NULL}};
const Segment SGI103939[] = {{&S102876[59], 27}, {NULL}};
const Segment SGI103940[] = {{&S102876[45], 6}, {NULL}};
const Segment SGI103941[] = {{&S102876[134], 3}, {NULL}};
const Segment SGI103942[] = {{&S102876[100], 5}, {NULL}};
const Segment SGI103943[] = {{&S102876[0], 14}, {NULL}};
const Segment SGI103944[] = {{&S102878[56], 10}, {NULL}};
const Segment SGI103945[] = {{&S102878[56], 27}, {NULL}};
const Segment SGI103946[] = {{&S102878[42], 6}, {NULL}};
const Segment SGI103947[] = {{&S102878[131], 3}, {NULL}};
const Segment SGI103948[] = {{&S102878[97], 5}, {NULL}};
const Segment SGI103949[] = {{&S102878[0], 12}, {NULL}};
const Segment SGI103950[] = {{&S95760[59], 7}, {NULL}};
const Segment SGI103951[] = {{&S95760[59], 24}, {NULL}};
const Segment SGI103952[] = {{&S95760[45], 6}, {NULL}};
const Segment SGI103953[] = {{&S95760[131], 3}, {NULL}};
const Segment SGI103954[] = {{&S95760[97], 5}, {NULL}};
const Segment SGI103955[] = {{&S95760[0], 14}, {NULL}};
const Segment SGI103956[] = {{&S79655[34], 9}, {NULL}};
const Segment SGI103957[] = {{&S79655[27], 6}, {NULL}};
const Segment SGI103958[] = {{&S79655[96], 9}, {NULL}};
const Segment SGI103959[] = {{&S79655[69], 6}, {NULL}};
const Segment SGI103960[] = {{&S79697[44], 21}, {NULL}};
const Segment SGI103961[] = {{&S79697[44], 33}, {NULL}};
const Segment SGI103962[] = {{&S79697[30], 6}, {NULL}};
const Segment SGI103963[] = {{&S79697[31], 3}, {NULL}};
const Segment SGI103964[] = {{&S79697[91], 6}, {NULL}};
const Segment SGI103965[] = {{&S79698[44], 22}, {NULL}};
const Segment SGI103966[] = {{&S79698[44], 34}, {NULL}};
const Segment SGI103967[] = {{&S79698[30], 6}, {NULL}};
const Segment SGI103968[] = {{&S79698[31], 3}, {NULL}};
const Segment SGI103969[] = {{&S79698[92], 6}, {NULL}};
const Segment SGI103970[] = {{&S102949[34], 11}, {NULL}};
const Segment SGI103971[] = {{&S102949[27], 6}, {NULL}};
const Segment SGI103972[] = {{&S79723[34], 26}, {NULL}};
const Segment SGI103973[] = {{&S79723[27], 6}, {NULL}};
const Segment SGI103974[] = {{&S102949[110], 2}, {NULL}};
const Segment SGI103975[] = {{&S102949[76], 6}, {NULL}};
const Segment SGI103976[] = {{&S102949[144], 8}, {NULL}};
const Segment SGI103977[] = {{&S79723[113], 9}, {NULL}};
const Segment SGI103978[] = {{&S79723[86], 6}, {NULL}};
const Segment SGI103979[] = {{&S102950[34], 10}, {NULL}};
const Segment SGI103980[] = {{&S102950[27], 6}, {NULL}};
const Segment SGI103981[] = {{&S79724[34], 25}, {NULL}};
const Segment SGI103982[] = {{&S102950[109], 2}, {NULL}};
const Segment SGI103983[] = {{&S79724[27], 6}, {NULL}};
const Segment SGI103984[] = {{&S102950[75], 6}, {NULL}};
const Segment SGI103985[] = {{&S102950[143], 8}, {NULL}};
const Segment SGI103986[] = {{&S79724[112], 9}, {NULL}};
const Segment SGI103987[] = {{&S79724[85], 6}, {NULL}};
const Segment SGI103988[] = {{&S79736[34], 11}, {NULL}};
const Segment SGI103989[] = {{&S79736[27], 6}, {NULL}};
const Segment SGI103990[] = {{&S79736[98], 9}, {NULL}};
const Segment SGI103991[] = {{&S79736[71], 6}, {NULL}};
const Segment SGI103992[] = {{&S79736[139], 8}, {NULL}};
const Segment SGI103993[] = {{&S79763[34], 17}, {NULL}};
const Segment SGI103994[] = {{&S79763[27], 6}, {NULL}};
const Segment SGI103995[] = {{&S79763[104], 9}, {NULL}};
const Segment SGI103996[] = {{&S79763[77], 6}, {NULL}};
const Segment SGI103997[] = {{&S79765[44], 10}, {NULL}};
const Segment SGI103998[] = {{&S79765[44], 22}, {NULL}};
const Segment SGI103999[] = {{&S79765[30], 6}, {NULL}};
const Segment SGI104000[] = {{&S79765[31], 3}, {NULL}};
const Segment SGI104001[] = {{&S79765[80], 6}, {NULL}};
const Segment SGI104002[] = {{&S79775[34], 13}, {NULL}};
const Segment SGI104003[] = {{&S79775[27], 6}, {NULL}};
const Segment SGI104004[] = {{&S79775[100], 9}, {NULL}};
const Segment SGI104005[] = {{&S79775[73], 6}, {NULL}};
const Segment SGI104006[] = {{&S94126[34], 7}, {NULL}};
const Segment SGI104007[] = {{&S94126[27], 6}, {NULL}};
const Segment SGI104008[] = {{&S94126[101], 2}, {NULL}};
const Segment SGI104009[] = {{&S94126[67], 6}, {NULL}};
const Segment SGI104010[] = {{&S94126[128], 8}, {NULL}};
const Segment SGI104011[] = {{&S79796[44], 41}, {NULL}};
const Segment SGI104012[] = {{&S79796[44], 53}, {NULL}};
const Segment SGI104013[] = {{&S79796[30], 6}, {NULL}};
const Segment SGI104014[] = {{&S79796[111], 6}, {NULL}};
const Segment SGI104015[] = {{&S79796[138], 13}, {NULL}};
const Segment SGI104016[] = {{&S79799[44], 12}, {NULL}};
const Segment SGI104017[] = {{&S79799[44], 33}, {NULL}};
const Segment SGI104018[] = {{&S79799[30], 6}, {NULL}};
const Segment SGI104019[] = {{&S79799[91], 6}, {NULL}};
const Segment SGI104020[] = {{&S79799[118], 13}, {NULL}};
const Segment SGI104021[] = {{&S79800[34], 13}, {NULL}};
const Segment SGI104022[] = {{&S79800[27], 6}, {NULL}};
const Segment SGI104023[] = {{&S79800[100], 9}, {NULL}};
const Segment SGI104024[] = {{&S79800[73], 6}, {NULL}};
const Segment SGI104025[] = {{&S79832[44], 14}, {NULL}};
const Segment SGI104026[] = {{&S79832[44], 26}, {NULL}};
const Segment SGI104027[] = {{&S79832[30], 6}, {NULL}};
const Segment SGI104028[] = {{&S79832[31], 3}, {NULL}};
const Segment SGI104029[] = {{&S79832[84], 6}, {NULL}};
const Segment SGI104030[] = {{&S79833[44], 14}, {NULL}};
const Segment SGI104031[] = {{&S79833[44], 26}, {NULL}};
const Segment SGI104032[] = {{&S79833[30], 6}, {NULL}};
const Segment SGI104033[] = {{&S79833[31], 3}, {NULL}};
const Segment SGI104034[] = {{&S79833[84], 6}, {NULL}};
const Segment SGI104035[] = {{&S94179[32], 10}, {NULL}};
const Segment SGI104036[] = {{&S94179[27], 4}, {NULL}};
const Segment SGI104037[] = {{&S94179[102], 2}, {NULL}};
const Segment SGI104038[] = {{&S94179[68], 6}, {NULL}};
const Segment SGI104039[] = {{&S94179[129], 8}, {NULL}};
const Segment SGI104040[] = {{&S79871[34], 6}, {NULL}};
const Segment SGI104041[] = {{&S79871[27], 6}, {NULL}};
const Segment SGI104042[] = {{&S79871[93], 9}, {NULL}};
const Segment SGI104043[] = {{&S79871[66], 6}, {NULL}};
const Segment SGI104044[] = {{&S79875[34], 14}, {NULL}};
const Segment SGI104045[] = {{&S79875[27], 6}, {NULL}};
const Segment SGI104046[] = {{&S79875[101], 9}, {NULL}};
const Segment SGI104047[] = {{&S79875[74], 6}, {NULL}};
const Segment SGI104048[] = {{&S103099[56], 10}, {NULL}};
const Segment SGI104049[] = {{&S103099[56], 27}, {NULL}};
const Segment SGI104050[] = {{&S103099[42], 6}, {NULL}};
const Segment SGI104051[] = {{&S103099[131], 3}, {NULL}};
const Segment SGI104052[] = {{&S103099[97], 5}, {NULL}};
const Segment SGI104053[] = {{&S103099[0], 12}, {NULL}};
const Segment SGI104054[] = {{&S103114[45], 4}, {NULL}};
const Segment SGI104055[] = {{&S103114[30], 6}, {NULL}};
const Segment SGI104056[] = {{&S79899[44], 18}, {NULL}};
const Segment SGI104057[] = {{&S79899[44], 28}, {NULL}};
const Segment SGI104058[] = {{&S79899[30], 6}, {NULL}};
const Segment SGI104059[] = {{&S103114[109], 3}, {NULL}};
const Segment SGI104060[] = {{&S79899[31], 3}, {NULL}};
const Segment SGI104061[] = {{&S103114[75], 5}, {NULL}};
const Segment SGI104062[] = {{&S79899[86], 6}, {NULL}};
const Segment SGI104063[] = {{&S103114[133], 13}, {NULL}};
const Segment SGI104064[] = {{&S79900[44], 18}, {NULL}};
const Segment SGI104065[] = {{&S103115[59], 10}, {NULL}};
const Segment SGI104066[] = {{&S79900[44], 32}, {NULL}};
const Segment SGI104067[] = {{&S79900[30], 6}, {NULL}};
const Segment SGI104068[] = {{&S103115[59], 27}, {NULL}};
const Segment SGI104069[] = {{&S103115[45], 6}, {NULL}};
const Segment SGI104070[] = {{&S79900[31], 3}, {NULL}};
const Segment SGI104071[] = {{&S103115[134], 3}, {NULL}};
const Segment SGI104072[] = {{&S79900[90], 6}, {NULL}};
const Segment SGI104073[] = {{&S103115[100], 5}, {NULL}};
const Segment SGI104074[] = {{&S103115[0], 14}, {NULL}};
const Segment SGI104075[] = {{&S79911[59], 11}, {NULL}};
const Segment SGI104076[] = {{&S79911[59], 23}, {NULL}};
const Segment SGI104077[] = {{&S79911[45], 6}, {NULL}};
const Segment SGI104078[] = {{&S79911[46], 3}, {NULL}};
const Segment SGI104079[] = {{&S79911[96], 6}, {NULL}};
const Segment SGI104080[] = {{&S79911[0], 14}, {NULL}};
const Segment SGI104081[] = {{&S79916[34], 6}, {NULL}};
const Segment SGI104082[] = {{&S79916[27], 6}, {NULL}};
const Segment SGI104083[] = {{&S79916[93], 9}, {NULL}};
const Segment SGI104084[] = {{&S79916[66], 6}, {NULL}};
const Segment SGI104085[] = {{&S79922[34], 9}, {NULL}};
const Segment SGI104086[] = {{&S79922[27], 6}, {NULL}};
const Segment SGI104087[] = {{&S79922[96], 9}, {NULL}};
const Segment SGI104088[] = {{&S79922[69], 6}, {NULL}};
const Segment SGI104089[] = {{&S79922[137], 8}, {NULL}};
const Segment SGI104090[] = {{&S103146[44], 10}, {NULL}};
const Segment SGI104091[] = {{&S103146[44], 27}, {NULL}};
const Segment SGI104092[] = {{&S103146[30], 6}, {NULL}};
const Segment SGI104093[] = {{&S103146[119], 3}, {NULL}};
const Segment SGI104094[] = {{&S103146[85], 5}, {NULL}};
const Segment SGI104095[] = {{&S103146[153], 3}, {NULL}};
const Segment SGI104096[] = {{&S79941[44], 16}, {NULL}};
const Segment SGI104097[] = {{&S79941[44], 28}, {NULL}};
const Segment SGI104098[] = {{&S79941[30], 6}, {NULL}};
const Segment SGI104099[] = {{&S79941[31], 3}, {NULL}};
const Segment SGI104100[] = {{&S79941[86], 6}, {NULL}};
const Segment SGI104101[] = {{&S79942[44], 18}, {NULL}};
const Segment SGI104102[] = {{&S79942[30], 6}, {NULL}};
const Segment SGI104103[] = {{&S79942[31], 3}, {NULL}};
const Segment SGI104104[] = {{&S79942[90], 6}, {NULL}};
const Segment SGI104105[] = {{&S79945[44], 8}, {NULL}};
const Segment SGI104106[] = {{&S79945[44], 20}, {NULL}};
const Segment SGI104107[] = {{&S79945[30], 6}, {NULL}};
const Segment SGI104108[] = {{&S79945[31], 3}, {NULL}};
const Segment SGI104109[] = {{&S79945[78], 6}, {NULL}};
const Segment SGI104110[] = {{&S79954[44], 12}, {NULL}};
const Segment SGI104111[] = {{&S79954[44], 33}, {NULL}};
const Segment SGI104112[] = {{&S79954[30], 6}, {NULL}};
const Segment SGI104113[] = {{&S103167[59], 10}, {NULL}};
const Segment SGI104114[] = {{&S103167[59], 27}, {NULL}};
const Segment SGI104115[] = {{&S103167[45], 6}, {NULL}};
const Segment SGI104116[] = {{&S79954[126], 3}, {NULL}};
const Segment SGI104117[] = {{&S79954[91], 6}, {NULL}};
const Segment SGI104118[] = {{&S79954[146], 8}, {NULL}};
const Segment SGI104119[] = {{&S103167[134], 3}, {NULL}};
const Segment SGI104120[] = {{&S79954[155], 13}, {NULL}};
const Segment SGI104121[] = {{&S103167[100], 5}, {NULL}};
const Segment SGI104122[] = {{&S103167[0], 14}, {NULL}};
const Segment SGI104123[] = {{&S79957[34], 9}, {NULL}};
const Segment SGI104124[] = {{&S79957[27], 6}, {NULL}};
const Segment SGI104125[] = {{&S79957[96], 9}, {NULL}};
const Segment SGI104126[] = {{&S79957[69], 6}, {NULL}};
const Segment SGI104127[] = {{&S96006[44], 10}, {NULL}};
const Segment SGI104128[] = {{&S96006[44], 20}, {NULL}};
const Segment SGI104129[] = {{&S96006[30], 6}, {NULL}};
const Segment SGI104130[] = {{&S96006[112], 3}, {NULL}};
const Segment SGI104131[] = {{&S96006[78], 5}, {NULL}};
const Segment SGI104132[] = {{&S79990[44], 13}, {NULL}};
const Segment SGI104133[] = {{&S79990[44], 31}, {NULL}};
const Segment SGI104134[] = {{&S79990[30], 6}, {NULL}};
const Segment SGI104135[] = {{&S79990[31], 3}, {NULL}};
const Segment SGI104136[] = {{&S79990[89], 6}, {NULL}};
const Segment SGI104137[] = {{&S79991[44], 13}, {NULL}};
const Segment SGI104138[] = {{&S79991[44], 22}, {NULL}};
const Segment SGI104139[] = {{&S79991[30], 6}, {NULL}};
const Segment SGI104140[] = {{&S79991[31], 3}, {NULL}};
const Segment SGI104141[] = {{&S79991[80], 6}, {NULL}};
const Segment SGI104142[] = {{&S79993[44], 13}, {NULL}};
const Segment SGI104143[] = {{&S79993[44], 25}, {NULL}};
const Segment SGI104144[] = {{&S79993[30], 6}, {NULL}};
const Segment SGI104145[] = {{&S79993[31], 3}, {NULL}};
const Segment SGI104146[] = {{&S79993[83], 6}, {NULL}};
const Segment SGI104147[] = {{&S80009[34], 8}, {NULL}};
const Segment SGI104148[] = {{&S80009[27], 6}, {NULL}};
const Segment SGI104149[] = {{&S80009[95], 9}, {NULL}};
const Segment SGI104150[] = {{&S80009[68], 6}, {NULL}};
const Segment SGI104151[] = {{&S80045[44], 10}, {NULL}};
const Segment SGI104152[] = {{&S80045[44], 20}, {NULL}};
const Segment SGI104153[] = {{&S80045[30], 6}, {NULL}};
const Segment SGI104154[] = {{&S80045[31], 3}, {NULL}};
const Segment SGI104155[] = {{&S80045[78], 6}, {NULL}};
const Segment SGI104156[] = {{&S80048[38], 10}, {NULL}};
const Segment SGI104157[] = {{&S80048[31], 6}, {NULL}};
const Segment SGI104158[] = {{&S96084[45], 7}, {NULL}};
const Segment SGI104159[] = {{&S96084[30], 6}, {NULL}};
const Segment SGI104160[] = {{&S96084[111], 3}, {NULL}};
const Segment SGI104161[] = {{&S96084[77], 5}, {NULL}};
const Segment SGI104162[] = {{&S80049[44], 10}, {NULL}};
const Segment SGI104163[] = {{&S80049[44], 23}, {NULL}};
const Segment SGI104164[] = {{&S80049[30], 6}, {NULL}};
const Segment SGI104165[] = {{&S96084[135], 13}, {NULL}};
const Segment SGI104166[] = {{&S80049[31], 3}, {NULL}};
const Segment SGI104167[] = {{&S80049[81], 6}, {NULL}};
const Segment SGI104168[] = {{&S80063[44], 10}, {NULL}};
const Segment SGI104169[] = {{&S80063[44], 22}, {NULL}};
const Segment SGI104170[] = {{&S80063[30], 6}, {NULL}};
const Segment SGI104171[] = {{&S80063[31], 3}, {NULL}};
const Segment SGI104172[] = {{&S80063[80], 6}, {NULL}};
const Segment SGI104173[] = {{&S80078[34], 21}, {NULL}};
const Segment SGI104174[] = {{&S80078[27], 6}, {NULL}};
const Segment SGI104175[] = {{&S80078[108], 9}, {NULL}};
const Segment SGI104176[] = {{&S80078[81], 6}, {NULL}};
const Segment SGI104177[] = {{&S103339[44], 10}, {NULL}};
const Segment SGI104178[] = {{&S103339[44], 27}, {NULL}};
const Segment SGI104179[] = {{&S103339[30], 6}, {NULL}};
const Segment SGI104180[] = {{&S103339[164], 3}, {NULL}};
const Segment SGI104181[] = {{&S103339[130], 3}, {NULL}};
const Segment SGI104182[] = {{&S103339[95], 6}, {NULL}};
const Segment SGI104183[] = {{&S103339[150], 8}, {NULL}};
const Segment SGI104184[] = {{&S103347[59], 10}, {NULL}};
const Segment SGI104185[] = {{&S103347[59], 27}, {NULL}};
const Segment SGI104186[] = {{&S103347[45], 6}, {NULL}};
const Segment SGI104187[] = {{&S103347[134], 3}, {NULL}};
const Segment SGI104188[] = {{&S103347[100], 5}, {NULL}};
const Segment SGI104189[] = {{&S103347[0], 14}, {NULL}};
const Segment SGI104190[] = {{&S80151[34], 9}, {NULL}};
const Segment SGI104191[] = {{&S80151[27], 6}, {NULL}};
const Segment SGI104192[] = {{&S80151[96], 9}, {NULL}};
const Segment SGI104193[] = {{&S80151[69], 6}, {NULL}};
const Segment SGI104194[] = {{&S103396[59], 15}, {NULL}};
const Segment SGI104195[] = {{&S103396[59], 36}, {NULL}};
const Segment SGI104196[] = {{&S103396[45], 6}, {NULL}};
const Segment SGI104197[] = {{&S103396[143], 3}, {NULL}};
const Segment SGI104198[] = {{&S103396[109], 5}, {NULL}};
const Segment SGI104199[] = {{&S103396[0], 14}, {NULL}};
const Segment SGI104200[] = {{&S94418[42], 5}, {NULL}};
const Segment SGI104201[] = {{&S94418[42], 20}, {NULL}};
const Segment SGI104202[] = {{&S94418[30], 4}, {NULL}};
const Segment SGI104203[] = {{&S94418[111], 3}, {NULL}};
const Segment SGI104204[] = {{&S94418[76], 6}, {NULL}};
const Segment SGI104205[] = {{&S80215[44], 7}, {NULL}};
const Segment SGI104206[] = {{&S80215[44], 19}, {NULL}};
const Segment SGI104207[] = {{&S80215[30], 6}, {NULL}};
const Segment SGI104208[] = {{&S80215[31], 3}, {NULL}};
const Segment SGI104209[] = {{&S80215[77], 6}, {NULL}};
const Segment SGI104210[] = {{&S80216[34], 7}, {NULL}};
const Segment SGI104211[] = {{&S80216[27], 6}, {NULL}};
const Segment SGI104212[] = {{&S80216[95], 9}, {NULL}};
const Segment SGI104213[] = {{&S80216[68], 6}, {NULL}};
const Segment SGI104214[] = {{&S80219[56], 4}, {NULL}};
const Segment SGI104215[] = {{&S80219[56], 26}, {NULL}};
const Segment SGI104216[] = {{&S80219[42], 6}, {NULL}};
const Segment SGI104217[] = {{&S80219[43], 3}, {NULL}};
const Segment SGI104218[] = {{&S80219[96], 6}, {NULL}};
const Segment SGI104219[] = {{&S96270[44], 7}, {NULL}};
const Segment SGI104220[] = {{&S96270[44], 24}, {NULL}};
const Segment SGI104221[] = {{&S96270[30], 6}, {NULL}};
const Segment SGI104222[] = {{&S96270[82], 6}, {NULL}};
const Segment SGI104223[] = {{&S96270[109], 13}, {NULL}};
const Segment SGI104224[] = {{&S80229[59], 4}, {NULL}};
const Segment SGI104225[] = {{&S80229[59], 26}, {NULL}};
const Segment SGI104226[] = {{&S80229[45], 6}, {NULL}};
const Segment SGI104227[] = {{&S80229[46], 3}, {NULL}};
const Segment SGI104228[] = {{&S80229[99], 6}, {NULL}};
const Segment SGI104229[] = {{&S80229[0], 14}, {NULL}};
const Segment SGI104230[] = {{&S80236[34], 6}, {NULL}};
const Segment SGI104231[] = {{&S80236[27], 6}, {NULL}};
const Segment SGI104232[] = {{&S80236[93], 9}, {NULL}};
const Segment SGI104233[] = {{&S80236[66], 6}, {NULL}};
const Segment SGI104234[] = {{&S94538[34], 4}, {NULL}};
const Segment SGI104235[] = {{&S94538[27], 6}, {NULL}};
const Segment SGI104236[] = {{&S94538[98], 2}, {NULL}};
const Segment SGI104237[] = {{&S94538[64], 6}, {NULL}};
const Segment SGI104238[] = {{&S94538[125], 8}, {NULL}};
const Segment SGI104239[] = {{&S80255[44], 14}, {NULL}};
const Segment SGI104240[] = {{&S80255[44], 26}, {NULL}};
const Segment SGI104241[] = {{&S80255[30], 6}, {NULL}};
const Segment SGI104242[] = {{&S80255[84], 6}, {NULL}};
const Segment SGI104243[] = {{&S80255[111], 13}, {NULL}};
const Segment SGI104244[] = {{&S80288[44], 9}, {NULL}};
const Segment SGI104245[] = {{&S80288[44], 21}, {NULL}};
const Segment SGI104246[] = {{&S80288[30], 6}, {NULL}};
const Segment SGI104247[] = {{&S80288[31], 3}, {NULL}};
const Segment SGI104248[] = {{&S80288[79], 6}, {NULL}};
const Segment SGI104249[] = {{&S96398[59], 7}, {NULL}};
const Segment SGI104250[] = {{&S96398[59], 24}, {NULL}};
const Segment SGI104251[] = {{&S96398[45], 6}, {NULL}};
const Segment SGI104252[] = {{&S96398[131], 3}, {NULL}};
const Segment SGI104253[] = {{&S96398[97], 5}, {NULL}};
const Segment SGI104254[] = {{&S96398[0], 14}, {NULL}};
const Segment SGI104255[] = {{&S80307[44], 20}, {NULL}};
const Segment SGI104256[] = {{&S80307[44], 32}, {NULL}};
const Segment SGI104257[] = {{&S80307[30], 6}, {NULL}};
const Segment SGI104258[] = {{&S80307[31], 3}, {NULL}};
const Segment SGI104259[] = {{&S80307[90], 6}, {NULL}};
const Segment SGI104260[] = {{&S80309[44], 14}, {NULL}};
const Segment SGI104261[] = {{&S80309[44], 31}, {NULL}};
const Segment SGI104262[] = {{&S80309[30], 6}, {NULL}};
const Segment SGI104263[] = {{&S80309[89], 6}, {NULL}};
const Segment SGI104264[] = {{&S80309[116], 13}, {NULL}};
const Segment SGI104265[] = {{&S80311[44], 10}, {NULL}};
const Segment SGI104266[] = {{&S80311[44], 22}, {NULL}};
const Segment SGI104267[] = {{&S80311[30], 6}, {NULL}};
const Segment SGI104268[] = {{&S80311[111], 2}, {NULL}};
const Segment SGI104269[] = {{&S80311[80], 5}, {NULL}};
const Segment SGI104270[] = {{&S80318[34], 9}, {NULL}};
const Segment SGI104271[] = {{&S80318[27], 6}, {NULL}};
const Segment SGI104272[] = {{&S80318[96], 9}, {NULL}};
const Segment SGI104273[] = {{&S80318[69], 6}, {NULL}};
const Segment SGI104274[] = {{&S80318[137], 8}, {NULL}};
const Segment SGI104275[] = {{&S96420[45], 14}, {NULL}};
const Segment SGI104276[] = {{&S96420[30], 6}, {NULL}};
const Segment SGI104277[] = {{&S96420[123], 3}, {NULL}};
const Segment SGI104278[] = {{&S96420[89], 5}, {NULL}};
const Segment SGI104279[] = {{&S96420[147], 13}, {NULL}};
const Segment SGI104280[] = {{&S80330[34], 17}, {NULL}};
const Segment SGI104281[] = {{&S80330[27], 6}, {NULL}};
const Segment SGI104282[] = {{&S80330[104], 9}, {NULL}};
const Segment SGI104283[] = {{&S80330[77], 6}, {NULL}};
const Segment SGI104284[] = {{&S96435[44], 10}, {NULL}};
const Segment SGI104285[] = {{&S96435[44], 25}, {NULL}};
const Segment SGI104286[] = {{&S96435[30], 6}, {NULL}};
const Segment SGI104287[] = {{&S96435[117], 3}, {NULL}};
const Segment SGI104288[] = {{&S96435[83], 5}, {NULL}};
const Segment SGI104289[] = {{&S96435[141], 11}, {NULL}};
const Segment SGI104290[] = {{&S92950[42], 9}, {NULL}};
const Segment SGI104291[] = {{&S92950[42], 21}, {NULL}};
const Segment SGI104292[] = {{&S92950[30], 4}, {NULL}};
const Segment SGI104293[] = {{&S80343[44], 7}, {NULL}};
const Segment SGI104294[] = {{&S80343[44], 22}, {NULL}};
const Segment SGI104295[] = {{&S92950[31], 3}, {NULL}};
const Segment SGI104296[] = {{&S80343[30], 6}, {NULL}};
const Segment SGI104297[] = {{&S92950[77], 6}, {NULL}};
const Segment SGI104298[] = {{&S80343[31], 3}, {NULL}};
const Segment SGI104299[] = {{&S80343[80], 6}, {NULL}};
const Segment SGI104300[] = {{&S80350[44], 13}, {NULL}};
const Segment SGI104301[] = {{&S80350[44], 25}, {NULL}};
const Segment SGI104302[] = {{&S80350[30], 6}, {NULL}};
const Segment SGI104303[] = {{&S80350[31], 3}, {NULL}};
const Segment SGI104304[] = {{&S80350[83], 6}, {NULL}};
const Segment SGI104305[] = {{&S96459[44], 6}, {NULL}};
const Segment SGI104306[] = {{&S96459[44], 24}, {NULL}};
const Segment SGI104307[] = {{&S96459[30], 6}, {NULL}};
const Segment SGI104308[] = {{&S96459[126], 3}, {NULL}};
const Segment SGI104309[] = {{&S96459[92], 5}, {NULL}};
const Segment SGI104310[] = {{&S96459[65], 3}, {NULL}};
const Segment SGI104311[] = {{&S103668[58], 10}, {NULL}};
const Segment SGI104312[] = {{&S103668[58], 27}, {NULL}};
const Segment SGI104313[] = {{&S103668[44], 6}, {NULL}};
const Segment SGI104314[] = {{&S103668[133], 3}, {NULL}};
const Segment SGI104315[] = {{&S103668[99], 5}, {NULL}};
const Segment SGI104316[] = {{&S103668[0], 14}, {NULL}};
const Segment SGI104317[] = {{&S94743[42], 9}, {NULL}};
const Segment SGI104318[] = {{&S94743[42], 20}, {NULL}};
const Segment SGI104319[] = {{&S94743[30], 4}, {NULL}};
const Segment SGI104320[] = {{&S94743[111], 3}, {NULL}};
const Segment SGI104321[] = {{&S94743[76], 6}, {NULL}};
const Segment SGI104322[] = {{&S80358[31], 8}, {NULL}};
const Segment SGI104323[] = {{&S80358[54], 3}, {NULL}};
const Segment SGI104324[] = {{&S80360[39], 8}, {NULL}};
const Segment SGI104325[] = {{&S80360[103], 3}, {NULL}};
const Segment SGI104326[] = {{&S80360[92], 3}, {NULL}};
const Segment SGI104327[] = {{&S80360[69], 6}, {NULL}};
const Segment SGI104328[] = {{&S80362[44], 16}, {NULL}};
const Segment SGI104329[] = {{&S80362[44], 28}, {NULL}};
const Segment SGI104330[] = {{&S80362[30], 6}, {NULL}};
const Segment SGI104331[] = {{&S80362[31], 3}, {NULL}};
const Segment SGI104332[] = {{&S80362[86], 6}, {NULL}};
const Segment SGI104333[] = {{&S80363[38], 16}, {NULL}};
const Segment SGI104334[] = {{&S80363[31], 6}, {NULL}};
const Segment SGI104335[] = {{&S103711[45], 10}, {NULL}};
const Segment SGI104336[] = {{&S103711[30], 6}, {NULL}};
const Segment SGI104337[] = {{&S103711[120], 3}, {NULL}};
const Segment SGI104338[] = {{&S103711[86], 5}, {NULL}};
const Segment SGI104339[] = {{&S103711[144], 13}, {NULL}};
const Segment SGI104340[] = {{&S103714[59], 10}, {NULL}};
const Segment SGI104341[] = {{&S103714[59], 27}, {NULL}};
const Segment SGI104342[] = {{&S103714[45], 6}, {NULL}};
const Segment SGI104343[] = {{&S103714[134], 3}, {NULL}};
const Segment SGI104344[] = {{&S103714[100], 5}, {NULL}};
const Segment SGI104345[] = {{&S103714[0], 14}, {NULL}};
const Segment SGI104346[] = {{&S80381[34], 12}, {NULL}};
const Segment SGI104347[] = {{&S80381[27], 6}, {NULL}};
const Segment SGI104348[] = {{&S80381[99], 9}, {NULL}};
const Segment SGI104349[] = {{&S80381[72], 6}, {NULL}};
const Segment SGI104350[] = {{&S103720[45], 4}, {NULL}};
const Segment SGI104351[] = {{&S103720[30], 6}, {NULL}};
const Segment SGI104352[] = {{&S103720[109], 3}, {NULL}};
const Segment SGI104353[] = {{&S103720[75], 5}, {NULL}};
const Segment SGI104354[] = {{&S103720[133], 13}, {NULL}};
const Segment SGI104355[] = {{&S80407[44], 11}, {NULL}};
const Segment SGI104356[] = {{&S80407[44], 23}, {NULL}};
const Segment SGI104357[] = {{&S80407[30], 6}, {NULL}};
const Segment SGI104358[] = {{&S80407[31], 3}, {NULL}};
const Segment SGI104359[] = {{&S80407[81], 6}, {NULL}};
const Segment SGI104360[] = {{&S80408[46], 11}, {NULL}};
const Segment SGI104361[] = {{&S80408[46], 36}, {NULL}};
const Segment SGI104362[] = {{&S80408[32], 6}, {NULL}};
const Segment SGI104363[] = {{&S80409[38], 11}, {NULL}};
const Segment SGI104364[] = {{&S80409[31], 6}, {NULL}};
const Segment SGI104365[] = {{&S80410[44], 11}, {NULL}};
const Segment SGI104366[] = {{&S80410[44], 23}, {NULL}};
const Segment SGI104367[] = {{&S80410[30], 6}, {NULL}};
const Segment SGI104368[] = {{&S80410[84], 6}, {NULL}};
const Segment SGI104369[] = {{&S80419[44], 8}, {NULL}};
const Segment SGI104370[] = {{&S80419[44], 20}, {NULL}};
const Segment SGI104371[] = {{&S80419[30], 6}, {NULL}};
const Segment SGI104372[] = {{&S80419[31], 3}, {NULL}};
const Segment SGI104373[] = {{&S80419[78], 6}, {NULL}};
const Segment SGI104374[] = {{&S96543[44], 7}, {NULL}};
const Segment SGI104375[] = {{&S96543[44], 24}, {NULL}};
const Segment SGI104376[] = {{&S96543[30], 6}, {NULL}};
const Segment SGI104377[] = {{&S96543[144], 3}, {NULL}};
const Segment SGI104378[] = {{&S96543[110], 5}, {NULL}};
const Segment SGI104379[] = {{&S96543[70], 11}, {NULL}};
const Segment SGI104380[] = {{&S103773[44], 10}, {NULL}};
const Segment SGI104381[] = {{&S103773[44], 27}, {NULL}};
const Segment SGI104382[] = {{&S103773[30], 6}, {NULL}};
const Segment SGI104383[] = {{&S103773[119], 3}, {NULL}};
const Segment SGI104384[] = {{&S103773[85], 5}, {NULL}};
const Segment SGI104385[] = {{&S103773[130], 14}, {NULL}};
const Segment SGI104386[] = {{&S103792[45], 10}, {NULL}};
const Segment SGI104387[] = {{&S103792[30], 6}, {NULL}};
const Segment SGI104388[] = {{&S103792[120], 3}, {NULL}};
const Segment SGI104389[] = {{&S103792[86], 5}, {NULL}};
const Segment SGI104390[] = {{&S103792[144], 13}, {NULL}};
const Segment SGI104391[] = {{&S94893[44], 6}, {NULL}};
const Segment SGI104392[] = {{&S94893[44], 17}, {NULL}};
const Segment SGI104393[] = {{&S94893[30], 6}, {NULL}};
const Segment SGI104394[] = {{&S94893[110], 3}, {NULL}};
const Segment SGI104395[] = {{&S94893[75], 6}, {NULL}};
const Segment SGI104396[] = {{&S94894[44], 6}, {NULL}};
const Segment SGI104397[] = {{&S94894[44], 18}, {NULL}};
const Segment SGI104398[] = {{&S94894[30], 6}, {NULL}};
const Segment SGI104399[] = {{&S94894[111], 3}, {NULL}};
const Segment SGI104400[] = {{&S94894[76], 6}, {NULL}};
const Segment SGI104401[] = {{&S103830[34], 10}, {NULL}};
const Segment SGI104402[] = {{&S103830[27], 6}, {NULL}};
const Segment SGI104403[] = {{&S103830[109], 2}, {NULL}};
const Segment SGI104404[] = {{&S103830[75], 6}, {NULL}};
const Segment SGI104405[] = {{&S103830[143], 8}, {NULL}};
const Segment SGI104406[] = {{&S94973[57], 10}, {NULL}};
const Segment SGI104407[] = {{&S94973[57], 22}, {NULL}};
const Segment SGI104408[] = {{&S94973[45], 4}, {NULL}};
const Segment SGI104409[] = {{&S94973[128], 3}, {NULL}};
const Segment SGI104410[] = {{&S94973[93], 6}, {NULL}};
const Segment SGI104411[] = {{&S94973[0], 14}, {NULL}};
const Segment SGI104412[] = {{&S94974[57], 11}, {NULL}};
const Segment SGI104413[] = {{&S94974[57], 23}, {NULL}};
const Segment SGI104414[] = {{&S94974[45], 4}, {NULL}};
const Segment SGI104415[] = {{&S94974[129], 3}, {NULL}};
const Segment SGI104416[] = {{&S94974[94], 6}, {NULL}};
const Segment SGI104417[] = {{&S94974[0], 14}, {NULL}};
const Segment SGI104418[] = {{&S104001[59], 10}, {NULL}};
const Segment SGI104419[] = {{&S104001[59], 27}, {NULL}};
const Segment SGI104420[] = {{&S104001[45], 6}, {NULL}};
const Segment SGI104421[] = {{&S104001[134], 3}, {NULL}};
const Segment SGI104422[] = {{&S104001[100], 5}, {NULL}};
const Segment SGI104423[] = {{&S104001[0], 14}, {NULL}};
const Segment SGI104424[] = {{&S80528[34], 22}, {NULL}};
const Segment SGI104425[] = {{&S80528[27], 6}, {NULL}};
const Segment SGI104426[] = {{&S104006[45], 14}, {NULL}};
const Segment SGI104427[] = {{&S104006[30], 6}, {NULL}};
const Segment SGI104428[] = {{&S80528[109], 9}, {NULL}};
const Segment SGI104429[] = {{&S80528[82], 6}, {NULL}};
const Segment SGI104430[] = {{&S104006[119], 3}, {NULL}};
const Segment SGI104431[] = {{&S104006[85], 5}, {NULL}};
const Segment SGI104432[] = {{&S104006[143], 13}, {NULL}};
const Segment SGI104433[] = {{&S80529[34], 16}, {NULL}};
const Segment SGI104434[] = {{&S80529[27], 6}, {NULL}};
const Segment SGI104435[] = {{&S80529[103], 9}, {NULL}};
const Segment SGI104436[] = {{&S80529[76], 6}, {NULL}};
const Segment SGI104437[] = {{&S80530[34], 19}, {NULL}};
const Segment SGI104438[] = {{&S80530[27], 6}, {NULL}};
const Segment SGI104439[] = {{&S80530[106], 9}, {NULL}};
const Segment SGI104440[] = {{&S80530[79], 6}, {NULL}};
const Segment SGI104441[] = {{&S80531[34], 22}, {NULL}};
const Segment SGI104442[] = {{&S80531[27], 6}, {NULL}};
const Segment SGI104443[] = {{&S80531[109], 9}, {NULL}};
const Segment SGI104444[] = {{&S80531[82], 6}, {NULL}};
const Segment SGI104445[] = {{&S80533[34], 13}, {NULL}};
const Segment SGI104446[] = {{&S80533[27], 6}, {NULL}};
const Segment SGI104447[] = {{&S80533[100], 9}, {NULL}};
const Segment SGI104448[] = {{&S80533[73], 6}, {NULL}};
const Segment SGI104449[] = {{&S80623[34], 11}, {NULL}};
const Segment SGI104450[] = {{&S80623[27], 6}, {NULL}};
const Segment SGI104451[] = {{&S80623[98], 9}, {NULL}};
const Segment SGI104452[] = {{&S80623[71], 6}, {NULL}};
const Segment SGI104453[] = {{&S96950[44], 10}, {NULL}};
const Segment SGI104454[] = {{&S96950[44], 20}, {NULL}};
const Segment SGI104455[] = {{&S96950[30], 6}, {NULL}};
const Segment SGI104456[] = {{&S96950[112], 3}, {NULL}};
const Segment SGI104457[] = {{&S96950[78], 5}, {NULL}};
const Segment SGI104458[] = {{&S93437[67], 4}, {NULL}};
const Segment SGI104459[] = {{&S93437[67], 23}, {NULL}};
const Segment SGI104460[] = {{&S93437[44], 15}, {NULL}};
const Segment SGI104461[] = {{&S93437[45], 3}, {NULL}};
const Segment SGI104462[] = {{&S93437[104], 6}, {NULL}};
const Segment SGI104463[] = {{&S96989[44], 7}, {NULL}};
const Segment SGI104464[] = {{&S96989[44], 24}, {NULL}};
const Segment SGI104465[] = {{&S96989[30], 6}, {NULL}};
const Segment SGI104466[] = {{&S96989[57], 3}, {NULL}};
const Segment SGI104467[] = {{&S96989[101], 5}, {NULL}};
const Segment SGI104468[] = {{&S96989[35], 1}, {NULL}};
const Segment SGI104469[] = {{&S96989[70], 11}, {NULL}};
const Segment SGI104470[] = {{&S89609[78], 6}, {NULL}};
const Segment SGI104471[] = {{&S89609[0], 13}, {NULL}};
const Segment SGI104472[] = {{&S80761[44], 14}, {NULL}};
const Segment SGI104473[] = {{&S80761[44], 26}, {NULL}};
const Segment SGI104474[] = {{&S80761[30], 6}, {NULL}};
const Segment SGI104475[] = {{&S80761[31], 3}, {NULL}};
const Segment SGI104476[] = {{&S80761[84], 6}, {NULL}};
const Segment SGI104477[] = {{&S80779[34], 18}, {NULL}};
const Segment SGI104478[] = {{&S80779[27], 6}, {NULL}};
const Segment SGI104479[] = {{&S80779[105], 9}, {NULL}};
const Segment SGI104480[] = {{&S80779[78], 6}, {NULL}};
const Segment SGI104481[] = {{&S80782[34], 11}, {NULL}};
const Segment SGI104482[] = {{&S80782[27], 6}, {NULL}};
const Segment SGI104483[] = {{&S80782[98], 9}, {NULL}};
const Segment SGI104484[] = {{&S80782[71], 6}, {NULL}};
const Segment SGI104485[] = {{&S80782[139], 8}, {NULL}};
const Segment SGI104486[] = {{&S97020[44], 7}, {NULL}};
const Segment SGI104487[] = {{&S97020[44], 24}, {NULL}};
const Segment SGI104488[] = {{&S97020[30], 6}, {NULL}};
const Segment SGI104489[] = {{&S97020[131], 3}, {NULL}};
const Segment SGI104490[] = {{&S97020[97], 5}, {NULL}};
const Segment SGI104491[] = {{&S97020[35], 1}, {NULL}};
const Segment SGI104492[] = {{&S97020[70], 11}, {NULL}};
const Segment SGI104493[] = {{&S80810[44], 15}, {NULL}};
const Segment SGI104494[] = {{&S80810[44], 27}, {NULL}};
const Segment SGI104495[] = {{&S80810[30], 6}, {NULL}};
const Segment SGI104496[] = {{&S80810[116], 2}, {NULL}};
const Segment SGI104497[] = {{&S80810[85], 5}, {NULL}};
const Segment SGI104498[] = {{&S104385[44], 18}, {NULL}};
const Segment SGI104499[] = {{&S104385[44], 35}, {NULL}};
const Segment SGI104500[] = {{&S104385[30], 6}, {NULL}};
const Segment SGI104501[] = {{&S104385[134], 3}, {NULL}};
const Segment SGI104502[] = {{&S104385[100], 5}, {NULL}};
const Segment SGI104503[] = {{&S104385[79], 8}, {NULL}};
const Segment SGI104504[] = {{&S80812[44], 15}, {NULL}};
const Segment SGI104505[] = {{&S80812[44], 26}, {NULL}};
const Segment SGI104506[] = {{&S80812[30], 6}, {NULL}};
const Segment SGI104507[] = {{&S80812[115], 2}, {NULL}};
const Segment SGI104508[] = {{&S80812[84], 5}, {NULL}};
const Segment SGI104509[] = {{&S80836[44], 12}, {NULL}};
const Segment SGI104510[] = {{&S80836[44], 24}, {NULL}};
const Segment SGI104511[] = {{&S80836[30], 6}, {NULL}};
const Segment SGI104512[] = {{&S80836[31], 3}, {NULL}};
const Segment SGI104513[] = {{&S80836[82], 6}, {NULL}};
const Segment SGI104514[] = {{&S104418[59], 10}, {NULL}};
const Segment SGI104515[] = {{&S104418[59], 27}, {NULL}};
const Segment SGI104516[] = {{&S104418[45], 6}, {NULL}};
const Segment SGI104517[] = {{&S104418[134], 3}, {NULL}};
const Segment SGI104518[] = {{&S104418[100], 5}, {NULL}};
const Segment SGI104519[] = {{&S104418[0], 14}, {NULL}};
const Segment SGI104520[] = {{&S80858[34], 16}, {NULL}};
const Segment SGI104521[] = {{&S80858[27], 6}, {NULL}};
const Segment SGI104522[] = {{&S80858[103], 9}, {NULL}};
const Segment SGI104523[] = {{&S80858[76], 6}, {NULL}};
const Segment SGI104524[] = {{&S80859[34], 16}, {NULL}};
const Segment SGI104525[] = {{&S80859[27], 6}, {NULL}};
const Segment SGI104526[] = {{&S80859[103], 9}, {NULL}};
const Segment SGI104527[] = {{&S80859[76], 6}, {NULL}};
const Segment SGI104528[] = {{&S104438[45], 10}, {NULL}};
const Segment SGI104529[] = {{&S104438[30], 6}, {NULL}};
const Segment SGI104530[] = {{&S104438[120], 3}, {NULL}};
const Segment SGI104531[] = {{&S104438[86], 5}, {NULL}};
const Segment SGI104532[] = {{&S104438[144], 13}, {NULL}};
const Segment SGI104533[] = {{&S104453[45], 10}, {NULL}};
const Segment SGI104534[] = {{&S104453[30], 6}, {NULL}};
const Segment SGI104535[] = {{&S104453[120], 3}, {NULL}};
const Segment SGI104536[] = {{&S104453[86], 5}, {NULL}};
const Segment SGI104537[] = {{&S104453[144], 13}, {NULL}};
const Segment SGI104538[] = {{&S80889[58], 41}, {NULL}};
const Segment SGI104539[] = {{&S80889[44], 6}, {NULL}};
const Segment SGI104540[] = {{&S80889[131], 6}, {NULL}};
const Segment SGI104541[] = {{&S80890[58], 14}, {NULL}};
const Segment SGI104542[] = {{&S80890[58], 31}, {NULL}};
const Segment SGI104543[] = {{&S80890[44], 6}, {NULL}};
const Segment SGI104544[] = {{&S80890[45], 3}, {NULL}};
const Segment SGI104545[] = {{&S80890[103], 6}, {NULL}};
const Segment SGI104546[] = {{&S97180[44], 10}, {NULL}};
const Segment SGI104547[] = {{&S97180[44], 25}, {NULL}};
const Segment SGI104548[] = {{&S97180[30], 6}, {NULL}};
const Segment SGI104549[] = {{&S97180[83], 6}, {NULL}};
const Segment SGI104550[] = {{&S97180[110], 13}, {NULL}};
const Segment SGI104551[] = {{&S95485[42], 10}, {NULL}};
const Segment SGI104552[] = {{&S95485[42], 21}, {NULL}};
const Segment SGI104553[] = {{&S95485[30], 4}, {NULL}};
const Segment SGI104554[] = {{&S95485[112], 3}, {NULL}};
const Segment SGI104555[] = {{&S95485[77], 6}, {NULL}};
const Segment SGI104556[] = {{&S95485[130], 13}, {NULL}};
const Segment SGI104557[] = {{&S95486[42], 10}, {NULL}};
const Segment SGI104558[] = {{&S95486[42], 22}, {NULL}};
const Segment SGI104559[] = {{&S95486[30], 4}, {NULL}};
const Segment SGI104560[] = {{&S95486[113], 3}, {NULL}};
const Segment SGI104561[] = {{&S95486[78], 6}, {NULL}};
const Segment SGI104562[] = {{&S95486[131], 11}, {NULL}};
const Segment SGI104563[] = {{&S95487[42], 11}, {NULL}};
const Segment SGI104564[] = {{&S95487[42], 23}, {NULL}};
const Segment SGI104565[] = {{&S95487[30], 4}, {NULL}};
const Segment SGI104566[] = {{&S95487[114], 3}, {NULL}};
const Segment SGI104567[] = {{&S95487[79], 6}, {NULL}};
const Segment SGI104568[] = {{&S95487[132], 11}, {NULL}};
const Segment SGI104569[] = {{&S80907[44], 10}, {NULL}};
const Segment SGI104570[] = {{&S80907[44], 22}, {NULL}};
const Segment SGI104571[] = {{&S80907[30], 6}, {NULL}};
const Segment SGI104572[] = {{&S80907[31], 3}, {NULL}};
const Segment SGI104573[] = {{&S80907[80], 6}, {NULL}};
const Segment SGI104574[] = {{&S80908[44], 8}, {NULL}};
const Segment SGI104575[] = {{&S80908[44], 20}, {NULL}};
const Segment SGI104576[] = {{&S80908[30], 6}, {NULL}};
const Segment SGI104577[] = {{&S80908[31], 3}, {NULL}};
const Segment SGI104578[] = {{&S80908[78], 6}, {NULL}};
const Segment SGI104579[] = {{&S104560[34], 10}, {NULL}};
const Segment SGI104580[] = {{&S104560[27], 6}, {NULL}};
const Segment SGI104581[] = {{&S104560[109], 2}, {NULL}};
const Segment SGI104582[] = {{&S104560[75], 6}, {NULL}};
const Segment SGI104583[] = {{&S104560[143], 8}, {NULL}};
const Segment SGI104584[] = {{&S80987[34], 10}, {NULL}};
const Segment SGI104585[] = {{&S80987[27], 6}, {NULL}};
const Segment SGI104586[] = {{&S80987[97], 9}, {NULL}};
const Segment SGI104587[] = {{&S80987[70], 6}, {NULL}};
const Segment SGI104588[] = {{&S81027[44], 9}, {NULL}};
const Segment SGI104589[] = {{&S81027[44], 21}, {NULL}};
const Segment SGI104590[] = {{&S81027[30], 6}, {NULL}};
const Segment SGI104591[] = {{&S81027[79], 6}, {NULL}};
const Segment SGI104592[] = {{&S81027[106], 13}, {NULL}};
const Segment SGI104593[] = {{&S81068[34], 22}, {NULL}};
const Segment SGI104594[] = {{&S81068[27], 6}, {NULL}};
const Segment SGI104595[] = {{&S81068[109], 9}, {NULL}};
const Segment SGI104596[] = {{&S81068[82], 6}, {NULL}};
const Segment SGI104597[] = {{&S81069[34], 6}, {NULL}};
const Segment SGI104598[] = {{&S81069[27], 6}, {NULL}};
const Segment SGI104599[] = {{&S81069[93], 9}, {NULL}};
const Segment SGI104600[] = {{&S81069[66], 6}, {NULL}};
const Segment SGI104601[] = {{&S81082[45], 9}, {NULL}};
const Segment SGI104602[] = {{&S81082[30], 6}, {NULL}};
const Segment SGI104603[] = {{&S81082[31], 3}, {NULL}};
const Segment SGI104604[] = {{&S81082[80], 5}, {NULL}};
const Segment SGI104605[] = {{&S81082[138], 13}, {NULL}};
const Segment SGI104606[] = {{&S104782[59], 10}, {NULL}};
const Segment SGI104607[] = {{&S104782[59], 27}, {NULL}};
const Segment SGI104608[] = {{&S104782[45], 6}, {NULL}};
const Segment SGI104609[] = {{&S104782[134], 3}, {NULL}};
const Segment SGI104610[] = {{&S104782[100], 5}, {NULL}};
const Segment SGI104611[] = {{&S104782[0], 14}, {NULL}};
const Segment SGI104612[] = {{&S104783[56], 10}, {NULL}};
const Segment SGI104613[] = {{&S104783[56], 27}, {NULL}};
const Segment SGI104614[] = {{&S104783[42], 6}, {NULL}};
const Segment SGI104615[] = {{&S104783[131], 3}, {NULL}};
const Segment SGI104616[] = {{&S104783[97], 5}, {NULL}};
const Segment SGI104617[] = {{&S104783[0], 12}, {NULL}};
const Segment SGI104618[] = {{&S81136[44], 18}, {NULL}};
const Segment SGI104619[] = {{&S81136[44], 29}, {NULL}};
const Segment SGI104620[] = {{&S81136[30], 6}, {NULL}};
const Segment SGI104621[] = {{&S81136[31], 3}, {NULL}};
const Segment SGI104622[] = {{&S81136[87], 6}, {NULL}};
const Segment SGI104623[] = {{&S81137[44], 12}, {NULL}};
const Segment SGI104624[] = {{&S81137[44], 24}, {NULL}};
const Segment SGI104625[] = {{&S81137[30], 6}, {NULL}};
const Segment SGI104626[] = {{&S81137[31], 3}, {NULL}};
const Segment SGI104627[] = {{&S81137[82], 6}, {NULL}};
const Segment SGI104628[] = {{&S94028[42], 6}, {NULL}};
const Segment SGI104629[] = {{&S94028[42], 18}, {NULL}};
const Segment SGI104630[] = {{&S94028[30], 4}, {NULL}};
const Segment SGI104631[] = {{&S94028[31], 3}, {NULL}};
const Segment SGI104632[] = {{&S94028[74], 6}, {NULL}};
const Segment SGI104633[] = {{&S94028[127], 6}, {NULL}};
const Segment SGI104634[] = {{&S104799[44], 12}, {NULL}};
const Segment SGI104635[] = {{&S104799[44], 39}, {NULL}};
const Segment SGI104636[] = {{&S104799[30], 6}, {NULL}};
const Segment SGI104637[] = {{&S104799[138], 3}, {NULL}};
const Segment SGI104638[] = {{&S104799[104], 5}, {NULL}};
const Segment SGI104639[] = {{&S104799[83], 8}, {NULL}};
const Segment SGI104640[] = {{&S81157[44], 11}, {NULL}};
const Segment SGI104641[] = {{&S81157[44], 23}, {NULL}};
const Segment SGI104642[] = {{&S81157[30], 6}, {NULL}};
const Segment SGI104643[] = {{&S81157[81], 6}, {NULL}};
const Segment SGI104644[] = {{&S81157[108], 13}, {NULL}};
const Segment SGI104645[] = {{&S81185[44], 9}, {NULL}};
const Segment SGI104646[] = {{&S81185[44], 20}, {NULL}};
const Segment SGI104647[] = {{&S81185[30], 6}, {NULL}};
const Segment SGI104648[] = {{&S81185[31], 3}, {NULL}};
const Segment SGI104649[] = {{&S81185[78], 6}, {NULL}};
const Segment SGI104650[] = {{&S81194[45], 14}, {NULL}};
const Segment SGI104651[] = {{&S81194[30], 6}, {NULL}};
const Segment SGI104652[] = {{&S81194[31], 3}, {NULL}};
const Segment SGI104653[] = {{&S81194[85], 5}, {NULL}};
const Segment SGI104654[] = {{&S81194[143], 13}, {NULL}};
const Segment SGI104655[] = {{&S81199[44], 10}, {NULL}};
const Segment SGI104656[] = {{&S81199[44], 22}, {NULL}};
const Segment SGI104657[] = {{&S81199[30], 6}, {NULL}};
const Segment SGI104658[] = {{&S81199[80], 6}, {NULL}};
const Segment SGI104659[] = {{&S81199[107], 13}, {NULL}};
const Segment SGI104660[] = {{&S104857[44], 9}, {NULL}};
const Segment SGI104661[] = {{&S104857[44], 21}, {NULL}};
const Segment SGI104662[] = {{&S104857[30], 6}, {NULL}};
const Segment SGI104663[] = {{&S104857[113], 3}, {NULL}};
const Segment SGI104664[] = {{&S104857[79], 5}, {NULL}};
const Segment SGI104665[] = {{&S81218[44], 10}, {NULL}};
const Segment SGI104666[] = {{&S81218[44], 22}, {NULL}};
const Segment SGI104667[] = {{&S81218[30], 6}, {NULL}};
const Segment SGI104668[] = {{&S81218[80], 6}, {NULL}};
const Segment SGI104669[] = {{&S81218[107], 13}, {NULL}};
const Segment SGI104670[] = {{&S81223[44], 14}, {NULL}};
const Segment SGI104671[] = {{&S81223[44], 26}, {NULL}};
const Segment SGI104672[] = {{&S81223[30], 6}, {NULL}};
const Segment SGI104673[] = {{&S81223[84], 6}, {NULL}};
const Segment SGI104674[] = {{&S81223[111], 13}, {NULL}};
const Segment SGI104675[] = {{&S97472[34], 12}, {NULL}};
const Segment SGI104676[] = {{&S97472[27], 6}, {NULL}};
const Segment SGI104677[] = {{&S97472[105], 2}, {NULL}};
const Segment SGI104678[] = {{&S97472[71], 6}, {NULL}};
const Segment SGI104679[] = {{&S97472[139], 8}, {NULL}};
const Segment SGI104680[] = {{&S81268[34], 9}, {NULL}};
const Segment SGI104681[] = {{&S81268[27], 6}, {NULL}};
const Segment SGI104682[] = {{&S81268[96], 9}, {NULL}};
const Segment SGI104683[] = {{&S81268[69], 6}, {NULL}};
const Segment SGI104684[] = {{&S81273[44], 8}, {NULL}};
const Segment SGI104685[] = {{&S81273[44], 15}, {NULL}};
const Segment SGI104686[] = {{&S81273[30], 6}, {NULL}};
const Segment SGI104687[] = {{&S81273[31], 3}, {NULL}};
const Segment SGI104688[] = {{&S81273[73], 6}, {NULL}};
const Segment SGI104689[] = {{&S95781[42], 10}, {NULL}};
const Segment SGI104690[] = {{&S95781[42], 22}, {NULL}};
const Segment SGI104691[] = {{&S95781[30], 4}, {NULL}};
const Segment SGI104692[] = {{&S95781[137], 4}, {NULL}};
const Segment SGI104693[] = {{&S95781[163], 3}, {NULL}};
const Segment SGI104694[] = {{&S95781[113], 3}, {&S95781[212], 5}, {NULL}};
const Segment SGI104695[] = {{&S95781[78], 6}, {NULL}};
const Segment SGI104696[] = {{&S81293[34], 9}, {NULL}};
const Segment SGI104697[] = {{&S81293[27], 6}, {NULL}};
const Segment SGI104698[] = {{&S81293[96], 9}, {NULL}};
const Segment SGI104699[] = {{&S81293[69], 6}, {NULL}};
const Segment SGI104700[] = {{&S81293[137], 8}, {NULL}};
const Segment SGI104701[] = {{&S81294[34], 9}, {NULL}};
const Segment SGI104702[] = {{&S81294[27], 6}, {NULL}};
const Segment SGI104703[] = {{&S81294[96], 9}, {NULL}};
const Segment SGI104704[] = {{&S81294[69], 6}, {NULL}};
const Segment SGI104705[] = {{&S81294[137], 8}, {NULL}};
const Segment SGI104706[] = {{&S81326[44], 8}, {NULL}};
const Segment SGI104707[] = {{&S81326[44], 20}, {NULL}};
const Segment SGI104708[] = {{&S81326[30], 6}, {NULL}};
const Segment SGI104709[] = {{&S81326[31], 3}, {NULL}};
const Segment SGI104710[] = {{&S81326[78], 6}, {NULL}};
const Segment SGI104711[] = {{&S81329[34], 6}, {NULL}};
const Segment SGI104712[] = {{&S81329[27], 6}, {NULL}};
const Segment SGI104713[] = {{&S81329[93], 9}, {NULL}};
const Segment SGI104714[] = {{&S81329[66], 6}, {NULL}};
const Segment SGI104715[] = {{&S81332[44], 13}, {NULL}};
const Segment SGI104716[] = {{&S81332[44], 25}, {NULL}};
const Segment SGI104717[] = {{&S81332[30], 6}, {NULL}};
const Segment SGI104718[] = {{&S81332[31], 3}, {NULL}};
const Segment SGI104719[] = {{&S81332[83], 6}, {NULL}};
const Segment SGI104720[] = {{&S81336[44], 11}, {NULL}};
const Segment SGI104721[] = {{&S81336[44], 23}, {NULL}};
const Segment SGI104722[] = {{&S81336[30], 6}, {NULL}};
const Segment SGI104723[] = {{&S81336[31], 3}, {NULL}};
const Segment SGI104724[] = {{&S81336[81], 6}, {NULL}};
const Segment SGI104725[] = {{&S97592[44], 7}, {NULL}};
const Segment SGI104726[] = {{&S97592[44], 24}, {NULL}};
const Segment SGI104727[] = {{&S97592[30], 6}, {NULL}};
const Segment SGI104728[] = {{&S97592[82], 6}, {NULL}};
const Segment SGI104729[] = {{&S97592[109], 13}, {NULL}};
const Segment SGI104730[] = {{&S81354[34], 14}, {NULL}};
const Segment SGI104731[] = {{&S81354[27], 6}, {NULL}};
const Segment SGI104732[] = {{&S81354[106], 9}, {NULL}};
const Segment SGI104733[] = {{&S81354[79], 6}, {NULL}};
const Segment SGI104734[] = {{&S81356[34], 12}, {NULL}};
const Segment SGI104735[] = {{&S81356[27], 6}, {NULL}};
const Segment SGI104736[] = {{&S81356[99], 9}, {NULL}};
const Segment SGI104737[] = {{&S81356[72], 6}, {NULL}};
const Segment SGI104738[] = {{&S81377[34], 6}, {NULL}};
const Segment SGI104739[] = {{&S81377[27], 6}, {NULL}};
const Segment SGI104740[] = {{&S81377[93], 9}, {NULL}};
const Segment SGI104741[] = {{&S81377[66], 6}, {NULL}};
const Segment SGI104742[] = {{&S86901[44], 14}, {NULL}};
const Segment SGI104743[] = {{&S86901[44], 26}, {NULL}};
const Segment SGI104744[] = {{&S86901[30], 6}, {NULL}};
const Segment SGI104745[] = {{&S86901[115], 2}, {NULL}};
const Segment SGI104746[] = {{&S86901[84], 5}, {NULL}};
const Segment SGI104747[] = {{&S81398[44], 13}, {NULL}};
const Segment SGI104748[] = {{&S81398[44], 25}, {NULL}};
const Segment SGI104749[] = {{&S81398[30], 6}, {NULL}};
const Segment SGI104750[] = {{&S81398[31], 3}, {NULL}};
const Segment SGI104751[] = {{&S81398[83], 6}, {NULL}};
const Segment SGI104752[] = {{&S81399[44], 13}, {NULL}};
const Segment SGI104753[] = {{&S81399[44], 25}, {NULL}};
const Segment SGI104754[] = {{&S81399[30], 6}, {NULL}};
const Segment SGI104755[] = {{&S81399[86], 4}, {NULL}};
const Segment SGI104756[] = {{&S81399[86], 3}, {NULL}};
const Segment SGI104757[] = {{&S81400[44], 13}, {NULL}};
const Segment SGI104758[] = {{&S81400[44], 25}, {NULL}};
const Segment SGI104759[] = {{&S81400[30], 6}, {NULL}};
const Segment SGI104760[] = {{&S81400[86], 6}, {NULL}};
const Segment SGI104761[] = {{&S81401[44], 9}, {NULL}};
const Segment SGI104762[] = {{&S81401[44], 35}, {NULL}};
const Segment SGI104763[] = {{&S81401[30], 6}, {NULL}};
const Segment SGI104764[] = {{&S81401[31], 3}, {NULL}};
const Segment SGI104765[] = {{&S81401[93], 6}, {NULL}};
const Segment SGI104766[] = {{&S97696[44], 7}, {NULL}};
const Segment SGI104767[] = {{&S97696[44], 25}, {NULL}};
const Segment SGI104768[] = {{&S97696[30], 6}, {NULL}};
const Segment SGI104769[] = {{&S105079[34], 10}, {NULL}};
const Segment SGI104770[] = {{&S105079[27], 6}, {NULL}};
const Segment SGI104771[] = {{&S97696[57], 3}, {NULL}};
const Segment SGI104772[] = {{&S97696[102], 5}, {NULL}};
const Segment SGI104773[] = {{&S97696[35], 1}, {NULL}};
const Segment SGI104774[] = {{&S97696[71], 11}, {NULL}};
const Segment SGI104775[] = {{&S105079[109], 2}, {NULL}};
const Segment SGI104776[] = {{&S105079[75], 6}, {NULL}};
const Segment SGI104777[] = {{&S105079[143], 8}, {NULL}};
const Segment SGI104778[] = {{&S81456[44], 14}, {NULL}};
const Segment SGI104779[] = {{&S81456[44], 26}, {NULL}};
const Segment SGI104780[] = {{&S81456[30], 6}, {NULL}};
const Segment SGI104781[] = {{&S81456[31], 3}, {NULL}};
const Segment SGI104782[] = {{&S81456[84], 6}, {NULL}};
const Segment SGI104783[] = {{&S81465[44], 12}, {NULL}};
const Segment SGI104784[] = {{&S81465[44], 24}, {NULL}};
const Segment SGI104785[] = {{&S81465[30], 6}, {NULL}};
const Segment SGI104786[] = {{&S81465[31], 3}, {NULL}};
const Segment SGI104787[] = {{&S81465[82], 6}, {NULL}};
const Segment SGI104788[] = {{&S96015[37], 8}, {NULL}};
const Segment SGI104789[] = {{&S96015[102], 3}, {NULL}};
const Segment SGI104790[] = {{&S96015[91], 3}, {NULL}};
const Segment SGI104791[] = {{&S96015[68], 6}, {NULL}};
const Segment SGI104792[] = {{&S96016[29], 8}, {NULL}};
const Segment SGI104793[] = {{&S96016[53], 3}, {NULL}};
const Segment SGI104794[] = {{&S97859[34], 7}, {NULL}};
const Segment SGI104795[] = {{&S97859[27], 6}, {NULL}};
const Segment SGI104796[] = {{&S97859[106], 2}, {NULL}};
const Segment SGI104797[] = {{&S97859[72], 6}, {NULL}};
const Segment SGI104798[] = {{&S97859[140], 8}, {NULL}};
const Segment SGI104799[] = {{&S81612[34], 13}, {NULL}};
const Segment SGI104800[] = {{&S81612[27], 6}, {NULL}};
const Segment SGI104801[] = {{&S81612[100], 9}, {NULL}};
const Segment SGI104802[] = {{&S81612[73], 6}, {NULL}};
const Segment SGI104803[] = {{&S90506[44], 12}, {NULL}};
const Segment SGI104804[] = {{&S90506[44], 33}, {NULL}};
const Segment SGI104805[] = {{&S90506[30], 6}, {NULL}};
const Segment SGI104806[] = {{&S90506[31], 3}, {NULL}};
const Segment SGI104807[] = {{&S90506[91], 5}, {NULL}};
const Segment SGI104808[] = {{&S90506[0], 13}, {NULL}};
const Segment SGI104809[] = {{&S97903[44], 7}, {NULL}};
const Segment SGI104810[] = {{&S97903[44], 24}, {NULL}};
const Segment SGI104811[] = {{&S97903[30], 6}, {NULL}};
const Segment SGI104812[] = {{&S97903[115], 3}, {NULL}};
const Segment SGI104813[] = {{&S97903[82], 5}, {NULL}};
const Segment SGI104814[] = {{&S97903[119], 14}, {NULL}};
const Segment SGI104815[] = {{&S96280[32], 10}, {NULL}};
const Segment SGI104816[] = {{&S96280[27], 4}, {NULL}};
const Segment SGI104817[] = {{&S96280[101], 2}, {NULL}};
const Segment SGI104818[] = {{&S96280[67], 6}, {NULL}};
const Segment SGI104819[] = {{&S96280[128], 8}, {NULL}};
const Segment SGI104820[] = {{&S90644[71], 6}, {NULL}};
const Segment SGI104821[] = {{&S90644[0], 6}, {NULL}};
const Segment SGI104822[] = {{&S81646[34], 9}, {NULL}};
const Segment SGI104823[] = {{&S81646[27], 6}, {NULL}};
const Segment SGI104824[] = {{&S81646[96], 9}, {NULL}};
const Segment SGI104825[] = {{&S81646[69], 6}, {NULL}};
const Segment SGI104826[] = {{&S81646[137], 8}, {NULL}};
const Segment SGI104827[] = {{&S96344[42], 11}, {NULL}};
const Segment SGI104828[] = {{&S96344[42], 23}, {NULL}};
const Segment SGI104829[] = {{&S96344[30], 4}, {NULL}};
const Segment SGI104830[] = {{&S96344[114], 3}, {NULL}};
const Segment SGI104831[] = {{&S96344[79], 6}, {NULL}};
const Segment SGI104832[] = {{&S105508[44], 18}, {NULL}};
const Segment SGI104833[] = {{&S105508[44], 35}, {NULL}};
const Segment SGI104834[] = {{&S105508[30], 6}, {NULL}};
const Segment SGI104835[] = {{&S105508[134], 3}, {NULL}};
const Segment SGI104836[] = {{&S105508[100], 5}, {NULL}};
const Segment SGI104837[] = {{&S105508[79], 8}, {NULL}};
const Segment SGI104838[] = {{&S98102[44], 10}, {NULL}};
const Segment SGI104839[] = {{&S98102[44], 20}, {NULL}};
const Segment SGI104840[] = {{&S98102[30], 6}, {NULL}};
const Segment SGI104841[] = {{&S98102[112], 3}, {NULL}};
const Segment SGI104842[] = {{&S98102[78], 5}, {NULL}};
const Segment SGI104843[] = {{&S81688[34], 10}, {NULL}};
const Segment SGI104844[] = {{&S81688[27], 6}, {NULL}};
const Segment SGI104845[] = {{&S81688[97], 9}, {NULL}};
const Segment SGI104846[] = {{&S81688[70], 6}, {NULL}};
const Segment SGI104847[] = {{&S98118[44], 22}, {NULL}};
const Segment SGI104848[] = {{&S98118[30], 6}, {NULL}};
const Segment SGI104849[] = {{&S98118[147], 3}, {NULL}};
const Segment SGI104850[] = {{&S98118[113], 5}, {NULL}};
const Segment SGI104851[] = {{&S98118[35], 1}, {NULL}};
const Segment SGI104852[] = {{&S98118[82], 11}, {NULL}};
const Segment SGI104853[] = {{&S105533[44], 7}, {NULL}};
const Segment SGI104854[] = {{&S105533[44], 31}, {NULL}};
const Segment SGI104855[] = {{&S105533[30], 6}, {NULL}};
const Segment SGI104856[] = {{&S105533[130], 3}, {NULL}};
const Segment SGI104857[] = {{&S105533[96], 5}, {NULL}};
const Segment SGI104858[] = {{&S105533[75], 8}, {NULL}};
const Segment SGI104859[] = {{&S81708[34], 10}, {NULL}};
const Segment SGI104860[] = {{&S81708[27], 6}, {NULL}};
const Segment SGI104861[] = {{&S81708[97], 9}, {NULL}};
const Segment SGI104862[] = {{&S81708[70], 6}, {NULL}};
const Segment SGI104863[] = {{&S81708[132], 8}, {NULL}};
const Segment SGI104864[] = {{&S81709[34], 10}, {NULL}};
const Segment SGI104865[] = {{&S81709[27], 6}, {NULL}};
const Segment SGI104866[] = {{&S81709[97], 9}, {NULL}};
const Segment SGI104867[] = {{&S81709[70], 6}, {NULL}};
const Segment SGI104868[] = {{&S81709[131], 8}, {NULL}};
const Segment SGI104869[] = {{&S81726[34], 6}, {NULL}};
const Segment SGI104870[] = {{&S81726[27], 6}, {NULL}};
const Segment SGI104871[] = {{&S81726[93], 9}, {NULL}};
const Segment SGI104872[] = {{&S81726[66], 6}, {NULL}};
const Segment SGI104873[] = {{&S81726[127], 8}, {NULL}};
const Segment SGI104874[] = {{&S81748[34], 6}, {NULL}};
const Segment SGI104875[] = {{&S81748[27], 6}, {NULL}};
const Segment SGI104876[] = {{&S81748[93], 9}, {NULL}};
const Segment SGI104877[] = {{&S81748[66], 6}, {NULL}};
const Segment SGI104878[] = {{&S81806[44], 16}, {NULL}};
const Segment SGI104879[] = {{&S81806[44], 28}, {NULL}};
const Segment SGI104880[] = {{&S81806[30], 6}, {NULL}};
const Segment SGI104881[] = {{&S81806[31], 3}, {NULL}};
const Segment SGI104882[] = {{&S81806[86], 6}, {NULL}};
const Segment SGI104883[] = {{&S98250[59], 7}, {NULL}};
const Segment SGI104884[] = {{&S98250[59], 24}, {NULL}};
const Segment SGI104885[] = {{&S98250[45], 6}, {NULL}};
const Segment SGI104886[] = {{&S98250[131], 3}, {NULL}};
const Segment SGI104887[] = {{&S98250[97], 5}, {NULL}};
const Segment SGI104888[] = {{&S98250[0], 14}, {NULL}};
const Segment SGI104889[] = {{&S98286[44], 12}, {NULL}};
const Segment SGI104890[] = {{&S98286[44], 23}, {NULL}};
const Segment SGI104891[] = {{&S98286[30], 6}, {NULL}};
const Segment SGI104892[] = {{&S98286[114], 3}, {NULL}};
const Segment SGI104893[] = {{&S98286[81], 5}, {NULL}};
const Segment SGI104894[] = {{&S98286[118], 14}, {NULL}};
const Segment SGI104895[] = {{&S81910[34], 6}, {NULL}};
const Segment SGI104896[] = {{&S81910[27], 6}, {NULL}};
const Segment SGI104897[] = {{&S81910[93], 9}, {NULL}};
const Segment SGI104898[] = {{&S81910[66], 6}, {NULL}};
const Segment SGI104899[] = {{&S81911[34], 9}, {NULL}};
const Segment SGI104900[] = {{&S81911[27], 6}, {NULL}};
const Segment SGI104901[] = {{&S81911[96], 9}, {NULL}};
const Segment SGI104902[] = {{&S81911[69], 6}, {NULL}};
const Segment SGI104903[] = {{&S81968[44], 7}, {NULL}};
const Segment SGI104904[] = {{&S81968[44], 19}, {NULL}};
const Segment SGI104905[] = {{&S81968[30], 6}, {NULL}};
const Segment SGI104906[] = {{&S81968[31], 3}, {NULL}};
const Segment SGI104907[] = {{&S81968[77], 6}, {NULL}};
const Segment SGI104908[] = {{&S81976[44], 6}, {NULL}};
const Segment SGI104909[] = {{&S81976[44], 18}, {NULL}};
const Segment SGI104910[] = {{&S81976[30], 6}, {NULL}};
const Segment SGI104911[] = {{&S81976[31], 3}, {NULL}};
const Segment SGI104912[] = {{&S81976[76], 6}, {NULL}};
const Segment SGI104913[] = {{&S81976[146], 3}, {NULL}};
const Segment SGI104914[] = {{&S81977[44], 6}, {NULL}};
const Segment SGI104915[] = {{&S81977[44], 18}, {NULL}};
const Segment SGI104916[] = {{&S81977[30], 6}, {NULL}};
const Segment SGI104917[] = {{&S81977[31], 3}, {NULL}};
const Segment SGI104918[] = {{&S81977[76], 6}, {NULL}};
const Segment SGI104919[] = {{&S81977[139], 3}, {NULL}};
const Segment SGI104920[] = {{&S81982[34], 14}, {NULL}};
const Segment SGI104921[] = {{&S81982[27], 6}, {NULL}};
const Segment SGI104922[] = {{&S81982[101], 9}, {NULL}};
const Segment SGI104923[] = {{&S81982[74], 6}, {NULL}};
const Segment SGI104924[] = {{&S105744[44], 10}, {NULL}};
const Segment SGI104925[] = {{&S105744[44], 27}, {NULL}};
const Segment SGI104926[] = {{&S105744[30], 6}, {NULL}};
const Segment SGI104927[] = {{&S105744[119], 3}, {NULL}};
const Segment SGI104928[] = {{&S105744[85], 5}, {NULL}};
const Segment SGI104929[] = {{&S105744[153], 3}, {NULL}};
const Segment SGI104930[] = {{&S82008[44], 12}, {NULL}};
const Segment SGI104931[] = {{&S82008[44], 33}, {NULL}};
const Segment SGI104932[] = {{&S82008[30], 6}, {NULL}};
const Segment SGI104933[] = {{&S82008[91], 6}, {NULL}};
const Segment SGI104934[] = {{&S82008[118], 13}, {NULL}};
const Segment SGI104935[] = {{&S96687[44], 10}, {NULL}};
const Segment SGI104936[] = {{&S96687[44], 21}, {NULL}};
const Segment SGI104937[] = {{&S96687[30], 6}, {NULL}};
const Segment SGI104938[] = {{&S96687[114], 3}, {NULL}};
const Segment SGI104939[] = {{&S96687[79], 6}, {NULL}};
const Segment SGI104940[] = {{&S96689[43], 10}, {NULL}};
const Segment SGI104941[] = {{&S96689[30], 4}, {NULL}};
const Segment SGI104942[] = {{&S96689[113], 3}, {NULL}};
const Segment SGI104943[] = {{&S96689[79], 5}, {NULL}};
const Segment SGI104944[] = {{&S96689[137], 13}, {NULL}};
const Segment SGI104945[] = {{&S96690[42], 10}, {NULL}};
const Segment SGI104946[] = {{&S96690[42], 22}, {NULL}};
const Segment SGI104947[] = {{&S96690[30], 4}, {NULL}};
const Segment SGI104948[] = {{&S96690[113], 3}, {NULL}};
const Segment SGI104949[] = {{&S96690[78], 6}, {NULL}};
const Segment SGI104950[] = {{&S96690[133], 8}, {NULL}};
const Segment SGI104951[] = {{&S96690[142], 13}, {NULL}};
const Segment SGI104952[] = {{&S82037[44], 10}, {NULL}};
const Segment SGI104953[] = {{&S82037[44], 22}, {NULL}};
const Segment SGI104954[] = {{&S82037[30], 6}, {NULL}};
const Segment SGI104955[] = {{&S82037[111], 2}, {NULL}};
const Segment SGI104956[] = {{&S96692[42], 10}, {NULL}};
const Segment SGI104957[] = {{&S82037[80], 5}, {NULL}};
const Segment SGI104958[] = {{&S96692[42], 22}, {NULL}};
const Segment SGI104959[] = {{&S96692[30], 4}, {NULL}};
const Segment SGI104960[] = {{&S96692[113], 3}, {NULL}};
const Segment SGI104961[] = {{&S96692[78], 6}, {NULL}};
const Segment SGI104962[] = {{&S96692[131], 13}, {NULL}};
const Segment SGI104963[] = {{&S96693[42], 11}, {NULL}};
const Segment SGI104964[] = {{&S96693[42], 23}, {NULL}};
const Segment SGI104965[] = {{&S96693[30], 4}, {NULL}};
const Segment SGI104966[] = {{&S96693[114], 3}, {NULL}};
const Segment SGI104967[] = {{&S96693[79], 6}, {NULL}};
const Segment SGI104968[] = {{&S96693[132], 13}, {NULL}};
const Segment SGI104969[] = {{&S82056[34], 11}, {NULL}};
const Segment SGI104970[] = {{&S82056[27], 6}, {NULL}};
const Segment SGI104971[] = {{&S82056[98], 9}, {NULL}};
const Segment SGI104972[] = {{&S82056[71], 6}, {NULL}};
const Segment SGI104973[] = {{&S82057[34], 13}, {NULL}};
const Segment SGI104974[] = {{&S82057[27], 6}, {NULL}};
const Segment SGI104975[] = {{&S82057[100], 9}, {NULL}};
const Segment SGI104976[] = {{&S82057[73], 6}, {NULL}};
const Segment SGI104977[] = {{&S82073[34], 18}, {NULL}};
const Segment SGI104978[] = {{&S82073[27], 6}, {NULL}};
const Segment SGI104979[] = {{&S82073[105], 9}, {NULL}};
const Segment SGI104980[] = {{&S82073[78], 6}, {NULL}};
const Segment SGI104981[] = {{&S105820[44], 10}, {NULL}};
const Segment SGI104982[] = {{&S105820[44], 27}, {NULL}};
const Segment SGI104983[] = {{&S105820[30], 6}, {NULL}};
const Segment SGI104984[] = {{&S105820[119], 3}, {NULL}};
const Segment SGI104985[] = {{&S105820[85], 5}, {NULL}};
const Segment SGI104986[] = {{&S105820[142], 10}, {NULL}};
const Segment SGI104987[] = {{&S82138[44], 9}, {NULL}};
const Segment SGI104988[] = {{&S82138[44], 21}, {NULL}};
const Segment SGI104989[] = {{&S82138[30], 6}, {NULL}};
const Segment SGI104990[] = {{&S82138[31], 3}, {NULL}};
const Segment SGI104991[] = {{&S82138[79], 6}, {NULL}};
const Segment SGI104992[] = {{&S82154[34], 9}, {NULL}};
const Segment SGI104993[] = {{&S82154[27], 6}, {NULL}};
const Segment SGI104994[] = {{&S82154[96], 9}, {NULL}};
const Segment SGI104995[] = {{&S82154[69], 6}, {NULL}};
const Segment SGI104996[] = {{&S82155[34], 9}, {NULL}};
const Segment SGI104997[] = {{&S82155[27], 6}, {NULL}};
const Segment SGI104998[] = {{&S82155[96], 9}, {NULL}};
const Segment SGI104999[] = {{&S82155[69], 6}, {NULL}};
const Segment SGI105000[] = {{&S91123[44], 11}, {NULL}};
const Segment SGI105001[] = {{&S91123[44], 30}, {NULL}};
const Segment SGI105002[] = {{&S91123[30], 6}, {NULL}};
const Segment SGI105003[] = {{&S91123[31], 3}, {NULL}};
const Segment SGI105004[] = {{&S91123[88], 5}, {NULL}};
const Segment SGI105005[] = {{&S91123[0], 13}, {NULL}};
const Segment SGI105006[] = {{&S91124[44], 11}, {NULL}};
const Segment SGI105007[] = {{&S91124[44], 21}, {NULL}};
const Segment SGI105008[] = {{&S91124[30], 6}, {NULL}};
const Segment SGI105009[] = {{&S91124[31], 3}, {NULL}};
const Segment SGI105010[] = {{&S91124[79], 5}, {NULL}};
const Segment SGI105011[] = {{&S91124[0], 13}, {NULL}};
const Segment SGI105012[] = {{&S82170[34], 6}, {NULL}};
const Segment SGI105013[] = {{&S82170[27], 6}, {NULL}};
const Segment SGI105014[] = {{&S82170[93], 9}, {NULL}};
const Segment SGI105015[] = {{&S91125[59], 11}, {NULL}};
const Segment SGI105016[] = {{&S82170[66], 6}, {NULL}};
const Segment SGI105017[] = {{&S91125[59], 30}, {NULL}};
const Segment SGI105018[] = {{&S91125[30], 21}, {NULL}};
const Segment SGI105019[] = {{&S91125[31], 3}, {NULL}};
const Segment SGI105020[] = {{&S91125[103], 5}, {NULL}};
const Segment SGI105021[] = {{&S91125[0], 13}, {NULL}};
const Segment SGI105022[] = {{&S91143[44], 10}, {NULL}};
const Segment SGI105023[] = {{&S91143[44], 21}, {NULL}};
const Segment SGI105024[] = {{&S91143[30], 6}, {NULL}};
const Segment SGI105025[] = {{&S91143[31], 3}, {NULL}};
const Segment SGI105026[] = {{&S91143[103], 5}, {NULL}};
const Segment SGI105027[] = {{&S91143[0], 13}, {NULL}};
const Segment SGI105028[] = {{&S82201[44], 13}, {NULL}};
const Segment SGI105029[] = {{&S82201[44], 25}, {NULL}};
const Segment SGI105030[] = {{&S82201[30], 6}, {NULL}};
const Segment SGI105031[] = {{&S82201[31], 3}, {NULL}};
const Segment SGI105032[] = {{&S82201[83], 6}, {NULL}};
const Segment SGI105033[] = {{&S82203[38], 13}, {NULL}};
const Segment SGI105034[] = {{&S82203[31], 6}, {NULL}};
const Segment SGI105035[] = {{&S82228[44], 13}, {NULL}};
const Segment SGI105036[] = {{&S82228[44], 25}, {NULL}};
const Segment SGI105037[] = {{&S82228[30], 6}, {NULL}};
const Segment SGI105038[] = {{&S82228[31], 3}, {NULL}};
const Segment SGI105039[] = {{&S82228[83], 5}, {NULL}};
const Segment SGI105040[] = {{&S98594[44], 12}, {NULL}};
const Segment SGI105041[] = {{&S98594[44], 23}, {NULL}};
const Segment SGI105042[] = {{&S98594[30], 6}, {NULL}};
const Segment SGI105043[] = {{&S98594[134], 3}, {NULL}};
const Segment SGI105044[] = {{&S98594[100], 5}, {NULL}};
const Segment SGI105045[] = {{&S98594[35], 1}, {NULL}};
const Segment SGI105046[] = {{&S98594[69], 11}, {NULL}};
const Segment SGI105047[] = {{&S98630[44], 7}, {NULL}};
const Segment SGI105048[] = {{&S98630[44], 25}, {NULL}};
const Segment SGI105049[] = {{&S98630[30], 6}, {NULL}};
const Segment SGI105050[] = {{&S98630[57], 3}, {NULL}};
const Segment SGI105051[] = {{&S98630[102], 5}, {NULL}};
const Segment SGI105052[] = {{&S98630[35], 1}, {NULL}};
const Segment SGI105053[] = {{&S98630[71], 11}, {NULL}};
const Segment SGI105054[] = {{&S96946[42], 5}, {NULL}};
const Segment SGI105055[] = {{&S96946[42], 17}, {NULL}};
const Segment SGI105056[] = {{&S96946[30], 4}, {NULL}};
const Segment SGI105057[] = {{&S96946[108], 3}, {NULL}};
const Segment SGI105058[] = {{&S96946[73], 6}, {NULL}};
const Segment SGI105059[] = {{&S96947[42], 12}, {NULL}};
const Segment SGI105060[] = {{&S96947[42], 24}, {NULL}};
const Segment SGI105061[] = {{&S96947[30], 4}, {NULL}};
const Segment SGI105062[] = {{&S96947[115], 3}, {NULL}};
const Segment SGI105063[] = {{&S96947[80], 6}, {NULL}};
const Segment SGI105064[] = {{&S105983[34], 7}, {NULL}};
const Segment SGI105065[] = {{&S105983[27], 6}, {NULL}};
const Segment SGI105066[] = {{&S105983[106], 2}, {NULL}};
const Segment SGI105067[] = {{&S105983[72], 6}, {NULL}};
const Segment SGI105068[] = {{&S105983[140], 8}, {NULL}};
const Segment SGI105069[] = {{&S82316[44], 11}, {NULL}};
const Segment SGI105070[] = {{&S82316[44], 23}, {NULL}};
const Segment SGI105071[] = {{&S82316[30], 6}, {NULL}};
const Segment SGI105072[] = {{&S82316[31], 3}, {NULL}};
const Segment SGI105073[] = {{&S82316[81], 6}, {NULL}};
const Segment SGI105074[] = {{&S82316[141], 11}, {NULL}};
const Segment SGI105075[] = {{&S82332[34], 9}, {NULL}};
const Segment SGI105076[] = {{&S82332[27], 6}, {NULL}};
const Segment SGI105077[] = {{&S82332[96], 9}, {NULL}};
const Segment SGI105078[] = {{&S82332[69], 6}, {NULL}};
const Segment SGI105079[] = {{&S94910[42], 10}, {NULL}};
const Segment SGI105080[] = {{&S94910[42], 20}, {NULL}};
const Segment SGI105081[] = {{&S94910[30], 4}, {NULL}};
const Segment SGI105082[] = {{&S94910[31], 3}, {NULL}};
const Segment SGI105083[] = {{&S94910[76], 5}, {NULL}};
const Segment SGI105084[] = {{&S82391[34], 17}, {NULL}};
const Segment SGI105085[] = {{&S82391[27], 6}, {NULL}};
const Segment SGI105086[] = {{&S82391[104], 9}, {NULL}};
const Segment SGI105087[] = {{&S82391[77], 6}, {NULL}};
const Segment SGI105088[] = {{&S82399[34], 15}, {NULL}};
const Segment SGI105089[] = {{&S82399[27], 6}, {NULL}};
const Segment SGI105090[] = {{&S82399[102], 9}, {NULL}};
const Segment SGI105091[] = {{&S82399[75], 6}, {NULL}};
const Segment SGI105092[] = {{&S82403[44], 15}, {NULL}};
const Segment SGI105093[] = {{&S82403[44], 27}, {NULL}};
const Segment SGI105094[] = {{&S82403[30], 6}, {NULL}};
const Segment SGI105095[] = {{&S82403[31], 3}, {NULL}};
const Segment SGI105096[] = {{&S82403[85], 6}, {NULL}};
const Segment SGI105097[] = {{&S82461[31], 8}, {NULL}};
const Segment SGI105098[] = {{&S82461[54], 3}, {NULL}};
const Segment SGI105099[] = {{&S82462[31], 9}, {NULL}};
const Segment SGI105100[] = {{&S82462[55], 3}, {NULL}};
const Segment SGI105101[] = {{&S82467[57], 13}, {NULL}};
const Segment SGI105102[] = {{&S82467[57], 29}, {NULL}};
const Segment SGI105103[] = {{&S82467[43], 6}, {NULL}};
const Segment SGI105104[] = {{&S82467[44], 3}, {NULL}};
const Segment SGI105105[] = {{&S82467[100], 6}, {NULL}};
const Segment SGI105106[] = {{&S82467[0], 12}, {NULL}};
const Segment SGI105107[] = {{&S82468[44], 4}, {NULL}};
const Segment SGI105108[] = {{&S82468[44], 26}, {NULL}};
const Segment SGI105109[] = {{&S82468[30], 6}, {NULL}};
const Segment SGI105110[] = {{&S82468[31], 3}, {NULL}};
const Segment SGI105111[] = {{&S82468[84], 6}, {NULL}};
const Segment SGI105112[] = {{&S82474[44], 14}, {NULL}};
const Segment SGI105113[] = {{&S82474[44], 25}, {NULL}};
const Segment SGI105114[] = {{&S82474[30], 6}, {NULL}};
const Segment SGI105115[] = {{&S82474[31], 3}, {NULL}};
const Segment SGI105116[] = {{&S82474[83], 5}, {NULL}};
const Segment SGI105117[] = {{&S97124[34], 7}, {NULL}};
const Segment SGI105118[] = {{&S97124[27], 6}, {NULL}};
const Segment SGI105119[] = {{&S97124[101], 2}, {NULL}};
const Segment SGI105120[] = {{&S97124[67], 6}, {NULL}};
const Segment SGI105121[] = {{&S97124[128], 8}, {NULL}};
const Segment SGI105122[] = {{&S97131[44], 11}, {NULL}};
const Segment SGI105123[] = {{&S97131[44], 23}, {NULL}};
const Segment SGI105124[] = {{&S97131[30], 6}, {NULL}};
const Segment SGI105125[] = {{&S97131[116], 3}, {NULL}};
const Segment SGI105126[] = {{&S97131[81], 6}, {NULL}};
const Segment SGI105127[] = {{&S82493[34], 14}, {NULL}};
const Segment SGI105128[] = {{&S82493[27], 6}, {NULL}};
const Segment SGI105129[] = {{&S82493[101], 9}, {NULL}};
const Segment SGI105130[] = {{&S82493[74], 6}, {NULL}};
const Segment SGI105131[] = {{&S82541[44], 7}, {NULL}};
const Segment SGI105132[] = {{&S82541[44], 21}, {NULL}};
const Segment SGI105133[] = {{&S82541[30], 6}, {NULL}};
const Segment SGI105134[] = {{&S82541[31], 3}, {NULL}};
const Segment SGI105135[] = {{&S82541[79], 6}, {NULL}};
const Segment SGI105136[] = {{&S82543[44], 13}, {NULL}};
const Segment SGI105137[] = {{&S82543[44], 25}, {NULL}};
const Segment SGI105138[] = {{&S82543[30], 6}, {NULL}};
const Segment SGI105139[] = {{&S82543[31], 3}, {NULL}};
const Segment SGI105140[] = {{&S82543[83], 6}, {NULL}};
const Segment SGI105141[] = {{&S82544[44], 13}, {NULL}};
const Segment SGI105142[] = {{&S82544[44], 25}, {NULL}};
const Segment SGI105143[] = {{&S82544[30], 6}, {NULL}};
const Segment SGI105144[] = {{&S82544[86], 6}, {NULL}};
const Segment SGI105145[] = {{&S82546[44], 9}, {NULL}};
const Segment SGI105146[] = {{&S82546[44], 19}, {NULL}};
const Segment SGI105147[] = {{&S82546[30], 6}, {NULL}};
const Segment SGI105148[] = {{&S82546[31], 3}, {NULL}};
const Segment SGI105149[] = {{&S82546[77], 6}, {NULL}};
const Segment SGI105150[] = {{&S82547[44], 10}, {NULL}};
const Segment SGI105151[] = {{&S82547[44], 20}, {NULL}};
const Segment SGI105152[] = {{&S82547[30], 6}, {NULL}};
const Segment SGI105153[] = {{&S82547[31], 3}, {NULL}};
const Segment SGI105154[] = {{&S82547[78], 6}, {NULL}};
const Segment SGI105155[] = {{&S91510[44], 9}, {NULL}};
const Segment SGI105156[] = {{&S91510[44], 19}, {NULL}};
const Segment SGI105157[] = {{&S91510[30], 6}, {NULL}};
const Segment SGI105158[] = {{&S91510[31], 3}, {NULL}};
const Segment SGI105159[] = {{&S91510[77], 5}, {NULL}};
const Segment SGI105160[] = {{&S91510[0], 13}, {NULL}};
const Segment SGI105161[] = {{&S82577[34], 6}, {NULL}};
const Segment SGI105162[] = {{&S82577[27], 6}, {NULL}};
const Segment SGI105163[] = {{&S82577[93], 9}, {NULL}};
const Segment SGI105164[] = {{&S82577[66], 6}, {NULL}};
const Segment SGI105165[] = {{&S98894[44], 10}, {NULL}};
const Segment SGI105166[] = {{&S98894[44], 27}, {NULL}};
const Segment SGI105167[] = {{&S98894[30], 6}, {NULL}};
const Segment SGI105168[] = {{&S98894[138], 3}, {NULL}};
const Segment SGI105169[] = {{&S98894[104], 5}, {NULL}};
const Segment SGI105170[] = {{&S98894[35], 1}, {NULL}};
const Segment SGI105171[] = {{&S98894[73], 11}, {NULL}};
const Segment SGI105172[] = {{&S82613[34], 8}, {NULL}};
const Segment SGI105173[] = {{&S82613[27], 6}, {NULL}};
const Segment SGI105174[] = {{&S82613[95], 9}, {NULL}};
const Segment SGI105175[] = {{&S82613[68], 6}, {NULL}};
const Segment SGI105176[] = {{&S82638[34], 6}, {NULL}};
const Segment SGI105177[] = {{&S82638[27], 6}, {NULL}};
const Segment SGI105178[] = {{&S82638[93], 9}, {NULL}};
const Segment SGI105179[] = {{&S82638[66], 6}, {NULL}};
const Segment SGI105180[] = {{&S82648[44], 14}, {NULL}};
const Segment SGI105181[] = {{&S82648[44], 31}, {NULL}};
const Segment SGI105182[] = {{&S82648[30], 6}, {NULL}};
const Segment SGI105183[] = {{&S82648[89], 6}, {NULL}};
const Segment SGI105184[] = {{&S82648[116], 13}, {NULL}};
const Segment SGI105185[] = {{&S82685[44], 17}, {NULL}};
const Segment SGI105186[] = {{&S82685[44], 29}, {NULL}};
const Segment SGI105187[] = {{&S82685[30], 6}, {NULL}};
const Segment SGI105188[] = {{&S82685[31], 3}, {NULL}};
const Segment SGI105189[] = {{&S82685[87], 6}, {NULL}};
const Segment SGI105190[] = {{&S82690[34], 14}, {NULL}};
const Segment SGI105191[] = {{&S82690[27], 6}, {NULL}};
const Segment SGI105192[] = {{&S82690[101], 9}, {NULL}};
const Segment SGI105193[] = {{&S82690[74], 6}, {NULL}};
const Segment SGI105194[] = {{&S82702[44], 5}, {NULL}};
const Segment SGI105195[] = {{&S82702[44], 17}, {NULL}};
const Segment SGI105196[] = {{&S82702[30], 6}, {NULL}};
const Segment SGI105197[] = {{&S82702[31], 3}, {NULL}};
const Segment SGI105198[] = {{&S82702[75], 5}, {NULL}};
const Segment SGI105199[] = {{&S82711[34], 13}, {NULL}};
const Segment SGI105200[] = {{&S82711[27], 6}, {NULL}};
const Segment SGI105201[] = {{&S82711[100], 9}, {NULL}};
const Segment SGI105202[] = {{&S82711[73], 6}, {NULL}};
const Segment SGI105203[] = {{&S82711[141], 8}, {NULL}};
const Segment SGI105204[] = {{&S82729[44], 41}, {NULL}};
const Segment SGI105205[] = {{&S82729[44], 53}, {NULL}};
const Segment SGI105206[] = {{&S82729[30], 6}, {NULL}};
const Segment SGI105207[] = {{&S82729[111], 6}, {NULL}};
const Segment SGI105208[] = {{&S82729[138], 13}, {NULL}};
const Segment SGI105209[] = {{&S82749[44], 9}, {NULL}};
const Segment SGI105210[] = {{&S82749[44], 19}, {NULL}};
const Segment SGI105211[] = {{&S82749[30], 6}, {NULL}};
const Segment SGI105212[] = {{&S82749[31], 3}, {NULL}};
const Segment SGI105213[] = {{&S82749[77], 6}, {NULL}};
const Segment SGI105214[] = {{&S82765[34], 10}, {NULL}};
const Segment SGI105215[] = {{&S82765[27], 6}, {NULL}};
const Segment SGI105216[] = {{&S82765[97], 9}, {NULL}};
const Segment SGI105217[] = {{&S82765[70], 6}, {NULL}};
const Segment SGI105218[] = {{&S99073[44], 18}, {NULL}};
const Segment SGI105219[] = {{&S99073[44], 29}, {NULL}};
const Segment SGI105220[] = {{&S99073[30], 6}, {NULL}};
const Segment SGI105221[] = {{&S99073[140], 3}, {NULL}};
const Segment SGI105222[] = {{&S99073[106], 5}, {NULL}};
const Segment SGI105223[] = {{&S99073[35], 1}, {NULL}};
const Segment SGI105224[] = {{&S99073[75], 11}, {NULL}};
const Segment SGI105225[] = {{&S82783[34], 10}, {NULL}};
const Segment SGI105226[] = {{&S82783[27], 6}, {NULL}};
const Segment SGI105227[] = {{&S82783[97], 9}, {NULL}};
const Segment SGI105228[] = {{&S82783[70], 6}, {NULL}};
const Segment SGI105229[] = {{&S82784[34], 10}, {NULL}};
const Segment SGI105230[] = {{&S82784[27], 6}, {NULL}};
const Segment SGI105231[] = {{&S82784[97], 9}, {NULL}};
const Segment SGI105232[] = {{&S82784[70], 6}, {NULL}};
const Segment SGI105233[] = {{&S82785[34], 10}, {NULL}};
const Segment SGI105234[] = {{&S82785[27], 6}, {NULL}};
const Segment SGI105235[] = {{&S82785[97], 9}, {NULL}};
const Segment SGI105236[] = {{&S82785[70], 6}, {NULL}};
const Segment SGI105237[] = {{&S82788[34], 10}, {NULL}};
const Segment SGI105238[] = {{&S82788[27], 6}, {NULL}};
const Segment SGI105239[] = {{&S82788[97], 9}, {NULL}};
const Segment SGI105240[] = {{&S82788[70], 6}, {NULL}};
const Segment SGI105241[] = {{&S82789[34], 10}, {NULL}};
const Segment SGI105242[] = {{&S82789[27], 6}, {NULL}};
const Segment SGI105243[] = {{&S82789[97], 9}, {NULL}};
const Segment SGI105244[] = {{&S82789[70], 6}, {NULL}};
const Segment SGI105245[] = {{&S82790[34], 10}, {NULL}};
const Segment SGI105246[] = {{&S82790[27], 6}, {NULL}};
const Segment SGI105247[] = {{&S82790[94], 9}, {NULL}};
const Segment SGI105248[] = {{&S82790[67], 6}, {NULL}};
const Segment SGI105249[] = {{&S82791[34], 10}, {NULL}};
const Segment SGI105250[] = {{&S82791[27], 6}, {NULL}};
const Segment SGI105251[] = {{&S82791[97], 9}, {NULL}};
const Segment SGI105252[] = {{&S82791[70], 6}, {NULL}};
const Segment SGI105253[] = {{&S82797[34], 14}, {NULL}};
const Segment SGI105254[] = {{&S82797[27], 6}, {NULL}};
const Segment SGI105255[] = {{&S82797[101], 9}, {NULL}};
const Segment SGI105256[] = {{&S82797[74], 6}, {NULL}};
const Segment SGI105257[] = {{&S82797[142], 8}, {NULL}};
const Segment SGI105258[] = {{&S82798[34], 14}, {NULL}};
const Segment SGI105259[] = {{&S82798[27], 6}, {NULL}};
const Segment SGI105260[] = {{&S82798[101], 9}, {NULL}};
const Segment SGI105261[] = {{&S82798[74], 6}, {NULL}};
const Segment SGI105262[] = {{&S82798[142], 8}, {NULL}};
const Segment SGI105263[] = {{&S82802[34], 18}, {NULL}};
const Segment SGI105264[] = {{&S82802[27], 6}, {NULL}};
const Segment SGI105265[] = {{&S82802[105], 9}, {NULL}};
const Segment SGI105266[] = {{&S82802[78], 6}, {NULL}};
const Segment SGI105267[] = {{&S82802[139], 8}, {NULL}};
const Segment SGI105268[] = {{&S82805[34], 17}, {NULL}};
const Segment SGI105269[] = {{&S82805[27], 6}, {NULL}};
const Segment SGI105270[] = {{&S82805[104], 9}, {NULL}};
const Segment SGI105271[] = {{&S82805[77], 6}, {NULL}};
const Segment SGI105272[] = {{&S82843[34], 6}, {NULL}};
const Segment SGI105273[] = {{&S82843[27], 6}, {NULL}};
const Segment SGI105274[] = {{&S82843[93], 9}, {NULL}};
const Segment SGI105275[] = {{&S82843[66], 6}, {NULL}};
const Segment SGI105276[] = {{&S99188[59], 7}, {NULL}};
const Segment SGI105277[] = {{&S99188[59], 24}, {NULL}};
const Segment SGI105278[] = {{&S99188[45], 6}, {NULL}};
const Segment SGI105279[] = {{&S99188[131], 3}, {NULL}};
const Segment SGI105280[] = {{&S99188[97], 5}, {NULL}};
const Segment SGI105281[] = {{&S99188[0], 14}, {NULL}};
const Segment SGI105282[] = {{&S99203[44], 7}, {NULL}};
const Segment SGI105283[] = {{&S99203[44], 17}, {NULL}};
const Segment SGI105284[] = {{&S99203[30], 6}, {NULL}};
const Segment SGI105285[] = {{&S99203[109], 3}, {NULL}};
const Segment SGI105286[] = {{&S99203[75], 5}, {NULL}};
const Segment SGI105287[] = {{&S91808[59], 12}, {NULL}};
const Segment SGI105288[] = {{&S91808[59], 33}, {NULL}};
const Segment SGI105289[] = {{&S91808[45], 6}, {NULL}};
const Segment SGI105290[] = {{&S91808[46], 3}, {NULL}};
const Segment SGI105291[] = {{&S91808[106], 5}, {NULL}};
const Segment SGI105292[] = {{&S91808[0], 28}, {NULL}};
const Segment SGI105293[] = {{&S97368[42], 9}, {NULL}};
const Segment SGI105294[] = {{&S97368[42], 21}, {NULL}};
const Segment SGI105295[] = {{&S97368[30], 4}, {NULL}};
const Segment SGI105296[] = {{&S97368[112], 3}, {NULL}};
const Segment SGI105297[] = {{&S97368[77], 6}, {NULL}};
const Segment SGI105298[] = {{&S82865[44], 21}, {NULL}};
const Segment SGI105299[] = {{&S82865[44], 33}, {NULL}};
const Segment SGI105300[] = {{&S82865[30], 6}, {NULL}};
const Segment SGI105301[] = {{&S82865[91], 6}, {NULL}};
const Segment SGI105302[] = {{&S82865[118], 13}, {NULL}};
const Segment SGI105303[] = {{&S82866[44], 18}, {NULL}};
const Segment SGI105304[] = {{&S82866[44], 30}, {NULL}};
const Segment SGI105305[] = {{&S82866[30], 6}, {NULL}};
const Segment SGI105306[] = {{&S82866[88], 6}, {NULL}};
const Segment SGI105307[] = {{&S82866[115], 13}, {NULL}};
const Segment SGI105308[] = {{&S82868[44], 9}, {NULL}};
const Segment SGI105309[] = {{&S82868[44], 21}, {NULL}};
const Segment SGI105310[] = {{&S82868[30], 6}, {NULL}};
const Segment SGI105311[] = {{&S82868[79], 6}, {NULL}};
const Segment SGI105312[] = {{&S82868[106], 13}, {NULL}};
const Segment SGI105313[] = {{&S82903[44], 4}, {NULL}};
const Segment SGI105314[] = {{&S82903[44], 16}, {NULL}};
const Segment SGI105315[] = {{&S82903[30], 6}, {NULL}};
const Segment SGI105316[] = {{&S82903[77], 6}, {NULL}};
const Segment SGI105317[] = {{&S82904[34], 6}, {NULL}};
const Segment SGI105318[] = {{&S82904[27], 6}, {NULL}};
const Segment SGI105319[] = {{&S82904[97], 9}, {NULL}};
const Segment SGI105320[] = {{&S82904[70], 6}, {NULL}};
const Segment SGI105321[] = {{&S82908[45], 14}, {NULL}};
const Segment SGI105322[] = {{&S82908[30], 6}, {NULL}};
const Segment SGI105323[] = {{&S82908[31], 3}, {NULL}};
const Segment SGI105324[] = {{&S82908[85], 5}, {NULL}};
const Segment SGI105325[] = {{&S82908[143], 13}, {NULL}};
const Segment SGI105326[] = {{&S82925[34], 13}, {NULL}};
const Segment SGI105327[] = {{&S82925[27], 6}, {NULL}};
const Segment SGI105328[] = {{&S82925[100], 9}, {NULL}};
const Segment SGI105329[] = {{&S82925[73], 6}, {NULL}};
const Segment SGI105330[] = {{&S99359[44], 14}, {NULL}};
const Segment SGI105331[] = {{&S99359[44], 30}, {NULL}};
const Segment SGI105332[] = {{&S99359[30], 6}, {NULL}};
const Segment SGI105333[] = {{&S99359[88], 6}, {NULL}};
const Segment SGI105334[] = {{&S99359[115], 13}, {NULL}};
const Segment SGI105335[] = {{&S82941[52], 10}, {NULL}};
const Segment SGI105336[] = {{&S82941[52], 22}, {NULL}};
const Segment SGI105337[] = {{&S82941[38], 6}, {NULL}};
const Segment SGI105338[] = {{&S82941[88], 5}, {NULL}};
const Segment SGI105339[] = {{&S82941[0], 14}, {NULL}};
const Segment SGI105340[] = {{&S99387[44], 6}, {NULL}};
const Segment SGI105341[] = {{&S99387[44], 17}, {NULL}};
const Segment SGI105342[] = {{&S99387[30], 6}, {NULL}};
const Segment SGI105343[] = {{&S99387[128], 3}, {NULL}};
const Segment SGI105344[] = {{&S99387[94], 5}, {NULL}};
const Segment SGI105345[] = {{&S99387[35], 1}, {NULL}};
const Segment SGI105346[] = {{&S99387[63], 11}, {NULL}};
const Segment SGI105347[] = {{&S83053[34], 13}, {NULL}};
const Segment SGI105348[] = {{&S83053[27], 6}, {NULL}};
const Segment SGI105349[] = {{&S83053[104], 9}, {NULL}};
const Segment SGI105350[] = {{&S83053[77], 6}, {NULL}};
const Segment SGI105351[] = {{&S83063[59], 10}, {NULL}};
const Segment SGI105352[] = {{&S83063[59], 22}, {NULL}};
const Segment SGI105353[] = {{&S83063[45], 6}, {NULL}};
const Segment SGI105354[] = {{&S83063[46], 3}, {NULL}};
const Segment SGI105355[] = {{&S83063[95], 6}, {NULL}};
const Segment SGI105356[] = {{&S83063[0], 14}, {NULL}};
const Segment SGI105357[] = {{&S97668[42], 14}, {NULL}};
const Segment SGI105358[] = {{&S97668[42], 25}, {NULL}};
const Segment SGI105359[] = {{&S97668[30], 4}, {NULL}};
const Segment SGI105360[] = {{&S97668[116], 3}, {NULL}};
const Segment SGI105361[] = {{&S97668[81], 6}, {NULL}};
const Segment SGI105362[] = {{&S83070[34], 6}, {NULL}};
const Segment SGI105363[] = {{&S83070[27], 6}, {NULL}};
const Segment SGI105364[] = {{&S83070[93], 9}, {NULL}};
const Segment SGI105365[] = {{&S83070[66], 6}, {NULL}};
const Segment SGI105366[] = {{&S99618[44], 8}, {NULL}};
const Segment SGI105367[] = {{&S99618[44], 29}, {&S99618[74], 18}, {NULL}};
const Segment SGI105368[] = {{&S99618[30], 6}, {NULL}};
const Segment SGI105369[] = {{&S99618[140], 3}, {NULL}};
const Segment SGI105370[] = {{&S99618[106], 5}, {NULL}};
const Segment SGI105371[] = {{&S99618[35], 1}, {NULL}};
const Segment SGI105372[] = {{&S99618[75], 11}, {NULL}};
const Segment SGI105373[] = {{&S97682[42], 9}, {NULL}};
const Segment SGI105374[] = {{&S97682[42], 21}, {NULL}};
const Segment SGI105375[] = {{&S97682[30], 4}, {NULL}};
const Segment SGI105376[] = {{&S97682[112], 3}, {NULL}};
const Segment SGI105377[] = {{&S97682[77], 6}, {NULL}};
const Segment SGI105378[] = {{&S97688[44], 8}, {NULL}};
const Segment SGI105379[] = {{&S97688[44], 20}, {NULL}};
const Segment SGI105380[] = {{&S97688[30], 6}, {NULL}};
const Segment SGI105381[] = {{&S97688[113], 3}, {NULL}};
const Segment SGI105382[] = {{&S97688[78], 6}, {NULL}};
const Segment SGI105383[] = {{&S99632[44], 8}, {NULL}};
const Segment SGI105384[] = {{&S99632[44], 18}, {NULL}};
const Segment SGI105385[] = {{&S99632[30], 6}, {NULL}};
const Segment SGI105386[] = {{&S97689[42], 8}, {NULL}};
const Segment SGI105387[] = {{&S97689[42], 20}, {NULL}};
const Segment SGI105388[] = {{&S97689[30], 4}, {NULL}};
const Segment SGI105389[] = {{&S99632[110], 3}, {NULL}};
const Segment SGI105390[] = {{&S97689[111], 3}, {NULL}};
const Segment SGI105391[] = {{&S99632[76], 5}, {NULL}};
const Segment SGI105392[] = {{&S97689[76], 6}, {NULL}};
const Segment SGI105393[] = {{&S83107[44], 14}, {NULL}};
const Segment SGI105394[] = {{&S83107[44], 26}, {NULL}};
const Segment SGI105395[] = {{&S83107[30], 6}, {NULL}};
const Segment SGI105396[] = {{&S83107[84], 6}, {NULL}};
const Segment SGI105397[] = {{&S83107[111], 13}, {NULL}};
const Segment SGI105398[] = {{&S83108[44], 15}, {NULL}};
const Segment SGI105399[] = {{&S83108[44], 27}, {NULL}};
const Segment SGI105400[] = {{&S83108[30], 6}, {NULL}};
const Segment SGI105401[] = {{&S83108[85], 6}, {NULL}};
const Segment SGI105402[] = {{&S83108[112], 13}, {NULL}};
const Segment SGI105403[] = {{&S83112[34], 11}, {NULL}};
const Segment SGI105404[] = {{&S83112[27], 6}, {NULL}};
const Segment SGI105405[] = {{&S83112[98], 9}, {NULL}};
const Segment SGI105406[] = {{&S83112[71], 6}, {NULL}};
const Segment SGI105407[] = {{&S99683[44], 10}, {NULL}};
const Segment SGI105408[] = {{&S99683[44], 27}, {NULL}};
const Segment SGI105409[] = {{&S99683[30], 6}, {NULL}};
const Segment SGI105410[] = {{&S99683[138], 3}, {NULL}};
const Segment SGI105411[] = {{&S99683[104], 5}, {NULL}};
const Segment SGI105412[] = {{&S99683[35], 1}, {NULL}};
const Segment SGI105413[] = {{&S99683[73], 11}, {NULL}};
const Segment SGI105414[] = {{&S83147[44], 9}, {NULL}};
const Segment SGI105415[] = {{&S83147[44], 21}, {NULL}};
const Segment SGI105416[] = {{&S83147[30], 6}, {NULL}};
const Segment SGI105417[] = {{&S83147[31], 3}, {NULL}};
const Segment SGI105418[] = {{&S83147[79], 6}, {NULL}};
const Segment SGI105419[] = {{&S83216[44], 17}, {NULL}};
const Segment SGI105420[] = {{&S83216[44], 42}, {NULL}};
const Segment SGI105421[] = {{&S83216[30], 6}, {NULL}};
const Segment SGI105422[] = {{&S83216[31], 3}, {NULL}};
const Segment SGI105423[] = {{&S83216[100], 6}, {NULL}};
const Segment SGI105424[] = {{&S83234[34], 12}, {NULL}};
const Segment SGI105425[] = {{&S83234[27], 6}, {NULL}};
const Segment SGI105426[] = {{&S83234[99], 9}, {NULL}};
const Segment SGI105427[] = {{&S83234[72], 6}, {NULL}};
const Segment SGI105428[] = {{&S83234[140], 8}, {NULL}};
const Segment SGI105429[] = {{&S83237[44], 11}, {NULL}};
const Segment SGI105430[] = {{&S83237[44], 23}, {NULL}};
const Segment SGI105431[] = {{&S83237[30], 6}, {NULL}};
const Segment SGI105432[] = {{&S83237[81], 6}, {NULL}};
const Segment SGI105433[] = {{&S83237[108], 13}, {NULL}};
const Segment SGI105434[] = {{&S83241[44], 10}, {NULL}};
const Segment SGI105435[] = {{&S83241[44], 22}, {NULL}};
const Segment SGI105436[] = {{&S83241[30], 6}, {NULL}};
const Segment SGI105437[] = {{&S83241[31], 3}, {NULL}};
const Segment SGI105438[] = {{&S83241[80], 5}, {NULL}};
const Segment SGI105439[] = {{&S83241[117], 14}, {NULL}};
const Segment SGI105440[] = {{&S83244[34], 13}, {NULL}};
const Segment SGI105441[] = {{&S83244[27], 6}, {NULL}};
const Segment SGI105442[] = {{&S83244[100], 9}, {NULL}};
const Segment SGI105443[] = {{&S83244[73], 6}, {NULL}};
const Segment SGI105444[] = {{&S83244[141], 8}, {NULL}};
const Segment SGI105445[] = {{&S99776[59], 7}, {NULL}};
const Segment SGI105446[] = {{&S99776[59], 24}, {NULL}};
const Segment SGI105447[] = {{&S99776[45], 6}, {NULL}};
const Segment SGI105448[] = {{&S99776[131], 3}, {NULL}};
const Segment SGI105449[] = {{&S99776[97], 5}, {NULL}};
const Segment SGI105450[] = {{&S99776[0], 14}, {NULL}};
const Segment SGI105451[] = {{&S99779[59], 6}, {NULL}};
const Segment SGI105452[] = {{&S99779[59], 17}, {NULL}};
const Segment SGI105453[] = {{&S99779[45], 6}, {NULL}};
const Segment SGI105454[] = {{&S99779[124], 3}, {NULL}};
const Segment SGI105455[] = {{&S99779[90], 5}, {NULL}};
const Segment SGI105456[] = {{&S99779[0], 14}, {NULL}};
const Segment SGI105457[] = {{&S83312[44], 14}, {NULL}};
const Segment SGI105458[] = {{&S83312[44], 26}, {NULL}};
const Segment SGI105459[] = {{&S83312[30], 6}, {NULL}};
const Segment SGI105460[] = {{&S83312[31], 3}, {NULL}};
const Segment SGI105461[] = {{&S83312[84], 6}, {NULL}};
const Segment SGI105462[] = {{&S97919[42], 10}, {NULL}};
const Segment SGI105463[] = {{&S97919[42], 22}, {NULL}};
const Segment SGI105464[] = {{&S97919[30], 4}, {NULL}};
const Segment SGI105465[] = {{&S97919[78], 6}, {NULL}};
const Segment SGI105466[] = {{&S97919[105], 13}, {NULL}};
const Segment SGI105467[] = {{&S99791[45], 10}, {NULL}};
const Segment SGI105468[] = {{&S99791[30], 6}, {NULL}};
const Segment SGI105469[] = {{&S99791[118], 3}, {NULL}};
const Segment SGI105470[] = {{&S99791[84], 5}, {NULL}};
const Segment SGI105471[] = {{&S99791[142], 13}, {NULL}};
const Segment SGI105472[] = {{&S99793[45], 14}, {NULL}};
const Segment SGI105473[] = {{&S99793[30], 6}, {NULL}};
const Segment SGI105474[] = {{&S99793[124], 3}, {NULL}};
const Segment SGI105475[] = {{&S99793[90], 5}, {NULL}};
const Segment SGI105476[] = {{&S99793[148], 13}, {NULL}};
const Segment SGI105477[] = {{&S83318[45], 14}, {NULL}};
const Segment SGI105478[] = {{&S83318[30], 6}, {NULL}};
const Segment SGI105479[] = {{&S83318[31], 3}, {NULL}};
const Segment SGI105480[] = {{&S83318[85], 5}, {NULL}};
const Segment SGI105481[] = {{&S83318[143], 13}, {NULL}};
const Segment SGI105482[] = {{&S99879[34], 7}, {NULL}};
const Segment SGI105483[] = {{&S99879[27], 6}, {NULL}};
const Segment SGI105484[] = {{&S99879[100], 2}, {NULL}};
const Segment SGI105485[] = {{&S99879[66], 6}, {NULL}};
const Segment SGI105486[] = {{&S99879[134], 8}, {NULL}};
const Segment SGI105487[] = {{&S83353[34], 7}, {NULL}};
const Segment SGI105488[] = {{&S83353[27], 6}, {NULL}};
const Segment SGI105489[] = {{&S83353[94], 9}, {NULL}};
const Segment SGI105490[] = {{&S83353[67], 6}, {NULL}};
const Segment SGI105491[] = {{&S83375[34], 10}, {NULL}};
const Segment SGI105492[] = {{&S83375[27], 6}, {NULL}};
const Segment SGI105493[] = {{&S83375[97], 9}, {NULL}};
const Segment SGI105494[] = {{&S83375[70], 6}, {NULL}};
const Segment SGI105495[] = {{&S83375[138], 8}, {NULL}};
const Segment SGI105496[] = {{&S100002[44], 7}, {NULL}};
const Segment SGI105497[] = {{&S100002[44], 24}, {NULL}};
const Segment SGI105498[] = {{&S100002[30], 6}, {NULL}};
const Segment SGI105499[] = {{&S100002[115], 3}, {NULL}};
const Segment SGI105500[] = {{&S100002[82], 5}, {NULL}};
const Segment SGI105501[] = {{&S100002[119], 14}, {NULL}};
const Segment SGI105502[] = {{&S83386[44], 10}, {NULL}};
const Segment SGI105503[] = {{&S83386[44], 27}, {NULL}};
const Segment SGI105504[] = {{&S83386[30], 6}, {NULL}};
const Segment SGI105505[] = {{&S83386[31], 3}, {NULL}};
const Segment SGI105506[] = {{&S83386[85], 5}, {NULL}};
const Segment SGI105507[] = {{&S83409[34], 8}, {NULL}};
const Segment SGI105508[] = {{&S83409[27], 6}, {NULL}};
const Segment SGI105509[] = {{&S83409[95], 9}, {NULL}};
const Segment SGI105510[] = {{&S83409[68], 6}, {NULL}};
const Segment SGI105511[] = {{&S98173[43], 10}, {NULL}};
const Segment SGI105512[] = {{&S98173[30], 4}, {NULL}};
const Segment SGI105513[] = {{&S98173[113], 3}, {NULL}};
const Segment SGI105514[] = {{&S98173[79], 5}, {NULL}};
const Segment SGI105515[] = {{&S98173[137], 13}, {NULL}};
const Segment SGI105516[] = {{&S98174[43], 11}, {NULL}};
const Segment SGI105517[] = {{&S98174[30], 4}, {NULL}};
const Segment SGI105518[] = {{&S98174[114], 3}, {NULL}};
const Segment SGI105519[] = {{&S98174[80], 5}, {NULL}};
const Segment SGI105520[] = {{&S98174[138], 13}, {NULL}};
const Segment SGI105521[] = {{&S98178[43], 10}, {NULL}};
const Segment SGI105522[] = {{&S98178[30], 4}, {NULL}};
const Segment SGI105523[] = {{&S98178[112], 3}, {NULL}};
const Segment SGI105524[] = {{&S98178[78], 5}, {NULL}};
const Segment SGI105525[] = {{&S98178[136], 13}, {NULL}};
const Segment SGI105526[] = {{&S98179[43], 11}, {NULL}};
const Segment SGI105527[] = {{&S98179[30], 4}, {NULL}};
const Segment SGI105528[] = {{&S98179[113], 3}, {NULL}};
const Segment SGI105529[] = {{&S98179[79], 5}, {NULL}};
const Segment SGI105530[] = {{&S98179[137], 13}, {NULL}};
const Segment SGI105531[] = {{&S98180[43], 10}, {NULL}};
const Segment SGI105532[] = {{&S98180[30], 4}, {NULL}};
const Segment SGI105533[] = {{&S98180[112], 3}, {NULL}};
const Segment SGI105534[] = {{&S98180[78], 5}, {NULL}};
const Segment SGI105535[] = {{&S98180[136], 13}, {NULL}};
const Segment SGI105536[] = {{&S83451[34], 17}, {NULL}};
const Segment SGI105537[] = {{&S83451[27], 6}, {NULL}};
const Segment SGI105538[] = {{&S83451[104], 9}, {NULL}};
const Segment SGI105539[] = {{&S83451[77], 6}, {NULL}};
const Segment SGI105540[] = {{&S83460[34], 5}, {NULL}};
const Segment SGI105541[] = {{&S83460[27], 6}, {NULL}};
const Segment SGI105542[] = {{&S83460[92], 9}, {NULL}};
const Segment SGI105543[] = {{&S83460[65], 6}, {NULL}};
const Segment SGI105544[] = {{&S83460[126], 8}, {NULL}};
const Segment SGI105545[] = {{&S83463[58], 4}, {NULL}};
const Segment SGI105546[] = {{&S83463[58], 26}, {NULL}};
const Segment SGI105547[] = {{&S83463[44], 6}, {NULL}};
const Segment SGI105548[] = {{&S83463[45], 3}, {NULL}};
const Segment SGI105549[] = {{&S83463[98], 6}, {NULL}};
const Segment SGI105550[] = {{&S83463[0], 14}, {NULL}};
const Segment SGI105551[] = {{&S83504[44], 24}, {NULL}};
const Segment SGI105552[] = {{&S83504[44], 36}, {NULL}};
const Segment SGI105553[] = {{&S83504[30], 6}, {NULL}};
const Segment SGI105554[] = {{&S83504[31], 3}, {NULL}};
const Segment SGI105555[] = {{&S83504[94], 6}, {NULL}};
const Segment SGI105556[] = {{&S83507[34], 18}, {NULL}};
const Segment SGI105557[] = {{&S83507[27], 6}, {NULL}};
const Segment SGI105558[] = {{&S83507[105], 9}, {NULL}};
const Segment SGI105559[] = {{&S83507[78], 6}, {NULL}};
const Segment SGI105560[] = {{&S100161[44], 10}, {NULL}};
const Segment SGI105561[] = {{&S100161[44], 20}, {NULL}};
const Segment SGI105562[] = {{&S100161[30], 6}, {NULL}};
const Segment SGI105563[] = {{&S100161[112], 3}, {NULL}};
const Segment SGI105564[] = {{&S100161[78], 5}, {NULL}};
const Segment SGI105565[] = {{&S100182[44], 10}, {NULL}};
const Segment SGI105566[] = {{&S100182[44], 20}, {NULL}};
const Segment SGI105567[] = {{&S100182[30], 6}, {NULL}};
const Segment SGI105568[] = {{&S100182[112], 3}, {NULL}};
const Segment SGI105569[] = {{&S100182[78], 5}, {NULL}};
const Segment SGI105570[] = {{&S83528[34], 15}, {NULL}};
const Segment SGI105571[] = {{&S83528[27], 6}, {NULL}};
const Segment SGI105572[] = {{&S83528[102], 9}, {NULL}};
const Segment SGI105573[] = {{&S83528[75], 6}, {NULL}};
const Segment SGI105574[] = {{&S83553[34], 17}, {NULL}};
const Segment SGI105575[] = {{&S83553[27], 6}, {NULL}};
const Segment SGI105576[] = {{&S83553[112], 9}, {NULL}};
const Segment SGI105577[] = {{&S83553[85], 6}, {NULL}};
const Segment SGI105578[] = {{&S83553[146], 8}, {NULL}};
const Segment SGI105579[] = {{&S83554[34], 17}, {NULL}};
const Segment SGI105580[] = {{&S83554[27], 6}, {NULL}};
const Segment SGI105581[] = {{&S83554[112], 9}, {NULL}};
const Segment SGI105582[] = {{&S83554[85], 6}, {NULL}};
const Segment SGI105583[] = {{&S83554[146], 8}, {NULL}};
const Segment SGI105584[] = {{&S98434[42], 10}, {NULL}};
const Segment SGI105585[] = {{&S98434[42], 21}, {NULL}};
const Segment SGI105586[] = {{&S98434[30], 4}, {NULL}};
const Segment SGI105587[] = {{&S98434[112], 3}, {NULL}};
const Segment SGI105588[] = {{&S98434[77], 6}, {NULL}};
const Segment SGI105589[] = {{&S98434[130], 13}, {NULL}};
const Segment SGI105590[] = {{&S83588[44], 11}, {NULL}};
const Segment SGI105591[] = {{&S83588[44], 23}, {NULL}};
const Segment SGI105592[] = {{&S83588[30], 6}, {NULL}};
const Segment SGI105593[] = {{&S83588[31], 3}, {NULL}};
const Segment SGI105594[] = {{&S83588[81], 6}, {NULL}};
const Segment SGI105595[] = {{&S83630[34], 22}, {NULL}};
const Segment SGI105596[] = {{&S83630[27], 6}, {NULL}};
const Segment SGI105597[] = {{&S83630[109], 9}, {NULL}};
const Segment SGI105598[] = {{&S83630[82], 6}, {NULL}};
const Segment SGI105599[] = {{&S83631[34], 8}, {NULL}};
const Segment SGI105600[] = {{&S83631[27], 6}, {NULL}};
const Segment SGI105601[] = {{&S83631[95], 9}, {NULL}};
const Segment SGI105602[] = {{&S83631[68], 6}, {NULL}};
const Segment SGI105603[] = {{&S83633[44], 10}, {NULL}};
const Segment SGI105604[] = {{&S83633[44], 22}, {NULL}};
const Segment SGI105605[] = {{&S83633[30], 6}, {NULL}};
const Segment SGI105606[] = {{&S83633[31], 3}, {NULL}};
const Segment SGI105607[] = {{&S83633[80], 6}, {NULL}};
const Segment SGI105608[] = {{&S83633[139], 10}, {NULL}};
const Segment SGI105609[] = {{&S83675[44], 12}, {NULL}};
const Segment SGI105610[] = {{&S83675[44], 24}, {NULL}};
const Segment SGI105611[] = {{&S83675[30], 6}, {NULL}};
const Segment SGI105612[] = {{&S83675[31], 3}, {NULL}};
const Segment SGI105613[] = {{&S83675[82], 6}, {NULL}};
const Segment SGI105614[] = {{&S83676[44], 8}, {NULL}};
const Segment SGI105615[] = {{&S83676[44], 19}, {NULL}};
const Segment SGI105616[] = {{&S83676[30], 6}, {NULL}};
const Segment SGI105617[] = {{&S83676[31], 3}, {NULL}};
const Segment SGI105618[] = {{&S83676[77], 6}, {NULL}};
const Segment SGI105619[] = {{&S83688[44], 8}, {NULL}};
const Segment SGI105620[] = {{&S83688[44], 20}, {NULL}};
const Segment SGI105621[] = {{&S83688[30], 6}, {NULL}};
const Segment SGI105622[] = {{&S83688[31], 3}, {NULL}};
const Segment SGI105623[] = {{&S83688[78], 6}, {NULL}};
const Segment SGI105624[] = {{&S100468[44], 11}, {NULL}};
const Segment SGI105625[] = {{&S100468[44], 22}, {NULL}};
const Segment SGI105626[] = {{&S100468[30], 6}, {NULL}};
const Segment SGI105627[] = {{&S100468[80], 6}, {NULL}};
const Segment SGI105628[] = {{&S100468[107], 13}, {NULL}};
const Segment SGI105629[] = {{&S100474[44], 7}, {NULL}};
const Segment SGI105630[] = {{&S100474[44], 17}, {NULL}};
const Segment SGI105631[] = {{&S100474[30], 6}, {NULL}};
const Segment SGI105632[] = {{&S100474[75], 6}, {NULL}};
const Segment SGI105633[] = {{&S100474[102], 13}, {NULL}};
const Segment SGI105634[] = {{&S83746[44], 18}, {NULL}};
const Segment SGI105635[] = {{&S83746[44], 30}, {NULL}};
const Segment SGI105636[] = {{&S83746[30], 6}, {NULL}};
const Segment SGI105637[] = {{&S83746[31], 3}, {NULL}};
const Segment SGI105638[] = {{&S83746[95], 6}, {NULL}};
const Segment SGI105639[] = {{&S83746[74], 8}, {NULL}};
const Segment SGI105640[] = {{&S83747[44], 35}, {NULL}};
const Segment SGI105641[] = {{&S83747[44], 47}, {NULL}};
const Segment SGI105642[] = {{&S83747[30], 6}, {NULL}};
const Segment SGI105643[] = {{&S83747[31], 3}, {NULL}};
const Segment SGI105644[] = {{&S83747[112], 6}, {NULL}};
const Segment SGI105645[] = {{&S83747[91], 8}, {NULL}};
const Segment SGI105646[] = {{&S83748[44], 18}, {NULL}};
const Segment SGI105647[] = {{&S83748[44], 30}, {NULL}};
const Segment SGI105648[] = {{&S83748[30], 6}, {NULL}};
const Segment SGI105649[] = {{&S83748[31], 3}, {NULL}};
const Segment SGI105650[] = {{&S83748[88], 6}, {NULL}};
const Segment SGI105651[] = {{&S83748[148], 7}, {NULL}};
const Segment SGI105652[] = {{&S83749[44], 21}, {NULL}};
const Segment SGI105653[] = {{&S83749[44], 33}, {NULL}};
const Segment SGI105654[] = {{&S83749[30], 6}, {NULL}};
const Segment SGI105655[] = {{&S83749[31], 3}, {NULL}};
const Segment SGI105656[] = {{&S83749[91], 6}, {NULL}};
const Segment SGI105657[] = {{&S83749[151], 7}, {NULL}};
const Segment SGI105658[] = {{&S83758[44], 11}, {NULL}};
const Segment SGI105659[] = {{&S83758[44], 23}, {NULL}};
const Segment SGI105660[] = {{&S83758[30], 6}, {NULL}};
const Segment SGI105661[] = {{&S83758[31], 3}, {NULL}};
const Segment SGI105662[] = {{&S83758[81], 6}, {NULL}};
const Segment SGI105663[] = {{&S100516[44], 7}, {NULL}};
const Segment SGI105664[] = {{&S100516[44], 24}, {NULL}};
const Segment SGI105665[] = {{&S100516[30], 6}, {NULL}};
const Segment SGI105666[] = {{&S100516[63], 3}, {NULL}};
const Segment SGI105667[] = {{&S100516[82], 6}, {NULL}};
const Segment SGI105668[] = {{&S100516[137], 8}, {NULL}};
const Segment SGI105669[] = {{&S100516[146], 13}, {NULL}};
const Segment SGI105670[] = {{&S98722[50], 6}, {NULL}};
const Segment SGI105671[] = {{&S98722[50], 17}, {NULL}};
const Segment SGI105672[] = {{&S98722[38], 4}, {NULL}};
const Segment SGI105673[] = {{&S98722[116], 3}, {NULL}};
const Segment SGI105674[] = {{&S98722[81], 6}, {NULL}};
const Segment SGI105675[] = {{&S100526[44], 7}, {NULL}};
const Segment SGI105676[] = {{&S100526[44], 24}, {NULL}};
const Segment SGI105677[] = {{&S100526[30], 6}, {NULL}};
const Segment SGI105678[] = {{&S100526[115], 3}, {NULL}};
const Segment SGI105679[] = {{&S100526[82], 5}, {NULL}};
const Segment SGI105680[] = {{&S100526[119], 14}, {NULL}};
const Segment SGI105681[] = {{&S100572[44], 7}, {NULL}};
const Segment SGI105682[] = {{&S100572[44], 17}, {NULL}};
const Segment SGI105683[] = {{&S100572[30], 6}, {NULL}};
const Segment SGI105684[] = {{&S100572[75], 6}, {NULL}};
const Segment SGI105685[] = {{&S100572[102], 13}, {NULL}};
const Segment SGI105686[] = {{&S83767[31], 8}, {NULL}};
const Segment SGI105687[] = {{&S83767[55], 3}, {NULL}};
const Segment SGI105688[] = {{&S98866[42], 9}, {NULL}};
const Segment SGI105689[] = {{&S98866[42], 18}, {NULL}};
const Segment SGI105690[] = {{&S98866[30], 4}, {NULL}};
const Segment SGI105691[] = {{&S98866[109], 3}, {NULL}};
const Segment SGI105692[] = {{&S98866[74], 6}, {NULL}};
const Segment SGI105693[] = {{&S83847[34], 7}, {NULL}};
const Segment SGI105694[] = {{&S83847[27], 6}, {NULL}};
const Segment SGI105695[] = {{&S83847[94], 9}, {NULL}};
const Segment SGI105696[] = {{&S83847[67], 6}, {NULL}};
const Segment SGI105697[] = {{&S83852[59], 14}, {NULL}};
const Segment SGI105698[] = {{&S83852[59], 31}, {NULL}};
const Segment SGI105699[] = {{&S83852[45], 6}, {NULL}};
const Segment SGI105700[] = {{&S83852[46], 3}, {NULL}};
const Segment SGI105701[] = {{&S83852[104], 6}, {NULL}};
const Segment SGI105702[] = {{&S83852[0], 14}, {NULL}};
const Segment SGI105703[] = {{&S83938[44], 13}, {NULL}};
const Segment SGI105704[] = {{&S83938[44], 25}, {NULL}};
const Segment SGI105705[] = {{&S83938[30], 6}, {NULL}};
const Segment SGI105706[] = {{&S83938[31], 3}, {NULL}};
const Segment SGI105707[] = {{&S83938[83], 6}, {NULL}};
const Segment SGI105708[] = {{&S83939[44], 13}, {NULL}};
const Segment SGI105709[] = {{&S83939[44], 25}, {NULL}};
const Segment SGI105710[] = {{&S83939[30], 6}, {NULL}};
const Segment SGI105711[] = {{&S83939[86], 6}, {NULL}};
const Segment SGI105712[] = {{&S83940[44], 15}, {NULL}};
const Segment SGI105713[] = {{&S83940[44], 27}, {NULL}};
const Segment SGI105714[] = {{&S83940[30], 6}, {NULL}};
const Segment SGI105715[] = {{&S83940[31], 3}, {NULL}};
const Segment SGI105716[] = {{&S83940[85], 6}, {NULL}};
const Segment SGI105717[] = {{&S84001[44], 8}, {NULL}};
const Segment SGI105718[] = {{&S84001[44], 20}, {NULL}};
const Segment SGI105719[] = {{&S84001[30], 6}, {NULL}};
const Segment SGI105720[] = {{&S84001[31], 3}, {NULL}};
const Segment SGI105721[] = {{&S84001[78], 6}, {NULL}};
const Segment SGI105722[] = {{&S84007[44], 16}, {NULL}};
const Segment SGI105723[] = {{&S84007[44], 42}, {NULL}};
const Segment SGI105724[] = {{&S84007[30], 6}, {NULL}};
const Segment SGI105725[] = {{&S84007[31], 3}, {NULL}};
const Segment SGI105726[] = {{&S84007[100], 6}, {NULL}};
const Segment SGI105727[] = {{&S99095[44], 7}, {NULL}};
const Segment SGI105728[] = {{&S99095[44], 18}, {NULL}};
const Segment SGI105729[] = {{&S99095[30], 6}, {NULL}};
const Segment SGI105730[] = {{&S99095[111], 3}, {NULL}};
const Segment SGI105731[] = {{&S99095[76], 6}, {NULL}};
const Segment SGI105732[] = {{&S99142[29], 8}, {NULL}};
const Segment SGI105733[] = {{&S99142[53], 3}, {NULL}};
const Segment SGI105734[] = {{&S99143[37], 8}, {NULL}};
const Segment SGI105735[] = {{&S99143[102], 3}, {NULL}};
const Segment SGI105736[] = {{&S99143[27], 3}, {NULL}};
const Segment SGI105737[] = {{&S99143[68], 6}, {NULL}};
const Segment SGI105738[] = {{&S99146[58], 10}, {NULL}};
const Segment SGI105739[] = {{&S99146[53], 4}, {NULL}};
const Segment SGI105740[] = {{&S100885[59], 7}, {NULL}};
const Segment SGI105741[] = {{&S100885[59], 24}, {NULL}};
const Segment SGI105742[] = {{&S100885[45], 6}, {NULL}};
const Segment SGI105743[] = {{&S100885[131], 3}, {NULL}};
const Segment SGI105744[] = {{&S100885[97], 5}, {NULL}};
const Segment SGI105745[] = {{&S100885[0], 14}, {NULL}};
const Segment SGI105746[] = {{&S84043[34], 17}, {NULL}};
const Segment SGI105747[] = {{&S84043[27], 6}, {NULL}};
const Segment SGI105748[] = {{&S84043[104], 9}, {NULL}};
const Segment SGI105749[] = {{&S84043[77], 6}, {NULL}};
const Segment SGI105750[] = {{&S84049[44], 21}, {NULL}};
const Segment SGI105751[] = {{&S84049[44], 33}, {NULL}};
const Segment SGI105752[] = {{&S84049[30], 6}, {NULL}};
const Segment SGI105753[] = {{&S84049[91], 6}, {NULL}};
const Segment SGI105754[] = {{&S84049[118], 13}, {NULL}};
const Segment SGI105755[] = {{&S84115[34], 10}, {NULL}};
const Segment SGI105756[] = {{&S84115[27], 6}, {NULL}};
const Segment SGI105757[] = {{&S84115[94], 9}, {NULL}};
const Segment SGI105758[] = {{&S84115[67], 6}, {NULL}};
const Segment SGI105759[] = {{&S84117[30], 6}, {NULL}};
const Segment SGI105760[] = {{&S84117[31], 3}, {NULL}};
const Segment SGI105761[] = {{&S84117[86], 5}, {NULL}};
const Segment SGI105762[] = {{&S84117[144], 13}, {NULL}};
const Segment SGI105763[] = {{&S84132[44], 10}, {NULL}};
const Segment SGI105764[] = {{&S84132[44], 22}, {NULL}};
const Segment SGI105765[] = {{&S84132[30], 6}, {NULL}};
const Segment SGI105766[] = {{&S84132[31], 3}, {NULL}};
const Segment SGI105767[] = {{&S84132[80], 6}, {NULL}};
const Segment SGI105768[] = {{&S84132[133], 11}, {NULL}};
const Segment SGI105769[] = {{&S84133[44], 10}, {NULL}};
const Segment SGI105770[] = {{&S84133[44], 22}, {NULL}};
const Segment SGI105771[] = {{&S84133[30], 6}, {NULL}};
const Segment SGI105772[] = {{&S84133[31], 3}, {NULL}};
const Segment SGI105773[] = {{&S84133[80], 6}, {NULL}};
const Segment SGI105774[] = {{&S84133[134], 12}, {NULL}};
const Segment SGI105775[] = {{&S84210[34], 13}, {NULL}};
const Segment SGI105776[] = {{&S84210[27], 6}, {NULL}};
const Segment SGI105777[] = {{&S84210[100], 9}, {NULL}};
const Segment SGI105778[] = {{&S84210[73], 6}, {NULL}};
const Segment SGI105779[] = {{&S99406[32], 7}, {NULL}};
const Segment SGI105780[] = {{&S99406[27], 4}, {NULL}};
const Segment SGI105781[] = {{&S99406[98], 2}, {NULL}};
const Segment SGI105782[] = {{&S99406[64], 6}, {NULL}};
const Segment SGI105783[] = {{&S99406[125], 8}, {NULL}};
const Segment SGI105784[] = {{&S84236[34], 6}, {NULL}};
const Segment SGI105785[] = {{&S84236[27], 6}, {NULL}};
const Segment SGI105786[] = {{&S84236[93], 9}, {NULL}};
const Segment SGI105787[] = {{&S84236[66], 6}, {NULL}};
const Segment SGI105788[] = {{&S99428[42], 6}, {NULL}};
const Segment SGI105789[] = {{&S99428[42], 17}, {NULL}};
const Segment SGI105790[] = {{&S99428[30], 4}, {NULL}};
const Segment SGI105791[] = {{&S99428[108], 3}, {NULL}};
const Segment SGI105792[] = {{&S99428[73], 6}, {NULL}};
const Segment SGI105793[] = {{&S84279[44], 24}, {NULL}};
const Segment SGI105794[] = {{&S84279[44], 36}, {NULL}};
const Segment SGI105795[] = {{&S84279[30], 6}, {NULL}};
const Segment SGI105796[] = {{&S84279[31], 3}, {NULL}};
const Segment SGI105797[] = {{&S84279[94], 6}, {NULL}};
const Segment SGI105798[] = {{&S84294[34], 22}, {NULL}};
const Segment SGI105799[] = {{&S84294[27], 6}, {NULL}};
const Segment SGI105800[] = {{&S84294[109], 9}, {NULL}};
const Segment SGI105801[] = {{&S84294[82], 6}, {NULL}};
const Segment SGI105802[] = {{&S84303[34], 9}, {NULL}};
const Segment SGI105803[] = {{&S84303[27], 6}, {NULL}};
const Segment SGI105804[] = {{&S84303[104], 9}, {NULL}};
const Segment SGI105805[] = {{&S84303[77], 6}, {NULL}};
const Segment SGI105806[] = {{&S101271[44], 12}, {NULL}};
const Segment SGI105807[] = {{&S101271[44], 22}, {NULL}};
const Segment SGI105808[] = {{&S101271[30], 6}, {NULL}};
const Segment SGI105809[] = {{&S101271[133], 3}, {NULL}};
const Segment SGI105810[] = {{&S101271[99], 5}, {NULL}};
const Segment SGI105811[] = {{&S101271[35], 1}, {NULL}};
const Segment SGI105812[] = {{&S101271[68], 11}, {NULL}};
const Segment SGI105813[] = {{&S84396[34], 10}, {NULL}};
const Segment SGI105814[] = {{&S84396[27], 6}, {NULL}};
const Segment SGI105815[] = {{&S84396[102], 9}, {NULL}};
const Segment SGI105816[] = {{&S84396[75], 6}, {NULL}};
const Segment SGI105817[] = {{&S84401[44], 10}, {NULL}};
const Segment SGI105818[] = {{&S84401[44], 22}, {NULL}};
const Segment SGI105819[] = {{&S84401[30], 6}, {NULL}};
const Segment SGI105820[] = {{&S84401[31], 3}, {NULL}};
const Segment SGI105821[] = {{&S84401[80], 6}, {NULL}};
const Segment SGI105822[] = {{&S101338[44], 9}, {NULL}};
const Segment SGI105823[] = {{&S101338[44], 19}, {NULL}};
const Segment SGI105824[] = {{&S101338[30], 6}, {NULL}};
const Segment SGI105825[] = {{&S101338[111], 3}, {NULL}};
const Segment SGI105826[] = {{&S101338[77], 5}, {NULL}};
const Segment SGI105827[] = {{&S84424[34], 7}, {NULL}};
const Segment SGI105828[] = {{&S84424[27], 6}, {NULL}};
const Segment SGI105829[] = {{&S84424[99], 9}, {NULL}};
const Segment SGI105830[] = {{&S84424[72], 6}, {NULL}};
const Segment SGI105831[] = {{&S101348[44], 11}, {NULL}};
const Segment SGI105832[] = {{&S101348[44], 21}, {NULL}};
const Segment SGI105833[] = {{&S101348[30], 6}, {NULL}};
const Segment SGI105834[] = {{&S101348[113], 3}, {NULL}};
const Segment SGI105835[] = {{&S101348[79], 5}, {NULL}};
const Segment SGI105836[] = {{&S84428[34], 14}, {NULL}};
const Segment SGI105837[] = {{&S84428[27], 6}, {NULL}};
const Segment SGI105838[] = {{&S84428[101], 9}, {NULL}};
const Segment SGI105839[] = {{&S84428[74], 6}, {NULL}};
const Segment SGI105840[] = {{&S84438[34], 7}, {NULL}};
const Segment SGI105841[] = {{&S84438[27], 6}, {NULL}};
const Segment SGI105842[] = {{&S84438[99], 9}, {NULL}};
const Segment SGI105843[] = {{&S84438[72], 6}, {NULL}};
const Segment SGI105844[] = {{&S84438[140], 8}, {NULL}};
const Segment SGI105845[] = {{&S84442[34], 16}, {NULL}};
const Segment SGI105846[] = {{&S84442[27], 6}, {NULL}};
const Segment SGI105847[] = {{&S84442[103], 9}, {NULL}};
const Segment SGI105848[] = {{&S84442[76], 6}, {NULL}};
const Segment SGI105849[] = {{&S84479[44], 11}, {NULL}};
const Segment SGI105850[] = {{&S84479[44], 23}, {NULL}};
const Segment SGI105851[] = {{&S84479[30], 6}, {NULL}};
const Segment SGI105852[] = {{&S84479[31], 3}, {NULL}};
const Segment SGI105853[] = {{&S84479[81], 6}, {NULL}};
const Segment SGI105854[] = {{&S99637[42], 18}, {NULL}};
const Segment SGI105855[] = {{&S99637[42], 30}, {NULL}};
const Segment SGI105856[] = {{&S99637[30], 4}, {NULL}};
const Segment SGI105857[] = {{&S99637[121], 3}, {NULL}};
const Segment SGI105858[] = {{&S99637[86], 6}, {NULL}};
const Segment SGI105859[] = {{&S84513[59], 10}, {NULL}};
const Segment SGI105860[] = {{&S84513[59], 22}, {NULL}};
const Segment SGI105861[] = {{&S84513[45], 6}, {NULL}};
const Segment SGI105862[] = {{&S84513[46], 3}, {NULL}};
const Segment SGI105863[] = {{&S84513[95], 6}, {NULL}};
const Segment SGI105864[] = {{&S84513[0], 14}, {NULL}};
const Segment SGI105865[] = {{&S93982[44], 16}, {NULL}};
const Segment SGI105866[] = {{&S93982[44], 26}, {NULL}};
const Segment SGI105867[] = {{&S93982[30], 6}, {NULL}};
const Segment SGI105868[] = {{&S93982[31], 3}, {NULL}};
const Segment SGI105869[] = {{&S93982[84], 5}, {NULL}};
const Segment SGI105870[] = {{&S93982[0], 13}, {NULL}};
const Segment SGI105871[] = {{&S84518[45], 14}, {NULL}};
const Segment SGI105872[] = {{&S84518[30], 6}, {NULL}};
const Segment SGI105873[] = {{&S84518[31], 3}, {NULL}};
const Segment SGI105874[] = {{&S84518[85], 5}, {NULL}};
const Segment SGI105875[] = {{&S84518[143], 13}, {NULL}};
const Segment SGI105876[] = {{&S93988[44], 8}, {NULL}};
const Segment SGI105877[] = {{&S93988[44], 18}, {NULL}};
const Segment SGI105878[] = {{&S93988[30], 6}, {NULL}};
const Segment SGI105879[] = {{&S93988[31], 3}, {NULL}};
const Segment SGI105880[] = {{&S93988[76], 5}, {NULL}};
const Segment SGI105881[] = {{&S93988[0], 13}, {NULL}};
const Segment SGI105882[] = {{&S84522[44], 9}, {NULL}};
const Segment SGI105883[] = {{&S84522[44], 50}, {NULL}};
const Segment SGI105884[] = {{&S84522[30], 6}, {NULL}};
const Segment SGI105885[] = {{&S84522[31], 3}, {NULL}};
const Segment SGI105886[] = {{&S84522[108], 6}, {NULL}};
const Segment SGI105887[] = {{&S84526[44], 10}, {NULL}};
const Segment SGI105888[] = {{&S84526[44], 22}, {NULL}};
const Segment SGI105889[] = {{&S84526[30], 6}, {NULL}};
const Segment SGI105890[] = {{&S84526[31], 3}, {NULL}};
const Segment SGI105891[] = {{&S84526[80], 6}, {NULL}};
const Segment SGI105892[] = {{&S84526[143], 3}, {NULL}};
const Segment SGI105893[] = {{&S84553[34], 6}, {NULL}};
const Segment SGI105894[] = {{&S84553[27], 6}, {NULL}};
const Segment SGI105895[] = {{&S84553[93], 9}, {NULL}};
const Segment SGI105896[] = {{&S84553[66], 6}, {NULL}};
const Segment SGI105897[] = {{&S84553[127], 8}, {NULL}};
const Segment SGI105898[] = {{&S84594[44], 10}, {NULL}};
const Segment SGI105899[] = {{&S84594[44], 27}, {NULL}};
const Segment SGI105900[] = {{&S84594[30], 6}, {NULL}};
const Segment SGI105901[] = {{&S84594[31], 3}, {NULL}};
const Segment SGI105902[] = {{&S84594[85], 6}, {NULL}};
const Segment SGI105903[] = {{&S84602[45], 10}, {NULL}};
const Segment SGI105904[] = {{&S84602[30], 6}, {NULL}};
const Segment SGI105905[] = {{&S84602[31], 3}, {NULL}};
const Segment SGI105906[] = {{&S84602[81], 5}, {NULL}};
const Segment SGI105907[] = {{&S84602[139], 13}, {NULL}};
const Segment SGI105908[] = {{&S84603[44], 13}, {NULL}};
const Segment SGI105909[] = {{&S84603[44], 20}, {NULL}};
const Segment SGI105910[] = {{&S84603[30], 6}, {NULL}};
const Segment SGI105911[] = {{&S84603[31], 3}, {NULL}};
const Segment SGI105912[] = {{&S84603[78], 6}, {NULL}};
const Segment SGI105913[] = {{&S84607[45], 15}, {NULL}};
const Segment SGI105914[] = {{&S84607[30], 6}, {NULL}};
const Segment SGI105915[] = {{&S84607[31], 3}, {NULL}};
const Segment SGI105916[] = {{&S84607[86], 5}, {NULL}};
const Segment SGI105917[] = {{&S84607[144], 13}, {NULL}};
const Segment SGI105918[] = {{&S84615[44], 14}, {NULL}};
const Segment SGI105919[] = {{&S84615[44], 31}, {NULL}};
const Segment SGI105920[] = {{&S84615[30], 6}, {NULL}};
const Segment SGI105921[] = {{&S84615[31], 3}, {NULL}};
const Segment SGI105922[] = {{&S84615[89], 5}, {NULL}};
const Segment SGI105923[] = {{&S84615[126], 14}, {NULL}};
const Segment SGI105924[] = {{&S84655[44], 14}, {NULL}};
const Segment SGI105925[] = {{&S84655[44], 31}, {NULL}};
const Segment SGI105926[] = {{&S84655[30], 6}, {NULL}};
const Segment SGI105927[] = {{&S84655[89], 6}, {NULL}};
const Segment SGI105928[] = {{&S84655[116], 13}, {NULL}};
const Segment SGI105929[] = {{&S94113[75], 6}, {NULL}};
const Segment SGI105930[] = {{&S94113[0], 10}, {NULL}};
const Segment SGI105931[] = {{&S84698[34], 14}, {NULL}};
const Segment SGI105932[] = {{&S84698[27], 6}, {NULL}};
const Segment SGI105933[] = {{&S84698[101], 9}, {NULL}};
const Segment SGI105934[] = {{&S84698[74], 6}, {NULL}};
const Segment SGI105935[] = {{&S84699[34], 10}, {NULL}};
const Segment SGI105936[] = {{&S84699[27], 6}, {NULL}};
const Segment SGI105937[] = {{&S84699[97], 9}, {NULL}};
const Segment SGI105938[] = {{&S84699[70], 6}, {NULL}};
const Segment SGI105939[] = {{&S84736[34], 11}, {NULL}};
const Segment SGI105940[] = {{&S84736[27], 6}, {NULL}};
const Segment SGI105941[] = {{&S84736[98], 9}, {NULL}};
const Segment SGI105942[] = {{&S84736[71], 6}, {NULL}};
const Segment SGI105943[] = {{&S84758[44], 41}, {NULL}};
const Segment SGI105944[] = {{&S84758[44], 53}, {NULL}};
const Segment SGI105945[] = {{&S84758[30], 6}, {NULL}};
const Segment SGI105946[] = {{&S84758[111], 6}, {NULL}};
const Segment SGI105947[] = {{&S84758[138], 13}, {NULL}};
const Segment SGI105948[] = {{&S84777[44], 14}, {NULL}};
const Segment SGI105949[] = {{&S84777[44], 26}, {NULL}};
const Segment SGI105950[] = {{&S84777[30], 6}, {NULL}};
const Segment SGI105951[] = {{&S84777[31], 3}, {NULL}};
const Segment SGI105952[] = {{&S84777[84], 6}, {NULL}};
const Segment SGI105953[] = {{&S84778[44], 19}, {NULL}};
const Segment SGI105954[] = {{&S84778[44], 31}, {NULL}};
const Segment SGI105955[] = {{&S84778[30], 6}, {NULL}};
const Segment SGI105956[] = {{&S84778[31], 3}, {NULL}};
const Segment SGI105957[] = {{&S84778[89], 6}, {NULL}};
const Segment SGI105958[] = {{&S84782[44], 25}, {NULL}};
const Segment SGI105959[] = {{&S84782[44], 36}, {NULL}};
const Segment SGI105960[] = {{&S84782[30], 6}, {NULL}};
const Segment SGI105961[] = {{&S84782[31], 3}, {NULL}};
const Segment SGI105962[] = {{&S84782[94], 6}, {NULL}};
const Segment SGI105963[] = {{&S84785[34], 8}, {NULL}};
const Segment SGI105964[] = {{&S84785[27], 6}, {NULL}};
const Segment SGI105965[] = {{&S84785[95], 9}, {NULL}};
const Segment SGI105966[] = {{&S84785[68], 6}, {NULL}};
const Segment SGI105967[] = {{&S84797[44], 6}, {NULL}};
const Segment SGI105968[] = {{&S84797[44], 18}, {NULL}};
const Segment SGI105969[] = {{&S84797[30], 6}, {NULL}};
const Segment SGI105970[] = {{&S84797[31], 3}, {NULL}};
const Segment SGI105971[] = {{&S84797[76], 6}, {NULL}};
const Segment SGI105972[] = {{&S99962[32], 15}, {NULL}};
const Segment SGI105973[] = {{&S99962[27], 4}, {NULL}};
const Segment SGI105974[] = {{&S99962[107], 2}, {NULL}};
const Segment SGI105975[] = {{&S99962[73], 6}, {NULL}};
const Segment SGI105976[] = {{&S99962[134], 8}, {NULL}};
const Segment SGI105977[] = {{&S84821[44], 18}, {NULL}};
const Segment SGI105978[] = {{&S84821[44], 30}, {NULL}};
const Segment SGI105979[] = {{&S84821[30], 6}, {NULL}};
const Segment SGI105980[] = {{&S84821[88], 6}, {NULL}};
const Segment SGI105981[] = {{&S84821[115], 13}, {NULL}};
const Segment SGI105982[] = {{&S101726[44], 7}, {NULL}};
const Segment SGI105983[] = {{&S101726[44], 18}, {NULL}};
const Segment SGI105984[] = {{&S101726[30], 6}, {NULL}};
const Segment SGI105985[] = {{&S101726[134], 3}, {NULL}};
const Segment SGI105986[] = {{&S101726[100], 5}, {NULL}};
const Segment SGI105987[] = {{&S84840[34], 6}, {NULL}};
const Segment SGI105988[] = {{&S84840[27], 6}, {NULL}};
const Segment SGI105989[] = {{&S84840[93], 9}, {NULL}};
const Segment SGI105990[] = {{&S84840[66], 6}, {NULL}};
const Segment SGI105991[] = {{&S84854[44], 11}, {NULL}};
const Segment SGI105992[] = {{&S84854[44], 23}, {NULL}};
const Segment SGI105993[] = {{&S84854[30], 6}, {NULL}};
const Segment SGI105994[] = {{&S84854[31], 3}, {NULL}};
const Segment SGI105995[] = {{&S84854[81], 6}, {NULL}};
const Segment SGI105996[] = {{&S84873[34], 7}, {NULL}};
const Segment SGI105997[] = {{&S84873[27], 6}, {NULL}};
const Segment SGI105998[] = {{&S84873[94], 9}, {NULL}};
const Segment SGI105999[] = {{&S84873[67], 6}, {NULL}};
const Segment SGI106000[] = {{&S84876[34], 5}, {NULL}};
const Segment SGI106001[] = {{&S84876[27], 6}, {NULL}};
const Segment SGI106002[] = {{&S84876[92], 9}, {NULL}};
const Segment SGI106003[] = {{&S84876[65], 6}, {NULL}};
const Segment SGI106004[] = {{&S101770[44], 10}, {NULL}};
const Segment SGI106005[] = {{&S101770[44], 26}, {NULL}};
const Segment SGI106006[] = {{&S101770[30], 6}, {NULL}};
const Segment SGI106007[] = {{&S101770[117], 3}, {NULL}};
const Segment SGI106008[] = {{&S101770[84], 5}, {NULL}};
const Segment SGI106009[] = {{&S101770[121], 14}, {NULL}};
const Segment SGI106010[] = {{&S84882[44], 14}, {NULL}};
const Segment SGI106011[] = {{&S84882[44], 26}, {NULL}};
const Segment SGI106012[] = {{&S84882[30], 6}, {NULL}};
const Segment SGI106013[] = {{&S84882[31], 3}, {NULL}};
const Segment SGI106014[] = {{&S84882[84], 6}, {NULL}};
const Segment SGI106015[] = {{&S101776[44], 13}, {NULL}};
const Segment SGI106016[] = {{&S101776[44], 24}, {NULL}};
const Segment SGI106017[] = {{&S101776[30], 6}, {NULL}};
const Segment SGI106018[] = {{&S101776[135], 3}, {NULL}};
const Segment SGI106019[] = {{&S101776[101], 5}, {NULL}};
const Segment SGI106020[] = {{&S101776[35], 1}, {NULL}};
const Segment SGI106021[] = {{&S101776[70], 11}, {NULL}};
const Segment SGI106022[] = {{&S101778[44], 10}, {NULL}};
const Segment SGI106023[] = {{&S101778[44], 27}, {NULL}};
const Segment SGI106024[] = {{&S101778[30], 6}, {NULL}};
const Segment SGI106025[] = {{&S101778[138], 3}, {NULL}};
const Segment SGI106026[] = {{&S101778[104], 5}, {NULL}};
const Segment SGI106027[] = {{&S101778[35], 1}, {NULL}};
const Segment SGI106028[] = {{&S101778[73], 11}, {NULL}};
const Segment SGI106029[] = {{&S84893[44], 6}, {NULL}};
const Segment SGI106030[] = {{&S84893[44], 27}, {NULL}};
const Segment SGI106031[] = {{&S84893[30], 6}, {NULL}};
const Segment SGI106032[] = {{&S84893[31], 3}, {NULL}};
const Segment SGI106033[] = {{&S84893[85], 5}, {NULL}};
const Segment SGI106034[] = {{&S84894[38], 6}, {NULL}};
const Segment SGI106035[] = {{&S84894[31], 6}, {NULL}};
const Segment SGI106036[] = {{&S84899[34], 17}, {NULL}};
const Segment SGI106037[] = {{&S84899[27], 6}, {NULL}};
const Segment SGI106038[] = {{&S84899[104], 9}, {NULL}};
const Segment SGI106039[] = {{&S84899[77], 6}, {NULL}};
const Segment SGI106040[] = {{&S84902[44], 14}, {NULL}};
const Segment SGI106041[] = {{&S84902[44], 26}, {NULL}};
const Segment SGI106042[] = {{&S84902[30], 6}, {NULL}};
const Segment SGI106043[] = {{&S84902[31], 3}, {NULL}};
const Segment SGI106044[] = {{&S84902[84], 6}, {NULL}};
const Segment SGI106045[] = {{&S84906[44], 41}, {NULL}};
const Segment SGI106046[] = {{&S84906[44], 53}, {NULL}};
const Segment SGI106047[] = {{&S84906[30], 6}, {NULL}};
const Segment SGI106048[] = {{&S84906[111], 6}, {NULL}};
const Segment SGI106049[] = {{&S84906[138], 13}, {NULL}};
const Segment SGI106050[] = {{&S84909[34], 6}, {NULL}};
const Segment SGI106051[] = {{&S84909[27], 6}, {NULL}};
const Segment SGI106052[] = {{&S84909[93], 9}, {NULL}};
const Segment SGI106053[] = {{&S84909[66], 6}, {NULL}};
const Segment SGI106054[] = {{&S84940[34], 8}, {NULL}};
const Segment SGI106055[] = {{&S84940[27], 6}, {NULL}};
const Segment SGI106056[] = {{&S84940[95], 9}, {NULL}};
const Segment SGI106057[] = {{&S84940[68], 6}, {NULL}};
const Segment SGI106058[] = {{&S84941[34], 9}, {NULL}};
const Segment SGI106059[] = {{&S84941[27], 6}, {NULL}};
const Segment SGI106060[] = {{&S84941[96], 9}, {NULL}};
const Segment SGI106061[] = {{&S84941[69], 6}, {NULL}};
const Segment SGI106062[] = {{&S98325[32], 17}, {NULL}};
const Segment SGI106063[] = {{&S98325[27], 4}, {NULL}};
const Segment SGI106064[] = {{&S98325[102], 9}, {NULL}};
const Segment SGI106065[] = {{&S98325[75], 6}, {NULL}};
const Segment SGI106066[] = {{&S84974[44], 11}, {NULL}};
const Segment SGI106067[] = {{&S84974[44], 23}, {NULL}};
const Segment SGI106068[] = {{&S84974[30], 6}, {NULL}};
const Segment SGI106069[] = {{&S84974[31], 3}, {NULL}};
const Segment SGI106070[] = {{&S84974[81], 6}, {NULL}};
const Segment SGI106071[] = {{&S84999[44], 12}, {NULL}};
const Segment SGI106072[] = {{&S84999[44], 33}, {NULL}};
const Segment SGI106073[] = {{&S84999[30], 6}, {NULL}};
const Segment SGI106074[] = {{&S84999[31], 3}, {NULL}};
const Segment SGI106075[] = {{&S84999[91], 5}, {NULL}};
const Segment SGI106076[] = {{&S85064[34], 19}, {NULL}};
const Segment SGI106077[] = {{&S85064[27], 6}, {NULL}};
const Segment SGI106078[] = {{&S85064[106], 9}, {NULL}};
const Segment SGI106079[] = {{&S85064[79], 6}, {NULL}};
const Segment SGI106080[] = {{&S102054[44], 16}, {NULL}};
const Segment SGI106081[] = {{&S102054[44], 27}, {NULL}};
const Segment SGI106082[] = {{&S102054[30], 6}, {NULL}};
const Segment SGI106083[] = {{&S102054[138], 3}, {NULL}};
const Segment SGI106084[] = {{&S102054[104], 5}, {NULL}};
const Segment SGI106085[] = {{&S102054[35], 1}, {NULL}};
const Segment SGI106086[] = {{&S102054[73], 11}, {NULL}};
const Segment SGI106087[] = {{&S85077[59], 10}, {NULL}};
const Segment SGI106088[] = {{&S85077[59], 22}, {NULL}};
const Segment SGI106089[] = {{&S85077[45], 6}, {NULL}};
const Segment SGI106090[] = {{&S85077[46], 3}, {NULL}};
const Segment SGI106091[] = {{&S85077[95], 6}, {NULL}};
const Segment SGI106092[] = {{&S85077[0], 14}, {NULL}};
const Segment SGI106093[] = {{&S102087[57], 7}, {NULL}};
const Segment SGI106094[] = {{&S102087[57], 24}, {NULL}};
const Segment SGI106095[] = {{&S102087[43], 6}, {NULL}};
const Segment SGI106096[] = {{&S102087[129], 3}, {NULL}};
const Segment SGI106097[] = {{&S102087[95], 5}, {NULL}};
const Segment SGI106098[] = {{&S102132[59], 7}, {NULL}};
const Segment SGI106099[] = {{&S102132[59], 24}, {NULL}};
const Segment SGI106100[] = {{&S102132[45], 6}, {NULL}};
const Segment SGI106101[] = {{&S102132[131], 3}, {NULL}};
const Segment SGI106102[] = {{&S102132[97], 5}, {NULL}};
const Segment SGI106103[] = {{&S102132[0], 14}, {NULL}};
const Segment SGI106104[] = {{&S85125[34], 14}, {NULL}};
const Segment SGI106105[] = {{&S85125[27], 6}, {NULL}};
const Segment SGI106106[] = {{&S85125[101], 9}, {NULL}};
const Segment SGI106107[] = {{&S85125[74], 6}, {NULL}};
const Segment SGI106108[] = {{&S85125[142], 8}, {NULL}};
const Segment SGI106109[] = {{&S85126[34], 14}, {NULL}};
const Segment SGI106110[] = {{&S85126[27], 6}, {NULL}};
const Segment SGI106111[] = {{&S85126[101], 9}, {NULL}};
const Segment SGI106112[] = {{&S85126[74], 6}, {NULL}};
const Segment SGI106113[] = {{&S85126[142], 8}, {NULL}};
const Segment SGI106114[] = {{&S85166[44], 7}, {NULL}};
const Segment SGI106115[] = {{&S85166[44], 19}, {NULL}};
const Segment SGI106116[] = {{&S85166[30], 6}, {NULL}};
const Segment SGI106117[] = {{&S85166[31], 3}, {NULL}};
const Segment SGI106118[] = {{&S85166[77], 6}, {NULL}};
const Segment SGI106119[] = {{&S85168[44], 12}, {NULL}};
const Segment SGI106120[] = {{&S85168[44], 24}, {NULL}};
const Segment SGI106121[] = {{&S85168[30], 6}, {NULL}};
const Segment SGI106122[] = {{&S85168[31], 3}, {NULL}};
const Segment SGI106123[] = {{&S85168[82], 6}, {NULL}};
const Segment SGI106124[] = {{&S85172[34], 9}, {NULL}};
const Segment SGI106125[] = {{&S85172[27], 6}, {NULL}};
const Segment SGI106126[] = {{&S85172[96], 9}, {NULL}};
const Segment SGI106127[] = {{&S85172[69], 6}, {NULL}};
const Segment SGI106128[] = {{&S85173[34], 8}, {NULL}};
const Segment SGI106129[] = {{&S85173[27], 6}, {NULL}};
const Segment SGI106130[] = {{&S85173[95], 9}, {NULL}};
const Segment SGI106131[] = {{&S85173[68], 6}, {NULL}};
const Segment SGI106132[] = {{&S85175[44], 17}, {NULL}};
const Segment SGI106133[] = {{&S85175[44], 42}, {NULL}};
const Segment SGI106134[] = {{&S85175[30], 6}, {NULL}};
const Segment SGI106135[] = {{&S85175[31], 3}, {NULL}};
const Segment SGI106136[] = {{&S85175[100], 6}, {NULL}};
const Segment SGI106137[] = {{&S102220[38], 7}, {NULL}};
const Segment SGI106138[] = {{&S102220[31], 6}, {NULL}};
const Segment SGI106139[] = {{&S85226[34], 8}, {NULL}};
const Segment SGI106140[] = {{&S85226[27], 6}, {NULL}};
const Segment SGI106141[] = {{&S85226[95], 9}, {NULL}};
const Segment SGI106142[] = {{&S85226[68], 6}, {NULL}};
const Segment SGI106143[] = {{&S85323[34], 9}, {NULL}};
const Segment SGI106144[] = {{&S85323[27], 6}, {NULL}};
const Segment SGI106145[] = {{&S85323[96], 9}, {NULL}};
const Segment SGI106146[] = {{&S85323[69], 6}, {NULL}};
const Segment SGI106147[] = {{&S85348[45], 10}, {NULL}};
const Segment SGI106148[] = {{&S85348[30], 6}, {NULL}};
const Segment SGI106149[] = {{&S85348[31], 3}, {NULL}};
const Segment SGI106150[] = {{&S85348[81], 5}, {NULL}};
const Segment SGI106151[] = {{&S85348[139], 13}, {NULL}};
const Segment SGI106152[] = {{&S85349[45], 11}, {NULL}};
const Segment SGI106153[] = {{&S85349[30], 6}, {NULL}};
const Segment SGI106154[] = {{&S85349[31], 3}, {NULL}};
const Segment SGI106155[] = {{&S85349[82], 5}, {NULL}};
const Segment SGI106156[] = {{&S85349[140], 13}, {NULL}};
const Segment SGI106157[] = {{&S85350[45], 11}, {NULL}};
const Segment SGI106158[] = {{&S85350[30], 6}, {NULL}};
const Segment SGI106159[] = {{&S85350[31], 3}, {NULL}};
const Segment SGI106160[] = {{&S85350[82], 5}, {NULL}};
const Segment SGI106161[] = {{&S85350[140], 13}, {NULL}};
const Segment SGI106162[] = {{&S85382[44], 11}, {NULL}};
const Segment SGI106163[] = {{&S85382[44], 23}, {NULL}};
const Segment SGI106164[] = {{&S85382[30], 6}, {NULL}};
const Segment SGI106165[] = {{&S85382[81], 6}, {NULL}};
const Segment SGI106166[] = {{&S85382[108], 13}, {NULL}};
const Segment SGI106167[] = {{&S94909[44], 10}, {NULL}};
const Segment SGI106168[] = {{&S94909[44], 20}, {NULL}};
const Segment SGI106169[] = {{&S94909[30], 6}, {NULL}};
const Segment SGI106170[] = {{&S94909[31], 3}, {NULL}};
const Segment SGI106171[] = {{&S94909[78], 5}, {NULL}};
const Segment SGI106172[] = {{&S94909[0], 13}, {NULL}};
const Segment SGI106173[] = {{&S85471[44], 6}, {NULL}};
const Segment SGI106174[] = {{&S85471[44], 18}, {NULL}};
const Segment SGI106175[] = {{&S85471[30], 6}, {NULL}};
const Segment SGI106176[] = {{&S85471[31], 3}, {NULL}};
const Segment SGI106177[] = {{&S85471[76], 6}, {NULL}};
const Segment SGI106178[] = {{&S85476[39], 8}, {NULL}};
const Segment SGI106179[] = {{&S85476[104], 3}, {NULL}};
const Segment SGI106180[] = {{&S85476[93], 3}, {NULL}};
const Segment SGI106181[] = {{&S85476[70], 6}, {NULL}};
const Segment SGI106182[] = {{&S85477[31], 8}, {NULL}};
const Segment SGI106183[] = {{&S85477[55], 3}, {NULL}};
const Segment SGI106184[] = {{&S85479[44], 10}, {NULL}};
const Segment SGI106185[] = {{&S85479[44], 22}, {NULL}};
const Segment SGI106186[] = {{&S85479[30], 6}, {NULL}};
const Segment SGI106187[] = {{&S85479[31], 3}, {NULL}};
const Segment SGI106188[] = {{&S85479[80], 6}, {NULL}};
const Segment SGI106189[] = {{&S85479[133], 11}, {NULL}};
const Segment SGI106190[] = {{&S98928[32], 17}, {NULL}};
const Segment SGI106191[] = {{&S98928[27], 4}, {NULL}};
const Segment SGI106192[] = {{&S98928[102], 9}, {NULL}};
const Segment SGI106193[] = {{&S98928[75], 6}, {NULL}};
const Segment SGI106194[] = {{&S85521[44], 18}, {NULL}};
const Segment SGI106195[] = {{&S85521[44], 30}, {NULL}};
const Segment SGI106196[] = {{&S85521[30], 6}, {NULL}};
const Segment SGI106197[] = {{&S85521[88], 6}, {NULL}};
const Segment SGI106198[] = {{&S85521[115], 13}, {NULL}};
const Segment SGI106199[] = {{&S85525[34], 12}, {NULL}};
const Segment SGI106200[] = {{&S85525[27], 6}, {NULL}};
const Segment SGI106201[] = {{&S85525[99], 9}, {NULL}};
const Segment SGI106202[] = {{&S85525[72], 6}, {NULL}};
const Segment SGI106203[] = {{&S85525[140], 8}, {NULL}};
const Segment SGI106204[] = {{&S85527[34], 10}, {NULL}};
const Segment SGI106205[] = {{&S85527[27], 6}, {NULL}};
const Segment SGI106206[] = {{&S85527[97], 9}, {NULL}};
const Segment SGI106207[] = {{&S85527[70], 6}, {NULL}};
const Segment SGI106208[] = {{&S102473[44], 7}, {NULL}};
const Segment SGI106209[] = {{&S102473[44], 22}, {NULL}};
const Segment SGI106210[] = {{&S102473[30], 6}, {NULL}};
const Segment SGI106211[] = {{&S102473[133], 3}, {NULL}};
const Segment SGI106212[] = {{&S102473[99], 5}, {NULL}};
const Segment SGI106213[] = {{&S102473[35], 1}, {NULL}};
const Segment SGI106214[] = {{&S102473[68], 11}, {NULL}};
const Segment SGI106215[] = {{&S102475[44], 10}, {NULL}};
const Segment SGI106216[] = {{&S102475[44], 21}, {NULL}};
const Segment SGI106217[] = {{&S102475[30], 6}, {NULL}};
const Segment SGI106218[] = {{&S102475[132], 3}, {NULL}};
const Segment SGI106219[] = {{&S102475[98], 5}, {NULL}};
const Segment SGI106220[] = {{&S102475[35], 1}, {NULL}};
const Segment SGI106221[] = {{&S102475[67], 11}, {NULL}};
const Segment SGI106222[] = {{&S85540[34], 15}, {NULL}};
const Segment SGI106223[] = {{&S85540[27], 6}, {NULL}};
const Segment SGI106224[] = {{&S85540[104], 9}, {NULL}};
const Segment SGI106225[] = {{&S85540[77], 6}, {NULL}};
const Segment SGI106226[] = {{&S102480[38], 7}, {NULL}};
const Segment SGI106227[] = {{&S102480[31], 6}, {NULL}};
const Segment SGI106228[] = {{&S102484[44], 8}, {NULL}};
const Segment SGI106229[] = {{&S102484[44], 20}, {NULL}};
const Segment SGI106230[] = {{&S102484[30], 6}, {NULL}};
const Segment SGI106231[] = {{&S102484[127], 3}, {NULL}};
const Segment SGI106232[] = {{&S102484[93], 5}, {NULL}};
const Segment SGI106233[] = {{&S102484[35], 1}, {NULL}};
const Segment SGI106234[] = {{&S102484[66], 11}, {NULL}};
const Segment SGI106235[] = {{&S85577[44], 6}, {NULL}};
const Segment SGI106236[] = {{&S85577[44], 18}, {NULL}};
const Segment SGI106237[] = {{&S85577[30], 6}, {NULL}};
const Segment SGI106238[] = {{&S85577[31], 3}, {NULL}};
const Segment SGI106239[] = {{&S85577[76], 6}, {NULL}};
const Segment SGI106240[] = {{&S85590[34], 14}, {NULL}};
const Segment SGI106241[] = {{&S85590[27], 6}, {NULL}};
const Segment SGI106242[] = {{&S85590[106], 9}, {NULL}};
const Segment SGI106243[] = {{&S85590[79], 6}, {NULL}};
const Segment SGI106244[] = {{&S85593[44], 10}, {NULL}};
const Segment SGI106245[] = {{&S85593[44], 22}, {NULL}};
const Segment SGI106246[] = {{&S85593[30], 6}, {NULL}};
const Segment SGI106247[] = {{&S85593[31], 3}, {NULL}};
const Segment SGI106248[] = {{&S85593[80], 5}, {NULL}};
const Segment SGI106249[] = {{&S85593[117], 14}, {NULL}};
const Segment SGI106250[] = {{&S85614[34], 6}, {NULL}};
const Segment SGI106251[] = {{&S85614[27], 6}, {NULL}};
const Segment SGI106252[] = {{&S85614[93], 9}, {NULL}};
const Segment SGI106253[] = {{&S85614[66], 6}, {NULL}};
const Segment SGI106254[] = {{&S85628[34], 9}, {NULL}};
const Segment SGI106255[] = {{&S85628[27], 6}, {NULL}};
const Segment SGI106256[] = {{&S85628[96], 9}, {NULL}};
const Segment SGI106257[] = {{&S85628[69], 6}, {NULL}};
const Segment SGI106258[] = {{&S95171[44], 10}, {NULL}};
const Segment SGI106259[] = {{&S95171[44], 20}, {NULL}};
const Segment SGI106260[] = {{&S95171[30], 6}, {NULL}};
const Segment SGI106261[] = {{&S95171[31], 3}, {NULL}};
const Segment SGI106262[] = {{&S95171[78], 5}, {NULL}};
const Segment SGI106263[] = {{&S95171[0], 13}, {NULL}};
const Segment SGI106264[] = {{&S85666[44], 14}, {NULL}};
const Segment SGI106265[] = {{&S85666[44], 26}, {NULL}};
const Segment SGI106266[] = {{&S85666[30], 6}, {NULL}};
const Segment SGI106267[] = {{&S85666[31], 3}, {NULL}};
const Segment SGI106268[] = {{&S85666[84], 6}, {NULL}};
const Segment SGI106269[] = {{&S85667[44], 15}, {NULL}};
const Segment SGI106270[] = {{&S85667[44], 27}, {NULL}};
const Segment SGI106271[] = {{&S85667[30], 6}, {NULL}};
const Segment SGI106272[] = {{&S85667[31], 3}, {NULL}};
const Segment SGI106273[] = {{&S85667[85], 6}, {NULL}};
const Segment SGI106274[] = {{&S85679[34], 9}, {NULL}};
const Segment SGI106275[] = {{&S85679[27], 6}, {NULL}};
const Segment SGI106276[] = {{&S85679[96], 9}, {NULL}};
const Segment SGI106277[] = {{&S85679[69], 6}, {NULL}};
const Segment SGI106278[] = {{&S102634[44], 7}, {NULL}};
const Segment SGI106279[] = {{&S102634[44], 24}, {NULL}};
const Segment SGI106280[] = {{&S102634[30], 6}, {NULL}};
const Segment SGI106281[] = {{&S102634[135], 3}, {NULL}};
const Segment SGI106282[] = {{&S102634[101], 5}, {NULL}};
const Segment SGI106283[] = {{&S102634[35], 1}, {NULL}};
const Segment SGI106284[] = {{&S102634[70], 11}, {NULL}};
const Segment SGI106285[] = {{&S85694[34], 20}, {NULL}};
const Segment SGI106286[] = {{&S85694[27], 6}, {NULL}};
const Segment SGI106287[] = {{&S85694[122], 9}, {NULL}};
const Segment SGI106288[] = {{&S85694[95], 6}, {NULL}};
const Segment SGI106289[] = {{&S85695[34], 21}, {NULL}};
const Segment SGI106290[] = {{&S85695[27], 6}, {NULL}};
const Segment SGI106291[] = {{&S85695[123], 9}, {NULL}};
const Segment SGI106292[] = {{&S85695[96], 6}, {NULL}};
const Segment SGI106293[] = {{&S102661[44], 6}, {NULL}};
const Segment SGI106294[] = {{&S102661[44], 16}, {NULL}};
const Segment SGI106295[] = {{&S102661[30], 6}, {NULL}};
const Segment SGI106296[] = {{&S102661[108], 3}, {NULL}};
const Segment SGI106297[] = {{&S102661[74], 5}, {NULL}};
const Segment SGI106298[] = {{&S85750[34], 17}, {NULL}};
const Segment SGI106299[] = {{&S85750[27], 6}, {NULL}};
const Segment SGI106300[] = {{&S85750[104], 9}, {NULL}};
const Segment SGI106301[] = {{&S85750[77], 6}, {NULL}};
const Segment SGI106302[] = {{&S85751[34], 7}, {NULL}};
const Segment SGI106303[] = {{&S85751[27], 6}, {NULL}};
const Segment SGI106304[] = {{&S85751[94], 9}, {NULL}};
const Segment SGI106305[] = {{&S85751[67], 6}, {NULL}};
const Segment SGI106306[] = {{&S85808[30], 6}, {NULL}};
const Segment SGI106307[] = {{&S85808[31], 3}, {NULL}};
const Segment SGI106308[] = {{&S85808[86], 5}, {NULL}};
const Segment SGI106309[] = {{&S85808[144], 13}, {NULL}};
const Segment SGI106310[] = {{&S85809[44], 19}, {NULL}};
const Segment SGI106311[] = {{&S85809[44], 31}, {NULL}};
const Segment SGI106312[] = {{&S85809[30], 6}, {NULL}};
const Segment SGI106313[] = {{&S85809[31], 3}, {NULL}};
const Segment SGI106314[] = {{&S85809[89], 6}, {NULL}};
const Segment SGI106315[] = {{&S85812[44], 9}, {NULL}};
const Segment SGI106316[] = {{&S85812[44], 21}, {NULL}};
const Segment SGI106317[] = {{&S85812[30], 6}, {NULL}};
const Segment SGI106318[] = {{&S85812[31], 3}, {NULL}};
const Segment SGI106319[] = {{&S85812[79], 6}, {NULL}};
const Segment SGI106320[] = {{&S85855[34], 8}, {NULL}};
const Segment SGI106321[] = {{&S85855[27], 6}, {NULL}};
const Segment SGI106322[] = {{&S85855[95], 9}, {NULL}};
const Segment SGI106323[] = {{&S85855[68], 6}, {NULL}};
const Segment SGI106324[] = {{&S85859[34], 6}, {NULL}};
const Segment SGI106325[] = {{&S85859[27], 6}, {NULL}};
const Segment SGI106326[] = {{&S85859[93], 9}, {NULL}};
const Segment SGI106327[] = {{&S85859[66], 6}, {NULL}};
const Segment SGI106328[] = {{&S85859[127], 8}, {NULL}};
const Segment SGI106329[] = {{&S85873[34], 15}, {NULL}};
const Segment SGI106330[] = {{&S85873[27], 6}, {NULL}};
const Segment SGI106331[] = {{&S85873[102], 9}, {NULL}};
const Segment SGI106332[] = {{&S85873[75], 6}, {NULL}};
const Segment SGI106333[] = {{&S85873[136], 8}, {NULL}};
const Segment SGI106334[] = {{&S85930[34], 6}, {NULL}};
const Segment SGI106335[] = {{&S85930[27], 6}, {NULL}};
const Segment SGI106336[] = {{&S85930[93], 9}, {NULL}};
const Segment SGI106337[] = {{&S85930[66], 6}, {NULL}};
const Segment SGI106338[] = {{&S85946[34], 14}, {NULL}};
const Segment SGI106339[] = {{&S85946[27], 6}, {NULL}};
const Segment SGI106340[] = {{&S85946[101], 9}, {NULL}};
const Segment SGI106341[] = {{&S85946[74], 6}, {NULL}};
const Segment SGI106342[] = {{&S85963[34], 14}, {NULL}};
const Segment SGI106343[] = {{&S85963[27], 6}, {NULL}};
const Segment SGI106344[] = {{&S85963[101], 9}, {NULL}};
const Segment SGI106345[] = {{&S85963[74], 6}, {NULL}};
const Segment SGI106346[] = {{&S85967[34], 16}, {NULL}};
const Segment SGI106347[] = {{&S85967[27], 6}, {NULL}};
const Segment SGI106348[] = {{&S85967[103], 9}, {NULL}};
const Segment SGI106349[] = {{&S85967[76], 6}, {NULL}};
const Segment SGI106350[] = {{&S100850[29], 8}, {NULL}};
const Segment SGI106351[] = {{&S100850[53], 3}, {NULL}};
const Segment SGI106352[] = {{&S100851[37], 8}, {NULL}};
const Segment SGI106353[] = {{&S100851[102], 3}, {NULL}};
const Segment SGI106354[] = {{&S100851[91], 3}, {NULL}};
const Segment SGI106355[] = {{&S100851[68], 6}, {NULL}};
const Segment SGI106356[] = {{&S85981[34], 7}, {NULL}};
const Segment SGI106357[] = {{&S85981[27], 6}, {NULL}};
const Segment SGI106358[] = {{&S85981[99], 9}, {NULL}};
const Segment SGI106359[] = {{&S85981[72], 6}, {NULL}};
const Segment SGI106360[] = {{&S100862[42], 10}, {NULL}};
const Segment SGI106361[] = {{&S100862[42], 21}, {NULL}};
const Segment SGI106362[] = {{&S100862[30], 4}, {NULL}};
const Segment SGI106363[] = {{&S100862[112], 3}, {NULL}};
const Segment SGI106364[] = {{&S100862[77], 6}, {NULL}};
const Segment SGI106365[] = {{&S100862[130], 13}, {NULL}};
const Segment SGI106366[] = {{&S85987[44], 14}, {NULL}};
const Segment SGI106367[] = {{&S85987[44], 31}, {NULL}};
const Segment SGI106368[] = {{&S85987[30], 6}, {NULL}};
const Segment SGI106369[] = {{&S85987[31], 3}, {NULL}};
const Segment SGI106370[] = {{&S85987[89], 5}, {NULL}};
const Segment SGI106371[] = {{&S85987[126], 14}, {NULL}};
const Segment SGI106372[] = {{&S85999[34], 9}, {NULL}};
const Segment SGI106373[] = {{&S85999[27], 6}, {NULL}};
const Segment SGI106374[] = {{&S85999[96], 9}, {NULL}};
const Segment SGI106375[] = {{&S85999[69], 6}, {NULL}};
const Segment SGI106376[] = {{&S85999[137], 8}, {NULL}};
const Segment SGI106377[] = {{&S86000[34], 10}, {NULL}};
const Segment SGI106378[] = {{&S86000[27], 6}, {NULL}};
const Segment SGI106379[] = {{&S86000[97], 9}, {NULL}};
const Segment SGI106380[] = {{&S86000[70], 6}, {NULL}};
const Segment SGI106381[] = {{&S86000[138], 8}, {NULL}};
const Segment SGI106382[] = {{&S86049[34], 18}, {NULL}};
const Segment SGI106383[] = {{&S86049[27], 6}, {NULL}};
const Segment SGI106384[] = {{&S86049[109], 9}, {NULL}};
const Segment SGI106385[] = {{&S86049[82], 6}, {NULL}};
const Segment SGI106386[] = {{&S86052[44], 4}, {NULL}};
const Segment SGI106387[] = {{&S86052[44], 16}, {NULL}};
const Segment SGI106388[] = {{&S86052[30], 6}, {NULL}};
const Segment SGI106389[] = {{&S86052[31], 3}, {NULL}};
const Segment SGI106390[] = {{&S86052[74], 6}, {NULL}};
const Segment SGI106391[] = {{&S86053[44], 4}, {NULL}};
const Segment SGI106392[] = {{&S86053[44], 16}, {NULL}};
const Segment SGI106393[] = {{&S86053[30], 6}, {NULL}};
const Segment SGI106394[] = {{&S86053[31], 3}, {NULL}};
const Segment SGI106395[] = {{&S86053[74], 6}, {NULL}};
const Segment SGI106396[] = {{&S103063[44], 10}, {NULL}};
const Segment SGI106397[] = {{&S103063[44], 21}, {NULL}};
const Segment SGI106398[] = {{&S103063[30], 6}, {NULL}};
const Segment SGI106399[] = {{&S103063[132], 3}, {NULL}};
const Segment SGI106400[] = {{&S103063[98], 5}, {NULL}};
const Segment SGI106401[] = {{&S103063[35], 1}, {NULL}};
const Segment SGI106402[] = {{&S103063[67], 11}, {NULL}};
const Segment SGI106403[] = {{&S103103[44], 7}, {NULL}};
const Segment SGI106404[] = {{&S103103[44], 17}, {NULL}};
const Segment SGI106405[] = {{&S103103[30], 6}, {NULL}};
const Segment SGI106406[] = {{&S103103[109], 3}, {NULL}};
const Segment SGI106407[] = {{&S103103[75], 5}, {NULL}};
const Segment SGI106408[] = {{&S103117[44], 7}, {NULL}};
const Segment SGI106409[] = {{&S103117[44], 25}, {NULL}};
const Segment SGI106410[] = {{&S103117[30], 6}, {NULL}};
const Segment SGI106411[] = {{&S103117[57], 3}, {NULL}};
const Segment SGI106412[] = {{&S103117[102], 5}, {NULL}};
const Segment SGI106413[] = {{&S103117[35], 1}, {NULL}};
const Segment SGI106414[] = {{&S103117[71], 11}, {NULL}};
const Segment SGI106415[] = {{&S86133[44], 9}, {NULL}};
const Segment SGI106416[] = {{&S86133[44], 21}, {NULL}};
const Segment SGI106417[] = {{&S86133[30], 6}, {NULL}};
const Segment SGI106418[] = {{&S86133[31], 3}, {NULL}};
const Segment SGI106419[] = {{&S86133[79], 6}, {NULL}};
const Segment SGI106420[] = {{&S86139[34], 14}, {NULL}};
const Segment SGI106421[] = {{&S86139[27], 6}, {NULL}};
const Segment SGI106422[] = {{&S86139[101], 9}, {NULL}};
const Segment SGI106423[] = {{&S86139[74], 6}, {NULL}};
const Segment SGI106424[] = {{&S86140[44], 13}, {NULL}};
const Segment SGI106425[] = {{&S86140[44], 25}, {NULL}};
const Segment SGI106426[] = {{&S86140[30], 6}, {NULL}};
const Segment SGI106427[] = {{&S86140[31], 3}, {NULL}};
const Segment SGI106428[] = {{&S86140[83], 6}, {NULL}};
const Segment SGI106429[] = {{&S86187[44], 9}, {NULL}};
const Segment SGI106430[] = {{&S86187[44], 21}, {NULL}};
const Segment SGI106431[] = {{&S86187[30], 6}, {NULL}};
const Segment SGI106432[] = {{&S86187[79], 6}, {NULL}};
const Segment SGI106433[] = {{&S86187[106], 13}, {NULL}};
const Segment SGI106434[] = {{&S101023[44], 10}, {NULL}};
const Segment SGI106435[] = {{&S101023[44], 21}, {NULL}};
const Segment SGI106436[] = {{&S101023[30], 6}, {NULL}};
const Segment SGI106437[] = {{&S101023[114], 3}, {NULL}};
const Segment SGI106438[] = {{&S101023[79], 6}, {NULL}};
const Segment SGI106439[] = {{&S86252[34], 7}, {NULL}};
const Segment SGI106440[] = {{&S86252[27], 6}, {NULL}};
const Segment SGI106441[] = {{&S86252[94], 9}, {NULL}};
const Segment SGI106442[] = {{&S86252[67], 6}, {NULL}};
const Segment SGI106443[] = {{&S86254[44], 12}, {NULL}};
const Segment SGI106444[] = {{&S86254[44], 37}, {NULL}};
const Segment SGI106445[] = {{&S86254[30], 6}, {NULL}};
const Segment SGI106446[] = {{&S86254[31], 3}, {NULL}};
const Segment SGI106447[] = {{&S86254[95], 6}, {NULL}};
const Segment SGI106448[] = {{&S86304[44], 9}, {NULL}};
const Segment SGI106449[] = {{&S86304[44], 21}, {NULL}};
const Segment SGI106450[] = {{&S86304[30], 6}, {NULL}};
const Segment SGI106451[] = {{&S86304[79], 6}, {NULL}};
const Segment SGI106452[] = {{&S86304[106], 13}, {NULL}};
const Segment SGI106453[] = {{&S86308[44], 11}, {NULL}};
const Segment SGI106454[] = {{&S86308[44], 23}, {NULL}};
const Segment SGI106455[] = {{&S86308[30], 6}, {NULL}};
const Segment SGI106456[] = {{&S86308[81], 6}, {NULL}};
const Segment SGI106457[] = {{&S86308[108], 13}, {NULL}};
const Segment SGI106458[] = {{&S86309[34], 18}, {NULL}};
const Segment SGI106459[] = {{&S86309[27], 6}, {NULL}};
const Segment SGI106460[] = {{&S86309[105], 9}, {NULL}};
const Segment SGI106461[] = {{&S86309[78], 6}, {NULL}};
const Segment SGI106462[] = {{&S86309[146], 8}, {NULL}};
const Segment SGI106463[] = {{&S103344[44], 18}, {NULL}};
const Segment SGI106464[] = {{&S103344[44], 30}, {NULL}};
const Segment SGI106465[] = {{&S103344[30], 6}, {NULL}};
const Segment SGI106466[] = {{&S103344[141], 3}, {NULL}};
const Segment SGI106467[] = {{&S103344[107], 5}, {NULL}};
const Segment SGI106468[] = {{&S103344[35], 1}, {NULL}};
const Segment SGI106469[] = {{&S103344[76], 11}, {NULL}};
const Segment SGI106470[] = {{&S86343[44], 18}, {NULL}};
const Segment SGI106471[] = {{&S86343[44], 30}, {NULL}};
const Segment SGI106472[] = {{&S86343[30], 6}, {NULL}};
const Segment SGI106473[] = {{&S86343[31], 3}, {NULL}};
const Segment SGI106474[] = {{&S86343[88], 6}, {NULL}};
const Segment SGI106475[] = {{&S86349[45], 9}, {NULL}};
const Segment SGI106476[] = {{&S86349[30], 6}, {NULL}};
const Segment SGI106477[] = {{&S86349[31], 3}, {NULL}};
const Segment SGI106478[] = {{&S86349[80], 5}, {NULL}};
const Segment SGI106479[] = {{&S86349[138], 13}, {NULL}};
const Segment SGI106480[] = {{&S103381[31], 4}, {NULL}};
const Segment SGI106481[] = {{&S103381[50], 3}, {NULL}};
const Segment SGI106482[] = {{&S103387[44], 7}, {NULL}};
const Segment SGI106483[] = {{&S103387[44], 18}, {NULL}};
const Segment SGI106484[] = {{&S103387[30], 6}, {NULL}};
const Segment SGI106485[] = {{&S103387[125], 3}, {NULL}};
const Segment SGI106486[] = {{&S103387[91], 5}, {NULL}};
const Segment SGI106487[] = {{&S103387[35], 1}, {NULL}};
const Segment SGI106488[] = {{&S103387[64], 11}, {NULL}};
const Segment SGI106489[] = {{&S103395[59], 7}, {NULL}};
const Segment SGI106490[] = {{&S103395[59], 17}, {NULL}};
const Segment SGI106491[] = {{&S103395[45], 6}, {NULL}};
const Segment SGI106492[] = {{&S103395[124], 3}, {NULL}};
const Segment SGI106493[] = {{&S103395[90], 5}, {NULL}};
const Segment SGI106494[] = {{&S103395[0], 14}, {NULL}};
const Segment SGI106495[] = {{&S95992[44], 10}, {NULL}};
const Segment SGI106496[] = {{&S95992[44], 20}, {NULL}};
const Segment SGI106497[] = {{&S95992[30], 6}, {NULL}};
const Segment SGI106498[] = {{&S95992[31], 3}, {NULL}};
const Segment SGI106499[] = {{&S95992[78], 5}, {NULL}};
const Segment SGI106500[] = {{&S95992[0], 13}, {NULL}};
const Segment SGI106501[] = {{&S86411[34], 23}, {NULL}};
const Segment SGI106502[] = {{&S86411[27], 6}, {NULL}};
const Segment SGI106503[] = {{&S86411[110], 9}, {NULL}};
const Segment SGI106504[] = {{&S86411[83], 6}, {NULL}};
const Segment SGI106505[] = {{&S101239[60], 10}, {NULL}};
const Segment SGI106506[] = {{&S101239[55], 4}, {NULL}};
const Segment SGI106507[] = {{&S101240[59], 11}, {NULL}};
const Segment SGI106508[] = {{&S101240[54], 4}, {NULL}};
const Segment SGI106509[] = {{&S101241[60], 11}, {NULL}};
const Segment SGI106510[] = {{&S101241[55], 4}, {NULL}};
const Segment SGI106511[] = {{&S101242[60], 11}, {NULL}};
const Segment SGI106512[] = {{&S101242[55], 4}, {NULL}};
const Segment SGI106513[] = {{&S101243[43], 10}, {NULL}};
const Segment SGI106514[] = {{&S101243[30], 4}, {NULL}};
const Segment SGI106515[] = {{&S101243[113], 3}, {NULL}};
const Segment SGI106516[] = {{&S101243[79], 5}, {NULL}};
const Segment SGI106517[] = {{&S101243[137], 13}, {NULL}};
const Segment SGI106518[] = {{&S86425[44], 10}, {NULL}};
const Segment SGI106519[] = {{&S86425[44], 20}, {NULL}};
const Segment SGI106520[] = {{&S86425[30], 6}, {NULL}};
const Segment SGI106521[] = {{&S86425[31], 3}, {NULL}};
const Segment SGI106522[] = {{&S86425[78], 5}, {NULL}};
const Segment SGI106523[] = {{&S101248[43], 10}, {NULL}};
const Segment SGI106524[] = {{&S101248[30], 4}, {NULL}};
const Segment SGI106525[] = {{&S101248[113], 3}, {NULL}};
const Segment SGI106526[] = {{&S101248[79], 5}, {NULL}};
const Segment SGI106527[] = {{&S101248[137], 13}, {NULL}};
const Segment SGI106528[] = {{&S86453[34], 7}, {NULL}};
const Segment SGI106529[] = {{&S86453[27], 6}, {NULL}};
const Segment SGI106530[] = {{&S86453[94], 9}, {NULL}};
const Segment SGI106531[] = {{&S86453[67], 6}, {NULL}};
const Segment SGI106532[] = {{&S86457[59], 10}, {NULL}};
const Segment SGI106533[] = {{&S86457[59], 22}, {NULL}};
const Segment SGI106534[] = {{&S86457[45], 6}, {NULL}};
const Segment SGI106535[] = {{&S86457[46], 3}, {NULL}};
const Segment SGI106536[] = {{&S86457[95], 6}, {NULL}};
const Segment SGI106537[] = {{&S86457[0], 14}, {NULL}};
const Segment SGI106538[] = {{&S99978[32], 17}, {NULL}};
const Segment SGI106539[] = {{&S99978[27], 4}, {NULL}};
const Segment SGI106540[] = {{&S99978[102], 9}, {NULL}};
const Segment SGI106541[] = {{&S99978[75], 6}, {NULL}};
const Segment SGI106542[] = {{&S86494[34], 15}, {NULL}};
const Segment SGI106543[] = {{&S86494[27], 6}, {NULL}};
const Segment SGI106544[] = {{&S86494[104], 9}, {NULL}};
const Segment SGI106545[] = {{&S86494[77], 6}, {NULL}};
const Segment SGI106546[] = {{&S86497[44], 6}, {NULL}};
const Segment SGI106547[] = {{&S86497[44], 18}, {NULL}};
const Segment SGI106548[] = {{&S86497[30], 6}, {NULL}};
const Segment SGI106549[] = {{&S86497[76], 6}, {NULL}};
const Segment SGI106550[] = {{&S86497[103], 13}, {NULL}};
const Segment SGI106551[] = {{&S86498[44], 6}, {NULL}};
const Segment SGI106552[] = {{&S86498[44], 18}, {NULL}};
const Segment SGI106553[] = {{&S86498[30], 6}, {NULL}};
const Segment SGI106554[] = {{&S86498[76], 6}, {NULL}};
const Segment SGI106555[] = {{&S86498[103], 13}, {NULL}};
const Segment SGI106556[] = {{&S86506[34], 15}, {NULL}};
const Segment SGI106557[] = {{&S86506[27], 6}, {NULL}};
const Segment SGI106558[] = {{&S86506[102], 9}, {NULL}};
const Segment SGI106559[] = {{&S86506[75], 6}, {NULL}};
const Segment SGI106560[] = {{&S86553[44], 17}, {NULL}};
const Segment SGI106561[] = {{&S86553[44], 28}, {NULL}};
const Segment SGI106562[] = {{&S86553[30], 6}, {NULL}};
const Segment SGI106563[] = {{&S86553[31], 3}, {NULL}};
const Segment SGI106564[] = {{&S86553[86], 6}, {NULL}};
const Segment SGI106565[] = {{&S101401[42], 10}, {NULL}};
const Segment SGI106566[] = {{&S101401[42], 22}, {NULL}};
const Segment SGI106567[] = {{&S101401[30], 4}, {NULL}};
const Segment SGI106568[] = {{&S101401[78], 6}, {NULL}};
const Segment SGI106569[] = {{&S101401[105], 13}, {NULL}};
const Segment SGI106570[] = {{&S101406[43], 10}, {NULL}};
const Segment SGI106571[] = {{&S101406[30], 4}, {NULL}};
const Segment SGI106572[] = {{&S101406[113], 3}, {NULL}};
const Segment SGI106573[] = {{&S101406[79], 5}, {NULL}};
const Segment SGI106574[] = {{&S101406[137], 13}, {NULL}};
const Segment SGI106575[] = {{&S86572[34], 12}, {NULL}};
const Segment SGI106576[] = {{&S86572[27], 6}, {NULL}};
const Segment SGI106577[] = {{&S86572[99], 9}, {NULL}};
const Segment SGI106578[] = {{&S86572[72], 6}, {NULL}};
const Segment SGI106579[] = {{&S86592[34], 8}, {NULL}};
const Segment SGI106580[] = {{&S86592[27], 6}, {NULL}};
const Segment SGI106581[] = {{&S86592[95], 9}, {NULL}};
const Segment SGI106582[] = {{&S86592[68], 6}, {NULL}};
const Segment SGI106583[] = {{&S86608[45], 9}, {NULL}};
const Segment SGI106584[] = {{&S86608[30], 6}, {NULL}};
const Segment SGI106585[] = {{&S86608[31], 3}, {NULL}};
const Segment SGI106586[] = {{&S86608[80], 5}, {NULL}};
const Segment SGI106587[] = {{&S86608[138], 13}, {NULL}};
const Segment SGI106588[] = {{&S86617[34], 7}, {NULL}};
const Segment SGI106589[] = {{&S86617[27], 6}, {NULL}};
const Segment SGI106590[] = {{&S103716[44], 10}, {NULL}};
const Segment SGI106591[] = {{&S103716[44], 27}, {NULL}};
const Segment SGI106592[] = {{&S86617[94], 9}, {NULL}};
const Segment SGI106593[] = {{&S86617[67], 6}, {NULL}};
const Segment SGI106594[] = {{&S103716[30], 6}, {NULL}};
const Segment SGI106595[] = {{&S103716[138], 3}, {NULL}};
const Segment SGI106596[] = {{&S103716[104], 5}, {NULL}};
const Segment SGI106597[] = {{&S103716[35], 1}, {NULL}};
const Segment SGI106598[] = {{&S103716[73], 11}, {NULL}};
const Segment SGI106599[] = {{&S86689[45], 14}, {NULL}};
const Segment SGI106600[] = {{&S86689[30], 6}, {NULL}};
const Segment SGI106601[] = {{&S86689[31], 3}, {NULL}};
const Segment SGI106602[] = {{&S86689[85], 5}, {NULL}};
const Segment SGI106603[] = {{&S86689[143], 13}, {NULL}};
const Segment SGI106604[] = {{&S86692[44], 18}, {NULL}};
const Segment SGI106605[] = {{&S86692[44], 30}, {NULL}};
const Segment SGI106606[] = {{&S86692[30], 6}, {NULL}};
const Segment SGI106607[] = {{&S86692[31], 3}, {NULL}};
const Segment SGI106608[] = {{&S86692[88], 6}, {NULL}};
const Segment SGI106609[] = {{&S86711[45], 6}, {NULL}};
const Segment SGI106610[] = {{&S86711[30], 6}, {NULL}};
const Segment SGI106611[] = {{&S86711[31], 3}, {NULL}};
const Segment SGI106612[] = {{&S86711[77], 5}, {NULL}};
const Segment SGI106613[] = {{&S86711[135], 13}, {NULL}};
const Segment SGI106614[] = {{&S86730[44], 10}, {NULL}};
const Segment SGI106615[] = {{&S86730[44], 22}, {NULL}};
const Segment SGI106616[] = {{&S86730[30], 6}, {NULL}};
const Segment SGI106617[] = {{&S86730[31], 3}, {NULL}};
const Segment SGI106618[] = {{&S86730[80], 6}, {NULL}};
const Segment SGI106619[] = {{&S86735[34], 6}, {NULL}};
const Segment SGI106620[] = {{&S86735[27], 6}, {NULL}};
const Segment SGI106621[] = {{&S86735[93], 9}, {NULL}};
const Segment SGI106622[] = {{&S86735[66], 6}, {NULL}};
const Segment SGI106623[] = {{&S86752[34], 18}, {NULL}};
const Segment SGI106624[] = {{&S86752[27], 6}, {NULL}};
const Segment SGI106625[] = {{&S86752[105], 9}, {NULL}};
const Segment SGI106626[] = {{&S86752[78], 6}, {NULL}};
const Segment SGI106627[] = {{&S86753[34], 13}, {NULL}};
const Segment SGI106628[] = {{&S86753[27], 6}, {NULL}};
const Segment SGI106629[] = {{&S86753[100], 9}, {NULL}};
const Segment SGI106630[] = {{&S86753[73], 6}, {NULL}};
const Segment SGI106631[] = {{&S86757[34], 14}, {NULL}};
const Segment SGI106632[] = {{&S86757[27], 6}, {NULL}};
const Segment SGI106633[] = {{&S86757[101], 9}, {NULL}};
const Segment SGI106634[] = {{&S86757[74], 6}, {NULL}};
const Segment SGI106635[] = {{&S86757[142], 8}, {NULL}};
const Segment SGI106636[] = {{&S86794[44], 19}, {NULL}};
const Segment SGI106637[] = {{&S86794[44], 31}, {NULL}};
const Segment SGI106638[] = {{&S86794[30], 6}, {NULL}};
const Segment SGI106639[] = {{&S86794[31], 3}, {NULL}};
const Segment SGI106640[] = {{&S86794[89], 6}, {NULL}};
const Segment SGI106641[] = {{&S86798[44], 14}, {NULL}};
const Segment SGI106642[] = {{&S86798[44], 26}, {NULL}};
const Segment SGI106643[] = {{&S86798[30], 6}, {NULL}};
const Segment SGI106644[] = {{&S86798[31], 3}, {NULL}};
const Segment SGI106645[] = {{&S86798[84], 6}, {NULL}};
const Segment SGI106646[] = {{&S86814[34], 8}, {NULL}};
const Segment SGI106647[] = {{&S86814[27], 6}, {NULL}};
const Segment SGI106648[] = {{&S86814[95], 9}, {NULL}};
const Segment SGI106649[] = {{&S86814[68], 6}, {NULL}};
const Segment SGI106650[] = {{&S86820[44], 18}, {NULL}};
const Segment SGI106651[] = {{&S86820[44], 30}, {NULL}};
const Segment SGI106652[] = {{&S86820[30], 6}, {NULL}};
const Segment SGI106653[] = {{&S86820[31], 3}, {NULL}};
const Segment SGI106654[] = {{&S86820[88], 6}, {NULL}};
const Segment SGI106655[] = {{&S86820[148], 11}, {NULL}};
const Segment SGI106656[] = {{&S101556[43], 10}, {NULL}};
const Segment SGI106657[] = {{&S101556[30], 4}, {NULL}};
const Segment SGI106658[] = {{&S101556[112], 3}, {NULL}};
const Segment SGI106659[] = {{&S86821[44], 21}, {NULL}};
const Segment SGI106660[] = {{&S101556[78], 5}, {NULL}};
const Segment SGI106661[] = {{&S86821[44], 33}, {NULL}};
const Segment SGI106662[] = {{&S101556[136], 13}, {NULL}};
const Segment SGI106663[] = {{&S86821[30], 6}, {NULL}};
const Segment SGI106664[] = {{&S86821[31], 3}, {NULL}};
const Segment SGI106665[] = {{&S86821[91], 6}, {NULL}};
const Segment SGI106666[] = {{&S86821[151], 11}, {NULL}};
const Segment SGI106667[] = {{&S101558[42], 10}, {NULL}};
const Segment SGI106668[] = {{&S101558[42], 22}, {NULL}};
const Segment SGI106669[] = {{&S101558[30], 4}, {NULL}};
const Segment SGI106670[] = {{&S101558[113], 3}, {NULL}};
const Segment SGI106671[] = {{&S101558[78], 6}, {NULL}};
const Segment SGI106672[] = {{&S101558[130], 10}, {NULL}};
const Segment SGI106673[] = {{&S101566[42], 10}, {NULL}};
const Segment SGI106674[] = {{&S101566[42], 22}, {NULL}};
const Segment SGI106675[] = {{&S101566[30], 4}, {NULL}};
const Segment SGI106676[] = {{&S101566[113], 3}, {NULL}};
const Segment SGI106677[] = {{&S101566[78], 6}, {NULL}};
const Segment SGI106678[] = {{&S101566[131], 13}, {NULL}};
const Segment SGI106679[] = {{&S103955[44], 10}, {NULL}};
const Segment SGI106680[] = {{&S103955[44], 21}, {NULL}};
const Segment SGI106681[] = {{&S103955[30], 6}, {NULL}};
const Segment SGI106682[] = {{&S101577[43], 10}, {NULL}};
const Segment SGI106683[] = {{&S101577[30], 4}, {NULL}};
const Segment SGI106684[] = {{&S103955[132], 3}, {NULL}};
const Segment SGI106685[] = {{&S101577[113], 3}, {NULL}};
const Segment SGI106686[] = {{&S103955[98], 5}, {NULL}};
const Segment SGI106687[] = {{&S103955[35], 1}, {NULL}};
const Segment SGI106688[] = {{&S101577[79], 5}, {NULL}};
const Segment SGI106689[] = {{&S103955[67], 11}, {NULL}};
const Segment SGI106690[] = {{&S101577[137], 13}, {NULL}};
const Segment SGI106691[] = {{&S86865[34], 6}, {NULL}};
const Segment SGI106692[] = {{&S86865[27], 6}, {NULL}};
const Segment SGI106693[] = {{&S86865[93], 9}, {NULL}};
const Segment SGI106694[] = {{&S86865[66], 6}, {NULL}};
const Segment SGI106695[] = {{&S86928[57], 11}, {NULL}};
const Segment SGI106696[] = {{&S86928[57], 23}, {NULL}};
const Segment SGI106697[] = {{&S86928[43], 6}, {NULL}};
const Segment SGI106698[] = {{&S86928[44], 3}, {NULL}};
const Segment SGI106699[] = {{&S86928[94], 6}, {NULL}};
const Segment SGI106700[] = {{&S86928[0], 12}, {NULL}};
const Segment SGI106701[] = {{&S86932[34], 10}, {NULL}};
const Segment SGI106702[] = {{&S86932[27], 6}, {NULL}};
const Segment SGI106703[] = {{&S86932[96], 9}, {NULL}};
const Segment SGI106704[] = {{&S86932[69], 6}, {NULL}};
const Segment SGI106705[] = {{&S86936[34], 10}, {NULL}};
const Segment SGI106706[] = {{&S86936[27], 6}, {NULL}};
const Segment SGI106707[] = {{&S86936[105], 9}, {NULL}};
const Segment SGI106708[] = {{&S86936[78], 6}, {NULL}};
const Segment SGI106709[] = {{&S86951[45], 9}, {NULL}};
const Segment SGI106710[] = {{&S86951[30], 6}, {NULL}};
const Segment SGI106711[] = {{&S86951[31], 3}, {NULL}};
const Segment SGI106712[] = {{&S86951[80], 5}, {NULL}};
const Segment SGI106713[] = {{&S86951[138], 13}, {NULL}};
const Segment SGI106714[] = {{&S86953[34], 15}, {NULL}};
const Segment SGI106715[] = {{&S86953[27], 6}, {NULL}};
const Segment SGI106716[] = {{&S86953[102], 9}, {NULL}};
const Segment SGI106717[] = {{&S86953[75], 6}, {NULL}};
const Segment SGI106718[] = {{&S101749[60], 10}, {NULL}};
const Segment SGI106719[] = {{&S101749[55], 4}, {NULL}};
const Segment SGI106720[] = {{&S101750[60], 11}, {NULL}};
const Segment SGI106721[] = {{&S101750[55], 4}, {NULL}};
const Segment SGI106722[] = {{&S101758[57], 10}, {NULL}};
const Segment SGI106723[] = {{&S101758[57], 21}, {NULL}};
const Segment SGI106724[] = {{&S101758[45], 4}, {NULL}};
const Segment SGI106725[] = {{&S104009[45], 7}, {NULL}};
const Segment SGI106726[] = {{&S101758[127], 3}, {NULL}};
const Segment SGI106727[] = {{&S104009[30], 6}, {NULL}};
const Segment SGI106728[] = {{&S101758[92], 6}, {NULL}};
const Segment SGI106729[] = {{&S101758[0], 14}, {NULL}};
const Segment SGI106730[] = {{&S104009[117], 3}, {NULL}};
const Segment SGI106731[] = {{&S104009[83], 5}, {NULL}};
const Segment SGI106732[] = {{&S104009[141], 13}, {NULL}};
const Segment SGI106733[] = {{&S104011[45], 7}, {NULL}};
const Segment SGI106734[] = {{&S104011[30], 6}, {NULL}};
const Segment SGI106735[] = {{&S104011[117], 3}, {NULL}};
const Segment SGI106736[] = {{&S104011[83], 5}, {NULL}};
const Segment SGI106737[] = {{&S104011[141], 13}, {NULL}};
const Segment SGI106738[] = {{&S87057[44], 8}, {NULL}};
const Segment SGI106739[] = {{&S87057[44], 20}, {NULL}};
const Segment SGI106740[] = {{&S87057[30], 6}, {NULL}};
const Segment SGI106741[] = {{&S87057[31], 3}, {NULL}};
const Segment SGI106742[] = {{&S87057[78], 6}, {NULL}};
const Segment SGI106743[] = {{&S87072[44], 24}, {NULL}};
const Segment SGI106744[] = {{&S87072[44], 36}, {NULL}};
const Segment SGI106745[] = {{&S87072[30], 6}, {NULL}};
const Segment SGI106746[] = {{&S87072[31], 3}, {NULL}};
const Segment SGI106747[] = {{&S87072[94], 6}, {NULL}};
const Segment SGI106748[] = {{&S87084[34], 7}, {NULL}};
const Segment SGI106749[] = {{&S87084[27], 6}, {NULL}};
const Segment SGI106750[] = {{&S87084[94], 9}, {NULL}};
const Segment SGI106751[] = {{&S87084[67], 6}, {NULL}};
const Segment SGI106752[] = {{&S87100[34], 10}, {NULL}};
const Segment SGI106753[] = {{&S87100[27], 6}, {NULL}};
const Segment SGI106754[] = {{&S87100[97], 9}, {NULL}};
const Segment SGI106755[] = {{&S87100[70], 6}, {NULL}};
const Segment SGI106756[] = {{&S87104[34], 11}, {NULL}};
const Segment SGI106757[] = {{&S87104[27], 6}, {NULL}};
const Segment SGI106758[] = {{&S87104[98], 9}, {NULL}};
const Segment SGI106759[] = {{&S87104[71], 6}, {NULL}};
const Segment SGI106760[] = {{&S87117[34], 7}, {NULL}};
const Segment SGI106761[] = {{&S87117[27], 6}, {NULL}};
const Segment SGI106762[] = {{&S87117[95], 9}, {NULL}};
const Segment SGI106763[] = {{&S87117[68], 6}, {NULL}};
const Segment SGI106764[] = {{&S87126[34], 17}, {NULL}};
const Segment SGI106765[] = {{&S87126[27], 6}, {NULL}};
const Segment SGI106766[] = {{&S87126[104], 9}, {NULL}};
const Segment SGI106767[] = {{&S87126[77], 6}, {NULL}};
const Segment SGI106768[] = {{&S101891[42], 10}, {NULL}};
const Segment SGI106769[] = {{&S101891[42], 21}, {NULL}};
const Segment SGI106770[] = {{&S101891[30], 4}, {NULL}};
const Segment SGI106771[] = {{&S101891[112], 3}, {NULL}};
const Segment SGI106772[] = {{&S101891[77], 6}, {NULL}};
const Segment SGI106773[] = {{&S101891[129], 10}, {NULL}};
const Segment SGI106774[] = {{&S87130[34], 9}, {NULL}};
const Segment SGI106775[] = {{&S87130[27], 6}, {NULL}};
const Segment SGI106776[] = {{&S87130[104], 9}, {NULL}};
const Segment SGI106777[] = {{&S87130[77], 6}, {NULL}};
const Segment SGI106778[] = {{&S87137[44], 11}, {NULL}};
const Segment SGI106779[] = {{&S87137[44], 23}, {NULL}};
const Segment SGI106780[] = {{&S87137[30], 6}, {NULL}};
const Segment SGI106781[] = {{&S101900[42], 10}, {NULL}};
const Segment SGI106782[] = {{&S87137[31], 3}, {NULL}};
const Segment SGI106783[] = {{&S101900[42], 22}, {NULL}};
const Segment SGI106784[] = {{&S101900[30], 4}, {NULL}};
const Segment SGI106785[] = {{&S87137[81], 6}, {NULL}};
const Segment SGI106786[] = {{&S87137[141], 7}, {NULL}};
const Segment SGI106787[] = {{&S101900[78], 6}, {NULL}};
const Segment SGI106788[] = {{&S101900[105], 13}, {NULL}};
const Segment SGI106789[] = {{&S87141[34], 9}, {NULL}};
const Segment SGI106790[] = {{&S87141[27], 6}, {NULL}};
const Segment SGI106791[] = {{&S87141[104], 9}, {NULL}};
const Segment SGI106792[] = {{&S87141[77], 6}, {NULL}};
const Segment SGI106793[] = {{&S87143[34], 18}, {NULL}};
const Segment SGI106794[] = {{&S87143[27], 6}, {NULL}};
const Segment SGI106795[] = {{&S87143[105], 9}, {NULL}};
const Segment SGI106796[] = {{&S87143[78], 6}, {NULL}};
const Segment SGI106797[] = {{&S87143[139], 8}, {NULL}};
const Segment SGI106798[] = {{&S87144[34], 18}, {NULL}};
const Segment SGI106799[] = {{&S87144[27], 6}, {NULL}};
const Segment SGI106800[] = {{&S87144[105], 9}, {NULL}};
const Segment SGI106801[] = {{&S87144[78], 6}, {NULL}};
const Segment SGI106802[] = {{&S87144[139], 8}, {NULL}};
const Segment SGI106803[] = {{&S87176[34], 15}, {NULL}};
const Segment SGI106804[] = {{&S87176[27], 6}, {NULL}};
const Segment SGI106805[] = {{&S87176[102], 9}, {NULL}};
const Segment SGI106806[] = {{&S87176[75], 6}, {NULL}};
const Segment SGI106807[] = {{&S87201[34], 6}, {NULL}};
const Segment SGI106808[] = {{&S87201[27], 6}, {NULL}};
const Segment SGI106809[] = {{&S87201[91], 9}, {NULL}};
const Segment SGI106810[] = {{&S87201[64], 6}, {NULL}};
const Segment SGI106811[] = {{&S87202[34], 8}, {NULL}};
const Segment SGI106812[] = {{&S87202[27], 6}, {NULL}};
const Segment SGI106813[] = {{&S87202[93], 9}, {NULL}};
const Segment SGI106814[] = {{&S87202[66], 6}, {NULL}};
const Segment SGI106815[] = {{&S87203[34], 18}, {NULL}};
const Segment SGI106816[] = {{&S87203[27], 6}, {NULL}};
const Segment SGI106817[] = {{&S87203[103], 9}, {NULL}};
const Segment SGI106818[] = {{&S87203[76], 6}, {NULL}};
const Segment SGI106819[] = {{&S87205[44], 11}, {NULL}};
const Segment SGI106820[] = {{&S87205[44], 23}, {NULL}};
const Segment SGI106821[] = {{&S87205[30], 6}, {NULL}};
const Segment SGI106822[] = {{&S87205[31], 3}, {NULL}};
const Segment SGI106823[] = {{&S87205[81], 6}, {NULL}};
const Segment SGI106824[] = {{&S87209[44], 11}, {NULL}};
const Segment SGI106825[] = {{&S87209[44], 23}, {NULL}};
const Segment SGI106826[] = {{&S87209[30], 6}, {NULL}};
const Segment SGI106827[] = {{&S87209[31], 3}, {NULL}};
const Segment SGI106828[] = {{&S87209[81], 6}, {NULL}};
const Segment SGI106829[] = {{&S87225[34], 15}, {NULL}};
const Segment SGI106830[] = {{&S87225[27], 6}, {NULL}};
const Segment SGI106831[] = {{&S87225[104], 9}, {NULL}};
const Segment SGI106832[] = {{&S87225[77], 6}, {NULL}};
const Segment SGI106833[] = {{&S87226[34], 6}, {NULL}};
const Segment SGI106834[] = {{&S87226[27], 6}, {NULL}};
const Segment SGI106835[] = {{&S87226[93], 9}, {NULL}};
const Segment SGI106836[] = {{&S87226[66], 6}, {NULL}};
const Segment SGI106837[] = {{&S102030[42], 5}, {NULL}};
const Segment SGI106838[] = {{&S102030[42], 17}, {NULL}};
const Segment SGI106839[] = {{&S102030[30], 4}, {NULL}};
const Segment SGI106840[] = {{&S102030[108], 3}, {NULL}};
const Segment SGI106841[] = {{&S102030[73], 6}, {NULL}};
const Segment SGI106842[] = {{&S102060[44], 11}, {NULL}};
const Segment SGI106843[] = {{&S102060[44], 22}, {NULL}};
const Segment SGI106844[] = {{&S102060[30], 6}, {NULL}};
const Segment SGI106845[] = {{&S102060[114], 3}, {NULL}};
const Segment SGI106846[] = {{&S102060[80], 5}, {NULL}};
const Segment SGI106847[] = {{&S102061[42], 11}, {NULL}};
const Segment SGI106848[] = {{&S102061[42], 22}, {NULL}};
const Segment SGI106849[] = {{&S102061[30], 4}, {NULL}};
const Segment SGI106850[] = {{&S102061[112], 3}, {NULL}};
const Segment SGI106851[] = {{&S102061[78], 5}, {NULL}};
const Segment SGI106852[] = {{&S102068[44], 8}, {NULL}};
const Segment SGI106853[] = {{&S102068[44], 19}, {NULL}};
const Segment SGI106854[] = {{&S102068[30], 6}, {NULL}};
const Segment SGI106855[] = {{&S102068[112], 3}, {NULL}};
const Segment SGI106856[] = {{&S102068[77], 6}, {NULL}};
const Segment SGI106857[] = {{&S87310[44], 13}, {NULL}};
const Segment SGI106858[] = {{&S87310[44], 25}, {NULL}};
const Segment SGI106859[] = {{&S87310[30], 6}, {NULL}};
const Segment SGI106860[] = {{&S87310[31], 3}, {NULL}};
const Segment SGI106861[] = {{&S87310[83], 6}, {NULL}};
const Segment SGI106862[] = {{&S87311[27], 6}, {NULL}};
const Segment SGI106863[] = {{&S87311[102], 9}, {NULL}};
const Segment SGI106864[] = {{&S87311[75], 6}, {NULL}};
const Segment SGI106865[] = {{&S87321[34], 6}, {NULL}};
const Segment SGI106866[] = {{&S87321[27], 6}, {NULL}};
const Segment SGI106867[] = {{&S87321[93], 9}, {NULL}};
const Segment SGI106868[] = {{&S87321[66], 6}, {NULL}};
const Segment SGI106869[] = {{&S87328[45], 21}, {NULL}};
const Segment SGI106870[] = {{&S87328[30], 6}, {NULL}};
const Segment SGI106871[] = {{&S87328[31], 3}, {NULL}};
const Segment SGI106872[] = {{&S87328[92], 5}, {NULL}};
const Segment SGI106873[] = {{&S87328[150], 13}, {NULL}};
const Segment SGI106874[] = {{&S87329[45], 18}, {NULL}};
const Segment SGI106875[] = {{&S87329[30], 6}, {NULL}};
const Segment SGI106876[] = {{&S87329[31], 3}, {NULL}};
const Segment SGI106877[] = {{&S87329[89], 5}, {NULL}};
const Segment SGI106878[] = {{&S87329[147], 13}, {NULL}};
const Segment SGI106879[] = {{&S87335[45], 4}, {NULL}};
const Segment SGI106880[] = {{&S87335[30], 6}, {NULL}};
const Segment SGI106881[] = {{&S87335[31], 3}, {NULL}};
const Segment SGI106882[] = {{&S87335[85], 5}, {NULL}};
const Segment SGI106883[] = {{&S87335[143], 13}, {NULL}};
const Segment SGI106884[] = {{&S87336[34], 6}, {NULL}};
const Segment SGI106885[] = {{&S87336[27], 6}, {NULL}};
const Segment SGI106886[] = {{&S87336[93], 9}, {NULL}};
const Segment SGI106887[] = {{&S87336[66], 6}, {NULL}};
const Segment SGI106888[] = {{&S102100[55], 10}, {NULL}};
const Segment SGI106889[] = {{&S102100[55], 22}, {NULL}};
const Segment SGI106890[] = {{&S102100[43], 4}, {NULL}};
const Segment SGI106891[] = {{&S102100[126], 3}, {NULL}};
const Segment SGI106892[] = {{&S102100[91], 6}, {NULL}};
const Segment SGI106893[] = {{&S102100[0], 12}, {NULL}};
const Segment SGI106894[] = {{&S102105[57], 10}, {NULL}};
const Segment SGI106895[] = {{&S102105[57], 22}, {NULL}};
const Segment SGI106896[] = {{&S102105[45], 4}, {NULL}};
const Segment SGI106897[] = {{&S102105[128], 3}, {NULL}};
const Segment SGI106898[] = {{&S102105[93], 6}, {NULL}};
const Segment SGI106899[] = {{&S102105[0], 14}, {NULL}};
const Segment SGI106900[] = {{&S102106[57], 10}, {NULL}};
const Segment SGI106901[] = {{&S102106[57], 22}, {NULL}};
const Segment SGI106902[] = {{&S102106[45], 4}, {NULL}};
const Segment SGI106903[] = {{&S102106[128], 3}, {NULL}};
const Segment SGI106904[] = {{&S102106[93], 6}, {NULL}};
const Segment SGI106905[] = {{&S102106[0], 14}, {NULL}};
const Segment SGI106906[] = {{&S87356[34], 17}, {NULL}};
const Segment SGI106907[] = {{&S87356[27], 6}, {NULL}};
const Segment SGI106908[] = {{&S87356[104], 9}, {NULL}};
const Segment SGI106909[] = {{&S87356[77], 6}, {NULL}};
const Segment SGI106910[] = {{&S87373[34], 7}, {NULL}};
const Segment SGI106911[] = {{&S87373[27], 6}, {NULL}};
const Segment SGI106912[] = {{&S87373[99], 9}, {NULL}};
const Segment SGI106913[] = {{&S87373[72], 6}, {NULL}};
const Segment SGI106914[] = {{&S87373[140], 8}, {NULL}};
const Segment SGI106915[] = {{&S87376[34], 9}, {NULL}};
const Segment SGI106916[] = {{&S87376[27], 6}, {NULL}};
const Segment SGI106917[] = {{&S87376[104], 9}, {NULL}};
const Segment SGI106918[] = {{&S87376[77], 6}, {NULL}};
const Segment SGI106919[] = {{&S87380[44], 12}, {NULL}};
const Segment SGI106920[] = {{&S87380[44], 33}, {NULL}};
const Segment SGI106921[] = {{&S87380[30], 6}, {NULL}};
const Segment SGI106922[] = {{&S87380[126], 3}, {NULL}};
const Segment SGI106923[] = {{&S87380[91], 6}, {NULL}};
const Segment SGI106924[] = {{&S87380[146], 8}, {NULL}};
const Segment SGI106925[] = {{&S87380[155], 13}, {NULL}};
const Segment SGI106926[] = {{&S104373[44], 7}, {NULL}};
const Segment SGI106927[] = {{&S104373[44], 40}, {NULL}};
const Segment SGI106928[] = {{&S104373[30], 6}, {NULL}};
const Segment SGI106929[] = {{&S104373[132], 3}, {NULL}};
const Segment SGI106930[] = {{&S104373[98], 5}, {NULL}};
const Segment SGI106931[] = {{&S104380[45], 7}, {NULL}};
const Segment SGI106932[] = {{&S104380[30], 6}, {NULL}};
const Segment SGI106933[] = {{&S104380[116], 3}, {NULL}};
const Segment SGI106934[] = {{&S104380[82], 5}, {NULL}};
const Segment SGI106935[] = {{&S104380[140], 13}, {NULL}};
const Segment SGI106936[] = {{&S87446[34], 8}, {NULL}};
const Segment SGI106937[] = {{&S87446[27], 6}, {NULL}};
const Segment SGI106938[] = {{&S87446[95], 9}, {NULL}};
const Segment SGI106939[] = {{&S87446[68], 6}, {NULL}};
const Segment SGI106940[] = {{&S87466[45], 10}, {NULL}};
const Segment SGI106941[] = {{&S87466[30], 6}, {NULL}};
const Segment SGI106942[] = {{&S87466[31], 3}, {NULL}};
const Segment SGI106943[] = {{&S87466[81], 5}, {NULL}};
const Segment SGI106944[] = {{&S87466[139], 13}, {NULL}};
const Segment SGI106945[] = {{&S87470[34], 7}, {NULL}};
const Segment SGI106946[] = {{&S87470[27], 6}, {NULL}};
const Segment SGI106947[] = {{&S87470[99], 9}, {NULL}};
const Segment SGI106948[] = {{&S87470[72], 6}, {NULL}};
const Segment SGI106949[] = {{&S87500[44], 9}, {NULL}};
const Segment SGI106950[] = {{&S87500[44], 21}, {NULL}};
const Segment SGI106951[] = {{&S87500[30], 6}, {NULL}};
const Segment SGI106952[] = {{&S87500[79], 6}, {NULL}};
const Segment SGI106953[] = {{&S87500[106], 13}, {NULL}};
const Segment SGI106954[] = {{&S102278[42], 10}, {NULL}};
const Segment SGI106955[] = {{&S102278[42], 22}, {NULL}};
const Segment SGI106956[] = {{&S102278[30], 4}, {NULL}};
const Segment SGI106957[] = {{&S102278[120], 3}, {NULL}};
const Segment SGI106958[] = {{&S102278[85], 6}, {NULL}};
const Segment SGI106959[] = {{&S102278[64], 8}, {NULL}};
const Segment SGI106960[] = {{&S102285[42], 10}, {NULL}};
const Segment SGI106961[] = {{&S102285[42], 22}, {NULL}};
const Segment SGI106962[] = {{&S102285[30], 4}, {NULL}};
const Segment SGI106963[] = {{&S102285[156], 3}, {NULL}};
const Segment SGI106964[] = {{&S102285[122], 3}, {NULL}};
const Segment SGI106965[] = {{&S102285[87], 6}, {NULL}};
const Segment SGI106966[] = {{&S102285[142], 8}, {NULL}};
const Segment SGI106967[] = {{&S87564[44], 8}, {NULL}};
const Segment SGI106968[] = {{&S87564[44], 20}, {NULL}};
const Segment SGI106969[] = {{&S87564[30], 6}, {NULL}};
const Segment SGI106970[] = {{&S87564[31], 3}, {NULL}};
const Segment SGI106971[] = {{&S87564[78], 6}, {NULL}};
const Segment SGI106972[] = {{&S102295[43], 10}, {NULL}};
const Segment SGI106973[] = {{&S87565[44], 8}, {NULL}};
const Segment SGI106974[] = {{&S102295[30], 4}, {NULL}};
const Segment SGI106975[] = {{&S87565[44], 20}, {NULL}};
const Segment SGI106976[] = {{&S87565[30], 6}, {NULL}};
const Segment SGI106977[] = {{&S102295[112], 3}, {NULL}};
const Segment SGI106978[] = {{&S87565[31], 3}, {NULL}};
const Segment SGI106979[] = {{&S102295[78], 5}, {NULL}};
const Segment SGI106980[] = {{&S87565[78], 6}, {NULL}};
const Segment SGI106981[] = {{&S102295[136], 13}, {NULL}};
const Segment SGI106982[] = {{&S102296[43], 10}, {NULL}};
const Segment SGI106983[] = {{&S102296[30], 4}, {NULL}};
const Segment SGI106984[] = {{&S102296[113], 3}, {NULL}};
const Segment SGI106985[] = {{&S102296[79], 5}, {NULL}};
const Segment SGI106986[] = {{&S102296[137], 13}, {NULL}};
const Segment SGI106987[] = {{&S102303[43], 10}, {NULL}};
const Segment SGI106988[] = {{&S102303[30], 4}, {NULL}};
const Segment SGI106989[] = {{&S102303[113], 3}, {NULL}};
const Segment SGI106990[] = {{&S102303[79], 5}, {NULL}};
const Segment SGI106991[] = {{&S102303[137], 13}, {NULL}};
const Segment SGI106992[] = {{&S87598[45], 10}, {NULL}};
const Segment SGI106993[] = {{&S87598[30], 6}, {NULL}};
const Segment SGI106994[] = {{&S87598[31], 3}, {NULL}};
const Segment SGI106995[] = {{&S87598[81], 5}, {NULL}};
const Segment SGI106996[] = {{&S87598[139], 13}, {NULL}};
const Segment SGI106997[] = {{&S87613[34], 13}, {NULL}};
const Segment SGI106998[] = {{&S87613[27], 6}, {NULL}};
const Segment SGI106999[] = {{&S87613[100], 9}, {NULL}};
const Segment SGI107000[] = {{&S87613[73], 6}, {NULL}};
const Segment SGI107001[] = {{&S87647[34], 15}, {NULL}};
const Segment SGI107002[] = {{&S87647[27], 6}, {NULL}};
const Segment SGI107003[] = {{&S87647[102], 9}, {NULL}};
const Segment SGI107004[] = {{&S87647[75], 6}, {NULL}};
const Segment SGI107005[] = {{&S5120[44], 10}, {NULL}};
const Segment SGI107006[] = {{&S5120[44], 22}, {NULL}};
const Segment SGI107007[] = {{&S5120[30], 6}, {NULL}};
const Segment SGI107008[] = {{&S5120[31], 3}, {NULL}};
const Segment SGI107009[] = {{&S5120[80], 6}, {NULL}};
const Segment SGI107010[] = {{&S5125[44], 10}, {NULL}};
const Segment SGI107011[] = {{&S5125[44], 25}, {NULL}};
const Segment SGI107012[] = {{&S5125[30], 6}, {NULL}};
const Segment SGI107013[] = {{&S5125[31], 3}, {NULL}};
const Segment SGI107014[] = {{&S5125[83], 6}, {NULL}};
const Segment SGI107015[] = {{&S5129[44], 10}, {NULL}};
const Segment SGI107016[] = {{&S5129[44], 22}, {NULL}};
const Segment SGI107017[] = {{&S5129[30], 6}, {NULL}};
const Segment SGI107018[] = {{&S5129[80], 6}, {NULL}};
const Segment SGI107019[] = {{&S5130[44], 10}, {NULL}};
const Segment SGI107020[] = {{&S5130[44], 22}, {NULL}};
const Segment SGI107021[] = {{&S5130[30], 6}, {NULL}};
const Segment SGI107022[] = {{&S5130[80], 6}, {NULL}};
const Segment SGI107023[] = {{&S5133[44], 10}, {NULL}};
const Segment SGI107024[] = {{&S5133[44], 22}, {NULL}};
const Segment SGI107025[] = {{&S5133[30], 6}, {NULL}};
const Segment SGI107026[] = {{&S5133[31], 3}, {NULL}};
const Segment SGI107027[] = {{&S5133[80], 6}, {NULL}};
const Segment SGI107028[] = {{&S5135[38], 10}, {NULL}};
const Segment SGI107029[] = {{&S5135[31], 6}, {NULL}};
const Segment SGI107030[] = {{&S87689[44], 45}, {NULL}};
const Segment SGI107031[] = {{&S87689[44], 57}, {NULL}};
const Segment SGI107032[] = {{&S87689[30], 6}, {NULL}};
const Segment SGI107033[] = {{&S87689[31], 3}, {NULL}};
const Segment SGI107034[] = {{&S87689[115], 6}, {NULL}};
const Segment SGI107035[] = {{&S87690[44], 12}, {NULL}};
const Segment SGI107036[] = {{&S87690[44], 24}, {NULL}};
const Segment SGI107037[] = {{&S87690[30], 6}, {NULL}};
const Segment SGI107038[] = {{&S87690[31], 3}, {NULL}};
const Segment SGI107039[] = {{&S87690[82], 6}, {NULL}};
const Segment SGI107040[] = {{&S5142[44], 10}, {NULL}};
const Segment SGI107041[] = {{&S5142[44], 25}, {NULL}};
const Segment SGI107042[] = {{&S5142[30], 6}, {NULL}};
const Segment SGI107043[] = {{&S5142[31], 3}, {NULL}};
const Segment SGI107044[] = {{&S5142[83], 6}, {NULL}};
const Segment SGI107045[] = {{&S87693[34], 8}, {NULL}};
const Segment SGI107046[] = {{&S87693[27], 6}, {NULL}};
const Segment SGI107047[] = {{&S87693[95], 9}, {NULL}};
const Segment SGI107048[] = {{&S87693[68], 6}, {NULL}};
const Segment SGI107049[] = {{&S5145[44], 21}, {NULL}};
const Segment SGI107050[] = {{&S5145[44], 33}, {NULL}};
const Segment SGI107051[] = {{&S5145[30], 6}, {NULL}};
const Segment SGI107052[] = {{&S5145[31], 3}, {NULL}};
const Segment SGI107053[] = {{&S5145[91], 6}, {NULL}};
const Segment SGI107054[] = {{&S87696[34], 9}, {NULL}};
const Segment SGI107055[] = {{&S87696[27], 6}, {NULL}};
const Segment SGI107056[] = {{&S87696[96], 9}, {NULL}};
const Segment SGI107057[] = {{&S87696[69], 6}, {NULL}};
const Segment SGI107058[] = {{&S5151[44], 11}, {NULL}};
const Segment SGI107059[] = {{&S5151[44], 23}, {NULL}};
const Segment SGI107060[] = {{&S5151[30], 6}, {NULL}};
const Segment SGI107061[] = {{&S5151[84], 6}, {NULL}};
const Segment SGI107062[] = {{&S5153[44], 10}, {NULL}};
const Segment SGI107063[] = {{&S5153[44], 22}, {NULL}};
const Segment SGI107064[] = {{&S5153[30], 6}, {NULL}};
const Segment SGI107065[] = {{&S5153[31], 3}, {NULL}};
const Segment SGI107066[] = {{&S5153[80], 6}, {NULL}};
const Segment SGI107067[] = {{&S5154[44], 10}, {NULL}};
const Segment SGI107068[] = {{&S5154[44], 21}, {NULL}};
const Segment SGI107069[] = {{&S5154[30], 6}, {NULL}};
const Segment SGI107070[] = {{&S5154[31], 3}, {NULL}};
const Segment SGI107071[] = {{&S5154[79], 6}, {NULL}};
const Segment SGI107072[] = {{&S5155[44], 11}, {NULL}};
const Segment SGI107073[] = {{&S5155[44], 23}, {NULL}};
const Segment SGI107074[] = {{&S5155[30], 6}, {NULL}};
const Segment SGI107075[] = {{&S5155[31], 3}, {NULL}};
const Segment SGI107076[] = {{&S5155[81], 6}, {NULL}};
const Segment SGI107077[] = {{&S5157[44], 10}, {NULL}};
const Segment SGI107078[] = {{&S5157[44], 31}, {NULL}};
const Segment SGI107079[] = {{&S5157[30], 6}, {NULL}};
const Segment SGI107080[] = {{&S5157[31], 3}, {NULL}};
const Segment SGI107081[] = {{&S5157[89], 6}, {NULL}};
const Segment SGI107082[] = {{&S5158[44], 10}, {NULL}};
const Segment SGI107083[] = {{&S5158[44], 22}, {NULL}};
const Segment SGI107084[] = {{&S5158[30], 6}, {NULL}};
const Segment SGI107085[] = {{&S5158[83], 6}, {NULL}};
const Segment SGI107086[] = {{&S5159[44], 10}, {NULL}};
const Segment SGI107087[] = {{&S5159[44], 19}, {NULL}};
const Segment SGI107088[] = {{&S5159[30], 6}, {NULL}};
const Segment SGI107089[] = {{&S5159[31], 3}, {NULL}};
const Segment SGI107090[] = {{&S5159[77], 6}, {NULL}};
const Segment SGI107091[] = {{&S5160[44], 11}, {NULL}};
const Segment SGI107092[] = {{&S5160[44], 23}, {NULL}};
const Segment SGI107093[] = {{&S5160[30], 6}, {NULL}};
const Segment SGI107094[] = {{&S5160[31], 3}, {NULL}};
const Segment SGI107095[] = {{&S5160[81], 6}, {NULL}};
const Segment SGI107096[] = {{&S5162[44], 10}, {NULL}};
const Segment SGI107097[] = {{&S5162[44], 22}, {NULL}};
const Segment SGI107098[] = {{&S5162[30], 6}, {NULL}};
const Segment SGI107099[] = {{&S5162[31], 3}, {NULL}};
const Segment SGI107100[] = {{&S5162[80], 6}, {NULL}};
const Segment SGI107101[] = {{&S5164[38], 11}, {NULL}};
const Segment SGI107102[] = {{&S5164[31], 6}, {NULL}};
const Segment SGI107103[] = {{&S5165[44], 29}, {NULL}};
const Segment SGI107104[] = {{&S5165[44], 41}, {NULL}};
const Segment SGI107105[] = {{&S5165[30], 6}, {NULL}};
const Segment SGI107106[] = {{&S5165[31], 3}, {NULL}};
const Segment SGI107107[] = {{&S5165[99], 6}, {NULL}};
const Segment SGI107108[] = {{&S5168[44], 10}, {NULL}};
const Segment SGI107109[] = {{&S5168[44], 31}, {NULL}};
const Segment SGI107110[] = {{&S5168[30], 6}, {NULL}};
const Segment SGI107111[] = {{&S5168[31], 3}, {NULL}};
const Segment SGI107112[] = {{&S5168[89], 6}, {NULL}};
const Segment SGI107113[] = {{&S5169[44], 28}, {NULL}};
const Segment SGI107114[] = {{&S5169[44], 40}, {NULL}};
const Segment SGI107115[] = {{&S5169[30], 6}, {NULL}};
const Segment SGI107116[] = {{&S5169[31], 3}, {NULL}};
const Segment SGI107117[] = {{&S5169[98], 6}, {NULL}};
const Segment SGI107118[] = {{&S5170[42], 10}, {NULL}};
const Segment SGI107119[] = {{&S5170[30], 6}, {NULL}};
const Segment SGI107120[] = {{&S5170[31], 3}, {NULL}};
const Segment SGI107121[] = {{&S5170[78], 6}, {NULL}};
const Segment SGI107122[] = {{&S5171[44], 11}, {NULL}};
const Segment SGI107123[] = {{&S5171[44], 23}, {NULL}};
const Segment SGI107124[] = {{&S5171[30], 6}, {NULL}};
const Segment SGI107125[] = {{&S5171[31], 3}, {NULL}};
const Segment SGI107126[] = {{&S5171[81], 6}, {NULL}};
const Segment SGI107127[] = {{&S5172[44], 29}, {NULL}};
const Segment SGI107128[] = {{&S5172[44], 41}, {NULL}};
const Segment SGI107129[] = {{&S5172[30], 6}, {NULL}};
const Segment SGI107130[] = {{&S5172[31], 3}, {NULL}};
const Segment SGI107131[] = {{&S5172[99], 6}, {NULL}};
const Segment SGI107132[] = {{&S5175[44], 29}, {NULL}};
const Segment SGI107133[] = {{&S5175[44], 41}, {NULL}};
const Segment SGI107134[] = {{&S5175[30], 6}, {NULL}};
const Segment SGI107135[] = {{&S5175[31], 3}, {NULL}};
const Segment SGI107136[] = {{&S5175[99], 6}, {NULL}};
const Segment SGI107137[] = {{&S104776[44], 7}, {NULL}};
const Segment SGI107138[] = {{&S104776[44], 19}, {NULL}};
const Segment SGI107139[] = {{&S104776[30], 6}, {NULL}};
const Segment SGI107140[] = {{&S5177[44], 10}, {NULL}};
const Segment SGI107141[] = {{&S5177[44], 22}, {NULL}};
const Segment SGI107142[] = {{&S5177[30], 6}, {NULL}};
const Segment SGI107143[] = {{&S104776[126], 3}, {NULL}};
const Segment SGI107144[] = {{&S104776[92], 5}, {NULL}};
const Segment SGI107145[] = {{&S5177[80], 6}, {NULL}};
const Segment SGI107146[] = {{&S104776[35], 1}, {NULL}};
const Segment SGI107147[] = {{&S104776[65], 11}, {NULL}};
const Segment SGI107148[] = {{&S5179[44], 10}, {NULL}};
const Segment SGI107149[] = {{&S5179[44], 28}, {NULL}};
const Segment SGI107150[] = {{&S5179[30], 6}, {NULL}};
const Segment SGI107151[] = {{&S5179[31], 3}, {NULL}};
const Segment SGI107152[] = {{&S5179[86], 6}, {NULL}};
const Segment SGI107153[] = {{&S5180[42], 11}, {NULL}};
const Segment SGI107154[] = {{&S5180[30], 6}, {NULL}};
const Segment SGI107155[] = {{&S5180[31], 3}, {NULL}};
const Segment SGI107156[] = {{&S5180[79], 6}, {NULL}};
const Segment SGI107157[] = {{&S5182[44], 10}, {NULL}};
const Segment SGI107158[] = {{&S5182[44], 22}, {NULL}};
const Segment SGI107159[] = {{&S5182[30], 6}, {NULL}};
const Segment SGI107160[] = {{&S5182[83], 6}, {NULL}};
const Segment SGI107161[] = {{&S5183[42], 29}, {NULL}};
const Segment SGI107162[] = {{&S5183[30], 6}, {NULL}};
const Segment SGI107163[] = {{&S5183[31], 3}, {NULL}};
const Segment SGI107164[] = {{&S5183[97], 6}, {NULL}};
const Segment SGI107165[] = {{&S5185[44], 10}, {NULL}};
const Segment SGI107166[] = {{&S5185[44], 22}, {NULL}};
const Segment SGI107167[] = {{&S5185[30], 6}, {NULL}};
const Segment SGI107168[] = {{&S5185[31], 3}, {NULL}};
const Segment SGI107169[] = {{&S5185[80], 6}, {NULL}};
const Segment SGI107170[] = {{&S5187[38], 11}, {NULL}};
const Segment SGI107171[] = {{&S5187[31], 6}, {NULL}};
const Segment SGI107172[] = {{&S104795[44], 12}, {NULL}};
const Segment SGI107173[] = {{&S104795[44], 23}, {NULL}};
const Segment SGI107174[] = {{&S104795[30], 6}, {NULL}};
const Segment SGI107175[] = {{&S104795[134], 3}, {NULL}};
const Segment SGI107176[] = {{&S104795[100], 5}, {NULL}};
const Segment SGI107177[] = {{&S104795[35], 1}, {NULL}};
const Segment SGI107178[] = {{&S104795[69], 11}, {NULL}};
const Segment SGI107179[] = {{&S5190[44], 10}, {NULL}};
const Segment SGI107180[] = {{&S5190[44], 28}, {NULL}};
const Segment SGI107181[] = {{&S5190[30], 6}, {NULL}};
const Segment SGI107182[] = {{&S5190[31], 3}, {NULL}};
const Segment SGI107183[] = {{&S5190[86], 6}, {NULL}};
const Segment SGI107184[] = {{&S5193[44], 29}, {NULL}};
const Segment SGI107185[] = {{&S5193[44], 41}, {NULL}};
const Segment SGI107186[] = {{&S5193[30], 6}, {NULL}};
const Segment SGI107187[] = {{&S5193[31], 3}, {NULL}};
const Segment SGI107188[] = {{&S5193[99], 6}, {NULL}};
const Segment SGI107189[] = {{&S5194[44], 28}, {NULL}};
const Segment SGI107190[] = {{&S5194[44], 40}, {NULL}};
const Segment SGI107191[] = {{&S5194[30], 6}, {NULL}};
const Segment SGI107192[] = {{&S5194[31], 3}, {NULL}};
const Segment SGI107193[] = {{&S5194[98], 6}, {NULL}};
const Segment SGI107194[] = {{&S87709[45], 41}, {NULL}};
const Segment SGI107195[] = {{&S87709[30], 6}, {NULL}};
const Segment SGI107196[] = {{&S87709[31], 3}, {NULL}};
const Segment SGI107197[] = {{&S5195[44], 10}, {NULL}};
const Segment SGI107198[] = {{&S87709[112], 5}, {NULL}};
const Segment SGI107199[] = {{&S5195[44], 22}, {NULL}};
const Segment SGI107200[] = {{&S5195[30], 6}, {NULL}};
const Segment SGI107201[] = {{&S87709[170], 13}, {NULL}};
const Segment SGI107202[] = {{&S5195[83], 6}, {NULL}};
const Segment SGI107203[] = {{&S87711[45], 41}, {NULL}};
const Segment SGI107204[] = {{&S87711[30], 6}, {NULL}};
const Segment SGI107205[] = {{&S87711[31], 3}, {NULL}};
const Segment SGI107206[] = {{&S87711[112], 5}, {NULL}};
const Segment SGI107207[] = {{&S87711[170], 13}, {NULL}};
const Segment SGI107208[] = {{&S5198[44], 10}, {NULL}};
const Segment SGI107209[] = {{&S5198[44], 28}, {NULL}};
const Segment SGI107210[] = {{&S5198[30], 6}, {NULL}};
const Segment SGI107211[] = {{&S5198[31], 3}, {NULL}};
const Segment SGI107212[] = {{&S5198[86], 6}, {NULL}};
const Segment SGI107213[] = {{&S5201[44], 11}, {NULL}};
const Segment SGI107214[] = {{&S5201[44], 23}, {NULL}};
const Segment SGI107215[] = {{&S5201[30], 6}, {NULL}};
const Segment SGI107216[] = {{&S5201[31], 3}, {NULL}};
const Segment SGI107217[] = {{&S5201[81], 5}, {NULL}};
const Segment SGI107218[] = {{&S5202[44], 10}, {NULL}};
const Segment SGI107219[] = {{&S5202[44], 22}, {NULL}};
const Segment SGI107220[] = {{&S5202[30], 6}, {NULL}};
const Segment SGI107221[] = {{&S5202[31], 3}, {NULL}};
const Segment SGI107222[] = {{&S5202[80], 6}, {NULL}};
const Segment SGI107223[] = {{&S87720[34], 15}, {NULL}};
const Segment SGI107224[] = {{&S87720[27], 6}, {NULL}};
const Segment SGI107225[] = {{&S5203[44], 10}, {NULL}};
const Segment SGI107226[] = {{&S5203[44], 27}, {NULL}};
const Segment SGI107227[] = {{&S5203[30], 6}, {NULL}};
const Segment SGI107228[] = {{&S87720[104], 9}, {NULL}};
const Segment SGI107229[] = {{&S87720[77], 6}, {NULL}};
const Segment SGI107230[] = {{&S5203[31], 3}, {NULL}};
const Segment SGI107231[] = {{&S5203[85], 6}, {NULL}};
const Segment SGI107232[] = {{&S5204[42], 29}, {NULL}};
const Segment SGI107233[] = {{&S5204[30], 6}, {NULL}};
const Segment SGI107234[] = {{&S5204[31], 3}, {NULL}};
const Segment SGI107235[] = {{&S5204[97], 6}, {NULL}};
const Segment SGI107236[] = {{&S5207[44], 10}, {NULL}};
const Segment SGI107237[] = {{&S5207[44], 27}, {NULL}};
const Segment SGI107238[] = {{&S5207[30], 6}, {NULL}};
const Segment SGI107239[] = {{&S5207[31], 3}, {NULL}};
const Segment SGI107240[] = {{&S5207[85], 6}, {NULL}};
const Segment SGI107241[] = {{&S87736[34], 16}, {NULL}};
const Segment SGI107242[] = {{&S87736[27], 6}, {NULL}};
const Segment SGI107243[] = {{&S87736[103], 9}, {NULL}};
const Segment SGI107244[] = {{&S87736[76], 6}, {NULL}};
const Segment SGI107245[] = {{&S87736[144], 8}, {NULL}};
const Segment SGI107246[] = {{&S87737[34], 9}, {NULL}};
const Segment SGI107247[] = {{&S87737[27], 6}, {NULL}};
const Segment SGI107248[] = {{&S87737[96], 9}, {NULL}};
const Segment SGI107249[] = {{&S87737[69], 6}, {NULL}};
const Segment SGI107250[] = {{&S87737[137], 8}, {NULL}};
const Segment SGI107251[] = {{&S87739[34], 18}, {NULL}};
const Segment SGI107252[] = {{&S102498[60], 10}, {NULL}};
const Segment SGI107253[] = {{&S87739[27], 6}, {NULL}};
const Segment SGI107254[] = {{&S102498[55], 4}, {NULL}};
const Segment SGI107255[] = {{&S87739[105], 9}, {NULL}};
const Segment SGI107256[] = {{&S87739[78], 6}, {NULL}};
const Segment SGI107257[] = {{&S87740[34], 13}, {NULL}};
const Segment SGI107258[] = {{&S87740[27], 6}, {NULL}};
const Segment SGI107259[] = {{&S87740[100], 9}, {NULL}};
const Segment SGI107260[] = {{&S87740[73], 6}, {NULL}};
const Segment SGI107261[] = {{&S102511[42], 10}, {NULL}};
const Segment SGI107262[] = {{&S87753[34], 14}, {NULL}};
const Segment SGI107263[] = {{&S102511[42], 22}, {NULL}};
const Segment SGI107264[] = {{&S87753[27], 6}, {NULL}};
const Segment SGI107265[] = {{&S102511[30], 4}, {NULL}};
const Segment SGI107266[] = {{&S102511[144], 4}, {NULL}};
const Segment SGI107267[] = {{&S102511[170], 3}, {NULL}};
const Segment SGI107268[] = {{&S87753[101], 9}, {NULL}};
const Segment SGI107269[] = {{&S87753[74], 6}, {NULL}};
const Segment SGI107270[] = {{&S102511[113], 3}, {&S102511[219], 5}, {NULL}};
const Segment SGI107271[] = {{&S87753[135], 8}, {NULL}};
const Segment SGI107272[] = {{&S102511[78], 6}, {NULL}};
const Segment SGI107273[] = {{&S102512[42], 10}, {NULL}};
const Segment SGI107274[] = {{&S102512[42], 22}, {NULL}};
const Segment SGI107275[] = {{&S102512[30], 4}, {NULL}};
const Segment SGI107276[] = {{&S102512[137], 4}, {NULL}};
const Segment SGI107277[] = {{&S102512[163], 3}, {NULL}};
const Segment SGI107278[] = {{&S102512[113], 3}, {&S102512[212], 5}, {NULL}};
const Segment SGI107279[] = {{&S102512[78], 6}, {NULL}};
const Segment SGI107280[] = {{&S87757[34], 9}, {NULL}};
const Segment SGI107281[] = {{&S87757[27], 6}, {NULL}};
const Segment SGI107282[] = {{&S87757[96], 9}, {NULL}};
const Segment SGI107283[] = {{&S87757[69], 6}, {NULL}};
const Segment SGI107284[] = {{&S102519[43], 10}, {NULL}};
const Segment SGI107285[] = {{&S102519[30], 4}, {NULL}};
const Segment SGI107286[] = {{&S102519[113], 3}, {NULL}};
const Segment SGI107287[] = {{&S102519[79], 5}, {NULL}};
const Segment SGI107288[] = {{&S102519[137], 13}, {NULL}};
const Segment SGI107289[] = {{&S87775[34], 10}, {NULL}};
const Segment SGI107290[] = {{&S87775[27], 6}, {NULL}};
const Segment SGI107291[] = {{&S87775[97], 9}, {NULL}};
const Segment SGI107292[] = {{&S87775[70], 6}, {NULL}};
const Segment SGI107293[] = {{&S104960[44], 10}, {NULL}};
const Segment SGI107294[] = {{&S104960[44], 20}, {NULL}};
const Segment SGI107295[] = {{&S104960[30], 6}, {NULL}};
const Segment SGI107296[] = {{&S104960[111], 3}, {NULL}};
const Segment SGI107297[] = {{&S104960[78], 5}, {NULL}};
const Segment SGI107298[] = {{&S104960[115], 14}, {NULL}};
const Segment SGI107299[] = {{&S104961[44], 10}, {NULL}};
const Segment SGI107300[] = {{&S104961[44], 23}, {NULL}};
const Segment SGI107301[] = {{&S104961[30], 6}, {NULL}};
const Segment SGI107302[] = {{&S104961[114], 3}, {NULL}};
const Segment SGI107303[] = {{&S104961[81], 5}, {NULL}};
const Segment SGI107304[] = {{&S104961[118], 14}, {NULL}};
const Segment SGI107305[] = {{&S101353[42], 10}, {NULL}};
const Segment SGI107306[] = {{&S101353[42], 21}, {NULL}};
const Segment SGI107307[] = {{&S101353[30], 4}, {NULL}};
const Segment SGI107308[] = {{&S101353[31], 3}, {NULL}};
const Segment SGI107309[] = {{&S101353[77], 5}, {NULL}};
const Segment SGI107310[] = {{&S87826[44], 8}, {NULL}};
const Segment SGI107311[] = {{&S87826[44], 20}, {NULL}};
const Segment SGI107312[] = {{&S87826[30], 6}, {NULL}};
const Segment SGI107313[] = {{&S87826[31], 3}, {NULL}};
const Segment SGI107314[] = {{&S87826[78], 6}, {NULL}};
const Segment SGI107315[] = {{&S87832[34], 7}, {NULL}};
const Segment SGI107316[] = {{&S87832[27], 6}, {NULL}};
const Segment SGI107317[] = {{&S87832[95], 9}, {NULL}};
const Segment SGI107318[] = {{&S87832[68], 6}, {NULL}};
const Segment SGI107319[] = {{&S87852[44], 9}, {NULL}};
const Segment SGI107320[] = {{&S87852[44], 21}, {NULL}};
const Segment SGI107321[] = {{&S87852[30], 6}, {NULL}};
const Segment SGI107322[] = {{&S87852[31], 3}, {NULL}};
const Segment SGI107323[] = {{&S87852[79], 6}, {NULL}};
const Segment SGI107324[] = {{&S87870[34], 10}, {NULL}};
const Segment SGI107325[] = {{&S87870[27], 6}, {NULL}};
const Segment SGI107326[] = {{&S87870[97], 9}, {NULL}};
const Segment SGI107327[] = {{&S87870[70], 6}, {NULL}};
const Segment SGI107328[] = {{&S102689[61], 10}, {NULL}};
const Segment SGI107329[] = {{&S102689[56], 4}, {NULL}};
const Segment SGI107330[] = {{&S102690[56], 10}, {NULL}};
const Segment SGI107331[] = {{&S102690[56], 22}, {NULL}};
const Segment SGI107332[] = {{&S102690[44], 4}, {NULL}};
const Segment SGI107333[] = {{&S102690[127], 3}, {NULL}};
const Segment SGI107334[] = {{&S102690[92], 6}, {NULL}};
const Segment SGI107335[] = {{&S102691[43], 10}, {NULL}};
const Segment SGI107336[] = {{&S102691[30], 4}, {NULL}};
const Segment SGI107337[] = {{&S102691[113], 3}, {NULL}};
const Segment SGI107338[] = {{&S102691[79], 5}, {NULL}};
const Segment SGI107339[] = {{&S102691[137], 13}, {NULL}};
const Segment SGI107340[] = {{&S102707[42], 10}, {NULL}};
const Segment SGI107341[] = {{&S102707[42], 22}, {NULL}};
const Segment SGI107342[] = {{&S102707[30], 4}, {NULL}};
const Segment SGI107343[] = {{&S102707[113], 3}, {NULL}};
const Segment SGI107344[] = {{&S102707[78], 6}, {NULL}};
const Segment SGI107345[] = {{&S102707[131], 13}, {NULL}};
const Segment SGI107346[] = {{&S102712[42], 10}, {NULL}};
const Segment SGI107347[] = {{&S102712[42], 22}, {NULL}};
const Segment SGI107348[] = {{&S102712[30], 4}, {NULL}};
const Segment SGI107349[] = {{&S102712[113], 3}, {NULL}};
const Segment SGI107350[] = {{&S102712[78], 6}, {NULL}};
const Segment SGI107351[] = {{&S102712[138], 11}, {NULL}};
const Segment SGI107352[] = {{&S102713[42], 10}, {NULL}};
const Segment SGI107353[] = {{&S102713[42], 22}, {NULL}};
const Segment SGI107354[] = {{&S102713[30], 4}, {NULL}};
const Segment SGI107355[] = {{&S102713[113], 3}, {NULL}};
const Segment SGI107356[] = {{&S102713[78], 6}, {NULL}};
const Segment SGI107357[] = {{&S102713[131], 11}, {NULL}};
const Segment SGI107358[] = {{&S102715[43], 10}, {NULL}};
const Segment SGI107359[] = {{&S102715[30], 4}, {NULL}};
const Segment SGI107360[] = {{&S102715[112], 3}, {NULL}};
const Segment SGI107361[] = {{&S102715[78], 5}, {NULL}};
const Segment SGI107362[] = {{&S102715[136], 13}, {NULL}};
const Segment SGI107363[] = {{&S87936[34], 11}, {NULL}};
const Segment SGI107364[] = {{&S87936[27], 6}, {NULL}};
const Segment SGI107365[] = {{&S87936[98], 9}, {NULL}};
const Segment SGI107366[] = {{&S87936[71], 6}, {NULL}};
const Segment SGI107367[] = {{&S87937[44], 21}, {NULL}};
const Segment SGI107368[] = {{&S87937[44], 33}, {NULL}};
const Segment SGI107369[] = {{&S87937[30], 6}, {NULL}};
const Segment SGI107370[] = {{&S87937[31], 3}, {NULL}};
const Segment SGI107371[] = {{&S87937[91], 6}, {NULL}};
const Segment SGI107372[] = {{&S87950[44], 6}, {NULL}};
const Segment SGI107373[] = {{&S87950[44], 18}, {NULL}};
const Segment SGI107374[] = {{&S87950[30], 6}, {NULL}};
const Segment SGI107375[] = {{&S87950[31], 3}, {NULL}};
const Segment SGI107376[] = {{&S87950[76], 6}, {NULL}};
const Segment SGI107377[] = {{&S87977[44], 11}, {NULL}};
const Segment SGI107378[] = {{&S87977[44], 23}, {NULL}};
const Segment SGI107379[] = {{&S87977[30], 6}, {NULL}};
const Segment SGI107380[] = {{&S87977[81], 6}, {NULL}};
const Segment SGI107381[] = {{&S87977[108], 13}, {NULL}};
const Segment SGI107382[] = {{&S88068[44], 13}, {NULL}};
const Segment SGI107383[] = {{&S88068[44], 25}, {NULL}};
const Segment SGI107384[] = {{&S88068[30], 6}, {NULL}};
const Segment SGI107385[] = {{&S88068[31], 3}, {NULL}};
const Segment SGI107386[] = {{&S88068[83], 6}, {NULL}};
const Segment SGI107387[] = {{&S88086[34], 9}, {NULL}};
const Segment SGI107388[] = {{&S88086[27], 6}, {NULL}};
const Segment SGI107389[] = {{&S88086[96], 9}, {NULL}};
const Segment SGI107390[] = {{&S88086[69], 6}, {NULL}};
const Segment SGI107391[] = {{&S88095[34], 16}, {NULL}};
const Segment SGI107392[] = {{&S88095[27], 6}, {NULL}};
const Segment SGI107393[] = {{&S88095[103], 9}, {NULL}};
const Segment SGI107394[] = {{&S88095[76], 6}, {NULL}};
const Segment SGI107395[] = {{&S88101[44], 7}, {NULL}};
const Segment SGI107396[] = {{&S88101[44], 19}, {NULL}};
const Segment SGI107397[] = {{&S88101[30], 6}, {NULL}};
const Segment SGI107398[] = {{&S88101[31], 3}, {NULL}};
const Segment SGI107399[] = {{&S88101[77], 6}, {NULL}};
const Segment SGI107400[] = {{&S88106[44], 14}, {NULL}};
const Segment SGI107401[] = {{&S88106[44], 31}, {NULL}};
const Segment SGI107402[] = {{&S88106[30], 6}, {NULL}};
const Segment SGI107403[] = {{&S88106[31], 3}, {NULL}};
const Segment SGI107404[] = {{&S88106[89], 5}, {NULL}};
const Segment SGI107405[] = {{&S88106[126], 14}, {NULL}};
const Segment SGI107406[] = {{&S102781[42], 7}, {NULL}};
const Segment SGI107407[] = {{&S102781[42], 19}, {NULL}};
const Segment SGI107408[] = {{&S102781[30], 4}, {NULL}};
const Segment SGI107409[] = {{&S102781[109], 3}, {NULL}};
const Segment SGI107410[] = {{&S102781[75], 5}, {NULL}};
const Segment SGI107411[] = {{&S102869[43], 10}, {NULL}};
const Segment SGI107412[] = {{&S102869[30], 4}, {NULL}};
const Segment SGI107413[] = {{&S102869[113], 3}, {NULL}};
const Segment SGI107414[] = {{&S102869[79], 5}, {NULL}};
const Segment SGI107415[] = {{&S102869[137], 13}, {NULL}};
const Segment SGI107416[] = {{&S105454[44], 7}, {NULL}};
const Segment SGI107417[] = {{&S105454[44], 25}, {NULL}};
const Segment SGI107418[] = {{&S105454[30], 6}, {NULL}};
const Segment SGI107419[] = {{&S105454[57], 3}, {NULL}};
const Segment SGI107420[] = {{&S105454[98], 5}, {NULL}};
const Segment SGI107421[] = {{&S105454[35], 1}, {NULL}};
const Segment SGI107422[] = {{&S105454[71], 11}, {NULL}};
const Segment SGI107423[] = {{&S105457[38], 7}, {NULL}};
const Segment SGI107424[] = {{&S105457[31], 6}, {NULL}};
const Segment SGI107425[] = {{&S105461[44], 7}, {NULL}};
const Segment SGI107426[] = {{&S105461[44], 18}, {NULL}};
const Segment SGI107427[] = {{&S105461[30], 6}, {NULL}};
const Segment SGI107428[] = {{&S105461[110], 3}, {NULL}};
const Segment SGI107429[] = {{&S105461[76], 5}, {NULL}};
const Segment SGI107430[] = {{&S105469[44], 21}, {NULL}};
const Segment SGI107431[] = {{&S105469[44], 33}, {NULL}};
const Segment SGI107432[] = {{&S105469[30], 6}, {NULL}};
const Segment SGI107433[] = {{&S105469[144], 3}, {NULL}};
const Segment SGI107434[] = {{&S105469[110], 5}, {NULL}};
const Segment SGI107435[] = {{&S105469[35], 1}, {NULL}};
const Segment SGI107436[] = {{&S105469[79], 11}, {NULL}};
const Segment SGI107437[] = {{&S102904[42], 10}, {NULL}};
const Segment SGI107438[] = {{&S102904[42], 22}, {NULL}};
const Segment SGI107439[] = {{&S102904[30], 4}, {NULL}};
const Segment SGI107440[] = {{&S102904[113], 3}, {NULL}};
const Segment SGI107441[] = {{&S102904[78], 6}, {NULL}};
const Segment SGI107442[] = {{&S5817[44], 9}, {NULL}};
const Segment SGI107443[] = {{&S5817[44], 21}, {NULL}};
const Segment SGI107444[] = {{&S5817[30], 6}, {NULL}};
const Segment SGI107445[] = {{&S5817[31], 3}, {NULL}};
const Segment SGI107446[] = {{&S5817[79], 6}, {NULL}};
const Segment SGI107447[] = {{&S102929[42], 10}, {NULL}};
const Segment SGI107448[] = {{&S102929[42], 21}, {NULL}};
const Segment SGI107449[] = {{&S102929[30], 4}, {NULL}};
const Segment SGI107450[] = {{&S102929[112], 3}, {NULL}};
const Segment SGI107451[] = {{&S102929[77], 6}, {NULL}};
const Segment SGI107452[] = {{&S102929[130], 11}, {NULL}};
const Segment SGI107453[] = {{&S5822[23], 15}, {NULL}};
const Segment SGI107454[] = {{&S5822[31], 19}, {NULL}};
const Segment SGI107455[] = {{&S5823[44], 10}, {NULL}};
const Segment SGI107456[] = {{&S5823[44], 22}, {NULL}};
const Segment SGI107457[] = {{&S5823[30], 6}, {NULL}};
const Segment SGI107458[] = {{&S5823[31], 3}, {NULL}};
const Segment SGI107459[] = {{&S5823[80], 6}, {NULL}};
const Segment SGI107460[] = {{&S105535[45], 14}, {NULL}};
const Segment SGI107461[] = {{&S105535[30], 6}, {NULL}};
const Segment SGI107462[] = {{&S105535[118], 3}, {NULL}};
const Segment SGI107463[] = {{&S105535[84], 5}, {NULL}};
const Segment SGI107464[] = {{&S105535[142], 13}, {NULL}};
const Segment SGI107465[] = {{&S5834[44], 11}, {NULL}};
const Segment SGI107466[] = {{&S5834[44], 23}, {NULL}};
const Segment SGI107467[] = {{&S5834[30], 6}, {NULL}};
const Segment SGI107468[] = {{&S5834[31], 3}, {NULL}};
const Segment SGI107469[] = {{&S5834[81], 6}, {NULL}};
const Segment SGI107470[] = {{&S5839[44], 20}, {NULL}};
const Segment SGI107471[] = {{&S5839[44], 32}, {NULL}};
const Segment SGI107472[] = {{&S5839[30], 6}, {NULL}};
const Segment SGI107473[] = {{&S5839[31], 3}, {NULL}};
const Segment SGI107474[] = {{&S5839[90], 6}, {NULL}};
const Segment SGI107475[] = {{&S5842[44], 6}, {NULL}};
const Segment SGI107476[] = {{&S5842[44], 18}, {NULL}};
const Segment SGI107477[] = {{&S5842[30], 6}, {NULL}};
const Segment SGI107478[] = {{&S5842[31], 3}, {NULL}};
const Segment SGI107479[] = {{&S5842[76], 6}, {NULL}};
const Segment SGI107480[] = {{&S5848[44], 18}, {NULL}};
const Segment SGI107481[] = {{&S5848[44], 30}, {NULL}};
const Segment SGI107482[] = {{&S5848[30], 6}, {NULL}};
const Segment SGI107483[] = {{&S5848[31], 3}, {NULL}};
const Segment SGI107484[] = {{&S5848[88], 6}, {NULL}};
const Segment SGI107485[] = {{&S5849[23], 15}, {NULL}};
const Segment SGI107486[] = {{&S5849[31], 19}, {NULL}};
const Segment SGI107487[] = {{&S5859[23], 15}, {NULL}};
const Segment SGI107488[] = {{&S5859[31], 19}, {NULL}};
const Segment SGI107489[] = {{&S5869[44], 6}, {NULL}};
const Segment SGI107490[] = {{&S5869[44], 18}, {NULL}};
const Segment SGI107491[] = {{&S5869[30], 6}, {NULL}};
const Segment SGI107492[] = {{&S5869[31], 3}, {NULL}};
const Segment SGI107493[] = {{&S5869[76], 6}, {NULL}};
const Segment SGI107494[] = {{&S5875[44], 7}, {NULL}};
const Segment SGI107495[] = {{&S105589[44], 10}, {NULL}};
const Segment SGI107496[] = {{&S5875[44], 19}, {NULL}};
const Segment SGI107497[] = {{&S5875[30], 6}, {NULL}};
const Segment SGI107498[] = {{&S105589[44], 21}, {NULL}};
const Segment SGI107499[] = {{&S105589[30], 6}, {NULL}};
const Segment SGI107500[] = {{&S5875[31], 3}, {NULL}};
const Segment SGI107501[] = {{&S105589[112], 3}, {NULL}};
const Segment SGI107502[] = {{&S5875[77], 6}, {NULL}};
const Segment SGI107503[] = {{&S105589[79], 5}, {NULL}};
const Segment SGI107504[] = {{&S105589[116], 14}, {NULL}};
const Segment SGI107505[] = {{&S5876[44], 6}, {NULL}};
const Segment SGI107506[] = {{&S5876[44], 18}, {NULL}};
const Segment SGI107507[] = {{&S5876[30], 6}, {NULL}};
const Segment SGI107508[] = {{&S5876[31], 3}, {NULL}};
const Segment SGI107509[] = {{&S5876[76], 6}, {NULL}};
const Segment SGI107510[] = {{&S5886[44], 6}, {NULL}};
const Segment SGI107511[] = {{&S5886[44], 18}, {NULL}};
const Segment SGI107512[] = {{&S5886[30], 6}, {NULL}};
const Segment SGI107513[] = {{&S5886[31], 3}, {NULL}};
const Segment SGI107514[] = {{&S5886[76], 6}, {NULL}};
const Segment SGI107515[] = {{&S5887[44], 9}, {NULL}};
const Segment SGI107516[] = {{&S5887[44], 21}, {NULL}};
const Segment SGI107517[] = {{&S5887[30], 6}, {NULL}};
const Segment SGI107518[] = {{&S5887[31], 3}, {NULL}};
const Segment SGI107519[] = {{&S5887[79], 6}, {NULL}};
const Segment SGI107520[] = {{&S88193[44], 7}, {NULL}};
const Segment SGI107521[] = {{&S88193[44], 19}, {NULL}};
const Segment SGI107522[] = {{&S88193[30], 6}, {NULL}};
const Segment SGI107523[] = {{&S5893[44], 14}, {NULL}};
const Segment SGI107524[] = {{&S5893[44], 26}, {NULL}};
const Segment SGI107525[] = {{&S88193[31], 3}, {NULL}};
const Segment SGI107526[] = {{&S5893[30], 6}, {NULL}};
const Segment SGI107527[] = {{&S88193[77], 6}, {NULL}};
const Segment SGI107528[] = {{&S5893[31], 3}, {NULL}};
const Segment SGI107529[] = {{&S5893[84], 6}, {NULL}};
const Segment SGI107530[] = {{&S88201[45], 10}, {NULL}};
const Segment SGI107531[] = {{&S88201[30], 6}, {NULL}};
const Segment SGI107532[] = {{&S88201[31], 3}, {NULL}};
const Segment SGI107533[] = {{&S88201[81], 5}, {NULL}};
const Segment SGI107534[] = {{&S88201[139], 13}, {NULL}};
const Segment SGI107535[] = {{&S5909[44], 7}, {NULL}};
const Segment SGI107536[] = {{&S5909[44], 19}, {NULL}};
const Segment SGI107537[] = {{&S5909[30], 6}, {NULL}};
const Segment SGI107538[] = {{&S5909[31], 3}, {NULL}};
const Segment SGI107539[] = {{&S5909[77], 6}, {NULL}};
const Segment SGI107540[] = {{&S88215[45], 9}, {NULL}};
const Segment SGI107541[] = {{&S88215[30], 6}, {NULL}};
const Segment SGI107542[] = {{&S88215[31], 3}, {NULL}};
const Segment SGI107543[] = {{&S88215[80], 5}, {NULL}};
const Segment SGI107544[] = {{&S88215[138], 13}, {NULL}};
const Segment SGI107545[] = {{&S105636[44], 10}, {NULL}};
const Segment SGI107546[] = {{&S105636[44], 22}, {NULL}};
const Segment SGI107547[] = {{&S105636[30], 6}, {NULL}};
const Segment SGI107548[] = {{&S105636[133], 3}, {NULL}};
const Segment SGI107549[] = {{&S105636[99], 5}, {NULL}};
const Segment SGI107550[] = {{&S105636[35], 1}, {NULL}};
const Segment SGI107551[] = {{&S105636[68], 11}, {NULL}};
const Segment SGI107552[] = {{&S5928[44], 18}, {NULL}};
const Segment SGI107553[] = {{&S5928[44], 30}, {NULL}};
const Segment SGI107554[] = {{&S5928[30], 6}, {NULL}};
const Segment SGI107555[] = {{&S5928[31], 3}, {NULL}};
const Segment SGI107556[] = {{&S5928[88], 6}, {NULL}};
const Segment SGI107557[] = {{&S103070[42], 15}, {NULL}};
const Segment SGI107558[] = {{&S103070[42], 30}, {NULL}};
const Segment SGI107559[] = {{&S103070[30], 4}, {NULL}};
const Segment SGI107560[] = {{&S103070[121], 3}, {NULL}};
const Segment SGI107561[] = {{&S103070[86], 6}, {NULL}};
const Segment SGI107562[] = {{&S5941[44], 10}, {NULL}};
const Segment SGI107563[] = {{&S5941[44], 22}, {NULL}};
const Segment SGI107564[] = {{&S5941[30], 6}, {NULL}};
const Segment SGI107565[] = {{&S5941[31], 3}, {NULL}};
const Segment SGI107566[] = {{&S5941[80], 6}, {NULL}};
const Segment SGI107567[] = {{&S88254[44], 4}, {NULL}};
const Segment SGI107568[] = {{&S88254[44], 23}, {NULL}};
const Segment SGI107569[] = {{&S88254[30], 6}, {NULL}};
const Segment SGI107570[] = {{&S88254[31], 3}, {NULL}};
const Segment SGI107571[] = {{&S88254[81], 6}, {NULL}};
const Segment SGI107572[] = {{&S103096[42], 9}, {NULL}};
const Segment SGI107573[] = {{&S103096[42], 20}, {NULL}};
const Segment SGI107574[] = {{&S103096[30], 4}, {NULL}};
const Segment SGI107575[] = {{&S103096[111], 3}, {NULL}};
const Segment SGI107576[] = {{&S103096[76], 6}, {NULL}};
const Segment SGI107577[] = {{&S5954[44], 15}, {NULL}};
const Segment SGI107578[] = {{&S5954[44], 27}, {NULL}};
const Segment SGI107579[] = {{&S5954[30], 6}, {NULL}};
const Segment SGI107580[] = {{&S5954[31], 3}, {NULL}};
const Segment SGI107581[] = {{&S5954[85], 6}, {NULL}};
const Segment SGI107582[] = {{&S5956[44], 13}, {NULL}};
const Segment SGI107583[] = {{&S5956[44], 25}, {NULL}};
const Segment SGI107584[] = {{&S5956[30], 6}, {NULL}};
const Segment SGI107585[] = {{&S5956[31], 3}, {NULL}};
const Segment SGI107586[] = {{&S5956[83], 6}, {NULL}};
const Segment SGI107587[] = {{&S5963[44], 12}, {NULL}};
const Segment SGI107588[] = {{&S5963[44], 24}, {NULL}};
const Segment SGI107589[] = {{&S5963[30], 6}, {NULL}};
const Segment SGI107590[] = {{&S5963[31], 3}, {NULL}};
const Segment SGI107591[] = {{&S5963[82], 6}, {NULL}};
const Segment SGI107592[] = {{&S5964[44], 18}, {NULL}};
const Segment SGI107593[] = {{&S5964[44], 30}, {NULL}};
const Segment SGI107594[] = {{&S5964[30], 6}, {NULL}};
const Segment SGI107595[] = {{&S5964[31], 3}, {NULL}};
const Segment SGI107596[] = {{&S5964[88], 6}, {NULL}};
const Segment SGI107597[] = {{&S88273[27], 6}, {NULL}};
const Segment SGI107598[] = {{&S88273[74], 9}, {NULL}};
const Segment SGI107599[] = {{&S88273[47], 6}, {NULL}};
const Segment SGI107600[] = {{&S5966[44], 8}, {NULL}};
const Segment SGI107601[] = {{&S5966[44], 20}, {NULL}};
const Segment SGI107602[] = {{&S5966[30], 6}, {NULL}};
const Segment SGI107603[] = {{&S5966[31], 3}, {NULL}};
const Segment SGI107604[] = {{&S5966[78], 6}, {NULL}};
const Segment SGI107605[] = {{&S5968[44], 11}, {NULL}};
const Segment SGI107606[] = {{&S5968[44], 23}, {NULL}};
const Segment SGI107607[] = {{&S5968[30], 6}, {NULL}};
const Segment SGI107608[] = {{&S5968[31], 3}, {NULL}};
const Segment SGI107609[] = {{&S5968[81], 6}, {NULL}};
const Segment SGI107610[] = {{&S5969[44], 8}, {NULL}};
const Segment SGI107611[] = {{&S5969[44], 20}, {NULL}};
const Segment SGI107612[] = {{&S5969[30], 6}, {NULL}};
const Segment SGI107613[] = {{&S5969[31], 3}, {NULL}};
const Segment SGI107614[] = {{&S5969[78], 6}, {NULL}};
const Segment SGI107615[] = {{&S88283[34], 6}, {NULL}};
const Segment SGI107616[] = {{&S88283[27], 6}, {NULL}};
const Segment SGI107617[] = {{&S88283[93], 9}, {NULL}};
const Segment SGI107618[] = {{&S88283[66], 6}, {NULL}};
const Segment SGI107619[] = {{&S5975[44], 7}, {NULL}};
const Segment SGI107620[] = {{&S5975[44], 19}, {NULL}};
const Segment SGI107621[] = {{&S5975[30], 6}, {NULL}};
const Segment SGI107622[] = {{&S5975[31], 3}, {NULL}};
const Segment SGI107623[] = {{&S5975[77], 6}, {NULL}};
const Segment SGI107624[] = {{&S5978[44], 11}, {NULL}};
const Segment SGI107625[] = {{&S5978[44], 23}, {NULL}};
const Segment SGI107626[] = {{&S5978[30], 6}, {NULL}};
const Segment SGI107627[] = {{&S5978[31], 3}, {NULL}};
const Segment SGI107628[] = {{&S5978[81], 6}, {NULL}};
const Segment SGI107629[] = {{&S103132[43], 10}, {NULL}};
const Segment SGI107630[] = {{&S103132[30], 4}, {NULL}};
const Segment SGI107631[] = {{&S103132[112], 3}, {NULL}};
const Segment SGI107632[] = {{&S103132[78], 5}, {NULL}};
const Segment SGI107633[] = {{&S103132[136], 13}, {NULL}};
const Segment SGI107634[] = {{&S5979[44], 7}, {NULL}};
const Segment SGI107635[] = {{&S5979[44], 17}, {NULL}};
const Segment SGI107636[] = {{&S5979[30], 6}, {NULL}};
const Segment SGI107637[] = {{&S5979[31], 3}, {NULL}};
const Segment SGI107638[] = {{&S5979[75], 6}, {NULL}};
const Segment SGI107639[] = {{&S103133[43], 10}, {NULL}};
const Segment SGI107640[] = {{&S103133[30], 4}, {NULL}};
const Segment SGI107641[] = {{&S103133[112], 3}, {NULL}};
const Segment SGI107642[] = {{&S103133[78], 5}, {NULL}};
const Segment SGI107643[] = {{&S103133[136], 13}, {NULL}};
const Segment SGI107644[] = {{&S5981[44], 9}, {NULL}};
const Segment SGI107645[] = {{&S5981[44], 21}, {NULL}};
const Segment SGI107646[] = {{&S5981[30], 6}, {NULL}};
const Segment SGI107647[] = {{&S5981[31], 3}, {NULL}};
const Segment SGI107648[] = {{&S5981[79], 6}, {NULL}};
const Segment SGI107649[] = {{&S103135[42], 10}, {NULL}};
const Segment SGI107650[] = {{&S103135[42], 21}, {NULL}};
const Segment SGI107651[] = {{&S103135[30], 4}, {NULL}};
const Segment SGI107652[] = {{&S103135[119], 3}, {NULL}};
const Segment SGI107653[] = {{&S103135[84], 6}, {NULL}};
const Segment SGI107654[] = {{&S103135[63], 8}, {NULL}};
const Segment SGI107655[] = {{&S5982[44], 17}, {NULL}};
const Segment SGI107656[] = {{&S5982[30], 6}, {NULL}};
const Segment SGI107657[] = {{&S5982[133], 3}, {NULL}};
const Segment SGI107658[] = {{&S5982[98], 6}, {NULL}};
const Segment SGI107659[] = {{&S103136[40], 10}, {NULL}};
const Segment SGI107660[] = {{&S5982[153], 8}, {NULL}};
const Segment SGI107661[] = {{&S103136[30], 4}, {NULL}};
const Segment SGI107662[] = {{&S103136[117], 3}, {NULL}};
const Segment SGI107663[] = {{&S103136[82], 6}, {NULL}};
const Segment SGI107664[] = {{&S103136[61], 8}, {NULL}};
const Segment SGI107665[] = {{&S103138[43], 10}, {NULL}};
const Segment SGI107666[] = {{&S103138[30], 4}, {NULL}};
const Segment SGI107667[] = {{&S103138[113], 3}, {NULL}};
const Segment SGI107668[] = {{&S103138[79], 5}, {NULL}};
const Segment SGI107669[] = {{&S103138[137], 13}, {NULL}};
const Segment SGI107670[] = {{&S5986[44], 10}, {NULL}};
const Segment SGI107671[] = {{&S5986[44], 22}, {NULL}};
const Segment SGI107672[] = {{&S5986[30], 6}, {NULL}};
const Segment SGI107673[] = {{&S5986[31], 3}, {NULL}};
const Segment SGI107674[] = {{&S5986[80], 6}, {NULL}};
const Segment SGI107675[] = {{&S103139[42], 10}, {NULL}};
const Segment SGI107676[] = {{&S103139[42], 22}, {NULL}};
const Segment SGI107677[] = {{&S103139[30], 4}, {NULL}};
const Segment SGI107678[] = {{&S103139[113], 3}, {NULL}};
const Segment SGI107679[] = {{&S103139[78], 6}, {NULL}};
const Segment SGI107680[] = {{&S103139[133], 8}, {NULL}};
const Segment SGI107681[] = {{&S103139[142], 13}, {NULL}};
const Segment SGI107682[] = {{&S88315[44], 9}, {NULL}};
const Segment SGI107683[] = {{&S88315[44], 35}, {NULL}};
const Segment SGI107684[] = {{&S88315[30], 6}, {NULL}};
const Segment SGI107685[] = {{&S88315[31], 3}, {NULL}};
const Segment SGI107686[] = {{&S88315[93], 6}, {NULL}};
const Segment SGI107687[] = {{&S6000[44], 18}, {NULL}};
const Segment SGI107688[] = {{&S6000[44], 30}, {NULL}};
const Segment SGI107689[] = {{&S6000[30], 6}, {NULL}};
const Segment SGI107690[] = {{&S88319[44], 10}, {NULL}};
const Segment SGI107691[] = {{&S88319[44], 27}, {NULL}};
const Segment SGI107692[] = {{&S88319[30], 6}, {NULL}};
const Segment SGI107693[] = {{&S6000[31], 3}, {NULL}};
const Segment SGI107694[] = {{&S103151[57], 10}, {NULL}};
const Segment SGI107695[] = {{&S6000[88], 6}, {NULL}};
const Segment SGI107696[] = {{&S103151[57], 22}, {NULL}};
const Segment SGI107697[] = {{&S103151[45], 4}, {NULL}};
const Segment SGI107698[] = {{&S88319[31], 3}, {NULL}};
const Segment SGI107699[] = {{&S88319[85], 5}, {NULL}};
const Segment SGI107700[] = {{&S103151[128], 3}, {NULL}};
const Segment SGI107701[] = {{&S103151[93], 6}, {NULL}};
const Segment SGI107702[] = {{&S103151[0], 14}, {NULL}};
const Segment SGI107703[] = {{&S103152[57], 10}, {NULL}};
const Segment SGI107704[] = {{&S103152[57], 22}, {NULL}};
const Segment SGI107705[] = {{&S103152[45], 4}, {NULL}};
const Segment SGI107706[] = {{&S103152[128], 3}, {NULL}};
const Segment SGI107707[] = {{&S103152[93], 6}, {NULL}};
const Segment SGI107708[] = {{&S103152[0], 14}, {NULL}};
const Segment SGI107709[] = {{&S103153[57], 10}, {NULL}};
const Segment SGI107710[] = {{&S103153[57], 21}, {NULL}};
const Segment SGI107711[] = {{&S103153[45], 4}, {NULL}};
const Segment SGI107712[] = {{&S103153[127], 3}, {NULL}};
const Segment SGI107713[] = {{&S6004[44], 8}, {NULL}};
const Segment SGI107714[] = {{&S6004[44], 20}, {NULL}};
const Segment SGI107715[] = {{&S103153[92], 6}, {NULL}};
const Segment SGI107716[] = {{&S6004[30], 6}, {NULL}};
const Segment SGI107717[] = {{&S103153[0], 14}, {NULL}};
const Segment SGI107718[] = {{&S6004[31], 3}, {NULL}};
const Segment SGI107719[] = {{&S6004[78], 6}, {NULL}};
const Segment SGI107720[] = {{&S88324[34], 9}, {NULL}};
const Segment SGI107721[] = {{&S88324[27], 6}, {NULL}};
const Segment SGI107722[] = {{&S88324[96], 9}, {NULL}};
const Segment SGI107723[] = {{&S88324[69], 6}, {NULL}};
const Segment SGI107724[] = {{&S105732[45], 10}, {NULL}};
const Segment SGI107725[] = {{&S105732[30], 6}, {NULL}};
const Segment SGI107726[] = {{&S6006[44], 7}, {NULL}};
const Segment SGI107727[] = {{&S6006[44], 19}, {NULL}};
const Segment SGI107728[] = {{&S6006[30], 6}, {NULL}};
const Segment SGI107729[] = {{&S105732[116], 3}, {NULL}};
const Segment SGI107730[] = {{&S6006[31], 3}, {NULL}};
const Segment SGI107731[] = {{&S105732[82], 5}, {NULL}};
const Segment SGI107732[] = {{&S6006[77], 6}, {NULL}};
const Segment SGI107733[] = {{&S105732[140], 13}, {NULL}};
const Segment SGI107734[] = {{&S6008[44], 9}, {NULL}};
const Segment SGI107735[] = {{&S6008[44], 21}, {NULL}};
const Segment SGI107736[] = {{&S6008[30], 6}, {NULL}};
const Segment SGI107737[] = {{&S6008[31], 3}, {NULL}};
const Segment SGI107738[] = {{&S6008[79], 6}, {NULL}};
const Segment SGI107739[] = {{&S103161[42], 10}, {NULL}};
const Segment SGI107740[] = {{&S103161[42], 21}, {NULL}};
const Segment SGI107741[] = {{&S103161[30], 4}, {NULL}};
const Segment SGI107742[] = {{&S6009[44], 15}, {NULL}};
const Segment SGI107743[] = {{&S6009[44], 27}, {NULL}};
const Segment SGI107744[] = {{&S6009[30], 6}, {NULL}};
const Segment SGI107745[] = {{&S103161[112], 3}, {NULL}};
const Segment SGI107746[] = {{&S6009[31], 3}, {NULL}};
const Segment SGI107747[] = {{&S103161[77], 6}, {NULL}};
const Segment SGI107748[] = {{&S6009[85], 6}, {NULL}};
const Segment SGI107749[] = {{&S103161[129], 10}, {NULL}};
const Segment SGI107750[] = {{&S103162[42], 11}, {NULL}};
const Segment SGI107751[] = {{&S103162[42], 22}, {NULL}};
const Segment SGI107752[] = {{&S103162[30], 4}, {NULL}};
const Segment SGI107753[] = {{&S103162[113], 3}, {NULL}};
const Segment SGI107754[] = {{&S103162[78], 6}, {NULL}};
const Segment SGI107755[] = {{&S103162[130], 10}, {NULL}};
const Segment SGI107756[] = {{&S6012[44], 5}, {NULL}};
const Segment SGI107757[] = {{&S6012[44], 17}, {NULL}};
const Segment SGI107758[] = {{&S6012[30], 6}, {NULL}};
const Segment SGI107759[] = {{&S103163[42], 11}, {NULL}};
const Segment SGI107760[] = {{&S103163[42], 22}, {NULL}};
const Segment SGI107761[] = {{&S103163[30], 4}, {NULL}};
const Segment SGI107762[] = {{&S6012[31], 3}, {NULL}};
const Segment SGI107763[] = {{&S6012[75], 6}, {NULL}};
const Segment SGI107764[] = {{&S103163[113], 3}, {NULL}};
const Segment SGI107765[] = {{&S103163[78], 6}, {NULL}};
const Segment SGI107766[] = {{&S103163[130], 10}, {NULL}};
const Segment SGI107767[] = {{&S88337[44], 6}, {NULL}};
const Segment SGI107768[] = {{&S88337[44], 28}, {NULL}};
const Segment SGI107769[] = {{&S88337[30], 6}, {NULL}};
const Segment SGI107770[] = {{&S88337[31], 3}, {NULL}};
const Segment SGI107771[] = {{&S88337[86], 6}, {NULL}};
const Segment SGI107772[] = {{&S6015[44], 12}, {NULL}};
const Segment SGI107773[] = {{&S6015[44], 24}, {NULL}};
const Segment SGI107774[] = {{&S6015[30], 6}, {NULL}};
const Segment SGI107775[] = {{&S6015[31], 3}, {NULL}};
const Segment SGI107776[] = {{&S88338[44], 6}, {NULL}};
const Segment SGI107777[] = {{&S6015[82], 6}, {NULL}};
const Segment SGI107778[] = {{&S88338[44], 15}, {NULL}};
const Segment SGI107779[] = {{&S88338[30], 6}, {NULL}};
const Segment SGI107780[] = {{&S88338[31], 3}, {NULL}};
const Segment SGI107781[] = {{&S88338[73], 6}, {NULL}};
const Segment SGI107782[] = {{&S103170[42], 10}, {NULL}};
const Segment SGI107783[] = {{&S103170[42], 22}, {NULL}};
const Segment SGI107784[] = {{&S103170[30], 4}, {NULL}};
const Segment SGI107785[] = {{&S103170[113], 3}, {NULL}};
const Segment SGI107786[] = {{&S6019[44], 8}, {NULL}};
const Segment SGI107787[] = {{&S6019[44], 20}, {NULL}};
const Segment SGI107788[] = {{&S103170[78], 6}, {NULL}};
const Segment SGI107789[] = {{&S6019[30], 6}, {NULL}};
const Segment SGI107790[] = {{&S103170[130], 10}, {NULL}};
const Segment SGI107791[] = {{&S6019[31], 3}, {NULL}};
const Segment SGI107792[] = {{&S6019[78], 6}, {NULL}};
const Segment SGI107793[] = {{&S103171[42], 11}, {NULL}};
const Segment SGI107794[] = {{&S103171[42], 23}, {NULL}};
const Segment SGI107795[] = {{&S103171[30], 4}, {NULL}};
const Segment SGI107796[] = {{&S103171[114], 3}, {NULL}};
const Segment SGI107797[] = {{&S103171[79], 6}, {NULL}};
const Segment SGI107798[] = {{&S103171[131], 10}, {NULL}};
const Segment SGI107799[] = {{&S88345[34], 11}, {NULL}};
const Segment SGI107800[] = {{&S88345[27], 6}, {NULL}};
const Segment SGI107801[] = {{&S103172[57], 11}, {NULL}};
const Segment SGI107802[] = {{&S103172[57], 23}, {NULL}};
const Segment SGI107803[] = {{&S103172[45], 4}, {NULL}};
const Segment SGI107804[] = {{&S88345[98], 9}, {NULL}};
const Segment SGI107805[] = {{&S88345[71], 6}, {NULL}};
const Segment SGI107806[] = {{&S103172[129], 3}, {NULL}};
const Segment SGI107807[] = {{&S103172[94], 6}, {NULL}};
const Segment SGI107808[] = {{&S103172[0], 14}, {NULL}};
const Segment SGI107809[] = {{&S103173[57], 10}, {NULL}};
const Segment SGI107810[] = {{&S103173[57], 22}, {NULL}};
const Segment SGI107811[] = {{&S103173[45], 4}, {NULL}};
const Segment SGI107812[] = {{&S103173[128], 3}, {NULL}};
const Segment SGI107813[] = {{&S103173[93], 6}, {NULL}};
const Segment SGI107814[] = {{&S103173[0], 14}, {NULL}};
const Segment SGI107815[] = {{&S6025[44], 5}, {NULL}};
const Segment SGI107816[] = {{&S6025[44], 17}, {NULL}};
const Segment SGI107817[] = {{&S6025[30], 6}, {NULL}};
const Segment SGI107818[] = {{&S6025[31], 3}, {NULL}};
const Segment SGI107819[] = {{&S6025[75], 6}, {NULL}};
const Segment SGI107820[] = {{&S103176[42], 10}, {NULL}};
const Segment SGI107821[] = {{&S103176[42], 21}, {NULL}};
const Segment SGI107822[] = {{&S103176[30], 4}, {NULL}};
const Segment SGI107823[] = {{&S88352[44], 10}, {NULL}};
const Segment SGI107824[] = {{&S103176[112], 3}, {NULL}};
const Segment SGI107825[] = {{&S88352[44], 22}, {NULL}};
const Segment SGI107826[] = {{&S88352[30], 6}, {NULL}};
const Segment SGI107827[] = {{&S103176[77], 6}, {NULL}};
const Segment SGI107828[] = {{&S103176[130], 13}, {NULL}};
const Segment SGI107829[] = {{&S88352[31], 3}, {NULL}};
const Segment SGI107830[] = {{&S88352[80], 5}, {NULL}};
const Segment SGI107831[] = {{&S103177[42], 10}, {NULL}};
const Segment SGI107832[] = {{&S103177[42], 21}, {NULL}};
const Segment SGI107833[] = {{&S103177[30], 4}, {NULL}};
const Segment SGI107834[] = {{&S103177[112], 3}, {NULL}};
const Segment SGI107835[] = {{&S103177[77], 6}, {NULL}};
const Segment SGI107836[] = {{&S103177[137], 13}, {NULL}};
const Segment SGI107837[] = {{&S105761[44], 10}, {NULL}};
const Segment SGI107838[] = {{&S105761[44], 23}, {NULL}};
const Segment SGI107839[] = {{&S105761[30], 6}, {NULL}};
const Segment SGI107840[] = {{&S105761[115], 3}, {NULL}};
const Segment SGI107841[] = {{&S105761[81], 5}, {NULL}};
const Segment SGI107842[] = {{&S6032[44], 13}, {NULL}};
const Segment SGI107843[] = {{&S105761[139], 11}, {NULL}};
const Segment SGI107844[] = {{&S6032[44], 25}, {NULL}};
const Segment SGI107845[] = {{&S6032[30], 6}, {NULL}};
const Segment SGI107846[] = {{&S6032[31], 3}, {NULL}};
const Segment SGI107847[] = {{&S6032[83], 6}, {NULL}};
const Segment SGI107848[] = {{&S6033[44], 8}, {NULL}};
const Segment SGI107849[] = {{&S6033[44], 20}, {NULL}};
const Segment SGI107850[] = {{&S6033[30], 6}, {NULL}};
const Segment SGI107851[] = {{&S6033[31], 3}, {NULL}};
const Segment SGI107852[] = {{&S6033[78], 6}, {NULL}};
const Segment SGI107853[] = {{&S6034[44], 11}, {NULL}};
const Segment SGI107854[] = {{&S6034[44], 23}, {NULL}};
const Segment SGI107855[] = {{&S6034[30], 6}, {NULL}};
const Segment SGI107856[] = {{&S6034[31], 3}, {NULL}};
const Segment SGI107857[] = {{&S6034[81], 6}, {NULL}};
const Segment SGI107858[] = {{&S88363[34], 9}, {NULL}};
const Segment SGI107859[] = {{&S88363[27], 6}, {NULL}};
const Segment SGI107860[] = {{&S88363[96], 9}, {NULL}};
const Segment SGI107861[] = {{&S88363[69], 6}, {NULL}};
const Segment SGI107862[] = {{&S103187[43], 10}, {NULL}};
const Segment SGI107863[] = {{&S103187[30], 4}, {NULL}};
const Segment SGI107864[] = {{&S103187[113], 3}, {NULL}};
const Segment SGI107865[] = {{&S103187[79], 5}, {NULL}};
const Segment SGI107866[] = {{&S103187[137], 13}, {NULL}};
const Segment SGI107867[] = {{&S88365[44], 6}, {NULL}};
const Segment SGI107868[] = {{&S88365[44], 18}, {NULL}};
const Segment SGI107869[] = {{&S88365[30], 6}, {NULL}};
const Segment SGI107870[] = {{&S88365[31], 3}, {NULL}};
const Segment SGI107871[] = {{&S88365[76], 6}, {NULL}};
const Segment SGI107872[] = {{&S88365[130], 12}, {NULL}};
const Segment SGI107873[] = {{&S6040[44], 9}, {NULL}};
const Segment SGI107874[] = {{&S6040[44], 21}, {NULL}};
const Segment SGI107875[] = {{&S103189[43], 10}, {NULL}};
const Segment SGI107876[] = {{&S6040[30], 6}, {NULL}};
const Segment SGI107877[] = {{&S103189[30], 4}, {NULL}};
const Segment SGI107878[] = {{&S6040[31], 3}, {NULL}};
const Segment SGI107879[] = {{&S103189[112], 3}, {NULL}};
const Segment SGI107880[] = {{&S6040[79], 6}, {NULL}};
const Segment SGI107881[] = {{&S103189[78], 5}, {NULL}};
const Segment SGI107882[] = {{&S88366[44], 6}, {NULL}};
const Segment SGI107883[] = {{&S88366[44], 18}, {NULL}};
const Segment SGI107884[] = {{&S103189[136], 13}, {NULL}};
const Segment SGI107885[] = {{&S88366[30], 6}, {NULL}};
const Segment SGI107886[] = {{&S88366[31], 3}, {NULL}};
const Segment SGI107887[] = {{&S88366[76], 6}, {NULL}};
const Segment SGI107888[] = {{&S88366[129], 11}, {NULL}};
const Segment SGI107889[] = {{&S6041[44], 11}, {NULL}};
const Segment SGI107890[] = {{&S6041[44], 23}, {NULL}};
const Segment SGI107891[] = {{&S6041[30], 6}, {NULL}};
const Segment SGI107892[] = {{&S6041[31], 3}, {NULL}};
const Segment SGI107893[] = {{&S6041[81], 6}, {NULL}};
const Segment SGI107894[] = {{&S88368[44], 10}, {NULL}};
const Segment SGI107895[] = {{&S6042[44], 10}, {NULL}};
const Segment SGI107896[] = {{&S88368[44], 22}, {NULL}};
const Segment SGI107897[] = {{&S6042[44], 22}, {NULL}};
const Segment SGI107898[] = {{&S88368[30], 6}, {NULL}};
const Segment SGI107899[] = {{&S6042[30], 6}, {NULL}};
const Segment SGI107900[] = {{&S6042[31], 3}, {NULL}};
const Segment SGI107901[] = {{&S6042[80], 6}, {NULL}};
const Segment SGI107902[] = {{&S88368[80], 6}, {NULL}};
const Segment SGI107903[] = {{&S88368[107], 13}, {NULL}};
const Segment SGI107904[] = {{&S6043[44], 15}, {NULL}};
const Segment SGI107905[] = {{&S6043[44], 27}, {NULL}};
const Segment SGI107906[] = {{&S6043[30], 6}, {NULL}};
const Segment SGI107907[] = {{&S6043[31], 3}, {NULL}};
const Segment SGI107908[] = {{&S6043[85], 6}, {NULL}};
const Segment SGI107909[] = {{&S6044[44], 14}, {NULL}};
const Segment SGI107910[] = {{&S6044[44], 26}, {NULL}};
const Segment SGI107911[] = {{&S6044[30], 6}, {NULL}};
const Segment SGI107912[] = {{&S6044[31], 3}, {NULL}};
const Segment SGI107913[] = {{&S6044[84], 6}, {NULL}};
const Segment SGI107914[] = {{&S88375[44], 9}, {NULL}};
const Segment SGI107915[] = {{&S88375[44], 21}, {NULL}};
const Segment SGI107916[] = {{&S88375[30], 6}, {NULL}};
const Segment SGI107917[] = {{&S88375[79], 6}, {NULL}};
const Segment SGI107918[] = {{&S88375[106], 13}, {NULL}};
const Segment SGI107919[] = {{&S6049[44], 17}, {NULL}};
const Segment SGI107920[] = {{&S6049[44], 29}, {NULL}};
const Segment SGI107921[] = {{&S6049[30], 6}, {NULL}};
const Segment SGI107922[] = {{&S6049[31], 3}, {NULL}};
const Segment SGI107923[] = {{&S6049[87], 6}, {NULL}};
const Segment SGI107924[] = {{&S6052[44], 16}, {NULL}};
const Segment SGI107925[] = {{&S6052[44], 28}, {NULL}};
const Segment SGI107926[] = {{&S6052[30], 6}, {NULL}};
const Segment SGI107927[] = {{&S6052[31], 3}, {NULL}};
const Segment SGI107928[] = {{&S6052[86], 6}, {NULL}};
const Segment SGI107929[] = {{&S6053[44], 6}, {NULL}};
const Segment SGI107930[] = {{&S6053[44], 18}, {NULL}};
const Segment SGI107931[] = {{&S6053[30], 6}, {NULL}};
const Segment SGI107932[] = {{&S6053[31], 3}, {NULL}};
const Segment SGI107933[] = {{&S6053[76], 6}, {NULL}};
const Segment SGI107934[] = {{&S6056[44], 12}, {NULL}};
const Segment SGI107935[] = {{&S6056[44], 24}, {NULL}};
const Segment SGI107936[] = {{&S6056[30], 6}, {NULL}};
const Segment SGI107937[] = {{&S6056[31], 3}, {NULL}};
const Segment SGI107938[] = {{&S6056[82], 6}, {NULL}};
const Segment SGI107939[] = {{&S88388[52], 11}, {NULL}};
const Segment SGI107940[] = {{&S88388[52], 23}, {NULL}};
const Segment SGI107941[] = {{&S88388[38], 6}, {NULL}};
const Segment SGI107942[] = {{&S88388[89], 5}, {NULL}};
const Segment SGI107943[] = {{&S88388[0], 14}, {NULL}};
const Segment SGI107944[] = {{&S6057[44], 10}, {NULL}};
const Segment SGI107945[] = {{&S6057[44], 22}, {NULL}};
const Segment SGI107946[] = {{&S6057[30], 6}, {NULL}};
const Segment SGI107947[] = {{&S6057[31], 3}, {NULL}};
const Segment SGI107948[] = {{&S6057[80], 6}, {NULL}};
const Segment SGI107949[] = {{&S6058[44], 10}, {NULL}};
const Segment SGI107950[] = {{&S6058[44], 22}, {NULL}};
const Segment SGI107951[] = {{&S6058[30], 6}, {NULL}};
const Segment SGI107952[] = {{&S6058[31], 3}, {NULL}};
const Segment SGI107953[] = {{&S6058[80], 6}, {NULL}};
const Segment SGI107954[] = {{&S6059[44], 8}, {NULL}};
const Segment SGI107955[] = {{&S6059[44], 20}, {NULL}};
const Segment SGI107956[] = {{&S6059[30], 6}, {NULL}};
const Segment SGI107957[] = {{&S6059[31], 3}, {NULL}};
const Segment SGI107958[] = {{&S6059[78], 6}, {NULL}};
const Segment SGI107959[] = {{&S6060[44], 5}, {NULL}};
const Segment SGI107960[] = {{&S6060[44], 17}, {NULL}};
const Segment SGI107961[] = {{&S6060[30], 6}, {NULL}};
const Segment SGI107962[] = {{&S6060[31], 3}, {NULL}};
const Segment SGI107963[] = {{&S6060[75], 6}, {NULL}};
const Segment SGI107964[] = {{&S6061[44], 19}, {NULL}};
const Segment SGI107965[] = {{&S6061[44], 31}, {NULL}};
const Segment SGI107966[] = {{&S6061[30], 6}, {NULL}};
const Segment SGI107967[] = {{&S6061[31], 3}, {NULL}};
const Segment SGI107968[] = {{&S6061[89], 6}, {NULL}};
const Segment SGI107969[] = {{&S6062[44], 12}, {NULL}};
const Segment SGI107970[] = {{&S6062[44], 24}, {NULL}};
const Segment SGI107971[] = {{&S6062[30], 6}, {NULL}};
const Segment SGI107972[] = {{&S6062[31], 3}, {NULL}};
const Segment SGI107973[] = {{&S6062[82], 6}, {NULL}};
const Segment SGI107974[] = {{&S6069[44], 9}, {NULL}};
const Segment SGI107975[] = {{&S6069[44], 21}, {NULL}};
const Segment SGI107976[] = {{&S6069[30], 6}, {NULL}};
const Segment SGI107977[] = {{&S6069[31], 3}, {NULL}};
const Segment SGI107978[] = {{&S6069[79], 6}, {NULL}};
const Segment SGI107979[] = {{&S6073[44], 10}, {NULL}};
const Segment SGI107980[] = {{&S6073[44], 22}, {NULL}};
const Segment SGI107981[] = {{&S6073[30], 6}, {NULL}};
const Segment SGI107982[] = {{&S6073[31], 3}, {NULL}};
const Segment SGI107983[] = {{&S6073[80], 6}, {NULL}};
const Segment SGI107984[] = {{&S6074[44], 6}, {NULL}};
const Segment SGI107985[] = {{&S6074[44], 18}, {NULL}};
const Segment SGI107986[] = {{&S6074[30], 6}, {NULL}};
const Segment SGI107987[] = {{&S6074[31], 3}, {NULL}};
const Segment SGI107988[] = {{&S6074[76], 6}, {NULL}};
const Segment SGI107989[] = {{&S6075[44], 7}, {NULL}};
const Segment SGI107990[] = {{&S6075[44], 19}, {NULL}};
const Segment SGI107991[] = {{&S6075[30], 6}, {NULL}};
const Segment SGI107992[] = {{&S6075[31], 3}, {NULL}};
const Segment SGI107993[] = {{&S6075[77], 6}, {NULL}};
const Segment SGI107994[] = {{&S6076[44], 8}, {NULL}};
const Segment SGI107995[] = {{&S6076[44], 20}, {NULL}};
const Segment SGI107996[] = {{&S6076[30], 6}, {NULL}};
const Segment SGI107997[] = {{&S6076[31], 3}, {NULL}};
const Segment SGI107998[] = {{&S6076[78], 6}, {NULL}};
const Segment SGI107999[] = {{&S6077[44], 10}, {NULL}};
const Segment SGI108000[] = {{&S6077[44], 22}, {NULL}};
const Segment SGI108001[] = {{&S6077[30], 6}, {NULL}};
const Segment SGI108002[] = {{&S6077[31], 3}, {NULL}};
const Segment SGI108003[] = {{&S6077[80], 6}, {NULL}};
const Segment SGI108004[] = {{&S6078[44], 9}, {NULL}};
const Segment SGI108005[] = {{&S6078[44], 21}, {NULL}};
const Segment SGI108006[] = {{&S6078[30], 6}, {NULL}};
const Segment SGI108007[] = {{&S6078[31], 3}, {NULL}};
const Segment SGI108008[] = {{&S6078[79], 6}, {NULL}};
const Segment SGI108009[] = {{&S6080[44], 8}, {NULL}};
const Segment SGI108010[] = {{&S6080[44], 20}, {NULL}};
const Segment SGI108011[] = {{&S6080[30], 6}, {NULL}};
const Segment SGI108012[] = {{&S6080[31], 3}, {NULL}};
const Segment SGI108013[] = {{&S6080[78], 6}, {NULL}};
const Segment SGI108014[] = {{&S6082[44], 6}, {NULL}};
const Segment SGI108015[] = {{&S6082[44], 18}, {NULL}};
const Segment SGI108016[] = {{&S6082[30], 6}, {NULL}};
const Segment SGI108017[] = {{&S6082[31], 3}, {NULL}};
const Segment SGI108018[] = {{&S6082[76], 6}, {NULL}};
const Segment SGI108019[] = {{&S88435[34], 12}, {NULL}};
const Segment SGI108020[] = {{&S88435[27], 6}, {NULL}};
const Segment SGI108021[] = {{&S88435[99], 9}, {NULL}};
const Segment SGI108022[] = {{&S88435[72], 6}, {NULL}};
const Segment SGI108023[] = {{&S6086[44], 6}, {NULL}};
const Segment SGI108024[] = {{&S6086[44], 18}, {NULL}};
const Segment SGI108025[] = {{&S6086[30], 6}, {NULL}};
const Segment SGI108026[] = {{&S6086[31], 3}, {NULL}};
const Segment SGI108027[] = {{&S6086[76], 6}, {NULL}};
const Segment SGI108028[] = {{&S6088[44], 11}, {NULL}};
const Segment SGI108029[] = {{&S6088[44], 23}, {NULL}};
const Segment SGI108030[] = {{&S6088[30], 6}, {NULL}};
const Segment SGI108031[] = {{&S6088[31], 3}, {NULL}};
const Segment SGI108032[] = {{&S6088[81], 6}, {NULL}};
const Segment SGI108033[] = {{&S88443[34], 8}, {NULL}};
const Segment SGI108034[] = {{&S88443[27], 6}, {NULL}};
const Segment SGI108035[] = {{&S88443[96], 9}, {NULL}};
const Segment SGI108036[] = {{&S88443[69], 6}, {NULL}};
const Segment SGI108037[] = {{&S6096[44], 13}, {NULL}};
const Segment SGI108038[] = {{&S6096[44], 25}, {NULL}};
const Segment SGI108039[] = {{&S6096[30], 6}, {NULL}};
const Segment SGI108040[] = {{&S6096[31], 3}, {NULL}};
const Segment SGI108041[] = {{&S6096[83], 6}, {NULL}};
const Segment SGI108042[] = {{&S6097[44], 7}, {NULL}};
const Segment SGI108043[] = {{&S6097[44], 19}, {NULL}};
const Segment SGI108044[] = {{&S6097[30], 6}, {NULL}};
const Segment SGI108045[] = {{&S88450[34], 13}, {NULL}};
const Segment SGI108046[] = {{&S6097[31], 3}, {NULL}};
const Segment SGI108047[] = {{&S88450[27], 6}, {NULL}};
const Segment SGI108048[] = {{&S6097[77], 6}, {NULL}};
const Segment SGI108049[] = {{&S88450[100], 9}, {NULL}};
const Segment SGI108050[] = {{&S88450[73], 6}, {NULL}};
const Segment SGI108051[] = {{&S6098[44], 7}, {NULL}};
const Segment SGI108052[] = {{&S6098[44], 19}, {NULL}};
const Segment SGI108053[] = {{&S6098[30], 6}, {NULL}};
const Segment SGI108054[] = {{&S6098[31], 3}, {NULL}};
const Segment SGI108055[] = {{&S6098[77], 6}, {NULL}};
const Segment SGI108056[] = {{&S6099[44], 18}, {NULL}};
const Segment SGI108057[] = {{&S6099[44], 30}, {NULL}};
const Segment SGI108058[] = {{&S6099[30], 6}, {NULL}};
const Segment SGI108059[] = {{&S6099[31], 3}, {NULL}};
const Segment SGI108060[] = {{&S6099[88], 6}, {NULL}};
const Segment SGI108061[] = {{&S6100[44], 10}, {NULL}};
const Segment SGI108062[] = {{&S6100[44], 22}, {NULL}};
const Segment SGI108063[] = {{&S6100[30], 6}, {NULL}};
const Segment SGI108064[] = {{&S6100[31], 3}, {NULL}};
const Segment SGI108065[] = {{&S6100[80], 6}, {NULL}};
const Segment SGI108066[] = {{&S6101[44], 16}, {NULL}};
const Segment SGI108067[] = {{&S6101[44], 28}, {NULL}};
const Segment SGI108068[] = {{&S6101[30], 6}, {NULL}};
const Segment SGI108069[] = {{&S6101[31], 3}, {NULL}};
const Segment SGI108070[] = {{&S6101[86], 6}, {NULL}};
const Segment SGI108071[] = {{&S6103[44], 7}, {NULL}};
const Segment SGI108072[] = {{&S6103[44], 19}, {NULL}};
const Segment SGI108073[] = {{&S6103[30], 6}, {NULL}};
const Segment SGI108074[] = {{&S6103[31], 3}, {NULL}};
const Segment SGI108075[] = {{&S6103[77], 6}, {NULL}};
const Segment SGI108076[] = {{&S6104[44], 15}, {NULL}};
const Segment SGI108077[] = {{&S6104[44], 27}, {NULL}};
const Segment SGI108078[] = {{&S6104[30], 6}, {NULL}};
const Segment SGI108079[] = {{&S6104[31], 3}, {NULL}};
const Segment SGI108080[] = {{&S6104[85], 6}, {NULL}};
const Segment SGI108081[] = {{&S6108[44], 17}, {NULL}};
const Segment SGI108082[] = {{&S6108[44], 29}, {NULL}};
const Segment SGI108083[] = {{&S6108[30], 6}, {NULL}};
const Segment SGI108084[] = {{&S6108[31], 3}, {NULL}};
const Segment SGI108085[] = {{&S6108[87], 6}, {NULL}};
const Segment SGI108086[] = {{&S6109[44], 10}, {NULL}};
const Segment SGI108087[] = {{&S6109[44], 22}, {NULL}};
const Segment SGI108088[] = {{&S6109[30], 6}, {NULL}};
const Segment SGI108089[] = {{&S6109[31], 3}, {NULL}};
const Segment SGI108090[] = {{&S6109[80], 6}, {NULL}};
const Segment SGI108091[] = {{&S88470[34], 10}, {NULL}};
const Segment SGI108092[] = {{&S88470[27], 6}, {NULL}};
const Segment SGI108093[] = {{&S105876[44], 10}, {NULL}};
const Segment SGI108094[] = {{&S88470[97], 9}, {NULL}};
const Segment SGI108095[] = {{&S105876[44], 23}, {NULL}};
const Segment SGI108096[] = {{&S88470[70], 6}, {NULL}};
const Segment SGI108097[] = {{&S105876[30], 6}, {NULL}};
const Segment SGI108098[] = {{&S105876[115], 3}, {NULL}};
const Segment SGI108099[] = {{&S105876[81], 5}, {NULL}};
const Segment SGI108100[] = {{&S6111[44], 9}, {NULL}};
const Segment SGI108101[] = {{&S6111[44], 21}, {NULL}};
const Segment SGI108102[] = {{&S105876[146], 7}, {NULL}};
const Segment SGI108103[] = {{&S6111[30], 6}, {NULL}};
const Segment SGI108104[] = {{&S6111[31], 3}, {NULL}};
const Segment SGI108105[] = {{&S6111[79], 6}, {NULL}};
const Segment SGI108106[] = {{&S6112[44], 8}, {NULL}};
const Segment SGI108107[] = {{&S6112[44], 20}, {NULL}};
const Segment SGI108108[] = {{&S6112[30], 6}, {NULL}};
const Segment SGI108109[] = {{&S6112[31], 3}, {NULL}};
const Segment SGI108110[] = {{&S6112[78], 6}, {NULL}};
const Segment SGI108111[] = {{&S6114[44], 20}, {NULL}};
const Segment SGI108112[] = {{&S6114[44], 32}, {NULL}};
const Segment SGI108113[] = {{&S6114[30], 6}, {NULL}};
const Segment SGI108114[] = {{&S6114[31], 3}, {NULL}};
const Segment SGI108115[] = {{&S6114[90], 6}, {NULL}};
const Segment SGI108116[] = {{&S6115[44], 9}, {NULL}};
const Segment SGI108117[] = {{&S6115[44], 21}, {NULL}};
const Segment SGI108118[] = {{&S6115[30], 6}, {NULL}};
const Segment SGI108119[] = {{&S6115[31], 3}, {NULL}};
const Segment SGI108120[] = {{&S6115[79], 6}, {NULL}};
const Segment SGI108121[] = {{&S6116[44], 14}, {NULL}};
const Segment SGI108122[] = {{&S6116[44], 26}, {NULL}};
const Segment SGI108123[] = {{&S6116[30], 6}, {NULL}};
const Segment SGI108124[] = {{&S6116[31], 3}, {NULL}};
const Segment SGI108125[] = {{&S6116[84], 6}, {NULL}};
const Segment SGI108126[] = {{&S88484[34], 15}, {NULL}};
const Segment SGI108127[] = {{&S88484[27], 6}, {NULL}};
const Segment SGI108128[] = {{&S88484[102], 9}, {NULL}};
const Segment SGI108129[] = {{&S88484[75], 6}, {NULL}};
const Segment SGI108130[] = {{&S6126[44], 4}, {NULL}};
const Segment SGI108131[] = {{&S6126[44], 16}, {NULL}};
const Segment SGI108132[] = {{&S6126[30], 6}, {NULL}};
const Segment SGI108133[] = {{&S6126[31], 3}, {NULL}};
const Segment SGI108134[] = {{&S6126[74], 6}, {NULL}};
const Segment SGI108135[] = {{&S88490[34], 10}, {NULL}};
const Segment SGI108136[] = {{&S88490[27], 6}, {NULL}};
const Segment SGI108137[] = {{&S88490[97], 9}, {NULL}};
const Segment SGI108138[] = {{&S88490[70], 6}, {NULL}};
const Segment SGI108139[] = {{&S6129[44], 9}, {NULL}};
const Segment SGI108140[] = {{&S6129[44], 21}, {NULL}};
const Segment SGI108141[] = {{&S6129[30], 6}, {NULL}};
const Segment SGI108142[] = {{&S6129[31], 3}, {NULL}};
const Segment SGI108143[] = {{&S6129[79], 6}, {NULL}};
const Segment SGI108144[] = {{&S6132[44], 6}, {NULL}};
const Segment SGI108145[] = {{&S6132[44], 16}, {NULL}};
const Segment SGI108146[] = {{&S6132[30], 6}, {NULL}};
const Segment SGI108147[] = {{&S6132[31], 3}, {NULL}};
const Segment SGI108148[] = {{&S6132[74], 6}, {NULL}};
const Segment SGI108149[] = {{&S6134[44], 9}, {NULL}};
const Segment SGI108150[] = {{&S6134[44], 21}, {NULL}};
const Segment SGI108151[] = {{&S6134[30], 6}, {NULL}};
const Segment SGI108152[] = {{&S6134[31], 3}, {NULL}};
const Segment SGI108153[] = {{&S6134[79], 6}, {NULL}};
const Segment SGI108154[] = {{&S6136[44], 6}, {NULL}};
const Segment SGI108155[] = {{&S6136[44], 18}, {NULL}};
const Segment SGI108156[] = {{&S6136[30], 6}, {NULL}};
const Segment SGI108157[] = {{&S6136[31], 3}, {NULL}};
const Segment SGI108158[] = {{&S6136[76], 6}, {NULL}};
const Segment SGI108159[] = {{&S88504[34], 10}, {NULL}};
const Segment SGI108160[] = {{&S88504[27], 6}, {NULL}};
const Segment SGI108161[] = {{&S88504[97], 9}, {NULL}};
const Segment SGI108162[] = {{&S88504[70], 6}, {NULL}};
const Segment SGI108163[] = {{&S88505[34], 10}, {NULL}};
const Segment SGI108164[] = {{&S88505[27], 6}, {NULL}};
const Segment SGI108165[] = {{&S6139[44], 7}, {NULL}};
const Segment SGI108166[] = {{&S88505[94], 9}, {NULL}};
const Segment SGI108167[] = {{&S6139[44], 19}, {NULL}};
const Segment SGI108168[] = {{&S88505[67], 6}, {NULL}};
const Segment SGI108169[] = {{&S6139[30], 6}, {NULL}};
const Segment SGI108170[] = {{&S6139[31], 3}, {NULL}};
const Segment SGI108171[] = {{&S6139[77], 6}, {NULL}};
const Segment SGI108172[] = {{&S6142[44], 11}, {NULL}};
const Segment SGI108173[] = {{&S6142[44], 23}, {NULL}};
const Segment SGI108174[] = {{&S6142[30], 6}, {NULL}};
const Segment SGI108175[] = {{&S6142[31], 3}, {NULL}};
const Segment SGI108176[] = {{&S6142[81], 6}, {NULL}};
const Segment SGI108177[] = {{&S6143[44], 7}, {NULL}};
const Segment SGI108178[] = {{&S6143[44], 19}, {NULL}};
const Segment SGI108179[] = {{&S6143[30], 6}, {NULL}};
const Segment SGI108180[] = {{&S6143[31], 3}, {NULL}};
const Segment SGI108181[] = {{&S6143[77], 6}, {NULL}};
const Segment SGI108182[] = {{&S6145[44], 31}, {NULL}};
const Segment SGI108183[] = {{&S6145[30], 6}, {NULL}};
const Segment SGI108184[] = {{&S6146[44], 12}, {NULL}};
const Segment SGI108185[] = {{&S6146[44], 24}, {NULL}};
const Segment SGI108186[] = {{&S6146[30], 6}, {NULL}};
const Segment SGI108187[] = {{&S6146[31], 3}, {NULL}};
const Segment SGI108188[] = {{&S6146[82], 6}, {NULL}};
const Segment SGI108189[] = {{&S6147[44], 6}, {NULL}};
const Segment SGI108190[] = {{&S6147[44], 18}, {NULL}};
const Segment SGI108191[] = {{&S6147[30], 6}, {NULL}};
const Segment SGI108192[] = {{&S6147[79], 6}, {NULL}};
const Segment SGI108193[] = {{&S6148[44], 9}, {NULL}};
const Segment SGI108194[] = {{&S6148[44], 21}, {NULL}};
const Segment SGI108195[] = {{&S6148[30], 6}, {NULL}};
const Segment SGI108196[] = {{&S6148[31], 3}, {NULL}};
const Segment SGI108197[] = {{&S6148[79], 6}, {NULL}};
const Segment SGI108198[] = {{&S6151[44], 10}, {NULL}};
const Segment SGI108199[] = {{&S6151[44], 22}, {NULL}};
const Segment SGI108200[] = {{&S6151[30], 6}, {NULL}};
const Segment SGI108201[] = {{&S6151[31], 3}, {NULL}};
const Segment SGI108202[] = {{&S6151[80], 6}, {NULL}};
const Segment SGI108203[] = {{&S6154[44], 5}, {NULL}};
const Segment SGI108204[] = {{&S6154[44], 17}, {NULL}};
const Segment SGI108205[] = {{&S6154[30], 6}, {NULL}};
const Segment SGI108206[] = {{&S6154[31], 3}, {NULL}};
const Segment SGI108207[] = {{&S6154[75], 6}, {NULL}};
const Segment SGI108208[] = {{&S6155[44], 7}, {NULL}};
const Segment SGI108209[] = {{&S6155[44], 19}, {NULL}};
const Segment SGI108210[] = {{&S6155[30], 6}, {NULL}};
const Segment SGI108211[] = {{&S6155[31], 3}, {NULL}};
const Segment SGI108212[] = {{&S6155[77], 6}, {NULL}};
const Segment SGI108213[] = {{&S6156[44], 11}, {NULL}};
const Segment SGI108214[] = {{&S6156[44], 23}, {NULL}};
const Segment SGI108215[] = {{&S6156[30], 6}, {NULL}};
const Segment SGI108216[] = {{&S6156[31], 3}, {NULL}};
const Segment SGI108217[] = {{&S6156[81], 6}, {NULL}};
const Segment SGI108218[] = {{&S6157[44], 12}, {NULL}};
const Segment SGI108219[] = {{&S6157[44], 24}, {NULL}};
const Segment SGI108220[] = {{&S6157[30], 6}, {NULL}};
const Segment SGI108221[] = {{&S6157[31], 3}, {NULL}};
const Segment SGI108222[] = {{&S6157[82], 6}, {NULL}};
const Segment SGI108223[] = {{&S88535[44], 12}, {NULL}};
const Segment SGI108224[] = {{&S88535[44], 33}, {NULL}};
const Segment SGI108225[] = {{&S88535[30], 6}, {NULL}};
const Segment SGI108226[] = {{&S88535[91], 6}, {NULL}};
const Segment SGI108227[] = {{&S88535[118], 13}, {NULL}};
const Segment SGI108228[] = {{&S6158[44], 7}, {NULL}};
const Segment SGI108229[] = {{&S6158[44], 19}, {NULL}};
const Segment SGI108230[] = {{&S6158[30], 6}, {NULL}};
const Segment SGI108231[] = {{&S6158[31], 3}, {NULL}};
const Segment SGI108232[] = {{&S6158[77], 6}, {NULL}};
const Segment SGI108233[] = {{&S6159[44], 12}, {NULL}};
const Segment SGI108234[] = {{&S6159[44], 24}, {NULL}};
const Segment SGI108235[] = {{&S6159[30], 6}, {NULL}};
const Segment SGI108236[] = {{&S6159[31], 3}, {NULL}};
const Segment SGI108237[] = {{&S6159[82], 6}, {NULL}};
const Segment SGI108238[] = {{&S88538[34], 7}, {NULL}};
const Segment SGI108239[] = {{&S88538[27], 6}, {NULL}};
const Segment SGI108240[] = {{&S88538[95], 9}, {NULL}};
const Segment SGI108241[] = {{&S88538[68], 6}, {NULL}};
const Segment SGI108242[] = {{&S6162[44], 11}, {NULL}};
const Segment SGI108243[] = {{&S6162[44], 26}, {NULL}};
const Segment SGI108244[] = {{&S6162[30], 6}, {NULL}};
const Segment SGI108245[] = {{&S6162[31], 3}, {NULL}};
const Segment SGI108246[] = {{&S6162[84], 6}, {NULL}};
const Segment SGI108247[] = {{&S6164[44], 6}, {NULL}};
const Segment SGI108248[] = {{&S6164[44], 18}, {NULL}};
const Segment SGI108249[] = {{&S6164[30], 6}, {NULL}};
const Segment SGI108250[] = {{&S6164[31], 3}, {NULL}};
const Segment SGI108251[] = {{&S6164[76], 6}, {NULL}};
const Segment SGI108252[] = {{&S6166[44], 6}, {NULL}};
const Segment SGI108253[] = {{&S6166[44], 18}, {NULL}};
const Segment SGI108254[] = {{&S6166[30], 6}, {NULL}};
const Segment SGI108255[] = {{&S6166[31], 3}, {NULL}};
const Segment SGI108256[] = {{&S6166[76], 6}, {NULL}};
const Segment SGI108257[] = {{&S6167[44], 10}, {NULL}};
const Segment SGI108258[] = {{&S6167[44], 22}, {NULL}};
const Segment SGI108259[] = {{&S6167[30], 6}, {NULL}};
const Segment SGI108260[] = {{&S6167[31], 3}, {NULL}};
const Segment SGI108261[] = {{&S6167[80], 6}, {NULL}};
const Segment SGI108262[] = {{&S6168[44], 4}, {NULL}};
const Segment SGI108263[] = {{&S6168[44], 16}, {NULL}};
const Segment SGI108264[] = {{&S6168[30], 6}, {NULL}};
const Segment SGI108265[] = {{&S6168[31], 3}, {NULL}};
const Segment SGI108266[] = {{&S6168[74], 6}, {NULL}};
const Segment SGI108267[] = {{&S6169[44], 8}, {NULL}};
const Segment SGI108268[] = {{&S6169[44], 20}, {NULL}};
const Segment SGI108269[] = {{&S6169[30], 6}, {NULL}};
const Segment SGI108270[] = {{&S6169[31], 3}, {NULL}};
const Segment SGI108271[] = {{&S6169[78], 6}, {NULL}};
const Segment SGI108272[] = {{&S6170[44], 11}, {NULL}};
const Segment SGI108273[] = {{&S6170[44], 23}, {NULL}};
const Segment SGI108274[] = {{&S6170[30], 6}, {NULL}};
const Segment SGI108275[] = {{&S6170[31], 3}, {NULL}};
const Segment SGI108276[] = {{&S6170[81], 6}, {NULL}};
const Segment SGI108277[] = {{&S6171[44], 10}, {NULL}};
const Segment SGI108278[] = {{&S6171[44], 22}, {NULL}};
const Segment SGI108279[] = {{&S6171[30], 6}, {NULL}};
const Segment SGI108280[] = {{&S6171[31], 3}, {NULL}};
const Segment SGI108281[] = {{&S6171[80], 6}, {NULL}};
const Segment SGI108282[] = {{&S6172[44], 12}, {NULL}};
const Segment SGI108283[] = {{&S6172[44], 34}, {NULL}};
const Segment SGI108284[] = {{&S6172[30], 6}, {NULL}};
const Segment SGI108285[] = {{&S6172[31], 3}, {NULL}};
const Segment SGI108286[] = {{&S6172[92], 6}, {NULL}};
const Segment SGI108287[] = {{&S6173[44], 17}, {NULL}};
const Segment SGI108288[] = {{&S6173[44], 29}, {NULL}};
const Segment SGI108289[] = {{&S6173[30], 6}, {NULL}};
const Segment SGI108290[] = {{&S6173[31], 3}, {NULL}};
const Segment SGI108291[] = {{&S6173[87], 6}, {NULL}};
const Segment SGI108292[] = {{&S6174[44], 7}, {NULL}};
const Segment SGI108293[] = {{&S6174[44], 19}, {NULL}};
const Segment SGI108294[] = {{&S6174[30], 6}, {NULL}};
const Segment SGI108295[] = {{&S6174[31], 3}, {NULL}};
const Segment SGI108296[] = {{&S6174[77], 6}, {NULL}};
const Segment SGI108297[] = {{&S6175[44], 32}, {NULL}};
const Segment SGI108298[] = {{&S6175[44], 44}, {NULL}};
const Segment SGI108299[] = {{&S6175[30], 6}, {NULL}};
const Segment SGI108300[] = {{&S6175[31], 3}, {NULL}};
const Segment SGI108301[] = {{&S6175[102], 6}, {NULL}};
const Segment SGI108302[] = {{&S103427[59], 10}, {NULL}};
const Segment SGI108303[] = {{&S103427[54], 4}, {NULL}};
const Segment SGI108304[] = {{&S6176[44], 9}, {NULL}};
const Segment SGI108305[] = {{&S6176[44], 21}, {NULL}};
const Segment SGI108306[] = {{&S6176[30], 6}, {NULL}};
const Segment SGI108307[] = {{&S6176[31], 3}, {NULL}};
const Segment SGI108308[] = {{&S6176[79], 6}, {NULL}};
const Segment SGI108309[] = {{&S103428[43], 10}, {NULL}};
const Segment SGI108310[] = {{&S103428[30], 4}, {NULL}};
const Segment SGI108311[] = {{&S103428[113], 3}, {NULL}};
const Segment SGI108312[] = {{&S103428[79], 5}, {NULL}};
const Segment SGI108313[] = {{&S103428[137], 13}, {NULL}};
const Segment SGI108314[] = {{&S6177[44], 9}, {NULL}};
const Segment SGI108315[] = {{&S6177[44], 21}, {NULL}};
const Segment SGI108316[] = {{&S6177[30], 6}, {NULL}};
const Segment SGI108317[] = {{&S6177[31], 3}, {NULL}};
const Segment SGI108318[] = {{&S6177[79], 6}, {NULL}};
const Segment SGI108319[] = {{&S103429[43], 10}, {NULL}};
const Segment SGI108320[] = {{&S103429[30], 4}, {NULL}};
const Segment SGI108321[] = {{&S103429[113], 3}, {NULL}};
const Segment SGI108322[] = {{&S103429[79], 5}, {NULL}};
const Segment SGI108323[] = {{&S103429[137], 13}, {NULL}};
const Segment SGI108324[] = {{&S6180[44], 12}, {NULL}};
const Segment SGI108325[] = {{&S6180[44], 24}, {NULL}};
const Segment SGI108326[] = {{&S6180[30], 6}, {NULL}};
const Segment SGI108327[] = {{&S6180[31], 3}, {NULL}};
const Segment SGI108328[] = {{&S6180[82], 6}, {NULL}};
const Segment SGI108329[] = {{&S103432[57], 10}, {NULL}};
const Segment SGI108330[] = {{&S103432[57], 22}, {NULL}};
const Segment SGI108331[] = {{&S103432[45], 4}, {NULL}};
const Segment SGI108332[] = {{&S103432[93], 5}, {NULL}};
const Segment SGI108333[] = {{&S103432[0], 14}, {NULL}};
const Segment SGI108334[] = {{&S103433[57], 11}, {NULL}};
const Segment SGI108335[] = {{&S6182[44], 8}, {NULL}};
const Segment SGI108336[] = {{&S103433[57], 23}, {NULL}};
const Segment SGI108337[] = {{&S6182[44], 20}, {NULL}};
const Segment SGI108338[] = {{&S103433[45], 4}, {NULL}};
const Segment SGI108339[] = {{&S6182[30], 6}, {NULL}};
const Segment SGI108340[] = {{&S6182[31], 3}, {NULL}};
const Segment SGI108341[] = {{&S103433[94], 5}, {NULL}};
const Segment SGI108342[] = {{&S6182[78], 6}, {NULL}};
const Segment SGI108343[] = {{&S103433[0], 14}, {NULL}};
const Segment SGI108344[] = {{&S103434[42], 10}, {NULL}};
const Segment SGI108345[] = {{&S103434[42], 22}, {NULL}};
const Segment SGI108346[] = {{&S6183[44], 14}, {NULL}};
const Segment SGI108347[] = {{&S103434[30], 4}, {NULL}};
const Segment SGI108348[] = {{&S6183[44], 26}, {NULL}};
const Segment SGI108349[] = {{&S6183[30], 6}, {NULL}};
const Segment SGI108350[] = {{&S103434[112], 3}, {NULL}};
const Segment SGI108351[] = {{&S6183[31], 3}, {NULL}};
const Segment SGI108352[] = {{&S103434[78], 5}, {NULL}};
const Segment SGI108353[] = {{&S6183[84], 6}, {NULL}};
const Segment SGI108354[] = {{&S103434[123], 14}, {NULL}};
const Segment SGI108355[] = {{&S6184[44], 8}, {NULL}};
const Segment SGI108356[] = {{&S6184[44], 20}, {NULL}};
const Segment SGI108357[] = {{&S6184[30], 6}, {NULL}};
const Segment SGI108358[] = {{&S6184[31], 3}, {NULL}};
const Segment SGI108359[] = {{&S6184[78], 6}, {NULL}};
const Segment SGI108360[] = {{&S6185[44], 5}, {NULL}};
const Segment SGI108361[] = {{&S6185[44], 17}, {NULL}};
const Segment SGI108362[] = {{&S6185[30], 6}, {NULL}};
const Segment SGI108363[] = {{&S6185[31], 3}, {NULL}};
const Segment SGI108364[] = {{&S6185[75], 6}, {NULL}};
const Segment SGI108365[] = {{&S6186[44], 8}, {NULL}};
const Segment SGI108366[] = {{&S6186[44], 20}, {NULL}};
const Segment SGI108367[] = {{&S6186[30], 6}, {NULL}};
const Segment SGI108368[] = {{&S6186[31], 3}, {NULL}};
const Segment SGI108369[] = {{&S6186[78], 6}, {NULL}};
const Segment SGI108370[] = {{&S6187[44], 5}, {NULL}};
const Segment SGI108371[] = {{&S6187[44], 17}, {NULL}};
const Segment SGI108372[] = {{&S6187[30], 6}, {NULL}};
const Segment SGI108373[] = {{&S6187[31], 3}, {NULL}};
const Segment SGI108374[] = {{&S6187[75], 6}, {NULL}};
const Segment SGI108375[] = {{&S103446[42], 10}, {NULL}};
const Segment SGI108376[] = {{&S6190[44], 7}, {NULL}};
const Segment SGI108377[] = {{&S6190[44], 19}, {NULL}};
const Segment SGI108378[] = {{&S103446[42], 22}, {NULL}};
const Segment SGI108379[] = {{&S6190[30], 6}, {NULL}};
const Segment SGI108380[] = {{&S103446[30], 4}, {NULL}};
const Segment SGI108381[] = {{&S103446[157], 3}, {NULL}};
const Segment SGI108382[] = {{&S6190[31], 3}, {NULL}};
const Segment SGI108383[] = {{&S103446[123], 3}, {NULL}};
const Segment SGI108384[] = {{&S6190[77], 6}, {NULL}};
const Segment SGI108385[] = {{&S103446[88], 6}, {NULL}};
const Segment SGI108386[] = {{&S103446[143], 8}, {NULL}};
const Segment SGI108387[] = {{&S6191[44], 10}, {NULL}};
const Segment SGI108388[] = {{&S6191[44], 22}, {NULL}};
const Segment SGI108389[] = {{&S6191[30], 6}, {NULL}};
const Segment SGI108390[] = {{&S6191[31], 3}, {NULL}};
const Segment SGI108391[] = {{&S6191[80], 6}, {NULL}};
const Segment SGI108392[] = {{&S103449[42], 11}, {NULL}};
const Segment SGI108393[] = {{&S103449[42], 23}, {NULL}};
const Segment SGI108394[] = {{&S6193[44], 7}, {NULL}};
const Segment SGI108395[] = {{&S103449[30], 4}, {NULL}};
const Segment SGI108396[] = {{&S6193[44], 19}, {NULL}};
const Segment SGI108397[] = {{&S6193[30], 6}, {NULL}};
const Segment SGI108398[] = {{&S103449[114], 3}, {NULL}};
const Segment SGI108399[] = {{&S6193[31], 3}, {NULL}};
const Segment SGI108400[] = {{&S6193[77], 6}, {NULL}};
const Segment SGI108401[] = {{&S103449[79], 6}, {NULL}};
const Segment SGI108402[] = {{&S103449[131], 10}, {NULL}};
const Segment SGI108403[] = {{&S6194[44], 12}, {NULL}};
const Segment SGI108404[] = {{&S6194[44], 24}, {NULL}};
const Segment SGI108405[] = {{&S6194[30], 6}, {NULL}};
const Segment SGI108406[] = {{&S103450[42], 10}, {NULL}};
const Segment SGI108407[] = {{&S103450[42], 22}, {NULL}};
const Segment SGI108408[] = {{&S6194[31], 3}, {NULL}};
const Segment SGI108409[] = {{&S103450[30], 4}, {NULL}};
const Segment SGI108410[] = {{&S6194[82], 6}, {NULL}};
const Segment SGI108411[] = {{&S103450[113], 3}, {NULL}};
const Segment SGI108412[] = {{&S103450[78], 6}, {NULL}};
const Segment SGI108413[] = {{&S103450[130], 10}, {NULL}};
const Segment SGI108414[] = {{&S6195[42], 9}, {NULL}};
const Segment SGI108415[] = {{&S6195[42], 17}, {NULL}};
const Segment SGI108416[] = {{&S6195[28], 6}, {NULL}};
const Segment SGI108417[] = {{&S88607[34], 7}, {NULL}};
const Segment SGI108418[] = {{&S6195[29], 3}, {NULL}};
const Segment SGI108419[] = {{&S88607[27], 6}, {NULL}};
const Segment SGI108420[] = {{&S6195[72], 6}, {NULL}};
const Segment SGI108421[] = {{&S88607[95], 9}, {NULL}};
const Segment SGI108422[] = {{&S88607[68], 6}, {NULL}};
const Segment SGI108423[] = {{&S6196[44], 16}, {NULL}};
const Segment SGI108424[] = {{&S6196[44], 28}, {NULL}};
const Segment SGI108425[] = {{&S6196[30], 6}, {NULL}};
const Segment SGI108426[] = {{&S6196[31], 3}, {NULL}};
const Segment SGI108427[] = {{&S6196[86], 6}, {NULL}};
const Segment SGI108428[] = {{&S88609[45], 15}, {NULL}};
const Segment SGI108429[] = {{&S88609[30], 6}, {NULL}};
const Segment SGI108430[] = {{&S88609[31], 3}, {NULL}};
const Segment SGI108431[] = {{&S6197[44], 5}, {NULL}};
const Segment SGI108432[] = {{&S6197[44], 17}, {NULL}};
const Segment SGI108433[] = {{&S6197[30], 6}, {NULL}};
const Segment SGI108434[] = {{&S88609[86], 5}, {NULL}};
const Segment SGI108435[] = {{&S88609[144], 13}, {NULL}};
const Segment SGI108436[] = {{&S6197[31], 3}, {NULL}};
const Segment SGI108437[] = {{&S6197[75], 6}, {NULL}};
const Segment SGI108438[] = {{&S6199[44], 9}, {NULL}};
const Segment SGI108439[] = {{&S6199[44], 21}, {NULL}};
const Segment SGI108440[] = {{&S6199[30], 6}, {NULL}};
const Segment SGI108441[] = {{&S6199[31], 3}, {NULL}};
const Segment SGI108442[] = {{&S6199[79], 6}, {NULL}};
const Segment SGI108443[] = {{&S103459[42], 10}, {NULL}};
const Segment SGI108444[] = {{&S103459[42], 22}, {NULL}};
const Segment SGI108445[] = {{&S103459[30], 4}, {NULL}};
const Segment SGI108446[] = {{&S103459[113], 3}, {NULL}};
const Segment SGI108447[] = {{&S103459[78], 6}, {NULL}};
const Segment SGI108448[] = {{&S103459[131], 11}, {NULL}};
const Segment SGI108449[] = {{&S6200[44], 5}, {NULL}};
const Segment SGI108450[] = {{&S6200[44], 17}, {NULL}};
const Segment SGI108451[] = {{&S6200[30], 6}, {NULL}};
const Segment SGI108452[] = {{&S6200[31], 3}, {NULL}};
const Segment SGI108453[] = {{&S6200[75], 6}, {NULL}};
const Segment SGI108454[] = {{&S88615[44], 18}, {NULL}};
const Segment SGI108455[] = {{&S88615[44], 30}, {NULL}};
const Segment SGI108456[] = {{&S88615[30], 6}, {NULL}};
const Segment SGI108457[] = {{&S88615[31], 3}, {NULL}};
const Segment SGI108458[] = {{&S88615[88], 6}, {NULL}};
const Segment SGI108459[] = {{&S6201[44], 7}, {NULL}};
const Segment SGI108460[] = {{&S6201[44], 19}, {NULL}};
const Segment SGI108461[] = {{&S6201[30], 6}, {NULL}};
const Segment SGI108462[] = {{&S6201[31], 3}, {NULL}};
const Segment SGI108463[] = {{&S6201[77], 6}, {NULL}};
const Segment SGI108464[] = {{&S6203[44], 15}, {NULL}};
const Segment SGI108465[] = {{&S6203[44], 27}, {NULL}};
const Segment SGI108466[] = {{&S6203[30], 6}, {NULL}};
const Segment SGI108467[] = {{&S6203[31], 3}, {NULL}};
const Segment SGI108468[] = {{&S6203[85], 6}, {NULL}};
const Segment SGI108469[] = {{&S88620[34], 9}, {NULL}};
const Segment SGI108470[] = {{&S88620[27], 6}, {NULL}};
const Segment SGI108471[] = {{&S88620[96], 9}, {NULL}};
const Segment SGI108472[] = {{&S88620[69], 6}, {NULL}};
const Segment SGI108473[] = {{&S6204[44], 7}, {NULL}};
const Segment SGI108474[] = {{&S6204[44], 19}, {NULL}};
const Segment SGI108475[] = {{&S6204[30], 6}, {NULL}};
const Segment SGI108476[] = {{&S6204[31], 3}, {NULL}};
const Segment SGI108477[] = {{&S6204[77], 6}, {NULL}};
const Segment SGI108478[] = {{&S6205[44], 10}, {NULL}};
const Segment SGI108479[] = {{&S6205[44], 22}, {NULL}};
const Segment SGI108480[] = {{&S6205[30], 6}, {NULL}};
const Segment SGI108481[] = {{&S6205[31], 3}, {NULL}};
const Segment SGI108482[] = {{&S6205[80], 6}, {NULL}};
const Segment SGI108483[] = {{&S6206[44], 9}, {NULL}};
const Segment SGI108484[] = {{&S6206[44], 21}, {NULL}};
const Segment SGI108485[] = {{&S6206[30], 6}, {NULL}};
const Segment SGI108486[] = {{&S6206[31], 3}, {NULL}};
const Segment SGI108487[] = {{&S6206[79], 6}, {NULL}};
const Segment SGI108488[] = {{&S6207[44], 12}, {NULL}};
const Segment SGI108489[] = {{&S6207[44], 24}, {NULL}};
const Segment SGI108490[] = {{&S6207[30], 6}, {NULL}};
const Segment SGI108491[] = {{&S6207[31], 3}, {NULL}};
const Segment SGI108492[] = {{&S6207[82], 6}, {NULL}};
const Segment SGI108493[] = {{&S6209[44], 5}, {NULL}};
const Segment SGI108494[] = {{&S6209[44], 17}, {NULL}};
const Segment SGI108495[] = {{&S6209[30], 6}, {NULL}};
const Segment SGI108496[] = {{&S6209[31], 3}, {NULL}};
const Segment SGI108497[] = {{&S6209[75], 6}, {NULL}};
const Segment SGI108498[] = {{&S88632[34], 13}, {NULL}};
const Segment SGI108499[] = {{&S88632[27], 6}, {NULL}};
const Segment SGI108500[] = {{&S6211[44], 10}, {NULL}};
const Segment SGI108501[] = {{&S6211[44], 22}, {NULL}};
const Segment SGI108502[] = {{&S88632[100], 9}, {NULL}};
const Segment SGI108503[] = {{&S6211[30], 6}, {NULL}};
const Segment SGI108504[] = {{&S88632[73], 6}, {NULL}};
const Segment SGI108505[] = {{&S6211[31], 3}, {NULL}};
const Segment SGI108506[] = {{&S6211[80], 6}, {NULL}};
const Segment SGI108507[] = {{&S6212[44], 11}, {NULL}};
const Segment SGI108508[] = {{&S6212[44], 23}, {NULL}};
const Segment SGI108509[] = {{&S6212[30], 6}, {NULL}};
const Segment SGI108510[] = {{&S6212[31], 3}, {NULL}};
const Segment SGI108511[] = {{&S6212[81], 6}, {NULL}};
const Segment SGI108512[] = {{&S6213[44], 9}, {NULL}};
const Segment SGI108513[] = {{&S6213[44], 22}, {NULL}};
const Segment SGI108514[] = {{&S6213[30], 6}, {NULL}};
const Segment SGI108515[] = {{&S6213[31], 3}, {NULL}};
const Segment SGI108516[] = {{&S6213[80], 6}, {NULL}};
const Segment SGI108517[] = {{&S6215[44], 12}, {NULL}};
const Segment SGI108518[] = {{&S6215[44], 24}, {NULL}};
const Segment SGI108519[] = {{&S6215[30], 6}, {NULL}};
const Segment SGI108520[] = {{&S6215[31], 3}, {NULL}};
const Segment SGI108521[] = {{&S6215[82], 6}, {NULL}};
const Segment SGI108522[] = {{&S6217[44], 8}, {NULL}};
const Segment SGI108523[] = {{&S6217[44], 20}, {NULL}};
const Segment SGI108524[] = {{&S6217[30], 6}, {NULL}};
const Segment SGI108525[] = {{&S6217[31], 3}, {NULL}};
const Segment SGI108526[] = {{&S6217[78], 6}, {NULL}};
const Segment SGI108527[] = {{&S6218[44], 4}, {NULL}};
const Segment SGI108528[] = {{&S6218[44], 16}, {NULL}};
const Segment SGI108529[] = {{&S6218[30], 6}, {NULL}};
const Segment SGI108530[] = {{&S6218[31], 3}, {NULL}};
const Segment SGI108531[] = {{&S6218[74], 6}, {NULL}};
const Segment SGI108532[] = {{&S6220[44], 11}, {NULL}};
const Segment SGI108533[] = {{&S6220[44], 23}, {NULL}};
const Segment SGI108534[] = {{&S6220[30], 6}, {NULL}};
const Segment SGI108535[] = {{&S6220[31], 3}, {NULL}};
const Segment SGI108536[] = {{&S6220[81], 6}, {NULL}};
const Segment SGI108537[] = {{&S6222[44], 12}, {NULL}};
const Segment SGI108538[] = {{&S6222[44], 24}, {NULL}};
const Segment SGI108539[] = {{&S6222[30], 6}, {NULL}};
const Segment SGI108540[] = {{&S6222[31], 3}, {NULL}};
const Segment SGI108541[] = {{&S6222[82], 6}, {NULL}};
const Segment SGI108542[] = {{&S6226[44], 21}, {NULL}};
const Segment SGI108543[] = {{&S6226[44], 33}, {NULL}};
const Segment SGI108544[] = {{&S6226[30], 6}, {NULL}};
const Segment SGI108545[] = {{&S6226[31], 3}, {NULL}};
const Segment SGI108546[] = {{&S6226[91], 6}, {NULL}};
const Segment SGI108547[] = {{&S88658[34], 18}, {NULL}};
const Segment SGI108548[] = {{&S88658[27], 6}, {NULL}};
const Segment SGI108549[] = {{&S88658[105], 9}, {NULL}};
const Segment SGI108550[] = {{&S88658[78], 6}, {NULL}};
const Segment SGI108551[] = {{&S6227[44], 9}, {NULL}};
const Segment SGI108552[] = {{&S88658[139], 8}, {NULL}};
const Segment SGI108553[] = {{&S6227[44], 44}, {NULL}};
const Segment SGI108554[] = {{&S6227[30], 6}, {NULL}};
const Segment SGI108555[] = {{&S6227[31], 3}, {NULL}};
const Segment SGI108556[] = {{&S6227[102], 6}, {NULL}};
const Segment SGI108557[] = {{&S6229[44], 7}, {NULL}};
const Segment SGI108558[] = {{&S6229[44], 19}, {NULL}};
const Segment SGI108559[] = {{&S6229[30], 6}, {NULL}};
const Segment SGI108560[] = {{&S6229[31], 3}, {NULL}};
const Segment SGI108561[] = {{&S6229[77], 6}, {NULL}};
const Segment SGI108562[] = {{&S6234[44], 17}, {NULL}};
const Segment SGI108563[] = {{&S6234[44], 29}, {NULL}};
const Segment SGI108564[] = {{&S6234[30], 6}, {NULL}};
const Segment SGI108565[] = {{&S6234[31], 3}, {NULL}};
const Segment SGI108566[] = {{&S6234[87], 6}, {NULL}};
const Segment SGI108567[] = {{&S6236[44], 12}, {NULL}};
const Segment SGI108568[] = {{&S6236[44], 24}, {NULL}};
const Segment SGI108569[] = {{&S6236[30], 6}, {NULL}};
const Segment SGI108570[] = {{&S6236[31], 3}, {NULL}};
const Segment SGI108571[] = {{&S6236[82], 6}, {NULL}};
const Segment SGI108572[] = {{&S88675[34], 11}, {NULL}};
const Segment SGI108573[] = {{&S6239[44], 5}, {NULL}};
const Segment SGI108574[] = {{&S88675[27], 6}, {NULL}};
const Segment SGI108575[] = {{&S6239[44], 17}, {NULL}};
const Segment SGI108576[] = {{&S6239[30], 6}, {NULL}};
const Segment SGI108577[] = {{&S88675[98], 9}, {NULL}};
const Segment SGI108578[] = {{&S6239[31], 3}, {NULL}};
const Segment SGI108579[] = {{&S88675[71], 6}, {NULL}};
const Segment SGI108580[] = {{&S6239[75], 6}, {NULL}};
const Segment SGI108581[] = {{&S6242[44], 6}, {NULL}};
const Segment SGI108582[] = {{&S6242[44], 18}, {NULL}};
const Segment SGI108583[] = {{&S6242[30], 6}, {NULL}};
const Segment SGI108584[] = {{&S6242[31], 3}, {NULL}};
const Segment SGI108585[] = {{&S6242[76], 6}, {NULL}};
const Segment SGI108586[] = {{&S6247[44], 11}, {NULL}};
const Segment SGI108587[] = {{&S6247[44], 23}, {NULL}};
const Segment SGI108588[] = {{&S6247[30], 6}, {NULL}};
const Segment SGI108589[] = {{&S6247[31], 3}, {NULL}};
const Segment SGI108590[] = {{&S6247[81], 6}, {NULL}};
const Segment SGI108591[] = {{&S6248[44], 8}, {NULL}};
const Segment SGI108592[] = {{&S6248[44], 20}, {NULL}};
const Segment SGI108593[] = {{&S6248[30], 6}, {NULL}};
const Segment SGI108594[] = {{&S6248[31], 3}, {NULL}};
const Segment SGI108595[] = {{&S6248[78], 6}, {NULL}};
const Segment SGI108596[] = {{&S6249[44], 7}, {NULL}};
const Segment SGI108597[] = {{&S6249[44], 19}, {NULL}};
const Segment SGI108598[] = {{&S6249[30], 6}, {NULL}};
const Segment SGI108599[] = {{&S6249[31], 3}, {NULL}};
const Segment SGI108600[] = {{&S6249[77], 6}, {NULL}};
const Segment SGI108601[] = {{&S6250[44], 6}, {NULL}};
const Segment SGI108602[] = {{&S6250[44], 18}, {NULL}};
const Segment SGI108603[] = {{&S6250[30], 6}, {NULL}};
const Segment SGI108604[] = {{&S6250[31], 3}, {NULL}};
const Segment SGI108605[] = {{&S6250[76], 6}, {NULL}};
const Segment SGI108606[] = {{&S6251[44], 7}, {NULL}};
const Segment SGI108607[] = {{&S6251[44], 19}, {NULL}};
const Segment SGI108608[] = {{&S6251[30], 6}, {NULL}};
const Segment SGI108609[] = {{&S6251[31], 3}, {NULL}};
const Segment SGI108610[] = {{&S6251[77], 6}, {NULL}};
const Segment SGI108611[] = {{&S6252[44], 16}, {NULL}};
const Segment SGI108612[] = {{&S6252[44], 28}, {NULL}};
const Segment SGI108613[] = {{&S6252[30], 6}, {NULL}};
const Segment SGI108614[] = {{&S6252[31], 3}, {NULL}};
const Segment SGI108615[] = {{&S6252[86], 6}, {NULL}};
const Segment SGI108616[] = {{&S6253[44], 12}, {NULL}};
const Segment SGI108617[] = {{&S6253[44], 24}, {NULL}};
const Segment SGI108618[] = {{&S6253[30], 6}, {NULL}};
const Segment SGI108619[] = {{&S6253[31], 3}, {NULL}};
const Segment SGI108620[] = {{&S6253[82], 6}, {NULL}};
const Segment SGI108621[] = {{&S6254[44], 10}, {NULL}};
const Segment SGI108622[] = {{&S6254[44], 22}, {NULL}};
const Segment SGI108623[] = {{&S6254[30], 6}, {NULL}};
const Segment SGI108624[] = {{&S6254[31], 3}, {NULL}};
const Segment SGI108625[] = {{&S6254[80], 6}, {NULL}};
const Segment SGI108626[] = {{&S6255[44], 9}, {NULL}};
const Segment SGI108627[] = {{&S6255[44], 21}, {NULL}};
const Segment SGI108628[] = {{&S6255[30], 6}, {NULL}};
const Segment SGI108629[] = {{&S6255[31], 3}, {NULL}};
const Segment SGI108630[] = {{&S6255[79], 6}, {NULL}};
const Segment SGI108631[] = {{&S6256[44], 10}, {NULL}};
const Segment SGI108632[] = {{&S6256[44], 22}, {NULL}};
const Segment SGI108633[] = {{&S6256[30], 6}, {NULL}};
const Segment SGI108634[] = {{&S6256[31], 3}, {NULL}};
const Segment SGI108635[] = {{&S6256[80], 6}, {NULL}};
const Segment SGI108636[] = {{&S6258[44], 20}, {NULL}};
const Segment SGI108637[] = {{&S6258[44], 38}, {NULL}};
const Segment SGI108638[] = {{&S6258[30], 6}, {NULL}};
const Segment SGI108639[] = {{&S6258[31], 3}, {NULL}};
const Segment SGI108640[] = {{&S6258[96], 6}, {NULL}};
const Segment SGI108641[] = {{&S6259[44], 7}, {NULL}};
const Segment SGI108642[] = {{&S6259[44], 19}, {NULL}};
const Segment SGI108643[] = {{&S6259[30], 6}, {NULL}};
const Segment SGI108644[] = {{&S6259[31], 3}, {NULL}};
const Segment SGI108645[] = {{&S6259[77], 6}, {NULL}};
const Segment SGI108646[] = {{&S6260[44], 9}, {NULL}};
const Segment SGI108647[] = {{&S6260[44], 21}, {NULL}};
const Segment SGI108648[] = {{&S6260[30], 6}, {NULL}};
const Segment SGI108649[] = {{&S6260[31], 3}, {NULL}};
const Segment SGI108650[] = {{&S6260[79], 6}, {NULL}};
const Segment SGI108651[] = {{&S6261[44], 13}, {NULL}};
const Segment SGI108652[] = {{&S6261[44], 25}, {NULL}};
const Segment SGI108653[] = {{&S6261[30], 6}, {NULL}};
const Segment SGI108654[] = {{&S6261[31], 3}, {NULL}};
const Segment SGI108655[] = {{&S6261[83], 6}, {NULL}};
const Segment SGI108656[] = {{&S6262[44], 9}, {NULL}};
const Segment SGI108657[] = {{&S6262[44], 21}, {NULL}};
const Segment SGI108658[] = {{&S6262[30], 6}, {NULL}};
const Segment SGI108659[] = {{&S6262[31], 3}, {NULL}};
const Segment SGI108660[] = {{&S88721[44], 7}, {NULL}};
const Segment SGI108661[] = {{&S6262[79], 6}, {NULL}};
const Segment SGI108662[] = {{&S88721[44], 18}, {NULL}};
const Segment SGI108663[] = {{&S88721[30], 6}, {NULL}};
const Segment SGI108664[] = {{&S88721[31], 3}, {NULL}};
const Segment SGI108665[] = {{&S88721[76], 6}, {NULL}};
const Segment SGI108666[] = {{&S88722[44], 7}, {NULL}};
const Segment SGI108667[] = {{&S88722[44], 19}, {NULL}};
const Segment SGI108668[] = {{&S88722[30], 6}, {NULL}};
const Segment SGI108669[] = {{&S6264[44], 5}, {NULL}};
const Segment SGI108670[] = {{&S6264[44], 17}, {NULL}};
const Segment SGI108671[] = {{&S6264[30], 6}, {NULL}};
const Segment SGI108672[] = {{&S88722[31], 3}, {NULL}};
const Segment SGI108673[] = {{&S88722[77], 6}, {NULL}};
const Segment SGI108674[] = {{&S6264[31], 3}, {NULL}};
const Segment SGI108675[] = {{&S6264[75], 6}, {NULL}};
const Segment SGI108676[] = {{&S6265[44], 11}, {NULL}};
const Segment SGI108677[] = {{&S6265[44], 23}, {NULL}};
const Segment SGI108678[] = {{&S6265[30], 6}, {NULL}};
const Segment SGI108679[] = {{&S6265[31], 3}, {NULL}};
const Segment SGI108680[] = {{&S6265[81], 6}, {NULL}};
const Segment SGI108681[] = {{&S6266[44], 8}, {NULL}};
const Segment SGI108682[] = {{&S6266[44], 20}, {NULL}};
const Segment SGI108683[] = {{&S6266[30], 6}, {NULL}};
const Segment SGI108684[] = {{&S6266[31], 3}, {NULL}};
const Segment SGI108685[] = {{&S6266[78], 6}, {NULL}};
const Segment SGI108686[] = {{&S6268[44], 6}, {NULL}};
const Segment SGI108687[] = {{&S6268[44], 18}, {NULL}};
const Segment SGI108688[] = {{&S6268[30], 6}, {NULL}};
const Segment SGI108689[] = {{&S6268[31], 3}, {NULL}};
const Segment SGI108690[] = {{&S6268[76], 6}, {NULL}};
const Segment SGI108691[] = {{&S6270[44], 8}, {NULL}};
const Segment SGI108692[] = {{&S6270[44], 51}, {NULL}};
const Segment SGI108693[] = {{&S6270[30], 6}, {NULL}};
const Segment SGI108694[] = {{&S6270[31], 3}, {NULL}};
const Segment SGI108695[] = {{&S6270[109], 6}, {NULL}};
const Segment SGI108696[] = {{&S88733[45], 9}, {NULL}};
const Segment SGI108697[] = {{&S88733[30], 6}, {NULL}};
const Segment SGI108698[] = {{&S88733[31], 3}, {NULL}};
const Segment SGI108699[] = {{&S88733[80], 5}, {NULL}};
const Segment SGI108700[] = {{&S88733[138], 13}, {NULL}};
const Segment SGI108701[] = {{&S6271[44], 7}, {NULL}};
const Segment SGI108702[] = {{&S6271[44], 19}, {NULL}};
const Segment SGI108703[] = {{&S6271[30], 6}, {NULL}};
const Segment SGI108704[] = {{&S6271[31], 3}, {NULL}};
const Segment SGI108705[] = {{&S6271[77], 6}, {NULL}};
const Segment SGI108706[] = {{&S6272[44], 16}, {NULL}};
const Segment SGI108707[] = {{&S6272[44], 28}, {NULL}};
const Segment SGI108708[] = {{&S6272[30], 6}, {NULL}};
const Segment SGI108709[] = {{&S6272[31], 3}, {NULL}};
const Segment SGI108710[] = {{&S6272[86], 6}, {NULL}};
const Segment SGI108711[] = {{&S88736[34], 11}, {NULL}};
const Segment SGI108712[] = {{&S88736[27], 6}, {NULL}};
const Segment SGI108713[] = {{&S6273[44], 8}, {NULL}};
const Segment SGI108714[] = {{&S6273[44], 20}, {NULL}};
const Segment SGI108715[] = {{&S6273[30], 6}, {NULL}};
const Segment SGI108716[] = {{&S88736[98], 9}, {NULL}};
const Segment SGI108717[] = {{&S88736[71], 6}, {NULL}};
const Segment SGI108718[] = {{&S6273[31], 3}, {NULL}};
const Segment SGI108719[] = {{&S6273[78], 6}, {NULL}};
const Segment SGI108720[] = {{&S6274[44], 12}, {NULL}};
const Segment SGI108721[] = {{&S6274[44], 36}, {NULL}};
const Segment SGI108722[] = {{&S6274[30], 6}, {NULL}};
const Segment SGI108723[] = {{&S6274[31], 3}, {NULL}};
const Segment SGI108724[] = {{&S6274[94], 6}, {NULL}};
const Segment SGI108725[] = {{&S88739[59], 10}, {NULL}};
const Segment SGI108726[] = {{&S88739[59], 22}, {NULL}};
const Segment SGI108727[] = {{&S88739[45], 6}, {NULL}};
const Segment SGI108728[] = {{&S88739[46], 3}, {NULL}};
const Segment SGI108729[] = {{&S88739[95], 6}, {NULL}};
const Segment SGI108730[] = {{&S88739[0], 14}, {NULL}};
const Segment SGI108731[] = {{&S6275[44], 12}, {NULL}};
const Segment SGI108732[] = {{&S6275[44], 24}, {NULL}};
const Segment SGI108733[] = {{&S6275[30], 6}, {NULL}};
const Segment SGI108734[] = {{&S6275[31], 3}, {NULL}};
const Segment SGI108735[] = {{&S6275[82], 6}, {NULL}};
const Segment SGI108736[] = {{&S6276[44], 5}, {NULL}};
const Segment SGI108737[] = {{&S6276[44], 27}, {NULL}};
const Segment SGI108738[] = {{&S6276[30], 6}, {NULL}};
const Segment SGI108739[] = {{&S6276[31], 3}, {NULL}};
const Segment SGI108740[] = {{&S6276[85], 6}, {NULL}};
const Segment SGI108741[] = {{&S6277[44], 9}, {NULL}};
const Segment SGI108742[] = {{&S6277[44], 21}, {NULL}};
const Segment SGI108743[] = {{&S6277[30], 6}, {NULL}};
const Segment SGI108744[] = {{&S6277[31], 3}, {NULL}};
const Segment SGI108745[] = {{&S6277[79], 6}, {NULL}};
const Segment SGI108746[] = {{&S6278[44], 13}, {NULL}};
const Segment SGI108747[] = {{&S6278[44], 25}, {NULL}};
const Segment SGI108748[] = {{&S6278[30], 6}, {NULL}};
const Segment SGI108749[] = {{&S6278[31], 3}, {NULL}};
const Segment SGI108750[] = {{&S6278[83], 6}, {NULL}};
const Segment SGI108751[] = {{&S6279[44], 4}, {NULL}};
const Segment SGI108752[] = {{&S6279[44], 16}, {NULL}};
const Segment SGI108753[] = {{&S6279[30], 6}, {NULL}};
const Segment SGI108754[] = {{&S6279[31], 3}, {NULL}};
const Segment SGI108755[] = {{&S6279[74], 6}, {NULL}};
const Segment SGI108756[] = {{&S6281[44], 10}, {NULL}};
const Segment SGI108757[] = {{&S6281[44], 22}, {NULL}};
const Segment SGI108758[] = {{&S6281[30], 6}, {NULL}};
const Segment SGI108759[] = {{&S6281[31], 3}, {NULL}};
const Segment SGI108760[] = {{&S6281[80], 6}, {NULL}};
const Segment SGI108761[] = {{&S6282[44], 11}, {NULL}};
const Segment SGI108762[] = {{&S6282[44], 26}, {NULL}};
const Segment SGI108763[] = {{&S6282[30], 6}, {NULL}};
const Segment SGI108764[] = {{&S6282[31], 3}, {NULL}};
const Segment SGI108765[] = {{&S88755[34], 9}, {NULL}};
const Segment SGI108766[] = {{&S6282[84], 6}, {NULL}};
const Segment SGI108767[] = {{&S88755[27], 6}, {NULL}};
const Segment SGI108768[] = {{&S88755[96], 9}, {NULL}};
const Segment SGI108769[] = {{&S88755[69], 6}, {NULL}};
const Segment SGI108770[] = {{&S6283[44], 6}, {NULL}};
const Segment SGI108771[] = {{&S6283[44], 18}, {NULL}};
const Segment SGI108772[] = {{&S6283[30], 6}, {NULL}};
const Segment SGI108773[] = {{&S6283[31], 3}, {NULL}};
const Segment SGI108774[] = {{&S6283[76], 6}, {NULL}};
const Segment SGI108775[] = {{&S6284[44], 11}, {NULL}};
const Segment SGI108776[] = {{&S6284[44], 23}, {NULL}};
const Segment SGI108777[] = {{&S6284[30], 6}, {NULL}};
const Segment SGI108778[] = {{&S6284[31], 3}, {NULL}};
const Segment SGI108779[] = {{&S6284[81], 6}, {NULL}};
const Segment SGI108780[] = {{&S6288[44], 9}, {NULL}};
const Segment SGI108781[] = {{&S6288[44], 33}, {NULL}};
const Segment SGI108782[] = {{&S6288[30], 6}, {NULL}};
const Segment SGI108783[] = {{&S6288[31], 3}, {NULL}};
const Segment SGI108784[] = {{&S6288[91], 6}, {NULL}};
const Segment SGI108785[] = {{&S6294[44], 5}, {NULL}};
const Segment SGI108786[] = {{&S6294[44], 17}, {NULL}};
const Segment SGI108787[] = {{&S6294[30], 6}, {NULL}};
const Segment SGI108788[] = {{&S6294[31], 3}, {NULL}};
const Segment SGI108789[] = {{&S6294[75], 6}, {NULL}};
const Segment SGI108790[] = {{&S88772[34], 6}, {NULL}};
const Segment SGI108791[] = {{&S88772[27], 6}, {NULL}};
const Segment SGI108792[] = {{&S88772[93], 9}, {NULL}};
const Segment SGI108793[] = {{&S88772[66], 6}, {NULL}};
const Segment SGI108794[] = {{&S6296[44], 6}, {NULL}};
const Segment SGI108795[] = {{&S6296[44], 18}, {NULL}};
const Segment SGI108796[] = {{&S6296[30], 6}, {NULL}};
const Segment SGI108797[] = {{&S6296[31], 3}, {NULL}};
const Segment SGI108798[] = {{&S6296[76], 6}, {NULL}};
const Segment SGI108799[] = {{&S6297[44], 6}, {NULL}};
const Segment SGI108800[] = {{&S6297[44], 18}, {NULL}};
const Segment SGI108801[] = {{&S6297[30], 6}, {NULL}};
const Segment SGI108802[] = {{&S6297[31], 3}, {NULL}};
const Segment SGI108803[] = {{&S6297[76], 6}, {NULL}};
const Segment SGI108804[] = {{&S103649[44], 7}, {NULL}};
const Segment SGI108805[] = {{&S103649[44], 18}, {NULL}};
const Segment SGI108806[] = {{&S103649[30], 6}, {NULL}};
const Segment SGI108807[] = {{&S103649[111], 3}, {NULL}};
const Segment SGI108808[] = {{&S103649[76], 6}, {NULL}};
const Segment SGI108809[] = {{&S6298[44], 6}, {NULL}};
const Segment SGI108810[] = {{&S6298[44], 18}, {NULL}};
const Segment SGI108811[] = {{&S6298[30], 6}, {NULL}};
const Segment SGI108812[] = {{&S6298[31], 3}, {NULL}};
const Segment SGI108813[] = {{&S6298[76], 6}, {NULL}};
const Segment SGI108814[] = {{&S88779[34], 17}, {NULL}};
const Segment SGI108815[] = {{&S88779[27], 6}, {NULL}};
const Segment SGI108816[] = {{&S88779[112], 9}, {NULL}};
const Segment SGI108817[] = {{&S88779[85], 6}, {NULL}};
const Segment SGI108818[] = {{&S88779[146], 8}, {NULL}};
const Segment SGI108819[] = {{&S6300[44], 7}, {NULL}};
const Segment SGI108820[] = {{&S6300[44], 19}, {NULL}};
const Segment SGI108821[] = {{&S6300[30], 6}, {NULL}};
const Segment SGI108822[] = {{&S6300[31], 3}, {NULL}};
const Segment SGI108823[] = {{&S88780[34], 17}, {NULL}};
const Segment SGI108824[] = {{&S6300[77], 6}, {NULL}};
const Segment SGI108825[] = {{&S88780[27], 6}, {NULL}};
const Segment SGI108826[] = {{&S88780[112], 9}, {NULL}};
const Segment SGI108827[] = {{&S88780[85], 6}, {NULL}};
const Segment SGI108828[] = {{&S88780[146], 8}, {NULL}};
const Segment SGI108829[] = {{&S6303[44], 6}, {NULL}};
const Segment SGI108830[] = {{&S6303[44], 18}, {NULL}};
const Segment SGI108831[] = {{&S6303[30], 6}, {NULL}};
const Segment SGI108832[] = {{&S6303[31], 3}, {NULL}};
const Segment SGI108833[] = {{&S6303[76], 6}, {NULL}};
const Segment SGI108834[] = {{&S6304[44], 10}, {NULL}};
const Segment SGI108835[] = {{&S6304[44], 22}, {NULL}};
const Segment SGI108836[] = {{&S6304[30], 6}, {NULL}};
const Segment SGI108837[] = {{&S6304[31], 3}, {NULL}};
const Segment SGI108838[] = {{&S6304[80], 6}, {NULL}};
const Segment SGI108839[] = {{&S6305[44], 8}, {NULL}};
const Segment SGI108840[] = {{&S6305[44], 20}, {NULL}};
const Segment SGI108841[] = {{&S6305[30], 6}, {NULL}};
const Segment SGI108842[] = {{&S6305[31], 3}, {NULL}};
const Segment SGI108843[] = {{&S6305[78], 6}, {NULL}};
const Segment SGI108844[] = {{&S6308[44], 7}, {NULL}};
const Segment SGI108845[] = {{&S6308[44], 65}, {NULL}};
const Segment SGI108846[] = {{&S6308[30], 6}, {NULL}};
const Segment SGI108847[] = {{&S6308[31], 3}, {NULL}};
const Segment SGI108848[] = {{&S6308[123], 6}, {NULL}};
const Segment SGI108849[] = {{&S6309[44], 10}, {NULL}};
const Segment SGI108850[] = {{&S6309[44], 22}, {NULL}};
const Segment SGI108851[] = {{&S6309[30], 6}, {NULL}};
const Segment SGI108852[] = {{&S6309[31], 3}, {NULL}};
const Segment SGI108853[] = {{&S6309[80], 6}, {NULL}};
const Segment SGI108854[] = {{&S6311[44], 11}, {NULL}};
const Segment SGI108855[] = {{&S6311[44], 23}, {NULL}};
const Segment SGI108856[] = {{&S6311[30], 6}, {NULL}};
const Segment SGI108857[] = {{&S6311[31], 3}, {NULL}};
const Segment SGI108858[] = {{&S6311[81], 6}, {NULL}};
const Segment SGI108859[] = {{&S6312[44], 17}, {NULL}};
const Segment SGI108860[] = {{&S6312[44], 29}, {NULL}};
const Segment SGI108861[] = {{&S6312[30], 6}, {NULL}};
const Segment SGI108862[] = {{&S6312[31], 3}, {NULL}};
const Segment SGI108863[] = {{&S6312[87], 6}, {NULL}};
const Segment SGI108864[] = {{&S6313[44], 7}, {NULL}};
const Segment SGI108865[] = {{&S6313[44], 19}, {NULL}};
const Segment SGI108866[] = {{&S6313[30], 6}, {NULL}};
const Segment SGI108867[] = {{&S6313[31], 3}, {NULL}};
const Segment SGI108868[] = {{&S6313[77], 6}, {NULL}};
const Segment SGI108869[] = {{&S6314[44], 7}, {NULL}};
const Segment SGI108870[] = {{&S6314[44], 19}, {NULL}};
const Segment SGI108871[] = {{&S6314[30], 6}, {NULL}};
const Segment SGI108872[] = {{&S6314[31], 3}, {NULL}};
const Segment SGI108873[] = {{&S6314[77], 6}, {NULL}};
const Segment SGI108874[] = {{&S6315[44], 9}, {NULL}};
const Segment SGI108875[] = {{&S6315[44], 22}, {NULL}};
const Segment SGI108876[] = {{&S6315[30], 6}, {NULL}};
const Segment SGI108877[] = {{&S6315[31], 3}, {NULL}};
const Segment SGI108878[] = {{&S6315[80], 6}, {NULL}};
const Segment SGI108879[] = {{&S6316[44], 12}, {NULL}};
const Segment SGI108880[] = {{&S6316[44], 24}, {NULL}};
const Segment SGI108881[] = {{&S6316[30], 6}, {NULL}};
const Segment SGI108882[] = {{&S6316[31], 3}, {NULL}};
const Segment SGI108883[] = {{&S6316[82], 6}, {NULL}};
const Segment SGI108884[] = {{&S88811[44], 14}, {NULL}};
const Segment SGI108885[] = {{&S88811[44], 26}, {NULL}};
const Segment SGI108886[] = {{&S88811[30], 6}, {NULL}};
const Segment SGI108887[] = {{&S88811[84], 6}, {NULL}};
const Segment SGI108888[] = {{&S88811[111], 13}, {NULL}};
const Segment SGI108889[] = {{&S6318[44], 9}, {NULL}};
const Segment SGI108890[] = {{&S6318[44], 21}, {NULL}};
const Segment SGI108891[] = {{&S6318[30], 6}, {NULL}};
const Segment SGI108892[] = {{&S6318[31], 3}, {NULL}};
const Segment SGI108893[] = {{&S6318[79], 6}, {NULL}};
const Segment SGI108894[] = {{&S6319[44], 10}, {NULL}};
const Segment SGI108895[] = {{&S6319[44], 22}, {NULL}};
const Segment SGI108896[] = {{&S6319[30], 6}, {NULL}};
const Segment SGI108897[] = {{&S6319[31], 3}, {NULL}};
const Segment SGI108898[] = {{&S6319[80], 6}, {NULL}};
const Segment SGI108899[] = {{&S6321[44], 7}, {NULL}};
const Segment SGI108900[] = {{&S6321[44], 22}, {NULL}};
const Segment SGI108901[] = {{&S6321[30], 6}, {NULL}};
const Segment SGI108902[] = {{&S6321[31], 3}, {NULL}};
const Segment SGI108903[] = {{&S6321[80], 6}, {NULL}};
const Segment SGI108904[] = {{&S6322[44], 7}, {NULL}};
const Segment SGI108905[] = {{&S6322[44], 22}, {NULL}};
const Segment SGI108906[] = {{&S6322[30], 6}, {NULL}};
const Segment SGI108907[] = {{&S6322[31], 3}, {NULL}};
const Segment SGI108908[] = {{&S6322[80], 6}, {NULL}};
const Segment SGI108909[] = {{&S6323[44], 7}, {NULL}};
const Segment SGI108910[] = {{&S6323[44], 19}, {NULL}};
const Segment SGI108911[] = {{&S6323[30], 6}, {NULL}};
const Segment SGI108912[] = {{&S6323[31], 3}, {NULL}};
const Segment SGI108913[] = {{&S6323[77], 6}, {NULL}};
const Segment SGI108914[] = {{&S6324[44], 7}, {NULL}};
const Segment SGI108915[] = {{&S6324[44], 19}, {NULL}};
const Segment SGI108916[] = {{&S6324[30], 6}, {NULL}};
const Segment SGI108917[] = {{&S6324[31], 3}, {NULL}};
const Segment SGI108918[] = {{&S6324[77], 6}, {NULL}};
const Segment SGI108919[] = {{&S6326[44], 9}, {NULL}};
const Segment SGI108920[] = {{&S6326[44], 21}, {NULL}};
const Segment SGI108921[] = {{&S6326[30], 6}, {NULL}};
const Segment SGI108922[] = {{&S6326[31], 3}, {NULL}};
const Segment SGI108923[] = {{&S6326[79], 6}, {NULL}};
const Segment SGI108924[] = {{&S103710[44], 9}, {NULL}};
const Segment SGI108925[] = {{&S103710[44], 26}, {NULL}};
const Segment SGI108926[] = {{&S103710[30], 6}, {NULL}};
const Segment SGI108927[] = {{&S103710[118], 3}, {NULL}};
const Segment SGI108928[] = {{&S103710[84], 5}, {NULL}};
const Segment SGI108929[] = {{&S6328[44], 12}, {NULL}};
const Segment SGI108930[] = {{&S6328[44], 24}, {NULL}};
const Segment SGI108931[] = {{&S6328[30], 6}, {NULL}};
const Segment SGI108932[] = {{&S6328[31], 3}, {NULL}};
const Segment SGI108933[] = {{&S6328[82], 6}, {NULL}};
const Segment SGI108934[] = {{&S6333[44], 6}, {NULL}};
const Segment SGI108935[] = {{&S6333[44], 18}, {NULL}};
const Segment SGI108936[] = {{&S6333[30], 6}, {NULL}};
const Segment SGI108937[] = {{&S6333[31], 3}, {NULL}};
const Segment SGI108938[] = {{&S6333[76], 6}, {NULL}};
const Segment SGI108939[] = {{&S6335[44], 6}, {NULL}};
const Segment SGI108940[] = {{&S6335[44], 18}, {NULL}};
const Segment SGI108941[] = {{&S6335[30], 6}, {NULL}};
const Segment SGI108942[] = {{&S6335[31], 3}, {NULL}};
const Segment SGI108943[] = {{&S6335[76], 6}, {NULL}};
const Segment SGI108944[] = {{&S6336[44], 6}, {NULL}};
const Segment SGI108945[] = {{&S6336[44], 18}, {NULL}};
const Segment SGI108946[] = {{&S6336[30], 6}, {NULL}};
const Segment SGI108947[] = {{&S6336[31], 3}, {NULL}};
const Segment SGI108948[] = {{&S6336[76], 6}, {NULL}};
const Segment SGI108949[] = {{&S6338[44], 19}, {NULL}};
const Segment SGI108950[] = {{&S6338[44], 31}, {NULL}};
const Segment SGI108951[] = {{&S6338[30], 6}, {NULL}};
const Segment SGI108952[] = {{&S88850[44], 14}, {NULL}};
const Segment SGI108953[] = {{&S88850[44], 26}, {NULL}};
const Segment SGI108954[] = {{&S88850[30], 6}, {NULL}};
const Segment SGI108955[] = {{&S6338[31], 3}, {NULL}};
const Segment SGI108956[] = {{&S6338[89], 6}, {NULL}};
const Segment SGI108957[] = {{&S88850[31], 3}, {NULL}};
const Segment SGI108958[] = {{&S88850[84], 6}, {NULL}};
const Segment SGI108959[] = {{&S6340[44], 6}, {NULL}};
const Segment SGI108960[] = {{&S6340[44], 22}, {NULL}};
const Segment SGI108961[] = {{&S6340[30], 6}, {NULL}};
const Segment SGI108962[] = {{&S6340[31], 3}, {NULL}};
const Segment SGI108963[] = {{&S6340[80], 6}, {NULL}};
const Segment SGI108964[] = {{&S88854[34], 21}, {NULL}};
const Segment SGI108965[] = {{&S88854[27], 6}, {NULL}};
const Segment SGI108966[] = {{&S88854[108], 9}, {NULL}};
const Segment SGI108967[] = {{&S88854[81], 6}, {NULL}};
const Segment SGI108968[] = {{&S6342[44], 7}, {NULL}};
const Segment SGI108969[] = {{&S6342[44], 19}, {NULL}};
const Segment SGI108970[] = {{&S6342[30], 6}, {NULL}};
const Segment SGI108971[] = {{&S6342[31], 3}, {NULL}};
const Segment SGI108972[] = {{&S6342[77], 6}, {NULL}};
const Segment SGI108973[] = {{&S88855[44], 14}, {NULL}};
const Segment SGI108974[] = {{&S88855[44], 26}, {NULL}};
const Segment SGI108975[] = {{&S88855[30], 6}, {NULL}};
const Segment SGI108976[] = {{&S88855[31], 3}, {NULL}};
const Segment SGI108977[] = {{&S88855[84], 6}, {NULL}};
const Segment SGI108978[] = {{&S6343[44], 8}, {NULL}};
const Segment SGI108979[] = {{&S6343[44], 20}, {NULL}};
const Segment SGI108980[] = {{&S6343[30], 6}, {NULL}};
const Segment SGI108981[] = {{&S6343[31], 3}, {NULL}};
const Segment SGI108982[] = {{&S6343[78], 6}, {NULL}};
const Segment SGI108983[] = {{&S103737[42], 10}, {NULL}};
const Segment SGI108984[] = {{&S103737[42], 22}, {NULL}};
const Segment SGI108985[] = {{&S103737[30], 4}, {NULL}};
const Segment SGI108986[] = {{&S103737[120], 3}, {NULL}};
const Segment SGI108987[] = {{&S103737[85], 6}, {NULL}};
const Segment SGI108988[] = {{&S103737[64], 8}, {NULL}};
const Segment SGI108989[] = {{&S6345[44], 16}, {NULL}};
const Segment SGI108990[] = {{&S6345[44], 28}, {NULL}};
const Segment SGI108991[] = {{&S6345[30], 6}, {NULL}};
const Segment SGI108992[] = {{&S6345[31], 3}, {NULL}};
const Segment SGI108993[] = {{&S6345[86], 6}, {NULL}};
const Segment SGI108994[] = {{&S103738[42], 10}, {NULL}};
const Segment SGI108995[] = {{&S103738[42], 22}, {NULL}};
const Segment SGI108996[] = {{&S103738[30], 4}, {NULL}};
const Segment SGI108997[] = {{&S103738[120], 3}, {NULL}};
const Segment SGI108998[] = {{&S103738[85], 6}, {NULL}};
const Segment SGI108999[] = {{&S103738[64], 8}, {NULL}};
const Segment SGI109000[] = {{&S6346[44], 6}, {NULL}};
const Segment SGI109001[] = {{&S6346[44], 18}, {NULL}};
const Segment SGI109002[] = {{&S6346[30], 6}, {NULL}};
const Segment SGI109003[] = {{&S6346[31], 3}, {NULL}};
const Segment SGI109004[] = {{&S6346[76], 6}, {NULL}};
const Segment SGI109005[] = {{&S103739[42], 10}, {NULL}};
const Segment SGI109006[] = {{&S103739[42], 22}, {NULL}};
const Segment SGI109007[] = {{&S103739[30], 4}, {NULL}};
const Segment SGI109008[] = {{&S103739[113], 3}, {NULL}};
const Segment SGI109009[] = {{&S103739[78], 6}, {NULL}};
const Segment SGI109010[] = {{&S103739[138], 7}, {NULL}};
const Segment SGI109011[] = {{&S6349[44], 12}, {NULL}};
const Segment SGI109012[] = {{&S6349[44], 24}, {NULL}};
const Segment SGI109013[] = {{&S6349[30], 6}, {NULL}};
const Segment SGI109014[] = {{&S6349[31], 3}, {NULL}};
const Segment SGI109015[] = {{&S6349[82], 6}, {NULL}};
const Segment SGI109016[] = {{&S103743[57], 10}, {NULL}};
const Segment SGI109017[] = {{&S103743[57], 22}, {NULL}};
const Segment SGI109018[] = {{&S103743[45], 4}, {NULL}};
const Segment SGI109019[] = {{&S103743[128], 3}, {NULL}};
const Segment SGI109020[] = {{&S103743[93], 6}, {NULL}};
const Segment SGI109021[] = {{&S6350[44], 5}, {NULL}};
const Segment SGI109022[] = {{&S103743[0], 14}, {NULL}};
const Segment SGI109023[] = {{&S6350[44], 23}, {NULL}};
const Segment SGI109024[] = {{&S6350[30], 6}, {NULL}};
const Segment SGI109025[] = {{&S6350[31], 3}, {NULL}};
const Segment SGI109026[] = {{&S6350[81], 6}, {NULL}};
const Segment SGI109027[] = {{&S103744[42], 10}, {NULL}};
const Segment SGI109028[] = {{&S103744[42], 22}, {NULL}};
const Segment SGI109029[] = {{&S103744[30], 4}, {NULL}};
const Segment SGI109030[] = {{&S103744[113], 3}, {NULL}};
const Segment SGI109031[] = {{&S6351[44], 9}, {NULL}};
const Segment SGI109032[] = {{&S6351[44], 21}, {NULL}};
const Segment SGI109033[] = {{&S6351[30], 6}, {NULL}};
const Segment SGI109034[] = {{&S103744[78], 6}, {NULL}};
const Segment SGI109035[] = {{&S103744[131], 13}, {NULL}};
const Segment SGI109036[] = {{&S6351[31], 3}, {NULL}};
const Segment SGI109037[] = {{&S6351[79], 6}, {NULL}};
const Segment SGI109038[] = {{&S6352[44], 7}, {NULL}};
const Segment SGI109039[] = {{&S6352[44], 19}, {NULL}};
const Segment SGI109040[] = {{&S6352[30], 6}, {NULL}};
const Segment SGI109041[] = {{&S6352[31], 3}, {NULL}};
const Segment SGI109042[] = {{&S6352[77], 6}, {NULL}};
const Segment SGI109043[] = {{&S6355[44], 10}, {NULL}};
const Segment SGI109044[] = {{&S6355[44], 22}, {NULL}};
const Segment SGI109045[] = {{&S6355[30], 6}, {NULL}};
const Segment SGI109046[] = {{&S6355[31], 3}, {NULL}};
const Segment SGI109047[] = {{&S6355[80], 6}, {NULL}};
const Segment SGI109048[] = {{&S6356[44], 12}, {NULL}};
const Segment SGI109049[] = {{&S6356[44], 24}, {NULL}};
const Segment SGI109050[] = {{&S6356[30], 6}, {NULL}};
const Segment SGI109051[] = {{&S6356[31], 3}, {NULL}};
const Segment SGI109052[] = {{&S6356[82], 6}, {NULL}};
const Segment SGI109053[] = {{&S103753[57], 10}, {NULL}};
const Segment SGI109054[] = {{&S103753[57], 22}, {NULL}};
const Segment SGI109055[] = {{&S103753[45], 4}, {NULL}};
const Segment SGI109056[] = {{&S103753[128], 3}, {NULL}};
const Segment SGI109057[] = {{&S6357[44], 21}, {NULL}};
const Segment SGI109058[] = {{&S103753[93], 6}, {NULL}};
const Segment SGI109059[] = {{&S6357[44], 33}, {NULL}};
const Segment SGI109060[] = {{&S6357[30], 6}, {NULL}};
const Segment SGI109061[] = {{&S103753[0], 14}, {NULL}};
const Segment SGI109062[] = {{&S6357[31], 3}, {NULL}};
const Segment SGI109063[] = {{&S6357[91], 6}, {NULL}};
const Segment SGI109064[] = {{&S6358[44], 8}, {NULL}};
const Segment SGI109065[] = {{&S6358[44], 20}, {NULL}};
const Segment SGI109066[] = {{&S6358[30], 6}, {NULL}};
const Segment SGI109067[] = {{&S6358[31], 3}, {NULL}};
const Segment SGI109068[] = {{&S6358[78], 6}, {NULL}};
const Segment SGI109069[] = {{&S6359[44], 7}, {NULL}};
const Segment SGI109070[] = {{&S6359[44], 19}, {NULL}};
const Segment SGI109071[] = {{&S6359[30], 6}, {NULL}};
const Segment SGI109072[] = {{&S6359[31], 3}, {NULL}};
const Segment SGI109073[] = {{&S6359[77], 6}, {NULL}};
const Segment SGI109074[] = {{&S6360[44], 8}, {NULL}};
const Segment SGI109075[] = {{&S6360[44], 53}, {NULL}};
const Segment SGI109076[] = {{&S6360[30], 6}, {NULL}};
const Segment SGI109077[] = {{&S6360[31], 3}, {NULL}};
const Segment SGI109078[] = {{&S6360[111], 6}, {NULL}};
const Segment SGI109079[] = {{&S6361[44], 24}, {NULL}};
const Segment SGI109080[] = {{&S6361[44], 36}, {NULL}};
const Segment SGI109081[] = {{&S6361[30], 6}, {NULL}};
const Segment SGI109082[] = {{&S6361[31], 3}, {NULL}};
const Segment SGI109083[] = {{&S6361[94], 6}, {NULL}};
const Segment SGI109084[] = {{&S6362[44], 7}, {NULL}};
const Segment SGI109085[] = {{&S6362[44], 22}, {NULL}};
const Segment SGI109086[] = {{&S6362[30], 6}, {NULL}};
const Segment SGI109087[] = {{&S6362[31], 3}, {NULL}};
const Segment SGI109088[] = {{&S6362[80], 6}, {NULL}};
const Segment SGI109089[] = {{&S103765[42], 10}, {NULL}};
const Segment SGI109090[] = {{&S103765[42], 21}, {NULL}};
const Segment SGI109091[] = {{&S103765[30], 4}, {NULL}};
const Segment SGI109092[] = {{&S103765[156], 3}, {NULL}};
const Segment SGI109093[] = {{&S103765[31], 3}, {NULL}};
const Segment SGI109094[] = {{&S103765[87], 6}, {NULL}};
const Segment SGI109095[] = {{&S6363[44], 9}, {NULL}};
const Segment SGI109096[] = {{&S103765[142], 8}, {NULL}};
const Segment SGI109097[] = {{&S6363[44], 22}, {NULL}};
const Segment SGI109098[] = {{&S6363[30], 6}, {NULL}};
const Segment SGI109099[] = {{&S6363[31], 3}, {NULL}};
const Segment SGI109100[] = {{&S6363[80], 6}, {NULL}};
const Segment SGI109101[] = {{&S6365[44], 13}, {NULL}};
const Segment SGI109102[] = {{&S6365[44], 25}, {NULL}};
const Segment SGI109103[] = {{&S6365[30], 6}, {NULL}};
const Segment SGI109104[] = {{&S6365[31], 3}, {NULL}};
const Segment SGI109105[] = {{&S6365[83], 6}, {NULL}};
const Segment SGI109106[] = {{&S6367[44], 9}, {NULL}};
const Segment SGI109107[] = {{&S6367[44], 21}, {NULL}};
const Segment SGI109108[] = {{&S6367[30], 6}, {NULL}};
const Segment SGI109109[] = {{&S6367[31], 3}, {NULL}};
const Segment SGI109110[] = {{&S6367[79], 6}, {NULL}};
const Segment SGI109111[] = {{&S6368[44], 7}, {NULL}};
const Segment SGI109112[] = {{&S6368[44], 19}, {NULL}};
const Segment SGI109113[] = {{&S6368[30], 6}, {NULL}};
const Segment SGI109114[] = {{&S6368[31], 3}, {NULL}};
const Segment SGI109115[] = {{&S6368[77], 6}, {NULL}};
const Segment SGI109116[] = {{&S6370[44], 8}, {NULL}};
const Segment SGI109117[] = {{&S6370[44], 20}, {NULL}};
const Segment SGI109118[] = {{&S6370[30], 6}, {NULL}};
const Segment SGI109119[] = {{&S6370[31], 3}, {NULL}};
const Segment SGI109120[] = {{&S6370[78], 6}, {NULL}};
const Segment SGI109121[] = {{&S6371[44], 12}, {NULL}};
const Segment SGI109122[] = {{&S6371[44], 24}, {NULL}};
const Segment SGI109123[] = {{&S6371[30], 6}, {NULL}};
const Segment SGI109124[] = {{&S6371[31], 3}, {NULL}};
const Segment SGI109125[] = {{&S6371[82], 6}, {NULL}};
const Segment SGI109126[] = {{&S6373[44], 7}, {NULL}};
const Segment SGI109127[] = {{&S6373[44], 28}, {NULL}};
const Segment SGI109128[] = {{&S6373[30], 6}, {NULL}};
const Segment SGI109129[] = {{&S6373[31], 3}, {NULL}};
const Segment SGI109130[] = {{&S6373[86], 6}, {NULL}};
const Segment SGI109131[] = {{&S6374[44], 8}, {NULL}};
const Segment SGI109132[] = {{&S6374[44], 20}, {NULL}};
const Segment SGI109133[] = {{&S6374[30], 6}, {NULL}};
const Segment SGI109134[] = {{&S6374[31], 3}, {NULL}};
const Segment SGI109135[] = {{&S6374[78], 6}, {NULL}};
const Segment SGI109136[] = {{&S6376[44], 10}, {NULL}};
const Segment SGI109137[] = {{&S6376[44], 22}, {NULL}};
const Segment SGI109138[] = {{&S6376[30], 6}, {NULL}};
const Segment SGI109139[] = {{&S6376[31], 3}, {NULL}};
const Segment SGI109140[] = {{&S6376[80], 6}, {NULL}};
const Segment SGI109141[] = {{&S6378[44], 7}, {NULL}};
const Segment SGI109142[] = {{&S6378[44], 24}, {NULL}};
const Segment SGI109143[] = {{&S6378[30], 6}, {NULL}};
const Segment SGI109144[] = {{&S6378[31], 3}, {NULL}};
const Segment SGI109145[] = {{&S6378[82], 6}, {NULL}};
const Segment SGI109146[] = {{&S6381[44], 9}, {NULL}};
const Segment SGI109147[] = {{&S6381[44], 21}, {NULL}};
const Segment SGI109148[] = {{&S6381[30], 6}, {NULL}};
const Segment SGI109149[] = {{&S6381[31], 3}, {NULL}};
const Segment SGI109150[] = {{&S6381[79], 6}, {NULL}};
const Segment SGI109151[] = {{&S6382[44], 13}, {NULL}};
const Segment SGI109152[] = {{&S6382[44], 25}, {NULL}};
const Segment SGI109153[] = {{&S6382[30], 6}, {NULL}};
const Segment SGI109154[] = {{&S6382[31], 3}, {NULL}};
const Segment SGI109155[] = {{&S6382[83], 6}, {NULL}};
const Segment SGI109156[] = {{&S6383[44], 9}, {NULL}};
const Segment SGI109157[] = {{&S6383[44], 21}, {NULL}};
const Segment SGI109158[] = {{&S6383[30], 6}, {NULL}};
const Segment SGI109159[] = {{&S6383[31], 3}, {NULL}};
const Segment SGI109160[] = {{&S6383[79], 6}, {NULL}};
const Segment SGI109161[] = {{&S6384[44], 9}, {NULL}};
const Segment SGI109162[] = {{&S6384[44], 21}, {NULL}};
const Segment SGI109163[] = {{&S6384[30], 6}, {NULL}};
const Segment SGI109164[] = {{&S6384[31], 3}, {NULL}};
const Segment SGI109165[] = {{&S6384[79], 6}, {NULL}};
const Segment SGI109166[] = {{&S6387[44], 12}, {NULL}};
const Segment SGI109167[] = {{&S6387[44], 24}, {NULL}};
const Segment SGI109168[] = {{&S6387[30], 6}, {NULL}};
const Segment SGI109169[] = {{&S6387[31], 3}, {NULL}};
const Segment SGI109170[] = {{&S6387[82], 6}, {NULL}};
const Segment SGI109171[] = {{&S6388[44], 8}, {NULL}};
const Segment SGI109172[] = {{&S6388[44], 20}, {NULL}};
const Segment SGI109173[] = {{&S6388[30], 6}, {NULL}};
const Segment SGI109174[] = {{&S6388[31], 3}, {NULL}};
const Segment SGI109175[] = {{&S6388[78], 6}, {NULL}};
const Segment SGI109176[] = {{&S6391[44], 12}, {NULL}};
const Segment SGI109177[] = {{&S6391[44], 24}, {NULL}};
const Segment SGI109178[] = {{&S6391[30], 6}, {NULL}};
const Segment SGI109179[] = {{&S6391[31], 3}, {NULL}};
const Segment SGI109180[] = {{&S6391[82], 6}, {NULL}};
const Segment SGI109181[] = {{&S6392[44], 10}, {NULL}};
const Segment SGI109182[] = {{&S6392[44], 22}, {NULL}};
const Segment SGI109183[] = {{&S6392[30], 6}, {NULL}};
const Segment SGI109184[] = {{&S6392[31], 3}, {NULL}};
const Segment SGI109185[] = {{&S6392[80], 6}, {NULL}};
const Segment SGI109186[] = {{&S6393[44], 13}, {NULL}};
const Segment SGI109187[] = {{&S6393[44], 25}, {NULL}};
const Segment SGI109188[] = {{&S6393[30], 6}, {NULL}};
const Segment SGI109189[] = {{&S6393[31], 3}, {NULL}};
const Segment SGI109190[] = {{&S6393[83], 6}, {NULL}};
const Segment SGI109191[] = {{&S6396[44], 7}, {NULL}};
const Segment SGI109192[] = {{&S6396[44], 19}, {NULL}};
const Segment SGI109193[] = {{&S6396[30], 6}, {NULL}};
const Segment SGI109194[] = {{&S6396[31], 3}, {NULL}};
const Segment SGI109195[] = {{&S6396[77], 6}, {NULL}};
const Segment SGI109196[] = {{&S6404[44], 11}, {NULL}};
const Segment SGI109197[] = {{&S6404[44], 23}, {NULL}};
const Segment SGI109198[] = {{&S6404[30], 6}, {NULL}};
const Segment SGI109199[] = {{&S6404[31], 3}, {NULL}};
const Segment SGI109200[] = {{&S6404[81], 6}, {NULL}};
const Segment SGI109201[] = {{&S6405[44], 11}, {NULL}};
const Segment SGI109202[] = {{&S6405[44], 23}, {NULL}};
const Segment SGI109203[] = {{&S6405[30], 6}, {NULL}};
const Segment SGI109204[] = {{&S6405[31], 3}, {NULL}};
const Segment SGI109205[] = {{&S6405[81], 6}, {NULL}};
const Segment SGI109206[] = {{&S6406[44], 7}, {NULL}};
const Segment SGI109207[] = {{&S6406[44], 19}, {NULL}};
const Segment SGI109208[] = {{&S6406[30], 6}, {NULL}};
const Segment SGI109209[] = {{&S6406[31], 3}, {NULL}};
const Segment SGI109210[] = {{&S6406[77], 6}, {NULL}};
const Segment SGI109211[] = {{&S88860[34], 6}, {NULL}};
const Segment SGI109212[] = {{&S88860[27], 6}, {NULL}};
const Segment SGI109213[] = {{&S88860[104], 9}, {NULL}};
const Segment SGI109214[] = {{&S88860[77], 6}, {NULL}};
const Segment SGI109215[] = {{&S6413[44], 31}, {NULL}};
const Segment SGI109216[] = {{&S6413[30], 6}, {NULL}};
const Segment SGI109217[] = {{&S88863[34], 14}, {NULL}};
const Segment SGI109218[] = {{&S88863[27], 6}, {NULL}};
const Segment SGI109219[] = {{&S88863[101], 9}, {NULL}};
const Segment SGI109220[] = {{&S6414[44], 5}, {NULL}};
const Segment SGI109221[] = {{&S88863[74], 6}, {NULL}};
const Segment SGI109222[] = {{&S6414[44], 17}, {NULL}};
const Segment SGI109223[] = {{&S6414[30], 6}, {NULL}};
const Segment SGI109224[] = {{&S88864[34], 14}, {NULL}};
const Segment SGI109225[] = {{&S88864[27], 6}, {NULL}};
const Segment SGI109226[] = {{&S88864[101], 9}, {NULL}};
const Segment SGI109227[] = {{&S88864[74], 6}, {NULL}};
const Segment SGI109228[] = {{&S88865[34], 7}, {NULL}};
const Segment SGI109229[] = {{&S88865[27], 6}, {NULL}};
const Segment SGI109230[] = {{&S88865[94], 9}, {NULL}};
const Segment SGI109231[] = {{&S88865[67], 6}, {NULL}};
const Segment SGI109232[] = {{&S103788[42], 10}, {NULL}};
const Segment SGI109233[] = {{&S103788[42], 21}, {NULL}};
const Segment SGI109234[] = {{&S103788[30], 4}, {NULL}};
const Segment SGI109235[] = {{&S103788[112], 3}, {NULL}};
const Segment SGI109236[] = {{&S103788[77], 6}, {NULL}};
const Segment SGI109237[] = {{&S103788[130], 11}, {NULL}};
const Segment SGI109238[] = {{&S88883[44], 9}, {NULL}};
const Segment SGI109239[] = {{&S88883[44], 21}, {NULL}};
const Segment SGI109240[] = {{&S88883[30], 6}, {NULL}};
const Segment SGI109241[] = {{&S103789[42], 10}, {NULL}};
const Segment SGI109242[] = {{&S103789[42], 21}, {NULL}};
const Segment SGI109243[] = {{&S103789[30], 4}, {NULL}};
const Segment SGI109244[] = {{&S88883[31], 3}, {NULL}};
const Segment SGI109245[] = {{&S103789[112], 3}, {NULL}};
const Segment SGI109246[] = {{&S88883[79], 6}, {NULL}};
const Segment SGI109247[] = {{&S88883[139], 11}, {NULL}};
const Segment SGI109248[] = {{&S103789[77], 6}, {NULL}};
const Segment SGI109249[] = {{&S103789[137], 11}, {NULL}};
const Segment SGI109250[] = {{&S88892[44], 10}, {NULL}};
const Segment SGI109251[] = {{&S88892[44], 22}, {NULL}};
const Segment SGI109252[] = {{&S88892[30], 6}, {NULL}};
const Segment SGI109253[] = {{&S88892[31], 3}, {NULL}};
const Segment SGI109254[] = {{&S88892[80], 6}, {NULL}};
const Segment SGI109255[] = {{&S88892[133], 11}, {NULL}};
const Segment SGI109256[] = {{&S88896[34], 13}, {NULL}};
const Segment SGI109257[] = {{&S88896[27], 6}, {NULL}};
const Segment SGI109258[] = {{&S88896[100], 9}, {NULL}};
const Segment SGI109259[] = {{&S88896[73], 6}, {NULL}};
const Segment SGI109260[] = {{&S88897[34], 18}, {NULL}};
const Segment SGI109261[] = {{&S88897[27], 6}, {NULL}};
const Segment SGI109262[] = {{&S88897[105], 9}, {NULL}};
const Segment SGI109263[] = {{&S88897[78], 6}, {NULL}};
const Segment SGI109264[] = {{&S88905[34], 22}, {NULL}};
const Segment SGI109265[] = {{&S88905[27], 6}, {NULL}};
const Segment SGI109266[] = {{&S88905[109], 9}, {NULL}};
const Segment SGI109267[] = {{&S88905[82], 6}, {NULL}};
const Segment SGI109268[] = {{&S88917[44], 41}, {NULL}};
const Segment SGI109269[] = {{&S88917[44], 53}, {NULL}};
const Segment SGI109270[] = {{&S88917[30], 6}, {NULL}};
const Segment SGI109271[] = {{&S88917[31], 3}, {NULL}};
const Segment SGI109272[] = {{&S88917[111], 6}, {NULL}};
const Segment SGI109273[] = {{&S88923[34], 6}, {NULL}};
const Segment SGI109274[] = {{&S88923[27], 6}, {NULL}};
const Segment SGI109275[] = {{&S88923[93], 9}, {NULL}};
const Segment SGI109276[] = {{&S88923[66], 6}, {NULL}};
const Segment SGI109277[] = {{&S88923[127], 8}, {NULL}};
const Segment SGI109278[] = {{&S6414[31], 3}, {NULL}};
const Segment SGI109279[] = {{&S88924[44], 6}, {NULL}};
const Segment SGI109280[] = {{&S88924[44], 18}, {NULL}};
const Segment SGI109281[] = {{&S6414[75], 6}, {NULL}};
const Segment SGI109282[] = {{&S88924[30], 6}, {NULL}};
const Segment SGI109283[] = {{&S88924[31], 3}, {NULL}};
const Segment SGI109284[] = {{&S88924[76], 6}, {NULL}};
const Segment SGI109285[] = {{&S88924[128], 10}, {NULL}};
const Segment SGI109286[] = {{&S6415[44], 23}, {NULL}};
const Segment SGI109287[] = {{&S6415[44], 35}, {NULL}};
const Segment SGI109288[] = {{&S6415[30], 6}, {NULL}};
const Segment SGI109289[] = {{&S88925[44], 6}, {NULL}};
const Segment SGI109290[] = {{&S6415[31], 3}, {NULL}};
const Segment SGI109291[] = {{&S88925[44], 18}, {NULL}};
const Segment SGI109292[] = {{&S88925[30], 6}, {NULL}};
const Segment SGI109293[] = {{&S6415[93], 6}, {NULL}};
const Segment SGI109294[] = {{&S88925[31], 3}, {NULL}};
const Segment SGI109295[] = {{&S88925[76], 6}, {NULL}};
const Segment SGI109296[] = {{&S88925[135], 10}, {NULL}};
const Segment SGI109297[] = {{&S6416[44], 8}, {NULL}};
const Segment SGI109298[] = {{&S6416[44], 20}, {NULL}};
const Segment SGI109299[] = {{&S6416[30], 6}, {NULL}};
const Segment SGI109300[] = {{&S6416[31], 3}, {NULL}};
const Segment SGI109301[] = {{&S6416[78], 6}, {NULL}};
const Segment SGI109302[] = {{&S6417[44], 31}, {NULL}};
const Segment SGI109303[] = {{&S6417[30], 6}, {NULL}};
const Segment SGI109304[] = {{&S6419[44], 6}, {NULL}};
const Segment SGI109305[] = {{&S6419[44], 18}, {NULL}};
const Segment SGI109306[] = {{&S6419[30], 6}, {NULL}};
const Segment SGI109307[] = {{&S6419[31], 3}, {NULL}};
const Segment SGI109308[] = {{&S6419[76], 6}, {NULL}};
const Segment SGI109309[] = {{&S6421[44], 10}, {NULL}};
const Segment SGI109310[] = {{&S6421[44], 22}, {NULL}};
const Segment SGI109311[] = {{&S6421[30], 6}, {NULL}};
const Segment SGI109312[] = {{&S6421[31], 3}, {NULL}};
const Segment SGI109313[] = {{&S6421[80], 6}, {NULL}};
const Segment SGI109314[] = {{&S6424[44], 5}, {NULL}};
const Segment SGI109315[] = {{&S6424[44], 17}, {NULL}};
const Segment SGI109316[] = {{&S6424[30], 6}, {NULL}};
const Segment SGI109317[] = {{&S6424[31], 3}, {NULL}};
const Segment SGI109318[] = {{&S6424[75], 6}, {NULL}};
const Segment SGI109319[] = {{&S6426[44], 8}, {NULL}};
const Segment SGI109320[] = {{&S6426[44], 20}, {NULL}};
const Segment SGI109321[] = {{&S6426[30], 6}, {NULL}};
const Segment SGI109322[] = {{&S6426[31], 3}, {NULL}};
const Segment SGI109323[] = {{&S6426[78], 6}, {NULL}};
const Segment SGI109324[] = {{&S6429[44], 11}, {NULL}};
const Segment SGI109325[] = {{&S6429[44], 23}, {NULL}};
const Segment SGI109326[] = {{&S6429[30], 6}, {NULL}};
const Segment SGI109327[] = {{&S6429[31], 3}, {NULL}};
const Segment SGI109328[] = {{&S6429[81], 6}, {NULL}};
const Segment SGI109329[] = {{&S6432[44], 8}, {NULL}};
const Segment SGI109330[] = {{&S6432[44], 20}, {NULL}};
const Segment SGI109331[] = {{&S6432[30], 6}, {NULL}};
const Segment SGI109332[] = {{&S6432[31], 3}, {NULL}};
const Segment SGI109333[] = {{&S6432[78], 6}, {NULL}};
const Segment SGI109334[] = {{&S6433[44], 7}, {NULL}};
const Segment SGI109335[] = {{&S6433[44], 19}, {NULL}};
const Segment SGI109336[] = {{&S6433[30], 6}, {NULL}};
const Segment SGI109337[] = {{&S6433[31], 3}, {NULL}};
const Segment SGI109338[] = {{&S6433[77], 6}, {NULL}};
const Segment SGI109339[] = {{&S6434[44], 9}, {NULL}};
const Segment SGI109340[] = {{&S6434[44], 21}, {NULL}};
const Segment SGI109341[] = {{&S6434[30], 6}, {NULL}};
const Segment SGI109342[] = {{&S6434[31], 3}, {NULL}};
const Segment SGI109343[] = {{&S6434[79], 6}, {NULL}};
const Segment SGI109344[] = {{&S6435[44], 21}, {NULL}};
const Segment SGI109345[] = {{&S6435[44], 33}, {NULL}};
const Segment SGI109346[] = {{&S6435[30], 6}, {NULL}};
const Segment SGI109347[] = {{&S6435[31], 3}, {NULL}};
const Segment SGI109348[] = {{&S6435[91], 6}, {NULL}};
const Segment SGI109349[] = {{&S6436[44], 12}, {NULL}};
const Segment SGI109350[] = {{&S6436[44], 24}, {NULL}};
const Segment SGI109351[] = {{&S6436[30], 6}, {NULL}};
const Segment SGI109352[] = {{&S6436[31], 3}, {NULL}};
const Segment SGI109353[] = {{&S6436[82], 6}, {NULL}};
const Segment SGI109354[] = {{&S6437[44], 14}, {NULL}};
const Segment SGI109355[] = {{&S6437[44], 26}, {NULL}};
const Segment SGI109356[] = {{&S6437[30], 6}, {NULL}};
const Segment SGI109357[] = {{&S6437[31], 3}, {NULL}};
const Segment SGI109358[] = {{&S6437[84], 6}, {NULL}};
const Segment SGI109359[] = {{&S6438[44], 14}, {NULL}};
const Segment SGI109360[] = {{&S6438[44], 26}, {NULL}};
const Segment SGI109361[] = {{&S6438[30], 6}, {NULL}};
const Segment SGI109362[] = {{&S6438[31], 3}, {NULL}};
const Segment SGI109363[] = {{&S6438[84], 6}, {NULL}};
const Segment SGI109364[] = {{&S6439[44], 14}, {NULL}};
const Segment SGI109365[] = {{&S6439[44], 26}, {NULL}};
const Segment SGI109366[] = {{&S6439[30], 6}, {NULL}};
const Segment SGI109367[] = {{&S6439[31], 3}, {NULL}};
const Segment SGI109368[] = {{&S6439[84], 6}, {NULL}};
const Segment SGI109369[] = {{&S6440[44], 13}, {NULL}};
const Segment SGI109370[] = {{&S6440[44], 25}, {NULL}};
const Segment SGI109371[] = {{&S6440[30], 6}, {NULL}};
const Segment SGI109372[] = {{&S6440[31], 3}, {NULL}};
const Segment SGI109373[] = {{&S6440[83], 6}, {NULL}};
const Segment SGI109374[] = {{&S6442[44], 11}, {NULL}};
const Segment SGI109375[] = {{&S6442[44], 23}, {NULL}};
const Segment SGI109376[] = {{&S6442[30], 6}, {NULL}};
const Segment SGI109377[] = {{&S6442[31], 3}, {NULL}};
const Segment SGI109378[] = {{&S6442[81], 6}, {NULL}};
const Segment SGI109379[] = {{&S6444[44], 12}, {NULL}};
const Segment SGI109380[] = {{&S6444[44], 24}, {NULL}};
const Segment SGI109381[] = {{&S6444[30], 6}, {NULL}};
const Segment SGI109382[] = {{&S6444[31], 3}, {NULL}};
const Segment SGI109383[] = {{&S6444[82], 6}, {NULL}};
const Segment SGI109384[] = {{&S6446[44], 12}, {NULL}};
const Segment SGI109385[] = {{&S6446[44], 24}, {NULL}};
const Segment SGI109386[] = {{&S6446[30], 6}, {NULL}};
const Segment SGI109387[] = {{&S6446[31], 3}, {NULL}};
const Segment SGI109388[] = {{&S6446[82], 6}, {NULL}};
const Segment SGI109389[] = {{&S6447[43], 7}, {NULL}};
const Segment SGI109390[] = {{&S6447[43], 19}, {NULL}};
const Segment SGI109391[] = {{&S6447[29], 6}, {NULL}};
const Segment SGI109392[] = {{&S6447[30], 3}, {NULL}};
const Segment SGI109393[] = {{&S6447[76], 6}, {NULL}};
const Segment SGI109394[] = {{&S6448[44], 5}, {NULL}};
const Segment SGI109395[] = {{&S6448[44], 17}, {NULL}};
const Segment SGI109396[] = {{&S6448[30], 6}, {NULL}};
const Segment SGI109397[] = {{&S6448[31], 3}, {NULL}};
const Segment SGI109398[] = {{&S6448[75], 6}, {NULL}};
const Segment SGI109399[] = {{&S6449[44], 11}, {NULL}};
const Segment SGI109400[] = {{&S6449[44], 23}, {NULL}};
const Segment SGI109401[] = {{&S6449[30], 6}, {NULL}};
const Segment SGI109402[] = {{&S6449[31], 3}, {NULL}};
const Segment SGI109403[] = {{&S6449[81], 6}, {NULL}};
const Segment SGI109404[] = {{&S6451[44], 6}, {NULL}};
const Segment SGI109405[] = {{&S6451[44], 18}, {NULL}};
const Segment SGI109406[] = {{&S6451[30], 6}, {NULL}};
const Segment SGI109407[] = {{&S6451[31], 3}, {NULL}};
const Segment SGI109408[] = {{&S6451[76], 6}, {NULL}};
const Segment SGI109409[] = {{&S6452[44], 15}, {NULL}};
const Segment SGI109410[] = {{&S6452[44], 27}, {NULL}};
const Segment SGI109411[] = {{&S6452[30], 6}, {NULL}};
const Segment SGI109412[] = {{&S6452[31], 3}, {NULL}};
const Segment SGI109413[] = {{&S6452[85], 6}, {NULL}};
const Segment SGI109414[] = {{&S6453[44], 9}, {NULL}};
const Segment SGI109415[] = {{&S6453[44], 32}, {NULL}};
const Segment SGI109416[] = {{&S6453[30], 6}, {NULL}};
const Segment SGI109417[] = {{&S6453[31], 3}, {NULL}};
const Segment SGI109418[] = {{&S6453[90], 6}, {NULL}};
const Segment SGI109419[] = {{&S6454[44], 5}, {NULL}};
const Segment SGI109420[] = {{&S6454[44], 17}, {NULL}};
const Segment SGI109421[] = {{&S6454[30], 6}, {NULL}};
const Segment SGI109422[] = {{&S6454[31], 3}, {NULL}};
const Segment SGI109423[] = {{&S6454[75], 6}, {NULL}};
const Segment SGI109424[] = {{&S6455[44], 9}, {NULL}};
const Segment SGI109425[] = {{&S6455[44], 21}, {NULL}};
const Segment SGI109426[] = {{&S6455[30], 6}, {NULL}};
const Segment SGI109427[] = {{&S6455[31], 3}, {NULL}};
const Segment SGI109428[] = {{&S6455[79], 6}, {NULL}};
const Segment SGI109429[] = {{&S6459[44], 13}, {NULL}};
const Segment SGI109430[] = {{&S6459[44], 25}, {NULL}};
const Segment SGI109431[] = {{&S6459[30], 6}, {NULL}};
const Segment SGI109432[] = {{&S6459[31], 3}, {NULL}};
const Segment SGI109433[] = {{&S6459[83], 6}, {NULL}};
const Segment SGI109434[] = {{&S6462[44], 12}, {NULL}};
const Segment SGI109435[] = {{&S6462[44], 24}, {NULL}};
const Segment SGI109436[] = {{&S6462[30], 6}, {NULL}};
const Segment SGI109437[] = {{&S6462[31], 3}, {NULL}};
const Segment SGI109438[] = {{&S6462[82], 6}, {NULL}};
const Segment SGI109439[] = {{&S6466[44], 14}, {NULL}};
const Segment SGI109440[] = {{&S6466[44], 26}, {NULL}};
const Segment SGI109441[] = {{&S6466[30], 6}, {NULL}};
const Segment SGI109442[] = {{&S6466[31], 3}, {NULL}};
const Segment SGI109443[] = {{&S6466[84], 6}, {NULL}};
const Segment SGI109444[] = {{&S6468[44], 7}, {NULL}};
const Segment SGI109445[] = {{&S6468[44], 19}, {NULL}};
const Segment SGI109446[] = {{&S6468[30], 6}, {NULL}};
const Segment SGI109447[] = {{&S6468[31], 3}, {NULL}};
const Segment SGI109448[] = {{&S6468[77], 6}, {NULL}};
const Segment SGI109449[] = {{&S6470[44], 19}, {NULL}};
const Segment SGI109450[] = {{&S6470[44], 31}, {NULL}};
const Segment SGI109451[] = {{&S6470[30], 6}, {NULL}};
const Segment SGI109452[] = {{&S6470[31], 3}, {NULL}};
const Segment SGI109453[] = {{&S6470[89], 6}, {NULL}};
const Segment SGI109454[] = {{&S6471[44], 8}, {NULL}};
const Segment SGI109455[] = {{&S6471[44], 20}, {NULL}};
const Segment SGI109456[] = {{&S6471[30], 6}, {NULL}};
const Segment SGI109457[] = {{&S6471[31], 3}, {NULL}};
const Segment SGI109458[] = {{&S6471[78], 6}, {NULL}};
const Segment SGI109459[] = {{&S6476[44], 28}, {NULL}};
const Segment SGI109460[] = {{&S6476[44], 40}, {NULL}};
const Segment SGI109461[] = {{&S6476[30], 6}, {NULL}};
const Segment SGI109462[] = {{&S6476[31], 3}, {NULL}};
const Segment SGI109463[] = {{&S6476[98], 6}, {NULL}};
const Segment SGI109464[] = {{&S6477[44], 6}, {NULL}};
const Segment SGI109465[] = {{&S6477[44], 18}, {NULL}};
const Segment SGI109466[] = {{&S6477[30], 6}, {NULL}};
const Segment SGI109467[] = {{&S6477[79], 6}, {NULL}};
const Segment SGI109468[] = {{&S6478[44], 9}, {NULL}};
const Segment SGI109469[] = {{&S6478[44], 21}, {NULL}};
const Segment SGI109470[] = {{&S6478[30], 6}, {NULL}};
const Segment SGI109471[] = {{&S6478[31], 3}, {NULL}};
const Segment SGI109472[] = {{&S6478[79], 6}, {NULL}};
const Segment SGI109473[] = {{&S6480[44], 14}, {NULL}};
const Segment SGI109474[] = {{&S6480[44], 26}, {NULL}};
const Segment SGI109475[] = {{&S6480[30], 6}, {NULL}};
const Segment SGI109476[] = {{&S6480[31], 3}, {NULL}};
const Segment SGI109477[] = {{&S6480[84], 6}, {NULL}};
const Segment SGI109478[] = {{&S6481[44], 8}, {NULL}};
const Segment SGI109479[] = {{&S6481[44], 20}, {NULL}};
const Segment SGI109480[] = {{&S6481[30], 6}, {NULL}};
const Segment SGI109481[] = {{&S6481[31], 3}, {NULL}};
const Segment SGI109482[] = {{&S6481[78], 6}, {NULL}};
const Segment SGI109483[] = {{&S6483[42], 10}, {NULL}};
const Segment SGI109484[] = {{&S6483[30], 6}, {NULL}};
const Segment SGI109485[] = {{&S6483[31], 3}, {NULL}};
const Segment SGI109486[] = {{&S6483[78], 6}, {NULL}};
const Segment SGI109487[] = {{&S6484[44], 13}, {NULL}};
const Segment SGI109488[] = {{&S6484[44], 25}, {NULL}};
const Segment SGI109489[] = {{&S6484[30], 6}, {NULL}};
const Segment SGI109490[] = {{&S6484[31], 3}, {NULL}};
const Segment SGI109491[] = {{&S6484[83], 6}, {NULL}};
const Segment SGI109492[] = {{&S6486[44], 4}, {NULL}};
const Segment SGI109493[] = {{&S6486[44], 16}, {NULL}};
const Segment SGI109494[] = {{&S6486[30], 6}, {NULL}};
const Segment SGI109495[] = {{&S6486[31], 3}, {NULL}};
const Segment SGI109496[] = {{&S6486[74], 6}, {NULL}};
const Segment SGI109497[] = {{&S6487[44], 10}, {NULL}};
const Segment SGI109498[] = {{&S6487[44], 22}, {NULL}};
const Segment SGI109499[] = {{&S6487[30], 6}, {NULL}};
const Segment SGI109500[] = {{&S6487[31], 3}, {NULL}};
const Segment SGI109501[] = {{&S6487[80], 6}, {NULL}};
const Segment SGI109502[] = {{&S6488[44], 11}, {NULL}};
const Segment SGI109503[] = {{&S6488[44], 23}, {NULL}};
const Segment SGI109504[] = {{&S6488[30], 6}, {NULL}};
const Segment SGI109505[] = {{&S6488[31], 3}, {NULL}};
const Segment SGI109506[] = {{&S6488[81], 6}, {NULL}};
const Segment SGI109507[] = {{&S6489[44], 7}, {NULL}};
const Segment SGI109508[] = {{&S6489[44], 19}, {NULL}};
const Segment SGI109509[] = {{&S6489[30], 6}, {NULL}};
const Segment SGI109510[] = {{&S6489[31], 3}, {NULL}};
const Segment SGI109511[] = {{&S6489[77], 6}, {NULL}};
const Segment SGI109512[] = {{&S6491[44], 8}, {NULL}};
const Segment SGI109513[] = {{&S6491[44], 20}, {NULL}};
const Segment SGI109514[] = {{&S6491[30], 6}, {NULL}};
const Segment SGI109515[] = {{&S6491[31], 3}, {NULL}};
const Segment SGI109516[] = {{&S6491[78], 6}, {NULL}};
const Segment SGI109517[] = {{&S6494[44], 16}, {NULL}};
const Segment SGI109518[] = {{&S6494[44], 28}, {NULL}};
const Segment SGI109519[] = {{&S6494[30], 6}, {NULL}};
const Segment SGI109520[] = {{&S6494[31], 3}, {NULL}};
const Segment SGI109521[] = {{&S6494[86], 6}, {NULL}};
const Segment SGI109522[] = {{&S6496[44], 9}, {NULL}};
const Segment SGI109523[] = {{&S6496[44], 21}, {NULL}};
const Segment SGI109524[] = {{&S6496[30], 6}, {NULL}};
const Segment SGI109525[] = {{&S6496[31], 3}, {NULL}};
const Segment SGI109526[] = {{&S6496[79], 6}, {NULL}};
const Segment SGI109527[] = {{&S6497[44], 7}, {NULL}};
const Segment SGI109528[] = {{&S6497[44], 19}, {NULL}};
const Segment SGI109529[] = {{&S6497[30], 6}, {NULL}};
const Segment SGI109530[] = {{&S6497[31], 3}, {NULL}};
const Segment SGI109531[] = {{&S6497[77], 6}, {NULL}};
const Segment SGI109532[] = {{&S6498[44], 8}, {NULL}};
const Segment SGI109533[] = {{&S6498[44], 61}, {NULL}};
const Segment SGI109534[] = {{&S6498[30], 6}, {NULL}};
const Segment SGI109535[] = {{&S6498[31], 3}, {NULL}};
const Segment SGI109536[] = {{&S6498[120], 6}, {NULL}};
const Segment SGI109537[] = {{&S6500[44], 6}, {NULL}};
const Segment SGI109538[] = {{&S6500[44], 63}, {NULL}};
const Segment SGI109539[] = {{&S6500[30], 6}, {NULL}};
const Segment SGI109540[] = {{&S6500[31], 3}, {NULL}};
const Segment SGI109541[] = {{&S6500[121], 6}, {NULL}};
const Segment SGI109542[] = {{&S6503[44], 9}, {NULL}};
const Segment SGI109543[] = {{&S6503[44], 21}, {NULL}};
const Segment SGI109544[] = {{&S6503[30], 6}, {NULL}};
const Segment SGI109545[] = {{&S6503[31], 3}, {NULL}};
const Segment SGI109546[] = {{&S6503[79], 6}, {NULL}};
const Segment SGI109547[] = {{&S6504[44], 18}, {NULL}};
const Segment SGI109548[] = {{&S6504[44], 30}, {NULL}};
const Segment SGI109549[] = {{&S6504[30], 6}, {NULL}};
const Segment SGI109550[] = {{&S6504[31], 3}, {NULL}};
const Segment SGI109551[] = {{&S6504[88], 6}, {NULL}};
const Segment SGI109552[] = {{&S6505[44], 7}, {NULL}};
const Segment SGI109553[] = {{&S6505[44], 19}, {NULL}};
const Segment SGI109554[] = {{&S6505[30], 6}, {NULL}};
const Segment SGI109555[] = {{&S6505[31], 3}, {NULL}};
const Segment SGI109556[] = {{&S6505[77], 6}, {NULL}};
const Segment SGI109557[] = {{&S88981[44], 25}, {NULL}};
const Segment SGI109558[] = {{&S88981[44], 37}, {NULL}};
const Segment SGI109559[] = {{&S88981[30], 6}, {NULL}};
const Segment SGI109560[] = {{&S88981[126], 2}, {NULL}};
const Segment SGI109561[] = {{&S88981[95], 5}, {NULL}};
const Segment SGI109562[] = {{&S6507[44], 8}, {NULL}};
const Segment SGI109563[] = {{&S6507[44], 20}, {NULL}};
const Segment SGI109564[] = {{&S6507[30], 6}, {NULL}};
const Segment SGI109565[] = {{&S6507[31], 3}, {NULL}};
const Segment SGI109566[] = {{&S6507[78], 6}, {NULL}};
const Segment SGI109567[] = {{&S6508[44], 3}, {NULL}};
const Segment SGI109568[] = {{&S6508[44], 15}, {NULL}};
const Segment SGI109569[] = {{&S6508[30], 6}, {NULL}};
const Segment SGI109570[] = {{&S6508[31], 3}, {NULL}};
const Segment SGI109571[] = {{&S6508[73], 6}, {NULL}};
const Segment SGI109572[] = {{&S6509[44], 16}, {NULL}};
const Segment SGI109573[] = {{&S6509[44], 28}, {NULL}};
const Segment SGI109574[] = {{&S6509[30], 6}, {NULL}};
const Segment SGI109575[] = {{&S6509[31], 3}, {NULL}};
const Segment SGI109576[] = {{&S6509[86], 6}, {NULL}};
const Segment SGI109577[] = {{&S6510[44], 7}, {NULL}};
const Segment SGI109578[] = {{&S6510[44], 19}, {NULL}};
const Segment SGI109579[] = {{&S6510[30], 6}, {NULL}};
const Segment SGI109580[] = {{&S6510[31], 3}, {NULL}};
const Segment SGI109581[] = {{&S6510[77], 6}, {NULL}};
const Segment SGI109582[] = {{&S6511[44], 6}, {NULL}};
const Segment SGI109583[] = {{&S6511[44], 18}, {NULL}};
const Segment SGI109584[] = {{&S6511[30], 6}, {NULL}};
const Segment SGI109585[] = {{&S6511[31], 3}, {NULL}};
const Segment SGI109586[] = {{&S6511[76], 6}, {NULL}};
const Segment SGI109587[] = {{&S6512[44], 7}, {NULL}};
const Segment SGI109588[] = {{&S6512[44], 19}, {NULL}};
const Segment SGI109589[] = {{&S6512[30], 6}, {NULL}};
const Segment SGI109590[] = {{&S6512[31], 3}, {NULL}};
const Segment SGI109591[] = {{&S6512[77], 6}, {NULL}};
const Segment SGI109592[] = {{&S6513[44], 7}, {NULL}};
const Segment SGI109593[] = {{&S88994[34], 6}, {NULL}};
const Segment SGI109594[] = {{&S6513[44], 30}, {NULL}};
const Segment SGI109595[] = {{&S6513[30], 6}, {NULL}};
const Segment SGI109596[] = {{&S88994[27], 6}, {NULL}};
const Segment SGI109597[] = {{&S6513[31], 3}, {NULL}};
const Segment SGI109598[] = {{&S88994[93], 9}, {NULL}};
const Segment SGI109599[] = {{&S6513[88], 6}, {NULL}};
const Segment SGI109600[] = {{&S88994[66], 6}, {NULL}};
const Segment SGI109601[] = {{&S6514[44], 6}, {NULL}};
const Segment SGI109602[] = {{&S6514[44], 18}, {NULL}};
const Segment SGI109603[] = {{&S6514[30], 6}, {NULL}};
const Segment SGI109604[] = {{&S6514[31], 3}, {NULL}};
const Segment SGI109605[] = {{&S6514[76], 6}, {NULL}};
const Segment SGI109606[] = {{&S88996[44], 13}, {NULL}};
const Segment SGI109607[] = {{&S88996[44], 25}, {NULL}};
const Segment SGI109608[] = {{&S88996[30], 6}, {NULL}};
const Segment SGI109609[] = {{&S88996[31], 3}, {NULL}};
const Segment SGI109610[] = {{&S88996[83], 6}, {NULL}};
const Segment SGI109611[] = {{&S6515[44], 14}, {NULL}};
const Segment SGI109612[] = {{&S6515[44], 26}, {NULL}};
const Segment SGI109613[] = {{&S6515[30], 6}, {NULL}};
const Segment SGI109614[] = {{&S6515[31], 3}, {NULL}};
const Segment SGI109615[] = {{&S6515[84], 6}, {NULL}};
const Segment SGI109616[] = {{&S6517[44], 8}, {NULL}};
const Segment SGI109617[] = {{&S6517[44], 20}, {NULL}};
const Segment SGI109618[] = {{&S6517[30], 6}, {NULL}};
const Segment SGI109619[] = {{&S6517[31], 3}, {NULL}};
const Segment SGI109620[] = {{&S6517[78], 6}, {NULL}};
const Segment SGI109621[] = {{&S6518[44], 12}, {NULL}};
const Segment SGI109622[] = {{&S6518[44], 22}, {NULL}};
const Segment SGI109623[] = {{&S6518[30], 6}, {NULL}};
const Segment SGI109624[] = {{&S6518[31], 3}, {NULL}};
const Segment SGI109625[] = {{&S6518[80], 6}, {NULL}};
const Segment SGI109626[] = {{&S89003[34], 8}, {NULL}};
const Segment SGI109627[] = {{&S89003[27], 6}, {NULL}};
const Segment SGI109628[] = {{&S89003[95], 9}, {NULL}};
const Segment SGI109629[] = {{&S89003[68], 6}, {NULL}};
const Segment SGI109630[] = {{&S6520[44], 8}, {NULL}};
const Segment SGI109631[] = {{&S6520[44], 20}, {NULL}};
const Segment SGI109632[] = {{&S6520[30], 6}, {NULL}};
const Segment SGI109633[] = {{&S6520[31], 3}, {NULL}};
const Segment SGI109634[] = {{&S6520[78], 6}, {NULL}};
const Segment SGI109635[] = {{&S6522[44], 6}, {NULL}};
const Segment SGI109636[] = {{&S6522[44], 18}, {NULL}};
const Segment SGI109637[] = {{&S6522[30], 6}, {NULL}};
const Segment SGI109638[] = {{&S6522[31], 3}, {NULL}};
const Segment SGI109639[] = {{&S89008[34], 4}, {NULL}};
const Segment SGI109640[] = {{&S6522[76], 6}, {NULL}};
const Segment SGI109641[] = {{&S89008[27], 6}, {NULL}};
const Segment SGI109642[] = {{&S89008[91], 9}, {NULL}};
const Segment SGI109643[] = {{&S89008[64], 6}, {NULL}};
const Segment SGI109644[] = {{&S6523[44], 14}, {NULL}};
const Segment SGI109645[] = {{&S6523[44], 26}, {NULL}};
const Segment SGI109646[] = {{&S6523[30], 6}, {NULL}};
const Segment SGI109647[] = {{&S6523[31], 3}, {NULL}};
const Segment SGI109648[] = {{&S89009[45], 4}, {NULL}};
const Segment SGI109649[] = {{&S6523[84], 6}, {NULL}};
const Segment SGI109650[] = {{&S89009[30], 6}, {NULL}};
const Segment SGI109651[] = {{&S89009[31], 3}, {NULL}};
const Segment SGI109652[] = {{&S89009[85], 5}, {NULL}};
const Segment SGI109653[] = {{&S89009[143], 13}, {NULL}};
const Segment SGI109654[] = {{&S6525[44], 18}, {NULL}};
const Segment SGI109655[] = {{&S6525[44], 30}, {NULL}};
const Segment SGI109656[] = {{&S6525[30], 6}, {NULL}};
const Segment SGI109657[] = {{&S6525[31], 3}, {NULL}};
const Segment SGI109658[] = {{&S6525[88], 6}, {NULL}};
const Segment SGI109659[] = {{&S6526[44], 7}, {NULL}};
const Segment SGI109660[] = {{&S6526[44], 27}, {NULL}};
const Segment SGI109661[] = {{&S6526[30], 6}, {NULL}};
const Segment SGI109662[] = {{&S6526[31], 3}, {NULL}};
const Segment SGI109663[] = {{&S6526[85], 6}, {NULL}};
const Segment SGI109664[] = {{&S6527[44], 7}, {NULL}};
const Segment SGI109665[] = {{&S6527[44], 19}, {NULL}};
const Segment SGI109666[] = {{&S6527[30], 6}, {NULL}};
const Segment SGI109667[] = {{&S6527[31], 3}, {NULL}};
const Segment SGI109668[] = {{&S6527[77], 6}, {NULL}};
const Segment SGI109669[] = {{&S6528[44], 13}, {NULL}};
const Segment SGI109670[] = {{&S6528[44], 25}, {NULL}};
const Segment SGI109671[] = {{&S6528[30], 6}, {NULL}};
const Segment SGI109672[] = {{&S6528[31], 3}, {NULL}};
const Segment SGI109673[] = {{&S6528[83], 6}, {NULL}};
const Segment SGI109674[] = {{&S6530[44], 12}, {NULL}};
const Segment SGI109675[] = {{&S6530[44], 24}, {NULL}};
const Segment SGI109676[] = {{&S6530[30], 6}, {NULL}};
const Segment SGI109677[] = {{&S6530[31], 3}, {NULL}};
const Segment SGI109678[] = {{&S6530[82], 6}, {NULL}};
const Segment SGI109679[] = {{&S6533[44], 10}, {NULL}};
const Segment SGI109680[] = {{&S6533[44], 22}, {NULL}};
const Segment SGI109681[] = {{&S6533[30], 6}, {NULL}};
const Segment SGI109682[] = {{&S6533[31], 3}, {NULL}};
const Segment SGI109683[] = {{&S6533[80], 6}, {NULL}};
const Segment SGI109684[] = {{&S6534[44], 7}, {NULL}};
const Segment SGI109685[] = {{&S6534[44], 29}, {NULL}};
const Segment SGI109686[] = {{&S6534[30], 6}, {NULL}};
const Segment SGI109687[] = {{&S6534[31], 3}, {NULL}};
const Segment SGI109688[] = {{&S6534[87], 6}, {NULL}};
const Segment SGI109689[] = {{&S6535[44], 9}, {NULL}};
const Segment SGI109690[] = {{&S6535[44], 21}, {NULL}};
const Segment SGI109691[] = {{&S6535[30], 6}, {NULL}};
const Segment SGI109692[] = {{&S6535[31], 3}, {NULL}};
const Segment SGI109693[] = {{&S6535[79], 6}, {NULL}};
const Segment SGI109694[] = {{&S6536[44], 8}, {NULL}};
const Segment SGI109695[] = {{&S6536[44], 20}, {NULL}};
const Segment SGI109696[] = {{&S6536[30], 6}, {NULL}};
const Segment SGI109697[] = {{&S6536[31], 3}, {NULL}};
const Segment SGI109698[] = {{&S6536[78], 6}, {NULL}};
const Segment SGI109699[] = {{&S6537[44], 15}, {NULL}};
const Segment SGI109700[] = {{&S6537[44], 27}, {NULL}};
const Segment SGI109701[] = {{&S6537[30], 6}, {NULL}};
const Segment SGI109702[] = {{&S6537[31], 3}, {NULL}};
const Segment SGI109703[] = {{&S6537[85], 6}, {NULL}};
const Segment SGI109704[] = {{&S89035[34], 8}, {NULL}};
const Segment SGI109705[] = {{&S89035[27], 6}, {NULL}};
const Segment SGI109706[] = {{&S89035[95], 9}, {NULL}};
const Segment SGI109707[] = {{&S89035[68], 6}, {NULL}};
const Segment SGI109708[] = {{&S6538[44], 11}, {NULL}};
const Segment SGI109709[] = {{&S6538[44], 23}, {NULL}};
const Segment SGI109710[] = {{&S6538[30], 6}, {NULL}};
const Segment SGI109711[] = {{&S6538[31], 3}, {NULL}};
const Segment SGI109712[] = {{&S6538[81], 6}, {NULL}};
const Segment SGI109713[] = {{&S103961[44], 11}, {NULL}};
const Segment SGI109714[] = {{&S103961[44], 32}, {NULL}};
const Segment SGI109715[] = {{&S103961[30], 6}, {NULL}};
const Segment SGI109716[] = {{&S103961[124], 3}, {NULL}};
const Segment SGI109717[] = {{&S103961[90], 5}, {NULL}};
const Segment SGI109718[] = {{&S6543[44], 17}, {NULL}};
const Segment SGI109719[] = {{&S6543[44], 29}, {NULL}};
const Segment SGI109720[] = {{&S6543[30], 6}, {NULL}};
const Segment SGI109721[] = {{&S6543[31], 3}, {NULL}};
const Segment SGI109722[] = {{&S6543[87], 6}, {NULL}};
const Segment SGI109723[] = {{&S89045[44], 13}, {NULL}};
const Segment SGI109724[] = {{&S89045[30], 6}, {NULL}};
const Segment SGI109725[] = {{&S89045[85], 6}, {NULL}};
const Segment SGI109726[] = {{&S89045[112], 13}, {NULL}};
const Segment SGI109727[] = {{&S104037[42], 10}, {NULL}};
const Segment SGI109728[] = {{&S104037[42], 22}, {NULL}};
const Segment SGI109729[] = {{&S104037[30], 4}, {NULL}};
const Segment SGI109730[] = {{&S104037[113], 3}, {NULL}};
const Segment SGI109731[] = {{&S104037[78], 6}, {NULL}};
const Segment SGI109732[] = {{&S104039[60], 10}, {NULL}};
const Segment SGI109733[] = {{&S104039[55], 4}, {NULL}};
const Segment SGI109734[] = {{&S104043[57], 10}, {NULL}};
const Segment SGI109735[] = {{&S104043[57], 22}, {NULL}};
const Segment SGI109736[] = {{&S104043[45], 4}, {NULL}};
const Segment SGI109737[] = {{&S104043[128], 3}, {NULL}};
const Segment SGI109738[] = {{&S104043[93], 6}, {NULL}};
const Segment SGI109739[] = {{&S104043[0], 14}, {NULL}};
const Segment SGI109740[] = {{&S104044[57], 11}, {NULL}};
const Segment SGI109741[] = {{&S104044[57], 23}, {NULL}};
const Segment SGI109742[] = {{&S104044[45], 4}, {NULL}};
const Segment SGI109743[] = {{&S104044[129], 3}, {NULL}};
const Segment SGI109744[] = {{&S104044[94], 6}, {NULL}};
const Segment SGI109745[] = {{&S104044[0], 14}, {NULL}};
const Segment SGI109746[] = {{&S89144[34], 5}, {NULL}};
const Segment SGI109747[] = {{&S89144[27], 6}, {NULL}};
const Segment SGI109748[] = {{&S89144[92], 9}, {NULL}};
const Segment SGI109749[] = {{&S89144[65], 6}, {NULL}};
const Segment SGI109750[] = {{&S104047[57], 10}, {NULL}};
const Segment SGI109751[] = {{&S104047[57], 22}, {NULL}};
const Segment SGI109752[] = {{&S104047[45], 4}, {NULL}};
const Segment SGI109753[] = {{&S104047[93], 5}, {NULL}};
const Segment SGI109754[] = {{&S104047[0], 14}, {NULL}};
const Segment SGI109755[] = {{&S104048[42], 10}, {NULL}};
const Segment SGI109756[] = {{&S104048[42], 22}, {NULL}};
const Segment SGI109757[] = {{&S104048[30], 4}, {NULL}};
const Segment SGI109758[] = {{&S104048[112], 3}, {NULL}};
const Segment SGI109759[] = {{&S89147[34], 15}, {NULL}};
const Segment SGI109760[] = {{&S89147[27], 6}, {NULL}};
const Segment SGI109761[] = {{&S104048[78], 5}, {NULL}};
const Segment SGI109762[] = {{&S104048[123], 14}, {NULL}};
const Segment SGI109763[] = {{&S89147[102], 9}, {NULL}};
const Segment SGI109764[] = {{&S89147[75], 6}, {NULL}};
const Segment SGI109765[] = {{&S104050[57], 10}, {NULL}};
const Segment SGI109766[] = {{&S104050[57], 22}, {NULL}};
const Segment SGI109767[] = {{&S104050[45], 4}, {NULL}};
const Segment SGI109768[] = {{&S104050[128], 3}, {NULL}};
const Segment SGI109769[] = {{&S104050[93], 6}, {NULL}};
const Segment SGI109770[] = {{&S104050[0], 14}, {NULL}};
const Segment SGI109771[] = {{&S89151[44], 12}, {NULL}};
const Segment SGI109772[] = {{&S89151[44], 37}, {NULL}};
const Segment SGI109773[] = {{&S89151[30], 6}, {NULL}};
const Segment SGI109774[] = {{&S104051[43], 10}, {NULL}};
const Segment SGI109775[] = {{&S89151[126], 2}, {NULL}};
const Segment SGI109776[] = {{&S104051[30], 4}, {NULL}};
const Segment SGI109777[] = {{&S89151[95], 5}, {NULL}};
const Segment SGI109778[] = {{&S104051[113], 3}, {NULL}};
const Segment SGI109779[] = {{&S104051[79], 5}, {NULL}};
const Segment SGI109780[] = {{&S104051[137], 13}, {NULL}};
const Segment SGI109781[] = {{&S89152[34], 9}, {NULL}};
const Segment SGI109782[] = {{&S89152[27], 6}, {NULL}};
const Segment SGI109783[] = {{&S89152[96], 9}, {NULL}};
const Segment SGI109784[] = {{&S89152[69], 6}, {NULL}};
const Segment SGI109785[] = {{&S104052[43], 11}, {NULL}};
const Segment SGI109786[] = {{&S104052[30], 4}, {NULL}};
const Segment SGI109787[] = {{&S104052[114], 3}, {NULL}};
const Segment SGI109788[] = {{&S104052[80], 5}, {NULL}};
const Segment SGI109789[] = {{&S104052[138], 13}, {NULL}};
const Segment SGI109790[] = {{&S89157[34], 11}, {NULL}};
const Segment SGI109791[] = {{&S89157[27], 6}, {NULL}};
const Segment SGI109792[] = {{&S89157[98], 9}, {NULL}};
const Segment SGI109793[] = {{&S89157[71], 6}, {NULL}};
const Segment SGI109794[] = {{&S89159[44], 13}, {NULL}};
const Segment SGI109795[] = {{&S89159[44], 32}, {NULL}};
const Segment SGI109796[] = {{&S89159[30], 6}, {NULL}};
const Segment SGI109797[] = {{&S89159[31], 3}, {NULL}};
const Segment SGI109798[] = {{&S89159[90], 6}, {NULL}};
const Segment SGI109799[] = {{&S6547[44], 11}, {NULL}};
const Segment SGI109800[] = {{&S6547[44], 23}, {NULL}};
const Segment SGI109801[] = {{&S6547[30], 6}, {NULL}};
const Segment SGI109802[] = {{&S6547[31], 3}, {NULL}};
const Segment SGI109803[] = {{&S6547[81], 6}, {NULL}};
const Segment SGI109804[] = {{&S6548[44], 21}, {NULL}};
const Segment SGI109805[] = {{&S6548[44], 33}, {NULL}};
const Segment SGI109806[] = {{&S6548[30], 6}, {NULL}};
const Segment SGI109807[] = {{&S6548[31], 3}, {NULL}};
const Segment SGI109808[] = {{&S6548[91], 6}, {NULL}};
const Segment SGI109809[] = {{&S89166[34], 9}, {NULL}};
const Segment SGI109810[] = {{&S89166[27], 6}, {NULL}};
const Segment SGI109811[] = {{&S89166[96], 9}, {NULL}};
const Segment SGI109812[] = {{&S89166[69], 6}, {NULL}};
const Segment SGI109813[] = {{&S6551[44], 10}, {NULL}};
const Segment SGI109814[] = {{&S6551[44], 22}, {NULL}};
const Segment SGI109815[] = {{&S6551[30], 6}, {NULL}};
const Segment SGI109816[] = {{&S6551[31], 3}, {NULL}};
const Segment SGI109817[] = {{&S6551[80], 6}, {NULL}};
const Segment SGI109818[] = {{&S6552[44], 15}, {NULL}};
const Segment SGI109819[] = {{&S6552[44], 27}, {NULL}};
const Segment SGI109820[] = {{&S6552[30], 6}, {NULL}};
const Segment SGI109821[] = {{&S6552[31], 3}, {NULL}};
const Segment SGI109822[] = {{&S6552[85], 6}, {NULL}};
const Segment SGI109823[] = {{&S6553[44], 26}, {NULL}};
const Segment SGI109824[] = {{&S6553[44], 38}, {NULL}};
const Segment SGI109825[] = {{&S6553[30], 6}, {NULL}};
const Segment SGI109826[] = {{&S6553[31], 3}, {NULL}};
const Segment SGI109827[] = {{&S6553[96], 6}, {NULL}};
const Segment SGI109828[] = {{&S6554[44], 6}, {NULL}};
const Segment SGI109829[] = {{&S6554[44], 18}, {NULL}};
const Segment SGI109830[] = {{&S6554[30], 6}, {NULL}};
const Segment SGI109831[] = {{&S6554[31], 3}, {NULL}};
const Segment SGI109832[] = {{&S6554[76], 6}, {NULL}};
const Segment SGI109833[] = {{&S6555[44], 21}, {NULL}};
const Segment SGI109834[] = {{&S6555[44], 33}, {NULL}};
const Segment SGI109835[] = {{&S6555[30], 6}, {NULL}};
const Segment SGI109836[] = {{&S6555[31], 3}, {NULL}};
const Segment SGI109837[] = {{&S6555[91], 6}, {NULL}};
const Segment SGI109838[] = {{&S6559[44], 14}, {NULL}};
const Segment SGI109839[] = {{&S6559[44], 26}, {NULL}};
const Segment SGI109840[] = {{&S6559[30], 6}, {NULL}};
const Segment SGI109841[] = {{&S6559[31], 3}, {NULL}};
const Segment SGI109842[] = {{&S6559[84], 6}, {NULL}};
const Segment SGI109843[] = {{&S6561[44], 7}, {NULL}};
const Segment SGI109844[] = {{&S6561[44], 19}, {NULL}};
const Segment SGI109845[] = {{&S6561[30], 6}, {NULL}};
const Segment SGI109846[] = {{&S6561[31], 3}, {NULL}};
const Segment SGI109847[] = {{&S6561[77], 6}, {NULL}};
const Segment SGI109848[] = {{&S6565[44], 19}, {NULL}};
const Segment SGI109849[] = {{&S6565[44], 31}, {NULL}};
const Segment SGI109850[] = {{&S6565[30], 6}, {NULL}};
const Segment SGI109851[] = {{&S6565[31], 3}, {NULL}};
const Segment SGI109852[] = {{&S6565[89], 6}, {NULL}};
const Segment SGI109853[] = {{&S6568[44], 10}, {NULL}};
const Segment SGI109854[] = {{&S6568[44], 22}, {NULL}};
const Segment SGI109855[] = {{&S6568[30], 6}, {NULL}};
const Segment SGI109856[] = {{&S6568[31], 3}, {NULL}};
const Segment SGI109857[] = {{&S6568[80], 6}, {NULL}};
const Segment SGI109858[] = {{&S89195[44], 11}, {NULL}};
const Segment SGI109859[] = {{&S89195[44], 23}, {NULL}};
const Segment SGI109860[] = {{&S89195[30], 6}, {NULL}};
const Segment SGI109861[] = {{&S89195[81], 6}, {NULL}};
const Segment SGI109862[] = {{&S89195[108], 13}, {NULL}};
const Segment SGI109863[] = {{&S6570[44], 14}, {NULL}};
const Segment SGI109864[] = {{&S6570[44], 26}, {NULL}};
const Segment SGI109865[] = {{&S6570[30], 6}, {NULL}};
const Segment SGI109866[] = {{&S6570[31], 3}, {NULL}};
const Segment SGI109867[] = {{&S6570[84], 6}, {NULL}};
const Segment SGI109868[] = {{&S6571[44], 16}, {NULL}};
const Segment SGI109869[] = {{&S6571[44], 28}, {NULL}};
const Segment SGI109870[] = {{&S6571[30], 6}, {NULL}};
const Segment SGI109871[] = {{&S6571[31], 3}, {NULL}};
const Segment SGI109872[] = {{&S6571[86], 6}, {NULL}};
const Segment SGI109873[] = {{&S6572[44], 7}, {NULL}};
const Segment SGI109874[] = {{&S6572[44], 19}, {NULL}};
const Segment SGI109875[] = {{&S6572[30], 6}, {NULL}};
const Segment SGI109876[] = {{&S6572[31], 3}, {NULL}};
const Segment SGI109877[] = {{&S6572[77], 6}, {NULL}};
const Segment SGI109878[] = {{&S6574[44], 8}, {NULL}};
const Segment SGI109879[] = {{&S6574[44], 19}, {NULL}};
const Segment SGI109880[] = {{&S6574[30], 6}, {NULL}};
const Segment SGI109881[] = {{&S6574[31], 3}, {NULL}};
const Segment SGI109882[] = {{&S6574[77], 5}, {NULL}};
const Segment SGI109883[] = {{&S6575[44], 8}, {NULL}};
const Segment SGI109884[] = {{&S6575[44], 20}, {NULL}};
const Segment SGI109885[] = {{&S6575[30], 6}, {NULL}};
const Segment SGI109886[] = {{&S6575[31], 3}, {NULL}};
const Segment SGI109887[] = {{&S6575[78], 6}, {NULL}};
const Segment SGI109888[] = {{&S6577[44], 10}, {NULL}};
const Segment SGI109889[] = {{&S6577[44], 22}, {NULL}};
const Segment SGI109890[] = {{&S6577[30], 6}, {NULL}};
const Segment SGI109891[] = {{&S6577[31], 3}, {NULL}};
const Segment SGI109892[] = {{&S6577[80], 6}, {NULL}};
const Segment SGI109893[] = {{&S6578[44], 9}, {NULL}};
const Segment SGI109894[] = {{&S6578[44], 22}, {NULL}};
const Segment SGI109895[] = {{&S6578[30], 6}, {NULL}};
const Segment SGI109896[] = {{&S6578[31], 3}, {NULL}};
const Segment SGI109897[] = {{&S6578[80], 6}, {NULL}};
const Segment SGI109898[] = {{&S6579[44], 5}, {NULL}};
const Segment SGI109899[] = {{&S6579[44], 17}, {NULL}};
const Segment SGI109900[] = {{&S6579[30], 6}, {NULL}};
const Segment SGI109901[] = {{&S6579[31], 3}, {NULL}};
const Segment SGI109902[] = {{&S6579[75], 6}, {NULL}};
const Segment SGI109903[] = {{&S6580[44], 14}, {NULL}};
const Segment SGI109904[] = {{&S6580[44], 37}, {NULL}};
const Segment SGI109905[] = {{&S6580[30], 6}, {NULL}};
const Segment SGI109906[] = {{&S6580[31], 3}, {NULL}};
const Segment SGI109907[] = {{&S6580[95], 6}, {NULL}};
const Segment SGI109908[] = {{&S6581[44], 12}, {NULL}};
const Segment SGI109909[] = {{&S6581[44], 24}, {NULL}};
const Segment SGI109910[] = {{&S6581[30], 6}, {NULL}};
const Segment SGI109911[] = {{&S6581[31], 3}, {NULL}};
const Segment SGI109912[] = {{&S6581[82], 6}, {NULL}};
const Segment SGI109913[] = {{&S6582[44], 11}, {NULL}};
const Segment SGI109914[] = {{&S6582[44], 23}, {NULL}};
const Segment SGI109915[] = {{&S6582[30], 6}, {NULL}};
const Segment SGI109916[] = {{&S6582[31], 3}, {NULL}};
const Segment SGI109917[] = {{&S6582[81], 6}, {NULL}};
const Segment SGI109918[] = {{&S6583[44], 5}, {NULL}};
const Segment SGI109919[] = {{&S6583[44], 17}, {NULL}};
const Segment SGI109920[] = {{&S6583[30], 6}, {NULL}};
const Segment SGI109921[] = {{&S6583[31], 3}, {NULL}};
const Segment SGI109922[] = {{&S6583[75], 6}, {NULL}};
const Segment SGI109923[] = {{&S6584[44], 12}, {NULL}};
const Segment SGI109924[] = {{&S6584[44], 30}, {NULL}};
const Segment SGI109925[] = {{&S6584[30], 6}, {NULL}};
const Segment SGI109926[] = {{&S6584[31], 3}, {NULL}};
const Segment SGI109927[] = {{&S6584[88], 6}, {NULL}};
const Segment SGI109928[] = {{&S6587[44], 13}, {NULL}};
const Segment SGI109929[] = {{&S6587[44], 25}, {NULL}};
const Segment SGI109930[] = {{&S6587[30], 6}, {NULL}};
const Segment SGI109931[] = {{&S6587[31], 3}, {NULL}};
const Segment SGI109932[] = {{&S6587[83], 6}, {NULL}};
const Segment SGI109933[] = {{&S6588[44], 12}, {NULL}};
const Segment SGI109934[] = {{&S6588[44], 24}, {NULL}};
const Segment SGI109935[] = {{&S6588[30], 6}, {NULL}};
const Segment SGI109936[] = {{&S6588[31], 3}, {NULL}};
const Segment SGI109937[] = {{&S6588[82], 6}, {NULL}};
const Segment SGI109938[] = {{&S6589[44], 9}, {NULL}};
const Segment SGI109939[] = {{&S6589[44], 21}, {NULL}};
const Segment SGI109940[] = {{&S6589[30], 6}, {NULL}};
const Segment SGI109941[] = {{&S6589[31], 3}, {NULL}};
const Segment SGI109942[] = {{&S6589[79], 6}, {NULL}};
const Segment SGI109943[] = {{&S6590[44], 6}, {NULL}};
const Segment SGI109944[] = {{&S6590[44], 18}, {NULL}};
const Segment SGI109945[] = {{&S6590[30], 6}, {NULL}};
const Segment SGI109946[] = {{&S6590[31], 3}, {NULL}};
const Segment SGI109947[] = {{&S6590[76], 6}, {NULL}};
const Segment SGI109948[] = {{&S6592[44], 7}, {NULL}};
const Segment SGI109949[] = {{&S6592[44], 19}, {NULL}};
const Segment SGI109950[] = {{&S6592[30], 6}, {NULL}};
const Segment SGI109951[] = {{&S6592[31], 3}, {NULL}};
const Segment SGI109952[] = {{&S6592[77], 6}, {NULL}};
const Segment SGI109953[] = {{&S6593[44], 11}, {NULL}};
const Segment SGI109954[] = {{&S6593[44], 23}, {NULL}};
const Segment SGI109955[] = {{&S6593[30], 6}, {NULL}};
const Segment SGI109956[] = {{&S6593[31], 3}, {NULL}};
const Segment SGI109957[] = {{&S6593[81], 6}, {NULL}};
const Segment SGI109958[] = {{&S6594[44], 7}, {NULL}};
const Segment SGI109959[] = {{&S6594[44], 19}, {NULL}};
const Segment SGI109960[] = {{&S6594[30], 6}, {NULL}};
const Segment SGI109961[] = {{&S6594[31], 3}, {NULL}};
const Segment SGI109962[] = {{&S6594[77], 6}, {NULL}};
const Segment SGI109963[] = {{&S6595[44], 8}, {NULL}};
const Segment SGI109964[] = {{&S6595[44], 20}, {NULL}};
const Segment SGI109965[] = {{&S6595[30], 6}, {NULL}};
const Segment SGI109966[] = {{&S6595[31], 3}, {NULL}};
const Segment SGI109967[] = {{&S6595[78], 6}, {NULL}};
const Segment SGI109968[] = {{&S6596[44], 4}, {NULL}};
const Segment SGI109969[] = {{&S6596[44], 16}, {NULL}};
const Segment SGI109970[] = {{&S6596[30], 6}, {NULL}};
const Segment SGI109971[] = {{&S6596[31], 3}, {NULL}};
const Segment SGI109972[] = {{&S6596[74], 6}, {NULL}};
const Segment SGI109973[] = {{&S89253[34], 11}, {NULL}};
const Segment SGI109974[] = {{&S89253[27], 6}, {NULL}};
const Segment SGI109975[] = {{&S6597[44], 4}, {NULL}};
const Segment SGI109976[] = {{&S6597[44], 27}, {NULL}};
const Segment SGI109977[] = {{&S6597[30], 6}, {NULL}};
const Segment SGI109978[] = {{&S89253[98], 9}, {NULL}};
const Segment SGI109979[] = {{&S89253[71], 6}, {NULL}};
const Segment SGI109980[] = {{&S6597[31], 3}, {NULL}};
const Segment SGI109981[] = {{&S6597[85], 6}, {NULL}};
const Segment SGI109982[] = {{&S6598[44], 5}, {NULL}};
const Segment SGI109983[] = {{&S6598[44], 17}, {NULL}};
const Segment SGI109984[] = {{&S6598[30], 6}, {NULL}};
const Segment SGI109985[] = {{&S6598[31], 3}, {NULL}};
const Segment SGI109986[] = {{&S89255[44], 10}, {NULL}};
const Segment SGI109987[] = {{&S89255[44], 32}, {NULL}};
const Segment SGI109988[] = {{&S6598[75], 6}, {NULL}};
const Segment SGI109989[] = {{&S89255[30], 6}, {NULL}};
const Segment SGI109990[] = {{&S89255[31], 3}, {NULL}};
const Segment SGI109991[] = {{&S89255[90], 6}, {NULL}};
const Segment SGI109992[] = {{&S89256[44], 10}, {NULL}};
const Segment SGI109993[] = {{&S89256[44], 29}, {NULL}};
const Segment SGI109994[] = {{&S89256[30], 6}, {NULL}};
const Segment SGI109995[] = {{&S89256[31], 3}, {NULL}};
const Segment SGI109996[] = {{&S89256[87], 6}, {NULL}};
const Segment SGI109997[] = {{&S6601[44], 11}, {NULL}};
const Segment SGI109998[] = {{&S6601[44], 23}, {NULL}};
const Segment SGI109999[] = {{&S6601[30], 6}, {NULL}};
const Segment SGI110000[] = {{&S6601[31], 3}, {NULL}};
const Segment SGI110001[] = {{&S6601[81], 6}, {NULL}};
const Segment SGI110002[] = {{&S6602[44], 5}, {NULL}};
const Segment SGI110003[] = {{&S6602[44], 17}, {NULL}};
const Segment SGI110004[] = {{&S6602[30], 6}, {NULL}};
const Segment SGI110005[] = {{&S6602[31], 3}, {NULL}};
const Segment SGI110006[] = {{&S6602[75], 6}, {NULL}};
const Segment SGI110007[] = {{&S6603[44], 13}, {NULL}};
const Segment SGI110008[] = {{&S6603[44], 25}, {NULL}};
const Segment SGI110009[] = {{&S6603[30], 6}, {NULL}};
const Segment SGI110010[] = {{&S6603[31], 3}, {NULL}};
const Segment SGI110011[] = {{&S6603[83], 6}, {NULL}};
const Segment SGI110012[] = {{&S6604[44], 8}, {NULL}};
const Segment SGI110013[] = {{&S6604[44], 20}, {NULL}};
const Segment SGI110014[] = {{&S6604[30], 6}, {NULL}};
const Segment SGI110015[] = {{&S6604[31], 3}, {NULL}};
const Segment SGI110016[] = {{&S6604[78], 5}, {NULL}};
const Segment SGI110017[] = {{&S6605[44], 8}, {NULL}};
const Segment SGI110018[] = {{&S6605[44], 20}, {NULL}};
const Segment SGI110019[] = {{&S6605[30], 6}, {NULL}};
const Segment SGI110020[] = {{&S6605[31], 3}, {NULL}};
const Segment SGI110021[] = {{&S6605[78], 6}, {NULL}};
const Segment SGI110022[] = {{&S6606[44], 7}, {NULL}};
const Segment SGI110023[] = {{&S6606[44], 19}, {NULL}};
const Segment SGI110024[] = {{&S6606[30], 6}, {NULL}};
const Segment SGI110025[] = {{&S6606[31], 3}, {NULL}};
const Segment SGI110026[] = {{&S6606[77], 6}, {NULL}};
const Segment SGI110027[] = {{&S6607[44], 4}, {NULL}};
const Segment SGI110028[] = {{&S6607[44], 16}, {NULL}};
const Segment SGI110029[] = {{&S6607[30], 6}, {NULL}};
const Segment SGI110030[] = {{&S6607[31], 3}, {NULL}};
const Segment SGI110031[] = {{&S6607[74], 6}, {NULL}};
const Segment SGI110032[] = {{&S104066[42], 10}, {NULL}};
const Segment SGI110033[] = {{&S104066[42], 21}, {NULL}};
const Segment SGI110034[] = {{&S104066[30], 4}, {NULL}};
const Segment SGI110035[] = {{&S104066[112], 3}, {NULL}};
const Segment SGI110036[] = {{&S104066[77], 6}, {NULL}};
const Segment SGI110037[] = {{&S6609[44], 13}, {NULL}};
const Segment SGI110038[] = {{&S6609[44], 25}, {NULL}};
const Segment SGI110039[] = {{&S104066[129], 10}, {NULL}};
const Segment SGI110040[] = {{&S6609[30], 6}, {NULL}};
const Segment SGI110041[] = {{&S6609[31], 3}, {NULL}};
const Segment SGI110042[] = {{&S6609[83], 6}, {NULL}};
const Segment SGI110043[] = {{&S6610[44], 9}, {NULL}};
const Segment SGI110044[] = {{&S6610[44], 21}, {NULL}};
const Segment SGI110045[] = {{&S6610[30], 6}, {NULL}};
const Segment SGI110046[] = {{&S6610[31], 3}, {NULL}};
const Segment SGI110047[] = {{&S6610[79], 6}, {NULL}};
const Segment SGI110048[] = {{&S104070[57], 10}, {NULL}};
const Segment SGI110049[] = {{&S104070[57], 22}, {NULL}};
const Segment SGI110050[] = {{&S104070[45], 4}, {NULL}};
const Segment SGI110051[] = {{&S6612[44], 9}, {NULL}};
const Segment SGI110052[] = {{&S6612[44], 21}, {NULL}};
const Segment SGI110053[] = {{&S6612[30], 6}, {NULL}};
const Segment SGI110054[] = {{&S104070[128], 3}, {NULL}};
const Segment SGI110055[] = {{&S6612[31], 3}, {NULL}};
const Segment SGI110056[] = {{&S104070[93], 6}, {NULL}};
const Segment SGI110057[] = {{&S6612[79], 6}, {NULL}};
const Segment SGI110058[] = {{&S104070[0], 14}, {NULL}};
const Segment SGI110059[] = {{&S104071[57], 10}, {NULL}};
const Segment SGI110060[] = {{&S104071[57], 22}, {NULL}};
const Segment SGI110061[] = {{&S104071[45], 4}, {NULL}};
const Segment SGI110062[] = {{&S104071[128], 3}, {NULL}};
const Segment SGI110063[] = {{&S104071[93], 6}, {NULL}};
const Segment SGI110064[] = {{&S104071[0], 14}, {NULL}};
const Segment SGI110065[] = {{&S6614[44], 11}, {NULL}};
const Segment SGI110066[] = {{&S6614[44], 23}, {NULL}};
const Segment SGI110067[] = {{&S6614[30], 6}, {NULL}};
const Segment SGI110068[] = {{&S6614[31], 3}, {NULL}};
const Segment SGI110069[] = {{&S6614[81], 6}, {NULL}};
const Segment SGI110070[] = {{&S6615[44], 14}, {NULL}};
const Segment SGI110071[] = {{&S6615[44], 26}, {NULL}};
const Segment SGI110072[] = {{&S6615[30], 6}, {NULL}};
const Segment SGI110073[] = {{&S6615[31], 3}, {NULL}};
const Segment SGI110074[] = {{&S6615[84], 6}, {NULL}};
const Segment SGI110075[] = {{&S6616[44], 8}, {NULL}};
const Segment SGI110076[] = {{&S6616[44], 20}, {NULL}};
const Segment SGI110077[] = {{&S6616[30], 6}, {NULL}};
const Segment SGI110078[] = {{&S6616[31], 3}, {NULL}};
const Segment SGI110079[] = {{&S104075[57], 10}, {NULL}};
const Segment SGI110080[] = {{&S6616[78], 6}, {NULL}};
const Segment SGI110081[] = {{&S104075[57], 22}, {NULL}};
const Segment SGI110082[] = {{&S104075[45], 4}, {NULL}};
const Segment SGI110083[] = {{&S104075[128], 3}, {NULL}};
const Segment SGI110084[] = {{&S104075[93], 6}, {NULL}};
const Segment SGI110085[] = {{&S104075[0], 14}, {NULL}};
const Segment SGI110086[] = {{&S6617[44], 15}, {NULL}};
const Segment SGI110087[] = {{&S6617[44], 27}, {NULL}};
const Segment SGI110088[] = {{&S6617[30], 6}, {NULL}};
const Segment SGI110089[] = {{&S6617[31], 3}, {NULL}};
const Segment SGI110090[] = {{&S6617[85], 6}, {NULL}};
const Segment SGI110091[] = {{&S104076[57], 10}, {NULL}};
const Segment SGI110092[] = {{&S104076[57], 22}, {NULL}};
const Segment SGI110093[] = {{&S104076[45], 4}, {NULL}};
const Segment SGI110094[] = {{&S104076[128], 3}, {NULL}};
const Segment SGI110095[] = {{&S104076[93], 6}, {NULL}};
const Segment SGI110096[] = {{&S104076[0], 14}, {NULL}};
const Segment SGI110097[] = {{&S6619[44], 8}, {NULL}};
const Segment SGI110098[] = {{&S6619[44], 16}, {NULL}};
const Segment SGI110099[] = {{&S6619[30], 6}, {NULL}};
const Segment SGI110100[] = {{&S6619[31], 3}, {NULL}};
const Segment SGI110101[] = {{&S6619[74], 6}, {NULL}};
const Segment SGI110102[] = {{&S104077[42], 10}, {NULL}};
const Segment SGI110103[] = {{&S104077[42], 22}, {NULL}};
const Segment SGI110104[] = {{&S104077[30], 4}, {NULL}};
const Segment SGI110105[] = {{&S104077[113], 3}, {NULL}};
const Segment SGI110106[] = {{&S104077[78], 6}, {NULL}};
const Segment SGI110107[] = {{&S104077[130], 10}, {NULL}};
const Segment SGI110108[] = {{&S6620[44], 13}, {NULL}};
const Segment SGI110109[] = {{&S6620[44], 25}, {NULL}};
const Segment SGI110110[] = {{&S6620[30], 6}, {NULL}};
const Segment SGI110111[] = {{&S6620[31], 3}, {NULL}};
const Segment SGI110112[] = {{&S6620[83], 6}, {NULL}};
const Segment SGI110113[] = {{&S6621[44], 8}, {NULL}};
const Segment SGI110114[] = {{&S6621[44], 20}, {NULL}};
const Segment SGI110115[] = {{&S6621[30], 6}, {NULL}};
const Segment SGI110116[] = {{&S6621[31], 3}, {NULL}};
const Segment SGI110117[] = {{&S6621[78], 6}, {NULL}};
const Segment SGI110118[] = {{&S6627[44], 29}, {NULL}};
const Segment SGI110119[] = {{&S6627[44], 41}, {NULL}};
const Segment SGI110120[] = {{&S6627[30], 6}, {NULL}};
const Segment SGI110121[] = {{&S6627[31], 3}, {NULL}};
const Segment SGI110122[] = {{&S6627[99], 6}, {NULL}};
const Segment SGI110123[] = {{&S6628[44], 9}, {NULL}};
const Segment SGI110124[] = {{&S6628[44], 21}, {NULL}};
const Segment SGI110125[] = {{&S6628[30], 6}, {NULL}};
const Segment SGI110126[] = {{&S6628[31], 3}, {NULL}};
const Segment SGI110127[] = {{&S6628[79], 6}, {NULL}};
const Segment SGI110128[] = {{&S104088[42], 10}, {NULL}};
const Segment SGI110129[] = {{&S6629[44], 21}, {NULL}};
const Segment SGI110130[] = {{&S104088[42], 22}, {NULL}};
const Segment SGI110131[] = {{&S6629[44], 33}, {NULL}};
const Segment SGI110132[] = {{&S104088[30], 4}, {NULL}};
const Segment SGI110133[] = {{&S6629[30], 6}, {NULL}};
const Segment SGI110134[] = {{&S104088[113], 3}, {NULL}};
const Segment SGI110135[] = {{&S6629[31], 3}, {NULL}};
const Segment SGI110136[] = {{&S6629[91], 6}, {NULL}};
const Segment SGI110137[] = {{&S104088[78], 6}, {NULL}};
const Segment SGI110138[] = {{&S104088[131], 13}, {NULL}};
const Segment SGI110139[] = {{&S104089[42], 11}, {NULL}};
const Segment SGI110140[] = {{&S104089[42], 23}, {NULL}};
const Segment SGI110141[] = {{&S104089[30], 4}, {NULL}};
const Segment SGI110142[] = {{&S104089[114], 3}, {NULL}};
const Segment SGI110143[] = {{&S6631[44], 7}, {NULL}};
const Segment SGI110144[] = {{&S6631[44], 19}, {NULL}};
const Segment SGI110145[] = {{&S6631[30], 6}, {NULL}};
const Segment SGI110146[] = {{&S104089[79], 6}, {NULL}};
const Segment SGI110147[] = {{&S104089[132], 13}, {NULL}};
const Segment SGI110148[] = {{&S6631[31], 3}, {NULL}};
const Segment SGI110149[] = {{&S6631[77], 6}, {NULL}};
const Segment SGI110150[] = {{&S104091[57], 10}, {NULL}};
const Segment SGI110151[] = {{&S6633[38], 6}, {NULL}};
const Segment SGI110152[] = {{&S104091[57], 22}, {NULL}};
const Segment SGI110153[] = {{&S6633[31], 6}, {NULL}};
const Segment SGI110154[] = {{&S104091[45], 4}, {NULL}};
const Segment SGI110155[] = {{&S104091[128], 3}, {NULL}};
const Segment SGI110156[] = {{&S104091[93], 6}, {NULL}};
const Segment SGI110157[] = {{&S104091[0], 14}, {NULL}};
const Segment SGI110158[] = {{&S6634[53], 6}, {NULL}};
const Segment SGI110159[] = {{&S104092[57], 10}, {NULL}};
const Segment SGI110160[] = {{&S104092[57], 22}, {NULL}};
const Segment SGI110161[] = {{&S6634[115], 6}, {NULL}};
const Segment SGI110162[] = {{&S104092[45], 4}, {NULL}};
const Segment SGI110163[] = {{&S104092[128], 3}, {NULL}};
const Segment SGI110164[] = {{&S104092[93], 6}, {NULL}};
const Segment SGI110165[] = {{&S104092[0], 14}, {NULL}};
const Segment SGI110166[] = {{&S6638[44], 7}, {NULL}};
const Segment SGI110167[] = {{&S6638[44], 19}, {NULL}};
const Segment SGI110168[] = {{&S6638[30], 6}, {NULL}};
const Segment SGI110169[] = {{&S6638[31], 3}, {NULL}};
const Segment SGI110170[] = {{&S6638[77], 6}, {NULL}};
const Segment SGI110171[] = {{&S104096[42], 10}, {NULL}};
const Segment SGI110172[] = {{&S104096[42], 22}, {NULL}};
const Segment SGI110173[] = {{&S104096[30], 4}, {NULL}};
const Segment SGI110174[] = {{&S104096[156], 3}, {NULL}};
const Segment SGI110175[] = {{&S104096[122], 3}, {NULL}};
const Segment SGI110176[] = {{&S104096[87], 6}, {NULL}};
const Segment SGI110177[] = {{&S104096[142], 8}, {NULL}};
const Segment SGI110178[] = {{&S6640[44], 9}, {NULL}};
const Segment SGI110179[] = {{&S6640[44], 27}, {NULL}};
const Segment SGI110180[] = {{&S6640[30], 6}, {NULL}};
const Segment SGI110181[] = {{&S6640[31], 3}, {NULL}};
const Segment SGI110182[] = {{&S6640[85], 5}, {NULL}};
const Segment SGI110183[] = {{&S6641[44], 8}, {NULL}};
const Segment SGI110184[] = {{&S6641[44], 20}, {NULL}};
const Segment SGI110185[] = {{&S6641[30], 6}, {NULL}};
const Segment SGI110186[] = {{&S6641[31], 3}, {NULL}};
const Segment SGI110187[] = {{&S6641[78], 6}, {NULL}};
const Segment SGI110188[] = {{&S6642[44], 5}, {NULL}};
const Segment SGI110189[] = {{&S6642[44], 17}, {NULL}};
const Segment SGI110190[] = {{&S6642[30], 6}, {NULL}};
const Segment SGI110191[] = {{&S6642[31], 3}, {NULL}};
const Segment SGI110192[] = {{&S6642[75], 6}, {NULL}};
const Segment SGI110193[] = {{&S104100[42], 10}, {NULL}};
const Segment SGI110194[] = {{&S104100[42], 22}, {NULL}};
const Segment SGI110195[] = {{&S104100[30], 4}, {NULL}};
const Segment SGI110196[] = {{&S104100[113], 3}, {NULL}};
const Segment SGI110197[] = {{&S104100[78], 6}, {NULL}};
const Segment SGI110198[] = {{&S104100[130], 10}, {NULL}};
const Segment SGI110199[] = {{&S6643[44], 11}, {NULL}};
const Segment SGI110200[] = {{&S6643[44], 23}, {NULL}};
const Segment SGI110201[] = {{&S6643[30], 6}, {NULL}};
const Segment SGI110202[] = {{&S6643[31], 3}, {NULL}};
const Segment SGI110203[] = {{&S6643[81], 6}, {NULL}};
const Segment SGI110204[] = {{&S6644[44], 8}, {NULL}};
const Segment SGI110205[] = {{&S6644[44], 20}, {NULL}};
const Segment SGI110206[] = {{&S6644[30], 6}, {NULL}};
const Segment SGI110207[] = {{&S6644[31], 3}, {NULL}};
const Segment SGI110208[] = {{&S6644[78], 6}, {NULL}};
const Segment SGI110209[] = {{&S104103[42], 10}, {NULL}};
const Segment SGI110210[] = {{&S104103[42], 22}, {NULL}};
const Segment SGI110211[] = {{&S104103[30], 4}, {NULL}};
const Segment SGI110212[] = {{&S104103[113], 3}, {NULL}};
const Segment SGI110213[] = {{&S6645[44], 17}, {NULL}};
const Segment SGI110214[] = {{&S6645[44], 29}, {NULL}};
const Segment SGI110215[] = {{&S6645[30], 6}, {NULL}};
const Segment SGI110216[] = {{&S104103[78], 6}, {NULL}};
const Segment SGI110217[] = {{&S104103[131], 11}, {NULL}};
const Segment SGI110218[] = {{&S6645[31], 3}, {NULL}};
const Segment SGI110219[] = {{&S6645[87], 6}, {NULL}};
const Segment SGI110220[] = {{&S6646[44], 7}, {NULL}};
const Segment SGI110221[] = {{&S6646[44], 19}, {NULL}};
const Segment SGI110222[] = {{&S6646[30], 6}, {NULL}};
const Segment SGI110223[] = {{&S6646[31], 3}, {NULL}};
const Segment SGI110224[] = {{&S6646[77], 6}, {NULL}};
const Segment SGI110225[] = {{&S6647[44], 8}, {NULL}};
const Segment SGI110226[] = {{&S6647[44], 17}, {NULL}};
const Segment SGI110227[] = {{&S6647[30], 6}, {NULL}};
const Segment SGI110228[] = {{&S6647[31], 3}, {NULL}};
const Segment SGI110229[] = {{&S6647[75], 6}, {NULL}};
const Segment SGI110230[] = {{&S6648[44], 10}, {NULL}};
const Segment SGI110231[] = {{&S6648[44], 22}, {NULL}};
const Segment SGI110232[] = {{&S6648[30], 6}, {NULL}};
const Segment SGI110233[] = {{&S6648[31], 3}, {NULL}};
const Segment SGI110234[] = {{&S6648[80], 6}, {NULL}};
const Segment SGI110235[] = {{&S6649[44], 21}, {NULL}};
const Segment SGI110236[] = {{&S6649[44], 33}, {NULL}};
const Segment SGI110237[] = {{&S6649[30], 6}, {NULL}};
const Segment SGI110238[] = {{&S6649[31], 3}, {NULL}};
const Segment SGI110239[] = {{&S6649[91], 6}, {NULL}};
const Segment SGI110240[] = {{&S6650[44], 5}, {NULL}};
const Segment SGI110241[] = {{&S6650[44], 17}, {NULL}};
const Segment SGI110242[] = {{&S6650[30], 6}, {NULL}};
const Segment SGI110243[] = {{&S6650[31], 3}, {NULL}};
const Segment SGI110244[] = {{&S6650[75], 6}, {NULL}};
const Segment SGI110245[] = {{&S6651[44], 7}, {NULL}};
const Segment SGI110246[] = {{&S6651[44], 19}, {NULL}};
const Segment SGI110247[] = {{&S6651[30], 6}, {NULL}};
const Segment SGI110248[] = {{&S6651[31], 3}, {NULL}};
const Segment SGI110249[] = {{&S6651[77], 6}, {NULL}};
const Segment SGI110250[] = {{&S6652[44], 6}, {NULL}};
const Segment SGI110251[] = {{&S6652[44], 18}, {NULL}};
const Segment SGI110252[] = {{&S6652[30], 6}, {NULL}};
const Segment SGI110253[] = {{&S6652[31], 3}, {NULL}};
const Segment SGI110254[] = {{&S6652[76], 6}, {NULL}};
const Segment SGI110255[] = {{&S6653[44], 7}, {NULL}};
const Segment SGI110256[] = {{&S6653[44], 19}, {NULL}};
const Segment SGI110257[] = {{&S6653[30], 6}, {NULL}};
const Segment SGI110258[] = {{&S6653[31], 3}, {NULL}};
const Segment SGI110259[] = {{&S6653[77], 6}, {NULL}};
const Segment SGI110260[] = {{&S6654[44], 13}, {NULL}};
const Segment SGI110261[] = {{&S6654[44], 25}, {NULL}};
const Segment SGI110262[] = {{&S6654[30], 6}, {NULL}};
const Segment SGI110263[] = {{&S6654[31], 3}, {NULL}};
const Segment SGI110264[] = {{&S6654[83], 6}, {NULL}};
const Segment SGI110265[] = {{&S6655[44], 8}, {NULL}};
const Segment SGI110266[] = {{&S6655[44], 20}, {NULL}};
const Segment SGI110267[] = {{&S6655[30], 6}, {NULL}};
const Segment SGI110268[] = {{&S6655[31], 3}, {NULL}};
const Segment SGI110269[] = {{&S6655[78], 6}, {NULL}};
const Segment SGI110270[] = {{&S6657[44], 16}, {NULL}};
const Segment SGI110271[] = {{&S6657[44], 28}, {NULL}};
const Segment SGI110272[] = {{&S6657[30], 6}, {NULL}};
const Segment SGI110273[] = {{&S6657[31], 3}, {NULL}};
const Segment SGI110274[] = {{&S6657[86], 6}, {NULL}};
const Segment SGI110275[] = {{&S6658[44], 5}, {NULL}};
const Segment SGI110276[] = {{&S6658[44], 17}, {NULL}};
const Segment SGI110277[] = {{&S6658[30], 6}, {NULL}};
const Segment SGI110278[] = {{&S6658[31], 3}, {NULL}};
const Segment SGI110279[] = {{&S6658[75], 6}, {NULL}};
const Segment SGI110280[] = {{&S6659[44], 6}, {NULL}};
const Segment SGI110281[] = {{&S6659[44], 18}, {NULL}};
const Segment SGI110282[] = {{&S6659[30], 6}, {NULL}};
const Segment SGI110283[] = {{&S6659[31], 3}, {NULL}};
const Segment SGI110284[] = {{&S6659[76], 6}, {NULL}};
const Segment SGI110285[] = {{&S6661[44], 8}, {NULL}};
const Segment SGI110286[] = {{&S6661[44], 20}, {NULL}};
const Segment SGI110287[] = {{&S6661[30], 6}, {NULL}};
const Segment SGI110288[] = {{&S6661[31], 3}, {NULL}};
const Segment SGI110289[] = {{&S6661[78], 6}, {NULL}};
const Segment SGI110290[] = {{&S6662[44], 7}, {NULL}};
const Segment SGI110291[] = {{&S6662[44], 19}, {NULL}};
const Segment SGI110292[] = {{&S6662[30], 6}, {NULL}};
const Segment SGI110293[] = {{&S6662[31], 3}, {NULL}};
const Segment SGI110294[] = {{&S6662[77], 5}, {NULL}};
const Segment SGI110295[] = {{&S6663[44], 13}, {NULL}};
const Segment SGI110296[] = {{&S6663[44], 30}, {NULL}};
const Segment SGI110297[] = {{&S6663[30], 6}, {NULL}};
const Segment SGI110298[] = {{&S6663[31], 3}, {NULL}};
const Segment SGI110299[] = {{&S6663[88], 6}, {NULL}};
const Segment SGI110300[] = {{&S6666[44], 6}, {NULL}};
const Segment SGI110301[] = {{&S6666[44], 18}, {NULL}};
const Segment SGI110302[] = {{&S6666[30], 6}, {NULL}};
const Segment SGI110303[] = {{&S6666[31], 3}, {NULL}};
const Segment SGI110304[] = {{&S6666[76], 6}, {NULL}};
const Segment SGI110305[] = {{&S6667[43], 15}, {NULL}};
const Segment SGI110306[] = {{&S6667[43], 27}, {NULL}};
const Segment SGI110307[] = {{&S6667[30], 6}, {NULL}};
const Segment SGI110308[] = {{&S6667[31], 3}, {NULL}};
const Segment SGI110309[] = {{&S6667[84], 6}, {NULL}};
const Segment SGI110310[] = {{&S6676[44], 6}, {NULL}};
const Segment SGI110311[] = {{&S6676[44], 18}, {NULL}};
const Segment SGI110312[] = {{&S6676[30], 6}, {NULL}};
const Segment SGI110313[] = {{&S6676[31], 3}, {NULL}};
const Segment SGI110314[] = {{&S6676[76], 6}, {NULL}};
const Segment SGI110315[] = {{&S6677[44], 6}, {NULL}};
const Segment SGI110316[] = {{&S6677[44], 18}, {NULL}};
const Segment SGI110317[] = {{&S6677[30], 6}, {NULL}};
const Segment SGI110318[] = {{&S6677[31], 3}, {NULL}};
const Segment SGI110319[] = {{&S6677[76], 6}, {NULL}};
const Segment SGI110320[] = {{&S6683[44], 6}, {NULL}};
const Segment SGI110321[] = {{&S6683[44], 18}, {NULL}};
const Segment SGI110322[] = {{&S6683[30], 6}, {NULL}};
const Segment SGI110323[] = {{&S6683[31], 3}, {NULL}};
const Segment SGI110324[] = {{&S6683[76], 6}, {NULL}};
const Segment SGI110325[] = {{&S6684[44], 15}, {NULL}};
const Segment SGI110326[] = {{&S6684[44], 27}, {NULL}};
const Segment SGI110327[] = {{&S6684[30], 6}, {NULL}};
const Segment SGI110328[] = {{&S6684[31], 3}, {NULL}};
const Segment SGI110329[] = {{&S89308[34], 6}, {NULL}};
const Segment SGI110330[] = {{&S6684[85], 6}, {NULL}};
const Segment SGI110331[] = {{&S89308[27], 6}, {NULL}};
const Segment SGI110332[] = {{&S89308[93], 9}, {NULL}};
const Segment SGI110333[] = {{&S89308[66], 6}, {NULL}};
const Segment SGI110334[] = {{&S6686[44], 5}, {NULL}};
const Segment SGI110335[] = {{&S6686[44], 17}, {NULL}};
const Segment SGI110336[] = {{&S6686[30], 6}, {NULL}};
const Segment SGI110337[] = {{&S6686[31], 3}, {NULL}};
const Segment SGI110338[] = {{&S6686[75], 6}, {NULL}};
const Segment SGI110339[] = {{&S6687[44], 7}, {NULL}};
const Segment SGI110340[] = {{&S6687[44], 19}, {NULL}};
const Segment SGI110341[] = {{&S6687[30], 6}, {NULL}};
const Segment SGI110342[] = {{&S6687[31], 3}, {NULL}};
const Segment SGI110343[] = {{&S6687[77], 6}, {NULL}};
const Segment SGI110344[] = {{&S6688[44], 9}, {NULL}};
const Segment SGI110345[] = {{&S6688[44], 21}, {NULL}};
const Segment SGI110346[] = {{&S6688[30], 6}, {NULL}};
const Segment SGI110347[] = {{&S6688[31], 3}, {NULL}};
const Segment SGI110348[] = {{&S6688[79], 6}, {NULL}};
const Segment SGI110349[] = {{&S6689[44], 6}, {NULL}};
const Segment SGI110350[] = {{&S6689[44], 18}, {NULL}};
const Segment SGI110351[] = {{&S6689[30], 6}, {NULL}};
const Segment SGI110352[] = {{&S6689[31], 3}, {NULL}};
const Segment SGI110353[] = {{&S6689[76], 6}, {NULL}};
const Segment SGI110354[] = {{&S6691[44], 9}, {NULL}};
const Segment SGI110355[] = {{&S6691[44], 21}, {NULL}};
const Segment SGI110356[] = {{&S6691[30], 6}, {NULL}};
const Segment SGI110357[] = {{&S6691[31], 3}, {NULL}};
const Segment SGI110358[] = {{&S6691[79], 6}, {NULL}};
const Segment SGI110359[] = {{&S6692[44], 6}, {NULL}};
const Segment SGI110360[] = {{&S6692[44], 18}, {NULL}};
const Segment SGI110361[] = {{&S6692[30], 6}, {NULL}};
const Segment SGI110362[] = {{&S6692[31], 3}, {NULL}};
const Segment SGI110363[] = {{&S6692[76], 6}, {NULL}};
const Segment SGI110364[] = {{&S6693[44], 5}, {NULL}};
const Segment SGI110365[] = {{&S6693[44], 17}, {NULL}};
const Segment SGI110366[] = {{&S6693[30], 6}, {NULL}};
const Segment SGI110367[] = {{&S6693[31], 3}, {NULL}};
const Segment SGI110368[] = {{&S6693[75], 6}, {NULL}};
const Segment SGI110369[] = {{&S6694[44], 11}, {NULL}};
const Segment SGI110370[] = {{&S6694[44], 23}, {NULL}};
const Segment SGI110371[] = {{&S6694[30], 6}, {NULL}};
const Segment SGI110372[] = {{&S6694[31], 3}, {NULL}};
const Segment SGI110373[] = {{&S6694[81], 6}, {NULL}};
const Segment SGI110374[] = {{&S89325[34], 15}, {NULL}};
const Segment SGI110375[] = {{&S89325[27], 6}, {NULL}};
const Segment SGI110376[] = {{&S89325[102], 9}, {NULL}};
const Segment SGI110377[] = {{&S89325[75], 6}, {NULL}};
const Segment SGI110378[] = {{&S6695[44], 12}, {NULL}};
const Segment SGI110379[] = {{&S6695[44], 24}, {NULL}};
const Segment SGI110380[] = {{&S6695[30], 6}, {NULL}};
const Segment SGI110381[] = {{&S6695[31], 3}, {NULL}};
const Segment SGI110382[] = {{&S6695[82], 6}, {NULL}};
const Segment SGI110383[] = {{&S6697[44], 7}, {NULL}};
const Segment SGI110384[] = {{&S6697[44], 19}, {NULL}};
const Segment SGI110385[] = {{&S6697[30], 6}, {NULL}};
const Segment SGI110386[] = {{&S6697[31], 3}, {NULL}};
const Segment SGI110387[] = {{&S6697[77], 6}, {NULL}};
const Segment SGI110388[] = {{&S6698[44], 8}, {NULL}};
const Segment SGI110389[] = {{&S6698[44], 20}, {NULL}};
const Segment SGI110390[] = {{&S6698[30], 6}, {NULL}};
const Segment SGI110391[] = {{&S6698[31], 3}, {NULL}};
const Segment SGI110392[] = {{&S6698[78], 6}, {NULL}};
const Segment SGI110393[] = {{&S6699[44], 8}, {NULL}};
const Segment SGI110394[] = {{&S6699[44], 20}, {NULL}};
const Segment SGI110395[] = {{&S6699[30], 6}, {NULL}};
const Segment SGI110396[] = {{&S6699[31], 3}, {NULL}};
const Segment SGI110397[] = {{&S6699[78], 6}, {NULL}};
const Segment SGI110398[] = {{&S6701[44], 7}, {NULL}};
const Segment SGI110399[] = {{&S6701[44], 19}, {NULL}};
const Segment SGI110400[] = {{&S6701[30], 6}, {NULL}};
const Segment SGI110401[] = {{&S6701[31], 3}, {NULL}};
const Segment SGI110402[] = {{&S89335[34], 14}, {NULL}};
const Segment SGI110403[] = {{&S6701[77], 6}, {NULL}};
const Segment SGI110404[] = {{&S89335[27], 6}, {NULL}};
const Segment SGI110405[] = {{&S89335[106], 9}, {NULL}};
const Segment SGI110406[] = {{&S89335[79], 6}, {NULL}};
const Segment SGI110407[] = {{&S89335[147], 8}, {NULL}};
const Segment SGI110408[] = {{&S6702[44], 16}, {NULL}};
const Segment SGI110409[] = {{&S6702[44], 28}, {NULL}};
const Segment SGI110410[] = {{&S6702[30], 6}, {NULL}};
const Segment SGI110411[] = {{&S6702[31], 3}, {NULL}};
const Segment SGI110412[] = {{&S6702[86], 6}, {NULL}};
const Segment SGI110413[] = {{&S6704[44], 17}, {NULL}};
const Segment SGI110414[] = {{&S6704[44], 29}, {NULL}};
const Segment SGI110415[] = {{&S6704[30], 6}, {NULL}};
const Segment SGI110416[] = {{&S6704[31], 3}, {NULL}};
const Segment SGI110417[] = {{&S6704[87], 6}, {NULL}};
const Segment SGI110418[] = {{&S6705[44], 6}, {NULL}};
const Segment SGI110419[] = {{&S6705[44], 18}, {NULL}};
const Segment SGI110420[] = {{&S6705[30], 6}, {NULL}};
const Segment SGI110421[] = {{&S6705[31], 3}, {NULL}};
const Segment SGI110422[] = {{&S6705[76], 6}, {NULL}};
const Segment SGI110423[] = {{&S6707[44], 14}, {NULL}};
const Segment SGI110424[] = {{&S6707[44], 26}, {NULL}};
const Segment SGI110425[] = {{&S6707[30], 6}, {NULL}};
const Segment SGI110426[] = {{&S6707[31], 3}, {NULL}};
const Segment SGI110427[] = {{&S6707[84], 6}, {NULL}};
const Segment SGI110428[] = {{&S89344[44], 10}, {NULL}};
const Segment SGI110429[] = {{&S89344[44], 22}, {NULL}};
const Segment SGI110430[] = {{&S89344[30], 6}, {NULL}};
const Segment SGI110431[] = {{&S89344[31], 3}, {NULL}};
const Segment SGI110432[] = {{&S89344[80], 5}, {NULL}};
const Segment SGI110433[] = {{&S89344[117], 14}, {NULL}};
const Segment SGI110434[] = {{&S6708[44], 7}, {NULL}};
const Segment SGI110435[] = {{&S6708[44], 19}, {NULL}};
const Segment SGI110436[] = {{&S6708[30], 6}, {NULL}};
const Segment SGI110437[] = {{&S6708[31], 3}, {NULL}};
const Segment SGI110438[] = {{&S6708[77], 6}, {NULL}};
const Segment SGI110439[] = {{&S6709[44], 11}, {NULL}};
const Segment SGI110440[] = {{&S6709[44], 23}, {NULL}};
const Segment SGI110441[] = {{&S6709[30], 6}, {NULL}};
const Segment SGI110442[] = {{&S6709[31], 3}, {NULL}};
const Segment SGI110443[] = {{&S6709[81], 6}, {NULL}};
const Segment SGI110444[] = {{&S6711[44], 11}, {NULL}};
const Segment SGI110445[] = {{&S6711[44], 23}, {NULL}};
const Segment SGI110446[] = {{&S6711[30], 6}, {NULL}};
const Segment SGI110447[] = {{&S6711[31], 3}, {NULL}};
const Segment SGI110448[] = {{&S6711[81], 6}, {NULL}};
const Segment SGI110449[] = {{&S6712[44], 9}, {NULL}};
const Segment SGI110450[] = {{&S6712[44], 21}, {NULL}};
const Segment SGI110451[] = {{&S6712[30], 6}, {NULL}};
const Segment SGI110452[] = {{&S6712[31], 3}, {NULL}};
const Segment SGI110453[] = {{&S6712[79], 6}, {NULL}};
const Segment SGI110454[] = {{&S6713[44], 13}, {NULL}};
const Segment SGI110455[] = {{&S6713[44], 25}, {NULL}};
const Segment SGI110456[] = {{&S6713[30], 6}, {NULL}};
const Segment SGI110457[] = {{&S6713[31], 3}, {NULL}};
const Segment SGI110458[] = {{&S6713[83], 6}, {NULL}};
const Segment SGI110459[] = {{&S6714[44], 9}, {NULL}};
const Segment SGI110460[] = {{&S6714[44], 21}, {NULL}};
const Segment SGI110461[] = {{&S6714[30], 6}, {NULL}};
const Segment SGI110462[] = {{&S6714[31], 3}, {NULL}};
const Segment SGI110463[] = {{&S6714[79], 6}, {NULL}};
const Segment SGI110464[] = {{&S6715[44], 7}, {NULL}};
const Segment SGI110465[] = {{&S6715[44], 19}, {NULL}};
const Segment SGI110466[] = {{&S6715[30], 6}, {NULL}};
const Segment SGI110467[] = {{&S6715[31], 3}, {NULL}};
const Segment SGI110468[] = {{&S6715[77], 6}, {NULL}};
const Segment SGI110469[] = {{&S6717[44], 6}, {NULL}};
const Segment SGI110470[] = {{&S6717[44], 18}, {NULL}};
const Segment SGI110471[] = {{&S6717[30], 6}, {NULL}};
const Segment SGI110472[] = {{&S6717[31], 3}, {NULL}};
const Segment SGI110473[] = {{&S6717[76], 6}, {NULL}};
const Segment SGI110474[] = {{&S6718[44], 11}, {NULL}};
const Segment SGI110475[] = {{&S6718[44], 23}, {NULL}};
const Segment SGI110476[] = {{&S6718[30], 6}, {NULL}};
const Segment SGI110477[] = {{&S6718[31], 3}, {NULL}};
const Segment SGI110478[] = {{&S6718[81], 6}, {NULL}};
const Segment SGI110479[] = {{&S6719[44], 6}, {NULL}};
const Segment SGI110480[] = {{&S6719[44], 18}, {NULL}};
const Segment SGI110481[] = {{&S6719[30], 6}, {NULL}};
const Segment SGI110482[] = {{&S6719[31], 3}, {NULL}};
const Segment SGI110483[] = {{&S6719[76], 6}, {NULL}};
const Segment SGI110484[] = {{&S6720[44], 7}, {NULL}};
const Segment SGI110485[] = {{&S6720[44], 19}, {NULL}};
const Segment SGI110486[] = {{&S6720[30], 6}, {NULL}};
const Segment SGI110487[] = {{&S6720[31], 3}, {NULL}};
const Segment SGI110488[] = {{&S6720[77], 6}, {NULL}};
const Segment SGI110489[] = {{&S6722[44], 19}, {NULL}};
const Segment SGI110490[] = {{&S6722[44], 35}, {NULL}};
const Segment SGI110491[] = {{&S6722[30], 6}, {NULL}};
const Segment SGI110492[] = {{&S6722[31], 3}, {NULL}};
const Segment SGI110493[] = {{&S6722[93], 6}, {NULL}};
const Segment SGI110494[] = {{&S6723[44], 8}, {NULL}};
const Segment SGI110495[] = {{&S6723[44], 20}, {NULL}};
const Segment SGI110496[] = {{&S6723[30], 6}, {NULL}};
const Segment SGI110497[] = {{&S6723[31], 3}, {NULL}};
const Segment SGI110498[] = {{&S6723[78], 6}, {NULL}};
const Segment SGI110499[] = {{&S6724[44], 8}, {NULL}};
const Segment SGI110500[] = {{&S6724[44], 20}, {NULL}};
const Segment SGI110501[] = {{&S6724[30], 6}, {NULL}};
const Segment SGI110502[] = {{&S6724[31], 3}, {NULL}};
const Segment SGI110503[] = {{&S6724[78], 6}, {NULL}};
const Segment SGI110504[] = {{&S6725[44], 15}, {NULL}};
const Segment SGI110505[] = {{&S6725[44], 27}, {NULL}};
const Segment SGI110506[] = {{&S6725[30], 6}, {NULL}};
const Segment SGI110507[] = {{&S6725[31], 3}, {NULL}};
const Segment SGI110508[] = {{&S6725[85], 6}, {NULL}};
const Segment SGI110509[] = {{&S6727[44], 9}, {NULL}};
const Segment SGI110510[] = {{&S6727[44], 21}, {NULL}};
const Segment SGI110511[] = {{&S6727[30], 6}, {NULL}};
const Segment SGI110512[] = {{&S6727[31], 3}, {NULL}};
const Segment SGI110513[] = {{&S6727[79], 6}, {NULL}};
const Segment SGI110514[] = {{&S6729[44], 34}, {NULL}};
const Segment SGI110515[] = {{&S6729[44], 46}, {NULL}};
const Segment SGI110516[] = {{&S6729[30], 6}, {NULL}};
const Segment SGI110517[] = {{&S6729[31], 3}, {NULL}};
const Segment SGI110518[] = {{&S6729[104], 6}, {NULL}};
const Segment SGI110519[] = {{&S6730[44], 10}, {NULL}};
const Segment SGI110520[] = {{&S6730[44], 22}, {NULL}};
const Segment SGI110521[] = {{&S6730[30], 6}, {NULL}};
const Segment SGI110522[] = {{&S6730[31], 3}, {NULL}};
const Segment SGI110523[] = {{&S6730[80], 6}, {NULL}};
const Segment SGI110524[] = {{&S6731[44], 15}, {NULL}};
const Segment SGI110525[] = {{&S6731[44], 27}, {NULL}};
const Segment SGI110526[] = {{&S6731[30], 6}, {NULL}};
const Segment SGI110527[] = {{&S6731[31], 3}, {NULL}};
const Segment SGI110528[] = {{&S6731[85], 6}, {NULL}};
const Segment SGI110529[] = {{&S6734[44], 7}, {NULL}};
const Segment SGI110530[] = {{&S6734[44], 19}, {NULL}};
const Segment SGI110531[] = {{&S6734[30], 6}, {NULL}};
const Segment SGI110532[] = {{&S6734[31], 3}, {NULL}};
const Segment SGI110533[] = {{&S6734[77], 6}, {NULL}};
const Segment SGI110534[] = {{&S6735[44], 10}, {NULL}};
const Segment SGI110535[] = {{&S6735[44], 22}, {NULL}};
const Segment SGI110536[] = {{&S6735[30], 6}, {NULL}};
const Segment SGI110537[] = {{&S6735[31], 3}, {NULL}};
const Segment SGI110538[] = {{&S6735[80], 6}, {NULL}};
const Segment SGI110539[] = {{&S6736[44], 9}, {NULL}};
const Segment SGI110540[] = {{&S6736[44], 18}, {NULL}};
const Segment SGI110541[] = {{&S6736[30], 6}, {NULL}};
const Segment SGI110542[] = {{&S6736[31], 3}, {NULL}};
const Segment SGI110543[] = {{&S6736[76], 6}, {NULL}};
const Segment SGI110544[] = {{&S6737[44], 8}, {NULL}};
const Segment SGI110545[] = {{&S6737[44], 25}, {NULL}};
const Segment SGI110546[] = {{&S6737[30], 6}, {NULL}};
const Segment SGI110547[] = {{&S6737[31], 3}, {NULL}};
const Segment SGI110548[] = {{&S6737[83], 6}, {NULL}};
const Segment SGI110549[] = {{&S6738[44], 6}, {NULL}};
const Segment SGI110550[] = {{&S6738[44], 18}, {NULL}};
const Segment SGI110551[] = {{&S6738[30], 6}, {NULL}};
const Segment SGI110552[] = {{&S6738[31], 3}, {NULL}};
const Segment SGI110553[] = {{&S6738[76], 6}, {NULL}};
const Segment SGI110554[] = {{&S6741[44], 9}, {NULL}};
const Segment SGI110555[] = {{&S6741[44], 21}, {NULL}};
const Segment SGI110556[] = {{&S6741[30], 6}, {NULL}};
const Segment SGI110557[] = {{&S6741[31], 3}, {NULL}};
const Segment SGI110558[] = {{&S6741[79], 6}, {NULL}};
const Segment SGI110559[] = {{&S6742[44], 13}, {NULL}};
const Segment SGI110560[] = {{&S6742[44], 36}, {NULL}};
const Segment SGI110561[] = {{&S6742[30], 6}, {NULL}};
const Segment SGI110562[] = {{&S6742[31], 3}, {NULL}};
const Segment SGI110563[] = {{&S6742[94], 6}, {NULL}};
const Segment SGI110564[] = {{&S6744[44], 5}, {NULL}};
const Segment SGI110565[] = {{&S6744[44], 17}, {NULL}};
const Segment SGI110566[] = {{&S6744[30], 6}, {NULL}};
const Segment SGI110567[] = {{&S6744[31], 3}, {NULL}};
const Segment SGI110568[] = {{&S6744[75], 6}, {NULL}};
const Segment SGI110569[] = {{&S6745[44], 10}, {NULL}};
const Segment SGI110570[] = {{&S6745[44], 22}, {NULL}};
const Segment SGI110571[] = {{&S6745[30], 6}, {NULL}};
const Segment SGI110572[] = {{&S6745[31], 3}, {NULL}};
const Segment SGI110573[] = {{&S6745[80], 6}, {NULL}};
const Segment SGI110574[] = {{&S6746[44], 7}, {NULL}};
const Segment SGI110575[] = {{&S6746[44], 29}, {NULL}};
const Segment SGI110576[] = {{&S6746[30], 6}, {NULL}};
const Segment SGI110577[] = {{&S6746[31], 3}, {NULL}};
const Segment SGI110578[] = {{&S6746[87], 6}, {NULL}};
const Segment SGI110579[] = {{&S6747[44], 12}, {NULL}};
const Segment SGI110580[] = {{&S6747[44], 24}, {NULL}};
const Segment SGI110581[] = {{&S6747[30], 6}, {NULL}};
const Segment SGI110582[] = {{&S6747[31], 3}, {NULL}};
const Segment SGI110583[] = {{&S6747[82], 6}, {NULL}};
const Segment SGI110584[] = {{&S6751[44], 11}, {NULL}};
const Segment SGI110585[] = {{&S6751[44], 23}, {NULL}};
const Segment SGI110586[] = {{&S6751[30], 6}, {NULL}};
const Segment SGI110587[] = {{&S6751[31], 3}, {NULL}};
const Segment SGI110588[] = {{&S6751[81], 6}, {NULL}};
const Segment SGI110589[] = {{&S6756[44], 8}, {NULL}};
const Segment SGI110590[] = {{&S6756[44], 20}, {NULL}};
const Segment SGI110591[] = {{&S6756[30], 6}, {NULL}};
const Segment SGI110592[] = {{&S6756[31], 3}, {NULL}};
const Segment SGI110593[] = {{&S6756[78], 6}, {NULL}};
const Segment SGI110594[] = {{&S6761[44], 17}, {NULL}};
const Segment SGI110595[] = {{&S6761[44], 29}, {NULL}};
const Segment SGI110596[] = {{&S6761[30], 6}, {NULL}};
const Segment SGI110597[] = {{&S6761[31], 3}, {NULL}};
const Segment SGI110598[] = {{&S6761[87], 6}, {NULL}};
const Segment SGI110599[] = {{&S6762[44], 8}, {NULL}};
const Segment SGI110600[] = {{&S6762[44], 20}, {NULL}};
const Segment SGI110601[] = {{&S6762[30], 6}, {NULL}};
const Segment SGI110602[] = {{&S6762[31], 3}, {NULL}};
const Segment SGI110603[] = {{&S6762[78], 6}, {NULL}};
const Segment SGI110604[] = {{&S6768[44], 7}, {NULL}};
const Segment SGI110605[] = {{&S6768[44], 19}, {NULL}};
const Segment SGI110606[] = {{&S6768[30], 6}, {NULL}};
const Segment SGI110607[] = {{&S6768[31], 3}, {NULL}};
const Segment SGI110608[] = {{&S6768[77], 6}, {NULL}};
const Segment SGI110609[] = {{&S104320[44], 6}, {NULL}};
const Segment SGI110610[] = {{&S104320[44], 17}, {NULL}};
const Segment SGI110611[] = {{&S104320[30], 6}, {NULL}};
const Segment SGI110612[] = {{&S104320[110], 3}, {NULL}};
const Segment SGI110613[] = {{&S104320[75], 6}, {NULL}};
const Segment SGI110614[] = {{&S6770[44], 14}, {NULL}};
const Segment SGI110615[] = {{&S6770[44], 26}, {NULL}};
const Segment SGI110616[] = {{&S6770[30], 6}, {NULL}};
const Segment SGI110617[] = {{&S6770[31], 3}, {NULL}};
const Segment SGI110618[] = {{&S6770[84], 6}, {NULL}};
const Segment SGI110619[] = {{&S6771[44], 14}, {NULL}};
const Segment SGI110620[] = {{&S6771[44], 26}, {NULL}};
const Segment SGI110621[] = {{&S6771[30], 6}, {NULL}};
const Segment SGI110622[] = {{&S6771[31], 3}, {NULL}};
const Segment SGI110623[] = {{&S6771[84], 6}, {NULL}};
const Segment SGI110624[] = {{&S6772[44], 13}, {NULL}};
const Segment SGI110625[] = {{&S6772[44], 25}, {NULL}};
const Segment SGI110626[] = {{&S6772[30], 6}, {NULL}};
const Segment SGI110627[] = {{&S6772[31], 3}, {NULL}};
const Segment SGI110628[] = {{&S6772[83], 6}, {NULL}};
const Segment SGI110629[] = {{&S6775[44], 10}, {NULL}};
const Segment SGI110630[] = {{&S6775[44], 22}, {NULL}};
const Segment SGI110631[] = {{&S6775[30], 6}, {NULL}};
const Segment SGI110632[] = {{&S6775[31], 3}, {NULL}};
const Segment SGI110633[] = {{&S6775[80], 6}, {NULL}};
const Segment SGI110634[] = {{&S6777[44], 11}, {NULL}};
const Segment SGI110635[] = {{&S6777[44], 30}, {NULL}};
const Segment SGI110636[] = {{&S6777[30], 6}, {NULL}};
const Segment SGI110637[] = {{&S6777[31], 3}, {NULL}};
const Segment SGI110638[] = {{&S6777[88], 6}, {NULL}};
const Segment SGI110639[] = {{&S6778[44], 5}, {NULL}};
const Segment SGI110640[] = {{&S6778[44], 17}, {NULL}};
const Segment SGI110641[] = {{&S6778[30], 6}, {NULL}};
const Segment SGI110642[] = {{&S6778[31], 3}, {NULL}};
const Segment SGI110643[] = {{&S6778[75], 6}, {NULL}};
const Segment SGI110644[] = {{&S6779[44], 14}, {NULL}};
const Segment SGI110645[] = {{&S6779[44], 26}, {NULL}};
const Segment SGI110646[] = {{&S6779[30], 6}, {NULL}};
const Segment SGI110647[] = {{&S6779[31], 3}, {NULL}};
const Segment SGI110648[] = {{&S6779[84], 6}, {NULL}};
const Segment SGI110649[] = {{&S6780[44], 7}, {NULL}};
const Segment SGI110650[] = {{&S6780[44], 19}, {NULL}};
const Segment SGI110651[] = {{&S6780[30], 6}, {NULL}};
const Segment SGI110652[] = {{&S6780[31], 3}, {NULL}};
const Segment SGI110653[] = {{&S6780[77], 6}, {NULL}};
const Segment SGI110654[] = {{&S6782[44], 21}, {NULL}};
const Segment SGI110655[] = {{&S6782[44], 33}, {NULL}};
const Segment SGI110656[] = {{&S6782[30], 6}, {NULL}};
const Segment SGI110657[] = {{&S6782[31], 3}, {NULL}};
const Segment SGI110658[] = {{&S6782[91], 6}, {NULL}};
const Segment SGI110659[] = {{&S6783[44], 5}, {NULL}};
const Segment SGI110660[] = {{&S6783[44], 17}, {NULL}};
const Segment SGI110661[] = {{&S6783[30], 6}, {NULL}};
const Segment SGI110662[] = {{&S6783[31], 3}, {NULL}};
const Segment SGI110663[] = {{&S6783[75], 6}, {NULL}};
const Segment SGI110664[] = {{&S6784[44], 7}, {NULL}};
const Segment SGI110665[] = {{&S6784[44], 19}, {NULL}};
const Segment SGI110666[] = {{&S6784[30], 6}, {NULL}};
const Segment SGI110667[] = {{&S6784[31], 3}, {NULL}};
const Segment SGI110668[] = {{&S6784[77], 6}, {NULL}};
const Segment SGI110669[] = {{&S6785[44], 11}, {NULL}};
const Segment SGI110670[] = {{&S6785[44], 23}, {NULL}};
const Segment SGI110671[] = {{&S6785[30], 6}, {NULL}};
const Segment SGI110672[] = {{&S6785[31], 3}, {NULL}};
const Segment SGI110673[] = {{&S6785[81], 6}, {NULL}};
const Segment SGI110674[] = {{&S6787[44], 4}, {NULL}};
const Segment SGI110675[] = {{&S6787[44], 16}, {NULL}};
const Segment SGI110676[] = {{&S6787[30], 6}, {NULL}};
const Segment SGI110677[] = {{&S6787[31], 3}, {NULL}};
const Segment SGI110678[] = {{&S6787[74], 6}, {NULL}};
const Segment SGI110679[] = {{&S6788[44], 7}, {NULL}};
const Segment SGI110680[] = {{&S6788[44], 19}, {NULL}};
const Segment SGI110681[] = {{&S6788[30], 6}, {NULL}};
const Segment SGI110682[] = {{&S6788[31], 3}, {NULL}};
const Segment SGI110683[] = {{&S6788[77], 6}, {NULL}};
const Segment SGI110684[] = {{&S6792[44], 11}, {NULL}};
const Segment SGI110685[] = {{&S6792[44], 23}, {NULL}};
const Segment SGI110686[] = {{&S6792[30], 6}, {NULL}};
const Segment SGI110687[] = {{&S6792[31], 3}, {NULL}};
const Segment SGI110688[] = {{&S6792[81], 6}, {NULL}};
const Segment SGI110689[] = {{&S104365[44], 7}, {NULL}};
const Segment SGI110690[] = {{&S104365[44], 19}, {NULL}};
const Segment SGI110691[] = {{&S104365[30], 6}, {NULL}};
const Segment SGI110692[] = {{&S104365[119], 3}, {NULL}};
const Segment SGI110693[] = {{&S104365[84], 6}, {NULL}};
const Segment SGI110694[] = {{&S104365[63], 8}, {NULL}};
const Segment SGI110695[] = {{&S6793[44], 9}, {NULL}};
const Segment SGI110696[] = {{&S6793[44], 21}, {NULL}};
const Segment SGI110697[] = {{&S6793[30], 6}, {NULL}};
const Segment SGI110698[] = {{&S6793[31], 3}, {NULL}};
const Segment SGI110699[] = {{&S6793[79], 6}, {NULL}};
const Segment SGI110700[] = {{&S6794[44], 6}, {NULL}};
const Segment SGI110701[] = {{&S6794[44], 18}, {NULL}};
const Segment SGI110702[] = {{&S6794[30], 6}, {NULL}};
const Segment SGI110703[] = {{&S6794[31], 3}, {NULL}};
const Segment SGI110704[] = {{&S6794[76], 6}, {NULL}};
const Segment SGI110705[] = {{&S6795[44], 8}, {NULL}};
const Segment SGI110706[] = {{&S6795[44], 20}, {NULL}};
const Segment SGI110707[] = {{&S6795[30], 6}, {NULL}};
const Segment SGI110708[] = {{&S6795[31], 3}, {NULL}};
const Segment SGI110709[] = {{&S6795[78], 6}, {NULL}};
const Segment SGI110710[] = {{&S6798[44], 11}, {NULL}};
const Segment SGI110711[] = {{&S6798[44], 23}, {NULL}};
const Segment SGI110712[] = {{&S6798[30], 6}, {NULL}};
const Segment SGI110713[] = {{&S6798[31], 3}, {NULL}};
const Segment SGI110714[] = {{&S6798[81], 6}, {NULL}};
const Segment SGI110715[] = {{&S6800[44], 24}, {NULL}};
const Segment SGI110716[] = {{&S6800[44], 36}, {NULL}};
const Segment SGI110717[] = {{&S6800[30], 6}, {NULL}};
const Segment SGI110718[] = {{&S6800[31], 3}, {NULL}};
const Segment SGI110719[] = {{&S6800[94], 6}, {NULL}};
const Segment SGI110720[] = {{&S6802[44], 7}, {NULL}};
const Segment SGI110721[] = {{&S6802[44], 19}, {NULL}};
const Segment SGI110722[] = {{&S6802[30], 6}, {NULL}};
const Segment SGI110723[] = {{&S6802[31], 3}, {NULL}};
const Segment SGI110724[] = {{&S6802[77], 6}, {NULL}};
const Segment SGI110725[] = {{&S6804[44], 34}, {NULL}};
const Segment SGI110726[] = {{&S6804[30], 6}, {NULL}};
const Segment SGI110727[] = {{&S6805[44], 9}, {NULL}};
const Segment SGI110728[] = {{&S6805[44], 32}, {NULL}};
const Segment SGI110729[] = {{&S6805[30], 6}, {NULL}};
const Segment SGI110730[] = {{&S6805[31], 3}, {NULL}};
const Segment SGI110731[] = {{&S6805[90], 6}, {NULL}};
const Segment SGI110732[] = {{&S6806[44], 6}, {NULL}};
const Segment SGI110733[] = {{&S6806[44], 18}, {NULL}};
const Segment SGI110734[] = {{&S6806[30], 6}, {NULL}};
const Segment SGI110735[] = {{&S6806[31], 3}, {NULL}};
const Segment SGI110736[] = {{&S6806[76], 6}, {NULL}};
const Segment SGI110737[] = {{&S6809[44], 9}, {NULL}};
const Segment SGI110738[] = {{&S6809[44], 21}, {NULL}};
const Segment SGI110739[] = {{&S6809[30], 6}, {NULL}};
const Segment SGI110740[] = {{&S6809[31], 3}, {NULL}};
const Segment SGI110741[] = {{&S6809[79], 6}, {NULL}};
const Segment SGI110742[] = {{&S6810[44], 11}, {NULL}};
const Segment SGI110743[] = {{&S6810[44], 20}, {NULL}};
const Segment SGI110744[] = {{&S6810[30], 6}, {NULL}};
const Segment SGI110745[] = {{&S6810[31], 3}, {NULL}};
const Segment SGI110746[] = {{&S6810[78], 6}, {NULL}};
const Segment SGI110747[] = {{&S6811[47], 6}, {NULL}};
const Segment SGI110748[] = {{&S6811[25], 3}, {NULL}};
const Segment SGI110749[] = {{&S6811[121], 6}, {NULL}};
const Segment SGI110750[] = {{&S6812[44], 13}, {NULL}};
const Segment SGI110751[] = {{&S6812[44], 25}, {NULL}};
const Segment SGI110752[] = {{&S6812[30], 6}, {NULL}};
const Segment SGI110753[] = {{&S6812[31], 3}, {NULL}};
const Segment SGI110754[] = {{&S6812[83], 6}, {NULL}};
const Segment SGI110755[] = {{&S6813[44], 13}, {NULL}};
const Segment SGI110756[] = {{&S6813[44], 25}, {NULL}};
const Segment SGI110757[] = {{&S6813[30], 6}, {&S6813[52], 17}, {NULL}};
const Segment SGI110758[] = {{&S6813[31], 3}, {NULL}};
const Segment SGI110759[] = {{&S6813[83], 6}, {NULL}};
const Segment SGI110760[] = {{&S6814[44], 10}, {NULL}};
const Segment SGI110761[] = {{&S6814[44], 22}, {NULL}};
const Segment SGI110762[] = {{&S6814[30], 6}, {NULL}};
const Segment SGI110763[] = {{&S6814[31], 3}, {NULL}};
const Segment SGI110764[] = {{&S6814[80], 6}, {NULL}};
const Segment SGI110765[] = {{&S6815[44], 14}, {NULL}};
const Segment SGI110766[] = {{&S6815[44], 26}, {NULL}};
const Segment SGI110767[] = {{&S6815[30], 6}, {NULL}};
const Segment SGI110768[] = {{&S6815[31], 3}, {NULL}};
const Segment SGI110769[] = {{&S6815[84], 6}, {NULL}};
const Segment SGI110770[] = {{&S6816[44], 6}, {NULL}};
const Segment SGI110771[] = {{&S6816[44], 18}, {NULL}};
const Segment SGI110772[] = {{&S6816[30], 6}, {NULL}};
const Segment SGI110773[] = {{&S6816[31], 3}, {NULL}};
const Segment SGI110774[] = {{&S6816[76], 6}, {NULL}};
const Segment SGI110775[] = {{&S6817[44], 9}, {NULL}};
const Segment SGI110776[] = {{&S6817[44], 27}, {NULL}};
const Segment SGI110777[] = {{&S6817[30], 6}, {NULL}};
const Segment SGI110778[] = {{&S6817[31], 3}, {NULL}};
const Segment SGI110779[] = {{&S6817[85], 6}, {NULL}};
const Segment SGI110780[] = {{&S104412[42], 10}, {NULL}};
const Segment SGI110781[] = {{&S104412[42], 21}, {NULL}};
const Segment SGI110782[] = {{&S104412[30], 4}, {NULL}};
const Segment SGI110783[] = {{&S104412[112], 3}, {NULL}};
const Segment SGI110784[] = {{&S104412[77], 6}, {NULL}};
const Segment SGI110785[] = {{&S6819[44], 5}, {NULL}};
const Segment SGI110786[] = {{&S6819[44], 17}, {NULL}};
const Segment SGI110787[] = {{&S6819[30], 6}, {NULL}};
const Segment SGI110788[] = {{&S6819[31], 3}, {NULL}};
const Segment SGI110789[] = {{&S6819[75], 6}, {NULL}};
const Segment SGI110790[] = {{&S6820[44], 6}, {NULL}};
const Segment SGI110791[] = {{&S6820[44], 18}, {NULL}};
const Segment SGI110792[] = {{&S6820[30], 6}, {NULL}};
const Segment SGI110793[] = {{&S6820[31], 3}, {NULL}};
const Segment SGI110794[] = {{&S6820[76], 6}, {NULL}};
const Segment SGI110795[] = {{&S104416[54], 10}, {NULL}};
const Segment SGI110796[] = {{&S6821[44], 9}, {NULL}};
const Segment SGI110797[] = {{&S104416[54], 22}, {NULL}};
const Segment SGI110798[] = {{&S6821[44], 21}, {NULL}};
const Segment SGI110799[] = {{&S104416[42], 4}, {NULL}};
const Segment SGI110800[] = {{&S6821[30], 6}, {NULL}};
const Segment SGI110801[] = {{&S6821[31], 3}, {NULL}};
const Segment SGI110802[] = {{&S104416[125], 3}, {NULL}};
const Segment SGI110803[] = {{&S6821[79], 6}, {NULL}};
const Segment SGI110804[] = {{&S104416[90], 6}, {NULL}};
const Segment SGI110805[] = {{&S104416[0], 12}, {NULL}};
const Segment SGI110806[] = {{&S6822[44], 8}, {NULL}};
const Segment SGI110807[] = {{&S6822[44], 20}, {NULL}};
const Segment SGI110808[] = {{&S6822[30], 6}, {NULL}};
const Segment SGI110809[] = {{&S6822[31], 3}, {NULL}};
const Segment SGI110810[] = {{&S6822[78], 6}, {NULL}};
const Segment SGI110811[] = {{&S6823[44], 8}, {NULL}};
const Segment SGI110812[] = {{&S6823[44], 20}, {NULL}};
const Segment SGI110813[] = {{&S6823[30], 6}, {NULL}};
const Segment SGI110814[] = {{&S6823[31], 3}, {NULL}};
const Segment SGI110815[] = {{&S6823[78], 6}, {NULL}};
const Segment SGI110816[] = {{&S6824[44], 16}, {NULL}};
const Segment SGI110817[] = {{&S6824[44], 28}, {NULL}};
const Segment SGI110818[] = {{&S6824[30], 6}, {NULL}};
const Segment SGI110819[] = {{&S6824[31], 3}, {NULL}};
const Segment SGI110820[] = {{&S6824[86], 6}, {NULL}};
const Segment SGI110821[] = {{&S104422[42], 10}, {NULL}};
const Segment SGI110822[] = {{&S104422[42], 22}, {NULL}};
const Segment SGI110823[] = {{&S104422[30], 4}, {NULL}};
const Segment SGI110824[] = {{&S104422[113], 3}, {NULL}};
const Segment SGI110825[] = {{&S104422[78], 6}, {NULL}};
const Segment SGI110826[] = {{&S104422[131], 11}, {NULL}};
const Segment SGI110827[] = {{&S6826[44], 11}, {NULL}};
const Segment SGI110828[] = {{&S6826[44], 23}, {NULL}};
const Segment SGI110829[] = {{&S6826[30], 6}, {NULL}};
const Segment SGI110830[] = {{&S104423[42], 10}, {NULL}};
const Segment SGI110831[] = {{&S6826[31], 3}, {NULL}};
const Segment SGI110832[] = {{&S104423[42], 21}, {NULL}};
const Segment SGI110833[] = {{&S104423[30], 4}, {NULL}};
const Segment SGI110834[] = {{&S6826[81], 6}, {NULL}};
const Segment SGI110835[] = {{&S104423[156], 3}, {NULL}};
const Segment SGI110836[] = {{&S104423[31], 3}, {NULL}};
const Segment SGI110837[] = {{&S104423[87], 6}, {NULL}};
const Segment SGI110838[] = {{&S104423[142], 8}, {NULL}};
const Segment SGI110839[] = {{&S6827[44], 4}, {NULL}};
const Segment SGI110840[] = {{&S6827[44], 16}, {NULL}};
const Segment SGI110841[] = {{&S6827[30], 6}, {NULL}};
const Segment SGI110842[] = {{&S6827[31], 3}, {NULL}};
const Segment SGI110843[] = {{&S6827[74], 6}, {NULL}};
const Segment SGI110844[] = {{&S6829[44], 18}, {NULL}};
const Segment SGI110845[] = {{&S6829[44], 34}, {NULL}};
const Segment SGI110846[] = {{&S6829[30], 6}, {NULL}};
const Segment SGI110847[] = {{&S6829[31], 3}, {NULL}};
const Segment SGI110848[] = {{&S6829[92], 6}, {NULL}};
const Segment SGI110849[] = {{&S6831[44], 10}, {NULL}};
const Segment SGI110850[] = {{&S6831[44], 22}, {NULL}};
const Segment SGI110851[] = {{&S6831[30], 6}, {NULL}};
const Segment SGI110852[] = {{&S6831[31], 3}, {NULL}};
const Segment SGI110853[] = {{&S6831[80], 6}, {NULL}};
const Segment SGI110854[] = {{&S6832[44], 7}, {NULL}};
const Segment SGI110855[] = {{&S6832[44], 19}, {NULL}};
const Segment SGI110856[] = {{&S6832[30], 6}, {NULL}};
const Segment SGI110857[] = {{&S6832[31], 3}, {NULL}};
const Segment SGI110858[] = {{&S6832[77], 6}, {NULL}};
const Segment SGI110859[] = {{&S104433[42], 10}, {NULL}};
const Segment SGI110860[] = {{&S104433[42], 27}, {NULL}};
const Segment SGI110861[] = {{&S104433[30], 4}, {NULL}};
const Segment SGI110862[] = {{&S104433[117], 3}, {NULL}};
const Segment SGI110863[] = {{&S6834[44], 9}, {NULL}};
const Segment SGI110864[] = {{&S104433[83], 5}, {NULL}};
const Segment SGI110865[] = {{&S6834[44], 21}, {NULL}};
const Segment SGI110866[] = {{&S6834[30], 6}, {NULL}};
const Segment SGI110867[] = {{&S6834[31], 3}, {NULL}};
const Segment SGI110868[] = {{&S6834[79], 6}, {NULL}};
const Segment SGI110869[] = {{&S6835[44], 8}, {NULL}};
const Segment SGI110870[] = {{&S6835[44], 20}, {NULL}};
const Segment SGI110871[] = {{&S6835[30], 6}, {NULL}};
const Segment SGI110872[] = {{&S6835[31], 3}, {NULL}};
const Segment SGI110873[] = {{&S6835[78], 6}, {NULL}};
const Segment SGI110874[] = {{&S6836[44], 8}, {NULL}};
const Segment SGI110875[] = {{&S6836[44], 20}, {NULL}};
const Segment SGI110876[] = {{&S6836[30], 6}, {NULL}};
const Segment SGI110877[] = {{&S6836[31], 3}, {NULL}};
const Segment SGI110878[] = {{&S6836[78], 6}, {NULL}};
const Segment SGI110879[] = {{&S6837[44], 14}, {NULL}};
const Segment SGI110880[] = {{&S6837[44], 26}, {NULL}};
const Segment SGI110881[] = {{&S6837[30], 6}, {NULL}};
const Segment SGI110882[] = {{&S6837[31], 3}, {NULL}};
const Segment SGI110883[] = {{&S6837[84], 6}, {NULL}};
const Segment SGI110884[] = {{&S6838[44], 11}, {NULL}};
const Segment SGI110885[] = {{&S6838[44], 23}, {NULL}};
const Segment SGI110886[] = {{&S6838[30], 6}, {NULL}};
const Segment SGI110887[] = {{&S6838[31], 3}, {NULL}};
const Segment SGI110888[] = {{&S6838[81], 6}, {NULL}};
const Segment SGI110889[] = {{&S6840[44], 8}, {NULL}};
const Segment SGI110890[] = {{&S6840[44], 20}, {NULL}};
const Segment SGI110891[] = {{&S6840[30], 6}, {NULL}};
const Segment SGI110892[] = {{&S6840[31], 3}, {NULL}};
const Segment SGI110893[] = {{&S6840[78], 6}, {NULL}};
const Segment SGI110894[] = {{&S6841[44], 17}, {NULL}};
const Segment SGI110895[] = {{&S6841[44], 29}, {NULL}};
const Segment SGI110896[] = {{&S6841[30], 6}, {NULL}};
const Segment SGI110897[] = {{&S6841[31], 3}, {NULL}};
const Segment SGI110898[] = {{&S6841[87], 6}, {NULL}};
const Segment SGI110899[] = {{&S6842[44], 6}, {NULL}};
const Segment SGI110900[] = {{&S6842[44], 18}, {NULL}};
const Segment SGI110901[] = {{&S6842[30], 6}, {NULL}};
const Segment SGI110902[] = {{&S6842[31], 3}, {NULL}};
const Segment SGI110903[] = {{&S6842[76], 6}, {NULL}};
const Segment SGI110904[] = {{&S6843[44], 9}, {NULL}};
const Segment SGI110905[] = {{&S6843[44], 21}, {NULL}};
const Segment SGI110906[] = {{&S6843[30], 6}, {NULL}};
const Segment SGI110907[] = {{&S6843[31], 3}, {NULL}};
const Segment SGI110908[] = {{&S6843[79], 6}, {NULL}};
const Segment SGI110909[] = {{&S6844[44], 7}, {NULL}};
const Segment SGI110910[] = {{&S6844[44], 19}, {NULL}};
const Segment SGI110911[] = {{&S6844[30], 6}, {NULL}};
const Segment SGI110912[] = {{&S6844[31], 3}, {NULL}};
const Segment SGI110913[] = {{&S6844[77], 6}, {NULL}};
const Segment SGI110914[] = {{&S6845[44], 8}, {NULL}};
const Segment SGI110915[] = {{&S6845[44], 20}, {NULL}};
const Segment SGI110916[] = {{&S6845[30], 6}, {NULL}};
const Segment SGI110917[] = {{&S6845[31], 3}, {NULL}};
const Segment SGI110918[] = {{&S6845[78], 6}, {NULL}};
const Segment SGI110919[] = {{&S6846[44], 4}, {NULL}};
const Segment SGI110920[] = {{&S6846[44], 16}, {NULL}};
const Segment SGI110921[] = {{&S6846[30], 6}, {NULL}};
const Segment SGI110922[] = {{&S104454[42], 10}, {NULL}};
const Segment SGI110923[] = {{&S6846[31], 3}, {NULL}};
const Segment SGI110924[] = {{&S104454[42], 22}, {NULL}};
const Segment SGI110925[] = {{&S6846[74], 6}, {NULL}};
const Segment SGI110926[] = {{&S104454[30], 4}, {NULL}};
const Segment SGI110927[] = {{&S104454[113], 3}, {NULL}};
const Segment SGI110928[] = {{&S104454[78], 6}, {NULL}};
const Segment SGI110929[] = {{&S104454[131], 13}, {NULL}};
const Segment SGI110930[] = {{&S6847[44], 6}, {NULL}};
const Segment SGI110931[] = {{&S6847[44], 18}, {NULL}};
const Segment SGI110932[] = {{&S6847[30], 6}, {NULL}};
const Segment SGI110933[] = {{&S6847[31], 3}, {NULL}};
const Segment SGI110934[] = {{&S6847[76], 6}, {NULL}};
const Segment SGI110935[] = {{&S104457[43], 10}, {NULL}};
const Segment SGI110936[] = {{&S104457[30], 4}, {NULL}};
const Segment SGI110937[] = {{&S104457[113], 3}, {NULL}};
const Segment SGI110938[] = {{&S104457[79], 5}, {NULL}};
const Segment SGI110939[] = {{&S104457[137], 13}, {NULL}};
const Segment SGI110940[] = {{&S6850[44], 21}, {NULL}};
const Segment SGI110941[] = {{&S6850[44], 33}, {NULL}};
const Segment SGI110942[] = {{&S6850[30], 6}, {NULL}};
const Segment SGI110943[] = {{&S6850[31], 3}, {NULL}};
const Segment SGI110944[] = {{&S6850[91], 6}, {NULL}};
const Segment SGI110945[] = {{&S6851[44], 8}, {NULL}};
const Segment SGI110946[] = {{&S6851[44], 20}, {NULL}};
const Segment SGI110947[] = {{&S6851[30], 6}, {NULL}};
const Segment SGI110948[] = {{&S6851[31], 3}, {NULL}};
const Segment SGI110949[] = {{&S6851[78], 6}, {NULL}};
const Segment SGI110950[] = {{&S104460[42], 11}, {NULL}};
const Segment SGI110951[] = {{&S104460[42], 23}, {NULL}};
const Segment SGI110952[] = {{&S104460[30], 4}, {NULL}};
const Segment SGI110953[] = {{&S104460[114], 3}, {NULL}};
const Segment SGI110954[] = {{&S104460[79], 6}, {NULL}};
const Segment SGI110955[] = {{&S104460[132], 13}, {NULL}};
const Segment SGI110956[] = {{&S104461[42], 10}, {NULL}};
const Segment SGI110957[] = {{&S104461[42], 22}, {NULL}};
const Segment SGI110958[] = {{&S104461[30], 4}, {NULL}};
const Segment SGI110959[] = {{&S104461[113], 3}, {NULL}};
const Segment SGI110960[] = {{&S6856[44], 13}, {NULL}};
const Segment SGI110961[] = {{&S6856[44], 25}, {NULL}};
const Segment SGI110962[] = {{&S104461[78], 6}, {NULL}};
const Segment SGI110963[] = {{&S6856[30], 6}, {NULL}};
const Segment SGI110964[] = {{&S104461[131], 13}, {NULL}};
const Segment SGI110965[] = {{&S6856[31], 3}, {NULL}};
const Segment SGI110966[] = {{&S6856[83], 6}, {NULL}};
const Segment SGI110967[] = {{&S6857[44], 10}, {NULL}};
const Segment SGI110968[] = {{&S6857[44], 22}, {NULL}};
const Segment SGI110969[] = {{&S6857[30], 6}, {NULL}};
const Segment SGI110970[] = {{&S6857[31], 3}, {NULL}};
const Segment SGI110971[] = {{&S6857[80], 6}, {NULL}};
const Segment SGI110972[] = {{&S104463[42], 10}, {NULL}};
const Segment SGI110973[] = {{&S104463[42], 22}, {NULL}};
const Segment SGI110974[] = {{&S104463[30], 4}, {NULL}};
const Segment SGI110975[] = {{&S104463[157], 3}, {NULL}};
const Segment SGI110976[] = {{&S104463[123], 3}, {NULL}};
const Segment SGI110977[] = {{&S104463[88], 6}, {NULL}};
const Segment SGI110978[] = {{&S104463[143], 8}, {NULL}};
const Segment SGI110979[] = {{&S6859[44], 5}, {NULL}};
const Segment SGI110980[] = {{&S6859[44], 17}, {NULL}};
const Segment SGI110981[] = {{&S6859[30], 6}, {NULL}};
const Segment SGI110982[] = {{&S6859[31], 3}, {NULL}};
const Segment SGI110983[] = {{&S6859[75], 6}, {NULL}};
const Segment SGI110984[] = {{&S6860[44], 10}, {NULL}};
const Segment SGI110985[] = {{&S6860[44], 22}, {NULL}};
const Segment SGI110986[] = {{&S6860[30], 6}, {NULL}};
const Segment SGI110987[] = {{&S6860[31], 3}, {NULL}};
const Segment SGI110988[] = {{&S6860[80], 6}, {NULL}};
const Segment SGI110989[] = {{&S6862[44], 11}, {NULL}};
const Segment SGI110990[] = {{&S6862[44], 23}, {NULL}};
const Segment SGI110991[] = {{&S6862[30], 6}, {NULL}};
const Segment SGI110992[] = {{&S6862[31], 3}, {NULL}};
const Segment SGI110993[] = {{&S6862[81], 6}, {NULL}};
const Segment SGI110994[] = {{&S104470[42], 10}, {NULL}};
const Segment SGI110995[] = {{&S104470[42], 21}, {NULL}};
const Segment SGI110996[] = {{&S104470[30], 4}, {NULL}};
const Segment SGI110997[] = {{&S6863[44], 19}, {NULL}};
const Segment SGI110998[] = {{&S6863[44], 31}, {NULL}};
const Segment SGI110999[] = {{&S6863[30], 6}, {NULL}};
const Segment SGI111000[] = {{&S104470[112], 3}, {NULL}};
const Segment SGI111001[] = {{&S6863[31], 3}, {NULL}};
const Segment SGI111002[] = {{&S104470[77], 6}, {NULL}};
const Segment SGI111003[] = {{&S6863[89], 6}, {NULL}};
const Segment SGI111004[] = {{&S104470[129], 10}, {NULL}};
const Segment SGI111005[] = {{&S6864[44], 8}, {NULL}};
const Segment SGI111006[] = {{&S6864[44], 20}, {NULL}};
const Segment SGI111007[] = {{&S6864[30], 6}, {NULL}};
const Segment SGI111008[] = {{&S6864[31], 3}, {NULL}};
const Segment SGI111009[] = {{&S6864[78], 6}, {NULL}};
const Segment SGI111010[] = {{&S6865[44], 10}, {NULL}};
const Segment SGI111011[] = {{&S6865[44], 22}, {NULL}};
const Segment SGI111012[] = {{&S6865[30], 6}, {NULL}};
const Segment SGI111013[] = {{&S6865[31], 3}, {NULL}};
const Segment SGI111014[] = {{&S6865[80], 6}, {NULL}};
const Segment SGI111015[] = {{&S6866[50], 13}, {NULL}};
const Segment SGI111016[] = {{&S6866[50], 25}, {NULL}};
const Segment SGI111017[] = {{&S6866[30], 11}, {NULL}};
const Segment SGI111018[] = {{&S6866[31], 3}, {NULL}};
const Segment SGI111019[] = {{&S6866[89], 6}, {NULL}};
const Segment SGI111020[] = {{&S104476[42], 10}, {NULL}};
const Segment SGI111021[] = {{&S104476[42], 22}, {NULL}};
const Segment SGI111022[] = {{&S104476[30], 4}, {NULL}};
const Segment SGI111023[] = {{&S104476[113], 3}, {NULL}};
const Segment SGI111024[] = {{&S104476[78], 6}, {NULL}};
const Segment SGI111025[] = {{&S104476[130], 10}, {NULL}};
const Segment SGI111026[] = {{&S6867[44], 19}, {NULL}};
const Segment SGI111027[] = {{&S6867[44], 31}, {NULL}};
const Segment SGI111028[] = {{&S6867[30], 6}, {NULL}};
const Segment SGI111029[] = {{&S6867[31], 3}, {NULL}};
const Segment SGI111030[] = {{&S6867[89], 6}, {NULL}};
const Segment SGI111031[] = {{&S104477[41], 10}, {NULL}};
const Segment SGI111032[] = {{&S104477[41], 22}, {NULL}};
const Segment SGI111033[] = {{&S104477[29], 4}, {NULL}};
const Segment SGI111034[] = {{&S104477[77], 6}, {NULL}};
const Segment SGI111035[] = {{&S104477[104], 13}, {NULL}};
const Segment SGI111036[] = {{&S6868[44], 13}, {NULL}};
const Segment SGI111037[] = {{&S6868[44], 25}, {NULL}};
const Segment SGI111038[] = {{&S6868[30], 6}, {NULL}};
const Segment SGI111039[] = {{&S6868[31], 3}, {NULL}};
const Segment SGI111040[] = {{&S6868[83], 6}, {NULL}};
const Segment SGI111041[] = {{&S6869[44], 15}, {NULL}};
const Segment SGI111042[] = {{&S6869[44], 27}, {NULL}};
const Segment SGI111043[] = {{&S6869[30], 6}, {NULL}};
const Segment SGI111044[] = {{&S6869[31], 3}, {NULL}};
const Segment SGI111045[] = {{&S6869[85], 6}, {NULL}};
const Segment SGI111046[] = {{&S6870[44], 6}, {NULL}};
const Segment SGI111047[] = {{&S6870[44], 49}, {NULL}};
const Segment SGI111048[] = {{&S6870[30], 6}, {NULL}};
const Segment SGI111049[] = {{&S6870[31], 3}, {NULL}};
const Segment SGI111050[] = {{&S6870[107], 6}, {NULL}};
const Segment SGI111051[] = {{&S6871[44], 10}, {NULL}};
const Segment SGI111052[] = {{&S6871[44], 22}, {NULL}};
const Segment SGI111053[] = {{&S6871[30], 6}, {NULL}};
const Segment SGI111054[] = {{&S6871[31], 3}, {NULL}};
const Segment SGI111055[] = {{&S6871[80], 6}, {NULL}};
const Segment SGI111056[] = {{&S104484[42], 10}, {NULL}};
const Segment SGI111057[] = {{&S104484[42], 22}, {NULL}};
const Segment SGI111058[] = {{&S104484[30], 4}, {NULL}};
const Segment SGI111059[] = {{&S104484[113], 3}, {NULL}};
const Segment SGI111060[] = {{&S104484[78], 6}, {NULL}};
const Segment SGI111061[] = {{&S104484[130], 10}, {NULL}};
const Segment SGI111062[] = {{&S6872[44], 22}, {NULL}};
const Segment SGI111063[] = {{&S6872[44], 34}, {NULL}};
const Segment SGI111064[] = {{&S6872[30], 6}, {NULL}};
const Segment SGI111065[] = {{&S6872[31], 3}, {NULL}};
const Segment SGI111066[] = {{&S6872[92], 5}, {NULL}};
const Segment SGI111067[] = {{&S104486[42], 10}, {NULL}};
const Segment SGI111068[] = {{&S6873[44], 4}, {NULL}};
const Segment SGI111069[] = {{&S104486[42], 22}, {NULL}};
const Segment SGI111070[] = {{&S104486[30], 4}, {NULL}};
const Segment SGI111071[] = {{&S6873[44], 16}, {NULL}};
const Segment SGI111072[] = {{&S6873[30], 6}, {NULL}};
const Segment SGI111073[] = {{&S104486[111], 3}, {NULL}};
const Segment SGI111074[] = {{&S6873[31], 3}, {NULL}};
const Segment SGI111075[] = {{&S6873[74], 6}, {NULL}};
const Segment SGI111076[] = {{&S104486[78], 5}, {NULL}};
const Segment SGI111077[] = {{&S104486[115], 14}, {NULL}};
const Segment SGI111078[] = {{&S6877[44], 4}, {NULL}};
const Segment SGI111079[] = {{&S6877[44], 16}, {NULL}};
const Segment SGI111080[] = {{&S6877[30], 6}, {NULL}};
const Segment SGI111081[] = {{&S6877[31], 3}, {NULL}};
const Segment SGI111082[] = {{&S6877[74], 6}, {NULL}};
const Segment SGI111083[] = {{&S6878[44], 5}, {NULL}};
const Segment SGI111084[] = {{&S6878[44], 17}, {NULL}};
const Segment SGI111085[] = {{&S6878[30], 6}, {NULL}};
const Segment SGI111086[] = {{&S6878[31], 3}, {NULL}};
const Segment SGI111087[] = {{&S6878[75], 6}, {NULL}};
const Segment SGI111088[] = {{&S104495[57], 10}, {NULL}};
const Segment SGI111089[] = {{&S104495[57], 21}, {NULL}};
const Segment SGI111090[] = {{&S6879[44], 9}, {NULL}};
const Segment SGI111091[] = {{&S104495[45], 4}, {NULL}};
const Segment SGI111092[] = {{&S6879[44], 21}, {NULL}};
const Segment SGI111093[] = {{&S6879[30], 6}, {NULL}};
const Segment SGI111094[] = {{&S104495[127], 3}, {NULL}};
const Segment SGI111095[] = {{&S6879[31], 3}, {NULL}};
const Segment SGI111096[] = {{&S104495[92], 6}, {NULL}};
const Segment SGI111097[] = {{&S6879[79], 6}, {NULL}};
const Segment SGI111098[] = {{&S104495[0], 14}, {NULL}};
const Segment SGI111099[] = {{&S104496[57], 10}, {NULL}};
const Segment SGI111100[] = {{&S6880[44], 10}, {NULL}};
const Segment SGI111101[] = {{&S104496[57], 21}, {NULL}};
const Segment SGI111102[] = {{&S6880[44], 22}, {NULL}};
const Segment SGI111103[] = {{&S104496[45], 4}, {NULL}};
const Segment SGI111104[] = {{&S6880[30], 6}, {NULL}};
const Segment SGI111105[] = {{&S104496[127], 3}, {NULL}};
const Segment SGI111106[] = {{&S6880[31], 3}, {NULL}};
const Segment SGI111107[] = {{&S6880[80], 6}, {NULL}};
const Segment SGI111108[] = {{&S104496[92], 6}, {NULL}};
const Segment SGI111109[] = {{&S104496[0], 14}, {NULL}};
const Segment SGI111110[] = {{&S6881[44], 6}, {NULL}};
const Segment SGI111111[] = {{&S6881[44], 18}, {NULL}};
const Segment SGI111112[] = {{&S6881[30], 6}, {NULL}};
const Segment SGI111113[] = {{&S104497[42], 10}, {NULL}};
const Segment SGI111114[] = {{&S104497[42], 22}, {NULL}};
const Segment SGI111115[] = {{&S104497[30], 4}, {NULL}};
const Segment SGI111116[] = {{&S6881[31], 3}, {NULL}};
const Segment SGI111117[] = {{&S6881[76], 6}, {NULL}};
const Segment SGI111118[] = {{&S104497[113], 3}, {NULL}};
const Segment SGI111119[] = {{&S104497[78], 6}, {NULL}};
const Segment SGI111120[] = {{&S104497[137], 11}, {NULL}};
const Segment SGI111121[] = {{&S6882[44], 8}, {NULL}};
const Segment SGI111122[] = {{&S6882[44], 24}, {NULL}};
const Segment SGI111123[] = {{&S6882[30], 6}, {NULL}};
const Segment SGI111124[] = {{&S6882[31], 3}, {NULL}};
const Segment SGI111125[] = {{&S6882[82], 6}, {NULL}};
const Segment SGI111126[] = {{&S104499[42], 10}, {NULL}};
const Segment SGI111127[] = {{&S104499[42], 22}, {NULL}};
const Segment SGI111128[] = {{&S104499[30], 4}, {NULL}};
const Segment SGI111129[] = {{&S104499[113], 3}, {NULL}};
const Segment SGI111130[] = {{&S104499[78], 6}, {NULL}};
const Segment SGI111131[] = {{&S6883[44], 5}, {NULL}};
const Segment SGI111132[] = {{&S104499[131], 11}, {NULL}};
const Segment SGI111133[] = {{&S6883[44], 17}, {NULL}};
const Segment SGI111134[] = {{&S6883[30], 6}, {NULL}};
const Segment SGI111135[] = {{&S6883[31], 3}, {NULL}};
const Segment SGI111136[] = {{&S6883[75], 6}, {NULL}};
const Segment SGI111137[] = {{&S6884[44], 11}, {NULL}};
const Segment SGI111138[] = {{&S6884[44], 23}, {NULL}};
const Segment SGI111139[] = {{&S6884[30], 6}, {NULL}};
const Segment SGI111140[] = {{&S6884[31], 3}, {NULL}};
const Segment SGI111141[] = {{&S6884[81], 6}, {NULL}};
const Segment SGI111142[] = {{&S6885[44], 8}, {NULL}};
const Segment SGI111143[] = {{&S6885[44], 20}, {NULL}};
const Segment SGI111144[] = {{&S6885[30], 6}, {NULL}};
const Segment SGI111145[] = {{&S6885[31], 3}, {NULL}};
const Segment SGI111146[] = {{&S6885[78], 6}, {NULL}};
const Segment SGI111147[] = {{&S104504[42], 10}, {NULL}};
const Segment SGI111148[] = {{&S104504[42], 22}, {NULL}};
const Segment SGI111149[] = {{&S104504[30], 4}, {NULL}};
const Segment SGI111150[] = {{&S104504[113], 3}, {NULL}};
const Segment SGI111151[] = {{&S104504[78], 6}, {NULL}};
const Segment SGI111152[] = {{&S104504[131], 11}, {NULL}};
const Segment SGI111153[] = {{&S6886[44], 6}, {NULL}};
const Segment SGI111154[] = {{&S6886[44], 63}, {NULL}};
const Segment SGI111155[] = {{&S6886[30], 6}, {NULL}};
const Segment SGI111156[] = {{&S6886[31], 3}, {NULL}};
const Segment SGI111157[] = {{&S6886[121], 6}, {NULL}};
const Segment SGI111158[] = {{&S6887[44], 9}, {NULL}};
const Segment SGI111159[] = {{&S6887[44], 21}, {NULL}};
const Segment SGI111160[] = {{&S6887[30], 6}, {NULL}};
const Segment SGI111161[] = {{&S6887[31], 3}, {NULL}};
const Segment SGI111162[] = {{&S6887[79], 5}, {NULL}};
const Segment SGI111163[] = {{&S104507[42], 10}, {NULL}};
const Segment SGI111164[] = {{&S104507[42], 22}, {NULL}};
const Segment SGI111165[] = {{&S104507[30], 4}, {NULL}};
const Segment SGI111166[] = {{&S104507[113], 3}, {NULL}};
const Segment SGI111167[] = {{&S104507[78], 6}, {NULL}};
const Segment SGI111168[] = {{&S104507[131], 11}, {NULL}};
const Segment SGI111169[] = {{&S6888[44], 11}, {NULL}};
const Segment SGI111170[] = {{&S6888[44], 23}, {NULL}};
const Segment SGI111171[] = {{&S6888[30], 6}, {NULL}};
const Segment SGI111172[] = {{&S6888[31], 3}, {NULL}};
const Segment SGI111173[] = {{&S6888[81], 6}, {NULL}};
const Segment SGI111174[] = {{&S6889[44], 8}, {NULL}};
const Segment SGI111175[] = {{&S6889[44], 20}, {NULL}};
const Segment SGI111176[] = {{&S6889[30], 6}, {NULL}};
const Segment SGI111177[] = {{&S6889[31], 3}, {NULL}};
const Segment SGI111178[] = {{&S6889[78], 6}, {NULL}};
const Segment SGI111179[] = {{&S6890[44], 8}, {NULL}};
const Segment SGI111180[] = {{&S6890[44], 20}, {NULL}};
const Segment SGI111181[] = {{&S6890[30], 6}, {NULL}};
const Segment SGI111182[] = {{&S6890[31], 3}, {NULL}};
const Segment SGI111183[] = {{&S6890[78], 6}, {NULL}};
const Segment SGI111184[] = {{&S104512[42], 10}, {NULL}};
const Segment SGI111185[] = {{&S104512[42], 22}, {NULL}};
const Segment SGI111186[] = {{&S104512[30], 4}, {NULL}};
const Segment SGI111187[] = {{&S104512[113], 3}, {NULL}};
const Segment SGI111188[] = {{&S104512[78], 6}, {NULL}};
const Segment SGI111189[] = {{&S104512[138], 13}, {NULL}};
const Segment SGI111190[] = {{&S6891[44], 9}, {NULL}};
const Segment SGI111191[] = {{&S6891[44], 18}, {NULL}};
const Segment SGI111192[] = {{&S6891[30], 6}, {NULL}};
const Segment SGI111193[] = {{&S6891[31], 3}, {NULL}};
const Segment SGI111194[] = {{&S6891[76], 6}, {NULL}};
const Segment SGI111195[] = {{&S6892[44], 5}, {NULL}};
const Segment SGI111196[] = {{&S104514[42], 10}, {NULL}};
const Segment SGI111197[] = {{&S6892[44], 25}, {NULL}};
const Segment SGI111198[] = {{&S104514[42], 22}, {NULL}};
const Segment SGI111199[] = {{&S6892[30], 6}, {NULL}};
const Segment SGI111200[] = {{&S104514[30], 4}, {NULL}};
const Segment SGI111201[] = {{&S6892[31], 3}, {NULL}};
const Segment SGI111202[] = {{&S6892[83], 6}, {NULL}};
const Segment SGI111203[] = {{&S104514[78], 6}, {NULL}};
const Segment SGI111204[] = {{&S104514[105], 13}, {NULL}};
const Segment SGI111205[] = {{&S6893[44], 4}, {NULL}};
const Segment SGI111206[] = {{&S6893[44], 16}, {NULL}};
const Segment SGI111207[] = {{&S6893[30], 6}, {NULL}};
const Segment SGI111208[] = {{&S6893[31], 3}, {NULL}};
const Segment SGI111209[] = {{&S6893[74], 6}, {NULL}};
const Segment SGI111210[] = {{&S6894[44], 11}, {NULL}};
const Segment SGI111211[] = {{&S6894[44], 23}, {NULL}};
const Segment SGI111212[] = {{&S6894[30], 6}, {NULL}};
const Segment SGI111213[] = {{&S6894[31], 3}, {NULL}};
const Segment SGI111214[] = {{&S6894[81], 6}, {NULL}};
const Segment SGI111215[] = {{&S6895[44], 9}, {NULL}};
const Segment SGI111216[] = {{&S6895[44], 21}, {NULL}};
const Segment SGI111217[] = {{&S6895[30], 6}, {NULL}};
const Segment SGI111218[] = {{&S6895[31], 3}, {NULL}};
const Segment SGI111219[] = {{&S6895[79], 6}, {NULL}};
const Segment SGI111220[] = {{&S6904[44], 8}, {NULL}};
const Segment SGI111221[] = {{&S6904[44], 20}, {NULL}};
const Segment SGI111222[] = {{&S6904[30], 6}, {NULL}};
const Segment SGI111223[] = {{&S6904[31], 3}, {NULL}};
const Segment SGI111224[] = {{&S6904[78], 6}, {NULL}};
const Segment SGI111225[] = {{&S6906[44], 8}, {NULL}};
const Segment SGI111226[] = {{&S6906[44], 20}, {NULL}};
const Segment SGI111227[] = {{&S6906[30], 6}, {NULL}};
const Segment SGI111228[] = {{&S6906[31], 3}, {NULL}};
const Segment SGI111229[] = {{&S6906[78], 6}, {NULL}};
const Segment SGI111230[] = {{&S6907[44], 8}, {NULL}};
const Segment SGI111231[] = {{&S6907[44], 20}, {NULL}};
const Segment SGI111232[] = {{&S6907[30], 6}, {NULL}};
const Segment SGI111233[] = {{&S6907[31], 3}, {NULL}};
const Segment SGI111234[] = {{&S6907[78], 6}, {NULL}};
const Segment SGI111235[] = {{&S6909[44], 6}, {NULL}};
const Segment SGI111236[] = {{&S6909[44], 18}, {NULL}};
const Segment SGI111237[] = {{&S6909[30], 6}, {NULL}};
const Segment SGI111238[] = {{&S6909[31], 3}, {NULL}};
const Segment SGI111239[] = {{&S6909[76], 6}, {NULL}};
const Segment SGI111240[] = {{&S6911[44], 6}, {NULL}};
const Segment SGI111241[] = {{&S6911[44], 18}, {NULL}};
const Segment SGI111242[] = {{&S6911[30], 6}, {NULL}};
const Segment SGI111243[] = {{&S6911[31], 3}, {NULL}};
const Segment SGI111244[] = {{&S6911[76], 6}, {NULL}};
const Segment SGI111245[] = {{&S6913[44], 8}, {NULL}};
const Segment SGI111246[] = {{&S6913[44], 20}, {NULL}};
const Segment SGI111247[] = {{&S6913[30], 6}, {NULL}};
const Segment SGI111248[] = {{&S6913[31], 3}, {NULL}};
const Segment SGI111249[] = {{&S6913[78], 6}, {NULL}};
const Segment SGI111250[] = {{&S6914[44], 18}, {NULL}};
const Segment SGI111251[] = {{&S6914[44], 27}, {NULL}};
const Segment SGI111252[] = {{&S6914[30], 6}, {NULL}};
const Segment SGI111253[] = {{&S6914[31], 3}, {NULL}};
const Segment SGI111254[] = {{&S6914[85], 6}, {NULL}};
const Segment SGI111255[] = {{&S6915[44], 7}, {NULL}};
const Segment SGI111256[] = {{&S6915[44], 19}, {NULL}};
const Segment SGI111257[] = {{&S6915[30], 6}, {NULL}};
const Segment SGI111258[] = {{&S6915[31], 3}, {NULL}};
const Segment SGI111259[] = {{&S6915[77], 6}, {NULL}};
const Segment SGI111260[] = {{&S6916[44], 10}, {NULL}};
const Segment SGI111261[] = {{&S6916[44], 22}, {NULL}};
const Segment SGI111262[] = {{&S6916[30], 6}, {NULL}};
const Segment SGI111263[] = {{&S6916[31], 3}, {NULL}};
const Segment SGI111264[] = {{&S6916[80], 6}, {NULL}};
const Segment SGI111265[] = {{&S6917[44], 10}, {NULL}};
const Segment SGI111266[] = {{&S6917[44], 22}, {NULL}};
const Segment SGI111267[] = {{&S6917[30], 6}, {NULL}};
const Segment SGI111268[] = {{&S6917[31], 3}, {NULL}};
const Segment SGI111269[] = {{&S6917[80], 6}, {NULL}};
const Segment SGI111270[] = {{&S6918[44], 6}, {NULL}};
const Segment SGI111271[] = {{&S6918[44], 18}, {NULL}};
const Segment SGI111272[] = {{&S6918[30], 6}, {NULL}};
const Segment SGI111273[] = {{&S6918[31], 3}, {NULL}};
const Segment SGI111274[] = {{&S6918[76], 6}, {NULL}};
const Segment SGI111275[] = {{&S6921[44], 6}, {NULL}};
const Segment SGI111276[] = {{&S6921[44], 18}, {NULL}};
const Segment SGI111277[] = {{&S6921[30], 6}, {NULL}};
const Segment SGI111278[] = {{&S6921[31], 3}, {NULL}};
const Segment SGI111279[] = {{&S6921[76], 6}, {NULL}};
const Segment SGI111280[] = {{&S6924[44], 11}, {NULL}};
const Segment SGI111281[] = {{&S6924[44], 23}, {NULL}};
const Segment SGI111282[] = {{&S6924[30], 6}, {NULL}};
const Segment SGI111283[] = {{&S6924[31], 3}, {NULL}};
const Segment SGI111284[] = {{&S6924[81], 6}, {NULL}};
const Segment SGI111285[] = {{&S6925[44], 11}, {NULL}};
const Segment SGI111286[] = {{&S6925[44], 23}, {NULL}};
const Segment SGI111287[] = {{&S6925[30], 6}, {NULL}};
const Segment SGI111288[] = {{&S6925[31], 3}, {NULL}};
const Segment SGI111289[] = {{&S6925[81], 6}, {NULL}};
const Segment SGI111290[] = {{&S6926[44], 14}, {NULL}};
const Segment SGI111291[] = {{&S6926[44], 26}, {NULL}};
const Segment SGI111292[] = {{&S6926[30], 6}, {NULL}};
const Segment SGI111293[] = {{&S6926[87], 6}, {NULL}};
const Segment SGI111294[] = {{&S6927[44], 10}, {NULL}};
const Segment SGI111295[] = {{&S6927[44], 22}, {NULL}};
const Segment SGI111296[] = {{&S6927[30], 6}, {NULL}};
const Segment SGI111297[] = {{&S6927[31], 3}, {NULL}};
const Segment SGI111298[] = {{&S6927[80], 6}, {NULL}};
const Segment SGI111299[] = {{&S6928[44], 6}, {NULL}};
const Segment SGI111300[] = {{&S6928[44], 18}, {NULL}};
const Segment SGI111301[] = {{&S6928[30], 6}, {NULL}};
const Segment SGI111302[] = {{&S6928[31], 3}, {NULL}};
const Segment SGI111303[] = {{&S6928[76], 6}, {NULL}};
const Segment SGI111304[] = {{&S6931[44], 16}, {NULL}};
const Segment SGI111305[] = {{&S6931[44], 28}, {NULL}};
const Segment SGI111306[] = {{&S6931[30], 6}, {NULL}};
const Segment SGI111307[] = {{&S6931[31], 3}, {NULL}};
const Segment SGI111308[] = {{&S6931[86], 6}, {NULL}};
const Segment SGI111309[] = {{&S6938[44], 16}, {NULL}};
const Segment SGI111310[] = {{&S6938[44], 28}, {NULL}};
const Segment SGI111311[] = {{&S6938[30], 6}, {NULL}};
const Segment SGI111312[] = {{&S6938[31], 3}, {NULL}};
const Segment SGI111313[] = {{&S6938[86], 6}, {NULL}};
const Segment SGI111314[] = {{&S6942[44], 23}, {NULL}};
const Segment SGI111315[] = {{&S6942[44], 35}, {NULL}};
const Segment SGI111316[] = {{&S6942[30], 6}, {NULL}};
const Segment SGI111317[] = {{&S6942[31], 3}, {NULL}};
const Segment SGI111318[] = {{&S6942[93], 6}, {NULL}};
const Segment SGI111319[] = {{&S6943[44], 5}, {NULL}};
const Segment SGI111320[] = {{&S6943[44], 17}, {NULL}};
const Segment SGI111321[] = {{&S6943[30], 6}, {NULL}};
const Segment SGI111322[] = {{&S6943[78], 6}, {NULL}};
const Segment SGI111323[] = {{&S6944[44], 7}, {NULL}};
const Segment SGI111324[] = {{&S6944[44], 19}, {NULL}};
const Segment SGI111325[] = {{&S6944[30], 6}, {NULL}};
const Segment SGI111326[] = {{&S6944[31], 3}, {NULL}};
const Segment SGI111327[] = {{&S6944[77], 6}, {NULL}};
const Segment SGI111328[] = {{&S6946[44], 14}, {NULL}};
const Segment SGI111329[] = {{&S6946[44], 26}, {NULL}};
const Segment SGI111330[] = {{&S6946[30], 6}, {NULL}};
const Segment SGI111331[] = {{&S6946[31], 3}, {NULL}};
const Segment SGI111332[] = {{&S6946[84], 6}, {NULL}};
const Segment SGI111333[] = {{&S6948[44], 14}, {NULL}};
const Segment SGI111334[] = {{&S6948[44], 26}, {NULL}};
const Segment SGI111335[] = {{&S6948[30], 6}, {NULL}};
const Segment SGI111336[] = {{&S6948[31], 3}, {NULL}};
const Segment SGI111337[] = {{&S6948[84], 6}, {NULL}};
const Segment SGI111338[] = {{&S6949[44], 6}, {NULL}};
const Segment SGI111339[] = {{&S6949[44], 18}, {NULL}};
const Segment SGI111340[] = {{&S6949[30], 6}, {NULL}};
const Segment SGI111341[] = {{&S6949[31], 3}, {NULL}};
const Segment SGI111342[] = {{&S6949[76], 6}, {NULL}};
const Segment SGI111343[] = {{&S6951[44], 7}, {NULL}};
const Segment SGI111344[] = {{&S6951[44], 19}, {NULL}};
const Segment SGI111345[] = {{&S6951[30], 6}, {NULL}};
const Segment SGI111346[] = {{&S6951[31], 3}, {NULL}};
const Segment SGI111347[] = {{&S6951[77], 6}, {NULL}};
const Segment SGI111348[] = {{&S6952[44], 10}, {NULL}};
const Segment SGI111349[] = {{&S6952[44], 22}, {NULL}};
const Segment SGI111350[] = {{&S6952[30], 6}, {NULL}};
const Segment SGI111351[] = {{&S6952[31], 3}, {NULL}};
const Segment SGI111352[] = {{&S6952[80], 6}, {NULL}};
const Segment SGI111353[] = {{&S6953[44], 17}, {NULL}};
const Segment SGI111354[] = {{&S6953[44], 29}, {NULL}};
const Segment SGI111355[] = {{&S6953[30], 6}, {NULL}};
const Segment SGI111356[] = {{&S6953[31], 3}, {NULL}};
const Segment SGI111357[] = {{&S6953[87], 6}, {NULL}};
const Segment SGI111358[] = {{&S6954[44], 9}, {NULL}};
const Segment SGI111359[] = {{&S6954[44], 21}, {NULL}};
const Segment SGI111360[] = {{&S6954[30], 6}, {NULL}};
const Segment SGI111361[] = {{&S6954[31], 3}, {NULL}};
const Segment SGI111362[] = {{&S6954[79], 6}, {NULL}};
const Segment SGI111363[] = {{&S6955[44], 11}, {NULL}};
const Segment SGI111364[] = {{&S6955[44], 23}, {NULL}};
const Segment SGI111365[] = {{&S6955[30], 6}, {NULL}};
const Segment SGI111366[] = {{&S6955[31], 3}, {NULL}};
const Segment SGI111367[] = {{&S6955[81], 6}, {NULL}};
const Segment SGI111368[] = {{&S6956[44], 8}, {NULL}};
const Segment SGI111369[] = {{&S6956[44], 31}, {NULL}};
const Segment SGI111370[] = {{&S6956[30], 6}, {NULL}};
const Segment SGI111371[] = {{&S6956[31], 3}, {NULL}};
const Segment SGI111372[] = {{&S6956[89], 6}, {NULL}};
const Segment SGI111373[] = {{&S6958[44], 9}, {NULL}};
const Segment SGI111374[] = {{&S6958[44], 21}, {NULL}};
const Segment SGI111375[] = {{&S6958[30], 6}, {NULL}};
const Segment SGI111376[] = {{&S6958[31], 3}, {NULL}};
const Segment SGI111377[] = {{&S6958[79], 6}, {NULL}};
const Segment SGI111378[] = {{&S6959[44], 14}, {NULL}};
const Segment SGI111379[] = {{&S6959[44], 26}, {NULL}};
const Segment SGI111380[] = {{&S6959[30], 6}, {NULL}};
const Segment SGI111381[] = {{&S6959[31], 3}, {NULL}};
const Segment SGI111382[] = {{&S6959[84], 6}, {NULL}};
const Segment SGI111383[] = {{&S6960[44], 8}, {NULL}};
const Segment SGI111384[] = {{&S6960[44], 33}, {NULL}};
const Segment SGI111385[] = {{&S6960[30], 6}, {NULL}};
const Segment SGI111386[] = {{&S6960[31], 3}, {NULL}};
const Segment SGI111387[] = {{&S6960[91], 6}, {NULL}};
const Segment SGI111388[] = {{&S6961[44], 7}, {NULL}};
const Segment SGI111389[] = {{&S6961[44], 19}, {NULL}};
const Segment SGI111390[] = {{&S6961[30], 6}, {NULL}};
const Segment SGI111391[] = {{&S6961[31], 3}, {NULL}};
const Segment SGI111392[] = {{&S6961[77], 6}, {NULL}};
const Segment SGI111393[] = {{&S6962[44], 8}, {NULL}};
const Segment SGI111394[] = {{&S6962[44], 20}, {NULL}};
const Segment SGI111395[] = {{&S6962[30], 6}, {NULL}};
const Segment SGI111396[] = {{&S6962[81], 6}, {NULL}};
const Segment SGI111397[] = {{&S6965[44], 20}, {NULL}};
const Segment SGI111398[] = {{&S6965[44], 32}, {NULL}};
const Segment SGI111399[] = {{&S6965[30], 6}, {NULL}};
const Segment SGI111400[] = {{&S6965[31], 3}, {NULL}};
const Segment SGI111401[] = {{&S6965[90], 6}, {NULL}};
const Segment SGI111402[] = {{&S6966[44], 10}, {NULL}};
const Segment SGI111403[] = {{&S6966[44], 22}, {NULL}};
const Segment SGI111404[] = {{&S6966[30], 6}, {NULL}};
const Segment SGI111405[] = {{&S6966[31], 3}, {NULL}};
const Segment SGI111406[] = {{&S6966[80], 6}, {NULL}};
const Segment SGI111407[] = {{&S6970[44], 10}, {NULL}};
const Segment SGI111408[] = {{&S6970[44], 22}, {NULL}};
const Segment SGI111409[] = {{&S6970[30], 6}, {NULL}};
const Segment SGI111410[] = {{&S6970[31], 3}, {NULL}};
const Segment SGI111411[] = {{&S6970[80], 6}, {NULL}};
const Segment SGI111412[] = {{&S6971[44], 7}, {NULL}};
const Segment SGI111413[] = {{&S6971[44], 19}, {NULL}};
const Segment SGI111414[] = {{&S6971[30], 6}, {NULL}};
const Segment SGI111415[] = {{&S6971[31], 3}, {NULL}};
const Segment SGI111416[] = {{&S6971[77], 6}, {NULL}};
const Segment SGI111417[] = {{&S104638[44], 13}, {NULL}};
const Segment SGI111418[] = {{&S104638[44], 28}, {NULL}};
const Segment SGI111419[] = {{&S104638[30], 6}, {NULL}};
const Segment SGI111420[] = {{&S104638[121], 3}, {NULL}};
const Segment SGI111421[] = {{&S104638[86], 6}, {NULL}};
const Segment SGI111422[] = {{&S6976[44], 9}, {NULL}};
const Segment SGI111423[] = {{&S6976[44], 21}, {NULL}};
const Segment SGI111424[] = {{&S6976[30], 6}, {NULL}};
const Segment SGI111425[] = {{&S6976[31], 3}, {NULL}};
const Segment SGI111426[] = {{&S6976[79], 6}, {NULL}};
const Segment SGI111427[] = {{&S6977[44], 13}, {NULL}};
const Segment SGI111428[] = {{&S6977[44], 25}, {NULL}};
const Segment SGI111429[] = {{&S6977[30], 6}, {NULL}};
const Segment SGI111430[] = {{&S6977[31], 3}, {NULL}};
const Segment SGI111431[] = {{&S6977[83], 6}, {NULL}};
const Segment SGI111432[] = {{&S6978[44], 7}, {NULL}};
const Segment SGI111433[] = {{&S6978[44], 19}, {NULL}};
const Segment SGI111434[] = {{&S6978[30], 6}, {NULL}};
const Segment SGI111435[] = {{&S6978[31], 3}, {NULL}};
const Segment SGI111436[] = {{&S6978[77], 6}, {NULL}};
const Segment SGI111437[] = {{&S6979[44], 9}, {NULL}};
const Segment SGI111438[] = {{&S6979[44], 21}, {NULL}};
const Segment SGI111439[] = {{&S6979[30], 6}, {NULL}};
const Segment SGI111440[] = {{&S6979[31], 3}, {NULL}};
const Segment SGI111441[] = {{&S6979[79], 6}, {NULL}};
const Segment SGI111442[] = {{&S6980[44], 6}, {NULL}};
const Segment SGI111443[] = {{&S6980[44], 18}, {NULL}};
const Segment SGI111444[] = {{&S6980[30], 6}, {NULL}};
const Segment SGI111445[] = {{&S6980[31], 3}, {NULL}};
const Segment SGI111446[] = {{&S6980[76], 6}, {NULL}};
const Segment SGI111447[] = {{&S6981[44], 22}, {NULL}};
const Segment SGI111448[] = {{&S6981[44], 34}, {NULL}};
const Segment SGI111449[] = {{&S6981[30], 6}, {NULL}};
const Segment SGI111450[] = {{&S6981[31], 3}, {NULL}};
const Segment SGI111451[] = {{&S6981[92], 6}, {NULL}};
const Segment SGI111452[] = {{&S6983[44], 9}, {NULL}};
const Segment SGI111453[] = {{&S6983[44], 21}, {NULL}};
const Segment SGI111454[] = {{&S6983[30], 6}, {NULL}};
const Segment SGI111455[] = {{&S6983[31], 3}, {NULL}};
const Segment SGI111456[] = {{&S6983[79], 6}, {NULL}};
const Segment SGI111457[] = {{&S6984[44], 18}, {NULL}};
const Segment SGI111458[] = {{&S6984[44], 30}, {NULL}};
const Segment SGI111459[] = {{&S6984[30], 6}, {NULL}};
const Segment SGI111460[] = {{&S6984[31], 3}, {NULL}};
const Segment SGI111461[] = {{&S6984[88], 6}, {NULL}};
const Segment SGI111462[] = {{&S6985[44], 10}, {NULL}};
const Segment SGI111463[] = {{&S6985[44], 22}, {NULL}};
const Segment SGI111464[] = {{&S6985[30], 6}, {NULL}};
const Segment SGI111465[] = {{&S6985[31], 3}, {NULL}};
const Segment SGI111466[] = {{&S6985[80], 6}, {NULL}};
const Segment SGI111467[] = {{&S6987[44], 5}, {NULL}};
const Segment SGI111468[] = {{&S6987[44], 17}, {NULL}};
const Segment SGI111469[] = {{&S6987[30], 6}, {NULL}};
const Segment SGI111470[] = {{&S6987[31], 3}, {NULL}};
const Segment SGI111471[] = {{&S6987[75], 6}, {NULL}};
const Segment SGI111472[] = {{&S6988[44], 9}, {NULL}};
const Segment SGI111473[] = {{&S6988[44], 21}, {NULL}};
const Segment SGI111474[] = {{&S6988[30], 6}, {NULL}};
const Segment SGI111475[] = {{&S6988[31], 3}, {NULL}};
const Segment SGI111476[] = {{&S6988[79], 6}, {NULL}};
const Segment SGI111477[] = {{&S6989[44], 10}, {NULL}};
const Segment SGI111478[] = {{&S6989[44], 22}, {NULL}};
const Segment SGI111479[] = {{&S6989[30], 6}, {NULL}};
const Segment SGI111480[] = {{&S6989[31], 3}, {NULL}};
const Segment SGI111481[] = {{&S6989[80], 6}, {NULL}};
const Segment SGI111482[] = {{&S6990[44], 10}, {NULL}};
const Segment SGI111483[] = {{&S6990[44], 22}, {NULL}};
const Segment SGI111484[] = {{&S6990[30], 6}, {NULL}};
const Segment SGI111485[] = {{&S6990[31], 3}, {NULL}};
const Segment SGI111486[] = {{&S6990[80], 6}, {NULL}};
const Segment SGI111487[] = {{&S6991[44], 7}, {NULL}};
const Segment SGI111488[] = {{&S6991[44], 19}, {NULL}};
const Segment SGI111489[] = {{&S6991[30], 6}, {NULL}};
const Segment SGI111490[] = {{&S6991[31], 3}, {NULL}};
const Segment SGI111491[] = {{&S6991[77], 6}, {NULL}};
const Segment SGI111492[] = {{&S6992[44], 11}, {NULL}};
const Segment SGI111493[] = {{&S6992[44], 23}, {NULL}};
const Segment SGI111494[] = {{&S6992[30], 6}, {NULL}};
const Segment SGI111495[] = {{&S6992[31], 3}, {NULL}};
const Segment SGI111496[] = {{&S6992[81], 6}, {NULL}};
const Segment SGI111497[] = {{&S6993[44], 10}, {NULL}};
const Segment SGI111498[] = {{&S6993[44], 22}, {NULL}};
const Segment SGI111499[] = {{&S6993[30], 6}, {NULL}};
const Segment SGI111500[] = {{&S6993[31], 3}, {NULL}};
const Segment SGI111501[] = {{&S6993[80], 6}, {NULL}};
const Segment SGI111502[] = {{&S6994[44], 18}, {NULL}};
const Segment SGI111503[] = {{&S6994[44], 30}, {NULL}};
const Segment SGI111504[] = {{&S6994[30], 6}, {NULL}};
const Segment SGI111505[] = {{&S6994[31], 3}, {NULL}};
const Segment SGI111506[] = {{&S6994[88], 6}, {NULL}};
const Segment SGI111507[] = {{&S6995[44], 9}, {NULL}};
const Segment SGI111508[] = {{&S6995[44], 21}, {NULL}};
const Segment SGI111509[] = {{&S6995[30], 6}, {NULL}};
const Segment SGI111510[] = {{&S6995[31], 3}, {NULL}};
const Segment SGI111511[] = {{&S6995[79], 6}, {NULL}};
const Segment SGI111512[] = {{&S89378[34], 10}, {NULL}};
const Segment SGI111513[] = {{&S89378[27], 6}, {NULL}};
const Segment SGI111514[] = {{&S89378[97], 9}, {NULL}};
const Segment SGI111515[] = {{&S89378[70], 6}, {NULL}};
const Segment SGI111516[] = {{&S6996[44], 13}, {NULL}};
const Segment SGI111517[] = {{&S6996[44], 25}, {NULL}};
const Segment SGI111518[] = {{&S6996[30], 6}, {NULL}};
const Segment SGI111519[] = {{&S6996[31], 3}, {NULL}};
const Segment SGI111520[] = {{&S6996[83], 6}, {NULL}};
const Segment SGI111521[] = {{&S6997[44], 12}, {NULL}};
const Segment SGI111522[] = {{&S6997[44], 24}, {NULL}};
const Segment SGI111523[] = {{&S6997[30], 6}, {NULL}};
const Segment SGI111524[] = {{&S6997[31], 3}, {NULL}};
const Segment SGI111525[] = {{&S6997[82], 6}, {NULL}};
const Segment SGI111526[] = {{&S6998[44], 11}, {NULL}};
const Segment SGI111527[] = {{&S6998[44], 23}, {NULL}};
const Segment SGI111528[] = {{&S6998[30], 6}, {NULL}};
const Segment SGI111529[] = {{&S6998[31], 3}, {NULL}};
const Segment SGI111530[] = {{&S6998[81], 6}, {NULL}};
const Segment SGI111531[] = {{&S7000[44], 15}, {NULL}};
const Segment SGI111532[] = {{&S7000[44], 27}, {NULL}};
const Segment SGI111533[] = {{&S7000[30], 6}, {NULL}};
const Segment SGI111534[] = {{&S7000[31], 3}, {NULL}};
const Segment SGI111535[] = {{&S7000[85], 6}, {NULL}};
const Segment SGI111536[] = {{&S7002[44], 9}, {NULL}};
const Segment SGI111537[] = {{&S7002[44], 21}, {NULL}};
const Segment SGI111538[] = {{&S7002[30], 6}, {NULL}};
const Segment SGI111539[] = {{&S7002[31], 3}, {NULL}};
const Segment SGI111540[] = {{&S7002[79], 6}, {NULL}};
const Segment SGI111541[] = {{&S7003[44], 5}, {NULL}};
const Segment SGI111542[] = {{&S7003[44], 17}, {NULL}};
const Segment SGI111543[] = {{&S7003[30], 6}, {NULL}};
const Segment SGI111544[] = {{&S7003[31], 3}, {NULL}};
const Segment SGI111545[] = {{&S7003[75], 6}, {NULL}};
const Segment SGI111546[] = {{&S7004[44], 13}, {NULL}};
const Segment SGI111547[] = {{&S7004[44], 25}, {NULL}};
const Segment SGI111548[] = {{&S7004[30], 6}, {NULL}};
const Segment SGI111549[] = {{&S7004[31], 3}, {NULL}};
const Segment SGI111550[] = {{&S7004[83], 6}, {NULL}};
const Segment SGI111551[] = {{&S7005[44], 7}, {NULL}};
const Segment SGI111552[] = {{&S7005[44], 19}, {NULL}};
const Segment SGI111553[] = {{&S7005[30], 6}, {NULL}};
const Segment SGI111554[] = {{&S7005[31], 3}, {NULL}};
const Segment SGI111555[] = {{&S7005[77], 6}, {NULL}};
const Segment SGI111556[] = {{&S7006[44], 8}, {NULL}};
const Segment SGI111557[] = {{&S7006[44], 20}, {NULL}};
const Segment SGI111558[] = {{&S7006[30], 6}, {NULL}};
const Segment SGI111559[] = {{&S7006[31], 3}, {NULL}};
const Segment SGI111560[] = {{&S7006[78], 6}, {NULL}};
const Segment SGI111561[] = {{&S7007[44], 12}, {NULL}};
const Segment SGI111562[] = {{&S7007[44], 24}, {NULL}};
const Segment SGI111563[] = {{&S7007[30], 6}, {NULL}};
const Segment SGI111564[] = {{&S7007[31], 3}, {NULL}};
const Segment SGI111565[] = {{&S7007[82], 6}, {NULL}};
const Segment SGI111566[] = {{&S7008[44], 10}, {NULL}};
const Segment SGI111567[] = {{&S7008[44], 22}, {NULL}};
const Segment SGI111568[] = {{&S7008[30], 6}, {NULL}};
const Segment SGI111569[] = {{&S7008[31], 3}, {NULL}};
const Segment SGI111570[] = {{&S7008[80], 6}, {NULL}};
const Segment SGI111571[] = {{&S7009[44], 5}, {NULL}};
const Segment SGI111572[] = {{&S7009[44], 17}, {NULL}};
const Segment SGI111573[] = {{&S7009[30], 6}, {NULL}};
const Segment SGI111574[] = {{&S7009[31], 3}, {NULL}};
const Segment SGI111575[] = {{&S7009[75], 6}, {NULL}};
const Segment SGI111576[] = {{&S7011[44], 10}, {NULL}};
const Segment SGI111577[] = {{&S7011[44], 22}, {NULL}};
const Segment SGI111578[] = {{&S7011[30], 6}, {NULL}};
const Segment SGI111579[] = {{&S7011[31], 3}, {NULL}};
const Segment SGI111580[] = {{&S7011[80], 6}, {NULL}};
const Segment SGI111581[] = {{&S7012[44], 8}, {NULL}};
const Segment SGI111582[] = {{&S7012[44], 20}, {NULL}};
const Segment SGI111583[] = {{&S7012[30], 6}, {NULL}};
const Segment SGI111584[] = {{&S7012[31], 3}, {NULL}};
const Segment SGI111585[] = {{&S7012[78], 6}, {NULL}};
const Segment SGI111586[] = {{&S7013[44], 4}, {NULL}};
const Segment SGI111587[] = {{&S7013[44], 16}, {NULL}};
const Segment SGI111588[] = {{&S7013[30], 6}, {NULL}};
const Segment SGI111589[] = {{&S7013[31], 3}, {NULL}};
const Segment SGI111590[] = {{&S7013[74], 6}, {NULL}};
const Segment SGI111591[] = {{&S7014[44], 11}, {NULL}};
const Segment SGI111592[] = {{&S7014[44], 23}, {NULL}};
const Segment SGI111593[] = {{&S7014[30], 6}, {NULL}};
const Segment SGI111594[] = {{&S7014[31], 3}, {NULL}};
const Segment SGI111595[] = {{&S7014[81], 6}, {NULL}};
const Segment SGI111596[] = {{&S7015[44], 8}, {NULL}};
const Segment SGI111597[] = {{&S7015[44], 20}, {NULL}};
const Segment SGI111598[] = {{&S7015[30], 6}, {NULL}};
const Segment SGI111599[] = {{&S7015[31], 3}, {NULL}};
const Segment SGI111600[] = {{&S7015[78], 6}, {NULL}};
const Segment SGI111601[] = {{&S7016[44], 12}, {NULL}};
const Segment SGI111602[] = {{&S7016[44], 24}, {NULL}};
const Segment SGI111603[] = {{&S7016[30], 6}, {NULL}};
const Segment SGI111604[] = {{&S7016[31], 3}, {NULL}};
const Segment SGI111605[] = {{&S7016[82], 6}, {NULL}};
const Segment SGI111606[] = {{&S7017[44], 16}, {NULL}};
const Segment SGI111607[] = {{&S7017[44], 28}, {NULL}};
const Segment SGI111608[] = {{&S7017[30], 6}, {NULL}};
const Segment SGI111609[] = {{&S7017[31], 3}, {NULL}};
const Segment SGI111610[] = {{&S7017[86], 6}, {NULL}};
const Segment SGI111611[] = {{&S7018[44], 7}, {NULL}};
const Segment SGI111612[] = {{&S7018[44], 19}, {NULL}};
const Segment SGI111613[] = {{&S7018[30], 6}, {NULL}};
const Segment SGI111614[] = {{&S7018[31], 3}, {NULL}};
const Segment SGI111615[] = {{&S7018[77], 6}, {NULL}};
const Segment SGI111616[] = {{&S7020[44], 5}, {NULL}};
const Segment SGI111617[] = {{&S7020[44], 17}, {NULL}};
const Segment SGI111618[] = {{&S7020[30], 6}, {NULL}};
const Segment SGI111619[] = {{&S7020[31], 3}, {NULL}};
const Segment SGI111620[] = {{&S7020[75], 6}, {NULL}};
const Segment SGI111621[] = {{&S89435[44], 12}, {NULL}};
const Segment SGI111622[] = {{&S89435[44], 24}, {NULL}};
const Segment SGI111623[] = {{&S89435[30], 6}, {NULL}};
const Segment SGI111624[] = {{&S89435[31], 3}, {NULL}};
const Segment SGI111625[] = {{&S7021[44], 9}, {NULL}};
const Segment SGI111626[] = {{&S89435[82], 6}, {NULL}};
const Segment SGI111627[] = {{&S7021[30], 6}, {NULL}};
const Segment SGI111628[] = {{&S7021[31], 3}, {NULL}};
const Segment SGI111629[] = {{&S7021[81], 6}, {NULL}};
const Segment SGI111630[] = {{&S7022[38], 7}, {NULL}};
const Segment SGI111631[] = {{&S7022[31], 6}, {NULL}};
const Segment SGI111632[] = {{&S7023[44], 5}, {NULL}};
const Segment SGI111633[] = {{&S7023[44], 17}, {NULL}};
const Segment SGI111634[] = {{&S7023[30], 6}, {NULL}};
const Segment SGI111635[] = {{&S7023[31], 3}, {NULL}};
const Segment SGI111636[] = {{&S7023[75], 6}, {NULL}};
const Segment SGI111637[] = {{&S7024[44], 11}, {NULL}};
const Segment SGI111638[] = {{&S7024[44], 23}, {NULL}};
const Segment SGI111639[] = {{&S7024[30], 6}, {NULL}};
const Segment SGI111640[] = {{&S7024[31], 3}, {NULL}};
const Segment SGI111641[] = {{&S7024[81], 6}, {NULL}};
const Segment SGI111642[] = {{&S7025[44], 5}, {NULL}};
const Segment SGI111643[] = {{&S7025[44], 17}, {NULL}};
const Segment SGI111644[] = {{&S7025[30], 6}, {NULL}};
const Segment SGI111645[] = {{&S7025[31], 3}, {NULL}};
const Segment SGI111646[] = {{&S7025[75], 6}, {NULL}};
const Segment SGI111647[] = {{&S7026[44], 9}, {NULL}};
const Segment SGI111648[] = {{&S7026[44], 21}, {NULL}};
const Segment SGI111649[] = {{&S7026[30], 6}, {NULL}};
const Segment SGI111650[] = {{&S7026[31], 3}, {NULL}};
const Segment SGI111651[] = {{&S7026[79], 6}, {NULL}};
const Segment SGI111652[] = {{&S89447[34], 5}, {NULL}};
const Segment SGI111653[] = {{&S89447[27], 6}, {NULL}};
const Segment SGI111654[] = {{&S7027[44], 5}, {NULL}};
const Segment SGI111655[] = {{&S7027[44], 17}, {NULL}};
const Segment SGI111656[] = {{&S7027[30], 6}, {NULL}};
const Segment SGI111657[] = {{&S89447[92], 9}, {NULL}};
const Segment SGI111658[] = {{&S89447[65], 6}, {NULL}};
const Segment SGI111659[] = {{&S7027[31], 3}, {NULL}};
const Segment SGI111660[] = {{&S7027[75], 6}, {NULL}};
const Segment SGI111661[] = {{&S7028[44], 8}, {NULL}};
const Segment SGI111662[] = {{&S7028[44], 20}, {NULL}};
const Segment SGI111663[] = {{&S7028[30], 6}, {NULL}};
const Segment SGI111664[] = {{&S7028[31], 3}, {NULL}};
const Segment SGI111665[] = {{&S7028[78], 6}, {NULL}};
const Segment SGI111666[] = {{&S7029[44], 22}, {NULL}};
const Segment SGI111667[] = {{&S7029[44], 31}, {NULL}};
const Segment SGI111668[] = {{&S7029[30], 6}, {NULL}};
const Segment SGI111669[] = {{&S89451[34], 16}, {NULL}};
const Segment SGI111670[] = {{&S89451[27], 6}, {NULL}};
const Segment SGI111671[] = {{&S7029[31], 3}, {NULL}};
const Segment SGI111672[] = {{&S7029[89], 6}, {NULL}};
const Segment SGI111673[] = {{&S89451[117], 9}, {NULL}};
const Segment SGI111674[] = {{&S89451[90], 6}, {NULL}};
const Segment SGI111675[] = {{&S7030[44], 18}, {NULL}};
const Segment SGI111676[] = {{&S7030[44], 30}, {NULL}};
const Segment SGI111677[] = {{&S7030[30], 6}, {NULL}};
const Segment SGI111678[] = {{&S7030[31], 3}, {NULL}};
const Segment SGI111679[] = {{&S7030[88], 6}, {NULL}};
const Segment SGI111680[] = {{&S7031[44], 6}, {NULL}};
const Segment SGI111681[] = {{&S7031[44], 18}, {NULL}};
const Segment SGI111682[] = {{&S7031[30], 6}, {NULL}};
const Segment SGI111683[] = {{&S7031[31], 3}, {NULL}};
const Segment SGI111684[] = {{&S7031[76], 6}, {NULL}};
const Segment SGI111685[] = {{&S7032[44], 6}, {NULL}};
const Segment SGI111686[] = {{&S7032[44], 18}, {NULL}};
const Segment SGI111687[] = {{&S7032[30], 6}, {NULL}};
const Segment SGI111688[] = {{&S7032[31], 3}, {NULL}};
const Segment SGI111689[] = {{&S7032[76], 6}, {NULL}};
const Segment SGI111690[] = {{&S7033[44], 8}, {NULL}};
const Segment SGI111691[] = {{&S7033[44], 24}, {NULL}};
const Segment SGI111692[] = {{&S7033[30], 6}, {NULL}};
const Segment SGI111693[] = {{&S7033[31], 3}, {NULL}};
const Segment SGI111694[] = {{&S7033[82], 6}, {NULL}};
const Segment SGI111695[] = {{&S7034[44], 11}, {NULL}};
const Segment SGI111696[] = {{&S7034[44], 23}, {NULL}};
const Segment SGI111697[] = {{&S7034[30], 6}, {NULL}};
const Segment SGI111698[] = {{&S7034[31], 3}, {NULL}};
const Segment SGI111699[] = {{&S7034[81], 6}, {NULL}};
const Segment SGI111700[] = {{&S7035[43], 9}, {NULL}};
const Segment SGI111701[] = {{&S7035[43], 21}, {NULL}};
const Segment SGI111702[] = {{&S7035[29], 6}, {NULL}};
const Segment SGI111703[] = {{&S7035[30], 3}, {NULL}};
const Segment SGI111704[] = {{&S7035[78], 6}, {NULL}};
const Segment SGI111705[] = {{&S7036[44], 6}, {NULL}};
const Segment SGI111706[] = {{&S7036[44], 18}, {NULL}};
const Segment SGI111707[] = {{&S7036[30], 6}, {NULL}};
const Segment SGI111708[] = {{&S89465[45], 15}, {NULL}};
const Segment SGI111709[] = {{&S89465[30], 6}, {NULL}};
const Segment SGI111710[] = {{&S7036[31], 3}, {NULL}};
const Segment SGI111711[] = {{&S7036[76], 6}, {NULL}};
const Segment SGI111712[] = {{&S89465[31], 3}, {NULL}};
const Segment SGI111713[] = {{&S89465[86], 5}, {NULL}};
const Segment SGI111714[] = {{&S89465[144], 13}, {NULL}};
const Segment SGI111715[] = {{&S7037[44], 11}, {NULL}};
const Segment SGI111716[] = {{&S7037[44], 23}, {NULL}};
const Segment SGI111717[] = {{&S7037[30], 6}, {NULL}};
const Segment SGI111718[] = {{&S7037[31], 3}, {NULL}};
const Segment SGI111719[] = {{&S89466[30], 6}, {NULL}};
const Segment SGI111720[] = {{&S7037[81], 6}, {NULL}};
const Segment SGI111721[] = {{&S89466[31], 3}, {NULL}};
const Segment SGI111722[] = {{&S89466[86], 5}, {NULL}};
const Segment SGI111723[] = {{&S89466[144], 13}, {NULL}};
const Segment SGI111724[] = {{&S7038[44], 6}, {NULL}};
const Segment SGI111725[] = {{&S7038[44], 18}, {NULL}};
const Segment SGI111726[] = {{&S7038[30], 6}, {NULL}};
const Segment SGI111727[] = {{&S7038[31], 3}, {NULL}};
const Segment SGI111728[] = {{&S7038[76], 6}, {NULL}};
const Segment SGI111729[] = {{&S89468[34], 10}, {NULL}};
const Segment SGI111730[] = {{&S89468[27], 6}, {NULL}};
const Segment SGI111731[] = {{&S7039[44], 11}, {NULL}};
const Segment SGI111732[] = {{&S7039[44], 23}, {NULL}};
const Segment SGI111733[] = {{&S89468[97], 9}, {NULL}};
const Segment SGI111734[] = {{&S7039[30], 6}, {NULL}};
const Segment SGI111735[] = {{&S89468[70], 6}, {NULL}};
const Segment SGI111736[] = {{&S7039[31], 3}, {NULL}};
const Segment SGI111737[] = {{&S7039[81], 6}, {NULL}};
const Segment SGI111738[] = {{&S7040[44], 5}, {NULL}};
const Segment SGI111739[] = {{&S7040[44], 17}, {NULL}};
const Segment SGI111740[] = {{&S7040[30], 6}, {NULL}};
const Segment SGI111741[] = {{&S7040[31], 3}, {NULL}};
const Segment SGI111742[] = {{&S7040[75], 6}, {NULL}};
const Segment SGI111743[] = {{&S7041[44], 9}, {NULL}};
const Segment SGI111744[] = {{&S7041[44], 21}, {NULL}};
const Segment SGI111745[] = {{&S7041[30], 6}, {NULL}};
const Segment SGI111746[] = {{&S7041[31], 3}, {NULL}};
const Segment SGI111747[] = {{&S7041[79], 6}, {NULL}};
const Segment SGI111748[] = {{&S7042[44], 10}, {NULL}};
const Segment SGI111749[] = {{&S7042[44], 22}, {NULL}};
const Segment SGI111750[] = {{&S7042[30], 6}, {NULL}};
const Segment SGI111751[] = {{&S7042[31], 3}, {NULL}};
const Segment SGI111752[] = {{&S7042[80], 6}, {NULL}};
const Segment SGI111753[] = {{&S89476[44], 11}, {NULL}};
const Segment SGI111754[] = {{&S89476[44], 23}, {NULL}};
const Segment SGI111755[] = {{&S89476[30], 6}, {NULL}};
const Segment SGI111756[] = {{&S89476[81], 6}, {NULL}};
const Segment SGI111757[] = {{&S89476[108], 13}, {NULL}};
const Segment SGI111758[] = {{&S7044[44], 8}, {NULL}};
const Segment SGI111759[] = {{&S7044[44], 20}, {NULL}};
const Segment SGI111760[] = {{&S7044[30], 6}, {NULL}};
const Segment SGI111761[] = {{&S7044[31], 3}, {NULL}};
const Segment SGI111762[] = {{&S7044[78], 6}, {NULL}};
const Segment SGI111763[] = {{&S89483[34], 17}, {NULL}};
const Segment SGI111764[] = {{&S89483[27], 6}, {NULL}};
const Segment SGI111765[] = {{&S89483[104], 9}, {NULL}};
const Segment SGI111766[] = {{&S89483[77], 6}, {NULL}};
const Segment SGI111767[] = {{&S7053[44], 8}, {NULL}};
const Segment SGI111768[] = {{&S7053[44], 20}, {NULL}};
const Segment SGI111769[] = {{&S7053[30], 6}, {NULL}};
const Segment SGI111770[] = {{&S7053[31], 3}, {NULL}};
const Segment SGI111771[] = {{&S7053[78], 6}, {NULL}};
const Segment SGI111772[] = {{&S7055[44], 23}, {NULL}};
const Segment SGI111773[] = {{&S7055[44], 35}, {NULL}};
const Segment SGI111774[] = {{&S7055[30], 6}, {NULL}};
const Segment SGI111775[] = {{&S7055[31], 3}, {NULL}};
const Segment SGI111776[] = {{&S7055[93], 6}, {NULL}};
const Segment SGI111777[] = {{&S7056[44], 6}, {NULL}};
const Segment SGI111778[] = {{&S7056[44], 18}, {NULL}};
const Segment SGI111779[] = {{&S7056[30], 6}, {NULL}};
const Segment SGI111780[] = {{&S7056[31], 3}, {NULL}};
const Segment SGI111781[] = {{&S7056[76], 6}, {NULL}};
const Segment SGI111782[] = {{&S7057[44], 10}, {NULL}};
const Segment SGI111783[] = {{&S7057[44], 22}, {NULL}};
const Segment SGI111784[] = {{&S7057[30], 6}, {NULL}};
const Segment SGI111785[] = {{&S7057[31], 3}, {NULL}};
const Segment SGI111786[] = {{&S7057[80], 6}, {NULL}};
const Segment SGI111787[] = {{&S7060[44], 12}, {NULL}};
const Segment SGI111788[] = {{&S7060[44], 24}, {NULL}};
const Segment SGI111789[] = {{&S7060[30], 6}, {NULL}};
const Segment SGI111790[] = {{&S7060[31], 3}, {NULL}};
const Segment SGI111791[] = {{&S7060[82], 6}, {NULL}};
const Segment SGI111792[] = {{&S7061[44], 6}, {NULL}};
const Segment SGI111793[] = {{&S7061[44], 18}, {NULL}};
const Segment SGI111794[] = {{&S7061[30], 6}, {NULL}};
const Segment SGI111795[] = {{&S7061[31], 3}, {NULL}};
const Segment SGI111796[] = {{&S7061[76], 6}, {NULL}};
const Segment SGI111797[] = {{&S7062[44], 9}, {NULL}};
const Segment SGI111798[] = {{&S7062[44], 21}, {NULL}};
const Segment SGI111799[] = {{&S7062[30], 6}, {NULL}};
const Segment SGI111800[] = {{&S7062[31], 3}, {NULL}};
const Segment SGI111801[] = {{&S7062[79], 6}, {NULL}};
const Segment SGI111802[] = {{&S7064[44], 5}, {NULL}};
const Segment SGI111803[] = {{&S7064[44], 17}, {NULL}};
const Segment SGI111804[] = {{&S7064[30], 6}, {NULL}};
const Segment SGI111805[] = {{&S7064[31], 3}, {NULL}};
const Segment SGI111806[] = {{&S7064[75], 6}, {NULL}};
const Segment SGI111807[] = {{&S7066[44], 17}, {NULL}};
const Segment SGI111808[] = {{&S7066[44], 29}, {NULL}};
const Segment SGI111809[] = {{&S7066[30], 6}, {NULL}};
const Segment SGI111810[] = {{&S7066[31], 3}, {NULL}};
const Segment SGI111811[] = {{&S7066[87], 6}, {NULL}};
const Segment SGI111812[] = {{&S7067[44], 12}, {NULL}};
const Segment SGI111813[] = {{&S7067[44], 24}, {NULL}};
const Segment SGI111814[] = {{&S7067[30], 6}, {NULL}};
const Segment SGI111815[] = {{&S7067[31], 3}, {NULL}};
const Segment SGI111816[] = {{&S7067[82], 6}, {NULL}};
const Segment SGI111817[] = {{&S7069[44], 14}, {NULL}};
const Segment SGI111818[] = {{&S7069[44], 26}, {NULL}};
const Segment SGI111819[] = {{&S7069[30], 6}, {NULL}};
const Segment SGI111820[] = {{&S7069[31], 3}, {NULL}};
const Segment SGI111821[] = {{&S7069[84], 6}, {NULL}};
const Segment SGI111822[] = {{&S7070[44], 10}, {NULL}};
const Segment SGI111823[] = {{&S7070[44], 22}, {NULL}};
const Segment SGI111824[] = {{&S7070[30], 6}, {NULL}};
const Segment SGI111825[] = {{&S7070[31], 3}, {NULL}};
const Segment SGI111826[] = {{&S7070[80], 6}, {NULL}};
const Segment SGI111827[] = {{&S7071[44], 11}, {NULL}};
const Segment SGI111828[] = {{&S7071[44], 23}, {NULL}};
const Segment SGI111829[] = {{&S7071[30], 6}, {NULL}};
const Segment SGI111830[] = {{&S7071[31], 3}, {NULL}};
const Segment SGI111831[] = {{&S7071[81], 6}, {NULL}};
const Segment SGI111832[] = {{&S7073[44], 6}, {NULL}};
const Segment SGI111833[] = {{&S7073[44], 18}, {NULL}};
const Segment SGI111834[] = {{&S7073[30], 6}, {NULL}};
const Segment SGI111835[] = {{&S7073[31], 3}, {NULL}};
const Segment SGI111836[] = {{&S7073[76], 6}, {NULL}};
const Segment SGI111837[] = {{&S7074[44], 6}, {NULL}};
const Segment SGI111838[] = {{&S7074[44], 18}, {NULL}};
const Segment SGI111839[] = {{&S7074[30], 6}, {NULL}};
const Segment SGI111840[] = {{&S7074[31], 3}, {NULL}};
const Segment SGI111841[] = {{&S7074[76], 6}, {NULL}};
const Segment SGI111842[] = {{&S7075[44], 10}, {NULL}};
const Segment SGI111843[] = {{&S7075[44], 22}, {NULL}};
const Segment SGI111844[] = {{&S7075[30], 6}, {NULL}};
const Segment SGI111845[] = {{&S7075[31], 3}, {NULL}};
const Segment SGI111846[] = {{&S7075[80], 6}, {NULL}};
const Segment SGI111847[] = {{&S7077[44], 19}, {NULL}};
const Segment SGI111848[] = {{&S7077[44], 31}, {NULL}};
const Segment SGI111849[] = {{&S7077[30], 6}, {NULL}};
const Segment SGI111850[] = {{&S7077[31], 3}, {NULL}};
const Segment SGI111851[] = {{&S7077[89], 6}, {NULL}};
const Segment SGI111852[] = {{&S7078[44], 10}, {NULL}};
const Segment SGI111853[] = {{&S7078[44], 22}, {NULL}};
const Segment SGI111854[] = {{&S7078[30], 6}, {NULL}};
const Segment SGI111855[] = {{&S7078[31], 3}, {NULL}};
const Segment SGI111856[] = {{&S7078[80], 6}, {NULL}};
const Segment SGI111857[] = {{&S7079[38], 5}, {NULL}};
const Segment SGI111858[] = {{&S7079[31], 6}, {NULL}};
const Segment SGI111859[] = {{&S7080[44], 13}, {NULL}};
const Segment SGI111860[] = {{&S7080[44], 25}, {NULL}};
const Segment SGI111861[] = {{&S7080[30], 6}, {NULL}};
const Segment SGI111862[] = {{&S7080[31], 3}, {NULL}};
const Segment SGI111863[] = {{&S7080[83], 6}, {NULL}};
const Segment SGI111864[] = {{&S7081[44], 9}, {NULL}};
const Segment SGI111865[] = {{&S7081[44], 21}, {NULL}};
const Segment SGI111866[] = {{&S7081[30], 6}, {NULL}};
const Segment SGI111867[] = {{&S7081[31], 3}, {NULL}};
const Segment SGI111868[] = {{&S7081[79], 6}, {NULL}};
const Segment SGI111869[] = {{&S7083[44], 12}, {NULL}};
const Segment SGI111870[] = {{&S7083[44], 24}, {NULL}};
const Segment SGI111871[] = {{&S7083[30], 6}, {NULL}};
const Segment SGI111872[] = {{&S7083[31], 3}, {NULL}};
const Segment SGI111873[] = {{&S89544[34], 9}, {NULL}};
const Segment SGI111874[] = {{&S7083[82], 6}, {NULL}};
const Segment SGI111875[] = {{&S89544[27], 6}, {NULL}};
const Segment SGI111876[] = {{&S89544[104], 9}, {NULL}};
const Segment SGI111877[] = {{&S89544[77], 6}, {NULL}};
const Segment SGI111878[] = {{&S7084[44], 8}, {NULL}};
const Segment SGI111879[] = {{&S7084[44], 20}, {NULL}};
const Segment SGI111880[] = {{&S7084[30], 6}, {NULL}};
const Segment SGI111881[] = {{&S7084[31], 3}, {NULL}};
const Segment SGI111882[] = {{&S7084[78], 6}, {NULL}};
const Segment SGI111883[] = {{&S7085[44], 23}, {NULL}};
const Segment SGI111884[] = {{&S7085[44], 35}, {NULL}};
const Segment SGI111885[] = {{&S7085[30], 6}, {NULL}};
const Segment SGI111886[] = {{&S7085[31], 3}, {NULL}};
const Segment SGI111887[] = {{&S7085[93], 6}, {NULL}};
const Segment SGI111888[] = {{&S7086[44], 11}, {NULL}};
const Segment SGI111889[] = {{&S7086[44], 23}, {NULL}};
const Segment SGI111890[] = {{&S7086[30], 6}, {NULL}};
const Segment SGI111891[] = {{&S7086[31], 3}, {NULL}};
const Segment SGI111892[] = {{&S7086[81], 6}, {NULL}};
const Segment SGI111893[] = {{&S7088[44], 30}, {NULL}};
const Segment SGI111894[] = {{&S7088[44], 42}, {NULL}};
const Segment SGI111895[] = {{&S7088[30], 6}, {NULL}};
const Segment SGI111896[] = {{&S7088[31], 3}, {NULL}};
const Segment SGI111897[] = {{&S7088[100], 6}, {NULL}};
const Segment SGI111898[] = {{&S7089[44], 11}, {NULL}};
const Segment SGI111899[] = {{&S7089[44], 23}, {NULL}};
const Segment SGI111900[] = {{&S7089[30], 6}, {NULL}};
const Segment SGI111901[] = {{&S7089[31], 3}, {NULL}};
const Segment SGI111902[] = {{&S7089[81], 6}, {NULL}};
const Segment SGI111903[] = {{&S7092[44], 6}, {NULL}};
const Segment SGI111904[] = {{&S7092[44], 18}, {NULL}};
const Segment SGI111905[] = {{&S7092[30], 6}, {NULL}};
const Segment SGI111906[] = {{&S7092[31], 3}, {NULL}};
const Segment SGI111907[] = {{&S7092[76], 6}, {NULL}};
const Segment SGI111908[] = {{&S7093[44], 9}, {NULL}};
const Segment SGI111909[] = {{&S7093[44], 27}, {NULL}};
const Segment SGI111910[] = {{&S7093[30], 6}, {NULL}};
const Segment SGI111911[] = {{&S7093[31], 3}, {NULL}};
const Segment SGI111912[] = {{&S7093[85], 6}, {NULL}};
const Segment SGI111913[] = {{&S7094[44], 9}, {NULL}};
const Segment SGI111914[] = {{&S7094[44], 21}, {NULL}};
const Segment SGI111915[] = {{&S7094[30], 6}, {NULL}};
const Segment SGI111916[] = {{&S7094[31], 3}, {NULL}};
const Segment SGI111917[] = {{&S7094[79], 6}, {NULL}};
const Segment SGI111918[] = {{&S7095[44], 15}, {NULL}};
const Segment SGI111919[] = {{&S7095[44], 31}, {NULL}};
const Segment SGI111920[] = {{&S7095[30], 6}, {NULL}};
const Segment SGI111921[] = {{&S7095[31], 3}, {NULL}};
const Segment SGI111922[] = {{&S7095[89], 6}, {NULL}};
const Segment SGI111923[] = {{&S7097[44], 6}, {NULL}};
const Segment SGI111924[] = {{&S7097[44], 18}, {NULL}};
const Segment SGI111925[] = {{&S7097[30], 6}, {NULL}};
const Segment SGI111926[] = {{&S7097[31], 3}, {NULL}};
const Segment SGI111927[] = {{&S7097[76], 6}, {NULL}};
const Segment SGI111928[] = {{&S7098[44], 9}, {NULL}};
const Segment SGI111929[] = {{&S7098[44], 21}, {NULL}};
const Segment SGI111930[] = {{&S7098[30], 6}, {NULL}};
const Segment SGI111931[] = {{&S7098[31], 3}, {NULL}};
const Segment SGI111932[] = {{&S7098[79], 6}, {NULL}};
const Segment SGI111933[] = {{&S7101[44], 10}, {NULL}};
const Segment SGI111934[] = {{&S7101[44], 22}, {NULL}};
const Segment SGI111935[] = {{&S7101[30], 6}, {NULL}};
const Segment SGI111936[] = {{&S7101[31], 3}, {NULL}};
const Segment SGI111937[] = {{&S7101[80], 6}, {NULL}};
const Segment SGI111938[] = {{&S7102[44], 6}, {NULL}};
const Segment SGI111939[] = {{&S7102[44], 18}, {NULL}};
const Segment SGI111940[] = {{&S7102[30], 6}, {NULL}};
const Segment SGI111941[] = {{&S7102[31], 3}, {NULL}};
const Segment SGI111942[] = {{&S7102[76], 6}, {NULL}};
const Segment SGI111943[] = {{&S89584[44], 8}, {NULL}};
const Segment SGI111944[] = {{&S89584[44], 20}, {NULL}};
const Segment SGI111945[] = {{&S89584[30], 6}, {NULL}};
const Segment SGI111946[] = {{&S7103[44], 4}, {NULL}};
const Segment SGI111947[] = {{&S7103[44], 16}, {NULL}};
const Segment SGI111948[] = {{&S7103[30], 6}, {NULL}};
const Segment SGI111949[] = {{&S89584[31], 3}, {NULL}};
const Segment SGI111950[] = {{&S89584[78], 6}, {NULL}};
const Segment SGI111951[] = {{&S7103[31], 3}, {NULL}};
const Segment SGI111952[] = {{&S7103[74], 6}, {NULL}};
const Segment SGI111953[] = {{&S7105[44], 15}, {NULL}};
const Segment SGI111954[] = {{&S7105[44], 27}, {NULL}};
const Segment SGI111955[] = {{&S7105[30], 6}, {NULL}};
const Segment SGI111956[] = {{&S7105[31], 3}, {NULL}};
const Segment SGI111957[] = {{&S7105[85], 6}, {NULL}};
const Segment SGI111958[] = {{&S7106[44], 7}, {NULL}};
const Segment SGI111959[] = {{&S7106[44], 19}, {NULL}};
const Segment SGI111960[] = {{&S7106[30], 6}, {NULL}};
const Segment SGI111961[] = {{&S7106[31], 3}, {NULL}};
const Segment SGI111962[] = {{&S7106[77], 6}, {NULL}};
const Segment SGI111963[] = {{&S7107[44], 7}, {NULL}};
const Segment SGI111964[] = {{&S7107[44], 19}, {NULL}};
const Segment SGI111965[] = {{&S7107[30], 6}, {NULL}};
const Segment SGI111966[] = {{&S7107[31], 3}, {NULL}};
const Segment SGI111967[] = {{&S7107[77], 6}, {NULL}};
const Segment SGI111968[] = {{&S7108[44], 5}, {NULL}};
const Segment SGI111969[] = {{&S7108[44], 17}, {NULL}};
const Segment SGI111970[] = {{&S7108[30], 6}, {NULL}};
const Segment SGI111971[] = {{&S7108[31], 3}, {NULL}};
const Segment SGI111972[] = {{&S7108[75], 6}, {NULL}};
const Segment SGI111973[] = {{&S89595[44], 8}, {NULL}};
const Segment SGI111974[] = {{&S89595[44], 20}, {NULL}};
const Segment SGI111975[] = {{&S89595[30], 6}, {NULL}};
const Segment SGI111976[] = {{&S7109[44], 13}, {NULL}};
const Segment SGI111977[] = {{&S7109[44], 25}, {NULL}};
const Segment SGI111978[] = {{&S89595[31], 3}, {NULL}};
const Segment SGI111979[] = {{&S7109[30], 6}, {NULL}};
const Segment SGI111980[] = {{&S89595[78], 6}, {NULL}};
const Segment SGI111981[] = {{&S7109[31], 3}, {NULL}};
const Segment SGI111982[] = {{&S7109[83], 6}, {NULL}};
const Segment SGI111983[] = {{&S7111[44], 7}, {NULL}};
const Segment SGI111984[] = {{&S7111[44], 19}, {NULL}};
const Segment SGI111985[] = {{&S7111[30], 6}, {NULL}};
const Segment SGI111986[] = {{&S7111[31], 3}, {NULL}};
const Segment SGI111987[] = {{&S7111[77], 6}, {NULL}};
const Segment SGI111988[] = {{&S89599[59], 11}, {NULL}};
const Segment SGI111989[] = {{&S89599[59], 23}, {NULL}};
const Segment SGI111990[] = {{&S89599[45], 6}, {NULL}};
const Segment SGI111991[] = {{&S89599[46], 3}, {NULL}};
const Segment SGI111992[] = {{&S7112[44], 22}, {NULL}};
const Segment SGI111993[] = {{&S89599[96], 6}, {NULL}};
const Segment SGI111994[] = {{&S7112[44], 34}, {NULL}};
const Segment SGI111995[] = {{&S7112[30], 6}, {NULL}};
const Segment SGI111996[] = {{&S89599[0], 14}, {NULL}};
const Segment SGI111997[] = {{&S7112[31], 3}, {NULL}};
const Segment SGI111998[] = {{&S7112[92], 6}, {NULL}};
const Segment SGI111999[] = {{&S7114[44], 7}, {NULL}};
const Segment SGI112000[] = {{&S7114[44], 19}, {NULL}};
const Segment SGI112001[] = {{&S7114[30], 6}, {NULL}};
const Segment SGI112002[] = {{&S7114[31], 3}, {NULL}};
const Segment SGI112003[] = {{&S7114[77], 6}, {NULL}};
const Segment SGI112004[] = {{&S7115[44], 10}, {NULL}};
const Segment SGI112005[] = {{&S7115[44], 22}, {NULL}};
const Segment SGI112006[] = {{&S7115[30], 6}, {NULL}};
const Segment SGI112007[] = {{&S7115[31], 3}, {NULL}};
const Segment SGI112008[] = {{&S7115[80], 6}, {NULL}};
const Segment SGI112009[] = {{&S7116[44], 7}, {NULL}};
const Segment SGI112010[] = {{&S7116[44], 19}, {NULL}};
const Segment SGI112011[] = {{&S7116[30], 6}, {NULL}};
const Segment SGI112012[] = {{&S7116[31], 3}, {NULL}};
const Segment SGI112013[] = {{&S7116[77], 6}, {NULL}};
const Segment SGI112014[] = {{&S7117[44], 31}, {NULL}};
const Segment SGI112015[] = {{&S7117[44], 43}, {NULL}};
const Segment SGI112016[] = {{&S7117[30], 6}, {NULL}};
const Segment SGI112017[] = {{&S7117[31], 3}, {NULL}};
const Segment SGI112018[] = {{&S7117[101], 6}, {NULL}};
const Segment SGI112019[] = {{&S89610[44], 12}, {NULL}};
const Segment SGI112020[] = {{&S89610[44], 24}, {NULL}};
const Segment SGI112021[] = {{&S89610[30], 6}, {NULL}};
const Segment SGI112022[] = {{&S7118[44], 5}, {NULL}};
const Segment SGI112023[] = {{&S7118[44], 27}, {NULL}};
const Segment SGI112024[] = {{&S89610[31], 3}, {NULL}};
const Segment SGI112025[] = {{&S7118[30], 6}, {NULL}};
const Segment SGI112026[] = {{&S89610[82], 6}, {NULL}};
const Segment SGI112027[] = {{&S7118[31], 3}, {NULL}};
const Segment SGI112028[] = {{&S7118[85], 6}, {NULL}};
const Segment SGI112029[] = {{&S7120[44], 15}, {NULL}};
const Segment SGI112030[] = {{&S7120[44], 27}, {NULL}};
const Segment SGI112031[] = {{&S7120[30], 6}, {NULL}};
const Segment SGI112032[] = {{&S7120[31], 3}, {NULL}};
const Segment SGI112033[] = {{&S7120[85], 6}, {NULL}};
const Segment SGI112034[] = {{&S7121[44], 9}, {NULL}};
const Segment SGI112035[] = {{&S7121[44], 21}, {NULL}};
const Segment SGI112036[] = {{&S7121[30], 6}, {NULL}};
const Segment SGI112037[] = {{&S7121[31], 3}, {NULL}};
const Segment SGI112038[] = {{&S7121[79], 6}, {NULL}};
const Segment SGI112039[] = {{&S7122[44], 6}, {NULL}};
const Segment SGI112040[] = {{&S7122[44], 18}, {NULL}};
const Segment SGI112041[] = {{&S7122[30], 6}, {NULL}};
const Segment SGI112042[] = {{&S7122[31], 3}, {NULL}};
const Segment SGI112043[] = {{&S7122[76], 6}, {NULL}};
const Segment SGI112044[] = {{&S7123[44], 9}, {NULL}};
const Segment SGI112045[] = {{&S7123[44], 21}, {NULL}};
const Segment SGI112046[] = {{&S7123[30], 6}, {NULL}};
const Segment SGI112047[] = {{&S7123[31], 3}, {NULL}};
const Segment SGI112048[] = {{&S7123[79], 6}, {NULL}};
const Segment SGI112049[] = {{&S7124[44], 19}, {NULL}};
const Segment SGI112050[] = {{&S7124[44], 31}, {NULL}};
const Segment SGI112051[] = {{&S7124[30], 6}, {NULL}};
const Segment SGI112052[] = {{&S7124[31], 3}, {NULL}};
const Segment SGI112053[] = {{&S7124[89], 6}, {NULL}};
const Segment SGI112054[] = {{&S7126[44], 7}, {NULL}};
const Segment SGI112055[] = {{&S7126[44], 19}, {NULL}};
const Segment SGI112056[] = {{&S7126[30], 6}, {NULL}};
const Segment SGI112057[] = {{&S7126[31], 3}, {NULL}};
const Segment SGI112058[] = {{&S7126[77], 6}, {NULL}};
const Segment SGI112059[] = {{&S7128[44], 9}, {NULL}};
const Segment SGI112060[] = {{&S7128[44], 21}, {NULL}};
const Segment SGI112061[] = {{&S7128[30], 6}, {NULL}};
const Segment SGI112062[] = {{&S7128[31], 3}, {NULL}};
const Segment SGI112063[] = {{&S7128[79], 6}, {NULL}};
const Segment SGI112064[] = {{&S7129[44], 9}, {NULL}};
const Segment SGI112065[] = {{&S7129[44], 21}, {NULL}};
const Segment SGI112066[] = {{&S7129[30], 6}, {NULL}};
const Segment SGI112067[] = {{&S7129[31], 3}, {NULL}};
const Segment SGI112068[] = {{&S7129[79], 6}, {NULL}};
const Segment SGI112069[] = {{&S89632[45], 9}, {NULL}};
const Segment SGI112070[] = {{&S89632[30], 6}, {NULL}};
const Segment SGI112071[] = {{&S89632[31], 3}, {NULL}};
const Segment SGI112072[] = {{&S89632[80], 5}, {NULL}};
const Segment SGI112073[] = {{&S89632[138], 13}, {NULL}};
const Segment SGI112074[] = {{&S7131[44], 12}, {NULL}};
const Segment SGI112075[] = {{&S7131[44], 24}, {NULL}};
const Segment SGI112076[] = {{&S7131[30], 6}, {NULL}};
const Segment SGI112077[] = {{&S7131[31], 3}, {NULL}};
const Segment SGI112078[] = {{&S7131[82], 6}, {NULL}};
const Segment SGI112079[] = {{&S7132[44], 8}, {NULL}};
const Segment SGI112080[] = {{&S7132[44], 17}, {NULL}};
const Segment SGI112081[] = {{&S7132[30], 6}, {NULL}};
const Segment SGI112082[] = {{&S7132[31], 3}, {NULL}};
const Segment SGI112083[] = {{&S7132[75], 6}, {NULL}};
const Segment SGI112084[] = {{&S7134[44], 7}, {NULL}};
const Segment SGI112085[] = {{&S7134[44], 19}, {NULL}};
const Segment SGI112086[] = {{&S7134[30], 6}, {NULL}};
const Segment SGI112087[] = {{&S7134[31], 3}, {NULL}};
const Segment SGI112088[] = {{&S7134[77], 6}, {NULL}};
const Segment SGI112089[] = {{&S89641[34], 15}, {NULL}};
const Segment SGI112090[] = {{&S89641[27], 6}, {NULL}};
const Segment SGI112091[] = {{&S7135[44], 8}, {NULL}};
const Segment SGI112092[] = {{&S7135[44], 20}, {NULL}};
const Segment SGI112093[] = {{&S7135[30], 6}, {NULL}};
const Segment SGI112094[] = {{&S89641[102], 9}, {NULL}};
const Segment SGI112095[] = {{&S89641[75], 6}, {NULL}};
const Segment SGI112096[] = {{&S7135[31], 3}, {NULL}};
const Segment SGI112097[] = {{&S7135[78], 6}, {NULL}};
const Segment SGI112098[] = {{&S7136[44], 4}, {NULL}};
const Segment SGI112099[] = {{&S7136[44], 16}, {NULL}};
const Segment SGI112100[] = {{&S7136[30], 6}, {NULL}};
const Segment SGI112101[] = {{&S7136[31], 3}, {NULL}};
const Segment SGI112102[] = {{&S7136[74], 6}, {NULL}};
const Segment SGI112103[] = {{&S7138[44], 7}, {NULL}};
const Segment SGI112104[] = {{&S7138[44], 19}, {NULL}};
const Segment SGI112105[] = {{&S7138[30], 6}, {NULL}};
const Segment SGI112106[] = {{&S7138[31], 3}, {NULL}};
const Segment SGI112107[] = {{&S7138[77], 6}, {NULL}};
const Segment SGI112108[] = {{&S104863[42], 10}, {NULL}};
const Segment SGI112109[] = {{&S104863[42], 22}, {NULL}};
const Segment SGI112110[] = {{&S104863[30], 4}, {NULL}};
const Segment SGI112111[] = {{&S104863[113], 3}, {NULL}};
const Segment SGI112112[] = {{&S104863[78], 6}, {NULL}};
const Segment SGI112113[] = {{&S7143[44], 6}, {NULL}};
const Segment SGI112114[] = {{&S7143[44], 18}, {NULL}};
const Segment SGI112115[] = {{&S7143[30], 6}, {NULL}};
const Segment SGI112116[] = {{&S7143[31], 3}, {NULL}};
const Segment SGI112117[] = {{&S7143[76], 6}, {NULL}};
const Segment SGI112118[] = {{&S7145[44], 15}, {NULL}};
const Segment SGI112119[] = {{&S7145[44], 27}, {NULL}};
const Segment SGI112120[] = {{&S104866[42], 10}, {NULL}};
const Segment SGI112121[] = {{&S7145[30], 6}, {NULL}};
const Segment SGI112122[] = {{&S104866[42], 22}, {NULL}};
const Segment SGI112123[] = {{&S104866[30], 4}, {NULL}};
const Segment SGI112124[] = {{&S7145[31], 3}, {NULL}};
const Segment SGI112125[] = {{&S7145[85], 6}, {NULL}};
const Segment SGI112126[] = {{&S104866[113], 3}, {NULL}};
const Segment SGI112127[] = {{&S104866[78], 6}, {NULL}};
const Segment SGI112128[] = {{&S104866[131], 13}, {NULL}};
const Segment SGI112129[] = {{&S7146[44], 11}, {NULL}};
const Segment SGI112130[] = {{&S7146[44], 23}, {NULL}};
const Segment SGI112131[] = {{&S7146[30], 6}, {NULL}};
const Segment SGI112132[] = {{&S7146[31], 3}, {NULL}};
const Segment SGI112133[] = {{&S7146[81], 6}, {NULL}};
const Segment SGI112134[] = {{&S104868[29], 8}, {NULL}};
const Segment SGI112135[] = {{&S104868[53], 3}, {NULL}};
const Segment SGI112136[] = {{&S7147[44], 13}, {NULL}};
const Segment SGI112137[] = {{&S7147[44], 25}, {NULL}};
const Segment SGI112138[] = {{&S7147[30], 6}, {NULL}};
const Segment SGI112139[] = {{&S7147[31], 3}, {NULL}};
const Segment SGI112140[] = {{&S7147[83], 6}, {NULL}};
const Segment SGI112141[] = {{&S104869[55], 10}, {NULL}};
const Segment SGI112142[] = {{&S104869[55], 22}, {NULL}};
const Segment SGI112143[] = {{&S104869[43], 4}, {NULL}};
const Segment SGI112144[] = {{&S104869[126], 3}, {NULL}};
const Segment SGI112145[] = {{&S104869[91], 6}, {NULL}};
const Segment SGI112146[] = {{&S104869[0], 12}, {NULL}};
const Segment SGI112147[] = {{&S7149[38], 8}, {NULL}};
const Segment SGI112148[] = {{&S104870[57], 10}, {NULL}};
const Segment SGI112149[] = {{&S7149[31], 6}, {NULL}};
const Segment SGI112150[] = {{&S104870[57], 22}, {NULL}};
const Segment SGI112151[] = {{&S104870[45], 4}, {NULL}};
const Segment SGI112152[] = {{&S104870[128], 3}, {NULL}};
const Segment SGI112153[] = {{&S104870[93], 6}, {NULL}};
const Segment SGI112154[] = {{&S104870[0], 14}, {NULL}};
const Segment SGI112155[] = {{&S7150[44], 6}, {NULL}};
const Segment SGI112156[] = {{&S7150[44], 18}, {NULL}};
const Segment SGI112157[] = {{&S7150[30], 6}, {NULL}};
const Segment SGI112158[] = {{&S7150[31], 3}, {NULL}};
const Segment SGI112159[] = {{&S104871[43], 10}, {NULL}};
const Segment SGI112160[] = {{&S7150[76], 6}, {NULL}};
const Segment SGI112161[] = {{&S104871[30], 4}, {NULL}};
const Segment SGI112162[] = {{&S104871[113], 3}, {NULL}};
const Segment SGI112163[] = {{&S104871[79], 5}, {NULL}};
const Segment SGI112164[] = {{&S104871[137], 13}, {NULL}};
const Segment SGI112165[] = {{&S7151[44], 8}, {NULL}};
const Segment SGI112166[] = {{&S7151[44], 20}, {NULL}};
const Segment SGI112167[] = {{&S7151[30], 6}, {NULL}};
const Segment SGI112168[] = {{&S7151[31], 3}, {NULL}};
const Segment SGI112169[] = {{&S7151[78], 6}, {NULL}};
const Segment SGI112170[] = {{&S104874[42], 10}, {NULL}};
const Segment SGI112171[] = {{&S104874[42], 22}, {NULL}};
const Segment SGI112172[] = {{&S104874[30], 4}, {NULL}};
const Segment SGI112173[] = {{&S104874[120], 3}, {NULL}};
const Segment SGI112174[] = {{&S104874[85], 6}, {NULL}};
const Segment SGI112175[] = {{&S104874[64], 8}, {NULL}};
const Segment SGI112176[] = {{&S7154[44], 11}, {NULL}};
const Segment SGI112177[] = {{&S7154[44], 23}, {NULL}};
const Segment SGI112178[] = {{&S7154[30], 6}, {NULL}};
const Segment SGI112179[] = {{&S7154[31], 3}, {NULL}};
const Segment SGI112180[] = {{&S7154[81], 6}, {NULL}};
const Segment SGI112181[] = {{&S104875[55], 10}, {NULL}};
const Segment SGI112182[] = {{&S104875[55], 21}, {NULL}};
const Segment SGI112183[] = {{&S104875[43], 4}, {NULL}};
const Segment SGI112184[] = {{&S104875[125], 3}, {NULL}};
const Segment SGI112185[] = {{&S104875[90], 6}, {NULL}};
const Segment SGI112186[] = {{&S104875[0], 12}, {NULL}};
const Segment SGI112187[] = {{&S104876[60], 10}, {NULL}};
const Segment SGI112188[] = {{&S7156[44], 22}, {NULL}};
const Segment SGI112189[] = {{&S104876[55], 4}, {NULL}};
const Segment SGI112190[] = {{&S7156[30], 6}, {NULL}};
const Segment SGI112191[] = {{&S7156[31], 3}, {NULL}};
const Segment SGI112192[] = {{&S7156[100], 6}, {NULL}};
const Segment SGI112193[] = {{&S7158[44], 9}, {NULL}};
const Segment SGI112194[] = {{&S7158[44], 21}, {NULL}};
const Segment SGI112195[] = {{&S7158[30], 6}, {NULL}};
const Segment SGI112196[] = {{&S7158[31], 3}, {NULL}};
const Segment SGI112197[] = {{&S7158[79], 6}, {NULL}};
const Segment SGI112198[] = {{&S104879[55], 10}, {NULL}};
const Segment SGI112199[] = {{&S104879[55], 22}, {NULL}};
const Segment SGI112200[] = {{&S104879[43], 4}, {NULL}};
const Segment SGI112201[] = {{&S104879[126], 3}, {NULL}};
const Segment SGI112202[] = {{&S104879[91], 6}, {NULL}};
const Segment SGI112203[] = {{&S104879[0], 12}, {NULL}};
const Segment SGI112204[] = {{&S7159[44], 10}, {NULL}};
const Segment SGI112205[] = {{&S7159[44], 22}, {NULL}};
const Segment SGI112206[] = {{&S7159[30], 6}, {NULL}};
const Segment SGI112207[] = {{&S7159[31], 3}, {NULL}};
const Segment SGI112208[] = {{&S7159[80], 6}, {NULL}};
const Segment SGI112209[] = {{&S7161[44], 14}, {NULL}};
const Segment SGI112210[] = {{&S7161[44], 26}, {NULL}};
const Segment SGI112211[] = {{&S7161[30], 6}, {NULL}};
const Segment SGI112212[] = {{&S7161[31], 3}, {NULL}};
const Segment SGI112213[] = {{&S7161[84], 6}, {NULL}};
const Segment SGI112214[] = {{&S7162[44], 16}, {NULL}};
const Segment SGI112215[] = {{&S7162[44], 28}, {NULL}};
const Segment SGI112216[] = {{&S7162[30], 6}, {NULL}};
const Segment SGI112217[] = {{&S7162[31], 3}, {NULL}};
const Segment SGI112218[] = {{&S7162[86], 6}, {NULL}};
const Segment SGI112219[] = {{&S104886[43], 10}, {NULL}};
const Segment SGI112220[] = {{&S104886[30], 4}, {NULL}};
const Segment SGI112221[] = {{&S7163[44], 8}, {NULL}};
const Segment SGI112222[] = {{&S7163[44], 20}, {NULL}};
const Segment SGI112223[] = {{&S7163[30], 6}, {NULL}};
const Segment SGI112224[] = {{&S104886[112], 3}, {NULL}};
const Segment SGI112225[] = {{&S104886[78], 5}, {NULL}};
const Segment SGI112226[] = {{&S7163[31], 3}, {NULL}};
const Segment SGI112227[] = {{&S104886[136], 13}, {NULL}};
const Segment SGI112228[] = {{&S7163[78], 6}, {NULL}};
const Segment SGI112229[] = {{&S7164[44], 9}, {NULL}};
const Segment SGI112230[] = {{&S7164[44], 21}, {NULL}};
const Segment SGI112231[] = {{&S7164[30], 6}, {NULL}};
const Segment SGI112232[] = {{&S7164[31], 3}, {NULL}};
const Segment SGI112233[] = {{&S7164[79], 6}, {NULL}};
const Segment SGI112234[] = {{&S7165[44], 7}, {NULL}};
const Segment SGI112235[] = {{&S7165[44], 19}, {NULL}};
const Segment SGI112236[] = {{&S7165[30], 6}, {NULL}};
const Segment SGI112237[] = {{&S7165[31], 3}, {NULL}};
const Segment SGI112238[] = {{&S7165[77], 6}, {NULL}};
const Segment SGI112239[] = {{&S7166[44], 5}, {NULL}};
const Segment SGI112240[] = {{&S7166[44], 17}, {NULL}};
const Segment SGI112241[] = {{&S7166[30], 6}, {NULL}};
const Segment SGI112242[] = {{&S7166[31], 3}, {NULL}};
const Segment SGI112243[] = {{&S7166[75], 6}, {NULL}};
const Segment SGI112244[] = {{&S7168[44], 12}, {NULL}};
const Segment SGI112245[] = {{&S7168[44], 24}, {NULL}};
const Segment SGI112246[] = {{&S7168[30], 6}, {NULL}};
const Segment SGI112247[] = {{&S7168[31], 3}, {NULL}};
const Segment SGI112248[] = {{&S7168[82], 6}, {NULL}};
const Segment SGI112249[] = {{&S7169[44], 12}, {NULL}};
const Segment SGI112250[] = {{&S7169[44], 24}, {NULL}};
const Segment SGI112251[] = {{&S7169[30], 6}, {NULL}};
const Segment SGI112252[] = {{&S7169[31], 3}, {NULL}};
const Segment SGI112253[] = {{&S7169[82], 6}, {NULL}};
const Segment SGI112254[] = {{&S7170[44], 11}, {NULL}};
const Segment SGI112255[] = {{&S7170[44], 23}, {NULL}};
const Segment SGI112256[] = {{&S7170[30], 6}, {NULL}};
const Segment SGI112257[] = {{&S7170[31], 3}, {NULL}};
const Segment SGI112258[] = {{&S7170[81], 6}, {NULL}};
const Segment SGI112259[] = {{&S7171[44], 14}, {NULL}};
const Segment SGI112260[] = {{&S7171[44], 26}, {NULL}};
const Segment SGI112261[] = {{&S7171[30], 6}, {NULL}};
const Segment SGI112262[] = {{&S7171[31], 3}, {NULL}};
const Segment SGI112263[] = {{&S7171[84], 6}, {NULL}};
const Segment SGI112264[] = {{&S7172[44], 12}, {NULL}};
const Segment SGI112265[] = {{&S7172[44], 24}, {NULL}};
const Segment SGI112266[] = {{&S7172[30], 6}, {NULL}};
const Segment SGI112267[] = {{&S7172[31], 3}, {NULL}};
const Segment SGI112268[] = {{&S7172[82], 6}, {NULL}};
const Segment SGI112269[] = {{&S104903[43], 10}, {NULL}};
const Segment SGI112270[] = {{&S104903[30], 4}, {NULL}};
const Segment SGI112271[] = {{&S104903[113], 3}, {NULL}};
const Segment SGI112272[] = {{&S104903[79], 5}, {NULL}};
const Segment SGI112273[] = {{&S104903[137], 13}, {NULL}};
const Segment SGI112274[] = {{&S7173[44], 7}, {NULL}};
const Segment SGI112275[] = {{&S7173[44], 19}, {NULL}};
const Segment SGI112276[] = {{&S7173[30], 6}, {NULL}};
const Segment SGI112277[] = {{&S7173[31], 3}, {NULL}};
const Segment SGI112278[] = {{&S7173[77], 6}, {NULL}};
const Segment SGI112279[] = {{&S7174[44], 22}, {NULL}};
const Segment SGI112280[] = {{&S7174[44], 34}, {NULL}};
const Segment SGI112281[] = {{&S7174[30], 6}, {NULL}};
const Segment SGI112282[] = {{&S7174[31], 3}, {NULL}};
const Segment SGI112283[] = {{&S7174[92], 6}, {NULL}};
const Segment SGI112284[] = {{&S7175[44], 6}, {NULL}};
const Segment SGI112285[] = {{&S7175[44], 18}, {NULL}};
const Segment SGI112286[] = {{&S7175[30], 6}, {NULL}};
const Segment SGI112287[] = {{&S7175[31], 3}, {NULL}};
const Segment SGI112288[] = {{&S7175[76], 6}, {NULL}};
const Segment SGI112289[] = {{&S104908[42], 10}, {NULL}};
const Segment SGI112290[] = {{&S104908[42], 22}, {NULL}};
const Segment SGI112291[] = {{&S104908[30], 4}, {NULL}};
const Segment SGI112292[] = {{&S104908[113], 3}, {NULL}};
const Segment SGI112293[] = {{&S104908[78], 6}, {NULL}};
const Segment SGI112294[] = {{&S104908[131], 11}, {NULL}};
const Segment SGI112295[] = {{&S7176[44], 10}, {NULL}};
const Segment SGI112296[] = {{&S7176[44], 22}, {NULL}};
const Segment SGI112297[] = {{&S7176[30], 6}, {NULL}};
const Segment SGI112298[] = {{&S7176[31], 3}, {NULL}};
const Segment SGI112299[] = {{&S7176[80], 6}, {NULL}};
const Segment SGI112300[] = {{&S104909[32], 10}, {NULL}};
const Segment SGI112301[] = {{&S104909[27], 4}, {NULL}};
const Segment SGI112302[] = {{&S7177[44], 9}, {NULL}};
const Segment SGI112303[] = {{&S104909[103], 2}, {NULL}};
const Segment SGI112304[] = {{&S7177[44], 21}, {NULL}};
const Segment SGI112305[] = {{&S7177[30], 6}, {NULL}};
const Segment SGI112306[] = {{&S104909[69], 6}, {NULL}};
const Segment SGI112307[] = {{&S7177[31], 3}, {NULL}};
const Segment SGI112308[] = {{&S7177[79], 6}, {NULL}};
const Segment SGI112309[] = {{&S7178[44], 5}, {NULL}};
const Segment SGI112310[] = {{&S7178[44], 17}, {NULL}};
const Segment SGI112311[] = {{&S7178[30], 6}, {NULL}};
const Segment SGI112312[] = {{&S7178[31], 3}, {NULL}};
const Segment SGI112313[] = {{&S7178[75], 6}, {NULL}};
const Segment SGI112314[] = {{&S7179[44], 8}, {NULL}};
const Segment SGI112315[] = {{&S7179[44], 20}, {NULL}};
const Segment SGI112316[] = {{&S7179[30], 6}, {NULL}};
const Segment SGI112317[] = {{&S7179[31], 3}, {NULL}};
const Segment SGI112318[] = {{&S7179[78], 6}, {NULL}};
const Segment SGI112319[] = {{&S104913[42], 10}, {NULL}};
const Segment SGI112320[] = {{&S104913[42], 21}, {NULL}};
const Segment SGI112321[] = {{&S104913[30], 4}, {NULL}};
const Segment SGI112322[] = {{&S104913[156], 3}, {NULL}};
const Segment SGI112323[] = {{&S104913[31], 3}, {NULL}};
const Segment SGI112324[] = {{&S7180[44], 8}, {NULL}};
const Segment SGI112325[] = {{&S7180[44], 20}, {NULL}};
const Segment SGI112326[] = {{&S7180[30], 6}, {NULL}};
const Segment SGI112327[] = {{&S104913[87], 6}, {NULL}};
const Segment SGI112328[] = {{&S104913[142], 8}, {NULL}};
const Segment SGI112329[] = {{&S7180[31], 3}, {NULL}};
const Segment SGI112330[] = {{&S7180[78], 6}, {NULL}};
const Segment SGI112331[] = {{&S7181[44], 11}, {NULL}};
const Segment SGI112332[] = {{&S7181[44], 23}, {NULL}};
const Segment SGI112333[] = {{&S7181[30], 6}, {NULL}};
const Segment SGI112334[] = {{&S7181[31], 3}, {NULL}};
const Segment SGI112335[] = {{&S7181[81], 6}, {NULL}};
const Segment SGI112336[] = {{&S7182[44], 11}, {NULL}};
const Segment SGI112337[] = {{&S7182[44], 23}, {NULL}};
const Segment SGI112338[] = {{&S7182[30], 6}, {NULL}};
const Segment SGI112339[] = {{&S7182[31], 3}, {NULL}};
const Segment SGI112340[] = {{&S7182[81], 6}, {NULL}};
const Segment SGI112341[] = {{&S89740[59], 13}, {NULL}};
const Segment SGI112342[] = {{&S89740[59], 29}, {NULL}};
const Segment SGI112343[] = {{&S89740[45], 6}, {NULL}};
const Segment SGI112344[] = {{&S89740[46], 3}, {NULL}};
const Segment SGI112345[] = {{&S89740[102], 6}, {NULL}};
const Segment SGI112346[] = {{&S89740[0], 14}, {NULL}};
const Segment SGI112347[] = {{&S7184[44], 7}, {NULL}};
const Segment SGI112348[] = {{&S7184[44], 19}, {NULL}};
const Segment SGI112349[] = {{&S7184[30], 6}, {NULL}};
const Segment SGI112350[] = {{&S7184[31], 3}, {NULL}};
const Segment SGI112351[] = {{&S7184[77], 6}, {NULL}};
const Segment SGI112352[] = {{&S89742[34], 7}, {NULL}};
const Segment SGI112353[] = {{&S89742[27], 6}, {NULL}};
const Segment SGI112354[] = {{&S89742[94], 9}, {NULL}};
const Segment SGI112355[] = {{&S89742[67], 6}, {NULL}};
const Segment SGI112356[] = {{&S89747[34], 7}, {NULL}};
const Segment SGI112357[] = {{&S89747[27], 6}, {NULL}};
const Segment SGI112358[] = {{&S89747[95], 9}, {NULL}};
const Segment SGI112359[] = {{&S89747[68], 6}, {NULL}};
const Segment SGI112360[] = {{&S7191[44], 8}, {NULL}};
const Segment SGI112361[] = {{&S7191[44], 20}, {NULL}};
const Segment SGI112362[] = {{&S7191[30], 6}, {NULL}};
const Segment SGI112363[] = {{&S7191[31], 3}, {NULL}};
const Segment SGI112364[] = {{&S7191[78], 6}, {NULL}};
const Segment SGI112365[] = {{&S104926[42], 11}, {NULL}};
const Segment SGI112366[] = {{&S104926[42], 23}, {NULL}};
const Segment SGI112367[] = {{&S104926[30], 4}, {NULL}};
const Segment SGI112368[] = {{&S104926[114], 3}, {NULL}};
const Segment SGI112369[] = {{&S104926[79], 6}, {NULL}};
const Segment SGI112370[] = {{&S104926[132], 13}, {NULL}};
const Segment SGI112371[] = {{&S7193[44], 11}, {NULL}};
const Segment SGI112372[] = {{&S7193[44], 23}, {NULL}};
const Segment SGI112373[] = {{&S7193[30], 6}, {NULL}};
const Segment SGI112374[] = {{&S104927[42], 10}, {NULL}};
const Segment SGI112375[] = {{&S7193[31], 3}, {NULL}};
const Segment SGI112376[] = {{&S104927[42], 22}, {NULL}};
const Segment SGI112377[] = {{&S104927[30], 4}, {NULL}};
const Segment SGI112378[] = {{&S7193[81], 6}, {NULL}};
const Segment SGI112379[] = {{&S104927[113], 3}, {NULL}};
const Segment SGI112380[] = {{&S104927[78], 6}, {NULL}};
const Segment SGI112381[] = {{&S104927[131], 13}, {NULL}};
const Segment SGI112382[] = {{&S7195[44], 14}, {NULL}};
const Segment SGI112383[] = {{&S7195[44], 23}, {NULL}};
const Segment SGI112384[] = {{&S7195[30], 6}, {NULL}};
const Segment SGI112385[] = {{&S7195[31], 3}, {NULL}};
const Segment SGI112386[] = {{&S7195[81], 6}, {NULL}};
const Segment SGI112387[] = {{&S7196[44], 7}, {NULL}};
const Segment SGI112388[] = {{&S7196[44], 19}, {NULL}};
const Segment SGI112389[] = {{&S7196[30], 6}, {NULL}};
const Segment SGI112390[] = {{&S7196[31], 3}, {NULL}};
const Segment SGI112391[] = {{&S7196[77], 6}, {NULL}};
const Segment SGI112392[] = {{&S104931[42], 10}, {NULL}};
const Segment SGI112393[] = {{&S104931[42], 22}, {NULL}};
const Segment SGI112394[] = {{&S104931[30], 4}, {NULL}};
const Segment SGI112395[] = {{&S104931[112], 3}, {NULL}};
const Segment SGI112396[] = {{&S104931[78], 5}, {NULL}};
const Segment SGI112397[] = {{&S104931[123], 14}, {NULL}};
const Segment SGI112398[] = {{&S7199[44], 10}, {NULL}};
const Segment SGI112399[] = {{&S7199[44], 22}, {NULL}};
const Segment SGI112400[] = {{&S7199[30], 6}, {NULL}};
const Segment SGI112401[] = {{&S7199[31], 3}, {NULL}};
const Segment SGI112402[] = {{&S7199[80], 6}, {NULL}};
const Segment SGI112403[] = {{&S7201[44], 11}, {NULL}};
const Segment SGI112404[] = {{&S7201[44], 23}, {NULL}};
const Segment SGI112405[] = {{&S7201[30], 6}, {NULL}};
const Segment SGI112406[] = {{&S7201[31], 3}, {NULL}};
const Segment SGI112407[] = {{&S7201[81], 6}, {NULL}};
const Segment SGI112408[] = {{&S7202[44], 12}, {NULL}};
const Segment SGI112409[] = {{&S7202[44], 24}, {NULL}};
const Segment SGI112410[] = {{&S7202[30], 6}, {NULL}};
const Segment SGI112411[] = {{&S7202[85], 6}, {NULL}};
const Segment SGI112412[] = {{&S104938[43], 10}, {NULL}};
const Segment SGI112413[] = {{&S104938[30], 4}, {NULL}};
const Segment SGI112414[] = {{&S104938[113], 3}, {NULL}};
const Segment SGI112415[] = {{&S104938[79], 5}, {NULL}};
const Segment SGI112416[] = {{&S104938[137], 13}, {NULL}};
const Segment SGI112417[] = {{&S7205[44], 6}, {NULL}};
const Segment SGI112418[] = {{&S7205[44], 18}, {NULL}};
const Segment SGI112419[] = {{&S7205[30], 6}, {NULL}};
const Segment SGI112420[] = {{&S104940[42], 10}, {NULL}};
const Segment SGI112421[] = {{&S7205[31], 3}, {NULL}};
const Segment SGI112422[] = {{&S104940[42], 22}, {NULL}};
const Segment SGI112423[] = {{&S104940[30], 4}, {NULL}};
const Segment SGI112424[] = {{&S7205[76], 6}, {NULL}};
const Segment SGI112425[] = {{&S104940[113], 3}, {NULL}};
const Segment SGI112426[] = {{&S104940[78], 6}, {NULL}};
const Segment SGI112427[] = {{&S104940[131], 11}, {NULL}};
const Segment SGI112428[] = {{&S7206[44], 8}, {NULL}};
const Segment SGI112429[] = {{&S7206[44], 20}, {NULL}};
const Segment SGI112430[] = {{&S104941[29], 8}, {NULL}};
const Segment SGI112431[] = {{&S7206[30], 6}, {NULL}};
const Segment SGI112432[] = {{&S104941[53], 3}, {NULL}};
const Segment SGI112433[] = {{&S7206[31], 3}, {NULL}};
const Segment SGI112434[] = {{&S7206[78], 6}, {NULL}};
const Segment SGI112435[] = {{&S104942[37], 8}, {NULL}};
const Segment SGI112436[] = {{&S7207[44], 17}, {NULL}};
const Segment SGI112437[] = {{&S104942[102], 3}, {NULL}};
const Segment SGI112438[] = {{&S7207[30], 6}, {NULL}};
const Segment SGI112439[] = {{&S104942[91], 3}, {NULL}};
const Segment SGI112440[] = {{&S7207[31], 3}, {NULL}};
const Segment SGI112441[] = {{&S104942[68], 6}, {NULL}};
const Segment SGI112442[] = {{&S7207[75], 6}, {NULL}};
const Segment SGI112443[] = {{&S7208[44], 16}, {NULL}};
const Segment SGI112444[] = {{&S7208[44], 28}, {NULL}};
const Segment SGI112445[] = {{&S7208[30], 6}, {NULL}};
const Segment SGI112446[] = {{&S7208[31], 3}, {NULL}};
const Segment SGI112447[] = {{&S7208[86], 6}, {NULL}};
const Segment SGI112448[] = {{&S104944[43], 10}, {NULL}};
const Segment SGI112449[] = {{&S104944[30], 4}, {NULL}};
const Segment SGI112450[] = {{&S104944[113], 3}, {NULL}};
const Segment SGI112451[] = {{&S104944[79], 5}, {NULL}};
const Segment SGI112452[] = {{&S104944[137], 13}, {NULL}};
const Segment SGI112453[] = {{&S7210[44], 16}, {NULL}};
const Segment SGI112454[] = {{&S7210[44], 28}, {NULL}};
const Segment SGI112455[] = {{&S7210[30], 6}, {NULL}};
const Segment SGI112456[] = {{&S7210[31], 3}, {NULL}};
const Segment SGI112457[] = {{&S7210[86], 6}, {NULL}};
const Segment SGI112458[] = {{&S7211[44], 7}, {NULL}};
const Segment SGI112459[] = {{&S7211[44], 19}, {NULL}};
const Segment SGI112460[] = {{&S7211[30], 6}, {NULL}};
const Segment SGI112461[] = {{&S7211[31], 3}, {NULL}};
const Segment SGI112462[] = {{&S7211[77], 6}, {NULL}};
const Segment SGI112463[] = {{&S89781[34], 7}, {NULL}};
const Segment SGI112464[] = {{&S89781[27], 6}, {NULL}};
const Segment SGI112465[] = {{&S89781[94], 9}, {NULL}};
const Segment SGI112466[] = {{&S89781[67], 6}, {NULL}};
const Segment SGI112467[] = {{&S7212[44], 10}, {NULL}};
const Segment SGI112468[] = {{&S7212[44], 22}, {NULL}};
const Segment SGI112469[] = {{&S7212[30], 6}, {NULL}};
const Segment SGI112470[] = {{&S7212[31], 3}, {NULL}};
const Segment SGI112471[] = {{&S7212[80], 6}, {NULL}};
const Segment SGI112472[] = {{&S7213[44], 11}, {NULL}};
const Segment SGI112473[] = {{&S7213[44], 23}, {NULL}};
const Segment SGI112474[] = {{&S7213[30], 6}, {NULL}};
const Segment SGI112475[] = {{&S7213[31], 3}, {NULL}};
const Segment SGI112476[] = {{&S7213[81], 6}, {NULL}};
const Segment SGI112477[] = {{&S104951[57], 10}, {NULL}};
const Segment SGI112478[] = {{&S104951[57], 21}, {NULL}};
const Segment SGI112479[] = {{&S104951[45], 4}, {NULL}};
const Segment SGI112480[] = {{&S104951[127], 3}, {NULL}};
const Segment SGI112481[] = {{&S7214[44], 14}, {NULL}};
const Segment SGI112482[] = {{&S104951[92], 6}, {NULL}};
const Segment SGI112483[] = {{&S7214[44], 26}, {NULL}};
const Segment SGI112484[] = {{&S7214[30], 6}, {NULL}};
const Segment SGI112485[] = {{&S104951[0], 14}, {NULL}};
const Segment SGI112486[] = {{&S7214[31], 3}, {NULL}};
const Segment SGI112487[] = {{&S7214[84], 6}, {NULL}};
const Segment SGI112488[] = {{&S7215[44], 9}, {NULL}};
const Segment SGI112489[] = {{&S7215[44], 21}, {NULL}};
const Segment SGI112490[] = {{&S7215[30], 6}, {NULL}};
const Segment SGI112491[] = {{&S7215[31], 3}, {NULL}};
const Segment SGI112492[] = {{&S104953[57], 10}, {NULL}};
const Segment SGI112493[] = {{&S7215[79], 6}, {NULL}};
const Segment SGI112494[] = {{&S104953[57], 22}, {NULL}};
const Segment SGI112495[] = {{&S104953[45], 4}, {NULL}};
const Segment SGI112496[] = {{&S104953[128], 3}, {NULL}};
const Segment SGI112497[] = {{&S104953[93], 6}, {NULL}};
const Segment SGI112498[] = {{&S104953[0], 14}, {NULL}};
const Segment SGI112499[] = {{&S7216[44], 8}, {NULL}};
const Segment SGI112500[] = {{&S7216[44], 20}, {NULL}};
const Segment SGI112501[] = {{&S7216[30], 6}, {NULL}};
const Segment SGI112502[] = {{&S7216[31], 3}, {NULL}};
const Segment SGI112503[] = {{&S7216[78], 6}, {NULL}};
const Segment SGI112504[] = {{&S7217[44], 8}, {NULL}};
const Segment SGI112505[] = {{&S7217[44], 20}, {NULL}};
const Segment SGI112506[] = {{&S7217[30], 6}, {NULL}};
const Segment SGI112507[] = {{&S7217[31], 3}, {NULL}};
const Segment SGI112508[] = {{&S7217[78], 6}, {NULL}};
const Segment SGI112509[] = {{&S7218[44], 9}, {NULL}};
const Segment SGI112510[] = {{&S7218[44], 23}, {NULL}};
const Segment SGI112511[] = {{&S7218[30], 6}, {NULL}};
const Segment SGI112512[] = {{&S7218[31], 3}, {NULL}};
const Segment SGI112513[] = {{&S7218[81], 6}, {NULL}};
const Segment SGI112514[] = {{&S89796[34], 9}, {NULL}};
const Segment SGI112515[] = {{&S89796[27], 6}, {NULL}};
const Segment SGI112516[] = {{&S89796[96], 9}, {NULL}};
const Segment SGI112517[] = {{&S89796[69], 6}, {NULL}};
const Segment SGI112518[] = {{&S7219[44], 14}, {NULL}};
const Segment SGI112519[] = {{&S7219[44], 26}, {NULL}};
const Segment SGI112520[] = {{&S7219[30], 6}, {NULL}};
const Segment SGI112521[] = {{&S7219[31], 3}, {NULL}};
const Segment SGI112522[] = {{&S7219[84], 6}, {NULL}};
const Segment SGI112523[] = {{&S104963[57], 10}, {NULL}};
const Segment SGI112524[] = {{&S7222[44], 4}, {NULL}};
const Segment SGI112525[] = {{&S104963[57], 21}, {NULL}};
const Segment SGI112526[] = {{&S7222[44], 16}, {NULL}};
const Segment SGI112527[] = {{&S7222[30], 6}, {NULL}};
const Segment SGI112528[] = {{&S104963[45], 4}, {NULL}};
const Segment SGI112529[] = {{&S7222[31], 3}, {NULL}};
const Segment SGI112530[] = {{&S104963[127], 3}, {NULL}};
const Segment SGI112531[] = {{&S7222[74], 6}, {NULL}};
const Segment SGI112532[] = {{&S104963[92], 6}, {NULL}};
const Segment SGI112533[] = {{&S104963[0], 14}, {NULL}};
const Segment SGI112534[] = {{&S104964[42], 10}, {NULL}};
const Segment SGI112535[] = {{&S104964[42], 22}, {NULL}};
const Segment SGI112536[] = {{&S104964[30], 4}, {NULL}};
const Segment SGI112537[] = {{&S104964[113], 3}, {NULL}};
const Segment SGI112538[] = {{&S104964[78], 6}, {NULL}};
const Segment SGI112539[] = {{&S7224[44], 11}, {NULL}};
const Segment SGI112540[] = {{&S104964[131], 11}, {NULL}};
const Segment SGI112541[] = {{&S7224[44], 31}, {NULL}};
const Segment SGI112542[] = {{&S7224[30], 6}, {NULL}};
const Segment SGI112543[] = {{&S7224[31], 3}, {NULL}};
const Segment SGI112544[] = {{&S89804[34], 7}, {NULL}};
const Segment SGI112545[] = {{&S7224[89], 6}, {NULL}};
const Segment SGI112546[] = {{&S89804[27], 6}, {NULL}};
const Segment SGI112547[] = {{&S89804[99], 9}, {NULL}};
const Segment SGI112548[] = {{&S89804[72], 6}, {NULL}};
const Segment SGI112549[] = {{&S89804[140], 8}, {NULL}};
const Segment SGI112550[] = {{&S7225[44], 9}, {NULL}};
const Segment SGI112551[] = {{&S7225[44], 21}, {NULL}};
const Segment SGI112552[] = {{&S7225[30], 6}, {NULL}};
const Segment SGI112553[] = {{&S7225[31], 3}, {NULL}};
const Segment SGI112554[] = {{&S7225[79], 6}, {NULL}};
const Segment SGI112555[] = {{&S104967[42], 10}, {NULL}};
const Segment SGI112556[] = {{&S104967[42], 22}, {NULL}};
const Segment SGI112557[] = {{&S104967[30], 4}, {NULL}};
const Segment SGI112558[] = {{&S104967[120], 3}, {NULL}};
const Segment SGI112559[] = {{&S104967[85], 6}, {NULL}};
const Segment SGI112560[] = {{&S104967[64], 8}, {NULL}};
const Segment SGI112561[] = {{&S89807[34], 8}, {NULL}};
const Segment SGI112562[] = {{&S89807[27], 6}, {NULL}};
const Segment SGI112563[] = {{&S89807[95], 9}, {NULL}};
const Segment SGI112564[] = {{&S7227[44], 9}, {NULL}};
const Segment SGI112565[] = {{&S7227[44], 32}, {NULL}};
const Segment SGI112566[] = {{&S89807[68], 6}, {NULL}};
const Segment SGI112567[] = {{&S7227[30], 6}, {NULL}};
const Segment SGI112568[] = {{&S7227[31], 3}, {NULL}};
const Segment SGI112569[] = {{&S7227[90], 6}, {NULL}};
const Segment SGI112570[] = {{&S7228[44], 9}, {NULL}};
const Segment SGI112571[] = {{&S7228[44], 21}, {NULL}};
const Segment SGI112572[] = {{&S7228[30], 6}, {NULL}};
const Segment SGI112573[] = {{&S104970[42], 10}, {NULL}};
const Segment SGI112574[] = {{&S104970[42], 21}, {NULL}};
const Segment SGI112575[] = {{&S104970[30], 4}, {NULL}};
const Segment SGI112576[] = {{&S7228[31], 3}, {NULL}};
const Segment SGI112577[] = {{&S7228[79], 6}, {NULL}};
const Segment SGI112578[] = {{&S104970[112], 3}, {NULL}};
const Segment SGI112579[] = {{&S104970[77], 6}, {NULL}};
const Segment SGI112580[] = {{&S104970[130], 13}, {NULL}};
const Segment SGI112581[] = {{&S7229[44], 15}, {NULL}};
const Segment SGI112582[] = {{&S7229[44], 36}, {NULL}};
const Segment SGI112583[] = {{&S7229[30], 6}, {NULL}};
const Segment SGI112584[] = {{&S104971[42], 10}, {NULL}};
const Segment SGI112585[] = {{&S89812[44], 12}, {NULL}};
const Segment SGI112586[] = {{&S104971[42], 21}, {NULL}};
const Segment SGI112587[] = {{&S89812[44], 24}, {NULL}};
const Segment SGI112588[] = {{&S7229[31], 3}, {NULL}};
const Segment SGI112589[] = {{&S104971[30], 4}, {NULL}};
const Segment SGI112590[] = {{&S89812[30], 6}, {NULL}};
const Segment SGI112591[] = {{&S7229[94], 5}, {NULL}};
const Segment SGI112592[] = {{&S104971[112], 3}, {NULL}};
const Segment SGI112593[] = {{&S89812[31], 3}, {NULL}};
const Segment SGI112594[] = {{&S104971[77], 6}, {NULL}};
const Segment SGI112595[] = {{&S89812[82], 6}, {NULL}};
const Segment SGI112596[] = {{&S104971[137], 13}, {NULL}};
const Segment SGI112597[] = {{&S89812[142], 7}, {NULL}};
const Segment SGI112598[] = {{&S7230[44], 15}, {NULL}};
const Segment SGI112599[] = {{&S7230[44], 27}, {NULL}};
const Segment SGI112600[] = {{&S7230[30], 6}, {NULL}};
const Segment SGI112601[] = {{&S7230[31], 3}, {NULL}};
const Segment SGI112602[] = {{&S7230[85], 6}, {NULL}};
const Segment SGI112603[] = {{&S104973[42], 10}, {NULL}};
const Segment SGI112604[] = {{&S104973[42], 22}, {NULL}};
const Segment SGI112605[] = {{&S104973[30], 4}, {NULL}};
const Segment SGI112606[] = {{&S104973[111], 3}, {NULL}};
const Segment SGI112607[] = {{&S104973[78], 5}, {NULL}};
const Segment SGI112608[] = {{&S7231[44], 10}, {NULL}};
const Segment SGI112609[] = {{&S104973[115], 14}, {NULL}};
const Segment SGI112610[] = {{&S7231[44], 22}, {NULL}};
const Segment SGI112611[] = {{&S7231[30], 6}, {NULL}};
const Segment SGI112612[] = {{&S7231[31], 3}, {NULL}};
const Segment SGI112613[] = {{&S7231[80], 6}, {NULL}};
const Segment SGI112614[] = {{&S104974[42], 11}, {NULL}};
const Segment SGI112615[] = {{&S104974[42], 23}, {NULL}};
const Segment SGI112616[] = {{&S104974[30], 4}, {NULL}};
const Segment SGI112617[] = {{&S104974[112], 3}, {NULL}};
const Segment SGI112618[] = {{&S104974[79], 5}, {NULL}};
const Segment SGI112619[] = {{&S104974[116], 14}, {NULL}};
const Segment SGI112620[] = {{&S7233[44], 5}, {NULL}};
const Segment SGI112621[] = {{&S7233[44], 17}, {NULL}};
const Segment SGI112622[] = {{&S7233[30], 6}, {NULL}};
const Segment SGI112623[] = {{&S7233[31], 3}, {NULL}};
const Segment SGI112624[] = {{&S7233[75], 6}, {NULL}};
const Segment SGI112625[] = {{&S7234[44], 21}, {NULL}};
const Segment SGI112626[] = {{&S7234[44], 33}, {NULL}};
const Segment SGI112627[] = {{&S7234[30], 6}, {NULL}};
const Segment SGI112628[] = {{&S7234[31], 3}, {NULL}};
const Segment SGI112629[] = {{&S7234[91], 6}, {NULL}};
const Segment SGI112630[] = {{&S7235[44], 8}, {NULL}};
const Segment SGI112631[] = {{&S7235[44], 26}, {NULL}};
const Segment SGI112632[] = {{&S7235[30], 6}, {NULL}};
const Segment SGI112633[] = {{&S7235[31], 3}, {NULL}};
const Segment SGI112634[] = {{&S7235[84], 6}, {NULL}};
const Segment SGI112635[] = {{&S7236[44], 11}, {NULL}};
const Segment SGI112636[] = {{&S7236[44], 31}, {NULL}};
const Segment SGI112637[] = {{&S7236[30], 6}, {NULL}};
const Segment SGI112638[] = {{&S7236[31], 3}, {NULL}};
const Segment SGI112639[] = {{&S7236[89], 6}, {NULL}};
const Segment SGI112640[] = {{&S7238[44], 6}, {NULL}};
const Segment SGI112641[] = {{&S7238[44], 18}, {NULL}};
const Segment SGI112642[] = {{&S7238[30], 6}, {NULL}};
const Segment SGI112643[] = {{&S104983[42], 10}, {NULL}};
const Segment SGI112644[] = {{&S104983[42], 22}, {NULL}};
const Segment SGI112645[] = {{&S104983[30], 4}, {NULL}};
const Segment SGI112646[] = {{&S7238[31], 3}, {NULL}};
const Segment SGI112647[] = {{&S7238[76], 6}, {NULL}};
const Segment SGI112648[] = {{&S104983[113], 3}, {NULL}};
const Segment SGI112649[] = {{&S104983[78], 6}, {NULL}};
const Segment SGI112650[] = {{&S104983[131], 13}, {NULL}};
const Segment SGI112651[] = {{&S7239[44], 7}, {NULL}};
const Segment SGI112652[] = {{&S7239[44], 19}, {NULL}};
const Segment SGI112653[] = {{&S7239[30], 6}, {NULL}};
const Segment SGI112654[] = {{&S7239[31], 3}, {NULL}};
const Segment SGI112655[] = {{&S7239[77], 6}, {NULL}};
const Segment SGI112656[] = {{&S104985[42], 10}, {NULL}};
const Segment SGI112657[] = {{&S104985[42], 22}, {NULL}};
const Segment SGI112658[] = {{&S7240[44], 12}, {NULL}};
const Segment SGI112659[] = {{&S104985[30], 4}, {NULL}};
const Segment SGI112660[] = {{&S7240[44], 24}, {NULL}};
const Segment SGI112661[] = {{&S7240[30], 6}, {NULL}};
const Segment SGI112662[] = {{&S104985[113], 3}, {NULL}};
const Segment SGI112663[] = {{&S7240[31], 3}, {NULL}};
const Segment SGI112664[] = {{&S104985[78], 6}, {NULL}};
const Segment SGI112665[] = {{&S7240[82], 6}, {NULL}};
const Segment SGI112666[] = {{&S104985[131], 11}, {NULL}};
const Segment SGI112667[] = {{&S7241[44], 12}, {NULL}};
const Segment SGI112668[] = {{&S7241[44], 24}, {NULL}};
const Segment SGI112669[] = {{&S7241[30], 6}, {NULL}};
const Segment SGI112670[] = {{&S7241[31], 3}, {NULL}};
const Segment SGI112671[] = {{&S7241[82], 6}, {NULL}};
const Segment SGI112672[] = {{&S104988[44], 10}, {NULL}};
const Segment SGI112673[] = {{&S89836[44], 18}, {NULL}};
const Segment SGI112674[] = {{&S104988[44], 21}, {NULL}};
const Segment SGI112675[] = {{&S7242[44], 8}, {NULL}};
const Segment SGI112676[] = {{&S89836[44], 30}, {NULL}};
const Segment SGI112677[] = {{&S104988[30], 6}, {NULL}};
const Segment SGI112678[] = {{&S7242[44], 20}, {NULL}};
const Segment SGI112679[] = {{&S89836[30], 6}, {NULL}};
const Segment SGI112680[] = {{&S7242[30], 6}, {NULL}};
const Segment SGI112681[] = {{&S104988[114], 3}, {NULL}};
const Segment SGI112682[] = {{&S7242[31], 3}, {NULL}};
const Segment SGI112683[] = {{&S7242[78], 6}, {NULL}};
const Segment SGI112684[] = {{&S89836[88], 6}, {NULL}};
const Segment SGI112685[] = {{&S104988[79], 6}, {NULL}};
const Segment SGI112686[] = {{&S104988[131], 10}, {NULL}};
const Segment SGI112687[] = {{&S89836[115], 13}, {NULL}};
const Segment SGI112688[] = {{&S7243[44], 8}, {NULL}};
const Segment SGI112689[] = {{&S7243[44], 20}, {NULL}};
const Segment SGI112690[] = {{&S7243[30], 6}, {NULL}};
const Segment SGI112691[] = {{&S104989[42], 10}, {NULL}};
const Segment SGI112692[] = {{&S104989[42], 22}, {NULL}};
const Segment SGI112693[] = {{&S104989[30], 4}, {NULL}};
const Segment SGI112694[] = {{&S7243[31], 3}, {NULL}};
const Segment SGI112695[] = {{&S7243[78], 6}, {NULL}};
const Segment SGI112696[] = {{&S104989[113], 3}, {NULL}};
const Segment SGI112697[] = {{&S104989[78], 6}, {NULL}};
const Segment SGI112698[] = {{&S104989[131], 11}, {NULL}};
const Segment SGI112699[] = {{&S7244[44], 10}, {NULL}};
const Segment SGI112700[] = {{&S7244[44], 42}, {NULL}};
const Segment SGI112701[] = {{&S7244[30], 6}, {NULL}};
const Segment SGI112702[] = {{&S7244[31], 3}, {NULL}};
const Segment SGI112703[] = {{&S7244[100], 6}, {NULL}};
const Segment SGI112704[] = {{&S7245[44], 20}, {NULL}};
const Segment SGI112705[] = {{&S7245[44], 32}, {NULL}};
const Segment SGI112706[] = {{&S7245[30], 6}, {NULL}};
const Segment SGI112707[] = {{&S7245[31], 3}, {NULL}};
const Segment SGI112708[] = {{&S7245[90], 6}, {NULL}};
const Segment SGI112709[] = {{&S104994[42], 10}, {NULL}};
const Segment SGI112710[] = {{&S104994[42], 22}, {NULL}};
const Segment SGI112711[] = {{&S104994[30], 4}, {NULL}};
const Segment SGI112712[] = {{&S104994[78], 6}, {NULL}};
const Segment SGI112713[] = {{&S7247[44], 10}, {NULL}};
const Segment SGI112714[] = {{&S104994[105], 13}, {NULL}};
const Segment SGI112715[] = {{&S7247[44], 22}, {NULL}};
const Segment SGI112716[] = {{&S7247[30], 6}, {NULL}};
const Segment SGI112717[] = {{&S7247[31], 3}, {NULL}};
const Segment SGI112718[] = {{&S7247[80], 6}, {NULL}};
const Segment SGI112719[] = {{&S7248[44], 14}, {NULL}};
const Segment SGI112720[] = {{&S104996[42], 10}, {NULL}};
const Segment SGI112721[] = {{&S7248[44], 26}, {NULL}};
const Segment SGI112722[] = {{&S7248[30], 6}, {NULL}};
const Segment SGI112723[] = {{&S104996[42], 21}, {NULL}};
const Segment SGI112724[] = {{&S104996[30], 4}, {NULL}};
const Segment SGI112725[] = {{&S7248[31], 3}, {NULL}};
const Segment SGI112726[] = {{&S7248[84], 6}, {NULL}};
const Segment SGI112727[] = {{&S104996[77], 6}, {NULL}};
const Segment SGI112728[] = {{&S104996[104], 13}, {NULL}};
const Segment SGI112729[] = {{&S89848[44], 41}, {NULL}};
const Segment SGI112730[] = {{&S89848[44], 53}, {NULL}};
const Segment SGI112731[] = {{&S89848[30], 6}, {NULL}};
const Segment SGI112732[] = {{&S7249[44], 71}, {NULL}};
const Segment SGI112733[] = {{&S7249[44], 92}, {NULL}};
const Segment SGI112734[] = {{&S7249[30], 6}, {NULL}};
const Segment SGI112735[] = {{&S89848[111], 6}, {NULL}};
const Segment SGI112736[] = {{&S89848[138], 13}, {NULL}};
const Segment SGI112737[] = {{&S7249[31], 3}, {NULL}};
const Segment SGI112738[] = {{&S7249[109], 5}, {NULL}};
const Segment SGI112739[] = {{&S104999[43], 10}, {NULL}};
const Segment SGI112740[] = {{&S104999[30], 4}, {NULL}};
const Segment SGI112741[] = {{&S104999[112], 3}, {NULL}};
const Segment SGI112742[] = {{&S104999[78], 5}, {NULL}};
const Segment SGI112743[] = {{&S104999[136], 13}, {NULL}};
const Segment SGI112744[] = {{&S7251[44], 8}, {NULL}};
const Segment SGI112745[] = {{&S7251[44], 20}, {NULL}};
const Segment SGI112746[] = {{&S7251[30], 6}, {NULL}};
const Segment SGI112747[] = {{&S7251[31], 3}, {NULL}};
const Segment SGI112748[] = {{&S7251[78], 6}, {NULL}};
const Segment SGI112749[] = {{&S7252[44], 6}, {NULL}};
const Segment SGI112750[] = {{&S7252[44], 18}, {NULL}};
const Segment SGI112751[] = {{&S7252[30], 6}, {NULL}};
const Segment SGI112752[] = {{&S105001[42], 10}, {NULL}};
const Segment SGI112753[] = {{&S105001[42], 22}, {NULL}};
const Segment SGI112754[] = {{&S105001[30], 4}, {NULL}};
const Segment SGI112755[] = {{&S7252[31], 3}, {NULL}};
const Segment SGI112756[] = {{&S7252[76], 6}, {NULL}};
const Segment SGI112757[] = {{&S105001[78], 6}, {NULL}};
const Segment SGI112758[] = {{&S105001[105], 13}, {NULL}};
const Segment SGI112759[] = {{&S7254[44], 15}, {NULL}};
const Segment SGI112760[] = {{&S7254[44], 27}, {NULL}};
const Segment SGI112761[] = {{&S7254[30], 6}, {NULL}};
const Segment SGI112762[] = {{&S7254[31], 3}, {NULL}};
const Segment SGI112763[] = {{&S7254[85], 6}, {NULL}};
const Segment SGI112764[] = {{&S7255[44], 11}, {NULL}};
const Segment SGI112765[] = {{&S7255[44], 23}, {NULL}};
const Segment SGI112766[] = {{&S7255[30], 6}, {NULL}};
const Segment SGI112767[] = {{&S7255[84], 6}, {NULL}};
const Segment SGI112768[] = {{&S7256[44], 10}, {NULL}};
const Segment SGI112769[] = {{&S7256[44], 22}, {NULL}};
const Segment SGI112770[] = {{&S7256[30], 6}, {NULL}};
const Segment SGI112771[] = {{&S7256[31], 3}, {NULL}};
const Segment SGI112772[] = {{&S7256[80], 6}, {NULL}};
const Segment SGI112773[] = {{&S105008[42], 10}, {NULL}};
const Segment SGI112774[] = {{&S105008[42], 22}, {NULL}};
const Segment SGI112775[] = {{&S105008[30], 4}, {NULL}};
const Segment SGI112776[] = {{&S105008[113], 3}, {NULL}};
const Segment SGI112777[] = {{&S105008[78], 6}, {NULL}};
const Segment SGI112778[] = {{&S105008[131], 13}, {NULL}};
const Segment SGI112779[] = {{&S7258[44], 9}, {NULL}};
const Segment SGI112780[] = {{&S7258[44], 21}, {NULL}};
const Segment SGI112781[] = {{&S7258[30], 6}, {NULL}};
const Segment SGI112782[] = {{&S7258[31], 3}, {NULL}};
const Segment SGI112783[] = {{&S7258[79], 6}, {NULL}};
const Segment SGI112784[] = {{&S7262[44], 11}, {NULL}};
const Segment SGI112785[] = {{&S7262[44], 30}, {NULL}};
const Segment SGI112786[] = {{&S7262[30], 6}, {NULL}};
const Segment SGI112787[] = {{&S7262[91], 6}, {NULL}};
const Segment SGI112788[] = {{&S7263[44], 12}, {NULL}};
const Segment SGI112789[] = {{&S7263[44], 24}, {NULL}};
const Segment SGI112790[] = {{&S7263[30], 6}, {NULL}};
const Segment SGI112791[] = {{&S7263[31], 3}, {NULL}};
const Segment SGI112792[] = {{&S7263[82], 6}, {NULL}};
const Segment SGI112793[] = {{&S7264[44], 12}, {NULL}};
const Segment SGI112794[] = {{&S7264[44], 24}, {NULL}};
const Segment SGI112795[] = {{&S7264[30], 6}, {NULL}};
const Segment SGI112796[] = {{&S7264[31], 3}, {NULL}};
const Segment SGI112797[] = {{&S7264[82], 6}, {NULL}};
const Segment SGI112798[] = {{&S7265[44], 8}, {NULL}};
const Segment SGI112799[] = {{&S7265[44], 20}, {NULL}};
const Segment SGI112800[] = {{&S7265[30], 6}, {NULL}};
const Segment SGI112801[] = {{&S7265[31], 3}, {NULL}};
const Segment SGI112802[] = {{&S7265[78], 6}, {NULL}};
const Segment SGI112803[] = {{&S105020[42], 10}, {NULL}};
const Segment SGI112804[] = {{&S105020[42], 22}, {NULL}};
const Segment SGI112805[] = {{&S105020[30], 4}, {NULL}};
const Segment SGI112806[] = {{&S7266[44], 17}, {NULL}};
const Segment SGI112807[] = {{&S7266[44], 29}, {NULL}};
const Segment SGI112808[] = {{&S7266[30], 6}, {NULL}};
const Segment SGI112809[] = {{&S105020[111], 3}, {NULL}};
const Segment SGI112810[] = {{&S105020[78], 5}, {NULL}};
const Segment SGI112811[] = {{&S7266[31], 3}, {NULL}};
const Segment SGI112812[] = {{&S7266[87], 6}, {NULL}};
const Segment SGI112813[] = {{&S105020[115], 14}, {NULL}};
const Segment SGI112814[] = {{&S7267[38], 6}, {NULL}};
const Segment SGI112815[] = {{&S7267[31], 6}, {NULL}};
const Segment SGI112816[] = {{&S7268[44], 7}, {NULL}};
const Segment SGI112817[] = {{&S7268[44], 19}, {NULL}};
const Segment SGI112818[] = {{&S7268[30], 6}, {NULL}};
const Segment SGI112819[] = {{&S7268[31], 3}, {NULL}};
const Segment SGI112820[] = {{&S7268[77], 6}, {NULL}};
const Segment SGI112821[] = {{&S7270[44], 4}, {NULL}};
const Segment SGI112822[] = {{&S7270[44], 16}, {NULL}};
const Segment SGI112823[] = {{&S7270[30], 6}, {NULL}};
const Segment SGI112824[] = {{&S7270[31], 3}, {NULL}};
const Segment SGI112825[] = {{&S7270[74], 5}, {NULL}};
const Segment SGI112826[] = {{&S7271[44], 13}, {NULL}};
const Segment SGI112827[] = {{&S7271[44], 25}, {NULL}};
const Segment SGI112828[] = {{&S7271[30], 6}, {NULL}};
const Segment SGI112829[] = {{&S7271[31], 3}, {NULL}};
const Segment SGI112830[] = {{&S7271[83], 6}, {NULL}};
const Segment SGI112831[] = {{&S7272[44], 18}, {NULL}};
const Segment SGI112832[] = {{&S7272[44], 30}, {NULL}};
const Segment SGI112833[] = {{&S7272[30], 6}, {NULL}};
const Segment SGI112834[] = {{&S89889[34], 7}, {NULL}};
const Segment SGI112835[] = {{&S89889[27], 6}, {NULL}};
const Segment SGI112836[] = {{&S7272[31], 3}, {NULL}};
const Segment SGI112837[] = {{&S7272[88], 6}, {NULL}};
const Segment SGI112838[] = {{&S89889[97], 9}, {NULL}};
const Segment SGI112839[] = {{&S89889[70], 6}, {NULL}};
const Segment SGI112840[] = {{&S7273[44], 10}, {NULL}};
const Segment SGI112841[] = {{&S7273[44], 22}, {NULL}};
const Segment SGI112842[] = {{&S7273[30], 6}, {NULL}};
const Segment SGI112843[] = {{&S7273[31], 3}, {NULL}};
const Segment SGI112844[] = {{&S7273[80], 6}, {NULL}};
const Segment SGI112845[] = {{&S89891[34], 13}, {NULL}};
const Segment SGI112846[] = {{&S89891[27], 6}, {NULL}};
const Segment SGI112847[] = {{&S89891[100], 9}, {NULL}};
const Segment SGI112848[] = {{&S89891[73], 6}, {NULL}};
const Segment SGI112849[] = {{&S7274[44], 7}, {NULL}};
const Segment SGI112850[] = {{&S7274[44], 19}, {NULL}};
const Segment SGI112851[] = {{&S7274[30], 6}, {NULL}};
const Segment SGI112852[] = {{&S7274[31], 3}, {NULL}};
const Segment SGI112853[] = {{&S7274[77], 6}, {NULL}};
const Segment SGI112854[] = {{&S7275[44], 7}, {NULL}};
const Segment SGI112855[] = {{&S7275[44], 19}, {NULL}};
const Segment SGI112856[] = {{&S7275[30], 6}, {NULL}};
const Segment SGI112857[] = {{&S89893[44], 9}, {NULL}};
const Segment SGI112858[] = {{&S89893[44], 27}, {NULL}};
const Segment SGI112859[] = {{&S89893[30], 6}, {NULL}};
const Segment SGI112860[] = {{&S7275[31], 3}, {NULL}};
const Segment SGI112861[] = {{&S7275[77], 6}, {NULL}};
const Segment SGI112862[] = {{&S89893[31], 3}, {NULL}};
const Segment SGI112863[] = {{&S89893[85], 6}, {NULL}};
const Segment SGI112864[] = {{&S89894[44], 9}, {NULL}};
const Segment SGI112865[] = {{&S89894[44], 18}, {NULL}};
const Segment SGI112866[] = {{&S89894[30], 6}, {NULL}};
const Segment SGI112867[] = {{&S89894[31], 3}, {NULL}};
const Segment SGI112868[] = {{&S7277[44], 8}, {NULL}};
const Segment SGI112869[] = {{&S89894[76], 6}, {NULL}};
const Segment SGI112870[] = {{&S7277[44], 20}, {NULL}};
const Segment SGI112871[] = {{&S7277[30], 6}, {NULL}};
const Segment SGI112872[] = {{&S7277[31], 3}, {NULL}};
const Segment SGI112873[] = {{&S7277[78], 6}, {NULL}};
const Segment SGI112874[] = {{&S7278[44], 10}, {NULL}};
const Segment SGI112875[] = {{&S7278[44], 22}, {NULL}};
const Segment SGI112876[] = {{&S7278[30], 6}, {NULL}};
const Segment SGI112877[] = {{&S7278[31], 3}, {NULL}};
const Segment SGI112878[] = {{&S7278[80], 6}, {NULL}};
const Segment SGI112879[] = {{&S7279[44], 8}, {NULL}};
const Segment SGI112880[] = {{&S7279[44], 20}, {NULL}};
const Segment SGI112881[] = {{&S7279[30], 6}, {NULL}};
const Segment SGI112882[] = {{&S7279[31], 3}, {NULL}};
const Segment SGI112883[] = {{&S7279[78], 6}, {NULL}};
const Segment SGI112884[] = {{&S7280[44], 14}, {NULL}};
const Segment SGI112885[] = {{&S7280[44], 26}, {NULL}};
const Segment SGI112886[] = {{&S7280[30], 6}, {NULL}};
const Segment SGI112887[] = {{&S7280[31], 3}, {NULL}};
const Segment SGI112888[] = {{&S7280[84], 6}, {NULL}};
const Segment SGI112889[] = {{&S7281[44], 5}, {NULL}};
const Segment SGI112890[] = {{&S7281[44], 17}, {NULL}};
const Segment SGI112891[] = {{&S7281[30], 6}, {NULL}};
const Segment SGI112892[] = {{&S7281[31], 3}, {NULL}};
const Segment SGI112893[] = {{&S7281[75], 6}, {NULL}};
const Segment SGI112894[] = {{&S89904[44], 23}, {NULL}};
const Segment SGI112895[] = {{&S89904[44], 41}, {NULL}};
const Segment SGI112896[] = {{&S89904[30], 6}, {NULL}};
const Segment SGI112897[] = {{&S7282[44], 6}, {NULL}};
const Segment SGI112898[] = {{&S7282[44], 18}, {NULL}};
const Segment SGI112899[] = {{&S7282[30], 6}, {NULL}};
const Segment SGI112900[] = {{&S89904[31], 3}, {NULL}};
const Segment SGI112901[] = {{&S89904[99], 6}, {NULL}};
const Segment SGI112902[] = {{&S7282[31], 3}, {NULL}};
const Segment SGI112903[] = {{&S7282[76], 6}, {NULL}};
const Segment SGI112904[] = {{&S89905[44], 24}, {NULL}};
const Segment SGI112905[] = {{&S7283[44], 10}, {NULL}};
const Segment SGI112906[] = {{&S89905[44], 42}, {NULL}};
const Segment SGI112907[] = {{&S7283[44], 22}, {NULL}};
const Segment SGI112908[] = {{&S7283[30], 6}, {NULL}};
const Segment SGI112909[] = {{&S89905[30], 6}, {NULL}};
const Segment SGI112910[] = {{&S7283[31], 3}, {NULL}};
const Segment SGI112911[] = {{&S89905[31], 3}, {NULL}};
const Segment SGI112912[] = {{&S7283[80], 6}, {NULL}};
const Segment SGI112913[] = {{&S89905[100], 6}, {NULL}};
const Segment SGI112914[] = {{&S7286[44], 7}, {NULL}};
const Segment SGI112915[] = {{&S7286[44], 19}, {NULL}};
const Segment SGI112916[] = {{&S7286[30], 6}, {NULL}};
const Segment SGI112917[] = {{&S7286[31], 3}, {NULL}};
const Segment SGI112918[] = {{&S7286[77], 6}, {NULL}};
const Segment SGI112919[] = {{&S7287[44], 10}, {NULL}};
const Segment SGI112920[] = {{&S7287[44], 22}, {NULL}};
const Segment SGI112921[] = {{&S7287[30], 6}, {NULL}};
const Segment SGI112922[] = {{&S7287[31], 3}, {NULL}};
const Segment SGI112923[] = {{&S7287[80], 6}, {NULL}};
const Segment SGI112924[] = {{&S7289[44], 11}, {NULL}};
const Segment SGI112925[] = {{&S7289[44], 23}, {NULL}};
const Segment SGI112926[] = {{&S7289[30], 6}, {NULL}};
const Segment SGI112927[] = {{&S7289[31], 3}, {NULL}};
const Segment SGI112928[] = {{&S7289[81], 6}, {NULL}};
const Segment SGI112929[] = {{&S7291[44], 10}, {NULL}};
const Segment SGI112930[] = {{&S7291[30], 6}, {NULL}};
const Segment SGI112931[] = {{&S7291[31], 3}, {NULL}};
const Segment SGI112932[] = {{&S7291[82], 6}, {NULL}};
const Segment SGI112933[] = {{&S7293[44], 7}, {NULL}};
const Segment SGI112934[] = {{&S7293[44], 19}, {NULL}};
const Segment SGI112935[] = {{&S7293[30], 6}, {NULL}};
const Segment SGI112936[] = {{&S7293[31], 3}, {NULL}};
const Segment SGI112937[] = {{&S7293[77], 6}, {NULL}};
const Segment SGI112938[] = {{&S7294[44], 9}, {NULL}};
const Segment SGI112939[] = {{&S7294[44], 21}, {NULL}};
const Segment SGI112940[] = {{&S7294[30], 6}, {NULL}};
const Segment SGI112941[] = {{&S7294[31], 3}, {NULL}};
const Segment SGI112942[] = {{&S7294[79], 6}, {NULL}};
const Segment SGI112943[] = {{&S7296[44], 11}, {NULL}};
const Segment SGI112944[] = {{&S7296[44], 23}, {NULL}};
const Segment SGI112945[] = {{&S7296[30], 6}, {NULL}};
const Segment SGI112946[] = {{&S89914[34], 9}, {NULL}};
const Segment SGI112947[] = {{&S7296[31], 3}, {NULL}};
const Segment SGI112948[] = {{&S89914[27], 6}, {NULL}};
const Segment SGI112949[] = {{&S7296[81], 6}, {NULL}};
const Segment SGI112950[] = {{&S89914[96], 9}, {NULL}};
const Segment SGI112951[] = {{&S89914[69], 6}, {NULL}};
const Segment SGI112952[] = {{&S7297[44], 7}, {NULL}};
const Segment SGI112953[] = {{&S7297[44], 19}, {NULL}};
const Segment SGI112954[] = {{&S7297[30], 6}, {NULL}};
const Segment SGI112955[] = {{&S7297[31], 3}, {NULL}};
const Segment SGI112956[] = {{&S7297[77], 6}, {NULL}};
const Segment SGI112957[] = {{&S7298[44], 7}, {NULL}};
const Segment SGI112958[] = {{&S7298[44], 19}, {NULL}};
const Segment SGI112959[] = {{&S7298[30], 6}, {NULL}};
const Segment SGI112960[] = {{&S7298[31], 3}, {NULL}};
const Segment SGI112961[] = {{&S7298[77], 6}, {NULL}};
const Segment SGI112962[] = {{&S7299[44], 31}, {NULL}};
const Segment SGI112963[] = {{&S7299[30], 6}, {NULL}};
const Segment SGI112964[] = {{&S7300[44], 7}, {NULL}};
const Segment SGI112965[] = {{&S7300[44], 19}, {NULL}};
const Segment SGI112966[] = {{&S7300[30], 6}, {NULL}};
const Segment SGI112967[] = {{&S7300[31], 3}, {NULL}};
const Segment SGI112968[] = {{&S7300[77], 6}, {NULL}};
const Segment SGI112969[] = {{&S7301[44], 6}, {NULL}};
const Segment SGI112970[] = {{&S7301[44], 18}, {NULL}};
const Segment SGI112971[] = {{&S7301[30], 6}, {NULL}};
const Segment SGI112972[] = {{&S7301[31], 3}, {NULL}};
const Segment SGI112973[] = {{&S7301[76], 6}, {NULL}};
const Segment SGI112974[] = {{&S7302[44], 14}, {NULL}};
const Segment SGI112975[] = {{&S7302[44], 26}, {NULL}};
const Segment SGI112976[] = {{&S7302[30], 6}, {NULL}};
const Segment SGI112977[] = {{&S7302[31], 3}, {NULL}};
const Segment SGI112978[] = {{&S7302[84], 6}, {NULL}};
const Segment SGI112979[] = {{&S7303[44], 12}, {NULL}};
const Segment SGI112980[] = {{&S7303[44], 24}, {NULL}};
const Segment SGI112981[] = {{&S7303[30], 6}, {NULL}};
const Segment SGI112982[] = {{&S89926[45], 10}, {NULL}};
const Segment SGI112983[] = {{&S7303[31], 3}, {NULL}};
const Segment SGI112984[] = {{&S89926[30], 6}, {NULL}};
const Segment SGI112985[] = {{&S7303[82], 6}, {NULL}};
const Segment SGI112986[] = {{&S89926[31], 3}, {NULL}};
const Segment SGI112987[] = {{&S89926[81], 5}, {NULL}};
const Segment SGI112988[] = {{&S89926[139], 13}, {NULL}};
const Segment SGI112989[] = {{&S7304[44], 7}, {NULL}};
const Segment SGI112990[] = {{&S7304[44], 19}, {NULL}};
const Segment SGI112991[] = {{&S7304[30], 6}, {NULL}};
const Segment SGI112992[] = {{&S7304[31], 3}, {NULL}};
const Segment SGI112993[] = {{&S7304[77], 6}, {NULL}};
const Segment SGI112994[] = {{&S7305[42], 5}, {NULL}};
const Segment SGI112995[] = {{&S7305[30], 6}, {NULL}};
const Segment SGI112996[] = {{&S7305[31], 3}, {NULL}};
const Segment SGI112997[] = {{&S7305[73], 6}, {NULL}};
const Segment SGI112998[] = {{&S7307[44], 9}, {NULL}};
const Segment SGI112999[] = {{&S7307[44], 21}, {NULL}};
const Segment SGI113000[] = {{&S7307[30], 6}, {NULL}};
const Segment SGI113001[] = {{&S7307[31], 3}, {NULL}};
const Segment SGI113002[] = {{&S7307[79], 6}, {NULL}};
const Segment SGI113003[] = {{&S7308[44], 20}, {NULL}};
const Segment SGI113004[] = {{&S7308[44], 32}, {NULL}};
const Segment SGI113005[] = {{&S7308[30], 6}, {NULL}};
const Segment SGI113006[] = {{&S7308[31], 3}, {NULL}};
const Segment SGI113007[] = {{&S7308[90], 6}, {NULL}};
const Segment SGI113008[] = {{&S7314[44], 7}, {NULL}};
const Segment SGI113009[] = {{&S7314[44], 19}, {NULL}};
const Segment SGI113010[] = {{&S7314[30], 6}, {NULL}};
const Segment SGI113011[] = {{&S7314[31], 3}, {NULL}};
const Segment SGI113012[] = {{&S7314[77], 6}, {NULL}};
const Segment SGI113013[] = {{&S7316[44], 6}, {NULL}};
const Segment SGI113014[] = {{&S7316[44], 18}, {NULL}};
const Segment SGI113015[] = {{&S7316[30], 6}, {NULL}};
const Segment SGI113016[] = {{&S7316[31], 3}, {NULL}};
const Segment SGI113017[] = {{&S7316[76], 6}, {NULL}};
const Segment SGI113018[] = {{&S7317[44], 6}, {NULL}};
const Segment SGI113019[] = {{&S7317[44], 18}, {NULL}};
const Segment SGI113020[] = {{&S7317[30], 6}, {NULL}};
const Segment SGI113021[] = {{&S7317[79], 6}, {NULL}};
const Segment SGI113022[] = {{&S7318[44], 16}, {NULL}};
const Segment SGI113023[] = {{&S7318[44], 28}, {NULL}};
const Segment SGI113024[] = {{&S7318[30], 6}, {NULL}};
const Segment SGI113025[] = {{&S7318[31], 3}, {NULL}};
const Segment SGI113026[] = {{&S7318[86], 6}, {NULL}};
const Segment SGI113027[] = {{&S89947[34], 18}, {NULL}};
const Segment SGI113028[] = {{&S89947[27], 6}, {NULL}};
const Segment SGI113029[] = {{&S89947[105], 9}, {NULL}};
const Segment SGI113030[] = {{&S89947[78], 6}, {NULL}};
const Segment SGI113031[] = {{&S7320[44], 11}, {NULL}};
const Segment SGI113032[] = {{&S89948[34], 13}, {NULL}};
const Segment SGI113033[] = {{&S7320[44], 23}, {NULL}};
const Segment SGI113034[] = {{&S7320[30], 6}, {NULL}};
const Segment SGI113035[] = {{&S89948[27], 6}, {NULL}};
const Segment SGI113036[] = {{&S7320[31], 3}, {NULL}};
const Segment SGI113037[] = {{&S89948[100], 9}, {NULL}};
const Segment SGI113038[] = {{&S7320[81], 6}, {NULL}};
const Segment SGI113039[] = {{&S89948[73], 6}, {NULL}};
const Segment SGI113040[] = {{&S7324[44], 7}, {NULL}};
const Segment SGI113041[] = {{&S7324[44], 19}, {NULL}};
const Segment SGI113042[] = {{&S7324[30], 6}, {NULL}};
const Segment SGI113043[] = {{&S7324[31], 3}, {NULL}};
const Segment SGI113044[] = {{&S7324[77], 6}, {NULL}};
const Segment SGI113045[] = {{&S7325[44], 12}, {NULL}};
const Segment SGI113046[] = {{&S7325[44], 24}, {NULL}};
const Segment SGI113047[] = {{&S7325[30], 6}, {NULL}};
const Segment SGI113048[] = {{&S7325[31], 3}, {NULL}};
const Segment SGI113049[] = {{&S7325[82], 6}, {NULL}};
const Segment SGI113050[] = {{&S7326[44], 8}, {NULL}};
const Segment SGI113051[] = {{&S7326[44], 20}, {NULL}};
const Segment SGI113052[] = {{&S7326[30], 6}, {NULL}};
const Segment SGI113053[] = {{&S7326[31], 3}, {NULL}};
const Segment SGI113054[] = {{&S7326[78], 6}, {NULL}};
const Segment SGI113055[] = {{&S7327[44], 9}, {NULL}};
const Segment SGI113056[] = {{&S7327[44], 21}, {NULL}};
const Segment SGI113057[] = {{&S7327[30], 6}, {NULL}};
const Segment SGI113058[] = {{&S7327[31], 3}, {NULL}};
const Segment SGI113059[] = {{&S7327[79], 6}, {NULL}};
const Segment SGI113060[] = {{&S7328[44], 7}, {NULL}};
const Segment SGI113061[] = {{&S7328[44], 19}, {NULL}};
const Segment SGI113062[] = {{&S7328[30], 6}, {NULL}};
const Segment SGI113063[] = {{&S7328[31], 3}, {NULL}};
const Segment SGI113064[] = {{&S7328[77], 6}, {NULL}};
const Segment SGI113065[] = {{&S7330[44], 8}, {NULL}};
const Segment SGI113066[] = {{&S7330[44], 20}, {NULL}};
const Segment SGI113067[] = {{&S7330[30], 6}, {NULL}};
const Segment SGI113068[] = {{&S7330[31], 3}, {NULL}};
const Segment SGI113069[] = {{&S7330[78], 6}, {NULL}};
const Segment SGI113070[] = {{&S7331[44], 6}, {NULL}};
const Segment SGI113071[] = {{&S7331[44], 18}, {NULL}};
const Segment SGI113072[] = {{&S7331[30], 6}, {NULL}};
const Segment SGI113073[] = {{&S7331[31], 3}, {NULL}};
const Segment SGI113074[] = {{&S7331[76], 6}, {NULL}};
const Segment SGI113075[] = {{&S7332[44], 10}, {NULL}};
const Segment SGI113076[] = {{&S7332[44], 22}, {NULL}};
const Segment SGI113077[] = {{&S7332[30], 6}, {NULL}};
const Segment SGI113078[] = {{&S7332[31], 3}, {NULL}};
const Segment SGI113079[] = {{&S7332[80], 6}, {NULL}};
const Segment SGI113080[] = {{&S7334[44], 17}, {NULL}};
const Segment SGI113081[] = {{&S7334[44], 29}, {NULL}};
const Segment SGI113082[] = {{&S7334[30], 6}, {NULL}};
const Segment SGI113083[] = {{&S89969[44], 6}, {NULL}};
const Segment SGI113084[] = {{&S7334[31], 3}, {NULL}};
const Segment SGI113085[] = {{&S89969[44], 18}, {NULL}};
const Segment SGI113086[] = {{&S89969[30], 6}, {NULL}};
const Segment SGI113087[] = {{&S7334[87], 5}, {NULL}};
const Segment SGI113088[] = {{&S89969[76], 6}, {NULL}};
const Segment SGI113089[] = {{&S89969[103], 13}, {NULL}};
const Segment SGI113090[] = {{&S7335[44], 9}, {NULL}};
const Segment SGI113091[] = {{&S7335[44], 21}, {NULL}};
const Segment SGI113092[] = {{&S7335[30], 6}, {NULL}};
const Segment SGI113093[] = {{&S7335[31], 3}, {NULL}};
const Segment SGI113094[] = {{&S7335[79], 6}, {NULL}};
const Segment SGI113095[] = {{&S7338[44], 7}, {NULL}};
const Segment SGI113096[] = {{&S7338[44], 19}, {NULL}};
const Segment SGI113097[] = {{&S7338[30], 6}, {NULL}};
const Segment SGI113098[] = {{&S7338[31], 3}, {NULL}};
const Segment SGI113099[] = {{&S7338[77], 6}, {NULL}};
const Segment SGI113100[] = {{&S7339[44], 11}, {NULL}};
const Segment SGI113101[] = {{&S7339[44], 23}, {NULL}};
const Segment SGI113102[] = {{&S7339[30], 6}, {NULL}};
const Segment SGI113103[] = {{&S7339[31], 3}, {NULL}};
const Segment SGI113104[] = {{&S7339[81], 6}, {NULL}};
const Segment SGI113105[] = {{&S7341[44], 13}, {NULL}};
const Segment SGI113106[] = {{&S7341[44], 25}, {NULL}};
const Segment SGI113107[] = {{&S7341[30], 6}, {NULL}};
const Segment SGI113108[] = {{&S7341[31], 3}, {NULL}};
const Segment SGI113109[] = {{&S7341[83], 6}, {NULL}};
const Segment SGI113110[] = {{&S7345[44], 5}, {NULL}};
const Segment SGI113111[] = {{&S7345[44], 17}, {NULL}};
const Segment SGI113112[] = {{&S7345[30], 6}, {NULL}};
const Segment SGI113113[] = {{&S7345[31], 3}, {NULL}};
const Segment SGI113114[] = {{&S7345[75], 6}, {NULL}};
const Segment SGI113115[] = {{&S7346[44], 10}, {NULL}};
const Segment SGI113116[] = {{&S7346[44], 22}, {NULL}};
const Segment SGI113117[] = {{&S7346[30], 6}, {NULL}};
const Segment SGI113118[] = {{&S7346[31], 3}, {NULL}};
const Segment SGI113119[] = {{&S7346[80], 6}, {NULL}};
const Segment SGI113120[] = {{&S7349[44], 6}, {NULL}};
const Segment SGI113121[] = {{&S7349[44], 18}, {NULL}};
const Segment SGI113122[] = {{&S7349[30], 6}, {NULL}};
const Segment SGI113123[] = {{&S7349[31], 3}, {NULL}};
const Segment SGI113124[] = {{&S7349[76], 6}, {NULL}};
const Segment SGI113125[] = {{&S7351[44], 6}, {NULL}};
const Segment SGI113126[] = {{&S7351[44], 18}, {NULL}};
const Segment SGI113127[] = {{&S7351[30], 6}, {NULL}};
const Segment SGI113128[] = {{&S7351[31], 3}, {NULL}};
const Segment SGI113129[] = {{&S7351[76], 6}, {NULL}};
const Segment SGI113130[] = {{&S7352[44], 12}, {NULL}};
const Segment SGI113131[] = {{&S7352[44], 24}, {NULL}};
const Segment SGI113132[] = {{&S7352[30], 6}, {NULL}};
const Segment SGI113133[] = {{&S7352[31], 3}, {NULL}};
const Segment SGI113134[] = {{&S7352[82], 6}, {NULL}};
const Segment SGI113135[] = {{&S7353[44], 11}, {NULL}};
const Segment SGI113136[] = {{&S7353[44], 36}, {NULL}};
const Segment SGI113137[] = {{&S7353[30], 6}, {NULL}};
const Segment SGI113138[] = {{&S7353[31], 3}, {NULL}};
const Segment SGI113139[] = {{&S7353[94], 6}, {NULL}};
const Segment SGI113140[] = {{&S7354[44], 19}, {NULL}};
const Segment SGI113141[] = {{&S7354[44], 31}, {NULL}};
const Segment SGI113142[] = {{&S7354[30], 6}, {NULL}};
const Segment SGI113143[] = {{&S7354[31], 3}, {NULL}};
const Segment SGI113144[] = {{&S7354[89], 6}, {NULL}};
const Segment SGI113145[] = {{&S7355[44], 9}, {NULL}};
const Segment SGI113146[] = {{&S7355[44], 21}, {NULL}};
const Segment SGI113147[] = {{&S7355[30], 6}, {NULL}};
const Segment SGI113148[] = {{&S7355[31], 3}, {NULL}};
const Segment SGI113149[] = {{&S7355[79], 6}, {NULL}};
const Segment SGI113150[] = {{&S7358[44], 9}, {NULL}};
const Segment SGI113151[] = {{&S7358[44], 21}, {NULL}};
const Segment SGI113152[] = {{&S7358[30], 6}, {NULL}};
const Segment SGI113153[] = {{&S7358[31], 3}, {NULL}};
const Segment SGI113154[] = {{&S7358[79], 6}, {NULL}};
const Segment SGI113155[] = {{&S7359[44], 7}, {NULL}};
const Segment SGI113156[] = {{&S7359[44], 19}, {NULL}};
const Segment SGI113157[] = {{&S7359[30], 6}, {NULL}};
const Segment SGI113158[] = {{&S7359[31], 3}, {NULL}};
const Segment SGI113159[] = {{&S7359[77], 6}, {NULL}};
const Segment SGI113160[] = {{&S7360[44], 11}, {NULL}};
const Segment SGI113161[] = {{&S7360[44], 23}, {NULL}};
const Segment SGI113162[] = {{&S7360[30], 6}, {NULL}};
const Segment SGI113163[] = {{&S7360[31], 3}, {NULL}};
const Segment SGI113164[] = {{&S7360[81], 6}, {NULL}};
const Segment SGI113165[] = {{&S7361[44], 8}, {NULL}};
const Segment SGI113166[] = {{&S7361[44], 30}, {NULL}};
const Segment SGI113167[] = {{&S7361[30], 6}, {NULL}};
const Segment SGI113168[] = {{&S7361[31], 3}, {NULL}};
const Segment SGI113169[] = {{&S7361[88], 6}, {NULL}};
const Segment SGI113170[] = {{&S7362[44], 6}, {NULL}};
const Segment SGI113171[] = {{&S7362[44], 18}, {NULL}};
const Segment SGI113172[] = {{&S7362[30], 6}, {NULL}};
const Segment SGI113173[] = {{&S7362[31], 3}, {NULL}};
const Segment SGI113174[] = {{&S7362[76], 6}, {NULL}};
const Segment SGI113175[] = {{&S7363[44], 7}, {NULL}};
const Segment SGI113176[] = {{&S7363[44], 19}, {NULL}};
const Segment SGI113177[] = {{&S7363[30], 6}, {NULL}};
const Segment SGI113178[] = {{&S7363[31], 3}, {NULL}};
const Segment SGI113179[] = {{&S7363[77], 6}, {NULL}};
const Segment SGI113180[] = {{&S7365[44], 9}, {NULL}};
const Segment SGI113181[] = {{&S7365[44], 21}, {NULL}};
const Segment SGI113182[] = {{&S7365[30], 6}, {NULL}};
const Segment SGI113183[] = {{&S7365[31], 3}, {NULL}};
const Segment SGI113184[] = {{&S7365[79], 6}, {NULL}};
const Segment SGI113185[] = {{&S7366[44], 21}, {NULL}};
const Segment SGI113186[] = {{&S7366[44], 33}, {NULL}};
const Segment SGI113187[] = {{&S7366[30], 6}, {NULL}};
const Segment SGI113188[] = {{&S7366[31], 3}, {NULL}};
const Segment SGI113189[] = {{&S7366[91], 6}, {NULL}};
const Segment SGI113190[] = {{&S7368[44], 7}, {NULL}};
const Segment SGI113191[] = {{&S7368[44], 19}, {NULL}};
const Segment SGI113192[] = {{&S7368[30], 6}, {NULL}};
const Segment SGI113193[] = {{&S7368[31], 3}, {NULL}};
const Segment SGI113194[] = {{&S7368[77], 6}, {NULL}};
const Segment SGI113195[] = {{&S7369[44], 7}, {NULL}};
const Segment SGI113196[] = {{&S7369[44], 19}, {NULL}};
const Segment SGI113197[] = {{&S7369[30], 6}, {NULL}};
const Segment SGI113198[] = {{&S7369[31], 3}, {NULL}};
const Segment SGI113199[] = {{&S7369[77], 6}, {NULL}};
const Segment SGI113200[] = {{&S7370[44], 14}, {NULL}};
const Segment SGI113201[] = {{&S7370[44], 26}, {NULL}};
const Segment SGI113202[] = {{&S7370[30], 6}, {NULL}};
const Segment SGI113203[] = {{&S7370[31], 3}, {NULL}};
const Segment SGI113204[] = {{&S7370[84], 6}, {NULL}};
const Segment SGI113205[] = {{&S7371[44], 8}, {NULL}};
const Segment SGI113206[] = {{&S7371[44], 20}, {NULL}};
const Segment SGI113207[] = {{&S7371[30], 6}, {NULL}};
const Segment SGI113208[] = {{&S7371[31], 3}, {NULL}};
const Segment SGI113209[] = {{&S7371[78], 6}, {NULL}};
const Segment SGI113210[] = {{&S7372[44], 7}, {NULL}};
const Segment SGI113211[] = {{&S7372[44], 19}, {NULL}};
const Segment SGI113212[] = {{&S7372[30], 6}, {NULL}};
const Segment SGI113213[] = {{&S7372[31], 3}, {NULL}};
const Segment SGI113214[] = {{&S7372[77], 6}, {NULL}};
const Segment SGI113215[] = {{&S7373[44], 7}, {NULL}};
const Segment SGI113216[] = {{&S7373[44], 19}, {NULL}};
const Segment SGI113217[] = {{&S7373[30], 6}, {NULL}};
const Segment SGI113218[] = {{&S7373[31], 3}, {NULL}};
const Segment SGI113219[] = {{&S7373[77], 6}, {NULL}};
const Segment SGI113220[] = {{&S90050[44], 22}, {NULL}};
const Segment SGI113221[] = {{&S90050[44], 34}, {NULL}};
const Segment SGI113222[] = {{&S90050[30], 6}, {NULL}};
const Segment SGI113223[] = {{&S90050[31], 3}, {NULL}};
const Segment SGI113224[] = {{&S90050[92], 6}, {NULL}};
const Segment SGI113225[] = {{&S7375[44], 18}, {NULL}};
const Segment SGI113226[] = {{&S7375[44], 30}, {NULL}};
const Segment SGI113227[] = {{&S7375[30], 6}, {NULL}};
const Segment SGI113228[] = {{&S7375[31], 3}, {NULL}};
const Segment SGI113229[] = {{&S7375[88], 6}, {NULL}};
const Segment SGI113230[] = {{&S7381[44], 16}, {NULL}};
const Segment SGI113231[] = {{&S7381[44], 28}, {NULL}};
const Segment SGI113232[] = {{&S7381[30], 6}, {NULL}};
const Segment SGI113233[] = {{&S7381[31], 3}, {NULL}};
const Segment SGI113234[] = {{&S7381[86], 6}, {NULL}};
const Segment SGI113235[] = {{&S90061[34], 4}, {NULL}};
const Segment SGI113236[] = {{&S90061[27], 6}, {NULL}};
const Segment SGI113237[] = {{&S90061[91], 9}, {NULL}};
const Segment SGI113238[] = {{&S90061[64], 6}, {NULL}};
const Segment SGI113239[] = {{&S7389[44], 16}, {NULL}};
const Segment SGI113240[] = {{&S7389[44], 28}, {NULL}};
const Segment SGI113241[] = {{&S7389[30], 6}, {NULL}};
const Segment SGI113242[] = {{&S7389[31], 3}, {NULL}};
const Segment SGI113243[] = {{&S7389[86], 6}, {NULL}};
const Segment SGI113244[] = {{&S7393[44], 8}, {NULL}};
const Segment SGI113245[] = {{&S7393[44], 18}, {NULL}};
const Segment SGI113246[] = {{&S7393[30], 6}, {NULL}};
const Segment SGI113247[] = {{&S7393[31], 3}, {NULL}};
const Segment SGI113248[] = {{&S7393[76], 6}, {NULL}};
const Segment SGI113249[] = {{&S7394[44], 6}, {NULL}};
const Segment SGI113250[] = {{&S7394[44], 18}, {NULL}};
const Segment SGI113251[] = {{&S7394[30], 6}, {NULL}};
const Segment SGI113252[] = {{&S7394[31], 3}, {NULL}};
const Segment SGI113253[] = {{&S7394[76], 6}, {NULL}};
const Segment SGI113254[] = {{&S7397[44], 7}, {NULL}};
const Segment SGI113255[] = {{&S7397[44], 19}, {NULL}};
const Segment SGI113256[] = {{&S7397[30], 6}, {NULL}};
const Segment SGI113257[] = {{&S7397[31], 3}, {NULL}};
const Segment SGI113258[] = {{&S7397[77], 6}, {NULL}};
const Segment SGI113259[] = {{&S7398[44], 10}, {NULL}};
const Segment SGI113260[] = {{&S7398[44], 22}, {NULL}};
const Segment SGI113261[] = {{&S7398[30], 6}, {NULL}};
const Segment SGI113262[] = {{&S7398[31], 3}, {NULL}};
const Segment SGI113263[] = {{&S7398[80], 6}, {NULL}};
const Segment SGI113264[] = {{&S7405[44], 10}, {NULL}};
const Segment SGI113265[] = {{&S7405[44], 22}, {NULL}};
const Segment SGI113266[] = {{&S7405[30], 6}, {NULL}};
const Segment SGI113267[] = {{&S7405[31], 3}, {NULL}};
const Segment SGI113268[] = {{&S7405[80], 6}, {NULL}};
const Segment SGI113269[] = {{&S7406[44], 15}, {NULL}};
const Segment SGI113270[] = {{&S7406[44], 27}, {NULL}};
const Segment SGI113271[] = {{&S7406[30], 6}, {NULL}};
const Segment SGI113272[] = {{&S7406[31], 3}, {NULL}};
const Segment SGI113273[] = {{&S7406[85], 6}, {NULL}};
const Segment SGI113274[] = {{&S7408[44], 7}, {NULL}};
const Segment SGI113275[] = {{&S7408[44], 19}, {NULL}};
const Segment SGI113276[] = {{&S7408[30], 6}, {NULL}};
const Segment SGI113277[] = {{&S7408[80], 6}, {NULL}};
const Segment SGI113278[] = {{&S7410[44], 9}, {NULL}};
const Segment SGI113279[] = {{&S7410[44], 21}, {NULL}};
const Segment SGI113280[] = {{&S7410[30], 6}, {NULL}};
const Segment SGI113281[] = {{&S7410[31], 3}, {NULL}};
const Segment SGI113282[] = {{&S7410[79], 6}, {NULL}};
const Segment SGI113283[] = {{&S7411[44], 7}, {NULL}};
const Segment SGI113284[] = {{&S7411[44], 19}, {NULL}};
const Segment SGI113285[] = {{&S7411[30], 6}, {NULL}};
const Segment SGI113286[] = {{&S7411[31], 3}, {NULL}};
const Segment SGI113287[] = {{&S7411[77], 6}, {NULL}};
const Segment SGI113288[] = {{&S7415[38], 9}, {NULL}};
const Segment SGI113289[] = {{&S7415[31], 6}, {NULL}};
const Segment SGI113290[] = {{&S7417[44], 7}, {NULL}};
const Segment SGI113291[] = {{&S7417[44], 19}, {NULL}};
const Segment SGI113292[] = {{&S7417[30], 6}, {NULL}};
const Segment SGI113293[] = {{&S7417[31], 3}, {NULL}};
const Segment SGI113294[] = {{&S7417[77], 6}, {NULL}};
const Segment SGI113295[] = {{&S7418[44], 11}, {NULL}};
const Segment SGI113296[] = {{&S7418[44], 23}, {NULL}};
const Segment SGI113297[] = {{&S7418[30], 6}, {NULL}};
const Segment SGI113298[] = {{&S7418[31], 3}, {NULL}};
const Segment SGI113299[] = {{&S7418[81], 6}, {NULL}};
const Segment SGI113300[] = {{&S7419[44], 8}, {NULL}};
const Segment SGI113301[] = {{&S7419[44], 20}, {NULL}};
const Segment SGI113302[] = {{&S7419[30], 6}, {NULL}};
const Segment SGI113303[] = {{&S7419[31], 3}, {NULL}};
const Segment SGI113304[] = {{&S7419[78], 6}, {NULL}};
const Segment SGI113305[] = {{&S7420[44], 8}, {NULL}};
const Segment SGI113306[] = {{&S7420[44], 20}, {NULL}};
const Segment SGI113307[] = {{&S7420[30], 6}, {NULL}};
const Segment SGI113308[] = {{&S7420[31], 3}, {NULL}};
const Segment SGI113309[] = {{&S7420[78], 6}, {NULL}};
const Segment SGI113310[] = {{&S7421[44], 8}, {NULL}};
const Segment SGI113311[] = {{&S7421[44], 20}, {NULL}};
const Segment SGI113312[] = {{&S7421[30], 6}, {NULL}};
const Segment SGI113313[] = {{&S7421[31], 3}, {NULL}};
const Segment SGI113314[] = {{&S7421[78], 6}, {NULL}};
const Segment SGI113315[] = {{&S7422[44], 13}, {NULL}};
const Segment SGI113316[] = {{&S7422[44], 25}, {NULL}};
const Segment SGI113317[] = {{&S7422[30], 6}, {NULL}};
const Segment SGI113318[] = {{&S7422[31], 3}, {NULL}};
const Segment SGI113319[] = {{&S7422[83], 6}, {NULL}};
const Segment SGI113320[] = {{&S7423[44], 4}, {NULL}};
const Segment SGI113321[] = {{&S7423[44], 16}, {NULL}};
const Segment SGI113322[] = {{&S7423[30], 6}, {NULL}};
const Segment SGI113323[] = {{&S7423[31], 3}, {NULL}};
const Segment SGI113324[] = {{&S7423[74], 6}, {NULL}};
const Segment SGI113325[] = {{&S7428[38], 7}, {NULL}};
const Segment SGI113326[] = {{&S7428[31], 6}, {NULL}};
const Segment SGI113327[] = {{&S7429[44], 13}, {NULL}};
const Segment SGI113328[] = {{&S7429[44], 25}, {NULL}};
const Segment SGI113329[] = {{&S7429[30], 6}, {NULL}};
const Segment SGI113330[] = {{&S7429[31], 3}, {NULL}};
const Segment SGI113331[] = {{&S7429[83], 6}, {NULL}};
const Segment SGI113332[] = {{&S7430[44], 10}, {NULL}};
const Segment SGI113333[] = {{&S7430[44], 22}, {NULL}};
const Segment SGI113334[] = {{&S7430[30], 6}, {NULL}};
const Segment SGI113335[] = {{&S7430[31], 3}, {NULL}};
const Segment SGI113336[] = {{&S7430[80], 6}, {NULL}};
const Segment SGI113337[] = {{&S7431[44], 9}, {NULL}};
const Segment SGI113338[] = {{&S7431[44], 21}, {NULL}};
const Segment SGI113339[] = {{&S7431[30], 6}, {NULL}};
const Segment SGI113340[] = {{&S7431[31], 3}, {NULL}};
const Segment SGI113341[] = {{&S7431[79], 6}, {NULL}};
const Segment SGI113342[] = {{&S7432[44], 12}, {NULL}};
const Segment SGI113343[] = {{&S7432[44], 24}, {NULL}};
const Segment SGI113344[] = {{&S7432[30], 6}, {NULL}};
const Segment SGI113345[] = {{&S7432[31], 3}, {NULL}};
const Segment SGI113346[] = {{&S7432[82], 6}, {NULL}};
const Segment SGI113347[] = {{&S7440[44], 5}, {NULL}};
const Segment SGI113348[] = {{&S7440[44], 17}, {NULL}};
const Segment SGI113349[] = {{&S7440[30], 6}, {NULL}};
const Segment SGI113350[] = {{&S7440[31], 3}, {NULL}};
const Segment SGI113351[] = {{&S7440[75], 6}, {NULL}};
const Segment SGI113352[] = {{&S7441[44], 14}, {NULL}};
const Segment SGI113353[] = {{&S7441[44], 26}, {NULL}};
const Segment SGI113354[] = {{&S7441[30], 6}, {NULL}};
const Segment SGI113355[] = {{&S7441[31], 3}, {NULL}};
const Segment SGI113356[] = {{&S7441[84], 6}, {NULL}};
const Segment SGI113357[] = {{&S7442[44], 7}, {NULL}};
const Segment SGI113358[] = {{&S7442[44], 18}, {NULL}};
const Segment SGI113359[] = {{&S7442[30], 6}, {NULL}};
const Segment SGI113360[] = {{&S7442[31], 3}, {NULL}};
const Segment SGI113361[] = {{&S7442[76], 6}, {NULL}};
const Segment SGI113362[] = {{&S7443[44], 31}, {NULL}};
const Segment SGI113363[] = {{&S7443[30], 6}, {NULL}};
const Segment SGI113364[] = {{&S7443[31], 3}, {NULL}};
const Segment SGI113365[] = {{&S7443[107], 6}, {NULL}};
const Segment SGI113366[] = {{&S7444[44], 17}, {NULL}};
const Segment SGI113367[] = {{&S7444[44], 29}, {NULL}};
const Segment SGI113368[] = {{&S7444[30], 6}, {NULL}};
const Segment SGI113369[] = {{&S7444[31], 3}, {NULL}};
const Segment SGI113370[] = {{&S7444[87], 6}, {NULL}};
const Segment SGI113371[] = {{&S7446[44], 10}, {NULL}};
const Segment SGI113372[] = {{&S7446[44], 22}, {NULL}};
const Segment SGI113373[] = {{&S7446[30], 6}, {NULL}};
const Segment SGI113374[] = {{&S7446[31], 3}, {NULL}};
const Segment SGI113375[] = {{&S7446[80], 6}, {NULL}};
const Segment SGI113376[] = {{&S7447[44], 6}, {NULL}};
const Segment SGI113377[] = {{&S7447[44], 22}, {NULL}};
const Segment SGI113378[] = {{&S7447[30], 6}, {NULL}};
const Segment SGI113379[] = {{&S7447[83], 6}, {NULL}};
const Segment SGI113380[] = {{&S7449[44], 11}, {NULL}};
const Segment SGI113381[] = {{&S7449[44], 23}, {NULL}};
const Segment SGI113382[] = {{&S7449[30], 6}, {NULL}};
const Segment SGI113383[] = {{&S7449[31], 3}, {NULL}};
const Segment SGI113384[] = {{&S7449[81], 6}, {NULL}};
const Segment SGI113385[] = {{&S7450[44], 5}, {NULL}};
const Segment SGI113386[] = {{&S7450[44], 17}, {NULL}};
const Segment SGI113387[] = {{&S7450[30], 6}, {NULL}};
const Segment SGI113388[] = {{&S7450[31], 3}, {NULL}};
const Segment SGI113389[] = {{&S7450[75], 6}, {NULL}};
const Segment SGI113390[] = {{&S7451[44], 7}, {NULL}};
const Segment SGI113391[] = {{&S7451[44], 19}, {NULL}};
const Segment SGI113392[] = {{&S7451[30], 6}, {NULL}};
const Segment SGI113393[] = {{&S7451[31], 3}, {NULL}};
const Segment SGI113394[] = {{&S7451[77], 6}, {NULL}};
const Segment SGI113395[] = {{&S7452[44], 8}, {NULL}};
const Segment SGI113396[] = {{&S7452[44], 20}, {NULL}};
const Segment SGI113397[] = {{&S7452[30], 6}, {NULL}};
const Segment SGI113398[] = {{&S7452[31], 3}, {NULL}};
const Segment SGI113399[] = {{&S7452[78], 6}, {NULL}};
const Segment SGI113400[] = {{&S7453[44], 8}, {NULL}};
const Segment SGI113401[] = {{&S7453[44], 20}, {NULL}};
const Segment SGI113402[] = {{&S7453[30], 6}, {NULL}};
const Segment SGI113403[] = {{&S7453[31], 3}, {NULL}};
const Segment SGI113404[] = {{&S7453[78], 6}, {NULL}};
const Segment SGI113405[] = {{&S7454[44], 9}, {NULL}};
const Segment SGI113406[] = {{&S7454[44], 27}, {NULL}};
const Segment SGI113407[] = {{&S7454[30], 6}, {NULL}};
const Segment SGI113408[] = {{&S7454[31], 3}, {NULL}};
const Segment SGI113409[] = {{&S7454[85], 5}, {NULL}};
const Segment SGI113410[] = {{&S7455[44], 10}, {NULL}};
const Segment SGI113411[] = {{&S7455[44], 22}, {NULL}};
const Segment SGI113412[] = {{&S7455[30], 6}, {NULL}};
const Segment SGI113413[] = {{&S7455[31], 3}, {NULL}};
const Segment SGI113414[] = {{&S7455[80], 6}, {NULL}};
const Segment SGI113415[] = {{&S7456[44], 13}, {NULL}};
const Segment SGI113416[] = {{&S7456[44], 25}, {NULL}};
const Segment SGI113417[] = {{&S7456[30], 6}, {NULL}};
const Segment SGI113418[] = {{&S7456[31], 3}, {NULL}};
const Segment SGI113419[] = {{&S7456[83], 6}, {NULL}};
const Segment SGI113420[] = {{&S7459[44], 7}, {NULL}};
const Segment SGI113421[] = {{&S7459[44], 18}, {NULL}};
const Segment SGI113422[] = {{&S7459[30], 6}, {NULL}};
const Segment SGI113423[] = {{&S7459[31], 3}, {NULL}};
const Segment SGI113424[] = {{&S7459[76], 6}, {NULL}};
const Segment SGI113425[] = {{&S7460[44], 7}, {NULL}};
const Segment SGI113426[] = {{&S7460[44], 19}, {NULL}};
const Segment SGI113427[] = {{&S7460[30], 6}, {NULL}};
const Segment SGI113428[] = {{&S7460[31], 3}, {NULL}};
const Segment SGI113429[] = {{&S7460[77], 6}, {NULL}};
const Segment SGI113430[] = {{&S7461[44], 11}, {NULL}};
const Segment SGI113431[] = {{&S7461[44], 23}, {NULL}};
const Segment SGI113432[] = {{&S7461[30], 6}, {NULL}};
const Segment SGI113433[] = {{&S7461[31], 3}, {NULL}};
const Segment SGI113434[] = {{&S7461[81], 6}, {NULL}};
const Segment SGI113435[] = {{&S7462[44], 13}, {NULL}};
const Segment SGI113436[] = {{&S7462[44], 25}, {NULL}};
const Segment SGI113437[] = {{&S7462[30], 6}, {NULL}};
const Segment SGI113438[] = {{&S7462[31], 3}, {NULL}};
const Segment SGI113439[] = {{&S7462[83], 6}, {NULL}};
const Segment SGI113440[] = {{&S7464[44], 5}, {NULL}};
const Segment SGI113441[] = {{&S7464[44], 17}, {NULL}};
const Segment SGI113442[] = {{&S7464[30], 6}, {NULL}};
const Segment SGI113443[] = {{&S7464[31], 3}, {NULL}};
const Segment SGI113444[] = {{&S7464[75], 6}, {NULL}};
const Segment SGI113445[] = {{&S7466[44], 7}, {NULL}};
const Segment SGI113446[] = {{&S7466[44], 19}, {NULL}};
const Segment SGI113447[] = {{&S7466[30], 6}, {NULL}};
const Segment SGI113448[] = {{&S7466[31], 3}, {NULL}};
const Segment SGI113449[] = {{&S7466[77], 6}, {NULL}};
const Segment SGI113450[] = {{&S7467[44], 9}, {NULL}};
const Segment SGI113451[] = {{&S7467[44], 21}, {NULL}};
const Segment SGI113452[] = {{&S7467[30], 6}, {NULL}};
const Segment SGI113453[] = {{&S7467[31], 3}, {NULL}};
const Segment SGI113454[] = {{&S7467[79], 6}, {NULL}};
const Segment SGI113455[] = {{&S7468[38], 8}, {NULL}};
const Segment SGI113456[] = {{&S7468[31], 6}, {NULL}};
const Segment SGI113457[] = {{&S7469[44], 5}, {NULL}};
const Segment SGI113458[] = {{&S7469[44], 17}, {NULL}};
const Segment SGI113459[] = {{&S7469[30], 6}, {NULL}};
const Segment SGI113460[] = {{&S7469[31], 3}, {NULL}};
const Segment SGI113461[] = {{&S7469[75], 6}, {NULL}};
const Segment SGI113462[] = {{&S7470[44], 5}, {NULL}};
const Segment SGI113463[] = {{&S7470[44], 17}, {NULL}};
const Segment SGI113464[] = {{&S7470[30], 6}, {NULL}};
const Segment SGI113465[] = {{&S7470[31], 3}, {NULL}};
const Segment SGI113466[] = {{&S7470[75], 6}, {NULL}};
const Segment SGI113467[] = {{&S7471[44], 11}, {NULL}};
const Segment SGI113468[] = {{&S7471[44], 26}, {NULL}};
const Segment SGI113469[] = {{&S7471[30], 6}, {NULL}};
const Segment SGI113470[] = {{&S7471[31], 3}, {NULL}};
const Segment SGI113471[] = {{&S7471[84], 6}, {NULL}};
const Segment SGI113472[] = {{&S7472[44], 13}, {NULL}};
const Segment SGI113473[] = {{&S7472[44], 25}, {NULL}};
const Segment SGI113474[] = {{&S7472[30], 6}, {NULL}};
const Segment SGI113475[] = {{&S7472[31], 3}, {NULL}};
const Segment SGI113476[] = {{&S7472[83], 6}, {NULL}};
const Segment SGI113477[] = {{&S7474[44], 4}, {NULL}};
const Segment SGI113478[] = {{&S7474[44], 16}, {NULL}};
const Segment SGI113479[] = {{&S7474[30], 6}, {NULL}};
const Segment SGI113480[] = {{&S7474[31], 3}, {NULL}};
const Segment SGI113481[] = {{&S7474[74], 6}, {NULL}};
const Segment SGI113482[] = {{&S7475[44], 10}, {NULL}};
const Segment SGI113483[] = {{&S7475[44], 22}, {NULL}};
const Segment SGI113484[] = {{&S7475[30], 6}, {NULL}};
const Segment SGI113485[] = {{&S7475[31], 3}, {NULL}};
const Segment SGI113486[] = {{&S7475[80], 6}, {NULL}};
const Segment SGI113487[] = {{&S7476[38], 7}, {NULL}};
const Segment SGI113488[] = {{&S7476[31], 6}, {NULL}};
const Segment SGI113489[] = {{&S7478[44], 6}, {NULL}};
const Segment SGI113490[] = {{&S7478[44], 18}, {NULL}};
const Segment SGI113491[] = {{&S7478[30], 6}, {NULL}};
const Segment SGI113492[] = {{&S7478[31], 3}, {NULL}};
const Segment SGI113493[] = {{&S7478[76], 6}, {NULL}};
const Segment SGI113494[] = {{&S7479[44], 10}, {NULL}};
const Segment SGI113495[] = {{&S7479[44], 22}, {NULL}};
const Segment SGI113496[] = {{&S7479[30], 6}, {NULL}};
const Segment SGI113497[] = {{&S7479[31], 3}, {NULL}};
const Segment SGI113498[] = {{&S7479[80], 6}, {NULL}};
const Segment SGI113499[] = {{&S90112[34], 9}, {NULL}};
const Segment SGI113500[] = {{&S90112[27], 6}, {NULL}};
const Segment SGI113501[] = {{&S7480[44], 15}, {NULL}};
const Segment SGI113502[] = {{&S90112[96], 9}, {NULL}};
const Segment SGI113503[] = {{&S7480[44], 27}, {NULL}};
const Segment SGI113504[] = {{&S90112[69], 6}, {NULL}};
const Segment SGI113505[] = {{&S7480[30], 6}, {NULL}};
const Segment SGI113506[] = {{&S90112[137], 8}, {NULL}};
const Segment SGI113507[] = {{&S7480[31], 3}, {NULL}};
const Segment SGI113508[] = {{&S7480[85], 6}, {NULL}};
const Segment SGI113509[] = {{&S7481[44], 10}, {NULL}};
const Segment SGI113510[] = {{&S7481[44], 22}, {NULL}};
const Segment SGI113511[] = {{&S7481[30], 6}, {NULL}};
const Segment SGI113512[] = {{&S7481[31], 3}, {NULL}};
const Segment SGI113513[] = {{&S7481[80], 6}, {NULL}};
const Segment SGI113514[] = {{&S7482[44], 5}, {NULL}};
const Segment SGI113515[] = {{&S7482[44], 17}, {NULL}};
const Segment SGI113516[] = {{&S7482[30], 6}, {NULL}};
const Segment SGI113517[] = {{&S7482[31], 3}, {NULL}};
const Segment SGI113518[] = {{&S7482[75], 6}, {NULL}};
const Segment SGI113519[] = {{&S7483[44], 5}, {NULL}};
const Segment SGI113520[] = {{&S7483[44], 17}, {NULL}};
const Segment SGI113521[] = {{&S7483[30], 6}, {NULL}};
const Segment SGI113522[] = {{&S7483[31], 3}, {NULL}};
const Segment SGI113523[] = {{&S7483[75], 6}, {NULL}};
const Segment SGI113524[] = {{&S7484[44], 7}, {NULL}};
const Segment SGI113525[] = {{&S7484[44], 19}, {NULL}};
const Segment SGI113526[] = {{&S7484[30], 6}, {NULL}};
const Segment SGI113527[] = {{&S7484[31], 3}, {NULL}};
const Segment SGI113528[] = {{&S7484[77], 6}, {NULL}};
const Segment SGI113529[] = {{&S7485[44], 10}, {NULL}};
const Segment SGI113530[] = {{&S7485[44], 22}, {NULL}};
const Segment SGI113531[] = {{&S7485[30], 6}, {NULL}};
const Segment SGI113532[] = {{&S7485[31], 3}, {NULL}};
const Segment SGI113533[] = {{&S7485[80], 6}, {NULL}};
const Segment SGI113534[] = {{&S7488[44], 8}, {NULL}};
const Segment SGI113535[] = {{&S7488[44], 20}, {NULL}};
const Segment SGI113536[] = {{&S7488[30], 6}, {NULL}};
const Segment SGI113537[] = {{&S7488[31], 3}, {NULL}};
const Segment SGI113538[] = {{&S7488[78], 6}, {NULL}};
const Segment SGI113539[] = {{&S7490[44], 9}, {NULL}};
const Segment SGI113540[] = {{&S7490[44], 21}, {NULL}};
const Segment SGI113541[] = {{&S7490[30], 6}, {NULL}};
const Segment SGI113542[] = {{&S7490[31], 3}, {NULL}};
const Segment SGI113543[] = {{&S7490[79], 6}, {NULL}};
const Segment SGI113544[] = {{&S7491[44], 11}, {NULL}};
const Segment SGI113545[] = {{&S7491[44], 23}, {NULL}};
const Segment SGI113546[] = {{&S7491[30], 6}, {NULL}};
const Segment SGI113547[] = {{&S7491[31], 3}, {NULL}};
const Segment SGI113548[] = {{&S7491[81], 6}, {NULL}};
const Segment SGI113549[] = {{&S90131[44], 10}, {NULL}};
const Segment SGI113550[] = {{&S90131[44], 22}, {NULL}};
const Segment SGI113551[] = {{&S90131[30], 6}, {NULL}};
const Segment SGI113552[] = {{&S90131[80], 6}, {NULL}};
const Segment SGI113553[] = {{&S90131[107], 13}, {NULL}};
const Segment SGI113554[] = {{&S7492[38], 8}, {NULL}};
const Segment SGI113555[] = {{&S7492[31], 6}, {NULL}};
const Segment SGI113556[] = {{&S7493[44], 14}, {NULL}};
const Segment SGI113557[] = {{&S7493[44], 26}, {NULL}};
const Segment SGI113558[] = {{&S7493[30], 6}, {NULL}};
const Segment SGI113559[] = {{&S7493[31], 3}, {NULL}};
const Segment SGI113560[] = {{&S7493[84], 6}, {NULL}};
const Segment SGI113561[] = {{&S7494[44], 10}, {NULL}};
const Segment SGI113562[] = {{&S7494[44], 22}, {NULL}};
const Segment SGI113563[] = {{&S7494[30], 6}, {NULL}};
const Segment SGI113564[] = {{&S7494[31], 3}, {NULL}};
const Segment SGI113565[] = {{&S7494[80], 6}, {NULL}};
const Segment SGI113566[] = {{&S7495[38], 9}, {NULL}};
const Segment SGI113567[] = {{&S7495[31], 6}, {NULL}};
const Segment SGI113568[] = {{&S7496[44], 8}, {NULL}};
const Segment SGI113569[] = {{&S7496[44], 20}, {NULL}};
const Segment SGI113570[] = {{&S7496[30], 6}, {NULL}};
const Segment SGI113571[] = {{&S7496[31], 3}, {NULL}};
const Segment SGI113572[] = {{&S7496[78], 6}, {NULL}};
const Segment SGI113573[] = {{&S7498[44], 9}, {NULL}};
const Segment SGI113574[] = {{&S7498[44], 22}, {NULL}};
const Segment SGI113575[] = {{&S7498[30], 6}, {NULL}};
const Segment SGI113576[] = {{&S7498[31], 3}, {NULL}};
const Segment SGI113577[] = {{&S7498[80], 6}, {NULL}};
const Segment SGI113578[] = {{&S7499[44], 8}, {NULL}};
const Segment SGI113579[] = {{&S7499[44], 20}, {NULL}};
const Segment SGI113580[] = {{&S7499[30], 6}, {NULL}};
const Segment SGI113581[] = {{&S7499[31], 3}, {NULL}};
const Segment SGI113582[] = {{&S7499[78], 6}, {NULL}};
const Segment SGI113583[] = {{&S7500[44], 6}, {NULL}};
const Segment SGI113584[] = {{&S7500[44], 18}, {NULL}};
const Segment SGI113585[] = {{&S7500[30], 6}, {NULL}};
const Segment SGI113586[] = {{&S7500[31], 3}, {NULL}};
const Segment SGI113587[] = {{&S7500[76], 6}, {NULL}};
const Segment SGI113588[] = {{&S7501[44], 7}, {NULL}};
const Segment SGI113589[] = {{&S7501[44], 19}, {NULL}};
const Segment SGI113590[] = {{&S7501[30], 6}, {NULL}};
const Segment SGI113591[] = {{&S7501[31], 3}, {NULL}};
const Segment SGI113592[] = {{&S7501[77], 6}, {NULL}};
const Segment SGI113593[] = {{&S7502[44], 7}, {NULL}};
const Segment SGI113594[] = {{&S7502[44], 19}, {NULL}};
const Segment SGI113595[] = {{&S7502[30], 6}, {NULL}};
const Segment SGI113596[] = {{&S7502[31], 3}, {NULL}};
const Segment SGI113597[] = {{&S7502[77], 6}, {NULL}};
const Segment SGI113598[] = {{&S7503[44], 11}, {NULL}};
const Segment SGI113599[] = {{&S7503[44], 23}, {NULL}};
const Segment SGI113600[] = {{&S7503[30], 6}, {NULL}};
const Segment SGI113601[] = {{&S7503[31], 3}, {NULL}};
const Segment SGI113602[] = {{&S7503[81], 6}, {NULL}};
const Segment SGI113603[] = {{&S7506[44], 10}, {NULL}};
const Segment SGI113604[] = {{&S7506[44], 22}, {NULL}};
const Segment SGI113605[] = {{&S7506[30], 6}, {NULL}};
const Segment SGI113606[] = {{&S7506[31], 3}, {NULL}};
const Segment SGI113607[] = {{&S7506[80], 6}, {NULL}};
const Segment SGI113608[] = {{&S7508[44], 8}, {NULL}};
const Segment SGI113609[] = {{&S7508[44], 20}, {NULL}};
const Segment SGI113610[] = {{&S7508[30], 6}, {NULL}};
const Segment SGI113611[] = {{&S7508[31], 3}, {NULL}};
const Segment SGI113612[] = {{&S7508[78], 6}, {NULL}};
const Segment SGI113613[] = {{&S7509[44], 12}, {NULL}};
const Segment SGI113614[] = {{&S7509[44], 24}, {NULL}};
const Segment SGI113615[] = {{&S7509[30], 6}, {NULL}};
const Segment SGI113616[] = {{&S7509[31], 3}, {NULL}};
const Segment SGI113617[] = {{&S7509[82], 6}, {NULL}};
const Segment SGI113618[] = {{&S7510[44], 10}, {NULL}};
const Segment SGI113619[] = {{&S7510[44], 22}, {NULL}};
const Segment SGI113620[] = {{&S7510[30], 6}, {NULL}};
const Segment SGI113621[] = {{&S7510[31], 3}, {NULL}};
const Segment SGI113622[] = {{&S7510[80], 6}, {NULL}};
const Segment SGI113623[] = {{&S7512[44], 9}, {NULL}};
const Segment SGI113624[] = {{&S7512[44], 21}, {NULL}};
const Segment SGI113625[] = {{&S7512[30], 6}, {NULL}};
const Segment SGI113626[] = {{&S7512[31], 3}, {NULL}};
const Segment SGI113627[] = {{&S7512[79], 6}, {NULL}};
const Segment SGI113628[] = {{&S7514[44], 10}, {NULL}};
const Segment SGI113629[] = {{&S7514[44], 22}, {NULL}};
const Segment SGI113630[] = {{&S7514[30], 6}, {NULL}};
const Segment SGI113631[] = {{&S7514[31], 3}, {NULL}};
const Segment SGI113632[] = {{&S7514[80], 6}, {NULL}};
const Segment SGI113633[] = {{&S7515[44], 9}, {NULL}};
const Segment SGI113634[] = {{&S7515[44], 21}, {NULL}};
const Segment SGI113635[] = {{&S7515[30], 6}, {NULL}};
const Segment SGI113636[] = {{&S7515[31], 3}, {NULL}};
const Segment SGI113637[] = {{&S7515[79], 6}, {NULL}};
const Segment SGI113638[] = {{&S7517[44], 19}, {NULL}};
const Segment SGI113639[] = {{&S7517[44], 31}, {NULL}};
const Segment SGI113640[] = {{&S7517[30], 6}, {NULL}};
const Segment SGI113641[] = {{&S7517[31], 3}, {NULL}};
const Segment SGI113642[] = {{&S7517[89], 6}, {NULL}};
const Segment SGI113643[] = {{&S7519[44], 4}, {NULL}};
const Segment SGI113644[] = {{&S7519[44], 16}, {NULL}};
const Segment SGI113645[] = {{&S7519[30], 6}, {NULL}};
const Segment SGI113646[] = {{&S7519[31], 3}, {NULL}};
const Segment SGI113647[] = {{&S7519[74], 6}, {NULL}};
const Segment SGI113648[] = {{&S7520[44], 7}, {NULL}};
const Segment SGI113649[] = {{&S7520[44], 29}, {NULL}};
const Segment SGI113650[] = {{&S7520[30], 6}, {NULL}};
const Segment SGI113651[] = {{&S7520[31], 3}, {NULL}};
const Segment SGI113652[] = {{&S7520[87], 6}, {NULL}};
const Segment SGI113653[] = {{&S7521[44], 6}, {NULL}};
const Segment SGI113654[] = {{&S7521[44], 18}, {NULL}};
const Segment SGI113655[] = {{&S7521[30], 6}, {NULL}};
const Segment SGI113656[] = {{&S7521[31], 3}, {NULL}};
const Segment SGI113657[] = {{&S7521[76], 6}, {NULL}};
const Segment SGI113658[] = {{&S7523[44], 4}, {NULL}};
const Segment SGI113659[] = {{&S7523[44], 16}, {NULL}};
const Segment SGI113660[] = {{&S7523[30], 6}, {NULL}};
const Segment SGI113661[] = {{&S7523[31], 3}, {NULL}};
const Segment SGI113662[] = {{&S7523[74], 6}, {NULL}};
const Segment SGI113663[] = {{&S7524[44], 8}, {NULL}};
const Segment SGI113664[] = {{&S7524[44], 20}, {NULL}};
const Segment SGI113665[] = {{&S7524[30], 6}, {NULL}};
const Segment SGI113666[] = {{&S7524[31], 3}, {NULL}};
const Segment SGI113667[] = {{&S7524[78], 6}, {NULL}};
const Segment SGI113668[] = {{&S7525[44], 7}, {NULL}};
const Segment SGI113669[] = {{&S7525[44], 19}, {NULL}};
const Segment SGI113670[] = {{&S7525[30], 6}, {NULL}};
const Segment SGI113671[] = {{&S7525[31], 3}, {NULL}};
const Segment SGI113672[] = {{&S7525[77], 6}, {NULL}};
const Segment SGI113673[] = {{&S7526[44], 7}, {NULL}};
const Segment SGI113674[] = {{&S7526[44], 28}, {NULL}};
const Segment SGI113675[] = {{&S7526[30], 6}, {NULL}};
const Segment SGI113676[] = {{&S7526[31], 3}, {NULL}};
const Segment SGI113677[] = {{&S7526[86], 5}, {NULL}};
const Segment SGI113678[] = {{&S7527[44], 13}, {NULL}};
const Segment SGI113679[] = {{&S7527[44], 25}, {NULL}};
const Segment SGI113680[] = {{&S7527[30], 6}, {NULL}};
const Segment SGI113681[] = {{&S7527[31], 3}, {NULL}};
const Segment SGI113682[] = {{&S7527[83], 6}, {NULL}};
const Segment SGI113683[] = {{&S7528[44], 5}, {NULL}};
const Segment SGI113684[] = {{&S7528[44], 15}, {NULL}};
const Segment SGI113685[] = {{&S7528[30], 6}, {NULL}};
const Segment SGI113686[] = {{&S7528[31], 3}, {NULL}};
const Segment SGI113687[] = {{&S7528[73], 6}, {NULL}};
const Segment SGI113688[] = {{&S7529[44], 14}, {NULL}};
const Segment SGI113689[] = {{&S7529[44], 26}, {NULL}};
const Segment SGI113690[] = {{&S7529[30], 6}, {NULL}};
const Segment SGI113691[] = {{&S7529[31], 3}, {NULL}};
const Segment SGI113692[] = {{&S7529[84], 6}, {NULL}};
const Segment SGI113693[] = {{&S7531[44], 6}, {NULL}};
const Segment SGI113694[] = {{&S7531[44], 18}, {NULL}};
const Segment SGI113695[] = {{&S7531[30], 6}, {NULL}};
const Segment SGI113696[] = {{&S7531[31], 3}, {NULL}};
const Segment SGI113697[] = {{&S7531[76], 6}, {NULL}};
const Segment SGI113698[] = {{&S7532[44], 5}, {NULL}};
const Segment SGI113699[] = {{&S7532[44], 17}, {NULL}};
const Segment SGI113700[] = {{&S7532[30], 6}, {NULL}};
const Segment SGI113701[] = {{&S7532[31], 3}, {NULL}};
const Segment SGI113702[] = {{&S7532[75], 6}, {NULL}};
const Segment SGI113703[] = {{&S90210[44], 9}, {NULL}};
const Segment SGI113704[] = {{&S90210[44], 21}, {NULL}};
const Segment SGI113705[] = {{&S7533[44], 6}, {NULL}};
const Segment SGI113706[] = {{&S90210[30], 6}, {NULL}};
const Segment SGI113707[] = {{&S7533[44], 18}, {NULL}};
const Segment SGI113708[] = {{&S7533[30], 6}, {NULL}};
const Segment SGI113709[] = {{&S90210[31], 3}, {NULL}};
const Segment SGI113710[] = {{&S7533[31], 3}, {NULL}};
const Segment SGI113711[] = {{&S90210[79], 6}, {NULL}};
const Segment SGI113712[] = {{&S7533[76], 6}, {NULL}};
const Segment SGI113713[] = {{&S90211[44], 9}, {NULL}};
const Segment SGI113714[] = {{&S7534[44], 12}, {NULL}};
const Segment SGI113715[] = {{&S90211[44], 21}, {NULL}};
const Segment SGI113716[] = {{&S7534[44], 24}, {NULL}};
const Segment SGI113717[] = {{&S90211[30], 6}, {NULL}};
const Segment SGI113718[] = {{&S7534[30], 6}, {NULL}};
const Segment SGI113719[] = {{&S90211[31], 3}, {NULL}};
const Segment SGI113720[] = {{&S7534[31], 3}, {NULL}};
const Segment SGI113721[] = {{&S90211[79], 6}, {NULL}};
const Segment SGI113722[] = {{&S7534[82], 6}, {NULL}};
const Segment SGI113723[] = {{&S7535[44], 8}, {NULL}};
const Segment SGI113724[] = {{&S7535[44], 18}, {NULL}};
const Segment SGI113725[] = {{&S7535[30], 6}, {NULL}};
const Segment SGI113726[] = {{&S7535[31], 3}, {NULL}};
const Segment SGI113727[] = {{&S7535[76], 6}, {NULL}};
const Segment SGI113728[] = {{&S7537[44], 10}, {NULL}};
const Segment SGI113729[] = {{&S7537[44], 22}, {NULL}};
const Segment SGI113730[] = {{&S7537[30], 6}, {NULL}};
const Segment SGI113731[] = {{&S7537[31], 3}, {NULL}};
const Segment SGI113732[] = {{&S7537[80], 6}, {NULL}};
const Segment SGI113733[] = {{&S7538[44], 12}, {NULL}};
const Segment SGI113734[] = {{&S7538[44], 24}, {NULL}};
const Segment SGI113735[] = {{&S7538[30], 6}, {NULL}};
const Segment SGI113736[] = {{&S7538[31], 3}, {NULL}};
const Segment SGI113737[] = {{&S7538[82], 6}, {NULL}};
const Segment SGI113738[] = {{&S7539[44], 12}, {NULL}};
const Segment SGI113739[] = {{&S7539[44], 24}, {NULL}};
const Segment SGI113740[] = {{&S7539[30], 6}, {NULL}};
const Segment SGI113741[] = {{&S7539[31], 3}, {NULL}};
const Segment SGI113742[] = {{&S7539[82], 6}, {NULL}};
const Segment SGI113743[] = {{&S7541[44], 6}, {NULL}};
const Segment SGI113744[] = {{&S7541[44], 18}, {NULL}};
const Segment SGI113745[] = {{&S7541[30], 6}, {NULL}};
const Segment SGI113746[] = {{&S7541[31], 3}, {NULL}};
const Segment SGI113747[] = {{&S7541[76], 6}, {NULL}};
const Segment SGI113748[] = {{&S90228[44], 18}, {NULL}};
const Segment SGI113749[] = {{&S90228[44], 29}, {NULL}};
const Segment SGI113750[] = {{&S90228[30], 6}, {NULL}};
const Segment SGI113751[] = {{&S90228[31], 3}, {NULL}};
const Segment SGI113752[] = {{&S90228[87], 6}, {NULL}};
const Segment SGI113753[] = {{&S7544[44], 17}, {NULL}};
const Segment SGI113754[] = {{&S7544[44], 29}, {NULL}};
const Segment SGI113755[] = {{&S7544[30], 6}, {NULL}};
const Segment SGI113756[] = {{&S7544[31], 3}, {NULL}};
const Segment SGI113757[] = {{&S7544[87], 6}, {NULL}};
const Segment SGI113758[] = {{&S7545[44], 6}, {NULL}};
const Segment SGI113759[] = {{&S7545[44], 18}, {NULL}};
const Segment SGI113760[] = {{&S7545[30], 6}, {NULL}};
const Segment SGI113761[] = {{&S7545[31], 3}, {NULL}};
const Segment SGI113762[] = {{&S7545[76], 6}, {NULL}};
const Segment SGI113763[] = {{&S90232[34], 14}, {NULL}};
const Segment SGI113764[] = {{&S90232[27], 6}, {NULL}};
const Segment SGI113765[] = {{&S90232[101], 9}, {NULL}};
const Segment SGI113766[] = {{&S90232[74], 6}, {NULL}};
const Segment SGI113767[] = {{&S7546[57], 17}, {NULL}};
const Segment SGI113768[] = {{&S7546[44], 3}, {NULL}};
const Segment SGI113769[] = {{&S7546[87], 6}, {NULL}};
const Segment SGI113770[] = {{&S7547[44], 5}, {NULL}};
const Segment SGI113771[] = {{&S7547[44], 17}, {NULL}};
const Segment SGI113772[] = {{&S7547[30], 6}, {NULL}};
const Segment SGI113773[] = {{&S7547[31], 3}, {NULL}};
const Segment SGI113774[] = {{&S7547[75], 6}, {NULL}};
const Segment SGI113775[] = {{&S7549[44], 7}, {NULL}};
const Segment SGI113776[] = {{&S7549[44], 19}, {NULL}};
const Segment SGI113777[] = {{&S7549[30], 6}, {NULL}};
const Segment SGI113778[] = {{&S7549[31], 3}, {NULL}};
const Segment SGI113779[] = {{&S7549[77], 6}, {NULL}};
const Segment SGI113780[] = {{&S90241[34], 10}, {NULL}};
const Segment SGI113781[] = {{&S90241[27], 6}, {NULL}};
const Segment SGI113782[] = {{&S90241[110], 9}, {NULL}};
const Segment SGI113783[] = {{&S7551[44], 6}, {NULL}};
const Segment SGI113784[] = {{&S90241[83], 6}, {NULL}};
const Segment SGI113785[] = {{&S7551[44], 18}, {NULL}};
const Segment SGI113786[] = {{&S7551[30], 6}, {NULL}};
const Segment SGI113787[] = {{&S7551[31], 3}, {NULL}};
const Segment SGI113788[] = {{&S7551[76], 5}, {NULL}};
const Segment SGI113789[] = {{&S7552[44], 4}, {NULL}};
const Segment SGI113790[] = {{&S7552[44], 16}, {NULL}};
const Segment SGI113791[] = {{&S7552[30], 6}, {NULL}};
const Segment SGI113792[] = {{&S7552[31], 3}, {NULL}};
const Segment SGI113793[] = {{&S7552[74], 6}, {NULL}};
const Segment SGI113794[] = {{&S7553[44], 13}, {NULL}};
const Segment SGI113795[] = {{&S7553[44], 25}, {NULL}};
const Segment SGI113796[] = {{&S7553[30], 6}, {NULL}};
const Segment SGI113797[] = {{&S7553[31], 3}, {NULL}};
const Segment SGI113798[] = {{&S7553[83], 6}, {NULL}};
const Segment SGI113799[] = {{&S7556[44], 9}, {NULL}};
const Segment SGI113800[] = {{&S7556[44], 21}, {NULL}};
const Segment SGI113801[] = {{&S7556[30], 6}, {NULL}};
const Segment SGI113802[] = {{&S7556[31], 3}, {NULL}};
const Segment SGI113803[] = {{&S7556[79], 6}, {NULL}};
const Segment SGI113804[] = {{&S7557[44], 14}, {NULL}};
const Segment SGI113805[] = {{&S7557[44], 26}, {NULL}};
const Segment SGI113806[] = {{&S7557[30], 6}, {NULL}};
const Segment SGI113807[] = {{&S7557[31], 3}, {NULL}};
const Segment SGI113808[] = {{&S7557[84], 6}, {NULL}};
const Segment SGI113809[] = {{&S7559[44], 12}, {NULL}};
const Segment SGI113810[] = {{&S7559[44], 24}, {NULL}};
const Segment SGI113811[] = {{&S7559[30], 6}, {NULL}};
const Segment SGI113812[] = {{&S7559[31], 3}, {NULL}};
const Segment SGI113813[] = {{&S7559[82], 6}, {NULL}};
const Segment SGI113814[] = {{&S7560[44], 7}, {NULL}};
const Segment SGI113815[] = {{&S7560[44], 19}, {NULL}};
const Segment SGI113816[] = {{&S7560[30], 6}, {NULL}};
const Segment SGI113817[] = {{&S7560[31], 3}, {NULL}};
const Segment SGI113818[] = {{&S7560[77], 6}, {NULL}};
const Segment SGI113819[] = {{&S7561[44], 11}, {NULL}};
const Segment SGI113820[] = {{&S7561[44], 23}, {NULL}};
const Segment SGI113821[] = {{&S7561[30], 6}, {NULL}};
const Segment SGI113822[] = {{&S7561[31], 3}, {NULL}};
const Segment SGI113823[] = {{&S7561[81], 6}, {NULL}};
const Segment SGI113824[] = {{&S7562[44], 10}, {NULL}};
const Segment SGI113825[] = {{&S7562[44], 22}, {NULL}};
const Segment SGI113826[] = {{&S7562[30], 6}, {NULL}};
const Segment SGI113827[] = {{&S7562[31], 3}, {NULL}};
const Segment SGI113828[] = {{&S90261[44], 23}, {NULL}};
const Segment SGI113829[] = {{&S90261[44], 35}, {NULL}};
const Segment SGI113830[] = {{&S7562[80], 6}, {NULL}};
const Segment SGI113831[] = {{&S90261[30], 6}, {NULL}};
const Segment SGI113832[] = {{&S90261[31], 3}, {NULL}};
const Segment SGI113833[] = {{&S90261[93], 6}, {NULL}};
const Segment SGI113834[] = {{&S7563[44], 7}, {NULL}};
const Segment SGI113835[] = {{&S7563[44], 19}, {NULL}};
const Segment SGI113836[] = {{&S7563[30], 6}, {NULL}};
const Segment SGI113837[] = {{&S7563[31], 3}, {NULL}};
const Segment SGI113838[] = {{&S7563[77], 6}, {NULL}};
const Segment SGI113839[] = {{&S7564[44], 10}, {NULL}};
const Segment SGI113840[] = {{&S7564[44], 22}, {NULL}};
const Segment SGI113841[] = {{&S7564[30], 6}, {NULL}};
const Segment SGI113842[] = {{&S7564[31], 3}, {NULL}};
const Segment SGI113843[] = {{&S7564[80], 6}, {NULL}};
const Segment SGI113844[] = {{&S7565[44], 17}, {NULL}};
const Segment SGI113845[] = {{&S7565[44], 29}, {NULL}};
const Segment SGI113846[] = {{&S7565[30], 6}, {NULL}};
const Segment SGI113847[] = {{&S7565[31], 3}, {NULL}};
const Segment SGI113848[] = {{&S7565[87], 6}, {NULL}};
const Segment SGI113849[] = {{&S7567[42], 21}, {NULL}};
const Segment SGI113850[] = {{&S7567[30], 6}, {NULL}};
const Segment SGI113851[] = {{&S7567[31], 3}, {NULL}};
const Segment SGI113852[] = {{&S7567[89], 6}, {NULL}};
const Segment SGI113853[] = {{&S7569[44], 4}, {NULL}};
const Segment SGI113854[] = {{&S7569[44], 22}, {NULL}};
const Segment SGI113855[] = {{&S7569[30], 6}, {NULL}};
const Segment SGI113856[] = {{&S7569[31], 3}, {NULL}};
const Segment SGI113857[] = {{&S7569[80], 5}, {NULL}};
const Segment SGI113858[] = {{&S7571[44], 9}, {NULL}};
const Segment SGI113859[] = {{&S7571[44], 21}, {NULL}};
const Segment SGI113860[] = {{&S7571[30], 6}, {NULL}};
const Segment SGI113861[] = {{&S7571[31], 3}, {NULL}};
const Segment SGI113862[] = {{&S7571[79], 6}, {NULL}};
const Segment SGI113863[] = {{&S7573[44], 11}, {NULL}};
const Segment SGI113864[] = {{&S7573[44], 23}, {NULL}};
const Segment SGI113865[] = {{&S7573[30], 6}, {NULL}};
const Segment SGI113866[] = {{&S7573[31], 3}, {NULL}};
const Segment SGI113867[] = {{&S7573[81], 6}, {NULL}};
const Segment SGI113868[] = {{&S7574[44], 8}, {NULL}};
const Segment SGI113869[] = {{&S7574[44], 20}, {NULL}};
const Segment SGI113870[] = {{&S7574[30], 6}, {NULL}};
const Segment SGI113871[] = {{&S7574[31], 3}, {NULL}};
const Segment SGI113872[] = {{&S7574[78], 6}, {NULL}};
const Segment SGI113873[] = {{&S90285[34], 7}, {NULL}};
const Segment SGI113874[] = {{&S7575[44], 12}, {NULL}};
const Segment SGI113875[] = {{&S7575[44], 24}, {NULL}};
const Segment SGI113876[] = {{&S90285[27], 6}, {NULL}};
const Segment SGI113877[] = {{&S7575[30], 6}, {NULL}};
const Segment SGI113878[] = {{&S7575[31], 3}, {NULL}};
const Segment SGI113879[] = {{&S90285[94], 9}, {NULL}};
const Segment SGI113880[] = {{&S90285[67], 6}, {NULL}};
const Segment SGI113881[] = {{&S7575[82], 6}, {NULL}};
const Segment SGI113882[] = {{&S90286[34], 6}, {NULL}};
const Segment SGI113883[] = {{&S90286[27], 6}, {NULL}};
const Segment SGI113884[] = {{&S90286[93], 9}, {NULL}};
const Segment SGI113885[] = {{&S90286[66], 6}, {NULL}};
const Segment SGI113886[] = {{&S7577[44], 9}, {NULL}};
const Segment SGI113887[] = {{&S7577[44], 21}, {NULL}};
const Segment SGI113888[] = {{&S7577[30], 6}, {NULL}};
const Segment SGI113889[] = {{&S7577[31], 3}, {NULL}};
const Segment SGI113890[] = {{&S7577[79], 6}, {NULL}};
const Segment SGI113891[] = {{&S90287[34], 6}, {NULL}};
const Segment SGI113892[] = {{&S90287[27], 6}, {NULL}};
const Segment SGI113893[] = {{&S90287[93], 9}, {NULL}};
const Segment SGI113894[] = {{&S90287[66], 6}, {NULL}};
const Segment SGI113895[] = {{&S7578[44], 8}, {NULL}};
const Segment SGI113896[] = {{&S7578[44], 20}, {NULL}};
const Segment SGI113897[] = {{&S7578[30], 6}, {NULL}};
const Segment SGI113898[] = {{&S7578[31], 3}, {NULL}};
const Segment SGI113899[] = {{&S7578[78], 6}, {NULL}};
const Segment SGI113900[] = {{&S7579[44], 7}, {NULL}};
const Segment SGI113901[] = {{&S7579[44], 19}, {NULL}};
const Segment SGI113902[] = {{&S7579[30], 6}, {NULL}};
const Segment SGI113903[] = {{&S7579[31], 3}, {NULL}};
const Segment SGI113904[] = {{&S7579[77], 6}, {NULL}};
const Segment SGI113905[] = {{&S7580[44], 8}, {NULL}};
const Segment SGI113906[] = {{&S7580[44], 20}, {NULL}};
const Segment SGI113907[] = {{&S7580[30], 6}, {NULL}};
const Segment SGI113908[] = {{&S7580[31], 3}, {NULL}};
const Segment SGI113909[] = {{&S7580[78], 6}, {NULL}};
const Segment SGI113910[] = {{&S7582[44], 5}, {NULL}};
const Segment SGI113911[] = {{&S7582[44], 28}, {NULL}};
const Segment SGI113912[] = {{&S7582[30], 6}, {NULL}};
const Segment SGI113913[] = {{&S7582[31], 3}, {NULL}};
const Segment SGI113914[] = {{&S7582[86], 6}, {NULL}};
const Segment SGI113915[] = {{&S7583[44], 9}, {NULL}};
const Segment SGI113916[] = {{&S7583[44], 38}, {NULL}};
const Segment SGI113917[] = {{&S7583[30], 6}, {NULL}};
const Segment SGI113918[] = {{&S7583[31], 3}, {NULL}};
const Segment SGI113919[] = {{&S7583[96], 6}, {NULL}};
const Segment SGI113920[] = {{&S7585[44], 7}, {NULL}};
const Segment SGI113921[] = {{&S7585[44], 19}, {NULL}};
const Segment SGI113922[] = {{&S7585[30], 6}, {NULL}};
const Segment SGI113923[] = {{&S7585[31], 3}, {NULL}};
const Segment SGI113924[] = {{&S7585[77], 6}, {NULL}};
const Segment SGI113925[] = {{&S7586[44], 9}, {NULL}};
const Segment SGI113926[] = {{&S7586[44], 21}, {NULL}};
const Segment SGI113927[] = {{&S7586[30], 6}, {NULL}};
const Segment SGI113928[] = {{&S90305[34], 11}, {NULL}};
const Segment SGI113929[] = {{&S90305[27], 6}, {NULL}};
const Segment SGI113930[] = {{&S7586[31], 3}, {NULL}};
const Segment SGI113931[] = {{&S7586[79], 6}, {NULL}};
const Segment SGI113932[] = {{&S90305[98], 9}, {NULL}};
const Segment SGI113933[] = {{&S90305[71], 6}, {NULL}};
const Segment SGI113934[] = {{&S7587[44], 7}, {NULL}};
const Segment SGI113935[] = {{&S7587[44], 19}, {NULL}};
const Segment SGI113936[] = {{&S7587[30], 6}, {NULL}};
const Segment SGI113937[] = {{&S7587[31], 3}, {NULL}};
const Segment SGI113938[] = {{&S7587[77], 6}, {NULL}};
const Segment SGI113939[] = {{&S7589[44], 19}, {NULL}};
const Segment SGI113940[] = {{&S7589[30], 6}, {NULL}};
const Segment SGI113941[] = {{&S7589[31], 3}, {NULL}};
const Segment SGI113942[] = {{&S7589[94], 6}, {NULL}};
const Segment SGI113943[] = {{&S7590[44], 13}, {NULL}};
const Segment SGI113944[] = {{&S7590[44], 25}, {NULL}};
const Segment SGI113945[] = {{&S7590[30], 6}, {NULL}};
const Segment SGI113946[] = {{&S7590[31], 3}, {NULL}};
const Segment SGI113947[] = {{&S7590[83], 6}, {NULL}};
const Segment SGI113948[] = {{&S7591[44], 8}, {NULL}};
const Segment SGI113949[] = {{&S7591[44], 20}, {NULL}};
const Segment SGI113950[] = {{&S7591[30], 6}, {NULL}};
const Segment SGI113951[] = {{&S7591[31], 3}, {NULL}};
const Segment SGI113952[] = {{&S7591[78], 6}, {NULL}};
const Segment SGI113953[] = {{&S7592[44], 7}, {NULL}};
const Segment SGI113954[] = {{&S7592[44], 19}, {NULL}};
const Segment SGI113955[] = {{&S7592[30], 6}, {NULL}};
const Segment SGI113956[] = {{&S7592[31], 3}, {NULL}};
const Segment SGI113957[] = {{&S7592[77], 6}, {NULL}};
const Segment SGI113958[] = {{&S7593[44], 10}, {NULL}};
const Segment SGI113959[] = {{&S7593[44], 21}, {NULL}};
const Segment SGI113960[] = {{&S7593[30], 6}, {NULL}};
const Segment SGI113961[] = {{&S7593[31], 3}, {NULL}};
const Segment SGI113962[] = {{&S7593[79], 6}, {NULL}};
const Segment SGI113963[] = {{&S7594[44], 7}, {NULL}};
const Segment SGI113964[] = {{&S7594[44], 19}, {NULL}};
const Segment SGI113965[] = {{&S7594[30], 6}, {NULL}};
const Segment SGI113966[] = {{&S7594[31], 3}, {NULL}};
const Segment SGI113967[] = {{&S7594[77], 6}, {NULL}};
const Segment SGI113968[] = {{&S7596[44], 11}, {NULL}};
const Segment SGI113969[] = {{&S7596[44], 23}, {NULL}};
const Segment SGI113970[] = {{&S7596[30], 6}, {NULL}};
const Segment SGI113971[] = {{&S7596[31], 3}, {NULL}};
const Segment SGI113972[] = {{&S7596[81], 6}, {NULL}};
const Segment SGI113973[] = {{&S7597[38], 10}, {NULL}};
const Segment SGI113974[] = {{&S7597[31], 6}, {NULL}};
const Segment SGI113975[] = {{&S7598[44], 10}, {NULL}};
const Segment SGI113976[] = {{&S7598[44], 22}, {NULL}};
const Segment SGI113977[] = {{&S7598[30], 6}, {NULL}};
const Segment SGI113978[] = {{&S7598[83], 6}, {NULL}};
const Segment SGI113979[] = {{&S7599[44], 13}, {NULL}};
const Segment SGI113980[] = {{&S7599[44], 25}, {NULL}};
const Segment SGI113981[] = {{&S7599[30], 6}, {NULL}};
const Segment SGI113982[] = {{&S7599[31], 3}, {NULL}};
const Segment SGI113983[] = {{&S7599[83], 6}, {NULL}};
const Segment SGI113984[] = {{&S7601[44], 8}, {NULL}};
const Segment SGI113985[] = {{&S7601[44], 20}, {NULL}};
const Segment SGI113986[] = {{&S7601[30], 6}, {NULL}};
const Segment SGI113987[] = {{&S7601[31], 3}, {NULL}};
const Segment SGI113988[] = {{&S7601[78], 6}, {NULL}};
const Segment SGI113989[] = {{&S7602[44], 8}, {NULL}};
const Segment SGI113990[] = {{&S7602[44], 20}, {NULL}};
const Segment SGI113991[] = {{&S7602[30], 6}, {NULL}};
const Segment SGI113992[] = {{&S7602[31], 3}, {NULL}};
const Segment SGI113993[] = {{&S7602[78], 6}, {NULL}};
const Segment SGI113994[] = {{&S7603[44], 7}, {NULL}};
const Segment SGI113995[] = {{&S7603[44], 19}, {NULL}};
const Segment SGI113996[] = {{&S7603[30], 6}, {NULL}};
const Segment SGI113997[] = {{&S7603[31], 3}, {NULL}};
const Segment SGI113998[] = {{&S7603[77], 6}, {NULL}};
const Segment SGI113999[] = {{&S7604[44], 14}, {NULL}};
const Segment SGI114000[] = {{&S7604[44], 26}, {NULL}};
const Segment SGI114001[] = {{&S7604[30], 6}, {NULL}};
const Segment SGI114002[] = {{&S7604[31], 3}, {NULL}};
const Segment SGI114003[] = {{&S7604[84], 6}, {NULL}};
const Segment SGI114004[] = {{&S7607[44], 10}, {NULL}};
const Segment SGI114005[] = {{&S7607[44], 22}, {NULL}};
const Segment SGI114006[] = {{&S7607[30], 6}, {NULL}};
const Segment SGI114007[] = {{&S7607[83], 6}, {NULL}};
const Segment SGI114008[] = {{&S7609[44], 10}, {NULL}};
const Segment SGI114009[] = {{&S7609[44], 22}, {NULL}};
const Segment SGI114010[] = {{&S7609[30], 6}, {NULL}};
const Segment SGI114011[] = {{&S7609[31], 3}, {NULL}};
const Segment SGI114012[] = {{&S7609[80], 6}, {NULL}};
const Segment SGI114013[] = {{&S7610[44], 8}, {NULL}};
const Segment SGI114014[] = {{&S7610[44], 20}, {NULL}};
const Segment SGI114015[] = {{&S7610[30], 6}, {NULL}};
const Segment SGI114016[] = {{&S7610[31], 3}, {NULL}};
const Segment SGI114017[] = {{&S7610[78], 6}, {NULL}};
const Segment SGI114018[] = {{&S7611[44], 10}, {NULL}};
const Segment SGI114019[] = {{&S7611[44], 22}, {NULL}};
const Segment SGI114020[] = {{&S7611[30], 6}, {NULL}};
const Segment SGI114021[] = {{&S7611[31], 3}, {NULL}};
const Segment SGI114022[] = {{&S7611[80], 6}, {NULL}};
const Segment SGI114023[] = {{&S7612[44], 22}, {NULL}};
const Segment SGI114024[] = {{&S7612[30], 6}, {NULL}};
const Segment SGI114025[] = {{&S7612[31], 3}, {NULL}};
const Segment SGI114026[] = {{&S7612[98], 6}, {NULL}};
const Segment SGI114027[] = {{&S7613[44], 7}, {NULL}};
const Segment SGI114028[] = {{&S7613[44], 19}, {NULL}};
const Segment SGI114029[] = {{&S7613[30], 6}, {NULL}};
const Segment SGI114030[] = {{&S7613[31], 3}, {NULL}};
const Segment SGI114031[] = {{&S7613[77], 6}, {NULL}};
const Segment SGI114032[] = {{&S7614[44], 12}, {NULL}};
const Segment SGI114033[] = {{&S7614[44], 24}, {NULL}};
const Segment SGI114034[] = {{&S7614[30], 6}, {NULL}};
const Segment SGI114035[] = {{&S7614[31], 3}, {NULL}};
const Segment SGI114036[] = {{&S7614[82], 6}, {NULL}};
const Segment SGI114037[] = {{&S7615[44], 6}, {NULL}};
const Segment SGI114038[] = {{&S7615[44], 18}, {NULL}};
const Segment SGI114039[] = {{&S7615[30], 6}, {NULL}};
const Segment SGI114040[] = {{&S7615[31], 3}, {NULL}};
const Segment SGI114041[] = {{&S7615[76], 6}, {NULL}};
const Segment SGI114042[] = {{&S7616[44], 11}, {NULL}};
const Segment SGI114043[] = {{&S7616[44], 23}, {NULL}};
const Segment SGI114044[] = {{&S7616[30], 6}, {NULL}};
const Segment SGI114045[] = {{&S7616[31], 3}, {NULL}};
const Segment SGI114046[] = {{&S7616[81], 6}, {NULL}};
const Segment SGI114047[] = {{&S7617[44], 9}, {NULL}};
const Segment SGI114048[] = {{&S7617[44], 21}, {NULL}};
const Segment SGI114049[] = {{&S7617[30], 6}, {NULL}};
const Segment SGI114050[] = {{&S7617[31], 3}, {NULL}};
const Segment SGI114051[] = {{&S7617[79], 6}, {NULL}};
const Segment SGI114052[] = {{&S7619[44], 9}, {NULL}};
const Segment SGI114053[] = {{&S7619[44], 21}, {NULL}};
const Segment SGI114054[] = {{&S7619[30], 6}, {NULL}};
const Segment SGI114055[] = {{&S7619[31], 3}, {NULL}};
const Segment SGI114056[] = {{&S7619[79], 6}, {NULL}};
const Segment SGI114057[] = {{&S7620[44], 10}, {NULL}};
const Segment SGI114058[] = {{&S7620[44], 22}, {NULL}};
const Segment SGI114059[] = {{&S7620[30], 6}, {NULL}};
const Segment SGI114060[] = {{&S7620[31], 3}, {NULL}};
const Segment SGI114061[] = {{&S7620[80], 6}, {NULL}};
const Segment SGI114062[] = {{&S7621[44], 7}, {NULL}};
const Segment SGI114063[] = {{&S7621[44], 19}, {NULL}};
const Segment SGI114064[] = {{&S7621[30], 6}, {NULL}};
const Segment SGI114065[] = {{&S7621[31], 3}, {NULL}};
const Segment SGI114066[] = {{&S7621[77], 6}, {NULL}};
const Segment SGI114067[] = {{&S7622[44], 6}, {NULL}};
const Segment SGI114068[] = {{&S7622[44], 18}, {NULL}};
const Segment SGI114069[] = {{&S7622[30], 6}, {NULL}};
const Segment SGI114070[] = {{&S7622[31], 3}, {NULL}};
const Segment SGI114071[] = {{&S7622[76], 6}, {NULL}};
const Segment SGI114072[] = {{&S7624[44], 14}, {NULL}};
const Segment SGI114073[] = {{&S7624[44], 26}, {NULL}};
const Segment SGI114074[] = {{&S7624[30], 6}, {NULL}};
const Segment SGI114075[] = {{&S7624[31], 3}, {NULL}};
const Segment SGI114076[] = {{&S7624[84], 6}, {NULL}};
const Segment SGI114077[] = {{&S7625[44], 6}, {NULL}};
const Segment SGI114078[] = {{&S7625[44], 18}, {NULL}};
const Segment SGI114079[] = {{&S7625[30], 6}, {NULL}};
const Segment SGI114080[] = {{&S7625[31], 3}, {NULL}};
const Segment SGI114081[] = {{&S7625[76], 6}, {NULL}};
const Segment SGI114082[] = {{&S7626[44], 8}, {NULL}};
const Segment SGI114083[] = {{&S7626[44], 20}, {NULL}};
const Segment SGI114084[] = {{&S7626[30], 6}, {NULL}};
const Segment SGI114085[] = {{&S7626[31], 3}, {NULL}};
const Segment SGI114086[] = {{&S7626[78], 6}, {NULL}};
const Segment SGI114087[] = {{&S7627[44], 11}, {NULL}};
const Segment SGI114088[] = {{&S7627[44], 23}, {NULL}};
const Segment SGI114089[] = {{&S7627[30], 6}, {NULL}};
const Segment SGI114090[] = {{&S7627[31], 3}, {NULL}};
const Segment SGI114091[] = {{&S7627[81], 6}, {NULL}};
const Segment SGI114092[] = {{&S7628[44], 20}, {NULL}};
const Segment SGI114093[] = {{&S7628[30], 6}, {NULL}};
const Segment SGI114094[] = {{&S7628[31], 3}, {NULL}};
const Segment SGI114095[] = {{&S7628[96], 6}, {NULL}};
const Segment SGI114096[] = {{&S7629[44], 8}, {NULL}};
const Segment SGI114097[] = {{&S7629[44], 20}, {NULL}};
const Segment SGI114098[] = {{&S7629[30], 6}, {NULL}};
const Segment SGI114099[] = {{&S7629[31], 3}, {NULL}};
const Segment SGI114100[] = {{&S7629[78], 6}, {NULL}};
const Segment SGI114101[] = {{&S7630[44], 12}, {NULL}};
const Segment SGI114102[] = {{&S7630[44], 24}, {NULL}};
const Segment SGI114103[] = {{&S7630[30], 6}, {NULL}};
const Segment SGI114104[] = {{&S7630[31], 3}, {NULL}};
const Segment SGI114105[] = {{&S7630[82], 6}, {NULL}};
const Segment SGI114106[] = {{&S7631[44], 11}, {NULL}};
const Segment SGI114107[] = {{&S7631[44], 23}, {NULL}};
const Segment SGI114108[] = {{&S7631[30], 6}, {NULL}};
const Segment SGI114109[] = {{&S7631[31], 3}, {NULL}};
const Segment SGI114110[] = {{&S105576[42], 11}, {NULL}};
const Segment SGI114111[] = {{&S7631[81], 6}, {NULL}};
const Segment SGI114112[] = {{&S105576[42], 22}, {NULL}};
const Segment SGI114113[] = {{&S105576[30], 4}, {NULL}};
const Segment SGI114114[] = {{&S105576[113], 3}, {NULL}};
const Segment SGI114115[] = {{&S105576[78], 6}, {NULL}};
const Segment SGI114116[] = {{&S105576[131], 13}, {NULL}};
const Segment SGI114117[] = {{&S7632[44], 15}, {NULL}};
const Segment SGI114118[] = {{&S7632[44], 27}, {NULL}};
const Segment SGI114119[] = {{&S7632[30], 6}, {NULL}};
const Segment SGI114120[] = {{&S90398[27], 6}, {NULL}};
const Segment SGI114121[] = {{&S7632[31], 3}, {NULL}};
const Segment SGI114122[] = {{&S7632[85], 6}, {NULL}};
const Segment SGI114123[] = {{&S90398[106], 9}, {NULL}};
const Segment SGI114124[] = {{&S90398[79], 6}, {NULL}};
const Segment SGI114125[] = {{&S90399[34], 16}, {NULL}};
const Segment SGI114126[] = {{&S90399[27], 6}, {NULL}};
const Segment SGI114127[] = {{&S7634[44], 11}, {NULL}};
const Segment SGI114128[] = {{&S90399[103], 9}, {NULL}};
const Segment SGI114129[] = {{&S7634[44], 23}, {NULL}};
const Segment SGI114130[] = {{&S90399[76], 6}, {NULL}};
const Segment SGI114131[] = {{&S7634[30], 6}, {NULL}};
const Segment SGI114132[] = {{&S7634[31], 3}, {NULL}};
const Segment SGI114133[] = {{&S7634[81], 6}, {NULL}};
const Segment SGI114134[] = {{&S7635[44], 14}, {NULL}};
const Segment SGI114135[] = {{&S7635[44], 26}, {NULL}};
const Segment SGI114136[] = {{&S7635[30], 6}, {NULL}};
const Segment SGI114137[] = {{&S7635[31], 3}, {NULL}};
const Segment SGI114138[] = {{&S7635[84], 6}, {NULL}};
const Segment SGI114139[] = {{&S7636[44], 18}, {NULL}};
const Segment SGI114140[] = {{&S90403[34], 14}, {NULL}};
const Segment SGI114141[] = {{&S7636[44], 27}, {NULL}};
const Segment SGI114142[] = {{&S7636[30], 6}, {NULL}};
const Segment SGI114143[] = {{&S90403[27], 6}, {NULL}};
const Segment SGI114144[] = {{&S7636[31], 3}, {NULL}};
const Segment SGI114145[] = {{&S7636[85], 6}, {NULL}};
const Segment SGI114146[] = {{&S90403[106], 9}, {NULL}};
const Segment SGI114147[] = {{&S90403[79], 6}, {NULL}};
const Segment SGI114148[] = {{&S7637[44], 13}, {NULL}};
const Segment SGI114149[] = {{&S7637[44], 25}, {NULL}};
const Segment SGI114150[] = {{&S7637[30], 6}, {NULL}};
const Segment SGI114151[] = {{&S7637[31], 3}, {NULL}};
const Segment SGI114152[] = {{&S7637[83], 6}, {NULL}};
const Segment SGI114153[] = {{&S7638[44], 6}, {NULL}};
const Segment SGI114154[] = {{&S7638[44], 18}, {NULL}};
const Segment SGI114155[] = {{&S7638[30], 6}, {NULL}};
const Segment SGI114156[] = {{&S7638[31], 3}, {NULL}};
const Segment SGI114157[] = {{&S7638[76], 6}, {NULL}};
const Segment SGI114158[] = {{&S7639[44], 6}, {NULL}};
const Segment SGI114159[] = {{&S7639[44], 18}, {NULL}};
const Segment SGI114160[] = {{&S7639[30], 6}, {NULL}};
const Segment SGI114161[] = {{&S7639[31], 3}, {NULL}};
const Segment SGI114162[] = {{&S7639[76], 5}, {NULL}};
const Segment SGI114163[] = {{&S7640[44], 10}, {NULL}};
const Segment SGI114164[] = {{&S7640[44], 22}, {NULL}};
const Segment SGI114165[] = {{&S7640[30], 6}, {NULL}};
const Segment SGI114166[] = {{&S7640[31], 3}, {NULL}};
const Segment SGI114167[] = {{&S7640[80], 6}, {NULL}};
const Segment SGI114168[] = {{&S7641[44], 7}, {NULL}};
const Segment SGI114169[] = {{&S7641[44], 19}, {NULL}};
const Segment SGI114170[] = {{&S7641[30], 6}, {NULL}};
const Segment SGI114171[] = {{&S7641[31], 3}, {NULL}};
const Segment SGI114172[] = {{&S7641[77], 6}, {NULL}};
const Segment SGI114173[] = {{&S105595[43], 10}, {NULL}};
const Segment SGI114174[] = {{&S105595[30], 4}, {NULL}};
const Segment SGI114175[] = {{&S105595[112], 3}, {NULL}};
const Segment SGI114176[] = {{&S105595[78], 5}, {NULL}};
const Segment SGI114177[] = {{&S7642[44], 10}, {NULL}};
const Segment SGI114178[] = {{&S105595[136], 13}, {NULL}};
const Segment SGI114179[] = {{&S7642[44], 22}, {NULL}};
const Segment SGI114180[] = {{&S7642[30], 6}, {NULL}};
const Segment SGI114181[] = {{&S7642[31], 3}, {NULL}};
const Segment SGI114182[] = {{&S7642[80], 6}, {NULL}};
const Segment SGI114183[] = {{&S105596[43], 10}, {NULL}};
const Segment SGI114184[] = {{&S105596[30], 4}, {NULL}};
const Segment SGI114185[] = {{&S105596[113], 3}, {NULL}};
const Segment SGI114186[] = {{&S7643[44], 8}, {NULL}};
const Segment SGI114187[] = {{&S7643[44], 20}, {NULL}};
const Segment SGI114188[] = {{&S105596[79], 5}, {NULL}};
const Segment SGI114189[] = {{&S7643[30], 6}, {NULL}};
const Segment SGI114190[] = {{&S105596[137], 13}, {NULL}};
const Segment SGI114191[] = {{&S7643[31], 3}, {NULL}};
const Segment SGI114192[] = {{&S7643[78], 6}, {NULL}};
const Segment SGI114193[] = {{&S7644[44], 16}, {NULL}};
const Segment SGI114194[] = {{&S7644[44], 28}, {NULL}};
const Segment SGI114195[] = {{&S7644[30], 6}, {NULL}};
const Segment SGI114196[] = {{&S7644[31], 3}, {NULL}};
const Segment SGI114197[] = {{&S7644[86], 6}, {NULL}};
const Segment SGI114198[] = {{&S105599[43], 10}, {NULL}};
const Segment SGI114199[] = {{&S105599[30], 4}, {NULL}};
const Segment SGI114200[] = {{&S105599[113], 3}, {NULL}};
const Segment SGI114201[] = {{&S7645[44], 71}, {NULL}};
const Segment SGI114202[] = {{&S105599[79], 5}, {NULL}};
const Segment SGI114203[] = {{&S7645[44], 92}, {NULL}};
const Segment SGI114204[] = {{&S105599[137], 13}, {NULL}};
const Segment SGI114205[] = {{&S7645[30], 6}, {NULL}};
const Segment SGI114206[] = {{&S7645[31], 3}, {NULL}};
const Segment SGI114207[] = {{&S7645[109], 5}, {NULL}};
const Segment SGI114208[] = {{&S105600[42], 10}, {NULL}};
const Segment SGI114209[] = {{&S105600[42], 22}, {NULL}};
const Segment SGI114210[] = {{&S105600[30], 4}, {NULL}};
const Segment SGI114211[] = {{&S90423[34], 8}, {NULL}};
const Segment SGI114212[] = {{&S90423[27], 6}, {NULL}};
const Segment SGI114213[] = {{&S105600[113], 3}, {NULL}};
const Segment SGI114214[] = {{&S105600[78], 6}, {NULL}};
const Segment SGI114215[] = {{&S7646[44], 14}, {NULL}};
const Segment SGI114216[] = {{&S90423[95], 9}, {NULL}};
const Segment SGI114217[] = {{&S7646[44], 26}, {NULL}};
const Segment SGI114218[] = {{&S90423[68], 6}, {NULL}};
const Segment SGI114219[] = {{&S7646[30], 6}, {NULL}};
const Segment SGI114220[] = {{&S7646[31], 3}, {NULL}};
const Segment SGI114221[] = {{&S7646[84], 6}, {NULL}};
const Segment SGI114222[] = {{&S105601[42], 10}, {NULL}};
const Segment SGI114223[] = {{&S90424[34], 19}, {NULL}};
const Segment SGI114224[] = {{&S105601[42], 22}, {NULL}};
const Segment SGI114225[] = {{&S90424[27], 6}, {NULL}};
const Segment SGI114226[] = {{&S105601[30], 4}, {NULL}};
const Segment SGI114227[] = {{&S105601[120], 3}, {NULL}};
const Segment SGI114228[] = {{&S7647[44], 5}, {NULL}};
const Segment SGI114229[] = {{&S90424[106], 9}, {NULL}};
const Segment SGI114230[] = {{&S7647[44], 48}, {NULL}};
const Segment SGI114231[] = {{&S90424[79], 6}, {NULL}};
const Segment SGI114232[] = {{&S105601[85], 6}, {NULL}};
const Segment SGI114233[] = {{&S7647[30], 6}, {NULL}};
const Segment SGI114234[] = {{&S105601[64], 8}, {NULL}};
const Segment SGI114235[] = {{&S7647[31], 3}, {NULL}};
const Segment SGI114236[] = {{&S7647[106], 6}, {NULL}};
const Segment SGI114237[] = {{&S7648[44], 13}, {NULL}};
const Segment SGI114238[] = {{&S7648[44], 25}, {NULL}};
const Segment SGI114239[] = {{&S7648[30], 6}, {NULL}};
const Segment SGI114240[] = {{&S105603[43], 10}, {NULL}};
const Segment SGI114241[] = {{&S7648[31], 3}, {NULL}};
const Segment SGI114242[] = {{&S7648[83], 6}, {NULL}};
const Segment SGI114243[] = {{&S105603[30], 4}, {NULL}};
const Segment SGI114244[] = {{&S105603[113], 3}, {NULL}};
const Segment SGI114245[] = {{&S105603[79], 5}, {NULL}};
const Segment SGI114246[] = {{&S105603[137], 13}, {NULL}};
const Segment SGI114247[] = {{&S7649[44], 4}, {NULL}};
const Segment SGI114248[] = {{&S7649[44], 16}, {NULL}};
const Segment SGI114249[] = {{&S7649[30], 6}, {NULL}};
const Segment SGI114250[] = {{&S7649[31], 3}, {NULL}};
const Segment SGI114251[] = {{&S7649[74], 6}, {NULL}};
const Segment SGI114252[] = {{&S105604[57], 10}, {NULL}};
const Segment SGI114253[] = {{&S105604[57], 21}, {NULL}};
const Segment SGI114254[] = {{&S105604[45], 4}, {NULL}};
const Segment SGI114255[] = {{&S105604[127], 3}, {NULL}};
const Segment SGI114256[] = {{&S105604[92], 6}, {NULL}};
const Segment SGI114257[] = {{&S105604[0], 14}, {NULL}};
const Segment SGI114258[] = {{&S7650[44], 10}, {NULL}};
const Segment SGI114259[] = {{&S7650[30], 6}, {NULL}};
const Segment SGI114260[] = {{&S7650[85], 6}, {NULL}};
const Segment SGI114261[] = {{&S105606[29], 8}, {NULL}};
const Segment SGI114262[] = {{&S105606[53], 3}, {NULL}};
const Segment SGI114263[] = {{&S7653[44], 5}, {NULL}};
const Segment SGI114264[] = {{&S7653[44], 17}, {NULL}};
const Segment SGI114265[] = {{&S105607[60], 10}, {NULL}};
const Segment SGI114266[] = {{&S7653[30], 6}, {NULL}};
const Segment SGI114267[] = {{&S105607[55], 4}, {NULL}};
const Segment SGI114268[] = {{&S7653[31], 3}, {NULL}};
const Segment SGI114269[] = {{&S7653[75], 6}, {NULL}};
const Segment SGI114270[] = {{&S7654[44], 22}, {NULL}};
const Segment SGI114271[] = {{&S7654[30], 6}, {NULL}};
const Segment SGI114272[] = {{&S7654[31], 3}, {NULL}};
const Segment SGI114273[] = {{&S7654[98], 6}, {NULL}};
const Segment SGI114274[] = {{&S7655[44], 11}, {NULL}};
const Segment SGI114275[] = {{&S7655[44], 23}, {NULL}};
const Segment SGI114276[] = {{&S7655[30], 6}, {NULL}};
const Segment SGI114277[] = {{&S7655[31], 3}, {NULL}};
const Segment SGI114278[] = {{&S7655[81], 6}, {NULL}};
const Segment SGI114279[] = {{&S7656[44], 6}, {NULL}};
const Segment SGI114280[] = {{&S7656[44], 18}, {NULL}};
const Segment SGI114281[] = {{&S7656[30], 6}, {NULL}};
const Segment SGI114282[] = {{&S7656[31], 3}, {NULL}};
const Segment SGI114283[] = {{&S7656[76], 6}, {NULL}};
const Segment SGI114284[] = {{&S105614[57], 10}, {NULL}};
const Segment SGI114285[] = {{&S7657[44], 7}, {NULL}};
const Segment SGI114286[] = {{&S105614[57], 22}, {NULL}};
const Segment SGI114287[] = {{&S7657[44], 19}, {NULL}};
const Segment SGI114288[] = {{&S105614[45], 4}, {NULL}};
const Segment SGI114289[] = {{&S7657[30], 6}, {NULL}};
const Segment SGI114290[] = {{&S105614[128], 3}, {NULL}};
const Segment SGI114291[] = {{&S7657[31], 3}, {NULL}};
const Segment SGI114292[] = {{&S105614[93], 6}, {NULL}};
const Segment SGI114293[] = {{&S7657[77], 6}, {NULL}};
const Segment SGI114294[] = {{&S105614[0], 14}, {NULL}};
const Segment SGI114295[] = {{&S90445[34], 11}, {NULL}};
const Segment SGI114296[] = {{&S90445[27], 6}, {NULL}};
const Segment SGI114297[] = {{&S90445[98], 9}, {NULL}};
const Segment SGI114298[] = {{&S90445[71], 6}, {NULL}};
const Segment SGI114299[] = {{&S7658[44], 12}, {NULL}};
const Segment SGI114300[] = {{&S7658[44], 24}, {NULL}};
const Segment SGI114301[] = {{&S7658[30], 6}, {NULL}};
const Segment SGI114302[] = {{&S7658[31], 3}, {NULL}};
const Segment SGI114303[] = {{&S7658[82], 6}, {NULL}};
const Segment SGI114304[] = {{&S105617[42], 10}, {NULL}};
const Segment SGI114305[] = {{&S7659[44], 6}, {NULL}};
const Segment SGI114306[] = {{&S105617[42], 22}, {NULL}};
const Segment SGI114307[] = {{&S7659[44], 18}, {NULL}};
const Segment SGI114308[] = {{&S105617[30], 4}, {NULL}};
const Segment SGI114309[] = {{&S7659[30], 6}, {NULL}};
const Segment SGI114310[] = {{&S105617[157], 3}, {NULL}};
const Segment SGI114311[] = {{&S7659[31], 3}, {NULL}};
const Segment SGI114312[] = {{&S105617[123], 3}, {NULL}};
const Segment SGI114313[] = {{&S7659[76], 6}, {NULL}};
const Segment SGI114314[] = {{&S105617[88], 6}, {NULL}};
const Segment SGI114315[] = {{&S105617[143], 8}, {NULL}};
const Segment SGI114316[] = {{&S7660[44], 11}, {NULL}};
const Segment SGI114317[] = {{&S7660[44], 23}, {NULL}};
const Segment SGI114318[] = {{&S7660[30], 6}, {NULL}};
const Segment SGI114319[] = {{&S7660[31], 3}, {NULL}};
const Segment SGI114320[] = {{&S7660[81], 6}, {NULL}};
const Segment SGI114321[] = {{&S7661[44], 8}, {NULL}};
const Segment SGI114322[] = {{&S7661[44], 20}, {NULL}};
const Segment SGI114323[] = {{&S7661[30], 6}, {NULL}};
const Segment SGI114324[] = {{&S7661[31], 3}, {NULL}};
const Segment SGI114325[] = {{&S7661[78], 6}, {NULL}};
const Segment SGI114326[] = {{&S105620[42], 10}, {NULL}};
const Segment SGI114327[] = {{&S105620[42], 22}, {NULL}};
const Segment SGI114328[] = {{&S105620[30], 4}, {NULL}};
const Segment SGI114329[] = {{&S105620[113], 3}, {NULL}};
const Segment SGI114330[] = {{&S105620[78], 6}, {NULL}};
const Segment SGI114331[] = {{&S105620[131], 13}, {NULL}};
const Segment SGI114332[] = {{&S7662[44], 5}, {NULL}};
const Segment SGI114333[] = {{&S7662[44], 17}, {NULL}};
const Segment SGI114334[] = {{&S7662[30], 6}, {NULL}};
const Segment SGI114335[] = {{&S7662[31], 3}, {NULL}};
const Segment SGI114336[] = {{&S7662[75], 6}, {NULL}};
const Segment SGI114337[] = {{&S105621[55], 10}, {NULL}};
const Segment SGI114338[] = {{&S105621[55], 21}, {NULL}};
const Segment SGI114339[] = {{&S105621[43], 4}, {NULL}};
const Segment SGI114340[] = {{&S105621[125], 3}, {NULL}};
const Segment SGI114341[] = {{&S105621[90], 6}, {NULL}};
const Segment SGI114342[] = {{&S7663[44], 10}, {NULL}};
const Segment SGI114343[] = {{&S7663[44], 22}, {NULL}};
const Segment SGI114344[] = {{&S7663[30], 6}, {NULL}};
const Segment SGI114345[] = {{&S105621[0], 12}, {NULL}};
const Segment SGI114346[] = {{&S7663[31], 3}, {NULL}};
const Segment SGI114347[] = {{&S7663[80], 6}, {NULL}};
const Segment SGI114348[] = {{&S7665[44], 25}, {NULL}};
const Segment SGI114349[] = {{&S7665[44], 37}, {NULL}};
const Segment SGI114350[] = {{&S7665[30], 6}, {NULL}};
const Segment SGI114351[] = {{&S7665[31], 3}, {NULL}};
const Segment SGI114352[] = {{&S105624[57], 10}, {NULL}};
const Segment SGI114353[] = {{&S7665[95], 6}, {NULL}};
const Segment SGI114354[] = {{&S105624[57], 22}, {NULL}};
const Segment SGI114355[] = {{&S105624[45], 4}, {NULL}};
const Segment SGI114356[] = {{&S105624[128], 3}, {NULL}};
const Segment SGI114357[] = {{&S105624[93], 6}, {NULL}};
const Segment SGI114358[] = {{&S105624[0], 14}, {NULL}};
const Segment SGI114359[] = {{&S90462[44], 15}, {NULL}};
const Segment SGI114360[] = {{&S90462[44], 27}, {NULL}};
const Segment SGI114361[] = {{&S90462[30], 6}, {NULL}};
const Segment SGI114362[] = {{&S7668[44], 14}, {NULL}};
const Segment SGI114363[] = {{&S7668[44], 26}, {NULL}};
const Segment SGI114364[] = {{&S7668[30], 6}, {NULL}};
const Segment SGI114365[] = {{&S90462[85], 6}, {NULL}};
const Segment SGI114366[] = {{&S7668[31], 3}, {NULL}};
const Segment SGI114367[] = {{&S90462[112], 13}, {NULL}};
const Segment SGI114368[] = {{&S7668[84], 6}, {NULL}};
const Segment SGI114369[] = {{&S7669[44], 16}, {NULL}};
const Segment SGI114370[] = {{&S7669[44], 28}, {NULL}};
const Segment SGI114371[] = {{&S7669[30], 6}, {NULL}};
const Segment SGI114372[] = {{&S105628[57], 10}, {NULL}};
const Segment SGI114373[] = {{&S7669[31], 3}, {NULL}};
const Segment SGI114374[] = {{&S105628[57], 22}, {NULL}};
const Segment SGI114375[] = {{&S105628[45], 4}, {NULL}};
const Segment SGI114376[] = {{&S7669[86], 5}, {NULL}};
const Segment SGI114377[] = {{&S105628[128], 3}, {NULL}};
const Segment SGI114378[] = {{&S105628[93], 6}, {NULL}};
const Segment SGI114379[] = {{&S105628[0], 14}, {NULL}};
const Segment SGI114380[] = {{&S7670[44], 7}, {NULL}};
const Segment SGI114381[] = {{&S7670[44], 28}, {NULL}};
const Segment SGI114382[] = {{&S7670[30], 6}, {NULL}};
const Segment SGI114383[] = {{&S105629[60], 10}, {NULL}};
const Segment SGI114384[] = {{&S105629[55], 4}, {NULL}};
const Segment SGI114385[] = {{&S7670[89], 4}, {NULL}};
const Segment SGI114386[] = {{&S7670[89], 3}, {NULL}};
const Segment SGI114387[] = {{&S7672[44], 12}, {NULL}};
const Segment SGI114388[] = {{&S7672[44], 24}, {NULL}};
const Segment SGI114389[] = {{&S7672[30], 6}, {NULL}};
const Segment SGI114390[] = {{&S7672[31], 3}, {NULL}};
const Segment SGI114391[] = {{&S105631[57], 10}, {NULL}};
const Segment SGI114392[] = {{&S7672[82], 6}, {NULL}};
const Segment SGI114393[] = {{&S105631[57], 21}, {NULL}};
const Segment SGI114394[] = {{&S105631[45], 4}, {NULL}};
const Segment SGI114395[] = {{&S105631[127], 3}, {NULL}};
const Segment SGI114396[] = {{&S105631[92], 6}, {NULL}};
const Segment SGI114397[] = {{&S105631[0], 14}, {NULL}};
const Segment SGI114398[] = {{&S7673[44], 9}, {NULL}};
const Segment SGI114399[] = {{&S7673[44], 25}, {NULL}};
const Segment SGI114400[] = {{&S7673[30], 6}, {NULL}};
const Segment SGI114401[] = {{&S7673[31], 3}, {NULL}};
const Segment SGI114402[] = {{&S7673[83], 6}, {NULL}};
const Segment SGI114403[] = {{&S7674[44], 14}, {NULL}};
const Segment SGI114404[] = {{&S7674[44], 26}, {NULL}};
const Segment SGI114405[] = {{&S7674[30], 6}, {NULL}};
const Segment SGI114406[] = {{&S7674[87], 6}, {NULL}};
const Segment SGI114407[] = {{&S7676[44], 11}, {NULL}};
const Segment SGI114408[] = {{&S7676[44], 23}, {NULL}};
const Segment SGI114409[] = {{&S7676[30], 6}, {NULL}};
const Segment SGI114410[] = {{&S7676[31], 3}, {NULL}};
const Segment SGI114411[] = {{&S7676[81], 6}, {NULL}};
const Segment SGI114412[] = {{&S7677[44], 6}, {NULL}};
const Segment SGI114413[] = {{&S7677[44], 18}, {NULL}};
const Segment SGI114414[] = {{&S7677[30], 6}, {NULL}};
const Segment SGI114415[] = {{&S105637[43], 10}, {NULL}};
const Segment SGI114416[] = {{&S105637[30], 4}, {NULL}};
const Segment SGI114417[] = {{&S7677[31], 3}, {NULL}};
const Segment SGI114418[] = {{&S7677[76], 6}, {NULL}};
const Segment SGI114419[] = {{&S105637[113], 3}, {NULL}};
const Segment SGI114420[] = {{&S105637[79], 5}, {NULL}};
const Segment SGI114421[] = {{&S105637[137], 13}, {NULL}};
const Segment SGI114422[] = {{&S7678[44], 10}, {NULL}};
const Segment SGI114423[] = {{&S7678[44], 22}, {NULL}};
const Segment SGI114424[] = {{&S7678[30], 6}, {NULL}};
const Segment SGI114425[] = {{&S7678[31], 3}, {NULL}};
const Segment SGI114426[] = {{&S7678[80], 6}, {NULL}};
const Segment SGI114427[] = {{&S7680[44], 14}, {NULL}};
const Segment SGI114428[] = {{&S7680[44], 26}, {NULL}};
const Segment SGI114429[] = {{&S7680[30], 6}, {NULL}};
const Segment SGI114430[] = {{&S7680[31], 3}, {NULL}};
const Segment SGI114431[] = {{&S7680[84], 6}, {NULL}};
const Segment SGI114432[] = {{&S105641[42], 10}, {NULL}};
const Segment SGI114433[] = {{&S105641[42], 22}, {NULL}};
const Segment SGI114434[] = {{&S105641[30], 4}, {NULL}};
const Segment SGI114435[] = {{&S7681[44], 84}, {NULL}};
const Segment SGI114436[] = {{&S105641[113], 3}, {NULL}};
const Segment SGI114437[] = {{&S7681[44], 96}, {NULL}};
const Segment SGI114438[] = {{&S7681[30], 6}, {NULL}};
const Segment SGI114439[] = {{&S105641[78], 6}, {NULL}};
const Segment SGI114440[] = {{&S105641[131], 13}, {NULL}};
const Segment SGI114441[] = {{&S7681[31], 3}, {NULL}};
const Segment SGI114442[] = {{&S7681[121], 6}, {NULL}};
const Segment SGI114443[] = {{&S90486[34], 15}, {NULL}};
const Segment SGI114444[] = {{&S90486[27], 6}, {NULL}};
const Segment SGI114445[] = {{&S90486[102], 9}, {NULL}};
const Segment SGI114446[] = {{&S90486[75], 6}, {NULL}};
const Segment SGI114447[] = {{&S7682[44], 6}, {NULL}};
const Segment SGI114448[] = {{&S7682[44], 18}, {NULL}};
const Segment SGI114449[] = {{&S7682[30], 6}, {NULL}};
const Segment SGI114450[] = {{&S7682[31], 3}, {NULL}};
const Segment SGI114451[] = {{&S7682[76], 6}, {NULL}};
const Segment SGI114452[] = {{&S7683[44], 7}, {NULL}};
const Segment SGI114453[] = {{&S7683[44], 19}, {NULL}};
const Segment SGI114454[] = {{&S7683[30], 6}, {NULL}};
const Segment SGI114455[] = {{&S7683[31], 3}, {NULL}};
const Segment SGI114456[] = {{&S7683[77], 6}, {NULL}};
const Segment SGI114457[] = {{&S7684[44], 10}, {NULL}};
const Segment SGI114458[] = {{&S7684[44], 22}, {NULL}};
const Segment SGI114459[] = {{&S7684[30], 6}, {NULL}};
const Segment SGI114460[] = {{&S7684[31], 3}, {NULL}};
const Segment SGI114461[] = {{&S7684[80], 6}, {NULL}};
const Segment SGI114462[] = {{&S7686[44], 7}, {NULL}};
const Segment SGI114463[] = {{&S7686[44], 19}, {NULL}};
const Segment SGI114464[] = {{&S7686[30], 6}, {NULL}};
const Segment SGI114465[] = {{&S7686[31], 3}, {NULL}};
const Segment SGI114466[] = {{&S7686[77], 6}, {NULL}};
const Segment SGI114467[] = {{&S7687[44], 7}, {NULL}};
const Segment SGI114468[] = {{&S7687[44], 19}, {NULL}};
const Segment SGI114469[] = {{&S7687[30], 6}, {NULL}};
const Segment SGI114470[] = {{&S7687[31], 3}, {NULL}};
const Segment SGI114471[] = {{&S7687[77], 6}, {NULL}};
const Segment SGI114472[] = {{&S105651[57], 10}, {NULL}};
const Segment SGI114473[] = {{&S105651[57], 21}, {NULL}};
const Segment SGI114474[] = {{&S105651[45], 4}, {NULL}};
const Segment SGI114475[] = {{&S105651[127], 3}, {NULL}};
const Segment SGI114476[] = {{&S105651[92], 6}, {NULL}};
const Segment SGI114477[] = {{&S105651[0], 14}, {NULL}};
const Segment SGI114478[] = {{&S7688[44], 8}, {NULL}};
const Segment SGI114479[] = {{&S7688[44], 20}, {NULL}};
const Segment SGI114480[] = {{&S7688[30], 6}, {NULL}};
const Segment SGI114481[] = {{&S7688[31], 3}, {NULL}};
const Segment SGI114482[] = {{&S7688[78], 6}, {NULL}};
const Segment SGI114483[] = {{&S7689[44], 8}, {NULL}};
const Segment SGI114484[] = {{&S7689[44], 20}, {NULL}};
const Segment SGI114485[] = {{&S7689[30], 6}, {NULL}};
const Segment SGI114486[] = {{&S7689[31], 3}, {NULL}};
const Segment SGI114487[] = {{&S7689[78], 6}, {NULL}};
const Segment SGI114488[] = {{&S7690[44], 9}, {NULL}};
const Segment SGI114489[] = {{&S7690[44], 21}, {NULL}};
const Segment SGI114490[] = {{&S7690[30], 6}, {NULL}};
const Segment SGI114491[] = {{&S7690[31], 3}, {NULL}};
const Segment SGI114492[] = {{&S7690[79], 6}, {NULL}};
const Segment SGI114493[] = {{&S7691[44], 8}, {NULL}};
const Segment SGI114494[] = {{&S7691[44], 20}, {NULL}};
const Segment SGI114495[] = {{&S105656[57], 10}, {NULL}};
const Segment SGI114496[] = {{&S7691[30], 6}, {NULL}};
const Segment SGI114497[] = {{&S105656[57], 22}, {NULL}};
const Segment SGI114498[] = {{&S105656[45], 4}, {NULL}};
const Segment SGI114499[] = {{&S7691[31], 3}, {NULL}};
const Segment SGI114500[] = {{&S7691[78], 6}, {NULL}};
const Segment SGI114501[] = {{&S105656[128], 3}, {NULL}};
const Segment SGI114502[] = {{&S105656[93], 6}, {NULL}};
const Segment SGI114503[] = {{&S105656[0], 14}, {NULL}};
const Segment SGI114504[] = {{&S7692[44], 6}, {NULL}};
const Segment SGI114505[] = {{&S7692[44], 18}, {NULL}};
const Segment SGI114506[] = {{&S7692[30], 6}, {NULL}};
const Segment SGI114507[] = {{&S7692[31], 3}, {NULL}};
const Segment SGI114508[] = {{&S7692[76], 6}, {NULL}};
const Segment SGI114509[] = {{&S105658[57], 10}, {NULL}};
const Segment SGI114510[] = {{&S105658[57], 21}, {NULL}};
const Segment SGI114511[] = {{&S105658[45], 4}, {NULL}};
const Segment SGI114512[] = {{&S105658[127], 3}, {NULL}};
const Segment SGI114513[] = {{&S105658[92], 6}, {NULL}};
const Segment SGI114514[] = {{&S105658[0], 14}, {NULL}};
const Segment SGI114515[] = {{&S7694[44], 5}, {NULL}};
const Segment SGI114516[] = {{&S7694[44], 17}, {NULL}};
const Segment SGI114517[] = {{&S7694[30], 6}, {NULL}};
const Segment SGI114518[] = {{&S7694[31], 3}, {NULL}};
const Segment SGI114519[] = {{&S7694[75], 6}, {NULL}};
const Segment SGI114520[] = {{&S7695[44], 13}, {NULL}};
const Segment SGI114521[] = {{&S7695[44], 25}, {NULL}};
const Segment SGI114522[] = {{&S7695[30], 6}, {NULL}};
const Segment SGI114523[] = {{&S7695[31], 3}, {NULL}};
const Segment SGI114524[] = {{&S7695[83], 6}, {NULL}};
const Segment SGI114525[] = {{&S7696[44], 5}, {NULL}};
const Segment SGI114526[] = {{&S7696[44], 30}, {NULL}};
const Segment SGI114527[] = {{&S7696[30], 6}, {NULL}};
const Segment SGI114528[] = {{&S7696[31], 3}, {NULL}};
const Segment SGI114529[] = {{&S7696[88], 6}, {NULL}};
const Segment SGI114530[] = {{&S7697[44], 8}, {NULL}};
const Segment SGI114531[] = {{&S7697[44], 20}, {NULL}};
const Segment SGI114532[] = {{&S7697[30], 6}, {NULL}};
const Segment SGI114533[] = {{&S7697[31], 3}, {NULL}};
const Segment SGI114534[] = {{&S7697[78], 6}, {NULL}};
const Segment SGI114535[] = {{&S7698[44], 22}, {NULL}};
const Segment SGI114536[] = {{&S7698[44], 34}, {NULL}};
const Segment SGI114537[] = {{&S7698[30], 6}, {NULL}};
const Segment SGI114538[] = {{&S7698[31], 3}, {NULL}};
const Segment SGI114539[] = {{&S7698[92], 6}, {NULL}};
const Segment SGI114540[] = {{&S7699[44], 9}, {NULL}};
const Segment SGI114541[] = {{&S7699[44], 21}, {NULL}};
const Segment SGI114542[] = {{&S7699[30], 6}, {NULL}};
const Segment SGI114543[] = {{&S7699[31], 3}, {NULL}};
const Segment SGI114544[] = {{&S7699[79], 6}, {NULL}};
const Segment SGI114545[] = {{&S7700[44], 15}, {NULL}};
const Segment SGI114546[] = {{&S7700[44], 26}, {NULL}};
const Segment SGI114547[] = {{&S7700[30], 6}, {NULL}};
const Segment SGI114548[] = {{&S7700[31], 3}, {NULL}};
const Segment SGI114549[] = {{&S7700[84], 6}, {NULL}};
const Segment SGI114550[] = {{&S7701[44], 7}, {NULL}};
const Segment SGI114551[] = {{&S7701[44], 19}, {NULL}};
const Segment SGI114552[] = {{&S7701[30], 6}, {NULL}};
const Segment SGI114553[] = {{&S7701[31], 3}, {NULL}};
const Segment SGI114554[] = {{&S7701[77], 5}, {NULL}};
const Segment SGI114555[] = {{&S105673[42], 10}, {NULL}};
const Segment SGI114556[] = {{&S105673[42], 22}, {NULL}};
const Segment SGI114557[] = {{&S105673[30], 4}, {NULL}};
const Segment SGI114558[] = {{&S105673[113], 3}, {NULL}};
const Segment SGI114559[] = {{&S105673[78], 6}, {NULL}};
const Segment SGI114560[] = {{&S105673[131], 13}, {NULL}};
const Segment SGI114561[] = {{&S7702[44], 7}, {NULL}};
const Segment SGI114562[] = {{&S7702[44], 19}, {NULL}};
const Segment SGI114563[] = {{&S7702[30], 6}, {NULL}};
const Segment SGI114564[] = {{&S7702[31], 3}, {NULL}};
const Segment SGI114565[] = {{&S7702[77], 6}, {NULL}};
const Segment SGI114566[] = {{&S90526[34], 21}, {NULL}};
const Segment SGI114567[] = {{&S90526[27], 6}, {NULL}};
const Segment SGI114568[] = {{&S90526[108], 9}, {NULL}};
const Segment SGI114569[] = {{&S7703[44], 10}, {NULL}};
const Segment SGI114570[] = {{&S90526[81], 6}, {NULL}};
const Segment SGI114571[] = {{&S7703[44], 22}, {NULL}};
const Segment SGI114572[] = {{&S7703[30], 6}, {NULL}};
const Segment SGI114573[] = {{&S90526[149], 8}, {NULL}};
const Segment SGI114574[] = {{&S7703[31], 3}, {NULL}};
const Segment SGI114575[] = {{&S105675[57], 10}, {NULL}};
const Segment SGI114576[] = {{&S105675[57], 22}, {NULL}};
const Segment SGI114577[] = {{&S7703[80], 6}, {NULL}};
const Segment SGI114578[] = {{&S105675[45], 4}, {NULL}};
const Segment SGI114579[] = {{&S105675[128], 3}, {NULL}};
const Segment SGI114580[] = {{&S105675[93], 6}, {NULL}};
const Segment SGI114581[] = {{&S105675[0], 14}, {NULL}};
const Segment SGI114582[] = {{&S7704[44], 14}, {NULL}};
const Segment SGI114583[] = {{&S7704[44], 26}, {NULL}};
const Segment SGI114584[] = {{&S7704[30], 6}, {NULL}};
const Segment SGI114585[] = {{&S7704[31], 3}, {NULL}};
const Segment SGI114586[] = {{&S90528[44], 10}, {NULL}};
const Segment SGI114587[] = {{&S7704[84], 6}, {NULL}};
const Segment SGI114588[] = {{&S90528[44], 22}, {NULL}};
const Segment SGI114589[] = {{&S90528[30], 6}, {NULL}};
const Segment SGI114590[] = {{&S90528[80], 6}, {NULL}};
const Segment SGI114591[] = {{&S90528[107], 13}, {NULL}};
const Segment SGI114592[] = {{&S7705[44], 6}, {NULL}};
const Segment SGI114593[] = {{&S7705[44], 18}, {NULL}};
const Segment SGI114594[] = {{&S7705[30], 6}, {NULL}};
const Segment SGI114595[] = {{&S7705[31], 3}, {NULL}};
const Segment SGI114596[] = {{&S7705[76], 6}, {NULL}};
const Segment SGI114597[] = {{&S7707[44], 15}, {NULL}};
const Segment SGI114598[] = {{&S7707[44], 27}, {NULL}};
const Segment SGI114599[] = {{&S7707[30], 6}, {NULL}};
const Segment SGI114600[] = {{&S7707[31], 3}, {NULL}};
const Segment SGI114601[] = {{&S7707[85], 6}, {NULL}};
const Segment SGI114602[] = {{&S7708[44], 5}, {NULL}};
const Segment SGI114603[] = {{&S7708[44], 17}, {NULL}};
const Segment SGI114604[] = {{&S7708[30], 6}, {NULL}};
const Segment SGI114605[] = {{&S7708[31], 3}, {NULL}};
const Segment SGI114606[] = {{&S7708[75], 6}, {NULL}};
const Segment SGI114607[] = {{&S7709[44], 8}, {NULL}};
const Segment SGI114608[] = {{&S7709[44], 20}, {NULL}};
const Segment SGI114609[] = {{&S7709[30], 6}, {NULL}};
const Segment SGI114610[] = {{&S105684[43], 10}, {NULL}};
const Segment SGI114611[] = {{&S105684[30], 4}, {NULL}};
const Segment SGI114612[] = {{&S7709[31], 3}, {NULL}};
const Segment SGI114613[] = {{&S7709[78], 6}, {NULL}};
const Segment SGI114614[] = {{&S105684[113], 3}, {NULL}};
const Segment SGI114615[] = {{&S105684[79], 5}, {NULL}};
const Segment SGI114616[] = {{&S105684[137], 13}, {NULL}};
const Segment SGI114617[] = {{&S7710[44], 9}, {NULL}};
const Segment SGI114618[] = {{&S7710[44], 38}, {NULL}};
const Segment SGI114619[] = {{&S7710[30], 6}, {NULL}};
const Segment SGI114620[] = {{&S7710[31], 3}, {NULL}};
const Segment SGI114621[] = {{&S7710[96], 6}, {NULL}};
const Segment SGI114622[] = {{&S7716[44], 6}, {NULL}};
const Segment SGI114623[] = {{&S7716[44], 18}, {NULL}};
const Segment SGI114624[] = {{&S7716[30], 6}, {NULL}};
const Segment SGI114625[] = {{&S7716[31], 3}, {NULL}};
const Segment SGI114626[] = {{&S105691[43], 10}, {NULL}};
const Segment SGI114627[] = {{&S7716[76], 6}, {NULL}};
const Segment SGI114628[] = {{&S105691[30], 4}, {NULL}};
const Segment SGI114629[] = {{&S105691[113], 3}, {NULL}};
const Segment SGI114630[] = {{&S105691[79], 5}, {NULL}};
const Segment SGI114631[] = {{&S105691[137], 13}, {NULL}};
const Segment SGI114632[] = {{&S7719[44], 6}, {NULL}};
const Segment SGI114633[] = {{&S7719[44], 18}, {NULL}};
const Segment SGI114634[] = {{&S7719[30], 6}, {NULL}};
const Segment SGI114635[] = {{&S7719[31], 3}, {NULL}};
const Segment SGI114636[] = {{&S7719[76], 6}, {NULL}};
const Segment SGI114637[] = {{&S7721[44], 7}, {NULL}};
const Segment SGI114638[] = {{&S7721[44], 19}, {NULL}};
const Segment SGI114639[] = {{&S7721[30], 6}, {NULL}};
const Segment SGI114640[] = {{&S7721[31], 3}, {NULL}};
const Segment SGI114641[] = {{&S105696[56], 10}, {NULL}};
const Segment SGI114642[] = {{&S7721[77], 6}, {NULL}};
const Segment SGI114643[] = {{&S105696[56], 22}, {NULL}};
const Segment SGI114644[] = {{&S105696[44], 4}, {NULL}};
const Segment SGI114645[] = {{&S105696[127], 3}, {NULL}};
const Segment SGI114646[] = {{&S105696[92], 6}, {NULL}};
const Segment SGI114647[] = {{&S7722[38], 5}, {NULL}};
const Segment SGI114648[] = {{&S7722[31], 6}, {NULL}};
const Segment SGI114649[] = {{&S105699[57], 10}, {NULL}};
const Segment SGI114650[] = {{&S105699[57], 21}, {NULL}};
const Segment SGI114651[] = {{&S7724[44], 8}, {NULL}};
const Segment SGI114652[] = {{&S105699[45], 4}, {NULL}};
const Segment SGI114653[] = {{&S7724[44], 20}, {NULL}};
const Segment SGI114654[] = {{&S7724[30], 6}, {NULL}};
const Segment SGI114655[] = {{&S105699[127], 3}, {NULL}};
const Segment SGI114656[] = {{&S7724[31], 3}, {NULL}};
const Segment SGI114657[] = {{&S105699[92], 6}, {NULL}};
const Segment SGI114658[] = {{&S7724[78], 6}, {NULL}};
const Segment SGI114659[] = {{&S105699[0], 14}, {NULL}};
const Segment SGI114660[] = {{&S7726[44], 10}, {NULL}};
const Segment SGI114661[] = {{&S7726[44], 22}, {NULL}};
const Segment SGI114662[] = {{&S7726[30], 6}, {NULL}};
const Segment SGI114663[] = {{&S7726[31], 3}, {NULL}};
const Segment SGI114664[] = {{&S7726[80], 6}, {NULL}};
const Segment SGI114665[] = {{&S105703[57], 10}, {NULL}};
const Segment SGI114666[] = {{&S105703[57], 22}, {NULL}};
const Segment SGI114667[] = {{&S105703[45], 4}, {NULL}};
const Segment SGI114668[] = {{&S105703[128], 3}, {NULL}};
const Segment SGI114669[] = {{&S105703[93], 6}, {NULL}};
const Segment SGI114670[] = {{&S105703[0], 14}, {NULL}};
const Segment SGI114671[] = {{&S7728[44], 13}, {NULL}};
const Segment SGI114672[] = {{&S7728[44], 25}, {NULL}};
const Segment SGI114673[] = {{&S7728[30], 6}, {NULL}};
const Segment SGI114674[] = {{&S7728[31], 3}, {NULL}};
const Segment SGI114675[] = {{&S7728[83], 6}, {NULL}};
const Segment SGI114676[] = {{&S7729[44], 13}, {NULL}};
const Segment SGI114677[] = {{&S7729[44], 25}, {NULL}};
const Segment SGI114678[] = {{&S90569[34], 9}, {NULL}};
const Segment SGI114679[] = {{&S7729[30], 6}, {NULL}};
const Segment SGI114680[] = {{&S90569[27], 6}, {NULL}};
const Segment SGI114681[] = {{&S7729[31], 3}, {NULL}};
const Segment SGI114682[] = {{&S7729[83], 6}, {NULL}};
const Segment SGI114683[] = {{&S90569[96], 9}, {NULL}};
const Segment SGI114684[] = {{&S90569[69], 6}, {NULL}};
const Segment SGI114685[] = {{&S7730[44], 9}, {NULL}};
const Segment SGI114686[] = {{&S7730[44], 21}, {NULL}};
const Segment SGI114687[] = {{&S7730[30], 6}, {NULL}};
const Segment SGI114688[] = {{&S7730[31], 3}, {NULL}};
const Segment SGI114689[] = {{&S7730[79], 6}, {NULL}};
const Segment SGI114690[] = {{&S7731[44], 6}, {NULL}};
const Segment SGI114691[] = {{&S7731[44], 18}, {NULL}};
const Segment SGI114692[] = {{&S7731[30], 6}, {NULL}};
const Segment SGI114693[] = {{&S7731[31], 3}, {NULL}};
const Segment SGI114694[] = {{&S7731[76], 6}, {NULL}};
const Segment SGI114695[] = {{&S7732[44], 9}, {NULL}};
const Segment SGI114696[] = {{&S7732[44], 21}, {NULL}};
const Segment SGI114697[] = {{&S7732[30], 6}, {NULL}};
const Segment SGI114698[] = {{&S7732[31], 3}, {NULL}};
const Segment SGI114699[] = {{&S7732[79], 6}, {NULL}};
const Segment SGI114700[] = {{&S7733[44], 6}, {NULL}};
const Segment SGI114701[] = {{&S7733[44], 18}, {NULL}};
const Segment SGI114702[] = {{&S7733[30], 6}, {NULL}};
const Segment SGI114703[] = {{&S7733[31], 3}, {NULL}};
const Segment SGI114704[] = {{&S7733[76], 6}, {NULL}};
const Segment SGI114705[] = {{&S7734[44], 8}, {NULL}};
const Segment SGI114706[] = {{&S7734[44], 20}, {NULL}};
const Segment SGI114707[] = {{&S7734[30], 6}, {NULL}};
const Segment SGI114708[] = {{&S7734[31], 3}, {NULL}};
const Segment SGI114709[] = {{&S7734[78], 6}, {NULL}};
const Segment SGI114710[] = {{&S7736[44], 16}, {NULL}};
const Segment SGI114711[] = {{&S7736[44], 28}, {NULL}};
const Segment SGI114712[] = {{&S7736[30], 6}, {NULL}};
const Segment SGI114713[] = {{&S7736[31], 3}, {NULL}};
const Segment SGI114714[] = {{&S7736[86], 6}, {NULL}};
const Segment SGI114715[] = {{&S105719[55], 10}, {NULL}};
const Segment SGI114716[] = {{&S105719[55], 21}, {NULL}};
const Segment SGI114717[] = {{&S105719[43], 4}, {NULL}};
const Segment SGI114718[] = {{&S7737[44], 13}, {NULL}};
const Segment SGI114719[] = {{&S7737[44], 25}, {NULL}};
const Segment SGI114720[] = {{&S105719[125], 3}, {NULL}};
const Segment SGI114721[] = {{&S7737[30], 6}, {NULL}};
const Segment SGI114722[] = {{&S105719[90], 6}, {NULL}};
const Segment SGI114723[] = {{&S7737[31], 3}, {NULL}};
const Segment SGI114724[] = {{&S105719[0], 12}, {NULL}};
const Segment SGI114725[] = {{&S7737[83], 6}, {NULL}};
const Segment SGI114726[] = {{&S105720[55], 10}, {NULL}};
const Segment SGI114727[] = {{&S105720[55], 21}, {NULL}};
const Segment SGI114728[] = {{&S105720[43], 4}, {NULL}};
const Segment SGI114729[] = {{&S7738[44], 13}, {NULL}};
const Segment SGI114730[] = {{&S7738[44], 25}, {NULL}};
const Segment SGI114731[] = {{&S105720[125], 3}, {NULL}};
const Segment SGI114732[] = {{&S7738[30], 6}, {NULL}};
const Segment SGI114733[] = {{&S105720[90], 6}, {NULL}};
const Segment SGI114734[] = {{&S105720[0], 12}, {NULL}};
const Segment SGI114735[] = {{&S7738[31], 3}, {NULL}};
const Segment SGI114736[] = {{&S7738[83], 6}, {NULL}};
const Segment SGI114737[] = {{&S7739[44], 11}, {NULL}};
const Segment SGI114738[] = {{&S7739[44], 23}, {NULL}};
const Segment SGI114739[] = {{&S7739[30], 6}, {NULL}};
const Segment SGI114740[] = {{&S7739[31], 3}, {NULL}};
const Segment SGI114741[] = {{&S7739[81], 6}, {NULL}};
const Segment SGI114742[] = {{&S7740[44], 8}, {NULL}};
const Segment SGI114743[] = {{&S7740[44], 20}, {NULL}};
const Segment SGI114744[] = {{&S7740[30], 6}, {NULL}};
const Segment SGI114745[] = {{&S7740[31], 3}, {NULL}};
const Segment SGI114746[] = {{&S7740[78], 6}, {NULL}};
const Segment SGI114747[] = {{&S7741[44], 11}, {NULL}};
const Segment SGI114748[] = {{&S7741[44], 23}, {NULL}};
const Segment SGI114749[] = {{&S7741[30], 6}, {NULL}};
const Segment SGI114750[] = {{&S7741[31], 3}, {NULL}};
const Segment SGI114751[] = {{&S7741[81], 6}, {NULL}};
const Segment SGI114752[] = {{&S7742[44], 6}, {NULL}};
const Segment SGI114753[] = {{&S7742[44], 18}, {NULL}};
const Segment SGI114754[] = {{&S7742[30], 6}, {NULL}};
const Segment SGI114755[] = {{&S7742[31], 3}, {NULL}};
const Segment SGI114756[] = {{&S7742[76], 6}, {NULL}};
const Segment SGI114757[] = {{&S7744[44], 8}, {NULL}};
const Segment SGI114758[] = {{&S7744[44], 20}, {NULL}};
const Segment SGI114759[] = {{&S7744[30], 6}, {NULL}};
const Segment SGI114760[] = {{&S7744[31], 3}, {NULL}};
const Segment SGI114761[] = {{&S7744[78], 6}, {NULL}};
const Segment SGI114762[] = {{&S105731[42], 10}, {NULL}};
const Segment SGI114763[] = {{&S105731[42], 22}, {NULL}};
const Segment SGI114764[] = {{&S105731[30], 4}, {NULL}};
const Segment SGI114765[] = {{&S105731[112], 3}, {NULL}};
const Segment SGI114766[] = {{&S105731[78], 5}, {NULL}};
const Segment SGI114767[] = {{&S105731[123], 14}, {NULL}};
const Segment SGI114768[] = {{&S7745[44], 8}, {NULL}};
const Segment SGI114769[] = {{&S7745[44], 20}, {NULL}};
const Segment SGI114770[] = {{&S7745[30], 6}, {NULL}};
const Segment SGI114771[] = {{&S7745[31], 3}, {NULL}};
const Segment SGI114772[] = {{&S7745[78], 6}, {NULL}};
const Segment SGI114773[] = {{&S105733[42], 10}, {NULL}};
const Segment SGI114774[] = {{&S105733[42], 22}, {NULL}};
const Segment SGI114775[] = {{&S105733[30], 4}, {NULL}};
const Segment SGI114776[] = {{&S105733[112], 3}, {NULL}};
const Segment SGI114777[] = {{&S105733[78], 5}, {NULL}};
const Segment SGI114778[] = {{&S105733[123], 14}, {NULL}};
const Segment SGI114779[] = {{&S7747[44], 7}, {NULL}};
const Segment SGI114780[] = {{&S7747[44], 19}, {NULL}};
const Segment SGI114781[] = {{&S7747[30], 6}, {NULL}};
const Segment SGI114782[] = {{&S7747[31], 3}, {NULL}};
const Segment SGI114783[] = {{&S7747[77], 6}, {NULL}};
const Segment SGI114784[] = {{&S7748[38], 7}, {NULL}};
const Segment SGI114785[] = {{&S7748[31], 6}, {NULL}};
const Segment SGI114786[] = {{&S7750[44], 10}, {NULL}};
const Segment SGI114787[] = {{&S7750[44], 22}, {NULL}};
const Segment SGI114788[] = {{&S7750[30], 6}, {NULL}};
const Segment SGI114789[] = {{&S7750[31], 3}, {NULL}};
const Segment SGI114790[] = {{&S7750[80], 6}, {NULL}};
const Segment SGI114791[] = {{&S7751[44], 9}, {NULL}};
const Segment SGI114792[] = {{&S7751[44], 21}, {NULL}};
const Segment SGI114793[] = {{&S7751[30], 6}, {NULL}};
const Segment SGI114794[] = {{&S7751[31], 3}, {NULL}};
const Segment SGI114795[] = {{&S7751[79], 6}, {NULL}};
const Segment SGI114796[] = {{&S90612[44], 10}, {NULL}};
const Segment SGI114797[] = {{&S90612[44], 22}, {NULL}};
const Segment SGI114798[] = {{&S105741[57], 10}, {NULL}};
const Segment SGI114799[] = {{&S90612[30], 6}, {NULL}};
const Segment SGI114800[] = {{&S105741[57], 22}, {NULL}};
const Segment SGI114801[] = {{&S105741[45], 4}, {NULL}};
const Segment SGI114802[] = {{&S7752[44], 9}, {NULL}};
const Segment SGI114803[] = {{&S7752[44], 21}, {NULL}};
const Segment SGI114804[] = {{&S105741[128], 3}, {NULL}};
const Segment SGI114805[] = {{&S7752[30], 6}, {NULL}};
const Segment SGI114806[] = {{&S90612[80], 6}, {NULL}};
const Segment SGI114807[] = {{&S105741[93], 6}, {NULL}};
const Segment SGI114808[] = {{&S90612[107], 13}, {NULL}};
const Segment SGI114809[] = {{&S7752[31], 3}, {NULL}};
const Segment SGI114810[] = {{&S105741[0], 14}, {NULL}};
const Segment SGI114811[] = {{&S7752[79], 6}, {NULL}};
const Segment SGI114812[] = {{&S7754[44], 39}, {NULL}};
const Segment SGI114813[] = {{&S7754[30], 6}, {NULL}};
const Segment SGI114814[] = {{&S7754[115], 6}, {NULL}};
const Segment SGI114815[] = {{&S105746[57], 10}, {NULL}};
const Segment SGI114816[] = {{&S105746[57], 22}, {NULL}};
const Segment SGI114817[] = {{&S105746[45], 4}, {NULL}};
const Segment SGI114818[] = {{&S105746[128], 3}, {NULL}};
const Segment SGI114819[] = {{&S105746[93], 6}, {NULL}};
const Segment SGI114820[] = {{&S105746[0], 14}, {NULL}};
const Segment SGI114821[] = {{&S7756[44], 5}, {NULL}};
const Segment SGI114822[] = {{&S7756[44], 17}, {NULL}};
const Segment SGI114823[] = {{&S7756[30], 6}, {NULL}};
const Segment SGI114824[] = {{&S7756[31], 3}, {NULL}};
const Segment SGI114825[] = {{&S7756[75], 6}, {NULL}};
const Segment SGI114826[] = {{&S7758[44], 9}, {NULL}};
const Segment SGI114827[] = {{&S7758[44], 21}, {NULL}};
const Segment SGI114828[] = {{&S7758[30], 6}, {NULL}};
const Segment SGI114829[] = {{&S7758[31], 3}, {NULL}};
const Segment SGI114830[] = {{&S7758[79], 6}, {NULL}};
const Segment SGI114831[] = {{&S7759[44], 10}, {NULL}};
const Segment SGI114832[] = {{&S7759[44], 22}, {NULL}};
const Segment SGI114833[] = {{&S7759[30], 6}, {NULL}};
const Segment SGI114834[] = {{&S7759[31], 3}, {NULL}};
const Segment SGI114835[] = {{&S7759[80], 6}, {NULL}};
const Segment SGI114836[] = {{&S7760[44], 11}, {NULL}};
const Segment SGI114837[] = {{&S7760[44], 23}, {NULL}};
const Segment SGI114838[] = {{&S7760[30], 6}, {NULL}};
const Segment SGI114839[] = {{&S105752[57], 10}, {NULL}};
const Segment SGI114840[] = {{&S105752[57], 21}, {NULL}};
const Segment SGI114841[] = {{&S105752[45], 4}, {NULL}};
const Segment SGI114842[] = {{&S7760[31], 3}, {NULL}};
const Segment SGI114843[] = {{&S7760[81], 6}, {NULL}};
const Segment SGI114844[] = {{&S105752[127], 3}, {NULL}};
const Segment SGI114845[] = {{&S105752[92], 6}, {NULL}};
const Segment SGI114846[] = {{&S105752[0], 14}, {NULL}};
const Segment SGI114847[] = {{&S7761[44], 12}, {NULL}};
const Segment SGI114848[] = {{&S7761[44], 24}, {NULL}};
const Segment SGI114849[] = {{&S7761[30], 6}, {NULL}};
const Segment SGI114850[] = {{&S7761[31], 3}, {NULL}};
const Segment SGI114851[] = {{&S7761[82], 6}, {NULL}};
const Segment SGI114852[] = {{&S7762[44], 9}, {NULL}};
const Segment SGI114853[] = {{&S7762[44], 31}, {NULL}};
const Segment SGI114854[] = {{&S7762[30], 6}, {NULL}};
const Segment SGI114855[] = {{&S7762[31], 3}, {NULL}};
const Segment SGI114856[] = {{&S7762[89], 6}, {NULL}};
const Segment SGI114857[] = {{&S7763[44], 9}, {NULL}};
const Segment SGI114858[] = {{&S7763[44], 21}, {NULL}};
const Segment SGI114859[] = {{&S7763[30], 6}, {NULL}};
const Segment SGI114860[] = {{&S7763[31], 3}, {NULL}};
const Segment SGI114861[] = {{&S7763[79], 6}, {NULL}};
const Segment SGI114862[] = {{&S7764[44], 11}, {NULL}};
const Segment SGI114863[] = {{&S7764[44], 23}, {NULL}};
const Segment SGI114864[] = {{&S7764[30], 6}, {NULL}};
const Segment SGI114865[] = {{&S7764[31], 3}, {NULL}};
const Segment SGI114866[] = {{&S7764[81], 6}, {NULL}};
const Segment SGI114867[] = {{&S7765[44], 9}, {NULL}};
const Segment SGI114868[] = {{&S7765[44], 21}, {NULL}};
const Segment SGI114869[] = {{&S7765[30], 6}, {NULL}};
const Segment SGI114870[] = {{&S7765[31], 3}, {NULL}};
const Segment SGI114871[] = {{&S7765[79], 6}, {NULL}};
const Segment SGI114872[] = {{&S105762[57], 10}, {NULL}};
const Segment SGI114873[] = {{&S105762[57], 22}, {NULL}};
const Segment SGI114874[] = {{&S105762[45], 4}, {NULL}};
const Segment SGI114875[] = {{&S105762[128], 3}, {NULL}};
const Segment SGI114876[] = {{&S105762[93], 6}, {NULL}};
const Segment SGI114877[] = {{&S105762[0], 14}, {NULL}};
const Segment SGI114878[] = {{&S7766[44], 10}, {NULL}};
const Segment SGI114879[] = {{&S7766[44], 22}, {NULL}};
const Segment SGI114880[] = {{&S7766[30], 6}, {NULL}};
const Segment SGI114881[] = {{&S7766[31], 3}, {NULL}};
const Segment SGI114882[] = {{&S7766[80], 6}, {NULL}};
const Segment SGI114883[] = {{&S7767[44], 11}, {NULL}};
const Segment SGI114884[] = {{&S7767[44], 23}, {NULL}};
const Segment SGI114885[] = {{&S7767[30], 6}, {NULL}};
const Segment SGI114886[] = {{&S7767[31], 3}, {NULL}};
const Segment SGI114887[] = {{&S7767[81], 6}, {NULL}};
const Segment SGI114888[] = {{&S7769[44], 8}, {NULL}};
const Segment SGI114889[] = {{&S7769[44], 20}, {NULL}};
const Segment SGI114890[] = {{&S7769[30], 6}, {NULL}};
const Segment SGI114891[] = {{&S7769[31], 3}, {NULL}};
const Segment SGI114892[] = {{&S7769[78], 6}, {NULL}};
const Segment SGI114893[] = {{&S7770[44], 6}, {NULL}};
const Segment SGI114894[] = {{&S7770[44], 18}, {NULL}};
const Segment SGI114895[] = {{&S7770[30], 6}, {NULL}};
const Segment SGI114896[] = {{&S7770[31], 3}, {NULL}};
const Segment SGI114897[] = {{&S7770[76], 6}, {NULL}};
const Segment SGI114898[] = {{&S7772[44], 12}, {NULL}};
const Segment SGI114899[] = {{&S7772[44], 33}, {NULL}};
const Segment SGI114900[] = {{&S7772[30], 6}, {NULL}};
const Segment SGI114901[] = {{&S7772[31], 3}, {NULL}};
const Segment SGI114902[] = {{&S7772[91], 6}, {NULL}};
const Segment SGI114903[] = {{&S7773[44], 13}, {NULL}};
const Segment SGI114904[] = {{&S7773[44], 25}, {NULL}};
const Segment SGI114905[] = {{&S7773[30], 6}, {NULL}};
const Segment SGI114906[] = {{&S7773[31], 3}, {NULL}};
const Segment SGI114907[] = {{&S7773[83], 6}, {NULL}};
const Segment SGI114908[] = {{&S7774[44], 18}, {NULL}};
const Segment SGI114909[] = {{&S7774[44], 30}, {NULL}};
const Segment SGI114910[] = {{&S7774[30], 6}, {NULL}};
const Segment SGI114911[] = {{&S7774[31], 3}, {NULL}};
const Segment SGI114912[] = {{&S7774[88], 6}, {NULL}};
const Segment SGI114913[] = {{&S7775[44], 9}, {NULL}};
const Segment SGI114914[] = {{&S7775[44], 21}, {NULL}};
const Segment SGI114915[] = {{&S7775[30], 6}, {NULL}};
const Segment SGI114916[] = {{&S7775[31], 3}, {NULL}};
const Segment SGI114917[] = {{&S7775[79], 6}, {NULL}};
const Segment SGI114918[] = {{&S105779[57], 10}, {NULL}};
const Segment SGI114919[] = {{&S105779[57], 22}, {NULL}};
const Segment SGI114920[] = {{&S105779[45], 4}, {NULL}};
const Segment SGI114921[] = {{&S105779[128], 3}, {NULL}};
const Segment SGI114922[] = {{&S105779[93], 6}, {NULL}};
const Segment SGI114923[] = {{&S105779[0], 14}, {NULL}};
const Segment SGI114924[] = {{&S7778[44], 12}, {NULL}};
const Segment SGI114925[] = {{&S7778[44], 34}, {NULL}};
const Segment SGI114926[] = {{&S7778[30], 6}, {NULL}};
const Segment SGI114927[] = {{&S7778[31], 3}, {NULL}};
const Segment SGI114928[] = {{&S7778[92], 6}, {NULL}};
const Segment SGI114929[] = {{&S7779[44], 8}, {NULL}};
const Segment SGI114930[] = {{&S7779[44], 20}, {NULL}};
const Segment SGI114931[] = {{&S7779[30], 6}, {NULL}};
const Segment SGI114932[] = {{&S7779[31], 3}, {NULL}};
const Segment SGI114933[] = {{&S7779[78], 6}, {NULL}};
const Segment SGI114934[] = {{&S105783[42], 10}, {NULL}};
const Segment SGI114935[] = {{&S105783[42], 22}, {NULL}};
const Segment SGI114936[] = {{&S105783[30], 4}, {NULL}};
const Segment SGI114937[] = {{&S105783[111], 3}, {NULL}};
const Segment SGI114938[] = {{&S105783[78], 5}, {NULL}};
const Segment SGI114939[] = {{&S105783[115], 14}, {NULL}};
const Segment SGI114940[] = {{&S7780[44], 8}, {NULL}};
const Segment SGI114941[] = {{&S7780[44], 16}, {NULL}};
const Segment SGI114942[] = {{&S7780[30], 6}, {NULL}};
const Segment SGI114943[] = {{&S7780[31], 3}, {NULL}};
const Segment SGI114944[] = {{&S7780[74], 6}, {NULL}};
const Segment SGI114945[] = {{&S105784[57], 10}, {NULL}};
const Segment SGI114946[] = {{&S105784[57], 22}, {NULL}};
const Segment SGI114947[] = {{&S105784[45], 4}, {NULL}};
const Segment SGI114948[] = {{&S105784[128], 3}, {NULL}};
const Segment SGI114949[] = {{&S105784[93], 6}, {NULL}};
const Segment SGI114950[] = {{&S105784[0], 14}, {NULL}};
const Segment SGI114951[] = {{&S7781[44], 5}, {NULL}};
const Segment SGI114952[] = {{&S7781[44], 26}, {NULL}};
const Segment SGI114953[] = {{&S7781[30], 6}, {NULL}};
const Segment SGI114954[] = {{&S7781[31], 3}, {NULL}};
const Segment SGI114955[] = {{&S7781[84], 5}, {NULL}};
const Segment SGI114956[] = {{&S105785[57], 10}, {NULL}};
const Segment SGI114957[] = {{&S105785[57], 21}, {NULL}};
const Segment SGI114958[] = {{&S105785[45], 4}, {NULL}};
const Segment SGI114959[] = {{&S105785[127], 3}, {NULL}};
const Segment SGI114960[] = {{&S105785[92], 6}, {NULL}};
const Segment SGI114961[] = {{&S7782[44], 10}, {NULL}};
const Segment SGI114962[] = {{&S7782[44], 22}, {NULL}};
const Segment SGI114963[] = {{&S105785[0], 14}, {NULL}};
const Segment SGI114964[] = {{&S7782[30], 6}, {NULL}};
const Segment SGI114965[] = {{&S7782[31], 3}, {NULL}};
const Segment SGI114966[] = {{&S7782[80], 6}, {NULL}};
const Segment SGI114967[] = {{&S7783[44], 11}, {NULL}};
const Segment SGI114968[] = {{&S7783[44], 23}, {NULL}};
const Segment SGI114969[] = {{&S7783[30], 6}, {NULL}};
const Segment SGI114970[] = {{&S7783[31], 3}, {NULL}};
const Segment SGI114971[] = {{&S105787[29], 8}, {NULL}};
const Segment SGI114972[] = {{&S105787[53], 3}, {NULL}};
const Segment SGI114973[] = {{&S7783[81], 6}, {NULL}};
const Segment SGI114974[] = {{&S7784[44], 7}, {NULL}};
const Segment SGI114975[] = {{&S7784[44], 19}, {NULL}};
const Segment SGI114976[] = {{&S7784[30], 6}, {NULL}};
const Segment SGI114977[] = {{&S7784[31], 3}, {NULL}};
const Segment SGI114978[] = {{&S7784[77], 6}, {NULL}};
const Segment SGI114979[] = {{&S105789[42], 10}, {NULL}};
const Segment SGI114980[] = {{&S105789[42], 21}, {NULL}};
const Segment SGI114981[] = {{&S105789[30], 4}, {NULL}};
const Segment SGI114982[] = {{&S105789[112], 3}, {NULL}};
const Segment SGI114983[] = {{&S105789[77], 6}, {NULL}};
const Segment SGI114984[] = {{&S105789[130], 11}, {NULL}};
const Segment SGI114985[] = {{&S7786[42], 6}, {NULL}};
const Segment SGI114986[] = {{&S7786[30], 6}, {NULL}};
const Segment SGI114987[] = {{&S7786[31], 3}, {NULL}};
const Segment SGI114988[] = {{&S7786[74], 6}, {NULL}};
const Segment SGI114989[] = {{&S7787[44], 9}, {NULL}};
const Segment SGI114990[] = {{&S7787[44], 16}, {NULL}};
const Segment SGI114991[] = {{&S7787[30], 6}, {NULL}};
const Segment SGI114992[] = {{&S7787[31], 3}, {NULL}};
const Segment SGI114993[] = {{&S7787[74], 6}, {NULL}};
const Segment SGI114994[] = {{&S105794[42], 10}, {NULL}};
const Segment SGI114995[] = {{&S7788[43], 23}, {NULL}};
const Segment SGI114996[] = {{&S105794[42], 22}, {NULL}};
const Segment SGI114997[] = {{&S7788[30], 6}, {NULL}};
const Segment SGI114998[] = {{&S105794[30], 4}, {NULL}};
const Segment SGI114999[] = {{&S7788[31], 3}, {NULL}};
const Segment SGI115000[] = {{&S105794[111], 3}, {NULL}};
const Segment SGI115001[] = {{&S7788[80], 5}, {NULL}};
const Segment SGI115002[] = {{&S105794[78], 5}, {NULL}};
const Segment SGI115003[] = {{&S105794[115], 14}, {NULL}};
const Segment SGI115004[] = {{&S7789[44], 9}, {NULL}};
const Segment SGI115005[] = {{&S7789[44], 21}, {NULL}};
const Segment SGI115006[] = {{&S7789[30], 6}, {NULL}};
const Segment SGI115007[] = {{&S7789[31], 3}, {NULL}};
const Segment SGI115008[] = {{&S7789[79], 6}, {NULL}};
const Segment SGI115009[] = {{&S7790[44], 13}, {NULL}};
const Segment SGI115010[] = {{&S7790[44], 25}, {NULL}};
const Segment SGI115011[] = {{&S7790[30], 6}, {NULL}};
const Segment SGI115012[] = {{&S7790[31], 3}, {NULL}};
const Segment SGI115013[] = {{&S7790[83], 6}, {NULL}};
const Segment SGI115014[] = {{&S105798[57], 10}, {NULL}};
const Segment SGI115015[] = {{&S105798[57], 22}, {NULL}};
const Segment SGI115016[] = {{&S105798[45], 4}, {NULL}};
const Segment SGI115017[] = {{&S105798[128], 3}, {NULL}};
const Segment SGI115018[] = {{&S7791[44], 16}, {NULL}};
const Segment SGI115019[] = {{&S105798[93], 6}, {NULL}};
const Segment SGI115020[] = {{&S7791[44], 28}, {NULL}};
const Segment SGI115021[] = {{&S7791[30], 6}, {NULL}};
const Segment SGI115022[] = {{&S105798[0], 14}, {NULL}};
const Segment SGI115023[] = {{&S7791[31], 3}, {NULL}};
const Segment SGI115024[] = {{&S7791[86], 5}, {NULL}};
const Segment SGI115025[] = {{&S105799[42], 10}, {NULL}};
const Segment SGI115026[] = {{&S105799[42], 22}, {NULL}};
const Segment SGI115027[] = {{&S105799[30], 4}, {NULL}};
const Segment SGI115028[] = {{&S105799[113], 3}, {NULL}};
const Segment SGI115029[] = {{&S105799[78], 6}, {NULL}};
const Segment SGI115030[] = {{&S7793[44], 6}, {NULL}};
const Segment SGI115031[] = {{&S105799[131], 13}, {NULL}};
const Segment SGI115032[] = {{&S7793[44], 18}, {NULL}};
const Segment SGI115033[] = {{&S7793[30], 6}, {NULL}};
const Segment SGI115034[] = {{&S7793[31], 3}, {NULL}};
const Segment SGI115035[] = {{&S7793[76], 6}, {NULL}};
const Segment SGI115036[] = {{&S7794[44], 6}, {NULL}};
const Segment SGI115037[] = {{&S7794[44], 18}, {NULL}};
const Segment SGI115038[] = {{&S7794[30], 6}, {NULL}};
const Segment SGI115039[] = {{&S7794[31], 3}, {NULL}};
const Segment SGI115040[] = {{&S7794[76], 6}, {NULL}};
const Segment SGI115041[] = {{&S105802[42], 10}, {NULL}};
const Segment SGI115042[] = {{&S105802[42], 21}, {NULL}};
const Segment SGI115043[] = {{&S105802[30], 4}, {NULL}};
const Segment SGI115044[] = {{&S105802[110], 3}, {NULL}};
const Segment SGI115045[] = {{&S105802[77], 5}, {NULL}};
const Segment SGI115046[] = {{&S105802[114], 14}, {NULL}};
const Segment SGI115047[] = {{&S7795[44], 6}, {NULL}};
const Segment SGI115048[] = {{&S7795[44], 18}, {NULL}};
const Segment SGI115049[] = {{&S7795[30], 6}, {NULL}};
const Segment SGI115050[] = {{&S7795[31], 3}, {NULL}};
const Segment SGI115051[] = {{&S7795[76], 6}, {NULL}};
const Segment SGI115052[] = {{&S105803[42], 10}, {NULL}};
const Segment SGI115053[] = {{&S105803[42], 21}, {NULL}};
const Segment SGI115054[] = {{&S105803[30], 4}, {NULL}};
const Segment SGI115055[] = {{&S105803[31], 3}, {NULL}};
const Segment SGI115056[] = {{&S105803[77], 6}, {NULL}};
const Segment SGI115057[] = {{&S105803[132], 8}, {NULL}};
const Segment SGI115058[] = {{&S105803[141], 13}, {NULL}};
const Segment SGI115059[] = {{&S7796[44], 8}, {NULL}};
const Segment SGI115060[] = {{&S7796[44], 20}, {NULL}};
const Segment SGI115061[] = {{&S7796[30], 6}, {NULL}};
const Segment SGI115062[] = {{&S7796[31], 3}, {NULL}};
const Segment SGI115063[] = {{&S7796[78], 6}, {NULL}};
const Segment SGI115064[] = {{&S105815[42], 10}, {NULL}};
const Segment SGI115065[] = {{&S7798[44], 12}, {NULL}};
const Segment SGI115066[] = {{&S105815[42], 21}, {NULL}};
const Segment SGI115067[] = {{&S7798[44], 24}, {NULL}};
const Segment SGI115068[] = {{&S105815[30], 4}, {NULL}};
const Segment SGI115069[] = {{&S7798[30], 6}, {NULL}};
const Segment SGI115070[] = {{&S7798[31], 3}, {NULL}};
const Segment SGI115071[] = {{&S105815[112], 3}, {NULL}};
const Segment SGI115072[] = {{&S7798[82], 6}, {NULL}};
const Segment SGI115073[] = {{&S105815[77], 6}, {NULL}};
const Segment SGI115074[] = {{&S105815[137], 13}, {NULL}};
const Segment SGI115075[] = {{&S7799[44], 14}, {NULL}};
const Segment SGI115076[] = {{&S7799[44], 26}, {NULL}};
const Segment SGI115077[] = {{&S7799[30], 6}, {NULL}};
const Segment SGI115078[] = {{&S105816[42], 10}, {NULL}};
const Segment SGI115079[] = {{&S7799[31], 3}, {NULL}};
const Segment SGI115080[] = {{&S105816[42], 21}, {NULL}};
const Segment SGI115081[] = {{&S105816[30], 4}, {NULL}};
const Segment SGI115082[] = {{&S7799[84], 6}, {NULL}};
const Segment SGI115083[] = {{&S105816[77], 6}, {NULL}};
const Segment SGI115084[] = {{&S105816[104], 13}, {NULL}};
const Segment SGI115085[] = {{&S7800[44], 9}, {NULL}};
const Segment SGI115086[] = {{&S7800[44], 21}, {NULL}};
const Segment SGI115087[] = {{&S7800[30], 6}, {NULL}};
const Segment SGI115088[] = {{&S7800[31], 3}, {NULL}};
const Segment SGI115089[] = {{&S7800[79], 6}, {NULL}};
const Segment SGI115090[] = {{&S7801[44], 8}, {NULL}};
const Segment SGI115091[] = {{&S7801[44], 23}, {NULL}};
const Segment SGI115092[] = {{&S7801[30], 6}, {NULL}};
const Segment SGI115093[] = {{&S7801[31], 3}, {NULL}};
const Segment SGI115094[] = {{&S7801[81], 6}, {NULL}};
const Segment SGI115095[] = {{&S7802[44], 6}, {NULL}};
const Segment SGI115096[] = {{&S7802[44], 18}, {NULL}};
const Segment SGI115097[] = {{&S7802[30], 6}, {NULL}};
const Segment SGI115098[] = {{&S7802[31], 3}, {NULL}};
const Segment SGI115099[] = {{&S7802[76], 6}, {NULL}};
const Segment SGI115100[] = {{&S105823[42], 11}, {NULL}};
const Segment SGI115101[] = {{&S105823[42], 23}, {NULL}};
const Segment SGI115102[] = {{&S7804[44], 5}, {NULL}};
const Segment SGI115103[] = {{&S105823[30], 4}, {NULL}};
const Segment SGI115104[] = {{&S7804[44], 17}, {NULL}};
const Segment SGI115105[] = {{&S7804[30], 6}, {NULL}};
const Segment SGI115106[] = {{&S105823[114], 3}, {NULL}};
const Segment SGI115107[] = {{&S7804[31], 3}, {NULL}};
const Segment SGI115108[] = {{&S7804[75], 6}, {NULL}};
const Segment SGI115109[] = {{&S105823[79], 6}, {NULL}};
const Segment SGI115110[] = {{&S105823[131], 10}, {NULL}};
const Segment SGI115111[] = {{&S7805[44], 12}, {NULL}};
const Segment SGI115112[] = {{&S7805[44], 24}, {NULL}};
const Segment SGI115113[] = {{&S7805[30], 6}, {NULL}};
const Segment SGI115114[] = {{&S7805[31], 3}, {NULL}};
const Segment SGI115115[] = {{&S7805[82], 5}, {NULL}};
const Segment SGI115116[] = {{&S7806[44], 16}, {NULL}};
const Segment SGI115117[] = {{&S7806[44], 28}, {NULL}};
const Segment SGI115118[] = {{&S7806[30], 6}, {NULL}};
const Segment SGI115119[] = {{&S7806[31], 3}, {NULL}};
const Segment SGI115120[] = {{&S7806[86], 6}, {NULL}};
const Segment SGI115121[] = {{&S7807[44], 9}, {NULL}};
const Segment SGI115122[] = {{&S7807[44], 28}, {NULL}};
const Segment SGI115123[] = {{&S7807[30], 6}, {NULL}};
const Segment SGI115124[] = {{&S7807[31], 3}, {NULL}};
const Segment SGI115125[] = {{&S7807[86], 6}, {NULL}};
const Segment SGI115126[] = {{&S7808[44], 8}, {NULL}};
const Segment SGI115127[] = {{&S7808[44], 20}, {NULL}};
const Segment SGI115128[] = {{&S7808[30], 6}, {NULL}};
const Segment SGI115129[] = {{&S7808[31], 3}, {NULL}};
const Segment SGI115130[] = {{&S7808[78], 6}, {NULL}};
const Segment SGI115131[] = {{&S7809[44], 10}, {NULL}};
const Segment SGI115132[] = {{&S105832[42], 10}, {NULL}};
const Segment SGI115133[] = {{&S7809[44], 22}, {NULL}};
const Segment SGI115134[] = {{&S105832[42], 21}, {NULL}};
const Segment SGI115135[] = {{&S7809[30], 6}, {NULL}};
const Segment SGI115136[] = {{&S105832[30], 4}, {NULL}};
const Segment SGI115137[] = {{&S7809[31], 3}, {NULL}};
const Segment SGI115138[] = {{&S105832[110], 3}, {NULL}};
const Segment SGI115139[] = {{&S7809[80], 6}, {NULL}};
const Segment SGI115140[] = {{&S105832[77], 5}, {NULL}};
const Segment SGI115141[] = {{&S105832[114], 14}, {NULL}};
const Segment SGI115142[] = {{&S7810[44], 7}, {NULL}};
const Segment SGI115143[] = {{&S105833[42], 10}, {NULL}};
const Segment SGI115144[] = {{&S7810[44], 39}, {NULL}};
const Segment SGI115145[] = {{&S105833[42], 22}, {NULL}};
const Segment SGI115146[] = {{&S7810[30], 6}, {NULL}};
const Segment SGI115147[] = {{&S105833[30], 4}, {NULL}};
const Segment SGI115148[] = {{&S7810[31], 3}, {NULL}};
const Segment SGI115149[] = {{&S105833[111], 3}, {NULL}};
const Segment SGI115150[] = {{&S7810[97], 6}, {NULL}};
const Segment SGI115151[] = {{&S105833[78], 5}, {NULL}};
const Segment SGI115152[] = {{&S105833[115], 14}, {NULL}};
const Segment SGI115153[] = {{&S7811[44], 6}, {NULL}};
const Segment SGI115154[] = {{&S7811[44], 18}, {NULL}};
const Segment SGI115155[] = {{&S105834[57], 10}, {NULL}};
const Segment SGI115156[] = {{&S7811[30], 6}, {NULL}};
const Segment SGI115157[] = {{&S105834[57], 21}, {NULL}};
const Segment SGI115158[] = {{&S105834[45], 4}, {NULL}};
const Segment SGI115159[] = {{&S90731[34], 6}, {NULL}};
const Segment SGI115160[] = {{&S7811[31], 3}, {NULL}};
const Segment SGI115161[] = {{&S90731[27], 6}, {NULL}};
const Segment SGI115162[] = {{&S105834[127], 3}, {NULL}};
const Segment SGI115163[] = {{&S7811[76], 6}, {NULL}};
const Segment SGI115164[] = {{&S105834[92], 6}, {NULL}};
const Segment SGI115165[] = {{&S105834[0], 14}, {NULL}};
const Segment SGI115166[] = {{&S90731[93], 9}, {NULL}};
const Segment SGI115167[] = {{&S90731[66], 6}, {NULL}};
const Segment SGI115168[] = {{&S7812[44], 10}, {NULL}};
const Segment SGI115169[] = {{&S7812[44], 22}, {NULL}};
const Segment SGI115170[] = {{&S7812[30], 6}, {NULL}};
const Segment SGI115171[] = {{&S7812[31], 3}, {NULL}};
const Segment SGI115172[] = {{&S7812[80], 6}, {NULL}};
const Segment SGI115173[] = {{&S105836[42], 10}, {NULL}};
const Segment SGI115174[] = {{&S105836[42], 22}, {NULL}};
const Segment SGI115175[] = {{&S105836[30], 4}, {NULL}};
const Segment SGI115176[] = {{&S105836[113], 3}, {NULL}};
const Segment SGI115177[] = {{&S105836[78], 6}, {NULL}};
const Segment SGI115178[] = {{&S7813[44], 15}, {NULL}};
const Segment SGI115179[] = {{&S7813[44], 27}, {NULL}};
const Segment SGI115180[] = {{&S105836[155], 3}, {NULL}};
const Segment SGI115181[] = {{&S7813[30], 6}, {NULL}};
const Segment SGI115182[] = {{&S7813[31], 3}, {NULL}};
const Segment SGI115183[] = {{&S7813[85], 6}, {NULL}};
const Segment SGI115184[] = {{&S105837[55], 10}, {NULL}};
const Segment SGI115185[] = {{&S105837[55], 22}, {NULL}};
const Segment SGI115186[] = {{&S105837[43], 4}, {NULL}};
const Segment SGI115187[] = {{&S105837[126], 3}, {NULL}};
const Segment SGI115188[] = {{&S105837[91], 6}, {NULL}};
const Segment SGI115189[] = {{&S7814[44], 13}, {NULL}};
const Segment SGI115190[] = {{&S7814[44], 25}, {NULL}};
const Segment SGI115191[] = {{&S105837[0], 12}, {NULL}};
const Segment SGI115192[] = {{&S7814[30], 6}, {NULL}};
const Segment SGI115193[] = {{&S7814[31], 3}, {NULL}};
const Segment SGI115194[] = {{&S7814[83], 6}, {NULL}};
const Segment SGI115195[] = {{&S7815[44], 5}, {NULL}};
const Segment SGI115196[] = {{&S7815[44], 17}, {NULL}};
const Segment SGI115197[] = {{&S7815[30], 6}, {NULL}};
const Segment SGI115198[] = {{&S105839[42], 10}, {NULL}};
const Segment SGI115199[] = {{&S7815[31], 3}, {NULL}};
const Segment SGI115200[] = {{&S105839[42], 22}, {NULL}};
const Segment SGI115201[] = {{&S105839[30], 4}, {NULL}};
const Segment SGI115202[] = {{&S7815[75], 6}, {NULL}};
const Segment SGI115203[] = {{&S105839[113], 3}, {NULL}};
const Segment SGI115204[] = {{&S105839[78], 6}, {NULL}};
const Segment SGI115205[] = {{&S105839[155], 3}, {NULL}};
const Segment SGI115206[] = {{&S7816[44], 9}, {NULL}};
const Segment SGI115207[] = {{&S7816[44], 21}, {NULL}};
const Segment SGI115208[] = {{&S7816[30], 6}, {NULL}};
const Segment SGI115209[] = {{&S7816[31], 3}, {NULL}};
const Segment SGI115210[] = {{&S7816[79], 6}, {NULL}};
const Segment SGI115211[] = {{&S7817[44], 6}, {NULL}};
const Segment SGI115212[] = {{&S7817[44], 18}, {NULL}};
const Segment SGI115213[] = {{&S7817[30], 6}, {NULL}};
const Segment SGI115214[] = {{&S7817[31], 3}, {NULL}};
const Segment SGI115215[] = {{&S7817[76], 6}, {NULL}};
const Segment SGI115216[] = {{&S7818[44], 14}, {NULL}};
const Segment SGI115217[] = {{&S7818[44], 26}, {NULL}};
const Segment SGI115218[] = {{&S7818[30], 6}, {NULL}};
const Segment SGI115219[] = {{&S7818[31], 3}, {NULL}};
const Segment SGI115220[] = {{&S7818[84], 6}, {NULL}};
const Segment SGI115221[] = {{&S7819[44], 15}, {NULL}};
const Segment SGI115222[] = {{&S7819[44], 22}, {NULL}};
const Segment SGI115223[] = {{&S7819[30], 6}, {NULL}};
const Segment SGI115224[] = {{&S7819[31], 3}, {NULL}};
const Segment SGI115225[] = {{&S7819[80], 6}, {NULL}};
const Segment SGI115226[] = {{&S7820[44], 18}, {NULL}};
const Segment SGI115227[] = {{&S7820[44], 30}, {NULL}};
const Segment SGI115228[] = {{&S7820[30], 6}, {NULL}};
const Segment SGI115229[] = {{&S7820[31], 3}, {NULL}};
const Segment SGI115230[] = {{&S7820[88], 6}, {NULL}};
const Segment SGI115231[] = {{&S105850[42], 10}, {NULL}};
const Segment SGI115232[] = {{&S105850[42], 22}, {NULL}};
const Segment SGI115233[] = {{&S105850[30], 4}, {NULL}};
const Segment SGI115234[] = {{&S105850[113], 3}, {NULL}};
const Segment SGI115235[] = {{&S105850[78], 6}, {NULL}};
const Segment SGI115236[] = {{&S105850[131], 13}, {NULL}};
const Segment SGI115237[] = {{&S105852[57], 10}, {NULL}};
const Segment SGI115238[] = {{&S105852[57], 22}, {NULL}};
const Segment SGI115239[] = {{&S105852[45], 4}, {NULL}};
const Segment SGI115240[] = {{&S105852[128], 3}, {NULL}};
const Segment SGI115241[] = {{&S105852[93], 6}, {NULL}};
const Segment SGI115242[] = {{&S7826[44], 10}, {NULL}};
const Segment SGI115243[] = {{&S105852[0], 14}, {NULL}};
const Segment SGI115244[] = {{&S7826[44], 22}, {NULL}};
const Segment SGI115245[] = {{&S7826[30], 6}, {NULL}};
const Segment SGI115246[] = {{&S7826[31], 3}, {NULL}};
const Segment SGI115247[] = {{&S7826[80], 6}, {NULL}};
const Segment SGI115248[] = {{&S105854[42], 10}, {NULL}};
const Segment SGI115249[] = {{&S105854[42], 21}, {NULL}};
const Segment SGI115250[] = {{&S105854[30], 4}, {NULL}};
const Segment SGI115251[] = {{&S105854[77], 6}, {NULL}};
const Segment SGI115252[] = {{&S105854[104], 13}, {NULL}};
const Segment SGI115253[] = {{&S7829[44], 20}, {NULL}};
const Segment SGI115254[] = {{&S7829[44], 32}, {NULL}};
const Segment SGI115255[] = {{&S7829[30], 6}, {NULL}};
const Segment SGI115256[] = {{&S7829[31], 3}, {NULL}};
const Segment SGI115257[] = {{&S7829[90], 6}, {NULL}};
const Segment SGI115258[] = {{&S7830[44], 8}, {NULL}};
const Segment SGI115259[] = {{&S7830[44], 20}, {NULL}};
const Segment SGI115260[] = {{&S7830[30], 6}, {NULL}};
const Segment SGI115261[] = {{&S105857[42], 10}, {NULL}};
const Segment SGI115262[] = {{&S105857[42], 21}, {NULL}};
const Segment SGI115263[] = {{&S105857[30], 4}, {NULL}};
const Segment SGI115264[] = {{&S7830[31], 3}, {NULL}};
const Segment SGI115265[] = {{&S7830[78], 6}, {NULL}};
const Segment SGI115266[] = {{&S105857[77], 6}, {NULL}};
const Segment SGI115267[] = {{&S105857[104], 13}, {NULL}};
const Segment SGI115268[] = {{&S7831[44], 7}, {NULL}};
const Segment SGI115269[] = {{&S7831[44], 19}, {NULL}};
const Segment SGI115270[] = {{&S7831[30], 6}, {NULL}};
const Segment SGI115271[] = {{&S105858[42], 10}, {NULL}};
const Segment SGI115272[] = {{&S7831[31], 3}, {NULL}};
const Segment SGI115273[] = {{&S105858[42], 22}, {NULL}};
const Segment SGI115274[] = {{&S105858[30], 4}, {NULL}};
const Segment SGI115275[] = {{&S7831[77], 6}, {NULL}};
const Segment SGI115276[] = {{&S105858[111], 3}, {NULL}};
const Segment SGI115277[] = {{&S105858[78], 5}, {NULL}};
const Segment SGI115278[] = {{&S105858[115], 14}, {NULL}};
const Segment SGI115279[] = {{&S7833[44], 7}, {NULL}};
const Segment SGI115280[] = {{&S7833[44], 19}, {NULL}};
const Segment SGI115281[] = {{&S7833[30], 6}, {NULL}};
const Segment SGI115282[] = {{&S7833[31], 3}, {NULL}};
const Segment SGI115283[] = {{&S7833[77], 6}, {NULL}};
const Segment SGI115284[] = {{&S105860[42], 10}, {NULL}};
const Segment SGI115285[] = {{&S105860[42], 21}, {NULL}};
const Segment SGI115286[] = {{&S105860[30], 4}, {NULL}};
const Segment SGI115287[] = {{&S105860[110], 3}, {NULL}};
const Segment SGI115288[] = {{&S105860[77], 5}, {NULL}};
const Segment SGI115289[] = {{&S105860[114], 14}, {NULL}};
const Segment SGI115290[] = {{&S7835[44], 8}, {NULL}};
const Segment SGI115291[] = {{&S7835[44], 20}, {NULL}};
const Segment SGI115292[] = {{&S7835[30], 6}, {NULL}};
const Segment SGI115293[] = {{&S7835[31], 3}, {NULL}};
const Segment SGI115294[] = {{&S7835[78], 6}, {NULL}};
const Segment SGI115295[] = {{&S105862[57], 10}, {NULL}};
const Segment SGI115296[] = {{&S105862[57], 22}, {NULL}};
const Segment SGI115297[] = {{&S105862[45], 4}, {NULL}};
const Segment SGI115298[] = {{&S105862[128], 3}, {NULL}};
const Segment SGI115299[] = {{&S105862[93], 6}, {NULL}};
const Segment SGI115300[] = {{&S105862[0], 14}, {NULL}};
const Segment SGI115301[] = {{&S7836[44], 5}, {NULL}};
const Segment SGI115302[] = {{&S7836[44], 17}, {NULL}};
const Segment SGI115303[] = {{&S7836[30], 6}, {NULL}};
const Segment SGI115304[] = {{&S7836[31], 3}, {NULL}};
const Segment SGI115305[] = {{&S7836[75], 6}, {NULL}};
const Segment SGI115306[] = {{&S7838[44], 6}, {NULL}};
const Segment SGI115307[] = {{&S7838[44], 18}, {NULL}};
const Segment SGI115308[] = {{&S7838[30], 6}, {NULL}};
const Segment SGI115309[] = {{&S7838[31], 3}, {NULL}};
const Segment SGI115310[] = {{&S7838[76], 6}, {NULL}};
const Segment SGI115311[] = {{&S7839[38], 6}, {NULL}};
const Segment SGI115312[] = {{&S7839[31], 6}, {NULL}};
const Segment SGI115313[] = {{&S7840[44], 9}, {NULL}};
const Segment SGI115314[] = {{&S7840[44], 21}, {NULL}};
const Segment SGI115315[] = {{&S7840[30], 6}, {NULL}};
const Segment SGI115316[] = {{&S7840[31], 3}, {NULL}};
const Segment SGI115317[] = {{&S7840[79], 6}, {NULL}};
const Segment SGI115318[] = {{&S7841[44], 14}, {NULL}};
const Segment SGI115319[] = {{&S7841[44], 26}, {NULL}};
const Segment SGI115320[] = {{&S7841[30], 6}, {NULL}};
const Segment SGI115321[] = {{&S7841[31], 3}, {NULL}};
const Segment SGI115322[] = {{&S7841[84], 6}, {NULL}};
const Segment SGI115323[] = {{&S7842[44], 11}, {NULL}};
const Segment SGI115324[] = {{&S7842[44], 23}, {NULL}};
const Segment SGI115325[] = {{&S7842[30], 6}, {NULL}};
const Segment SGI115326[] = {{&S7842[31], 3}, {NULL}};
const Segment SGI115327[] = {{&S7842[81], 6}, {NULL}};
const Segment SGI115328[] = {{&S7843[38], 6}, {NULL}};
const Segment SGI115329[] = {{&S7843[31], 6}, {NULL}};
const Segment SGI115330[] = {{&S7844[44], 6}, {NULL}};
const Segment SGI115331[] = {{&S7844[44], 18}, {NULL}};
const Segment SGI115332[] = {{&S7844[30], 6}, {NULL}};
const Segment SGI115333[] = {{&S7844[31], 3}, {NULL}};
const Segment SGI115334[] = {{&S7844[76], 6}, {NULL}};
const Segment SGI115335[] = {{&S7845[44], 10}, {NULL}};
const Segment SGI115336[] = {{&S7845[44], 22}, {NULL}};
const Segment SGI115337[] = {{&S7845[30], 6}, {NULL}};
const Segment SGI115338[] = {{&S7845[31], 3}, {NULL}};
const Segment SGI115339[] = {{&S7845[80], 6}, {NULL}};
const Segment SGI115340[] = {{&S90789[34], 7}, {NULL}};
const Segment SGI115341[] = {{&S90789[27], 6}, {NULL}};
const Segment SGI115342[] = {{&S90789[94], 9}, {NULL}};
const Segment SGI115343[] = {{&S90789[67], 6}, {NULL}};
const Segment SGI115344[] = {{&S7847[44], 12}, {NULL}};
const Segment SGI115345[] = {{&S7847[44], 24}, {NULL}};
const Segment SGI115346[] = {{&S7847[30], 6}, {NULL}};
const Segment SGI115347[] = {{&S90790[44], 7}, {NULL}};
const Segment SGI115348[] = {{&S7847[31], 3}, {NULL}};
const Segment SGI115349[] = {{&S90790[44], 19}, {NULL}};
const Segment SGI115350[] = {{&S90790[30], 6}, {NULL}};
const Segment SGI115351[] = {{&S7847[82], 6}, {NULL}};
const Segment SGI115352[] = {{&S90790[31], 3}, {NULL}};
const Segment SGI115353[] = {{&S90790[77], 6}, {NULL}};
const Segment SGI115354[] = {{&S7848[44], 6}, {NULL}};
const Segment SGI115355[] = {{&S7848[44], 18}, {NULL}};
const Segment SGI115356[] = {{&S7848[30], 6}, {NULL}};
const Segment SGI115357[] = {{&S7848[31], 3}, {NULL}};
const Segment SGI115358[] = {{&S7848[76], 6}, {NULL}};
const Segment SGI115359[] = {{&S7850[44], 10}, {NULL}};
const Segment SGI115360[] = {{&S7850[44], 25}, {NULL}};
const Segment SGI115361[] = {{&S7850[30], 6}, {NULL}};
const Segment SGI115362[] = {{&S7850[31], 3}, {NULL}};
const Segment SGI115363[] = {{&S7850[83], 6}, {NULL}};
const Segment SGI115364[] = {{&S7851[44], 13}, {NULL}};
const Segment SGI115365[] = {{&S7851[44], 25}, {NULL}};
const Segment SGI115366[] = {{&S7851[30], 6}, {NULL}};
const Segment SGI115367[] = {{&S7851[31], 3}, {NULL}};
const Segment SGI115368[] = {{&S7851[83], 6}, {NULL}};
const Segment SGI115369[] = {{&S7853[44], 8}, {NULL}};
const Segment SGI115370[] = {{&S7853[44], 20}, {NULL}};
const Segment SGI115371[] = {{&S7853[30], 6}, {NULL}};
const Segment SGI115372[] = {{&S7853[31], 3}, {NULL}};
const Segment SGI115373[] = {{&S7853[78], 6}, {NULL}};
const Segment SGI115374[] = {{&S7854[44], 9}, {NULL}};
const Segment SGI115375[] = {{&S7854[44], 21}, {NULL}};
const Segment SGI115376[] = {{&S7854[30], 6}, {NULL}};
const Segment SGI115377[] = {{&S7854[31], 3}, {NULL}};
const Segment SGI115378[] = {{&S7854[79], 6}, {NULL}};
const Segment SGI115379[] = {{&S7855[44], 8}, {NULL}};
const Segment SGI115380[] = {{&S7855[44], 20}, {NULL}};
const Segment SGI115381[] = {{&S7855[30], 6}, {NULL}};
const Segment SGI115382[] = {{&S7855[31], 3}, {NULL}};
const Segment SGI115383[] = {{&S7855[78], 6}, {NULL}};
const Segment SGI115384[] = {{&S7856[44], 5}, {NULL}};
const Segment SGI115385[] = {{&S7856[44], 17}, {NULL}};
const Segment SGI115386[] = {{&S7856[30], 6}, {NULL}};
const Segment SGI115387[] = {{&S7856[31], 3}, {NULL}};
const Segment SGI115388[] = {{&S7856[75], 6}, {NULL}};
const Segment SGI115389[] = {{&S7857[44], 4}, {NULL}};
const Segment SGI115390[] = {{&S7857[44], 16}, {NULL}};
const Segment SGI115391[] = {{&S7857[30], 6}, {NULL}};
const Segment SGI115392[] = {{&S7857[31], 3}, {NULL}};
const Segment SGI115393[] = {{&S7857[74], 6}, {NULL}};
const Segment SGI115394[] = {{&S7858[44], 11}, {NULL}};
const Segment SGI115395[] = {{&S7858[44], 23}, {NULL}};
const Segment SGI115396[] = {{&S90809[59], 4}, {NULL}};
const Segment SGI115397[] = {{&S7858[30], 6}, {NULL}};
const Segment SGI115398[] = {{&S90809[59], 26}, {NULL}};
const Segment SGI115399[] = {{&S90809[45], 6}, {NULL}};
const Segment SGI115400[] = {{&S7858[31], 3}, {NULL}};
const Segment SGI115401[] = {{&S7858[81], 6}, {NULL}};
const Segment SGI115402[] = {{&S90809[46], 3}, {NULL}};
const Segment SGI115403[] = {{&S90809[99], 6}, {NULL}};
const Segment SGI115404[] = {{&S90809[0], 14}, {NULL}};
const Segment SGI115405[] = {{&S7859[44], 17}, {NULL}};
const Segment SGI115406[] = {{&S7859[44], 29}, {NULL}};
const Segment SGI115407[] = {{&S7859[30], 6}, {NULL}};
const Segment SGI115408[] = {{&S7859[31], 3}, {NULL}};
const Segment SGI115409[] = {{&S7859[87], 6}, {NULL}};
const Segment SGI115410[] = {{&S7860[44], 6}, {NULL}};
const Segment SGI115411[] = {{&S7860[44], 18}, {NULL}};
const Segment SGI115412[] = {{&S7860[30], 6}, {NULL}};
const Segment SGI115413[] = {{&S7860[31], 3}, {NULL}};
const Segment SGI115414[] = {{&S7860[76], 6}, {NULL}};
const Segment SGI115415[] = {{&S7862[44], 11}, {NULL}};
const Segment SGI115416[] = {{&S7862[44], 23}, {NULL}};
const Segment SGI115417[] = {{&S7862[30], 6}, {NULL}};
const Segment SGI115418[] = {{&S7862[31], 3}, {NULL}};
const Segment SGI115419[] = {{&S7862[81], 6}, {NULL}};
const Segment SGI115420[] = {{&S7863[44], 7}, {NULL}};
const Segment SGI115421[] = {{&S7863[44], 19}, {NULL}};
const Segment SGI115422[] = {{&S7863[30], 6}, {NULL}};
const Segment SGI115423[] = {{&S7863[31], 3}, {NULL}};
const Segment SGI115424[] = {{&S7863[77], 6}, {NULL}};
const Segment SGI115425[] = {{&S7864[44], 12}, {NULL}};
const Segment SGI115426[] = {{&S7864[44], 24}, {NULL}};
const Segment SGI115427[] = {{&S7864[30], 6}, {NULL}};
const Segment SGI115428[] = {{&S7864[31], 3}, {NULL}};
const Segment SGI115429[] = {{&S7864[82], 6}, {NULL}};
const Segment SGI115430[] = {{&S7865[44], 10}, {NULL}};
const Segment SGI115431[] = {{&S7865[44], 22}, {NULL}};
const Segment SGI115432[] = {{&S7865[30], 6}, {NULL}};
const Segment SGI115433[] = {{&S7865[31], 3}, {NULL}};
const Segment SGI115434[] = {{&S7865[80], 6}, {NULL}};
const Segment SGI115435[] = {{&S7867[44], 9}, {NULL}};
const Segment SGI115436[] = {{&S7867[44], 21}, {NULL}};
const Segment SGI115437[] = {{&S7867[30], 6}, {NULL}};
const Segment SGI115438[] = {{&S7867[31], 3}, {NULL}};
const Segment SGI115439[] = {{&S7867[79], 6}, {NULL}};
const Segment SGI115440[] = {{&S7868[44], 5}, {NULL}};
const Segment SGI115441[] = {{&S7868[44], 17}, {NULL}};
const Segment SGI115442[] = {{&S7868[30], 6}, {NULL}};
const Segment SGI115443[] = {{&S7868[31], 3}, {NULL}};
const Segment SGI115444[] = {{&S7868[75], 6}, {NULL}};
const Segment SGI115445[] = {{&S105867[43], 10}, {NULL}};
const Segment SGI115446[] = {{&S105867[30], 4}, {NULL}};
const Segment SGI115447[] = {{&S90829[44], 20}, {NULL}};
const Segment SGI115448[] = {{&S105867[113], 3}, {NULL}};
const Segment SGI115449[] = {{&S90829[44], 47}, {NULL}};
const Segment SGI115450[] = {{&S7869[44], 6}, {NULL}};
const Segment SGI115451[] = {{&S90829[30], 6}, {NULL}};
const Segment SGI115452[] = {{&S7869[44], 18}, {NULL}};
const Segment SGI115453[] = {{&S105867[79], 5}, {NULL}};
const Segment SGI115454[] = {{&S7869[30], 6}, {NULL}};
const Segment SGI115455[] = {{&S105867[137], 13}, {NULL}};
const Segment SGI115456[] = {{&S90829[31], 3}, {NULL}};
const Segment SGI115457[] = {{&S7869[31], 3}, {NULL}};
const Segment SGI115458[] = {{&S7869[76], 6}, {NULL}};
const Segment SGI115459[] = {{&S90829[105], 6}, {NULL}};
const Segment SGI115460[] = {{&S7870[44], 13}, {NULL}};
const Segment SGI115461[] = {{&S7870[44], 25}, {NULL}};
const Segment SGI115462[] = {{&S7870[30], 6}, {NULL}};
const Segment SGI115463[] = {{&S7870[31], 3}, {NULL}};
const Segment SGI115464[] = {{&S7870[83], 6}, {NULL}};
const Segment SGI115465[] = {{&S7871[44], 7}, {NULL}};
const Segment SGI115466[] = {{&S7871[44], 19}, {NULL}};
const Segment SGI115467[] = {{&S7871[30], 6}, {NULL}};
const Segment SGI115468[] = {{&S7871[31], 3}, {NULL}};
const Segment SGI115469[] = {{&S7871[77], 6}, {NULL}};
const Segment SGI115470[] = {{&S7872[44], 11}, {NULL}};
const Segment SGI115471[] = {{&S7872[44], 23}, {NULL}};
const Segment SGI115472[] = {{&S7872[30], 6}, {NULL}};
const Segment SGI115473[] = {{&S7872[31], 3}, {NULL}};
const Segment SGI115474[] = {{&S7872[81], 6}, {NULL}};
const Segment SGI115475[] = {{&S7873[44], 10}, {NULL}};
const Segment SGI115476[] = {{&S7873[44], 22}, {NULL}};
const Segment SGI115477[] = {{&S7873[30], 6}, {NULL}};
const Segment SGI115478[] = {{&S7873[31], 3}, {NULL}};
const Segment SGI115479[] = {{&S7873[80], 5}, {NULL}};
const Segment SGI115480[] = {{&S7874[44], 6}, {NULL}};
const Segment SGI115481[] = {{&S7874[44], 18}, {NULL}};
const Segment SGI115482[] = {{&S7874[30], 6}, {NULL}};
const Segment SGI115483[] = {{&S7874[31], 3}, {NULL}};
const Segment SGI115484[] = {{&S7874[76], 6}, {NULL}};
const Segment SGI115485[] = {{&S105877[42], 10}, {NULL}};
const Segment SGI115486[] = {{&S105877[42], 21}, {NULL}};
const Segment SGI115487[] = {{&S105877[30], 4}, {NULL}};
const Segment SGI115488[] = {{&S105877[112], 3}, {NULL}};
const Segment SGI115489[] = {{&S105877[77], 6}, {NULL}};
const Segment SGI115490[] = {{&S105877[137], 11}, {NULL}};
const Segment SGI115491[] = {{&S7875[44], 13}, {NULL}};
const Segment SGI115492[] = {{&S7875[44], 25}, {NULL}};
const Segment SGI115493[] = {{&S7875[30], 6}, {NULL}};
const Segment SGI115494[] = {{&S7875[31], 3}, {NULL}};
const Segment SGI115495[] = {{&S7875[83], 6}, {NULL}};
const Segment SGI115496[] = {{&S7876[44], 21}, {NULL}};
const Segment SGI115497[] = {{&S7876[44], 33}, {NULL}};
const Segment SGI115498[] = {{&S7876[30], 6}, {NULL}};
const Segment SGI115499[] = {{&S7876[31], 3}, {NULL}};
const Segment SGI115500[] = {{&S7876[91], 6}, {NULL}};
const Segment SGI115501[] = {{&S7877[44], 8}, {NULL}};
const Segment SGI115502[] = {{&S7877[44], 20}, {NULL}};
const Segment SGI115503[] = {{&S7877[30], 6}, {NULL}};
const Segment SGI115504[] = {{&S7877[31], 3}, {NULL}};
const Segment SGI115505[] = {{&S7877[78], 6}, {NULL}};
const Segment SGI115506[] = {{&S105883[42], 10}, {NULL}};
const Segment SGI115507[] = {{&S105883[42], 22}, {NULL}};
const Segment SGI115508[] = {{&S105883[30], 4}, {NULL}};
const Segment SGI115509[] = {{&S7878[44], 7}, {NULL}};
const Segment SGI115510[] = {{&S7878[44], 19}, {NULL}};
const Segment SGI115511[] = {{&S105883[113], 3}, {NULL}};
const Segment SGI115512[] = {{&S7878[30], 6}, {NULL}};
const Segment SGI115513[] = {{&S105883[78], 6}, {NULL}};
const Segment SGI115514[] = {{&S105883[131], 13}, {NULL}};
const Segment SGI115515[] = {{&S7878[31], 3}, {NULL}};
const Segment SGI115516[] = {{&S7878[77], 6}, {NULL}};
const Segment SGI115517[] = {{&S105884[32], 10}, {NULL}};
const Segment SGI115518[] = {{&S7879[44], 7}, {NULL}};
const Segment SGI115519[] = {{&S105884[27], 4}, {NULL}};
const Segment SGI115520[] = {{&S7879[44], 19}, {NULL}};
const Segment SGI115521[] = {{&S7879[30], 6}, {NULL}};
const Segment SGI115522[] = {{&S7879[31], 3}, {NULL}};
const Segment SGI115523[] = {{&S105884[102], 2}, {NULL}};
const Segment SGI115524[] = {{&S7879[77], 6}, {NULL}};
const Segment SGI115525[] = {{&S105884[68], 6}, {NULL}};
const Segment SGI115526[] = {{&S105884[130], 8}, {NULL}};
const Segment SGI115527[] = {{&S7880[44], 8}, {NULL}};
const Segment SGI115528[] = {{&S7880[44], 20}, {NULL}};
const Segment SGI115529[] = {{&S7880[30], 6}, {NULL}};
const Segment SGI115530[] = {{&S90851[45], 9}, {NULL}};
const Segment SGI115531[] = {{&S90851[30], 6}, {NULL}};
const Segment SGI115532[] = {{&S7880[31], 3}, {NULL}};
const Segment SGI115533[] = {{&S7880[78], 6}, {NULL}};
const Segment SGI115534[] = {{&S90851[31], 3}, {NULL}};
const Segment SGI115535[] = {{&S90851[80], 5}, {NULL}};
const Segment SGI115536[] = {{&S90851[138], 13}, {NULL}};
const Segment SGI115537[] = {{&S7881[44], 15}, {NULL}};
const Segment SGI115538[] = {{&S7881[44], 27}, {NULL}};
const Segment SGI115539[] = {{&S7881[30], 6}, {NULL}};
const Segment SGI115540[] = {{&S7881[31], 3}, {NULL}};
const Segment SGI115541[] = {{&S7881[85], 6}, {NULL}};
const Segment SGI115542[] = {{&S7882[44], 4}, {NULL}};
const Segment SGI115543[] = {{&S7882[44], 16}, {NULL}};
const Segment SGI115544[] = {{&S7882[30], 6}, {NULL}};
const Segment SGI115545[] = {{&S7882[31], 3}, {NULL}};
const Segment SGI115546[] = {{&S7882[74], 6}, {NULL}};
const Segment SGI115547[] = {{&S7883[44], 11}, {NULL}};
const Segment SGI115548[] = {{&S7883[44], 23}, {NULL}};
const Segment SGI115549[] = {{&S7883[30], 6}, {NULL}};
const Segment SGI115550[] = {{&S7883[31], 3}, {NULL}};
const Segment SGI115551[] = {{&S7883[81], 6}, {NULL}};
const Segment SGI115552[] = {{&S7884[44], 6}, {NULL}};
const Segment SGI115553[] = {{&S7884[44], 18}, {NULL}};
const Segment SGI115554[] = {{&S7884[30], 6}, {NULL}};
const Segment SGI115555[] = {{&S7884[31], 3}, {NULL}};
const Segment SGI115556[] = {{&S7884[76], 6}, {NULL}};
const Segment SGI115557[] = {{&S105893[42], 10}, {NULL}};
const Segment SGI115558[] = {{&S105893[42], 22}, {NULL}};
const Segment SGI115559[] = {{&S105893[30], 4}, {NULL}};
const Segment SGI115560[] = {{&S105893[113], 3}, {NULL}};
const Segment SGI115561[] = {{&S105893[78], 6}, {NULL}};
const Segment SGI115562[] = {{&S105893[131], 13}, {NULL}};
const Segment SGI115563[] = {{&S7886[44], 13}, {NULL}};
const Segment SGI115564[] = {{&S7886[44], 25}, {NULL}};
const Segment SGI115565[] = {{&S7886[30], 6}, {NULL}};
const Segment SGI115566[] = {{&S7886[31], 3}, {NULL}};
const Segment SGI115567[] = {{&S7886[83], 6}, {NULL}};
const Segment SGI115568[] = {{&S105896[32], 10}, {NULL}};
const Segment SGI115569[] = {{&S105896[27], 4}, {NULL}};
const Segment SGI115570[] = {{&S7887[44], 6}, {NULL}};
const Segment SGI115571[] = {{&S7887[44], 18}, {NULL}};
const Segment SGI115572[] = {{&S7887[30], 6}, {NULL}};
const Segment SGI115573[] = {{&S105896[101], 2}, {NULL}};
const Segment SGI115574[] = {{&S7887[31], 3}, {NULL}};
const Segment SGI115575[] = {{&S105896[67], 6}, {NULL}};
const Segment SGI115576[] = {{&S7887[76], 6}, {NULL}};
const Segment SGI115577[] = {{&S105896[129], 8}, {NULL}};
const Segment SGI115578[] = {{&S7889[44], 11}, {NULL}};
const Segment SGI115579[] = {{&S7889[44], 23}, {NULL}};
const Segment SGI115580[] = {{&S7889[30], 6}, {NULL}};
const Segment SGI115581[] = {{&S7889[31], 3}, {NULL}};
const Segment SGI115582[] = {{&S7889[81], 6}, {NULL}};
const Segment SGI115583[] = {{&S105899[32], 10}, {NULL}};
const Segment SGI115584[] = {{&S105899[27], 4}, {NULL}};
const Segment SGI115585[] = {{&S7890[44], 10}, {NULL}};
const Segment SGI115586[] = {{&S105899[102], 2}, {NULL}};
const Segment SGI115587[] = {{&S7890[44], 22}, {NULL}};
const Segment SGI115588[] = {{&S105899[68], 6}, {NULL}};
const Segment SGI115589[] = {{&S7890[30], 6}, {NULL}};
const Segment SGI115590[] = {{&S105899[130], 8}, {NULL}};
const Segment SGI115591[] = {{&S7890[31], 3}, {NULL}};
const Segment SGI115592[] = {{&S7890[80], 6}, {NULL}};
const Segment SGI115593[] = {{&S105900[57], 10}, {NULL}};
const Segment SGI115594[] = {{&S105900[57], 22}, {NULL}};
const Segment SGI115595[] = {{&S105900[45], 4}, {NULL}};
const Segment SGI115596[] = {{&S7891[44], 8}, {NULL}};
const Segment SGI115597[] = {{&S7891[44], 20}, {NULL}};
const Segment SGI115598[] = {{&S105900[128], 3}, {NULL}};
const Segment SGI115599[] = {{&S7891[30], 6}, {NULL}};
const Segment SGI115600[] = {{&S105900[93], 6}, {NULL}};
const Segment SGI115601[] = {{&S105900[0], 14}, {NULL}};
const Segment SGI115602[] = {{&S7891[31], 3}, {NULL}};
const Segment SGI115603[] = {{&S7891[78], 6}, {NULL}};
const Segment SGI115604[] = {{&S7892[44], 7}, {NULL}};
const Segment SGI115605[] = {{&S7892[44], 19}, {NULL}};
const Segment SGI115606[] = {{&S7892[30], 6}, {NULL}};
const Segment SGI115607[] = {{&S7892[31], 3}, {NULL}};
const Segment SGI115608[] = {{&S7892[77], 6}, {NULL}};
const Segment SGI115609[] = {{&S7893[44], 6}, {NULL}};
const Segment SGI115610[] = {{&S7893[44], 18}, {NULL}};
const Segment SGI115611[] = {{&S7893[30], 6}, {NULL}};
const Segment SGI115612[] = {{&S7893[31], 3}, {NULL}};
const Segment SGI115613[] = {{&S7893[76], 6}, {NULL}};
const Segment SGI115614[] = {{&S7894[44], 6}, {NULL}};
const Segment SGI115615[] = {{&S7894[44], 18}, {NULL}};
const Segment SGI115616[] = {{&S7894[30], 6}, {NULL}};
const Segment SGI115617[] = {{&S7894[31], 3}, {NULL}};
const Segment SGI115618[] = {{&S7894[76], 6}, {NULL}};
const Segment SGI115619[] = {{&S7895[44], 6}, {NULL}};
const Segment SGI115620[] = {{&S7895[44], 18}, {NULL}};
const Segment SGI115621[] = {{&S7895[30], 6}, {NULL}};
const Segment SGI115622[] = {{&S7895[31], 3}, {NULL}};
const Segment SGI115623[] = {{&S7895[76], 6}, {NULL}};
const Segment SGI115624[] = {{&S7897[44], 13}, {NULL}};
const Segment SGI115625[] = {{&S7897[44], 25}, {NULL}};
const Segment SGI115626[] = {{&S7897[30], 6}, {NULL}};
const Segment SGI115627[] = {{&S7897[31], 3}, {NULL}};
const Segment SGI115628[] = {{&S7897[83], 6}, {NULL}};
const Segment SGI115629[] = {{&S7898[44], 10}, {NULL}};
const Segment SGI115630[] = {{&S7898[44], 22}, {NULL}};
const Segment SGI115631[] = {{&S7898[30], 6}, {NULL}};
const Segment SGI115632[] = {{&S7898[31], 3}, {NULL}};
const Segment SGI115633[] = {{&S7898[80], 6}, {NULL}};
const Segment SGI115634[] = {{&S7899[44], 13}, {NULL}};
const Segment SGI115635[] = {{&S7899[44], 25}, {NULL}};
const Segment SGI115636[] = {{&S7899[30], 6}, {NULL}};
const Segment SGI115637[] = {{&S7899[31], 3}, {NULL}};
const Segment SGI115638[] = {{&S7899[83], 6}, {NULL}};
const Segment SGI115639[] = {{&S7900[44], 8}, {NULL}};
const Segment SGI115640[] = {{&S7900[44], 20}, {NULL}};
const Segment SGI115641[] = {{&S7900[30], 6}, {NULL}};
const Segment SGI115642[] = {{&S7900[31], 3}, {NULL}};
const Segment SGI115643[] = {{&S7900[78], 6}, {NULL}};
const Segment SGI115644[] = {{&S7902[44], 34}, {NULL}};
const Segment SGI115645[] = {{&S7902[30], 6}, {NULL}};
const Segment SGI115646[] = {{&S7902[31], 3}, {NULL}};
const Segment SGI115647[] = {{&S7902[111], 6}, {NULL}};
const Segment SGI115648[] = {{&S7903[44], 12}, {NULL}};
const Segment SGI115649[] = {{&S7903[44], 27}, {NULL}};
const Segment SGI115650[] = {{&S7903[30], 6}, {NULL}};
const Segment SGI115651[] = {{&S7903[31], 3}, {NULL}};
const Segment SGI115652[] = {{&S7903[85], 6}, {NULL}};
const Segment SGI115653[] = {{&S7904[44], 6}, {NULL}};
const Segment SGI115654[] = {{&S7904[44], 18}, {NULL}};
const Segment SGI115655[] = {{&S7904[30], 6}, {NULL}};
const Segment SGI115656[] = {{&S7904[31], 3}, {NULL}};
const Segment SGI115657[] = {{&S7904[76], 6}, {NULL}};
const Segment SGI115658[] = {{&S7905[44], 9}, {NULL}};
const Segment SGI115659[] = {{&S7905[44], 21}, {NULL}};
const Segment SGI115660[] = {{&S7905[30], 6}, {NULL}};
const Segment SGI115661[] = {{&S7905[31], 3}, {NULL}};
const Segment SGI115662[] = {{&S7905[79], 6}, {NULL}};
const Segment SGI115663[] = {{&S7908[44], 8}, {NULL}};
const Segment SGI115664[] = {{&S7908[44], 20}, {NULL}};
const Segment SGI115665[] = {{&S7908[30], 6}, {NULL}};
const Segment SGI115666[] = {{&S7908[31], 3}, {NULL}};
const Segment SGI115667[] = {{&S7908[78], 6}, {NULL}};
const Segment SGI115668[] = {{&S7909[44], 10}, {NULL}};
const Segment SGI115669[] = {{&S7909[44], 22}, {NULL}};
const Segment SGI115670[] = {{&S7909[30], 6}, {NULL}};
const Segment SGI115671[] = {{&S7909[31], 3}, {NULL}};
const Segment SGI115672[] = {{&S7909[80], 6}, {NULL}};
const Segment SGI115673[] = {{&S7910[44], 11}, {NULL}};
const Segment SGI115674[] = {{&S7910[44], 23}, {NULL}};
const Segment SGI115675[] = {{&S7910[30], 6}, {NULL}};
const Segment SGI115676[] = {{&S7910[31], 3}, {NULL}};
const Segment SGI115677[] = {{&S7910[81], 6}, {NULL}};
const Segment SGI115678[] = {{&S7911[44], 24}, {NULL}};
const Segment SGI115679[] = {{&S7911[30], 6}, {NULL}};
const Segment SGI115680[] = {{&S7911[31], 3}, {NULL}};
const Segment SGI115681[] = {{&S7911[100], 5}, {NULL}};
const Segment SGI115682[] = {{&S90912[44], 23}, {NULL}};
const Segment SGI115683[] = {{&S7912[44], 10}, {NULL}};
const Segment SGI115684[] = {{&S90912[44], 35}, {NULL}};
const Segment SGI115685[] = {{&S7912[44], 22}, {NULL}};
const Segment SGI115686[] = {{&S90912[30], 6}, {NULL}};
const Segment SGI115687[] = {{&S7912[30], 6}, {NULL}};
const Segment SGI115688[] = {{&S7912[31], 3}, {NULL}};
const Segment SGI115689[] = {{&S7912[80], 6}, {NULL}};
const Segment SGI115690[] = {{&S90912[93], 6}, {NULL}};
const Segment SGI115691[] = {{&S90912[120], 13}, {NULL}};
const Segment SGI115692[] = {{&S7913[44], 6}, {NULL}};
const Segment SGI115693[] = {{&S7913[44], 18}, {NULL}};
const Segment SGI115694[] = {{&S7913[30], 6}, {NULL}};
const Segment SGI115695[] = {{&S7913[31], 3}, {NULL}};
const Segment SGI115696[] = {{&S7913[76], 6}, {NULL}};
const Segment SGI115697[] = {{&S7914[44], 11}, {NULL}};
const Segment SGI115698[] = {{&S7914[44], 27}, {NULL}};
const Segment SGI115699[] = {{&S7914[30], 6}, {NULL}};
const Segment SGI115700[] = {{&S7914[31], 3}, {NULL}};
const Segment SGI115701[] = {{&S7914[85], 6}, {NULL}};
const Segment SGI115702[] = {{&S7915[44], 6}, {NULL}};
const Segment SGI115703[] = {{&S7915[44], 18}, {NULL}};
const Segment SGI115704[] = {{&S7915[30], 6}, {NULL}};
const Segment SGI115705[] = {{&S90917[44], 18}, {NULL}};
const Segment SGI115706[] = {{&S90917[44], 30}, {NULL}};
const Segment SGI115707[] = {{&S7915[31], 3}, {NULL}};
const Segment SGI115708[] = {{&S90917[30], 6}, {NULL}};
const Segment SGI115709[] = {{&S7915[76], 6}, {NULL}};
const Segment SGI115710[] = {{&S90917[88], 6}, {NULL}};
const Segment SGI115711[] = {{&S90917[115], 13}, {NULL}};
const Segment SGI115712[] = {{&S7916[44], 8}, {NULL}};
const Segment SGI115713[] = {{&S7916[44], 20}, {NULL}};
const Segment SGI115714[] = {{&S7916[30], 6}, {NULL}};
const Segment SGI115715[] = {{&S7916[31], 3}, {NULL}};
const Segment SGI115716[] = {{&S7916[78], 6}, {NULL}};
const Segment SGI115717[] = {{&S7917[44], 14}, {NULL}};
const Segment SGI115718[] = {{&S7917[44], 26}, {NULL}};
const Segment SGI115719[] = {{&S7917[30], 6}, {NULL}};
const Segment SGI115720[] = {{&S7917[87], 6}, {NULL}};
const Segment SGI115721[] = {{&S90921[34], 10}, {NULL}};
const Segment SGI115722[] = {{&S90921[27], 6}, {NULL}};
const Segment SGI115723[] = {{&S7918[44], 6}, {NULL}};
const Segment SGI115724[] = {{&S7918[44], 18}, {NULL}};
const Segment SGI115725[] = {{&S7918[30], 6}, {NULL}};
const Segment SGI115726[] = {{&S90921[97], 9}, {NULL}};
const Segment SGI115727[] = {{&S90921[70], 6}, {NULL}};
const Segment SGI115728[] = {{&S7918[31], 3}, {NULL}};
const Segment SGI115729[] = {{&S7918[76], 6}, {NULL}};
const Segment SGI115730[] = {{&S7919[44], 37}, {NULL}};
const Segment SGI115731[] = {{&S7919[30], 6}, {NULL}};
const Segment SGI115732[] = {{&S7919[31], 3}, {NULL}};
const Segment SGI115733[] = {{&S7919[113], 6}, {NULL}};
const Segment SGI115734[] = {{&S7922[44], 5}, {NULL}};
const Segment SGI115735[] = {{&S7922[44], 17}, {NULL}};
const Segment SGI115736[] = {{&S7922[30], 6}, {NULL}};
const Segment SGI115737[] = {{&S7922[31], 3}, {NULL}};
const Segment SGI115738[] = {{&S7922[75], 6}, {NULL}};
const Segment SGI115739[] = {{&S7923[44], 9}, {NULL}};
const Segment SGI115740[] = {{&S7923[44], 38}, {NULL}};
const Segment SGI115741[] = {{&S7923[30], 6}, {NULL}};
const Segment SGI115742[] = {{&S7923[31], 3}, {NULL}};
const Segment SGI115743[] = {{&S90931[44], 14}, {NULL}};
const Segment SGI115744[] = {{&S90931[44], 26}, {NULL}};
const Segment SGI115745[] = {{&S7923[96], 6}, {NULL}};
const Segment SGI115746[] = {{&S90931[30], 6}, {NULL}};
const Segment SGI115747[] = {{&S90931[84], 6}, {NULL}};
const Segment SGI115748[] = {{&S90931[111], 13}, {NULL}};
const Segment SGI115749[] = {{&S7924[44], 12}, {NULL}};
const Segment SGI115750[] = {{&S7924[44], 24}, {NULL}};
const Segment SGI115751[] = {{&S7924[30], 6}, {NULL}};
const Segment SGI115752[] = {{&S7924[31], 3}, {NULL}};
const Segment SGI115753[] = {{&S7924[82], 6}, {NULL}};
const Segment SGI115754[] = {{&S7925[44], 6}, {NULL}};
const Segment SGI115755[] = {{&S7925[44], 18}, {NULL}};
const Segment SGI115756[] = {{&S7925[30], 6}, {NULL}};
const Segment SGI115757[] = {{&S7925[31], 3}, {NULL}};
const Segment SGI115758[] = {{&S7925[76], 6}, {NULL}};
const Segment SGI115759[] = {{&S7926[44], 7}, {NULL}};
const Segment SGI115760[] = {{&S7926[44], 19}, {NULL}};
const Segment SGI115761[] = {{&S7926[30], 6}, {NULL}};
const Segment SGI115762[] = {{&S7926[31], 3}, {NULL}};
const Segment SGI115763[] = {{&S7926[77], 6}, {NULL}};
const Segment SGI115764[] = {{&S7927[44], 13}, {NULL}};
const Segment SGI115765[] = {{&S7927[44], 25}, {NULL}};
const Segment SGI115766[] = {{&S7927[30], 6}, {NULL}};
const Segment SGI115767[] = {{&S7927[31], 3}, {NULL}};
const Segment SGI115768[] = {{&S7927[83], 6}, {NULL}};
const Segment SGI115769[] = {{&S7928[44], 15}, {NULL}};
const Segment SGI115770[] = {{&S7928[44], 27}, {NULL}};
const Segment SGI115771[] = {{&S7928[30], 6}, {NULL}};
const Segment SGI115772[] = {{&S7928[31], 3}, {NULL}};
const Segment SGI115773[] = {{&S7928[85], 6}, {NULL}};
const Segment SGI115774[] = {{&S7931[44], 7}, {NULL}};
const Segment SGI115775[] = {{&S7931[44], 19}, {NULL}};
const Segment SGI115776[] = {{&S7931[30], 6}, {NULL}};
const Segment SGI115777[] = {{&S7931[31], 3}, {NULL}};
const Segment SGI115778[] = {{&S7931[77], 6}, {NULL}};
const Segment SGI115779[] = {{&S7933[44], 13}, {NULL}};
const Segment SGI115780[] = {{&S7933[44], 25}, {NULL}};
const Segment SGI115781[] = {{&S7933[30], 6}, {NULL}};
const Segment SGI115782[] = {{&S7933[31], 3}, {NULL}};
const Segment SGI115783[] = {{&S7933[83], 6}, {NULL}};
const Segment SGI115784[] = {{&S7934[44], 21}, {NULL}};
const Segment SGI115785[] = {{&S7934[44], 33}, {NULL}};
const Segment SGI115786[] = {{&S7934[30], 6}, {NULL}};
const Segment SGI115787[] = {{&S7934[31], 3}, {NULL}};
const Segment SGI115788[] = {{&S7934[91], 6}, {NULL}};
const Segment SGI115789[] = {{&S7935[44], 6}, {NULL}};
const Segment SGI115790[] = {{&S7935[44], 29}, {NULL}};
const Segment SGI115791[] = {{&S7935[30], 6}, {NULL}};
const Segment SGI115792[] = {{&S7935[31], 3}, {NULL}};
const Segment SGI115793[] = {{&S7935[87], 6}, {NULL}};
const Segment SGI115794[] = {{&S7936[44], 11}, {NULL}};
const Segment SGI115795[] = {{&S7936[44], 26}, {NULL}};
const Segment SGI115796[] = {{&S7936[30], 6}, {NULL}};
const Segment SGI115797[] = {{&S7936[31], 3}, {NULL}};
const Segment SGI115798[] = {{&S7936[84], 6}, {NULL}};
const Segment SGI115799[] = {{&S7937[44], 33}, {NULL}};
const Segment SGI115800[] = {{&S7937[44], 45}, {NULL}};
const Segment SGI115801[] = {{&S7937[30], 6}, {NULL}};
const Segment SGI115802[] = {{&S7937[31], 3}, {NULL}};
const Segment SGI115803[] = {{&S7937[103], 6}, {NULL}};
const Segment SGI115804[] = {{&S7938[44], 23}, {NULL}};
const Segment SGI115805[] = {{&S7938[44], 35}, {NULL}};
const Segment SGI115806[] = {{&S7938[30], 6}, {NULL}};
const Segment SGI115807[] = {{&S7938[31], 3}, {NULL}};
const Segment SGI115808[] = {{&S7938[93], 6}, {NULL}};
const Segment SGI115809[] = {{&S7941[44], 14}, {NULL}};
const Segment SGI115810[] = {{&S7941[44], 26}, {NULL}};
const Segment SGI115811[] = {{&S7941[30], 6}, {NULL}};
const Segment SGI115812[] = {{&S7941[31], 3}, {NULL}};
const Segment SGI115813[] = {{&S7941[84], 6}, {NULL}};
const Segment SGI115814[] = {{&S90971[59], 14}, {NULL}};
const Segment SGI115815[] = {{&S90971[59], 31}, {NULL}};
const Segment SGI115816[] = {{&S90971[45], 6}, {NULL}};
const Segment SGI115817[] = {{&S7942[44], 6}, {NULL}};
const Segment SGI115818[] = {{&S7942[44], 18}, {NULL}};
const Segment SGI115819[] = {{&S7942[30], 6}, {NULL}};
const Segment SGI115820[] = {{&S90971[104], 5}, {NULL}};
const Segment SGI115821[] = {{&S7942[31], 3}, {NULL}};
const Segment SGI115822[] = {{&S90971[0], 14}, {NULL}};
const Segment SGI115823[] = {{&S7942[76], 6}, {NULL}};
const Segment SGI115824[] = {{&S90972[44], 14}, {NULL}};
const Segment SGI115825[] = {{&S90972[44], 31}, {NULL}};
const Segment SGI115826[] = {{&S90972[30], 6}, {NULL}};
const Segment SGI115827[] = {{&S7943[44], 14}, {NULL}};
const Segment SGI115828[] = {{&S7943[44], 26}, {NULL}};
const Segment SGI115829[] = {{&S90972[31], 3}, {NULL}};
const Segment SGI115830[] = {{&S7943[30], 6}, {NULL}};
const Segment SGI115831[] = {{&S90972[89], 5}, {NULL}};
const Segment SGI115832[] = {{&S7943[31], 3}, {NULL}};
const Segment SGI115833[] = {{&S90972[134], 14}, {NULL}};
const Segment SGI115834[] = {{&S7943[84], 6}, {NULL}};
const Segment SGI115835[] = {{&S7944[44], 9}, {NULL}};
const Segment SGI115836[] = {{&S7944[44], 21}, {NULL}};
const Segment SGI115837[] = {{&S7944[30], 6}, {NULL}};
const Segment SGI115838[] = {{&S90974[59], 14}, {NULL}};
const Segment SGI115839[] = {{&S7944[31], 3}, {NULL}};
const Segment SGI115840[] = {{&S90974[59], 26}, {NULL}};
const Segment SGI115841[] = {{&S90974[45], 6}, {NULL}};
const Segment SGI115842[] = {{&S7944[79], 6}, {NULL}};
const Segment SGI115843[] = {{&S90974[46], 3}, {NULL}};
const Segment SGI115844[] = {{&S90974[99], 6}, {NULL}};
const Segment SGI115845[] = {{&S90974[0], 14}, {NULL}};
const Segment SGI115846[] = {{&S7945[44], 17}, {NULL}};
const Segment SGI115847[] = {{&S7945[44], 29}, {NULL}};
const Segment SGI115848[] = {{&S7945[30], 6}, {NULL}};
const Segment SGI115849[] = {{&S90975[34], 8}, {NULL}};
const Segment SGI115850[] = {{&S7945[31], 3}, {NULL}};
const Segment SGI115851[] = {{&S90975[27], 6}, {NULL}};
const Segment SGI115852[] = {{&S7945[87], 6}, {NULL}};
const Segment SGI115853[] = {{&S90975[95], 9}, {NULL}};
const Segment SGI115854[] = {{&S90975[68], 6}, {NULL}};
const Segment SGI115855[] = {{&S7946[44], 8}, {NULL}};
const Segment SGI115856[] = {{&S7946[44], 20}, {NULL}};
const Segment SGI115857[] = {{&S7946[30], 6}, {NULL}};
const Segment SGI115858[] = {{&S7946[31], 3}, {NULL}};
const Segment SGI115859[] = {{&S7946[78], 6}, {NULL}};
const Segment SGI115860[] = {{&S7947[44], 7}, {NULL}};
const Segment SGI115861[] = {{&S7947[44], 19}, {NULL}};
const Segment SGI115862[] = {{&S7947[30], 6}, {NULL}};
const Segment SGI115863[] = {{&S7947[31], 3}, {NULL}};
const Segment SGI115864[] = {{&S7947[77], 6}, {NULL}};
const Segment SGI115865[] = {{&S7949[44], 7}, {NULL}};
const Segment SGI115866[] = {{&S7949[44], 19}, {NULL}};
const Segment SGI115867[] = {{&S7949[30], 6}, {NULL}};
const Segment SGI115868[] = {{&S7949[31], 3}, {NULL}};
const Segment SGI115869[] = {{&S7949[77], 6}, {NULL}};
const Segment SGI115870[] = {{&S7950[44], 6}, {NULL}};
const Segment SGI115871[] = {{&S7950[44], 18}, {NULL}};
const Segment SGI115872[] = {{&S7950[30], 6}, {NULL}};
const Segment SGI115873[] = {{&S7950[31], 3}, {NULL}};
const Segment SGI115874[] = {{&S7950[76], 6}, {NULL}};
const Segment SGI115875[] = {{&S7951[44], 15}, {NULL}};
const Segment SGI115876[] = {{&S7951[44], 27}, {NULL}};
const Segment SGI115877[] = {{&S7951[30], 6}, {NULL}};
const Segment SGI115878[] = {{&S7951[31], 3}, {NULL}};
const Segment SGI115879[] = {{&S7951[85], 6}, {NULL}};
const Segment SGI115880[] = {{&S7952[44], 16}, {NULL}};
const Segment SGI115881[] = {{&S7952[44], 28}, {NULL}};
const Segment SGI115882[] = {{&S7952[30], 6}, {NULL}};
const Segment SGI115883[] = {{&S7952[31], 3}, {NULL}};
const Segment SGI115884[] = {{&S7952[86], 6}, {NULL}};
const Segment SGI115885[] = {{&S7954[44], 12}, {NULL}};
const Segment SGI115886[] = {{&S7954[44], 24}, {NULL}};
const Segment SGI115887[] = {{&S7954[30], 6}, {NULL}};
const Segment SGI115888[] = {{&S7954[31], 3}, {NULL}};
const Segment SGI115889[] = {{&S7954[82], 6}, {NULL}};
const Segment SGI115890[] = {{&S7955[44], 21}, {NULL}};
const Segment SGI115891[] = {{&S7955[30], 6}, {NULL}};
const Segment SGI115892[] = {{&S7955[31], 3}, {NULL}};
const Segment SGI115893[] = {{&S7955[96], 6}, {NULL}};
const Segment SGI115894[] = {{&S7956[44], 11}, {NULL}};
const Segment SGI115895[] = {{&S7956[44], 26}, {NULL}};
const Segment SGI115896[] = {{&S7956[30], 6}, {NULL}};
const Segment SGI115897[] = {{&S7956[31], 3}, {NULL}};
const Segment SGI115898[] = {{&S7956[84], 6}, {NULL}};
const Segment SGI115899[] = {{&S7957[44], 6}, {NULL}};
const Segment SGI115900[] = {{&S7957[44], 18}, {NULL}};
const Segment SGI115901[] = {{&S7957[30], 6}, {NULL}};
const Segment SGI115902[] = {{&S7957[31], 3}, {NULL}};
const Segment SGI115903[] = {{&S7957[76], 6}, {NULL}};
const Segment SGI115904[] = {{&S7959[44], 6}, {NULL}};
const Segment SGI115905[] = {{&S7959[44], 18}, {NULL}};
const Segment SGI115906[] = {{&S7959[30], 6}, {NULL}};
const Segment SGI115907[] = {{&S7959[31], 3}, {NULL}};
const Segment SGI115908[] = {{&S7959[76], 6}, {NULL}};
const Segment SGI115909[] = {{&S7961[44], 6}, {NULL}};
const Segment SGI115910[] = {{&S7961[44], 18}, {NULL}};
const Segment SGI115911[] = {{&S7961[30], 6}, {NULL}};
const Segment SGI115912[] = {{&S7961[31], 3}, {NULL}};
const Segment SGI115913[] = {{&S7961[76], 6}, {NULL}};
const Segment SGI115914[] = {{&S7962[44], 7}, {NULL}};
const Segment SGI115915[] = {{&S7962[44], 19}, {NULL}};
const Segment SGI115916[] = {{&S7962[30], 6}, {NULL}};
const Segment SGI115917[] = {{&S7962[31], 3}, {NULL}};
const Segment SGI115918[] = {{&S7962[77], 6}, {NULL}};
const Segment SGI115919[] = {{&S7963[44], 9}, {NULL}};
const Segment SGI115920[] = {{&S7963[44], 21}, {NULL}};
const Segment SGI115921[] = {{&S7963[30], 6}, {NULL}};
const Segment SGI115922[] = {{&S7963[31], 3}, {NULL}};
const Segment SGI115923[] = {{&S7963[79], 6}, {NULL}};
const Segment SGI115924[] = {{&S7964[44], 6}, {NULL}};
const Segment SGI115925[] = {{&S7964[44], 18}, {NULL}};
const Segment SGI115926[] = {{&S7964[30], 6}, {NULL}};
const Segment SGI115927[] = {{&S7964[31], 3}, {NULL}};
const Segment SGI115928[] = {{&S7964[76], 6}, {NULL}};
const Segment SGI115929[] = {{&S7965[44], 8}, {NULL}};
const Segment SGI115930[] = {{&S7965[44], 20}, {NULL}};
const Segment SGI115931[] = {{&S7965[30], 6}, {NULL}};
const Segment SGI115932[] = {{&S7965[31], 3}, {NULL}};
const Segment SGI115933[] = {{&S7965[78], 6}, {NULL}};
const Segment SGI115934[] = {{&S7966[44], 10}, {NULL}};
const Segment SGI115935[] = {{&S7966[44], 22}, {NULL}};
const Segment SGI115936[] = {{&S7966[30], 6}, {NULL}};
const Segment SGI115937[] = {{&S7966[31], 3}, {NULL}};
const Segment SGI115938[] = {{&S7966[80], 6}, {NULL}};
const Segment SGI115939[] = {{&S7967[44], 10}, {NULL}};
const Segment SGI115940[] = {{&S7967[44], 19}, {NULL}};
const Segment SGI115941[] = {{&S7967[30], 6}, {NULL}};
const Segment SGI115942[] = {{&S7967[31], 3}, {NULL}};
const Segment SGI115943[] = {{&S7967[77], 6}, {NULL}};
const Segment SGI115944[] = {{&S7968[44], 11}, {NULL}};
const Segment SGI115945[] = {{&S7968[44], 23}, {NULL}};
const Segment SGI115946[] = {{&S7968[30], 6}, {NULL}};
const Segment SGI115947[] = {{&S7968[31], 3}, {NULL}};
const Segment SGI115948[] = {{&S7968[81], 6}, {NULL}};
const Segment SGI115949[] = {{&S7969[44], 8}, {NULL}};
const Segment SGI115950[] = {{&S7969[44], 20}, {NULL}};
const Segment SGI115951[] = {{&S7969[30], 6}, {NULL}};
const Segment SGI115952[] = {{&S7969[31], 3}, {NULL}};
const Segment SGI115953[] = {{&S7969[78], 6}, {NULL}};
const Segment SGI115954[] = {{&S7970[44], 8}, {NULL}};
const Segment SGI115955[] = {{&S7970[44], 20}, {NULL}};
const Segment SGI115956[] = {{&S7970[30], 6}, {NULL}};
const Segment SGI115957[] = {{&S7970[31], 3}, {NULL}};
const Segment SGI115958[] = {{&S7970[78], 6}, {NULL}};
const Segment SGI115959[] = {{&S7972[44], 6}, {NULL}};
const Segment SGI115960[] = {{&S7972[44], 18}, {NULL}};
const Segment SGI115961[] = {{&S7972[30], 6}, {NULL}};
const Segment SGI115962[] = {{&S7972[31], 3}, {NULL}};
const Segment SGI115963[] = {{&S7972[76], 6}, {NULL}};
const Segment SGI115964[] = {{&S7973[44], 21}, {NULL}};
const Segment SGI115965[] = {{&S7973[44], 33}, {NULL}};
const Segment SGI115966[] = {{&S7973[30], 6}, {NULL}};
const Segment SGI115967[] = {{&S7973[31], 3}, {NULL}};
const Segment SGI115968[] = {{&S7973[91], 6}, {NULL}};
const Segment SGI115969[] = {{&S7974[44], 10}, {NULL}};
const Segment SGI115970[] = {{&S7974[44], 23}, {NULL}};
const Segment SGI115971[] = {{&S7974[30], 6}, {NULL}};
const Segment SGI115972[] = {{&S7974[31], 3}, {NULL}};
const Segment SGI115973[] = {{&S7974[81], 6}, {NULL}};
const Segment SGI115974[] = {{&S7976[44], 14}, {NULL}};
const Segment SGI115975[] = {{&S7976[44], 26}, {NULL}};
const Segment SGI115976[] = {{&S7976[30], 6}, {NULL}};
const Segment SGI115977[] = {{&S7976[31], 3}, {NULL}};
const Segment SGI115978[] = {{&S7976[84], 6}, {NULL}};
const Segment SGI115979[] = {{&S7977[44], 12}, {NULL}};
const Segment SGI115980[] = {{&S7977[44], 24}, {NULL}};
const Segment SGI115981[] = {{&S7977[30], 6}, {NULL}};
const Segment SGI115982[] = {{&S7977[31], 3}, {NULL}};
const Segment SGI115983[] = {{&S7977[82], 6}, {NULL}};
const Segment SGI115984[] = {{&S7978[44], 10}, {NULL}};
const Segment SGI115985[] = {{&S7978[44], 22}, {NULL}};
const Segment SGI115986[] = {{&S7978[30], 6}, {NULL}};
const Segment SGI115987[] = {{&S7978[31], 3}, {NULL}};
const Segment SGI115988[] = {{&S7978[80], 6}, {NULL}};
const Segment SGI115989[] = {{&S91045[44], 17}, {NULL}};
const Segment SGI115990[] = {{&S91045[44], 38}, {NULL}};
const Segment SGI115991[] = {{&S91045[30], 6}, {NULL}};
const Segment SGI115992[] = {{&S91045[31], 3}, {NULL}};
const Segment SGI115993[] = {{&S91045[96], 6}, {NULL}};
const Segment SGI115994[] = {{&S7979[82], 6}, {NULL}};
const Segment SGI115995[] = {{&S7980[44], 8}, {NULL}};
const Segment SGI115996[] = {{&S7980[44], 20}, {NULL}};
const Segment SGI115997[] = {{&S7980[30], 6}, {NULL}};
const Segment SGI115998[] = {{&S7980[31], 3}, {NULL}};
const Segment SGI115999[] = {{&S7980[78], 6}, {NULL}};
const Segment SGI116000[] = {{&S7981[44], 14}, {NULL}};
const Segment SGI116001[] = {{&S7981[44], 26}, {NULL}};
const Segment SGI116002[] = {{&S7981[30], 6}, {NULL}};
const Segment SGI116003[] = {{&S7981[31], 3}, {NULL}};
const Segment SGI116004[] = {{&S7981[84], 6}, {NULL}};
const Segment SGI116005[] = {{&S7983[60], 5}, {NULL}};
const Segment SGI116006[] = {{&S7983[43], 16}, {NULL}};
const Segment SGI116007[] = {{&S7983[30], 6}, {NULL}};
const Segment SGI116008[] = {{&S7983[31], 3}, {NULL}};
const Segment SGI116009[] = {{&S7983[88], 5}, {NULL}};
const Segment SGI116010[] = {{&S7985[44], 9}, {NULL}};
const Segment SGI116011[] = {{&S7985[44], 21}, {NULL}};
const Segment SGI116012[] = {{&S7985[30], 6}, {NULL}};
const Segment SGI116013[] = {{&S7985[31], 3}, {NULL}};
const Segment SGI116014[] = {{&S7985[79], 6}, {NULL}};
const Segment SGI116015[] = {{&S7986[44], 6}, {NULL}};
const Segment SGI116016[] = {{&S7986[44], 18}, {NULL}};
const Segment SGI116017[] = {{&S7986[30], 6}, {NULL}};
const Segment SGI116018[] = {{&S7986[31], 3}, {NULL}};
const Segment SGI116019[] = {{&S7986[76], 6}, {NULL}};
const Segment SGI116020[] = {{&S7987[44], 6}, {NULL}};
const Segment SGI116021[] = {{&S7987[44], 18}, {NULL}};
const Segment SGI116022[] = {{&S7987[30], 6}, {NULL}};
const Segment SGI116023[] = {{&S7987[31], 3}, {NULL}};
const Segment SGI116024[] = {{&S7987[76], 6}, {NULL}};
const Segment SGI116025[] = {{&S7988[44], 11}, {NULL}};
const Segment SGI116026[] = {{&S7988[30], 6}, {NULL}};
const Segment SGI116027[] = {{&S7988[31], 3}, {NULL}};
const Segment SGI116028[] = {{&S7988[83], 6}, {NULL}};
const Segment SGI116029[] = {{&S7989[44], 19}, {NULL}};
const Segment SGI116030[] = {{&S7989[44], 31}, {NULL}};
const Segment SGI116031[] = {{&S7989[30], 6}, {NULL}};
const Segment SGI116032[] = {{&S7989[31], 3}, {NULL}};
const Segment SGI116033[] = {{&S7989[89], 6}, {NULL}};
const Segment SGI116034[] = {{&S7991[44], 7}, {NULL}};
const Segment SGI116035[] = {{&S7991[44], 19}, {NULL}};
const Segment SGI116036[] = {{&S7991[30], 6}, {NULL}};
const Segment SGI116037[] = {{&S7991[31], 3}, {NULL}};
const Segment SGI116038[] = {{&S7991[77], 6}, {NULL}};
const Segment SGI116039[] = {{&S7992[44], 8}, {NULL}};
const Segment SGI116040[] = {{&S7992[44], 20}, {NULL}};
const Segment SGI116041[] = {{&S7992[30], 6}, {NULL}};
const Segment SGI116042[] = {{&S7992[31], 3}, {NULL}};
const Segment SGI116043[] = {{&S7992[78], 6}, {NULL}};
const Segment SGI116044[] = {{&S7993[44], 12}, {NULL}};
const Segment SGI116045[] = {{&S7993[44], 24}, {NULL}};
const Segment SGI116046[] = {{&S7993[30], 6}, {NULL}};
const Segment SGI116047[] = {{&S7993[31], 3}, {NULL}};
const Segment SGI116048[] = {{&S7993[82], 6}, {NULL}};
const Segment SGI116049[] = {{&S7995[44], 6}, {NULL}};
const Segment SGI116050[] = {{&S7995[44], 18}, {NULL}};
const Segment SGI116051[] = {{&S7995[30], 6}, {NULL}};
const Segment SGI116052[] = {{&S7995[31], 3}, {NULL}};
const Segment SGI116053[] = {{&S7995[76], 6}, {NULL}};
const Segment SGI116054[] = {{&S7999[44], 20}, {NULL}};
const Segment SGI116055[] = {{&S7999[44], 32}, {NULL}};
const Segment SGI116056[] = {{&S7999[30], 6}, {NULL}};
const Segment SGI116057[] = {{&S7999[31], 3}, {NULL}};
const Segment SGI116058[] = {{&S7999[90], 6}, {NULL}};
const Segment SGI116059[] = {{&S8000[44], 8}, {NULL}};
const Segment SGI116060[] = {{&S8000[44], 20}, {NULL}};
const Segment SGI116061[] = {{&S8000[30], 6}, {NULL}};
const Segment SGI116062[] = {{&S8000[31], 3}, {NULL}};
const Segment SGI116063[] = {{&S8000[78], 6}, {NULL}};
const Segment SGI116064[] = {{&S8001[44], 12}, {NULL}};
const Segment SGI116065[] = {{&S8001[44], 24}, {NULL}};
const Segment SGI116066[] = {{&S8001[30], 6}, {NULL}};
const Segment SGI116067[] = {{&S8001[31], 3}, {NULL}};
const Segment SGI116068[] = {{&S8001[82], 6}, {NULL}};
const Segment SGI116069[] = {{&S8002[44], 6}, {NULL}};
const Segment SGI116070[] = {{&S8002[44], 18}, {NULL}};
const Segment SGI116071[] = {{&S8002[30], 6}, {NULL}};
const Segment SGI116072[] = {{&S8002[31], 3}, {NULL}};
const Segment SGI116073[] = {{&S8002[76], 6}, {NULL}};
const Segment SGI116074[] = {{&S8005[44], 14}, {NULL}};
const Segment SGI116075[] = {{&S8005[44], 26}, {NULL}};
const Segment SGI116076[] = {{&S8005[30], 6}, {NULL}};
const Segment SGI116077[] = {{&S8005[31], 3}, {NULL}};
const Segment SGI116078[] = {{&S8005[84], 6}, {NULL}};
const Segment SGI116079[] = {{&S8007[44], 6}, {NULL}};
const Segment SGI116080[] = {{&S8007[44], 18}, {NULL}};
const Segment SGI116081[] = {{&S8007[30], 6}, {NULL}};
const Segment SGI116082[] = {{&S8007[31], 3}, {NULL}};
const Segment SGI116083[] = {{&S8007[76], 6}, {NULL}};
const Segment SGI116084[] = {{&S8008[44], 12}, {NULL}};
const Segment SGI116085[] = {{&S8008[44], 24}, {NULL}};
const Segment SGI116086[] = {{&S8008[30], 6}, {NULL}};
const Segment SGI116087[] = {{&S8008[31], 3}, {NULL}};
const Segment SGI116088[] = {{&S8008[82], 6}, {NULL}};
const Segment SGI116089[] = {{&S8009[44], 9}, {NULL}};
const Segment SGI116090[] = {{&S8009[44], 21}, {NULL}};
const Segment SGI116091[] = {{&S8009[30], 6}, {NULL}};
const Segment SGI116092[] = {{&S8009[31], 3}, {NULL}};
const Segment SGI116093[] = {{&S8009[79], 6}, {NULL}};
const Segment SGI116094[] = {{&S8010[44], 18}, {NULL}};
const Segment SGI116095[] = {{&S8010[44], 30}, {NULL}};
const Segment SGI116096[] = {{&S8010[30], 6}, {NULL}};
const Segment SGI116097[] = {{&S8010[31], 3}, {NULL}};
const Segment SGI116098[] = {{&S8010[88], 6}, {NULL}};
const Segment SGI116099[] = {{&S8011[44], 6}, {NULL}};
const Segment SGI116100[] = {{&S8011[44], 18}, {NULL}};
const Segment SGI116101[] = {{&S8011[30], 6}, {NULL}};
const Segment SGI116102[] = {{&S8011[31], 3}, {NULL}};
const Segment SGI116103[] = {{&S8011[76], 5}, {NULL}};
const Segment SGI116104[] = {{&S8012[44], 8}, {NULL}};
const Segment SGI116105[] = {{&S8012[44], 20}, {NULL}};
const Segment SGI116106[] = {{&S8012[30], 6}, {NULL}};
const Segment SGI116107[] = {{&S8012[31], 3}, {NULL}};
const Segment SGI116108[] = {{&S8012[78], 6}, {NULL}};
const Segment SGI116109[] = {{&S8013[44], 10}, {NULL}};
const Segment SGI116110[] = {{&S8013[44], 22}, {NULL}};
const Segment SGI116111[] = {{&S8013[30], 6}, {NULL}};
const Segment SGI116112[] = {{&S8013[31], 3}, {NULL}};
const Segment SGI116113[] = {{&S8013[80], 6}, {NULL}};
const Segment SGI116114[] = {{&S8014[44], 8}, {NULL}};
const Segment SGI116115[] = {{&S8014[44], 20}, {NULL}};
const Segment SGI116116[] = {{&S8014[30], 6}, {NULL}};
const Segment SGI116117[] = {{&S8014[31], 3}, {NULL}};
const Segment SGI116118[] = {{&S8014[78], 6}, {NULL}};
const Segment SGI116119[] = {{&S8017[44], 6}, {NULL}};
const Segment SGI116120[] = {{&S8017[44], 18}, {NULL}};
const Segment SGI116121[] = {{&S8017[30], 6}, {NULL}};
const Segment SGI116122[] = {{&S8017[31], 3}, {NULL}};
const Segment SGI116123[] = {{&S8017[76], 6}, {NULL}};
const Segment SGI116124[] = {{&S8020[44], 14}, {NULL}};
const Segment SGI116125[] = {{&S8020[44], 26}, {NULL}};
const Segment SGI116126[] = {{&S8020[30], 6}, {NULL}};
const Segment SGI116127[] = {{&S8020[31], 3}, {NULL}};
const Segment SGI116128[] = {{&S8020[84], 6}, {NULL}};
const Segment SGI116129[] = {{&S8021[44], 14}, {NULL}};
const Segment SGI116130[] = {{&S8021[44], 26}, {NULL}};
const Segment SGI116131[] = {{&S8021[30], 6}, {NULL}};
const Segment SGI116132[] = {{&S8021[31], 3}, {NULL}};
const Segment SGI116133[] = {{&S8021[84], 6}, {NULL}};
const Segment SGI116134[] = {{&S91150[34], 5}, {NULL}};
const Segment SGI116135[] = {{&S91150[27], 6}, {NULL}};
const Segment SGI116136[] = {{&S91150[92], 9}, {NULL}};
const Segment SGI116137[] = {{&S91150[65], 6}, {NULL}};
const Segment SGI116138[] = {{&S8023[44], 5}, {NULL}};
const Segment SGI116139[] = {{&S8023[44], 17}, {NULL}};
const Segment SGI116140[] = {{&S8023[30], 6}, {NULL}};
const Segment SGI116141[] = {{&S8023[31], 3}, {NULL}};
const Segment SGI116142[] = {{&S8023[75], 6}, {NULL}};
const Segment SGI116143[] = {{&S91155[34], 19}, {NULL}};
const Segment SGI116144[] = {{&S91155[27], 6}, {NULL}};
const Segment SGI116145[] = {{&S91155[111], 9}, {NULL}};
const Segment SGI116146[] = {{&S91155[84], 6}, {NULL}};
const Segment SGI116147[] = {{&S8024[44], 16}, {NULL}};
const Segment SGI116148[] = {{&S8024[44], 28}, {NULL}};
const Segment SGI116149[] = {{&S8024[30], 6}, {NULL}};
const Segment SGI116150[] = {{&S8024[31], 3}, {NULL}};
const Segment SGI116151[] = {{&S8024[86], 6}, {NULL}};
const Segment SGI116152[] = {{&S8025[44], 11}, {NULL}};
const Segment SGI116153[] = {{&S8025[44], 23}, {NULL}};
const Segment SGI116154[] = {{&S8025[30], 6}, {NULL}};
const Segment SGI116155[] = {{&S8025[31], 3}, {NULL}};
const Segment SGI116156[] = {{&S8025[81], 6}, {NULL}};
const Segment SGI116157[] = {{&S8026[44], 9}, {NULL}};
const Segment SGI116158[] = {{&S8026[44], 21}, {NULL}};
const Segment SGI116159[] = {{&S8026[30], 6}, {NULL}};
const Segment SGI116160[] = {{&S8026[31], 3}, {NULL}};
const Segment SGI116161[] = {{&S8026[79], 6}, {NULL}};
const Segment SGI116162[] = {{&S8028[44], 14}, {NULL}};
const Segment SGI116163[] = {{&S8028[44], 26}, {NULL}};
const Segment SGI116164[] = {{&S8028[30], 6}, {NULL}};
const Segment SGI116165[] = {{&S8028[31], 3}, {NULL}};
const Segment SGI116166[] = {{&S8028[84], 6}, {NULL}};
const Segment SGI116167[] = {{&S8029[44], 8}, {NULL}};
const Segment SGI116168[] = {{&S8029[44], 20}, {NULL}};
const Segment SGI116169[] = {{&S8029[30], 6}, {NULL}};
const Segment SGI116170[] = {{&S8029[31], 3}, {NULL}};
const Segment SGI116171[] = {{&S8029[78], 6}, {NULL}};
const Segment SGI116172[] = {{&S8030[44], 11}, {NULL}};
const Segment SGI116173[] = {{&S8030[44], 23}, {NULL}};
const Segment SGI116174[] = {{&S8030[30], 6}, {NULL}};
const Segment SGI116175[] = {{&S8030[31], 3}, {NULL}};
const Segment SGI116176[] = {{&S8030[81], 6}, {NULL}};
const Segment SGI116177[] = {{&S8032[44], 14}, {NULL}};
const Segment SGI116178[] = {{&S8032[44], 26}, {NULL}};
const Segment SGI116179[] = {{&S8032[30], 6}, {NULL}};
const Segment SGI116180[] = {{&S8032[31], 3}, {NULL}};
const Segment SGI116181[] = {{&S8032[84], 6}, {NULL}};
const Segment SGI116182[] = {{&S8033[44], 6}, {NULL}};
const Segment SGI116183[] = {{&S8033[44], 18}, {NULL}};
const Segment SGI116184[] = {{&S8033[30], 6}, {NULL}};
const Segment SGI116185[] = {{&S8033[31], 3}, {NULL}};
const Segment SGI116186[] = {{&S8033[76], 6}, {NULL}};
const Segment SGI116187[] = {{&S8034[44], 4}, {NULL}};
const Segment SGI116188[] = {{&S8034[44], 25}, {NULL}};
const Segment SGI116189[] = {{&S8034[30], 6}, {NULL}};
const Segment SGI116190[] = {{&S8034[31], 3}, {NULL}};
const Segment SGI116191[] = {{&S8034[83], 5}, {NULL}};
const Segment SGI116192[] = {{&S8035[44], 15}, {NULL}};
const Segment SGI116193[] = {{&S8035[44], 27}, {NULL}};
const Segment SGI116194[] = {{&S8035[30], 6}, {NULL}};
const Segment SGI116195[] = {{&S8035[31], 3}, {NULL}};
const Segment SGI116196[] = {{&S8035[85], 6}, {NULL}};
const Segment SGI116197[] = {{&S8036[44], 9}, {NULL}};
const Segment SGI116198[] = {{&S8036[44], 21}, {NULL}};
const Segment SGI116199[] = {{&S8036[30], 6}, {NULL}};
const Segment SGI116200[] = {{&S8036[31], 3}, {NULL}};
const Segment SGI116201[] = {{&S8036[79], 6}, {NULL}};
const Segment SGI116202[] = {{&S8038[44], 8}, {NULL}};
const Segment SGI116203[] = {{&S8038[44], 20}, {NULL}};
const Segment SGI116204[] = {{&S8038[30], 6}, {NULL}};
const Segment SGI116205[] = {{&S8038[31], 3}, {NULL}};
const Segment SGI116206[] = {{&S8038[78], 6}, {NULL}};
const Segment SGI116207[] = {{&S8039[44], 10}, {NULL}};
const Segment SGI116208[] = {{&S8039[44], 22}, {NULL}};
const Segment SGI116209[] = {{&S8039[30], 6}, {NULL}};
const Segment SGI116210[] = {{&S8039[31], 3}, {NULL}};
const Segment SGI116211[] = {{&S8039[80], 6}, {NULL}};
const Segment SGI116212[] = {{&S8040[44], 18}, {NULL}};
const Segment SGI116213[] = {{&S8040[44], 30}, {NULL}};
const Segment SGI116214[] = {{&S8040[30], 6}, {NULL}};
const Segment SGI116215[] = {{&S8040[31], 3}, {NULL}};
const Segment SGI116216[] = {{&S8040[88], 6}, {NULL}};
const Segment SGI116217[] = {{&S8041[44], 11}, {NULL}};
const Segment SGI116218[] = {{&S8041[44], 23}, {NULL}};
const Segment SGI116219[] = {{&S8041[30], 6}, {NULL}};
const Segment SGI116220[] = {{&S8041[31], 3}, {NULL}};
const Segment SGI116221[] = {{&S8041[81], 6}, {NULL}};
const Segment SGI116222[] = {{&S8042[44], 9}, {NULL}};
const Segment SGI116223[] = {{&S8042[44], 21}, {NULL}};
const Segment SGI116224[] = {{&S8042[30], 6}, {NULL}};
const Segment SGI116225[] = {{&S8042[31], 3}, {NULL}};
const Segment SGI116226[] = {{&S8042[79], 6}, {NULL}};
const Segment SGI116227[] = {{&S8043[44], 6}, {NULL}};
const Segment SGI116228[] = {{&S8043[44], 18}, {NULL}};
const Segment SGI116229[] = {{&S8043[30], 6}, {NULL}};
const Segment SGI116230[] = {{&S8043[31], 3}, {NULL}};
const Segment SGI116231[] = {{&S8043[76], 5}, {NULL}};
const Segment SGI116232[] = {{&S8045[44], 5}, {NULL}};
const Segment SGI116233[] = {{&S8045[44], 17}, {NULL}};
const Segment SGI116234[] = {{&S8045[30], 6}, {NULL}};
const Segment SGI116235[] = {{&S8045[31], 3}, {NULL}};
const Segment SGI116236[] = {{&S8045[75], 6}, {NULL}};
const Segment SGI116237[] = {{&S8046[44], 12}, {NULL}};
const Segment SGI116238[] = {{&S8046[44], 24}, {NULL}};
const Segment SGI116239[] = {{&S8046[30], 6}, {NULL}};
const Segment SGI116240[] = {{&S8046[31], 3}, {NULL}};
const Segment SGI116241[] = {{&S8046[82], 6}, {NULL}};
const Segment SGI116242[] = {{&S8047[44], 9}, {NULL}};
const Segment SGI116243[] = {{&S8047[44], 21}, {NULL}};
const Segment SGI116244[] = {{&S8047[30], 6}, {NULL}};
const Segment SGI116245[] = {{&S8047[31], 3}, {NULL}};
const Segment SGI116246[] = {{&S8047[79], 6}, {NULL}};
const Segment SGI116247[] = {{&S8049[44], 10}, {NULL}};
const Segment SGI116248[] = {{&S8049[44], 22}, {NULL}};
const Segment SGI116249[] = {{&S8049[30], 6}, {NULL}};
const Segment SGI116250[] = {{&S8049[31], 3}, {NULL}};
const Segment SGI116251[] = {{&S8049[80], 6}, {NULL}};
const Segment SGI116252[] = {{&S8050[44], 7}, {NULL}};
const Segment SGI116253[] = {{&S8050[44], 19}, {NULL}};
const Segment SGI116254[] = {{&S8050[30], 6}, {NULL}};
const Segment SGI116255[] = {{&S8050[31], 3}, {NULL}};
const Segment SGI116256[] = {{&S8050[77], 6}, {NULL}};
const Segment SGI116257[] = {{&S8051[44], 5}, {NULL}};
const Segment SGI116258[] = {{&S8051[44], 17}, {NULL}};
const Segment SGI116259[] = {{&S8051[30], 6}, {NULL}};
const Segment SGI116260[] = {{&S8051[31], 3}, {NULL}};
const Segment SGI116261[] = {{&S8051[75], 6}, {NULL}};
const Segment SGI116262[] = {{&S8052[44], 12}, {NULL}};
const Segment SGI116263[] = {{&S8052[44], 24}, {NULL}};
const Segment SGI116264[] = {{&S91203[44], 14}, {NULL}};
const Segment SGI116265[] = {{&S8052[30], 6}, {NULL}};
const Segment SGI116266[] = {{&S91203[44], 26}, {NULL}};
const Segment SGI116267[] = {{&S91203[30], 6}, {NULL}};
const Segment SGI116268[] = {{&S8052[31], 3}, {NULL}};
const Segment SGI116269[] = {{&S8052[82], 6}, {NULL}};
const Segment SGI116270[] = {{&S91203[31], 3}, {NULL}};
const Segment SGI116271[] = {{&S91203[84], 6}, {NULL}};
const Segment SGI116272[] = {{&S8053[44], 7}, {NULL}};
const Segment SGI116273[] = {{&S8053[44], 19}, {NULL}};
const Segment SGI116274[] = {{&S8053[30], 6}, {NULL}};
const Segment SGI116275[] = {{&S8053[31], 3}, {NULL}};
const Segment SGI116276[] = {{&S8053[77], 6}, {NULL}};
const Segment SGI116277[] = {{&S8054[44], 16}, {NULL}};
const Segment SGI116278[] = {{&S8054[44], 28}, {NULL}};
const Segment SGI116279[] = {{&S8054[30], 6}, {NULL}};
const Segment SGI116280[] = {{&S8054[31], 3}, {NULL}};
const Segment SGI116281[] = {{&S8054[86], 6}, {NULL}};
const Segment SGI116282[] = {{&S8055[44], 7}, {NULL}};
const Segment SGI116283[] = {{&S8055[44], 25}, {NULL}};
const Segment SGI116284[] = {{&S8055[30], 6}, {NULL}};
const Segment SGI116285[] = {{&S8055[31], 3}, {NULL}};
const Segment SGI116286[] = {{&S8055[83], 5}, {NULL}};
const Segment SGI116287[] = {{&S8056[44], 9}, {NULL}};
const Segment SGI116288[] = {{&S8056[44], 29}, {NULL}};
const Segment SGI116289[] = {{&S8056[30], 6}, {NULL}};
const Segment SGI116290[] = {{&S8056[31], 3}, {NULL}};
const Segment SGI116291[] = {{&S8056[87], 6}, {NULL}};
const Segment SGI116292[] = {{&S8057[44], 10}, {NULL}};
const Segment SGI116293[] = {{&S8057[44], 22}, {NULL}};
const Segment SGI116294[] = {{&S8057[30], 6}, {NULL}};
const Segment SGI116295[] = {{&S8057[31], 3}, {NULL}};
const Segment SGI116296[] = {{&S8057[80], 6}, {NULL}};
const Segment SGI116297[] = {{&S8058[44], 9}, {NULL}};
const Segment SGI116298[] = {{&S8058[44], 22}, {NULL}};
const Segment SGI116299[] = {{&S8058[30], 6}, {NULL}};
const Segment SGI116300[] = {{&S8058[31], 3}, {NULL}};
const Segment SGI116301[] = {{&S8058[80], 6}, {NULL}};
const Segment SGI116302[] = {{&S8059[44], 8}, {NULL}};
const Segment SGI116303[] = {{&S8059[44], 20}, {NULL}};
const Segment SGI116304[] = {{&S8059[30], 6}, {NULL}};
const Segment SGI116305[] = {{&S8059[31], 3}, {NULL}};
const Segment SGI116306[] = {{&S8059[78], 6}, {NULL}};
const Segment SGI116307[] = {{&S8060[44], 7}, {NULL}};
const Segment SGI116308[] = {{&S8060[44], 19}, {NULL}};
const Segment SGI116309[] = {{&S8060[30], 6}, {NULL}};
const Segment SGI116310[] = {{&S8060[31], 3}, {NULL}};
const Segment SGI116311[] = {{&S8060[77], 6}, {NULL}};
const Segment SGI116312[] = {{&S8062[44], 10}, {NULL}};
const Segment SGI116313[] = {{&S8062[44], 22}, {NULL}};
const Segment SGI116314[] = {{&S8062[30], 6}, {NULL}};
const Segment SGI116315[] = {{&S8062[31], 3}, {NULL}};
const Segment SGI116316[] = {{&S8062[80], 6}, {NULL}};
const Segment SGI116317[] = {{&S8063[44], 10}, {NULL}};
const Segment SGI116318[] = {{&S8063[44], 22}, {NULL}};
const Segment SGI116319[] = {{&S8063[30], 6}, {NULL}};
const Segment SGI116320[] = {{&S8063[31], 3}, {NULL}};
const Segment SGI116321[] = {{&S8063[80], 6}, {NULL}};
const Segment SGI116322[] = {{&S8064[44], 10}, {NULL}};
const Segment SGI116323[] = {{&S8064[44], 22}, {NULL}};
const Segment SGI116324[] = {{&S8064[30], 6}, {NULL}};
const Segment SGI116325[] = {{&S8064[31], 3}, {NULL}};
const Segment SGI116326[] = {{&S8064[80], 6}, {NULL}};
const Segment SGI116327[] = {{&S8065[44], 8}, {NULL}};
const Segment SGI116328[] = {{&S8065[44], 20}, {NULL}};
const Segment SGI116329[] = {{&S8065[30], 6}, {NULL}};
const Segment SGI116330[] = {{&S8065[31], 3}, {NULL}};
const Segment SGI116331[] = {{&S8065[78], 6}, {NULL}};
const Segment SGI116332[] = {{&S8066[44], 11}, {NULL}};
const Segment SGI116333[] = {{&S8066[44], 23}, {NULL}};
const Segment SGI116334[] = {{&S8066[30], 6}, {NULL}};
const Segment SGI116335[] = {{&S8066[31], 3}, {NULL}};
const Segment SGI116336[] = {{&S8066[81], 6}, {NULL}};
const Segment SGI116337[] = {{&S8067[44], 10}, {NULL}};
const Segment SGI116338[] = {{&S8067[44], 22}, {NULL}};
const Segment SGI116339[] = {{&S8067[30], 6}, {NULL}};
const Segment SGI116340[] = {{&S8067[31], 3}, {NULL}};
const Segment SGI116341[] = {{&S8067[80], 6}, {NULL}};
const Segment SGI116342[] = {{&S8068[44], 11}, {NULL}};
const Segment SGI116343[] = {{&S8068[44], 23}, {NULL}};
const Segment SGI116344[] = {{&S8068[30], 6}, {NULL}};
const Segment SGI116345[] = {{&S8068[31], 3}, {NULL}};
const Segment SGI116346[] = {{&S8068[81], 6}, {NULL}};
const Segment SGI116347[] = {{&S8069[44], 13}, {NULL}};
const Segment SGI116348[] = {{&S8069[44], 25}, {NULL}};
const Segment SGI116349[] = {{&S8069[30], 6}, {NULL}};
const Segment SGI116350[] = {{&S8069[31], 3}, {NULL}};
const Segment SGI116351[] = {{&S8069[83], 6}, {NULL}};
const Segment SGI116352[] = {{&S8071[44], 5}, {NULL}};
const Segment SGI116353[] = {{&S8071[44], 17}, {NULL}};
const Segment SGI116354[] = {{&S8071[30], 6}, {NULL}};
const Segment SGI116355[] = {{&S8071[31], 3}, {NULL}};
const Segment SGI116356[] = {{&S8071[75], 6}, {NULL}};
const Segment SGI116357[] = {{&S8072[44], 10}, {NULL}};
const Segment SGI116358[] = {{&S8072[44], 22}, {NULL}};
const Segment SGI116359[] = {{&S8072[30], 6}, {NULL}};
const Segment SGI116360[] = {{&S8072[31], 3}, {NULL}};
const Segment SGI116361[] = {{&S8072[80], 6}, {NULL}};
const Segment SGI116362[] = {{&S8073[44], 9}, {NULL}};
const Segment SGI116363[] = {{&S8073[44], 28}, {NULL}};
const Segment SGI116364[] = {{&S8073[30], 6}, {NULL}};
const Segment SGI116365[] = {{&S8073[31], 3}, {NULL}};
const Segment SGI116366[] = {{&S8073[86], 6}, {NULL}};
const Segment SGI116367[] = {{&S8076[44], 6}, {NULL}};
const Segment SGI116368[] = {{&S8076[44], 18}, {NULL}};
const Segment SGI116369[] = {{&S8076[30], 6}, {NULL}};
const Segment SGI116370[] = {{&S8076[31], 3}, {NULL}};
const Segment SGI116371[] = {{&S8076[76], 6}, {NULL}};
const Segment SGI116372[] = {{&S8078[44], 9}, {NULL}};
const Segment SGI116373[] = {{&S8078[44], 21}, {NULL}};
const Segment SGI116374[] = {{&S8078[30], 6}, {NULL}};
const Segment SGI116375[] = {{&S8078[31], 3}, {NULL}};
const Segment SGI116376[] = {{&S8078[79], 6}, {NULL}};
const Segment SGI116377[] = {{&S8080[44], 13}, {NULL}};
const Segment SGI116378[] = {{&S8080[44], 25}, {NULL}};
const Segment SGI116379[] = {{&S8080[30], 6}, {NULL}};
const Segment SGI116380[] = {{&S8080[31], 3}, {NULL}};
const Segment SGI116381[] = {{&S8080[83], 6}, {NULL}};
const Segment SGI116382[] = {{&S8081[44], 9}, {NULL}};
const Segment SGI116383[] = {{&S8081[44], 21}, {NULL}};
const Segment SGI116384[] = {{&S8081[30], 6}, {NULL}};
const Segment SGI116385[] = {{&S8081[31], 3}, {NULL}};
const Segment SGI116386[] = {{&S8081[79], 6}, {NULL}};
const Segment SGI116387[] = {{&S8082[44], 9}, {NULL}};
const Segment SGI116388[] = {{&S8082[44], 30}, {NULL}};
const Segment SGI116389[] = {{&S8082[30], 6}, {NULL}};
const Segment SGI116390[] = {{&S8082[31], 3}, {NULL}};
const Segment SGI116391[] = {{&S8082[88], 6}, {NULL}};
const Segment SGI116392[] = {{&S8083[44], 16}, {NULL}};
const Segment SGI116393[] = {{&S8083[44], 28}, {NULL}};
const Segment SGI116394[] = {{&S8083[30], 6}, {NULL}};
const Segment SGI116395[] = {{&S8083[31], 3}, {NULL}};
const Segment SGI116396[] = {{&S8083[86], 6}, {NULL}};
const Segment SGI116397[] = {{&S8084[44], 7}, {NULL}};
const Segment SGI116398[] = {{&S8084[44], 19}, {NULL}};
const Segment SGI116399[] = {{&S8084[30], 6}, {NULL}};
const Segment SGI116400[] = {{&S8084[31], 3}, {NULL}};
const Segment SGI116401[] = {{&S8084[77], 6}, {NULL}};
const Segment SGI116402[] = {{&S8086[44], 12}, {NULL}};
const Segment SGI116403[] = {{&S8086[44], 24}, {NULL}};
const Segment SGI116404[] = {{&S8086[30], 6}, {NULL}};
const Segment SGI116405[] = {{&S8086[31], 3}, {NULL}};
const Segment SGI116406[] = {{&S8086[82], 6}, {NULL}};
const Segment SGI116407[] = {{&S8087[44], 12}, {NULL}};
const Segment SGI116408[] = {{&S8087[44], 24}, {NULL}};
const Segment SGI116409[] = {{&S8087[30], 6}, {NULL}};
const Segment SGI116410[] = {{&S8087[31], 3}, {NULL}};
const Segment SGI116411[] = {{&S8087[82], 6}, {NULL}};
const Segment SGI116412[] = {{&S91260[44], 10}, {NULL}};
const Segment SGI116413[] = {{&S91260[44], 22}, {NULL}};
const Segment SGI116414[] = {{&S91260[30], 6}, {NULL}};
const Segment SGI116415[] = {{&S91260[31], 3}, {NULL}};
const Segment SGI116416[] = {{&S91260[80], 6}, {NULL}};
const Segment SGI116417[] = {{&S8089[44], 6}, {NULL}};
const Segment SGI116418[] = {{&S8089[44], 18}, {NULL}};
const Segment SGI116419[] = {{&S8089[30], 6}, {NULL}};
const Segment SGI116420[] = {{&S8089[31], 3}, {NULL}};
const Segment SGI116421[] = {{&S8089[76], 6}, {NULL}};
const Segment SGI116422[] = {{&S8090[44], 13}, {NULL}};
const Segment SGI116423[] = {{&S8090[44], 25}, {NULL}};
const Segment SGI116424[] = {{&S8090[30], 6}, {NULL}};
const Segment SGI116425[] = {{&S8090[31], 3}, {NULL}};
const Segment SGI116426[] = {{&S8090[83], 6}, {NULL}};
const Segment SGI116427[] = {{&S8091[44], 13}, {NULL}};
const Segment SGI116428[] = {{&S8091[44], 25}, {NULL}};
const Segment SGI116429[] = {{&S8091[30], 6}, {NULL}};
const Segment SGI116430[] = {{&S8091[31], 3}, {NULL}};
const Segment SGI116431[] = {{&S8091[83], 6}, {NULL}};
const Segment SGI116432[] = {{&S8092[44], 5}, {NULL}};
const Segment SGI116433[] = {{&S8092[44], 17}, {NULL}};
const Segment SGI116434[] = {{&S8092[30], 6}, {NULL}};
const Segment SGI116435[] = {{&S8092[31], 3}, {NULL}};
const Segment SGI116436[] = {{&S8092[75], 6}, {NULL}};
const Segment SGI116437[] = {{&S8094[44], 31}, {NULL}};
const Segment SGI116438[] = {{&S8094[30], 6}, {NULL}};
const Segment SGI116439[] = {{&S8094[31], 3}, {NULL}};
const Segment SGI116440[] = {{&S8094[108], 6}, {NULL}};
const Segment SGI116441[] = {{&S8095[44], 11}, {NULL}};
const Segment SGI116442[] = {{&S8095[44], 23}, {NULL}};
const Segment SGI116443[] = {{&S8095[30], 6}, {NULL}};
const Segment SGI116444[] = {{&S8095[31], 3}, {NULL}};
const Segment SGI116445[] = {{&S8095[81], 6}, {NULL}};
const Segment SGI116446[] = {{&S8098[44], 5}, {NULL}};
const Segment SGI116447[] = {{&S8098[44], 17}, {NULL}};
const Segment SGI116448[] = {{&S8098[30], 6}, {NULL}};
const Segment SGI116449[] = {{&S8098[31], 3}, {NULL}};
const Segment SGI116450[] = {{&S8098[75], 6}, {NULL}};
const Segment SGI116451[] = {{&S8099[44], 6}, {NULL}};
const Segment SGI116452[] = {{&S8099[44], 18}, {NULL}};
const Segment SGI116453[] = {{&S8099[30], 6}, {NULL}};
const Segment SGI116454[] = {{&S8099[31], 3}, {NULL}};
const Segment SGI116455[] = {{&S8099[76], 6}, {NULL}};
const Segment SGI116456[] = {{&S8100[44], 13}, {NULL}};
const Segment SGI116457[] = {{&S8100[44], 25}, {NULL}};
const Segment SGI116458[] = {{&S8100[30], 6}, {NULL}};
const Segment SGI116459[] = {{&S8100[31], 3}, {NULL}};
const Segment SGI116460[] = {{&S8100[83], 6}, {NULL}};
const Segment SGI116461[] = {{&S8101[44], 4}, {NULL}};
const Segment SGI116462[] = {{&S8101[44], 16}, {NULL}};
const Segment SGI116463[] = {{&S8101[30], 6}, {NULL}};
const Segment SGI116464[] = {{&S8101[31], 3}, {NULL}};
const Segment SGI116465[] = {{&S8101[74], 6}, {NULL}};
const Segment SGI116466[] = {{&S8102[44], 31}, {NULL}};
const Segment SGI116467[] = {{&S8102[30], 6}, {NULL}};
const Segment SGI116468[] = {{&S8102[31], 3}, {NULL}};
const Segment SGI116469[] = {{&S8102[108], 6}, {NULL}};
const Segment SGI116470[] = {{&S8104[44], 9}, {NULL}};
const Segment SGI116471[] = {{&S8104[44], 21}, {NULL}};
const Segment SGI116472[] = {{&S8104[30], 6}, {NULL}};
const Segment SGI116473[] = {{&S8104[31], 3}, {NULL}};
const Segment SGI116474[] = {{&S8104[79], 6}, {NULL}};
const Segment SGI116475[] = {{&S8105[44], 11}, {NULL}};
const Segment SGI116476[] = {{&S8105[44], 23}, {NULL}};
const Segment SGI116477[] = {{&S8105[30], 6}, {NULL}};
const Segment SGI116478[] = {{&S8105[31], 3}, {NULL}};
const Segment SGI116479[] = {{&S8105[81], 6}, {NULL}};
const Segment SGI116480[] = {{&S8106[44], 11}, {NULL}};
const Segment SGI116481[] = {{&S8106[44], 23}, {NULL}};
const Segment SGI116482[] = {{&S8106[30], 6}, {NULL}};
const Segment SGI116483[] = {{&S8106[31], 3}, {NULL}};
const Segment SGI116484[] = {{&S8106[81], 6}, {NULL}};
const Segment SGI116485[] = {{&S8107[44], 7}, {NULL}};
const Segment SGI116486[] = {{&S8107[44], 19}, {NULL}};
const Segment SGI116487[] = {{&S8107[30], 6}, {NULL}};
const Segment SGI116488[] = {{&S8107[31], 3}, {NULL}};
const Segment SGI116489[] = {{&S8107[77], 6}, {NULL}};
const Segment SGI116490[] = {{&S8108[44], 14}, {NULL}};
const Segment SGI116491[] = {{&S8108[44], 26}, {NULL}};
const Segment SGI116492[] = {{&S8108[30], 6}, {NULL}};
const Segment SGI116493[] = {{&S8108[31], 3}, {NULL}};
const Segment SGI116494[] = {{&S8108[84], 6}, {NULL}};
const Segment SGI116495[] = {{&S8109[44], 8}, {NULL}};
const Segment SGI116496[] = {{&S8109[44], 20}, {NULL}};
const Segment SGI116497[] = {{&S8109[30], 6}, {NULL}};
const Segment SGI116498[] = {{&S8109[31], 3}, {NULL}};
const Segment SGI116499[] = {{&S8109[78], 6}, {NULL}};
const Segment SGI116500[] = {{&S8110[44], 6}, {NULL}};
const Segment SGI116501[] = {{&S91313[44], 7}, {NULL}};
const Segment SGI116502[] = {{&S8110[44], 18}, {NULL}};
const Segment SGI116503[] = {{&S91313[44], 19}, {NULL}};
const Segment SGI116504[] = {{&S91313[30], 6}, {NULL}};
const Segment SGI116505[] = {{&S8110[30], 6}, {NULL}};
const Segment SGI116506[] = {{&S8110[31], 3}, {NULL}};
const Segment SGI116507[] = {{&S91313[31], 3}, {NULL}};
const Segment SGI116508[] = {{&S91313[77], 6}, {NULL}};
const Segment SGI116509[] = {{&S8110[76], 6}, {NULL}};
const Segment SGI116510[] = {{&S8111[38], 6}, {NULL}};
const Segment SGI116511[] = {{&S8111[31], 6}, {NULL}};
const Segment SGI116512[] = {{&S8112[44], 8}, {NULL}};
const Segment SGI116513[] = {{&S8112[44], 20}, {NULL}};
const Segment SGI116514[] = {{&S8112[30], 6}, {NULL}};
const Segment SGI116515[] = {{&S8112[31], 3}, {NULL}};
const Segment SGI116516[] = {{&S8112[78], 6}, {NULL}};
const Segment SGI116517[] = {{&S8113[44], 14}, {NULL}};
const Segment SGI116518[] = {{&S8113[44], 26}, {NULL}};
const Segment SGI116519[] = {{&S8113[30], 6}, {NULL}};
const Segment SGI116520[] = {{&S8113[31], 3}, {NULL}};
const Segment SGI116521[] = {{&S8113[84], 6}, {NULL}};
const Segment SGI116522[] = {{&S8114[44], 8}, {NULL}};
const Segment SGI116523[] = {{&S8114[44], 20}, {NULL}};
const Segment SGI116524[] = {{&S8114[30], 6}, {NULL}};
const Segment SGI116525[] = {{&S8114[31], 3}, {NULL}};
const Segment SGI116526[] = {{&S8114[78], 6}, {NULL}};
const Segment SGI116527[] = {{&S91322[34], 45}, {NULL}};
const Segment SGI116528[] = {{&S91322[27], 6}, {NULL}};
const Segment SGI116529[] = {{&S91322[132], 9}, {NULL}};
const Segment SGI116530[] = {{&S91322[105], 6}, {NULL}};
const Segment SGI116531[] = {{&S8115[44], 6}, {NULL}};
const Segment SGI116532[] = {{&S8115[44], 18}, {NULL}};
const Segment SGI116533[] = {{&S8115[30], 6}, {NULL}};
const Segment SGI116534[] = {{&S8115[31], 3}, {NULL}};
const Segment SGI116535[] = {{&S8115[76], 6}, {NULL}};
const Segment SGI116536[] = {{&S8116[44], 7}, {NULL}};
const Segment SGI116537[] = {{&S8116[44], 19}, {NULL}};
const Segment SGI116538[] = {{&S8116[30], 6}, {NULL}};
const Segment SGI116539[] = {{&S8116[31], 3}, {NULL}};
const Segment SGI116540[] = {{&S8116[77], 6}, {NULL}};
const Segment SGI116541[] = {{&S8118[38], 10}, {NULL}};
const Segment SGI116542[] = {{&S8118[31], 6}, {NULL}};
const Segment SGI116543[] = {{&S8119[44], 14}, {NULL}};
const Segment SGI116544[] = {{&S8119[44], 26}, {NULL}};
const Segment SGI116545[] = {{&S8119[30], 6}, {NULL}};
const Segment SGI116546[] = {{&S8119[31], 3}, {NULL}};
const Segment SGI116547[] = {{&S8119[84], 6}, {NULL}};
const Segment SGI116548[] = {{&S8120[44], 4}, {NULL}};
const Segment SGI116549[] = {{&S8120[44], 16}, {NULL}};
const Segment SGI116550[] = {{&S8120[30], 6}, {NULL}};
const Segment SGI116551[] = {{&S8120[31], 3}, {NULL}};
const Segment SGI116552[] = {{&S8120[74], 6}, {NULL}};
const Segment SGI116553[] = {{&S8121[44], 18}, {NULL}};
const Segment SGI116554[] = {{&S8121[44], 30}, {NULL}};
const Segment SGI116555[] = {{&S8121[30], 6}, {NULL}};
const Segment SGI116556[] = {{&S8121[31], 3}, {NULL}};
const Segment SGI116557[] = {{&S8121[88], 6}, {NULL}};
const Segment SGI116558[] = {{&S8122[44], 14}, {NULL}};
const Segment SGI116559[] = {{&S8122[44], 26}, {NULL}};
const Segment SGI116560[] = {{&S8122[30], 6}, {NULL}};
const Segment SGI116561[] = {{&S8122[31], 3}, {NULL}};
const Segment SGI116562[] = {{&S8122[84], 6}, {NULL}};
const Segment SGI116563[] = {{&S8123[44], 8}, {NULL}};
const Segment SGI116564[] = {{&S8123[44], 20}, {NULL}};
const Segment SGI116565[] = {{&S8123[30], 6}, {NULL}};
const Segment SGI116566[] = {{&S91361[34], 11}, {NULL}};
const Segment SGI116567[] = {{&S91361[27], 6}, {NULL}};
const Segment SGI116568[] = {{&S91361[98], 9}, {NULL}};
const Segment SGI116569[] = {{&S91361[71], 6}, {NULL}};
const Segment SGI116570[] = {{&S91403[34], 7}, {NULL}};
const Segment SGI116571[] = {{&S91403[27], 6}, {NULL}};
const Segment SGI116572[] = {{&S91403[94], 9}, {NULL}};
const Segment SGI116573[] = {{&S91403[67], 6}, {NULL}};
const Segment SGI116574[] = {{&S91404[34], 6}, {NULL}};
const Segment SGI116575[] = {{&S91404[27], 6}, {NULL}};
const Segment SGI116576[] = {{&S91404[93], 9}, {NULL}};
const Segment SGI116577[] = {{&S91404[66], 6}, {NULL}};
const Segment SGI116578[] = {{&S8123[31], 3}, {NULL}};
const Segment SGI116579[] = {{&S8123[78], 6}, {NULL}};
const Segment SGI116580[] = {{&S8124[44], 7}, {NULL}};
const Segment SGI116581[] = {{&S8124[44], 19}, {NULL}};
const Segment SGI116582[] = {{&S8124[30], 6}, {NULL}};
const Segment SGI116583[] = {{&S8124[31], 3}, {NULL}};
const Segment SGI116584[] = {{&S8124[77], 6}, {NULL}};
const Segment SGI116585[] = {{&S8125[44], 10}, {NULL}};
const Segment SGI116586[] = {{&S8125[44], 22}, {NULL}};
const Segment SGI116587[] = {{&S8125[30], 6}, {NULL}};
const Segment SGI116588[] = {{&S8125[31], 3}, {NULL}};
const Segment SGI116589[] = {{&S8125[80], 6}, {NULL}};
const Segment SGI116590[] = {{&S8126[44], 9}, {NULL}};
const Segment SGI116591[] = {{&S8126[44], 21}, {NULL}};
const Segment SGI116592[] = {{&S8126[30], 6}, {NULL}};
const Segment SGI116593[] = {{&S8126[31], 3}, {NULL}};
const Segment SGI116594[] = {{&S8126[79], 6}, {NULL}};
const Segment SGI116595[] = {{&S91444[34], 41}, {NULL}};
const Segment SGI116596[] = {{&S91444[27], 6}, {NULL}};
const Segment SGI116597[] = {{&S91444[128], 9}, {NULL}};
const Segment SGI116598[] = {{&S8127[44], 18}, {NULL}};
const Segment SGI116599[] = {{&S91444[101], 6}, {NULL}};
const Segment SGI116600[] = {{&S8127[44], 30}, {NULL}};
const Segment SGI116601[] = {{&S8127[30], 6}, {NULL}};
const Segment SGI116602[] = {{&S91444[169], 8}, {NULL}};
const Segment SGI116603[] = {{&S8127[31], 3}, {NULL}};
const Segment SGI116604[] = {{&S8127[88], 6}, {NULL}};
const Segment SGI116605[] = {{&S8128[44], 20}, {NULL}};
const Segment SGI116606[] = {{&S8128[44], 32}, {NULL}};
const Segment SGI116607[] = {{&S8128[30], 6}, {NULL}};
const Segment SGI116608[] = {{&S8128[31], 3}, {NULL}};
const Segment SGI116609[] = {{&S8128[90], 6}, {NULL}};
const Segment SGI116610[] = {{&S8129[44], 8}, {NULL}};
const Segment SGI116611[] = {{&S8129[44], 20}, {NULL}};
const Segment SGI116612[] = {{&S8129[30], 6}, {NULL}};
const Segment SGI116613[] = {{&S8129[31], 3}, {NULL}};
const Segment SGI116614[] = {{&S8129[78], 6}, {NULL}};
const Segment SGI116615[] = {{&S8130[44], 5}, {NULL}};
const Segment SGI116616[] = {{&S8130[44], 17}, {NULL}};
const Segment SGI116617[] = {{&S8130[30], 6}, {NULL}};
const Segment SGI116618[] = {{&S8130[31], 3}, {NULL}};
const Segment SGI116619[] = {{&S8130[75], 6}, {NULL}};
const Segment SGI116620[] = {{&S8131[44], 8}, {NULL}};
const Segment SGI116621[] = {{&S8131[44], 20}, {NULL}};
const Segment SGI116622[] = {{&S8131[30], 6}, {NULL}};
const Segment SGI116623[] = {{&S8131[31], 3}, {NULL}};
const Segment SGI116624[] = {{&S8131[78], 6}, {NULL}};
const Segment SGI116625[] = {{&S8132[44], 13}, {NULL}};
const Segment SGI116626[] = {{&S8132[44], 32}, {NULL}};
const Segment SGI116627[] = {{&S8132[30], 6}, {NULL}};
const Segment SGI116628[] = {{&S8132[31], 3}, {NULL}};
const Segment SGI116629[] = {{&S8132[90], 6}, {NULL}};
const Segment SGI116630[] = {{&S8133[44], 12}, {NULL}};
const Segment SGI116631[] = {{&S8133[44], 24}, {NULL}};
const Segment SGI116632[] = {{&S8133[30], 6}, {NULL}};
const Segment SGI116633[] = {{&S8133[31], 3}, {NULL}};
const Segment SGI116634[] = {{&S8133[82], 6}, {NULL}};
const Segment SGI116635[] = {{&S8134[44], 12}, {NULL}};
const Segment SGI116636[] = {{&S8134[44], 24}, {NULL}};
const Segment SGI116637[] = {{&S8134[30], 6}, {NULL}};
const Segment SGI116638[] = {{&S8134[31], 3}, {NULL}};
const Segment SGI116639[] = {{&S8134[82], 6}, {NULL}};
const Segment SGI116640[] = {{&S8135[44], 6}, {NULL}};
const Segment SGI116641[] = {{&S8135[44], 18}, {NULL}};
const Segment SGI116642[] = {{&S8135[30], 6}, {NULL}};
const Segment SGI116643[] = {{&S8135[31], 3}, {NULL}};
const Segment SGI116644[] = {{&S8135[76], 6}, {NULL}};
const Segment SGI116645[] = {{&S8137[44], 11}, {NULL}};
const Segment SGI116646[] = {{&S8137[44], 23}, {NULL}};
const Segment SGI116647[] = {{&S8137[30], 6}, {NULL}};
const Segment SGI116648[] = {{&S8137[31], 3}, {NULL}};
const Segment SGI116649[] = {{&S8137[81], 6}, {NULL}};
const Segment SGI116650[] = {{&S8138[44], 9}, {NULL}};
const Segment SGI116651[] = {{&S8138[44], 21}, {NULL}};
const Segment SGI116652[] = {{&S8138[30], 6}, {NULL}};
const Segment SGI116653[] = {{&S8138[31], 3}, {NULL}};
const Segment SGI116654[] = {{&S8138[79], 6}, {NULL}};
const Segment SGI116655[] = {{&S8139[44], 14}, {NULL}};
const Segment SGI116656[] = {{&S8139[44], 26}, {NULL}};
const Segment SGI116657[] = {{&S8139[30], 6}, {NULL}};
const Segment SGI116658[] = {{&S8139[31], 3}, {NULL}};
const Segment SGI116659[] = {{&S8139[84], 6}, {NULL}};
const Segment SGI116660[] = {{&S8140[44], 15}, {NULL}};
const Segment SGI116661[] = {{&S8140[44], 27}, {NULL}};
const Segment SGI116662[] = {{&S8140[30], 6}, {NULL}};
const Segment SGI116663[] = {{&S8140[31], 3}, {NULL}};
const Segment SGI116664[] = {{&S8140[85], 6}, {NULL}};
const Segment SGI116665[] = {{&S8141[44], 22}, {NULL}};
const Segment SGI116666[] = {{&S8141[44], 34}, {NULL}};
const Segment SGI116667[] = {{&S8141[30], 6}, {NULL}};
const Segment SGI116668[] = {{&S8141[31], 3}, {NULL}};
const Segment SGI116669[] = {{&S8141[92], 6}, {NULL}};
const Segment SGI116670[] = {{&S8142[44], 8}, {NULL}};
const Segment SGI116671[] = {{&S8142[44], 20}, {NULL}};
const Segment SGI116672[] = {{&S8142[30], 6}, {NULL}};
const Segment SGI116673[] = {{&S8142[31], 3}, {NULL}};
const Segment SGI116674[] = {{&S8142[78], 6}, {NULL}};
const Segment SGI116675[] = {{&S8143[44], 7}, {NULL}};
const Segment SGI116676[] = {{&S8143[44], 19}, {NULL}};
const Segment SGI116677[] = {{&S8143[30], 6}, {NULL}};
const Segment SGI116678[] = {{&S8143[31], 3}, {NULL}};
const Segment SGI116679[] = {{&S8143[77], 6}, {NULL}};
const Segment SGI116680[] = {{&S8144[44], 7}, {NULL}};
const Segment SGI116681[] = {{&S8144[44], 19}, {NULL}};
const Segment SGI116682[] = {{&S8144[30], 6}, {NULL}};
const Segment SGI116683[] = {{&S8144[31], 3}, {NULL}};
const Segment SGI116684[] = {{&S8144[77], 6}, {NULL}};
const Segment SGI116685[] = {{&S8145[44], 8}, {NULL}};
const Segment SGI116686[] = {{&S8145[44], 20}, {NULL}};
const Segment SGI116687[] = {{&S8145[30], 6}, {NULL}};
const Segment SGI116688[] = {{&S8145[31], 3}, {NULL}};
const Segment SGI116689[] = {{&S8145[78], 6}, {NULL}};
const Segment SGI116690[] = {{&S8146[44], 9}, {NULL}};
const Segment SGI116691[] = {{&S8146[44], 21}, {NULL}};
const Segment SGI116692[] = {{&S8146[30], 6}, {NULL}};
const Segment SGI116693[] = {{&S8146[31], 3}, {NULL}};
const Segment SGI116694[] = {{&S8146[79], 6}, {NULL}};
const Segment SGI116695[] = {{&S8147[44], 33}, {NULL}};
const Segment SGI116696[] = {{&S8147[44], 45}, {NULL}};
const Segment SGI116697[] = {{&S8147[30], 6}, {NULL}};
const Segment SGI116698[] = {{&S8147[31], 3}, {NULL}};
const Segment SGI116699[] = {{&S8147[103], 6}, {NULL}};
const Segment SGI116700[] = {{&S8150[44], 9}, {NULL}};
const Segment SGI116701[] = {{&S8150[44], 21}, {NULL}};
const Segment SGI116702[] = {{&S8150[30], 6}, {NULL}};
const Segment SGI116703[] = {{&S8150[31], 3}, {NULL}};
const Segment SGI116704[] = {{&S8150[79], 6}, {NULL}};
const Segment SGI116705[] = {{&S8151[44], 8}, {NULL}};
const Segment SGI116706[] = {{&S8151[44], 20}, {NULL}};
const Segment SGI116707[] = {{&S8151[30], 6}, {NULL}};
const Segment SGI116708[] = {{&S8151[31], 3}, {NULL}};
const Segment SGI116709[] = {{&S8151[78], 6}, {NULL}};
const Segment SGI116710[] = {{&S8152[44], 5}, {NULL}};
const Segment SGI116711[] = {{&S8152[44], 26}, {NULL}};
const Segment SGI116712[] = {{&S8152[30], 6}, {NULL}};
const Segment SGI116713[] = {{&S8152[31], 3}, {NULL}};
const Segment SGI116714[] = {{&S8152[84], 5}, {NULL}};
const Segment SGI116715[] = {{&S8153[44], 10}, {NULL}};
const Segment SGI116716[] = {{&S8153[44], 31}, {NULL}};
const Segment SGI116717[] = {{&S8153[30], 6}, {NULL}};
const Segment SGI116718[] = {{&S8153[31], 3}, {NULL}};
const Segment SGI116719[] = {{&S8153[89], 5}, {NULL}};
const Segment SGI116720[] = {{&S8154[44], 4}, {NULL}};
const Segment SGI116721[] = {{&S8154[44], 22}, {NULL}};
const Segment SGI116722[] = {{&S8154[30], 6}, {NULL}};
const Segment SGI116723[] = {{&S8154[31], 3}, {NULL}};
const Segment SGI116724[] = {{&S8154[80], 5}, {NULL}};
const Segment SGI116725[] = {{&S8155[44], 10}, {NULL}};
const Segment SGI116726[] = {{&S8155[44], 22}, {NULL}};
const Segment SGI116727[] = {{&S8155[30], 6}, {NULL}};
const Segment SGI116728[] = {{&S8155[31], 3}, {NULL}};
const Segment SGI116729[] = {{&S8155[80], 6}, {NULL}};
const Segment SGI116730[] = {{&S8156[44], 14}, {NULL}};
const Segment SGI116731[] = {{&S8156[44], 26}, {NULL}};
const Segment SGI116732[] = {{&S8156[30], 6}, {NULL}};
const Segment SGI116733[] = {{&S8156[31], 3}, {NULL}};
const Segment SGI116734[] = {{&S8156[84], 6}, {NULL}};
const Segment SGI116735[] = {{&S8157[44], 6}, {NULL}};
const Segment SGI116736[] = {{&S8157[44], 18}, {NULL}};
const Segment SGI116737[] = {{&S8157[30], 6}, {NULL}};
const Segment SGI116738[] = {{&S8157[31], 3}, {NULL}};
const Segment SGI116739[] = {{&S8157[76], 6}, {NULL}};
const Segment SGI116740[] = {{&S8158[44], 5}, {NULL}};
const Segment SGI116741[] = {{&S8158[44], 17}, {NULL}};
const Segment SGI116742[] = {{&S8158[30], 6}, {NULL}};
const Segment SGI116743[] = {{&S8158[31], 3}, {NULL}};
const Segment SGI116744[] = {{&S8158[75], 6}, {NULL}};
const Segment SGI116745[] = {{&S8161[44], 7}, {NULL}};
const Segment SGI116746[] = {{&S8161[44], 19}, {NULL}};
const Segment SGI116747[] = {{&S8161[30], 6}, {NULL}};
const Segment SGI116748[] = {{&S8161[31], 3}, {NULL}};
const Segment SGI116749[] = {{&S8161[77], 6}, {NULL}};
const Segment SGI116750[] = {{&S8162[44], 8}, {NULL}};
const Segment SGI116751[] = {{&S8162[44], 20}, {NULL}};
const Segment SGI116752[] = {{&S8162[30], 6}, {NULL}};
const Segment SGI116753[] = {{&S8162[31], 3}, {NULL}};
const Segment SGI116754[] = {{&S8162[78], 6}, {NULL}};
const Segment SGI116755[] = {{&S8163[44], 15}, {NULL}};
const Segment SGI116756[] = {{&S8163[44], 27}, {NULL}};
const Segment SGI116757[] = {{&S8163[30], 6}, {NULL}};
const Segment SGI116758[] = {{&S8163[31], 3}, {NULL}};
const Segment SGI116759[] = {{&S8163[85], 6}, {NULL}};
const Segment SGI116760[] = {{&S8164[44], 17}, {NULL}};
const Segment SGI116761[] = {{&S8164[44], 29}, {NULL}};
const Segment SGI116762[] = {{&S8164[30], 6}, {NULL}};
const Segment SGI116763[] = {{&S8164[31], 3}, {NULL}};
const Segment SGI116764[] = {{&S8164[87], 6}, {NULL}};
const Segment SGI116765[] = {{&S8165[44], 37}, {NULL}};
const Segment SGI116766[] = {{&S8165[30], 6}, {NULL}};
const Segment SGI116767[] = {{&S8165[31], 3}, {NULL}};
const Segment SGI116768[] = {{&S8165[113], 6}, {NULL}};
const Segment SGI116769[] = {{&S8166[44], 11}, {NULL}};
const Segment SGI116770[] = {{&S8166[44], 23}, {NULL}};
const Segment SGI116771[] = {{&S8166[30], 6}, {NULL}};
const Segment SGI116772[] = {{&S8166[31], 3}, {NULL}};
const Segment SGI116773[] = {{&S8166[81], 6}, {NULL}};
const Segment SGI116774[] = {{&S8167[44], 8}, {NULL}};
const Segment SGI116775[] = {{&S8167[44], 20}, {NULL}};
const Segment SGI116776[] = {{&S8167[30], 6}, {NULL}};
const Segment SGI116777[] = {{&S8167[31], 3}, {NULL}};
const Segment SGI116778[] = {{&S8167[78], 6}, {NULL}};
const Segment SGI116779[] = {{&S8168[44], 9}, {NULL}};
const Segment SGI116780[] = {{&S8168[44], 21}, {NULL}};
const Segment SGI116781[] = {{&S8168[30], 6}, {NULL}};
const Segment SGI116782[] = {{&S8168[31], 3}, {NULL}};
const Segment SGI116783[] = {{&S8168[79], 6}, {NULL}};
const Segment SGI116784[] = {{&S8169[44], 12}, {NULL}};
const Segment SGI116785[] = {{&S8169[44], 24}, {NULL}};
const Segment SGI116786[] = {{&S8169[30], 6}, {NULL}};
const Segment SGI116787[] = {{&S8169[31], 3}, {NULL}};
const Segment SGI116788[] = {{&S8169[82], 6}, {NULL}};
const Segment SGI116789[] = {{&S8173[44], 7}, {NULL}};
const Segment SGI116790[] = {{&S8173[44], 19}, {NULL}};
const Segment SGI116791[] = {{&S8173[30], 6}, {NULL}};
const Segment SGI116792[] = {{&S8173[31], 3}, {NULL}};
const Segment SGI116793[] = {{&S8173[77], 6}, {NULL}};
const Segment SGI116794[] = {{&S8174[44], 6}, {NULL}};
const Segment SGI116795[] = {{&S8174[44], 18}, {NULL}};
const Segment SGI116796[] = {{&S8174[30], 6}, {NULL}};
const Segment SGI116797[] = {{&S8174[31], 3}, {NULL}};
const Segment SGI116798[] = {{&S8174[76], 6}, {NULL}};
const Segment SGI116799[] = {{&S8175[44], 5}, {NULL}};
const Segment SGI116800[] = {{&S8175[44], 30}, {NULL}};
const Segment SGI116801[] = {{&S8175[30], 6}, {NULL}};
const Segment SGI116802[] = {{&S8175[31], 3}, {NULL}};
const Segment SGI116803[] = {{&S8175[88], 6}, {NULL}};
const Segment SGI116804[] = {{&S91549[45], 9}, {NULL}};
const Segment SGI116805[] = {{&S91549[30], 6}, {NULL}};
const Segment SGI116806[] = {{&S91549[31], 3}, {NULL}};
const Segment SGI116807[] = {{&S91549[80], 5}, {NULL}};
const Segment SGI116808[] = {{&S91549[138], 13}, {NULL}};
const Segment SGI116809[] = {{&S8177[44], 6}, {NULL}};
const Segment SGI116810[] = {{&S8177[44], 18}, {NULL}};
const Segment SGI116811[] = {{&S8177[30], 6}, {NULL}};
const Segment SGI116812[] = {{&S8177[31], 3}, {NULL}};
const Segment SGI116813[] = {{&S8177[76], 5}, {NULL}};
const Segment SGI116814[] = {{&S8178[44], 13}, {NULL}};
const Segment SGI116815[] = {{&S8178[44], 25}, {NULL}};
const Segment SGI116816[] = {{&S8178[30], 6}, {NULL}};
const Segment SGI116817[] = {{&S8178[31], 3}, {NULL}};
const Segment SGI116818[] = {{&S8178[83], 6}, {NULL}};
const Segment SGI116819[] = {{&S8179[44], 18}, {NULL}};
const Segment SGI116820[] = {{&S8179[44], 30}, {NULL}};
const Segment SGI116821[] = {{&S8179[30], 6}, {NULL}};
const Segment SGI116822[] = {{&S8179[31], 3}, {NULL}};
const Segment SGI116823[] = {{&S8179[88], 6}, {NULL}};
const Segment SGI116824[] = {{&S8180[44], 9}, {NULL}};
const Segment SGI116825[] = {{&S8180[44], 34}, {NULL}};
const Segment SGI116826[] = {{&S8180[30], 6}, {NULL}};
const Segment SGI116827[] = {{&S8180[31], 3}, {NULL}};
const Segment SGI116828[] = {{&S8180[92], 6}, {NULL}};
const Segment SGI116829[] = {{&S8181[44], 16}, {NULL}};
const Segment SGI116830[] = {{&S8181[44], 28}, {NULL}};
const Segment SGI116831[] = {{&S8181[30], 6}, {NULL}};
const Segment SGI116832[] = {{&S8181[31], 3}, {NULL}};
const Segment SGI116833[] = {{&S8181[86], 6}, {NULL}};
const Segment SGI116834[] = {{&S8182[44], 6}, {NULL}};
const Segment SGI116835[] = {{&S8182[44], 18}, {NULL}};
const Segment SGI116836[] = {{&S8182[30], 6}, {NULL}};
const Segment SGI116837[] = {{&S8182[31], 3}, {NULL}};
const Segment SGI116838[] = {{&S8182[76], 6}, {NULL}};
const Segment SGI116839[] = {{&S8183[44], 9}, {NULL}};
const Segment SGI116840[] = {{&S8183[44], 21}, {NULL}};
const Segment SGI116841[] = {{&S8183[30], 6}, {NULL}};
const Segment SGI116842[] = {{&S8183[31], 3}, {NULL}};
const Segment SGI116843[] = {{&S8183[79], 6}, {NULL}};
const Segment SGI116844[] = {{&S8184[44], 8}, {NULL}};
const Segment SGI116845[] = {{&S8184[44], 20}, {NULL}};
const Segment SGI116846[] = {{&S8184[30], 6}, {NULL}};
const Segment SGI116847[] = {{&S8184[31], 3}, {NULL}};
const Segment SGI116848[] = {{&S8184[78], 6}, {NULL}};
const Segment SGI116849[] = {{&S8185[44], 16}, {NULL}};
const Segment SGI116850[] = {{&S8185[44], 28}, {NULL}};
const Segment SGI116851[] = {{&S8185[30], 6}, {NULL}};
const Segment SGI116852[] = {{&S91568[12], 1}, {NULL}};
const Segment SGI116853[] = {{&S91568[57], 23}, {NULL}};
const Segment SGI116854[] = {{&S8185[31], 3}, {NULL}};
const Segment SGI116855[] = {{&S91568[43], 6}, {NULL}};
const Segment SGI116856[] = {{&S8185[86], 6}, {NULL}};
const Segment SGI116857[] = {{&S91568[44], 3}, {NULL}};
const Segment SGI116858[] = {{&S91568[94], 6}, {NULL}};
const Segment SGI116859[] = {{&S91568[0], 12}, {NULL}};
const Segment SGI116860[] = {{&S8186[44], 7}, {NULL}};
const Segment SGI116861[] = {{&S8186[44], 19}, {NULL}};
const Segment SGI116862[] = {{&S8186[30], 6}, {NULL}};
const Segment SGI116863[] = {{&S8186[31], 3}, {NULL}};
const Segment SGI116864[] = {{&S8186[77], 6}, {NULL}};
const Segment SGI116865[] = {{&S8187[44], 10}, {NULL}};
const Segment SGI116866[] = {{&S8187[44], 22}, {NULL}};
const Segment SGI116867[] = {{&S8187[30], 6}, {NULL}};
const Segment SGI116868[] = {{&S8187[31], 3}, {NULL}};
const Segment SGI116869[] = {{&S8187[80], 6}, {NULL}};
const Segment SGI116870[] = {{&S8188[44], 7}, {NULL}};
const Segment SGI116871[] = {{&S8188[44], 19}, {NULL}};
const Segment SGI116872[] = {{&S8188[30], 6}, {NULL}};
const Segment SGI116873[] = {{&S8188[31], 3}, {NULL}};
const Segment SGI116874[] = {{&S8188[77], 6}, {NULL}};
const Segment SGI116875[] = {{&S8189[44], 11}, {NULL}};
const Segment SGI116876[] = {{&S8189[44], 23}, {NULL}};
const Segment SGI116877[] = {{&S8189[30], 6}, {NULL}};
const Segment SGI116878[] = {{&S8189[31], 3}, {NULL}};
const Segment SGI116879[] = {{&S8189[81], 6}, {NULL}};
const Segment SGI116880[] = {{&S8191[44], 9}, {NULL}};
const Segment SGI116881[] = {{&S8191[44], 21}, {NULL}};
const Segment SGI116882[] = {{&S8191[30], 6}, {NULL}};
const Segment SGI116883[] = {{&S8191[31], 3}, {NULL}};
const Segment SGI116884[] = {{&S8191[79], 6}, {NULL}};
const Segment SGI116885[] = {{&S8192[44], 6}, {NULL}};
const Segment SGI116886[] = {{&S8192[44], 18}, {NULL}};
const Segment SGI116887[] = {{&S8192[30], 6}, {NULL}};
const Segment SGI116888[] = {{&S8192[31], 3}, {NULL}};
const Segment SGI116889[] = {{&S8192[76], 6}, {NULL}};
const Segment SGI116890[] = {{&S8194[44], 8}, {NULL}};
const Segment SGI116891[] = {{&S8194[44], 20}, {NULL}};
const Segment SGI116892[] = {{&S8194[30], 6}, {NULL}};
const Segment SGI116893[] = {{&S8194[31], 3}, {NULL}};
const Segment SGI116894[] = {{&S8194[78], 6}, {NULL}};
const Segment SGI116895[] = {{&S8195[44], 12}, {NULL}};
const Segment SGI116896[] = {{&S8195[44], 24}, {NULL}};
const Segment SGI116897[] = {{&S8195[30], 6}, {NULL}};
const Segment SGI116898[] = {{&S8195[31], 3}, {NULL}};
const Segment SGI116899[] = {{&S8195[82], 6}, {NULL}};
const Segment SGI116900[] = {{&S8196[44], 10}, {NULL}};
const Segment SGI116901[] = {{&S8196[44], 21}, {NULL}};
const Segment SGI116902[] = {{&S8196[30], 6}, {NULL}};
const Segment SGI116903[] = {{&S8196[31], 3}, {NULL}};
const Segment SGI116904[] = {{&S8196[79], 6}, {NULL}};
const Segment SGI116905[] = {{&S8197[44], 6}, {NULL}};
const Segment SGI116906[] = {{&S8197[44], 18}, {NULL}};
const Segment SGI116907[] = {{&S8197[30], 6}, {NULL}};
const Segment SGI116908[] = {{&S8197[31], 3}, {NULL}};
const Segment SGI116909[] = {{&S8197[76], 6}, {NULL}};
const Segment SGI116910[] = {{&S91594[44], 11}, {NULL}};
const Segment SGI116911[] = {{&S91594[44], 23}, {NULL}};
const Segment SGI116912[] = {{&S8198[44], 15}, {NULL}};
const Segment SGI116913[] = {{&S91594[30], 6}, {NULL}};
const Segment SGI116914[] = {{&S8198[44], 27}, {NULL}};
const Segment SGI116915[] = {{&S8198[30], 6}, {NULL}};
const Segment SGI116916[] = {{&S8198[31], 3}, {NULL}};
const Segment SGI116917[] = {{&S8198[85], 6}, {NULL}};
const Segment SGI116918[] = {{&S91594[81], 6}, {NULL}};
const Segment SGI116919[] = {{&S91594[108], 13}, {NULL}};
const Segment SGI116920[] = {{&S8200[44], 8}, {NULL}};
const Segment SGI116921[] = {{&S8200[44], 20}, {NULL}};
const Segment SGI116922[] = {{&S8200[30], 6}, {NULL}};
const Segment SGI116923[] = {{&S8200[31], 3}, {NULL}};
const Segment SGI116924[] = {{&S8200[78], 6}, {NULL}};
const Segment SGI116925[] = {{&S8203[44], 7}, {NULL}};
const Segment SGI116926[] = {{&S8203[44], 19}, {NULL}};
const Segment SGI116927[] = {{&S8203[30], 6}, {NULL}};
const Segment SGI116928[] = {{&S8203[31], 3}, {NULL}};
const Segment SGI116929[] = {{&S8203[77], 6}, {NULL}};
const Segment SGI116930[] = {{&S8204[44], 5}, {NULL}};
const Segment SGI116931[] = {{&S8204[44], 17}, {NULL}};
const Segment SGI116932[] = {{&S8204[30], 6}, {NULL}};
const Segment SGI116933[] = {{&S8204[31], 3}, {NULL}};
const Segment SGI116934[] = {{&S8204[75], 6}, {NULL}};
const Segment SGI116935[] = {{&S8208[44], 7}, {NULL}};
const Segment SGI116936[] = {{&S8208[44], 19}, {NULL}};
const Segment SGI116937[] = {{&S8208[30], 6}, {NULL}};
const Segment SGI116938[] = {{&S8208[31], 3}, {NULL}};
const Segment SGI116939[] = {{&S8208[77], 6}, {NULL}};
const Segment SGI116940[] = {{&S8210[44], 9}, {NULL}};
const Segment SGI116941[] = {{&S8210[44], 21}, {NULL}};
const Segment SGI116942[] = {{&S8210[30], 6}, {NULL}};
const Segment SGI116943[] = {{&S8210[31], 3}, {NULL}};
const Segment SGI116944[] = {{&S8210[79], 6}, {NULL}};
const Segment SGI116945[] = {{&S8211[44], 7}, {NULL}};
const Segment SGI116946[] = {{&S8211[44], 19}, {NULL}};
const Segment SGI116947[] = {{&S8211[30], 6}, {NULL}};
const Segment SGI116948[] = {{&S8211[31], 3}, {NULL}};
const Segment SGI116949[] = {{&S8211[77], 6}, {NULL}};
const Segment SGI116950[] = {{&S8212[44], 24}, {NULL}};
const Segment SGI116951[] = {{&S8212[44], 36}, {NULL}};
const Segment SGI116952[] = {{&S8212[30], 6}, {NULL}};
const Segment SGI116953[] = {{&S8212[31], 3}, {NULL}};
const Segment SGI116954[] = {{&S8212[94], 6}, {NULL}};
const Segment SGI116955[] = {{&S8213[44], 10}, {NULL}};
const Segment SGI116956[] = {{&S8213[44], 23}, {NULL}};
const Segment SGI116957[] = {{&S8213[30], 6}, {NULL}};
const Segment SGI116958[] = {{&S8213[31], 3}, {NULL}};
const Segment SGI116959[] = {{&S8213[81], 6}, {NULL}};
const Segment SGI116960[] = {{&S8214[44], 9}, {NULL}};
const Segment SGI116961[] = {{&S8214[44], 30}, {NULL}};
const Segment SGI116962[] = {{&S8214[30], 6}, {NULL}};
const Segment SGI116963[] = {{&S8214[31], 3}, {NULL}};
const Segment SGI116964[] = {{&S8214[88], 5}, {NULL}};
const Segment SGI116965[] = {{&S8215[44], 8}, {NULL}};
const Segment SGI116966[] = {{&S8215[44], 26}, {NULL}};
const Segment SGI116967[] = {{&S8215[30], 6}, {NULL}};
const Segment SGI116968[] = {{&S8215[31], 3}, {NULL}};
const Segment SGI116969[] = {{&S8215[84], 6}, {NULL}};
const Segment SGI116970[] = {{&S8216[44], 9}, {NULL}};
const Segment SGI116971[] = {{&S8216[44], 21}, {NULL}};
const Segment SGI116972[] = {{&S8216[30], 6}, {NULL}};
const Segment SGI116973[] = {{&S8216[31], 3}, {NULL}};
const Segment SGI116974[] = {{&S8216[79], 6}, {NULL}};
const Segment SGI116975[] = {{&S8217[44], 7}, {NULL}};
const Segment SGI116976[] = {{&S8217[44], 19}, {NULL}};
const Segment SGI116977[] = {{&S8217[30], 6}, {NULL}};
const Segment SGI116978[] = {{&S8217[31], 3}, {NULL}};
const Segment SGI116979[] = {{&S8217[77], 6}, {NULL}};
const Segment SGI116980[] = {{&S8218[44], 6}, {NULL}};
const Segment SGI116981[] = {{&S8218[44], 18}, {NULL}};
const Segment SGI116982[] = {{&S8218[30], 6}, {NULL}};
const Segment SGI116983[] = {{&S91627[44], 6}, {NULL}};
const Segment SGI116984[] = {{&S91627[44], 18}, {NULL}};
const Segment SGI116985[] = {{&S91627[30], 6}, {NULL}};
const Segment SGI116986[] = {{&S8218[31], 3}, {NULL}};
const Segment SGI116987[] = {{&S8218[76], 6}, {NULL}};
const Segment SGI116988[] = {{&S91627[31], 3}, {NULL}};
const Segment SGI116989[] = {{&S91627[76], 5}, {NULL}};
const Segment SGI116990[] = {{&S91627[113], 14}, {NULL}};
const Segment SGI116991[] = {{&S8219[44], 9}, {NULL}};
const Segment SGI116992[] = {{&S8219[44], 21}, {NULL}};
const Segment SGI116993[] = {{&S8219[30], 6}, {NULL}};
const Segment SGI116994[] = {{&S8219[31], 3}, {NULL}};
const Segment SGI116995[] = {{&S8219[79], 6}, {NULL}};
const Segment SGI116996[] = {{&S91629[34], 15}, {NULL}};
const Segment SGI116997[] = {{&S91629[27], 6}, {NULL}};
const Segment SGI116998[] = {{&S91629[102], 9}, {NULL}};
const Segment SGI116999[] = {{&S91629[75], 6}, {NULL}};
const Segment SGI117000[] = {{&S8220[44], 9}, {NULL}};
const Segment SGI117001[] = {{&S8220[44], 21}, {NULL}};
const Segment SGI117002[] = {{&S8220[30], 6}, {NULL}};
const Segment SGI117003[] = {{&S8220[31], 3}, {NULL}};
const Segment SGI117004[] = {{&S8220[79], 6}, {NULL}};
const Segment SGI117005[] = {{&S8222[44], 7}, {NULL}};
const Segment SGI117006[] = {{&S8222[44], 19}, {NULL}};
const Segment SGI117007[] = {{&S8222[30], 6}, {NULL}};
const Segment SGI117008[] = {{&S8222[31], 3}, {NULL}};
const Segment SGI117009[] = {{&S8222[77], 6}, {NULL}};
const Segment SGI117010[] = {{&S8223[44], 6}, {NULL}};
const Segment SGI117011[] = {{&S8223[44], 18}, {NULL}};
const Segment SGI117012[] = {{&S8223[30], 6}, {NULL}};
const Segment SGI117013[] = {{&S8223[31], 3}, {NULL}};
const Segment SGI117014[] = {{&S8223[76], 6}, {NULL}};
const Segment SGI117015[] = {{&S8224[44], 6}, {NULL}};
const Segment SGI117016[] = {{&S8224[44], 18}, {NULL}};
const Segment SGI117017[] = {{&S8224[30], 6}, {NULL}};
const Segment SGI117018[] = {{&S8224[31], 3}, {NULL}};
const Segment SGI117019[] = {{&S8224[76], 6}, {NULL}};
const Segment SGI117020[] = {{&S8225[44], 6}, {NULL}};
const Segment SGI117021[] = {{&S8225[44], 18}, {NULL}};
const Segment SGI117022[] = {{&S8225[30], 6}, {NULL}};
const Segment SGI117023[] = {{&S8225[31], 3}, {NULL}};
const Segment SGI117024[] = {{&S8225[76], 6}, {NULL}};
const Segment SGI117025[] = {{&S8226[44], 6}, {NULL}};
const Segment SGI117026[] = {{&S8226[44], 18}, {NULL}};
const Segment SGI117027[] = {{&S8226[30], 6}, {NULL}};
const Segment SGI117028[] = {{&S8226[31], 3}, {NULL}};
const Segment SGI117029[] = {{&S8226[76], 6}, {NULL}};
const Segment SGI117030[] = {{&S8227[44], 21}, {NULL}};
const Segment SGI117031[] = {{&S8227[44], 33}, {NULL}};
const Segment SGI117032[] = {{&S8227[30], 6}, {NULL}};
const Segment SGI117033[] = {{&S8227[31], 3}, {NULL}};
const Segment SGI117034[] = {{&S8227[91], 6}, {NULL}};
const Segment SGI117035[] = {{&S8229[44], 13}, {NULL}};
const Segment SGI117036[] = {{&S8229[44], 31}, {NULL}};
const Segment SGI117037[] = {{&S8229[30], 6}, {NULL}};
const Segment SGI117038[] = {{&S8229[31], 3}, {NULL}};
const Segment SGI117039[] = {{&S8229[89], 5}, {NULL}};
const Segment SGI117040[] = {{&S8230[38], 14}, {NULL}};
const Segment SGI117041[] = {{&S8230[31], 6}, {NULL}};
const Segment SGI117042[] = {{&S8231[44], 10}, {NULL}};
const Segment SGI117043[] = {{&S8231[44], 22}, {NULL}};
const Segment SGI117044[] = {{&S8231[30], 6}, {NULL}};
const Segment SGI117045[] = {{&S8231[31], 3}, {NULL}};
const Segment SGI117046[] = {{&S8231[80], 6}, {NULL}};
const Segment SGI117047[] = {{&S8232[44], 17}, {NULL}};
const Segment SGI117048[] = {{&S8232[44], 26}, {NULL}};
const Segment SGI117049[] = {{&S8232[30], 6}, {NULL}};
const Segment SGI117050[] = {{&S8232[31], 3}, {NULL}};
const Segment SGI117051[] = {{&S8232[84], 6}, {NULL}};
const Segment SGI117052[] = {{&S8233[44], 10}, {NULL}};
const Segment SGI117053[] = {{&S8233[44], 22}, {NULL}};
const Segment SGI117054[] = {{&S8233[30], 6}, {NULL}};
const Segment SGI117055[] = {{&S8233[31], 3}, {NULL}};
const Segment SGI117056[] = {{&S8233[80], 6}, {NULL}};
const Segment SGI117057[] = {{&S8234[44], 10}, {NULL}};
const Segment SGI117058[] = {{&S8234[44], 22}, {NULL}};
const Segment SGI117059[] = {{&S8234[30], 6}, {NULL}};
const Segment SGI117060[] = {{&S8234[31], 3}, {NULL}};
const Segment SGI117061[] = {{&S8234[80], 5}, {NULL}};
const Segment SGI117062[] = {{&S8235[44], 6}, {NULL}};
const Segment SGI117063[] = {{&S8235[44], 18}, {NULL}};
const Segment SGI117064[] = {{&S8235[30], 6}, {NULL}};
const Segment SGI117065[] = {{&S8235[31], 3}, {NULL}};
const Segment SGI117066[] = {{&S8235[76], 6}, {NULL}};
const Segment SGI117067[] = {{&S8236[44], 12}, {NULL}};
const Segment SGI117068[] = {{&S8236[44], 24}, {NULL}};
const Segment SGI117069[] = {{&S8236[30], 6}, {NULL}};
const Segment SGI117070[] = {{&S8236[31], 3}, {NULL}};
const Segment SGI117071[] = {{&S8236[82], 6}, {NULL}};
const Segment SGI117072[] = {{&S8237[44], 7}, {NULL}};
const Segment SGI117073[] = {{&S8237[44], 19}, {NULL}};
const Segment SGI117074[] = {{&S8237[30], 6}, {NULL}};
const Segment SGI117075[] = {{&S8237[31], 3}, {NULL}};
const Segment SGI117076[] = {{&S8237[77], 6}, {NULL}};
const Segment SGI117077[] = {{&S8238[44], 9}, {NULL}};
const Segment SGI117078[] = {{&S8238[44], 21}, {NULL}};
const Segment SGI117079[] = {{&S8238[30], 6}, {NULL}};
const Segment SGI117080[] = {{&S8238[31], 3}, {NULL}};
const Segment SGI117081[] = {{&S8238[79], 6}, {NULL}};
const Segment SGI117082[] = {{&S8239[44], 6}, {NULL}};
const Segment SGI117083[] = {{&S8239[44], 18}, {NULL}};
const Segment SGI117084[] = {{&S8239[30], 6}, {NULL}};
const Segment SGI117085[] = {{&S8239[31], 3}, {NULL}};
const Segment SGI117086[] = {{&S8239[76], 6}, {NULL}};
const Segment SGI117087[] = {{&S8240[44], 6}, {NULL}};
const Segment SGI117088[] = {{&S8240[44], 18}, {NULL}};
const Segment SGI117089[] = {{&S8240[30], 6}, {NULL}};
const Segment SGI117090[] = {{&S8240[31], 3}, {NULL}};
const Segment SGI117091[] = {{&S8240[76], 6}, {NULL}};
const Segment SGI117092[] = {{&S8241[44], 13}, {NULL}};
const Segment SGI117093[] = {{&S8241[44], 25}, {NULL}};
const Segment SGI117094[] = {{&S8241[30], 6}, {NULL}};
const Segment SGI117095[] = {{&S8241[31], 3}, {NULL}};
const Segment SGI117096[] = {{&S8241[83], 6}, {NULL}};
const Segment SGI117097[] = {{&S8242[44], 10}, {NULL}};
const Segment SGI117098[] = {{&S8242[44], 22}, {NULL}};
const Segment SGI117099[] = {{&S8242[30], 6}, {NULL}};
const Segment SGI117100[] = {{&S8242[31], 3}, {NULL}};
const Segment SGI117101[] = {{&S8242[80], 6}, {NULL}};
const Segment SGI117102[] = {{&S8244[44], 23}, {NULL}};
const Segment SGI117103[] = {{&S8244[44], 35}, {NULL}};
const Segment SGI117104[] = {{&S8244[30], 6}, {NULL}};
const Segment SGI117105[] = {{&S8244[31], 3}, {NULL}};
const Segment SGI117106[] = {{&S8244[93], 6}, {NULL}};
const Segment SGI117107[] = {{&S8247[44], 7}, {NULL}};
const Segment SGI117108[] = {{&S8247[44], 19}, {NULL}};
const Segment SGI117109[] = {{&S8247[30], 6}, {NULL}};
const Segment SGI117110[] = {{&S8247[31], 3}, {NULL}};
const Segment SGI117111[] = {{&S8247[77], 6}, {NULL}};
const Segment SGI117112[] = {{&S8248[44], 7}, {NULL}};
const Segment SGI117113[] = {{&S8248[44], 19}, {NULL}};
const Segment SGI117114[] = {{&S8248[30], 6}, {NULL}};
const Segment SGI117115[] = {{&S8248[31], 3}, {NULL}};
const Segment SGI117116[] = {{&S8248[77], 6}, {NULL}};
const Segment SGI117117[] = {{&S8249[44], 5}, {NULL}};
const Segment SGI117118[] = {{&S8249[44], 17}, {NULL}};
const Segment SGI117119[] = {{&S8249[30], 6}, {NULL}};
const Segment SGI117120[] = {{&S8249[31], 3}, {NULL}};
const Segment SGI117121[] = {{&S8249[75], 6}, {NULL}};
const Segment SGI117122[] = {{&S8251[44], 6}, {NULL}};
const Segment SGI117123[] = {{&S8251[44], 18}, {NULL}};
const Segment SGI117124[] = {{&S8251[30], 6}, {NULL}};
const Segment SGI117125[] = {{&S8251[31], 3}, {NULL}};
const Segment SGI117126[] = {{&S8251[76], 6}, {NULL}};
const Segment SGI117127[] = {{&S8252[44], 8}, {NULL}};
const Segment SGI117128[] = {{&S8252[44], 20}, {NULL}};
const Segment SGI117129[] = {{&S8252[30], 6}, {NULL}};
const Segment SGI117130[] = {{&S8252[31], 3}, {NULL}};
const Segment SGI117131[] = {{&S8252[78], 6}, {NULL}};
const Segment SGI117132[] = {{&S8253[44], 9}, {NULL}};
const Segment SGI117133[] = {{&S8253[44], 21}, {NULL}};
const Segment SGI117134[] = {{&S8253[30], 6}, {NULL}};
const Segment SGI117135[] = {{&S8253[31], 3}, {NULL}};
const Segment SGI117136[] = {{&S8253[79], 6}, {NULL}};
const Segment SGI117137[] = {{&S8254[44], 7}, {NULL}};
const Segment SGI117138[] = {{&S8254[44], 19}, {NULL}};
const Segment SGI117139[] = {{&S8254[30], 6}, {NULL}};
const Segment SGI117140[] = {{&S8254[31], 3}, {NULL}};
const Segment SGI117141[] = {{&S8254[77], 6}, {NULL}};
const Segment SGI117142[] = {{&S8256[44], 9}, {NULL}};
const Segment SGI117143[] = {{&S8256[44], 21}, {NULL}};
const Segment SGI117144[] = {{&S8256[30], 6}, {NULL}};
const Segment SGI117145[] = {{&S8256[31], 3}, {NULL}};
const Segment SGI117146[] = {{&S8256[79], 6}, {NULL}};
const Segment SGI117147[] = {{&S8257[44], 32}, {NULL}};
const Segment SGI117148[] = {{&S8257[30], 6}, {NULL}};
const Segment SGI117149[] = {{&S8257[31], 3}, {NULL}};
const Segment SGI117150[] = {{&S8257[109], 6}, {NULL}};
const Segment SGI117151[] = {{&S8258[44], 32}, {NULL}};
const Segment SGI117152[] = {{&S8258[30], 6}, {NULL}};
const Segment SGI117153[] = {{&S8258[31], 3}, {NULL}};
const Segment SGI117154[] = {{&S8258[109], 6}, {NULL}};
const Segment SGI117155[] = {{&S8259[59], 6}, {NULL}};
const Segment SGI117156[] = {{&S8259[42], 16}, {NULL}};
const Segment SGI117157[] = {{&S8259[29], 6}, {NULL}};
const Segment SGI117158[] = {{&S8259[30], 3}, {NULL}};
const Segment SGI117159[] = {{&S8259[88], 9}, {NULL}};
const Segment SGI117160[] = {{&S8261[38], 9}, {NULL}};
const Segment SGI117161[] = {{&S8261[31], 6}, {NULL}};
const Segment SGI117162[] = {{&S8263[44], 14}, {NULL}};
const Segment SGI117163[] = {{&S8263[44], 23}, {NULL}};
const Segment SGI117164[] = {{&S8263[30], 6}, {NULL}};
const Segment SGI117165[] = {{&S8263[84], 6}, {NULL}};
const Segment SGI117166[] = {{&S8264[44], 10}, {NULL}};
const Segment SGI117167[] = {{&S8264[44], 22}, {NULL}};
const Segment SGI117168[] = {{&S8264[30], 6}, {NULL}};
const Segment SGI117169[] = {{&S8264[31], 3}, {NULL}};
const Segment SGI117170[] = {{&S8264[80], 6}, {NULL}};
const Segment SGI117171[] = {{&S8265[44], 12}, {NULL}};
const Segment SGI117172[] = {{&S8265[44], 24}, {NULL}};
const Segment SGI117173[] = {{&S8265[30], 6}, {NULL}};
const Segment SGI117174[] = {{&S8265[31], 3}, {NULL}};
const Segment SGI117175[] = {{&S8265[82], 6}, {NULL}};
const Segment SGI117176[] = {{&S8266[44], 6}, {NULL}};
const Segment SGI117177[] = {{&S8266[44], 18}, {NULL}};
const Segment SGI117178[] = {{&S8266[30], 6}, {NULL}};
const Segment SGI117179[] = {{&S8266[31], 3}, {NULL}};
const Segment SGI117180[] = {{&S8266[76], 6}, {NULL}};
const Segment SGI117181[] = {{&S8267[44], 10}, {NULL}};
const Segment SGI117182[] = {{&S8267[44], 25}, {NULL}};
const Segment SGI117183[] = {{&S8267[30], 6}, {NULL}};
const Segment SGI117184[] = {{&S8267[31], 3}, {NULL}};
const Segment SGI117185[] = {{&S8267[83], 6}, {NULL}};
const Segment SGI117186[] = {{&S8269[44], 8}, {NULL}};
const Segment SGI117187[] = {{&S8269[44], 20}, {NULL}};
const Segment SGI117188[] = {{&S8269[30], 6}, {NULL}};
const Segment SGI117189[] = {{&S8269[31], 3}, {NULL}};
const Segment SGI117190[] = {{&S8269[78], 6}, {NULL}};
const Segment SGI117191[] = {{&S8270[44], 8}, {NULL}};
const Segment SGI117192[] = {{&S8270[44], 20}, {NULL}};
const Segment SGI117193[] = {{&S8270[30], 6}, {NULL}};
const Segment SGI117194[] = {{&S8270[31], 3}, {NULL}};
const Segment SGI117195[] = {{&S8270[78], 6}, {NULL}};
const Segment SGI117196[] = {{&S8271[44], 7}, {NULL}};
const Segment SGI117197[] = {{&S8271[44], 19}, {NULL}};
const Segment SGI117198[] = {{&S8271[30], 6}, {NULL}};
const Segment SGI117199[] = {{&S8271[31], 3}, {NULL}};
const Segment SGI117200[] = {{&S8271[77], 6}, {NULL}};
const Segment SGI117201[] = {{&S8272[44], 6}, {NULL}};
const Segment SGI117202[] = {{&S8272[44], 18}, {NULL}};
const Segment SGI117203[] = {{&S8272[30], 6}, {NULL}};
const Segment SGI117204[] = {{&S8272[31], 3}, {NULL}};
const Segment SGI117205[] = {{&S8272[76], 6}, {NULL}};
const Segment SGI117206[] = {{&S8273[44], 9}, {NULL}};
const Segment SGI117207[] = {{&S8273[44], 21}, {NULL}};
const Segment SGI117208[] = {{&S8273[30], 6}, {NULL}};
const Segment SGI117209[] = {{&S8273[31], 3}, {NULL}};
const Segment SGI117210[] = {{&S8273[79], 6}, {NULL}};
const Segment SGI117211[] = {{&S8274[44], 14}, {NULL}};
const Segment SGI117212[] = {{&S8274[44], 26}, {NULL}};
const Segment SGI117213[] = {{&S8274[30], 6}, {NULL}};
const Segment SGI117214[] = {{&S8274[31], 3}, {NULL}};
const Segment SGI117215[] = {{&S8274[84], 6}, {NULL}};
const Segment SGI117216[] = {{&S8275[44], 18}, {NULL}};
const Segment SGI117217[] = {{&S8275[44], 30}, {NULL}};
const Segment SGI117218[] = {{&S8275[30], 6}, {NULL}};
const Segment SGI117219[] = {{&S8275[91], 6}, {NULL}};
const Segment SGI117220[] = {{&S8276[44], 8}, {NULL}};
const Segment SGI117221[] = {{&S8276[44], 20}, {NULL}};
const Segment SGI117222[] = {{&S8276[30], 6}, {NULL}};
const Segment SGI117223[] = {{&S8276[31], 3}, {NULL}};
const Segment SGI117224[] = {{&S8276[78], 6}, {NULL}};
const Segment SGI117225[] = {{&S8277[44], 17}, {NULL}};
const Segment SGI117226[] = {{&S8277[44], 29}, {NULL}};
const Segment SGI117227[] = {{&S8277[30], 6}, {NULL}};
const Segment SGI117228[] = {{&S8277[31], 3}, {NULL}};
const Segment SGI117229[] = {{&S8277[87], 6}, {NULL}};
const Segment SGI117230[] = {{&S8283[44], 5}, {NULL}};
const Segment SGI117231[] = {{&S8283[44], 17}, {NULL}};
const Segment SGI117232[] = {{&S8283[30], 6}, {NULL}};
const Segment SGI117233[] = {{&S8283[31], 3}, {NULL}};
const Segment SGI117234[] = {{&S8283[75], 6}, {NULL}};
const Segment SGI117235[] = {{&S8284[44], 6}, {NULL}};
const Segment SGI117236[] = {{&S8284[44], 18}, {NULL}};
const Segment SGI117237[] = {{&S8284[30], 6}, {NULL}};
const Segment SGI117238[] = {{&S91721[34], 6}, {NULL}};
const Segment SGI117239[] = {{&S91721[27], 6}, {NULL}};
const Segment SGI117240[] = {{&S8284[31], 3}, {NULL}};
const Segment SGI117241[] = {{&S8284[76], 6}, {NULL}};
const Segment SGI117242[] = {{&S91721[93], 9}, {NULL}};
const Segment SGI117243[] = {{&S91721[66], 6}, {NULL}};
const Segment SGI117244[] = {{&S8285[44], 9}, {NULL}};
const Segment SGI117245[] = {{&S8285[44], 22}, {NULL}};
const Segment SGI117246[] = {{&S8285[30], 6}, {NULL}};
const Segment SGI117247[] = {{&S8285[31], 3}, {NULL}};
const Segment SGI117248[] = {{&S8285[80], 6}, {NULL}};
const Segment SGI117249[] = {{&S91723[44], 6}, {NULL}};
const Segment SGI117250[] = {{&S91723[44], 18}, {NULL}};
const Segment SGI117251[] = {{&S91723[30], 6}, {NULL}};
const Segment SGI117252[] = {{&S91723[31], 3}, {NULL}};
const Segment SGI117253[] = {{&S8286[44], 8}, {NULL}};
const Segment SGI117254[] = {{&S91723[76], 5}, {NULL}};
const Segment SGI117255[] = {{&S8286[44], 20}, {NULL}};
const Segment SGI117256[] = {{&S8286[30], 6}, {NULL}};
const Segment SGI117257[] = {{&S8286[31], 3}, {NULL}};
const Segment SGI117258[] = {{&S8286[78], 6}, {NULL}};
const Segment SGI117259[] = {{&S8287[44], 9}, {NULL}};
const Segment SGI117260[] = {{&S8287[44], 21}, {NULL}};
const Segment SGI117261[] = {{&S8287[30], 6}, {NULL}};
const Segment SGI117262[] = {{&S8287[31], 3}, {NULL}};
const Segment SGI117263[] = {{&S8287[79], 6}, {NULL}};
const Segment SGI117264[] = {{&S8288[44], 5}, {NULL}};
const Segment SGI117265[] = {{&S8288[44], 17}, {NULL}};
const Segment SGI117266[] = {{&S8288[30], 6}, {NULL}};
const Segment SGI117267[] = {{&S8288[31], 3}, {NULL}};
const Segment SGI117268[] = {{&S8288[75], 6}, {NULL}};
const Segment SGI117269[] = {{&S8289[44], 15}, {NULL}};
const Segment SGI117270[] = {{&S8289[44], 27}, {NULL}};
const Segment SGI117271[] = {{&S8289[30], 6}, {NULL}};
const Segment SGI117272[] = {{&S8289[31], 3}, {NULL}};
const Segment SGI117273[] = {{&S8289[85], 6}, {NULL}};
const Segment SGI117274[] = {{&S8290[44], 9}, {NULL}};
const Segment SGI117275[] = {{&S8290[44], 21}, {NULL}};
const Segment SGI117276[] = {{&S8290[30], 6}, {NULL}};
const Segment SGI117277[] = {{&S8290[31], 3}, {NULL}};
const Segment SGI117278[] = {{&S8290[79], 6}, {NULL}};
const Segment SGI117279[] = {{&S8291[44], 8}, {NULL}};
const Segment SGI117280[] = {{&S8291[44], 20}, {NULL}};
const Segment SGI117281[] = {{&S8291[30], 6}, {NULL}};
const Segment SGI117282[] = {{&S8291[31], 3}, {NULL}};
const Segment SGI117283[] = {{&S8291[78], 6}, {NULL}};
const Segment SGI117284[] = {{&S8292[44], 10}, {NULL}};
const Segment SGI117285[] = {{&S8292[44], 26}, {NULL}};
const Segment SGI117286[] = {{&S8292[30], 6}, {NULL}};
const Segment SGI117287[] = {{&S8292[31], 3}, {NULL}};
const Segment SGI117288[] = {{&S8292[84], 6}, {NULL}};
const Segment SGI117289[] = {{&S8293[44], 12}, {NULL}};
const Segment SGI117290[] = {{&S8293[44], 30}, {NULL}};
const Segment SGI117291[] = {{&S8293[30], 6}, {NULL}};
const Segment SGI117292[] = {{&S8293[31], 3}, {NULL}};
const Segment SGI117293[] = {{&S8293[88], 5}, {NULL}};
const Segment SGI117294[] = {{&S8295[44], 5}, {NULL}};
const Segment SGI117295[] = {{&S8295[44], 17}, {NULL}};
const Segment SGI117296[] = {{&S91752[44], 10}, {NULL}};
const Segment SGI117297[] = {{&S8295[30], 6}, {NULL}};
const Segment SGI117298[] = {{&S91752[44], 22}, {NULL}};
const Segment SGI117299[] = {{&S91752[30], 6}, {NULL}};
const Segment SGI117300[] = {{&S8295[31], 3}, {NULL}};
const Segment SGI117301[] = {{&S8295[75], 6}, {NULL}};
const Segment SGI117302[] = {{&S91752[111], 2}, {NULL}};
const Segment SGI117303[] = {{&S91752[80], 5}, {NULL}};
const Segment SGI117304[] = {{&S8296[44], 6}, {NULL}};
const Segment SGI117305[] = {{&S8296[44], 18}, {NULL}};
const Segment SGI117306[] = {{&S8296[30], 6}, {NULL}};
const Segment SGI117307[] = {{&S8296[31], 3}, {NULL}};
const Segment SGI117308[] = {{&S8296[76], 6}, {NULL}};
const Segment SGI117309[] = {{&S91755[44], 10}, {NULL}};
const Segment SGI117310[] = {{&S91755[44], 22}, {NULL}};
const Segment SGI117311[] = {{&S91755[30], 6}, {NULL}};
const Segment SGI117312[] = {{&S91755[31], 3}, {NULL}};
const Segment SGI117313[] = {{&S91755[80], 6}, {NULL}};
const Segment SGI117314[] = {{&S91755[143], 3}, {NULL}};
const Segment SGI117315[] = {{&S8299[44], 13}, {NULL}};
const Segment SGI117316[] = {{&S8299[44], 29}, {NULL}};
const Segment SGI117317[] = {{&S8299[30], 6}, {NULL}};
const Segment SGI117318[] = {{&S8299[31], 3}, {NULL}};
const Segment SGI117319[] = {{&S8299[87], 6}, {NULL}};
const Segment SGI117320[] = {{&S8300[44], 7}, {NULL}};
const Segment SGI117321[] = {{&S8300[44], 19}, {NULL}};
const Segment SGI117322[] = {{&S8300[30], 6}, {NULL}};
const Segment SGI117323[] = {{&S8300[31], 3}, {NULL}};
const Segment SGI117324[] = {{&S8300[77], 6}, {NULL}};
const Segment SGI117325[] = {{&S8301[44], 9}, {NULL}};
const Segment SGI117326[] = {{&S8301[44], 21}, {NULL}};
const Segment SGI117327[] = {{&S8301[30], 6}, {NULL}};
const Segment SGI117328[] = {{&S8301[31], 3}, {NULL}};
const Segment SGI117329[] = {{&S8301[79], 6}, {NULL}};
const Segment SGI117330[] = {{&S8302[44], 17}, {NULL}};
const Segment SGI117331[] = {{&S8302[30], 6}, {NULL}};
const Segment SGI117332[] = {{&S8302[31], 3}, {NULL}};
const Segment SGI117333[] = {{&S8302[75], 6}, {NULL}};
const Segment SGI117334[] = {{&S8303[44], 6}, {NULL}};
const Segment SGI117335[] = {{&S8303[44], 18}, {NULL}};
const Segment SGI117336[] = {{&S8303[30], 6}, {NULL}};
const Segment SGI117337[] = {{&S8303[31], 3}, {NULL}};
const Segment SGI117338[] = {{&S8303[76], 5}, {NULL}};
const Segment SGI117339[] = {{&S8304[44], 12}, {NULL}};
const Segment SGI117340[] = {{&S8304[44], 24}, {NULL}};
const Segment SGI117341[] = {{&S8304[30], 6}, {NULL}};
const Segment SGI117342[] = {{&S8304[31], 3}, {NULL}};
const Segment SGI117343[] = {{&S8304[82], 6}, {NULL}};
const Segment SGI117344[] = {{&S8308[44], 8}, {NULL}};
const Segment SGI117345[] = {{&S8308[44], 17}, {NULL}};
const Segment SGI117346[] = {{&S8308[30], 6}, {NULL}};
const Segment SGI117347[] = {{&S8308[31], 3}, {NULL}};
const Segment SGI117348[] = {{&S8308[75], 6}, {NULL}};
const Segment SGI117349[] = {{&S8310[44], 7}, {NULL}};
const Segment SGI117350[] = {{&S8310[44], 19}, {NULL}};
const Segment SGI117351[] = {{&S8310[30], 6}, {NULL}};
const Segment SGI117352[] = {{&S8310[31], 3}, {NULL}};
const Segment SGI117353[] = {{&S8310[77], 6}, {NULL}};
const Segment SGI117354[] = {{&S8311[44], 11}, {NULL}};
const Segment SGI117355[] = {{&S8311[44], 23}, {NULL}};
const Segment SGI117356[] = {{&S8311[30], 6}, {NULL}};
const Segment SGI117357[] = {{&S8311[31], 3}, {NULL}};
const Segment SGI117358[] = {{&S8311[81], 6}, {NULL}};
const Segment SGI117359[] = {{&S91778[34], 9}, {NULL}};
const Segment SGI117360[] = {{&S91778[27], 6}, {NULL}};
const Segment SGI117361[] = {{&S91778[96], 9}, {NULL}};
const Segment SGI117362[] = {{&S8312[38], 4}, {NULL}};
const Segment SGI117363[] = {{&S91778[69], 6}, {NULL}};
const Segment SGI117364[] = {{&S8312[31], 6}, {NULL}};
const Segment SGI117365[] = {{&S8313[44], 8}, {NULL}};
const Segment SGI117366[] = {{&S8313[44], 20}, {NULL}};
const Segment SGI117367[] = {{&S8313[30], 6}, {NULL}};
const Segment SGI117368[] = {{&S8313[31], 3}, {NULL}};
const Segment SGI117369[] = {{&S8313[78], 6}, {NULL}};
const Segment SGI117370[] = {{&S8315[44], 9}, {NULL}};
const Segment SGI117371[] = {{&S8315[44], 21}, {NULL}};
const Segment SGI117372[] = {{&S8315[30], 6}, {NULL}};
const Segment SGI117373[] = {{&S8315[31], 3}, {NULL}};
const Segment SGI117374[] = {{&S8315[79], 6}, {NULL}};
const Segment SGI117375[] = {{&S8316[44], 20}, {NULL}};
const Segment SGI117376[] = {{&S8316[44], 32}, {NULL}};
const Segment SGI117377[] = {{&S8316[30], 6}, {NULL}};
const Segment SGI117378[] = {{&S8316[31], 3}, {NULL}};
const Segment SGI117379[] = {{&S8316[90], 6}, {NULL}};
const Segment SGI117380[] = {{&S8317[44], 33}, {NULL}};
const Segment SGI117381[] = {{&S8317[44], 45}, {NULL}};
const Segment SGI117382[] = {{&S8317[30], 6}, {NULL}};
const Segment SGI117383[] = {{&S8317[31], 3}, {NULL}};
const Segment SGI117384[] = {{&S8317[103], 6}, {NULL}};
const Segment SGI117385[] = {{&S8318[44], 4}, {NULL}};
const Segment SGI117386[] = {{&S8318[44], 16}, {NULL}};
const Segment SGI117387[] = {{&S8318[30], 6}, {NULL}};
const Segment SGI117388[] = {{&S8318[31], 3}, {NULL}};
const Segment SGI117389[] = {{&S8318[74], 6}, {NULL}};
const Segment SGI117390[] = {{&S8319[44], 9}, {NULL}};
const Segment SGI117391[] = {{&S8319[44], 21}, {NULL}};
const Segment SGI117392[] = {{&S8319[30], 6}, {NULL}};
const Segment SGI117393[] = {{&S8319[31], 3}, {NULL}};
const Segment SGI117394[] = {{&S8319[79], 6}, {NULL}};
const Segment SGI117395[] = {{&S8320[44], 7}, {NULL}};
const Segment SGI117396[] = {{&S8320[44], 19}, {NULL}};
const Segment SGI117397[] = {{&S8320[30], 6}, {NULL}};
const Segment SGI117398[] = {{&S8320[31], 3}, {NULL}};
const Segment SGI117399[] = {{&S8320[77], 6}, {NULL}};
const Segment SGI117400[] = {{&S8321[44], 8}, {NULL}};
const Segment SGI117401[] = {{&S8321[44], 20}, {NULL}};
const Segment SGI117402[] = {{&S8321[30], 6}, {NULL}};
const Segment SGI117403[] = {{&S8321[31], 3}, {NULL}};
const Segment SGI117404[] = {{&S8321[78], 6}, {NULL}};
const Segment SGI117405[] = {{&S8322[44], 18}, {NULL}};
const Segment SGI117406[] = {{&S8322[44], 30}, {NULL}};
const Segment SGI117407[] = {{&S8322[30], 6}, {NULL}};
const Segment SGI117408[] = {{&S8322[31], 3}, {NULL}};
const Segment SGI117409[] = {{&S8322[88], 6}, {NULL}};
const Segment SGI117410[] = {{&S8323[38], 12}, {NULL}};
const Segment SGI117411[] = {{&S8323[31], 6}, {NULL}};
const Segment SGI117412[] = {{&S8324[44], 8}, {NULL}};
const Segment SGI117413[] = {{&S8324[44], 20}, {NULL}};
const Segment SGI117414[] = {{&S8324[30], 6}, {NULL}};
const Segment SGI117415[] = {{&S8324[31], 3}, {NULL}};
const Segment SGI117416[] = {{&S8324[78], 6}, {NULL}};
const Segment SGI117417[] = {{&S8325[44], 6}, {NULL}};
const Segment SGI117418[] = {{&S8325[44], 18}, {NULL}};
const Segment SGI117419[] = {{&S8325[30], 6}, {NULL}};
const Segment SGI117420[] = {{&S8325[31], 3}, {NULL}};
const Segment SGI117421[] = {{&S8325[76], 6}, {NULL}};
const Segment SGI117422[] = {{&S8326[44], 11}, {NULL}};
const Segment SGI117423[] = {{&S8326[44], 23}, {NULL}};
const Segment SGI117424[] = {{&S8326[30], 6}, {NULL}};
const Segment SGI117425[] = {{&S8326[31], 3}, {NULL}};
const Segment SGI117426[] = {{&S8326[81], 6}, {NULL}};
const Segment SGI117427[] = {{&S8327[44], 6}, {NULL}};
const Segment SGI117428[] = {{&S8327[44], 18}, {NULL}};
const Segment SGI117429[] = {{&S8327[30], 6}, {NULL}};
const Segment SGI117430[] = {{&S8327[31], 3}, {NULL}};
const Segment SGI117431[] = {{&S8327[76], 6}, {NULL}};
const Segment SGI117432[] = {{&S8328[44], 11}, {NULL}};
const Segment SGI117433[] = {{&S8328[44], 23}, {NULL}};
const Segment SGI117434[] = {{&S8328[30], 6}, {NULL}};
const Segment SGI117435[] = {{&S8328[31], 3}, {NULL}};
const Segment SGI117436[] = {{&S8328[81], 6}, {NULL}};
const Segment SGI117437[] = {{&S8330[44], 17}, {NULL}};
const Segment SGI117438[] = {{&S8330[44], 29}, {NULL}};
const Segment SGI117439[] = {{&S8330[30], 6}, {NULL}};
const Segment SGI117440[] = {{&S8330[31], 3}, {NULL}};
const Segment SGI117441[] = {{&S8330[87], 6}, {NULL}};
const Segment SGI117442[] = {{&S91814[45], 5}, {NULL}};
const Segment SGI117443[] = {{&S91814[30], 6}, {NULL}};
const Segment SGI117444[] = {{&S91814[31], 3}, {NULL}};
const Segment SGI117445[] = {{&S91814[76], 5}, {NULL}};
const Segment SGI117446[] = {{&S91814[134], 13}, {NULL}};
const Segment SGI117447[] = {{&S8331[44], 12}, {NULL}};
const Segment SGI117448[] = {{&S8331[44], 24}, {NULL}};
const Segment SGI117449[] = {{&S8331[30], 6}, {NULL}};
const Segment SGI117450[] = {{&S8331[31], 3}, {NULL}};
const Segment SGI117451[] = {{&S8331[82], 6}, {NULL}};
const Segment SGI117452[] = {{&S8333[44], 8}, {NULL}};
const Segment SGI117453[] = {{&S8333[44], 20}, {NULL}};
const Segment SGI117454[] = {{&S8333[30], 6}, {NULL}};
const Segment SGI117455[] = {{&S8333[31], 3}, {NULL}};
const Segment SGI117456[] = {{&S8333[78], 5}, {NULL}};
const Segment SGI117457[] = {{&S8334[44], 11}, {NULL}};
const Segment SGI117458[] = {{&S8334[44], 23}, {NULL}};
const Segment SGI117459[] = {{&S8334[30], 6}, {NULL}};
const Segment SGI117460[] = {{&S8334[31], 3}, {NULL}};
const Segment SGI117461[] = {{&S8334[81], 6}, {NULL}};
const Segment SGI117462[] = {{&S8335[44], 13}, {NULL}};
const Segment SGI117463[] = {{&S8335[44], 25}, {NULL}};
const Segment SGI117464[] = {{&S8335[30], 6}, {NULL}};
const Segment SGI117465[] = {{&S8335[31], 3}, {NULL}};
const Segment SGI117466[] = {{&S8335[83], 6}, {NULL}};
const Segment SGI117467[] = {{&S91823[52], 13}, {NULL}};
const Segment SGI117468[] = {{&S91823[52], 29}, {NULL}};
const Segment SGI117469[] = {{&S8336[44], 10}, {NULL}};
const Segment SGI117470[] = {{&S91823[38], 6}, {NULL}};
const Segment SGI117471[] = {{&S8336[44], 22}, {NULL}};
const Segment SGI117472[] = {{&S8336[30], 6}, {NULL}};
const Segment SGI117473[] = {{&S8336[31], 3}, {NULL}};
const Segment SGI117474[] = {{&S91823[95], 5}, {NULL}};
const Segment SGI117475[] = {{&S8336[80], 6}, {NULL}};
const Segment SGI117476[] = {{&S91823[0], 14}, {NULL}};
const Segment SGI117477[] = {{&S8338[44], 7}, {NULL}};
const Segment SGI117478[] = {{&S8338[44], 19}, {NULL}};
const Segment SGI117479[] = {{&S8338[30], 6}, {NULL}};
const Segment SGI117480[] = {{&S8338[31], 3}, {NULL}};
const Segment SGI117481[] = {{&S8338[77], 6}, {NULL}};
const Segment SGI117482[] = {{&S8339[44], 13}, {NULL}};
const Segment SGI117483[] = {{&S8339[44], 25}, {NULL}};
const Segment SGI117484[] = {{&S8339[30], 6}, {NULL}};
const Segment SGI117485[] = {{&S8339[31], 3}, {NULL}};
const Segment SGI117486[] = {{&S8339[83], 6}, {NULL}};
const Segment SGI117487[] = {{&S91828[52], 14}, {NULL}};
const Segment SGI117488[] = {{&S91828[52], 26}, {NULL}};
const Segment SGI117489[] = {{&S91828[38], 6}, {NULL}};
const Segment SGI117490[] = {{&S91828[92], 5}, {NULL}};
const Segment SGI117491[] = {{&S91828[0], 14}, {NULL}};
const Segment SGI117492[] = {{&S8340[44], 10}, {NULL}};
const Segment SGI117493[] = {{&S8340[44], 25}, {NULL}};
const Segment SGI117494[] = {{&S8340[30], 6}, {NULL}};
const Segment SGI117495[] = {{&S8340[31], 3}, {NULL}};
const Segment SGI117496[] = {{&S8340[83], 6}, {NULL}};
const Segment SGI117497[] = {{&S8341[44], 14}, {NULL}};
const Segment SGI117498[] = {{&S8341[44], 26}, {NULL}};
const Segment SGI117499[] = {{&S8341[30], 6}, {NULL}};
const Segment SGI117500[] = {{&S8341[31], 3}, {NULL}};
const Segment SGI117501[] = {{&S8341[84], 6}, {NULL}};
const Segment SGI117502[] = {{&S8343[44], 11}, {NULL}};
const Segment SGI117503[] = {{&S8343[44], 23}, {NULL}};
const Segment SGI117504[] = {{&S8343[30], 6}, {NULL}};
const Segment SGI117505[] = {{&S8343[31], 3}, {NULL}};
const Segment SGI117506[] = {{&S8343[81], 6}, {NULL}};
const Segment SGI117507[] = {{&S8344[44], 9}, {NULL}};
const Segment SGI117508[] = {{&S8344[44], 21}, {NULL}};
const Segment SGI117509[] = {{&S8344[30], 6}, {NULL}};
const Segment SGI117510[] = {{&S8344[31], 3}, {NULL}};
const Segment SGI117511[] = {{&S8344[79], 6}, {NULL}};
const Segment SGI117512[] = {{&S8345[44], 6}, {NULL}};
const Segment SGI117513[] = {{&S8345[44], 18}, {NULL}};
const Segment SGI117514[] = {{&S8345[30], 6}, {NULL}};
const Segment SGI117515[] = {{&S8345[31], 3}, {NULL}};
const Segment SGI117516[] = {{&S8345[76], 6}, {NULL}};
const Segment SGI117517[] = {{&S8346[38], 9}, {NULL}};
const Segment SGI117518[] = {{&S8346[31], 6}, {NULL}};
const Segment SGI117519[] = {{&S8347[44], 7}, {NULL}};
const Segment SGI117520[] = {{&S8347[44], 19}, {NULL}};
const Segment SGI117521[] = {{&S8347[30], 6}, {NULL}};
const Segment SGI117522[] = {{&S8347[31], 3}, {NULL}};
const Segment SGI117523[] = {{&S8347[77], 6}, {NULL}};
const Segment SGI117524[] = {{&S8350[44], 12}, {NULL}};
const Segment SGI117525[] = {{&S8350[44], 24}, {NULL}};
const Segment SGI117526[] = {{&S8350[30], 6}, {NULL}};
const Segment SGI117527[] = {{&S8350[31], 3}, {NULL}};
const Segment SGI117528[] = {{&S8350[82], 6}, {NULL}};
const Segment SGI117529[] = {{&S8351[44], 6}, {NULL}};
const Segment SGI117530[] = {{&S8351[44], 18}, {NULL}};
const Segment SGI117531[] = {{&S8351[30], 6}, {NULL}};
const Segment SGI117532[] = {{&S8351[31], 3}, {NULL}};
const Segment SGI117533[] = {{&S8351[76], 6}, {NULL}};
const Segment SGI117534[] = {{&S8352[44], 8}, {NULL}};
const Segment SGI117535[] = {{&S8352[44], 20}, {NULL}};
const Segment SGI117536[] = {{&S8352[30], 6}, {NULL}};
const Segment SGI117537[] = {{&S8352[31], 3}, {NULL}};
const Segment SGI117538[] = {{&S8352[78], 6}, {NULL}};
const Segment SGI117539[] = {{&S8353[44], 11}, {NULL}};
const Segment SGI117540[] = {{&S8353[44], 23}, {NULL}};
const Segment SGI117541[] = {{&S8353[30], 6}, {NULL}};
const Segment SGI117542[] = {{&S8353[31], 3}, {NULL}};
const Segment SGI117543[] = {{&S8353[81], 6}, {NULL}};
const Segment SGI117544[] = {{&S8355[44], 23}, {NULL}};
const Segment SGI117545[] = {{&S8355[30], 6}, {NULL}};
const Segment SGI117546[] = {{&S8355[31], 3}, {NULL}};
const Segment SGI117547[] = {{&S8355[99], 6}, {NULL}};
const Segment SGI117548[] = {{&S8356[44], 8}, {NULL}};
const Segment SGI117549[] = {{&S8356[44], 20}, {NULL}};
const Segment SGI117550[] = {{&S8356[30], 6}, {NULL}};
const Segment SGI117551[] = {{&S8356[31], 3}, {NULL}};
const Segment SGI117552[] = {{&S8356[78], 6}, {NULL}};
const Segment SGI117553[] = {{&S8357[44], 11}, {NULL}};
const Segment SGI117554[] = {{&S8357[44], 23}, {NULL}};
const Segment SGI117555[] = {{&S8357[30], 6}, {NULL}};
const Segment SGI117556[] = {{&S8357[31], 3}, {NULL}};
const Segment SGI117557[] = {{&S8357[81], 6}, {NULL}};
const Segment SGI117558[] = {{&S8360[44], 13}, {NULL}};
const Segment SGI117559[] = {{&S8360[44], 25}, {NULL}};
const Segment SGI117560[] = {{&S8360[30], 6}, {NULL}};
const Segment SGI117561[] = {{&S8360[31], 3}, {NULL}};
const Segment SGI117562[] = {{&S8360[83], 6}, {NULL}};
const Segment SGI117563[] = {{&S8361[38], 10}, {NULL}};
const Segment SGI117564[] = {{&S8361[31], 6}, {NULL}};
const Segment SGI117565[] = {{&S8362[44], 8}, {NULL}};
const Segment SGI117566[] = {{&S8362[44], 20}, {NULL}};
const Segment SGI117567[] = {{&S8362[30], 6}, {NULL}};
const Segment SGI117568[] = {{&S8362[31], 3}, {NULL}};
const Segment SGI117569[] = {{&S8362[78], 6}, {NULL}};
const Segment SGI117570[] = {{&S8365[44], 15}, {NULL}};
const Segment SGI117571[] = {{&S8365[44], 27}, {NULL}};
const Segment SGI117572[] = {{&S8365[30], 6}, {NULL}};
const Segment SGI117573[] = {{&S8365[31], 3}, {NULL}};
const Segment SGI117574[] = {{&S8365[85], 6}, {NULL}};
const Segment SGI117575[] = {{&S8366[44], 7}, {NULL}};
const Segment SGI117576[] = {{&S8366[44], 19}, {NULL}};
const Segment SGI117577[] = {{&S8366[30], 6}, {NULL}};
const Segment SGI117578[] = {{&S8366[31], 3}, {NULL}};
const Segment SGI117579[] = {{&S8366[77], 6}, {NULL}};
const Segment SGI117580[] = {{&S8369[44], 7}, {NULL}};
const Segment SGI117581[] = {{&S8369[44], 19}, {NULL}};
const Segment SGI117582[] = {{&S8369[30], 6}, {NULL}};
const Segment SGI117583[] = {{&S8369[31], 3}, {NULL}};
const Segment SGI117584[] = {{&S8369[77], 6}, {NULL}};
const Segment SGI117585[] = {{&S8370[44], 11}, {NULL}};
const Segment SGI117586[] = {{&S8370[44], 23}, {NULL}};
const Segment SGI117587[] = {{&S8370[30], 6}, {NULL}};
const Segment SGI117588[] = {{&S8370[31], 3}, {NULL}};
const Segment SGI117589[] = {{&S8370[81], 6}, {NULL}};
const Segment SGI117590[] = {{&S8371[44], 23}, {NULL}};
const Segment SGI117591[] = {{&S8371[44], 35}, {NULL}};
const Segment SGI117592[] = {{&S8371[30], 6}, {NULL}};
const Segment SGI117593[] = {{&S8371[31], 3}, {NULL}};
const Segment SGI117594[] = {{&S8371[93], 6}, {NULL}};
const Segment SGI117595[] = {{&S8372[44], 9}, {NULL}};
const Segment SGI117596[] = {{&S8372[44], 21}, {NULL}};
const Segment SGI117597[] = {{&S8372[30], 6}, {NULL}};
const Segment SGI117598[] = {{&S8372[31], 3}, {NULL}};
const Segment SGI117599[] = {{&S8372[79], 6}, {NULL}};
const Segment SGI117600[] = {{&S8374[44], 10}, {NULL}};
const Segment SGI117601[] = {{&S8374[44], 25}, {NULL}};
const Segment SGI117602[] = {{&S8374[30], 6}, {NULL}};
const Segment SGI117603[] = {{&S8374[31], 3}, {NULL}};
const Segment SGI117604[] = {{&S8374[83], 6}, {NULL}};
const Segment SGI117605[] = {{&S8375[44], 9}, {NULL}};
const Segment SGI117606[] = {{&S8375[44], 21}, {NULL}};
const Segment SGI117607[] = {{&S8375[30], 6}, {NULL}};
const Segment SGI117608[] = {{&S8375[31], 3}, {NULL}};
const Segment SGI117609[] = {{&S8375[79], 6}, {NULL}};
const Segment SGI117610[] = {{&S8376[44], 7}, {NULL}};
const Segment SGI117611[] = {{&S8376[44], 19}, {NULL}};
const Segment SGI117612[] = {{&S8376[30], 6}, {NULL}};
const Segment SGI117613[] = {{&S8376[31], 3}, {NULL}};
const Segment SGI117614[] = {{&S8376[77], 6}, {NULL}};
const Segment SGI117615[] = {{&S8377[44], 20}, {NULL}};
const Segment SGI117616[] = {{&S8377[44], 32}, {NULL}};
const Segment SGI117617[] = {{&S8377[30], 6}, {NULL}};
const Segment SGI117618[] = {{&S8377[31], 3}, {NULL}};
const Segment SGI117619[] = {{&S8377[90], 6}, {NULL}};
const Segment SGI117620[] = {{&S8378[44], 5}, {NULL}};
const Segment SGI117621[] = {{&S8378[44], 17}, {NULL}};
const Segment SGI117622[] = {{&S8378[30], 6}, {NULL}};
const Segment SGI117623[] = {{&S8378[31], 3}, {NULL}};
const Segment SGI117624[] = {{&S8378[75], 6}, {NULL}};
const Segment SGI117625[] = {{&S8379[44], 6}, {NULL}};
const Segment SGI117626[] = {{&S8379[44], 18}, {NULL}};
const Segment SGI117627[] = {{&S8379[30], 6}, {NULL}};
const Segment SGI117628[] = {{&S8379[31], 3}, {NULL}};
const Segment SGI117629[] = {{&S8379[76], 6}, {NULL}};
const Segment SGI117630[] = {{&S8380[44], 74}, {NULL}};
const Segment SGI117631[] = {{&S8380[44], 95}, {NULL}};
const Segment SGI117632[] = {{&S8380[30], 6}, {NULL}};
const Segment SGI117633[] = {{&S8380[31], 3}, {NULL}};
const Segment SGI117634[] = {{&S8380[112], 5}, {NULL}};
const Segment SGI117635[] = {{&S8381[44], 17}, {NULL}};
const Segment SGI117636[] = {{&S8381[44], 29}, {NULL}};
const Segment SGI117637[] = {{&S8381[30], 6}, {NULL}};
const Segment SGI117638[] = {{&S8381[31], 3}, {NULL}};
const Segment SGI117639[] = {{&S8381[87], 6}, {NULL}};
const Segment SGI117640[] = {{&S8382[44], 12}, {NULL}};
const Segment SGI117641[] = {{&S8382[44], 24}, {NULL}};
const Segment SGI117642[] = {{&S8382[30], 6}, {NULL}};
const Segment SGI117643[] = {{&S8382[31], 3}, {NULL}};
const Segment SGI117644[] = {{&S8382[82], 6}, {NULL}};
const Segment SGI117645[] = {{&S91909[57], 4}, {NULL}};
const Segment SGI117646[] = {{&S91909[57], 26}, {NULL}};
const Segment SGI117647[] = {{&S91909[43], 6}, {NULL}};
const Segment SGI117648[] = {{&S8384[44], 6}, {NULL}};
const Segment SGI117649[] = {{&S8384[44], 18}, {NULL}};
const Segment SGI117650[] = {{&S8384[30], 6}, {NULL}};
const Segment SGI117651[] = {{&S91909[44], 3}, {NULL}};
const Segment SGI117652[] = {{&S91909[97], 6}, {NULL}};
const Segment SGI117653[] = {{&S8384[31], 3}, {NULL}};
const Segment SGI117654[] = {{&S8384[76], 6}, {NULL}};
const Segment SGI117655[] = {{&S91909[0], 12}, {NULL}};
const Segment SGI117656[] = {{&S8385[44], 9}, {NULL}};
const Segment SGI117657[] = {{&S8385[44], 21}, {NULL}};
const Segment SGI117658[] = {{&S8385[30], 6}, {NULL}};
const Segment SGI117659[] = {{&S8385[31], 3}, {NULL}};
const Segment SGI117660[] = {{&S8385[79], 6}, {NULL}};
const Segment SGI117661[] = {{&S8386[44], 14}, {NULL}};
const Segment SGI117662[] = {{&S8386[44], 26}, {NULL}};
const Segment SGI117663[] = {{&S8386[30], 6}, {NULL}};
const Segment SGI117664[] = {{&S8386[31], 3}, {NULL}};
const Segment SGI117665[] = {{&S8386[84], 6}, {NULL}};
const Segment SGI117666[] = {{&S8387[44], 6}, {NULL}};
const Segment SGI117667[] = {{&S8387[44], 18}, {NULL}};
const Segment SGI117668[] = {{&S8387[30], 6}, {NULL}};
const Segment SGI117669[] = {{&S8387[31], 3}, {NULL}};
const Segment SGI117670[] = {{&S8387[76], 6}, {NULL}};
const Segment SGI117671[] = {{&S8388[44], 6}, {NULL}};
const Segment SGI117672[] = {{&S8388[44], 18}, {NULL}};
const Segment SGI117673[] = {{&S8388[30], 6}, {NULL}};
const Segment SGI117674[] = {{&S8388[31], 3}, {NULL}};
const Segment SGI117675[] = {{&S8388[76], 6}, {NULL}};
const Segment SGI117676[] = {{&S8389[44], 7}, {NULL}};
const Segment SGI117677[] = {{&S8389[44], 19}, {NULL}};
const Segment SGI117678[] = {{&S8389[30], 6}, {NULL}};
const Segment SGI117679[] = {{&S8389[31], 3}, {NULL}};
const Segment SGI117680[] = {{&S8389[77], 6}, {NULL}};
const Segment SGI117681[] = {{&S8391[44], 8}, {NULL}};
const Segment SGI117682[] = {{&S8391[44], 20}, {NULL}};
const Segment SGI117683[] = {{&S8391[30], 6}, {NULL}};
const Segment SGI117684[] = {{&S8391[31], 3}, {NULL}};
const Segment SGI117685[] = {{&S8391[78], 6}, {NULL}};
const Segment SGI117686[] = {{&S8392[44], 13}, {NULL}};
const Segment SGI117687[] = {{&S8392[44], 25}, {NULL}};
const Segment SGI117688[] = {{&S8392[30], 6}, {NULL}};
const Segment SGI117689[] = {{&S8392[31], 3}, {NULL}};
const Segment SGI117690[] = {{&S8392[83], 6}, {NULL}};
const Segment SGI117691[] = {{&S91924[45], 6}, {NULL}};
const Segment SGI117692[] = {{&S91924[30], 6}, {NULL}};
const Segment SGI117693[] = {{&S91924[31], 3}, {NULL}};
const Segment SGI117694[] = {{&S91924[77], 5}, {NULL}};
const Segment SGI117695[] = {{&S91924[135], 13}, {NULL}};
const Segment SGI117696[] = {{&S8393[44], 9}, {NULL}};
const Segment SGI117697[] = {{&S8393[44], 21}, {NULL}};
const Segment SGI117698[] = {{&S8393[30], 6}, {NULL}};
const Segment SGI117699[] = {{&S8393[31], 3}, {NULL}};
const Segment SGI117700[] = {{&S8393[79], 6}, {NULL}};
const Segment SGI117701[] = {{&S8395[44], 8}, {NULL}};
const Segment SGI117702[] = {{&S8395[44], 20}, {NULL}};
const Segment SGI117703[] = {{&S8395[30], 6}, {NULL}};
const Segment SGI117704[] = {{&S8395[31], 3}, {NULL}};
const Segment SGI117705[] = {{&S91928[34], 9}, {NULL}};
const Segment SGI117706[] = {{&S8395[78], 6}, {NULL}};
const Segment SGI117707[] = {{&S91928[27], 6}, {NULL}};
const Segment SGI117708[] = {{&S91928[96], 9}, {NULL}};
const Segment SGI117709[] = {{&S91928[69], 6}, {NULL}};
const Segment SGI117710[] = {{&S8396[44], 11}, {NULL}};
const Segment SGI117711[] = {{&S8396[44], 23}, {NULL}};
const Segment SGI117712[] = {{&S8396[30], 6}, {NULL}};
const Segment SGI117713[] = {{&S8396[31], 3}, {NULL}};
const Segment SGI117714[] = {{&S8396[81], 6}, {NULL}};
const Segment SGI117715[] = {{&S8398[44], 7}, {NULL}};
const Segment SGI117716[] = {{&S8398[44], 19}, {NULL}};
const Segment SGI117717[] = {{&S8398[30], 6}, {NULL}};
const Segment SGI117718[] = {{&S8398[31], 3}, {NULL}};
const Segment SGI117719[] = {{&S8398[77], 6}, {NULL}};
const Segment SGI117720[] = {{&S8399[38], 14}, {NULL}};
const Segment SGI117721[] = {{&S8399[31], 6}, {NULL}};
const Segment SGI117722[] = {{&S8400[44], 7}, {NULL}};
const Segment SGI117723[] = {{&S8400[44], 22}, {NULL}};
const Segment SGI117724[] = {{&S8400[30], 6}, {NULL}};
const Segment SGI117725[] = {{&S8400[31], 3}, {NULL}};
const Segment SGI117726[] = {{&S8400[80], 6}, {NULL}};
const Segment SGI117727[] = {{&S8401[44], 10}, {NULL}};
const Segment SGI117728[] = {{&S8401[44], 25}, {NULL}};
const Segment SGI117729[] = {{&S8401[30], 6}, {NULL}};
const Segment SGI117730[] = {{&S8401[31], 3}, {NULL}};
const Segment SGI117731[] = {{&S8401[83], 6}, {NULL}};
const Segment SGI117732[] = {{&S91938[34], 9}, {NULL}};
const Segment SGI117733[] = {{&S91938[27], 6}, {NULL}};
const Segment SGI117734[] = {{&S8402[44], 7}, {NULL}};
const Segment SGI117735[] = {{&S91938[96], 9}, {NULL}};
const Segment SGI117736[] = {{&S8402[44], 19}, {NULL}};
const Segment SGI117737[] = {{&S91938[69], 6}, {NULL}};
const Segment SGI117738[] = {{&S8402[30], 6}, {NULL}};
const Segment SGI117739[] = {{&S8402[31], 3}, {NULL}};
const Segment SGI117740[] = {{&S8402[77], 6}, {NULL}};
const Segment SGI117741[] = {{&S8403[44], 8}, {NULL}};
const Segment SGI117742[] = {{&S8403[44], 20}, {NULL}};
const Segment SGI117743[] = {{&S8403[30], 6}, {NULL}};
const Segment SGI117744[] = {{&S8403[31], 3}, {NULL}};
const Segment SGI117745[] = {{&S8403[78], 6}, {NULL}};
const Segment SGI117746[] = {{&S8404[38], 14}, {NULL}};
const Segment SGI117747[] = {{&S8404[31], 6}, {NULL}};
const Segment SGI117748[] = {{&S8405[44], 8}, {NULL}};
const Segment SGI117749[] = {{&S8405[44], 20}, {NULL}};
const Segment SGI117750[] = {{&S8405[30], 6}, {NULL}};
const Segment SGI117751[] = {{&S8405[31], 3}, {NULL}};
const Segment SGI117752[] = {{&S8405[78], 6}, {NULL}};
const Segment SGI117753[] = {{&S8408[44], 11}, {NULL}};
const Segment SGI117754[] = {{&S8408[44], 23}, {NULL}};
const Segment SGI117755[] = {{&S8408[30], 6}, {NULL}};
const Segment SGI117756[] = {{&S8408[31], 3}, {NULL}};
const Segment SGI117757[] = {{&S8408[81], 6}, {NULL}};
const Segment SGI117758[] = {{&S8410[44], 8}, {NULL}};
const Segment SGI117759[] = {{&S8410[44], 31}, {NULL}};
const Segment SGI117760[] = {{&S8410[30], 6}, {NULL}};
const Segment SGI117761[] = {{&S8410[31], 3}, {NULL}};
const Segment SGI117762[] = {{&S8410[89], 6}, {NULL}};
const Segment SGI117763[] = {{&S8411[44], 8}, {NULL}};
const Segment SGI117764[] = {{&S8411[44], 20}, {NULL}};
const Segment SGI117765[] = {{&S8411[30], 6}, {NULL}};
const Segment SGI117766[] = {{&S8411[31], 3}, {NULL}};
const Segment SGI117767[] = {{&S8411[78], 6}, {NULL}};
const Segment SGI117768[] = {{&S8413[44], 9}, {NULL}};
const Segment SGI117769[] = {{&S8413[44], 21}, {NULL}};
const Segment SGI117770[] = {{&S8413[30], 6}, {NULL}};
const Segment SGI117771[] = {{&S8413[31], 3}, {NULL}};
const Segment SGI117772[] = {{&S8413[79], 6}, {NULL}};
const Segment SGI117773[] = {{&S8414[44], 7}, {NULL}};
const Segment SGI117774[] = {{&S8414[44], 19}, {NULL}};
const Segment SGI117775[] = {{&S8414[30], 6}, {NULL}};
const Segment SGI117776[] = {{&S8414[31], 3}, {NULL}};
const Segment SGI117777[] = {{&S8414[77], 6}, {NULL}};
const Segment SGI117778[] = {{&S8415[44], 9}, {NULL}};
const Segment SGI117779[] = {{&S8415[44], 21}, {NULL}};
const Segment SGI117780[] = {{&S8415[30], 6}, {NULL}};
const Segment SGI117781[] = {{&S8415[31], 3}, {NULL}};
const Segment SGI117782[] = {{&S8415[79], 6}, {NULL}};
const Segment SGI117783[] = {{&S8416[44], 10}, {NULL}};
const Segment SGI117784[] = {{&S8416[44], 33}, {NULL}};
const Segment SGI117785[] = {{&S8416[30], 6}, {NULL}};
const Segment SGI117786[] = {{&S8416[31], 3}, {NULL}};
const Segment SGI117787[] = {{&S8416[91], 6}, {NULL}};
const Segment SGI117788[] = {{&S8417[44], 8}, {NULL}};
const Segment SGI117789[] = {{&S8417[44], 30}, {NULL}};
const Segment SGI117790[] = {{&S8417[30], 6}, {NULL}};
const Segment SGI117791[] = {{&S8417[31], 3}, {NULL}};
const Segment SGI117792[] = {{&S8417[88], 6}, {NULL}};
const Segment SGI117793[] = {{&S8418[44], 9}, {NULL}};
const Segment SGI117794[] = {{&S8418[44], 21}, {NULL}};
const Segment SGI117795[] = {{&S8418[30], 6}, {NULL}};
const Segment SGI117796[] = {{&S8418[31], 3}, {NULL}};
const Segment SGI117797[] = {{&S8418[79], 6}, {NULL}};
const Segment SGI117798[] = {{&S8419[44], 12}, {NULL}};
const Segment SGI117799[] = {{&S8419[44], 30}, {NULL}};
const Segment SGI117800[] = {{&S8419[30], 6}, {NULL}};
const Segment SGI117801[] = {{&S8419[31], 3}, {NULL}};
const Segment SGI117802[] = {{&S8419[88], 5}, {NULL}};
const Segment SGI117803[] = {{&S8420[44], 15}, {NULL}};
const Segment SGI117804[] = {{&S8420[44], 31}, {NULL}};
const Segment SGI117805[] = {{&S8420[30], 6}, {NULL}};
const Segment SGI117806[] = {{&S8420[31], 3}, {NULL}};
const Segment SGI117807[] = {{&S8420[89], 6}, {NULL}};
const Segment SGI117808[] = {{&S8421[44], 14}, {NULL}};
const Segment SGI117809[] = {{&S8421[44], 26}, {NULL}};
const Segment SGI117810[] = {{&S8421[30], 6}, {NULL}};
const Segment SGI117811[] = {{&S8421[31], 3}, {NULL}};
const Segment SGI117812[] = {{&S8421[84], 6}, {NULL}};
const Segment SGI117813[] = {{&S8424[44], 6}, {NULL}};
const Segment SGI117814[] = {{&S8424[44], 18}, {NULL}};
const Segment SGI117815[] = {{&S8424[30], 6}, {NULL}};
const Segment SGI117816[] = {{&S8424[31], 3}, {NULL}};
const Segment SGI117817[] = {{&S8424[76], 6}, {NULL}};
const Segment SGI117818[] = {{&S8426[44], 8}, {NULL}};
const Segment SGI117819[] = {{&S8426[44], 20}, {NULL}};
const Segment SGI117820[] = {{&S8426[30], 6}, {NULL}};
const Segment SGI117821[] = {{&S8426[31], 3}, {NULL}};
const Segment SGI117822[] = {{&S8426[78], 6}, {NULL}};
const Segment SGI117823[] = {{&S8427[44], 7}, {NULL}};
const Segment SGI117824[] = {{&S8427[44], 19}, {NULL}};
const Segment SGI117825[] = {{&S8427[30], 6}, {NULL}};
const Segment SGI117826[] = {{&S8427[31], 3}, {NULL}};
const Segment SGI117827[] = {{&S8427[77], 6}, {NULL}};
const Segment SGI117828[] = {{&S8431[44], 6}, {NULL}};
const Segment SGI117829[] = {{&S8431[44], 18}, {NULL}};
const Segment SGI117830[] = {{&S8431[30], 6}, {NULL}};
const Segment SGI117831[] = {{&S8431[31], 3}, {NULL}};
const Segment SGI117832[] = {{&S8431[76], 6}, {NULL}};
const Segment SGI117833[] = {{&S8432[44], 10}, {NULL}};
const Segment SGI117834[] = {{&S8432[44], 22}, {NULL}};
const Segment SGI117835[] = {{&S8432[30], 6}, {NULL}};
const Segment SGI117836[] = {{&S8432[31], 3}, {NULL}};
const Segment SGI117837[] = {{&S8432[80], 6}, {NULL}};
const Segment SGI117838[] = {{&S8433[44], 12}, {NULL}};
const Segment SGI117839[] = {{&S8433[44], 24}, {NULL}};
const Segment SGI117840[] = {{&S8433[30], 6}, {NULL}};
const Segment SGI117841[] = {{&S8433[31], 3}, {NULL}};
const Segment SGI117842[] = {{&S8433[82], 6}, {NULL}};
const Segment SGI117843[] = {{&S8434[44], 7}, {NULL}};
const Segment SGI117844[] = {{&S8434[44], 19}, {NULL}};
const Segment SGI117845[] = {{&S8434[30], 6}, {NULL}};
const Segment SGI117846[] = {{&S8434[31], 3}, {NULL}};
const Segment SGI117847[] = {{&S8434[77], 6}, {NULL}};
const Segment SGI117848[] = {{&S8435[44], 7}, {NULL}};
const Segment SGI117849[] = {{&S8435[44], 19}, {NULL}};
const Segment SGI117850[] = {{&S8435[30], 6}, {NULL}};
const Segment SGI117851[] = {{&S8435[31], 3}, {NULL}};
const Segment SGI117852[] = {{&S8435[77], 6}, {NULL}};
const Segment SGI117853[] = {{&S8438[44], 19}, {NULL}};
const Segment SGI117854[] = {{&S8438[44], 31}, {NULL}};
const Segment SGI117855[] = {{&S8438[30], 6}, {&S8438[52], 23}, {NULL}};
const Segment SGI117856[] = {{&S8438[31], 3}, {NULL}};
const Segment SGI117857[] = {{&S8438[89], 6}, {NULL}};
const Segment SGI117858[] = {{&S8439[44], 7}, {NULL}};
const Segment SGI117859[] = {{&S8439[44], 19}, {NULL}};
const Segment SGI117860[] = {{&S8439[30], 6}, {NULL}};
const Segment SGI117861[] = {{&S8439[31], 3}, {NULL}};
const Segment SGI117862[] = {{&S8439[77], 6}, {NULL}};
const Segment SGI117863[] = {{&S8441[44], 15}, {NULL}};
const Segment SGI117864[] = {{&S8441[44], 27}, {NULL}};
const Segment SGI117865[] = {{&S8441[30], 6}, {NULL}};
const Segment SGI117866[] = {{&S8441[31], 3}, {NULL}};
const Segment SGI117867[] = {{&S8441[85], 6}, {NULL}};
const Segment SGI117868[] = {{&S8442[44], 7}, {NULL}};
const Segment SGI117869[] = {{&S8442[44], 19}, {NULL}};
const Segment SGI117870[] = {{&S8442[30], 6}, {NULL}};
const Segment SGI117871[] = {{&S8442[31], 3}, {NULL}};
const Segment SGI117872[] = {{&S8442[77], 6}, {NULL}};
const Segment SGI117873[] = {{&S92008[44], 11}, {NULL}};
const Segment SGI117874[] = {{&S92008[44], 23}, {NULL}};
const Segment SGI117875[] = {{&S92008[30], 6}, {NULL}};
const Segment SGI117876[] = {{&S8443[44], 13}, {NULL}};
const Segment SGI117877[] = {{&S8443[44], 25}, {NULL}};
const Segment SGI117878[] = {{&S8443[30], 6}, {NULL}};
const Segment SGI117879[] = {{&S92008[81], 6}, {NULL}};
const Segment SGI117880[] = {{&S92008[108], 13}, {NULL}};
const Segment SGI117881[] = {{&S8443[31], 3}, {NULL}};
const Segment SGI117882[] = {{&S8443[83], 6}, {NULL}};
const Segment SGI117883[] = {{&S8444[44], 14}, {NULL}};
const Segment SGI117884[] = {{&S8444[44], 26}, {NULL}};
const Segment SGI117885[] = {{&S8444[30], 6}, {NULL}};
const Segment SGI117886[] = {{&S8444[31], 3}, {NULL}};
const Segment SGI117887[] = {{&S8444[84], 6}, {NULL}};
const Segment SGI117888[] = {{&S8445[44], 21}, {NULL}};
const Segment SGI117889[] = {{&S8445[44], 42}, {NULL}};
const Segment SGI117890[] = {{&S8445[30], 6}, {NULL}};
const Segment SGI117891[] = {{&S8445[31], 3}, {NULL}};
const Segment SGI117892[] = {{&S8445[100], 6}, {NULL}};
const Segment SGI117893[] = {{&S8446[44], 8}, {NULL}};
const Segment SGI117894[] = {{&S8446[44], 20}, {NULL}};
const Segment SGI117895[] = {{&S8446[30], 6}, {NULL}};
const Segment SGI117896[] = {{&S8446[31], 3}, {NULL}};
const Segment SGI117897[] = {{&S8446[78], 6}, {NULL}};
const Segment SGI117898[] = {{&S8447[44], 9}, {NULL}};
const Segment SGI117899[] = {{&S8447[44], 21}, {NULL}};
const Segment SGI117900[] = {{&S8447[30], 6}, {NULL}};
const Segment SGI117901[] = {{&S8447[31], 3}, {NULL}};
const Segment SGI117902[] = {{&S8447[79], 6}, {NULL}};
const Segment SGI117903[] = {{&S92017[34], 19}, {NULL}};
const Segment SGI117904[] = {{&S92017[27], 6}, {NULL}};
const Segment SGI117905[] = {{&S8448[44], 8}, {NULL}};
const Segment SGI117906[] = {{&S8448[44], 20}, {NULL}};
const Segment SGI117907[] = {{&S8448[30], 6}, {NULL}};
const Segment SGI117908[] = {{&S92017[106], 9}, {NULL}};
const Segment SGI117909[] = {{&S92017[79], 6}, {NULL}};
const Segment SGI117910[] = {{&S8448[31], 3}, {NULL}};
const Segment SGI117911[] = {{&S92017[140], 8}, {NULL}};
const Segment SGI117912[] = {{&S8448[78], 6}, {NULL}};
const Segment SGI117913[] = {{&S8452[44], 13}, {NULL}};
const Segment SGI117914[] = {{&S8452[44], 25}, {NULL}};
const Segment SGI117915[] = {{&S8452[30], 6}, {NULL}};
const Segment SGI117916[] = {{&S8452[31], 3}, {NULL}};
const Segment SGI117917[] = {{&S8452[83], 6}, {NULL}};
const Segment SGI117918[] = {{&S8453[44], 10}, {NULL}};
const Segment SGI117919[] = {{&S8453[44], 22}, {NULL}};
const Segment SGI117920[] = {{&S8453[30], 6}, {NULL}};
const Segment SGI117921[] = {{&S8453[31], 3}, {NULL}};
const Segment SGI117922[] = {{&S8453[80], 6}, {NULL}};
const Segment SGI117923[] = {{&S8454[44], 10}, {NULL}};
const Segment SGI117924[] = {{&S8454[44], 22}, {NULL}};
const Segment SGI117925[] = {{&S8454[30], 6}, {NULL}};
const Segment SGI117926[] = {{&S8454[31], 3}, {NULL}};
const Segment SGI117927[] = {{&S8454[80], 6}, {NULL}};
const Segment SGI117928[] = {{&S8455[38], 10}, {NULL}};
const Segment SGI117929[] = {{&S8455[31], 6}, {NULL}};
const Segment SGI117930[] = {{&S8456[44], 30}, {NULL}};
const Segment SGI117931[] = {{&S8456[44], 42}, {NULL}};
const Segment SGI117932[] = {{&S8456[30], 6}, {NULL}};
const Segment SGI117933[] = {{&S8456[31], 3}, {NULL}};
const Segment SGI117934[] = {{&S8456[100], 6}, {NULL}};
const Segment SGI117935[] = {{&S8457[44], 8}, {NULL}};
const Segment SGI117936[] = {{&S8457[44], 20}, {NULL}};
const Segment SGI117937[] = {{&S8457[30], 6}, {NULL}};
const Segment SGI117938[] = {{&S8457[31], 3}, {NULL}};
const Segment SGI117939[] = {{&S8457[78], 6}, {NULL}};
const Segment SGI117940[] = {{&S8458[44], 10}, {NULL}};
const Segment SGI117941[] = {{&S8458[44], 22}, {NULL}};
const Segment SGI117942[] = {{&S8458[30], 6}, {NULL}};
const Segment SGI117943[] = {{&S8458[31], 3}, {NULL}};
const Segment SGI117944[] = {{&S8458[80], 6}, {NULL}};
const Segment SGI117945[] = {{&S8459[44], 11}, {NULL}};
const Segment SGI117946[] = {{&S8459[44], 23}, {NULL}};
const Segment SGI117947[] = {{&S8459[30], 6}, {NULL}};
const Segment SGI117948[] = {{&S8459[31], 3}, {NULL}};
const Segment SGI117949[] = {{&S8459[81], 6}, {NULL}};
const Segment SGI117950[] = {{&S8460[44], 30}, {NULL}};
const Segment SGI117951[] = {{&S8460[44], 42}, {NULL}};
const Segment SGI117952[] = {{&S8460[30], 6}, {NULL}};
const Segment SGI117953[] = {{&S8460[31], 3}, {NULL}};
const Segment SGI117954[] = {{&S8460[100], 6}, {NULL}};
const Segment SGI117955[] = {{&S8461[44], 11}, {NULL}};
const Segment SGI117956[] = {{&S8461[44], 23}, {NULL}};
const Segment SGI117957[] = {{&S8461[30], 6}, {NULL}};
const Segment SGI117958[] = {{&S8461[31], 3}, {NULL}};
const Segment SGI117959[] = {{&S8461[81], 6}, {NULL}};
const Segment SGI117960[] = {{&S8462[44], 29}, {NULL}};
const Segment SGI117961[] = {{&S8462[44], 41}, {NULL}};
const Segment SGI117962[] = {{&S8462[30], 6}, {NULL}};
const Segment SGI117963[] = {{&S8462[31], 3}, {NULL}};
const Segment SGI117964[] = {{&S8462[99], 6}, {NULL}};
const Segment SGI117965[] = {{&S8463[38], 10}, {NULL}};
const Segment SGI117966[] = {{&S8463[31], 6}, {NULL}};
const Segment SGI117967[] = {{&S8464[44], 30}, {NULL}};
const Segment SGI117968[] = {{&S8464[44], 42}, {NULL}};
const Segment SGI117969[] = {{&S8464[30], 6}, {NULL}};
const Segment SGI117970[] = {{&S8464[31], 3}, {NULL}};
const Segment SGI117971[] = {{&S8464[100], 6}, {NULL}};
const Segment SGI117972[] = {{&S8465[44], 30}, {NULL}};
const Segment SGI117973[] = {{&S8465[44], 42}, {NULL}};
const Segment SGI117974[] = {{&S8465[30], 6}, {NULL}};
const Segment SGI117975[] = {{&S8465[31], 3}, {NULL}};
const Segment SGI117976[] = {{&S8465[100], 6}, {NULL}};
const Segment SGI117977[] = {{&S8466[44], 30}, {NULL}};
const Segment SGI117978[] = {{&S8466[44], 42}, {NULL}};
const Segment SGI117979[] = {{&S8466[30], 6}, {NULL}};
const Segment SGI117980[] = {{&S8466[31], 3}, {NULL}};
const Segment SGI117981[] = {{&S8466[100], 6}, {NULL}};
const Segment SGI117982[] = {{&S92054[34], 9}, {NULL}};
const Segment SGI117983[] = {{&S92054[27], 6}, {NULL}};
const Segment SGI117984[] = {{&S92054[96], 9}, {NULL}};
const Segment SGI117985[] = {{&S92054[69], 6}, {NULL}};
const Segment SGI117986[] = {{&S8467[44], 10}, {NULL}};
const Segment SGI117987[] = {{&S8467[44], 22}, {NULL}};
const Segment SGI117988[] = {{&S8467[30], 6}, {NULL}};
const Segment SGI117989[] = {{&S8467[31], 3}, {NULL}};
const Segment SGI117990[] = {{&S8467[80], 6}, {NULL}};
const Segment SGI117991[] = {{&S8468[44], 30}, {NULL}};
const Segment SGI117992[] = {{&S8468[44], 42}, {NULL}};
const Segment SGI117993[] = {{&S8468[30], 6}, {NULL}};
const Segment SGI117994[] = {{&S8468[31], 3}, {NULL}};
const Segment SGI117995[] = {{&S8468[100], 6}, {NULL}};
const Segment SGI117996[] = {{&S8469[38], 10}, {NULL}};
const Segment SGI117997[] = {{&S8469[31], 6}, {NULL}};
const Segment SGI117998[] = {{&S8470[44], 29}, {NULL}};
const Segment SGI117999[] = {{&S8470[44], 41}, {NULL}};
const Segment SGI118000[] = {{&S8470[30], 6}, {NULL}};
const Segment SGI118001[] = {{&S8470[31], 3}, {NULL}};
const Segment SGI118002[] = {{&S8470[99], 6}, {NULL}};
const Segment SGI118003[] = {{&S8471[44], 30}, {NULL}};
const Segment SGI118004[] = {{&S8471[44], 42}, {NULL}};
const Segment SGI118005[] = {{&S8471[30], 6}, {NULL}};
const Segment SGI118006[] = {{&S8471[31], 3}, {NULL}};
const Segment SGI118007[] = {{&S8471[100], 6}, {NULL}};
const Segment SGI118008[] = {{&S8472[44], 30}, {NULL}};
const Segment SGI118009[] = {{&S8472[44], 42}, {NULL}};
const Segment SGI118010[] = {{&S8472[30], 6}, {NULL}};
const Segment SGI118011[] = {{&S8472[31], 3}, {NULL}};
const Segment SGI118012[] = {{&S8472[100], 6}, {NULL}};
const Segment SGI118013[] = {{&S8473[44], 10}, {NULL}};
const Segment SGI118014[] = {{&S8473[44], 22}, {NULL}};
const Segment SGI118015[] = {{&S8473[30], 6}, {NULL}};
const Segment SGI118016[] = {{&S8473[31], 3}, {NULL}};
const Segment SGI118017[] = {{&S8473[80], 6}, {NULL}};
const Segment SGI118018[] = {{&S8474[42], 10}, {NULL}};
const Segment SGI118019[] = {{&S8474[30], 6}, {NULL}};
const Segment SGI118020[] = {{&S8474[31], 3}, {NULL}};
const Segment SGI118021[] = {{&S8474[78], 6}, {NULL}};
const Segment SGI118022[] = {{&S8475[44], 10}, {NULL}};
const Segment SGI118023[] = {{&S8475[44], 22}, {NULL}};
const Segment SGI118024[] = {{&S8475[30], 6}, {NULL}};
const Segment SGI118025[] = {{&S8475[83], 6}, {NULL}};
const Segment SGI118026[] = {{&S8476[38], 11}, {NULL}};
const Segment SGI118027[] = {{&S8476[31], 6}, {NULL}};
const Segment SGI118028[] = {{&S8477[44], 29}, {NULL}};
const Segment SGI118029[] = {{&S8477[44], 41}, {NULL}};
const Segment SGI118030[] = {{&S8477[30], 6}, {NULL}};
const Segment SGI118031[] = {{&S8477[31], 3}, {NULL}};
const Segment SGI118032[] = {{&S8477[99], 6}, {NULL}};
const Segment SGI118033[] = {{&S8478[44], 30}, {NULL}};
const Segment SGI118034[] = {{&S8478[44], 42}, {NULL}};
const Segment SGI118035[] = {{&S8478[30], 6}, {NULL}};
const Segment SGI118036[] = {{&S8478[31], 3}, {NULL}};
const Segment SGI118037[] = {{&S8478[100], 6}, {NULL}};
const Segment SGI118038[] = {{&S8479[44], 30}, {NULL}};
const Segment SGI118039[] = {{&S8479[44], 42}, {NULL}};
const Segment SGI118040[] = {{&S8479[30], 6}, {NULL}};
const Segment SGI118041[] = {{&S8479[31], 3}, {NULL}};
const Segment SGI118042[] = {{&S8479[100], 6}, {NULL}};
const Segment SGI118043[] = {{&S8480[44], 30}, {NULL}};
const Segment SGI118044[] = {{&S8480[44], 42}, {NULL}};
const Segment SGI118045[] = {{&S8480[30], 6}, {NULL}};
const Segment SGI118046[] = {{&S8480[31], 3}, {NULL}};
const Segment SGI118047[] = {{&S8480[100], 6}, {NULL}};
const Segment SGI118048[] = {{&S8481[44], 10}, {NULL}};
const Segment SGI118049[] = {{&S8481[44], 22}, {NULL}};
const Segment SGI118050[] = {{&S8481[30], 6}, {NULL}};
const Segment SGI118051[] = {{&S8481[83], 4}, {NULL}};
const Segment SGI118052[] = {{&S8481[83], 3}, {NULL}};
const Segment SGI118053[] = {{&S8482[44], 10}, {NULL}};
const Segment SGI118054[] = {{&S8482[44], 22}, {NULL}};
const Segment SGI118055[] = {{&S8482[30], 6}, {NULL}};
const Segment SGI118056[] = {{&S8482[83], 6}, {NULL}};
const Segment SGI118057[] = {{&S8483[38], 8}, {NULL}};
const Segment SGI118058[] = {{&S8483[31], 6}, {NULL}};
const Segment SGI118059[] = {{&S8484[44], 30}, {NULL}};
const Segment SGI118060[] = {{&S8484[44], 42}, {NULL}};
const Segment SGI118061[] = {{&S8484[30], 6}, {NULL}};
const Segment SGI118062[] = {{&S8484[31], 3}, {NULL}};
const Segment SGI118063[] = {{&S8484[100], 6}, {NULL}};
const Segment SGI118064[] = {{&S8485[38], 11}, {NULL}};
const Segment SGI118065[] = {{&S8485[31], 6}, {NULL}};
const Segment SGI118066[] = {{&S8486[44], 30}, {NULL}};
const Segment SGI118067[] = {{&S8486[44], 42}, {NULL}};
const Segment SGI118068[] = {{&S8486[30], 6}, {NULL}};
const Segment SGI118069[] = {{&S8486[31], 3}, {NULL}};
const Segment SGI118070[] = {{&S8486[100], 6}, {NULL}};
const Segment SGI118071[] = {{&S8487[44], 29}, {NULL}};
const Segment SGI118072[] = {{&S8487[44], 41}, {NULL}};
const Segment SGI118073[] = {{&S8487[30], 6}, {NULL}};
const Segment SGI118074[] = {{&S8487[31], 3}, {NULL}};
const Segment SGI118075[] = {{&S8487[99], 6}, {NULL}};
const Segment SGI118076[] = {{&S8488[44], 10}, {NULL}};
const Segment SGI118077[] = {{&S8488[44], 22}, {NULL}};
const Segment SGI118078[] = {{&S8488[30], 6}, {NULL}};
const Segment SGI118079[] = {{&S8488[31], 3}, {NULL}};
const Segment SGI118080[] = {{&S8488[80], 6}, {NULL}};
const Segment SGI118081[] = {{&S8489[42], 10}, {NULL}};
const Segment SGI118082[] = {{&S8489[30], 6}, {NULL}};
const Segment SGI118083[] = {{&S8489[31], 3}, {NULL}};
const Segment SGI118084[] = {{&S8489[78], 6}, {NULL}};
const Segment SGI118085[] = {{&S8490[44], 10}, {NULL}};
const Segment SGI118086[] = {{&S8490[44], 22}, {NULL}};
const Segment SGI118087[] = {{&S8490[30], 6}, {NULL}};
const Segment SGI118088[] = {{&S8490[80], 6}, {NULL}};
const Segment SGI118089[] = {{&S8491[44], 30}, {NULL}};
const Segment SGI118090[] = {{&S8491[44], 42}, {NULL}};
const Segment SGI118091[] = {{&S8491[30], 6}, {NULL}};
const Segment SGI118092[] = {{&S8491[31], 3}, {NULL}};
const Segment SGI118093[] = {{&S8491[100], 6}, {NULL}};
const Segment SGI118094[] = {{&S8492[42], 11}, {NULL}};
const Segment SGI118095[] = {{&S8492[30], 6}, {NULL}};
const Segment SGI118096[] = {{&S8492[31], 3}, {NULL}};
const Segment SGI118097[] = {{&S8492[79], 6}, {NULL}};
const Segment SGI118098[] = {{&S8493[44], 10}, {NULL}};
const Segment SGI118099[] = {{&S8493[44], 22}, {NULL}};
const Segment SGI118100[] = {{&S8493[30], 6}, {NULL}};
const Segment SGI118101[] = {{&S8493[31], 3}, {NULL}};
const Segment SGI118102[] = {{&S8493[80], 6}, {NULL}};
const Segment SGI118103[] = {{&S8494[44], 7}, {NULL}};
const Segment SGI118104[] = {{&S8494[44], 19}, {NULL}};
const Segment SGI118105[] = {{&S8494[30], 6}, {NULL}};
const Segment SGI118106[] = {{&S8494[31], 3}, {NULL}};
const Segment SGI118107[] = {{&S8494[77], 6}, {NULL}};
const Segment SGI118108[] = {{&S8495[42], 30}, {NULL}};
const Segment SGI118109[] = {{&S8495[30], 6}, {NULL}};
const Segment SGI118110[] = {{&S8495[31], 3}, {NULL}};
const Segment SGI118111[] = {{&S8495[98], 6}, {NULL}};
const Segment SGI118112[] = {{&S8496[44], 10}, {NULL}};
const Segment SGI118113[] = {{&S8496[44], 22}, {NULL}};
const Segment SGI118114[] = {{&S8496[30], 6}, {NULL}};
const Segment SGI118115[] = {{&S8496[83], 6}, {NULL}};
const Segment SGI118116[] = {{&S8497[44], 10}, {NULL}};
const Segment SGI118117[] = {{&S8497[44], 22}, {NULL}};
const Segment SGI118118[] = {{&S8497[30], 6}, {NULL}};
const Segment SGI118119[] = {{&S8497[80], 6}, {NULL}};
const Segment SGI118120[] = {{&S8498[44], 30}, {NULL}};
const Segment SGI118121[] = {{&S8498[44], 42}, {NULL}};
const Segment SGI118122[] = {{&S8498[30], 6}, {NULL}};
const Segment SGI118123[] = {{&S8498[31], 3}, {NULL}};
const Segment SGI118124[] = {{&S8498[100], 6}, {NULL}};
const Segment SGI118125[] = {{&S8500[44], 11}, {NULL}};
const Segment SGI118126[] = {{&S8500[44], 23}, {NULL}};
const Segment SGI118127[] = {{&S8500[30], 6}, {NULL}};
const Segment SGI118128[] = {{&S8500[31], 3}, {NULL}};
const Segment SGI118129[] = {{&S8500[81], 6}, {NULL}};
const Segment SGI118130[] = {{&S8501[44], 10}, {NULL}};
const Segment SGI118131[] = {{&S8501[44], 22}, {NULL}};
const Segment SGI118132[] = {{&S8501[30], 6}, {NULL}};
const Segment SGI118133[] = {{&S8501[80], 6}, {NULL}};
const Segment SGI118134[] = {{&S8502[42], 10}, {NULL}};
const Segment SGI118135[] = {{&S8502[30], 6}, {NULL}};
const Segment SGI118136[] = {{&S8502[31], 3}, {NULL}};
const Segment SGI118137[] = {{&S8502[78], 6}, {NULL}};
const Segment SGI118138[] = {{&S92137[34], 11}, {NULL}};
const Segment SGI118139[] = {{&S92137[27], 6}, {NULL}};
const Segment SGI118140[] = {{&S92137[98], 9}, {NULL}};
const Segment SGI118141[] = {{&S92137[71], 6}, {NULL}};
const Segment SGI118142[] = {{&S8503[44], 10}, {NULL}};
const Segment SGI118143[] = {{&S8503[44], 22}, {NULL}};
const Segment SGI118144[] = {{&S8503[30], 6}, {NULL}};
const Segment SGI118145[] = {{&S8503[83], 4}, {NULL}};
const Segment SGI118146[] = {{&S8503[83], 3}, {NULL}};
const Segment SGI118147[] = {{&S8504[42], 11}, {NULL}};
const Segment SGI118148[] = {{&S8504[30], 6}, {NULL}};
const Segment SGI118149[] = {{&S8504[31], 3}, {NULL}};
const Segment SGI118150[] = {{&S8504[79], 6}, {NULL}};
const Segment SGI118151[] = {{&S8506[44], 22}, {NULL}};
const Segment SGI118152[] = {{&S8506[30], 6}, {NULL}};
const Segment SGI118153[] = {{&S8507[42], 30}, {NULL}};
const Segment SGI118154[] = {{&S8507[30], 6}, {NULL}};
const Segment SGI118155[] = {{&S8507[31], 3}, {NULL}};
const Segment SGI118156[] = {{&S8507[98], 6}, {NULL}};
const Segment SGI118157[] = {{&S92145[34], 12}, {NULL}};
const Segment SGI118158[] = {{&S92145[27], 6}, {NULL}};
const Segment SGI118159[] = {{&S92145[99], 9}, {NULL}};
const Segment SGI118160[] = {{&S92145[72], 6}, {NULL}};
const Segment SGI118161[] = {{&S8511[44], 10}, {NULL}};
const Segment SGI118162[] = {{&S8511[44], 22}, {NULL}};
const Segment SGI118163[] = {{&S8511[30], 6}, {NULL}};
const Segment SGI118164[] = {{&S8511[31], 3}, {NULL}};
const Segment SGI118165[] = {{&S8511[80], 6}, {NULL}};
const Segment SGI118166[] = {{&S8512[44], 10}, {NULL}};
const Segment SGI118167[] = {{&S8512[44], 22}, {NULL}};
const Segment SGI118168[] = {{&S8512[30], 6}, {NULL}};
const Segment SGI118169[] = {{&S8512[83], 6}, {NULL}};
const Segment SGI118170[] = {{&S8513[44], 22}, {NULL}};
const Segment SGI118171[] = {{&S8513[30], 6}, {NULL}};
const Segment SGI118172[] = {{&S92155[45], 21}, {NULL}};
const Segment SGI118173[] = {{&S92155[30], 6}, {NULL}};
const Segment SGI118174[] = {{&S92155[31], 3}, {NULL}};
const Segment SGI118175[] = {{&S8514[42], 29}, {NULL}};
const Segment SGI118176[] = {{&S92155[92], 5}, {NULL}};
const Segment SGI118177[] = {{&S8514[30], 6}, {NULL}};
const Segment SGI118178[] = {{&S92155[150], 13}, {NULL}};
const Segment SGI118179[] = {{&S8514[31], 3}, {NULL}};
const Segment SGI118180[] = {{&S8514[97], 6}, {NULL}};
const Segment SGI118181[] = {{&S8515[44], 7}, {NULL}};
const Segment SGI118182[] = {{&S8515[44], 19}, {NULL}};
const Segment SGI118183[] = {{&S8515[30], 6}, {NULL}};
const Segment SGI118184[] = {{&S8515[31], 3}, {NULL}};
const Segment SGI118185[] = {{&S8515[77], 6}, {NULL}};
const Segment SGI118186[] = {{&S92182[34], 7}, {NULL}};
const Segment SGI118187[] = {{&S92182[27], 6}, {NULL}};
const Segment SGI118188[] = {{&S92182[94], 9}, {NULL}};
const Segment SGI118189[] = {{&S92182[67], 6}, {NULL}};
const Segment SGI118190[] = {{&S92196[45], 9}, {NULL}};
const Segment SGI118191[] = {{&S92196[30], 6}, {NULL}};
const Segment SGI118192[] = {{&S92196[31], 3}, {NULL}};
const Segment SGI118193[] = {{&S92196[80], 5}, {NULL}};
const Segment SGI118194[] = {{&S92196[138], 13}, {NULL}};
const Segment SGI118195[] = {{&S92210[44], 9}, {NULL}};
const Segment SGI118196[] = {{&S92210[44], 21}, {NULL}};
const Segment SGI118197[] = {{&S92210[30], 6}, {NULL}};
const Segment SGI118198[] = {{&S92210[31], 3}, {NULL}};
const Segment SGI118199[] = {{&S92210[79], 6}, {NULL}};
const Segment SGI118200[] = {{&S92210[138], 10}, {NULL}};
const Segment SGI118201[] = {{&S92214[44], 6}, {NULL}};
const Segment SGI118202[] = {{&S92214[44], 29}, {NULL}};
const Segment SGI118203[] = {{&S92214[30], 6}, {NULL}};
const Segment SGI118204[] = {{&S92214[87], 6}, {NULL}};
const Segment SGI118205[] = {{&S92214[114], 13}, {NULL}};
const Segment SGI118206[] = {{&S92228[44], 13}, {NULL}};
const Segment SGI118207[] = {{&S92228[44], 25}, {NULL}};
const Segment SGI118208[] = {{&S92228[30], 6}, {NULL}};
const Segment SGI118209[] = {{&S92228[31], 3}, {NULL}};
const Segment SGI118210[] = {{&S92228[83], 6}, {NULL}};
const Segment SGI118211[] = {{&S92228[142], 10}, {NULL}};
const Segment SGI118212[] = {{&S92230[44], 11}, {NULL}};
const Segment SGI118213[] = {{&S92230[44], 23}, {NULL}};
const Segment SGI118214[] = {{&S92230[30], 6}, {NULL}};
const Segment SGI118215[] = {{&S92230[31], 3}, {NULL}};
const Segment SGI118216[] = {{&S92230[81], 5}, {NULL}};
const Segment SGI118217[] = {{&S8516[44], 10}, {NULL}};
const Segment SGI118218[] = {{&S8516[44], 25}, {NULL}};
const Segment SGI118219[] = {{&S8516[30], 6}, {NULL}};
const Segment SGI118220[] = {{&S8516[31], 3}, {NULL}};
const Segment SGI118221[] = {{&S8516[83], 6}, {NULL}};
const Segment SGI118222[] = {{&S8517[38], 10}, {NULL}};
const Segment SGI118223[] = {{&S8517[31], 6}, {NULL}};
const Segment SGI118224[] = {{&S8521[44], 10}, {NULL}};
const Segment SGI118225[] = {{&S8521[44], 31}, {NULL}};
const Segment SGI118226[] = {{&S8521[30], 6}, {NULL}};
const Segment SGI118227[] = {{&S8521[31], 3}, {NULL}};
const Segment SGI118228[] = {{&S8521[89], 5}, {NULL}};
const Segment SGI118229[] = {{&S8522[42], 29}, {NULL}};
const Segment SGI118230[] = {{&S8522[30], 6}, {NULL}};
const Segment SGI118231[] = {{&S8522[31], 3}, {NULL}};
const Segment SGI118232[] = {{&S8522[97], 6}, {NULL}};
const Segment SGI118233[] = {{&S8524[44], 10}, {NULL}};
const Segment SGI118234[] = {{&S8524[44], 22}, {NULL}};
const Segment SGI118235[] = {{&S8524[30], 6}, {NULL}};
const Segment SGI118236[] = {{&S8524[31], 3}, {NULL}};
const Segment SGI118237[] = {{&S8524[80], 6}, {NULL}};
const Segment SGI118238[] = {{&S8526[42], 30}, {NULL}};
const Segment SGI118239[] = {{&S8526[30], 6}, {NULL}};
const Segment SGI118240[] = {{&S8526[31], 3}, {NULL}};
const Segment SGI118241[] = {{&S8526[98], 6}, {NULL}};
const Segment SGI118242[] = {{&S8527[42], 30}, {NULL}};
const Segment SGI118243[] = {{&S8527[30], 6}, {NULL}};
const Segment SGI118244[] = {{&S8527[31], 3}, {NULL}};
const Segment SGI118245[] = {{&S8527[98], 6}, {NULL}};
const Segment SGI118246[] = {{&S8528[38], 7}, {NULL}};
const Segment SGI118247[] = {{&S8528[31], 6}, {NULL}};
const Segment SGI118248[] = {{&S8531[44], 10}, {NULL}};
const Segment SGI118249[] = {{&S8531[44], 33}, {NULL}};
const Segment SGI118250[] = {{&S8531[30], 6}, {NULL}};
const Segment SGI118251[] = {{&S8531[31], 3}, {NULL}};
const Segment SGI118252[] = {{&S8531[91], 5}, {NULL}};
const Segment SGI118253[] = {{&S8532[44], 8}, {NULL}};
const Segment SGI118254[] = {{&S8532[44], 20}, {NULL}};
const Segment SGI118255[] = {{&S8532[30], 6}, {NULL}};
const Segment SGI118256[] = {{&S8532[31], 3}, {NULL}};
const Segment SGI118257[] = {{&S8532[78], 6}, {NULL}};
const Segment SGI118258[] = {{&S92335[44], 9}, {NULL}};
const Segment SGI118259[] = {{&S92335[44], 21}, {NULL}};
const Segment SGI118260[] = {{&S92335[30], 6}, {NULL}};
const Segment SGI118261[] = {{&S92335[31], 3}, {NULL}};
const Segment SGI118262[] = {{&S92335[79], 6}, {NULL}};
const Segment SGI118263[] = {{&S92354[34], 23}, {NULL}};
const Segment SGI118264[] = {{&S92354[27], 6}, {NULL}};
const Segment SGI118265[] = {{&S92354[110], 9}, {NULL}};
const Segment SGI118266[] = {{&S92354[83], 6}, {NULL}};
const Segment SGI118267[] = {{&S92400[44], 17}, {NULL}};
const Segment SGI118268[] = {{&S92400[44], 29}, {NULL}};
const Segment SGI118269[] = {{&S92400[30], 6}, {NULL}};
const Segment SGI118270[] = {{&S92400[31], 3}, {NULL}};
const Segment SGI118271[] = {{&S92400[87], 6}, {NULL}};
const Segment SGI118272[] = {{&S92411[34], 6}, {NULL}};
const Segment SGI118273[] = {{&S92411[27], 6}, {NULL}};
const Segment SGI118274[] = {{&S92411[93], 9}, {NULL}};
const Segment SGI118275[] = {{&S92411[66], 6}, {NULL}};
const Segment SGI118276[] = {{&S92417[44], 11}, {NULL}};
const Segment SGI118277[] = {{&S92417[44], 23}, {NULL}};
const Segment SGI118278[] = {{&S92417[30], 6}, {NULL}};
const Segment SGI118279[] = {{&S92417[31], 3}, {NULL}};
const Segment SGI118280[] = {{&S92417[81], 5}, {NULL}};
const Segment SGI118281[] = {{&S92417[118], 14}, {NULL}};
const Segment SGI118282[] = {{&S92430[34], 9}, {NULL}};
const Segment SGI118283[] = {{&S92430[27], 6}, {NULL}};
const Segment SGI118284[] = {{&S92430[96], 9}, {NULL}};
const Segment SGI118285[] = {{&S92430[69], 6}, {NULL}};
const Segment SGI118286[] = {{&S92432[34], 6}, {NULL}};
const Segment SGI118287[] = {{&S92432[27], 6}, {NULL}};
const Segment SGI118288[] = {{&S92432[93], 9}, {NULL}};
const Segment SGI118289[] = {{&S92432[66], 6}, {NULL}};
const Segment SGI118290[] = {{&S92459[34], 14}, {NULL}};
const Segment SGI118291[] = {{&S92459[27], 6}, {NULL}};
const Segment SGI118292[] = {{&S92459[101], 9}, {NULL}};
const Segment SGI118293[] = {{&S92459[74], 6}, {NULL}};
const Segment SGI118294[] = {{&S92459[142], 8}, {NULL}};
const Segment SGI118295[] = {{&S92512[44], 7}, {NULL}};
const Segment SGI118296[] = {{&S92512[44], 19}, {NULL}};
const Segment SGI118297[] = {{&S92512[30], 6}, {NULL}};
const Segment SGI118298[] = {{&S92512[31], 3}, {NULL}};
const Segment SGI118299[] = {{&S92512[77], 6}, {NULL}};
const Segment SGI118300[] = {{&S92514[44], 7}, {NULL}};
const Segment SGI118301[] = {{&S92514[44], 19}, {NULL}};
const Segment SGI118302[] = {{&S92514[30], 6}, {NULL}};
const Segment SGI118303[] = {{&S92514[31], 3}, {NULL}};
const Segment SGI118304[] = {{&S92514[77], 6}, {NULL}};
const Segment SGI118305[] = {{&S92538[59], 11}, {NULL}};
const Segment SGI118306[] = {{&S92538[59], 23}, {NULL}};
const Segment SGI118307[] = {{&S92538[45], 6}, {NULL}};
const Segment SGI118308[] = {{&S92538[46], 3}, {NULL}};
const Segment SGI118309[] = {{&S92538[96], 6}, {NULL}};
const Segment SGI118310[] = {{&S92538[0], 14}, {NULL}};
const Segment SGI118311[] = {{&S92540[34], 12}, {NULL}};
const Segment SGI118312[] = {{&S92540[27], 6}, {NULL}};
const Segment SGI118313[] = {{&S92540[99], 9}, {NULL}};
const Segment SGI118314[] = {{&S92540[72], 6}, {NULL}};
const Segment SGI118315[] = {{&S103365[44], 6}, {NULL}};
const Segment SGI118316[] = {{&S103365[44], 16}, {NULL}};
const Segment SGI118317[] = {{&S103365[30], 6}, {NULL}};
const Segment SGI118318[] = {{&S103365[31], 3}, {NULL}};
const Segment SGI118319[] = {{&S103365[74], 5}, {NULL}};
const Segment SGI118320[] = {{&S103365[0], 13}, {NULL}};
const Segment SGI118321[] = {{&S92549[44], 11}, {NULL}};
const Segment SGI118322[] = {{&S92549[44], 23}, {NULL}};
const Segment SGI118323[] = {{&S92549[30], 6}, {NULL}};
const Segment SGI118324[] = {{&S92549[31], 3}, {NULL}};
const Segment SGI118325[] = {{&S92549[81], 5}, {NULL}};
const Segment SGI118326[] = {{&S92549[118], 14}, {NULL}};
const Segment SGI118327[] = {{&S92551[45], 18}, {NULL}};
const Segment SGI118328[] = {{&S92551[30], 6}, {NULL}};
const Segment SGI118329[] = {{&S92551[31], 3}, {NULL}};
const Segment SGI118330[] = {{&S92551[89], 5}, {NULL}};
const Segment SGI118331[] = {{&S92551[147], 13}, {NULL}};
const Segment SGI118332[] = {{&S92557[59], 10}, {NULL}};
const Segment SGI118333[] = {{&S92557[59], 22}, {NULL}};
const Segment SGI118334[] = {{&S92557[45], 6}, {NULL}};
const Segment SGI118335[] = {{&S92557[46], 3}, {NULL}};
const Segment SGI118336[] = {{&S92557[95], 6}, {NULL}};
const Segment SGI118337[] = {{&S92557[0], 14}, {NULL}};
const Segment SGI118338[] = {{&S92559[45], 9}, {NULL}};
const Segment SGI118339[] = {{&S92559[30], 6}, {NULL}};
const Segment SGI118340[] = {{&S92559[31], 3}, {NULL}};
const Segment SGI118341[] = {{&S92559[80], 5}, {NULL}};
const Segment SGI118342[] = {{&S92559[138], 13}, {NULL}};
const Segment SGI118343[] = {{&S92567[34], 8}, {NULL}};
const Segment SGI118344[] = {{&S92567[27], 6}, {NULL}};
const Segment SGI118345[] = {{&S92567[95], 9}, {NULL}};
const Segment SGI118346[] = {{&S92567[68], 6}, {NULL}};
const Segment SGI118347[] = {{&S92569[34], 7}, {NULL}};
const Segment SGI118348[] = {{&S92569[27], 6}, {NULL}};
const Segment SGI118349[] = {{&S92569[94], 9}, {NULL}};
const Segment SGI118350[] = {{&S92569[67], 6}, {NULL}};
const Segment SGI118351[] = {{&S92590[34], 15}, {NULL}};
const Segment SGI118352[] = {{&S92590[27], 6}, {NULL}};
const Segment SGI118353[] = {{&S92590[102], 9}, {NULL}};
const Segment SGI118354[] = {{&S92590[75], 6}, {NULL}};
const Segment SGI118355[] = {{&S92590[136], 8}, {NULL}};
const Segment SGI118356[] = {{&S92598[44], 6}, {NULL}};
const Segment SGI118357[] = {{&S92598[44], 18}, {NULL}};
const Segment SGI118358[] = {{&S92598[30], 6}, {NULL}};
const Segment SGI118359[] = {{&S92598[76], 6}, {NULL}};
const Segment SGI118360[] = {{&S92598[103], 13}, {NULL}};
const Segment SGI118361[] = {{&S92618[44], 9}, {NULL}};
const Segment SGI118362[] = {{&S92618[44], 21}, {NULL}};
const Segment SGI118363[] = {{&S92618[30], 6}, {NULL}};
const Segment SGI118364[] = {{&S92618[79], 6}, {NULL}};
const Segment SGI118365[] = {{&S92618[106], 13}, {NULL}};
const Segment SGI118366[] = {{&S92621[44], 9}, {NULL}};
const Segment SGI118367[] = {{&S92621[44], 21}, {NULL}};
const Segment SGI118368[] = {{&S92621[30], 6}, {NULL}};
const Segment SGI118369[] = {{&S92621[79], 6}, {NULL}};
const Segment SGI118370[] = {{&S92621[106], 13}, {NULL}};
const Segment SGI118371[] = {{&S92701[44], 8}, {NULL}};
const Segment SGI118372[] = {{&S92701[44], 20}, {NULL}};
const Segment SGI118373[] = {{&S92701[30], 6}, {NULL}};
const Segment SGI118374[] = {{&S92701[81], 6}, {NULL}};
const Segment SGI118375[] = {{&S92702[45], 21}, {NULL}};
const Segment SGI118376[] = {{&S92702[30], 6}, {NULL}};
const Segment SGI118377[] = {{&S92702[31], 3}, {NULL}};
const Segment SGI118378[] = {{&S92702[92], 5}, {NULL}};
const Segment SGI118379[] = {{&S92702[150], 13}, {NULL}};
const Segment SGI118380[] = {{&S92703[45], 18}, {NULL}};
const Segment SGI118381[] = {{&S92703[30], 6}, {NULL}};
const Segment SGI118382[] = {{&S92703[31], 3}, {NULL}};
const Segment SGI118383[] = {{&S92703[89], 5}, {NULL}};
const Segment SGI118384[] = {{&S92703[147], 13}, {NULL}};
const Segment SGI118385[] = {{&S92707[44], 8}, {NULL}};
const Segment SGI118386[] = {{&S92707[44], 20}, {NULL}};
const Segment SGI118387[] = {{&S92707[30], 6}, {NULL}};
const Segment SGI118388[] = {{&S92707[31], 3}, {NULL}};
const Segment SGI118389[] = {{&S92707[78], 6}, {NULL}};
const Segment SGI118390[] = {{&S92736[44], 12}, {NULL}};
const Segment SGI118391[] = {{&S92736[44], 24}, {NULL}};
const Segment SGI118392[] = {{&S92736[30], 6}, {NULL}};
const Segment SGI118393[] = {{&S92736[31], 3}, {NULL}};
const Segment SGI118394[] = {{&S92736[82], 6}, {NULL}};
const Segment SGI118395[] = {{&S92736[142], 11}, {NULL}};
const Segment SGI118396[] = {{&S92739[34], 8}, {NULL}};
const Segment SGI118397[] = {{&S92739[27], 6}, {NULL}};
const Segment SGI118398[] = {{&S92739[95], 9}, {NULL}};
const Segment SGI118399[] = {{&S92739[68], 6}, {NULL}};
const Segment SGI118400[] = {{&S92753[34], 10}, {NULL}};
const Segment SGI118401[] = {{&S92753[27], 6}, {NULL}};
const Segment SGI118402[] = {{&S92753[97], 9}, {NULL}};
const Segment SGI118403[] = {{&S92753[70], 6}, {NULL}};
const Segment SGI118404[] = {{&S92763[34], 7}, {NULL}};
const Segment SGI118405[] = {{&S92763[27], 6}, {NULL}};
const Segment SGI118406[] = {{&S92763[94], 9}, {NULL}};
const Segment SGI118407[] = {{&S92763[67], 6}, {NULL}};
const Segment SGI118408[] = {{&S92772[34], 14}, {NULL}};
const Segment SGI118409[] = {{&S92772[27], 6}, {NULL}};
const Segment SGI118410[] = {{&S92772[101], 9}, {NULL}};
const Segment SGI118411[] = {{&S92772[74], 6}, {NULL}};
const Segment SGI118412[] = {{&S92772[142], 8}, {NULL}};
const Segment SGI118413[] = {{&S92793[44], 11}, {NULL}};
const Segment SGI118414[] = {{&S92793[44], 23}, {NULL}};
const Segment SGI118415[] = {{&S92793[30], 6}, {NULL}};
const Segment SGI118416[] = {{&S92793[31], 3}, {NULL}};
const Segment SGI118417[] = {{&S92793[81], 6}, {NULL}};
const Segment SGI118418[] = {{&S92793[141], 7}, {NULL}};
const Segment SGI118419[] = {{&S92796[52], 14}, {NULL}};
const Segment SGI118420[] = {{&S92796[52], 26}, {NULL}};
const Segment SGI118421[] = {{&S92796[38], 6}, {NULL}};
const Segment SGI118422[] = {{&S92796[92], 5}, {NULL}};
const Segment SGI118423[] = {{&S92796[0], 14}, {NULL}};
const Segment SGI118424[] = {{&S92800[34], 23}, {NULL}};
const Segment SGI118425[] = {{&S92800[27], 6}, {NULL}};
const Segment SGI118426[] = {{&S92800[110], 9}, {NULL}};
const Segment SGI118427[] = {{&S92800[83], 6}, {NULL}};
const Segment SGI118428[] = {{&S92859[11], 1}, {NULL}};
const Segment SGI118429[] = {{&S92859[44], 13}, {NULL}};
const Segment SGI118430[] = {{&S92859[30], 6}, {NULL}};
const Segment SGI118431[] = {{&S92859[102], 2}, {NULL}};
const Segment SGI118432[] = {{&S92859[71], 5}, {NULL}};
const Segment SGI118433[] = {{&S92866[44], 10}, {NULL}};
const Segment SGI118434[] = {{&S92866[44], 22}, {NULL}};
const Segment SGI118435[] = {{&S92866[30], 6}, {NULL}};
const Segment SGI118436[] = {{&S92866[31], 3}, {NULL}};
const Segment SGI118437[] = {{&S92866[87], 6}, {NULL}};
const Segment SGI118438[] = {{&S92866[66], 8}, {NULL}};
const Segment SGI118439[] = {{&S92906[34], 7}, {NULL}};
const Segment SGI118440[] = {{&S92906[27], 6}, {NULL}};
const Segment SGI118441[] = {{&S92906[94], 9}, {NULL}};
const Segment SGI118442[] = {{&S92906[67], 6}, {NULL}};
const Segment SGI118443[] = {{&S92916[34], 22}, {NULL}};
const Segment SGI118444[] = {{&S92916[27], 6}, {NULL}};
const Segment SGI118445[] = {{&S92916[109], 9}, {NULL}};
const Segment SGI118446[] = {{&S92916[82], 6}, {NULL}};
const Segment SGI118447[] = {{&S92922[44], 9}, {NULL}};
const Segment SGI118448[] = {{&S92922[44], 21}, {NULL}};
const Segment SGI118449[] = {{&S92922[30], 6}, {NULL}};
const Segment SGI118450[] = {{&S92922[31], 3}, {NULL}};
const Segment SGI118451[] = {{&S92922[79], 6}, {NULL}};
const Segment SGI118452[] = {{&S92923[44], 6}, {NULL}};
const Segment SGI118453[] = {{&S92923[44], 18}, {NULL}};
const Segment SGI118454[] = {{&S92923[30], 6}, {NULL}};
const Segment SGI118455[] = {{&S92923[31], 3}, {NULL}};
const Segment SGI118456[] = {{&S92923[76], 5}, {NULL}};
const Segment SGI118457[] = {{&S92923[113], 14}, {NULL}};
const Segment SGI118458[] = {{&S92925[34], 10}, {NULL}};
const Segment SGI118459[] = {{&S92925[27], 6}, {NULL}};
const Segment SGI118460[] = {{&S92925[97], 9}, {NULL}};
const Segment SGI118461[] = {{&S92925[70], 6}, {NULL}};
const Segment SGI118462[] = {{&S92926[34], 17}, {NULL}};
const Segment SGI118463[] = {{&S92926[27], 6}, {NULL}};
const Segment SGI118464[] = {{&S92926[104], 9}, {NULL}};
const Segment SGI118465[] = {{&S92926[77], 6}, {NULL}};
const Segment SGI118466[] = {{&S92938[34], 6}, {NULL}};
const Segment SGI118467[] = {{&S92938[27], 6}, {NULL}};
const Segment SGI118468[] = {{&S92938[93], 9}, {NULL}};
const Segment SGI118469[] = {{&S92938[66], 6}, {NULL}};
const Segment SGI118470[] = {{&S92944[34], 7}, {NULL}};
const Segment SGI118471[] = {{&S92944[27], 6}, {NULL}};
const Segment SGI118472[] = {{&S92944[95], 9}, {NULL}};
const Segment SGI118473[] = {{&S92944[68], 6}, {NULL}};
const Segment SGI118474[] = {{&S92947[44], 8}, {NULL}};
const Segment SGI118475[] = {{&S92947[44], 37}, {NULL}};
const Segment SGI118476[] = {{&S92947[30], 6}, {NULL}};
const Segment SGI118477[] = {{&S92947[31], 3}, {NULL}};
const Segment SGI118478[] = {{&S92947[95], 6}, {NULL}};
const Segment SGI118479[] = {{&S92951[44], 9}, {NULL}};
const Segment SGI118480[] = {{&S92951[44], 21}, {NULL}};
const Segment SGI118481[] = {{&S92951[30], 6}, {NULL}};
const Segment SGI118482[] = {{&S92951[31], 3}, {NULL}};
const Segment SGI118483[] = {{&S92951[79], 6}, {NULL}};
const Segment SGI118484[] = {{&S92956[34], 9}, {NULL}};
const Segment SGI118485[] = {{&S92956[27], 6}, {NULL}};
const Segment SGI118486[] = {{&S92956[96], 9}, {NULL}};
const Segment SGI118487[] = {{&S92956[69], 6}, {NULL}};
const Segment SGI118488[] = {{&S92960[34], 8}, {NULL}};
const Segment SGI118489[] = {{&S92960[27], 6}, {NULL}};
const Segment SGI118490[] = {{&S92960[95], 9}, {NULL}};
const Segment SGI118491[] = {{&S92960[68], 6}, {NULL}};
const Segment SGI118492[] = {{&S92961[34], 11}, {NULL}};
const Segment SGI118493[] = {{&S92961[27], 6}, {NULL}};
const Segment SGI118494[] = {{&S92961[98], 9}, {NULL}};
const Segment SGI118495[] = {{&S92961[71], 6}, {NULL}};
const Segment SGI118496[] = {{&S92964[34], 9}, {NULL}};
const Segment SGI118497[] = {{&S92964[27], 6}, {NULL}};
const Segment SGI118498[] = {{&S92964[96], 9}, {NULL}};
const Segment SGI118499[] = {{&S92964[69], 6}, {NULL}};
const Segment SGI118500[] = {{&S92983[34], 6}, {NULL}};
const Segment SGI118501[] = {{&S92983[27], 6}, {NULL}};
const Segment SGI118502[] = {{&S92983[93], 9}, {NULL}};
const Segment SGI118503[] = {{&S92983[66], 6}, {NULL}};
const Segment SGI118504[] = {{&S92994[44], 10}, {NULL}};
const Segment SGI118505[] = {{&S92994[44], 22}, {NULL}};
const Segment SGI118506[] = {{&S92994[30], 6}, {NULL}};
const Segment SGI118507[] = {{&S92994[31], 3}, {NULL}};
const Segment SGI118508[] = {{&S92994[80], 6}, {NULL}};
const Segment SGI118509[] = {{&S92995[44], 10}, {NULL}};
const Segment SGI118510[] = {{&S92995[44], 21}, {NULL}};
const Segment SGI118511[] = {{&S92995[30], 6}, {NULL}};
const Segment SGI118512[] = {{&S92995[31], 3}, {NULL}};
const Segment SGI118513[] = {{&S92995[79], 6}, {NULL}};
const Segment SGI118514[] = {{&S93012[44], 9}, {NULL}};
const Segment SGI118515[] = {{&S93012[44], 21}, {NULL}};
const Segment SGI118516[] = {{&S93012[30], 6}, {NULL}};
const Segment SGI118517[] = {{&S93012[31], 3}, {NULL}};
const Segment SGI118518[] = {{&S93012[79], 6}, {NULL}};
const Segment SGI118519[] = {{&S93012[139], 7}, {NULL}};
const Segment SGI118520[] = {{&S93042[44], 12}, {NULL}};
const Segment SGI118521[] = {{&S93042[44], 33}, {NULL}};
const Segment SGI118522[] = {{&S93042[30], 6}, {NULL}};
const Segment SGI118523[] = {{&S93042[91], 6}, {NULL}};
const Segment SGI118524[] = {{&S93042[118], 13}, {NULL}};
const Segment SGI118525[] = {{&S93085[34], 13}, {NULL}};
const Segment SGI118526[] = {{&S93085[27], 6}, {NULL}};
const Segment SGI118527[] = {{&S93085[100], 9}, {NULL}};
const Segment SGI118528[] = {{&S93085[73], 6}, {NULL}};
const Segment SGI118529[] = {{&S93093[57], 10}, {NULL}};
const Segment SGI118530[] = {{&S93093[57], 22}, {NULL}};
const Segment SGI118531[] = {{&S93093[43], 6}, {NULL}};
const Segment SGI118532[] = {{&S93093[44], 3}, {NULL}};
const Segment SGI118533[] = {{&S93093[93], 6}, {NULL}};
const Segment SGI118534[] = {{&S93093[0], 12}, {NULL}};
const Segment SGI118535[] = {{&S93104[44], 12}, {NULL}};
const Segment SGI118536[] = {{&S93104[44], 24}, {NULL}};
const Segment SGI118537[] = {{&S93104[30], 6}, {NULL}};
const Segment SGI118538[] = {{&S93104[31], 3}, {NULL}};
const Segment SGI118539[] = {{&S93104[82], 6}, {NULL}};
const Segment SGI118540[] = {{&S93116[44], 8}, {NULL}};
const Segment SGI118541[] = {{&S93116[44], 20}, {NULL}};
const Segment SGI118542[] = {{&S93116[30], 6}, {NULL}};
const Segment SGI118543[] = {{&S93116[31], 3}, {NULL}};
const Segment SGI118544[] = {{&S93116[78], 6}, {NULL}};
const Segment SGI118545[] = {{&S93117[44], 39}, {NULL}};
const Segment SGI118546[] = {{&S93117[30], 6}, {NULL}};
const Segment SGI118547[] = {{&S93117[31], 3}, {NULL}};
const Segment SGI118548[] = {{&S93117[117], 6}, {NULL}};
const Segment SGI118549[] = {{&S93118[44], 14}, {NULL}};
const Segment SGI118550[] = {{&S93118[44], 32}, {NULL}};
const Segment SGI118551[] = {{&S93118[30], 6}, {NULL}};
const Segment SGI118552[] = {{&S93118[31], 3}, {NULL}};
const Segment SGI118553[] = {{&S93118[90], 6}, {NULL}};
const Segment SGI118554[] = {{&S93122[34], 12}, {NULL}};
const Segment SGI118555[] = {{&S93122[27], 6}, {NULL}};
const Segment SGI118556[] = {{&S93122[99], 9}, {NULL}};
const Segment SGI118557[] = {{&S93122[72], 6}, {NULL}};
const Segment SGI118558[] = {{&S93143[34], 8}, {NULL}};
const Segment SGI118559[] = {{&S93143[27], 6}, {NULL}};
const Segment SGI118560[] = {{&S93143[101], 9}, {NULL}};
const Segment SGI118561[] = {{&S93143[74], 6}, {NULL}};
const Segment SGI118562[] = {{&S93145[34], 9}, {NULL}};
const Segment SGI118563[] = {{&S93145[27], 6}, {NULL}};
const Segment SGI118564[] = {{&S93145[96], 9}, {NULL}};
const Segment SGI118565[] = {{&S93145[69], 6}, {NULL}};
const Segment SGI118566[] = {{&S93152[34], 8}, {NULL}};
const Segment SGI118567[] = {{&S93152[27], 6}, {NULL}};
const Segment SGI118568[] = {{&S93152[101], 9}, {NULL}};
const Segment SGI118569[] = {{&S93152[74], 6}, {NULL}};
const Segment SGI118570[] = {{&S93163[34], 6}, {NULL}};
const Segment SGI118571[] = {{&S93163[27], 6}, {NULL}};
const Segment SGI118572[] = {{&S93163[93], 9}, {NULL}};
const Segment SGI118573[] = {{&S93163[66], 6}, {NULL}};
const Segment SGI118574[] = {{&S93164[34], 19}, {NULL}};
const Segment SGI118575[] = {{&S93164[27], 6}, {NULL}};
const Segment SGI118576[] = {{&S93164[106], 9}, {NULL}};
const Segment SGI118577[] = {{&S93164[79], 6}, {NULL}};
const Segment SGI118578[] = {{&S93164[140], 8}, {NULL}};
const Segment SGI118579[] = {{&S93244[34], 11}, {NULL}};
const Segment SGI118580[] = {{&S93244[27], 6}, {NULL}};
const Segment SGI118581[] = {{&S93244[98], 9}, {NULL}};
const Segment SGI118582[] = {{&S93244[71], 6}, {NULL}};
const Segment SGI118583[] = {{&S93244[139], 8}, {NULL}};
const Segment SGI118584[] = {{&S93330[59], 10}, {NULL}};
const Segment SGI118585[] = {{&S93330[59], 22}, {NULL}};
const Segment SGI118586[] = {{&S93330[45], 6}, {NULL}};
const Segment SGI118587[] = {{&S93330[46], 3}, {NULL}};
const Segment SGI118588[] = {{&S93330[95], 6}, {NULL}};
const Segment SGI118589[] = {{&S93330[0], 14}, {NULL}};
const Segment SGI118590[] = {{&S93355[44], 14}, {NULL}};
const Segment SGI118591[] = {{&S93355[44], 26}, {NULL}};
const Segment SGI118592[] = {{&S93355[30], 6}, {NULL}};
const Segment SGI118593[] = {{&S93355[31], 3}, {NULL}};
const Segment SGI118594[] = {{&S93355[84], 6}, {NULL}};
const Segment SGI118595[] = {{&S93360[44], 7}, {NULL}};
const Segment SGI118596[] = {{&S93360[44], 18}, {NULL}};
const Segment SGI118597[] = {{&S93360[30], 6}, {NULL}};
const Segment SGI118598[] = {{&S93360[31], 3}, {NULL}};
const Segment SGI118599[] = {{&S93360[76], 6}, {NULL}};
const Segment SGI118600[] = {{&S93363[37], 8}, {NULL}};
const Segment SGI118601[] = {{&S93363[30], 6}, {NULL}};
const Segment SGI118602[] = {{&S93363[31], 3}, {NULL}};
const Segment SGI118603[] = {{&S93363[81], 5}, {NULL}};
const Segment SGI118604[] = {{&S93363[139], 13}, {NULL}};
const Segment SGI118605[] = {{&S93369[44], 10}, {NULL}};
const Segment SGI118606[] = {{&S93369[44], 51}, {NULL}};
const Segment SGI118607[] = {{&S93369[30], 6}, {NULL}};
const Segment SGI118608[] = {{&S93369[31], 3}, {NULL}};
const Segment SGI118609[] = {{&S93369[109], 6}, {NULL}};
const Segment SGI118610[] = {{&S93380[34], 8}, {NULL}};
const Segment SGI118611[] = {{&S93380[27], 6}, {NULL}};
const Segment SGI118612[] = {{&S93380[95], 9}, {NULL}};
const Segment SGI118613[] = {{&S93380[68], 6}, {NULL}};
const Segment SGI118614[] = {{&S93403[34], 18}, {NULL}};
const Segment SGI118615[] = {{&S93403[27], 6}, {NULL}};
const Segment SGI118616[] = {{&S93403[105], 9}, {NULL}};
const Segment SGI118617[] = {{&S93403[78], 6}, {NULL}};
const Segment SGI118618[] = {{&S93403[139], 8}, {NULL}};
const Segment SGI118619[] = {{&S93412[34], 6}, {NULL}};
const Segment SGI118620[] = {{&S93412[27], 6}, {NULL}};
const Segment SGI118621[] = {{&S93412[93], 9}, {NULL}};
const Segment SGI118622[] = {{&S93412[66], 6}, {NULL}};
const Segment SGI118623[] = {{&S93420[34], 7}, {NULL}};
const Segment SGI118624[] = {{&S93420[27], 6}, {NULL}};
const Segment SGI118625[] = {{&S93420[94], 9}, {NULL}};
const Segment SGI118626[] = {{&S93420[67], 6}, {NULL}};
const Segment SGI118627[] = {{&S93449[44], 10}, {NULL}};
const Segment SGI118628[] = {{&S93449[44], 22}, {NULL}};
const Segment SGI118629[] = {{&S93449[30], 6}, {NULL}};
const Segment SGI118630[] = {{&S93449[31], 3}, {NULL}};
const Segment SGI118631[] = {{&S93449[80], 6}, {NULL}};
const Segment SGI118632[] = {{&S93449[140], 13}, {NULL}};
const Segment SGI118633[] = {{&S93492[44], 41}, {NULL}};
const Segment SGI118634[] = {{&S93492[44], 53}, {NULL}};
const Segment SGI118635[] = {{&S93492[30], 6}, {NULL}};
const Segment SGI118636[] = {{&S93492[111], 6}, {NULL}};
const Segment SGI118637[] = {{&S93492[138], 13}, {NULL}};
const Segment SGI118638[] = {{&S93567[44], 9}, {NULL}};
const Segment SGI118639[] = {{&S93567[44], 21}, {NULL}};
const Segment SGI118640[] = {{&S93567[30], 6}, {NULL}};
const Segment SGI118641[] = {{&S93567[31], 3}, {NULL}};
const Segment SGI118642[] = {{&S93567[79], 6}, {NULL}};
const Segment SGI118643[] = {{&S93600[34], 6}, {NULL}};
const Segment SGI118644[] = {{&S93600[27], 6}, {NULL}};
const Segment SGI118645[] = {{&S93600[93], 9}, {NULL}};
const Segment SGI118646[] = {{&S93600[66], 6}, {NULL}};
const Segment SGI118647[] = {{&S93641[34], 9}, {NULL}};
const Segment SGI118648[] = {{&S93641[27], 6}, {NULL}};
const Segment SGI118649[] = {{&S93641[96], 9}, {NULL}};
const Segment SGI118650[] = {{&S93641[69], 6}, {NULL}};
const Segment SGI118651[] = {{&S93660[44], 19}, {NULL}};
const Segment SGI118652[] = {{&S93660[44], 31}, {NULL}};
const Segment SGI118653[] = {{&S93660[30], 6}, {NULL}};
const Segment SGI118654[] = {{&S93660[31], 3}, {NULL}};
const Segment SGI118655[] = {{&S93660[89], 6}, {NULL}};
const Segment SGI118656[] = {{&S93660[143], 12}, {NULL}};
const Segment SGI118657[] = {{&S93683[34], 16}, {NULL}};
const Segment SGI118658[] = {{&S93683[27], 6}, {NULL}};
const Segment SGI118659[] = {{&S93683[103], 9}, {NULL}};
const Segment SGI118660[] = {{&S93683[76], 6}, {NULL}};
const Segment SGI118661[] = {{&S93683[144], 8}, {NULL}};
const Segment SGI118662[] = {{&S93690[34], 9}, {NULL}};
const Segment SGI118663[] = {{&S93690[27], 6}, {NULL}};
const Segment SGI118664[] = {{&S93690[96], 9}, {NULL}};
const Segment SGI118665[] = {{&S93690[69], 6}, {NULL}};
const Segment SGI118666[] = {{&S93791[44], 7}, {NULL}};
const Segment SGI118667[] = {{&S93791[44], 19}, {NULL}};
const Segment SGI118668[] = {{&S93791[30], 6}, {NULL}};
const Segment SGI118669[] = {{&S93791[31], 3}, {NULL}};
const Segment SGI118670[] = {{&S93791[77], 6}, {NULL}};
const Segment SGI118671[] = {{&S93803[44], 35}, {NULL}};
const Segment SGI118672[] = {{&S93803[44], 47}, {NULL}};
const Segment SGI118673[] = {{&S93803[30], 6}, {NULL}};
const Segment SGI118674[] = {{&S93803[31], 3}, {NULL}};
const Segment SGI118675[] = {{&S93803[112], 6}, {NULL}};
const Segment SGI118676[] = {{&S93803[91], 8}, {NULL}};
const Segment SGI118677[] = {{&S93804[44], 35}, {NULL}};
const Segment SGI118678[] = {{&S93804[44], 47}, {NULL}};
const Segment SGI118679[] = {{&S93804[30], 6}, {NULL}};
const Segment SGI118680[] = {{&S93804[31], 3}, {NULL}};
const Segment SGI118681[] = {{&S93804[105], 6}, {NULL}};
const Segment SGI118682[] = {{&S93804[165], 7}, {NULL}};
const Segment SGI118683[] = {{&S93813[34], 11}, {NULL}};
const Segment SGI118684[] = {{&S93813[27], 6}, {NULL}};
const Segment SGI118685[] = {{&S93813[98], 9}, {NULL}};
const Segment SGI118686[] = {{&S93813[71], 6}, {NULL}};
const Segment SGI118687[] = {{&S93814[34], 19}, {NULL}};
const Segment SGI118688[] = {{&S93814[27], 6}, {NULL}};
const Segment SGI118689[] = {{&S93814[106], 9}, {NULL}};
const Segment SGI118690[] = {{&S93814[79], 6}, {NULL}};
const Segment SGI118691[] = {{&S93816[44], 12}, {NULL}};
const Segment SGI118692[] = {{&S93816[44], 24}, {NULL}};
const Segment SGI118693[] = {{&S93816[30], 6}, {NULL}};
const Segment SGI118694[] = {{&S93816[31], 3}, {NULL}};
const Segment SGI118695[] = {{&S93816[82], 6}, {NULL}};
const Segment SGI118696[] = {{&S93816[141], 10}, {NULL}};
const Segment SGI118697[] = {{&S3031[42], 7}, {NULL}};
const Segment SGI118698[] = {{&S3031[42], 19}, {NULL}};
const Segment SGI118699[] = {{&S3031[30], 4}, {NULL}};
const Segment SGI118700[] = {{&S3031[110], 3}, {NULL}};
const Segment SGI118701[] = {{&S3031[75], 6}, {NULL}};
const Segment SGI118702[] = {{&S93817[34], 4}, {NULL}};
const Segment SGI118703[] = {{&S93817[27], 6}, {NULL}};
const Segment SGI118704[] = {{&S93817[91], 9}, {NULL}};
const Segment SGI118705[] = {{&S93817[64], 6}, {NULL}};
const Segment SGI118706[] = {{&S93823[45], 14}, {NULL}};
const Segment SGI118707[] = {{&S93823[30], 6}, {NULL}};
const Segment SGI118708[] = {{&S93823[31], 3}, {NULL}};
const Segment SGI118709[] = {{&S93823[85], 5}, {NULL}};
const Segment SGI118710[] = {{&S93823[143], 13}, {NULL}};
const Segment SGI118711[] = {{&S93824[34], 7}, {NULL}};
const Segment SGI118712[] = {{&S93824[27], 6}, {NULL}};
const Segment SGI118713[] = {{&S93824[95], 9}, {NULL}};
const Segment SGI118714[] = {{&S93824[68], 6}, {NULL}};
const Segment SGI118715[] = {{&S93857[44], 11}, {NULL}};
const Segment SGI118716[] = {{&S93857[44], 23}, {NULL}};
const Segment SGI118717[] = {{&S93857[30], 6}, {NULL}};
const Segment SGI118718[] = {{&S93857[81], 6}, {NULL}};
const Segment SGI118719[] = {{&S93857[108], 13}, {NULL}};
const Segment SGI118720[] = {{&S93858[34], 8}, {NULL}};
const Segment SGI118721[] = {{&S93858[27], 6}, {NULL}};
const Segment SGI118722[] = {{&S93858[95], 9}, {NULL}};
const Segment SGI118723[] = {{&S93858[68], 6}, {NULL}};
const Segment SGI118724[] = {{&S93873[45], 5}, {NULL}};
const Segment SGI118725[] = {{&S93873[30], 6}, {NULL}};
const Segment SGI118726[] = {{&S93873[31], 3}, {NULL}};
const Segment SGI118727[] = {{&S93873[76], 5}, {NULL}};
const Segment SGI118728[] = {{&S93873[134], 13}, {NULL}};
const Segment SGI118729[] = {{&S93879[34], 9}, {NULL}};
const Segment SGI118730[] = {{&S93879[27], 6}, {NULL}};
const Segment SGI118731[] = {{&S93879[96], 9}, {NULL}};
const Segment SGI118732[] = {{&S93879[69], 6}, {NULL}};
const Segment SGI118733[] = {{&S93892[34], 8}, {NULL}};
const Segment SGI118734[] = {{&S93892[27], 6}, {NULL}};
const Segment SGI118735[] = {{&S93892[95], 9}, {NULL}};
const Segment SGI118736[] = {{&S93892[68], 6}, {NULL}};
const Segment SGI118737[] = {{&S93908[45], 9}, {NULL}};
const Segment SGI118738[] = {{&S93908[30], 6}, {NULL}};
const Segment SGI118739[] = {{&S93908[31], 3}, {NULL}};
const Segment SGI118740[] = {{&S93908[80], 5}, {NULL}};
const Segment SGI118741[] = {{&S93908[138], 13}, {NULL}};
const Segment SGI118742[] = {{&S93910[44], 6}, {NULL}};
const Segment SGI118743[] = {{&S93910[44], 18}, {NULL}};
const Segment SGI118744[] = {{&S93910[30], 6}, {NULL}};
const Segment SGI118745[] = {{&S93910[31], 3}, {NULL}};
const Segment SGI118746[] = {{&S93910[76], 5}, {NULL}};
const Segment SGI118747[] = {{&S93910[113], 14}, {NULL}};
const Segment SGI118748[] = {{&S104827[44], 6}, {NULL}};
const Segment SGI118749[] = {{&S104827[44], 16}, {NULL}};
const Segment SGI118750[] = {{&S104827[30], 6}, {NULL}};
const Segment SGI118751[] = {{&S104827[31], 3}, {NULL}};
const Segment SGI118752[] = {{&S104827[74], 5}, {NULL}};
const Segment SGI118753[] = {{&S104827[0], 13}, {NULL}};
const Segment SGI118754[] = {{&S93959[34], 13}, {NULL}};
const Segment SGI118755[] = {{&S93959[27], 6}, {NULL}};
const Segment SGI118756[] = {{&S93959[100], 9}, {NULL}};
const Segment SGI118757[] = {{&S93959[73], 6}, {NULL}};
const Segment SGI118758[] = {{&S93989[44], 8}, {NULL}};
const Segment SGI118759[] = {{&S93989[44], 18}, {NULL}};
const Segment SGI118760[] = {{&S93989[30], 6}, {NULL}};
const Segment SGI118761[] = {{&S93989[31], 3}, {NULL}};
const Segment SGI118762[] = {{&S93989[76], 5}, {NULL}};
const Segment SGI118763[] = {{&S94015[44], 9}, {NULL}};
const Segment SGI118764[] = {{&S94015[44], 21}, {NULL}};
const Segment SGI118765[] = {{&S94015[30], 6}, {NULL}};
const Segment SGI118766[] = {{&S94015[31], 3}, {NULL}};
const Segment SGI118767[] = {{&S94015[79], 6}, {NULL}};
const Segment SGI118768[] = {{&S94044[57], 11}, {NULL}};
const Segment SGI118769[] = {{&S94044[57], 23}, {NULL}};
const Segment SGI118770[] = {{&S94044[43], 6}, {NULL}};
const Segment SGI118771[] = {{&S94044[44], 3}, {NULL}};
const Segment SGI118772[] = {{&S94044[94], 6}, {NULL}};
const Segment SGI118773[] = {{&S94044[0], 12}, {NULL}};
const Segment SGI118774[] = {{&S94067[45], 11}, {NULL}};
const Segment SGI118775[] = {{&S94067[30], 6}, {NULL}};
const Segment SGI118776[] = {{&S94067[31], 3}, {NULL}};
const Segment SGI118777[] = {{&S94067[82], 5}, {NULL}};
const Segment SGI118778[] = {{&S94067[140], 13}, {NULL}};
const Segment SGI118779[] = {{&S94079[44], 10}, {NULL}};
const Segment SGI118780[] = {{&S94079[44], 22}, {NULL}};
const Segment SGI118781[] = {{&S94079[30], 6}, {NULL}};
const Segment SGI118782[] = {{&S94079[80], 6}, {NULL}};
const Segment SGI118783[] = {{&S94079[107], 13}, {NULL}};
const Segment SGI118784[] = {{&S94093[34], 11}, {NULL}};
const Segment SGI118785[] = {{&S94093[27], 6}, {NULL}};
const Segment SGI118786[] = {{&S94093[98], 9}, {NULL}};
const Segment SGI118787[] = {{&S94093[71], 6}, {NULL}};
const Segment SGI118788[] = {{&S94116[34], 10}, {NULL}};
const Segment SGI118789[] = {{&S94116[27], 6}, {NULL}};
const Segment SGI118790[] = {{&S94116[97], 9}, {NULL}};
const Segment SGI118791[] = {{&S94116[70], 6}, {NULL}};
const Segment SGI118792[] = {{&S94171[34], 7}, {NULL}};
const Segment SGI118793[] = {{&S94171[27], 6}, {NULL}};
const Segment SGI118794[] = {{&S94171[95], 9}, {NULL}};
const Segment SGI118795[] = {{&S94171[68], 6}, {NULL}};
const Segment SGI118796[] = {{&S94178[34], 41}, {NULL}};
const Segment SGI118797[] = {{&S94178[27], 6}, {NULL}};
const Segment SGI118798[] = {{&S94178[128], 9}, {NULL}};
const Segment SGI118799[] = {{&S94178[101], 6}, {NULL}};
const Segment SGI118800[] = {{&S94178[169], 8}, {NULL}};
const Segment SGI118801[] = {{&S94181[34], 12}, {NULL}};
const Segment SGI118802[] = {{&S94181[27], 6}, {NULL}};
const Segment SGI118803[] = {{&S94181[99], 9}, {NULL}};
const Segment SGI118804[] = {{&S94181[72], 6}, {NULL}};
const Segment SGI118805[] = {{&S94252[34], 10}, {NULL}};
const Segment SGI118806[] = {{&S94252[27], 6}, {NULL}};
const Segment SGI118807[] = {{&S94252[106], 9}, {NULL}};
const Segment SGI118808[] = {{&S94252[79], 6}, {NULL}};
const Segment SGI118809[] = {{&S94253[34], 10}, {NULL}};
const Segment SGI118810[] = {{&S94253[27], 6}, {NULL}};
const Segment SGI118811[] = {{&S94253[97], 9}, {NULL}};
const Segment SGI118812[] = {{&S94253[70], 6}, {NULL}};
const Segment SGI118813[] = {{&S94293[34], 16}, {NULL}};
const Segment SGI118814[] = {{&S94293[27], 6}, {NULL}};
const Segment SGI118815[] = {{&S94293[103], 9}, {NULL}};
const Segment SGI118816[] = {{&S94293[76], 6}, {NULL}};
const Segment SGI118817[] = {{&S94296[34], 8}, {NULL}};
const Segment SGI118818[] = {{&S94296[27], 6}, {NULL}};
const Segment SGI118819[] = {{&S94296[95], 9}, {NULL}};
const Segment SGI118820[] = {{&S94296[68], 6}, {NULL}};
const Segment SGI118821[] = {{&S94300[34], 9}, {NULL}};
const Segment SGI118822[] = {{&S94300[27], 6}, {NULL}};
const Segment SGI118823[] = {{&S94300[96], 9}, {NULL}};
const Segment SGI118824[] = {{&S94300[69], 6}, {NULL}};
const Segment SGI118825[] = {{&S94307[34], 7}, {NULL}};
const Segment SGI118826[] = {{&S94307[27], 6}, {NULL}};
const Segment SGI118827[] = {{&S94307[95], 9}, {NULL}};
const Segment SGI118828[] = {{&S94307[68], 6}, {NULL}};
const Segment SGI118829[] = {{&S94318[44], 14}, {NULL}};
const Segment SGI118830[] = {{&S94318[44], 26}, {NULL}};
const Segment SGI118831[] = {{&S94318[30], 6}, {NULL}};
const Segment SGI118832[] = {{&S94318[31], 3}, {NULL}};
const Segment SGI118833[] = {{&S94318[84], 5}, {NULL}};
const Segment SGI118834[] = {{&S94373[34], 17}, {NULL}};
const Segment SGI118835[] = {{&S94373[27], 6}, {NULL}};
const Segment SGI118836[] = {{&S94373[120], 9}, {NULL}};
const Segment SGI118837[] = {{&S94373[93], 6}, {NULL}};
const Segment SGI118838[] = {{&S94373[154], 8}, {NULL}};
const Segment SGI118839[] = {{&S94377[34], 10}, {NULL}};
const Segment SGI118840[] = {{&S94377[27], 6}, {NULL}};
const Segment SGI118841[] = {{&S94377[97], 9}, {NULL}};
const Segment SGI118842[] = {{&S94377[70], 6}, {NULL}};
const Segment SGI118843[] = {{&S94377[138], 8}, {NULL}};
const Segment SGI118844[] = {{&S94429[44], 9}, {NULL}};
const Segment SGI118845[] = {{&S94429[44], 20}, {NULL}};
const Segment SGI118846[] = {{&S94429[30], 6}, {NULL}};
const Segment SGI118847[] = {{&S94429[31], 3}, {NULL}};
const Segment SGI118848[] = {{&S94429[78], 6}, {NULL}};
const Segment SGI118849[] = {{&S94455[34], 14}, {NULL}};
const Segment SGI118850[] = {{&S94455[27], 6}, {NULL}};
const Segment SGI118851[] = {{&S94455[101], 9}, {NULL}};
const Segment SGI118852[] = {{&S94455[74], 6}, {NULL}};
const Segment SGI118853[] = {{&S94483[34], 7}, {NULL}};
const Segment SGI118854[] = {{&S94483[27], 6}, {NULL}};
const Segment SGI118855[] = {{&S94483[94], 9}, {NULL}};
const Segment SGI118856[] = {{&S94483[67], 6}, {NULL}};
const Segment SGI118857[] = {{&S94484[34], 8}, {NULL}};
const Segment SGI118858[] = {{&S94484[27], 6}, {NULL}};
const Segment SGI118859[] = {{&S94484[95], 9}, {NULL}};
const Segment SGI118860[] = {{&S94484[68], 6}, {NULL}};
const Segment SGI118861[] = {{&S94497[44], 9}, {NULL}};
const Segment SGI118862[] = {{&S94497[44], 21}, {NULL}};
const Segment SGI118863[] = {{&S94497[30], 6}, {NULL}};
const Segment SGI118864[] = {{&S94497[31], 3}, {NULL}};
const Segment SGI118865[] = {{&S94497[79], 6}, {NULL}};
const Segment SGI118866[] = {{&S94498[34], 11}, {NULL}};
const Segment SGI118867[] = {{&S94498[27], 6}, {NULL}};
const Segment SGI118868[] = {{&S94498[98], 9}, {NULL}};
const Segment SGI118869[] = {{&S94498[71], 6}, {NULL}};
const Segment SGI118870[] = {{&S5150[44], 10}, {NULL}};
const Segment SGI118871[] = {{&S5150[44], 22}, {NULL}};
const Segment SGI118872[] = {{&S5150[30], 6}, {NULL}};
const Segment SGI118873[] = {{&S5150[31], 3}, {NULL}};
const Segment SGI118874[] = {{&S5150[80], 5}, {NULL}};
const Segment SGI118875[] = {{&S94518[45], 18}, {NULL}};
const Segment SGI118876[] = {{&S94518[30], 6}, {NULL}};
const Segment SGI118877[] = {{&S94518[31], 3}, {NULL}};
const Segment SGI118878[] = {{&S94518[89], 5}, {NULL}};
const Segment SGI118879[] = {{&S94518[147], 13}, {NULL}};
const Segment SGI118880[] = {{&S94525[34], 14}, {NULL}};
const Segment SGI118881[] = {{&S94525[27], 6}, {NULL}};
const Segment SGI118882[] = {{&S94525[101], 9}, {NULL}};
const Segment SGI118883[] = {{&S94525[74], 6}, {NULL}};
const Segment SGI118884[] = {{&S5173[44], 7}, {NULL}};
const Segment SGI118885[] = {{&S5173[44], 19}, {NULL}};
const Segment SGI118886[] = {{&S5173[30], 6}, {NULL}};
const Segment SGI118887[] = {{&S5173[31], 3}, {NULL}};
const Segment SGI118888[] = {{&S5173[77], 5}, {NULL}};
const Segment SGI118889[] = {{&S94568[44], 10}, {NULL}};
const Segment SGI118890[] = {{&S94568[44], 22}, {NULL}};
const Segment SGI118891[] = {{&S94568[30], 6}, {NULL}};
const Segment SGI118892[] = {{&S94568[31], 3}, {NULL}};
const Segment SGI118893[] = {{&S94568[80], 6}, {NULL}};
const Segment SGI118894[] = {{&S5208[44], 11}, {NULL}};
const Segment SGI118895[] = {{&S5208[44], 24}, {NULL}};
const Segment SGI118896[] = {{&S5208[30], 6}, {NULL}};
const Segment SGI118897[] = {{&S5208[31], 3}, {NULL}};
const Segment SGI118898[] = {{&S5208[82], 5}, {NULL}};
const Segment SGI118899[] = {{&S5211[38], 10}, {NULL}};
const Segment SGI118900[] = {{&S5211[31], 6}, {NULL}};
const Segment SGI118901[] = {{&S94573[44], 23}, {NULL}};
const Segment SGI118902[] = {{&S94573[44], 35}, {NULL}};
const Segment SGI118903[] = {{&S94573[30], 6}, {NULL}};
const Segment SGI118904[] = {{&S94573[93], 6}, {NULL}};
const Segment SGI118905[] = {{&S94573[120], 13}, {NULL}};
const Segment SGI118906[] = {{&S94594[44], 13}, {NULL}};
const Segment SGI118907[] = {{&S94594[44], 25}, {NULL}};
const Segment SGI118908[] = {{&S94594[30], 6}, {NULL}};
const Segment SGI118909[] = {{&S94594[31], 3}, {NULL}};
const Segment SGI118910[] = {{&S94594[83], 6}, {NULL}};
const Segment SGI118911[] = {{&S94594[143], 7}, {NULL}};
const Segment SGI118912[] = {{&S94597[44], 14}, {NULL}};
const Segment SGI118913[] = {{&S94597[44], 31}, {NULL}};
const Segment SGI118914[] = {{&S94597[30], 6}, {NULL}};
const Segment SGI118915[] = {{&S94597[31], 3}, {NULL}};
const Segment SGI118916[] = {{&S94597[89], 5}, {NULL}};
const Segment SGI118917[] = {{&S94597[126], 14}, {NULL}};
const Segment SGI118918[] = {{&S94681[44], 6}, {NULL}};
const Segment SGI118919[] = {{&S94681[44], 18}, {NULL}};
const Segment SGI118920[] = {{&S94681[30], 6}, {NULL}};
const Segment SGI118921[] = {{&S94681[107], 2}, {NULL}};
const Segment SGI118922[] = {{&S94681[76], 5}, {NULL}};
const Segment SGI118923[] = {{&S94711[34], 10}, {NULL}};
const Segment SGI118924[] = {{&S94711[27], 6}, {NULL}};
const Segment SGI118925[] = {{&S94711[97], 9}, {NULL}};
const Segment SGI118926[] = {{&S94711[70], 6}, {NULL}};
const Segment SGI118927[] = {{&S94720[59], 11}, {NULL}};
const Segment SGI118928[] = {{&S94720[59], 23}, {NULL}};
const Segment SGI118929[] = {{&S94720[45], 6}, {NULL}};
const Segment SGI118930[] = {{&S94720[46], 3}, {NULL}};
const Segment SGI118931[] = {{&S94720[96], 6}, {NULL}};
const Segment SGI118932[] = {{&S94720[0], 14}, {NULL}};
const Segment SGI118933[] = {{&S94733[34], 22}, {NULL}};
const Segment SGI118934[] = {{&S94733[27], 6}, {NULL}};
const Segment SGI118935[] = {{&S94733[109], 9}, {NULL}};
const Segment SGI118936[] = {{&S94733[82], 6}, {NULL}};
const Segment SGI118937[] = {{&S94762[34], 7}, {NULL}};
const Segment SGI118938[] = {{&S94762[27], 6}, {NULL}};
const Segment SGI118939[] = {{&S94762[94], 9}, {NULL}};
const Segment SGI118940[] = {{&S94762[67], 6}, {NULL}};
const Segment SGI118941[] = {{&S94775[34], 14}, {NULL}};
const Segment SGI118942[] = {{&S94775[27], 6}, {NULL}};
const Segment SGI118943[] = {{&S94775[101], 9}, {NULL}};
const Segment SGI118944[] = {{&S94775[74], 6}, {NULL}};
const Segment SGI118945[] = {{&S94775[135], 8}, {NULL}};
const Segment SGI118946[] = {{&S94785[27], 6}, {NULL}};
const Segment SGI118947[] = {{&S94785[102], 9}, {NULL}};
const Segment SGI118948[] = {{&S94785[75], 6}, {NULL}};
const Segment SGI118949[] = {{&S11290[44], 10}, {NULL}};
const Segment SGI118950[] = {{&S11290[44], 22}, {NULL}};
const Segment SGI118951[] = {{&S11290[30], 6}, {NULL}};
const Segment SGI118952[] = {{&S11290[80], 6}, {NULL}};
const Segment SGI118953[] = {{&S11295[44], 10}, {NULL}};
const Segment SGI118954[] = {{&S11295[44], 22}, {NULL}};
const Segment SGI118955[] = {{&S11295[30], 6}, {NULL}};
const Segment SGI118956[] = {{&S11295[31], 3}, {NULL}};
const Segment SGI118957[] = {{&S11295[80], 6}, {NULL}};
const Segment SGI118958[] = {{&S11299[44], 10}, {NULL}};
const Segment SGI118959[] = {{&S11299[44], 22}, {NULL}};
const Segment SGI118960[] = {{&S11299[30], 6}, {NULL}};
const Segment SGI118961[] = {{&S11299[31], 3}, {NULL}};
const Segment SGI118962[] = {{&S11299[80], 6}, {NULL}};
const Segment SGI118963[] = {{&S11302[38], 10}, {NULL}};
const Segment SGI118964[] = {{&S11302[31], 6}, {NULL}};
const Segment SGI118965[] = {{&S11304[44], 10}, {NULL}};
const Segment SGI118966[] = {{&S11304[44], 22}, {NULL}};
const Segment SGI118967[] = {{&S11304[30], 6}, {NULL}};
const Segment SGI118968[] = {{&S11304[31], 3}, {NULL}};
const Segment SGI118969[] = {{&S11304[80], 6}, {NULL}};
const Segment SGI118970[] = {{&S11309[44], 11}, {NULL}};
const Segment SGI118971[] = {{&S11309[44], 23}, {NULL}};
const Segment SGI118972[] = {{&S11309[30], 6}, {NULL}};
const Segment SGI118973[] = {{&S11309[31], 3}, {NULL}};
const Segment SGI118974[] = {{&S11309[81], 6}, {NULL}};
const Segment SGI118975[] = {{&S11311[44], 29}, {NULL}};
const Segment SGI118976[] = {{&S11311[44], 41}, {NULL}};
const Segment SGI118977[] = {{&S11311[30], 6}, {NULL}};
const Segment SGI118978[] = {{&S11311[31], 3}, {NULL}};
const Segment SGI118979[] = {{&S11311[99], 6}, {NULL}};
const Segment SGI118980[] = {{&S11312[44], 11}, {NULL}};
const Segment SGI118981[] = {{&S11312[44], 23}, {NULL}};
const Segment SGI118982[] = {{&S11312[30], 6}, {NULL}};
const Segment SGI118983[] = {{&S11312[31], 3}, {NULL}};
const Segment SGI118984[] = {{&S11312[81], 6}, {NULL}};
const Segment SGI118985[] = {{&S11313[44], 11}, {NULL}};
const Segment SGI118986[] = {{&S11313[44], 23}, {NULL}};
const Segment SGI118987[] = {{&S11313[30], 6}, {NULL}};
const Segment SGI118988[] = {{&S11313[31], 3}, {NULL}};
const Segment SGI118989[] = {{&S11313[81], 6}, {NULL}};
const Segment SGI118990[] = {{&S11316[44], 10}, {NULL}};
const Segment SGI118991[] = {{&S11316[44], 22}, {NULL}};
const Segment SGI118992[] = {{&S11316[30], 6}, {NULL}};
const Segment SGI118993[] = {{&S11316[83], 6}, {NULL}};
const Segment SGI118994[] = {{&S11317[44], 11}, {NULL}};
const Segment SGI118995[] = {{&S11317[44], 23}, {NULL}};
const Segment SGI118996[] = {{&S11317[30], 6}, {NULL}};
const Segment SGI118997[] = {{&S11317[31], 3}, {NULL}};
const Segment SGI118998[] = {{&S11317[81], 6}, {NULL}};
const Segment SGI118999[] = {{&S11318[44], 10}, {NULL}};
const Segment SGI119000[] = {{&S11318[44], 22}, {NULL}};
const Segment SGI119001[] = {{&S11318[30], 6}, {NULL}};
const Segment SGI119002[] = {{&S11318[31], 3}, {NULL}};
const Segment SGI119003[] = {{&S11318[80], 6}, {NULL}};
const Segment SGI119004[] = {{&S11320[38], 10}, {NULL}};
const Segment SGI119005[] = {{&S11320[31], 6}, {NULL}};
const Segment SGI119006[] = {{&S11321[44], 11}, {NULL}};
const Segment SGI119007[] = {{&S11321[44], 23}, {NULL}};
const Segment SGI119008[] = {{&S11321[30], 6}, {NULL}};
const Segment SGI119009[] = {{&S11321[31], 3}, {NULL}};
const Segment SGI119010[] = {{&S11321[81], 6}, {NULL}};
const Segment SGI119011[] = {{&S11322[44], 8}, {NULL}};
const Segment SGI119012[] = {{&S11322[44], 20}, {NULL}};
const Segment SGI119013[] = {{&S11322[30], 6}, {NULL}};
const Segment SGI119014[] = {{&S11322[31], 3}, {NULL}};
const Segment SGI119015[] = {{&S11322[78], 6}, {NULL}};
const Segment SGI119016[] = {{&S11323[44], 11}, {NULL}};
const Segment SGI119017[] = {{&S11323[44], 23}, {NULL}};
const Segment SGI119018[] = {{&S11323[30], 6}, {NULL}};
const Segment SGI119019[] = {{&S11323[31], 3}, {NULL}};
const Segment SGI119020[] = {{&S11323[81], 6}, {NULL}};
const Segment SGI119021[] = {{&S11324[42], 10}, {NULL}};
const Segment SGI119022[] = {{&S11324[30], 6}, {NULL}};
const Segment SGI119023[] = {{&S11324[31], 3}, {NULL}};
const Segment SGI119024[] = {{&S11324[78], 6}, {NULL}};
const Segment SGI119025[] = {{&S11325[38], 11}, {NULL}};
const Segment SGI119026[] = {{&S11325[31], 6}, {NULL}};
const Segment SGI119027[] = {{&S11326[44], 11}, {NULL}};
const Segment SGI119028[] = {{&S11326[44], 23}, {NULL}};
const Segment SGI119029[] = {{&S11326[30], 6}, {NULL}};
const Segment SGI119030[] = {{&S11326[31], 3}, {NULL}};
const Segment SGI119031[] = {{&S11326[81], 6}, {NULL}};
const Segment SGI119032[] = {{&S11327[44], 10}, {NULL}};
const Segment SGI119033[] = {{&S11327[44], 22}, {NULL}};
const Segment SGI119034[] = {{&S11327[30], 6}, {NULL}};
const Segment SGI119035[] = {{&S11327[80], 6}, {NULL}};
const Segment SGI119036[] = {{&S11328[44], 8}, {NULL}};
const Segment SGI119037[] = {{&S11328[44], 20}, {NULL}};
const Segment SGI119038[] = {{&S11328[30], 6}, {NULL}};
const Segment SGI119039[] = {{&S11328[31], 3}, {NULL}};
const Segment SGI119040[] = {{&S11328[78], 6}, {NULL}};
const Segment SGI119041[] = {{&S11329[38], 11}, {NULL}};
const Segment SGI119042[] = {{&S11329[31], 6}, {NULL}};
const Segment SGI119043[] = {{&S11330[38], 11}, {NULL}};
const Segment SGI119044[] = {{&S11330[31], 6}, {NULL}};
const Segment SGI119045[] = {{&S11331[44], 10}, {NULL}};
const Segment SGI119046[] = {{&S11331[44], 22}, {NULL}};
const Segment SGI119047[] = {{&S11331[30], 6}, {NULL}};
const Segment SGI119048[] = {{&S11331[31], 3}, {NULL}};
const Segment SGI119049[] = {{&S11331[80], 6}, {NULL}};
const Segment SGI119050[] = {{&S94853[44], 11}, {NULL}};
const Segment SGI119051[] = {{&S94853[44], 23}, {NULL}};
const Segment SGI119052[] = {{&S11332[38], 11}, {NULL}};
const Segment SGI119053[] = {{&S94853[30], 6}, {NULL}};
const Segment SGI119054[] = {{&S11332[31], 6}, {NULL}};
const Segment SGI119055[] = {{&S94853[81], 6}, {NULL}};
const Segment SGI119056[] = {{&S94853[108], 13}, {NULL}};
const Segment SGI119057[] = {{&S11333[44], 29}, {NULL}};
const Segment SGI119058[] = {{&S11333[44], 41}, {NULL}};
const Segment SGI119059[] = {{&S11333[30], 6}, {NULL}};
const Segment SGI119060[] = {{&S11333[31], 3}, {NULL}};
const Segment SGI119061[] = {{&S11333[99], 6}, {NULL}};
const Segment SGI119062[] = {{&S11334[44], 29}, {NULL}};
const Segment SGI119063[] = {{&S11334[44], 41}, {NULL}};
const Segment SGI119064[] = {{&S11334[30], 6}, {NULL}};
const Segment SGI119065[] = {{&S11334[31], 3}, {NULL}};
const Segment SGI119066[] = {{&S11334[99], 6}, {NULL}};
const Segment SGI119067[] = {{&S94857[44], 11}, {NULL}};
const Segment SGI119068[] = {{&S94857[44], 23}, {NULL}};
const Segment SGI119069[] = {{&S94857[30], 6}, {NULL}};
const Segment SGI119070[] = {{&S94857[31], 3}, {NULL}};
const Segment SGI119071[] = {{&S94857[81], 5}, {NULL}};
const Segment SGI119072[] = {{&S94857[118], 14}, {NULL}};
const Segment SGI119073[] = {{&S11336[42], 11}, {NULL}};
const Segment SGI119074[] = {{&S11336[30], 6}, {NULL}};
const Segment SGI119075[] = {{&S11336[31], 3}, {NULL}};
const Segment SGI119076[] = {{&S11336[79], 6}, {NULL}};
const Segment SGI119077[] = {{&S94859[34], 7}, {NULL}};
const Segment SGI119078[] = {{&S94859[27], 6}, {NULL}};
const Segment SGI119079[] = {{&S94859[94], 9}, {NULL}};
const Segment SGI119080[] = {{&S94859[67], 6}, {NULL}};
const Segment SGI119081[] = {{&S11337[38], 8}, {NULL}};
const Segment SGI119082[] = {{&S11337[31], 6}, {NULL}};
const Segment SGI119083[] = {{&S94859[128], 8}, {NULL}};
const Segment SGI119084[] = {{&S11338[44], 10}, {NULL}};
const Segment SGI119085[] = {{&S11338[44], 22}, {NULL}};
const Segment SGI119086[] = {{&S11338[30], 6}, {NULL}};
const Segment SGI119087[] = {{&S11338[83], 6}, {NULL}};
const Segment SGI119088[] = {{&S11339[38], 11}, {NULL}};
const Segment SGI119089[] = {{&S11339[31], 6}, {NULL}};
const Segment SGI119090[] = {{&S94875[34], 41}, {NULL}};
const Segment SGI119091[] = {{&S94875[27], 6}, {NULL}};
const Segment SGI119092[] = {{&S94875[128], 9}, {NULL}};
const Segment SGI119093[] = {{&S94875[101], 6}, {NULL}};
const Segment SGI119094[] = {{&S94911[44], 10}, {NULL}};
const Segment SGI119095[] = {{&S94911[44], 20}, {NULL}};
const Segment SGI119096[] = {{&S94911[30], 6}, {NULL}};
const Segment SGI119097[] = {{&S94911[31], 3}, {NULL}};
const Segment SGI119098[] = {{&S94911[78], 6}, {NULL}};
const Segment SGI119099[] = {{&S94927[44], 10}, {NULL}};
const Segment SGI119100[] = {{&S94927[44], 28}, {NULL}};
const Segment SGI119101[] = {{&S94927[30], 6}, {NULL}};
const Segment SGI119102[] = {{&S94927[31], 3}, {NULL}};
const Segment SGI119103[] = {{&S94927[86], 6}, {NULL}};
const Segment SGI119104[] = {{&S94929[59], 11}, {NULL}};
const Segment SGI119105[] = {{&S94929[59], 23}, {NULL}};
const Segment SGI119106[] = {{&S94929[45], 6}, {NULL}};
const Segment SGI119107[] = {{&S94929[46], 3}, {NULL}};
const Segment SGI119108[] = {{&S94929[96], 6}, {NULL}};
const Segment SGI119109[] = {{&S94929[0], 14}, {NULL}};
const Segment SGI119110[] = {{&S94943[34], 7}, {NULL}};
const Segment SGI119111[] = {{&S94943[27], 6}, {NULL}};
const Segment SGI119112[] = {{&S94943[94], 9}, {NULL}};
const Segment SGI119113[] = {{&S94943[67], 6}, {NULL}};
const Segment SGI119114[] = {{&S94959[44], 10}, {NULL}};
const Segment SGI119115[] = {{&S94959[44], 22}, {NULL}};
const Segment SGI119116[] = {{&S94959[30], 6}, {NULL}};
const Segment SGI119117[] = {{&S94959[31], 3}, {NULL}};
const Segment SGI119118[] = {{&S94959[80], 6}, {NULL}};
const Segment SGI119119[] = {{&S94965[57], 6}, {NULL}};
const Segment SGI119120[] = {{&S94965[57], 18}, {NULL}};
const Segment SGI119121[] = {{&S94965[43], 6}, {NULL}};
const Segment SGI119122[] = {{&S94965[44], 3}, {NULL}};
const Segment SGI119123[] = {{&S94965[89], 6}, {NULL}};
const Segment SGI119124[] = {{&S94965[0], 12}, {NULL}};
const Segment SGI119125[] = {{&S94966[34], 12}, {NULL}};
const Segment SGI119126[] = {{&S94966[27], 6}, {NULL}};
const Segment SGI119127[] = {{&S94966[99], 9}, {NULL}};
const Segment SGI119128[] = {{&S94966[72], 6}, {NULL}};
const Segment SGI119129[] = {{&S94986[44], 11}, {NULL}};
const Segment SGI119130[] = {{&S94986[44], 23}, {NULL}};
const Segment SGI119131[] = {{&S94986[30], 6}, {NULL}};
const Segment SGI119132[] = {{&S94986[31], 3}, {NULL}};
const Segment SGI119133[] = {{&S94986[81], 5}, {NULL}};
const Segment SGI119134[] = {{&S94986[118], 14}, {NULL}};
const Segment SGI119135[] = {{&S94990[44], 6}, {NULL}};
const Segment SGI119136[] = {{&S94990[44], 18}, {NULL}};
const Segment SGI119137[] = {{&S94990[30], 6}, {NULL}};
const Segment SGI119138[] = {{&S94990[76], 6}, {NULL}};
const Segment SGI119139[] = {{&S94990[103], 13}, {NULL}};
const Segment SGI119140[] = {{&S94998[27], 6}, {NULL}};
const Segment SGI119141[] = {{&S94998[102], 9}, {NULL}};
const Segment SGI119142[] = {{&S94998[75], 6}, {NULL}};
const Segment SGI119143[] = {{&S95021[44], 13}, {NULL}};
const Segment SGI119144[] = {{&S95021[44], 29}, {NULL}};
const Segment SGI119145[] = {{&S95021[30], 6}, {NULL}};
const Segment SGI119146[] = {{&S95021[31], 3}, {NULL}};
const Segment SGI119147[] = {{&S95021[87], 5}, {NULL}};
const Segment SGI119148[] = {{&S95026[34], 7}, {NULL}};
const Segment SGI119149[] = {{&S95026[27], 6}, {NULL}};
const Segment SGI119150[] = {{&S95026[94], 9}, {NULL}};
const Segment SGI119151[] = {{&S95026[67], 6}, {NULL}};
const Segment SGI119152[] = {{&S95026[128], 8}, {NULL}};
const Segment SGI119153[] = {{&S95053[44], 11}, {NULL}};
const Segment SGI119154[] = {{&S95053[44], 23}, {NULL}};
const Segment SGI119155[] = {{&S95053[30], 6}, {NULL}};
const Segment SGI119156[] = {{&S95053[31], 3}, {NULL}};
const Segment SGI119157[] = {{&S95053[81], 6}, {NULL}};
const Segment SGI119158[] = {{&S95058[44], 16}, {NULL}};
const Segment SGI119159[] = {{&S95058[44], 28}, {NULL}};
const Segment SGI119160[] = {{&S95058[30], 6}, {NULL}};
const Segment SGI119161[] = {{&S95058[31], 3}, {NULL}};
const Segment SGI119162[] = {{&S95058[86], 6}, {NULL}};
const Segment SGI119163[] = {{&S95058[146], 7}, {NULL}};
const Segment SGI119164[] = {{&S95060[44], 14}, {NULL}};
const Segment SGI119165[] = {{&S95060[44], 26}, {NULL}};
const Segment SGI119166[] = {{&S95060[30], 6}, {NULL}};
const Segment SGI119167[] = {{&S95060[31], 3}, {NULL}};
const Segment SGI119168[] = {{&S95060[84], 6}, {NULL}};
const Segment SGI119169[] = {{&S95060[144], 7}, {NULL}};
const Segment SGI119170[] = {{&S95065[52], 11}, {NULL}};
const Segment SGI119171[] = {{&S95065[52], 23}, {NULL}};
const Segment SGI119172[] = {{&S95065[38], 6}, {NULL}};
const Segment SGI119173[] = {{&S95065[89], 5}, {NULL}};
const Segment SGI119174[] = {{&S95065[0], 14}, {NULL}};
const Segment SGI119175[] = {{&S95081[34], 8}, {NULL}};
const Segment SGI119176[] = {{&S95081[27], 6}, {NULL}};
const Segment SGI119177[] = {{&S95081[95], 9}, {NULL}};
const Segment SGI119178[] = {{&S95081[68], 6}, {NULL}};
const Segment SGI119179[] = {{&S2961[44], 10}, {NULL}};
const Segment SGI119180[] = {{&S2961[44], 22}, {NULL}};
const Segment SGI119181[] = {{&S2961[30], 6}, {NULL}};
const Segment SGI119182[] = {{&S2961[115], 3}, {NULL}};
const Segment SGI119183[] = {{&S2961[80], 6}, {NULL}};
const Segment SGI119184[] = {{&S2962[44], 10}, {NULL}};
const Segment SGI119185[] = {{&S2962[44], 22}, {NULL}};
const Segment SGI119186[] = {{&S2962[30], 6}, {NULL}};
const Segment SGI119187[] = {{&S2962[115], 3}, {NULL}};
const Segment SGI119188[] = {{&S2962[80], 6}, {NULL}};
const Segment SGI119189[] = {{&S2963[44], 10}, {NULL}};
const Segment SGI119190[] = {{&S2963[44], 22}, {NULL}};
const Segment SGI119191[] = {{&S2963[30], 6}, {NULL}};
const Segment SGI119192[] = {{&S2963[115], 3}, {NULL}};
const Segment SGI119193[] = {{&S2963[80], 6}, {NULL}};
const Segment SGI119194[] = {{&S2964[44], 10}, {NULL}};
const Segment SGI119195[] = {{&S2964[44], 22}, {NULL}};
const Segment SGI119196[] = {{&S2964[30], 6}, {NULL}};
const Segment SGI119197[] = {{&S2964[115], 3}, {NULL}};
const Segment SGI119198[] = {{&S2964[80], 6}, {NULL}};
const Segment SGI119199[] = {{&S2965[44], 10}, {NULL}};
const Segment SGI119200[] = {{&S2965[44], 22}, {NULL}};
const Segment SGI119201[] = {{&S2965[30], 6}, {NULL}};
const Segment SGI119202[] = {{&S2965[115], 3}, {NULL}};
const Segment SGI119203[] = {{&S2965[80], 6}, {NULL}};
const Segment SGI119204[] = {{&S2966[38], 10}, {NULL}};
const Segment SGI119205[] = {{&S2966[31], 6}, {NULL}};
const Segment SGI119206[] = {{&S2967[44], 10}, {NULL}};
const Segment SGI119207[] = {{&S2967[44], 22}, {NULL}};
const Segment SGI119208[] = {{&S2967[30], 6}, {NULL}};
const Segment SGI119209[] = {{&S2967[115], 3}, {NULL}};
const Segment SGI119210[] = {{&S2967[80], 6}, {NULL}};
const Segment SGI119211[] = {{&S2968[44], 10}, {NULL}};
const Segment SGI119212[] = {{&S2968[44], 22}, {NULL}};
const Segment SGI119213[] = {{&S2968[30], 6}, {NULL}};
const Segment SGI119214[] = {{&S2968[115], 3}, {NULL}};
const Segment SGI119215[] = {{&S2968[80], 6}, {NULL}};
const Segment SGI119216[] = {{&S2969[44], 10}, {NULL}};
const Segment SGI119217[] = {{&S2969[44], 22}, {NULL}};
const Segment SGI119218[] = {{&S2969[30], 6}, {NULL}};
const Segment SGI119219[] = {{&S2969[115], 3}, {NULL}};
const Segment SGI119220[] = {{&S2969[80], 6}, {NULL}};
const Segment SGI119221[] = {{&S2970[44], 10}, {NULL}};
const Segment SGI119222[] = {{&S2970[44], 22}, {NULL}};
const Segment SGI119223[] = {{&S2970[30], 6}, {NULL}};
const Segment SGI119224[] = {{&S2970[115], 3}, {NULL}};
const Segment SGI119225[] = {{&S2970[80], 6}, {NULL}};
const Segment SGI119226[] = {{&S2971[44], 10}, {NULL}};
const Segment SGI119227[] = {{&S2971[44], 22}, {NULL}};
const Segment SGI119228[] = {{&S2971[30], 6}, {NULL}};
const Segment SGI119229[] = {{&S2971[115], 3}, {NULL}};
const Segment SGI119230[] = {{&S2971[80], 6}, {NULL}};
const Segment SGI119231[] = {{&S2972[44], 10}, {NULL}};
const Segment SGI119232[] = {{&S2972[44], 22}, {NULL}};
const Segment SGI119233[] = {{&S2972[30], 6}, {NULL}};
const Segment SGI119234[] = {{&S2972[115], 3}, {NULL}};
const Segment SGI119235[] = {{&S2972[80], 6}, {NULL}};
const Segment SGI119236[] = {{&S2973[44], 10}, {NULL}};
const Segment SGI119237[] = {{&S2973[44], 22}, {NULL}};
const Segment SGI119238[] = {{&S2973[30], 6}, {NULL}};
const Segment SGI119239[] = {{&S2973[115], 3}, {NULL}};
const Segment SGI119240[] = {{&S2973[80], 6}, {NULL}};
const Segment SGI119241[] = {{&S95172[44], 10}, {NULL}};
const Segment SGI119242[] = {{&S95172[44], 20}, {NULL}};
const Segment SGI119243[] = {{&S95172[30], 6}, {NULL}};
const Segment SGI119244[] = {{&S2975[44], 30}, {NULL}};
const Segment SGI119245[] = {{&S2975[44], 42}, {NULL}};
const Segment SGI119246[] = {{&S2975[30], 6}, {NULL}};
const Segment SGI119247[] = {{&S95172[31], 3}, {NULL}};
const Segment SGI119248[] = {{&S95172[78], 6}, {NULL}};
const Segment SGI119249[] = {{&S2975[135], 3}, {NULL}};
const Segment SGI119250[] = {{&S2975[100], 6}, {NULL}};
const Segment SGI119251[] = {{&S2978[44], 10}, {NULL}};
const Segment SGI119252[] = {{&S2978[44], 22}, {NULL}};
const Segment SGI119253[] = {{&S2978[30], 6}, {NULL}};
const Segment SGI119254[] = {{&S2978[80], 6}, {NULL}};
const Segment SGI119255[] = {{&S2979[44], 10}, {NULL}};
const Segment SGI119256[] = {{&S2979[44], 22}, {NULL}};
const Segment SGI119257[] = {{&S2979[30], 6}, {NULL}};
const Segment SGI119258[] = {{&S2979[83], 6}, {NULL}};
const Segment SGI119259[] = {{&S2981[44], 10}, {NULL}};
const Segment SGI119260[] = {{&S2981[44], 22}, {NULL}};
const Segment SGI119261[] = {{&S2981[30], 6}, {NULL}};
const Segment SGI119262[] = {{&S2981[115], 3}, {NULL}};
const Segment SGI119263[] = {{&S2981[80], 6}, {NULL}};
const Segment SGI119264[] = {{&S2982[38], 10}, {NULL}};
const Segment SGI119265[] = {{&S2982[31], 6}, {NULL}};
const Segment SGI119266[] = {{&S2983[44], 11}, {NULL}};
const Segment SGI119267[] = {{&S2983[44], 23}, {NULL}};
const Segment SGI119268[] = {{&S2983[30], 6}, {NULL}};
const Segment SGI119269[] = {{&S2983[116], 3}, {NULL}};
const Segment SGI119270[] = {{&S2983[81], 6}, {NULL}};
const Segment SGI119271[] = {{&S2985[38], 10}, {NULL}};
const Segment SGI119272[] = {{&S2985[31], 6}, {NULL}};
const Segment SGI119273[] = {{&S2986[42], 10}, {NULL}};
const Segment SGI119274[] = {{&S2986[30], 6}, {NULL}};
const Segment SGI119275[] = {{&S2986[113], 3}, {NULL}};
const Segment SGI119276[] = {{&S2986[78], 6}, {NULL}};
const Segment SGI119277[] = {{&S2987[42], 10}, {NULL}};
const Segment SGI119278[] = {{&S2987[30], 6}, {NULL}};
const Segment SGI119279[] = {{&S2987[113], 3}, {NULL}};
const Segment SGI119280[] = {{&S2987[78], 6}, {NULL}};
const Segment SGI119281[] = {{&S2988[44], 11}, {NULL}};
const Segment SGI119282[] = {{&S2988[44], 23}, {NULL}};
const Segment SGI119283[] = {{&S2988[30], 6}, {NULL}};
const Segment SGI119284[] = {{&S2988[116], 3}, {NULL}};
const Segment SGI119285[] = {{&S2988[81], 6}, {NULL}};
const Segment SGI119286[] = {{&S2990[44], 29}, {NULL}};
const Segment SGI119287[] = {{&S2990[44], 41}, {NULL}};
const Segment SGI119288[] = {{&S2990[30], 6}, {NULL}};
const Segment SGI119289[] = {{&S2990[134], 3}, {NULL}};
const Segment SGI119290[] = {{&S2990[99], 6}, {NULL}};
const Segment SGI119291[] = {{&S2991[44], 11}, {NULL}};
const Segment SGI119292[] = {{&S2991[44], 23}, {NULL}};
const Segment SGI119293[] = {{&S2991[30], 6}, {NULL}};
const Segment SGI119294[] = {{&S2991[116], 3}, {NULL}};
const Segment SGI119295[] = {{&S2991[81], 6}, {NULL}};
const Segment SGI119296[] = {{&S2992[38], 11}, {NULL}};
const Segment SGI119297[] = {{&S2992[31], 6}, {NULL}};
const Segment SGI119298[] = {{&S2993[44], 10}, {NULL}};
const Segment SGI119299[] = {{&S2993[44], 22}, {NULL}};
const Segment SGI119300[] = {{&S2993[30], 6}, {NULL}};
const Segment SGI119301[] = {{&S2993[115], 3}, {NULL}};
const Segment SGI119302[] = {{&S2993[80], 6}, {NULL}};
const Segment SGI119303[] = {{&S2995[38], 10}, {NULL}};
const Segment SGI119304[] = {{&S2995[31], 6}, {NULL}};
const Segment SGI119305[] = {{&S2996[44], 7}, {NULL}};
const Segment SGI119306[] = {{&S2996[44], 19}, {NULL}};
const Segment SGI119307[] = {{&S2996[30], 6}, {NULL}};
const Segment SGI119308[] = {{&S2996[112], 3}, {NULL}};
const Segment SGI119309[] = {{&S2996[77], 6}, {NULL}};
const Segment SGI119310[] = {{&S2998[38], 11}, {NULL}};
const Segment SGI119311[] = {{&S2998[31], 6}, {NULL}};
const Segment SGI119312[] = {{&S3000[44], 8}, {NULL}};
const Segment SGI119313[] = {{&S3000[44], 20}, {NULL}};
const Segment SGI119314[] = {{&S3000[30], 6}, {NULL}};
const Segment SGI119315[] = {{&S3000[113], 3}, {NULL}};
const Segment SGI119316[] = {{&S3000[78], 6}, {NULL}};
const Segment SGI119317[] = {{&S95219[59], 13}, {NULL}};
const Segment SGI119318[] = {{&S95219[59], 25}, {NULL}};
const Segment SGI119319[] = {{&S95219[45], 6}, {NULL}};
const Segment SGI119320[] = {{&S95219[46], 3}, {NULL}};
const Segment SGI119321[] = {{&S95219[98], 6}, {NULL}};
const Segment SGI119322[] = {{&S95219[0], 14}, {NULL}};
const Segment SGI119323[] = {{&S3005[44], 10}, {NULL}};
const Segment SGI119324[] = {{&S3005[44], 22}, {NULL}};
const Segment SGI119325[] = {{&S3005[30], 6}, {NULL}};
const Segment SGI119326[] = {{&S4430[42], 10}, {NULL}};
const Segment SGI119327[] = {{&S3005[115], 3}, {NULL}};
const Segment SGI119328[] = {{&S4430[42], 22}, {NULL}};
const Segment SGI119329[] = {{&S4430[30], 4}, {NULL}};
const Segment SGI119330[] = {{&S3005[80], 6}, {NULL}};
const Segment SGI119331[] = {{&S4430[113], 3}, {NULL}};
const Segment SGI119332[] = {{&S4430[78], 6}, {NULL}};
const Segment SGI119333[] = {{&S3006[44], 7}, {NULL}};
const Segment SGI119334[] = {{&S3006[44], 19}, {NULL}};
const Segment SGI119335[] = {{&S3006[30], 6}, {NULL}};
const Segment SGI119336[] = {{&S3006[112], 3}, {NULL}};
const Segment SGI119337[] = {{&S3006[77], 6}, {NULL}};
const Segment SGI119338[] = {{&S95225[59], 9}, {NULL}};
const Segment SGI119339[] = {{&S3008[42], 10}, {NULL}};
const Segment SGI119340[] = {{&S95225[59], 21}, {NULL}};
const Segment SGI119341[] = {{&S3008[30], 6}, {NULL}};
const Segment SGI119342[] = {{&S95225[45], 6}, {NULL}};
const Segment SGI119343[] = {{&S3008[113], 3}, {NULL}};
const Segment SGI119344[] = {{&S95225[46], 3}, {NULL}};
const Segment SGI119345[] = {{&S3008[78], 6}, {NULL}};
const Segment SGI119346[] = {{&S95225[94], 6}, {NULL}};
const Segment SGI119347[] = {{&S95225[0], 14}, {NULL}};
const Segment SGI119348[] = {{&S3009[44], 10}, {NULL}};
const Segment SGI119349[] = {{&S3009[44], 22}, {NULL}};
const Segment SGI119350[] = {{&S3009[30], 6}, {NULL}};
const Segment SGI119351[] = {{&S3009[83], 6}, {NULL}};
const Segment SGI119352[] = {{&S3010[44], 11}, {NULL}};
const Segment SGI119353[] = {{&S3010[44], 23}, {NULL}};
const Segment SGI119354[] = {{&S3010[30], 6}, {NULL}};
const Segment SGI119355[] = {{&S3010[116], 3}, {NULL}};
const Segment SGI119356[] = {{&S3010[81], 6}, {NULL}};
const Segment SGI119357[] = {{&S95229[45], 13}, {NULL}};
const Segment SGI119358[] = {{&S95229[30], 6}, {NULL}};
const Segment SGI119359[] = {{&S95229[31], 3}, {NULL}};
const Segment SGI119360[] = {{&S95229[84], 5}, {NULL}};
const Segment SGI119361[] = {{&S95229[142], 13}, {NULL}};
const Segment SGI119362[] = {{&S95233[34], 6}, {NULL}};
const Segment SGI119363[] = {{&S95233[27], 6}, {NULL}};
const Segment SGI119364[] = {{&S95233[93], 9}, {NULL}};
const Segment SGI119365[] = {{&S95233[66], 6}, {NULL}};
const Segment SGI119366[] = {{&S3017[44], 10}, {NULL}};
const Segment SGI119367[] = {{&S3017[44], 22}, {NULL}};
const Segment SGI119368[] = {{&S3017[30], 6}, {NULL}};
const Segment SGI119369[] = {{&S3017[115], 3}, {NULL}};
const Segment SGI119370[] = {{&S3017[80], 6}, {NULL}};
const Segment SGI119371[] = {{&S95235[44], 21}, {NULL}};
const Segment SGI119372[] = {{&S95235[44], 33}, {NULL}};
const Segment SGI119373[] = {{&S95235[30], 6}, {NULL}};
const Segment SGI119374[] = {{&S3018[44], 11}, {NULL}};
const Segment SGI119375[] = {{&S3018[44], 23}, {NULL}};
const Segment SGI119376[] = {{&S3018[30], 6}, {NULL}};
const Segment SGI119377[] = {{&S95235[91], 6}, {NULL}};
const Segment SGI119378[] = {{&S3018[116], 3}, {NULL}};
const Segment SGI119379[] = {{&S95235[118], 13}, {NULL}};
const Segment SGI119380[] = {{&S3018[81], 6}, {NULL}};
const Segment SGI119381[] = {{&S3019[44], 10}, {NULL}};
const Segment SGI119382[] = {{&S3019[44], 22}, {NULL}};
const Segment SGI119383[] = {{&S3019[30], 6}, {NULL}};
const Segment SGI119384[] = {{&S3019[83], 6}, {NULL}};
const Segment SGI119385[] = {{&S3022[44], 10}, {NULL}};
const Segment SGI119386[] = {{&S3022[44], 22}, {NULL}};
const Segment SGI119387[] = {{&S3022[30], 6}, {NULL}};
const Segment SGI119388[] = {{&S3022[80], 6}, {NULL}};
const Segment SGI119389[] = {{&S3028[44], 8}, {NULL}};
const Segment SGI119390[] = {{&S3028[44], 20}, {NULL}};
const Segment SGI119391[] = {{&S3028[30], 6}, {NULL}};
const Segment SGI119392[] = {{&S3028[113], 3}, {NULL}};
const Segment SGI119393[] = {{&S3028[78], 6}, {NULL}};
const Segment SGI119394[] = {{&S3029[44], 10}, {NULL}};
const Segment SGI119395[] = {{&S3029[44], 22}, {NULL}};
const Segment SGI119396[] = {{&S3029[30], 6}, {NULL}};
const Segment SGI119397[] = {{&S3029[115], 3}, {NULL}};
const Segment SGI119398[] = {{&S3029[80], 6}, {NULL}};
const Segment SGI119399[] = {{&S3040[42], 11}, {NULL}};
const Segment SGI119400[] = {{&S3040[30], 6}, {NULL}};
const Segment SGI119401[] = {{&S3040[114], 3}, {NULL}};
const Segment SGI119402[] = {{&S3040[79], 6}, {NULL}};
const Segment SGI119403[] = {{&S3043[44], 10}, {NULL}};
const Segment SGI119404[] = {{&S3043[44], 22}, {NULL}};
const Segment SGI119405[] = {{&S3043[30], 6}, {NULL}};
const Segment SGI119406[] = {{&S3043[115], 3}, {NULL}};
const Segment SGI119407[] = {{&S3043[80], 6}, {NULL}};
const Segment SGI119408[] = {{&S3045[38], 11}, {NULL}};
const Segment SGI119409[] = {{&S3045[31], 6}, {NULL}};
const Segment SGI119410[] = {{&S3051[44], 19}, {NULL}};
const Segment SGI119411[] = {{&S3051[44], 40}, {NULL}};
const Segment SGI119412[] = {{&S3051[30], 6}, {NULL}};
const Segment SGI119413[] = {{&S3051[132], 3}, {NULL}};
const Segment SGI119414[] = {{&S3051[98], 5}, {NULL}};
const Segment SGI119415[] = {{&S3054[38], 10}, {NULL}};
const Segment SGI119416[] = {{&S3054[31], 6}, {NULL}};
const Segment SGI119417[] = {{&S3055[44], 8}, {NULL}};
const Segment SGI119418[] = {{&S3055[44], 20}, {NULL}};
const Segment SGI119419[] = {{&S3055[30], 6}, {NULL}};
const Segment SGI119420[] = {{&S3055[113], 3}, {NULL}};
const Segment SGI119421[] = {{&S3055[78], 6}, {NULL}};
const Segment SGI119422[] = {{&S3058[44], 10}, {NULL}};
const Segment SGI119423[] = {{&S3058[44], 24}, {NULL}};
const Segment SGI119424[] = {{&S3058[30], 6}, {NULL}};
const Segment SGI119425[] = {{&S3058[117], 3}, {NULL}};
const Segment SGI119426[] = {{&S3058[82], 6}, {NULL}};
const Segment SGI119427[] = {{&S3061[38], 7}, {NULL}};
const Segment SGI119428[] = {{&S3061[31], 6}, {NULL}};
const Segment SGI119429[] = {{&S3062[44], 10}, {NULL}};
const Segment SGI119430[] = {{&S3062[44], 31}, {NULL}};
const Segment SGI119431[] = {{&S3062[30], 6}, {NULL}};
const Segment SGI119432[] = {{&S3062[123], 3}, {NULL}};
const Segment SGI119433[] = {{&S3062[89], 5}, {NULL}};
const Segment SGI119434[] = {{&S3063[44], 10}, {NULL}};
const Segment SGI119435[] = {{&S3063[44], 22}, {NULL}};
const Segment SGI119436[] = {{&S3063[30], 6}, {NULL}};
const Segment SGI119437[] = {{&S3063[114], 3}, {NULL}};
const Segment SGI119438[] = {{&S3063[80], 5}, {NULL}};
const Segment SGI119439[] = {{&S3065[44], 10}, {NULL}};
const Segment SGI119440[] = {{&S3065[44], 24}, {NULL}};
const Segment SGI119441[] = {{&S3065[30], 6}, {NULL}};
const Segment SGI119442[] = {{&S3065[117], 3}, {NULL}};
const Segment SGI119443[] = {{&S3065[82], 6}, {NULL}};
const Segment SGI119444[] = {{&S3074[44], 11}, {NULL}};
const Segment SGI119445[] = {{&S3074[44], 32}, {NULL}};
const Segment SGI119446[] = {{&S3074[30], 6}, {NULL}};
const Segment SGI119447[] = {{&S3074[124], 3}, {NULL}};
const Segment SGI119448[] = {{&S3074[90], 5}, {NULL}};
const Segment SGI119449[] = {{&S95285[34], 14}, {NULL}};
const Segment SGI119450[] = {{&S95285[27], 6}, {NULL}};
const Segment SGI119451[] = {{&S95285[108], 9}, {NULL}};
const Segment SGI119452[] = {{&S95285[81], 6}, {NULL}};
const Segment SGI119453[] = {{&S3079[44], 10}, {NULL}};
const Segment SGI119454[] = {{&S3079[44], 26}, {NULL}};
const Segment SGI119455[] = {{&S3079[30], 6}, {NULL}};
const Segment SGI119456[] = {{&S3079[119], 3}, {NULL}};
const Segment SGI119457[] = {{&S3079[84], 6}, {NULL}};
const Segment SGI119458[] = {{&S95302[34], 7}, {NULL}};
const Segment SGI119459[] = {{&S95302[27], 6}, {NULL}};
const Segment SGI119460[] = {{&S95302[94], 9}, {NULL}};
const Segment SGI119461[] = {{&S95302[67], 6}, {NULL}};
const Segment SGI119462[] = {{&S95303[34], 7}, {NULL}};
const Segment SGI119463[] = {{&S95303[27], 6}, {NULL}};
const Segment SGI119464[] = {{&S95303[94], 9}, {NULL}};
const Segment SGI119465[] = {{&S95303[67], 6}, {NULL}};
const Segment SGI119466[] = {{&S95327[34], 9}, {NULL}};
const Segment SGI119467[] = {{&S95327[27], 6}, {NULL}};
const Segment SGI119468[] = {{&S95327[96], 9}, {NULL}};
const Segment SGI119469[] = {{&S95327[69], 6}, {NULL}};
const Segment SGI119470[] = {{&S95348[34], 10}, {NULL}};
const Segment SGI119471[] = {{&S95348[27], 6}, {NULL}};
const Segment SGI119472[] = {{&S95348[97], 9}, {NULL}};
const Segment SGI119473[] = {{&S95348[70], 6}, {NULL}};
const Segment SGI119474[] = {{&S95348[138], 8}, {NULL}};
const Segment SGI119475[] = {{&S95349[34], 9}, {NULL}};
const Segment SGI119476[] = {{&S95349[27], 6}, {NULL}};
const Segment SGI119477[] = {{&S95349[96], 9}, {NULL}};
const Segment SGI119478[] = {{&S95349[69], 6}, {NULL}};
const Segment SGI119479[] = {{&S95349[137], 8}, {NULL}};
const Segment SGI119480[] = {{&S95351[34], 9}, {NULL}};
const Segment SGI119481[] = {{&S95351[27], 6}, {NULL}};
const Segment SGI119482[] = {{&S95351[96], 9}, {NULL}};
const Segment SGI119483[] = {{&S95351[69], 6}, {NULL}};
const Segment SGI119484[] = {{&S5897[44], 13}, {NULL}};
const Segment SGI119485[] = {{&S5897[44], 25}, {NULL}};
const Segment SGI119486[] = {{&S5897[30], 6}, {NULL}};
const Segment SGI119487[] = {{&S5897[31], 3}, {NULL}};
const Segment SGI119488[] = {{&S5897[83], 5}, {NULL}};
const Segment SGI119489[] = {{&S95432[57], 10}, {NULL}};
const Segment SGI119490[] = {{&S95432[57], 22}, {NULL}};
const Segment SGI119491[] = {{&S95432[43], 6}, {NULL}};
const Segment SGI119492[] = {{&S95432[44], 3}, {NULL}};
const Segment SGI119493[] = {{&S95432[93], 6}, {NULL}};
const Segment SGI119494[] = {{&S95432[0], 12}, {NULL}};
const Segment SGI119495[] = {{&S95438[59], 14}, {NULL}};
const Segment SGI119496[] = {{&S95438[59], 31}, {NULL}};
const Segment SGI119497[] = {{&S95438[45], 6}, {NULL}};
const Segment SGI119498[] = {{&S95438[46], 3}, {NULL}};
const Segment SGI119499[] = {{&S95438[104], 6}, {NULL}};
const Segment SGI119500[] = {{&S95438[0], 14}, {NULL}};
const Segment SGI119501[] = {{&S95442[57], 11}, {NULL}};
const Segment SGI119502[] = {{&S95442[57], 23}, {NULL}};
const Segment SGI119503[] = {{&S95442[43], 6}, {NULL}};
const Segment SGI119504[] = {{&S95442[44], 3}, {NULL}};
const Segment SGI119505[] = {{&S95442[94], 6}, {NULL}};
const Segment SGI119506[] = {{&S95442[0], 12}, {NULL}};
const Segment SGI119507[] = {{&S95451[34], 10}, {NULL}};
const Segment SGI119508[] = {{&S95451[27], 6}, {NULL}};
const Segment SGI119509[] = {{&S95451[98], 9}, {NULL}};
const Segment SGI119510[] = {{&S95451[71], 6}, {NULL}};
const Segment SGI119511[] = {{&S95474[34], 18}, {NULL}};
const Segment SGI119512[] = {{&S95474[27], 6}, {NULL}};
const Segment SGI119513[] = {{&S95474[104], 9}, {NULL}};
const Segment SGI119514[] = {{&S95474[77], 6}, {NULL}};
const Segment SGI119515[] = {{&S95475[59], 14}, {NULL}};
const Segment SGI119516[] = {{&S95475[59], 26}, {NULL}};
const Segment SGI119517[] = {{&S95475[45], 6}, {NULL}};
const Segment SGI119518[] = {{&S95475[46], 3}, {NULL}};
const Segment SGI119519[] = {{&S95475[99], 6}, {NULL}};
const Segment SGI119520[] = {{&S95475[0], 14}, {NULL}};
const Segment SGI119521[] = {{&S95481[45], 17}, {NULL}};
const Segment SGI119522[] = {{&S95481[30], 6}, {NULL}};
const Segment SGI119523[] = {{&S95481[31], 3}, {NULL}};
const Segment SGI119524[] = {{&S95481[96], 5}, {NULL}};
const Segment SGI119525[] = {{&S95481[154], 13}, {NULL}};
const Segment SGI119526[] = {{&S95484[44], 10}, {NULL}};
const Segment SGI119527[] = {{&S95484[44], 22}, {NULL}};
const Segment SGI119528[] = {{&S95484[30], 6}, {NULL}};
const Segment SGI119529[] = {{&S95484[158], 3}, {NULL}};
const Segment SGI119530[] = {{&S95484[124], 3}, {NULL}};
const Segment SGI119531[] = {{&S95484[89], 6}, {NULL}};
const Segment SGI119532[] = {{&S95484[144], 8}, {NULL}};
const Segment SGI119533[] = {{&S95490[44], 9}, {NULL}};
const Segment SGI119534[] = {{&S95490[44], 21}, {NULL}};
const Segment SGI119535[] = {{&S95490[30], 6}, {NULL}};
const Segment SGI119536[] = {{&S95490[31], 3}, {NULL}};
const Segment SGI119537[] = {{&S95490[79], 6}, {NULL}};
const Segment SGI119538[] = {{&S95504[34], 7}, {NULL}};
const Segment SGI119539[] = {{&S95504[27], 6}, {NULL}};
const Segment SGI119540[] = {{&S95504[95], 9}, {NULL}};
const Segment SGI119541[] = {{&S95504[68], 6}, {NULL}};
const Segment SGI119542[] = {{&S95508[44], 41}, {NULL}};
const Segment SGI119543[] = {{&S95508[44], 53}, {NULL}};
const Segment SGI119544[] = {{&S95508[30], 6}, {NULL}};
const Segment SGI119545[] = {{&S95508[111], 6}, {NULL}};
const Segment SGI119546[] = {{&S95508[138], 13}, {NULL}};
const Segment SGI119547[] = {{&S6079[44], 6}, {NULL}};
const Segment SGI119548[] = {{&S6079[44], 18}, {NULL}};
const Segment SGI119549[] = {{&S6079[30], 6}, {NULL}};
const Segment SGI119550[] = {{&S6079[31], 3}, {NULL}};
const Segment SGI119551[] = {{&S6079[76], 6}, {NULL}};
const Segment SGI119552[] = {{&S95575[59], 8}, {NULL}};
const Segment SGI119553[] = {{&S95575[59], 20}, {NULL}};
const Segment SGI119554[] = {{&S95575[45], 6}, {NULL}};
const Segment SGI119555[] = {{&S95575[46], 3}, {NULL}};
const Segment SGI119556[] = {{&S95575[93], 6}, {NULL}};
const Segment SGI119557[] = {{&S95575[0], 14}, {NULL}};
const Segment SGI119558[] = {{&S95576[45], 9}, {NULL}};
const Segment SGI119559[] = {{&S95576[30], 6}, {NULL}};
const Segment SGI119560[] = {{&S95576[31], 3}, {NULL}};
const Segment SGI119561[] = {{&S95576[80], 5}, {NULL}};
const Segment SGI119562[] = {{&S95576[138], 13}, {NULL}};
const Segment SGI119563[] = {{&S95583[44], 11}, {NULL}};
const Segment SGI119564[] = {{&S95583[44], 23}, {NULL}};
const Segment SGI119565[] = {{&S95583[30], 6}, {NULL}};
const Segment SGI119566[] = {{&S95583[81], 6}, {NULL}};
const Segment SGI119567[] = {{&S95583[108], 13}, {NULL}};
const Segment SGI119568[] = {{&S6152[44], 5}, {NULL}};
const Segment SGI119569[] = {{&S6152[44], 17}, {NULL}};
const Segment SGI119570[] = {{&S6152[30], 6}, {NULL}};
const Segment SGI119571[] = {{&S6152[31], 3}, {NULL}};
const Segment SGI119572[] = {{&S6152[75], 6}, {NULL}};
const Segment SGI119573[] = {{&S6153[44], 4}, {NULL}};
const Segment SGI119574[] = {{&S6153[44], 15}, {NULL}};
const Segment SGI119575[] = {{&S6153[30], 6}, {NULL}};
const Segment SGI119576[] = {{&S6153[31], 3}, {NULL}};
const Segment SGI119577[] = {{&S6153[73], 5}, {NULL}};
const Segment SGI119578[] = {{&S95611[34], 8}, {NULL}};
const Segment SGI119579[] = {{&S95611[27], 6}, {NULL}};
const Segment SGI119580[] = {{&S95611[95], 9}, {NULL}};
const Segment SGI119581[] = {{&S95611[68], 6}, {NULL}};
const Segment SGI119582[] = {{&S95618[34], 7}, {NULL}};
const Segment SGI119583[] = {{&S95618[27], 6}, {NULL}};
const Segment SGI119584[] = {{&S95618[94], 9}, {NULL}};
const Segment SGI119585[] = {{&S95618[67], 6}, {NULL}};
const Segment SGI119586[] = {{&S95625[44], 9}, {NULL}};
const Segment SGI119587[] = {{&S95625[44], 21}, {NULL}};
const Segment SGI119588[] = {{&S95625[30], 6}, {NULL}};
const Segment SGI119589[] = {{&S95625[79], 6}, {NULL}};
const Segment SGI119590[] = {{&S95625[106], 13}, {NULL}};
const Segment SGI119591[] = {{&S95658[34], 9}, {NULL}};
const Segment SGI119592[] = {{&S95658[27], 6}, {NULL}};
const Segment SGI119593[] = {{&S95658[96], 9}, {NULL}};
const Segment SGI119594[] = {{&S95658[69], 6}, {NULL}};
const Segment SGI119595[] = {{&S95658[137], 8}, {NULL}};
const Segment SGI119596[] = {{&S95734[34], 24}, {NULL}};
const Segment SGI119597[] = {{&S95734[27], 6}, {NULL}};
const Segment SGI119598[] = {{&S95734[111], 9}, {NULL}};
const Segment SGI119599[] = {{&S95734[84], 6}, {NULL}};
const Segment SGI119600[] = {{&S95776[45], 13}, {NULL}};
const Segment SGI119601[] = {{&S95776[30], 6}, {NULL}};
const Segment SGI119602[] = {{&S95776[31], 3}, {NULL}};
const Segment SGI119603[] = {{&S95776[88], 5}, {NULL}};
const Segment SGI119604[] = {{&S95776[146], 13}, {NULL}};
const Segment SGI119605[] = {{&S95778[34], 13}, {NULL}};
const Segment SGI119606[] = {{&S95778[27], 6}, {NULL}};
const Segment SGI119607[] = {{&S95778[100], 9}, {NULL}};
const Segment SGI119608[] = {{&S95778[73], 6}, {NULL}};
const Segment SGI119609[] = {{&S6263[44], 21}, {NULL}};
const Segment SGI119610[] = {{&S6263[44], 33}, {NULL}};
const Segment SGI119611[] = {{&S6263[30], 6}, {&S6263[52], 25}, {NULL}};
const Segment SGI119612[] = {{&S6263[31], 3}, {NULL}};
const Segment SGI119613[] = {{&S6263[91], 5}, {NULL}};
const Segment SGI119614[] = {{&S95791[45], 9}, {NULL}};
const Segment SGI119615[] = {{&S95791[30], 6}, {NULL}};
const Segment SGI119616[] = {{&S95791[31], 3}, {NULL}};
const Segment SGI119617[] = {{&S95791[80], 5}, {NULL}};
const Segment SGI119618[] = {{&S95791[138], 13}, {NULL}};
const Segment SGI119619[] = {{&S95793[45], 14}, {NULL}};
const Segment SGI119620[] = {{&S95793[30], 6}, {NULL}};
const Segment SGI119621[] = {{&S95793[31], 3}, {NULL}};
const Segment SGI119622[] = {{&S95793[90], 5}, {NULL}};
const Segment SGI119623[] = {{&S95793[148], 13}, {NULL}};
const Segment SGI119624[] = {{&S95799[44], 14}, {NULL}};
const Segment SGI119625[] = {{&S95799[44], 26}, {NULL}};
const Segment SGI119626[] = {{&S95799[30], 6}, {NULL}};
const Segment SGI119627[] = {{&S95799[31], 3}, {NULL}};
const Segment SGI119628[] = {{&S95799[84], 6}, {NULL}};
const Segment SGI119629[] = {{&S95807[34], 8}, {NULL}};
const Segment SGI119630[] = {{&S95807[27], 6}, {NULL}};
const Segment SGI119631[] = {{&S95807[95], 9}, {NULL}};
const Segment SGI119632[] = {{&S95807[68], 6}, {NULL}};
const Segment SGI119633[] = {{&S12313[44], 20}, {NULL}};
const Segment SGI119634[] = {{&S12313[44], 32}, {NULL}};
const Segment SGI119635[] = {{&S12313[30], 6}, {NULL}};
const Segment SGI119636[] = {{&S12313[31], 3}, {NULL}};
const Segment SGI119637[] = {{&S12313[90], 6}, {NULL}};
const Segment SGI119638[] = {{&S95820[44], 11}, {NULL}};
const Segment SGI119639[] = {{&S95820[44], 23}, {NULL}};
const Segment SGI119640[] = {{&S95820[30], 6}, {NULL}};
const Segment SGI119641[] = {{&S95820[31], 3}, {NULL}};
const Segment SGI119642[] = {{&S95820[81], 5}, {NULL}};
const Segment SGI119643[] = {{&S95820[118], 14}, {NULL}};
const Segment SGI119644[] = {{&S12326[44], 11}, {NULL}};
const Segment SGI119645[] = {{&S12326[44], 23}, {NULL}};
const Segment SGI119646[] = {{&S12326[30], 6}, {NULL}};
const Segment SGI119647[] = {{&S12326[31], 3}, {NULL}};
const Segment SGI119648[] = {{&S12326[81], 6}, {NULL}};
const Segment SGI119649[] = {{&S2974[44], 10}, {NULL}};
const Segment SGI119650[] = {{&S2974[44], 22}, {NULL}};
const Segment SGI119651[] = {{&S2974[30], 6}, {NULL}};
const Segment SGI119652[] = {{&S2974[115], 3}, {NULL}};
const Segment SGI119653[] = {{&S2974[80], 6}, {NULL}};
const Segment SGI119654[] = {{&S12330[44], 11}, {NULL}};
const Segment SGI119655[] = {{&S2976[44], 10}, {NULL}};
const Segment SGI119656[] = {{&S12330[44], 23}, {NULL}};
const Segment SGI119657[] = {{&S2976[44], 22}, {NULL}};
const Segment SGI119658[] = {{&S12330[30], 6}, {NULL}};
const Segment SGI119659[] = {{&S2976[30], 6}, {NULL}};
const Segment SGI119660[] = {{&S12330[31], 3}, {NULL}};
const Segment SGI119661[] = {{&S2976[115], 3}, {NULL}};
const Segment SGI119662[] = {{&S12330[81], 6}, {NULL}};
const Segment SGI119663[] = {{&S2976[80], 6}, {NULL}};
const Segment SGI119664[] = {{&S2977[38], 10}, {NULL}};
const Segment SGI119665[] = {{&S2977[31], 6}, {NULL}};
const Segment SGI119666[] = {{&S12334[38], 11}, {NULL}};
const Segment SGI119667[] = {{&S2980[44], 10}, {NULL}};
const Segment SGI119668[] = {{&S12334[31], 6}, {NULL}};
const Segment SGI119669[] = {{&S2980[44], 22}, {NULL}};
const Segment SGI119670[] = {{&S2980[30], 6}, {NULL}};
const Segment SGI119671[] = {{&S2980[115], 3}, {NULL}};
const Segment SGI119672[] = {{&S2980[80], 6}, {NULL}};
const Segment SGI119673[] = {{&S12335[44], 20}, {NULL}};
const Segment SGI119674[] = {{&S12335[44], 32}, {NULL}};
const Segment SGI119675[] = {{&S12335[30], 6}, {NULL}};
const Segment SGI119676[] = {{&S12335[31], 3}, {NULL}};
const Segment SGI119677[] = {{&S12335[90], 6}, {NULL}};
const Segment SGI119678[] = {{&S12336[44], 20}, {NULL}};
const Segment SGI119679[] = {{&S12336[44], 32}, {NULL}};
const Segment SGI119680[] = {{&S12336[30], 6}, {NULL}};
const Segment SGI119681[] = {{&S12336[31], 3}, {NULL}};
const Segment SGI119682[] = {{&S2984[44], 10}, {NULL}};
const Segment SGI119683[] = {{&S12336[90], 6}, {NULL}};
const Segment SGI119684[] = {{&S2984[44], 22}, {NULL}};
const Segment SGI119685[] = {{&S2984[30], 6}, {NULL}};
const Segment SGI119686[] = {{&S2984[115], 3}, {NULL}};
const Segment SGI119687[] = {{&S2984[80], 6}, {NULL}};
const Segment SGI119688[] = {{&S12338[44], 11}, {NULL}};
const Segment SGI119689[] = {{&S12338[44], 23}, {NULL}};
const Segment SGI119690[] = {{&S12338[30], 6}, {NULL}};
const Segment SGI119691[] = {{&S12338[84], 6}, {NULL}};
const Segment SGI119692[] = {{&S95848[44], 11}, {NULL}};
const Segment SGI119693[] = {{&S95848[44], 23}, {NULL}};
const Segment SGI119694[] = {{&S95848[30], 6}, {NULL}};
const Segment SGI119695[] = {{&S95848[81], 6}, {NULL}};
const Segment SGI119696[] = {{&S95848[108], 13}, {NULL}};
const Segment SGI119697[] = {{&S12340[44], 7}, {NULL}};
const Segment SGI119698[] = {{&S2989[38], 10}, {NULL}};
const Segment SGI119699[] = {{&S12340[44], 19}, {NULL}};
const Segment SGI119700[] = {{&S2989[31], 6}, {NULL}};
const Segment SGI119701[] = {{&S12340[30], 6}, {NULL}};
const Segment SGI119702[] = {{&S12340[31], 3}, {NULL}};
const Segment SGI119703[] = {{&S12340[77], 6}, {NULL}};
const Segment SGI119704[] = {{&S12342[44], 20}, {NULL}};
const Segment SGI119705[] = {{&S12342[44], 32}, {NULL}};
const Segment SGI119706[] = {{&S12342[30], 6}, {NULL}};
const Segment SGI119707[] = {{&S12342[31], 3}, {NULL}};
const Segment SGI119708[] = {{&S12342[90], 6}, {NULL}};
const Segment SGI119709[] = {{&S2994[44], 30}, {NULL}};
const Segment SGI119710[] = {{&S2994[44], 42}, {NULL}};
const Segment SGI119711[] = {{&S2994[30], 6}, {NULL}};
const Segment SGI119712[] = {{&S2994[135], 3}, {NULL}};
const Segment SGI119713[] = {{&S2994[100], 6}, {NULL}};
const Segment SGI119714[] = {{&S12345[44], 11}, {NULL}};
const Segment SGI119715[] = {{&S12345[44], 23}, {NULL}};
const Segment SGI119716[] = {{&S12345[30], 6}, {NULL}};
const Segment SGI119717[] = {{&S12345[31], 3}, {NULL}};
const Segment SGI119718[] = {{&S12345[81], 6}, {NULL}};
const Segment SGI119719[] = {{&S12346[44], 20}, {NULL}};
const Segment SGI119720[] = {{&S2997[44], 10}, {NULL}};
const Segment SGI119721[] = {{&S12346[44], 32}, {NULL}};
const Segment SGI119722[] = {{&S2997[44], 22}, {NULL}};
const Segment SGI119723[] = {{&S12346[30], 6}, {NULL}};
const Segment SGI119724[] = {{&S2997[30], 6}, {NULL}};
const Segment SGI119725[] = {{&S12346[31], 3}, {NULL}};
const Segment SGI119726[] = {{&S2997[83], 6}, {NULL}};
const Segment SGI119727[] = {{&S12346[90], 6}, {NULL}};
const Segment SGI119728[] = {{&S2999[44], 29}, {NULL}};
const Segment SGI119729[] = {{&S2999[44], 41}, {NULL}};
const Segment SGI119730[] = {{&S2999[30], 6}, {NULL}};
const Segment SGI119731[] = {{&S12348[44], 22}, {NULL}};
const Segment SGI119732[] = {{&S12348[44], 34}, {NULL}};
const Segment SGI119733[] = {{&S12348[30], 6}, {NULL}};
const Segment SGI119734[] = {{&S2999[134], 3}, {NULL}};
const Segment SGI119735[] = {{&S2999[99], 6}, {NULL}};
const Segment SGI119736[] = {{&S12348[31], 3}, {NULL}};
const Segment SGI119737[] = {{&S12348[92], 6}, {NULL}};
const Segment SGI119738[] = {{&S12349[44], 21}, {NULL}};
const Segment SGI119739[] = {{&S12349[44], 33}, {NULL}};
const Segment SGI119740[] = {{&S12349[30], 6}, {NULL}};
const Segment SGI119741[] = {{&S12349[31], 3}, {NULL}};
const Segment SGI119742[] = {{&S12349[91], 6}, {NULL}};
const Segment SGI119743[] = {{&S3003[44], 30}, {NULL}};
const Segment SGI119744[] = {{&S3003[44], 42}, {NULL}};
const Segment SGI119745[] = {{&S3003[30], 6}, {NULL}};
const Segment SGI119746[] = {{&S12350[44], 22}, {NULL}};
const Segment SGI119747[] = {{&S12350[44], 34}, {NULL}};
const Segment SGI119748[] = {{&S12350[30], 6}, {NULL}};
const Segment SGI119749[] = {{&S3003[135], 3}, {NULL}};
const Segment SGI119750[] = {{&S3003[100], 6}, {NULL}};
const Segment SGI119751[] = {{&S12350[31], 3}, {NULL}};
const Segment SGI119752[] = {{&S12350[92], 6}, {NULL}};
const Segment SGI119753[] = {{&S3004[44], 30}, {NULL}};
const Segment SGI119754[] = {{&S3004[44], 42}, {NULL}};
const Segment SGI119755[] = {{&S12351[44], 22}, {NULL}};
const Segment SGI119756[] = {{&S3004[30], 6}, {NULL}};
const Segment SGI119757[] = {{&S12351[44], 34}, {NULL}};
const Segment SGI119758[] = {{&S12351[30], 6}, {NULL}};
const Segment SGI119759[] = {{&S3004[135], 3}, {NULL}};
const Segment SGI119760[] = {{&S12351[31], 3}, {NULL}};
const Segment SGI119761[] = {{&S3004[100], 6}, {NULL}};
const Segment SGI119762[] = {{&S12351[92], 6}, {NULL}};
const Segment SGI119763[] = {{&S95871[44], 11}, {NULL}};
const Segment SGI119764[] = {{&S95871[44], 23}, {NULL}};
const Segment SGI119765[] = {{&S95871[30], 6}, {NULL}};
const Segment SGI119766[] = {{&S95871[81], 6}, {NULL}};
const Segment SGI119767[] = {{&S95871[108], 13}, {NULL}};
const Segment SGI119768[] = {{&S3007[42], 10}, {NULL}};
const Segment SGI119769[] = {{&S3007[30], 6}, {NULL}};
const Segment SGI119770[] = {{&S3007[113], 3}, {NULL}};
const Segment SGI119771[] = {{&S3007[78], 6}, {NULL}};
const Segment SGI119772[] = {{&S3011[44], 29}, {NULL}};
const Segment SGI119773[] = {{&S3011[44], 41}, {NULL}};
const Segment SGI119774[] = {{&S3011[30], 6}, {NULL}};
const Segment SGI119775[] = {{&S3011[134], 3}, {NULL}};
const Segment SGI119776[] = {{&S3011[99], 6}, {NULL}};
const Segment SGI119777[] = {{&S3012[38], 10}, {NULL}};
const Segment SGI119778[] = {{&S3012[31], 6}, {NULL}};
const Segment SGI119779[] = {{&S3013[44], 10}, {NULL}};
const Segment SGI119780[] = {{&S3013[44], 22}, {NULL}};
const Segment SGI119781[] = {{&S3013[30], 6}, {NULL}};
const Segment SGI119782[] = {{&S3013[115], 3}, {NULL}};
const Segment SGI119783[] = {{&S3013[80], 6}, {NULL}};
const Segment SGI119784[] = {{&S3014[44], 11}, {NULL}};
const Segment SGI119785[] = {{&S3014[44], 23}, {NULL}};
const Segment SGI119786[] = {{&S3014[30], 6}, {NULL}};
const Segment SGI119787[] = {{&S3014[116], 3}, {NULL}};
const Segment SGI119788[] = {{&S3014[81], 6}, {NULL}};
const Segment SGI119789[] = {{&S3015[38], 11}, {NULL}};
const Segment SGI119790[] = {{&S3015[31], 6}, {NULL}};
const Segment SGI119791[] = {{&S3016[44], 30}, {NULL}};
const Segment SGI119792[] = {{&S3016[44], 42}, {NULL}};
const Segment SGI119793[] = {{&S3016[30], 6}, {NULL}};
const Segment SGI119794[] = {{&S3016[135], 3}, {NULL}};
const Segment SGI119795[] = {{&S3016[100], 6}, {NULL}};
const Segment SGI119796[] = {{&S3021[44], 30}, {NULL}};
const Segment SGI119797[] = {{&S3021[44], 42}, {NULL}};
const Segment SGI119798[] = {{&S3021[30], 6}, {NULL}};
const Segment SGI119799[] = {{&S3021[135], 3}, {NULL}};
const Segment SGI119800[] = {{&S3021[100], 6}, {NULL}};
const Segment SGI119801[] = {{&S6372[44], 7}, {NULL}};
const Segment SGI119802[] = {{&S6372[44], 19}, {NULL}};
const Segment SGI119803[] = {{&S6372[30], 6}, {NULL}};
const Segment SGI119804[] = {{&S6372[31], 3}, {NULL}};
const Segment SGI119805[] = {{&S6372[77], 6}, {NULL}};
const Segment SGI119806[] = {{&S3024[44], 10}, {NULL}};
const Segment SGI119807[] = {{&S3024[44], 22}, {NULL}};
const Segment SGI119808[] = {{&S3024[30], 6}, {NULL}};
const Segment SGI119809[] = {{&S3024[80], 6}, {NULL}};
const Segment SGI119810[] = {{&S3025[44], 11}, {NULL}};
const Segment SGI119811[] = {{&S3025[44], 23}, {NULL}};
const Segment SGI119812[] = {{&S3025[30], 6}, {NULL}};
const Segment SGI119813[] = {{&S3025[116], 3}, {NULL}};
const Segment SGI119814[] = {{&S3025[81], 6}, {NULL}};
const Segment SGI119815[] = {{&S3026[44], 10}, {NULL}};
const Segment SGI119816[] = {{&S3026[44], 22}, {NULL}};
const Segment SGI119817[] = {{&S3026[30], 6}, {NULL}};
const Segment SGI119818[] = {{&S3026[115], 3}, {NULL}};
const Segment SGI119819[] = {{&S3026[80], 6}, {NULL}};
const Segment SGI119820[] = {{&S3030[42], 10}, {NULL}};
const Segment SGI119821[] = {{&S3030[30], 6}, {NULL}};
const Segment SGI119822[] = {{&S3030[113], 3}, {NULL}};
const Segment SGI119823[] = {{&S3030[78], 6}, {NULL}};
const Segment SGI119824[] = {{&S3034[44], 11}, {NULL}};
const Segment SGI119825[] = {{&S3034[44], 23}, {NULL}};
const Segment SGI119826[] = {{&S3034[30], 6}, {NULL}};
const Segment SGI119827[] = {{&S3034[116], 3}, {NULL}};
const Segment SGI119828[] = {{&S3034[81], 6}, {NULL}};
const Segment SGI119829[] = {{&S3035[44], 10}, {NULL}};
const Segment SGI119830[] = {{&S3035[44], 22}, {NULL}};
const Segment SGI119831[] = {{&S3035[30], 6}, {NULL}};
const Segment SGI119832[] = {{&S3035[83], 6}, {NULL}};
const Segment SGI119833[] = {{&S3036[44], 29}, {NULL}};
const Segment SGI119834[] = {{&S3036[44], 41}, {NULL}};
const Segment SGI119835[] = {{&S3036[30], 6}, {NULL}};
const Segment SGI119836[] = {{&S3036[134], 3}, {NULL}};
const Segment SGI119837[] = {{&S3036[99], 6}, {NULL}};
const Segment SGI119838[] = {{&S3037[44], 30}, {NULL}};
const Segment SGI119839[] = {{&S95924[44], 12}, {NULL}};
const Segment SGI119840[] = {{&S3037[44], 42}, {NULL}};
const Segment SGI119841[] = {{&S95924[44], 33}, {NULL}};
const Segment SGI119842[] = {{&S3037[30], 6}, {NULL}};
const Segment SGI119843[] = {{&S95924[30], 6}, {NULL}};
const Segment SGI119844[] = {{&S3037[135], 3}, {NULL}};
const Segment SGI119845[] = {{&S95924[126], 3}, {NULL}};
const Segment SGI119846[] = {{&S3037[100], 6}, {NULL}};
const Segment SGI119847[] = {{&S95924[91], 6}, {NULL}};
const Segment SGI119848[] = {{&S95924[146], 8}, {NULL}};
const Segment SGI119849[] = {{&S95924[155], 13}, {NULL}};
const Segment SGI119850[] = {{&S3038[44], 29}, {NULL}};
const Segment SGI119851[] = {{&S3038[44], 41}, {NULL}};
const Segment SGI119852[] = {{&S3038[30], 6}, {NULL}};
const Segment SGI119853[] = {{&S3038[134], 3}, {NULL}};
const Segment SGI119854[] = {{&S3038[99], 6}, {NULL}};
const Segment SGI119855[] = {{&S3039[44], 10}, {NULL}};
const Segment SGI119856[] = {{&S3039[44], 35}, {NULL}};
const Segment SGI119857[] = {{&S3039[30], 6}, {NULL}};
const Segment SGI119858[] = {{&S3039[128], 3}, {NULL}};
const Segment SGI119859[] = {{&S3039[93], 6}, {NULL}};
const Segment SGI119860[] = {{&S3044[38], 10}, {NULL}};
const Segment SGI119861[] = {{&S3044[31], 6}, {NULL}};
const Segment SGI119862[] = {{&S3048[44], 12}, {NULL}};
const Segment SGI119863[] = {{&S3048[44], 24}, {NULL}};
const Segment SGI119864[] = {{&S3048[30], 6}, {NULL}};
const Segment SGI119865[] = {{&S3048[116], 3}, {NULL}};
const Segment SGI119866[] = {{&S3048[82], 5}, {NULL}};
const Segment SGI119867[] = {{&S3049[56], 10}, {NULL}};
const Segment SGI119868[] = {{&S3049[56], 22}, {NULL}};
const Segment SGI119869[] = {{&S3049[30], 18}, {NULL}};
const Segment SGI119870[] = {{&S3049[127], 3}, {NULL}};
const Segment SGI119871[] = {{&S3049[92], 6}, {NULL}};
const Segment SGI119872[] = {{&S3052[44], 30}, {NULL}};
const Segment SGI119873[] = {{&S3052[44], 42}, {NULL}};
const Segment SGI119874[] = {{&S3052[30], 6}, {NULL}};
const Segment SGI119875[] = {{&S3052[135], 3}, {NULL}};
const Segment SGI119876[] = {{&S3052[100], 6}, {NULL}};
const Segment SGI119877[] = {{&S12352[44], 22}, {NULL}};
const Segment SGI119878[] = {{&S12352[44], 34}, {NULL}};
const Segment SGI119879[] = {{&S12352[30], 6}, {NULL}};
const Segment SGI119880[] = {{&S12352[31], 3}, {NULL}};
const Segment SGI119881[] = {{&S3053[44], 30}, {NULL}};
const Segment SGI119882[] = {{&S3053[44], 42}, {NULL}};
const Segment SGI119883[] = {{&S12352[92], 6}, {NULL}};
const Segment SGI119884[] = {{&S3053[30], 6}, {NULL}};
const Segment SGI119885[] = {{&S3053[135], 3}, {NULL}};
const Segment SGI119886[] = {{&S3053[100], 6}, {NULL}};
const Segment SGI119887[] = {{&S12353[44], 22}, {NULL}};
const Segment SGI119888[] = {{&S12353[44], 34}, {NULL}};
const Segment SGI119889[] = {{&S12353[30], 6}, {NULL}};
const Segment SGI119890[] = {{&S12353[31], 3}, {NULL}};
const Segment SGI119891[] = {{&S12353[92], 6}, {NULL}};
const Segment SGI119892[] = {{&S12354[44], 12}, {NULL}};
const Segment SGI119893[] = {{&S12354[44], 24}, {NULL}};
const Segment SGI119894[] = {{&S12354[30], 6}, {NULL}};
const Segment SGI119895[] = {{&S12354[31], 3}, {NULL}};
const Segment SGI119896[] = {{&S3056[42], 10}, {NULL}};
const Segment SGI119897[] = {{&S12354[82], 6}, {NULL}};
const Segment SGI119898[] = {{&S3056[30], 6}, {NULL}};
const Segment SGI119899[] = {{&S3056[113], 3}, {NULL}};
const Segment SGI119900[] = {{&S3056[78], 6}, {NULL}};
const Segment SGI119901[] = {{&S12355[44], 22}, {NULL}};
const Segment SGI119902[] = {{&S12355[44], 34}, {NULL}};
const Segment SGI119903[] = {{&S12355[30], 6}, {NULL}};
const Segment SGI119904[] = {{&S12355[31], 3}, {NULL}};
const Segment SGI119905[] = {{&S12355[92], 6}, {NULL}};
const Segment SGI119906[] = {{&S12356[44], 22}, {NULL}};
const Segment SGI119907[] = {{&S12356[44], 34}, {NULL}};
const Segment SGI119908[] = {{&S12356[30], 6}, {NULL}};
const Segment SGI119909[] = {{&S12356[31], 3}, {NULL}};
const Segment SGI119910[] = {{&S3059[44], 30}, {NULL}};
const Segment SGI119911[] = {{&S12356[92], 6}, {NULL}};
const Segment SGI119912[] = {{&S3059[44], 42}, {NULL}};
const Segment SGI119913[] = {{&S3059[30], 6}, {NULL}};
const Segment SGI119914[] = {{&S3059[135], 3}, {NULL}};
const Segment SGI119915[] = {{&S3059[100], 6}, {NULL}};
const Segment SGI119916[] = {{&S12357[44], 7}, {NULL}};
const Segment SGI119917[] = {{&S12357[44], 19}, {NULL}};
const Segment SGI119918[] = {{&S12357[30], 6}, {NULL}};
const Segment SGI119919[] = {{&S12357[31], 3}, {NULL}};
const Segment SGI119920[] = {{&S12357[77], 6}, {NULL}};
const Segment SGI119921[] = {{&S12358[44], 22}, {NULL}};
const Segment SGI119922[] = {{&S12358[44], 34}, {NULL}};
const Segment SGI119923[] = {{&S12358[30], 6}, {NULL}};
const Segment SGI119924[] = {{&S12358[31], 3}, {NULL}};
const Segment SGI119925[] = {{&S12358[92], 6}, {NULL}};
const Segment SGI119926[] = {{&S3064[38], 11}, {NULL}};
const Segment SGI119927[] = {{&S3064[31], 6}, {NULL}};
const Segment SGI119928[] = {{&S12359[44], 11}, {NULL}};
const Segment SGI119929[] = {{&S12359[44], 23}, {NULL}};
const Segment SGI119930[] = {{&S12359[30], 6}, {NULL}};
const Segment SGI119931[] = {{&S12359[81], 6}, {NULL}};
const Segment SGI119932[] = {{&S12360[44], 11}, {NULL}};
const Segment SGI119933[] = {{&S12360[44], 23}, {NULL}};
const Segment SGI119934[] = {{&S12360[30], 6}, {NULL}};
const Segment SGI119935[] = {{&S3066[44], 10}, {NULL}};
const Segment SGI119936[] = {{&S3066[44], 22}, {NULL}};
const Segment SGI119937[] = {{&S12360[84], 6}, {NULL}};
const Segment SGI119938[] = {{&S3066[30], 6}, {NULL}};
const Segment SGI119939[] = {{&S3066[115], 3}, {NULL}};
const Segment SGI119940[] = {{&S3066[80], 6}, {NULL}};
const Segment SGI119941[] = {{&S12361[42], 11}, {NULL}};
const Segment SGI119942[] = {{&S12361[30], 6}, {NULL}};
const Segment SGI119943[] = {{&S12361[31], 3}, {NULL}};
const Segment SGI119944[] = {{&S12361[79], 6}, {NULL}};
const Segment SGI119945[] = {{&S3067[44], 30}, {NULL}};
const Segment SGI119946[] = {{&S3067[44], 42}, {NULL}};
const Segment SGI119947[] = {{&S3067[30], 6}, {NULL}};
const Segment SGI119948[] = {{&S3067[135], 3}, {NULL}};
const Segment SGI119949[] = {{&S3067[100], 6}, {NULL}};
const Segment SGI119950[] = {{&S12362[44], 23}, {NULL}};
const Segment SGI119951[] = {{&S12362[44], 35}, {NULL}};
const Segment SGI119952[] = {{&S12362[30], 6}, {NULL}};
const Segment SGI119953[] = {{&S12362[31], 3}, {NULL}};
const Segment SGI119954[] = {{&S12362[93], 6}, {NULL}};
const Segment SGI119955[] = {{&S3068[44], 8}, {NULL}};
const Segment SGI119956[] = {{&S3068[44], 20}, {NULL}};
const Segment SGI119957[] = {{&S3068[30], 6}, {NULL}};
const Segment SGI119958[] = {{&S3068[113], 3}, {NULL}};
const Segment SGI119959[] = {{&S3068[78], 6}, {NULL}};
const Segment SGI119960[] = {{&S12363[42], 11}, {NULL}};
const Segment SGI119961[] = {{&S12363[30], 6}, {NULL}};
const Segment SGI119962[] = {{&S12363[31], 3}, {NULL}};
const Segment SGI119963[] = {{&S12363[79], 6}, {NULL}};
const Segment SGI119964[] = {{&S3069[38], 8}, {NULL}};
const Segment SGI119965[] = {{&S3069[31], 6}, {NULL}};
const Segment SGI119966[] = {{&S3071[44], 11}, {NULL}};
const Segment SGI119967[] = {{&S3071[44], 23}, {NULL}};
const Segment SGI119968[] = {{&S3071[30], 6}, {NULL}};
const Segment SGI119969[] = {{&S3071[116], 3}, {NULL}};
const Segment SGI119970[] = {{&S3071[81], 6}, {NULL}};
const Segment SGI119971[] = {{&S3072[44], 29}, {NULL}};
const Segment SGI119972[] = {{&S3072[44], 41}, {NULL}};
const Segment SGI119973[] = {{&S3072[30], 6}, {NULL}};
const Segment SGI119974[] = {{&S3072[134], 3}, {NULL}};
const Segment SGI119975[] = {{&S3072[99], 6}, {NULL}};
const Segment SGI119976[] = {{&S3073[44], 30}, {NULL}};
const Segment SGI119977[] = {{&S3073[44], 42}, {NULL}};
const Segment SGI119978[] = {{&S3073[30], 6}, {NULL}};
const Segment SGI119979[] = {{&S3073[135], 3}, {NULL}};
const Segment SGI119980[] = {{&S3073[100], 6}, {NULL}};
const Segment SGI119981[] = {{&S12371[44], 9}, {NULL}};
const Segment SGI119982[] = {{&S12371[44], 21}, {NULL}};
const Segment SGI119983[] = {{&S12371[30], 6}, {NULL}};
const Segment SGI119984[] = {{&S12371[31], 3}, {NULL}};
const Segment SGI119985[] = {{&S12371[79], 6}, {NULL}};
const Segment SGI119986[] = {{&S3075[44], 11}, {NULL}};
const Segment SGI119987[] = {{&S3075[44], 23}, {NULL}};
const Segment SGI119988[] = {{&S3075[30], 6}, {NULL}};
const Segment SGI119989[] = {{&S3075[116], 3}, {NULL}};
const Segment SGI119990[] = {{&S3075[81], 6}, {NULL}};
const Segment SGI119991[] = {{&S3076[38], 11}, {NULL}};
const Segment SGI119992[] = {{&S3076[31], 6}, {NULL}};
const Segment SGI119993[] = {{&S12374[44], 20}, {NULL}};
const Segment SGI119994[] = {{&S12374[44], 32}, {NULL}};
const Segment SGI119995[] = {{&S12374[30], 6}, {NULL}};
const Segment SGI119996[] = {{&S12374[31], 3}, {NULL}};
const Segment SGI119997[] = {{&S12374[90], 6}, {NULL}};
const Segment SGI119998[] = {{&S3077[44], 30}, {NULL}};
const Segment SGI119999[] = {{&S3077[44], 42}, {NULL}};
const Segment SGI120000[] = {{&S3077[30], 6}, {NULL}};
const Segment SGI120001[] = {{&S3077[135], 3}, {NULL}};
const Segment SGI120002[] = {{&S3077[100], 6}, {NULL}};
const Segment SGI120003[] = {{&S12375[44], 19}, {NULL}};
const Segment SGI120004[] = {{&S12375[44], 31}, {NULL}};
const Segment SGI120005[] = {{&S12375[30], 6}, {NULL}};
const Segment SGI120006[] = {{&S12375[31], 3}, {NULL}};
const Segment SGI120007[] = {{&S12375[89], 6}, {NULL}};
const Segment SGI120008[] = {{&S3078[44], 30}, {NULL}};
const Segment SGI120009[] = {{&S3078[44], 42}, {NULL}};
const Segment SGI120010[] = {{&S3078[30], 6}, {NULL}};
const Segment SGI120011[] = {{&S3078[135], 3}, {NULL}};
const Segment SGI120012[] = {{&S3078[100], 6}, {NULL}};
const Segment SGI120013[] = {{&S95993[44], 10}, {NULL}};
const Segment SGI120014[] = {{&S95993[44], 20}, {NULL}};
const Segment SGI120015[] = {{&S95993[30], 6}, {NULL}};
const Segment SGI120016[] = {{&S95993[31], 3}, {NULL}};
const Segment SGI120017[] = {{&S12377[44], 20}, {NULL}};
const Segment SGI120018[] = {{&S95993[78], 5}, {NULL}};
const Segment SGI120019[] = {{&S12377[44], 32}, {NULL}};
const Segment SGI120020[] = {{&S12377[30], 6}, {NULL}};
const Segment SGI120021[] = {{&S12377[31], 3}, {NULL}};
const Segment SGI120022[] = {{&S12377[90], 6}, {NULL}};
const Segment SGI120023[] = {{&S3080[38], 11}, {NULL}};
const Segment SGI120024[] = {{&S3080[31], 6}, {NULL}};
const Segment SGI120025[] = {{&S12378[44], 21}, {NULL}};
const Segment SGI120026[] = {{&S12378[44], 33}, {NULL}};
const Segment SGI120027[] = {{&S12378[30], 6}, {NULL}};
const Segment SGI120028[] = {{&S3081[44], 10}, {NULL}};
const Segment SGI120029[] = {{&S12378[31], 3}, {NULL}};
const Segment SGI120030[] = {{&S3081[44], 22}, {NULL}};
const Segment SGI120031[] = {{&S3081[30], 6}, {NULL}};
const Segment SGI120032[] = {{&S12378[91], 6}, {NULL}};
const Segment SGI120033[] = {{&S3081[127], 3}, {NULL}};
const Segment SGI120034[] = {{&S3081[80], 6}, {NULL}};
const Segment SGI120035[] = {{&S6441[44], 5}, {NULL}};
const Segment SGI120036[] = {{&S6441[44], 17}, {NULL}};
const Segment SGI120037[] = {{&S6441[30], 6}, {NULL}};
const Segment SGI120038[] = {{&S6441[31], 3}, {NULL}};
const Segment SGI120039[] = {{&S6441[75], 6}, {NULL}};
const Segment SGI120040[] = {{&S3083[44], 10}, {NULL}};
const Segment SGI120041[] = {{&S3083[44], 22}, {NULL}};
const Segment SGI120042[] = {{&S3083[30], 6}, {NULL}};
const Segment SGI120043[] = {{&S3083[80], 6}, {NULL}};
const Segment SGI120044[] = {{&S3084[38], 11}, {NULL}};
const Segment SGI120045[] = {{&S3084[31], 6}, {NULL}};
const Segment SGI120046[] = {{&S3085[42], 11}, {NULL}};
const Segment SGI120047[] = {{&S3085[30], 6}, {NULL}};
const Segment SGI120048[] = {{&S3085[114], 3}, {NULL}};
const Segment SGI120049[] = {{&S3085[79], 6}, {NULL}};
const Segment SGI120050[] = {{&S3087[42], 11}, {NULL}};
const Segment SGI120051[] = {{&S3087[30], 6}, {NULL}};
const Segment SGI120052[] = {{&S3087[114], 3}, {NULL}};
const Segment SGI120053[] = {{&S3087[79], 6}, {NULL}};
const Segment SGI120054[] = {{&S3088[44], 10}, {NULL}};
const Segment SGI120055[] = {{&S3088[44], 22}, {NULL}};
const Segment SGI120056[] = {{&S3088[30], 6}, {NULL}};
const Segment SGI120057[] = {{&S3088[115], 3}, {NULL}};
const Segment SGI120058[] = {{&S3088[80], 6}, {NULL}};
const Segment SGI120059[] = {{&S3089[42], 10}, {NULL}};
const Segment SGI120060[] = {{&S3089[30], 6}, {NULL}};
const Segment SGI120061[] = {{&S3089[125], 3}, {NULL}};
const Segment SGI120062[] = {{&S3089[78], 6}, {NULL}};
const Segment SGI120063[] = {{&S6474[44], 6}, {NULL}};
const Segment SGI120064[] = {{&S6474[44], 17}, {NULL}};
const Segment SGI120065[] = {{&S6474[30], 6}, {NULL}};
const Segment SGI120066[] = {{&S6474[31], 3}, {NULL}};
const Segment SGI120067[] = {{&S6474[75], 5}, {NULL}};
const Segment SGI120068[] = {{&S96037[34], 17}, {NULL}};
const Segment SGI120069[] = {{&S96037[27], 6}, {NULL}};
const Segment SGI120070[] = {{&S96037[117], 9}, {NULL}};
const Segment SGI120071[] = {{&S96037[90], 6}, {NULL}};
const Segment SGI120072[] = {{&S96040[44], 13}, {NULL}};
const Segment SGI120073[] = {{&S96040[44], 25}, {NULL}};
const Segment SGI120074[] = {{&S96040[30], 6}, {NULL}};
const Segment SGI120075[] = {{&S96040[31], 3}, {NULL}};
const Segment SGI120076[] = {{&S96040[83], 6}, {NULL}};
const Segment SGI120077[] = {{&S96042[31], 39}, {NULL}};
const Segment SGI120078[] = {{&S96042[85], 3}, {NULL}};
const Segment SGI120079[] = {{&S96044[34], 10}, {NULL}};
const Segment SGI120080[] = {{&S96044[27], 6}, {NULL}};
const Segment SGI120081[] = {{&S96044[97], 9}, {NULL}};
const Segment SGI120082[] = {{&S96044[70], 6}, {NULL}};
const Segment SGI120083[] = {{&S6493[44], 9}, {NULL}};
const Segment SGI120084[] = {{&S6493[44], 20}, {NULL}};
const Segment SGI120085[] = {{&S6493[30], 6}, {NULL}};
const Segment SGI120086[] = {{&S6493[31], 3}, {NULL}};
const Segment SGI120087[] = {{&S6493[78], 5}, {NULL}};
const Segment SGI120088[] = {{&S96061[59], 10}, {NULL}};
const Segment SGI120089[] = {{&S96061[59], 22}, {NULL}};
const Segment SGI120090[] = {{&S96061[45], 6}, {NULL}};
const Segment SGI120091[] = {{&S96061[46], 3}, {NULL}};
const Segment SGI120092[] = {{&S96061[95], 6}, {NULL}};
const Segment SGI120093[] = {{&S96061[0], 14}, {NULL}};
const Segment SGI120094[] = {{&S96064[44], 10}, {NULL}};
const Segment SGI120095[] = {{&S96064[44], 22}, {NULL}};
const Segment SGI120096[] = {{&S96064[30], 6}, {NULL}};
const Segment SGI120097[] = {{&S96064[31], 3}, {NULL}};
const Segment SGI120098[] = {{&S96064[80], 6}, {NULL}};
const Segment SGI120099[] = {{&S96064[132], 10}, {NULL}};
const Segment SGI120100[] = {{&S96066[44], 10}, {NULL}};
const Segment SGI120101[] = {{&S96066[44], 22}, {NULL}};
const Segment SGI120102[] = {{&S96066[30], 6}, {NULL}};
const Segment SGI120103[] = {{&S96066[31], 3}, {NULL}};
const Segment SGI120104[] = {{&S96066[80], 6}, {NULL}};
const Segment SGI120105[] = {{&S96066[133], 11}, {NULL}};
const Segment SGI120106[] = {{&S96076[59], 11}, {NULL}};
const Segment SGI120107[] = {{&S96076[59], 23}, {NULL}};
const Segment SGI120108[] = {{&S96076[45], 6}, {NULL}};
const Segment SGI120109[] = {{&S96076[46], 3}, {NULL}};
const Segment SGI120110[] = {{&S96076[96], 6}, {NULL}};
const Segment SGI120111[] = {{&S96076[0], 14}, {NULL}};
const Segment SGI120112[] = {{&S96080[45], 9}, {NULL}};
const Segment SGI120113[] = {{&S96080[30], 6}, {NULL}};
const Segment SGI120114[] = {{&S96080[31], 3}, {NULL}};
const Segment SGI120115[] = {{&S96080[80], 5}, {NULL}};
const Segment SGI120116[] = {{&S96080[138], 13}, {NULL}};
const Segment SGI120117[] = {{&S96092[44], 18}, {NULL}};
const Segment SGI120118[] = {{&S96092[44], 30}, {NULL}};
const Segment SGI120119[] = {{&S96092[30], 6}, {NULL}};
const Segment SGI120120[] = {{&S96092[88], 6}, {NULL}};
const Segment SGI120121[] = {{&S96092[115], 13}, {NULL}};
const Segment SGI120122[] = {{&S6557[44], 12}, {NULL}};
const Segment SGI120123[] = {{&S6557[44], 24}, {NULL}};
const Segment SGI120124[] = {{&S6557[30], 6}, {NULL}};
const Segment SGI120125[] = {{&S6557[31], 3}, {NULL}};
const Segment SGI120126[] = {{&S6557[82], 5}, {NULL}};
const Segment SGI120127[] = {{&S96116[34], 10}, {NULL}};
const Segment SGI120128[] = {{&S96116[27], 6}, {NULL}};
const Segment SGI120129[] = {{&S96116[97], 9}, {NULL}};
const Segment SGI120130[] = {{&S96116[70], 6}, {NULL}};
const Segment SGI120131[] = {{&S96137[44], 10}, {NULL}};
const Segment SGI120132[] = {{&S96137[44], 22}, {NULL}};
const Segment SGI120133[] = {{&S96137[30], 6}, {NULL}};
const Segment SGI120134[] = {{&S96137[31], 3}, {NULL}};
const Segment SGI120135[] = {{&S96137[80], 6}, {NULL}};
const Segment SGI120136[] = {{&S96160[34], 15}, {NULL}};
const Segment SGI120137[] = {{&S96160[27], 6}, {NULL}};
const Segment SGI120138[] = {{&S96160[102], 9}, {NULL}};
const Segment SGI120139[] = {{&S96160[75], 6}, {NULL}};
const Segment SGI120140[] = {{&S96160[136], 8}, {NULL}};
const Segment SGI120141[] = {{&S96162[45], 9}, {NULL}};
const Segment SGI120142[] = {{&S96162[30], 6}, {NULL}};
const Segment SGI120143[] = {{&S96162[31], 3}, {NULL}};
const Segment SGI120144[] = {{&S96162[80], 5}, {NULL}};
const Segment SGI120145[] = {{&S96162[138], 13}, {NULL}};
const Segment SGI120146[] = {{&S96164[34], 6}, {NULL}};
const Segment SGI120147[] = {{&S96164[27], 6}, {NULL}};
const Segment SGI120148[] = {{&S96164[93], 9}, {NULL}};
const Segment SGI120149[] = {{&S96164[66], 6}, {NULL}};
const Segment SGI120150[] = {{&S6630[44], 5}, {NULL}};
const Segment SGI120151[] = {{&S6630[44], 16}, {NULL}};
const Segment SGI120152[] = {{&S6630[30], 6}, {NULL}};
const Segment SGI120153[] = {{&S6630[31], 3}, {NULL}};
const Segment SGI120154[] = {{&S6630[74], 5}, {NULL}};
const Segment SGI120155[] = {{&S6656[44], 4}, {NULL}};
const Segment SGI120156[] = {{&S6656[44], 16}, {NULL}};
const Segment SGI120157[] = {{&S6656[30], 6}, {NULL}};
const Segment SGI120158[] = {{&S6656[31], 3}, {NULL}};
const Segment SGI120159[] = {{&S6656[74], 6}, {NULL}};
const Segment SGI120160[] = {{&S96196[45], 19}, {NULL}};
const Segment SGI120161[] = {{&S96196[30], 6}, {NULL}};
const Segment SGI120162[] = {{&S96196[31], 3}, {NULL}};
const Segment SGI120163[] = {{&S96196[90], 5}, {NULL}};
const Segment SGI120164[] = {{&S96196[148], 13}, {NULL}};
const Segment SGI120165[] = {{&S96210[34], 6}, {NULL}};
const Segment SGI120166[] = {{&S96210[27], 6}, {NULL}};
const Segment SGI120167[] = {{&S96210[104], 9}, {NULL}};
const Segment SGI120168[] = {{&S96210[77], 6}, {NULL}};
const Segment SGI120169[] = {{&S96214[34], 6}, {NULL}};
const Segment SGI120170[] = {{&S96214[27], 6}, {NULL}};
const Segment SGI120171[] = {{&S96214[93], 9}, {NULL}};
const Segment SGI120172[] = {{&S96214[66], 6}, {NULL}};
const Segment SGI120173[] = {{&S96214[127], 8}, {NULL}};
const Segment SGI120174[] = {{&S96220[44], 41}, {NULL}};
const Segment SGI120175[] = {{&S96220[44], 53}, {NULL}};
const Segment SGI120176[] = {{&S96220[30], 6}, {NULL}};
const Segment SGI120177[] = {{&S96220[31], 3}, {NULL}};
const Segment SGI120178[] = {{&S96220[111], 5}, {NULL}};
const Segment SGI120179[] = {{&S96220[148], 14}, {NULL}};
const Segment SGI120180[] = {{&S12662[44], 10}, {NULL}};
const Segment SGI120181[] = {{&S12662[44], 22}, {NULL}};
const Segment SGI120182[] = {{&S12662[30], 6}, {NULL}};
const Segment SGI120183[] = {{&S12662[31], 3}, {NULL}};
const Segment SGI120184[] = {{&S12662[80], 6}, {NULL}};
const Segment SGI120185[] = {{&S12663[44], 30}, {NULL}};
const Segment SGI120186[] = {{&S12663[44], 42}, {NULL}};
const Segment SGI120187[] = {{&S12663[30], 6}, {NULL}};
const Segment SGI120188[] = {{&S12663[31], 3}, {NULL}};
const Segment SGI120189[] = {{&S12663[100], 6}, {NULL}};
const Segment SGI120190[] = {{&S12664[38], 10}, {NULL}};
const Segment SGI120191[] = {{&S12664[31], 6}, {NULL}};
const Segment SGI120192[] = {{&S12665[44], 29}, {NULL}};
const Segment SGI120193[] = {{&S12665[44], 41}, {NULL}};
const Segment SGI120194[] = {{&S12665[30], 6}, {NULL}};
const Segment SGI120195[] = {{&S12665[31], 3}, {NULL}};
const Segment SGI120196[] = {{&S12665[99], 6}, {NULL}};
const Segment SGI120197[] = {{&S12667[44], 29}, {NULL}};
const Segment SGI120198[] = {{&S12667[44], 41}, {NULL}};
const Segment SGI120199[] = {{&S12667[30], 6}, {NULL}};
const Segment SGI120200[] = {{&S12667[31], 3}, {NULL}};
const Segment SGI120201[] = {{&S12667[99], 6}, {NULL}};
const Segment SGI120202[] = {{&S12668[42], 10}, {NULL}};
const Segment SGI120203[] = {{&S12668[30], 6}, {NULL}};
const Segment SGI120204[] = {{&S12668[31], 3}, {NULL}};
const Segment SGI120205[] = {{&S12668[78], 6}, {NULL}};
const Segment SGI120206[] = {{&S12669[44], 30}, {NULL}};
const Segment SGI120207[] = {{&S12669[44], 42}, {NULL}};
const Segment SGI120208[] = {{&S12669[30], 6}, {NULL}};
const Segment SGI120209[] = {{&S12669[31], 3}, {NULL}};
const Segment SGI120210[] = {{&S12669[100], 6}, {NULL}};
const Segment SGI120211[] = {{&S96238[34], 11}, {NULL}};
const Segment SGI120212[] = {{&S96238[27], 6}, {NULL}};
const Segment SGI120213[] = {{&S12670[44], 29}, {NULL}};
const Segment SGI120214[] = {{&S12670[44], 41}, {NULL}};
const Segment SGI120215[] = {{&S12670[30], 6}, {NULL}};
const Segment SGI120216[] = {{&S96238[98], 9}, {NULL}};
const Segment SGI120217[] = {{&S96238[71], 6}, {NULL}};
const Segment SGI120218[] = {{&S12670[31], 3}, {NULL}};
const Segment SGI120219[] = {{&S12670[99], 6}, {NULL}};
const Segment SGI120220[] = {{&S12671[44], 10}, {NULL}};
const Segment SGI120221[] = {{&S12671[44], 22}, {NULL}};
const Segment SGI120222[] = {{&S12671[30], 6}, {NULL}};
const Segment SGI120223[] = {{&S12671[31], 3}, {NULL}};
const Segment SGI120224[] = {{&S12671[80], 6}, {NULL}};
const Segment SGI120225[] = {{&S12676[44], 29}, {NULL}};
const Segment SGI120226[] = {{&S12676[44], 41}, {NULL}};
const Segment SGI120227[] = {{&S12676[30], 6}, {NULL}};
const Segment SGI120228[] = {{&S12676[31], 3}, {NULL}};
const Segment SGI120229[] = {{&S12676[99], 6}, {NULL}};
const Segment SGI120230[] = {{&S12678[44], 30}, {NULL}};
const Segment SGI120231[] = {{&S12678[44], 42}, {NULL}};
const Segment SGI120232[] = {{&S12678[30], 6}, {NULL}};
const Segment SGI120233[] = {{&S12678[31], 3}, {NULL}};
const Segment SGI120234[] = {{&S12678[100], 6}, {NULL}};
const Segment SGI120235[] = {{&S12681[44], 10}, {NULL}};
const Segment SGI120236[] = {{&S12681[44], 22}, {NULL}};
const Segment SGI120237[] = {{&S12681[30], 6}, {NULL}};
const Segment SGI120238[] = {{&S12681[83], 6}, {NULL}};
const Segment SGI120239[] = {{&S12683[44], 30}, {NULL}};
const Segment SGI120240[] = {{&S12683[44], 42}, {NULL}};
const Segment SGI120241[] = {{&S12683[30], 6}, {NULL}};
const Segment SGI120242[] = {{&S12683[31], 3}, {NULL}};
const Segment SGI120243[] = {{&S12683[100], 6}, {NULL}};
const Segment SGI120244[] = {{&S12684[44], 10}, {NULL}};
const Segment SGI120245[] = {{&S12684[44], 22}, {NULL}};
const Segment SGI120246[] = {{&S12684[30], 6}, {NULL}};
const Segment SGI120247[] = {{&S12684[125], 3}, {NULL}};
const Segment SGI120248[] = {{&S12684[90], 6}, {NULL}};
const Segment SGI120249[] = {{&S12684[145], 8}, {NULL}};
const Segment SGI120250[] = {{&S12685[44], 33}, {NULL}};
const Segment SGI120251[] = {{&S12685[44], 45}, {NULL}};
const Segment SGI120252[] = {{&S12685[30], 6}, {NULL}};
const Segment SGI120253[] = {{&S12685[31], 3}, {NULL}};
const Segment SGI120254[] = {{&S12685[103], 6}, {NULL}};
const Segment SGI120255[] = {{&S12686[42], 30}, {NULL}};
const Segment SGI120256[] = {{&S12686[30], 6}, {NULL}};
const Segment SGI120257[] = {{&S12686[31], 3}, {NULL}};
const Segment SGI120258[] = {{&S12686[98], 6}, {NULL}};
const Segment SGI120259[] = {{&S12689[44], 29}, {NULL}};
const Segment SGI120260[] = {{&S12689[44], 41}, {NULL}};
const Segment SGI120261[] = {{&S12689[30], 6}, {NULL}};
const Segment SGI120262[] = {{&S12689[31], 3}, {NULL}};
const Segment SGI120263[] = {{&S12689[99], 6}, {NULL}};
const Segment SGI120264[] = {{&S12691[44], 10}, {&S12691[175], 18}, {NULL}};
const Segment SGI120265[] = {{&S12691[44], 22}, {NULL}};
const Segment SGI120266[] = {{&S12691[30], 6}, {NULL}};
const Segment SGI120267[] = {{&S12691[31], 3}, {NULL}};
const Segment SGI120268[] = {{&S12691[80], 6}, {NULL}};
const Segment SGI120269[] = {{&S12692[44], 30}, {NULL}};
const Segment SGI120270[] = {{&S12692[44], 42}, {NULL}};
const Segment SGI120271[] = {{&S12692[30], 6}, {NULL}};
const Segment SGI120272[] = {{&S12692[31], 3}, {NULL}};
const Segment SGI120273[] = {{&S12692[100], 6}, {NULL}};
const Segment SGI120274[] = {{&S12693[44], 10}, {NULL}};
const Segment SGI120275[] = {{&S12693[44], 34}, {NULL}};
const Segment SGI120276[] = {{&S12693[30], 6}, {NULL}};
const Segment SGI120277[] = {{&S12694[44], 10}, {NULL}};
const Segment SGI120278[] = {{&S12694[44], 22}, {&S12694[235], 5}, {NULL}};
const Segment SGI120279[] = {{&S12694[30], 6}, {NULL}};
const Segment SGI120280[] = {{&S12694[31], 3}, {NULL}};
const Segment SGI120281[] = {{&S12694[80], 6}, {NULL}};
const Segment SGI120282[] = {{&S12695[44], 10}, {NULL}};
const Segment SGI120283[] = {{&S12695[44], 22}, {NULL}};
const Segment SGI120284[] = {{&S12695[30], 6}, {NULL}};
const Segment SGI120285[] = {{&S12695[80], 6}, {NULL}};
const Segment SGI120286[] = {{&S12697[44], 30}, {NULL}};
const Segment SGI120287[] = {{&S12697[44], 42}, {NULL}};
const Segment SGI120288[] = {{&S12697[30], 6}, {NULL}};
const Segment SGI120289[] = {{&S12697[31], 3}, {NULL}};
const Segment SGI120290[] = {{&S12697[100], 6}, {NULL}};
const Segment SGI120291[] = {{&S12701[44], 33}, {NULL}};
const Segment SGI120292[] = {{&S12701[44], 45}, {NULL}};
const Segment SGI120293[] = {{&S12701[30], 6}, {NULL}};
const Segment SGI120294[] = {{&S12701[31], 3}, {NULL}};
const Segment SGI120295[] = {{&S12701[103], 6}, {NULL}};
const Segment SGI120296[] = {{&S12702[44], 10}, {NULL}};
const Segment SGI120297[] = {{&S12702[44], 19}, {NULL}};
const Segment SGI120298[] = {{&S12702[30], 6}, {NULL}};
const Segment SGI120299[] = {{&S12702[31], 3}, {NULL}};
const Segment SGI120300[] = {{&S12702[77], 5}, {NULL}};
const Segment SGI120301[] = {{&S12703[45], 10}, {NULL}};
const Segment SGI120302[] = {{&S12703[45], 34}, {NULL}};
const Segment SGI120303[] = {{&S12703[31], 6}, {NULL}};
const Segment SGI120304[] = {{&S12709[44], 10}, {NULL}};
const Segment SGI120305[] = {{&S12709[44], 22}, {NULL}};
const Segment SGI120306[] = {{&S12709[30], 6}, {NULL}};
const Segment SGI120307[] = {{&S12709[31], 3}, {NULL}};
const Segment SGI120308[] = {{&S12709[80], 6}, {NULL}};
const Segment SGI120309[] = {{&S12711[44], 10}, {NULL}};
const Segment SGI120310[] = {{&S12711[44], 22}, {NULL}};
const Segment SGI120311[] = {{&S12711[30], 6}, {NULL}};
const Segment SGI120312[] = {{&S12711[31], 3}, {NULL}};
const Segment SGI120313[] = {{&S12711[80], 6}, {NULL}};
const Segment SGI120314[] = {{&S12712[44], 30}, {NULL}};
const Segment SGI120315[] = {{&S12712[44], 42}, {NULL}};
const Segment SGI120316[] = {{&S12712[30], 6}, {NULL}};
const Segment SGI120317[] = {{&S12712[31], 3}, {NULL}};
const Segment SGI120318[] = {{&S12712[100], 6}, {NULL}};
const Segment SGI120319[] = {{&S12716[46], 10}, {NULL}};
const Segment SGI120320[] = {{&S12716[46], 34}, {NULL}};
const Segment SGI120321[] = {{&S12716[32], 6}, {NULL}};
const Segment SGI120322[] = {{&S12717[44], 30}, {NULL}};
const Segment SGI120323[] = {{&S12717[44], 42}, {NULL}};
const Segment SGI120324[] = {{&S12717[30], 6}, {NULL}};
const Segment SGI120325[] = {{&S12717[31], 3}, {NULL}};
const Segment SGI120326[] = {{&S12717[100], 6}, {NULL}};
const Segment SGI120327[] = {{&S12719[47], 10}, {NULL}};
const Segment SGI120328[] = {{&S12719[47], 34}, {NULL}};
const Segment SGI120329[] = {{&S12719[33], 6}, {NULL}};
const Segment SGI120330[] = {{&S12720[46], 10}, {NULL}};
const Segment SGI120331[] = {{&S12720[46], 34}, {NULL}};
const Segment SGI120332[] = {{&S12720[32], 6}, {NULL}};
const Segment SGI120333[] = {{&S12721[44], 10}, {NULL}};
const Segment SGI120334[] = {{&S12721[44], 27}, {NULL}};
const Segment SGI120335[] = {{&S12721[30], 6}, {NULL}};
const Segment SGI120336[] = {{&S12721[31], 3}, {NULL}};
const Segment SGI120337[] = {{&S12721[85], 6}, {NULL}};
const Segment SGI120338[] = {{&S96332[44], 18}, {NULL}};
const Segment SGI120339[] = {{&S96332[44], 30}, {NULL}};
const Segment SGI120340[] = {{&S96332[30], 6}, {NULL}};
const Segment SGI120341[] = {{&S96332[119], 2}, {NULL}};
const Segment SGI120342[] = {{&S96332[88], 5}, {NULL}};
const Segment SGI120343[] = {{&S12723[42], 30}, {NULL}};
const Segment SGI120344[] = {{&S12723[30], 6}, {NULL}};
const Segment SGI120345[] = {{&S12723[31], 3}, {NULL}};
const Segment SGI120346[] = {{&S96333[44], 21}, {NULL}};
const Segment SGI120347[] = {{&S96333[44], 33}, {NULL}};
const Segment SGI120348[] = {{&S12723[98], 6}, {NULL}};
const Segment SGI120349[] = {{&S96333[30], 6}, {NULL}};
const Segment SGI120350[] = {{&S96333[122], 2}, {NULL}};
const Segment SGI120351[] = {{&S96333[91], 5}, {NULL}};
const Segment SGI120352[] = {{&S12726[46], 10}, {NULL}};
const Segment SGI120353[] = {{&S12726[46], 34}, {NULL}};
const Segment SGI120354[] = {{&S12726[32], 6}, {NULL}};
const Segment SGI120355[] = {{&S12727[46], 10}, {NULL}};
const Segment SGI120356[] = {{&S12727[46], 34}, {NULL}};
const Segment SGI120357[] = {{&S12727[32], 6}, {NULL}};
const Segment SGI120358[] = {{&S12729[46], 10}, {NULL}};
const Segment SGI120359[] = {{&S12729[46], 34}, {NULL}};
const Segment SGI120360[] = {{&S12729[32], 6}, {NULL}};
const Segment SGI120361[] = {{&S96345[44], 10}, {NULL}};
const Segment SGI120362[] = {{&S96345[44], 28}, {NULL}};
const Segment SGI120363[] = {{&S96345[30], 6}, {NULL}};
const Segment SGI120364[] = {{&S96345[31], 3}, {NULL}};
const Segment SGI120365[] = {{&S96345[86], 6}, {NULL}};
const Segment SGI120366[] = {{&S12738[44], 10}, {NULL}};
const Segment SGI120367[] = {{&S12738[44], 22}, {NULL}};
const Segment SGI120368[] = {{&S12738[30], 6}, {NULL}};
const Segment SGI120369[] = {{&S12738[31], 3}, {NULL}};
const Segment SGI120370[] = {{&S12738[80], 6}, {NULL}};
const Segment SGI120371[] = {{&S12742[44], 10}, {NULL}};
const Segment SGI120372[] = {{&S12742[44], 22}, {NULL}};
const Segment SGI120373[] = {{&S12742[30], 6}, {NULL}};
const Segment SGI120374[] = {{&S12742[116], 3}, {NULL}};
const Segment SGI120375[] = {{&S12742[80], 7}, {NULL}};
const Segment SGI120376[] = {{&S12742[136], 8}, {NULL}};
const Segment SGI120377[] = {{&S12746[44], 10}, {NULL}};
const Segment SGI120378[] = {{&S12746[44], 48}, {NULL}};
const Segment SGI120379[] = {{&S12746[30], 6}, {NULL}};
const Segment SGI120380[] = {{&S12746[31], 3}, {NULL}};
const Segment SGI120381[] = {{&S12750[44], 10}, {NULL}};
const Segment SGI120382[] = {{&S12750[44], 22}, {NULL}};
const Segment SGI120383[] = {{&S12750[30], 6}, {NULL}};
const Segment SGI120384[] = {{&S12750[80], 3}, {NULL}};
const Segment SGI120385[] = {{&S12752[44], 10}, {NULL}};
const Segment SGI120386[] = {{&S12752[44], 22}, {NULL}};
const Segment SGI120387[] = {{&S96370[34], 17}, {NULL}};
const Segment SGI120388[] = {{&S12752[30], 6}, {NULL}};
const Segment SGI120389[] = {{&S96370[27], 6}, {NULL}};
const Segment SGI120390[] = {{&S12752[31], 3}, {NULL}};
const Segment SGI120391[] = {{&S12752[80], 5}, {NULL}};
const Segment SGI120392[] = {{&S96370[117], 9}, {NULL}};
const Segment SGI120393[] = {{&S96370[90], 6}, {NULL}};
const Segment SGI120394[] = {{&S12753[64], 10}, {NULL}};
const Segment SGI120395[] = {{&S12753[64], 22}, {NULL}};
const Segment SGI120396[] = {{&S12753[50], 6}, {NULL}};
const Segment SGI120397[] = {{&S12755[44], 29}, {NULL}};
const Segment SGI120398[] = {{&S12755[44], 41}, {NULL}};
const Segment SGI120399[] = {{&S12755[30], 6}, {NULL}};
const Segment SGI120400[] = {{&S12755[31], 3}, {NULL}};
const Segment SGI120401[] = {{&S12755[99], 6}, {NULL}};
const Segment SGI120402[] = {{&S12756[38], 10}, {NULL}};
const Segment SGI120403[] = {{&S12756[31], 6}, {NULL}};
const Segment SGI120404[] = {{&S96378[34], 18}, {NULL}};
const Segment SGI120405[] = {{&S96378[27], 6}, {NULL}};
const Segment SGI120406[] = {{&S12757[44], 30}, {NULL}};
const Segment SGI120407[] = {{&S12757[44], 42}, {NULL}};
const Segment SGI120408[] = {{&S12757[30], 6}, {NULL}};
const Segment SGI120409[] = {{&S96378[105], 9}, {NULL}};
const Segment SGI120410[] = {{&S96378[78], 6}, {NULL}};
const Segment SGI120411[] = {{&S12757[31], 3}, {NULL}};
const Segment SGI120412[] = {{&S12757[100], 6}, {NULL}};
const Segment SGI120413[] = {{&S6849[44], 12}, {NULL}};
const Segment SGI120414[] = {{&S6849[44], 24}, {NULL}};
const Segment SGI120415[] = {{&S6849[30], 6}, {NULL}};
const Segment SGI120416[] = {{&S6849[31], 3}, {NULL}};
const Segment SGI120417[] = {{&S6849[82], 5}, {NULL}};
const Segment SGI120418[] = {{&S96405[45], 14}, {NULL}};
const Segment SGI120419[] = {{&S96405[30], 6}, {NULL}};
const Segment SGI120420[] = {{&S96405[31], 3}, {NULL}};
const Segment SGI120421[] = {{&S96405[85], 5}, {NULL}};
const Segment SGI120422[] = {{&S96405[143], 13}, {NULL}};
const Segment SGI120423[] = {{&S96429[59], 13}, {NULL}};
const Segment SGI120424[] = {{&S96429[59], 25}, {NULL}};
const Segment SGI120425[] = {{&S96429[45], 6}, {NULL}};
const Segment SGI120426[] = {{&S96429[46], 3}, {NULL}};
const Segment SGI120427[] = {{&S96429[98], 6}, {NULL}};
const Segment SGI120428[] = {{&S96429[0], 14}, {NULL}};
const Segment SGI120429[] = {{&S96438[34], 6}, {NULL}};
const Segment SGI120430[] = {{&S96438[27], 6}, {NULL}};
const Segment SGI120431[] = {{&S96438[93], 9}, {NULL}};
const Segment SGI120432[] = {{&S96438[66], 6}, {NULL}};
const Segment SGI120433[] = {{&S96439[34], 11}, {NULL}};
const Segment SGI120434[] = {{&S96439[27], 6}, {NULL}};
const Segment SGI120435[] = {{&S96439[98], 9}, {NULL}};
const Segment SGI120436[] = {{&S96439[71], 6}, {NULL}};
const Segment SGI120437[] = {{&S96448[34], 14}, {NULL}};
const Segment SGI120438[] = {{&S96448[27], 6}, {NULL}};
const Segment SGI120439[] = {{&S96448[108], 9}, {NULL}};
const Segment SGI120440[] = {{&S96448[81], 6}, {NULL}};
const Segment SGI120441[] = {{&S96465[34], 9}, {NULL}};
const Segment SGI120442[] = {{&S96465[27], 6}, {NULL}};
const Segment SGI120443[] = {{&S96465[96], 9}, {NULL}};
const Segment SGI120444[] = {{&S96465[69], 6}, {NULL}};
const Segment SGI120445[] = {{&S96517[44], 6}, {NULL}};
const Segment SGI120446[] = {{&S96517[44], 18}, {NULL}};
const Segment SGI120447[] = {{&S96517[30], 6}, {NULL}};
const Segment SGI120448[] = {{&S96517[31], 3}, {NULL}};
const Segment SGI120449[] = {{&S96517[76], 6}, {NULL}};
const Segment SGI120450[] = {{&S96517[153], 3}, {NULL}};
const Segment SGI120451[] = {{&S7090[44], 7}, {NULL}};
const Segment SGI120452[] = {{&S7090[44], 19}, {NULL}};
const Segment SGI120453[] = {{&S7090[30], 6}, {NULL}};
const Segment SGI120454[] = {{&S7090[31], 3}, {NULL}};
const Segment SGI120455[] = {{&S7090[77], 5}, {NULL}};
const Segment SGI120456[] = {{&S96575[34], 18}, {NULL}};
const Segment SGI120457[] = {{&S96575[27], 6}, {NULL}};
const Segment SGI120458[] = {{&S7113[44], 6}, {NULL}};
const Segment SGI120459[] = {{&S7113[44], 18}, {NULL}};
const Segment SGI120460[] = {{&S7113[30], 6}, {NULL}};
const Segment SGI120461[] = {{&S7113[31], 3}, {NULL}};
const Segment SGI120462[] = {{&S96575[105], 9}, {NULL}};
const Segment SGI120463[] = {{&S7113[76], 6}, {NULL}};
const Segment SGI120464[] = {{&S96575[78], 6}, {NULL}};
const Segment SGI120465[] = {{&S96584[34], 9}, {NULL}};
const Segment SGI120466[] = {{&S96584[27], 6}, {NULL}};
const Segment SGI120467[] = {{&S96584[96], 9}, {NULL}};
const Segment SGI120468[] = {{&S96584[69], 6}, {NULL}};
const Segment SGI120469[] = {{&S4326[38], 10}, {NULL}};
const Segment SGI120470[] = {{&S4326[31], 6}, {NULL}};
const Segment SGI120471[] = {{&S4327[44], 10}, {NULL}};
const Segment SGI120472[] = {{&S4327[44], 22}, {NULL}};
const Segment SGI120473[] = {{&S4327[30], 6}, {NULL}};
const Segment SGI120474[] = {{&S4327[115], 3}, {NULL}};
const Segment SGI120475[] = {{&S4327[80], 6}, {NULL}};
const Segment SGI120476[] = {{&S4328[44], 10}, {NULL}};
const Segment SGI120477[] = {{&S4328[44], 22}, {NULL}};
const Segment SGI120478[] = {{&S4328[30], 6}, {NULL}};
const Segment SGI120479[] = {{&S4328[115], 3}, {NULL}};
const Segment SGI120480[] = {{&S4328[80], 6}, {NULL}};
const Segment SGI120481[] = {{&S7144[44], 6}, {NULL}};
const Segment SGI120482[] = {{&S7144[44], 18}, {NULL}};
const Segment SGI120483[] = {{&S7144[30], 6}, {NULL}};
const Segment SGI120484[] = {{&S7144[31], 3}, {NULL}};
const Segment SGI120485[] = {{&S4329[44], 10}, {NULL}};
const Segment SGI120486[] = {{&S7144[76], 6}, {NULL}};
const Segment SGI120487[] = {{&S4329[44], 22}, {NULL}};
const Segment SGI120488[] = {{&S4329[30], 6}, {NULL}};
const Segment SGI120489[] = {{&S4329[115], 3}, {NULL}};
const Segment SGI120490[] = {{&S4329[80], 6}, {NULL}};
const Segment SGI120491[] = {{&S4330[44], 11}, {NULL}};
const Segment SGI120492[] = {{&S4330[44], 23}, {NULL}};
const Segment SGI120493[] = {{&S4330[30], 6}, {NULL}};
const Segment SGI120494[] = {{&S4330[116], 3}, {NULL}};
const Segment SGI120495[] = {{&S4330[81], 6}, {NULL}};
const Segment SGI120496[] = {{&S7148[44], 11}, {NULL}};
const Segment SGI120497[] = {{&S4331[44], 18}, {NULL}};
const Segment SGI120498[] = {{&S7148[44], 23}, {NULL}};
const Segment SGI120499[] = {{&S4331[44], 30}, {NULL}};
const Segment SGI120500[] = {{&S7148[30], 6}, {NULL}};
const Segment SGI120501[] = {{&S4331[30], 6}, {NULL}};
const Segment SGI120502[] = {{&S7148[31], 3}, {NULL}};
const Segment SGI120503[] = {{&S7148[81], 6}, {NULL}};
const Segment SGI120504[] = {{&S4331[123], 3}, {NULL}};
const Segment SGI120505[] = {{&S4331[88], 6}, {NULL}};
const Segment SGI120506[] = {{&S4332[44], 30}, {NULL}};
const Segment SGI120507[] = {{&S4332[44], 42}, {NULL}};
const Segment SGI120508[] = {{&S4332[30], 6}, {NULL}};
const Segment SGI120509[] = {{&S4332[135], 3}, {NULL}};
const Segment SGI120510[] = {{&S4332[100], 6}, {NULL}};
const Segment SGI120511[] = {{&S4333[44], 11}, {NULL}};
const Segment SGI120512[] = {{&S4333[44], 23}, {NULL}};
const Segment SGI120513[] = {{&S4333[30], 6}, {NULL}};
const Segment SGI120514[] = {{&S4333[116], 3}, {NULL}};
const Segment SGI120515[] = {{&S4333[81], 6}, {NULL}};
const Segment SGI120516[] = {{&S4334[44], 10}, {NULL}};
const Segment SGI120517[] = {{&S4334[44], 22}, {NULL}};
const Segment SGI120518[] = {{&S4334[30], 6}, {NULL}};
const Segment SGI120519[] = {{&S4334[115], 3}, {NULL}};
const Segment SGI120520[] = {{&S4334[80], 6}, {NULL}};
const Segment SGI120521[] = {{&S4335[44], 11}, {NULL}};
const Segment SGI120522[] = {{&S4335[44], 23}, {NULL}};
const Segment SGI120523[] = {{&S4335[30], 6}, {NULL}};
const Segment SGI120524[] = {{&S4335[116], 3}, {NULL}};
const Segment SGI120525[] = {{&S4335[81], 6}, {NULL}};
const Segment SGI120526[] = {{&S4336[44], 11}, {NULL}};
const Segment SGI120527[] = {{&S4336[44], 23}, {NULL}};
const Segment SGI120528[] = {{&S4336[30], 6}, {NULL}};
const Segment SGI120529[] = {{&S4336[116], 3}, {NULL}};
const Segment SGI120530[] = {{&S4336[81], 6}, {NULL}};
const Segment SGI120531[] = {{&S4337[44], 10}, {NULL}};
const Segment SGI120532[] = {{&S4337[44], 22}, {NULL}};
const Segment SGI120533[] = {{&S4337[30], 6}, {NULL}};
const Segment SGI120534[] = {{&S4337[115], 3}, {NULL}};
const Segment SGI120535[] = {{&S4337[80], 6}, {NULL}};
const Segment SGI120536[] = {{&S4338[44], 29}, {NULL}};
const Segment SGI120537[] = {{&S4338[44], 41}, {NULL}};
const Segment SGI120538[] = {{&S4338[30], 6}, {NULL}};
const Segment SGI120539[] = {{&S4338[134], 3}, {NULL}};
const Segment SGI120540[] = {{&S4338[99], 6}, {NULL}};
const Segment SGI120541[] = {{&S4339[44], 30}, {NULL}};
const Segment SGI120542[] = {{&S4339[44], 42}, {NULL}};
const Segment SGI120543[] = {{&S4339[30], 6}, {NULL}};
const Segment SGI120544[] = {{&S4339[135], 3}, {NULL}};
const Segment SGI120545[] = {{&S4339[100], 6}, {NULL}};
const Segment SGI120546[] = {{&S4340[44], 11}, {NULL}};
const Segment SGI120547[] = {{&S4340[44], 23}, {NULL}};
const Segment SGI120548[] = {{&S4340[30], 6}, {NULL}};
const Segment SGI120549[] = {{&S4340[116], 3}, {NULL}};
const Segment SGI120550[] = {{&S4340[81], 6}, {NULL}};
const Segment SGI120551[] = {{&S4341[44], 10}, {NULL}};
const Segment SGI120552[] = {{&S4341[44], 22}, {NULL}};
const Segment SGI120553[] = {{&S4341[30], 6}, {NULL}};
const Segment SGI120554[] = {{&S4341[83], 6}, {NULL}};
const Segment SGI120555[] = {{&S4342[44], 29}, {NULL}};
const Segment SGI120556[] = {{&S4342[44], 41}, {NULL}};
const Segment SGI120557[] = {{&S4342[30], 6}, {NULL}};
const Segment SGI120558[] = {{&S4342[134], 3}, {NULL}};
const Segment SGI120559[] = {{&S4342[99], 6}, {NULL}};
const Segment SGI120560[] = {{&S4343[38], 10}, {NULL}};
const Segment SGI120561[] = {{&S4343[31], 6}, {NULL}};
const Segment SGI120562[] = {{&S4344[38], 10}, {NULL}};
const Segment SGI120563[] = {{&S4344[31], 6}, {NULL}};
const Segment SGI120564[] = {{&S4345[38], 10}, {NULL}};
const Segment SGI120565[] = {{&S4345[31], 6}, {NULL}};
const Segment SGI120566[] = {{&S4346[44], 30}, {NULL}};
const Segment SGI120567[] = {{&S4346[44], 42}, {NULL}};
const Segment SGI120568[] = {{&S4346[30], 6}, {NULL}};
const Segment SGI120569[] = {{&S4346[135], 3}, {NULL}};
const Segment SGI120570[] = {{&S4346[100], 6}, {NULL}};
const Segment SGI120571[] = {{&S4347[38], 18}, {NULL}};
const Segment SGI120572[] = {{&S4347[31], 6}, {NULL}};
const Segment SGI120573[] = {{&S7183[44], 9}, {NULL}};
const Segment SGI120574[] = {{&S7183[44], 30}, {NULL}};
const Segment SGI120575[] = {{&S7183[30], 6}, {NULL}};
const Segment SGI120576[] = {{&S7183[31], 3}, {NULL}};
const Segment SGI120577[] = {{&S7183[88], 5}, {NULL}};
const Segment SGI120578[] = {{&S4349[44], 8}, {NULL}};
const Segment SGI120579[] = {{&S4349[44], 20}, {NULL}};
const Segment SGI120580[] = {{&S4349[30], 6}, {NULL}};
const Segment SGI120581[] = {{&S4349[113], 3}, {NULL}};
const Segment SGI120582[] = {{&S4349[78], 6}, {NULL}};
const Segment SGI120583[] = {{&S4350[38], 11}, {NULL}};
const Segment SGI120584[] = {{&S4350[31], 6}, {NULL}};
const Segment SGI120585[] = {{&S4351[42], 10}, {NULL}};
const Segment SGI120586[] = {{&S4351[30], 6}, {NULL}};
const Segment SGI120587[] = {{&S4351[113], 3}, {NULL}};
const Segment SGI120588[] = {{&S4351[78], 6}, {NULL}};
const Segment SGI120589[] = {{&S4352[44], 11}, {NULL}};
const Segment SGI120590[] = {{&S4352[44], 23}, {NULL}};
const Segment SGI120591[] = {{&S4352[30], 6}, {NULL}};
const Segment SGI120592[] = {{&S4352[116], 3}, {NULL}};
const Segment SGI120593[] = {{&S4352[81], 6}, {NULL}};
const Segment SGI120594[] = {{&S4353[38], 10}, {NULL}};
const Segment SGI120595[] = {{&S4353[31], 6}, {NULL}};
const Segment SGI120596[] = {{&S4354[38], 11}, {NULL}};
const Segment SGI120597[] = {{&S4354[31], 6}, {NULL}};
const Segment SGI120598[] = {{&S4359[44], 10}, {NULL}};
const Segment SGI120599[] = {{&S4359[44], 22}, {NULL}};
const Segment SGI120600[] = {{&S4359[30], 6}, {NULL}};
const Segment SGI120601[] = {{&S4359[115], 3}, {NULL}};
const Segment SGI120602[] = {{&S4359[80], 6}, {NULL}};
const Segment SGI120603[] = {{&S4362[44], 10}, {NULL}};
const Segment SGI120604[] = {{&S4362[44], 22}, {NULL}};
const Segment SGI120605[] = {{&S4362[30], 6}, {NULL}};
const Segment SGI120606[] = {{&S4362[115], 3}, {NULL}};
const Segment SGI120607[] = {{&S4362[80], 6}, {NULL}};
const Segment SGI120608[] = {{&S7209[44], 12}, {NULL}};
const Segment SGI120609[] = {{&S7209[44], 24}, {NULL}};
const Segment SGI120610[] = {{&S7209[30], 6}, {NULL}};
const Segment SGI120611[] = {{&S7209[31], 3}, {NULL}};
const Segment SGI120612[] = {{&S7209[82], 6}, {NULL}};
const Segment SGI120613[] = {{&S4370[38], 11}, {NULL}};
const Segment SGI120614[] = {{&S4370[31], 6}, {NULL}};
const Segment SGI120615[] = {{&S4371[38], 11}, {NULL}};
const Segment SGI120616[] = {{&S4371[31], 6}, {NULL}};
const Segment SGI120617[] = {{&S4372[44], 30}, {NULL}};
const Segment SGI120618[] = {{&S4372[44], 42}, {NULL}};
const Segment SGI120619[] = {{&S4372[30], 6}, {NULL}};
const Segment SGI120620[] = {{&S4372[135], 3}, {NULL}};
const Segment SGI120621[] = {{&S4372[100], 6}, {NULL}};
const Segment SGI120622[] = {{&S4374[44], 11}, {NULL}};
const Segment SGI120623[] = {{&S4374[44], 23}, {NULL}};
const Segment SGI120624[] = {{&S4374[30], 6}, {NULL}};
const Segment SGI120625[] = {{&S4374[116], 3}, {NULL}};
const Segment SGI120626[] = {{&S4374[81], 6}, {NULL}};
const Segment SGI120627[] = {{&S4375[44], 10}, {NULL}};
const Segment SGI120628[] = {{&S4375[44], 22}, {NULL}};
const Segment SGI120629[] = {{&S4375[30], 6}, {NULL}};
const Segment SGI120630[] = {{&S4375[83], 6}, {NULL}};
const Segment SGI120631[] = {{&S7226[44], 5}, {NULL}};
const Segment SGI120632[] = {{&S7226[44], 17}, {NULL}};
const Segment SGI120633[] = {{&S7226[30], 6}, {NULL}};
const Segment SGI120634[] = {{&S7226[31], 3}, {NULL}};
const Segment SGI120635[] = {{&S7226[75], 6}, {NULL}};
const Segment SGI120636[] = {{&S4381[38], 11}, {NULL}};
const Segment SGI120637[] = {{&S4381[31], 6}, {NULL}};
const Segment SGI120638[] = {{&S4382[49], 10}, {NULL}};
const Segment SGI120639[] = {{&S4382[49], 22}, {NULL}};
const Segment SGI120640[] = {{&S4382[30], 11}, {NULL}};
const Segment SGI120641[] = {{&S4382[120], 3}, {NULL}};
const Segment SGI120642[] = {{&S4382[85], 6}, {NULL}};
const Segment SGI120643[] = {{&S4384[54], 10}, {NULL}};
const Segment SGI120644[] = {{&S4384[54], 22}, {NULL}};
const Segment SGI120645[] = {{&S4384[30], 16}, {NULL}};
const Segment SGI120646[] = {{&S4384[125], 3}, {NULL}};
const Segment SGI120647[] = {{&S4384[90], 6}, {NULL}};
const Segment SGI120648[] = {{&S4387[38], 11}, {NULL}};
const Segment SGI120649[] = {{&S4387[31], 6}, {NULL}};
const Segment SGI120650[] = {{&S4395[44], 10}, {NULL}};
const Segment SGI120651[] = {{&S4395[44], 22}, {NULL}};
const Segment SGI120652[] = {{&S4395[30], 6}, {NULL}};
const Segment SGI120653[] = {{&S4395[83], 6}, {NULL}};
const Segment SGI120654[] = {{&S4396[38], 8}, {NULL}};
const Segment SGI120655[] = {{&S4396[31], 6}, {NULL}};
const Segment SGI120656[] = {{&S4397[38], 10}, {NULL}};
const Segment SGI120657[] = {{&S4397[31], 6}, {NULL}};
const Segment SGI120658[] = {{&S4401[44], 7}, {NULL}};
const Segment SGI120659[] = {{&S4401[44], 19}, {NULL}};
const Segment SGI120660[] = {{&S4401[30], 6}, {NULL}};
const Segment SGI120661[] = {{&S4401[112], 3}, {NULL}};
const Segment SGI120662[] = {{&S4401[77], 6}, {NULL}};
const Segment SGI120663[] = {{&S4409[44], 10}, {NULL}};
const Segment SGI120664[] = {{&S4409[44], 22}, {NULL}};
const Segment SGI120665[] = {{&S4409[30], 6}, {NULL}};
const Segment SGI120666[] = {{&S4409[115], 3}, {NULL}};
const Segment SGI120667[] = {{&S4409[80], 6}, {NULL}};
const Segment SGI120668[] = {{&S4410[44], 22}, {NULL}};
const Segment SGI120669[] = {{&S4410[30], 6}, {NULL}};
const Segment SGI120670[] = {{&S4418[42], 10}, {NULL}};
const Segment SGI120671[] = {{&S4418[30], 6}, {NULL}};
const Segment SGI120672[] = {{&S4418[113], 3}, {NULL}};
const Segment SGI120673[] = {{&S4418[78], 6}, {NULL}};
const Segment SGI120674[] = {{&S4431[44], 8}, {NULL}};
const Segment SGI120675[] = {{&S4431[44], 20}, {NULL}};
const Segment SGI120676[] = {{&S4431[30], 6}, {NULL}};
const Segment SGI120677[] = {{&S4431[113], 3}, {NULL}};
const Segment SGI120678[] = {{&S4431[78], 6}, {NULL}};
const Segment SGI120679[] = {{&S4437[44], 10}, {NULL}};
const Segment SGI120680[] = {{&S4437[44], 22}, {NULL}};
const Segment SGI120681[] = {{&S4437[30], 6}, {NULL}};
const Segment SGI120682[] = {{&S4437[114], 3}, {NULL}};
const Segment SGI120683[] = {{&S4437[80], 5}, {NULL}};
const Segment SGI120684[] = {{&S4445[55], 10}, {NULL}};
const Segment SGI120685[] = {{&S4445[55], 22}, {NULL}};
const Segment SGI120686[] = {{&S4445[30], 17}, {NULL}};
const Segment SGI120687[] = {{&S4445[126], 3}, {NULL}};
const Segment SGI120688[] = {{&S4445[91], 6}, {NULL}};
const Segment SGI120689[] = {{&S4447[44], 13}, {NULL}};
const Segment SGI120690[] = {{&S4447[44], 34}, {NULL}};
const Segment SGI120691[] = {{&S4447[30], 6}, {NULL}};
const Segment SGI120692[] = {{&S4447[126], 3}, {NULL}};
const Segment SGI120693[] = {{&S4447[92], 5}, {NULL}};
const Segment SGI120694[] = {{&S7309[44], 7}, {NULL}};
const Segment SGI120695[] = {{&S7309[44], 19}, {NULL}};
const Segment SGI120696[] = {{&S7309[30], 6}, {NULL}};
const Segment SGI120697[] = {{&S7309[31], 3}, {NULL}};
const Segment SGI120698[] = {{&S7309[77], 5}, {NULL}};
const Segment SGI120699[] = {{&S4463[44], 10}, {NULL}};
const Segment SGI120700[] = {{&S4463[44], 22}, {NULL}};
const Segment SGI120701[] = {{&S4463[30], 6}, {NULL}};
const Segment SGI120702[] = {{&S4463[115], 3}, {NULL}};
const Segment SGI120703[] = {{&S4463[80], 6}, {NULL}};
const Segment SGI120704[] = {{&S4464[44], 10}, {NULL}};
const Segment SGI120705[] = {{&S4464[44], 33}, {NULL}};
const Segment SGI120706[] = {{&S4464[30], 6}, {NULL}};
const Segment SGI120707[] = {{&S4464[126], 3}, {NULL}};
const Segment SGI120708[] = {{&S4464[91], 6}, {NULL}};
const Segment SGI120709[] = {{&S7329[44], 9}, {NULL}};
const Segment SGI120710[] = {{&S7329[44], 21}, {NULL}};
const Segment SGI120711[] = {{&S7329[30], 6}, {NULL}};
const Segment SGI120712[] = {{&S7329[31], 3}, {NULL}};
const Segment SGI120713[] = {{&S7329[79], 5}, {NULL}};
const Segment SGI120714[] = {{&S7433[44], 7}, {NULL}};
const Segment SGI120715[] = {{&S7433[44], 19}, {NULL}};
const Segment SGI120716[] = {{&S7433[30], 6}, {NULL}};
const Segment SGI120717[] = {{&S7433[31], 3}, {NULL}};
const Segment SGI120718[] = {{&S7433[77], 6}, {NULL}};
const Segment SGI120719[] = {{&S96661[59], 14}, {NULL}};
const Segment SGI120720[] = {{&S96661[59], 31}, {NULL}};
const Segment SGI120721[] = {{&S96661[45], 6}, {NULL}};
const Segment SGI120722[] = {{&S96661[46], 3}, {NULL}};
const Segment SGI120723[] = {{&S96661[104], 6}, {NULL}};
const Segment SGI120724[] = {{&S96661[0], 14}, {NULL}};
const Segment SGI120725[] = {{&S96662[34], 24}, {NULL}};
const Segment SGI120726[] = {{&S96662[27], 6}, {NULL}};
const Segment SGI120727[] = {{&S96662[111], 9}, {NULL}};
const Segment SGI120728[] = {{&S96662[84], 6}, {NULL}};
const Segment SGI120729[] = {{&S96669[34], 9}, {NULL}};
const Segment SGI120730[] = {{&S96669[27], 6}, {NULL}};
const Segment SGI120731[] = {{&S96669[94], 9}, {NULL}};
const Segment SGI120732[] = {{&S96669[67], 6}, {NULL}};
const Segment SGI120733[] = {{&S96674[44], 8}, {NULL}};
const Segment SGI120734[] = {{&S96674[44], 20}, {NULL}};
const Segment SGI120735[] = {{&S96674[30], 6}, {NULL}};
const Segment SGI120736[] = {{&S96674[31], 3}, {NULL}};
const Segment SGI120737[] = {{&S96674[78], 6}, {NULL}};
const Segment SGI120738[] = {{&S96679[34], 8}, {NULL}};
const Segment SGI120739[] = {{&S96679[27], 6}, {NULL}};
const Segment SGI120740[] = {{&S96679[95], 9}, {NULL}};
const Segment SGI120741[] = {{&S96679[68], 6}, {NULL}};
const Segment SGI120742[] = {{&S96700[59], 10}, {NULL}};
const Segment SGI120743[] = {{&S96700[45], 6}, {NULL}};
const Segment SGI120744[] = {{&S96700[46], 3}, {NULL}};
const Segment SGI120745[] = {{&S96700[85], 6}, {NULL}};
const Segment SGI120746[] = {{&S96700[0], 14}, {NULL}};
const Segment SGI120747[] = {{&S96703[45], 5}, {NULL}};
const Segment SGI120748[] = {{&S96703[30], 6}, {NULL}};
const Segment SGI120749[] = {{&S96703[31], 3}, {NULL}};
const Segment SGI120750[] = {{&S96703[76], 5}, {NULL}};
const Segment SGI120751[] = {{&S96703[134], 13}, {NULL}};
const Segment SGI120752[] = {{&S7489[44], 16}, {NULL}};
const Segment SGI120753[] = {{&S7489[44], 28}, {NULL}};
const Segment SGI120754[] = {{&S7489[30], 6}, {NULL}};
const Segment SGI120755[] = {{&S7489[31], 3}, {NULL}};
const Segment SGI120756[] = {{&S7489[86], 5}, {NULL}};
const Segment SGI120757[] = {{&S7518[44], 16}, {NULL}};
const Segment SGI120758[] = {{&S7518[44], 28}, {NULL}};
const Segment SGI120759[] = {{&S7518[30], 6}, {NULL}};
const Segment SGI120760[] = {{&S7518[31], 3}, {NULL}};
const Segment SGI120761[] = {{&S7518[86], 6}, {NULL}};
const Segment SGI120762[] = {{&S7595[44], 12}, {NULL}};
const Segment SGI120763[] = {{&S7595[44], 24}, {NULL}};
const Segment SGI120764[] = {{&S7595[30], 6}, {NULL}};
const Segment SGI120765[] = {{&S7595[31], 3}, {NULL}};
const Segment SGI120766[] = {{&S7595[82], 6}, {NULL}};
const Segment SGI120767[] = {{&S96718[44], 6}, {NULL}};
const Segment SGI120768[] = {{&S96718[44], 18}, {NULL}};
const Segment SGI120769[] = {{&S96718[30], 6}, {NULL}};
const Segment SGI120770[] = {{&S96718[31], 3}, {NULL}};
const Segment SGI120771[] = {{&S96718[76], 6}, {NULL}};
const Segment SGI120772[] = {{&S96718[128], 10}, {NULL}};
const Segment SGI120773[] = {{&S96738[44], 35}, {NULL}};
const Segment SGI120774[] = {{&S96738[44], 47}, {NULL}};
const Segment SGI120775[] = {{&S96738[30], 6}, {NULL}};
const Segment SGI120776[] = {{&S96738[31], 3}, {NULL}};
const Segment SGI120777[] = {{&S96738[105], 6}, {NULL}};
const Segment SGI120778[] = {{&S96738[165], 11}, {NULL}};
const Segment SGI120779[] = {{&S96740[44], 11}, {NULL}};
const Segment SGI120780[] = {{&S96740[44], 23}, {NULL}};
const Segment SGI120781[] = {{&S96740[30], 6}, {NULL}};
const Segment SGI120782[] = {{&S96740[31], 3}, {NULL}};
const Segment SGI120783[] = {{&S96740[81], 5}, {NULL}};
const Segment SGI120784[] = {{&S96740[118], 14}, {NULL}};
const Segment SGI120785[] = {{&S96744[45], 12}, {NULL}};
const Segment SGI120786[] = {{&S96744[30], 6}, {NULL}};
const Segment SGI120787[] = {{&S96744[31], 3}, {NULL}};
const Segment SGI120788[] = {{&S96744[83], 5}, {NULL}};
const Segment SGI120789[] = {{&S96744[141], 13}, {NULL}};
const Segment SGI120790[] = {{&S96767[34], 17}, {NULL}};
const Segment SGI120791[] = {{&S96767[27], 6}, {NULL}};
const Segment SGI120792[] = {{&S96767[120], 9}, {NULL}};
const Segment SGI120793[] = {{&S96767[93], 6}, {NULL}};
const Segment SGI120794[] = {{&S96767[154], 8}, {NULL}};
const Segment SGI120795[] = {{&S96815[44], 6}, {NULL}};
const Segment SGI120796[] = {{&S96815[44], 18}, {NULL}};
const Segment SGI120797[] = {{&S96815[30], 6}, {NULL}};
const Segment SGI120798[] = {{&S96815[76], 6}, {NULL}};
const Segment SGI120799[] = {{&S96815[103], 13}, {NULL}};
const Segment SGI120800[] = {{&S96848[44], 6}, {NULL}};
const Segment SGI120801[] = {{&S96848[44], 18}, {NULL}};
const Segment SGI120802[] = {{&S96848[30], 6}, {NULL}};
const Segment SGI120803[] = {{&S96848[31], 3}, {NULL}};
const Segment SGI120804[] = {{&S96848[76], 6}, {NULL}};
const Segment SGI120805[] = {{&S96848[130], 12}, {NULL}};
const Segment SGI120806[] = {{&S96898[34], 6}, {NULL}};
const Segment SGI120807[] = {{&S96898[27], 6}, {NULL}};
const Segment SGI120808[] = {{&S96898[93], 9}, {NULL}};
const Segment SGI120809[] = {{&S96898[66], 6}, {NULL}};
const Segment SGI120810[] = {{&S96898[127], 8}, {NULL}};
const Segment SGI120811[] = {{&S96901[34], 18}, {NULL}};
const Segment SGI120812[] = {{&S96901[27], 6}, {NULL}};
const Segment SGI120813[] = {{&S96901[105], 9}, {NULL}};
const Segment SGI120814[] = {{&S96901[78], 6}, {NULL}};
const Segment SGI120815[] = {{&S7671[44], 14}, {NULL}};
const Segment SGI120816[] = {{&S7671[44], 25}, {NULL}};
const Segment SGI120817[] = {{&S7671[30], 6}, {NULL}};
const Segment SGI120818[] = {{&S7671[31], 3}, {NULL}};
const Segment SGI120819[] = {{&S7671[83], 5}, {NULL}};
const Segment SGI120820[] = {{&S96954[44], 6}, {NULL}};
const Segment SGI120821[] = {{&S96954[44], 18}, {NULL}};
const Segment SGI120822[] = {{&S96954[30], 6}, {NULL}};
const Segment SGI120823[] = {{&S96954[107], 2}, {NULL}};
const Segment SGI120824[] = {{&S96954[76], 5}, {NULL}};
const Segment SGI120825[] = {{&S96956[44], 6}, {NULL}};
const Segment SGI120826[] = {{&S96956[44], 18}, {NULL}};
const Segment SGI120827[] = {{&S96956[30], 6}, {NULL}};
const Segment SGI120828[] = {{&S96956[107], 2}, {NULL}};
const Segment SGI120829[] = {{&S96956[76], 5}, {NULL}};
const Segment SGI120830[] = {{&S96957[44], 6}, {NULL}};
const Segment SGI120831[] = {{&S96957[44], 17}, {NULL}};
const Segment SGI120832[] = {{&S96957[30], 6}, {NULL}};
const Segment SGI120833[] = {{&S96957[106], 2}, {NULL}};
const Segment SGI120834[] = {{&S96957[75], 5}, {NULL}};
const Segment SGI120835[] = {{&S5019[44], 10}, {NULL}};
const Segment SGI120836[] = {{&S5019[44], 22}, {NULL}};
const Segment SGI120837[] = {{&S5019[30], 6}, {NULL}};
const Segment SGI120838[] = {{&S5019[115], 3}, {NULL}};
const Segment SGI120839[] = {{&S5019[80], 6}, {NULL}};
const Segment SGI120840[] = {{&S5020[44], 10}, {NULL}};
const Segment SGI120841[] = {{&S5020[44], 22}, {NULL}};
const Segment SGI120842[] = {{&S5020[30], 6}, {NULL}};
const Segment SGI120843[] = {{&S5020[115], 3}, {NULL}};
const Segment SGI120844[] = {{&S5020[80], 6}, {NULL}};
const Segment SGI120845[] = {{&S5021[44], 10}, {NULL}};
const Segment SGI120846[] = {{&S5021[44], 22}, {NULL}};
const Segment SGI120847[] = {{&S5021[30], 6}, {NULL}};
const Segment SGI120848[] = {{&S5021[115], 3}, {NULL}};
const Segment SGI120849[] = {{&S5021[80], 6}, {NULL}};
const Segment SGI120850[] = {{&S5022[44], 10}, {NULL}};
const Segment SGI120851[] = {{&S5022[44], 22}, {NULL}};
const Segment SGI120852[] = {{&S5022[30], 6}, {NULL}};
const Segment SGI120853[] = {{&S5022[115], 3}, {NULL}};
const Segment SGI120854[] = {{&S5022[80], 6}, {NULL}};
const Segment SGI120855[] = {{&S5023[44], 18}, {NULL}};
const Segment SGI120856[] = {{&S5023[44], 30}, {NULL}};
const Segment SGI120857[] = {{&S5023[30], 6}, {NULL}};
const Segment SGI120858[] = {{&S5023[123], 3}, {NULL}};
const Segment SGI120859[] = {{&S5023[88], 6}, {NULL}};
const Segment SGI120860[] = {{&S5024[44], 10}, {NULL}};
const Segment SGI120861[] = {{&S5024[44], 22}, {NULL}};
const Segment SGI120862[] = {{&S5024[30], 6}, {NULL}};
const Segment SGI120863[] = {{&S5024[115], 3}, {NULL}};
const Segment SGI120864[] = {{&S5024[80], 6}, {NULL}};
const Segment SGI120865[] = {{&S5025[44], 18}, {NULL}};
const Segment SGI120866[] = {{&S5025[44], 29}, {NULL}};
const Segment SGI120867[] = {{&S5025[30], 6}, {NULL}};
const Segment SGI120868[] = {{&S5025[122], 3}, {NULL}};
const Segment SGI120869[] = {{&S5025[87], 6}, {NULL}};
const Segment SGI120870[] = {{&S5026[44], 11}, {NULL}};
const Segment SGI120871[] = {{&S5026[44], 23}, {NULL}};
const Segment SGI120872[] = {{&S5026[30], 6}, {NULL}};
const Segment SGI120873[] = {{&S5026[116], 3}, {NULL}};
const Segment SGI120874[] = {{&S5026[81], 6}, {NULL}};
const Segment SGI120875[] = {{&S5027[44], 10}, {NULL}};
const Segment SGI120876[] = {{&S96996[34], 7}, {NULL}};
const Segment SGI120877[] = {{&S5027[44], 22}, {NULL}};
const Segment SGI120878[] = {{&S5027[30], 6}, {NULL}};
const Segment SGI120879[] = {{&S96996[27], 6}, {NULL}};
const Segment SGI120880[] = {{&S5027[115], 3}, {NULL}};
const Segment SGI120881[] = {{&S96996[94], 9}, {NULL}};
const Segment SGI120882[] = {{&S5027[80], 6}, {NULL}};
const Segment SGI120883[] = {{&S96996[67], 6}, {NULL}};
const Segment SGI120884[] = {{&S5028[44], 18}, {NULL}};
const Segment SGI120885[] = {{&S5028[44], 30}, {NULL}};
const Segment SGI120886[] = {{&S5028[30], 6}, {NULL}};
const Segment SGI120887[] = {{&S5028[123], 3}, {NULL}};
const Segment SGI120888[] = {{&S5028[88], 6}, {NULL}};
const Segment SGI120889[] = {{&S5029[44], 11}, {NULL}};
const Segment SGI120890[] = {{&S5029[44], 23}, {NULL}};
const Segment SGI120891[] = {{&S5029[30], 6}, {NULL}};
const Segment SGI120892[] = {{&S5029[116], 3}, {NULL}};
const Segment SGI120893[] = {{&S5029[81], 6}, {NULL}};
const Segment SGI120894[] = {{&S5030[44], 10}, {NULL}};
const Segment SGI120895[] = {{&S5030[44], 22}, {NULL}};
const Segment SGI120896[] = {{&S5030[30], 6}, {NULL}};
const Segment SGI120897[] = {{&S5030[115], 3}, {NULL}};
const Segment SGI120898[] = {{&S5030[80], 6}, {NULL}};
const Segment SGI120899[] = {{&S97003[44], 9}, {NULL}};
const Segment SGI120900[] = {{&S97003[44], 21}, {NULL}};
const Segment SGI120901[] = {{&S97003[30], 6}, {NULL}};
const Segment SGI120902[] = {{&S5031[44], 30}, {NULL}};
const Segment SGI120903[] = {{&S5031[44], 42}, {NULL}};
const Segment SGI120904[] = {{&S5031[30], 6}, {NULL}};
const Segment SGI120905[] = {{&S97003[31], 3}, {NULL}};
const Segment SGI120906[] = {{&S97003[79], 6}, {NULL}};
const Segment SGI120907[] = {{&S5031[135], 3}, {NULL}};
const Segment SGI120908[] = {{&S5031[100], 6}, {NULL}};
const Segment SGI120909[] = {{&S5032[44], 10}, {NULL}};
const Segment SGI120910[] = {{&S5032[44], 22}, {NULL}};
const Segment SGI120911[] = {{&S5032[30], 6}, {NULL}};
const Segment SGI120912[] = {{&S5032[115], 3}, {NULL}};
const Segment SGI120913[] = {{&S5032[80], 6}, {NULL}};
const Segment SGI120914[] = {{&S5033[44], 10}, {NULL}};
const Segment SGI120915[] = {{&S5033[44], 22}, {NULL}};
const Segment SGI120916[] = {{&S5033[30], 6}, {NULL}};
const Segment SGI120917[] = {{&S5033[115], 3}, {NULL}};
const Segment SGI120918[] = {{&S5033[80], 6}, {NULL}};
const Segment SGI120919[] = {{&S5034[38], 10}, {NULL}};
const Segment SGI120920[] = {{&S5034[31], 6}, {NULL}};
const Segment SGI120921[] = {{&S5036[38], 10}, {NULL}};
const Segment SGI120922[] = {{&S5036[31], 6}, {NULL}};
const Segment SGI120923[] = {{&S97012[34], 11}, {NULL}};
const Segment SGI120924[] = {{&S97012[27], 6}, {NULL}};
const Segment SGI120925[] = {{&S97012[98], 9}, {NULL}};
const Segment SGI120926[] = {{&S97012[71], 6}, {NULL}};
const Segment SGI120927[] = {{&S5037[44], 30}, {NULL}};
const Segment SGI120928[] = {{&S5037[44], 42}, {NULL}};
const Segment SGI120929[] = {{&S5037[30], 6}, {NULL}};
const Segment SGI120930[] = {{&S5037[135], 3}, {NULL}};
const Segment SGI120931[] = {{&S5037[100], 6}, {NULL}};
const Segment SGI120932[] = {{&S5039[44], 11}, {NULL}};
const Segment SGI120933[] = {{&S5039[44], 23}, {NULL}};
const Segment SGI120934[] = {{&S5039[30], 6}, {NULL}};
const Segment SGI120935[] = {{&S5039[116], 3}, {NULL}};
const Segment SGI120936[] = {{&S5039[81], 6}, {NULL}};
const Segment SGI120937[] = {{&S5041[44], 11}, {NULL}};
const Segment SGI120938[] = {{&S5041[44], 23}, {NULL}};
const Segment SGI120939[] = {{&S5041[30], 6}, {NULL}};
const Segment SGI120940[] = {{&S5041[116], 3}, {NULL}};
const Segment SGI120941[] = {{&S5041[81], 6}, {NULL}};
const Segment SGI120942[] = {{&S5042[44], 11}, {NULL}};
const Segment SGI120943[] = {{&S5042[44], 23}, {NULL}};
const Segment SGI120944[] = {{&S5042[30], 6}, {NULL}};
const Segment SGI120945[] = {{&S5042[116], 3}, {NULL}};
const Segment SGI120946[] = {{&S5042[81], 6}, {NULL}};
const Segment SGI120947[] = {{&S5043[44], 11}, {NULL}};
const Segment SGI120948[] = {{&S5043[44], 23}, {NULL}};
const Segment SGI120949[] = {{&S5043[30], 6}, {NULL}};
const Segment SGI120950[] = {{&S5043[116], 3}, {NULL}};
const Segment SGI120951[] = {{&S5043[81], 6}, {NULL}};
const Segment SGI120952[] = {{&S5044[38], 11}, {NULL}};
const Segment SGI120953[] = {{&S5044[31], 6}, {NULL}};
const Segment SGI120954[] = {{&S5045[44], 10}, {NULL}};
const Segment SGI120955[] = {{&S5045[44], 22}, {NULL}};
const Segment SGI120956[] = {{&S5045[30], 6}, {NULL}};
const Segment SGI120957[] = {{&S5045[80], 6}, {NULL}};
const Segment SGI120958[] = {{&S5046[42], 10}, {NULL}};
const Segment SGI120959[] = {{&S5046[30], 6}, {NULL}};
const Segment SGI120960[] = {{&S5046[113], 3}, {NULL}};
const Segment SGI120961[] = {{&S5046[78], 6}, {NULL}};
const Segment SGI120962[] = {{&S5048[38], 11}, {NULL}};
const Segment SGI120963[] = {{&S5048[31], 6}, {NULL}};
const Segment SGI120964[] = {{&S97032[34], 7}, {NULL}};
const Segment SGI120965[] = {{&S97032[27], 6}, {NULL}};
const Segment SGI120966[] = {{&S97032[95], 9}, {NULL}};
const Segment SGI120967[] = {{&S97032[68], 6}, {NULL}};
const Segment SGI120968[] = {{&S5049[38], 18}, {NULL}};
const Segment SGI120969[] = {{&S5049[31], 6}, {NULL}};
const Segment SGI120970[] = {{&S5050[44], 30}, {NULL}};
const Segment SGI120971[] = {{&S5050[44], 42}, {NULL}};
const Segment SGI120972[] = {{&S5050[30], 6}, {NULL}};
const Segment SGI120973[] = {{&S5050[135], 3}, {NULL}};
const Segment SGI120974[] = {{&S5050[100], 6}, {NULL}};
const Segment SGI120975[] = {{&S5051[44], 30}, {NULL}};
const Segment SGI120976[] = {{&S5051[44], 42}, {NULL}};
const Segment SGI120977[] = {{&S5051[30], 6}, {NULL}};
const Segment SGI120978[] = {{&S5051[135], 3}, {NULL}};
const Segment SGI120979[] = {{&S5051[100], 6}, {NULL}};
const Segment SGI120980[] = {{&S5052[44], 11}, {NULL}};
const Segment SGI120981[] = {{&S5052[44], 23}, {NULL}};
const Segment SGI120982[] = {{&S5052[30], 6}, {NULL}};
const Segment SGI120983[] = {{&S5052[116], 3}, {NULL}};
const Segment SGI120984[] = {{&S5052[81], 6}, {NULL}};
const Segment SGI120985[] = {{&S5054[44], 18}, {NULL}};
const Segment SGI120986[] = {{&S5054[44], 30}, {NULL}};
const Segment SGI120987[] = {{&S5054[30], 6}, {NULL}};
const Segment SGI120988[] = {{&S5054[123], 3}, {NULL}};
const Segment SGI120989[] = {{&S5054[88], 6}, {NULL}};
const Segment SGI120990[] = {{&S5055[42], 11}, {NULL}};
const Segment SGI120991[] = {{&S5055[30], 6}, {NULL}};
const Segment SGI120992[] = {{&S5055[114], 3}, {NULL}};
const Segment SGI120993[] = {{&S5055[79], 6}, {NULL}};
const Segment SGI120994[] = {{&S5056[44], 11}, {NULL}};
const Segment SGI120995[] = {{&S5056[44], 23}, {NULL}};
const Segment SGI120996[] = {{&S5056[30], 6}, {NULL}};
const Segment SGI120997[] = {{&S5056[116], 3}, {NULL}};
const Segment SGI120998[] = {{&S5056[81], 6}, {NULL}};
const Segment SGI120999[] = {{&S5060[38], 11}, {NULL}};
const Segment SGI121000[] = {{&S5060[31], 6}, {NULL}};
const Segment SGI121001[] = {{&S5061[44], 18}, {NULL}};
const Segment SGI121002[] = {{&S5061[44], 30}, {NULL}};
const Segment SGI121003[] = {{&S5061[30], 6}, {NULL}};
const Segment SGI121004[] = {{&S5061[123], 3}, {NULL}};
const Segment SGI121005[] = {{&S5061[88], 6}, {NULL}};
const Segment SGI121006[] = {{&S7713[44], 12}, {NULL}};
const Segment SGI121007[] = {{&S7713[44], 24}, {NULL}};
const Segment SGI121008[] = {{&S7713[30], 6}, {NULL}};
const Segment SGI121009[] = {{&S7713[31], 3}, {NULL}};
const Segment SGI121010[] = {{&S7713[82], 6}, {NULL}};
const Segment SGI121011[] = {{&S5065[44], 30}, {NULL}};
const Segment SGI121012[] = {{&S5065[44], 42}, {NULL}};
const Segment SGI121013[] = {{&S5065[30], 6}, {NULL}};
const Segment SGI121014[] = {{&S5065[135], 3}, {NULL}};
const Segment SGI121015[] = {{&S5065[100], 6}, {NULL}};
const Segment SGI121016[] = {{&S5074[42], 18}, {NULL}};
const Segment SGI121017[] = {{&S5074[30], 6}, {NULL}};
const Segment SGI121018[] = {{&S5074[121], 3}, {NULL}};
const Segment SGI121019[] = {{&S5074[86], 6}, {NULL}};
const Segment SGI121020[] = {{&S5080[38], 11}, {NULL}};
const Segment SGI121021[] = {{&S5080[31], 6}, {NULL}};
const Segment SGI121022[] = {{&S4348[44], 10}, {NULL}};
const Segment SGI121023[] = {{&S4348[44], 22}, {NULL}};
const Segment SGI121024[] = {{&S4348[30], 6}, {NULL}};
const Segment SGI121025[] = {{&S4348[115], 3}, {NULL}};
const Segment SGI121026[] = {{&S4348[80], 6}, {NULL}};
const Segment SGI121027[] = {{&S5081[38], 11}, {NULL}};
const Segment SGI121028[] = {{&S5081[31], 6}, {NULL}};
const Segment SGI121029[] = {{&S4355[38], 10}, {NULL}};
const Segment SGI121030[] = {{&S4355[31], 6}, {NULL}};
const Segment SGI121031[] = {{&S4356[44], 18}, {NULL}};
const Segment SGI121032[] = {{&S4356[44], 30}, {NULL}};
const Segment SGI121033[] = {{&S4356[30], 6}, {NULL}};
const Segment SGI121034[] = {{&S4356[123], 3}, {NULL}};
const Segment SGI121035[] = {{&S4356[88], 6}, {NULL}};
const Segment SGI121036[] = {{&S7743[44], 7}, {NULL}};
const Segment SGI121037[] = {{&S7743[44], 19}, {NULL}};
const Segment SGI121038[] = {{&S7743[30], 6}, {NULL}};
const Segment SGI121039[] = {{&S7743[31], 3}, {NULL}};
const Segment SGI121040[] = {{&S7743[77], 6}, {NULL}};
const Segment SGI121041[] = {{&S4357[44], 11}, {NULL}};
const Segment SGI121042[] = {{&S4357[44], 23}, {NULL}};
const Segment SGI121043[] = {{&S4357[30], 6}, {NULL}};
const Segment SGI121044[] = {{&S4357[116], 3}, {NULL}};
const Segment SGI121045[] = {{&S97052[45], 9}, {NULL}};
const Segment SGI121046[] = {{&S4357[81], 6}, {NULL}};
const Segment SGI121047[] = {{&S97052[30], 6}, {NULL}};
const Segment SGI121048[] = {{&S97052[31], 3}, {NULL}};
const Segment SGI121049[] = {{&S97052[80], 5}, {NULL}};
const Segment SGI121050[] = {{&S97052[138], 13}, {NULL}};
const Segment SGI121051[] = {{&S4358[44], 10}, {NULL}};
const Segment SGI121052[] = {{&S4358[44], 22}, {NULL}};
const Segment SGI121053[] = {{&S4358[30], 6}, {NULL}};
const Segment SGI121054[] = {{&S4358[115], 3}, {NULL}};
const Segment SGI121055[] = {{&S4358[80], 6}, {NULL}};
const Segment SGI121056[] = {{&S4360[44], 10}, {NULL}};
const Segment SGI121057[] = {{&S4360[44], 22}, {NULL}};
const Segment SGI121058[] = {{&S4360[30], 6}, {NULL}};
const Segment SGI121059[] = {{&S4360[115], 3}, {NULL}};
const Segment SGI121060[] = {{&S4360[80], 6}, {NULL}};
const Segment SGI121061[] = {{&S4361[44], 10}, {NULL}};
const Segment SGI121062[] = {{&S4361[44], 22}, {NULL}};
const Segment SGI121063[] = {{&S4361[30], 6}, {NULL}};
const Segment SGI121064[] = {{&S4361[115], 3}, {NULL}};
const Segment SGI121065[] = {{&S4361[80], 6}, {NULL}};
const Segment SGI121066[] = {{&S4363[38], 18}, {NULL}};
const Segment SGI121067[] = {{&S4363[31], 6}, {NULL}};
const Segment SGI121068[] = {{&S4364[44], 10}, {NULL}};
const Segment SGI121069[] = {{&S4364[44], 22}, {NULL}};
const Segment SGI121070[] = {{&S4364[30], 6}, {NULL}};
const Segment SGI121071[] = {{&S4364[115], 3}, {NULL}};
const Segment SGI121072[] = {{&S4364[80], 6}, {NULL}};
const Segment SGI121073[] = {{&S4365[44], 10}, {NULL}};
const Segment SGI121074[] = {{&S4365[44], 22}, {NULL}};
const Segment SGI121075[] = {{&S4365[30], 6}, {NULL}};
const Segment SGI121076[] = {{&S7757[44], 11}, {NULL}};
const Segment SGI121077[] = {{&S7757[44], 23}, {NULL}};
const Segment SGI121078[] = {{&S4365[115], 3}, {NULL}};
const Segment SGI121079[] = {{&S7757[30], 6}, {NULL}};
const Segment SGI121080[] = {{&S7757[31], 3}, {NULL}};
const Segment SGI121081[] = {{&S4365[80], 6}, {NULL}};
const Segment SGI121082[] = {{&S7757[81], 6}, {NULL}};
const Segment SGI121083[] = {{&S4366[44], 30}, {NULL}};
const Segment SGI121084[] = {{&S4366[44], 42}, {NULL}};
const Segment SGI121085[] = {{&S4366[30], 6}, {NULL}};
const Segment SGI121086[] = {{&S4366[135], 3}, {NULL}};
const Segment SGI121087[] = {{&S4366[100], 6}, {NULL}};
const Segment SGI121088[] = {{&S97068[44], 10}, {NULL}};
const Segment SGI121089[] = {{&S97068[44], 22}, {NULL}};
const Segment SGI121090[] = {{&S97068[30], 6}, {NULL}};
const Segment SGI121091[] = {{&S4367[44], 30}, {NULL}};
const Segment SGI121092[] = {{&S4367[44], 42}, {NULL}};
const Segment SGI121093[] = {{&S97068[31], 3}, {NULL}};
const Segment SGI121094[] = {{&S4367[30], 6}, {NULL}};
const Segment SGI121095[] = {{&S97068[80], 6}, {NULL}};
const Segment SGI121096[] = {{&S4367[135], 3}, {NULL}};
const Segment SGI121097[] = {{&S4367[100], 6}, {NULL}};
const Segment SGI121098[] = {{&S4368[44], 11}, {NULL}};
const Segment SGI121099[] = {{&S4368[44], 23}, {NULL}};
const Segment SGI121100[] = {{&S4368[30], 6}, {NULL}};
const Segment SGI121101[] = {{&S4368[116], 3}, {NULL}};
const Segment SGI121102[] = {{&S4368[81], 6}, {NULL}};
const Segment SGI121103[] = {{&S4369[44], 11}, {NULL}};
const Segment SGI121104[] = {{&S4369[44], 23}, {NULL}};
const Segment SGI121105[] = {{&S4369[30], 6}, {NULL}};
const Segment SGI121106[] = {{&S4369[116], 3}, {NULL}};
const Segment SGI121107[] = {{&S4369[81], 6}, {NULL}};
const Segment SGI121108[] = {{&S4373[38], 10}, {NULL}};
const Segment SGI121109[] = {{&S4373[31], 6}, {NULL}};
const Segment SGI121110[] = {{&S4377[44], 10}, {NULL}};
const Segment SGI121111[] = {{&S4377[44], 22}, {NULL}};
const Segment SGI121112[] = {{&S4377[30], 6}, {NULL}};
const Segment SGI121113[] = {{&S4377[115], 3}, {NULL}};
const Segment SGI121114[] = {{&S4377[80], 6}, {NULL}};
const Segment SGI121115[] = {{&S97082[34], 8}, {NULL}};
const Segment SGI121116[] = {{&S97082[27], 6}, {NULL}};
const Segment SGI121117[] = {{&S97082[95], 9}, {NULL}};
const Segment SGI121118[] = {{&S97082[68], 6}, {NULL}};
const Segment SGI121119[] = {{&S4378[38], 11}, {NULL}};
const Segment SGI121120[] = {{&S4378[31], 6}, {NULL}};
const Segment SGI121121[] = {{&S4379[44], 10}, {NULL}};
const Segment SGI121122[] = {{&S4379[44], 22}, {NULL}};
const Segment SGI121123[] = {{&S4379[30], 6}, {NULL}};
const Segment SGI121124[] = {{&S4379[83], 6}, {NULL}};
const Segment SGI121125[] = {{&S4380[44], 11}, {NULL}};
const Segment SGI121126[] = {{&S4380[44], 23}, {NULL}};
const Segment SGI121127[] = {{&S4380[30], 6}, {NULL}};
const Segment SGI121128[] = {{&S4380[116], 3}, {NULL}};
const Segment SGI121129[] = {{&S4380[81], 6}, {NULL}};
const Segment SGI121130[] = {{&S97088[34], 13}, {NULL}};
const Segment SGI121131[] = {{&S97088[27], 6}, {NULL}};
const Segment SGI121132[] = {{&S97088[104], 9}, {NULL}};
const Segment SGI121133[] = {{&S97088[77], 6}, {NULL}};
const Segment SGI121134[] = {{&S4383[38], 11}, {NULL}};
const Segment SGI121135[] = {{&S4383[31], 6}, {NULL}};
const Segment SGI121136[] = {{&S4385[44], 30}, {NULL}};
const Segment SGI121137[] = {{&S4385[44], 42}, {NULL}};
const Segment SGI121138[] = {{&S4385[30], 6}, {NULL}};
const Segment SGI121139[] = {{&S4385[135], 3}, {NULL}};
const Segment SGI121140[] = {{&S4385[100], 6}, {NULL}};
const Segment SGI121141[] = {{&S4386[44], 11}, {NULL}};
const Segment SGI121142[] = {{&S4386[44], 23}, {NULL}};
const Segment SGI121143[] = {{&S4386[30], 6}, {NULL}};
const Segment SGI121144[] = {{&S4386[116], 3}, {NULL}};
const Segment SGI121145[] = {{&S4386[81], 6}, {NULL}};
const Segment SGI121146[] = {{&S4389[38], 10}, {NULL}};
const Segment SGI121147[] = {{&S4389[31], 6}, {NULL}};
const Segment SGI121148[] = {{&S4390[44], 11}, {NULL}};
const Segment SGI121149[] = {{&S4390[44], 23}, {NULL}};
const Segment SGI121150[] = {{&S4390[30], 6}, {NULL}};
const Segment SGI121151[] = {{&S4390[116], 3}, {NULL}};
const Segment SGI121152[] = {{&S4390[81], 6}, {NULL}};
const Segment SGI121153[] = {{&S4391[44], 11}, {NULL}};
const Segment SGI121154[] = {{&S4391[44], 23}, {NULL}};
const Segment SGI121155[] = {{&S4391[30], 6}, {NULL}};
const Segment SGI121156[] = {{&S4391[116], 3}, {NULL}};
const Segment SGI121157[] = {{&S4391[81], 6}, {NULL}};
const Segment SGI121158[] = {{&S4392[44], 30}, {NULL}};
const Segment SGI121159[] = {{&S4392[44], 42}, {NULL}};
const Segment SGI121160[] = {{&S4392[30], 6}, {NULL}};
const Segment SGI121161[] = {{&S4392[135], 3}, {NULL}};
const Segment SGI121162[] = {{&S4392[100], 6}, {NULL}};
const Segment SGI121163[] = {{&S4393[44], 30}, {NULL}};
const Segment SGI121164[] = {{&S4393[44], 42}, {NULL}};
const Segment SGI121165[] = {{&S4393[30], 6}, {NULL}};
const Segment SGI121166[] = {{&S4393[135], 3}, {NULL}};
const Segment SGI121167[] = {{&S4393[100], 6}, {NULL}};
const Segment SGI121168[] = {{&S5083[44], 18}, {NULL}};
const Segment SGI121169[] = {{&S4398[49], 10}, {NULL}};
const Segment SGI121170[] = {{&S5083[44], 30}, {NULL}};
const Segment SGI121171[] = {{&S4398[49], 22}, {NULL}};
const Segment SGI121172[] = {{&S5083[30], 6}, {NULL}};
const Segment SGI121173[] = {{&S4398[30], 11}, {NULL}};
const Segment SGI121174[] = {{&S4398[120], 3}, {NULL}};
const Segment SGI121175[] = {{&S5083[91], 6}, {NULL}};
const Segment SGI121176[] = {{&S4398[85], 6}, {NULL}};
const Segment SGI121177[] = {{&S4399[38], 11}, {NULL}};
const Segment SGI121178[] = {{&S4399[31], 6}, {NULL}};
const Segment SGI121179[] = {{&S4400[44], 10}, {NULL}};
const Segment SGI121180[] = {{&S4400[44], 22}, {NULL}};
const Segment SGI121181[] = {{&S4400[30], 6}, {NULL}};
const Segment SGI121182[] = {{&S4400[115], 3}, {NULL}};
const Segment SGI121183[] = {{&S4400[80], 6}, {NULL}};
const Segment SGI121184[] = {{&S4402[44], 30}, {NULL}};
const Segment SGI121185[] = {{&S4402[44], 42}, {NULL}};
const Segment SGI121186[] = {{&S4402[30], 6}, {NULL}};
const Segment SGI121187[] = {{&S4402[135], 3}, {NULL}};
const Segment SGI121188[] = {{&S4402[100], 6}, {NULL}};
const Segment SGI121189[] = {{&S5090[44], 11}, {NULL}};
const Segment SGI121190[] = {{&S5090[44], 23}, {NULL}};
const Segment SGI121191[] = {{&S5090[30], 6}, {NULL}};
const Segment SGI121192[] = {{&S5090[84], 6}, {NULL}};
const Segment SGI121193[] = {{&S4403[44], 29}, {NULL}};
const Segment SGI121194[] = {{&S4403[44], 41}, {NULL}};
const Segment SGI121195[] = {{&S4403[30], 6}, {NULL}};
const Segment SGI121196[] = {{&S4403[134], 3}, {NULL}};
const Segment SGI121197[] = {{&S4403[99], 6}, {NULL}};
const Segment SGI121198[] = {{&S4404[44], 30}, {NULL}};
const Segment SGI121199[] = {{&S4404[44], 42}, {NULL}};
const Segment SGI121200[] = {{&S4404[30], 6}, {NULL}};
const Segment SGI121201[] = {{&S4404[135], 3}, {NULL}};
const Segment SGI121202[] = {{&S4404[100], 6}, {NULL}};
const Segment SGI121203[] = {{&S5094[44], 18}, {NULL}};
const Segment SGI121204[] = {{&S5094[44], 30}, {NULL}};
const Segment SGI121205[] = {{&S5094[30], 6}, {NULL}};
const Segment SGI121206[] = {{&S4405[44], 29}, {NULL}};
const Segment SGI121207[] = {{&S4405[44], 41}, {NULL}};
const Segment SGI121208[] = {{&S5094[88], 6}, {NULL}};
const Segment SGI121209[] = {{&S4405[30], 6}, {NULL}};
const Segment SGI121210[] = {{&S4405[134], 3}, {NULL}};
const Segment SGI121211[] = {{&S97125[34], 13}, {NULL}};
const Segment SGI121212[] = {{&S4405[99], 6}, {NULL}};
const Segment SGI121213[] = {{&S97125[27], 6}, {NULL}};
const Segment SGI121214[] = {{&S7821[44], 7}, {NULL}};
const Segment SGI121215[] = {{&S7821[44], 19}, {NULL}};
const Segment SGI121216[] = {{&S7821[30], 6}, {NULL}};
const Segment SGI121217[] = {{&S7821[31], 3}, {NULL}};
const Segment SGI121218[] = {{&S7821[77], 5}, {NULL}};
const Segment SGI121219[] = {{&S97125[100], 9}, {NULL}};
const Segment SGI121220[] = {{&S97125[73], 6}, {NULL}};
const Segment SGI121221[] = {{&S97125[141], 8}, {NULL}};
const Segment SGI121222[] = {{&S4406[38], 10}, {NULL}};
const Segment SGI121223[] = {{&S4406[31], 6}, {NULL}};
const Segment SGI121224[] = {{&S5096[38], 10}, {NULL}};
const Segment SGI121225[] = {{&S5096[31], 6}, {NULL}};
const Segment SGI121226[] = {{&S4407[38], 10}, {NULL}};
const Segment SGI121227[] = {{&S4407[31], 6}, {NULL}};
const Segment SGI121228[] = {{&S5097[38], 11}, {NULL}};
const Segment SGI121229[] = {{&S5097[31], 6}, {NULL}};
const Segment SGI121230[] = {{&S4408[44], 10}, {NULL}};
const Segment SGI121231[] = {{&S4408[44], 22}, {NULL}};
const Segment SGI121232[] = {{&S4408[30], 6}, {NULL}};
const Segment SGI121233[] = {{&S97128[34], 8}, {NULL}};
const Segment SGI121234[] = {{&S97128[27], 6}, {NULL}};
const Segment SGI121235[] = {{&S4408[115], 3}, {NULL}};
const Segment SGI121236[] = {{&S5098[38], 11}, {NULL}};
const Segment SGI121237[] = {{&S5098[31], 6}, {NULL}};
const Segment SGI121238[] = {{&S4408[80], 6}, {NULL}};
const Segment SGI121239[] = {{&S97128[95], 9}, {NULL}};
const Segment SGI121240[] = {{&S97128[68], 6}, {NULL}};
const Segment SGI121241[] = {{&S4411[38], 11}, {NULL}};
const Segment SGI121242[] = {{&S4411[31], 6}, {NULL}};
const Segment SGI121243[] = {{&S4412[42], 10}, {NULL}};
const Segment SGI121244[] = {{&S4412[30], 6}, {NULL}};
const Segment SGI121245[] = {{&S4412[113], 3}, {NULL}};
const Segment SGI121246[] = {{&S4412[78], 6}, {NULL}};
const Segment SGI121247[] = {{&S7832[44], 11}, {NULL}};
const Segment SGI121248[] = {{&S7832[44], 23}, {NULL}};
const Segment SGI121249[] = {{&S4414[44], 10}, {NULL}};
const Segment SGI121250[] = {{&S7832[30], 6}, {NULL}};
const Segment SGI121251[] = {{&S7832[31], 3}, {NULL}};
const Segment SGI121252[] = {{&S4414[44], 22}, {NULL}};
const Segment SGI121253[] = {{&S7832[81], 6}, {NULL}};
const Segment SGI121254[] = {{&S4414[30], 6}, {NULL}};
const Segment SGI121255[] = {{&S4414[115], 3}, {NULL}};
const Segment SGI121256[] = {{&S4414[80], 6}, {NULL}};
const Segment SGI121257[] = {{&S97136[34], 7}, {NULL}};
const Segment SGI121258[] = {{&S97136[27], 6}, {NULL}};
const Segment SGI121259[] = {{&S97136[95], 9}, {NULL}};
const Segment SGI121260[] = {{&S97136[68], 6}, {NULL}};
const Segment SGI121261[] = {{&S4416[44], 11}, {NULL}};
const Segment SGI121262[] = {{&S4416[44], 23}, {NULL}};
const Segment SGI121263[] = {{&S4416[30], 6}, {NULL}};
const Segment SGI121264[] = {{&S4416[116], 3}, {NULL}};
const Segment SGI121265[] = {{&S4416[81], 6}, {NULL}};
const Segment SGI121266[] = {{&S4419[44], 10}, {NULL}};
const Segment SGI121267[] = {{&S4419[44], 22}, {NULL}};
const Segment SGI121268[] = {{&S4419[30], 6}, {NULL}};
const Segment SGI121269[] = {{&S4419[114], 3}, {NULL}};
const Segment SGI121270[] = {{&S4419[80], 5}, {NULL}};
const Segment SGI121271[] = {{&S97141[59], 6}, {NULL}};
const Segment SGI121272[] = {{&S97141[59], 18}, {NULL}};
const Segment SGI121273[] = {{&S97141[45], 6}, {NULL}};
const Segment SGI121274[] = {{&S5113[44], 29}, {NULL}};
const Segment SGI121275[] = {{&S4420[42], 18}, {NULL}};
const Segment SGI121276[] = {{&S97141[46], 3}, {NULL}};
const Segment SGI121277[] = {{&S5113[44], 41}, {NULL}};
const Segment SGI121278[] = {{&S5113[30], 6}, {NULL}};
const Segment SGI121279[] = {{&S4420[30], 6}, {NULL}};
const Segment SGI121280[] = {{&S97141[91], 6}, {NULL}};
const Segment SGI121281[] = {{&S97141[0], 14}, {NULL}};
const Segment SGI121282[] = {{&S4420[121], 3}, {NULL}};
const Segment SGI121283[] = {{&S5113[134], 3}, {NULL}};
const Segment SGI121284[] = {{&S4420[86], 6}, {NULL}};
const Segment SGI121285[] = {{&S5113[99], 6}, {NULL}};
const Segment SGI121286[] = {{&S4421[44], 30}, {NULL}};
const Segment SGI121287[] = {{&S4421[44], 42}, {NULL}};
const Segment SGI121288[] = {{&S4421[30], 6}, {NULL}};
const Segment SGI121289[] = {{&S4421[135], 3}, {NULL}};
const Segment SGI121290[] = {{&S4421[100], 6}, {NULL}};
const Segment SGI121291[] = {{&S4422[38], 11}, {NULL}};
const Segment SGI121292[] = {{&S4422[31], 6}, {NULL}};
const Segment SGI121293[] = {{&S4423[44], 18}, {NULL}};
const Segment SGI121294[] = {{&S4423[44], 30}, {NULL}};
const Segment SGI121295[] = {{&S4423[30], 6}, {NULL}};
const Segment SGI121296[] = {{&S4423[123], 3}, {NULL}};
const Segment SGI121297[] = {{&S4423[88], 6}, {NULL}};
const Segment SGI121298[] = {{&S97148[44], 10}, {NULL}};
const Segment SGI121299[] = {{&S97148[44], 22}, {NULL}};
const Segment SGI121300[] = {{&S97148[30], 6}, {NULL}};
const Segment SGI121301[] = {{&S4424[44], 30}, {NULL}};
const Segment SGI121302[] = {{&S4424[44], 42}, {NULL}};
const Segment SGI121303[] = {{&S4424[30], 6}, {NULL}};
const Segment SGI121304[] = {{&S97148[31], 3}, {NULL}};
const Segment SGI121305[] = {{&S97148[80], 5}, {NULL}};
const Segment SGI121306[] = {{&S4424[135], 3}, {NULL}};
const Segment SGI121307[] = {{&S97148[117], 14}, {NULL}};
const Segment SGI121308[] = {{&S4424[100], 6}, {NULL}};
const Segment SGI121309[] = {{&S4425[55], 10}, {NULL}};
const Segment SGI121310[] = {{&S4425[55], 22}, {NULL}};
const Segment SGI121311[] = {{&S4425[30], 17}, {NULL}};
const Segment SGI121312[] = {{&S4425[126], 3}, {NULL}};
const Segment SGI121313[] = {{&S4425[91], 6}, {NULL}};
const Segment SGI121314[] = {{&S4427[44], 29}, {NULL}};
const Segment SGI121315[] = {{&S4427[44], 41}, {NULL}};
const Segment SGI121316[] = {{&S4427[30], 6}, {NULL}};
const Segment SGI121317[] = {{&S4427[134], 3}, {NULL}};
const Segment SGI121318[] = {{&S4427[99], 6}, {NULL}};
const Segment SGI121319[] = {{&S4428[38], 10}, {NULL}};
const Segment SGI121320[] = {{&S4428[31], 6}, {NULL}};
const Segment SGI121321[] = {{&S97155[44], 18}, {NULL}};
const Segment SGI121322[] = {{&S97155[44], 30}, {NULL}};
const Segment SGI121323[] = {{&S97155[30], 6}, {NULL}};
const Segment SGI121324[] = {{&S97155[31], 3}, {NULL}};
const Segment SGI121325[] = {{&S97155[88], 5}, {NULL}};
const Segment SGI121326[] = {{&S4429[38], 11}, {NULL}};
const Segment SGI121327[] = {{&S4429[31], 6}, {NULL}};
const Segment SGI121328[] = {{&S97155[125], 14}, {NULL}};
const Segment SGI121329[] = {{&S4433[44], 18}, {NULL}};
const Segment SGI121330[] = {{&S4433[44], 30}, {NULL}};
const Segment SGI121331[] = {{&S4433[30], 6}, {NULL}};
const Segment SGI121332[] = {{&S4433[123], 3}, {NULL}};
const Segment SGI121333[] = {{&S4433[88], 6}, {NULL}};
const Segment SGI121334[] = {{&S4434[38], 10}, {NULL}};
const Segment SGI121335[] = {{&S4434[31], 6}, {NULL}};
const Segment SGI121336[] = {{&S4435[42], 10}, {NULL}};
const Segment SGI121337[] = {{&S4435[30], 6}, {NULL}};
const Segment SGI121338[] = {{&S4435[113], 3}, {NULL}};
const Segment SGI121339[] = {{&S4435[78], 6}, {NULL}};
const Segment SGI121340[] = {{&S4436[44], 30}, {NULL}};
const Segment SGI121341[] = {{&S4436[44], 42}, {NULL}};
const Segment SGI121342[] = {{&S4436[30], 6}, {NULL}};
const Segment SGI121343[] = {{&S4436[135], 3}, {NULL}};
const Segment SGI121344[] = {{&S4436[100], 6}, {NULL}};
const Segment SGI121345[] = {{&S4441[42], 10}, {NULL}};
const Segment SGI121346[] = {{&S4441[30], 6}, {NULL}};
const Segment SGI121347[] = {{&S4441[113], 3}, {NULL}};
const Segment SGI121348[] = {{&S4441[78], 6}, {NULL}};
const Segment SGI121349[] = {{&S4443[42], 10}, {NULL}};
const Segment SGI121350[] = {{&S4443[30], 6}, {NULL}};
const Segment SGI121351[] = {{&S4443[113], 3}, {NULL}};
const Segment SGI121352[] = {{&S4443[78], 6}, {NULL}};
const Segment SGI121353[] = {{&S4444[44], 18}, {NULL}};
const Segment SGI121354[] = {{&S4444[44], 30}, {NULL}};
const Segment SGI121355[] = {{&S4444[30], 6}, {NULL}};
const Segment SGI121356[] = {{&S4444[123], 3}, {NULL}};
const Segment SGI121357[] = {{&S4444[88], 6}, {NULL}};
const Segment SGI121358[] = {{&S97177[34], 10}, {NULL}};
const Segment SGI121359[] = {{&S97177[27], 6}, {NULL}};
const Segment SGI121360[] = {{&S4446[44], 10}, {NULL}};
const Segment SGI121361[] = {{&S4446[44], 25}, {NULL}};
const Segment SGI121362[] = {{&S4446[30], 6}, {NULL}};
const Segment SGI121363[] = {{&S97177[97], 9}, {NULL}};
const Segment SGI121364[] = {{&S97177[70], 6}, {NULL}};
const Segment SGI121365[] = {{&S4446[118], 3}, {NULL}};
const Segment SGI121366[] = {{&S4446[83], 6}, {NULL}};
const Segment SGI121367[] = {{&S4448[44], 30}, {NULL}};
const Segment SGI121368[] = {{&S4448[44], 42}, {NULL}};
const Segment SGI121369[] = {{&S4448[30], 6}, {NULL}};
const Segment SGI121370[] = {{&S4448[135], 3}, {NULL}};
const Segment SGI121371[] = {{&S4448[100], 6}, {NULL}};
const Segment SGI121372[] = {{&S4449[44], 18}, {NULL}};
const Segment SGI121373[] = {{&S4449[44], 30}, {NULL}};
const Segment SGI121374[] = {{&S4449[30], 6}, {NULL}};
const Segment SGI121375[] = {{&S4449[123], 3}, {NULL}};
const Segment SGI121376[] = {{&S4449[88], 6}, {NULL}};
const Segment SGI121377[] = {{&S4452[44], 10}, {NULL}};
const Segment SGI121378[] = {{&S4452[44], 22}, {NULL}};
const Segment SGI121379[] = {{&S4452[30], 6}, {NULL}};
const Segment SGI121380[] = {{&S4452[83], 6}, {NULL}};
const Segment SGI121381[] = {{&S4453[44], 10}, {NULL}};
const Segment SGI121382[] = {{&S4453[44], 22}, {NULL}};
const Segment SGI121383[] = {{&S4453[30], 6}, {NULL}};
const Segment SGI121384[] = {{&S4453[115], 3}, {NULL}};
const Segment SGI121385[] = {{&S4453[80], 6}, {NULL}};
const Segment SGI121386[] = {{&S97193[44], 6}, {NULL}};
const Segment SGI121387[] = {{&S97193[44], 18}, {NULL}};
const Segment SGI121388[] = {{&S97193[30], 6}, {NULL}};
const Segment SGI121389[] = {{&S97193[76], 6}, {NULL}};
const Segment SGI121390[] = {{&S97193[103], 13}, {NULL}};
const Segment SGI121391[] = {{&S97201[34], 18}, {NULL}};
const Segment SGI121392[] = {{&S97201[27], 6}, {NULL}};
const Segment SGI121393[] = {{&S97201[105], 9}, {NULL}};
const Segment SGI121394[] = {{&S97201[78], 6}, {NULL}};
const Segment SGI121395[] = {{&S97201[139], 8}, {NULL}};
const Segment SGI121396[] = {{&S4454[44], 30}, {NULL}};
const Segment SGI121397[] = {{&S4454[44], 42}, {NULL}};
const Segment SGI121398[] = {{&S4454[30], 6}, {NULL}};
const Segment SGI121399[] = {{&S4454[135], 3}, {NULL}};
const Segment SGI121400[] = {{&S4454[100], 6}, {NULL}};
const Segment SGI121401[] = {{&S4455[44], 29}, {NULL}};
const Segment SGI121402[] = {{&S4455[44], 41}, {NULL}};
const Segment SGI121403[] = {{&S4455[30], 6}, {NULL}};
const Segment SGI121404[] = {{&S4455[134], 3}, {NULL}};
const Segment SGI121405[] = {{&S4455[99], 6}, {NULL}};
const Segment SGI121406[] = {{&S4456[44], 29}, {NULL}};
const Segment SGI121407[] = {{&S4456[44], 41}, {NULL}};
const Segment SGI121408[] = {{&S4456[30], 6}, {NULL}};
const Segment SGI121409[] = {{&S4456[134], 3}, {NULL}};
const Segment SGI121410[] = {{&S4456[99], 6}, {NULL}};
const Segment SGI121411[] = {{&S97223[44], 9}, {NULL}};
const Segment SGI121412[] = {{&S97223[44], 21}, {NULL}};
const Segment SGI121413[] = {{&S97223[30], 6}, {NULL}};
const Segment SGI121414[] = {{&S97223[79], 6}, {NULL}};
const Segment SGI121415[] = {{&S97223[106], 13}, {NULL}};
const Segment SGI121416[] = {{&S7939[44], 6}, {NULL}};
const Segment SGI121417[] = {{&S7939[44], 18}, {NULL}};
const Segment SGI121418[] = {{&S7939[30], 6}, {NULL}};
const Segment SGI121419[] = {{&S7939[31], 3}, {NULL}};
const Segment SGI121420[] = {{&S7939[76], 6}, {NULL}};
const Segment SGI121421[] = {{&S7975[44], 6}, {NULL}};
const Segment SGI121422[] = {{&S7975[44], 18}, {NULL}};
const Segment SGI121423[] = {{&S7975[30], 6}, {NULL}};
const Segment SGI121424[] = {{&S7975[31], 3}, {NULL}};
const Segment SGI121425[] = {{&S7975[76], 5}, {NULL}};
const Segment SGI121426[] = {{&S97301[45], 11}, {NULL}};
const Segment SGI121427[] = {{&S97301[30], 6}, {NULL}};
const Segment SGI121428[] = {{&S7998[44], 18}, {NULL}};
const Segment SGI121429[] = {{&S7998[44], 30}, {NULL}};
const Segment SGI121430[] = {{&S7998[30], 6}, {NULL}};
const Segment SGI121431[] = {{&S7998[31], 3}, {NULL}};
const Segment SGI121432[] = {{&S97301[31], 3}, {NULL}};
const Segment SGI121433[] = {{&S7998[88], 6}, {NULL}};
const Segment SGI121434[] = {{&S97301[82], 5}, {NULL}};
const Segment SGI121435[] = {{&S97301[140], 13}, {NULL}};
const Segment SGI121436[] = {{&S97307[44], 9}, {NULL}};
const Segment SGI121437[] = {{&S97307[44], 35}, {NULL}};
const Segment SGI121438[] = {{&S97307[30], 6}, {NULL}};
const Segment SGI121439[] = {{&S97307[31], 3}, {NULL}};
const Segment SGI121440[] = {{&S97307[93], 6}, {NULL}};
const Segment SGI121441[] = {{&S97308[44], 11}, {NULL}};
const Segment SGI121442[] = {{&S97308[44], 23}, {NULL}};
const Segment SGI121443[] = {{&S97308[30], 6}, {NULL}};
const Segment SGI121444[] = {{&S97308[31], 3}, {NULL}};
const Segment SGI121445[] = {{&S97308[81], 6}, {NULL}};
const Segment SGI121446[] = {{&S8019[44], 18}, {NULL}};
const Segment SGI121447[] = {{&S8019[44], 30}, {NULL}};
const Segment SGI121448[] = {{&S8019[30], 6}, {NULL}};
const Segment SGI121449[] = {{&S8019[31], 3}, {NULL}};
const Segment SGI121450[] = {{&S8019[88], 5}, {NULL}};
const Segment SGI121451[] = {{&S97321[45], 14}, {NULL}};
const Segment SGI121452[] = {{&S97321[30], 6}, {NULL}};
const Segment SGI121453[] = {{&S97321[31], 3}, {NULL}};
const Segment SGI121454[] = {{&S97321[85], 5}, {NULL}};
const Segment SGI121455[] = {{&S97321[143], 13}, {NULL}};
const Segment SGI121456[] = {{&S4458[68], 10}, {NULL}};
const Segment SGI121457[] = {{&S4458[68], 22}, {NULL}};
const Segment SGI121458[] = {{&S4458[30], 30}, {NULL}};
const Segment SGI121459[] = {{&S4458[139], 3}, {NULL}};
const Segment SGI121460[] = {{&S4458[104], 6}, {NULL}};
const Segment SGI121461[] = {{&S4459[44], 10}, {NULL}};
const Segment SGI121462[] = {{&S4459[44], 22}, {NULL}};
const Segment SGI121463[] = {{&S4459[30], 6}, {NULL}};
const Segment SGI121464[] = {{&S4459[127], 3}, {NULL}};
const Segment SGI121465[] = {{&S4459[80], 6}, {NULL}};
const Segment SGI121466[] = {{&S4460[44], 18}, {NULL}};
const Segment SGI121467[] = {{&S4460[44], 30}, {NULL}};
const Segment SGI121468[] = {{&S4460[30], 6}, {NULL}};
const Segment SGI121469[] = {{&S4460[123], 3}, {NULL}};
const Segment SGI121470[] = {{&S4460[88], 6}, {NULL}};
const Segment SGI121471[] = {{&S97339[45], 12}, {NULL}};
const Segment SGI121472[] = {{&S97339[30], 6}, {NULL}};
const Segment SGI121473[] = {{&S97339[31], 3}, {NULL}};
const Segment SGI121474[] = {{&S97339[83], 5}, {NULL}};
const Segment SGI121475[] = {{&S97339[141], 13}, {NULL}};
const Segment SGI121476[] = {{&S97348[59], 15}, {NULL}};
const Segment SGI121477[] = {{&S97348[59], 27}, {NULL}};
const Segment SGI121478[] = {{&S97348[45], 6}, {NULL}};
const Segment SGI121479[] = {{&S97348[46], 3}, {NULL}};
const Segment SGI121480[] = {{&S97348[100], 6}, {NULL}};
const Segment SGI121481[] = {{&S97348[0], 14}, {NULL}};
const Segment SGI121482[] = {{&S97351[44], 12}, {NULL}};
const Segment SGI121483[] = {{&S97351[44], 24}, {NULL}};
const Segment SGI121484[] = {{&S97351[30], 6}, {NULL}};
const Segment SGI121485[] = {{&S97351[31], 3}, {NULL}};
const Segment SGI121486[] = {{&S97351[82], 6}, {NULL}};
const Segment SGI121487[] = {{&S97354[59], 10}, {NULL}};
const Segment SGI121488[] = {{&S97354[59], 22}, {NULL}};
const Segment SGI121489[] = {{&S97354[45], 6}, {NULL}};
const Segment SGI121490[] = {{&S97354[46], 3}, {NULL}};
const Segment SGI121491[] = {{&S97354[95], 6}, {NULL}};
const Segment SGI121492[] = {{&S97354[0], 14}, {NULL}};
const Segment SGI121493[] = {{&S97363[45], 6}, {NULL}};
const Segment SGI121494[] = {{&S97363[30], 6}, {NULL}};
const Segment SGI121495[] = {{&S97363[31], 3}, {NULL}};
const Segment SGI121496[] = {{&S97363[77], 5}, {NULL}};
const Segment SGI121497[] = {{&S97363[135], 13}, {NULL}};
const Segment SGI121498[] = {{&S97411[34], 11}, {NULL}};
const Segment SGI121499[] = {{&S97411[27], 6}, {NULL}};
const Segment SGI121500[] = {{&S97411[98], 9}, {NULL}};
const Segment SGI121501[] = {{&S97411[71], 6}, {NULL}};
const Segment SGI121502[] = {{&S8136[44], 8}, {NULL}};
const Segment SGI121503[] = {{&S8136[44], 20}, {NULL}};
const Segment SGI121504[] = {{&S8136[30], 6}, {NULL}};
const Segment SGI121505[] = {{&S8136[31], 3}, {NULL}};
const Segment SGI121506[] = {{&S8136[78], 5}, {NULL}};
const Segment SGI121507[] = {{&S97445[44], 11}, {NULL}};
const Segment SGI121508[] = {{&S97445[44], 23}, {NULL}};
const Segment SGI121509[] = {{&S97445[30], 6}, {NULL}};
const Segment SGI121510[] = {{&S97445[31], 3}, {NULL}};
const Segment SGI121511[] = {{&S97445[81], 5}, {NULL}};
const Segment SGI121512[] = {{&S97446[34], 6}, {NULL}};
const Segment SGI121513[] = {{&S97446[27], 6}, {NULL}};
const Segment SGI121514[] = {{&S97446[93], 9}, {NULL}};
const Segment SGI121515[] = {{&S97446[66], 6}, {NULL}};
const Segment SGI121516[] = {{&S97446[134], 8}, {NULL}};
const Segment SGI121517[] = {{&S4462[38], 11}, {NULL}};
const Segment SGI121518[] = {{&S4462[31], 6}, {NULL}};
const Segment SGI121519[] = {{&S97448[34], 7}, {NULL}};
const Segment SGI121520[] = {{&S97448[27], 6}, {NULL}};
const Segment SGI121521[] = {{&S97448[95], 9}, {NULL}};
const Segment SGI121522[] = {{&S97448[68], 6}, {NULL}};
const Segment SGI121523[] = {{&S97449[34], 8}, {NULL}};
const Segment SGI121524[] = {{&S97449[27], 6}, {NULL}};
const Segment SGI121525[] = {{&S97449[96], 9}, {NULL}};
const Segment SGI121526[] = {{&S97449[69], 6}, {NULL}};
const Segment SGI121527[] = {{&S8172[44], 6}, {NULL}};
const Segment SGI121528[] = {{&S8172[44], 27}, {NULL}};
const Segment SGI121529[] = {{&S8172[30], 6}, {NULL}};
const Segment SGI121530[] = {{&S8172[31], 3}, {NULL}};
const Segment SGI121531[] = {{&S8172[85], 5}, {NULL}};
const Segment SGI121532[] = {{&S97473[44], 11}, {NULL}};
const Segment SGI121533[] = {{&S97473[44], 23}, {NULL}};
const Segment SGI121534[] = {{&S97473[30], 6}, {NULL}};
const Segment SGI121535[] = {{&S97473[31], 3}, {NULL}};
const Segment SGI121536[] = {{&S97473[81], 5}, {NULL}};
const Segment SGI121537[] = {{&S97473[118], 14}, {NULL}};
const Segment SGI121538[] = {{&S8201[44], 13}, {NULL}};
const Segment SGI121539[] = {{&S8201[44], 25}, {NULL}};
const Segment SGI121540[] = {{&S8201[30], 6}, {NULL}};
const Segment SGI121541[] = {{&S8201[31], 3}, {NULL}};
const Segment SGI121542[] = {{&S8201[83], 6}, {NULL}};
const Segment SGI121543[] = {{&S97519[44], 6}, {NULL}};
const Segment SGI121544[] = {{&S97519[44], 29}, {NULL}};
const Segment SGI121545[] = {{&S97519[30], 6}, {NULL}};
const Segment SGI121546[] = {{&S97519[87], 6}, {NULL}};
const Segment SGI121547[] = {{&S97519[114], 13}, {NULL}};
const Segment SGI121548[] = {{&S97524[34], 8}, {NULL}};
const Segment SGI121549[] = {{&S97524[27], 6}, {NULL}};
const Segment SGI121550[] = {{&S97524[95], 9}, {NULL}};
const Segment SGI121551[] = {{&S97524[68], 6}, {NULL}};
const Segment SGI121552[] = {{&S97556[44], 9}, {NULL}};
const Segment SGI121553[] = {{&S97556[44], 21}, {NULL}};
const Segment SGI121554[] = {{&S97556[30], 6}, {NULL}};
const Segment SGI121555[] = {{&S97556[79], 6}, {NULL}};
const Segment SGI121556[] = {{&S97556[106], 13}, {NULL}};
const Segment SGI121557[] = {{&S97558[27], 6}, {NULL}};
const Segment SGI121558[] = {{&S97558[102], 9}, {NULL}};
const Segment SGI121559[] = {{&S97558[75], 6}, {NULL}};
const Segment SGI121560[] = {{&S97590[34], 9}, {NULL}};
const Segment SGI121561[] = {{&S97590[27], 6}, {NULL}};
const Segment SGI121562[] = {{&S97590[96], 9}, {NULL}};
const Segment SGI121563[] = {{&S97590[69], 6}, {NULL}};
const Segment SGI121564[] = {{&S97614[34], 8}, {NULL}};
const Segment SGI121565[] = {{&S97614[27], 6}, {NULL}};
const Segment SGI121566[] = {{&S97614[95], 9}, {NULL}};
const Segment SGI121567[] = {{&S97614[68], 6}, {NULL}};
const Segment SGI121568[] = {{&S97677[44], 41}, {NULL}};
const Segment SGI121569[] = {{&S97677[44], 53}, {NULL}};
const Segment SGI121570[] = {{&S97677[30], 6}, {NULL}};
const Segment SGI121571[] = {{&S97677[142], 2}, {NULL}};
const Segment SGI121572[] = {{&S97677[111], 5}, {NULL}};
const Segment SGI121573[] = {{&S8268[44], 17}, {NULL}};
const Segment SGI121574[] = {{&S8268[44], 29}, {NULL}};
const Segment SGI121575[] = {{&S8268[30], 6}, {NULL}};
const Segment SGI121576[] = {{&S8268[31], 3}, {NULL}};
const Segment SGI121577[] = {{&S8268[87], 5}, {NULL}};
const Segment SGI121578[] = {{&S97678[44], 35}, {NULL}};
const Segment SGI121579[] = {{&S97678[44], 47}, {NULL}};
const Segment SGI121580[] = {{&S97678[30], 6}, {NULL}};
const Segment SGI121581[] = {{&S97678[136], 2}, {NULL}};
const Segment SGI121582[] = {{&S97678[105], 5}, {NULL}};
const Segment SGI121583[] = {{&S97720[34], 17}, {NULL}};
const Segment SGI121584[] = {{&S97720[27], 6}, {NULL}};
const Segment SGI121585[] = {{&S97720[104], 9}, {NULL}};
const Segment SGI121586[] = {{&S97720[77], 6}, {NULL}};
const Segment SGI121587[] = {{&S97723[45], 41}, {NULL}};
const Segment SGI121588[] = {{&S97723[30], 6}, {NULL}};
const Segment SGI121589[] = {{&S97723[31], 3}, {NULL}};
const Segment SGI121590[] = {{&S97723[112], 5}, {NULL}};
const Segment SGI121591[] = {{&S97723[170], 13}, {NULL}};
const Segment SGI121592[] = {{&S97744[44], 10}, {NULL}};
const Segment SGI121593[] = {{&S97744[44], 22}, {NULL}};
const Segment SGI121594[] = {{&S97744[30], 6}, {NULL}};
const Segment SGI121595[] = {{&S97744[111], 2}, {NULL}};
const Segment SGI121596[] = {{&S97744[80], 5}, {NULL}};
const Segment SGI121597[] = {{&S97752[44], 10}, {NULL}};
const Segment SGI121598[] = {{&S97752[44], 22}, {NULL}};
const Segment SGI121599[] = {{&S97752[30], 6}, {NULL}};
const Segment SGI121600[] = {{&S97752[31], 3}, {NULL}};
const Segment SGI121601[] = {{&S97752[80], 6}, {NULL}};
const Segment SGI121602[] = {{&S97752[132], 10}, {NULL}};
const Segment SGI121603[] = {{&S97766[59], 10}, {NULL}};
const Segment SGI121604[] = {{&S97766[59], 22}, {NULL}};
const Segment SGI121605[] = {{&S97766[45], 6}, {NULL}};
const Segment SGI121606[] = {{&S97766[46], 3}, {NULL}};
const Segment SGI121607[] = {{&S97766[95], 6}, {NULL}};
const Segment SGI121608[] = {{&S97766[0], 14}, {NULL}};
const Segment SGI121609[] = {{&S97769[44], 10}, {NULL}};
const Segment SGI121610[] = {{&S97769[44], 22}, {NULL}};
const Segment SGI121611[] = {{&S97769[30], 6}, {NULL}};
const Segment SGI121612[] = {{&S97769[31], 3}, {NULL}};
const Segment SGI121613[] = {{&S97769[80], 5}, {NULL}};
const Segment SGI121614[] = {{&S97769[117], 14}, {NULL}};
const Segment SGI121615[] = {{&S97771[34], 12}, {NULL}};
const Segment SGI121616[] = {{&S97771[27], 6}, {NULL}};
const Segment SGI121617[] = {{&S97771[99], 9}, {NULL}};
const Segment SGI121618[] = {{&S97771[72], 6}, {NULL}};
const Segment SGI121619[] = {{&S97803[34], 6}, {NULL}};
const Segment SGI121620[] = {{&S97803[27], 6}, {NULL}};
const Segment SGI121621[] = {{&S97803[93], 9}, {NULL}};
const Segment SGI121622[] = {{&S97803[66], 6}, {NULL}};
const Segment SGI121623[] = {{&S97808[45], 6}, {NULL}};
const Segment SGI121624[] = {{&S97808[30], 6}, {NULL}};
const Segment SGI121625[] = {{&S97808[31], 3}, {NULL}};
const Segment SGI121626[] = {{&S97808[88], 5}, {NULL}};
const Segment SGI121627[] = {{&S97808[146], 13}, {NULL}};
const Segment SGI121628[] = {{&S97821[34], 9}, {NULL}};
const Segment SGI121629[] = {{&S97821[27], 6}, {NULL}};
const Segment SGI121630[] = {{&S97821[125], 9}, {NULL}};
const Segment SGI121631[] = {{&S97821[98], 6}, {NULL}};
const Segment SGI121632[] = {{&S8505[44], 10}, {NULL}};
const Segment SGI121633[] = {{&S8505[44], 22}, {NULL}};
const Segment SGI121634[] = {{&S8505[30], 6}, {NULL}};
const Segment SGI121635[] = {{&S8505[31], 3}, {NULL}};
const Segment SGI121636[] = {{&S8505[80], 6}, {NULL}};
const Segment SGI121637[] = {{&S97837[34], 11}, {NULL}};
const Segment SGI121638[] = {{&S97837[27], 6}, {NULL}};
const Segment SGI121639[] = {{&S97837[98], 9}, {NULL}};
const Segment SGI121640[] = {{&S97837[71], 6}, {NULL}};
const Segment SGI121641[] = {{&S8508[44], 7}, {NULL}};
const Segment SGI121642[] = {{&S8508[44], 19}, {NULL}};
const Segment SGI121643[] = {{&S8508[30], 6}, {NULL}};
const Segment SGI121644[] = {{&S8508[31], 3}, {NULL}};
const Segment SGI121645[] = {{&S8508[77], 6}, {NULL}};
const Segment SGI121646[] = {{&S97849[34], 5}, {NULL}};
const Segment SGI121647[] = {{&S97849[27], 6}, {NULL}};
const Segment SGI121648[] = {{&S97849[92], 9}, {NULL}};
const Segment SGI121649[] = {{&S97849[65], 6}, {NULL}};
const Segment SGI121650[] = {{&S8518[44], 8}, {NULL}};
const Segment SGI121651[] = {{&S8518[44], 20}, {NULL}};
const Segment SGI121652[] = {{&S8518[30], 6}, {NULL}};
const Segment SGI121653[] = {{&S8518[31], 3}, {NULL}};
const Segment SGI121654[] = {{&S8518[78], 6}, {NULL}};
const Segment SGI121655[] = {{&S8520[44], 7}, {NULL}};
const Segment SGI121656[] = {{&S8520[44], 19}, {NULL}};
const Segment SGI121657[] = {{&S8520[30], 6}, {NULL}};
const Segment SGI121658[] = {{&S8520[31], 3}, {NULL}};
const Segment SGI121659[] = {{&S8520[77], 5}, {NULL}};
const Segment SGI121660[] = {{&S8534[44], 7}, {NULL}};
const Segment SGI121661[] = {{&S8534[44], 19}, {NULL}};
const Segment SGI121662[] = {{&S8534[30], 6}, {NULL}};
const Segment SGI121663[] = {{&S8534[31], 3}, {NULL}};
const Segment SGI121664[] = {{&S8534[77], 5}, {NULL}};
const Segment SGI121665[] = {{&S97896[44], 11}, {NULL}};
const Segment SGI121666[] = {{&S97896[44], 23}, {NULL}};
const Segment SGI121667[] = {{&S97896[30], 6}, {NULL}};
const Segment SGI121668[] = {{&S97896[31], 3}, {NULL}};
const Segment SGI121669[] = {{&S97896[81], 5}, {NULL}};
const Segment SGI121670[] = {{&S97896[118], 14}, {NULL}};
const Segment SGI121671[] = {{&S97901[34], 9}, {NULL}};
const Segment SGI121672[] = {{&S97901[27], 6}, {NULL}};
const Segment SGI121673[] = {{&S97901[96], 9}, {NULL}};
const Segment SGI121674[] = {{&S97901[69], 6}, {NULL}};
const Segment SGI121675[] = {{&S97902[44], 17}, {NULL}};
const Segment SGI121676[] = {{&S97902[44], 29}, {NULL}};
const Segment SGI121677[] = {{&S97902[30], 6}, {NULL}};
const Segment SGI121678[] = {{&S97902[87], 6}, {NULL}};
const Segment SGI121679[] = {{&S97902[114], 13}, {NULL}};
const Segment SGI121680[] = {{&S97960[44], 11}, {NULL}};
const Segment SGI121681[] = {{&S97960[44], 23}, {NULL}};
const Segment SGI121682[] = {{&S97960[30], 6}, {NULL}};
const Segment SGI121683[] = {{&S97960[81], 6}, {NULL}};
const Segment SGI121684[] = {{&S97960[108], 13}, {NULL}};
const Segment SGI121685[] = {{&S5035[44], 10}, {NULL}};
const Segment SGI121686[] = {{&S5035[44], 22}, {NULL}};
const Segment SGI121687[] = {{&S5035[30], 6}, {NULL}};
const Segment SGI121688[] = {{&S5035[115], 3}, {NULL}};
const Segment SGI121689[] = {{&S5035[80], 6}, {NULL}};
const Segment SGI121690[] = {{&S5038[44], 10}, {NULL}};
const Segment SGI121691[] = {{&S5038[44], 22}, {NULL}};
const Segment SGI121692[] = {{&S5038[30], 6}, {NULL}};
const Segment SGI121693[] = {{&S5038[115], 3}, {NULL}};
const Segment SGI121694[] = {{&S5038[80], 6}, {NULL}};
const Segment SGI121695[] = {{&S5040[44], 18}, {NULL}};
const Segment SGI121696[] = {{&S5040[44], 30}, {NULL}};
const Segment SGI121697[] = {{&S5040[30], 6}, {NULL}};
const Segment SGI121698[] = {{&S5040[123], 3}, {NULL}};
const Segment SGI121699[] = {{&S5040[88], 6}, {NULL}};
const Segment SGI121700[] = {{&S15060[11], 1}, {NULL}};
const Segment SGI121701[] = {{&S15060[44], 13}, {NULL}};
const Segment SGI121702[] = {{&S15060[30], 6}, {NULL}};
const Segment SGI121703[] = {{&S15060[31], 3}, {NULL}};
const Segment SGI121704[] = {{&S15060[71], 6}, {NULL}};
const Segment SGI121705[] = {{&S15061[11], 1}, {NULL}};
const Segment SGI121706[] = {{&S15061[44], 13}, {NULL}};
const Segment SGI121707[] = {{&S15061[30], 6}, {NULL}};
const Segment SGI121708[] = {{&S5047[38], 18}, {NULL}};
const Segment SGI121709[] = {{&S5047[31], 6}, {NULL}};
const Segment SGI121710[] = {{&S15061[31], 3}, {NULL}};
const Segment SGI121711[] = {{&S15061[71], 6}, {NULL}};
const Segment SGI121712[] = {{&S15062[11], 1}, {NULL}};
const Segment SGI121713[] = {{&S15062[44], 12}, {NULL}};
const Segment SGI121714[] = {{&S15062[30], 6}, {NULL}};
const Segment SGI121715[] = {{&S15062[31], 3}, {NULL}};
const Segment SGI121716[] = {{&S15062[70], 6}, {NULL}};
const Segment SGI121717[] = {{&S15063[11], 1}, {NULL}};
const Segment SGI121718[] = {{&S15063[52], 13}, {NULL}};
const Segment SGI121719[] = {{&S15063[30], 14}, {NULL}};
const Segment SGI121720[] = {{&S15063[31], 3}, {NULL}};
const Segment SGI121721[] = {{&S15063[79], 6}, {NULL}};
const Segment SGI121722[] = {{&S5053[44], 10}, {NULL}};
const Segment SGI121723[] = {{&S5053[44], 22}, {NULL}};
const Segment SGI121724[] = {{&S5053[30], 6}, {NULL}};
const Segment SGI121725[] = {{&S5053[115], 3}, {NULL}};
const Segment SGI121726[] = {{&S5053[80], 6}, {NULL}};
const Segment SGI121727[] = {{&S15067[11], 1}, {NULL}};
const Segment SGI121728[] = {{&S15067[44], 11}, {NULL}};
const Segment SGI121729[] = {{&S15067[30], 6}, {NULL}};
const Segment SGI121730[] = {{&S15067[31], 3}, {NULL}};
const Segment SGI121731[] = {{&S15067[69], 6}, {NULL}};
const Segment SGI121732[] = {{&S15068[11], 1}, {NULL}};
const Segment SGI121733[] = {{&S15068[44], 17}, {NULL}};
const Segment SGI121734[] = {{&S5057[44], 11}, {NULL}};
const Segment SGI121735[] = {{&S15068[30], 6}, {NULL}};
const Segment SGI121736[] = {{&S5057[44], 23}, {NULL}};
const Segment SGI121737[] = {{&S5057[30], 6}, {NULL}};
const Segment SGI121738[] = {{&S15068[31], 3}, {NULL}};
const Segment SGI121739[] = {{&S15068[75], 6}, {NULL}};
const Segment SGI121740[] = {{&S5057[116], 3}, {NULL}};
const Segment SGI121741[] = {{&S5057[81], 6}, {NULL}};
const Segment SGI121742[] = {{&S5058[44], 11}, {NULL}};
const Segment SGI121743[] = {{&S15069[11], 1}, {NULL}};
const Segment SGI121744[] = {{&S5058[44], 23}, {NULL}};
const Segment SGI121745[] = {{&S5058[30], 6}, {NULL}};
const Segment SGI121746[] = {{&S15069[44], 13}, {NULL}};
const Segment SGI121747[] = {{&S15069[30], 6}, {NULL}};
const Segment SGI121748[] = {{&S5058[116], 3}, {NULL}};
const Segment SGI121749[] = {{&S15069[31], 3}, {NULL}};
const Segment SGI121750[] = {{&S5058[81], 6}, {NULL}};
const Segment SGI121751[] = {{&S15069[71], 6}, {NULL}};
const Segment SGI121752[] = {{&S5059[44], 11}, {NULL}};
const Segment SGI121753[] = {{&S5059[44], 23}, {NULL}};
const Segment SGI121754[] = {{&S5059[30], 6}, {NULL}};
const Segment SGI121755[] = {{&S5059[116], 3}, {NULL}};
const Segment SGI121756[] = {{&S5059[81], 6}, {NULL}};
const Segment SGI121757[] = {{&S15072[11], 1}, {NULL}};
const Segment SGI121758[] = {{&S15072[44], 13}, {NULL}};
const Segment SGI121759[] = {{&S15072[30], 6}, {NULL}};
const Segment SGI121760[] = {{&S15072[71], 6}, {NULL}};
const Segment SGI121761[] = {{&S5062[38], 10}, {NULL}};
const Segment SGI121762[] = {{&S5062[31], 6}, {NULL}};
const Segment SGI121763[] = {{&S5063[44], 30}, {NULL}};
const Segment SGI121764[] = {{&S5063[44], 42}, {NULL}};
const Segment SGI121765[] = {{&S5063[30], 6}, {NULL}};
const Segment SGI121766[] = {{&S5063[135], 3}, {NULL}};
const Segment SGI121767[] = {{&S5063[100], 6}, {NULL}};
const Segment SGI121768[] = {{&S15075[11], 1}, {NULL}};
const Segment SGI121769[] = {{&S15075[44], 13}, {NULL}};
const Segment SGI121770[] = {{&S15075[30], 6}, {NULL}};
const Segment SGI121771[] = {{&S15075[31], 3}, {NULL}};
const Segment SGI121772[] = {{&S15075[71], 6}, {NULL}};
const Segment SGI121773[] = {{&S97996[44], 21}, {NULL}};
const Segment SGI121774[] = {{&S5064[44], 30}, {NULL}};
const Segment SGI121775[] = {{&S97996[44], 33}, {NULL}};
const Segment SGI121776[] = {{&S5064[44], 42}, {NULL}};
const Segment SGI121777[] = {{&S97996[30], 6}, {NULL}};
const Segment SGI121778[] = {{&S5064[30], 6}, {NULL}};
const Segment SGI121779[] = {{&S5064[135], 3}, {NULL}};
const Segment SGI121780[] = {{&S97996[91], 6}, {NULL}};
const Segment SGI121781[] = {{&S5064[100], 6}, {NULL}};
const Segment SGI121782[] = {{&S97996[118], 13}, {NULL}};
const Segment SGI121783[] = {{&S15091[11], 1}, {NULL}};
const Segment SGI121784[] = {{&S15091[44], 13}, {NULL}};
const Segment SGI121785[] = {{&S15091[30], 6}, {NULL}};
const Segment SGI121786[] = {{&S15091[31], 3}, {NULL}};
const Segment SGI121787[] = {{&S15091[71], 6}, {NULL}};
const Segment SGI121788[] = {{&S15094[11], 1}, {NULL}};
const Segment SGI121789[] = {{&S15094[44], 13}, {NULL}};
const Segment SGI121790[] = {{&S15094[30], 6}, {NULL}};
const Segment SGI121791[] = {{&S15094[31], 3}, {NULL}};
const Segment SGI121792[] = {{&S15094[71], 6}, {NULL}};
const Segment SGI121793[] = {{&S98018[34], 6}, {NULL}};
const Segment SGI121794[] = {{&S98018[27], 6}, {NULL}};
const Segment SGI121795[] = {{&S98018[93], 9}, {NULL}};
const Segment SGI121796[] = {{&S98018[66], 6}, {NULL}};
const Segment SGI121797[] = {{&S15097[11], 1}, {NULL}};
const Segment SGI121798[] = {{&S15097[52], 13}, {NULL}};
const Segment SGI121799[] = {{&S15097[30], 14}, {NULL}};
const Segment SGI121800[] = {{&S15097[31], 3}, {NULL}};
const Segment SGI121801[] = {{&S15097[79], 6}, {NULL}};
const Segment SGI121802[] = {{&S15098[44], 27}, {NULL}};
const Segment SGI121803[] = {{&S15098[44], 39}, {NULL}};
const Segment SGI121804[] = {{&S15098[30], 6}, {&S15098[52], 31}, {NULL}};
const Segment SGI121805[] = {{&S15098[31], 3}, {NULL}};
const Segment SGI121806[] = {{&S15098[97], 6}, {NULL}};
const Segment SGI121807[] = {{&S15102[11], 1}, {NULL}};
const Segment SGI121808[] = {{&S15102[44], 13}, {NULL}};
const Segment SGI121809[] = {{&S15102[30], 6}, {NULL}};
const Segment SGI121810[] = {{&S15102[31], 3}, {NULL}};
const Segment SGI121811[] = {{&S15102[71], 6}, {NULL}};
const Segment SGI121812[] = {{&S15103[44], 26}, {NULL}};
const Segment SGI121813[] = {{&S15103[44], 38}, {NULL}};
const Segment SGI121814[] = {{&S15103[30], 6}, {NULL}};
const Segment SGI121815[] = {{&S15103[31], 3}, {NULL}};
const Segment SGI121816[] = {{&S15103[96], 6}, {NULL}};
const Segment SGI121817[] = {{&S98029[34], 11}, {NULL}};
const Segment SGI121818[] = {{&S98029[27], 6}, {NULL}};
const Segment SGI121819[] = {{&S98029[98], 9}, {NULL}};
const Segment SGI121820[] = {{&S98029[71], 6}, {NULL}};
const Segment SGI121821[] = {{&S98029[139], 8}, {NULL}};
const Segment SGI121822[] = {{&S15105[11], 1}, {NULL}};
const Segment SGI121823[] = {{&S15105[52], 13}, {NULL}};
const Segment SGI121824[] = {{&S15105[30], 14}, {NULL}};
const Segment SGI121825[] = {{&S15105[31], 3}, {NULL}};
const Segment SGI121826[] = {{&S15105[79], 6}, {NULL}};
const Segment SGI121827[] = {{&S98034[34], 21}, {NULL}};
const Segment SGI121828[] = {{&S98034[27], 6}, {NULL}};
const Segment SGI121829[] = {{&S98034[108], 9}, {NULL}};
const Segment SGI121830[] = {{&S98034[81], 6}, {NULL}};
const Segment SGI121831[] = {{&S15112[44], 5}, {NULL}};
const Segment SGI121832[] = {{&S15112[44], 17}, {NULL}};
const Segment SGI121833[] = {{&S98037[34], 41}, {NULL}};
const Segment SGI121834[] = {{&S15112[30], 6}, {NULL}};
const Segment SGI121835[] = {{&S98037[27], 6}, {NULL}};
const Segment SGI121836[] = {{&S15112[31], 3}, {NULL}};
const Segment SGI121837[] = {{&S15112[75], 6}, {NULL}};
const Segment SGI121838[] = {{&S98037[128], 9}, {NULL}};
const Segment SGI121839[] = {{&S98037[101], 6}, {NULL}};
const Segment SGI121840[] = {{&S15123[44], 15}, {NULL}};
const Segment SGI121841[] = {{&S15123[44], 27}, {NULL}};
const Segment SGI121842[] = {{&S15123[30], 6}, {NULL}};
const Segment SGI121843[] = {{&S15123[31], 3}, {NULL}};
const Segment SGI121844[] = {{&S15123[85], 6}, {NULL}};
const Segment SGI121845[] = {{&S15124[44], 7}, {NULL}};
const Segment SGI121846[] = {{&S15124[44], 19}, {NULL}};
const Segment SGI121847[] = {{&S15124[30], 6}, {NULL}};
const Segment SGI121848[] = {{&S15124[31], 3}, {NULL}};
const Segment SGI121849[] = {{&S15124[77], 6}, {NULL}};
const Segment SGI121850[] = {{&S15129[11], 1}, {NULL}};
const Segment SGI121851[] = {{&S15129[44], 13}, {NULL}};
const Segment SGI121852[] = {{&S15129[30], 6}, {NULL}};
const Segment SGI121853[] = {{&S15129[31], 3}, {NULL}};
const Segment SGI121854[] = {{&S15129[71], 6}, {NULL}};
const Segment SGI121855[] = {{&S15132[11], 1}, {NULL}};
const Segment SGI121856[] = {{&S15132[44], 13}, {NULL}};
const Segment SGI121857[] = {{&S15132[30], 6}, {NULL}};
const Segment SGI121858[] = {{&S15132[31], 3}, {NULL}};
const Segment SGI121859[] = {{&S15132[71], 6}, {NULL}};
const Segment SGI121860[] = {{&S15137[11], 1}, {NULL}};
const Segment SGI121861[] = {{&S15137[44], 17}, {NULL}};
const Segment SGI121862[] = {{&S15137[30], 6}, {NULL}};
const Segment SGI121863[] = {{&S15137[31], 3}, {NULL}};
const Segment SGI121864[] = {{&S15137[75], 6}, {NULL}};
const Segment SGI121865[] = {{&S15139[44], 13}, {NULL}};
const Segment SGI121866[] = {{&S15139[44], 25}, {NULL}};
const Segment SGI121867[] = {{&S15139[30], 6}, {NULL}};
const Segment SGI121868[] = {{&S15139[31], 3}, {NULL}};
const Segment SGI121869[] = {{&S15139[83], 6}, {NULL}};
const Segment SGI121870[] = {{&S15141[44], 5}, {NULL}};
const Segment SGI121871[] = {{&S15141[44], 17}, {NULL}};
const Segment SGI121872[] = {{&S15141[30], 6}, {NULL}};
const Segment SGI121873[] = {{&S15141[31], 3}, {NULL}};
const Segment SGI121874[] = {{&S15141[75], 6}, {NULL}};
const Segment SGI121875[] = {{&S15149[44], 16}, {NULL}};
const Segment SGI121876[] = {{&S15149[44], 28}, {NULL}};
const Segment SGI121877[] = {{&S15149[30], 6}, {NULL}};
const Segment SGI121878[] = {{&S15149[31], 3}, {NULL}};
const Segment SGI121879[] = {{&S15149[86], 6}, {NULL}};
const Segment SGI121880[] = {{&S15150[44], 21}, {NULL}};
const Segment SGI121881[] = {{&S15150[44], 33}, {NULL}};
const Segment SGI121882[] = {{&S15150[30], 6}, {NULL}};
const Segment SGI121883[] = {{&S15150[31], 3}, {NULL}};
const Segment SGI121884[] = {{&S15150[91], 6}, {NULL}};
const Segment SGI121885[] = {{&S15151[44], 7}, {NULL}};
const Segment SGI121886[] = {{&S15151[44], 19}, {NULL}};
const Segment SGI121887[] = {{&S15151[30], 6}, {NULL}};
const Segment SGI121888[] = {{&S15151[31], 3}, {NULL}};
const Segment SGI121889[] = {{&S15151[77], 6}, {NULL}};
const Segment SGI121890[] = {{&S15152[30], 6}, {NULL}};
const Segment SGI121891[] = {{&S15152[31], 3}, {NULL}};
const Segment SGI121892[] = {{&S15152[57], 6}, {NULL}};
const Segment SGI121893[] = {{&S15156[44], 9}, {NULL}};
const Segment SGI121894[] = {{&S15156[44], 21}, {NULL}};
const Segment SGI121895[] = {{&S15156[30], 6}, {NULL}};
const Segment SGI121896[] = {{&S15156[31], 3}, {NULL}};
const Segment SGI121897[] = {{&S15156[79], 6}, {NULL}};
const Segment SGI121898[] = {{&S15158[44], 11}, {NULL}};
const Segment SGI121899[] = {{&S15158[44], 23}, {NULL}};
const Segment SGI121900[] = {{&S15158[30], 6}, {NULL}};
const Segment SGI121901[] = {{&S15158[31], 3}, {NULL}};
const Segment SGI121902[] = {{&S15158[81], 6}, {NULL}};
const Segment SGI121903[] = {{&S15159[11], 1}, {NULL}};
const Segment SGI121904[] = {{&S15159[44], 13}, {NULL}};
const Segment SGI121905[] = {{&S5066[38], 10}, {NULL}};
const Segment SGI121906[] = {{&S15159[30], 6}, {NULL}};
const Segment SGI121907[] = {{&S5066[31], 6}, {NULL}};
const Segment SGI121908[] = {{&S15159[31], 3}, {NULL}};
const Segment SGI121909[] = {{&S15159[71], 6}, {NULL}};
const Segment SGI121910[] = {{&S5067[38], 11}, {NULL}};
const Segment SGI121911[] = {{&S5067[31], 6}, {NULL}};
const Segment SGI121912[] = {{&S15160[38], 5}, {NULL}};
const Segment SGI121913[] = {{&S15160[31], 6}, {NULL}};
const Segment SGI121914[] = {{&S5068[38], 10}, {NULL}};
const Segment SGI121915[] = {{&S5068[31], 6}, {NULL}};
const Segment SGI121916[] = {{&S5069[44], 11}, {NULL}};
const Segment SGI121917[] = {{&S5069[44], 23}, {NULL}};
const Segment SGI121918[] = {{&S5069[30], 6}, {NULL}};
const Segment SGI121919[] = {{&S5069[116], 3}, {NULL}};
const Segment SGI121920[] = {{&S5069[81], 6}, {NULL}};
const Segment SGI121921[] = {{&S5070[44], 30}, {NULL}};
const Segment SGI121922[] = {{&S5070[44], 42}, {NULL}};
const Segment SGI121923[] = {{&S5070[30], 6}, {NULL}};
const Segment SGI121924[] = {{&S15164[44], 7}, {NULL}};
const Segment SGI121925[] = {{&S15164[44], 19}, {NULL}};
const Segment SGI121926[] = {{&S15164[30], 6}, {NULL}};
const Segment SGI121927[] = {{&S5070[135], 3}, {NULL}};
const Segment SGI121928[] = {{&S5070[100], 6}, {NULL}};
const Segment SGI121929[] = {{&S15164[31], 3}, {NULL}};
const Segment SGI121930[] = {{&S15164[77], 6}, {NULL}};
const Segment SGI121931[] = {{&S5071[38], 11}, {NULL}};
const Segment SGI121932[] = {{&S5071[31], 6}, {NULL}};
const Segment SGI121933[] = {{&S15165[37], 7}, {NULL}};
const Segment SGI121934[] = {{&S15165[30], 6}, {NULL}};
const Segment SGI121935[] = {{&S15165[73], 6}, {NULL}};
const Segment SGI121936[] = {{&S5072[44], 29}, {NULL}};
const Segment SGI121937[] = {{&S5072[44], 41}, {NULL}};
const Segment SGI121938[] = {{&S5072[30], 6}, {NULL}};
const Segment SGI121939[] = {{&S5072[134], 3}, {NULL}};
const Segment SGI121940[] = {{&S5072[99], 6}, {NULL}};
const Segment SGI121941[] = {{&S15167[44], 16}, {NULL}};
const Segment SGI121942[] = {{&S15167[44], 28}, {NULL}};
const Segment SGI121943[] = {{&S15167[30], 6}, {NULL}};
const Segment SGI121944[] = {{&S5073[38], 11}, {NULL}};
const Segment SGI121945[] = {{&S5073[31], 6}, {NULL}};
const Segment SGI121946[] = {{&S15167[31], 3}, {NULL}};
const Segment SGI121947[] = {{&S15167[86], 6}, {NULL}};
const Segment SGI121948[] = {{&S15168[44], 11}, {NULL}};
const Segment SGI121949[] = {{&S15168[44], 23}, {NULL}};
const Segment SGI121950[] = {{&S15168[30], 6}, {&S15168[52], 15}, {NULL}};
const Segment SGI121951[] = {{&S15168[31], 3}, {NULL}};
const Segment SGI121952[] = {{&S15168[81], 6}, {NULL}};
const Segment SGI121953[] = {{&S15170[44], 7}, {NULL}};
const Segment SGI121954[] = {{&S15170[44], 19}, {NULL}};
const Segment SGI121955[] = {{&S15170[30], 6}, {NULL}};
const Segment SGI121956[] = {{&S15170[31], 3}, {NULL}};
const Segment SGI121957[] = {{&S15170[77], 6}, {NULL}};
const Segment SGI121958[] = {{&S15171[44], 8}, {NULL}};
const Segment SGI121959[] = {{&S15171[44], 20}, {NULL}};
const Segment SGI121960[] = {{&S15171[30], 6}, {NULL}};
const Segment SGI121961[] = {{&S15171[31], 3}, {NULL}};
const Segment SGI121962[] = {{&S15171[78], 6}, {NULL}};
const Segment SGI121963[] = {{&S15172[44], 10}, {NULL}};
const Segment SGI121964[] = {{&S15172[44], 22}, {NULL}};
const Segment SGI121965[] = {{&S15172[30], 6}, {NULL}};
const Segment SGI121966[] = {{&S98129[44], 11}, {NULL}};
const Segment SGI121967[] = {{&S98129[44], 23}, {NULL}};
const Segment SGI121968[] = {{&S15172[31], 3}, {NULL}};
const Segment SGI121969[] = {{&S98129[30], 6}, {NULL}};
const Segment SGI121970[] = {{&S15172[80], 6}, {NULL}};
const Segment SGI121971[] = {{&S98129[31], 3}, {NULL}};
const Segment SGI121972[] = {{&S98129[88], 6}, {NULL}};
const Segment SGI121973[] = {{&S98129[67], 8}, {NULL}};
const Segment SGI121974[] = {{&S15173[44], 36}, {NULL}};
const Segment SGI121975[] = {{&S15173[44], 48}, {NULL}};
const Segment SGI121976[] = {{&S15173[30], 6}, {NULL}};
const Segment SGI121977[] = {{&S15173[31], 3}, {NULL}};
const Segment SGI121978[] = {{&S15173[106], 6}, {NULL}};
const Segment SGI121979[] = {{&S15176[44], 12}, {NULL}};
const Segment SGI121980[] = {{&S15176[44], 35}, {NULL}};
const Segment SGI121981[] = {{&S15176[30], 6}, {NULL}};
const Segment SGI121982[] = {{&S15176[31], 3}, {NULL}};
const Segment SGI121983[] = {{&S15176[93], 6}, {NULL}};
const Segment SGI121984[] = {{&S15178[44], 8}, {NULL}};
const Segment SGI121985[] = {{&S15178[44], 20}, {NULL}};
const Segment SGI121986[] = {{&S15178[30], 6}, {NULL}};
const Segment SGI121987[] = {{&S15178[31], 3}, {NULL}};
const Segment SGI121988[] = {{&S15178[78], 6}, {NULL}};
const Segment SGI121989[] = {{&S15179[44], 10}, {NULL}};
const Segment SGI121990[] = {{&S15179[44], 22}, {NULL}};
const Segment SGI121991[] = {{&S15179[30], 6}, {NULL}};
const Segment SGI121992[] = {{&S15179[31], 3}, {NULL}};
const Segment SGI121993[] = {{&S15179[80], 6}, {NULL}};
const Segment SGI121994[] = {{&S15182[44], 13}, {NULL}};
const Segment SGI121995[] = {{&S98146[34], 13}, {NULL}};
const Segment SGI121996[] = {{&S15182[44], 25}, {NULL}};
const Segment SGI121997[] = {{&S15182[30], 6}, {NULL}};
const Segment SGI121998[] = {{&S98146[27], 6}, {NULL}};
const Segment SGI121999[] = {{&S15182[31], 3}, {NULL}};
const Segment SGI122000[] = {{&S98146[100], 9}, {NULL}};
const Segment SGI122001[] = {{&S15182[83], 6}, {NULL}};
const Segment SGI122002[] = {{&S98146[73], 6}, {NULL}};
const Segment SGI122003[] = {{&S15183[44], 9}, {NULL}};
const Segment SGI122004[] = {{&S15183[44], 21}, {NULL}};
const Segment SGI122005[] = {{&S15183[30], 6}, {NULL}};
const Segment SGI122006[] = {{&S15183[31], 3}, {NULL}};
const Segment SGI122007[] = {{&S15183[79], 6}, {NULL}};
const Segment SGI122008[] = {{&S15184[44], 5}, {NULL}};
const Segment SGI122009[] = {{&S15184[44], 17}, {NULL}};
const Segment SGI122010[] = {{&S15184[30], 6}, {NULL}};
const Segment SGI122011[] = {{&S15184[31], 3}, {NULL}};
const Segment SGI122012[] = {{&S15184[75], 6}, {NULL}};
const Segment SGI122013[] = {{&S15185[44], 12}, {NULL}};
const Segment SGI122014[] = {{&S15185[44], 24}, {NULL}};
const Segment SGI122015[] = {{&S15185[30], 6}, {NULL}};
const Segment SGI122016[] = {{&S15185[31], 3}, {NULL}};
const Segment SGI122017[] = {{&S15185[82], 6}, {NULL}};
const Segment SGI122018[] = {{&S15186[44], 6}, {NULL}};
const Segment SGI122019[] = {{&S15186[44], 18}, {NULL}};
const Segment SGI122020[] = {{&S15186[30], 6}, {NULL}};
const Segment SGI122021[] = {{&S15186[31], 3}, {NULL}};
const Segment SGI122022[] = {{&S15186[76], 6}, {NULL}};
const Segment SGI122023[] = {{&S15188[44], 8}, {NULL}};
const Segment SGI122024[] = {{&S15188[44], 20}, {NULL}};
const Segment SGI122025[] = {{&S15188[30], 6}, {NULL}};
const Segment SGI122026[] = {{&S15188[31], 3}, {NULL}};
const Segment SGI122027[] = {{&S15188[78], 6}, {NULL}};
const Segment SGI122028[] = {{&S15190[44], 8}, {NULL}};
const Segment SGI122029[] = {{&S15190[44], 20}, {NULL}};
const Segment SGI122030[] = {{&S15190[30], 6}, {NULL}};
const Segment SGI122031[] = {{&S15190[31], 3}, {NULL}};
const Segment SGI122032[] = {{&S15190[78], 6}, {NULL}};
const Segment SGI122033[] = {{&S15192[44], 4}, {NULL}};
const Segment SGI122034[] = {{&S15192[44], 16}, {NULL}};
const Segment SGI122035[] = {{&S15192[30], 6}, {NULL}};
const Segment SGI122036[] = {{&S15192[31], 3}, {NULL}};
const Segment SGI122037[] = {{&S15192[74], 6}, {NULL}};
const Segment SGI122038[] = {{&S15193[44], 9}, {NULL}};
const Segment SGI122039[] = {{&S15193[44], 44}, {NULL}};
const Segment SGI122040[] = {{&S15193[30], 6}, {NULL}};
const Segment SGI122041[] = {{&S15193[31], 3}, {NULL}};
const Segment SGI122042[] = {{&S15193[102], 6}, {NULL}};
const Segment SGI122043[] = {{&S98169[34], 10}, {NULL}};
const Segment SGI122044[] = {{&S98169[27], 6}, {NULL}};
const Segment SGI122045[] = {{&S15194[44], 5}, {NULL}};
const Segment SGI122046[] = {{&S15194[44], 17}, {NULL}};
const Segment SGI122047[] = {{&S98169[94], 9}, {NULL}};
const Segment SGI122048[] = {{&S15194[30], 6}, {NULL}};
const Segment SGI122049[] = {{&S98169[67], 6}, {NULL}};
const Segment SGI122050[] = {{&S15194[31], 3}, {NULL}};
const Segment SGI122051[] = {{&S15194[75], 6}, {NULL}};
const Segment SGI122052[] = {{&S15195[44], 11}, {NULL}};
const Segment SGI122053[] = {{&S15195[44], 23}, {NULL}};
const Segment SGI122054[] = {{&S15195[30], 6}, {NULL}};
const Segment SGI122055[] = {{&S15195[31], 3}, {NULL}};
const Segment SGI122056[] = {{&S15195[81], 6}, {NULL}};
const Segment SGI122057[] = {{&S15196[30], 6}, {NULL}};
const Segment SGI122058[] = {{&S15196[31], 3}, {NULL}};
const Segment SGI122059[] = {{&S15196[50], 5}, {NULL}};
const Segment SGI122060[] = {{&S5075[44], 11}, {NULL}};
const Segment SGI122061[] = {{&S5075[44], 23}, {NULL}};
const Segment SGI122062[] = {{&S5075[30], 6}, {NULL}};
const Segment SGI122063[] = {{&S5075[116], 3}, {NULL}};
const Segment SGI122064[] = {{&S5075[81], 6}, {NULL}};
const Segment SGI122065[] = {{&S15197[44], 9}, {NULL}};
const Segment SGI122066[] = {{&S15197[44], 21}, {NULL}};
const Segment SGI122067[] = {{&S15197[30], 6}, {NULL}};
const Segment SGI122068[] = {{&S15197[31], 3}, {NULL}};
const Segment SGI122069[] = {{&S15197[79], 6}, {NULL}};
const Segment SGI122070[] = {{&S5076[38], 11}, {NULL}};
const Segment SGI122071[] = {{&S5076[31], 6}, {NULL}};
const Segment SGI122072[] = {{&S5077[44], 30}, {NULL}};
const Segment SGI122073[] = {{&S5077[44], 42}, {NULL}};
const Segment SGI122074[] = {{&S5077[30], 6}, {NULL}};
const Segment SGI122075[] = {{&S5077[135], 3}, {NULL}};
const Segment SGI122076[] = {{&S5077[100], 6}, {NULL}};
const Segment SGI122077[] = {{&S5078[44], 30}, {NULL}};
const Segment SGI122078[] = {{&S5078[44], 42}, {NULL}};
const Segment SGI122079[] = {{&S5078[30], 6}, {NULL}};
const Segment SGI122080[] = {{&S5078[135], 3}, {NULL}};
const Segment SGI122081[] = {{&S5078[100], 6}, {NULL}};
const Segment SGI122082[] = {{&S5079[42], 10}, {NULL}};
const Segment SGI122083[] = {{&S5079[30], 6}, {NULL}};
const Segment SGI122084[] = {{&S5079[113], 3}, {NULL}};
const Segment SGI122085[] = {{&S5079[78], 6}, {NULL}};
const Segment SGI122086[] = {{&S15206[44], 7}, {NULL}};
const Segment SGI122087[] = {{&S15206[44], 19}, {NULL}};
const Segment SGI122088[] = {{&S5082[38], 11}, {NULL}};
const Segment SGI122089[] = {{&S15206[30], 6}, {NULL}};
const Segment SGI122090[] = {{&S5082[31], 6}, {NULL}};
const Segment SGI122091[] = {{&S15206[31], 3}, {NULL}};
const Segment SGI122092[] = {{&S98185[44], 8}, {NULL}};
const Segment SGI122093[] = {{&S15206[77], 6}, {NULL}};
const Segment SGI122094[] = {{&S98185[44], 20}, {NULL}};
const Segment SGI122095[] = {{&S98185[30], 6}, {NULL}};
const Segment SGI122096[] = {{&S98185[31], 3}, {NULL}};
const Segment SGI122097[] = {{&S98185[78], 6}, {NULL}};
const Segment SGI122098[] = {{&S5084[44], 11}, {NULL}};
const Segment SGI122099[] = {{&S5084[44], 23}, {NULL}};
const Segment SGI122100[] = {{&S5084[30], 6}, {NULL}};
const Segment SGI122101[] = {{&S5084[116], 3}, {NULL}};
const Segment SGI122102[] = {{&S5084[81], 6}, {NULL}};
const Segment SGI122103[] = {{&S5085[44], 11}, {NULL}};
const Segment SGI122104[] = {{&S5085[44], 23}, {NULL}};
const Segment SGI122105[] = {{&S5085[30], 6}, {NULL}};
const Segment SGI122106[] = {{&S5085[116], 3}, {NULL}};
const Segment SGI122107[] = {{&S5085[81], 6}, {NULL}};
const Segment SGI122108[] = {{&S5086[44], 18}, {NULL}};
const Segment SGI122109[] = {{&S5086[44], 30}, {NULL}};
const Segment SGI122110[] = {{&S5086[30], 6}, {NULL}};
const Segment SGI122111[] = {{&S5086[123], 3}, {NULL}};
const Segment SGI122112[] = {{&S5086[88], 6}, {NULL}};
const Segment SGI122113[] = {{&S15212[44], 7}, {NULL}};
const Segment SGI122114[] = {{&S15212[44], 19}, {NULL}};
const Segment SGI122115[] = {{&S15212[30], 6}, {NULL}};
const Segment SGI122116[] = {{&S15212[31], 3}, {NULL}};
const Segment SGI122117[] = {{&S15212[77], 6}, {NULL}};
const Segment SGI122118[] = {{&S5087[44], 30}, {NULL}};
const Segment SGI122119[] = {{&S5087[44], 42}, {NULL}};
const Segment SGI122120[] = {{&S5087[30], 6}, {NULL}};
const Segment SGI122121[] = {{&S5087[135], 3}, {NULL}};
const Segment SGI122122[] = {{&S5087[100], 6}, {NULL}};
const Segment SGI122123[] = {{&S15213[44], 8}, {NULL}};
const Segment SGI122124[] = {{&S15213[44], 20}, {NULL}};
const Segment SGI122125[] = {{&S15213[30], 6}, {NULL}};
const Segment SGI122126[] = {{&S15213[31], 3}, {NULL}};
const Segment SGI122127[] = {{&S15213[78], 6}, {NULL}};
const Segment SGI122128[] = {{&S5088[38], 11}, {NULL}};
const Segment SGI122129[] = {{&S5088[31], 6}, {NULL}};
const Segment SGI122130[] = {{&S5089[44], 30}, {NULL}};
const Segment SGI122131[] = {{&S5089[44], 42}, {NULL}};
const Segment SGI122132[] = {{&S5089[30], 6}, {NULL}};
const Segment SGI122133[] = {{&S5089[135], 3}, {NULL}};
const Segment SGI122134[] = {{&S5089[100], 6}, {NULL}};
const Segment SGI122135[] = {{&S15217[44], 7}, {NULL}};
const Segment SGI122136[] = {{&S15217[44], 19}, {NULL}};
const Segment SGI122137[] = {{&S15217[30], 6}, {NULL}};
const Segment SGI122138[] = {{&S15217[31], 3}, {NULL}};
const Segment SGI122139[] = {{&S15217[77], 6}, {NULL}};
const Segment SGI122140[] = {{&S5091[44], 18}, {NULL}};
const Segment SGI122141[] = {{&S5091[44], 30}, {NULL}};
const Segment SGI122142[] = {{&S5091[30], 6}, {NULL}};
const Segment SGI122143[] = {{&S5091[123], 3}, {NULL}};
const Segment SGI122144[] = {{&S5091[88], 6}, {NULL}};
const Segment SGI122145[] = {{&S5092[38], 11}, {NULL}};
const Segment SGI122146[] = {{&S5092[31], 6}, {NULL}};
const Segment SGI122147[] = {{&S5093[44], 18}, {NULL}};
const Segment SGI122148[] = {{&S5093[44], 30}, {NULL}};
const Segment SGI122149[] = {{&S5093[30], 6}, {NULL}};
const Segment SGI122150[] = {{&S5093[123], 3}, {NULL}};
const Segment SGI122151[] = {{&S5093[88], 6}, {NULL}};
const Segment SGI122152[] = {{&S15222[44], 18}, {NULL}};
const Segment SGI122153[] = {{&S15222[44], 30}, {NULL}};
const Segment SGI122154[] = {{&S15222[30], 6}, {NULL}};
const Segment SGI122155[] = {{&S15222[31], 3}, {NULL}};
const Segment SGI122156[] = {{&S15222[88], 6}, {NULL}};
const Segment SGI122157[] = {{&S5095[44], 30}, {NULL}};
const Segment SGI122158[] = {{&S5095[44], 42}, {NULL}};
const Segment SGI122159[] = {{&S5095[30], 6}, {NULL}};
const Segment SGI122160[] = {{&S5095[135], 3}, {NULL}};
const Segment SGI122161[] = {{&S5095[100], 6}, {NULL}};
const Segment SGI122162[] = {{&S15224[44], 6}, {NULL}};
const Segment SGI122163[] = {{&S15224[44], 18}, {NULL}};
const Segment SGI122164[] = {{&S15224[30], 6}, {NULL}};
const Segment SGI122165[] = {{&S15224[31], 3}, {NULL}};
const Segment SGI122166[] = {{&S15224[76], 6}, {NULL}};
const Segment SGI122167[] = {{&S15225[44], 11}, {NULL}};
const Segment SGI122168[] = {{&S15225[44], 23}, {NULL}};
const Segment SGI122169[] = {{&S15225[30], 6}, {NULL}};
const Segment SGI122170[] = {{&S98210[44], 41}, {NULL}};
const Segment SGI122171[] = {{&S15225[31], 3}, {NULL}};
const Segment SGI122172[] = {{&S98210[44], 53}, {NULL}};
const Segment SGI122173[] = {{&S15225[81], 6}, {NULL}};
const Segment SGI122174[] = {{&S98210[30], 6}, {NULL}};
const Segment SGI122175[] = {{&S5099[38], 11}, {NULL}};
const Segment SGI122176[] = {{&S5099[31], 6}, {NULL}};
const Segment SGI122177[] = {{&S98210[31], 3}, {NULL}};
const Segment SGI122178[] = {{&S98210[111], 5}, {NULL}};
const Segment SGI122179[] = {{&S98210[148], 14}, {NULL}};
const Segment SGI122180[] = {{&S15226[117], 7}, {NULL}};
const Segment SGI122181[] = {{&S15226[117], 19}, {NULL}};
const Segment SGI122182[] = {{&S15226[103], 6}, {NULL}};
const Segment SGI122183[] = {{&S5100[44], 11}, {NULL}};
const Segment SGI122184[] = {{&S5100[44], 23}, {NULL}};
const Segment SGI122185[] = {{&S5100[30], 6}, {NULL}};
const Segment SGI122186[] = {{&S15226[22], 3}, {NULL}};
const Segment SGI122187[] = {{&S5100[116], 3}, {NULL}};
const Segment SGI122188[] = {{&S15226[65], 6}, {NULL}};
const Segment SGI122189[] = {{&S5100[81], 6}, {NULL}};
const Segment SGI122190[] = {{&S5101[44], 18}, {NULL}};
const Segment SGI122191[] = {{&S5101[44], 30}, {NULL}};
const Segment SGI122192[] = {{&S5101[30], 6}, {NULL}};
const Segment SGI122193[] = {{&S5101[123], 3}, {NULL}};
const Segment SGI122194[] = {{&S5101[88], 6}, {NULL}};
const Segment SGI122195[] = {{&S5102[42], 11}, {NULL}};
const Segment SGI122196[] = {{&S5102[30], 6}, {NULL}};
const Segment SGI122197[] = {{&S15229[44], 9}, {NULL}};
const Segment SGI122198[] = {{&S15229[44], 21}, {NULL}};
const Segment SGI122199[] = {{&S15229[30], 6}, {NULL}};
const Segment SGI122200[] = {{&S5102[114], 3}, {NULL}};
const Segment SGI122201[] = {{&S5102[79], 6}, {NULL}};
const Segment SGI122202[] = {{&S15229[31], 3}, {NULL}};
const Segment SGI122203[] = {{&S15229[79], 6}, {NULL}};
const Segment SGI122204[] = {{&S5103[44], 30}, {NULL}};
const Segment SGI122205[] = {{&S5103[44], 42}, {NULL}};
const Segment SGI122206[] = {{&S5103[30], 6}, {NULL}};
const Segment SGI122207[] = {{&S5103[135], 3}, {NULL}};
const Segment SGI122208[] = {{&S5103[100], 6}, {NULL}};
const Segment SGI122209[] = {{&S5104[44], 29}, {NULL}};
const Segment SGI122210[] = {{&S5104[44], 41}, {NULL}};
const Segment SGI122211[] = {{&S5104[30], 6}, {NULL}};
const Segment SGI122212[] = {{&S5104[134], 3}, {NULL}};
const Segment SGI122213[] = {{&S5104[99], 6}, {NULL}};
const Segment SGI122214[] = {{&S15233[44], 12}, {NULL}};
const Segment SGI122215[] = {{&S15233[44], 24}, {NULL}};
const Segment SGI122216[] = {{&S15233[30], 6}, {NULL}};
const Segment SGI122217[] = {{&S15233[31], 3}, {NULL}};
const Segment SGI122218[] = {{&S15233[82], 6}, {NULL}};
const Segment SGI122219[] = {{&S5105[44], 18}, {NULL}};
const Segment SGI122220[] = {{&S5105[44], 30}, {NULL}};
const Segment SGI122221[] = {{&S5105[30], 6}, {NULL}};
const Segment SGI122222[] = {{&S5105[123], 3}, {NULL}};
const Segment SGI122223[] = {{&S5105[88], 6}, {NULL}};
const Segment SGI122224[] = {{&S98223[45], 9}, {NULL}};
const Segment SGI122225[] = {{&S98223[30], 6}, {NULL}};
const Segment SGI122226[] = {{&S15234[44], 7}, {NULL}};
const Segment SGI122227[] = {{&S15234[44], 19}, {NULL}};
const Segment SGI122228[] = {{&S15234[30], 6}, {NULL}};
const Segment SGI122229[] = {{&S98223[31], 3}, {NULL}};
const Segment SGI122230[] = {{&S15234[31], 3}, {NULL}};
const Segment SGI122231[] = {{&S98223[80], 5}, {NULL}};
const Segment SGI122232[] = {{&S15234[77], 6}, {NULL}};
const Segment SGI122233[] = {{&S98223[138], 13}, {NULL}};
const Segment SGI122234[] = {{&S5106[44], 30}, {NULL}};
const Segment SGI122235[] = {{&S5106[44], 42}, {NULL}};
const Segment SGI122236[] = {{&S5106[30], 6}, {NULL}};
const Segment SGI122237[] = {{&S5106[135], 3}, {NULL}};
const Segment SGI122238[] = {{&S5106[100], 6}, {NULL}};
const Segment SGI122239[] = {{&S15235[44], 11}, {NULL}};
const Segment SGI122240[] = {{&S15235[44], 23}, {NULL}};
const Segment SGI122241[] = {{&S15235[30], 6}, {NULL}};
const Segment SGI122242[] = {{&S15235[31], 3}, {NULL}};
const Segment SGI122243[] = {{&S5107[42], 10}, {NULL}};
const Segment SGI122244[] = {{&S15235[81], 6}, {NULL}};
const Segment SGI122245[] = {{&S5107[30], 6}, {NULL}};
const Segment SGI122246[] = {{&S5107[113], 3}, {NULL}};
const Segment SGI122247[] = {{&S5107[78], 6}, {NULL}};
const Segment SGI122248[] = {{&S15236[44], 9}, {NULL}};
const Segment SGI122249[] = {{&S15236[44], 21}, {NULL}};
const Segment SGI122250[] = {{&S15236[30], 6}, {NULL}};
const Segment SGI122251[] = {{&S5108[42], 11}, {NULL}};
const Segment SGI122252[] = {{&S15236[31], 3}, {NULL}};
const Segment SGI122253[] = {{&S5108[30], 6}, {NULL}};
const Segment SGI122254[] = {{&S15236[79], 6}, {NULL}};
const Segment SGI122255[] = {{&S5108[114], 3}, {NULL}};
const Segment SGI122256[] = {{&S5108[79], 6}, {NULL}};
const Segment SGI122257[] = {{&S15237[44], 14}, {NULL}};
const Segment SGI122258[] = {{&S15237[44], 24}, {NULL}};
const Segment SGI122259[] = {{&S15237[30], 6}, {NULL}};
const Segment SGI122260[] = {{&S5109[44], 30}, {NULL}};
const Segment SGI122261[] = {{&S5109[44], 42}, {NULL}};
const Segment SGI122262[] = {{&S5109[30], 6}, {NULL}};
const Segment SGI122263[] = {{&S15237[31], 3}, {NULL}};
const Segment SGI122264[] = {{&S15237[82], 6}, {NULL}};
const Segment SGI122265[] = {{&S5109[135], 3}, {NULL}};
const Segment SGI122266[] = {{&S5109[100], 6}, {NULL}};
const Segment SGI122267[] = {{&S15238[44], 10}, {NULL}};
const Segment SGI122268[] = {{&S15238[44], 22}, {NULL}};
const Segment SGI122269[] = {{&S15238[30], 6}, {NULL}};
const Segment SGI122270[] = {{&S15238[31], 3}, {NULL}};
const Segment SGI122271[] = {{&S15238[80], 6}, {NULL}};
const Segment SGI122272[] = {{&S5112[44], 18}, {NULL}};
const Segment SGI122273[] = {{&S5112[44], 30}, {NULL}};
const Segment SGI122274[] = {{&S5112[30], 6}, {NULL}};
const Segment SGI122275[] = {{&S5112[123], 3}, {NULL}};
const Segment SGI122276[] = {{&S5112[88], 6}, {NULL}};
const Segment SGI122277[] = {{&S15241[44], 11}, {NULL}};
const Segment SGI122278[] = {{&S15241[44], 23}, {NULL}};
const Segment SGI122279[] = {{&S15241[30], 6}, {NULL}};
const Segment SGI122280[] = {{&S15241[31], 3}, {NULL}};
const Segment SGI122281[] = {{&S15241[81], 6}, {NULL}};
const Segment SGI122282[] = {{&S5114[42], 18}, {NULL}};
const Segment SGI122283[] = {{&S5114[30], 6}, {NULL}};
const Segment SGI122284[] = {{&S5114[121], 3}, {NULL}};
const Segment SGI122285[] = {{&S5114[86], 6}, {NULL}};
const Segment SGI122286[] = {{&S15243[44], 8}, {NULL}};
const Segment SGI122287[] = {{&S15243[44], 20}, {NULL}};
const Segment SGI122288[] = {{&S15243[30], 6}, {NULL}};
const Segment SGI122289[] = {{&S5115[42], 11}, {NULL}};
const Segment SGI122290[] = {{&S5115[30], 6}, {NULL}};
const Segment SGI122291[] = {{&S15243[31], 3}, {NULL}};
const Segment SGI122292[] = {{&S15243[78], 6}, {NULL}};
const Segment SGI122293[] = {{&S5115[114], 3}, {NULL}};
const Segment SGI122294[] = {{&S5115[79], 6}, {NULL}};
const Segment SGI122295[] = {{&S15244[44], 31}, {NULL}};
const Segment SGI122296[] = {{&S15244[44], 43}, {NULL}};
const Segment SGI122297[] = {{&S15244[30], 6}, {NULL}};
const Segment SGI122298[] = {{&S15244[31], 3}, {NULL}};
const Segment SGI122299[] = {{&S15244[101], 6}, {NULL}};
const Segment SGI122300[] = {{&S98244[34], 13}, {NULL}};
const Segment SGI122301[] = {{&S98244[27], 6}, {NULL}};
const Segment SGI122302[] = {{&S98244[104], 9}, {NULL}};
const Segment SGI122303[] = {{&S98244[77], 6}, {NULL}};
const Segment SGI122304[] = {{&S15247[44], 8}, {NULL}};
const Segment SGI122305[] = {{&S15247[44], 20}, {NULL}};
const Segment SGI122306[] = {{&S15247[30], 6}, {NULL}};
const Segment SGI122307[] = {{&S15247[31], 3}, {NULL}};
const Segment SGI122308[] = {{&S15247[78], 6}, {NULL}};
const Segment SGI122309[] = {{&S15248[44], 10}, {NULL}};
const Segment SGI122310[] = {{&S15248[44], 22}, {NULL}};
const Segment SGI122311[] = {{&S15248[30], 6}, {NULL}};
const Segment SGI122312[] = {{&S15248[31], 3}, {NULL}};
const Segment SGI122313[] = {{&S15248[80], 6}, {NULL}};
const Segment SGI122314[] = {{&S15250[44], 7}, {NULL}};
const Segment SGI122315[] = {{&S15250[44], 19}, {NULL}};
const Segment SGI122316[] = {{&S15250[30], 6}, {NULL}};
const Segment SGI122317[] = {{&S15250[31], 3}, {NULL}};
const Segment SGI122318[] = {{&S15250[77], 6}, {NULL}};
const Segment SGI122319[] = {{&S15252[44], 8}, {NULL}};
const Segment SGI122320[] = {{&S15252[44], 20}, {NULL}};
const Segment SGI122321[] = {{&S15252[30], 6}, {NULL}};
const Segment SGI122322[] = {{&S15252[31], 3}, {NULL}};
const Segment SGI122323[] = {{&S15252[78], 6}, {NULL}};
const Segment SGI122324[] = {{&S15253[44], 14}, {NULL}};
const Segment SGI122325[] = {{&S15253[44], 26}, {NULL}};
const Segment SGI122326[] = {{&S15253[30], 6}, {NULL}};
const Segment SGI122327[] = {{&S15253[31], 3}, {NULL}};
const Segment SGI122328[] = {{&S15253[84], 6}, {NULL}};
const Segment SGI122329[] = {{&S15255[44], 12}, {NULL}};
const Segment SGI122330[] = {{&S15255[44], 24}, {NULL}};
const Segment SGI122331[] = {{&S15255[30], 6}, {NULL}};
const Segment SGI122332[] = {{&S15255[31], 3}, {NULL}};
const Segment SGI122333[] = {{&S15255[82], 6}, {NULL}};
const Segment SGI122334[] = {{&S15256[44], 12}, {NULL}};
const Segment SGI122335[] = {{&S15256[44], 24}, {NULL}};
const Segment SGI122336[] = {{&S15256[30], 6}, {NULL}};
const Segment SGI122337[] = {{&S15256[31], 3}, {NULL}};
const Segment SGI122338[] = {{&S15256[82], 6}, {NULL}};
const Segment SGI122339[] = {{&S15257[44], 16}, {NULL}};
const Segment SGI122340[] = {{&S15257[44], 28}, {NULL}};
const Segment SGI122341[] = {{&S15257[30], 6}, {NULL}};
const Segment SGI122342[] = {{&S15257[31], 3}, {NULL}};
const Segment SGI122343[] = {{&S15257[86], 6}, {NULL}};
const Segment SGI122344[] = {{&S15258[44], 12}, {NULL}};
const Segment SGI122345[] = {{&S15258[44], 24}, {NULL}};
const Segment SGI122346[] = {{&S15258[30], 6}, {NULL}};
const Segment SGI122347[] = {{&S15258[31], 3}, {NULL}};
const Segment SGI122348[] = {{&S15258[82], 6}, {NULL}};
const Segment SGI122349[] = {{&S98267[45], 12}, {NULL}};
const Segment SGI122350[] = {{&S98267[30], 6}, {NULL}};
const Segment SGI122351[] = {{&S98267[31], 3}, {NULL}};
const Segment SGI122352[] = {{&S98267[83], 5}, {NULL}};
const Segment SGI122353[] = {{&S98267[141], 13}, {NULL}};
const Segment SGI122354[] = {{&S15259[44], 7}, {NULL}};
const Segment SGI122355[] = {{&S15259[44], 19}, {NULL}};
const Segment SGI122356[] = {{&S15259[30], 6}, {NULL}};
const Segment SGI122357[] = {{&S15259[31], 3}, {NULL}};
const Segment SGI122358[] = {{&S15259[77], 6}, {NULL}};
const Segment SGI122359[] = {{&S15260[44], 8}, {NULL}};
const Segment SGI122360[] = {{&S15260[44], 20}, {NULL}};
const Segment SGI122361[] = {{&S15260[30], 6}, {NULL}};
const Segment SGI122362[] = {{&S15260[31], 3}, {NULL}};
const Segment SGI122363[] = {{&S15260[78], 6}, {NULL}};
const Segment SGI122364[] = {{&S15261[44], 13}, {NULL}};
const Segment SGI122365[] = {{&S15261[44], 36}, {NULL}};
const Segment SGI122366[] = {{&S15261[30], 6}, {NULL}};
const Segment SGI122367[] = {{&S15261[31], 3}, {NULL}};
const Segment SGI122368[] = {{&S98272[44], 6}, {NULL}};
const Segment SGI122369[] = {{&S98272[44], 18}, {NULL}};
const Segment SGI122370[] = {{&S15261[94], 6}, {NULL}};
const Segment SGI122371[] = {{&S98272[30], 6}, {NULL}};
const Segment SGI122372[] = {{&S98272[31], 3}, {NULL}};
const Segment SGI122373[] = {{&S98272[76], 5}, {NULL}};
const Segment SGI122374[] = {{&S98272[113], 14}, {NULL}};
const Segment SGI122375[] = {{&S15262[44], 6}, {NULL}};
const Segment SGI122376[] = {{&S15262[44], 18}, {NULL}};
const Segment SGI122377[] = {{&S15262[30], 6}, {NULL}};
const Segment SGI122378[] = {{&S15262[31], 3}, {NULL}};
const Segment SGI122379[] = {{&S15262[76], 6}, {NULL}};
const Segment SGI122380[] = {{&S15263[44], 8}, {NULL}};
const Segment SGI122381[] = {{&S15263[44], 20}, {NULL}};
const Segment SGI122382[] = {{&S15263[30], 6}, {NULL}};
const Segment SGI122383[] = {{&S15263[31], 3}, {NULL}};
const Segment SGI122384[] = {{&S15263[78], 6}, {NULL}};
const Segment SGI122385[] = {{&S15266[44], 9}, {NULL}};
const Segment SGI122386[] = {{&S15266[44], 21}, {NULL}};
const Segment SGI122387[] = {{&S15266[30], 6}, {NULL}};
const Segment SGI122388[] = {{&S15266[31], 3}, {NULL}};
const Segment SGI122389[] = {{&S15266[79], 6}, {NULL}};
const Segment SGI122390[] = {{&S15270[44], 6}, {NULL}};
const Segment SGI122391[] = {{&S15270[44], 18}, {NULL}};
const Segment SGI122392[] = {{&S15270[30], 6}, {NULL}};
const Segment SGI122393[] = {{&S15270[31], 3}, {NULL}};
const Segment SGI122394[] = {{&S15270[76], 6}, {NULL}};
const Segment SGI122395[] = {{&S15271[44], 7}, {NULL}};
const Segment SGI122396[] = {{&S15271[44], 19}, {NULL}};
const Segment SGI122397[] = {{&S15271[30], 6}, {NULL}};
const Segment SGI122398[] = {{&S15271[31], 3}, {NULL}};
const Segment SGI122399[] = {{&S15271[77], 6}, {NULL}};
const Segment SGI122400[] = {{&S98289[45], 9}, {NULL}};
const Segment SGI122401[] = {{&S98289[30], 6}, {NULL}};
const Segment SGI122402[] = {{&S98289[31], 3}, {NULL}};
const Segment SGI122403[] = {{&S98289[80], 5}, {NULL}};
const Segment SGI122404[] = {{&S98289[138], 13}, {NULL}};
const Segment SGI122405[] = {{&S15274[44], 12}, {NULL}};
const Segment SGI122406[] = {{&S15274[44], 24}, {NULL}};
const Segment SGI122407[] = {{&S15274[30], 6}, {NULL}};
const Segment SGI122408[] = {{&S15274[31], 3}, {NULL}};
const Segment SGI122409[] = {{&S15274[82], 6}, {NULL}};
const Segment SGI122410[] = {{&S15277[44], 11}, {NULL}};
const Segment SGI122411[] = {{&S15277[44], 23}, {NULL}};
const Segment SGI122412[] = {{&S15277[30], 6}, {NULL}};
const Segment SGI122413[] = {{&S15277[31], 3}, {NULL}};
const Segment SGI122414[] = {{&S15277[81], 6}, {NULL}};
const Segment SGI122415[] = {{&S15280[44], 16}, {NULL}};
const Segment SGI122416[] = {{&S15280[44], 28}, {NULL}};
const Segment SGI122417[] = {{&S15280[30], 6}, {NULL}};
const Segment SGI122418[] = {{&S15280[31], 3}, {NULL}};
const Segment SGI122419[] = {{&S15280[86], 6}, {NULL}};
const Segment SGI122420[] = {{&S15281[44], 7}, {NULL}};
const Segment SGI122421[] = {{&S15281[44], 18}, {NULL}};
const Segment SGI122422[] = {{&S15281[30], 6}, {NULL}};
const Segment SGI122423[] = {{&S15281[31], 3}, {NULL}};
const Segment SGI122424[] = {{&S15281[76], 6}, {NULL}};
const Segment SGI122425[] = {{&S15285[44], 11}, {NULL}};
const Segment SGI122426[] = {{&S15285[44], 23}, {NULL}};
const Segment SGI122427[] = {{&S15285[30], 6}, {NULL}};
const Segment SGI122428[] = {{&S15285[84], 6}, {NULL}};
const Segment SGI122429[] = {{&S15289[44], 8}, {NULL}};
const Segment SGI122430[] = {{&S15289[44], 20}, {NULL}};
const Segment SGI122431[] = {{&S15289[30], 6}, {NULL}};
const Segment SGI122432[] = {{&S15289[31], 3}, {NULL}};
const Segment SGI122433[] = {{&S15289[78], 6}, {NULL}};
const Segment SGI122434[] = {{&S15294[44], 7}, {NULL}};
const Segment SGI122435[] = {{&S15294[44], 19}, {NULL}};
const Segment SGI122436[] = {{&S15294[30], 6}, {NULL}};
const Segment SGI122437[] = {{&S15294[31], 3}, {NULL}};
const Segment SGI122438[] = {{&S15294[77], 6}, {NULL}};
const Segment SGI122439[] = {{&S15295[44], 6}, {NULL}};
const Segment SGI122440[] = {{&S15295[44], 18}, {NULL}};
const Segment SGI122441[] = {{&S15295[30], 6}, {NULL}};
const Segment SGI122442[] = {{&S15295[31], 3}, {NULL}};
const Segment SGI122443[] = {{&S15295[76], 6}, {NULL}};
const Segment SGI122444[] = {{&S15297[44], 13}, {NULL}};
const Segment SGI122445[] = {{&S15297[44], 25}, {NULL}};
const Segment SGI122446[] = {{&S15297[30], 6}, {NULL}};
const Segment SGI122447[] = {{&S15297[31], 3}, {NULL}};
const Segment SGI122448[] = {{&S15297[83], 6}, {NULL}};
const Segment SGI122449[] = {{&S15298[44], 6}, {NULL}};
const Segment SGI122450[] = {{&S15298[44], 18}, {NULL}};
const Segment SGI122451[] = {{&S15298[30], 6}, {NULL}};
const Segment SGI122452[] = {{&S15298[31], 3}, {NULL}};
const Segment SGI122453[] = {{&S15298[76], 6}, {NULL}};
const Segment SGI122454[] = {{&S98295[34], 18}, {NULL}};
const Segment SGI122455[] = {{&S98295[27], 6}, {NULL}};
const Segment SGI122456[] = {{&S98295[109], 9}, {NULL}};
const Segment SGI122457[] = {{&S98295[82], 6}, {NULL}};
const Segment SGI122458[] = {{&S15301[44], 9}, {NULL}};
const Segment SGI122459[] = {{&S15301[44], 21}, {NULL}};
const Segment SGI122460[] = {{&S15301[30], 6}, {NULL}};
const Segment SGI122461[] = {{&S15301[31], 3}, {NULL}};
const Segment SGI122462[] = {{&S15301[79], 6}, {NULL}};
const Segment SGI122463[] = {{&S15302[44], 11}, {NULL}};
const Segment SGI122464[] = {{&S15302[44], 23}, {NULL}};
const Segment SGI122465[] = {{&S15302[30], 6}, {NULL}};
const Segment SGI122466[] = {{&S15302[31], 3}, {NULL}};
const Segment SGI122467[] = {{&S15302[81], 6}, {NULL}};
const Segment SGI122468[] = {{&S15303[44], 6}, {NULL}};
const Segment SGI122469[] = {{&S15303[44], 18}, {NULL}};
const Segment SGI122470[] = {{&S15303[30], 6}, {NULL}};
const Segment SGI122471[] = {{&S15303[31], 3}, {NULL}};
const Segment SGI122472[] = {{&S15303[76], 6}, {NULL}};
const Segment SGI122473[] = {{&S15306[44], 8}, {NULL}};
const Segment SGI122474[] = {{&S15306[44], 20}, {NULL}};
const Segment SGI122475[] = {{&S15306[30], 6}, {NULL}};
const Segment SGI122476[] = {{&S15306[31], 3}, {NULL}};
const Segment SGI122477[] = {{&S15306[78], 6}, {NULL}};
const Segment SGI122478[] = {{&S98307[34], 8}, {NULL}};
const Segment SGI122479[] = {{&S98307[27], 6}, {NULL}};
const Segment SGI122480[] = {{&S98307[95], 9}, {NULL}};
const Segment SGI122481[] = {{&S98307[68], 6}, {NULL}};
const Segment SGI122482[] = {{&S15307[44], 8}, {NULL}};
const Segment SGI122483[] = {{&S15307[44], 20}, {NULL}};
const Segment SGI122484[] = {{&S15307[30], 6}, {NULL}};
const Segment SGI122485[] = {{&S15307[31], 3}, {NULL}};
const Segment SGI122486[] = {{&S15307[78], 6}, {NULL}};
const Segment SGI122487[] = {{&S15309[44], 5}, {NULL}};
const Segment SGI122488[] = {{&S15309[44], 17}, {NULL}};
const Segment SGI122489[] = {{&S15309[30], 6}, {NULL}};
const Segment SGI122490[] = {{&S15309[31], 3}, {NULL}};
const Segment SGI122491[] = {{&S15309[75], 6}, {NULL}};
const Segment SGI122492[] = {{&S98323[34], 8}, {NULL}};
const Segment SGI122493[] = {{&S98323[27], 6}, {NULL}};
const Segment SGI122494[] = {{&S98323[95], 9}, {NULL}};
const Segment SGI122495[] = {{&S98323[68], 6}, {NULL}};
const Segment SGI122496[] = {{&S98386[44], 18}, {NULL}};
const Segment SGI122497[] = {{&S98386[44], 30}, {NULL}};
const Segment SGI122498[] = {{&S98386[30], 6}, {NULL}};
const Segment SGI122499[] = {{&S98386[88], 6}, {NULL}};
const Segment SGI122500[] = {{&S98386[115], 13}, {NULL}};
const Segment SGI122501[] = {{&S98401[34], 11}, {NULL}};
const Segment SGI122502[] = {{&S98401[27], 6}, {NULL}};
const Segment SGI122503[] = {{&S98401[98], 9}, {NULL}};
const Segment SGI122504[] = {{&S98401[71], 6}, {NULL}};
const Segment SGI122505[] = {{&S15310[44], 7}, {NULL}};
const Segment SGI122506[] = {{&S15310[44], 19}, {NULL}};
const Segment SGI122507[] = {{&S15310[30], 6}, {NULL}};
const Segment SGI122508[] = {{&S15310[31], 3}, {NULL}};
const Segment SGI122509[] = {{&S15310[77], 6}, {NULL}};
const Segment SGI122510[] = {{&S15311[44], 9}, {NULL}};
const Segment SGI122511[] = {{&S15311[44], 22}, {NULL}};
const Segment SGI122512[] = {{&S15311[30], 6}, {NULL}};
const Segment SGI122513[] = {{&S15311[31], 3}, {NULL}};
const Segment SGI122514[] = {{&S15311[80], 6}, {NULL}};
const Segment SGI122515[] = {{&S15313[44], 28}, {NULL}};
const Segment SGI122516[] = {{&S15313[44], 40}, {NULL}};
const Segment SGI122517[] = {{&S15313[30], 6}, {NULL}};
const Segment SGI122518[] = {{&S15313[31], 3}, {NULL}};
const Segment SGI122519[] = {{&S15313[98], 6}, {NULL}};
const Segment SGI122520[] = {{&S15314[44], 16}, {NULL}};
const Segment SGI122521[] = {{&S15314[44], 28}, {NULL}};
const Segment SGI122522[] = {{&S15314[30], 6}, {NULL}};
const Segment SGI122523[] = {{&S15314[31], 3}, {NULL}};
const Segment SGI122524[] = {{&S15314[86], 6}, {NULL}};
const Segment SGI122525[] = {{&S15315[11], 1}, {NULL}};
const Segment SGI122526[] = {{&S15315[44], 22}, {NULL}};
const Segment SGI122527[] = {{&S15315[30], 6}, {NULL}};
const Segment SGI122528[] = {{&S15315[31], 3}, {NULL}};
const Segment SGI122529[] = {{&S15315[80], 6}, {NULL}};
const Segment SGI122530[] = {{&S15316[44], 11}, {NULL}};
const Segment SGI122531[] = {{&S15316[44], 26}, {NULL}};
const Segment SGI122532[] = {{&S15316[30], 6}, {NULL}};
const Segment SGI122533[] = {{&S15316[31], 3}, {NULL}};
const Segment SGI122534[] = {{&S15316[84], 6}, {NULL}};
const Segment SGI122535[] = {{&S15317[44], 11}, {NULL}};
const Segment SGI122536[] = {{&S15317[44], 23}, {NULL}};
const Segment SGI122537[] = {{&S15317[30], 6}, {NULL}};
const Segment SGI122538[] = {{&S15317[31], 3}, {NULL}};
const Segment SGI122539[] = {{&S15317[81], 6}, {NULL}};
const Segment SGI122540[] = {{&S15318[44], 6}, {NULL}};
const Segment SGI122541[] = {{&S15318[44], 18}, {NULL}};
const Segment SGI122542[] = {{&S15318[30], 6}, {NULL}};
const Segment SGI122543[] = {{&S15318[31], 3}, {NULL}};
const Segment SGI122544[] = {{&S15318[76], 6}, {NULL}};
const Segment SGI122545[] = {{&S15319[44], 14}, {NULL}};
const Segment SGI122546[] = {{&S15319[44], 26}, {NULL}};
const Segment SGI122547[] = {{&S15319[30], 6}, {NULL}};
const Segment SGI122548[] = {{&S15319[31], 3}, {NULL}};
const Segment SGI122549[] = {{&S15319[84], 6}, {NULL}};
const Segment SGI122550[] = {{&S15320[44], 5}, {NULL}};
const Segment SGI122551[] = {{&S15320[44], 17}, {NULL}};
const Segment SGI122552[] = {{&S15320[30], 6}, {NULL}};
const Segment SGI122553[] = {{&S15320[31], 3}, {NULL}};
const Segment SGI122554[] = {{&S15320[75], 6}, {NULL}};
const Segment SGI122555[] = {{&S15322[44], 11}, {NULL}};
const Segment SGI122556[] = {{&S15322[44], 23}, {NULL}};
const Segment SGI122557[] = {{&S15322[30], 6}, {NULL}};
const Segment SGI122558[] = {{&S15322[31], 3}, {NULL}};
const Segment SGI122559[] = {{&S15322[81], 6}, {NULL}};
const Segment SGI122560[] = {{&S15326[44], 9}, {NULL}};
const Segment SGI122561[] = {{&S15326[44], 21}, {NULL}};
const Segment SGI122562[] = {{&S15326[30], 6}, {NULL}};
const Segment SGI122563[] = {{&S15326[31], 3}, {NULL}};
const Segment SGI122564[] = {{&S15326[79], 6}, {NULL}};
const Segment SGI122565[] = {{&S15327[44], 16}, {NULL}};
const Segment SGI122566[] = {{&S15327[44], 28}, {NULL}};
const Segment SGI122567[] = {{&S15327[30], 6}, {NULL}};
const Segment SGI122568[] = {{&S15327[31], 3}, {NULL}};
const Segment SGI122569[] = {{&S15327[86], 6}, {NULL}};
const Segment SGI122570[] = {{&S15329[44], 21}, {NULL}};
const Segment SGI122571[] = {{&S15329[44], 33}, {NULL}};
const Segment SGI122572[] = {{&S15329[30], 6}, {NULL}};
const Segment SGI122573[] = {{&S15329[31], 3}, {NULL}};
const Segment SGI122574[] = {{&S15329[91], 6}, {NULL}};
const Segment SGI122575[] = {{&S15331[11], 1}, {NULL}};
const Segment SGI122576[] = {{&S15331[44], 18}, {NULL}};
const Segment SGI122577[] = {{&S15331[30], 6}, {NULL}};
const Segment SGI122578[] = {{&S15331[31], 3}, {NULL}};
const Segment SGI122579[] = {{&S15331[76], 6}, {NULL}};
const Segment SGI122580[] = {{&S15333[44], 5}, {NULL}};
const Segment SGI122581[] = {{&S15333[44], 17}, {NULL}};
const Segment SGI122582[] = {{&S15333[30], 6}, {NULL}};
const Segment SGI122583[] = {{&S15333[31], 3}, {NULL}};
const Segment SGI122584[] = {{&S15333[75], 6}, {NULL}};
const Segment SGI122585[] = {{&S15334[44], 14}, {NULL}};
const Segment SGI122586[] = {{&S15334[44], 26}, {NULL}};
const Segment SGI122587[] = {{&S15334[30], 6}, {NULL}};
const Segment SGI122588[] = {{&S15334[31], 3}, {NULL}};
const Segment SGI122589[] = {{&S15334[84], 6}, {NULL}};
const Segment SGI122590[] = {{&S15335[44], 9}, {NULL}};
const Segment SGI122591[] = {{&S15335[44], 21}, {NULL}};
const Segment SGI122592[] = {{&S15335[30], 6}, {NULL}};
const Segment SGI122593[] = {{&S15335[31], 3}, {NULL}};
const Segment SGI122594[] = {{&S15335[79], 6}, {NULL}};
const Segment SGI122595[] = {{&S15336[44], 7}, {NULL}};
const Segment SGI122596[] = {{&S15336[44], 30}, {NULL}};
const Segment SGI122597[] = {{&S15336[30], 6}, {NULL}};
const Segment SGI122598[] = {{&S15336[31], 3}, {NULL}};
const Segment SGI122599[] = {{&S15336[88], 6}, {NULL}};
const Segment SGI122600[] = {{&S15337[44], 10}, {NULL}};
const Segment SGI122601[] = {{&S15337[44], 22}, {NULL}};
const Segment SGI122602[] = {{&S15337[30], 6}, {NULL}};
const Segment SGI122603[] = {{&S15337[31], 3}, {NULL}};
const Segment SGI122604[] = {{&S15337[80], 6}, {NULL}};
const Segment SGI122605[] = {{&S15338[44], 21}, {NULL}};
const Segment SGI122606[] = {{&S15338[44], 33}, {NULL}};
const Segment SGI122607[] = {{&S15338[30], 6}, {NULL}};
const Segment SGI122608[] = {{&S15338[31], 3}, {NULL}};
const Segment SGI122609[] = {{&S15338[91], 6}, {NULL}};
const Segment SGI122610[] = {{&S15339[44], 9}, {NULL}};
const Segment SGI122611[] = {{&S15339[44], 31}, {NULL}};
const Segment SGI122612[] = {{&S15339[30], 6}, {NULL}};
const Segment SGI122613[] = {{&S15339[31], 3}, {NULL}};
const Segment SGI122614[] = {{&S15339[89], 6}, {NULL}};
const Segment SGI122615[] = {{&S15341[44], 20}, {NULL}};
const Segment SGI122616[] = {{&S15341[44], 32}, {NULL}};
const Segment SGI122617[] = {{&S15341[30], 6}, {NULL}};
const Segment SGI122618[] = {{&S15341[31], 3}, {NULL}};
const Segment SGI122619[] = {{&S15341[90], 6}, {NULL}};
const Segment SGI122620[] = {{&S15342[44], 9}, {NULL}};
const Segment SGI122621[] = {{&S15342[44], 21}, {NULL}};
const Segment SGI122622[] = {{&S15342[30], 6}, {NULL}};
const Segment SGI122623[] = {{&S15342[31], 3}, {NULL}};
const Segment SGI122624[] = {{&S15342[79], 6}, {NULL}};
const Segment SGI122625[] = {{&S15343[44], 29}, {NULL}};
const Segment SGI122626[] = {{&S15343[44], 41}, {NULL}};
const Segment SGI122627[] = {{&S15343[30], 6}, {NULL}};
const Segment SGI122628[] = {{&S15343[31], 3}, {NULL}};
const Segment SGI122629[] = {{&S15343[99], 6}, {NULL}};
const Segment SGI122630[] = {{&S98445[34], 5}, {NULL}};
const Segment SGI122631[] = {{&S98445[27], 6}, {NULL}};
const Segment SGI122632[] = {{&S98445[92], 9}, {NULL}};
const Segment SGI122633[] = {{&S98445[65], 6}, {NULL}};
const Segment SGI122634[] = {{&S15347[44], 14}, {NULL}};
const Segment SGI122635[] = {{&S15347[44], 23}, {NULL}};
const Segment SGI122636[] = {{&S15347[30], 6}, {NULL}};
const Segment SGI122637[] = {{&S15347[31], 3}, {NULL}};
const Segment SGI122638[] = {{&S15347[81], 6}, {NULL}};
const Segment SGI122639[] = {{&S15348[44], 9}, {NULL}};
const Segment SGI122640[] = {{&S15348[44], 21}, {NULL}};
const Segment SGI122641[] = {{&S15348[30], 6}, {NULL}};
const Segment SGI122642[] = {{&S15348[31], 3}, {NULL}};
const Segment SGI122643[] = {{&S15348[79], 6}, {NULL}};
const Segment SGI122644[] = {{&S15351[44], 7}, {NULL}};
const Segment SGI122645[] = {{&S15351[44], 19}, {NULL}};
const Segment SGI122646[] = {{&S15351[30], 6}, {NULL}};
const Segment SGI122647[] = {{&S15351[31], 3}, {NULL}};
const Segment SGI122648[] = {{&S15351[77], 6}, {NULL}};
const Segment SGI122649[] = {{&S15352[44], 15}, {NULL}};
const Segment SGI122650[] = {{&S15352[44], 27}, {NULL}};
const Segment SGI122651[] = {{&S15352[30], 6}, {NULL}};
const Segment SGI122652[] = {{&S15352[31], 3}, {NULL}};
const Segment SGI122653[] = {{&S15352[85], 6}, {NULL}};
const Segment SGI122654[] = {{&S15353[44], 6}, {NULL}};
const Segment SGI122655[] = {{&S15353[44], 18}, {NULL}};
const Segment SGI122656[] = {{&S15353[30], 6}, {NULL}};
const Segment SGI122657[] = {{&S15353[31], 3}, {NULL}};
const Segment SGI122658[] = {{&S15353[76], 6}, {NULL}};
const Segment SGI122659[] = {{&S15355[11], 1}, {NULL}};
const Segment SGI122660[] = {{&S15355[44], 23}, {NULL}};
const Segment SGI122661[] = {{&S15355[30], 6}, {NULL}};
const Segment SGI122662[] = {{&S15355[31], 3}, {NULL}};
const Segment SGI122663[] = {{&S15355[81], 6}, {NULL}};
const Segment SGI122664[] = {{&S15356[44], 4}, {NULL}};
const Segment SGI122665[] = {{&S15356[44], 16}, {NULL}};
const Segment SGI122666[] = {{&S15356[30], 6}, {NULL}};
const Segment SGI122667[] = {{&S15356[31], 3}, {NULL}};
const Segment SGI122668[] = {{&S15356[74], 6}, {NULL}};
const Segment SGI122669[] = {{&S15357[38], 9}, {NULL}};
const Segment SGI122670[] = {{&S15357[31], 6}, {NULL}};
const Segment SGI122671[] = {{&S15360[44], 16}, {NULL}};
const Segment SGI122672[] = {{&S15360[44], 28}, {NULL}};
const Segment SGI122673[] = {{&S15360[30], 6}, {NULL}};
const Segment SGI122674[] = {{&S15360[31], 3}, {NULL}};
const Segment SGI122675[] = {{&S15360[86], 6}, {NULL}};
const Segment SGI122676[] = {{&S98471[44], 11}, {NULL}};
const Segment SGI122677[] = {{&S98471[44], 23}, {NULL}};
const Segment SGI122678[] = {{&S98471[30], 6}, {NULL}};
const Segment SGI122679[] = {{&S98471[81], 6}, {NULL}};
const Segment SGI122680[] = {{&S98471[108], 13}, {NULL}};
const Segment SGI122681[] = {{&S15361[44], 10}, {NULL}};
const Segment SGI122682[] = {{&S15361[44], 22}, {NULL}};
const Segment SGI122683[] = {{&S15361[30], 6}, {NULL}};
const Segment SGI122684[] = {{&S15361[31], 3}, {NULL}};
const Segment SGI122685[] = {{&S15361[80], 6}, {NULL}};
const Segment SGI122686[] = {{&S15363[44], 19}, {NULL}};
const Segment SGI122687[] = {{&S15363[44], 31}, {NULL}};
const Segment SGI122688[] = {{&S15363[30], 6}, {NULL}};
const Segment SGI122689[] = {{&S15363[31], 3}, {NULL}};
const Segment SGI122690[] = {{&S15363[89], 6}, {NULL}};
const Segment SGI122691[] = {{&S15364[44], 9}, {NULL}};
const Segment SGI122692[] = {{&S15364[44], 20}, {NULL}};
const Segment SGI122693[] = {{&S15364[30], 6}, {NULL}};
const Segment SGI122694[] = {{&S15364[31], 3}, {NULL}};
const Segment SGI122695[] = {{&S15364[78], 5}, {NULL}};
const Segment SGI122696[] = {{&S15365[44], 7}, {NULL}};
const Segment SGI122697[] = {{&S15365[44], 19}, {NULL}};
const Segment SGI122698[] = {{&S15365[30], 6}, {NULL}};
const Segment SGI122699[] = {{&S15365[31], 3}, {NULL}};
const Segment SGI122700[] = {{&S15365[77], 6}, {NULL}};
const Segment SGI122701[] = {{&S98480[34], 11}, {NULL}};
const Segment SGI122702[] = {{&S98480[27], 6}, {NULL}};
const Segment SGI122703[] = {{&S98480[98], 9}, {NULL}};
const Segment SGI122704[] = {{&S98480[71], 6}, {NULL}};
const Segment SGI122705[] = {{&S15366[44], 27}, {NULL}};
const Segment SGI122706[] = {{&S15366[44], 39}, {NULL}};
const Segment SGI122707[] = {{&S15366[30], 6}, {NULL}};
const Segment SGI122708[] = {{&S15366[31], 3}, {NULL}};
const Segment SGI122709[] = {{&S15366[97], 6}, {NULL}};
const Segment SGI122710[] = {{&S15367[44], 14}, {NULL}};
const Segment SGI122711[] = {{&S15367[44], 26}, {NULL}};
const Segment SGI122712[] = {{&S15367[30], 6}, {NULL}};
const Segment SGI122713[] = {{&S15367[31], 3}, {NULL}};
const Segment SGI122714[] = {{&S15367[84], 6}, {NULL}};
const Segment SGI122715[] = {{&S15368[44], 8}, {NULL}};
const Segment SGI122716[] = {{&S15368[44], 20}, {NULL}};
const Segment SGI122717[] = {{&S15368[30], 6}, {NULL}};
const Segment SGI122718[] = {{&S15368[31], 3}, {NULL}};
const Segment SGI122719[] = {{&S15368[78], 5}, {NULL}};
const Segment SGI122720[] = {{&S15370[44], 8}, {NULL}};
const Segment SGI122721[] = {{&S15370[44], 20}, {NULL}};
const Segment SGI122722[] = {{&S15370[30], 6}, {NULL}};
const Segment SGI122723[] = {{&S15370[31], 3}, {NULL}};
const Segment SGI122724[] = {{&S15370[78], 6}, {NULL}};
const Segment SGI122725[] = {{&S15371[44], 8}, {NULL}};
const Segment SGI122726[] = {{&S15371[44], 20}, {NULL}};
const Segment SGI122727[] = {{&S15371[30], 6}, {NULL}};
const Segment SGI122728[] = {{&S15371[31], 3}, {NULL}};
const Segment SGI122729[] = {{&S15371[78], 6}, {NULL}};
const Segment SGI122730[] = {{&S98603[34], 9}, {NULL}};
const Segment SGI122731[] = {{&S98603[27], 6}, {NULL}};
const Segment SGI122732[] = {{&S98603[96], 9}, {NULL}};
const Segment SGI122733[] = {{&S15372[44], 5}, {NULL}};
const Segment SGI122734[] = {{&S98603[69], 6}, {NULL}};
const Segment SGI122735[] = {{&S15372[44], 14}, {NULL}};
const Segment SGI122736[] = {{&S15372[30], 6}, {NULL}};
const Segment SGI122737[] = {{&S15372[31], 3}, {NULL}};
const Segment SGI122738[] = {{&S15372[72], 6}, {NULL}};
const Segment SGI122739[] = {{&S15373[44], 19}, {NULL}};
const Segment SGI122740[] = {{&S15373[44], 31}, {NULL}};
const Segment SGI122741[] = {{&S15373[30], 6}, {NULL}};
const Segment SGI122742[] = {{&S15373[31], 3}, {NULL}};
const Segment SGI122743[] = {{&S15373[89], 6}, {NULL}};
const Segment SGI122744[] = {{&S15374[44], 30}, {NULL}};
const Segment SGI122745[] = {{&S15374[44], 42}, {NULL}};
const Segment SGI122746[] = {{&S15374[30], 6}, {NULL}};
const Segment SGI122747[] = {{&S15374[31], 3}, {NULL}};
const Segment SGI122748[] = {{&S15374[100], 6}, {NULL}};
const Segment SGI122749[] = {{&S15375[44], 5}, {NULL}};
const Segment SGI122750[] = {{&S15375[44], 17}, {NULL}};
const Segment SGI122751[] = {{&S15375[30], 6}, {NULL}};
const Segment SGI122752[] = {{&S15375[31], 3}, {NULL}};
const Segment SGI122753[] = {{&S15375[75], 6}, {NULL}};
const Segment SGI122754[] = {{&S98610[34], 10}, {NULL}};
const Segment SGI122755[] = {{&S98610[27], 6}, {NULL}};
const Segment SGI122756[] = {{&S98610[97], 9}, {NULL}};
const Segment SGI122757[] = {{&S98610[70], 6}, {NULL}};
const Segment SGI122758[] = {{&S15376[44], 11}, {NULL}};
const Segment SGI122759[] = {{&S15376[44], 23}, {NULL}};
const Segment SGI122760[] = {{&S15376[30], 6}, {NULL}};
const Segment SGI122761[] = {{&S15376[31], 3}, {NULL}};
const Segment SGI122762[] = {{&S15376[81], 6}, {NULL}};
const Segment SGI122763[] = {{&S15377[44], 21}, {NULL}};
const Segment SGI122764[] = {{&S15377[44], 33}, {NULL}};
const Segment SGI122765[] = {{&S15377[30], 6}, {NULL}};
const Segment SGI122766[] = {{&S15377[31], 3}, {NULL}};
const Segment SGI122767[] = {{&S15377[91], 6}, {NULL}};
const Segment SGI122768[] = {{&S15382[44], 8}, {NULL}};
const Segment SGI122769[] = {{&S15382[44], 20}, {NULL}};
const Segment SGI122770[] = {{&S15382[30], 6}, {NULL}};
const Segment SGI122771[] = {{&S15382[31], 3}, {NULL}};
const Segment SGI122772[] = {{&S15382[78], 6}, {NULL}};
const Segment SGI122773[] = {{&S15383[44], 10}, {NULL}};
const Segment SGI122774[] = {{&S15383[44], 22}, {NULL}};
const Segment SGI122775[] = {{&S15383[30], 6}, {NULL}};
const Segment SGI122776[] = {{&S15383[31], 3}, {NULL}};
const Segment SGI122777[] = {{&S15383[80], 6}, {NULL}};
const Segment SGI122778[] = {{&S15384[44], 29}, {NULL}};
const Segment SGI122779[] = {{&S15384[44], 41}, {NULL}};
const Segment SGI122780[] = {{&S15384[30], 6}, {NULL}};
const Segment SGI122781[] = {{&S15384[31], 3}, {NULL}};
const Segment SGI122782[] = {{&S15384[99], 6}, {NULL}};
const Segment SGI122783[] = {{&S15388[38], 21}, {NULL}};
const Segment SGI122784[] = {{&S15388[31], 6}, {NULL}};
const Segment SGI122785[] = {{&S15389[22], 9}, {NULL}};
const Segment SGI122786[] = {{&S15389[44], 21}, {NULL}};
const Segment SGI122787[] = {{&S15389[30], 6}, {&S15389[52], 13}, {NULL}};
const Segment SGI122788[] = {{&S15389[31], 3}, {NULL}};
const Segment SGI122789[] = {{&S15389[79], 6}, {NULL}};
const Segment SGI122790[] = {{&S15390[44], 9}, {NULL}};
const Segment SGI122791[] = {{&S15390[44], 21}, {NULL}};
const Segment SGI122792[] = {{&S15390[30], 6}, {NULL}};
const Segment SGI122793[] = {{&S15390[31], 3}, {NULL}};
const Segment SGI122794[] = {{&S15390[79], 6}, {NULL}};
const Segment SGI122795[] = {{&S15392[44], 19}, {NULL}};
const Segment SGI122796[] = {{&S15392[44], 31}, {NULL}};
const Segment SGI122797[] = {{&S15392[30], 6}, {NULL}};
const Segment SGI122798[] = {{&S15392[31], 3}, {NULL}};
const Segment SGI122799[] = {{&S15392[89], 6}, {NULL}};
const Segment SGI122800[] = {{&S15393[44], 22}, {NULL}};
const Segment SGI122801[] = {{&S15393[44], 34}, {NULL}};
const Segment SGI122802[] = {{&S15393[30], 6}, {NULL}};
const Segment SGI122803[] = {{&S15393[31], 3}, {NULL}};
const Segment SGI122804[] = {{&S15393[92], 6}, {NULL}};
const Segment SGI122805[] = {{&S15395[44], 23}, {NULL}};
const Segment SGI122806[] = {{&S15395[44], 35}, {NULL}};
const Segment SGI122807[] = {{&S15395[30], 6}, {NULL}};
const Segment SGI122808[] = {{&S15395[31], 3}, {NULL}};
const Segment SGI122809[] = {{&S15395[93], 6}, {NULL}};
const Segment SGI122810[] = {{&S15396[11], 1}, {NULL}};
const Segment SGI122811[] = {{&S15396[44], 24}, {NULL}};
const Segment SGI122812[] = {{&S15396[30], 6}, {NULL}};
const Segment SGI122813[] = {{&S15396[31], 3}, {NULL}};
const Segment SGI122814[] = {{&S15396[82], 6}, {NULL}};
const Segment SGI122815[] = {{&S15397[44], 13}, {NULL}};
const Segment SGI122816[] = {{&S15397[44], 25}, {NULL}};
const Segment SGI122817[] = {{&S15397[30], 6}, {NULL}};
const Segment SGI122818[] = {{&S15397[31], 3}, {NULL}};
const Segment SGI122819[] = {{&S15397[83], 6}, {NULL}};
const Segment SGI122820[] = {{&S98643[44], 10}, {NULL}};
const Segment SGI122821[] = {{&S98643[44], 22}, {NULL}};
const Segment SGI122822[] = {{&S98643[30], 6}, {NULL}};
const Segment SGI122823[] = {{&S98643[80], 6}, {NULL}};
const Segment SGI122824[] = {{&S15399[44], 6}, {NULL}};
const Segment SGI122825[] = {{&S98643[107], 13}, {NULL}};
const Segment SGI122826[] = {{&S15399[44], 18}, {NULL}};
const Segment SGI122827[] = {{&S15399[30], 6}, {NULL}};
const Segment SGI122828[] = {{&S15399[31], 3}, {NULL}};
const Segment SGI122829[] = {{&S15399[76], 6}, {NULL}};
const Segment SGI122830[] = {{&S15404[38], 9}, {NULL}};
const Segment SGI122831[] = {{&S15404[31], 6}, {NULL}};
const Segment SGI122832[] = {{&S15406[44], 38}, {NULL}};
const Segment SGI122833[] = {{&S15406[44], 50}, {NULL}};
const Segment SGI122834[] = {{&S15406[30], 6}, {NULL}};
const Segment SGI122835[] = {{&S15406[31], 3}, {NULL}};
const Segment SGI122836[] = {{&S15406[108], 6}, {NULL}};
const Segment SGI122837[] = {{&S15408[44], 5}, {NULL}};
const Segment SGI122838[] = {{&S15408[44], 17}, {NULL}};
const Segment SGI122839[] = {{&S15408[30], 6}, {NULL}};
const Segment SGI122840[] = {{&S15408[31], 3}, {NULL}};
const Segment SGI122841[] = {{&S15408[75], 6}, {NULL}};
const Segment SGI122842[] = {{&S98654[59], 4}, {NULL}};
const Segment SGI122843[] = {{&S98654[59], 26}, {NULL}};
const Segment SGI122844[] = {{&S98654[45], 6}, {NULL}};
const Segment SGI122845[] = {{&S98654[46], 3}, {NULL}};
const Segment SGI122846[] = {{&S98654[99], 6}, {NULL}};
const Segment SGI122847[] = {{&S15409[44], 24}, {NULL}};
const Segment SGI122848[] = {{&S98654[0], 14}, {NULL}};
const Segment SGI122849[] = {{&S15409[44], 36}, {NULL}};
const Segment SGI122850[] = {{&S15409[30], 6}, {NULL}};
const Segment SGI122851[] = {{&S15409[31], 3}, {NULL}};
const Segment SGI122852[] = {{&S15409[94], 6}, {NULL}};
const Segment SGI122853[] = {{&S98655[59], 6}, {NULL}};
const Segment SGI122854[] = {{&S98655[59], 18}, {NULL}};
const Segment SGI122855[] = {{&S98655[45], 6}, {NULL}};
const Segment SGI122856[] = {{&S98655[46], 3}, {NULL}};
const Segment SGI122857[] = {{&S98655[91], 6}, {NULL}};
const Segment SGI122858[] = {{&S98655[0], 14}, {NULL}};
const Segment SGI122859[] = {{&S15413[44], 30}, {NULL}};
const Segment SGI122860[] = {{&S15413[44], 42}, {NULL}};
const Segment SGI122861[] = {{&S15413[30], 6}, {NULL}};
const Segment SGI122862[] = {{&S15413[31], 3}, {NULL}};
const Segment SGI122863[] = {{&S15413[100], 6}, {NULL}};
const Segment SGI122864[] = {{&S15414[44], 15}, {NULL}};
const Segment SGI122865[] = {{&S15414[44], 27}, {NULL}};
const Segment SGI122866[] = {{&S15414[30], 6}, {NULL}};
const Segment SGI122867[] = {{&S15414[31], 3}, {NULL}};
const Segment SGI122868[] = {{&S15414[85], 6}, {NULL}};
const Segment SGI122869[] = {{&S15415[44], 19}, {NULL}};
const Segment SGI122870[] = {{&S15415[44], 31}, {NULL}};
const Segment SGI122871[] = {{&S15415[30], 6}, {NULL}};
const Segment SGI122872[] = {{&S15415[31], 3}, {NULL}};
const Segment SGI122873[] = {{&S15415[89], 6}, {NULL}};
const Segment SGI122874[] = {{&S15416[44], 18}, {NULL}};
const Segment SGI122875[] = {{&S15416[44], 30}, {NULL}};
const Segment SGI122876[] = {{&S15416[30], 6}, {NULL}};
const Segment SGI122877[] = {{&S98663[45], 9}, {NULL}};
const Segment SGI122878[] = {{&S98663[30], 6}, {NULL}};
const Segment SGI122879[] = {{&S15416[31], 3}, {NULL}};
const Segment SGI122880[] = {{&S15416[88], 6}, {NULL}};
const Segment SGI122881[] = {{&S98663[31], 3}, {NULL}};
const Segment SGI122882[] = {{&S98663[80], 5}, {NULL}};
const Segment SGI122883[] = {{&S98663[138], 13}, {NULL}};
const Segment SGI122884[] = {{&S15417[44], 6}, {NULL}};
const Segment SGI122885[] = {{&S15417[44], 24}, {NULL}};
const Segment SGI122886[] = {{&S15417[30], 6}, {NULL}};
const Segment SGI122887[] = {{&S15417[31], 3}, {NULL}};
const Segment SGI122888[] = {{&S15417[82], 5}, {NULL}};
const Segment SGI122889[] = {{&S98665[59], 13}, {NULL}};
const Segment SGI122890[] = {{&S98665[45], 6}, {NULL}};
const Segment SGI122891[] = {{&S98665[46], 3}, {NULL}};
const Segment SGI122892[] = {{&S98665[100], 6}, {NULL}};
const Segment SGI122893[] = {{&S98665[0], 14}, {NULL}};
const Segment SGI122894[] = {{&S15419[44], 10}, {NULL}};
const Segment SGI122895[] = {{&S15419[44], 22}, {NULL}};
const Segment SGI122896[] = {{&S15419[30], 6}, {NULL}};
const Segment SGI122897[] = {{&S15419[31], 3}, {NULL}};
const Segment SGI122898[] = {{&S15419[80], 5}, {NULL}};
const Segment SGI122899[] = {{&S15420[44], 9}, {NULL}};
const Segment SGI122900[] = {{&S15420[44], 21}, {NULL}};
const Segment SGI122901[] = {{&S15420[30], 6}, {NULL}};
const Segment SGI122902[] = {{&S15420[31], 3}, {NULL}};
const Segment SGI122903[] = {{&S15420[79], 6}, {NULL}};
const Segment SGI122904[] = {{&S15422[11], 1}, {NULL}};
const Segment SGI122905[] = {{&S15422[44], 19}, {NULL}};
const Segment SGI122906[] = {{&S15422[30], 6}, {NULL}};
const Segment SGI122907[] = {{&S15422[31], 3}, {NULL}};
const Segment SGI122908[] = {{&S15422[77], 6}, {NULL}};
const Segment SGI122909[] = {{&S15423[44], 32}, {NULL}};
const Segment SGI122910[] = {{&S15423[44], 44}, {NULL}};
const Segment SGI122911[] = {{&S15423[30], 6}, {NULL}};
const Segment SGI122912[] = {{&S15423[31], 3}, {NULL}};
const Segment SGI122913[] = {{&S15423[102], 6}, {NULL}};
const Segment SGI122914[] = {{&S15427[34], 6}, {NULL}};
const Segment SGI122915[] = {{&S15428[38], 16}, {NULL}};
const Segment SGI122916[] = {{&S15428[31], 6}, {NULL}};
const Segment SGI122917[] = {{&S15430[44], 17}, {NULL}};
const Segment SGI122918[] = {{&S15430[44], 29}, {NULL}};
const Segment SGI122919[] = {{&S15430[30], 6}, {NULL}};
const Segment SGI122920[] = {{&S15430[31], 3}, {NULL}};
const Segment SGI122921[] = {{&S15430[87], 6}, {NULL}};
const Segment SGI122922[] = {{&S15432[11], 1}, {NULL}};
const Segment SGI122923[] = {{&S15432[44], 23}, {NULL}};
const Segment SGI122924[] = {{&S15432[30], 6}, {NULL}};
const Segment SGI122925[] = {{&S15432[31], 3}, {NULL}};
const Segment SGI122926[] = {{&S15432[81], 6}, {NULL}};
const Segment SGI122927[] = {{&S15433[44], 8}, {NULL}};
const Segment SGI122928[] = {{&S15433[44], 20}, {NULL}};
const Segment SGI122929[] = {{&S15433[30], 6}, {NULL}};
const Segment SGI122930[] = {{&S15433[31], 3}, {NULL}};
const Segment SGI122931[] = {{&S15433[78], 6}, {NULL}};
const Segment SGI122932[] = {{&S98687[45], 14}, {NULL}};
const Segment SGI122933[] = {{&S15434[11], 1}, {NULL}};
const Segment SGI122934[] = {{&S15434[44], 9}, {NULL}};
const Segment SGI122935[] = {{&S98687[30], 6}, {NULL}};
const Segment SGI122936[] = {{&S15434[30], 6}, {NULL}};
const Segment SGI122937[] = {{&S15434[31], 3}, {NULL}};
const Segment SGI122938[] = {{&S98687[31], 3}, {NULL}};
const Segment SGI122939[] = {{&S15434[67], 6}, {NULL}};
const Segment SGI122940[] = {{&S98687[85], 5}, {NULL}};
const Segment SGI122941[] = {{&S98687[143], 13}, {NULL}};
const Segment SGI122942[] = {{&S15436[44], 9}, {NULL}};
const Segment SGI122943[] = {{&S15436[44], 21}, {NULL}};
const Segment SGI122944[] = {{&S15436[30], 6}, {NULL}};
const Segment SGI122945[] = {{&S15436[31], 3}, {NULL}};
const Segment SGI122946[] = {{&S15436[79], 6}, {NULL}};
const Segment SGI122947[] = {{&S15437[44], 14}, {NULL}};
const Segment SGI122948[] = {{&S15437[44], 26}, {NULL}};
const Segment SGI122949[] = {{&S15437[30], 6}, {NULL}};
const Segment SGI122950[] = {{&S15437[31], 3}, {NULL}};
const Segment SGI122951[] = {{&S15437[84], 6}, {NULL}};
const Segment SGI122952[] = {{&S15438[44], 10}, {NULL}};
const Segment SGI122953[] = {{&S15438[44], 22}, {NULL}};
const Segment SGI122954[] = {{&S15438[30], 6}, {NULL}};
const Segment SGI122955[] = {{&S15438[31], 3}, {NULL}};
const Segment SGI122956[] = {{&S15438[80], 6}, {NULL}};
const Segment SGI122957[] = {{&S15439[38], 6}, {NULL}};
const Segment SGI122958[] = {{&S15439[31], 6}, {NULL}};
const Segment SGI122959[] = {{&S15440[44], 25}, {NULL}};
const Segment SGI122960[] = {{&S15440[44], 37}, {NULL}};
const Segment SGI122961[] = {{&S15440[30], 6}, {NULL}};
const Segment SGI122962[] = {{&S98697[44], 10}, {NULL}};
const Segment SGI122963[] = {{&S98697[44], 22}, {NULL}};
const Segment SGI122964[] = {{&S98697[30], 6}, {NULL}};
const Segment SGI122965[] = {{&S15440[31], 3}, {NULL}};
const Segment SGI122966[] = {{&S15440[95], 6}, {NULL}};
const Segment SGI122967[] = {{&S98697[31], 3}, {NULL}};
const Segment SGI122968[] = {{&S98697[80], 6}, {NULL}};
const Segment SGI122969[] = {{&S15441[44], 22}, {NULL}};
const Segment SGI122970[] = {{&S15441[44], 34}, {NULL}};
const Segment SGI122971[] = {{&S15441[30], 6}, {NULL}};
const Segment SGI122972[] = {{&S15441[31], 3}, {NULL}};
const Segment SGI122973[] = {{&S15441[92], 6}, {NULL}};
const Segment SGI122974[] = {{&S15442[44], 33}, {NULL}};
const Segment SGI122975[] = {{&S15442[44], 45}, {NULL}};
const Segment SGI122976[] = {{&S15442[30], 6}, {NULL}};
const Segment SGI122977[] = {{&S15442[31], 3}, {NULL}};
const Segment SGI122978[] = {{&S15442[103], 6}, {NULL}};
const Segment SGI122979[] = {{&S15444[38], 5}, {NULL}};
const Segment SGI122980[] = {{&S15444[31], 6}, {NULL}};
const Segment SGI122981[] = {{&S15446[44], 9}, {NULL}};
const Segment SGI122982[] = {{&S15446[44], 21}, {NULL}};
const Segment SGI122983[] = {{&S15446[30], 6}, {NULL}};
const Segment SGI122984[] = {{&S15446[31], 3}, {NULL}};
const Segment SGI122985[] = {{&S15446[79], 5}, {NULL}};
const Segment SGI122986[] = {{&S98713[34], 14}, {NULL}};
const Segment SGI122987[] = {{&S15453[44], 9}, {NULL}};
const Segment SGI122988[] = {{&S98713[27], 6}, {NULL}};
const Segment SGI122989[] = {{&S15453[44], 21}, {NULL}};
const Segment SGI122990[] = {{&S15453[30], 6}, {NULL}};
const Segment SGI122991[] = {{&S15453[31], 3}, {NULL}};
const Segment SGI122992[] = {{&S98713[101], 9}, {NULL}};
const Segment SGI122993[] = {{&S98713[74], 6}, {NULL}};
const Segment SGI122994[] = {{&S15453[79], 6}, {NULL}};
const Segment SGI122995[] = {{&S15454[44], 23}, {NULL}};
const Segment SGI122996[] = {{&S15454[44], 35}, {NULL}};
const Segment SGI122997[] = {{&S15454[30], 6}, {NULL}};
const Segment SGI122998[] = {{&S15454[31], 3}, {NULL}};
const Segment SGI122999[] = {{&S15454[93], 6}, {NULL}};
const Segment SGI123000[] = {{&S98715[45], 9}, {NULL}};
const Segment SGI123001[] = {{&S98715[30], 6}, {NULL}};
const Segment SGI123002[] = {{&S98715[31], 3}, {NULL}};
const Segment SGI123003[] = {{&S98715[80], 5}, {NULL}};
const Segment SGI123004[] = {{&S98715[138], 13}, {NULL}};
const Segment SGI123005[] = {{&S15455[44], 9}, {NULL}};
const Segment SGI123006[] = {{&S15455[44], 21}, {NULL}};
const Segment SGI123007[] = {{&S15455[30], 6}, {NULL}};
const Segment SGI123008[] = {{&S15455[31], 3}, {NULL}};
const Segment SGI123009[] = {{&S15455[79], 6}, {NULL}};
const Segment SGI123010[] = {{&S15456[44], 23}, {NULL}};
const Segment SGI123011[] = {{&S15456[44], 35}, {NULL}};
const Segment SGI123012[] = {{&S15456[30], 6}, {NULL}};
const Segment SGI123013[] = {{&S15456[31], 3}, {NULL}};
const Segment SGI123014[] = {{&S15456[93], 6}, {NULL}};
const Segment SGI123015[] = {{&S98719[45], 10}, {NULL}};
const Segment SGI123016[] = {{&S98719[30], 6}, {NULL}};
const Segment SGI123017[] = {{&S15457[44], 8}, {NULL}};
const Segment SGI123018[] = {{&S15457[44], 20}, {NULL}};
const Segment SGI123019[] = {{&S15457[30], 6}, {NULL}};
const Segment SGI123020[] = {{&S98719[31], 3}, {NULL}};
const Segment SGI123021[] = {{&S15457[31], 3}, {NULL}};
const Segment SGI123022[] = {{&S98719[81], 5}, {NULL}};
const Segment SGI123023[] = {{&S15457[78], 6}, {NULL}};
const Segment SGI123024[] = {{&S98719[139], 13}, {NULL}};
const Segment SGI123025[] = {{&S15458[44], 6}, {NULL}};
const Segment SGI123026[] = {{&S15458[44], 23}, {NULL}};
const Segment SGI123027[] = {{&S15458[30], 6}, {NULL}};
const Segment SGI123028[] = {{&S15458[31], 3}, {NULL}};
const Segment SGI123029[] = {{&S15458[81], 6}, {NULL}};
const Segment SGI123030[] = {{&S15460[44], 28}, {NULL}};
const Segment SGI123031[] = {{&S15460[44], 40}, {NULL}};
const Segment SGI123032[] = {{&S15460[30], 6}, {NULL}};
const Segment SGI123033[] = {{&S15460[31], 3}, {NULL}};
const Segment SGI123034[] = {{&S15460[98], 6}, {NULL}};
const Segment SGI123035[] = {{&S15462[44], 6}, {NULL}};
const Segment SGI123036[] = {{&S15462[44], 18}, {NULL}};
const Segment SGI123037[] = {{&S15462[30], 6}, {NULL}};
const Segment SGI123038[] = {{&S15462[31], 3}, {NULL}};
const Segment SGI123039[] = {{&S15462[76], 6}, {NULL}};
const Segment SGI123040[] = {{&S15463[11], 1}, {NULL}};
const Segment SGI123041[] = {{&S15463[44], 27}, {NULL}};
const Segment SGI123042[] = {{&S15463[30], 6}, {NULL}};
const Segment SGI123043[] = {{&S15463[31], 3}, {NULL}};
const Segment SGI123044[] = {{&S15463[85], 6}, {NULL}};
const Segment SGI123045[] = {{&S15467[38], 8}, {NULL}};
const Segment SGI123046[] = {{&S15467[31], 6}, {NULL}};
const Segment SGI123047[] = {{&S15468[44], 22}, {NULL}};
const Segment SGI123048[] = {{&S15468[44], 34}, {NULL}};
const Segment SGI123049[] = {{&S15468[30], 6}, {NULL}};
const Segment SGI123050[] = {{&S15468[31], 3}, {NULL}};
const Segment SGI123051[] = {{&S15468[92], 6}, {NULL}};
const Segment SGI123052[] = {{&S98736[27], 6}, {NULL}};
const Segment SGI123053[] = {{&S98736[102], 9}, {NULL}};
const Segment SGI123054[] = {{&S98736[75], 6}, {NULL}};
const Segment SGI123055[] = {{&S15469[44], 9}, {NULL}};
const Segment SGI123056[] = {{&S15469[44], 21}, {NULL}};
const Segment SGI123057[] = {{&S15469[30], 6}, {NULL}};
const Segment SGI123058[] = {{&S15469[31], 3}, {NULL}};
const Segment SGI123059[] = {{&S15469[79], 6}, {NULL}};
const Segment SGI123060[] = {{&S15470[44], 19}, {NULL}};
const Segment SGI123061[] = {{&S15470[44], 31}, {NULL}};
const Segment SGI123062[] = {{&S15470[30], 6}, {NULL}};
const Segment SGI123063[] = {{&S15470[31], 3}, {NULL}};
const Segment SGI123064[] = {{&S15470[89], 6}, {NULL}};
const Segment SGI123065[] = {{&S15472[44], 27}, {NULL}};
const Segment SGI123066[] = {{&S15472[44], 39}, {NULL}};
const Segment SGI123067[] = {{&S15472[30], 6}, {NULL}};
const Segment SGI123068[] = {{&S15472[31], 3}, {NULL}};
const Segment SGI123069[] = {{&S15472[97], 6}, {NULL}};
const Segment SGI123070[] = {{&S98742[34], 10}, {NULL}};
const Segment SGI123071[] = {{&S98742[27], 6}, {NULL}};
const Segment SGI123072[] = {{&S98742[110], 9}, {NULL}};
const Segment SGI123073[] = {{&S98742[83], 6}, {NULL}};
const Segment SGI123074[] = {{&S15473[44], 32}, {NULL}};
const Segment SGI123075[] = {{&S15473[44], 44}, {NULL}};
const Segment SGI123076[] = {{&S15473[30], 6}, {NULL}};
const Segment SGI123077[] = {{&S15473[31], 3}, {NULL}};
const Segment SGI123078[] = {{&S15473[102], 6}, {NULL}};
const Segment SGI123079[] = {{&S15474[44], 9}, {NULL}};
const Segment SGI123080[] = {{&S15474[44], 21}, {NULL}};
const Segment SGI123081[] = {{&S15474[30], 6}, {NULL}};
const Segment SGI123082[] = {{&S15474[31], 3}, {NULL}};
const Segment SGI123083[] = {{&S15474[79], 6}, {NULL}};
const Segment SGI123084[] = {{&S98863[34], 18}, {NULL}};
const Segment SGI123085[] = {{&S98863[27], 6}, {NULL}};
const Segment SGI123086[] = {{&S98863[109], 9}, {NULL}};
const Segment SGI123087[] = {{&S98863[82], 6}, {NULL}};
const Segment SGI123088[] = {{&S98873[34], 13}, {NULL}};
const Segment SGI123089[] = {{&S98873[27], 6}, {NULL}};
const Segment SGI123090[] = {{&S98873[104], 9}, {NULL}};
const Segment SGI123091[] = {{&S98873[77], 6}, {NULL}};
const Segment SGI123092[] = {{&S98898[44], 10}, {NULL}};
const Segment SGI123093[] = {{&S98898[44], 22}, {NULL}};
const Segment SGI123094[] = {{&S98898[30], 6}, {NULL}};
const Segment SGI123095[] = {{&S98898[31], 3}, {NULL}};
const Segment SGI123096[] = {{&S98898[80], 6}, {NULL}};
const Segment SGI123097[] = {{&S98914[44], 14}, {NULL}};
const Segment SGI123098[] = {{&S98914[44], 38}, {NULL}};
const Segment SGI123099[] = {{&S98914[30], 6}, {NULL}};
const Segment SGI123100[] = {{&S98914[31], 3}, {NULL}};
const Segment SGI123101[] = {{&S98914[96], 6}, {NULL}};
const Segment SGI123102[] = {{&S98914[156], 7}, {NULL}};
const Segment SGI123103[] = {{&S98971[44], 11}, {NULL}};
const Segment SGI123104[] = {{&S98971[44], 23}, {NULL}};
const Segment SGI123105[] = {{&S98971[30], 6}, {NULL}};
const Segment SGI123106[] = {{&S98971[81], 6}, {NULL}};
const Segment SGI123107[] = {{&S98971[108], 13}, {NULL}};
const Segment SGI123108[] = {{&S98985[44], 9}, {NULL}};
const Segment SGI123109[] = {{&S98985[44], 21}, {NULL}};
const Segment SGI123110[] = {{&S98985[30], 6}, {NULL}};
const Segment SGI123111[] = {{&S98985[79], 6}, {NULL}};
const Segment SGI123112[] = {{&S98985[106], 13}, {NULL}};
const Segment SGI123113[] = {{&S99001[34], 18}, {NULL}};
const Segment SGI123114[] = {{&S99001[27], 6}, {NULL}};
const Segment SGI123115[] = {{&S99001[105], 9}, {NULL}};
const Segment SGI123116[] = {{&S99001[78], 6}, {NULL}};
const Segment SGI123117[] = {{&S15782[44], 7}, {NULL}};
const Segment SGI123118[] = {{&S15782[44], 24}, {NULL}};
const Segment SGI123119[] = {{&S15782[30], 6}, {NULL}};
const Segment SGI123120[] = {{&S15782[31], 3}, {NULL}};
const Segment SGI123121[] = {{&S15782[82], 6}, {NULL}};
const Segment SGI123122[] = {{&S15783[44], 7}, {NULL}};
const Segment SGI123123[] = {{&S15783[44], 24}, {NULL}};
const Segment SGI123124[] = {{&S15783[30], 6}, {NULL}};
const Segment SGI123125[] = {{&S15783[31], 3}, {NULL}};
const Segment SGI123126[] = {{&S15783[82], 6}, {NULL}};
const Segment SGI123127[] = {{&S99119[31], 9}, {NULL}};
const Segment SGI123128[] = {{&S99119[55], 3}, {NULL}};
const Segment SGI123129[] = {{&S15784[44], 7}, {NULL}};
const Segment SGI123130[] = {{&S15784[44], 24}, {NULL}};
const Segment SGI123131[] = {{&S15784[30], 6}, {NULL}};
const Segment SGI123132[] = {{&S15784[85], 6}, {NULL}};
const Segment SGI123133[] = {{&S15786[44], 7}, {NULL}};
const Segment SGI123134[] = {{&S15786[44], 24}, {NULL}};
const Segment SGI123135[] = {{&S15786[30], 6}, {NULL}};
const Segment SGI123136[] = {{&S15786[31], 3}, {NULL}};
const Segment SGI123137[] = {{&S15786[82], 6}, {NULL}};
const Segment SGI123138[] = {{&S15788[38], 7}, {NULL}};
const Segment SGI123139[] = {{&S15788[31], 6}, {NULL}};
const Segment SGI123140[] = {{&S15789[44], 7}, {NULL}};
const Segment SGI123141[] = {{&S15789[44], 23}, {NULL}};
const Segment SGI123142[] = {{&S15789[30], 6}, {NULL}};
const Segment SGI123143[] = {{&S15789[31], 3}, {NULL}};
const Segment SGI123144[] = {{&S15789[81], 6}, {NULL}};
const Segment SGI123145[] = {{&S15790[44], 7}, {NULL}};
const Segment SGI123146[] = {{&S15790[44], 23}, {NULL}};
const Segment SGI123147[] = {{&S15790[30], 6}, {NULL}};
const Segment SGI123148[] = {{&S15790[84], 6}, {NULL}};
const Segment SGI123149[] = {{&S15791[44], 7}, {NULL}};
const Segment SGI123150[] = {{&S15791[44], 23}, {NULL}};
const Segment SGI123151[] = {{&S15791[30], 6}, {NULL}};
const Segment SGI123152[] = {{&S15791[31], 3}, {NULL}};
const Segment SGI123153[] = {{&S15791[81], 6}, {NULL}};
const Segment SGI123154[] = {{&S15792[44], 7}, {NULL}};
const Segment SGI123155[] = {{&S15792[44], 23}, {NULL}};
const Segment SGI123156[] = {{&S15792[30], 6}, {NULL}};
const Segment SGI123157[] = {{&S15792[31], 3}, {NULL}};
const Segment SGI123158[] = {{&S15792[81], 6}, {NULL}};
const Segment SGI123159[] = {{&S15793[44], 7}, {NULL}};
const Segment SGI123160[] = {{&S15793[44], 24}, {NULL}};
const Segment SGI123161[] = {{&S15793[30], 6}, {NULL}};
const Segment SGI123162[] = {{&S15793[85], 6}, {NULL}};
const Segment SGI123163[] = {{&S15794[44], 7}, {NULL}};
const Segment SGI123164[] = {{&S15794[44], 23}, {NULL}};
const Segment SGI123165[] = {{&S15794[30], 6}, {NULL}};
const Segment SGI123166[] = {{&S15794[31], 3}, {NULL}};
const Segment SGI123167[] = {{&S15794[81], 6}, {NULL}};
const Segment SGI123168[] = {{&S15796[44], 7}, {NULL}};
const Segment SGI123169[] = {{&S15796[44], 23}, {NULL}};
const Segment SGI123170[] = {{&S15796[30], 6}, {NULL}};
const Segment SGI123171[] = {{&S15796[84], 6}, {NULL}};
const Segment SGI123172[] = {{&S15797[44], 7}, {NULL}};
const Segment SGI123173[] = {{&S15797[44], 23}, {NULL}};
const Segment SGI123174[] = {{&S15797[30], 6}, {NULL}};
const Segment SGI123175[] = {{&S15797[84], 6}, {NULL}};
const Segment SGI123176[] = {{&S15799[44], 7}, {NULL}};
const Segment SGI123177[] = {{&S15799[44], 19}, {NULL}};
const Segment SGI123178[] = {{&S15799[30], 6}, {NULL}};
const Segment SGI123179[] = {{&S15799[31], 3}, {NULL}};
const Segment SGI123180[] = {{&S15799[77], 6}, {NULL}};
const Segment SGI123181[] = {{&S15800[42], 7}, {NULL}};
const Segment SGI123182[] = {{&S15800[30], 6}, {NULL}};
const Segment SGI123183[] = {{&S15800[31], 3}, {NULL}};
const Segment SGI123184[] = {{&S15800[79], 6}, {NULL}};
const Segment SGI123185[] = {{&S99153[44], 10}, {NULL}};
const Segment SGI123186[] = {{&S99153[44], 22}, {NULL}};
const Segment SGI123187[] = {{&S99153[30], 6}, {NULL}};
const Segment SGI123188[] = {{&S99153[31], 3}, {NULL}};
const Segment SGI123189[] = {{&S99153[80], 6}, {NULL}};
const Segment SGI123190[] = {{&S15801[44], 23}, {NULL}};
const Segment SGI123191[] = {{&S99153[132], 10}, {NULL}};
const Segment SGI123192[] = {{&S15801[30], 6}, {NULL}};
const Segment SGI123193[] = {{&S99164[34], 14}, {NULL}};
const Segment SGI123194[] = {{&S99164[27], 6}, {NULL}};
const Segment SGI123195[] = {{&S99164[101], 9}, {NULL}};
const Segment SGI123196[] = {{&S99164[74], 6}, {NULL}};
const Segment SGI123197[] = {{&S99174[34], 10}, {NULL}};
const Segment SGI123198[] = {{&S99174[27], 6}, {NULL}};
const Segment SGI123199[] = {{&S99174[94], 9}, {NULL}};
const Segment SGI123200[] = {{&S99174[67], 6}, {NULL}};
const Segment SGI123201[] = {{&S99178[44], 21}, {NULL}};
const Segment SGI123202[] = {{&S99178[44], 33}, {NULL}};
const Segment SGI123203[] = {{&S99178[30], 6}, {NULL}};
const Segment SGI123204[] = {{&S99178[31], 3}, {NULL}};
const Segment SGI123205[] = {{&S99178[91], 5}, {NULL}};
const Segment SGI123206[] = {{&S99178[128], 14}, {NULL}};
const Segment SGI123207[] = {{&S99186[34], 7}, {NULL}};
const Segment SGI123208[] = {{&S99186[27], 6}, {NULL}};
const Segment SGI123209[] = {{&S99186[94], 9}, {NULL}};
const Segment SGI123210[] = {{&S99186[67], 6}, {NULL}};
const Segment SGI123211[] = {{&S99209[45], 13}, {NULL}};
const Segment SGI123212[] = {{&S99209[30], 6}, {NULL}};
const Segment SGI123213[] = {{&S99209[31], 3}, {NULL}};
const Segment SGI123214[] = {{&S99209[88], 5}, {NULL}};
const Segment SGI123215[] = {{&S99209[146], 13}, {NULL}};
const Segment SGI123216[] = {{&S99227[45], 7}, {NULL}};
const Segment SGI123217[] = {{&S99227[30], 6}, {NULL}};
const Segment SGI123218[] = {{&S99227[31], 3}, {NULL}};
const Segment SGI123219[] = {{&S99227[78], 5}, {NULL}};
const Segment SGI123220[] = {{&S99227[136], 13}, {NULL}};
const Segment SGI123221[] = {{&S99234[44], 11}, {NULL}};
const Segment SGI123222[] = {{&S99234[44], 23}, {NULL}};
const Segment SGI123223[] = {{&S99234[30], 6}, {NULL}};
const Segment SGI123224[] = {{&S99234[81], 6}, {NULL}};
const Segment SGI123225[] = {{&S99234[108], 13}, {NULL}};
const Segment SGI123226[] = {{&S99255[44], 13}, {NULL}};
const Segment SGI123227[] = {{&S99255[44], 25}, {NULL}};
const Segment SGI123228[] = {{&S99255[30], 6}, {NULL}};
const Segment SGI123229[] = {{&S99255[31], 3}, {NULL}};
const Segment SGI123230[] = {{&S99255[83], 6}, {NULL}};
const Segment SGI123231[] = {{&S99255[143], 11}, {NULL}};
const Segment SGI123232[] = {{&S99260[34], 18}, {NULL}};
const Segment SGI123233[] = {{&S99260[27], 6}, {NULL}};
const Segment SGI123234[] = {{&S99260[105], 9}, {NULL}};
const Segment SGI123235[] = {{&S99260[78], 6}, {NULL}};
const Segment SGI123236[] = {{&S99273[44], 14}, {NULL}};
const Segment SGI123237[] = {{&S99273[44], 26}, {NULL}};
const Segment SGI123238[] = {{&S99273[30], 6}, {NULL}};
const Segment SGI123239[] = {{&S99273[31], 3}, {NULL}};
const Segment SGI123240[] = {{&S99273[84], 5}, {NULL}};
const Segment SGI123241[] = {{&S99310[44], 41}, {NULL}};
const Segment SGI123242[] = {{&S99310[44], 53}, {NULL}};
const Segment SGI123243[] = {{&S99310[30], 6}, {NULL}};
const Segment SGI123244[] = {{&S99310[31], 3}, {NULL}};
const Segment SGI123245[] = {{&S99310[111], 5}, {NULL}};
const Segment SGI123246[] = {{&S99310[148], 14}, {NULL}};
const Segment SGI123247[] = {{&S99329[44], 14}, {NULL}};
const Segment SGI123248[] = {{&S99329[44], 26}, {NULL}};
const Segment SGI123249[] = {{&S99329[30], 6}, {NULL}};
const Segment SGI123250[] = {{&S99329[84], 6}, {NULL}};
const Segment SGI123251[] = {{&S99329[111], 13}, {NULL}};
const Segment SGI123252[] = {{&S99360[44], 41}, {NULL}};
const Segment SGI123253[] = {{&S99360[44], 53}, {NULL}};
const Segment SGI123254[] = {{&S99360[30], 6}, {NULL}};
const Segment SGI123255[] = {{&S99360[31], 3}, {NULL}};
const Segment SGI123256[] = {{&S99360[111], 5}, {NULL}};
const Segment SGI123257[] = {{&S99360[148], 14}, {NULL}};
const Segment SGI123258[] = {{&S99365[44], 13}, {NULL}};
const Segment SGI123259[] = {{&S99365[44], 25}, {NULL}};
const Segment SGI123260[] = {{&S99365[30], 6}, {NULL}};
const Segment SGI123261[] = {{&S99365[83], 6}, {NULL}};
const Segment SGI123262[] = {{&S99365[110], 13}, {NULL}};
const Segment SGI123263[] = {{&S99366[44], 13}, {NULL}};
const Segment SGI123264[] = {{&S99366[30], 6}, {NULL}};
const Segment SGI123265[] = {{&S99366[85], 6}, {NULL}};
const Segment SGI123266[] = {{&S99366[112], 13}, {NULL}};
const Segment SGI123267[] = {{&S99382[44], 6}, {NULL}};
const Segment SGI123268[] = {{&S99382[44], 29}, {NULL}};
const Segment SGI123269[] = {{&S99382[30], 6}, {NULL}};
const Segment SGI123270[] = {{&S99382[87], 6}, {NULL}};
const Segment SGI123271[] = {{&S99382[114], 13}, {NULL}};
const Segment SGI123272[] = {{&S99415[44], 13}, {NULL}};
const Segment SGI123273[] = {{&S99415[44], 25}, {NULL}};
const Segment SGI123274[] = {{&S99415[30], 6}, {NULL}};
const Segment SGI123275[] = {{&S99415[31], 3}, {NULL}};
const Segment SGI123276[] = {{&S99415[83], 5}, {NULL}};
const Segment SGI123277[] = {{&S99415[120], 14}, {NULL}};
const Segment SGI123278[] = {{&S99442[44], 6}, {NULL}};
const Segment SGI123279[] = {{&S99442[44], 18}, {NULL}};
const Segment SGI123280[] = {{&S99442[30], 6}, {NULL}};
const Segment SGI123281[] = {{&S99442[31], 3}, {NULL}};
const Segment SGI123282[] = {{&S99442[76], 6}, {NULL}};
const Segment SGI123283[] = {{&S99442[129], 7}, {NULL}};
const Segment SGI123284[] = {{&S99500[44], 18}, {NULL}};
const Segment SGI123285[] = {{&S99500[44], 30}, {NULL}};
const Segment SGI123286[] = {{&S99500[30], 6}, {NULL}};
const Segment SGI123287[] = {{&S99500[88], 6}, {NULL}};
const Segment SGI123288[] = {{&S99500[115], 13}, {NULL}};
const Segment SGI123289[] = {{&S99501[34], 8}, {NULL}};
const Segment SGI123290[] = {{&S99501[27], 6}, {NULL}};
const Segment SGI123291[] = {{&S99501[95], 9}, {NULL}};
const Segment SGI123292[] = {{&S99501[68], 6}, {NULL}};
const Segment SGI123293[] = {{&S99504[44], 14}, {NULL}};
const Segment SGI123294[] = {{&S99504[44], 31}, {NULL}};
const Segment SGI123295[] = {{&S99504[30], 6}, {NULL}};
const Segment SGI123296[] = {{&S99504[89], 6}, {NULL}};
const Segment SGI123297[] = {{&S99504[116], 13}, {NULL}};
const Segment SGI123298[] = {{&S99511[44], 18}, {NULL}};
const Segment SGI123299[] = {{&S99511[44], 30}, {NULL}};
const Segment SGI123300[] = {{&S99511[30], 6}, {NULL}};
const Segment SGI123301[] = {{&S99511[88], 6}, {NULL}};
const Segment SGI123302[] = {{&S99511[115], 13}, {NULL}};
const Segment SGI123303[] = {{&S99634[34], 9}, {NULL}};
const Segment SGI123304[] = {{&S99634[27], 6}, {NULL}};
const Segment SGI123305[] = {{&S99634[96], 9}, {NULL}};
const Segment SGI123306[] = {{&S99634[69], 6}, {NULL}};
const Segment SGI123307[] = {{&S99636[34], 11}, {NULL}};
const Segment SGI123308[] = {{&S99636[27], 6}, {NULL}};
const Segment SGI123309[] = {{&S99636[98], 9}, {NULL}};
const Segment SGI123310[] = {{&S99636[71], 6}, {NULL}};
const Segment SGI123311[] = {{&S99701[45], 10}, {NULL}};
const Segment SGI123312[] = {{&S99701[30], 6}, {NULL}};
const Segment SGI123313[] = {{&S99701[31], 3}, {NULL}};
const Segment SGI123314[] = {{&S99701[81], 5}, {NULL}};
const Segment SGI123315[] = {{&S99701[139], 13}, {NULL}};
const Segment SGI123316[] = {{&S99703[44], 10}, {NULL}};
const Segment SGI123317[] = {{&S99703[44], 22}, {NULL}};
const Segment SGI123318[] = {{&S99703[30], 6}, {NULL}};
const Segment SGI123319[] = {{&S99703[31], 3}, {NULL}};
const Segment SGI123320[] = {{&S99703[80], 6}, {NULL}};
const Segment SGI123321[] = {{&S99703[132], 10}, {NULL}};
const Segment SGI123322[] = {{&S99727[34], 18}, {NULL}};
const Segment SGI123323[] = {{&S99727[27], 6}, {NULL}};
const Segment SGI123324[] = {{&S99727[105], 9}, {NULL}};
const Segment SGI123325[] = {{&S99727[78], 6}, {NULL}};
const Segment SGI123326[] = {{&S99729[34], 11}, {NULL}};
const Segment SGI123327[] = {{&S99729[27], 6}, {NULL}};
const Segment SGI123328[] = {{&S99729[98], 9}, {NULL}};
const Segment SGI123329[] = {{&S99729[71], 6}, {NULL}};
const Segment SGI123330[] = {{&S99762[59], 14}, {NULL}};
const Segment SGI123331[] = {{&S99762[59], 26}, {NULL}};
const Segment SGI123332[] = {{&S99762[45], 6}, {NULL}};
const Segment SGI123333[] = {{&S99762[46], 3}, {NULL}};
const Segment SGI123334[] = {{&S99762[99], 6}, {NULL}};
const Segment SGI123335[] = {{&S99762[0], 14}, {NULL}};
const Segment SGI123336[] = {{&S99783[45], 41}, {NULL}};
const Segment SGI123337[] = {{&S99783[30], 6}, {NULL}};
const Segment SGI123338[] = {{&S99783[31], 3}, {NULL}};
const Segment SGI123339[] = {{&S99783[112], 5}, {NULL}};
const Segment SGI123340[] = {{&S99783[170], 13}, {NULL}};
const Segment SGI123341[] = {{&S99789[45], 9}, {NULL}};
const Segment SGI123342[] = {{&S99789[30], 6}, {NULL}};
const Segment SGI123343[] = {{&S99789[31], 3}, {NULL}};
const Segment SGI123344[] = {{&S99789[80], 5}, {NULL}};
const Segment SGI123345[] = {{&S99789[138], 13}, {NULL}};
const Segment SGI123346[] = {{&S99808[34], 10}, {NULL}};
const Segment SGI123347[] = {{&S99808[27], 6}, {NULL}};
const Segment SGI123348[] = {{&S99808[97], 9}, {NULL}};
const Segment SGI123349[] = {{&S99808[70], 6}, {NULL}};
const Segment SGI123350[] = {{&S99815[44], 11}, {NULL}};
const Segment SGI123351[] = {{&S99815[44], 23}, {NULL}};
const Segment SGI123352[] = {{&S99815[30], 6}, {NULL}};
const Segment SGI123353[] = {{&S99815[81], 6}, {NULL}};
const Segment SGI123354[] = {{&S99815[108], 13}, {NULL}};
const Segment SGI123355[] = {{&S99842[34], 15}, {NULL}};
const Segment SGI123356[] = {{&S99842[27], 6}, {NULL}};
const Segment SGI123357[] = {{&S99842[102], 9}, {NULL}};
const Segment SGI123358[] = {{&S99842[75], 6}, {NULL}};
const Segment SGI123359[] = {{&S99847[34], 21}, {NULL}};
const Segment SGI123360[] = {{&S99847[27], 6}, {NULL}};
const Segment SGI123361[] = {{&S99847[108], 9}, {NULL}};
const Segment SGI123362[] = {{&S99847[81], 6}, {NULL}};
const Segment SGI123363[] = {{&S99847[149], 8}, {NULL}};
const Segment SGI123364[] = {{&S99884[34], 7}, {NULL}};
const Segment SGI123365[] = {{&S99884[27], 6}, {NULL}};
const Segment SGI123366[] = {{&S99884[94], 9}, {NULL}};
const Segment SGI123367[] = {{&S99884[67], 6}, {NULL}};
const Segment SGI123368[] = {{&S99892[34], 17}, {NULL}};
const Segment SGI123369[] = {{&S99892[27], 6}, {NULL}};
const Segment SGI123370[] = {{&S99892[104], 9}, {NULL}};
const Segment SGI123371[] = {{&S99892[77], 6}, {NULL}};
const Segment SGI123372[] = {{&S99892[145], 8}, {NULL}};
const Segment SGI123373[] = {{&S99914[44], 11}, {NULL}};
const Segment SGI123374[] = {{&S99914[44], 23}, {NULL}};
const Segment SGI123375[] = {{&S99914[30], 6}, {NULL}};
const Segment SGI123376[] = {{&S99914[31], 3}, {NULL}};
const Segment SGI123377[] = {{&S99914[81], 5}, {NULL}};
const Segment SGI123378[] = {{&S99914[118], 14}, {NULL}};
const Segment SGI123379[] = {{&S99922[44], 13}, {NULL}};
const Segment SGI123380[] = {{&S99922[44], 29}, {NULL}};
const Segment SGI123381[] = {{&S99922[30], 6}, {NULL}};
const Segment SGI123382[] = {{&S99922[87], 6}, {NULL}};
const Segment SGI123383[] = {{&S99922[114], 13}, {NULL}};
const Segment SGI123384[] = {{&S99930[44], 5}, {NULL}};
const Segment SGI123385[] = {{&S99930[44], 17}, {NULL}};
const Segment SGI123386[] = {{&S99930[30], 6}, {NULL}};
const Segment SGI123387[] = {{&S99930[75], 6}, {NULL}};
const Segment SGI123388[] = {{&S99930[102], 13}, {NULL}};
const Segment SGI123389[] = {{&S99942[44], 11}, {NULL}};
const Segment SGI123390[] = {{&S99942[44], 23}, {NULL}};
const Segment SGI123391[] = {{&S99942[30], 6}, {NULL}};
const Segment SGI123392[] = {{&S99942[31], 3}, {NULL}};
const Segment SGI123393[] = {{&S99942[81], 6}, {NULL}};
const Segment SGI123394[] = {{&S99942[141], 11}, {NULL}};
const Segment SGI123395[] = {{&S99961[44], 13}, {NULL}};
const Segment SGI123396[] = {{&S99961[44], 25}, {NULL}};
const Segment SGI123397[] = {{&S99961[30], 6}, {NULL}};
const Segment SGI123398[] = {{&S99961[83], 6}, {NULL}};
const Segment SGI123399[] = {{&S99961[110], 13}, {NULL}};
const Segment SGI123400[] = {{&S99965[34], 15}, {NULL}};
const Segment SGI123401[] = {{&S99965[27], 6}, {NULL}};
const Segment SGI123402[] = {{&S99965[102], 9}, {NULL}};
const Segment SGI123403[] = {{&S99965[75], 6}, {NULL}};
const Segment SGI123404[] = {{&S100004[44], 15}, {NULL}};
const Segment SGI123405[] = {{&S100004[44], 27}, {NULL}};
const Segment SGI123406[] = {{&S100004[30], 6}, {NULL}};
const Segment SGI123407[] = {{&S100004[85], 6}, {NULL}};
const Segment SGI123408[] = {{&S100004[112], 13}, {NULL}};
const Segment SGI123409[] = {{&S100005[52], 12}, {NULL}};
const Segment SGI123410[] = {{&S100005[52], 33}, {NULL}};
const Segment SGI123411[] = {{&S100005[38], 6}, {NULL}};
const Segment SGI123412[] = {{&S100005[99], 5}, {NULL}};
const Segment SGI123413[] = {{&S100005[0], 14}, {NULL}};
const Segment SGI123414[] = {{&S100018[34], 6}, {NULL}};
const Segment SGI123415[] = {{&S100018[27], 6}, {NULL}};
const Segment SGI123416[] = {{&S100018[93], 9}, {NULL}};
const Segment SGI123417[] = {{&S100018[66], 6}, {NULL}};
const Segment SGI123418[] = {{&S100018[134], 8}, {NULL}};
const Segment SGI123419[] = {{&S100022[34], 15}, {NULL}};
const Segment SGI123420[] = {{&S100022[27], 6}, {NULL}};
const Segment SGI123421[] = {{&S100022[102], 9}, {NULL}};
const Segment SGI123422[] = {{&S100022[75], 6}, {NULL}};
const Segment SGI123423[] = {{&S100022[143], 8}, {NULL}};
const Segment SGI123424[] = {{&S100027[44], 14}, {NULL}};
const Segment SGI123425[] = {{&S100027[44], 26}, {NULL}};
const Segment SGI123426[] = {{&S100027[30], 6}, {NULL}};
const Segment SGI123427[] = {{&S100027[119], 3}, {NULL}};
const Segment SGI123428[] = {{&S100027[84], 6}, {NULL}};
const Segment SGI123429[] = {{&S100027[139], 8}, {NULL}};
const Segment SGI123430[] = {{&S100027[148], 13}, {NULL}};
const Segment SGI123431[] = {{&S100039[44], 10}, {NULL}};
const Segment SGI123432[] = {{&S100039[44], 22}, {NULL}};
const Segment SGI123433[] = {{&S100039[30], 6}, {NULL}};
const Segment SGI123434[] = {{&S100039[115], 3}, {NULL}};
const Segment SGI123435[] = {{&S100039[80], 6}, {NULL}};
const Segment SGI123436[] = {{&S100039[135], 8}, {NULL}};
const Segment SGI123437[] = {{&S100039[144], 13}, {NULL}};
const Segment SGI123438[] = {{&S100070[44], 6}, {NULL}};
const Segment SGI123439[] = {{&S100070[44], 18}, {NULL}};
const Segment SGI123440[] = {{&S100070[30], 6}, {NULL}};
const Segment SGI123441[] = {{&S100070[76], 6}, {NULL}};
const Segment SGI123442[] = {{&S100070[103], 13}, {NULL}};
const Segment SGI123443[] = {{&S100078[34], 8}, {NULL}};
const Segment SGI123444[] = {{&S100078[27], 6}, {NULL}};
const Segment SGI123445[] = {{&S100078[95], 9}, {NULL}};
const Segment SGI123446[] = {{&S100078[68], 6}, {NULL}};
const Segment SGI123447[] = {{&S100202[30], 6}, {NULL}};
const Segment SGI123448[] = {{&S100202[31], 3}, {NULL}};
const Segment SGI123449[] = {{&S100202[86], 5}, {NULL}};
const Segment SGI123450[] = {{&S100202[144], 13}, {NULL}};
const Segment SGI123451[] = {{&S100217[57], 10}, {NULL}};
const Segment SGI123452[] = {{&S100217[57], 22}, {NULL}};
const Segment SGI123453[] = {{&S100217[43], 6}, {NULL}};
const Segment SGI123454[] = {{&S100217[44], 3}, {NULL}};
const Segment SGI123455[] = {{&S100217[93], 6}, {NULL}};
const Segment SGI123456[] = {{&S100217[0], 12}, {NULL}};
const Segment SGI123457[] = {{&S100259[44], 18}, {NULL}};
const Segment SGI123458[] = {{&S100259[44], 30}, {NULL}};
const Segment SGI123459[] = {{&S100259[30], 6}, {NULL}};
const Segment SGI123460[] = {{&S100259[119], 2}, {NULL}};
const Segment SGI123461[] = {{&S100259[88], 5}, {NULL}};
const Segment SGI123462[] = {{&S100277[45], 10}, {NULL}};
const Segment SGI123463[] = {{&S100277[30], 6}, {NULL}};
const Segment SGI123464[] = {{&S100277[31], 3}, {NULL}};
const Segment SGI123465[] = {{&S100277[81], 5}, {NULL}};
const Segment SGI123466[] = {{&S100277[139], 13}, {NULL}};
const Segment SGI123467[] = {{&S100279[44], 10}, {NULL}};
const Segment SGI123468[] = {{&S100279[44], 22}, {NULL}};
const Segment SGI123469[] = {{&S100279[30], 6}, {NULL}};
const Segment SGI123470[] = {{&S100279[115], 3}, {NULL}};
const Segment SGI123471[] = {{&S100279[80], 6}, {NULL}};
const Segment SGI123472[] = {{&S100279[135], 8}, {NULL}};
const Segment SGI123473[] = {{&S100279[144], 13}, {NULL}};
const Segment SGI123474[] = {{&S100288[34], 14}, {NULL}};
const Segment SGI123475[] = {{&S100288[27], 6}, {NULL}};
const Segment SGI123476[] = {{&S100288[101], 9}, {NULL}};
const Segment SGI123477[] = {{&S100288[74], 6}, {NULL}};
const Segment SGI123478[] = {{&S100289[45], 41}, {NULL}};
const Segment SGI123479[] = {{&S100289[30], 6}, {NULL}};
const Segment SGI123480[] = {{&S100289[31], 3}, {NULL}};
const Segment SGI123481[] = {{&S100289[112], 5}, {NULL}};
const Segment SGI123482[] = {{&S100289[170], 13}, {NULL}};
const Segment SGI123483[] = {{&S100290[45], 13}, {NULL}};
const Segment SGI123484[] = {{&S100290[30], 6}, {NULL}};
const Segment SGI123485[] = {{&S100290[31], 3}, {NULL}};
const Segment SGI123486[] = {{&S100290[88], 5}, {NULL}};
const Segment SGI123487[] = {{&S100290[146], 13}, {NULL}};
const Segment SGI123488[] = {{&S100302[59], 41}, {NULL}};
const Segment SGI123489[] = {{&S100302[59], 53}, {NULL}};
const Segment SGI123490[] = {{&S100302[45], 6}, {NULL}};
const Segment SGI123491[] = {{&S100302[46], 3}, {NULL}};
const Segment SGI123492[] = {{&S100302[126], 6}, {NULL}};
const Segment SGI123493[] = {{&S100302[0], 14}, {NULL}};
const Segment SGI123494[] = {{&S100323[44], 11}, {NULL}};
const Segment SGI123495[] = {{&S100323[44], 35}, {NULL}};
const Segment SGI123496[] = {{&S100323[30], 6}, {NULL}};
const Segment SGI123497[] = {{&S100323[31], 3}, {NULL}};
const Segment SGI123498[] = {{&S100323[93], 5}, {NULL}};
const Segment SGI123499[] = {{&S100345[45], 9}, {NULL}};
const Segment SGI123500[] = {{&S100345[30], 6}, {NULL}};
const Segment SGI123501[] = {{&S100345[31], 3}, {NULL}};
const Segment SGI123502[] = {{&S100345[80], 5}, {NULL}};
const Segment SGI123503[] = {{&S100345[138], 13}, {NULL}};
const Segment SGI123504[] = {{&S100363[34], 10}, {NULL}};
const Segment SGI123505[] = {{&S100363[27], 6}, {NULL}};
const Segment SGI123506[] = {{&S100363[110], 9}, {NULL}};
const Segment SGI123507[] = {{&S100363[83], 6}, {NULL}};
const Segment SGI123508[] = {{&S100368[44], 14}, {NULL}};
const Segment SGI123509[] = {{&S100368[44], 26}, {NULL}};
const Segment SGI123510[] = {{&S100368[30], 6}, {NULL}};
const Segment SGI123511[] = {{&S100368[84], 6}, {NULL}};
const Segment SGI123512[] = {{&S100368[111], 13}, {NULL}};
const Segment SGI123513[] = {{&S100370[44], 11}, {NULL}};
const Segment SGI123514[] = {{&S100370[44], 23}, {NULL}};
const Segment SGI123515[] = {{&S100370[30], 6}, {NULL}};
const Segment SGI123516[] = {{&S100370[31], 3}, {NULL}};
const Segment SGI123517[] = {{&S100370[81], 5}, {NULL}};
const Segment SGI123518[] = {{&S100370[118], 14}, {NULL}};
const Segment SGI123519[] = {{&S100386[34], 11}, {NULL}};
const Segment SGI123520[] = {{&S100386[27], 6}, {NULL}};
const Segment SGI123521[] = {{&S100386[98], 9}, {NULL}};
const Segment SGI123522[] = {{&S100386[71], 6}, {NULL}};
const Segment SGI123523[] = {{&S100388[34], 9}, {NULL}};
const Segment SGI123524[] = {{&S100388[27], 6}, {NULL}};
const Segment SGI123525[] = {{&S100388[96], 9}, {NULL}};
const Segment SGI123526[] = {{&S100388[69], 6}, {NULL}};
const Segment SGI123527[] = {{&S100388[130], 8}, {NULL}};
const Segment SGI123528[] = {{&S100442[34], 10}, {NULL}};
const Segment SGI123529[] = {{&S100442[27], 6}, {NULL}};
const Segment SGI123530[] = {{&S100442[97], 9}, {NULL}};
const Segment SGI123531[] = {{&S100442[70], 6}, {NULL}};
const Segment SGI123532[] = {{&S100454[34], 10}, {NULL}};
const Segment SGI123533[] = {{&S100454[27], 6}, {NULL}};
const Segment SGI123534[] = {{&S100454[96], 9}, {NULL}};
const Segment SGI123535[] = {{&S100454[69], 6}, {NULL}};
const Segment SGI123536[] = {{&S100479[44], 13}, {NULL}};
const Segment SGI123537[] = {{&S100479[44], 29}, {NULL}};
const Segment SGI123538[] = {{&S100479[30], 6}, {NULL}};
const Segment SGI123539[] = {{&S100479[87], 6}, {NULL}};
const Segment SGI123540[] = {{&S100479[114], 13}, {NULL}};
const Segment SGI123541[] = {{&S100485[44], 23}, {NULL}};
const Segment SGI123542[] = {{&S100485[44], 35}, {NULL}};
const Segment SGI123543[] = {{&S100485[30], 6}, {NULL}};
const Segment SGI123544[] = {{&S100485[93], 6}, {NULL}};
const Segment SGI123545[] = {{&S100485[120], 13}, {NULL}};
const Segment SGI123546[] = {{&S100498[34], 10}, {NULL}};
const Segment SGI123547[] = {{&S100498[27], 6}, {NULL}};
const Segment SGI123548[] = {{&S100498[97], 9}, {NULL}};
const Segment SGI123549[] = {{&S100498[70], 6}, {NULL}};
const Segment SGI123550[] = {{&S100522[44], 7}, {NULL}};
const Segment SGI123551[] = {{&S100522[44], 24}, {NULL}};
const Segment SGI123552[] = {{&S100522[30], 6}, {NULL}};
const Segment SGI123553[] = {{&S100522[82], 6}, {NULL}};
const Segment SGI123554[] = {{&S100522[109], 13}, {NULL}};
const Segment SGI123555[] = {{&S100524[44], 13}, {NULL}};
const Segment SGI123556[] = {{&S100524[44], 25}, {NULL}};
const Segment SGI123557[] = {{&S100524[30], 6}, {NULL}};
const Segment SGI123558[] = {{&S100524[83], 6}, {NULL}};
const Segment SGI123559[] = {{&S100524[110], 13}, {NULL}};
const Segment SGI123560[] = {{&S100546[44], 10}, {NULL}};
const Segment SGI123561[] = {{&S100546[44], 22}, {NULL}};
const Segment SGI123562[] = {{&S100546[30], 6}, {NULL}};
const Segment SGI123563[] = {{&S100546[31], 3}, {NULL}};
const Segment SGI123564[] = {{&S100546[80], 6}, {NULL}};
const Segment SGI123565[] = {{&S11335[44], 7}, {NULL}};
const Segment SGI123566[] = {{&S11335[44], 19}, {NULL}};
const Segment SGI123567[] = {{&S11335[30], 6}, {NULL}};
const Segment SGI123568[] = {{&S11335[31], 3}, {NULL}};
const Segment SGI123569[] = {{&S11335[77], 5}, {NULL}};
const Segment SGI123570[] = {{&S11340[44], 7}, {NULL}};
const Segment SGI123571[] = {{&S11340[44], 30}, {NULL}};
const Segment SGI123572[] = {{&S11340[30], 6}, {NULL}};
const Segment SGI123573[] = {{&S11340[31], 3}, {NULL}};
const Segment SGI123574[] = {{&S11340[88], 5}, {NULL}};
const Segment SGI123575[] = {{&S11341[44], 7}, {NULL}};
const Segment SGI123576[] = {{&S11341[44], 30}, {NULL}};
const Segment SGI123577[] = {{&S11341[30], 6}, {NULL}};
const Segment SGI123578[] = {{&S11341[31], 3}, {NULL}};
const Segment SGI123579[] = {{&S11341[88], 5}, {NULL}};
const Segment SGI123580[] = {{&S11343[44], 7}, {NULL}};
const Segment SGI123581[] = {{&S11343[44], 30}, {NULL}};
const Segment SGI123582[] = {{&S11343[30], 6}, {NULL}};
const Segment SGI123583[] = {{&S11343[31], 3}, {NULL}};
const Segment SGI123584[] = {{&S11343[88], 5}, {NULL}};
const Segment SGI123585[] = {{&S100607[59], 41}, {NULL}};
const Segment SGI123586[] = {{&S100607[59], 53}, {NULL}};
const Segment SGI123587[] = {{&S100607[45], 6}, {NULL}};
const Segment SGI123588[] = {{&S100607[46], 3}, {NULL}};
const Segment SGI123589[] = {{&S100607[126], 6}, {NULL}};
const Segment SGI123590[] = {{&S100607[0], 14}, {NULL}};
const Segment SGI123591[] = {{&S100631[44], 7}, {NULL}};
const Segment SGI123592[] = {{&S100631[44], 24}, {NULL}};
const Segment SGI123593[] = {{&S100631[30], 6}, {NULL}};
const Segment SGI123594[] = {{&S100631[31], 3}, {NULL}};
const Segment SGI123595[] = {{&S100631[82], 5}, {NULL}};
const Segment SGI123596[] = {{&S100631[119], 14}, {NULL}};
const Segment SGI123597[] = {{&S100683[34], 7}, {NULL}};
const Segment SGI123598[] = {{&S100683[27], 6}, {NULL}};
const Segment SGI123599[] = {{&S100683[94], 9}, {NULL}};
const Segment SGI123600[] = {{&S100683[67], 6}, {NULL}};
const Segment SGI123601[] = {{&S100823[59], 11}, {NULL}};
const Segment SGI123602[] = {{&S100823[59], 23}, {NULL}};
const Segment SGI123603[] = {{&S100823[45], 6}, {NULL}};
const Segment SGI123604[] = {{&S100823[46], 3}, {NULL}};
const Segment SGI123605[] = {{&S100823[96], 6}, {NULL}};
const Segment SGI123606[] = {{&S100823[0], 14}, {NULL}};
const Segment SGI123607[] = {{&S100829[45], 41}, {NULL}};
const Segment SGI123608[] = {{&S100829[30], 6}, {NULL}};
const Segment SGI123609[] = {{&S100829[31], 3}, {NULL}};
const Segment SGI123610[] = {{&S100829[112], 5}, {NULL}};
const Segment SGI123611[] = {{&S100829[170], 13}, {NULL}};
const Segment SGI123612[] = {{&S100834[44], 13}, {NULL}};
const Segment SGI123613[] = {{&S100834[44], 25}, {NULL}};
const Segment SGI123614[] = {{&S100834[30], 6}, {NULL}};
const Segment SGI123615[] = {{&S100834[31], 3}, {NULL}};
const Segment SGI123616[] = {{&S100834[83], 6}, {NULL}};
const Segment SGI123617[] = {{&S100839[34], 6}, {NULL}};
const Segment SGI123618[] = {{&S100839[27], 6}, {NULL}};
const Segment SGI123619[] = {{&S100839[93], 9}, {NULL}};
const Segment SGI123620[] = {{&S100839[66], 6}, {NULL}};
const Segment SGI123621[] = {{&S10479[43], 10}, {NULL}};
const Segment SGI123622[] = {{&S10479[43], 22}, {NULL}};
const Segment SGI123623[] = {{&S10479[30], 5}, {NULL}};
const Segment SGI123624[] = {{&S100841[44], 15}, {NULL}};
const Segment SGI123625[] = {{&S10479[113], 3}, {NULL}};
const Segment SGI123626[] = {{&S100841[44], 27}, {NULL}};
const Segment SGI123627[] = {{&S100841[30], 6}, {NULL}};
const Segment SGI123628[] = {{&S10479[79], 5}, {NULL}};
const Segment SGI123629[] = {{&S100841[31], 3}, {NULL}};
const Segment SGI123630[] = {{&S100841[85], 6}, {NULL}};
const Segment SGI123631[] = {{&S100855[45], 10}, {NULL}};
const Segment SGI123632[] = {{&S100855[30], 6}, {NULL}};
const Segment SGI123633[] = {{&S100855[31], 3}, {NULL}};
const Segment SGI123634[] = {{&S100855[81], 5}, {NULL}};
const Segment SGI123635[] = {{&S100855[139], 13}, {NULL}};
const Segment SGI123636[] = {{&S100857[44], 10}, {NULL}};
const Segment SGI123637[] = {{&S100857[44], 22}, {NULL}};
const Segment SGI123638[] = {{&S100857[30], 6}, {NULL}};
const Segment SGI123639[] = {{&S100857[115], 3}, {NULL}};
const Segment SGI123640[] = {{&S100857[80], 6}, {NULL}};
const Segment SGI123641[] = {{&S100857[135], 8}, {NULL}};
const Segment SGI123642[] = {{&S100857[144], 13}, {NULL}};
const Segment SGI123643[] = {{&S100886[34], 4}, {NULL}};
const Segment SGI123644[] = {{&S100886[27], 6}, {NULL}};
const Segment SGI123645[] = {{&S100886[91], 9}, {NULL}};
const Segment SGI123646[] = {{&S100886[64], 6}, {NULL}};
const Segment SGI123647[] = {{&S100899[59], 7}, {NULL}};
const Segment SGI123648[] = {{&S100899[59], 19}, {NULL}};
const Segment SGI123649[] = {{&S100899[45], 6}, {NULL}};
const Segment SGI123650[] = {{&S100899[46], 3}, {NULL}};
const Segment SGI123651[] = {{&S100899[92], 6}, {NULL}};
const Segment SGI123652[] = {{&S100899[0], 14}, {NULL}};
const Segment SGI123653[] = {{&S100917[45], 16}, {NULL}};
const Segment SGI123654[] = {{&S100917[30], 6}, {NULL}};
const Segment SGI123655[] = {{&S100917[31], 3}, {NULL}};
const Segment SGI123656[] = {{&S100917[87], 5}, {NULL}};
const Segment SGI123657[] = {{&S100917[145], 13}, {NULL}};
const Segment SGI123658[] = {{&S100928[34], 6}, {NULL}};
const Segment SGI123659[] = {{&S100928[27], 6}, {NULL}};
const Segment SGI123660[] = {{&S100928[104], 9}, {NULL}};
const Segment SGI123661[] = {{&S100928[77], 6}, {NULL}};
const Segment SGI123662[] = {{&S100931[44], 13}, {NULL}};
const Segment SGI123663[] = {{&S100931[44], 25}, {NULL}};
const Segment SGI123664[] = {{&S100931[30], 6}, {NULL}};
const Segment SGI123665[] = {{&S100931[83], 6}, {NULL}};
const Segment SGI123666[] = {{&S100931[110], 13}, {NULL}};
const Segment SGI123667[] = {{&S100942[45], 18}, {NULL}};
const Segment SGI123668[] = {{&S100942[30], 6}, {NULL}};
const Segment SGI123669[] = {{&S100942[31], 3}, {NULL}};
const Segment SGI123670[] = {{&S100942[89], 5}, {NULL}};
const Segment SGI123671[] = {{&S100942[147], 13}, {NULL}};
const Segment SGI123672[] = {{&S101000[44], 13}, {NULL}};
const Segment SGI123673[] = {{&S101000[44], 25}, {NULL}};
const Segment SGI123674[] = {{&S101000[30], 6}, {NULL}};
const Segment SGI123675[] = {{&S101000[114], 2}, {NULL}};
const Segment SGI123676[] = {{&S101000[83], 5}, {NULL}};
const Segment SGI123677[] = {{&S101066[44], 10}, {NULL}};
const Segment SGI123678[] = {{&S101066[44], 22}, {NULL}};
const Segment SGI123679[] = {{&S101066[30], 6}, {NULL}};
const Segment SGI123680[] = {{&S101066[31], 3}, {NULL}};
const Segment SGI123681[] = {{&S101066[80], 6}, {NULL}};
const Segment SGI123682[] = {{&S101066[132], 10}, {NULL}};
const Segment SGI123683[] = {{&S101070[44], 10}, {NULL}};
const Segment SGI123684[] = {{&S101070[44], 22}, {NULL}};
const Segment SGI123685[] = {{&S101070[30], 6}, {NULL}};
const Segment SGI123686[] = {{&S101070[111], 2}, {NULL}};
const Segment SGI123687[] = {{&S101070[80], 5}, {NULL}};
const Segment SGI123688[] = {{&S101104[34], 7}, {NULL}};
const Segment SGI123689[] = {{&S101104[27], 6}, {NULL}};
const Segment SGI123690[] = {{&S101104[95], 9}, {NULL}};
const Segment SGI123691[] = {{&S101104[68], 6}, {NULL}};
const Segment SGI123692[] = {{&S101108[44], 11}, {NULL}};
const Segment SGI123693[] = {{&S101108[44], 23}, {NULL}};
const Segment SGI123694[] = {{&S101108[30], 6}, {NULL}};
const Segment SGI123695[] = {{&S101108[31], 3}, {NULL}};
const Segment SGI123696[] = {{&S101108[81], 5}, {NULL}};
const Segment SGI123697[] = {{&S101108[126], 14}, {NULL}};
const Segment SGI123698[] = {{&S101219[44], 9}, {NULL}};
const Segment SGI123699[] = {{&S101219[44], 18}, {NULL}};
const Segment SGI123700[] = {{&S101219[30], 6}, {NULL}};
const Segment SGI123701[] = {{&S101219[31], 3}, {NULL}};
const Segment SGI123702[] = {{&S101219[76], 6}, {NULL}};
const Segment SGI123703[] = {{&S101226[34], 6}, {NULL}};
const Segment SGI123704[] = {{&S101226[27], 6}, {NULL}};
const Segment SGI123705[] = {{&S101226[93], 9}, {NULL}};
const Segment SGI123706[] = {{&S101226[66], 6}, {NULL}};
const Segment SGI123707[] = {{&S101259[44], 10}, {NULL}};
const Segment SGI123708[] = {{&S101259[44], 22}, {NULL}};
const Segment SGI123709[] = {{&S101259[30], 6}, {NULL}};
const Segment SGI123710[] = {{&S101259[31], 3}, {NULL}};
const Segment SGI123711[] = {{&S101259[80], 5}, {NULL}};
const Segment SGI123712[] = {{&S101259[117], 14}, {NULL}};
const Segment SGI123713[] = {{&S101261[34], 23}, {NULL}};
const Segment SGI123714[] = {{&S101261[27], 6}, {NULL}};
const Segment SGI123715[] = {{&S101261[110], 9}, {NULL}};
const Segment SGI123716[] = {{&S101261[83], 6}, {NULL}};
const Segment SGI123717[] = {{&S101270[31], 12}, {NULL}};
const Segment SGI123718[] = {{&S101270[58], 3}, {NULL}};
const Segment SGI123719[] = {{&S101272[59], 10}, {NULL}};
const Segment SGI123720[] = {{&S101272[59], 22}, {NULL}};
const Segment SGI123721[] = {{&S101272[45], 6}, {NULL}};
const Segment SGI123722[] = {{&S101272[46], 3}, {NULL}};
const Segment SGI123723[] = {{&S101272[95], 6}, {NULL}};
const Segment SGI123724[] = {{&S101272[0], 14}, {NULL}};
const Segment SGI123725[] = {{&S101364[44], 9}, {NULL}};
const Segment SGI123726[] = {{&S101364[44], 20}, {NULL}};
const Segment SGI123727[] = {{&S101364[30], 6}, {NULL}};
const Segment SGI123728[] = {{&S101364[31], 3}, {NULL}};
const Segment SGI123729[] = {{&S101364[78], 5}, {NULL}};
const Segment SGI123730[] = {{&S101402[45], 10}, {NULL}};
const Segment SGI123731[] = {{&S101402[30], 6}, {NULL}};
const Segment SGI123732[] = {{&S101402[31], 3}, {NULL}};
const Segment SGI123733[] = {{&S101402[81], 5}, {NULL}};
const Segment SGI123734[] = {{&S101402[139], 13}, {NULL}};
const Segment SGI123735[] = {{&S101404[45], 10}, {NULL}};
const Segment SGI123736[] = {{&S101404[30], 6}, {NULL}};
const Segment SGI123737[] = {{&S101404[31], 3}, {NULL}};
const Segment SGI123738[] = {{&S101404[81], 5}, {NULL}};
const Segment SGI123739[] = {{&S101404[139], 13}, {NULL}};
const Segment SGI123740[] = {{&S101415[57], 13}, {NULL}};
const Segment SGI123741[] = {{&S101415[57], 29}, {NULL}};
const Segment SGI123742[] = {{&S101415[43], 6}, {NULL}};
const Segment SGI123743[] = {{&S101415[44], 3}, {NULL}};
const Segment SGI123744[] = {{&S101415[100], 6}, {NULL}};
const Segment SGI123745[] = {{&S101415[0], 12}, {NULL}};
const Segment SGI123746[] = {{&S17996[44], 11}, {NULL}};
const Segment SGI123747[] = {{&S17996[44], 23}, {NULL}};
const Segment SGI123748[] = {{&S17996[30], 6}, {NULL}};
const Segment SGI123749[] = {{&S17996[31], 3}, {NULL}};
const Segment SGI123750[] = {{&S17996[81], 6}, {NULL}};
const Segment SGI123751[] = {{&S18003[44], 11}, {NULL}};
const Segment SGI123752[] = {{&S18003[44], 23}, {NULL}};
const Segment SGI123753[] = {{&S18003[30], 6}, {NULL}};
const Segment SGI123754[] = {{&S18003[31], 3}, {NULL}};
const Segment SGI123755[] = {{&S18003[81], 6}, {NULL}};
const Segment SGI123756[] = {{&S18004[44], 12}, {NULL}};
const Segment SGI123757[] = {{&S18004[44], 24}, {NULL}};
const Segment SGI123758[] = {{&S18004[30], 6}, {NULL}};
const Segment SGI123759[] = {{&S18004[31], 3}, {NULL}};
const Segment SGI123760[] = {{&S18004[82], 6}, {NULL}};
const Segment SGI123761[] = {{&S18005[38], 11}, {NULL}};
const Segment SGI123762[] = {{&S18005[31], 6}, {NULL}};
const Segment SGI123763[] = {{&S18006[44], 11}, {NULL}};
const Segment SGI123764[] = {{&S18006[44], 23}, {NULL}};
const Segment SGI123765[] = {{&S18006[30], 6}, {NULL}};
const Segment SGI123766[] = {{&S18006[31], 3}, {NULL}};
const Segment SGI123767[] = {{&S18006[81], 6}, {NULL}};
const Segment SGI123768[] = {{&S18007[44], 7}, {NULL}};
const Segment SGI123769[] = {{&S18007[44], 19}, {NULL}};
const Segment SGI123770[] = {{&S18007[30], 6}, {NULL}};
const Segment SGI123771[] = {{&S18007[31], 3}, {NULL}};
const Segment SGI123772[] = {{&S18007[77], 6}, {NULL}};
const Segment SGI123773[] = {{&S18008[44], 22}, {NULL}};
const Segment SGI123774[] = {{&S18008[44], 34}, {NULL}};
const Segment SGI123775[] = {{&S18008[30], 6}, {NULL}};
const Segment SGI123776[] = {{&S18008[31], 3}, {NULL}};
const Segment SGI123777[] = {{&S18008[92], 6}, {NULL}};
const Segment SGI123778[] = {{&S18009[44], 22}, {NULL}};
const Segment SGI123779[] = {{&S18009[44], 34}, {NULL}};
const Segment SGI123780[] = {{&S18009[30], 6}, {NULL}};
const Segment SGI123781[] = {{&S18009[31], 3}, {NULL}};
const Segment SGI123782[] = {{&S18009[92], 6}, {NULL}};
const Segment SGI123783[] = {{&S18010[38], 11}, {NULL}};
const Segment SGI123784[] = {{&S18010[31], 6}, {NULL}};
const Segment SGI123785[] = {{&S18011[44], 21}, {NULL}};
const Segment SGI123786[] = {{&S18011[44], 33}, {NULL}};
const Segment SGI123787[] = {{&S18011[30], 6}, {NULL}};
const Segment SGI123788[] = {{&S18011[31], 3}, {NULL}};
const Segment SGI123789[] = {{&S18011[91], 6}, {NULL}};
const Segment SGI123790[] = {{&S18012[44], 22}, {NULL}};
const Segment SGI123791[] = {{&S18012[44], 34}, {NULL}};
const Segment SGI123792[] = {{&S18012[30], 6}, {NULL}};
const Segment SGI123793[] = {{&S18012[31], 3}, {NULL}};
const Segment SGI123794[] = {{&S18012[92], 6}, {NULL}};
const Segment SGI123795[] = {{&S18013[44], 22}, {NULL}};
const Segment SGI123796[] = {{&S18013[44], 34}, {NULL}};
const Segment SGI123797[] = {{&S18013[30], 6}, {NULL}};
const Segment SGI123798[] = {{&S18013[31], 3}, {NULL}};
const Segment SGI123799[] = {{&S18013[92], 6}, {NULL}};
const Segment SGI123800[] = {{&S18017[44], 23}, {NULL}};
const Segment SGI123801[] = {{&S18017[44], 35}, {NULL}};
const Segment SGI123802[] = {{&S18017[30], 6}, {NULL}};
const Segment SGI123803[] = {{&S18017[31], 3}, {NULL}};
const Segment SGI123804[] = {{&S18017[93], 6}, {NULL}};
const Segment SGI123805[] = {{&S18018[44], 23}, {NULL}};
const Segment SGI123806[] = {{&S18018[44], 35}, {NULL}};
const Segment SGI123807[] = {{&S18018[30], 6}, {NULL}};
const Segment SGI123808[] = {{&S18018[31], 3}, {NULL}};
const Segment SGI123809[] = {{&S18018[93], 6}, {NULL}};
const Segment SGI123810[] = {{&S18019[44], 23}, {NULL}};
const Segment SGI123811[] = {{&S18019[44], 35}, {NULL}};
const Segment SGI123812[] = {{&S18019[30], 6}, {NULL}};
const Segment SGI123813[] = {{&S18019[31], 3}, {NULL}};
const Segment SGI123814[] = {{&S18019[93], 6}, {NULL}};
const Segment SGI123815[] = {{&S18020[44], 11}, {NULL}};
const Segment SGI123816[] = {{&S18020[44], 23}, {NULL}};
const Segment SGI123817[] = {{&S18020[30], 6}, {NULL}};
const Segment SGI123818[] = {{&S18020[84], 6}, {NULL}};
const Segment SGI123819[] = {{&S18022[44], 18}, {NULL}};
const Segment SGI123820[] = {{&S18022[44], 30}, {NULL}};
const Segment SGI123821[] = {{&S18022[30], 6}, {NULL}};
const Segment SGI123822[] = {{&S18022[31], 3}, {NULL}};
const Segment SGI123823[] = {{&S18022[88], 6}, {NULL}};
const Segment SGI123824[] = {{&S18025[44], 9}, {NULL}};
const Segment SGI123825[] = {{&S18025[44], 21}, {NULL}};
const Segment SGI123826[] = {{&S18025[30], 6}, {NULL}};
const Segment SGI123827[] = {{&S18025[31], 3}, {NULL}};
const Segment SGI123828[] = {{&S18025[79], 6}, {NULL}};
const Segment SGI123829[] = {{&S18028[44], 21}, {NULL}};
const Segment SGI123830[] = {{&S18028[44], 33}, {NULL}};
const Segment SGI123831[] = {{&S18028[30], 6}, {NULL}};
const Segment SGI123832[] = {{&S18028[31], 3}, {NULL}};
const Segment SGI123833[] = {{&S18028[91], 6}, {NULL}};
const Segment SGI123834[] = {{&S18032[44], 20}, {NULL}};
const Segment SGI123835[] = {{&S18032[44], 32}, {NULL}};
const Segment SGI123836[] = {{&S18032[30], 6}, {NULL}};
const Segment SGI123837[] = {{&S18032[31], 3}, {NULL}};
const Segment SGI123838[] = {{&S18032[90], 6}, {NULL}};
const Segment SGI123839[] = {{&S18033[44], 19}, {NULL}};
const Segment SGI123840[] = {{&S18033[44], 31}, {NULL}};
const Segment SGI123841[] = {{&S18033[30], 6}, {NULL}};
const Segment SGI123842[] = {{&S18033[31], 3}, {NULL}};
const Segment SGI123843[] = {{&S18033[89], 6}, {NULL}};
const Segment SGI123844[] = {{&S18037[44], 22}, {NULL}};
const Segment SGI123845[] = {{&S18037[44], 34}, {NULL}};
const Segment SGI123846[] = {{&S18037[30], 6}, {NULL}};
const Segment SGI123847[] = {{&S18037[31], 3}, {NULL}};
const Segment SGI123848[] = {{&S18037[92], 6}, {NULL}};
const Segment SGI123849[] = {{&S18038[44], 20}, {NULL}};
const Segment SGI123850[] = {{&S18038[44], 32}, {NULL}};
const Segment SGI123851[] = {{&S18038[30], 6}, {NULL}};
const Segment SGI123852[] = {{&S18038[31], 3}, {NULL}};
const Segment SGI123853[] = {{&S18038[90], 6}, {NULL}};
const Segment SGI123854[] = {{&S18041[44], 12}, {NULL}};
const Segment SGI123855[] = {{&S18041[44], 24}, {NULL}};
const Segment SGI123856[] = {{&S18041[30], 6}, {NULL}};
const Segment SGI123857[] = {{&S18041[31], 3}, {NULL}};
const Segment SGI123858[] = {{&S18041[82], 6}, {NULL}};
const Segment SGI123859[] = {{&S18042[44], 11}, {NULL}};
const Segment SGI123860[] = {{&S18042[44], 23}, {NULL}};
const Segment SGI123861[] = {{&S18042[30], 6}, {NULL}};
const Segment SGI123862[] = {{&S18042[31], 3}, {NULL}};
const Segment SGI123863[] = {{&S18042[81], 6}, {NULL}};
const Segment SGI123864[] = {{&S101527[44], 11}, {NULL}};
const Segment SGI123865[] = {{&S101527[44], 33}, {NULL}};
const Segment SGI123866[] = {{&S101527[30], 6}, {NULL}};
const Segment SGI123867[] = {{&S101527[31], 3}, {NULL}};
const Segment SGI123868[] = {{&S101527[91], 6}, {NULL}};
const Segment SGI123869[] = {{&S101547[57], 4}, {NULL}};
const Segment SGI123870[] = {{&S101547[57], 26}, {NULL}};
const Segment SGI123871[] = {{&S101547[43], 6}, {NULL}};
const Segment SGI123872[] = {{&S101547[44], 3}, {NULL}};
const Segment SGI123873[] = {{&S101547[97], 6}, {NULL}};
const Segment SGI123874[] = {{&S101547[0], 12}, {NULL}};
const Segment SGI123875[] = {{&S101548[34], 5}, {NULL}};
const Segment SGI123876[] = {{&S101548[27], 6}, {NULL}};
const Segment SGI123877[] = {{&S101548[92], 9}, {NULL}};
const Segment SGI123878[] = {{&S101548[65], 6}, {NULL}};
const Segment SGI123879[] = {{&S12381[44], 11}, {NULL}};
const Segment SGI123880[] = {{&S12381[44], 23}, {NULL}};
const Segment SGI123881[] = {{&S12381[30], 6}, {NULL}};
const Segment SGI123882[] = {{&S12381[31], 3}, {NULL}};
const Segment SGI123883[] = {{&S12381[81], 5}, {NULL}};
const Segment SGI123884[] = {{&S101559[45], 10}, {NULL}};
const Segment SGI123885[] = {{&S101559[30], 6}, {NULL}};
const Segment SGI123886[] = {{&S101559[31], 3}, {NULL}};
const Segment SGI123887[] = {{&S101559[81], 5}, {NULL}};
const Segment SGI123888[] = {{&S101559[139], 13}, {NULL}};
const Segment SGI123889[] = {{&S101580[59], 13}, {NULL}};
const Segment SGI123890[] = {{&S101580[59], 25}, {NULL}};
const Segment SGI123891[] = {{&S101580[45], 6}, {NULL}};
const Segment SGI123892[] = {{&S101580[46], 3}, {NULL}};
const Segment SGI123893[] = {{&S101580[98], 6}, {NULL}};
const Segment SGI123894[] = {{&S101580[0], 14}, {NULL}};
const Segment SGI123895[] = {{&S101593[34], 9}, {NULL}};
const Segment SGI123896[] = {{&S101593[27], 6}, {NULL}};
const Segment SGI123897[] = {{&S101593[96], 9}, {NULL}};
const Segment SGI123898[] = {{&S101593[69], 6}, {NULL}};
const Segment SGI123899[] = {{&S101719[44], 15}, {NULL}};
const Segment SGI123900[] = {{&S101719[44], 24}, {NULL}};
const Segment SGI123901[] = {{&S101719[30], 6}, {NULL}};
const Segment SGI123902[] = {{&S101719[31], 3}, {NULL}};
const Segment SGI123903[] = {{&S101719[82], 6}, {NULL}};
const Segment SGI123904[] = {{&S101725[44], 10}, {NULL}};
const Segment SGI123905[] = {{&S101725[44], 20}, {NULL}};
const Segment SGI123906[] = {{&S101725[30], 6}, {NULL}};
const Segment SGI123907[] = {{&S101725[31], 3}, {NULL}};
const Segment SGI123908[] = {{&S101725[78], 6}, {NULL}};
const Segment SGI123909[] = {{&S18299[44], 10}, {NULL}};
const Segment SGI123910[] = {{&S18299[44], 22}, {NULL}};
const Segment SGI123911[] = {{&S18299[30], 6}, {NULL}};
const Segment SGI123912[] = {{&S18299[31], 3}, {NULL}};
const Segment SGI123913[] = {{&S18299[80], 6}, {NULL}};
const Segment SGI123914[] = {{&S18300[38], 10}, {NULL}};
const Segment SGI123915[] = {{&S18300[31], 6}, {NULL}};
const Segment SGI123916[] = {{&S18303[42], 10}, {NULL}};
const Segment SGI123917[] = {{&S18303[30], 6}, {NULL}};
const Segment SGI123918[] = {{&S18303[31], 3}, {NULL}};
const Segment SGI123919[] = {{&S18303[78], 6}, {NULL}};
const Segment SGI123920[] = {{&S18309[44], 10}, {NULL}};
const Segment SGI123921[] = {{&S18309[44], 22}, {NULL}};
const Segment SGI123922[] = {{&S18309[30], 6}, {NULL}};
const Segment SGI123923[] = {{&S101741[34], 17}, {NULL}};
const Segment SGI123924[] = {{&S18309[31], 3}, {NULL}};
const Segment SGI123925[] = {{&S101741[27], 6}, {NULL}};
const Segment SGI123926[] = {{&S18309[80], 6}, {NULL}};
const Segment SGI123927[] = {{&S101741[104], 9}, {NULL}};
const Segment SGI123928[] = {{&S101741[77], 6}, {NULL}};
const Segment SGI123929[] = {{&S18310[58], 10}, {NULL}};
const Segment SGI123930[] = {{&S18310[58], 34}, {NULL}};
const Segment SGI123931[] = {{&S18310[44], 6}, {NULL}};
const Segment SGI123932[] = {{&S101745[44], 10}, {NULL}};
const Segment SGI123933[] = {{&S101745[44], 22}, {NULL}};
const Segment SGI123934[] = {{&S101745[30], 6}, {NULL}};
const Segment SGI123935[] = {{&S18313[44], 10}, {NULL}};
const Segment SGI123936[] = {{&S18313[44], 22}, {NULL}};
const Segment SGI123937[] = {{&S101745[31], 3}, {NULL}};
const Segment SGI123938[] = {{&S18313[30], 6}, {NULL}};
const Segment SGI123939[] = {{&S101745[87], 6}, {NULL}};
const Segment SGI123940[] = {{&S101745[66], 8}, {NULL}};
const Segment SGI123941[] = {{&S18313[83], 6}, {NULL}};
const Segment SGI123942[] = {{&S18316[55], 18}, {NULL}};
const Segment SGI123943[] = {{&S18316[48], 6}, {NULL}};
const Segment SGI123944[] = {{&S18317[44], 10}, {NULL}};
const Segment SGI123945[] = {{&S18317[44], 34}, {NULL}};
const Segment SGI123946[] = {{&S18317[30], 6}, {NULL}};
const Segment SGI123947[] = {{&S101752[58], 10}, {NULL}};
const Segment SGI123948[] = {{&S101752[58], 22}, {NULL}};
const Segment SGI123949[] = {{&S101752[44], 6}, {NULL}};
const Segment SGI123950[] = {{&S101752[45], 3}, {NULL}};
const Segment SGI123951[] = {{&S101752[94], 6}, {NULL}};
const Segment SGI123952[] = {{&S101752[0], 14}, {NULL}};
const Segment SGI123953[] = {{&S18319[44], 10}, {NULL}};
const Segment SGI123954[] = {{&S18319[44], 22}, {NULL}};
const Segment SGI123955[] = {{&S18319[30], 6}, {NULL}};
const Segment SGI123956[] = {{&S18319[125], 3}, {NULL}};
const Segment SGI123957[] = {{&S18319[90], 6}, {NULL}};
const Segment SGI123958[] = {{&S18319[145], 8}, {NULL}};
const Segment SGI123959[] = {{&S101753[44], 10}, {NULL}};
const Segment SGI123960[] = {{&S101753[44], 22}, {NULL}};
const Segment SGI123961[] = {{&S101753[30], 6}, {NULL}};
const Segment SGI123962[] = {{&S101753[159], 3}, {NULL}};
const Segment SGI123963[] = {{&S101753[125], 3}, {NULL}};
const Segment SGI123964[] = {{&S101753[90], 6}, {NULL}};
const Segment SGI123965[] = {{&S101753[145], 8}, {NULL}};
const Segment SGI123966[] = {{&S18325[44], 10}, {NULL}};
const Segment SGI123967[] = {{&S18325[44], 22}, {NULL}};
const Segment SGI123968[] = {{&S18325[30], 6}, {NULL}};
const Segment SGI123969[] = {{&S18325[80], 6}, {NULL}};
const Segment SGI123970[] = {{&S101760[45], 10}, {NULL}};
const Segment SGI123971[] = {{&S101760[30], 6}, {NULL}};
const Segment SGI123972[] = {{&S101760[31], 3}, {NULL}};
const Segment SGI123973[] = {{&S101760[81], 5}, {NULL}};
const Segment SGI123974[] = {{&S101760[139], 13}, {NULL}};
const Segment SGI123975[] = {{&S18328[30], 1}, {NULL}};
const Segment SGI123976[] = {{&S18328[63], 9}, {NULL}};
const Segment SGI123977[] = {{&S18328[49], 6}, {NULL}};
const Segment SGI123978[] = {{&S18330[44], 10}, {NULL}};
const Segment SGI123979[] = {{&S18330[44], 66}, {NULL}};
const Segment SGI123980[] = {{&S18330[30], 6}, {NULL}};
const Segment SGI123981[] = {{&S18331[44], 10}, {NULL}};
const Segment SGI123982[] = {{&S18331[44], 22}, {NULL}};
const Segment SGI123983[] = {{&S18331[30], 6}, {NULL}};
const Segment SGI123984[] = {{&S18331[31], 3}, {NULL}};
const Segment SGI123985[] = {{&S18331[80], 5}, {NULL}};
const Segment SGI123986[] = {{&S18333[44], 10}, {NULL}};
const Segment SGI123987[] = {{&S18333[44], 19}, {NULL}};
const Segment SGI123988[] = {{&S18333[30], 6}, {NULL}};
const Segment SGI123989[] = {{&S18333[31], 3}, {NULL}};
const Segment SGI123990[] = {{&S18333[77], 5}, {NULL}};
const Segment SGI123991[] = {{&S18335[44], 10}, {NULL}};
const Segment SGI123992[] = {{&S18335[44], 22}, {NULL}};
const Segment SGI123993[] = {{&S18335[30], 6}, {NULL}};
const Segment SGI123994[] = {{&S101774[44], 15}, {NULL}};
const Segment SGI123995[] = {{&S18335[8], 3}, {&S18335[31], 3}, {NULL}};
const Segment SGI123996[] = {{&S101774[44], 27}, {NULL}};
const Segment SGI123997[] = {{&S101774[30], 6}, {NULL}};
const Segment SGI123998[] = {{&S18335[80], 6}, {NULL}};
const Segment SGI123999[] = {{&S101774[31], 3}, {NULL}};
const Segment SGI124000[] = {{&S101774[92], 6}, {NULL}};
const Segment SGI124001[] = {{&S101774[71], 8}, {NULL}};
const Segment SGI124002[] = {{&S18336[44], 10}, {NULL}};
const Segment SGI124003[] = {{&S18336[44], 22}, {NULL}};
const Segment SGI124004[] = {{&S18336[30], 6}, {NULL}};
const Segment SGI124005[] = {{&S18336[31], 3}, {NULL}};
const Segment SGI124006[] = {{&S18336[80], 6}, {NULL}};
const Segment SGI124007[] = {{&S18337[44], 10}, {NULL}};
const Segment SGI124008[] = {{&S18337[44], 22}, {NULL}};
const Segment SGI124009[] = {{&S18337[30], 6}, {NULL}};
const Segment SGI124010[] = {{&S18337[80], 6}, {NULL}};
const Segment SGI124011[] = {{&S101779[34], 7}, {NULL}};
const Segment SGI124012[] = {{&S101779[27], 6}, {NULL}};
const Segment SGI124013[] = {{&S101779[94], 9}, {NULL}};
const Segment SGI124014[] = {{&S101779[67], 6}, {NULL}};
const Segment SGI124015[] = {{&S18338[44], 16}, {NULL}};
const Segment SGI124016[] = {{&S18338[30], 6}, {NULL}};
const Segment SGI124017[] = {{&S18340[65], 10}, {NULL}};
const Segment SGI124018[] = {{&S18340[65], 22}, {NULL}};
const Segment SGI124019[] = {{&S18340[51], 6}, {NULL}};
const Segment SGI124020[] = {{&S18341[59], 10}, {NULL}};
const Segment SGI124021[] = {{&S18341[59], 34}, {NULL}};
const Segment SGI124022[] = {{&S18341[45], 6}, {NULL}};
const Segment SGI124023[] = {{&S18342[44], 10}, {NULL}};
const Segment SGI124024[] = {{&S18342[44], 22}, {NULL}};
const Segment SGI124025[] = {{&S18342[30], 6}, {NULL}};
const Segment SGI124026[] = {{&S18342[83], 6}, {NULL}};
const Segment SGI124027[] = {{&S18343[44], 10}, {&S18343[175], 18}, {NULL}};
const Segment SGI124028[] = {{&S18343[44], 22}, {NULL}};
const Segment SGI124029[] = {{&S18343[30], 6}, {NULL}};
const Segment SGI124030[] = {{&S18343[31], 3}, {NULL}};
const Segment SGI124031[] = {{&S18343[80], 6}, {NULL}};
const Segment SGI124032[] = {{&S18344[44], 10}, {NULL}};
const Segment SGI124033[] = {{&S18344[44], 22}, {NULL}};
const Segment SGI124034[] = {{&S18344[30], 6}, {NULL}};
const Segment SGI124035[] = {{&S18344[31], 3}, {NULL}};
const Segment SGI124036[] = {{&S18344[80], 6}, {NULL}};
const Segment SGI124037[] = {{&S18345[44], 10}, {NULL}};
const Segment SGI124038[] = {{&S18345[44], 22}, {NULL}};
const Segment SGI124039[] = {{&S18345[30], 6}, {NULL}};
const Segment SGI124040[] = {{&S18345[116], 3}, {NULL}};
const Segment SGI124041[] = {{&S18345[80], 7}, {NULL}};
const Segment SGI124042[] = {{&S18345[136], 8}, {NULL}};
const Segment SGI124043[] = {{&S18349[44], 337}, {NULL}};
const Segment SGI124044[] = {{&S18349[30], 6}, {NULL}};
const Segment SGI124045[] = {{&S18350[44], 10}, {NULL}};
const Segment SGI124046[] = {{&S18350[44], 22}, {NULL}};
const Segment SGI124047[] = {{&S18350[30], 6}, {NULL}};
const Segment SGI124048[] = {{&S18350[80], 4}, {NULL}};
const Segment SGI124049[] = {{&S101852[44], 11}, {NULL}};
const Segment SGI124050[] = {{&S101852[44], 23}, {NULL}};
const Segment SGI124051[] = {{&S101852[30], 6}, {NULL}};
const Segment SGI124052[] = {{&S101852[31], 3}, {NULL}};
const Segment SGI124053[] = {{&S101852[81], 6}, {NULL}};
const Segment SGI124054[] = {{&S101856[44], 12}, {NULL}};
const Segment SGI124055[] = {{&S101856[44], 24}, {NULL}};
const Segment SGI124056[] = {{&S101856[30], 6}, {NULL}};
const Segment SGI124057[] = {{&S101856[113], 2}, {NULL}};
const Segment SGI124058[] = {{&S101856[82], 5}, {NULL}};
const Segment SGI124059[] = {{&S101865[45], 11}, {NULL}};
const Segment SGI124060[] = {{&S101865[30], 6}, {NULL}};
const Segment SGI124061[] = {{&S101865[31], 3}, {NULL}};
const Segment SGI124062[] = {{&S101865[82], 5}, {NULL}};
const Segment SGI124063[] = {{&S101865[140], 13}, {NULL}};
const Segment SGI124064[] = {{&S101889[45], 11}, {NULL}};
const Segment SGI124065[] = {{&S101889[30], 6}, {NULL}};
const Segment SGI124066[] = {{&S101889[31], 3}, {NULL}};
const Segment SGI124067[] = {{&S101889[82], 5}, {NULL}};
const Segment SGI124068[] = {{&S101889[140], 13}, {NULL}};
const Segment SGI124069[] = {{&S101897[45], 10}, {NULL}};
const Segment SGI124070[] = {{&S101897[30], 6}, {NULL}};
const Segment SGI124071[] = {{&S101897[31], 3}, {NULL}};
const Segment SGI124072[] = {{&S101897[81], 5}, {NULL}};
const Segment SGI124073[] = {{&S101897[139], 13}, {NULL}};
const Segment SGI124074[] = {{&S102072[59], 41}, {NULL}};
const Segment SGI124075[] = {{&S102072[59], 53}, {NULL}};
const Segment SGI124076[] = {{&S102072[45], 6}, {NULL}};
const Segment SGI124077[] = {{&S102072[46], 3}, {NULL}};
const Segment SGI124078[] = {{&S102072[126], 6}, {NULL}};
const Segment SGI124079[] = {{&S102072[0], 14}, {NULL}};
const Segment SGI124080[] = {{&S10456[44], 10}, {NULL}};
const Segment SGI124081[] = {{&S10456[44], 22}, {NULL}};
const Segment SGI124082[] = {{&S10456[30], 6}, {NULL}};
const Segment SGI124083[] = {{&S10456[115], 3}, {NULL}};
const Segment SGI124084[] = {{&S10456[80], 6}, {NULL}};
const Segment SGI124085[] = {{&S10457[38], 10}, {NULL}};
const Segment SGI124086[] = {{&S10457[31], 6}, {NULL}};
const Segment SGI124087[] = {{&S10458[44], 10}, {NULL}};
const Segment SGI124088[] = {{&S10458[44], 22}, {NULL}};
const Segment SGI124089[] = {{&S10458[30], 6}, {NULL}};
const Segment SGI124090[] = {{&S10458[115], 3}, {NULL}};
const Segment SGI124091[] = {{&S10458[80], 6}, {NULL}};
const Segment SGI124092[] = {{&S10459[44], 10}, {NULL}};
const Segment SGI124093[] = {{&S10459[44], 22}, {NULL}};
const Segment SGI124094[] = {{&S10459[30], 6}, {NULL}};
const Segment SGI124095[] = {{&S10459[115], 3}, {NULL}};
const Segment SGI124096[] = {{&S10459[80], 6}, {NULL}};
const Segment SGI124097[] = {{&S10460[44], 10}, {NULL}};
const Segment SGI124098[] = {{&S10460[44], 22}, {NULL}};
const Segment SGI124099[] = {{&S10460[30], 6}, {NULL}};
const Segment SGI124100[] = {{&S10460[115], 3}, {NULL}};
const Segment SGI124101[] = {{&S10460[80], 6}, {NULL}};
const Segment SGI124102[] = {{&S10461[44], 11}, {NULL}};
const Segment SGI124103[] = {{&S10461[44], 23}, {NULL}};
const Segment SGI124104[] = {{&S10461[30], 6}, {NULL}};
const Segment SGI124105[] = {{&S10461[116], 3}, {NULL}};
const Segment SGI124106[] = {{&S10461[81], 6}, {NULL}};
const Segment SGI124107[] = {{&S10466[44], 11}, {NULL}};
const Segment SGI124108[] = {{&S10466[44], 23}, {NULL}};
const Segment SGI124109[] = {{&S10466[30], 6}, {NULL}};
const Segment SGI124110[] = {{&S10466[116], 3}, {NULL}};
const Segment SGI124111[] = {{&S10466[81], 6}, {NULL}};
const Segment SGI124112[] = {{&S10467[44], 10}, {NULL}};
const Segment SGI124113[] = {{&S10467[44], 19}, {NULL}};
const Segment SGI124114[] = {{&S10467[30], 6}, {NULL}};
const Segment SGI124115[] = {{&S10467[112], 3}, {NULL}};
const Segment SGI124116[] = {{&S10467[77], 6}, {NULL}};
const Segment SGI124117[] = {{&S10468[44], 10}, {NULL}};
const Segment SGI124118[] = {{&S10468[44], 21}, {NULL}};
const Segment SGI124119[] = {{&S10468[30], 6}, {NULL}};
const Segment SGI124120[] = {{&S10468[114], 3}, {NULL}};
const Segment SGI124121[] = {{&S10468[79], 6}, {NULL}};
const Segment SGI124122[] = {{&S10469[44], 10}, {NULL}};
const Segment SGI124123[] = {{&S10469[44], 22}, {NULL}};
const Segment SGI124124[] = {{&S10469[30], 6}, {NULL}};
const Segment SGI124125[] = {{&S10469[115], 3}, {NULL}};
const Segment SGI124126[] = {{&S10469[80], 6}, {NULL}};
const Segment SGI124127[] = {{&S10480[44], 10}, {NULL}};
const Segment SGI124128[] = {{&S10480[44], 22}, {NULL}};
const Segment SGI124129[] = {{&S10480[30], 6}, {NULL}};
const Segment SGI124130[] = {{&S10480[115], 3}, {NULL}};
const Segment SGI124131[] = {{&S10480[80], 6}, {NULL}};
const Segment SGI124132[] = {{&S10492[44], 11}, {NULL}};
const Segment SGI124133[] = {{&S10492[44], 23}, {NULL}};
const Segment SGI124134[] = {{&S10492[30], 6}, {NULL}};
const Segment SGI124135[] = {{&S10492[116], 3}, {NULL}};
const Segment SGI124136[] = {{&S10492[81], 6}, {NULL}};
const Segment SGI124137[] = {{&S10493[44], 10}, {NULL}};
const Segment SGI124138[] = {{&S10493[44], 33}, {NULL}};
const Segment SGI124139[] = {{&S10493[30], 6}, {NULL}};
const Segment SGI124140[] = {{&S10493[125], 3}, {NULL}};
const Segment SGI124141[] = {{&S10493[91], 5}, {NULL}};
const Segment SGI124142[] = {{&S10494[44], 7}, {NULL}};
const Segment SGI124143[] = {{&S10494[44], 28}, {NULL}};
const Segment SGI124144[] = {{&S10494[30], 6}, {NULL}};
const Segment SGI124145[] = {{&S10494[120], 3}, {NULL}};
const Segment SGI124146[] = {{&S10494[86], 5}, {NULL}};
const Segment SGI124147[] = {{&S10503[44], 10}, {NULL}};
const Segment SGI124148[] = {{&S10503[44], 22}, {NULL}};
const Segment SGI124149[] = {{&S10503[30], 6}, {NULL}};
const Segment SGI124150[] = {{&S10503[83], 6}, {NULL}};
const Segment SGI124151[] = {{&S102108[44], 10}, {NULL}};
const Segment SGI124152[] = {{&S102108[44], 22}, {NULL}};
const Segment SGI124153[] = {{&S102108[30], 6}, {NULL}};
const Segment SGI124154[] = {{&S102108[31], 3}, {NULL}};
const Segment SGI124155[] = {{&S102108[80], 6}, {NULL}};
const Segment SGI124156[] = {{&S102108[132], 10}, {NULL}};
const Segment SGI124157[] = {{&S102113[59], 10}, {NULL}};
const Segment SGI124158[] = {{&S102113[59], 22}, {NULL}};
const Segment SGI124159[] = {{&S102113[45], 6}, {NULL}};
const Segment SGI124160[] = {{&S102113[46], 3}, {NULL}};
const Segment SGI124161[] = {{&S102113[95], 6}, {NULL}};
const Segment SGI124162[] = {{&S102113[0], 14}, {NULL}};
const Segment SGI124163[] = {{&S102129[31], 19}, {NULL}};
const Segment SGI124164[] = {{&S102129[65], 3}, {NULL}};
const Segment SGI124165[] = {{&S18933[44], 18}, {NULL}};
const Segment SGI124166[] = {{&S18933[44], 30}, {NULL}};
const Segment SGI124167[] = {{&S18933[30], 6}, {NULL}};
const Segment SGI124168[] = {{&S18933[31], 3}, {NULL}};
const Segment SGI124169[] = {{&S18933[88], 6}, {NULL}};
const Segment SGI124170[] = {{&S18934[44], 10}, {NULL}};
const Segment SGI124171[] = {{&S18934[44], 22}, {NULL}};
const Segment SGI124172[] = {{&S18934[30], 6}, {NULL}};
const Segment SGI124173[] = {{&S18934[31], 3}, {NULL}};
const Segment SGI124174[] = {{&S18934[80], 6}, {NULL}};
const Segment SGI124175[] = {{&S18935[44], 11}, {NULL}};
const Segment SGI124176[] = {{&S18935[44], 23}, {NULL}};
const Segment SGI124177[] = {{&S18935[30], 6}, {NULL}};
const Segment SGI124178[] = {{&S18935[31], 3}, {NULL}};
const Segment SGI124179[] = {{&S18935[81], 6}, {NULL}};
const Segment SGI124180[] = {{&S18936[44], 11}, {NULL}};
const Segment SGI124181[] = {{&S18936[44], 23}, {NULL}};
const Segment SGI124182[] = {{&S18936[30], 6}, {NULL}};
const Segment SGI124183[] = {{&S18936[31], 3}, {NULL}};
const Segment SGI124184[] = {{&S18936[81], 6}, {NULL}};
const Segment SGI124185[] = {{&S18937[44], 11}, {NULL}};
const Segment SGI124186[] = {{&S18937[44], 23}, {NULL}};
const Segment SGI124187[] = {{&S18937[30], 6}, {NULL}};
const Segment SGI124188[] = {{&S18937[31], 3}, {NULL}};
const Segment SGI124189[] = {{&S18937[81], 6}, {NULL}};
const Segment SGI124190[] = {{&S18938[44], 11}, {NULL}};
const Segment SGI124191[] = {{&S18938[44], 23}, {NULL}};
const Segment SGI124192[] = {{&S18938[30], 6}, {NULL}};
const Segment SGI124193[] = {{&S18938[31], 3}, {NULL}};
const Segment SGI124194[] = {{&S18938[81], 6}, {NULL}};
const Segment SGI124195[] = {{&S18939[38], 10}, {NULL}};
const Segment SGI124196[] = {{&S18939[31], 6}, {NULL}};
const Segment SGI124197[] = {{&S18940[44], 11}, {NULL}};
const Segment SGI124198[] = {{&S18940[44], 23}, {NULL}};
const Segment SGI124199[] = {{&S18940[30], 6}, {NULL}};
const Segment SGI124200[] = {{&S18940[31], 3}, {NULL}};
const Segment SGI124201[] = {{&S18940[81], 6}, {NULL}};
const Segment SGI124202[] = {{&S18941[38], 18}, {NULL}};
const Segment SGI124203[] = {{&S18941[31], 6}, {NULL}};
const Segment SGI124204[] = {{&S18942[44], 18}, {NULL}};
const Segment SGI124205[] = {{&S18942[44], 30}, {NULL}};
const Segment SGI124206[] = {{&S18942[30], 6}, {NULL}};
const Segment SGI124207[] = {{&S18942[31], 3}, {NULL}};
const Segment SGI124208[] = {{&S18942[88], 6}, {NULL}};
const Segment SGI124209[] = {{&S18943[38], 11}, {NULL}};
const Segment SGI124210[] = {{&S18943[31], 6}, {NULL}};
const Segment SGI124211[] = {{&S18944[38], 11}, {NULL}};
const Segment SGI124212[] = {{&S18944[31], 6}, {NULL}};
const Segment SGI124213[] = {{&S18945[38], 11}, {NULL}};
const Segment SGI124214[] = {{&S18945[31], 6}, {NULL}};
const Segment SGI124215[] = {{&S18946[44], 11}, {NULL}};
const Segment SGI124216[] = {{&S18946[44], 23}, {NULL}};
const Segment SGI124217[] = {{&S18946[30], 6}, {NULL}};
const Segment SGI124218[] = {{&S18946[81], 6}, {NULL}};
const Segment SGI124219[] = {{&S18947[38], 11}, {NULL}};
const Segment SGI124220[] = {{&S18947[31], 6}, {NULL}};
const Segment SGI124221[] = {{&S18948[42], 10}, {NULL}};
const Segment SGI124222[] = {{&S18948[30], 6}, {NULL}};
const Segment SGI124223[] = {{&S18948[31], 3}, {NULL}};
const Segment SGI124224[] = {{&S18948[78], 6}, {NULL}};
const Segment SGI124225[] = {{&S18949[44], 13}, {NULL}};
const Segment SGI124226[] = {{&S18949[44], 25}, {NULL}};
const Segment SGI124227[] = {{&S18949[30], 6}, {NULL}};
const Segment SGI124228[] = {{&S18949[31], 3}, {NULL}};
const Segment SGI124229[] = {{&S18949[83], 6}, {NULL}};
const Segment SGI124230[] = {{&S18950[42], 11}, {NULL}};
const Segment SGI124231[] = {{&S18950[30], 6}, {NULL}};
const Segment SGI124232[] = {{&S18950[31], 3}, {NULL}};
const Segment SGI124233[] = {{&S18950[79], 6}, {NULL}};
const Segment SGI124234[] = {{&S18951[38], 11}, {NULL}};
const Segment SGI124235[] = {{&S18951[31], 6}, {NULL}};
const Segment SGI124236[] = {{&S18952[44], 18}, {NULL}};
const Segment SGI124237[] = {{&S18952[44], 30}, {NULL}};
const Segment SGI124238[] = {{&S18952[30], 6}, {NULL}};
const Segment SGI124239[] = {{&S18952[91], 6}, {NULL}};
const Segment SGI124240[] = {{&S18953[44], 10}, {NULL}};
const Segment SGI124241[] = {{&S18953[44], 22}, {NULL}};
const Segment SGI124242[] = {{&S18953[30], 6}, {NULL}};
const Segment SGI124243[] = {{&S18953[80], 6}, {NULL}};
const Segment SGI124244[] = {{&S18954[42], 18}, {NULL}};
const Segment SGI124245[] = {{&S18954[30], 6}, {NULL}};
const Segment SGI124246[] = {{&S18954[31], 3}, {NULL}};
const Segment SGI124247[] = {{&S18954[86], 6}, {NULL}};
const Segment SGI124248[] = {{&S18955[44], 11}, {NULL}};
const Segment SGI124249[] = {{&S18955[44], 23}, {NULL}};
const Segment SGI124250[] = {{&S18955[30], 6}, {NULL}};
const Segment SGI124251[] = {{&S18955[84], 6}, {NULL}};
const Segment SGI124252[] = {{&S18956[44], 18}, {NULL}};
const Segment SGI124253[] = {{&S18956[44], 30}, {NULL}};
const Segment SGI124254[] = {{&S18956[30], 6}, {NULL}};
const Segment SGI124255[] = {{&S18956[88], 6}, {NULL}};
const Segment SGI124256[] = {{&S18957[44], 14}, {NULL}};
const Segment SGI124257[] = {{&S18957[44], 26}, {NULL}};
const Segment SGI124258[] = {{&S18957[30], 6}, {NULL}};
const Segment SGI124259[] = {{&S18957[31], 3}, {NULL}};
const Segment SGI124260[] = {{&S18957[84], 6}, {NULL}};
const Segment SGI124261[] = {{&S18958[44], 11}, {NULL}};
const Segment SGI124262[] = {{&S18958[44], 23}, {NULL}};
const Segment SGI124263[] = {{&S18958[30], 6}, {NULL}};
const Segment SGI124264[] = {{&S18958[31], 3}, {NULL}};
const Segment SGI124265[] = {{&S18958[81], 6}, {NULL}};
const Segment SGI124266[] = {{&S18959[38], 11}, {NULL}};
const Segment SGI124267[] = {{&S18959[31], 6}, {NULL}};
const Segment SGI124268[] = {{&S18960[44], 11}, {NULL}};
const Segment SGI124269[] = {{&S18960[44], 23}, {NULL}};
const Segment SGI124270[] = {{&S18960[30], 6}, {NULL}};
const Segment SGI124271[] = {{&S18960[84], 6}, {NULL}};
const Segment SGI124272[] = {{&S18961[44], 11}, {NULL}};
const Segment SGI124273[] = {{&S18961[44], 23}, {NULL}};
const Segment SGI124274[] = {{&S18961[30], 6}, {NULL}};
const Segment SGI124275[] = {{&S18961[84], 6}, {NULL}};
const Segment SGI124276[] = {{&S18963[44], 10}, {NULL}};
const Segment SGI124277[] = {{&S18963[44], 22}, {NULL}};
const Segment SGI124278[] = {{&S18963[30], 6}, {NULL}};
const Segment SGI124279[] = {{&S18963[31], 3}, {NULL}};
const Segment SGI124280[] = {{&S18963[80], 6}, {NULL}};
const Segment SGI124281[] = {{&S18988[44], 11}, {NULL}};
const Segment SGI124282[] = {{&S18988[44], 23}, {NULL}};
const Segment SGI124283[] = {{&S18988[30], 6}, {NULL}};
const Segment SGI124284[] = {{&S18988[31], 3}, {NULL}};
const Segment SGI124285[] = {{&S18988[81], 6}, {NULL}};
const Segment SGI124286[] = {{&S18993[44], 11}, {NULL}};
const Segment SGI124287[] = {{&S18993[44], 23}, {NULL}};
const Segment SGI124288[] = {{&S18993[30], 6}, {NULL}};
const Segment SGI124289[] = {{&S18993[31], 3}, {NULL}};
const Segment SGI124290[] = {{&S18993[81], 6}, {NULL}};
const Segment SGI124291[] = {{&S18995[44], 11}, {NULL}};
const Segment SGI124292[] = {{&S18995[44], 23}, {NULL}};
const Segment SGI124293[] = {{&S18995[30], 6}, {NULL}};
const Segment SGI124294[] = {{&S18995[31], 3}, {NULL}};
const Segment SGI124295[] = {{&S18995[81], 6}, {NULL}};
const Segment SGI124296[] = {{&S18999[44], 11}, {NULL}};
const Segment SGI124297[] = {{&S18999[44], 23}, {NULL}};
const Segment SGI124298[] = {{&S18999[30], 6}, {NULL}};
const Segment SGI124299[] = {{&S18999[31], 3}, {NULL}};
const Segment SGI124300[] = {{&S18999[81], 6}, {NULL}};
const Segment SGI124301[] = {{&S102239[44], 6}, {NULL}};
const Segment SGI124302[] = {{&S102239[44], 18}, {NULL}};
const Segment SGI124303[] = {{&S102239[30], 6}, {NULL}};
const Segment SGI124304[] = {{&S102239[107], 2}, {NULL}};
const Segment SGI124305[] = {{&S102239[76], 5}, {NULL}};
const Segment SGI124306[] = {{&S102244[44], 15}, {NULL}};
const Segment SGI124307[] = {{&S102244[44], 27}, {NULL}};
const Segment SGI124308[] = {{&S102244[30], 6}, {NULL}};
const Segment SGI124309[] = {{&S102244[116], 2}, {NULL}};
const Segment SGI124310[] = {{&S102244[85], 5}, {NULL}};
const Segment SGI124311[] = {{&S102308[34], 10}, {NULL}};
const Segment SGI124312[] = {{&S102308[27], 6}, {NULL}};
const Segment SGI124313[] = {{&S102308[97], 9}, {NULL}};
const Segment SGI124314[] = {{&S102308[70], 6}, {NULL}};
const Segment SGI124315[] = {{&S102308[132], 8}, {NULL}};
const Segment SGI124316[] = {{&S19207[44], 10}, {NULL}};
const Segment SGI124317[] = {{&S19207[44], 22}, {NULL}};
const Segment SGI124318[] = {{&S19207[30], 6}, {NULL}};
const Segment SGI124319[] = {{&S19207[31], 3}, {NULL}};
const Segment SGI124320[] = {{&S19207[80], 6}, {NULL}};
const Segment SGI124321[] = {{&S19209[38], 10}, {NULL}};
const Segment SGI124322[] = {{&S19209[31], 6}, {NULL}};
const Segment SGI124323[] = {{&S102449[44], 10}, {NULL}};
const Segment SGI124324[] = {{&S102449[44], 22}, {NULL}};
const Segment SGI124325[] = {{&S102449[30], 6}, {NULL}};
const Segment SGI124326[] = {{&S102449[111], 2}, {NULL}};
const Segment SGI124327[] = {{&S102449[80], 5}, {NULL}};
const Segment SGI124328[] = {{&S19211[42], 10}, {NULL}};
const Segment SGI124329[] = {{&S19211[30], 6}, {NULL}};
const Segment SGI124330[] = {{&S19211[31], 3}, {NULL}};
const Segment SGI124331[] = {{&S19211[78], 6}, {NULL}};
const Segment SGI124332[] = {{&S102450[44], 10}, {NULL}};
const Segment SGI124333[] = {{&S102450[44], 22}, {NULL}};
const Segment SGI124334[] = {{&S102450[30], 6}, {NULL}};
const Segment SGI124335[] = {{&S102450[111], 2}, {NULL}};
const Segment SGI124336[] = {{&S102450[80], 5}, {NULL}};
const Segment SGI124337[] = {{&S19212[44], 30}, {NULL}};
const Segment SGI124338[] = {{&S19212[44], 42}, {NULL}};
const Segment SGI124339[] = {{&S19212[30], 6}, {NULL}};
const Segment SGI124340[] = {{&S19212[31], 3}, {NULL}};
const Segment SGI124341[] = {{&S19212[100], 6}, {NULL}};
const Segment SGI124342[] = {{&S19214[44], 11}, {NULL}};
const Segment SGI124343[] = {{&S19214[44], 23}, {NULL}};
const Segment SGI124344[] = {{&S19214[30], 6}, {NULL}};
const Segment SGI124345[] = {{&S19214[31], 3}, {NULL}};
const Segment SGI124346[] = {{&S19214[81], 6}, {NULL}};
const Segment SGI124347[] = {{&S19215[44], 10}, {NULL}};
const Segment SGI124348[] = {{&S19215[44], 22}, {NULL}};
const Segment SGI124349[] = {{&S19215[30], 6}, {NULL}};
const Segment SGI124350[] = {{&S19215[83], 6}, {NULL}};
const Segment SGI124351[] = {{&S19216[44], 29}, {NULL}};
const Segment SGI124352[] = {{&S19216[44], 41}, {NULL}};
const Segment SGI124353[] = {{&S19216[30], 6}, {NULL}};
const Segment SGI124354[] = {{&S19216[31], 3}, {NULL}};
const Segment SGI124355[] = {{&S19216[99], 6}, {NULL}};
const Segment SGI124356[] = {{&S19217[44], 10}, {NULL}};
const Segment SGI124357[] = {{&S19217[44], 22}, {NULL}};
const Segment SGI124358[] = {{&S19217[30], 6}, {NULL}};
const Segment SGI124359[] = {{&S19217[31], 3}, {NULL}};
const Segment SGI124360[] = {{&S19217[80], 6}, {NULL}};
const Segment SGI124361[] = {{&S19224[44], 10}, {NULL}};
const Segment SGI124362[] = {{&S19224[44], 22}, {NULL}};
const Segment SGI124363[] = {{&S19224[30], 6}, {NULL}};
const Segment SGI124364[] = {{&S19224[83], 4}, {NULL}};
const Segment SGI124365[] = {{&S19224[83], 3}, {NULL}};
const Segment SGI124366[] = {{&S19226[44], 10}, {NULL}};
const Segment SGI124367[] = {{&S19226[44], 22}, {NULL}};
const Segment SGI124368[] = {{&S19226[30], 6}, {NULL}};
const Segment SGI124369[] = {{&S19226[31], 3}, {NULL}};
const Segment SGI124370[] = {{&S19226[80], 6}, {NULL}};
const Segment SGI124371[] = {{&S19229[44], 10}, {NULL}};
const Segment SGI124372[] = {{&S19229[44], 34}, {NULL}};
const Segment SGI124373[] = {{&S19229[30], 6}, {NULL}};
const Segment SGI124374[] = {{&S19230[65], 10}, {NULL}};
const Segment SGI124375[] = {{&S19230[65], 22}, {NULL}};
const Segment SGI124376[] = {{&S19230[51], 6}, {NULL}};
const Segment SGI124377[] = {{&S19231[44], 10}, {NULL}};
const Segment SGI124378[] = {{&S19231[44], 22}, {NULL}};
const Segment SGI124379[] = {{&S19231[30], 6}, {NULL}};
const Segment SGI124380[] = {{&S19231[124], 3}, {NULL}};
const Segment SGI124381[] = {{&S19231[89], 6}, {NULL}};
const Segment SGI124382[] = {{&S19231[144], 8}, {NULL}};
const Segment SGI124383[] = {{&S19232[44], 10}, {&S19232[44], 10}, {NULL}};
const Segment SGI124384[] = {{&S19232[44], 22}, {&S19232[44], 22}, {NULL}};
const Segment SGI124385[] = {{&S19232[30], 6}, {NULL}};
const Segment SGI124386[] = {{&S19232[31], 3}, {NULL}};
const Segment SGI124387[] = {{&S19232[110], 6}, {NULL}};
const Segment SGI124388[] = {{&S19233[44], 10}, {NULL}};
const Segment SGI124389[] = {{&S19233[44], 22}, {NULL}};
const Segment SGI124390[] = {{&S19233[30], 6}, {NULL}};
const Segment SGI124391[] = {{&S19233[80], 6}, {NULL}};
const Segment SGI124392[] = {{&S19234[44], 10}, {&S19234[175], 18}, {NULL}};
const Segment SGI124393[] = {{&S19234[44], 22}, {NULL}};
const Segment SGI124394[] = {{&S19234[30], 6}, {NULL}};
const Segment SGI124395[] = {{&S19234[31], 3}, {NULL}};
const Segment SGI124396[] = {{&S19234[80], 6}, {NULL}};
const Segment SGI124397[] = {{&S19235[44], 22}, {NULL}};
const Segment SGI124398[] = {{&S19235[30], 6}, {NULL}};
const Segment SGI124399[] = {{&S19236[44], 10}, {NULL}};
const Segment SGI124400[] = {{&S19236[44], 34}, {NULL}};
const Segment SGI124401[] = {{&S19236[30], 6}, {NULL}};
const Segment SGI124402[] = {{&S19238[44], 10}, {NULL}};
const Segment SGI124403[] = {{&S19238[44], 21}, {NULL}};
const Segment SGI124404[] = {{&S19238[30], 6}, {NULL}};
const Segment SGI124405[] = {{&S19238[31], 3}, {NULL}};
const Segment SGI124406[] = {{&S19238[79], 6}, {NULL}};
const Segment SGI124407[] = {{&S19239[44], 10}, {NULL}};
const Segment SGI124408[] = {{&S19239[44], 34}, {NULL}};
const Segment SGI124409[] = {{&S19239[30], 6}, {NULL}};
const Segment SGI124410[] = {{&S19240[44], 10}, {NULL}};
const Segment SGI124411[] = {{&S19240[44], 34}, {NULL}};
const Segment SGI124412[] = {{&S19240[30], 6}, {NULL}};
const Segment SGI124413[] = {{&S102501[59], 10}, {NULL}};
const Segment SGI124414[] = {{&S102501[59], 22}, {NULL}};
const Segment SGI124415[] = {{&S102501[45], 6}, {NULL}};
const Segment SGI124416[] = {{&S102501[46], 3}, {NULL}};
const Segment SGI124417[] = {{&S102501[95], 6}, {NULL}};
const Segment SGI124418[] = {{&S102501[0], 14}, {NULL}};
const Segment SGI124419[] = {{&S19241[44], 10}, {NULL}};
const Segment SGI124420[] = {{&S19241[44], 22}, {&S19241[227], 5}, {NULL}};
const Segment SGI124421[] = {{&S19241[30], 6}, {NULL}};
const Segment SGI124422[] = {{&S19241[31], 3}, {NULL}};
const Segment SGI124423[] = {{&S19241[80], 6}, {NULL}};
const Segment SGI124424[] = {{&S19242[44], 10}, {NULL}};
const Segment SGI124425[] = {{&S19242[44], 19}, {NULL}};
const Segment SGI124426[] = {{&S19242[30], 6}, {NULL}};
const Segment SGI124427[] = {{&S19242[31], 3}, {NULL}};
const Segment SGI124428[] = {{&S19242[77], 5}, {NULL}};
const Segment SGI124429[] = {{&S19243[44], 29}, {NULL}};
const Segment SGI124430[] = {{&S19243[44], 41}, {NULL}};
const Segment SGI124431[] = {{&S19243[30], 6}, {NULL}};
const Segment SGI124432[] = {{&S19243[31], 3}, {NULL}};
const Segment SGI124433[] = {{&S19243[99], 6}, {NULL}};
const Segment SGI124434[] = {{&S19244[44], 33}, {NULL}};
const Segment SGI124435[] = {{&S19244[44], 45}, {NULL}};
const Segment SGI124436[] = {{&S19244[30], 6}, {NULL}};
const Segment SGI124437[] = {{&S19244[31], 3}, {NULL}};
const Segment SGI124438[] = {{&S19244[103], 6}, {NULL}};
const Segment SGI124439[] = {{&S102510[44], 10}, {NULL}};
const Segment SGI124440[] = {{&S102510[44], 22}, {NULL}};
const Segment SGI124441[] = {{&S102510[30], 6}, {NULL}};
const Segment SGI124442[] = {{&S19245[44], 33}, {NULL}};
const Segment SGI124443[] = {{&S19245[44], 45}, {NULL}};
const Segment SGI124444[] = {{&S102510[31], 3}, {NULL}};
const Segment SGI124445[] = {{&S19245[30], 6}, {NULL}};
const Segment SGI124446[] = {{&S102510[80], 6}, {NULL}};
const Segment SGI124447[] = {{&S102510[133], 11}, {NULL}};
const Segment SGI124448[] = {{&S19245[31], 3}, {NULL}};
const Segment SGI124449[] = {{&S19245[103], 6}, {NULL}};
const Segment SGI124450[] = {{&S19246[55], 18}, {NULL}};
const Segment SGI124451[] = {{&S19246[48], 6}, {NULL}};
const Segment SGI124452[] = {{&S102514[45], 10}, {NULL}};
const Segment SGI124453[] = {{&S19248[42], 10}, {NULL}};
const Segment SGI124454[] = {{&S102514[30], 6}, {NULL}};
const Segment SGI124455[] = {{&S19248[30], 6}, {NULL}};
const Segment SGI124456[] = {{&S102514[31], 3}, {NULL}};
const Segment SGI124457[] = {{&S19248[78], 6}, {NULL}};
const Segment SGI124458[] = {{&S102514[81], 5}, {NULL}};
const Segment SGI124459[] = {{&S102514[139], 13}, {NULL}};
const Segment SGI124460[] = {{&S19249[38], 11}, {NULL}};
const Segment SGI124461[] = {{&S19249[31], 6}, {NULL}};
const Segment SGI124462[] = {{&S19250[44], 10}, {NULL}};
const Segment SGI124463[] = {{&S19250[44], 34}, {NULL}};
const Segment SGI124464[] = {{&S19250[30], 6}, {NULL}};
const Segment SGI124465[] = {{&S19251[44], 10}, {NULL}};
const Segment SGI124466[] = {{&S19251[44], 34}, {NULL}};
const Segment SGI124467[] = {{&S19251[30], 6}, {NULL}};
const Segment SGI124468[] = {{&S19255[35], 26}, {NULL}};
const Segment SGI124469[] = {{&S19258[45], 10}, {NULL}};
const Segment SGI124470[] = {{&S19258[45], 34}, {NULL}};
const Segment SGI124471[] = {{&S19258[31], 6}, {NULL}};
const Segment SGI124472[] = {{&S19261[44], 11}, {NULL}};
const Segment SGI124473[] = {{&S19261[44], 23}, {NULL}};
const Segment SGI124474[] = {{&S19261[30], 6}, {NULL}};
const Segment SGI124475[] = {{&S19261[84], 6}, {NULL}};
const Segment SGI124476[] = {{&S19262[44], 11}, {NULL}};
const Segment SGI124477[] = {{&S19262[44], 20}, {NULL}};
const Segment SGI124478[] = {{&S19262[30], 6}, {NULL}};
const Segment SGI124479[] = {{&S19262[31], 3}, {NULL}};
const Segment SGI124480[] = {{&S19262[78], 5}, {NULL}};
const Segment SGI124481[] = {{&S19263[44], 10}, {NULL}};
const Segment SGI124482[] = {{&S19263[44], 22}, {&S19263[271], 7}, {NULL}};
const Segment SGI124483[] = {{&S19263[30], 6}, {NULL}};
const Segment SGI124484[] = {{&S19263[31], 3}, {NULL}};
const Segment SGI124485[] = {{&S19263[80], 6}, {NULL}};
const Segment SGI124486[] = {{&S19264[20], 1}, {NULL}};
const Segment SGI124487[] = {{&S19264[53], 17}, {NULL}};
const Segment SGI124488[] = {{&S19264[39], 6}, {NULL}};
const Segment SGI124489[] = {{&S19266[44], 33}, {NULL}};
const Segment SGI124490[] = {{&S19266[44], 45}, {NULL}};
const Segment SGI124491[] = {{&S19266[30], 6}, {NULL}};
const Segment SGI124492[] = {{&S19266[31], 3}, {NULL}};
const Segment SGI124493[] = {{&S19266[103], 6}, {NULL}};
const Segment SGI124494[] = {{&S19267[44], 10}, {NULL}};
const Segment SGI124495[] = {{&S19267[44], 22}, {NULL}};
const Segment SGI124496[] = {{&S19267[30], 6}, {NULL}};
const Segment SGI124497[] = {{&S19267[116], 3}, {NULL}};
const Segment SGI124498[] = {{&S19267[80], 7}, {NULL}};
const Segment SGI124499[] = {{&S19267[136], 8}, {NULL}};
const Segment SGI124500[] = {{&S19268[44], 10}, {NULL}};
const Segment SGI124501[] = {{&S19268[44], 22}, {&S19268[74], 22}, {NULL}};
const Segment SGI124502[] = {{&S19268[30], 6}, {NULL}};
const Segment SGI124503[] = {{&S19268[31], 3}, {NULL}};
const Segment SGI124504[] = {{&S19268[110], 6}, {NULL}};
const Segment SGI124505[] = {{&S19270[44], 53}, {NULL}};
const Segment SGI124506[] = {{&S19270[44], 65}, {NULL}};
const Segment SGI124507[] = {{&S19270[30], 6}, {NULL}};
const Segment SGI124508[] = {{&S19270[31], 3}, {NULL}};
const Segment SGI124509[] = {{&S19270[123], 6}, {NULL}};
const Segment SGI124510[] = {{&S19271[44], 10}, {NULL}};
const Segment SGI124511[] = {{&S19271[44], 22}, {NULL}};
const Segment SGI124512[] = {{&S19271[30], 6}, {NULL}};
const Segment SGI124513[] = {{&S19271[31], 3}, {NULL}};
const Segment SGI124514[] = {{&S19271[80], 5}, {NULL}};
const Segment SGI124515[] = {{&S19272[46], 10}, {NULL}};
const Segment SGI124516[] = {{&S19272[46], 34}, {NULL}};
const Segment SGI124517[] = {{&S19272[32], 6}, {NULL}};
const Segment SGI124518[] = {{&S19275[44], 29}, {NULL}};
const Segment SGI124519[] = {{&S19275[44], 41}, {NULL}};
const Segment SGI124520[] = {{&S19275[30], 6}, {NULL}};
const Segment SGI124521[] = {{&S19275[31], 3}, {NULL}};
const Segment SGI124522[] = {{&S19275[99], 6}, {NULL}};
const Segment SGI124523[] = {{&S19276[44], 49}, {NULL}};
const Segment SGI124524[] = {{&S19276[44], 61}, {NULL}};
const Segment SGI124525[] = {{&S19276[30], 6}, {NULL}};
const Segment SGI124526[] = {{&S19276[31], 3}, {NULL}};
const Segment SGI124527[] = {{&S19276[119], 6}, {NULL}};
const Segment SGI124528[] = {{&S19278[44], 49}, {NULL}};
const Segment SGI124529[] = {{&S19278[44], 61}, {NULL}};
const Segment SGI124530[] = {{&S19278[30], 6}, {NULL}};
const Segment SGI124531[] = {{&S19278[31], 3}, {NULL}};
const Segment SGI124532[] = {{&S19278[119], 6}, {NULL}};
const Segment SGI124533[] = {{&S19280[44], 10}, {NULL}};
const Segment SGI124534[] = {{&S19280[44], 22}, {NULL}};
const Segment SGI124535[] = {{&S19280[30], 6}, {NULL}};
const Segment SGI124536[] = {{&S19280[31], 3}, {NULL}};
const Segment SGI124537[] = {{&S19281[45], 10}, {NULL}};
const Segment SGI124538[] = {{&S19281[45], 34}, {NULL}};
const Segment SGI124539[] = {{&S19281[31], 6}, {NULL}};
const Segment SGI124540[] = {{&S19283[101], 29}, {NULL}};
const Segment SGI124541[] = {{&S19285[44], 10}, {NULL}};
const Segment SGI124542[] = {{&S19285[44], 22}, {NULL}};
const Segment SGI124543[] = {{&S19285[30], 6}, {NULL}};
const Segment SGI124544[] = {{&S19285[31], 3}, {NULL}};
const Segment SGI124545[] = {{&S19285[80], 5}, {NULL}};
const Segment SGI124546[] = {{&S19286[44], 10}, {NULL}};
const Segment SGI124547[] = {{&S19286[44], 22}, {NULL}};
const Segment SGI124548[] = {{&S19286[30], 6}, {NULL}};
const Segment SGI124549[] = {{&S19286[80], 6}, {NULL}};
const Segment SGI124550[] = {{&S19287[42], 10}, {NULL}};
const Segment SGI124551[] = {{&S19287[30], 6}, {NULL}};
const Segment SGI124552[] = {{&S19287[31], 3}, {NULL}};
const Segment SGI124553[] = {{&S19287[78], 6}, {NULL}};
const Segment SGI124554[] = {{&S19288[42], 11}, {NULL}};
const Segment SGI124555[] = {{&S19288[30], 6}, {NULL}};
const Segment SGI124556[] = {{&S19288[31], 3}, {NULL}};
const Segment SGI124557[] = {{&S19288[79], 6}, {NULL}};
const Segment SGI124558[] = {{&S19289[44], 10}, {&S19289[44], 10}, {NULL}};
const Segment SGI124559[] = {{&S19289[44], 22}, {&S19289[74], 22}, {NULL}};
const Segment SGI124560[] = {{&S19289[30], 6}, {NULL}};
const Segment SGI124561[] = {{&S19289[31], 3}, {NULL}};
const Segment SGI124562[] = {{&S19289[110], 6}, {NULL}};
const Segment SGI124563[] = {{&S19291[44], 10}, {NULL}};
const Segment SGI124564[] = {{&S19291[44], 22}, {NULL}};
const Segment SGI124565[] = {{&S19291[30], 6}, {NULL}};
const Segment SGI124566[] = {{&S19291[31], 3}, {NULL}};
const Segment SGI124567[] = {{&S19291[80], 6}, {NULL}};
const Segment SGI124568[] = {{&S19292[44], 10}, {NULL}};
const Segment SGI124569[] = {{&S19292[44], 22}, {NULL}};
const Segment SGI124570[] = {{&S19292[30], 6}, {NULL}};
const Segment SGI124571[] = {{&S19292[31], 3}, {NULL}};
const Segment SGI124572[] = {{&S19292[80], 6}, {NULL}};
const Segment SGI124573[] = {{&S19297[44], 10}, {NULL}};
const Segment SGI124574[] = {{&S19297[44], 47}, {NULL}};
const Segment SGI124575[] = {{&S19297[30], 6}, {NULL}};
const Segment SGI124576[] = {{&S19297[31], 3}, {NULL}};
const Segment SGI124577[] = {{&S19297[105], 6}, {NULL}};
const Segment SGI124578[] = {{&S19298[44], 30}, {NULL}};
const Segment SGI124579[] = {{&S19298[44], 42}, {NULL}};
const Segment SGI124580[] = {{&S19298[30], 6}, {NULL}};
const Segment SGI124581[] = {{&S19298[31], 3}, {NULL}};
const Segment SGI124582[] = {{&S19298[100], 6}, {NULL}};
const Segment SGI124583[] = {{&S19300[42], 10}, {NULL}};
const Segment SGI124584[] = {{&S19300[30], 6}, {NULL}};
const Segment SGI124585[] = {{&S19300[31], 3}, {NULL}};
const Segment SGI124586[] = {{&S19300[78], 6}, {NULL}};
const Segment SGI124587[] = {{&S19301[38], 10}, {NULL}};
const Segment SGI124588[] = {{&S19301[111], 6}, {NULL}};
const Segment SGI124589[] = {{&S19301[31], 6}, {NULL}};
const Segment SGI124590[] = {{&S19302[44], 29}, {NULL}};
const Segment SGI124591[] = {{&S19302[44], 41}, {NULL}};
const Segment SGI124592[] = {{&S19302[30], 6}, {NULL}};
const Segment SGI124593[] = {{&S19302[31], 3}, {NULL}};
const Segment SGI124594[] = {{&S19302[99], 6}, {NULL}};
const Segment SGI124595[] = {{&S19303[44], 10}, {NULL}};
const Segment SGI124596[] = {{&S19303[44], 22}, {NULL}};
const Segment SGI124597[] = {{&S19303[30], 6}, {NULL}};
const Segment SGI124598[] = {{&S19303[31], 3}, {NULL}};
const Segment SGI124599[] = {{&S19303[80], 6}, {NULL}};
const Segment SGI124600[] = {{&S102648[44], 17}, {NULL}};
const Segment SGI124601[] = {{&S102648[44], 29}, {NULL}};
const Segment SGI124602[] = {{&S102648[30], 6}, {NULL}};
const Segment SGI124603[] = {{&S102648[118], 2}, {NULL}};
const Segment SGI124604[] = {{&S102648[87], 5}, {NULL}};
const Segment SGI124605[] = {{&S102649[44], 19}, {NULL}};
const Segment SGI124606[] = {{&S102649[44], 31}, {NULL}};
const Segment SGI124607[] = {{&S102649[30], 6}, {NULL}};
const Segment SGI124608[] = {{&S102649[31], 3}, {NULL}};
const Segment SGI124609[] = {{&S102649[96], 6}, {NULL}};
const Segment SGI124610[] = {{&S102649[75], 8}, {NULL}};
const Segment SGI124611[] = {{&S102668[45], 9}, {NULL}};
const Segment SGI124612[] = {{&S102668[30], 6}, {NULL}};
const Segment SGI124613[] = {{&S102668[31], 3}, {NULL}};
const Segment SGI124614[] = {{&S102668[80], 5}, {NULL}};
const Segment SGI124615[] = {{&S102668[138], 13}, {NULL}};
const Segment SGI124616[] = {{&S13065[42], 4}, {NULL}};
const Segment SGI124617[] = {{&S13065[42], 13}, {NULL}};
const Segment SGI124618[] = {{&S13065[30], 4}, {NULL}};
const Segment SGI124619[] = {{&S13065[103], 3}, {NULL}};
const Segment SGI124620[] = {{&S13065[69], 5}, {NULL}};
const Segment SGI124621[] = {{&S13066[42], 6}, {NULL}};
const Segment SGI124622[] = {{&S13066[42], 16}, {NULL}};
const Segment SGI124623[] = {{&S13066[30], 4}, {NULL}};
const Segment SGI124624[] = {{&S13066[106], 3}, {NULL}};
const Segment SGI124625[] = {{&S13066[72], 5}, {NULL}};
const Segment SGI124626[] = {{&S102696[44], 10}, {NULL}};
const Segment SGI124627[] = {{&S102696[44], 22}, {NULL}};
const Segment SGI124628[] = {{&S102696[30], 6}, {NULL}};
const Segment SGI124629[] = {{&S102696[139], 4}, {NULL}};
const Segment SGI124630[] = {{&S102696[31], 3}, {&S102696[192], 5}, {NULL}};
const Segment SGI124631[] = {{&S102696[80], 6}, {NULL}};
const Segment SGI124632[] = {{&S19480[44], 11}, {NULL}};
const Segment SGI124633[] = {{&S19480[44], 28}, {NULL}};
const Segment SGI124634[] = {{&S19480[30], 6}, {NULL}};
const Segment SGI124635[] = {{&S102701[45], 10}, {NULL}};
const Segment SGI124636[] = {{&S19480[31], 3}, {NULL}};
const Segment SGI124637[] = {{&S102701[30], 6}, {NULL}};
const Segment SGI124638[] = {{&S19480[86], 6}, {NULL}};
const Segment SGI124639[] = {{&S102701[31], 3}, {NULL}};
const Segment SGI124640[] = {{&S102701[81], 5}, {NULL}};
const Segment SGI124641[] = {{&S102701[139], 13}, {NULL}};
const Segment SGI124642[] = {{&S19481[44], 7}, {NULL}};
const Segment SGI124643[] = {{&S19481[44], 24}, {NULL}};
const Segment SGI124644[] = {{&S19481[30], 6}, {NULL}};
const Segment SGI124645[] = {{&S19481[31], 3}, {NULL}};
const Segment SGI124646[] = {{&S19481[82], 6}, {NULL}};
const Segment SGI124647[] = {{&S19482[44], 13}, {NULL}};
const Segment SGI124648[] = {{&S19482[44], 30}, {NULL}};
const Segment SGI124649[] = {{&S19482[30], 6}, {NULL}};
const Segment SGI124650[] = {{&S102704[44], 10}, {NULL}};
const Segment SGI124651[] = {{&S102704[44], 22}, {NULL}};
const Segment SGI124652[] = {{&S102704[30], 6}, {NULL}};
const Segment SGI124653[] = {{&S19482[31], 3}, {NULL}};
const Segment SGI124654[] = {{&S19482[88], 6}, {NULL}};
const Segment SGI124655[] = {{&S102704[111], 2}, {NULL}};
const Segment SGI124656[] = {{&S102704[80], 5}, {NULL}};
const Segment SGI124657[] = {{&S19483[44], 11}, {NULL}};
const Segment SGI124658[] = {{&S19483[44], 28}, {NULL}};
const Segment SGI124659[] = {{&S19483[30], 6}, {NULL}};
const Segment SGI124660[] = {{&S19483[31], 3}, {NULL}};
const Segment SGI124661[] = {{&S19483[86], 6}, {NULL}};
const Segment SGI124662[] = {{&S19484[44], 11}, {NULL}};
const Segment SGI124663[] = {{&S19484[44], 28}, {NULL}};
const Segment SGI124664[] = {{&S19484[30], 6}, {NULL}};
const Segment SGI124665[] = {{&S19484[31], 3}, {NULL}};
const Segment SGI124666[] = {{&S19484[86], 6}, {NULL}};
const Segment SGI124667[] = {{&S102708[44], 10}, {NULL}};
const Segment SGI124668[] = {{&S102708[44], 22}, {NULL}};
const Segment SGI124669[] = {{&S102708[30], 6}, {NULL}};
const Segment SGI124670[] = {{&S102708[31], 3}, {NULL}};
const Segment SGI124671[] = {{&S102708[80], 6}, {NULL}};
const Segment SGI124672[] = {{&S102708[133], 13}, {NULL}};
const Segment SGI124673[] = {{&S19486[44], 11}, {NULL}};
const Segment SGI124674[] = {{&S19486[44], 28}, {NULL}};
const Segment SGI124675[] = {{&S19486[30], 6}, {NULL}};
const Segment SGI124676[] = {{&S19486[31], 3}, {NULL}};
const Segment SGI124677[] = {{&S19486[86], 6}, {NULL}};
const Segment SGI124678[] = {{&S19487[44], 12}, {NULL}};
const Segment SGI124679[] = {{&S19487[44], 28}, {NULL}};
const Segment SGI124680[] = {{&S19487[30], 6}, {NULL}};
const Segment SGI124681[] = {{&S13087[42], 6}, {NULL}};
const Segment SGI124682[] = {{&S13087[42], 18}, {NULL}};
const Segment SGI124683[] = {{&S19487[31], 3}, {NULL}};
const Segment SGI124684[] = {{&S13087[30], 4}, {NULL}};
const Segment SGI124685[] = {{&S19487[86], 6}, {NULL}};
const Segment SGI124686[] = {{&S13087[109], 3}, {NULL}};
const Segment SGI124687[] = {{&S13087[74], 6}, {NULL}};
const Segment SGI124688[] = {{&S19488[44], 11}, {NULL}};
const Segment SGI124689[] = {{&S19488[44], 28}, {NULL}};
const Segment SGI124690[] = {{&S19488[30], 6}, {NULL}};
const Segment SGI124691[] = {{&S19488[31], 3}, {NULL}};
const Segment SGI124692[] = {{&S19488[86], 6}, {NULL}};
const Segment SGI124693[] = {{&S19489[44], 12}, {NULL}};
const Segment SGI124694[] = {{&S19489[44], 28}, {NULL}};
const Segment SGI124695[] = {{&S19489[30], 6}, {NULL}};
const Segment SGI124696[] = {{&S19489[31], 3}, {NULL}};
const Segment SGI124697[] = {{&S19489[86], 6}, {NULL}};
const Segment SGI124698[] = {{&S19490[44], 8}, {NULL}};
const Segment SGI124699[] = {{&S19490[44], 24}, {NULL}};
const Segment SGI124700[] = {{&S19490[30], 6}, {NULL}};
const Segment SGI124701[] = {{&S19490[31], 3}, {NULL}};
const Segment SGI124702[] = {{&S19490[82], 6}, {NULL}};
const Segment SGI124703[] = {{&S19491[44], 13}, {NULL}};
const Segment SGI124704[] = {{&S19491[44], 30}, {NULL}};
const Segment SGI124705[] = {{&S19491[30], 6}, {NULL}};
const Segment SGI124706[] = {{&S19491[31], 3}, {NULL}};
const Segment SGI124707[] = {{&S19491[88], 6}, {NULL}};
const Segment SGI124708[] = {{&S19492[44], 12}, {NULL}};
const Segment SGI124709[] = {{&S19492[44], 29}, {NULL}};
const Segment SGI124710[] = {{&S102723[34], 9}, {NULL}};
const Segment SGI124711[] = {{&S19492[30], 6}, {NULL}};
const Segment SGI124712[] = {{&S102723[27], 6}, {NULL}};
const Segment SGI124713[] = {{&S19492[31], 3}, {NULL}};
const Segment SGI124714[] = {{&S102723[96], 9}, {NULL}};
const Segment SGI124715[] = {{&S19492[87], 6}, {NULL}};
const Segment SGI124716[] = {{&S102723[69], 6}, {NULL}};
const Segment SGI124717[] = {{&S19493[44], 8}, {NULL}};
const Segment SGI124718[] = {{&S19493[44], 24}, {NULL}};
const Segment SGI124719[] = {{&S19493[30], 6}, {NULL}};
const Segment SGI124720[] = {{&S19493[31], 3}, {NULL}};
const Segment SGI124721[] = {{&S19493[82], 6}, {NULL}};
const Segment SGI124722[] = {{&S19494[44], 12}, {NULL}};
const Segment SGI124723[] = {{&S19494[44], 29}, {NULL}};
const Segment SGI124724[] = {{&S19494[30], 6}, {NULL}};
const Segment SGI124725[] = {{&S19494[31], 3}, {NULL}};
const Segment SGI124726[] = {{&S19494[87], 6}, {NULL}};
const Segment SGI124727[] = {{&S19495[44], 7}, {NULL}};
const Segment SGI124728[] = {{&S19495[44], 24}, {NULL}};
const Segment SGI124729[] = {{&S19495[30], 6}, {NULL}};
const Segment SGI124730[] = {{&S19495[31], 3}, {NULL}};
const Segment SGI124731[] = {{&S19495[82], 6}, {NULL}};
const Segment SGI124732[] = {{&S19496[44], 11}, {NULL}};
const Segment SGI124733[] = {{&S19496[44], 28}, {NULL}};
const Segment SGI124734[] = {{&S19496[30], 6}, {NULL}};
const Segment SGI124735[] = {{&S19496[31], 3}, {NULL}};
const Segment SGI124736[] = {{&S19496[86], 6}, {NULL}};
const Segment SGI124737[] = {{&S19497[44], 7}, {NULL}};
const Segment SGI124738[] = {{&S19497[44], 24}, {NULL}};
const Segment SGI124739[] = {{&S19497[30], 6}, {NULL}};
const Segment SGI124740[] = {{&S19497[31], 3}, {NULL}};
const Segment SGI124741[] = {{&S19497[82], 6}, {NULL}};
const Segment SGI124742[] = {{&S19498[44], 11}, {NULL}};
const Segment SGI124743[] = {{&S19498[44], 28}, {NULL}};
const Segment SGI124744[] = {{&S19498[30], 6}, {NULL}};
const Segment SGI124745[] = {{&S19498[31], 3}, {NULL}};
const Segment SGI124746[] = {{&S19498[86], 6}, {NULL}};
const Segment SGI124747[] = {{&S19499[38], 8}, {NULL}};
const Segment SGI124748[] = {{&S19499[31], 6}, {NULL}};
const Segment SGI124749[] = {{&S19500[44], 12}, {NULL}};
const Segment SGI124750[] = {{&S19500[44], 29}, {NULL}};
const Segment SGI124751[] = {{&S19500[30], 6}, {NULL}};
const Segment SGI124752[] = {{&S19500[31], 3}, {NULL}};
const Segment SGI124753[] = {{&S19500[87], 6}, {NULL}};
const Segment SGI124754[] = {{&S19501[44], 7}, {NULL}};
const Segment SGI124755[] = {{&S19501[44], 24}, {NULL}};
const Segment SGI124756[] = {{&S19501[30], 6}, {NULL}};
const Segment SGI124757[] = {{&S19501[31], 3}, {NULL}};
const Segment SGI124758[] = {{&S19501[82], 6}, {NULL}};
const Segment SGI124759[] = {{&S19502[38], 7}, {NULL}};
const Segment SGI124760[] = {{&S19502[31], 6}, {NULL}};
const Segment SGI124761[] = {{&S19503[44], 13}, {NULL}};
const Segment SGI124762[] = {{&S19503[44], 30}, {NULL}};
const Segment SGI124763[] = {{&S19503[30], 6}, {NULL}};
const Segment SGI124764[] = {{&S19503[31], 3}, {NULL}};
const Segment SGI124765[] = {{&S19503[88], 6}, {NULL}};
const Segment SGI124766[] = {{&S19504[44], 12}, {NULL}};
const Segment SGI124767[] = {{&S19504[44], 28}, {NULL}};
const Segment SGI124768[] = {{&S19504[30], 6}, {NULL}};
const Segment SGI124769[] = {{&S19504[31], 3}, {NULL}};
const Segment SGI124770[] = {{&S19504[86], 6}, {NULL}};
const Segment SGI124771[] = {{&S19505[44], 11}, {NULL}};
const Segment SGI124772[] = {{&S19505[44], 28}, {NULL}};
const Segment SGI124773[] = {{&S19505[30], 6}, {NULL}};
const Segment SGI124774[] = {{&S19505[31], 3}, {NULL}};
const Segment SGI124775[] = {{&S19505[86], 6}, {NULL}};
const Segment SGI124776[] = {{&S19506[44], 11}, {NULL}};
const Segment SGI124777[] = {{&S19506[44], 28}, {NULL}};
const Segment SGI124778[] = {{&S19506[30], 6}, {NULL}};
const Segment SGI124779[] = {{&S19506[31], 3}, {NULL}};
const Segment SGI124780[] = {{&S19506[86], 6}, {NULL}};
const Segment SGI124781[] = {{&S19507[44], 11}, {NULL}};
const Segment SGI124782[] = {{&S19507[44], 28}, {NULL}};
const Segment SGI124783[] = {{&S19507[30], 6}, {NULL}};
const Segment SGI124784[] = {{&S19507[31], 3}, {NULL}};
const Segment SGI124785[] = {{&S19507[86], 6}, {NULL}};
const Segment SGI124786[] = {{&S19508[38], 7}, {NULL}};
const Segment SGI124787[] = {{&S19508[31], 6}, {NULL}};
const Segment SGI124788[] = {{&S19509[44], 12}, {NULL}};
const Segment SGI124789[] = {{&S19509[44], 29}, {NULL}};
const Segment SGI124790[] = {{&S19509[30], 6}, {NULL}};
const Segment SGI124791[] = {{&S19509[31], 3}, {NULL}};
const Segment SGI124792[] = {{&S19509[87], 6}, {NULL}};
const Segment SGI124793[] = {{&S19510[44], 8}, {NULL}};
const Segment SGI124794[] = {{&S19510[44], 24}, {NULL}};
const Segment SGI124795[] = {{&S19510[30], 6}, {NULL}};
const Segment SGI124796[] = {{&S19510[31], 3}, {NULL}};
const Segment SGI124797[] = {{&S19510[82], 6}, {NULL}};
const Segment SGI124798[] = {{&S19511[44], 12}, {NULL}};
const Segment SGI124799[] = {{&S19511[44], 29}, {NULL}};
const Segment SGI124800[] = {{&S19511[30], 6}, {NULL}};
const Segment SGI124801[] = {{&S19511[31], 3}, {NULL}};
const Segment SGI124802[] = {{&S19511[87], 6}, {NULL}};
const Segment SGI124803[] = {{&S19512[44], 8}, {NULL}};
const Segment SGI124804[] = {{&S19512[44], 25}, {NULL}};
const Segment SGI124805[] = {{&S19512[30], 6}, {NULL}};
const Segment SGI124806[] = {{&S19512[31], 3}, {NULL}};
const Segment SGI124807[] = {{&S19512[83], 6}, {NULL}};
const Segment SGI124808[] = {{&S19513[44], 11}, {NULL}};
const Segment SGI124809[] = {{&S19513[44], 28}, {NULL}};
const Segment SGI124810[] = {{&S19513[30], 6}, {NULL}};
const Segment SGI124811[] = {{&S19513[31], 3}, {NULL}};
const Segment SGI124812[] = {{&S19513[86], 6}, {NULL}};
const Segment SGI124813[] = {{&S19514[44], 7}, {NULL}};
const Segment SGI124814[] = {{&S19514[44], 24}, {NULL}};
const Segment SGI124815[] = {{&S19514[30], 6}, {NULL}};
const Segment SGI124816[] = {{&S19514[31], 3}, {NULL}};
const Segment SGI124817[] = {{&S19514[82], 6}, {NULL}};
const Segment SGI124818[] = {{&S19515[44], 7}, {NULL}};
const Segment SGI124819[] = {{&S19515[44], 24}, {NULL}};
const Segment SGI124820[] = {{&S19515[30], 6}, {NULL}};
const Segment SGI124821[] = {{&S19515[31], 3}, {NULL}};
const Segment SGI124822[] = {{&S19515[82], 6}, {NULL}};
const Segment SGI124823[] = {{&S19516[44], 12}, {NULL}};
const Segment SGI124824[] = {{&S19516[44], 28}, {NULL}};
const Segment SGI124825[] = {{&S19516[30], 6}, {NULL}};
const Segment SGI124826[] = {{&S19516[31], 3}, {NULL}};
const Segment SGI124827[] = {{&S19516[86], 6}, {NULL}};
const Segment SGI124828[] = {{&S19517[44], 7}, {NULL}};
const Segment SGI124829[] = {{&S19517[44], 24}, {NULL}};
const Segment SGI124830[] = {{&S19517[30], 6}, {NULL}};
const Segment SGI124831[] = {{&S19517[31], 3}, {NULL}};
const Segment SGI124832[] = {{&S19517[82], 6}, {NULL}};
const Segment SGI124833[] = {{&S19518[44], 8}, {NULL}};
const Segment SGI124834[] = {{&S19518[44], 24}, {NULL}};
const Segment SGI124835[] = {{&S19518[30], 6}, {NULL}};
const Segment SGI124836[] = {{&S19518[31], 3}, {NULL}};
const Segment SGI124837[] = {{&S19518[82], 6}, {NULL}};
const Segment SGI124838[] = {{&S19520[42], 11}, {NULL}};
const Segment SGI124839[] = {{&S19520[30], 6}, {NULL}};
const Segment SGI124840[] = {{&S19520[31], 3}, {NULL}};
const Segment SGI124841[] = {{&S19520[84], 6}, {NULL}};
const Segment SGI124842[] = {{&S19521[44], 12}, {NULL}};
const Segment SGI124843[] = {{&S19521[44], 28}, {NULL}};
const Segment SGI124844[] = {{&S19521[30], 6}, {NULL}};
const Segment SGI124845[] = {{&S19521[31], 3}, {NULL}};
const Segment SGI124846[] = {{&S19521[86], 6}, {NULL}};
const Segment SGI124847[] = {{&S19522[44], 7}, {NULL}};
const Segment SGI124848[] = {{&S19522[44], 24}, {NULL}};
const Segment SGI124849[] = {{&S19522[30], 6}, {NULL}};
const Segment SGI124850[] = {{&S19522[31], 3}, {NULL}};
const Segment SGI124851[] = {{&S19522[82], 6}, {NULL}};
const Segment SGI124852[] = {{&S19523[42], 12}, {NULL}};
const Segment SGI124853[] = {{&S19523[30], 6}, {NULL}};
const Segment SGI124854[] = {{&S19523[31], 3}, {NULL}};
const Segment SGI124855[] = {{&S19523[84], 6}, {NULL}};
const Segment SGI124856[] = {{&S19524[44], 8}, {NULL}};
const Segment SGI124857[] = {{&S19524[44], 24}, {NULL}};
const Segment SGI124858[] = {{&S19524[30], 6}, {NULL}};
const Segment SGI124859[] = {{&S19524[31], 3}, {NULL}};
const Segment SGI124860[] = {{&S19524[82], 6}, {NULL}};
const Segment SGI124861[] = {{&S19525[38], 8}, {NULL}};
const Segment SGI124862[] = {{&S19525[31], 6}, {NULL}};
const Segment SGI124863[] = {{&S102836[44], 9}, {NULL}};
const Segment SGI124864[] = {{&S102836[44], 21}, {NULL}};
const Segment SGI124865[] = {{&S102836[30], 6}, {NULL}};
const Segment SGI124866[] = {{&S102836[110], 2}, {NULL}};
const Segment SGI124867[] = {{&S102836[79], 5}, {NULL}};
const Segment SGI124868[] = {{&S102864[44], 13}, {NULL}};
const Segment SGI124869[] = {{&S102864[44], 25}, {NULL}};
const Segment SGI124870[] = {{&S102864[30], 6}, {NULL}};
const Segment SGI124871[] = {{&S102864[31], 3}, {NULL}};
const Segment SGI124872[] = {{&S102864[83], 6}, {NULL}};
const Segment SGI124873[] = {{&S102897[44], 10}, {NULL}};
const Segment SGI124874[] = {{&S102897[44], 22}, {NULL}};
const Segment SGI124875[] = {{&S102897[30], 6}, {NULL}};
const Segment SGI124876[] = {{&S102897[111], 2}, {NULL}};
const Segment SGI124877[] = {{&S102897[80], 5}, {NULL}};
const Segment SGI124878[] = {{&S102916[45], 10}, {NULL}};
const Segment SGI124879[] = {{&S102916[30], 6}, {NULL}};
const Segment SGI124880[] = {{&S102916[31], 3}, {NULL}};
const Segment SGI124881[] = {{&S102916[81], 5}, {NULL}};
const Segment SGI124882[] = {{&S102916[139], 13}, {NULL}};
const Segment SGI124883[] = {{&S19643[44], 18}, {NULL}};
const Segment SGI124884[] = {{&S19643[44], 30}, {NULL}};
const Segment SGI124885[] = {{&S19643[30], 6}, {NULL}};
const Segment SGI124886[] = {{&S19643[31], 3}, {NULL}};
const Segment SGI124887[] = {{&S19643[88], 6}, {NULL}};
const Segment SGI124888[] = {{&S19648[44], 21}, {NULL}};
const Segment SGI124889[] = {{&S19648[44], 33}, {NULL}};
const Segment SGI124890[] = {{&S19648[30], 6}, {NULL}};
const Segment SGI124891[] = {{&S19648[31], 3}, {NULL}};
const Segment SGI124892[] = {{&S19648[91], 6}, {NULL}};
const Segment SGI124893[] = {{&S19658[44], 18}, {NULL}};
const Segment SGI124894[] = {{&S19658[44], 30}, {NULL}};
const Segment SGI124895[] = {{&S19658[30], 6}, {NULL}};
const Segment SGI124896[] = {{&S19658[31], 3}, {NULL}};
const Segment SGI124897[] = {{&S19658[88], 6}, {NULL}};
const Segment SGI124898[] = {{&S19659[44], 14}, {NULL}};
const Segment SGI124899[] = {{&S19659[44], 26}, {NULL}};
const Segment SGI124900[] = {{&S19659[30], 6}, {NULL}};
const Segment SGI124901[] = {{&S19659[31], 3}, {NULL}};
const Segment SGI124902[] = {{&S19659[84], 6}, {NULL}};
const Segment SGI124903[] = {{&S19660[44], 21}, {NULL}};
const Segment SGI124904[] = {{&S19660[44], 33}, {NULL}};
const Segment SGI124905[] = {{&S19660[30], 6}, {NULL}};
const Segment SGI124906[] = {{&S19660[31], 3}, {NULL}};
const Segment SGI124907[] = {{&S19660[91], 6}, {NULL}};
const Segment SGI124908[] = {{&S19661[44], 18}, {NULL}};
const Segment SGI124909[] = {{&S19661[44], 30}, {NULL}};
const Segment SGI124910[] = {{&S19661[30], 6}, {NULL}};
const Segment SGI124911[] = {{&S19661[31], 3}, {NULL}};
const Segment SGI124912[] = {{&S19661[88], 6}, {NULL}};
const Segment SGI124913[] = {{&S19663[44], 21}, {NULL}};
const Segment SGI124914[] = {{&S19663[44], 33}, {NULL}};
const Segment SGI124915[] = {{&S19663[30], 6}, {NULL}};
const Segment SGI124916[] = {{&S19663[31], 3}, {NULL}};
const Segment SGI124917[] = {{&S19663[91], 6}, {NULL}};
const Segment SGI124918[] = {{&S19664[44], 21}, {NULL}};
const Segment SGI124919[] = {{&S19664[44], 33}, {NULL}};
const Segment SGI124920[] = {{&S19664[30], 6}, {NULL}};
const Segment SGI124921[] = {{&S19664[31], 3}, {NULL}};
const Segment SGI124922[] = {{&S19664[91], 6}, {NULL}};
const Segment SGI124923[] = {{&S19665[44], 18}, {NULL}};
const Segment SGI124924[] = {{&S19665[44], 30}, {NULL}};
const Segment SGI124925[] = {{&S19665[30], 6}, {NULL}};
const Segment SGI124926[] = {{&S19665[31], 3}, {NULL}};
const Segment SGI124927[] = {{&S19665[88], 6}, {NULL}};
const Segment SGI124928[] = {{&S19666[44], 21}, {NULL}};
const Segment SGI124929[] = {{&S19666[44], 33}, {NULL}};
const Segment SGI124930[] = {{&S19666[30], 6}, {NULL}};
const Segment SGI124931[] = {{&S19666[31], 3}, {NULL}};
const Segment SGI124932[] = {{&S19666[91], 6}, {NULL}};
const Segment SGI124933[] = {{&S19667[38], 21}, {NULL}};
const Segment SGI124934[] = {{&S19667[31], 6}, {NULL}};
const Segment SGI124935[] = {{&S19668[44], 18}, {NULL}};
const Segment SGI124936[] = {{&S19668[44], 30}, {NULL}};
const Segment SGI124937[] = {{&S19668[30], 6}, {NULL}};
const Segment SGI124938[] = {{&S19668[31], 3}, {NULL}};
const Segment SGI124939[] = {{&S19668[88], 6}, {NULL}};
const Segment SGI124940[] = {{&S19669[38], 21}, {NULL}};
const Segment SGI124941[] = {{&S19669[31], 6}, {NULL}};
const Segment SGI124942[] = {{&S13603[42], 13}, {NULL}};
const Segment SGI124943[] = {{&S13603[42], 25}, {NULL}};
const Segment SGI124944[] = {{&S13603[30], 4}, {NULL}};
const Segment SGI124945[] = {{&S13603[116], 3}, {NULL}};
const Segment SGI124946[] = {{&S13603[81], 6}, {NULL}};
const Segment SGI124947[] = {{&S103082[44], 6}, {NULL}};
const Segment SGI124948[] = {{&S103082[44], 18}, {NULL}};
const Segment SGI124949[] = {{&S103082[30], 6}, {NULL}};
const Segment SGI124950[] = {{&S103082[31], 3}, {NULL}};
const Segment SGI124951[] = {{&S103082[83], 6}, {NULL}};
const Segment SGI124952[] = {{&S103082[62], 8}, {NULL}};
const Segment SGI124953[] = {{&S103084[44], 10}, {NULL}};
const Segment SGI124954[] = {{&S103084[44], 22}, {NULL}};
const Segment SGI124955[] = {{&S103084[30], 6}, {NULL}};
const Segment SGI124956[] = {{&S103084[111], 2}, {NULL}};
const Segment SGI124957[] = {{&S103084[80], 5}, {NULL}};
const Segment SGI124958[] = {{&S103091[44], 7}, {NULL}};
const Segment SGI124959[] = {{&S103091[44], 19}, {NULL}};
const Segment SGI124960[] = {{&S103091[30], 6}, {NULL}};
const Segment SGI124961[] = {{&S103091[31], 3}, {NULL}};
const Segment SGI124962[] = {{&S103091[77], 6}, {NULL}};
const Segment SGI124963[] = {{&S103100[31], 9}, {NULL}};
const Segment SGI124964[] = {{&S103100[56], 3}, {NULL}};
const Segment SGI124965[] = {{&S13667[42], 8}, {NULL}};
const Segment SGI124966[] = {{&S13667[42], 17}, {NULL}};
const Segment SGI124967[] = {{&S13667[30], 4}, {NULL}};
const Segment SGI124968[] = {{&S13667[108], 3}, {NULL}};
const Segment SGI124969[] = {{&S13667[73], 6}, {NULL}};
const Segment SGI124970[] = {{&S103158[59], 10}, {NULL}};
const Segment SGI124971[] = {{&S103158[59], 22}, {NULL}};
const Segment SGI124972[] = {{&S103158[45], 6}, {NULL}};
const Segment SGI124973[] = {{&S103158[95], 5}, {NULL}};
const Segment SGI124974[] = {{&S103158[0], 14}, {NULL}};
const Segment SGI124975[] = {{&S13678[42], 82}, {NULL}};
const Segment SGI124976[] = {{&S13678[42], 103}, {NULL}};
const Segment SGI124977[] = {{&S13678[30], 4}, {NULL}};
const Segment SGI124978[] = {{&S13678[192], 3}, {NULL}};
const Segment SGI124979[] = {{&S13678[118], 5}, {NULL}};
const Segment SGI124980[] = {{&S103185[44], 10}, {NULL}};
const Segment SGI124981[] = {{&S103185[44], 22}, {NULL}};
const Segment SGI124982[] = {{&S103185[30], 6}, {NULL}};
const Segment SGI124983[] = {{&S103185[80], 6}, {NULL}};
const Segment SGI124984[] = {{&S103185[107], 13}, {NULL}};
const Segment SGI124985[] = {{&S10462[44], 10}, {NULL}};
const Segment SGI124986[] = {{&S10462[44], 22}, {NULL}};
const Segment SGI124987[] = {{&S10462[30], 6}, {NULL}};
const Segment SGI124988[] = {{&S10462[115], 3}, {NULL}};
const Segment SGI124989[] = {{&S10462[80], 6}, {NULL}};
const Segment SGI124990[] = {{&S10463[38], 10}, {NULL}};
const Segment SGI124991[] = {{&S10463[31], 6}, {NULL}};
const Segment SGI124992[] = {{&S10464[44], 10}, {NULL}};
const Segment SGI124993[] = {{&S10464[44], 22}, {NULL}};
const Segment SGI124994[] = {{&S10464[30], 6}, {NULL}};
const Segment SGI124995[] = {{&S10464[115], 3}, {NULL}};
const Segment SGI124996[] = {{&S10464[80], 6}, {NULL}};
const Segment SGI124997[] = {{&S10465[44], 10}, {NULL}};
const Segment SGI124998[] = {{&S10465[44], 21}, {NULL}};
const Segment SGI124999[] = {{&S10465[30], 6}, {NULL}};
const Segment SGI125000[] = {{&S10465[114], 3}, {NULL}};
const Segment SGI125001[] = {{&S10465[79], 6}, {NULL}};
const Segment SGI125002[] = {{&S10470[38], 11}, {NULL}};
const Segment SGI125003[] = {{&S10470[31], 6}, {NULL}};
const Segment SGI125004[] = {{&S13828[42], 17}, {NULL}};
const Segment SGI125005[] = {{&S13828[30], 4}, {NULL}};
const Segment SGI125006[] = {{&S13828[31], 3}, {NULL}};
const Segment SGI125007[] = {{&S13828[73], 6}, {NULL}};
const Segment SGI125008[] = {{&S13834[42], 6}, {NULL}};
const Segment SGI125009[] = {{&S13834[42], 18}, {NULL}};
const Segment SGI125010[] = {{&S13834[30], 4}, {NULL}};
const Segment SGI125011[] = {{&S13834[109], 3}, {NULL}};
const Segment SGI125012[] = {{&S13834[74], 6}, {NULL}};
const Segment SGI125013[] = {{&S103371[44], 12}, {NULL}};
const Segment SGI125014[] = {{&S103371[44], 22}, {NULL}};
const Segment SGI125015[] = {{&S103371[30], 6}, {NULL}};
const Segment SGI125016[] = {{&S103371[31], 3}, {NULL}};
const Segment SGI125017[] = {{&S103371[80], 6}, {NULL}};
const Segment SGI125018[] = {{&S103375[44], 10}, {NULL}};
const Segment SGI125019[] = {{&S103375[44], 22}, {NULL}};
const Segment SGI125020[] = {{&S103375[30], 6}, {NULL}};
const Segment SGI125021[] = {{&S103375[31], 3}, {NULL}};
const Segment SGI125022[] = {{&S103375[80], 6}, {NULL}};
const Segment SGI125023[] = {{&S103375[150], 3}, {NULL}};
const Segment SGI125024[] = {{&S103388[45], 10}, {NULL}};
const Segment SGI125025[] = {{&S103388[30], 6}, {NULL}};
const Segment SGI125026[] = {{&S103388[31], 3}, {NULL}};
const Segment SGI125027[] = {{&S103388[81], 5}, {NULL}};
const Segment SGI125028[] = {{&S103388[139], 13}, {NULL}};
const Segment SGI125029[] = {{&S103402[59], 11}, {NULL}};
const Segment SGI125030[] = {{&S103402[59], 23}, {NULL}};
const Segment SGI125031[] = {{&S103402[45], 6}, {NULL}};
const Segment SGI125032[] = {{&S103402[46], 3}, {NULL}};
const Segment SGI125033[] = {{&S103402[96], 6}, {NULL}};
const Segment SGI125034[] = {{&S103402[0], 14}, {NULL}};
const Segment SGI125035[] = {{&S103403[59], 11}, {NULL}};
const Segment SGI125036[] = {{&S103403[59], 23}, {NULL}};
const Segment SGI125037[] = {{&S103403[45], 6}, {NULL}};
const Segment SGI125038[] = {{&S103403[46], 3}, {NULL}};
const Segment SGI125039[] = {{&S103403[96], 6}, {NULL}};
const Segment SGI125040[] = {{&S103403[0], 14}, {NULL}};
const Segment SGI125041[] = {{&S13921[42], 6}, {NULL}};
const Segment SGI125042[] = {{&S13921[42], 18}, {NULL}};
const Segment SGI125043[] = {{&S13921[30], 4}, {NULL}};
const Segment SGI125044[] = {{&S13921[109], 3}, {NULL}};
const Segment SGI125045[] = {{&S13921[74], 6}, {NULL}};
const Segment SGI125046[] = {{&S103422[44], 10}, {NULL}};
const Segment SGI125047[] = {{&S103422[44], 27}, {NULL}};
const Segment SGI125048[] = {{&S103422[30], 6}, {NULL}};
const Segment SGI125049[] = {{&S103422[31], 3}, {NULL}};
const Segment SGI125050[] = {{&S103422[85], 6}, {NULL}};
const Segment SGI125051[] = {{&S13954[42], 82}, {NULL}};
const Segment SGI125052[] = {{&S13954[42], 103}, {NULL}};
const Segment SGI125053[] = {{&S13954[30], 4}, {NULL}};
const Segment SGI125054[] = {{&S13954[192], 3}, {NULL}};
const Segment SGI125055[] = {{&S13954[118], 5}, {NULL}};
const Segment SGI125056[] = {{&S13970[42], 10}, {NULL}};
const Segment SGI125057[] = {{&S13970[42], 22}, {NULL}};
const Segment SGI125058[] = {{&S13970[30], 4}, {NULL}};
const Segment SGI125059[] = {{&S13970[113], 3}, {NULL}};
const Segment SGI125060[] = {{&S13970[78], 6}, {NULL}};
const Segment SGI125061[] = {{&S10471[44], 10}, {NULL}};
const Segment SGI125062[] = {{&S10471[44], 22}, {NULL}};
const Segment SGI125063[] = {{&S10471[30], 6}, {NULL}};
const Segment SGI125064[] = {{&S10471[83], 6}, {NULL}};
const Segment SGI125065[] = {{&S10472[44], 11}, {NULL}};
const Segment SGI125066[] = {{&S10472[44], 23}, {NULL}};
const Segment SGI125067[] = {{&S10472[30], 6}, {NULL}};
const Segment SGI125068[] = {{&S10472[116], 3}, {NULL}};
const Segment SGI125069[] = {{&S10472[81], 6}, {NULL}};
const Segment SGI125070[] = {{&S10473[44], 11}, {NULL}};
const Segment SGI125071[] = {{&S10473[44], 23}, {NULL}};
const Segment SGI125072[] = {{&S10473[30], 6}, {NULL}};
const Segment SGI125073[] = {{&S10473[116], 3}, {NULL}};
const Segment SGI125074[] = {{&S10473[81], 6}, {NULL}};
const Segment SGI125075[] = {{&S10474[44], 10}, {NULL}};
const Segment SGI125076[] = {{&S10474[44], 20}, {NULL}};
const Segment SGI125077[] = {{&S10474[30], 6}, {NULL}};
const Segment SGI125078[] = {{&S10474[113], 3}, {NULL}};
const Segment SGI125079[] = {{&S10474[78], 6}, {NULL}};
const Segment SGI125080[] = {{&S10475[44], 11}, {NULL}};
const Segment SGI125081[] = {{&S10475[44], 22}, {NULL}};
const Segment SGI125082[] = {{&S10475[30], 6}, {NULL}};
const Segment SGI125083[] = {{&S10475[115], 3}, {NULL}};
const Segment SGI125084[] = {{&S10475[80], 6}, {NULL}};
const Segment SGI125085[] = {{&S10476[44], 29}, {NULL}};
const Segment SGI125086[] = {{&S10476[44], 41}, {NULL}};
const Segment SGI125087[] = {{&S10476[30], 6}, {NULL}};
const Segment SGI125088[] = {{&S10476[134], 3}, {NULL}};
const Segment SGI125089[] = {{&S10476[99], 6}, {NULL}};
const Segment SGI125090[] = {{&S10477[44], 10}, {NULL}};
const Segment SGI125091[] = {{&S10477[44], 22}, {NULL}};
const Segment SGI125092[] = {{&S10477[30], 6}, {NULL}};
const Segment SGI125093[] = {{&S10477[114], 3}, {NULL}};
const Segment SGI125094[] = {{&S10477[80], 5}, {NULL}};
const Segment SGI125095[] = {{&S10478[38], 11}, {NULL}};
const Segment SGI125096[] = {{&S10478[31], 6}, {NULL}};
const Segment SGI125097[] = {{&S10481[56], 10}, {NULL}};
const Segment SGI125098[] = {{&S10481[56], 22}, {NULL}};
const Segment SGI125099[] = {{&S10481[30], 18}, {NULL}};
const Segment SGI125100[] = {{&S10481[127], 3}, {NULL}};
const Segment SGI125101[] = {{&S10481[92], 6}, {NULL}};
const Segment SGI125102[] = {{&S10482[44], 11}, {NULL}};
const Segment SGI125103[] = {{&S10482[44], 23}, {NULL}};
const Segment SGI125104[] = {{&S10482[30], 6}, {NULL}};
const Segment SGI125105[] = {{&S10482[116], 3}, {NULL}};
const Segment SGI125106[] = {{&S10482[81], 6}, {NULL}};
const Segment SGI125107[] = {{&S10483[38], 11}, {NULL}};
const Segment SGI125108[] = {{&S10483[31], 6}, {NULL}};
const Segment SGI125109[] = {{&S10484[44], 10}, {NULL}};
const Segment SGI125110[] = {{&S10484[44], 22}, {NULL}};
const Segment SGI125111[] = {{&S10484[30], 6}, {NULL}};
const Segment SGI125112[] = {{&S10484[115], 3}, {NULL}};
const Segment SGI125113[] = {{&S10484[80], 6}, {NULL}};
const Segment SGI125114[] = {{&S10485[42], 10}, {NULL}};
const Segment SGI125115[] = {{&S10485[30], 6}, {NULL}};
const Segment SGI125116[] = {{&S10485[113], 3}, {NULL}};
const Segment SGI125117[] = {{&S10485[78], 6}, {NULL}};
const Segment SGI125118[] = {{&S10486[44], 11}, {NULL}};
const Segment SGI125119[] = {{&S10486[44], 23}, {NULL}};
const Segment SGI125120[] = {{&S10486[30], 6}, {NULL}};
const Segment SGI125121[] = {{&S10486[116], 3}, {NULL}};
const Segment SGI125122[] = {{&S10486[81], 6}, {NULL}};
const Segment SGI125123[] = {{&S10487[44], 28}, {NULL}};
const Segment SGI125124[] = {{&S10487[44], 40}, {NULL}};
const Segment SGI125125[] = {{&S10487[30], 6}, {NULL}};
const Segment SGI125126[] = {{&S10487[133], 3}, {NULL}};
const Segment SGI125127[] = {{&S10487[98], 6}, {NULL}};
const Segment SGI125128[] = {{&S10488[44], 29}, {NULL}};
const Segment SGI125129[] = {{&S10488[44], 41}, {NULL}};
const Segment SGI125130[] = {{&S10488[30], 6}, {NULL}};
const Segment SGI125131[] = {{&S10488[134], 3}, {NULL}};
const Segment SGI125132[] = {{&S10488[99], 6}, {NULL}};
const Segment SGI125133[] = {{&S10489[44], 11}, {NULL}};
const Segment SGI125134[] = {{&S10489[44], 23}, {NULL}};
const Segment SGI125135[] = {{&S10489[30], 6}, {NULL}};
const Segment SGI125136[] = {{&S10489[116], 3}, {NULL}};
const Segment SGI125137[] = {{&S10489[81], 6}, {NULL}};
const Segment SGI125138[] = {{&S10490[44], 28}, {NULL}};
const Segment SGI125139[] = {{&S10490[44], 40}, {NULL}};
const Segment SGI125140[] = {{&S10490[30], 6}, {NULL}};
const Segment SGI125141[] = {{&S10490[133], 3}, {NULL}};
const Segment SGI125142[] = {{&S10490[98], 6}, {NULL}};
const Segment SGI125143[] = {{&S10491[44], 29}, {NULL}};
const Segment SGI125144[] = {{&S10491[44], 41}, {NULL}};
const Segment SGI125145[] = {{&S10491[30], 6}, {NULL}};
const Segment SGI125146[] = {{&S10491[134], 3}, {NULL}};
const Segment SGI125147[] = {{&S10491[99], 6}, {NULL}};
const Segment SGI125148[] = {{&S10495[44], 10}, {NULL}};
const Segment SGI125149[] = {{&S10495[44], 26}, {NULL}};
const Segment SGI125150[] = {{&S10495[30], 6}, {NULL}};
const Segment SGI125151[] = {{&S10495[119], 3}, {NULL}};
const Segment SGI125152[] = {{&S10495[84], 6}, {NULL}};
const Segment SGI125153[] = {{&S10496[44], 11}, {NULL}};
const Segment SGI125154[] = {{&S10496[44], 23}, {NULL}};
const Segment SGI125155[] = {{&S10496[30], 6}, {NULL}};
const Segment SGI125156[] = {{&S10496[84], 6}, {NULL}};
const Segment SGI125157[] = {{&S10497[44], 10}, {NULL}};
const Segment SGI125158[] = {{&S10497[44], 22}, {NULL}};
const Segment SGI125159[] = {{&S10497[30], 6}, {NULL}};
const Segment SGI125160[] = {{&S10497[115], 3}, {NULL}};
const Segment SGI125161[] = {{&S10497[80], 6}, {NULL}};
const Segment SGI125162[] = {{&S10498[44], 28}, {NULL}};
const Segment SGI125163[] = {{&S10498[44], 40}, {NULL}};
const Segment SGI125164[] = {{&S10498[30], 6}, {NULL}};
const Segment SGI125165[] = {{&S10498[133], 3}, {NULL}};
const Segment SGI125166[] = {{&S10498[98], 6}, {NULL}};
const Segment SGI125167[] = {{&S10499[44], 29}, {NULL}};
const Segment SGI125168[] = {{&S10499[44], 41}, {NULL}};
const Segment SGI125169[] = {{&S10499[30], 6}, {NULL}};
const Segment SGI125170[] = {{&S10499[134], 3}, {NULL}};
const Segment SGI125171[] = {{&S10499[99], 6}, {NULL}};
const Segment SGI125172[] = {{&S10500[42], 11}, {NULL}};
const Segment SGI125173[] = {{&S10500[30], 6}, {NULL}};
const Segment SGI125174[] = {{&S10500[114], 3}, {NULL}};
const Segment SGI125175[] = {{&S10500[79], 6}, {NULL}};
const Segment SGI125176[] = {{&S10501[44], 11}, {NULL}};
const Segment SGI125177[] = {{&S10501[44], 23}, {NULL}};
const Segment SGI125178[] = {{&S10501[30], 6}, {NULL}};
const Segment SGI125179[] = {{&S10501[115], 3}, {NULL}};
const Segment SGI125180[] = {{&S10501[81], 5}, {NULL}};
const Segment SGI125181[] = {{&S10504[44], 10}, {NULL}};
const Segment SGI125182[] = {{&S10504[44], 28}, {NULL}};
const Segment SGI125183[] = {{&S10504[30], 6}, {NULL}};
const Segment SGI125184[] = {{&S10504[121], 3}, {NULL}};
const Segment SGI125185[] = {{&S10504[86], 6}, {NULL}};
const Segment SGI125186[] = {{&S14012[42], 4}, {NULL}};
const Segment SGI125187[] = {{&S14012[42], 16}, {NULL}};
const Segment SGI125188[] = {{&S14012[30], 5}, {NULL}};
const Segment SGI125189[] = {{&S14012[107], 3}, {NULL}};
const Segment SGI125190[] = {{&S14012[72], 6}, {NULL}};
const Segment SGI125191[] = {{&S14015[42], 8}, {NULL}};
const Segment SGI125192[] = {{&S14015[42], 16}, {NULL}};
const Segment SGI125193[] = {{&S14015[30], 4}, {NULL}};
const Segment SGI125194[] = {{&S14015[107], 3}, {NULL}};
const Segment SGI125195[] = {{&S14015[72], 6}, {NULL}};
const Segment SGI125196[] = {{&S103658[44], 10}, {NULL}};
const Segment SGI125197[] = {{&S103658[44], 22}, {NULL}};
const Segment SGI125198[] = {{&S103658[30], 6}, {NULL}};
const Segment SGI125199[] = {{&S103658[111], 2}, {NULL}};
const Segment SGI125200[] = {{&S103658[80], 5}, {NULL}};
const Segment SGI125201[] = {{&S103661[44], 10}, {NULL}};
const Segment SGI125202[] = {{&S103661[44], 22}, {NULL}};
const Segment SGI125203[] = {{&S103661[30], 6}, {NULL}};
const Segment SGI125204[] = {{&S103661[111], 2}, {NULL}};
const Segment SGI125205[] = {{&S103661[80], 5}, {NULL}};
const Segment SGI125206[] = {{&S103700[44], 9}, {NULL}};
const Segment SGI125207[] = {{&S103700[44], 19}, {NULL}};
const Segment SGI125208[] = {{&S103700[30], 6}, {NULL}};
const Segment SGI125209[] = {{&S103700[31], 3}, {NULL}};
const Segment SGI125210[] = {{&S103700[77], 6}, {NULL}};
const Segment SGI125211[] = {{&S103745[44], 10}, {NULL}};
const Segment SGI125212[] = {{&S103745[44], 22}, {NULL}};
const Segment SGI125213[] = {{&S103745[30], 6}, {NULL}};
const Segment SGI125214[] = {{&S103745[31], 3}, {NULL}};
const Segment SGI125215[] = {{&S103745[80], 6}, {NULL}};
const Segment SGI125216[] = {{&S103745[143], 3}, {NULL}};
const Segment SGI125217[] = {{&S103747[59], 10}, {NULL}};
const Segment SGI125218[] = {{&S103747[59], 22}, {NULL}};
const Segment SGI125219[] = {{&S103747[45], 6}, {NULL}};
const Segment SGI125220[] = {{&S103747[95], 5}, {NULL}};
const Segment SGI125221[] = {{&S103747[0], 14}, {NULL}};
const Segment SGI125222[] = {{&S103749[44], 10}, {NULL}};
const Segment SGI125223[] = {{&S103749[44], 22}, {NULL}};
const Segment SGI125224[] = {{&S103749[30], 6}, {NULL}};
const Segment SGI125225[] = {{&S103749[31], 3}, {NULL}};
const Segment SGI125226[] = {{&S103749[80], 5}, {NULL}};
const Segment SGI125227[] = {{&S103749[125], 14}, {NULL}};
const Segment SGI125228[] = {{&S103755[45], 10}, {NULL}};
const Segment SGI125229[] = {{&S103755[30], 6}, {NULL}};
const Segment SGI125230[] = {{&S103755[31], 3}, {NULL}};
const Segment SGI125231[] = {{&S103755[81], 5}, {NULL}};
const Segment SGI125232[] = {{&S103755[139], 13}, {NULL}};
const Segment SGI125233[] = {{&S103758[59], 10}, {NULL}};
const Segment SGI125234[] = {{&S103758[59], 22}, {NULL}};
const Segment SGI125235[] = {{&S103758[45], 6}, {NULL}};
const Segment SGI125236[] = {{&S103758[46], 3}, {NULL}};
const Segment SGI125237[] = {{&S103758[95], 6}, {NULL}};
const Segment SGI125238[] = {{&S103758[0], 14}, {NULL}};
const Segment SGI125239[] = {{&S103760[44], 10}, {NULL}};
const Segment SGI125240[] = {{&S103760[44], 22}, {NULL}};
const Segment SGI125241[] = {{&S103760[30], 6}, {NULL}};
const Segment SGI125242[] = {{&S103760[139], 4}, {NULL}};
const Segment SGI125243[] = {{&S103760[31], 3}, {&S103760[192], 5}, {NULL}};
const Segment SGI125244[] = {{&S103760[80], 6}, {NULL}};
const Segment SGI125245[] = {{&S103763[44], 10}, {NULL}};
const Segment SGI125246[] = {{&S103763[44], 22}, {NULL}};
const Segment SGI125247[] = {{&S103763[30], 6}, {NULL}};
const Segment SGI125248[] = {{&S14063[42], 82}, {NULL}};
const Segment SGI125249[] = {{&S14063[42], 103}, {NULL}};
const Segment SGI125250[] = {{&S14063[30], 4}, {NULL}};
const Segment SGI125251[] = {{&S14063[192], 3}, {NULL}};
const Segment SGI125252[] = {{&S14063[118], 5}, {NULL}};
const Segment SGI125253[] = {{&S15086[44], 29}, {NULL}};
const Segment SGI125254[] = {{&S15086[44], 41}, {NULL}};
const Segment SGI125255[] = {{&S15086[30], 6}, {NULL}};
const Segment SGI125256[] = {{&S15086[31], 3}, {NULL}};
const Segment SGI125257[] = {{&S15086[99], 5}, {NULL}};
const Segment SGI125258[] = {{&S15119[44], 21}, {NULL}};
const Segment SGI125259[] = {{&S15119[44], 33}, {NULL}};
const Segment SGI125260[] = {{&S15119[30], 6}, {NULL}};
const Segment SGI125261[] = {{&S15119[31], 3}, {NULL}};
const Segment SGI125262[] = {{&S15119[91], 5}, {NULL}};
const Segment SGI125263[] = {{&S103763[31], 3}, {NULL}};
const Segment SGI125264[] = {{&S103763[80], 6}, {NULL}};
const Segment SGI125265[] = {{&S103763[133], 11}, {NULL}};
const Segment SGI125266[] = {{&S103768[45], 10}, {NULL}};
const Segment SGI125267[] = {{&S103768[30], 6}, {NULL}};
const Segment SGI125268[] = {{&S103768[31], 3}, {NULL}};
const Segment SGI125269[] = {{&S103768[81], 5}, {NULL}};
const Segment SGI125270[] = {{&S103768[139], 13}, {NULL}};
const Segment SGI125271[] = {{&S103778[44], 10}, {NULL}};
const Segment SGI125272[] = {{&S103778[44], 22}, {NULL}};
const Segment SGI125273[] = {{&S103778[30], 6}, {NULL}};
const Segment SGI125274[] = {{&S103778[80], 6}, {NULL}};
const Segment SGI125275[] = {{&S103778[107], 13}, {NULL}};
const Segment SGI125276[] = {{&S103794[45], 10}, {NULL}};
const Segment SGI125277[] = {{&S103794[30], 6}, {NULL}};
const Segment SGI125278[] = {{&S103794[31], 3}, {NULL}};
const Segment SGI125279[] = {{&S103794[81], 5}, {NULL}};
const Segment SGI125280[] = {{&S103794[139], 13}, {NULL}};
const Segment SGI125281[] = {{&S15181[44], 30}, {NULL}};
const Segment SGI125282[] = {{&S15181[44], 42}, {NULL}};
const Segment SGI125283[] = {{&S15181[30], 6}, {NULL}};
const Segment SGI125284[] = {{&S15181[31], 3}, {NULL}};
const Segment SGI125285[] = {{&S15181[100], 6}, {NULL}};
const Segment SGI125286[] = {{&S14298[44], 12}, {NULL}};
const Segment SGI125287[] = {{&S14298[44], 28}, {NULL}};
const Segment SGI125288[] = {{&S14298[30], 6}, {NULL}};
const Segment SGI125289[] = {{&S14298[121], 3}, {NULL}};
const Segment SGI125290[] = {{&S14298[86], 6}, {NULL}};
const Segment SGI125291[] = {{&S15219[44], 6}, {NULL}};
const Segment SGI125292[] = {{&S15219[44], 18}, {NULL}};
const Segment SGI125293[] = {{&S15219[30], 6}, {NULL}};
const Segment SGI125294[] = {{&S15219[31], 3}, {NULL}};
const Segment SGI125295[] = {{&S15219[76], 5}, {NULL}};
const Segment SGI125296[] = {{&S15223[44], 14}, {NULL}};
const Segment SGI125297[] = {{&S15223[44], 25}, {NULL}};
const Segment SGI125298[] = {{&S15223[30], 6}, {NULL}};
const Segment SGI125299[] = {{&S15223[31], 3}, {NULL}};
const Segment SGI125300[] = {{&S15223[83], 5}, {NULL}};
const Segment SGI125301[] = {{&S15251[44], 25}, {NULL}};
const Segment SGI125302[] = {{&S15251[44], 37}, {NULL}};
const Segment SGI125303[] = {{&S15251[30], 6}, {NULL}};
const Segment SGI125304[] = {{&S15251[31], 3}, {NULL}};
const Segment SGI125305[] = {{&S15251[95], 6}, {NULL}};
const Segment SGI125306[] = {{&S15300[44], 25}, {NULL}};
const Segment SGI125307[] = {{&S15300[44], 37}, {NULL}};
const Segment SGI125308[] = {{&S15300[30], 6}, {NULL}};
const Segment SGI125309[] = {{&S15300[31], 3}, {NULL}};
const Segment SGI125310[] = {{&S15300[95], 6}, {NULL}};
const Segment SGI125311[] = {{&S15330[44], 24}, {NULL}};
const Segment SGI125312[] = {{&S15330[44], 36}, {NULL}};
const Segment SGI125313[] = {{&S15330[30], 6}, {NULL}};
const Segment SGI125314[] = {{&S15330[31], 3}, {NULL}};
const Segment SGI125315[] = {{&S15330[94], 6}, {NULL}};
const Segment SGI125316[] = {{&S15369[44], 29}, {NULL}};
const Segment SGI125317[] = {{&S15369[44], 41}, {NULL}};
const Segment SGI125318[] = {{&S15369[30], 6}, {NULL}};
const Segment SGI125319[] = {{&S15369[31], 3}, {NULL}};
const Segment SGI125320[] = {{&S15369[99], 6}, {NULL}};
const Segment SGI125321[] = {{&S15387[22], 9}, {NULL}};
const Segment SGI125322[] = {{&S15387[44], 21}, {NULL}};
const Segment SGI125323[] = {{&S15387[30], 6}, {&S15387[52], 13}, {NULL}};
const Segment SGI125324[] = {{&S15387[31], 3}, {NULL}};
const Segment SGI125325[] = {{&S15387[79], 5}, {NULL}};
const Segment SGI125326[] = {{&S103965[44], 11}, {NULL}};
const Segment SGI125327[] = {{&S103965[44], 23}, {NULL}};
const Segment SGI125328[] = {{&S103965[30], 6}, {NULL}};
const Segment SGI125329[] = {{&S103965[112], 2}, {NULL}};
const Segment SGI125330[] = {{&S103965[81], 5}, {NULL}};
const Segment SGI125331[] = {{&S15418[38], 29}, {NULL}};
const Segment SGI125332[] = {{&S15418[31], 6}, {NULL}};
const Segment SGI125333[] = {{&S103993[45], 11}, {NULL}};
const Segment SGI125334[] = {{&S103993[30], 6}, {NULL}};
const Segment SGI125335[] = {{&S103993[31], 3}, {NULL}};
const Segment SGI125336[] = {{&S103993[82], 5}, {NULL}};
const Segment SGI125337[] = {{&S103993[140], 13}, {NULL}};
const Segment SGI125338[] = {{&S15459[22], 9}, {NULL}};
const Segment SGI125339[] = {{&S15459[44], 20}, {NULL}};
const Segment SGI125340[] = {{&S15459[30], 6}, {&S15459[52], 12}, {NULL}};
const Segment SGI125341[] = {{&S15459[31], 3}, {NULL}};
const Segment SGI125342[] = {{&S15459[78], 5}, {NULL}};
const Segment SGI125343[] = {{&S14444[42], 4}, {NULL}};
const Segment SGI125344[] = {{&S14444[42], 16}, {NULL}};
const Segment SGI125345[] = {{&S14444[30], 4}, {NULL}};
const Segment SGI125346[] = {{&S104041[44], 10}, {NULL}};
const Segment SGI125347[] = {{&S104041[44], 22}, {NULL}};
const Segment SGI125348[] = {{&S14444[107], 3}, {NULL}};
const Segment SGI125349[] = {{&S104041[30], 6}, {NULL}};
const Segment SGI125350[] = {{&S14444[72], 6}, {NULL}};
const Segment SGI125351[] = {{&S104041[31], 3}, {NULL}};
const Segment SGI125352[] = {{&S104041[87], 6}, {NULL}};
const Segment SGI125353[] = {{&S104041[66], 8}, {NULL}};
const Segment SGI125354[] = {{&S104054[45], 10}, {NULL}};
const Segment SGI125355[] = {{&S104054[30], 6}, {NULL}};
const Segment SGI125356[] = {{&S104054[31], 3}, {NULL}};
const Segment SGI125357[] = {{&S104054[81], 5}, {NULL}};
const Segment SGI125358[] = {{&S104054[139], 13}, {NULL}};
const Segment SGI125359[] = {{&S104094[44], 10}, {NULL}};
const Segment SGI125360[] = {{&S104094[44], 22}, {NULL}};
const Segment SGI125361[] = {{&S104094[30], 6}, {NULL}};
const Segment SGI125362[] = {{&S104094[31], 3}, {NULL}};
const Segment SGI125363[] = {{&S104094[80], 5}, {NULL}};
const Segment SGI125364[] = {{&S104094[117], 14}, {NULL}};
const Segment SGI125365[] = {{&S13056[44], 23}, {NULL}};
const Segment SGI125366[] = {{&S13056[44], 35}, {NULL}};
const Segment SGI125367[] = {{&S13056[30], 6}, {NULL}};
const Segment SGI125368[] = {{&S13056[128], 3}, {NULL}};
const Segment SGI125369[] = {{&S13056[93], 6}, {NULL}};
const Segment SGI125370[] = {{&S13057[44], 18}, {NULL}};
const Segment SGI125371[] = {{&S13057[44], 30}, {NULL}};
const Segment SGI125372[] = {{&S13057[30], 6}, {NULL}};
const Segment SGI125373[] = {{&S13057[123], 3}, {NULL}};
const Segment SGI125374[] = {{&S13057[88], 6}, {NULL}};
const Segment SGI125375[] = {{&S13058[44], 9}, {NULL}};
const Segment SGI125376[] = {{&S13058[44], 21}, {NULL}};
const Segment SGI125377[] = {{&S13058[30], 6}, {NULL}};
const Segment SGI125378[] = {{&S13058[114], 3}, {NULL}};
const Segment SGI125379[] = {{&S13058[79], 6}, {NULL}};
const Segment SGI125380[] = {{&S13059[44], 19}, {NULL}};
const Segment SGI125381[] = {{&S13059[44], 31}, {NULL}};
const Segment SGI125382[] = {{&S13059[30], 6}, {NULL}};
const Segment SGI125383[] = {{&S13059[124], 3}, {NULL}};
const Segment SGI125384[] = {{&S13059[89], 6}, {NULL}};
const Segment SGI125385[] = {{&S13060[44], 21}, {NULL}};
const Segment SGI125386[] = {{&S13060[44], 46}, {NULL}};
const Segment SGI125387[] = {{&S13060[30], 6}, {NULL}};
const Segment SGI125388[] = {{&S13060[139], 3}, {NULL}};
const Segment SGI125389[] = {{&S13060[104], 6}, {NULL}};
const Segment SGI125390[] = {{&S13062[44], 10}, {NULL}};
const Segment SGI125391[] = {{&S13062[44], 22}, {NULL}};
const Segment SGI125392[] = {{&S13062[30], 6}, {NULL}};
const Segment SGI125393[] = {{&S13062[115], 3}, {NULL}};
const Segment SGI125394[] = {{&S13062[80], 6}, {NULL}};
const Segment SGI125395[] = {{&S13063[44], 12}, {NULL}};
const Segment SGI125396[] = {{&S13063[44], 24}, {NULL}};
const Segment SGI125397[] = {{&S13063[30], 6}, {NULL}};
const Segment SGI125398[] = {{&S13063[117], 3}, {NULL}};
const Segment SGI125399[] = {{&S13063[82], 6}, {NULL}};
const Segment SGI125400[] = {{&S13064[44], 21}, {NULL}};
const Segment SGI125401[] = {{&S13064[44], 33}, {NULL}};
const Segment SGI125402[] = {{&S13064[30], 6}, {NULL}};
const Segment SGI125403[] = {{&S13064[126], 3}, {NULL}};
const Segment SGI125404[] = {{&S13064[91], 6}, {NULL}};
const Segment SGI125405[] = {{&S13067[44], 18}, {NULL}};
const Segment SGI125406[] = {{&S13067[44], 30}, {NULL}};
const Segment SGI125407[] = {{&S13067[30], 6}, {NULL}};
const Segment SGI125408[] = {{&S13067[123], 3}, {NULL}};
const Segment SGI125409[] = {{&S13067[88], 6}, {NULL}};
const Segment SGI125410[] = {{&S13068[44], 7}, {NULL}};
const Segment SGI125411[] = {{&S13068[44], 19}, {NULL}};
const Segment SGI125412[] = {{&S13068[30], 6}, {NULL}};
const Segment SGI125413[] = {{&S13068[112], 3}, {NULL}};
const Segment SGI125414[] = {{&S13068[77], 6}, {NULL}};
const Segment SGI125415[] = {{&S13069[44], 4}, {NULL}};
const Segment SGI125416[] = {{&S13069[44], 16}, {NULL}};
const Segment SGI125417[] = {{&S13069[30], 6}, {NULL}};
const Segment SGI125418[] = {{&S13069[109], 3}, {NULL}};
const Segment SGI125419[] = {{&S13069[74], 6}, {NULL}};
const Segment SGI125420[] = {{&S13070[44], 8}, {NULL}};
const Segment SGI125421[] = {{&S13070[44], 20}, {NULL}};
const Segment SGI125422[] = {{&S13070[30], 6}, {NULL}};
const Segment SGI125423[] = {{&S13070[113], 3}, {NULL}};
const Segment SGI125424[] = {{&S13070[78], 6}, {NULL}};
const Segment SGI125425[] = {{&S13071[44], 10}, {NULL}};
const Segment SGI125426[] = {{&S13071[44], 22}, {NULL}};
const Segment SGI125427[] = {{&S13071[30], 6}, {NULL}};
const Segment SGI125428[] = {{&S13071[115], 3}, {NULL}};
const Segment SGI125429[] = {{&S13071[80], 6}, {NULL}};
const Segment SGI125430[] = {{&S13072[44], 7}, {NULL}};
const Segment SGI125431[] = {{&S13072[44], 19}, {NULL}};
const Segment SGI125432[] = {{&S13072[30], 6}, {NULL}};
const Segment SGI125433[] = {{&S13072[112], 3}, {NULL}};
const Segment SGI125434[] = {{&S13072[77], 6}, {NULL}};
const Segment SGI125435[] = {{&S13074[44], 18}, {NULL}};
const Segment SGI125436[] = {{&S13074[44], 30}, {NULL}};
const Segment SGI125437[] = {{&S13074[30], 6}, {NULL}};
const Segment SGI125438[] = {{&S13074[123], 3}, {NULL}};
const Segment SGI125439[] = {{&S13074[88], 6}, {NULL}};
const Segment SGI125440[] = {{&S13075[44], 13}, {NULL}};
const Segment SGI125441[] = {{&S13075[44], 25}, {NULL}};
const Segment SGI125442[] = {{&S13075[30], 6}, {NULL}};
const Segment SGI125443[] = {{&S13075[118], 3}, {NULL}};
const Segment SGI125444[] = {{&S13075[83], 6}, {NULL}};
const Segment SGI125445[] = {{&S13076[44], 5}, {NULL}};
const Segment SGI125446[] = {{&S13076[44], 17}, {NULL}};
const Segment SGI125447[] = {{&S13076[30], 6}, {NULL}};
const Segment SGI125448[] = {{&S13076[110], 3}, {NULL}};
const Segment SGI125449[] = {{&S13076[75], 6}, {NULL}};
const Segment SGI125450[] = {{&S13077[44], 8}, {NULL}};
const Segment SGI125451[] = {{&S13077[44], 20}, {NULL}};
const Segment SGI125452[] = {{&S13077[30], 6}, {NULL}};
const Segment SGI125453[] = {{&S13077[113], 3}, {NULL}};
const Segment SGI125454[] = {{&S13077[78], 6}, {NULL}};
const Segment SGI125455[] = {{&S13078[44], 8}, {NULL}};
const Segment SGI125456[] = {{&S13078[44], 20}, {NULL}};
const Segment SGI125457[] = {{&S13078[30], 6}, {NULL}};
const Segment SGI125458[] = {{&S13078[113], 3}, {NULL}};
const Segment SGI125459[] = {{&S13078[78], 6}, {NULL}};
const Segment SGI125460[] = {{&S13079[44], 8}, {NULL}};
const Segment SGI125461[] = {{&S13079[44], 20}, {NULL}};
const Segment SGI125462[] = {{&S13079[30], 6}, {NULL}};
const Segment SGI125463[] = {{&S13079[113], 3}, {NULL}};
const Segment SGI125464[] = {{&S13079[78], 6}, {NULL}};
const Segment SGI125465[] = {{&S13080[44], 15}, {NULL}};
const Segment SGI125466[] = {{&S13080[44], 27}, {NULL}};
const Segment SGI125467[] = {{&S13080[30], 6}, {NULL}};
const Segment SGI125468[] = {{&S13080[120], 3}, {NULL}};
const Segment SGI125469[] = {{&S13080[85], 6}, {NULL}};
const Segment SGI125470[] = {{&S13081[44], 8}, {NULL}};
const Segment SGI125471[] = {{&S13081[44], 20}, {NULL}};
const Segment SGI125472[] = {{&S13081[30], 6}, {NULL}};
const Segment SGI125473[] = {{&S13081[113], 3}, {NULL}};
const Segment SGI125474[] = {{&S13081[78], 6}, {NULL}};
const Segment SGI125475[] = {{&S13082[44], 9}, {NULL}};
const Segment SGI125476[] = {{&S13082[44], 23}, {NULL}};
const Segment SGI125477[] = {{&S13082[30], 6}, {NULL}};
const Segment SGI125478[] = {{&S13082[116], 3}, {NULL}};
const Segment SGI125479[] = {{&S13082[81], 6}, {NULL}};
const Segment SGI125480[] = {{&S13083[44], 20}, {NULL}};
const Segment SGI125481[] = {{&S13083[44], 32}, {NULL}};
const Segment SGI125482[] = {{&S13083[30], 6}, {NULL}};
const Segment SGI125483[] = {{&S13083[125], 3}, {NULL}};
const Segment SGI125484[] = {{&S13083[90], 6}, {NULL}};
const Segment SGI125485[] = {{&S13084[44], 9}, {NULL}};
const Segment SGI125486[] = {{&S13084[44], 21}, {NULL}};
const Segment SGI125487[] = {{&S13084[30], 6}, {NULL}};
const Segment SGI125488[] = {{&S13084[114], 3}, {NULL}};
const Segment SGI125489[] = {{&S13084[79], 6}, {NULL}};
const Segment SGI125490[] = {{&S13085[44], 10}, {NULL}};
const Segment SGI125491[] = {{&S13085[44], 22}, {NULL}};
const Segment SGI125492[] = {{&S13085[30], 6}, {NULL}};
const Segment SGI125493[] = {{&S13085[115], 3}, {NULL}};
const Segment SGI125494[] = {{&S13085[80], 6}, {NULL}};
const Segment SGI125495[] = {{&S13086[44], 4}, {NULL}};
const Segment SGI125496[] = {{&S13086[44], 16}, {NULL}};
const Segment SGI125497[] = {{&S13086[30], 6}, {NULL}};
const Segment SGI125498[] = {{&S13086[109], 3}, {NULL}};
const Segment SGI125499[] = {{&S13086[74], 6}, {NULL}};
const Segment SGI125500[] = {{&S13088[44], 11}, {NULL}};
const Segment SGI125501[] = {{&S13088[44], 23}, {NULL}};
const Segment SGI125502[] = {{&S13088[30], 6}, {NULL}};
const Segment SGI125503[] = {{&S13088[116], 3}, {NULL}};
const Segment SGI125504[] = {{&S13088[81], 6}, {NULL}};
const Segment SGI125505[] = {{&S13089[44], 12}, {NULL}};
const Segment SGI125506[] = {{&S13089[44], 24}, {NULL}};
const Segment SGI125507[] = {{&S13089[30], 6}, {NULL}};
const Segment SGI125508[] = {{&S13089[117], 3}, {NULL}};
const Segment SGI125509[] = {{&S13089[82], 6}, {NULL}};
const Segment SGI125510[] = {{&S13092[44], 8}, {NULL}};
const Segment SGI125511[] = {{&S13092[44], 20}, {NULL}};
const Segment SGI125512[] = {{&S13092[30], 6}, {NULL}};
const Segment SGI125513[] = {{&S13092[113], 3}, {NULL}};
const Segment SGI125514[] = {{&S13092[78], 6}, {NULL}};
const Segment SGI125515[] = {{&S13093[44], 8}, {NULL}};
const Segment SGI125516[] = {{&S13093[44], 20}, {NULL}};
const Segment SGI125517[] = {{&S13093[30], 6}, {NULL}};
const Segment SGI125518[] = {{&S13093[113], 3}, {NULL}};
const Segment SGI125519[] = {{&S13093[78], 6}, {NULL}};
const Segment SGI125520[] = {{&S13094[44], 7}, {NULL}};
const Segment SGI125521[] = {{&S13094[44], 19}, {NULL}};
const Segment SGI125522[] = {{&S13094[30], 6}, {NULL}};
const Segment SGI125523[] = {{&S13094[112], 3}, {NULL}};
const Segment SGI125524[] = {{&S13094[77], 6}, {NULL}};
const Segment SGI125525[] = {{&S13095[44], 8}, {NULL}};
const Segment SGI125526[] = {{&S13095[44], 20}, {NULL}};
const Segment SGI125527[] = {{&S13095[30], 6}, {NULL}};
const Segment SGI125528[] = {{&S13095[113], 3}, {NULL}};
const Segment SGI125529[] = {{&S13095[78], 6}, {NULL}};
const Segment SGI125530[] = {{&S13096[44], 13}, {NULL}};
const Segment SGI125531[] = {{&S13096[44], 25}, {NULL}};
const Segment SGI125532[] = {{&S13096[30], 6}, {NULL}};
const Segment SGI125533[] = {{&S13096[118], 3}, {NULL}};
const Segment SGI125534[] = {{&S13096[83], 6}, {NULL}};
const Segment SGI125535[] = {{&S13097[44], 10}, {NULL}};
const Segment SGI125536[] = {{&S13097[44], 22}, {NULL}};
const Segment SGI125537[] = {{&S13097[30], 6}, {NULL}};
const Segment SGI125538[] = {{&S13097[115], 3}, {NULL}};
const Segment SGI125539[] = {{&S13097[80], 6}, {NULL}};
const Segment SGI125540[] = {{&S13098[44], 7}, {NULL}};
const Segment SGI125541[] = {{&S13098[44], 19}, {NULL}};
const Segment SGI125542[] = {{&S13098[30], 6}, {NULL}};
const Segment SGI125543[] = {{&S13098[112], 3}, {NULL}};
const Segment SGI125544[] = {{&S13098[77], 6}, {NULL}};
const Segment SGI125545[] = {{&S13099[44], 13}, {NULL}};
const Segment SGI125546[] = {{&S13099[44], 25}, {NULL}};
const Segment SGI125547[] = {{&S13099[30], 6}, {NULL}};
const Segment SGI125548[] = {{&S13099[118], 3}, {NULL}};
const Segment SGI125549[] = {{&S13099[83], 6}, {NULL}};
const Segment SGI125550[] = {{&S13100[44], 6}, {NULL}};
const Segment SGI125551[] = {{&S13100[44], 18}, {NULL}};
const Segment SGI125552[] = {{&S13100[30], 6}, {NULL}};
const Segment SGI125553[] = {{&S13100[111], 3}, {NULL}};
const Segment SGI125554[] = {{&S13100[76], 6}, {NULL}};
const Segment SGI125555[] = {{&S13101[44], 16}, {NULL}};
const Segment SGI125556[] = {{&S13101[44], 28}, {NULL}};
const Segment SGI125557[] = {{&S13101[30], 6}, {NULL}};
const Segment SGI125558[] = {{&S13101[121], 3}, {NULL}};
const Segment SGI125559[] = {{&S13101[86], 6}, {NULL}};
const Segment SGI125560[] = {{&S13102[44], 7}, {NULL}};
const Segment SGI125561[] = {{&S13102[44], 19}, {NULL}};
const Segment SGI125562[] = {{&S13102[30], 6}, {NULL}};
const Segment SGI125563[] = {{&S13102[112], 3}, {NULL}};
const Segment SGI125564[] = {{&S13102[77], 6}, {NULL}};
const Segment SGI125565[] = {{&S13103[44], 9}, {NULL}};
const Segment SGI125566[] = {{&S13103[44], 21}, {NULL}};
const Segment SGI125567[] = {{&S13103[30], 6}, {NULL}};
const Segment SGI125568[] = {{&S13103[114], 3}, {NULL}};
const Segment SGI125569[] = {{&S13103[79], 6}, {NULL}};
const Segment SGI125570[] = {{&S21250[44], 7}, {NULL}};
const Segment SGI125571[] = {{&S21250[44], 24}, {NULL}};
const Segment SGI125572[] = {{&S21250[30], 6}, {NULL}};
const Segment SGI125573[] = {{&S21250[31], 3}, {NULL}};
const Segment SGI125574[] = {{&S21250[82], 6}, {NULL}};
const Segment SGI125575[] = {{&S13104[44], 8}, {NULL}};
const Segment SGI125576[] = {{&S13104[44], 20}, {NULL}};
const Segment SGI125577[] = {{&S13104[30], 6}, {NULL}};
const Segment SGI125578[] = {{&S13104[113], 3}, {NULL}};
const Segment SGI125579[] = {{&S13104[78], 6}, {NULL}};
const Segment SGI125580[] = {{&S21251[44], 7}, {NULL}};
const Segment SGI125581[] = {{&S21251[44], 22}, {NULL}};
const Segment SGI125582[] = {{&S21251[30], 6}, {NULL}};
const Segment SGI125583[] = {{&S21251[31], 3}, {NULL}};
const Segment SGI125584[] = {{&S13105[44], 6}, {NULL}};
const Segment SGI125585[] = {{&S21251[80], 6}, {NULL}};
const Segment SGI125586[] = {{&S13105[44], 18}, {NULL}};
const Segment SGI125587[] = {{&S13105[30], 6}, {NULL}};
const Segment SGI125588[] = {{&S13105[111], 3}, {NULL}};
const Segment SGI125589[] = {{&S13105[76], 6}, {NULL}};
const Segment SGI125590[] = {{&S21252[44], 7}, {NULL}};
const Segment SGI125591[] = {{&S21252[44], 24}, {NULL}};
const Segment SGI125592[] = {{&S21252[30], 6}, {NULL}};
const Segment SGI125593[] = {{&S21252[31], 3}, {NULL}};
const Segment SGI125594[] = {{&S13106[44], 15}, {NULL}};
const Segment SGI125595[] = {{&S13106[44], 27}, {NULL}};
const Segment SGI125596[] = {{&S21252[82], 6}, {NULL}};
const Segment SGI125597[] = {{&S13106[30], 6}, {NULL}};
const Segment SGI125598[] = {{&S13106[120], 3}, {NULL}};
const Segment SGI125599[] = {{&S13106[85], 6}, {NULL}};
const Segment SGI125600[] = {{&S21253[44], 7}, {NULL}};
const Segment SGI125601[] = {{&S21253[44], 24}, {NULL}};
const Segment SGI125602[] = {{&S21253[30], 6}, {NULL}};
const Segment SGI125603[] = {{&S13107[44], 4}, {NULL}};
const Segment SGI125604[] = {{&S13107[44], 16}, {NULL}};
const Segment SGI125605[] = {{&S13107[30], 6}, {NULL}};
const Segment SGI125606[] = {{&S21253[85], 6}, {NULL}};
const Segment SGI125607[] = {{&S13107[109], 3}, {NULL}};
const Segment SGI125608[] = {{&S13107[74], 6}, {NULL}};
const Segment SGI125609[] = {{&S21254[44], 7}, {NULL}};
const Segment SGI125610[] = {{&S21254[44], 24}, {NULL}};
const Segment SGI125611[] = {{&S13108[44], 7}, {NULL}};
const Segment SGI125612[] = {{&S21254[30], 6}, {NULL}};
const Segment SGI125613[] = {{&S13108[44], 19}, {NULL}};
const Segment SGI125614[] = {{&S13108[30], 6}, {NULL}};
const Segment SGI125615[] = {{&S21254[31], 3}, {NULL}};
const Segment SGI125616[] = {{&S13108[112], 3}, {NULL}};
const Segment SGI125617[] = {{&S21254[82], 6}, {NULL}};
const Segment SGI125618[] = {{&S13108[77], 6}, {NULL}};
const Segment SGI125619[] = {{&S13109[44], 9}, {NULL}};
const Segment SGI125620[] = {{&S13109[44], 21}, {NULL}};
const Segment SGI125621[] = {{&S13109[30], 6}, {NULL}};
const Segment SGI125622[] = {{&S13109[114], 3}, {NULL}};
const Segment SGI125623[] = {{&S13109[79], 6}, {NULL}};
const Segment SGI125624[] = {{&S13110[44], 10}, {NULL}};
const Segment SGI125625[] = {{&S13110[44], 22}, {NULL}};
const Segment SGI125626[] = {{&S13110[30], 6}, {NULL}};
const Segment SGI125627[] = {{&S13110[115], 3}, {NULL}};
const Segment SGI125628[] = {{&S13110[80], 6}, {NULL}};
const Segment SGI125629[] = {{&S13111[44], 12}, {NULL}};
const Segment SGI125630[] = {{&S21260[38], 7}, {NULL}};
const Segment SGI125631[] = {{&S13111[44], 24}, {NULL}};
const Segment SGI125632[] = {{&S21260[31], 6}, {NULL}};
const Segment SGI125633[] = {{&S13111[30], 6}, {NULL}};
const Segment SGI125634[] = {{&S13111[117], 3}, {NULL}};
const Segment SGI125635[] = {{&S13111[82], 6}, {NULL}};
const Segment SGI125636[] = {{&S21261[44], 7}, {NULL}};
const Segment SGI125637[] = {{&S21261[44], 24}, {NULL}};
const Segment SGI125638[] = {{&S21261[30], 6}, {NULL}};
const Segment SGI125639[] = {{&S13112[44], 31}, {NULL}};
const Segment SGI125640[] = {{&S13112[44], 43}, {NULL}};
const Segment SGI125641[] = {{&S21261[31], 3}, {NULL}};
const Segment SGI125642[] = {{&S13112[30], 6}, {NULL}};
const Segment SGI125643[] = {{&S21261[82], 6}, {NULL}};
const Segment SGI125644[] = {{&S13112[136], 3}, {NULL}};
const Segment SGI125645[] = {{&S13112[101], 6}, {NULL}};
const Segment SGI125646[] = {{&S21262[44], 7}, {NULL}};
const Segment SGI125647[] = {{&S21262[44], 24}, {NULL}};
const Segment SGI125648[] = {{&S21262[30], 6}, {NULL}};
const Segment SGI125649[] = {{&S21262[85], 6}, {NULL}};
const Segment SGI125650[] = {{&S13114[44], 11}, {NULL}};
const Segment SGI125651[] = {{&S13114[44], 23}, {NULL}};
const Segment SGI125652[] = {{&S13114[30], 6}, {NULL}};
const Segment SGI125653[] = {{&S13114[116], 3}, {NULL}};
const Segment SGI125654[] = {{&S13114[81], 6}, {NULL}};
const Segment SGI125655[] = {{&S21264[44], 7}, {NULL}};
const Segment SGI125656[] = {{&S21264[44], 23}, {NULL}};
const Segment SGI125657[] = {{&S21264[30], 6}, {NULL}};
const Segment SGI125658[] = {{&S21264[31], 3}, {NULL}};
const Segment SGI125659[] = {{&S21264[81], 6}, {NULL}};
const Segment SGI125660[] = {{&S13116[44], 8}, {NULL}};
const Segment SGI125661[] = {{&S13116[44], 20}, {NULL}};
const Segment SGI125662[] = {{&S13116[30], 6}, {NULL}};
const Segment SGI125663[] = {{&S13116[113], 3}, {NULL}};
const Segment SGI125664[] = {{&S13116[78], 6}, {NULL}};
const Segment SGI125665[] = {{&S21265[38], 7}, {NULL}};
const Segment SGI125666[] = {{&S21265[31], 6}, {NULL}};
const Segment SGI125667[] = {{&S21266[44], 7}, {NULL}};
const Segment SGI125668[] = {{&S21266[44], 39}, {NULL}};
const Segment SGI125669[] = {{&S21266[30], 6}, {NULL}};
const Segment SGI125670[] = {{&S13118[44], 8}, {NULL}};
const Segment SGI125671[] = {{&S13118[44], 20}, {NULL}};
const Segment SGI125672[] = {{&S13118[30], 6}, {NULL}};
const Segment SGI125673[] = {{&S21266[31], 3}, {NULL}};
const Segment SGI125674[] = {{&S21266[97], 6}, {NULL}};
const Segment SGI125675[] = {{&S13118[113], 3}, {NULL}};
const Segment SGI125676[] = {{&S13118[78], 6}, {NULL}};
const Segment SGI125677[] = {{&S21267[44], 7}, {NULL}};
const Segment SGI125678[] = {{&S21267[44], 39}, {NULL}};
const Segment SGI125679[] = {{&S13119[44], 18}, {NULL}};
const Segment SGI125680[] = {{&S21267[30], 6}, {NULL}};
const Segment SGI125681[] = {{&S13119[44], 30}, {NULL}};
const Segment SGI125682[] = {{&S13119[30], 6}, {NULL}};
const Segment SGI125683[] = {{&S21267[31], 3}, {NULL}};
const Segment SGI125684[] = {{&S13119[123], 3}, {NULL}};
const Segment SGI125685[] = {{&S21267[97], 6}, {NULL}};
const Segment SGI125686[] = {{&S13119[88], 6}, {NULL}};
const Segment SGI125687[] = {{&S21268[44], 7}, {NULL}};
const Segment SGI125688[] = {{&S13120[44], 9}, {NULL}};
const Segment SGI125689[] = {{&S21268[44], 38}, {NULL}};
const Segment SGI125690[] = {{&S21268[30], 6}, {NULL}};
const Segment SGI125691[] = {{&S13120[44], 21}, {NULL}};
const Segment SGI125692[] = {{&S13120[30], 6}, {NULL}};
const Segment SGI125693[] = {{&S21268[31], 3}, {NULL}};
const Segment SGI125694[] = {{&S13120[114], 3}, {NULL}};
const Segment SGI125695[] = {{&S21268[96], 6}, {NULL}};
const Segment SGI125696[] = {{&S13120[79], 6}, {NULL}};
const Segment SGI125697[] = {{&S21269[42], 7}, {NULL}};
const Segment SGI125698[] = {{&S13121[44], 5}, {NULL}};
const Segment SGI125699[] = {{&S13121[44], 17}, {NULL}};
const Segment SGI125700[] = {{&S21269[30], 6}, {NULL}};
const Segment SGI125701[] = {{&S13121[30], 6}, {NULL}};
const Segment SGI125702[] = {{&S21269[31], 3}, {NULL}};
const Segment SGI125703[] = {{&S13121[110], 3}, {NULL}};
const Segment SGI125704[] = {{&S21269[80], 6}, {NULL}};
const Segment SGI125705[] = {{&S13121[75], 6}, {NULL}};
const Segment SGI125706[] = {{&S13122[44], 9}, {NULL}};
const Segment SGI125707[] = {{&S13122[44], 21}, {NULL}};
const Segment SGI125708[] = {{&S13122[30], 6}, {NULL}};
const Segment SGI125709[] = {{&S13122[114], 3}, {NULL}};
const Segment SGI125710[] = {{&S13122[79], 6}, {NULL}};
const Segment SGI125711[] = {{&S13123[44], 11}, {NULL}};
const Segment SGI125712[] = {{&S13123[44], 23}, {NULL}};
const Segment SGI125713[] = {{&S13123[30], 6}, {NULL}};
const Segment SGI125714[] = {{&S13123[116], 3}, {NULL}};
const Segment SGI125715[] = {{&S13123[81], 6}, {NULL}};
const Segment SGI125716[] = {{&S13124[44], 17}, {NULL}};
const Segment SGI125717[] = {{&S13124[44], 29}, {NULL}};
const Segment SGI125718[] = {{&S13124[30], 6}, {NULL}};
const Segment SGI125719[] = {{&S13124[122], 3}, {NULL}};
const Segment SGI125720[] = {{&S13124[87], 6}, {NULL}};
const Segment SGI125721[] = {{&S13125[44], 10}, {NULL}};
const Segment SGI125722[] = {{&S13125[44], 22}, {NULL}};
const Segment SGI125723[] = {{&S13125[30], 6}, {NULL}};
const Segment SGI125724[] = {{&S13125[115], 3}, {NULL}};
const Segment SGI125725[] = {{&S13125[80], 6}, {NULL}};
const Segment SGI125726[] = {{&S13126[44], 6}, {NULL}};
const Segment SGI125727[] = {{&S13126[44], 19}, {NULL}};
const Segment SGI125728[] = {{&S13126[30], 6}, {NULL}};
const Segment SGI125729[] = {{&S13126[112], 3}, {NULL}};
const Segment SGI125730[] = {{&S13126[77], 6}, {NULL}};
const Segment SGI125731[] = {{&S13127[44], 17}, {NULL}};
const Segment SGI125732[] = {{&S13127[44], 29}, {NULL}};
const Segment SGI125733[] = {{&S13127[30], 6}, {NULL}};
const Segment SGI125734[] = {{&S13127[122], 3}, {NULL}};
const Segment SGI125735[] = {{&S13127[87], 6}, {NULL}};
const Segment SGI125736[] = {{&S13128[44], 7}, {NULL}};
const Segment SGI125737[] = {{&S13128[44], 18}, {NULL}};
const Segment SGI125738[] = {{&S13128[30], 6}, {NULL}};
const Segment SGI125739[] = {{&S13128[111], 3}, {NULL}};
const Segment SGI125740[] = {{&S13128[76], 6}, {NULL}};
const Segment SGI125741[] = {{&S13129[44], 16}, {NULL}};
const Segment SGI125742[] = {{&S13129[44], 28}, {NULL}};
const Segment SGI125743[] = {{&S13129[30], 6}, {NULL}};
const Segment SGI125744[] = {{&S13129[121], 3}, {NULL}};
const Segment SGI125745[] = {{&S13129[86], 6}, {NULL}};
const Segment SGI125746[] = {{&S13130[44], 11}, {NULL}};
const Segment SGI125747[] = {{&S13130[44], 23}, {NULL}};
const Segment SGI125748[] = {{&S13130[30], 6}, {NULL}};
const Segment SGI125749[] = {{&S13130[116], 3}, {NULL}};
const Segment SGI125750[] = {{&S13130[81], 6}, {NULL}};
const Segment SGI125751[] = {{&S13131[44], 5}, {NULL}};
const Segment SGI125752[] = {{&S13131[44], 17}, {NULL}};
const Segment SGI125753[] = {{&S13131[30], 6}, {NULL}};
const Segment SGI125754[] = {{&S13131[110], 3}, {NULL}};
const Segment SGI125755[] = {{&S13131[75], 6}, {NULL}};
const Segment SGI125756[] = {{&S13132[44], 7}, {NULL}};
const Segment SGI125757[] = {{&S13132[44], 19}, {NULL}};
const Segment SGI125758[] = {{&S13132[30], 6}, {NULL}};
const Segment SGI125759[] = {{&S13132[112], 3}, {NULL}};
const Segment SGI125760[] = {{&S13132[77], 6}, {NULL}};
const Segment SGI125761[] = {{&S13133[44], 7}, {NULL}};
const Segment SGI125762[] = {{&S13133[44], 19}, {NULL}};
const Segment SGI125763[] = {{&S13133[30], 6}, {NULL}};
const Segment SGI125764[] = {{&S13133[112], 3}, {NULL}};
const Segment SGI125765[] = {{&S13133[77], 6}, {NULL}};
const Segment SGI125766[] = {{&S13134[44], 7}, {NULL}};
const Segment SGI125767[] = {{&S13134[44], 19}, {NULL}};
const Segment SGI125768[] = {{&S13134[30], 6}, {NULL}};
const Segment SGI125769[] = {{&S13134[112], 3}, {NULL}};
const Segment SGI125770[] = {{&S13134[77], 6}, {NULL}};
const Segment SGI125771[] = {{&S13135[44], 13}, {NULL}};
const Segment SGI125772[] = {{&S13135[44], 25}, {NULL}};
const Segment SGI125773[] = {{&S13135[30], 6}, {NULL}};
const Segment SGI125774[] = {{&S13135[118], 3}, {NULL}};
const Segment SGI125775[] = {{&S13135[83], 6}, {NULL}};
const Segment SGI125776[] = {{&S13136[44], 13}, {NULL}};
const Segment SGI125777[] = {{&S13136[44], 25}, {NULL}};
const Segment SGI125778[] = {{&S13136[30], 6}, {NULL}};
const Segment SGI125779[] = {{&S13136[118], 3}, {NULL}};
const Segment SGI125780[] = {{&S13136[83], 6}, {NULL}};
const Segment SGI125781[] = {{&S13137[44], 10}, {NULL}};
const Segment SGI125782[] = {{&S13137[44], 22}, {NULL}};
const Segment SGI125783[] = {{&S13137[30], 6}, {NULL}};
const Segment SGI125784[] = {{&S13137[115], 3}, {NULL}};
const Segment SGI125785[] = {{&S13137[80], 6}, {NULL}};
const Segment SGI125786[] = {{&S13138[44], 8}, {NULL}};
const Segment SGI125787[] = {{&S13138[44], 20}, {NULL}};
const Segment SGI125788[] = {{&S13138[30], 6}, {NULL}};
const Segment SGI125789[] = {{&S13138[113], 3}, {NULL}};
const Segment SGI125790[] = {{&S13138[78], 6}, {NULL}};
const Segment SGI125791[] = {{&S13139[44], 19}, {NULL}};
const Segment SGI125792[] = {{&S13139[44], 31}, {NULL}};
const Segment SGI125793[] = {{&S13139[30], 6}, {NULL}};
const Segment SGI125794[] = {{&S13139[124], 3}, {NULL}};
const Segment SGI125795[] = {{&S13139[89], 6}, {NULL}};
const Segment SGI125796[] = {{&S13140[44], 19}, {NULL}};
const Segment SGI125797[] = {{&S13140[44], 31}, {NULL}};
const Segment SGI125798[] = {{&S13140[30], 6}, {NULL}};
const Segment SGI125799[] = {{&S13140[124], 3}, {NULL}};
const Segment SGI125800[] = {{&S13140[89], 6}, {NULL}};
const Segment SGI125801[] = {{&S13141[44], 10}, {NULL}};
const Segment SGI125802[] = {{&S13141[44], 22}, {NULL}};
const Segment SGI125803[] = {{&S13141[30], 6}, {NULL}};
const Segment SGI125804[] = {{&S13141[115], 3}, {NULL}};
const Segment SGI125805[] = {{&S13141[80], 6}, {NULL}};
const Segment SGI125806[] = {{&S13142[44], 9}, {NULL}};
const Segment SGI125807[] = {{&S13142[44], 21}, {NULL}};
const Segment SGI125808[] = {{&S13142[30], 6}, {NULL}};
const Segment SGI125809[] = {{&S13142[114], 3}, {NULL}};
const Segment SGI125810[] = {{&S13142[79], 6}, {NULL}};
const Segment SGI125811[] = {{&S13143[44], 11}, {NULL}};
const Segment SGI125812[] = {{&S13143[44], 23}, {NULL}};
const Segment SGI125813[] = {{&S13143[30], 6}, {NULL}};
const Segment SGI125814[] = {{&S13143[116], 3}, {NULL}};
const Segment SGI125815[] = {{&S13143[81], 6}, {NULL}};
const Segment SGI125816[] = {{&S13145[44], 6}, {NULL}};
const Segment SGI125817[] = {{&S13145[44], 18}, {NULL}};
const Segment SGI125818[] = {{&S13145[30], 6}, {NULL}};
const Segment SGI125819[] = {{&S13145[111], 3}, {NULL}};
const Segment SGI125820[] = {{&S13145[76], 6}, {NULL}};
const Segment SGI125821[] = {{&S13146[44], 7}, {NULL}};
const Segment SGI125822[] = {{&S13146[44], 19}, {NULL}};
const Segment SGI125823[] = {{&S13146[30], 6}, {NULL}};
const Segment SGI125824[] = {{&S13146[112], 3}, {NULL}};
const Segment SGI125825[] = {{&S13146[77], 6}, {NULL}};
const Segment SGI125826[] = {{&S13147[44], 6}, {NULL}};
const Segment SGI125827[] = {{&S13147[44], 18}, {NULL}};
const Segment SGI125828[] = {{&S13147[30], 6}, {NULL}};
const Segment SGI125829[] = {{&S104344[44], 12}, {NULL}};
const Segment SGI125830[] = {{&S13147[111], 3}, {NULL}};
const Segment SGI125831[] = {{&S104344[44], 24}, {NULL}};
const Segment SGI125832[] = {{&S13147[76], 6}, {NULL}};
const Segment SGI125833[] = {{&S104344[30], 6}, {NULL}};
const Segment SGI125834[] = {{&S104344[31], 3}, {NULL}};
const Segment SGI125835[] = {{&S104344[82], 6}, {NULL}};
const Segment SGI125836[] = {{&S13148[23], 15}, {NULL}};
const Segment SGI125837[] = {{&S13148[31], 19}, {NULL}};
const Segment SGI125838[] = {{&S13149[44], 9}, {NULL}};
const Segment SGI125839[] = {{&S13149[44], 21}, {NULL}};
const Segment SGI125840[] = {{&S13149[30], 6}, {NULL}};
const Segment SGI125841[] = {{&S13149[114], 3}, {NULL}};
const Segment SGI125842[] = {{&S13149[79], 6}, {NULL}};
const Segment SGI125843[] = {{&S13150[44], 4}, {NULL}};
const Segment SGI125844[] = {{&S13150[44], 16}, {NULL}};
const Segment SGI125845[] = {{&S13150[30], 6}, {NULL}};
const Segment SGI125846[] = {{&S13150[109], 3}, {NULL}};
const Segment SGI125847[] = {{&S13150[74], 6}, {NULL}};
const Segment SGI125848[] = {{&S13151[44], 12}, {NULL}};
const Segment SGI125849[] = {{&S13151[44], 24}, {NULL}};
const Segment SGI125850[] = {{&S13151[30], 6}, {NULL}};
const Segment SGI125851[] = {{&S13151[117], 3}, {NULL}};
const Segment SGI125852[] = {{&S13151[82], 6}, {NULL}};
const Segment SGI125853[] = {{&S13152[44], 10}, {NULL}};
const Segment SGI125854[] = {{&S13152[44], 22}, {NULL}};
const Segment SGI125855[] = {{&S13152[30], 6}, {NULL}};
const Segment SGI125856[] = {{&S13152[115], 3}, {NULL}};
const Segment SGI125857[] = {{&S13152[80], 6}, {NULL}};
const Segment SGI125858[] = {{&S13153[44], 8}, {NULL}};
const Segment SGI125859[] = {{&S13153[44], 20}, {NULL}};
const Segment SGI125860[] = {{&S13153[30], 6}, {NULL}};
const Segment SGI125861[] = {{&S13153[113], 3}, {NULL}};
const Segment SGI125862[] = {{&S13153[78], 6}, {NULL}};
const Segment SGI125863[] = {{&S104355[44], 11}, {NULL}};
const Segment SGI125864[] = {{&S104355[44], 23}, {NULL}};
const Segment SGI125865[] = {{&S13154[44], 7}, {NULL}};
const Segment SGI125866[] = {{&S104355[30], 6}, {NULL}};
const Segment SGI125867[] = {{&S13154[44], 19}, {NULL}};
const Segment SGI125868[] = {{&S13154[30], 6}, {NULL}};
const Segment SGI125869[] = {{&S104355[112], 2}, {NULL}};
const Segment SGI125870[] = {{&S13154[112], 3}, {NULL}};
const Segment SGI125871[] = {{&S104355[81], 5}, {NULL}};
const Segment SGI125872[] = {{&S13154[77], 6}, {NULL}};
const Segment SGI125873[] = {{&S13155[44], 8}, {NULL}};
const Segment SGI125874[] = {{&S13155[44], 20}, {NULL}};
const Segment SGI125875[] = {{&S13155[30], 6}, {NULL}};
const Segment SGI125876[] = {{&S13155[113], 3}, {NULL}};
const Segment SGI125877[] = {{&S13155[78], 6}, {NULL}};
const Segment SGI125878[] = {{&S104357[44], 10}, {NULL}};
const Segment SGI125879[] = {{&S104357[44], 22}, {NULL}};
const Segment SGI125880[] = {{&S104357[30], 6}, {NULL}};
const Segment SGI125881[] = {{&S104357[31], 3}, {NULL}};
const Segment SGI125882[] = {{&S104357[80], 6}, {NULL}};
const Segment SGI125883[] = {{&S104357[150], 3}, {NULL}};
const Segment SGI125884[] = {{&S13156[44], 6}, {NULL}};
const Segment SGI125885[] = {{&S13156[44], 18}, {NULL}};
const Segment SGI125886[] = {{&S13156[30], 6}, {NULL}};
const Segment SGI125887[] = {{&S13156[111], 3}, {NULL}};
const Segment SGI125888[] = {{&S13156[76], 6}, {NULL}};
const Segment SGI125889[] = {{&S13157[44], 10}, {NULL}};
const Segment SGI125890[] = {{&S13157[44], 22}, {NULL}};
const Segment SGI125891[] = {{&S13157[30], 6}, {NULL}};
const Segment SGI125892[] = {{&S13157[114], 3}, {NULL}};
const Segment SGI125893[] = {{&S13157[80], 5}, {NULL}};
const Segment SGI125894[] = {{&S13158[44], 8}, {NULL}};
const Segment SGI125895[] = {{&S13158[44], 20}, {NULL}};
const Segment SGI125896[] = {{&S13158[30], 6}, {NULL}};
const Segment SGI125897[] = {{&S13158[113], 3}, {NULL}};
const Segment SGI125898[] = {{&S13158[78], 6}, {NULL}};
const Segment SGI125899[] = {{&S13159[44], 14}, {NULL}};
const Segment SGI125900[] = {{&S13159[44], 26}, {NULL}};
const Segment SGI125901[] = {{&S13159[30], 6}, {NULL}};
const Segment SGI125902[] = {{&S13159[119], 3}, {NULL}};
const Segment SGI125903[] = {{&S13159[84], 6}, {NULL}};
const Segment SGI125904[] = {{&S13160[44], 13}, {NULL}};
const Segment SGI125905[] = {{&S13160[44], 25}, {NULL}};
const Segment SGI125906[] = {{&S13160[30], 6}, {NULL}};
const Segment SGI125907[] = {{&S13160[118], 3}, {NULL}};
const Segment SGI125908[] = {{&S13160[83], 6}, {NULL}};
const Segment SGI125909[] = {{&S13161[44], 6}, {NULL}};
const Segment SGI125910[] = {{&S13161[44], 18}, {NULL}};
const Segment SGI125911[] = {{&S13161[30], 6}, {NULL}};
const Segment SGI125912[] = {{&S13161[111], 3}, {NULL}};
const Segment SGI125913[] = {{&S13161[76], 6}, {NULL}};
const Segment SGI125914[] = {{&S13162[44], 10}, {NULL}};
const Segment SGI125915[] = {{&S13162[44], 22}, {NULL}};
const Segment SGI125916[] = {{&S13162[30], 6}, {NULL}};
const Segment SGI125917[] = {{&S13162[115], 3}, {NULL}};
const Segment SGI125918[] = {{&S13162[80], 6}, {NULL}};
const Segment SGI125919[] = {{&S13163[44], 7}, {NULL}};
const Segment SGI125920[] = {{&S13163[44], 19}, {NULL}};
const Segment SGI125921[] = {{&S13163[30], 6}, {NULL}};
const Segment SGI125922[] = {{&S13163[112], 3}, {NULL}};
const Segment SGI125923[] = {{&S13163[77], 6}, {NULL}};
const Segment SGI125924[] = {{&S13164[44], 15}, {NULL}};
const Segment SGI125925[] = {{&S13164[44], 27}, {NULL}};
const Segment SGI125926[] = {{&S13164[30], 6}, {NULL}};
const Segment SGI125927[] = {{&S13164[120], 3}, {NULL}};
const Segment SGI125928[] = {{&S13164[85], 6}, {NULL}};
const Segment SGI125929[] = {{&S13165[44], 7}, {NULL}};
const Segment SGI125930[] = {{&S13165[44], 19}, {NULL}};
const Segment SGI125931[] = {{&S13165[30], 6}, {NULL}};
const Segment SGI125932[] = {{&S13165[112], 3}, {NULL}};
const Segment SGI125933[] = {{&S13165[77], 6}, {NULL}};
const Segment SGI125934[] = {{&S13166[44], 9}, {NULL}};
const Segment SGI125935[] = {{&S13166[44], 21}, {NULL}};
const Segment SGI125936[] = {{&S13166[30], 6}, {NULL}};
const Segment SGI125937[] = {{&S13166[114], 3}, {NULL}};
const Segment SGI125938[] = {{&S13166[79], 6}, {NULL}};
const Segment SGI125939[] = {{&S13167[44], 18}, {NULL}};
const Segment SGI125940[] = {{&S13167[44], 30}, {NULL}};
const Segment SGI125941[] = {{&S13167[30], 6}, {NULL}};
const Segment SGI125942[] = {{&S13167[123], 3}, {NULL}};
const Segment SGI125943[] = {{&S13167[88], 6}, {NULL}};
const Segment SGI125944[] = {{&S13168[44], 9}, {NULL}};
const Segment SGI125945[] = {{&S13168[44], 21}, {NULL}};
const Segment SGI125946[] = {{&S13168[30], 6}, {NULL}};
const Segment SGI125947[] = {{&S13168[114], 3}, {NULL}};
const Segment SGI125948[] = {{&S13168[79], 6}, {NULL}};
const Segment SGI125949[] = {{&S13169[44], 5}, {NULL}};
const Segment SGI125950[] = {{&S13169[44], 17}, {NULL}};
const Segment SGI125951[] = {{&S13169[30], 6}, {NULL}};
const Segment SGI125952[] = {{&S13169[110], 3}, {NULL}};
const Segment SGI125953[] = {{&S13169[75], 6}, {NULL}};
const Segment SGI125954[] = {{&S13170[44], 12}, {NULL}};
const Segment SGI125955[] = {{&S13170[44], 28}, {NULL}};
const Segment SGI125956[] = {{&S13170[30], 6}, {NULL}};
const Segment SGI125957[] = {{&S104386[30], 6}, {NULL}};
const Segment SGI125958[] = {{&S13170[121], 3}, {NULL}};
const Segment SGI125959[] = {{&S104386[31], 3}, {NULL}};
const Segment SGI125960[] = {{&S13170[86], 6}, {NULL}};
const Segment SGI125961[] = {{&S104386[125], 3}, {NULL}};
const Segment SGI125962[] = {{&S104386[57], 5}, {NULL}};
const Segment SGI125963[] = {{&S13171[44], 5}, {NULL}};
const Segment SGI125964[] = {{&S13171[44], 14}, {NULL}};
const Segment SGI125965[] = {{&S13171[30], 6}, {NULL}};
const Segment SGI125966[] = {{&S13171[107], 3}, {NULL}};
const Segment SGI125967[] = {{&S13171[72], 6}, {NULL}};
const Segment SGI125968[] = {{&S13172[44], 5}, {NULL}};
const Segment SGI125969[] = {{&S13172[44], 17}, {NULL}};
const Segment SGI125970[] = {{&S13172[30], 6}, {NULL}};
const Segment SGI125971[] = {{&S13172[110], 3}, {NULL}};
const Segment SGI125972[] = {{&S13172[75], 6}, {NULL}};
const Segment SGI125973[] = {{&S13173[44], 13}, {NULL}};
const Segment SGI125974[] = {{&S13173[44], 25}, {NULL}};
const Segment SGI125975[] = {{&S13173[30], 6}, {NULL}};
const Segment SGI125976[] = {{&S13173[118], 3}, {NULL}};
const Segment SGI125977[] = {{&S13173[83], 6}, {NULL}};
const Segment SGI125978[] = {{&S13174[44], 41}, {NULL}};
const Segment SGI125979[] = {{&S13174[30], 6}, {NULL}};
const Segment SGI125980[] = {{&S13174[148], 3}, {NULL}};
const Segment SGI125981[] = {{&S13174[113], 6}, {NULL}};
const Segment SGI125982[] = {{&S13175[44], 13}, {NULL}};
const Segment SGI125983[] = {{&S13175[44], 25}, {NULL}};
const Segment SGI125984[] = {{&S13175[30], 6}, {NULL}};
const Segment SGI125985[] = {{&S13175[118], 3}, {NULL}};
const Segment SGI125986[] = {{&S13175[83], 6}, {NULL}};
const Segment SGI125987[] = {{&S13177[44], 6}, {NULL}};
const Segment SGI125988[] = {{&S13177[44], 18}, {NULL}};
const Segment SGI125989[] = {{&S13177[30], 6}, {NULL}};
const Segment SGI125990[] = {{&S13177[111], 3}, {NULL}};
const Segment SGI125991[] = {{&S13177[76], 6}, {NULL}};
const Segment SGI125992[] = {{&S13179[44], 9}, {NULL}};
const Segment SGI125993[] = {{&S13179[44], 21}, {NULL}};
const Segment SGI125994[] = {{&S13179[30], 6}, {NULL}};
const Segment SGI125995[] = {{&S13179[114], 3}, {NULL}};
const Segment SGI125996[] = {{&S13179[79], 6}, {NULL}};
const Segment SGI125997[] = {{&S13180[44], 9}, {NULL}};
const Segment SGI125998[] = {{&S13180[44], 21}, {NULL}};
const Segment SGI125999[] = {{&S13180[30], 6}, {NULL}};
const Segment SGI126000[] = {{&S13180[114], 3}, {NULL}};
const Segment SGI126001[] = {{&S13180[79], 6}, {NULL}};
const Segment SGI126002[] = {{&S13181[44], 12}, {NULL}};
const Segment SGI126003[] = {{&S13181[44], 24}, {NULL}};
const Segment SGI126004[] = {{&S13181[30], 6}, {NULL}};
const Segment SGI126005[] = {{&S13181[117], 3}, {NULL}};
const Segment SGI126006[] = {{&S13181[82], 6}, {NULL}};
const Segment SGI126007[] = {{&S13182[44], 7}, {NULL}};
const Segment SGI126008[] = {{&S13182[44], 19}, {NULL}};
const Segment SGI126009[] = {{&S13182[30], 6}, {NULL}};
const Segment SGI126010[] = {{&S13182[112], 3}, {NULL}};
const Segment SGI126011[] = {{&S13182[77], 6}, {NULL}};
const Segment SGI126012[] = {{&S13183[44], 14}, {NULL}};
const Segment SGI126013[] = {{&S13183[44], 26}, {NULL}};
const Segment SGI126014[] = {{&S13183[30], 6}, {NULL}};
const Segment SGI126015[] = {{&S13183[119], 3}, {NULL}};
const Segment SGI126016[] = {{&S13183[84], 6}, {NULL}};
const Segment SGI126017[] = {{&S13184[44], 11}, {NULL}};
const Segment SGI126018[] = {{&S13184[44], 23}, {NULL}};
const Segment SGI126019[] = {{&S13184[30], 6}, {NULL}};
const Segment SGI126020[] = {{&S13184[116], 3}, {NULL}};
const Segment SGI126021[] = {{&S13184[81], 6}, {NULL}};
const Segment SGI126022[] = {{&S13185[44], 12}, {NULL}};
const Segment SGI126023[] = {{&S13185[44], 24}, {NULL}};
const Segment SGI126024[] = {{&S13185[30], 6}, {NULL}};
const Segment SGI126025[] = {{&S13185[117], 3}, {NULL}};
const Segment SGI126026[] = {{&S13185[82], 6}, {NULL}};
const Segment SGI126027[] = {{&S13186[44], 9}, {NULL}};
const Segment SGI126028[] = {{&S13186[44], 21}, {NULL}};
const Segment SGI126029[] = {{&S13186[30], 6}, {NULL}};
const Segment SGI126030[] = {{&S13186[114], 3}, {NULL}};
const Segment SGI126031[] = {{&S13186[79], 6}, {NULL}};
const Segment SGI126032[] = {{&S13188[44], 13}, {NULL}};
const Segment SGI126033[] = {{&S13188[44], 25}, {NULL}};
const Segment SGI126034[] = {{&S13188[30], 6}, {NULL}};
const Segment SGI126035[] = {{&S13188[118], 3}, {NULL}};
const Segment SGI126036[] = {{&S13188[83], 6}, {NULL}};
const Segment SGI126037[] = {{&S13189[44], 7}, {NULL}};
const Segment SGI126038[] = {{&S13189[44], 19}, {NULL}};
const Segment SGI126039[] = {{&S13189[30], 6}, {NULL}};
const Segment SGI126040[] = {{&S13189[112], 3}, {NULL}};
const Segment SGI126041[] = {{&S13189[77], 6}, {NULL}};
const Segment SGI126042[] = {{&S13190[44], 12}, {NULL}};
const Segment SGI126043[] = {{&S13190[44], 24}, {NULL}};
const Segment SGI126044[] = {{&S13190[30], 6}, {NULL}};
const Segment SGI126045[] = {{&S13190[117], 3}, {NULL}};
const Segment SGI126046[] = {{&S13190[82], 6}, {NULL}};
const Segment SGI126047[] = {{&S13191[44], 16}, {NULL}};
const Segment SGI126048[] = {{&S13191[44], 28}, {NULL}};
const Segment SGI126049[] = {{&S13191[30], 6}, {NULL}};
const Segment SGI126050[] = {{&S13191[121], 3}, {NULL}};
const Segment SGI126051[] = {{&S13191[86], 6}, {NULL}};
const Segment SGI126052[] = {{&S104427[59], 10}, {NULL}};
const Segment SGI126053[] = {{&S104427[59], 22}, {NULL}};
const Segment SGI126054[] = {{&S104427[45], 6}, {NULL}};
const Segment SGI126055[] = {{&S13192[44], 6}, {NULL}};
const Segment SGI126056[] = {{&S13192[44], 18}, {NULL}};
const Segment SGI126057[] = {{&S13192[30], 6}, {NULL}};
const Segment SGI126058[] = {{&S104427[46], 3}, {NULL}};
const Segment SGI126059[] = {{&S104427[95], 6}, {NULL}};
const Segment SGI126060[] = {{&S13192[111], 3}, {NULL}};
const Segment SGI126061[] = {{&S13192[76], 6}, {NULL}};
const Segment SGI126062[] = {{&S104427[0], 14}, {NULL}};
const Segment SGI126063[] = {{&S13193[44], 7}, {NULL}};
const Segment SGI126064[] = {{&S13193[44], 19}, {NULL}};
const Segment SGI126065[] = {{&S13193[30], 6}, {NULL}};
const Segment SGI126066[] = {{&S13193[112], 3}, {NULL}};
const Segment SGI126067[] = {{&S13193[77], 6}, {NULL}};
const Segment SGI126068[] = {{&S104429[58], 10}, {NULL}};
const Segment SGI126069[] = {{&S104429[58], 22}, {NULL}};
const Segment SGI126070[] = {{&S104429[44], 6}, {NULL}};
const Segment SGI126071[] = {{&S104429[45], 3}, {NULL}};
const Segment SGI126072[] = {{&S104429[94], 6}, {NULL}};
const Segment SGI126073[] = {{&S13194[44], 10}, {NULL}};
const Segment SGI126074[] = {{&S104429[0], 14}, {NULL}};
const Segment SGI126075[] = {{&S13194[44], 22}, {NULL}};
const Segment SGI126076[] = {{&S13194[30], 6}, {NULL}};
const Segment SGI126077[] = {{&S13194[115], 3}, {NULL}};
const Segment SGI126078[] = {{&S13194[80], 6}, {NULL}};
const Segment SGI126079[] = {{&S13195[44], 12}, {NULL}};
const Segment SGI126080[] = {{&S13195[44], 24}, {NULL}};
const Segment SGI126081[] = {{&S13195[30], 6}, {NULL}};
const Segment SGI126082[] = {{&S13195[117], 3}, {NULL}};
const Segment SGI126083[] = {{&S13195[82], 6}, {NULL}};
const Segment SGI126084[] = {{&S13196[44], 6}, {NULL}};
const Segment SGI126085[] = {{&S13196[44], 18}, {NULL}};
const Segment SGI126086[] = {{&S13196[30], 6}, {NULL}};
const Segment SGI126087[] = {{&S13196[111], 3}, {NULL}};
const Segment SGI126088[] = {{&S13196[76], 6}, {NULL}};
const Segment SGI126089[] = {{&S104434[59], 10}, {NULL}};
const Segment SGI126090[] = {{&S104434[59], 22}, {NULL}};
const Segment SGI126091[] = {{&S104434[45], 6}, {NULL}};
const Segment SGI126092[] = {{&S13197[44], 10}, {NULL}};
const Segment SGI126093[] = {{&S13197[44], 22}, {NULL}};
const Segment SGI126094[] = {{&S13197[30], 6}, {NULL}};
const Segment SGI126095[] = {{&S104434[95], 5}, {NULL}};
const Segment SGI126096[] = {{&S104434[0], 14}, {NULL}};
const Segment SGI126097[] = {{&S13197[115], 3}, {NULL}};
const Segment SGI126098[] = {{&S13197[80], 6}, {NULL}};
const Segment SGI126099[] = {{&S13198[44], 14}, {NULL}};
const Segment SGI126100[] = {{&S13198[44], 26}, {NULL}};
const Segment SGI126101[] = {{&S13198[30], 6}, {NULL}};
const Segment SGI126102[] = {{&S13198[119], 3}, {NULL}};
const Segment SGI126103[] = {{&S13198[84], 6}, {NULL}};
const Segment SGI126104[] = {{&S13199[44], 8}, {NULL}};
const Segment SGI126105[] = {{&S13199[44], 20}, {NULL}};
const Segment SGI126106[] = {{&S13199[30], 6}, {NULL}};
const Segment SGI126107[] = {{&S13199[113], 3}, {NULL}};
const Segment SGI126108[] = {{&S13199[78], 6}, {NULL}};
const Segment SGI126109[] = {{&S13200[44], 13}, {NULL}};
const Segment SGI126110[] = {{&S13200[44], 25}, {NULL}};
const Segment SGI126111[] = {{&S13200[30], 6}, {NULL}};
const Segment SGI126112[] = {{&S13200[118], 3}, {NULL}};
const Segment SGI126113[] = {{&S13200[83], 6}, {NULL}};
const Segment SGI126114[] = {{&S13201[44], 9}, {NULL}};
const Segment SGI126115[] = {{&S13201[44], 21}, {NULL}};
const Segment SGI126116[] = {{&S13201[30], 6}, {NULL}};
const Segment SGI126117[] = {{&S13201[114], 3}, {NULL}};
const Segment SGI126118[] = {{&S13201[79], 6}, {NULL}};
const Segment SGI126119[] = {{&S13202[44], 12}, {NULL}};
const Segment SGI126120[] = {{&S13202[44], 24}, {NULL}};
const Segment SGI126121[] = {{&S13202[30], 6}, {NULL}};
const Segment SGI126122[] = {{&S13202[117], 3}, {NULL}};
const Segment SGI126123[] = {{&S13202[82], 6}, {NULL}};
const Segment SGI126124[] = {{&S13203[44], 11}, {NULL}};
const Segment SGI126125[] = {{&S13203[44], 23}, {NULL}};
const Segment SGI126126[] = {{&S13203[30], 6}, {NULL}};
const Segment SGI126127[] = {{&S13203[116], 3}, {NULL}};
const Segment SGI126128[] = {{&S13203[81], 6}, {NULL}};
const Segment SGI126129[] = {{&S104446[59], 10}, {NULL}};
const Segment SGI126130[] = {{&S104446[59], 22}, {NULL}};
const Segment SGI126131[] = {{&S104446[45], 6}, {NULL}};
const Segment SGI126132[] = {{&S13204[44], 12}, {NULL}};
const Segment SGI126133[] = {{&S13204[44], 24}, {NULL}};
const Segment SGI126134[] = {{&S13204[30], 6}, {NULL}};
const Segment SGI126135[] = {{&S104446[46], 3}, {NULL}};
const Segment SGI126136[] = {{&S104446[95], 6}, {NULL}};
const Segment SGI126137[] = {{&S13204[117], 3}, {NULL}};
const Segment SGI126138[] = {{&S104446[0], 14}, {NULL}};
const Segment SGI126139[] = {{&S13204[82], 6}, {NULL}};
const Segment SGI126140[] = {{&S13205[44], 10}, {NULL}};
const Segment SGI126141[] = {{&S13205[44], 22}, {NULL}};
const Segment SGI126142[] = {{&S13205[30], 6}, {NULL}};
const Segment SGI126143[] = {{&S13205[115], 3}, {NULL}};
const Segment SGI126144[] = {{&S13205[80], 6}, {NULL}};
const Segment SGI126145[] = {{&S13206[44], 8}, {NULL}};
const Segment SGI126146[] = {{&S13206[44], 20}, {NULL}};
const Segment SGI126147[] = {{&S13206[30], 6}, {NULL}};
const Segment SGI126148[] = {{&S13206[113], 3}, {NULL}};
const Segment SGI126149[] = {{&S13206[78], 6}, {NULL}};
const Segment SGI126150[] = {{&S13207[44], 16}, {NULL}};
const Segment SGI126151[] = {{&S13207[44], 28}, {NULL}};
const Segment SGI126152[] = {{&S13207[30], 6}, {NULL}};
const Segment SGI126153[] = {{&S13207[121], 3}, {NULL}};
const Segment SGI126154[] = {{&S13207[86], 6}, {NULL}};
const Segment SGI126155[] = {{&S13208[44], 17}, {NULL}};
const Segment SGI126156[] = {{&S13208[44], 29}, {NULL}};
const Segment SGI126157[] = {{&S13208[30], 6}, {NULL}};
const Segment SGI126158[] = {{&S13208[122], 3}, {NULL}};
const Segment SGI126159[] = {{&S13208[87], 6}, {NULL}};
const Segment SGI126160[] = {{&S13209[44], 6}, {NULL}};
const Segment SGI126161[] = {{&S13209[44], 18}, {NULL}};
const Segment SGI126162[] = {{&S13209[30], 6}, {NULL}};
const Segment SGI126163[] = {{&S13209[110], 3}, {NULL}};
const Segment SGI126164[] = {{&S13209[76], 5}, {NULL}};
const Segment SGI126165[] = {{&S13210[44], 6}, {NULL}};
const Segment SGI126166[] = {{&S13210[44], 18}, {NULL}};
const Segment SGI126167[] = {{&S13210[30], 6}, {NULL}};
const Segment SGI126168[] = {{&S13210[111], 3}, {NULL}};
const Segment SGI126169[] = {{&S13210[76], 6}, {NULL}};
const Segment SGI126170[] = {{&S13211[44], 6}, {NULL}};
const Segment SGI126171[] = {{&S13211[44], 18}, {NULL}};
const Segment SGI126172[] = {{&S13211[30], 6}, {NULL}};
const Segment SGI126173[] = {{&S13211[111], 3}, {NULL}};
const Segment SGI126174[] = {{&S13211[76], 6}, {NULL}};
const Segment SGI126175[] = {{&S13212[44], 9}, {NULL}};
const Segment SGI126176[] = {{&S13212[44], 16}, {NULL}};
const Segment SGI126177[] = {{&S13212[30], 6}, {NULL}};
const Segment SGI126178[] = {{&S13212[109], 3}, {NULL}};
const Segment SGI126179[] = {{&S13212[74], 6}, {NULL}};
const Segment SGI126180[] = {{&S13213[44], 8}, {NULL}};
const Segment SGI126181[] = {{&S13213[44], 20}, {NULL}};
const Segment SGI126182[] = {{&S13213[30], 6}, {NULL}};
const Segment SGI126183[] = {{&S13213[113], 3}, {NULL}};
const Segment SGI126184[] = {{&S13213[78], 6}, {NULL}};
const Segment SGI126185[] = {{&S13214[44], 6}, {NULL}};
const Segment SGI126186[] = {{&S13214[44], 18}, {NULL}};
const Segment SGI126187[] = {{&S13214[30], 6}, {NULL}};
const Segment SGI126188[] = {{&S13214[111], 3}, {NULL}};
const Segment SGI126189[] = {{&S13214[76], 6}, {NULL}};
const Segment SGI126190[] = {{&S13215[44], 8}, {NULL}};
const Segment SGI126191[] = {{&S13215[44], 15}, {NULL}};
const Segment SGI126192[] = {{&S13215[30], 6}, {NULL}};
const Segment SGI126193[] = {{&S13215[108], 3}, {NULL}};
const Segment SGI126194[] = {{&S13215[73], 6}, {NULL}};
const Segment SGI126195[] = {{&S13216[44], 11}, {NULL}};
const Segment SGI126196[] = {{&S13216[44], 23}, {NULL}};
const Segment SGI126197[] = {{&S13216[30], 6}, {NULL}};
const Segment SGI126198[] = {{&S13216[116], 3}, {NULL}};
const Segment SGI126199[] = {{&S13216[81], 6}, {NULL}};
const Segment SGI126200[] = {{&S13217[44], 11}, {NULL}};
const Segment SGI126201[] = {{&S13217[44], 23}, {NULL}};
const Segment SGI126202[] = {{&S13217[30], 6}, {NULL}};
const Segment SGI126203[] = {{&S13217[116], 3}, {NULL}};
const Segment SGI126204[] = {{&S13217[81], 6}, {NULL}};
const Segment SGI126205[] = {{&S13218[44], 12}, {NULL}};
const Segment SGI126206[] = {{&S13218[44], 24}, {NULL}};
const Segment SGI126207[] = {{&S13218[30], 6}, {NULL}};
const Segment SGI126208[] = {{&S13218[117], 3}, {NULL}};
const Segment SGI126209[] = {{&S13218[82], 6}, {NULL}};
const Segment SGI126210[] = {{&S13219[44], 11}, {NULL}};
const Segment SGI126211[] = {{&S13219[44], 23}, {NULL}};
const Segment SGI126212[] = {{&S13219[30], 6}, {NULL}};
const Segment SGI126213[] = {{&S13219[116], 3}, {NULL}};
const Segment SGI126214[] = {{&S13219[81], 6}, {NULL}};
const Segment SGI126215[] = {{&S104474[44], 10}, {NULL}};
const Segment SGI126216[] = {{&S104474[44], 22}, {NULL}};
const Segment SGI126217[] = {{&S104474[30], 6}, {NULL}};
const Segment SGI126218[] = {{&S104474[31], 3}, {NULL}};
const Segment SGI126219[] = {{&S104474[80], 6}, {NULL}};
const Segment SGI126220[] = {{&S104474[133], 11}, {NULL}};
const Segment SGI126221[] = {{&S13220[44], 8}, {NULL}};
const Segment SGI126222[] = {{&S13220[44], 20}, {NULL}};
const Segment SGI126223[] = {{&S13220[30], 6}, {NULL}};
const Segment SGI126224[] = {{&S13220[113], 3}, {NULL}};
const Segment SGI126225[] = {{&S13220[78], 6}, {NULL}};
const Segment SGI126226[] = {{&S13221[44], 12}, {NULL}};
const Segment SGI126227[] = {{&S13221[44], 24}, {NULL}};
const Segment SGI126228[] = {{&S13221[30], 6}, {NULL}};
const Segment SGI126229[] = {{&S13221[117], 3}, {NULL}};
const Segment SGI126230[] = {{&S13221[82], 6}, {NULL}};
const Segment SGI126231[] = {{&S13222[44], 20}, {NULL}};
const Segment SGI126232[] = {{&S13222[44], 32}, {NULL}};
const Segment SGI126233[] = {{&S13222[30], 6}, {NULL}};
const Segment SGI126234[] = {{&S13222[125], 3}, {NULL}};
const Segment SGI126235[] = {{&S13222[90], 6}, {NULL}};
const Segment SGI126236[] = {{&S13223[44], 9}, {NULL}};
const Segment SGI126237[] = {{&S13223[44], 21}, {NULL}};
const Segment SGI126238[] = {{&S13223[30], 6}, {NULL}};
const Segment SGI126239[] = {{&S13223[114], 3}, {NULL}};
const Segment SGI126240[] = {{&S13223[79], 6}, {NULL}};
const Segment SGI126241[] = {{&S13224[44], 9}, {NULL}};
const Segment SGI126242[] = {{&S13224[44], 21}, {NULL}};
const Segment SGI126243[] = {{&S13224[30], 6}, {NULL}};
const Segment SGI126244[] = {{&S13224[114], 3}, {NULL}};
const Segment SGI126245[] = {{&S13224[79], 6}, {NULL}};
const Segment SGI126246[] = {{&S13225[44], 7}, {NULL}};
const Segment SGI126247[] = {{&S13225[44], 19}, {NULL}};
const Segment SGI126248[] = {{&S13225[30], 6}, {NULL}};
const Segment SGI126249[] = {{&S13225[112], 3}, {NULL}};
const Segment SGI126250[] = {{&S13225[77], 6}, {NULL}};
const Segment SGI126251[] = {{&S13226[44], 12}, {NULL}};
const Segment SGI126252[] = {{&S13226[44], 24}, {NULL}};
const Segment SGI126253[] = {{&S13226[30], 6}, {NULL}};
const Segment SGI126254[] = {{&S13226[117], 3}, {NULL}};
const Segment SGI126255[] = {{&S13226[82], 6}, {NULL}};
const Segment SGI126256[] = {{&S13227[44], 15}, {NULL}};
const Segment SGI126257[] = {{&S13227[44], 27}, {NULL}};
const Segment SGI126258[] = {{&S13227[30], 6}, {NULL}};
const Segment SGI126259[] = {{&S13227[120], 3}, {NULL}};
const Segment SGI126260[] = {{&S13227[85], 6}, {NULL}};
const Segment SGI126261[] = {{&S13228[44], 9}, {NULL}};
const Segment SGI126262[] = {{&S13228[44], 21}, {NULL}};
const Segment SGI126263[] = {{&S13228[30], 6}, {NULL}};
const Segment SGI126264[] = {{&S13228[114], 3}, {NULL}};
const Segment SGI126265[] = {{&S13228[79], 6}, {NULL}};
const Segment SGI126266[] = {{&S13229[44], 6}, {NULL}};
const Segment SGI126267[] = {{&S13229[44], 18}, {NULL}};
const Segment SGI126268[] = {{&S13229[30], 6}, {NULL}};
const Segment SGI126269[] = {{&S13229[111], 3}, {NULL}};
const Segment SGI126270[] = {{&S13229[76], 6}, {NULL}};
const Segment SGI126271[] = {{&S13230[44], 10}, {NULL}};
const Segment SGI126272[] = {{&S13230[44], 22}, {NULL}};
const Segment SGI126273[] = {{&S13230[30], 6}, {NULL}};
const Segment SGI126274[] = {{&S13230[115], 3}, {NULL}};
const Segment SGI126275[] = {{&S13230[80], 6}, {NULL}};
const Segment SGI126276[] = {{&S13231[44], 23}, {NULL}};
const Segment SGI126277[] = {{&S13231[44], 35}, {NULL}};
const Segment SGI126278[] = {{&S13231[30], 6}, {NULL}};
const Segment SGI126279[] = {{&S13231[128], 3}, {NULL}};
const Segment SGI126280[] = {{&S13231[93], 6}, {NULL}};
const Segment SGI126281[] = {{&S13232[44], 11}, {NULL}};
const Segment SGI126282[] = {{&S13232[44], 23}, {NULL}};
const Segment SGI126283[] = {{&S13232[30], 6}, {NULL}};
const Segment SGI126284[] = {{&S13232[116], 3}, {NULL}};
const Segment SGI126285[] = {{&S13232[81], 6}, {NULL}};
const Segment SGI126286[] = {{&S13233[44], 6}, {NULL}};
const Segment SGI126287[] = {{&S13233[44], 18}, {NULL}};
const Segment SGI126288[] = {{&S13233[30], 6}, {NULL}};
const Segment SGI126289[] = {{&S13233[111], 3}, {NULL}};
const Segment SGI126290[] = {{&S13233[76], 6}, {NULL}};
const Segment SGI126291[] = {{&S13235[44], 10}, {NULL}};
const Segment SGI126292[] = {{&S13235[44], 22}, {NULL}};
const Segment SGI126293[] = {{&S13235[30], 6}, {NULL}};
const Segment SGI126294[] = {{&S13235[115], 3}, {NULL}};
const Segment SGI126295[] = {{&S13235[80], 6}, {NULL}};
const Segment SGI126296[] = {{&S13236[44], 4}, {NULL}};
const Segment SGI126297[] = {{&S13236[44], 16}, {NULL}};
const Segment SGI126298[] = {{&S13236[30], 6}, {NULL}};
const Segment SGI126299[] = {{&S13236[109], 3}, {NULL}};
const Segment SGI126300[] = {{&S13236[74], 6}, {NULL}};
const Segment SGI126301[] = {{&S13237[44], 7}, {NULL}};
const Segment SGI126302[] = {{&S13237[44], 19}, {NULL}};
const Segment SGI126303[] = {{&S13237[30], 6}, {NULL}};
const Segment SGI126304[] = {{&S13237[112], 3}, {NULL}};
const Segment SGI126305[] = {{&S13237[77], 6}, {NULL}};
const Segment SGI126306[] = {{&S13238[44], 8}, {NULL}};
const Segment SGI126307[] = {{&S13238[44], 20}, {NULL}};
const Segment SGI126308[] = {{&S13238[30], 6}, {NULL}};
const Segment SGI126309[] = {{&S13238[113], 3}, {NULL}};
const Segment SGI126310[] = {{&S13238[78], 6}, {NULL}};
const Segment SGI126311[] = {{&S13239[44], 5}, {NULL}};
const Segment SGI126312[] = {{&S13239[44], 17}, {NULL}};
const Segment SGI126313[] = {{&S13239[30], 6}, {NULL}};
const Segment SGI126314[] = {{&S13239[110], 3}, {NULL}};
const Segment SGI126315[] = {{&S13239[75], 6}, {NULL}};
const Segment SGI126316[] = {{&S13240[44], 12}, {NULL}};
const Segment SGI126317[] = {{&S13240[44], 24}, {NULL}};
const Segment SGI126318[] = {{&S13240[30], 6}, {NULL}};
const Segment SGI126319[] = {{&S13240[117], 3}, {NULL}};
const Segment SGI126320[] = {{&S13240[82], 6}, {NULL}};
const Segment SGI126321[] = {{&S13241[44], 7}, {NULL}};
const Segment SGI126322[] = {{&S13241[44], 19}, {NULL}};
const Segment SGI126323[] = {{&S13241[30], 6}, {NULL}};
const Segment SGI126324[] = {{&S13241[112], 3}, {NULL}};
const Segment SGI126325[] = {{&S13241[77], 6}, {NULL}};
const Segment SGI126326[] = {{&S13242[44], 17}, {NULL}};
const Segment SGI126327[] = {{&S13242[44], 29}, {NULL}};
const Segment SGI126328[] = {{&S13242[30], 6}, {&S13242[52], 21}, {NULL}};
const Segment SGI126329[] = {{&S13242[122], 3}, {NULL}};
const Segment SGI126330[] = {{&S13242[87], 6}, {NULL}};
const Segment SGI126331[] = {{&S104518[34], 10}, {NULL}};
const Segment SGI126332[] = {{&S13243[44], 8}, {NULL}};
const Segment SGI126333[] = {{&S104518[27], 6}, {NULL}};
const Segment SGI126334[] = {{&S13243[44], 20}, {NULL}};
const Segment SGI126335[] = {{&S13243[30], 6}, {NULL}};
const Segment SGI126336[] = {{&S13243[113], 3}, {NULL}};
const Segment SGI126337[] = {{&S104518[97], 9}, {NULL}};
const Segment SGI126338[] = {{&S13243[78], 6}, {NULL}};
const Segment SGI126339[] = {{&S104518[70], 6}, {NULL}};
const Segment SGI126340[] = {{&S104518[132], 8}, {NULL}};
const Segment SGI126341[] = {{&S13244[44], 8}, {NULL}};
const Segment SGI126342[] = {{&S13244[44], 20}, {NULL}};
const Segment SGI126343[] = {{&S13244[30], 6}, {NULL}};
const Segment SGI126344[] = {{&S13244[113], 3}, {NULL}};
const Segment SGI126345[] = {{&S13244[78], 6}, {NULL}};
const Segment SGI126346[] = {{&S13245[44], 9}, {NULL}};
const Segment SGI126347[] = {{&S13245[44], 21}, {NULL}};
const Segment SGI126348[] = {{&S13245[30], 6}, {NULL}};
const Segment SGI126349[] = {{&S13245[114], 3}, {NULL}};
const Segment SGI126350[] = {{&S13245[79], 6}, {NULL}};
const Segment SGI126351[] = {{&S13246[44], 13}, {NULL}};
const Segment SGI126352[] = {{&S13246[44], 25}, {NULL}};
const Segment SGI126353[] = {{&S13246[30], 6}, {NULL}};
const Segment SGI126354[] = {{&S13246[118], 3}, {NULL}};
const Segment SGI126355[] = {{&S13246[83], 6}, {NULL}};
const Segment SGI126356[] = {{&S13247[44], 12}, {NULL}};
const Segment SGI126357[] = {{&S13247[44], 24}, {NULL}};
const Segment SGI126358[] = {{&S13247[30], 6}, {NULL}};
const Segment SGI126359[] = {{&S13247[117], 3}, {NULL}};
const Segment SGI126360[] = {{&S13247[82], 6}, {NULL}};
const Segment SGI126361[] = {{&S13248[44], 18}, {NULL}};
const Segment SGI126362[] = {{&S13248[44], 30}, {NULL}};
const Segment SGI126363[] = {{&S13248[30], 6}, {NULL}};
const Segment SGI126364[] = {{&S13248[123], 3}, {NULL}};
const Segment SGI126365[] = {{&S13248[88], 6}, {NULL}};
const Segment SGI126366[] = {{&S13249[44], 14}, {NULL}};
const Segment SGI126367[] = {{&S13249[44], 26}, {NULL}};
const Segment SGI126368[] = {{&S13249[30], 6}, {NULL}};
const Segment SGI126369[] = {{&S13249[119], 3}, {NULL}};
const Segment SGI126370[] = {{&S13249[84], 6}, {NULL}};
const Segment SGI126371[] = {{&S13250[44], 14}, {NULL}};
const Segment SGI126372[] = {{&S13250[44], 26}, {NULL}};
const Segment SGI126373[] = {{&S13250[30], 6}, {NULL}};
const Segment SGI126374[] = {{&S13250[119], 3}, {NULL}};
const Segment SGI126375[] = {{&S13250[84], 6}, {NULL}};
const Segment SGI126376[] = {{&S13251[44], 8}, {NULL}};
const Segment SGI126377[] = {{&S13251[44], 20}, {NULL}};
const Segment SGI126378[] = {{&S13251[30], 6}, {NULL}};
const Segment SGI126379[] = {{&S13251[113], 3}, {NULL}};
const Segment SGI126380[] = {{&S13251[78], 6}, {NULL}};
const Segment SGI126381[] = {{&S13252[44], 7}, {NULL}};
const Segment SGI126382[] = {{&S13252[44], 14}, {NULL}};
const Segment SGI126383[] = {{&S13252[30], 6}, {NULL}};
const Segment SGI126384[] = {{&S13252[107], 3}, {NULL}};
const Segment SGI126385[] = {{&S13252[72], 6}, {NULL}};
const Segment SGI126386[] = {{&S13253[44], 8}, {NULL}};
const Segment SGI126387[] = {{&S13253[44], 20}, {NULL}};
const Segment SGI126388[] = {{&S13253[30], 6}, {NULL}};
const Segment SGI126389[] = {{&S13253[113], 3}, {NULL}};
const Segment SGI126390[] = {{&S13253[78], 6}, {NULL}};
const Segment SGI126391[] = {{&S13254[44], 5}, {NULL}};
const Segment SGI126392[] = {{&S13254[44], 17}, {NULL}};
const Segment SGI126393[] = {{&S13254[30], 6}, {NULL}};
const Segment SGI126394[] = {{&S13254[110], 3}, {NULL}};
const Segment SGI126395[] = {{&S13254[75], 6}, {NULL}};
const Segment SGI126396[] = {{&S13255[44], 10}, {NULL}};
const Segment SGI126397[] = {{&S13255[44], 22}, {NULL}};
const Segment SGI126398[] = {{&S13255[30], 6}, {NULL}};
const Segment SGI126399[] = {{&S13255[115], 3}, {NULL}};
const Segment SGI126400[] = {{&S13255[80], 6}, {NULL}};
const Segment SGI126401[] = {{&S13256[44], 18}, {NULL}};
const Segment SGI126402[] = {{&S13256[44], 30}, {NULL}};
const Segment SGI126403[] = {{&S13256[30], 6}, {NULL}};
const Segment SGI126404[] = {{&S13256[123], 3}, {NULL}};
const Segment SGI126405[] = {{&S13256[88], 6}, {NULL}};
const Segment SGI126406[] = {{&S13257[44], 19}, {NULL}};
const Segment SGI126407[] = {{&S13257[44], 31}, {NULL}};
const Segment SGI126408[] = {{&S13257[30], 6}, {NULL}};
const Segment SGI126409[] = {{&S13257[124], 3}, {NULL}};
const Segment SGI126410[] = {{&S13257[89], 6}, {NULL}};
const Segment SGI126411[] = {{&S13258[44], 11}, {NULL}};
const Segment SGI126412[] = {{&S13258[44], 23}, {NULL}};
const Segment SGI126413[] = {{&S13258[30], 6}, {NULL}};
const Segment SGI126414[] = {{&S13258[116], 3}, {NULL}};
const Segment SGI126415[] = {{&S13258[81], 6}, {NULL}};
const Segment SGI126416[] = {{&S13259[44], 14}, {NULL}};
const Segment SGI126417[] = {{&S13259[44], 26}, {NULL}};
const Segment SGI126418[] = {{&S13259[30], 6}, {NULL}};
const Segment SGI126419[] = {{&S13259[119], 3}, {NULL}};
const Segment SGI126420[] = {{&S13259[84], 6}, {NULL}};
const Segment SGI126421[] = {{&S13260[44], 7}, {NULL}};
const Segment SGI126422[] = {{&S13260[44], 19}, {NULL}};
const Segment SGI126423[] = {{&S13260[30], 6}, {NULL}};
const Segment SGI126424[] = {{&S13260[112], 3}, {NULL}};
const Segment SGI126425[] = {{&S13260[77], 6}, {NULL}};
const Segment SGI126426[] = {{&S13261[44], 4}, {NULL}};
const Segment SGI126427[] = {{&S13261[44], 16}, {NULL}};
const Segment SGI126428[] = {{&S13261[30], 6}, {NULL}};
const Segment SGI126429[] = {{&S13261[109], 3}, {NULL}};
const Segment SGI126430[] = {{&S13261[74], 6}, {NULL}};
const Segment SGI126431[] = {{&S13262[44], 7}, {NULL}};
const Segment SGI126432[] = {{&S13262[44], 19}, {NULL}};
const Segment SGI126433[] = {{&S13262[30], 6}, {NULL}};
const Segment SGI126434[] = {{&S13262[112], 3}, {NULL}};
const Segment SGI126435[] = {{&S13262[77], 6}, {NULL}};
const Segment SGI126436[] = {{&S13263[44], 5}, {NULL}};
const Segment SGI126437[] = {{&S13263[44], 17}, {NULL}};
const Segment SGI126438[] = {{&S13263[30], 6}, {NULL}};
const Segment SGI126439[] = {{&S13263[110], 3}, {NULL}};
const Segment SGI126440[] = {{&S13263[75], 6}, {NULL}};
const Segment SGI126441[] = {{&S13264[44], 6}, {NULL}};
const Segment SGI126442[] = {{&S13264[44], 18}, {NULL}};
const Segment SGI126443[] = {{&S13264[30], 6}, {NULL}};
const Segment SGI126444[] = {{&S13264[111], 3}, {NULL}};
const Segment SGI126445[] = {{&S13264[76], 6}, {NULL}};
const Segment SGI126446[] = {{&S13266[44], 25}, {NULL}};
const Segment SGI126447[] = {{&S13266[44], 37}, {NULL}};
const Segment SGI126448[] = {{&S13266[30], 6}, {NULL}};
const Segment SGI126449[] = {{&S13266[130], 3}, {NULL}};
const Segment SGI126450[] = {{&S13266[95], 6}, {NULL}};
const Segment SGI126451[] = {{&S13267[44], 13}, {NULL}};
const Segment SGI126452[] = {{&S13267[44], 25}, {NULL}};
const Segment SGI126453[] = {{&S13267[30], 6}, {NULL}};
const Segment SGI126454[] = {{&S13267[118], 3}, {NULL}};
const Segment SGI126455[] = {{&S13267[83], 6}, {NULL}};
const Segment SGI126456[] = {{&S13268[44], 20}, {NULL}};
const Segment SGI126457[] = {{&S13268[44], 32}, {NULL}};
const Segment SGI126458[] = {{&S13268[30], 6}, {NULL}};
const Segment SGI126459[] = {{&S13268[125], 3}, {NULL}};
const Segment SGI126460[] = {{&S13268[90], 6}, {NULL}};
const Segment SGI126461[] = {{&S13269[44], 6}, {NULL}};
const Segment SGI126462[] = {{&S13269[44], 18}, {NULL}};
const Segment SGI126463[] = {{&S13269[30], 6}, {NULL}};
const Segment SGI126464[] = {{&S13269[111], 3}, {NULL}};
const Segment SGI126465[] = {{&S13269[76], 6}, {NULL}};
const Segment SGI126466[] = {{&S13270[44], 7}, {NULL}};
const Segment SGI126467[] = {{&S13270[44], 19}, {NULL}};
const Segment SGI126468[] = {{&S13270[30], 6}, {NULL}};
const Segment SGI126469[] = {{&S13270[112], 3}, {NULL}};
const Segment SGI126470[] = {{&S13270[77], 6}, {NULL}};
const Segment SGI126471[] = {{&S13271[44], 14}, {NULL}};
const Segment SGI126472[] = {{&S13271[44], 26}, {NULL}};
const Segment SGI126473[] = {{&S13271[30], 6}, {NULL}};
const Segment SGI126474[] = {{&S13271[119], 3}, {NULL}};
const Segment SGI126475[] = {{&S13271[84], 6}, {NULL}};
const Segment SGI126476[] = {{&S13272[44], 11}, {NULL}};
const Segment SGI126477[] = {{&S13272[44], 23}, {NULL}};
const Segment SGI126478[] = {{&S13272[30], 6}, {NULL}};
const Segment SGI126479[] = {{&S13272[116], 3}, {NULL}};
const Segment SGI126480[] = {{&S13272[81], 6}, {NULL}};
const Segment SGI126481[] = {{&S13273[44], 6}, {NULL}};
const Segment SGI126482[] = {{&S13273[44], 18}, {NULL}};
const Segment SGI126483[] = {{&S13273[30], 6}, {NULL}};
const Segment SGI126484[] = {{&S13273[111], 3}, {NULL}};
const Segment SGI126485[] = {{&S13273[76], 6}, {NULL}};
const Segment SGI126486[] = {{&S13274[44], 31}, {NULL}};
const Segment SGI126487[] = {{&S13274[44], 43}, {NULL}};
const Segment SGI126488[] = {{&S13274[30], 6}, {NULL}};
const Segment SGI126489[] = {{&S13274[136], 3}, {NULL}};
const Segment SGI126490[] = {{&S13274[101], 6}, {NULL}};
const Segment SGI126491[] = {{&S13275[44], 7}, {NULL}};
const Segment SGI126492[] = {{&S13275[44], 19}, {NULL}};
const Segment SGI126493[] = {{&S13275[30], 6}, {NULL}};
const Segment SGI126494[] = {{&S13275[112], 3}, {NULL}};
const Segment SGI126495[] = {{&S13275[77], 6}, {NULL}};
const Segment SGI126496[] = {{&S13276[44], 14}, {NULL}};
const Segment SGI126497[] = {{&S13276[44], 26}, {NULL}};
const Segment SGI126498[] = {{&S13276[30], 6}, {NULL}};
const Segment SGI126499[] = {{&S13276[119], 3}, {NULL}};
const Segment SGI126500[] = {{&S13276[84], 6}, {NULL}};
const Segment SGI126501[] = {{&S13277[44], 9}, {NULL}};
const Segment SGI126502[] = {{&S13277[44], 21}, {NULL}};
const Segment SGI126503[] = {{&S13277[30], 6}, {NULL}};
const Segment SGI126504[] = {{&S13277[114], 3}, {NULL}};
const Segment SGI126505[] = {{&S13277[79], 6}, {NULL}};
const Segment SGI126506[] = {{&S13278[44], 9}, {NULL}};
const Segment SGI126507[] = {{&S13278[44], 21}, {NULL}};
const Segment SGI126508[] = {{&S13278[30], 6}, {NULL}};
const Segment SGI126509[] = {{&S13278[114], 3}, {NULL}};
const Segment SGI126510[] = {{&S13278[79], 6}, {NULL}};
const Segment SGI126511[] = {{&S13279[44], 7}, {NULL}};
const Segment SGI126512[] = {{&S13279[44], 19}, {NULL}};
const Segment SGI126513[] = {{&S13279[30], 6}, {NULL}};
const Segment SGI126514[] = {{&S13279[112], 3}, {NULL}};
const Segment SGI126515[] = {{&S13279[77], 6}, {NULL}};
const Segment SGI126516[] = {{&S13280[44], 4}, {NULL}};
const Segment SGI126517[] = {{&S13280[44], 16}, {NULL}};
const Segment SGI126518[] = {{&S13280[30], 6}, {NULL}};
const Segment SGI126519[] = {{&S13280[109], 3}, {NULL}};
const Segment SGI126520[] = {{&S13280[74], 6}, {NULL}};
const Segment SGI126521[] = {{&S13281[44], 37}, {NULL}};
const Segment SGI126522[] = {{&S13281[30], 6}, {NULL}};
const Segment SGI126523[] = {{&S13281[113], 6}, {NULL}};
const Segment SGI126524[] = {{&S13282[44], 9}, {NULL}};
const Segment SGI126525[] = {{&S13282[44], 21}, {NULL}};
const Segment SGI126526[] = {{&S13282[30], 6}, {NULL}};
const Segment SGI126527[] = {{&S13282[114], 3}, {NULL}};
const Segment SGI126528[] = {{&S13282[79], 6}, {NULL}};
const Segment SGI126529[] = {{&S13283[44], 3}, {NULL}};
const Segment SGI126530[] = {{&S13283[44], 15}, {NULL}};
const Segment SGI126531[] = {{&S13283[30], 6}, {NULL}};
const Segment SGI126532[] = {{&S13283[108], 3}, {NULL}};
const Segment SGI126533[] = {{&S13283[73], 6}, {NULL}};
const Segment SGI126534[] = {{&S13284[44], 7}, {NULL}};
const Segment SGI126535[] = {{&S13284[44], 19}, {NULL}};
const Segment SGI126536[] = {{&S13284[30], 6}, {NULL}};
const Segment SGI126537[] = {{&S13284[112], 3}, {NULL}};
const Segment SGI126538[] = {{&S13284[77], 6}, {NULL}};
const Segment SGI126539[] = {{&S13285[44], 7}, {NULL}};
const Segment SGI126540[] = {{&S13285[44], 19}, {NULL}};
const Segment SGI126541[] = {{&S13285[30], 6}, {NULL}};
const Segment SGI126542[] = {{&S13285[112], 3}, {NULL}};
const Segment SGI126543[] = {{&S13285[77], 6}, {NULL}};
const Segment SGI126544[] = {{&S13286[44], 10}, {NULL}};
const Segment SGI126545[] = {{&S13286[44], 22}, {NULL}};
const Segment SGI126546[] = {{&S13286[30], 6}, {NULL}};
const Segment SGI126547[] = {{&S13286[115], 3}, {NULL}};
const Segment SGI126548[] = {{&S13286[80], 6}, {NULL}};
const Segment SGI126549[] = {{&S13287[44], 12}, {NULL}};
const Segment SGI126550[] = {{&S13287[44], 24}, {NULL}};
const Segment SGI126551[] = {{&S13287[30], 6}, {NULL}};
const Segment SGI126552[] = {{&S13287[117], 3}, {NULL}};
const Segment SGI126553[] = {{&S13287[82], 6}, {NULL}};
const Segment SGI126554[] = {{&S13288[44], 11}, {NULL}};
const Segment SGI126555[] = {{&S13288[44], 23}, {NULL}};
const Segment SGI126556[] = {{&S13288[30], 6}, {NULL}};
const Segment SGI126557[] = {{&S13288[116], 3}, {NULL}};
const Segment SGI126558[] = {{&S13288[81], 6}, {NULL}};
const Segment SGI126559[] = {{&S13289[44], 10}, {NULL}};
const Segment SGI126560[] = {{&S13289[44], 22}, {NULL}};
const Segment SGI126561[] = {{&S13289[30], 6}, {NULL}};
const Segment SGI126562[] = {{&S13289[115], 3}, {NULL}};
const Segment SGI126563[] = {{&S13289[80], 6}, {NULL}};
const Segment SGI126564[] = {{&S13290[44], 8}, {NULL}};
const Segment SGI126565[] = {{&S13290[44], 20}, {NULL}};
const Segment SGI126566[] = {{&S13290[30], 6}, {NULL}};
const Segment SGI126567[] = {{&S13290[113], 3}, {NULL}};
const Segment SGI126568[] = {{&S13290[78], 6}, {NULL}};
const Segment SGI126569[] = {{&S13291[44], 19}, {NULL}};
const Segment SGI126570[] = {{&S13291[44], 31}, {NULL}};
const Segment SGI126571[] = {{&S13291[30], 6}, {NULL}};
const Segment SGI126572[] = {{&S13291[124], 3}, {NULL}};
const Segment SGI126573[] = {{&S13291[89], 6}, {NULL}};
const Segment SGI126574[] = {{&S13292[44], 10}, {NULL}};
const Segment SGI126575[] = {{&S13292[44], 22}, {NULL}};
const Segment SGI126576[] = {{&S13292[30], 6}, {NULL}};
const Segment SGI126577[] = {{&S13292[115], 3}, {NULL}};
const Segment SGI126578[] = {{&S13292[80], 6}, {NULL}};
const Segment SGI126579[] = {{&S13293[44], 12}, {NULL}};
const Segment SGI126580[] = {{&S13293[44], 24}, {NULL}};
const Segment SGI126581[] = {{&S13293[30], 6}, {NULL}};
const Segment SGI126582[] = {{&S13293[117], 3}, {NULL}};
const Segment SGI126583[] = {{&S13293[82], 6}, {NULL}};
const Segment SGI126584[] = {{&S13294[44], 5}, {NULL}};
const Segment SGI126585[] = {{&S13294[44], 17}, {NULL}};
const Segment SGI126586[] = {{&S13294[30], 6}, {NULL}};
const Segment SGI126587[] = {{&S13294[110], 3}, {NULL}};
const Segment SGI126588[] = {{&S13294[75], 6}, {NULL}};
const Segment SGI126589[] = {{&S13295[44], 7}, {NULL}};
const Segment SGI126590[] = {{&S13295[44], 24}, {NULL}};
const Segment SGI126591[] = {{&S13295[30], 6}, {NULL}};
const Segment SGI126592[] = {{&S13295[117], 3}, {NULL}};
const Segment SGI126593[] = {{&S13295[82], 6}, {NULL}};
const Segment SGI126594[] = {{&S13296[44], 4}, {NULL}};
const Segment SGI126595[] = {{&S13296[44], 16}, {NULL}};
const Segment SGI126596[] = {{&S13296[30], 6}, {NULL}};
const Segment SGI126597[] = {{&S13296[109], 3}, {NULL}};
const Segment SGI126598[] = {{&S13296[74], 6}, {NULL}};
const Segment SGI126599[] = {{&S13298[44], 11}, {NULL}};
const Segment SGI126600[] = {{&S13298[44], 23}, {NULL}};
const Segment SGI126601[] = {{&S13298[30], 6}, {NULL}};
const Segment SGI126602[] = {{&S13298[116], 3}, {NULL}};
const Segment SGI126603[] = {{&S13298[81], 6}, {NULL}};
const Segment SGI126604[] = {{&S13299[44], 8}, {NULL}};
const Segment SGI126605[] = {{&S13299[44], 20}, {NULL}};
const Segment SGI126606[] = {{&S13299[30], 6}, {NULL}};
const Segment SGI126607[] = {{&S13299[113], 3}, {NULL}};
const Segment SGI126608[] = {{&S13299[78], 6}, {NULL}};
const Segment SGI126609[] = {{&S13301[44], 14}, {NULL}};
const Segment SGI126610[] = {{&S13301[44], 26}, {NULL}};
const Segment SGI126611[] = {{&S13301[30], 6}, {NULL}};
const Segment SGI126612[] = {{&S13301[119], 3}, {NULL}};
const Segment SGI126613[] = {{&S13301[84], 6}, {NULL}};
const Segment SGI126614[] = {{&S13302[44], 5}, {NULL}};
const Segment SGI126615[] = {{&S13302[44], 17}, {NULL}};
const Segment SGI126616[] = {{&S13302[30], 6}, {NULL}};
const Segment SGI126617[] = {{&S13302[110], 3}, {NULL}};
const Segment SGI126618[] = {{&S13302[75], 6}, {NULL}};
const Segment SGI126619[] = {{&S13304[44], 9}, {NULL}};
const Segment SGI126620[] = {{&S13304[44], 21}, {NULL}};
const Segment SGI126621[] = {{&S13304[30], 6}, {NULL}};
const Segment SGI126622[] = {{&S13304[114], 3}, {NULL}};
const Segment SGI126623[] = {{&S13304[79], 6}, {NULL}};
const Segment SGI126624[] = {{&S13305[44], 10}, {NULL}};
const Segment SGI126625[] = {{&S13305[44], 22}, {NULL}};
const Segment SGI126626[] = {{&S13305[30], 6}, {NULL}};
const Segment SGI126627[] = {{&S13305[115], 3}, {NULL}};
const Segment SGI126628[] = {{&S13305[80], 6}, {NULL}};
const Segment SGI126629[] = {{&S13306[44], 11}, {NULL}};
const Segment SGI126630[] = {{&S13306[44], 23}, {NULL}};
const Segment SGI126631[] = {{&S13306[30], 6}, {NULL}};
const Segment SGI126632[] = {{&S13306[116], 3}, {NULL}};
const Segment SGI126633[] = {{&S13306[81], 6}, {NULL}};
const Segment SGI126634[] = {{&S13307[44], 7}, {NULL}};
const Segment SGI126635[] = {{&S13307[44], 19}, {NULL}};
const Segment SGI126636[] = {{&S13307[30], 6}, {NULL}};
const Segment SGI126637[] = {{&S13307[112], 3}, {NULL}};
const Segment SGI126638[] = {{&S13307[77], 6}, {NULL}};
const Segment SGI126639[] = {{&S13309[44], 4}, {NULL}};
const Segment SGI126640[] = {{&S13309[44], 16}, {NULL}};
const Segment SGI126641[] = {{&S13309[30], 6}, {NULL}};
const Segment SGI126642[] = {{&S13309[109], 3}, {NULL}};
const Segment SGI126643[] = {{&S13309[74], 6}, {NULL}};
const Segment SGI126644[] = {{&S13310[44], 9}, {NULL}};
const Segment SGI126645[] = {{&S13310[44], 16}, {NULL}};
const Segment SGI126646[] = {{&S13310[30], 6}, {NULL}};
const Segment SGI126647[] = {{&S13310[109], 3}, {NULL}};
const Segment SGI126648[] = {{&S13310[74], 6}, {NULL}};
const Segment SGI126649[] = {{&S13311[44], 9}, {NULL}};
const Segment SGI126650[] = {{&S13311[44], 21}, {NULL}};
const Segment SGI126651[] = {{&S13311[30], 6}, {NULL}};
const Segment SGI126652[] = {{&S13311[114], 3}, {NULL}};
const Segment SGI126653[] = {{&S13311[79], 6}, {NULL}};
const Segment SGI126654[] = {{&S13312[44], 7}, {NULL}};
const Segment SGI126655[] = {{&S13312[44], 19}, {NULL}};
const Segment SGI126656[] = {{&S13312[30], 6}, {NULL}};
const Segment SGI126657[] = {{&S13312[112], 3}, {NULL}};
const Segment SGI126658[] = {{&S13312[77], 6}, {NULL}};
const Segment SGI126659[] = {{&S13313[44], 11}, {NULL}};
const Segment SGI126660[] = {{&S13313[44], 23}, {NULL}};
const Segment SGI126661[] = {{&S13313[30], 6}, {NULL}};
const Segment SGI126662[] = {{&S13313[116], 3}, {NULL}};
const Segment SGI126663[] = {{&S13313[81], 6}, {NULL}};
const Segment SGI126664[] = {{&S13314[44], 9}, {NULL}};
const Segment SGI126665[] = {{&S13314[44], 21}, {NULL}};
const Segment SGI126666[] = {{&S13314[30], 6}, {NULL}};
const Segment SGI126667[] = {{&S13314[114], 3}, {NULL}};
const Segment SGI126668[] = {{&S13314[79], 6}, {NULL}};
const Segment SGI126669[] = {{&S13315[44], 12}, {NULL}};
const Segment SGI126670[] = {{&S13315[44], 19}, {NULL}};
const Segment SGI126671[] = {{&S13315[30], 6}, {NULL}};
const Segment SGI126672[] = {{&S13315[112], 3}, {NULL}};
const Segment SGI126673[] = {{&S13315[77], 6}, {NULL}};
const Segment SGI126674[] = {{&S13316[44], 12}, {NULL}};
const Segment SGI126675[] = {{&S13316[44], 24}, {NULL}};
const Segment SGI126676[] = {{&S13316[30], 6}, {NULL}};
const Segment SGI126677[] = {{&S13316[117], 3}, {NULL}};
const Segment SGI126678[] = {{&S13316[82], 6}, {NULL}};
const Segment SGI126679[] = {{&S13317[44], 7}, {NULL}};
const Segment SGI126680[] = {{&S13317[44], 19}, {NULL}};
const Segment SGI126681[] = {{&S13317[30], 6}, {NULL}};
const Segment SGI126682[] = {{&S13317[112], 3}, {NULL}};
const Segment SGI126683[] = {{&S13317[77], 6}, {NULL}};
const Segment SGI126684[] = {{&S13318[44], 12}, {NULL}};
const Segment SGI126685[] = {{&S13318[44], 24}, {NULL}};
const Segment SGI126686[] = {{&S13318[30], 6}, {NULL}};
const Segment SGI126687[] = {{&S13318[117], 3}, {NULL}};
const Segment SGI126688[] = {{&S13318[82], 6}, {NULL}};
const Segment SGI126689[] = {{&S13320[44], 11}, {NULL}};
const Segment SGI126690[] = {{&S13320[44], 23}, {NULL}};
const Segment SGI126691[] = {{&S13320[30], 6}, {NULL}};
const Segment SGI126692[] = {{&S13320[116], 3}, {NULL}};
const Segment SGI126693[] = {{&S13320[81], 6}, {NULL}};
const Segment SGI126694[] = {{&S13321[44], 8}, {NULL}};
const Segment SGI126695[] = {{&S13321[44], 20}, {NULL}};
const Segment SGI126696[] = {{&S13321[30], 6}, {NULL}};
const Segment SGI126697[] = {{&S13321[113], 3}, {NULL}};
const Segment SGI126698[] = {{&S13321[78], 6}, {NULL}};
const Segment SGI126699[] = {{&S13323[44], 6}, {NULL}};
const Segment SGI126700[] = {{&S13323[44], 18}, {NULL}};
const Segment SGI126701[] = {{&S13323[30], 6}, {NULL}};
const Segment SGI126702[] = {{&S13323[111], 3}, {NULL}};
const Segment SGI126703[] = {{&S13323[76], 6}, {NULL}};
const Segment SGI126704[] = {{&S13324[44], 20}, {NULL}};
const Segment SGI126705[] = {{&S13324[30], 6}, {NULL}};
const Segment SGI126706[] = {{&S13324[127], 3}, {NULL}};
const Segment SGI126707[] = {{&S13324[92], 6}, {NULL}};
const Segment SGI126708[] = {{&S13325[44], 9}, {NULL}};
const Segment SGI126709[] = {{&S13325[44], 21}, {NULL}};
const Segment SGI126710[] = {{&S13325[30], 6}, {NULL}};
const Segment SGI126711[] = {{&S13325[114], 3}, {NULL}};
const Segment SGI126712[] = {{&S13325[79], 6}, {NULL}};
const Segment SGI126713[] = {{&S13326[44], 7}, {NULL}};
const Segment SGI126714[] = {{&S13326[44], 19}, {NULL}};
const Segment SGI126715[] = {{&S13326[30], 6}, {NULL}};
const Segment SGI126716[] = {{&S13326[112], 3}, {NULL}};
const Segment SGI126717[] = {{&S13326[77], 6}, {NULL}};
const Segment SGI126718[] = {{&S13327[44], 12}, {NULL}};
const Segment SGI126719[] = {{&S13327[44], 24}, {NULL}};
const Segment SGI126720[] = {{&S13327[30], 6}, {NULL}};
const Segment SGI126721[] = {{&S13327[117], 3}, {NULL}};
const Segment SGI126722[] = {{&S13327[82], 6}, {NULL}};
const Segment SGI126723[] = {{&S13328[44], 17}, {NULL}};
const Segment SGI126724[] = {{&S13328[44], 29}, {NULL}};
const Segment SGI126725[] = {{&S13328[30], 6}, {NULL}};
const Segment SGI126726[] = {{&S13328[122], 3}, {NULL}};
const Segment SGI126727[] = {{&S13328[87], 6}, {NULL}};
const Segment SGI126728[] = {{&S13329[44], 7}, {NULL}};
const Segment SGI126729[] = {{&S13329[44], 19}, {NULL}};
const Segment SGI126730[] = {{&S13329[30], 6}, {NULL}};
const Segment SGI126731[] = {{&S13329[112], 3}, {NULL}};
const Segment SGI126732[] = {{&S13329[77], 6}, {NULL}};
const Segment SGI126733[] = {{&S13331[44], 6}, {NULL}};
const Segment SGI126734[] = {{&S13331[44], 18}, {NULL}};
const Segment SGI126735[] = {{&S13331[30], 6}, {NULL}};
const Segment SGI126736[] = {{&S13331[111], 3}, {NULL}};
const Segment SGI126737[] = {{&S13331[76], 6}, {NULL}};
const Segment SGI126738[] = {{&S13333[44], 11}, {NULL}};
const Segment SGI126739[] = {{&S13333[44], 23}, {NULL}};
const Segment SGI126740[] = {{&S13333[30], 6}, {NULL}};
const Segment SGI126741[] = {{&S13333[116], 3}, {NULL}};
const Segment SGI126742[] = {{&S13333[81], 6}, {NULL}};
const Segment SGI126743[] = {{&S13334[44], 12}, {NULL}};
const Segment SGI126744[] = {{&S13334[44], 24}, {NULL}};
const Segment SGI126745[] = {{&S13334[30], 6}, {NULL}};
const Segment SGI126746[] = {{&S13334[117], 3}, {NULL}};
const Segment SGI126747[] = {{&S13334[82], 6}, {NULL}};
const Segment SGI126748[] = {{&S13335[44], 13}, {NULL}};
const Segment SGI126749[] = {{&S13335[44], 25}, {NULL}};
const Segment SGI126750[] = {{&S13335[30], 6}, {NULL}};
const Segment SGI126751[] = {{&S13335[118], 3}, {NULL}};
const Segment SGI126752[] = {{&S13335[83], 6}, {NULL}};
const Segment SGI126753[] = {{&S13336[44], 31}, {NULL}};
const Segment SGI126754[] = {{&S13336[44], 43}, {NULL}};
const Segment SGI126755[] = {{&S13336[30], 6}, {NULL}};
const Segment SGI126756[] = {{&S13336[136], 3}, {NULL}};
const Segment SGI126757[] = {{&S13336[101], 6}, {NULL}};
const Segment SGI126758[] = {{&S13337[44], 10}, {NULL}};
const Segment SGI126759[] = {{&S13337[44], 22}, {NULL}};
const Segment SGI126760[] = {{&S13337[30], 6}, {NULL}};
const Segment SGI126761[] = {{&S13337[115], 3}, {NULL}};
const Segment SGI126762[] = {{&S13337[80], 6}, {NULL}};
const Segment SGI126763[] = {{&S13338[44], 8}, {NULL}};
const Segment SGI126764[] = {{&S13338[44], 20}, {NULL}};
const Segment SGI126765[] = {{&S13338[30], 6}, {NULL}};
const Segment SGI126766[] = {{&S13338[113], 3}, {NULL}};
const Segment SGI126767[] = {{&S13338[78], 6}, {NULL}};
const Segment SGI126768[] = {{&S13341[44], 22}, {NULL}};
const Segment SGI126769[] = {{&S13341[44], 34}, {NULL}};
const Segment SGI126770[] = {{&S13341[30], 6}, {NULL}};
const Segment SGI126771[] = {{&S13341[127], 3}, {NULL}};
const Segment SGI126772[] = {{&S13341[92], 6}, {NULL}};
const Segment SGI126773[] = {{&S13342[44], 18}, {NULL}};
const Segment SGI126774[] = {{&S13342[44], 30}, {NULL}};
const Segment SGI126775[] = {{&S13342[30], 6}, {NULL}};
const Segment SGI126776[] = {{&S13342[123], 3}, {NULL}};
const Segment SGI126777[] = {{&S13342[88], 6}, {NULL}};
const Segment SGI126778[] = {{&S13344[44], 8}, {NULL}};
const Segment SGI126779[] = {{&S13344[44], 20}, {NULL}};
const Segment SGI126780[] = {{&S13344[30], 6}, {NULL}};
const Segment SGI126781[] = {{&S13344[113], 3}, {NULL}};
const Segment SGI126782[] = {{&S13344[78], 6}, {NULL}};
const Segment SGI126783[] = {{&S13347[44], 6}, {NULL}};
const Segment SGI126784[] = {{&S13347[44], 18}, {NULL}};
const Segment SGI126785[] = {{&S13347[30], 6}, {NULL}};
const Segment SGI126786[] = {{&S13347[111], 3}, {NULL}};
const Segment SGI126787[] = {{&S13347[76], 6}, {NULL}};
const Segment SGI126788[] = {{&S13349[44], 16}, {NULL}};
const Segment SGI126789[] = {{&S13349[44], 28}, {NULL}};
const Segment SGI126790[] = {{&S13349[30], 6}, {NULL}};
const Segment SGI126791[] = {{&S13349[121], 3}, {NULL}};
const Segment SGI126792[] = {{&S13349[86], 6}, {NULL}};
const Segment SGI126793[] = {{&S13352[44], 9}, {NULL}};
const Segment SGI126794[] = {{&S13352[44], 21}, {NULL}};
const Segment SGI126795[] = {{&S13352[30], 6}, {NULL}};
const Segment SGI126796[] = {{&S13352[114], 3}, {NULL}};
const Segment SGI126797[] = {{&S13352[79], 6}, {NULL}};
const Segment SGI126798[] = {{&S13353[44], 9}, {NULL}};
const Segment SGI126799[] = {{&S13353[44], 21}, {NULL}};
const Segment SGI126800[] = {{&S13353[30], 6}, {NULL}};
const Segment SGI126801[] = {{&S13353[114], 3}, {NULL}};
const Segment SGI126802[] = {{&S13353[79], 6}, {NULL}};
const Segment SGI126803[] = {{&S13354[44], 57}, {NULL}};
const Segment SGI126804[] = {{&S13354[30], 6}, {NULL}};
const Segment SGI126805[] = {{&S13354[164], 3}, {NULL}};
const Segment SGI126806[] = {{&S13354[129], 6}, {NULL}};
const Segment SGI126807[] = {{&S13356[44], 6}, {NULL}};
const Segment SGI126808[] = {{&S13356[44], 18}, {NULL}};
const Segment SGI126809[] = {{&S13356[30], 6}, {NULL}};
const Segment SGI126810[] = {{&S13356[111], 3}, {NULL}};
const Segment SGI126811[] = {{&S13356[76], 6}, {NULL}};
const Segment SGI126812[] = {{&S13357[44], 8}, {NULL}};
const Segment SGI126813[] = {{&S13357[44], 20}, {NULL}};
const Segment SGI126814[] = {{&S13357[30], 6}, {NULL}};
const Segment SGI126815[] = {{&S13357[113], 3}, {NULL}};
const Segment SGI126816[] = {{&S13357[78], 6}, {NULL}};
const Segment SGI126817[] = {{&S13359[44], 8}, {NULL}};
const Segment SGI126818[] = {{&S13359[44], 20}, {NULL}};
const Segment SGI126819[] = {{&S13359[30], 6}, {NULL}};
const Segment SGI126820[] = {{&S13359[113], 3}, {NULL}};
const Segment SGI126821[] = {{&S13359[78], 6}, {NULL}};
const Segment SGI126822[] = {{&S13360[44], 9}, {NULL}};
const Segment SGI126823[] = {{&S13360[44], 21}, {NULL}};
const Segment SGI126824[] = {{&S13360[30], 6}, {NULL}};
const Segment SGI126825[] = {{&S13360[114], 3}, {NULL}};
const Segment SGI126826[] = {{&S13360[79], 6}, {NULL}};
const Segment SGI126827[] = {{&S13362[44], 19}, {NULL}};
const Segment SGI126828[] = {{&S13362[44], 31}, {NULL}};
const Segment SGI126829[] = {{&S13362[30], 6}, {NULL}};
const Segment SGI126830[] = {{&S13362[124], 3}, {NULL}};
const Segment SGI126831[] = {{&S13362[89], 6}, {NULL}};
const Segment SGI126832[] = {{&S13363[44], 8}, {NULL}};
const Segment SGI126833[] = {{&S13363[44], 20}, {NULL}};
const Segment SGI126834[] = {{&S13363[30], 6}, {NULL}};
const Segment SGI126835[] = {{&S13363[113], 3}, {NULL}};
const Segment SGI126836[] = {{&S13363[78], 6}, {NULL}};
const Segment SGI126837[] = {{&S13364[44], 14}, {NULL}};
const Segment SGI126838[] = {{&S13364[44], 26}, {NULL}};
const Segment SGI126839[] = {{&S13364[30], 6}, {NULL}};
const Segment SGI126840[] = {{&S13364[119], 3}, {NULL}};
const Segment SGI126841[] = {{&S13364[84], 6}, {NULL}};
const Segment SGI126842[] = {{&S13366[44], 8}, {NULL}};
const Segment SGI126843[] = {{&S13366[44], 20}, {NULL}};
const Segment SGI126844[] = {{&S13366[30], 6}, {NULL}};
const Segment SGI126845[] = {{&S13366[113], 3}, {NULL}};
const Segment SGI126846[] = {{&S13366[78], 6}, {NULL}};
const Segment SGI126847[] = {{&S13368[44], 83}, {NULL}};
const Segment SGI126848[] = {{&S13368[44], 104}, {NULL}};
const Segment SGI126849[] = {{&S13368[30], 6}, {NULL}};
const Segment SGI126850[] = {{&S13368[92], 3}, {NULL}};
const Segment SGI126851[] = {{&S13368[121], 5}, {NULL}};
const Segment SGI126852[] = {{&S13369[44], 8}, {NULL}};
const Segment SGI126853[] = {{&S13369[44], 20}, {NULL}};
const Segment SGI126854[] = {{&S13369[30], 6}, {NULL}};
const Segment SGI126855[] = {{&S13369[113], 3}, {NULL}};
const Segment SGI126856[] = {{&S13369[78], 6}, {NULL}};
const Segment SGI126857[] = {{&S13370[44], 9}, {NULL}};
const Segment SGI126858[] = {{&S13370[44], 16}, {NULL}};
const Segment SGI126859[] = {{&S13370[30], 6}, {NULL}};
const Segment SGI126860[] = {{&S13370[109], 3}, {NULL}};
const Segment SGI126861[] = {{&S13370[74], 6}, {NULL}};
const Segment SGI126862[] = {{&S13371[44], 8}, {NULL}};
const Segment SGI126863[] = {{&S13371[44], 20}, {NULL}};
const Segment SGI126864[] = {{&S13371[30], 6}, {NULL}};
const Segment SGI126865[] = {{&S13371[113], 3}, {NULL}};
const Segment SGI126866[] = {{&S13371[78], 6}, {NULL}};
const Segment SGI126867[] = {{&S13372[44], 8}, {NULL}};
const Segment SGI126868[] = {{&S13372[44], 20}, {NULL}};
const Segment SGI126869[] = {{&S13372[30], 6}, {NULL}};
const Segment SGI126870[] = {{&S13372[113], 3}, {NULL}};
const Segment SGI126871[] = {{&S13372[78], 6}, {NULL}};
const Segment SGI126872[] = {{&S13373[44], 19}, {NULL}};
const Segment SGI126873[] = {{&S13373[44], 31}, {NULL}};
const Segment SGI126874[] = {{&S13373[30], 6}, {NULL}};
const Segment SGI126875[] = {{&S13373[124], 3}, {NULL}};
const Segment SGI126876[] = {{&S13373[89], 6}, {NULL}};
const Segment SGI126877[] = {{&S13374[44], 10}, {NULL}};
const Segment SGI126878[] = {{&S13374[44], 22}, {NULL}};
const Segment SGI126879[] = {{&S13374[30], 6}, {NULL}};
const Segment SGI126880[] = {{&S13374[115], 3}, {NULL}};
const Segment SGI126881[] = {{&S13374[80], 6}, {NULL}};
const Segment SGI126882[] = {{&S13375[44], 5}, {NULL}};
const Segment SGI126883[] = {{&S13375[44], 17}, {NULL}};
const Segment SGI126884[] = {{&S13375[30], 6}, {NULL}};
const Segment SGI126885[] = {{&S13375[110], 3}, {NULL}};
const Segment SGI126886[] = {{&S13375[75], 6}, {NULL}};
const Segment SGI126887[] = {{&S13376[44], 14}, {NULL}};
const Segment SGI126888[] = {{&S13376[44], 26}, {NULL}};
const Segment SGI126889[] = {{&S13376[30], 6}, {NULL}};
const Segment SGI126890[] = {{&S13376[119], 3}, {NULL}};
const Segment SGI126891[] = {{&S13376[84], 6}, {NULL}};
const Segment SGI126892[] = {{&S13377[44], 4}, {NULL}};
const Segment SGI126893[] = {{&S13377[44], 16}, {NULL}};
const Segment SGI126894[] = {{&S13377[30], 6}, {NULL}};
const Segment SGI126895[] = {{&S13377[109], 3}, {NULL}};
const Segment SGI126896[] = {{&S13377[74], 6}, {NULL}};
const Segment SGI126897[] = {{&S13378[44], 8}, {NULL}};
const Segment SGI126898[] = {{&S13378[44], 20}, {NULL}};
const Segment SGI126899[] = {{&S13378[30], 6}, {NULL}};
const Segment SGI126900[] = {{&S13378[113], 3}, {NULL}};
const Segment SGI126901[] = {{&S13378[78], 6}, {NULL}};
const Segment SGI126902[] = {{&S13379[44], 16}, {NULL}};
const Segment SGI126903[] = {{&S13379[44], 28}, {NULL}};
const Segment SGI126904[] = {{&S13379[30], 6}, {NULL}};
const Segment SGI126905[] = {{&S13379[121], 3}, {NULL}};
const Segment SGI126906[] = {{&S13379[86], 6}, {NULL}};
const Segment SGI126907[] = {{&S13380[44], 12}, {NULL}};
const Segment SGI126908[] = {{&S13380[44], 24}, {NULL}};
const Segment SGI126909[] = {{&S13380[30], 6}, {NULL}};
const Segment SGI126910[] = {{&S13380[117], 3}, {NULL}};
const Segment SGI126911[] = {{&S13380[82], 6}, {NULL}};
const Segment SGI126912[] = {{&S13381[44], 8}, {NULL}};
const Segment SGI126913[] = {{&S13381[44], 20}, {NULL}};
const Segment SGI126914[] = {{&S13381[30], 6}, {NULL}};
const Segment SGI126915[] = {{&S13381[113], 3}, {NULL}};
const Segment SGI126916[] = {{&S13381[78], 6}, {NULL}};
const Segment SGI126917[] = {{&S13382[44], 9}, {NULL}};
const Segment SGI126918[] = {{&S13382[44], 21}, {NULL}};
const Segment SGI126919[] = {{&S13382[30], 6}, {NULL}};
const Segment SGI126920[] = {{&S13382[114], 3}, {NULL}};
const Segment SGI126921[] = {{&S13382[79], 6}, {NULL}};
const Segment SGI126922[] = {{&S13383[44], 13}, {NULL}};
const Segment SGI126923[] = {{&S13383[44], 25}, {NULL}};
const Segment SGI126924[] = {{&S13383[30], 6}, {NULL}};
const Segment SGI126925[] = {{&S13383[118], 3}, {NULL}};
const Segment SGI126926[] = {{&S13383[83], 6}, {NULL}};
const Segment SGI126927[] = {{&S13384[44], 9}, {NULL}};
const Segment SGI126928[] = {{&S13384[44], 21}, {NULL}};
const Segment SGI126929[] = {{&S13384[30], 6}, {NULL}};
const Segment SGI126930[] = {{&S13384[114], 3}, {NULL}};
const Segment SGI126931[] = {{&S13384[79], 6}, {NULL}};
const Segment SGI126932[] = {{&S13385[44], 22}, {NULL}};
const Segment SGI126933[] = {{&S13385[44], 34}, {NULL}};
const Segment SGI126934[] = {{&S13385[30], 6}, {NULL}};
const Segment SGI126935[] = {{&S13385[127], 3}, {NULL}};
const Segment SGI126936[] = {{&S13385[92], 6}, {NULL}};
const Segment SGI126937[] = {{&S13386[44], 8}, {NULL}};
const Segment SGI126938[] = {{&S13386[44], 20}, {NULL}};
const Segment SGI126939[] = {{&S13386[30], 6}, {NULL}};
const Segment SGI126940[] = {{&S13386[113], 3}, {NULL}};
const Segment SGI126941[] = {{&S13386[78], 6}, {NULL}};
const Segment SGI126942[] = {{&S13387[44], 7}, {NULL}};
const Segment SGI126943[] = {{&S13387[44], 19}, {NULL}};
const Segment SGI126944[] = {{&S13387[30], 6}, {NULL}};
const Segment SGI126945[] = {{&S13387[112], 3}, {NULL}};
const Segment SGI126946[] = {{&S13387[77], 6}, {NULL}};
const Segment SGI126947[] = {{&S13388[44], 21}, {NULL}};
const Segment SGI126948[] = {{&S13388[44], 33}, {NULL}};
const Segment SGI126949[] = {{&S13388[30], 6}, {NULL}};
const Segment SGI126950[] = {{&S13388[126], 3}, {NULL}};
const Segment SGI126951[] = {{&S13388[91], 6}, {NULL}};
const Segment SGI126952[] = {{&S13389[44], 13}, {NULL}};
const Segment SGI126953[] = {{&S13389[44], 25}, {NULL}};
const Segment SGI126954[] = {{&S13389[30], 6}, {NULL}};
const Segment SGI126955[] = {{&S13389[118], 3}, {NULL}};
const Segment SGI126956[] = {{&S13389[83], 6}, {NULL}};
const Segment SGI126957[] = {{&S13390[44], 13}, {NULL}};
const Segment SGI126958[] = {{&S13390[44], 25}, {NULL}};
const Segment SGI126959[] = {{&S13390[30], 6}, {NULL}};
const Segment SGI126960[] = {{&S13390[118], 3}, {NULL}};
const Segment SGI126961[] = {{&S13390[83], 6}, {NULL}};
const Segment SGI126962[] = {{&S13391[44], 13}, {NULL}};
const Segment SGI126963[] = {{&S13391[44], 20}, {NULL}};
const Segment SGI126964[] = {{&S13391[30], 6}, {NULL}};
const Segment SGI126965[] = {{&S13391[113], 3}, {NULL}};
const Segment SGI126966[] = {{&S13391[78], 6}, {NULL}};
const Segment SGI126967[] = {{&S13392[44], 14}, {NULL}};
const Segment SGI126968[] = {{&S13392[44], 26}, {NULL}};
const Segment SGI126969[] = {{&S13392[30], 6}, {NULL}};
const Segment SGI126970[] = {{&S13392[119], 3}, {NULL}};
const Segment SGI126971[] = {{&S13392[84], 6}, {NULL}};
const Segment SGI126972[] = {{&S13393[44], 7}, {NULL}};
const Segment SGI126973[] = {{&S13393[44], 24}, {NULL}};
const Segment SGI126974[] = {{&S13393[30], 6}, {NULL}};
const Segment SGI126975[] = {{&S13393[117], 3}, {NULL}};
const Segment SGI126976[] = {{&S13393[82], 6}, {NULL}};
const Segment SGI126977[] = {{&S13394[44], 8}, {NULL}};
const Segment SGI126978[] = {{&S13394[44], 20}, {NULL}};
const Segment SGI126979[] = {{&S13394[30], 6}, {NULL}};
const Segment SGI126980[] = {{&S13394[113], 3}, {NULL}};
const Segment SGI126981[] = {{&S13394[78], 6}, {NULL}};
const Segment SGI126982[] = {{&S13395[44], 8}, {NULL}};
const Segment SGI126983[] = {{&S13395[44], 20}, {NULL}};
const Segment SGI126984[] = {{&S13395[30], 6}, {NULL}};
const Segment SGI126985[] = {{&S13395[113], 3}, {NULL}};
const Segment SGI126986[] = {{&S13395[78], 6}, {NULL}};
const Segment SGI126987[] = {{&S13396[44], 10}, {NULL}};
const Segment SGI126988[] = {{&S13396[44], 22}, {NULL}};
const Segment SGI126989[] = {{&S13396[30], 6}, {NULL}};
const Segment SGI126990[] = {{&S13396[115], 3}, {NULL}};
const Segment SGI126991[] = {{&S13396[80], 6}, {NULL}};
const Segment SGI126992[] = {{&S13397[44], 10}, {NULL}};
const Segment SGI126993[] = {{&S13397[44], 17}, {NULL}};
const Segment SGI126994[] = {{&S13397[30], 6}, {NULL}};
const Segment SGI126995[] = {{&S13397[110], 3}, {NULL}};
const Segment SGI126996[] = {{&S13397[75], 6}, {NULL}};
const Segment SGI126997[] = {{&S13399[44], 11}, {NULL}};
const Segment SGI126998[] = {{&S13399[44], 23}, {NULL}};
const Segment SGI126999[] = {{&S13399[30], 6}, {NULL}};
const Segment SGI127000[] = {{&S13399[116], 3}, {NULL}};
const Segment SGI127001[] = {{&S13399[81], 6}, {NULL}};
const Segment SGI127002[] = {{&S13400[44], 75}, {NULL}};
const Segment SGI127003[] = {{&S13400[44], 96}, {NULL}};
const Segment SGI127004[] = {{&S13400[30], 6}, {NULL}};
const Segment SGI127005[] = {{&S13400[84], 3}, {NULL}};
const Segment SGI127006[] = {{&S13400[113], 5}, {NULL}};
const Segment SGI127007[] = {{&S13402[44], 28}, {NULL}};
const Segment SGI127008[] = {{&S13402[44], 40}, {NULL}};
const Segment SGI127009[] = {{&S13402[30], 6}, {NULL}};
const Segment SGI127010[] = {{&S13402[133], 3}, {NULL}};
const Segment SGI127011[] = {{&S13402[98], 6}, {NULL}};
const Segment SGI127012[] = {{&S13403[44], 8}, {NULL}};
const Segment SGI127013[] = {{&S13403[44], 20}, {NULL}};
const Segment SGI127014[] = {{&S13403[30], 6}, {NULL}};
const Segment SGI127015[] = {{&S13403[113], 3}, {NULL}};
const Segment SGI127016[] = {{&S13403[78], 6}, {NULL}};
const Segment SGI127017[] = {{&S13404[44], 9}, {NULL}};
const Segment SGI127018[] = {{&S13404[44], 21}, {NULL}};
const Segment SGI127019[] = {{&S13404[30], 6}, {NULL}};
const Segment SGI127020[] = {{&S13404[114], 3}, {NULL}};
const Segment SGI127021[] = {{&S13404[79], 6}, {NULL}};
const Segment SGI127022[] = {{&S13405[44], 7}, {NULL}};
const Segment SGI127023[] = {{&S13405[44], 19}, {NULL}};
const Segment SGI127024[] = {{&S13405[30], 6}, {NULL}};
const Segment SGI127025[] = {{&S13405[112], 3}, {NULL}};
const Segment SGI127026[] = {{&S13405[77], 6}, {NULL}};
const Segment SGI127027[] = {{&S13406[44], 6}, {NULL}};
const Segment SGI127028[] = {{&S13406[44], 18}, {NULL}};
const Segment SGI127029[] = {{&S13406[30], 6}, {NULL}};
const Segment SGI127030[] = {{&S13406[111], 3}, {NULL}};
const Segment SGI127031[] = {{&S13406[76], 6}, {NULL}};
const Segment SGI127032[] = {{&S13407[38], 4}, {NULL}};
const Segment SGI127033[] = {{&S13407[31], 6}, {NULL}};
const Segment SGI127034[] = {{&S13408[44], 11}, {NULL}};
const Segment SGI127035[] = {{&S13408[44], 23}, {NULL}};
const Segment SGI127036[] = {{&S13408[30], 6}, {NULL}};
const Segment SGI127037[] = {{&S13408[116], 3}, {NULL}};
const Segment SGI127038[] = {{&S13408[81], 6}, {NULL}};
const Segment SGI127039[] = {{&S13409[44], 11}, {NULL}};
const Segment SGI127040[] = {{&S13409[44], 23}, {NULL}};
const Segment SGI127041[] = {{&S13409[30], 6}, {NULL}};
const Segment SGI127042[] = {{&S13409[116], 3}, {NULL}};
const Segment SGI127043[] = {{&S104772[44], 15}, {NULL}};
const Segment SGI127044[] = {{&S104772[44], 25}, {NULL}};
const Segment SGI127045[] = {{&S13409[81], 6}, {NULL}};
const Segment SGI127046[] = {{&S104772[30], 6}, {NULL}};
const Segment SGI127047[] = {{&S104772[31], 3}, {NULL}};
const Segment SGI127048[] = {{&S104772[83], 6}, {NULL}};
const Segment SGI127049[] = {{&S13410[44], 10}, {NULL}};
const Segment SGI127050[] = {{&S13410[44], 22}, {NULL}};
const Segment SGI127051[] = {{&S13410[30], 6}, {NULL}};
const Segment SGI127052[] = {{&S13410[115], 3}, {NULL}};
const Segment SGI127053[] = {{&S13410[80], 6}, {NULL}};
const Segment SGI127054[] = {{&S13411[44], 8}, {NULL}};
const Segment SGI127055[] = {{&S13411[44], 20}, {NULL}};
const Segment SGI127056[] = {{&S13411[30], 6}, {NULL}};
const Segment SGI127057[] = {{&S13411[113], 3}, {NULL}};
const Segment SGI127058[] = {{&S13411[78], 6}, {NULL}};
const Segment SGI127059[] = {{&S13412[44], 13}, {NULL}};
const Segment SGI127060[] = {{&S13412[44], 25}, {NULL}};
const Segment SGI127061[] = {{&S13412[30], 6}, {NULL}};
const Segment SGI127062[] = {{&S13412[118], 3}, {NULL}};
const Segment SGI127063[] = {{&S13412[83], 6}, {NULL}};
const Segment SGI127064[] = {{&S13413[44], 9}, {NULL}};
const Segment SGI127065[] = {{&S13413[44], 21}, {NULL}};
const Segment SGI127066[] = {{&S13413[30], 6}, {NULL}};
const Segment SGI127067[] = {{&S13413[114], 3}, {NULL}};
const Segment SGI127068[] = {{&S13413[79], 6}, {NULL}};
const Segment SGI127069[] = {{&S13414[44], 7}, {NULL}};
const Segment SGI127070[] = {{&S13414[44], 19}, {NULL}};
const Segment SGI127071[] = {{&S13414[30], 6}, {NULL}};
const Segment SGI127072[] = {{&S13414[112], 3}, {NULL}};
const Segment SGI127073[] = {{&S13414[77], 6}, {NULL}};
const Segment SGI127074[] = {{&S13415[44], 9}, {NULL}};
const Segment SGI127075[] = {{&S13415[44], 21}, {NULL}};
const Segment SGI127076[] = {{&S13415[30], 6}, {NULL}};
const Segment SGI127077[] = {{&S13415[114], 3}, {NULL}};
const Segment SGI127078[] = {{&S13415[79], 6}, {NULL}};
const Segment SGI127079[] = {{&S13416[44], 19}, {NULL}};
const Segment SGI127080[] = {{&S13416[44], 31}, {NULL}};
const Segment SGI127081[] = {{&S13416[30], 6}, {NULL}};
const Segment SGI127082[] = {{&S13416[124], 3}, {NULL}};
const Segment SGI127083[] = {{&S13416[89], 6}, {NULL}};
const Segment SGI127084[] = {{&S13418[44], 8}, {NULL}};
const Segment SGI127085[] = {{&S13418[44], 20}, {NULL}};
const Segment SGI127086[] = {{&S13418[30], 6}, {NULL}};
const Segment SGI127087[] = {{&S13418[113], 3}, {NULL}};
const Segment SGI127088[] = {{&S13418[78], 6}, {NULL}};
const Segment SGI127089[] = {{&S13419[44], 8}, {NULL}};
const Segment SGI127090[] = {{&S13419[44], 20}, {NULL}};
const Segment SGI127091[] = {{&S13419[30], 6}, {NULL}};
const Segment SGI127092[] = {{&S13419[113], 3}, {NULL}};
const Segment SGI127093[] = {{&S13419[78], 6}, {NULL}};
const Segment SGI127094[] = {{&S13420[44], 7}, {NULL}};
const Segment SGI127095[] = {{&S13420[44], 19}, {NULL}};
const Segment SGI127096[] = {{&S13420[30], 6}, {NULL}};
const Segment SGI127097[] = {{&S13420[112], 3}, {NULL}};
const Segment SGI127098[] = {{&S13420[77], 6}, {NULL}};
const Segment SGI127099[] = {{&S13421[44], 12}, {NULL}};
const Segment SGI127100[] = {{&S13421[44], 24}, {NULL}};
const Segment SGI127101[] = {{&S13421[30], 6}, {NULL}};
const Segment SGI127102[] = {{&S13421[117], 3}, {NULL}};
const Segment SGI127103[] = {{&S13421[82], 6}, {NULL}};
const Segment SGI127104[] = {{&S13422[44], 7}, {NULL}};
const Segment SGI127105[] = {{&S13422[44], 19}, {NULL}};
const Segment SGI127106[] = {{&S13422[30], 6}, {NULL}};
const Segment SGI127107[] = {{&S13422[112], 3}, {NULL}};
const Segment SGI127108[] = {{&S13422[77], 6}, {NULL}};
const Segment SGI127109[] = {{&S13423[44], 10}, {NULL}};
const Segment SGI127110[] = {{&S13423[44], 22}, {NULL}};
const Segment SGI127111[] = {{&S13423[30], 6}, {NULL}};
const Segment SGI127112[] = {{&S13423[115], 3}, {NULL}};
const Segment SGI127113[] = {{&S13423[80], 6}, {NULL}};
const Segment SGI127114[] = {{&S104801[44], 10}, {NULL}};
const Segment SGI127115[] = {{&S104801[44], 22}, {NULL}};
const Segment SGI127116[] = {{&S104801[30], 6}, {NULL}};
const Segment SGI127117[] = {{&S104801[111], 2}, {NULL}};
const Segment SGI127118[] = {{&S104801[80], 5}, {NULL}};
const Segment SGI127119[] = {{&S13424[44], 18}, {NULL}};
const Segment SGI127120[] = {{&S13424[44], 30}, {NULL}};
const Segment SGI127121[] = {{&S13424[30], 6}, {NULL}};
const Segment SGI127122[] = {{&S13424[123], 3}, {NULL}};
const Segment SGI127123[] = {{&S13424[88], 6}, {NULL}};
const Segment SGI127124[] = {{&S13425[44], 8}, {NULL}};
const Segment SGI127125[] = {{&S13425[44], 20}, {NULL}};
const Segment SGI127126[] = {{&S13425[30], 6}, {NULL}};
const Segment SGI127127[] = {{&S13425[113], 3}, {NULL}};
const Segment SGI127128[] = {{&S13425[78], 6}, {NULL}};
const Segment SGI127129[] = {{&S13426[44], 6}, {NULL}};
const Segment SGI127130[] = {{&S13426[44], 18}, {NULL}};
const Segment SGI127131[] = {{&S13426[30], 6}, {NULL}};
const Segment SGI127132[] = {{&S13426[111], 3}, {NULL}};
const Segment SGI127133[] = {{&S13426[76], 6}, {NULL}};
const Segment SGI127134[] = {{&S13428[44], 37}, {NULL}};
const Segment SGI127135[] = {{&S13428[30], 6}, {NULL}};
const Segment SGI127136[] = {{&S13428[113], 6}, {NULL}};
const Segment SGI127137[] = {{&S13430[44], 6}, {NULL}};
const Segment SGI127138[] = {{&S13430[44], 18}, {NULL}};
const Segment SGI127139[] = {{&S13430[30], 6}, {NULL}};
const Segment SGI127140[] = {{&S13430[111], 3}, {NULL}};
const Segment SGI127141[] = {{&S13430[76], 6}, {NULL}};
const Segment SGI127142[] = {{&S13431[44], 6}, {NULL}};
const Segment SGI127143[] = {{&S13431[44], 18}, {NULL}};
const Segment SGI127144[] = {{&S13431[30], 6}, {NULL}};
const Segment SGI127145[] = {{&S13431[111], 3}, {NULL}};
const Segment SGI127146[] = {{&S13431[76], 6}, {NULL}};
const Segment SGI127147[] = {{&S13432[44], 8}, {NULL}};
const Segment SGI127148[] = {{&S13432[44], 20}, {NULL}};
const Segment SGI127149[] = {{&S13432[30], 6}, {NULL}};
const Segment SGI127150[] = {{&S104816[44], 11}, {NULL}};
const Segment SGI127151[] = {{&S104816[44], 23}, {NULL}};
const Segment SGI127152[] = {{&S104816[30], 6}, {NULL}};
const Segment SGI127153[] = {{&S104816[31], 3}, {NULL}};
const Segment SGI127154[] = {{&S104816[81], 6}, {NULL}};
const Segment SGI127155[] = {{&S104816[151], 3}, {NULL}};
const Segment SGI127156[] = {{&S104836[44], 9}, {NULL}};
const Segment SGI127157[] = {{&S104836[44], 21}, {NULL}};
const Segment SGI127158[] = {{&S104836[30], 6}, {NULL}};
const Segment SGI127159[] = {{&S104836[110], 2}, {NULL}};
const Segment SGI127160[] = {{&S104836[79], 5}, {NULL}};
const Segment SGI127161[] = {{&S104856[44], 11}, {NULL}};
const Segment SGI127162[] = {{&S104856[44], 23}, {NULL}};
const Segment SGI127163[] = {{&S104856[30], 6}, {NULL}};
const Segment SGI127164[] = {{&S104856[112], 2}, {NULL}};
const Segment SGI127165[] = {{&S104856[81], 5}, {NULL}};
const Segment SGI127166[] = {{&S104872[57], 10}, {NULL}};
const Segment SGI127167[] = {{&S104872[57], 22}, {NULL}};
const Segment SGI127168[] = {{&S104872[43], 6}, {NULL}};
const Segment SGI127169[] = {{&S104872[44], 3}, {NULL}};
const Segment SGI127170[] = {{&S104872[93], 6}, {NULL}};
const Segment SGI127171[] = {{&S104872[0], 12}, {NULL}};
const Segment SGI127172[] = {{&S104887[45], 10}, {NULL}};
const Segment SGI127173[] = {{&S104887[30], 6}, {NULL}};
const Segment SGI127174[] = {{&S104887[31], 3}, {NULL}};
const Segment SGI127175[] = {{&S104887[81], 5}, {NULL}};
const Segment SGI127176[] = {{&S104887[139], 13}, {NULL}};
const Segment SGI127177[] = {{&S104888[45], 10}, {NULL}};
const Segment SGI127178[] = {{&S104888[30], 6}, {NULL}};
const Segment SGI127179[] = {{&S104888[31], 3}, {NULL}};
const Segment SGI127180[] = {{&S104888[81], 5}, {NULL}};
const Segment SGI127181[] = {{&S104888[139], 13}, {NULL}};
const Segment SGI127182[] = {{&S104891[57], 10}, {NULL}};
const Segment SGI127183[] = {{&S104891[57], 22}, {NULL}};
const Segment SGI127184[] = {{&S104891[43], 6}, {NULL}};
const Segment SGI127185[] = {{&S104891[44], 3}, {NULL}};
const Segment SGI127186[] = {{&S104891[93], 6}, {NULL}};
const Segment SGI127187[] = {{&S104891[0], 12}, {NULL}};
const Segment SGI127188[] = {{&S104892[57], 10}, {NULL}};
const Segment SGI127189[] = {{&S104892[57], 22}, {NULL}};
const Segment SGI127190[] = {{&S104892[43], 6}, {NULL}};
const Segment SGI127191[] = {{&S104892[44], 3}, {NULL}};
const Segment SGI127192[] = {{&S104892[93], 6}, {NULL}};
const Segment SGI127193[] = {{&S104892[0], 12}, {NULL}};
const Segment SGI127194[] = {{&S13432[113], 3}, {NULL}};
const Segment SGI127195[] = {{&S13432[78], 6}, {NULL}};
const Segment SGI127196[] = {{&S13433[44], 5}, {NULL}};
const Segment SGI127197[] = {{&S13433[44], 17}, {NULL}};
const Segment SGI127198[] = {{&S13433[30], 6}, {NULL}};
const Segment SGI127199[] = {{&S13433[110], 3}, {NULL}};
const Segment SGI127200[] = {{&S13433[75], 6}, {NULL}};
const Segment SGI127201[] = {{&S13434[44], 18}, {NULL}};
const Segment SGI127202[] = {{&S13434[44], 30}, {NULL}};
const Segment SGI127203[] = {{&S13434[30], 6}, {NULL}};
const Segment SGI127204[] = {{&S13434[123], 3}, {NULL}};
const Segment SGI127205[] = {{&S13434[88], 6}, {NULL}};
const Segment SGI127206[] = {{&S13435[44], 9}, {NULL}};
const Segment SGI127207[] = {{&S13435[44], 21}, {NULL}};
const Segment SGI127208[] = {{&S13435[30], 6}, {NULL}};
const Segment SGI127209[] = {{&S13435[114], 3}, {NULL}};
const Segment SGI127210[] = {{&S13435[79], 6}, {NULL}};
const Segment SGI127211[] = {{&S13436[44], 5}, {NULL}};
const Segment SGI127212[] = {{&S13436[44], 17}, {NULL}};
const Segment SGI127213[] = {{&S13436[30], 6}, {NULL}};
const Segment SGI127214[] = {{&S13436[110], 3}, {NULL}};
const Segment SGI127215[] = {{&S13436[75], 6}, {NULL}};
const Segment SGI127216[] = {{&S13437[44], 9}, {NULL}};
const Segment SGI127217[] = {{&S13437[44], 21}, {NULL}};
const Segment SGI127218[] = {{&S13437[30], 6}, {NULL}};
const Segment SGI127219[] = {{&S13437[114], 3}, {NULL}};
const Segment SGI127220[] = {{&S13437[79], 6}, {NULL}};
const Segment SGI127221[] = {{&S13439[44], 16}, {NULL}};
const Segment SGI127222[] = {{&S13439[44], 36}, {NULL}};
const Segment SGI127223[] = {{&S13439[30], 6}, {NULL}};
const Segment SGI127224[] = {{&S13439[129], 3}, {NULL}};
const Segment SGI127225[] = {{&S13439[94], 6}, {NULL}};
const Segment SGI127226[] = {{&S13440[44], 16}, {NULL}};
const Segment SGI127227[] = {{&S13440[44], 28}, {NULL}};
const Segment SGI127228[] = {{&S13440[30], 6}, {NULL}};
const Segment SGI127229[] = {{&S13440[121], 3}, {NULL}};
const Segment SGI127230[] = {{&S13440[86], 6}, {NULL}};
const Segment SGI127231[] = {{&S13441[44], 9}, {NULL}};
const Segment SGI127232[] = {{&S13441[44], 21}, {NULL}};
const Segment SGI127233[] = {{&S13441[30], 6}, {NULL}};
const Segment SGI127234[] = {{&S13441[114], 3}, {NULL}};
const Segment SGI127235[] = {{&S13441[79], 6}, {NULL}};
const Segment SGI127236[] = {{&S13442[44], 9}, {NULL}};
const Segment SGI127237[] = {{&S13442[44], 21}, {NULL}};
const Segment SGI127238[] = {{&S13442[30], 6}, {NULL}};
const Segment SGI127239[] = {{&S13442[114], 3}, {NULL}};
const Segment SGI127240[] = {{&S13442[79], 6}, {NULL}};
const Segment SGI127241[] = {{&S13443[38], 8}, {NULL}};
const Segment SGI127242[] = {{&S13443[31], 6}, {NULL}};
const Segment SGI127243[] = {{&S13444[44], 10}, {NULL}};
const Segment SGI127244[] = {{&S13444[44], 22}, {NULL}};
const Segment SGI127245[] = {{&S13444[30], 6}, {NULL}};
const Segment SGI127246[] = {{&S13444[115], 3}, {NULL}};
const Segment SGI127247[] = {{&S13444[80], 6}, {NULL}};
const Segment SGI127248[] = {{&S13446[44], 11}, {NULL}};
const Segment SGI127249[] = {{&S13446[44], 23}, {NULL}};
const Segment SGI127250[] = {{&S13446[30], 6}, {NULL}};
const Segment SGI127251[] = {{&S13446[116], 3}, {NULL}};
const Segment SGI127252[] = {{&S13446[81], 6}, {NULL}};
const Segment SGI127253[] = {{&S13449[44], 14}, {NULL}};
const Segment SGI127254[] = {{&S13449[44], 26}, {NULL}};
const Segment SGI127255[] = {{&S13449[30], 6}, {NULL}};
const Segment SGI127256[] = {{&S13449[119], 3}, {NULL}};
const Segment SGI127257[] = {{&S13449[84], 6}, {NULL}};
const Segment SGI127258[] = {{&S13450[44], 9}, {NULL}};
const Segment SGI127259[] = {{&S13450[44], 21}, {NULL}};
const Segment SGI127260[] = {{&S13450[30], 6}, {NULL}};
const Segment SGI127261[] = {{&S13450[114], 3}, {NULL}};
const Segment SGI127262[] = {{&S13450[79], 6}, {NULL}};
const Segment SGI127263[] = {{&S104946[44], 10}, {NULL}};
const Segment SGI127264[] = {{&S104946[44], 22}, {NULL}};
const Segment SGI127265[] = {{&S104946[30], 6}, {NULL}};
const Segment SGI127266[] = {{&S13451[44], 13}, {NULL}};
const Segment SGI127267[] = {{&S104946[139], 4}, {NULL}};
const Segment SGI127268[] = {{&S13451[44], 25}, {NULL}};
const Segment SGI127269[] = {{&S104946[165], 3}, {NULL}};
const Segment SGI127270[] = {{&S13451[30], 6}, {NULL}};
const Segment SGI127271[] = {{&S104946[31], 3}, {&S104946[214], 5}, {NULL}};
const Segment SGI127272[] = {{&S13451[118], 3}, {NULL}};
const Segment SGI127273[] = {{&S104946[80], 6}, {NULL}};
const Segment SGI127274[] = {{&S13451[83], 6}, {NULL}};
const Segment SGI127275[] = {{&S13452[44], 8}, {NULL}};
const Segment SGI127276[] = {{&S13452[44], 20}, {NULL}};
const Segment SGI127277[] = {{&S13452[30], 6}, {NULL}};
const Segment SGI127278[] = {{&S13452[113], 3}, {NULL}};
const Segment SGI127279[] = {{&S13452[78], 6}, {NULL}};
const Segment SGI127280[] = {{&S13453[44], 21}, {NULL}};
const Segment SGI127281[] = {{&S13453[44], 33}, {NULL}};
const Segment SGI127282[] = {{&S13453[30], 6}, {NULL}};
const Segment SGI127283[] = {{&S13453[126], 3}, {NULL}};
const Segment SGI127284[] = {{&S13453[91], 6}, {NULL}};
const Segment SGI127285[] = {{&S13454[44], 12}, {NULL}};
const Segment SGI127286[] = {{&S13454[44], 24}, {NULL}};
const Segment SGI127287[] = {{&S13454[30], 6}, {NULL}};
const Segment SGI127288[] = {{&S13454[117], 3}, {NULL}};
const Segment SGI127289[] = {{&S13454[82], 6}, {NULL}};
const Segment SGI127290[] = {{&S13457[44], 8}, {NULL}};
const Segment SGI127291[] = {{&S13457[44], 20}, {NULL}};
const Segment SGI127292[] = {{&S13457[30], 6}, {NULL}};
const Segment SGI127293[] = {{&S13457[113], 3}, {NULL}};
const Segment SGI127294[] = {{&S13457[78], 6}, {NULL}};
const Segment SGI127295[] = {{&S13459[44], 8}, {NULL}};
const Segment SGI127296[] = {{&S13459[44], 20}, {NULL}};
const Segment SGI127297[] = {{&S13459[30], 6}, {NULL}};
const Segment SGI127298[] = {{&S13459[113], 3}, {NULL}};
const Segment SGI127299[] = {{&S13459[78], 6}, {NULL}};
const Segment SGI127300[] = {{&S13460[44], 7}, {NULL}};
const Segment SGI127301[] = {{&S13460[44], 19}, {NULL}};
const Segment SGI127302[] = {{&S13460[30], 6}, {NULL}};
const Segment SGI127303[] = {{&S13460[112], 3}, {NULL}};
const Segment SGI127304[] = {{&S13460[77], 6}, {NULL}};
const Segment SGI127305[] = {{&S13461[44], 6}, {NULL}};
const Segment SGI127306[] = {{&S13461[44], 18}, {NULL}};
const Segment SGI127307[] = {{&S13461[30], 6}, {NULL}};
const Segment SGI127308[] = {{&S13461[111], 3}, {NULL}};
const Segment SGI127309[] = {{&S13461[76], 6}, {NULL}};
const Segment SGI127310[] = {{&S13462[44], 14}, {NULL}};
const Segment SGI127311[] = {{&S13462[44], 26}, {NULL}};
const Segment SGI127312[] = {{&S13462[30], 6}, {NULL}};
const Segment SGI127313[] = {{&S13462[119], 3}, {NULL}};
const Segment SGI127314[] = {{&S13462[84], 6}, {NULL}};
const Segment SGI127315[] = {{&S13463[44], 7}, {NULL}};
const Segment SGI127316[] = {{&S13463[44], 19}, {NULL}};
const Segment SGI127317[] = {{&S13463[30], 6}, {NULL}};
const Segment SGI127318[] = {{&S13463[112], 3}, {NULL}};
const Segment SGI127319[] = {{&S13463[77], 6}, {NULL}};
const Segment SGI127320[] = {{&S13465[44], 14}, {NULL}};
const Segment SGI127321[] = {{&S13465[44], 26}, {NULL}};
const Segment SGI127322[] = {{&S13465[30], 6}, {NULL}};
const Segment SGI127323[] = {{&S13465[119], 3}, {NULL}};
const Segment SGI127324[] = {{&S13465[84], 6}, {NULL}};
const Segment SGI127325[] = {{&S13467[44], 4}, {NULL}};
const Segment SGI127326[] = {{&S13467[44], 16}, {NULL}};
const Segment SGI127327[] = {{&S13467[30], 6}, {NULL}};
const Segment SGI127328[] = {{&S13467[109], 3}, {NULL}};
const Segment SGI127329[] = {{&S13467[74], 6}, {NULL}};
const Segment SGI127330[] = {{&S13468[44], 11}, {NULL}};
const Segment SGI127331[] = {{&S13468[44], 23}, {NULL}};
const Segment SGI127332[] = {{&S13468[30], 6}, {NULL}};
const Segment SGI127333[] = {{&S13468[116], 3}, {NULL}};
const Segment SGI127334[] = {{&S13468[81], 6}, {NULL}};
const Segment SGI127335[] = {{&S13469[44], 8}, {NULL}};
const Segment SGI127336[] = {{&S13469[44], 20}, {NULL}};
const Segment SGI127337[] = {{&S13469[30], 6}, {NULL}};
const Segment SGI127338[] = {{&S13469[113], 3}, {NULL}};
const Segment SGI127339[] = {{&S13469[78], 6}, {NULL}};
const Segment SGI127340[] = {{&S13470[44], 20}, {NULL}};
const Segment SGI127341[] = {{&S13470[44], 32}, {NULL}};
const Segment SGI127342[] = {{&S13470[30], 6}, {NULL}};
const Segment SGI127343[] = {{&S13470[125], 3}, {NULL}};
const Segment SGI127344[] = {{&S13470[90], 6}, {NULL}};
const Segment SGI127345[] = {{&S104978[44], 10}, {NULL}};
const Segment SGI127346[] = {{&S104978[44], 22}, {NULL}};
const Segment SGI127347[] = {{&S104978[30], 6}, {NULL}};
const Segment SGI127348[] = {{&S104978[31], 3}, {NULL}};
const Segment SGI127349[] = {{&S104978[80], 5}, {NULL}};
const Segment SGI127350[] = {{&S104978[117], 14}, {NULL}};
const Segment SGI127351[] = {{&S13471[44], 4}, {NULL}};
const Segment SGI127352[] = {{&S13471[44], 16}, {NULL}};
const Segment SGI127353[] = {{&S13471[30], 6}, {NULL}};
const Segment SGI127354[] = {{&S13471[109], 3}, {NULL}};
const Segment SGI127355[] = {{&S13471[74], 6}, {NULL}};
const Segment SGI127356[] = {{&S13472[44], 11}, {NULL}};
const Segment SGI127357[] = {{&S13472[44], 23}, {NULL}};
const Segment SGI127358[] = {{&S13472[30], 6}, {NULL}};
const Segment SGI127359[] = {{&S13472[116], 3}, {NULL}};
const Segment SGI127360[] = {{&S13472[81], 6}, {NULL}};
const Segment SGI127361[] = {{&S13473[44], 7}, {NULL}};
const Segment SGI127362[] = {{&S13473[44], 19}, {NULL}};
const Segment SGI127363[] = {{&S13473[30], 6}, {NULL}};
const Segment SGI127364[] = {{&S13473[112], 3}, {NULL}};
const Segment SGI127365[] = {{&S13473[77], 6}, {NULL}};
const Segment SGI127366[] = {{&S13474[44], 8}, {NULL}};
const Segment SGI127367[] = {{&S13474[44], 20}, {NULL}};
const Segment SGI127368[] = {{&S13474[30], 6}, {NULL}};
const Segment SGI127369[] = {{&S13474[113], 3}, {NULL}};
const Segment SGI127370[] = {{&S13474[78], 6}, {NULL}};
const Segment SGI127371[] = {{&S13475[44], 12}, {NULL}};
const Segment SGI127372[] = {{&S13475[44], 24}, {NULL}};
const Segment SGI127373[] = {{&S13475[30], 6}, {NULL}};
const Segment SGI127374[] = {{&S13475[117], 3}, {NULL}};
const Segment SGI127375[] = {{&S13475[82], 6}, {NULL}};
const Segment SGI127376[] = {{&S13477[44], 14}, {NULL}};
const Segment SGI127377[] = {{&S13477[44], 26}, {NULL}};
const Segment SGI127378[] = {{&S13477[30], 6}, {NULL}};
const Segment SGI127379[] = {{&S13477[119], 3}, {NULL}};
const Segment SGI127380[] = {{&S13477[84], 6}, {NULL}};
const Segment SGI127381[] = {{&S13478[44], 9}, {NULL}};
const Segment SGI127382[] = {{&S13478[44], 21}, {NULL}};
const Segment SGI127383[] = {{&S13478[30], 6}, {NULL}};
const Segment SGI127384[] = {{&S13478[114], 3}, {NULL}};
const Segment SGI127385[] = {{&S13478[79], 6}, {NULL}};
const Segment SGI127386[] = {{&S13479[44], 11}, {NULL}};
const Segment SGI127387[] = {{&S13479[44], 23}, {NULL}};
const Segment SGI127388[] = {{&S13479[30], 6}, {NULL}};
const Segment SGI127389[] = {{&S13479[116], 3}, {NULL}};
const Segment SGI127390[] = {{&S13479[81], 6}, {NULL}};
const Segment SGI127391[] = {{&S13480[44], 13}, {NULL}};
const Segment SGI127392[] = {{&S13480[44], 25}, {NULL}};
const Segment SGI127393[] = {{&S13480[30], 6}, {NULL}};
const Segment SGI127394[] = {{&S13480[118], 3}, {NULL}};
const Segment SGI127395[] = {{&S13480[83], 6}, {NULL}};
const Segment SGI127396[] = {{&S13482[44], 12}, {NULL}};
const Segment SGI127397[] = {{&S13482[44], 24}, {NULL}};
const Segment SGI127398[] = {{&S13482[30], 6}, {NULL}};
const Segment SGI127399[] = {{&S13482[117], 3}, {NULL}};
const Segment SGI127400[] = {{&S13482[82], 6}, {NULL}};
const Segment SGI127401[] = {{&S104997[44], 10}, {NULL}};
const Segment SGI127402[] = {{&S104997[44], 22}, {NULL}};
const Segment SGI127403[] = {{&S104997[30], 6}, {NULL}};
const Segment SGI127404[] = {{&S104997[31], 3}, {NULL}};
const Segment SGI127405[] = {{&S104997[80], 6}, {NULL}};
const Segment SGI127406[] = {{&S104997[133], 13}, {NULL}};
const Segment SGI127407[] = {{&S13485[44], 8}, {NULL}};
const Segment SGI127408[] = {{&S13485[44], 20}, {NULL}};
const Segment SGI127409[] = {{&S13485[30], 6}, {NULL}};
const Segment SGI127410[] = {{&S13485[113], 3}, {NULL}};
const Segment SGI127411[] = {{&S13485[78], 6}, {NULL}};
const Segment SGI127412[] = {{&S13486[44], 8}, {NULL}};
const Segment SGI127413[] = {{&S13486[44], 20}, {NULL}};
const Segment SGI127414[] = {{&S13486[30], 6}, {NULL}};
const Segment SGI127415[] = {{&S13486[113], 3}, {NULL}};
const Segment SGI127416[] = {{&S13486[78], 6}, {NULL}};
const Segment SGI127417[] = {{&S13487[44], 4}, {NULL}};
const Segment SGI127418[] = {{&S13487[44], 16}, {NULL}};
const Segment SGI127419[] = {{&S13487[30], 6}, {NULL}};
const Segment SGI127420[] = {{&S13487[109], 3}, {NULL}};
const Segment SGI127421[] = {{&S13487[74], 6}, {NULL}};
const Segment SGI127422[] = {{&S13489[44], 12}, {NULL}};
const Segment SGI127423[] = {{&S13489[44], 24}, {NULL}};
const Segment SGI127424[] = {{&S13489[30], 6}, {NULL}};
const Segment SGI127425[] = {{&S13489[117], 3}, {NULL}};
const Segment SGI127426[] = {{&S13489[82], 6}, {NULL}};
const Segment SGI127427[] = {{&S13490[44], 13}, {NULL}};
const Segment SGI127428[] = {{&S13490[44], 25}, {NULL}};
const Segment SGI127429[] = {{&S13490[30], 6}, {NULL}};
const Segment SGI127430[] = {{&S13490[118], 3}, {NULL}};
const Segment SGI127431[] = {{&S13490[83], 6}, {NULL}};
const Segment SGI127432[] = {{&S13491[44], 14}, {NULL}};
const Segment SGI127433[] = {{&S13491[44], 26}, {NULL}};
const Segment SGI127434[] = {{&S13491[30], 6}, {NULL}};
const Segment SGI127435[] = {{&S13491[119], 3}, {NULL}};
const Segment SGI127436[] = {{&S13491[84], 6}, {NULL}};
const Segment SGI127437[] = {{&S105010[45], 10}, {NULL}};
const Segment SGI127438[] = {{&S105010[30], 6}, {NULL}};
const Segment SGI127439[] = {{&S105010[31], 3}, {NULL}};
const Segment SGI127440[] = {{&S13492[44], 11}, {NULL}};
const Segment SGI127441[] = {{&S105010[81], 5}, {NULL}};
const Segment SGI127442[] = {{&S13492[44], 23}, {NULL}};
const Segment SGI127443[] = {{&S13492[30], 6}, {NULL}};
const Segment SGI127444[] = {{&S105010[139], 13}, {NULL}};
const Segment SGI127445[] = {{&S13492[116], 3}, {NULL}};
const Segment SGI127446[] = {{&S13492[81], 6}, {NULL}};
const Segment SGI127447[] = {{&S13493[44], 11}, {NULL}};
const Segment SGI127448[] = {{&S13493[44], 23}, {NULL}};
const Segment SGI127449[] = {{&S13493[30], 6}, {NULL}};
const Segment SGI127450[] = {{&S13493[116], 3}, {NULL}};
const Segment SGI127451[] = {{&S13493[81], 6}, {NULL}};
const Segment SGI127452[] = {{&S105013[59], 10}, {NULL}};
const Segment SGI127453[] = {{&S105013[59], 22}, {NULL}};
const Segment SGI127454[] = {{&S105013[45], 6}, {NULL}};
const Segment SGI127455[] = {{&S13494[44], 6}, {NULL}};
const Segment SGI127456[] = {{&S13494[44], 18}, {NULL}};
const Segment SGI127457[] = {{&S13494[30], 6}, {NULL}};
const Segment SGI127458[] = {{&S105013[46], 3}, {NULL}};
const Segment SGI127459[] = {{&S13494[111], 3}, {NULL}};
const Segment SGI127460[] = {{&S105013[95], 6}, {NULL}};
const Segment SGI127461[] = {{&S13494[76], 6}, {NULL}};
const Segment SGI127462[] = {{&S105013[0], 14}, {NULL}};
const Segment SGI127463[] = {{&S13495[44], 5}, {NULL}};
const Segment SGI127464[] = {{&S13495[44], 17}, {NULL}};
const Segment SGI127465[] = {{&S13495[30], 6}, {NULL}};
const Segment SGI127466[] = {{&S13495[110], 3}, {NULL}};
const Segment SGI127467[] = {{&S13495[75], 6}, {NULL}};
const Segment SGI127468[] = {{&S13497[44], 6}, {NULL}};
const Segment SGI127469[] = {{&S13497[44], 18}, {NULL}};
const Segment SGI127470[] = {{&S13497[30], 6}, {NULL}};
const Segment SGI127471[] = {{&S13497[111], 3}, {NULL}};
const Segment SGI127472[] = {{&S13497[76], 6}, {NULL}};
const Segment SGI127473[] = {{&S13498[44], 6}, {NULL}};
const Segment SGI127474[] = {{&S13498[44], 18}, {NULL}};
const Segment SGI127475[] = {{&S13498[30], 6}, {NULL}};
const Segment SGI127476[] = {{&S13498[110], 3}, {NULL}};
const Segment SGI127477[] = {{&S105019[59], 10}, {NULL}};
const Segment SGI127478[] = {{&S13498[76], 5}, {NULL}};
const Segment SGI127479[] = {{&S105019[59], 22}, {NULL}};
const Segment SGI127480[] = {{&S105019[45], 6}, {NULL}};
const Segment SGI127481[] = {{&S105019[46], 3}, {NULL}};
const Segment SGI127482[] = {{&S105019[95], 6}, {NULL}};
const Segment SGI127483[] = {{&S105019[0], 14}, {NULL}};
const Segment SGI127484[] = {{&S13499[44], 12}, {NULL}};
const Segment SGI127485[] = {{&S13499[44], 24}, {NULL}};
const Segment SGI127486[] = {{&S13499[30], 6}, {NULL}};
const Segment SGI127487[] = {{&S13499[117], 3}, {NULL}};
const Segment SGI127488[] = {{&S13499[82], 6}, {NULL}};
const Segment SGI127489[] = {{&S13500[44], 6}, {NULL}};
const Segment SGI127490[] = {{&S13500[44], 18}, {NULL}};
const Segment SGI127491[] = {{&S13500[30], 6}, {NULL}};
const Segment SGI127492[] = {{&S13500[111], 3}, {NULL}};
const Segment SGI127493[] = {{&S13500[76], 6}, {NULL}};
const Segment SGI127494[] = {{&S13501[44], 12}, {NULL}};
const Segment SGI127495[] = {{&S13501[44], 25}, {NULL}};
const Segment SGI127496[] = {{&S13501[30], 6}, {NULL}};
const Segment SGI127497[] = {{&S13501[118], 3}, {NULL}};
const Segment SGI127498[] = {{&S13501[83], 6}, {NULL}};
const Segment SGI127499[] = {{&S13502[44], 6}, {NULL}};
const Segment SGI127500[] = {{&S13502[44], 18}, {NULL}};
const Segment SGI127501[] = {{&S13502[30], 6}, {NULL}};
const Segment SGI127502[] = {{&S13502[111], 3}, {NULL}};
const Segment SGI127503[] = {{&S13502[76], 6}, {NULL}};
const Segment SGI127504[] = {{&S13503[44], 9}, {NULL}};
const Segment SGI127505[] = {{&S13503[44], 21}, {NULL}};
const Segment SGI127506[] = {{&S13503[30], 6}, {NULL}};
const Segment SGI127507[] = {{&S13503[114], 3}, {NULL}};
const Segment SGI127508[] = {{&S13503[79], 6}, {NULL}};
const Segment SGI127509[] = {{&S13504[44], 11}, {NULL}};
const Segment SGI127510[] = {{&S13504[44], 26}, {NULL}};
const Segment SGI127511[] = {{&S13504[30], 6}, {NULL}};
const Segment SGI127512[] = {{&S13504[119], 3}, {NULL}};
const Segment SGI127513[] = {{&S13504[84], 6}, {NULL}};
const Segment SGI127514[] = {{&S13506[44], 8}, {NULL}};
const Segment SGI127515[] = {{&S13506[44], 20}, {NULL}};
const Segment SGI127516[] = {{&S13506[30], 6}, {NULL}};
const Segment SGI127517[] = {{&S13506[113], 3}, {NULL}};
const Segment SGI127518[] = {{&S13506[78], 6}, {NULL}};
const Segment SGI127519[] = {{&S13507[44], 8}, {NULL}};
const Segment SGI127520[] = {{&S13507[44], 20}, {NULL}};
const Segment SGI127521[] = {{&S13507[30], 6}, {NULL}};
const Segment SGI127522[] = {{&S13507[113], 3}, {NULL}};
const Segment SGI127523[] = {{&S13507[78], 6}, {NULL}};
const Segment SGI127524[] = {{&S13508[44], 5}, {NULL}};
const Segment SGI127525[] = {{&S13508[44], 17}, {NULL}};
const Segment SGI127526[] = {{&S13508[30], 6}, {NULL}};
const Segment SGI127527[] = {{&S13508[110], 3}, {NULL}};
const Segment SGI127528[] = {{&S13508[75], 6}, {NULL}};
const Segment SGI127529[] = {{&S13509[44], 7}, {NULL}};
const Segment SGI127530[] = {{&S13509[44], 19}, {NULL}};
const Segment SGI127531[] = {{&S13509[30], 6}, {NULL}};
const Segment SGI127532[] = {{&S13509[112], 3}, {NULL}};
const Segment SGI127533[] = {{&S13509[77], 6}, {NULL}};
const Segment SGI127534[] = {{&S13510[44], 7}, {NULL}};
const Segment SGI127535[] = {{&S13510[44], 19}, {NULL}};
const Segment SGI127536[] = {{&S13510[30], 6}, {NULL}};
const Segment SGI127537[] = {{&S13510[112], 3}, {NULL}};
const Segment SGI127538[] = {{&S13510[77], 6}, {NULL}};
const Segment SGI127539[] = {{&S13511[44], 5}, {NULL}};
const Segment SGI127540[] = {{&S13511[44], 17}, {NULL}};
const Segment SGI127541[] = {{&S13511[30], 6}, {NULL}};
const Segment SGI127542[] = {{&S13511[110], 3}, {NULL}};
const Segment SGI127543[] = {{&S13511[75], 6}, {NULL}};
const Segment SGI127544[] = {{&S13512[44], 12}, {NULL}};
const Segment SGI127545[] = {{&S13512[44], 24}, {NULL}};
const Segment SGI127546[] = {{&S13512[30], 6}, {NULL}};
const Segment SGI127547[] = {{&S13512[117], 3}, {NULL}};
const Segment SGI127548[] = {{&S13512[82], 6}, {NULL}};
const Segment SGI127549[] = {{&S13513[44], 13}, {NULL}};
const Segment SGI127550[] = {{&S13513[44], 25}, {NULL}};
const Segment SGI127551[] = {{&S13513[30], 6}, {NULL}};
const Segment SGI127552[] = {{&S13513[118], 3}, {NULL}};
const Segment SGI127553[] = {{&S13513[83], 6}, {NULL}};
const Segment SGI127554[] = {{&S13514[44], 6}, {NULL}};
const Segment SGI127555[] = {{&S13514[44], 18}, {NULL}};
const Segment SGI127556[] = {{&S13514[30], 6}, {NULL}};
const Segment SGI127557[] = {{&S13514[111], 3}, {NULL}};
const Segment SGI127558[] = {{&S13514[76], 6}, {NULL}};
const Segment SGI127559[] = {{&S13515[44], 6}, {NULL}};
const Segment SGI127560[] = {{&S13515[44], 18}, {NULL}};
const Segment SGI127561[] = {{&S13515[30], 6}, {NULL}};
const Segment SGI127562[] = {{&S13515[111], 3}, {NULL}};
const Segment SGI127563[] = {{&S13515[76], 6}, {NULL}};
const Segment SGI127564[] = {{&S13516[44], 21}, {NULL}};
const Segment SGI127565[] = {{&S13516[44], 33}, {NULL}};
const Segment SGI127566[] = {{&S13516[30], 6}, {NULL}};
const Segment SGI127567[] = {{&S13516[126], 3}, {NULL}};
const Segment SGI127568[] = {{&S13516[91], 6}, {NULL}};
const Segment SGI127569[] = {{&S13517[44], 8}, {NULL}};
const Segment SGI127570[] = {{&S13517[44], 20}, {NULL}};
const Segment SGI127571[] = {{&S13517[30], 6}, {NULL}};
const Segment SGI127572[] = {{&S13517[113], 3}, {NULL}};
const Segment SGI127573[] = {{&S13517[78], 6}, {NULL}};
const Segment SGI127574[] = {{&S13518[44], 9}, {NULL}};
const Segment SGI127575[] = {{&S13518[44], 21}, {NULL}};
const Segment SGI127576[] = {{&S13518[30], 6}, {NULL}};
const Segment SGI127577[] = {{&S13518[114], 3}, {NULL}};
const Segment SGI127578[] = {{&S13518[79], 6}, {NULL}};
const Segment SGI127579[] = {{&S13519[44], 8}, {NULL}};
const Segment SGI127580[] = {{&S13519[44], 20}, {NULL}};
const Segment SGI127581[] = {{&S13519[30], 6}, {NULL}};
const Segment SGI127582[] = {{&S13519[113], 3}, {NULL}};
const Segment SGI127583[] = {{&S13519[78], 6}, {NULL}};
const Segment SGI127584[] = {{&S13521[44], 6}, {NULL}};
const Segment SGI127585[] = {{&S13521[44], 18}, {NULL}};
const Segment SGI127586[] = {{&S13521[30], 6}, {NULL}};
const Segment SGI127587[] = {{&S13521[111], 3}, {NULL}};
const Segment SGI127588[] = {{&S13521[76], 6}, {NULL}};
const Segment SGI127589[] = {{&S13522[44], 14}, {NULL}};
const Segment SGI127590[] = {{&S13522[44], 26}, {NULL}};
const Segment SGI127591[] = {{&S13522[30], 6}, {NULL}};
const Segment SGI127592[] = {{&S13522[119], 3}, {NULL}};
const Segment SGI127593[] = {{&S13522[84], 6}, {NULL}};
const Segment SGI127594[] = {{&S13523[57], 9}, {NULL}};
const Segment SGI127595[] = {{&S13523[50], 6}, {&S13523[57], 21}, {NULL}};
const Segment SGI127596[] = {{&S13523[30], 19}, {NULL}};
const Segment SGI127597[] = {{&S13523[127], 3}, {NULL}};
const Segment SGI127598[] = {{&S13523[92], 6}, {NULL}};
const Segment SGI127599[] = {{&S13524[44], 9}, {NULL}};
const Segment SGI127600[] = {{&S13524[44], 22}, {NULL}};
const Segment SGI127601[] = {{&S13524[30], 6}, {NULL}};
const Segment SGI127602[] = {{&S13524[115], 3}, {NULL}};
const Segment SGI127603[] = {{&S13524[80], 6}, {NULL}};
const Segment SGI127604[] = {{&S13525[38], 13}, {NULL}};
const Segment SGI127605[] = {{&S13525[31], 6}, {NULL}};
const Segment SGI127606[] = {{&S13526[44], 11}, {NULL}};
const Segment SGI127607[] = {{&S13526[44], 23}, {NULL}};
const Segment SGI127608[] = {{&S13526[30], 6}, {NULL}};
const Segment SGI127609[] = {{&S13526[116], 3}, {NULL}};
const Segment SGI127610[] = {{&S13526[81], 6}, {NULL}};
const Segment SGI127611[] = {{&S13527[44], 11}, {NULL}};
const Segment SGI127612[] = {{&S13527[44], 23}, {NULL}};
const Segment SGI127613[] = {{&S13527[30], 6}, {NULL}};
const Segment SGI127614[] = {{&S13527[116], 3}, {NULL}};
const Segment SGI127615[] = {{&S13527[81], 6}, {NULL}};
const Segment SGI127616[] = {{&S13528[44], 12}, {NULL}};
const Segment SGI127617[] = {{&S13528[44], 24}, {NULL}};
const Segment SGI127618[] = {{&S13528[30], 6}, {NULL}};
const Segment SGI127619[] = {{&S13528[117], 3}, {NULL}};
const Segment SGI127620[] = {{&S13528[82], 6}, {NULL}};
const Segment SGI127621[] = {{&S13529[44], 6}, {NULL}};
const Segment SGI127622[] = {{&S13529[44], 18}, {NULL}};
const Segment SGI127623[] = {{&S13529[30], 6}, {NULL}};
const Segment SGI127624[] = {{&S13529[111], 3}, {NULL}};
const Segment SGI127625[] = {{&S13529[76], 6}, {NULL}};
const Segment SGI127626[] = {{&S13530[44], 11}, {NULL}};
const Segment SGI127627[] = {{&S13530[44], 23}, {NULL}};
const Segment SGI127628[] = {{&S13530[30], 6}, {NULL}};
const Segment SGI127629[] = {{&S13530[116], 3}, {NULL}};
const Segment SGI127630[] = {{&S13530[81], 6}, {NULL}};
const Segment SGI127631[] = {{&S13531[44], 29}, {NULL}};
const Segment SGI127632[] = {{&S13531[44], 41}, {NULL}};
const Segment SGI127633[] = {{&S13531[30], 6}, {NULL}};
const Segment SGI127634[] = {{&S13531[134], 3}, {NULL}};
const Segment SGI127635[] = {{&S13531[99], 6}, {NULL}};
const Segment SGI127636[] = {{&S13532[44], 9}, {NULL}};
const Segment SGI127637[] = {{&S13532[44], 21}, {NULL}};
const Segment SGI127638[] = {{&S13532[30], 6}, {NULL}};
const Segment SGI127639[] = {{&S13532[114], 3}, {NULL}};
const Segment SGI127640[] = {{&S13532[79], 6}, {NULL}};
const Segment SGI127641[] = {{&S13533[44], 10}, {NULL}};
const Segment SGI127642[] = {{&S13533[44], 22}, {NULL}};
const Segment SGI127643[] = {{&S13533[30], 6}, {NULL}};
const Segment SGI127644[] = {{&S13533[115], 3}, {NULL}};
const Segment SGI127645[] = {{&S13533[80], 6}, {NULL}};
const Segment SGI127646[] = {{&S13534[44], 10}, {NULL}};
const Segment SGI127647[] = {{&S13534[44], 22}, {NULL}};
const Segment SGI127648[] = {{&S13534[30], 6}, {NULL}};
const Segment SGI127649[] = {{&S13534[115], 3}, {NULL}};
const Segment SGI127650[] = {{&S13534[80], 6}, {NULL}};
const Segment SGI127651[] = {{&S13535[44], 13}, {NULL}};
const Segment SGI127652[] = {{&S13535[44], 25}, {NULL}};
const Segment SGI127653[] = {{&S13535[30], 6}, {NULL}};
const Segment SGI127654[] = {{&S13535[118], 3}, {NULL}};
const Segment SGI127655[] = {{&S13535[83], 6}, {NULL}};
const Segment SGI127656[] = {{&S13536[44], 8}, {NULL}};
const Segment SGI127657[] = {{&S13536[44], 20}, {NULL}};
const Segment SGI127658[] = {{&S13536[30], 6}, {NULL}};
const Segment SGI127659[] = {{&S13536[113], 3}, {NULL}};
const Segment SGI127660[] = {{&S13536[78], 6}, {NULL}};
const Segment SGI127661[] = {{&S13537[44], 10}, {NULL}};
const Segment SGI127662[] = {{&S13537[44], 22}, {NULL}};
const Segment SGI127663[] = {{&S13537[30], 6}, {NULL}};
const Segment SGI127664[] = {{&S13537[115], 3}, {NULL}};
const Segment SGI127665[] = {{&S13537[80], 6}, {NULL}};
const Segment SGI127666[] = {{&S13538[44], 10}, {NULL}};
const Segment SGI127667[] = {{&S13538[44], 22}, {NULL}};
const Segment SGI127668[] = {{&S13538[30], 6}, {NULL}};
const Segment SGI127669[] = {{&S13538[115], 3}, {NULL}};
const Segment SGI127670[] = {{&S13538[80], 6}, {NULL}};
const Segment SGI127671[] = {{&S13539[44], 12}, {NULL}};
const Segment SGI127672[] = {{&S13539[44], 24}, {NULL}};
const Segment SGI127673[] = {{&S13539[30], 6}, {NULL}};
const Segment SGI127674[] = {{&S13539[117], 3}, {NULL}};
const Segment SGI127675[] = {{&S13539[82], 6}, {NULL}};
const Segment SGI127676[] = {{&S13540[44], 8}, {NULL}};
const Segment SGI127677[] = {{&S13540[44], 20}, {NULL}};
const Segment SGI127678[] = {{&S13540[30], 6}, {NULL}};
const Segment SGI127679[] = {{&S13540[113], 3}, {NULL}};
const Segment SGI127680[] = {{&S13540[78], 6}, {NULL}};
const Segment SGI127681[] = {{&S13541[44], 5}, {NULL}};
const Segment SGI127682[] = {{&S13541[44], 17}, {NULL}};
const Segment SGI127683[] = {{&S13541[30], 6}, {NULL}};
const Segment SGI127684[] = {{&S13541[110], 3}, {NULL}};
const Segment SGI127685[] = {{&S13541[75], 6}, {NULL}};
const Segment SGI127686[] = {{&S13543[44], 8}, {NULL}};
const Segment SGI127687[] = {{&S13543[44], 20}, {NULL}};
const Segment SGI127688[] = {{&S13543[30], 6}, {NULL}};
const Segment SGI127689[] = {{&S13543[113], 3}, {NULL}};
const Segment SGI127690[] = {{&S13543[78], 6}, {NULL}};
const Segment SGI127691[] = {{&S13544[44], 12}, {NULL}};
const Segment SGI127692[] = {{&S13544[44], 24}, {NULL}};
const Segment SGI127693[] = {{&S13544[30], 6}, {NULL}};
const Segment SGI127694[] = {{&S13544[117], 3}, {NULL}};
const Segment SGI127695[] = {{&S13544[82], 6}, {NULL}};
const Segment SGI127696[] = {{&S13545[44], 9}, {NULL}};
const Segment SGI127697[] = {{&S13545[44], 21}, {NULL}};
const Segment SGI127698[] = {{&S13545[30], 6}, {NULL}};
const Segment SGI127699[] = {{&S13545[114], 3}, {NULL}};
const Segment SGI127700[] = {{&S13545[79], 6}, {NULL}};
const Segment SGI127701[] = {{&S13547[44], 10}, {NULL}};
const Segment SGI127702[] = {{&S13547[44], 22}, {NULL}};
const Segment SGI127703[] = {{&S13547[30], 6}, {NULL}};
const Segment SGI127704[] = {{&S13547[115], 3}, {NULL}};
const Segment SGI127705[] = {{&S13547[80], 6}, {NULL}};
const Segment SGI127706[] = {{&S13548[44], 6}, {NULL}};
const Segment SGI127707[] = {{&S13548[44], 18}, {NULL}};
const Segment SGI127708[] = {{&S13548[30], 6}, {NULL}};
const Segment SGI127709[] = {{&S13548[111], 3}, {NULL}};
const Segment SGI127710[] = {{&S13548[76], 6}, {NULL}};
const Segment SGI127711[] = {{&S13549[44], 7}, {NULL}};
const Segment SGI127712[] = {{&S13549[44], 14}, {NULL}};
const Segment SGI127713[] = {{&S13549[30], 6}, {NULL}};
const Segment SGI127714[] = {{&S13549[107], 3}, {NULL}};
const Segment SGI127715[] = {{&S13549[72], 6}, {NULL}};
const Segment SGI127716[] = {{&S13550[44], 9}, {NULL}};
const Segment SGI127717[] = {{&S13550[44], 22}, {NULL}};
const Segment SGI127718[] = {{&S13550[30], 6}, {NULL}};
const Segment SGI127719[] = {{&S13550[115], 3}, {NULL}};
const Segment SGI127720[] = {{&S13550[80], 6}, {NULL}};
const Segment SGI127721[] = {{&S13551[44], 12}, {NULL}};
const Segment SGI127722[] = {{&S13551[44], 24}, {NULL}};
const Segment SGI127723[] = {{&S13551[30], 6}, {NULL}};
const Segment SGI127724[] = {{&S13551[117], 3}, {NULL}};
const Segment SGI127725[] = {{&S13551[82], 6}, {NULL}};
const Segment SGI127726[] = {{&S13552[44], 9}, {NULL}};
const Segment SGI127727[] = {{&S13552[44], 21}, {NULL}};
const Segment SGI127728[] = {{&S13552[30], 6}, {NULL}};
const Segment SGI127729[] = {{&S13552[114], 3}, {NULL}};
const Segment SGI127730[] = {{&S13552[79], 6}, {NULL}};
const Segment SGI127731[] = {{&S13553[44], 9}, {NULL}};
const Segment SGI127732[] = {{&S13553[44], 21}, {NULL}};
const Segment SGI127733[] = {{&S13553[30], 6}, {NULL}};
const Segment SGI127734[] = {{&S13553[114], 3}, {NULL}};
const Segment SGI127735[] = {{&S13553[79], 6}, {NULL}};
const Segment SGI127736[] = {{&S13554[44], 18}, {NULL}};
const Segment SGI127737[] = {{&S13554[44], 30}, {NULL}};
const Segment SGI127738[] = {{&S13554[30], 6}, {NULL}};
const Segment SGI127739[] = {{&S13554[123], 3}, {NULL}};
const Segment SGI127740[] = {{&S13554[88], 6}, {NULL}};
const Segment SGI127741[] = {{&S13556[44], 13}, {NULL}};
const Segment SGI127742[] = {{&S13556[44], 25}, {NULL}};
const Segment SGI127743[] = {{&S13556[30], 6}, {NULL}};
const Segment SGI127744[] = {{&S13556[118], 3}, {NULL}};
const Segment SGI127745[] = {{&S13556[83], 6}, {NULL}};
const Segment SGI127746[] = {{&S13557[44], 6}, {NULL}};
const Segment SGI127747[] = {{&S13557[44], 18}, {NULL}};
const Segment SGI127748[] = {{&S13557[30], 6}, {NULL}};
const Segment SGI127749[] = {{&S13557[111], 3}, {NULL}};
const Segment SGI127750[] = {{&S13557[76], 6}, {NULL}};
const Segment SGI127751[] = {{&S13558[44], 10}, {NULL}};
const Segment SGI127752[] = {{&S13558[44], 22}, {NULL}};
const Segment SGI127753[] = {{&S13558[30], 6}, {NULL}};
const Segment SGI127754[] = {{&S13558[115], 3}, {NULL}};
const Segment SGI127755[] = {{&S13558[80], 6}, {NULL}};
const Segment SGI127756[] = {{&S13559[44], 9}, {NULL}};
const Segment SGI127757[] = {{&S13559[44], 21}, {NULL}};
const Segment SGI127758[] = {{&S13559[30], 6}, {NULL}};
const Segment SGI127759[] = {{&S13559[114], 3}, {NULL}};
const Segment SGI127760[] = {{&S13559[79], 6}, {NULL}};
const Segment SGI127761[] = {{&S13560[44], 20}, {NULL}};
const Segment SGI127762[] = {{&S13560[44], 32}, {NULL}};
const Segment SGI127763[] = {{&S13560[30], 6}, {NULL}};
const Segment SGI127764[] = {{&S13560[125], 3}, {NULL}};
const Segment SGI127765[] = {{&S13560[90], 6}, {NULL}};
const Segment SGI127766[] = {{&S13561[44], 13}, {NULL}};
const Segment SGI127767[] = {{&S13561[44], 25}, {NULL}};
const Segment SGI127768[] = {{&S13561[30], 6}, {NULL}};
const Segment SGI127769[] = {{&S13561[118], 3}, {NULL}};
const Segment SGI127770[] = {{&S13561[83], 6}, {NULL}};
const Segment SGI127771[] = {{&S13562[44], 11}, {NULL}};
const Segment SGI127772[] = {{&S13562[44], 23}, {NULL}};
const Segment SGI127773[] = {{&S13562[30], 6}, {NULL}};
const Segment SGI127774[] = {{&S13562[116], 3}, {NULL}};
const Segment SGI127775[] = {{&S13562[81], 6}, {NULL}};
const Segment SGI127776[] = {{&S13563[44], 7}, {NULL}};
const Segment SGI127777[] = {{&S13563[44], 19}, {NULL}};
const Segment SGI127778[] = {{&S13563[30], 6}, {NULL}};
const Segment SGI127779[] = {{&S13563[112], 3}, {NULL}};
const Segment SGI127780[] = {{&S13563[77], 6}, {NULL}};
const Segment SGI127781[] = {{&S13564[44], 5}, {NULL}};
const Segment SGI127782[] = {{&S13564[44], 17}, {NULL}};
const Segment SGI127783[] = {{&S13564[30], 6}, {NULL}};
const Segment SGI127784[] = {{&S13564[110], 3}, {NULL}};
const Segment SGI127785[] = {{&S13564[75], 6}, {NULL}};
const Segment SGI127786[] = {{&S13565[44], 7}, {NULL}};
const Segment SGI127787[] = {{&S13565[44], 19}, {NULL}};
const Segment SGI127788[] = {{&S13565[30], 6}, {NULL}};
const Segment SGI127789[] = {{&S13565[112], 3}, {NULL}};
const Segment SGI127790[] = {{&S13565[77], 6}, {NULL}};
const Segment SGI127791[] = {{&S13566[44], 7}, {NULL}};
const Segment SGI127792[] = {{&S13566[44], 19}, {NULL}};
const Segment SGI127793[] = {{&S13566[30], 6}, {NULL}};
const Segment SGI127794[] = {{&S13566[112], 3}, {NULL}};
const Segment SGI127795[] = {{&S13566[77], 6}, {NULL}};
const Segment SGI127796[] = {{&S13567[44], 6}, {NULL}};
const Segment SGI127797[] = {{&S13567[44], 18}, {NULL}};
const Segment SGI127798[] = {{&S13567[30], 6}, {NULL}};
const Segment SGI127799[] = {{&S13567[111], 3}, {NULL}};
const Segment SGI127800[] = {{&S13567[76], 6}, {NULL}};
const Segment SGI127801[] = {{&S13569[44], 7}, {NULL}};
const Segment SGI127802[] = {{&S13569[44], 19}, {NULL}};
const Segment SGI127803[] = {{&S13569[30], 6}, {NULL}};
const Segment SGI127804[] = {{&S13569[112], 3}, {NULL}};
const Segment SGI127805[] = {{&S13569[77], 6}, {NULL}};
const Segment SGI127806[] = {{&S13571[44], 11}, {NULL}};
const Segment SGI127807[] = {{&S13571[44], 23}, {NULL}};
const Segment SGI127808[] = {{&S13571[30], 6}, {NULL}};
const Segment SGI127809[] = {{&S13571[116], 3}, {NULL}};
const Segment SGI127810[] = {{&S13571[81], 6}, {NULL}};
const Segment SGI127811[] = {{&S13572[42], 4}, {NULL}};
const Segment SGI127812[] = {{&S13572[30], 6}, {NULL}};
const Segment SGI127813[] = {{&S13572[107], 3}, {NULL}};
const Segment SGI127814[] = {{&S13572[72], 6}, {NULL}};
const Segment SGI127815[] = {{&S13573[44], 83}, {NULL}};
const Segment SGI127816[] = {{&S13573[44], 104}, {NULL}};
const Segment SGI127817[] = {{&S13573[30], 6}, {NULL}};
const Segment SGI127818[] = {{&S13573[195], 3}, {NULL}};
const Segment SGI127819[] = {{&S13573[121], 5}, {NULL}};
const Segment SGI127820[] = {{&S13574[44], 8}, {NULL}};
const Segment SGI127821[] = {{&S13574[44], 20}, {NULL}};
const Segment SGI127822[] = {{&S13574[30], 6}, {NULL}};
const Segment SGI127823[] = {{&S13574[113], 3}, {NULL}};
const Segment SGI127824[] = {{&S13574[78], 6}, {NULL}};
const Segment SGI127825[] = {{&S13575[44], 6}, {NULL}};
const Segment SGI127826[] = {{&S13575[44], 18}, {NULL}};
const Segment SGI127827[] = {{&S13575[30], 6}, {NULL}};
const Segment SGI127828[] = {{&S13575[111], 3}, {NULL}};
const Segment SGI127829[] = {{&S13575[76], 6}, {NULL}};
const Segment SGI127830[] = {{&S13579[44], 5}, {NULL}};
const Segment SGI127831[] = {{&S13579[44], 17}, {NULL}};
const Segment SGI127832[] = {{&S13579[30], 6}, {NULL}};
const Segment SGI127833[] = {{&S13579[110], 3}, {NULL}};
const Segment SGI127834[] = {{&S13579[75], 6}, {NULL}};
const Segment SGI127835[] = {{&S13581[44], 8}, {NULL}};
const Segment SGI127836[] = {{&S13581[44], 20}, {NULL}};
const Segment SGI127837[] = {{&S13581[30], 6}, {NULL}};
const Segment SGI127838[] = {{&S13581[113], 3}, {NULL}};
const Segment SGI127839[] = {{&S13581[78], 6}, {NULL}};
const Segment SGI127840[] = {{&S13582[44], 9}, {NULL}};
const Segment SGI127841[] = {{&S13582[44], 21}, {NULL}};
const Segment SGI127842[] = {{&S13582[30], 6}, {NULL}};
const Segment SGI127843[] = {{&S13582[114], 3}, {NULL}};
const Segment SGI127844[] = {{&S13582[79], 6}, {NULL}};
const Segment SGI127845[] = {{&S13583[44], 19}, {NULL}};
const Segment SGI127846[] = {{&S13583[44], 31}, {NULL}};
const Segment SGI127847[] = {{&S13583[30], 6}, {NULL}};
const Segment SGI127848[] = {{&S13583[124], 3}, {NULL}};
const Segment SGI127849[] = {{&S13583[89], 6}, {NULL}};
const Segment SGI127850[] = {{&S13584[44], 10}, {NULL}};
const Segment SGI127851[] = {{&S13584[44], 22}, {NULL}};
const Segment SGI127852[] = {{&S13584[30], 6}, {NULL}};
const Segment SGI127853[] = {{&S13584[115], 3}, {NULL}};
const Segment SGI127854[] = {{&S13584[80], 6}, {NULL}};
const Segment SGI127855[] = {{&S13585[44], 10}, {NULL}};
const Segment SGI127856[] = {{&S13585[44], 24}, {NULL}};
const Segment SGI127857[] = {{&S13585[30], 6}, {NULL}};
const Segment SGI127858[] = {{&S13585[117], 3}, {NULL}};
const Segment SGI127859[] = {{&S13585[82], 6}, {NULL}};
const Segment SGI127860[] = {{&S13586[44], 8}, {NULL}};
const Segment SGI127861[] = {{&S13586[44], 20}, {NULL}};
const Segment SGI127862[] = {{&S13586[30], 6}, {NULL}};
const Segment SGI127863[] = {{&S13586[113], 3}, {NULL}};
const Segment SGI127864[] = {{&S13586[78], 6}, {NULL}};
const Segment SGI127865[] = {{&S13587[44], 11}, {NULL}};
const Segment SGI127866[] = {{&S13587[44], 23}, {NULL}};
const Segment SGI127867[] = {{&S13587[30], 6}, {NULL}};
const Segment SGI127868[] = {{&S13587[116], 3}, {NULL}};
const Segment SGI127869[] = {{&S13587[81], 6}, {NULL}};
const Segment SGI127870[] = {{&S13589[44], 14}, {NULL}};
const Segment SGI127871[] = {{&S13589[44], 26}, {NULL}};
const Segment SGI127872[] = {{&S13589[30], 6}, {NULL}};
const Segment SGI127873[] = {{&S13589[119], 3}, {NULL}};
const Segment SGI127874[] = {{&S13589[84], 6}, {NULL}};
const Segment SGI127875[] = {{&S13590[44], 9}, {NULL}};
const Segment SGI127876[] = {{&S13590[44], 21}, {NULL}};
const Segment SGI127877[] = {{&S13590[30], 6}, {NULL}};
const Segment SGI127878[] = {{&S13590[114], 3}, {NULL}};
const Segment SGI127879[] = {{&S13590[79], 6}, {NULL}};
const Segment SGI127880[] = {{&S13594[44], 7}, {NULL}};
const Segment SGI127881[] = {{&S13594[44], 19}, {NULL}};
const Segment SGI127882[] = {{&S13594[30], 6}, {NULL}};
const Segment SGI127883[] = {{&S13594[112], 3}, {NULL}};
const Segment SGI127884[] = {{&S13594[77], 6}, {NULL}};
const Segment SGI127885[] = {{&S13595[44], 37}, {NULL}};
const Segment SGI127886[] = {{&S13595[30], 6}, {NULL}};
const Segment SGI127887[] = {{&S13595[113], 6}, {NULL}};
const Segment SGI127888[] = {{&S13596[58], 13}, {NULL}};
const Segment SGI127889[] = {{&S13596[58], 25}, {NULL}};
const Segment SGI127890[] = {{&S13596[44], 6}, {NULL}};
const Segment SGI127891[] = {{&S13596[132], 3}, {NULL}};
const Segment SGI127892[] = {{&S13596[97], 6}, {NULL}};
const Segment SGI127893[] = {{&S13597[44], 12}, {NULL}};
const Segment SGI127894[] = {{&S13597[44], 24}, {NULL}};
const Segment SGI127895[] = {{&S13597[30], 6}, {NULL}};
const Segment SGI127896[] = {{&S13597[117], 3}, {NULL}};
const Segment SGI127897[] = {{&S13597[82], 6}, {NULL}};
const Segment SGI127898[] = {{&S13598[44], 10}, {NULL}};
const Segment SGI127899[] = {{&S13598[44], 22}, {NULL}};
const Segment SGI127900[] = {{&S13598[30], 6}, {NULL}};
const Segment SGI127901[] = {{&S13598[115], 3}, {NULL}};
const Segment SGI127902[] = {{&S13598[80], 6}, {NULL}};
const Segment SGI127903[] = {{&S13599[44], 9}, {NULL}};
const Segment SGI127904[] = {{&S13599[44], 21}, {NULL}};
const Segment SGI127905[] = {{&S13599[30], 6}, {NULL}};
const Segment SGI127906[] = {{&S13599[114], 3}, {NULL}};
const Segment SGI127907[] = {{&S13599[79], 6}, {NULL}};
const Segment SGI127908[] = {{&S13601[44], 7}, {NULL}};
const Segment SGI127909[] = {{&S13601[44], 19}, {NULL}};
const Segment SGI127910[] = {{&S13601[30], 6}, {NULL}};
const Segment SGI127911[] = {{&S13601[112], 3}, {NULL}};
const Segment SGI127912[] = {{&S13601[77], 6}, {NULL}};
const Segment SGI127913[] = {{&S13604[44], 14}, {NULL}};
const Segment SGI127914[] = {{&S13604[44], 26}, {NULL}};
const Segment SGI127915[] = {{&S13604[30], 6}, {NULL}};
const Segment SGI127916[] = {{&S13604[119], 3}, {NULL}};
const Segment SGI127917[] = {{&S13604[84], 6}, {NULL}};
const Segment SGI127918[] = {{&S13606[44], 5}, {NULL}};
const Segment SGI127919[] = {{&S13606[44], 30}, {NULL}};
const Segment SGI127920[] = {{&S13606[30], 6}, {NULL}};
const Segment SGI127921[] = {{&S13606[123], 3}, {NULL}};
const Segment SGI127922[] = {{&S13606[88], 6}, {NULL}};
const Segment SGI127923[] = {{&S13607[44], 6}, {NULL}};
const Segment SGI127924[] = {{&S13607[44], 18}, {NULL}};
const Segment SGI127925[] = {{&S13607[30], 6}, {NULL}};
const Segment SGI127926[] = {{&S13607[111], 3}, {NULL}};
const Segment SGI127927[] = {{&S13607[76], 6}, {NULL}};
const Segment SGI127928[] = {{&S13609[44], 10}, {NULL}};
const Segment SGI127929[] = {{&S13609[44], 22}, {NULL}};
const Segment SGI127930[] = {{&S13609[30], 6}, {NULL}};
const Segment SGI127931[] = {{&S13609[115], 3}, {NULL}};
const Segment SGI127932[] = {{&S13609[80], 6}, {NULL}};
const Segment SGI127933[] = {{&S13610[44], 6}, {NULL}};
const Segment SGI127934[] = {{&S13610[44], 18}, {NULL}};
const Segment SGI127935[] = {{&S13610[30], 6}, {NULL}};
const Segment SGI127936[] = {{&S13610[111], 3}, {NULL}};
const Segment SGI127937[] = {{&S13610[76], 6}, {NULL}};
const Segment SGI127938[] = {{&S13611[44], 9}, {NULL}};
const Segment SGI127939[] = {{&S13611[44], 21}, {NULL}};
const Segment SGI127940[] = {{&S13611[30], 6}, {NULL}};
const Segment SGI127941[] = {{&S13611[114], 3}, {NULL}};
const Segment SGI127942[] = {{&S13611[79], 6}, {NULL}};
const Segment SGI127943[] = {{&S13612[44], 7}, {NULL}};
const Segment SGI127944[] = {{&S13612[44], 19}, {NULL}};
const Segment SGI127945[] = {{&S13612[30], 6}, {NULL}};
const Segment SGI127946[] = {{&S13612[112], 3}, {NULL}};
const Segment SGI127947[] = {{&S13612[77], 6}, {NULL}};
const Segment SGI127948[] = {{&S13614[44], 19}, {NULL}};
const Segment SGI127949[] = {{&S13614[44], 31}, {NULL}};
const Segment SGI127950[] = {{&S13614[30], 6}, {NULL}};
const Segment SGI127951[] = {{&S13614[124], 3}, {NULL}};
const Segment SGI127952[] = {{&S13614[89], 6}, {NULL}};
const Segment SGI127953[] = {{&S13615[44], 18}, {NULL}};
const Segment SGI127954[] = {{&S13615[44], 30}, {NULL}};
const Segment SGI127955[] = {{&S13615[30], 6}, {NULL}};
const Segment SGI127956[] = {{&S13615[123], 3}, {NULL}};
const Segment SGI127957[] = {{&S13615[88], 6}, {NULL}};
const Segment SGI127958[] = {{&S13616[44], 11}, {NULL}};
const Segment SGI127959[] = {{&S13616[44], 23}, {NULL}};
const Segment SGI127960[] = {{&S13616[30], 6}, {NULL}};
const Segment SGI127961[] = {{&S13616[116], 3}, {NULL}};
const Segment SGI127962[] = {{&S13616[81], 6}, {NULL}};
const Segment SGI127963[] = {{&S13617[44], 6}, {NULL}};
const Segment SGI127964[] = {{&S13617[44], 18}, {NULL}};
const Segment SGI127965[] = {{&S13617[30], 6}, {NULL}};
const Segment SGI127966[] = {{&S13617[111], 3}, {NULL}};
const Segment SGI127967[] = {{&S13617[76], 6}, {NULL}};
const Segment SGI127968[] = {{&S13618[44], 11}, {NULL}};
const Segment SGI127969[] = {{&S13618[44], 23}, {NULL}};
const Segment SGI127970[] = {{&S13618[30], 6}, {NULL}};
const Segment SGI127971[] = {{&S13618[116], 3}, {NULL}};
const Segment SGI127972[] = {{&S13618[81], 6}, {NULL}};
const Segment SGI127973[] = {{&S13619[44], 16}, {NULL}};
const Segment SGI127974[] = {{&S13619[44], 28}, {NULL}};
const Segment SGI127975[] = {{&S13619[30], 6}, {NULL}};
const Segment SGI127976[] = {{&S13619[121], 3}, {NULL}};
const Segment SGI127977[] = {{&S13619[86], 6}, {NULL}};
const Segment SGI127978[] = {{&S13621[44], 22}, {NULL}};
const Segment SGI127979[] = {{&S13621[44], 34}, {NULL}};
const Segment SGI127980[] = {{&S13621[30], 6}, {NULL}};
const Segment SGI127981[] = {{&S13621[127], 3}, {NULL}};
const Segment SGI127982[] = {{&S13621[92], 6}, {NULL}};
const Segment SGI127983[] = {{&S13622[44], 11}, {NULL}};
const Segment SGI127984[] = {{&S13622[44], 32}, {NULL}};
const Segment SGI127985[] = {{&S13622[30], 6}, {NULL}};
const Segment SGI127986[] = {{&S13622[125], 3}, {NULL}};
const Segment SGI127987[] = {{&S13622[90], 6}, {NULL}};
const Segment SGI127988[] = {{&S13623[44], 10}, {NULL}};
const Segment SGI127989[] = {{&S13623[44], 17}, {NULL}};
const Segment SGI127990[] = {{&S13623[30], 6}, {NULL}};
const Segment SGI127991[] = {{&S13623[110], 3}, {NULL}};
const Segment SGI127992[] = {{&S13623[75], 6}, {NULL}};
const Segment SGI127993[] = {{&S13627[44], 6}, {NULL}};
const Segment SGI127994[] = {{&S13627[44], 18}, {NULL}};
const Segment SGI127995[] = {{&S13627[30], 6}, {NULL}};
const Segment SGI127996[] = {{&S13627[111], 3}, {NULL}};
const Segment SGI127997[] = {{&S13627[76], 6}, {NULL}};
const Segment SGI127998[] = {{&S13628[44], 72}, {NULL}};
const Segment SGI127999[] = {{&S13628[44], 93}, {NULL}};
const Segment SGI128000[] = {{&S13628[30], 6}, {NULL}};
const Segment SGI128001[] = {{&S13628[81], 3}, {NULL}};
const Segment SGI128002[] = {{&S13628[110], 5}, {NULL}};
const Segment SGI128003[] = {{&S13629[44], 7}, {NULL}};
const Segment SGI128004[] = {{&S13629[44], 19}, {NULL}};
const Segment SGI128005[] = {{&S13629[30], 6}, {NULL}};
const Segment SGI128006[] = {{&S13629[112], 3}, {NULL}};
const Segment SGI128007[] = {{&S13629[77], 6}, {NULL}};
const Segment SGI128008[] = {{&S13631[44], 9}, {NULL}};
const Segment SGI128009[] = {{&S13631[44], 21}, {NULL}};
const Segment SGI128010[] = {{&S13631[30], 6}, {NULL}};
const Segment SGI128011[] = {{&S13631[114], 3}, {NULL}};
const Segment SGI128012[] = {{&S13631[79], 6}, {NULL}};
const Segment SGI128013[] = {{&S13632[44], 6}, {NULL}};
const Segment SGI128014[] = {{&S13632[44], 18}, {NULL}};
const Segment SGI128015[] = {{&S13632[30], 6}, {NULL}};
const Segment SGI128016[] = {{&S13632[111], 3}, {NULL}};
const Segment SGI128017[] = {{&S13632[76], 6}, {NULL}};
const Segment SGI128018[] = {{&S13633[44], 4}, {NULL}};
const Segment SGI128019[] = {{&S13633[44], 16}, {NULL}};
const Segment SGI128020[] = {{&S13633[30], 6}, {NULL}};
const Segment SGI128021[] = {{&S13633[109], 3}, {NULL}};
const Segment SGI128022[] = {{&S13633[74], 6}, {NULL}};
const Segment SGI128023[] = {{&S13634[44], 30}, {NULL}};
const Segment SGI128024[] = {{&S13634[44], 42}, {NULL}};
const Segment SGI128025[] = {{&S13634[30], 6}, {NULL}};
const Segment SGI128026[] = {{&S13634[135], 3}, {NULL}};
const Segment SGI128027[] = {{&S13634[100], 6}, {NULL}};
const Segment SGI128028[] = {{&S13635[44], 7}, {NULL}};
const Segment SGI128029[] = {{&S13635[44], 19}, {NULL}};
const Segment SGI128030[] = {{&S13635[30], 6}, {NULL}};
const Segment SGI128031[] = {{&S13635[112], 3}, {NULL}};
const Segment SGI128032[] = {{&S13635[77], 6}, {NULL}};
const Segment SGI128033[] = {{&S13637[44], 9}, {NULL}};
const Segment SGI128034[] = {{&S13637[44], 21}, {NULL}};
const Segment SGI128035[] = {{&S13637[30], 6}, {NULL}};
const Segment SGI128036[] = {{&S13637[114], 3}, {NULL}};
const Segment SGI128037[] = {{&S13637[79], 6}, {NULL}};
const Segment SGI128038[] = {{&S13638[44], 11}, {NULL}};
const Segment SGI128039[] = {{&S13638[44], 23}, {NULL}};
const Segment SGI128040[] = {{&S13638[30], 6}, {NULL}};
const Segment SGI128041[] = {{&S13638[116], 3}, {NULL}};
const Segment SGI128042[] = {{&S13638[81], 6}, {NULL}};
const Segment SGI128043[] = {{&S13639[44], 9}, {NULL}};
const Segment SGI128044[] = {{&S13639[44], 21}, {NULL}};
const Segment SGI128045[] = {{&S13639[30], 6}, {NULL}};
const Segment SGI128046[] = {{&S13639[114], 3}, {NULL}};
const Segment SGI128047[] = {{&S13639[79], 6}, {NULL}};
const Segment SGI128048[] = {{&S13640[38], 7}, {NULL}};
const Segment SGI128049[] = {{&S13640[31], 6}, {NULL}};
const Segment SGI128050[] = {{&S13642[44], 12}, {NULL}};
const Segment SGI128051[] = {{&S13642[44], 24}, {NULL}};
const Segment SGI128052[] = {{&S13642[30], 6}, {NULL}};
const Segment SGI128053[] = {{&S105453[44], 14}, {NULL}};
const Segment SGI128054[] = {{&S105453[44], 24}, {NULL}};
const Segment SGI128055[] = {{&S105453[30], 6}, {NULL}};
const Segment SGI128056[] = {{&S13642[117], 3}, {NULL}};
const Segment SGI128057[] = {{&S13642[82], 6}, {NULL}};
const Segment SGI128058[] = {{&S105453[31], 3}, {NULL}};
const Segment SGI128059[] = {{&S105453[82], 6}, {NULL}};
const Segment SGI128060[] = {{&S13643[44], 12}, {NULL}};
const Segment SGI128061[] = {{&S13643[44], 24}, {NULL}};
const Segment SGI128062[] = {{&S13643[30], 6}, {NULL}};
const Segment SGI128063[] = {{&S13643[117], 3}, {NULL}};
const Segment SGI128064[] = {{&S13643[82], 6}, {NULL}};
const Segment SGI128065[] = {{&S13644[44], 26}, {NULL}};
const Segment SGI128066[] = {{&S13644[44], 38}, {NULL}};
const Segment SGI128067[] = {{&S13644[30], 6}, {NULL}};
const Segment SGI128068[] = {{&S13644[131], 3}, {NULL}};
const Segment SGI128069[] = {{&S13644[96], 6}, {NULL}};
const Segment SGI128070[] = {{&S13645[44], 9}, {NULL}};
const Segment SGI128071[] = {{&S13645[44], 21}, {NULL}};
const Segment SGI128072[] = {{&S13645[30], 6}, {NULL}};
const Segment SGI128073[] = {{&S13645[114], 3}, {NULL}};
const Segment SGI128074[] = {{&S13645[79], 6}, {NULL}};
const Segment SGI128075[] = {{&S13646[44], 15}, {NULL}};
const Segment SGI128076[] = {{&S13646[44], 27}, {NULL}};
const Segment SGI128077[] = {{&S13646[30], 6}, {NULL}};
const Segment SGI128078[] = {{&S13646[120], 3}, {NULL}};
const Segment SGI128079[] = {{&S13646[85], 6}, {NULL}};
const Segment SGI128080[] = {{&S13647[44], 14}, {NULL}};
const Segment SGI128081[] = {{&S13647[44], 21}, {NULL}};
const Segment SGI128082[] = {{&S13647[30], 6}, {NULL}};
const Segment SGI128083[] = {{&S13647[114], 3}, {NULL}};
const Segment SGI128084[] = {{&S13647[79], 6}, {NULL}};
const Segment SGI128085[] = {{&S105463[30], 6}, {NULL}};
const Segment SGI128086[] = {{&S105463[31], 3}, {NULL}};
const Segment SGI128087[] = {{&S105463[125], 3}, {NULL}};
const Segment SGI128088[] = {{&S105463[57], 5}, {NULL}};
const Segment SGI128089[] = {{&S13648[44], 9}, {NULL}};
const Segment SGI128090[] = {{&S13648[44], 21}, {NULL}};
const Segment SGI128091[] = {{&S13648[30], 6}, {NULL}};
const Segment SGI128092[] = {{&S13648[114], 3}, {NULL}};
const Segment SGI128093[] = {{&S13648[79], 6}, {NULL}};
const Segment SGI128094[] = {{&S13649[44], 10}, {NULL}};
const Segment SGI128095[] = {{&S13649[44], 22}, {NULL}};
const Segment SGI128096[] = {{&S13649[30], 6}, {NULL}};
const Segment SGI128097[] = {{&S13649[115], 3}, {NULL}};
const Segment SGI128098[] = {{&S13649[80], 6}, {NULL}};
const Segment SGI128099[] = {{&S13651[44], 8}, {NULL}};
const Segment SGI128100[] = {{&S13651[44], 20}, {NULL}};
const Segment SGI128101[] = {{&S13651[30], 6}, {NULL}};
const Segment SGI128102[] = {{&S13651[113], 3}, {NULL}};
const Segment SGI128103[] = {{&S13651[78], 6}, {NULL}};
const Segment SGI128104[] = {{&S13653[44], 12}, {NULL}};
const Segment SGI128105[] = {{&S13653[44], 24}, {NULL}};
const Segment SGI128106[] = {{&S13653[30], 6}, {NULL}};
const Segment SGI128107[] = {{&S13653[117], 3}, {NULL}};
const Segment SGI128108[] = {{&S13653[82], 6}, {NULL}};
const Segment SGI128109[] = {{&S13654[44], 8}, {NULL}};
const Segment SGI128110[] = {{&S13654[44], 20}, {NULL}};
const Segment SGI128111[] = {{&S13654[30], 6}, {NULL}};
const Segment SGI128112[] = {{&S13654[113], 3}, {NULL}};
const Segment SGI128113[] = {{&S13654[78], 6}, {NULL}};
const Segment SGI128114[] = {{&S13655[44], 12}, {NULL}};
const Segment SGI128115[] = {{&S13655[44], 26}, {NULL}};
const Segment SGI128116[] = {{&S13655[30], 6}, {NULL}};
const Segment SGI128117[] = {{&S13655[119], 3}, {NULL}};
const Segment SGI128118[] = {{&S13655[84], 6}, {NULL}};
const Segment SGI128119[] = {{&S13656[44], 7}, {NULL}};
const Segment SGI128120[] = {{&S13656[44], 19}, {NULL}};
const Segment SGI128121[] = {{&S13656[30], 6}, {NULL}};
const Segment SGI128122[] = {{&S13656[112], 3}, {NULL}};
const Segment SGI128123[] = {{&S13656[77], 6}, {NULL}};
const Segment SGI128124[] = {{&S13657[44], 9}, {NULL}};
const Segment SGI128125[] = {{&S13657[44], 21}, {NULL}};
const Segment SGI128126[] = {{&S13657[30], 6}, {NULL}};
const Segment SGI128127[] = {{&S13657[114], 3}, {NULL}};
const Segment SGI128128[] = {{&S13657[79], 6}, {NULL}};
const Segment SGI128129[] = {{&S13658[44], 8}, {NULL}};
const Segment SGI128130[] = {{&S13658[44], 20}, {NULL}};
const Segment SGI128131[] = {{&S13658[30], 6}, {NULL}};
const Segment SGI128132[] = {{&S13658[113], 3}, {NULL}};
const Segment SGI128133[] = {{&S13658[78], 6}, {NULL}};
const Segment SGI128134[] = {{&S13660[44], 12}, {NULL}};
const Segment SGI128135[] = {{&S13660[44], 24}, {NULL}};
const Segment SGI128136[] = {{&S13660[30], 6}, {NULL}};
const Segment SGI128137[] = {{&S13660[117], 3}, {NULL}};
const Segment SGI128138[] = {{&S13660[82], 6}, {NULL}};
const Segment SGI128139[] = {{&S13661[44], 13}, {NULL}};
const Segment SGI128140[] = {{&S13661[44], 25}, {NULL}};
const Segment SGI128141[] = {{&S13661[30], 6}, {NULL}};
const Segment SGI128142[] = {{&S13661[118], 3}, {NULL}};
const Segment SGI128143[] = {{&S13661[83], 6}, {NULL}};
const Segment SGI128144[] = {{&S13662[44], 9}, {NULL}};
const Segment SGI128145[] = {{&S13662[44], 21}, {NULL}};
const Segment SGI128146[] = {{&S13662[30], 6}, {NULL}};
const Segment SGI128147[] = {{&S13662[114], 3}, {NULL}};
const Segment SGI128148[] = {{&S13662[79], 6}, {NULL}};
const Segment SGI128149[] = {{&S13663[44], 15}, {NULL}};
const Segment SGI128150[] = {{&S13663[44], 27}, {NULL}};
const Segment SGI128151[] = {{&S13663[30], 6}, {NULL}};
const Segment SGI128152[] = {{&S13663[120], 3}, {NULL}};
const Segment SGI128153[] = {{&S13663[85], 6}, {NULL}};
const Segment SGI128154[] = {{&S13664[44], 13}, {NULL}};
const Segment SGI128155[] = {{&S13664[44], 25}, {NULL}};
const Segment SGI128156[] = {{&S13664[30], 6}, {NULL}};
const Segment SGI128157[] = {{&S13664[118], 3}, {NULL}};
const Segment SGI128158[] = {{&S13664[83], 6}, {NULL}};
const Segment SGI128159[] = {{&S13665[44], 14}, {NULL}};
const Segment SGI128160[] = {{&S13665[44], 26}, {NULL}};
const Segment SGI128161[] = {{&S13665[30], 6}, {NULL}};
const Segment SGI128162[] = {{&S13665[31], 3}, {NULL}};
const Segment SGI128163[] = {{&S13665[84], 6}, {NULL}};
const Segment SGI128164[] = {{&S13668[44], 14}, {NULL}};
const Segment SGI128165[] = {{&S13668[44], 26}, {NULL}};
const Segment SGI128166[] = {{&S13668[30], 6}, {NULL}};
const Segment SGI128167[] = {{&S13668[119], 3}, {NULL}};
const Segment SGI128168[] = {{&S13668[84], 6}, {NULL}};
const Segment SGI128169[] = {{&S13671[44], 14}, {NULL}};
const Segment SGI128170[] = {{&S13671[44], 26}, {NULL}};
const Segment SGI128171[] = {{&S13671[30], 6}, {NULL}};
const Segment SGI128172[] = {{&S13671[119], 3}, {NULL}};
const Segment SGI128173[] = {{&S13671[84], 6}, {NULL}};
const Segment SGI128174[] = {{&S13673[44], 25}, {NULL}};
const Segment SGI128175[] = {{&S13673[44], 37}, {NULL}};
const Segment SGI128176[] = {{&S13673[30], 6}, {NULL}};
const Segment SGI128177[] = {{&S105513[44], 11}, {NULL}};
const Segment SGI128178[] = {{&S105513[44], 23}, {NULL}};
const Segment SGI128179[] = {{&S13673[130], 3}, {NULL}};
const Segment SGI128180[] = {{&S105513[30], 6}, {NULL}};
const Segment SGI128181[] = {{&S13673[95], 6}, {NULL}};
const Segment SGI128182[] = {{&S105513[31], 3}, {NULL}};
const Segment SGI128183[] = {{&S105513[81], 6}, {NULL}};
const Segment SGI128184[] = {{&S105513[151], 3}, {NULL}};
const Segment SGI128185[] = {{&S13674[44], 9}, {NULL}};
const Segment SGI128186[] = {{&S13674[44], 21}, {NULL}};
const Segment SGI128187[] = {{&S13674[30], 6}, {NULL}};
const Segment SGI128188[] = {{&S13674[114], 3}, {NULL}};
const Segment SGI128189[] = {{&S13674[79], 6}, {NULL}};
const Segment SGI128190[] = {{&S13675[44], 6}, {NULL}};
const Segment SGI128191[] = {{&S13675[44], 18}, {NULL}};
const Segment SGI128192[] = {{&S13675[30], 6}, {NULL}};
const Segment SGI128193[] = {{&S13675[111], 3}, {NULL}};
const Segment SGI128194[] = {{&S13675[76], 6}, {NULL}};
const Segment SGI128195[] = {{&S13679[44], 8}, {NULL}};
const Segment SGI128196[] = {{&S13679[44], 20}, {NULL}};
const Segment SGI128197[] = {{&S13679[30], 6}, {NULL}};
const Segment SGI128198[] = {{&S13679[113], 3}, {NULL}};
const Segment SGI128199[] = {{&S13679[78], 6}, {NULL}};
const Segment SGI128200[] = {{&S13680[44], 6}, {NULL}};
const Segment SGI128201[] = {{&S13680[44], 18}, {NULL}};
const Segment SGI128202[] = {{&S13680[30], 6}, {NULL}};
const Segment SGI128203[] = {{&S13680[111], 3}, {NULL}};
const Segment SGI128204[] = {{&S13680[76], 6}, {NULL}};
const Segment SGI128205[] = {{&S13681[44], 7}, {NULL}};
const Segment SGI128206[] = {{&S13681[44], 19}, {NULL}};
const Segment SGI128207[] = {{&S13681[30], 6}, {NULL}};
const Segment SGI128208[] = {{&S13681[111], 3}, {NULL}};
const Segment SGI128209[] = {{&S13681[77], 5}, {NULL}};
const Segment SGI128210[] = {{&S13682[44], 12}, {NULL}};
const Segment SGI128211[] = {{&S13682[44], 24}, {NULL}};
const Segment SGI128212[] = {{&S13682[30], 6}, {NULL}};
const Segment SGI128213[] = {{&S13682[117], 3}, {NULL}};
const Segment SGI128214[] = {{&S13682[82], 6}, {NULL}};
const Segment SGI128215[] = {{&S13683[44], 6}, {NULL}};
const Segment SGI128216[] = {{&S105529[44], 15}, {NULL}};
const Segment SGI128217[] = {{&S13683[44], 27}, {NULL}};
const Segment SGI128218[] = {{&S105529[44], 29}, {NULL}};
const Segment SGI128219[] = {{&S13683[30], 6}, {NULL}};
const Segment SGI128220[] = {{&S105529[30], 6}, {NULL}};
const Segment SGI128221[] = {{&S13683[119], 3}, {NULL}};
const Segment SGI128222[] = {{&S105529[31], 3}, {NULL}};
const Segment SGI128223[] = {{&S13683[85], 5}, {NULL}};
const Segment SGI128224[] = {{&S105529[94], 6}, {NULL}};
const Segment SGI128225[] = {{&S105529[73], 8}, {NULL}};
const Segment SGI128226[] = {{&S13684[44], 11}, {NULL}};
const Segment SGI128227[] = {{&S13684[44], 30}, {NULL}};
const Segment SGI128228[] = {{&S13684[30], 6}, {NULL}};
const Segment SGI128229[] = {{&S13684[123], 3}, {NULL}};
const Segment SGI128230[] = {{&S13684[88], 6}, {NULL}};
const Segment SGI128231[] = {{&S105532[44], 11}, {NULL}};
const Segment SGI128232[] = {{&S105532[44], 23}, {NULL}};
const Segment SGI128233[] = {{&S105532[30], 6}, {NULL}};
const Segment SGI128234[] = {{&S13686[44], 7}, {NULL}};
const Segment SGI128235[] = {{&S13686[44], 19}, {NULL}};
const Segment SGI128236[] = {{&S13686[30], 6}, {NULL}};
const Segment SGI128237[] = {{&S105532[31], 3}, {NULL}};
const Segment SGI128238[] = {{&S105532[81], 6}, {NULL}};
const Segment SGI128239[] = {{&S13686[112], 3}, {NULL}};
const Segment SGI128240[] = {{&S105532[151], 3}, {NULL}};
const Segment SGI128241[] = {{&S13686[77], 6}, {NULL}};
const Segment SGI128242[] = {{&S13687[44], 7}, {NULL}};
const Segment SGI128243[] = {{&S13687[44], 19}, {NULL}};
const Segment SGI128244[] = {{&S13687[30], 6}, {NULL}};
const Segment SGI128245[] = {{&S13687[112], 3}, {NULL}};
const Segment SGI128246[] = {{&S13687[77], 6}, {NULL}};
const Segment SGI128247[] = {{&S13688[44], 13}, {NULL}};
const Segment SGI128248[] = {{&S13688[44], 25}, {NULL}};
const Segment SGI128249[] = {{&S13688[30], 6}, {NULL}};
const Segment SGI128250[] = {{&S13688[118], 3}, {NULL}};
const Segment SGI128251[] = {{&S13688[83], 6}, {NULL}};
const Segment SGI128252[] = {{&S13689[44], 8}, {NULL}};
const Segment SGI128253[] = {{&S13689[44], 20}, {NULL}};
const Segment SGI128254[] = {{&S13689[30], 6}, {NULL}};
const Segment SGI128255[] = {{&S13689[113], 3}, {NULL}};
const Segment SGI128256[] = {{&S13689[78], 6}, {NULL}};
const Segment SGI128257[] = {{&S13690[44], 11}, {NULL}};
const Segment SGI128258[] = {{&S13690[44], 23}, {NULL}};
const Segment SGI128259[] = {{&S13690[30], 6}, {NULL}};
const Segment SGI128260[] = {{&S13690[116], 3}, {NULL}};
const Segment SGI128261[] = {{&S13690[81], 6}, {NULL}};
const Segment SGI128262[] = {{&S13691[44], 7}, {NULL}};
const Segment SGI128263[] = {{&S13691[44], 19}, {NULL}};
const Segment SGI128264[] = {{&S13691[30], 6}, {NULL}};
const Segment SGI128265[] = {{&S13691[112], 3}, {NULL}};
const Segment SGI128266[] = {{&S13691[77], 6}, {NULL}};
const Segment SGI128267[] = {{&S13692[44], 12}, {NULL}};
const Segment SGI128268[] = {{&S13692[44], 24}, {NULL}};
const Segment SGI128269[] = {{&S13692[30], 6}, {NULL}};
const Segment SGI128270[] = {{&S13692[117], 3}, {NULL}};
const Segment SGI128271[] = {{&S13692[82], 6}, {NULL}};
const Segment SGI128272[] = {{&S13693[44], 6}, {NULL}};
const Segment SGI128273[] = {{&S13693[44], 18}, {NULL}};
const Segment SGI128274[] = {{&S13693[30], 6}, {NULL}};
const Segment SGI128275[] = {{&S13693[111], 3}, {NULL}};
const Segment SGI128276[] = {{&S13693[76], 6}, {NULL}};
const Segment SGI128277[] = {{&S13694[44], 6}, {NULL}};
const Segment SGI128278[] = {{&S13694[44], 18}, {NULL}};
const Segment SGI128279[] = {{&S13694[30], 6}, {NULL}};
const Segment SGI128280[] = {{&S13694[111], 3}, {NULL}};
const Segment SGI128281[] = {{&S13694[76], 6}, {NULL}};
const Segment SGI128282[] = {{&S13695[44], 10}, {NULL}};
const Segment SGI128283[] = {{&S13695[44], 22}, {NULL}};
const Segment SGI128284[] = {{&S13695[30], 6}, {NULL}};
const Segment SGI128285[] = {{&S13695[115], 3}, {NULL}};
const Segment SGI128286[] = {{&S13695[80], 6}, {NULL}};
const Segment SGI128287[] = {{&S13696[44], 7}, {NULL}};
const Segment SGI128288[] = {{&S13696[44], 19}, {NULL}};
const Segment SGI128289[] = {{&S13696[30], 6}, {NULL}};
const Segment SGI128290[] = {{&S13696[112], 3}, {NULL}};
const Segment SGI128291[] = {{&S13696[77], 6}, {NULL}};
const Segment SGI128292[] = {{&S13697[44], 7}, {NULL}};
const Segment SGI128293[] = {{&S13697[44], 19}, {NULL}};
const Segment SGI128294[] = {{&S13697[30], 6}, {NULL}};
const Segment SGI128295[] = {{&S13697[112], 3}, {NULL}};
const Segment SGI128296[] = {{&S13697[77], 6}, {NULL}};
const Segment SGI128297[] = {{&S13698[44], 8}, {NULL}};
const Segment SGI128298[] = {{&S13698[44], 20}, {NULL}};
const Segment SGI128299[] = {{&S13698[30], 6}, {NULL}};
const Segment SGI128300[] = {{&S13698[113], 3}, {NULL}};
const Segment SGI128301[] = {{&S13698[78], 6}, {NULL}};
const Segment SGI128302[] = {{&S13699[44], 11}, {NULL}};
const Segment SGI128303[] = {{&S13699[44], 23}, {NULL}};
const Segment SGI128304[] = {{&S13699[30], 6}, {NULL}};
const Segment SGI128305[] = {{&S13699[116], 3}, {NULL}};
const Segment SGI128306[] = {{&S13699[81], 6}, {NULL}};
const Segment SGI128307[] = {{&S13700[44], 10}, {NULL}};
const Segment SGI128308[] = {{&S13700[44], 22}, {NULL}};
const Segment SGI128309[] = {{&S13700[30], 6}, {NULL}};
const Segment SGI128310[] = {{&S13700[31], 3}, {NULL}};
const Segment SGI128311[] = {{&S13700[80], 6}, {NULL}};
const Segment SGI128312[] = {{&S13701[44], 12}, {NULL}};
const Segment SGI128313[] = {{&S13701[44], 24}, {NULL}};
const Segment SGI128314[] = {{&S13701[30], 6}, {NULL}};
const Segment SGI128315[] = {{&S13701[117], 3}, {NULL}};
const Segment SGI128316[] = {{&S13701[82], 6}, {NULL}};
const Segment SGI128317[] = {{&S13702[44], 9}, {NULL}};
const Segment SGI128318[] = {{&S13702[44], 21}, {NULL}};
const Segment SGI128319[] = {{&S13702[30], 6}, {NULL}};
const Segment SGI128320[] = {{&S13702[114], 3}, {NULL}};
const Segment SGI128321[] = {{&S13702[79], 6}, {NULL}};
const Segment SGI128322[] = {{&S13703[44], 8}, {NULL}};
const Segment SGI128323[] = {{&S13703[44], 20}, {NULL}};
const Segment SGI128324[] = {{&S13703[30], 6}, {NULL}};
const Segment SGI128325[] = {{&S13703[113], 3}, {NULL}};
const Segment SGI128326[] = {{&S13703[78], 6}, {NULL}};
const Segment SGI128327[] = {{&S13704[44], 13}, {NULL}};
const Segment SGI128328[] = {{&S13704[44], 25}, {NULL}};
const Segment SGI128329[] = {{&S13704[30], 6}, {NULL}};
const Segment SGI128330[] = {{&S13704[118], 3}, {NULL}};
const Segment SGI128331[] = {{&S13704[83], 6}, {NULL}};
const Segment SGI128332[] = {{&S13705[44], 8}, {NULL}};
const Segment SGI128333[] = {{&S13705[44], 20}, {NULL}};
const Segment SGI128334[] = {{&S13705[30], 6}, {NULL}};
const Segment SGI128335[] = {{&S13705[113], 3}, {NULL}};
const Segment SGI128336[] = {{&S13705[78], 6}, {NULL}};
const Segment SGI128337[] = {{&S105573[44], 11}, {NULL}};
const Segment SGI128338[] = {{&S105573[44], 23}, {NULL}};
const Segment SGI128339[] = {{&S13706[44], 12}, {NULL}};
const Segment SGI128340[] = {{&S105573[30], 6}, {NULL}};
const Segment SGI128341[] = {{&S13706[44], 23}, {NULL}};
const Segment SGI128342[] = {{&S13706[30], 6}, {NULL}};
const Segment SGI128343[] = {{&S105573[31], 3}, {NULL}};
const Segment SGI128344[] = {{&S13706[116], 3}, {NULL}};
const Segment SGI128345[] = {{&S105573[88], 6}, {NULL}};
const Segment SGI128346[] = {{&S13706[81], 6}, {NULL}};
const Segment SGI128347[] = {{&S105573[67], 8}, {NULL}};
const Segment SGI128348[] = {{&S13707[44], 16}, {NULL}};
const Segment SGI128349[] = {{&S13707[44], 28}, {NULL}};
const Segment SGI128350[] = {{&S13707[30], 6}, {NULL}};
const Segment SGI128351[] = {{&S13707[121], 3}, {NULL}};
const Segment SGI128352[] = {{&S13707[86], 6}, {NULL}};
const Segment SGI128353[] = {{&S13708[44], 15}, {NULL}};
const Segment SGI128354[] = {{&S13708[44], 27}, {NULL}};
const Segment SGI128355[] = {{&S13708[30], 6}, {NULL}};
const Segment SGI128356[] = {{&S13708[120], 3}, {NULL}};
const Segment SGI128357[] = {{&S13708[85], 6}, {NULL}};
const Segment SGI128358[] = {{&S105577[45], 11}, {NULL}};
const Segment SGI128359[] = {{&S105577[30], 6}, {NULL}};
const Segment SGI128360[] = {{&S13709[44], 8}, {NULL}};
const Segment SGI128361[] = {{&S13709[44], 20}, {NULL}};
const Segment SGI128362[] = {{&S13709[30], 6}, {NULL}};
const Segment SGI128363[] = {{&S105577[31], 3}, {NULL}};
const Segment SGI128364[] = {{&S13709[113], 3}, {NULL}};
const Segment SGI128365[] = {{&S105577[82], 5}, {NULL}};
const Segment SGI128366[] = {{&S13709[78], 6}, {NULL}};
const Segment SGI128367[] = {{&S105577[140], 13}, {NULL}};
const Segment SGI128368[] = {{&S13710[44], 11}, {NULL}};
const Segment SGI128369[] = {{&S13710[44], 23}, {NULL}};
const Segment SGI128370[] = {{&S13710[30], 6}, {NULL}};
const Segment SGI128371[] = {{&S13710[116], 3}, {NULL}};
const Segment SGI128372[] = {{&S13710[81], 6}, {NULL}};
const Segment SGI128373[] = {{&S13711[44], 17}, {NULL}};
const Segment SGI128374[] = {{&S13711[44], 29}, {NULL}};
const Segment SGI128375[] = {{&S13711[30], 6}, {NULL}};
const Segment SGI128376[] = {{&S13711[122], 3}, {NULL}};
const Segment SGI128377[] = {{&S13711[87], 6}, {NULL}};
const Segment SGI128378[] = {{&S13712[44], 9}, {NULL}};
const Segment SGI128379[] = {{&S13712[44], 21}, {NULL}};
const Segment SGI128380[] = {{&S13712[30], 6}, {NULL}};
const Segment SGI128381[] = {{&S13712[114], 3}, {NULL}};
const Segment SGI128382[] = {{&S13712[79], 6}, {NULL}};
const Segment SGI128383[] = {{&S13713[44], 11}, {NULL}};
const Segment SGI128384[] = {{&S13713[44], 23}, {NULL}};
const Segment SGI128385[] = {{&S13713[30], 6}, {NULL}};
const Segment SGI128386[] = {{&S13713[116], 3}, {NULL}};
const Segment SGI128387[] = {{&S13713[81], 6}, {NULL}};
const Segment SGI128388[] = {{&S13714[44], 12}, {NULL}};
const Segment SGI128389[] = {{&S13714[44], 24}, {NULL}};
const Segment SGI128390[] = {{&S13714[30], 6}, {NULL}};
const Segment SGI128391[] = {{&S13714[117], 3}, {NULL}};
const Segment SGI128392[] = {{&S13714[82], 6}, {NULL}};
const Segment SGI128393[] = {{&S13715[44], 6}, {NULL}};
const Segment SGI128394[] = {{&S13715[44], 18}, {NULL}};
const Segment SGI128395[] = {{&S13715[30], 6}, {NULL}};
const Segment SGI128396[] = {{&S13715[111], 3}, {NULL}};
const Segment SGI128397[] = {{&S13715[76], 6}, {NULL}};
const Segment SGI128398[] = {{&S13716[44], 6}, {NULL}};
const Segment SGI128399[] = {{&S13716[44], 18}, {NULL}};
const Segment SGI128400[] = {{&S13716[30], 6}, {NULL}};
const Segment SGI128401[] = {{&S13716[111], 3}, {NULL}};
const Segment SGI128402[] = {{&S13716[76], 6}, {NULL}};
const Segment SGI128403[] = {{&S105594[44], 10}, {NULL}};
const Segment SGI128404[] = {{&S105594[44], 22}, {NULL}};
const Segment SGI128405[] = {{&S105594[30], 6}, {NULL}};
const Segment SGI128406[] = {{&S13717[44], 13}, {NULL}};
const Segment SGI128407[] = {{&S13717[44], 25}, {NULL}};
const Segment SGI128408[] = {{&S13717[30], 6}, {NULL}};
const Segment SGI128409[] = {{&S105594[31], 3}, {NULL}};
const Segment SGI128410[] = {{&S13717[118], 3}, {NULL}};
const Segment SGI128411[] = {{&S13717[83], 6}, {NULL}};
const Segment SGI128412[] = {{&S105594[80], 6}, {NULL}};
const Segment SGI128413[] = {{&S105594[143], 3}, {NULL}};
const Segment SGI128414[] = {{&S13718[44], 8}, {NULL}};
const Segment SGI128415[] = {{&S13718[44], 20}, {NULL}};
const Segment SGI128416[] = {{&S13718[30], 6}, {NULL}};
const Segment SGI128417[] = {{&S13718[113], 3}, {NULL}};
const Segment SGI128418[] = {{&S13718[78], 6}, {NULL}};
const Segment SGI128419[] = {{&S13719[44], 9}, {NULL}};
const Segment SGI128420[] = {{&S13719[44], 21}, {NULL}};
const Segment SGI128421[] = {{&S13719[30], 6}, {NULL}};
const Segment SGI128422[] = {{&S13719[114], 3}, {NULL}};
const Segment SGI128423[] = {{&S13719[79], 6}, {NULL}};
const Segment SGI128424[] = {{&S13720[44], 9}, {NULL}};
const Segment SGI128425[] = {{&S13720[44], 21}, {NULL}};
const Segment SGI128426[] = {{&S13720[30], 6}, {NULL}};
const Segment SGI128427[] = {{&S13720[114], 3}, {NULL}};
const Segment SGI128428[] = {{&S13720[79], 6}, {NULL}};
const Segment SGI128429[] = {{&S13722[44], 12}, {NULL}};
const Segment SGI128430[] = {{&S13722[44], 24}, {NULL}};
const Segment SGI128431[] = {{&S13722[30], 6}, {NULL}};
const Segment SGI128432[] = {{&S13722[117], 3}, {NULL}};
const Segment SGI128433[] = {{&S13722[82], 6}, {NULL}};
const Segment SGI128434[] = {{&S105605[59], 10}, {NULL}};
const Segment SGI128435[] = {{&S13724[44], 9}, {NULL}};
const Segment SGI128436[] = {{&S105605[59], 22}, {NULL}};
const Segment SGI128437[] = {{&S13724[44], 21}, {NULL}};
const Segment SGI128438[] = {{&S105605[45], 6}, {NULL}};
const Segment SGI128439[] = {{&S13724[30], 6}, {NULL}};
const Segment SGI128440[] = {{&S105605[46], 3}, {NULL}};
const Segment SGI128441[] = {{&S13724[114], 3}, {NULL}};
const Segment SGI128442[] = {{&S105605[95], 6}, {NULL}};
const Segment SGI128443[] = {{&S13724[79], 6}, {NULL}};
const Segment SGI128444[] = {{&S105605[0], 14}, {NULL}};
const Segment SGI128445[] = {{&S13725[44], 6}, {NULL}};
const Segment SGI128446[] = {{&S13725[44], 18}, {NULL}};
const Segment SGI128447[] = {{&S13725[30], 6}, {NULL}};
const Segment SGI128448[] = {{&S13725[111], 3}, {NULL}};
const Segment SGI128449[] = {{&S13725[76], 6}, {NULL}};
const Segment SGI128450[] = {{&S13726[44], 8}, {NULL}};
const Segment SGI128451[] = {{&S13726[44], 20}, {NULL}};
const Segment SGI128452[] = {{&S13726[30], 6}, {NULL}};
const Segment SGI128453[] = {{&S13726[113], 3}, {NULL}};
const Segment SGI128454[] = {{&S13726[78], 6}, {NULL}};
const Segment SGI128455[] = {{&S13728[44], 14}, {NULL}};
const Segment SGI128456[] = {{&S13728[44], 31}, {NULL}};
const Segment SGI128457[] = {{&S13728[30], 6}, {NULL}};
const Segment SGI128458[] = {{&S13728[124], 3}, {NULL}};
const Segment SGI128459[] = {{&S13728[89], 6}, {NULL}};
const Segment SGI128460[] = {{&S13729[44], 11}, {NULL}};
const Segment SGI128461[] = {{&S13729[44], 23}, {NULL}};
const Segment SGI128462[] = {{&S13729[30], 6}, {NULL}};
const Segment SGI128463[] = {{&S13729[116], 3}, {NULL}};
const Segment SGI128464[] = {{&S13729[81], 6}, {NULL}};
const Segment SGI128465[] = {{&S13730[44], 6}, {NULL}};
const Segment SGI128466[] = {{&S13730[44], 27}, {NULL}};
const Segment SGI128467[] = {{&S13730[30], 6}, {NULL}};
const Segment SGI128468[] = {{&S13730[119], 3}, {NULL}};
const Segment SGI128469[] = {{&S13730[85], 5}, {NULL}};
const Segment SGI128470[] = {{&S13731[44], 22}, {NULL}};
const Segment SGI128471[] = {{&S13731[44], 34}, {NULL}};
const Segment SGI128472[] = {{&S13731[30], 6}, {NULL}};
const Segment SGI128473[] = {{&S13731[127], 3}, {NULL}};
const Segment SGI128474[] = {{&S13731[92], 6}, {NULL}};
const Segment SGI128475[] = {{&S13732[44], 22}, {NULL}};
const Segment SGI128476[] = {{&S13732[44], 35}, {NULL}};
const Segment SGI128477[] = {{&S13732[30], 6}, {NULL}};
const Segment SGI128478[] = {{&S13732[128], 3}, {NULL}};
const Segment SGI128479[] = {{&S13732[93], 6}, {NULL}};
const Segment SGI128480[] = {{&S13735[44], 17}, {NULL}};
const Segment SGI128481[] = {{&S13735[44], 29}, {NULL}};
const Segment SGI128482[] = {{&S13735[30], 6}, {NULL}};
const Segment SGI128483[] = {{&S13735[122], 3}, {NULL}};
const Segment SGI128484[] = {{&S13735[87], 6}, {NULL}};
const Segment SGI128485[] = {{&S13737[44], 18}, {NULL}};
const Segment SGI128486[] = {{&S13737[44], 30}, {NULL}};
const Segment SGI128487[] = {{&S13737[30], 6}, {NULL}};
const Segment SGI128488[] = {{&S13737[123], 3}, {NULL}};
const Segment SGI128489[] = {{&S13737[88], 6}, {NULL}};
const Segment SGI128490[] = {{&S13738[44], 83}, {NULL}};
const Segment SGI128491[] = {{&S13738[44], 104}, {NULL}};
const Segment SGI128492[] = {{&S13738[30], 6}, {NULL}};
const Segment SGI128493[] = {{&S13738[92], 3}, {NULL}};
const Segment SGI128494[] = {{&S13738[121], 5}, {NULL}};
const Segment SGI128495[] = {{&S13739[44], 13}, {NULL}};
const Segment SGI128496[] = {{&S13739[44], 25}, {NULL}};
const Segment SGI128497[] = {{&S13739[30], 6}, {NULL}};
const Segment SGI128498[] = {{&S13739[118], 3}, {NULL}};
const Segment SGI128499[] = {{&S13739[83], 6}, {NULL}};
const Segment SGI128500[] = {{&S13741[44], 9}, {NULL}};
const Segment SGI128501[] = {{&S13741[44], 21}, {NULL}};
const Segment SGI128502[] = {{&S13741[30], 6}, {NULL}};
const Segment SGI128503[] = {{&S13741[114], 3}, {NULL}};
const Segment SGI128504[] = {{&S13741[79], 6}, {NULL}};
const Segment SGI128505[] = {{&S105634[59], 10}, {NULL}};
const Segment SGI128506[] = {{&S105634[59], 22}, {NULL}};
const Segment SGI128507[] = {{&S105634[45], 6}, {NULL}};
const Segment SGI128508[] = {{&S105634[46], 3}, {NULL}};
const Segment SGI128509[] = {{&S105634[95], 6}, {NULL}};
const Segment SGI128510[] = {{&S105634[0], 14}, {NULL}};
const Segment SGI128511[] = {{&S13742[44], 17}, {NULL}};
const Segment SGI128512[] = {{&S13742[44], 29}, {NULL}};
const Segment SGI128513[] = {{&S13742[30], 6}, {NULL}};
const Segment SGI128514[] = {{&S13742[122], 3}, {NULL}};
const Segment SGI128515[] = {{&S13742[87], 6}, {NULL}};
const Segment SGI128516[] = {{&S13743[44], 11}, {NULL}};
const Segment SGI128517[] = {{&S13743[44], 23}, {NULL}};
const Segment SGI128518[] = {{&S13743[30], 6}, {NULL}};
const Segment SGI128519[] = {{&S13743[116], 3}, {NULL}};
const Segment SGI128520[] = {{&S13743[81], 6}, {NULL}};
const Segment SGI128521[] = {{&S13744[44], 7}, {NULL}};
const Segment SGI128522[] = {{&S13744[44], 19}, {NULL}};
const Segment SGI128523[] = {{&S13744[30], 6}, {NULL}};
const Segment SGI128524[] = {{&S13744[112], 3}, {NULL}};
const Segment SGI128525[] = {{&S13744[77], 6}, {NULL}};
const Segment SGI128526[] = {{&S13745[44], 12}, {NULL}};
const Segment SGI128527[] = {{&S105639[59], 10}, {NULL}};
const Segment SGI128528[] = {{&S13745[44], 28}, {NULL}};
const Segment SGI128529[] = {{&S105639[59], 22}, {NULL}};
const Segment SGI128530[] = {{&S13745[30], 6}, {NULL}};
const Segment SGI128531[] = {{&S105639[45], 6}, {NULL}};
const Segment SGI128532[] = {{&S13745[121], 3}, {NULL}};
const Segment SGI128533[] = {{&S105639[46], 3}, {NULL}};
const Segment SGI128534[] = {{&S13745[86], 6}, {NULL}};
const Segment SGI128535[] = {{&S105639[95], 6}, {NULL}};
const Segment SGI128536[] = {{&S105639[0], 14}, {NULL}};
const Segment SGI128537[] = {{&S13746[44], 12}, {NULL}};
const Segment SGI128538[] = {{&S13746[44], 24}, {NULL}};
const Segment SGI128539[] = {{&S13746[30], 6}, {NULL}};
const Segment SGI128540[] = {{&S13746[117], 3}, {NULL}};
const Segment SGI128541[] = {{&S13746[82], 6}, {NULL}};
const Segment SGI128542[] = {{&S13747[44], 15}, {NULL}};
const Segment SGI128543[] = {{&S13747[44], 27}, {NULL}};
const Segment SGI128544[] = {{&S13747[30], 6}, {NULL}};
const Segment SGI128545[] = {{&S13747[120], 3}, {NULL}};
const Segment SGI128546[] = {{&S13747[85], 6}, {NULL}};
const Segment SGI128547[] = {{&S13749[44], 11}, {NULL}};
const Segment SGI128548[] = {{&S13749[44], 23}, {NULL}};
const Segment SGI128549[] = {{&S13749[30], 6}, {NULL}};
const Segment SGI128550[] = {{&S13749[116], 3}, {NULL}};
const Segment SGI128551[] = {{&S13749[81], 6}, {NULL}};
const Segment SGI128552[] = {{&S13750[44], 6}, {NULL}};
const Segment SGI128553[] = {{&S13750[44], 18}, {NULL}};
const Segment SGI128554[] = {{&S13750[30], 6}, {NULL}};
const Segment SGI128555[] = {{&S13750[111], 3}, {NULL}};
const Segment SGI128556[] = {{&S13750[76], 6}, {NULL}};
const Segment SGI128557[] = {{&S13751[44], 9}, {NULL}};
const Segment SGI128558[] = {{&S13751[44], 21}, {NULL}};
const Segment SGI128559[] = {{&S13751[30], 6}, {NULL}};
const Segment SGI128560[] = {{&S13751[114], 3}, {NULL}};
const Segment SGI128561[] = {{&S13751[79], 6}, {NULL}};
const Segment SGI128562[] = {{&S13752[44], 8}, {NULL}};
const Segment SGI128563[] = {{&S13752[44], 20}, {NULL}};
const Segment SGI128564[] = {{&S13752[30], 6}, {NULL}};
const Segment SGI128565[] = {{&S13752[113], 3}, {NULL}};
const Segment SGI128566[] = {{&S13752[78], 6}, {NULL}};
const Segment SGI128567[] = {{&S13754[44], 8}, {NULL}};
const Segment SGI128568[] = {{&S13754[44], 20}, {NULL}};
const Segment SGI128569[] = {{&S13754[30], 6}, {NULL}};
const Segment SGI128570[] = {{&S13754[113], 3}, {NULL}};
const Segment SGI128571[] = {{&S13754[78], 6}, {NULL}};
const Segment SGI128572[] = {{&S13755[44], 9}, {NULL}};
const Segment SGI128573[] = {{&S13755[44], 21}, {NULL}};
const Segment SGI128574[] = {{&S13755[30], 6}, {NULL}};
const Segment SGI128575[] = {{&S13755[114], 3}, {NULL}};
const Segment SGI128576[] = {{&S13755[79], 6}, {NULL}};
const Segment SGI128577[] = {{&S13757[44], 9}, {NULL}};
const Segment SGI128578[] = {{&S13757[44], 21}, {NULL}};
const Segment SGI128579[] = {{&S13757[30], 6}, {NULL}};
const Segment SGI128580[] = {{&S13757[114], 3}, {NULL}};
const Segment SGI128581[] = {{&S13757[79], 6}, {NULL}};
const Segment SGI128582[] = {{&S13760[44], 84}, {NULL}};
const Segment SGI128583[] = {{&S13760[44], 105}, {NULL}};
const Segment SGI128584[] = {{&S13760[30], 6}, {NULL}};
const Segment SGI128585[] = {{&S13760[196], 3}, {NULL}};
const Segment SGI128586[] = {{&S13760[122], 5}, {NULL}};
const Segment SGI128587[] = {{&S13761[44], 14}, {NULL}};
const Segment SGI128588[] = {{&S13761[44], 26}, {NULL}};
const Segment SGI128589[] = {{&S13761[30], 6}, {NULL}};
const Segment SGI128590[] = {{&S13761[119], 3}, {NULL}};
const Segment SGI128591[] = {{&S13761[84], 6}, {NULL}};
const Segment SGI128592[] = {{&S105664[31], 8}, {NULL}};
const Segment SGI128593[] = {{&S105664[55], 3}, {NULL}};
const Segment SGI128594[] = {{&S13762[44], 5}, {NULL}};
const Segment SGI128595[] = {{&S13762[44], 12}, {NULL}};
const Segment SGI128596[] = {{&S13762[30], 6}, {NULL}};
const Segment SGI128597[] = {{&S13762[105], 3}, {NULL}};
const Segment SGI128598[] = {{&S13762[70], 6}, {NULL}};
const Segment SGI128599[] = {{&S105666[44], 10}, {NULL}};
const Segment SGI128600[] = {{&S105666[44], 40}, {NULL}};
const Segment SGI128601[] = {{&S105666[30], 6}, {NULL}};
const Segment SGI128602[] = {{&S13763[44], 83}, {NULL}};
const Segment SGI128603[] = {{&S13763[44], 104}, {NULL}};
const Segment SGI128604[] = {{&S105666[31], 3}, {NULL}};
const Segment SGI128605[] = {{&S13763[30], 6}, {NULL}};
const Segment SGI128606[] = {{&S105666[98], 6}, {NULL}};
const Segment SGI128607[] = {{&S13763[195], 3}, {NULL}};
const Segment SGI128608[] = {{&S13763[121], 5}, {NULL}};
const Segment SGI128609[] = {{&S13764[44], 12}, {NULL}};
const Segment SGI128610[] = {{&S13764[44], 24}, {NULL}};
const Segment SGI128611[] = {{&S13764[30], 6}, {NULL}};
const Segment SGI128612[] = {{&S13764[117], 3}, {NULL}};
const Segment SGI128613[] = {{&S13764[82], 6}, {NULL}};
const Segment SGI128614[] = {{&S13767[44], 84}, {NULL}};
const Segment SGI128615[] = {{&S13767[44], 105}, {NULL}};
const Segment SGI128616[] = {{&S13767[30], 6}, {NULL}};
const Segment SGI128617[] = {{&S13767[196], 3}, {NULL}};
const Segment SGI128618[] = {{&S13767[122], 5}, {NULL}};
const Segment SGI128619[] = {{&S13768[44], 7}, {NULL}};
const Segment SGI128620[] = {{&S13768[44], 19}, {NULL}};
const Segment SGI128621[] = {{&S13768[30], 6}, {NULL}};
const Segment SGI128622[] = {{&S13768[112], 3}, {NULL}};
const Segment SGI128623[] = {{&S13768[77], 6}, {NULL}};
const Segment SGI128624[] = {{&S13769[44], 8}, {NULL}};
const Segment SGI128625[] = {{&S13769[44], 20}, {NULL}};
const Segment SGI128626[] = {{&S13769[30], 6}, {NULL}};
const Segment SGI128627[] = {{&S13769[113], 3}, {NULL}};
const Segment SGI128628[] = {{&S13769[78], 6}, {NULL}};
const Segment SGI128629[] = {{&S13770[58], 11}, {NULL}};
const Segment SGI128630[] = {{&S13770[58], 23}, {NULL}};
const Segment SGI128631[] = {{&S13770[44], 6}, {NULL}};
const Segment SGI128632[] = {{&S13770[130], 3}, {NULL}};
const Segment SGI128633[] = {{&S13770[95], 6}, {NULL}};
const Segment SGI128634[] = {{&S13771[44], 20}, {NULL}};
const Segment SGI128635[] = {{&S13771[44], 32}, {NULL}};
const Segment SGI128636[] = {{&S13771[30], 6}, {NULL}};
const Segment SGI128637[] = {{&S13771[125], 3}, {NULL}};
const Segment SGI128638[] = {{&S13771[90], 6}, {NULL}};
const Segment SGI128639[] = {{&S13772[44], 8}, {NULL}};
const Segment SGI128640[] = {{&S13772[44], 20}, {NULL}};
const Segment SGI128641[] = {{&S13772[30], 6}, {NULL}};
const Segment SGI128642[] = {{&S13772[113], 3}, {NULL}};
const Segment SGI128643[] = {{&S13772[78], 6}, {NULL}};
const Segment SGI128644[] = {{&S13773[44], 12}, {NULL}};
const Segment SGI128645[] = {{&S13773[30], 6}, {NULL}};
const Segment SGI128646[] = {{&S13773[119], 3}, {NULL}};
const Segment SGI128647[] = {{&S13773[84], 6}, {NULL}};
const Segment SGI128648[] = {{&S13774[44], 12}, {NULL}};
const Segment SGI128649[] = {{&S13774[30], 6}, {NULL}};
const Segment SGI128650[] = {{&S13774[119], 3}, {NULL}};
const Segment SGI128651[] = {{&S13774[84], 6}, {NULL}};
const Segment SGI128652[] = {{&S13775[44], 9}, {NULL}};
const Segment SGI128653[] = {{&S13775[44], 21}, {NULL}};
const Segment SGI128654[] = {{&S13775[30], 6}, {NULL}};
const Segment SGI128655[] = {{&S13775[31], 3}, {NULL}};
const Segment SGI128656[] = {{&S13775[79], 6}, {NULL}};
const Segment SGI128657[] = {{&S13776[44], 28}, {NULL}};
const Segment SGI128658[] = {{&S13776[44], 40}, {NULL}};
const Segment SGI128659[] = {{&S13776[30], 6}, {NULL}};
const Segment SGI128660[] = {{&S13776[133], 3}, {NULL}};
const Segment SGI128661[] = {{&S13776[98], 6}, {NULL}};
const Segment SGI128662[] = {{&S13778[44], 5}, {NULL}};
const Segment SGI128663[] = {{&S13778[44], 17}, {NULL}};
const Segment SGI128664[] = {{&S13778[30], 6}, {NULL}};
const Segment SGI128665[] = {{&S13778[110], 3}, {NULL}};
const Segment SGI128666[] = {{&S13778[75], 6}, {NULL}};
const Segment SGI128667[] = {{&S13779[44], 7}, {NULL}};
const Segment SGI128668[] = {{&S13779[44], 19}, {NULL}};
const Segment SGI128669[] = {{&S13779[30], 6}, {NULL}};
const Segment SGI128670[] = {{&S13779[112], 3}, {NULL}};
const Segment SGI128671[] = {{&S13779[77], 6}, {NULL}};
const Segment SGI128672[] = {{&S13780[44], 7}, {NULL}};
const Segment SGI128673[] = {{&S13780[44], 19}, {NULL}};
const Segment SGI128674[] = {{&S13780[30], 6}, {NULL}};
const Segment SGI128675[] = {{&S13780[112], 3}, {NULL}};
const Segment SGI128676[] = {{&S13780[77], 6}, {NULL}};
const Segment SGI128677[] = {{&S13781[44], 9}, {NULL}};
const Segment SGI128678[] = {{&S13781[44], 21}, {NULL}};
const Segment SGI128679[] = {{&S13781[30], 6}, {NULL}};
const Segment SGI128680[] = {{&S13781[114], 3}, {NULL}};
const Segment SGI128681[] = {{&S13781[79], 6}, {NULL}};
const Segment SGI128682[] = {{&S13782[44], 8}, {NULL}};
const Segment SGI128683[] = {{&S13782[44], 20}, {NULL}};
const Segment SGI128684[] = {{&S13782[30], 6}, {NULL}};
const Segment SGI128685[] = {{&S13782[113], 3}, {NULL}};
const Segment SGI128686[] = {{&S13782[78], 6}, {NULL}};
const Segment SGI128687[] = {{&S105701[44], 10}, {NULL}};
const Segment SGI128688[] = {{&S105701[44], 22}, {NULL}};
const Segment SGI128689[] = {{&S105701[30], 6}, {NULL}};
const Segment SGI128690[] = {{&S105701[31], 3}, {NULL}};
const Segment SGI128691[] = {{&S13783[44], 7}, {NULL}};
const Segment SGI128692[] = {{&S13783[44], 19}, {NULL}};
const Segment SGI128693[] = {{&S105701[80], 6}, {NULL}};
const Segment SGI128694[] = {{&S13783[30], 6}, {NULL}};
const Segment SGI128695[] = {{&S105701[133], 11}, {NULL}};
const Segment SGI128696[] = {{&S13783[112], 3}, {NULL}};
const Segment SGI128697[] = {{&S13783[77], 6}, {NULL}};
const Segment SGI128698[] = {{&S13784[44], 9}, {NULL}};
const Segment SGI128699[] = {{&S13784[44], 21}, {NULL}};
const Segment SGI128700[] = {{&S13784[30], 6}, {NULL}};
const Segment SGI128701[] = {{&S13784[114], 3}, {NULL}};
const Segment SGI128702[] = {{&S13784[79], 6}, {NULL}};
const Segment SGI128703[] = {{&S13785[44], 16}, {NULL}};
const Segment SGI128704[] = {{&S13785[44], 28}, {NULL}};
const Segment SGI128705[] = {{&S13785[30], 6}, {NULL}};
const Segment SGI128706[] = {{&S13785[121], 3}, {NULL}};
const Segment SGI128707[] = {{&S13785[86], 6}, {NULL}};
const Segment SGI128708[] = {{&S13786[38], 4}, {NULL}};
const Segment SGI128709[] = {{&S13786[31], 6}, {NULL}};
const Segment SGI128710[] = {{&S13787[44], 21}, {NULL}};
const Segment SGI128711[] = {{&S13787[44], 38}, {NULL}};
const Segment SGI128712[] = {{&S13787[30], 6}, {NULL}};
const Segment SGI128713[] = {{&S13787[130], 3}, {NULL}};
const Segment SGI128714[] = {{&S13787[96], 5}, {NULL}};
const Segment SGI128715[] = {{&S13788[44], 4}, {NULL}};
const Segment SGI128716[] = {{&S13788[44], 16}, {NULL}};
const Segment SGI128717[] = {{&S13788[30], 6}, {NULL}};
const Segment SGI128718[] = {{&S13788[109], 3}, {NULL}};
const Segment SGI128719[] = {{&S13788[74], 6}, {NULL}};
const Segment SGI128720[] = {{&S13789[44], 10}, {NULL}};
const Segment SGI128721[] = {{&S13789[44], 22}, {NULL}};
const Segment SGI128722[] = {{&S13789[30], 6}, {NULL}};
const Segment SGI128723[] = {{&S13789[115], 3}, {NULL}};
const Segment SGI128724[] = {{&S13789[80], 6}, {NULL}};
const Segment SGI128725[] = {{&S13790[44], 9}, {NULL}};
const Segment SGI128726[] = {{&S13790[44], 21}, {NULL}};
const Segment SGI128727[] = {{&S13790[30], 6}, {NULL}};
const Segment SGI128728[] = {{&S13790[114], 3}, {NULL}};
const Segment SGI128729[] = {{&S13790[79], 6}, {NULL}};
const Segment SGI128730[] = {{&S13791[44], 9}, {NULL}};
const Segment SGI128731[] = {{&S13791[44], 21}, {NULL}};
const Segment SGI128732[] = {{&S13791[30], 6}, {NULL}};
const Segment SGI128733[] = {{&S13791[114], 3}, {NULL}};
const Segment SGI128734[] = {{&S13791[79], 6}, {NULL}};
const Segment SGI128735[] = {{&S13792[44], 6}, {NULL}};
const Segment SGI128736[] = {{&S13792[44], 18}, {NULL}};
const Segment SGI128737[] = {{&S13792[30], 6}, {NULL}};
const Segment SGI128738[] = {{&S13792[111], 3}, {NULL}};
const Segment SGI128739[] = {{&S13792[76], 6}, {NULL}};
const Segment SGI128740[] = {{&S13793[44], 13}, {NULL}};
const Segment SGI128741[] = {{&S13793[44], 25}, {NULL}};
const Segment SGI128742[] = {{&S13793[30], 6}, {NULL}};
const Segment SGI128743[] = {{&S13793[118], 3}, {NULL}};
const Segment SGI128744[] = {{&S13793[83], 6}, {NULL}};
const Segment SGI128745[] = {{&S13794[44], 6}, {NULL}};
const Segment SGI128746[] = {{&S13794[44], 18}, {NULL}};
const Segment SGI128747[] = {{&S13794[30], 6}, {NULL}};
const Segment SGI128748[] = {{&S13794[111], 3}, {NULL}};
const Segment SGI128749[] = {{&S13794[76], 6}, {NULL}};
const Segment SGI128750[] = {{&S13795[44], 6}, {NULL}};
const Segment SGI128751[] = {{&S13795[44], 18}, {NULL}};
const Segment SGI128752[] = {{&S13795[30], 6}, {NULL}};
const Segment SGI128753[] = {{&S13795[79], 6}, {NULL}};
const Segment SGI128754[] = {{&S13796[44], 8}, {NULL}};
const Segment SGI128755[] = {{&S13796[44], 20}, {NULL}};
const Segment SGI128756[] = {{&S13796[30], 6}, {NULL}};
const Segment SGI128757[] = {{&S13796[113], 3}, {NULL}};
const Segment SGI128758[] = {{&S105725[59], 10}, {NULL}};
const Segment SGI128759[] = {{&S105725[59], 22}, {NULL}};
const Segment SGI128760[] = {{&S13796[78], 6}, {NULL}};
const Segment SGI128761[] = {{&S105725[45], 6}, {NULL}};
const Segment SGI128762[] = {{&S105725[46], 3}, {NULL}};
const Segment SGI128763[] = {{&S105725[95], 6}, {NULL}};
const Segment SGI128764[] = {{&S105725[0], 14}, {NULL}};
const Segment SGI128765[] = {{&S13797[44], 6}, {NULL}};
const Segment SGI128766[] = {{&S13797[44], 18}, {NULL}};
const Segment SGI128767[] = {{&S13797[30], 6}, {NULL}};
const Segment SGI128768[] = {{&S13797[111], 3}, {NULL}};
const Segment SGI128769[] = {{&S13797[76], 6}, {NULL}};
const Segment SGI128770[] = {{&S13798[44], 17}, {NULL}};
const Segment SGI128771[] = {{&S13798[44], 29}, {NULL}};
const Segment SGI128772[] = {{&S13798[30], 6}, {NULL}};
const Segment SGI128773[] = {{&S13798[122], 3}, {NULL}};
const Segment SGI128774[] = {{&S13798[87], 6}, {NULL}};
const Segment SGI128775[] = {{&S13799[44], 7}, {NULL}};
const Segment SGI128776[] = {{&S13799[44], 19}, {NULL}};
const Segment SGI128777[] = {{&S13799[30], 6}, {NULL}};
const Segment SGI128778[] = {{&S13799[112], 3}, {NULL}};
const Segment SGI128779[] = {{&S13799[77], 6}, {NULL}};
const Segment SGI128780[] = {{&S13800[44], 6}, {NULL}};
const Segment SGI128781[] = {{&S13800[44], 18}, {NULL}};
const Segment SGI128782[] = {{&S13800[30], 6}, {NULL}};
const Segment SGI128783[] = {{&S13800[111], 3}, {NULL}};
const Segment SGI128784[] = {{&S13800[76], 6}, {NULL}};
const Segment SGI128785[] = {{&S13801[44], 7}, {NULL}};
const Segment SGI128786[] = {{&S13801[44], 19}, {NULL}};
const Segment SGI128787[] = {{&S13801[30], 6}, {NULL}};
const Segment SGI128788[] = {{&S13801[112], 3}, {NULL}};
const Segment SGI128789[] = {{&S13801[77], 6}, {NULL}};
const Segment SGI128790[] = {{&S13803[44], 6}, {NULL}};
const Segment SGI128791[] = {{&S13803[44], 18}, {NULL}};
const Segment SGI128792[] = {{&S13803[30], 6}, {NULL}};
const Segment SGI128793[] = {{&S13803[111], 3}, {NULL}};
const Segment SGI128794[] = {{&S13803[76], 6}, {NULL}};
const Segment SGI128795[] = {{&S13804[38], 8}, {NULL}};
const Segment SGI128796[] = {{&S13804[31], 6}, {NULL}};
const Segment SGI128797[] = {{&S13806[44], 5}, {NULL}};
const Segment SGI128798[] = {{&S13806[44], 17}, {NULL}};
const Segment SGI128799[] = {{&S13806[30], 6}, {NULL}};
const Segment SGI128800[] = {{&S13806[110], 3}, {NULL}};
const Segment SGI128801[] = {{&S13806[75], 6}, {NULL}};
const Segment SGI128802[] = {{&S13807[44], 12}, {NULL}};
const Segment SGI128803[] = {{&S13807[44], 24}, {NULL}};
const Segment SGI128804[] = {{&S13807[30], 6}, {NULL}};
const Segment SGI128805[] = {{&S13807[117], 3}, {NULL}};
const Segment SGI128806[] = {{&S13807[82], 6}, {NULL}};
const Segment SGI128807[] = {{&S13808[44], 15}, {NULL}};
const Segment SGI128808[] = {{&S13808[44], 27}, {NULL}};
const Segment SGI128809[] = {{&S13808[30], 6}, {NULL}};
const Segment SGI128810[] = {{&S13808[120], 3}, {NULL}};
const Segment SGI128811[] = {{&S13808[85], 6}, {NULL}};
const Segment SGI128812[] = {{&S13810[44], 10}, {NULL}};
const Segment SGI128813[] = {{&S13810[44], 22}, {NULL}};
const Segment SGI128814[] = {{&S13810[30], 6}, {NULL}};
const Segment SGI128815[] = {{&S13810[115], 3}, {NULL}};
const Segment SGI128816[] = {{&S13810[80], 6}, {NULL}};
const Segment SGI128817[] = {{&S13811[44], 10}, {NULL}};
const Segment SGI128818[] = {{&S13811[44], 22}, {NULL}};
const Segment SGI128819[] = {{&S13811[30], 6}, {NULL}};
const Segment SGI128820[] = {{&S13811[115], 3}, {NULL}};
const Segment SGI128821[] = {{&S13811[80], 6}, {NULL}};
const Segment SGI128822[] = {{&S13812[44], 8}, {NULL}};
const Segment SGI128823[] = {{&S13812[44], 20}, {NULL}};
const Segment SGI128824[] = {{&S13812[30], 6}, {NULL}};
const Segment SGI128825[] = {{&S13812[113], 3}, {NULL}};
const Segment SGI128826[] = {{&S13812[78], 6}, {NULL}};
const Segment SGI128827[] = {{&S13813[44], 5}, {NULL}};
const Segment SGI128828[] = {{&S13813[44], 17}, {NULL}};
const Segment SGI128829[] = {{&S13813[30], 6}, {NULL}};
const Segment SGI128830[] = {{&S13813[110], 3}, {NULL}};
const Segment SGI128831[] = {{&S13813[75], 6}, {NULL}};
const Segment SGI128832[] = {{&S13814[44], 14}, {NULL}};
const Segment SGI128833[] = {{&S13814[44], 31}, {NULL}};
const Segment SGI128834[] = {{&S13814[30], 6}, {NULL}};
const Segment SGI128835[] = {{&S13814[92], 6}, {NULL}};
const Segment SGI128836[] = {{&S13815[44], 14}, {NULL}};
const Segment SGI128837[] = {{&S13815[44], 32}, {NULL}};
const Segment SGI128838[] = {{&S13815[30], 6}, {NULL}};
const Segment SGI128839[] = {{&S13815[125], 3}, {NULL}};
const Segment SGI128840[] = {{&S13815[90], 6}, {NULL}};
const Segment SGI128841[] = {{&S13816[44], 21}, {NULL}};
const Segment SGI128842[] = {{&S13816[44], 33}, {NULL}};
const Segment SGI128843[] = {{&S13816[30], 6}, {NULL}};
const Segment SGI128844[] = {{&S13816[126], 3}, {NULL}};
const Segment SGI128845[] = {{&S13816[91], 6}, {NULL}};
const Segment SGI128846[] = {{&S13817[44], 20}, {NULL}};
const Segment SGI128847[] = {{&S13817[44], 32}, {NULL}};
const Segment SGI128848[] = {{&S13817[30], 6}, {NULL}};
const Segment SGI128849[] = {{&S13817[125], 3}, {NULL}};
const Segment SGI128850[] = {{&S13817[90], 6}, {NULL}};
const Segment SGI128851[] = {{&S13818[44], 7}, {NULL}};
const Segment SGI128852[] = {{&S13818[44], 19}, {NULL}};
const Segment SGI128853[] = {{&S13818[30], 6}, {NULL}};
const Segment SGI128854[] = {{&S13818[112], 3}, {NULL}};
const Segment SGI128855[] = {{&S13818[77], 6}, {NULL}};
const Segment SGI128856[] = {{&S13820[44], 4}, {NULL}};
const Segment SGI128857[] = {{&S13820[44], 16}, {NULL}};
const Segment SGI128858[] = {{&S13820[30], 6}, {NULL}};
const Segment SGI128859[] = {{&S13820[109], 3}, {NULL}};
const Segment SGI128860[] = {{&S13820[74], 6}, {NULL}};
const Segment SGI128861[] = {{&S105767[44], 10}, {NULL}};
const Segment SGI128862[] = {{&S105767[44], 22}, {NULL}};
const Segment SGI128863[] = {{&S105767[30], 6}, {NULL}};
const Segment SGI128864[] = {{&S105767[31], 3}, {NULL}};
const Segment SGI128865[] = {{&S105767[80], 5}, {NULL}};
const Segment SGI128866[] = {{&S105767[117], 14}, {NULL}};
const Segment SGI128867[] = {{&S13821[44], 8}, {NULL}};
const Segment SGI128868[] = {{&S13821[44], 20}, {NULL}};
const Segment SGI128869[] = {{&S13821[30], 6}, {NULL}};
const Segment SGI128870[] = {{&S13821[113], 3}, {NULL}};
const Segment SGI128871[] = {{&S13821[78], 6}, {NULL}};
const Segment SGI128872[] = {{&S13822[44], 7}, {NULL}};
const Segment SGI128873[] = {{&S13822[44], 19}, {NULL}};
const Segment SGI128874[] = {{&S13822[30], 6}, {NULL}};
const Segment SGI128875[] = {{&S13822[112], 3}, {NULL}};
const Segment SGI128876[] = {{&S13822[77], 6}, {NULL}};
const Segment SGI128877[] = {{&S13823[44], 10}, {NULL}};
const Segment SGI128878[] = {{&S13823[44], 22}, {NULL}};
const Segment SGI128879[] = {{&S13823[30], 6}, {NULL}};
const Segment SGI128880[] = {{&S13823[115], 3}, {NULL}};
const Segment SGI128881[] = {{&S13823[80], 6}, {NULL}};
const Segment SGI128882[] = {{&S13824[44], 18}, {NULL}};
const Segment SGI128883[] = {{&S13824[44], 30}, {NULL}};
const Segment SGI128884[] = {{&S13824[30], 6}, {NULL}};
const Segment SGI128885[] = {{&S13824[123], 3}, {NULL}};
const Segment SGI128886[] = {{&S13824[88], 6}, {NULL}};
const Segment SGI128887[] = {{&S13825[44], 10}, {NULL}};
const Segment SGI128888[] = {{&S13825[44], 22}, {NULL}};
const Segment SGI128889[] = {{&S13825[30], 6}, {NULL}};
const Segment SGI128890[] = {{&S13825[115], 3}, {NULL}};
const Segment SGI128891[] = {{&S13825[80], 6}, {NULL}};
const Segment SGI128892[] = {{&S13826[44], 15}, {NULL}};
const Segment SGI128893[] = {{&S13826[44], 27}, {NULL}};
const Segment SGI128894[] = {{&S13826[30], 6}, {NULL}};
const Segment SGI128895[] = {{&S13826[120], 3}, {NULL}};
const Segment SGI128896[] = {{&S13826[85], 6}, {NULL}};
const Segment SGI128897[] = {{&S13827[44], 31}, {NULL}};
const Segment SGI128898[] = {{&S13827[44], 43}, {NULL}};
const Segment SGI128899[] = {{&S13827[30], 6}, {NULL}};
const Segment SGI128900[] = {{&S13827[136], 3}, {NULL}};
const Segment SGI128901[] = {{&S13827[101], 6}, {NULL}};
const Segment SGI128902[] = {{&S13829[44], 6}, {NULL}};
const Segment SGI128903[] = {{&S13829[44], 18}, {NULL}};
const Segment SGI128904[] = {{&S13829[30], 6}, {NULL}};
const Segment SGI128905[] = {{&S13829[111], 3}, {NULL}};
const Segment SGI128906[] = {{&S13829[76], 6}, {NULL}};
const Segment SGI128907[] = {{&S13833[44], 7}, {NULL}};
const Segment SGI128908[] = {{&S13833[44], 19}, {NULL}};
const Segment SGI128909[] = {{&S13833[30], 6}, {NULL}};
const Segment SGI128910[] = {{&S13833[112], 3}, {NULL}};
const Segment SGI128911[] = {{&S13833[77], 6}, {NULL}};
const Segment SGI128912[] = {{&S13835[44], 12}, {NULL}};
const Segment SGI128913[] = {{&S13835[44], 24}, {NULL}};
const Segment SGI128914[] = {{&S13835[30], 6}, {NULL}};
const Segment SGI128915[] = {{&S13835[117], 3}, {NULL}};
const Segment SGI128916[] = {{&S13835[82], 6}, {NULL}};
const Segment SGI128917[] = {{&S13836[44], 7}, {NULL}};
const Segment SGI128918[] = {{&S13836[44], 19}, {NULL}};
const Segment SGI128919[] = {{&S13836[30], 6}, {NULL}};
const Segment SGI128920[] = {{&S13836[112], 3}, {NULL}};
const Segment SGI128921[] = {{&S13836[77], 6}, {NULL}};
const Segment SGI128922[] = {{&S13837[38], 12}, {NULL}};
const Segment SGI128923[] = {{&S13837[31], 6}, {NULL}};
const Segment SGI128924[] = {{&S13838[44], 8}, {NULL}};
const Segment SGI128925[] = {{&S13838[44], 20}, {NULL}};
const Segment SGI128926[] = {{&S13838[30], 6}, {NULL}};
const Segment SGI128927[] = {{&S13838[113], 3}, {NULL}};
const Segment SGI128928[] = {{&S13838[78], 6}, {NULL}};
const Segment SGI128929[] = {{&S13841[44], 6}, {NULL}};
const Segment SGI128930[] = {{&S13841[44], 63}, {NULL}};
const Segment SGI128931[] = {{&S13841[30], 6}, {NULL}};
const Segment SGI128932[] = {{&S13841[156], 3}, {NULL}};
const Segment SGI128933[] = {{&S13841[121], 6}, {NULL}};
const Segment SGI128934[] = {{&S13843[44], 9}, {NULL}};
const Segment SGI128935[] = {{&S13843[44], 21}, {NULL}};
const Segment SGI128936[] = {{&S13843[30], 6}, {NULL}};
const Segment SGI128937[] = {{&S13843[114], 3}, {NULL}};
const Segment SGI128938[] = {{&S13843[79], 6}, {NULL}};
const Segment SGI128939[] = {{&S13844[44], 7}, {NULL}};
const Segment SGI128940[] = {{&S13844[44], 19}, {NULL}};
const Segment SGI128941[] = {{&S13844[30], 6}, {NULL}};
const Segment SGI128942[] = {{&S13844[112], 3}, {NULL}};
const Segment SGI128943[] = {{&S13844[77], 6}, {NULL}};
const Segment SGI128944[] = {{&S13846[38], 6}, {NULL}};
const Segment SGI128945[] = {{&S13846[31], 6}, {NULL}};
const Segment SGI128946[] = {{&S13848[44], 8}, {NULL}};
const Segment SGI128947[] = {{&S13848[44], 20}, {NULL}};
const Segment SGI128948[] = {{&S13848[30], 6}, {NULL}};
const Segment SGI128949[] = {{&S13848[113], 3}, {NULL}};
const Segment SGI128950[] = {{&S13848[78], 6}, {NULL}};
const Segment SGI128951[] = {{&S105810[44], 10}, {NULL}};
const Segment SGI128952[] = {{&S105810[44], 22}, {NULL}};
const Segment SGI128953[] = {{&S105810[30], 6}, {NULL}};
const Segment SGI128954[] = {{&S105810[31], 3}, {NULL}};
const Segment SGI128955[] = {{&S105810[80], 6}, {NULL}};
const Segment SGI128956[] = {{&S105810[133], 11}, {NULL}};
const Segment SGI128957[] = {{&S13849[44], 11}, {NULL}};
const Segment SGI128958[] = {{&S13849[44], 23}, {NULL}};
const Segment SGI128959[] = {{&S13849[30], 6}, {NULL}};
const Segment SGI128960[] = {{&S13849[116], 3}, {NULL}};
const Segment SGI128961[] = {{&S13849[81], 6}, {NULL}};
const Segment SGI128962[] = {{&S13850[44], 12}, {NULL}};
const Segment SGI128963[] = {{&S13850[44], 24}, {NULL}};
const Segment SGI128964[] = {{&S13850[30], 6}, {NULL}};
const Segment SGI128965[] = {{&S13850[117], 3}, {NULL}};
const Segment SGI128966[] = {{&S13850[82], 6}, {NULL}};
const Segment SGI128967[] = {{&S13851[44], 9}, {NULL}};
const Segment SGI128968[] = {{&S13851[44], 21}, {NULL}};
const Segment SGI128969[] = {{&S13851[30], 6}, {NULL}};
const Segment SGI128970[] = {{&S13851[114], 3}, {NULL}};
const Segment SGI128971[] = {{&S13851[79], 6}, {NULL}};
const Segment SGI128972[] = {{&S13853[44], 15}, {NULL}};
const Segment SGI128973[] = {{&S13853[44], 27}, {NULL}};
const Segment SGI128974[] = {{&S13853[30], 6}, {NULL}};
const Segment SGI128975[] = {{&S13853[120], 3}, {NULL}};
const Segment SGI128976[] = {{&S13853[85], 6}, {NULL}};
const Segment SGI128977[] = {{&S13854[44], 12}, {NULL}};
const Segment SGI128978[] = {{&S13854[44], 24}, {NULL}};
const Segment SGI128979[] = {{&S13854[30], 6}, {NULL}};
const Segment SGI128980[] = {{&S13854[117], 3}, {NULL}};
const Segment SGI128981[] = {{&S13854[82], 6}, {NULL}};
const Segment SGI128982[] = {{&S13856[44], 25}, {NULL}};
const Segment SGI128983[] = {{&S13856[44], 37}, {NULL}};
const Segment SGI128984[] = {{&S13856[30], 6}, {NULL}};
const Segment SGI128985[] = {{&S13856[130], 3}, {NULL}};
const Segment SGI128986[] = {{&S13856[95], 6}, {NULL}};
const Segment SGI128987[] = {{&S13857[44], 14}, {NULL}};
const Segment SGI128988[] = {{&S13857[44], 26}, {NULL}};
const Segment SGI128989[] = {{&S13857[30], 6}, {NULL}};
const Segment SGI128990[] = {{&S13857[119], 3}, {NULL}};
const Segment SGI128991[] = {{&S13857[84], 6}, {NULL}};
const Segment SGI128992[] = {{&S13859[44], 8}, {NULL}};
const Segment SGI128993[] = {{&S13859[44], 20}, {NULL}};
const Segment SGI128994[] = {{&S13859[30], 6}, {NULL}};
const Segment SGI128995[] = {{&S13859[113], 3}, {NULL}};
const Segment SGI128996[] = {{&S13859[78], 6}, {NULL}};
const Segment SGI128997[] = {{&S13860[44], 9}, {NULL}};
const Segment SGI128998[] = {{&S13860[44], 30}, {NULL}};
const Segment SGI128999[] = {{&S13860[30], 6}, {NULL}};
const Segment SGI129000[] = {{&S13860[123], 3}, {NULL}};
const Segment SGI129001[] = {{&S13860[88], 6}, {NULL}};
const Segment SGI129002[] = {{&S13861[44], 10}, {NULL}};
const Segment SGI129003[] = {{&S13861[44], 22}, {NULL}};
const Segment SGI129004[] = {{&S13861[30], 6}, {NULL}};
const Segment SGI129005[] = {{&S13861[115], 3}, {NULL}};
const Segment SGI129006[] = {{&S13861[80], 6}, {NULL}};
const Segment SGI129007[] = {{&S13862[44], 10}, {NULL}};
const Segment SGI129008[] = {{&S13862[44], 22}, {NULL}};
const Segment SGI129009[] = {{&S13862[30], 6}, {NULL}};
const Segment SGI129010[] = {{&S13862[115], 3}, {NULL}};
const Segment SGI129011[] = {{&S13862[80], 6}, {NULL}};
const Segment SGI129012[] = {{&S13863[44], 12}, {NULL}};
const Segment SGI129013[] = {{&S13863[44], 24}, {NULL}};
const Segment SGI129014[] = {{&S13863[30], 6}, {NULL}};
const Segment SGI129015[] = {{&S13863[117], 3}, {NULL}};
const Segment SGI129016[] = {{&S13863[82], 6}, {NULL}};
const Segment SGI129017[] = {{&S13866[44], 7}, {NULL}};
const Segment SGI129018[] = {{&S13866[44], 18}, {NULL}};
const Segment SGI129019[] = {{&S13866[30], 6}, {NULL}};
const Segment SGI129020[] = {{&S13866[111], 3}, {NULL}};
const Segment SGI129021[] = {{&S13866[76], 6}, {NULL}};
const Segment SGI129022[] = {{&S13867[44], 12}, {NULL}};
const Segment SGI129023[] = {{&S13867[44], 24}, {NULL}};
const Segment SGI129024[] = {{&S13867[30], 6}, {NULL}};
const Segment SGI129025[] = {{&S13867[117], 3}, {NULL}};
const Segment SGI129026[] = {{&S13867[82], 6}, {NULL}};
const Segment SGI129027[] = {{&S105843[58], 10}, {NULL}};
const Segment SGI129028[] = {{&S105843[58], 22}, {NULL}};
const Segment SGI129029[] = {{&S105843[44], 6}, {NULL}};
const Segment SGI129030[] = {{&S105843[45], 3}, {NULL}};
const Segment SGI129031[] = {{&S105843[94], 6}, {NULL}};
const Segment SGI129032[] = {{&S13868[44], 9}, {NULL}};
const Segment SGI129033[] = {{&S105843[0], 14}, {NULL}};
const Segment SGI129034[] = {{&S13868[44], 21}, {NULL}};
const Segment SGI129035[] = {{&S13868[30], 6}, {NULL}};
const Segment SGI129036[] = {{&S13868[114], 3}, {NULL}};
const Segment SGI129037[] = {{&S13868[79], 6}, {NULL}};
const Segment SGI129038[] = {{&S13869[44], 10}, {NULL}};
const Segment SGI129039[] = {{&S13869[44], 22}, {NULL}};
const Segment SGI129040[] = {{&S13869[30], 6}, {NULL}};
const Segment SGI129041[] = {{&S13869[115], 3}, {NULL}};
const Segment SGI129042[] = {{&S13869[80], 6}, {NULL}};
const Segment SGI129043[] = {{&S13871[44], 8}, {NULL}};
const Segment SGI129044[] = {{&S13871[44], 20}, {NULL}};
const Segment SGI129045[] = {{&S13871[30], 6}, {NULL}};
const Segment SGI129046[] = {{&S13871[113], 3}, {NULL}};
const Segment SGI129047[] = {{&S13871[78], 6}, {NULL}};
const Segment SGI129048[] = {{&S13872[44], 10}, {NULL}};
const Segment SGI129049[] = {{&S13872[44], 22}, {NULL}};
const Segment SGI129050[] = {{&S13872[30], 6}, {NULL}};
const Segment SGI129051[] = {{&S13872[115], 3}, {NULL}};
const Segment SGI129052[] = {{&S13872[80], 6}, {NULL}};
const Segment SGI129053[] = {{&S13873[44], 23}, {NULL}};
const Segment SGI129054[] = {{&S13873[44], 35}, {NULL}};
const Segment SGI129055[] = {{&S13873[30], 6}, {NULL}};
const Segment SGI129056[] = {{&S13873[128], 3}, {NULL}};
const Segment SGI129057[] = {{&S13873[93], 6}, {NULL}};
const Segment SGI129058[] = {{&S13874[44], 10}, {NULL}};
const Segment SGI129059[] = {{&S13874[44], 22}, {NULL}};
const Segment SGI129060[] = {{&S13874[30], 6}, {NULL}};
const Segment SGI129061[] = {{&S13874[115], 3}, {NULL}};
const Segment SGI129062[] = {{&S13874[80], 6}, {NULL}};
const Segment SGI129063[] = {{&S13876[44], 10}, {NULL}};
const Segment SGI129064[] = {{&S13876[44], 22}, {NULL}};
const Segment SGI129065[] = {{&S13876[30], 6}, {NULL}};
const Segment SGI129066[] = {{&S13876[115], 3}, {NULL}};
const Segment SGI129067[] = {{&S13876[80], 6}, {NULL}};
const Segment SGI129068[] = {{&S13877[44], 7}, {NULL}};
const Segment SGI129069[] = {{&S13877[44], 14}, {NULL}};
const Segment SGI129070[] = {{&S13877[30], 6}, {NULL}};
const Segment SGI129071[] = {{&S13877[107], 3}, {NULL}};
const Segment SGI129072[] = {{&S13877[72], 6}, {NULL}};
const Segment SGI129073[] = {{&S13878[44], 6}, {NULL}};
const Segment SGI129074[] = {{&S13878[44], 18}, {NULL}};
const Segment SGI129075[] = {{&S13878[30], 6}, {NULL}};
const Segment SGI129076[] = {{&S13878[111], 3}, {NULL}};
const Segment SGI129077[] = {{&S13878[76], 6}, {NULL}};
const Segment SGI129078[] = {{&S13880[44], 7}, {NULL}};
const Segment SGI129079[] = {{&S13880[44], 19}, {NULL}};
const Segment SGI129080[] = {{&S13880[30], 6}, {NULL}};
const Segment SGI129081[] = {{&S13880[112], 3}, {NULL}};
const Segment SGI129082[] = {{&S13880[77], 6}, {NULL}};
const Segment SGI129083[] = {{&S13881[44], 7}, {NULL}};
const Segment SGI129084[] = {{&S13881[44], 19}, {NULL}};
const Segment SGI129085[] = {{&S13881[30], 6}, {NULL}};
const Segment SGI129086[] = {{&S13881[112], 3}, {NULL}};
const Segment SGI129087[] = {{&S13881[77], 6}, {NULL}};
const Segment SGI129088[] = {{&S13882[44], 9}, {NULL}};
const Segment SGI129089[] = {{&S13882[44], 21}, {NULL}};
const Segment SGI129090[] = {{&S13882[30], 6}, {NULL}};
const Segment SGI129091[] = {{&S13882[114], 3}, {NULL}};
const Segment SGI129092[] = {{&S13882[79], 6}, {NULL}};
const Segment SGI129093[] = {{&S13883[44], 5}, {NULL}};
const Segment SGI129094[] = {{&S13883[44], 17}, {NULL}};
const Segment SGI129095[] = {{&S13883[30], 6}, {NULL}};
const Segment SGI129096[] = {{&S13883[110], 3}, {NULL}};
const Segment SGI129097[] = {{&S13883[75], 6}, {NULL}};
const Segment SGI129098[] = {{&S13884[44], 40}, {NULL}};
const Segment SGI129099[] = {{&S13884[30], 6}, {NULL}};
const Segment SGI129100[] = {{&S13884[147], 3}, {NULL}};
const Segment SGI129101[] = {{&S13884[112], 6}, {NULL}};
const Segment SGI129102[] = {{&S13885[44], 9}, {NULL}};
const Segment SGI129103[] = {{&S13885[44], 21}, {NULL}};
const Segment SGI129104[] = {{&S13885[30], 6}, {NULL}};
const Segment SGI129105[] = {{&S13885[114], 3}, {NULL}};
const Segment SGI129106[] = {{&S13885[79], 6}, {NULL}};
const Segment SGI129107[] = {{&S13886[44], 9}, {NULL}};
const Segment SGI129108[] = {{&S13886[44], 21}, {NULL}};
const Segment SGI129109[] = {{&S13886[30], 6}, {NULL}};
const Segment SGI129110[] = {{&S13886[114], 3}, {NULL}};
const Segment SGI129111[] = {{&S13886[79], 6}, {NULL}};
const Segment SGI129112[] = {{&S13887[44], 5}, {NULL}};
const Segment SGI129113[] = {{&S13887[44], 17}, {NULL}};
const Segment SGI129114[] = {{&S13887[30], 6}, {NULL}};
const Segment SGI129115[] = {{&S13887[110], 3}, {NULL}};
const Segment SGI129116[] = {{&S13887[75], 6}, {NULL}};
const Segment SGI129117[] = {{&S13888[44], 11}, {NULL}};
const Segment SGI129118[] = {{&S13888[44], 23}, {NULL}};
const Segment SGI129119[] = {{&S13888[30], 6}, {NULL}};
const Segment SGI129120[] = {{&S13888[116], 3}, {NULL}};
const Segment SGI129121[] = {{&S13888[81], 6}, {NULL}};
const Segment SGI129122[] = {{&S13889[44], 12}, {NULL}};
const Segment SGI129123[] = {{&S13889[44], 19}, {NULL}};
const Segment SGI129124[] = {{&S13889[30], 6}, {NULL}};
const Segment SGI129125[] = {{&S13889[112], 3}, {NULL}};
const Segment SGI129126[] = {{&S13889[77], 6}, {NULL}};
const Segment SGI129127[] = {{&S13890[44], 6}, {NULL}};
const Segment SGI129128[] = {{&S13890[44], 18}, {NULL}};
const Segment SGI129129[] = {{&S13890[30], 6}, {NULL}};
const Segment SGI129130[] = {{&S13890[111], 3}, {NULL}};
const Segment SGI129131[] = {{&S13890[76], 6}, {NULL}};
const Segment SGI129132[] = {{&S13892[44], 7}, {NULL}};
const Segment SGI129133[] = {{&S13892[44], 19}, {NULL}};
const Segment SGI129134[] = {{&S13892[30], 6}, {NULL}};
const Segment SGI129135[] = {{&S13892[112], 3}, {NULL}};
const Segment SGI129136[] = {{&S13892[77], 6}, {NULL}};
const Segment SGI129137[] = {{&S13893[44], 10}, {NULL}};
const Segment SGI129138[] = {{&S13893[44], 22}, {NULL}};
const Segment SGI129139[] = {{&S13893[30], 6}, {NULL}};
const Segment SGI129140[] = {{&S13893[115], 3}, {NULL}};
const Segment SGI129141[] = {{&S13893[80], 6}, {NULL}};
const Segment SGI129142[] = {{&S13894[44], 11}, {NULL}};
const Segment SGI129143[] = {{&S13894[44], 23}, {NULL}};
const Segment SGI129144[] = {{&S13894[30], 6}, {NULL}};
const Segment SGI129145[] = {{&S13894[116], 3}, {NULL}};
const Segment SGI129146[] = {{&S13894[81], 6}, {NULL}};
const Segment SGI129147[] = {{&S13895[44], 5}, {NULL}};
const Segment SGI129148[] = {{&S13895[44], 17}, {NULL}};
const Segment SGI129149[] = {{&S13895[30], 6}, {NULL}};
const Segment SGI129150[] = {{&S13895[110], 3}, {NULL}};
const Segment SGI129151[] = {{&S13895[75], 6}, {NULL}};
const Segment SGI129152[] = {{&S13897[44], 11}, {NULL}};
const Segment SGI129153[] = {{&S13897[44], 23}, {NULL}};
const Segment SGI129154[] = {{&S13897[30], 6}, {NULL}};
const Segment SGI129155[] = {{&S13897[116], 3}, {NULL}};
const Segment SGI129156[] = {{&S13897[81], 6}, {NULL}};
const Segment SGI129157[] = {{&S13898[44], 19}, {NULL}};
const Segment SGI129158[] = {{&S13898[44], 31}, {NULL}};
const Segment SGI129159[] = {{&S105897[44], 10}, {NULL}};
const Segment SGI129160[] = {{&S13898[30], 6}, {NULL}};
const Segment SGI129161[] = {{&S105897[44], 22}, {NULL}};
const Segment SGI129162[] = {{&S105897[30], 6}, {NULL}};
const Segment SGI129163[] = {{&S13898[124], 3}, {NULL}};
const Segment SGI129164[] = {{&S13898[89], 6}, {NULL}};
const Segment SGI129165[] = {{&S105897[31], 3}, {NULL}};
const Segment SGI129166[] = {{&S105897[80], 6}, {NULL}};
const Segment SGI129167[] = {{&S105897[157], 3}, {NULL}};
const Segment SGI129168[] = {{&S13899[44], 8}, {NULL}};
const Segment SGI129169[] = {{&S13899[44], 20}, {NULL}};
const Segment SGI129170[] = {{&S13899[30], 6}, {NULL}};
const Segment SGI129171[] = {{&S13899[113], 3}, {NULL}};
const Segment SGI129172[] = {{&S13899[78], 6}, {NULL}};
const Segment SGI129173[] = {{&S13901[44], 10}, {NULL}};
const Segment SGI129174[] = {{&S13901[44], 22}, {NULL}};
const Segment SGI129175[] = {{&S23105[44], 10}, {NULL}};
const Segment SGI129176[] = {{&S13901[30], 6}, {NULL}};
const Segment SGI129177[] = {{&S23105[44], 22}, {NULL}};
const Segment SGI129178[] = {{&S23105[30], 6}, {NULL}};
const Segment SGI129179[] = {{&S13901[115], 3}, {NULL}};
const Segment SGI129180[] = {{&S13901[80], 6}, {NULL}};
const Segment SGI129181[] = {{&S23105[31], 3}, {NULL}};
const Segment SGI129182[] = {{&S23105[80], 6}, {NULL}};
const Segment SGI129183[] = {{&S23106[44], 10}, {NULL}};
const Segment SGI129184[] = {{&S23106[44], 35}, {NULL}};
const Segment SGI129185[] = {{&S23106[30], 6}, {NULL}};
const Segment SGI129186[] = {{&S23106[31], 3}, {NULL}};
const Segment SGI129187[] = {{&S23106[93], 6}, {NULL}};
const Segment SGI129188[] = {{&S13904[44], 18}, {NULL}};
const Segment SGI129189[] = {{&S13904[44], 30}, {NULL}};
const Segment SGI129190[] = {{&S13904[30], 6}, {NULL}};
const Segment SGI129191[] = {{&S13904[123], 3}, {NULL}};
const Segment SGI129192[] = {{&S13904[88], 6}, {NULL}};
const Segment SGI129193[] = {{&S23108[44], 10}, {NULL}};
const Segment SGI129194[] = {{&S23108[44], 22}, {NULL}};
const Segment SGI129195[] = {{&S23108[30], 6}, {NULL}};
const Segment SGI129196[] = {{&S23108[31], 3}, {NULL}};
const Segment SGI129197[] = {{&S13905[44], 5}, {NULL}};
const Segment SGI129198[] = {{&S23108[80], 6}, {NULL}};
const Segment SGI129199[] = {{&S13905[44], 17}, {NULL}};
const Segment SGI129200[] = {{&S13905[30], 6}, {NULL}};
const Segment SGI129201[] = {{&S13905[110], 3}, {NULL}};
const Segment SGI129202[] = {{&S13905[75], 6}, {NULL}};
const Segment SGI129203[] = {{&S13906[44], 4}, {NULL}};
const Segment SGI129204[] = {{&S23110[38], 10}, {NULL}};
const Segment SGI129205[] = {{&S13906[44], 16}, {NULL}};
const Segment SGI129206[] = {{&S23110[31], 6}, {NULL}};
const Segment SGI129207[] = {{&S13906[30], 6}, {NULL}};
const Segment SGI129208[] = {{&S13906[109], 3}, {NULL}};
const Segment SGI129209[] = {{&S13906[74], 6}, {NULL}};
const Segment SGI129210[] = {{&S23111[44], 10}, {NULL}};
const Segment SGI129211[] = {{&S23111[44], 22}, {NULL}};
const Segment SGI129212[] = {{&S23111[30], 6}, {NULL}};
const Segment SGI129213[] = {{&S13907[44], 9}, {NULL}};
const Segment SGI129214[] = {{&S23111[31], 3}, {NULL}};
const Segment SGI129215[] = {{&S13907[44], 22}, {NULL}};
const Segment SGI129216[] = {{&S13907[30], 6}, {NULL}};
const Segment SGI129217[] = {{&S23111[80], 6}, {NULL}};
const Segment SGI129218[] = {{&S13907[115], 3}, {NULL}};
const Segment SGI129219[] = {{&S13907[80], 6}, {NULL}};
const Segment SGI129220[] = {{&S13908[44], 8}, {NULL}};
const Segment SGI129221[] = {{&S13908[44], 20}, {NULL}};
const Segment SGI129222[] = {{&S13908[30], 6}, {NULL}};
const Segment SGI129223[] = {{&S23113[44], 10}, {NULL}};
const Segment SGI129224[] = {{&S13908[112], 3}, {NULL}};
const Segment SGI129225[] = {{&S23113[44], 30}, {NULL}};
const Segment SGI129226[] = {{&S23113[30], 6}, {NULL}};
const Segment SGI129227[] = {{&S13908[78], 5}, {NULL}};
const Segment SGI129228[] = {{&S23113[31], 3}, {NULL}};
const Segment SGI129229[] = {{&S23113[88], 6}, {NULL}};
const Segment SGI129230[] = {{&S23114[44], 10}, {NULL}};
const Segment SGI129231[] = {{&S23114[44], 30}, {NULL}};
const Segment SGI129232[] = {{&S23114[30], 6}, {NULL}};
const Segment SGI129233[] = {{&S23114[31], 3}, {NULL}};
const Segment SGI129234[] = {{&S13910[44], 20}, {NULL}};
const Segment SGI129235[] = {{&S13910[44], 32}, {NULL}};
const Segment SGI129236[] = {{&S23114[88], 6}, {NULL}};
const Segment SGI129237[] = {{&S13910[30], 6}, {NULL}};
const Segment SGI129238[] = {{&S13910[125], 3}, {NULL}};
const Segment SGI129239[] = {{&S13910[90], 6}, {NULL}};
const Segment SGI129240[] = {{&S23115[44], 10}, {NULL}};
const Segment SGI129241[] = {{&S23115[44], 22}, {NULL}};
const Segment SGI129242[] = {{&S23115[30], 6}, {NULL}};
const Segment SGI129243[] = {{&S23115[31], 3}, {NULL}};
const Segment SGI129244[] = {{&S23115[80], 6}, {NULL}};
const Segment SGI129245[] = {{&S13912[44], 9}, {NULL}};
const Segment SGI129246[] = {{&S13912[44], 21}, {NULL}};
const Segment SGI129247[] = {{&S13912[30], 6}, {NULL}};
const Segment SGI129248[] = {{&S13912[114], 3}, {NULL}};
const Segment SGI129249[] = {{&S23116[44], 10}, {NULL}};
const Segment SGI129250[] = {{&S13912[79], 6}, {NULL}};
const Segment SGI129251[] = {{&S23116[44], 27}, {NULL}};
const Segment SGI129252[] = {{&S23116[30], 6}, {NULL}};
const Segment SGI129253[] = {{&S23116[31], 3}, {NULL}};
const Segment SGI129254[] = {{&S23116[85], 6}, {NULL}};
const Segment SGI129255[] = {{&S13913[44], 15}, {NULL}};
const Segment SGI129256[] = {{&S13913[44], 27}, {NULL}};
const Segment SGI129257[] = {{&S13913[30], 6}, {NULL}};
const Segment SGI129258[] = {{&S23117[44], 10}, {NULL}};
const Segment SGI129259[] = {{&S13913[120], 3}, {NULL}};
const Segment SGI129260[] = {{&S23117[44], 22}, {NULL}};
const Segment SGI129261[] = {{&S23117[30], 6}, {NULL}};
const Segment SGI129262[] = {{&S13913[85], 6}, {NULL}};
const Segment SGI129263[] = {{&S23117[31], 3}, {NULL}};
const Segment SGI129264[] = {{&S23117[80], 6}, {NULL}};
const Segment SGI129265[] = {{&S13914[44], 25}, {NULL}};
const Segment SGI129266[] = {{&S13914[44], 37}, {NULL}};
const Segment SGI129267[] = {{&S13914[30], 6}, {NULL}};
const Segment SGI129268[] = {{&S23118[44], 13}, {NULL}};
const Segment SGI129269[] = {{&S13914[130], 3}, {NULL}};
const Segment SGI129270[] = {{&S23118[44], 25}, {NULL}};
const Segment SGI129271[] = {{&S23118[30], 6}, {NULL}};
const Segment SGI129272[] = {{&S13914[95], 6}, {NULL}};
const Segment SGI129273[] = {{&S23118[31], 3}, {NULL}};
const Segment SGI129274[] = {{&S23118[83], 6}, {NULL}};
const Segment SGI129275[] = {{&S13915[44], 6}, {NULL}};
const Segment SGI129276[] = {{&S13915[44], 18}, {NULL}};
const Segment SGI129277[] = {{&S13915[30], 6}, {NULL}};
const Segment SGI129278[] = {{&S23119[38], 13}, {NULL}};
const Segment SGI129279[] = {{&S23119[31], 6}, {NULL}};
const Segment SGI129280[] = {{&S13915[111], 3}, {NULL}};
const Segment SGI129281[] = {{&S13915[76], 6}, {NULL}};
const Segment SGI129282[] = {{&S23120[44], 10}, {NULL}};
const Segment SGI129283[] = {{&S23120[44], 22}, {NULL}};
const Segment SGI129284[] = {{&S23120[30], 6}, {NULL}};
const Segment SGI129285[] = {{&S23120[80], 6}, {NULL}};
const Segment SGI129286[] = {{&S13917[44], 7}, {NULL}};
const Segment SGI129287[] = {{&S13917[44], 19}, {NULL}};
const Segment SGI129288[] = {{&S13917[30], 6}, {NULL}};
const Segment SGI129289[] = {{&S13917[112], 3}, {NULL}};
const Segment SGI129290[] = {{&S13917[77], 6}, {NULL}};
const Segment SGI129291[] = {{&S23121[44], 14}, {NULL}};
const Segment SGI129292[] = {{&S23121[44], 26}, {NULL}};
const Segment SGI129293[] = {{&S23121[30], 6}, {NULL}};
const Segment SGI129294[] = {{&S23121[31], 3}, {NULL}};
const Segment SGI129295[] = {{&S23121[84], 6}, {NULL}};
const Segment SGI129296[] = {{&S13918[44], 13}, {NULL}};
const Segment SGI129297[] = {{&S13918[44], 25}, {NULL}};
const Segment SGI129298[] = {{&S13918[30], 6}, {NULL}};
const Segment SGI129299[] = {{&S13918[118], 3}, {NULL}};
const Segment SGI129300[] = {{&S13918[83], 6}, {NULL}};
const Segment SGI129301[] = {{&S23122[42], 10}, {NULL}};
const Segment SGI129302[] = {{&S23122[30], 6}, {NULL}};
const Segment SGI129303[] = {{&S23122[31], 3}, {NULL}};
const Segment SGI129304[] = {{&S23122[78], 6}, {NULL}};
const Segment SGI129305[] = {{&S13919[44], 8}, {NULL}};
const Segment SGI129306[] = {{&S13919[44], 20}, {NULL}};
const Segment SGI129307[] = {{&S13919[30], 6}, {NULL}};
const Segment SGI129308[] = {{&S13919[113], 3}, {NULL}};
const Segment SGI129309[] = {{&S13919[78], 6}, {NULL}};
const Segment SGI129310[] = {{&S23123[44], 10}, {NULL}};
const Segment SGI129311[] = {{&S23123[44], 22}, {NULL}};
const Segment SGI129312[] = {{&S23123[30], 6}, {NULL}};
const Segment SGI129313[] = {{&S13920[44], 10}, {NULL}};
const Segment SGI129314[] = {{&S23123[31], 3}, {&S23123[158], 3}, {NULL}};
const Segment SGI129315[] = {{&S13920[44], 22}, {NULL}};
const Segment SGI129316[] = {{&S13920[30], 6}, {NULL}};
const Segment SGI129317[] = {{&S23123[80], 6}, {NULL}};
const Segment SGI129318[] = {{&S13920[115], 3}, {NULL}};
const Segment SGI129319[] = {{&S13920[80], 6}, {NULL}};
const Segment SGI129320[] = {{&S23124[38], 14}, {NULL}};
const Segment SGI129321[] = {{&S23124[31], 6}, {NULL}};
const Segment SGI129322[] = {{&S13922[44], 7}, {NULL}};
const Segment SGI129323[] = {{&S13922[44], 19}, {NULL}};
const Segment SGI129324[] = {{&S13922[30], 6}, {NULL}};
const Segment SGI129325[] = {{&S23125[44], 10}, {NULL}};
const Segment SGI129326[] = {{&S23125[44], 22}, {NULL}};
const Segment SGI129327[] = {{&S23125[30], 6}, {NULL}};
const Segment SGI129328[] = {{&S13922[111], 3}, {NULL}};
const Segment SGI129329[] = {{&S13922[77], 5}, {NULL}};
const Segment SGI129330[] = {{&S23125[83], 6}, {NULL}};
const Segment SGI129331[] = {{&S23126[44], 10}, {NULL}};
const Segment SGI129332[] = {{&S23126[44], 22}, {NULL}};
const Segment SGI129333[] = {{&S23126[30], 6}, {NULL}};
const Segment SGI129334[] = {{&S23126[31], 3}, {NULL}};
const Segment SGI129335[] = {{&S23126[80], 6}, {NULL}};
const Segment SGI129336[] = {{&S13924[44], 9}, {NULL}};
const Segment SGI129337[] = {{&S13924[44], 21}, {NULL}};
const Segment SGI129338[] = {{&S13924[30], 6}, {NULL}};
const Segment SGI129339[] = {{&S13924[114], 3}, {NULL}};
const Segment SGI129340[] = {{&S13924[79], 6}, {NULL}};
const Segment SGI129341[] = {{&S13925[44], 11}, {NULL}};
const Segment SGI129342[] = {{&S13925[44], 23}, {NULL}};
const Segment SGI129343[] = {{&S13925[30], 6}, {NULL}};
const Segment SGI129344[] = {{&S13925[116], 3}, {NULL}};
const Segment SGI129345[] = {{&S13925[81], 6}, {NULL}};
const Segment SGI129346[] = {{&S13927[44], 10}, {NULL}};
const Segment SGI129347[] = {{&S13927[44], 17}, {NULL}};
const Segment SGI129348[] = {{&S13927[30], 6}, {NULL}};
const Segment SGI129349[] = {{&S13927[110], 3}, {NULL}};
const Segment SGI129350[] = {{&S13927[75], 6}, {NULL}};
const Segment SGI129351[] = {{&S13928[44], 7}, {NULL}};
const Segment SGI129352[] = {{&S13928[44], 19}, {NULL}};
const Segment SGI129353[] = {{&S13928[30], 6}, {NULL}};
const Segment SGI129354[] = {{&S13928[112], 3}, {NULL}};
const Segment SGI129355[] = {{&S13928[77], 6}, {NULL}};
const Segment SGI129356[] = {{&S13929[44], 5}, {NULL}};
const Segment SGI129357[] = {{&S13929[44], 17}, {NULL}};
const Segment SGI129358[] = {{&S13929[30], 6}, {NULL}};
const Segment SGI129359[] = {{&S13929[110], 3}, {NULL}};
const Segment SGI129360[] = {{&S13929[75], 6}, {NULL}};
const Segment SGI129361[] = {{&S13930[44], 19}, {NULL}};
const Segment SGI129362[] = {{&S13930[44], 31}, {NULL}};
const Segment SGI129363[] = {{&S13930[30], 6}, {NULL}};
const Segment SGI129364[] = {{&S13930[124], 3}, {NULL}};
const Segment SGI129365[] = {{&S13930[89], 6}, {NULL}};
const Segment SGI129366[] = {{&S13931[44], 5}, {NULL}};
const Segment SGI129367[] = {{&S13931[44], 17}, {NULL}};
const Segment SGI129368[] = {{&S13931[30], 6}, {NULL}};
const Segment SGI129369[] = {{&S13931[110], 3}, {NULL}};
const Segment SGI129370[] = {{&S13931[75], 6}, {NULL}};
const Segment SGI129371[] = {{&S13932[44], 15}, {NULL}};
const Segment SGI129372[] = {{&S13932[44], 27}, {NULL}};
const Segment SGI129373[] = {{&S13932[30], 6}, {NULL}};
const Segment SGI129374[] = {{&S13932[120], 3}, {NULL}};
const Segment SGI129375[] = {{&S13932[85], 6}, {NULL}};
const Segment SGI129376[] = {{&S13935[44], 10}, {NULL}};
const Segment SGI129377[] = {{&S13935[44], 22}, {NULL}};
const Segment SGI129378[] = {{&S13935[30], 6}, {NULL}};
const Segment SGI129379[] = {{&S13935[115], 3}, {NULL}};
const Segment SGI129380[] = {{&S13935[80], 6}, {NULL}};
const Segment SGI129381[] = {{&S13937[44], 17}, {NULL}};
const Segment SGI129382[] = {{&S13937[44], 24}, {NULL}};
const Segment SGI129383[] = {{&S13937[30], 6}, {NULL}};
const Segment SGI129384[] = {{&S13937[117], 3}, {NULL}};
const Segment SGI129385[] = {{&S13937[82], 6}, {NULL}};
const Segment SGI129386[] = {{&S13938[44], 11}, {NULL}};
const Segment SGI129387[] = {{&S13938[44], 23}, {NULL}};
const Segment SGI129388[] = {{&S13938[30], 6}, {NULL}};
const Segment SGI129389[] = {{&S13938[116], 3}, {NULL}};
const Segment SGI129390[] = {{&S13938[81], 6}, {NULL}};
const Segment SGI129391[] = {{&S13940[44], 5}, {NULL}};
const Segment SGI129392[] = {{&S13940[44], 17}, {NULL}};
const Segment SGI129393[] = {{&S13940[30], 6}, {NULL}};
const Segment SGI129394[] = {{&S13940[110], 3}, {NULL}};
const Segment SGI129395[] = {{&S13940[75], 6}, {NULL}};
const Segment SGI129396[] = {{&S13941[44], 6}, {NULL}};
const Segment SGI129397[] = {{&S13941[44], 18}, {NULL}};
const Segment SGI129398[] = {{&S13941[30], 6}, {NULL}};
const Segment SGI129399[] = {{&S13941[111], 3}, {NULL}};
const Segment SGI129400[] = {{&S13941[76], 6}, {NULL}};
const Segment SGI129401[] = {{&S13942[44], 18}, {NULL}};
const Segment SGI129402[] = {{&S13942[44], 30}, {NULL}};
const Segment SGI129403[] = {{&S13942[30], 6}, {NULL}};
const Segment SGI129404[] = {{&S13942[123], 3}, {NULL}};
const Segment SGI129405[] = {{&S13942[88], 6}, {NULL}};
const Segment SGI129406[] = {{&S13944[54], 10}, {NULL}};
const Segment SGI129407[] = {{&S13944[47], 6}, {NULL}};
const Segment SGI129408[] = {{&S13945[44], 11}, {NULL}};
const Segment SGI129409[] = {{&S13945[44], 23}, {NULL}};
const Segment SGI129410[] = {{&S13945[30], 6}, {NULL}};
const Segment SGI129411[] = {{&S13945[116], 3}, {NULL}};
const Segment SGI129412[] = {{&S13945[81], 6}, {NULL}};
const Segment SGI129413[] = {{&S13946[44], 11}, {NULL}};
const Segment SGI129414[] = {{&S13946[44], 23}, {NULL}};
const Segment SGI129415[] = {{&S13946[30], 6}, {NULL}};
const Segment SGI129416[] = {{&S13946[116], 3}, {NULL}};
const Segment SGI129417[] = {{&S13946[81], 6}, {NULL}};
const Segment SGI129418[] = {{&S13947[44], 6}, {NULL}};
const Segment SGI129419[] = {{&S13947[44], 18}, {NULL}};
const Segment SGI129420[] = {{&S13947[30], 6}, {NULL}};
const Segment SGI129421[] = {{&S13947[111], 3}, {NULL}};
const Segment SGI129422[] = {{&S13947[76], 6}, {NULL}};
const Segment SGI129423[] = {{&S105978[34], 17}, {NULL}};
const Segment SGI129424[] = {{&S105978[27], 6}, {NULL}};
const Segment SGI129425[] = {{&S105978[112], 9}, {NULL}};
const Segment SGI129426[] = {{&S105978[85], 6}, {NULL}};
const Segment SGI129427[] = {{&S105978[146], 8}, {NULL}};
const Segment SGI129428[] = {{&S13948[44], 7}, {NULL}};
const Segment SGI129429[] = {{&S13948[44], 19}, {NULL}};
const Segment SGI129430[] = {{&S13948[30], 6}, {NULL}};
const Segment SGI129431[] = {{&S13948[112], 3}, {NULL}};
const Segment SGI129432[] = {{&S13948[77], 6}, {NULL}};
const Segment SGI129433[] = {{&S13950[44], 6}, {NULL}};
const Segment SGI129434[] = {{&S13950[44], 18}, {NULL}};
const Segment SGI129435[] = {{&S13950[30], 6}, {NULL}};
const Segment SGI129436[] = {{&S13950[110], 3}, {NULL}};
const Segment SGI129437[] = {{&S13950[76], 5}, {NULL}};
const Segment SGI129438[] = {{&S13951[44], 17}, {NULL}};
const Segment SGI129439[] = {{&S13951[44], 38}, {NULL}};
const Segment SGI129440[] = {{&S13951[30], 6}, {NULL}};
const Segment SGI129441[] = {{&S13951[131], 3}, {NULL}};
const Segment SGI129442[] = {{&S13951[96], 6}, {NULL}};
const Segment SGI129443[] = {{&S13952[44], 13}, {NULL}};
const Segment SGI129444[] = {{&S13952[44], 25}, {NULL}};
const Segment SGI129445[] = {{&S13952[30], 6}, {NULL}};
const Segment SGI129446[] = {{&S13952[118], 3}, {NULL}};
const Segment SGI129447[] = {{&S13952[83], 6}, {NULL}};
const Segment SGI129448[] = {{&S13953[44], 6}, {NULL}};
const Segment SGI129449[] = {{&S13953[44], 18}, {NULL}};
const Segment SGI129450[] = {{&S13953[30], 6}, {NULL}};
const Segment SGI129451[] = {{&S13953[110], 3}, {NULL}};
const Segment SGI129452[] = {{&S13953[76], 5}, {NULL}};
const Segment SGI129453[] = {{&S13959[44], 16}, {NULL}};
const Segment SGI129454[] = {{&S13959[44], 28}, {NULL}};
const Segment SGI129455[] = {{&S13959[30], 6}, {NULL}};
const Segment SGI129456[] = {{&S13959[121], 3}, {NULL}};
const Segment SGI129457[] = {{&S13959[86], 6}, {NULL}};
const Segment SGI129458[] = {{&S13960[44], 15}, {NULL}};
const Segment SGI129459[] = {{&S13960[44], 27}, {NULL}};
const Segment SGI129460[] = {{&S13960[30], 6}, {NULL}};
const Segment SGI129461[] = {{&S13960[120], 3}, {NULL}};
const Segment SGI129462[] = {{&S13960[85], 6}, {NULL}};
const Segment SGI129463[] = {{&S13963[44], 6}, {NULL}};
const Segment SGI129464[] = {{&S13963[44], 27}, {NULL}};
const Segment SGI129465[] = {{&S13963[30], 6}, {NULL}};
const Segment SGI129466[] = {{&S13963[119], 3}, {NULL}};
const Segment SGI129467[] = {{&S13963[85], 5}, {NULL}};
const Segment SGI129468[] = {{&S13966[44], 5}, {NULL}};
const Segment SGI129469[] = {{&S13966[44], 17}, {NULL}};
const Segment SGI129470[] = {{&S13966[30], 6}, {NULL}};
const Segment SGI129471[] = {{&S13966[110], 3}, {NULL}};
const Segment SGI129472[] = {{&S13966[75], 6}, {NULL}};
const Segment SGI129473[] = {{&S13971[44], 13}, {NULL}};
const Segment SGI129474[] = {{&S13971[44], 25}, {NULL}};
const Segment SGI129475[] = {{&S13971[30], 6}, {NULL}};
const Segment SGI129476[] = {{&S13971[118], 3}, {NULL}};
const Segment SGI129477[] = {{&S13971[83], 6}, {NULL}};
const Segment SGI129478[] = {{&S13972[44], 11}, {NULL}};
const Segment SGI129479[] = {{&S13972[44], 23}, {NULL}};
const Segment SGI129480[] = {{&S13972[30], 6}, {NULL}};
const Segment SGI129481[] = {{&S13972[116], 3}, {NULL}};
const Segment SGI129482[] = {{&S13972[81], 6}, {NULL}};
const Segment SGI129483[] = {{&S13974[44], 5}, {NULL}};
const Segment SGI129484[] = {{&S13974[44], 17}, {NULL}};
const Segment SGI129485[] = {{&S13974[30], 6}, {NULL}};
const Segment SGI129486[] = {{&S13974[110], 3}, {NULL}};
const Segment SGI129487[] = {{&S13974[75], 6}, {NULL}};
const Segment SGI129488[] = {{&S13977[44], 12}, {NULL}};
const Segment SGI129489[] = {{&S13977[44], 28}, {NULL}};
const Segment SGI129490[] = {{&S13977[30], 6}, {NULL}};
const Segment SGI129491[] = {{&S13977[121], 3}, {NULL}};
const Segment SGI129492[] = {{&S13977[86], 6}, {NULL}};
const Segment SGI129493[] = {{&S13978[44], 10}, {NULL}};
const Segment SGI129494[] = {{&S13978[44], 22}, {NULL}};
const Segment SGI129495[] = {{&S13978[30], 6}, {NULL}};
const Segment SGI129496[] = {{&S13978[115], 3}, {NULL}};
const Segment SGI129497[] = {{&S13978[80], 6}, {NULL}};
const Segment SGI129498[] = {{&S13981[44], 7}, {NULL}};
const Segment SGI129499[] = {{&S13981[44], 19}, {NULL}};
const Segment SGI129500[] = {{&S13981[30], 6}, {NULL}};
const Segment SGI129501[] = {{&S13981[112], 3}, {NULL}};
const Segment SGI129502[] = {{&S13981[77], 6}, {NULL}};
const Segment SGI129503[] = {{&S13982[44], 13}, {NULL}};
const Segment SGI129504[] = {{&S13982[44], 25}, {NULL}};
const Segment SGI129505[] = {{&S13982[30], 6}, {NULL}};
const Segment SGI129506[] = {{&S13982[118], 3}, {NULL}};
const Segment SGI129507[] = {{&S13982[83], 6}, {NULL}};
const Segment SGI129508[] = {{&S13983[44], 12}, {NULL}};
const Segment SGI129509[] = {{&S13983[44], 24}, {NULL}};
const Segment SGI129510[] = {{&S13983[30], 6}, {NULL}};
const Segment SGI129511[] = {{&S13983[117], 3}, {NULL}};
const Segment SGI129512[] = {{&S13983[82], 6}, {NULL}};
const Segment SGI129513[] = {{&S13984[44], 84}, {NULL}};
const Segment SGI129514[] = {{&S13984[44], 105}, {NULL}};
const Segment SGI129515[] = {{&S13984[30], 6}, {NULL}};
const Segment SGI129516[] = {{&S13984[196], 3}, {NULL}};
const Segment SGI129517[] = {{&S13984[122], 5}, {NULL}};
const Segment SGI129518[] = {{&S13985[44], 10}, {NULL}};
const Segment SGI129519[] = {{&S13985[44], 22}, {NULL}};
const Segment SGI129520[] = {{&S13985[30], 6}, {NULL}};
const Segment SGI129521[] = {{&S13985[115], 3}, {NULL}};
const Segment SGI129522[] = {{&S13985[80], 6}, {NULL}};
const Segment SGI129523[] = {{&S13986[44], 4}, {NULL}};
const Segment SGI129524[] = {{&S13986[44], 12}, {NULL}};
const Segment SGI129525[] = {{&S13986[30], 6}, {NULL}};
const Segment SGI129526[] = {{&S13986[105], 3}, {NULL}};
const Segment SGI129527[] = {{&S13986[70], 6}, {NULL}};
const Segment SGI129528[] = {{&S13987[44], 6}, {NULL}};
const Segment SGI129529[] = {{&S13987[44], 18}, {NULL}};
const Segment SGI129530[] = {{&S13987[30], 6}, {NULL}};
const Segment SGI129531[] = {{&S13987[111], 3}, {NULL}};
const Segment SGI129532[] = {{&S13987[76], 6}, {NULL}};
const Segment SGI129533[] = {{&S13988[44], 13}, {NULL}};
const Segment SGI129534[] = {{&S13988[44], 25}, {NULL}};
const Segment SGI129535[] = {{&S13988[30], 6}, {NULL}};
const Segment SGI129536[] = {{&S13988[118], 3}, {NULL}};
const Segment SGI129537[] = {{&S13988[83], 6}, {NULL}};
const Segment SGI129538[] = {{&S13989[44], 13}, {NULL}};
const Segment SGI129539[] = {{&S13989[44], 20}, {NULL}};
const Segment SGI129540[] = {{&S13989[30], 6}, {NULL}};
const Segment SGI129541[] = {{&S13989[113], 3}, {NULL}};
const Segment SGI129542[] = {{&S13989[78], 6}, {NULL}};
const Segment SGI129543[] = {{&S13991[44], 13}, {NULL}};
const Segment SGI129544[] = {{&S13991[44], 25}, {NULL}};
const Segment SGI129545[] = {{&S13991[30], 6}, {NULL}};
const Segment SGI129546[] = {{&S13991[118], 3}, {NULL}};
const Segment SGI129547[] = {{&S13991[83], 6}, {NULL}};
const Segment SGI129548[] = {{&S13993[44], 9}, {NULL}};
const Segment SGI129549[] = {{&S13993[44], 21}, {NULL}};
const Segment SGI129550[] = {{&S13993[30], 6}, {NULL}};
const Segment SGI129551[] = {{&S13993[114], 3}, {NULL}};
const Segment SGI129552[] = {{&S13993[79], 6}, {NULL}};
const Segment SGI129553[] = {{&S13995[44], 11}, {NULL}};
const Segment SGI129554[] = {{&S13995[44], 23}, {NULL}};
const Segment SGI129555[] = {{&S13995[30], 6}, {NULL}};
const Segment SGI129556[] = {{&S13995[116], 3}, {NULL}};
const Segment SGI129557[] = {{&S13995[81], 6}, {NULL}};
const Segment SGI129558[] = {{&S13073[44], 9}, {NULL}};
const Segment SGI129559[] = {{&S13073[44], 21}, {NULL}};
const Segment SGI129560[] = {{&S13073[30], 6}, {NULL}};
const Segment SGI129561[] = {{&S13073[114], 3}, {NULL}};
const Segment SGI129562[] = {{&S13073[79], 6}, {NULL}};
const Segment SGI129563[] = {{&S13996[44], 9}, {NULL}};
const Segment SGI129564[] = {{&S13996[44], 21}, {NULL}};
const Segment SGI129565[] = {{&S13996[30], 6}, {NULL}};
const Segment SGI129566[] = {{&S13996[114], 3}, {NULL}};
const Segment SGI129567[] = {{&S13996[79], 6}, {NULL}};
const Segment SGI129568[] = {{&S13997[44], 13}, {NULL}};
const Segment SGI129569[] = {{&S13997[44], 25}, {NULL}};
const Segment SGI129570[] = {{&S13997[30], 6}, {NULL}};
const Segment SGI129571[] = {{&S13997[118], 3}, {NULL}};
const Segment SGI129572[] = {{&S13997[83], 6}, {NULL}};
const Segment SGI129573[] = {{&S13998[44], 8}, {NULL}};
const Segment SGI129574[] = {{&S13998[44], 20}, {NULL}};
const Segment SGI129575[] = {{&S13998[30], 6}, {NULL}};
const Segment SGI129576[] = {{&S13998[113], 3}, {NULL}};
const Segment SGI129577[] = {{&S13998[78], 6}, {NULL}};
const Segment SGI129578[] = {{&S14000[44], 8}, {NULL}};
const Segment SGI129579[] = {{&S14000[44], 20}, {NULL}};
const Segment SGI129580[] = {{&S14000[30], 6}, {NULL}};
const Segment SGI129581[] = {{&S14000[113], 3}, {NULL}};
const Segment SGI129582[] = {{&S14000[78], 6}, {NULL}};
const Segment SGI129583[] = {{&S14001[44], 10}, {NULL}};
const Segment SGI129584[] = {{&S14001[44], 22}, {NULL}};
const Segment SGI129585[] = {{&S14001[30], 6}, {NULL}};
const Segment SGI129586[] = {{&S14001[115], 3}, {NULL}};
const Segment SGI129587[] = {{&S14001[80], 6}, {NULL}};
const Segment SGI129588[] = {{&S14002[44], 13}, {NULL}};
const Segment SGI129589[] = {{&S14002[44], 20}, {NULL}};
const Segment SGI129590[] = {{&S14002[30], 6}, {NULL}};
const Segment SGI129591[] = {{&S14002[113], 3}, {NULL}};
const Segment SGI129592[] = {{&S14002[78], 6}, {NULL}};
const Segment SGI129593[] = {{&S14003[44], 8}, {NULL}};
const Segment SGI129594[] = {{&S14003[44], 20}, {NULL}};
const Segment SGI129595[] = {{&S14003[30], 6}, {NULL}};
const Segment SGI129596[] = {{&S14003[113], 3}, {NULL}};
const Segment SGI129597[] = {{&S14003[78], 6}, {NULL}};
const Segment SGI129598[] = {{&S14004[44], 5}, {NULL}};
const Segment SGI129599[] = {{&S14004[44], 17}, {NULL}};
const Segment SGI129600[] = {{&S14004[30], 6}, {NULL}};
const Segment SGI129601[] = {{&S14004[110], 3}, {NULL}};
const Segment SGI129602[] = {{&S14004[75], 6}, {NULL}};
const Segment SGI129603[] = {{&S13090[44], 24}, {NULL}};
const Segment SGI129604[] = {{&S13090[44], 36}, {NULL}};
const Segment SGI129605[] = {{&S13090[30], 6}, {NULL}};
const Segment SGI129606[] = {{&S13090[129], 3}, {NULL}};
const Segment SGI129607[] = {{&S13090[94], 6}, {NULL}};
const Segment SGI129608[] = {{&S14007[44], 16}, {NULL}};
const Segment SGI129609[] = {{&S13091[44], 10}, {NULL}};
const Segment SGI129610[] = {{&S14007[44], 28}, {NULL}};
const Segment SGI129611[] = {{&S13091[44], 22}, {NULL}};
const Segment SGI129612[] = {{&S14007[30], 6}, {NULL}};
const Segment SGI129613[] = {{&S13091[30], 6}, {NULL}};
const Segment SGI129614[] = {{&S14007[121], 3}, {NULL}};
const Segment SGI129615[] = {{&S13091[115], 3}, {NULL}};
const Segment SGI129616[] = {{&S14007[86], 6}, {NULL}};
const Segment SGI129617[] = {{&S13091[80], 6}, {NULL}};
const Segment SGI129618[] = {{&S14010[44], 11}, {NULL}};
const Segment SGI129619[] = {{&S14010[44], 23}, {NULL}};
const Segment SGI129620[] = {{&S14010[30], 6}, {NULL}};
const Segment SGI129621[] = {{&S14010[116], 3}, {NULL}};
const Segment SGI129622[] = {{&S14010[81], 6}, {NULL}};
const Segment SGI129623[] = {{&S14011[44], 11}, {NULL}};
const Segment SGI129624[] = {{&S14011[44], 23}, {NULL}};
const Segment SGI129625[] = {{&S14011[30], 6}, {NULL}};
const Segment SGI129626[] = {{&S14011[116], 3}, {NULL}};
const Segment SGI129627[] = {{&S14011[81], 6}, {NULL}};
const Segment SGI129628[] = {{&S14013[44], 17}, {NULL}};
const Segment SGI129629[] = {{&S14013[44], 38}, {NULL}};
const Segment SGI129630[] = {{&S14013[30], 6}, {NULL}};
const Segment SGI129631[] = {{&S14013[131], 3}, {NULL}};
const Segment SGI129632[] = {{&S14013[96], 6}, {NULL}};
const Segment SGI129633[] = {{&S14016[44], 11}, {NULL}};
const Segment SGI129634[] = {{&S14016[44], 23}, {NULL}};
const Segment SGI129635[] = {{&S14016[30], 6}, {NULL}};
const Segment SGI129636[] = {{&S14016[116], 3}, {NULL}};
const Segment SGI129637[] = {{&S14016[81], 6}, {NULL}};
const Segment SGI129638[] = {{&S14018[44], 16}, {NULL}};
const Segment SGI129639[] = {{&S14018[44], 28}, {NULL}};
const Segment SGI129640[] = {{&S14018[30], 6}, {NULL}};
const Segment SGI129641[] = {{&S14018[121], 3}, {NULL}};
const Segment SGI129642[] = {{&S14018[86], 6}, {NULL}};
const Segment SGI129643[] = {{&S14019[44], 17}, {NULL}};
const Segment SGI129644[] = {{&S14019[44], 29}, {NULL}};
const Segment SGI129645[] = {{&S14019[30], 6}, {NULL}};
const Segment SGI129646[] = {{&S14019[122], 3}, {NULL}};
const Segment SGI129647[] = {{&S14019[87], 6}, {NULL}};
const Segment SGI129648[] = {{&S14021[44], 11}, {NULL}};
const Segment SGI129649[] = {{&S14021[44], 23}, {NULL}};
const Segment SGI129650[] = {{&S14021[30], 6}, {NULL}};
const Segment SGI129651[] = {{&S14021[116], 3}, {NULL}};
const Segment SGI129652[] = {{&S14021[81], 6}, {NULL}};
const Segment SGI129653[] = {{&S14022[44], 7}, {NULL}};
const Segment SGI129654[] = {{&S14022[44], 19}, {NULL}};
const Segment SGI129655[] = {{&S14022[30], 6}, {NULL}};
const Segment SGI129656[] = {{&S14022[112], 3}, {NULL}};
const Segment SGI129657[] = {{&S14022[77], 6}, {NULL}};
const Segment SGI129658[] = {{&S13115[44], 6}, {NULL}};
const Segment SGI129659[] = {{&S13115[44], 18}, {NULL}};
const Segment SGI129660[] = {{&S13115[30], 6}, {NULL}};
const Segment SGI129661[] = {{&S13115[111], 3}, {NULL}};
const Segment SGI129662[] = {{&S14023[44], 11}, {NULL}};
const Segment SGI129663[] = {{&S13115[76], 6}, {NULL}};
const Segment SGI129664[] = {{&S14023[44], 23}, {NULL}};
const Segment SGI129665[] = {{&S14023[30], 6}, {NULL}};
const Segment SGI129666[] = {{&S14023[116], 3}, {NULL}};
const Segment SGI129667[] = {{&S14023[81], 6}, {NULL}};
const Segment SGI129668[] = {{&S14024[44], 21}, {NULL}};
const Segment SGI129669[] = {{&S14024[44], 33}, {NULL}};
const Segment SGI129670[] = {{&S13117[44], 9}, {NULL}};
const Segment SGI129671[] = {{&S14024[30], 6}, {NULL}};
const Segment SGI129672[] = {{&S13117[44], 21}, {NULL}};
const Segment SGI129673[] = {{&S13117[30], 6}, {NULL}};
const Segment SGI129674[] = {{&S14024[126], 3}, {NULL}};
const Segment SGI129675[] = {{&S13117[114], 3}, {NULL}};
const Segment SGI129676[] = {{&S14024[91], 6}, {NULL}};
const Segment SGI129677[] = {{&S13117[79], 6}, {NULL}};
const Segment SGI129678[] = {{&S14025[44], 7}, {NULL}};
const Segment SGI129679[] = {{&S14025[44], 19}, {NULL}};
const Segment SGI129680[] = {{&S14025[30], 6}, {NULL}};
const Segment SGI129681[] = {{&S14025[112], 3}, {NULL}};
const Segment SGI129682[] = {{&S14025[77], 6}, {NULL}};
const Segment SGI129683[] = {{&S14027[44], 7}, {NULL}};
const Segment SGI129684[] = {{&S14027[44], 19}, {NULL}};
const Segment SGI129685[] = {{&S14027[30], 6}, {NULL}};
const Segment SGI129686[] = {{&S14027[112], 3}, {NULL}};
const Segment SGI129687[] = {{&S14027[77], 6}, {NULL}};
const Segment SGI129688[] = {{&S14028[44], 8}, {NULL}};
const Segment SGI129689[] = {{&S14028[44], 20}, {NULL}};
const Segment SGI129690[] = {{&S14028[30], 6}, {NULL}};
const Segment SGI129691[] = {{&S14028[113], 3}, {NULL}};
const Segment SGI129692[] = {{&S14028[78], 6}, {NULL}};
const Segment SGI129693[] = {{&S14029[44], 19}, {NULL}};
const Segment SGI129694[] = {{&S14029[44], 31}, {NULL}};
const Segment SGI129695[] = {{&S14029[30], 6}, {NULL}};
const Segment SGI129696[] = {{&S14029[124], 3}, {NULL}};
const Segment SGI129697[] = {{&S14029[89], 6}, {NULL}};
const Segment SGI129698[] = {{&S14030[44], 16}, {NULL}};
const Segment SGI129699[] = {{&S14030[44], 35}, {NULL}};
const Segment SGI129700[] = {{&S14030[30], 6}, {NULL}};
const Segment SGI129701[] = {{&S14030[128], 3}, {NULL}};
const Segment SGI129702[] = {{&S14030[93], 6}, {NULL}};
const Segment SGI129703[] = {{&S14031[44], 9}, {NULL}};
const Segment SGI129704[] = {{&S14031[44], 21}, {NULL}};
const Segment SGI129705[] = {{&S14031[30], 6}, {NULL}};
const Segment SGI129706[] = {{&S14031[114], 3}, {NULL}};
const Segment SGI129707[] = {{&S14031[79], 6}, {NULL}};
const Segment SGI129708[] = {{&S14032[44], 13}, {NULL}};
const Segment SGI129709[] = {{&S14032[44], 25}, {NULL}};
const Segment SGI129710[] = {{&S14032[30], 6}, {NULL}};
const Segment SGI129711[] = {{&S14032[118], 3}, {NULL}};
const Segment SGI129712[] = {{&S14032[83], 6}, {NULL}};
const Segment SGI129713[] = {{&S14033[44], 9}, {NULL}};
const Segment SGI129714[] = {{&S14033[44], 21}, {NULL}};
const Segment SGI129715[] = {{&S14033[30], 6}, {NULL}};
const Segment SGI129716[] = {{&S14033[114], 3}, {NULL}};
const Segment SGI129717[] = {{&S14033[79], 6}, {NULL}};
const Segment SGI129718[] = {{&S14036[44], 5}, {NULL}};
const Segment SGI129719[] = {{&S14036[44], 17}, {NULL}};
const Segment SGI129720[] = {{&S14036[30], 6}, {NULL}};
const Segment SGI129721[] = {{&S14036[110], 3}, {NULL}};
const Segment SGI129722[] = {{&S14036[75], 6}, {NULL}};
const Segment SGI129723[] = {{&S23329[44], 26}, {NULL}};
const Segment SGI129724[] = {{&S23329[44], 38}, {NULL}};
const Segment SGI129725[] = {{&S23329[30], 6}, {NULL}};
const Segment SGI129726[] = {{&S23329[31], 3}, {NULL}};
const Segment SGI129727[] = {{&S23329[96], 6}, {NULL}};
const Segment SGI129728[] = {{&S14040[44], 10}, {NULL}};
const Segment SGI129729[] = {{&S14040[44], 22}, {NULL}};
const Segment SGI129730[] = {{&S14040[30], 6}, {NULL}};
const Segment SGI129731[] = {{&S23332[44], 15}, {NULL}};
const Segment SGI129732[] = {{&S23332[44], 27}, {NULL}};
const Segment SGI129733[] = {{&S14040[115], 3}, {NULL}};
const Segment SGI129734[] = {{&S23332[30], 6}, {NULL}};
const Segment SGI129735[] = {{&S14040[80], 6}, {NULL}};
const Segment SGI129736[] = {{&S23332[31], 3}, {NULL}};
const Segment SGI129737[] = {{&S23332[85], 6}, {NULL}};
const Segment SGI129738[] = {{&S13144[44], 11}, {NULL}};
const Segment SGI129739[] = {{&S13144[44], 23}, {NULL}};
const Segment SGI129740[] = {{&S14041[44], 9}, {NULL}};
const Segment SGI129741[] = {{&S13144[30], 6}, {NULL}};
const Segment SGI129742[] = {{&S14041[44], 21}, {NULL}};
const Segment SGI129743[] = {{&S14041[30], 6}, {NULL}};
const Segment SGI129744[] = {{&S13144[116], 3}, {NULL}};
const Segment SGI129745[] = {{&S14041[114], 3}, {NULL}};
const Segment SGI129746[] = {{&S13144[81], 6}, {NULL}};
const Segment SGI129747[] = {{&S14041[79], 6}, {NULL}};
const Segment SGI129748[] = {{&S14042[44], 9}, {NULL}};
const Segment SGI129749[] = {{&S14042[44], 21}, {NULL}};
const Segment SGI129750[] = {{&S14042[30], 6}, {NULL}};
const Segment SGI129751[] = {{&S14042[114], 3}, {NULL}};
const Segment SGI129752[] = {{&S14042[79], 6}, {NULL}};
const Segment SGI129753[] = {{&S14043[44], 14}, {NULL}};
const Segment SGI129754[] = {{&S14043[44], 26}, {NULL}};
const Segment SGI129755[] = {{&S14043[30], 6}, {NULL}};
const Segment SGI129756[] = {{&S14043[119], 3}, {NULL}};
const Segment SGI129757[] = {{&S14043[84], 6}, {NULL}};
const Segment SGI129758[] = {{&S14045[44], 19}, {NULL}};
const Segment SGI129759[] = {{&S14045[44], 31}, {NULL}};
const Segment SGI129760[] = {{&S14045[30], 6}, {NULL}};
const Segment SGI129761[] = {{&S14045[124], 3}, {NULL}};
const Segment SGI129762[] = {{&S14045[89], 6}, {NULL}};
const Segment SGI129763[] = {{&S14047[44], 15}, {NULL}};
const Segment SGI129764[] = {{&S14047[44], 34}, {NULL}};
const Segment SGI129765[] = {{&S14047[30], 6}, {NULL}};
const Segment SGI129766[] = {{&S14047[127], 3}, {NULL}};
const Segment SGI129767[] = {{&S14047[92], 6}, {NULL}};
const Segment SGI129768[] = {{&S14048[44], 18}, {NULL}};
const Segment SGI129769[] = {{&S14048[44], 30}, {NULL}};
const Segment SGI129770[] = {{&S14048[30], 6}, {NULL}};
const Segment SGI129771[] = {{&S14048[123], 3}, {NULL}};
const Segment SGI129772[] = {{&S14048[88], 6}, {NULL}};
const Segment SGI129773[] = {{&S14050[44], 8}, {NULL}};
const Segment SGI129774[] = {{&S14050[44], 20}, {NULL}};
const Segment SGI129775[] = {{&S14050[30], 6}, {NULL}};
const Segment SGI129776[] = {{&S14050[113], 3}, {NULL}};
const Segment SGI129777[] = {{&S14050[78], 6}, {NULL}};
const Segment SGI129778[] = {{&S14051[44], 6}, {NULL}};
const Segment SGI129779[] = {{&S14051[44], 27}, {NULL}};
const Segment SGI129780[] = {{&S14051[30], 6}, {NULL}};
const Segment SGI129781[] = {{&S14051[119], 3}, {NULL}};
const Segment SGI129782[] = {{&S14051[85], 5}, {NULL}};
const Segment SGI129783[] = {{&S14052[44], 6}, {NULL}};
const Segment SGI129784[] = {{&S14052[44], 18}, {NULL}};
const Segment SGI129785[] = {{&S14052[30], 6}, {NULL}};
const Segment SGI129786[] = {{&S14052[111], 3}, {NULL}};
const Segment SGI129787[] = {{&S14052[76], 6}, {NULL}};
const Segment SGI129788[] = {{&S14055[44], 24}, {NULL}};
const Segment SGI129789[] = {{&S14055[44], 36}, {NULL}};
const Segment SGI129790[] = {{&S14055[30], 6}, {NULL}};
const Segment SGI129791[] = {{&S14055[129], 3}, {NULL}};
const Segment SGI129792[] = {{&S14055[94], 6}, {NULL}};
const Segment SGI129793[] = {{&S23359[44], 10}, {NULL}};
const Segment SGI129794[] = {{&S23359[44], 22}, {NULL}};
const Segment SGI129795[] = {{&S23359[30], 6}, {NULL}};
const Segment SGI129796[] = {{&S23359[31], 3}, {NULL}};
const Segment SGI129797[] = {{&S23359[80], 6}, {NULL}};
const Segment SGI129798[] = {{&S14057[44], 6}, {NULL}};
const Segment SGI129799[] = {{&S14057[44], 18}, {NULL}};
const Segment SGI129800[] = {{&S14057[30], 6}, {NULL}};
const Segment SGI129801[] = {{&S14057[111], 3}, {NULL}};
const Segment SGI129802[] = {{&S14057[76], 6}, {NULL}};
const Segment SGI129803[] = {{&S14058[44], 12}, {NULL}};
const Segment SGI129804[] = {{&S14058[44], 24}, {NULL}};
const Segment SGI129805[] = {{&S14058[30], 6}, {NULL}};
const Segment SGI129806[] = {{&S14058[117], 3}, {NULL}};
const Segment SGI129807[] = {{&S14058[82], 6}, {NULL}};
const Segment SGI129808[] = {{&S23363[44], 10}, {NULL}};
const Segment SGI129809[] = {{&S23363[44], 22}, {NULL}};
const Segment SGI129810[] = {{&S23363[30], 6}, {NULL}};
const Segment SGI129811[] = {{&S23363[31], 3}, {NULL}};
const Segment SGI129812[] = {{&S23363[80], 6}, {NULL}};
const Segment SGI129813[] = {{&S14059[44], 28}, {NULL}};
const Segment SGI129814[] = {{&S14059[44], 40}, {NULL}};
const Segment SGI129815[] = {{&S14059[30], 6}, {NULL}};
const Segment SGI129816[] = {{&S14059[133], 3}, {NULL}};
const Segment SGI129817[] = {{&S14059[98], 6}, {NULL}};
const Segment SGI129818[] = {{&S23364[44], 10}, {NULL}};
const Segment SGI129819[] = {{&S23364[44], 22}, {NULL}};
const Segment SGI129820[] = {{&S23364[30], 6}, {NULL}};
const Segment SGI129821[] = {{&S23364[31], 3}, {NULL}};
const Segment SGI129822[] = {{&S23364[80], 6}, {NULL}};
const Segment SGI129823[] = {{&S13176[44], 10}, {NULL}};
const Segment SGI129824[] = {{&S14060[44], 13}, {NULL}};
const Segment SGI129825[] = {{&S13176[44], 22}, {NULL}};
const Segment SGI129826[] = {{&S14060[44], 25}, {NULL}};
const Segment SGI129827[] = {{&S13176[30], 6}, {NULL}};
const Segment SGI129828[] = {{&S14060[30], 6}, {NULL}};
const Segment SGI129829[] = {{&S13176[115], 3}, {NULL}};
const Segment SGI129830[] = {{&S14060[118], 3}, {NULL}};
const Segment SGI129831[] = {{&S13176[80], 6}, {NULL}};
const Segment SGI129832[] = {{&S14060[83], 6}, {NULL}};
const Segment SGI129833[] = {{&S23365[44], 18}, {NULL}};
const Segment SGI129834[] = {{&S23365[44], 30}, {NULL}};
const Segment SGI129835[] = {{&S23365[30], 6}, {NULL}};
const Segment SGI129836[] = {{&S23365[31], 3}, {NULL}};
const Segment SGI129837[] = {{&S23365[88], 6}, {NULL}};
const Segment SGI129838[] = {{&S14061[44], 8}, {NULL}};
const Segment SGI129839[] = {{&S14061[44], 20}, {NULL}};
const Segment SGI129840[] = {{&S14061[30], 6}, {NULL}};
const Segment SGI129841[] = {{&S14061[113], 3}, {NULL}};
const Segment SGI129842[] = {{&S14061[78], 6}, {NULL}};
const Segment SGI129843[] = {{&S13178[44], 8}, {NULL}};
const Segment SGI129844[] = {{&S13178[44], 20}, {NULL}};
const Segment SGI129845[] = {{&S13178[30], 6}, {NULL}};
const Segment SGI129846[] = {{&S13178[113], 3}, {NULL}};
const Segment SGI129847[] = {{&S13178[78], 6}, {NULL}};
const Segment SGI129848[] = {{&S23368[44], 18}, {NULL}};
const Segment SGI129849[] = {{&S23368[44], 30}, {NULL}};
const Segment SGI129850[] = {{&S23368[30], 6}, {NULL}};
const Segment SGI129851[] = {{&S23368[31], 3}, {NULL}};
const Segment SGI129852[] = {{&S23368[88], 6}, {NULL}};
const Segment SGI129853[] = {{&S23369[44], 11}, {NULL}};
const Segment SGI129854[] = {{&S23369[44], 23}, {NULL}};
const Segment SGI129855[] = {{&S23369[30], 6}, {NULL}};
const Segment SGI129856[] = {{&S23369[31], 3}, {NULL}};
const Segment SGI129857[] = {{&S23369[81], 6}, {NULL}};
const Segment SGI129858[] = {{&S14065[44], 11}, {NULL}};
const Segment SGI129859[] = {{&S14065[44], 23}, {NULL}};
const Segment SGI129860[] = {{&S14065[30], 6}, {NULL}};
const Segment SGI129861[] = {{&S14065[116], 3}, {NULL}};
const Segment SGI129862[] = {{&S14065[81], 6}, {NULL}};
const Segment SGI129863[] = {{&S23370[44], 10}, {NULL}};
const Segment SGI129864[] = {{&S23370[44], 22}, {NULL}};
const Segment SGI129865[] = {{&S23370[30], 6}, {NULL}};
const Segment SGI129866[] = {{&S23370[31], 3}, {NULL}};
const Segment SGI129867[] = {{&S23370[80], 6}, {NULL}};
const Segment SGI129868[] = {{&S14066[44], 15}, {NULL}};
const Segment SGI129869[] = {{&S14066[44], 34}, {NULL}};
const Segment SGI129870[] = {{&S14066[30], 6}, {NULL}};
const Segment SGI129871[] = {{&S14066[127], 3}, {NULL}};
const Segment SGI129872[] = {{&S14066[92], 6}, {NULL}};
const Segment SGI129873[] = {{&S23371[38], 18}, {NULL}};
const Segment SGI129874[] = {{&S23371[31], 6}, {NULL}};
const Segment SGI129875[] = {{&S14067[44], 10}, {NULL}};
const Segment SGI129876[] = {{&S14067[44], 22}, {NULL}};
const Segment SGI129877[] = {{&S14067[30], 6}, {NULL}};
const Segment SGI129878[] = {{&S14067[115], 3}, {NULL}};
const Segment SGI129879[] = {{&S14067[80], 6}, {NULL}};
const Segment SGI129880[] = {{&S13187[44], 6}, {NULL}};
const Segment SGI129881[] = {{&S13187[44], 18}, {NULL}};
const Segment SGI129882[] = {{&S13187[30], 6}, {NULL}};
const Segment SGI129883[] = {{&S13187[111], 3}, {NULL}};
const Segment SGI129884[] = {{&S13187[76], 6}, {NULL}};
const Segment SGI129885[] = {{&S23374[38], 10}, {NULL}};
const Segment SGI129886[] = {{&S23374[31], 6}, {NULL}};
const Segment SGI129887[] = {{&S14069[44], 6}, {NULL}};
const Segment SGI129888[] = {{&S14069[44], 18}, {NULL}};
const Segment SGI129889[] = {{&S14069[30], 6}, {NULL}};
const Segment SGI129890[] = {{&S23375[38], 11}, {NULL}};
const Segment SGI129891[] = {{&S14069[111], 3}, {NULL}};
const Segment SGI129892[] = {{&S23375[31], 6}, {NULL}};
const Segment SGI129893[] = {{&S14069[76], 6}, {NULL}};
const Segment SGI129894[] = {{&S23376[42], 10}, {NULL}};
const Segment SGI129895[] = {{&S23376[30], 6}, {NULL}};
const Segment SGI129896[] = {{&S23376[31], 3}, {NULL}};
const Segment SGI129897[] = {{&S23376[78], 6}, {NULL}};
const Segment SGI129898[] = {{&S14071[44], 8}, {NULL}};
const Segment SGI129899[] = {{&S14071[44], 20}, {NULL}};
const Segment SGI129900[] = {{&S14071[30], 6}, {NULL}};
const Segment SGI129901[] = {{&S14071[113], 3}, {NULL}};
const Segment SGI129902[] = {{&S14071[78], 6}, {NULL}};
const Segment SGI129903[] = {{&S14072[44], 8}, {NULL}};
const Segment SGI129904[] = {{&S14072[44], 20}, {NULL}};
const Segment SGI129905[] = {{&S14072[30], 6}, {NULL}};
const Segment SGI129906[] = {{&S23378[42], 18}, {NULL}};
const Segment SGI129907[] = {{&S23378[30], 6}, {NULL}};
const Segment SGI129908[] = {{&S14072[113], 3}, {NULL}};
const Segment SGI129909[] = {{&S14072[78], 6}, {NULL}};
const Segment SGI129910[] = {{&S23378[31], 3}, {NULL}};
const Segment SGI129911[] = {{&S23378[86], 6}, {NULL}};
const Segment SGI129912[] = {{&S14073[44], 5}, {NULL}};
const Segment SGI129913[] = {{&S14073[44], 17}, {NULL}};
const Segment SGI129914[] = {{&S14073[30], 6}, {NULL}};
const Segment SGI129915[] = {{&S14073[110], 3}, {NULL}};
const Segment SGI129916[] = {{&S14073[75], 6}, {NULL}};
const Segment SGI129917[] = {{&S14074[44], 17}, {NULL}};
const Segment SGI129918[] = {{&S14074[44], 29}, {NULL}};
const Segment SGI129919[] = {{&S14074[30], 6}, {NULL}};
const Segment SGI129920[] = {{&S14074[122], 3}, {NULL}};
const Segment SGI129921[] = {{&S14074[87], 6}, {NULL}};
const Segment SGI129922[] = {{&S14075[44], 8}, {NULL}};
const Segment SGI129923[] = {{&S14075[44], 20}, {NULL}};
const Segment SGI129924[] = {{&S14075[30], 6}, {NULL}};
const Segment SGI129925[] = {{&S14075[113], 3}, {NULL}};
const Segment SGI129926[] = {{&S14075[78], 6}, {NULL}};
const Segment SGI129927[] = {{&S14076[44], 13}, {NULL}};
const Segment SGI129928[] = {{&S14076[44], 25}, {NULL}};
const Segment SGI129929[] = {{&S14076[30], 6}, {NULL}};
const Segment SGI129930[] = {{&S14076[118], 3}, {NULL}};
const Segment SGI129931[] = {{&S14076[83], 6}, {NULL}};
const Segment SGI129932[] = {{&S14077[44], 7}, {NULL}};
const Segment SGI129933[] = {{&S14077[44], 19}, {NULL}};
const Segment SGI129934[] = {{&S14077[30], 6}, {NULL}};
const Segment SGI129935[] = {{&S14077[112], 3}, {NULL}};
const Segment SGI129936[] = {{&S14077[77], 6}, {NULL}};
const Segment SGI129937[] = {{&S14078[44], 17}, {NULL}};
const Segment SGI129938[] = {{&S14078[44], 29}, {NULL}};
const Segment SGI129939[] = {{&S14078[30], 6}, {NULL}};
const Segment SGI129940[] = {{&S14078[122], 3}, {NULL}};
const Segment SGI129941[] = {{&S14078[87], 6}, {NULL}};
const Segment SGI129942[] = {{&S14079[44], 16}, {NULL}};
const Segment SGI129943[] = {{&S14079[44], 30}, {NULL}};
const Segment SGI129944[] = {{&S14079[30], 6}, {NULL}};
const Segment SGI129945[] = {{&S14079[123], 3}, {NULL}};
const Segment SGI129946[] = {{&S14079[88], 6}, {NULL}};
const Segment SGI129947[] = {{&S14082[58], 5}, {NULL}};
const Segment SGI129948[] = {{&S14082[41], 16}, {NULL}};
const Segment SGI129949[] = {{&S14082[28], 6}, {NULL}};
const Segment SGI129950[] = {{&S14082[29], 3}, {NULL}};
const Segment SGI129951[] = {{&S14082[86], 9}, {NULL}};
const Segment SGI129952[] = {{&S14083[44], 18}, {NULL}};
const Segment SGI129953[] = {{&S14083[44], 30}, {NULL}};
const Segment SGI129954[] = {{&S14083[30], 6}, {NULL}};
const Segment SGI129955[] = {{&S14083[123], 3}, {NULL}};
const Segment SGI129956[] = {{&S14083[88], 6}, {NULL}};
const Segment SGI129957[] = {{&S14084[44], 9}, {NULL}};
const Segment SGI129958[] = {{&S14084[44], 21}, {NULL}};
const Segment SGI129959[] = {{&S14084[30], 6}, {NULL}};
const Segment SGI129960[] = {{&S14084[114], 3}, {NULL}};
const Segment SGI129961[] = {{&S14084[79], 6}, {NULL}};
const Segment SGI129962[] = {{&S14085[42], 10}, {NULL}};
const Segment SGI129963[] = {{&S14085[30], 6}, {NULL}};
const Segment SGI129964[] = {{&S14085[113], 3}, {NULL}};
const Segment SGI129965[] = {{&S14085[78], 6}, {NULL}};
const Segment SGI129966[] = {{&S14086[44], 8}, {NULL}};
const Segment SGI129967[] = {{&S14086[44], 20}, {NULL}};
const Segment SGI129968[] = {{&S14086[30], 6}, {NULL}};
const Segment SGI129969[] = {{&S14086[113], 3}, {NULL}};
const Segment SGI129970[] = {{&S14086[78], 6}, {NULL}};
const Segment SGI129971[] = {{&S14087[44], 5}, {NULL}};
const Segment SGI129972[] = {{&S14087[44], 17}, {NULL}};
const Segment SGI129973[] = {{&S14087[30], 6}, {NULL}};
const Segment SGI129974[] = {{&S14087[110], 3}, {NULL}};
const Segment SGI129975[] = {{&S14087[75], 6}, {NULL}};
const Segment SGI129976[] = {{&S14088[44], 6}, {NULL}};
const Segment SGI129977[] = {{&S14088[44], 18}, {NULL}};
const Segment SGI129978[] = {{&S14088[30], 6}, {NULL}};
const Segment SGI129979[] = {{&S14088[111], 3}, {NULL}};
const Segment SGI129980[] = {{&S14088[76], 6}, {NULL}};
const Segment SGI129981[] = {{&S14089[44], 7}, {NULL}};
const Segment SGI129982[] = {{&S14089[44], 19}, {NULL}};
const Segment SGI129983[] = {{&S14089[30], 6}, {NULL}};
const Segment SGI129984[] = {{&S14089[112], 3}, {NULL}};
const Segment SGI129985[] = {{&S14089[77], 6}, {NULL}};
const Segment SGI129986[] = {{&S14090[44], 4}, {NULL}};
const Segment SGI129987[] = {{&S14090[44], 16}, {NULL}};
const Segment SGI129988[] = {{&S14090[30], 6}, {NULL}};
const Segment SGI129989[] = {{&S14090[109], 3}, {NULL}};
const Segment SGI129990[] = {{&S14090[74], 6}, {NULL}};
const Segment SGI129991[] = {{&S14091[44], 13}, {NULL}};
const Segment SGI129992[] = {{&S14091[44], 25}, {NULL}};
const Segment SGI129993[] = {{&S14091[30], 6}, {NULL}};
const Segment SGI129994[] = {{&S14091[118], 3}, {NULL}};
const Segment SGI129995[] = {{&S14091[83], 6}, {NULL}};
const Segment SGI129996[] = {{&S14092[44], 11}, {NULL}};
const Segment SGI129997[] = {{&S14092[44], 23}, {NULL}};
const Segment SGI129998[] = {{&S14092[30], 6}, {NULL}};
const Segment SGI129999[] = {{&S14092[116], 3}, {NULL}};
const Segment SGI130000[] = {{&S14092[81], 6}, {NULL}};
const Segment SGI130001[] = {{&S14093[44], 7}, {NULL}};
const Segment SGI130002[] = {{&S14093[44], 19}, {NULL}};
const Segment SGI130003[] = {{&S14093[30], 6}, {NULL}};
const Segment SGI130004[] = {{&S14093[112], 3}, {NULL}};
const Segment SGI130005[] = {{&S14093[77], 6}, {NULL}};
const Segment SGI130006[] = {{&S14094[44], 9}, {NULL}};
const Segment SGI130007[] = {{&S14094[44], 21}, {NULL}};
const Segment SGI130008[] = {{&S14094[30], 6}, {NULL}};
const Segment SGI130009[] = {{&S14094[114], 3}, {NULL}};
const Segment SGI130010[] = {{&S14094[79], 6}, {NULL}};
const Segment SGI130011[] = {{&S14095[44], 7}, {NULL}};
const Segment SGI130012[] = {{&S14095[44], 19}, {NULL}};
const Segment SGI130013[] = {{&S14095[30], 6}, {NULL}};
const Segment SGI130014[] = {{&S14095[112], 3}, {NULL}};
const Segment SGI130015[] = {{&S14095[77], 6}, {NULL}};
const Segment SGI130016[] = {{&S14096[44], 9}, {NULL}};
const Segment SGI130017[] = {{&S14096[44], 21}, {NULL}};
const Segment SGI130018[] = {{&S14096[30], 6}, {NULL}};
const Segment SGI130019[] = {{&S14096[114], 3}, {NULL}};
const Segment SGI130020[] = {{&S14096[79], 6}, {NULL}};
const Segment SGI130021[] = {{&S14097[44], 5}, {NULL}};
const Segment SGI130022[] = {{&S14097[44], 17}, {NULL}};
const Segment SGI130023[] = {{&S14097[30], 6}, {NULL}};
const Segment SGI130024[] = {{&S14097[110], 3}, {NULL}};
const Segment SGI130025[] = {{&S14097[75], 6}, {NULL}};
const Segment SGI130026[] = {{&S14098[38], 29}, {NULL}};
const Segment SGI130027[] = {{&S14098[31], 6}, {NULL}};
const Segment SGI130028[] = {{&S14099[44], 12}, {NULL}};
const Segment SGI130029[] = {{&S14099[44], 24}, {NULL}};
const Segment SGI130030[] = {{&S14099[30], 6}, {NULL}};
const Segment SGI130031[] = {{&S14099[117], 3}, {NULL}};
const Segment SGI130032[] = {{&S14099[82], 6}, {NULL}};
const Segment SGI130033[] = {{&S14100[44], 12}, {NULL}};
const Segment SGI130034[] = {{&S14100[44], 24}, {NULL}};
const Segment SGI130035[] = {{&S14100[30], 6}, {NULL}};
const Segment SGI130036[] = {{&S14100[117], 3}, {NULL}};
const Segment SGI130037[] = {{&S14100[82], 6}, {NULL}};
const Segment SGI130038[] = {{&S14101[44], 6}, {NULL}};
const Segment SGI130039[] = {{&S14101[44], 18}, {NULL}};
const Segment SGI130040[] = {{&S14101[30], 6}, {NULL}};
const Segment SGI130041[] = {{&S14101[111], 3}, {NULL}};
const Segment SGI130042[] = {{&S14101[76], 6}, {NULL}};
const Segment SGI130043[] = {{&S14102[44], 7}, {NULL}};
const Segment SGI130044[] = {{&S14102[44], 19}, {NULL}};
const Segment SGI130045[] = {{&S14102[30], 6}, {NULL}};
const Segment SGI130046[] = {{&S14102[112], 3}, {NULL}};
const Segment SGI130047[] = {{&S14102[77], 6}, {NULL}};
const Segment SGI130048[] = {{&S14103[44], 8}, {NULL}};
const Segment SGI130049[] = {{&S14103[44], 20}, {NULL}};
const Segment SGI130050[] = {{&S14103[30], 6}, {NULL}};
const Segment SGI130051[] = {{&S14103[113], 3}, {NULL}};
const Segment SGI130052[] = {{&S14103[78], 6}, {NULL}};
const Segment SGI130053[] = {{&S14104[44], 7}, {NULL}};
const Segment SGI130054[] = {{&S14104[44], 19}, {NULL}};
const Segment SGI130055[] = {{&S14104[30], 6}, {NULL}};
const Segment SGI130056[] = {{&S14104[112], 3}, {NULL}};
const Segment SGI130057[] = {{&S14104[77], 6}, {NULL}};
const Segment SGI130058[] = {{&S14105[44], 10}, {NULL}};
const Segment SGI130059[] = {{&S14105[44], 22}, {NULL}};
const Segment SGI130060[] = {{&S14105[30], 6}, {NULL}};
const Segment SGI130061[] = {{&S14105[115], 3}, {NULL}};
const Segment SGI130062[] = {{&S14105[80], 6}, {NULL}};
const Segment SGI130063[] = {{&S13265[44], 9}, {NULL}};
const Segment SGI130064[] = {{&S13265[44], 21}, {NULL}};
const Segment SGI130065[] = {{&S13265[30], 6}, {NULL}};
const Segment SGI130066[] = {{&S14106[44], 12}, {NULL}};
const Segment SGI130067[] = {{&S13265[114], 3}, {NULL}};
const Segment SGI130068[] = {{&S14106[44], 24}, {NULL}};
const Segment SGI130069[] = {{&S14106[30], 6}, {NULL}};
const Segment SGI130070[] = {{&S13265[79], 6}, {NULL}};
const Segment SGI130071[] = {{&S14106[117], 3}, {NULL}};
const Segment SGI130072[] = {{&S14106[82], 6}, {NULL}};
const Segment SGI130073[] = {{&S14107[44], 7}, {NULL}};
const Segment SGI130074[] = {{&S14107[44], 19}, {NULL}};
const Segment SGI130075[] = {{&S14107[30], 6}, {NULL}};
const Segment SGI130076[] = {{&S14107[112], 3}, {NULL}};
const Segment SGI130077[] = {{&S14107[77], 6}, {NULL}};
const Segment SGI130078[] = {{&S14108[44], 10}, {NULL}};
const Segment SGI130079[] = {{&S14108[44], 22}, {NULL}};
const Segment SGI130080[] = {{&S14108[30], 6}, {NULL}};
const Segment SGI130081[] = {{&S14108[115], 3}, {NULL}};
const Segment SGI130082[] = {{&S14108[80], 6}, {NULL}};
const Segment SGI130083[] = {{&S14109[44], 7}, {NULL}};
const Segment SGI130084[] = {{&S14109[44], 19}, {NULL}};
const Segment SGI130085[] = {{&S14109[30], 6}, {NULL}};
const Segment SGI130086[] = {{&S14109[112], 3}, {NULL}};
const Segment SGI130087[] = {{&S14109[77], 6}, {NULL}};
const Segment SGI130088[] = {{&S14112[44], 13}, {NULL}};
const Segment SGI130089[] = {{&S14112[44], 25}, {NULL}};
const Segment SGI130090[] = {{&S14112[30], 6}, {NULL}};
const Segment SGI130091[] = {{&S14112[118], 3}, {NULL}};
const Segment SGI130092[] = {{&S14112[83], 6}, {NULL}};
const Segment SGI130093[] = {{&S14113[44], 10}, {NULL}};
const Segment SGI130094[] = {{&S14113[44], 22}, {NULL}};
const Segment SGI130095[] = {{&S14113[30], 6}, {NULL}};
const Segment SGI130096[] = {{&S14113[115], 3}, {NULL}};
const Segment SGI130097[] = {{&S14113[80], 6}, {NULL}};
const Segment SGI130098[] = {{&S14114[38], 14}, {NULL}};
const Segment SGI130099[] = {{&S14114[31], 6}, {NULL}};
const Segment SGI130100[] = {{&S14115[44], 15}, {NULL}};
const Segment SGI130101[] = {{&S14115[44], 27}, {NULL}};
const Segment SGI130102[] = {{&S14115[30], 6}, {NULL}};
const Segment SGI130103[] = {{&S14115[120], 3}, {NULL}};
const Segment SGI130104[] = {{&S14115[85], 6}, {NULL}};
const Segment SGI130105[] = {{&S14116[44], 6}, {NULL}};
const Segment SGI130106[] = {{&S14116[44], 18}, {NULL}};
const Segment SGI130107[] = {{&S14116[30], 6}, {NULL}};
const Segment SGI130108[] = {{&S14116[111], 3}, {NULL}};
const Segment SGI130109[] = {{&S14116[76], 6}, {NULL}};
const Segment SGI130110[] = {{&S14117[44], 4}, {NULL}};
const Segment SGI130111[] = {{&S14117[44], 16}, {NULL}};
const Segment SGI130112[] = {{&S14117[30], 6}, {NULL}};
const Segment SGI130113[] = {{&S14117[109], 3}, {NULL}};
const Segment SGI130114[] = {{&S14117[74], 6}, {NULL}};
const Segment SGI130115[] = {{&S14118[44], 9}, {NULL}};
const Segment SGI130116[] = {{&S14118[44], 21}, {NULL}};
const Segment SGI130117[] = {{&S14118[30], 6}, {NULL}};
const Segment SGI130118[] = {{&S14118[114], 3}, {NULL}};
const Segment SGI130119[] = {{&S14118[79], 6}, {NULL}};
const Segment SGI130120[] = {{&S14119[44], 13}, {NULL}};
const Segment SGI130121[] = {{&S14119[44], 25}, {NULL}};
const Segment SGI130122[] = {{&S14119[30], 6}, {NULL}};
const Segment SGI130123[] = {{&S14119[118], 3}, {NULL}};
const Segment SGI130124[] = {{&S14119[83], 6}, {NULL}};
const Segment SGI130125[] = {{&S14120[44], 7}, {NULL}};
const Segment SGI130126[] = {{&S14120[44], 19}, {NULL}};
const Segment SGI130127[] = {{&S14120[30], 6}, {NULL}};
const Segment SGI130128[] = {{&S14120[112], 3}, {NULL}};
const Segment SGI130129[] = {{&S14120[77], 6}, {NULL}};
const Segment SGI130130[] = {{&S14121[44], 15}, {NULL}};
const Segment SGI130131[] = {{&S14121[44], 27}, {NULL}};
const Segment SGI130132[] = {{&S14121[30], 6}, {NULL}};
const Segment SGI130133[] = {{&S14121[120], 3}, {NULL}};
const Segment SGI130134[] = {{&S14121[85], 6}, {NULL}};
const Segment SGI130135[] = {{&S14123[44], 16}, {NULL}};
const Segment SGI130136[] = {{&S14123[44], 28}, {NULL}};
const Segment SGI130137[] = {{&S14123[30], 6}, {NULL}};
const Segment SGI130138[] = {{&S14123[121], 3}, {NULL}};
const Segment SGI130139[] = {{&S13297[44], 11}, {NULL}};
const Segment SGI130140[] = {{&S14123[86], 6}, {NULL}};
const Segment SGI130141[] = {{&S13297[44], 23}, {NULL}};
const Segment SGI130142[] = {{&S13297[30], 6}, {NULL}};
const Segment SGI130143[] = {{&S13297[116], 3}, {NULL}};
const Segment SGI130144[] = {{&S13297[81], 6}, {NULL}};
const Segment SGI130145[] = {{&S14124[44], 9}, {NULL}};
const Segment SGI130146[] = {{&S14124[44], 21}, {NULL}};
const Segment SGI130147[] = {{&S14124[30], 6}, {NULL}};
const Segment SGI130148[] = {{&S14124[114], 3}, {NULL}};
const Segment SGI130149[] = {{&S14124[79], 6}, {NULL}};
const Segment SGI130150[] = {{&S14125[44], 6}, {NULL}};
const Segment SGI130151[] = {{&S14125[44], 18}, {NULL}};
const Segment SGI130152[] = {{&S13300[44], 9}, {NULL}};
const Segment SGI130153[] = {{&S14125[30], 6}, {NULL}};
const Segment SGI130154[] = {{&S13300[44], 21}, {NULL}};
const Segment SGI130155[] = {{&S13300[30], 6}, {NULL}};
const Segment SGI130156[] = {{&S14125[111], 3}, {NULL}};
const Segment SGI130157[] = {{&S13300[114], 3}, {NULL}};
const Segment SGI130158[] = {{&S14125[76], 6}, {NULL}};
const Segment SGI130159[] = {{&S13300[79], 6}, {NULL}};
const Segment SGI130160[] = {{&S14126[44], 6}, {NULL}};
const Segment SGI130161[] = {{&S14126[44], 18}, {NULL}};
const Segment SGI130162[] = {{&S14126[30], 6}, {NULL}};
const Segment SGI130163[] = {{&S14126[111], 3}, {NULL}};
const Segment SGI130164[] = {{&S14126[76], 6}, {NULL}};
const Segment SGI130165[] = {{&S13303[44], 23}, {NULL}};
const Segment SGI130166[] = {{&S13303[44], 35}, {NULL}};
const Segment SGI130167[] = {{&S13303[30], 6}, {NULL}};
const Segment SGI130168[] = {{&S14127[44], 8}, {NULL}};
const Segment SGI130169[] = {{&S13303[128], 3}, {NULL}};
const Segment SGI130170[] = {{&S14127[44], 20}, {NULL}};
const Segment SGI130171[] = {{&S14127[30], 6}, {NULL}};
const Segment SGI130172[] = {{&S13303[93], 6}, {NULL}};
const Segment SGI130173[] = {{&S14127[113], 3}, {NULL}};
const Segment SGI130174[] = {{&S14127[78], 6}, {NULL}};
const Segment SGI130175[] = {{&S14128[44], 10}, {NULL}};
const Segment SGI130176[] = {{&S14128[44], 22}, {NULL}};
const Segment SGI130177[] = {{&S14128[30], 6}, {NULL}};
const Segment SGI130178[] = {{&S14128[115], 3}, {NULL}};
const Segment SGI130179[] = {{&S14128[80], 6}, {NULL}};
const Segment SGI130180[] = {{&S14129[44], 4}, {NULL}};
const Segment SGI130181[] = {{&S14129[44], 16}, {NULL}};
const Segment SGI130182[] = {{&S14129[30], 6}, {NULL}};
const Segment SGI130183[] = {{&S14129[109], 3}, {NULL}};
const Segment SGI130184[] = {{&S14129[74], 6}, {NULL}};
const Segment SGI130185[] = {{&S13308[44], 17}, {NULL}};
const Segment SGI130186[] = {{&S13308[44], 29}, {NULL}};
const Segment SGI130187[] = {{&S13308[30], 6}, {&S13308[52], 21}, {NULL}};
const Segment SGI130188[] = {{&S14130[44], 11}, {NULL}};
const Segment SGI130189[] = {{&S14130[44], 23}, {NULL}};
const Segment SGI130190[] = {{&S13308[122], 3}, {NULL}};
const Segment SGI130191[] = {{&S14130[30], 6}, {NULL}};
const Segment SGI130192[] = {{&S13308[87], 6}, {NULL}};
const Segment SGI130193[] = {{&S14130[116], 3}, {NULL}};
const Segment SGI130194[] = {{&S14130[81], 6}, {NULL}};
const Segment SGI130195[] = {{&S14132[44], 24}, {NULL}};
const Segment SGI130196[] = {{&S14132[44], 36}, {NULL}};
const Segment SGI130197[] = {{&S14132[30], 6}, {NULL}};
const Segment SGI130198[] = {{&S14132[129], 3}, {NULL}};
const Segment SGI130199[] = {{&S14132[94], 6}, {NULL}};
const Segment SGI130200[] = {{&S14133[44], 17}, {NULL}};
const Segment SGI130201[] = {{&S14133[44], 38}, {NULL}};
const Segment SGI130202[] = {{&S14133[30], 6}, {NULL}};
const Segment SGI130203[] = {{&S14133[131], 3}, {NULL}};
const Segment SGI130204[] = {{&S14133[96], 6}, {NULL}};
const Segment SGI130205[] = {{&S14136[44], 7}, {NULL}};
const Segment SGI130206[] = {{&S14136[44], 19}, {NULL}};
const Segment SGI130207[] = {{&S14136[30], 6}, {NULL}};
const Segment SGI130208[] = {{&S14136[112], 3}, {NULL}};
const Segment SGI130209[] = {{&S14136[77], 6}, {NULL}};
const Segment SGI130210[] = {{&S14137[44], 8}, {NULL}};
const Segment SGI130211[] = {{&S14137[44], 20}, {NULL}};
const Segment SGI130212[] = {{&S14137[30], 6}, {NULL}};
const Segment SGI130213[] = {{&S14137[113], 3}, {NULL}};
const Segment SGI130214[] = {{&S14137[78], 6}, {NULL}};
const Segment SGI130215[] = {{&S13319[44], 16}, {NULL}};
const Segment SGI130216[] = {{&S13319[44], 28}, {NULL}};
const Segment SGI130217[] = {{&S13319[30], 6}, {NULL}};
const Segment SGI130218[] = {{&S13319[121], 3}, {NULL}};
const Segment SGI130219[] = {{&S13319[86], 6}, {NULL}};
const Segment SGI130220[] = {{&S14138[44], 8}, {NULL}};
const Segment SGI130221[] = {{&S14138[44], 20}, {NULL}};
const Segment SGI130222[] = {{&S14138[30], 6}, {NULL}};
const Segment SGI130223[] = {{&S14138[113], 3}, {NULL}};
const Segment SGI130224[] = {{&S14138[78], 6}, {NULL}};
const Segment SGI130225[] = {{&S14139[44], 4}, {NULL}};
const Segment SGI130226[] = {{&S14139[44], 16}, {NULL}};
const Segment SGI130227[] = {{&S14139[30], 6}, {NULL}};
const Segment SGI130228[] = {{&S14139[109], 3}, {NULL}};
const Segment SGI130229[] = {{&S14139[74], 6}, {NULL}};
const Segment SGI130230[] = {{&S13322[44], 6}, {NULL}};
const Segment SGI130231[] = {{&S13322[44], 18}, {NULL}};
const Segment SGI130232[] = {{&S13322[30], 6}, {NULL}};
const Segment SGI130233[] = {{&S13322[111], 3}, {NULL}};
const Segment SGI130234[] = {{&S13322[76], 6}, {NULL}};
const Segment SGI130235[] = {{&S14140[44], 8}, {NULL}};
const Segment SGI130236[] = {{&S14140[44], 20}, {NULL}};
const Segment SGI130237[] = {{&S14140[30], 6}, {NULL}};
const Segment SGI130238[] = {{&S14140[113], 3}, {NULL}};
const Segment SGI130239[] = {{&S14140[78], 6}, {NULL}};
const Segment SGI130240[] = {{&S23510[44], 14}, {NULL}};
const Segment SGI130241[] = {{&S23510[44], 26}, {NULL}};
const Segment SGI130242[] = {{&S23510[30], 6}, {NULL}};
const Segment SGI130243[] = {{&S23510[31], 3}, {NULL}};
const Segment SGI130244[] = {{&S23510[84], 6}, {NULL}};
const Segment SGI130245[] = {{&S14141[44], 9}, {NULL}};
const Segment SGI130246[] = {{&S14141[44], 21}, {NULL}};
const Segment SGI130247[] = {{&S14141[30], 6}, {NULL}};
const Segment SGI130248[] = {{&S14141[114], 3}, {NULL}};
const Segment SGI130249[] = {{&S14141[79], 6}, {NULL}};
const Segment SGI130250[] = {{&S14142[44], 8}, {NULL}};
const Segment SGI130251[] = {{&S14142[44], 20}, {NULL}};
const Segment SGI130252[] = {{&S14142[30], 6}, {NULL}};
const Segment SGI130253[] = {{&S14142[113], 3}, {NULL}};
const Segment SGI130254[] = {{&S14142[78], 6}, {NULL}};
const Segment SGI130255[] = {{&S14143[44], 7}, {NULL}};
const Segment SGI130256[] = {{&S14143[44], 14}, {NULL}};
const Segment SGI130257[] = {{&S14143[30], 6}, {NULL}};
const Segment SGI130258[] = {{&S14143[107], 3}, {NULL}};
const Segment SGI130259[] = {{&S14143[72], 6}, {NULL}};
const Segment SGI130260[] = {{&S23515[44], 14}, {NULL}};
const Segment SGI130261[] = {{&S23515[44], 26}, {NULL}};
const Segment SGI130262[] = {{&S23515[30], 6}, {NULL}};
const Segment SGI130263[] = {{&S23515[31], 3}, {NULL}};
const Segment SGI130264[] = {{&S23515[84], 6}, {NULL}};
const Segment SGI130265[] = {{&S13330[44], 9}, {NULL}};
const Segment SGI130266[] = {{&S13330[44], 21}, {NULL}};
const Segment SGI130267[] = {{&S13330[30], 6}, {NULL}};
const Segment SGI130268[] = {{&S13330[114], 3}, {NULL}};
const Segment SGI130269[] = {{&S13330[79], 6}, {NULL}};
const Segment SGI130270[] = {{&S14145[44], 10}, {NULL}};
const Segment SGI130271[] = {{&S14145[44], 22}, {NULL}};
const Segment SGI130272[] = {{&S14145[30], 6}, {NULL}};
const Segment SGI130273[] = {{&S14145[115], 3}, {NULL}};
const Segment SGI130274[] = {{&S14145[80], 6}, {NULL}};
const Segment SGI130275[] = {{&S13332[44], 9}, {NULL}};
const Segment SGI130276[] = {{&S13332[44], 21}, {NULL}};
const Segment SGI130277[] = {{&S13332[30], 6}, {NULL}};
const Segment SGI130278[] = {{&S13332[114], 3}, {NULL}};
const Segment SGI130279[] = {{&S13332[79], 6}, {NULL}};
const Segment SGI130280[] = {{&S14147[44], 13}, {NULL}};
const Segment SGI130281[] = {{&S14147[44], 25}, {NULL}};
const Segment SGI130282[] = {{&S14147[30], 6}, {NULL}};
const Segment SGI130283[] = {{&S14147[118], 3}, {NULL}};
const Segment SGI130284[] = {{&S14147[83], 6}, {NULL}};
const Segment SGI130285[] = {{&S14148[44], 11}, {NULL}};
const Segment SGI130286[] = {{&S14148[44], 23}, {NULL}};
const Segment SGI130287[] = {{&S14148[30], 6}, {NULL}};
const Segment SGI130288[] = {{&S14148[116], 3}, {NULL}};
const Segment SGI130289[] = {{&S14148[81], 6}, {NULL}};
const Segment SGI130290[] = {{&S23523[38], 14}, {NULL}};
const Segment SGI130291[] = {{&S23523[31], 6}, {NULL}};
const Segment SGI130292[] = {{&S14149[44], 8}, {NULL}};
const Segment SGI130293[] = {{&S14149[44], 20}, {NULL}};
const Segment SGI130294[] = {{&S14149[30], 6}, {NULL}};
const Segment SGI130295[] = {{&S14149[113], 3}, {NULL}};
const Segment SGI130296[] = {{&S14149[78], 6}, {NULL}};
const Segment SGI130297[] = {{&S14150[44], 12}, {NULL}};
const Segment SGI130298[] = {{&S14150[44], 24}, {NULL}};
const Segment SGI130299[] = {{&S14150[30], 6}, {NULL}};
const Segment SGI130300[] = {{&S14150[117], 3}, {NULL}};
const Segment SGI130301[] = {{&S14150[82], 6}, {NULL}};
const Segment SGI130302[] = {{&S14152[44], 8}, {NULL}};
const Segment SGI130303[] = {{&S14152[44], 20}, {NULL}};
const Segment SGI130304[] = {{&S14152[30], 6}, {NULL}};
const Segment SGI130305[] = {{&S14152[113], 3}, {NULL}};
const Segment SGI130306[] = {{&S14152[78], 6}, {NULL}};
const Segment SGI130307[] = {{&S13343[44], 10}, {NULL}};
const Segment SGI130308[] = {{&S13343[44], 22}, {NULL}};
const Segment SGI130309[] = {{&S13343[30], 6}, {NULL}};
const Segment SGI130310[] = {{&S13343[115], 3}, {NULL}};
const Segment SGI130311[] = {{&S13343[80], 6}, {NULL}};
const Segment SGI130312[] = {{&S14153[44], 18}, {NULL}};
const Segment SGI130313[] = {{&S14153[44], 30}, {NULL}};
const Segment SGI130314[] = {{&S14153[30], 6}, {NULL}};
const Segment SGI130315[] = {{&S14153[123], 3}, {NULL}};
const Segment SGI130316[] = {{&S14153[88], 6}, {NULL}};
const Segment SGI130317[] = {{&S13345[44], 8}, {NULL}};
const Segment SGI130318[] = {{&S13345[44], 20}, {NULL}};
const Segment SGI130319[] = {{&S13345[30], 6}, {NULL}};
const Segment SGI130320[] = {{&S13345[113], 3}, {NULL}};
const Segment SGI130321[] = {{&S14154[44], 84}, {NULL}};
const Segment SGI130322[] = {{&S13345[78], 6}, {NULL}};
const Segment SGI130323[] = {{&S14154[44], 105}, {NULL}};
const Segment SGI130324[] = {{&S14154[30], 6}, {NULL}};
const Segment SGI130325[] = {{&S14154[196], 3}, {NULL}};
const Segment SGI130326[] = {{&S14154[122], 5}, {NULL}};
const Segment SGI130327[] = {{&S13346[44], 11}, {NULL}};
const Segment SGI130328[] = {{&S13346[44], 23}, {NULL}};
const Segment SGI130329[] = {{&S13346[30], 6}, {NULL}};
const Segment SGI130330[] = {{&S13346[116], 3}, {NULL}};
const Segment SGI130331[] = {{&S13346[81], 6}, {NULL}};
const Segment SGI130332[] = {{&S14155[44], 19}, {NULL}};
const Segment SGI130333[] = {{&S14155[44], 38}, {NULL}};
const Segment SGI130334[] = {{&S14155[30], 6}, {NULL}};
const Segment SGI130335[] = {{&S14155[131], 3}, {NULL}};
const Segment SGI130336[] = {{&S14155[96], 6}, {NULL}};
const Segment SGI130337[] = {{&S13348[44], 8}, {NULL}};
const Segment SGI130338[] = {{&S13348[44], 20}, {NULL}};
const Segment SGI130339[] = {{&S13348[30], 6}, {NULL}};
const Segment SGI130340[] = {{&S14156[44], 14}, {NULL}};
const Segment SGI130341[] = {{&S13348[113], 3}, {NULL}};
const Segment SGI130342[] = {{&S14156[44], 26}, {NULL}};
const Segment SGI130343[] = {{&S14156[30], 6}, {NULL}};
const Segment SGI130344[] = {{&S13348[78], 6}, {NULL}};
const Segment SGI130345[] = {{&S14156[119], 3}, {NULL}};
const Segment SGI130346[] = {{&S14156[84], 6}, {NULL}};
const Segment SGI130347[] = {{&S14157[44], 10}, {NULL}};
const Segment SGI130348[] = {{&S14157[44], 22}, {NULL}};
const Segment SGI130349[] = {{&S14157[30], 6}, {NULL}};
const Segment SGI130350[] = {{&S14157[115], 3}, {NULL}};
const Segment SGI130351[] = {{&S13350[44], 10}, {NULL}};
const Segment SGI130352[] = {{&S13350[44], 22}, {NULL}};
const Segment SGI130353[] = {{&S14157[80], 6}, {NULL}};
const Segment SGI130354[] = {{&S13350[30], 6}, {NULL}};
const Segment SGI130355[] = {{&S13350[115], 3}, {NULL}};
const Segment SGI130356[] = {{&S13350[80], 6}, {NULL}};
const Segment SGI130357[] = {{&S14158[44], 7}, {NULL}};
const Segment SGI130358[] = {{&S14158[44], 28}, {NULL}};
const Segment SGI130359[] = {{&S14158[30], 6}, {NULL}};
const Segment SGI130360[] = {{&S14158[120], 3}, {NULL}};
const Segment SGI130361[] = {{&S13351[44], 7}, {NULL}};
const Segment SGI130362[] = {{&S13351[44], 19}, {NULL}};
const Segment SGI130363[] = {{&S14158[86], 5}, {NULL}};
const Segment SGI130364[] = {{&S13351[30], 6}, {NULL}};
const Segment SGI130365[] = {{&S13351[112], 3}, {NULL}};
const Segment SGI130366[] = {{&S13351[77], 6}, {NULL}};
const Segment SGI130367[] = {{&S14159[44], 7}, {NULL}};
const Segment SGI130368[] = {{&S14159[44], 19}, {NULL}};
const Segment SGI130369[] = {{&S14159[30], 6}, {NULL}};
const Segment SGI130370[] = {{&S14159[112], 3}, {NULL}};
const Segment SGI130371[] = {{&S14159[77], 6}, {NULL}};
const Segment SGI130372[] = {{&S13355[44], 8}, {NULL}};
const Segment SGI130373[] = {{&S13355[44], 20}, {NULL}};
const Segment SGI130374[] = {{&S13355[30], 6}, {NULL}};
const Segment SGI130375[] = {{&S13355[113], 3}, {NULL}};
const Segment SGI130376[] = {{&S14163[44], 10}, {NULL}};
const Segment SGI130377[] = {{&S13355[78], 6}, {NULL}};
const Segment SGI130378[] = {{&S14163[44], 22}, {NULL}};
const Segment SGI130379[] = {{&S14163[30], 6}, {NULL}};
const Segment SGI130380[] = {{&S14163[115], 3}, {NULL}};
const Segment SGI130381[] = {{&S14163[80], 6}, {NULL}};
const Segment SGI130382[] = {{&S14165[44], 11}, {NULL}};
const Segment SGI130383[] = {{&S13358[44], 11}, {NULL}};
const Segment SGI130384[] = {{&S14165[44], 30}, {NULL}};
const Segment SGI130385[] = {{&S13358[44], 23}, {NULL}};
const Segment SGI130386[] = {{&S14165[30], 6}, {NULL}};
const Segment SGI130387[] = {{&S13358[30], 6}, {NULL}};
const Segment SGI130388[] = {{&S14165[123], 3}, {NULL}};
const Segment SGI130389[] = {{&S13358[116], 3}, {NULL}};
const Segment SGI130390[] = {{&S14165[88], 6}, {NULL}};
const Segment SGI130391[] = {{&S13358[81], 6}, {NULL}};
const Segment SGI130392[] = {{&S14166[44], 7}, {NULL}};
const Segment SGI130393[] = {{&S14166[44], 19}, {NULL}};
const Segment SGI130394[] = {{&S14166[30], 6}, {NULL}};
const Segment SGI130395[] = {{&S14166[112], 3}, {NULL}};
const Segment SGI130396[] = {{&S14166[77], 6}, {NULL}};
const Segment SGI130397[] = {{&S13361[44], 7}, {NULL}};
const Segment SGI130398[] = {{&S13361[44], 19}, {NULL}};
const Segment SGI130399[] = {{&S13361[30], 6}, {NULL}};
const Segment SGI130400[] = {{&S14167[44], 13}, {NULL}};
const Segment SGI130401[] = {{&S14167[44], 25}, {NULL}};
const Segment SGI130402[] = {{&S14167[30], 6}, {NULL}};
const Segment SGI130403[] = {{&S13361[112], 3}, {NULL}};
const Segment SGI130404[] = {{&S13361[77], 6}, {NULL}};
const Segment SGI130405[] = {{&S14167[118], 3}, {NULL}};
const Segment SGI130406[] = {{&S14167[83], 6}, {NULL}};
const Segment SGI130407[] = {{&S14168[44], 14}, {NULL}};
const Segment SGI130408[] = {{&S14168[44], 26}, {NULL}};
const Segment SGI130409[] = {{&S14168[30], 6}, {NULL}};
const Segment SGI130410[] = {{&S14168[119], 3}, {NULL}};
const Segment SGI130411[] = {{&S14168[84], 6}, {NULL}};
const Segment SGI130412[] = {{&S14170[44], 6}, {NULL}};
const Segment SGI130413[] = {{&S13365[44], 31}, {NULL}};
const Segment SGI130414[] = {{&S14170[44], 18}, {NULL}};
const Segment SGI130415[] = {{&S13365[30], 6}, {NULL}};
const Segment SGI130416[] = {{&S14170[30], 6}, {NULL}};
const Segment SGI130417[] = {{&S14170[111], 3}, {NULL}};
const Segment SGI130418[] = {{&S14170[76], 6}, {NULL}};
const Segment SGI130419[] = {{&S13367[44], 12}, {NULL}};
const Segment SGI130420[] = {{&S14172[44], 6}, {NULL}};
const Segment SGI130421[] = {{&S13367[44], 24}, {NULL}};
const Segment SGI130422[] = {{&S14172[44], 18}, {NULL}};
const Segment SGI130423[] = {{&S13367[30], 6}, {NULL}};
const Segment SGI130424[] = {{&S14172[30], 6}, {NULL}};
const Segment SGI130425[] = {{&S13367[117], 3}, {NULL}};
const Segment SGI130426[] = {{&S14172[111], 3}, {NULL}};
const Segment SGI130427[] = {{&S13367[82], 6}, {NULL}};
const Segment SGI130428[] = {{&S14172[76], 6}, {NULL}};
const Segment SGI130429[] = {{&S14174[44], 6}, {NULL}};
const Segment SGI130430[] = {{&S14174[44], 18}, {NULL}};
const Segment SGI130431[] = {{&S14174[30], 6}, {NULL}};
const Segment SGI130432[] = {{&S14174[111], 3}, {NULL}};
const Segment SGI130433[] = {{&S14174[76], 6}, {NULL}};
const Segment SGI130434[] = {{&S14176[44], 9}, {NULL}};
const Segment SGI130435[] = {{&S14176[44], 21}, {NULL}};
const Segment SGI130436[] = {{&S14176[30], 6}, {NULL}};
const Segment SGI130437[] = {{&S14176[114], 3}, {NULL}};
const Segment SGI130438[] = {{&S14176[79], 6}, {NULL}};
const Segment SGI130439[] = {{&S14180[44], 7}, {NULL}};
const Segment SGI130440[] = {{&S14180[44], 19}, {NULL}};
const Segment SGI130441[] = {{&S14180[30], 6}, {NULL}};
const Segment SGI130442[] = {{&S14180[112], 3}, {NULL}};
const Segment SGI130443[] = {{&S14180[77], 6}, {NULL}};
const Segment SGI130444[] = {{&S14181[44], 6}, {NULL}};
const Segment SGI130445[] = {{&S14181[44], 18}, {NULL}};
const Segment SGI130446[] = {{&S14181[30], 6}, {NULL}};
const Segment SGI130447[] = {{&S14181[111], 3}, {NULL}};
const Segment SGI130448[] = {{&S14181[76], 6}, {NULL}};
const Segment SGI130449[] = {{&S14182[44], 6}, {NULL}};
const Segment SGI130450[] = {{&S14182[44], 18}, {NULL}};
const Segment SGI130451[] = {{&S14182[30], 6}, {NULL}};
const Segment SGI130452[] = {{&S14182[111], 3}, {NULL}};
const Segment SGI130453[] = {{&S14182[76], 6}, {NULL}};
const Segment SGI130454[] = {{&S14183[44], 84}, {NULL}};
const Segment SGI130455[] = {{&S14183[44], 105}, {NULL}};
const Segment SGI130456[] = {{&S14183[30], 6}, {NULL}};
const Segment SGI130457[] = {{&S14183[196], 3}, {NULL}};
const Segment SGI130458[] = {{&S14183[122], 5}, {NULL}};
const Segment SGI130459[] = {{&S14187[44], 11}, {NULL}};
const Segment SGI130460[] = {{&S14187[44], 23}, {NULL}};
const Segment SGI130461[] = {{&S14187[30], 6}, {NULL}};
const Segment SGI130462[] = {{&S14187[116], 3}, {NULL}};
const Segment SGI130463[] = {{&S14187[81], 6}, {NULL}};
const Segment SGI130464[] = {{&S14188[44], 13}, {NULL}};
const Segment SGI130465[] = {{&S14188[44], 25}, {NULL}};
const Segment SGI130466[] = {{&S14188[30], 6}, {NULL}};
const Segment SGI130467[] = {{&S14188[118], 3}, {NULL}};
const Segment SGI130468[] = {{&S14188[83], 6}, {NULL}};
const Segment SGI130469[] = {{&S14190[44], 4}, {NULL}};
const Segment SGI130470[] = {{&S14190[44], 16}, {NULL}};
const Segment SGI130471[] = {{&S14190[30], 6}, {NULL}};
const Segment SGI130472[] = {{&S14190[109], 3}, {NULL}};
const Segment SGI130473[] = {{&S14190[74], 6}, {NULL}};
const Segment SGI130474[] = {{&S14191[44], 7}, {NULL}};
const Segment SGI130475[] = {{&S14191[44], 19}, {NULL}};
const Segment SGI130476[] = {{&S14191[30], 6}, {NULL}};
const Segment SGI130477[] = {{&S14191[112], 3}, {NULL}};
const Segment SGI130478[] = {{&S14191[77], 6}, {NULL}};
const Segment SGI130479[] = {{&S13398[44], 14}, {NULL}};
const Segment SGI130480[] = {{&S13398[44], 26}, {NULL}};
const Segment SGI130481[] = {{&S13398[30], 6}, {NULL}};
const Segment SGI130482[] = {{&S13398[119], 3}, {NULL}};
const Segment SGI130483[] = {{&S13398[84], 6}, {NULL}};
const Segment SGI130484[] = {{&S14192[44], 14}, {NULL}};
const Segment SGI130485[] = {{&S14192[44], 33}, {NULL}};
const Segment SGI130486[] = {{&S14192[30], 6}, {NULL}};
const Segment SGI130487[] = {{&S14192[126], 3}, {NULL}};
const Segment SGI130488[] = {{&S14192[91], 6}, {NULL}};
const Segment SGI130489[] = {{&S14193[44], 11}, {NULL}};
const Segment SGI130490[] = {{&S14193[44], 23}, {NULL}};
const Segment SGI130491[] = {{&S14193[30], 6}, {NULL}};
const Segment SGI130492[] = {{&S14193[116], 3}, {NULL}};
const Segment SGI130493[] = {{&S14193[81], 6}, {NULL}};
const Segment SGI130494[] = {{&S13401[44], 8}, {NULL}};
const Segment SGI130495[] = {{&S13401[44], 20}, {NULL}};
const Segment SGI130496[] = {{&S13401[30], 6}, {NULL}};
const Segment SGI130497[] = {{&S13401[112], 3}, {NULL}};
const Segment SGI130498[] = {{&S13401[78], 5}, {NULL}};
const Segment SGI130499[] = {{&S14194[44], 19}, {NULL}};
const Segment SGI130500[] = {{&S14194[44], 31}, {NULL}};
const Segment SGI130501[] = {{&S14194[30], 6}, {NULL}};
const Segment SGI130502[] = {{&S14194[124], 3}, {NULL}};
const Segment SGI130503[] = {{&S14194[89], 6}, {NULL}};
const Segment SGI130504[] = {{&S14197[44], 16}, {NULL}};
const Segment SGI130505[] = {{&S14197[44], 30}, {NULL}};
const Segment SGI130506[] = {{&S14197[30], 6}, {NULL}};
const Segment SGI130507[] = {{&S14197[123], 3}, {NULL}};
const Segment SGI130508[] = {{&S14197[88], 6}, {NULL}};
const Segment SGI130509[] = {{&S14199[44], 10}, {NULL}};
const Segment SGI130510[] = {{&S14199[44], 22}, {NULL}};
const Segment SGI130511[] = {{&S14199[30], 6}, {NULL}};
const Segment SGI130512[] = {{&S14199[31], 3}, {NULL}};
const Segment SGI130513[] = {{&S14199[80], 6}, {NULL}};
const Segment SGI130514[] = {{&S14201[44], 23}, {NULL}};
const Segment SGI130515[] = {{&S14201[44], 35}, {NULL}};
const Segment SGI130516[] = {{&S14201[30], 6}, {NULL}};
const Segment SGI130517[] = {{&S14201[128], 3}, {NULL}};
const Segment SGI130518[] = {{&S14201[93], 6}, {NULL}};
const Segment SGI130519[] = {{&S14202[44], 8}, {NULL}};
const Segment SGI130520[] = {{&S14202[44], 20}, {NULL}};
const Segment SGI130521[] = {{&S14202[30], 6}, {NULL}};
const Segment SGI130522[] = {{&S14202[113], 3}, {NULL}};
const Segment SGI130523[] = {{&S14202[78], 6}, {NULL}};
const Segment SGI130524[] = {{&S14207[44], 7}, {NULL}};
const Segment SGI130525[] = {{&S14207[44], 19}, {NULL}};
const Segment SGI130526[] = {{&S14207[30], 6}, {NULL}};
const Segment SGI130527[] = {{&S14207[112], 3}, {NULL}};
const Segment SGI130528[] = {{&S14207[77], 6}, {NULL}};
const Segment SGI130529[] = {{&S14209[44], 7}, {NULL}};
const Segment SGI130530[] = {{&S14209[44], 19}, {NULL}};
const Segment SGI130531[] = {{&S14209[30], 6}, {NULL}};
const Segment SGI130532[] = {{&S14209[112], 3}, {NULL}};
const Segment SGI130533[] = {{&S14209[77], 6}, {NULL}};
const Segment SGI130534[] = {{&S14210[44], 9}, {NULL}};
const Segment SGI130535[] = {{&S14210[44], 21}, {NULL}};
const Segment SGI130536[] = {{&S14210[30], 6}, {NULL}};
const Segment SGI130537[] = {{&S14210[114], 3}, {NULL}};
const Segment SGI130538[] = {{&S14210[79], 6}, {NULL}};
const Segment SGI130539[] = {{&S13427[44], 7}, {NULL}};
const Segment SGI130540[] = {{&S13427[44], 19}, {NULL}};
const Segment SGI130541[] = {{&S13427[30], 6}, {NULL}};
const Segment SGI130542[] = {{&S13427[112], 3}, {NULL}};
const Segment SGI130543[] = {{&S13427[77], 6}, {NULL}};
const Segment SGI130544[] = {{&S14212[44], 11}, {NULL}};
const Segment SGI130545[] = {{&S14212[44], 23}, {NULL}};
const Segment SGI130546[] = {{&S14212[30], 6}, {NULL}};
const Segment SGI130547[] = {{&S14212[116], 3}, {NULL}};
const Segment SGI130548[] = {{&S14212[81], 6}, {NULL}};
const Segment SGI130549[] = {{&S13429[44], 9}, {NULL}};
const Segment SGI130550[] = {{&S13429[44], 21}, {NULL}};
const Segment SGI130551[] = {{&S13429[30], 6}, {NULL}};
const Segment SGI130552[] = {{&S13429[114], 3}, {NULL}};
const Segment SGI130553[] = {{&S14213[44], 4}, {NULL}};
const Segment SGI130554[] = {{&S14213[44], 16}, {NULL}};
const Segment SGI130555[] = {{&S13429[79], 6}, {NULL}};
const Segment SGI130556[] = {{&S14213[30], 6}, {NULL}};
const Segment SGI130557[] = {{&S14213[109], 3}, {NULL}};
const Segment SGI130558[] = {{&S14213[74], 6}, {NULL}};
const Segment SGI130559[] = {{&S14215[44], 7}, {NULL}};
const Segment SGI130560[] = {{&S14215[44], 19}, {NULL}};
const Segment SGI130561[] = {{&S14215[30], 6}, {NULL}};
const Segment SGI130562[] = {{&S14215[112], 3}, {NULL}};
const Segment SGI130563[] = {{&S14215[77], 6}, {NULL}};
const Segment SGI130564[] = {{&S14216[44], 12}, {NULL}};
const Segment SGI130565[] = {{&S14216[30], 6}, {NULL}};
const Segment SGI130566[] = {{&S14216[119], 3}, {NULL}};
const Segment SGI130567[] = {{&S14216[84], 6}, {NULL}};
const Segment SGI130568[] = {{&S14217[44], 9}, {NULL}};
const Segment SGI130569[] = {{&S14217[44], 22}, {NULL}};
const Segment SGI130570[] = {{&S14217[30], 6}, {NULL}};
const Segment SGI130571[] = {{&S14217[115], 3}, {NULL}};
const Segment SGI130572[] = {{&S14217[80], 6}, {NULL}};
const Segment SGI130573[] = {{&S14218[44], 22}, {NULL}};
const Segment SGI130574[] = {{&S14218[44], 34}, {NULL}};
const Segment SGI130575[] = {{&S13438[44], 20}, {NULL}};
const Segment SGI130576[] = {{&S14218[30], 6}, {NULL}};
const Segment SGI130577[] = {{&S13438[44], 32}, {NULL}};
const Segment SGI130578[] = {{&S13438[30], 6}, {NULL}};
const Segment SGI130579[] = {{&S14218[127], 3}, {NULL}};
const Segment SGI130580[] = {{&S13438[125], 3}, {NULL}};
const Segment SGI130581[] = {{&S14218[92], 6}, {NULL}};
const Segment SGI130582[] = {{&S13438[90], 6}, {NULL}};
const Segment SGI130583[] = {{&S14219[44], 14}, {NULL}};
const Segment SGI130584[] = {{&S14219[44], 26}, {NULL}};
const Segment SGI130585[] = {{&S14219[30], 6}, {NULL}};
const Segment SGI130586[] = {{&S14219[119], 3}, {NULL}};
const Segment SGI130587[] = {{&S14219[84], 6}, {NULL}};
const Segment SGI130588[] = {{&S14220[44], 26}, {NULL}};
const Segment SGI130589[] = {{&S14220[44], 38}, {NULL}};
const Segment SGI130590[] = {{&S14220[30], 6}, {NULL}};
const Segment SGI130591[] = {{&S14220[131], 3}, {NULL}};
const Segment SGI130592[] = {{&S14220[96], 6}, {NULL}};
const Segment SGI130593[] = {{&S14221[44], 14}, {NULL}};
const Segment SGI130594[] = {{&S14221[44], 26}, {NULL}};
const Segment SGI130595[] = {{&S14221[30], 6}, {NULL}};
const Segment SGI130596[] = {{&S14221[119], 3}, {NULL}};
const Segment SGI130597[] = {{&S14221[84], 6}, {NULL}};
const Segment SGI130598[] = {{&S13445[44], 6}, {NULL}};
const Segment SGI130599[] = {{&S13445[44], 18}, {NULL}};
const Segment SGI130600[] = {{&S13445[30], 6}, {NULL}};
const Segment SGI130601[] = {{&S13445[111], 3}, {NULL}};
const Segment SGI130602[] = {{&S13445[76], 6}, {NULL}};
const Segment SGI130603[] = {{&S14223[44], 5}, {NULL}};
const Segment SGI130604[] = {{&S14223[44], 30}, {NULL}};
const Segment SGI130605[] = {{&S14223[30], 6}, {NULL}};
const Segment SGI130606[] = {{&S14223[123], 3}, {NULL}};
const Segment SGI130607[] = {{&S14223[88], 6}, {NULL}};
const Segment SGI130608[] = {{&S13447[44], 6}, {NULL}};
const Segment SGI130609[] = {{&S13447[44], 24}, {NULL}};
const Segment SGI130610[] = {{&S13447[30], 6}, {NULL}};
const Segment SGI130611[] = {{&S13447[117], 3}, {NULL}};
const Segment SGI130612[] = {{&S13447[82], 6}, {NULL}};
const Segment SGI130613[] = {{&S14224[44], 23}, {NULL}};
const Segment SGI130614[] = {{&S14224[44], 35}, {NULL}};
const Segment SGI130615[] = {{&S14224[30], 6}, {NULL}};
const Segment SGI130616[] = {{&S14224[128], 3}, {NULL}};
const Segment SGI130617[] = {{&S14224[93], 6}, {NULL}};
const Segment SGI130618[] = {{&S13448[44], 6}, {NULL}};
const Segment SGI130619[] = {{&S13448[44], 18}, {NULL}};
const Segment SGI130620[] = {{&S13448[30], 6}, {NULL}};
const Segment SGI130621[] = {{&S13448[110], 3}, {NULL}};
const Segment SGI130622[] = {{&S13448[76], 5}, {NULL}};
const Segment SGI130623[] = {{&S14225[44], 14}, {NULL}};
const Segment SGI130624[] = {{&S14225[44], 35}, {NULL}};
const Segment SGI130625[] = {{&S14225[30], 6}, {NULL}};
const Segment SGI130626[] = {{&S14225[128], 3}, {NULL}};
const Segment SGI130627[] = {{&S14225[93], 6}, {NULL}};
const Segment SGI130628[] = {{&S14226[44], 12}, {NULL}};
const Segment SGI130629[] = {{&S14226[44], 24}, {NULL}};
const Segment SGI130630[] = {{&S14226[30], 6}, {NULL}};
const Segment SGI130631[] = {{&S14226[117], 3}, {NULL}};
const Segment SGI130632[] = {{&S14226[82], 6}, {NULL}};
const Segment SGI130633[] = {{&S14227[44], 12}, {NULL}};
const Segment SGI130634[] = {{&S14227[44], 24}, {NULL}};
const Segment SGI130635[] = {{&S14227[30], 6}, {NULL}};
const Segment SGI130636[] = {{&S14227[117], 3}, {NULL}};
const Segment SGI130637[] = {{&S14227[82], 6}, {NULL}};
const Segment SGI130638[] = {{&S13455[44], 6}, {NULL}};
const Segment SGI130639[] = {{&S13455[44], 18}, {NULL}};
const Segment SGI130640[] = {{&S13455[30], 6}, {NULL}};
const Segment SGI130641[] = {{&S13455[111], 3}, {NULL}};
const Segment SGI130642[] = {{&S13455[76], 6}, {NULL}};
const Segment SGI130643[] = {{&S14230[44], 17}, {NULL}};
const Segment SGI130644[] = {{&S14230[44], 29}, {NULL}};
const Segment SGI130645[] = {{&S14230[30], 6}, {NULL}};
const Segment SGI130646[] = {{&S14230[122], 3}, {NULL}};
const Segment SGI130647[] = {{&S14230[87], 6}, {NULL}};
const Segment SGI130648[] = {{&S13456[44], 5}, {NULL}};
const Segment SGI130649[] = {{&S13456[44], 17}, {NULL}};
const Segment SGI130650[] = {{&S13456[30], 6}, {NULL}};
const Segment SGI130651[] = {{&S13456[110], 3}, {NULL}};
const Segment SGI130652[] = {{&S14231[44], 11}, {NULL}};
const Segment SGI130653[] = {{&S14231[44], 18}, {NULL}};
const Segment SGI130654[] = {{&S13456[75], 6}, {NULL}};
const Segment SGI130655[] = {{&S14231[30], 6}, {NULL}};
const Segment SGI130656[] = {{&S14231[111], 3}, {NULL}};
const Segment SGI130657[] = {{&S14231[76], 6}, {NULL}};
const Segment SGI130658[] = {{&S14232[44], 13}, {NULL}};
const Segment SGI130659[] = {{&S14232[44], 25}, {NULL}};
const Segment SGI130660[] = {{&S14232[30], 6}, {NULL}};
const Segment SGI130661[] = {{&S14232[118], 3}, {NULL}};
const Segment SGI130662[] = {{&S13458[44], 16}, {NULL}};
const Segment SGI130663[] = {{&S14232[83], 6}, {NULL}};
const Segment SGI130664[] = {{&S13458[44], 28}, {NULL}};
const Segment SGI130665[] = {{&S13458[30], 6}, {NULL}};
const Segment SGI130666[] = {{&S13458[121], 3}, {NULL}};
const Segment SGI130667[] = {{&S13458[86], 6}, {NULL}};
const Segment SGI130668[] = {{&S14234[44], 8}, {NULL}};
const Segment SGI130669[] = {{&S14234[44], 20}, {NULL}};
const Segment SGI130670[] = {{&S14234[30], 6}, {NULL}};
const Segment SGI130671[] = {{&S14234[113], 3}, {NULL}};
const Segment SGI130672[] = {{&S14234[78], 6}, {NULL}};
const Segment SGI130673[] = {{&S14235[44], 10}, {NULL}};
const Segment SGI130674[] = {{&S14235[44], 22}, {NULL}};
const Segment SGI130675[] = {{&S14235[30], 6}, {NULL}};
const Segment SGI130676[] = {{&S14235[115], 3}, {NULL}};
const Segment SGI130677[] = {{&S14235[80], 6}, {NULL}};
const Segment SGI130678[] = {{&S13464[44], 9}, {NULL}};
const Segment SGI130679[] = {{&S13464[44], 21}, {NULL}};
const Segment SGI130680[] = {{&S13464[30], 6}, {NULL}};
const Segment SGI130681[] = {{&S13464[114], 3}, {NULL}};
const Segment SGI130682[] = {{&S13464[79], 6}, {NULL}};
const Segment SGI130683[] = {{&S14238[44], 10}, {NULL}};
const Segment SGI130684[] = {{&S14238[44], 22}, {NULL}};
const Segment SGI130685[] = {{&S14238[30], 6}, {NULL}};
const Segment SGI130686[] = {{&S14238[115], 3}, {NULL}};
const Segment SGI130687[] = {{&S14238[80], 6}, {NULL}};
const Segment SGI130688[] = {{&S13466[44], 14}, {NULL}};
const Segment SGI130689[] = {{&S13466[44], 26}, {NULL}};
const Segment SGI130690[] = {{&S13466[30], 6}, {NULL}};
const Segment SGI130691[] = {{&S13466[119], 3}, {NULL}};
const Segment SGI130692[] = {{&S13466[84], 6}, {NULL}};
const Segment SGI130693[] = {{&S14241[44], 13}, {NULL}};
const Segment SGI130694[] = {{&S14241[44], 31}, {NULL}};
const Segment SGI130695[] = {{&S14241[30], 6}, {NULL}};
const Segment SGI130696[] = {{&S14241[123], 3}, {NULL}};
const Segment SGI130697[] = {{&S14241[89], 5}, {NULL}};
const Segment SGI130698[] = {{&S14243[44], 7}, {NULL}};
const Segment SGI130699[] = {{&S14243[44], 19}, {NULL}};
const Segment SGI130700[] = {{&S14243[30], 6}, {NULL}};
const Segment SGI130701[] = {{&S14243[112], 3}, {NULL}};
const Segment SGI130702[] = {{&S14243[77], 6}, {NULL}};
const Segment SGI130703[] = {{&S14244[44], 26}, {NULL}};
const Segment SGI130704[] = {{&S14244[44], 38}, {NULL}};
const Segment SGI130705[] = {{&S14244[30], 6}, {NULL}};
const Segment SGI130706[] = {{&S14244[131], 3}, {NULL}};
const Segment SGI130707[] = {{&S14244[96], 6}, {NULL}};
const Segment SGI130708[] = {{&S14245[44], 5}, {NULL}};
const Segment SGI130709[] = {{&S14245[44], 17}, {NULL}};
const Segment SGI130710[] = {{&S14245[30], 6}, {NULL}};
const Segment SGI130711[] = {{&S14245[110], 3}, {NULL}};
const Segment SGI130712[] = {{&S14245[75], 6}, {NULL}};
const Segment SGI130713[] = {{&S23680[44], 19}, {NULL}};
const Segment SGI130714[] = {{&S23680[44], 31}, {NULL}};
const Segment SGI130715[] = {{&S23680[30], 6}, {NULL}};
const Segment SGI130716[] = {{&S23680[31], 3}, {NULL}};
const Segment SGI130717[] = {{&S23680[89], 6}, {NULL}};
const Segment SGI130718[] = {{&S14250[44], 8}, {NULL}};
const Segment SGI130719[] = {{&S14250[44], 20}, {NULL}};
const Segment SGI130720[] = {{&S14250[30], 6}, {NULL}};
const Segment SGI130721[] = {{&S14250[112], 3}, {NULL}};
const Segment SGI130722[] = {{&S14250[78], 5}, {NULL}};
const Segment SGI130723[] = {{&S23682[44], 19}, {NULL}};
const Segment SGI130724[] = {{&S23682[44], 31}, {NULL}};
const Segment SGI130725[] = {{&S23682[30], 6}, {NULL}};
const Segment SGI130726[] = {{&S23682[31], 3}, {NULL}};
const Segment SGI130727[] = {{&S23682[89], 6}, {NULL}};
const Segment SGI130728[] = {{&S14252[44], 10}, {NULL}};
const Segment SGI130729[] = {{&S14252[44], 22}, {NULL}};
const Segment SGI130730[] = {{&S14252[30], 6}, {NULL}};
const Segment SGI130731[] = {{&S14252[115], 3}, {NULL}};
const Segment SGI130732[] = {{&S14252[80], 6}, {NULL}};
const Segment SGI130733[] = {{&S14253[44], 9}, {NULL}};
const Segment SGI130734[] = {{&S14253[44], 21}, {NULL}};
const Segment SGI130735[] = {{&S14253[30], 6}, {NULL}};
const Segment SGI130736[] = {{&S14253[114], 3}, {NULL}};
const Segment SGI130737[] = {{&S14253[79], 6}, {NULL}};
const Segment SGI130738[] = {{&S14254[44], 10}, {NULL}};
const Segment SGI130739[] = {{&S14254[44], 22}, {NULL}};
const Segment SGI130740[] = {{&S14254[30], 6}, {NULL}};
const Segment SGI130741[] = {{&S14254[115], 3}, {NULL}};
const Segment SGI130742[] = {{&S14254[80], 6}, {NULL}};
const Segment SGI130743[] = {{&S14255[44], 84}, {NULL}};
const Segment SGI130744[] = {{&S14255[44], 105}, {NULL}};
const Segment SGI130745[] = {{&S14255[30], 6}, {NULL}};
const Segment SGI130746[] = {{&S14255[196], 3}, {NULL}};
const Segment SGI130747[] = {{&S14255[122], 5}, {NULL}};
const Segment SGI130748[] = {{&S14258[44], 14}, {NULL}};
const Segment SGI130749[] = {{&S14258[44], 26}, {NULL}};
const Segment SGI130750[] = {{&S14258[30], 6}, {NULL}};
const Segment SGI130751[] = {{&S14258[119], 3}, {NULL}};
const Segment SGI130752[] = {{&S14258[84], 6}, {NULL}};
const Segment SGI130753[] = {{&S14260[44], 25}, {NULL}};
const Segment SGI130754[] = {{&S14260[44], 37}, {NULL}};
const Segment SGI130755[] = {{&S14260[30], 6}, {NULL}};
const Segment SGI130756[] = {{&S14260[130], 3}, {NULL}};
const Segment SGI130757[] = {{&S14260[95], 6}, {NULL}};
const Segment SGI130758[] = {{&S14262[44], 9}, {NULL}};
const Segment SGI130759[] = {{&S14262[44], 21}, {NULL}};
const Segment SGI130760[] = {{&S14262[30], 6}, {NULL}};
const Segment SGI130761[] = {{&S14262[114], 3}, {NULL}};
const Segment SGI130762[] = {{&S14262[79], 6}, {NULL}};
const Segment SGI130763[] = {{&S14264[44], 7}, {NULL}};
const Segment SGI130764[] = {{&S14264[44], 19}, {NULL}};
const Segment SGI130765[] = {{&S14264[30], 6}, {NULL}};
const Segment SGI130766[] = {{&S14264[112], 3}, {NULL}};
const Segment SGI130767[] = {{&S14264[77], 6}, {NULL}};
const Segment SGI130768[] = {{&S14265[44], 20}, {NULL}};
const Segment SGI130769[] = {{&S14265[44], 32}, {NULL}};
const Segment SGI130770[] = {{&S14265[30], 6}, {NULL}};
const Segment SGI130771[] = {{&S14265[125], 3}, {NULL}};
const Segment SGI130772[] = {{&S14265[90], 6}, {NULL}};
const Segment SGI130773[] = {{&S14266[44], 9}, {NULL}};
const Segment SGI130774[] = {{&S14266[44], 21}, {NULL}};
const Segment SGI130775[] = {{&S14266[30], 6}, {NULL}};
const Segment SGI130776[] = {{&S14266[114], 3}, {NULL}};
const Segment SGI130777[] = {{&S14266[79], 6}, {NULL}};
const Segment SGI130778[] = {{&S14269[44], 7}, {NULL}};
const Segment SGI130779[] = {{&S14269[44], 19}, {NULL}};
const Segment SGI130780[] = {{&S14269[30], 6}, {NULL}};
const Segment SGI130781[] = {{&S14269[111], 3}, {NULL}};
const Segment SGI130782[] = {{&S14269[77], 5}, {NULL}};
const Segment SGI130783[] = {{&S14270[44], 7}, {NULL}};
const Segment SGI130784[] = {{&S14270[44], 19}, {NULL}};
const Segment SGI130785[] = {{&S14270[30], 6}, {NULL}};
const Segment SGI130786[] = {{&S14270[112], 3}, {NULL}};
const Segment SGI130787[] = {{&S14270[77], 6}, {NULL}};
const Segment SGI130788[] = {{&S14271[44], 17}, {NULL}};
const Segment SGI130789[] = {{&S14271[44], 29}, {NULL}};
const Segment SGI130790[] = {{&S14271[30], 6}, {NULL}};
const Segment SGI130791[] = {{&S14271[122], 3}, {NULL}};
const Segment SGI130792[] = {{&S14271[87], 6}, {NULL}};
const Segment SGI130793[] = {{&S14272[44], 7}, {NULL}};
const Segment SGI130794[] = {{&S14272[44], 19}, {NULL}};
const Segment SGI130795[] = {{&S14272[30], 6}, {NULL}};
const Segment SGI130796[] = {{&S14272[112], 3}, {NULL}};
const Segment SGI130797[] = {{&S14272[77], 6}, {NULL}};
const Segment SGI130798[] = {{&S14273[44], 7}, {NULL}};
const Segment SGI130799[] = {{&S14273[44], 19}, {NULL}};
const Segment SGI130800[] = {{&S14273[30], 6}, {NULL}};
const Segment SGI130801[] = {{&S14273[112], 3}, {NULL}};
const Segment SGI130802[] = {{&S14273[77], 6}, {NULL}};
const Segment SGI130803[] = {{&S14274[44], 25}, {NULL}};
const Segment SGI130804[] = {{&S14274[44], 37}, {NULL}};
const Segment SGI130805[] = {{&S14274[30], 6}, {NULL}};
const Segment SGI130806[] = {{&S14274[130], 3}, {NULL}};
const Segment SGI130807[] = {{&S14274[95], 6}, {NULL}};
const Segment SGI130808[] = {{&S14275[44], 7}, {NULL}};
const Segment SGI130809[] = {{&S14275[44], 19}, {NULL}};
const Segment SGI130810[] = {{&S14275[30], 6}, {NULL}};
const Segment SGI130811[] = {{&S14275[112], 3}, {NULL}};
const Segment SGI130812[] = {{&S14275[77], 6}, {NULL}};
const Segment SGI130813[] = {{&S13476[44], 14}, {NULL}};
const Segment SGI130814[] = {{&S13476[44], 26}, {NULL}};
const Segment SGI130815[] = {{&S13476[30], 6}, {NULL}};
const Segment SGI130816[] = {{&S13476[119], 3}, {NULL}};
const Segment SGI130817[] = {{&S14276[44], 6}, {NULL}};
const Segment SGI130818[] = {{&S14276[44], 18}, {NULL}};
const Segment SGI130819[] = {{&S13476[84], 6}, {NULL}};
const Segment SGI130820[] = {{&S14276[30], 6}, {NULL}};
const Segment SGI130821[] = {{&S14276[111], 3}, {NULL}};
const Segment SGI130822[] = {{&S14276[76], 6}, {NULL}};
const Segment SGI130823[] = {{&S14277[44], 9}, {NULL}};
const Segment SGI130824[] = {{&S14277[44], 21}, {NULL}};
const Segment SGI130825[] = {{&S14277[30], 6}, {NULL}};
const Segment SGI130826[] = {{&S14277[114], 3}, {NULL}};
const Segment SGI130827[] = {{&S14277[79], 6}, {NULL}};
const Segment SGI130828[] = {{&S14279[44], 12}, {NULL}};
const Segment SGI130829[] = {{&S14279[44], 24}, {NULL}};
const Segment SGI130830[] = {{&S14279[30], 6}, {NULL}};
const Segment SGI130831[] = {{&S13481[44], 6}, {NULL}};
const Segment SGI130832[] = {{&S13481[44], 18}, {NULL}};
const Segment SGI130833[] = {{&S13481[30], 6}, {NULL}};
const Segment SGI130834[] = {{&S14279[117], 3}, {NULL}};
const Segment SGI130835[] = {{&S14279[82], 6}, {NULL}};
const Segment SGI130836[] = {{&S13481[111], 3}, {NULL}};
const Segment SGI130837[] = {{&S13481[76], 6}, {NULL}};
const Segment SGI130838[] = {{&S14280[44], 12}, {NULL}};
const Segment SGI130839[] = {{&S14280[44], 24}, {NULL}};
const Segment SGI130840[] = {{&S14280[30], 6}, {NULL}};
const Segment SGI130841[] = {{&S14280[117], 3}, {NULL}};
const Segment SGI130842[] = {{&S14280[82], 6}, {NULL}};
const Segment SGI130843[] = {{&S13483[44], 7}, {NULL}};
const Segment SGI130844[] = {{&S13483[44], 19}, {NULL}};
const Segment SGI130845[] = {{&S13483[30], 6}, {NULL}};
const Segment SGI130846[] = {{&S13483[112], 3}, {NULL}};
const Segment SGI130847[] = {{&S13483[77], 6}, {NULL}};
const Segment SGI130848[] = {{&S14281[54], 9}, {NULL}};
const Segment SGI130849[] = {{&S14281[47], 6}, {NULL}};
const Segment SGI130850[] = {{&S13484[44], 6}, {NULL}};
const Segment SGI130851[] = {{&S13484[44], 18}, {NULL}};
const Segment SGI130852[] = {{&S13484[30], 6}, {NULL}};
const Segment SGI130853[] = {{&S13484[111], 3}, {NULL}};
const Segment SGI130854[] = {{&S13484[76], 6}, {NULL}};
const Segment SGI130855[] = {{&S14283[44], 11}, {NULL}};
const Segment SGI130856[] = {{&S14283[44], 23}, {NULL}};
const Segment SGI130857[] = {{&S14283[30], 6}, {NULL}};
const Segment SGI130858[] = {{&S14283[116], 3}, {NULL}};
const Segment SGI130859[] = {{&S14283[81], 6}, {NULL}};
const Segment SGI130860[] = {{&S13488[44], 8}, {NULL}};
const Segment SGI130861[] = {{&S13488[44], 20}, {NULL}};
const Segment SGI130862[] = {{&S13488[30], 6}, {NULL}};
const Segment SGI130863[] = {{&S14285[44], 12}, {NULL}};
const Segment SGI130864[] = {{&S14285[44], 24}, {NULL}};
const Segment SGI130865[] = {{&S13488[113], 3}, {NULL}};
const Segment SGI130866[] = {{&S14285[30], 6}, {NULL}};
const Segment SGI130867[] = {{&S13488[78], 6}, {NULL}};
const Segment SGI130868[] = {{&S14285[117], 3}, {NULL}};
const Segment SGI130869[] = {{&S14285[82], 6}, {NULL}};
const Segment SGI130870[] = {{&S14287[44], 13}, {NULL}};
const Segment SGI130871[] = {{&S14287[44], 25}, {NULL}};
const Segment SGI130872[] = {{&S14287[30], 6}, {NULL}};
const Segment SGI130873[] = {{&S14287[118], 3}, {NULL}};
const Segment SGI130874[] = {{&S14287[83], 6}, {NULL}};
const Segment SGI130875[] = {{&S14289[44], 8}, {NULL}};
const Segment SGI130876[] = {{&S14289[44], 20}, {NULL}};
const Segment SGI130877[] = {{&S14289[30], 6}, {NULL}};
const Segment SGI130878[] = {{&S14289[113], 3}, {NULL}};
const Segment SGI130879[] = {{&S14289[78], 6}, {NULL}};
const Segment SGI130880[] = {{&S13496[44], 10}, {NULL}};
const Segment SGI130881[] = {{&S13496[44], 22}, {NULL}};
const Segment SGI130882[] = {{&S13496[30], 6}, {NULL}};
const Segment SGI130883[] = {{&S13496[115], 3}, {NULL}};
const Segment SGI130884[] = {{&S14290[44], 9}, {NULL}};
const Segment SGI130885[] = {{&S14290[44], 21}, {NULL}};
const Segment SGI130886[] = {{&S13496[80], 6}, {NULL}};
const Segment SGI130887[] = {{&S14290[30], 6}, {NULL}};
const Segment SGI130888[] = {{&S14290[114], 3}, {NULL}};
const Segment SGI130889[] = {{&S14290[79], 6}, {NULL}};
const Segment SGI130890[] = {{&S14291[44], 10}, {NULL}};
const Segment SGI130891[] = {{&S14291[44], 22}, {NULL}};
const Segment SGI130892[] = {{&S14291[30], 6}, {NULL}};
const Segment SGI130893[] = {{&S14291[115], 3}, {NULL}};
const Segment SGI130894[] = {{&S14291[80], 6}, {NULL}};
const Segment SGI130895[] = {{&S14293[44], 9}, {NULL}};
const Segment SGI130896[] = {{&S14293[44], 21}, {NULL}};
const Segment SGI130897[] = {{&S14293[30], 6}, {NULL}};
const Segment SGI130898[] = {{&S14293[114], 3}, {NULL}};
const Segment SGI130899[] = {{&S14293[79], 6}, {NULL}};
const Segment SGI130900[] = {{&S14295[44], 17}, {NULL}};
const Segment SGI130901[] = {{&S14295[44], 38}, {NULL}};
const Segment SGI130902[] = {{&S14295[30], 6}, {NULL}};
const Segment SGI130903[] = {{&S14295[131], 3}, {NULL}};
const Segment SGI130904[] = {{&S14295[96], 6}, {NULL}};
const Segment SGI130905[] = {{&S14296[44], 7}, {NULL}};
const Segment SGI130906[] = {{&S14296[44], 19}, {NULL}};
const Segment SGI130907[] = {{&S14296[30], 6}, {NULL}};
const Segment SGI130908[] = {{&S14296[112], 3}, {NULL}};
const Segment SGI130909[] = {{&S14296[77], 6}, {NULL}};
const Segment SGI130910[] = {{&S14299[44], 21}, {NULL}};
const Segment SGI130911[] = {{&S14299[44], 33}, {NULL}};
const Segment SGI130912[] = {{&S14299[30], 6}, {NULL}};
const Segment SGI130913[] = {{&S14299[126], 3}, {NULL}};
const Segment SGI130914[] = {{&S14299[91], 6}, {NULL}};
const Segment SGI130915[] = {{&S14303[44], 5}, {NULL}};
const Segment SGI130916[] = {{&S14303[44], 17}, {NULL}};
const Segment SGI130917[] = {{&S14303[30], 6}, {NULL}};
const Segment SGI130918[] = {{&S14303[110], 3}, {NULL}};
const Segment SGI130919[] = {{&S14303[75], 6}, {NULL}};
const Segment SGI130920[] = {{&S14304[44], 10}, {NULL}};
const Segment SGI130921[] = {{&S14304[44], 22}, {NULL}};
const Segment SGI130922[] = {{&S14304[30], 6}, {NULL}};
const Segment SGI130923[] = {{&S14304[115], 3}, {NULL}};
const Segment SGI130924[] = {{&S14304[80], 6}, {NULL}};
const Segment SGI130925[] = {{&S14306[44], 19}, {NULL}};
const Segment SGI130926[] = {{&S14306[44], 31}, {NULL}};
const Segment SGI130927[] = {{&S14306[30], 6}, {NULL}};
const Segment SGI130928[] = {{&S14306[124], 3}, {NULL}};
const Segment SGI130929[] = {{&S14306[89], 6}, {NULL}};
const Segment SGI130930[] = {{&S14308[44], 11}, {NULL}};
const Segment SGI130931[] = {{&S14308[44], 23}, {NULL}};
const Segment SGI130932[] = {{&S14308[30], 6}, {NULL}};
const Segment SGI130933[] = {{&S14308[31], 3}, {NULL}};
const Segment SGI130934[] = {{&S14308[81], 6}, {NULL}};
const Segment SGI130935[] = {{&S14311[44], 10}, {NULL}};
const Segment SGI130936[] = {{&S14311[44], 22}, {NULL}};
const Segment SGI130937[] = {{&S14311[30], 6}, {NULL}};
const Segment SGI130938[] = {{&S14311[115], 3}, {NULL}};
const Segment SGI130939[] = {{&S14311[80], 6}, {NULL}};
const Segment SGI130940[] = {{&S14312[44], 7}, {NULL}};
const Segment SGI130941[] = {{&S14312[44], 19}, {NULL}};
const Segment SGI130942[] = {{&S14312[30], 6}, {NULL}};
const Segment SGI130943[] = {{&S14312[112], 3}, {NULL}};
const Segment SGI130944[] = {{&S14312[77], 6}, {NULL}};
const Segment SGI130945[] = {{&S14314[44], 10}, {NULL}};
const Segment SGI130946[] = {{&S14314[44], 22}, {NULL}};
const Segment SGI130947[] = {{&S14314[30], 6}, {NULL}};
const Segment SGI130948[] = {{&S14314[115], 3}, {NULL}};
const Segment SGI130949[] = {{&S14314[80], 6}, {NULL}};
const Segment SGI130950[] = {{&S14315[44], 20}, {NULL}};
const Segment SGI130951[] = {{&S14315[44], 32}, {NULL}};
const Segment SGI130952[] = {{&S14315[30], 6}, {NULL}};
const Segment SGI130953[] = {{&S14315[125], 3}, {NULL}};
const Segment SGI130954[] = {{&S14315[90], 6}, {NULL}};
const Segment SGI130955[] = {{&S14316[44], 12}, {NULL}};
const Segment SGI130956[] = {{&S14316[44], 24}, {NULL}};
const Segment SGI130957[] = {{&S14316[30], 6}, {NULL}};
const Segment SGI130958[] = {{&S14316[117], 3}, {NULL}};
const Segment SGI130959[] = {{&S14316[82], 6}, {NULL}};
const Segment SGI130960[] = {{&S14317[44], 9}, {NULL}};
const Segment SGI130961[] = {{&S14317[44], 21}, {NULL}};
const Segment SGI130962[] = {{&S14317[30], 6}, {NULL}};
const Segment SGI130963[] = {{&S14317[114], 3}, {NULL}};
const Segment SGI130964[] = {{&S14317[79], 6}, {NULL}};
const Segment SGI130965[] = {{&S14318[44], 6}, {NULL}};
const Segment SGI130966[] = {{&S14318[44], 18}, {NULL}};
const Segment SGI130967[] = {{&S14318[30], 6}, {NULL}};
const Segment SGI130968[] = {{&S14318[111], 3}, {NULL}};
const Segment SGI130969[] = {{&S14318[76], 6}, {NULL}};
const Segment SGI130970[] = {{&S14320[44], 8}, {NULL}};
const Segment SGI130971[] = {{&S14320[44], 22}, {NULL}};
const Segment SGI130972[] = {{&S14320[30], 6}, {NULL}};
const Segment SGI130973[] = {{&S14320[115], 3}, {NULL}};
const Segment SGI130974[] = {{&S14320[80], 6}, {NULL}};
const Segment SGI130975[] = {{&S14321[44], 7}, {NULL}};
const Segment SGI130976[] = {{&S14321[44], 19}, {NULL}};
const Segment SGI130977[] = {{&S14321[30], 6}, {NULL}};
const Segment SGI130978[] = {{&S14321[112], 3}, {NULL}};
const Segment SGI130979[] = {{&S14321[77], 6}, {NULL}};
const Segment SGI130980[] = {{&S14325[44], 9}, {NULL}};
const Segment SGI130981[] = {{&S14325[44], 21}, {NULL}};
const Segment SGI130982[] = {{&S14325[30], 6}, {NULL}};
const Segment SGI130983[] = {{&S14325[114], 3}, {NULL}};
const Segment SGI130984[] = {{&S14325[79], 6}, {NULL}};
const Segment SGI130985[] = {{&S14326[44], 14}, {NULL}};
const Segment SGI130986[] = {{&S14326[44], 26}, {NULL}};
const Segment SGI130987[] = {{&S14326[30], 6}, {NULL}};
const Segment SGI130988[] = {{&S14326[119], 3}, {NULL}};
const Segment SGI130989[] = {{&S14326[84], 6}, {NULL}};
const Segment SGI130990[] = {{&S14327[44], 7}, {NULL}};
const Segment SGI130991[] = {{&S14327[44], 19}, {NULL}};
const Segment SGI130992[] = {{&S14327[30], 6}, {NULL}};
const Segment SGI130993[] = {{&S14327[112], 3}, {NULL}};
const Segment SGI130994[] = {{&S14327[77], 6}, {NULL}};
const Segment SGI130995[] = {{&S14328[44], 8}, {NULL}};
const Segment SGI130996[] = {{&S14328[44], 20}, {NULL}};
const Segment SGI130997[] = {{&S14328[30], 6}, {NULL}};
const Segment SGI130998[] = {{&S14328[113], 3}, {NULL}};
const Segment SGI130999[] = {{&S14328[78], 6}, {NULL}};
const Segment SGI131000[] = {{&S14329[44], 13}, {NULL}};
const Segment SGI131001[] = {{&S14329[44], 25}, {NULL}};
const Segment SGI131002[] = {{&S14329[30], 6}, {NULL}};
const Segment SGI131003[] = {{&S14329[118], 3}, {NULL}};
const Segment SGI131004[] = {{&S14329[83], 6}, {NULL}};
const Segment SGI131005[] = {{&S14330[44], 10}, {NULL}};
const Segment SGI131006[] = {{&S14330[44], 22}, {NULL}};
const Segment SGI131007[] = {{&S14330[30], 6}, {NULL}};
const Segment SGI131008[] = {{&S14330[115], 3}, {NULL}};
const Segment SGI131009[] = {{&S14330[80], 6}, {NULL}};
const Segment SGI131010[] = {{&S14332[44], 10}, {NULL}};
const Segment SGI131011[] = {{&S14332[44], 22}, {NULL}};
const Segment SGI131012[] = {{&S14332[30], 6}, {NULL}};
const Segment SGI131013[] = {{&S14332[115], 3}, {NULL}};
const Segment SGI131014[] = {{&S14332[80], 6}, {NULL}};
const Segment SGI131015[] = {{&S14333[44], 6}, {NULL}};
const Segment SGI131016[] = {{&S14333[44], 18}, {NULL}};
const Segment SGI131017[] = {{&S14333[30], 6}, {NULL}};
const Segment SGI131018[] = {{&S14333[111], 3}, {NULL}};
const Segment SGI131019[] = {{&S14333[76], 6}, {NULL}};
const Segment SGI131020[] = {{&S14335[44], 15}, {NULL}};
const Segment SGI131021[] = {{&S14335[44], 34}, {NULL}};
const Segment SGI131022[] = {{&S14335[30], 6}, {NULL}};
const Segment SGI131023[] = {{&S14335[127], 3}, {NULL}};
const Segment SGI131024[] = {{&S14335[92], 6}, {NULL}};
const Segment SGI131025[] = {{&S14337[44], 9}, {NULL}};
const Segment SGI131026[] = {{&S14337[44], 21}, {NULL}};
const Segment SGI131027[] = {{&S14337[30], 6}, {NULL}};
const Segment SGI131028[] = {{&S14337[114], 3}, {NULL}};
const Segment SGI131029[] = {{&S14337[79], 6}, {NULL}};
const Segment SGI131030[] = {{&S14338[44], 15}, {NULL}};
const Segment SGI131031[] = {{&S14338[44], 27}, {NULL}};
const Segment SGI131032[] = {{&S14338[30], 6}, {NULL}};
const Segment SGI131033[] = {{&S14338[120], 3}, {NULL}};
const Segment SGI131034[] = {{&S14338[85], 6}, {NULL}};
const Segment SGI131035[] = {{&S14339[44], 10}, {NULL}};
const Segment SGI131036[] = {{&S14339[44], 22}, {NULL}};
const Segment SGI131037[] = {{&S14339[30], 6}, {NULL}};
const Segment SGI131038[] = {{&S14339[115], 3}, {NULL}};
const Segment SGI131039[] = {{&S14339[80], 6}, {NULL}};
const Segment SGI131040[] = {{&S14340[44], 8}, {NULL}};
const Segment SGI131041[] = {{&S14340[44], 20}, {NULL}};
const Segment SGI131042[] = {{&S14340[30], 6}, {NULL}};
const Segment SGI131043[] = {{&S14340[113], 3}, {NULL}};
const Segment SGI131044[] = {{&S14340[78], 6}, {NULL}};
const Segment SGI131045[] = {{&S14343[44], 10}, {NULL}};
const Segment SGI131046[] = {{&S14343[44], 22}, {NULL}};
const Segment SGI131047[] = {{&S14343[30], 6}, {NULL}};
const Segment SGI131048[] = {{&S14343[115], 3}, {NULL}};
const Segment SGI131049[] = {{&S14343[80], 6}, {NULL}};
const Segment SGI131050[] = {{&S14344[44], 14}, {NULL}};
const Segment SGI131051[] = {{&S14344[44], 35}, {NULL}};
const Segment SGI131052[] = {{&S14344[30], 6}, {NULL}};
const Segment SGI131053[] = {{&S14344[127], 3}, {NULL}};
const Segment SGI131054[] = {{&S14344[93], 5}, {NULL}};
const Segment SGI131055[] = {{&S14348[44], 9}, {NULL}};
const Segment SGI131056[] = {{&S14348[44], 21}, {NULL}};
const Segment SGI131057[] = {{&S14348[30], 6}, {NULL}};
const Segment SGI131058[] = {{&S14348[114], 3}, {NULL}};
const Segment SGI131059[] = {{&S14348[79], 6}, {NULL}};
const Segment SGI131060[] = {{&S14349[44], 8}, {NULL}};
const Segment SGI131061[] = {{&S14349[44], 20}, {NULL}};
const Segment SGI131062[] = {{&S14349[30], 6}, {NULL}};
const Segment SGI131063[] = {{&S14349[113], 3}, {NULL}};
const Segment SGI131064[] = {{&S14349[78], 6}, {NULL}};
const Segment SGI131065[] = {{&S14350[44], 13}, {NULL}};
const Segment SGI131066[] = {{&S14350[44], 25}, {NULL}};
const Segment SGI131067[] = {{&S14350[30], 6}, {NULL}};
const Segment SGI131068[] = {{&S14350[118], 3}, {NULL}};
const Segment SGI131069[] = {{&S14350[83], 6}, {NULL}};
const Segment SGI131070[] = {{&S14351[44], 7}, {NULL}};
const Segment SGI131071[] = {{&S14351[44], 19}, {NULL}};
const Segment SGI131072[] = {{&S14351[30], 6}, {NULL}};
const Segment SGI131073[] = {{&S14351[112], 3}, {NULL}};
const Segment SGI131074[] = {{&S14351[77], 6}, {NULL}};
const Segment SGI131075[] = {{&S14354[44], 13}, {NULL}};
const Segment SGI131076[] = {{&S14354[44], 25}, {NULL}};
const Segment SGI131077[] = {{&S14354[30], 6}, {NULL}};
const Segment SGI131078[] = {{&S14354[118], 3}, {NULL}};
const Segment SGI131079[] = {{&S14354[83], 6}, {NULL}};
const Segment SGI131080[] = {{&S14355[44], 16}, {NULL}};
const Segment SGI131081[] = {{&S14355[44], 28}, {NULL}};
const Segment SGI131082[] = {{&S14355[30], 6}, {NULL}};
const Segment SGI131083[] = {{&S14355[121], 3}, {NULL}};
const Segment SGI131084[] = {{&S14355[86], 6}, {NULL}};
const Segment SGI131085[] = {{&S23894[44], 11}, {NULL}};
const Segment SGI131086[] = {{&S23894[44], 23}, {NULL}};
const Segment SGI131087[] = {{&S23894[30], 6}, {NULL}};
const Segment SGI131088[] = {{&S23894[31], 3}, {NULL}};
const Segment SGI131089[] = {{&S23894[81], 6}, {NULL}};
const Segment SGI131090[] = {{&S23895[44], 13}, {NULL}};
const Segment SGI131091[] = {{&S23895[44], 25}, {NULL}};
const Segment SGI131092[] = {{&S14358[44], 7}, {NULL}};
const Segment SGI131093[] = {{&S23895[30], 6}, {NULL}};
const Segment SGI131094[] = {{&S14358[44], 19}, {NULL}};
const Segment SGI131095[] = {{&S14358[30], 6}, {NULL}};
const Segment SGI131096[] = {{&S23895[31], 3}, {NULL}};
const Segment SGI131097[] = {{&S14358[111], 3}, {NULL}};
const Segment SGI131098[] = {{&S23895[83], 6}, {NULL}};
const Segment SGI131099[] = {{&S14358[77], 5}, {NULL}};
const Segment SGI131100[] = {{&S14359[44], 7}, {NULL}};
const Segment SGI131101[] = {{&S14359[44], 19}, {NULL}};
const Segment SGI131102[] = {{&S14359[30], 6}, {NULL}};
const Segment SGI131103[] = {{&S14359[112], 3}, {NULL}};
const Segment SGI131104[] = {{&S14359[77], 6}, {NULL}};
const Segment SGI131105[] = {{&S14361[44], 18}, {NULL}};
const Segment SGI131106[] = {{&S14361[44], 40}, {NULL}};
const Segment SGI131107[] = {{&S14361[30], 6}, {NULL}};
const Segment SGI131108[] = {{&S14361[133], 3}, {NULL}};
const Segment SGI131109[] = {{&S14361[98], 6}, {NULL}};
const Segment SGI131110[] = {{&S14363[44], 9}, {NULL}};
const Segment SGI131111[] = {{&S14363[44], 21}, {NULL}};
const Segment SGI131112[] = {{&S14363[30], 6}, {NULL}};
const Segment SGI131113[] = {{&S14363[114], 3}, {NULL}};
const Segment SGI131114[] = {{&S14363[79], 6}, {NULL}};
const Segment SGI131115[] = {{&S14364[44], 11}, {NULL}};
const Segment SGI131116[] = {{&S14364[44], 23}, {NULL}};
const Segment SGI131117[] = {{&S14364[30], 6}, {NULL}};
const Segment SGI131118[] = {{&S14364[116], 3}, {NULL}};
const Segment SGI131119[] = {{&S14364[81], 6}, {NULL}};
const Segment SGI131120[] = {{&S14365[44], 5}, {NULL}};
const Segment SGI131121[] = {{&S14365[44], 17}, {NULL}};
const Segment SGI131122[] = {{&S14365[30], 6}, {NULL}};
const Segment SGI131123[] = {{&S14365[110], 3}, {NULL}};
const Segment SGI131124[] = {{&S14365[75], 6}, {NULL}};
const Segment SGI131125[] = {{&S13542[44], 16}, {NULL}};
const Segment SGI131126[] = {{&S13542[44], 35}, {NULL}};
const Segment SGI131127[] = {{&S13542[30], 6}, {NULL}};
const Segment SGI131128[] = {{&S13542[128], 3}, {NULL}};
const Segment SGI131129[] = {{&S13542[93], 6}, {NULL}};
const Segment SGI131130[] = {{&S13546[44], 6}, {NULL}};
const Segment SGI131131[] = {{&S13546[44], 18}, {NULL}};
const Segment SGI131132[] = {{&S13546[30], 6}, {NULL}};
const Segment SGI131133[] = {{&S13546[111], 3}, {NULL}};
const Segment SGI131134[] = {{&S13546[76], 6}, {NULL}};
const Segment SGI131135[] = {{&S13555[44], 7}, {NULL}};
const Segment SGI131136[] = {{&S13555[44], 19}, {NULL}};
const Segment SGI131137[] = {{&S13555[30], 6}, {NULL}};
const Segment SGI131138[] = {{&S13555[112], 3}, {NULL}};
const Segment SGI131139[] = {{&S13555[77], 6}, {NULL}};
const Segment SGI131140[] = {{&S13568[44], 41}, {NULL}};
const Segment SGI131141[] = {{&S13568[30], 6}, {NULL}};
const Segment SGI131142[] = {{&S13568[134], 3}, {NULL}};
const Segment SGI131143[] = {{&S13568[99], 6}, {NULL}};
const Segment SGI131144[] = {{&S13570[44], 12}, {NULL}};
const Segment SGI131145[] = {{&S13570[44], 24}, {NULL}};
const Segment SGI131146[] = {{&S13570[30], 6}, {NULL}};
const Segment SGI131147[] = {{&S13570[117], 3}, {NULL}};
const Segment SGI131148[] = {{&S13570[82], 6}, {NULL}};
const Segment SGI131149[] = {{&S14366[44], 9}, {NULL}};
const Segment SGI131150[] = {{&S14366[44], 19}, {NULL}};
const Segment SGI131151[] = {{&S14366[30], 6}, {NULL}};
const Segment SGI131152[] = {{&S14366[112], 3}, {NULL}};
const Segment SGI131153[] = {{&S14366[77], 6}, {NULL}};
const Segment SGI131154[] = {{&S13576[44], 10}, {NULL}};
const Segment SGI131155[] = {{&S13576[44], 22}, {NULL}};
const Segment SGI131156[] = {{&S13576[30], 6}, {NULL}};
const Segment SGI131157[] = {{&S13576[115], 3}, {NULL}};
const Segment SGI131158[] = {{&S13576[80], 6}, {NULL}};
const Segment SGI131159[] = {{&S14367[44], 79}, {NULL}};
const Segment SGI131160[] = {{&S14367[44], 100}, {NULL}};
const Segment SGI131161[] = {{&S14367[30], 6}, {NULL}};
const Segment SGI131162[] = {{&S14367[191], 3}, {NULL}};
const Segment SGI131163[] = {{&S14367[117], 5}, {NULL}};
const Segment SGI131164[] = {{&S13577[44], 13}, {NULL}};
const Segment SGI131165[] = {{&S13577[44], 25}, {NULL}};
const Segment SGI131166[] = {{&S13577[30], 6}, {NULL}};
const Segment SGI131167[] = {{&S13577[118], 3}, {NULL}};
const Segment SGI131168[] = {{&S13577[83], 6}, {NULL}};
const Segment SGI131169[] = {{&S13578[44], 22}, {NULL}};
const Segment SGI131170[] = {{&S13578[44], 34}, {NULL}};
const Segment SGI131171[] = {{&S13578[30], 6}, {NULL}};
const Segment SGI131172[] = {{&S14369[44], 8}, {NULL}};
const Segment SGI131173[] = {{&S14369[44], 20}, {NULL}};
const Segment SGI131174[] = {{&S14369[30], 6}, {NULL}};
const Segment SGI131175[] = {{&S13578[127], 3}, {NULL}};
const Segment SGI131176[] = {{&S13578[92], 6}, {NULL}};
const Segment SGI131177[] = {{&S14369[113], 3}, {NULL}};
const Segment SGI131178[] = {{&S14369[78], 6}, {NULL}};
const Segment SGI131179[] = {{&S13580[44], 10}, {NULL}};
const Segment SGI131180[] = {{&S13580[44], 22}, {NULL}};
const Segment SGI131181[] = {{&S14371[44], 9}, {NULL}};
const Segment SGI131182[] = {{&S13580[30], 6}, {NULL}};
const Segment SGI131183[] = {{&S14371[44], 21}, {NULL}};
const Segment SGI131184[] = {{&S14371[30], 6}, {NULL}};
const Segment SGI131185[] = {{&S13580[115], 3}, {NULL}};
const Segment SGI131186[] = {{&S14371[114], 3}, {NULL}};
const Segment SGI131187[] = {{&S13580[80], 6}, {NULL}};
const Segment SGI131188[] = {{&S14371[79], 6}, {NULL}};
const Segment SGI131189[] = {{&S14375[44], 10}, {NULL}};
const Segment SGI131190[] = {{&S14375[44], 22}, {NULL}};
const Segment SGI131191[] = {{&S14375[30], 6}, {NULL}};
const Segment SGI131192[] = {{&S14375[115], 3}, {NULL}};
const Segment SGI131193[] = {{&S14375[80], 6}, {NULL}};
const Segment SGI131194[] = {{&S14376[44], 7}, {NULL}};
const Segment SGI131195[] = {{&S14376[44], 19}, {NULL}};
const Segment SGI131196[] = {{&S14376[30], 6}, {NULL}};
const Segment SGI131197[] = {{&S14376[111], 3}, {NULL}};
const Segment SGI131198[] = {{&S14376[77], 5}, {NULL}};
const Segment SGI131199[] = {{&S14377[44], 6}, {NULL}};
const Segment SGI131200[] = {{&S13588[44], 7}, {NULL}};
const Segment SGI131201[] = {{&S14377[44], 63}, {NULL}};
const Segment SGI131202[] = {{&S13588[44], 19}, {NULL}};
const Segment SGI131203[] = {{&S14377[30], 6}, {NULL}};
const Segment SGI131204[] = {{&S13588[30], 6}, {NULL}};
const Segment SGI131205[] = {{&S13588[112], 3}, {NULL}};
const Segment SGI131206[] = {{&S14377[156], 3}, {NULL}};
const Segment SGI131207[] = {{&S13588[77], 6}, {NULL}};
const Segment SGI131208[] = {{&S14377[121], 6}, {NULL}};
const Segment SGI131209[] = {{&S14378[44], 9}, {NULL}};
const Segment SGI131210[] = {{&S14378[44], 21}, {NULL}};
const Segment SGI131211[] = {{&S14378[30], 6}, {NULL}};
const Segment SGI131212[] = {{&S14378[114], 3}, {NULL}};
const Segment SGI131213[] = {{&S14378[79], 6}, {NULL}};
const Segment SGI131214[] = {{&S13591[44], 13}, {NULL}};
const Segment SGI131215[] = {{&S13591[44], 25}, {NULL}};
const Segment SGI131216[] = {{&S13591[30], 6}, {NULL}};
const Segment SGI131217[] = {{&S13591[118], 3}, {NULL}};
const Segment SGI131218[] = {{&S13591[83], 6}, {NULL}};
const Segment SGI131219[] = {{&S14380[44], 10}, {NULL}};
const Segment SGI131220[] = {{&S14380[44], 24}, {NULL}};
const Segment SGI131221[] = {{&S14380[30], 6}, {NULL}};
const Segment SGI131222[] = {{&S14380[117], 3}, {NULL}};
const Segment SGI131223[] = {{&S14380[82], 6}, {NULL}};
const Segment SGI131224[] = {{&S13592[44], 7}, {NULL}};
const Segment SGI131225[] = {{&S13592[44], 19}, {NULL}};
const Segment SGI131226[] = {{&S13592[30], 6}, {NULL}};
const Segment SGI131227[] = {{&S13592[112], 3}, {NULL}};
const Segment SGI131228[] = {{&S13592[77], 6}, {NULL}};
const Segment SGI131229[] = {{&S13593[78], 6}, {NULL}};
const Segment SGI131230[] = {{&S14385[44], 13}, {NULL}};
const Segment SGI131231[] = {{&S14385[44], 25}, {NULL}};
const Segment SGI131232[] = {{&S14385[30], 6}, {NULL}};
const Segment SGI131233[] = {{&S14385[118], 3}, {NULL}};
const Segment SGI131234[] = {{&S14385[83], 6}, {NULL}};
const Segment SGI131235[] = {{&S14387[44], 5}, {NULL}};
const Segment SGI131236[] = {{&S14387[44], 17}, {NULL}};
const Segment SGI131237[] = {{&S14387[30], 6}, {NULL}};
const Segment SGI131238[] = {{&S14387[110], 3}, {NULL}};
const Segment SGI131239[] = {{&S14387[75], 6}, {NULL}};
const Segment SGI131240[] = {{&S14388[44], 9}, {NULL}};
const Segment SGI131241[] = {{&S14388[44], 21}, {NULL}};
const Segment SGI131242[] = {{&S14388[30], 6}, {NULL}};
const Segment SGI131243[] = {{&S14388[114], 3}, {NULL}};
const Segment SGI131244[] = {{&S14388[79], 6}, {NULL}};
const Segment SGI131245[] = {{&S14389[44], 6}, {NULL}};
const Segment SGI131246[] = {{&S14389[44], 18}, {NULL}};
const Segment SGI131247[] = {{&S14389[30], 6}, {NULL}};
const Segment SGI131248[] = {{&S14389[111], 3}, {NULL}};
const Segment SGI131249[] = {{&S13602[44], 8}, {NULL}};
const Segment SGI131250[] = {{&S14389[76], 6}, {NULL}};
const Segment SGI131251[] = {{&S13602[44], 20}, {NULL}};
const Segment SGI131252[] = {{&S13602[30], 6}, {NULL}};
const Segment SGI131253[] = {{&S13602[31], 3}, {NULL}};
const Segment SGI131254[] = {{&S13602[78], 6}, {NULL}};
const Segment SGI131255[] = {{&S14392[44], 14}, {NULL}};
const Segment SGI131256[] = {{&S14392[44], 26}, {NULL}};
const Segment SGI131257[] = {{&S14392[30], 6}, {NULL}};
const Segment SGI131258[] = {{&S14392[119], 3}, {NULL}};
const Segment SGI131259[] = {{&S14392[84], 6}, {NULL}};
const Segment SGI131260[] = {{&S13605[44], 7}, {NULL}};
const Segment SGI131261[] = {{&S13605[44], 19}, {NULL}};
const Segment SGI131262[] = {{&S13605[30], 6}, {NULL}};
const Segment SGI131263[] = {{&S13605[112], 3}, {NULL}};
const Segment SGI131264[] = {{&S13605[77], 6}, {NULL}};
const Segment SGI131265[] = {{&S14393[44], 10}, {NULL}};
const Segment SGI131266[] = {{&S14393[44], 22}, {NULL}};
const Segment SGI131267[] = {{&S14393[30], 6}, {NULL}};
const Segment SGI131268[] = {{&S14393[115], 3}, {NULL}};
const Segment SGI131269[] = {{&S14393[80], 6}, {NULL}};
const Segment SGI131270[] = {{&S13608[44], 11}, {NULL}};
const Segment SGI131271[] = {{&S13608[44], 23}, {NULL}};
const Segment SGI131272[] = {{&S13608[30], 6}, {NULL}};
const Segment SGI131273[] = {{&S13608[116], 3}, {NULL}};
const Segment SGI131274[] = {{&S13608[81], 6}, {NULL}};
const Segment SGI131275[] = {{&S14396[44], 8}, {NULL}};
const Segment SGI131276[] = {{&S14396[44], 20}, {NULL}};
const Segment SGI131277[] = {{&S14396[30], 6}, {NULL}};
const Segment SGI131278[] = {{&S14396[113], 3}, {NULL}};
const Segment SGI131279[] = {{&S14396[78], 6}, {NULL}};
const Segment SGI131280[] = {{&S13613[44], 7}, {NULL}};
const Segment SGI131281[] = {{&S13613[44], 19}, {NULL}};
const Segment SGI131282[] = {{&S13613[30], 6}, {NULL}};
const Segment SGI131283[] = {{&S13613[112], 3}, {NULL}};
const Segment SGI131284[] = {{&S13613[77], 6}, {NULL}};
const Segment SGI131285[] = {{&S14401[44], 11}, {NULL}};
const Segment SGI131286[] = {{&S14401[44], 23}, {NULL}};
const Segment SGI131287[] = {{&S14401[30], 6}, {NULL}};
const Segment SGI131288[] = {{&S14401[116], 3}, {NULL}};
const Segment SGI131289[] = {{&S14401[81], 6}, {NULL}};
const Segment SGI131290[] = {{&S13620[44], 5}, {NULL}};
const Segment SGI131291[] = {{&S13620[44], 17}, {NULL}};
const Segment SGI131292[] = {{&S13620[30], 6}, {NULL}};
const Segment SGI131293[] = {{&S14407[44], 6}, {NULL}};
const Segment SGI131294[] = {{&S13620[31], 3}, {NULL}};
const Segment SGI131295[] = {{&S14407[44], 18}, {NULL}};
const Segment SGI131296[] = {{&S14407[30], 6}, {NULL}};
const Segment SGI131297[] = {{&S13620[75], 6}, {NULL}};
const Segment SGI131298[] = {{&S14407[111], 3}, {NULL}};
const Segment SGI131299[] = {{&S14407[76], 6}, {NULL}};
const Segment SGI131300[] = {{&S14408[44], 10}, {NULL}};
const Segment SGI131301[] = {{&S14408[44], 30}, {NULL}};
const Segment SGI131302[] = {{&S14408[30], 6}, {NULL}};
const Segment SGI131303[] = {{&S14408[122], 3}, {NULL}};
const Segment SGI131304[] = {{&S14408[88], 6}, {NULL}};
const Segment SGI131305[] = {{&S13624[44], 6}, {NULL}};
const Segment SGI131306[] = {{&S13624[44], 26}, {NULL}};
const Segment SGI131307[] = {{&S13624[30], 6}, {NULL}};
const Segment SGI131308[] = {{&S13624[119], 3}, {NULL}};
const Segment SGI131309[] = {{&S13624[84], 6}, {NULL}};
const Segment SGI131310[] = {{&S14410[44], 12}, {NULL}};
const Segment SGI131311[] = {{&S14410[44], 24}, {NULL}};
const Segment SGI131312[] = {{&S14410[30], 6}, {NULL}};
const Segment SGI131313[] = {{&S14410[117], 3}, {NULL}};
const Segment SGI131314[] = {{&S14410[82], 6}, {NULL}};
const Segment SGI131315[] = {{&S13626[44], 17}, {NULL}};
const Segment SGI131316[] = {{&S13626[44], 29}, {NULL}};
const Segment SGI131317[] = {{&S13626[30], 6}, {NULL}};
const Segment SGI131318[] = {{&S13626[122], 3}, {NULL}};
const Segment SGI131319[] = {{&S13626[87], 6}, {NULL}};
const Segment SGI131320[] = {{&S14411[44], 13}, {NULL}};
const Segment SGI131321[] = {{&S14411[44], 25}, {NULL}};
const Segment SGI131322[] = {{&S14411[30], 6}, {NULL}};
const Segment SGI131323[] = {{&S14411[118], 3}, {NULL}};
const Segment SGI131324[] = {{&S14411[83], 6}, {NULL}};
const Segment SGI131325[] = {{&S14412[44], 15}, {NULL}};
const Segment SGI131326[] = {{&S14412[44], 27}, {NULL}};
const Segment SGI131327[] = {{&S14412[30], 6}, {NULL}};
const Segment SGI131328[] = {{&S14412[120], 3}, {NULL}};
const Segment SGI131329[] = {{&S14412[85], 6}, {NULL}};
const Segment SGI131330[] = {{&S13630[44], 13}, {NULL}};
const Segment SGI131331[] = {{&S13630[44], 25}, {NULL}};
const Segment SGI131332[] = {{&S13630[30], 6}, {NULL}};
const Segment SGI131333[] = {{&S13630[118], 3}, {NULL}};
const Segment SGI131334[] = {{&S13630[83], 6}, {NULL}};
const Segment SGI131335[] = {{&S14414[44], 9}, {NULL}};
const Segment SGI131336[] = {{&S14414[44], 21}, {NULL}};
const Segment SGI131337[] = {{&S14414[30], 6}, {NULL}};
const Segment SGI131338[] = {{&S14414[114], 3}, {NULL}};
const Segment SGI131339[] = {{&S14414[79], 6}, {NULL}};
const Segment SGI131340[] = {{&S14415[44], 6}, {NULL}};
const Segment SGI131341[] = {{&S14415[44], 18}, {NULL}};
const Segment SGI131342[] = {{&S14415[30], 6}, {NULL}};
const Segment SGI131343[] = {{&S14415[111], 3}, {NULL}};
const Segment SGI131344[] = {{&S14415[76], 6}, {NULL}};
const Segment SGI131345[] = {{&S14416[44], 16}, {NULL}};
const Segment SGI131346[] = {{&S14416[44], 28}, {NULL}};
const Segment SGI131347[] = {{&S14416[30], 6}, {NULL}};
const Segment SGI131348[] = {{&S14416[121], 3}, {NULL}};
const Segment SGI131349[] = {{&S14416[86], 6}, {NULL}};
const Segment SGI131350[] = {{&S13636[44], 9}, {NULL}};
const Segment SGI131351[] = {{&S13636[44], 21}, {NULL}};
const Segment SGI131352[] = {{&S13636[30], 6}, {NULL}};
const Segment SGI131353[] = {{&S13636[114], 3}, {NULL}};
const Segment SGI131354[] = {{&S13636[79], 6}, {NULL}};
const Segment SGI131355[] = {{&S14418[44], 4}, {NULL}};
const Segment SGI131356[] = {{&S14418[44], 16}, {NULL}};
const Segment SGI131357[] = {{&S14418[30], 6}, {NULL}};
const Segment SGI131358[] = {{&S14418[109], 3}, {NULL}};
const Segment SGI131359[] = {{&S14418[74], 6}, {NULL}};
const Segment SGI131360[] = {{&S14421[44], 5}, {NULL}};
const Segment SGI131361[] = {{&S14421[44], 17}, {NULL}};
const Segment SGI131362[] = {{&S14421[30], 6}, {NULL}};
const Segment SGI131363[] = {{&S14421[110], 3}, {NULL}};
const Segment SGI131364[] = {{&S14421[75], 6}, {NULL}};
const Segment SGI131365[] = {{&S13641[44], 16}, {NULL}};
const Segment SGI131366[] = {{&S13641[44], 35}, {NULL}};
const Segment SGI131367[] = {{&S13641[30], 6}, {NULL}};
const Segment SGI131368[] = {{&S13641[128], 3}, {NULL}};
const Segment SGI131369[] = {{&S13641[93], 6}, {NULL}};
const Segment SGI131370[] = {{&S14422[44], 11}, {NULL}};
const Segment SGI131371[] = {{&S14422[44], 23}, {NULL}};
const Segment SGI131372[] = {{&S14422[30], 6}, {NULL}};
const Segment SGI131373[] = {{&S14422[116], 3}, {NULL}};
const Segment SGI131374[] = {{&S14422[81], 6}, {NULL}};
const Segment SGI131375[] = {{&S14423[38], 5}, {NULL}};
const Segment SGI131376[] = {{&S14423[31], 6}, {NULL}};
const Segment SGI131377[] = {{&S14424[44], 17}, {NULL}};
const Segment SGI131378[] = {{&S14424[44], 29}, {NULL}};
const Segment SGI131379[] = {{&S14424[30], 6}, {NULL}};
const Segment SGI131380[] = {{&S14424[122], 3}, {NULL}};
const Segment SGI131381[] = {{&S14424[87], 6}, {NULL}};
const Segment SGI131382[] = {{&S14425[44], 28}, {NULL}};
const Segment SGI131383[] = {{&S14425[44], 40}, {NULL}};
const Segment SGI131384[] = {{&S14425[30], 6}, {NULL}};
const Segment SGI131385[] = {{&S14425[133], 3}, {NULL}};
const Segment SGI131386[] = {{&S14425[98], 6}, {NULL}};
const Segment SGI131387[] = {{&S14426[44], 21}, {NULL}};
const Segment SGI131388[] = {{&S14426[44], 33}, {NULL}};
const Segment SGI131389[] = {{&S14426[30], 6}, {NULL}};
const Segment SGI131390[] = {{&S14426[126], 3}, {NULL}};
const Segment SGI131391[] = {{&S14426[91], 6}, {NULL}};
const Segment SGI131392[] = {{&S13650[44], 11}, {NULL}};
const Segment SGI131393[] = {{&S13650[44], 22}, {NULL}};
const Segment SGI131394[] = {{&S13650[30], 6}, {NULL}};
const Segment SGI131395[] = {{&S13650[115], 3}, {NULL}};
const Segment SGI131396[] = {{&S14427[44], 11}, {NULL}};
const Segment SGI131397[] = {{&S13650[80], 6}, {NULL}};
const Segment SGI131398[] = {{&S14427[44], 23}, {NULL}};
const Segment SGI131399[] = {{&S14427[30], 6}, {NULL}};
const Segment SGI131400[] = {{&S14427[116], 3}, {NULL}};
const Segment SGI131401[] = {{&S14427[81], 6}, {NULL}};
const Segment SGI131402[] = {{&S13652[44], 7}, {NULL}};
const Segment SGI131403[] = {{&S13652[44], 19}, {NULL}};
const Segment SGI131404[] = {{&S13652[30], 6}, {NULL}};
const Segment SGI131405[] = {{&S14429[44], 12}, {NULL}};
const Segment SGI131406[] = {{&S13652[112], 3}, {NULL}};
const Segment SGI131407[] = {{&S14429[44], 24}, {NULL}};
const Segment SGI131408[] = {{&S14429[30], 6}, {NULL}};
const Segment SGI131409[] = {{&S13652[77], 6}, {NULL}};
const Segment SGI131410[] = {{&S14429[117], 3}, {NULL}};
const Segment SGI131411[] = {{&S14429[82], 6}, {NULL}};
const Segment SGI131412[] = {{&S14430[44], 9}, {NULL}};
const Segment SGI131413[] = {{&S14430[44], 21}, {NULL}};
const Segment SGI131414[] = {{&S14430[30], 6}, {NULL}};
const Segment SGI131415[] = {{&S14430[114], 3}, {NULL}};
const Segment SGI131416[] = {{&S14430[79], 6}, {NULL}};
const Segment SGI131417[] = {{&S14431[44], 11}, {NULL}};
const Segment SGI131418[] = {{&S14431[44], 23}, {NULL}};
const Segment SGI131419[] = {{&S14431[30], 6}, {NULL}};
const Segment SGI131420[] = {{&S14431[116], 3}, {NULL}};
const Segment SGI131421[] = {{&S14431[81], 6}, {NULL}};
const Segment SGI131422[] = {{&S14432[44], 14}, {NULL}};
const Segment SGI131423[] = {{&S14432[44], 26}, {NULL}};
const Segment SGI131424[] = {{&S14432[30], 6}, {NULL}};
const Segment SGI131425[] = {{&S14432[119], 3}, {NULL}};
const Segment SGI131426[] = {{&S14432[84], 6}, {NULL}};
const Segment SGI131427[] = {{&S13659[44], 15}, {NULL}};
const Segment SGI131428[] = {{&S13659[44], 27}, {NULL}};
const Segment SGI131429[] = {{&S13659[30], 6}, {NULL}};
const Segment SGI131430[] = {{&S13659[31], 3}, {NULL}};
const Segment SGI131431[] = {{&S13659[85], 6}, {NULL}};
const Segment SGI131432[] = {{&S14433[44], 7}, {NULL}};
const Segment SGI131433[] = {{&S14433[44], 19}, {NULL}};
const Segment SGI131434[] = {{&S14433[30], 6}, {NULL}};
const Segment SGI131435[] = {{&S14433[112], 3}, {NULL}};
const Segment SGI131436[] = {{&S14433[77], 6}, {NULL}};
const Segment SGI131437[] = {{&S14434[42], 20}, {NULL}};
const Segment SGI131438[] = {{&S14434[30], 6}, {NULL}};
const Segment SGI131439[] = {{&S14434[123], 3}, {NULL}};
const Segment SGI131440[] = {{&S14434[88], 6}, {NULL}};
const Segment SGI131441[] = {{&S14435[44], 13}, {NULL}};
const Segment SGI131442[] = {{&S14435[44], 25}, {NULL}};
const Segment SGI131443[] = {{&S14435[30], 6}, {NULL}};
const Segment SGI131444[] = {{&S14435[118], 3}, {NULL}};
const Segment SGI131445[] = {{&S14435[83], 6}, {NULL}};
const Segment SGI131446[] = {{&S14436[44], 10}, {NULL}};
const Segment SGI131447[] = {{&S14436[44], 22}, {NULL}};
const Segment SGI131448[] = {{&S14436[30], 6}, {NULL}};
const Segment SGI131449[] = {{&S13666[44], 4}, {NULL}};
const Segment SGI131450[] = {{&S13666[44], 16}, {NULL}};
const Segment SGI131451[] = {{&S14436[115], 3}, {NULL}};
const Segment SGI131452[] = {{&S13666[30], 6}, {NULL}};
const Segment SGI131453[] = {{&S14436[80], 6}, {NULL}};
const Segment SGI131454[] = {{&S13666[109], 3}, {NULL}};
const Segment SGI131455[] = {{&S13666[74], 6}, {NULL}};
const Segment SGI131456[] = {{&S14437[44], 16}, {NULL}};
const Segment SGI131457[] = {{&S14437[44], 28}, {NULL}};
const Segment SGI131458[] = {{&S14437[30], 6}, {NULL}};
const Segment SGI131459[] = {{&S14437[121], 3}, {NULL}};
const Segment SGI131460[] = {{&S14437[86], 6}, {NULL}};
const Segment SGI131461[] = {{&S14438[44], 26}, {NULL}};
const Segment SGI131462[] = {{&S13669[44], 16}, {NULL}};
const Segment SGI131463[] = {{&S14438[44], 38}, {NULL}};
const Segment SGI131464[] = {{&S13669[44], 28}, {NULL}};
const Segment SGI131465[] = {{&S14438[30], 6}, {NULL}};
const Segment SGI131466[] = {{&S13669[30], 6}, {NULL}};
const Segment SGI131467[] = {{&S14438[131], 3}, {NULL}};
const Segment SGI131468[] = {{&S13669[121], 3}, {NULL}};
const Segment SGI131469[] = {{&S14438[96], 6}, {NULL}};
const Segment SGI131470[] = {{&S13669[86], 6}, {NULL}};
const Segment SGI131471[] = {{&S14439[44], 9}, {NULL}};
const Segment SGI131472[] = {{&S13670[44], 8}, {NULL}};
const Segment SGI131473[] = {{&S14439[44], 21}, {NULL}};
const Segment SGI131474[] = {{&S13670[44], 20}, {NULL}};
const Segment SGI131475[] = {{&S14439[30], 6}, {NULL}};
const Segment SGI131476[] = {{&S13670[30], 6}, {NULL}};
const Segment SGI131477[] = {{&S14439[114], 3}, {NULL}};
const Segment SGI131478[] = {{&S13670[113], 3}, {NULL}};
const Segment SGI131479[] = {{&S14439[79], 6}, {NULL}};
const Segment SGI131480[] = {{&S13670[78], 6}, {NULL}};
const Segment SGI131481[] = {{&S14440[44], 84}, {NULL}};
const Segment SGI131482[] = {{&S14440[44], 105}, {NULL}};
const Segment SGI131483[] = {{&S14440[30], 6}, {NULL}};
const Segment SGI131484[] = {{&S14440[196], 3}, {NULL}};
const Segment SGI131485[] = {{&S13672[44], 12}, {NULL}};
const Segment SGI131486[] = {{&S13672[44], 24}, {NULL}};
const Segment SGI131487[] = {{&S14440[122], 5}, {NULL}};
const Segment SGI131488[] = {{&S13672[30], 6}, {NULL}};
const Segment SGI131489[] = {{&S13672[117], 3}, {NULL}};
const Segment SGI131490[] = {{&S13672[82], 6}, {NULL}};
const Segment SGI131491[] = {{&S14442[44], 6}, {NULL}};
const Segment SGI131492[] = {{&S14442[44], 18}, {NULL}};
const Segment SGI131493[] = {{&S14442[30], 6}, {NULL}};
const Segment SGI131494[] = {{&S14442[110], 3}, {NULL}};
const Segment SGI131495[] = {{&S14442[76], 5}, {NULL}};
const Segment SGI131496[] = {{&S13676[44], 16}, {NULL}};
const Segment SGI131497[] = {{&S13676[44], 28}, {NULL}};
const Segment SGI131498[] = {{&S13676[30], 6}, {NULL}};
const Segment SGI131499[] = {{&S13676[121], 3}, {NULL}};
const Segment SGI131500[] = {{&S13676[86], 6}, {NULL}};
const Segment SGI131501[] = {{&S14445[44], 9}, {NULL}};
const Segment SGI131502[] = {{&S14445[44], 30}, {NULL}};
const Segment SGI131503[] = {{&S14445[30], 6}, {NULL}};
const Segment SGI131504[] = {{&S14445[122], 3}, {NULL}};
const Segment SGI131505[] = {{&S14445[88], 5}, {NULL}};
const Segment SGI131506[] = {{&S13677[44], 22}, {NULL}};
const Segment SGI131507[] = {{&S13677[44], 34}, {NULL}};
const Segment SGI131508[] = {{&S13677[30], 6}, {NULL}};
const Segment SGI131509[] = {{&S13677[127], 3}, {NULL}};
const Segment SGI131510[] = {{&S13677[92], 6}, {NULL}};
const Segment SGI131511[] = {{&S14446[44], 8}, {NULL}};
const Segment SGI131512[] = {{&S14446[44], 20}, {NULL}};
const Segment SGI131513[] = {{&S14446[30], 6}, {NULL}};
const Segment SGI131514[] = {{&S14446[113], 3}, {NULL}};
const Segment SGI131515[] = {{&S14446[78], 6}, {NULL}};
const Segment SGI131516[] = {{&S14450[44], 6}, {NULL}};
const Segment SGI131517[] = {{&S14450[44], 18}, {NULL}};
const Segment SGI131518[] = {{&S14450[30], 6}, {NULL}};
const Segment SGI131519[] = {{&S14450[111], 3}, {NULL}};
const Segment SGI131520[] = {{&S14450[76], 6}, {NULL}};
const Segment SGI131521[] = {{&S13685[44], 7}, {NULL}};
const Segment SGI131522[] = {{&S13685[44], 19}, {NULL}};
const Segment SGI131523[] = {{&S13685[30], 6}, {NULL}};
const Segment SGI131524[] = {{&S13685[112], 3}, {NULL}};
const Segment SGI131525[] = {{&S13685[77], 6}, {NULL}};
const Segment SGI131526[] = {{&S14452[44], 5}, {NULL}};
const Segment SGI131527[] = {{&S14452[44], 17}, {NULL}};
const Segment SGI131528[] = {{&S14452[30], 6}, {NULL}};
const Segment SGI131529[] = {{&S14452[110], 3}, {NULL}};
const Segment SGI131530[] = {{&S14452[75], 6}, {NULL}};
const Segment SGI131531[] = {{&S14454[44], 11}, {NULL}};
const Segment SGI131532[] = {{&S14454[44], 23}, {NULL}};
const Segment SGI131533[] = {{&S14454[30], 6}, {NULL}};
const Segment SGI131534[] = {{&S14454[116], 3}, {NULL}};
const Segment SGI131535[] = {{&S14454[81], 6}, {NULL}};
const Segment SGI131536[] = {{&S14455[44], 84}, {NULL}};
const Segment SGI131537[] = {{&S14455[44], 105}, {NULL}};
const Segment SGI131538[] = {{&S14455[30], 6}, {NULL}};
const Segment SGI131539[] = {{&S14455[196], 3}, {NULL}};
const Segment SGI131540[] = {{&S14455[122], 5}, {NULL}};
const Segment SGI131541[] = {{&S14456[44], 9}, {NULL}};
const Segment SGI131542[] = {{&S14456[44], 21}, {NULL}};
const Segment SGI131543[] = {{&S14456[30], 6}, {NULL}};
const Segment SGI131544[] = {{&S14456[114], 3}, {NULL}};
const Segment SGI131545[] = {{&S14456[79], 6}, {NULL}};
const Segment SGI131546[] = {{&S14457[44], 9}, {NULL}};
const Segment SGI131547[] = {{&S14457[44], 21}, {NULL}};
const Segment SGI131548[] = {{&S14457[30], 6}, {NULL}};
const Segment SGI131549[] = {{&S14457[114], 3}, {NULL}};
const Segment SGI131550[] = {{&S14457[79], 6}, {NULL}};
const Segment SGI131551[] = {{&S14458[44], 7}, {NULL}};
const Segment SGI131552[] = {{&S14458[44], 19}, {NULL}};
const Segment SGI131553[] = {{&S14458[30], 6}, {NULL}};
const Segment SGI131554[] = {{&S14458[112], 3}, {NULL}};
const Segment SGI131555[] = {{&S14458[77], 6}, {NULL}};
const Segment SGI131556[] = {{&S14459[44], 84}, {NULL}};
const Segment SGI131557[] = {{&S14459[44], 105}, {NULL}};
const Segment SGI131558[] = {{&S14459[30], 6}, {NULL}};
const Segment SGI131559[] = {{&S14459[196], 3}, {NULL}};
const Segment SGI131560[] = {{&S14459[122], 5}, {NULL}};
const Segment SGI131561[] = {{&S14460[44], 11}, {NULL}};
const Segment SGI131562[] = {{&S14460[44], 23}, {NULL}};
const Segment SGI131563[] = {{&S14460[30], 6}, {NULL}};
const Segment SGI131564[] = {{&S14460[116], 3}, {NULL}};
const Segment SGI131565[] = {{&S14460[81], 6}, {NULL}};
const Segment SGI131566[] = {{&S14461[44], 8}, {NULL}};
const Segment SGI131567[] = {{&S14461[44], 20}, {NULL}};
const Segment SGI131568[] = {{&S14461[30], 6}, {NULL}};
const Segment SGI131569[] = {{&S14461[113], 3}, {NULL}};
const Segment SGI131570[] = {{&S14461[78], 6}, {NULL}};
const Segment SGI131571[] = {{&S14462[44], 10}, {NULL}};
const Segment SGI131572[] = {{&S14462[44], 22}, {NULL}};
const Segment SGI131573[] = {{&S14462[30], 6}, {NULL}};
const Segment SGI131574[] = {{&S14462[115], 3}, {NULL}};
const Segment SGI131575[] = {{&S14462[80], 6}, {NULL}};
const Segment SGI131576[] = {{&S14464[44], 9}, {NULL}};
const Segment SGI131577[] = {{&S14464[44], 21}, {NULL}};
const Segment SGI131578[] = {{&S14464[30], 6}, {NULL}};
const Segment SGI131579[] = {{&S14464[114], 3}, {NULL}};
const Segment SGI131580[] = {{&S14464[79], 6}, {NULL}};
const Segment SGI131581[] = {{&S14465[44], 21}, {NULL}};
const Segment SGI131582[] = {{&S14465[44], 33}, {NULL}};
const Segment SGI131583[] = {{&S14465[30], 6}, {NULL}};
const Segment SGI131584[] = {{&S14465[126], 3}, {NULL}};
const Segment SGI131585[] = {{&S14465[91], 6}, {NULL}};
const Segment SGI131586[] = {{&S14467[44], 14}, {NULL}};
const Segment SGI131587[] = {{&S14467[44], 26}, {NULL}};
const Segment SGI131588[] = {{&S14467[30], 6}, {NULL}};
const Segment SGI131589[] = {{&S14467[119], 3}, {NULL}};
const Segment SGI131590[] = {{&S14467[84], 6}, {NULL}};
const Segment SGI131591[] = {{&S14469[44], 9}, {NULL}};
const Segment SGI131592[] = {{&S14469[44], 21}, {NULL}};
const Segment SGI131593[] = {{&S14469[30], 6}, {NULL}};
const Segment SGI131594[] = {{&S14469[114], 3}, {NULL}};
const Segment SGI131595[] = {{&S14469[79], 6}, {NULL}};
const Segment SGI131596[] = {{&S14471[44], 7}, {NULL}};
const Segment SGI131597[] = {{&S14471[44], 19}, {NULL}};
const Segment SGI131598[] = {{&S14471[30], 6}, {NULL}};
const Segment SGI131599[] = {{&S14471[112], 3}, {NULL}};
const Segment SGI131600[] = {{&S14471[77], 6}, {NULL}};
const Segment SGI131601[] = {{&S13721[44], 11}, {NULL}};
const Segment SGI131602[] = {{&S13721[44], 23}, {NULL}};
const Segment SGI131603[] = {{&S13721[30], 6}, {NULL}};
const Segment SGI131604[] = {{&S13721[116], 3}, {NULL}};
const Segment SGI131605[] = {{&S14472[44], 7}, {NULL}};
const Segment SGI131606[] = {{&S13721[81], 6}, {NULL}};
const Segment SGI131607[] = {{&S14472[44], 19}, {NULL}};
const Segment SGI131608[] = {{&S14472[30], 6}, {NULL}};
const Segment SGI131609[] = {{&S14472[112], 3}, {NULL}};
const Segment SGI131610[] = {{&S14472[77], 6}, {NULL}};
const Segment SGI131611[] = {{&S14473[44], 13}, {NULL}};
const Segment SGI131612[] = {{&S14473[44], 25}, {NULL}};
const Segment SGI131613[] = {{&S14473[30], 6}, {NULL}};
const Segment SGI131614[] = {{&S13723[44], 6}, {NULL}};
const Segment SGI131615[] = {{&S13723[44], 18}, {NULL}};
const Segment SGI131616[] = {{&S14473[118], 3}, {NULL}};
const Segment SGI131617[] = {{&S13723[30], 6}, {NULL}};
const Segment SGI131618[] = {{&S14473[83], 6}, {NULL}};
const Segment SGI131619[] = {{&S13723[111], 3}, {NULL}};
const Segment SGI131620[] = {{&S13723[76], 6}, {NULL}};
const Segment SGI131621[] = {{&S14476[44], 10}, {NULL}};
const Segment SGI131622[] = {{&S14476[44], 22}, {NULL}};
const Segment SGI131623[] = {{&S14476[30], 6}, {NULL}};
const Segment SGI131624[] = {{&S14476[115], 3}, {NULL}};
const Segment SGI131625[] = {{&S14476[80], 6}, {NULL}};
const Segment SGI131626[] = {{&S13727[44], 7}, {NULL}};
const Segment SGI131627[] = {{&S13727[44], 19}, {NULL}};
const Segment SGI131628[] = {{&S13727[30], 6}, {NULL}};
const Segment SGI131629[] = {{&S13727[112], 3}, {NULL}};
const Segment SGI131630[] = {{&S13727[77], 6}, {NULL}};
const Segment SGI131631[] = {{&S14478[44], 13}, {NULL}};
const Segment SGI131632[] = {{&S14478[44], 25}, {NULL}};
const Segment SGI131633[] = {{&S14478[30], 6}, {NULL}};
const Segment SGI131634[] = {{&S14478[118], 3}, {NULL}};
const Segment SGI131635[] = {{&S14478[83], 6}, {NULL}};
const Segment SGI131636[] = {{&S14479[44], 10}, {NULL}};
const Segment SGI131637[] = {{&S14479[44], 22}, {NULL}};
const Segment SGI131638[] = {{&S14479[30], 6}, {NULL}};
const Segment SGI131639[] = {{&S14479[114], 3}, {NULL}};
const Segment SGI131640[] = {{&S14479[80], 5}, {NULL}};
const Segment SGI131641[] = {{&S14480[44], 8}, {NULL}};
const Segment SGI131642[] = {{&S14480[44], 20}, {NULL}};
const Segment SGI131643[] = {{&S14480[30], 6}, {NULL}};
const Segment SGI131644[] = {{&S14480[113], 3}, {NULL}};
const Segment SGI131645[] = {{&S14480[78], 6}, {NULL}};
const Segment SGI131646[] = {{&S13733[44], 7}, {NULL}};
const Segment SGI131647[] = {{&S13733[44], 19}, {NULL}};
const Segment SGI131648[] = {{&S13733[30], 6}, {NULL}};
const Segment SGI131649[] = {{&S13733[112], 3}, {NULL}};
const Segment SGI131650[] = {{&S13733[77], 6}, {NULL}};
const Segment SGI131651[] = {{&S13734[44], 16}, {NULL}};
const Segment SGI131652[] = {{&S13734[44], 28}, {NULL}};
const Segment SGI131653[] = {{&S13734[30], 6}, {NULL}};
const Segment SGI131654[] = {{&S13734[121], 3}, {NULL}};
const Segment SGI131655[] = {{&S14482[44], 19}, {NULL}};
const Segment SGI131656[] = {{&S13734[86], 6}, {NULL}};
const Segment SGI131657[] = {{&S14482[44], 38}, {NULL}};
const Segment SGI131658[] = {{&S14482[30], 6}, {NULL}};
const Segment SGI131659[] = {{&S14482[131], 3}, {NULL}};
const Segment SGI131660[] = {{&S14482[96], 6}, {NULL}};
const Segment SGI131661[] = {{&S14484[44], 7}, {NULL}};
const Segment SGI131662[] = {{&S14484[44], 19}, {NULL}};
const Segment SGI131663[] = {{&S14484[30], 6}, {NULL}};
const Segment SGI131664[] = {{&S14484[112], 3}, {NULL}};
const Segment SGI131665[] = {{&S14484[77], 6}, {NULL}};
const Segment SGI131666[] = {{&S14485[44], 11}, {NULL}};
const Segment SGI131667[] = {{&S14485[44], 23}, {NULL}};
const Segment SGI131668[] = {{&S14485[30], 6}, {NULL}};
const Segment SGI131669[] = {{&S14485[116], 3}, {NULL}};
const Segment SGI131670[] = {{&S14485[81], 6}, {NULL}};
const Segment SGI131671[] = {{&S14488[44], 15}, {NULL}};
const Segment SGI131672[] = {{&S14488[44], 34}, {NULL}};
const Segment SGI131673[] = {{&S14488[30], 6}, {NULL}};
const Segment SGI131674[] = {{&S14488[127], 3}, {NULL}};
const Segment SGI131675[] = {{&S14488[92], 6}, {NULL}};
const Segment SGI131676[] = {{&S14490[44], 12}, {NULL}};
const Segment SGI131677[] = {{&S14490[44], 24}, {NULL}};
const Segment SGI131678[] = {{&S14490[30], 6}, {NULL}};
const Segment SGI131679[] = {{&S14490[117], 3}, {NULL}};
const Segment SGI131680[] = {{&S14490[82], 6}, {NULL}};
const Segment SGI131681[] = {{&S14491[44], 8}, {NULL}};
const Segment SGI131682[] = {{&S14491[44], 20}, {NULL}};
const Segment SGI131683[] = {{&S14491[30], 6}, {NULL}};
const Segment SGI131684[] = {{&S14491[113], 3}, {NULL}};
const Segment SGI131685[] = {{&S14491[78], 6}, {NULL}};
const Segment SGI131686[] = {{&S14492[44], 12}, {NULL}};
const Segment SGI131687[] = {{&S14492[44], 24}, {NULL}};
const Segment SGI131688[] = {{&S14492[30], 6}, {NULL}};
const Segment SGI131689[] = {{&S14492[117], 3}, {NULL}};
const Segment SGI131690[] = {{&S14492[82], 6}, {NULL}};
const Segment SGI131691[] = {{&S14494[44], 6}, {NULL}};
const Segment SGI131692[] = {{&S14494[44], 18}, {NULL}};
const Segment SGI131693[] = {{&S14494[30], 6}, {NULL}};
const Segment SGI131694[] = {{&S14494[111], 3}, {NULL}};
const Segment SGI131695[] = {{&S14494[76], 6}, {NULL}};
const Segment SGI131696[] = {{&S14495[44], 10}, {NULL}};
const Segment SGI131697[] = {{&S14495[44], 22}, {NULL}};
const Segment SGI131698[] = {{&S14495[30], 6}, {NULL}};
const Segment SGI131699[] = {{&S14495[115], 3}, {NULL}};
const Segment SGI131700[] = {{&S14495[80], 6}, {NULL}};
const Segment SGI131701[] = {{&S24222[44], 7}, {NULL}};
const Segment SGI131702[] = {{&S24222[44], 24}, {NULL}};
const Segment SGI131703[] = {{&S24222[30], 6}, {NULL}};
const Segment SGI131704[] = {{&S24222[31], 3}, {NULL}};
const Segment SGI131705[] = {{&S24222[82], 6}, {NULL}};
const Segment SGI131706[] = {{&S24224[44], 7}, {NULL}};
const Segment SGI131707[] = {{&S24224[44], 24}, {NULL}};
const Segment SGI131708[] = {{&S24224[30], 6}, {NULL}};
const Segment SGI131709[] = {{&S24224[31], 3}, {NULL}};
const Segment SGI131710[] = {{&S24224[82], 6}, {NULL}};
const Segment SGI131711[] = {{&S24225[44], 7}, {NULL}};
const Segment SGI131712[] = {{&S24225[44], 24}, {NULL}};
const Segment SGI131713[] = {{&S24225[30], 6}, {NULL}};
const Segment SGI131714[] = {{&S24225[85], 6}, {NULL}};
const Segment SGI131715[] = {{&S24226[38], 7}, {NULL}};
const Segment SGI131716[] = {{&S24226[31], 6}, {NULL}};
const Segment SGI131717[] = {{&S24228[44], 7}, {NULL}};
const Segment SGI131718[] = {{&S24228[44], 25}, {NULL}};
const Segment SGI131719[] = {{&S24228[30], 6}, {NULL}};
const Segment SGI131720[] = {{&S24228[31], 3}, {NULL}};
const Segment SGI131721[] = {{&S24228[83], 6}, {NULL}};
const Segment SGI131722[] = {{&S24229[44], 7}, {NULL}};
const Segment SGI131723[] = {{&S24229[44], 24}, {NULL}};
const Segment SGI131724[] = {{&S24229[30], 6}, {NULL}};
const Segment SGI131725[] = {{&S24229[31], 3}, {NULL}};
const Segment SGI131726[] = {{&S24229[82], 6}, {NULL}};
const Segment SGI131727[] = {{&S24230[44], 7}, {NULL}};
const Segment SGI131728[] = {{&S24230[44], 25}, {NULL}};
const Segment SGI131729[] = {{&S24230[30], 6}, {NULL}};
const Segment SGI131730[] = {{&S24230[86], 6}, {NULL}};
const Segment SGI131731[] = {{&S24231[44], 7}, {NULL}};
const Segment SGI131732[] = {{&S24231[44], 25}, {NULL}};
const Segment SGI131733[] = {{&S24231[30], 6}, {NULL}};
const Segment SGI131734[] = {{&S24231[31], 3}, {NULL}};
const Segment SGI131735[] = {{&S24231[83], 6}, {NULL}};
const Segment SGI131736[] = {{&S13736[44], 16}, {NULL}};
const Segment SGI131737[] = {{&S13736[44], 28}, {NULL}};
const Segment SGI131738[] = {{&S13736[30], 6}, {NULL}};
const Segment SGI131739[] = {{&S13736[121], 3}, {NULL}};
const Segment SGI131740[] = {{&S13736[86], 6}, {NULL}};
const Segment SGI131741[] = {{&S13740[44], 31}, {NULL}};
const Segment SGI131742[] = {{&S13740[30], 6}, {NULL}};
const Segment SGI131743[] = {{&S13748[44], 15}, {NULL}};
const Segment SGI131744[] = {{&S13748[44], 26}, {NULL}};
const Segment SGI131745[] = {{&S13748[30], 6}, {NULL}};
const Segment SGI131746[] = {{&S13748[119], 3}, {NULL}};
const Segment SGI131747[] = {{&S13748[84], 6}, {NULL}};
const Segment SGI131748[] = {{&S13753[34], 16}, {NULL}};
const Segment SGI131749[] = {{&S13753[27], 6}, {NULL}};
const Segment SGI131750[] = {{&S13753[111], 3}, {NULL}};
const Segment SGI131751[] = {{&S13753[76], 6}, {NULL}};
const Segment SGI131752[] = {{&S13756[44], 8}, {NULL}};
const Segment SGI131753[] = {{&S13756[44], 20}, {NULL}};
const Segment SGI131754[] = {{&S13756[30], 6}, {NULL}};
const Segment SGI131755[] = {{&S13756[113], 3}, {NULL}};
const Segment SGI131756[] = {{&S13756[78], 6}, {NULL}};
const Segment SGI131757[] = {{&S13758[44], 10}, {NULL}};
const Segment SGI131758[] = {{&S13758[44], 27}, {NULL}};
const Segment SGI131759[] = {{&S13758[30], 6}, {NULL}};
const Segment SGI131760[] = {{&S13758[119], 3}, {NULL}};
const Segment SGI131761[] = {{&S13758[85], 5}, {NULL}};
const Segment SGI131762[] = {{&S13759[44], 16}, {NULL}};
const Segment SGI131763[] = {{&S13759[44], 35}, {NULL}};
const Segment SGI131764[] = {{&S13759[30], 6}, {NULL}};
const Segment SGI131765[] = {{&S13759[128], 3}, {NULL}};
const Segment SGI131766[] = {{&S13759[93], 6}, {NULL}};
const Segment SGI131767[] = {{&S13765[44], 6}, {NULL}};
const Segment SGI131768[] = {{&S13765[44], 18}, {NULL}};
const Segment SGI131769[] = {{&S13765[30], 6}, {NULL}};
const Segment SGI131770[] = {{&S13765[111], 3}, {NULL}};
const Segment SGI131771[] = {{&S13765[76], 6}, {NULL}};
const Segment SGI131772[] = {{&S13766[44], 11}, {NULL}};
const Segment SGI131773[] = {{&S13766[44], 23}, {NULL}};
const Segment SGI131774[] = {{&S13766[30], 6}, {NULL}};
const Segment SGI131775[] = {{&S13766[116], 3}, {NULL}};
const Segment SGI131776[] = {{&S13766[81], 6}, {NULL}};
const Segment SGI131777[] = {{&S13777[44], 16}, {NULL}};
const Segment SGI131778[] = {{&S13777[44], 28}, {NULL}};
const Segment SGI131779[] = {{&S13777[30], 6}, {NULL}};
const Segment SGI131780[] = {{&S13777[121], 3}, {NULL}};
const Segment SGI131781[] = {{&S13777[86], 6}, {NULL}};
const Segment SGI131782[] = {{&S24609[44], 29}, {NULL}};
const Segment SGI131783[] = {{&S24609[44], 41}, {NULL}};
const Segment SGI131784[] = {{&S24609[30], 6}, {NULL}};
const Segment SGI131785[] = {{&S24609[31], 3}, {NULL}};
const Segment SGI131786[] = {{&S24609[99], 6}, {NULL}};
const Segment SGI131787[] = {{&S24610[44], 41}, {NULL}};
const Segment SGI131788[] = {{&S24610[44], 53}, {NULL}};
const Segment SGI131789[] = {{&S24610[30], 6}, {NULL}};
const Segment SGI131790[] = {{&S24610[31], 3}, {NULL}};
const Segment SGI131791[] = {{&S24610[111], 6}, {NULL}};
const Segment SGI131792[] = {{&S24611[44], 41}, {NULL}};
const Segment SGI131793[] = {{&S24611[44], 53}, {NULL}};
const Segment SGI131794[] = {{&S24611[30], 6}, {NULL}};
const Segment SGI131795[] = {{&S24611[31], 3}, {NULL}};
const Segment SGI131796[] = {{&S24611[111], 6}, {NULL}};
const Segment SGI131797[] = {{&S13802[44], 14}, {NULL}};
const Segment SGI131798[] = {{&S13802[44], 26}, {NULL}};
const Segment SGI131799[] = {{&S13802[30], 6}, {NULL}};
const Segment SGI131800[] = {{&S13802[87], 6}, {NULL}};
const Segment SGI131801[] = {{&S24612[38], 41}, {NULL}};
const Segment SGI131802[] = {{&S24612[31], 6}, {NULL}};
const Segment SGI131803[] = {{&S24613[44], 29}, {NULL}};
const Segment SGI131804[] = {{&S24613[44], 41}, {NULL}};
const Segment SGI131805[] = {{&S24613[30], 6}, {NULL}};
const Segment SGI131806[] = {{&S24613[31], 3}, {NULL}};
const Segment SGI131807[] = {{&S24613[99], 6}, {NULL}};
const Segment SGI131808[] = {{&S13805[44], 6}, {NULL}};
const Segment SGI131809[] = {{&S13805[44], 18}, {NULL}};
const Segment SGI131810[] = {{&S13805[30], 6}, {NULL}};
const Segment SGI131811[] = {{&S13805[111], 3}, {NULL}};
const Segment SGI131812[] = {{&S13805[76], 6}, {NULL}};
const Segment SGI131813[] = {{&S13809[44], 18}, {NULL}};
const Segment SGI131814[] = {{&S13809[44], 39}, {NULL}};
const Segment SGI131815[] = {{&S13809[30], 6}, {NULL}};
const Segment SGI131816[] = {{&S13809[132], 3}, {NULL}};
const Segment SGI131817[] = {{&S13809[97], 6}, {NULL}};
const Segment SGI131818[] = {{&S13819[44], 7}, {NULL}};
const Segment SGI131819[] = {{&S13819[44], 19}, {NULL}};
const Segment SGI131820[] = {{&S13819[30], 6}, {NULL}};
const Segment SGI131821[] = {{&S13819[112], 3}, {NULL}};
const Segment SGI131822[] = {{&S13819[77], 6}, {NULL}};
const Segment SGI131823[] = {{&S13830[44], 19}, {NULL}};
const Segment SGI131824[] = {{&S13830[44], 31}, {NULL}};
const Segment SGI131825[] = {{&S13830[30], 6}, {NULL}};
const Segment SGI131826[] = {{&S13830[124], 3}, {NULL}};
const Segment SGI131827[] = {{&S13830[89], 6}, {NULL}};
const Segment SGI131828[] = {{&S13831[44], 10}, {NULL}};
const Segment SGI131829[] = {{&S13831[44], 22}, {NULL}};
const Segment SGI131830[] = {{&S13831[30], 6}, {NULL}};
const Segment SGI131831[] = {{&S13831[115], 3}, {NULL}};
const Segment SGI131832[] = {{&S13831[80], 6}, {NULL}};
const Segment SGI131833[] = {{&S13832[44], 8}, {NULL}};
const Segment SGI131834[] = {{&S13832[44], 20}, {NULL}};
const Segment SGI131835[] = {{&S13832[30], 6}, {NULL}};
const Segment SGI131836[] = {{&S13832[113], 3}, {NULL}};
const Segment SGI131837[] = {{&S13832[78], 6}, {NULL}};
const Segment SGI131838[] = {{&S13839[44], 13}, {NULL}};
const Segment SGI131839[] = {{&S13839[44], 25}, {NULL}};
const Segment SGI131840[] = {{&S13839[30], 6}, {NULL}};
const Segment SGI131841[] = {{&S13839[118], 3}, {NULL}};
const Segment SGI131842[] = {{&S13839[83], 6}, {NULL}};
const Segment SGI131843[] = {{&S13840[44], 16}, {NULL}};
const Segment SGI131844[] = {{&S13840[44], 28}, {NULL}};
const Segment SGI131845[] = {{&S13840[30], 6}, {NULL}};
const Segment SGI131846[] = {{&S13840[121], 3}, {NULL}};
const Segment SGI131847[] = {{&S13840[86], 6}, {NULL}};
const Segment SGI131848[] = {{&S13842[44], 10}, {NULL}};
const Segment SGI131849[] = {{&S13842[44], 22}, {NULL}};
const Segment SGI131850[] = {{&S13842[30], 6}, {NULL}};
const Segment SGI131851[] = {{&S13842[115], 3}, {NULL}};
const Segment SGI131852[] = {{&S13842[80], 6}, {NULL}};
const Segment SGI131853[] = {{&S13845[44], 9}, {NULL}};
const Segment SGI131854[] = {{&S13845[44], 21}, {NULL}};
const Segment SGI131855[] = {{&S13845[30], 6}, {NULL}};
const Segment SGI131856[] = {{&S13845[114], 3}, {NULL}};
const Segment SGI131857[] = {{&S13845[79], 6}, {NULL}};
const Segment SGI131858[] = {{&S13847[44], 17}, {NULL}};
const Segment SGI131859[] = {{&S13847[44], 29}, {NULL}};
const Segment SGI131860[] = {{&S13847[30], 6}, {NULL}};
const Segment SGI131861[] = {{&S13847[122], 3}, {NULL}};
const Segment SGI131862[] = {{&S13847[87], 6}, {NULL}};
const Segment SGI131863[] = {{&S13852[44], 7}, {NULL}};
const Segment SGI131864[] = {{&S13852[44], 19}, {NULL}};
const Segment SGI131865[] = {{&S13852[30], 6}, {NULL}};
const Segment SGI131866[] = {{&S13852[112], 3}, {NULL}};
const Segment SGI131867[] = {{&S13852[77], 6}, {NULL}};
const Segment SGI131868[] = {{&S13855[44], 10}, {NULL}};
const Segment SGI131869[] = {{&S13855[44], 22}, {NULL}};
const Segment SGI131870[] = {{&S13855[30], 6}, {NULL}};
const Segment SGI131871[] = {{&S13855[115], 3}, {NULL}};
const Segment SGI131872[] = {{&S13855[80], 6}, {NULL}};
const Segment SGI131873[] = {{&S13864[44], 11}, {NULL}};
const Segment SGI131874[] = {{&S13864[44], 23}, {NULL}};
const Segment SGI131875[] = {{&S13864[30], 6}, {NULL}};
const Segment SGI131876[] = {{&S13864[116], 3}, {NULL}};
const Segment SGI131877[] = {{&S13864[81], 6}, {NULL}};
const Segment SGI131878[] = {{&S13865[44], 8}, {NULL}};
const Segment SGI131879[] = {{&S13865[44], 20}, {NULL}};
const Segment SGI131880[] = {{&S13865[30], 6}, {NULL}};
const Segment SGI131881[] = {{&S13865[113], 3}, {NULL}};
const Segment SGI131882[] = {{&S13865[78], 6}, {NULL}};
const Segment SGI131883[] = {{&S13870[44], 7}, {NULL}};
const Segment SGI131884[] = {{&S13870[44], 19}, {NULL}};
const Segment SGI131885[] = {{&S13870[30], 6}, {NULL}};
const Segment SGI131886[] = {{&S13870[112], 3}, {NULL}};
const Segment SGI131887[] = {{&S13870[77], 6}, {NULL}};
const Segment SGI131888[] = {{&S13875[44], 10}, {NULL}};
const Segment SGI131889[] = {{&S13875[44], 22}, {NULL}};
const Segment SGI131890[] = {{&S13875[30], 6}, {NULL}};
const Segment SGI131891[] = {{&S13875[115], 3}, {NULL}};
const Segment SGI131892[] = {{&S13875[80], 6}, {NULL}};
const Segment SGI131893[] = {{&S13879[44], 5}, {NULL}};
const Segment SGI131894[] = {{&S13879[44], 17}, {NULL}};
const Segment SGI131895[] = {{&S13879[30], 6}, {NULL}};
const Segment SGI131896[] = {{&S13879[110], 3}, {NULL}};
const Segment SGI131897[] = {{&S13879[75], 6}, {NULL}};
const Segment SGI131898[] = {{&S13891[44], 8}, {NULL}};
const Segment SGI131899[] = {{&S13891[44], 20}, {NULL}};
const Segment SGI131900[] = {{&S13891[30], 6}, {NULL}};
const Segment SGI131901[] = {{&S13891[113], 3}, {NULL}};
const Segment SGI131902[] = {{&S13891[78], 6}, {NULL}};
const Segment SGI131903[] = {{&S13896[44], 10}, {NULL}};
const Segment SGI131904[] = {{&S13896[44], 22}, {NULL}};
const Segment SGI131905[] = {{&S13896[30], 6}, {NULL}};
const Segment SGI131906[] = {{&S13896[115], 3}, {NULL}};
const Segment SGI131907[] = {{&S13896[80], 6}, {NULL}};
const Segment SGI131908[] = {{&S13900[44], 21}, {NULL}};
const Segment SGI131909[] = {{&S13900[44], 33}, {NULL}};
const Segment SGI131910[] = {{&S13900[30], 6}, {NULL}};
const Segment SGI131911[] = {{&S13900[126], 3}, {NULL}};
const Segment SGI131912[] = {{&S13900[91], 6}, {NULL}};
const Segment SGI131913[] = {{&S13902[44], 11}, {NULL}};
const Segment SGI131914[] = {{&S13902[44], 23}, {NULL}};
const Segment SGI131915[] = {{&S13902[30], 6}, {NULL}};
const Segment SGI131916[] = {{&S13902[116], 3}, {NULL}};
const Segment SGI131917[] = {{&S13902[81], 6}, {NULL}};
const Segment SGI131918[] = {{&S13903[44], 14}, {NULL}};
const Segment SGI131919[] = {{&S13903[44], 26}, {NULL}};
const Segment SGI131920[] = {{&S13903[30], 6}, {NULL}};
const Segment SGI131921[] = {{&S13903[119], 3}, {NULL}};
const Segment SGI131922[] = {{&S13903[84], 6}, {NULL}};
const Segment SGI131923[] = {{&S13909[44], 16}, {NULL}};
const Segment SGI131924[] = {{&S13909[44], 35}, {NULL}};
const Segment SGI131925[] = {{&S13909[30], 6}, {NULL}};
const Segment SGI131926[] = {{&S13909[128], 3}, {NULL}};
const Segment SGI131927[] = {{&S13909[93], 6}, {NULL}};
const Segment SGI131928[] = {{&S13911[44], 10}, {NULL}};
const Segment SGI131929[] = {{&S13911[44], 22}, {NULL}};
const Segment SGI131930[] = {{&S13911[30], 6}, {NULL}};
const Segment SGI131931[] = {{&S13911[31], 3}, {NULL}};
const Segment SGI131932[] = {{&S13911[80], 6}, {NULL}};
const Segment SGI131933[] = {{&S13916[38], 21}, {NULL}};
const Segment SGI131934[] = {{&S13916[31], 6}, {NULL}};
const Segment SGI131935[] = {{&S13926[44], 11}, {NULL}};
const Segment SGI131936[] = {{&S13926[44], 22}, {NULL}};
const Segment SGI131937[] = {{&S13926[30], 6}, {NULL}};
const Segment SGI131938[] = {{&S13926[115], 3}, {NULL}};
const Segment SGI131939[] = {{&S13926[80], 6}, {NULL}};
const Segment SGI131940[] = {{&S13933[44], 12}, {NULL}};
const Segment SGI131941[] = {{&S13933[44], 24}, {NULL}};
const Segment SGI131942[] = {{&S13933[30], 6}, {NULL}};
const Segment SGI131943[] = {{&S13933[117], 3}, {NULL}};
const Segment SGI131944[] = {{&S13933[82], 6}, {NULL}};
const Segment SGI131945[] = {{&S13934[44], 13}, {NULL}};
const Segment SGI131946[] = {{&S13934[44], 25}, {NULL}};
const Segment SGI131947[] = {{&S13934[30], 6}, {NULL}};
const Segment SGI131948[] = {{&S13934[118], 3}, {NULL}};
const Segment SGI131949[] = {{&S13934[83], 6}, {NULL}};
const Segment SGI131950[] = {{&S13936[44], 13}, {NULL}};
const Segment SGI131951[] = {{&S13936[44], 25}, {NULL}};
const Segment SGI131952[] = {{&S13936[30], 6}, {NULL}};
const Segment SGI131953[] = {{&S13936[118], 3}, {NULL}};
const Segment SGI131954[] = {{&S13936[83], 6}, {NULL}};
const Segment SGI131955[] = {{&S13943[44], 7}, {NULL}};
const Segment SGI131956[] = {{&S13943[44], 28}, {NULL}};
const Segment SGI131957[] = {{&S13943[30], 6}, {NULL}};
const Segment SGI131958[] = {{&S13943[120], 3}, {NULL}};
const Segment SGI131959[] = {{&S13943[86], 5}, {NULL}};
const Segment SGI131960[] = {{&S13949[44], 9}, {NULL}};
const Segment SGI131961[] = {{&S13949[44], 21}, {NULL}};
const Segment SGI131962[] = {{&S13949[30], 6}, {NULL}};
const Segment SGI131963[] = {{&S13949[114], 3}, {NULL}};
const Segment SGI131964[] = {{&S13949[79], 6}, {NULL}};
const Segment SGI131965[] = {{&S13955[44], 7}, {NULL}};
const Segment SGI131966[] = {{&S13955[44], 19}, {NULL}};
const Segment SGI131967[] = {{&S13955[30], 6}, {NULL}};
const Segment SGI131968[] = {{&S13955[112], 3}, {NULL}};
const Segment SGI131969[] = {{&S13955[77], 6}, {NULL}};
const Segment SGI131970[] = {{&S13957[44], 8}, {NULL}};
const Segment SGI131971[] = {{&S13957[44], 20}, {NULL}};
const Segment SGI131972[] = {{&S13957[30], 6}, {NULL}};
const Segment SGI131973[] = {{&S13957[113], 3}, {NULL}};
const Segment SGI131974[] = {{&S13957[78], 6}, {NULL}};
const Segment SGI131975[] = {{&S13958[44], 8}, {NULL}};
const Segment SGI131976[] = {{&S13958[44], 20}, {NULL}};
const Segment SGI131977[] = {{&S13958[30], 6}, {NULL}};
const Segment SGI131978[] = {{&S13958[113], 3}, {NULL}};
const Segment SGI131979[] = {{&S13958[78], 6}, {NULL}};
const Segment SGI131980[] = {{&S13961[44], 10}, {NULL}};
const Segment SGI131981[] = {{&S13961[44], 22}, {NULL}};
const Segment SGI131982[] = {{&S13961[30], 6}, {NULL}};
const Segment SGI131983[] = {{&S13961[115], 3}, {NULL}};
const Segment SGI131984[] = {{&S13961[80], 6}, {NULL}};
const Segment SGI131985[] = {{&S13962[38], 20}, {NULL}};
const Segment SGI131986[] = {{&S13962[31], 6}, {NULL}};
const Segment SGI131987[] = {{&S13964[44], 10}, {NULL}};
const Segment SGI131988[] = {{&S13964[44], 22}, {NULL}};
const Segment SGI131989[] = {{&S13964[30], 6}, {NULL}};
const Segment SGI131990[] = {{&S13964[115], 3}, {NULL}};
const Segment SGI131991[] = {{&S13964[80], 6}, {NULL}};
const Segment SGI131992[] = {{&S13965[44], 9}, {NULL}};
const Segment SGI131993[] = {{&S13965[44], 21}, {NULL}};
const Segment SGI131994[] = {{&S13965[30], 6}, {NULL}};
const Segment SGI131995[] = {{&S13965[114], 3}, {NULL}};
const Segment SGI131996[] = {{&S13965[79], 6}, {NULL}};
const Segment SGI131997[] = {{&S13967[44], 18}, {NULL}};
const Segment SGI131998[] = {{&S13967[44], 30}, {NULL}};
const Segment SGI131999[] = {{&S13967[30], 6}, {NULL}};
const Segment SGI132000[] = {{&S13967[123], 3}, {NULL}};
const Segment SGI132001[] = {{&S13967[88], 6}, {NULL}};
const Segment SGI132002[] = {{&S13968[44], 10}, {NULL}};
const Segment SGI132003[] = {{&S13968[44], 22}, {NULL}};
const Segment SGI132004[] = {{&S13968[30], 6}, {NULL}};
const Segment SGI132005[] = {{&S13968[115], 3}, {NULL}};
const Segment SGI132006[] = {{&S13968[80], 6}, {NULL}};
const Segment SGI132007[] = {{&S13969[44], 12}, {NULL}};
const Segment SGI132008[] = {{&S13969[44], 24}, {NULL}};
const Segment SGI132009[] = {{&S13969[30], 6}, {NULL}};
const Segment SGI132010[] = {{&S13969[117], 3}, {NULL}};
const Segment SGI132011[] = {{&S13969[82], 6}, {NULL}};
const Segment SGI132012[] = {{&S13975[44], 84}, {NULL}};
const Segment SGI132013[] = {{&S13975[44], 96}, {NULL}};
const Segment SGI132014[] = {{&S13975[30], 6}, {NULL}};
const Segment SGI132015[] = {{&S13975[189], 3}, {NULL}};
const Segment SGI132016[] = {{&S13975[121], 6}, {NULL}};
const Segment SGI132017[] = {{&S13976[44], 4}, {NULL}};
const Segment SGI132018[] = {{&S13976[44], 14}, {NULL}};
const Segment SGI132019[] = {{&S13976[30], 6}, {NULL}};
const Segment SGI132020[] = {{&S13976[107], 3}, {NULL}};
const Segment SGI132021[] = {{&S13976[72], 6}, {NULL}};
const Segment SGI132022[] = {{&S13979[44], 10}, {NULL}};
const Segment SGI132023[] = {{&S13979[44], 22}, {NULL}};
const Segment SGI132024[] = {{&S13979[30], 6}, {NULL}};
const Segment SGI132025[] = {{&S13979[115], 3}, {NULL}};
const Segment SGI132026[] = {{&S13979[80], 6}, {NULL}};
const Segment SGI132027[] = {{&S13980[44], 10}, {NULL}};
const Segment SGI132028[] = {{&S13980[44], 22}, {NULL}};
const Segment SGI132029[] = {{&S13980[30], 6}, {NULL}};
const Segment SGI132030[] = {{&S13980[115], 3}, {NULL}};
const Segment SGI132031[] = {{&S13980[80], 6}, {NULL}};
const Segment SGI132032[] = {{&S13990[44], 33}, {NULL}};
const Segment SGI132033[] = {{&S13990[30], 6}, {NULL}};
const Segment SGI132034[] = {{&S13992[44], 5}, {NULL}};
const Segment SGI132035[] = {{&S13992[44], 17}, {NULL}};
const Segment SGI132036[] = {{&S13992[30], 6}, {NULL}};
const Segment SGI132037[] = {{&S13992[110], 3}, {NULL}};
const Segment SGI132038[] = {{&S13992[75], 6}, {NULL}};
const Segment SGI132039[] = {{&S13994[44], 13}, {NULL}};
const Segment SGI132040[] = {{&S13994[44], 25}, {NULL}};
const Segment SGI132041[] = {{&S13994[30], 6}, {NULL}};
const Segment SGI132042[] = {{&S13994[118], 3}, {NULL}};
const Segment SGI132043[] = {{&S13994[83], 6}, {NULL}};
const Segment SGI132044[] = {{&S13999[44], 9}, {NULL}};
const Segment SGI132045[] = {{&S13999[44], 21}, {NULL}};
const Segment SGI132046[] = {{&S13999[30], 6}, {NULL}};
const Segment SGI132047[] = {{&S13999[113], 3}, {NULL}};
const Segment SGI132048[] = {{&S13999[79], 5}, {NULL}};
const Segment SGI132049[] = {{&S14005[44], 7}, {NULL}};
const Segment SGI132050[] = {{&S14005[44], 19}, {NULL}};
const Segment SGI132051[] = {{&S14005[30], 6}, {NULL}};
const Segment SGI132052[] = {{&S14005[112], 3}, {NULL}};
const Segment SGI132053[] = {{&S14005[77], 6}, {NULL}};
const Segment SGI132054[] = {{&S14006[44], 13}, {NULL}};
const Segment SGI132055[] = {{&S14006[44], 25}, {NULL}};
const Segment SGI132056[] = {{&S14006[30], 6}, {NULL}};
const Segment SGI132057[] = {{&S14006[118], 3}, {NULL}};
const Segment SGI132058[] = {{&S14006[83], 6}, {NULL}};
const Segment SGI132059[] = {{&S14008[44], 6}, {NULL}};
const Segment SGI132060[] = {{&S14008[44], 18}, {NULL}};
const Segment SGI132061[] = {{&S14008[30], 6}, {NULL}};
const Segment SGI132062[] = {{&S14008[111], 3}, {NULL}};
const Segment SGI132063[] = {{&S14008[76], 6}, {NULL}};
const Segment SGI132064[] = {{&S14009[44], 17}, {NULL}};
const Segment SGI132065[] = {{&S14009[44], 29}, {NULL}};
const Segment SGI132066[] = {{&S14009[30], 6}, {NULL}};
const Segment SGI132067[] = {{&S14009[122], 3}, {NULL}};
const Segment SGI132068[] = {{&S14009[87], 6}, {NULL}};
const Segment SGI132069[] = {{&S14014[44], 19}, {NULL}};
const Segment SGI132070[] = {{&S14014[44], 31}, {NULL}};
const Segment SGI132071[] = {{&S14014[30], 6}, {NULL}};
const Segment SGI132072[] = {{&S14014[124], 3}, {NULL}};
const Segment SGI132073[] = {{&S14014[89], 6}, {NULL}};
const Segment SGI132074[] = {{&S14017[44], 12}, {NULL}};
const Segment SGI132075[] = {{&S14017[44], 21}, {NULL}};
const Segment SGI132076[] = {{&S14017[30], 6}, {NULL}};
const Segment SGI132077[] = {{&S14017[114], 3}, {NULL}};
const Segment SGI132078[] = {{&S14017[79], 6}, {NULL}};
const Segment SGI132079[] = {{&S14020[44], 6}, {NULL}};
const Segment SGI132080[] = {{&S14020[44], 18}, {NULL}};
const Segment SGI132081[] = {{&S14020[30], 6}, {NULL}};
const Segment SGI132082[] = {{&S14020[111], 3}, {NULL}};
const Segment SGI132083[] = {{&S14020[76], 6}, {NULL}};
const Segment SGI132084[] = {{&S24885[44], 10}, {NULL}};
const Segment SGI132085[] = {{&S24885[44], 22}, {NULL}};
const Segment SGI132086[] = {{&S24885[30], 6}, {NULL}};
const Segment SGI132087[] = {{&S24885[31], 3}, {NULL}};
const Segment SGI132088[] = {{&S24885[80], 6}, {NULL}};
const Segment SGI132089[] = {{&S24886[38], 10}, {NULL}};
const Segment SGI132090[] = {{&S24886[31], 6}, {NULL}};
const Segment SGI132091[] = {{&S24887[44], 10}, {NULL}};
const Segment SGI132092[] = {{&S24887[44], 22}, {NULL}};
const Segment SGI132093[] = {{&S24887[30], 6}, {NULL}};
const Segment SGI132094[] = {{&S24887[31], 3}, {NULL}};
const Segment SGI132095[] = {{&S24887[80], 6}, {NULL}};
const Segment SGI132096[] = {{&S14026[44], 10}, {NULL}};
const Segment SGI132097[] = {{&S14026[44], 21}, {NULL}};
const Segment SGI132098[] = {{&S14026[30], 6}, {NULL}};
const Segment SGI132099[] = {{&S14026[114], 3}, {NULL}};
const Segment SGI132100[] = {{&S14026[79], 6}, {NULL}};
const Segment SGI132101[] = {{&S24888[42], 10}, {NULL}};
const Segment SGI132102[] = {{&S24888[30], 6}, {NULL}};
const Segment SGI132103[] = {{&S24888[31], 3}, {NULL}};
const Segment SGI132104[] = {{&S24888[78], 6}, {NULL}};
const Segment SGI132105[] = {{&S24889[44], 10}, {NULL}};
const Segment SGI132106[] = {{&S24889[44], 22}, {NULL}};
const Segment SGI132107[] = {{&S24889[30], 6}, {NULL}};
const Segment SGI132108[] = {{&S24889[31], 3}, {NULL}};
const Segment SGI132109[] = {{&S24889[80], 6}, {NULL}};
const Segment SGI132110[] = {{&S24890[44], 30}, {NULL}};
const Segment SGI132111[] = {{&S24890[44], 42}, {NULL}};
const Segment SGI132112[] = {{&S24890[30], 6}, {NULL}};
const Segment SGI132113[] = {{&S24890[31], 3}, {NULL}};
const Segment SGI132114[] = {{&S24890[100], 6}, {NULL}};
const Segment SGI132115[] = {{&S24891[44], 10}, {NULL}};
const Segment SGI132116[] = {{&S24891[44], 22}, {NULL}};
const Segment SGI132117[] = {{&S24891[30], 6}, {NULL}};
const Segment SGI132118[] = {{&S24891[83], 6}, {NULL}};
const Segment SGI132119[] = {{&S14034[44], 10}, {NULL}};
const Segment SGI132120[] = {{&S14034[44], 30}, {NULL}};
const Segment SGI132121[] = {{&S14034[30], 6}, {NULL}};
const Segment SGI132122[] = {{&S14034[123], 3}, {NULL}};
const Segment SGI132123[] = {{&S14034[88], 6}, {NULL}};
const Segment SGI132124[] = {{&S14035[44], 8}, {NULL}};
const Segment SGI132125[] = {{&S14035[44], 20}, {NULL}};
const Segment SGI132126[] = {{&S14035[30], 6}, {NULL}};
const Segment SGI132127[] = {{&S14035[113], 3}, {NULL}};
const Segment SGI132128[] = {{&S14035[78], 6}, {NULL}};
const Segment SGI132129[] = {{&S14037[44], 10}, {NULL}};
const Segment SGI132130[] = {{&S14037[44], 21}, {NULL}};
const Segment SGI132131[] = {{&S14037[30], 6}, {NULL}};
const Segment SGI132132[] = {{&S14037[114], 3}, {NULL}};
const Segment SGI132133[] = {{&S14037[79], 6}, {NULL}};
const Segment SGI132134[] = {{&S14038[44], 6}, {NULL}};
const Segment SGI132135[] = {{&S14038[44], 18}, {NULL}};
const Segment SGI132136[] = {{&S14038[30], 6}, {NULL}};
const Segment SGI132137[] = {{&S14038[111], 3}, {NULL}};
const Segment SGI132138[] = {{&S14038[76], 6}, {NULL}};
const Segment SGI132139[] = {{&S14039[44], 5}, {NULL}};
const Segment SGI132140[] = {{&S14039[44], 17}, {NULL}};
const Segment SGI132141[] = {{&S14039[30], 6}, {NULL}};
const Segment SGI132142[] = {{&S14039[110], 3}, {NULL}};
const Segment SGI132143[] = {{&S14039[75], 6}, {NULL}};
const Segment SGI132144[] = {{&S14044[44], 19}, {NULL}};
const Segment SGI132145[] = {{&S14044[44], 34}, {NULL}};
const Segment SGI132146[] = {{&S14044[30], 6}, {NULL}};
const Segment SGI132147[] = {{&S14044[127], 3}, {NULL}};
const Segment SGI132148[] = {{&S14044[92], 6}, {NULL}};
const Segment SGI132149[] = {{&S14046[44], 22}, {NULL}};
const Segment SGI132150[] = {{&S14046[44], 34}, {NULL}};
const Segment SGI132151[] = {{&S14046[30], 6}, {NULL}};
const Segment SGI132152[] = {{&S24937[44], 25}, {NULL}};
const Segment SGI132153[] = {{&S24937[44], 37}, {NULL}};
const Segment SGI132154[] = {{&S24937[30], 6}, {NULL}};
const Segment SGI132155[] = {{&S24937[31], 3}, {NULL}};
const Segment SGI132156[] = {{&S24937[95], 6}, {NULL}};
const Segment SGI132157[] = {{&S24941[44], 25}, {NULL}};
const Segment SGI132158[] = {{&S24941[44], 37}, {NULL}};
const Segment SGI132159[] = {{&S24941[30], 6}, {NULL}};
const Segment SGI132160[] = {{&S24941[31], 3}, {NULL}};
const Segment SGI132161[] = {{&S24941[95], 6}, {NULL}};
const Segment SGI132162[] = {{&S24999[44], 10}, {NULL}};
const Segment SGI132163[] = {{&S24999[44], 22}, {NULL}};
const Segment SGI132164[] = {{&S24999[30], 6}, {NULL}};
const Segment SGI132165[] = {{&S24999[31], 3}, {NULL}};
const Segment SGI132166[] = {{&S24999[80], 6}, {NULL}};
const Segment SGI132167[] = {{&S25001[44], 10}, {NULL}};
const Segment SGI132168[] = {{&S25001[44], 22}, {NULL}};
const Segment SGI132169[] = {{&S25001[30], 6}, {NULL}};
const Segment SGI132170[] = {{&S25001[31], 3}, {NULL}};
const Segment SGI132171[] = {{&S25001[80], 6}, {NULL}};
const Segment SGI132172[] = {{&S25002[44], 11}, {NULL}};
const Segment SGI132173[] = {{&S25002[44], 23}, {NULL}};
const Segment SGI132174[] = {{&S25002[30], 6}, {NULL}};
const Segment SGI132175[] = {{&S25002[31], 3}, {NULL}};
const Segment SGI132176[] = {{&S25002[81], 6}, {NULL}};
const Segment SGI132177[] = {{&S25003[38], 10}, {NULL}};
const Segment SGI132178[] = {{&S25003[31], 6}, {NULL}};
const Segment SGI132179[] = {{&S25004[44], 11}, {NULL}};
const Segment SGI132180[] = {{&S25004[44], 23}, {NULL}};
const Segment SGI132181[] = {{&S25004[30], 6}, {NULL}};
const Segment SGI132182[] = {{&S25004[31], 3}, {NULL}};
const Segment SGI132183[] = {{&S25004[81], 6}, {NULL}};
const Segment SGI132184[] = {{&S25005[44], 11}, {NULL}};
const Segment SGI132185[] = {{&S25005[44], 23}, {NULL}};
const Segment SGI132186[] = {{&S25005[30], 6}, {NULL}};
const Segment SGI132187[] = {{&S25005[31], 3}, {NULL}};
const Segment SGI132188[] = {{&S25005[81], 6}, {NULL}};
const Segment SGI132189[] = {{&S25006[38], 11}, {NULL}};
const Segment SGI132190[] = {{&S25006[31], 6}, {NULL}};
const Segment SGI132191[] = {{&S25007[42], 10}, {NULL}};
const Segment SGI132192[] = {{&S25007[30], 6}, {NULL}};
const Segment SGI132193[] = {{&S25007[31], 3}, {NULL}};
const Segment SGI132194[] = {{&S25007[78], 6}, {NULL}};
const Segment SGI132195[] = {{&S25008[44], 29}, {NULL}};
const Segment SGI132196[] = {{&S25008[44], 41}, {NULL}};
const Segment SGI132197[] = {{&S25008[30], 6}, {NULL}};
const Segment SGI132198[] = {{&S25008[31], 3}, {NULL}};
const Segment SGI132199[] = {{&S25008[99], 6}, {NULL}};
const Segment SGI132200[] = {{&S25009[44], 29}, {NULL}};
const Segment SGI132201[] = {{&S25009[44], 41}, {NULL}};
const Segment SGI132202[] = {{&S25009[30], 6}, {NULL}};
const Segment SGI132203[] = {{&S25009[31], 3}, {NULL}};
const Segment SGI132204[] = {{&S25009[99], 6}, {NULL}};
const Segment SGI132205[] = {{&S25014[44], 10}, {NULL}};
const Segment SGI132206[] = {{&S25014[44], 22}, {NULL}};
const Segment SGI132207[] = {{&S25014[30], 6}, {NULL}};
const Segment SGI132208[] = {{&S25014[125], 3}, {NULL}};
const Segment SGI132209[] = {{&S25014[90], 6}, {NULL}};
const Segment SGI132210[] = {{&S25014[145], 8}, {NULL}};
const Segment SGI132211[] = {{&S25015[38], 11}, {NULL}};
const Segment SGI132212[] = {{&S25015[31], 6}, {NULL}};
const Segment SGI132213[] = {{&S14046[127], 3}, {NULL}};
const Segment SGI132214[] = {{&S14046[92], 6}, {NULL}};
const Segment SGI132215[] = {{&S25016[44], 11}, {NULL}};
const Segment SGI132216[] = {{&S25016[44], 23}, {NULL}};
const Segment SGI132217[] = {{&S25016[30], 6}, {NULL}};
const Segment SGI132218[] = {{&S25016[31], 3}, {NULL}};
const Segment SGI132219[] = {{&S25016[81], 6}, {NULL}};
const Segment SGI132220[] = {{&S14049[34], 17}, {NULL}};
const Segment SGI132221[] = {{&S14049[27], 6}, {NULL}};
const Segment SGI132222[] = {{&S14049[112], 3}, {NULL}};
const Segment SGI132223[] = {{&S25017[38], 11}, {NULL}};
const Segment SGI132224[] = {{&S25017[31], 6}, {NULL}};
const Segment SGI132225[] = {{&S14049[77], 6}, {NULL}};
const Segment SGI132226[] = {{&S25018[44], 10}, {NULL}};
const Segment SGI132227[] = {{&S25018[44], 22}, {NULL}};
const Segment SGI132228[] = {{&S25018[30], 6}, {NULL}};
const Segment SGI132229[] = {{&S25018[31], 3}, {NULL}};
const Segment SGI132230[] = {{&S25018[80], 6}, {NULL}};
const Segment SGI132231[] = {{&S25019[44], 10}, {NULL}};
const Segment SGI132232[] = {{&S14053[44], 10}, {NULL}};
const Segment SGI132233[] = {{&S25019[44], 22}, {NULL}};
const Segment SGI132234[] = {{&S14053[44], 22}, {NULL}};
const Segment SGI132235[] = {{&S25019[30], 6}, {NULL}};
const Segment SGI132236[] = {{&S14053[30], 6}, {NULL}};
const Segment SGI132237[] = {{&S14053[115], 3}, {NULL}};
const Segment SGI132238[] = {{&S25019[80], 6}, {NULL}};
const Segment SGI132239[] = {{&S14053[80], 6}, {NULL}};
const Segment SGI132240[] = {{&S25020[44], 10}, {NULL}};
const Segment SGI132241[] = {{&S25020[44], 22}, {NULL}};
const Segment SGI132242[] = {{&S25020[30], 6}, {NULL}};
const Segment SGI132243[] = {{&S14054[44], 8}, {NULL}};
const Segment SGI132244[] = {{&S14054[44], 20}, {NULL}};
const Segment SGI132245[] = {{&S14054[30], 6}, {NULL}};
const Segment SGI132246[] = {{&S25020[31], 3}, {NULL}};
const Segment SGI132247[] = {{&S25020[80], 6}, {NULL}};
const Segment SGI132248[] = {{&S14054[113], 3}, {NULL}};
const Segment SGI132249[] = {{&S14054[78], 6}, {NULL}};
const Segment SGI132250[] = {{&S25021[44], 11}, {NULL}};
const Segment SGI132251[] = {{&S25021[44], 30}, {NULL}};
const Segment SGI132252[] = {{&S25021[30], 6}, {NULL}};
const Segment SGI132253[] = {{&S14056[44], 41}, {NULL}};
const Segment SGI132254[] = {{&S14056[30], 6}, {NULL}};
const Segment SGI132255[] = {{&S25022[44], 10}, {NULL}};
const Segment SGI132256[] = {{&S25022[44], 21}, {NULL}};
const Segment SGI132257[] = {{&S25022[30], 6}, {NULL}};
const Segment SGI132258[] = {{&S25022[31], 3}, {NULL}};
const Segment SGI132259[] = {{&S25022[79], 6}, {NULL}};
const Segment SGI132260[] = {{&S25023[44], 10}, {NULL}};
const Segment SGI132261[] = {{&S25023[44], 22}, {NULL}};
const Segment SGI132262[] = {{&S25023[30], 6}, {NULL}};
const Segment SGI132263[] = {{&S25023[80], 6}, {NULL}};
const Segment SGI132264[] = {{&S25024[44], 10}, {NULL}};
const Segment SGI132265[] = {{&S25024[44], 22}, {NULL}};
const Segment SGI132266[] = {{&S25024[30], 6}, {NULL}};
const Segment SGI132267[] = {{&S25024[31], 3}, {NULL}};
const Segment SGI132268[] = {{&S25024[80], 6}, {NULL}};
const Segment SGI132269[] = {{&S25025[44], 11}, {NULL}};
const Segment SGI132270[] = {{&S25025[44], 23}, {NULL}};
const Segment SGI132271[] = {{&S25025[30], 6}, {NULL}};
const Segment SGI132272[] = {{&S25025[81], 6}, {NULL}};
const Segment SGI132273[] = {{&S25027[42], 11}, {NULL}};
const Segment SGI132274[] = {{&S25027[30], 6}, {NULL}};
const Segment SGI132275[] = {{&S25027[31], 3}, {NULL}};
const Segment SGI132276[] = {{&S25027[79], 6}, {NULL}};
const Segment SGI132277[] = {{&S25028[42], 10}, {NULL}};
const Segment SGI132278[] = {{&S25028[30], 6}, {NULL}};
const Segment SGI132279[] = {{&S25028[78], 6}, {NULL}};
const Segment SGI132280[] = {{&S25029[44], 10}, {NULL}};
const Segment SGI132281[] = {{&S25029[44], 28}, {NULL}};
const Segment SGI132282[] = {{&S25029[30], 6}, {NULL}};
const Segment SGI132283[] = {{&S25030[44], 10}, {&S25030[175], 18}, {NULL}};
const Segment SGI132284[] = {{&S25030[44], 22}, {NULL}};
const Segment SGI132285[] = {{&S25030[30], 6}, {NULL}};
const Segment SGI132286[] = {{&S25030[31], 3}, {NULL}};
const Segment SGI132287[] = {{&S25030[80], 6}, {NULL}};
const Segment SGI132288[] = {{&S25031[44], 29}, {NULL}};
const Segment SGI132289[] = {{&S25031[44], 41}, {NULL}};
const Segment SGI132290[] = {{&S25031[30], 6}, {NULL}};
const Segment SGI132291[] = {{&S25031[31], 3}, {NULL}};
const Segment SGI132292[] = {{&S25031[99], 6}, {NULL}};
const Segment SGI132293[] = {{&S25032[44], 10}, {NULL}};
const Segment SGI132294[] = {{&S25032[44], 28}, {NULL}};
const Segment SGI132295[] = {{&S25032[30], 6}, {NULL}};
const Segment SGI132296[] = {{&S25033[65], 10}, {NULL}};
const Segment SGI132297[] = {{&S25033[65], 22}, {NULL}};
const Segment SGI132298[] = {{&S25033[51], 6}, {NULL}};
const Segment SGI132299[] = {{&S25034[44], 9}, {NULL}};
const Segment SGI132300[] = {{&S25034[30], 6}, {NULL}};
const Segment SGI132301[] = {{&S25036[44], 10}, {NULL}};
const Segment SGI132302[] = {{&S25036[44], 28}, {NULL}};
const Segment SGI132303[] = {{&S25036[30], 6}, {NULL}};
const Segment SGI132304[] = {{&S25037[44], 29}, {NULL}};
const Segment SGI132305[] = {{&S25037[44], 41}, {NULL}};
const Segment SGI132306[] = {{&S25037[30], 6}, {NULL}};
const Segment SGI132307[] = {{&S25037[31], 3}, {NULL}};
const Segment SGI132308[] = {{&S25037[99], 6}, {NULL}};
const Segment SGI132309[] = {{&S25038[44], 10}, {NULL}};
const Segment SGI132310[] = {{&S25038[44], 28}, {NULL}};
const Segment SGI132311[] = {{&S25038[30], 6}, {NULL}};
const Segment SGI132312[] = {{&S25039[44], 11}, {NULL}};
const Segment SGI132313[] = {{&S25039[44], 23}, {NULL}};
const Segment SGI132314[] = {{&S25039[30], 6}, {NULL}};
const Segment SGI132315[] = {{&S25039[31], 3}, {NULL}};
const Segment SGI132316[] = {{&S25039[81], 6}, {NULL}};
const Segment SGI132317[] = {{&S25040[55], 18}, {NULL}};
const Segment SGI132318[] = {{&S25040[48], 6}, {NULL}};
const Segment SGI132319[] = {{&S25041[44], 28}, {NULL}};
const Segment SGI132320[] = {{&S25041[44], 40}, {NULL}};
const Segment SGI132321[] = {{&S25041[30], 6}, {NULL}};
const Segment SGI132322[] = {{&S25041[31], 3}, {NULL}};
const Segment SGI132323[] = {{&S25041[98], 6}, {NULL}};
const Segment SGI132324[] = {{&S25042[52], 11}, {NULL}};
const Segment SGI132325[] = {{&S25042[52], 23}, {NULL}};
const Segment SGI132326[] = {{&S25042[38], 6}, {NULL}};
const Segment SGI132327[] = {{&S25043[52], 10}, {NULL}};
const Segment SGI132328[] = {{&S25043[52], 22}, {NULL}};
const Segment SGI132329[] = {{&S25043[38], 6}, {NULL}};
const Segment SGI132330[] = {{&S25044[44], 11}, {NULL}};
const Segment SGI132331[] = {{&S25044[44], 23}, {NULL}};
const Segment SGI132332[] = {{&S25044[30], 6}, {NULL}};
const Segment SGI132333[] = {{&S25044[31], 3}, {NULL}};
const Segment SGI132334[] = {{&S25044[81], 5}, {NULL}};
const Segment SGI132335[] = {{&S25045[52], 11}, {NULL}};
const Segment SGI132336[] = {{&S25045[52], 23}, {NULL}};
const Segment SGI132337[] = {{&S25045[38], 6}, {NULL}};
const Segment SGI132338[] = {{&S25046[44], 10}, {NULL}};
const Segment SGI132339[] = {{&S25046[44], 28}, {NULL}};
const Segment SGI132340[] = {{&S25046[30], 6}, {NULL}};
const Segment SGI132341[] = {{&S25047[30], 1}, {NULL}};
const Segment SGI132342[] = {{&S25047[63], 10}, {NULL}};
const Segment SGI132343[] = {{&S25047[49], 6}, {NULL}};
const Segment SGI132344[] = {{&S25048[44], 10}, {NULL}};
const Segment SGI132345[] = {{&S25048[44], 19}, {NULL}};
const Segment SGI132346[] = {{&S25048[30], 6}, {NULL}};
const Segment SGI132347[] = {{&S25048[31], 3}, {NULL}};
const Segment SGI132348[] = {{&S25048[77], 5}, {NULL}};
const Segment SGI132349[] = {{&S25049[42], 10}, {NULL}};
const Segment SGI132350[] = {{&S25049[30], 6}, {NULL}};
const Segment SGI132351[] = {{&S25049[114], 3}, {NULL}};
const Segment SGI132352[] = {{&S25049[78], 7}, {NULL}};
const Segment SGI132353[] = {{&S25049[134], 8}, {NULL}};
const Segment SGI132354[] = {{&S25050[44], 29}, {NULL}};
const Segment SGI132355[] = {{&S25050[44], 41}, {NULL}};
const Segment SGI132356[] = {{&S25050[30], 6}, {NULL}};
const Segment SGI132357[] = {{&S25050[31], 3}, {NULL}};
const Segment SGI132358[] = {{&S25050[99], 6}, {NULL}};
const Segment SGI132359[] = {{&S25051[44], 11}, {NULL}};
const Segment SGI132360[] = {{&S25051[44], 30}, {NULL}};
const Segment SGI132361[] = {{&S25051[30], 6}, {NULL}};
const Segment SGI132362[] = {{&S25052[55], 19}, {NULL}};
const Segment SGI132363[] = {{&S25052[48], 6}, {NULL}};
const Segment SGI132364[] = {{&S25053[42], 29}, {NULL}};
const Segment SGI132365[] = {{&S25053[30], 6}, {NULL}};
const Segment SGI132366[] = {{&S25053[31], 3}, {NULL}};
const Segment SGI132367[] = {{&S25053[97], 6}, {NULL}};
const Segment SGI132368[] = {{&S25054[30], 1}, {NULL}};
const Segment SGI132369[] = {{&S25054[63], 10}, {NULL}};
const Segment SGI132370[] = {{&S25054[49], 6}, {NULL}};
const Segment SGI132371[] = {{&S25055[44], 10}, {NULL}};
const Segment SGI132372[] = {{&S25055[44], 22}, {NULL}};
const Segment SGI132373[] = {{&S25055[30], 6}, {NULL}};
const Segment SGI132374[] = {{&S25055[31], 3}, {NULL}};
const Segment SGI132375[] = {{&S25055[80], 5}, {NULL}};
const Segment SGI132376[] = {{&S25057[44], 10}, {NULL}};
const Segment SGI132377[] = {{&S25057[44], 22}, {NULL}};
const Segment SGI132378[] = {{&S25057[30], 6}, {NULL}};
const Segment SGI132379[] = {{&S25057[116], 3}, {NULL}};
const Segment SGI132380[] = {{&S25057[80], 7}, {NULL}};
const Segment SGI132381[] = {{&S25057[136], 8}, {NULL}};
const Segment SGI132382[] = {{&S25058[44], 13}, {NULL}};
const Segment SGI132383[] = {{&S25058[44], 38}, {NULL}};
const Segment SGI132384[] = {{&S25058[30], 6}, {NULL}};
const Segment SGI132385[] = {{&S25058[31], 3}, {NULL}};
const Segment SGI132386[] = {{&S25058[96], 6}, {NULL}};
const Segment SGI132387[] = {{&S25060[44], 10}, {NULL}};
const Segment SGI132388[] = {{&S25060[44], 22}, {NULL}};
const Segment SGI132389[] = {{&S25060[30], 6}, {NULL}};
const Segment SGI132390[] = {{&S25060[31], 3}, {NULL}};
const Segment SGI132391[] = {{&S25060[80], 6}, {NULL}};
const Segment SGI132392[] = {{&S25063[44], 11}, {NULL}};
const Segment SGI132393[] = {{&S25063[44], 23}, {NULL}};
const Segment SGI132394[] = {{&S25063[30], 6}, {NULL}};
const Segment SGI132395[] = {{&S25063[31], 3}, {NULL}};
const Segment SGI132396[] = {{&S25063[81], 6}, {NULL}};
const Segment SGI132397[] = {{&S25064[44], 11}, {NULL}};
const Segment SGI132398[] = {{&S25064[44], 23}, {NULL}};
const Segment SGI132399[] = {{&S25064[30], 6}, {NULL}};
const Segment SGI132400[] = {{&S25064[117], 3}, {NULL}};
const Segment SGI132401[] = {{&S25064[81], 7}, {NULL}};
const Segment SGI132402[] = {{&S25064[137], 8}, {NULL}};
const Segment SGI132403[] = {{&S25065[46], 10}, {NULL}};
const Segment SGI132404[] = {{&S25065[46], 28}, {NULL}};
const Segment SGI132405[] = {{&S25065[32], 6}, {NULL}};
const Segment SGI132406[] = {{&S25067[37], 6}, {NULL}};
const Segment SGI132407[] = {{&S25068[42], 11}, {NULL}};
const Segment SGI132408[] = {{&S25068[30], 6}, {NULL}};
const Segment SGI132409[] = {{&S25068[115], 3}, {NULL}};
const Segment SGI132410[] = {{&S25068[79], 7}, {NULL}};
const Segment SGI132411[] = {{&S25068[135], 8}, {NULL}};
const Segment SGI132412[] = {{&S25070[44], 28}, {NULL}};
const Segment SGI132413[] = {{&S25070[44], 40}, {NULL}};
const Segment SGI132414[] = {{&S25070[30], 6}, {NULL}};
const Segment SGI132415[] = {{&S25070[31], 3}, {NULL}};
const Segment SGI132416[] = {{&S25070[98], 6}, {NULL}};
const Segment SGI132417[] = {{&S25071[60], 10}, {NULL}};
const Segment SGI132418[] = {{&S25071[60], 28}, {NULL}};
const Segment SGI132419[] = {{&S25071[46], 6}, {NULL}};
const Segment SGI132420[] = {{&S25072[44], 28}, {NULL}};
const Segment SGI132421[] = {{&S25072[44], 40}, {NULL}};
const Segment SGI132422[] = {{&S25072[30], 6}, {NULL}};
const Segment SGI132423[] = {{&S25072[31], 3}, {NULL}};
const Segment SGI132424[] = {{&S25072[98], 6}, {NULL}};
const Segment SGI132425[] = {{&S25073[44], 10}, {NULL}};
const Segment SGI132426[] = {{&S25073[44], 25}, {NULL}};
const Segment SGI132427[] = {{&S25073[30], 6}, {NULL}};
const Segment SGI132428[] = {{&S25073[31], 3}, {NULL}};
const Segment SGI132429[] = {{&S25073[83], 6}, {NULL}};
const Segment SGI132430[] = {{&S25074[45], 11}, {NULL}};
const Segment SGI132431[] = {{&S25074[45], 30}, {NULL}};
const Segment SGI132432[] = {{&S25074[31], 6}, {NULL}};
const Segment SGI132433[] = {{&S25075[46], 10}, {NULL}};
const Segment SGI132434[] = {{&S25075[32], 6}, {NULL}};
const Segment SGI132435[] = {{&S25076[44], 21}, {NULL}};
const Segment SGI132436[] = {{&S25076[44], 33}, {NULL}};
const Segment SGI132437[] = {{&S25076[30], 6}, {NULL}};
const Segment SGI132438[] = {{&S25076[31], 3}, {NULL}};
const Segment SGI132439[] = {{&S25076[91], 6}, {NULL}};
const Segment SGI132440[] = {{&S25077[44], 10}, {NULL}};
const Segment SGI132441[] = {{&S25077[44], 19}, {NULL}};
const Segment SGI132442[] = {{&S25077[30], 6}, {NULL}};
const Segment SGI132443[] = {{&S25077[31], 3}, {NULL}};
const Segment SGI132444[] = {{&S25077[77], 6}, {NULL}};
const Segment SGI132445[] = {{&S14056[134], 3}, {NULL}};
const Segment SGI132446[] = {{&S14056[99], 6}, {NULL}};
const Segment SGI132447[] = {{&S25078[44], 10}, {NULL}};
const Segment SGI132448[] = {{&S25078[44], 22}, {NULL}};
const Segment SGI132449[] = {{&S25078[30], 6}, {NULL}};
const Segment SGI132450[] = {{&S25078[83], 4}, {NULL}};
const Segment SGI132451[] = {{&S25078[83], 3}, {NULL}};
const Segment SGI132452[] = {{&S25079[44], 10}, {NULL}};
const Segment SGI132453[] = {{&S25079[30], 6}, {NULL}};
const Segment SGI132454[] = {{&S25080[44], 10}, {NULL}};
const Segment SGI132455[] = {{&S25080[44], 22}, {NULL}};
const Segment SGI132456[] = {{&S25080[30], 6}, {NULL}};
const Segment SGI132457[] = {{&S25080[31], 3}, {NULL}};
const Segment SGI132458[] = {{&S25080[80], 6}, {NULL}};
const Segment SGI132459[] = {{&S14062[44], 11}, {NULL}};
const Segment SGI132460[] = {{&S14062[44], 23}, {NULL}};
const Segment SGI132461[] = {{&S14062[30], 6}, {NULL}};
const Segment SGI132462[] = {{&S25081[65], 11}, {NULL}};
const Segment SGI132463[] = {{&S25081[65], 23}, {NULL}};
const Segment SGI132464[] = {{&S25081[51], 6}, {NULL}};
const Segment SGI132465[] = {{&S14062[116], 3}, {NULL}};
const Segment SGI132466[] = {{&S14062[81], 6}, {NULL}};
const Segment SGI132467[] = {{&S25082[44], 10}, {NULL}};
const Segment SGI132468[] = {{&S25082[44], 22}, {NULL}};
const Segment SGI132469[] = {{&S25082[30], 6}, {NULL}};
const Segment SGI132470[] = {{&S25082[83], 6}, {NULL}};
const Segment SGI132471[] = {{&S14064[44], 84}, {NULL}};
const Segment SGI132472[] = {{&S14064[44], 105}, {NULL}};
const Segment SGI132473[] = {{&S14064[30], 6}, {NULL}};
const Segment SGI132474[] = {{&S14064[196], 3}, {NULL}};
const Segment SGI132475[] = {{&S25083[44], 10}, {NULL}};
const Segment SGI132476[] = {{&S25083[44], 22}, {NULL}};
const Segment SGI132477[] = {{&S14064[122], 5}, {NULL}};
const Segment SGI132478[] = {{&S25083[30], 6}, {NULL}};
const Segment SGI132479[] = {{&S25083[31], 3}, {NULL}};
const Segment SGI132480[] = {{&S25084[44], 18}, {NULL}};
const Segment SGI132481[] = {{&S25084[44], 30}, {NULL}};
const Segment SGI132482[] = {{&S25084[30], 6}, {NULL}};
const Segment SGI132483[] = {{&S25084[31], 3}, {NULL}};
const Segment SGI132484[] = {{&S25084[88], 6}, {NULL}};
const Segment SGI132485[] = {{&S25085[44], 22}, {NULL}};
const Segment SGI132486[] = {{&S25085[30], 6}, {NULL}};
const Segment SGI132487[] = {{&S14068[44], 8}, {NULL}};
const Segment SGI132488[] = {{&S14068[44], 20}, {NULL}};
const Segment SGI132489[] = {{&S14068[30], 6}, {NULL}};
const Segment SGI132490[] = {{&S14068[113], 3}, {NULL}};
const Segment SGI132491[] = {{&S14068[78], 6}, {NULL}};
const Segment SGI132492[] = {{&S25086[44], 10}, {NULL}};
const Segment SGI132493[] = {{&S25086[44], 22}, {&S25086[212], 7}, {NULL}};
const Segment SGI132494[] = {{&S25086[30], 6}, {NULL}};
const Segment SGI132495[] = {{&S14070[44], 6}, {NULL}};
const Segment SGI132496[] = {{&S25086[31], 3}, {NULL}};
const Segment SGI132497[] = {{&S14070[44], 18}, {NULL}};
const Segment SGI132498[] = {{&S14070[30], 6}, {NULL}};
const Segment SGI132499[] = {{&S25086[80], 6}, {NULL}};
const Segment SGI132500[] = {{&S14070[111], 3}, {NULL}};
const Segment SGI132501[] = {{&S14070[76], 6}, {NULL}};
const Segment SGI132502[] = {{&S25089[34], 11}, {NULL}};
const Segment SGI132503[] = {{&S25089[34], 23}, {NULL}};
const Segment SGI132504[] = {{&S25091[44], 32}, {NULL}};
const Segment SGI132505[] = {{&S25091[44], 44}, {NULL}};
const Segment SGI132506[] = {{&S25091[30], 6}, {NULL}};
const Segment SGI132507[] = {{&S25091[31], 3}, {NULL}};
const Segment SGI132508[] = {{&S25091[102], 6}, {NULL}};
const Segment SGI132509[] = {{&S25092[44], 11}, {NULL}};
const Segment SGI132510[] = {{&S25092[44], 23}, {NULL}};
const Segment SGI132511[] = {{&S25092[30], 6}, {NULL}};
const Segment SGI132512[] = {{&S25092[31], 3}, {NULL}};
const Segment SGI132513[] = {{&S25092[81], 5}, {NULL}};
const Segment SGI132514[] = {{&S14080[44], 15}, {NULL}};
const Segment SGI132515[] = {{&S14080[44], 27}, {NULL}};
const Segment SGI132516[] = {{&S14080[30], 6}, {NULL}};
const Segment SGI132517[] = {{&S14080[120], 3}, {NULL}};
const Segment SGI132518[] = {{&S14080[85], 6}, {NULL}};
const Segment SGI132519[] = {{&S25094[44], 11}, {NULL}};
const Segment SGI132520[] = {{&S25094[44], 23}, {NULL}};
const Segment SGI132521[] = {{&S25094[30], 6}, {NULL}};
const Segment SGI132522[] = {{&S14081[44], 6}, {NULL}};
const Segment SGI132523[] = {{&S14081[44], 16}, {NULL}};
const Segment SGI132524[] = {{&S14081[30], 6}, {NULL}};
const Segment SGI132525[] = {{&S25094[117], 3}, {NULL}};
const Segment SGI132526[] = {{&S25094[81], 7}, {NULL}};
const Segment SGI132527[] = {{&S14081[109], 3}, {NULL}};
const Segment SGI132528[] = {{&S25094[137], 8}, {NULL}};
const Segment SGI132529[] = {{&S14081[74], 6}, {NULL}};
const Segment SGI132530[] = {{&S25095[38], 17}, {NULL}};
const Segment SGI132531[] = {{&S25095[25], 6}, {NULL}};
const Segment SGI132532[] = {{&S25096[99], 29}, {NULL}};
const Segment SGI132533[] = {{&S25099[44], 10}, {NULL}};
const Segment SGI132534[] = {{&S25099[44], 22}, {NULL}};
const Segment SGI132535[] = {{&S25099[30], 6}, {NULL}};
const Segment SGI132536[] = {{&S25099[31], 3}, {NULL}};
const Segment SGI132537[] = {{&S25099[80], 6}, {NULL}};
const Segment SGI132538[] = {{&S25100[38], 10}, {NULL}};
const Segment SGI132539[] = {{&S25100[31], 6}, {NULL}};
const Segment SGI132540[] = {{&S25102[42], 10}, {NULL}};
const Segment SGI132541[] = {{&S25102[30], 6}, {NULL}};
const Segment SGI132542[] = {{&S25102[31], 3}, {NULL}};
const Segment SGI132543[] = {{&S25102[78], 6}, {NULL}};
const Segment SGI132544[] = {{&S25106[44], 10}, {NULL}};
const Segment SGI132545[] = {{&S25106[44], 22}, {NULL}};
const Segment SGI132546[] = {{&S25106[30], 6}, {NULL}};
const Segment SGI132547[] = {{&S25106[80], 6}, {NULL}};
const Segment SGI132548[] = {{&S25113[44], 10}, {&S25113[175], 18}, {NULL}};
const Segment SGI132549[] = {{&S25113[44], 22}, {NULL}};
const Segment SGI132550[] = {{&S25113[30], 6}, {NULL}};
const Segment SGI132551[] = {{&S25113[31], 3}, {NULL}};
const Segment SGI132552[] = {{&S25113[80], 6}, {NULL}};
const Segment SGI132553[] = {{&S14110[44], 5}, {NULL}};
const Segment SGI132554[] = {{&S14110[44], 17}, {NULL}};
const Segment SGI132555[] = {{&S14110[30], 6}, {NULL}};
const Segment SGI132556[] = {{&S14110[110], 3}, {NULL}};
const Segment SGI132557[] = {{&S14110[75], 6}, {NULL}};
const Segment SGI132558[] = {{&S14111[44], 8}, {NULL}};
const Segment SGI132559[] = {{&S14111[44], 26}, {NULL}};
const Segment SGI132560[] = {{&S14111[30], 6}, {NULL}};
const Segment SGI132561[] = {{&S14111[119], 3}, {NULL}};
const Segment SGI132562[] = {{&S14111[84], 6}, {NULL}};
const Segment SGI132563[] = {{&S25117[58], 10}, {NULL}};
const Segment SGI132564[] = {{&S25117[58], 34}, {NULL}};
const Segment SGI132565[] = {{&S25117[44], 6}, {NULL}};
const Segment SGI132566[] = {{&S25119[44], 10}, {NULL}};
const Segment SGI132567[] = {{&S25119[44], 22}, {NULL}};
const Segment SGI132568[] = {{&S25119[30], 6}, {NULL}};
const Segment SGI132569[] = {{&S25119[124], 3}, {NULL}};
const Segment SGI132570[] = {{&S25119[89], 6}, {NULL}};
const Segment SGI132571[] = {{&S25119[144], 8}, {NULL}};
const Segment SGI132572[] = {{&S25120[44], 10}, {NULL}};
const Segment SGI132573[] = {{&S25120[44], 88}, {NULL}};
const Segment SGI132574[] = {{&S25120[30], 6}, {NULL}};
const Segment SGI132575[] = {{&S25121[44], 10}, {NULL}};
const Segment SGI132576[] = {{&S25121[44], 22}, {NULL}};
const Segment SGI132577[] = {{&S25121[30], 6}, {NULL}};
const Segment SGI132578[] = {{&S25121[31], 3}, {NULL}};
const Segment SGI132579[] = {{&S25121[80], 6}, {NULL}};
const Segment SGI132580[] = {{&S14122[44], 20}, {NULL}};
const Segment SGI132581[] = {{&S14122[44], 32}, {NULL}};
const Segment SGI132582[] = {{&S14122[30], 6}, {NULL}};
const Segment SGI132583[] = {{&S14122[125], 3}, {NULL}};
const Segment SGI132584[] = {{&S14122[90], 6}, {NULL}};
const Segment SGI132585[] = {{&S25122[44], 10}, {NULL}};
const Segment SGI132586[] = {{&S25122[44], 22}, {NULL}};
const Segment SGI132587[] = {{&S25122[30], 6}, {NULL}};
const Segment SGI132588[] = {{&S25122[83], 6}, {NULL}};
const Segment SGI132589[] = {{&S25123[44], 10}, {NULL}};
const Segment SGI132590[] = {{&S25123[44], 22}, {NULL}};
const Segment SGI132591[] = {{&S25123[30], 6}, {NULL}};
const Segment SGI132592[] = {{&S25123[83], 4}, {NULL}};
const Segment SGI132593[] = {{&S25123[83], 3}, {NULL}};
const Segment SGI132594[] = {{&S25124[44], 10}, {NULL}};
const Segment SGI132595[] = {{&S25124[44], 22}, {NULL}};
const Segment SGI132596[] = {{&S25124[30], 6}, {NULL}};
const Segment SGI132597[] = {{&S25124[31], 3}, {NULL}};
const Segment SGI132598[] = {{&S25124[80], 6}, {NULL}};
const Segment SGI132599[] = {{&S14131[44], 10}, {NULL}};
const Segment SGI132600[] = {{&S14131[44], 28}, {NULL}};
const Segment SGI132601[] = {{&S14131[30], 6}, {NULL}};
const Segment SGI132602[] = {{&S14131[121], 3}, {NULL}};
const Segment SGI132603[] = {{&S14131[86], 6}, {NULL}};
const Segment SGI132604[] = {{&S25128[44], 10}, {NULL}};
const Segment SGI132605[] = {{&S25128[44], 22}, {NULL}};
const Segment SGI132606[] = {{&S25128[30], 6}, {NULL}};
const Segment SGI132607[] = {{&S15610[44], 11}, {NULL}};
const Segment SGI132608[] = {{&S15610[44], 23}, {NULL}};
const Segment SGI132609[] = {{&S15610[30], 6}, {NULL}};
const Segment SGI132610[] = {{&S25128[80], 6}, {NULL}};
const Segment SGI132611[] = {{&S15610[116], 3}, {NULL}};
const Segment SGI132612[] = {{&S15610[81], 6}, {NULL}};
const Segment SGI132613[] = {{&S14134[44], 6}, {NULL}};
const Segment SGI132614[] = {{&S14134[44], 18}, {NULL}};
const Segment SGI132615[] = {{&S14134[30], 6}, {NULL}};
const Segment SGI132616[] = {{&S25129[65], 10}, {NULL}};
const Segment SGI132617[] = {{&S25129[65], 22}, {NULL}};
const Segment SGI132618[] = {{&S25129[51], 6}, {NULL}};
const Segment SGI132619[] = {{&S14134[111], 3}, {NULL}};
const Segment SGI132620[] = {{&S14134[76], 6}, {NULL}};
const Segment SGI132621[] = {{&S15611[44], 11}, {NULL}};
const Segment SGI132622[] = {{&S15611[44], 23}, {NULL}};
const Segment SGI132623[] = {{&S15611[30], 6}, {NULL}};
const Segment SGI132624[] = {{&S15611[116], 3}, {NULL}};
const Segment SGI132625[] = {{&S15611[81], 6}, {NULL}};
const Segment SGI132626[] = {{&S14135[44], 8}, {NULL}};
const Segment SGI132627[] = {{&S14135[44], 20}, {NULL}};
const Segment SGI132628[] = {{&S14135[30], 6}, {NULL}};
const Segment SGI132629[] = {{&S14135[113], 3}, {NULL}};
const Segment SGI132630[] = {{&S14135[78], 6}, {NULL}};
const Segment SGI132631[] = {{&S15612[44], 11}, {NULL}};
const Segment SGI132632[] = {{&S15612[44], 23}, {NULL}};
const Segment SGI132633[] = {{&S15612[30], 6}, {NULL}};
const Segment SGI132634[] = {{&S15612[116], 3}, {NULL}};
const Segment SGI132635[] = {{&S25131[44], 10}, {NULL}};
const Segment SGI132636[] = {{&S25131[44], 22}, {NULL}};
const Segment SGI132637[] = {{&S15612[81], 6}, {NULL}};
const Segment SGI132638[] = {{&S25131[30], 6}, {NULL}};
const Segment SGI132639[] = {{&S25131[8], 3}, {&S25131[31], 3}, {NULL}};
const Segment SGI132640[] = {{&S25131[80], 6}, {NULL}};
const Segment SGI132641[] = {{&S15613[44], 11}, {NULL}};
const Segment SGI132642[] = {{&S15613[44], 23}, {NULL}};
const Segment SGI132643[] = {{&S15613[30], 6}, {NULL}};
const Segment SGI132644[] = {{&S15613[116], 3}, {NULL}};
const Segment SGI132645[] = {{&S15613[81], 6}, {NULL}};
const Segment SGI132646[] = {{&S25133[44], 10}, {NULL}};
const Segment SGI132647[] = {{&S25133[44], 22}, {NULL}};
const Segment SGI132648[] = {{&S15614[44], 7}, {NULL}};
const Segment SGI132649[] = {{&S25133[30], 6}, {NULL}};
const Segment SGI132650[] = {{&S15614[44], 19}, {NULL}};
const Segment SGI132651[] = {{&S15614[30], 6}, {NULL}};
const Segment SGI132652[] = {{&S25133[31], 3}, {NULL}};
const Segment SGI132653[] = {{&S15614[112], 3}, {NULL}};
const Segment SGI132654[] = {{&S25133[80], 5}, {NULL}};
const Segment SGI132655[] = {{&S15614[77], 6}, {NULL}};
const Segment SGI132656[] = {{&S25134[44], 10}, {NULL}};
const Segment SGI132657[] = {{&S25134[44], 22}, {NULL}};
const Segment SGI132658[] = {{&S25134[30], 6}, {NULL}};
const Segment SGI132659[] = {{&S25134[31], 3}, {NULL}};
const Segment SGI132660[] = {{&S15616[44], 7}, {NULL}};
const Segment SGI132661[] = {{&S15616[44], 19}, {NULL}};
const Segment SGI132662[] = {{&S15616[30], 6}, {NULL}};
const Segment SGI132663[] = {{&S15616[112], 3}, {NULL}};
const Segment SGI132664[] = {{&S15616[77], 6}, {NULL}};
const Segment SGI132665[] = {{&S25135[44], 10}, {NULL}};
const Segment SGI132666[] = {{&S14144[44], 8}, {NULL}};
const Segment SGI132667[] = {{&S25135[44], 22}, {NULL}};
const Segment SGI132668[] = {{&S25135[30], 6}, {NULL}};
const Segment SGI132669[] = {{&S14144[44], 27}, {NULL}};
const Segment SGI132670[] = {{&S14144[30], 6}, {NULL}};
const Segment SGI132671[] = {{&S25135[116], 3}, {NULL}};
const Segment SGI132672[] = {{&S14144[120], 3}, {NULL}};
const Segment SGI132673[] = {{&S25135[80], 7}, {NULL}};
const Segment SGI132674[] = {{&S14144[85], 6}, {NULL}};
const Segment SGI132675[] = {{&S25135[136], 8}, {NULL}};
const Segment SGI132676[] = {{&S15617[38], 11}, {NULL}};
const Segment SGI132677[] = {{&S15617[31], 6}, {NULL}};
const Segment SGI132678[] = {{&S14146[44], 12}, {NULL}};
const Segment SGI132679[] = {{&S14146[44], 24}, {NULL}};
const Segment SGI132680[] = {{&S25137[44], 10}, {NULL}};
const Segment SGI132681[] = {{&S14146[30], 6}, {NULL}};
const Segment SGI132682[] = {{&S25137[44], 22}, {NULL}};
const Segment SGI132683[] = {{&S25137[30], 6}, {NULL}};
const Segment SGI132684[] = {{&S14146[117], 3}, {NULL}};
const Segment SGI132685[] = {{&S14146[82], 6}, {NULL}};
const Segment SGI132686[] = {{&S25137[31], 3}, {NULL}};
const Segment SGI132687[] = {{&S15620[44], 7}, {NULL}};
const Segment SGI132688[] = {{&S15620[44], 19}, {NULL}};
const Segment SGI132689[] = {{&S15620[30], 6}, {NULL}};
const Segment SGI132690[] = {{&S15620[112], 3}, {NULL}};
const Segment SGI132691[] = {{&S25138[44], 10}, {NULL}};
const Segment SGI132692[] = {{&S15620[77], 6}, {NULL}};
const Segment SGI132693[] = {{&S25138[44], 22}, {NULL}};
const Segment SGI132694[] = {{&S25138[30], 6}, {NULL}};
const Segment SGI132695[] = {{&S25138[31], 3}, {NULL}};
const Segment SGI132696[] = {{&S25138[80], 6}, {NULL}};
const Segment SGI132697[] = {{&S15623[44], 11}, {NULL}};
const Segment SGI132698[] = {{&S15623[44], 23}, {NULL}};
const Segment SGI132699[] = {{&S15623[30], 6}, {NULL}};
const Segment SGI132700[] = {{&S15623[116], 3}, {NULL}};
const Segment SGI132701[] = {{&S14151[44], 31}, {NULL}};
const Segment SGI132702[] = {{&S15623[81], 6}, {NULL}};
const Segment SGI132703[] = {{&S14151[30], 6}, {NULL}};
const Segment SGI132704[] = {{&S15624[44], 11}, {NULL}};
const Segment SGI132705[] = {{&S15624[44], 23}, {NULL}};
const Segment SGI132706[] = {{&S15624[30], 6}, {NULL}};
const Segment SGI132707[] = {{&S15624[84], 6}, {NULL}};
const Segment SGI132708[] = {{&S15625[44], 12}, {NULL}};
const Segment SGI132709[] = {{&S15625[44], 24}, {NULL}};
const Segment SGI132710[] = {{&S15625[30], 6}, {NULL}};
const Segment SGI132711[] = {{&S15625[117], 3}, {NULL}};
const Segment SGI132712[] = {{&S15625[82], 6}, {NULL}};
const Segment SGI132713[] = {{&S15626[44], 21}, {NULL}};
const Segment SGI132714[] = {{&S15626[44], 33}, {NULL}};
const Segment SGI132715[] = {{&S15626[30], 6}, {NULL}};
const Segment SGI132716[] = {{&S15626[126], 3}, {NULL}};
const Segment SGI132717[] = {{&S15626[91], 6}, {NULL}};
const Segment SGI132718[] = {{&S15627[44], 23}, {NULL}};
const Segment SGI132719[] = {{&S15627[44], 35}, {NULL}};
const Segment SGI132720[] = {{&S15627[30], 6}, {NULL}};
const Segment SGI132721[] = {{&S15627[128], 3}, {NULL}};
const Segment SGI132722[] = {{&S15627[93], 6}, {NULL}};
const Segment SGI132723[] = {{&S14160[44], 9}, {NULL}};
const Segment SGI132724[] = {{&S14160[44], 21}, {NULL}};
const Segment SGI132725[] = {{&S14160[30], 6}, {NULL}};
const Segment SGI132726[] = {{&S14160[114], 3}, {NULL}};
const Segment SGI132727[] = {{&S14160[79], 6}, {NULL}};
const Segment SGI132728[] = {{&S15628[38], 11}, {NULL}};
const Segment SGI132729[] = {{&S15628[31], 6}, {NULL}};
const Segment SGI132730[] = {{&S14161[44], 10}, {NULL}};
const Segment SGI132731[] = {{&S14161[44], 22}, {NULL}};
const Segment SGI132732[] = {{&S14161[30], 6}, {NULL}};
const Segment SGI132733[] = {{&S15629[44], 23}, {NULL}};
const Segment SGI132734[] = {{&S15629[44], 35}, {NULL}};
const Segment SGI132735[] = {{&S14161[115], 3}, {NULL}};
const Segment SGI132736[] = {{&S15629[30], 6}, {NULL}};
const Segment SGI132737[] = {{&S14161[80], 6}, {NULL}};
const Segment SGI132738[] = {{&S15629[128], 3}, {NULL}};
const Segment SGI132739[] = {{&S15629[93], 6}, {NULL}};
const Segment SGI132740[] = {{&S14162[44], 6}, {NULL}};
const Segment SGI132741[] = {{&S14162[44], 18}, {NULL}};
const Segment SGI132742[] = {{&S14162[30], 6}, {NULL}};
const Segment SGI132743[] = {{&S14162[111], 3}, {NULL}};
const Segment SGI132744[] = {{&S14162[76], 6}, {NULL}};
const Segment SGI132745[] = {{&S15630[44], 23}, {NULL}};
const Segment SGI132746[] = {{&S15630[44], 35}, {NULL}};
const Segment SGI132747[] = {{&S15630[30], 6}, {NULL}};
const Segment SGI132748[] = {{&S15630[128], 3}, {NULL}};
const Segment SGI132749[] = {{&S15630[93], 6}, {NULL}};
const Segment SGI132750[] = {{&S14164[44], 10}, {NULL}};
const Segment SGI132751[] = {{&S14164[44], 22}, {NULL}};
const Segment SGI132752[] = {{&S14164[30], 6}, {NULL}};
const Segment SGI132753[] = {{&S15631[44], 22}, {NULL}};
const Segment SGI132754[] = {{&S15631[44], 34}, {NULL}};
const Segment SGI132755[] = {{&S15631[30], 6}, {NULL}};
const Segment SGI132756[] = {{&S14164[115], 3}, {NULL}};
const Segment SGI132757[] = {{&S14164[80], 6}, {NULL}};
const Segment SGI132758[] = {{&S15631[127], 3}, {NULL}};
const Segment SGI132759[] = {{&S15631[92], 6}, {NULL}};
const Segment SGI132760[] = {{&S15632[44], 23}, {NULL}};
const Segment SGI132761[] = {{&S15632[44], 35}, {NULL}};
const Segment SGI132762[] = {{&S15632[30], 6}, {NULL}};
const Segment SGI132763[] = {{&S15632[128], 3}, {NULL}};
const Segment SGI132764[] = {{&S15632[93], 6}, {NULL}};
const Segment SGI132765[] = {{&S15633[44], 11}, {NULL}};
const Segment SGI132766[] = {{&S15633[44], 23}, {NULL}};
const Segment SGI132767[] = {{&S15633[30], 6}, {NULL}};
const Segment SGI132768[] = {{&S15633[116], 3}, {NULL}};
const Segment SGI132769[] = {{&S15633[81], 6}, {NULL}};
const Segment SGI132770[] = {{&S14169[44], 9}, {NULL}};
const Segment SGI132771[] = {{&S14169[44], 21}, {NULL}};
const Segment SGI132772[] = {{&S14169[30], 6}, {NULL}};
const Segment SGI132773[] = {{&S14169[114], 3}, {NULL}};
const Segment SGI132774[] = {{&S14169[79], 6}, {NULL}};
const Segment SGI132775[] = {{&S15634[44], 22}, {NULL}};
const Segment SGI132776[] = {{&S15634[44], 34}, {NULL}};
const Segment SGI132777[] = {{&S15634[30], 6}, {NULL}};
const Segment SGI132778[] = {{&S15634[127], 3}, {NULL}};
const Segment SGI132779[] = {{&S15634[92], 6}, {NULL}};
const Segment SGI132780[] = {{&S14171[44], 7}, {NULL}};
const Segment SGI132781[] = {{&S14171[44], 19}, {NULL}};
const Segment SGI132782[] = {{&S14171[30], 6}, {NULL}};
const Segment SGI132783[] = {{&S14171[112], 3}, {NULL}};
const Segment SGI132784[] = {{&S14171[77], 6}, {NULL}};
const Segment SGI132785[] = {{&S15636[44], 23}, {NULL}};
const Segment SGI132786[] = {{&S15636[44], 35}, {NULL}};
const Segment SGI132787[] = {{&S15636[30], 6}, {NULL}};
const Segment SGI132788[] = {{&S15636[128], 3}, {NULL}};
const Segment SGI132789[] = {{&S15636[93], 6}, {NULL}};
const Segment SGI132790[] = {{&S14173[44], 6}, {NULL}};
const Segment SGI132791[] = {{&S14173[44], 18}, {NULL}};
const Segment SGI132792[] = {{&S14173[30], 6}, {NULL}};
const Segment SGI132793[] = {{&S14173[111], 3}, {NULL}};
const Segment SGI132794[] = {{&S15637[44], 23}, {NULL}};
const Segment SGI132795[] = {{&S14173[76], 6}, {NULL}};
const Segment SGI132796[] = {{&S15637[44], 35}, {NULL}};
const Segment SGI132797[] = {{&S15637[30], 6}, {NULL}};
const Segment SGI132798[] = {{&S15637[128], 3}, {NULL}};
const Segment SGI132799[] = {{&S15637[93], 6}, {NULL}};
const Segment SGI132800[] = {{&S15638[44], 22}, {NULL}};
const Segment SGI132801[] = {{&S14175[44], 9}, {NULL}};
const Segment SGI132802[] = {{&S15638[44], 34}, {NULL}};
const Segment SGI132803[] = {{&S15638[30], 6}, {NULL}};
const Segment SGI132804[] = {{&S14175[44], 21}, {NULL}};
const Segment SGI132805[] = {{&S14175[30], 6}, {NULL}};
const Segment SGI132806[] = {{&S15638[127], 3}, {NULL}};
const Segment SGI132807[] = {{&S14175[114], 3}, {NULL}};
const Segment SGI132808[] = {{&S15638[92], 6}, {NULL}};
const Segment SGI132809[] = {{&S14175[79], 6}, {NULL}};
const Segment SGI132810[] = {{&S15639[44], 23}, {NULL}};
const Segment SGI132811[] = {{&S15639[44], 35}, {NULL}};
const Segment SGI132812[] = {{&S15639[30], 6}, {NULL}};
const Segment SGI132813[] = {{&S15639[128], 3}, {NULL}};
const Segment SGI132814[] = {{&S14177[44], 10}, {NULL}};
const Segment SGI132815[] = {{&S14177[44], 22}, {NULL}};
const Segment SGI132816[] = {{&S15639[93], 6}, {NULL}};
const Segment SGI132817[] = {{&S14177[30], 6}, {NULL}};
const Segment SGI132818[] = {{&S14177[115], 3}, {NULL}};
const Segment SGI132819[] = {{&S14177[80], 6}, {NULL}};
const Segment SGI132820[] = {{&S15640[44], 11}, {NULL}};
const Segment SGI132821[] = {{&S15640[44], 23}, {NULL}};
const Segment SGI132822[] = {{&S15640[30], 6}, {NULL}};
const Segment SGI132823[] = {{&S14178[44], 8}, {NULL}};
const Segment SGI132824[] = {{&S14178[44], 20}, {NULL}};
const Segment SGI132825[] = {{&S14178[30], 6}, {NULL}};
const Segment SGI132826[] = {{&S15640[84], 6}, {NULL}};
const Segment SGI132827[] = {{&S14178[113], 3}, {NULL}};
const Segment SGI132828[] = {{&S14178[78], 6}, {NULL}};
const Segment SGI132829[] = {{&S15641[44], 22}, {NULL}};
const Segment SGI132830[] = {{&S15641[44], 34}, {NULL}};
const Segment SGI132831[] = {{&S15641[30], 6}, {NULL}};
const Segment SGI132832[] = {{&S14179[44], 43}, {NULL}};
const Segment SGI132833[] = {{&S15641[127], 3}, {NULL}};
const Segment SGI132834[] = {{&S14179[30], 6}, {NULL}};
const Segment SGI132835[] = {{&S15641[92], 6}, {NULL}};
const Segment SGI132836[] = {{&S14179[119], 6}, {NULL}};
const Segment SGI132837[] = {{&S15642[44], 23}, {NULL}};
const Segment SGI132838[] = {{&S15642[44], 35}, {NULL}};
const Segment SGI132839[] = {{&S15642[30], 6}, {NULL}};
const Segment SGI132840[] = {{&S15642[128], 3}, {NULL}};
const Segment SGI132841[] = {{&S15642[93], 6}, {NULL}};
const Segment SGI132842[] = {{&S15643[44], 20}, {NULL}};
const Segment SGI132843[] = {{&S15643[44], 32}, {NULL}};
const Segment SGI132844[] = {{&S15643[30], 6}, {NULL}};
const Segment SGI132845[] = {{&S15643[125], 3}, {NULL}};
const Segment SGI132846[] = {{&S15643[90], 6}, {NULL}};
const Segment SGI132847[] = {{&S14184[44], 6}, {NULL}};
const Segment SGI132848[] = {{&S14184[44], 18}, {NULL}};
const Segment SGI132849[] = {{&S14184[30], 6}, {NULL}};
const Segment SGI132850[] = {{&S14184[111], 3}, {NULL}};
const Segment SGI132851[] = {{&S14184[76], 6}, {NULL}};
const Segment SGI132852[] = {{&S15645[44], 22}, {NULL}};
const Segment SGI132853[] = {{&S15645[44], 34}, {NULL}};
const Segment SGI132854[] = {{&S15645[30], 6}, {NULL}};
const Segment SGI132855[] = {{&S14185[44], 10}, {NULL}};
const Segment SGI132856[] = {{&S14185[44], 22}, {NULL}};
const Segment SGI132857[] = {{&S15645[127], 3}, {NULL}};
const Segment SGI132858[] = {{&S14185[30], 6}, {NULL}};
const Segment SGI132859[] = {{&S15645[92], 6}, {NULL}};
const Segment SGI132860[] = {{&S14185[115], 3}, {NULL}};
const Segment SGI132861[] = {{&S14185[80], 6}, {NULL}};
const Segment SGI132862[] = {{&S14186[44], 84}, {NULL}};
const Segment SGI132863[] = {{&S14186[44], 105}, {NULL}};
const Segment SGI132864[] = {{&S14186[30], 6}, {NULL}};
const Segment SGI132865[] = {{&S14186[196], 3}, {NULL}};
const Segment SGI132866[] = {{&S14186[122], 5}, {NULL}};
const Segment SGI132867[] = {{&S15649[42], 11}, {NULL}};
const Segment SGI132868[] = {{&S15649[30], 6}, {NULL}};
const Segment SGI132869[] = {{&S15649[114], 3}, {NULL}};
const Segment SGI132870[] = {{&S15649[79], 6}, {NULL}};
const Segment SGI132871[] = {{&S14189[44], 14}, {NULL}};
const Segment SGI132872[] = {{&S14189[44], 26}, {NULL}};
const Segment SGI132873[] = {{&S14189[30], 6}, {NULL}};
const Segment SGI132874[] = {{&S15650[42], 11}, {NULL}};
const Segment SGI132875[] = {{&S15650[30], 6}, {NULL}};
const Segment SGI132876[] = {{&S14189[119], 3}, {NULL}};
const Segment SGI132877[] = {{&S14189[84], 6}, {NULL}};
const Segment SGI132878[] = {{&S15650[114], 3}, {NULL}};
const Segment SGI132879[] = {{&S15650[79], 6}, {NULL}};
const Segment SGI132880[] = {{&S15651[44], 20}, {NULL}};
const Segment SGI132881[] = {{&S15651[44], 32}, {NULL}};
const Segment SGI132882[] = {{&S15651[30], 6}, {NULL}};
const Segment SGI132883[] = {{&S15651[125], 3}, {NULL}};
const Segment SGI132884[] = {{&S15651[90], 6}, {NULL}};
const Segment SGI132885[] = {{&S15653[44], 11}, {NULL}};
const Segment SGI132886[] = {{&S15653[44], 23}, {NULL}};
const Segment SGI132887[] = {{&S15653[30], 6}, {NULL}};
const Segment SGI132888[] = {{&S15653[116], 3}, {NULL}};
const Segment SGI132889[] = {{&S15653[81], 6}, {NULL}};
const Segment SGI132890[] = {{&S14195[44], 12}, {NULL}};
const Segment SGI132891[] = {{&S14195[44], 24}, {NULL}};
const Segment SGI132892[] = {{&S14195[30], 6}, {NULL}};
const Segment SGI132893[] = {{&S14195[117], 3}, {NULL}};
const Segment SGI132894[] = {{&S14195[82], 6}, {NULL}};
const Segment SGI132895[] = {{&S15654[44], 11}, {NULL}};
const Segment SGI132896[] = {{&S15654[44], 23}, {NULL}};
const Segment SGI132897[] = {{&S15654[30], 6}, {NULL}};
const Segment SGI132898[] = {{&S15654[116], 3}, {NULL}};
const Segment SGI132899[] = {{&S15654[81], 6}, {NULL}};
const Segment SGI132900[] = {{&S14196[44], 12}, {NULL}};
const Segment SGI132901[] = {{&S14196[44], 24}, {NULL}};
const Segment SGI132902[] = {{&S14196[30], 6}, {NULL}};
const Segment SGI132903[] = {{&S14196[117], 3}, {NULL}};
const Segment SGI132904[] = {{&S14196[82], 6}, {NULL}};
const Segment SGI132905[] = {{&S15655[44], 23}, {NULL}};
const Segment SGI132906[] = {{&S15655[44], 35}, {NULL}};
const Segment SGI132907[] = {{&S15655[30], 6}, {NULL}};
const Segment SGI132908[] = {{&S15655[128], 3}, {NULL}};
const Segment SGI132909[] = {{&S15655[93], 6}, {NULL}};
const Segment SGI132910[] = {{&S14198[42], 17}, {NULL}};
const Segment SGI132911[] = {{&S14198[30], 6}, {NULL}};
const Segment SGI132912[] = {{&S14198[120], 3}, {NULL}};
const Segment SGI132913[] = {{&S14198[85], 6}, {NULL}};
const Segment SGI132914[] = {{&S15657[44], 22}, {NULL}};
const Segment SGI132915[] = {{&S15657[44], 34}, {NULL}};
const Segment SGI132916[] = {{&S15657[30], 6}, {NULL}};
const Segment SGI132917[] = {{&S15657[127], 3}, {NULL}};
const Segment SGI132918[] = {{&S15657[92], 6}, {NULL}};
const Segment SGI132919[] = {{&S14200[44], 37}, {NULL}};
const Segment SGI132920[] = {{&S14200[30], 6}, {NULL}};
const Segment SGI132921[] = {{&S14200[113], 6}, {NULL}};
const Segment SGI132922[] = {{&S15660[38], 11}, {NULL}};
const Segment SGI132923[] = {{&S15660[31], 6}, {NULL}};
const Segment SGI132924[] = {{&S15661[44], 9}, {NULL}};
const Segment SGI132925[] = {{&S14203[44], 15}, {NULL}};
const Segment SGI132926[] = {{&S15661[44], 21}, {NULL}};
const Segment SGI132927[] = {{&S15661[30], 6}, {NULL}};
const Segment SGI132928[] = {{&S14203[44], 27}, {NULL}};
const Segment SGI132929[] = {{&S14203[30], 6}, {NULL}};
const Segment SGI132930[] = {{&S15661[113], 3}, {NULL}};
const Segment SGI132931[] = {{&S14203[120], 3}, {NULL}};
const Segment SGI132932[] = {{&S15661[79], 6}, {NULL}};
const Segment SGI132933[] = {{&S14203[85], 6}, {NULL}};
const Segment SGI132934[] = {{&S15662[44], 7}, {NULL}};
const Segment SGI132935[] = {{&S15662[44], 19}, {NULL}};
const Segment SGI132936[] = {{&S14204[44], 5}, {NULL}};
const Segment SGI132937[] = {{&S15662[30], 6}, {NULL}};
const Segment SGI132938[] = {{&S14204[44], 17}, {NULL}};
const Segment SGI132939[] = {{&S14204[30], 6}, {NULL}};
const Segment SGI132940[] = {{&S15662[112], 3}, {NULL}};
const Segment SGI132941[] = {{&S14204[110], 3}, {NULL}};
const Segment SGI132942[] = {{&S15662[77], 6}, {NULL}};
const Segment SGI132943[] = {{&S14204[75], 6}, {NULL}};
const Segment SGI132944[] = {{&S14205[44], 9}, {NULL}};
const Segment SGI132945[] = {{&S14205[44], 21}, {NULL}};
const Segment SGI132946[] = {{&S14205[30], 6}, {NULL}};
const Segment SGI132947[] = {{&S14205[114], 3}, {NULL}};
const Segment SGI132948[] = {{&S14205[79], 6}, {NULL}};
const Segment SGI132949[] = {{&S15665[44], 23}, {NULL}};
const Segment SGI132950[] = {{&S15665[44], 35}, {NULL}};
const Segment SGI132951[] = {{&S15665[30], 6}, {NULL}};
const Segment SGI132952[] = {{&S15665[128], 3}, {NULL}};
const Segment SGI132953[] = {{&S15665[93], 6}, {NULL}};
const Segment SGI132954[] = {{&S14206[44], 10}, {NULL}};
const Segment SGI132955[] = {{&S14206[44], 56}, {NULL}};
const Segment SGI132956[] = {{&S14206[30], 6}, {NULL}};
const Segment SGI132957[] = {{&S14206[149], 3}, {NULL}};
const Segment SGI132958[] = {{&S14206[114], 6}, {NULL}};
const Segment SGI132959[] = {{&S14208[44], 15}, {NULL}};
const Segment SGI132960[] = {{&S14208[44], 27}, {NULL}};
const Segment SGI132961[] = {{&S14208[30], 6}, {NULL}};
const Segment SGI132962[] = {{&S14208[120], 3}, {NULL}};
const Segment SGI132963[] = {{&S14208[85], 6}, {NULL}};
const Segment SGI132964[] = {{&S15669[44], 11}, {NULL}};
const Segment SGI132965[] = {{&S15669[44], 29}, {NULL}};
const Segment SGI132966[] = {{&S15669[30], 6}, {NULL}};
const Segment SGI132967[] = {{&S15669[122], 3}, {NULL}};
const Segment SGI132968[] = {{&S15669[87], 6}, {NULL}};
const Segment SGI132969[] = {{&S14211[44], 11}, {NULL}};
const Segment SGI132970[] = {{&S14211[44], 23}, {NULL}};
const Segment SGI132971[] = {{&S14211[30], 6}, {NULL}};
const Segment SGI132972[] = {{&S14211[116], 3}, {NULL}};
const Segment SGI132973[] = {{&S14211[81], 6}, {NULL}};
const Segment SGI132974[] = {{&S14214[44], 7}, {NULL}};
const Segment SGI132975[] = {{&S14214[44], 25}, {NULL}};
const Segment SGI132976[] = {{&S14214[30], 6}, {NULL}};
const Segment SGI132977[] = {{&S14214[118], 3}, {NULL}};
const Segment SGI132978[] = {{&S14214[83], 6}, {NULL}};
const Segment SGI132979[] = {{&S14222[44], 11}, {NULL}};
const Segment SGI132980[] = {{&S14222[44], 23}, {NULL}};
const Segment SGI132981[] = {{&S14222[30], 6}, {NULL}};
const Segment SGI132982[] = {{&S14222[116], 3}, {NULL}};
const Segment SGI132983[] = {{&S14222[81], 6}, {NULL}};
const Segment SGI132984[] = {{&S14228[44], 15}, {NULL}};
const Segment SGI132985[] = {{&S14228[44], 27}, {NULL}};
const Segment SGI132986[] = {{&S14228[30], 6}, {NULL}};
const Segment SGI132987[] = {{&S14228[120], 3}, {NULL}};
const Segment SGI132988[] = {{&S14228[85], 6}, {NULL}};
const Segment SGI132989[] = {{&S14229[44], 24}, {NULL}};
const Segment SGI132990[] = {{&S14229[44], 36}, {NULL}};
const Segment SGI132991[] = {{&S14229[30], 6}, {NULL}};
const Segment SGI132992[] = {{&S14229[129], 3}, {NULL}};
const Segment SGI132993[] = {{&S14229[94], 6}, {NULL}};
const Segment SGI132994[] = {{&S14233[44], 6}, {NULL}};
const Segment SGI132995[] = {{&S14233[44], 18}, {NULL}};
const Segment SGI132996[] = {{&S14233[30], 6}, {NULL}};
const Segment SGI132997[] = {{&S14233[111], 3}, {NULL}};
const Segment SGI132998[] = {{&S14233[76], 6}, {NULL}};
const Segment SGI132999[] = {{&S14236[44], 10}, {NULL}};
const Segment SGI133000[] = {{&S14236[44], 22}, {NULL}};
const Segment SGI133001[] = {{&S14236[30], 6}, {NULL}};
const Segment SGI133002[] = {{&S14236[115], 3}, {NULL}};
const Segment SGI133003[] = {{&S14236[80], 6}, {NULL}};
const Segment SGI133004[] = {{&S14237[44], 12}, {NULL}};
const Segment SGI133005[] = {{&S14237[44], 24}, {NULL}};
const Segment SGI133006[] = {{&S14237[30], 6}, {NULL}};
const Segment SGI133007[] = {{&S14237[117], 3}, {NULL}};
const Segment SGI133008[] = {{&S14237[82], 6}, {NULL}};
const Segment SGI133009[] = {{&S14239[44], 6}, {NULL}};
const Segment SGI133010[] = {{&S14239[44], 18}, {NULL}};
const Segment SGI133011[] = {{&S14239[30], 6}, {NULL}};
const Segment SGI133012[] = {{&S14239[111], 3}, {NULL}};
const Segment SGI133013[] = {{&S14239[76], 6}, {NULL}};
const Segment SGI133014[] = {{&S14240[44], 9}, {NULL}};
const Segment SGI133015[] = {{&S14240[44], 21}, {NULL}};
const Segment SGI133016[] = {{&S14240[30], 6}, {NULL}};
const Segment SGI133017[] = {{&S14240[114], 3}, {NULL}};
const Segment SGI133018[] = {{&S14240[79], 6}, {NULL}};
const Segment SGI133019[] = {{&S14242[44], 8}, {NULL}};
const Segment SGI133020[] = {{&S14242[44], 20}, {NULL}};
const Segment SGI133021[] = {{&S14242[30], 6}, {NULL}};
const Segment SGI133022[] = {{&S14242[113], 3}, {NULL}};
const Segment SGI133023[] = {{&S14242[78], 6}, {NULL}};
const Segment SGI133024[] = {{&S14246[44], 11}, {NULL}};
const Segment SGI133025[] = {{&S14246[44], 23}, {NULL}};
const Segment SGI133026[] = {{&S14246[30], 6}, {NULL}};
const Segment SGI133027[] = {{&S14246[116], 3}, {NULL}};
const Segment SGI133028[] = {{&S14246[81], 6}, {NULL}};
const Segment SGI133029[] = {{&S14247[44], 20}, {NULL}};
const Segment SGI133030[] = {{&S14247[44], 44}, {NULL}};
const Segment SGI133031[] = {{&S14247[30], 6}, {NULL}};
const Segment SGI133032[] = {{&S14247[137], 3}, {NULL}};
const Segment SGI133033[] = {{&S14247[102], 6}, {NULL}};
const Segment SGI133034[] = {{&S14248[42], 16}, {NULL}};
const Segment SGI133035[] = {{&S14248[30], 6}, {NULL}};
const Segment SGI133036[] = {{&S14248[119], 3}, {NULL}};
const Segment SGI133037[] = {{&S14248[84], 6}, {NULL}};
const Segment SGI133038[] = {{&S14249[44], 7}, {NULL}};
const Segment SGI133039[] = {{&S14249[44], 19}, {NULL}};
const Segment SGI133040[] = {{&S14249[30], 6}, {NULL}};
const Segment SGI133041[] = {{&S14249[112], 3}, {NULL}};
const Segment SGI133042[] = {{&S14249[77], 6}, {NULL}};
const Segment SGI133043[] = {{&S14251[44], 14}, {NULL}};
const Segment SGI133044[] = {{&S14251[44], 26}, {NULL}};
const Segment SGI133045[] = {{&S14251[30], 6}, {NULL}};
const Segment SGI133046[] = {{&S14251[119], 3}, {NULL}};
const Segment SGI133047[] = {{&S14251[84], 6}, {NULL}};
const Segment SGI133048[] = {{&S14256[44], 16}, {NULL}};
const Segment SGI133049[] = {{&S14256[44], 27}, {NULL}};
const Segment SGI133050[] = {{&S14256[30], 6}, {NULL}};
const Segment SGI133051[] = {{&S14256[120], 3}, {NULL}};
const Segment SGI133052[] = {{&S14256[85], 6}, {NULL}};
const Segment SGI133053[] = {{&S14257[44], 5}, {NULL}};
const Segment SGI133054[] = {{&S14257[44], 17}, {NULL}};
const Segment SGI133055[] = {{&S14257[30], 6}, {NULL}};
const Segment SGI133056[] = {{&S14257[110], 3}, {NULL}};
const Segment SGI133057[] = {{&S14257[75], 6}, {NULL}};
const Segment SGI133058[] = {{&S14259[44], 6}, {NULL}};
const Segment SGI133059[] = {{&S14259[44], 18}, {NULL}};
const Segment SGI133060[] = {{&S14259[30], 6}, {NULL}};
const Segment SGI133061[] = {{&S14259[111], 3}, {NULL}};
const Segment SGI133062[] = {{&S14259[76], 6}, {NULL}};
const Segment SGI133063[] = {{&S14261[44], 8}, {NULL}};
const Segment SGI133064[] = {{&S14261[44], 20}, {NULL}};
const Segment SGI133065[] = {{&S14261[30], 6}, {NULL}};
const Segment SGI133066[] = {{&S14261[113], 3}, {NULL}};
const Segment SGI133067[] = {{&S14261[78], 6}, {NULL}};
const Segment SGI133068[] = {{&S14263[44], 15}, {NULL}};
const Segment SGI133069[] = {{&S14263[44], 27}, {NULL}};
const Segment SGI133070[] = {{&S14263[30], 6}, {NULL}};
const Segment SGI133071[] = {{&S14263[120], 3}, {NULL}};
const Segment SGI133072[] = {{&S14263[85], 6}, {NULL}};
const Segment SGI133073[] = {{&S14267[44], 17}, {NULL}};
const Segment SGI133074[] = {{&S14267[44], 29}, {NULL}};
const Segment SGI133075[] = {{&S14267[30], 6}, {NULL}};
const Segment SGI133076[] = {{&S14267[122], 3}, {NULL}};
const Segment SGI133077[] = {{&S14267[87], 6}, {NULL}};
const Segment SGI133078[] = {{&S14278[44], 10}, {NULL}};
const Segment SGI133079[] = {{&S14278[44], 22}, {NULL}};
const Segment SGI133080[] = {{&S14278[30], 6}, {NULL}};
const Segment SGI133081[] = {{&S14278[115], 3}, {NULL}};
const Segment SGI133082[] = {{&S14278[80], 6}, {NULL}};
const Segment SGI133083[] = {{&S14282[44], 11}, {NULL}};
const Segment SGI133084[] = {{&S14282[44], 23}, {NULL}};
const Segment SGI133085[] = {{&S14282[30], 6}, {NULL}};
const Segment SGI133086[] = {{&S14282[116], 3}, {NULL}};
const Segment SGI133087[] = {{&S14282[81], 6}, {NULL}};
const Segment SGI133088[] = {{&S14284[44], 10}, {NULL}};
const Segment SGI133089[] = {{&S14284[44], 22}, {NULL}};
const Segment SGI133090[] = {{&S14284[30], 6}, {NULL}};
const Segment SGI133091[] = {{&S14284[115], 3}, {NULL}};
const Segment SGI133092[] = {{&S14284[80], 6}, {NULL}};
const Segment SGI133093[] = {{&S14286[44], 4}, {NULL}};
const Segment SGI133094[] = {{&S14286[44], 16}, {NULL}};
const Segment SGI133095[] = {{&S14286[30], 6}, {NULL}};
const Segment SGI133096[] = {{&S14286[109], 3}, {NULL}};
const Segment SGI133097[] = {{&S14286[74], 6}, {NULL}};
const Segment SGI133098[] = {{&S14292[44], 11}, {NULL}};
const Segment SGI133099[] = {{&S14292[44], 23}, {NULL}};
const Segment SGI133100[] = {{&S14292[30], 6}, {NULL}};
const Segment SGI133101[] = {{&S14292[116], 3}, {NULL}};
const Segment SGI133102[] = {{&S14292[81], 6}, {NULL}};
const Segment SGI133103[] = {{&S14294[44], 6}, {NULL}};
const Segment SGI133104[] = {{&S14294[44], 24}, {NULL}};
const Segment SGI133105[] = {{&S14294[30], 6}, {NULL}};
const Segment SGI133106[] = {{&S14294[117], 3}, {NULL}};
const Segment SGI133107[] = {{&S14294[82], 6}, {NULL}};
const Segment SGI133108[] = {{&S14297[44], 14}, {NULL}};
const Segment SGI133109[] = {{&S14297[44], 26}, {NULL}};
const Segment SGI133110[] = {{&S14297[30], 6}, {NULL}};
const Segment SGI133111[] = {{&S14297[119], 3}, {NULL}};
const Segment SGI133112[] = {{&S14297[84], 6}, {NULL}};
const Segment SGI133113[] = {{&S14300[44], 10}, {NULL}};
const Segment SGI133114[] = {{&S14300[44], 22}, {NULL}};
const Segment SGI133115[] = {{&S14300[30], 6}, {NULL}};
const Segment SGI133116[] = {{&S14300[115], 3}, {NULL}};
const Segment SGI133117[] = {{&S14300[80], 6}, {NULL}};
const Segment SGI133118[] = {{&S14301[44], 14}, {NULL}};
const Segment SGI133119[] = {{&S14301[44], 31}, {NULL}};
const Segment SGI133120[] = {{&S14301[30], 6}, {NULL}};
const Segment SGI133121[] = {{&S14301[124], 3}, {NULL}};
const Segment SGI133122[] = {{&S14301[89], 6}, {NULL}};
const Segment SGI133123[] = {{&S14302[38], 10}, {NULL}};
const Segment SGI133124[] = {{&S14302[31], 6}, {NULL}};
const Segment SGI133125[] = {{&S14305[44], 4}, {NULL}};
const Segment SGI133126[] = {{&S14305[44], 16}, {NULL}};
const Segment SGI133127[] = {{&S14305[30], 6}, {NULL}};
const Segment SGI133128[] = {{&S14305[109], 3}, {NULL}};
const Segment SGI133129[] = {{&S14305[74], 6}, {NULL}};
const Segment SGI133130[] = {{&S14307[44], 20}, {NULL}};
const Segment SGI133131[] = {{&S14307[44], 32}, {NULL}};
const Segment SGI133132[] = {{&S14307[30], 6}, {NULL}};
const Segment SGI133133[] = {{&S14307[125], 3}, {NULL}};
const Segment SGI133134[] = {{&S14307[90], 6}, {NULL}};
const Segment SGI133135[] = {{&S14309[44], 84}, {NULL}};
const Segment SGI133136[] = {{&S14309[44], 105}, {NULL}};
const Segment SGI133137[] = {{&S14309[30], 6}, {NULL}};
const Segment SGI133138[] = {{&S14309[196], 3}, {NULL}};
const Segment SGI133139[] = {{&S14309[122], 5}, {NULL}};
const Segment SGI133140[] = {{&S14310[34], 6}, {NULL}};
const Segment SGI133141[] = {{&S14310[27], 6}, {NULL}};
const Segment SGI133142[] = {{&S14310[101], 3}, {NULL}};
const Segment SGI133143[] = {{&S14310[66], 6}, {NULL}};
const Segment SGI133144[] = {{&S14313[44], 10}, {NULL}};
const Segment SGI133145[] = {{&S14313[44], 22}, {NULL}};
const Segment SGI133146[] = {{&S14313[30], 6}, {NULL}};
const Segment SGI133147[] = {{&S14313[115], 3}, {NULL}};
const Segment SGI133148[] = {{&S14313[80], 6}, {NULL}};
const Segment SGI133149[] = {{&S14319[44], 8}, {NULL}};
const Segment SGI133150[] = {{&S14319[44], 20}, {NULL}};
const Segment SGI133151[] = {{&S14319[30], 6}, {NULL}};
const Segment SGI133152[] = {{&S14319[113], 3}, {NULL}};
const Segment SGI133153[] = {{&S14319[78], 6}, {NULL}};
const Segment SGI133154[] = {{&S14322[44], 84}, {NULL}};
const Segment SGI133155[] = {{&S14322[44], 105}, {NULL}};
const Segment SGI133156[] = {{&S14322[30], 6}, {NULL}};
const Segment SGI133157[] = {{&S14322[196], 3}, {NULL}};
const Segment SGI133158[] = {{&S14322[122], 5}, {NULL}};
const Segment SGI133159[] = {{&S14323[44], 6}, {NULL}};
const Segment SGI133160[] = {{&S14323[44], 18}, {NULL}};
const Segment SGI133161[] = {{&S14323[30], 6}, {NULL}};
const Segment SGI133162[] = {{&S14323[111], 3}, {NULL}};
const Segment SGI133163[] = {{&S14323[76], 6}, {NULL}};
const Segment SGI133164[] = {{&S14324[44], 10}, {NULL}};
const Segment SGI133165[] = {{&S14324[44], 22}, {NULL}};
const Segment SGI133166[] = {{&S14324[30], 6}, {NULL}};
const Segment SGI133167[] = {{&S14324[115], 3}, {NULL}};
const Segment SGI133168[] = {{&S14324[80], 6}, {NULL}};
const Segment SGI133169[] = {{&S14331[44], 14}, {NULL}};
const Segment SGI133170[] = {{&S14331[44], 26}, {NULL}};
const Segment SGI133171[] = {{&S14331[30], 6}, {NULL}};
const Segment SGI133172[] = {{&S14331[119], 3}, {NULL}};
const Segment SGI133173[] = {{&S14331[84], 6}, {NULL}};
const Segment SGI133174[] = {{&S25430[44], 10}, {NULL}};
const Segment SGI133175[] = {{&S25430[44], 22}, {NULL}};
const Segment SGI133176[] = {{&S25430[30], 6}, {NULL}};
const Segment SGI133177[] = {{&S25430[31], 3}, {NULL}};
const Segment SGI133178[] = {{&S25430[80], 6}, {NULL}};
const Segment SGI133179[] = {{&S14334[44], 7}, {NULL}};
const Segment SGI133180[] = {{&S14334[44], 19}, {NULL}};
const Segment SGI133181[] = {{&S14334[30], 6}, {NULL}};
const Segment SGI133182[] = {{&S14334[112], 3}, {NULL}};
const Segment SGI133183[] = {{&S14334[77], 6}, {NULL}};
const Segment SGI133184[] = {{&S25431[44], 10}, {NULL}};
const Segment SGI133185[] = {{&S25431[44], 22}, {NULL}};
const Segment SGI133186[] = {{&S25431[30], 6}, {NULL}};
const Segment SGI133187[] = {{&S25431[31], 3}, {NULL}};
const Segment SGI133188[] = {{&S25431[80], 6}, {NULL}};
const Segment SGI133189[] = {{&S14336[44], 7}, {NULL}};
const Segment SGI133190[] = {{&S14336[44], 19}, {NULL}};
const Segment SGI133191[] = {{&S14336[30], 6}, {NULL}};
const Segment SGI133192[] = {{&S14336[112], 3}, {NULL}};
const Segment SGI133193[] = {{&S14336[77], 6}, {NULL}};
const Segment SGI133194[] = {{&S25434[44], 10}, {NULL}};
const Segment SGI133195[] = {{&S25434[44], 22}, {NULL}};
const Segment SGI133196[] = {{&S25434[30], 6}, {NULL}};
const Segment SGI133197[] = {{&S25434[31], 3}, {NULL}};
const Segment SGI133198[] = {{&S25434[80], 6}, {NULL}};
const Segment SGI133199[] = {{&S25435[38], 10}, {NULL}};
const Segment SGI133200[] = {{&S25435[31], 6}, {NULL}};
const Segment SGI133201[] = {{&S14341[44], 13}, {NULL}};
const Segment SGI133202[] = {{&S14341[44], 25}, {NULL}};
const Segment SGI133203[] = {{&S14341[30], 6}, {NULL}};
const Segment SGI133204[] = {{&S25436[42], 10}, {NULL}};
const Segment SGI133205[] = {{&S25436[30], 6}, {NULL}};
const Segment SGI133206[] = {{&S14341[118], 3}, {NULL}};
const Segment SGI133207[] = {{&S14341[83], 6}, {NULL}};
const Segment SGI133208[] = {{&S25436[31], 3}, {NULL}};
const Segment SGI133209[] = {{&S25436[78], 6}, {NULL}};
const Segment SGI133210[] = {{&S14342[44], 10}, {NULL}};
const Segment SGI133211[] = {{&S14342[44], 22}, {NULL}};
const Segment SGI133212[] = {{&S14342[30], 6}, {NULL}};
const Segment SGI133213[] = {{&S25437[44], 28}, {NULL}};
const Segment SGI133214[] = {{&S25437[44], 40}, {NULL}};
const Segment SGI133215[] = {{&S25437[30], 6}, {NULL}};
const Segment SGI133216[] = {{&S14342[83], 6}, {NULL}};
const Segment SGI133217[] = {{&S25437[31], 3}, {NULL}};
const Segment SGI133218[] = {{&S25437[98], 6}, {NULL}};
const Segment SGI133219[] = {{&S25438[38], 10}, {NULL}};
const Segment SGI133220[] = {{&S25438[31], 6}, {NULL}};
const Segment SGI133221[] = {{&S14345[44], 6}, {NULL}};
const Segment SGI133222[] = {{&S14345[44], 18}, {NULL}};
const Segment SGI133223[] = {{&S14345[30], 6}, {NULL}};
const Segment SGI133224[] = {{&S14345[111], 3}, {NULL}};
const Segment SGI133225[] = {{&S25439[44], 28}, {NULL}};
const Segment SGI133226[] = {{&S14345[76], 6}, {NULL}};
const Segment SGI133227[] = {{&S25439[44], 40}, {NULL}};
const Segment SGI133228[] = {{&S25439[30], 6}, {NULL}};
const Segment SGI133229[] = {{&S25439[31], 3}, {NULL}};
const Segment SGI133230[] = {{&S25439[98], 6}, {NULL}};
const Segment SGI133231[] = {{&S14346[44], 14}, {NULL}};
const Segment SGI133232[] = {{&S14346[44], 26}, {NULL}};
const Segment SGI133233[] = {{&S14346[30], 6}, {NULL}};
const Segment SGI133234[] = {{&S14346[119], 3}, {NULL}};
const Segment SGI133235[] = {{&S14346[84], 6}, {NULL}};
const Segment SGI133236[] = {{&S25441[44], 10}, {NULL}};
const Segment SGI133237[] = {{&S14347[44], 6}, {NULL}};
const Segment SGI133238[] = {{&S25441[44], 74}, {NULL}};
const Segment SGI133239[] = {{&S14347[44], 18}, {NULL}};
const Segment SGI133240[] = {{&S25441[30], 6}, {NULL}};
const Segment SGI133241[] = {{&S14347[30], 6}, {NULL}};
const Segment SGI133242[] = {{&S14347[111], 3}, {NULL}};
const Segment SGI133243[] = {{&S14347[76], 6}, {NULL}};
const Segment SGI133244[] = {{&S25442[44], 10}, {NULL}};
const Segment SGI133245[] = {{&S25442[44], 22}, {NULL}};
const Segment SGI133246[] = {{&S25442[30], 6}, {NULL}};
const Segment SGI133247[] = {{&S25442[31], 3}, {NULL}};
const Segment SGI133248[] = {{&S25442[80], 6}, {NULL}};
const Segment SGI133249[] = {{&S25443[44], 10}, {NULL}};
const Segment SGI133250[] = {{&S25443[44], 22}, {NULL}};
const Segment SGI133251[] = {{&S25443[30], 6}, {NULL}};
const Segment SGI133252[] = {{&S25443[80], 6}, {NULL}};
const Segment SGI133253[] = {{&S25444[44], 10}, {NULL}};
const Segment SGI133254[] = {{&S14352[44], 7}, {NULL}};
const Segment SGI133255[] = {{&S25444[44], 22}, {NULL}};
const Segment SGI133256[] = {{&S14352[44], 19}, {NULL}};
const Segment SGI133257[] = {{&S25444[30], 6}, {NULL}};
const Segment SGI133258[] = {{&S14352[30], 6}, {NULL}};
const Segment SGI133259[] = {{&S14352[112], 3}, {NULL}};
const Segment SGI133260[] = {{&S25444[83], 6}, {NULL}};
const Segment SGI133261[] = {{&S14352[77], 6}, {NULL}};
const Segment SGI133262[] = {{&S25445[44], 10}, {NULL}};
const Segment SGI133263[] = {{&S14353[44], 16}, {NULL}};
const Segment SGI133264[] = {{&S25445[44], 22}, {NULL}};
const Segment SGI133265[] = {{&S14353[44], 28}, {NULL}};
const Segment SGI133266[] = {{&S14353[30], 6}, {NULL}};
const Segment SGI133267[] = {{&S25445[30], 6}, {NULL}};
const Segment SGI133268[] = {{&S14353[121], 3}, {NULL}};
const Segment SGI133269[] = {{&S25445[125], 3}, {NULL}};
const Segment SGI133270[] = {{&S14353[86], 6}, {NULL}};
const Segment SGI133271[] = {{&S25445[90], 6}, {NULL}};
const Segment SGI133272[] = {{&S25445[145], 8}, {NULL}};
const Segment SGI133273[] = {{&S25446[34], 10}, {NULL}};
const Segment SGI133274[] = {{&S25446[34], 22}, {NULL}};
const Segment SGI133275[] = {{&S14356[44], 6}, {NULL}};
const Segment SGI133276[] = {{&S25447[44], 10}, {NULL}};
const Segment SGI133277[] = {{&S14356[44], 18}, {NULL}};
const Segment SGI133278[] = {{&S25447[44], 28}, {NULL}};
const Segment SGI133279[] = {{&S14356[30], 6}, {NULL}};
const Segment SGI133280[] = {{&S25447[30], 6}, {NULL}};
const Segment SGI133281[] = {{&S14356[111], 3}, {NULL}};
const Segment SGI133282[] = {{&S14356[76], 6}, {NULL}};
const Segment SGI133283[] = {{&S25448[42], 10}, {NULL}};
const Segment SGI133284[] = {{&S14357[44], 10}, {NULL}};
const Segment SGI133285[] = {{&S25448[30], 6}, {NULL}};
const Segment SGI133286[] = {{&S14357[44], 22}, {NULL}};
const Segment SGI133287[] = {{&S14357[30], 6}, {NULL}};
const Segment SGI133288[] = {{&S25448[31], 3}, {NULL}};
const Segment SGI133289[] = {{&S14357[115], 3}, {NULL}};
const Segment SGI133290[] = {{&S25448[78], 6}, {NULL}};
const Segment SGI133291[] = {{&S14357[80], 6}, {NULL}};
const Segment SGI133292[] = {{&S25450[44], 10}, {NULL}};
const Segment SGI133293[] = {{&S25450[44], 22}, {NULL}};
const Segment SGI133294[] = {{&S25450[30], 6}, {NULL}};
const Segment SGI133295[] = {{&S14360[44], 18}, {NULL}};
const Segment SGI133296[] = {{&S14360[44], 30}, {NULL}};
const Segment SGI133297[] = {{&S14360[30], 6}, {NULL}};
const Segment SGI133298[] = {{&S25450[31], 3}, {NULL}};
const Segment SGI133299[] = {{&S25450[80], 5}, {NULL}};
const Segment SGI133300[] = {{&S14360[123], 3}, {NULL}};
const Segment SGI133301[] = {{&S14360[88], 6}, {NULL}};
const Segment SGI133302[] = {{&S25451[45], 10}, {NULL}};
const Segment SGI133303[] = {{&S25451[45], 29}, {NULL}};
const Segment SGI133304[] = {{&S25451[31], 6}, {NULL}};
const Segment SGI133305[] = {{&S14362[44], 7}, {NULL}};
const Segment SGI133306[] = {{&S14362[44], 19}, {NULL}};
const Segment SGI133307[] = {{&S14362[30], 6}, {NULL}};
const Segment SGI133308[] = {{&S14362[112], 3}, {NULL}};
const Segment SGI133309[] = {{&S14362[77], 6}, {NULL}};
const Segment SGI133310[] = {{&S25452[34], 10}, {NULL}};
const Segment SGI133311[] = {{&S25452[27], 6}, {NULL}};
const Segment SGI133312[] = {{&S25452[28], 3}, {NULL}};
const Segment SGI133313[] = {{&S25452[70], 6}, {NULL}};
const Segment SGI133314[] = {{&S25453[44], 10}, {NULL}};
const Segment SGI133315[] = {{&S25453[44], 22}, {NULL}};
const Segment SGI133316[] = {{&S25453[30], 6}, {NULL}};
const Segment SGI133317[] = {{&S25453[31], 3}, {NULL}};
const Segment SGI133318[] = {{&S25453[80], 6}, {NULL}};
const Segment SGI133319[] = {{&S25454[44], 10}, {NULL}};
const Segment SGI133320[] = {{&S25454[44], 28}, {NULL}};
const Segment SGI133321[] = {{&S25454[30], 6}, {NULL}};
const Segment SGI133322[] = {{&S14368[44], 6}, {NULL}};
const Segment SGI133323[] = {{&S25455[44], 10}, {NULL}};
const Segment SGI133324[] = {{&S14368[44], 17}, {NULL}};
const Segment SGI133325[] = {{&S25455[44], 28}, {NULL}};
const Segment SGI133326[] = {{&S14368[30], 6}, {NULL}};
const Segment SGI133327[] = {{&S25455[30], 6}, {NULL}};
const Segment SGI133328[] = {{&S14368[110], 3}, {NULL}};
const Segment SGI133329[] = {{&S14368[75], 6}, {NULL}};
const Segment SGI133330[] = {{&S25456[44], 10}, {&S25456[175], 18}, {NULL}};
const Segment SGI133331[] = {{&S25456[44], 22}, {NULL}};
const Segment SGI133332[] = {{&S25456[30], 6}, {NULL}};
const Segment SGI133333[] = {{&S25456[31], 3}, {NULL}};
const Segment SGI133334[] = {{&S14370[44], 6}, {NULL}};
const Segment SGI133335[] = {{&S25456[80], 6}, {NULL}};
const Segment SGI133336[] = {{&S14370[44], 18}, {NULL}};
const Segment SGI133337[] = {{&S14370[30], 6}, {NULL}};
const Segment SGI133338[] = {{&S14370[110], 3}, {NULL}};
const Segment SGI133339[] = {{&S14370[76], 5}, {NULL}};
const Segment SGI133340[] = {{&S25457[44], 10}, {NULL}};
const Segment SGI133341[] = {{&S25457[44], 22}, {NULL}};
const Segment SGI133342[] = {{&S25457[30], 6}, {NULL}};
const Segment SGI133343[] = {{&S25457[31], 3}, {&S25457[151], 3}, {NULL}};
const Segment SGI133344[] = {{&S25457[80], 6}, {NULL}};
const Segment SGI133345[] = {{&S14372[44], 10}, {NULL}};
const Segment SGI133346[] = {{&S14372[44], 22}, {NULL}};
const Segment SGI133347[] = {{&S14372[30], 6}, {NULL}};
const Segment SGI133348[] = {{&S14372[115], 3}, {NULL}};
const Segment SGI133349[] = {{&S14372[80], 6}, {NULL}};
const Segment SGI133350[] = {{&S25458[44], 28}, {NULL}};
const Segment SGI133351[] = {{&S25458[44], 40}, {NULL}};
const Segment SGI133352[] = {{&S25458[30], 6}, {NULL}};
const Segment SGI133353[] = {{&S25458[31], 3}, {NULL}};
const Segment SGI133354[] = {{&S25458[98], 6}, {NULL}};
const Segment SGI133355[] = {{&S14373[44], 8}, {NULL}};
const Segment SGI133356[] = {{&S14373[44], 20}, {NULL}};
const Segment SGI133357[] = {{&S14373[30], 6}, {NULL}};
const Segment SGI133358[] = {{&S14373[113], 3}, {NULL}};
const Segment SGI133359[] = {{&S14373[78], 6}, {NULL}};
const Segment SGI133360[] = {{&S25459[58], 10}, {NULL}};
const Segment SGI133361[] = {{&S25459[58], 28}, {NULL}};
const Segment SGI133362[] = {{&S25459[44], 6}, {NULL}};
const Segment SGI133363[] = {{&S14374[44], 11}, {NULL}};
const Segment SGI133364[] = {{&S14374[44], 23}, {NULL}};
const Segment SGI133365[] = {{&S14374[30], 6}, {NULL}};
const Segment SGI133366[] = {{&S14374[116], 3}, {NULL}};
const Segment SGI133367[] = {{&S14374[81], 6}, {NULL}};
const Segment SGI133368[] = {{&S25460[30], 1}, {NULL}};
const Segment SGI133369[] = {{&S25460[63], 9}, {NULL}};
const Segment SGI133370[] = {{&S25460[49], 6}, {NULL}};
const Segment SGI133371[] = {{&S25461[55], 18}, {NULL}};
const Segment SGI133372[] = {{&S25461[48], 6}, {NULL}};
const Segment SGI133373[] = {{&S25462[99], 29}, {NULL}};
const Segment SGI133374[] = {{&S14379[44], 17}, {NULL}};
const Segment SGI133375[] = {{&S14379[44], 29}, {NULL}};
const Segment SGI133376[] = {{&S14379[30], 6}, {NULL}};
const Segment SGI133377[] = {{&S14379[122], 3}, {NULL}};
const Segment SGI133378[] = {{&S14379[87], 6}, {NULL}};
const Segment SGI133379[] = {{&S25463[44], 10}, {NULL}};
const Segment SGI133380[] = {{&S25463[44], 22}, {NULL}};
const Segment SGI133381[] = {{&S25463[30], 6}, {NULL}};
const Segment SGI133382[] = {{&S25463[31], 3}, {NULL}};
const Segment SGI133383[] = {{&S25463[80], 5}, {NULL}};
const Segment SGI133384[] = {{&S14381[44], 31}, {NULL}};
const Segment SGI133385[] = {{&S14381[30], 6}, {NULL}};
const Segment SGI133386[] = {{&S25465[44], 10}, {NULL}};
const Segment SGI133387[] = {{&S25465[44], 22}, {NULL}};
const Segment SGI133388[] = {{&S25465[30], 6}, {NULL}};
const Segment SGI133389[] = {{&S25465[116], 3}, {NULL}};
const Segment SGI133390[] = {{&S25465[80], 7}, {NULL}};
const Segment SGI133391[] = {{&S25465[136], 8}, {NULL}};
const Segment SGI133392[] = {{&S14382[44], 10}, {NULL}};
const Segment SGI133393[] = {{&S14382[44], 22}, {NULL}};
const Segment SGI133394[] = {{&S14382[30], 6}, {NULL}};
const Segment SGI133395[] = {{&S14382[115], 3}, {NULL}};
const Segment SGI133396[] = {{&S14382[80], 6}, {NULL}};
const Segment SGI133397[] = {{&S14383[44], 15}, {NULL}};
const Segment SGI133398[] = {{&S14383[44], 33}, {NULL}};
const Segment SGI133399[] = {{&S14383[30], 6}, {NULL}};
const Segment SGI133400[] = {{&S14383[126], 3}, {NULL}};
const Segment SGI133401[] = {{&S14383[91], 6}, {NULL}};
const Segment SGI133402[] = {{&S25468[44], 10}, {NULL}};
const Segment SGI133403[] = {{&S25468[44], 22}, {NULL}};
const Segment SGI133404[] = {{&S25468[30], 13}, {NULL}};
const Segment SGI133405[] = {{&S25468[31], 3}, {NULL}};
const Segment SGI133406[] = {{&S25468[80], 6}, {NULL}};
const Segment SGI133407[] = {{&S14384[44], 18}, {NULL}};
const Segment SGI133408[] = {{&S14384[44], 30}, {NULL}};
const Segment SGI133409[] = {{&S14384[30], 6}, {NULL}};
const Segment SGI133410[] = {{&S14384[123], 3}, {NULL}};
const Segment SGI133411[] = {{&S14384[88], 6}, {NULL}};
const Segment SGI133412[] = {{&S25469[46], 10}, {NULL}};
const Segment SGI133413[] = {{&S25469[46], 28}, {NULL}};
const Segment SGI133414[] = {{&S25469[32], 6}, {NULL}};
const Segment SGI133415[] = {{&S25470[65], 10}, {NULL}};
const Segment SGI133416[] = {{&S25470[65], 22}, {NULL}};
const Segment SGI133417[] = {{&S25470[51], 6}, {NULL}};
const Segment SGI133418[] = {{&S14386[44], 12}, {NULL}};
const Segment SGI133419[] = {{&S14386[44], 24}, {NULL}};
const Segment SGI133420[] = {{&S14386[30], 6}, {NULL}};
const Segment SGI133421[] = {{&S14386[117], 3}, {NULL}};
const Segment SGI133422[] = {{&S14386[82], 6}, {NULL}};
const Segment SGI133423[] = {{&S25473[46], 10}, {NULL}};
const Segment SGI133424[] = {{&S25473[46], 28}, {NULL}};
const Segment SGI133425[] = {{&S25473[32], 6}, {NULL}};
const Segment SGI133426[] = {{&S25474[59], 10}, {NULL}};
const Segment SGI133427[] = {{&S25474[59], 28}, {NULL}};
const Segment SGI133428[] = {{&S25474[45], 6}, {NULL}};
const Segment SGI133429[] = {{&S14390[44], 12}, {NULL}};
const Segment SGI133430[] = {{&S14390[44], 24}, {NULL}};
const Segment SGI133431[] = {{&S14390[30], 6}, {NULL}};
const Segment SGI133432[] = {{&S14390[117], 3}, {NULL}};
const Segment SGI133433[] = {{&S14390[82], 6}, {NULL}};
const Segment SGI133434[] = {{&S14391[44], 13}, {NULL}};
const Segment SGI133435[] = {{&S14391[44], 25}, {NULL}};
const Segment SGI133436[] = {{&S14391[30], 6}, {NULL}};
const Segment SGI133437[] = {{&S25476[37], 6}, {NULL}};
const Segment SGI133438[] = {{&S14391[118], 3}, {NULL}};
const Segment SGI133439[] = {{&S14391[83], 6}, {NULL}};
const Segment SGI133440[] = {{&S14394[44], 6}, {NULL}};
const Segment SGI133441[] = {{&S14394[44], 18}, {NULL}};
const Segment SGI133442[] = {{&S14394[30], 6}, {NULL}};
const Segment SGI133443[] = {{&S14394[111], 3}, {NULL}};
const Segment SGI133444[] = {{&S14394[76], 6}, {NULL}};
const Segment SGI133445[] = {{&S14395[44], 9}, {NULL}};
const Segment SGI133446[] = {{&S14395[44], 21}, {NULL}};
const Segment SGI133447[] = {{&S14395[30], 6}, {NULL}};
const Segment SGI133448[] = {{&S14395[114], 3}, {NULL}};
const Segment SGI133449[] = {{&S14395[79], 6}, {NULL}};
const Segment SGI133450[] = {{&S25483[42], 28}, {NULL}};
const Segment SGI133451[] = {{&S14397[44], 7}, {NULL}};
const Segment SGI133452[] = {{&S14397[44], 19}, {NULL}};
const Segment SGI133453[] = {{&S25483[30], 6}, {NULL}};
const Segment SGI133454[] = {{&S14397[30], 6}, {NULL}};
const Segment SGI133455[] = {{&S25483[31], 3}, {NULL}};
const Segment SGI133456[] = {{&S14397[112], 3}, {NULL}};
const Segment SGI133457[] = {{&S25483[96], 6}, {NULL}};
const Segment SGI133458[] = {{&S14397[77], 6}, {NULL}};
const Segment SGI133459[] = {{&S14398[44], 15}, {NULL}};
const Segment SGI133460[] = {{&S14398[44], 27}, {NULL}};
const Segment SGI133461[] = {{&S25484[45], 10}, {NULL}};
const Segment SGI133462[] = {{&S14398[30], 6}, {NULL}};
const Segment SGI133463[] = {{&S25484[45], 22}, {NULL}};
const Segment SGI133464[] = {{&S25484[31], 6}, {NULL}};
const Segment SGI133465[] = {{&S14398[120], 3}, {NULL}};
const Segment SGI133466[] = {{&S25484[81], 6}, {NULL}};
const Segment SGI133467[] = {{&S14398[85], 6}, {NULL}};
const Segment SGI133468[] = {{&S14399[44], 12}, {NULL}};
const Segment SGI133469[] = {{&S14399[44], 24}, {NULL}};
const Segment SGI133470[] = {{&S14399[30], 6}, {NULL}};
const Segment SGI133471[] = {{&S14399[117], 3}, {NULL}};
const Segment SGI133472[] = {{&S14399[82], 6}, {NULL}};
const Segment SGI133473[] = {{&S14400[44], 10}, {NULL}};
const Segment SGI133474[] = {{&S14400[44], 22}, {NULL}};
const Segment SGI133475[] = {{&S14400[30], 6}, {NULL}};
const Segment SGI133476[] = {{&S14400[115], 3}, {NULL}};
const Segment SGI133477[] = {{&S14400[80], 6}, {NULL}};
const Segment SGI133478[] = {{&S25487[44], 10}, {NULL}};
const Segment SGI133479[] = {{&S25487[44], 22}, {&S25487[205], 7}, {NULL}};
const Segment SGI133480[] = {{&S25487[30], 6}, {NULL}};
const Segment SGI133481[] = {{&S25487[31], 3}, {NULL}};
const Segment SGI133482[] = {{&S25487[80], 6}, {NULL}};
const Segment SGI133483[] = {{&S14402[44], 17}, {NULL}};
const Segment SGI133484[] = {{&S14402[44], 29}, {NULL}};
const Segment SGI133485[] = {{&S14402[30], 6}, {NULL}};
const Segment SGI133486[] = {{&S25488[46], 9}, {NULL}};
const Segment SGI133487[] = {{&S14402[122], 3}, {NULL}};
const Segment SGI133488[] = {{&S25488[32], 6}, {NULL}};
const Segment SGI133489[] = {{&S14402[87], 6}, {NULL}};
const Segment SGI133490[] = {{&S14403[44], 7}, {NULL}};
const Segment SGI133491[] = {{&S14403[44], 18}, {NULL}};
const Segment SGI133492[] = {{&S14403[30], 6}, {NULL}};
const Segment SGI133493[] = {{&S25489[44], 10}, {NULL}};
const Segment SGI133494[] = {{&S14403[111], 3}, {NULL}};
const Segment SGI133495[] = {{&S25489[44], 22}, {NULL}};
const Segment SGI133496[] = {{&S14403[76], 6}, {NULL}};
const Segment SGI133497[] = {{&S25489[30], 6}, {NULL}};
const Segment SGI133498[] = {{&S25489[31], 3}, {&S25489[151], 3}, {NULL}};
const Segment SGI133499[] = {{&S25489[80], 6}, {NULL}};
const Segment SGI133500[] = {{&S14404[44], 14}, {NULL}};
const Segment SGI133501[] = {{&S14404[44], 31}, {NULL}};
const Segment SGI133502[] = {{&S14404[30], 6}, {NULL}};
const Segment SGI133503[] = {{&S14404[92], 6}, {NULL}};
const Segment SGI133504[] = {{&S25490[44], 10}, {NULL}};
const Segment SGI133505[] = {{&S25490[44], 19}, {NULL}};
const Segment SGI133506[] = {{&S25490[30], 6}, {NULL}};
const Segment SGI133507[] = {{&S25490[31], 3}, {NULL}};
const Segment SGI133508[] = {{&S25490[77], 5}, {NULL}};
const Segment SGI133509[] = {{&S14405[44], 13}, {NULL}};
const Segment SGI133510[] = {{&S14405[44], 25}, {NULL}};
const Segment SGI133511[] = {{&S14405[30], 6}, {NULL}};
const Segment SGI133512[] = {{&S14405[118], 3}, {NULL}};
const Segment SGI133513[] = {{&S14405[83], 6}, {NULL}};
const Segment SGI133514[] = {{&S25491[44], 10}, {NULL}};
const Segment SGI133515[] = {{&S25491[44], 22}, {NULL}};
const Segment SGI133516[] = {{&S25491[30], 6}, {NULL}};
const Segment SGI133517[] = {{&S14406[44], 5}, {NULL}};
const Segment SGI133518[] = {{&S25491[31], 3}, {&S25491[195], 3}, {NULL}};
const Segment SGI133519[] = {{&S14406[44], 17}, {NULL}};
const Segment SGI133520[] = {{&S14406[30], 6}, {NULL}};
const Segment SGI133521[] = {{&S25491[80], 6}, {NULL}};
const Segment SGI133522[] = {{&S14406[110], 3}, {NULL}};
const Segment SGI133523[] = {{&S14406[75], 6}, {NULL}};
const Segment SGI133524[] = {{&S25492[44], 10}, {NULL}};
const Segment SGI133525[] = {{&S25492[44], 22}, {NULL}};
const Segment SGI133526[] = {{&S25492[30], 6}, {NULL}};
const Segment SGI133527[] = {{&S25492[116], 3}, {NULL}};
const Segment SGI133528[] = {{&S25492[80], 7}, {NULL}};
const Segment SGI133529[] = {{&S25492[136], 8}, {NULL}};
const Segment SGI133530[] = {{&S14413[44], 20}, {NULL}};
const Segment SGI133531[] = {{&S14413[44], 32}, {NULL}};
const Segment SGI133532[] = {{&S14413[30], 6}, {NULL}};
const Segment SGI133533[] = {{&S14413[125], 3}, {NULL}};
const Segment SGI133534[] = {{&S14413[90], 6}, {NULL}};
const Segment SGI133535[] = {{&S14417[44], 10}, {NULL}};
const Segment SGI133536[] = {{&S14417[44], 22}, {NULL}};
const Segment SGI133537[] = {{&S14417[30], 6}, {NULL}};
const Segment SGI133538[] = {{&S14417[115], 3}, {NULL}};
const Segment SGI133539[] = {{&S14417[80], 6}, {NULL}};
const Segment SGI133540[] = {{&S14419[44], 6}, {NULL}};
const Segment SGI133541[] = {{&S14419[44], 18}, {NULL}};
const Segment SGI133542[] = {{&S14419[30], 6}, {NULL}};
const Segment SGI133543[] = {{&S14419[111], 3}, {NULL}};
const Segment SGI133544[] = {{&S14419[76], 6}, {NULL}};
const Segment SGI133545[] = {{&S14420[44], 19}, {NULL}};
const Segment SGI133546[] = {{&S14420[44], 31}, {NULL}};
const Segment SGI133547[] = {{&S14420[30], 6}, {NULL}};
const Segment SGI133548[] = {{&S14420[124], 3}, {NULL}};
const Segment SGI133549[] = {{&S14420[89], 6}, {NULL}};
const Segment SGI133550[] = {{&S14428[44], 14}, {NULL}};
const Segment SGI133551[] = {{&S14428[44], 26}, {NULL}};
const Segment SGI133552[] = {{&S14428[30], 6}, {NULL}};
const Segment SGI133553[] = {{&S14428[119], 3}, {NULL}};
const Segment SGI133554[] = {{&S14428[84], 6}, {NULL}};
const Segment SGI133555[] = {{&S14441[44], 10}, {NULL}};
const Segment SGI133556[] = {{&S14441[44], 27}, {NULL}};
const Segment SGI133557[] = {{&S14441[30], 6}, {NULL}};
const Segment SGI133558[] = {{&S14441[120], 3}, {NULL}};
const Segment SGI133559[] = {{&S14441[85], 6}, {NULL}};
const Segment SGI133560[] = {{&S14443[44], 17}, {NULL}};
const Segment SGI133561[] = {{&S14443[44], 29}, {NULL}};
const Segment SGI133562[] = {{&S14443[30], 6}, {NULL}};
const Segment SGI133563[] = {{&S14443[122], 3}, {NULL}};
const Segment SGI133564[] = {{&S14443[87], 6}, {NULL}};
const Segment SGI133565[] = {{&S14447[44], 6}, {NULL}};
const Segment SGI133566[] = {{&S14447[44], 18}, {NULL}};
const Segment SGI133567[] = {{&S14447[30], 6}, {NULL}};
const Segment SGI133568[] = {{&S14447[111], 3}, {NULL}};
const Segment SGI133569[] = {{&S14447[76], 6}, {NULL}};
const Segment SGI133570[] = {{&S14448[44], 5}, {NULL}};
const Segment SGI133571[] = {{&S14448[44], 17}, {NULL}};
const Segment SGI133572[] = {{&S14448[30], 6}, {NULL}};
const Segment SGI133573[] = {{&S14448[110], 3}, {NULL}};
const Segment SGI133574[] = {{&S14448[75], 6}, {NULL}};
const Segment SGI133575[] = {{&S14449[44], 5}, {NULL}};
const Segment SGI133576[] = {{&S14449[44], 17}, {NULL}};
const Segment SGI133577[] = {{&S14449[30], 6}, {NULL}};
const Segment SGI133578[] = {{&S14449[110], 3}, {NULL}};
const Segment SGI133579[] = {{&S14449[75], 6}, {NULL}};
const Segment SGI133580[] = {{&S14451[44], 12}, {NULL}};
const Segment SGI133581[] = {{&S14451[44], 24}, {NULL}};
const Segment SGI133582[] = {{&S14451[30], 6}, {NULL}};
const Segment SGI133583[] = {{&S14451[117], 3}, {NULL}};
const Segment SGI133584[] = {{&S14451[82], 6}, {NULL}};
const Segment SGI133585[] = {{&S14453[44], 7}, {NULL}};
const Segment SGI133586[] = {{&S14453[44], 19}, {NULL}};
const Segment SGI133587[] = {{&S14453[30], 6}, {NULL}};
const Segment SGI133588[] = {{&S14453[112], 3}, {NULL}};
const Segment SGI133589[] = {{&S14453[77], 6}, {NULL}};
const Segment SGI133590[] = {{&S14463[44], 9}, {NULL}};
const Segment SGI133591[] = {{&S14463[44], 21}, {NULL}};
const Segment SGI133592[] = {{&S14463[30], 6}, {NULL}};
const Segment SGI133593[] = {{&S14463[114], 3}, {NULL}};
const Segment SGI133594[] = {{&S14463[79], 6}, {NULL}};
const Segment SGI133595[] = {{&S14466[38], 5}, {NULL}};
const Segment SGI133596[] = {{&S14466[31], 6}, {NULL}};
const Segment SGI133597[] = {{&S14468[44], 7}, {NULL}};
const Segment SGI133598[] = {{&S14468[44], 19}, {NULL}};
const Segment SGI133599[] = {{&S14468[30], 6}, {NULL}};
const Segment SGI133600[] = {{&S14468[112], 3}, {NULL}};
const Segment SGI133601[] = {{&S14468[77], 6}, {NULL}};
const Segment SGI133602[] = {{&S14470[44], 8}, {NULL}};
const Segment SGI133603[] = {{&S14470[44], 20}, {NULL}};
const Segment SGI133604[] = {{&S14470[30], 6}, {NULL}};
const Segment SGI133605[] = {{&S14470[113], 3}, {NULL}};
const Segment SGI133606[] = {{&S14470[78], 6}, {NULL}};
const Segment SGI133607[] = {{&S14474[44], 12}, {NULL}};
const Segment SGI133608[] = {{&S14474[44], 24}, {NULL}};
const Segment SGI133609[] = {{&S14474[30], 6}, {NULL}};
const Segment SGI133610[] = {{&S14474[117], 3}, {NULL}};
const Segment SGI133611[] = {{&S14474[82], 6}, {NULL}};
const Segment SGI133612[] = {{&S14475[44], 22}, {NULL}};
const Segment SGI133613[] = {{&S14475[44], 34}, {NULL}};
const Segment SGI133614[] = {{&S14475[30], 6}, {NULL}};
const Segment SGI133615[] = {{&S14475[127], 3}, {NULL}};
const Segment SGI133616[] = {{&S14475[92], 6}, {NULL}};
const Segment SGI133617[] = {{&S14477[44], 9}, {NULL}};
const Segment SGI133618[] = {{&S14477[44], 21}, {NULL}};
const Segment SGI133619[] = {{&S14477[30], 6}, {NULL}};
const Segment SGI133620[] = {{&S14477[114], 3}, {NULL}};
const Segment SGI133621[] = {{&S14477[79], 6}, {NULL}};
const Segment SGI133622[] = {{&S14481[44], 14}, {NULL}};
const Segment SGI133623[] = {{&S14481[44], 26}, {NULL}};
const Segment SGI133624[] = {{&S14481[30], 6}, {NULL}};
const Segment SGI133625[] = {{&S14481[119], 3}, {NULL}};
const Segment SGI133626[] = {{&S14481[84], 6}, {NULL}};
const Segment SGI133627[] = {{&S14483[44], 10}, {NULL}};
const Segment SGI133628[] = {{&S14483[44], 22}, {NULL}};
const Segment SGI133629[] = {{&S14483[30], 6}, {NULL}};
const Segment SGI133630[] = {{&S14483[114], 3}, {NULL}};
const Segment SGI133631[] = {{&S14483[80], 5}, {NULL}};
const Segment SGI133632[] = {{&S14486[44], 7}, {NULL}};
const Segment SGI133633[] = {{&S14486[44], 19}, {NULL}};
const Segment SGI133634[] = {{&S14486[30], 6}, {NULL}};
const Segment SGI133635[] = {{&S14486[112], 3}, {NULL}};
const Segment SGI133636[] = {{&S14486[77], 6}, {NULL}};
const Segment SGI133637[] = {{&S14487[44], 14}, {NULL}};
const Segment SGI133638[] = {{&S14487[44], 26}, {NULL}};
const Segment SGI133639[] = {{&S14487[30], 6}, {NULL}};
const Segment SGI133640[] = {{&S14487[119], 3}, {NULL}};
const Segment SGI133641[] = {{&S14487[84], 6}, {NULL}};
const Segment SGI133642[] = {{&S14489[44], 27}, {NULL}};
const Segment SGI133643[] = {{&S14489[44], 39}, {NULL}};
const Segment SGI133644[] = {{&S14489[30], 6}, {NULL}};
const Segment SGI133645[] = {{&S14489[132], 3}, {NULL}};
const Segment SGI133646[] = {{&S14489[97], 6}, {NULL}};
const Segment SGI133647[] = {{&S14493[44], 7}, {NULL}};
const Segment SGI133648[] = {{&S14493[44], 19}, {NULL}};
const Segment SGI133649[] = {{&S14493[30], 6}, {NULL}};
const Segment SGI133650[] = {{&S14493[112], 3}, {NULL}};
const Segment SGI133651[] = {{&S14493[77], 6}, {NULL}};
const Segment SGI133652[] = {{&S14496[44], 13}, {NULL}};
const Segment SGI133653[] = {{&S14496[44], 25}, {NULL}};
const Segment SGI133654[] = {{&S14496[30], 6}, {NULL}};
const Segment SGI133655[] = {{&S14496[118], 3}, {NULL}};
const Segment SGI133656[] = {{&S14496[83], 6}, {NULL}};
const Segment SGI133657[] = {{&S25634[44], 10}, {NULL}};
const Segment SGI133658[] = {{&S25634[44], 22}, {NULL}};
const Segment SGI133659[] = {{&S25634[30], 6}, {NULL}};
const Segment SGI133660[] = {{&S25634[31], 3}, {NULL}};
const Segment SGI133661[] = {{&S25634[80], 6}, {NULL}};
const Segment SGI133662[] = {{&S25635[44], 11}, {NULL}};
const Segment SGI133663[] = {{&S25635[44], 23}, {NULL}};
const Segment SGI133664[] = {{&S25635[30], 6}, {NULL}};
const Segment SGI133665[] = {{&S25635[31], 3}, {NULL}};
const Segment SGI133666[] = {{&S25635[81], 6}, {NULL}};
const Segment SGI133667[] = {{&S25636[38], 10}, {NULL}};
const Segment SGI133668[] = {{&S25636[31], 6}, {NULL}};
const Segment SGI133669[] = {{&S25637[42], 10}, {NULL}};
const Segment SGI133670[] = {{&S25637[30], 6}, {NULL}};
const Segment SGI133671[] = {{&S25637[31], 3}, {NULL}};
const Segment SGI133672[] = {{&S25637[78], 6}, {NULL}};
const Segment SGI133673[] = {{&S25638[44], 10}, {NULL}};
const Segment SGI133674[] = {{&S25638[44], 22}, {NULL}};
const Segment SGI133675[] = {{&S25638[30], 6}, {NULL}};
const Segment SGI133676[] = {{&S25638[83], 6}, {NULL}};
const Segment SGI133677[] = {{&S25639[44], 11}, {NULL}};
const Segment SGI133678[] = {{&S25639[44], 23}, {NULL}};
const Segment SGI133679[] = {{&S25639[30], 6}, {NULL}};
const Segment SGI133680[] = {{&S25639[31], 3}, {NULL}};
const Segment SGI133681[] = {{&S25639[81], 6}, {NULL}};
const Segment SGI133682[] = {{&S25640[44], 11}, {NULL}};
const Segment SGI133683[] = {{&S25640[44], 23}, {NULL}};
const Segment SGI133684[] = {{&S25640[30], 6}, {NULL}};
const Segment SGI133685[] = {{&S25640[84], 6}, {NULL}};
const Segment SGI133686[] = {{&S25641[38], 11}, {NULL}};
const Segment SGI133687[] = {{&S25641[31], 6}, {NULL}};
const Segment SGI133688[] = {{&S25642[38], 11}, {NULL}};
const Segment SGI133689[] = {{&S25642[31], 6}, {NULL}};
const Segment SGI133690[] = {{&S25643[44], 10}, {NULL}};
const Segment SGI133691[] = {{&S25643[44], 27}, {NULL}};
const Segment SGI133692[] = {{&S25643[30], 6}, {NULL}};
const Segment SGI133693[] = {{&S25643[31], 3}, {NULL}};
const Segment SGI133694[] = {{&S25643[85], 6}, {NULL}};
const Segment SGI133695[] = {{&S25644[44], 10}, {NULL}};
const Segment SGI133696[] = {{&S25644[44], 22}, {NULL}};
const Segment SGI133697[] = {{&S25644[30], 6}, {NULL}};
const Segment SGI133698[] = {{&S25644[83], 4}, {NULL}};
const Segment SGI133699[] = {{&S25644[83], 3}, {NULL}};
const Segment SGI133700[] = {{&S25645[44], 10}, {NULL}};
const Segment SGI133701[] = {{&S25645[44], 22}, {NULL}};
const Segment SGI133702[] = {{&S25645[30], 6}, {NULL}};
const Segment SGI133703[] = {{&S25645[31], 3}, {NULL}};
const Segment SGI133704[] = {{&S25645[80], 6}, {NULL}};
const Segment SGI133705[] = {{&S25646[44], 10}, {NULL}};
const Segment SGI133706[] = {{&S25646[44], 34}, {NULL}};
const Segment SGI133707[] = {{&S25646[30], 6}, {NULL}};
const Segment SGI133708[] = {{&S25646[31], 3}, {NULL}};
const Segment SGI133709[] = {{&S25646[92], 6}, {NULL}};
const Segment SGI133710[] = {{&S25647[44], 10}, {NULL}};
const Segment SGI133711[] = {{&S25647[44], 33}, {NULL}};
const Segment SGI133712[] = {{&S25647[30], 6}, {NULL}};
const Segment SGI133713[] = {{&S25647[31], 3}, {NULL}};
const Segment SGI133714[] = {{&S25647[91], 6}, {NULL}};
const Segment SGI133715[] = {{&S25648[44], 10}, {NULL}};
const Segment SGI133716[] = {{&S25648[44], 34}, {NULL}};
const Segment SGI133717[] = {{&S25648[30], 6}, {NULL}};
const Segment SGI133718[] = {{&S25648[31], 3}, {NULL}};
const Segment SGI133719[] = {{&S25648[92], 5}, {NULL}};
const Segment SGI133720[] = {{&S25649[44], 10}, {NULL}};
const Segment SGI133721[] = {{&S25649[44], 27}, {NULL}};
const Segment SGI133722[] = {{&S25649[30], 6}, {NULL}};
const Segment SGI133723[] = {{&S25649[31], 3}, {NULL}};
const Segment SGI133724[] = {{&S25649[85], 6}, {NULL}};
const Segment SGI133725[] = {{&S25650[44], 10}, {NULL}};
const Segment SGI133726[] = {{&S25650[44], 19}, {NULL}};
const Segment SGI133727[] = {{&S25650[30], 6}, {NULL}};
const Segment SGI133728[] = {{&S25650[31], 3}, {NULL}};
const Segment SGI133729[] = {{&S25650[77], 6}, {NULL}};
const Segment SGI133730[] = {{&S25651[44], 10}, {NULL}};
const Segment SGI133731[] = {{&S25651[44], 19}, {NULL}};
const Segment SGI133732[] = {{&S25651[30], 6}, {NULL}};
const Segment SGI133733[] = {{&S25651[31], 3}, {NULL}};
const Segment SGI133734[] = {{&S25651[77], 6}, {NULL}};
const Segment SGI133735[] = {{&S26246[44], 11}, {NULL}};
const Segment SGI133736[] = {{&S26246[44], 33}, {NULL}};
const Segment SGI133737[] = {{&S26246[30], 6}, {NULL}};
const Segment SGI133738[] = {{&S26246[31], 3}, {NULL}};
const Segment SGI133739[] = {{&S26246[91], 6}, {NULL}};
const Segment SGI133740[] = {{&S26247[44], 11}, {NULL}};
const Segment SGI133741[] = {{&S26247[44], 41}, {NULL}};
const Segment SGI133742[] = {{&S26247[30], 6}, {NULL}};
const Segment SGI133743[] = {{&S26247[31], 3}, {NULL}};
const Segment SGI133744[] = {{&S26247[99], 6}, {NULL}};
const Segment SGI133745[] = {{&S26248[44], 11}, {NULL}};
const Segment SGI133746[] = {{&S26248[44], 37}, {NULL}};
const Segment SGI133747[] = {{&S26248[30], 6}, {NULL}};
const Segment SGI133748[] = {{&S26248[31], 3}, {NULL}};
const Segment SGI133749[] = {{&S26248[95], 6}, {NULL}};
const Segment SGI133750[] = {{&S26249[44], 7}, {NULL}};
const Segment SGI133751[] = {{&S26249[44], 29}, {NULL}};
const Segment SGI133752[] = {{&S26249[30], 6}, {NULL}};
const Segment SGI133753[] = {{&S26249[31], 3}, {NULL}};
const Segment SGI133754[] = {{&S26249[87], 6}, {NULL}};
const Segment SGI133755[] = {{&S26250[44], 11}, {NULL}};
const Segment SGI133756[] = {{&S26250[44], 33}, {NULL}};
const Segment SGI133757[] = {{&S26250[30], 6}, {NULL}};
const Segment SGI133758[] = {{&S26250[31], 3}, {NULL}};
const Segment SGI133759[] = {{&S26250[91], 6}, {NULL}};
const Segment SGI133760[] = {{&S26251[44], 11}, {NULL}};
const Segment SGI133761[] = {{&S26251[44], 33}, {NULL}};
const Segment SGI133762[] = {{&S26251[30], 6}, {NULL}};
const Segment SGI133763[] = {{&S26251[31], 3}, {NULL}};
const Segment SGI133764[] = {{&S26251[91], 6}, {NULL}};
const Segment SGI133765[] = {{&S26252[44], 7}, {NULL}};
const Segment SGI133766[] = {{&S26252[44], 29}, {NULL}};
const Segment SGI133767[] = {{&S26252[30], 6}, {NULL}};
const Segment SGI133768[] = {{&S26252[31], 3}, {NULL}};
const Segment SGI133769[] = {{&S26252[87], 6}, {NULL}};
const Segment SGI133770[] = {{&S26253[44], 7}, {NULL}};
const Segment SGI133771[] = {{&S26253[44], 37}, {NULL}};
const Segment SGI133772[] = {{&S26253[30], 6}, {NULL}};
const Segment SGI133773[] = {{&S26253[31], 3}, {NULL}};
const Segment SGI133774[] = {{&S26253[95], 6}, {NULL}};
const Segment SGI133775[] = {{&S26254[44], 11}, {NULL}};
const Segment SGI133776[] = {{&S26254[44], 37}, {NULL}};
const Segment SGI133777[] = {{&S26254[30], 6}, {NULL}};
const Segment SGI133778[] = {{&S26254[31], 3}, {NULL}};
const Segment SGI133779[] = {{&S26254[95], 6}, {NULL}};
const Segment SGI133780[] = {{&S26255[44], 11}, {NULL}};
const Segment SGI133781[] = {{&S26255[44], 41}, {NULL}};
const Segment SGI133782[] = {{&S26255[30], 6}, {NULL}};
const Segment SGI133783[] = {{&S26255[31], 3}, {NULL}};
const Segment SGI133784[] = {{&S26255[99], 6}, {NULL}};
const Segment SGI133785[] = {{&S26256[44], 7}, {NULL}};
const Segment SGI133786[] = {{&S26256[44], 33}, {NULL}};
const Segment SGI133787[] = {{&S26256[30], 6}, {NULL}};
const Segment SGI133788[] = {{&S26256[94], 6}, {NULL}};
const Segment SGI133789[] = {{&S26257[44], 11}, {NULL}};
const Segment SGI133790[] = {{&S26257[44], 37}, {NULL}};
const Segment SGI133791[] = {{&S26257[30], 6}, {NULL}};
const Segment SGI133792[] = {{&S26257[31], 3}, {NULL}};
const Segment SGI133793[] = {{&S26257[95], 6}, {NULL}};
const Segment SGI133794[] = {{&S26258[44], 7}, {NULL}};
const Segment SGI133795[] = {{&S26258[44], 29}, {NULL}};
const Segment SGI133796[] = {{&S26258[30], 6}, {NULL}};
const Segment SGI133797[] = {{&S26258[31], 3}, {NULL}};
const Segment SGI133798[] = {{&S26258[87], 6}, {NULL}};
const Segment SGI133799[] = {{&S26259[44], 11}, {NULL}};
const Segment SGI133800[] = {{&S26259[44], 33}, {NULL}};
const Segment SGI133801[] = {{&S26259[30], 6}, {NULL}};
const Segment SGI133802[] = {{&S26259[31], 3}, {NULL}};
const Segment SGI133803[] = {{&S26259[91], 6}, {NULL}};
const Segment SGI133804[] = {{&S26260[44], 11}, {NULL}};
const Segment SGI133805[] = {{&S26260[44], 38}, {NULL}};
const Segment SGI133806[] = {{&S26260[30], 6}, {NULL}};
const Segment SGI133807[] = {{&S26260[31], 3}, {NULL}};
const Segment SGI133808[] = {{&S26260[96], 6}, {NULL}};
const Segment SGI133809[] = {{&S26261[44], 7}, {NULL}};
const Segment SGI133810[] = {{&S26261[44], 34}, {NULL}};
const Segment SGI133811[] = {{&S26261[30], 6}, {NULL}};
const Segment SGI133812[] = {{&S26261[31], 3}, {NULL}};
const Segment SGI133813[] = {{&S26261[92], 6}, {NULL}};
const Segment SGI133814[] = {{&S26262[44], 12}, {NULL}};
const Segment SGI133815[] = {{&S26262[44], 32}, {NULL}};
const Segment SGI133816[] = {{&S26262[30], 6}, {NULL}};
const Segment SGI133817[] = {{&S26262[31], 3}, {NULL}};
const Segment SGI133818[] = {{&S26262[90], 6}, {NULL}};
const Segment SGI133819[] = {{&S26263[44], 11}, {NULL}};
const Segment SGI133820[] = {{&S26263[44], 38}, {NULL}};
const Segment SGI133821[] = {{&S26263[30], 6}, {NULL}};
const Segment SGI133822[] = {{&S26263[31], 3}, {NULL}};
const Segment SGI133823[] = {{&S26263[96], 6}, {NULL}};
const Segment SGI133824[] = {{&S26264[44], 7}, {NULL}};
const Segment SGI133825[] = {{&S26264[44], 33}, {NULL}};
const Segment SGI133826[] = {{&S26264[30], 6}, {NULL}};
const Segment SGI133827[] = {{&S26264[31], 3}, {NULL}};
const Segment SGI133828[] = {{&S26264[91], 6}, {NULL}};
const Segment SGI133829[] = {{&S26265[44], 11}, {NULL}};
const Segment SGI133830[] = {{&S26265[44], 33}, {NULL}};
const Segment SGI133831[] = {{&S26265[30], 6}, {NULL}};
const Segment SGI133832[] = {{&S26265[31], 3}, {NULL}};
const Segment SGI133833[] = {{&S26265[91], 6}, {NULL}};
const Segment SGI133834[] = {{&S26266[44], 7}, {NULL}};
const Segment SGI133835[] = {{&S26266[44], 37}, {NULL}};
const Segment SGI133836[] = {{&S26266[30], 6}, {NULL}};
const Segment SGI133837[] = {{&S26266[31], 3}, {NULL}};
const Segment SGI133838[] = {{&S26266[95], 6}, {NULL}};
const Segment SGI133839[] = {{&S26267[44], 12}, {NULL}};
const Segment SGI133840[] = {{&S26267[44], 32}, {NULL}};
const Segment SGI133841[] = {{&S26267[30], 6}, {NULL}};
const Segment SGI133842[] = {{&S26267[31], 3}, {NULL}};
const Segment SGI133843[] = {{&S26267[90], 6}, {NULL}};
const Segment SGI133844[] = {{&S26268[44], 7}, {NULL}};
const Segment SGI133845[] = {{&S26268[44], 29}, {NULL}};
const Segment SGI133846[] = {{&S26268[30], 6}, {NULL}};
const Segment SGI133847[] = {{&S26268[31], 3}, {NULL}};
const Segment SGI133848[] = {{&S26268[87], 6}, {NULL}};
const Segment SGI133849[] = {{&S26269[38], 7}, {NULL}};
const Segment SGI133850[] = {{&S26269[31], 6}, {NULL}};
const Segment SGI133851[] = {{&S26270[44], 11}, {NULL}};
const Segment SGI133852[] = {{&S26270[44], 33}, {NULL}};
const Segment SGI133853[] = {{&S26270[30], 6}, {NULL}};
const Segment SGI133854[] = {{&S26270[31], 3}, {NULL}};
const Segment SGI133855[] = {{&S26270[91], 6}, {NULL}};
const Segment SGI133856[] = {{&S26271[44], 7}, {NULL}};
const Segment SGI133857[] = {{&S26271[44], 34}, {NULL}};
const Segment SGI133858[] = {{&S26271[30], 6}, {NULL}};
const Segment SGI133859[] = {{&S26271[31], 3}, {NULL}};
const Segment SGI133860[] = {{&S26271[92], 6}, {NULL}};
const Segment SGI133861[] = {{&S26272[44], 7}, {NULL}};
const Segment SGI133862[] = {{&S26272[44], 29}, {NULL}};
const Segment SGI133863[] = {{&S26272[30], 6}, {NULL}};
const Segment SGI133864[] = {{&S26272[31], 3}, {NULL}};
const Segment SGI133865[] = {{&S26272[87], 6}, {NULL}};
const Segment SGI133866[] = {{&S26273[44], 11}, {NULL}};
const Segment SGI133867[] = {{&S26273[44], 50}, {NULL}};
const Segment SGI133868[] = {{&S26273[30], 6}, {NULL}};
const Segment SGI133869[] = {{&S26273[31], 3}, {NULL}};
const Segment SGI133870[] = {{&S26273[108], 6}, {NULL}};
const Segment SGI133871[] = {{&S26274[44], 11}, {NULL}};
const Segment SGI133872[] = {{&S26274[44], 39}, {NULL}};
const Segment SGI133873[] = {{&S26274[30], 6}, {NULL}};
const Segment SGI133874[] = {{&S26274[31], 3}, {NULL}};
const Segment SGI133875[] = {{&S26274[97], 6}, {NULL}};
const Segment SGI133876[] = {{&S26275[44], 11}, {NULL}};
const Segment SGI133877[] = {{&S26275[44], 37}, {NULL}};
const Segment SGI133878[] = {{&S26275[30], 6}, {NULL}};
const Segment SGI133879[] = {{&S26275[31], 3}, {NULL}};
const Segment SGI133880[] = {{&S26275[95], 6}, {NULL}};
const Segment SGI133881[] = {{&S26276[44], 11}, {NULL}};
const Segment SGI133882[] = {{&S26276[44], 37}, {NULL}};
const Segment SGI133883[] = {{&S26276[30], 6}, {NULL}};
const Segment SGI133884[] = {{&S26276[31], 3}, {NULL}};
const Segment SGI133885[] = {{&S26276[95], 6}, {NULL}};
const Segment SGI133886[] = {{&S26277[44], 11}, {NULL}};
const Segment SGI133887[] = {{&S26277[44], 36}, {NULL}};
const Segment SGI133888[] = {{&S26277[30], 6}, {NULL}};
const Segment SGI133889[] = {{&S26277[31], 3}, {NULL}};
const Segment SGI133890[] = {{&S26277[94], 6}, {NULL}};
const Segment SGI133891[] = {{&S26278[44], 11}, {NULL}};
const Segment SGI133892[] = {{&S26278[44], 37}, {NULL}};
const Segment SGI133893[] = {{&S26278[30], 6}, {NULL}};
const Segment SGI133894[] = {{&S26278[31], 3}, {NULL}};
const Segment SGI133895[] = {{&S26278[95], 6}, {NULL}};
const Segment SGI133896[] = {{&S26279[44], 11}, {NULL}};
const Segment SGI133897[] = {{&S26279[44], 37}, {NULL}};
const Segment SGI133898[] = {{&S26279[30], 6}, {NULL}};
const Segment SGI133899[] = {{&S26279[31], 3}, {NULL}};
const Segment SGI133900[] = {{&S26279[95], 6}, {NULL}};
const Segment SGI133901[] = {{&S26280[44], 11}, {NULL}};
const Segment SGI133902[] = {{&S26280[44], 33}, {NULL}};
const Segment SGI133903[] = {{&S26280[30], 6}, {NULL}};
const Segment SGI133904[] = {{&S26280[31], 3}, {NULL}};
const Segment SGI133905[] = {{&S26280[91], 6}, {NULL}};
const Segment SGI133906[] = {{&S26281[44], 7}, {NULL}};
const Segment SGI133907[] = {{&S26281[44], 33}, {NULL}};
const Segment SGI133908[] = {{&S26281[30], 6}, {NULL}};
const Segment SGI133909[] = {{&S26281[31], 3}, {NULL}};
const Segment SGI133910[] = {{&S26281[91], 6}, {NULL}};
const Segment SGI133911[] = {{&S26282[44], 7}, {NULL}};
const Segment SGI133912[] = {{&S26282[44], 29}, {NULL}};
const Segment SGI133913[] = {{&S26282[30], 6}, {NULL}};
const Segment SGI133914[] = {{&S26282[90], 6}, {NULL}};
const Segment SGI133915[] = {{&S26283[44], 7}, {NULL}};
const Segment SGI133916[] = {{&S26283[44], 35}, {NULL}};
const Segment SGI133917[] = {{&S26283[30], 6}, {NULL}};
const Segment SGI133918[] = {{&S26283[31], 3}, {NULL}};
const Segment SGI133919[] = {{&S26283[93], 6}, {NULL}};
const Segment SGI133920[] = {{&S26284[44], 11}, {NULL}};
const Segment SGI133921[] = {{&S26284[44], 39}, {NULL}};
const Segment SGI133922[] = {{&S26284[30], 6}, {NULL}};
const Segment SGI133923[] = {{&S26284[31], 3}, {NULL}};
const Segment SGI133924[] = {{&S26284[97], 6}, {NULL}};
const Segment SGI133925[] = {{&S26285[44], 7}, {NULL}};
const Segment SGI133926[] = {{&S26285[44], 19}, {NULL}};
const Segment SGI133927[] = {{&S26285[30], 6}, {NULL}};
const Segment SGI133928[] = {{&S26285[31], 3}, {NULL}};
const Segment SGI133929[] = {{&S26285[77], 6}, {NULL}};
const Segment SGI133930[] = {{&S26286[44], 11}, {NULL}};
const Segment SGI133931[] = {{&S26286[44], 48}, {NULL}};
const Segment SGI133932[] = {{&S26286[30], 6}, {NULL}};
const Segment SGI133933[] = {{&S26286[31], 3}, {NULL}};
const Segment SGI133934[] = {{&S26286[106], 6}, {NULL}};
const Segment SGI133935[] = {{&S26287[44], 12}, {NULL}};
const Segment SGI133936[] = {{&S26287[44], 32}, {NULL}};
const Segment SGI133937[] = {{&S26287[30], 6}, {NULL}};
const Segment SGI133938[] = {{&S26287[31], 3}, {NULL}};
const Segment SGI133939[] = {{&S26287[90], 6}, {NULL}};
const Segment SGI133940[] = {{&S26288[44], 7}, {NULL}};
const Segment SGI133941[] = {{&S26288[44], 35}, {NULL}};
const Segment SGI133942[] = {{&S26288[30], 6}, {NULL}};
const Segment SGI133943[] = {{&S26288[96], 6}, {NULL}};
const Segment SGI133944[] = {{&S26289[44], 7}, {NULL}};
const Segment SGI133945[] = {{&S26289[44], 29}, {NULL}};
const Segment SGI133946[] = {{&S26289[30], 6}, {NULL}};
const Segment SGI133947[] = {{&S26289[31], 3}, {NULL}};
const Segment SGI133948[] = {{&S26289[87], 6}, {NULL}};
const Segment SGI133949[] = {{&S26290[44], 7}, {NULL}};
const Segment SGI133950[] = {{&S26290[44], 32}, {NULL}};
const Segment SGI133951[] = {{&S26290[30], 6}, {NULL}};
const Segment SGI133952[] = {{&S26290[93], 6}, {NULL}};
const Segment SGI133953[] = {{&S26291[44], 11}, {NULL}};
const Segment SGI133954[] = {{&S26291[44], 39}, {NULL}};
const Segment SGI133955[] = {{&S26291[30], 6}, {NULL}};
const Segment SGI133956[] = {{&S26291[31], 3}, {NULL}};
const Segment SGI133957[] = {{&S26291[97], 6}, {NULL}};
const Segment SGI133958[] = {{&S26292[44], 12}, {NULL}};
const Segment SGI133959[] = {{&S26292[44], 32}, {NULL}};
const Segment SGI133960[] = {{&S26292[30], 6}, {NULL}};
const Segment SGI133961[] = {{&S26292[31], 3}, {NULL}};
const Segment SGI133962[] = {{&S26292[90], 6}, {NULL}};
const Segment SGI133963[] = {{&S26293[44], 7}, {NULL}};
const Segment SGI133964[] = {{&S26293[44], 33}, {NULL}};
const Segment SGI133965[] = {{&S26293[30], 6}, {NULL}};
const Segment SGI133966[] = {{&S26293[31], 3}, {NULL}};
const Segment SGI133967[] = {{&S26293[91], 6}, {NULL}};
const Segment SGI133968[] = {{&S26294[44], 7}, {NULL}};
const Segment SGI133969[] = {{&S26294[44], 33}, {NULL}};
const Segment SGI133970[] = {{&S26294[30], 6}, {NULL}};
const Segment SGI133971[] = {{&S26294[31], 3}, {NULL}};
const Segment SGI133972[] = {{&S26294[91], 6}, {NULL}};
const Segment SGI133973[] = {{&S26295[44], 7}, {NULL}};
const Segment SGI133974[] = {{&S26295[44], 34}, {NULL}};
const Segment SGI133975[] = {{&S26295[30], 6}, {NULL}};
const Segment SGI133976[] = {{&S26295[95], 6}, {NULL}};
const Segment SGI133977[] = {{&S26296[44], 11}, {NULL}};
const Segment SGI133978[] = {{&S26296[44], 33}, {NULL}};
const Segment SGI133979[] = {{&S26296[30], 6}, {NULL}};
const Segment SGI133980[] = {{&S26296[31], 3}, {NULL}};
const Segment SGI133981[] = {{&S26296[91], 6}, {NULL}};
const Segment SGI133982[] = {{&S26298[44], 7}, {NULL}};
const Segment SGI133983[] = {{&S26298[44], 46}, {NULL}};
const Segment SGI133984[] = {{&S26298[30], 6}, {NULL}};
const Segment SGI133985[] = {{&S26298[31], 3}, {NULL}};
const Segment SGI133986[] = {{&S26298[104], 6}, {NULL}};
const Segment SGI133987[] = {{&S26299[44], 8}, {NULL}};
const Segment SGI133988[] = {{&S26299[44], 28}, {NULL}};
const Segment SGI133989[] = {{&S26299[30], 6}, {NULL}};
const Segment SGI133990[] = {{&S26299[31], 3}, {NULL}};
const Segment SGI133991[] = {{&S26299[86], 6}, {NULL}};
const Segment SGI133992[] = {{&S26300[44], 7}, {NULL}};
const Segment SGI133993[] = {{&S26300[44], 33}, {NULL}};
const Segment SGI133994[] = {{&S26300[30], 6}, {NULL}};
const Segment SGI133995[] = {{&S26300[31], 3}, {NULL}};
const Segment SGI133996[] = {{&S26300[91], 6}, {NULL}};
const Segment SGI133997[] = {{&S26301[44], 7}, {NULL}};
const Segment SGI133998[] = {{&S26301[44], 33}, {NULL}};
const Segment SGI133999[] = {{&S26301[30], 6}, {NULL}};
const Segment SGI134000[] = {{&S26301[31], 3}, {NULL}};
const Segment SGI134001[] = {{&S26301[91], 6}, {NULL}};
const Segment SGI134002[] = {{&S26302[44], 7}, {NULL}};
const Segment SGI134003[] = {{&S26302[44], 32}, {NULL}};
const Segment SGI134004[] = {{&S26302[30], 6}, {NULL}};
const Segment SGI134005[] = {{&S26302[31], 3}, {NULL}};
const Segment SGI134006[] = {{&S26302[90], 6}, {NULL}};
const Segment SGI134007[] = {{&S26303[44], 7}, {NULL}};
const Segment SGI134008[] = {{&S26303[44], 44}, {NULL}};
const Segment SGI134009[] = {{&S26303[30], 6}, {NULL}};
const Segment SGI134010[] = {{&S26303[31], 3}, {NULL}};
const Segment SGI134011[] = {{&S26303[102], 6}, {NULL}};
const Segment SGI134012[] = {{&S26304[44], 7}, {NULL}};
const Segment SGI134013[] = {{&S26304[44], 29}, {NULL}};
const Segment SGI134014[] = {{&S26304[30], 6}, {NULL}};
const Segment SGI134015[] = {{&S26304[31], 3}, {NULL}};
const Segment SGI134016[] = {{&S26304[87], 6}, {NULL}};
const Segment SGI134017[] = {{&S26305[44], 11}, {NULL}};
const Segment SGI134018[] = {{&S26305[44], 32}, {NULL}};
const Segment SGI134019[] = {{&S26305[30], 6}, {NULL}};
const Segment SGI134020[] = {{&S26305[31], 3}, {NULL}};
const Segment SGI134021[] = {{&S26305[90], 6}, {NULL}};
const Segment SGI134022[] = {{&S26306[44], 7}, {NULL}};
const Segment SGI134023[] = {{&S26306[44], 35}, {NULL}};
const Segment SGI134024[] = {{&S26306[30], 6}, {NULL}};
const Segment SGI134025[] = {{&S26306[31], 3}, {NULL}};
const Segment SGI134026[] = {{&S26306[93], 6}, {NULL}};
const Segment SGI134027[] = {{&S26307[44], 7}, {NULL}};
const Segment SGI134028[] = {{&S26307[44], 35}, {NULL}};
const Segment SGI134029[] = {{&S26307[30], 6}, {NULL}};
const Segment SGI134030[] = {{&S26307[96], 6}, {NULL}};
const Segment SGI134031[] = {{&S26308[44], 11}, {NULL}};
const Segment SGI134032[] = {{&S26308[44], 41}, {NULL}};
const Segment SGI134033[] = {{&S26308[30], 6}, {NULL}};
const Segment SGI134034[] = {{&S26308[31], 3}, {NULL}};
const Segment SGI134035[] = {{&S26308[99], 6}, {NULL}};
const Segment SGI134036[] = {{&S26309[42], 11}, {NULL}};
const Segment SGI134037[] = {{&S26309[30], 6}, {NULL}};
const Segment SGI134038[] = {{&S26309[31], 3}, {NULL}};
const Segment SGI134039[] = {{&S26309[97], 6}, {NULL}};
const Segment SGI134040[] = {{&S26310[44], 7}, {NULL}};
const Segment SGI134041[] = {{&S26310[44], 35}, {NULL}};
const Segment SGI134042[] = {{&S26310[30], 6}, {NULL}};
const Segment SGI134043[] = {{&S26310[31], 3}, {NULL}};
const Segment SGI134044[] = {{&S26310[93], 6}, {NULL}};
const Segment SGI134045[] = {{&S26311[44], 11}, {NULL}};
const Segment SGI134046[] = {{&S26311[44], 48}, {NULL}};
const Segment SGI134047[] = {{&S26311[30], 6}, {NULL}};
const Segment SGI134048[] = {{&S26311[31], 3}, {NULL}};
const Segment SGI134049[] = {{&S26311[106], 6}, {NULL}};
const Segment SGI134050[] = {{&S26312[44], 11}, {NULL}};
const Segment SGI134051[] = {{&S26312[44], 33}, {NULL}};
const Segment SGI134052[] = {{&S26312[30], 6}, {NULL}};
const Segment SGI134053[] = {{&S26312[31], 3}, {NULL}};
const Segment SGI134054[] = {{&S26312[91], 6}, {NULL}};
const Segment SGI134055[] = {{&S26313[44], 11}, {NULL}};
const Segment SGI134056[] = {{&S26313[44], 48}, {NULL}};
const Segment SGI134057[] = {{&S26313[30], 6}, {NULL}};
const Segment SGI134058[] = {{&S26313[31], 3}, {NULL}};
const Segment SGI134059[] = {{&S26313[106], 6}, {NULL}};
const Segment SGI134060[] = {{&S15615[44], 11}, {NULL}};
const Segment SGI134061[] = {{&S15615[44], 23}, {NULL}};
const Segment SGI134062[] = {{&S15615[30], 6}, {NULL}};
const Segment SGI134063[] = {{&S15615[116], 3}, {NULL}};
const Segment SGI134064[] = {{&S15615[81], 6}, {NULL}};
const Segment SGI134065[] = {{&S26457[44], 8}, {NULL}};
const Segment SGI134066[] = {{&S26457[44], 25}, {NULL}};
const Segment SGI134067[] = {{&S26457[30], 6}, {NULL}};
const Segment SGI134068[] = {{&S26457[31], 3}, {NULL}};
const Segment SGI134069[] = {{&S26457[83], 6}, {NULL}};
const Segment SGI134070[] = {{&S26460[44], 7}, {NULL}};
const Segment SGI134071[] = {{&S26460[44], 24}, {NULL}};
const Segment SGI134072[] = {{&S26460[30], 6}, {NULL}};
const Segment SGI134073[] = {{&S15621[44], 7}, {NULL}};
const Segment SGI134074[] = {{&S15621[44], 19}, {NULL}};
const Segment SGI134075[] = {{&S15621[30], 6}, {NULL}};
const Segment SGI134076[] = {{&S26460[31], 3}, {NULL}};
const Segment SGI134077[] = {{&S26460[82], 6}, {NULL}};
const Segment SGI134078[] = {{&S15621[112], 3}, {NULL}};
const Segment SGI134079[] = {{&S15621[77], 6}, {NULL}};
const Segment SGI134080[] = {{&S26462[44], 7}, {NULL}};
const Segment SGI134081[] = {{&S26462[44], 23}, {NULL}};
const Segment SGI134082[] = {{&S26462[30], 6}, {NULL}};
const Segment SGI134083[] = {{&S26462[31], 3}, {NULL}};
const Segment SGI134084[] = {{&S26462[81], 6}, {NULL}};
const Segment SGI134085[] = {{&S26463[38], 8}, {NULL}};
const Segment SGI134086[] = {{&S26463[31], 6}, {NULL}};
const Segment SGI134087[] = {{&S26464[42], 8}, {NULL}};
const Segment SGI134088[] = {{&S26464[30], 6}, {NULL}};
const Segment SGI134089[] = {{&S26464[31], 3}, {NULL}};
const Segment SGI134090[] = {{&S26464[81], 6}, {NULL}};
const Segment SGI134091[] = {{&S26465[44], 7}, {NULL}};
const Segment SGI134092[] = {{&S26465[44], 23}, {NULL}};
const Segment SGI134093[] = {{&S26465[30], 6}, {NULL}};
const Segment SGI134094[] = {{&S26465[31], 3}, {NULL}};
const Segment SGI134095[] = {{&S26465[81], 6}, {NULL}};
const Segment SGI134096[] = {{&S26466[38], 7}, {NULL}};
const Segment SGI134097[] = {{&S26466[31], 6}, {NULL}};
const Segment SGI134098[] = {{&S26467[44], 7}, {NULL}};
const Segment SGI134099[] = {{&S26467[44], 23}, {NULL}};
const Segment SGI134100[] = {{&S26467[30], 6}, {NULL}};
const Segment SGI134101[] = {{&S26467[31], 3}, {NULL}};
const Segment SGI134102[] = {{&S26467[81], 6}, {NULL}};
const Segment SGI134103[] = {{&S26468[44], 7}, {NULL}};
const Segment SGI134104[] = {{&S26468[44], 22}, {NULL}};
const Segment SGI134105[] = {{&S15635[38], 11}, {NULL}};
const Segment SGI134106[] = {{&S26468[30], 6}, {NULL}};
const Segment SGI134107[] = {{&S15635[31], 6}, {NULL}};
const Segment SGI134108[] = {{&S26468[31], 3}, {NULL}};
const Segment SGI134109[] = {{&S26468[80], 6}, {NULL}};
const Segment SGI134110[] = {{&S26469[44], 7}, {NULL}};
const Segment SGI134111[] = {{&S26469[44], 24}, {NULL}};
const Segment SGI134112[] = {{&S26469[30], 6}, {NULL}};
const Segment SGI134113[] = {{&S26469[85], 6}, {NULL}};
const Segment SGI134114[] = {{&S26470[44], 7}, {NULL}};
const Segment SGI134115[] = {{&S26470[44], 23}, {NULL}};
const Segment SGI134116[] = {{&S26470[30], 6}, {NULL}};
const Segment SGI134117[] = {{&S26470[31], 3}, {NULL}};
const Segment SGI134118[] = {{&S26470[81], 6}, {NULL}};
const Segment SGI134119[] = {{&S26471[44], 7}, {NULL}};
const Segment SGI134120[] = {{&S26471[44], 23}, {NULL}};
const Segment SGI134121[] = {{&S26471[30], 6}, {NULL}};
const Segment SGI134122[] = {{&S26471[84], 6}, {NULL}};
const Segment SGI134123[] = {{&S26472[38], 7}, {NULL}};
const Segment SGI134124[] = {{&S26472[31], 6}, {NULL}};
const Segment SGI134125[] = {{&S26473[44], 7}, {NULL}};
const Segment SGI134126[] = {{&S26473[44], 22}, {NULL}};
const Segment SGI134127[] = {{&S26473[30], 6}, {NULL}};
const Segment SGI134128[] = {{&S26473[31], 3}, {NULL}};
const Segment SGI134129[] = {{&S26473[80], 6}, {NULL}};
const Segment SGI134130[] = {{&S26474[44], 7}, {NULL}};
const Segment SGI134131[] = {{&S26474[44], 22}, {NULL}};
const Segment SGI134132[] = {{&S26474[30], 6}, {NULL}};
const Segment SGI134133[] = {{&S26474[83], 6}, {NULL}};
const Segment SGI134134[] = {{&S15648[38], 7}, {NULL}};
const Segment SGI134135[] = {{&S15648[31], 6}, {NULL}};
const Segment SGI134136[] = {{&S26475[44], 7}, {NULL}};
const Segment SGI134137[] = {{&S26475[44], 22}, {NULL}};
const Segment SGI134138[] = {{&S26475[30], 6}, {NULL}};
const Segment SGI134139[] = {{&S26475[31], 3}, {NULL}};
const Segment SGI134140[] = {{&S26475[80], 6}, {NULL}};
const Segment SGI134141[] = {{&S26476[44], 8}, {NULL}};
const Segment SGI134142[] = {{&S26476[44], 25}, {NULL}};
const Segment SGI134143[] = {{&S26476[30], 6}, {NULL}};
const Segment SGI134144[] = {{&S26476[31], 3}, {NULL}};
const Segment SGI134145[] = {{&S26476[83], 6}, {NULL}};
const Segment SGI134146[] = {{&S15663[44], 20}, {NULL}};
const Segment SGI134147[] = {{&S15663[44], 32}, {NULL}};
const Segment SGI134148[] = {{&S15663[30], 6}, {NULL}};
const Segment SGI134149[] = {{&S15663[125], 3}, {NULL}};
const Segment SGI134150[] = {{&S15663[90], 6}, {NULL}};
const Segment SGI134151[] = {{&S15666[44], 20}, {NULL}};
const Segment SGI134152[] = {{&S15666[44], 32}, {NULL}};
const Segment SGI134153[] = {{&S15666[30], 6}, {NULL}};
const Segment SGI134154[] = {{&S15666[125], 3}, {NULL}};
const Segment SGI134155[] = {{&S15666[90], 6}, {NULL}};
const Segment SGI134156[] = {{&S19689[11], 1}, {NULL}};
const Segment SGI134157[] = {{&S19689[42], 13}, {NULL}};
const Segment SGI134158[] = {{&S19689[30], 4}, {NULL}};
const Segment SGI134159[] = {{&S19689[104], 3}, {NULL}};
const Segment SGI134160[] = {{&S19689[69], 6}, {NULL}};
const Segment SGI134161[] = {{&S19699[42], 6}, {NULL}};
const Segment SGI134162[] = {{&S19699[42], 18}, {NULL}};
const Segment SGI134163[] = {{&S19699[30], 4}, {NULL}};
const Segment SGI134164[] = {{&S19699[109], 3}, {NULL}};
const Segment SGI134165[] = {{&S19699[74], 6}, {NULL}};
const Segment SGI134166[] = {{&S17704[44], 10}, {NULL}};
const Segment SGI134167[] = {{&S17704[44], 22}, {NULL}};
const Segment SGI134168[] = {{&S17704[30], 6}, {NULL}};
const Segment SGI134169[] = {{&S17704[115], 3}, {NULL}};
const Segment SGI134170[] = {{&S17704[80], 6}, {NULL}};
const Segment SGI134171[] = {{&S17705[38], 10}, {NULL}};
const Segment SGI134172[] = {{&S17705[31], 6}, {NULL}};
const Segment SGI134173[] = {{&S17706[44], 10}, {NULL}};
const Segment SGI134174[] = {{&S17706[44], 22}, {NULL}};
const Segment SGI134175[] = {{&S17706[30], 6}, {NULL}};
const Segment SGI134176[] = {{&S17706[115], 3}, {NULL}};
const Segment SGI134177[] = {{&S17706[80], 6}, {NULL}};
const Segment SGI134178[] = {{&S17707[44], 30}, {NULL}};
const Segment SGI134179[] = {{&S17707[44], 42}, {NULL}};
const Segment SGI134180[] = {{&S17707[30], 6}, {NULL}};
const Segment SGI134181[] = {{&S17707[135], 3}, {NULL}};
const Segment SGI134182[] = {{&S17707[100], 6}, {NULL}};
const Segment SGI134183[] = {{&S17709[44], 30}, {NULL}};
const Segment SGI134184[] = {{&S17709[44], 42}, {NULL}};
const Segment SGI134185[] = {{&S17709[30], 6}, {NULL}};
const Segment SGI134186[] = {{&S17709[135], 3}, {NULL}};
const Segment SGI134187[] = {{&S17709[100], 6}, {NULL}};
const Segment SGI134188[] = {{&S17711[44], 30}, {NULL}};
const Segment SGI134189[] = {{&S17711[44], 42}, {NULL}};
const Segment SGI134190[] = {{&S17711[30], 6}, {NULL}};
const Segment SGI134191[] = {{&S17711[135], 3}, {NULL}};
const Segment SGI134192[] = {{&S17711[100], 6}, {NULL}};
const Segment SGI134193[] = {{&S17716[44], 29}, {NULL}};
const Segment SGI134194[] = {{&S17716[44], 41}, {NULL}};
const Segment SGI134195[] = {{&S17716[30], 6}, {NULL}};
const Segment SGI134196[] = {{&S17716[134], 3}, {NULL}};
const Segment SGI134197[] = {{&S17716[99], 6}, {NULL}};
const Segment SGI134198[] = {{&S17717[38], 10}, {NULL}};
const Segment SGI134199[] = {{&S17717[31], 6}, {NULL}};
const Segment SGI134200[] = {{&S17721[44], 30}, {NULL}};
const Segment SGI134201[] = {{&S17721[44], 42}, {NULL}};
const Segment SGI134202[] = {{&S17721[30], 6}, {NULL}};
const Segment SGI134203[] = {{&S17721[135], 3}, {NULL}};
const Segment SGI134204[] = {{&S17721[100], 6}, {NULL}};
const Segment SGI134205[] = {{&S17722[44], 30}, {NULL}};
const Segment SGI134206[] = {{&S17722[44], 42}, {NULL}};
const Segment SGI134207[] = {{&S17722[30], 6}, {NULL}};
const Segment SGI134208[] = {{&S17722[135], 3}, {NULL}};
const Segment SGI134209[] = {{&S17722[100], 6}, {NULL}};
const Segment SGI134210[] = {{&S27623[44], 15}, {NULL}};
const Segment SGI134211[] = {{&S27623[44], 27}, {NULL}};
const Segment SGI134212[] = {{&S27623[30], 6}, {NULL}};
const Segment SGI134213[] = {{&S27623[31], 3}, {NULL}};
const Segment SGI134214[] = {{&S27623[85], 6}, {NULL}};
const Segment SGI134215[] = {{&S27629[44], 11}, {NULL}};
const Segment SGI134216[] = {{&S27629[44], 23}, {NULL}};
const Segment SGI134217[] = {{&S27629[30], 6}, {NULL}};
const Segment SGI134218[] = {{&S27629[31], 3}, {NULL}};
const Segment SGI134219[] = {{&S27629[81], 6}, {NULL}};
const Segment SGI134220[] = {{&S27630[44], 11}, {NULL}};
const Segment SGI134221[] = {{&S27630[44], 23}, {NULL}};
const Segment SGI134222[] = {{&S27630[30], 6}, {NULL}};
const Segment SGI134223[] = {{&S27630[31], 3}, {NULL}};
const Segment SGI134224[] = {{&S27630[81], 6}, {NULL}};
const Segment SGI134225[] = {{&S27641[44], 11}, {NULL}};
const Segment SGI134226[] = {{&S27641[44], 23}, {NULL}};
const Segment SGI134227[] = {{&S27641[30], 6}, {NULL}};
const Segment SGI134228[] = {{&S27641[31], 3}, {NULL}};
const Segment SGI134229[] = {{&S27641[81], 6}, {NULL}};
const Segment SGI134230[] = {{&S27644[44], 15}, {NULL}};
const Segment SGI134231[] = {{&S27644[44], 27}, {NULL}};
const Segment SGI134232[] = {{&S27644[30], 6}, {NULL}};
const Segment SGI134233[] = {{&S27644[31], 3}, {NULL}};
const Segment SGI134234[] = {{&S27644[85], 6}, {NULL}};
const Segment SGI134235[] = {{&S27648[38], 11}, {NULL}};
const Segment SGI134236[] = {{&S27648[31], 6}, {NULL}};
const Segment SGI134237[] = {{&S27652[38], 15}, {NULL}};
const Segment SGI134238[] = {{&S27652[31], 6}, {NULL}};
const Segment SGI134239[] = {{&S18276[44], 10}, {NULL}};
const Segment SGI134240[] = {{&S18276[44], 22}, {NULL}};
const Segment SGI134241[] = {{&S18276[30], 6}, {NULL}};
const Segment SGI134242[] = {{&S18276[115], 3}, {NULL}};
const Segment SGI134243[] = {{&S18276[80], 6}, {NULL}};
const Segment SGI134244[] = {{&S18277[38], 10}, {NULL}};
const Segment SGI134245[] = {{&S18277[31], 6}, {NULL}};
const Segment SGI134246[] = {{&S18278[44], 30}, {NULL}};
const Segment SGI134247[] = {{&S18278[44], 42}, {NULL}};
const Segment SGI134248[] = {{&S18278[30], 6}, {NULL}};
const Segment SGI134249[] = {{&S18278[135], 3}, {NULL}};
const Segment SGI134250[] = {{&S18278[100], 6}, {NULL}};
const Segment SGI134251[] = {{&S18279[44], 30}, {NULL}};
const Segment SGI134252[] = {{&S18279[44], 42}, {NULL}};
const Segment SGI134253[] = {{&S18279[30], 6}, {NULL}};
const Segment SGI134254[] = {{&S18279[135], 3}, {NULL}};
const Segment SGI134255[] = {{&S18279[100], 6}, {NULL}};
const Segment SGI134256[] = {{&S18280[44], 30}, {NULL}};
const Segment SGI134257[] = {{&S18280[44], 42}, {NULL}};
const Segment SGI134258[] = {{&S18280[30], 6}, {NULL}};
const Segment SGI134259[] = {{&S18280[135], 3}, {NULL}};
const Segment SGI134260[] = {{&S18280[100], 6}, {NULL}};
const Segment SGI134261[] = {{&S18282[56], 10}, {NULL}};
const Segment SGI134262[] = {{&S18282[56], 22}, {NULL}};
const Segment SGI134263[] = {{&S18282[30], 18}, {NULL}};
const Segment SGI134264[] = {{&S18282[127], 3}, {NULL}};
const Segment SGI134265[] = {{&S18282[92], 6}, {NULL}};
const Segment SGI134266[] = {{&S18283[42], 10}, {NULL}};
const Segment SGI134267[] = {{&S18283[30], 6}, {NULL}};
const Segment SGI134268[] = {{&S18283[113], 3}, {NULL}};
const Segment SGI134269[] = {{&S18283[78], 6}, {NULL}};
const Segment SGI134270[] = {{&S18285[44], 30}, {NULL}};
const Segment SGI134271[] = {{&S18285[44], 42}, {NULL}};
const Segment SGI134272[] = {{&S18285[30], 6}, {NULL}};
const Segment SGI134273[] = {{&S18285[135], 3}, {NULL}};
const Segment SGI134274[] = {{&S18285[100], 6}, {NULL}};
const Segment SGI134275[] = {{&S18288[44], 10}, {NULL}};
const Segment SGI134276[] = {{&S18288[44], 22}, {NULL}};
const Segment SGI134277[] = {{&S18288[30], 6}, {NULL}};
const Segment SGI134278[] = {{&S18288[114], 3}, {NULL}};
const Segment SGI134279[] = {{&S18288[80], 5}, {NULL}};
const Segment SGI134280[] = {{&S18291[44], 10}, {NULL}};
const Segment SGI134281[] = {{&S18291[44], 22}, {NULL}};
const Segment SGI134282[] = {{&S18291[30], 6}, {NULL}};
const Segment SGI134283[] = {{&S18291[114], 3}, {NULL}};
const Segment SGI134284[] = {{&S18291[80], 5}, {NULL}};
const Segment SGI134285[] = {{&S18293[44], 10}, {NULL}};
const Segment SGI134286[] = {{&S18293[44], 22}, {NULL}};
const Segment SGI134287[] = {{&S18293[30], 6}, {NULL}};
const Segment SGI134288[] = {{&S18293[115], 3}, {NULL}};
const Segment SGI134289[] = {{&S18293[80], 6}, {NULL}};
const Segment SGI134290[] = {{&S18294[42], 10}, {NULL}};
const Segment SGI134291[] = {{&S18294[30], 6}, {NULL}};
const Segment SGI134292[] = {{&S18294[112], 3}, {NULL}};
const Segment SGI134293[] = {{&S18294[78], 5}, {NULL}};
const Segment SGI134294[] = {{&S18531[44], 5}, {NULL}};
const Segment SGI134295[] = {{&S18531[44], 26}, {NULL}};
const Segment SGI134296[] = {{&S18531[30], 6}, {NULL}};
const Segment SGI134297[] = {{&S18531[118], 3}, {NULL}};
const Segment SGI134298[] = {{&S18531[84], 5}, {NULL}};
const Segment SGI134299[] = {{&S28353[44], 101}, {NULL}};
const Segment SGI134300[] = {{&S28353[44], 113}, {NULL}};
const Segment SGI134301[] = {{&S28353[30], 6}, {NULL}};
const Segment SGI134302[] = {{&S28353[31], 3}, {NULL}};
const Segment SGI134303[] = {{&S28353[171], 6}, {NULL}};
const Segment SGI134304[] = {{&S28354[44], 19}, {NULL}};
const Segment SGI134305[] = {{&S28354[44], 31}, {NULL}};
const Segment SGI134306[] = {{&S28354[30], 6}, {NULL}};
const Segment SGI134307[] = {{&S28354[31], 3}, {NULL}};
const Segment SGI134308[] = {{&S28354[89], 6}, {NULL}};
const Segment SGI134309[] = {{&S28355[44], 18}, {NULL}};
const Segment SGI134310[] = {{&S28355[44], 30}, {NULL}};
const Segment SGI134311[] = {{&S28355[30], 6}, {NULL}};
const Segment SGI134312[] = {{&S28355[91], 6}, {NULL}};
const Segment SGI134313[] = {{&S28357[38], 18}, {NULL}};
const Segment SGI134314[] = {{&S28357[31], 6}, {NULL}};
const Segment SGI134315[] = {{&S28358[44], 18}, {NULL}};
const Segment SGI134316[] = {{&S28358[44], 30}, {NULL}};
const Segment SGI134317[] = {{&S28358[30], 6}, {NULL}};
const Segment SGI134318[] = {{&S28358[31], 3}, {NULL}};
const Segment SGI134319[] = {{&S28358[88], 6}, {NULL}};
const Segment SGI134320[] = {{&S28359[44], 23}, {NULL}};
const Segment SGI134321[] = {{&S28359[44], 35}, {NULL}};
const Segment SGI134322[] = {{&S28359[30], 6}, {NULL}};
const Segment SGI134323[] = {{&S28359[31], 3}, {NULL}};
const Segment SGI134324[] = {{&S28359[93], 6}, {NULL}};
const Segment SGI134325[] = {{&S28362[44], 18}, {NULL}};
const Segment SGI134326[] = {{&S28362[44], 30}, {NULL}};
const Segment SGI134327[] = {{&S28362[30], 6}, {NULL}};
const Segment SGI134328[] = {{&S28362[91], 4}, {NULL}};
const Segment SGI134329[] = {{&S28362[91], 3}, {NULL}};
const Segment SGI134330[] = {{&S28363[44], 10}, {NULL}};
const Segment SGI134331[] = {{&S28363[44], 22}, {NULL}};
const Segment SGI134332[] = {{&S28363[30], 6}, {NULL}};
const Segment SGI134333[] = {{&S28363[31], 3}, {NULL}};
const Segment SGI134334[] = {{&S28363[80], 6}, {NULL}};
const Segment SGI134335[] = {{&S28364[42], 19}, {NULL}};
const Segment SGI134336[] = {{&S28364[30], 6}, {NULL}};
const Segment SGI134337[] = {{&S28364[31], 3}, {NULL}};
const Segment SGI134338[] = {{&S28364[87], 6}, {NULL}};
const Segment SGI134339[] = {{&S28365[44], 30}, {NULL}};
const Segment SGI134340[] = {{&S28365[30], 6}, {NULL}};
const Segment SGI134341[] = {{&S28366[44], 18}, {NULL}};
const Segment SGI134342[] = {{&S28366[44], 30}, {NULL}};
const Segment SGI134343[] = {{&S28366[30], 6}, {NULL}};
const Segment SGI134344[] = {{&S28366[31], 3}, {NULL}};
const Segment SGI134345[] = {{&S28366[88], 6}, {NULL}};
const Segment SGI134346[] = {{&S28367[44], 18}, {NULL}};
const Segment SGI134347[] = {{&S28367[44], 30}, {NULL}};
const Segment SGI134348[] = {{&S28367[30], 6}, {NULL}};
const Segment SGI134349[] = {{&S28367[31], 3}, {NULL}};
const Segment SGI134350[] = {{&S28367[88], 6}, {NULL}};
const Segment SGI134351[] = {{&S28368[42], 18}, {NULL}};
const Segment SGI134352[] = {{&S28368[30], 6}, {NULL}};
const Segment SGI134353[] = {{&S28368[31], 3}, {NULL}};
const Segment SGI134354[] = {{&S28368[86], 6}, {NULL}};
const Segment SGI134355[] = {{&S28369[44], 23}, {NULL}};
const Segment SGI134356[] = {{&S28369[44], 41}, {NULL}};
const Segment SGI134357[] = {{&S28369[30], 6}, {NULL}};
const Segment SGI134358[] = {{&S28369[31], 3}, {NULL}};
const Segment SGI134359[] = {{&S28369[99], 6}, {NULL}};
const Segment SGI134360[] = {{&S28370[44], 23}, {NULL}};
const Segment SGI134361[] = {{&S28370[44], 35}, {NULL}};
const Segment SGI134362[] = {{&S28370[30], 6}, {NULL}};
const Segment SGI134363[] = {{&S28370[31], 3}, {NULL}};
const Segment SGI134364[] = {{&S28370[93], 6}, {NULL}};
const Segment SGI134365[] = {{&S28382[44], 10}, {NULL}};
const Segment SGI134366[] = {{&S28382[44], 22}, {NULL}};
const Segment SGI134367[] = {{&S28382[30], 6}, {NULL}};
const Segment SGI134368[] = {{&S28382[31], 3}, {NULL}};
const Segment SGI134369[] = {{&S28382[80], 6}, {NULL}};
const Segment SGI134370[] = {{&S28383[38], 10}, {NULL}};
const Segment SGI134371[] = {{&S28383[31], 6}, {NULL}};
const Segment SGI134372[] = {{&S28384[42], 10}, {NULL}};
const Segment SGI134373[] = {{&S28384[30], 6}, {NULL}};
const Segment SGI134374[] = {{&S28384[31], 3}, {NULL}};
const Segment SGI134375[] = {{&S28384[78], 6}, {NULL}};
const Segment SGI134376[] = {{&S28386[44], 10}, {NULL}};
const Segment SGI134377[] = {{&S28386[44], 22}, {NULL}};
const Segment SGI134378[] = {{&S28386[30], 6}, {NULL}};
const Segment SGI134379[] = {{&S28386[31], 3}, {NULL}};
const Segment SGI134380[] = {{&S28386[80], 6}, {NULL}};
const Segment SGI134381[] = {{&S28388[44], 10}, {NULL}};
const Segment SGI134382[] = {{&S28388[44], 34}, {NULL}};
const Segment SGI134383[] = {{&S28388[30], 6}, {NULL}};
const Segment SGI134384[] = {{&S28390[65], 10}, {NULL}};
const Segment SGI134385[] = {{&S28390[65], 22}, {NULL}};
const Segment SGI134386[] = {{&S28390[51], 6}, {NULL}};
const Segment SGI134387[] = {{&S28391[44], 10}, {NULL}};
const Segment SGI134388[] = {{&S28391[44], 34}, {NULL}};
const Segment SGI134389[] = {{&S28391[30], 6}, {NULL}};
const Segment SGI134390[] = {{&S28393[34], 10}, {NULL}};
const Segment SGI134391[] = {{&S28393[34], 22}, {NULL}};
const Segment SGI134392[] = {{&S28394[44], 10}, {NULL}};
const Segment SGI134393[] = {{&S28394[44], 34}, {NULL}};
const Segment SGI134394[] = {{&S28394[30], 6}, {NULL}};
const Segment SGI134395[] = {{&S28395[46], 9}, {NULL}};
const Segment SGI134396[] = {{&S28395[32], 6}, {NULL}};
const Segment SGI134397[] = {{&S28397[44], 10}, {NULL}};
const Segment SGI134398[] = {{&S28397[44], 22}, {NULL}};
const Segment SGI134399[] = {{&S28397[30], 6}, {NULL}};
const Segment SGI134400[] = {{&S28397[8], 3}, {&S28397[31], 3}, {NULL}};
const Segment SGI134401[] = {{&S28397[80], 6}, {NULL}};
const Segment SGI134402[] = {{&S28398[44], 10}, {NULL}};
const Segment SGI134403[] = {{&S28398[44], 22}, {NULL}};
const Segment SGI134404[] = {{&S28398[30], 6}, {NULL}};
const Segment SGI134405[] = {{&S28398[80], 6}, {NULL}};
const Segment SGI134406[] = {{&S28399[44], 10}, {NULL}};
const Segment SGI134407[] = {{&S28399[44], 22}, {NULL}};
const Segment SGI134408[] = {{&S28399[30], 6}, {NULL}};
const Segment SGI134409[] = {{&S28399[125], 3}, {NULL}};
const Segment SGI134410[] = {{&S28399[90], 6}, {NULL}};
const Segment SGI134411[] = {{&S28399[145], 8}, {NULL}};
const Segment SGI134412[] = {{&S28400[44], 10}, {NULL}};
const Segment SGI134413[] = {{&S28400[44], 22}, {NULL}};
const Segment SGI134414[] = {{&S28400[30], 6}, {NULL}};
const Segment SGI134415[] = {{&S28400[116], 3}, {NULL}};
const Segment SGI134416[] = {{&S28400[80], 7}, {NULL}};
const Segment SGI134417[] = {{&S28400[136], 8}, {NULL}};
const Segment SGI134418[] = {{&S28403[44], 10}, {NULL}};
const Segment SGI134419[] = {{&S28403[44], 22}, {&S28403[208], 7}, {NULL}};
const Segment SGI134420[] = {{&S28403[30], 6}, {NULL}};
const Segment SGI134421[] = {{&S28403[31], 3}, {NULL}};
const Segment SGI134422[] = {{&S28403[80], 6}, {NULL}};
const Segment SGI134423[] = {{&S28404[55], 18}, {NULL}};
const Segment SGI134424[] = {{&S28404[48], 6}, {NULL}};
const Segment SGI134425[] = {{&S28407[44], 10}, {NULL}};
const Segment SGI134426[] = {{&S28407[44], 19}, {NULL}};
const Segment SGI134427[] = {{&S28407[30], 6}, {NULL}};
const Segment SGI134428[] = {{&S28407[31], 3}, {NULL}};
const Segment SGI134429[] = {{&S28407[77], 5}, {NULL}};
const Segment SGI134430[] = {{&S28408[44], 10}, {NULL}};
const Segment SGI134431[] = {{&S28408[44], 32}, {NULL}};
const Segment SGI134432[] = {{&S28408[30], 6}, {NULL}};
const Segment SGI134433[] = {{&S28408[31], 3}, {NULL}};
const Segment SGI134434[] = {{&S28408[90], 6}, {NULL}};
const Segment SGI134435[] = {{&S28409[30], 1}, {NULL}};
const Segment SGI134436[] = {{&S28409[63], 9}, {NULL}};
const Segment SGI134437[] = {{&S28409[49], 6}, {NULL}};
const Segment SGI134438[] = {{&S28410[44], 10}, {NULL}};
const Segment SGI134439[] = {{&S28410[44], 22}, {NULL}};
const Segment SGI134440[] = {{&S28410[30], 6}, {NULL}};
const Segment SGI134441[] = {{&S28410[31], 3}, {NULL}};
const Segment SGI134442[] = {{&S28410[80], 6}, {NULL}};
const Segment SGI134443[] = {{&S28411[44], 10}, {NULL}};
const Segment SGI134444[] = {{&S28411[44], 22}, {NULL}};
const Segment SGI134445[] = {{&S28411[30], 6}, {NULL}};
const Segment SGI134446[] = {{&S28411[31], 3}, {&S28411[151], 3}, {NULL}};
const Segment SGI134447[] = {{&S28411[80], 6}, {NULL}};
const Segment SGI134448[] = {{&S28412[46], 10}, {NULL}};
const Segment SGI134449[] = {{&S28412[46], 34}, {NULL}};
const Segment SGI134450[] = {{&S28412[32], 6}, {NULL}};
const Segment SGI134451[] = {{&S28413[105], 29}, {NULL}};
const Segment SGI134452[] = {{&S28414[37], 6}, {NULL}};
const Segment SGI134453[] = {{&S28416[44], 10}, {NULL}};
const Segment SGI134454[] = {{&S28416[44], 22}, {NULL}};
const Segment SGI134455[] = {{&S28416[30], 6}, {NULL}};
const Segment SGI134456[] = {{&S28416[83], 6}, {NULL}};
const Segment SGI134457[] = {{&S28417[44], 10}, {NULL}};
const Segment SGI134458[] = {{&S28417[44], 22}, {NULL}};
const Segment SGI134459[] = {{&S28417[30], 6}, {NULL}};
const Segment SGI134460[] = {{&S28417[31], 3}, {NULL}};
const Segment SGI134461[] = {{&S28417[80], 5}, {NULL}};
const Segment SGI134462[] = {{&S28418[44], 10}, {NULL}};
const Segment SGI134463[] = {{&S28418[44], 41}, {NULL}};
const Segment SGI134464[] = {{&S28418[30], 6}, {NULL}};
const Segment SGI134465[] = {{&S28418[31], 3}, {NULL}};
const Segment SGI134466[] = {{&S28418[99], 6}, {NULL}};
const Segment SGI134467[] = {{&S28420[42], 10}, {NULL}};
const Segment SGI134468[] = {{&S28420[30], 6}, {NULL}};
const Segment SGI134469[] = {{&S28420[31], 3}, {NULL}};
const Segment SGI134470[] = {{&S28420[78], 6}, {NULL}};
const Segment SGI134471[] = {{&S28461[44], 11}, {NULL}};
const Segment SGI134472[] = {{&S28461[44], 23}, {NULL}};
const Segment SGI134473[] = {{&S28461[30], 6}, {NULL}};
const Segment SGI134474[] = {{&S28461[31], 3}, {NULL}};
const Segment SGI134475[] = {{&S28461[81], 6}, {NULL}};
const Segment SGI134476[] = {{&S28462[44], 30}, {NULL}};
const Segment SGI134477[] = {{&S28462[44], 42}, {NULL}};
const Segment SGI134478[] = {{&S28462[30], 6}, {NULL}};
const Segment SGI134479[] = {{&S28462[31], 3}, {NULL}};
const Segment SGI134480[] = {{&S28462[100], 6}, {NULL}};
const Segment SGI134481[] = {{&S28463[38], 11}, {NULL}};
const Segment SGI134482[] = {{&S28463[31], 6}, {NULL}};
const Segment SGI134483[] = {{&S28464[44], 31}, {NULL}};
const Segment SGI134484[] = {{&S28464[44], 43}, {NULL}};
const Segment SGI134485[] = {{&S28464[30], 6}, {NULL}};
const Segment SGI134486[] = {{&S28464[31], 3}, {NULL}};
const Segment SGI134487[] = {{&S28464[101], 6}, {NULL}};
const Segment SGI134488[] = {{&S28465[44], 30}, {NULL}};
const Segment SGI134489[] = {{&S28465[44], 42}, {NULL}};
const Segment SGI134490[] = {{&S28465[30], 6}, {NULL}};
const Segment SGI134491[] = {{&S28465[31], 3}, {NULL}};
const Segment SGI134492[] = {{&S28465[100], 6}, {NULL}};
const Segment SGI134493[] = {{&S17708[44], 10}, {NULL}};
const Segment SGI134494[] = {{&S17708[44], 22}, {NULL}};
const Segment SGI134495[] = {{&S17708[30], 6}, {NULL}};
const Segment SGI134496[] = {{&S17708[115], 3}, {NULL}};
const Segment SGI134497[] = {{&S17708[80], 6}, {NULL}};
const Segment SGI134498[] = {{&S17710[38], 10}, {NULL}};
const Segment SGI134499[] = {{&S17710[31], 6}, {NULL}};
const Segment SGI134500[] = {{&S17712[44], 30}, {NULL}};
const Segment SGI134501[] = {{&S17712[44], 42}, {NULL}};
const Segment SGI134502[] = {{&S17712[30], 6}, {NULL}};
const Segment SGI134503[] = {{&S17712[135], 3}, {NULL}};
const Segment SGI134504[] = {{&S17712[100], 6}, {NULL}};
const Segment SGI134505[] = {{&S17713[44], 10}, {NULL}};
const Segment SGI134506[] = {{&S17713[44], 22}, {NULL}};
const Segment SGI134507[] = {{&S17713[30], 6}, {NULL}};
const Segment SGI134508[] = {{&S17713[115], 3}, {NULL}};
const Segment SGI134509[] = {{&S17713[80], 6}, {NULL}};
const Segment SGI134510[] = {{&S17714[44], 10}, {NULL}};
const Segment SGI134511[] = {{&S17714[44], 22}, {NULL}};
const Segment SGI134512[] = {{&S17714[30], 6}, {NULL}};
const Segment SGI134513[] = {{&S17714[115], 3}, {NULL}};
const Segment SGI134514[] = {{&S17714[80], 6}, {NULL}};
const Segment SGI134515[] = {{&S17715[44], 30}, {NULL}};
const Segment SGI134516[] = {{&S17715[44], 42}, {NULL}};
const Segment SGI134517[] = {{&S17715[30], 6}, {NULL}};
const Segment SGI134518[] = {{&S17715[135], 3}, {NULL}};
const Segment SGI134519[] = {{&S17715[100], 6}, {NULL}};
const Segment SGI134520[] = {{&S17718[38], 10}, {NULL}};
const Segment SGI134521[] = {{&S17718[31], 6}, {NULL}};
const Segment SGI134522[] = {{&S17719[44], 29}, {NULL}};
const Segment SGI134523[] = {{&S17719[44], 41}, {NULL}};
const Segment SGI134524[] = {{&S17719[30], 6}, {NULL}};
const Segment SGI134525[] = {{&S17719[134], 3}, {NULL}};
const Segment SGI134526[] = {{&S17719[99], 6}, {NULL}};
const Segment SGI134527[] = {{&S17720[44], 30}, {NULL}};
const Segment SGI134528[] = {{&S17720[44], 42}, {NULL}};
const Segment SGI134529[] = {{&S17720[30], 6}, {NULL}};
const Segment SGI134530[] = {{&S17720[135], 3}, {NULL}};
const Segment SGI134531[] = {{&S17720[100], 6}, {NULL}};
const Segment SGI134532[] = {{&S17723[44], 30}, {NULL}};
const Segment SGI134533[] = {{&S17723[44], 42}, {NULL}};
const Segment SGI134534[] = {{&S17723[30], 6}, {NULL}};
const Segment SGI134535[] = {{&S17723[135], 3}, {NULL}};
const Segment SGI134536[] = {{&S17723[100], 6}, {NULL}};
const Segment SGI134537[] = {{&S17724[38], 10}, {NULL}};
const Segment SGI134538[] = {{&S17724[31], 6}, {NULL}};
const Segment SGI134539[] = {{&S17725[44], 30}, {NULL}};
const Segment SGI134540[] = {{&S17725[44], 42}, {NULL}};
const Segment SGI134541[] = {{&S17725[30], 6}, {NULL}};
const Segment SGI134542[] = {{&S17725[135], 3}, {NULL}};
const Segment SGI134543[] = {{&S17725[100], 6}, {NULL}};
const Segment SGI134544[] = {{&S17726[44], 10}, {NULL}};
const Segment SGI134545[] = {{&S17726[44], 22}, {NULL}};
const Segment SGI134546[] = {{&S17726[30], 6}, {NULL}};
const Segment SGI134547[] = {{&S17726[115], 3}, {NULL}};
const Segment SGI134548[] = {{&S17726[80], 6}, {NULL}};
const Segment SGI134549[] = {{&S17727[42], 10}, {NULL}};
const Segment SGI134550[] = {{&S17727[30], 6}, {NULL}};
const Segment SGI134551[] = {{&S17727[113], 3}, {NULL}};
const Segment SGI134552[] = {{&S17727[78], 6}, {NULL}};
const Segment SGI134553[] = {{&S17728[44], 30}, {NULL}};
const Segment SGI134554[] = {{&S17728[44], 42}, {NULL}};
const Segment SGI134555[] = {{&S17728[30], 6}, {NULL}};
const Segment SGI134556[] = {{&S17728[135], 3}, {NULL}};
const Segment SGI134557[] = {{&S17728[100], 6}, {NULL}};
const Segment SGI134558[] = {{&S28689[44], 10}, {NULL}};
const Segment SGI134559[] = {{&S28689[44], 25}, {NULL}};
const Segment SGI134560[] = {{&S28689[30], 6}, {NULL}};
const Segment SGI134561[] = {{&S28689[31], 3}, {NULL}};
const Segment SGI134562[] = {{&S28689[83], 6}, {NULL}};
const Segment SGI134563[] = {{&S28690[44], 10}, {NULL}};
const Segment SGI134564[] = {{&S28690[44], 25}, {NULL}};
const Segment SGI134565[] = {{&S28690[30], 6}, {NULL}};
const Segment SGI134566[] = {{&S28690[31], 3}, {NULL}};
const Segment SGI134567[] = {{&S28690[83], 6}, {NULL}};
const Segment SGI134568[] = {{&S28691[44], 11}, {NULL}};
const Segment SGI134569[] = {{&S28691[44], 27}, {NULL}};
const Segment SGI134570[] = {{&S28691[30], 6}, {NULL}};
const Segment SGI134571[] = {{&S28691[31], 3}, {NULL}};
const Segment SGI134572[] = {{&S28691[85], 6}, {NULL}};
const Segment SGI134573[] = {{&S28692[44], 11}, {NULL}};
const Segment SGI134574[] = {{&S28692[44], 27}, {NULL}};
const Segment SGI134575[] = {{&S28692[30], 6}, {NULL}};
const Segment SGI134576[] = {{&S28692[31], 3}, {NULL}};
const Segment SGI134577[] = {{&S28692[85], 6}, {NULL}};
const Segment SGI134578[] = {{&S28693[44], 11}, {NULL}};
const Segment SGI134579[] = {{&S28693[44], 28}, {NULL}};
const Segment SGI134580[] = {{&S28693[30], 6}, {NULL}};
const Segment SGI134581[] = {{&S28693[31], 3}, {NULL}};
const Segment SGI134582[] = {{&S28693[86], 6}, {NULL}};
const Segment SGI134583[] = {{&S28694[44], 11}, {NULL}};
const Segment SGI134584[] = {{&S28694[44], 31}, {NULL}};
const Segment SGI134585[] = {{&S28694[30], 6}, {NULL}};
const Segment SGI134586[] = {{&S28694[31], 3}, {NULL}};
const Segment SGI134587[] = {{&S28694[89], 6}, {NULL}};
const Segment SGI134588[] = {{&S28695[44], 11}, {NULL}};
const Segment SGI134589[] = {{&S28695[44], 29}, {NULL}};
const Segment SGI134590[] = {{&S28695[30], 6}, {NULL}};
const Segment SGI134591[] = {{&S28695[31], 3}, {NULL}};
const Segment SGI134592[] = {{&S28695[87], 6}, {NULL}};
const Segment SGI134593[] = {{&S28696[44], 11}, {NULL}};
const Segment SGI134594[] = {{&S28696[44], 28}, {NULL}};
const Segment SGI134595[] = {{&S28696[30], 6}, {NULL}};
const Segment SGI134596[] = {{&S28696[31], 3}, {NULL}};
const Segment SGI134597[] = {{&S28696[86], 6}, {NULL}};
const Segment SGI134598[] = {{&S28697[44], 7}, {NULL}};
const Segment SGI134599[] = {{&S28697[44], 24}, {NULL}};
const Segment SGI134600[] = {{&S28697[30], 6}, {NULL}};
const Segment SGI134601[] = {{&S28697[31], 3}, {NULL}};
const Segment SGI134602[] = {{&S28697[82], 6}, {NULL}};
const Segment SGI134603[] = {{&S28698[44], 7}, {NULL}};
const Segment SGI134604[] = {{&S28698[44], 24}, {NULL}};
const Segment SGI134605[] = {{&S28698[30], 6}, {NULL}};
const Segment SGI134606[] = {{&S28698[31], 3}, {NULL}};
const Segment SGI134607[] = {{&S28698[82], 6}, {NULL}};
const Segment SGI134608[] = {{&S28699[44], 11}, {NULL}};
const Segment SGI134609[] = {{&S28699[44], 28}, {NULL}};
const Segment SGI134610[] = {{&S28699[30], 6}, {NULL}};
const Segment SGI134611[] = {{&S28699[31], 3}, {NULL}};
const Segment SGI134612[] = {{&S28699[86], 6}, {NULL}};
const Segment SGI134613[] = {{&S28700[38], 7}, {NULL}};
const Segment SGI134614[] = {{&S28700[31], 6}, {NULL}};
const Segment SGI134615[] = {{&S28701[44], 11}, {NULL}};
const Segment SGI134616[] = {{&S28701[44], 27}, {NULL}};
const Segment SGI134617[] = {{&S28701[30], 6}, {NULL}};
const Segment SGI134618[] = {{&S28701[31], 3}, {NULL}};
const Segment SGI134619[] = {{&S28701[85], 6}, {NULL}};
const Segment SGI134620[] = {{&S28702[44], 11}, {NULL}};
const Segment SGI134621[] = {{&S28702[44], 28}, {NULL}};
const Segment SGI134622[] = {{&S28702[30], 6}, {NULL}};
const Segment SGI134623[] = {{&S28702[31], 3}, {NULL}};
const Segment SGI134624[] = {{&S28702[86], 6}, {NULL}};
const Segment SGI134625[] = {{&S28703[44], 11}, {NULL}};
const Segment SGI134626[] = {{&S28703[44], 28}, {NULL}};
const Segment SGI134627[] = {{&S28703[30], 6}, {NULL}};
const Segment SGI134628[] = {{&S28703[31], 3}, {NULL}};
const Segment SGI134629[] = {{&S28703[86], 6}, {NULL}};
const Segment SGI134630[] = {{&S28704[44], 12}, {NULL}};
const Segment SGI134631[] = {{&S28704[44], 29}, {NULL}};
const Segment SGI134632[] = {{&S28704[30], 6}, {NULL}};
const Segment SGI134633[] = {{&S28704[31], 3}, {NULL}};
const Segment SGI134634[] = {{&S28704[87], 6}, {NULL}};
const Segment SGI134635[] = {{&S28705[44], 7}, {NULL}};
const Segment SGI134636[] = {{&S28705[44], 24}, {NULL}};
const Segment SGI134637[] = {{&S28705[30], 6}, {NULL}};
const Segment SGI134638[] = {{&S28705[31], 3}, {NULL}};
const Segment SGI134639[] = {{&S28705[82], 6}, {NULL}};
const Segment SGI134640[] = {{&S28706[44], 7}, {NULL}};
const Segment SGI134641[] = {{&S28706[44], 23}, {NULL}};
const Segment SGI134642[] = {{&S28706[30], 6}, {NULL}};
const Segment SGI134643[] = {{&S28706[31], 3}, {NULL}};
const Segment SGI134644[] = {{&S28706[81], 6}, {NULL}};
const Segment SGI134645[] = {{&S28707[44], 11}, {NULL}};
const Segment SGI134646[] = {{&S28707[44], 27}, {NULL}};
const Segment SGI134647[] = {{&S28707[30], 6}, {NULL}};
const Segment SGI134648[] = {{&S28707[31], 3}, {NULL}};
const Segment SGI134649[] = {{&S28707[85], 6}, {NULL}};
const Segment SGI134650[] = {{&S28708[44], 11}, {NULL}};
const Segment SGI134651[] = {{&S28708[44], 27}, {NULL}};
const Segment SGI134652[] = {{&S28708[30], 6}, {NULL}};
const Segment SGI134653[] = {{&S28708[31], 3}, {NULL}};
const Segment SGI134654[] = {{&S28708[85], 6}, {NULL}};
const Segment SGI134655[] = {{&S28709[44], 7}, {NULL}};
const Segment SGI134656[] = {{&S28709[44], 23}, {NULL}};
const Segment SGI134657[] = {{&S28709[30], 6}, {NULL}};
const Segment SGI134658[] = {{&S28709[31], 3}, {NULL}};
const Segment SGI134659[] = {{&S28709[81], 6}, {NULL}};
const Segment SGI134660[] = {{&S28710[44], 7}, {NULL}};
const Segment SGI134661[] = {{&S28710[44], 24}, {NULL}};
const Segment SGI134662[] = {{&S28710[30], 6}, {NULL}};
const Segment SGI134663[] = {{&S28710[31], 3}, {NULL}};
const Segment SGI134664[] = {{&S28710[82], 6}, {NULL}};
const Segment SGI134665[] = {{&S28711[44], 12}, {NULL}};
const Segment SGI134666[] = {{&S28711[44], 28}, {NULL}};
const Segment SGI134667[] = {{&S28711[30], 6}, {NULL}};
const Segment SGI134668[] = {{&S28711[31], 3}, {NULL}};
const Segment SGI134669[] = {{&S28711[86], 6}, {NULL}};
const Segment SGI134670[] = {{&S28712[44], 7}, {NULL}};
const Segment SGI134671[] = {{&S28712[44], 24}, {NULL}};
const Segment SGI134672[] = {{&S28712[30], 6}, {NULL}};
const Segment SGI134673[] = {{&S28712[31], 3}, {NULL}};
const Segment SGI134674[] = {{&S28712[82], 6}, {NULL}};
const Segment SGI134675[] = {{&S28713[44], 7}, {NULL}};
const Segment SGI134676[] = {{&S28713[44], 23}, {NULL}};
const Segment SGI134677[] = {{&S28713[30], 6}, {NULL}};
const Segment SGI134678[] = {{&S28713[31], 3}, {NULL}};
const Segment SGI134679[] = {{&S28713[81], 6}, {NULL}};
const Segment SGI134680[] = {{&S28714[38], 7}, {NULL}};
const Segment SGI134681[] = {{&S28714[31], 6}, {NULL}};
const Segment SGI134682[] = {{&S28715[44], 11}, {NULL}};
const Segment SGI134683[] = {{&S28715[44], 27}, {NULL}};
const Segment SGI134684[] = {{&S28715[30], 6}, {NULL}};
const Segment SGI134685[] = {{&S28715[31], 3}, {NULL}};
const Segment SGI134686[] = {{&S28715[85], 6}, {NULL}};
const Segment SGI134687[] = {{&S21462[42], 14}, {NULL}};
const Segment SGI134688[] = {{&S21462[42], 26}, {NULL}};
const Segment SGI134689[] = {{&S21462[30], 4}, {NULL}};
const Segment SGI134690[] = {{&S21462[117], 3}, {NULL}};
const Segment SGI134691[] = {{&S21462[82], 6}, {NULL}};
const Segment SGI134692[] = {{&S21463[42], 14}, {NULL}};
const Segment SGI134693[] = {{&S21463[42], 26}, {NULL}};
const Segment SGI134694[] = {{&S21463[30], 4}, {NULL}};
const Segment SGI134695[] = {{&S21463[82], 5}, {NULL}};
const Segment SGI134696[] = {{&S21464[36], 14}, {NULL}};
const Segment SGI134697[] = {{&S21464[31], 4}, {NULL}};
const Segment SGI134698[] = {{&S21472[42], 14}, {NULL}};
const Segment SGI134699[] = {{&S21472[42], 26}, {NULL}};
const Segment SGI134700[] = {{&S21472[30], 4}, {NULL}};
const Segment SGI134701[] = {{&S21472[117], 3}, {NULL}};
const Segment SGI134702[] = {{&S21472[82], 6}, {NULL}};
const Segment SGI134703[] = {{&S21495[42], 14}, {NULL}};
const Segment SGI134704[] = {{&S21495[42], 33}, {NULL}};
const Segment SGI134705[] = {{&S21495[30], 4}, {NULL}};
const Segment SGI134706[] = {{&S21495[124], 3}, {NULL}};
const Segment SGI134707[] = {{&S21495[89], 6}, {NULL}};
const Segment SGI134708[] = {{&S28855[44], 10}, {NULL}};
const Segment SGI134709[] = {{&S28855[44], 22}, {NULL}};
const Segment SGI134710[] = {{&S28855[30], 6}, {NULL}};
const Segment SGI134711[] = {{&S28855[31], 3}, {NULL}};
const Segment SGI134712[] = {{&S28855[80], 6}, {NULL}};
const Segment SGI134713[] = {{&S28856[44], 10}, {NULL}};
const Segment SGI134714[] = {{&S28856[44], 22}, {NULL}};
const Segment SGI134715[] = {{&S28856[30], 6}, {NULL}};
const Segment SGI134716[] = {{&S28856[31], 3}, {NULL}};
const Segment SGI134717[] = {{&S28856[80], 6}, {NULL}};
const Segment SGI134718[] = {{&S28857[38], 10}, {NULL}};
const Segment SGI134719[] = {{&S28857[31], 6}, {NULL}};
const Segment SGI134720[] = {{&S28858[42], 10}, {NULL}};
const Segment SGI134721[] = {{&S28858[30], 6}, {NULL}};
const Segment SGI134722[] = {{&S28858[31], 3}, {NULL}};
const Segment SGI134723[] = {{&S28858[78], 6}, {NULL}};
const Segment SGI134724[] = {{&S28862[44], 10}, {NULL}};
const Segment SGI134725[] = {{&S28862[44], 75}, {NULL}};
const Segment SGI134726[] = {{&S28862[30], 6}, {NULL}};
const Segment SGI134727[] = {{&S28863[44], 10}, {NULL}};
const Segment SGI134728[] = {{&S28863[44], 80}, {NULL}};
const Segment SGI134729[] = {{&S28863[30], 6}, {NULL}};
const Segment SGI134730[] = {{&S28865[44], 10}, {NULL}};
const Segment SGI134731[] = {{&S28865[44], 22}, {NULL}};
const Segment SGI134732[] = {{&S28865[30], 6}, {NULL}};
const Segment SGI134733[] = {{&S28865[31], 3}, {NULL}};
const Segment SGI134734[] = {{&S28865[80], 6}, {NULL}};
const Segment SGI134735[] = {{&S28866[44], 10}, {NULL}};
const Segment SGI134736[] = {{&S28866[44], 22}, {NULL}};
const Segment SGI134737[] = {{&S28866[30], 6}, {NULL}};
const Segment SGI134738[] = {{&S28866[31], 3}, {NULL}};
const Segment SGI134739[] = {{&S28866[80], 6}, {NULL}};
const Segment SGI134740[] = {{&S28869[65], 10}, {NULL}};
const Segment SGI134741[] = {{&S28869[65], 22}, {NULL}};
const Segment SGI134742[] = {{&S28869[51], 6}, {NULL}};
const Segment SGI134743[] = {{&S28870[34], 10}, {NULL}};
const Segment SGI134744[] = {{&S28870[34], 22}, {NULL}};
const Segment SGI134745[] = {{&S28871[44], 10}, {NULL}};
const Segment SGI134746[] = {{&S28871[44], 22}, {NULL}};
const Segment SGI134747[] = {{&S28871[30], 6}, {NULL}};
const Segment SGI134748[] = {{&S28871[80], 6}, {NULL}};
const Segment SGI134749[] = {{&S28872[44], 10}, {NULL}};
const Segment SGI134750[] = {{&S28872[44], 22}, {NULL}};
const Segment SGI134751[] = {{&S28872[30], 6}, {NULL}};
const Segment SGI134752[] = {{&S28872[8], 3}, {&S28872[31], 3}, {NULL}};
const Segment SGI134753[] = {{&S28872[80], 6}, {NULL}};
const Segment SGI134754[] = {{&S28873[44], 10}, {NULL}};
const Segment SGI134755[] = {{&S28873[44], 22}, {NULL}};
const Segment SGI134756[] = {{&S28873[30], 6}, {NULL}};
const Segment SGI134757[] = {{&S28873[83], 6}, {NULL}};
const Segment SGI134758[] = {{&S28874[44], 10}, {NULL}};
const Segment SGI134759[] = {{&S28874[44], 34}, {NULL}};
const Segment SGI134760[] = {{&S28874[30], 6}, {NULL}};
const Segment SGI134761[] = {{&S28876[44], 10}, {NULL}};
const Segment SGI134762[] = {{&S28876[44], 66}, {NULL}};
const Segment SGI134763[] = {{&S28876[30], 6}, {NULL}};
const Segment SGI134764[] = {{&S28877[103], 29}, {NULL}};
const Segment SGI134765[] = {{&S28878[44], 10}, {NULL}};
const Segment SGI134766[] = {{&S28878[44], 22}, {NULL}};
const Segment SGI134767[] = {{&S28878[30], 6}, {NULL}};
const Segment SGI134768[] = {{&S28878[80], 6}, {NULL}};
const Segment SGI134769[] = {{&S28881[30], 1}, {NULL}};
const Segment SGI134770[] = {{&S28881[63], 9}, {NULL}};
const Segment SGI134771[] = {{&S28881[49], 6}, {NULL}};
const Segment SGI134772[] = {{&S28883[149], 18}, {NULL}};
const Segment SGI134773[] = {{&S28884[44], 10}, {NULL}};
const Segment SGI134774[] = {{&S28884[44], 22}, {NULL}};
const Segment SGI134775[] = {{&S28884[30], 6}, {NULL}};
const Segment SGI134776[] = {{&S28884[31], 3}, {NULL}};
const Segment SGI134777[] = {{&S28884[80], 5}, {NULL}};
const Segment SGI134778[] = {{&S28885[53], 18}, {NULL}};
const Segment SGI134779[] = {{&S28885[46], 6}, {NULL}};
const Segment SGI134780[] = {{&S28886[44], 10}, {NULL}};
const Segment SGI134781[] = {{&S28886[44], 22}, {NULL}};
const Segment SGI134782[] = {{&S28886[30], 6}, {NULL}};
const Segment SGI134783[] = {{&S28886[116], 3}, {NULL}};
const Segment SGI134784[] = {{&S28886[80], 7}, {NULL}};
const Segment SGI134785[] = {{&S28886[136], 8}, {NULL}};
const Segment SGI134786[] = {{&S28887[44], 10}, {NULL}};
const Segment SGI134787[] = {{&S28887[44], 22}, {NULL}};
const Segment SGI134788[] = {{&S28887[30], 6}, {NULL}};
const Segment SGI134789[] = {{&S28887[31], 3}, {NULL}};
const Segment SGI134790[] = {{&S28887[80], 6}, {NULL}};
const Segment SGI134791[] = {{&S28888[46], 10}, {NULL}};
const Segment SGI134792[] = {{&S28888[46], 73}, {NULL}};
const Segment SGI134793[] = {{&S28888[32], 6}, {NULL}};
const Segment SGI134794[] = {{&S28889[46], 9}, {NULL}};
const Segment SGI134795[] = {{&S28889[32], 6}, {NULL}};
const Segment SGI134796[] = {{&S28891[46], 10}, {NULL}};
const Segment SGI134797[] = {{&S28891[46], 34}, {NULL}};
const Segment SGI134798[] = {{&S28891[32], 6}, {NULL}};
const Segment SGI134799[] = {{&S28892[37], 6}, {NULL}};
const Segment SGI134800[] = {{&S28896[59], 10}, {NULL}};
const Segment SGI134801[] = {{&S28896[59], 34}, {NULL}};
const Segment SGI134802[] = {{&S28896[45], 6}, {NULL}};
const Segment SGI134803[] = {{&S28898[44], 10}, {NULL}};
const Segment SGI134804[] = {{&S28898[44], 22}, {NULL}};
const Segment SGI134805[] = {{&S28898[30], 6}, {NULL}};
const Segment SGI134806[] = {{&S28898[31], 3}, {&S28898[151], 3}, {NULL}};
const Segment SGI134807[] = {{&S28898[80], 6}, {NULL}};
const Segment SGI134808[] = {{&S28899[44], 10}, {NULL}};
const Segment SGI134809[] = {{&S28899[44], 22}, {NULL}};
const Segment SGI134810[] = {{&S28899[30], 6}, {NULL}};
const Segment SGI134811[] = {{&S28899[31], 3}, {NULL}};
const Segment SGI134812[] = {{&S28899[80], 6}, {NULL}};
const Segment SGI134813[] = {{&S28900[44], 22}, {NULL}};
const Segment SGI134814[] = {{&S28900[30], 6}, {NULL}};
const Segment SGI134815[] = {{&S28900[115], 3}, {NULL}};
const Segment SGI134816[] = {{&S28900[80], 6}, {NULL}};
const Segment SGI134817[] = {{&S28900[135], 8}, {NULL}};
const Segment SGI134818[] = {{&S28901[44], 10}, {NULL}};
const Segment SGI134819[] = {{&S28901[44], 22}, {NULL}};
const Segment SGI134820[] = {{&S28901[30], 6}, {NULL}};
const Segment SGI134821[] = {{&S28901[31], 3}, {NULL}};
const Segment SGI134822[] = {{&S28901[80], 6}, {NULL}};
const Segment SGI134823[] = {{&S28904[44], 9}, {NULL}};
const Segment SGI134824[] = {{&S28904[30], 6}, {NULL}};
const Segment SGI134825[] = {{&S28904[31], 3}, {NULL}};
const Segment SGI134826[] = {{&S28904[67], 5}, {NULL}};
const Segment SGI134827[] = {{&S28936[44], 6}, {NULL}};
const Segment SGI134828[] = {{&S28936[44], 18}, {NULL}};
const Segment SGI134829[] = {{&S28936[30], 6}, {NULL}};
const Segment SGI134830[] = {{&S28936[31], 3}, {NULL}};
const Segment SGI134831[] = {{&S28936[76], 6}, {NULL}};
const Segment SGI134832[] = {{&S28937[44], 6}, {NULL}};
const Segment SGI134833[] = {{&S28937[44], 17}, {NULL}};
const Segment SGI134834[] = {{&S28937[30], 6}, {NULL}};
const Segment SGI134835[] = {{&S28937[31], 3}, {NULL}};
const Segment SGI134836[] = {{&S28937[75], 6}, {NULL}};
const Segment SGI134837[] = {{&S28938[44], 6}, {NULL}};
const Segment SGI134838[] = {{&S28938[44], 18}, {NULL}};
const Segment SGI134839[] = {{&S28938[30], 6}, {NULL}};
const Segment SGI134840[] = {{&S28938[31], 3}, {NULL}};
const Segment SGI134841[] = {{&S28938[76], 6}, {NULL}};
const Segment SGI134842[] = {{&S28940[44], 6}, {NULL}};
const Segment SGI134843[] = {{&S28940[44], 18}, {NULL}};
const Segment SGI134844[] = {{&S28940[30], 6}, {NULL}};
const Segment SGI134845[] = {{&S28940[31], 3}, {NULL}};
const Segment SGI134846[] = {{&S28940[76], 6}, {NULL}};
const Segment SGI134847[] = {{&S28941[44], 6}, {NULL}};
const Segment SGI134848[] = {{&S28941[44], 27}, {NULL}};
const Segment SGI134849[] = {{&S28941[30], 6}, {NULL}};
const Segment SGI134850[] = {{&S28941[31], 3}, {NULL}};
const Segment SGI134851[] = {{&S28941[85], 6}, {NULL}};
const Segment SGI134852[] = {{&S28942[38], 6}, {NULL}};
const Segment SGI134853[] = {{&S28942[31], 6}, {NULL}};
const Segment SGI134854[] = {{&S18281[44], 10}, {NULL}};
const Segment SGI134855[] = {{&S18281[44], 22}, {NULL}};
const Segment SGI134856[] = {{&S18281[30], 6}, {NULL}};
const Segment SGI134857[] = {{&S18281[115], 3}, {NULL}};
const Segment SGI134858[] = {{&S18281[80], 6}, {NULL}};
const Segment SGI134859[] = {{&S18284[38], 10}, {NULL}};
const Segment SGI134860[] = {{&S18284[31], 6}, {NULL}};
const Segment SGI134861[] = {{&S18286[44], 30}, {NULL}};
const Segment SGI134862[] = {{&S18286[44], 42}, {NULL}};
const Segment SGI134863[] = {{&S18286[30], 6}, {NULL}};
const Segment SGI134864[] = {{&S18286[135], 3}, {NULL}};
const Segment SGI134865[] = {{&S18286[100], 6}, {NULL}};
const Segment SGI134866[] = {{&S18287[42], 10}, {NULL}};
const Segment SGI134867[] = {{&S18287[30], 6}, {NULL}};
const Segment SGI134868[] = {{&S18287[113], 3}, {NULL}};
const Segment SGI134869[] = {{&S18287[78], 6}, {NULL}};
const Segment SGI134870[] = {{&S18289[44], 10}, {NULL}};
const Segment SGI134871[] = {{&S18289[44], 22}, {NULL}};
const Segment SGI134872[] = {{&S18289[30], 6}, {NULL}};
const Segment SGI134873[] = {{&S18289[114], 3}, {NULL}};
const Segment SGI134874[] = {{&S18289[80], 5}, {NULL}};
const Segment SGI134875[] = {{&S18290[44], 30}, {NULL}};
const Segment SGI134876[] = {{&S18290[44], 42}, {NULL}};
const Segment SGI134877[] = {{&S18290[30], 6}, {NULL}};
const Segment SGI134878[] = {{&S18290[135], 3}, {NULL}};
const Segment SGI134879[] = {{&S18290[100], 6}, {NULL}};
const Segment SGI134880[] = {{&S18292[44], 30}, {NULL}};
const Segment SGI134881[] = {{&S18292[44], 42}, {NULL}};
const Segment SGI134882[] = {{&S18292[30], 6}, {NULL}};
const Segment SGI134883[] = {{&S18292[135], 3}, {NULL}};
const Segment SGI134884[] = {{&S18292[100], 6}, {NULL}};
const Segment SGI134885[] = {{&S18295[44], 30}, {NULL}};
const Segment SGI134886[] = {{&S18295[44], 42}, {NULL}};
const Segment SGI134887[] = {{&S18295[30], 6}, {NULL}};
const Segment SGI134888[] = {{&S18295[135], 3}, {NULL}};
const Segment SGI134889[] = {{&S18295[100], 6}, {NULL}};
const Segment SGI134890[] = {{&S18296[44], 7}, {NULL}};
const Segment SGI134891[] = {{&S18296[44], 19}, {NULL}};
const Segment SGI134892[] = {{&S18296[30], 6}, {NULL}};
const Segment SGI134893[] = {{&S18296[111], 3}, {NULL}};
const Segment SGI134894[] = {{&S18296[77], 5}, {NULL}};
const Segment SGI134895[] = {{&S18297[44], 30}, {NULL}};
const Segment SGI134896[] = {{&S18297[44], 42}, {NULL}};
const Segment SGI134897[] = {{&S18297[30], 6}, {NULL}};
const Segment SGI134898[] = {{&S18297[135], 3}, {NULL}};
const Segment SGI134899[] = {{&S18297[100], 6}, {NULL}};
const Segment SGI134900[] = {{&S18497[44], 11}, {NULL}};
const Segment SGI134901[] = {{&S18497[44], 23}, {NULL}};
const Segment SGI134902[] = {{&S18497[30], 6}, {NULL}};
const Segment SGI134903[] = {{&S18497[116], 3}, {NULL}};
const Segment SGI134904[] = {{&S18497[81], 6}, {NULL}};
const Segment SGI134905[] = {{&S18498[38], 11}, {NULL}};
const Segment SGI134906[] = {{&S18498[31], 6}, {NULL}};
const Segment SGI134907[] = {{&S18499[38], 9}, {NULL}};
const Segment SGI134908[] = {{&S18499[31], 6}, {NULL}};
const Segment SGI134909[] = {{&S18500[44], 9}, {NULL}};
const Segment SGI134910[] = {{&S18500[44], 21}, {NULL}};
const Segment SGI134911[] = {{&S18500[30], 6}, {NULL}};
const Segment SGI134912[] = {{&S18500[114], 3}, {NULL}};
const Segment SGI134913[] = {{&S18500[79], 6}, {NULL}};
const Segment SGI134914[] = {{&S18501[44], 8}, {NULL}};
const Segment SGI134915[] = {{&S18501[44], 20}, {NULL}};
const Segment SGI134916[] = {{&S18501[30], 6}, {NULL}};
const Segment SGI134917[] = {{&S18501[113], 3}, {NULL}};
const Segment SGI134918[] = {{&S18501[78], 6}, {NULL}};
const Segment SGI134919[] = {{&S18502[44], 20}, {NULL}};
const Segment SGI134920[] = {{&S18502[44], 32}, {NULL}};
const Segment SGI134921[] = {{&S18502[30], 6}, {NULL}};
const Segment SGI134922[] = {{&S18502[125], 3}, {NULL}};
const Segment SGI134923[] = {{&S18502[90], 6}, {NULL}};
const Segment SGI134924[] = {{&S18503[44], 20}, {NULL}};
const Segment SGI134925[] = {{&S18503[44], 32}, {NULL}};
const Segment SGI134926[] = {{&S18503[30], 6}, {NULL}};
const Segment SGI134927[] = {{&S18503[125], 3}, {NULL}};
const Segment SGI134928[] = {{&S18503[90], 6}, {NULL}};
const Segment SGI134929[] = {{&S18504[38], 8}, {NULL}};
const Segment SGI134930[] = {{&S18504[31], 6}, {NULL}};
const Segment SGI134931[] = {{&S18505[38], 11}, {NULL}};
const Segment SGI134932[] = {{&S18505[31], 6}, {NULL}};
const Segment SGI134933[] = {{&S18506[44], 14}, {NULL}};
const Segment SGI134934[] = {{&S18506[44], 26}, {NULL}};
const Segment SGI134935[] = {{&S18506[30], 6}, {NULL}};
const Segment SGI134936[] = {{&S18506[119], 3}, {NULL}};
const Segment SGI134937[] = {{&S18506[84], 6}, {NULL}};
const Segment SGI134938[] = {{&S18507[38], 14}, {NULL}};
const Segment SGI134939[] = {{&S18507[31], 6}, {NULL}};
const Segment SGI134940[] = {{&S18508[44], 9}, {NULL}};
const Segment SGI134941[] = {{&S18508[44], 21}, {NULL}};
const Segment SGI134942[] = {{&S18508[30], 6}, {NULL}};
const Segment SGI134943[] = {{&S18508[114], 3}, {NULL}};
const Segment SGI134944[] = {{&S18508[79], 6}, {NULL}};
const Segment SGI134945[] = {{&S18509[44], 24}, {NULL}};
const Segment SGI134946[] = {{&S18509[44], 36}, {NULL}};
const Segment SGI134947[] = {{&S18509[30], 6}, {NULL}};
const Segment SGI134948[] = {{&S18509[129], 3}, {NULL}};
const Segment SGI134949[] = {{&S18509[94], 6}, {NULL}};
const Segment SGI134950[] = {{&S18510[44], 25}, {NULL}};
const Segment SGI134951[] = {{&S18510[44], 37}, {NULL}};
const Segment SGI134952[] = {{&S18510[30], 6}, {NULL}};
const Segment SGI134953[] = {{&S18510[130], 3}, {NULL}};
const Segment SGI134954[] = {{&S18510[95], 6}, {NULL}};
const Segment SGI134955[] = {{&S18511[44], 21}, {NULL}};
const Segment SGI134956[] = {{&S18511[44], 33}, {NULL}};
const Segment SGI134957[] = {{&S18511[30], 6}, {NULL}};
const Segment SGI134958[] = {{&S18511[126], 3}, {NULL}};
const Segment SGI134959[] = {{&S18511[91], 6}, {NULL}};
const Segment SGI134960[] = {{&S18512[44], 20}, {NULL}};
const Segment SGI134961[] = {{&S18512[44], 32}, {NULL}};
const Segment SGI134962[] = {{&S18512[30], 6}, {NULL}};
const Segment SGI134963[] = {{&S18512[125], 3}, {NULL}};
const Segment SGI134964[] = {{&S18512[90], 6}, {NULL}};
const Segment SGI134965[] = {{&S18513[44], 21}, {NULL}};
const Segment SGI134966[] = {{&S18513[44], 33}, {NULL}};
const Segment SGI134967[] = {{&S18513[30], 6}, {NULL}};
const Segment SGI134968[] = {{&S18513[126], 3}, {NULL}};
const Segment SGI134969[] = {{&S18513[91], 6}, {NULL}};
const Segment SGI134970[] = {{&S18514[44], 31}, {NULL}};
const Segment SGI134971[] = {{&S18514[44], 43}, {NULL}};
const Segment SGI134972[] = {{&S18514[30], 6}, {NULL}};
const Segment SGI134973[] = {{&S18514[136], 3}, {NULL}};
const Segment SGI134974[] = {{&S18514[101], 6}, {NULL}};
const Segment SGI134975[] = {{&S18515[44], 21}, {NULL}};
const Segment SGI134976[] = {{&S18515[44], 33}, {NULL}};
const Segment SGI134977[] = {{&S18515[30], 6}, {NULL}};
const Segment SGI134978[] = {{&S18515[126], 3}, {NULL}};
const Segment SGI134979[] = {{&S18515[91], 6}, {NULL}};
const Segment SGI134980[] = {{&S18516[44], 31}, {NULL}};
const Segment SGI134981[] = {{&S18516[44], 43}, {NULL}};
const Segment SGI134982[] = {{&S18516[30], 6}, {NULL}};
const Segment SGI134983[] = {{&S18516[136], 3}, {NULL}};
const Segment SGI134984[] = {{&S18516[101], 6}, {NULL}};
const Segment SGI134985[] = {{&S18517[44], 25}, {NULL}};
const Segment SGI134986[] = {{&S18517[44], 37}, {NULL}};
const Segment SGI134987[] = {{&S18517[30], 6}, {NULL}};
const Segment SGI134988[] = {{&S18517[130], 3}, {NULL}};
const Segment SGI134989[] = {{&S18517[95], 6}, {NULL}};
const Segment SGI134990[] = {{&S18518[44], 21}, {NULL}};
const Segment SGI134991[] = {{&S18518[44], 33}, {NULL}};
const Segment SGI134992[] = {{&S18518[30], 6}, {NULL}};
const Segment SGI134993[] = {{&S18518[126], 3}, {NULL}};
const Segment SGI134994[] = {{&S18518[91], 6}, {NULL}};
const Segment SGI134995[] = {{&S18519[42], 9}, {NULL}};
const Segment SGI134996[] = {{&S18519[30], 6}, {NULL}};
const Segment SGI134997[] = {{&S18519[112], 3}, {NULL}};
const Segment SGI134998[] = {{&S18519[77], 6}, {NULL}};
const Segment SGI134999[] = {{&S18520[44], 21}, {NULL}};
const Segment SGI135000[] = {{&S18520[44], 33}, {NULL}};
const Segment SGI135001[] = {{&S18520[30], 6}, {NULL}};
const Segment SGI135002[] = {{&S18520[126], 3}, {NULL}};
const Segment SGI135003[] = {{&S18520[91], 6}, {NULL}};
const Segment SGI135004[] = {{&S18521[44], 21}, {NULL}};
const Segment SGI135005[] = {{&S18521[44], 33}, {NULL}};
const Segment SGI135006[] = {{&S18521[30], 6}, {NULL}};
const Segment SGI135007[] = {{&S18521[126], 3}, {NULL}};
const Segment SGI135008[] = {{&S18521[91], 6}, {NULL}};
const Segment SGI135009[] = {{&S18522[44], 32}, {NULL}};
const Segment SGI135010[] = {{&S18522[44], 44}, {NULL}};
const Segment SGI135011[] = {{&S18522[30], 6}, {NULL}};
const Segment SGI135012[] = {{&S18522[137], 3}, {NULL}};
const Segment SGI135013[] = {{&S18522[102], 6}, {NULL}};
const Segment SGI135014[] = {{&S18523[44], 32}, {NULL}};
const Segment SGI135015[] = {{&S18523[44], 44}, {NULL}};
const Segment SGI135016[] = {{&S18523[30], 6}, {NULL}};
const Segment SGI135017[] = {{&S18523[137], 3}, {NULL}};
const Segment SGI135018[] = {{&S18523[102], 6}, {NULL}};
const Segment SGI135019[] = {{&S18524[44], 24}, {NULL}};
const Segment SGI135020[] = {{&S18524[44], 36}, {NULL}};
const Segment SGI135021[] = {{&S18524[30], 6}, {NULL}};
const Segment SGI135022[] = {{&S18524[129], 3}, {NULL}};
const Segment SGI135023[] = {{&S18524[94], 6}, {NULL}};
const Segment SGI135024[] = {{&S18525[44], 32}, {NULL}};
const Segment SGI135025[] = {{&S18525[44], 44}, {NULL}};
const Segment SGI135026[] = {{&S18525[30], 6}, {NULL}};
const Segment SGI135027[] = {{&S18525[137], 3}, {NULL}};
const Segment SGI135028[] = {{&S18525[102], 6}, {NULL}};
const Segment SGI135029[] = {{&S18526[44], 31}, {NULL}};
const Segment SGI135030[] = {{&S18526[44], 43}, {NULL}};
const Segment SGI135031[] = {{&S18526[30], 6}, {NULL}};
const Segment SGI135032[] = {{&S18526[136], 3}, {NULL}};
const Segment SGI135033[] = {{&S18526[101], 6}, {NULL}};
const Segment SGI135034[] = {{&S18527[44], 21}, {NULL}};
const Segment SGI135035[] = {{&S18527[44], 33}, {NULL}};
const Segment SGI135036[] = {{&S18527[30], 6}, {NULL}};
const Segment SGI135037[] = {{&S18527[126], 3}, {NULL}};
const Segment SGI135038[] = {{&S18527[91], 6}, {NULL}};
const Segment SGI135039[] = {{&S18528[44], 32}, {NULL}};
const Segment SGI135040[] = {{&S18528[44], 44}, {NULL}};
const Segment SGI135041[] = {{&S18528[30], 6}, {NULL}};
const Segment SGI135042[] = {{&S18528[137], 3}, {NULL}};
const Segment SGI135043[] = {{&S18528[102], 6}, {NULL}};
const Segment SGI135044[] = {{&S18529[44], 25}, {NULL}};
const Segment SGI135045[] = {{&S18529[44], 37}, {NULL}};
const Segment SGI135046[] = {{&S18529[30], 6}, {NULL}};
const Segment SGI135047[] = {{&S18529[130], 3}, {NULL}};
const Segment SGI135048[] = {{&S18529[95], 6}, {NULL}};
const Segment SGI135049[] = {{&S19691[44], 17}, {NULL}};
const Segment SGI135050[] = {{&S19691[44], 29}, {NULL}};
const Segment SGI135051[] = {{&S19691[30], 6}, {NULL}};
const Segment SGI135052[] = {{&S19691[122], 3}, {NULL}};
const Segment SGI135053[] = {{&S19691[87], 6}, {NULL}};
const Segment SGI135054[] = {{&S19692[44], 5}, {NULL}};
const Segment SGI135055[] = {{&S19692[44], 17}, {NULL}};
const Segment SGI135056[] = {{&S19692[30], 6}, {NULL}};
const Segment SGI135057[] = {{&S19692[110], 3}, {NULL}};
const Segment SGI135058[] = {{&S19692[75], 6}, {NULL}};
const Segment SGI135059[] = {{&S19693[44], 17}, {NULL}};
const Segment SGI135060[] = {{&S19693[44], 29}, {NULL}};
const Segment SGI135061[] = {{&S19693[30], 6}, {NULL}};
const Segment SGI135062[] = {{&S19693[122], 3}, {NULL}};
const Segment SGI135063[] = {{&S19693[87], 6}, {NULL}};
const Segment SGI135064[] = {{&S19694[44], 27}, {NULL}};
const Segment SGI135065[] = {{&S19694[44], 39}, {NULL}};
const Segment SGI135066[] = {{&S19694[30], 6}, {&S19694[52], 31}, {NULL}};
const Segment SGI135067[] = {{&S19694[132], 3}, {NULL}};
const Segment SGI135068[] = {{&S19694[97], 6}, {NULL}};
const Segment SGI135069[] = {{&S19696[44], 9}, {NULL}};
const Segment SGI135070[] = {{&S19696[44], 21}, {NULL}};
const Segment SGI135071[] = {{&S19696[30], 6}, {NULL}};
const Segment SGI135072[] = {{&S19696[114], 3}, {NULL}};
const Segment SGI135073[] = {{&S19696[79], 6}, {NULL}};
const Segment SGI135074[] = {{&S18530[42], 8}, {NULL}};
const Segment SGI135075[] = {{&S18530[30], 6}, {NULL}};
const Segment SGI135076[] = {{&S18530[111], 3}, {NULL}};
const Segment SGI135077[] = {{&S18530[76], 6}, {NULL}};
const Segment SGI135078[] = {{&S19697[44], 7}, {NULL}};
const Segment SGI135079[] = {{&S19697[44], 19}, {NULL}};
const Segment SGI135080[] = {{&S19697[30], 6}, {NULL}};
const Segment SGI135081[] = {{&S19697[112], 3}, {NULL}};
const Segment SGI135082[] = {{&S19697[77], 6}, {NULL}};
const Segment SGI135083[] = {{&S19698[44], 4}, {NULL}};
const Segment SGI135084[] = {{&S19698[44], 16}, {NULL}};
const Segment SGI135085[] = {{&S19698[30], 6}, {NULL}};
const Segment SGI135086[] = {{&S19698[109], 3}, {NULL}};
const Segment SGI135087[] = {{&S19698[74], 6}, {NULL}};
const Segment SGI135088[] = {{&S19700[44], 21}, {NULL}};
const Segment SGI135089[] = {{&S19700[44], 33}, {NULL}};
const Segment SGI135090[] = {{&S19700[30], 6}, {NULL}};
const Segment SGI135091[] = {{&S19700[126], 3}, {NULL}};
const Segment SGI135092[] = {{&S19700[91], 6}, {NULL}};
const Segment SGI135093[] = {{&S19701[44], 7}, {NULL}};
const Segment SGI135094[] = {{&S19701[44], 19}, {NULL}};
const Segment SGI135095[] = {{&S19701[30], 6}, {NULL}};
const Segment SGI135096[] = {{&S19701[112], 3}, {NULL}};
const Segment SGI135097[] = {{&S19701[77], 6}, {NULL}};
const Segment SGI135098[] = {{&S19703[44], 11}, {NULL}};
const Segment SGI135099[] = {{&S19703[44], 23}, {NULL}};
const Segment SGI135100[] = {{&S19703[30], 6}, {NULL}};
const Segment SGI135101[] = {{&S19703[116], 3}, {NULL}};
const Segment SGI135102[] = {{&S19703[81], 6}, {NULL}};
const Segment SGI135103[] = {{&S19705[44], 13}, {NULL}};
const Segment SGI135104[] = {{&S19705[44], 25}, {NULL}};
const Segment SGI135105[] = {{&S19705[30], 6}, {NULL}};
const Segment SGI135106[] = {{&S19705[118], 3}, {NULL}};
const Segment SGI135107[] = {{&S19705[83], 6}, {NULL}};
const Segment SGI135108[] = {{&S19706[44], 8}, {NULL}};
const Segment SGI135109[] = {{&S19706[44], 20}, {NULL}};
const Segment SGI135110[] = {{&S19706[30], 6}, {NULL}};
const Segment SGI135111[] = {{&S19706[113], 3}, {NULL}};
const Segment SGI135112[] = {{&S19706[78], 6}, {NULL}};
const Segment SGI135113[] = {{&S19707[44], 16}, {NULL}};
const Segment SGI135114[] = {{&S19707[44], 28}, {NULL}};
const Segment SGI135115[] = {{&S19707[30], 6}, {NULL}};
const Segment SGI135116[] = {{&S19707[121], 3}, {NULL}};
const Segment SGI135117[] = {{&S19707[86], 6}, {NULL}};
const Segment SGI135118[] = {{&S19708[44], 7}, {NULL}};
const Segment SGI135119[] = {{&S19708[44], 19}, {NULL}};
const Segment SGI135120[] = {{&S19708[30], 6}, {NULL}};
const Segment SGI135121[] = {{&S19708[112], 3}, {NULL}};
const Segment SGI135122[] = {{&S19708[77], 6}, {NULL}};
const Segment SGI135123[] = {{&S19710[44], 8}, {NULL}};
const Segment SGI135124[] = {{&S19710[44], 20}, {NULL}};
const Segment SGI135125[] = {{&S19710[30], 6}, {NULL}};
const Segment SGI135126[] = {{&S19710[113], 3}, {NULL}};
const Segment SGI135127[] = {{&S19710[78], 6}, {NULL}};
const Segment SGI135128[] = {{&S19711[44], 10}, {NULL}};
const Segment SGI135129[] = {{&S19711[44], 22}, {NULL}};
const Segment SGI135130[] = {{&S19711[30], 6}, {NULL}};
const Segment SGI135131[] = {{&S19711[115], 3}, {NULL}};
const Segment SGI135132[] = {{&S19711[80], 6}, {NULL}};
const Segment SGI135133[] = {{&S19712[44], 4}, {NULL}};
const Segment SGI135134[] = {{&S19712[44], 16}, {NULL}};
const Segment SGI135135[] = {{&S19712[30], 6}, {NULL}};
const Segment SGI135136[] = {{&S19712[109], 3}, {NULL}};
const Segment SGI135137[] = {{&S19712[74], 6}, {NULL}};
const Segment SGI135138[] = {{&S19713[44], 7}, {NULL}};
const Segment SGI135139[] = {{&S19713[44], 19}, {NULL}};
const Segment SGI135140[] = {{&S19713[30], 6}, {NULL}};
const Segment SGI135141[] = {{&S19713[112], 3}, {NULL}};
const Segment SGI135142[] = {{&S19713[77], 6}, {NULL}};
const Segment SGI135143[] = {{&S19714[30], 6}, {NULL}};
const Segment SGI135144[] = {{&S19714[104], 3}, {NULL}};
const Segment SGI135145[] = {{&S19714[57], 6}, {NULL}};
const Segment SGI135146[] = {{&S19715[44], 17}, {NULL}};
const Segment SGI135147[] = {{&S19715[44], 29}, {NULL}};
const Segment SGI135148[] = {{&S19715[30], 6}, {NULL}};
const Segment SGI135149[] = {{&S19715[122], 3}, {NULL}};
const Segment SGI135150[] = {{&S19715[87], 6}, {NULL}};
const Segment SGI135151[] = {{&S19716[44], 14}, {NULL}};
const Segment SGI135152[] = {{&S19716[44], 26}, {NULL}};
const Segment SGI135153[] = {{&S19716[30], 6}, {NULL}};
const Segment SGI135154[] = {{&S19716[119], 3}, {NULL}};
const Segment SGI135155[] = {{&S19716[84], 6}, {NULL}};
const Segment SGI135156[] = {{&S19717[44], 7}, {NULL}};
const Segment SGI135157[] = {{&S19717[44], 19}, {NULL}};
const Segment SGI135158[] = {{&S19717[30], 6}, {NULL}};
const Segment SGI135159[] = {{&S19717[112], 3}, {NULL}};
const Segment SGI135160[] = {{&S19717[77], 6}, {NULL}};
const Segment SGI135161[] = {{&S19718[44], 8}, {NULL}};
const Segment SGI135162[] = {{&S19718[44], 20}, {NULL}};
const Segment SGI135163[] = {{&S19718[30], 6}, {NULL}};
const Segment SGI135164[] = {{&S19718[113], 3}, {NULL}};
const Segment SGI135165[] = {{&S19718[78], 6}, {NULL}};
const Segment SGI135166[] = {{&S19719[44], 27}, {NULL}};
const Segment SGI135167[] = {{&S19719[44], 39}, {NULL}};
const Segment SGI135168[] = {{&S19719[30], 6}, {&S19719[52], 31}, {NULL}};
const Segment SGI135169[] = {{&S19719[132], 3}, {NULL}};
const Segment SGI135170[] = {{&S19719[97], 6}, {NULL}};
const Segment SGI135171[] = {{&S19720[11], 1}, {NULL}};
const Segment SGI135172[] = {{&S19720[44], 13}, {NULL}};
const Segment SGI135173[] = {{&S19720[30], 6}, {NULL}};
const Segment SGI135174[] = {{&S19720[106], 3}, {NULL}};
const Segment SGI135175[] = {{&S19720[71], 6}, {NULL}};
const Segment SGI135176[] = {{&S19722[44], 4}, {NULL}};
const Segment SGI135177[] = {{&S19722[44], 16}, {NULL}};
const Segment SGI135178[] = {{&S19722[30], 6}, {NULL}};
const Segment SGI135179[] = {{&S19722[109], 3}, {NULL}};
const Segment SGI135180[] = {{&S19722[74], 6}, {NULL}};
const Segment SGI135181[] = {{&S19723[11], 1}, {NULL}};
const Segment SGI135182[] = {{&S19723[44], 8}, {NULL}};
const Segment SGI135183[] = {{&S19723[30], 6}, {NULL}};
const Segment SGI135184[] = {{&S19723[101], 3}, {NULL}};
const Segment SGI135185[] = {{&S19723[66], 6}, {NULL}};
const Segment SGI135186[] = {{&S19724[44], 24}, {NULL}};
const Segment SGI135187[] = {{&S19724[44], 36}, {NULL}};
const Segment SGI135188[] = {{&S19724[30], 6}, {NULL}};
const Segment SGI135189[] = {{&S19724[129], 3}, {NULL}};
const Segment SGI135190[] = {{&S19724[94], 6}, {NULL}};
const Segment SGI135191[] = {{&S19725[44], 38}, {NULL}};
const Segment SGI135192[] = {{&S19725[44], 50}, {NULL}};
const Segment SGI135193[] = {{&S19725[30], 6}, {NULL}};
const Segment SGI135194[] = {{&S19725[143], 3}, {NULL}};
const Segment SGI135195[] = {{&S19725[108], 6}, {NULL}};
const Segment SGI135196[] = {{&S19726[22], 9}, {NULL}};
const Segment SGI135197[] = {{&S19726[44], 21}, {NULL}};
const Segment SGI135198[] = {{&S19726[30], 6}, {&S19726[52], 13}, {NULL}};
const Segment SGI135199[] = {{&S19726[114], 3}, {NULL}};
const Segment SGI135200[] = {{&S19726[79], 6}, {NULL}};
const Segment SGI135201[] = {{&S19727[44], 29}, {NULL}};
const Segment SGI135202[] = {{&S19727[44], 41}, {NULL}};
const Segment SGI135203[] = {{&S19727[30], 6}, {NULL}};
const Segment SGI135204[] = {{&S19727[134], 3}, {NULL}};
const Segment SGI135205[] = {{&S19727[99], 6}, {NULL}};
const Segment SGI135206[] = {{&S19728[44], 19}, {NULL}};
const Segment SGI135207[] = {{&S19728[44], 31}, {NULL}};
const Segment SGI135208[] = {{&S19728[30], 6}, {NULL}};
const Segment SGI135209[] = {{&S19728[124], 3}, {NULL}};
const Segment SGI135210[] = {{&S19728[89], 6}, {NULL}};
const Segment SGI135211[] = {{&S19729[44], 25}, {NULL}};
const Segment SGI135212[] = {{&S19729[44], 37}, {NULL}};
const Segment SGI135213[] = {{&S19729[30], 6}, {NULL}};
const Segment SGI135214[] = {{&S19729[130], 3}, {NULL}};
const Segment SGI135215[] = {{&S19729[95], 6}, {NULL}};
const Segment SGI135216[] = {{&S19730[44], 10}, {NULL}};
const Segment SGI135217[] = {{&S19730[44], 22}, {NULL}};
const Segment SGI135218[] = {{&S19730[30], 6}, {NULL}};
const Segment SGI135219[] = {{&S19730[115], 3}, {NULL}};
const Segment SGI135220[] = {{&S19730[80], 6}, {NULL}};
const Segment SGI135221[] = {{&S19731[44], 24}, {NULL}};
const Segment SGI135222[] = {{&S19731[44], 36}, {NULL}};
const Segment SGI135223[] = {{&S19731[30], 6}, {NULL}};
const Segment SGI135224[] = {{&S19731[129], 3}, {NULL}};
const Segment SGI135225[] = {{&S19731[94], 6}, {NULL}};
const Segment SGI135226[] = {{&S19732[44], 9}, {NULL}};
const Segment SGI135227[] = {{&S19732[44], 34}, {NULL}};
const Segment SGI135228[] = {{&S19732[30], 6}, {NULL}};
const Segment SGI135229[] = {{&S19732[127], 3}, {NULL}};
const Segment SGI135230[] = {{&S19732[92], 6}, {NULL}};
const Segment SGI135231[] = {{&S19733[44], 22}, {NULL}};
const Segment SGI135232[] = {{&S19733[44], 34}, {NULL}};
const Segment SGI135233[] = {{&S19733[30], 6}, {NULL}};
const Segment SGI135234[] = {{&S19733[127], 3}, {NULL}};
const Segment SGI135235[] = {{&S19733[92], 6}, {NULL}};
const Segment SGI135236[] = {{&S19734[44], 31}, {NULL}};
const Segment SGI135237[] = {{&S19734[44], 43}, {NULL}};
const Segment SGI135238[] = {{&S19734[30], 6}, {NULL}};
const Segment SGI135239[] = {{&S19734[136], 3}, {NULL}};
const Segment SGI135240[] = {{&S19734[101], 6}, {NULL}};
const Segment SGI135241[] = {{&S19736[44], 28}, {NULL}};
const Segment SGI135242[] = {{&S19736[44], 40}, {NULL}};
const Segment SGI135243[] = {{&S19736[30], 6}, {NULL}};
const Segment SGI135244[] = {{&S19736[133], 3}, {NULL}};
const Segment SGI135245[] = {{&S19736[98], 6}, {NULL}};
const Segment SGI135246[] = {{&S19738[44], 27}, {NULL}};
const Segment SGI135247[] = {{&S19738[44], 39}, {NULL}};
const Segment SGI135248[] = {{&S19738[30], 6}, {NULL}};
const Segment SGI135249[] = {{&S19738[132], 3}, {NULL}};
const Segment SGI135250[] = {{&S19738[97], 6}, {NULL}};
const Segment SGI135251[] = {{&S19739[44], 14}, {NULL}};
const Segment SGI135252[] = {{&S19739[44], 26}, {NULL}};
const Segment SGI135253[] = {{&S19739[30], 6}, {NULL}};
const Segment SGI135254[] = {{&S19739[119], 3}, {NULL}};
const Segment SGI135255[] = {{&S19739[84], 6}, {NULL}};
const Segment SGI135256[] = {{&S19740[44], 27}, {NULL}};
const Segment SGI135257[] = {{&S19740[44], 39}, {NULL}};
const Segment SGI135258[] = {{&S19740[30], 6}, {NULL}};
const Segment SGI135259[] = {{&S19740[132], 3}, {NULL}};
const Segment SGI135260[] = {{&S19740[97], 6}, {NULL}};
const Segment SGI135261[] = {{&S19741[44], 8}, {NULL}};
const Segment SGI135262[] = {{&S19741[44], 20}, {NULL}};
const Segment SGI135263[] = {{&S19741[30], 6}, {NULL}};
const Segment SGI135264[] = {{&S19741[113], 3}, {NULL}};
const Segment SGI135265[] = {{&S19741[78], 6}, {NULL}};
const Segment SGI135266[] = {{&S19742[44], 22}, {NULL}};
const Segment SGI135267[] = {{&S19742[44], 34}, {NULL}};
const Segment SGI135268[] = {{&S19742[30], 6}, {NULL}};
const Segment SGI135269[] = {{&S19742[127], 3}, {NULL}};
const Segment SGI135270[] = {{&S19742[92], 6}, {NULL}};
const Segment SGI135271[] = {{&S19743[44], 31}, {NULL}};
const Segment SGI135272[] = {{&S19743[44], 43}, {NULL}};
const Segment SGI135273[] = {{&S19743[30], 6}, {NULL}};
const Segment SGI135274[] = {{&S19743[136], 3}, {NULL}};
const Segment SGI135275[] = {{&S19743[101], 6}, {NULL}};
const Segment SGI135276[] = {{&S19744[44], 16}, {NULL}};
const Segment SGI135277[] = {{&S19744[44], 28}, {NULL}};
const Segment SGI135278[] = {{&S19744[30], 6}, {NULL}};
const Segment SGI135279[] = {{&S19744[121], 3}, {NULL}};
const Segment SGI135280[] = {{&S19744[86], 6}, {NULL}};
const Segment SGI135281[] = {{&S19745[44], 30}, {NULL}};
const Segment SGI135282[] = {{&S19745[44], 42}, {NULL}};
const Segment SGI135283[] = {{&S19745[30], 6}, {NULL}};
const Segment SGI135284[] = {{&S19745[135], 3}, {NULL}};
const Segment SGI135285[] = {{&S19745[100], 6}, {NULL}};
const Segment SGI135286[] = {{&S19747[58], 13}, {NULL}};
const Segment SGI135287[] = {{&S19747[58], 25}, {NULL}};
const Segment SGI135288[] = {{&S19747[44], 6}, {NULL}};
const Segment SGI135289[] = {{&S19747[132], 3}, {NULL}};
const Segment SGI135290[] = {{&S19747[97], 6}, {NULL}};
const Segment SGI135291[] = {{&S19748[44], 11}, {NULL}};
const Segment SGI135292[] = {{&S19748[44], 26}, {NULL}};
const Segment SGI135293[] = {{&S19748[30], 6}, {NULL}};
const Segment SGI135294[] = {{&S19748[119], 3}, {NULL}};
const Segment SGI135295[] = {{&S19748[84], 6}, {NULL}};
const Segment SGI135296[] = {{&S19749[44], 4}, {NULL}};
const Segment SGI135297[] = {{&S19749[44], 16}, {NULL}};
const Segment SGI135298[] = {{&S19749[30], 6}, {NULL}};
const Segment SGI135299[] = {{&S19749[109], 3}, {NULL}};
const Segment SGI135300[] = {{&S19749[74], 6}, {NULL}};
const Segment SGI135301[] = {{&S19750[11], 1}, {NULL}};
const Segment SGI135302[] = {{&S19750[44], 13}, {NULL}};
const Segment SGI135303[] = {{&S19750[30], 6}, {NULL}};
const Segment SGI135304[] = {{&S19750[106], 3}, {NULL}};
const Segment SGI135305[] = {{&S19750[71], 6}, {NULL}};
const Segment SGI135306[] = {{&S19751[44], 5}, {NULL}};
const Segment SGI135307[] = {{&S19751[44], 17}, {NULL}};
const Segment SGI135308[] = {{&S19751[30], 6}, {NULL}};
const Segment SGI135309[] = {{&S19751[110], 3}, {NULL}};
const Segment SGI135310[] = {{&S19751[75], 6}, {NULL}};
const Segment SGI135311[] = {{&S19752[44], 33}, {NULL}};
const Segment SGI135312[] = {{&S19752[44], 45}, {NULL}};
const Segment SGI135313[] = {{&S19752[30], 6}, {NULL}};
const Segment SGI135314[] = {{&S19752[138], 3}, {NULL}};
const Segment SGI135315[] = {{&S19752[103], 6}, {NULL}};
const Segment SGI135316[] = {{&S19753[44], 22}, {NULL}};
const Segment SGI135317[] = {{&S19753[44], 34}, {NULL}};
const Segment SGI135318[] = {{&S19753[30], 6}, {NULL}};
const Segment SGI135319[] = {{&S19753[127], 3}, {NULL}};
const Segment SGI135320[] = {{&S19753[92], 6}, {NULL}};
const Segment SGI135321[] = {{&S19754[44], 25}, {NULL}};
const Segment SGI135322[] = {{&S19754[44], 37}, {NULL}};
const Segment SGI135323[] = {{&S19754[30], 6}, {NULL}};
const Segment SGI135324[] = {{&S19754[130], 3}, {NULL}};
const Segment SGI135325[] = {{&S19754[95], 6}, {NULL}};
const Segment SGI135326[] = {{&S19755[44], 7}, {NULL}};
const Segment SGI135327[] = {{&S19755[44], 19}, {NULL}};
const Segment SGI135328[] = {{&S19755[30], 6}, {NULL}};
const Segment SGI135329[] = {{&S19755[112], 3}, {NULL}};
const Segment SGI135330[] = {{&S19755[77], 6}, {NULL}};
const Segment SGI135331[] = {{&S19756[44], 25}, {NULL}};
const Segment SGI135332[] = {{&S19756[44], 37}, {NULL}};
const Segment SGI135333[] = {{&S19756[30], 6}, {NULL}};
const Segment SGI135334[] = {{&S19756[130], 3}, {NULL}};
const Segment SGI135335[] = {{&S19756[95], 6}, {NULL}};
const Segment SGI135336[] = {{&S19757[44], 27}, {NULL}};
const Segment SGI135337[] = {{&S19757[44], 39}, {NULL}};
const Segment SGI135338[] = {{&S19757[30], 6}, {NULL}};
const Segment SGI135339[] = {{&S19757[132], 3}, {NULL}};
const Segment SGI135340[] = {{&S19757[97], 6}, {NULL}};
const Segment SGI135341[] = {{&S19758[44], 19}, {NULL}};
const Segment SGI135342[] = {{&S19758[44], 31}, {NULL}};
const Segment SGI135343[] = {{&S19758[30], 6}, {NULL}};
const Segment SGI135344[] = {{&S19758[124], 3}, {NULL}};
const Segment SGI135345[] = {{&S19758[89], 6}, {NULL}};
const Segment SGI135346[] = {{&S19759[44], 9}, {NULL}};
const Segment SGI135347[] = {{&S19759[44], 21}, {NULL}};
const Segment SGI135348[] = {{&S19759[30], 6}, {NULL}};
const Segment SGI135349[] = {{&S19759[114], 3}, {NULL}};
const Segment SGI135350[] = {{&S19759[79], 6}, {NULL}};
const Segment SGI135351[] = {{&S19760[44], 16}, {NULL}};
const Segment SGI135352[] = {{&S19760[44], 28}, {NULL}};
const Segment SGI135353[] = {{&S19760[30], 6}, {NULL}};
const Segment SGI135354[] = {{&S19760[121], 3}, {NULL}};
const Segment SGI135355[] = {{&S19760[86], 6}, {NULL}};
const Segment SGI135356[] = {{&S19761[44], 5}, {NULL}};
const Segment SGI135357[] = {{&S19761[44], 17}, {NULL}};
const Segment SGI135358[] = {{&S19761[30], 6}, {NULL}};
const Segment SGI135359[] = {{&S19761[110], 3}, {NULL}};
const Segment SGI135360[] = {{&S19761[75], 6}, {NULL}};
const Segment SGI135361[] = {{&S19762[44], 7}, {NULL}};
const Segment SGI135362[] = {{&S19762[44], 19}, {NULL}};
const Segment SGI135363[] = {{&S19762[30], 6}, {NULL}};
const Segment SGI135364[] = {{&S19762[112], 3}, {NULL}};
const Segment SGI135365[] = {{&S19762[77], 6}, {NULL}};
const Segment SGI135366[] = {{&S19763[44], 28}, {NULL}};
const Segment SGI135367[] = {{&S19763[44], 40}, {NULL}};
const Segment SGI135368[] = {{&S19763[30], 6}, {NULL}};
const Segment SGI135369[] = {{&S19763[133], 3}, {NULL}};
const Segment SGI135370[] = {{&S19763[98], 6}, {NULL}};
const Segment SGI135371[] = {{&S19764[58], 13}, {NULL}};
const Segment SGI135372[] = {{&S19764[58], 25}, {NULL}};
const Segment SGI135373[] = {{&S19764[44], 6}, {NULL}};
const Segment SGI135374[] = {{&S19764[132], 3}, {NULL}};
const Segment SGI135375[] = {{&S19764[97], 6}, {NULL}};
const Segment SGI135376[] = {{&S19765[44], 28}, {NULL}};
const Segment SGI135377[] = {{&S19765[44], 40}, {NULL}};
const Segment SGI135378[] = {{&S19765[30], 6}, {NULL}};
const Segment SGI135379[] = {{&S19765[133], 3}, {NULL}};
const Segment SGI135380[] = {{&S19765[98], 6}, {NULL}};
const Segment SGI135381[] = {{&S19768[44], 12}, {NULL}};
const Segment SGI135382[] = {{&S19768[44], 24}, {NULL}};
const Segment SGI135383[] = {{&S19768[30], 6}, {NULL}};
const Segment SGI135384[] = {{&S19768[117], 3}, {NULL}};
const Segment SGI135385[] = {{&S19768[82], 6}, {NULL}};
const Segment SGI135386[] = {{&S19769[28], 1}, {NULL}};
const Segment SGI135387[] = {{&S19769[47], 6}, {NULL}};
const Segment SGI135388[] = {{&S19770[44], 18}, {NULL}};
const Segment SGI135389[] = {{&S19770[44], 30}, {NULL}};
const Segment SGI135390[] = {{&S19770[30], 6}, {NULL}};
const Segment SGI135391[] = {{&S19770[123], 3}, {NULL}};
const Segment SGI135392[] = {{&S19770[88], 6}, {NULL}};
const Segment SGI135393[] = {{&S19771[44], 32}, {NULL}};
const Segment SGI135394[] = {{&S19771[44], 44}, {NULL}};
const Segment SGI135395[] = {{&S19771[30], 6}, {NULL}};
const Segment SGI135396[] = {{&S19771[137], 3}, {NULL}};
const Segment SGI135397[] = {{&S19771[102], 6}, {NULL}};
const Segment SGI135398[] = {{&S19772[44], 25}, {NULL}};
const Segment SGI135399[] = {{&S19772[44], 37}, {NULL}};
const Segment SGI135400[] = {{&S19772[30], 6}, {NULL}};
const Segment SGI135401[] = {{&S19772[130], 3}, {NULL}};
const Segment SGI135402[] = {{&S19772[95], 6}, {NULL}};
const Segment SGI135403[] = {{&S19773[44], 6}, {NULL}};
const Segment SGI135404[] = {{&S19773[44], 18}, {NULL}};
const Segment SGI135405[] = {{&S19773[30], 6}, {NULL}};
const Segment SGI135406[] = {{&S19773[111], 3}, {NULL}};
const Segment SGI135407[] = {{&S19773[76], 6}, {NULL}};
const Segment SGI135408[] = {{&S19775[38], 5}, {NULL}};
const Segment SGI135409[] = {{&S19775[31], 6}, {NULL}};
const Segment SGI135410[] = {{&S19776[44], 8}, {NULL}};
const Segment SGI135411[] = {{&S19776[44], 20}, {NULL}};
const Segment SGI135412[] = {{&S19776[30], 6}, {NULL}};
const Segment SGI135413[] = {{&S19776[113], 3}, {NULL}};
const Segment SGI135414[] = {{&S19776[78], 6}, {NULL}};
const Segment SGI135415[] = {{&S19777[44], 19}, {NULL}};
const Segment SGI135416[] = {{&S19777[44], 31}, {NULL}};
const Segment SGI135417[] = {{&S19777[30], 6}, {NULL}};
const Segment SGI135418[] = {{&S19777[124], 3}, {NULL}};
const Segment SGI135419[] = {{&S19777[89], 6}, {NULL}};
const Segment SGI135420[] = {{&S19779[44], 29}, {NULL}};
const Segment SGI135421[] = {{&S19779[44], 41}, {NULL}};
const Segment SGI135422[] = {{&S19779[30], 6}, {NULL}};
const Segment SGI135423[] = {{&S19779[134], 3}, {NULL}};
const Segment SGI135424[] = {{&S19779[99], 6}, {NULL}};
const Segment SGI135425[] = {{&S19780[44], 20}, {NULL}};
const Segment SGI135426[] = {{&S19780[44], 32}, {NULL}};
const Segment SGI135427[] = {{&S19780[30], 6}, {NULL}};
const Segment SGI135428[] = {{&S19780[125], 3}, {NULL}};
const Segment SGI135429[] = {{&S19780[90], 6}, {NULL}};
const Segment SGI135430[] = {{&S19781[44], 15}, {NULL}};
const Segment SGI135431[] = {{&S19781[44], 27}, {NULL}};
const Segment SGI135432[] = {{&S19781[30], 6}, {NULL}};
const Segment SGI135433[] = {{&S19781[120], 3}, {NULL}};
const Segment SGI135434[] = {{&S19781[85], 6}, {NULL}};
const Segment SGI135435[] = {{&S19782[44], 21}, {NULL}};
const Segment SGI135436[] = {{&S19782[44], 33}, {NULL}};
const Segment SGI135437[] = {{&S19782[30], 6}, {NULL}};
const Segment SGI135438[] = {{&S19782[126], 3}, {NULL}};
const Segment SGI135439[] = {{&S19782[91], 6}, {NULL}};
const Segment SGI135440[] = {{&S19783[44], 10}, {NULL}};
const Segment SGI135441[] = {{&S19783[44], 22}, {NULL}};
const Segment SGI135442[] = {{&S19783[30], 6}, {NULL}};
const Segment SGI135443[] = {{&S19783[115], 3}, {NULL}};
const Segment SGI135444[] = {{&S19783[80], 6}, {NULL}};
const Segment SGI135445[] = {{&S19784[44], 6}, {NULL}};
const Segment SGI135446[] = {{&S19784[44], 18}, {NULL}};
const Segment SGI135447[] = {{&S19784[30], 6}, {NULL}};
const Segment SGI135448[] = {{&S19784[111], 3}, {NULL}};
const Segment SGI135449[] = {{&S19784[76], 6}, {NULL}};
const Segment SGI135450[] = {{&S19786[44], 5}, {NULL}};
const Segment SGI135451[] = {{&S19786[44], 17}, {NULL}};
const Segment SGI135452[] = {{&S19786[30], 6}, {NULL}};
const Segment SGI135453[] = {{&S19786[110], 3}, {NULL}};
const Segment SGI135454[] = {{&S19786[75], 6}, {NULL}};
const Segment SGI135455[] = {{&S19788[44], 6}, {NULL}};
const Segment SGI135456[] = {{&S19788[44], 18}, {NULL}};
const Segment SGI135457[] = {{&S19788[30], 6}, {NULL}};
const Segment SGI135458[] = {{&S19788[111], 3}, {NULL}};
const Segment SGI135459[] = {{&S19788[76], 6}, {NULL}};
const Segment SGI135460[] = {{&S19790[44], 27}, {NULL}};
const Segment SGI135461[] = {{&S19790[44], 39}, {NULL}};
const Segment SGI135462[] = {{&S19790[30], 6}, {NULL}};
const Segment SGI135463[] = {{&S19790[132], 3}, {NULL}};
const Segment SGI135464[] = {{&S19790[97], 6}, {NULL}};
const Segment SGI135465[] = {{&S19791[44], 18}, {NULL}};
const Segment SGI135466[] = {{&S19791[44], 30}, {NULL}};
const Segment SGI135467[] = {{&S19791[30], 6}, {NULL}};
const Segment SGI135468[] = {{&S19791[123], 3}, {NULL}};
const Segment SGI135469[] = {{&S19791[88], 6}, {NULL}};
const Segment SGI135470[] = {{&S19792[38], 25}, {NULL}};
const Segment SGI135471[] = {{&S19792[31], 6}, {NULL}};
const Segment SGI135472[] = {{&S19793[44], 33}, {NULL}};
const Segment SGI135473[] = {{&S19793[44], 45}, {NULL}};
const Segment SGI135474[] = {{&S19793[30], 6}, {NULL}};
const Segment SGI135475[] = {{&S19793[138], 3}, {NULL}};
const Segment SGI135476[] = {{&S19793[103], 6}, {NULL}};
const Segment SGI135477[] = {{&S19795[44], 24}, {NULL}};
const Segment SGI135478[] = {{&S19795[44], 36}, {NULL}};
const Segment SGI135479[] = {{&S19795[30], 6}, {NULL}};
const Segment SGI135480[] = {{&S19795[129], 3}, {NULL}};
const Segment SGI135481[] = {{&S19795[94], 6}, {NULL}};
const Segment SGI135482[] = {{&S19796[38], 31}, {NULL}};
const Segment SGI135483[] = {{&S19796[31], 6}, {NULL}};
const Segment SGI135484[] = {{&S19797[44], 21}, {NULL}};
const Segment SGI135485[] = {{&S19797[44], 33}, {NULL}};
const Segment SGI135486[] = {{&S19797[30], 6}, {NULL}};
const Segment SGI135487[] = {{&S19797[126], 3}, {NULL}};
const Segment SGI135488[] = {{&S19797[91], 6}, {NULL}};
const Segment SGI135489[] = {{&S19798[38], 31}, {NULL}};
const Segment SGI135490[] = {{&S19798[31], 6}, {NULL}};
const Segment SGI135491[] = {{&S19799[44], 31}, {NULL}};
const Segment SGI135492[] = {{&S19799[44], 43}, {NULL}};
const Segment SGI135493[] = {{&S19799[30], 6}, {NULL}};
const Segment SGI135494[] = {{&S19799[136], 3}, {NULL}};
const Segment SGI135495[] = {{&S19799[101], 6}, {NULL}};
const Segment SGI135496[] = {{&S19800[44], 9}, {NULL}};
const Segment SGI135497[] = {{&S19800[44], 21}, {NULL}};
const Segment SGI135498[] = {{&S19800[30], 6}, {NULL}};
const Segment SGI135499[] = {{&S19800[114], 3}, {NULL}};
const Segment SGI135500[] = {{&S19800[79], 6}, {NULL}};
const Segment SGI135501[] = {{&S19801[44], 22}, {NULL}};
const Segment SGI135502[] = {{&S19801[44], 34}, {NULL}};
const Segment SGI135503[] = {{&S19801[30], 6}, {NULL}};
const Segment SGI135504[] = {{&S19801[127], 3}, {NULL}};
const Segment SGI135505[] = {{&S19801[92], 6}, {NULL}};
const Segment SGI135506[] = {{&S19804[44], 26}, {NULL}};
const Segment SGI135507[] = {{&S19804[44], 38}, {NULL}};
const Segment SGI135508[] = {{&S19804[30], 6}, {NULL}};
const Segment SGI135509[] = {{&S19804[131], 3}, {NULL}};
const Segment SGI135510[] = {{&S19804[96], 6}, {NULL}};
const Segment SGI135511[] = {{&S19805[38], 27}, {NULL}};
const Segment SGI135512[] = {{&S19805[31], 6}, {NULL}};
const Segment SGI135513[] = {{&S19806[44], 9}, {NULL}};
const Segment SGI135514[] = {{&S19806[44], 21}, {NULL}};
const Segment SGI135515[] = {{&S19806[30], 6}, {NULL}};
const Segment SGI135516[] = {{&S19806[113], 3}, {NULL}};
const Segment SGI135517[] = {{&S19806[79], 5}, {NULL}};
const Segment SGI135518[] = {{&S19808[38], 27}, {NULL}};
const Segment SGI135519[] = {{&S19808[31], 6}, {NULL}};
const Segment SGI135520[] = {{&S19812[44], 10}, {NULL}};
const Segment SGI135521[] = {{&S19812[44], 22}, {NULL}};
const Segment SGI135522[] = {{&S19812[30], 6}, {NULL}};
const Segment SGI135523[] = {{&S19812[115], 3}, {NULL}};
const Segment SGI135524[] = {{&S19812[80], 6}, {NULL}};
const Segment SGI135525[] = {{&S19813[44], 29}, {NULL}};
const Segment SGI135526[] = {{&S19813[44], 41}, {NULL}};
const Segment SGI135527[] = {{&S19813[30], 6}, {NULL}};
const Segment SGI135528[] = {{&S19813[134], 3}, {NULL}};
const Segment SGI135529[] = {{&S19813[99], 6}, {NULL}};
const Segment SGI135530[] = {{&S19814[11], 1}, {NULL}};
const Segment SGI135531[] = {{&S19814[44], 13}, {NULL}};
const Segment SGI135532[] = {{&S19814[30], 6}, {NULL}};
const Segment SGI135533[] = {{&S19814[106], 3}, {NULL}};
const Segment SGI135534[] = {{&S19814[71], 6}, {NULL}};
const Segment SGI135535[] = {{&S19815[44], 11}, {NULL}};
const Segment SGI135536[] = {{&S19815[44], 23}, {NULL}};
const Segment SGI135537[] = {{&S19815[30], 6}, {NULL}};
const Segment SGI135538[] = {{&S19815[116], 3}, {NULL}};
const Segment SGI135539[] = {{&S19815[81], 6}, {NULL}};
const Segment SGI135540[] = {{&S19816[44], 11}, {NULL}};
const Segment SGI135541[] = {{&S19816[44], 23}, {NULL}};
const Segment SGI135542[] = {{&S19816[30], 6}, {NULL}};
const Segment SGI135543[] = {{&S19816[116], 3}, {NULL}};
const Segment SGI135544[] = {{&S19816[81], 6}, {NULL}};
const Segment SGI135545[] = {{&S19817[44], 10}, {NULL}};
const Segment SGI135546[] = {{&S19817[44], 22}, {NULL}};
const Segment SGI135547[] = {{&S19817[30], 6}, {NULL}};
const Segment SGI135548[] = {{&S19817[115], 3}, {NULL}};
const Segment SGI135549[] = {{&S19817[80], 6}, {NULL}};
const Segment SGI135550[] = {{&S19818[44], 22}, {NULL}};
const Segment SGI135551[] = {{&S19818[44], 34}, {NULL}};
const Segment SGI135552[] = {{&S19818[30], 6}, {NULL}};
const Segment SGI135553[] = {{&S19818[127], 3}, {NULL}};
const Segment SGI135554[] = {{&S19818[92], 6}, {NULL}};
const Segment SGI135555[] = {{&S19823[44], 22}, {NULL}};
const Segment SGI135556[] = {{&S19823[44], 34}, {NULL}};
const Segment SGI135557[] = {{&S19823[30], 6}, {NULL}};
const Segment SGI135558[] = {{&S19823[127], 3}, {NULL}};
const Segment SGI135559[] = {{&S19823[92], 6}, {NULL}};
const Segment SGI135560[] = {{&S19824[44], 9}, {NULL}};
const Segment SGI135561[] = {{&S19824[44], 21}, {NULL}};
const Segment SGI135562[] = {{&S19824[30], 6}, {NULL}};
const Segment SGI135563[] = {{&S19824[114], 3}, {NULL}};
const Segment SGI135564[] = {{&S19824[79], 6}, {NULL}};
const Segment SGI135565[] = {{&S19825[44], 18}, {NULL}};
const Segment SGI135566[] = {{&S19825[44], 30}, {NULL}};
const Segment SGI135567[] = {{&S19825[30], 6}, {NULL}};
const Segment SGI135568[] = {{&S19825[123], 3}, {NULL}};
const Segment SGI135569[] = {{&S19825[88], 6}, {NULL}};
const Segment SGI135570[] = {{&S19826[44], 20}, {NULL}};
const Segment SGI135571[] = {{&S19826[44], 32}, {NULL}};
const Segment SGI135572[] = {{&S19826[30], 6}, {NULL}};
const Segment SGI135573[] = {{&S19826[124], 3}, {NULL}};
const Segment SGI135574[] = {{&S19826[90], 5}, {NULL}};
const Segment SGI135575[] = {{&S19827[44], 13}, {NULL}};
const Segment SGI135576[] = {{&S19827[44], 25}, {NULL}};
const Segment SGI135577[] = {{&S19827[30], 6}, {NULL}};
const Segment SGI135578[] = {{&S19827[118], 3}, {NULL}};
const Segment SGI135579[] = {{&S19827[83], 6}, {NULL}};
const Segment SGI135580[] = {{&S19829[44], 28}, {NULL}};
const Segment SGI135581[] = {{&S19829[44], 40}, {NULL}};
const Segment SGI135582[] = {{&S19829[30], 6}, {NULL}};
const Segment SGI135583[] = {{&S19829[133], 3}, {NULL}};
const Segment SGI135584[] = {{&S19829[98], 6}, {NULL}};
const Segment SGI135585[] = {{&S19830[38], 22}, {NULL}};
const Segment SGI135586[] = {{&S19830[31], 6}, {NULL}};
const Segment SGI135587[] = {{&S19831[38], 16}, {NULL}};
const Segment SGI135588[] = {{&S19831[31], 6}, {NULL}};
const Segment SGI135589[] = {{&S19833[44], 5}, {NULL}};
const Segment SGI135590[] = {{&S19833[44], 17}, {NULL}};
const Segment SGI135591[] = {{&S19833[30], 6}, {NULL}};
const Segment SGI135592[] = {{&S19833[110], 3}, {NULL}};
const Segment SGI135593[] = {{&S19833[75], 6}, {NULL}};
const Segment SGI135594[] = {{&S19834[44], 8}, {&S19834[11], 1}, {NULL}};
const Segment SGI135595[] = {{&S19834[44], 20}, {&S19834[130], 26}, {NULL}};
const Segment SGI135596[] = {{&S19834[30], 6}, {NULL}};
const Segment SGI135597[] = {{&S19834[113], 3}, {NULL}};
const Segment SGI135598[] = {{&S19834[78], 6}, {NULL}};
const Segment SGI135599[] = {{&S19835[44], 24}, {NULL}};
const Segment SGI135600[] = {{&S19835[44], 36}, {NULL}};
const Segment SGI135601[] = {{&S19835[30], 6}, {NULL}};
const Segment SGI135602[] = {{&S19835[129], 3}, {NULL}};
const Segment SGI135603[] = {{&S19835[94], 6}, {NULL}};
const Segment SGI135604[] = {{&S19836[44], 26}, {NULL}};
const Segment SGI135605[] = {{&S19836[44], 38}, {NULL}};
const Segment SGI135606[] = {{&S19836[30], 6}, {NULL}};
const Segment SGI135607[] = {{&S19836[131], 3}, {NULL}};
const Segment SGI135608[] = {{&S19836[96], 6}, {NULL}};
const Segment SGI135609[] = {{&S19838[38], 25}, {NULL}};
const Segment SGI135610[] = {{&S19838[31], 6}, {NULL}};
const Segment SGI135611[] = {{&S19839[38], 28}, {NULL}};
const Segment SGI135612[] = {{&S19839[31], 6}, {NULL}};
const Segment SGI135613[] = {{&S22490[32], 7}, {NULL}};
const Segment SGI135614[] = {{&S22490[27], 4}, {NULL}};
const Segment SGI135615[] = {{&S22490[92], 9}, {NULL}};
const Segment SGI135616[] = {{&S22490[65], 6}, {NULL}};
const Segment SGI135617[] = {{&S19842[44], 5}, {NULL}};
const Segment SGI135618[] = {{&S19842[44], 17}, {NULL}};
const Segment SGI135619[] = {{&S19842[30], 6}, {NULL}};
const Segment SGI135620[] = {{&S19842[110], 3}, {NULL}};
const Segment SGI135621[] = {{&S19842[75], 6}, {NULL}};
const Segment SGI135622[] = {{&S19843[44], 18}, {NULL}};
const Segment SGI135623[] = {{&S19843[44], 30}, {NULL}};
const Segment SGI135624[] = {{&S19843[30], 6}, {NULL}};
const Segment SGI135625[] = {{&S19843[123], 3}, {NULL}};
const Segment SGI135626[] = {{&S19843[88], 6}, {NULL}};
const Segment SGI135627[] = {{&S19845[44], 17}, {NULL}};
const Segment SGI135628[] = {{&S19845[44], 29}, {NULL}};
const Segment SGI135629[] = {{&S19845[30], 6}, {NULL}};
const Segment SGI135630[] = {{&S19845[122], 3}, {NULL}};
const Segment SGI135631[] = {{&S19845[87], 6}, {NULL}};
const Segment SGI135632[] = {{&S19846[44], 16}, {NULL}};
const Segment SGI135633[] = {{&S19846[44], 28}, {NULL}};
const Segment SGI135634[] = {{&S19846[30], 6}, {NULL}};
const Segment SGI135635[] = {{&S19846[121], 3}, {NULL}};
const Segment SGI135636[] = {{&S19846[86], 6}, {NULL}};
const Segment SGI135637[] = {{&S19847[38], 25}, {NULL}};
const Segment SGI135638[] = {{&S19847[31], 6}, {NULL}};
const Segment SGI135639[] = {{&S19848[44], 8}, {NULL}};
const Segment SGI135640[] = {{&S19848[44], 20}, {NULL}};
const Segment SGI135641[] = {{&S19848[30], 6}, {NULL}};
const Segment SGI135642[] = {{&S19848[112], 3}, {NULL}};
const Segment SGI135643[] = {{&S19848[78], 5}, {NULL}};
const Segment SGI135644[] = {{&S19849[38], 21}, {NULL}};
const Segment SGI135645[] = {{&S19849[31], 6}, {NULL}};
const Segment SGI135646[] = {{&S19850[44], 19}, {NULL}};
const Segment SGI135647[] = {{&S19850[44], 31}, {NULL}};
const Segment SGI135648[] = {{&S19850[30], 6}, {NULL}};
const Segment SGI135649[] = {{&S19850[124], 3}, {NULL}};
const Segment SGI135650[] = {{&S19850[89], 6}, {NULL}};
const Segment SGI135651[] = {{&S19851[44], 5}, {NULL}};
const Segment SGI135652[] = {{&S19851[44], 17}, {NULL}};
const Segment SGI135653[] = {{&S19851[30], 6}, {NULL}};
const Segment SGI135654[] = {{&S19851[110], 3}, {NULL}};
const Segment SGI135655[] = {{&S19851[75], 6}, {NULL}};
const Segment SGI135656[] = {{&S19852[11], 1}, {NULL}};
const Segment SGI135657[] = {{&S19852[44], 13}, {NULL}};
const Segment SGI135658[] = {{&S19852[30], 6}, {NULL}};
const Segment SGI135659[] = {{&S19852[106], 3}, {NULL}};
const Segment SGI135660[] = {{&S19852[71], 6}, {NULL}};
const Segment SGI135661[] = {{&S19853[38], 22}, {NULL}};
const Segment SGI135662[] = {{&S19853[31], 6}, {NULL}};
const Segment SGI135663[] = {{&S19854[38], 20}, {NULL}};
const Segment SGI135664[] = {{&S19854[31], 6}, {NULL}};
const Segment SGI135665[] = {{&S19855[22], 9}, {NULL}};
const Segment SGI135666[] = {{&S19855[44], 21}, {NULL}};
const Segment SGI135667[] = {{&S19855[30], 6}, {&S19855[52], 13}, {NULL}};
const Segment SGI135668[] = {{&S19855[114], 3}, {NULL}};
const Segment SGI135669[] = {{&S19855[79], 6}, {NULL}};
const Segment SGI135670[] = {{&S19857[44], 32}, {NULL}};
const Segment SGI135671[] = {{&S19857[44], 44}, {NULL}};
const Segment SGI135672[] = {{&S19857[30], 6}, {NULL}};
const Segment SGI135673[] = {{&S19857[137], 3}, {NULL}};
const Segment SGI135674[] = {{&S19857[102], 6}, {NULL}};
const Segment SGI135675[] = {{&S19858[44], 14}, {NULL}};
const Segment SGI135676[] = {{&S19858[44], 26}, {NULL}};
const Segment SGI135677[] = {{&S19858[30], 6}, {NULL}};
const Segment SGI135678[] = {{&S19858[119], 3}, {NULL}};
const Segment SGI135679[] = {{&S19858[84], 6}, {NULL}};
const Segment SGI135680[] = {{&S19860[38], 20}, {NULL}};
const Segment SGI135681[] = {{&S19860[31], 6}, {NULL}};
const Segment SGI135682[] = {{&S20145[34], 10}, {NULL}};
const Segment SGI135683[] = {{&S20145[27], 6}, {NULL}};
const Segment SGI135684[] = {{&S20145[97], 9}, {NULL}};
const Segment SGI135685[] = {{&S20145[70], 6}, {NULL}};
const Segment SGI135686[] = {{&S20146[34], 10}, {NULL}};
const Segment SGI135687[] = {{&S20146[27], 6}, {NULL}};
const Segment SGI135688[] = {{&S20146[97], 9}, {NULL}};
const Segment SGI135689[] = {{&S20146[70], 6}, {NULL}};
const Segment SGI135690[] = {{&S20147[34], 10}, {NULL}};
const Segment SGI135691[] = {{&S20147[27], 6}, {NULL}};
const Segment SGI135692[] = {{&S20147[97], 9}, {NULL}};
const Segment SGI135693[] = {{&S20147[70], 6}, {NULL}};
const Segment SGI135694[] = {{&S20148[34], 10}, {NULL}};
const Segment SGI135695[] = {{&S20148[27], 6}, {NULL}};
const Segment SGI135696[] = {{&S20148[97], 9}, {NULL}};
const Segment SGI135697[] = {{&S20148[70], 6}, {NULL}};
const Segment SGI135698[] = {{&S20149[34], 18}, {NULL}};
const Segment SGI135699[] = {{&S20149[27], 6}, {NULL}};
const Segment SGI135700[] = {{&S20149[105], 9}, {NULL}};
const Segment SGI135701[] = {{&S20149[78], 6}, {NULL}};
const Segment SGI135702[] = {{&S20150[34], 11}, {NULL}};
const Segment SGI135703[] = {{&S20150[27], 6}, {NULL}};
const Segment SGI135704[] = {{&S20150[98], 9}, {NULL}};
const Segment SGI135705[] = {{&S20150[71], 6}, {NULL}};
const Segment SGI135706[] = {{&S20151[34], 11}, {NULL}};
const Segment SGI135707[] = {{&S20151[27], 6}, {NULL}};
const Segment SGI135708[] = {{&S20151[98], 9}, {NULL}};
const Segment SGI135709[] = {{&S20151[71], 6}, {NULL}};
const Segment SGI135710[] = {{&S20152[34], 11}, {NULL}};
const Segment SGI135711[] = {{&S20152[27], 6}, {NULL}};
const Segment SGI135712[] = {{&S20152[98], 9}, {NULL}};
const Segment SGI135713[] = {{&S20152[71], 6}, {NULL}};
const Segment SGI135714[] = {{&S20153[34], 11}, {NULL}};
const Segment SGI135715[] = {{&S20153[27], 6}, {NULL}};
const Segment SGI135716[] = {{&S20153[98], 9}, {NULL}};
const Segment SGI135717[] = {{&S20153[71], 6}, {NULL}};
const Segment SGI135718[] = {{&S20154[34], 10}, {NULL}};
const Segment SGI135719[] = {{&S20154[27], 6}, {NULL}};
const Segment SGI135720[] = {{&S20154[97], 9}, {NULL}};
const Segment SGI135721[] = {{&S20154[70], 6}, {NULL}};
const Segment SGI135722[] = {{&S20155[34], 10}, {NULL}};
const Segment SGI135723[] = {{&S20155[27], 6}, {NULL}};
const Segment SGI135724[] = {{&S20155[97], 9}, {NULL}};
const Segment SGI135725[] = {{&S20155[70], 6}, {NULL}};
const Segment SGI135726[] = {{&S20156[34], 11}, {NULL}};
const Segment SGI135727[] = {{&S20156[27], 6}, {NULL}};
const Segment SGI135728[] = {{&S20156[98], 9}, {NULL}};
const Segment SGI135729[] = {{&S20156[71], 6}, {NULL}};
const Segment SGI135730[] = {{&S20158[34], 8}, {NULL}};
const Segment SGI135731[] = {{&S20158[27], 6}, {NULL}};
const Segment SGI135732[] = {{&S20158[95], 9}, {NULL}};
const Segment SGI135733[] = {{&S20158[68], 6}, {NULL}};
const Segment SGI135734[] = {{&S20159[34], 11}, {NULL}};
const Segment SGI135735[] = {{&S20159[27], 6}, {NULL}};
const Segment SGI135736[] = {{&S20159[98], 9}, {NULL}};
const Segment SGI135737[] = {{&S20159[71], 6}, {NULL}};
const Segment SGI135738[] = {{&S20163[34], 11}, {NULL}};
const Segment SGI135739[] = {{&S20163[27], 6}, {NULL}};
const Segment SGI135740[] = {{&S20163[98], 9}, {NULL}};
const Segment SGI135741[] = {{&S20163[71], 6}, {NULL}};
const Segment SGI135742[] = {{&S20173[34], 10}, {NULL}};
const Segment SGI135743[] = {{&S20173[27], 6}, {NULL}};
const Segment SGI135744[] = {{&S20173[97], 9}, {NULL}};
const Segment SGI135745[] = {{&S20173[70], 6}, {NULL}};
const Segment SGI135746[] = {{&S20177[44], 10}, {NULL}};
const Segment SGI135747[] = {{&S20177[27], 16}, {NULL}};
const Segment SGI135748[] = {{&S20177[107], 9}, {NULL}};
const Segment SGI135749[] = {{&S20177[80], 6}, {NULL}};
const Segment SGI135750[] = {{&S29865[44], 7}, {NULL}};
const Segment SGI135751[] = {{&S29865[44], 24}, {NULL}};
const Segment SGI135752[] = {{&S29865[30], 6}, {NULL}};
const Segment SGI135753[] = {{&S29865[31], 3}, {NULL}};
const Segment SGI135754[] = {{&S29865[82], 6}, {NULL}};
const Segment SGI135755[] = {{&S29867[44], 7}, {NULL}};
const Segment SGI135756[] = {{&S29867[44], 24}, {NULL}};
const Segment SGI135757[] = {{&S29867[30], 6}, {NULL}};
const Segment SGI135758[] = {{&S29867[31], 3}, {NULL}};
const Segment SGI135759[] = {{&S29867[82], 6}, {NULL}};
const Segment SGI135760[] = {{&S29869[38], 7}, {NULL}};
const Segment SGI135761[] = {{&S29869[31], 6}, {NULL}};
const Segment SGI135762[] = {{&S29870[44], 7}, {NULL}};
const Segment SGI135763[] = {{&S29870[44], 24}, {NULL}};
const Segment SGI135764[] = {{&S29870[30], 6}, {NULL}};
const Segment SGI135765[] = {{&S29870[31], 3}, {NULL}};
const Segment SGI135766[] = {{&S29870[82], 6}, {NULL}};
const Segment SGI135767[] = {{&S29871[44], 7}, {NULL}};
const Segment SGI135768[] = {{&S29871[44], 24}, {NULL}};
const Segment SGI135769[] = {{&S29871[30], 6}, {NULL}};
const Segment SGI135770[] = {{&S29871[85], 6}, {NULL}};
const Segment SGI135771[] = {{&S29874[44], 7}, {NULL}};
const Segment SGI135772[] = {{&S29874[44], 24}, {NULL}};
const Segment SGI135773[] = {{&S29874[30], 6}, {NULL}};
const Segment SGI135774[] = {{&S29874[85], 6}, {NULL}};
const Segment SGI135775[] = {{&S29875[44], 7}, {NULL}};
const Segment SGI135776[] = {{&S29875[44], 24}, {NULL}};
const Segment SGI135777[] = {{&S29875[30], 6}, {NULL}};
const Segment SGI135778[] = {{&S29875[31], 3}, {NULL}};
const Segment SGI135779[] = {{&S29875[82], 6}, {NULL}};
const Segment SGI135780[] = {{&S29876[44], 7}, {NULL}};
const Segment SGI135781[] = {{&S29876[44], 23}, {NULL}};
const Segment SGI135782[] = {{&S29876[30], 6}, {NULL}};
const Segment SGI135783[] = {{&S29876[31], 3}, {NULL}};
const Segment SGI135784[] = {{&S29876[81], 6}, {NULL}};
const Segment SGI135785[] = {{&S29877[38], 7}, {NULL}};
const Segment SGI135786[] = {{&S29877[31], 6}, {NULL}};
const Segment SGI135787[] = {{&S30045[44], 11}, {NULL}};
const Segment SGI135788[] = {{&S30045[44], 23}, {NULL}};
const Segment SGI135789[] = {{&S30045[30], 6}, {NULL}};
const Segment SGI135790[] = {{&S30045[31], 3}, {NULL}};
const Segment SGI135791[] = {{&S30045[81], 6}, {NULL}};
const Segment SGI135792[] = {{&S30050[44], 12}, {NULL}};
const Segment SGI135793[] = {{&S30050[44], 24}, {NULL}};
const Segment SGI135794[] = {{&S30050[30], 6}, {NULL}};
const Segment SGI135795[] = {{&S30050[31], 3}, {NULL}};
const Segment SGI135796[] = {{&S30050[82], 6}, {NULL}};
const Segment SGI135797[] = {{&S30052[44], 7}, {NULL}};
const Segment SGI135798[] = {{&S30052[44], 19}, {NULL}};
const Segment SGI135799[] = {{&S30052[30], 6}, {NULL}};
const Segment SGI135800[] = {{&S30052[31], 3}, {NULL}};
const Segment SGI135801[] = {{&S30052[77], 6}, {NULL}};
const Segment SGI135802[] = {{&S30055[44], 7}, {NULL}};
const Segment SGI135803[] = {{&S30055[44], 19}, {NULL}};
const Segment SGI135804[] = {{&S30055[30], 6}, {NULL}};
const Segment SGI135805[] = {{&S30055[31], 3}, {NULL}};
const Segment SGI135806[] = {{&S30055[77], 6}, {NULL}};
const Segment SGI135807[] = {{&S30056[44], 15}, {NULL}};
const Segment SGI135808[] = {{&S30056[44], 43}, {NULL}};
const Segment SGI135809[] = {{&S30056[30], 6}, {NULL}};
const Segment SGI135810[] = {{&S30056[31], 3}, {NULL}};
const Segment SGI135811[] = {{&S30056[101], 6}, {NULL}};
const Segment SGI135812[] = {{&S30057[44], 15}, {NULL}};
const Segment SGI135813[] = {{&S30057[44], 43}, {NULL}};
const Segment SGI135814[] = {{&S30057[30], 6}, {NULL}};
const Segment SGI135815[] = {{&S30057[31], 3}, {NULL}};
const Segment SGI135816[] = {{&S30057[101], 6}, {NULL}};
const Segment SGI135817[] = {{&S30058[44], 15}, {NULL}};
const Segment SGI135818[] = {{&S30058[44], 43}, {NULL}};
const Segment SGI135819[] = {{&S30058[30], 6}, {NULL}};
const Segment SGI135820[] = {{&S30058[31], 3}, {NULL}};
const Segment SGI135821[] = {{&S30058[101], 6}, {NULL}};
const Segment SGI135822[] = {{&S30059[44], 12}, {NULL}};
const Segment SGI135823[] = {{&S30059[44], 40}, {NULL}};
const Segment SGI135824[] = {{&S30059[30], 6}, {NULL}};
const Segment SGI135825[] = {{&S30059[31], 3}, {NULL}};
const Segment SGI135826[] = {{&S30059[98], 6}, {NULL}};
const Segment SGI135827[] = {{&S30060[44], 15}, {NULL}};
const Segment SGI135828[] = {{&S30060[44], 43}, {NULL}};
const Segment SGI135829[] = {{&S30060[30], 6}, {NULL}};
const Segment SGI135830[] = {{&S30060[31], 3}, {NULL}};
const Segment SGI135831[] = {{&S30060[101], 6}, {NULL}};
const Segment SGI135832[] = {{&S30061[38], 12}, {NULL}};
const Segment SGI135833[] = {{&S30061[31], 6}, {NULL}};
const Segment SGI135834[] = {{&S30062[44], 12}, {NULL}};
const Segment SGI135835[] = {{&S30062[44], 40}, {NULL}};
const Segment SGI135836[] = {{&S30062[30], 6}, {NULL}};
const Segment SGI135837[] = {{&S30062[31], 3}, {NULL}};
const Segment SGI135838[] = {{&S30062[98], 6}, {NULL}};
const Segment SGI135839[] = {{&S30063[44], 15}, {NULL}};
const Segment SGI135840[] = {{&S30063[44], 43}, {NULL}};
const Segment SGI135841[] = {{&S30063[30], 6}, {NULL}};
const Segment SGI135842[] = {{&S30063[31], 3}, {NULL}};
const Segment SGI135843[] = {{&S30063[101], 6}, {NULL}};
const Segment SGI135844[] = {{&S30064[44], 12}, {NULL}};
const Segment SGI135845[] = {{&S30064[44], 40}, {NULL}};
const Segment SGI135846[] = {{&S30064[30], 6}, {NULL}};
const Segment SGI135847[] = {{&S30064[31], 3}, {NULL}};
const Segment SGI135848[] = {{&S30064[98], 6}, {NULL}};
const Segment SGI135849[] = {{&S30065[44], 15}, {NULL}};
const Segment SGI135850[] = {{&S30065[44], 43}, {NULL}};
const Segment SGI135851[] = {{&S30065[30], 6}, {NULL}};
const Segment SGI135852[] = {{&S30065[31], 3}, {NULL}};
const Segment SGI135853[] = {{&S30065[101], 6}, {NULL}};
const Segment SGI135854[] = {{&S30066[42], 12}, {NULL}};
const Segment SGI135855[] = {{&S30066[30], 6}, {NULL}};
const Segment SGI135856[] = {{&S30066[31], 3}, {NULL}};
const Segment SGI135857[] = {{&S30066[96], 6}, {NULL}};
const Segment SGI135858[] = {{&S30067[44], 12}, {NULL}};
const Segment SGI135859[] = {{&S30067[44], 29}, {NULL}};
const Segment SGI135860[] = {{&S30067[30], 6}, {NULL}};
const Segment SGI135861[] = {{&S30067[31], 3}, {NULL}};
const Segment SGI135862[] = {{&S30067[87], 6}, {NULL}};
const Segment SGI135863[] = {{&S30068[44], 12}, {NULL}};
const Segment SGI135864[] = {{&S30068[44], 40}, {NULL}};
const Segment SGI135865[] = {{&S30068[30], 6}, {NULL}};
const Segment SGI135866[] = {{&S30068[31], 3}, {NULL}};
const Segment SGI135867[] = {{&S30068[98], 6}, {NULL}};
const Segment SGI135868[] = {{&S30070[44], 12}, {NULL}};
const Segment SGI135869[] = {{&S30070[44], 40}, {NULL}};
const Segment SGI135870[] = {{&S30070[30], 6}, {NULL}};
const Segment SGI135871[] = {{&S30070[31], 3}, {NULL}};
const Segment SGI135872[] = {{&S30070[98], 6}, {NULL}};
const Segment SGI135873[] = {{&S30071[44], 12}, {NULL}};
const Segment SGI135874[] = {{&S30071[44], 40}, {NULL}};
const Segment SGI135875[] = {{&S30071[30], 6}, {NULL}};
const Segment SGI135876[] = {{&S30071[31], 3}, {NULL}};
const Segment SGI135877[] = {{&S30071[98], 6}, {NULL}};
const Segment SGI135878[] = {{&S30072[44], 15}, {NULL}};
const Segment SGI135879[] = {{&S30072[44], 43}, {NULL}};
const Segment SGI135880[] = {{&S30072[30], 6}, {NULL}};
const Segment SGI135881[] = {{&S30072[31], 3}, {NULL}};
const Segment SGI135882[] = {{&S30072[101], 6}, {NULL}};
const Segment SGI135883[] = {{&S30073[44], 12}, {NULL}};
const Segment SGI135884[] = {{&S30073[44], 40}, {NULL}};
const Segment SGI135885[] = {{&S30073[30], 6}, {NULL}};
const Segment SGI135886[] = {{&S30073[98], 6}, {NULL}};
const Segment SGI135887[] = {{&S30074[44], 15}, {NULL}};
const Segment SGI135888[] = {{&S30074[44], 43}, {NULL}};
const Segment SGI135889[] = {{&S30074[30], 6}, {NULL}};
const Segment SGI135890[] = {{&S30074[31], 3}, {NULL}};
const Segment SGI135891[] = {{&S30074[101], 6}, {NULL}};
const Segment SGI135892[] = {{&S30075[44], 15}, {NULL}};
const Segment SGI135893[] = {{&S30075[44], 43}, {NULL}};
const Segment SGI135894[] = {{&S30075[30], 6}, {NULL}};
const Segment SGI135895[] = {{&S30075[31], 3}, {NULL}};
const Segment SGI135896[] = {{&S30075[101], 6}, {NULL}};
const Segment SGI135897[] = {{&S30076[55], 21}, {NULL}};
const Segment SGI135898[] = {{&S30076[48], 6}, {NULL}};
const Segment SGI135899[] = {{&S30077[42], 15}, {NULL}};
const Segment SGI135900[] = {{&S30077[30], 6}, {NULL}};
const Segment SGI135901[] = {{&S30077[31], 3}, {NULL}};
const Segment SGI135902[] = {{&S30077[99], 6}, {NULL}};
const Segment SGI135903[] = {{&S30078[60], 12}, {NULL}};
const Segment SGI135904[] = {{&S30078[60], 40}, {NULL}};
const Segment SGI135905[] = {{&S30078[46], 6}, {NULL}};
const Segment SGI135906[] = {{&S30078[47], 3}, {NULL}};
const Segment SGI135907[] = {{&S30078[119], 5}, {NULL}};
const Segment SGI135908[] = {{&S30079[44], 7}, {NULL}};
const Segment SGI135909[] = {{&S30079[44], 15}, {NULL}};
const Segment SGI135910[] = {{&S30079[30], 6}, {NULL}};
const Segment SGI135911[] = {{&S30079[31], 3}, {NULL}};
const Segment SGI135912[] = {{&S30079[73], 6}, {NULL}};
const Segment SGI135913[] = {{&S20625[44], 10}, {NULL}};
const Segment SGI135914[] = {{&S20625[44], 22}, {NULL}};
const Segment SGI135915[] = {{&S20625[30], 6}, {NULL}};
const Segment SGI135916[] = {{&S20625[115], 3}, {NULL}};
const Segment SGI135917[] = {{&S20625[80], 6}, {NULL}};
const Segment SGI135918[] = {{&S20626[38], 10}, {NULL}};
const Segment SGI135919[] = {{&S20626[31], 6}, {NULL}};
const Segment SGI135920[] = {{&S20628[42], 10}, {NULL}};
const Segment SGI135921[] = {{&S20628[30], 6}, {NULL}};
const Segment SGI135922[] = {{&S20628[113], 3}, {NULL}};
const Segment SGI135923[] = {{&S20628[78], 6}, {NULL}};
const Segment SGI135924[] = {{&S20630[44], 10}, {NULL}};
const Segment SGI135925[] = {{&S20630[44], 22}, {NULL}};
const Segment SGI135926[] = {{&S20630[30], 6}, {NULL}};
const Segment SGI135927[] = {{&S20630[115], 3}, {NULL}};
const Segment SGI135928[] = {{&S20630[80], 6}, {NULL}};
const Segment SGI135929[] = {{&S20631[44], 10}, {NULL}};
const Segment SGI135930[] = {{&S20631[44], 22}, {NULL}};
const Segment SGI135931[] = {{&S20631[30], 6}, {NULL}};
const Segment SGI135932[] = {{&S20631[115], 3}, {NULL}};
const Segment SGI135933[] = {{&S20631[80], 6}, {NULL}};
const Segment SGI135934[] = {{&S20634[44], 10}, {NULL}};
const Segment SGI135935[] = {{&S20634[44], 22}, {NULL}};
const Segment SGI135936[] = {{&S20634[30], 6}, {NULL}};
const Segment SGI135937[] = {{&S20634[116], 3}, {NULL}};
const Segment SGI135938[] = {{&S20634[80], 7}, {NULL}};
const Segment SGI135939[] = {{&S20634[136], 8}, {NULL}};
const Segment SGI135940[] = {{&S20635[44], 10}, {NULL}};
const Segment SGI135941[] = {{&S20635[44], 22}, {NULL}};
const Segment SGI135942[] = {{&S20635[30], 6}, {NULL}};
const Segment SGI135943[] = {{&S20635[8], 3}, {&S20635[129], 3}, {NULL}};
const Segment SGI135944[] = {{&S20635[80], 6}, {NULL}};
const Segment SGI135945[] = {{&S20636[44], 10}, {&S20636[175], 18}, {NULL}};
const Segment SGI135946[] = {{&S20636[44], 22}, {NULL}};
const Segment SGI135947[] = {{&S20636[30], 6}, {NULL}};
const Segment SGI135948[] = {{&S20636[115], 3}, {NULL}};
const Segment SGI135949[] = {{&S20636[80], 6}, {NULL}};
const Segment SGI135950[] = {{&S20637[44], 10}, {NULL}};
const Segment SGI135951[] = {{&S20637[44], 22}, {NULL}};
const Segment SGI135952[] = {{&S20637[30], 6}, {NULL}};
const Segment SGI135953[] = {{&S20637[115], 3}, {NULL}};
const Segment SGI135954[] = {{&S20637[80], 6}, {NULL}};
const Segment SGI135955[] = {{&S20638[46], 10}, {NULL}};
const Segment SGI135956[] = {{&S20638[46], 34}, {NULL}};
const Segment SGI135957[] = {{&S20638[32], 6}, {NULL}};
const Segment SGI135958[] = {{&S20640[47], 10}, {NULL}};
const Segment SGI135959[] = {{&S20640[47], 34}, {NULL}};
const Segment SGI135960[] = {{&S20640[33], 6}, {NULL}};
const Segment SGI135961[] = {{&S20641[44], 10}, {NULL}};
const Segment SGI135962[] = {{&S20641[44], 22}, {NULL}};
const Segment SGI135963[] = {{&S20641[30], 6}, {NULL}};
const Segment SGI135964[] = {{&S20641[83], 6}, {NULL}};
const Segment SGI135965[] = {{&S20642[44], 10}, {NULL}};
const Segment SGI135966[] = {{&S20642[44], 22}, {NULL}};
const Segment SGI135967[] = {{&S20642[30], 6}, {NULL}};
const Segment SGI135968[] = {{&S20642[127], 3}, {NULL}};
const Segment SGI135969[] = {{&S20642[80], 6}, {NULL}};
const Segment SGI135970[] = {{&S20645[64], 10}, {NULL}};
const Segment SGI135971[] = {{&S20645[64], 22}, {NULL}};
const Segment SGI135972[] = {{&S20645[50], 6}, {NULL}};
const Segment SGI135973[] = {{&S20646[38], 10}, {NULL}};
const Segment SGI135974[] = {{&S20646[111], 6}, {NULL}};
const Segment SGI135975[] = {{&S20646[31], 6}, {NULL}};
const Segment SGI135976[] = {{&S20648[43], 10}, {NULL}};
const Segment SGI135977[] = {{&S20648[43], 22}, {NULL}};
const Segment SGI135978[] = {{&S20648[114], 3}, {NULL}};
const Segment SGI135979[] = {{&S20648[79], 6}, {NULL}};
const Segment SGI135980[] = {{&S20649[44], 10}, {NULL}};
const Segment SGI135981[] = {{&S20649[44], 22}, {NULL}};
const Segment SGI135982[] = {{&S20649[30], 6}, {NULL}};
const Segment SGI135983[] = {{&S20649[80], 6}, {NULL}};
const Segment SGI135984[] = {{&S20651[44], 10}, {NULL}};
const Segment SGI135985[] = {{&S20651[44], 22}, {NULL}};
const Segment SGI135986[] = {{&S20651[30], 6}, {NULL}};
const Segment SGI135987[] = {{&S20651[115], 3}, {NULL}};
const Segment SGI135988[] = {{&S20651[80], 6}, {NULL}};
const Segment SGI135989[] = {{&S20651[135], 8}, {NULL}};
const Segment SGI135990[] = {{&S20653[44], 27}, {&S20653[79], 26}, {NULL}};
const Segment SGI135991[] = {{&S20653[30], 6}, {NULL}};
const Segment SGI135992[] = {{&S20653[115], 3}, {NULL}};
const Segment SGI135993[] = {{&S20655[44], 10}, {&S20655[44], 10}, {NULL}};
const Segment SGI135994[] = {{&S20655[44], 22}, {&S20655[44], 22}, {NULL}};
const Segment SGI135995[] = {{&S20655[30], 6}, {NULL}};
const Segment SGI135996[] = {{&S20655[150], 3}, {NULL}};
const Segment SGI135997[] = {{&S20655[115], 6}, {NULL}};
const Segment SGI135998[] = {{&S20656[44], 10}, {NULL}};
const Segment SGI135999[] = {{&S20656[44], 22}, {NULL}};
const Segment SGI136000[] = {{&S20656[30], 6}, {NULL}};
const Segment SGI136001[] = {{&S20656[80], 6}, {NULL}};
const Segment SGI136002[] = {{&S20657[44], 10}, {NULL}};
const Segment SGI136003[] = {{&S20657[44], 20}, {NULL}};
const Segment SGI136004[] = {{&S20657[30], 6}, {NULL}};
const Segment SGI136005[] = {{&S20658[44], 10}, {NULL}};
const Segment SGI136006[] = {{&S20658[44], 257}, {NULL}};
const Segment SGI136007[] = {{&S20658[30], 6}, {NULL}};
const Segment SGI136008[] = {{&S30259[44], 12}, {NULL}};
const Segment SGI136009[] = {{&S30259[44], 24}, {NULL}};
const Segment SGI136010[] = {{&S30259[30], 6}, {NULL}};
const Segment SGI136011[] = {{&S30259[31], 3}, {NULL}};
const Segment SGI136012[] = {{&S30259[82], 6}, {NULL}};
const Segment SGI136013[] = {{&S30261[44], 12}, {NULL}};
const Segment SGI136014[] = {{&S30261[44], 24}, {NULL}};
const Segment SGI136015[] = {{&S30261[30], 6}, {NULL}};
const Segment SGI136016[] = {{&S30261[31], 3}, {NULL}};
const Segment SGI136017[] = {{&S30261[82], 6}, {NULL}};
const Segment SGI136018[] = {{&S30263[44], 7}, {NULL}};
const Segment SGI136019[] = {{&S30263[44], 24}, {NULL}};
const Segment SGI136020[] = {{&S30263[30], 6}, {NULL}};
const Segment SGI136021[] = {{&S30263[31], 3}, {NULL}};
const Segment SGI136022[] = {{&S30263[82], 6}, {NULL}};
const Segment SGI136023[] = {{&S30265[38], 7}, {NULL}};
const Segment SGI136024[] = {{&S30265[31], 6}, {NULL}};
const Segment SGI136025[] = {{&S30267[44], 7}, {NULL}};
const Segment SGI136026[] = {{&S30267[44], 24}, {NULL}};
const Segment SGI136027[] = {{&S30267[30], 6}, {NULL}};
const Segment SGI136028[] = {{&S30267[31], 3}, {NULL}};
const Segment SGI136029[] = {{&S30267[82], 6}, {NULL}};
const Segment SGI136030[] = {{&S30268[44], 7}, {NULL}};
const Segment SGI136031[] = {{&S30268[44], 23}, {NULL}};
const Segment SGI136032[] = {{&S30268[30], 6}, {NULL}};
const Segment SGI136033[] = {{&S30268[31], 3}, {NULL}};
const Segment SGI136034[] = {{&S30268[81], 6}, {NULL}};
const Segment SGI136035[] = {{&S30300[44], 14}, {NULL}};
const Segment SGI136036[] = {{&S30300[44], 25}, {NULL}};
const Segment SGI136037[] = {{&S30300[30], 6}, {NULL}};
const Segment SGI136038[] = {{&S30300[31], 3}, {NULL}};
const Segment SGI136039[] = {{&S30300[83], 6}, {NULL}};
const Segment SGI136040[] = {{&S30301[44], 14}, {NULL}};
const Segment SGI136041[] = {{&S30301[44], 26}, {NULL}};
const Segment SGI136042[] = {{&S30301[30], 6}, {NULL}};
const Segment SGI136043[] = {{&S30301[31], 3}, {NULL}};
const Segment SGI136044[] = {{&S30301[84], 6}, {NULL}};
const Segment SGI136045[] = {{&S30304[44], 14}, {NULL}};
const Segment SGI136046[] = {{&S30304[44], 26}, {NULL}};
const Segment SGI136047[] = {{&S30304[30], 6}, {NULL}};
const Segment SGI136048[] = {{&S30304[31], 3}, {NULL}};
const Segment SGI136049[] = {{&S30304[84], 6}, {NULL}};
const Segment SGI136050[] = {{&S30305[44], 14}, {NULL}};
const Segment SGI136051[] = {{&S30305[44], 26}, {NULL}};
const Segment SGI136052[] = {{&S30305[30], 6}, {NULL}};
const Segment SGI136053[] = {{&S30305[31], 3}, {NULL}};
const Segment SGI136054[] = {{&S30305[84], 6}, {NULL}};
const Segment SGI136055[] = {{&S30306[44], 14}, {NULL}};
const Segment SGI136056[] = {{&S30306[44], 26}, {NULL}};
const Segment SGI136057[] = {{&S30306[30], 6}, {NULL}};
const Segment SGI136058[] = {{&S30306[31], 3}, {NULL}};
const Segment SGI136059[] = {{&S30306[84], 6}, {NULL}};
const Segment SGI136060[] = {{&S30307[44], 14}, {NULL}};
const Segment SGI136061[] = {{&S30307[44], 26}, {NULL}};
const Segment SGI136062[] = {{&S30307[30], 6}, {NULL}};
const Segment SGI136063[] = {{&S30307[84], 5}, {NULL}};
const Segment SGI136064[] = {{&S30308[44], 14}, {NULL}};
const Segment SGI136065[] = {{&S30308[44], 26}, {NULL}};
const Segment SGI136066[] = {{&S30308[30], 6}, {NULL}};
const Segment SGI136067[] = {{&S30308[31], 3}, {NULL}};
const Segment SGI136068[] = {{&S30308[84], 6}, {NULL}};
const Segment SGI136069[] = {{&S30309[44], 14}, {NULL}};
const Segment SGI136070[] = {{&S30309[44], 26}, {NULL}};
const Segment SGI136071[] = {{&S30309[30], 6}, {NULL}};
const Segment SGI136072[] = {{&S30309[84], 5}, {NULL}};
const Segment SGI136073[] = {{&S30310[44], 14}, {NULL}};
const Segment SGI136074[] = {{&S30310[44], 26}, {NULL}};
const Segment SGI136075[] = {{&S30310[30], 6}, {NULL}};
const Segment SGI136076[] = {{&S30310[31], 3}, {NULL}};
const Segment SGI136077[] = {{&S30310[84], 6}, {NULL}};
const Segment SGI136078[] = {{&S30311[44], 14}, {NULL}};
const Segment SGI136079[] = {{&S30311[44], 26}, {NULL}};
const Segment SGI136080[] = {{&S30311[30], 6}, {NULL}};
const Segment SGI136081[] = {{&S30311[84], 5}, {NULL}};
const Segment SGI136082[] = {{&S30313[44], 14}, {NULL}};
const Segment SGI136083[] = {{&S30313[44], 26}, {NULL}};
const Segment SGI136084[] = {{&S30313[30], 6}, {NULL}};
const Segment SGI136085[] = {{&S30313[31], 3}, {NULL}};
const Segment SGI136086[] = {{&S30313[84], 6}, {NULL}};
const Segment SGI136087[] = {{&S30314[44], 14}, {NULL}};
const Segment SGI136088[] = {{&S30314[44], 26}, {NULL}};
const Segment SGI136089[] = {{&S30314[30], 6}, {NULL}};
const Segment SGI136090[] = {{&S30314[31], 3}, {NULL}};
const Segment SGI136091[] = {{&S30314[84], 6}, {NULL}};
const Segment SGI136092[] = {{&S30315[44], 14}, {NULL}};
const Segment SGI136093[] = {{&S30315[44], 26}, {NULL}};
const Segment SGI136094[] = {{&S30315[30], 6}, {NULL}};
const Segment SGI136095[] = {{&S30315[31], 3}, {NULL}};
const Segment SGI136096[] = {{&S30315[84], 6}, {NULL}};
const Segment SGI136097[] = {{&S30316[44], 14}, {NULL}};
const Segment SGI136098[] = {{&S30316[44], 26}, {NULL}};
const Segment SGI136099[] = {{&S30316[30], 6}, {NULL}};
const Segment SGI136100[] = {{&S30316[84], 6}, {NULL}};
const Segment SGI136101[] = {{&S30319[44], 14}, {NULL}};
const Segment SGI136102[] = {{&S30319[44], 26}, {NULL}};
const Segment SGI136103[] = {{&S30319[30], 6}, {NULL}};
const Segment SGI136104[] = {{&S30319[31], 3}, {NULL}};
const Segment SGI136105[] = {{&S30319[84], 6}, {NULL}};
const Segment SGI136106[] = {{&S30321[38], 14}, {NULL}};
const Segment SGI136107[] = {{&S30321[31], 6}, {NULL}};
const Segment SGI136108[] = {{&S30324[44], 26}, {NULL}};
const Segment SGI136109[] = {{&S30324[44], 38}, {NULL}};
const Segment SGI136110[] = {{&S30324[30], 6}, {NULL}};
const Segment SGI136111[] = {{&S30324[31], 3}, {NULL}};
const Segment SGI136112[] = {{&S30324[96], 6}, {NULL}};
const Segment SGI136113[] = {{&S30325[51], 14}, {NULL}};
const Segment SGI136114[] = {{&S30325[51], 26}, {NULL}};
const Segment SGI136115[] = {{&S30325[30], 13}, {NULL}};
const Segment SGI136116[] = {{&S30325[31], 3}, {NULL}};
const Segment SGI136117[] = {{&S30325[91], 6}, {NULL}};
const Segment SGI136118[] = {{&S30326[44], 14}, {NULL}};
const Segment SGI136119[] = {{&S30326[44], 26}, {NULL}};
const Segment SGI136120[] = {{&S30326[30], 6}, {NULL}};
const Segment SGI136121[] = {{&S30326[31], 3}, {NULL}};
const Segment SGI136122[] = {{&S30326[84], 6}, {NULL}};
const Segment SGI136123[] = {{&S30328[44], 14}, {NULL}};
const Segment SGI136124[] = {{&S30328[44], 25}, {NULL}};
const Segment SGI136125[] = {{&S30328[30], 6}, {NULL}};
const Segment SGI136126[] = {{&S30328[31], 3}, {NULL}};
const Segment SGI136127[] = {{&S30328[83], 6}, {NULL}};
const Segment SGI136128[] = {{&S30329[44], 14}, {NULL}};
const Segment SGI136129[] = {{&S30329[44], 26}, {NULL}};
const Segment SGI136130[] = {{&S30329[30], 6}, {NULL}};
const Segment SGI136131[] = {{&S30329[31], 3}, {NULL}};
const Segment SGI136132[] = {{&S30329[84], 6}, {NULL}};
const Segment SGI136133[] = {{&S30330[44], 14}, {NULL}};
const Segment SGI136134[] = {{&S30330[44], 26}, {NULL}};
const Segment SGI136135[] = {{&S30330[30], 6}, {NULL}};
const Segment SGI136136[] = {{&S30330[84], 5}, {NULL}};
const Segment SGI136137[] = {{&S19702[44], 17}, {NULL}};
const Segment SGI136138[] = {{&S19702[44], 29}, {NULL}};
const Segment SGI136139[] = {{&S19702[30], 6}, {NULL}};
const Segment SGI136140[] = {{&S30331[44], 14}, {NULL}};
const Segment SGI136141[] = {{&S30331[44], 26}, {NULL}};
const Segment SGI136142[] = {{&S30331[30], 6}, {NULL}};
const Segment SGI136143[] = {{&S19702[122], 3}, {NULL}};
const Segment SGI136144[] = {{&S19702[87], 6}, {NULL}};
const Segment SGI136145[] = {{&S30331[31], 3}, {NULL}};
const Segment SGI136146[] = {{&S30331[84], 6}, {NULL}};
const Segment SGI136147[] = {{&S30332[44], 14}, {NULL}};
const Segment SGI136148[] = {{&S30332[44], 26}, {NULL}};
const Segment SGI136149[] = {{&S30332[30], 6}, {NULL}};
const Segment SGI136150[] = {{&S30332[31], 3}, {NULL}};
const Segment SGI136151[] = {{&S30332[84], 6}, {NULL}};
const Segment SGI136152[] = {{&S30335[44], 14}, {NULL}};
const Segment SGI136153[] = {{&S30335[44], 26}, {NULL}};
const Segment SGI136154[] = {{&S30335[30], 6}, {NULL}};
const Segment SGI136155[] = {{&S30335[31], 3}, {NULL}};
const Segment SGI136156[] = {{&S30335[84], 6}, {NULL}};
const Segment SGI136157[] = {{&S30337[44], 14}, {NULL}};
const Segment SGI136158[] = {{&S30337[44], 26}, {NULL}};
const Segment SGI136159[] = {{&S30337[30], 6}, {NULL}};
const Segment SGI136160[] = {{&S30337[87], 6}, {NULL}};
const Segment SGI136161[] = {{&S30338[44], 10}, {NULL}};
const Segment SGI136162[] = {{&S30338[44], 22}, {NULL}};
const Segment SGI136163[] = {{&S30338[30], 6}, {NULL}};
const Segment SGI136164[] = {{&S30338[31], 3}, {NULL}};
const Segment SGI136165[] = {{&S30338[80], 6}, {NULL}};
const Segment SGI136166[] = {{&S30343[44], 14}, {NULL}};
const Segment SGI136167[] = {{&S30343[44], 26}, {NULL}};
const Segment SGI136168[] = {{&S30343[30], 6}, {NULL}};
const Segment SGI136169[] = {{&S30343[87], 6}, {NULL}};
const Segment SGI136170[] = {{&S30344[44], 14}, {NULL}};
const Segment SGI136171[] = {{&S30344[44], 26}, {NULL}};
const Segment SGI136172[] = {{&S30344[30], 6}, {NULL}};
const Segment SGI136173[] = {{&S30344[87], 4}, {NULL}};
const Segment SGI136174[] = {{&S30344[87], 3}, {NULL}};
const Segment SGI136175[] = {{&S19721[44], 15}, {NULL}};
const Segment SGI136176[] = {{&S19721[44], 27}, {NULL}};
const Segment SGI136177[] = {{&S19721[30], 6}, {NULL}};
const Segment SGI136178[] = {{&S19721[120], 3}, {NULL}};
const Segment SGI136179[] = {{&S19721[85], 6}, {NULL}};
const Segment SGI136180[] = {{&S30345[44], 14}, {NULL}};
const Segment SGI136181[] = {{&S30345[44], 26}, {NULL}};
const Segment SGI136182[] = {{&S30345[30], 6}, {NULL}};
const Segment SGI136183[] = {{&S30345[84], 6}, {NULL}};
const Segment SGI136184[] = {{&S30347[44], 17}, {NULL}};
const Segment SGI136185[] = {{&S30347[44], 29}, {NULL}};
const Segment SGI136186[] = {{&S30347[30], 6}, {NULL}};
const Segment SGI136187[] = {{&S30347[31], 3}, {NULL}};
const Segment SGI136188[] = {{&S30347[87], 6}, {NULL}};
const Segment SGI136189[] = {{&S30350[44], 14}, {NULL}};
const Segment SGI136190[] = {{&S30350[44], 26}, {NULL}};
const Segment SGI136191[] = {{&S30350[30], 6}, {NULL}};
const Segment SGI136192[] = {{&S30350[87], 6}, {NULL}};
const Segment SGI136193[] = {{&S30352[44], 14}, {NULL}};
const Segment SGI136194[] = {{&S30352[44], 23}, {NULL}};
const Segment SGI136195[] = {{&S30352[30], 6}, {NULL}};
const Segment SGI136196[] = {{&S30352[31], 3}, {NULL}};
const Segment SGI136197[] = {{&S30352[81], 6}, {NULL}};
const Segment SGI136198[] = {{&S19735[38], 15}, {NULL}};
const Segment SGI136199[] = {{&S19735[31], 6}, {NULL}};
const Segment SGI136200[] = {{&S19746[44], 20}, {NULL}};
const Segment SGI136201[] = {{&S19746[44], 41}, {NULL}};
const Segment SGI136202[] = {{&S19746[30], 6}, {NULL}};
const Segment SGI136203[] = {{&S19746[134], 3}, {NULL}};
const Segment SGI136204[] = {{&S19746[99], 6}, {NULL}};
const Segment SGI136205[] = {{&S19767[44], 15}, {NULL}};
const Segment SGI136206[] = {{&S19767[44], 27}, {NULL}};
const Segment SGI136207[] = {{&S19767[30], 6}, {NULL}};
const Segment SGI136208[] = {{&S19767[120], 3}, {NULL}};
const Segment SGI136209[] = {{&S19767[85], 6}, {NULL}};
const Segment SGI136210[] = {{&S19787[44], 9}, {NULL}};
const Segment SGI136211[] = {{&S19787[44], 21}, {NULL}};
const Segment SGI136212[] = {{&S19787[30], 6}, {NULL}};
const Segment SGI136213[] = {{&S19787[114], 3}, {NULL}};
const Segment SGI136214[] = {{&S19787[79], 6}, {NULL}};
const Segment SGI136215[] = {{&S19789[44], 19}, {NULL}};
const Segment SGI136216[] = {{&S19789[44], 31}, {NULL}};
const Segment SGI136217[] = {{&S19789[30], 6}, {NULL}};
const Segment SGI136218[] = {{&S19789[124], 3}, {NULL}};
const Segment SGI136219[] = {{&S19789[89], 6}, {NULL}};
const Segment SGI136220[] = {{&S19802[44], 9}, {NULL}};
const Segment SGI136221[] = {{&S19802[44], 21}, {NULL}};
const Segment SGI136222[] = {{&S19802[30], 6}, {NULL}};
const Segment SGI136223[] = {{&S19802[114], 3}, {NULL}};
const Segment SGI136224[] = {{&S19802[79], 6}, {NULL}};
const Segment SGI136225[] = {{&S19803[44], 5}, {NULL}};
const Segment SGI136226[] = {{&S19803[44], 17}, {NULL}};
const Segment SGI136227[] = {{&S19803[30], 6}, {NULL}};
const Segment SGI136228[] = {{&S19803[110], 3}, {NULL}};
const Segment SGI136229[] = {{&S19803[75], 6}, {NULL}};
const Segment SGI136230[] = {{&S19807[44], 16}, {NULL}};
const Segment SGI136231[] = {{&S19807[44], 28}, {NULL}};
const Segment SGI136232[] = {{&S19807[30], 6}, {NULL}};
const Segment SGI136233[] = {{&S19807[121], 3}, {NULL}};
const Segment SGI136234[] = {{&S19807[86], 6}, {NULL}};
const Segment SGI136235[] = {{&S19810[44], 20}, {NULL}};
const Segment SGI136236[] = {{&S19810[44], 32}, {NULL}};
const Segment SGI136237[] = {{&S19810[30], 6}, {NULL}};
const Segment SGI136238[] = {{&S19810[125], 3}, {NULL}};
const Segment SGI136239[] = {{&S19810[90], 6}, {NULL}};
const Segment SGI136240[] = {{&S19811[44], 5}, {NULL}};
const Segment SGI136241[] = {{&S19811[44], 17}, {NULL}};
const Segment SGI136242[] = {{&S19811[30], 6}, {NULL}};
const Segment SGI136243[] = {{&S19811[110], 3}, {NULL}};
const Segment SGI136244[] = {{&S19811[75], 6}, {NULL}};
const Segment SGI136245[] = {{&S19819[44], 9}, {NULL}};
const Segment SGI136246[] = {{&S19819[44], 21}, {NULL}};
const Segment SGI136247[] = {{&S19819[30], 6}, {NULL}};
const Segment SGI136248[] = {{&S19819[114], 3}, {NULL}};
const Segment SGI136249[] = {{&S19819[79], 6}, {NULL}};
const Segment SGI136250[] = {{&S19820[38], 9}, {NULL}};
const Segment SGI136251[] = {{&S19820[31], 6}, {NULL}};
const Segment SGI136252[] = {{&S19828[44], 18}, {NULL}};
const Segment SGI136253[] = {{&S19828[44], 30}, {NULL}};
const Segment SGI136254[] = {{&S19828[30], 6}, {NULL}};
const Segment SGI136255[] = {{&S19828[123], 3}, {NULL}};
const Segment SGI136256[] = {{&S19828[88], 6}, {NULL}};
const Segment SGI136257[] = {{&S19832[44], 20}, {NULL}};
const Segment SGI136258[] = {{&S19832[44], 32}, {NULL}};
const Segment SGI136259[] = {{&S19832[30], 6}, {NULL}};
const Segment SGI136260[] = {{&S19832[125], 3}, {NULL}};
const Segment SGI136261[] = {{&S19832[90], 6}, {NULL}};
const Segment SGI136262[] = {{&S19840[44], 28}, {NULL}};
const Segment SGI136263[] = {{&S19840[44], 40}, {NULL}};
const Segment SGI136264[] = {{&S19840[30], 6}, {NULL}};
const Segment SGI136265[] = {{&S19840[133], 3}, {NULL}};
const Segment SGI136266[] = {{&S19840[98], 6}, {NULL}};
const Segment SGI136267[] = {{&S19844[44], 20}, {NULL}};
const Segment SGI136268[] = {{&S19844[44], 32}, {NULL}};
const Segment SGI136269[] = {{&S19844[30], 6}, {NULL}};
const Segment SGI136270[] = {{&S19844[125], 3}, {NULL}};
const Segment SGI136271[] = {{&S19844[90], 6}, {NULL}};
const Segment SGI136272[] = {{&S19856[34], 6}, {NULL}};
const Segment SGI136273[] = {{&S19859[11], 1}, {NULL}};
const Segment SGI136274[] = {{&S19859[44], 13}, {NULL}};
const Segment SGI136275[] = {{&S19859[30], 6}, {NULL}};
const Segment SGI136276[] = {{&S19859[106], 3}, {NULL}};
const Segment SGI136277[] = {{&S19859[71], 6}, {NULL}};
const Segment SGI136278[] = {{&S21033[44], 10}, {NULL}};
const Segment SGI136279[] = {{&S21033[44], 22}, {NULL}};
const Segment SGI136280[] = {{&S21033[30], 6}, {NULL}};
const Segment SGI136281[] = {{&S21033[115], 3}, {NULL}};
const Segment SGI136282[] = {{&S21033[80], 6}, {NULL}};
const Segment SGI136283[] = {{&S21034[44], 10}, {NULL}};
const Segment SGI136284[] = {{&S21034[44], 22}, {NULL}};
const Segment SGI136285[] = {{&S21034[30], 6}, {NULL}};
const Segment SGI136286[] = {{&S21034[115], 3}, {NULL}};
const Segment SGI136287[] = {{&S21034[80], 6}, {NULL}};
const Segment SGI136288[] = {{&S21036[44], 10}, {NULL}};
const Segment SGI136289[] = {{&S21036[44], 22}, {NULL}};
const Segment SGI136290[] = {{&S21036[30], 6}, {NULL}};
const Segment SGI136291[] = {{&S21036[115], 3}, {NULL}};
const Segment SGI136292[] = {{&S21036[80], 6}, {NULL}};
const Segment SGI136293[] = {{&S21037[44], 10}, {NULL}};
const Segment SGI136294[] = {{&S21037[44], 22}, {NULL}};
const Segment SGI136295[] = {{&S21037[30], 6}, {NULL}};
const Segment SGI136296[] = {{&S21037[115], 3}, {NULL}};
const Segment SGI136297[] = {{&S21037[80], 6}, {NULL}};
const Segment SGI136298[] = {{&S21039[38], 10}, {NULL}};
const Segment SGI136299[] = {{&S21039[31], 6}, {NULL}};
const Segment SGI136300[] = {{&S30355[44], 14}, {NULL}};
const Segment SGI136301[] = {{&S30355[44], 26}, {NULL}};
const Segment SGI136302[] = {{&S30355[30], 6}, {NULL}};
const Segment SGI136303[] = {{&S30355[31], 3}, {NULL}};
const Segment SGI136304[] = {{&S30355[84], 6}, {NULL}};
const Segment SGI136305[] = {{&S30357[44], 14}, {NULL}};
const Segment SGI136306[] = {{&S30357[44], 25}, {NULL}};
const Segment SGI136307[] = {{&S30357[30], 6}, {NULL}};
const Segment SGI136308[] = {{&S30357[31], 3}, {NULL}};
const Segment SGI136309[] = {{&S30357[83], 6}, {NULL}};
const Segment SGI136310[] = {{&S30424[44], 10}, {NULL}};
const Segment SGI136311[] = {{&S30424[44], 22}, {NULL}};
const Segment SGI136312[] = {{&S30424[30], 6}, {NULL}};
const Segment SGI136313[] = {{&S30424[31], 3}, {NULL}};
const Segment SGI136314[] = {{&S30424[80], 6}, {NULL}};
const Segment SGI136315[] = {{&S30425[38], 10}, {NULL}};
const Segment SGI136316[] = {{&S30425[31], 6}, {NULL}};
const Segment SGI136317[] = {{&S30426[42], 10}, {NULL}};
const Segment SGI136318[] = {{&S30426[30], 6}, {NULL}};
const Segment SGI136319[] = {{&S30426[31], 3}, {NULL}};
const Segment SGI136320[] = {{&S30426[78], 6}, {NULL}};
const Segment SGI136321[] = {{&S23939[42], 10}, {NULL}};
const Segment SGI136322[] = {{&S23939[42], 25}, {NULL}};
const Segment SGI136323[] = {{&S23939[30], 4}, {NULL}};
const Segment SGI136324[] = {{&S23939[116], 3}, {NULL}};
const Segment SGI136325[] = {{&S23939[81], 6}, {NULL}};
const Segment SGI136326[] = {{&S23939[136], 8}, {NULL}};
const Segment SGI136327[] = {{&S20157[34], 10}, {NULL}};
const Segment SGI136328[] = {{&S20157[27], 6}, {NULL}};
const Segment SGI136329[] = {{&S20157[97], 9}, {NULL}};
const Segment SGI136330[] = {{&S20157[70], 6}, {NULL}};
const Segment SGI136331[] = {{&S20160[34], 10}, {NULL}};
const Segment SGI136332[] = {{&S20160[27], 6}, {NULL}};
const Segment SGI136333[] = {{&S20160[97], 9}, {NULL}};
const Segment SGI136334[] = {{&S20160[70], 6}, {NULL}};
const Segment SGI136335[] = {{&S20161[34], 18}, {NULL}};
const Segment SGI136336[] = {{&S20161[27], 6}, {NULL}};
const Segment SGI136337[] = {{&S20161[105], 9}, {NULL}};
const Segment SGI136338[] = {{&S20161[78], 6}, {NULL}};
const Segment SGI136339[] = {{&S20162[34], 10}, {NULL}};
const Segment SGI136340[] = {{&S20162[27], 6}, {NULL}};
const Segment SGI136341[] = {{&S20162[97], 9}, {NULL}};
const Segment SGI136342[] = {{&S20162[70], 6}, {NULL}};
const Segment SGI136343[] = {{&S20164[34], 10}, {NULL}};
const Segment SGI136344[] = {{&S20164[27], 6}, {NULL}};
const Segment SGI136345[] = {{&S20164[97], 9}, {NULL}};
const Segment SGI136346[] = {{&S20164[70], 6}, {NULL}};
const Segment SGI136347[] = {{&S21465[44], 14}, {NULL}};
const Segment SGI136348[] = {{&S21465[44], 26}, {NULL}};
const Segment SGI136349[] = {{&S20165[34], 10}, {NULL}};
const Segment SGI136350[] = {{&S21465[30], 6}, {NULL}};
const Segment SGI136351[] = {{&S20165[27], 6}, {NULL}};
const Segment SGI136352[] = {{&S21465[119], 3}, {NULL}};
const Segment SGI136353[] = {{&S21465[84], 6}, {NULL}};
const Segment SGI136354[] = {{&S20165[97], 9}, {NULL}};
const Segment SGI136355[] = {{&S20165[70], 6}, {NULL}};
const Segment SGI136356[] = {{&S21466[38], 14}, {NULL}};
const Segment SGI136357[] = {{&S21466[31], 6}, {NULL}};
const Segment SGI136358[] = {{&S20166[34], 10}, {NULL}};
const Segment SGI136359[] = {{&S20166[27], 6}, {NULL}};
const Segment SGI136360[] = {{&S20166[97], 9}, {NULL}};
const Segment SGI136361[] = {{&S20166[70], 6}, {NULL}};
const Segment SGI136362[] = {{&S21467[44], 14}, {NULL}};
const Segment SGI136363[] = {{&S21467[44], 26}, {NULL}};
const Segment SGI136364[] = {{&S21467[30], 6}, {NULL}};
const Segment SGI136365[] = {{&S20167[34], 11}, {NULL}};
const Segment SGI136366[] = {{&S21467[119], 3}, {NULL}};
const Segment SGI136367[] = {{&S20167[27], 6}, {NULL}};
const Segment SGI136368[] = {{&S21467[84], 6}, {NULL}};
const Segment SGI136369[] = {{&S20167[98], 9}, {NULL}};
const Segment SGI136370[] = {{&S20167[71], 6}, {NULL}};
const Segment SGI136371[] = {{&S21468[44], 14}, {NULL}};
const Segment SGI136372[] = {{&S21468[44], 26}, {NULL}};
const Segment SGI136373[] = {{&S21468[30], 6}, {NULL}};
const Segment SGI136374[] = {{&S20168[34], 11}, {NULL}};
const Segment SGI136375[] = {{&S21468[119], 3}, {NULL}};
const Segment SGI136376[] = {{&S20168[27], 6}, {NULL}};
const Segment SGI136377[] = {{&S21468[84], 6}, {NULL}};
const Segment SGI136378[] = {{&S20168[98], 9}, {NULL}};
const Segment SGI136379[] = {{&S20168[71], 6}, {NULL}};
const Segment SGI136380[] = {{&S21469[44], 14}, {NULL}};
const Segment SGI136381[] = {{&S21469[44], 26}, {NULL}};
const Segment SGI136382[] = {{&S21469[30], 6}, {NULL}};
const Segment SGI136383[] = {{&S20169[34], 11}, {NULL}};
const Segment SGI136384[] = {{&S20169[27], 6}, {NULL}};
const Segment SGI136385[] = {{&S21469[119], 3}, {NULL}};
const Segment SGI136386[] = {{&S21469[84], 6}, {NULL}};
const Segment SGI136387[] = {{&S20169[98], 9}, {NULL}};
const Segment SGI136388[] = {{&S20169[71], 6}, {NULL}};
const Segment SGI136389[] = {{&S21470[44], 14}, {NULL}};
const Segment SGI136390[] = {{&S21470[44], 26}, {NULL}};
const Segment SGI136391[] = {{&S21470[30], 6}, {NULL}};
const Segment SGI136392[] = {{&S20170[34], 11}, {NULL}};
const Segment SGI136393[] = {{&S20170[27], 6}, {NULL}};
const Segment SGI136394[] = {{&S21470[119], 3}, {NULL}};
const Segment SGI136395[] = {{&S21470[84], 6}, {NULL}};
const Segment SGI136396[] = {{&S20170[98], 9}, {NULL}};
const Segment SGI136397[] = {{&S20170[71], 6}, {NULL}};
const Segment SGI136398[] = {{&S21471[44], 14}, {NULL}};
const Segment SGI136399[] = {{&S21471[44], 26}, {NULL}};
const Segment SGI136400[] = {{&S20171[39], 10}, {NULL}};
const Segment SGI136401[] = {{&S21471[30], 6}, {NULL}};
const Segment SGI136402[] = {{&S20171[27], 11}, {NULL}};
const Segment SGI136403[] = {{&S21471[84], 5}, {NULL}};
const Segment SGI136404[] = {{&S20171[102], 9}, {NULL}};
const Segment SGI136405[] = {{&S20171[75], 6}, {NULL}};
const Segment SGI136406[] = {{&S21473[44], 14}, {NULL}};
const Segment SGI136407[] = {{&S21473[44], 26}, {NULL}};
const Segment SGI136408[] = {{&S21473[30], 6}, {NULL}};
const Segment SGI136409[] = {{&S21473[119], 3}, {NULL}};
const Segment SGI136410[] = {{&S21473[84], 6}, {NULL}};
const Segment SGI136411[] = {{&S20174[34], 10}, {NULL}};
const Segment SGI136412[] = {{&S20174[27], 6}, {NULL}};
const Segment SGI136413[] = {{&S21474[44], 14}, {NULL}};
const Segment SGI136414[] = {{&S20174[97], 9}, {NULL}};
const Segment SGI136415[] = {{&S21474[44], 26}, {NULL}};
const Segment SGI136416[] = {{&S20174[70], 6}, {NULL}};
const Segment SGI136417[] = {{&S21474[30], 6}, {NULL}};
const Segment SGI136418[] = {{&S21474[119], 3}, {NULL}};
const Segment SGI136419[] = {{&S21474[84], 6}, {NULL}};
const Segment SGI136420[] = {{&S20175[34], 11}, {NULL}};
const Segment SGI136421[] = {{&S20175[27], 6}, {NULL}};
const Segment SGI136422[] = {{&S21475[44], 14}, {NULL}};
const Segment SGI136423[] = {{&S21475[44], 26}, {NULL}};
const Segment SGI136424[] = {{&S21475[30], 6}, {NULL}};
const Segment SGI136425[] = {{&S20175[98], 9}, {NULL}};
const Segment SGI136426[] = {{&S20175[71], 6}, {NULL}};
const Segment SGI136427[] = {{&S21475[119], 3}, {NULL}};
const Segment SGI136428[] = {{&S21475[84], 6}, {NULL}};
const Segment SGI136429[] = {{&S20176[34], 11}, {NULL}};
const Segment SGI136430[] = {{&S20176[27], 6}, {NULL}};
const Segment SGI136431[] = {{&S21476[44], 14}, {NULL}};
const Segment SGI136432[] = {{&S21476[44], 26}, {NULL}};
const Segment SGI136433[] = {{&S21476[30], 6}, {NULL}};
const Segment SGI136434[] = {{&S20176[98], 9}, {NULL}};
const Segment SGI136435[] = {{&S20176[71], 6}, {NULL}};
const Segment SGI136436[] = {{&S21476[119], 3}, {NULL}};
const Segment SGI136437[] = {{&S21476[84], 6}, {NULL}};
const Segment SGI136438[] = {{&S21477[44], 14}, {NULL}};
const Segment SGI136439[] = {{&S21477[44], 26}, {NULL}};
const Segment SGI136440[] = {{&S21477[30], 6}, {NULL}};
const Segment SGI136441[] = {{&S21477[119], 3}, {NULL}};
const Segment SGI136442[] = {{&S21477[84], 6}, {NULL}};
const Segment SGI136443[] = {{&S20178[34], 11}, {NULL}};
const Segment SGI136444[] = {{&S20178[27], 6}, {NULL}};
const Segment SGI136445[] = {{&S20178[98], 9}, {NULL}};
const Segment SGI136446[] = {{&S20178[71], 6}, {NULL}};
const Segment SGI136447[] = {{&S21478[44], 14}, {NULL}};
const Segment SGI136448[] = {{&S21478[44], 26}, {NULL}};
const Segment SGI136449[] = {{&S21478[30], 6}, {NULL}};
const Segment SGI136450[] = {{&S21478[87], 6}, {NULL}};
const Segment SGI136451[] = {{&S20179[34], 10}, {NULL}};
const Segment SGI136452[] = {{&S20179[27], 6}, {NULL}};
const Segment SGI136453[] = {{&S20179[97], 9}, {NULL}};
const Segment SGI136454[] = {{&S20179[70], 6}, {NULL}};
const Segment SGI136455[] = {{&S21479[44], 14}, {NULL}};
const Segment SGI136456[] = {{&S21479[44], 26}, {NULL}};
const Segment SGI136457[] = {{&S21479[30], 6}, {NULL}};
const Segment SGI136458[] = {{&S21479[119], 3}, {NULL}};
const Segment SGI136459[] = {{&S21479[84], 6}, {NULL}};
const Segment SGI136460[] = {{&S20180[34], 10}, {NULL}};
const Segment SGI136461[] = {{&S20180[27], 6}, {NULL}};
const Segment SGI136462[] = {{&S20180[97], 9}, {NULL}};
const Segment SGI136463[] = {{&S20180[70], 6}, {NULL}};
const Segment SGI136464[] = {{&S21480[44], 14}, {NULL}};
const Segment SGI136465[] = {{&S21480[44], 26}, {NULL}};
const Segment SGI136466[] = {{&S21480[30], 6}, {NULL}};
const Segment SGI136467[] = {{&S21480[84], 6}, {NULL}};
const Segment SGI136468[] = {{&S21481[44], 14}, {NULL}};
const Segment SGI136469[] = {{&S21481[44], 26}, {NULL}};
const Segment SGI136470[] = {{&S21481[30], 6}, {NULL}};
const Segment SGI136471[] = {{&S21481[119], 3}, {NULL}};
const Segment SGI136472[] = {{&S21481[84], 6}, {NULL}};
const Segment SGI136473[] = {{&S21482[44], 14}, {NULL}};
const Segment SGI136474[] = {{&S21482[44], 26}, {NULL}};
const Segment SGI136475[] = {{&S21482[30], 6}, {NULL}};
const Segment SGI136476[] = {{&S21482[119], 3}, {NULL}};
const Segment SGI136477[] = {{&S21482[84], 6}, {NULL}};
const Segment SGI136478[] = {{&S21489[44], 14}, {NULL}};
const Segment SGI136479[] = {{&S21489[44], 26}, {NULL}};
const Segment SGI136480[] = {{&S21489[30], 6}, {NULL}};
const Segment SGI136481[] = {{&S21489[84], 6}, {NULL}};
const Segment SGI136482[] = {{&S21491[44], 10}, {NULL}};
const Segment SGI136483[] = {{&S21491[44], 22}, {NULL}};
const Segment SGI136484[] = {{&S21491[30], 6}, {NULL}};
const Segment SGI136485[] = {{&S21491[115], 3}, {NULL}};
const Segment SGI136486[] = {{&S21491[80], 6}, {NULL}};
const Segment SGI136487[] = {{&S21492[44], 14}, {NULL}};
const Segment SGI136488[] = {{&S21492[44], 26}, {NULL}};
const Segment SGI136489[] = {{&S21492[30], 6}, {NULL}};
const Segment SGI136490[] = {{&S21492[84], 5}, {NULL}};
const Segment SGI136491[] = {{&S21493[42], 14}, {NULL}};
const Segment SGI136492[] = {{&S21493[30], 6}, {NULL}};
const Segment SGI136493[] = {{&S21493[117], 3}, {NULL}};
const Segment SGI136494[] = {{&S21493[82], 6}, {NULL}};
const Segment SGI136495[] = {{&S21496[44], 14}, {NULL}};
const Segment SGI136496[] = {{&S21496[44], 42}, {NULL}};
const Segment SGI136497[] = {{&S21496[30], 6}, {NULL}};
const Segment SGI136498[] = {{&S21496[135], 3}, {NULL}};
const Segment SGI136499[] = {{&S21496[100], 6}, {NULL}};
const Segment SGI136500[] = {{&S21497[38], 10}, {NULL}};
const Segment SGI136501[] = {{&S21497[31], 6}, {NULL}};
const Segment SGI136502[] = {{&S21595[44], 11}, {NULL}};
const Segment SGI136503[] = {{&S21595[44], 23}, {NULL}};
const Segment SGI136504[] = {{&S21595[30], 6}, {NULL}};
const Segment SGI136505[] = {{&S21595[116], 3}, {NULL}};
const Segment SGI136506[] = {{&S21595[81], 6}, {NULL}};
const Segment SGI136507[] = {{&S21596[44], 11}, {NULL}};
const Segment SGI136508[] = {{&S21596[44], 23}, {NULL}};
const Segment SGI136509[] = {{&S21596[30], 6}, {NULL}};
const Segment SGI136510[] = {{&S21596[116], 3}, {NULL}};
const Segment SGI136511[] = {{&S21596[81], 6}, {NULL}};
const Segment SGI136512[] = {{&S21598[44], 11}, {NULL}};
const Segment SGI136513[] = {{&S21598[44], 23}, {NULL}};
const Segment SGI136514[] = {{&S21598[30], 6}, {NULL}};
const Segment SGI136515[] = {{&S21598[116], 3}, {NULL}};
const Segment SGI136516[] = {{&S21598[81], 6}, {NULL}};
const Segment SGI136517[] = {{&S21599[38], 11}, {NULL}};
const Segment SGI136518[] = {{&S21599[31], 6}, {NULL}};
const Segment SGI136519[] = {{&S21600[44], 22}, {NULL}};
const Segment SGI136520[] = {{&S21600[44], 34}, {NULL}};
const Segment SGI136521[] = {{&S21600[30], 6}, {NULL}};
const Segment SGI136522[] = {{&S21600[127], 3}, {NULL}};
const Segment SGI136523[] = {{&S21600[92], 6}, {NULL}};
const Segment SGI136524[] = {{&S21601[44], 22}, {NULL}};
const Segment SGI136525[] = {{&S21601[44], 34}, {NULL}};
const Segment SGI136526[] = {{&S21601[30], 6}, {NULL}};
const Segment SGI136527[] = {{&S21601[127], 3}, {NULL}};
const Segment SGI136528[] = {{&S21601[92], 6}, {NULL}};
const Segment SGI136529[] = {{&S21602[44], 11}, {NULL}};
const Segment SGI136530[] = {{&S21602[44], 23}, {NULL}};
const Segment SGI136531[] = {{&S21602[30], 6}, {NULL}};
const Segment SGI136532[] = {{&S21602[116], 3}, {NULL}};
const Segment SGI136533[] = {{&S21602[81], 6}, {NULL}};
const Segment SGI136534[] = {{&S21603[44], 22}, {NULL}};
const Segment SGI136535[] = {{&S21603[44], 34}, {NULL}};
const Segment SGI136536[] = {{&S21603[30], 6}, {NULL}};
const Segment SGI136537[] = {{&S21603[127], 3}, {NULL}};
const Segment SGI136538[] = {{&S21603[92], 6}, {NULL}};
const Segment SGI136539[] = {{&S21604[44], 27}, {NULL}};
const Segment SGI136540[] = {{&S21604[44], 39}, {NULL}};
const Segment SGI136541[] = {{&S21604[30], 6}, {NULL}};
const Segment SGI136542[] = {{&S21604[132], 3}, {NULL}};
const Segment SGI136543[] = {{&S21604[97], 6}, {NULL}};
const Segment SGI136544[] = {{&S21605[44], 7}, {NULL}};
const Segment SGI136545[] = {{&S21605[44], 19}, {NULL}};
const Segment SGI136546[] = {{&S21605[30], 6}, {NULL}};
const Segment SGI136547[] = {{&S21605[112], 3}, {NULL}};
const Segment SGI136548[] = {{&S21605[77], 6}, {NULL}};
const Segment SGI136549[] = {{&S21607[44], 21}, {NULL}};
const Segment SGI136550[] = {{&S21607[44], 33}, {NULL}};
const Segment SGI136551[] = {{&S21607[30], 6}, {NULL}};
const Segment SGI136552[] = {{&S21607[126], 3}, {NULL}};
const Segment SGI136553[] = {{&S21607[91], 6}, {NULL}};
const Segment SGI136554[] = {{&S21608[44], 22}, {NULL}};
const Segment SGI136555[] = {{&S21608[44], 34}, {NULL}};
const Segment SGI136556[] = {{&S21608[30], 6}, {NULL}};
const Segment SGI136557[] = {{&S21608[127], 3}, {NULL}};
const Segment SGI136558[] = {{&S21608[92], 6}, {NULL}};
const Segment SGI136559[] = {{&S21609[44], 22}, {NULL}};
const Segment SGI136560[] = {{&S21609[44], 34}, {NULL}};
const Segment SGI136561[] = {{&S21609[30], 6}, {NULL}};
const Segment SGI136562[] = {{&S21609[127], 3}, {NULL}};
const Segment SGI136563[] = {{&S21609[92], 6}, {NULL}};
const Segment SGI136564[] = {{&S21610[44], 22}, {NULL}};
const Segment SGI136565[] = {{&S21610[44], 34}, {NULL}};
const Segment SGI136566[] = {{&S21610[30], 6}, {NULL}};
const Segment SGI136567[] = {{&S21610[127], 3}, {NULL}};
const Segment SGI136568[] = {{&S21610[92], 6}, {NULL}};
const Segment SGI136569[] = {{&S21611[44], 22}, {NULL}};
const Segment SGI136570[] = {{&S21611[44], 34}, {NULL}};
const Segment SGI136571[] = {{&S21611[30], 6}, {NULL}};
const Segment SGI136572[] = {{&S21611[127], 3}, {NULL}};
const Segment SGI136573[] = {{&S21611[92], 6}, {NULL}};
const Segment SGI136574[] = {{&S21612[44], 22}, {NULL}};
const Segment SGI136575[] = {{&S21612[44], 34}, {NULL}};
const Segment SGI136576[] = {{&S21612[30], 6}, {NULL}};
const Segment SGI136577[] = {{&S21612[127], 3}, {NULL}};
const Segment SGI136578[] = {{&S21612[92], 6}, {NULL}};
const Segment SGI136579[] = {{&S21613[44], 24}, {NULL}};
const Segment SGI136580[] = {{&S21613[44], 36}, {NULL}};
const Segment SGI136581[] = {{&S21613[30], 6}, {NULL}};
const Segment SGI136582[] = {{&S21613[129], 3}, {NULL}};
const Segment SGI136583[] = {{&S21613[94], 6}, {NULL}};
const Segment SGI136584[] = {{&S21614[44], 22}, {NULL}};
const Segment SGI136585[] = {{&S21614[44], 34}, {NULL}};
const Segment SGI136586[] = {{&S21614[30], 6}, {NULL}};
const Segment SGI136587[] = {{&S21614[127], 3}, {NULL}};
const Segment SGI136588[] = {{&S21614[92], 6}, {NULL}};
const Segment SGI136589[] = {{&S21615[44], 32}, {NULL}};
const Segment SGI136590[] = {{&S21615[44], 44}, {NULL}};
const Segment SGI136591[] = {{&S21615[30], 6}, {NULL}};
const Segment SGI136592[] = {{&S21615[137], 3}, {NULL}};
const Segment SGI136593[] = {{&S21615[102], 6}, {NULL}};
const Segment SGI136594[] = {{&S21616[44], 11}, {NULL}};
const Segment SGI136595[] = {{&S21616[44], 23}, {NULL}};
const Segment SGI136596[] = {{&S21616[30], 6}, {NULL}};
const Segment SGI136597[] = {{&S21616[84], 6}, {NULL}};
const Segment SGI136598[] = {{&S21617[44], 22}, {NULL}};
const Segment SGI136599[] = {{&S21617[44], 34}, {NULL}};
const Segment SGI136600[] = {{&S21617[30], 6}, {NULL}};
const Segment SGI136601[] = {{&S21617[127], 3}, {NULL}};
const Segment SGI136602[] = {{&S21617[92], 6}, {NULL}};
const Segment SGI136603[] = {{&S21618[44], 32}, {NULL}};
const Segment SGI136604[] = {{&S21618[44], 44}, {NULL}};
const Segment SGI136605[] = {{&S21618[30], 6}, {NULL}};
const Segment SGI136606[] = {{&S21618[137], 3}, {NULL}};
const Segment SGI136607[] = {{&S21618[102], 6}, {NULL}};
const Segment SGI136608[] = {{&S21619[44], 11}, {NULL}};
const Segment SGI136609[] = {{&S21619[44], 23}, {NULL}};
const Segment SGI136610[] = {{&S21619[30], 6}, {NULL}};
const Segment SGI136611[] = {{&S21619[116], 3}, {NULL}};
const Segment SGI136612[] = {{&S21619[81], 6}, {NULL}};
const Segment SGI136613[] = {{&S21620[44], 32}, {NULL}};
const Segment SGI136614[] = {{&S21620[44], 44}, {NULL}};
const Segment SGI136615[] = {{&S21620[30], 6}, {NULL}};
const Segment SGI136616[] = {{&S21620[137], 3}, {NULL}};
const Segment SGI136617[] = {{&S21620[102], 6}, {NULL}};
const Segment SGI136618[] = {{&S21621[44], 11}, {NULL}};
const Segment SGI136619[] = {{&S21621[44], 23}, {NULL}};
const Segment SGI136620[] = {{&S21621[30], 6}, {NULL}};
const Segment SGI136621[] = {{&S21621[116], 3}, {NULL}};
const Segment SGI136622[] = {{&S21621[81], 6}, {NULL}};
const Segment SGI136623[] = {{&S21622[44], 23}, {NULL}};
const Segment SGI136624[] = {{&S21622[44], 35}, {NULL}};
const Segment SGI136625[] = {{&S21622[30], 6}, {NULL}};
const Segment SGI136626[] = {{&S21622[128], 3}, {NULL}};
const Segment SGI136627[] = {{&S21622[93], 6}, {NULL}};
const Segment SGI136628[] = {{&S21623[38], 11}, {NULL}};
const Segment SGI136629[] = {{&S21623[31], 6}, {NULL}};
const Segment SGI136630[] = {{&S21624[38], 27}, {NULL}};
const Segment SGI136631[] = {{&S21624[31], 6}, {NULL}};
const Segment SGI136632[] = {{&S21625[44], 27}, {NULL}};
const Segment SGI136633[] = {{&S21625[44], 39}, {NULL}};
const Segment SGI136634[] = {{&S21625[30], 6}, {NULL}};
const Segment SGI136635[] = {{&S21625[132], 3}, {NULL}};
const Segment SGI136636[] = {{&S21625[97], 6}, {NULL}};
const Segment SGI136637[] = {{&S21626[42], 11}, {NULL}};
const Segment SGI136638[] = {{&S21626[30], 6}, {NULL}};
const Segment SGI136639[] = {{&S21626[114], 3}, {NULL}};
const Segment SGI136640[] = {{&S21626[79], 6}, {NULL}};
const Segment SGI136641[] = {{&S21627[44], 22}, {NULL}};
const Segment SGI136642[] = {{&S21627[44], 34}, {NULL}};
const Segment SGI136643[] = {{&S21627[30], 6}, {NULL}};
const Segment SGI136644[] = {{&S21627[127], 3}, {NULL}};
const Segment SGI136645[] = {{&S21627[92], 6}, {NULL}};
const Segment SGI136646[] = {{&S21628[44], 24}, {NULL}};
const Segment SGI136647[] = {{&S21628[44], 36}, {NULL}};
const Segment SGI136648[] = {{&S21628[30], 6}, {NULL}};
const Segment SGI136649[] = {{&S21628[129], 3}, {NULL}};
const Segment SGI136650[] = {{&S21628[94], 6}, {NULL}};
const Segment SGI136651[] = {{&S21629[44], 23}, {NULL}};
const Segment SGI136652[] = {{&S21629[44], 35}, {NULL}};
const Segment SGI136653[] = {{&S21629[30], 6}, {NULL}};
const Segment SGI136654[] = {{&S21629[128], 3}, {NULL}};
const Segment SGI136655[] = {{&S21629[93], 6}, {NULL}};
const Segment SGI136656[] = {{&S21630[44], 23}, {NULL}};
const Segment SGI136657[] = {{&S21630[44], 35}, {NULL}};
const Segment SGI136658[] = {{&S21630[30], 6}, {NULL}};
const Segment SGI136659[] = {{&S21630[128], 3}, {NULL}};
const Segment SGI136660[] = {{&S21630[93], 6}, {NULL}};
const Segment SGI136661[] = {{&S21631[42], 11}, {NULL}};
const Segment SGI136662[] = {{&S21631[30], 6}, {NULL}};
const Segment SGI136663[] = {{&S21631[114], 3}, {NULL}};
const Segment SGI136664[] = {{&S21631[79], 6}, {NULL}};
const Segment SGI136665[] = {{&S21633[44], 22}, {NULL}};
const Segment SGI136666[] = {{&S21633[44], 34}, {NULL}};
const Segment SGI136667[] = {{&S21633[30], 6}, {NULL}};
const Segment SGI136668[] = {{&S21633[127], 3}, {NULL}};
const Segment SGI136669[] = {{&S21633[92], 6}, {NULL}};
const Segment SGI136670[] = {{&S18662[42], 14}, {NULL}};
const Segment SGI136671[] = {{&S18662[42], 26}, {NULL}};
const Segment SGI136672[] = {{&S18662[30], 4}, {NULL}};
const Segment SGI136673[] = {{&S18662[117], 3}, {NULL}};
const Segment SGI136674[] = {{&S18662[82], 6}, {NULL}};
const Segment SGI136675[] = {{&S18663[36], 14}, {NULL}};
const Segment SGI136676[] = {{&S18663[31], 4}, {NULL}};
const Segment SGI136677[] = {{&S18665[48], 14}, {NULL}};
const Segment SGI136678[] = {{&S18665[48], 26}, {NULL}};
const Segment SGI136679[] = {{&S18665[30], 10}, {NULL}};
const Segment SGI136680[] = {{&S18665[123], 3}, {NULL}};
const Segment SGI136681[] = {{&S18665[88], 6}, {NULL}};
const Segment SGI136682[] = {{&S18668[42], 14}, {NULL}};
const Segment SGI136683[] = {{&S18668[42], 26}, {NULL}};
const Segment SGI136684[] = {{&S18668[30], 4}, {NULL}};
const Segment SGI136685[] = {{&S18668[117], 3}, {NULL}};
const Segment SGI136686[] = {{&S18668[82], 6}, {NULL}};
const Segment SGI136687[] = {{&S20627[44], 10}, {NULL}};
const Segment SGI136688[] = {{&S20627[44], 22}, {NULL}};
const Segment SGI136689[] = {{&S20627[30], 6}, {NULL}};
const Segment SGI136690[] = {{&S20627[115], 3}, {NULL}};
const Segment SGI136691[] = {{&S20627[80], 6}, {NULL}};
const Segment SGI136692[] = {{&S20629[38], 10}, {NULL}};
const Segment SGI136693[] = {{&S20629[31], 6}, {NULL}};
const Segment SGI136694[] = {{&S20632[44], 10}, {NULL}};
const Segment SGI136695[] = {{&S20632[44], 22}, {NULL}};
const Segment SGI136696[] = {{&S20632[30], 6}, {NULL}};
const Segment SGI136697[] = {{&S20632[80], 6}, {NULL}};
const Segment SGI136698[] = {{&S20633[44], 10}, {NULL}};
const Segment SGI136699[] = {{&S20633[44], 22}, {NULL}};
const Segment SGI136700[] = {{&S20633[30], 6}, {NULL}};
const Segment SGI136701[] = {{&S20633[115], 3}, {NULL}};
const Segment SGI136702[] = {{&S20633[80], 6}, {NULL}};
const Segment SGI136703[] = {{&S20647[44], 10}, {NULL}};
const Segment SGI136704[] = {{&S20647[44], 22}, {NULL}};
const Segment SGI136705[] = {{&S20647[30], 6}, {NULL}};
const Segment SGI136706[] = {{&S20647[115], 3}, {NULL}};
const Segment SGI136707[] = {{&S20647[80], 6}, {NULL}};
const Segment SGI136708[] = {{&S20650[52], 10}, {NULL}};
const Segment SGI136709[] = {{&S20650[52], 22}, {NULL}};
const Segment SGI136710[] = {{&S20650[38], 6}, {NULL}};
const Segment SGI136711[] = {{&S20652[44], 10}, {NULL}};
const Segment SGI136712[] = {{&S20652[44], 22}, {NULL}};
const Segment SGI136713[] = {{&S20652[30], 6}, {NULL}};
const Segment SGI136714[] = {{&S20652[83], 6}, {NULL}};
const Segment SGI136715[] = {{&S20654[44], 10}, {NULL}};
const Segment SGI136716[] = {{&S20654[44], 22}, {NULL}};
const Segment SGI136717[] = {{&S20654[30], 6}, {NULL}};
const Segment SGI136718[] = {{&S20654[115], 3}, {NULL}};
const Segment SGI136719[] = {{&S20654[80], 6}, {NULL}};
const Segment SGI136720[] = {{&S20654[135], 8}, {NULL}};
const Segment SGI136721[] = {{&S31537[44], 7}, {NULL}};
const Segment SGI136722[] = {{&S31537[44], 25}, {NULL}};
const Segment SGI136723[] = {{&S31537[30], 6}, {NULL}};
const Segment SGI136724[] = {{&S31537[31], 3}, {NULL}};
const Segment SGI136725[] = {{&S31537[83], 6}, {NULL}};
const Segment SGI136726[] = {{&S31539[38], 7}, {NULL}};
const Segment SGI136727[] = {{&S31539[31], 6}, {NULL}};
const Segment SGI136728[] = {{&S31543[44], 7}, {NULL}};
const Segment SGI136729[] = {{&S31543[44], 25}, {NULL}};
const Segment SGI136730[] = {{&S31543[30], 6}, {NULL}};
const Segment SGI136731[] = {{&S31543[31], 3}, {NULL}};
const Segment SGI136732[] = {{&S31543[83], 6}, {NULL}};
const Segment SGI136733[] = {{&S31544[38], 7}, {NULL}};
const Segment SGI136734[] = {{&S31544[31], 6}, {NULL}};
const Segment SGI136735[] = {{&S31545[44], 7}, {NULL}};
const Segment SGI136736[] = {{&S31545[44], 25}, {NULL}};
const Segment SGI136737[] = {{&S31545[30], 6}, {NULL}};
const Segment SGI136738[] = {{&S31545[31], 3}, {NULL}};
const Segment SGI136739[] = {{&S31545[83], 6}, {NULL}};
const Segment SGI136740[] = {{&S31546[44], 7}, {NULL}};
const Segment SGI136741[] = {{&S31546[44], 25}, {NULL}};
const Segment SGI136742[] = {{&S31546[30], 6}, {NULL}};
const Segment SGI136743[] = {{&S31546[31], 3}, {NULL}};
const Segment SGI136744[] = {{&S31546[83], 6}, {NULL}};
const Segment SGI136745[] = {{&S31547[44], 7}, {NULL}};
const Segment SGI136746[] = {{&S31547[44], 24}, {NULL}};
const Segment SGI136747[] = {{&S31547[30], 6}, {NULL}};
const Segment SGI136748[] = {{&S31547[31], 3}, {NULL}};
const Segment SGI136749[] = {{&S31547[82], 6}, {NULL}};
const Segment SGI136750[] = {{&S31548[44], 7}, {NULL}};
const Segment SGI136751[] = {{&S31548[44], 25}, {NULL}};
const Segment SGI136752[] = {{&S31548[30], 6}, {NULL}};
const Segment SGI136753[] = {{&S31548[31], 3}, {NULL}};
const Segment SGI136754[] = {{&S31548[83], 6}, {NULL}};
const Segment SGI136755[] = {{&S31549[44], 7}, {NULL}};
const Segment SGI136756[] = {{&S31549[44], 26}, {NULL}};
const Segment SGI136757[] = {{&S31549[30], 6}, {NULL}};
const Segment SGI136758[] = {{&S31549[31], 3}, {NULL}};
const Segment SGI136759[] = {{&S31549[84], 6}, {NULL}};
const Segment SGI136760[] = {{&S31550[44], 11}, {NULL}};
const Segment SGI136761[] = {{&S31550[44], 27}, {NULL}};
const Segment SGI136762[] = {{&S31550[30], 6}, {NULL}};
const Segment SGI136763[] = {{&S31550[31], 3}, {NULL}};
const Segment SGI136764[] = {{&S31550[85], 6}, {NULL}};
const Segment SGI136765[] = {{&S31551[44], 11}, {NULL}};
const Segment SGI136766[] = {{&S31551[44], 28}, {NULL}};
const Segment SGI136767[] = {{&S31551[30], 6}, {NULL}};
const Segment SGI136768[] = {{&S31551[31], 3}, {NULL}};
const Segment SGI136769[] = {{&S31551[86], 6}, {NULL}};
const Segment SGI136770[] = {{&S31552[44], 7}, {NULL}};
const Segment SGI136771[] = {{&S31552[44], 24}, {NULL}};
const Segment SGI136772[] = {{&S31552[30], 6}, {NULL}};
const Segment SGI136773[] = {{&S31552[31], 3}, {NULL}};
const Segment SGI136774[] = {{&S31552[82], 6}, {NULL}};
const Segment SGI136775[] = {{&S31553[44], 11}, {NULL}};
const Segment SGI136776[] = {{&S31553[44], 28}, {NULL}};
const Segment SGI136777[] = {{&S31553[30], 6}, {NULL}};
const Segment SGI136778[] = {{&S31553[31], 3}, {NULL}};
const Segment SGI136779[] = {{&S31553[86], 6}, {NULL}};
const Segment SGI136780[] = {{&S31554[44], 13}, {NULL}};
const Segment SGI136781[] = {{&S31554[44], 30}, {NULL}};
const Segment SGI136782[] = {{&S31554[30], 6}, {NULL}};
const Segment SGI136783[] = {{&S31554[31], 3}, {NULL}};
const Segment SGI136784[] = {{&S31554[88], 6}, {NULL}};
const Segment SGI136785[] = {{&S31555[44], 11}, {NULL}};
const Segment SGI136786[] = {{&S31555[44], 28}, {NULL}};
const Segment SGI136787[] = {{&S31555[30], 6}, {NULL}};
const Segment SGI136788[] = {{&S31555[31], 3}, {NULL}};
const Segment SGI136789[] = {{&S31555[86], 6}, {NULL}};
const Segment SGI136790[] = {{&S31556[44], 11}, {NULL}};
const Segment SGI136791[] = {{&S31556[44], 28}, {NULL}};
const Segment SGI136792[] = {{&S31556[30], 6}, {NULL}};
const Segment SGI136793[] = {{&S31556[31], 3}, {NULL}};
const Segment SGI136794[] = {{&S31556[86], 6}, {NULL}};
const Segment SGI136795[] = {{&S31557[44], 12}, {NULL}};
const Segment SGI136796[] = {{&S31557[44], 29}, {NULL}};
const Segment SGI136797[] = {{&S31557[30], 6}, {NULL}};
const Segment SGI136798[] = {{&S31557[31], 3}, {NULL}};
const Segment SGI136799[] = {{&S31557[87], 6}, {NULL}};
const Segment SGI136800[] = {{&S31558[44], 7}, {NULL}};
const Segment SGI136801[] = {{&S31558[44], 24}, {NULL}};
const Segment SGI136802[] = {{&S31558[30], 6}, {NULL}};
const Segment SGI136803[] = {{&S31558[31], 3}, {NULL}};
const Segment SGI136804[] = {{&S31558[82], 6}, {NULL}};
const Segment SGI136805[] = {{&S31559[44], 11}, {NULL}};
const Segment SGI136806[] = {{&S31559[44], 27}, {NULL}};
const Segment SGI136807[] = {{&S31559[30], 6}, {NULL}};
const Segment SGI136808[] = {{&S31559[31], 3}, {NULL}};
const Segment SGI136809[] = {{&S31559[85], 6}, {NULL}};
const Segment SGI136810[] = {{&S31560[44], 7}, {NULL}};
const Segment SGI136811[] = {{&S31560[44], 24}, {NULL}};
const Segment SGI136812[] = {{&S31560[30], 6}, {NULL}};
const Segment SGI136813[] = {{&S31560[31], 3}, {NULL}};
const Segment SGI136814[] = {{&S31560[82], 6}, {NULL}};
const Segment SGI136815[] = {{&S31561[38], 7}, {NULL}};
const Segment SGI136816[] = {{&S31561[31], 6}, {NULL}};
const Segment SGI136817[] = {{&S31562[44], 7}, {NULL}};
const Segment SGI136818[] = {{&S31562[44], 23}, {NULL}};
const Segment SGI136819[] = {{&S31562[30], 6}, {NULL}};
const Segment SGI136820[] = {{&S31562[31], 3}, {NULL}};
const Segment SGI136821[] = {{&S31562[81], 6}, {NULL}};
const Segment SGI136822[] = {{&S31563[44], 13}, {NULL}};
const Segment SGI136823[] = {{&S31563[44], 29}, {NULL}};
const Segment SGI136824[] = {{&S31563[30], 6}, {NULL}};
const Segment SGI136825[] = {{&S31563[31], 3}, {NULL}};
const Segment SGI136826[] = {{&S31563[87], 6}, {NULL}};
const Segment SGI136827[] = {{&S31564[44], 11}, {NULL}};
const Segment SGI136828[] = {{&S31564[44], 27}, {NULL}};
const Segment SGI136829[] = {{&S31564[30], 6}, {NULL}};
const Segment SGI136830[] = {{&S31564[31], 3}, {NULL}};
const Segment SGI136831[] = {{&S31564[85], 6}, {NULL}};
const Segment SGI136832[] = {{&S31565[44], 11}, {NULL}};
const Segment SGI136833[] = {{&S31565[44], 27}, {NULL}};
const Segment SGI136834[] = {{&S31565[30], 6}, {NULL}};
const Segment SGI136835[] = {{&S31565[31], 3}, {NULL}};
const Segment SGI136836[] = {{&S31565[85], 6}, {NULL}};
const Segment SGI136837[] = {{&S31566[44], 7}, {NULL}};
const Segment SGI136838[] = {{&S31566[44], 23}, {NULL}};
const Segment SGI136839[] = {{&S31566[30], 6}, {NULL}};
const Segment SGI136840[] = {{&S31566[31], 3}, {NULL}};
const Segment SGI136841[] = {{&S31566[81], 6}, {NULL}};
const Segment SGI136842[] = {{&S31567[44], 12}, {NULL}};
const Segment SGI136843[] = {{&S31567[44], 28}, {NULL}};
const Segment SGI136844[] = {{&S31567[30], 6}, {NULL}};
const Segment SGI136845[] = {{&S31567[31], 3}, {NULL}};
const Segment SGI136846[] = {{&S31567[86], 6}, {NULL}};
const Segment SGI136847[] = {{&S31568[44], 12}, {NULL}};
const Segment SGI136848[] = {{&S31568[44], 29}, {NULL}};
const Segment SGI136849[] = {{&S31568[30], 6}, {NULL}};
const Segment SGI136850[] = {{&S31568[31], 3}, {NULL}};
const Segment SGI136851[] = {{&S31568[87], 6}, {NULL}};
const Segment SGI136852[] = {{&S31569[44], 11}, {NULL}};
const Segment SGI136853[] = {{&S31569[44], 28}, {NULL}};
const Segment SGI136854[] = {{&S31569[30], 6}, {NULL}};
const Segment SGI136855[] = {{&S31569[31], 3}, {NULL}};
const Segment SGI136856[] = {{&S31569[86], 6}, {NULL}};
const Segment SGI136857[] = {{&S31570[44], 11}, {NULL}};
const Segment SGI136858[] = {{&S31570[44], 28}, {NULL}};
const Segment SGI136859[] = {{&S31570[30], 6}, {NULL}};
const Segment SGI136860[] = {{&S31570[31], 3}, {NULL}};
const Segment SGI136861[] = {{&S31570[86], 6}, {NULL}};
const Segment SGI136862[] = {{&S31571[44], 7}, {NULL}};
const Segment SGI136863[] = {{&S31571[44], 23}, {NULL}};
const Segment SGI136864[] = {{&S31571[30], 6}, {NULL}};
const Segment SGI136865[] = {{&S31571[31], 3}, {NULL}};
const Segment SGI136866[] = {{&S31571[81], 6}, {NULL}};
const Segment SGI136867[] = {{&S31572[38], 7}, {NULL}};
const Segment SGI136868[] = {{&S31572[31], 6}, {NULL}};
const Segment SGI136869[] = {{&S31573[44], 7}, {NULL}};
const Segment SGI136870[] = {{&S31573[44], 24}, {NULL}};
const Segment SGI136871[] = {{&S31573[30], 6}, {NULL}};
const Segment SGI136872[] = {{&S31573[31], 3}, {NULL}};
const Segment SGI136873[] = {{&S31573[82], 6}, {NULL}};
const Segment SGI136874[] = {{&S31574[44], 7}, {NULL}};
const Segment SGI136875[] = {{&S31574[44], 24}, {NULL}};
const Segment SGI136876[] = {{&S31574[30], 6}, {NULL}};
const Segment SGI136877[] = {{&S31574[31], 3}, {NULL}};
const Segment SGI136878[] = {{&S31574[82], 6}, {NULL}};
const Segment SGI136879[] = {{&S21035[44], 10}, {NULL}};
const Segment SGI136880[] = {{&S21035[44], 22}, {NULL}};
const Segment SGI136881[] = {{&S21035[30], 6}, {NULL}};
const Segment SGI136882[] = {{&S21035[115], 3}, {NULL}};
const Segment SGI136883[] = {{&S21035[80], 6}, {NULL}};
const Segment SGI136884[] = {{&S21038[38], 10}, {NULL}};
const Segment SGI136885[] = {{&S21038[31], 6}, {NULL}};
const Segment SGI136886[] = {{&S21040[44], 11}, {NULL}};
const Segment SGI136887[] = {{&S21040[44], 23}, {NULL}};
const Segment SGI136888[] = {{&S21040[30], 6}, {NULL}};
const Segment SGI136889[] = {{&S21040[116], 3}, {NULL}};
const Segment SGI136890[] = {{&S21040[81], 6}, {NULL}};
const Segment SGI136891[] = {{&S21041[44], 11}, {NULL}};
const Segment SGI136892[] = {{&S21041[44], 23}, {NULL}};
const Segment SGI136893[] = {{&S21041[30], 6}, {NULL}};
const Segment SGI136894[] = {{&S21041[116], 3}, {NULL}};
const Segment SGI136895[] = {{&S21041[81], 6}, {NULL}};
const Segment SGI136896[] = {{&S21042[44], 11}, {NULL}};
const Segment SGI136897[] = {{&S21042[44], 23}, {NULL}};
const Segment SGI136898[] = {{&S21042[30], 6}, {NULL}};
const Segment SGI136899[] = {{&S21042[116], 3}, {NULL}};
const Segment SGI136900[] = {{&S21042[81], 6}, {NULL}};
const Segment SGI136901[] = {{&S21043[44], 11}, {NULL}};
const Segment SGI136902[] = {{&S21043[44], 23}, {NULL}};
const Segment SGI136903[] = {{&S21043[30], 6}, {NULL}};
const Segment SGI136904[] = {{&S21043[116], 3}, {NULL}};
const Segment SGI136905[] = {{&S21043[81], 6}, {NULL}};
const Segment SGI136906[] = {{&S21044[44], 11}, {NULL}};
const Segment SGI136907[] = {{&S21044[44], 23}, {NULL}};
const Segment SGI136908[] = {{&S21044[30], 6}, {NULL}};
const Segment SGI136909[] = {{&S21044[116], 3}, {NULL}};
const Segment SGI136910[] = {{&S21044[81], 6}, {NULL}};
const Segment SGI136911[] = {{&S21045[44], 11}, {NULL}};
const Segment SGI136912[] = {{&S21045[44], 23}, {NULL}};
const Segment SGI136913[] = {{&S21045[30], 6}, {NULL}};
const Segment SGI136914[] = {{&S21045[116], 3}, {NULL}};
const Segment SGI136915[] = {{&S21045[81], 6}, {NULL}};
const Segment SGI136916[] = {{&S21046[44], 10}, {NULL}};
const Segment SGI136917[] = {{&S21046[44], 22}, {NULL}};
const Segment SGI136918[] = {{&S21046[30], 6}, {NULL}};
const Segment SGI136919[] = {{&S21046[83], 6}, {NULL}};
const Segment SGI136920[] = {{&S21047[42], 10}, {NULL}};
const Segment SGI136921[] = {{&S21047[30], 6}, {NULL}};
const Segment SGI136922[] = {{&S21047[113], 3}, {NULL}};
const Segment SGI136923[] = {{&S21047[78], 6}, {NULL}};
const Segment SGI136924[] = {{&S21048[38], 11}, {NULL}};
const Segment SGI136925[] = {{&S21048[31], 6}, {NULL}};
const Segment SGI136926[] = {{&S21049[38], 11}, {NULL}};
const Segment SGI136927[] = {{&S21049[31], 6}, {NULL}};
const Segment SGI136928[] = {{&S21050[44], 18}, {NULL}};
const Segment SGI136929[] = {{&S21050[44], 30}, {NULL}};
const Segment SGI136930[] = {{&S21050[30], 6}, {NULL}};
const Segment SGI136931[] = {{&S21050[123], 3}, {NULL}};
const Segment SGI136932[] = {{&S21050[88], 6}, {NULL}};
const Segment SGI136933[] = {{&S21051[44], 29}, {NULL}};
const Segment SGI136934[] = {{&S21051[44], 41}, {NULL}};
const Segment SGI136935[] = {{&S21051[30], 6}, {NULL}};
const Segment SGI136936[] = {{&S21051[134], 3}, {NULL}};
const Segment SGI136937[] = {{&S21051[99], 6}, {NULL}};
const Segment SGI136938[] = {{&S21052[38], 11}, {NULL}};
const Segment SGI136939[] = {{&S21052[31], 6}, {NULL}};
const Segment SGI136940[] = {{&S21053[44], 11}, {NULL}};
const Segment SGI136941[] = {{&S21053[44], 23}, {NULL}};
const Segment SGI136942[] = {{&S21053[30], 6}, {NULL}};
const Segment SGI136943[] = {{&S21053[116], 3}, {NULL}};
const Segment SGI136944[] = {{&S21053[81], 6}, {NULL}};
const Segment SGI136945[] = {{&S21054[38], 11}, {NULL}};
const Segment SGI136946[] = {{&S21054[31], 6}, {NULL}};
const Segment SGI136947[] = {{&S21055[44], 29}, {NULL}};
const Segment SGI136948[] = {{&S21055[44], 41}, {NULL}};
const Segment SGI136949[] = {{&S21055[30], 6}, {NULL}};
const Segment SGI136950[] = {{&S21055[134], 3}, {NULL}};
const Segment SGI136951[] = {{&S21055[99], 6}, {NULL}};
const Segment SGI136952[] = {{&S21056[44], 10}, {NULL}};
const Segment SGI136953[] = {{&S21056[44], 22}, {NULL}};
const Segment SGI136954[] = {{&S21056[30], 6}, {NULL}};
const Segment SGI136955[] = {{&S21056[115], 3}, {NULL}};
const Segment SGI136956[] = {{&S21056[80], 6}, {NULL}};
const Segment SGI136957[] = {{&S21057[44], 29}, {NULL}};
const Segment SGI136958[] = {{&S21057[44], 41}, {NULL}};
const Segment SGI136959[] = {{&S21057[30], 6}, {NULL}};
const Segment SGI136960[] = {{&S21057[134], 3}, {NULL}};
const Segment SGI136961[] = {{&S21057[99], 6}, {NULL}};
const Segment SGI136962[] = {{&S21059[44], 8}, {NULL}};
const Segment SGI136963[] = {{&S21059[44], 20}, {NULL}};
const Segment SGI136964[] = {{&S21059[30], 6}, {NULL}};
const Segment SGI136965[] = {{&S21059[113], 3}, {NULL}};
const Segment SGI136966[] = {{&S21059[78], 6}, {NULL}};
const Segment SGI136967[] = {{&S21060[44], 28}, {NULL}};
const Segment SGI136968[] = {{&S21060[44], 40}, {NULL}};
const Segment SGI136969[] = {{&S21060[30], 6}, {NULL}};
const Segment SGI136970[] = {{&S21060[133], 3}, {NULL}};
const Segment SGI136971[] = {{&S21060[98], 6}, {NULL}};
const Segment SGI136972[] = {{&S21061[38], 11}, {NULL}};
const Segment SGI136973[] = {{&S21061[31], 6}, {NULL}};
const Segment SGI136974[] = {{&S21062[44], 28}, {NULL}};
const Segment SGI136975[] = {{&S21062[44], 40}, {NULL}};
const Segment SGI136976[] = {{&S21062[30], 6}, {NULL}};
const Segment SGI136977[] = {{&S21062[133], 3}, {NULL}};
const Segment SGI136978[] = {{&S21062[98], 6}, {NULL}};
const Segment SGI136979[] = {{&S21063[44], 18}, {NULL}};
const Segment SGI136980[] = {{&S21063[44], 30}, {NULL}};
const Segment SGI136981[] = {{&S21063[30], 6}, {NULL}};
const Segment SGI136982[] = {{&S21063[123], 3}, {NULL}};
const Segment SGI136983[] = {{&S21063[88], 6}, {NULL}};
const Segment SGI136984[] = {{&S21064[44], 10}, {NULL}};
const Segment SGI136985[] = {{&S22473[34], 10}, {NULL}};
const Segment SGI136986[] = {{&S21064[44], 23}, {NULL}};
const Segment SGI136987[] = {{&S21064[30], 6}, {NULL}};
const Segment SGI136988[] = {{&S22473[27], 6}, {NULL}};
const Segment SGI136989[] = {{&S21064[116], 3}, {NULL}};
const Segment SGI136990[] = {{&S21064[81], 6}, {NULL}};
const Segment SGI136991[] = {{&S22473[97], 9}, {NULL}};
const Segment SGI136992[] = {{&S22473[70], 6}, {NULL}};
const Segment SGI136993[] = {{&S21065[44], 29}, {NULL}};
const Segment SGI136994[] = {{&S22474[34], 10}, {NULL}};
const Segment SGI136995[] = {{&S21065[44], 41}, {NULL}};
const Segment SGI136996[] = {{&S21065[30], 6}, {NULL}};
const Segment SGI136997[] = {{&S22474[27], 6}, {NULL}};
const Segment SGI136998[] = {{&S21065[134], 3}, {NULL}};
const Segment SGI136999[] = {{&S22474[97], 9}, {NULL}};
const Segment SGI137000[] = {{&S21065[99], 6}, {NULL}};
const Segment SGI137001[] = {{&S22474[70], 6}, {NULL}};
const Segment SGI137002[] = {{&S21066[38], 8}, {NULL}};
const Segment SGI137003[] = {{&S21066[31], 6}, {NULL}};
const Segment SGI137004[] = {{&S22475[34], 10}, {NULL}};
const Segment SGI137005[] = {{&S22475[27], 6}, {NULL}};
const Segment SGI137006[] = {{&S22475[97], 9}, {NULL}};
const Segment SGI137007[] = {{&S22475[70], 6}, {NULL}};
const Segment SGI137008[] = {{&S22476[34], 10}, {NULL}};
const Segment SGI137009[] = {{&S22476[27], 6}, {NULL}};
const Segment SGI137010[] = {{&S22476[97], 9}, {NULL}};
const Segment SGI137011[] = {{&S22476[70], 6}, {NULL}};
const Segment SGI137012[] = {{&S22480[34], 11}, {NULL}};
const Segment SGI137013[] = {{&S22480[27], 6}, {NULL}};
const Segment SGI137014[] = {{&S22480[98], 9}, {NULL}};
const Segment SGI137015[] = {{&S22480[71], 6}, {NULL}};
const Segment SGI137016[] = {{&S22481[34], 11}, {NULL}};
const Segment SGI137017[] = {{&S22481[27], 6}, {NULL}};
const Segment SGI137018[] = {{&S22481[98], 9}, {NULL}};
const Segment SGI137019[] = {{&S22481[71], 6}, {NULL}};
const Segment SGI137020[] = {{&S22482[34], 11}, {NULL}};
const Segment SGI137021[] = {{&S22482[27], 6}, {NULL}};
const Segment SGI137022[] = {{&S22482[98], 9}, {NULL}};
const Segment SGI137023[] = {{&S22482[71], 6}, {NULL}};
const Segment SGI137024[] = {{&S22485[34], 11}, {NULL}};
const Segment SGI137025[] = {{&S22485[27], 6}, {NULL}};
const Segment SGI137026[] = {{&S22485[98], 9}, {NULL}};
const Segment SGI137027[] = {{&S22485[71], 6}, {NULL}};
const Segment SGI137028[] = {{&S22487[34], 7}, {NULL}};
const Segment SGI137029[] = {{&S22487[27], 6}, {NULL}};
const Segment SGI137030[] = {{&S22487[94], 9}, {NULL}};
const Segment SGI137031[] = {{&S22487[67], 6}, {NULL}};
const Segment SGI137032[] = {{&S22488[34], 8}, {NULL}};
const Segment SGI137033[] = {{&S22488[27], 6}, {NULL}};
const Segment SGI137034[] = {{&S22488[95], 9}, {NULL}};
const Segment SGI137035[] = {{&S22488[68], 6}, {NULL}};
const Segment SGI137036[] = {{&S22497[34], 10}, {NULL}};
const Segment SGI137037[] = {{&S22497[27], 6}, {NULL}};
const Segment SGI137038[] = {{&S22497[97], 9}, {NULL}};
const Segment SGI137039[] = {{&S22497[70], 6}, {NULL}};
const Segment SGI137040[] = {{&S24726[32], 7}, {NULL}};
const Segment SGI137041[] = {{&S24726[27], 4}, {NULL}};
const Segment SGI137042[] = {{&S24726[92], 9}, {NULL}};
const Segment SGI137043[] = {{&S24726[65], 6}, {NULL}};
const Segment SGI137044[] = {{&S31760[44], 10}, {NULL}};
const Segment SGI137045[] = {{&S31760[44], 29}, {NULL}};
const Segment SGI137046[] = {{&S31760[30], 6}, {NULL}};
const Segment SGI137047[] = {{&S31760[31], 3}, {NULL}};
const Segment SGI137048[] = {{&S31760[87], 6}, {NULL}};
const Segment SGI137049[] = {{&S31761[44], 10}, {NULL}};
const Segment SGI137050[] = {{&S31761[44], 45}, {NULL}};
const Segment SGI137051[] = {{&S31761[30], 6}, {NULL}};
const Segment SGI137052[] = {{&S31761[31], 3}, {NULL}};
const Segment SGI137053[] = {{&S31761[103], 6}, {NULL}};
const Segment SGI137054[] = {{&S31763[44], 10}, {NULL}};
const Segment SGI137055[] = {{&S31763[44], 45}, {NULL}};
const Segment SGI137056[] = {{&S31763[30], 6}, {NULL}};
const Segment SGI137057[] = {{&S31763[31], 3}, {NULL}};
const Segment SGI137058[] = {{&S31763[103], 6}, {NULL}};
const Segment SGI137059[] = {{&S31765[44], 10}, {NULL}};
const Segment SGI137060[] = {{&S31765[44], 29}, {NULL}};
const Segment SGI137061[] = {{&S31765[30], 6}, {NULL}};
const Segment SGI137062[] = {{&S31765[31], 3}, {NULL}};
const Segment SGI137063[] = {{&S31765[87], 6}, {NULL}};
const Segment SGI137064[] = {{&S31767[44], 14}, {NULL}};
const Segment SGI137065[] = {{&S31767[44], 26}, {NULL}};
const Segment SGI137066[] = {{&S31767[30], 6}, {NULL}};
const Segment SGI137067[] = {{&S31767[31], 3}, {NULL}};
const Segment SGI137068[] = {{&S31767[84], 6}, {NULL}};
const Segment SGI137069[] = {{&S31768[44], 10}, {NULL}};
const Segment SGI137070[] = {{&S31768[44], 51}, {NULL}};
const Segment SGI137071[] = {{&S31768[30], 6}, {NULL}};
const Segment SGI137072[] = {{&S31768[31], 3}, {NULL}};
const Segment SGI137073[] = {{&S31768[109], 6}, {NULL}};
const Segment SGI137074[] = {{&S31769[44], 14}, {NULL}};
const Segment SGI137075[] = {{&S31769[44], 26}, {NULL}};
const Segment SGI137076[] = {{&S31769[30], 6}, {NULL}};
const Segment SGI137077[] = {{&S31769[31], 3}, {NULL}};
const Segment SGI137078[] = {{&S31769[84], 6}, {NULL}};
const Segment SGI137079[] = {{&S31772[44], 14}, {NULL}};
const Segment SGI137080[] = {{&S31772[44], 45}, {NULL}};
const Segment SGI137081[] = {{&S31772[30], 6}, {NULL}};
const Segment SGI137082[] = {{&S31772[31], 3}, {NULL}};
const Segment SGI137083[] = {{&S31772[103], 6}, {NULL}};
const Segment SGI137084[] = {{&S31773[44], 9}, {NULL}};
const Segment SGI137085[] = {{&S31773[44], 21}, {NULL}};
const Segment SGI137086[] = {{&S31773[30], 6}, {NULL}};
const Segment SGI137087[] = {{&S31773[31], 3}, {NULL}};
const Segment SGI137088[] = {{&S31773[79], 6}, {NULL}};
const Segment SGI137089[] = {{&S31774[44], 14}, {NULL}};
const Segment SGI137090[] = {{&S31774[44], 26}, {NULL}};
const Segment SGI137091[] = {{&S31774[30], 6}, {NULL}};
const Segment SGI137092[] = {{&S31774[31], 3}, {NULL}};
const Segment SGI137093[] = {{&S31774[84], 6}, {NULL}};
const Segment SGI137094[] = {{&S31775[44], 9}, {NULL}};
const Segment SGI137095[] = {{&S31775[44], 21}, {NULL}};
const Segment SGI137096[] = {{&S31775[30], 6}, {NULL}};
const Segment SGI137097[] = {{&S31775[31], 3}, {NULL}};
const Segment SGI137098[] = {{&S31775[79], 6}, {NULL}};
const Segment SGI137099[] = {{&S31776[44], 9}, {NULL}};
const Segment SGI137100[] = {{&S31776[44], 21}, {NULL}};
const Segment SGI137101[] = {{&S31776[30], 6}, {NULL}};
const Segment SGI137102[] = {{&S31776[31], 3}, {NULL}};
const Segment SGI137103[] = {{&S31776[79], 6}, {NULL}};
const Segment SGI137104[] = {{&S31777[44], 14}, {NULL}};
const Segment SGI137105[] = {{&S31777[44], 26}, {NULL}};
const Segment SGI137106[] = {{&S31777[30], 6}, {NULL}};
const Segment SGI137107[] = {{&S31777[31], 3}, {NULL}};
const Segment SGI137108[] = {{&S31777[84], 6}, {NULL}};
const Segment SGI137109[] = {{&S31778[38], 9}, {NULL}};
const Segment SGI137110[] = {{&S31778[31], 6}, {NULL}};
const Segment SGI137111[] = {{&S31779[44], 17}, {NULL}};
const Segment SGI137112[] = {{&S31779[44], 29}, {NULL}};
const Segment SGI137113[] = {{&S31779[30], 6}, {NULL}};
const Segment SGI137114[] = {{&S31779[31], 3}, {NULL}};
const Segment SGI137115[] = {{&S31779[87], 6}, {NULL}};
const Segment SGI137116[] = {{&S31780[38], 9}, {NULL}};
const Segment SGI137117[] = {{&S31780[31], 6}, {NULL}};
const Segment SGI137118[] = {{&S31781[44], 14}, {NULL}};
const Segment SGI137119[] = {{&S31781[44], 26}, {NULL}};
const Segment SGI137120[] = {{&S31781[30], 6}, {NULL}};
const Segment SGI137121[] = {{&S31781[31], 3}, {NULL}};
const Segment SGI137122[] = {{&S31781[84], 6}, {NULL}};
const Segment SGI137123[] = {{&S31782[44], 17}, {NULL}};
const Segment SGI137124[] = {{&S31782[44], 29}, {NULL}};
const Segment SGI137125[] = {{&S31782[30], 6}, {NULL}};
const Segment SGI137126[] = {{&S31782[31], 3}, {NULL}};
const Segment SGI137127[] = {{&S31782[87], 6}, {NULL}};
const Segment SGI137128[] = {{&S31783[38], 9}, {NULL}};
const Segment SGI137129[] = {{&S31783[31], 6}, {NULL}};
const Segment SGI137130[] = {{&S31784[44], 14}, {NULL}};
const Segment SGI137131[] = {{&S31784[44], 26}, {NULL}};
const Segment SGI137132[] = {{&S31784[30], 6}, {NULL}};
const Segment SGI137133[] = {{&S31784[31], 3}, {NULL}};
const Segment SGI137134[] = {{&S31784[84], 6}, {NULL}};
const Segment SGI137135[] = {{&S31785[44], 17}, {NULL}};
const Segment SGI137136[] = {{&S31785[44], 29}, {NULL}};
const Segment SGI137137[] = {{&S31785[30], 6}, {NULL}};
const Segment SGI137138[] = {{&S31785[31], 3}, {NULL}};
const Segment SGI137139[] = {{&S31785[87], 6}, {NULL}};
const Segment SGI137140[] = {{&S31786[41], 9}, {NULL}};
const Segment SGI137141[] = {{&S31786[30], 6}, {NULL}};
const Segment SGI137142[] = {{&S31786[31], 3}, {NULL}};
const Segment SGI137143[] = {{&S31786[76], 6}, {NULL}};
const Segment SGI137144[] = {{&S31787[41], 9}, {NULL}};
const Segment SGI137145[] = {{&S31787[30], 6}, {NULL}};
const Segment SGI137146[] = {{&S31787[31], 3}, {NULL}};
const Segment SGI137147[] = {{&S31787[76], 6}, {NULL}};
const Segment SGI137148[] = {{&S31935[44], 15}, {NULL}};
const Segment SGI137149[] = {{&S31935[44], 27}, {NULL}};
const Segment SGI137150[] = {{&S31935[30], 6}, {NULL}};
const Segment SGI137151[] = {{&S31935[31], 3}, {NULL}};
const Segment SGI137152[] = {{&S31935[85], 6}, {NULL}};
const Segment SGI137153[] = {{&S31936[44], 15}, {NULL}};
const Segment SGI137154[] = {{&S31936[44], 27}, {NULL}};
const Segment SGI137155[] = {{&S31936[30], 6}, {NULL}};
const Segment SGI137156[] = {{&S31936[31], 3}, {NULL}};
const Segment SGI137157[] = {{&S31936[100], 6}, {NULL}};
const Segment SGI137158[] = {{&S31936[73], 8}, {NULL}};
const Segment SGI137159[] = {{&S31937[44], 15}, {NULL}};
const Segment SGI137160[] = {{&S31937[44], 27}, {NULL}};
const Segment SGI137161[] = {{&S31937[30], 6}, {NULL}};
const Segment SGI137162[] = {{&S31937[31], 3}, {NULL}};
const Segment SGI137163[] = {{&S31937[85], 6}, {NULL}};
const Segment SGI137164[] = {{&S31938[44], 15}, {NULL}};
const Segment SGI137165[] = {{&S31938[44], 27}, {NULL}};
const Segment SGI137166[] = {{&S31938[30], 6}, {NULL}};
const Segment SGI137167[] = {{&S31938[31], 3}, {NULL}};
const Segment SGI137168[] = {{&S31938[100], 6}, {NULL}};
const Segment SGI137169[] = {{&S31938[73], 8}, {NULL}};
const Segment SGI137170[] = {{&S31940[44], 15}, {NULL}};
const Segment SGI137171[] = {{&S31940[44], 27}, {NULL}};
const Segment SGI137172[] = {{&S31940[30], 6}, {NULL}};
const Segment SGI137173[] = {{&S31940[31], 3}, {NULL}};
const Segment SGI137174[] = {{&S31940[100], 6}, {NULL}};
const Segment SGI137175[] = {{&S31940[73], 8}, {NULL}};
const Segment SGI137176[] = {{&S31941[38], 15}, {NULL}};
const Segment SGI137177[] = {{&S31941[31], 6}, {NULL}};
const Segment SGI137178[] = {{&S31942[11], 1}, {NULL}};
const Segment SGI137179[] = {{&S31942[44], 13}, {NULL}};
const Segment SGI137180[] = {{&S31942[30], 6}, {NULL}};
const Segment SGI137181[] = {{&S31942[31], 3}, {NULL}};
const Segment SGI137182[] = {{&S31942[109], 6}, {NULL}};
const Segment SGI137183[] = {{&S31942[59], 8}, {NULL}};
const Segment SGI137184[] = {{&S31944[44], 15}, {NULL}};
const Segment SGI137185[] = {{&S31944[44], 27}, {NULL}};
const Segment SGI137186[] = {{&S31944[30], 6}, {NULL}};
const Segment SGI137187[] = {{&S31944[31], 3}, {NULL}};
const Segment SGI137188[] = {{&S31944[100], 6}, {NULL}};
const Segment SGI137189[] = {{&S31944[73], 8}, {NULL}};
const Segment SGI137190[] = {{&S31974[38], 14}, {NULL}};
const Segment SGI137191[] = {{&S31974[31], 6}, {NULL}};
const Segment SGI137192[] = {{&S31975[44], 14}, {NULL}};
const Segment SGI137193[] = {{&S31975[44], 26}, {NULL}};
const Segment SGI137194[] = {{&S31975[30], 6}, {NULL}};
const Segment SGI137195[] = {{&S31975[31], 3}, {NULL}};
const Segment SGI137196[] = {{&S31975[84], 6}, {NULL}};
const Segment SGI137197[] = {{&S31976[44], 25}, {NULL}};
const Segment SGI137198[] = {{&S31976[44], 37}, {NULL}};
const Segment SGI137199[] = {{&S31976[30], 6}, {NULL}};
const Segment SGI137200[] = {{&S31976[31], 3}, {NULL}};
const Segment SGI137201[] = {{&S31976[95], 6}, {NULL}};
const Segment SGI137202[] = {{&S31977[44], 34}, {NULL}};
const Segment SGI137203[] = {{&S31977[44], 46}, {NULL}};
const Segment SGI137204[] = {{&S31977[30], 6}, {NULL}};
const Segment SGI137205[] = {{&S31977[31], 3}, {NULL}};
const Segment SGI137206[] = {{&S31977[104], 6}, {NULL}};
const Segment SGI137207[] = {{&S31978[44], 26}, {NULL}};
const Segment SGI137208[] = {{&S31978[44], 38}, {NULL}};
const Segment SGI137209[] = {{&S31978[30], 6}, {NULL}};
const Segment SGI137210[] = {{&S31978[31], 3}, {NULL}};
const Segment SGI137211[] = {{&S31978[96], 6}, {NULL}};
const Segment SGI137212[] = {{&S31979[44], 14}, {NULL}};
const Segment SGI137213[] = {{&S31979[44], 26}, {NULL}};
const Segment SGI137214[] = {{&S31979[30], 6}, {NULL}};
const Segment SGI137215[] = {{&S31979[31], 3}, {NULL}};
const Segment SGI137216[] = {{&S31979[84], 6}, {NULL}};
const Segment SGI137217[] = {{&S31980[44], 25}, {NULL}};
const Segment SGI137218[] = {{&S31980[44], 37}, {NULL}};
const Segment SGI137219[] = {{&S31980[30], 6}, {NULL}};
const Segment SGI137220[] = {{&S31980[31], 3}, {NULL}};
const Segment SGI137221[] = {{&S31980[95], 6}, {NULL}};
const Segment SGI137222[] = {{&S31981[44], 25}, {NULL}};
const Segment SGI137223[] = {{&S31981[44], 37}, {NULL}};
const Segment SGI137224[] = {{&S31981[30], 6}, {NULL}};
const Segment SGI137225[] = {{&S31981[31], 3}, {NULL}};
const Segment SGI137226[] = {{&S31981[95], 6}, {NULL}};
const Segment SGI137227[] = {{&S31982[44], 25}, {NULL}};
const Segment SGI137228[] = {{&S31982[44], 37}, {NULL}};
const Segment SGI137229[] = {{&S31982[30], 6}, {NULL}};
const Segment SGI137230[] = {{&S31982[31], 3}, {NULL}};
const Segment SGI137231[] = {{&S31982[95], 6}, {NULL}};
const Segment SGI137232[] = {{&S31983[44], 25}, {NULL}};
const Segment SGI137233[] = {{&S31983[44], 37}, {NULL}};
const Segment SGI137234[] = {{&S31983[30], 6}, {NULL}};
const Segment SGI137235[] = {{&S31983[31], 3}, {NULL}};
const Segment SGI137236[] = {{&S31983[95], 6}, {NULL}};
const Segment SGI137237[] = {{&S31984[44], 25}, {NULL}};
const Segment SGI137238[] = {{&S31984[44], 37}, {NULL}};
const Segment SGI137239[] = {{&S31984[30], 6}, {NULL}};
const Segment SGI137240[] = {{&S31984[31], 3}, {NULL}};
const Segment SGI137241[] = {{&S31984[95], 6}, {NULL}};
const Segment SGI137242[] = {{&S31985[44], 25}, {NULL}};
const Segment SGI137243[] = {{&S31985[44], 37}, {NULL}};
const Segment SGI137244[] = {{&S31985[30], 6}, {NULL}};
const Segment SGI137245[] = {{&S31985[31], 3}, {NULL}};
const Segment SGI137246[] = {{&S31985[95], 6}, {NULL}};
const Segment SGI137247[] = {{&S31986[44], 25}, {NULL}};
const Segment SGI137248[] = {{&S31986[44], 37}, {NULL}};
const Segment SGI137249[] = {{&S31986[30], 6}, {NULL}};
const Segment SGI137250[] = {{&S31986[31], 3}, {NULL}};
const Segment SGI137251[] = {{&S31986[95], 6}, {NULL}};
const Segment SGI137252[] = {{&S31987[44], 25}, {NULL}};
const Segment SGI137253[] = {{&S31987[44], 37}, {NULL}};
const Segment SGI137254[] = {{&S31987[30], 6}, {NULL}};
const Segment SGI137255[] = {{&S31987[31], 3}, {NULL}};
const Segment SGI137256[] = {{&S31987[95], 6}, {NULL}};
const Segment SGI137257[] = {{&S31988[44], 25}, {NULL}};
const Segment SGI137258[] = {{&S31988[44], 37}, {NULL}};
const Segment SGI137259[] = {{&S31988[30], 6}, {NULL}};
const Segment SGI137260[] = {{&S31988[31], 3}, {NULL}};
const Segment SGI137261[] = {{&S31988[95], 6}, {NULL}};
const Segment SGI137262[] = {{&S31989[44], 25}, {NULL}};
const Segment SGI137263[] = {{&S31989[44], 37}, {NULL}};
const Segment SGI137264[] = {{&S31989[30], 6}, {NULL}};
const Segment SGI137265[] = {{&S31989[31], 3}, {NULL}};
const Segment SGI137266[] = {{&S31989[95], 6}, {NULL}};
const Segment SGI137267[] = {{&S31990[44], 25}, {NULL}};
const Segment SGI137268[] = {{&S31990[44], 37}, {NULL}};
const Segment SGI137269[] = {{&S31990[30], 6}, {NULL}};
const Segment SGI137270[] = {{&S31990[31], 3}, {NULL}};
const Segment SGI137271[] = {{&S31990[95], 6}, {NULL}};
const Segment SGI137272[] = {{&S31991[44], 25}, {NULL}};
const Segment SGI137273[] = {{&S31991[44], 37}, {NULL}};
const Segment SGI137274[] = {{&S31991[30], 6}, {NULL}};
const Segment SGI137275[] = {{&S31991[31], 3}, {NULL}};
const Segment SGI137276[] = {{&S31991[95], 6}, {NULL}};
const Segment SGI137277[] = {{&S31992[44], 25}, {NULL}};
const Segment SGI137278[] = {{&S31992[44], 37}, {NULL}};
const Segment SGI137279[] = {{&S31992[30], 6}, {NULL}};
const Segment SGI137280[] = {{&S31992[31], 3}, {NULL}};
const Segment SGI137281[] = {{&S31992[95], 6}, {NULL}};
const Segment SGI137282[] = {{&S31993[44], 25}, {NULL}};
const Segment SGI137283[] = {{&S31993[44], 37}, {NULL}};
const Segment SGI137284[] = {{&S31993[30], 6}, {NULL}};
const Segment SGI137285[] = {{&S31993[31], 3}, {NULL}};
const Segment SGI137286[] = {{&S31993[95], 6}, {NULL}};
const Segment SGI137287[] = {{&S31994[44], 25}, {NULL}};
const Segment SGI137288[] = {{&S31994[44], 37}, {NULL}};
const Segment SGI137289[] = {{&S31994[30], 6}, {NULL}};
const Segment SGI137290[] = {{&S31994[31], 3}, {NULL}};
const Segment SGI137291[] = {{&S31994[95], 6}, {NULL}};
const Segment SGI137292[] = {{&S31995[44], 34}, {NULL}};
const Segment SGI137293[] = {{&S31995[44], 46}, {NULL}};
const Segment SGI137294[] = {{&S31995[30], 6}, {NULL}};
const Segment SGI137295[] = {{&S31995[31], 3}, {NULL}};
const Segment SGI137296[] = {{&S31995[104], 6}, {NULL}};
const Segment SGI137297[] = {{&S31996[44], 14}, {NULL}};
const Segment SGI137298[] = {{&S31996[44], 24}, {NULL}};
const Segment SGI137299[] = {{&S31996[30], 6}, {NULL}};
const Segment SGI137300[] = {{&S31996[31], 3}, {NULL}};
const Segment SGI137301[] = {{&S31996[82], 6}, {NULL}};
const Segment SGI137302[] = {{&S31997[44], 25}, {NULL}};
const Segment SGI137303[] = {{&S31997[44], 37}, {NULL}};
const Segment SGI137304[] = {{&S31997[30], 6}, {NULL}};
const Segment SGI137305[] = {{&S31997[31], 3}, {NULL}};
const Segment SGI137306[] = {{&S31997[95], 6}, {NULL}};
const Segment SGI137307[] = {{&S31998[44], 15}, {NULL}};
const Segment SGI137308[] = {{&S31998[44], 27}, {NULL}};
const Segment SGI137309[] = {{&S31998[30], 6}, {NULL}};
const Segment SGI137310[] = {{&S31998[31], 3}, {NULL}};
const Segment SGI137311[] = {{&S31998[85], 6}, {NULL}};
const Segment SGI137312[] = {{&S22778[44], 10}, {NULL}};
const Segment SGI137313[] = {{&S22778[44], 22}, {NULL}};
const Segment SGI137314[] = {{&S22778[30], 6}, {NULL}};
const Segment SGI137315[] = {{&S22778[115], 3}, {NULL}};
const Segment SGI137316[] = {{&S22778[80], 6}, {NULL}};
const Segment SGI137317[] = {{&S22779[38], 10}, {NULL}};
const Segment SGI137318[] = {{&S22779[31], 6}, {NULL}};
const Segment SGI137319[] = {{&S32048[38], 7}, {NULL}};
const Segment SGI137320[] = {{&S32048[31], 6}, {NULL}};
const Segment SGI137321[] = {{&S32049[44], 7}, {NULL}};
const Segment SGI137322[] = {{&S32049[44], 25}, {NULL}};
const Segment SGI137323[] = {{&S32049[30], 6}, {NULL}};
const Segment SGI137324[] = {{&S32049[31], 3}, {NULL}};
const Segment SGI137325[] = {{&S32049[83], 6}, {NULL}};
const Segment SGI137326[] = {{&S21483[44], 14}, {NULL}};
const Segment SGI137327[] = {{&S21483[44], 26}, {NULL}};
const Segment SGI137328[] = {{&S21483[30], 6}, {NULL}};
const Segment SGI137329[] = {{&S21483[119], 3}, {NULL}};
const Segment SGI137330[] = {{&S21483[84], 6}, {NULL}};
const Segment SGI137331[] = {{&S21484[44], 14}, {NULL}};
const Segment SGI137332[] = {{&S21484[44], 26}, {NULL}};
const Segment SGI137333[] = {{&S21484[30], 6}, {NULL}};
const Segment SGI137334[] = {{&S21484[119], 3}, {NULL}};
const Segment SGI137335[] = {{&S21484[84], 6}, {NULL}};
const Segment SGI137336[] = {{&S32052[44], 7}, {NULL}};
const Segment SGI137337[] = {{&S32052[44], 25}, {NULL}};
const Segment SGI137338[] = {{&S32052[30], 6}, {NULL}};
const Segment SGI137339[] = {{&S21485[44], 14}, {NULL}};
const Segment SGI137340[] = {{&S21485[44], 26}, {NULL}};
const Segment SGI137341[] = {{&S21485[30], 6}, {NULL}};
const Segment SGI137342[] = {{&S32052[31], 3}, {NULL}};
const Segment SGI137343[] = {{&S32052[83], 6}, {NULL}};
const Segment SGI137344[] = {{&S21485[119], 3}, {NULL}};
const Segment SGI137345[] = {{&S21485[84], 6}, {NULL}};
const Segment SGI137346[] = {{&S32053[44], 7}, {NULL}};
const Segment SGI137347[] = {{&S32053[44], 25}, {NULL}};
const Segment SGI137348[] = {{&S32053[30], 6}, {NULL}};
const Segment SGI137349[] = {{&S21486[38], 14}, {NULL}};
const Segment SGI137350[] = {{&S21486[31], 6}, {NULL}};
const Segment SGI137351[] = {{&S32053[31], 3}, {NULL}};
const Segment SGI137352[] = {{&S32053[83], 6}, {NULL}};
const Segment SGI137353[] = {{&S21487[44], 14}, {NULL}};
const Segment SGI137354[] = {{&S21487[44], 26}, {NULL}};
const Segment SGI137355[] = {{&S21487[30], 6}, {NULL}};
const Segment SGI137356[] = {{&S21487[119], 3}, {NULL}};
const Segment SGI137357[] = {{&S21487[84], 6}, {NULL}};
const Segment SGI137358[] = {{&S32055[44], 7}, {NULL}};
const Segment SGI137359[] = {{&S32055[44], 25}, {NULL}};
const Segment SGI137360[] = {{&S32055[30], 6}, {NULL}};
const Segment SGI137361[] = {{&S32055[31], 3}, {NULL}};
const Segment SGI137362[] = {{&S32055[83], 6}, {NULL}};
const Segment SGI137363[] = {{&S21488[44], 14}, {NULL}};
const Segment SGI137364[] = {{&S21488[44], 26}, {NULL}};
const Segment SGI137365[] = {{&S21488[30], 6}, {NULL}};
const Segment SGI137366[] = {{&S21488[87], 6}, {NULL}};
const Segment SGI137367[] = {{&S32056[38], 7}, {NULL}};
const Segment SGI137368[] = {{&S32056[31], 6}, {NULL}};
const Segment SGI137369[] = {{&S21490[44], 14}, {NULL}};
const Segment SGI137370[] = {{&S21490[44], 37}, {NULL}};
const Segment SGI137371[] = {{&S21490[30], 6}, {NULL}};
const Segment SGI137372[] = {{&S21490[130], 3}, {NULL}};
const Segment SGI137373[] = {{&S21490[95], 6}, {NULL}};
const Segment SGI137374[] = {{&S21494[44], 14}, {NULL}};
const Segment SGI137375[] = {{&S21494[44], 26}, {NULL}};
const Segment SGI137376[] = {{&S21494[30], 6}, {NULL}};
const Segment SGI137377[] = {{&S21494[119], 3}, {NULL}};
const Segment SGI137378[] = {{&S21494[84], 6}, {NULL}};
const Segment SGI137379[] = {{&S32067[44], 9}, {NULL}};
const Segment SGI137380[] = {{&S32067[44], 21}, {NULL}};
const Segment SGI137381[] = {{&S32067[30], 6}, {NULL}};
const Segment SGI137382[] = {{&S32067[31], 3}, {NULL}};
const Segment SGI137383[] = {{&S32067[79], 6}, {NULL}};
const Segment SGI137384[] = {{&S32069[44], 9}, {NULL}};
const Segment SGI137385[] = {{&S32069[44], 21}, {NULL}};
const Segment SGI137386[] = {{&S32069[30], 6}, {NULL}};
const Segment SGI137387[] = {{&S32069[31], 3}, {NULL}};
const Segment SGI137388[] = {{&S32069[79], 6}, {NULL}};
const Segment SGI137389[] = {{&S32070[44], 18}, {NULL}};
const Segment SGI137390[] = {{&S32070[44], 30}, {NULL}};
const Segment SGI137391[] = {{&S32070[30], 6}, {NULL}};
const Segment SGI137392[] = {{&S32070[31], 3}, {NULL}};
const Segment SGI137393[] = {{&S32070[88], 6}, {NULL}};
const Segment SGI137394[] = {{&S32072[44], 18}, {NULL}};
const Segment SGI137395[] = {{&S32072[44], 30}, {NULL}};
const Segment SGI137396[] = {{&S32072[30], 6}, {NULL}};
const Segment SGI137397[] = {{&S32072[31], 3}, {NULL}};
const Segment SGI137398[] = {{&S32072[88], 6}, {NULL}};
const Segment SGI137399[] = {{&S32101[44], 19}, {NULL}};
const Segment SGI137400[] = {{&S32101[44], 31}, {NULL}};
const Segment SGI137401[] = {{&S32101[30], 6}, {NULL}};
const Segment SGI137402[] = {{&S32101[31], 3}, {NULL}};
const Segment SGI137403[] = {{&S32101[89], 6}, {NULL}};
const Segment SGI137404[] = {{&S32104[44], 7}, {NULL}};
const Segment SGI137405[] = {{&S32104[44], 24}, {NULL}};
const Segment SGI137406[] = {{&S32104[30], 6}, {NULL}};
const Segment SGI137407[] = {{&S32104[31], 3}, {NULL}};
const Segment SGI137408[] = {{&S32104[82], 6}, {NULL}};
const Segment SGI137409[] = {{&S32105[44], 7}, {NULL}};
const Segment SGI137410[] = {{&S32105[44], 24}, {NULL}};
const Segment SGI137411[] = {{&S32105[30], 6}, {NULL}};
const Segment SGI137412[] = {{&S32105[31], 3}, {NULL}};
const Segment SGI137413[] = {{&S32105[82], 6}, {NULL}};
const Segment SGI137414[] = {{&S32107[44], 7}, {NULL}};
const Segment SGI137415[] = {{&S32107[44], 24}, {NULL}};
const Segment SGI137416[] = {{&S32107[30], 6}, {NULL}};
const Segment SGI137417[] = {{&S32107[31], 3}, {NULL}};
const Segment SGI137418[] = {{&S32107[82], 6}, {NULL}};
const Segment SGI137419[] = {{&S32110[38], 7}, {NULL}};
const Segment SGI137420[] = {{&S32110[31], 6}, {NULL}};
const Segment SGI137421[] = {{&S32111[44], 7}, {NULL}};
const Segment SGI137422[] = {{&S32111[44], 39}, {NULL}};
const Segment SGI137423[] = {{&S32111[30], 6}, {NULL}};
const Segment SGI137424[] = {{&S32111[31], 3}, {NULL}};
const Segment SGI137425[] = {{&S32111[97], 6}, {NULL}};
const Segment SGI137426[] = {{&S32112[44], 7}, {NULL}};
const Segment SGI137427[] = {{&S32112[44], 24}, {NULL}};
const Segment SGI137428[] = {{&S32112[30], 6}, {NULL}};
const Segment SGI137429[] = {{&S32112[31], 3}, {NULL}};
const Segment SGI137430[] = {{&S32112[82], 6}, {NULL}};
const Segment SGI137431[] = {{&S32113[44], 7}, {NULL}};
const Segment SGI137432[] = {{&S32113[44], 39}, {NULL}};
const Segment SGI137433[] = {{&S32113[30], 6}, {NULL}};
const Segment SGI137434[] = {{&S32113[31], 3}, {NULL}};
const Segment SGI137435[] = {{&S32113[97], 6}, {NULL}};
const Segment SGI137436[] = {{&S32114[38], 7}, {NULL}};
const Segment SGI137437[] = {{&S32114[31], 6}, {NULL}};
const Segment SGI137438[] = {{&S21597[44], 11}, {NULL}};
const Segment SGI137439[] = {{&S21597[44], 23}, {NULL}};
const Segment SGI137440[] = {{&S21597[30], 6}, {NULL}};
const Segment SGI137441[] = {{&S21597[116], 3}, {NULL}};
const Segment SGI137442[] = {{&S21597[81], 6}, {NULL}};
const Segment SGI137443[] = {{&S21606[38], 11}, {NULL}};
const Segment SGI137444[] = {{&S21606[31], 6}, {NULL}};
const Segment SGI137445[] = {{&S32196[44], 10}, {NULL}};
const Segment SGI137446[] = {{&S32196[44], 22}, {NULL}};
const Segment SGI137447[] = {{&S32196[30], 6}, {NULL}};
const Segment SGI137448[] = {{&S32196[31], 3}, {NULL}};
const Segment SGI137449[] = {{&S32196[80], 6}, {NULL}};
const Segment SGI137450[] = {{&S32197[38], 10}, {NULL}};
const Segment SGI137451[] = {{&S32197[31], 6}, {NULL}};
const Segment SGI137452[] = {{&S32198[44], 10}, {NULL}};
const Segment SGI137453[] = {{&S32198[44], 22}, {NULL}};
const Segment SGI137454[] = {{&S32198[30], 6}, {NULL}};
const Segment SGI137455[] = {{&S32198[31], 3}, {&S32198[175], 3}, {NULL}};
const Segment SGI137456[] = {{&S32198[80], 6}, {NULL}};
const Segment SGI137457[] = {{&S32199[42], 10}, {NULL}};
const Segment SGI137458[] = {{&S32199[30], 6}, {NULL}};
const Segment SGI137459[] = {{&S32199[31], 3}, {NULL}};
const Segment SGI137460[] = {{&S32199[78], 6}, {NULL}};
const Segment SGI137461[] = {{&S32200[44], 10}, {NULL}};
const Segment SGI137462[] = {{&S32200[44], 22}, {NULL}};
const Segment SGI137463[] = {{&S32200[30], 6}, {NULL}};
const Segment SGI137464[] = {{&S21632[44], 11}, {NULL}};
const Segment SGI137465[] = {{&S21632[44], 23}, {NULL}};
const Segment SGI137466[] = {{&S21632[30], 6}, {NULL}};
const Segment SGI137467[] = {{&S32200[31], 3}, {&S32200[158], 3}, {NULL}};
const Segment SGI137468[] = {{&S21632[116], 3}, {NULL}};
const Segment SGI137469[] = {{&S32200[80], 6}, {NULL}};
const Segment SGI137470[] = {{&S21632[81], 6}, {NULL}};
const Segment SGI137471[] = {{&S32407[44], 6}, {NULL}};
const Segment SGI137472[] = {{&S32407[44], 17}, {NULL}};
const Segment SGI137473[] = {{&S32407[30], 6}, {NULL}};
const Segment SGI137474[] = {{&S32407[31], 3}, {NULL}};
const Segment SGI137475[] = {{&S32407[75], 6}, {NULL}};
const Segment SGI137476[] = {{&S32408[44], 6}, {NULL}};
const Segment SGI137477[] = {{&S32408[44], 17}, {NULL}};
const Segment SGI137478[] = {{&S32408[30], 6}, {NULL}};
const Segment SGI137479[] = {{&S32408[31], 3}, {NULL}};
const Segment SGI137480[] = {{&S32408[75], 6}, {NULL}};
const Segment SGI137481[] = {{&S32410[44], 6}, {NULL}};
const Segment SGI137482[] = {{&S32410[44], 18}, {NULL}};
const Segment SGI137483[] = {{&S32410[30], 6}, {NULL}};
const Segment SGI137484[] = {{&S32410[31], 3}, {NULL}};
const Segment SGI137485[] = {{&S32410[76], 6}, {NULL}};
const Segment SGI137486[] = {{&S32411[44], 6}, {NULL}};
const Segment SGI137487[] = {{&S32411[44], 18}, {NULL}};
const Segment SGI137488[] = {{&S32411[30], 6}, {NULL}};
const Segment SGI137489[] = {{&S32411[31], 3}, {NULL}};
const Segment SGI137490[] = {{&S32411[76], 6}, {NULL}};
const Segment SGI137491[] = {{&S32419[44], 6}, {NULL}};
const Segment SGI137492[] = {{&S32419[44], 17}, {NULL}};
const Segment SGI137493[] = {{&S32419[30], 6}, {NULL}};
const Segment SGI137494[] = {{&S32419[31], 3}, {NULL}};
const Segment SGI137495[] = {{&S32419[75], 6}, {NULL}};
const Segment SGI137496[] = {{&S32422[44], 6}, {NULL}};
const Segment SGI137497[] = {{&S32422[44], 17}, {NULL}};
const Segment SGI137498[] = {{&S32422[30], 6}, {NULL}};
const Segment SGI137499[] = {{&S32422[31], 3}, {NULL}};
const Segment SGI137500[] = {{&S32422[75], 6}, {NULL}};
const Segment SGI137501[] = {{&S32424[44], 6}, {NULL}};
const Segment SGI137502[] = {{&S32424[44], 17}, {NULL}};
const Segment SGI137503[] = {{&S32424[30], 6}, {NULL}};
const Segment SGI137504[] = {{&S32424[31], 3}, {NULL}};
const Segment SGI137505[] = {{&S32424[75], 6}, {NULL}};
const Segment SGI137506[] = {{&S32426[44], 6}, {NULL}};
const Segment SGI137507[] = {{&S32426[44], 17}, {NULL}};
const Segment SGI137508[] = {{&S32426[30], 6}, {NULL}};
const Segment SGI137509[] = {{&S32426[31], 3}, {NULL}};
const Segment SGI137510[] = {{&S32426[75], 6}, {NULL}};
const Segment SGI137511[] = {{&S32427[44], 6}, {NULL}};
const Segment SGI137512[] = {{&S32427[44], 17}, {NULL}};
const Segment SGI137513[] = {{&S32427[30], 6}, {NULL}};
const Segment SGI137514[] = {{&S32427[31], 3}, {NULL}};
const Segment SGI137515[] = {{&S32427[75], 6}, {NULL}};
const Segment SGI137516[] = {{&S32430[44], 6}, {NULL}};
const Segment SGI137517[] = {{&S32430[44], 17}, {NULL}};
const Segment SGI137518[] = {{&S32430[30], 6}, {NULL}};
const Segment SGI137519[] = {{&S32430[31], 3}, {NULL}};
const Segment SGI137520[] = {{&S32430[75], 6}, {NULL}};
const Segment SGI137521[] = {{&S32431[44], 6}, {NULL}};
const Segment SGI137522[] = {{&S32431[44], 17}, {NULL}};
const Segment SGI137523[] = {{&S32431[30], 6}, {NULL}};
const Segment SGI137524[] = {{&S32431[31], 3}, {NULL}};
const Segment SGI137525[] = {{&S32431[75], 6}, {NULL}};
const Segment SGI137526[] = {{&S32432[44], 6}, {NULL}};
const Segment SGI137527[] = {{&S32432[44], 18}, {NULL}};
const Segment SGI137528[] = {{&S32432[30], 6}, {NULL}};
const Segment SGI137529[] = {{&S32432[31], 3}, {NULL}};
const Segment SGI137530[] = {{&S32432[76], 6}, {NULL}};
const Segment SGI137531[] = {{&S32433[44], 6}, {NULL}};
const Segment SGI137532[] = {{&S32433[44], 18}, {NULL}};
const Segment SGI137533[] = {{&S32433[30], 6}, {NULL}};
const Segment SGI137534[] = {{&S32433[31], 3}, {NULL}};
const Segment SGI137535[] = {{&S32433[76], 6}, {NULL}};
const Segment SGI137536[] = {{&S32434[44], 6}, {NULL}};
const Segment SGI137537[] = {{&S32434[44], 18}, {NULL}};
const Segment SGI137538[] = {{&S32434[30], 6}, {NULL}};
const Segment SGI137539[] = {{&S32434[31], 3}, {NULL}};
const Segment SGI137540[] = {{&S32434[76], 6}, {NULL}};
const Segment SGI137541[] = {{&S32435[38], 6}, {NULL}};
const Segment SGI137542[] = {{&S32435[31], 6}, {NULL}};
const Segment SGI137543[] = {{&S32436[44], 6}, {NULL}};
const Segment SGI137544[] = {{&S32436[44], 18}, {NULL}};
const Segment SGI137545[] = {{&S32436[30], 6}, {NULL}};
const Segment SGI137546[] = {{&S32436[31], 3}, {NULL}};
const Segment SGI137547[] = {{&S32436[76], 6}, {NULL}};
const Segment SGI137548[] = {{&S32437[44], 6}, {NULL}};
const Segment SGI137549[] = {{&S32437[44], 17}, {NULL}};
const Segment SGI137550[] = {{&S32437[30], 6}, {NULL}};
const Segment SGI137551[] = {{&S32437[31], 3}, {NULL}};
const Segment SGI137552[] = {{&S32437[75], 6}, {NULL}};
const Segment SGI137553[] = {{&S32438[44], 7}, {NULL}};
const Segment SGI137554[] = {{&S32438[44], 23}, {NULL}};
const Segment SGI137555[] = {{&S32438[30], 6}, {NULL}};
const Segment SGI137556[] = {{&S32438[31], 3}, {NULL}};
const Segment SGI137557[] = {{&S32438[81], 6}, {NULL}};
const Segment SGI137558[] = {{&S32440[38], 7}, {NULL}};
const Segment SGI137559[] = {{&S32440[31], 6}, {NULL}};
const Segment SGI137560[] = {{&S32492[44], 7}, {NULL}};
const Segment SGI137561[] = {{&S32492[44], 19}, {NULL}};
const Segment SGI137562[] = {{&S32492[30], 6}, {NULL}};
const Segment SGI137563[] = {{&S32492[31], 3}, {NULL}};
const Segment SGI137564[] = {{&S32492[77], 6}, {NULL}};
const Segment SGI137565[] = {{&S32496[44], 14}, {NULL}};
const Segment SGI137566[] = {{&S32496[44], 29}, {NULL}};
const Segment SGI137567[] = {{&S32496[30], 6}, {NULL}};
const Segment SGI137568[] = {{&S32496[31], 3}, {NULL}};
const Segment SGI137569[] = {{&S32496[87], 6}, {NULL}};
const Segment SGI137570[] = {{&S32497[44], 17}, {NULL}};
const Segment SGI137571[] = {{&S32497[44], 41}, {NULL}};
const Segment SGI137572[] = {{&S32497[30], 6}, {NULL}};
const Segment SGI137573[] = {{&S32497[31], 3}, {NULL}};
const Segment SGI137574[] = {{&S32497[99], 6}, {NULL}};
const Segment SGI137575[] = {{&S32498[44], 17}, {NULL}};
const Segment SGI137576[] = {{&S32498[44], 32}, {NULL}};
const Segment SGI137577[] = {{&S32498[30], 6}, {NULL}};
const Segment SGI137578[] = {{&S32498[31], 3}, {NULL}};
const Segment SGI137579[] = {{&S32498[90], 6}, {NULL}};
const Segment SGI137580[] = {{&S32499[44], 14}, {NULL}};
const Segment SGI137581[] = {{&S32499[44], 38}, {NULL}};
const Segment SGI137582[] = {{&S32499[30], 6}, {NULL}};
const Segment SGI137583[] = {{&S32499[31], 3}, {NULL}};
const Segment SGI137584[] = {{&S32499[96], 6}, {NULL}};
const Segment SGI137585[] = {{&S32500[44], 17}, {NULL}};
const Segment SGI137586[] = {{&S32500[44], 41}, {NULL}};
const Segment SGI137587[] = {{&S32500[30], 6}, {NULL}};
const Segment SGI137588[] = {{&S32500[31], 3}, {NULL}};
const Segment SGI137589[] = {{&S32500[99], 6}, {NULL}};
const Segment SGI137590[] = {{&S32501[44], 17}, {NULL}};
const Segment SGI137591[] = {{&S32501[44], 41}, {NULL}};
const Segment SGI137592[] = {{&S32501[30], 6}, {NULL}};
const Segment SGI137593[] = {{&S32501[31], 3}, {NULL}};
const Segment SGI137594[] = {{&S32501[99], 6}, {NULL}};
const Segment SGI137595[] = {{&S32502[44], 14}, {NULL}};
const Segment SGI137596[] = {{&S32502[44], 38}, {NULL}};
const Segment SGI137597[] = {{&S32502[30], 6}, {NULL}};
const Segment SGI137598[] = {{&S32502[31], 3}, {NULL}};
const Segment SGI137599[] = {{&S32502[96], 6}, {NULL}};
const Segment SGI137600[] = {{&S32504[44], 14}, {NULL}};
const Segment SGI137601[] = {{&S32504[44], 38}, {NULL}};
const Segment SGI137602[] = {{&S32504[30], 6}, {NULL}};
const Segment SGI137603[] = {{&S32504[31], 3}, {NULL}};
const Segment SGI137604[] = {{&S32504[96], 6}, {NULL}};
const Segment SGI137605[] = {{&S32505[44], 14}, {NULL}};
const Segment SGI137606[] = {{&S32505[44], 38}, {NULL}};
const Segment SGI137607[] = {{&S32505[30], 6}, {NULL}};
const Segment SGI137608[] = {{&S32505[31], 3}, {NULL}};
const Segment SGI137609[] = {{&S32505[96], 6}, {NULL}};
const Segment SGI137610[] = {{&S32506[44], 17}, {NULL}};
const Segment SGI137611[] = {{&S32506[44], 41}, {NULL}};
const Segment SGI137612[] = {{&S32506[30], 6}, {NULL}};
const Segment SGI137613[] = {{&S32506[31], 3}, {NULL}};
const Segment SGI137614[] = {{&S32506[99], 6}, {NULL}};
const Segment SGI137615[] = {{&S32507[44], 17}, {NULL}};
const Segment SGI137616[] = {{&S32507[44], 42}, {NULL}};
const Segment SGI137617[] = {{&S32507[30], 6}, {NULL}};
const Segment SGI137618[] = {{&S32507[31], 3}, {NULL}};
const Segment SGI137619[] = {{&S32507[100], 6}, {NULL}};
const Segment SGI137620[] = {{&S32508[44], 17}, {NULL}};
const Segment SGI137621[] = {{&S32508[44], 42}, {NULL}};
const Segment SGI137622[] = {{&S32508[30], 6}, {NULL}};
const Segment SGI137623[] = {{&S32508[31], 3}, {NULL}};
const Segment SGI137624[] = {{&S32508[100], 6}, {NULL}};
const Segment SGI137625[] = {{&S32509[44], 17}, {NULL}};
const Segment SGI137626[] = {{&S32509[44], 42}, {NULL}};
const Segment SGI137627[] = {{&S32509[30], 6}, {NULL}};
const Segment SGI137628[] = {{&S32509[31], 3}, {NULL}};
const Segment SGI137629[] = {{&S32509[100], 6}, {NULL}};
const Segment SGI137630[] = {{&S32510[44], 14}, {NULL}};
const Segment SGI137631[] = {{&S32510[44], 40}, {NULL}};
const Segment SGI137632[] = {{&S32510[30], 6}, {NULL}};
const Segment SGI137633[] = {{&S32510[31], 3}, {NULL}};
const Segment SGI137634[] = {{&S32510[98], 6}, {NULL}};
const Segment SGI137635[] = {{&S32511[44], 17}, {NULL}};
const Segment SGI137636[] = {{&S32511[44], 42}, {NULL}};
const Segment SGI137637[] = {{&S32511[30], 6}, {NULL}};
const Segment SGI137638[] = {{&S32511[31], 3}, {NULL}};
const Segment SGI137639[] = {{&S32511[100], 6}, {NULL}};
const Segment SGI137640[] = {{&S32512[44], 14}, {NULL}};
const Segment SGI137641[] = {{&S32512[44], 39}, {NULL}};
const Segment SGI137642[] = {{&S32512[30], 6}, {NULL}};
const Segment SGI137643[] = {{&S32512[31], 3}, {NULL}};
const Segment SGI137644[] = {{&S32512[97], 6}, {NULL}};
const Segment SGI137645[] = {{&S32513[38], 14}, {NULL}};
const Segment SGI137646[] = {{&S32513[31], 6}, {NULL}};
const Segment SGI137647[] = {{&S32514[44], 14}, {NULL}};
const Segment SGI137648[] = {{&S32514[44], 39}, {NULL}};
const Segment SGI137649[] = {{&S32514[30], 6}, {NULL}};
const Segment SGI137650[] = {{&S32514[31], 3}, {NULL}};
const Segment SGI137651[] = {{&S32514[97], 6}, {NULL}};
const Segment SGI137652[] = {{&S32515[44], 17}, {NULL}};
const Segment SGI137653[] = {{&S32515[44], 42}, {NULL}};
const Segment SGI137654[] = {{&S32515[30], 6}, {NULL}};
const Segment SGI137655[] = {{&S32515[31], 3}, {NULL}};
const Segment SGI137656[] = {{&S32515[100], 6}, {NULL}};
const Segment SGI137657[] = {{&S32516[44], 14}, {NULL}};
const Segment SGI137658[] = {{&S32516[44], 39}, {NULL}};
const Segment SGI137659[] = {{&S32516[30], 6}, {NULL}};
const Segment SGI137660[] = {{&S32516[97], 6}, {NULL}};
const Segment SGI137661[] = {{&S32517[44], 14}, {NULL}};
const Segment SGI137662[] = {{&S32517[44], 39}, {NULL}};
const Segment SGI137663[] = {{&S32517[30], 6}, {NULL}};
const Segment SGI137664[] = {{&S32517[31], 3}, {NULL}};
const Segment SGI137665[] = {{&S32517[97], 6}, {NULL}};
const Segment SGI137666[] = {{&S32518[44], 14}, {NULL}};
const Segment SGI137667[] = {{&S32518[44], 39}, {NULL}};
const Segment SGI137668[] = {{&S32518[30], 6}, {NULL}};
const Segment SGI137669[] = {{&S32518[31], 3}, {NULL}};
const Segment SGI137670[] = {{&S32518[97], 6}, {NULL}};
const Segment SGI137671[] = {{&S32519[44], 17}, {NULL}};
const Segment SGI137672[] = {{&S32519[44], 42}, {NULL}};
const Segment SGI137673[] = {{&S32519[30], 6}, {NULL}};
const Segment SGI137674[] = {{&S32519[31], 3}, {NULL}};
const Segment SGI137675[] = {{&S32519[100], 6}, {NULL}};
const Segment SGI137676[] = {{&S32520[44], 17}, {NULL}};
const Segment SGI137677[] = {{&S32520[44], 26}, {NULL}};
const Segment SGI137678[] = {{&S32520[30], 6}, {NULL}};
const Segment SGI137679[] = {{&S32520[31], 3}, {NULL}};
const Segment SGI137680[] = {{&S32520[84], 6}, {NULL}};
const Segment SGI137681[] = {{&S32521[44], 14}, {NULL}};
const Segment SGI137682[] = {{&S32521[44], 23}, {NULL}};
const Segment SGI137683[] = {{&S32521[30], 6}, {NULL}};
const Segment SGI137684[] = {{&S32521[31], 3}, {NULL}};
const Segment SGI137685[] = {{&S32521[81], 6}, {NULL}};
const Segment SGI137686[] = {{&S32522[44], 17}, {NULL}};
const Segment SGI137687[] = {{&S32522[44], 42}, {NULL}};
const Segment SGI137688[] = {{&S32522[30], 6}, {NULL}};
const Segment SGI137689[] = {{&S32522[31], 3}, {NULL}};
const Segment SGI137690[] = {{&S32522[100], 6}, {NULL}};
const Segment SGI137691[] = {{&S32523[44], 17}, {NULL}};
const Segment SGI137692[] = {{&S32523[44], 26}, {NULL}};
const Segment SGI137693[] = {{&S32523[30], 6}, {NULL}};
const Segment SGI137694[] = {{&S32523[31], 3}, {NULL}};
const Segment SGI137695[] = {{&S32523[84], 6}, {NULL}};
const Segment SGI137696[] = {{&S32524[44], 14}, {NULL}};
const Segment SGI137697[] = {{&S32524[44], 23}, {NULL}};
const Segment SGI137698[] = {{&S32524[30], 6}, {NULL}};
const Segment SGI137699[] = {{&S32524[31], 3}, {NULL}};
const Segment SGI137700[] = {{&S32524[81], 6}, {NULL}};
const Segment SGI137701[] = {{&S32525[44], 14}, {NULL}};
const Segment SGI137702[] = {{&S32525[44], 35}, {NULL}};
const Segment SGI137703[] = {{&S32525[30], 6}, {NULL}};
const Segment SGI137704[] = {{&S32525[31], 3}, {NULL}};
const Segment SGI137705[] = {{&S32525[93], 6}, {NULL}};
const Segment SGI137706[] = {{&S32526[44], 17}, {NULL}};
const Segment SGI137707[] = {{&S32526[44], 38}, {NULL}};
const Segment SGI137708[] = {{&S32526[30], 6}, {NULL}};
const Segment SGI137709[] = {{&S32526[31], 3}, {NULL}};
const Segment SGI137710[] = {{&S32526[96], 6}, {NULL}};
const Segment SGI137711[] = {{&S32527[42], 14}, {NULL}};
const Segment SGI137712[] = {{&S32527[30], 6}, {NULL}};
const Segment SGI137713[] = {{&S32527[31], 3}, {NULL}};
const Segment SGI137714[] = {{&S32527[95], 6}, {NULL}};
const Segment SGI137715[] = {{&S32528[44], 17}, {NULL}};
const Segment SGI137716[] = {{&S32528[44], 44}, {NULL}};
const Segment SGI137717[] = {{&S32528[30], 6}, {NULL}};
const Segment SGI137718[] = {{&S32528[31], 3}, {NULL}};
const Segment SGI137719[] = {{&S32528[102], 6}, {NULL}};
const Segment SGI137720[] = {{&S32529[44], 14}, {NULL}};
const Segment SGI137721[] = {{&S32529[44], 41}, {NULL}};
const Segment SGI137722[] = {{&S32529[30], 6}, {NULL}};
const Segment SGI137723[] = {{&S32529[31], 3}, {NULL}};
const Segment SGI137724[] = {{&S32529[99], 6}, {NULL}};
const Segment SGI137725[] = {{&S32760[44], 7}, {NULL}};
const Segment SGI137726[] = {{&S32760[44], 18}, {NULL}};
const Segment SGI137727[] = {{&S32760[30], 6}, {NULL}};
const Segment SGI137728[] = {{&S32760[31], 3}, {NULL}};
const Segment SGI137729[] = {{&S32760[76], 6}, {NULL}};
const Segment SGI137730[] = {{&S32761[38], 7}, {NULL}};
const Segment SGI137731[] = {{&S32761[31], 6}, {NULL}};
const Segment SGI137732[] = {{&S32762[44], 7}, {NULL}};
const Segment SGI137733[] = {{&S32762[44], 19}, {NULL}};
const Segment SGI137734[] = {{&S32762[30], 6}, {NULL}};
const Segment SGI137735[] = {{&S32762[31], 3}, {NULL}};
const Segment SGI137736[] = {{&S32762[77], 6}, {NULL}};
const Segment SGI137737[] = {{&S32763[38], 7}, {NULL}};
const Segment SGI137738[] = {{&S32763[31], 6}, {NULL}};
const Segment SGI137739[] = {{&S32764[42], 7}, {NULL}};
const Segment SGI137740[] = {{&S32764[30], 6}, {NULL}};
const Segment SGI137741[] = {{&S32764[31], 3}, {NULL}};
const Segment SGI137742[] = {{&S32764[74], 6}, {NULL}};
const Segment SGI137743[] = {{&S32765[44], 7}, {NULL}};
const Segment SGI137744[] = {{&S32765[44], 18}, {NULL}};
const Segment SGI137745[] = {{&S32765[30], 6}, {NULL}};
const Segment SGI137746[] = {{&S32765[31], 3}, {&S32765[154], 3}, {NULL}};
const Segment SGI137747[] = {{&S32765[76], 6}, {NULL}};
const Segment SGI137748[] = {{&S32766[44], 7}, {NULL}};
const Segment SGI137749[] = {{&S32766[44], 18}, {NULL}};
const Segment SGI137750[] = {{&S32766[30], 6}, {NULL}};
const Segment SGI137751[] = {{&S32766[31], 3}, {&S32766[154], 3}, {NULL}};
const Segment SGI137752[] = {{&S32766[76], 6}, {NULL}};
const Segment SGI137753[] = {{&S32827[34], 11}, {NULL}};
const Segment SGI137754[] = {{&S32827[27], 6}, {NULL}};
const Segment SGI137755[] = {{&S32827[98], 9}, {NULL}};
const Segment SGI137756[] = {{&S32827[71], 6}, {NULL}};
const Segment SGI137757[] = {{&S32832[34], 10}, {NULL}};
const Segment SGI137758[] = {{&S32832[27], 6}, {NULL}};
const Segment SGI137759[] = {{&S32832[97], 9}, {NULL}};
const Segment SGI137760[] = {{&S32832[70], 6}, {NULL}};
const Segment SGI137761[] = {{&S32838[34], 11}, {NULL}};
const Segment SGI137762[] = {{&S32838[27], 6}, {NULL}};
const Segment SGI137763[] = {{&S32838[98], 9}, {NULL}};
const Segment SGI137764[] = {{&S32838[71], 6}, {NULL}};
const Segment SGI137765[] = {{&S32839[34], 21}, {NULL}};
const Segment SGI137766[] = {{&S32839[27], 6}, {NULL}};
const Segment SGI137767[] = {{&S32839[108], 9}, {NULL}};
const Segment SGI137768[] = {{&S32839[81], 6}, {NULL}};
const Segment SGI137769[] = {{&S32840[34], 10}, {NULL}};
const Segment SGI137770[] = {{&S32840[27], 6}, {NULL}};
const Segment SGI137771[] = {{&S32840[97], 9}, {NULL}};
const Segment SGI137772[] = {{&S32840[70], 6}, {NULL}};
const Segment SGI137773[] = {{&S32841[34], 11}, {NULL}};
const Segment SGI137774[] = {{&S32841[27], 6}, {NULL}};
const Segment SGI137775[] = {{&S32841[98], 9}, {NULL}};
const Segment SGI137776[] = {{&S32841[71], 6}, {NULL}};
const Segment SGI137777[] = {{&S23904[44], 10}, {NULL}};
const Segment SGI137778[] = {{&S23904[44], 22}, {NULL}};
const Segment SGI137779[] = {{&S23904[30], 6}, {NULL}};
const Segment SGI137780[] = {{&S23904[115], 3}, {NULL}};
const Segment SGI137781[] = {{&S23904[80], 6}, {NULL}};
const Segment SGI137782[] = {{&S23905[44], 30}, {NULL}};
const Segment SGI137783[] = {{&S23905[44], 42}, {NULL}};
const Segment SGI137784[] = {{&S23905[30], 6}, {NULL}};
const Segment SGI137785[] = {{&S23905[135], 3}, {NULL}};
const Segment SGI137786[] = {{&S23905[100], 6}, {NULL}};
const Segment SGI137787[] = {{&S23906[42], 10}, {NULL}};
const Segment SGI137788[] = {{&S23906[30], 6}, {NULL}};
const Segment SGI137789[] = {{&S23906[113], 3}, {NULL}};
const Segment SGI137790[] = {{&S23906[78], 6}, {NULL}};
const Segment SGI137791[] = {{&S23907[38], 10}, {NULL}};
const Segment SGI137792[] = {{&S23907[31], 6}, {NULL}};
const Segment SGI137793[] = {{&S23909[44], 11}, {NULL}};
const Segment SGI137794[] = {{&S23909[44], 23}, {NULL}};
const Segment SGI137795[] = {{&S23909[30], 6}, {NULL}};
const Segment SGI137796[] = {{&S23909[116], 3}, {NULL}};
const Segment SGI137797[] = {{&S23909[81], 6}, {NULL}};
const Segment SGI137798[] = {{&S23912[44], 10}, {NULL}};
const Segment SGI137799[] = {{&S23912[44], 22}, {NULL}};
const Segment SGI137800[] = {{&S23912[30], 6}, {NULL}};
const Segment SGI137801[] = {{&S23912[115], 3}, {NULL}};
const Segment SGI137802[] = {{&S23912[80], 6}, {NULL}};
const Segment SGI137803[] = {{&S22477[34], 10}, {NULL}};
const Segment SGI137804[] = {{&S22477[27], 6}, {NULL}};
const Segment SGI137805[] = {{&S22477[97], 9}, {NULL}};
const Segment SGI137806[] = {{&S22477[70], 6}, {NULL}};
const Segment SGI137807[] = {{&S23913[44], 10}, {NULL}};
const Segment SGI137808[] = {{&S23913[44], 22}, {NULL}};
const Segment SGI137809[] = {{&S23913[30], 6}, {NULL}};
const Segment SGI137810[] = {{&S23913[115], 3}, {NULL}};
const Segment SGI137811[] = {{&S23913[80], 6}, {NULL}};
const Segment SGI137812[] = {{&S22478[34], 10}, {NULL}};
const Segment SGI137813[] = {{&S22478[27], 6}, {NULL}};
const Segment SGI137814[] = {{&S22478[97], 9}, {NULL}};
const Segment SGI137815[] = {{&S22478[70], 6}, {NULL}};
const Segment SGI137816[] = {{&S23914[58], 10}, {NULL}};
const Segment SGI137817[] = {{&S23914[58], 34}, {NULL}};
const Segment SGI137818[] = {{&S23914[44], 6}, {NULL}};
const Segment SGI137819[] = {{&S22479[34], 10}, {NULL}};
const Segment SGI137820[] = {{&S22479[27], 6}, {NULL}};
const Segment SGI137821[] = {{&S23915[44], 10}, {NULL}};
const Segment SGI137822[] = {{&S22479[97], 9}, {NULL}};
const Segment SGI137823[] = {{&S22479[70], 6}, {NULL}};
const Segment SGI137824[] = {{&S23915[44], 22}, {NULL}};
const Segment SGI137825[] = {{&S23915[30], 6}, {NULL}};
const Segment SGI137826[] = {{&S23915[80], 6}, {NULL}};
const Segment SGI137827[] = {{&S23916[44], 10}, {NULL}};
const Segment SGI137828[] = {{&S23916[44], 22}, {NULL}};
const Segment SGI137829[] = {{&S23916[30], 6}, {NULL}};
const Segment SGI137830[] = {{&S23916[83], 6}, {NULL}};
const Segment SGI137831[] = {{&S22483[34], 10}, {NULL}};
const Segment SGI137832[] = {{&S22483[27], 6}, {NULL}};
const Segment SGI137833[] = {{&S23917[44], 10}, {NULL}};
const Segment SGI137834[] = {{&S23917[44], 26}, {NULL}};
const Segment SGI137835[] = {{&S22483[97], 9}, {NULL}};
const Segment SGI137836[] = {{&S23917[30], 6}, {NULL}};
const Segment SGI137837[] = {{&S22483[70], 6}, {NULL}};
const Segment SGI137838[] = {{&S23917[119], 3}, {NULL}};
const Segment SGI137839[] = {{&S23917[84], 6}, {NULL}};
const Segment SGI137840[] = {{&S23918[46], 10}, {NULL}};
const Segment SGI137841[] = {{&S23918[46], 34}, {NULL}};
const Segment SGI137842[] = {{&S23918[32], 6}, {NULL}};
const Segment SGI137843[] = {{&S22486[34], 8}, {NULL}};
const Segment SGI137844[] = {{&S22486[27], 6}, {NULL}};
const Segment SGI137845[] = {{&S22486[95], 9}, {NULL}};
const Segment SGI137846[] = {{&S22486[68], 6}, {NULL}};
const Segment SGI137847[] = {{&S23919[44], 33}, {NULL}};
const Segment SGI137848[] = {{&S23919[44], 45}, {NULL}};
const Segment SGI137849[] = {{&S23919[30], 6}, {NULL}};
const Segment SGI137850[] = {{&S23919[138], 3}, {NULL}};
const Segment SGI137851[] = {{&S23919[103], 6}, {NULL}};
const Segment SGI137852[] = {{&S23920[59], 10}, {NULL}};
const Segment SGI137853[] = {{&S23920[59], 34}, {NULL}};
const Segment SGI137854[] = {{&S23920[45], 6}, {NULL}};
const Segment SGI137855[] = {{&S22489[34], 11}, {NULL}};
const Segment SGI137856[] = {{&S22489[27], 6}, {NULL}};
const Segment SGI137857[] = {{&S22489[98], 9}, {NULL}};
const Segment SGI137858[] = {{&S22489[71], 6}, {NULL}};
const Segment SGI137859[] = {{&S23921[44], 10}, {NULL}};
const Segment SGI137860[] = {{&S23921[44], 19}, {NULL}};
const Segment SGI137861[] = {{&S23921[30], 6}, {NULL}};
const Segment SGI137862[] = {{&S23921[111], 3}, {NULL}};
const Segment SGI137863[] = {{&S23921[77], 5}, {NULL}};
const Segment SGI137864[] = {{&S22491[34], 11}, {NULL}};
const Segment SGI137865[] = {{&S22491[27], 6}, {NULL}};
const Segment SGI137866[] = {{&S23922[44], 10}, {NULL}};
const Segment SGI137867[] = {{&S23922[44], 22}, {NULL}};
const Segment SGI137868[] = {{&S23922[30], 6}, {NULL}};
const Segment SGI137869[] = {{&S22491[98], 9}, {NULL}};
const Segment SGI137870[] = {{&S22491[71], 6}, {NULL}};
const Segment SGI137871[] = {{&S23922[115], 3}, {NULL}};
const Segment SGI137872[] = {{&S23922[80], 6}, {NULL}};
const Segment SGI137873[] = {{&S32959[44], 18}, {NULL}};
const Segment SGI137874[] = {{&S32959[44], 30}, {NULL}};
const Segment SGI137875[] = {{&S22492[34], 10}, {NULL}};
const Segment SGI137876[] = {{&S32959[30], 6}, {NULL}};
const Segment SGI137877[] = {{&S22492[27], 6}, {NULL}};
const Segment SGI137878[] = {{&S32959[31], 3}, {NULL}};
const Segment SGI137879[] = {{&S22492[97], 9}, {NULL}};
const Segment SGI137880[] = {{&S22492[70], 6}, {NULL}};
const Segment SGI137881[] = {{&S32959[88], 6}, {NULL}};
const Segment SGI137882[] = {{&S23923[44], 10}, {NULL}};
const Segment SGI137883[] = {{&S23923[44], 22}, {&S23923[234], 5}, {&S23923[221], 1}, {NULL}};
const Segment SGI137884[] = {{&S23923[30], 6}, {NULL}};
const Segment SGI137885[] = {{&S23923[115], 3}, {NULL}};
const Segment SGI137886[] = {{&S23923[80], 6}, {NULL}};
const Segment SGI137887[] = {{&S22493[46], 10}, {NULL}};
const Segment SGI137888[] = {{&S32960[38], 18}, {NULL}};
const Segment SGI137889[] = {{&S22493[27], 18}, {NULL}};
const Segment SGI137890[] = {{&S32960[31], 6}, {NULL}};
const Segment SGI137891[] = {{&S22493[109], 9}, {NULL}};
const Segment SGI137892[] = {{&S22493[82], 6}, {NULL}};
const Segment SGI137893[] = {{&S23924[44], 10}, {NULL}};
const Segment SGI137894[] = {{&S23924[44], 22}, {NULL}};
const Segment SGI137895[] = {{&S23924[30], 6}, {NULL}};
const Segment SGI137896[] = {{&S23924[115], 3}, {NULL}};
const Segment SGI137897[] = {{&S32961[42], 18}, {NULL}};
const Segment SGI137898[] = {{&S22494[34], 11}, {NULL}};
const Segment SGI137899[] = {{&S23924[80], 6}, {NULL}};
const Segment SGI137900[] = {{&S32961[30], 6}, {NULL}};
const Segment SGI137901[] = {{&S22494[27], 6}, {NULL}};
const Segment SGI137902[] = {{&S32961[31], 3}, {NULL}};
const Segment SGI137903[] = {{&S22494[98], 9}, {NULL}};
const Segment SGI137904[] = {{&S32961[86], 6}, {NULL}};
const Segment SGI137905[] = {{&S22494[71], 6}, {NULL}};
const Segment SGI137906[] = {{&S23925[42], 30}, {NULL}};
const Segment SGI137907[] = {{&S23925[30], 6}, {NULL}};
const Segment SGI137908[] = {{&S22495[34], 11}, {NULL}};
const Segment SGI137909[] = {{&S32962[44], 10}, {NULL}};
const Segment SGI137910[] = {{&S22495[27], 6}, {NULL}};
const Segment SGI137911[] = {{&S23925[133], 3}, {NULL}};
const Segment SGI137912[] = {{&S32962[44], 22}, {NULL}};
const Segment SGI137913[] = {{&S32962[30], 6}, {NULL}};
const Segment SGI137914[] = {{&S23925[98], 6}, {NULL}};
const Segment SGI137915[] = {{&S22495[98], 9}, {NULL}};
const Segment SGI137916[] = {{&S32962[31], 3}, {NULL}};
const Segment SGI137917[] = {{&S22495[71], 6}, {NULL}};
const Segment SGI137918[] = {{&S32962[80], 6}, {NULL}};
const Segment SGI137919[] = {{&S22496[34], 10}, {NULL}};
const Segment SGI137920[] = {{&S22496[27], 6}, {NULL}};
const Segment SGI137921[] = {{&S23927[44], 10}, {NULL}};
const Segment SGI137922[] = {{&S23927[44], 22}, {NULL}};
const Segment SGI137923[] = {{&S22496[97], 9}, {NULL}};
const Segment SGI137924[] = {{&S23927[30], 6}, {NULL}};
const Segment SGI137925[] = {{&S22496[70], 6}, {NULL}};
const Segment SGI137926[] = {{&S23927[85], 3}, {NULL}};
const Segment SGI137927[] = {{&S32964[44], 18}, {NULL}};
const Segment SGI137928[] = {{&S32964[44], 30}, {NULL}};
const Segment SGI137929[] = {{&S32964[30], 6}, {NULL}};
const Segment SGI137930[] = {{&S32964[88], 6}, {NULL}};
const Segment SGI137931[] = {{&S23928[38], 11}, {NULL}};
const Segment SGI137932[] = {{&S23928[31], 6}, {NULL}};
const Segment SGI137933[] = {{&S32965[44], 18}, {NULL}};
const Segment SGI137934[] = {{&S32965[44], 30}, {NULL}};
const Segment SGI137935[] = {{&S32965[30], 6}, {NULL}};
const Segment SGI137936[] = {{&S23929[44], 10}, {NULL}};
const Segment SGI137937[] = {{&S23929[44], 22}, {NULL}};
const Segment SGI137938[] = {{&S32965[31], 3}, {NULL}};
const Segment SGI137939[] = {{&S23929[30], 6}, {NULL}};
const Segment SGI137940[] = {{&S32965[88], 6}, {NULL}};
const Segment SGI137941[] = {{&S23929[83], 6}, {NULL}};
const Segment SGI137942[] = {{&S32966[44], 18}, {NULL}};
const Segment SGI137943[] = {{&S32966[44], 30}, {NULL}};
const Segment SGI137944[] = {{&S32966[30], 6}, {NULL}};
const Segment SGI137945[] = {{&S23930[44], 10}, {NULL}};
const Segment SGI137946[] = {{&S23930[44], 22}, {NULL}};
const Segment SGI137947[] = {{&S23930[30], 6}, {NULL}};
const Segment SGI137948[] = {{&S32966[88], 6}, {NULL}};
const Segment SGI137949[] = {{&S23930[127], 3}, {NULL}};
const Segment SGI137950[] = {{&S23930[80], 6}, {NULL}};
const Segment SGI137951[] = {{&S23933[44], 10}, {&S23933[175], 18}, {NULL}};
const Segment SGI137952[] = {{&S23933[44], 22}, {NULL}};
const Segment SGI137953[] = {{&S23933[30], 6}, {NULL}};
const Segment SGI137954[] = {{&S23933[115], 3}, {NULL}};
const Segment SGI137955[] = {{&S23933[80], 6}, {NULL}};
const Segment SGI137956[] = {{&S23934[44], 10}, {NULL}};
const Segment SGI137957[] = {{&S23934[44], 34}, {NULL}};
const Segment SGI137958[] = {{&S23934[30], 6}, {NULL}};
const Segment SGI137959[] = {{&S23934[126], 3}, {NULL}};
const Segment SGI137960[] = {{&S23934[92], 5}, {NULL}};
const Segment SGI137961[] = {{&S23935[42], 11}, {NULL}};
const Segment SGI137962[] = {{&S23935[30], 6}, {NULL}};
const Segment SGI137963[] = {{&S23935[114], 3}, {NULL}};
const Segment SGI137964[] = {{&S23935[79], 6}, {NULL}};
const Segment SGI137965[] = {{&S23936[42], 10}, {NULL}};
const Segment SGI137966[] = {{&S23936[30], 6}, {NULL}};
const Segment SGI137967[] = {{&S23936[113], 3}, {NULL}};
const Segment SGI137968[] = {{&S23936[78], 6}, {NULL}};
const Segment SGI137969[] = {{&S23937[44], 10}, {NULL}};
const Segment SGI137970[] = {{&S23937[44], 22}, {NULL}};
const Segment SGI137971[] = {{&S23937[30], 6}, {NULL}};
const Segment SGI137972[] = {{&S23937[114], 3}, {NULL}};
const Segment SGI137973[] = {{&S23937[80], 5}, {NULL}};
const Segment SGI137974[] = {{&S23941[44], 22}, {NULL}};
const Segment SGI137975[] = {{&S23941[30], 6}, {NULL}};
const Segment SGI137976[] = {{&S23942[44], 11}, {NULL}};
const Segment SGI137977[] = {{&S23942[44], 23}, {NULL}};
const Segment SGI137978[] = {{&S23942[30], 6}, {NULL}};
const Segment SGI137979[] = {{&S23942[116], 3}, {NULL}};
const Segment SGI137980[] = {{&S23942[81], 6}, {NULL}};
const Segment SGI137981[] = {{&S23947[44], 10}, {NULL}};
const Segment SGI137982[] = {{&S23947[44], 22}, {NULL}};
const Segment SGI137983[] = {{&S23947[30], 6}, {NULL}};
const Segment SGI137984[] = {{&S23947[115], 3}, {NULL}};
const Segment SGI137985[] = {{&S23947[80], 6}, {NULL}};
const Segment SGI137986[] = {{&S23947[135], 8}, {NULL}};
const Segment SGI137987[] = {{&S24160[34], 10}, {NULL}};
const Segment SGI137988[] = {{&S24160[27], 6}, {NULL}};
const Segment SGI137989[] = {{&S24160[97], 9}, {NULL}};
const Segment SGI137990[] = {{&S24160[70], 6}, {NULL}};
const Segment SGI137991[] = {{&S24161[34], 11}, {NULL}};
const Segment SGI137992[] = {{&S24161[27], 6}, {NULL}};
const Segment SGI137993[] = {{&S24161[98], 9}, {NULL}};
const Segment SGI137994[] = {{&S24161[71], 6}, {NULL}};
const Segment SGI137995[] = {{&S24164[34], 18}, {NULL}};
const Segment SGI137996[] = {{&S24164[27], 6}, {NULL}};
const Segment SGI137997[] = {{&S24164[105], 9}, {NULL}};
const Segment SGI137998[] = {{&S24164[78], 6}, {NULL}};
const Segment SGI137999[] = {{&S24170[34], 8}, {NULL}};
const Segment SGI138000[] = {{&S24170[27], 6}, {NULL}};
const Segment SGI138001[] = {{&S24170[95], 9}, {NULL}};
const Segment SGI138002[] = {{&S24170[68], 6}, {NULL}};
const Segment SGI138003[] = {{&S24172[34], 10}, {NULL}};
const Segment SGI138004[] = {{&S24172[27], 6}, {NULL}};
const Segment SGI138005[] = {{&S24172[97], 9}, {NULL}};
const Segment SGI138006[] = {{&S24172[70], 6}, {NULL}};
const Segment SGI138007[] = {{&S24174[34], 10}, {NULL}};
const Segment SGI138008[] = {{&S24174[27], 6}, {NULL}};
const Segment SGI138009[] = {{&S24174[97], 9}, {NULL}};
const Segment SGI138010[] = {{&S24174[70], 6}, {NULL}};
const Segment SGI138011[] = {{&S24176[34], 11}, {NULL}};
const Segment SGI138012[] = {{&S24176[27], 6}, {NULL}};
const Segment SGI138013[] = {{&S24176[98], 9}, {NULL}};
const Segment SGI138014[] = {{&S24176[71], 6}, {NULL}};
const Segment SGI138015[] = {{&S22780[44], 10}, {NULL}};
const Segment SGI138016[] = {{&S22780[44], 22}, {NULL}};
const Segment SGI138017[] = {{&S22780[30], 6}, {NULL}};
const Segment SGI138018[] = {{&S22780[115], 3}, {NULL}};
const Segment SGI138019[] = {{&S22780[80], 6}, {NULL}};
const Segment SGI138020[] = {{&S24180[34], 10}, {NULL}};
const Segment SGI138021[] = {{&S24180[27], 6}, {NULL}};
const Segment SGI138022[] = {{&S22781[38], 10}, {NULL}};
const Segment SGI138023[] = {{&S22781[31], 6}, {NULL}};
const Segment SGI138024[] = {{&S24180[97], 9}, {NULL}};
const Segment SGI138025[] = {{&S24180[70], 6}, {NULL}};
const Segment SGI138026[] = {{&S22782[42], 10}, {NULL}};
const Segment SGI138027[] = {{&S22782[30], 6}, {NULL}};
const Segment SGI138028[] = {{&S22782[113], 3}, {NULL}};
const Segment SGI138029[] = {{&S22782[78], 6}, {NULL}};
const Segment SGI138030[] = {{&S22783[44], 10}, {NULL}};
const Segment SGI138031[] = {{&S22783[44], 22}, {NULL}};
const Segment SGI138032[] = {{&S22783[30], 6}, {NULL}};
const Segment SGI138033[] = {{&S22783[115], 3}, {NULL}};
const Segment SGI138034[] = {{&S22783[80], 6}, {NULL}};
const Segment SGI138035[] = {{&S24185[34], 11}, {NULL}};
const Segment SGI138036[] = {{&S24185[27], 6}, {NULL}};
const Segment SGI138037[] = {{&S24185[98], 9}, {NULL}};
const Segment SGI138038[] = {{&S24185[71], 6}, {NULL}};
const Segment SGI138039[] = {{&S24186[34], 11}, {NULL}};
const Segment SGI138040[] = {{&S24186[27], 6}, {NULL}};
const Segment SGI138041[] = {{&S24186[98], 9}, {NULL}};
const Segment SGI138042[] = {{&S24186[71], 6}, {NULL}};
const Segment SGI138043[] = {{&S24187[34], 10}, {NULL}};
const Segment SGI138044[] = {{&S24187[27], 6}, {NULL}};
const Segment SGI138045[] = {{&S24187[97], 9}, {NULL}};
const Segment SGI138046[] = {{&S24187[70], 6}, {NULL}};
const Segment SGI138047[] = {{&S24190[34], 10}, {NULL}};
const Segment SGI138048[] = {{&S24190[27], 6}, {NULL}};
const Segment SGI138049[] = {{&S24190[97], 9}, {NULL}};
const Segment SGI138050[] = {{&S24190[70], 6}, {NULL}};
const Segment SGI138051[] = {{&S24191[34], 11}, {NULL}};
const Segment SGI138052[] = {{&S24191[27], 6}, {NULL}};
const Segment SGI138053[] = {{&S24191[98], 9}, {NULL}};
const Segment SGI138054[] = {{&S24191[71], 6}, {NULL}};
const Segment SGI138055[] = {{&S33170[44], 15}, {NULL}};
const Segment SGI138056[] = {{&S33170[44], 27}, {NULL}};
const Segment SGI138057[] = {{&S33170[30], 6}, {NULL}};
const Segment SGI138058[] = {{&S33170[31], 3}, {NULL}};
const Segment SGI138059[] = {{&S33170[85], 6}, {NULL}};
const Segment SGI138060[] = {{&S33172[44], 12}, {NULL}};
const Segment SGI138061[] = {{&S33172[44], 24}, {NULL}};
const Segment SGI138062[] = {{&S33172[30], 6}, {NULL}};
const Segment SGI138063[] = {{&S33172[31], 3}, {NULL}};
const Segment SGI138064[] = {{&S33172[82], 6}, {NULL}};
const Segment SGI138065[] = {{&S33173[38], 12}, {NULL}};
const Segment SGI138066[] = {{&S33173[31], 6}, {NULL}};
const Segment SGI138067[] = {{&S33175[44], 12}, {NULL}};
const Segment SGI138068[] = {{&S33175[44], 24}, {NULL}};
const Segment SGI138069[] = {{&S33175[30], 6}, {NULL}};
const Segment SGI138070[] = {{&S33175[31], 3}, {NULL}};
const Segment SGI138071[] = {{&S33175[82], 6}, {NULL}};
const Segment SGI138072[] = {{&S24265[34], 18}, {NULL}};
const Segment SGI138073[] = {{&S24265[27], 6}, {NULL}};
const Segment SGI138074[] = {{&S24265[105], 9}, {NULL}};
const Segment SGI138075[] = {{&S24265[78], 6}, {NULL}};
const Segment SGI138076[] = {{&S24266[34], 10}, {NULL}};
const Segment SGI138077[] = {{&S24266[27], 6}, {NULL}};
const Segment SGI138078[] = {{&S24266[97], 9}, {NULL}};
const Segment SGI138079[] = {{&S24266[70], 6}, {NULL}};
const Segment SGI138080[] = {{&S24267[34], 10}, {NULL}};
const Segment SGI138081[] = {{&S24267[27], 6}, {NULL}};
const Segment SGI138082[] = {{&S24267[97], 9}, {NULL}};
const Segment SGI138083[] = {{&S24267[70], 6}, {NULL}};
const Segment SGI138084[] = {{&S24268[34], 11}, {NULL}};
const Segment SGI138085[] = {{&S24268[27], 6}, {NULL}};
const Segment SGI138086[] = {{&S24268[98], 9}, {NULL}};
const Segment SGI138087[] = {{&S24268[71], 6}, {NULL}};
const Segment SGI138088[] = {{&S24269[34], 11}, {NULL}};
const Segment SGI138089[] = {{&S24269[27], 6}, {NULL}};
const Segment SGI138090[] = {{&S24269[98], 9}, {NULL}};
const Segment SGI138091[] = {{&S24269[71], 6}, {NULL}};
const Segment SGI138092[] = {{&S24270[34], 10}, {NULL}};
const Segment SGI138093[] = {{&S24270[27], 6}, {NULL}};
const Segment SGI138094[] = {{&S24270[97], 9}, {NULL}};
const Segment SGI138095[] = {{&S24270[70], 6}, {NULL}};
const Segment SGI138096[] = {{&S24271[34], 11}, {NULL}};
const Segment SGI138097[] = {{&S24271[27], 6}, {NULL}};
const Segment SGI138098[] = {{&S24271[98], 9}, {NULL}};
const Segment SGI138099[] = {{&S24271[71], 6}, {NULL}};
const Segment SGI138100[] = {{&S24276[34], 11}, {NULL}};
const Segment SGI138101[] = {{&S24276[27], 6}, {NULL}};
const Segment SGI138102[] = {{&S24276[98], 9}, {NULL}};
const Segment SGI138103[] = {{&S24276[71], 6}, {NULL}};
const Segment SGI138104[] = {{&S24277[34], 11}, {NULL}};
const Segment SGI138105[] = {{&S24277[27], 6}, {NULL}};
const Segment SGI138106[] = {{&S24277[98], 9}, {NULL}};
const Segment SGI138107[] = {{&S24277[71], 6}, {NULL}};
const Segment SGI138108[] = {{&S24278[34], 11}, {NULL}};
const Segment SGI138109[] = {{&S24278[27], 6}, {NULL}};
const Segment SGI138110[] = {{&S24278[98], 9}, {NULL}};
const Segment SGI138111[] = {{&S24278[71], 6}, {NULL}};
const Segment SGI138112[] = {{&S24279[34], 11}, {NULL}};
const Segment SGI138113[] = {{&S24279[27], 6}, {NULL}};
const Segment SGI138114[] = {{&S24279[98], 9}, {NULL}};
const Segment SGI138115[] = {{&S24279[71], 6}, {NULL}};
const Segment SGI138116[] = {{&S24294[34], 10}, {NULL}};
const Segment SGI138117[] = {{&S24294[27], 6}, {NULL}};
const Segment SGI138118[] = {{&S24294[97], 9}, {NULL}};
const Segment SGI138119[] = {{&S24294[70], 6}, {NULL}};
const Segment SGI138120[] = {{&S24300[34], 8}, {NULL}};
const Segment SGI138121[] = {{&S24300[27], 6}, {NULL}};
const Segment SGI138122[] = {{&S24300[95], 9}, {NULL}};
const Segment SGI138123[] = {{&S24300[68], 6}, {NULL}};
const Segment SGI138124[] = {{&S24301[34], 10}, {NULL}};
const Segment SGI138125[] = {{&S24301[27], 6}, {NULL}};
const Segment SGI138126[] = {{&S24301[97], 9}, {NULL}};
const Segment SGI138127[] = {{&S24301[70], 6}, {NULL}};
const Segment SGI138128[] = {{&S24303[34], 18}, {NULL}};
const Segment SGI138129[] = {{&S24303[27], 6}, {NULL}};
const Segment SGI138130[] = {{&S24303[105], 9}, {NULL}};
const Segment SGI138131[] = {{&S24303[78], 6}, {NULL}};
const Segment SGI138132[] = {{&S24308[34], 10}, {NULL}};
const Segment SGI138133[] = {{&S24308[27], 6}, {NULL}};
const Segment SGI138134[] = {{&S24308[97], 9}, {NULL}};
const Segment SGI138135[] = {{&S24308[70], 6}, {NULL}};
const Segment SGI138136[] = {{&S24309[34], 11}, {NULL}};
const Segment SGI138137[] = {{&S24309[27], 6}, {NULL}};
const Segment SGI138138[] = {{&S24309[98], 9}, {NULL}};
const Segment SGI138139[] = {{&S24309[71], 6}, {NULL}};
const Segment SGI138140[] = {{&S24310[34], 11}, {NULL}};
const Segment SGI138141[] = {{&S24310[27], 6}, {NULL}};
const Segment SGI138142[] = {{&S24310[98], 9}, {NULL}};
const Segment SGI138143[] = {{&S24310[71], 6}, {NULL}};
const Segment SGI138144[] = {{&S24311[34], 10}, {NULL}};
const Segment SGI138145[] = {{&S24311[27], 6}, {NULL}};
const Segment SGI138146[] = {{&S24311[97], 9}, {NULL}};
const Segment SGI138147[] = {{&S24311[70], 6}, {NULL}};
const Segment SGI138148[] = {{&S24312[34], 11}, {NULL}};
const Segment SGI138149[] = {{&S24312[27], 6}, {NULL}};
const Segment SGI138150[] = {{&S24312[98], 9}, {NULL}};
const Segment SGI138151[] = {{&S24312[71], 6}, {NULL}};
const Segment SGI138152[] = {{&S24313[34], 10}, {NULL}};
const Segment SGI138153[] = {{&S24313[27], 6}, {NULL}};
const Segment SGI138154[] = {{&S24313[97], 9}, {NULL}};
const Segment SGI138155[] = {{&S24313[70], 6}, {NULL}};
const Segment SGI138156[] = {{&S24314[34], 11}, {NULL}};
const Segment SGI138157[] = {{&S24314[27], 6}, {NULL}};
const Segment SGI138158[] = {{&S24314[98], 9}, {NULL}};
const Segment SGI138159[] = {{&S24314[71], 6}, {NULL}};
const Segment SGI138160[] = {{&S26491[42], 9}, {NULL}};
const Segment SGI138161[] = {{&S26491[42], 21}, {NULL}};
const Segment SGI138162[] = {{&S26491[30], 4}, {NULL}};
const Segment SGI138163[] = {{&S26491[112], 3}, {NULL}};
const Segment SGI138164[] = {{&S26491[77], 6}, {NULL}};
const Segment SGI138165[] = {{&S26494[42], 9}, {NULL}};
const Segment SGI138166[] = {{&S26494[42], 21}, {NULL}};
const Segment SGI138167[] = {{&S26494[30], 4}, {NULL}};
const Segment SGI138168[] = {{&S26494[112], 3}, {NULL}};
const Segment SGI138169[] = {{&S26494[77], 6}, {NULL}};
const Segment SGI138170[] = {{&S33326[38], 7}, {NULL}};
const Segment SGI138171[] = {{&S33326[31], 6}, {NULL}};
const Segment SGI138172[] = {{&S33328[44], 7}, {NULL}};
const Segment SGI138173[] = {{&S33328[44], 23}, {NULL}};
const Segment SGI138174[] = {{&S33328[30], 6}, {NULL}};
const Segment SGI138175[] = {{&S33328[31], 3}, {NULL}};
const Segment SGI138176[] = {{&S33328[81], 6}, {NULL}};
const Segment SGI138177[] = {{&S33329[44], 7}, {NULL}};
const Segment SGI138178[] = {{&S33329[44], 23}, {NULL}};
const Segment SGI138179[] = {{&S33329[30], 6}, {NULL}};
const Segment SGI138180[] = {{&S33329[31], 3}, {NULL}};
const Segment SGI138181[] = {{&S33329[81], 6}, {NULL}};
const Segment SGI138182[] = {{&S33330[44], 7}, {NULL}};
const Segment SGI138183[] = {{&S33330[44], 23}, {NULL}};
const Segment SGI138184[] = {{&S33330[30], 6}, {NULL}};
const Segment SGI138185[] = {{&S33330[84], 6}, {NULL}};
const Segment SGI138186[] = {{&S33331[44], 7}, {NULL}};
const Segment SGI138187[] = {{&S33331[44], 23}, {NULL}};
const Segment SGI138188[] = {{&S33331[30], 6}, {NULL}};
const Segment SGI138189[] = {{&S33331[31], 3}, {NULL}};
const Segment SGI138190[] = {{&S33331[81], 6}, {NULL}};
const Segment SGI138191[] = {{&S33332[44], 7}, {NULL}};
const Segment SGI138192[] = {{&S33332[44], 23}, {NULL}};
const Segment SGI138193[] = {{&S33332[30], 6}, {NULL}};
const Segment SGI138194[] = {{&S33332[31], 3}, {NULL}};
const Segment SGI138195[] = {{&S33332[81], 6}, {NULL}};
const Segment SGI138196[] = {{&S33336[37], 20}, {NULL}};
const Segment SGI138197[] = {{&S33336[30], 6}, {NULL}};
const Segment SGI138198[] = {{&S33336[31], 3}, {NULL}};
const Segment SGI138199[] = {{&S33336[103], 6}, {NULL}};
const Segment SGI138200[] = {{&S33337[37], 20}, {NULL}};
const Segment SGI138201[] = {{&S33337[30], 6}, {NULL}};
const Segment SGI138202[] = {{&S33337[31], 3}, {NULL}};
const Segment SGI138203[] = {{&S33337[103], 6}, {NULL}};
const Segment SGI138204[] = {{&S33339[37], 20}, {NULL}};
const Segment SGI138205[] = {{&S33339[30], 6}, {NULL}};
const Segment SGI138206[] = {{&S33339[31], 3}, {NULL}};
const Segment SGI138207[] = {{&S33339[103], 6}, {NULL}};
const Segment SGI138208[] = {{&S33340[44], 10}, {NULL}};
const Segment SGI138209[] = {{&S33340[44], 22}, {NULL}};
const Segment SGI138210[] = {{&S33340[30], 6}, {NULL}};
const Segment SGI138211[] = {{&S33340[31], 3}, {NULL}};
const Segment SGI138212[] = {{&S33340[80], 6}, {NULL}};
const Segment SGI138213[] = {{&S33342[37], 20}, {NULL}};
const Segment SGI138214[] = {{&S33342[30], 6}, {NULL}};
const Segment SGI138215[] = {{&S33342[31], 3}, {NULL}};
const Segment SGI138216[] = {{&S33342[103], 6}, {NULL}};
const Segment SGI138217[] = {{&S33343[37], 20}, {NULL}};
const Segment SGI138218[] = {{&S33343[30], 6}, {NULL}};
const Segment SGI138219[] = {{&S33343[31], 3}, {NULL}};
const Segment SGI138220[] = {{&S33343[103], 6}, {NULL}};
const Segment SGI138221[] = {{&S33344[37], 20}, {NULL}};
const Segment SGI138222[] = {{&S33344[30], 6}, {NULL}};
const Segment SGI138223[] = {{&S33344[31], 3}, {NULL}};
const Segment SGI138224[] = {{&S33344[103], 6}, {NULL}};
const Segment SGI138225[] = {{&S33345[44], 13}, {NULL}};
const Segment SGI138226[] = {{&S33345[44], 25}, {NULL}};
const Segment SGI138227[] = {{&S33345[30], 6}, {NULL}};
const Segment SGI138228[] = {{&S33345[31], 3}, {NULL}};
const Segment SGI138229[] = {{&S33345[83], 6}, {NULL}};
const Segment SGI138230[] = {{&S33346[37], 20}, {NULL}};
const Segment SGI138231[] = {{&S33346[30], 6}, {NULL}};
const Segment SGI138232[] = {{&S33346[31], 3}, {NULL}};
const Segment SGI138233[] = {{&S33346[103], 6}, {NULL}};
const Segment SGI138234[] = {{&S33347[37], 20}, {NULL}};
const Segment SGI138235[] = {{&S33347[30], 6}, {NULL}};
const Segment SGI138236[] = {{&S33347[31], 3}, {NULL}};
const Segment SGI138237[] = {{&S33347[103], 6}, {NULL}};
const Segment SGI138238[] = {{&S33348[37], 20}, {NULL}};
const Segment SGI138239[] = {{&S33348[30], 6}, {NULL}};
const Segment SGI138240[] = {{&S33348[31], 3}, {NULL}};
const Segment SGI138241[] = {{&S33348[103], 6}, {NULL}};
const Segment SGI138242[] = {{&S33349[37], 20}, {NULL}};
const Segment SGI138243[] = {{&S33349[30], 6}, {NULL}};
const Segment SGI138244[] = {{&S33349[31], 3}, {NULL}};
const Segment SGI138245[] = {{&S33349[103], 6}, {NULL}};
const Segment SGI138246[] = {{&S33350[37], 20}, {NULL}};
const Segment SGI138247[] = {{&S33350[30], 6}, {NULL}};
const Segment SGI138248[] = {{&S33350[31], 3}, {NULL}};
const Segment SGI138249[] = {{&S33350[103], 6}, {NULL}};
const Segment SGI138250[] = {{&S33351[37], 20}, {NULL}};
const Segment SGI138251[] = {{&S33351[30], 6}, {NULL}};
const Segment SGI138252[] = {{&S33351[31], 3}, {NULL}};
const Segment SGI138253[] = {{&S33351[103], 6}, {NULL}};
const Segment SGI138254[] = {{&S33352[37], 20}, {NULL}};
const Segment SGI138255[] = {{&S33352[30], 6}, {NULL}};
const Segment SGI138256[] = {{&S33352[31], 3}, {NULL}};
const Segment SGI138257[] = {{&S33352[103], 6}, {NULL}};
const Segment SGI138258[] = {{&S33353[37], 20}, {NULL}};
const Segment SGI138259[] = {{&S33353[30], 6}, {NULL}};
const Segment SGI138260[] = {{&S33353[31], 3}, {NULL}};
const Segment SGI138261[] = {{&S33353[103], 6}, {NULL}};
const Segment SGI138262[] = {{&S33354[38], 13}, {NULL}};
const Segment SGI138263[] = {{&S33354[31], 6}, {NULL}};
const Segment SGI138264[] = {{&S33355[38], 10}, {NULL}};
const Segment SGI138265[] = {{&S33355[31], 6}, {NULL}};
const Segment SGI138266[] = {{&S24619[34], 10}, {NULL}};
const Segment SGI138267[] = {{&S24619[27], 6}, {NULL}};
const Segment SGI138268[] = {{&S24619[97], 9}, {NULL}};
const Segment SGI138269[] = {{&S24619[70], 6}, {NULL}};
const Segment SGI138270[] = {{&S24623[34], 10}, {NULL}};
const Segment SGI138271[] = {{&S24623[27], 6}, {NULL}};
const Segment SGI138272[] = {{&S24623[97], 9}, {NULL}};
const Segment SGI138273[] = {{&S24623[70], 6}, {NULL}};
const Segment SGI138274[] = {{&S24625[34], 10}, {NULL}};
const Segment SGI138275[] = {{&S24625[27], 6}, {NULL}};
const Segment SGI138276[] = {{&S24625[97], 9}, {NULL}};
const Segment SGI138277[] = {{&S24625[70], 6}, {NULL}};
const Segment SGI138278[] = {{&S24627[34], 11}, {NULL}};
const Segment SGI138279[] = {{&S24627[27], 6}, {NULL}};
const Segment SGI138280[] = {{&S24627[98], 9}, {NULL}};
const Segment SGI138281[] = {{&S24627[71], 6}, {NULL}};
const Segment SGI138282[] = {{&S24628[34], 10}, {NULL}};
const Segment SGI138283[] = {{&S24628[27], 6}, {NULL}};
const Segment SGI138284[] = {{&S24628[97], 9}, {NULL}};
const Segment SGI138285[] = {{&S24628[70], 6}, {NULL}};
const Segment SGI138286[] = {{&S24629[34], 10}, {NULL}};
const Segment SGI138287[] = {{&S24629[27], 6}, {NULL}};
const Segment SGI138288[] = {{&S24629[97], 9}, {NULL}};
const Segment SGI138289[] = {{&S24629[70], 6}, {NULL}};
const Segment SGI138290[] = {{&S24630[34], 10}, {NULL}};
const Segment SGI138291[] = {{&S24630[27], 6}, {NULL}};
const Segment SGI138292[] = {{&S24630[97], 9}, {NULL}};
const Segment SGI138293[] = {{&S24630[70], 6}, {NULL}};
const Segment SGI138294[] = {{&S24632[34], 11}, {NULL}};
const Segment SGI138295[] = {{&S24632[27], 6}, {NULL}};
const Segment SGI138296[] = {{&S24632[98], 9}, {NULL}};
const Segment SGI138297[] = {{&S24632[71], 6}, {NULL}};
const Segment SGI138298[] = {{&S24634[34], 11}, {NULL}};
const Segment SGI138299[] = {{&S24634[27], 6}, {NULL}};
const Segment SGI138300[] = {{&S24634[98], 9}, {NULL}};
const Segment SGI138301[] = {{&S24634[71], 6}, {NULL}};
const Segment SGI138302[] = {{&S24636[34], 10}, {NULL}};
const Segment SGI138303[] = {{&S24636[27], 6}, {NULL}};
const Segment SGI138304[] = {{&S24636[97], 9}, {NULL}};
const Segment SGI138305[] = {{&S24636[70], 6}, {NULL}};
const Segment SGI138306[] = {{&S24639[34], 18}, {NULL}};
const Segment SGI138307[] = {{&S24639[27], 6}, {NULL}};
const Segment SGI138308[] = {{&S24639[105], 9}, {NULL}};
const Segment SGI138309[] = {{&S24639[78], 6}, {NULL}};
const Segment SGI138310[] = {{&S24641[34], 8}, {NULL}};
const Segment SGI138311[] = {{&S24641[27], 6}, {NULL}};
const Segment SGI138312[] = {{&S24641[95], 9}, {NULL}};
const Segment SGI138313[] = {{&S24641[68], 6}, {NULL}};
const Segment SGI138314[] = {{&S24646[34], 10}, {NULL}};
const Segment SGI138315[] = {{&S24646[27], 6}, {NULL}};
const Segment SGI138316[] = {{&S24646[97], 9}, {NULL}};
const Segment SGI138317[] = {{&S24646[70], 6}, {NULL}};
const Segment SGI138318[] = {{&S33621[44], 7}, {NULL}};
const Segment SGI138319[] = {{&S33621[44], 21}, {NULL}};
const Segment SGI138320[] = {{&S33621[30], 6}, {NULL}};
const Segment SGI138321[] = {{&S33621[31], 3}, {NULL}};
const Segment SGI138322[] = {{&S33621[79], 6}, {NULL}};
const Segment SGI138323[] = {{&S33623[44], 7}, {NULL}};
const Segment SGI138324[] = {{&S33623[44], 21}, {NULL}};
const Segment SGI138325[] = {{&S33623[30], 6}, {NULL}};
const Segment SGI138326[] = {{&S33623[31], 3}, {NULL}};
const Segment SGI138327[] = {{&S33623[79], 6}, {NULL}};
const Segment SGI138328[] = {{&S24651[34], 11}, {NULL}};
const Segment SGI138329[] = {{&S24651[27], 6}, {NULL}};
const Segment SGI138330[] = {{&S24651[98], 9}, {NULL}};
const Segment SGI138331[] = {{&S24651[71], 6}, {NULL}};
const Segment SGI138332[] = {{&S33624[44], 7}, {NULL}};
const Segment SGI138333[] = {{&S33624[44], 21}, {NULL}};
const Segment SGI138334[] = {{&S33624[30], 6}, {NULL}};
const Segment SGI138335[] = {{&S33624[31], 3}, {NULL}};
const Segment SGI138336[] = {{&S33624[79], 6}, {NULL}};
const Segment SGI138337[] = {{&S33625[38], 7}, {NULL}};
const Segment SGI138338[] = {{&S33625[31], 6}, {NULL}};
const Segment SGI138339[] = {{&S24655[34], 10}, {NULL}};
const Segment SGI138340[] = {{&S24655[27], 6}, {NULL}};
const Segment SGI138341[] = {{&S24655[97], 9}, {NULL}};
const Segment SGI138342[] = {{&S24655[70], 6}, {NULL}};
const Segment SGI138343[] = {{&S24656[34], 11}, {NULL}};
const Segment SGI138344[] = {{&S24656[27], 6}, {NULL}};
const Segment SGI138345[] = {{&S24656[98], 9}, {NULL}};
const Segment SGI138346[] = {{&S24656[71], 6}, {NULL}};
const Segment SGI138347[] = {{&S24657[34], 10}, {NULL}};
const Segment SGI138348[] = {{&S24657[27], 6}, {NULL}};
const Segment SGI138349[] = {{&S24657[97], 9}, {NULL}};
const Segment SGI138350[] = {{&S24657[70], 6}, {NULL}};
const Segment SGI138351[] = {{&S24658[34], 18}, {NULL}};
const Segment SGI138352[] = {{&S24658[27], 6}, {NULL}};
const Segment SGI138353[] = {{&S24658[105], 9}, {NULL}};
const Segment SGI138354[] = {{&S24658[78], 6}, {NULL}};
const Segment SGI138355[] = {{&S24660[34], 11}, {NULL}};
const Segment SGI138356[] = {{&S33636[44], 13}, {NULL}};
const Segment SGI138357[] = {{&S24660[27], 6}, {NULL}};
const Segment SGI138358[] = {{&S33636[44], 25}, {NULL}};
const Segment SGI138359[] = {{&S33636[30], 6}, {NULL}};
const Segment SGI138360[] = {{&S33636[31], 3}, {NULL}};
const Segment SGI138361[] = {{&S24660[98], 9}, {NULL}};
const Segment SGI138362[] = {{&S24660[71], 6}, {NULL}};
const Segment SGI138363[] = {{&S33636[83], 6}, {NULL}};
const Segment SGI138364[] = {{&S24661[34], 11}, {NULL}};
const Segment SGI138365[] = {{&S24661[27], 6}, {NULL}};
const Segment SGI138366[] = {{&S24661[98], 9}, {NULL}};
const Segment SGI138367[] = {{&S24661[71], 6}, {NULL}};
const Segment SGI138368[] = {{&S24663[34], 11}, {NULL}};
const Segment SGI138369[] = {{&S24663[27], 6}, {NULL}};
const Segment SGI138370[] = {{&S24663[98], 9}, {NULL}};
const Segment SGI138371[] = {{&S24663[71], 6}, {NULL}};
const Segment SGI138372[] = {{&S33829[44], 18}, {NULL}};
const Segment SGI138373[] = {{&S33829[44], 30}, {NULL}};
const Segment SGI138374[] = {{&S33829[30], 6}, {NULL}};
const Segment SGI138375[] = {{&S33829[31], 3}, {NULL}};
const Segment SGI138376[] = {{&S33829[88], 6}, {NULL}};
const Segment SGI138377[] = {{&S33830[38], 18}, {NULL}};
const Segment SGI138378[] = {{&S33830[31], 6}, {NULL}};
const Segment SGI138379[] = {{&S33831[42], 18}, {NULL}};
const Segment SGI138380[] = {{&S33831[30], 6}, {NULL}};
const Segment SGI138381[] = {{&S33831[31], 3}, {NULL}};
const Segment SGI138382[] = {{&S33831[86], 6}, {NULL}};
const Segment SGI138383[] = {{&S33832[44], 18}, {NULL}};
const Segment SGI138384[] = {{&S33832[44], 30}, {NULL}};
const Segment SGI138385[] = {{&S33832[30], 6}, {NULL}};
const Segment SGI138386[] = {{&S33832[31], 3}, {NULL}};
const Segment SGI138387[] = {{&S33832[88], 6}, {NULL}};
const Segment SGI138388[] = {{&S33834[44], 18}, {NULL}};
const Segment SGI138389[] = {{&S33834[44], 30}, {NULL}};
const Segment SGI138390[] = {{&S33834[30], 6}, {NULL}};
const Segment SGI138391[] = {{&S33834[88], 6}, {NULL}};
const Segment SGI138392[] = {{&S24706[34], 10}, {NULL}};
const Segment SGI138393[] = {{&S24706[27], 6}, {NULL}};
const Segment SGI138394[] = {{&S24706[97], 9}, {NULL}};
const Segment SGI138395[] = {{&S24706[70], 6}, {NULL}};
const Segment SGI138396[] = {{&S24707[34], 10}, {NULL}};
const Segment SGI138397[] = {{&S24707[27], 6}, {NULL}};
const Segment SGI138398[] = {{&S24707[97], 9}, {NULL}};
const Segment SGI138399[] = {{&S24707[70], 6}, {NULL}};
const Segment SGI138400[] = {{&S33963[44], 15}, {NULL}};
const Segment SGI138401[] = {{&S33963[44], 27}, {NULL}};
const Segment SGI138402[] = {{&S33963[30], 6}, {NULL}};
const Segment SGI138403[] = {{&S33963[31], 3}, {NULL}};
const Segment SGI138404[] = {{&S33963[85], 6}, {NULL}};
const Segment SGI138405[] = {{&S33966[44], 12}, {NULL}};
const Segment SGI138406[] = {{&S33966[44], 24}, {NULL}};
const Segment SGI138407[] = {{&S33966[30], 6}, {NULL}};
const Segment SGI138408[] = {{&S33966[31], 3}, {NULL}};
const Segment SGI138409[] = {{&S33966[82], 6}, {NULL}};
const Segment SGI138410[] = {{&S33971[38], 12}, {NULL}};
const Segment SGI138411[] = {{&S33971[31], 6}, {NULL}};
const Segment SGI138412[] = {{&S33972[44], 15}, {NULL}};
const Segment SGI138413[] = {{&S33972[44], 27}, {NULL}};
const Segment SGI138414[] = {{&S33972[30], 6}, {NULL}};
const Segment SGI138415[] = {{&S33972[31], 3}, {NULL}};
const Segment SGI138416[] = {{&S33972[85], 6}, {NULL}};
const Segment SGI138417[] = {{&S24710[34], 10}, {NULL}};
const Segment SGI138418[] = {{&S24710[27], 6}, {NULL}};
const Segment SGI138419[] = {{&S24710[97], 9}, {NULL}};
const Segment SGI138420[] = {{&S24710[70], 6}, {NULL}};
const Segment SGI138421[] = {{&S24711[34], 10}, {NULL}};
const Segment SGI138422[] = {{&S24711[27], 6}, {NULL}};
const Segment SGI138423[] = {{&S24711[97], 9}, {NULL}};
const Segment SGI138424[] = {{&S24711[70], 6}, {NULL}};
const Segment SGI138425[] = {{&S33976[44], 12}, {NULL}};
const Segment SGI138426[] = {{&S33976[44], 24}, {NULL}};
const Segment SGI138427[] = {{&S33976[30], 6}, {NULL}};
const Segment SGI138428[] = {{&S33976[31], 3}, {NULL}};
const Segment SGI138429[] = {{&S33976[82], 6}, {NULL}};
const Segment SGI138430[] = {{&S24712[34], 10}, {NULL}};
const Segment SGI138431[] = {{&S24712[27], 6}, {NULL}};
const Segment SGI138432[] = {{&S24712[97], 9}, {NULL}};
const Segment SGI138433[] = {{&S24712[70], 6}, {NULL}};
const Segment SGI138434[] = {{&S24713[34], 11}, {NULL}};
const Segment SGI138435[] = {{&S24713[27], 6}, {NULL}};
const Segment SGI138436[] = {{&S24713[98], 9}, {NULL}};
const Segment SGI138437[] = {{&S24713[71], 6}, {NULL}};
const Segment SGI138438[] = {{&S24716[34], 11}, {NULL}};
const Segment SGI138439[] = {{&S24716[27], 6}, {NULL}};
const Segment SGI138440[] = {{&S24716[98], 9}, {NULL}};
const Segment SGI138441[] = {{&S24716[71], 6}, {NULL}};
const Segment SGI138442[] = {{&S24720[34], 10}, {NULL}};
const Segment SGI138443[] = {{&S24720[27], 6}, {NULL}};
const Segment SGI138444[] = {{&S24720[97], 9}, {NULL}};
const Segment SGI138445[] = {{&S24720[70], 6}, {NULL}};
const Segment SGI138446[] = {{&S24721[34], 11}, {NULL}};
const Segment SGI138447[] = {{&S24721[27], 6}, {NULL}};
const Segment SGI138448[] = {{&S24721[98], 9}, {NULL}};
const Segment SGI138449[] = {{&S24721[71], 6}, {NULL}};
const Segment SGI138450[] = {{&S24722[34], 11}, {NULL}};
const Segment SGI138451[] = {{&S24722[27], 6}, {NULL}};
const Segment SGI138452[] = {{&S24722[98], 9}, {NULL}};
const Segment SGI138453[] = {{&S24722[71], 6}, {NULL}};
const Segment SGI138454[] = {{&S24723[34], 10}, {NULL}};
const Segment SGI138455[] = {{&S24723[27], 6}, {NULL}};
const Segment SGI138456[] = {{&S24723[97], 9}, {NULL}};
const Segment SGI138457[] = {{&S24723[70], 6}, {NULL}};
const Segment SGI138458[] = {{&S24724[34], 7}, {NULL}};
const Segment SGI138459[] = {{&S24724[27], 6}, {NULL}};
const Segment SGI138460[] = {{&S24724[94], 9}, {NULL}};
const Segment SGI138461[] = {{&S24724[67], 6}, {NULL}};
const Segment SGI138462[] = {{&S34026[44], 11}, {NULL}};
const Segment SGI138463[] = {{&S34026[44], 23}, {NULL}};
const Segment SGI138464[] = {{&S34026[30], 6}, {NULL}};
const Segment SGI138465[] = {{&S34026[31], 3}, {NULL}};
const Segment SGI138466[] = {{&S34026[81], 6}, {NULL}};
const Segment SGI138467[] = {{&S34027[44], 11}, {NULL}};
const Segment SGI138468[] = {{&S34027[44], 23}, {NULL}};
const Segment SGI138469[] = {{&S34027[30], 6}, {NULL}};
const Segment SGI138470[] = {{&S34027[31], 3}, {NULL}};
const Segment SGI138471[] = {{&S34027[81], 6}, {NULL}};
const Segment SGI138472[] = {{&S34028[38], 11}, {NULL}};
const Segment SGI138473[] = {{&S34028[31], 6}, {NULL}};
const Segment SGI138474[] = {{&S34029[44], 11}, {NULL}};
const Segment SGI138475[] = {{&S34029[44], 23}, {NULL}};
const Segment SGI138476[] = {{&S34029[30], 6}, {NULL}};
const Segment SGI138477[] = {{&S34029[31], 3}, {NULL}};
const Segment SGI138478[] = {{&S34029[81], 6}, {NULL}};
const Segment SGI138479[] = {{&S34030[38], 11}, {NULL}};
const Segment SGI138480[] = {{&S34030[31], 6}, {NULL}};
const Segment SGI138481[] = {{&S34031[38], 11}, {NULL}};
const Segment SGI138482[] = {{&S34031[31], 6}, {NULL}};
const Segment SGI138483[] = {{&S18664[44], 14}, {NULL}};
const Segment SGI138484[] = {{&S18664[44], 26}, {NULL}};
const Segment SGI138485[] = {{&S18664[30], 6}, {NULL}};
const Segment SGI138486[] = {{&S18664[119], 3}, {NULL}};
const Segment SGI138487[] = {{&S18664[84], 6}, {NULL}};
const Segment SGI138488[] = {{&S18666[38], 14}, {NULL}};
const Segment SGI138489[] = {{&S18666[31], 6}, {NULL}};
const Segment SGI138490[] = {{&S18667[44], 14}, {NULL}};
const Segment SGI138491[] = {{&S18667[44], 26}, {NULL}};
const Segment SGI138492[] = {{&S18667[30], 6}, {NULL}};
const Segment SGI138493[] = {{&S18667[119], 3}, {NULL}};
const Segment SGI138494[] = {{&S18667[84], 6}, {NULL}};
const Segment SGI138495[] = {{&S18669[44], 14}, {NULL}};
const Segment SGI138496[] = {{&S18669[44], 26}, {NULL}};
const Segment SGI138497[] = {{&S18669[30], 6}, {NULL}};
const Segment SGI138498[] = {{&S18669[119], 3}, {NULL}};
const Segment SGI138499[] = {{&S18669[84], 6}, {NULL}};
const Segment SGI138500[] = {{&S34048[34], 15}, {NULL}};
const Segment SGI138501[] = {{&S34048[27], 6}, {NULL}};
const Segment SGI138502[] = {{&S34048[102], 9}, {NULL}};
const Segment SGI138503[] = {{&S34048[75], 6}, {NULL}};
const Segment SGI138504[] = {{&S18670[44], 14}, {NULL}};
const Segment SGI138505[] = {{&S18670[44], 26}, {NULL}};
const Segment SGI138506[] = {{&S18670[30], 6}, {NULL}};
const Segment SGI138507[] = {{&S18670[87], 6}, {NULL}};
const Segment SGI138508[] = {{&S34049[34], 18}, {NULL}};
const Segment SGI138509[] = {{&S34049[27], 6}, {NULL}};
const Segment SGI138510[] = {{&S34049[105], 9}, {NULL}};
const Segment SGI138511[] = {{&S34049[78], 6}, {NULL}};
const Segment SGI138512[] = {{&S18673[44], 14}, {NULL}};
const Segment SGI138513[] = {{&S18673[44], 26}, {NULL}};
const Segment SGI138514[] = {{&S18673[30], 6}, {NULL}};
const Segment SGI138515[] = {{&S18673[119], 3}, {NULL}};
const Segment SGI138516[] = {{&S18673[84], 6}, {NULL}};
const Segment SGI138517[] = {{&S18674[44], 14}, {NULL}};
const Segment SGI138518[] = {{&S18674[44], 26}, {NULL}};
const Segment SGI138519[] = {{&S18674[30], 6}, {NULL}};
const Segment SGI138520[] = {{&S18674[119], 3}, {NULL}};
const Segment SGI138521[] = {{&S18674[84], 6}, {NULL}};
const Segment SGI138522[] = {{&S34055[34], 5}, {NULL}};
const Segment SGI138523[] = {{&S34055[27], 6}, {NULL}};
const Segment SGI138524[] = {{&S34055[92], 9}, {NULL}};
const Segment SGI138525[] = {{&S34055[65], 6}, {NULL}};
const Segment SGI138526[] = {{&S18679[44], 14}, {NULL}};
const Segment SGI138527[] = {{&S18679[44], 26}, {NULL}};
const Segment SGI138528[] = {{&S18679[30], 6}, {NULL}};
const Segment SGI138529[] = {{&S18679[119], 3}, {NULL}};
const Segment SGI138530[] = {{&S18679[84], 6}, {NULL}};
const Segment SGI138531[] = {{&S34058[34], 7}, {NULL}};
const Segment SGI138532[] = {{&S34058[27], 6}, {NULL}};
const Segment SGI138533[] = {{&S34058[94], 9}, {NULL}};
const Segment SGI138534[] = {{&S34058[67], 6}, {NULL}};
const Segment SGI138535[] = {{&S18680[42], 14}, {NULL}};
const Segment SGI138536[] = {{&S18680[30], 6}, {NULL}};
const Segment SGI138537[] = {{&S18680[117], 3}, {NULL}};
const Segment SGI138538[] = {{&S18680[82], 6}, {NULL}};
const Segment SGI138539[] = {{&S34061[34], 6}, {NULL}};
const Segment SGI138540[] = {{&S34061[27], 6}, {NULL}};
const Segment SGI138541[] = {{&S18683[44], 14}, {NULL}};
const Segment SGI138542[] = {{&S18683[44], 28}, {NULL}};
const Segment SGI138543[] = {{&S34061[93], 9}, {NULL}};
const Segment SGI138544[] = {{&S18683[30], 6}, {NULL}};
const Segment SGI138545[] = {{&S34061[66], 6}, {NULL}};
const Segment SGI138546[] = {{&S18683[121], 3}, {NULL}};
const Segment SGI138547[] = {{&S18683[86], 6}, {NULL}};
const Segment SGI138548[] = {{&S34062[34], 7}, {NULL}};
const Segment SGI138549[] = {{&S34062[27], 6}, {NULL}};
const Segment SGI138550[] = {{&S34062[94], 9}, {NULL}};
const Segment SGI138551[] = {{&S34062[67], 6}, {NULL}};
const Segment SGI138552[] = {{&S34063[34], 8}, {NULL}};
const Segment SGI138553[] = {{&S34063[27], 6}, {NULL}};
const Segment SGI138554[] = {{&S34063[95], 9}, {NULL}};
const Segment SGI138555[] = {{&S34063[68], 6}, {NULL}};
const Segment SGI138556[] = {{&S34069[34], 6}, {NULL}};
const Segment SGI138557[] = {{&S34069[27], 6}, {NULL}};
const Segment SGI138558[] = {{&S34069[93], 9}, {NULL}};
const Segment SGI138559[] = {{&S34069[66], 6}, {NULL}};
const Segment SGI138560[] = {{&S34070[34], 8}, {NULL}};
const Segment SGI138561[] = {{&S34070[27], 6}, {NULL}};
const Segment SGI138562[] = {{&S34070[95], 9}, {NULL}};
const Segment SGI138563[] = {{&S34070[68], 6}, {NULL}};
const Segment SGI138564[] = {{&S34071[34], 6}, {NULL}};
const Segment SGI138565[] = {{&S34071[27], 6}, {NULL}};
const Segment SGI138566[] = {{&S34071[93], 9}, {NULL}};
const Segment SGI138567[] = {{&S34071[66], 6}, {NULL}};
const Segment SGI138568[] = {{&S34073[34], 15}, {NULL}};
const Segment SGI138569[] = {{&S34073[27], 6}, {NULL}};
const Segment SGI138570[] = {{&S34073[102], 9}, {NULL}};
const Segment SGI138571[] = {{&S34073[75], 6}, {NULL}};
const Segment SGI138572[] = {{&S34075[34], 8}, {NULL}};
const Segment SGI138573[] = {{&S34075[27], 6}, {NULL}};
const Segment SGI138574[] = {{&S34075[95], 9}, {NULL}};
const Segment SGI138575[] = {{&S34075[68], 6}, {NULL}};
const Segment SGI138576[] = {{&S34076[34], 7}, {NULL}};
const Segment SGI138577[] = {{&S34076[27], 6}, {NULL}};
const Segment SGI138578[] = {{&S34076[94], 9}, {NULL}};
const Segment SGI138579[] = {{&S34076[67], 6}, {NULL}};
const Segment SGI138580[] = {{&S34077[34], 4}, {NULL}};
const Segment SGI138581[] = {{&S34077[27], 6}, {NULL}};
const Segment SGI138582[] = {{&S34077[91], 9}, {NULL}};
const Segment SGI138583[] = {{&S34077[64], 6}, {NULL}};
const Segment SGI138584[] = {{&S34078[34], 6}, {NULL}};
const Segment SGI138585[] = {{&S34078[27], 6}, {NULL}};
const Segment SGI138586[] = {{&S34078[93], 9}, {NULL}};
const Segment SGI138587[] = {{&S34078[66], 6}, {NULL}};
const Segment SGI138588[] = {{&S34080[34], 8}, {NULL}};
const Segment SGI138589[] = {{&S34080[27], 6}, {NULL}};
const Segment SGI138590[] = {{&S34080[95], 9}, {NULL}};
const Segment SGI138591[] = {{&S34080[68], 6}, {NULL}};
const Segment SGI138592[] = {{&S34082[34], 6}, {NULL}};
const Segment SGI138593[] = {{&S34082[27], 6}, {NULL}};
const Segment SGI138594[] = {{&S34082[93], 9}, {NULL}};
const Segment SGI138595[] = {{&S34082[66], 6}, {NULL}};
const Segment SGI138596[] = {{&S34085[34], 5}, {NULL}};
const Segment SGI138597[] = {{&S34085[27], 6}, {NULL}};
const Segment SGI138598[] = {{&S34085[92], 9}, {NULL}};
const Segment SGI138599[] = {{&S34085[65], 6}, {NULL}};
const Segment SGI138600[] = {{&S34086[34], 6}, {NULL}};
const Segment SGI138601[] = {{&S34086[27], 6}, {NULL}};
const Segment SGI138602[] = {{&S34086[93], 9}, {NULL}};
const Segment SGI138603[] = {{&S34086[66], 6}, {NULL}};
const Segment SGI138604[] = {{&S34087[34], 7}, {NULL}};
const Segment SGI138605[] = {{&S34087[27], 6}, {NULL}};
const Segment SGI138606[] = {{&S34087[94], 9}, {NULL}};
const Segment SGI138607[] = {{&S34087[67], 6}, {NULL}};
const Segment SGI138608[] = {{&S34088[34], 9}, {NULL}};
const Segment SGI138609[] = {{&S34088[27], 6}, {NULL}};
const Segment SGI138610[] = {{&S34088[96], 9}, {NULL}};
const Segment SGI138611[] = {{&S34088[69], 6}, {NULL}};
const Segment SGI138612[] = {{&S34090[34], 6}, {NULL}};
const Segment SGI138613[] = {{&S34090[27], 6}, {NULL}};
const Segment SGI138614[] = {{&S34090[93], 9}, {NULL}};
const Segment SGI138615[] = {{&S34090[66], 6}, {NULL}};
const Segment SGI138616[] = {{&S34094[34], 8}, {NULL}};
const Segment SGI138617[] = {{&S34094[27], 6}, {NULL}};
const Segment SGI138618[] = {{&S34094[95], 9}, {NULL}};
const Segment SGI138619[] = {{&S34094[68], 6}, {NULL}};
const Segment SGI138620[] = {{&S34095[34], 6}, {NULL}};
const Segment SGI138621[] = {{&S34095[27], 6}, {NULL}};
const Segment SGI138622[] = {{&S34095[93], 9}, {NULL}};
const Segment SGI138623[] = {{&S34095[66], 6}, {NULL}};
const Segment SGI138624[] = {{&S34096[34], 12}, {NULL}};
const Segment SGI138625[] = {{&S34096[27], 6}, {NULL}};
const Segment SGI138626[] = {{&S34096[99], 9}, {NULL}};
const Segment SGI138627[] = {{&S34096[72], 6}, {NULL}};
const Segment SGI138628[] = {{&S34097[34], 4}, {NULL}};
const Segment SGI138629[] = {{&S34097[27], 6}, {NULL}};
const Segment SGI138630[] = {{&S34097[91], 9}, {NULL}};
const Segment SGI138631[] = {{&S34097[64], 6}, {NULL}};
const Segment SGI138632[] = {{&S34098[34], 8}, {NULL}};
const Segment SGI138633[] = {{&S34098[27], 6}, {NULL}};
const Segment SGI138634[] = {{&S34098[95], 9}, {NULL}};
const Segment SGI138635[] = {{&S34098[68], 6}, {NULL}};
const Segment SGI138636[] = {{&S34099[34], 9}, {NULL}};
const Segment SGI138637[] = {{&S34099[27], 6}, {NULL}};
const Segment SGI138638[] = {{&S34099[96], 9}, {NULL}};
const Segment SGI138639[] = {{&S34099[69], 6}, {NULL}};
const Segment SGI138640[] = {{&S34100[34], 8}, {NULL}};
const Segment SGI138641[] = {{&S34100[27], 6}, {NULL}};
const Segment SGI138642[] = {{&S34100[95], 9}, {NULL}};
const Segment SGI138643[] = {{&S34100[68], 6}, {NULL}};
const Segment SGI138644[] = {{&S34101[34], 14}, {NULL}};
const Segment SGI138645[] = {{&S34101[27], 6}, {NULL}};
const Segment SGI138646[] = {{&S34101[101], 9}, {NULL}};
const Segment SGI138647[] = {{&S34101[74], 6}, {NULL}};
const Segment SGI138648[] = {{&S34103[34], 7}, {NULL}};
const Segment SGI138649[] = {{&S34103[27], 6}, {NULL}};
const Segment SGI138650[] = {{&S34103[94], 9}, {NULL}};
const Segment SGI138651[] = {{&S34103[67], 6}, {NULL}};
const Segment SGI138652[] = {{&S34104[34], 10}, {NULL}};
const Segment SGI138653[] = {{&S34104[27], 6}, {NULL}};
const Segment SGI138654[] = {{&S34104[97], 9}, {NULL}};
const Segment SGI138655[] = {{&S34104[70], 6}, {NULL}};
const Segment SGI138656[] = {{&S34105[34], 10}, {NULL}};
const Segment SGI138657[] = {{&S34105[27], 6}, {NULL}};
const Segment SGI138658[] = {{&S34105[97], 9}, {NULL}};
const Segment SGI138659[] = {{&S34105[70], 6}, {NULL}};
const Segment SGI138660[] = {{&S34106[34], 12}, {NULL}};
const Segment SGI138661[] = {{&S34106[27], 6}, {NULL}};
const Segment SGI138662[] = {{&S34106[99], 9}, {NULL}};
const Segment SGI138663[] = {{&S34106[72], 6}, {NULL}};
const Segment SGI138664[] = {{&S34107[34], 11}, {NULL}};
const Segment SGI138665[] = {{&S34107[27], 6}, {NULL}};
const Segment SGI138666[] = {{&S34107[98], 9}, {NULL}};
const Segment SGI138667[] = {{&S34107[71], 6}, {NULL}};
const Segment SGI138668[] = {{&S34108[34], 8}, {NULL}};
const Segment SGI138669[] = {{&S34108[27], 6}, {NULL}};
const Segment SGI138670[] = {{&S34108[95], 9}, {NULL}};
const Segment SGI138671[] = {{&S34108[68], 6}, {NULL}};
const Segment SGI138672[] = {{&S34111[34], 6}, {NULL}};
const Segment SGI138673[] = {{&S34111[27], 6}, {NULL}};
const Segment SGI138674[] = {{&S34111[93], 9}, {NULL}};
const Segment SGI138675[] = {{&S34111[66], 6}, {NULL}};
const Segment SGI138676[] = {{&S34113[34], 14}, {NULL}};
const Segment SGI138677[] = {{&S34113[27], 6}, {NULL}};
const Segment SGI138678[] = {{&S34113[101], 9}, {NULL}};
const Segment SGI138679[] = {{&S34113[74], 6}, {NULL}};
const Segment SGI138680[] = {{&S34114[34], 22}, {NULL}};
const Segment SGI138681[] = {{&S34114[27], 6}, {NULL}};
const Segment SGI138682[] = {{&S34114[109], 9}, {NULL}};
const Segment SGI138683[] = {{&S34114[82], 6}, {NULL}};
const Segment SGI138684[] = {{&S34115[34], 9}, {NULL}};
const Segment SGI138685[] = {{&S34115[27], 6}, {NULL}};
const Segment SGI138686[] = {{&S34115[96], 9}, {NULL}};
const Segment SGI138687[] = {{&S34115[69], 6}, {NULL}};
const Segment SGI138688[] = {{&S34116[34], 21}, {NULL}};
const Segment SGI138689[] = {{&S34116[27], 6}, {NULL}};
const Segment SGI138690[] = {{&S34116[108], 9}, {NULL}};
const Segment SGI138691[] = {{&S34116[81], 6}, {NULL}};
const Segment SGI138692[] = {{&S34117[34], 11}, {NULL}};
const Segment SGI138693[] = {{&S34117[27], 6}, {NULL}};
const Segment SGI138694[] = {{&S34117[98], 9}, {NULL}};
const Segment SGI138695[] = {{&S34117[71], 6}, {NULL}};
const Segment SGI138696[] = {{&S34118[34], 5}, {NULL}};
const Segment SGI138697[] = {{&S34118[27], 6}, {NULL}};
const Segment SGI138698[] = {{&S34118[92], 9}, {NULL}};
const Segment SGI138699[] = {{&S34118[65], 6}, {NULL}};
const Segment SGI138700[] = {{&S34120[34], 8}, {NULL}};
const Segment SGI138701[] = {{&S34120[27], 6}, {NULL}};
const Segment SGI138702[] = {{&S34120[95], 9}, {NULL}};
const Segment SGI138703[] = {{&S34120[68], 6}, {NULL}};
const Segment SGI138704[] = {{&S34121[34], 18}, {NULL}};
const Segment SGI138705[] = {{&S34121[27], 6}, {NULL}};
const Segment SGI138706[] = {{&S34121[105], 9}, {NULL}};
const Segment SGI138707[] = {{&S34121[78], 6}, {NULL}};
const Segment SGI138708[] = {{&S34122[34], 9}, {NULL}};
const Segment SGI138709[] = {{&S34122[27], 6}, {NULL}};
const Segment SGI138710[] = {{&S34122[97], 9}, {NULL}};
const Segment SGI138711[] = {{&S34122[70], 6}, {NULL}};
const Segment SGI138712[] = {{&S34123[34], 12}, {NULL}};
const Segment SGI138713[] = {{&S34123[27], 6}, {NULL}};
const Segment SGI138714[] = {{&S34123[99], 9}, {NULL}};
const Segment SGI138715[] = {{&S34123[72], 6}, {NULL}};
const Segment SGI138716[] = {{&S34124[34], 9}, {NULL}};
const Segment SGI138717[] = {{&S34124[27], 6}, {NULL}};
const Segment SGI138718[] = {{&S34124[96], 9}, {NULL}};
const Segment SGI138719[] = {{&S34124[69], 6}, {NULL}};
const Segment SGI138720[] = {{&S34125[34], 7}, {NULL}};
const Segment SGI138721[] = {{&S34125[27], 6}, {NULL}};
const Segment SGI138722[] = {{&S34125[94], 9}, {NULL}};
const Segment SGI138723[] = {{&S34125[67], 6}, {NULL}};
const Segment SGI138724[] = {{&S34126[34], 6}, {NULL}};
const Segment SGI138725[] = {{&S34126[27], 6}, {NULL}};
const Segment SGI138726[] = {{&S34126[93], 9}, {NULL}};
const Segment SGI138727[] = {{&S34126[66], 6}, {NULL}};
const Segment SGI138728[] = {{&S34127[34], 4}, {NULL}};
const Segment SGI138729[] = {{&S34127[27], 6}, {NULL}};
const Segment SGI138730[] = {{&S34127[91], 9}, {NULL}};
const Segment SGI138731[] = {{&S34127[64], 6}, {NULL}};
const Segment SGI138732[] = {{&S34128[34], 11}, {NULL}};
const Segment SGI138733[] = {{&S34128[27], 6}, {NULL}};
const Segment SGI138734[] = {{&S34128[98], 9}, {NULL}};
const Segment SGI138735[] = {{&S34128[71], 6}, {NULL}};
const Segment SGI138736[] = {{&S34129[34], 13}, {NULL}};
const Segment SGI138737[] = {{&S34129[27], 6}, {NULL}};
const Segment SGI138738[] = {{&S34129[100], 9}, {NULL}};
const Segment SGI138739[] = {{&S34129[73], 6}, {NULL}};
const Segment SGI138740[] = {{&S34130[34], 7}, {NULL}};
const Segment SGI138741[] = {{&S34130[27], 6}, {NULL}};
const Segment SGI138742[] = {{&S34130[94], 9}, {NULL}};
const Segment SGI138743[] = {{&S34130[67], 6}, {NULL}};
const Segment SGI138744[] = {{&S34131[34], 9}, {NULL}};
const Segment SGI138745[] = {{&S34131[27], 6}, {NULL}};
const Segment SGI138746[] = {{&S34131[96], 9}, {NULL}};
const Segment SGI138747[] = {{&S34131[69], 6}, {NULL}};
const Segment SGI138748[] = {{&S34132[34], 7}, {NULL}};
const Segment SGI138749[] = {{&S34132[27], 6}, {NULL}};
const Segment SGI138750[] = {{&S34132[94], 9}, {NULL}};
const Segment SGI138751[] = {{&S34132[67], 6}, {NULL}};
const Segment SGI138752[] = {{&S34133[34], 11}, {NULL}};
const Segment SGI138753[] = {{&S34133[27], 6}, {NULL}};
const Segment SGI138754[] = {{&S34133[98], 9}, {NULL}};
const Segment SGI138755[] = {{&S34133[71], 6}, {NULL}};
const Segment SGI138756[] = {{&S34134[34], 4}, {NULL}};
const Segment SGI138757[] = {{&S34134[27], 6}, {NULL}};
const Segment SGI138758[] = {{&S34134[91], 9}, {NULL}};
const Segment SGI138759[] = {{&S34134[64], 6}, {NULL}};
const Segment SGI138760[] = {{&S34135[34], 8}, {NULL}};
const Segment SGI138761[] = {{&S34135[27], 6}, {NULL}};
const Segment SGI138762[] = {{&S34135[95], 9}, {NULL}};
const Segment SGI138763[] = {{&S34135[68], 6}, {NULL}};
const Segment SGI138764[] = {{&S34137[34], 8}, {NULL}};
const Segment SGI138765[] = {{&S34137[27], 6}, {NULL}};
const Segment SGI138766[] = {{&S34137[95], 9}, {NULL}};
const Segment SGI138767[] = {{&S34137[68], 6}, {NULL}};
const Segment SGI138768[] = {{&S34139[34], 6}, {NULL}};
const Segment SGI138769[] = {{&S34139[27], 6}, {NULL}};
const Segment SGI138770[] = {{&S34139[93], 9}, {NULL}};
const Segment SGI138771[] = {{&S34139[66], 6}, {NULL}};
const Segment SGI138772[] = {{&S34141[34], 7}, {NULL}};
const Segment SGI138773[] = {{&S34141[27], 6}, {NULL}};
const Segment SGI138774[] = {{&S34141[94], 9}, {NULL}};
const Segment SGI138775[] = {{&S34141[67], 6}, {NULL}};
const Segment SGI138776[] = {{&S34142[34], 7}, {NULL}};
const Segment SGI138777[] = {{&S34142[27], 6}, {NULL}};
const Segment SGI138778[] = {{&S34142[94], 9}, {NULL}};
const Segment SGI138779[] = {{&S34142[67], 6}, {NULL}};
const Segment SGI138780[] = {{&S34144[34], 9}, {NULL}};
const Segment SGI138781[] = {{&S34144[27], 6}, {NULL}};
const Segment SGI138782[] = {{&S34144[96], 9}, {NULL}};
const Segment SGI138783[] = {{&S34144[69], 6}, {NULL}};
const Segment SGI138784[] = {{&S34145[34], 16}, {NULL}};
const Segment SGI138785[] = {{&S34145[27], 6}, {NULL}};
const Segment SGI138786[] = {{&S34145[103], 9}, {NULL}};
const Segment SGI138787[] = {{&S34145[76], 6}, {NULL}};
const Segment SGI138788[] = {{&S34146[34], 9}, {NULL}};
const Segment SGI138789[] = {{&S34146[27], 6}, {NULL}};
const Segment SGI138790[] = {{&S34146[96], 9}, {NULL}};
const Segment SGI138791[] = {{&S34146[69], 6}, {NULL}};
const Segment SGI138792[] = {{&S34150[34], 5}, {NULL}};
const Segment SGI138793[] = {{&S34150[27], 6}, {NULL}};
const Segment SGI138794[] = {{&S34150[92], 9}, {NULL}};
const Segment SGI138795[] = {{&S34150[65], 6}, {NULL}};
const Segment SGI138796[] = {{&S34151[34], 16}, {NULL}};
const Segment SGI138797[] = {{&S34151[27], 6}, {NULL}};
const Segment SGI138798[] = {{&S34151[103], 9}, {NULL}};
const Segment SGI138799[] = {{&S34151[76], 6}, {NULL}};
const Segment SGI138800[] = {{&S34152[34], 14}, {NULL}};
const Segment SGI138801[] = {{&S34152[27], 6}, {NULL}};
const Segment SGI138802[] = {{&S34152[101], 9}, {NULL}};
const Segment SGI138803[] = {{&S34152[74], 6}, {NULL}};
const Segment SGI138804[] = {{&S34153[34], 5}, {NULL}};
const Segment SGI138805[] = {{&S34153[27], 6}, {NULL}};
const Segment SGI138806[] = {{&S34153[92], 9}, {NULL}};
const Segment SGI138807[] = {{&S34153[65], 6}, {NULL}};
const Segment SGI138808[] = {{&S34154[34], 20}, {NULL}};
const Segment SGI138809[] = {{&S34154[27], 6}, {NULL}};
const Segment SGI138810[] = {{&S34154[107], 9}, {NULL}};
const Segment SGI138811[] = {{&S34154[80], 6}, {NULL}};
const Segment SGI138812[] = {{&S34156[34], 9}, {NULL}};
const Segment SGI138813[] = {{&S34156[27], 6}, {NULL}};
const Segment SGI138814[] = {{&S34156[96], 9}, {NULL}};
const Segment SGI138815[] = {{&S34156[69], 6}, {NULL}};
const Segment SGI138816[] = {{&S34157[34], 9}, {NULL}};
const Segment SGI138817[] = {{&S34157[27], 6}, {NULL}};
const Segment SGI138818[] = {{&S34157[96], 9}, {NULL}};
const Segment SGI138819[] = {{&S34157[69], 6}, {NULL}};
const Segment SGI138820[] = {{&S34158[34], 9}, {NULL}};
const Segment SGI138821[] = {{&S34158[27], 6}, {NULL}};
const Segment SGI138822[] = {{&S34158[96], 9}, {NULL}};
const Segment SGI138823[] = {{&S34158[69], 6}, {NULL}};
const Segment SGI138824[] = {{&S34159[34], 13}, {NULL}};
const Segment SGI138825[] = {{&S34159[27], 6}, {NULL}};
const Segment SGI138826[] = {{&S34159[100], 9}, {NULL}};
const Segment SGI138827[] = {{&S34159[73], 6}, {NULL}};
const Segment SGI138828[] = {{&S34163[34], 5}, {NULL}};
const Segment SGI138829[] = {{&S34163[27], 6}, {NULL}};
const Segment SGI138830[] = {{&S34163[92], 9}, {NULL}};
const Segment SGI138831[] = {{&S34163[65], 6}, {NULL}};
const Segment SGI138832[] = {{&S34164[34], 7}, {NULL}};
const Segment SGI138833[] = {{&S34164[27], 6}, {NULL}};
const Segment SGI138834[] = {{&S34164[93], 9}, {NULL}};
const Segment SGI138835[] = {{&S34164[66], 6}, {NULL}};
const Segment SGI138836[] = {{&S34165[34], 16}, {NULL}};
const Segment SGI138837[] = {{&S34165[27], 6}, {NULL}};
const Segment SGI138838[] = {{&S34165[103], 9}, {NULL}};
const Segment SGI138839[] = {{&S34165[76], 6}, {NULL}};
const Segment SGI138840[] = {{&S34166[34], 14}, {NULL}};
const Segment SGI138841[] = {{&S34166[27], 6}, {NULL}};
const Segment SGI138842[] = {{&S34166[101], 9}, {NULL}};
const Segment SGI138843[] = {{&S34166[74], 6}, {NULL}};
const Segment SGI138844[] = {{&S34167[34], 11}, {NULL}};
const Segment SGI138845[] = {{&S34167[27], 6}, {NULL}};
const Segment SGI138846[] = {{&S34167[98], 9}, {NULL}};
const Segment SGI138847[] = {{&S34167[71], 6}, {NULL}};
const Segment SGI138848[] = {{&S34169[34], 11}, {NULL}};
const Segment SGI138849[] = {{&S34169[27], 6}, {NULL}};
const Segment SGI138850[] = {{&S34169[98], 9}, {NULL}};
const Segment SGI138851[] = {{&S34169[71], 6}, {NULL}};
const Segment SGI138852[] = {{&S34170[34], 10}, {NULL}};
const Segment SGI138853[] = {{&S34170[27], 6}, {NULL}};
const Segment SGI138854[] = {{&S34170[97], 9}, {NULL}};
const Segment SGI138855[] = {{&S34170[70], 6}, {NULL}};
const Segment SGI138856[] = {{&S34171[34], 5}, {NULL}};
const Segment SGI138857[] = {{&S34171[27], 6}, {NULL}};
const Segment SGI138858[] = {{&S34171[92], 9}, {NULL}};
const Segment SGI138859[] = {{&S34171[65], 6}, {NULL}};
const Segment SGI138860[] = {{&S34172[34], 11}, {NULL}};
const Segment SGI138861[] = {{&S34172[27], 6}, {NULL}};
const Segment SGI138862[] = {{&S34172[98], 9}, {NULL}};
const Segment SGI138863[] = {{&S34172[71], 6}, {NULL}};
const Segment SGI138864[] = {{&S34173[34], 5}, {NULL}};
const Segment SGI138865[] = {{&S34173[27], 6}, {NULL}};
const Segment SGI138866[] = {{&S34173[92], 9}, {NULL}};
const Segment SGI138867[] = {{&S34173[65], 6}, {NULL}};
const Segment SGI138868[] = {{&S34174[34], 9}, {NULL}};
const Segment SGI138869[] = {{&S34174[27], 6}, {NULL}};
const Segment SGI138870[] = {{&S34174[96], 9}, {NULL}};
const Segment SGI138871[] = {{&S34174[69], 6}, {NULL}};
const Segment SGI138872[] = {{&S34175[34], 6}, {NULL}};
const Segment SGI138873[] = {{&S34175[27], 6}, {NULL}};
const Segment SGI138874[] = {{&S34175[93], 9}, {NULL}};
const Segment SGI138875[] = {{&S34175[66], 6}, {NULL}};
const Segment SGI138876[] = {{&S34176[34], 12}, {NULL}};
const Segment SGI138877[] = {{&S34176[27], 6}, {NULL}};
const Segment SGI138878[] = {{&S34176[111], 9}, {NULL}};
const Segment SGI138879[] = {{&S34176[84], 6}, {NULL}};
const Segment SGI138880[] = {{&S34177[34], 9}, {NULL}};
const Segment SGI138881[] = {{&S34177[27], 6}, {NULL}};
const Segment SGI138882[] = {{&S34177[102], 9}, {NULL}};
const Segment SGI138883[] = {{&S34177[75], 6}, {NULL}};
const Segment SGI138884[] = {{&S34178[34], 11}, {NULL}};
const Segment SGI138885[] = {{&S34178[27], 6}, {NULL}};
const Segment SGI138886[] = {{&S34178[98], 9}, {NULL}};
const Segment SGI138887[] = {{&S34178[71], 6}, {NULL}};
const Segment SGI138888[] = {{&S23908[44], 10}, {NULL}};
const Segment SGI138889[] = {{&S23908[44], 22}, {NULL}};
const Segment SGI138890[] = {{&S23908[30], 6}, {NULL}};
const Segment SGI138891[] = {{&S34179[34], 7}, {NULL}};
const Segment SGI138892[] = {{&S34179[27], 6}, {NULL}};
const Segment SGI138893[] = {{&S23908[115], 3}, {NULL}};
const Segment SGI138894[] = {{&S23908[80], 6}, {NULL}};
const Segment SGI138895[] = {{&S34179[140], 9}, {NULL}};
const Segment SGI138896[] = {{&S34179[113], 6}, {NULL}};
const Segment SGI138897[] = {{&S34180[34], 8}, {NULL}};
const Segment SGI138898[] = {{&S34180[27], 6}, {NULL}};
const Segment SGI138899[] = {{&S23910[38], 10}, {NULL}};
const Segment SGI138900[] = {{&S23910[31], 6}, {NULL}};
const Segment SGI138901[] = {{&S34180[95], 9}, {NULL}};
const Segment SGI138902[] = {{&S34180[68], 6}, {NULL}};
const Segment SGI138903[] = {{&S23911[44], 10}, {NULL}};
const Segment SGI138904[] = {{&S34181[34], 13}, {NULL}};
const Segment SGI138905[] = {{&S23911[44], 22}, {NULL}};
const Segment SGI138906[] = {{&S23911[30], 6}, {NULL}};
const Segment SGI138907[] = {{&S34181[27], 6}, {NULL}};
const Segment SGI138908[] = {{&S23911[115], 3}, {NULL}};
const Segment SGI138909[] = {{&S34181[111], 9}, {NULL}};
const Segment SGI138910[] = {{&S23911[80], 6}, {NULL}};
const Segment SGI138911[] = {{&S34181[84], 6}, {NULL}};
const Segment SGI138912[] = {{&S34182[34], 10}, {NULL}};
const Segment SGI138913[] = {{&S34182[27], 6}, {NULL}};
const Segment SGI138914[] = {{&S34182[97], 9}, {NULL}};
const Segment SGI138915[] = {{&S34182[70], 6}, {NULL}};
const Segment SGI138916[] = {{&S34183[34], 10}, {NULL}};
const Segment SGI138917[] = {{&S34183[27], 6}, {NULL}};
const Segment SGI138918[] = {{&S34183[97], 9}, {NULL}};
const Segment SGI138919[] = {{&S34183[70], 6}, {NULL}};
const Segment SGI138920[] = {{&S34184[34], 4}, {NULL}};
const Segment SGI138921[] = {{&S34184[27], 6}, {NULL}};
const Segment SGI138922[] = {{&S34184[91], 9}, {NULL}};
const Segment SGI138923[] = {{&S34184[64], 6}, {NULL}};
const Segment SGI138924[] = {{&S34185[34], 7}, {NULL}};
const Segment SGI138925[] = {{&S34185[27], 6}, {NULL}};
const Segment SGI138926[] = {{&S34185[94], 9}, {NULL}};
const Segment SGI138927[] = {{&S34185[67], 6}, {NULL}};
const Segment SGI138928[] = {{&S34186[34], 10}, {NULL}};
const Segment SGI138929[] = {{&S34186[27], 6}, {NULL}};
const Segment SGI138930[] = {{&S34186[109], 9}, {NULL}};
const Segment SGI138931[] = {{&S34186[82], 6}, {NULL}};
const Segment SGI138932[] = {{&S34187[34], 9}, {NULL}};
const Segment SGI138933[] = {{&S34187[27], 6}, {NULL}};
const Segment SGI138934[] = {{&S34187[96], 9}, {NULL}};
const Segment SGI138935[] = {{&S34187[69], 6}, {NULL}};
const Segment SGI138936[] = {{&S34188[34], 9}, {NULL}};
const Segment SGI138937[] = {{&S34188[27], 6}, {NULL}};
const Segment SGI138938[] = {{&S34188[96], 9}, {NULL}};
const Segment SGI138939[] = {{&S34188[69], 6}, {NULL}};
const Segment SGI138940[] = {{&S23926[44], 10}, {NULL}};
const Segment SGI138941[] = {{&S23926[44], 26}, {NULL}};
const Segment SGI138942[] = {{&S23926[30], 6}, {NULL}};
const Segment SGI138943[] = {{&S23926[119], 3}, {NULL}};
const Segment SGI138944[] = {{&S23926[84], 6}, {NULL}};
const Segment SGI138945[] = {{&S34189[34], 21}, {NULL}};
const Segment SGI138946[] = {{&S34189[27], 6}, {NULL}};
const Segment SGI138947[] = {{&S34189[108], 9}, {NULL}};
const Segment SGI138948[] = {{&S34189[81], 6}, {NULL}};
const Segment SGI138949[] = {{&S34191[34], 4}, {NULL}};
const Segment SGI138950[] = {{&S34191[27], 6}, {NULL}};
const Segment SGI138951[] = {{&S34191[91], 9}, {NULL}};
const Segment SGI138952[] = {{&S34191[64], 6}, {NULL}};
const Segment SGI138953[] = {{&S34192[34], 9}, {NULL}};
const Segment SGI138954[] = {{&S23931[44], 10}, {NULL}};
const Segment SGI138955[] = {{&S34192[27], 6}, {NULL}};
const Segment SGI138956[] = {{&S23931[44], 22}, {NULL}};
const Segment SGI138957[] = {{&S23931[30], 6}, {NULL}};
const Segment SGI138958[] = {{&S34192[96], 9}, {NULL}};
const Segment SGI138959[] = {{&S34192[69], 6}, {NULL}};
const Segment SGI138960[] = {{&S23931[80], 6}, {NULL}};
const Segment SGI138961[] = {{&S34193[34], 8}, {NULL}};
const Segment SGI138962[] = {{&S34193[27], 6}, {NULL}};
const Segment SGI138963[] = {{&S23932[42], 10}, {NULL}};
const Segment SGI138964[] = {{&S23932[30], 6}, {NULL}};
const Segment SGI138965[] = {{&S34193[95], 9}, {NULL}};
const Segment SGI138966[] = {{&S23932[113], 3}, {NULL}};
const Segment SGI138967[] = {{&S34193[68], 6}, {NULL}};
const Segment SGI138968[] = {{&S23932[78], 6}, {NULL}};
const Segment SGI138969[] = {{&S34196[34], 10}, {NULL}};
const Segment SGI138970[] = {{&S34196[27], 6}, {NULL}};
const Segment SGI138971[] = {{&S34196[97], 9}, {NULL}};
const Segment SGI138972[] = {{&S34196[70], 6}, {NULL}};
const Segment SGI138973[] = {{&S34197[34], 11}, {NULL}};
const Segment SGI138974[] = {{&S34197[27], 6}, {NULL}};
const Segment SGI138975[] = {{&S34197[98], 9}, {NULL}};
const Segment SGI138976[] = {{&S34197[71], 6}, {NULL}};
const Segment SGI138977[] = {{&S23938[43], 10}, {NULL}};
const Segment SGI138978[] = {{&S23938[43], 22}, {NULL}};
const Segment SGI138979[] = {{&S23938[114], 3}, {NULL}};
const Segment SGI138980[] = {{&S23938[79], 6}, {NULL}};
const Segment SGI138981[] = {{&S34200[34], 7}, {NULL}};
const Segment SGI138982[] = {{&S34200[27], 6}, {NULL}};
const Segment SGI138983[] = {{&S34200[94], 9}, {NULL}};
const Segment SGI138984[] = {{&S34200[67], 6}, {NULL}};
const Segment SGI138985[] = {{&S23940[44], 10}, {NULL}};
const Segment SGI138986[] = {{&S23940[44], 22}, {NULL}};
const Segment SGI138987[] = {{&S23940[30], 6}, {NULL}};
const Segment SGI138988[] = {{&S23940[83], 6}, {NULL}};
const Segment SGI138989[] = {{&S34202[34], 7}, {NULL}};
const Segment SGI138990[] = {{&S34202[27], 6}, {NULL}};
const Segment SGI138991[] = {{&S34202[94], 9}, {NULL}};
const Segment SGI138992[] = {{&S34202[67], 6}, {NULL}};
const Segment SGI138993[] = {{&S23943[44], 10}, {NULL}};
const Segment SGI138994[] = {{&S23943[44], 22}, {NULL}};
const Segment SGI138995[] = {{&S23943[30], 6}, {NULL}};
const Segment SGI138996[] = {{&S23943[127], 3}, {NULL}};
const Segment SGI138997[] = {{&S23943[80], 6}, {NULL}};
const Segment SGI138998[] = {{&S23944[65], 10}, {NULL}};
const Segment SGI138999[] = {{&S23944[65], 22}, {NULL}};
const Segment SGI139000[] = {{&S23944[51], 6}, {NULL}};
const Segment SGI139001[] = {{&S34205[34], 8}, {NULL}};
const Segment SGI139002[] = {{&S34205[27], 6}, {NULL}};
const Segment SGI139003[] = {{&S34205[95], 9}, {NULL}};
const Segment SGI139004[] = {{&S34205[68], 6}, {NULL}};
const Segment SGI139005[] = {{&S23945[44], 30}, {NULL}};
const Segment SGI139006[] = {{&S23945[44], 42}, {NULL}};
const Segment SGI139007[] = {{&S23945[30], 6}, {NULL}};
const Segment SGI139008[] = {{&S23945[135], 3}, {NULL}};
const Segment SGI139009[] = {{&S34206[34], 5}, {NULL}};
const Segment SGI139010[] = {{&S23945[100], 6}, {NULL}};
const Segment SGI139011[] = {{&S34206[27], 6}, {NULL}};
const Segment SGI139012[] = {{&S34206[92], 9}, {NULL}};
const Segment SGI139013[] = {{&S34206[65], 6}, {NULL}};
const Segment SGI139014[] = {{&S23946[44], 10}, {NULL}};
const Segment SGI139015[] = {{&S23946[44], 22}, {NULL}};
const Segment SGI139016[] = {{&S23946[30], 6}, {NULL}};
const Segment SGI139017[] = {{&S23946[115], 3}, {NULL}};
const Segment SGI139018[] = {{&S23946[80], 6}, {NULL}};
const Segment SGI139019[] = {{&S23946[135], 8}, {NULL}};
const Segment SGI139020[] = {{&S34208[34], 9}, {NULL}};
const Segment SGI139021[] = {{&S34208[27], 6}, {NULL}};
const Segment SGI139022[] = {{&S34208[96], 9}, {NULL}};
const Segment SGI139023[] = {{&S34208[69], 6}, {NULL}};
const Segment SGI139024[] = {{&S23948[44], 29}, {NULL}};
const Segment SGI139025[] = {{&S23948[44], 41}, {NULL}};
const Segment SGI139026[] = {{&S23948[30], 6}, {NULL}};
const Segment SGI139027[] = {{&S34209[34], 7}, {NULL}};
const Segment SGI139028[] = {{&S34209[27], 6}, {NULL}};
const Segment SGI139029[] = {{&S23948[134], 3}, {NULL}};
const Segment SGI139030[] = {{&S23948[99], 6}, {NULL}};
const Segment SGI139031[] = {{&S34209[94], 9}, {NULL}};
const Segment SGI139032[] = {{&S34209[67], 6}, {NULL}};
const Segment SGI139033[] = {{&S23949[44], 29}, {NULL}};
const Segment SGI139034[] = {{&S23949[44], 41}, {NULL}};
const Segment SGI139035[] = {{&S23949[30], 6}, {NULL}};
const Segment SGI139036[] = {{&S23949[134], 3}, {NULL}};
const Segment SGI139037[] = {{&S23949[99], 6}, {NULL}};
const Segment SGI139038[] = {{&S34211[34], 7}, {NULL}};
const Segment SGI139039[] = {{&S34211[27], 6}, {NULL}};
const Segment SGI139040[] = {{&S34211[94], 9}, {NULL}};
const Segment SGI139041[] = {{&S34211[67], 6}, {NULL}};
const Segment SGI139042[] = {{&S34212[34], 9}, {NULL}};
const Segment SGI139043[] = {{&S34212[27], 6}, {NULL}};
const Segment SGI139044[] = {{&S34212[97], 9}, {NULL}};
const Segment SGI139045[] = {{&S34212[70], 6}, {NULL}};
const Segment SGI139046[] = {{&S34215[34], 6}, {NULL}};
const Segment SGI139047[] = {{&S34215[27], 6}, {NULL}};
const Segment SGI139048[] = {{&S34215[93], 9}, {NULL}};
const Segment SGI139049[] = {{&S34215[66], 6}, {NULL}};
const Segment SGI139050[] = {{&S34216[34], 31}, {NULL}};
const Segment SGI139051[] = {{&S34216[27], 6}, {NULL}};
const Segment SGI139052[] = {{&S34216[118], 9}, {NULL}};
const Segment SGI139053[] = {{&S34216[91], 6}, {NULL}};
const Segment SGI139054[] = {{&S34217[34], 12}, {NULL}};
const Segment SGI139055[] = {{&S34217[27], 6}, {NULL}};
const Segment SGI139056[] = {{&S34217[99], 9}, {NULL}};
const Segment SGI139057[] = {{&S34217[72], 6}, {NULL}};
const Segment SGI139058[] = {{&S34218[34], 7}, {NULL}};
const Segment SGI139059[] = {{&S34218[27], 6}, {NULL}};
const Segment SGI139060[] = {{&S34218[94], 9}, {NULL}};
const Segment SGI139061[] = {{&S34218[67], 6}, {NULL}};
const Segment SGI139062[] = {{&S34219[27], 6}, {NULL}};
const Segment SGI139063[] = {{&S34219[117], 9}, {NULL}};
const Segment SGI139064[] = {{&S34219[90], 6}, {NULL}};
const Segment SGI139065[] = {{&S34220[34], 15}, {NULL}};
const Segment SGI139066[] = {{&S34220[27], 6}, {NULL}};
const Segment SGI139067[] = {{&S34220[102], 9}, {NULL}};
const Segment SGI139068[] = {{&S34220[75], 6}, {NULL}};
const Segment SGI139069[] = {{&S34221[34], 7}, {NULL}};
const Segment SGI139070[] = {{&S34221[27], 6}, {NULL}};
const Segment SGI139071[] = {{&S34221[94], 9}, {NULL}};
const Segment SGI139072[] = {{&S34221[67], 6}, {NULL}};
const Segment SGI139073[] = {{&S34224[34], 9}, {NULL}};
const Segment SGI139074[] = {{&S34224[27], 6}, {NULL}};
const Segment SGI139075[] = {{&S34224[96], 9}, {NULL}};
const Segment SGI139076[] = {{&S34224[69], 6}, {NULL}};
const Segment SGI139077[] = {{&S34225[34], 11}, {NULL}};
const Segment SGI139078[] = {{&S34225[27], 6}, {NULL}};
const Segment SGI139079[] = {{&S34225[98], 9}, {NULL}};
const Segment SGI139080[] = {{&S34225[71], 6}, {NULL}};
const Segment SGI139081[] = {{&S34226[34], 8}, {NULL}};
const Segment SGI139082[] = {{&S34226[27], 6}, {NULL}};
const Segment SGI139083[] = {{&S34226[95], 9}, {NULL}};
const Segment SGI139084[] = {{&S34226[68], 6}, {NULL}};
const Segment SGI139085[] = {{&S34227[34], 10}, {NULL}};
const Segment SGI139086[] = {{&S34227[27], 6}, {NULL}};
const Segment SGI139087[] = {{&S34227[97], 9}, {NULL}};
const Segment SGI139088[] = {{&S34227[70], 6}, {NULL}};
const Segment SGI139089[] = {{&S34228[34], 15}, {NULL}};
const Segment SGI139090[] = {{&S34228[27], 6}, {NULL}};
const Segment SGI139091[] = {{&S34228[102], 9}, {NULL}};
const Segment SGI139092[] = {{&S34228[75], 6}, {NULL}};
const Segment SGI139093[] = {{&S34229[34], 14}, {NULL}};
const Segment SGI139094[] = {{&S34229[27], 6}, {NULL}};
const Segment SGI139095[] = {{&S34229[101], 9}, {NULL}};
const Segment SGI139096[] = {{&S34229[74], 6}, {NULL}};
const Segment SGI139097[] = {{&S34230[34], 9}, {NULL}};
const Segment SGI139098[] = {{&S34230[27], 6}, {NULL}};
const Segment SGI139099[] = {{&S34230[96], 9}, {NULL}};
const Segment SGI139100[] = {{&S34230[69], 6}, {NULL}};
const Segment SGI139101[] = {{&S34231[34], 9}, {NULL}};
const Segment SGI139102[] = {{&S34231[27], 6}, {NULL}};
const Segment SGI139103[] = {{&S34231[96], 9}, {NULL}};
const Segment SGI139104[] = {{&S34231[69], 6}, {NULL}};
const Segment SGI139105[] = {{&S34232[34], 14}, {NULL}};
const Segment SGI139106[] = {{&S34232[27], 6}, {NULL}};
const Segment SGI139107[] = {{&S34232[101], 9}, {NULL}};
const Segment SGI139108[] = {{&S34232[74], 6}, {NULL}};
const Segment SGI139109[] = {{&S34233[34], 14}, {NULL}};
const Segment SGI139110[] = {{&S34233[27], 6}, {NULL}};
const Segment SGI139111[] = {{&S34233[101], 9}, {NULL}};
const Segment SGI139112[] = {{&S34233[74], 6}, {NULL}};
const Segment SGI139113[] = {{&S34234[34], 8}, {NULL}};
const Segment SGI139114[] = {{&S34234[27], 6}, {NULL}};
const Segment SGI139115[] = {{&S34234[95], 9}, {NULL}};
const Segment SGI139116[] = {{&S34234[68], 6}, {NULL}};
const Segment SGI139117[] = {{&S34235[34], 19}, {NULL}};
const Segment SGI139118[] = {{&S34235[27], 6}, {NULL}};
const Segment SGI139119[] = {{&S24970[44], 10}, {NULL}};
const Segment SGI139120[] = {{&S24970[44], 22}, {NULL}};
const Segment SGI139121[] = {{&S34235[106], 9}, {NULL}};
const Segment SGI139122[] = {{&S24970[30], 6}, {NULL}};
const Segment SGI139123[] = {{&S34235[79], 6}, {NULL}};
const Segment SGI139124[] = {{&S24970[115], 3}, {NULL}};
const Segment SGI139125[] = {{&S24970[80], 6}, {NULL}};
const Segment SGI139126[] = {{&S24971[38], 10}, {NULL}};
const Segment SGI139127[] = {{&S34237[34], 15}, {NULL}};
const Segment SGI139128[] = {{&S24971[31], 6}, {NULL}};
const Segment SGI139129[] = {{&S34237[27], 6}, {NULL}};
const Segment SGI139130[] = {{&S34237[102], 9}, {NULL}};
const Segment SGI139131[] = {{&S34237[75], 6}, {NULL}};
const Segment SGI139132[] = {{&S24972[44], 30}, {NULL}};
const Segment SGI139133[] = {{&S24972[44], 42}, {NULL}};
const Segment SGI139134[] = {{&S24972[30], 6}, {NULL}};
const Segment SGI139135[] = {{&S34238[34], 15}, {NULL}};
const Segment SGI139136[] = {{&S24972[135], 3}, {NULL}};
const Segment SGI139137[] = {{&S34238[27], 6}, {NULL}};
const Segment SGI139138[] = {{&S24972[100], 6}, {NULL}};
const Segment SGI139139[] = {{&S34238[102], 9}, {NULL}};
const Segment SGI139140[] = {{&S34238[75], 6}, {NULL}};
const Segment SGI139141[] = {{&S34239[34], 6}, {NULL}};
const Segment SGI139142[] = {{&S34239[27], 6}, {NULL}};
const Segment SGI139143[] = {{&S34239[93], 9}, {NULL}};
const Segment SGI139144[] = {{&S34239[66], 6}, {NULL}};
const Segment SGI139145[] = {{&S24975[44], 10}, {NULL}};
const Segment SGI139146[] = {{&S24975[44], 22}, {NULL}};
const Segment SGI139147[] = {{&S24975[30], 6}, {NULL}};
const Segment SGI139148[] = {{&S24975[115], 3}, {NULL}};
const Segment SGI139149[] = {{&S24975[80], 6}, {NULL}};
const Segment SGI139150[] = {{&S34241[34], 7}, {NULL}};
const Segment SGI139151[] = {{&S34241[27], 6}, {NULL}};
const Segment SGI139152[] = {{&S34241[94], 9}, {NULL}};
const Segment SGI139153[] = {{&S34241[67], 6}, {NULL}};
const Segment SGI139154[] = {{&S24976[44], 10}, {NULL}};
const Segment SGI139155[] = {{&S24976[44], 22}, {NULL}};
const Segment SGI139156[] = {{&S24976[30], 6}, {NULL}};
const Segment SGI139157[] = {{&S24976[115], 3}, {NULL}};
const Segment SGI139158[] = {{&S24976[80], 6}, {NULL}};
const Segment SGI139159[] = {{&S34243[34], 6}, {NULL}};
const Segment SGI139160[] = {{&S34243[27], 6}, {NULL}};
const Segment SGI139161[] = {{&S34243[93], 9}, {NULL}};
const Segment SGI139162[] = {{&S34243[66], 6}, {NULL}};
const Segment SGI139163[] = {{&S24978[44], 10}, {NULL}};
const Segment SGI139164[] = {{&S24978[44], 22}, {NULL}};
const Segment SGI139165[] = {{&S24978[30], 6}, {NULL}};
const Segment SGI139166[] = {{&S24978[115], 3}, {NULL}};
const Segment SGI139167[] = {{&S24978[80], 6}, {NULL}};
const Segment SGI139168[] = {{&S34244[34], 10}, {NULL}};
const Segment SGI139169[] = {{&S34244[27], 6}, {NULL}};
const Segment SGI139170[] = {{&S34244[97], 9}, {NULL}};
const Segment SGI139171[] = {{&S34244[70], 6}, {NULL}};
const Segment SGI139172[] = {{&S24979[44], 30}, {NULL}};
const Segment SGI139173[] = {{&S24979[44], 42}, {NULL}};
const Segment SGI139174[] = {{&S24979[30], 6}, {NULL}};
const Segment SGI139175[] = {{&S24979[135], 3}, {NULL}};
const Segment SGI139176[] = {{&S24979[100], 6}, {NULL}};
const Segment SGI139177[] = {{&S24980[44], 10}, {NULL}};
const Segment SGI139178[] = {{&S24980[44], 22}, {NULL}};
const Segment SGI139179[] = {{&S24980[30], 6}, {NULL}};
const Segment SGI139180[] = {{&S24980[115], 3}, {NULL}};
const Segment SGI139181[] = {{&S34247[34], 11}, {NULL}};
const Segment SGI139182[] = {{&S24980[80], 6}, {NULL}};
const Segment SGI139183[] = {{&S34247[27], 6}, {NULL}};
const Segment SGI139184[] = {{&S34247[101], 9}, {NULL}};
const Segment SGI139185[] = {{&S34247[74], 6}, {NULL}};
const Segment SGI139186[] = {{&S24981[42], 10}, {NULL}};
const Segment SGI139187[] = {{&S24981[30], 6}, {NULL}};
const Segment SGI139188[] = {{&S34248[34], 7}, {NULL}};
const Segment SGI139189[] = {{&S24981[113], 3}, {NULL}};
const Segment SGI139190[] = {{&S34248[27], 6}, {NULL}};
const Segment SGI139191[] = {{&S24981[78], 6}, {NULL}};
const Segment SGI139192[] = {{&S34248[94], 9}, {NULL}};
const Segment SGI139193[] = {{&S34248[67], 6}, {NULL}};
const Segment SGI139194[] = {{&S24982[59], 10}, {NULL}};
const Segment SGI139195[] = {{&S24982[59], 34}, {NULL}};
const Segment SGI139196[] = {{&S34249[34], 9}, {NULL}};
const Segment SGI139197[] = {{&S24982[45], 6}, {NULL}};
const Segment SGI139198[] = {{&S34249[27], 6}, {NULL}};
const Segment SGI139199[] = {{&S34249[98], 9}, {NULL}};
const Segment SGI139200[] = {{&S34249[71], 6}, {NULL}};
const Segment SGI139201[] = {{&S24983[65], 10}, {NULL}};
const Segment SGI139202[] = {{&S24983[65], 22}, {NULL}};
const Segment SGI139203[] = {{&S34250[34], 6}, {NULL}};
const Segment SGI139204[] = {{&S24983[51], 6}, {NULL}};
const Segment SGI139205[] = {{&S34250[27], 6}, {NULL}};
const Segment SGI139206[] = {{&S34250[93], 9}, {NULL}};
const Segment SGI139207[] = {{&S34250[66], 6}, {NULL}};
const Segment SGI139208[] = {{&S34251[34], 10}, {NULL}};
const Segment SGI139209[] = {{&S34251[27], 6}, {NULL}};
const Segment SGI139210[] = {{&S34251[97], 9}, {NULL}};
const Segment SGI139211[] = {{&S34251[70], 6}, {NULL}};
const Segment SGI139212[] = {{&S24985[44], 10}, {NULL}};
const Segment SGI139213[] = {{&S24985[44], 22}, {NULL}};
const Segment SGI139214[] = {{&S24985[30], 6}, {NULL}};
const Segment SGI139215[] = {{&S24985[115], 3}, {NULL}};
const Segment SGI139216[] = {{&S24985[80], 6}, {NULL}};
const Segment SGI139217[] = {{&S34252[34], 5}, {NULL}};
const Segment SGI139218[] = {{&S34252[27], 6}, {NULL}};
const Segment SGI139219[] = {{&S34252[92], 9}, {NULL}};
const Segment SGI139220[] = {{&S34252[65], 6}, {NULL}};
const Segment SGI139221[] = {{&S24986[44], 10}, {NULL}};
const Segment SGI139222[] = {{&S24986[44], 22}, {NULL}};
const Segment SGI139223[] = {{&S24986[30], 6}, {NULL}};
const Segment SGI139224[] = {{&S24986[116], 3}, {NULL}};
const Segment SGI139225[] = {{&S24986[80], 7}, {NULL}};
const Segment SGI139226[] = {{&S24986[136], 8}, {NULL}};
const Segment SGI139227[] = {{&S34253[34], 15}, {NULL}};
const Segment SGI139228[] = {{&S34253[27], 6}, {NULL}};
const Segment SGI139229[] = {{&S34253[102], 9}, {NULL}};
const Segment SGI139230[] = {{&S34253[75], 6}, {NULL}};
const Segment SGI139231[] = {{&S24987[44], 30}, {NULL}};
const Segment SGI139232[] = {{&S24987[44], 42}, {NULL}};
const Segment SGI139233[] = {{&S24987[30], 6}, {NULL}};
const Segment SGI139234[] = {{&S24987[135], 3}, {NULL}};
const Segment SGI139235[] = {{&S34254[34], 7}, {NULL}};
const Segment SGI139236[] = {{&S24987[100], 6}, {NULL}};
const Segment SGI139237[] = {{&S34254[27], 6}, {NULL}};
const Segment SGI139238[] = {{&S34254[94], 9}, {NULL}};
const Segment SGI139239[] = {{&S34254[67], 6}, {NULL}};
const Segment SGI139240[] = {{&S24988[44], 10}, {NULL}};
const Segment SGI139241[] = {{&S24988[44], 22}, {NULL}};
const Segment SGI139242[] = {{&S24988[30], 6}, {NULL}};
const Segment SGI139243[] = {{&S24988[83], 6}, {NULL}};
const Segment SGI139244[] = {{&S34255[34], 13}, {NULL}};
const Segment SGI139245[] = {{&S34255[27], 6}, {NULL}};
const Segment SGI139246[] = {{&S34255[100], 9}, {NULL}};
const Segment SGI139247[] = {{&S34255[73], 6}, {NULL}};
const Segment SGI139248[] = {{&S24989[44], 10}, {NULL}};
const Segment SGI139249[] = {{&S24989[44], 22}, {NULL}};
const Segment SGI139250[] = {{&S24989[30], 6}, {NULL}};
const Segment SGI139251[] = {{&S24989[127], 3}, {NULL}};
const Segment SGI139252[] = {{&S24989[80], 6}, {NULL}};
const Segment SGI139253[] = {{&S24990[44], 10}, {NULL}};
const Segment SGI139254[] = {{&S24990[44], 22}, {NULL}};
const Segment SGI139255[] = {{&S34258[34], 10}, {NULL}};
const Segment SGI139256[] = {{&S24990[30], 6}, {NULL}};
const Segment SGI139257[] = {{&S34258[27], 6}, {NULL}};
const Segment SGI139258[] = {{&S24990[80], 6}, {NULL}};
const Segment SGI139259[] = {{&S34258[97], 9}, {NULL}};
const Segment SGI139260[] = {{&S34258[70], 6}, {NULL}};
const Segment SGI139261[] = {{&S34259[34], 9}, {NULL}};
const Segment SGI139262[] = {{&S34259[27], 6}, {NULL}};
const Segment SGI139263[] = {{&S34259[96], 9}, {NULL}};
const Segment SGI139264[] = {{&S34259[69], 6}, {NULL}};
const Segment SGI139265[] = {{&S24993[44], 10}, {NULL}};
const Segment SGI139266[] = {{&S24993[44], 19}, {NULL}};
const Segment SGI139267[] = {{&S24993[30], 6}, {NULL}};
const Segment SGI139268[] = {{&S34260[34], 12}, {NULL}};
const Segment SGI139269[] = {{&S24993[111], 3}, {NULL}};
const Segment SGI139270[] = {{&S34260[27], 6}, {NULL}};
const Segment SGI139271[] = {{&S24993[77], 5}, {NULL}};
const Segment SGI139272[] = {{&S34260[99], 9}, {NULL}};
const Segment SGI139273[] = {{&S34260[72], 6}, {NULL}};
const Segment SGI139274[] = {{&S34261[34], 14}, {NULL}};
const Segment SGI139275[] = {{&S34261[27], 6}, {NULL}};
const Segment SGI139276[] = {{&S34261[101], 9}, {NULL}};
const Segment SGI139277[] = {{&S34261[74], 6}, {NULL}};
const Segment SGI139278[] = {{&S34262[34], 6}, {NULL}};
const Segment SGI139279[] = {{&S34262[27], 6}, {NULL}};
const Segment SGI139280[] = {{&S34262[93], 9}, {NULL}};
const Segment SGI139281[] = {{&S34262[66], 6}, {NULL}};
const Segment SGI139282[] = {{&S34263[34], 6}, {NULL}};
const Segment SGI139283[] = {{&S34263[27], 6}, {NULL}};
const Segment SGI139284[] = {{&S34263[93], 9}, {NULL}};
const Segment SGI139285[] = {{&S34263[66], 6}, {NULL}};
const Segment SGI139286[] = {{&S34264[34], 6}, {NULL}};
const Segment SGI139287[] = {{&S34264[27], 6}, {NULL}};
const Segment SGI139288[] = {{&S34264[93], 9}, {NULL}};
const Segment SGI139289[] = {{&S34264[66], 6}, {NULL}};
const Segment SGI139290[] = {{&S34265[34], 8}, {NULL}};
const Segment SGI139291[] = {{&S34265[27], 6}, {NULL}};
const Segment SGI139292[] = {{&S34265[95], 9}, {NULL}};
const Segment SGI139293[] = {{&S34265[68], 6}, {NULL}};
const Segment SGI139294[] = {{&S34266[34], 16}, {NULL}};
const Segment SGI139295[] = {{&S34266[27], 6}, {NULL}};
const Segment SGI139296[] = {{&S34266[113], 9}, {NULL}};
const Segment SGI139297[] = {{&S34266[86], 6}, {NULL}};
const Segment SGI139298[] = {{&S34267[34], 10}, {NULL}};
const Segment SGI139299[] = {{&S34267[27], 6}, {NULL}};
const Segment SGI139300[] = {{&S34267[97], 9}, {NULL}};
const Segment SGI139301[] = {{&S34267[70], 6}, {NULL}};
const Segment SGI139302[] = {{&S34269[34], 11}, {NULL}};
const Segment SGI139303[] = {{&S34269[27], 6}, {NULL}};
const Segment SGI139304[] = {{&S34269[98], 9}, {NULL}};
const Segment SGI139305[] = {{&S34269[71], 6}, {NULL}};
const Segment SGI139306[] = {{&S34270[34], 10}, {NULL}};
const Segment SGI139307[] = {{&S34270[27], 6}, {NULL}};
const Segment SGI139308[] = {{&S34270[97], 9}, {NULL}};
const Segment SGI139309[] = {{&S34270[70], 6}, {NULL}};
const Segment SGI139310[] = {{&S34271[34], 7}, {NULL}};
const Segment SGI139311[] = {{&S34271[27], 6}, {NULL}};
const Segment SGI139312[] = {{&S34271[94], 9}, {NULL}};
const Segment SGI139313[] = {{&S34271[67], 6}, {NULL}};
const Segment SGI139314[] = {{&S34272[34], 9}, {NULL}};
const Segment SGI139315[] = {{&S34272[27], 6}, {NULL}};
const Segment SGI139316[] = {{&S34272[96], 9}, {NULL}};
const Segment SGI139317[] = {{&S34272[69], 6}, {NULL}};
const Segment SGI139318[] = {{&S34273[34], 6}, {NULL}};
const Segment SGI139319[] = {{&S34273[27], 6}, {NULL}};
const Segment SGI139320[] = {{&S34273[93], 9}, {NULL}};
const Segment SGI139321[] = {{&S34273[66], 6}, {NULL}};
const Segment SGI139322[] = {{&S34275[34], 14}, {NULL}};
const Segment SGI139323[] = {{&S34275[27], 6}, {NULL}};
const Segment SGI139324[] = {{&S34275[101], 9}, {NULL}};
const Segment SGI139325[] = {{&S34275[74], 6}, {NULL}};
const Segment SGI139326[] = {{&S34276[34], 7}, {NULL}};
const Segment SGI139327[] = {{&S34276[27], 6}, {NULL}};
const Segment SGI139328[] = {{&S34276[94], 9}, {NULL}};
const Segment SGI139329[] = {{&S34276[67], 6}, {NULL}};
const Segment SGI139330[] = {{&S34277[34], 8}, {NULL}};
const Segment SGI139331[] = {{&S34277[27], 6}, {NULL}};
const Segment SGI139332[] = {{&S34277[95], 9}, {NULL}};
const Segment SGI139333[] = {{&S34277[68], 6}, {NULL}};
const Segment SGI139334[] = {{&S34278[34], 10}, {NULL}};
const Segment SGI139335[] = {{&S34278[27], 6}, {NULL}};
const Segment SGI139336[] = {{&S34278[97], 9}, {NULL}};
const Segment SGI139337[] = {{&S34278[70], 6}, {NULL}};
const Segment SGI139338[] = {{&S34279[34], 7}, {NULL}};
const Segment SGI139339[] = {{&S34279[27], 6}, {NULL}};
const Segment SGI139340[] = {{&S34279[97], 9}, {NULL}};
const Segment SGI139341[] = {{&S34279[70], 6}, {NULL}};
const Segment SGI139342[] = {{&S34280[34], 11}, {NULL}};
const Segment SGI139343[] = {{&S34280[27], 6}, {NULL}};
const Segment SGI139344[] = {{&S34280[98], 9}, {NULL}};
const Segment SGI139345[] = {{&S34280[71], 6}, {NULL}};
const Segment SGI139346[] = {{&S34281[34], 9}, {NULL}};
const Segment SGI139347[] = {{&S34281[27], 6}, {NULL}};
const Segment SGI139348[] = {{&S34281[96], 9}, {NULL}};
const Segment SGI139349[] = {{&S34281[69], 6}, {NULL}};
const Segment SGI139350[] = {{&S34283[34], 10}, {NULL}};
const Segment SGI139351[] = {{&S34283[27], 6}, {NULL}};
const Segment SGI139352[] = {{&S34283[97], 9}, {NULL}};
const Segment SGI139353[] = {{&S34283[70], 6}, {NULL}};
const Segment SGI139354[] = {{&S34284[34], 9}, {NULL}};
const Segment SGI139355[] = {{&S34284[27], 6}, {NULL}};
const Segment SGI139356[] = {{&S34284[96], 9}, {NULL}};
const Segment SGI139357[] = {{&S34284[69], 6}, {NULL}};
const Segment SGI139358[] = {{&S34285[34], 10}, {NULL}};
const Segment SGI139359[] = {{&S34285[27], 6}, {NULL}};
const Segment SGI139360[] = {{&S34285[97], 9}, {NULL}};
const Segment SGI139361[] = {{&S34285[70], 6}, {NULL}};
const Segment SGI139362[] = {{&S34286[34], 4}, {NULL}};
const Segment SGI139363[] = {{&S34286[27], 6}, {NULL}};
const Segment SGI139364[] = {{&S34286[91], 9}, {NULL}};
const Segment SGI139365[] = {{&S34286[64], 6}, {NULL}};
const Segment SGI139366[] = {{&S34287[34], 10}, {NULL}};
const Segment SGI139367[] = {{&S34287[27], 6}, {NULL}};
const Segment SGI139368[] = {{&S34287[97], 9}, {NULL}};
const Segment SGI139369[] = {{&S34287[70], 6}, {NULL}};
const Segment SGI139370[] = {{&S34288[34], 6}, {NULL}};
const Segment SGI139371[] = {{&S34288[27], 6}, {NULL}};
const Segment SGI139372[] = {{&S34288[93], 9}, {NULL}};
const Segment SGI139373[] = {{&S34288[66], 6}, {NULL}};
const Segment SGI139374[] = {{&S34289[34], 7}, {NULL}};
const Segment SGI139375[] = {{&S34289[27], 6}, {NULL}};
const Segment SGI139376[] = {{&S34289[106], 9}, {NULL}};
const Segment SGI139377[] = {{&S34289[79], 6}, {NULL}};
const Segment SGI139378[] = {{&S34290[34], 11}, {NULL}};
const Segment SGI139379[] = {{&S34290[27], 6}, {NULL}};
const Segment SGI139380[] = {{&S34290[98], 9}, {NULL}};
const Segment SGI139381[] = {{&S34290[71], 6}, {NULL}};
const Segment SGI139382[] = {{&S34291[34], 9}, {NULL}};
const Segment SGI139383[] = {{&S34291[27], 6}, {NULL}};
const Segment SGI139384[] = {{&S34291[96], 9}, {NULL}};
const Segment SGI139385[] = {{&S34291[69], 6}, {NULL}};
const Segment SGI139386[] = {{&S34292[34], 9}, {NULL}};
const Segment SGI139387[] = {{&S34292[27], 6}, {NULL}};
const Segment SGI139388[] = {{&S34292[96], 9}, {NULL}};
const Segment SGI139389[] = {{&S34292[69], 6}, {NULL}};
const Segment SGI139390[] = {{&S34293[34], 8}, {NULL}};
const Segment SGI139391[] = {{&S34293[27], 6}, {NULL}};
const Segment SGI139392[] = {{&S34293[95], 9}, {NULL}};
const Segment SGI139393[] = {{&S34293[68], 6}, {NULL}};
const Segment SGI139394[] = {{&S34294[34], 22}, {NULL}};
const Segment SGI139395[] = {{&S34294[27], 6}, {NULL}};
const Segment SGI139396[] = {{&S34294[109], 9}, {NULL}};
const Segment SGI139397[] = {{&S34294[82], 6}, {NULL}};
const Segment SGI139398[] = {{&S34296[34], 11}, {NULL}};
const Segment SGI139399[] = {{&S34296[27], 6}, {NULL}};
const Segment SGI139400[] = {{&S34296[98], 9}, {NULL}};
const Segment SGI139401[] = {{&S34296[71], 6}, {NULL}};
const Segment SGI139402[] = {{&S34298[34], 11}, {NULL}};
const Segment SGI139403[] = {{&S34298[27], 6}, {NULL}};
const Segment SGI139404[] = {{&S34298[101], 9}, {NULL}};
const Segment SGI139405[] = {{&S34298[74], 6}, {NULL}};
const Segment SGI139406[] = {{&S34299[34], 22}, {NULL}};
const Segment SGI139407[] = {{&S34299[27], 6}, {NULL}};
const Segment SGI139408[] = {{&S34299[109], 9}, {NULL}};
const Segment SGI139409[] = {{&S34299[82], 6}, {NULL}};
const Segment SGI139410[] = {{&S34300[34], 5}, {NULL}};
const Segment SGI139411[] = {{&S34300[27], 6}, {NULL}};
const Segment SGI139412[] = {{&S34300[92], 9}, {NULL}};
const Segment SGI139413[] = {{&S34300[65], 6}, {NULL}};
const Segment SGI139414[] = {{&S34301[34], 6}, {NULL}};
const Segment SGI139415[] = {{&S34301[27], 6}, {NULL}};
const Segment SGI139416[] = {{&S34301[93], 9}, {NULL}};
const Segment SGI139417[] = {{&S34301[66], 6}, {NULL}};
const Segment SGI139418[] = {{&S34302[34], 15}, {NULL}};
const Segment SGI139419[] = {{&S34302[27], 6}, {NULL}};
const Segment SGI139420[] = {{&S34302[102], 9}, {NULL}};
const Segment SGI139421[] = {{&S34302[75], 6}, {NULL}};
const Segment SGI139422[] = {{&S34303[34], 15}, {NULL}};
const Segment SGI139423[] = {{&S34303[27], 6}, {NULL}};
const Segment SGI139424[] = {{&S34303[102], 9}, {NULL}};
const Segment SGI139425[] = {{&S34303[75], 6}, {NULL}};
const Segment SGI139426[] = {{&S34304[34], 6}, {NULL}};
const Segment SGI139427[] = {{&S34304[27], 6}, {NULL}};
const Segment SGI139428[] = {{&S34304[93], 9}, {NULL}};
const Segment SGI139429[] = {{&S34304[66], 6}, {NULL}};
const Segment SGI139430[] = {{&S34305[34], 10}, {NULL}};
const Segment SGI139431[] = {{&S34305[27], 6}, {NULL}};
const Segment SGI139432[] = {{&S34305[97], 9}, {NULL}};
const Segment SGI139433[] = {{&S34305[70], 6}, {NULL}};
const Segment SGI139434[] = {{&S34306[34], 11}, {NULL}};
const Segment SGI139435[] = {{&S34306[27], 6}, {NULL}};
const Segment SGI139436[] = {{&S34306[98], 9}, {NULL}};
const Segment SGI139437[] = {{&S34306[71], 6}, {NULL}};
const Segment SGI139438[] = {{&S34307[34], 6}, {NULL}};
const Segment SGI139439[] = {{&S34307[27], 6}, {NULL}};
const Segment SGI139440[] = {{&S34307[93], 9}, {NULL}};
const Segment SGI139441[] = {{&S34307[66], 6}, {NULL}};
const Segment SGI139442[] = {{&S34309[34], 12}, {NULL}};
const Segment SGI139443[] = {{&S34309[27], 6}, {NULL}};
const Segment SGI139444[] = {{&S34309[99], 9}, {NULL}};
const Segment SGI139445[] = {{&S34309[72], 6}, {NULL}};
const Segment SGI139446[] = {{&S34310[34], 9}, {NULL}};
const Segment SGI139447[] = {{&S34310[27], 6}, {NULL}};
const Segment SGI139448[] = {{&S34310[96], 9}, {NULL}};
const Segment SGI139449[] = {{&S34310[69], 6}, {NULL}};
const Segment SGI139450[] = {{&S34316[34], 8}, {NULL}};
const Segment SGI139451[] = {{&S34316[27], 6}, {NULL}};
const Segment SGI139452[] = {{&S34316[95], 9}, {NULL}};
const Segment SGI139453[] = {{&S34316[68], 6}, {NULL}};
const Segment SGI139454[] = {{&S34320[34], 12}, {NULL}};
const Segment SGI139455[] = {{&S34320[27], 6}, {NULL}};
const Segment SGI139456[] = {{&S34320[99], 9}, {NULL}};
const Segment SGI139457[] = {{&S34320[72], 6}, {NULL}};
const Segment SGI139458[] = {{&S34323[34], 8}, {NULL}};
const Segment SGI139459[] = {{&S34323[27], 6}, {NULL}};
const Segment SGI139460[] = {{&S34323[95], 9}, {NULL}};
const Segment SGI139461[] = {{&S34323[68], 6}, {NULL}};
const Segment SGI139462[] = {{&S34324[34], 6}, {NULL}};
const Segment SGI139463[] = {{&S34324[27], 6}, {NULL}};
const Segment SGI139464[] = {{&S34324[93], 9}, {NULL}};
const Segment SGI139465[] = {{&S34324[66], 6}, {NULL}};
const Segment SGI139466[] = {{&S34326[34], 6}, {NULL}};
const Segment SGI139467[] = {{&S34326[27], 6}, {NULL}};
const Segment SGI139468[] = {{&S34326[93], 9}, {NULL}};
const Segment SGI139469[] = {{&S34326[66], 6}, {NULL}};
const Segment SGI139470[] = {{&S34327[34], 10}, {NULL}};
const Segment SGI139471[] = {{&S34327[27], 6}, {NULL}};
const Segment SGI139472[] = {{&S34327[97], 9}, {NULL}};
const Segment SGI139473[] = {{&S34327[70], 6}, {NULL}};
const Segment SGI139474[] = {{&S34328[34], 15}, {NULL}};
const Segment SGI139475[] = {{&S34328[27], 6}, {NULL}};
const Segment SGI139476[] = {{&S34328[102], 9}, {NULL}};
const Segment SGI139477[] = {{&S34328[75], 6}, {NULL}};
const Segment SGI139478[] = {{&S24162[34], 11}, {NULL}};
const Segment SGI139479[] = {{&S24162[27], 6}, {NULL}};
const Segment SGI139480[] = {{&S24162[98], 9}, {NULL}};
const Segment SGI139481[] = {{&S24162[71], 6}, {NULL}};
const Segment SGI139482[] = {{&S34329[34], 14}, {NULL}};
const Segment SGI139483[] = {{&S34329[27], 6}, {NULL}};
const Segment SGI139484[] = {{&S34329[101], 9}, {NULL}};
const Segment SGI139485[] = {{&S34329[74], 6}, {NULL}};
const Segment SGI139486[] = {{&S24163[34], 11}, {NULL}};
const Segment SGI139487[] = {{&S24163[27], 6}, {NULL}};
const Segment SGI139488[] = {{&S24163[98], 9}, {NULL}};
const Segment SGI139489[] = {{&S24163[71], 6}, {NULL}};
const Segment SGI139490[] = {{&S24165[34], 10}, {NULL}};
const Segment SGI139491[] = {{&S24165[27], 6}, {NULL}};
const Segment SGI139492[] = {{&S24165[97], 9}, {NULL}};
const Segment SGI139493[] = {{&S24165[70], 6}, {NULL}};
const Segment SGI139494[] = {{&S34332[34], 7}, {NULL}};
const Segment SGI139495[] = {{&S34332[27], 6}, {NULL}};
const Segment SGI139496[] = {{&S34332[94], 9}, {NULL}};
const Segment SGI139497[] = {{&S34332[67], 6}, {NULL}};
const Segment SGI139498[] = {{&S24166[34], 11}, {NULL}};
const Segment SGI139499[] = {{&S24166[27], 6}, {NULL}};
const Segment SGI139500[] = {{&S24166[98], 9}, {NULL}};
const Segment SGI139501[] = {{&S24166[71], 6}, {NULL}};
const Segment SGI139502[] = {{&S34333[34], 6}, {NULL}};
const Segment SGI139503[] = {{&S34333[27], 6}, {NULL}};
const Segment SGI139504[] = {{&S34333[93], 9}, {NULL}};
const Segment SGI139505[] = {{&S34333[66], 6}, {NULL}};
const Segment SGI139506[] = {{&S24167[34], 18}, {NULL}};
const Segment SGI139507[] = {{&S24167[27], 6}, {NULL}};
const Segment SGI139508[] = {{&S24167[105], 9}, {NULL}};
const Segment SGI139509[] = {{&S24167[78], 6}, {NULL}};
const Segment SGI139510[] = {{&S34335[34], 5}, {NULL}};
const Segment SGI139511[] = {{&S34335[27], 6}, {NULL}};
const Segment SGI139512[] = {{&S24168[34], 10}, {NULL}};
const Segment SGI139513[] = {{&S24168[27], 6}, {NULL}};
const Segment SGI139514[] = {{&S34335[92], 9}, {NULL}};
const Segment SGI139515[] = {{&S34335[65], 6}, {NULL}};
const Segment SGI139516[] = {{&S24168[97], 9}, {NULL}};
const Segment SGI139517[] = {{&S24168[70], 6}, {NULL}};
const Segment SGI139518[] = {{&S24169[34], 10}, {NULL}};
const Segment SGI139519[] = {{&S24169[27], 6}, {NULL}};
const Segment SGI139520[] = {{&S34337[34], 8}, {NULL}};
const Segment SGI139521[] = {{&S24169[97], 9}, {NULL}};
const Segment SGI139522[] = {{&S24169[70], 6}, {NULL}};
const Segment SGI139523[] = {{&S34337[27], 6}, {NULL}};
const Segment SGI139524[] = {{&S34337[95], 9}, {NULL}};
const Segment SGI139525[] = {{&S34337[68], 6}, {NULL}};
const Segment SGI139526[] = {{&S34338[34], 11}, {NULL}};
const Segment SGI139527[] = {{&S24171[34], 10}, {NULL}};
const Segment SGI139528[] = {{&S34338[27], 6}, {NULL}};
const Segment SGI139529[] = {{&S24171[27], 6}, {NULL}};
const Segment SGI139530[] = {{&S34338[98], 9}, {NULL}};
const Segment SGI139531[] = {{&S24171[97], 9}, {NULL}};
const Segment SGI139532[] = {{&S34338[71], 6}, {NULL}};
const Segment SGI139533[] = {{&S24171[70], 6}, {NULL}};
const Segment SGI139534[] = {{&S24173[34], 10}, {NULL}};
const Segment SGI139535[] = {{&S34340[34], 14}, {NULL}};
const Segment SGI139536[] = {{&S24173[27], 6}, {NULL}};
const Segment SGI139537[] = {{&S34340[27], 6}, {NULL}};
const Segment SGI139538[] = {{&S24173[97], 9}, {NULL}};
const Segment SGI139539[] = {{&S24173[70], 6}, {NULL}};
const Segment SGI139540[] = {{&S34340[101], 9}, {NULL}};
const Segment SGI139541[] = {{&S34340[74], 6}, {NULL}};
const Segment SGI139542[] = {{&S34341[34], 10}, {NULL}};
const Segment SGI139543[] = {{&S34341[27], 6}, {NULL}};
const Segment SGI139544[] = {{&S24175[34], 11}, {NULL}};
const Segment SGI139545[] = {{&S24175[27], 6}, {NULL}};
const Segment SGI139546[] = {{&S34341[97], 9}, {NULL}};
const Segment SGI139547[] = {{&S34341[70], 6}, {NULL}};
const Segment SGI139548[] = {{&S24175[98], 9}, {NULL}};
const Segment SGI139549[] = {{&S24175[71], 6}, {NULL}};
const Segment SGI139550[] = {{&S24177[34], 10}, {NULL}};
const Segment SGI139551[] = {{&S24177[27], 6}, {NULL}};
const Segment SGI139552[] = {{&S34343[34], 7}, {NULL}};
const Segment SGI139553[] = {{&S34343[27], 6}, {NULL}};
const Segment SGI139554[] = {{&S24177[97], 9}, {NULL}};
const Segment SGI139555[] = {{&S24177[70], 6}, {NULL}};
const Segment SGI139556[] = {{&S34343[94], 9}, {NULL}};
const Segment SGI139557[] = {{&S34343[67], 6}, {NULL}};
const Segment SGI139558[] = {{&S24178[34], 10}, {NULL}};
const Segment SGI139559[] = {{&S24178[27], 6}, {NULL}};
const Segment SGI139560[] = {{&S34344[34], 9}, {NULL}};
const Segment SGI139561[] = {{&S34344[27], 6}, {NULL}};
const Segment SGI139562[] = {{&S24178[97], 9}, {NULL}};
const Segment SGI139563[] = {{&S24178[70], 6}, {NULL}};
const Segment SGI139564[] = {{&S34344[96], 9}, {NULL}};
const Segment SGI139565[] = {{&S34344[69], 6}, {NULL}};
const Segment SGI139566[] = {{&S24179[34], 11}, {NULL}};
const Segment SGI139567[] = {{&S24179[27], 6}, {NULL}};
const Segment SGI139568[] = {{&S34345[34], 12}, {NULL}};
const Segment SGI139569[] = {{&S34345[27], 6}, {NULL}};
const Segment SGI139570[] = {{&S24179[98], 9}, {NULL}};
const Segment SGI139571[] = {{&S24179[71], 6}, {NULL}};
const Segment SGI139572[] = {{&S34345[99], 9}, {NULL}};
const Segment SGI139573[] = {{&S34345[72], 6}, {NULL}};
const Segment SGI139574[] = {{&S34346[34], 9}, {NULL}};
const Segment SGI139575[] = {{&S34346[27], 6}, {NULL}};
const Segment SGI139576[] = {{&S24181[34], 11}, {NULL}};
const Segment SGI139577[] = {{&S24181[27], 6}, {NULL}};
const Segment SGI139578[] = {{&S34346[96], 9}, {NULL}};
const Segment SGI139579[] = {{&S34346[69], 6}, {NULL}};
const Segment SGI139580[] = {{&S24181[98], 9}, {NULL}};
const Segment SGI139581[] = {{&S24181[71], 6}, {NULL}};
const Segment SGI139582[] = {{&S24182[34], 11}, {NULL}};
const Segment SGI139583[] = {{&S24182[27], 6}, {NULL}};
const Segment SGI139584[] = {{&S24182[98], 9}, {NULL}};
const Segment SGI139585[] = {{&S24182[71], 6}, {NULL}};
const Segment SGI139586[] = {{&S34348[34], 8}, {NULL}};
const Segment SGI139587[] = {{&S34348[27], 6}, {NULL}};
const Segment SGI139588[] = {{&S34348[95], 9}, {NULL}};
const Segment SGI139589[] = {{&S34348[68], 6}, {NULL}};
const Segment SGI139590[] = {{&S24183[34], 10}, {NULL}};
const Segment SGI139591[] = {{&S24183[27], 6}, {NULL}};
const Segment SGI139592[] = {{&S24183[97], 9}, {NULL}};
const Segment SGI139593[] = {{&S24183[70], 6}, {NULL}};
const Segment SGI139594[] = {{&S34349[34], 8}, {NULL}};
const Segment SGI139595[] = {{&S34349[27], 6}, {NULL}};
const Segment SGI139596[] = {{&S34349[95], 9}, {NULL}};
const Segment SGI139597[] = {{&S34349[68], 6}, {NULL}};
const Segment SGI139598[] = {{&S24184[34], 10}, {NULL}};
const Segment SGI139599[] = {{&S24184[27], 6}, {NULL}};
const Segment SGI139600[] = {{&S24184[97], 9}, {NULL}};
const Segment SGI139601[] = {{&S24184[70], 6}, {NULL}};
const Segment SGI139602[] = {{&S34350[34], 4}, {NULL}};
const Segment SGI139603[] = {{&S34350[27], 6}, {NULL}};
const Segment SGI139604[] = {{&S34350[91], 9}, {NULL}};
const Segment SGI139605[] = {{&S34350[64], 6}, {NULL}};
const Segment SGI139606[] = {{&S34351[34], 6}, {NULL}};
const Segment SGI139607[] = {{&S34351[27], 6}, {NULL}};
const Segment SGI139608[] = {{&S34351[93], 9}, {NULL}};
const Segment SGI139609[] = {{&S34351[66], 6}, {NULL}};
const Segment SGI139610[] = {{&S24188[34], 10}, {NULL}};
const Segment SGI139611[] = {{&S24188[27], 6}, {NULL}};
const Segment SGI139612[] = {{&S34352[34], 15}, {NULL}};
const Segment SGI139613[] = {{&S34352[27], 6}, {NULL}};
const Segment SGI139614[] = {{&S24188[97], 9}, {NULL}};
const Segment SGI139615[] = {{&S24188[70], 6}, {NULL}};
const Segment SGI139616[] = {{&S34352[102], 9}, {NULL}};
const Segment SGI139617[] = {{&S34352[75], 6}, {NULL}};
const Segment SGI139618[] = {{&S34353[34], 9}, {NULL}};
const Segment SGI139619[] = {{&S34353[27], 6}, {NULL}};
const Segment SGI139620[] = {{&S34353[96], 9}, {NULL}};
const Segment SGI139621[] = {{&S34353[69], 6}, {NULL}};
const Segment SGI139622[] = {{&S34355[34], 11}, {NULL}};
const Segment SGI139623[] = {{&S34355[27], 6}, {NULL}};
const Segment SGI139624[] = {{&S34355[98], 9}, {NULL}};
const Segment SGI139625[] = {{&S34355[71], 6}, {NULL}};
const Segment SGI139626[] = {{&S34356[34], 9}, {NULL}};
const Segment SGI139627[] = {{&S34356[27], 6}, {NULL}};
const Segment SGI139628[] = {{&S34356[96], 9}, {NULL}};
const Segment SGI139629[] = {{&S34356[69], 6}, {NULL}};
const Segment SGI139630[] = {{&S34357[34], 14}, {NULL}};
const Segment SGI139631[] = {{&S34357[27], 6}, {NULL}};
const Segment SGI139632[] = {{&S34357[101], 9}, {NULL}};
const Segment SGI139633[] = {{&S34357[74], 6}, {NULL}};
const Segment SGI139634[] = {{&S34358[34], 10}, {NULL}};
const Segment SGI139635[] = {{&S34358[27], 6}, {NULL}};
const Segment SGI139636[] = {{&S34358[97], 9}, {NULL}};
const Segment SGI139637[] = {{&S34358[70], 6}, {NULL}};
const Segment SGI139638[] = {{&S34360[34], 7}, {NULL}};
const Segment SGI139639[] = {{&S34360[27], 6}, {NULL}};
const Segment SGI139640[] = {{&S34360[94], 9}, {NULL}};
const Segment SGI139641[] = {{&S34360[67], 6}, {NULL}};
const Segment SGI139642[] = {{&S34361[34], 10}, {NULL}};
const Segment SGI139643[] = {{&S34361[27], 6}, {NULL}};
const Segment SGI139644[] = {{&S34361[97], 9}, {NULL}};
const Segment SGI139645[] = {{&S34361[70], 6}, {NULL}};
const Segment SGI139646[] = {{&S34362[34], 13}, {NULL}};
const Segment SGI139647[] = {{&S34362[27], 6}, {NULL}};
const Segment SGI139648[] = {{&S34362[100], 9}, {NULL}};
const Segment SGI139649[] = {{&S34362[73], 6}, {NULL}};
const Segment SGI139650[] = {{&S34363[34], 9}, {NULL}};
const Segment SGI139651[] = {{&S34363[27], 6}, {NULL}};
const Segment SGI139652[] = {{&S34363[91], 9}, {NULL}};
const Segment SGI139653[] = {{&S34363[64], 6}, {NULL}};
const Segment SGI139654[] = {{&S34364[34], 15}, {NULL}};
const Segment SGI139655[] = {{&S34364[27], 6}, {NULL}};
const Segment SGI139656[] = {{&S34364[102], 9}, {NULL}};
const Segment SGI139657[] = {{&S34364[75], 6}, {NULL}};
const Segment SGI139658[] = {{&S34365[34], 8}, {NULL}};
const Segment SGI139659[] = {{&S34365[27], 6}, {NULL}};
const Segment SGI139660[] = {{&S34365[95], 9}, {NULL}};
const Segment SGI139661[] = {{&S34365[68], 6}, {NULL}};
const Segment SGI139662[] = {{&S34366[34], 9}, {NULL}};
const Segment SGI139663[] = {{&S34366[27], 6}, {NULL}};
const Segment SGI139664[] = {{&S34366[96], 9}, {NULL}};
const Segment SGI139665[] = {{&S34366[69], 6}, {NULL}};
const Segment SGI139666[] = {{&S34367[34], 9}, {NULL}};
const Segment SGI139667[] = {{&S34367[27], 6}, {NULL}};
const Segment SGI139668[] = {{&S34367[96], 9}, {NULL}};
const Segment SGI139669[] = {{&S34367[69], 6}, {NULL}};
const Segment SGI139670[] = {{&S34368[34], 14}, {NULL}};
const Segment SGI139671[] = {{&S34368[27], 6}, {NULL}};
const Segment SGI139672[] = {{&S34368[101], 9}, {NULL}};
const Segment SGI139673[] = {{&S34368[74], 6}, {NULL}};
const Segment SGI139674[] = {{&S34369[34], 11}, {NULL}};
const Segment SGI139675[] = {{&S34369[27], 6}, {NULL}};
const Segment SGI139676[] = {{&S34369[98], 9}, {NULL}};
const Segment SGI139677[] = {{&S34369[71], 6}, {NULL}};
const Segment SGI139678[] = {{&S34370[34], 5}, {NULL}};
const Segment SGI139679[] = {{&S34370[27], 6}, {NULL}};
const Segment SGI139680[] = {{&S34370[92], 9}, {NULL}};
const Segment SGI139681[] = {{&S25197[44], 10}, {NULL}};
const Segment SGI139682[] = {{&S34370[65], 6}, {NULL}};
const Segment SGI139683[] = {{&S25197[44], 22}, {NULL}};
const Segment SGI139684[] = {{&S25197[30], 6}, {NULL}};
const Segment SGI139685[] = {{&S25197[115], 3}, {NULL}};
const Segment SGI139686[] = {{&S25197[80], 6}, {NULL}};
const Segment SGI139687[] = {{&S25198[38], 10}, {NULL}};
const Segment SGI139688[] = {{&S25198[31], 6}, {NULL}};
const Segment SGI139689[] = {{&S34372[34], 10}, {NULL}};
const Segment SGI139690[] = {{&S34372[27], 6}, {NULL}};
const Segment SGI139691[] = {{&S34372[97], 9}, {NULL}};
const Segment SGI139692[] = {{&S34372[70], 6}, {NULL}};
const Segment SGI139693[] = {{&S25199[42], 10}, {NULL}};
const Segment SGI139694[] = {{&S25199[30], 6}, {NULL}};
const Segment SGI139695[] = {{&S25199[113], 3}, {NULL}};
const Segment SGI139696[] = {{&S25199[78], 6}, {NULL}};
const Segment SGI139697[] = {{&S34374[34], 14}, {NULL}};
const Segment SGI139698[] = {{&S34374[27], 6}, {NULL}};
const Segment SGI139699[] = {{&S34374[101], 9}, {NULL}};
const Segment SGI139700[] = {{&S34374[74], 6}, {NULL}};
const Segment SGI139701[] = {{&S25201[44], 10}, {NULL}};
const Segment SGI139702[] = {{&S25201[44], 22}, {NULL}};
const Segment SGI139703[] = {{&S25201[30], 6}, {NULL}};
const Segment SGI139704[] = {{&S34375[34], 5}, {NULL}};
const Segment SGI139705[] = {{&S25201[115], 3}, {NULL}};
const Segment SGI139706[] = {{&S34375[27], 6}, {NULL}};
const Segment SGI139707[] = {{&S25201[80], 6}, {NULL}};
const Segment SGI139708[] = {{&S34375[92], 9}, {NULL}};
const Segment SGI139709[] = {{&S34375[65], 6}, {NULL}};
const Segment SGI139710[] = {{&S25202[44], 10}, {NULL}};
const Segment SGI139711[] = {{&S25202[44], 22}, {NULL}};
const Segment SGI139712[] = {{&S25202[30], 6}, {NULL}};
const Segment SGI139713[] = {{&S25202[115], 3}, {NULL}};
const Segment SGI139714[] = {{&S25202[80], 6}, {NULL}};
const Segment SGI139715[] = {{&S25205[44], 10}, {NULL}};
const Segment SGI139716[] = {{&S25205[44], 22}, {NULL}};
const Segment SGI139717[] = {{&S25205[30], 6}, {NULL}};
const Segment SGI139718[] = {{&S25205[115], 3}, {NULL}};
const Segment SGI139719[] = {{&S25205[80], 6}, {NULL}};
const Segment SGI139720[] = {{&S25206[44], 10}, {NULL}};
const Segment SGI139721[] = {{&S25206[44], 19}, {NULL}};
const Segment SGI139722[] = {{&S25206[30], 6}, {NULL}};
const Segment SGI139723[] = {{&S25206[111], 3}, {NULL}};
const Segment SGI139724[] = {{&S25206[77], 5}, {NULL}};
const Segment SGI139725[] = {{&S25207[44], 10}, {NULL}};
const Segment SGI139726[] = {{&S25207[44], 22}, {NULL}};
const Segment SGI139727[] = {{&S25207[30], 6}, {NULL}};
const Segment SGI139728[] = {{&S25207[80], 6}, {NULL}};
const Segment SGI139729[] = {{&S25208[65], 10}, {NULL}};
const Segment SGI139730[] = {{&S25208[65], 22}, {NULL}};
const Segment SGI139731[] = {{&S25208[51], 6}, {NULL}};
const Segment SGI139732[] = {{&S25209[44], 10}, {NULL}};
const Segment SGI139733[] = {{&S25209[44], 22}, {NULL}};
const Segment SGI139734[] = {{&S25209[30], 6}, {NULL}};
const Segment SGI139735[] = {{&S25209[83], 6}, {NULL}};
const Segment SGI139736[] = {{&S25210[45], 10}, {NULL}};
const Segment SGI139737[] = {{&S25210[45], 34}, {NULL}};
const Segment SGI139738[] = {{&S25210[31], 6}, {NULL}};
const Segment SGI139739[] = {{&S25211[44], 10}, {NULL}};
const Segment SGI139740[] = {{&S25211[44], 22}, {NULL}};
const Segment SGI139741[] = {{&S25211[30], 6}, {NULL}};
const Segment SGI139742[] = {{&S25211[114], 3}, {NULL}};
const Segment SGI139743[] = {{&S25211[80], 5}, {NULL}};
const Segment SGI139744[] = {{&S25212[30], 1}, {NULL}};
const Segment SGI139745[] = {{&S25212[63], 9}, {NULL}};
const Segment SGI139746[] = {{&S25212[49], 6}, {NULL}};
const Segment SGI139747[] = {{&S25213[44], 10}, {NULL}};
const Segment SGI139748[] = {{&S25213[44], 22}, {NULL}};
const Segment SGI139749[] = {{&S25213[30], 6}, {NULL}};
const Segment SGI139750[] = {{&S25213[127], 3}, {NULL}};
const Segment SGI139751[] = {{&S25213[80], 6}, {NULL}};
const Segment SGI139752[] = {{&S25214[44], 10}, {NULL}};
const Segment SGI139753[] = {{&S25214[44], 22}, {NULL}};
const Segment SGI139754[] = {{&S25214[30], 6}, {NULL}};
const Segment SGI139755[] = {{&S25214[8], 1}, {NULL}};
const Segment SGI139756[] = {{&S25215[44], 10}, {NULL}};
const Segment SGI139757[] = {{&S25215[44], 22}, {NULL}};
const Segment SGI139758[] = {{&S25215[30], 6}, {NULL}};
const Segment SGI139759[] = {{&S25215[80], 6}, {NULL}};
const Segment SGI139760[] = {{&S25216[44], 17}, {NULL}};
const Segment SGI139761[] = {{&S25216[44], 29}, {NULL}};
const Segment SGI139762[] = {{&S25216[30], 6}, {NULL}};
const Segment SGI139763[] = {{&S25216[115], 3}, {NULL}};
const Segment SGI139764[] = {{&S24272[34], 11}, {NULL}};
const Segment SGI139765[] = {{&S24272[27], 6}, {NULL}};
const Segment SGI139766[] = {{&S24272[98], 9}, {NULL}};
const Segment SGI139767[] = {{&S24272[71], 6}, {NULL}};
const Segment SGI139768[] = {{&S25217[44], 10}, {NULL}};
const Segment SGI139769[] = {{&S25217[44], 22}, {NULL}};
const Segment SGI139770[] = {{&S25217[30], 6}, {NULL}};
const Segment SGI139771[] = {{&S25217[116], 3}, {NULL}};
const Segment SGI139772[] = {{&S25217[80], 7}, {NULL}};
const Segment SGI139773[] = {{&S25217[136], 8}, {NULL}};
const Segment SGI139774[] = {{&S24273[34], 10}, {NULL}};
const Segment SGI139775[] = {{&S24273[27], 6}, {NULL}};
const Segment SGI139776[] = {{&S24273[97], 9}, {NULL}};
const Segment SGI139777[] = {{&S24273[70], 6}, {NULL}};
const Segment SGI139778[] = {{&S24274[34], 18}, {NULL}};
const Segment SGI139779[] = {{&S24274[27], 6}, {NULL}};
const Segment SGI139780[] = {{&S25218[44], 43}, {NULL}};
const Segment SGI139781[] = {{&S25218[30], 6}, {NULL}};
const Segment SGI139782[] = {{&S24274[105], 9}, {NULL}};
const Segment SGI139783[] = {{&S24274[78], 6}, {NULL}};
const Segment SGI139784[] = {{&S24275[34], 10}, {NULL}};
const Segment SGI139785[] = {{&S24275[27], 6}, {NULL}};
const Segment SGI139786[] = {{&S24275[97], 9}, {NULL}};
const Segment SGI139787[] = {{&S24275[70], 6}, {NULL}};
const Segment SGI139788[] = {{&S25219[44], 10}, {NULL}};
const Segment SGI139789[] = {{&S25219[44], 22}, {&S25219[225], 5}, {&S25219[147], 1}, {NULL}};
const Segment SGI139790[] = {{&S25219[30], 6}, {NULL}};
const Segment SGI139791[] = {{&S25219[115], 3}, {NULL}};
const Segment SGI139792[] = {{&S25219[80], 6}, {NULL}};
const Segment SGI139793[] = {{&S25220[44], 9}, {NULL}};
const Segment SGI139794[] = {{&S25220[30], 6}, {NULL}};
const Segment SGI139795[] = {{&S25221[44], 10}, {NULL}};
const Segment SGI139796[] = {{&S25221[44], 22}, {NULL}};
const Segment SGI139797[] = {{&S25221[30], 6}, {NULL}};
const Segment SGI139798[] = {{&S24280[34], 10}, {NULL}};
const Segment SGI139799[] = {{&S25221[115], 3}, {NULL}};
const Segment SGI139800[] = {{&S24280[27], 6}, {NULL}};
const Segment SGI139801[] = {{&S25221[80], 6}, {NULL}};
const Segment SGI139802[] = {{&S24280[97], 9}, {NULL}};
const Segment SGI139803[] = {{&S24280[70], 6}, {NULL}};
const Segment SGI139804[] = {{&S24281[34], 11}, {NULL}};
const Segment SGI139805[] = {{&S24281[27], 6}, {NULL}};
const Segment SGI139806[] = {{&S24281[98], 9}, {NULL}};
const Segment SGI139807[] = {{&S24281[71], 6}, {NULL}};
const Segment SGI139808[] = {{&S24282[34], 11}, {NULL}};
const Segment SGI139809[] = {{&S24282[27], 6}, {NULL}};
const Segment SGI139810[] = {{&S24282[98], 9}, {NULL}};
const Segment SGI139811[] = {{&S24282[71], 6}, {NULL}};
const Segment SGI139812[] = {{&S24283[34], 11}, {NULL}};
const Segment SGI139813[] = {{&S24283[27], 6}, {NULL}};
const Segment SGI139814[] = {{&S24283[98], 9}, {NULL}};
const Segment SGI139815[] = {{&S24283[71], 6}, {NULL}};
const Segment SGI139816[] = {{&S25227[44], 10}, {NULL}};
const Segment SGI139817[] = {{&S25227[44], 22}, {NULL}};
const Segment SGI139818[] = {{&S24284[34], 11}, {NULL}};
const Segment SGI139819[] = {{&S25227[30], 6}, {NULL}};
const Segment SGI139820[] = {{&S24284[27], 6}, {NULL}};
const Segment SGI139821[] = {{&S24284[98], 9}, {NULL}};
const Segment SGI139822[] = {{&S24284[71], 6}, {NULL}};
const Segment SGI139823[] = {{&S25227[115], 3}, {NULL}};
const Segment SGI139824[] = {{&S34376[34], 9}, {NULL}};
const Segment SGI139825[] = {{&S34376[27], 6}, {NULL}};
const Segment SGI139826[] = {{&S24285[34], 11}, {NULL}};
const Segment SGI139827[] = {{&S24285[27], 6}, {NULL}};
const Segment SGI139828[] = {{&S34376[96], 9}, {NULL}};
const Segment SGI139829[] = {{&S34376[69], 6}, {NULL}};
const Segment SGI139830[] = {{&S24285[98], 9}, {NULL}};
const Segment SGI139831[] = {{&S24285[71], 6}, {NULL}};
const Segment SGI139832[] = {{&S25228[44], 10}, {&S25228[44], 10}, {NULL}};
const Segment SGI139833[] = {{&S25228[44], 26}, {&S25228[71], 22}, {NULL}};
const Segment SGI139834[] = {{&S25228[30], 6}, {NULL}};
const Segment SGI139835[] = {{&S25228[115], 3}, {NULL}};
const Segment SGI139836[] = {{&S24286[34], 11}, {NULL}};
const Segment SGI139837[] = {{&S24286[27], 6}, {NULL}};
const Segment SGI139838[] = {{&S34378[34], 9}, {NULL}};
const Segment SGI139839[] = {{&S24286[98], 9}, {NULL}};
const Segment SGI139840[] = {{&S24286[71], 6}, {NULL}};
const Segment SGI139841[] = {{&S34378[27], 6}, {NULL}};
const Segment SGI139842[] = {{&S34378[103], 9}, {NULL}};
const Segment SGI139843[] = {{&S34378[76], 6}, {NULL}};
const Segment SGI139844[] = {{&S25229[44], 17}, {NULL}};
const Segment SGI139845[] = {{&S25229[30], 6}, {NULL}};
const Segment SGI139846[] = {{&S25229[108], 3}, {NULL}};
const Segment SGI139847[] = {{&S24287[34], 11}, {NULL}};
const Segment SGI139848[] = {{&S24287[27], 6}, {NULL}};
const Segment SGI139849[] = {{&S34379[34], 7}, {NULL}};
const Segment SGI139850[] = {{&S24287[98], 9}, {NULL}};
const Segment SGI139851[] = {{&S34379[27], 6}, {NULL}};
const Segment SGI139852[] = {{&S24287[71], 6}, {NULL}};
const Segment SGI139853[] = {{&S34379[94], 9}, {NULL}};
const Segment SGI139854[] = {{&S34379[67], 6}, {NULL}};
const Segment SGI139855[] = {{&S25231[44], 10}, {NULL}};
const Segment SGI139856[] = {{&S34380[34], 10}, {NULL}};
const Segment SGI139857[] = {{&S25231[44], 22}, {NULL}};
const Segment SGI139858[] = {{&S25231[30], 6}, {NULL}};
const Segment SGI139859[] = {{&S34380[27], 6}, {NULL}};
const Segment SGI139860[] = {{&S25231[125], 3}, {NULL}};
const Segment SGI139861[] = {{&S34380[97], 9}, {NULL}};
const Segment SGI139862[] = {{&S25231[90], 6}, {NULL}};
const Segment SGI139863[] = {{&S34380[70], 6}, {NULL}};
const Segment SGI139864[] = {{&S25231[145], 8}, {NULL}};
const Segment SGI139865[] = {{&S34381[34], 5}, {NULL}};
const Segment SGI139866[] = {{&S34381[27], 6}, {NULL}};
const Segment SGI139867[] = {{&S34381[92], 9}, {NULL}};
const Segment SGI139868[] = {{&S25232[44], 10}, {NULL}};
const Segment SGI139869[] = {{&S25232[44], 22}, {NULL}};
const Segment SGI139870[] = {{&S34381[65], 6}, {NULL}};
const Segment SGI139871[] = {{&S24289[34], 10}, {NULL}};
const Segment SGI139872[] = {{&S25232[30], 6}, {NULL}};
const Segment SGI139873[] = {{&S24289[27], 6}, {NULL}};
const Segment SGI139874[] = {{&S25232[115], 3}, {NULL}};
const Segment SGI139875[] = {{&S25232[80], 6}, {NULL}};
const Segment SGI139876[] = {{&S24289[97], 9}, {NULL}};
const Segment SGI139877[] = {{&S24289[70], 6}, {NULL}};
const Segment SGI139878[] = {{&S34382[34], 7}, {NULL}};
const Segment SGI139879[] = {{&S34382[27], 6}, {NULL}};
const Segment SGI139880[] = {{&S24290[34], 10}, {NULL}};
const Segment SGI139881[] = {{&S24290[27], 6}, {NULL}};
const Segment SGI139882[] = {{&S34382[94], 9}, {NULL}};
const Segment SGI139883[] = {{&S25233[44], 10}, {NULL}};
const Segment SGI139884[] = {{&S34382[67], 6}, {NULL}};
const Segment SGI139885[] = {{&S25233[44], 22}, {NULL}};
const Segment SGI139886[] = {{&S25233[30], 6}, {NULL}};
const Segment SGI139887[] = {{&S24290[97], 9}, {NULL}};
const Segment SGI139888[] = {{&S24290[70], 6}, {NULL}};
const Segment SGI139889[] = {{&S25233[115], 3}, {NULL}};
const Segment SGI139890[] = {{&S25233[80], 6}, {NULL}};
const Segment SGI139891[] = {{&S24291[34], 11}, {NULL}};
const Segment SGI139892[] = {{&S34383[34], 10}, {NULL}};
const Segment SGI139893[] = {{&S24291[27], 6}, {NULL}};
const Segment SGI139894[] = {{&S34383[27], 6}, {NULL}};
const Segment SGI139895[] = {{&S25234[44], 10}, {NULL}};
const Segment SGI139896[] = {{&S25234[44], 22}, {NULL}};
const Segment SGI139897[] = {{&S25234[30], 6}, {NULL}};
const Segment SGI139898[] = {{&S24291[98], 9}, {NULL}};
const Segment SGI139899[] = {{&S34383[105], 9}, {NULL}};
const Segment SGI139900[] = {{&S24291[71], 6}, {NULL}};
const Segment SGI139901[] = {{&S34383[78], 6}, {NULL}};
const Segment SGI139902[] = {{&S25234[115], 3}, {NULL}};
const Segment SGI139903[] = {{&S25234[80], 6}, {NULL}};
const Segment SGI139904[] = {{&S24292[34], 10}, {NULL}};
const Segment SGI139905[] = {{&S24292[27], 6}, {NULL}};
const Segment SGI139906[] = {{&S34384[34], 12}, {NULL}};
const Segment SGI139907[] = {{&S34384[27], 6}, {NULL}};
const Segment SGI139908[] = {{&S24292[97], 9}, {NULL}};
const Segment SGI139909[] = {{&S24292[70], 6}, {NULL}};
const Segment SGI139910[] = {{&S34384[99], 9}, {NULL}};
const Segment SGI139911[] = {{&S34384[72], 6}, {NULL}};
const Segment SGI139912[] = {{&S25236[38], 10}, {NULL}};
const Segment SGI139913[] = {{&S25236[31], 6}, {NULL}};
const Segment SGI139914[] = {{&S24293[34], 18}, {NULL}};
const Segment SGI139915[] = {{&S24293[27], 6}, {NULL}};
const Segment SGI139916[] = {{&S34385[34], 18}, {NULL}};
const Segment SGI139917[] = {{&S34385[27], 6}, {NULL}};
const Segment SGI139918[] = {{&S25237[42], 10}, {NULL}};
const Segment SGI139919[] = {{&S24293[105], 9}, {NULL}};
const Segment SGI139920[] = {{&S25237[30], 6}, {NULL}};
const Segment SGI139921[] = {{&S24293[78], 6}, {NULL}};
const Segment SGI139922[] = {{&S34385[105], 9}, {NULL}};
const Segment SGI139923[] = {{&S34385[78], 6}, {NULL}};
const Segment SGI139924[] = {{&S25237[113], 3}, {NULL}};
const Segment SGI139925[] = {{&S25237[78], 6}, {NULL}};
const Segment SGI139926[] = {{&S34386[34], 12}, {NULL}};
const Segment SGI139927[] = {{&S34386[27], 6}, {NULL}};
const Segment SGI139928[] = {{&S25238[44], 10}, {NULL}};
const Segment SGI139929[] = {{&S25238[44], 22}, {NULL}};
const Segment SGI139930[] = {{&S25238[30], 6}, {NULL}};
const Segment SGI139931[] = {{&S34386[99], 9}, {NULL}};
const Segment SGI139932[] = {{&S34386[72], 6}, {NULL}};
const Segment SGI139933[] = {{&S25238[115], 3}, {NULL}};
const Segment SGI139934[] = {{&S25238[80], 6}, {NULL}};
const Segment SGI139935[] = {{&S24295[34], 11}, {NULL}};
const Segment SGI139936[] = {{&S24295[27], 6}, {NULL}};
const Segment SGI139937[] = {{&S24295[98], 9}, {NULL}};
const Segment SGI139938[] = {{&S24295[71], 6}, {NULL}};
const Segment SGI139939[] = {{&S34387[34], 10}, {NULL}};
const Segment SGI139940[] = {{&S34387[27], 6}, {NULL}};
const Segment SGI139941[] = {{&S25239[44], 10}, {NULL}};
const Segment SGI139942[] = {{&S25239[44], 75}, {NULL}};
const Segment SGI139943[] = {{&S25239[30], 6}, {NULL}};
const Segment SGI139944[] = {{&S34387[97], 9}, {NULL}};
const Segment SGI139945[] = {{&S34387[70], 6}, {NULL}};
const Segment SGI139946[] = {{&S24296[34], 11}, {NULL}};
const Segment SGI139947[] = {{&S24296[27], 6}, {NULL}};
const Segment SGI139948[] = {{&S24296[98], 9}, {NULL}};
const Segment SGI139949[] = {{&S24296[71], 6}, {NULL}};
const Segment SGI139950[] = {{&S25240[44], 10}, {NULL}};
const Segment SGI139951[] = {{&S25240[44], 22}, {NULL}};
const Segment SGI139952[] = {{&S34388[34], 9}, {NULL}};
const Segment SGI139953[] = {{&S25240[30], 6}, {NULL}};
const Segment SGI139954[] = {{&S34388[27], 6}, {NULL}};
const Segment SGI139955[] = {{&S25240[115], 3}, {NULL}};
const Segment SGI139956[] = {{&S25240[80], 6}, {NULL}};
const Segment SGI139957[] = {{&S34388[96], 9}, {NULL}};
const Segment SGI139958[] = {{&S34388[69], 6}, {NULL}};
const Segment SGI139959[] = {{&S24297[34], 10}, {NULL}};
const Segment SGI139960[] = {{&S24297[27], 6}, {NULL}};
const Segment SGI139961[] = {{&S24297[97], 9}, {NULL}};
const Segment SGI139962[] = {{&S24297[70], 6}, {NULL}};
const Segment SGI139963[] = {{&S25241[44], 10}, {NULL}};
const Segment SGI139964[] = {{&S25241[44], 22}, {NULL}};
const Segment SGI139965[] = {{&S25241[30], 6}, {NULL}};
const Segment SGI139966[] = {{&S34389[34], 7}, {NULL}};
const Segment SGI139967[] = {{&S34389[27], 6}, {NULL}};
const Segment SGI139968[] = {{&S25241[80], 6}, {NULL}};
const Segment SGI139969[] = {{&S24298[34], 11}, {NULL}};
const Segment SGI139970[] = {{&S34389[94], 9}, {NULL}};
const Segment SGI139971[] = {{&S24298[27], 6}, {NULL}};
const Segment SGI139972[] = {{&S34389[67], 6}, {NULL}};
const Segment SGI139973[] = {{&S24298[98], 9}, {NULL}};
const Segment SGI139974[] = {{&S24298[71], 6}, {NULL}};
const Segment SGI139975[] = {{&S34390[34], 14}, {NULL}};
const Segment SGI139976[] = {{&S34390[27], 6}, {NULL}};
const Segment SGI139977[] = {{&S24299[34], 10}, {NULL}};
const Segment SGI139978[] = {{&S24299[27], 6}, {NULL}};
const Segment SGI139979[] = {{&S34390[101], 9}, {NULL}};
const Segment SGI139980[] = {{&S34390[74], 6}, {NULL}};
const Segment SGI139981[] = {{&S25244[45], 10}, {NULL}};
const Segment SGI139982[] = {{&S25244[45], 34}, {NULL}};
const Segment SGI139983[] = {{&S24299[97], 9}, {NULL}};
const Segment SGI139984[] = {{&S25244[31], 6}, {NULL}};
const Segment SGI139985[] = {{&S24299[70], 6}, {NULL}};
const Segment SGI139986[] = {{&S34392[34], 7}, {NULL}};
const Segment SGI139987[] = {{&S34392[27], 6}, {NULL}};
const Segment SGI139988[] = {{&S25245[44], 10}, {&S25245[175], 18}, {NULL}};
const Segment SGI139989[] = {{&S25245[44], 22}, {NULL}};
const Segment SGI139990[] = {{&S25245[30], 6}, {NULL}};
const Segment SGI139991[] = {{&S34392[94], 9}, {NULL}};
const Segment SGI139992[] = {{&S34392[67], 6}, {NULL}};
const Segment SGI139993[] = {{&S25245[115], 3}, {NULL}};
const Segment SGI139994[] = {{&S25245[80], 6}, {NULL}};
const Segment SGI139995[] = {{&S24302[34], 11}, {NULL}};
const Segment SGI139996[] = {{&S24302[27], 6}, {NULL}};
const Segment SGI139997[] = {{&S24302[98], 9}, {NULL}};
const Segment SGI139998[] = {{&S24302[71], 6}, {NULL}};
const Segment SGI139999[] = {{&S34393[34], 14}, {NULL}};
const Segment SGI140000[] = {{&S34393[27], 6}, {NULL}};
const Segment SGI140001[] = {{&S34393[101], 9}, {NULL}};
const Segment SGI140002[] = {{&S34393[74], 6}, {NULL}};
const Segment SGI140003[] = {{&S25247[44], 10}, {NULL}};
const Segment SGI140004[] = {{&S25247[44], 22}, {NULL}};
const Segment SGI140005[] = {{&S25247[30], 6}, {NULL}};
const Segment SGI140006[] = {{&S24304[34], 10}, {NULL}};
const Segment SGI140007[] = {{&S25247[8], 3}, {&S25247[129], 3}, {NULL}};
const Segment SGI140008[] = {{&S24304[27], 6}, {NULL}};
const Segment SGI140009[] = {{&S25247[80], 6}, {NULL}};
const Segment SGI140010[] = {{&S24304[97], 9}, {NULL}};
const Segment SGI140011[] = {{&S24304[70], 6}, {NULL}};
const Segment SGI140012[] = {{&S34395[34], 17}, {NULL}};
const Segment SGI140013[] = {{&S34395[27], 6}, {NULL}};
const Segment SGI140014[] = {{&S34395[104], 9}, {NULL}};
const Segment SGI140015[] = {{&S34395[77], 6}, {NULL}};
const Segment SGI140016[] = {{&S24305[34], 11}, {NULL}};
const Segment SGI140017[] = {{&S24305[27], 6}, {NULL}};
const Segment SGI140018[] = {{&S24305[98], 9}, {NULL}};
const Segment SGI140019[] = {{&S25249[44], 10}, {NULL}};
const Segment SGI140020[] = {{&S24305[71], 6}, {NULL}};
const Segment SGI140021[] = {{&S25249[44], 22}, {NULL}};
const Segment SGI140022[] = {{&S25249[30], 6}, {NULL}};
const Segment SGI140023[] = {{&S34396[34], 10}, {NULL}};
const Segment SGI140024[] = {{&S34396[27], 6}, {NULL}};
const Segment SGI140025[] = {{&S25249[127], 3}, {NULL}};
const Segment SGI140026[] = {{&S25249[80], 6}, {NULL}};
const Segment SGI140027[] = {{&S34396[97], 9}, {NULL}};
const Segment SGI140028[] = {{&S34396[70], 6}, {NULL}};
const Segment SGI140029[] = {{&S24306[34], 11}, {NULL}};
const Segment SGI140030[] = {{&S24306[27], 6}, {NULL}};
const Segment SGI140031[] = {{&S24306[98], 9}, {NULL}};
const Segment SGI140032[] = {{&S25250[65], 10}, {NULL}};
const Segment SGI140033[] = {{&S24306[71], 6}, {NULL}};
const Segment SGI140034[] = {{&S25250[65], 22}, {NULL}};
const Segment SGI140035[] = {{&S25250[51], 6}, {NULL}};
const Segment SGI140036[] = {{&S34397[34], 10}, {NULL}};
const Segment SGI140037[] = {{&S34397[27], 6}, {NULL}};
const Segment SGI140038[] = {{&S34397[97], 9}, {NULL}};
const Segment SGI140039[] = {{&S34397[70], 6}, {NULL}};
const Segment SGI140040[] = {{&S24307[34], 10}, {NULL}};
const Segment SGI140041[] = {{&S24307[27], 6}, {NULL}};
const Segment SGI140042[] = {{&S25251[51], 18}, {NULL}};
const Segment SGI140043[] = {{&S25251[44], 6}, {NULL}};
const Segment SGI140044[] = {{&S24307[97], 9}, {NULL}};
const Segment SGI140045[] = {{&S24307[70], 6}, {NULL}};
const Segment SGI140046[] = {{&S34398[34], 11}, {NULL}};
const Segment SGI140047[] = {{&S34398[27], 6}, {NULL}};
const Segment SGI140048[] = {{&S34398[98], 9}, {NULL}};
const Segment SGI140049[] = {{&S34398[71], 6}, {NULL}};
const Segment SGI140050[] = {{&S25252[44], 10}, {NULL}};
const Segment SGI140051[] = {{&S25252[44], 32}, {NULL}};
const Segment SGI140052[] = {{&S25252[30], 6}, {NULL}};
const Segment SGI140053[] = {{&S25252[125], 3}, {NULL}};
const Segment SGI140054[] = {{&S25252[90], 6}, {NULL}};
const Segment SGI140055[] = {{&S34399[34], 7}, {NULL}};
const Segment SGI140056[] = {{&S34399[27], 6}, {NULL}};
const Segment SGI140057[] = {{&S34399[94], 9}, {NULL}};
const Segment SGI140058[] = {{&S34399[67], 6}, {NULL}};
const Segment SGI140059[] = {{&S25253[44], 10}, {NULL}};
const Segment SGI140060[] = {{&S25253[44], 22}, {NULL}};
const Segment SGI140061[] = {{&S25253[30], 6}, {NULL}};
const Segment SGI140062[] = {{&S25253[83], 6}, {NULL}};
const Segment SGI140063[] = {{&S34401[34], 8}, {NULL}};
const Segment SGI140064[] = {{&S34401[27], 6}, {NULL}};
const Segment SGI140065[] = {{&S34401[95], 9}, {NULL}};
const Segment SGI140066[] = {{&S34401[68], 6}, {NULL}};
const Segment SGI140067[] = {{&S25255[30], 1}, {NULL}};
const Segment SGI140068[] = {{&S25255[63], 9}, {NULL}};
const Segment SGI140069[] = {{&S25255[49], 6}, {NULL}};
const Segment SGI140070[] = {{&S24315[34], 10}, {NULL}};
const Segment SGI140071[] = {{&S34402[34], 15}, {NULL}};
const Segment SGI140072[] = {{&S24315[27], 6}, {NULL}};
const Segment SGI140073[] = {{&S34402[27], 6}, {NULL}};
const Segment SGI140074[] = {{&S24315[97], 9}, {NULL}};
const Segment SGI140075[] = {{&S34402[102], 9}, {NULL}};
const Segment SGI140076[] = {{&S24315[70], 6}, {NULL}};
const Segment SGI140077[] = {{&S34402[75], 6}, {NULL}};
const Segment SGI140078[] = {{&S25256[44], 10}, {NULL}};
const Segment SGI140079[] = {{&S25256[44], 22}, {NULL}};
const Segment SGI140080[] = {{&S25256[30], 6}, {NULL}};
const Segment SGI140081[] = {{&S25256[116], 3}, {NULL}};
const Segment SGI140082[] = {{&S25256[80], 7}, {NULL}};
const Segment SGI140083[] = {{&S25256[136], 8}, {NULL}};
const Segment SGI140084[] = {{&S24316[34], 10}, {NULL}};
const Segment SGI140085[] = {{&S34403[34], 11}, {NULL}};
const Segment SGI140086[] = {{&S24316[27], 6}, {NULL}};
const Segment SGI140087[] = {{&S34403[27], 6}, {NULL}};
const Segment SGI140088[] = {{&S25257[34], 10}, {NULL}};
const Segment SGI140089[] = {{&S25257[34], 22}, {NULL}};
const Segment SGI140090[] = {{&S24316[97], 9}, {NULL}};
const Segment SGI140091[] = {{&S34403[98], 9}, {NULL}};
const Segment SGI140092[] = {{&S24316[70], 6}, {NULL}};
const Segment SGI140093[] = {{&S34403[71], 6}, {NULL}};
const Segment SGI140094[] = {{&S34404[34], 7}, {NULL}};
const Segment SGI140095[] = {{&S34404[27], 6}, {NULL}};
const Segment SGI140096[] = {{&S25258[44], 10}, {NULL}};
const Segment SGI140097[] = {{&S25258[44], 22}, {NULL}};
const Segment SGI140098[] = {{&S34404[94], 9}, {NULL}};
const Segment SGI140099[] = {{&S34404[67], 6}, {NULL}};
const Segment SGI140100[] = {{&S25258[30], 6}, {NULL}};
const Segment SGI140101[] = {{&S25258[115], 3}, {&S25258[151], 3}, {NULL}};
const Segment SGI140102[] = {{&S25258[80], 6}, {NULL}};
const Segment SGI140103[] = {{&S34405[34], 6}, {NULL}};
const Segment SGI140104[] = {{&S34405[27], 6}, {NULL}};
const Segment SGI140105[] = {{&S34405[93], 9}, {NULL}};
const Segment SGI140106[] = {{&S25259[37], 6}, {NULL}};
const Segment SGI140107[] = {{&S34405[66], 6}, {NULL}};
const Segment SGI140108[] = {{&S34406[34], 5}, {NULL}};
const Segment SGI140109[] = {{&S34406[27], 6}, {NULL}};
const Segment SGI140110[] = {{&S25260[44], 9}, {NULL}};
const Segment SGI140111[] = {{&S25260[30], 6}, {NULL}};
const Segment SGI140112[] = {{&S34406[92], 9}, {NULL}};
const Segment SGI140113[] = {{&S34406[65], 6}, {NULL}};
const Segment SGI140114[] = {{&S25261[44], 10}, {NULL}};
const Segment SGI140115[] = {{&S25261[44], 22}, {NULL}};
const Segment SGI140116[] = {{&S25261[30], 6}, {NULL}};
const Segment SGI140117[] = {{&S34407[34], 15}, {NULL}};
const Segment SGI140118[] = {{&S34407[27], 6}, {NULL}};
const Segment SGI140119[] = {{&S25261[114], 3}, {NULL}};
const Segment SGI140120[] = {{&S25261[80], 5}, {NULL}};
const Segment SGI140121[] = {{&S34407[97], 9}, {NULL}};
const Segment SGI140122[] = {{&S34407[70], 6}, {NULL}};
const Segment SGI140123[] = {{&S25262[42], 18}, {NULL}};
const Segment SGI140124[] = {{&S34408[34], 5}, {NULL}};
const Segment SGI140125[] = {{&S34408[27], 6}, {NULL}};
const Segment SGI140126[] = {{&S34408[92], 9}, {NULL}};
const Segment SGI140127[] = {{&S34408[65], 6}, {NULL}};
const Segment SGI140128[] = {{&S25263[102], 29}, {NULL}};
const Segment SGI140129[] = {{&S25266[44], 10}, {NULL}};
const Segment SGI140130[] = {{&S25266[44], 22}, {NULL}};
const Segment SGI140131[] = {{&S25266[30], 6}, {NULL}};
const Segment SGI140132[] = {{&S25266[114], 3}, {NULL}};
const Segment SGI140133[] = {{&S25266[80], 5}, {NULL}};
const Segment SGI140134[] = {{&S25267[44], 22}, {NULL}};
const Segment SGI140135[] = {{&S25267[30], 6}, {NULL}};
const Segment SGI140136[] = {{&S25267[115], 3}, {NULL}};
const Segment SGI140137[] = {{&S25267[80], 6}, {NULL}};
const Segment SGI140138[] = {{&S25267[135], 8}, {NULL}};
const Segment SGI140139[] = {{&S25269[44], 10}, {NULL}};
const Segment SGI140140[] = {{&S25269[44], 22}, {NULL}};
const Segment SGI140141[] = {{&S25269[30], 6}, {NULL}};
const Segment SGI140142[] = {{&S25269[115], 3}, {NULL}};
const Segment SGI140143[] = {{&S25269[80], 6}, {NULL}};
const Segment SGI140144[] = {{&S25269[135], 8}, {NULL}};
const Segment SGI140145[] = {{&S25270[38], 10}, {NULL}};
const Segment SGI140146[] = {{&S25270[111], 6}, {NULL}};
const Segment SGI140147[] = {{&S25270[31], 6}, {NULL}};
const Segment SGI140148[] = {{&S25271[44], 9}, {NULL}};
const Segment SGI140149[] = {{&S25271[30], 6}, {NULL}};
const Segment SGI140150[] = {{&S25271[101], 3}, {NULL}};
const Segment SGI140151[] = {{&S25271[67], 5}, {NULL}};
const Segment SGI140152[] = {{&S25274[42], 10}, {NULL}};
const Segment SGI140153[] = {{&S25274[30], 6}, {NULL}};
const Segment SGI140154[] = {{&S25274[113], 3}, {NULL}};
const Segment SGI140155[] = {{&S25274[78], 6}, {NULL}};
const Segment SGI140156[] = {{&S25302[44], 10}, {NULL}};
const Segment SGI140157[] = {{&S25302[44], 22}, {NULL}};
const Segment SGI140158[] = {{&S25302[30], 6}, {NULL}};
const Segment SGI140159[] = {{&S25302[115], 3}, {NULL}};
const Segment SGI140160[] = {{&S25302[80], 6}, {NULL}};
const Segment SGI140161[] = {{&S25303[44], 10}, {NULL}};
const Segment SGI140162[] = {{&S25303[44], 22}, {NULL}};
const Segment SGI140163[] = {{&S25303[30], 6}, {NULL}};
const Segment SGI140164[] = {{&S25303[115], 3}, {NULL}};
const Segment SGI140165[] = {{&S25303[80], 6}, {NULL}};
const Segment SGI140166[] = {{&S25305[38], 10}, {NULL}};
const Segment SGI140167[] = {{&S25305[31], 6}, {NULL}};
const Segment SGI140168[] = {{&S25306[42], 10}, {NULL}};
const Segment SGI140169[] = {{&S25306[30], 6}, {NULL}};
const Segment SGI140170[] = {{&S25306[113], 3}, {NULL}};
const Segment SGI140171[] = {{&S25306[78], 6}, {NULL}};
const Segment SGI140172[] = {{&S25307[44], 10}, {NULL}};
const Segment SGI140173[] = {{&S25307[44], 22}, {NULL}};
const Segment SGI140174[] = {{&S25307[30], 6}, {NULL}};
const Segment SGI140175[] = {{&S25307[115], 3}, {NULL}};
const Segment SGI140176[] = {{&S25307[80], 6}, {NULL}};
const Segment SGI140177[] = {{&S25308[44], 10}, {NULL}};
const Segment SGI140178[] = {{&S25308[44], 22}, {NULL}};
const Segment SGI140179[] = {{&S25308[30], 6}, {NULL}};
const Segment SGI140180[] = {{&S25308[115], 3}, {NULL}};
const Segment SGI140181[] = {{&S25308[80], 6}, {NULL}};
const Segment SGI140182[] = {{&S25309[44], 10}, {NULL}};
const Segment SGI140183[] = {{&S25309[44], 22}, {NULL}};
const Segment SGI140184[] = {{&S25309[30], 6}, {NULL}};
const Segment SGI140185[] = {{&S25309[115], 3}, {NULL}};
const Segment SGI140186[] = {{&S25309[80], 6}, {NULL}};
const Segment SGI140187[] = {{&S25310[58], 10}, {NULL}};
const Segment SGI140188[] = {{&S25310[58], 66}, {NULL}};
const Segment SGI140189[] = {{&S25310[44], 6}, {NULL}};
const Segment SGI140190[] = {{&S25313[44], 10}, {NULL}};
const Segment SGI140191[] = {{&S25313[44], 22}, {NULL}};
const Segment SGI140192[] = {{&S25313[30], 6}, {NULL}};
const Segment SGI140193[] = {{&S25313[115], 3}, {NULL}};
const Segment SGI140194[] = {{&S25313[80], 6}, {NULL}};
const Segment SGI140195[] = {{&S25314[46], 10}, {NULL}};
const Segment SGI140196[] = {{&S25314[46], 34}, {NULL}};
const Segment SGI140197[] = {{&S25314[32], 6}, {NULL}};
const Segment SGI140198[] = {{&S25315[105], 29}, {NULL}};
const Segment SGI140199[] = {{&S25316[30], 1}, {NULL}};
const Segment SGI140200[] = {{&S25316[63], 9}, {NULL}};
const Segment SGI140201[] = {{&S25316[49], 6}, {NULL}};
const Segment SGI140202[] = {{&S25318[37], 6}, {NULL}};
const Segment SGI140203[] = {{&S25319[34], 10}, {NULL}};
const Segment SGI140204[] = {{&S25319[34], 22}, {NULL}};
const Segment SGI140205[] = {{&S25320[44], 9}, {NULL}};
const Segment SGI140206[] = {{&S25320[30], 6}, {NULL}};
const Segment SGI140207[] = {{&S25322[44], 10}, {NULL}};
const Segment SGI140208[] = {{&S25322[44], 19}, {NULL}};
const Segment SGI140209[] = {{&S25322[30], 6}, {NULL}};
const Segment SGI140210[] = {{&S25322[111], 3}, {NULL}};
const Segment SGI140211[] = {{&S25322[77], 5}, {NULL}};
const Segment SGI140212[] = {{&S25323[44], 10}, {NULL}};
const Segment SGI140213[] = {{&S25323[44], 22}, {NULL}};
const Segment SGI140214[] = {{&S25323[30], 6}, {NULL}};
const Segment SGI140215[] = {{&S25323[114], 3}, {NULL}};
const Segment SGI140216[] = {{&S25323[80], 5}, {NULL}};
const Segment SGI140217[] = {{&S25324[44], 10}, {NULL}};
const Segment SGI140218[] = {{&S25324[44], 22}, {NULL}};
const Segment SGI140219[] = {{&S25324[30], 6}, {NULL}};
const Segment SGI140220[] = {{&S25324[116], 3}, {NULL}};
const Segment SGI140221[] = {{&S25324[80], 7}, {NULL}};
const Segment SGI140222[] = {{&S25324[136], 8}, {NULL}};
const Segment SGI140223[] = {{&S25325[44], 10}, {NULL}};
const Segment SGI140224[] = {{&S25325[44], 22}, {NULL}};
const Segment SGI140225[] = {{&S25325[30], 6}, {NULL}};
const Segment SGI140226[] = {{&S25325[8], 3}, {&S25325[129], 3}, {NULL}};
const Segment SGI140227[] = {{&S25325[80], 6}, {NULL}};
const Segment SGI140228[] = {{&S25326[65], 10}, {NULL}};
const Segment SGI140229[] = {{&S25326[65], 22}, {NULL}};
const Segment SGI140230[] = {{&S25326[51], 6}, {NULL}};
const Segment SGI140231[] = {{&S25327[55], 18}, {NULL}};
const Segment SGI140232[] = {{&S25327[48], 6}, {NULL}};
const Segment SGI140233[] = {{&S25329[21], 24}, {NULL}};
const Segment SGI140234[] = {{&S25330[71], 6}, {NULL}};
const Segment SGI140235[] = {{&S25330[30], 6}, {NULL}};
const Segment SGI140236[] = {{&S25330[127], 3}, {NULL}};
const Segment SGI140237[] = {{&S25330[91], 7}, {NULL}};
const Segment SGI140238[] = {{&S25330[147], 8}, {NULL}};
const Segment SGI140239[] = {{&S25331[44], 10}, {NULL}};
const Segment SGI140240[] = {{&S25331[44], 22}, {NULL}};
const Segment SGI140241[] = {{&S25331[30], 6}, {NULL}};
const Segment SGI140242[] = {{&S25331[83], 6}, {NULL}};
const Segment SGI140243[] = {{&S25332[44], 10}, {NULL}};
const Segment SGI140244[] = {{&S25332[44], 22}, {NULL}};
const Segment SGI140245[] = {{&S25332[30], 6}, {NULL}};
const Segment SGI140246[] = {{&S25332[115], 3}, {&S25332[151], 3}, {NULL}};
const Segment SGI140247[] = {{&S25332[80], 6}, {NULL}};
const Segment SGI140248[] = {{&S25333[44], 10}, {NULL}};
const Segment SGI140249[] = {{&S25333[44], 22}, {NULL}};
const Segment SGI140250[] = {{&S25333[30], 6}, {NULL}};
const Segment SGI140251[] = {{&S25333[80], 6}, {NULL}};
const Segment SGI140252[] = {{&S25335[44], 10}, {NULL}};
const Segment SGI140253[] = {{&S25335[44], 22}, {NULL}};
const Segment SGI140254[] = {{&S25335[30], 6}, {NULL}};
const Segment SGI140255[] = {{&S25335[114], 3}, {NULL}};
const Segment SGI140256[] = {{&S25335[80], 5}, {NULL}};
const Segment SGI140257[] = {{&S25338[44], 10}, {NULL}};
const Segment SGI140258[] = {{&S25338[44], 22}, {&S25338[205], 7}, {NULL}};
const Segment SGI140259[] = {{&S25338[30], 6}, {NULL}};
const Segment SGI140260[] = {{&S25338[115], 3}, {NULL}};
const Segment SGI140261[] = {{&S25338[80], 6}, {NULL}};
const Segment SGI140262[] = {{&S25340[44], 10}, {&S25340[176], 18}, {NULL}};
const Segment SGI140263[] = {{&S25340[44], 22}, {NULL}};
const Segment SGI140264[] = {{&S25340[30], 6}, {NULL}};
const Segment SGI140265[] = {{&S25340[115], 3}, {NULL}};
const Segment SGI140266[] = {{&S25340[80], 6}, {NULL}};
const Segment SGI140267[] = {{&S25342[44], 10}, {NULL}};
const Segment SGI140268[] = {{&S25342[44], 32}, {NULL}};
const Segment SGI140269[] = {{&S25342[30], 6}, {NULL}};
const Segment SGI140270[] = {{&S25342[125], 3}, {NULL}};
const Segment SGI140271[] = {{&S25342[90], 6}, {NULL}};
const Segment SGI140272[] = {{&S25343[44], 22}, {NULL}};
const Segment SGI140273[] = {{&S25343[30], 6}, {NULL}};
const Segment SGI140274[] = {{&S25343[115], 3}, {NULL}};
const Segment SGI140275[] = {{&S25343[80], 6}, {NULL}};
const Segment SGI140276[] = {{&S25343[135], 8}, {NULL}};
const Segment SGI140277[] = {{&S25344[44], 10}, {NULL}};
const Segment SGI140278[] = {{&S25344[44], 22}, {NULL}};
const Segment SGI140279[] = {{&S25344[30], 6}, {NULL}};
const Segment SGI140280[] = {{&S25344[115], 3}, {NULL}};
const Segment SGI140281[] = {{&S25344[80], 6}, {NULL}};
const Segment SGI140282[] = {{&S25344[135], 8}, {NULL}};
const Segment SGI140283[] = {{&S34556[44], 24}, {NULL}};
const Segment SGI140284[] = {{&S34556[44], 36}, {NULL}};
const Segment SGI140285[] = {{&S34556[30], 6}, {NULL}};
const Segment SGI140286[] = {{&S34556[31], 3}, {NULL}};
const Segment SGI140287[] = {{&S34556[94], 6}, {NULL}};
const Segment SGI140288[] = {{&S24620[34], 10}, {NULL}};
const Segment SGI140289[] = {{&S24620[27], 6}, {NULL}};
const Segment SGI140290[] = {{&S24620[97], 9}, {NULL}};
const Segment SGI140291[] = {{&S24620[70], 6}, {NULL}};
const Segment SGI140292[] = {{&S24621[34], 11}, {NULL}};
const Segment SGI140293[] = {{&S24621[27], 6}, {NULL}};
const Segment SGI140294[] = {{&S24621[98], 9}, {NULL}};
const Segment SGI140295[] = {{&S24621[71], 6}, {NULL}};
const Segment SGI140296[] = {{&S24622[34], 10}, {NULL}};
const Segment SGI140297[] = {{&S24622[27], 6}, {NULL}};
const Segment SGI140298[] = {{&S24622[97], 9}, {NULL}};
const Segment SGI140299[] = {{&S24622[70], 6}, {NULL}};
const Segment SGI140300[] = {{&S24624[34], 18}, {NULL}};
const Segment SGI140301[] = {{&S24624[27], 6}, {NULL}};
const Segment SGI140302[] = {{&S24624[105], 9}, {NULL}};
const Segment SGI140303[] = {{&S24624[78], 6}, {NULL}};
const Segment SGI140304[] = {{&S24626[34], 10}, {NULL}};
const Segment SGI140305[] = {{&S24626[27], 6}, {NULL}};
const Segment SGI140306[] = {{&S24626[97], 9}, {NULL}};
const Segment SGI140307[] = {{&S24626[70], 6}, {NULL}};
const Segment SGI140308[] = {{&S24631[34], 10}, {NULL}};
const Segment SGI140309[] = {{&S24631[27], 6}, {NULL}};
const Segment SGI140310[] = {{&S24631[97], 9}, {NULL}};
const Segment SGI140311[] = {{&S24631[70], 6}, {NULL}};
const Segment SGI140312[] = {{&S24633[34], 11}, {NULL}};
const Segment SGI140313[] = {{&S24633[27], 6}, {NULL}};
const Segment SGI140314[] = {{&S24633[98], 9}, {NULL}};
const Segment SGI140315[] = {{&S24633[71], 6}, {NULL}};
const Segment SGI140316[] = {{&S24635[34], 10}, {NULL}};
const Segment SGI140317[] = {{&S24635[27], 6}, {NULL}};
const Segment SGI140318[] = {{&S24635[97], 9}, {NULL}};
const Segment SGI140319[] = {{&S24635[70], 6}, {NULL}};
const Segment SGI140320[] = {{&S24637[34], 18}, {NULL}};
const Segment SGI140321[] = {{&S24637[27], 6}, {NULL}};
const Segment SGI140322[] = {{&S24637[105], 9}, {NULL}};
const Segment SGI140323[] = {{&S24637[78], 6}, {NULL}};
const Segment SGI140324[] = {{&S24638[34], 11}, {NULL}};
const Segment SGI140325[] = {{&S24638[27], 6}, {NULL}};
const Segment SGI140326[] = {{&S24638[98], 9}, {NULL}};
const Segment SGI140327[] = {{&S24638[71], 6}, {NULL}};
const Segment SGI140328[] = {{&S24640[34], 11}, {NULL}};
const Segment SGI140329[] = {{&S24640[27], 6}, {NULL}};
const Segment SGI140330[] = {{&S24640[98], 9}, {NULL}};
const Segment SGI140331[] = {{&S24640[71], 6}, {NULL}};
const Segment SGI140332[] = {{&S24642[34], 10}, {NULL}};
const Segment SGI140333[] = {{&S24642[27], 6}, {NULL}};
const Segment SGI140334[] = {{&S24642[97], 9}, {NULL}};
const Segment SGI140335[] = {{&S24642[70], 6}, {NULL}};
const Segment SGI140336[] = {{&S24643[34], 10}, {NULL}};
const Segment SGI140337[] = {{&S24643[27], 6}, {NULL}};
const Segment SGI140338[] = {{&S24643[97], 9}, {NULL}};
const Segment SGI140339[] = {{&S24643[70], 6}, {NULL}};
const Segment SGI140340[] = {{&S24644[34], 10}, {NULL}};
const Segment SGI140341[] = {{&S24644[27], 6}, {NULL}};
const Segment SGI140342[] = {{&S24644[97], 9}, {NULL}};
const Segment SGI140343[] = {{&S24644[70], 6}, {NULL}};
const Segment SGI140344[] = {{&S25570[34], 10}, {NULL}};
const Segment SGI140345[] = {{&S24645[34], 11}, {NULL}};
const Segment SGI140346[] = {{&S25570[27], 6}, {NULL}};
const Segment SGI140347[] = {{&S24645[27], 6}, {NULL}};
const Segment SGI140348[] = {{&S25570[97], 9}, {NULL}};
const Segment SGI140349[] = {{&S24645[98], 9}, {NULL}};
const Segment SGI140350[] = {{&S25570[70], 6}, {NULL}};
const Segment SGI140351[] = {{&S24645[71], 6}, {NULL}};
const Segment SGI140352[] = {{&S24647[34], 10}, {NULL}};
const Segment SGI140353[] = {{&S24647[27], 6}, {NULL}};
const Segment SGI140354[] = {{&S24647[97], 9}, {NULL}};
const Segment SGI140355[] = {{&S24647[70], 6}, {NULL}};
const Segment SGI140356[] = {{&S25574[34], 10}, {NULL}};
const Segment SGI140357[] = {{&S24648[34], 11}, {NULL}};
const Segment SGI140358[] = {{&S25574[27], 6}, {NULL}};
const Segment SGI140359[] = {{&S24648[27], 6}, {NULL}};
const Segment SGI140360[] = {{&S25574[97], 9}, {NULL}};
const Segment SGI140361[] = {{&S24648[98], 9}, {NULL}};
const Segment SGI140362[] = {{&S25574[70], 6}, {NULL}};
const Segment SGI140363[] = {{&S24648[71], 6}, {NULL}};
const Segment SGI140364[] = {{&S24649[34], 10}, {NULL}};
const Segment SGI140365[] = {{&S24649[27], 6}, {NULL}};
const Segment SGI140366[] = {{&S24649[97], 9}, {NULL}};
const Segment SGI140367[] = {{&S24649[70], 6}, {NULL}};
const Segment SGI140368[] = {{&S24650[34], 11}, {NULL}};
const Segment SGI140369[] = {{&S24650[27], 6}, {NULL}};
const Segment SGI140370[] = {{&S24650[98], 9}, {NULL}};
const Segment SGI140371[] = {{&S24650[71], 6}, {NULL}};
const Segment SGI140372[] = {{&S25579[34], 10}, {NULL}};
const Segment SGI140373[] = {{&S25579[27], 6}, {NULL}};
const Segment SGI140374[] = {{&S25579[97], 9}, {NULL}};
const Segment SGI140375[] = {{&S25579[70], 6}, {NULL}};
const Segment SGI140376[] = {{&S24652[34], 11}, {NULL}};
const Segment SGI140377[] = {{&S24652[27], 6}, {NULL}};
const Segment SGI140378[] = {{&S24652[98], 9}, {NULL}};
const Segment SGI140379[] = {{&S24652[71], 6}, {NULL}};
const Segment SGI140380[] = {{&S25580[34], 11}, {NULL}};
const Segment SGI140381[] = {{&S25580[27], 6}, {NULL}};
const Segment SGI140382[] = {{&S25580[98], 9}, {NULL}};
const Segment SGI140383[] = {{&S25580[71], 6}, {NULL}};
const Segment SGI140384[] = {{&S24653[34], 10}, {NULL}};
const Segment SGI140385[] = {{&S24653[27], 6}, {NULL}};
const Segment SGI140386[] = {{&S24653[97], 9}, {NULL}};
const Segment SGI140387[] = {{&S24653[70], 6}, {NULL}};
const Segment SGI140388[] = {{&S24654[34], 11}, {NULL}};
const Segment SGI140389[] = {{&S24654[27], 6}, {NULL}};
const Segment SGI140390[] = {{&S25583[34], 10}, {NULL}};
const Segment SGI140391[] = {{&S25583[27], 6}, {NULL}};
const Segment SGI140392[] = {{&S24654[98], 9}, {NULL}};
const Segment SGI140393[] = {{&S24654[71], 6}, {NULL}};
const Segment SGI140394[] = {{&S25583[97], 9}, {NULL}};
const Segment SGI140395[] = {{&S25583[70], 6}, {NULL}};
const Segment SGI140396[] = {{&S25584[34], 11}, {NULL}};
const Segment SGI140397[] = {{&S25584[27], 6}, {NULL}};
const Segment SGI140398[] = {{&S25584[98], 9}, {NULL}};
const Segment SGI140399[] = {{&S25584[71], 6}, {NULL}};
const Segment SGI140400[] = {{&S25587[34], 7}, {NULL}};
const Segment SGI140401[] = {{&S24659[34], 11}, {NULL}};
const Segment SGI140402[] = {{&S25587[27], 6}, {NULL}};
const Segment SGI140403[] = {{&S24659[27], 6}, {NULL}};
const Segment SGI140404[] = {{&S25587[94], 9}, {NULL}};
const Segment SGI140405[] = {{&S24659[98], 9}, {NULL}};
const Segment SGI140406[] = {{&S25587[67], 6}, {NULL}};
const Segment SGI140407[] = {{&S24659[71], 6}, {NULL}};
const Segment SGI140408[] = {{&S25588[34], 11}, {NULL}};
const Segment SGI140409[] = {{&S25588[27], 6}, {NULL}};
const Segment SGI140410[] = {{&S25588[98], 9}, {NULL}};
const Segment SGI140411[] = {{&S25588[71], 6}, {NULL}};
const Segment SGI140412[] = {{&S24704[34], 10}, {NULL}};
const Segment SGI140413[] = {{&S24704[27], 6}, {NULL}};
const Segment SGI140414[] = {{&S24704[97], 9}, {NULL}};
const Segment SGI140415[] = {{&S24704[70], 6}, {NULL}};
const Segment SGI140416[] = {{&S24705[34], 11}, {NULL}};
const Segment SGI140417[] = {{&S24705[27], 6}, {NULL}};
const Segment SGI140418[] = {{&S24705[98], 9}, {NULL}};
const Segment SGI140419[] = {{&S24705[71], 6}, {NULL}};
const Segment SGI140420[] = {{&S24709[34], 10}, {NULL}};
const Segment SGI140421[] = {{&S24709[27], 6}, {NULL}};
const Segment SGI140422[] = {{&S24709[97], 9}, {NULL}};
const Segment SGI140423[] = {{&S24709[70], 6}, {NULL}};
const Segment SGI140424[] = {{&S24714[34], 10}, {NULL}};
const Segment SGI140425[] = {{&S24714[27], 6}, {NULL}};
const Segment SGI140426[] = {{&S24714[97], 9}, {NULL}};
const Segment SGI140427[] = {{&S24714[70], 6}, {NULL}};
const Segment SGI140428[] = {{&S24715[34], 10}, {NULL}};
const Segment SGI140429[] = {{&S24715[27], 6}, {NULL}};
const Segment SGI140430[] = {{&S24715[97], 9}, {NULL}};
const Segment SGI140431[] = {{&S24715[70], 6}, {NULL}};
const Segment SGI140432[] = {{&S24717[34], 10}, {NULL}};
const Segment SGI140433[] = {{&S24717[27], 6}, {NULL}};
const Segment SGI140434[] = {{&S24717[97], 9}, {NULL}};
const Segment SGI140435[] = {{&S24717[70], 6}, {NULL}};
const Segment SGI140436[] = {{&S24719[34], 11}, {NULL}};
const Segment SGI140437[] = {{&S24719[27], 6}, {NULL}};
const Segment SGI140438[] = {{&S24719[98], 9}, {NULL}};
const Segment SGI140439[] = {{&S24719[71], 6}, {NULL}};
const Segment SGI140440[] = {{&S24725[34], 10}, {NULL}};
const Segment SGI140441[] = {{&S24725[27], 6}, {NULL}};
const Segment SGI140442[] = {{&S24725[97], 9}, {NULL}};
const Segment SGI140443[] = {{&S24725[70], 6}, {NULL}};
const Segment SGI140444[] = {{&S24727[34], 8}, {NULL}};
const Segment SGI140445[] = {{&S24727[27], 6}, {NULL}};
const Segment SGI140446[] = {{&S24727[95], 9}, {NULL}};
const Segment SGI140447[] = {{&S24727[68], 6}, {NULL}};
const Segment SGI140448[] = {{&S24728[34], 11}, {NULL}};
const Segment SGI140449[] = {{&S24728[27], 6}, {NULL}};
const Segment SGI140450[] = {{&S24728[98], 9}, {NULL}};
const Segment SGI140451[] = {{&S24728[71], 6}, {NULL}};
const Segment SGI140452[] = {{&S25671[34], 10}, {NULL}};
const Segment SGI140453[] = {{&S25671[27], 6}, {NULL}};
const Segment SGI140454[] = {{&S25671[97], 9}, {NULL}};
const Segment SGI140455[] = {{&S25671[70], 6}, {NULL}};
const Segment SGI140456[] = {{&S25672[34], 11}, {NULL}};
const Segment SGI140457[] = {{&S25672[27], 6}, {NULL}};
const Segment SGI140458[] = {{&S25672[98], 9}, {NULL}};
const Segment SGI140459[] = {{&S25672[71], 6}, {NULL}};
const Segment SGI140460[] = {{&S25673[34], 11}, {NULL}};
const Segment SGI140461[] = {{&S25673[27], 6}, {NULL}};
const Segment SGI140462[] = {{&S25673[98], 9}, {NULL}};
const Segment SGI140463[] = {{&S25673[71], 6}, {NULL}};
const Segment SGI140464[] = {{&S25675[34], 18}, {NULL}};
const Segment SGI140465[] = {{&S25675[27], 6}, {NULL}};
const Segment SGI140466[] = {{&S25675[105], 9}, {NULL}};
const Segment SGI140467[] = {{&S25675[78], 6}, {NULL}};
const Segment SGI140468[] = {{&S25678[34], 18}, {NULL}};
const Segment SGI140469[] = {{&S25678[27], 6}, {NULL}};
const Segment SGI140470[] = {{&S25678[105], 9}, {NULL}};
const Segment SGI140471[] = {{&S25678[78], 6}, {NULL}};
const Segment SGI140472[] = {{&S25679[34], 10}, {NULL}};
const Segment SGI140473[] = {{&S25679[27], 6}, {NULL}};
const Segment SGI140474[] = {{&S25679[97], 9}, {NULL}};
const Segment SGI140475[] = {{&S25679[70], 6}, {NULL}};
const Segment SGI140476[] = {{&S25681[34], 10}, {NULL}};
const Segment SGI140477[] = {{&S25681[27], 6}, {NULL}};
const Segment SGI140478[] = {{&S25681[97], 9}, {NULL}};
const Segment SGI140479[] = {{&S25681[70], 6}, {NULL}};
const Segment SGI140480[] = {{&S25683[34], 11}, {NULL}};
const Segment SGI140481[] = {{&S25683[27], 6}, {NULL}};
const Segment SGI140482[] = {{&S25683[98], 9}, {NULL}};
const Segment SGI140483[] = {{&S25683[71], 6}, {NULL}};
const Segment SGI140484[] = {{&S25684[34], 11}, {NULL}};
const Segment SGI140485[] = {{&S25684[27], 6}, {NULL}};
const Segment SGI140486[] = {{&S25684[98], 9}, {NULL}};
const Segment SGI140487[] = {{&S25684[71], 6}, {NULL}};
const Segment SGI140488[] = {{&S25685[34], 11}, {NULL}};
const Segment SGI140489[] = {{&S25685[27], 6}, {NULL}};
const Segment SGI140490[] = {{&S25685[98], 9}, {NULL}};
const Segment SGI140491[] = {{&S25685[71], 6}, {NULL}};
const Segment SGI140492[] = {{&S25686[34], 11}, {NULL}};
const Segment SGI140493[] = {{&S25686[27], 6}, {NULL}};
const Segment SGI140494[] = {{&S25686[98], 9}, {NULL}};
const Segment SGI140495[] = {{&S25686[71], 6}, {NULL}};
const Segment SGI140496[] = {{&S25692[34], 10}, {NULL}};
const Segment SGI140497[] = {{&S25692[27], 6}, {NULL}};
const Segment SGI140498[] = {{&S25692[97], 9}, {NULL}};
const Segment SGI140499[] = {{&S25692[70], 6}, {NULL}};
const Segment SGI140500[] = {{&S25694[34], 10}, {NULL}};
const Segment SGI140501[] = {{&S25694[27], 6}, {NULL}};
const Segment SGI140502[] = {{&S25694[97], 9}, {NULL}};
const Segment SGI140503[] = {{&S25694[70], 6}, {NULL}};
const Segment SGI140504[] = {{&S25698[34], 11}, {NULL}};
const Segment SGI140505[] = {{&S25698[27], 6}, {NULL}};
const Segment SGI140506[] = {{&S25698[98], 9}, {NULL}};
const Segment SGI140507[] = {{&S25698[71], 6}, {NULL}};
const Segment SGI140508[] = {{&S34976[44], 7}, {NULL}};
const Segment SGI140509[] = {{&S34976[44], 24}, {NULL}};
const Segment SGI140510[] = {{&S34976[30], 6}, {NULL}};
const Segment SGI140511[] = {{&S34976[31], 3}, {NULL}};
const Segment SGI140512[] = {{&S34976[82], 6}, {NULL}};
const Segment SGI140513[] = {{&S34978[38], 7}, {NULL}};
const Segment SGI140514[] = {{&S34978[31], 6}, {NULL}};
const Segment SGI140515[] = {{&S34979[44], 7}, {NULL}};
const Segment SGI140516[] = {{&S34979[44], 24}, {NULL}};
const Segment SGI140517[] = {{&S34979[30], 6}, {NULL}};
const Segment SGI140518[] = {{&S34979[31], 3}, {NULL}};
const Segment SGI140519[] = {{&S34979[82], 6}, {NULL}};
const Segment SGI140520[] = {{&S34981[44], 7}, {NULL}};
const Segment SGI140521[] = {{&S34981[44], 24}, {NULL}};
const Segment SGI140522[] = {{&S34981[30], 6}, {NULL}};
const Segment SGI140523[] = {{&S34981[31], 3}, {NULL}};
const Segment SGI140524[] = {{&S34981[82], 6}, {NULL}};
const Segment SGI140525[] = {{&S34982[44], 7}, {NULL}};
const Segment SGI140526[] = {{&S34982[44], 24}, {NULL}};
const Segment SGI140527[] = {{&S34982[30], 6}, {NULL}};
const Segment SGI140528[] = {{&S34982[31], 3}, {NULL}};
const Segment SGI140529[] = {{&S34982[82], 6}, {NULL}};
const Segment SGI140530[] = {{&S34983[38], 7}, {NULL}};
const Segment SGI140531[] = {{&S34983[31], 6}, {NULL}};
const Segment SGI140532[] = {{&S25805[34], 10}, {NULL}};
const Segment SGI140533[] = {{&S25805[27], 6}, {NULL}};
const Segment SGI140534[] = {{&S25805[97], 9}, {NULL}};
const Segment SGI140535[] = {{&S25805[70], 6}, {NULL}};
const Segment SGI140536[] = {{&S25810[34], 10}, {NULL}};
const Segment SGI140537[] = {{&S25810[27], 6}, {NULL}};
const Segment SGI140538[] = {{&S25810[97], 9}, {NULL}};
const Segment SGI140539[] = {{&S25810[70], 6}, {NULL}};
const Segment SGI140540[] = {{&S25812[34], 10}, {NULL}};
const Segment SGI140541[] = {{&S25812[27], 6}, {NULL}};
const Segment SGI140542[] = {{&S25812[97], 9}, {NULL}};
const Segment SGI140543[] = {{&S25812[70], 6}, {NULL}};
const Segment SGI140544[] = {{&S25813[34], 11}, {NULL}};
const Segment SGI140545[] = {{&S25813[27], 6}, {NULL}};
const Segment SGI140546[] = {{&S25813[98], 9}, {NULL}};
const Segment SGI140547[] = {{&S25813[71], 6}, {NULL}};
const Segment SGI140548[] = {{&S25815[34], 10}, {NULL}};
const Segment SGI140549[] = {{&S25815[27], 6}, {NULL}};
const Segment SGI140550[] = {{&S25815[97], 9}, {NULL}};
const Segment SGI140551[] = {{&S25815[70], 6}, {NULL}};
const Segment SGI140552[] = {{&S25816[34], 11}, {NULL}};
const Segment SGI140553[] = {{&S25816[27], 6}, {NULL}};
const Segment SGI140554[] = {{&S25816[98], 9}, {NULL}};
const Segment SGI140555[] = {{&S25816[71], 6}, {NULL}};
const Segment SGI140556[] = {{&S25818[34], 11}, {NULL}};
const Segment SGI140557[] = {{&S25818[27], 6}, {NULL}};
const Segment SGI140558[] = {{&S25818[98], 9}, {NULL}};
const Segment SGI140559[] = {{&S25818[71], 6}, {NULL}};
const Segment SGI140560[] = {{&S25827[44], 11}, {NULL}};
const Segment SGI140561[] = {{&S25827[44], 23}, {NULL}};
const Segment SGI140562[] = {{&S25827[30], 6}, {NULL}};
const Segment SGI140563[] = {{&S25827[116], 3}, {NULL}};
const Segment SGI140564[] = {{&S25827[81], 6}, {NULL}};
const Segment SGI140565[] = {{&S35029[44], 10}, {NULL}};
const Segment SGI140566[] = {{&S35029[44], 22}, {NULL}};
const Segment SGI140567[] = {{&S35029[30], 6}, {NULL}};
const Segment SGI140568[] = {{&S35029[31], 3}, {NULL}};
const Segment SGI140569[] = {{&S35029[80], 6}, {NULL}};
const Segment SGI140570[] = {{&S35032[44], 10}, {NULL}};
const Segment SGI140571[] = {{&S35032[44], 22}, {NULL}};
const Segment SGI140572[] = {{&S35032[30], 6}, {NULL}};
const Segment SGI140573[] = {{&S35032[31], 3}, {NULL}};
const Segment SGI140574[] = {{&S35032[80], 6}, {NULL}};
const Segment SGI140575[] = {{&S35036[44], 10}, {NULL}};
const Segment SGI140576[] = {{&S35036[44], 21}, {NULL}};
const Segment SGI140577[] = {{&S35036[30], 6}, {NULL}};
const Segment SGI140578[] = {{&S35036[31], 3}, {NULL}};
const Segment SGI140579[] = {{&S35036[79], 6}, {NULL}};
const Segment SGI140580[] = {{&S35037[44], 10}, {NULL}};
const Segment SGI140581[] = {{&S35037[44], 22}, {NULL}};
const Segment SGI140582[] = {{&S35037[30], 6}, {NULL}};
const Segment SGI140583[] = {{&S35037[31], 3}, {NULL}};
const Segment SGI140584[] = {{&S35037[80], 6}, {NULL}};
const Segment SGI140585[] = {{&S35038[38], 10}, {NULL}};
const Segment SGI140586[] = {{&S35038[31], 6}, {NULL}};
const Segment SGI140587[] = {{&S35039[44], 18}, {NULL}};
const Segment SGI140588[] = {{&S35039[44], 30}, {NULL}};
const Segment SGI140589[] = {{&S35039[30], 6}, {NULL}};
const Segment SGI140590[] = {{&S35039[31], 3}, {NULL}};
const Segment SGI140591[] = {{&S35039[88], 6}, {NULL}};
const Segment SGI140592[] = {{&S35040[38], 18}, {NULL}};
const Segment SGI140593[] = {{&S35040[31], 6}, {NULL}};
const Segment SGI140594[] = {{&S35046[44], 10}, {NULL}};
const Segment SGI140595[] = {{&S35046[44], 22}, {NULL}};
const Segment SGI140596[] = {{&S35046[30], 6}, {NULL}};
const Segment SGI140597[] = {{&S35046[31], 3}, {NULL}};
const Segment SGI140598[] = {{&S35046[80], 6}, {NULL}};
const Segment SGI140599[] = {{&S35047[38], 10}, {NULL}};
const Segment SGI140600[] = {{&S35047[31], 6}, {NULL}};
const Segment SGI140601[] = {{&S35048[44], 9}, {NULL}};
const Segment SGI140602[] = {{&S35048[44], 21}, {NULL}};
const Segment SGI140603[] = {{&S35048[30], 6}, {NULL}};
const Segment SGI140604[] = {{&S35048[31], 3}, {NULL}};
const Segment SGI140605[] = {{&S35048[79], 6}, {NULL}};
const Segment SGI140606[] = {{&S35049[38], 9}, {NULL}};
const Segment SGI140607[] = {{&S35049[31], 6}, {NULL}};
const Segment SGI140608[] = {{&S35050[42], 10}, {NULL}};
const Segment SGI140609[] = {{&S35050[30], 6}, {NULL}};
const Segment SGI140610[] = {{&S35050[31], 3}, {NULL}};
const Segment SGI140611[] = {{&S35050[78], 6}, {NULL}};
const Segment SGI140612[] = {{&S24973[44], 10}, {NULL}};
const Segment SGI140613[] = {{&S24973[44], 22}, {NULL}};
const Segment SGI140614[] = {{&S24973[30], 6}, {NULL}};
const Segment SGI140615[] = {{&S24973[115], 3}, {NULL}};
const Segment SGI140616[] = {{&S24973[80], 6}, {NULL}};
const Segment SGI140617[] = {{&S24974[44], 10}, {NULL}};
const Segment SGI140618[] = {{&S24974[44], 22}, {NULL}};
const Segment SGI140619[] = {{&S24974[30], 6}, {NULL}};
const Segment SGI140620[] = {{&S24974[115], 3}, {NULL}};
const Segment SGI140621[] = {{&S24974[80], 6}, {NULL}};
const Segment SGI140622[] = {{&S24977[38], 10}, {NULL}};
const Segment SGI140623[] = {{&S24977[31], 6}, {NULL}};
const Segment SGI140624[] = {{&S24984[42], 10}, {NULL}};
const Segment SGI140625[] = {{&S24984[30], 6}, {NULL}};
const Segment SGI140626[] = {{&S24984[113], 3}, {NULL}};
const Segment SGI140627[] = {{&S24984[78], 6}, {NULL}};
const Segment SGI140628[] = {{&S24991[44], 33}, {NULL}};
const Segment SGI140629[] = {{&S24991[44], 45}, {NULL}};
const Segment SGI140630[] = {{&S24991[30], 6}, {NULL}};
const Segment SGI140631[] = {{&S24991[138], 3}, {NULL}};
const Segment SGI140632[] = {{&S24991[103], 6}, {NULL}};
const Segment SGI140633[] = {{&S24994[44], 30}, {NULL}};
const Segment SGI140634[] = {{&S24994[44], 42}, {NULL}};
const Segment SGI140635[] = {{&S24994[30], 6}, {NULL}};
const Segment SGI140636[] = {{&S24994[135], 3}, {NULL}};
const Segment SGI140637[] = {{&S24994[100], 6}, {NULL}};
const Segment SGI140638[] = {{&S24995[44], 29}, {NULL}};
const Segment SGI140639[] = {{&S24995[44], 41}, {NULL}};
const Segment SGI140640[] = {{&S24995[30], 6}, {NULL}};
const Segment SGI140641[] = {{&S24995[134], 3}, {NULL}};
const Segment SGI140642[] = {{&S24995[99], 6}, {NULL}};
const Segment SGI140643[] = {{&S24996[38], 10}, {NULL}};
const Segment SGI140644[] = {{&S24996[31], 6}, {NULL}};
const Segment SGI140645[] = {{&S24997[44], 29}, {NULL}};
const Segment SGI140646[] = {{&S24997[44], 41}, {NULL}};
const Segment SGI140647[] = {{&S24997[30], 6}, {NULL}};
const Segment SGI140648[] = {{&S24997[134], 3}, {NULL}};
const Segment SGI140649[] = {{&S24997[99], 6}, {NULL}};
const Segment SGI140650[] = {{&S35149[44], 11}, {NULL}};
const Segment SGI140651[] = {{&S35149[44], 27}, {NULL}};
const Segment SGI140652[] = {{&S35149[30], 6}, {NULL}};
const Segment SGI140653[] = {{&S35149[31], 3}, {NULL}};
const Segment SGI140654[] = {{&S35149[85], 6}, {NULL}};
const Segment SGI140655[] = {{&S35150[44], 11}, {NULL}};
const Segment SGI140656[] = {{&S35150[44], 28}, {NULL}};
const Segment SGI140657[] = {{&S35150[30], 6}, {NULL}};
const Segment SGI140658[] = {{&S35150[31], 3}, {NULL}};
const Segment SGI140659[] = {{&S35150[86], 6}, {NULL}};
const Segment SGI140660[] = {{&S35151[44], 11}, {NULL}};
const Segment SGI140661[] = {{&S35151[44], 48}, {NULL}};
const Segment SGI140662[] = {{&S35151[30], 6}, {NULL}};
const Segment SGI140663[] = {{&S35151[31], 3}, {NULL}};
const Segment SGI140664[] = {{&S35151[106], 6}, {NULL}};
const Segment SGI140665[] = {{&S35152[44], 11}, {NULL}};
const Segment SGI140666[] = {{&S35152[44], 28}, {NULL}};
const Segment SGI140667[] = {{&S35152[30], 6}, {NULL}};
const Segment SGI140668[] = {{&S35152[31], 3}, {NULL}};
const Segment SGI140669[] = {{&S35152[86], 6}, {NULL}};
const Segment SGI140670[] = {{&S35153[44], 11}, {NULL}};
const Segment SGI140671[] = {{&S35153[44], 28}, {NULL}};
const Segment SGI140672[] = {{&S35153[30], 6}, {NULL}};
const Segment SGI140673[] = {{&S35153[31], 3}, {NULL}};
const Segment SGI140674[] = {{&S35153[86], 6}, {NULL}};
const Segment SGI140675[] = {{&S35154[44], 11}, {NULL}};
const Segment SGI140676[] = {{&S35154[44], 48}, {NULL}};
const Segment SGI140677[] = {{&S35154[30], 6}, {NULL}};
const Segment SGI140678[] = {{&S35154[31], 3}, {NULL}};
const Segment SGI140679[] = {{&S35154[106], 6}, {NULL}};
const Segment SGI140680[] = {{&S35155[44], 11}, {NULL}};
const Segment SGI140681[] = {{&S35155[44], 48}, {NULL}};
const Segment SGI140682[] = {{&S35155[30], 6}, {NULL}};
const Segment SGI140683[] = {{&S35155[31], 3}, {NULL}};
const Segment SGI140684[] = {{&S35155[106], 6}, {NULL}};
const Segment SGI140685[] = {{&S35156[44], 11}, {NULL}};
const Segment SGI140686[] = {{&S35156[44], 28}, {NULL}};
const Segment SGI140687[] = {{&S35156[30], 6}, {NULL}};
const Segment SGI140688[] = {{&S35156[31], 3}, {NULL}};
const Segment SGI140689[] = {{&S35156[86], 6}, {NULL}};
const Segment SGI140690[] = {{&S35157[44], 11}, {NULL}};
const Segment SGI140691[] = {{&S35157[44], 41}, {NULL}};
const Segment SGI140692[] = {{&S35157[30], 6}, {NULL}};
const Segment SGI140693[] = {{&S35157[31], 3}, {NULL}};
const Segment SGI140694[] = {{&S35157[99], 6}, {NULL}};
const Segment SGI140695[] = {{&S35158[44], 11}, {NULL}};
const Segment SGI140696[] = {{&S35158[44], 29}, {NULL}};
const Segment SGI140697[] = {{&S35158[30], 6}, {NULL}};
const Segment SGI140698[] = {{&S35158[31], 3}, {NULL}};
const Segment SGI140699[] = {{&S35158[87], 6}, {NULL}};
const Segment SGI140700[] = {{&S35159[44], 11}, {NULL}};
const Segment SGI140701[] = {{&S35159[44], 28}, {NULL}};
const Segment SGI140702[] = {{&S35159[30], 6}, {NULL}};
const Segment SGI140703[] = {{&S35159[31], 3}, {NULL}};
const Segment SGI140704[] = {{&S35159[86], 6}, {NULL}};
const Segment SGI140705[] = {{&S35160[44], 11}, {NULL}};
const Segment SGI140706[] = {{&S35160[44], 27}, {NULL}};
const Segment SGI140707[] = {{&S35160[30], 6}, {NULL}};
const Segment SGI140708[] = {{&S35160[31], 3}, {NULL}};
const Segment SGI140709[] = {{&S35160[85], 6}, {NULL}};
const Segment SGI140710[] = {{&S35161[44], 11}, {NULL}};
const Segment SGI140711[] = {{&S35161[44], 28}, {NULL}};
const Segment SGI140712[] = {{&S35161[30], 6}, {NULL}};
const Segment SGI140713[] = {{&S35161[31], 3}, {NULL}};
const Segment SGI140714[] = {{&S35161[86], 6}, {NULL}};
const Segment SGI140715[] = {{&S35162[44], 11}, {NULL}};
const Segment SGI140716[] = {{&S35162[44], 26}, {NULL}};
const Segment SGI140717[] = {{&S35162[30], 6}, {NULL}};
const Segment SGI140718[] = {{&S35162[31], 3}, {NULL}};
const Segment SGI140719[] = {{&S35162[84], 6}, {NULL}};
const Segment SGI140720[] = {{&S35163[44], 11}, {NULL}};
const Segment SGI140721[] = {{&S35163[44], 24}, {NULL}};
const Segment SGI140722[] = {{&S35163[30], 6}, {NULL}};
const Segment SGI140723[] = {{&S35163[31], 3}, {NULL}};
const Segment SGI140724[] = {{&S35163[82], 6}, {NULL}};
const Segment SGI140725[] = {{&S35164[44], 7}, {NULL}};
const Segment SGI140726[] = {{&S35164[44], 24}, {NULL}};
const Segment SGI140727[] = {{&S35164[30], 6}, {NULL}};
const Segment SGI140728[] = {{&S35164[31], 3}, {NULL}};
const Segment SGI140729[] = {{&S35164[82], 6}, {NULL}};
const Segment SGI140730[] = {{&S35165[44], 13}, {NULL}};
const Segment SGI140731[] = {{&S35165[44], 30}, {NULL}};
const Segment SGI140732[] = {{&S35165[30], 6}, {NULL}};
const Segment SGI140733[] = {{&S35165[31], 3}, {NULL}};
const Segment SGI140734[] = {{&S35165[88], 6}, {NULL}};
const Segment SGI140735[] = {{&S35166[44], 11}, {NULL}};
const Segment SGI140736[] = {{&S35166[44], 28}, {NULL}};
const Segment SGI140737[] = {{&S35166[30], 6}, {NULL}};
const Segment SGI140738[] = {{&S35166[31], 3}, {NULL}};
const Segment SGI140739[] = {{&S35166[86], 6}, {NULL}};
const Segment SGI140740[] = {{&S35167[44], 7}, {NULL}};
const Segment SGI140741[] = {{&S35167[44], 24}, {NULL}};
const Segment SGI140742[] = {{&S35167[30], 6}, {NULL}};
const Segment SGI140743[] = {{&S35167[31], 3}, {NULL}};
const Segment SGI140744[] = {{&S35167[82], 6}, {NULL}};
const Segment SGI140745[] = {{&S35168[38], 7}, {NULL}};
const Segment SGI140746[] = {{&S35168[31], 6}, {NULL}};
const Segment SGI140747[] = {{&S35169[44], 13}, {NULL}};
const Segment SGI140748[] = {{&S35169[44], 30}, {NULL}};
const Segment SGI140749[] = {{&S35169[30], 6}, {NULL}};
const Segment SGI140750[] = {{&S35169[31], 3}, {NULL}};
const Segment SGI140751[] = {{&S35169[88], 6}, {NULL}};
const Segment SGI140752[] = {{&S35170[44], 11}, {NULL}};
const Segment SGI140753[] = {{&S35170[44], 28}, {NULL}};
const Segment SGI140754[] = {{&S35170[30], 6}, {NULL}};
const Segment SGI140755[] = {{&S35170[31], 3}, {NULL}};
const Segment SGI140756[] = {{&S35170[86], 6}, {NULL}};
const Segment SGI140757[] = {{&S35171[44], 11}, {NULL}};
const Segment SGI140758[] = {{&S35171[44], 28}, {NULL}};
const Segment SGI140759[] = {{&S35171[30], 6}, {NULL}};
const Segment SGI140760[] = {{&S35171[31], 3}, {NULL}};
const Segment SGI140761[] = {{&S35171[86], 6}, {NULL}};
const Segment SGI140762[] = {{&S35172[44], 7}, {NULL}};
const Segment SGI140763[] = {{&S35172[44], 24}, {NULL}};
const Segment SGI140764[] = {{&S35172[30], 6}, {NULL}};
const Segment SGI140765[] = {{&S35172[31], 3}, {NULL}};
const Segment SGI140766[] = {{&S35172[82], 6}, {NULL}};
const Segment SGI140767[] = {{&S35173[44], 11}, {NULL}};
const Segment SGI140768[] = {{&S35173[44], 28}, {NULL}};
const Segment SGI140769[] = {{&S35173[30], 6}, {NULL}};
const Segment SGI140770[] = {{&S35173[31], 3}, {NULL}};
const Segment SGI140771[] = {{&S35173[86], 6}, {NULL}};
const Segment SGI140772[] = {{&S35174[44], 11}, {NULL}};
const Segment SGI140773[] = {{&S35174[44], 28}, {NULL}};
const Segment SGI140774[] = {{&S35174[30], 6}, {NULL}};
const Segment SGI140775[] = {{&S35174[31], 3}, {NULL}};
const Segment SGI140776[] = {{&S35174[86], 6}, {NULL}};
const Segment SGI140777[] = {{&S22808[32], 9}, {NULL}};
const Segment SGI140778[] = {{&S22808[27], 4}, {NULL}};
const Segment SGI140779[] = {{&S22808[94], 9}, {NULL}};
const Segment SGI140780[] = {{&S22808[67], 6}, {NULL}};
const Segment SGI140781[] = {{&S22812[39], 9}, {NULL}};
const Segment SGI140782[] = {{&S22812[39], 21}, {NULL}};
const Segment SGI140783[] = {{&S22812[27], 4}, {NULL}};
const Segment SGI140784[] = {{&S22812[100], 9}, {NULL}};
const Segment SGI140785[] = {{&S22812[74], 6}, {NULL}};
const Segment SGI140786[] = {{&S25200[38], 10}, {NULL}};
const Segment SGI140787[] = {{&S25200[31], 6}, {NULL}};
const Segment SGI140788[] = {{&S25203[44], 10}, {NULL}};
const Segment SGI140789[] = {{&S25203[44], 22}, {NULL}};
const Segment SGI140790[] = {{&S25203[30], 6}, {NULL}};
const Segment SGI140791[] = {{&S25203[115], 3}, {NULL}};
const Segment SGI140792[] = {{&S25203[80], 6}, {NULL}};
const Segment SGI140793[] = {{&S25204[44], 10}, {NULL}};
const Segment SGI140794[] = {{&S25204[44], 22}, {NULL}};
const Segment SGI140795[] = {{&S25204[30], 6}, {NULL}};
const Segment SGI140796[] = {{&S25204[115], 3}, {NULL}};
const Segment SGI140797[] = {{&S25204[80], 6}, {NULL}};
const Segment SGI140798[] = {{&S25222[44], 10}, {NULL}};
const Segment SGI140799[] = {{&S25222[44], 22}, {NULL}};
const Segment SGI140800[] = {{&S25222[30], 6}, {NULL}};
const Segment SGI140801[] = {{&S25222[83], 6}, {NULL}};
const Segment SGI140802[] = {{&S25223[44], 10}, {NULL}};
const Segment SGI140803[] = {{&S25223[44], 22}, {NULL}};
const Segment SGI140804[] = {{&S25223[30], 6}, {NULL}};
const Segment SGI140805[] = {{&S25223[127], 3}, {NULL}};
const Segment SGI140806[] = {{&S25223[80], 6}, {NULL}};
const Segment SGI140807[] = {{&S25225[44], 10}, {NULL}};
const Segment SGI140808[] = {{&S25225[44], 22}, {NULL}};
const Segment SGI140809[] = {{&S25225[30], 6}, {NULL}};
const Segment SGI140810[] = {{&S25225[125], 3}, {NULL}};
const Segment SGI140811[] = {{&S25225[90], 6}, {NULL}};
const Segment SGI140812[] = {{&S25225[145], 8}, {NULL}};
const Segment SGI140813[] = {{&S25226[42], 10}, {NULL}};
const Segment SGI140814[] = {{&S25226[30], 6}, {NULL}};
const Segment SGI140815[] = {{&S25226[113], 3}, {NULL}};
const Segment SGI140816[] = {{&S25226[78], 6}, {NULL}};
const Segment SGI140817[] = {{&S25235[44], 10}, {NULL}};
const Segment SGI140818[] = {{&S25235[44], 22}, {NULL}};
const Segment SGI140819[] = {{&S25235[30], 6}, {NULL}};
const Segment SGI140820[] = {{&S25235[115], 3}, {NULL}};
const Segment SGI140821[] = {{&S25235[80], 6}, {NULL}};
const Segment SGI140822[] = {{&S25242[38], 10}, {NULL}};
const Segment SGI140823[] = {{&S25242[31], 6}, {NULL}};
const Segment SGI140824[] = {{&S25243[44], 10}, {NULL}};
const Segment SGI140825[] = {{&S25243[44], 22}, {NULL}};
const Segment SGI140826[] = {{&S25243[30], 6}, {NULL}};
const Segment SGI140827[] = {{&S25243[115], 3}, {NULL}};
const Segment SGI140828[] = {{&S25243[80], 6}, {NULL}};
const Segment SGI140829[] = {{&S26049[44], 29}, {NULL}};
const Segment SGI140830[] = {{&S26049[44], 41}, {NULL}};
const Segment SGI140831[] = {{&S26049[30], 6}, {NULL}};
const Segment SGI140832[] = {{&S26049[134], 3}, {NULL}};
const Segment SGI140833[] = {{&S26049[99], 6}, {NULL}};
const Segment SGI140834[] = {{&S26050[38], 29}, {NULL}};
const Segment SGI140835[] = {{&S26050[31], 6}, {NULL}};
const Segment SGI140836[] = {{&S26051[44], 30}, {NULL}};
const Segment SGI140837[] = {{&S26051[44], 42}, {NULL}};
const Segment SGI140838[] = {{&S26051[30], 6}, {NULL}};
const Segment SGI140839[] = {{&S26051[135], 3}, {NULL}};
const Segment SGI140840[] = {{&S26051[100], 6}, {NULL}};
const Segment SGI140841[] = {{&S26052[38], 30}, {NULL}};
const Segment SGI140842[] = {{&S26052[31], 6}, {NULL}};
const Segment SGI140843[] = {{&S26053[44], 17}, {NULL}};
const Segment SGI140844[] = {{&S26053[44], 29}, {NULL}};
const Segment SGI140845[] = {{&S26053[30], 6}, {NULL}};
const Segment SGI140846[] = {{&S26053[122], 3}, {NULL}};
const Segment SGI140847[] = {{&S26053[87], 6}, {NULL}};
const Segment SGI140848[] = {{&S25304[44], 10}, {NULL}};
const Segment SGI140849[] = {{&S25304[44], 22}, {NULL}};
const Segment SGI140850[] = {{&S25304[30], 6}, {NULL}};
const Segment SGI140851[] = {{&S25304[115], 3}, {NULL}};
const Segment SGI140852[] = {{&S25304[80], 6}, {NULL}};
const Segment SGI140853[] = {{&S26054[44], 30}, {NULL}};
const Segment SGI140854[] = {{&S26054[44], 42}, {NULL}};
const Segment SGI140855[] = {{&S26054[30], 6}, {NULL}};
const Segment SGI140856[] = {{&S26054[135], 3}, {NULL}};
const Segment SGI140857[] = {{&S26054[100], 6}, {NULL}};
const Segment SGI140858[] = {{&S26055[44], 29}, {NULL}};
const Segment SGI140859[] = {{&S26055[44], 41}, {NULL}};
const Segment SGI140860[] = {{&S26055[30], 6}, {NULL}};
const Segment SGI140861[] = {{&S26055[134], 3}, {NULL}};
const Segment SGI140862[] = {{&S26055[99], 6}, {NULL}};
const Segment SGI140863[] = {{&S26057[38], 30}, {NULL}};
const Segment SGI140864[] = {{&S26057[31], 6}, {NULL}};
const Segment SGI140865[] = {{&S25311[44], 10}, {NULL}};
const Segment SGI140866[] = {{&S25311[44], 66}, {NULL}};
const Segment SGI140867[] = {{&S25311[30], 6}, {NULL}};
const Segment SGI140868[] = {{&S26058[44], 29}, {NULL}};
const Segment SGI140869[] = {{&S26058[44], 51}, {NULL}};
const Segment SGI140870[] = {{&S26058[30], 6}, {NULL}};
const Segment SGI140871[] = {{&S25312[38], 10}, {NULL}};
const Segment SGI140872[] = {{&S26058[144], 3}, {NULL}};
const Segment SGI140873[] = {{&S25312[31], 6}, {NULL}};
const Segment SGI140874[] = {{&S26058[109], 6}, {NULL}};
const Segment SGI140875[] = {{&S26059[44], 29}, {NULL}};
const Segment SGI140876[] = {{&S26059[44], 44}, {NULL}};
const Segment SGI140877[] = {{&S26059[30], 6}, {NULL}};
const Segment SGI140878[] = {{&S26059[137], 3}, {NULL}};
const Segment SGI140879[] = {{&S26059[102], 6}, {NULL}};
const Segment SGI140880[] = {{&S35454[44], 7}, {NULL}};
const Segment SGI140881[] = {{&S35454[44], 22}, {NULL}};
const Segment SGI140882[] = {{&S35454[30], 6}, {NULL}};
const Segment SGI140883[] = {{&S35454[31], 3}, {NULL}};
const Segment SGI140884[] = {{&S35454[80], 6}, {NULL}};
const Segment SGI140885[] = {{&S35455[44], 7}, {NULL}};
const Segment SGI140886[] = {{&S35455[44], 30}, {NULL}};
const Segment SGI140887[] = {{&S35455[30], 6}, {NULL}};
const Segment SGI140888[] = {{&S35455[31], 3}, {NULL}};
const Segment SGI140889[] = {{&S35455[88], 6}, {NULL}};
const Segment SGI140890[] = {{&S35456[44], 7}, {NULL}};
const Segment SGI140891[] = {{&S35456[44], 26}, {NULL}};
const Segment SGI140892[] = {{&S35456[30], 6}, {NULL}};
const Segment SGI140893[] = {{&S35456[31], 3}, {NULL}};
const Segment SGI140894[] = {{&S35456[84], 6}, {NULL}};
const Segment SGI140895[] = {{&S35457[44], 7}, {NULL}};
const Segment SGI140896[] = {{&S35457[44], 31}, {NULL}};
const Segment SGI140897[] = {{&S35457[30], 6}, {NULL}};
const Segment SGI140898[] = {{&S35457[31], 3}, {NULL}};
const Segment SGI140899[] = {{&S35457[89], 6}, {NULL}};
const Segment SGI140900[] = {{&S35458[44], 7}, {NULL}};
const Segment SGI140901[] = {{&S35458[44], 31}, {NULL}};
const Segment SGI140902[] = {{&S35458[30], 6}, {NULL}};
const Segment SGI140903[] = {{&S35458[31], 3}, {NULL}};
const Segment SGI140904[] = {{&S35458[89], 6}, {NULL}};
const Segment SGI140905[] = {{&S35459[38], 7}, {NULL}};
const Segment SGI140906[] = {{&S35459[31], 6}, {NULL}};
const Segment SGI140907[] = {{&S35460[44], 7}, {NULL}};
const Segment SGI140908[] = {{&S35460[44], 32}, {NULL}};
const Segment SGI140909[] = {{&S35460[30], 6}, {NULL}};
const Segment SGI140910[] = {{&S35460[31], 3}, {NULL}};
const Segment SGI140911[] = {{&S35460[90], 6}, {NULL}};
const Segment SGI140912[] = {{&S35461[44], 7}, {NULL}};
const Segment SGI140913[] = {{&S35461[44], 31}, {NULL}};
const Segment SGI140914[] = {{&S35461[30], 6}, {NULL}};
const Segment SGI140915[] = {{&S35461[31], 3}, {NULL}};
const Segment SGI140916[] = {{&S35461[89], 6}, {NULL}};
const Segment SGI140917[] = {{&S35462[44], 7}, {NULL}};
const Segment SGI140918[] = {{&S35462[44], 34}, {NULL}};
const Segment SGI140919[] = {{&S35462[30], 6}, {NULL}};
const Segment SGI140920[] = {{&S35462[31], 3}, {NULL}};
const Segment SGI140921[] = {{&S35462[92], 6}, {NULL}};
const Segment SGI140922[] = {{&S35463[44], 7}, {NULL}};
const Segment SGI140923[] = {{&S35463[44], 26}, {NULL}};
const Segment SGI140924[] = {{&S35463[30], 6}, {NULL}};
const Segment SGI140925[] = {{&S35463[31], 3}, {NULL}};
const Segment SGI140926[] = {{&S35463[84], 6}, {NULL}};
const Segment SGI140927[] = {{&S35464[44], 7}, {NULL}};
const Segment SGI140928[] = {{&S35464[44], 32}, {NULL}};
const Segment SGI140929[] = {{&S35464[30], 6}, {NULL}};
const Segment SGI140930[] = {{&S35464[93], 6}, {NULL}};
const Segment SGI140931[] = {{&S35465[38], 7}, {NULL}};
const Segment SGI140932[] = {{&S35465[31], 6}, {NULL}};
const Segment SGI140933[] = {{&S35466[42], 7}, {NULL}};
const Segment SGI140934[] = {{&S35466[30], 6}, {NULL}};
const Segment SGI140935[] = {{&S35466[31], 3}, {NULL}};
const Segment SGI140936[] = {{&S35466[87], 6}, {NULL}};
const Segment SGI140937[] = {{&S35467[44], 7}, {NULL}};
const Segment SGI140938[] = {{&S35467[44], 26}, {NULL}};
const Segment SGI140939[] = {{&S35467[30], 6}, {NULL}};
const Segment SGI140940[] = {{&S35467[31], 3}, {NULL}};
const Segment SGI140941[] = {{&S35467[84], 6}, {NULL}};
const Segment SGI140942[] = {{&S35468[44], 7}, {NULL}};
const Segment SGI140943[] = {{&S35468[44], 30}, {NULL}};
const Segment SGI140944[] = {{&S35468[30], 6}, {NULL}};
const Segment SGI140945[] = {{&S35468[91], 6}, {NULL}};
const Segment SGI140946[] = {{&S35469[55], 7}, {NULL}};
const Segment SGI140947[] = {{&S35469[55], 31}, {NULL}};
const Segment SGI140948[] = {{&S35469[41], 6}, {NULL}};
const Segment SGI140949[] = {{&S35469[42], 3}, {NULL}};
const Segment SGI140950[] = {{&S35469[105], 5}, {NULL}};
const Segment SGI140951[] = {{&S35593[44], 12}, {NULL}};
const Segment SGI140952[] = {{&S35593[44], 23}, {NULL}};
const Segment SGI140953[] = {{&S35593[30], 6}, {NULL}};
const Segment SGI140954[] = {{&S35593[31], 3}, {NULL}};
const Segment SGI140955[] = {{&S35593[81], 6}, {NULL}};
const Segment SGI140956[] = {{&S35595[44], 9}, {NULL}};
const Segment SGI140957[] = {{&S35595[44], 21}, {NULL}};
const Segment SGI140958[] = {{&S35595[30], 6}, {NULL}};
const Segment SGI140959[] = {{&S35595[31], 3}, {NULL}};
const Segment SGI140960[] = {{&S35595[79], 6}, {NULL}};
const Segment SGI140961[] = {{&S35598[44], 9}, {NULL}};
const Segment SGI140962[] = {{&S35598[44], 21}, {NULL}};
const Segment SGI140963[] = {{&S35598[30], 6}, {NULL}};
const Segment SGI140964[] = {{&S35598[31], 3}, {NULL}};
const Segment SGI140965[] = {{&S35598[79], 6}, {NULL}};
const Segment SGI140966[] = {{&S35601[44], 9}, {NULL}};
const Segment SGI140967[] = {{&S35601[44], 21}, {NULL}};
const Segment SGI140968[] = {{&S35601[30], 6}, {NULL}};
const Segment SGI140969[] = {{&S35601[31], 3}, {NULL}};
const Segment SGI140970[] = {{&S35601[79], 6}, {NULL}};
const Segment SGI140971[] = {{&S35603[44], 5}, {NULL}};
const Segment SGI140972[] = {{&S35603[44], 17}, {NULL}};
const Segment SGI140973[] = {{&S35603[30], 6}, {NULL}};
const Segment SGI140974[] = {{&S35603[31], 3}, {NULL}};
const Segment SGI140975[] = {{&S35603[75], 6}, {NULL}};
const Segment SGI140976[] = {{&S35604[44], 9}, {NULL}};
const Segment SGI140977[] = {{&S35604[44], 21}, {NULL}};
const Segment SGI140978[] = {{&S35604[30], 6}, {NULL}};
const Segment SGI140979[] = {{&S35604[31], 3}, {NULL}};
const Segment SGI140980[] = {{&S35604[79], 6}, {NULL}};
const Segment SGI140981[] = {{&S35606[38], 5}, {NULL}};
const Segment SGI140982[] = {{&S35606[31], 6}, {NULL}};
const Segment SGI140983[] = {{&S35607[38], 9}, {NULL}};
const Segment SGI140984[] = {{&S35607[31], 6}, {NULL}};
const Segment SGI140985[] = {{&S35608[44], 5}, {NULL}};
const Segment SGI140986[] = {{&S35608[44], 17}, {NULL}};
const Segment SGI140987[] = {{&S35608[30], 6}, {NULL}};
const Segment SGI140988[] = {{&S35608[75], 6}, {NULL}};
const Segment SGI140989[] = {{&S25571[34], 11}, {NULL}};
const Segment SGI140990[] = {{&S25571[27], 6}, {NULL}};
const Segment SGI140991[] = {{&S25571[98], 9}, {NULL}};
const Segment SGI140992[] = {{&S25571[71], 6}, {NULL}};
const Segment SGI140993[] = {{&S25572[34], 10}, {NULL}};
const Segment SGI140994[] = {{&S25572[27], 6}, {NULL}};
const Segment SGI140995[] = {{&S25572[97], 9}, {NULL}};
const Segment SGI140996[] = {{&S25572[70], 6}, {NULL}};
const Segment SGI140997[] = {{&S25573[34], 10}, {NULL}};
const Segment SGI140998[] = {{&S25573[27], 6}, {NULL}};
const Segment SGI140999[] = {{&S25573[97], 9}, {NULL}};
const Segment SGI141000[] = {{&S25573[70], 6}, {NULL}};
const Segment SGI141001[] = {{&S25575[34], 10}, {NULL}};
const Segment SGI141002[] = {{&S25575[27], 6}, {NULL}};
const Segment SGI141003[] = {{&S25575[97], 9}, {NULL}};
const Segment SGI141004[] = {{&S25575[70], 6}, {NULL}};
const Segment SGI141005[] = {{&S25576[34], 11}, {NULL}};
const Segment SGI141006[] = {{&S25576[27], 6}, {NULL}};
const Segment SGI141007[] = {{&S25576[98], 9}, {NULL}};
const Segment SGI141008[] = {{&S25576[71], 6}, {NULL}};
const Segment SGI141009[] = {{&S25577[34], 11}, {NULL}};
const Segment SGI141010[] = {{&S25577[27], 6}, {NULL}};
const Segment SGI141011[] = {{&S25577[98], 9}, {NULL}};
const Segment SGI141012[] = {{&S25577[71], 6}, {NULL}};
const Segment SGI141013[] = {{&S25578[34], 8}, {NULL}};
const Segment SGI141014[] = {{&S25578[27], 6}, {NULL}};
const Segment SGI141015[] = {{&S25578[95], 9}, {NULL}};
const Segment SGI141016[] = {{&S25578[68], 6}, {NULL}};
const Segment SGI141017[] = {{&S25582[34], 11}, {NULL}};
const Segment SGI141018[] = {{&S25582[27], 6}, {NULL}};
const Segment SGI141019[] = {{&S25582[98], 9}, {NULL}};
const Segment SGI141020[] = {{&S25582[71], 6}, {NULL}};
const Segment SGI141021[] = {{&S25585[34], 10}, {NULL}};
const Segment SGI141022[] = {{&S25585[27], 6}, {NULL}};
const Segment SGI141023[] = {{&S25585[97], 9}, {NULL}};
const Segment SGI141024[] = {{&S25585[70], 6}, {NULL}};
const Segment SGI141025[] = {{&S25670[34], 10}, {NULL}};
const Segment SGI141026[] = {{&S25670[27], 6}, {NULL}};
const Segment SGI141027[] = {{&S25670[97], 9}, {NULL}};
const Segment SGI141028[] = {{&S25670[70], 6}, {NULL}};
const Segment SGI141029[] = {{&S25674[34], 10}, {NULL}};
const Segment SGI141030[] = {{&S25674[27], 6}, {NULL}};
const Segment SGI141031[] = {{&S25674[97], 9}, {NULL}};
const Segment SGI141032[] = {{&S25674[70], 6}, {NULL}};
const Segment SGI141033[] = {{&S25676[34], 10}, {NULL}};
const Segment SGI141034[] = {{&S25676[27], 6}, {NULL}};
const Segment SGI141035[] = {{&S25676[97], 9}, {NULL}};
const Segment SGI141036[] = {{&S25676[70], 6}, {NULL}};
const Segment SGI141037[] = {{&S25677[34], 11}, {NULL}};
const Segment SGI141038[] = {{&S25677[27], 6}, {NULL}};
const Segment SGI141039[] = {{&S25677[98], 9}, {NULL}};
const Segment SGI141040[] = {{&S25677[71], 6}, {NULL}};
const Segment SGI141041[] = {{&S25680[34], 10}, {NULL}};
const Segment SGI141042[] = {{&S25680[27], 6}, {NULL}};
const Segment SGI141043[] = {{&S25680[97], 9}, {NULL}};
const Segment SGI141044[] = {{&S25680[70], 6}, {NULL}};
const Segment SGI141045[] = {{&S25682[34], 11}, {NULL}};
const Segment SGI141046[] = {{&S25682[27], 6}, {NULL}};
const Segment SGI141047[] = {{&S25682[98], 9}, {NULL}};
const Segment SGI141048[] = {{&S25682[71], 6}, {NULL}};
const Segment SGI141049[] = {{&S25687[34], 11}, {NULL}};
const Segment SGI141050[] = {{&S25687[27], 6}, {NULL}};
const Segment SGI141051[] = {{&S25687[98], 9}, {NULL}};
const Segment SGI141052[] = {{&S25687[71], 6}, {NULL}};
const Segment SGI141053[] = {{&S25688[34], 18}, {NULL}};
const Segment SGI141054[] = {{&S25688[27], 6}, {NULL}};
const Segment SGI141055[] = {{&S25688[105], 9}, {NULL}};
const Segment SGI141056[] = {{&S25688[78], 6}, {NULL}};
const Segment SGI141057[] = {{&S25689[34], 11}, {NULL}};
const Segment SGI141058[] = {{&S25689[27], 6}, {NULL}};
const Segment SGI141059[] = {{&S25689[98], 9}, {NULL}};
const Segment SGI141060[] = {{&S25689[71], 6}, {NULL}};
const Segment SGI141061[] = {{&S26492[44], 9}, {NULL}};
const Segment SGI141062[] = {{&S26492[44], 21}, {NULL}};
const Segment SGI141063[] = {{&S26492[30], 6}, {NULL}};
const Segment SGI141064[] = {{&S26492[114], 3}, {NULL}};
const Segment SGI141065[] = {{&S26492[79], 6}, {NULL}};
const Segment SGI141066[] = {{&S26493[44], 9}, {NULL}};
const Segment SGI141067[] = {{&S26493[44], 21}, {NULL}};
const Segment SGI141068[] = {{&S26493[30], 6}, {NULL}};
const Segment SGI141069[] = {{&S26493[114], 3}, {NULL}};
const Segment SGI141070[] = {{&S26493[79], 6}, {NULL}};
const Segment SGI141071[] = {{&S26495[38], 9}, {NULL}};
const Segment SGI141072[] = {{&S26495[31], 6}, {NULL}};
const Segment SGI141073[] = {{&S26496[44], 9}, {NULL}};
const Segment SGI141074[] = {{&S26496[44], 21}, {NULL}};
const Segment SGI141075[] = {{&S26496[30], 6}, {NULL}};
const Segment SGI141076[] = {{&S26496[114], 3}, {NULL}};
const Segment SGI141077[] = {{&S26496[79], 6}, {NULL}};
const Segment SGI141078[] = {{&S26497[44], 9}, {NULL}};
const Segment SGI141079[] = {{&S26497[44], 21}, {NULL}};
const Segment SGI141080[] = {{&S26497[30], 6}, {NULL}};
const Segment SGI141081[] = {{&S26497[114], 3}, {NULL}};
const Segment SGI141082[] = {{&S26497[79], 6}, {NULL}};
const Segment SGI141083[] = {{&S26498[44], 9}, {NULL}};
const Segment SGI141084[] = {{&S26498[44], 21}, {NULL}};
const Segment SGI141085[] = {{&S26498[30], 6}, {NULL}};
const Segment SGI141086[] = {{&S26498[82], 6}, {NULL}};
const Segment SGI141087[] = {{&S26499[44], 24}, {NULL}};
const Segment SGI141088[] = {{&S26499[44], 36}, {NULL}};
const Segment SGI141089[] = {{&S26499[30], 6}, {NULL}};
const Segment SGI141090[] = {{&S26499[129], 3}, {NULL}};
const Segment SGI141091[] = {{&S26499[94], 6}, {NULL}};
const Segment SGI141092[] = {{&S26500[44], 9}, {NULL}};
const Segment SGI141093[] = {{&S26500[44], 21}, {NULL}};
const Segment SGI141094[] = {{&S26500[30], 6}, {NULL}};
const Segment SGI141095[] = {{&S26500[114], 3}, {NULL}};
const Segment SGI141096[] = {{&S26500[79], 6}, {NULL}};
const Segment SGI141097[] = {{&S26517[34], 10}, {NULL}};
const Segment SGI141098[] = {{&S26517[27], 6}, {NULL}};
const Segment SGI141099[] = {{&S26517[97], 9}, {NULL}};
const Segment SGI141100[] = {{&S26517[70], 6}, {NULL}};
const Segment SGI141101[] = {{&S26519[34], 18}, {NULL}};
const Segment SGI141102[] = {{&S26519[27], 6}, {NULL}};
const Segment SGI141103[] = {{&S35874[34], 11}, {NULL}};
const Segment SGI141104[] = {{&S35874[27], 6}, {NULL}};
const Segment SGI141105[] = {{&S35874[98], 9}, {NULL}};
const Segment SGI141106[] = {{&S35874[71], 6}, {NULL}};
const Segment SGI141107[] = {{&S35875[34], 10}, {NULL}};
const Segment SGI141108[] = {{&S35875[27], 6}, {NULL}};
const Segment SGI141109[] = {{&S35875[97], 9}, {NULL}};
const Segment SGI141110[] = {{&S35875[70], 6}, {NULL}};
const Segment SGI141111[] = {{&S35876[34], 11}, {NULL}};
const Segment SGI141112[] = {{&S35876[27], 6}, {NULL}};
const Segment SGI141113[] = {{&S35876[98], 9}, {NULL}};
const Segment SGI141114[] = {{&S35876[71], 6}, {NULL}};
const Segment SGI141115[] = {{&S35878[34], 10}, {NULL}};
const Segment SGI141116[] = {{&S35878[27], 6}, {NULL}};
const Segment SGI141117[] = {{&S35878[97], 9}, {NULL}};
const Segment SGI141118[] = {{&S35878[70], 6}, {NULL}};
const Segment SGI141119[] = {{&S35879[34], 10}, {NULL}};
const Segment SGI141120[] = {{&S35879[27], 6}, {NULL}};
const Segment SGI141121[] = {{&S35879[97], 9}, {NULL}};
const Segment SGI141122[] = {{&S35879[70], 6}, {NULL}};
const Segment SGI141123[] = {{&S35880[34], 11}, {NULL}};
const Segment SGI141124[] = {{&S35880[27], 6}, {NULL}};
const Segment SGI141125[] = {{&S35880[98], 9}, {NULL}};
const Segment SGI141126[] = {{&S35880[71], 6}, {NULL}};
const Segment SGI141127[] = {{&S35881[34], 8}, {NULL}};
const Segment SGI141128[] = {{&S35881[27], 6}, {NULL}};
const Segment SGI141129[] = {{&S35881[95], 9}, {NULL}};
const Segment SGI141130[] = {{&S35881[68], 6}, {NULL}};
const Segment SGI141131[] = {{&S35882[34], 12}, {NULL}};
const Segment SGI141132[] = {{&S35882[27], 6}, {NULL}};
const Segment SGI141133[] = {{&S35882[99], 9}, {NULL}};
const Segment SGI141134[] = {{&S35882[72], 6}, {NULL}};
const Segment SGI141135[] = {{&S35883[34], 10}, {NULL}};
const Segment SGI141136[] = {{&S35883[27], 6}, {NULL}};
const Segment SGI141137[] = {{&S35883[97], 9}, {NULL}};
const Segment SGI141138[] = {{&S35883[70], 6}, {NULL}};
const Segment SGI141139[] = {{&S35885[34], 7}, {NULL}};
const Segment SGI141140[] = {{&S35885[27], 6}, {NULL}};
const Segment SGI141141[] = {{&S35885[94], 9}, {NULL}};
const Segment SGI141142[] = {{&S35885[67], 6}, {NULL}};
const Segment SGI141143[] = {{&S35886[34], 10}, {NULL}};
const Segment SGI141144[] = {{&S35886[27], 6}, {NULL}};
const Segment SGI141145[] = {{&S35886[97], 9}, {NULL}};
const Segment SGI141146[] = {{&S35886[70], 6}, {NULL}};
const Segment SGI141147[] = {{&S35887[34], 11}, {NULL}};
const Segment SGI141148[] = {{&S35887[27], 6}, {NULL}};
const Segment SGI141149[] = {{&S35887[98], 9}, {NULL}};
const Segment SGI141150[] = {{&S35887[71], 6}, {NULL}};
const Segment SGI141151[] = {{&S35891[34], 8}, {NULL}};
const Segment SGI141152[] = {{&S35891[27], 6}, {NULL}};
const Segment SGI141153[] = {{&S35891[95], 9}, {NULL}};
const Segment SGI141154[] = {{&S35891[68], 6}, {NULL}};
const Segment SGI141155[] = {{&S35892[34], 10}, {NULL}};
const Segment SGI141156[] = {{&S35892[27], 6}, {NULL}};
const Segment SGI141157[] = {{&S35892[97], 9}, {NULL}};
const Segment SGI141158[] = {{&S35892[70], 6}, {NULL}};
const Segment SGI141159[] = {{&S35916[44], 10}, {NULL}};
const Segment SGI141160[] = {{&S35916[44], 28}, {NULL}};
const Segment SGI141161[] = {{&S35916[30], 6}, {NULL}};
const Segment SGI141162[] = {{&S35916[31], 3}, {NULL}};
const Segment SGI141163[] = {{&S35916[86], 6}, {NULL}};
const Segment SGI141164[] = {{&S35921[44], 10}, {NULL}};
const Segment SGI141165[] = {{&S35921[44], 28}, {NULL}};
const Segment SGI141166[] = {{&S35921[30], 6}, {NULL}};
const Segment SGI141167[] = {{&S35921[31], 3}, {NULL}};
const Segment SGI141168[] = {{&S35921[86], 6}, {NULL}};
const Segment SGI141169[] = {{&S26519[105], 9}, {NULL}};
const Segment SGI141170[] = {{&S26519[78], 6}, {NULL}};
const Segment SGI141171[] = {{&S26526[34], 11}, {NULL}};
const Segment SGI141172[] = {{&S26526[27], 6}, {NULL}};
const Segment SGI141173[] = {{&S26526[98], 9}, {NULL}};
const Segment SGI141174[] = {{&S26526[71], 6}, {NULL}};
const Segment SGI141175[] = {{&S26528[34], 10}, {NULL}};
const Segment SGI141176[] = {{&S26528[27], 6}, {NULL}};
const Segment SGI141177[] = {{&S26528[97], 9}, {NULL}};
const Segment SGI141178[] = {{&S26528[70], 6}, {NULL}};
const Segment SGI141179[] = {{&S26532[34], 11}, {NULL}};
const Segment SGI141180[] = {{&S26532[27], 6}, {NULL}};
const Segment SGI141181[] = {{&S26532[98], 9}, {NULL}};
const Segment SGI141182[] = {{&S26532[71], 6}, {NULL}};
const Segment SGI141183[] = {{&S26533[34], 11}, {NULL}};
const Segment SGI141184[] = {{&S26533[27], 6}, {NULL}};
const Segment SGI141185[] = {{&S26533[98], 9}, {NULL}};
const Segment SGI141186[] = {{&S26533[71], 6}, {NULL}};
const Segment SGI141187[] = {{&S26534[34], 10}, {NULL}};
const Segment SGI141188[] = {{&S26534[27], 6}, {NULL}};
const Segment SGI141189[] = {{&S26534[97], 9}, {NULL}};
const Segment SGI141190[] = {{&S26534[70], 6}, {NULL}};
const Segment SGI141191[] = {{&S26535[34], 11}, {NULL}};
const Segment SGI141192[] = {{&S26535[27], 6}, {NULL}};
const Segment SGI141193[] = {{&S26535[98], 9}, {NULL}};
const Segment SGI141194[] = {{&S26535[71], 6}, {NULL}};
const Segment SGI141195[] = {{&S25690[34], 11}, {NULL}};
const Segment SGI141196[] = {{&S25690[27], 6}, {NULL}};
const Segment SGI141197[] = {{&S25690[98], 9}, {NULL}};
const Segment SGI141198[] = {{&S25690[71], 6}, {NULL}};
const Segment SGI141199[] = {{&S25691[34], 11}, {NULL}};
const Segment SGI141200[] = {{&S25691[27], 6}, {NULL}};
const Segment SGI141201[] = {{&S25691[98], 9}, {NULL}};
const Segment SGI141202[] = {{&S25691[71], 6}, {NULL}};
const Segment SGI141203[] = {{&S25693[34], 11}, {NULL}};
const Segment SGI141204[] = {{&S25693[27], 6}, {NULL}};
const Segment SGI141205[] = {{&S25693[98], 9}, {NULL}};
const Segment SGI141206[] = {{&S25693[71], 6}, {NULL}};
const Segment SGI141207[] = {{&S25695[34], 10}, {NULL}};
const Segment SGI141208[] = {{&S25695[27], 6}, {NULL}};
const Segment SGI141209[] = {{&S25695[97], 9}, {NULL}};
const Segment SGI141210[] = {{&S25695[70], 6}, {NULL}};
const Segment SGI141211[] = {{&S25696[34], 18}, {NULL}};
const Segment SGI141212[] = {{&S25696[27], 6}, {NULL}};
const Segment SGI141213[] = {{&S25696[105], 9}, {NULL}};
const Segment SGI141214[] = {{&S25696[78], 6}, {NULL}};
const Segment SGI141215[] = {{&S25697[34], 11}, {NULL}};
const Segment SGI141216[] = {{&S25697[27], 6}, {NULL}};
const Segment SGI141217[] = {{&S25697[98], 9}, {NULL}};
const Segment SGI141218[] = {{&S25697[71], 6}, {NULL}};
const Segment SGI141219[] = {{&S25699[34], 11}, {NULL}};
const Segment SGI141220[] = {{&S25699[27], 6}, {NULL}};
const Segment SGI141221[] = {{&S25699[98], 9}, {NULL}};
const Segment SGI141222[] = {{&S25699[71], 6}, {NULL}};
const Segment SGI141223[] = {{&S25700[34], 11}, {NULL}};
const Segment SGI141224[] = {{&S25700[27], 6}, {NULL}};
const Segment SGI141225[] = {{&S25700[98], 9}, {NULL}};
const Segment SGI141226[] = {{&S25700[71], 6}, {NULL}};
const Segment SGI141227[] = {{&S25701[34], 11}, {NULL}};
const Segment SGI141228[] = {{&S25701[27], 6}, {NULL}};
const Segment SGI141229[] = {{&S25701[98], 9}, {NULL}};
const Segment SGI141230[] = {{&S25701[71], 6}, {NULL}};
const Segment SGI141231[] = {{&S25702[34], 11}, {NULL}};
const Segment SGI141232[] = {{&S25702[27], 6}, {NULL}};
const Segment SGI141233[] = {{&S25702[98], 9}, {NULL}};
const Segment SGI141234[] = {{&S25702[71], 6}, {NULL}};
const Segment SGI141235[] = {{&S25802[34], 10}, {NULL}};
const Segment SGI141236[] = {{&S25802[27], 6}, {NULL}};
const Segment SGI141237[] = {{&S25802[97], 9}, {NULL}};
const Segment SGI141238[] = {{&S25802[70], 6}, {NULL}};
const Segment SGI141239[] = {{&S25803[34], 10}, {NULL}};
const Segment SGI141240[] = {{&S25803[27], 6}, {NULL}};
const Segment SGI141241[] = {{&S25803[97], 9}, {NULL}};
const Segment SGI141242[] = {{&S25803[70], 6}, {NULL}};
const Segment SGI141243[] = {{&S25804[34], 11}, {NULL}};
const Segment SGI141244[] = {{&S25804[27], 6}, {NULL}};
const Segment SGI141245[] = {{&S25804[98], 9}, {NULL}};
const Segment SGI141246[] = {{&S25804[71], 6}, {NULL}};
const Segment SGI141247[] = {{&S25806[34], 10}, {NULL}};
const Segment SGI141248[] = {{&S25806[27], 6}, {NULL}};
const Segment SGI141249[] = {{&S25806[97], 9}, {NULL}};
const Segment SGI141250[] = {{&S25806[70], 6}, {NULL}};
const Segment SGI141251[] = {{&S25807[34], 11}, {NULL}};
const Segment SGI141252[] = {{&S25807[27], 6}, {NULL}};
const Segment SGI141253[] = {{&S25807[98], 9}, {NULL}};
const Segment SGI141254[] = {{&S25807[71], 6}, {NULL}};
const Segment SGI141255[] = {{&S25808[34], 11}, {NULL}};
const Segment SGI141256[] = {{&S25808[27], 6}, {NULL}};
const Segment SGI141257[] = {{&S25808[98], 9}, {NULL}};
const Segment SGI141258[] = {{&S25808[71], 6}, {NULL}};
const Segment SGI141259[] = {{&S25809[34], 11}, {NULL}};
const Segment SGI141260[] = {{&S25809[27], 6}, {NULL}};
const Segment SGI141261[] = {{&S25809[98], 9}, {NULL}};
const Segment SGI141262[] = {{&S25809[71], 6}, {NULL}};
const Segment SGI141263[] = {{&S25811[34], 8}, {NULL}};
const Segment SGI141264[] = {{&S25811[27], 6}, {NULL}};
const Segment SGI141265[] = {{&S25811[95], 9}, {NULL}};
const Segment SGI141266[] = {{&S25811[68], 6}, {NULL}};
const Segment SGI141267[] = {{&S25817[34], 10}, {NULL}};
const Segment SGI141268[] = {{&S25817[27], 6}, {NULL}};
const Segment SGI141269[] = {{&S25817[97], 9}, {NULL}};
const Segment SGI141270[] = {{&S25817[70], 6}, {NULL}};
const Segment SGI141271[] = {{&S25828[44], 11}, {NULL}};
const Segment SGI141272[] = {{&S25828[44], 23}, {NULL}};
const Segment SGI141273[] = {{&S25828[30], 6}, {NULL}};
const Segment SGI141274[] = {{&S25828[116], 3}, {NULL}};
const Segment SGI141275[] = {{&S25828[81], 6}, {NULL}};
const Segment SGI141276[] = {{&S25829[44], 32}, {NULL}};
const Segment SGI141277[] = {{&S25829[44], 44}, {NULL}};
const Segment SGI141278[] = {{&S25829[30], 6}, {NULL}};
const Segment SGI141279[] = {{&S25829[137], 3}, {NULL}};
const Segment SGI141280[] = {{&S25829[102], 6}, {NULL}};
const Segment SGI141281[] = {{&S25830[38], 11}, {NULL}};
const Segment SGI141282[] = {{&S25830[31], 6}, {NULL}};
const Segment SGI141283[] = {{&S25831[44], 31}, {NULL}};
const Segment SGI141284[] = {{&S25831[44], 43}, {NULL}};
const Segment SGI141285[] = {{&S25831[30], 6}, {NULL}};
const Segment SGI141286[] = {{&S25831[136], 3}, {NULL}};
const Segment SGI141287[] = {{&S25831[101], 6}, {NULL}};
const Segment SGI141288[] = {{&S25832[44], 31}, {NULL}};
const Segment SGI141289[] = {{&S25832[44], 43}, {NULL}};
const Segment SGI141290[] = {{&S25832[30], 6}, {NULL}};
const Segment SGI141291[] = {{&S25832[136], 3}, {NULL}};
const Segment SGI141292[] = {{&S25832[101], 6}, {NULL}};
const Segment SGI141293[] = {{&S25833[44], 32}, {NULL}};
const Segment SGI141294[] = {{&S25833[44], 44}, {NULL}};
const Segment SGI141295[] = {{&S25833[30], 6}, {NULL}};
const Segment SGI141296[] = {{&S25833[137], 3}, {NULL}};
const Segment SGI141297[] = {{&S25833[102], 6}, {NULL}};
const Segment SGI141298[] = {{&S25834[44], 32}, {NULL}};
const Segment SGI141299[] = {{&S25834[44], 44}, {NULL}};
const Segment SGI141300[] = {{&S25834[30], 6}, {NULL}};
const Segment SGI141301[] = {{&S25834[137], 3}, {NULL}};
const Segment SGI141302[] = {{&S25834[102], 6}, {NULL}};
const Segment SGI141303[] = {{&S25835[42], 11}, {NULL}};
const Segment SGI141304[] = {{&S25835[30], 6}, {NULL}};
const Segment SGI141305[] = {{&S25835[114], 3}, {NULL}};
const Segment SGI141306[] = {{&S25835[79], 6}, {NULL}};
const Segment SGI141307[] = {{&S25836[44], 32}, {NULL}};
const Segment SGI141308[] = {{&S25836[44], 44}, {NULL}};
const Segment SGI141309[] = {{&S25836[30], 6}, {NULL}};
const Segment SGI141310[] = {{&S25836[137], 3}, {NULL}};
const Segment SGI141311[] = {{&S25836[102], 6}, {NULL}};
const Segment SGI141312[] = {{&S25837[44], 32}, {NULL}};
const Segment SGI141313[] = {{&S25837[44], 44}, {NULL}};
const Segment SGI141314[] = {{&S25837[30], 6}, {NULL}};
const Segment SGI141315[] = {{&S25837[137], 3}, {NULL}};
const Segment SGI141316[] = {{&S25837[102], 6}, {NULL}};
const Segment SGI141317[] = {{&S35990[44], 7}, {NULL}};
const Segment SGI141318[] = {{&S35990[44], 24}, {NULL}};
const Segment SGI141319[] = {{&S35990[30], 6}, {NULL}};
const Segment SGI141320[] = {{&S35990[31], 3}, {NULL}};
const Segment SGI141321[] = {{&S35990[82], 6}, {NULL}};
const Segment SGI141322[] = {{&S35991[44], 7}, {NULL}};
const Segment SGI141323[] = {{&S35991[44], 24}, {NULL}};
const Segment SGI141324[] = {{&S35991[30], 6}, {NULL}};
const Segment SGI141325[] = {{&S35991[31], 3}, {NULL}};
const Segment SGI141326[] = {{&S35991[82], 6}, {NULL}};
const Segment SGI141327[] = {{&S35992[44], 7}, {NULL}};
const Segment SGI141328[] = {{&S35992[44], 24}, {NULL}};
const Segment SGI141329[] = {{&S35992[30], 6}, {NULL}};
const Segment SGI141330[] = {{&S35992[31], 3}, {NULL}};
const Segment SGI141331[] = {{&S35992[82], 6}, {NULL}};
const Segment SGI141332[] = {{&S35994[44], 7}, {NULL}};
const Segment SGI141333[] = {{&S35994[44], 24}, {NULL}};
const Segment SGI141334[] = {{&S35994[30], 6}, {NULL}};
const Segment SGI141335[] = {{&S35994[31], 3}, {NULL}};
const Segment SGI141336[] = {{&S35994[82], 6}, {NULL}};
const Segment SGI141337[] = {{&S35995[38], 7}, {NULL}};
const Segment SGI141338[] = {{&S35995[31], 6}, {NULL}};
const Segment SGI141339[] = {{&S35996[42], 7}, {NULL}};
const Segment SGI141340[] = {{&S35996[30], 6}, {NULL}};
const Segment SGI141341[] = {{&S35996[31], 3}, {NULL}};
const Segment SGI141342[] = {{&S35996[80], 6}, {NULL}};
const Segment SGI141343[] = {{&S36027[44], 13}, {NULL}};
const Segment SGI141344[] = {{&S36027[44], 24}, {NULL}};
const Segment SGI141345[] = {{&S36027[30], 6}, {NULL}};
const Segment SGI141346[] = {{&S36027[31], 3}, {NULL}};
const Segment SGI141347[] = {{&S36027[82], 5}, {NULL}};
const Segment SGI141348[] = {{&S36028[44], 13}, {NULL}};
const Segment SGI141349[] = {{&S36028[44], 25}, {NULL}};
const Segment SGI141350[] = {{&S36028[30], 6}, {NULL}};
const Segment SGI141351[] = {{&S36028[31], 3}, {NULL}};
const Segment SGI141352[] = {{&S36028[83], 6}, {NULL}};
const Segment SGI141353[] = {{&S36029[44], 13}, {NULL}};
const Segment SGI141354[] = {{&S36029[44], 25}, {NULL}};
const Segment SGI141355[] = {{&S36029[30], 6}, {NULL}};
const Segment SGI141356[] = {{&S36029[31], 3}, {NULL}};
const Segment SGI141357[] = {{&S36029[83], 6}, {NULL}};
const Segment SGI141358[] = {{&S36030[44], 10}, {NULL}};
const Segment SGI141359[] = {{&S36030[44], 22}, {NULL}};
const Segment SGI141360[] = {{&S36030[30], 6}, {NULL}};
const Segment SGI141361[] = {{&S36030[31], 3}, {NULL}};
const Segment SGI141362[] = {{&S36030[80], 6}, {NULL}};
const Segment SGI141363[] = {{&S36031[44], 10}, {NULL}};
const Segment SGI141364[] = {{&S36031[44], 22}, {NULL}};
const Segment SGI141365[] = {{&S36031[30], 6}, {NULL}};
const Segment SGI141366[] = {{&S36031[31], 3}, {NULL}};
const Segment SGI141367[] = {{&S36031[80], 6}, {NULL}};
const Segment SGI141368[] = {{&S36032[44], 13}, {NULL}};
const Segment SGI141369[] = {{&S36032[44], 25}, {NULL}};
const Segment SGI141370[] = {{&S36032[30], 6}, {NULL}};
const Segment SGI141371[] = {{&S36032[31], 3}, {NULL}};
const Segment SGI141372[] = {{&S36032[83], 6}, {NULL}};
const Segment SGI141373[] = {{&S36033[38], 10}, {NULL}};
const Segment SGI141374[] = {{&S36033[31], 6}, {NULL}};
const Segment SGI141375[] = {{&S36034[38], 13}, {NULL}};
const Segment SGI141376[] = {{&S36034[31], 6}, {NULL}};
const Segment SGI141377[] = {{&S26820[34], 18}, {NULL}};
const Segment SGI141378[] = {{&S26820[27], 6}, {NULL}};
const Segment SGI141379[] = {{&S36036[42], 10}, {NULL}};
const Segment SGI141380[] = {{&S36036[30], 6}, {NULL}};
const Segment SGI141381[] = {{&S26820[105], 9}, {NULL}};
const Segment SGI141382[] = {{&S36036[31], 3}, {NULL}};
const Segment SGI141383[] = {{&S26820[78], 6}, {NULL}};
const Segment SGI141384[] = {{&S36036[78], 6}, {NULL}};
const Segment SGI141385[] = {{&S36037[44], 10}, {NULL}};
const Segment SGI141386[] = {{&S36037[44], 39}, {NULL}};
const Segment SGI141387[] = {{&S36037[30], 6}, {NULL}};
const Segment SGI141388[] = {{&S36037[31], 3}, {NULL}};
const Segment SGI141389[] = {{&S36037[97], 6}, {NULL}};
const Segment SGI141390[] = {{&S36038[44], 13}, {NULL}};
const Segment SGI141391[] = {{&S36038[44], 31}, {NULL}};
const Segment SGI141392[] = {{&S36038[30], 6}, {NULL}};
const Segment SGI141393[] = {{&S36038[31], 3}, {NULL}};
const Segment SGI141394[] = {{&S36038[89], 6}, {NULL}};
const Segment SGI141395[] = {{&S26825[34], 10}, {NULL}};
const Segment SGI141396[] = {{&S26825[27], 6}, {NULL}};
const Segment SGI141397[] = {{&S26825[97], 9}, {NULL}};
const Segment SGI141398[] = {{&S26825[70], 6}, {NULL}};
const Segment SGI141399[] = {{&S26827[34], 10}, {NULL}};
const Segment SGI141400[] = {{&S26827[27], 6}, {NULL}};
const Segment SGI141401[] = {{&S26827[97], 9}, {NULL}};
const Segment SGI141402[] = {{&S26827[70], 6}, {NULL}};
const Segment SGI141403[] = {{&S26830[34], 11}, {NULL}};
const Segment SGI141404[] = {{&S26830[27], 6}, {NULL}};
const Segment SGI141405[] = {{&S26830[98], 9}, {NULL}};
const Segment SGI141406[] = {{&S26830[71], 6}, {NULL}};
const Segment SGI141407[] = {{&S26831[34], 10}, {NULL}};
const Segment SGI141408[] = {{&S26831[27], 6}, {NULL}};
const Segment SGI141409[] = {{&S26831[97], 9}, {NULL}};
const Segment SGI141410[] = {{&S26831[70], 6}, {NULL}};
const Segment SGI141411[] = {{&S26836[34], 11}, {NULL}};
const Segment SGI141412[] = {{&S26836[27], 6}, {NULL}};
const Segment SGI141413[] = {{&S26836[98], 9}, {NULL}};
const Segment SGI141414[] = {{&S26836[71], 6}, {NULL}};
const Segment SGI141415[] = {{&S26837[34], 11}, {NULL}};
const Segment SGI141416[] = {{&S26837[27], 6}, {NULL}};
const Segment SGI141417[] = {{&S26837[98], 9}, {NULL}};
const Segment SGI141418[] = {{&S26837[71], 6}, {NULL}};
const Segment SGI141419[] = {{&S26838[34], 11}, {NULL}};
const Segment SGI141420[] = {{&S26838[27], 6}, {NULL}};
const Segment SGI141421[] = {{&S26838[98], 9}, {NULL}};
const Segment SGI141422[] = {{&S26838[71], 6}, {NULL}};
const Segment SGI141423[] = {{&S36122[34], 10}, {NULL}};
const Segment SGI141424[] = {{&S36122[27], 6}, {NULL}};
const Segment SGI141425[] = {{&S36122[97], 9}, {NULL}};
const Segment SGI141426[] = {{&S36122[70], 6}, {NULL}};
const Segment SGI141427[] = {{&S36123[34], 18}, {NULL}};
const Segment SGI141428[] = {{&S36123[27], 6}, {NULL}};
const Segment SGI141429[] = {{&S36123[105], 9}, {NULL}};
const Segment SGI141430[] = {{&S36123[78], 6}, {NULL}};
const Segment SGI141431[] = {{&S36124[34], 11}, {NULL}};
const Segment SGI141432[] = {{&S36124[27], 6}, {NULL}};
const Segment SGI141433[] = {{&S36124[98], 9}, {NULL}};
const Segment SGI141434[] = {{&S36124[71], 6}, {NULL}};
const Segment SGI141435[] = {{&S36125[34], 11}, {NULL}};
const Segment SGI141436[] = {{&S36125[27], 6}, {NULL}};
const Segment SGI141437[] = {{&S36125[98], 9}, {NULL}};
const Segment SGI141438[] = {{&S36125[71], 6}, {NULL}};
const Segment SGI141439[] = {{&S36126[34], 11}, {NULL}};
const Segment SGI141440[] = {{&S36126[27], 6}, {NULL}};
const Segment SGI141441[] = {{&S36126[98], 9}, {NULL}};
const Segment SGI141442[] = {{&S36126[71], 6}, {NULL}};
const Segment SGI141443[] = {{&S36127[34], 11}, {NULL}};
const Segment SGI141444[] = {{&S36127[27], 6}, {NULL}};
const Segment SGI141445[] = {{&S36127[98], 9}, {NULL}};
const Segment SGI141446[] = {{&S36127[71], 6}, {NULL}};
const Segment SGI141447[] = {{&S36128[34], 11}, {NULL}};
const Segment SGI141448[] = {{&S36128[27], 6}, {NULL}};
const Segment SGI141449[] = {{&S36128[98], 9}, {NULL}};
const Segment SGI141450[] = {{&S36128[71], 6}, {NULL}};
const Segment SGI141451[] = {{&S36129[34], 11}, {NULL}};
const Segment SGI141452[] = {{&S36129[27], 6}, {NULL}};
const Segment SGI141453[] = {{&S36129[98], 9}, {NULL}};
const Segment SGI141454[] = {{&S36129[71], 6}, {NULL}};
const Segment SGI141455[] = {{&S36130[34], 14}, {NULL}};
const Segment SGI141456[] = {{&S36130[27], 6}, {NULL}};
const Segment SGI141457[] = {{&S36130[101], 9}, {NULL}};
const Segment SGI141458[] = {{&S36130[74], 6}, {NULL}};
const Segment SGI141459[] = {{&S36148[44], 7}, {NULL}};
const Segment SGI141460[] = {{&S36148[44], 19}, {NULL}};
const Segment SGI141461[] = {{&S36148[30], 6}, {NULL}};
const Segment SGI141462[] = {{&S36148[31], 3}, {NULL}};
const Segment SGI141463[] = {{&S36148[77], 6}, {NULL}};
const Segment SGI141464[] = {{&S36149[38], 7}, {NULL}};
const Segment SGI141465[] = {{&S36149[31], 6}, {NULL}};
const Segment SGI141466[] = {{&S36150[44], 7}, {NULL}};
const Segment SGI141467[] = {{&S36150[44], 19}, {NULL}};
const Segment SGI141468[] = {{&S36150[30], 6}, {NULL}};
const Segment SGI141469[] = {{&S36150[31], 3}, {NULL}};
const Segment SGI141470[] = {{&S36150[77], 6}, {NULL}};
const Segment SGI141471[] = {{&S36151[42], 7}, {NULL}};
const Segment SGI141472[] = {{&S36151[30], 6}, {NULL}};
const Segment SGI141473[] = {{&S36151[31], 3}, {NULL}};
const Segment SGI141474[] = {{&S36151[75], 6}, {NULL}};
const Segment SGI141475[] = {{&S36152[44], 7}, {NULL}};
const Segment SGI141476[] = {{&S36152[44], 19}, {NULL}};
const Segment SGI141477[] = {{&S36152[30], 6}, {NULL}};
const Segment SGI141478[] = {{&S36152[77], 6}, {NULL}};
const Segment SGI141479[] = {{&S29800[32], 14}, {NULL}};
const Segment SGI141480[] = {{&S29800[27], 4}, {NULL}};
const Segment SGI141481[] = {{&S29800[99], 9}, {NULL}};
const Segment SGI141482[] = {{&S29800[72], 6}, {NULL}};
const Segment SGI141483[] = {{&S29802[32], 15}, {NULL}};
const Segment SGI141484[] = {{&S29802[27], 4}, {NULL}};
const Segment SGI141485[] = {{&S29802[100], 9}, {NULL}};
const Segment SGI141486[] = {{&S29802[73], 6}, {NULL}};
const Segment SGI141487[] = {{&S26056[44], 24}, {NULL}};
const Segment SGI141488[] = {{&S26056[44], 36}, {NULL}};
const Segment SGI141489[] = {{&S26056[30], 6}, {NULL}};
const Segment SGI141490[] = {{&S26056[129], 3}, {NULL}};
const Segment SGI141491[] = {{&S26056[94], 6}, {NULL}};
const Segment SGI141492[] = {{&S29957[32], 7}, {NULL}};
const Segment SGI141493[] = {{&S29957[27], 4}, {NULL}};
const Segment SGI141494[] = {{&S29957[92], 9}, {NULL}};
const Segment SGI141495[] = {{&S29957[65], 6}, {NULL}};
const Segment SGI141496[] = {{&S36737[44], 7}, {NULL}};
const Segment SGI141497[] = {{&S36737[44], 21}, {NULL}};
const Segment SGI141498[] = {{&S36737[30], 6}, {NULL}};
const Segment SGI141499[] = {{&S36737[31], 3}, {NULL}};
const Segment SGI141500[] = {{&S36737[79], 6}, {NULL}};
const Segment SGI141501[] = {{&S36739[44], 7}, {NULL}};
const Segment SGI141502[] = {{&S36739[44], 21}, {NULL}};
const Segment SGI141503[] = {{&S36739[30], 6}, {NULL}};
const Segment SGI141504[] = {{&S36739[31], 3}, {NULL}};
const Segment SGI141505[] = {{&S36739[79], 6}, {NULL}};
const Segment SGI141506[] = {{&S36740[38], 7}, {NULL}};
const Segment SGI141507[] = {{&S36740[31], 6}, {NULL}};
const Segment SGI141508[] = {{&S36810[34], 7}, {NULL}};
const Segment SGI141509[] = {{&S36810[27], 6}, {NULL}};
const Segment SGI141510[] = {{&S36810[110], 9}, {NULL}};
const Segment SGI141511[] = {{&S36810[83], 6}, {NULL}};
const Segment SGI141512[] = {{&S36811[34], 15}, {NULL}};
const Segment SGI141513[] = {{&S36811[27], 6}, {NULL}};
const Segment SGI141514[] = {{&S36811[119], 9}, {NULL}};
const Segment SGI141515[] = {{&S36811[92], 6}, {NULL}};
const Segment SGI141516[] = {{&S27581[34], 11}, {NULL}};
const Segment SGI141517[] = {{&S27581[27], 6}, {NULL}};
const Segment SGI141518[] = {{&S27581[98], 9}, {NULL}};
const Segment SGI141519[] = {{&S27581[71], 6}, {NULL}};
const Segment SGI141520[] = {{&S27583[34], 8}, {NULL}};
const Segment SGI141521[] = {{&S27583[27], 6}, {NULL}};
const Segment SGI141522[] = {{&S27583[95], 9}, {NULL}};
const Segment SGI141523[] = {{&S27583[68], 6}, {NULL}};
const Segment SGI141524[] = {{&S27586[34], 18}, {NULL}};
const Segment SGI141525[] = {{&S27586[27], 6}, {NULL}};
const Segment SGI141526[] = {{&S27586[105], 9}, {NULL}};
const Segment SGI141527[] = {{&S27586[78], 6}, {NULL}};
const Segment SGI141528[] = {{&S27587[34], 10}, {NULL}};
const Segment SGI141529[] = {{&S27587[27], 6}, {NULL}};
const Segment SGI141530[] = {{&S27587[97], 9}, {NULL}};
const Segment SGI141531[] = {{&S27587[70], 6}, {NULL}};
const Segment SGI141532[] = {{&S27588[34], 10}, {NULL}};
const Segment SGI141533[] = {{&S27588[27], 6}, {NULL}};
const Segment SGI141534[] = {{&S27588[97], 9}, {NULL}};
const Segment SGI141535[] = {{&S27588[70], 6}, {NULL}};
const Segment SGI141536[] = {{&S27593[34], 11}, {NULL}};
const Segment SGI141537[] = {{&S27593[27], 6}, {NULL}};
const Segment SGI141538[] = {{&S27593[98], 9}, {NULL}};
const Segment SGI141539[] = {{&S27593[71], 6}, {NULL}};
const Segment SGI141540[] = {{&S27595[34], 10}, {NULL}};
const Segment SGI141541[] = {{&S27595[27], 6}, {NULL}};
const Segment SGI141542[] = {{&S27595[97], 9}, {NULL}};
const Segment SGI141543[] = {{&S27595[70], 6}, {NULL}};
const Segment SGI141544[] = {{&S27597[34], 10}, {NULL}};
const Segment SGI141545[] = {{&S27597[27], 6}, {NULL}};
const Segment SGI141546[] = {{&S27597[97], 9}, {NULL}};
const Segment SGI141547[] = {{&S27597[70], 6}, {NULL}};
const Segment SGI141548[] = {{&S27599[34], 10}, {NULL}};
const Segment SGI141549[] = {{&S27599[27], 6}, {NULL}};
const Segment SGI141550[] = {{&S27599[97], 9}, {NULL}};
const Segment SGI141551[] = {{&S27599[70], 6}, {NULL}};
const Segment SGI141552[] = {{&S27600[34], 11}, {NULL}};
const Segment SGI141553[] = {{&S27600[27], 6}, {NULL}};
const Segment SGI141554[] = {{&S27600[98], 9}, {NULL}};
const Segment SGI141555[] = {{&S27600[71], 6}, {NULL}};
const Segment SGI141556[] = {{&S27602[34], 10}, {NULL}};
const Segment SGI141557[] = {{&S27602[27], 6}, {NULL}};
const Segment SGI141558[] = {{&S26501[38], 9}, {NULL}};
const Segment SGI141559[] = {{&S27602[97], 9}, {NULL}};
const Segment SGI141560[] = {{&S27602[70], 6}, {NULL}};
const Segment SGI141561[] = {{&S26501[31], 6}, {NULL}};
const Segment SGI141562[] = {{&S27603[34], 11}, {NULL}};
const Segment SGI141563[] = {{&S27603[27], 6}, {NULL}};
const Segment SGI141564[] = {{&S26502[44], 9}, {NULL}};
const Segment SGI141565[] = {{&S26502[44], 21}, {NULL}};
const Segment SGI141566[] = {{&S26502[30], 6}, {NULL}};
const Segment SGI141567[] = {{&S27603[98], 9}, {NULL}};
const Segment SGI141568[] = {{&S26502[114], 3}, {NULL}};
const Segment SGI141569[] = {{&S27603[71], 6}, {NULL}};
const Segment SGI141570[] = {{&S26502[79], 6}, {NULL}};
const Segment SGI141571[] = {{&S26503[44], 9}, {NULL}};
const Segment SGI141572[] = {{&S26503[44], 21}, {NULL}};
const Segment SGI141573[] = {{&S26503[30], 6}, {NULL}};
const Segment SGI141574[] = {{&S26503[82], 6}, {NULL}};
const Segment SGI141575[] = {{&S26504[42], 9}, {NULL}};
const Segment SGI141576[] = {{&S26504[30], 6}, {NULL}};
const Segment SGI141577[] = {{&S26504[112], 3}, {NULL}};
const Segment SGI141578[] = {{&S26504[77], 6}, {NULL}};
const Segment SGI141579[] = {{&S27607[34], 11}, {NULL}};
const Segment SGI141580[] = {{&S27607[27], 6}, {NULL}};
const Segment SGI141581[] = {{&S27607[98], 9}, {NULL}};
const Segment SGI141582[] = {{&S27607[71], 6}, {NULL}};
const Segment SGI141583[] = {{&S27609[34], 7}, {NULL}};
const Segment SGI141584[] = {{&S27609[27], 6}, {NULL}};
const Segment SGI141585[] = {{&S27609[94], 9}, {NULL}};
const Segment SGI141586[] = {{&S27609[67], 6}, {NULL}};
const Segment SGI141587[] = {{&S26516[34], 11}, {NULL}};
const Segment SGI141588[] = {{&S26516[27], 6}, {NULL}};
const Segment SGI141589[] = {{&S26516[98], 9}, {NULL}};
const Segment SGI141590[] = {{&S26516[71], 6}, {NULL}};
const Segment SGI141591[] = {{&S26518[34], 10}, {NULL}};
const Segment SGI141592[] = {{&S26518[27], 6}, {NULL}};
const Segment SGI141593[] = {{&S26518[97], 9}, {NULL}};
const Segment SGI141594[] = {{&S26518[70], 6}, {NULL}};
const Segment SGI141595[] = {{&S26520[34], 10}, {NULL}};
const Segment SGI141596[] = {{&S26520[27], 6}, {NULL}};
const Segment SGI141597[] = {{&S26520[97], 9}, {NULL}};
const Segment SGI141598[] = {{&S26520[70], 6}, {NULL}};
const Segment SGI141599[] = {{&S26521[34], 11}, {NULL}};
const Segment SGI141600[] = {{&S26521[27], 6}, {NULL}};
const Segment SGI141601[] = {{&S26521[98], 9}, {NULL}};
const Segment SGI141602[] = {{&S26521[71], 6}, {NULL}};
const Segment SGI141603[] = {{&S26522[34], 10}, {NULL}};
const Segment SGI141604[] = {{&S26522[27], 6}, {NULL}};
const Segment SGI141605[] = {{&S26522[97], 9}, {NULL}};
const Segment SGI141606[] = {{&S26522[70], 6}, {NULL}};
const Segment SGI141607[] = {{&S26523[34], 11}, {NULL}};
const Segment SGI141608[] = {{&S26523[27], 6}, {NULL}};
const Segment SGI141609[] = {{&S26523[98], 9}, {NULL}};
const Segment SGI141610[] = {{&S26523[71], 6}, {NULL}};
const Segment SGI141611[] = {{&S26524[34], 11}, {NULL}};
const Segment SGI141612[] = {{&S26524[27], 6}, {NULL}};
const Segment SGI141613[] = {{&S26524[98], 9}, {NULL}};
const Segment SGI141614[] = {{&S26524[71], 6}, {NULL}};
const Segment SGI141615[] = {{&S26525[34], 11}, {NULL}};
const Segment SGI141616[] = {{&S26525[27], 6}, {NULL}};
const Segment SGI141617[] = {{&S26525[98], 9}, {NULL}};
const Segment SGI141618[] = {{&S26525[71], 6}, {NULL}};
const Segment SGI141619[] = {{&S26527[34], 11}, {NULL}};
const Segment SGI141620[] = {{&S26527[27], 6}, {NULL}};
const Segment SGI141621[] = {{&S26527[98], 9}, {NULL}};
const Segment SGI141622[] = {{&S26527[71], 6}, {NULL}};
const Segment SGI141623[] = {{&S26529[34], 18}, {NULL}};
const Segment SGI141624[] = {{&S26529[27], 6}, {NULL}};
const Segment SGI141625[] = {{&S26529[105], 9}, {NULL}};
const Segment SGI141626[] = {{&S26529[78], 6}, {NULL}};
const Segment SGI141627[] = {{&S26530[34], 11}, {NULL}};
const Segment SGI141628[] = {{&S26530[27], 6}, {NULL}};
const Segment SGI141629[] = {{&S26530[98], 9}, {NULL}};
const Segment SGI141630[] = {{&S26530[71], 6}, {NULL}};
const Segment SGI141631[] = {{&S26531[34], 11}, {NULL}};
const Segment SGI141632[] = {{&S26531[27], 6}, {NULL}};
const Segment SGI141633[] = {{&S26531[98], 9}, {NULL}};
const Segment SGI141634[] = {{&S26531[71], 6}, {NULL}};
const Segment SGI141635[] = {{&S27698[44], 12}, {NULL}};
const Segment SGI141636[] = {{&S27698[44], 24}, {NULL}};
const Segment SGI141637[] = {{&S27698[30], 6}, {NULL}};
const Segment SGI141638[] = {{&S27698[117], 3}, {NULL}};
const Segment SGI141639[] = {{&S27698[82], 6}, {NULL}};
const Segment SGI141640[] = {{&S27699[38], 12}, {NULL}};
const Segment SGI141641[] = {{&S27699[31], 6}, {NULL}};
const Segment SGI141642[] = {{&S27700[44], 14}, {NULL}};
const Segment SGI141643[] = {{&S27700[44], 26}, {NULL}};
const Segment SGI141644[] = {{&S27700[30], 6}, {NULL}};
const Segment SGI141645[] = {{&S27700[119], 3}, {NULL}};
const Segment SGI141646[] = {{&S27700[84], 6}, {NULL}};
const Segment SGI141647[] = {{&S27701[44], 20}, {NULL}};
const Segment SGI141648[] = {{&S27701[44], 32}, {NULL}};
const Segment SGI141649[] = {{&S27701[30], 6}, {NULL}};
const Segment SGI141650[] = {{&S27701[125], 3}, {NULL}};
const Segment SGI141651[] = {{&S27701[90], 6}, {NULL}};
const Segment SGI141652[] = {{&S27702[44], 26}, {NULL}};
const Segment SGI141653[] = {{&S27702[44], 38}, {NULL}};
const Segment SGI141654[] = {{&S27702[30], 6}, {NULL}};
const Segment SGI141655[] = {{&S27702[131], 3}, {NULL}};
const Segment SGI141656[] = {{&S27702[96], 6}, {NULL}};
const Segment SGI141657[] = {{&S27703[44], 26}, {NULL}};
const Segment SGI141658[] = {{&S27703[44], 38}, {NULL}};
const Segment SGI141659[] = {{&S27703[30], 6}, {NULL}};
const Segment SGI141660[] = {{&S27703[131], 3}, {NULL}};
const Segment SGI141661[] = {{&S27703[96], 6}, {NULL}};
const Segment SGI141662[] = {{&S27704[44], 7}, {NULL}};
const Segment SGI141663[] = {{&S27704[44], 19}, {NULL}};
const Segment SGI141664[] = {{&S27704[30], 6}, {NULL}};
const Segment SGI141665[] = {{&S27704[112], 3}, {NULL}};
const Segment SGI141666[] = {{&S27704[77], 6}, {NULL}};
const Segment SGI141667[] = {{&S27705[44], 14}, {NULL}};
const Segment SGI141668[] = {{&S27705[44], 26}, {NULL}};
const Segment SGI141669[] = {{&S27705[30], 6}, {NULL}};
const Segment SGI141670[] = {{&S27705[119], 3}, {NULL}};
const Segment SGI141671[] = {{&S27705[84], 6}, {NULL}};
const Segment SGI141672[] = {{&S27706[44], 25}, {NULL}};
const Segment SGI141673[] = {{&S27706[44], 37}, {NULL}};
const Segment SGI141674[] = {{&S27706[30], 6}, {NULL}};
const Segment SGI141675[] = {{&S27706[130], 3}, {NULL}};
const Segment SGI141676[] = {{&S27706[95], 6}, {NULL}};
const Segment SGI141677[] = {{&S27707[44], 20}, {NULL}};
const Segment SGI141678[] = {{&S27707[44], 32}, {NULL}};
const Segment SGI141679[] = {{&S27707[30], 6}, {NULL}};
const Segment SGI141680[] = {{&S27707[125], 3}, {NULL}};
const Segment SGI141681[] = {{&S27707[90], 6}, {NULL}};
const Segment SGI141682[] = {{&S27708[44], 13}, {NULL}};
const Segment SGI141683[] = {{&S27708[44], 25}, {NULL}};
const Segment SGI141684[] = {{&S27708[30], 6}, {NULL}};
const Segment SGI141685[] = {{&S27708[117], 3}, {NULL}};
const Segment SGI141686[] = {{&S27708[83], 6}, {NULL}};
const Segment SGI141687[] = {{&S27709[44], 20}, {NULL}};
const Segment SGI141688[] = {{&S27709[44], 32}, {NULL}};
const Segment SGI141689[] = {{&S27709[30], 6}, {NULL}};
const Segment SGI141690[] = {{&S27709[125], 3}, {NULL}};
const Segment SGI141691[] = {{&S27709[90], 6}, {NULL}};
const Segment SGI141692[] = {{&S27710[38], 14}, {NULL}};
const Segment SGI141693[] = {{&S27710[31], 6}, {NULL}};
const Segment SGI141694[] = {{&S27711[44], 25}, {NULL}};
const Segment SGI141695[] = {{&S27711[44], 37}, {NULL}};
const Segment SGI141696[] = {{&S27711[30], 6}, {NULL}};
const Segment SGI141697[] = {{&S27711[130], 3}, {NULL}};
const Segment SGI141698[] = {{&S27711[95], 6}, {NULL}};
const Segment SGI141699[] = {{&S27712[42], 12}, {NULL}};
const Segment SGI141700[] = {{&S27712[30], 6}, {NULL}};
const Segment SGI141701[] = {{&S27712[115], 3}, {NULL}};
const Segment SGI141702[] = {{&S27712[80], 6}, {NULL}};
const Segment SGI141703[] = {{&S27713[44], 14}, {NULL}};
const Segment SGI141704[] = {{&S27713[44], 26}, {NULL}};
const Segment SGI141705[] = {{&S27713[30], 6}, {NULL}};
const Segment SGI141706[] = {{&S27713[119], 3}, {NULL}};
const Segment SGI141707[] = {{&S27713[84], 6}, {NULL}};
const Segment SGI141708[] = {{&S27714[44], 12}, {NULL}};
const Segment SGI141709[] = {{&S27714[44], 35}, {NULL}};
const Segment SGI141710[] = {{&S27714[30], 6}, {NULL}};
const Segment SGI141711[] = {{&S27714[128], 3}, {NULL}};
const Segment SGI141712[] = {{&S27714[93], 6}, {NULL}};
const Segment SGI141713[] = {{&S27715[44], 20}, {NULL}};
const Segment SGI141714[] = {{&S27715[44], 43}, {NULL}};
const Segment SGI141715[] = {{&S27715[30], 6}, {NULL}};
const Segment SGI141716[] = {{&S37028[44], 7}, {NULL}};
const Segment SGI141717[] = {{&S37028[44], 36}, {NULL}};
const Segment SGI141718[] = {{&S37028[30], 6}, {NULL}};
const Segment SGI141719[] = {{&S27715[136], 3}, {NULL}};
const Segment SGI141720[] = {{&S27715[101], 6}, {NULL}};
const Segment SGI141721[] = {{&S37028[31], 3}, {NULL}};
const Segment SGI141722[] = {{&S37028[94], 6}, {NULL}};
const Segment SGI141723[] = {{&S37029[44], 7}, {NULL}};
const Segment SGI141724[] = {{&S37029[44], 41}, {NULL}};
const Segment SGI141725[] = {{&S37029[30], 6}, {NULL}};
const Segment SGI141726[] = {{&S37029[31], 3}, {NULL}};
const Segment SGI141727[] = {{&S37029[99], 6}, {NULL}};
const Segment SGI141728[] = {{&S37030[44], 7}, {NULL}};
const Segment SGI141729[] = {{&S37030[44], 36}, {NULL}};
const Segment SGI141730[] = {{&S37030[30], 6}, {NULL}};
const Segment SGI141731[] = {{&S37030[31], 3}, {NULL}};
const Segment SGI141732[] = {{&S37030[94], 6}, {NULL}};
const Segment SGI141733[] = {{&S37031[44], 7}, {NULL}};
const Segment SGI141734[] = {{&S37031[44], 36}, {NULL}};
const Segment SGI141735[] = {{&S37031[30], 6}, {NULL}};
const Segment SGI141736[] = {{&S37031[31], 3}, {NULL}};
const Segment SGI141737[] = {{&S37031[94], 6}, {NULL}};
const Segment SGI141738[] = {{&S37032[38], 7}, {NULL}};
const Segment SGI141739[] = {{&S37032[31], 6}, {NULL}};
const Segment SGI141740[] = {{&S37033[44], 7}, {NULL}};
const Segment SGI141741[] = {{&S37033[44], 41}, {NULL}};
const Segment SGI141742[] = {{&S37033[30], 6}, {NULL}};
const Segment SGI141743[] = {{&S37033[31], 3}, {NULL}};
const Segment SGI141744[] = {{&S37033[99], 6}, {NULL}};
const Segment SGI141745[] = {{&S37034[44], 7}, {NULL}};
const Segment SGI141746[] = {{&S37034[44], 35}, {NULL}};
const Segment SGI141747[] = {{&S37034[30], 6}, {NULL}};
const Segment SGI141748[] = {{&S37034[31], 3}, {NULL}};
const Segment SGI141749[] = {{&S37034[93], 6}, {NULL}};
const Segment SGI141750[] = {{&S37035[37], 15}, {NULL}};
const Segment SGI141751[] = {{&S37065[44], 6}, {NULL}};
const Segment SGI141752[] = {{&S37065[44], 18}, {NULL}};
const Segment SGI141753[] = {{&S37065[30], 6}, {NULL}};
const Segment SGI141754[] = {{&S37065[31], 3}, {NULL}};
const Segment SGI141755[] = {{&S37065[76], 6}, {NULL}};
const Segment SGI141756[] = {{&S37066[44], 28}, {NULL}};
const Segment SGI141757[] = {{&S37066[44], 40}, {NULL}};
const Segment SGI141758[] = {{&S37066[30], 6}, {NULL}};
const Segment SGI141759[] = {{&S37066[31], 3}, {NULL}};
const Segment SGI141760[] = {{&S37066[98], 6}, {NULL}};
const Segment SGI141761[] = {{&S37069[38], 6}, {NULL}};
const Segment SGI141762[] = {{&S37069[31], 6}, {NULL}};
const Segment SGI141763[] = {{&S37154[44], 6}, {NULL}};
const Segment SGI141764[] = {{&S37154[44], 26}, {NULL}};
const Segment SGI141765[] = {{&S37154[30], 6}, {NULL}};
const Segment SGI141766[] = {{&S37154[31], 3}, {NULL}};
const Segment SGI141767[] = {{&S37154[84], 6}, {NULL}};
const Segment SGI141768[] = {{&S37156[44], 18}, {NULL}};
const Segment SGI141769[] = {{&S37156[44], 38}, {NULL}};
const Segment SGI141770[] = {{&S37156[30], 6}, {NULL}};
const Segment SGI141771[] = {{&S37156[31], 3}, {NULL}};
const Segment SGI141772[] = {{&S37156[96], 6}, {NULL}};
const Segment SGI141773[] = {{&S26819[34], 10}, {NULL}};
const Segment SGI141774[] = {{&S26819[27], 6}, {NULL}};
const Segment SGI141775[] = {{&S26819[97], 9}, {NULL}};
const Segment SGI141776[] = {{&S26819[70], 6}, {NULL}};
const Segment SGI141777[] = {{&S26821[34], 10}, {NULL}};
const Segment SGI141778[] = {{&S26821[27], 6}, {NULL}};
const Segment SGI141779[] = {{&S26821[97], 9}, {NULL}};
const Segment SGI141780[] = {{&S26821[70], 6}, {NULL}};
const Segment SGI141781[] = {{&S26822[34], 11}, {NULL}};
const Segment SGI141782[] = {{&S26822[27], 6}, {NULL}};
const Segment SGI141783[] = {{&S26822[98], 9}, {NULL}};
const Segment SGI141784[] = {{&S26822[71], 6}, {NULL}};
const Segment SGI141785[] = {{&S26823[34], 11}, {NULL}};
const Segment SGI141786[] = {{&S26823[27], 6}, {NULL}};
const Segment SGI141787[] = {{&S26823[98], 9}, {NULL}};
const Segment SGI141788[] = {{&S26823[71], 6}, {NULL}};
const Segment SGI141789[] = {{&S26824[34], 18}, {NULL}};
const Segment SGI141790[] = {{&S26824[27], 6}, {NULL}};
const Segment SGI141791[] = {{&S26824[105], 9}, {NULL}};
const Segment SGI141792[] = {{&S26824[78], 6}, {NULL}};
const Segment SGI141793[] = {{&S26826[34], 11}, {NULL}};
const Segment SGI141794[] = {{&S26826[27], 6}, {NULL}};
const Segment SGI141795[] = {{&S26826[98], 9}, {NULL}};
const Segment SGI141796[] = {{&S26826[71], 6}, {NULL}};
const Segment SGI141797[] = {{&S26828[34], 11}, {NULL}};
const Segment SGI141798[] = {{&S26828[27], 6}, {NULL}};
const Segment SGI141799[] = {{&S26828[98], 9}, {NULL}};
const Segment SGI141800[] = {{&S26828[71], 6}, {NULL}};
const Segment SGI141801[] = {{&S26829[34], 11}, {NULL}};
const Segment SGI141802[] = {{&S26829[27], 6}, {NULL}};
const Segment SGI141803[] = {{&S26829[98], 9}, {NULL}};
const Segment SGI141804[] = {{&S26829[71], 6}, {NULL}};
const Segment SGI141805[] = {{&S37224[44], 7}, {NULL}};
const Segment SGI141806[] = {{&S37224[44], 18}, {NULL}};
const Segment SGI141807[] = {{&S37224[30], 6}, {NULL}};
const Segment SGI141808[] = {{&S37224[31], 3}, {NULL}};
const Segment SGI141809[] = {{&S37224[76], 6}, {NULL}};
const Segment SGI141810[] = {{&S26832[34], 11}, {NULL}};
const Segment SGI141811[] = {{&S26832[27], 6}, {NULL}};
const Segment SGI141812[] = {{&S26832[98], 9}, {NULL}};
const Segment SGI141813[] = {{&S26832[71], 6}, {NULL}};
const Segment SGI141814[] = {{&S37225[38], 7}, {NULL}};
const Segment SGI141815[] = {{&S37225[31], 6}, {NULL}};
const Segment SGI141816[] = {{&S26833[34], 11}, {NULL}};
const Segment SGI141817[] = {{&S26833[27], 6}, {NULL}};
const Segment SGI141818[] = {{&S37226[44], 6}, {NULL}};
const Segment SGI141819[] = {{&S26833[98], 9}, {NULL}};
const Segment SGI141820[] = {{&S37226[44], 18}, {NULL}};
const Segment SGI141821[] = {{&S26833[71], 6}, {NULL}};
const Segment SGI141822[] = {{&S37226[30], 6}, {NULL}};
const Segment SGI141823[] = {{&S37226[31], 3}, {NULL}};
const Segment SGI141824[] = {{&S37226[76], 6}, {NULL}};
const Segment SGI141825[] = {{&S26834[34], 11}, {NULL}};
const Segment SGI141826[] = {{&S26834[27], 6}, {NULL}};
const Segment SGI141827[] = {{&S37227[38], 6}, {NULL}};
const Segment SGI141828[] = {{&S37227[31], 6}, {NULL}};
const Segment SGI141829[] = {{&S26834[98], 9}, {NULL}};
const Segment SGI141830[] = {{&S26834[71], 6}, {NULL}};
const Segment SGI141831[] = {{&S26835[34], 10}, {NULL}};
const Segment SGI141832[] = {{&S26835[27], 6}, {NULL}};
const Segment SGI141833[] = {{&S26835[97], 9}, {NULL}};
const Segment SGI141834[] = {{&S26835[70], 6}, {NULL}};
const Segment SGI141835[] = {{&S26918[44], 14}, {NULL}};
const Segment SGI141836[] = {{&S26918[44], 26}, {NULL}};
const Segment SGI141837[] = {{&S26918[30], 6}, {NULL}};
const Segment SGI141838[] = {{&S26918[119], 3}, {NULL}};
const Segment SGI141839[] = {{&S26918[84], 6}, {NULL}};
const Segment SGI141840[] = {{&S26919[44], 17}, {NULL}};
const Segment SGI141841[] = {{&S26919[44], 29}, {NULL}};
const Segment SGI141842[] = {{&S26919[30], 6}, {NULL}};
const Segment SGI141843[] = {{&S26919[122], 3}, {NULL}};
const Segment SGI141844[] = {{&S26919[87], 6}, {NULL}};
const Segment SGI141845[] = {{&S26920[44], 9}, {NULL}};
const Segment SGI141846[] = {{&S26920[44], 21}, {NULL}};
const Segment SGI141847[] = {{&S26920[30], 6}, {NULL}};
const Segment SGI141848[] = {{&S26920[114], 3}, {NULL}};
const Segment SGI141849[] = {{&S26920[79], 6}, {NULL}};
const Segment SGI141850[] = {{&S26921[44], 9}, {NULL}};
const Segment SGI141851[] = {{&S26921[44], 21}, {NULL}};
const Segment SGI141852[] = {{&S26921[30], 6}, {NULL}};
const Segment SGI141853[] = {{&S26921[114], 3}, {NULL}};
const Segment SGI141854[] = {{&S26921[79], 6}, {NULL}};
const Segment SGI141855[] = {{&S26922[44], 9}, {NULL}};
const Segment SGI141856[] = {{&S26922[44], 21}, {NULL}};
const Segment SGI141857[] = {{&S26922[30], 6}, {NULL}};
const Segment SGI141858[] = {{&S26922[114], 3}, {NULL}};
const Segment SGI141859[] = {{&S26922[79], 6}, {NULL}};
const Segment SGI141860[] = {{&S26923[38], 9}, {NULL}};
const Segment SGI141861[] = {{&S26923[31], 6}, {NULL}};
const Segment SGI141862[] = {{&S26924[44], 17}, {NULL}};
const Segment SGI141863[] = {{&S26924[44], 29}, {NULL}};
const Segment SGI141864[] = {{&S26924[30], 6}, {NULL}};
const Segment SGI141865[] = {{&S26924[122], 3}, {NULL}};
const Segment SGI141866[] = {{&S26924[87], 6}, {NULL}};
const Segment SGI141867[] = {{&S26925[44], 9}, {NULL}};
const Segment SGI141868[] = {{&S26925[44], 21}, {NULL}};
const Segment SGI141869[] = {{&S26925[30], 6}, {NULL}};
const Segment SGI141870[] = {{&S26925[114], 3}, {NULL}};
const Segment SGI141871[] = {{&S26925[79], 6}, {NULL}};
const Segment SGI141872[] = {{&S26926[44], 14}, {NULL}};
const Segment SGI141873[] = {{&S26926[44], 26}, {NULL}};
const Segment SGI141874[] = {{&S26926[30], 6}, {NULL}};
const Segment SGI141875[] = {{&S26926[119], 3}, {NULL}};
const Segment SGI141876[] = {{&S26926[84], 6}, {NULL}};
const Segment SGI141877[] = {{&S26927[38], 9}, {NULL}};
const Segment SGI141878[] = {{&S26927[31], 6}, {NULL}};
const Segment SGI141879[] = {{&S26928[44], 16}, {NULL}};
const Segment SGI141880[] = {{&S26928[44], 28}, {NULL}};
const Segment SGI141881[] = {{&S26928[30], 6}, {NULL}};
const Segment SGI141882[] = {{&S26928[121], 3}, {NULL}};
const Segment SGI141883[] = {{&S26928[86], 6}, {NULL}};
const Segment SGI141884[] = {{&S26929[44], 17}, {NULL}};
const Segment SGI141885[] = {{&S26929[44], 29}, {NULL}};
const Segment SGI141886[] = {{&S26929[30], 6}, {NULL}};
const Segment SGI141887[] = {{&S26929[122], 3}, {NULL}};
const Segment SGI141888[] = {{&S26929[87], 6}, {NULL}};
const Segment SGI141889[] = {{&S26930[44], 9}, {NULL}};
const Segment SGI141890[] = {{&S26930[44], 21}, {NULL}};
const Segment SGI141891[] = {{&S26930[30], 6}, {NULL}};
const Segment SGI141892[] = {{&S26930[114], 3}, {NULL}};
const Segment SGI141893[] = {{&S26930[79], 6}, {NULL}};
const Segment SGI141894[] = {{&S26931[44], 9}, {NULL}};
const Segment SGI141895[] = {{&S26931[44], 21}, {NULL}};
const Segment SGI141896[] = {{&S26931[30], 6}, {NULL}};
const Segment SGI141897[] = {{&S26931[114], 3}, {NULL}};
const Segment SGI141898[] = {{&S26931[79], 6}, {NULL}};
const Segment SGI141899[] = {{&S26932[38], 9}, {NULL}};
const Segment SGI141900[] = {{&S26932[31], 6}, {NULL}};
const Segment SGI141901[] = {{&S26933[41], 9}, {NULL}};
const Segment SGI141902[] = {{&S26933[30], 6}, {NULL}};
const Segment SGI141903[] = {{&S26933[111], 3}, {NULL}};
const Segment SGI141904[] = {{&S26933[76], 6}, {NULL}};
const Segment SGI141905[] = {{&S26934[38], 9}, {NULL}};
const Segment SGI141906[] = {{&S26934[31], 6}, {NULL}};
const Segment SGI141907[] = {{&S26935[44], 14}, {NULL}};
const Segment SGI141908[] = {{&S26935[44], 26}, {NULL}};
const Segment SGI141909[] = {{&S26935[30], 6}, {NULL}};
const Segment SGI141910[] = {{&S26935[119], 3}, {NULL}};
const Segment SGI141911[] = {{&S26935[84], 6}, {NULL}};
const Segment SGI141912[] = {{&S26936[44], 17}, {NULL}};
const Segment SGI141913[] = {{&S26936[44], 29}, {NULL}};
const Segment SGI141914[] = {{&S26936[30], 6}, {NULL}};
const Segment SGI141915[] = {{&S26936[122], 3}, {NULL}};
const Segment SGI141916[] = {{&S26936[87], 6}, {NULL}};
const Segment SGI141917[] = {{&S26937[44], 17}, {NULL}};
const Segment SGI141918[] = {{&S26937[44], 29}, {NULL}};
const Segment SGI141919[] = {{&S26937[30], 6}, {NULL}};
const Segment SGI141920[] = {{&S26937[122], 3}, {NULL}};
const Segment SGI141921[] = {{&S26937[87], 6}, {NULL}};
const Segment SGI141922[] = {{&S26938[44], 17}, {NULL}};
const Segment SGI141923[] = {{&S26938[44], 29}, {NULL}};
const Segment SGI141924[] = {{&S26938[30], 6}, {NULL}};
const Segment SGI141925[] = {{&S26938[122], 3}, {NULL}};
const Segment SGI141926[] = {{&S26938[87], 6}, {NULL}};
const Segment SGI141927[] = {{&S37445[44], 7}, {NULL}};
const Segment SGI141928[] = {{&S37445[44], 24}, {NULL}};
const Segment SGI141929[] = {{&S37445[30], 6}, {NULL}};
const Segment SGI141930[] = {{&S37445[31], 3}, {NULL}};
const Segment SGI141931[] = {{&S37445[82], 6}, {NULL}};
const Segment SGI141932[] = {{&S37448[38], 7}, {NULL}};
const Segment SGI141933[] = {{&S37448[31], 6}, {NULL}};
const Segment SGI141934[] = {{&S37449[44], 7}, {NULL}};
const Segment SGI141935[] = {{&S37449[44], 24}, {NULL}};
const Segment SGI141936[] = {{&S37449[30], 6}, {NULL}};
const Segment SGI141937[] = {{&S37449[85], 6}, {NULL}};
const Segment SGI141938[] = {{&S37450[44], 7}, {NULL}};
const Segment SGI141939[] = {{&S37450[44], 24}, {NULL}};
const Segment SGI141940[] = {{&S37450[30], 6}, {NULL}};
const Segment SGI141941[] = {{&S37450[31], 3}, {NULL}};
const Segment SGI141942[] = {{&S37450[82], 6}, {NULL}};
const Segment SGI141943[] = {{&S37451[44], 7}, {NULL}};
const Segment SGI141944[] = {{&S37451[44], 23}, {NULL}};
const Segment SGI141945[] = {{&S37451[30], 6}, {NULL}};
const Segment SGI141946[] = {{&S37451[31], 3}, {NULL}};
const Segment SGI141947[] = {{&S37451[81], 6}, {NULL}};
const Segment SGI141948[] = {{&S37584[44], 17}, {NULL}};
const Segment SGI141949[] = {{&S37584[44], 29}, {NULL}};
const Segment SGI141950[] = {{&S37584[30], 6}, {NULL}};
const Segment SGI141951[] = {{&S37584[31], 3}, {NULL}};
const Segment SGI141952[] = {{&S37584[87], 6}, {NULL}};
const Segment SGI141953[] = {{&S37585[44], 9}, {NULL}};
const Segment SGI141954[] = {{&S37585[44], 21}, {NULL}};
const Segment SGI141955[] = {{&S37585[30], 6}, {NULL}};
const Segment SGI141956[] = {{&S37585[31], 3}, {NULL}};
const Segment SGI141957[] = {{&S37585[79], 6}, {NULL}};
const Segment SGI141958[] = {{&S37586[44], 17}, {NULL}};
const Segment SGI141959[] = {{&S37586[44], 29}, {NULL}};
const Segment SGI141960[] = {{&S37586[30], 6}, {NULL}};
const Segment SGI141961[] = {{&S37586[31], 3}, {NULL}};
const Segment SGI141962[] = {{&S37586[87], 6}, {NULL}};
const Segment SGI141963[] = {{&S37587[44], 17}, {NULL}};
const Segment SGI141964[] = {{&S37587[44], 29}, {NULL}};
const Segment SGI141965[] = {{&S37587[30], 6}, {NULL}};
const Segment SGI141966[] = {{&S37587[31], 3}, {NULL}};
const Segment SGI141967[] = {{&S37587[87], 6}, {NULL}};
const Segment SGI141968[] = {{&S37588[38], 9}, {NULL}};
const Segment SGI141969[] = {{&S37588[31], 6}, {NULL}};
const Segment SGI141970[] = {{&S37589[44], 17}, {NULL}};
const Segment SGI141971[] = {{&S37589[44], 29}, {NULL}};
const Segment SGI141972[] = {{&S37589[30], 6}, {NULL}};
const Segment SGI141973[] = {{&S37589[31], 3}, {NULL}};
const Segment SGI141974[] = {{&S37589[87], 6}, {NULL}};
const Segment SGI141975[] = {{&S37619[44], 7}, {NULL}};
const Segment SGI141976[] = {{&S37619[44], 22}, {NULL}};
const Segment SGI141977[] = {{&S37619[30], 6}, {NULL}};
const Segment SGI141978[] = {{&S37619[31], 3}, {NULL}};
const Segment SGI141979[] = {{&S37619[80], 6}, {NULL}};
const Segment SGI141980[] = {{&S37620[44], 7}, {NULL}};
const Segment SGI141981[] = {{&S37620[44], 24}, {NULL}};
const Segment SGI141982[] = {{&S37620[30], 6}, {NULL}};
const Segment SGI141983[] = {{&S37620[31], 3}, {NULL}};
const Segment SGI141984[] = {{&S37620[82], 6}, {NULL}};
const Segment SGI141985[] = {{&S37621[38], 7}, {NULL}};
const Segment SGI141986[] = {{&S37621[31], 6}, {NULL}};
const Segment SGI141987[] = {{&S37622[44], 7}, {NULL}};
const Segment SGI141988[] = {{&S37622[44], 24}, {NULL}};
const Segment SGI141989[] = {{&S37622[30], 6}, {NULL}};
const Segment SGI141990[] = {{&S37622[31], 3}, {NULL}};
const Segment SGI141991[] = {{&S37622[82], 6}, {NULL}};
const Segment SGI141992[] = {{&S31146[42], 6}, {NULL}};
const Segment SGI141993[] = {{&S31146[42], 18}, {NULL}};
const Segment SGI141994[] = {{&S31146[30], 4}, {NULL}};
const Segment SGI141995[] = {{&S31146[109], 3}, {NULL}};
const Segment SGI141996[] = {{&S31146[74], 6}, {NULL}};
const Segment SGI141997[] = {{&S31147[42], 6}, {NULL}};
const Segment SGI141998[] = {{&S31147[42], 18}, {NULL}};
const Segment SGI141999[] = {{&S31147[30], 4}, {NULL}};
const Segment SGI142000[] = {{&S31147[109], 3}, {NULL}};
const Segment SGI142001[] = {{&S31147[74], 6}, {NULL}};
const Segment SGI142002[] = {{&S31148[42], 6}, {NULL}};
const Segment SGI142003[] = {{&S31148[42], 18}, {NULL}};
const Segment SGI142004[] = {{&S31148[30], 4}, {NULL}};
const Segment SGI142005[] = {{&S31148[109], 3}, {NULL}};
const Segment SGI142006[] = {{&S31148[74], 6}, {NULL}};
const Segment SGI142007[] = {{&S31149[42], 6}, {NULL}};
const Segment SGI142008[] = {{&S31149[42], 18}, {NULL}};
const Segment SGI142009[] = {{&S31149[30], 4}, {NULL}};
const Segment SGI142010[] = {{&S31149[109], 3}, {NULL}};
const Segment SGI142011[] = {{&S31149[74], 6}, {NULL}};
const Segment SGI142012[] = {{&S31150[42], 6}, {NULL}};
const Segment SGI142013[] = {{&S31150[42], 18}, {NULL}};
const Segment SGI142014[] = {{&S31150[30], 4}, {NULL}};
const Segment SGI142015[] = {{&S31150[109], 3}, {NULL}};
const Segment SGI142016[] = {{&S31150[74], 6}, {NULL}};
const Segment SGI142017[] = {{&S31151[42], 6}, {NULL}};
const Segment SGI142018[] = {{&S31151[42], 17}, {NULL}};
const Segment SGI142019[] = {{&S31151[30], 4}, {NULL}};
const Segment SGI142020[] = {{&S31151[108], 3}, {NULL}};
const Segment SGI142021[] = {{&S31151[73], 6}, {NULL}};
const Segment SGI142022[] = {{&S31152[42], 6}, {NULL}};
const Segment SGI142023[] = {{&S31152[42], 18}, {NULL}};
const Segment SGI142024[] = {{&S31152[30], 4}, {NULL}};
const Segment SGI142025[] = {{&S31152[109], 3}, {NULL}};
const Segment SGI142026[] = {{&S31152[74], 6}, {NULL}};
const Segment SGI142027[] = {{&S31161[42], 6}, {NULL}};
const Segment SGI142028[] = {{&S31161[42], 17}, {NULL}};
const Segment SGI142029[] = {{&S31161[30], 4}, {NULL}};
const Segment SGI142030[] = {{&S31161[108], 3}, {NULL}};
const Segment SGI142031[] = {{&S31161[73], 6}, {NULL}};
const Segment SGI142032[] = {{&S31165[42], 6}, {NULL}};
const Segment SGI142033[] = {{&S31165[42], 17}, {NULL}};
const Segment SGI142034[] = {{&S31165[30], 4}, {NULL}};
const Segment SGI142035[] = {{&S31165[31], 3}, {NULL}};
const Segment SGI142036[] = {{&S31165[73], 6}, {NULL}};
const Segment SGI142037[] = {{&S37755[44], 12}, {NULL}};
const Segment SGI142038[] = {{&S37755[44], 24}, {NULL}};
const Segment SGI142039[] = {{&S37755[30], 6}, {NULL}};
const Segment SGI142040[] = {{&S37755[31], 3}, {NULL}};
const Segment SGI142041[] = {{&S37755[82], 6}, {NULL}};
const Segment SGI142042[] = {{&S37756[44], 12}, {NULL}};
const Segment SGI142043[] = {{&S37756[44], 24}, {NULL}};
const Segment SGI142044[] = {{&S37756[30], 6}, {NULL}};
const Segment SGI142045[] = {{&S37756[82], 6}, {NULL}};
const Segment SGI142046[] = {{&S37758[44], 12}, {NULL}};
const Segment SGI142047[] = {{&S37758[44], 24}, {NULL}};
const Segment SGI142048[] = {{&S37758[30], 6}, {NULL}};
const Segment SGI142049[] = {{&S37758[31], 3}, {NULL}};
const Segment SGI142050[] = {{&S37758[82], 6}, {NULL}};
const Segment SGI142051[] = {{&S37763[38], 12}, {NULL}};
const Segment SGI142052[] = {{&S37763[31], 6}, {NULL}};
const Segment SGI142053[] = {{&S37766[44], 8}, {NULL}};
const Segment SGI142054[] = {{&S37766[44], 20}, {NULL}};
const Segment SGI142055[] = {{&S37766[30], 6}, {NULL}};
const Segment SGI142056[] = {{&S37766[31], 3}, {NULL}};
const Segment SGI142057[] = {{&S37766[78], 6}, {NULL}};
const Segment SGI142058[] = {{&S37771[44], 12}, {NULL}};
const Segment SGI142059[] = {{&S37771[44], 30}, {NULL}};
const Segment SGI142060[] = {{&S37771[30], 6}, {NULL}};
const Segment SGI142061[] = {{&S37771[31], 3}, {NULL}};
const Segment SGI142062[] = {{&S37771[88], 6}, {NULL}};
const Segment SGI142063[] = {{&S37772[44], 14}, {NULL}};
const Segment SGI142064[] = {{&S37772[44], 26}, {NULL}};
const Segment SGI142065[] = {{&S37772[30], 6}, {NULL}};
const Segment SGI142066[] = {{&S37772[31], 3}, {NULL}};
const Segment SGI142067[] = {{&S37772[84], 6}, {NULL}};
const Segment SGI142068[] = {{&S37773[44], 12}, {NULL}};
const Segment SGI142069[] = {{&S37773[44], 30}, {NULL}};
const Segment SGI142070[] = {{&S37773[30], 6}, {NULL}};
const Segment SGI142071[] = {{&S37773[31], 3}, {NULL}};
const Segment SGI142072[] = {{&S37773[88], 6}, {NULL}};
const Segment SGI142073[] = {{&S37811[44], 5}, {NULL}};
const Segment SGI142074[] = {{&S37811[44], 14}, {NULL}};
const Segment SGI142075[] = {{&S37811[30], 6}, {NULL}};
const Segment SGI142076[] = {{&S37811[31], 3}, {NULL}};
const Segment SGI142077[] = {{&S37811[72], 6}, {NULL}};
const Segment SGI142078[] = {{&S37814[44], 9}, {NULL}};
const Segment SGI142079[] = {{&S37814[44], 21}, {NULL}};
const Segment SGI142080[] = {{&S37814[30], 6}, {NULL}};
const Segment SGI142081[] = {{&S37814[31], 3}, {NULL}};
const Segment SGI142082[] = {{&S37814[79], 6}, {NULL}};
const Segment SGI142083[] = {{&S37815[44], 22}, {NULL}};
const Segment SGI142084[] = {{&S37815[44], 34}, {NULL}};
const Segment SGI142085[] = {{&S37815[30], 6}, {NULL}};
const Segment SGI142086[] = {{&S37815[31], 3}, {NULL}};
const Segment SGI142087[] = {{&S37815[92], 6}, {NULL}};
const Segment SGI142088[] = {{&S37818[38], 12}, {NULL}};
const Segment SGI142089[] = {{&S37818[31], 6}, {NULL}};
const Segment SGI142090[] = {{&S37819[38], 5}, {NULL}};
const Segment SGI142091[] = {{&S37819[31], 6}, {NULL}};
const Segment SGI142092[] = {{&S37820[44], 5}, {NULL}};
const Segment SGI142093[] = {{&S37820[44], 14}, {NULL}};
const Segment SGI142094[] = {{&S37820[30], 6}, {NULL}};
const Segment SGI142095[] = {{&S37820[31], 3}, {NULL}};
const Segment SGI142096[] = {{&S37820[72], 6}, {NULL}};
const Segment SGI142097[] = {{&S37821[44], 12}, {NULL}};
const Segment SGI142098[] = {{&S37821[44], 21}, {NULL}};
const Segment SGI142099[] = {{&S37821[30], 6}, {NULL}};
const Segment SGI142100[] = {{&S37821[31], 3}, {NULL}};
const Segment SGI142101[] = {{&S37821[79], 6}, {NULL}};
const Segment SGI142102[] = {{&S37824[44], 5}, {NULL}};
const Segment SGI142103[] = {{&S37824[44], 16}, {NULL}};
const Segment SGI142104[] = {{&S37824[30], 6}, {NULL}};
const Segment SGI142105[] = {{&S22809[34], 9}, {NULL}};
const Segment SGI142106[] = {{&S37824[31], 3}, {NULL}};
const Segment SGI142107[] = {{&S22809[27], 6}, {NULL}};
const Segment SGI142108[] = {{&S37824[74], 5}, {NULL}};
const Segment SGI142109[] = {{&S22809[96], 9}, {NULL}};
const Segment SGI142110[] = {{&S22809[69], 6}, {NULL}};
const Segment SGI142111[] = {{&S22810[34], 9}, {NULL}};
const Segment SGI142112[] = {{&S22810[27], 6}, {NULL}};
const Segment SGI142113[] = {{&S22810[96], 9}, {NULL}};
const Segment SGI142114[] = {{&S22810[69], 6}, {NULL}};
const Segment SGI142115[] = {{&S37827[38], 22}, {NULL}};
const Segment SGI142116[] = {{&S37827[31], 6}, {NULL}};
const Segment SGI142117[] = {{&S22813[42], 9}, {NULL}};
const Segment SGI142118[] = {{&S22813[42], 21}, {NULL}};
const Segment SGI142119[] = {{&S22813[27], 6}, {NULL}};
const Segment SGI142120[] = {{&S22813[103], 9}, {NULL}};
const Segment SGI142121[] = {{&S22813[77], 6}, {NULL}};
const Segment SGI142122[] = {{&S37831[38], 9}, {NULL}};
const Segment SGI142123[] = {{&S37831[31], 6}, {NULL}};
const Segment SGI142124[] = {{&S22814[34], 12}, {NULL}};
const Segment SGI142125[] = {{&S22814[27], 6}, {NULL}};
const Segment SGI142126[] = {{&S22814[99], 9}, {NULL}};
const Segment SGI142127[] = {{&S22814[72], 6}, {NULL}};
const Segment SGI142128[] = {{&S37852[44], 11}, {NULL}};
const Segment SGI142129[] = {{&S37852[44], 39}, {NULL}};
const Segment SGI142130[] = {{&S37852[30], 6}, {NULL}};
const Segment SGI142131[] = {{&S37852[31], 3}, {NULL}};
const Segment SGI142132[] = {{&S37852[97], 6}, {NULL}};
const Segment SGI142133[] = {{&S37853[44], 11}, {NULL}};
const Segment SGI142134[] = {{&S37853[44], 39}, {NULL}};
const Segment SGI142135[] = {{&S37853[30], 6}, {NULL}};
const Segment SGI142136[] = {{&S37853[31], 3}, {NULL}};
const Segment SGI142137[] = {{&S37853[97], 6}, {NULL}};
const Segment SGI142138[] = {{&S37854[44], 8}, {NULL}};
const Segment SGI142139[] = {{&S37854[44], 20}, {NULL}};
const Segment SGI142140[] = {{&S37854[30], 6}, {NULL}};
const Segment SGI142141[] = {{&S37854[31], 3}, {NULL}};
const Segment SGI142142[] = {{&S37854[78], 6}, {NULL}};
const Segment SGI142143[] = {{&S37855[44], 11}, {NULL}};
const Segment SGI142144[] = {{&S37855[44], 39}, {NULL}};
const Segment SGI142145[] = {{&S37855[30], 6}, {NULL}};
const Segment SGI142146[] = {{&S37855[31], 3}, {NULL}};
const Segment SGI142147[] = {{&S37855[97], 6}, {NULL}};
const Segment SGI142148[] = {{&S37856[44], 8}, {NULL}};
const Segment SGI142149[] = {{&S37856[44], 36}, {NULL}};
const Segment SGI142150[] = {{&S37856[30], 6}, {NULL}};
const Segment SGI142151[] = {{&S37856[31], 3}, {NULL}};
const Segment SGI142152[] = {{&S37856[94], 6}, {NULL}};
const Segment SGI142153[] = {{&S37857[44], 11}, {NULL}};
const Segment SGI142154[] = {{&S37857[44], 39}, {NULL}};
const Segment SGI142155[] = {{&S37857[30], 6}, {NULL}};
const Segment SGI142156[] = {{&S37857[31], 3}, {NULL}};
const Segment SGI142157[] = {{&S37857[97], 6}, {NULL}};
const Segment SGI142158[] = {{&S37858[38], 8}, {NULL}};
const Segment SGI142159[] = {{&S37858[31], 6}, {NULL}};
const Segment SGI142160[] = {{&S37859[44], 8}, {NULL}};
const Segment SGI142161[] = {{&S37859[44], 36}, {NULL}};
const Segment SGI142162[] = {{&S37859[30], 6}, {NULL}};
const Segment SGI142163[] = {{&S37859[31], 3}, {NULL}};
const Segment SGI142164[] = {{&S37859[94], 6}, {NULL}};
const Segment SGI142165[] = {{&S37860[44], 11}, {NULL}};
const Segment SGI142166[] = {{&S37860[44], 39}, {NULL}};
const Segment SGI142167[] = {{&S37860[30], 6}, {NULL}};
const Segment SGI142168[] = {{&S37860[31], 3}, {NULL}};
const Segment SGI142169[] = {{&S37860[97], 6}, {NULL}};
const Segment SGI142170[] = {{&S37861[44], 11}, {NULL}};
const Segment SGI142171[] = {{&S37861[44], 39}, {NULL}};
const Segment SGI142172[] = {{&S37861[30], 6}, {NULL}};
const Segment SGI142173[] = {{&S37861[31], 3}, {NULL}};
const Segment SGI142174[] = {{&S37861[97], 6}, {NULL}};
const Segment SGI142175[] = {{&S37862[44], 11}, {NULL}};
const Segment SGI142176[] = {{&S37862[44], 39}, {NULL}};
const Segment SGI142177[] = {{&S37862[30], 6}, {NULL}};
const Segment SGI142178[] = {{&S37862[31], 3}, {NULL}};
const Segment SGI142179[] = {{&S37862[97], 6}, {NULL}};
const Segment SGI142180[] = {{&S37863[44], 11}, {NULL}};
const Segment SGI142181[] = {{&S37863[44], 39}, {NULL}};
const Segment SGI142182[] = {{&S37863[30], 6}, {NULL}};
const Segment SGI142183[] = {{&S37863[31], 3}, {NULL}};
const Segment SGI142184[] = {{&S37863[97], 6}, {NULL}};
const Segment SGI142185[] = {{&S37864[44], 8}, {NULL}};
const Segment SGI142186[] = {{&S37864[44], 36}, {NULL}};
const Segment SGI142187[] = {{&S37864[30], 6}, {NULL}};
const Segment SGI142188[] = {{&S37864[31], 3}, {NULL}};
const Segment SGI142189[] = {{&S37864[94], 6}, {NULL}};
const Segment SGI142190[] = {{&S37865[44], 8}, {NULL}};
const Segment SGI142191[] = {{&S37865[44], 36}, {NULL}};
const Segment SGI142192[] = {{&S37865[30], 6}, {NULL}};
const Segment SGI142193[] = {{&S37865[31], 3}, {NULL}};
const Segment SGI142194[] = {{&S37865[94], 6}, {NULL}};
const Segment SGI142195[] = {{&S37866[44], 8}, {NULL}};
const Segment SGI142196[] = {{&S37866[44], 36}, {NULL}};
const Segment SGI142197[] = {{&S37866[30], 6}, {NULL}};
const Segment SGI142198[] = {{&S37866[94], 6}, {NULL}};
const Segment SGI142199[] = {{&S37890[44], 21}, {NULL}};
const Segment SGI142200[] = {{&S37890[44], 33}, {NULL}};
const Segment SGI142201[] = {{&S37890[30], 6}, {NULL}};
const Segment SGI142202[] = {{&S37890[31], 3}, {NULL}};
const Segment SGI142203[] = {{&S28371[44], 10}, {NULL}};
const Segment SGI142204[] = {{&S28371[44], 22}, {NULL}};
const Segment SGI142205[] = {{&S37890[91], 6}, {NULL}};
const Segment SGI142206[] = {{&S28371[30], 6}, {NULL}};
const Segment SGI142207[] = {{&S28371[115], 3}, {NULL}};
const Segment SGI142208[] = {{&S28371[80], 6}, {NULL}};
const Segment SGI142209[] = {{&S37891[44], 20}, {NULL}};
const Segment SGI142210[] = {{&S37891[44], 32}, {NULL}};
const Segment SGI142211[] = {{&S37891[30], 6}, {NULL}};
const Segment SGI142212[] = {{&S37891[31], 3}, {NULL}};
const Segment SGI142213[] = {{&S28372[38], 10}, {NULL}};
const Segment SGI142214[] = {{&S37891[90], 6}, {NULL}};
const Segment SGI142215[] = {{&S28372[31], 6}, {NULL}};
const Segment SGI142216[] = {{&S28374[44], 10}, {NULL}};
const Segment SGI142217[] = {{&S28374[44], 22}, {NULL}};
const Segment SGI142218[] = {{&S28374[30], 6}, {NULL}};
const Segment SGI142219[] = {{&S28374[115], 3}, {NULL}};
const Segment SGI142220[] = {{&S28374[80], 6}, {NULL}};
const Segment SGI142221[] = {{&S37895[44], 25}, {NULL}};
const Segment SGI142222[] = {{&S37895[44], 37}, {NULL}};
const Segment SGI142223[] = {{&S37895[30], 6}, {NULL}};
const Segment SGI142224[] = {{&S37895[31], 3}, {NULL}};
const Segment SGI142225[] = {{&S37895[95], 6}, {NULL}};
const Segment SGI142226[] = {{&S28376[42], 10}, {NULL}};
const Segment SGI142227[] = {{&S28376[30], 6}, {NULL}};
const Segment SGI142228[] = {{&S28376[113], 3}, {NULL}};
const Segment SGI142229[] = {{&S28376[78], 6}, {NULL}};
const Segment SGI142230[] = {{&S37897[44], 22}, {NULL}};
const Segment SGI142231[] = {{&S37897[44], 34}, {NULL}};
const Segment SGI142232[] = {{&S37897[30], 6}, {NULL}};
const Segment SGI142233[] = {{&S37897[31], 3}, {NULL}};
const Segment SGI142234[] = {{&S37897[92], 6}, {NULL}};
const Segment SGI142235[] = {{&S37898[44], 25}, {NULL}};
const Segment SGI142236[] = {{&S37898[44], 37}, {NULL}};
const Segment SGI142237[] = {{&S37898[30], 6}, {NULL}};
const Segment SGI142238[] = {{&S37898[31], 3}, {NULL}};
const Segment SGI142239[] = {{&S37898[95], 6}, {NULL}};
const Segment SGI142240[] = {{&S37899[44], 30}, {NULL}};
const Segment SGI142241[] = {{&S37899[44], 42}, {NULL}};
const Segment SGI142242[] = {{&S37899[30], 6}, {NULL}};
const Segment SGI142243[] = {{&S37899[31], 3}, {NULL}};
const Segment SGI142244[] = {{&S37899[100], 6}, {NULL}};
const Segment SGI142245[] = {{&S37901[44], 29}, {NULL}};
const Segment SGI142246[] = {{&S37901[44], 41}, {NULL}};
const Segment SGI142247[] = {{&S37901[30], 6}, {NULL}};
const Segment SGI142248[] = {{&S37901[31], 3}, {NULL}};
const Segment SGI142249[] = {{&S37901[99], 6}, {NULL}};
const Segment SGI142250[] = {{&S37903[44], 27}, {NULL}};
const Segment SGI142251[] = {{&S37903[44], 39}, {NULL}};
const Segment SGI142252[] = {{&S37903[30], 6}, {NULL}};
const Segment SGI142253[] = {{&S37903[31], 3}, {NULL}};
const Segment SGI142254[] = {{&S37903[97], 6}, {NULL}};
const Segment SGI142255[] = {{&S37905[44], 27}, {NULL}};
const Segment SGI142256[] = {{&S37905[44], 39}, {NULL}};
const Segment SGI142257[] = {{&S37905[30], 6}, {NULL}};
const Segment SGI142258[] = {{&S37905[31], 3}, {NULL}};
const Segment SGI142259[] = {{&S37905[97], 6}, {NULL}};
const Segment SGI142260[] = {{&S37907[44], 21}, {NULL}};
const Segment SGI142261[] = {{&S37907[44], 33}, {NULL}};
const Segment SGI142262[] = {{&S37907[30], 6}, {NULL}};
const Segment SGI142263[] = {{&S37907[31], 3}, {NULL}};
const Segment SGI142264[] = {{&S37907[91], 6}, {NULL}};
const Segment SGI142265[] = {{&S37908[44], 21}, {NULL}};
const Segment SGI142266[] = {{&S37908[44], 33}, {NULL}};
const Segment SGI142267[] = {{&S37908[30], 6}, {NULL}};
const Segment SGI142268[] = {{&S37908[31], 3}, {NULL}};
const Segment SGI142269[] = {{&S37908[91], 6}, {NULL}};
const Segment SGI142270[] = {{&S27575[34], 10}, {NULL}};
const Segment SGI142271[] = {{&S27575[27], 6}, {NULL}};
const Segment SGI142272[] = {{&S27575[97], 9}, {NULL}};
const Segment SGI142273[] = {{&S27575[70], 6}, {NULL}};
const Segment SGI142274[] = {{&S27576[34], 10}, {NULL}};
const Segment SGI142275[] = {{&S27576[27], 6}, {NULL}};
const Segment SGI142276[] = {{&S27576[97], 9}, {NULL}};
const Segment SGI142277[] = {{&S27576[70], 6}, {NULL}};
const Segment SGI142278[] = {{&S27577[34], 10}, {NULL}};
const Segment SGI142279[] = {{&S27577[27], 6}, {NULL}};
const Segment SGI142280[] = {{&S27577[97], 9}, {NULL}};
const Segment SGI142281[] = {{&S27577[70], 6}, {NULL}};
const Segment SGI142282[] = {{&S27578[34], 10}, {NULL}};
const Segment SGI142283[] = {{&S27578[27], 6}, {NULL}};
const Segment SGI142284[] = {{&S27578[97], 9}, {NULL}};
const Segment SGI142285[] = {{&S27578[70], 6}, {NULL}};
const Segment SGI142286[] = {{&S27579[34], 18}, {NULL}};
const Segment SGI142287[] = {{&S27579[27], 6}, {NULL}};
const Segment SGI142288[] = {{&S27579[105], 9}, {NULL}};
const Segment SGI142289[] = {{&S27579[78], 6}, {NULL}};
const Segment SGI142290[] = {{&S27580[34], 10}, {NULL}};
const Segment SGI142291[] = {{&S27580[27], 6}, {NULL}};
const Segment SGI142292[] = {{&S27580[97], 9}, {NULL}};
const Segment SGI142293[] = {{&S27580[70], 6}, {NULL}};
const Segment SGI142294[] = {{&S27582[34], 11}, {NULL}};
const Segment SGI142295[] = {{&S27582[27], 6}, {NULL}};
const Segment SGI142296[] = {{&S27582[98], 9}, {NULL}};
const Segment SGI142297[] = {{&S27582[71], 6}, {NULL}};
const Segment SGI142298[] = {{&S27584[34], 11}, {NULL}};
const Segment SGI142299[] = {{&S27584[27], 6}, {NULL}};
const Segment SGI142300[] = {{&S27584[98], 9}, {NULL}};
const Segment SGI142301[] = {{&S27584[71], 6}, {NULL}};
const Segment SGI142302[] = {{&S27585[34], 11}, {NULL}};
const Segment SGI142303[] = {{&S27585[27], 6}, {NULL}};
const Segment SGI142304[] = {{&S27585[98], 9}, {NULL}};
const Segment SGI142305[] = {{&S27585[71], 6}, {NULL}};
const Segment SGI142306[] = {{&S27589[34], 11}, {NULL}};
const Segment SGI142307[] = {{&S27589[27], 6}, {NULL}};
const Segment SGI142308[] = {{&S27589[98], 9}, {NULL}};
const Segment SGI142309[] = {{&S27589[71], 6}, {NULL}};
const Segment SGI142310[] = {{&S27590[34], 10}, {NULL}};
const Segment SGI142311[] = {{&S27590[27], 6}, {NULL}};
const Segment SGI142312[] = {{&S27590[97], 9}, {NULL}};
const Segment SGI142313[] = {{&S27590[70], 6}, {NULL}};
const Segment SGI142314[] = {{&S27591[34], 11}, {NULL}};
const Segment SGI142315[] = {{&S27591[27], 6}, {NULL}};
const Segment SGI142316[] = {{&S27591[98], 9}, {NULL}};
const Segment SGI142317[] = {{&S27591[71], 6}, {NULL}};
const Segment SGI142318[] = {{&S27592[34], 11}, {NULL}};
const Segment SGI142319[] = {{&S27592[27], 6}, {NULL}};
const Segment SGI142320[] = {{&S27592[98], 9}, {NULL}};
const Segment SGI142321[] = {{&S27592[71], 6}, {NULL}};
const Segment SGI142322[] = {{&S27594[34], 10}, {NULL}};
const Segment SGI142323[] = {{&S27594[27], 6}, {NULL}};
const Segment SGI142324[] = {{&S27594[97], 9}, {NULL}};
const Segment SGI142325[] = {{&S27594[70], 6}, {NULL}};
const Segment SGI142326[] = {{&S27596[34], 11}, {NULL}};
const Segment SGI142327[] = {{&S27596[27], 6}, {NULL}};
const Segment SGI142328[] = {{&S27596[98], 9}, {NULL}};
const Segment SGI142329[] = {{&S27596[71], 6}, {NULL}};
const Segment SGI142330[] = {{&S27598[34], 10}, {NULL}};
const Segment SGI142331[] = {{&S27598[27], 6}, {NULL}};
const Segment SGI142332[] = {{&S27598[97], 9}, {NULL}};
const Segment SGI142333[] = {{&S27598[70], 6}, {NULL}};
const Segment SGI142334[] = {{&S27601[34], 10}, {NULL}};
const Segment SGI142335[] = {{&S27601[27], 6}, {NULL}};
const Segment SGI142336[] = {{&S27601[97], 9}, {NULL}};
const Segment SGI142337[] = {{&S27601[70], 6}, {NULL}};
const Segment SGI142338[] = {{&S27604[34], 10}, {NULL}};
const Segment SGI142339[] = {{&S27604[27], 6}, {NULL}};
const Segment SGI142340[] = {{&S27604[97], 9}, {NULL}};
const Segment SGI142341[] = {{&S27604[70], 6}, {NULL}};
const Segment SGI142342[] = {{&S27605[34], 10}, {NULL}};
const Segment SGI142343[] = {{&S27605[27], 6}, {NULL}};
const Segment SGI142344[] = {{&S27605[97], 9}, {NULL}};
const Segment SGI142345[] = {{&S27605[70], 6}, {NULL}};
const Segment SGI142346[] = {{&S27606[34], 10}, {NULL}};
const Segment SGI142347[] = {{&S27606[27], 6}, {NULL}};
const Segment SGI142348[] = {{&S27606[97], 9}, {NULL}};
const Segment SGI142349[] = {{&S27606[70], 6}, {NULL}};
const Segment SGI142350[] = {{&S38000[44], 10}, {NULL}};
const Segment SGI142351[] = {{&S38000[44], 22}, {NULL}};
const Segment SGI142352[] = {{&S38000[30], 6}, {NULL}};
const Segment SGI142353[] = {{&S38000[31], 3}, {NULL}};
const Segment SGI142354[] = {{&S28480[34], 8}, {NULL}};
const Segment SGI142355[] = {{&S38000[80], 6}, {NULL}};
const Segment SGI142356[] = {{&S28480[27], 6}, {NULL}};
const Segment SGI142357[] = {{&S28480[95], 9}, {NULL}};
const Segment SGI142358[] = {{&S28480[68], 6}, {NULL}};
const Segment SGI142359[] = {{&S38001[44], 10}, {NULL}};
const Segment SGI142360[] = {{&S38001[44], 22}, {NULL}};
const Segment SGI142361[] = {{&S38001[30], 6}, {NULL}};
const Segment SGI142362[] = {{&S38001[31], 3}, {NULL}};
const Segment SGI142363[] = {{&S38001[80], 6}, {NULL}};
const Segment SGI142364[] = {{&S28482[34], 18}, {NULL}};
const Segment SGI142365[] = {{&S28482[27], 6}, {NULL}};
const Segment SGI142366[] = {{&S38002[38], 10}, {NULL}};
const Segment SGI142367[] = {{&S38002[31], 6}, {NULL}};
const Segment SGI142368[] = {{&S28482[105], 9}, {NULL}};
const Segment SGI142369[] = {{&S28482[78], 6}, {NULL}};
const Segment SGI142370[] = {{&S38003[44], 11}, {NULL}};
const Segment SGI142371[] = {{&S38003[44], 23}, {NULL}};
const Segment SGI142372[] = {{&S38003[30], 6}, {NULL}};
const Segment SGI142373[] = {{&S38003[31], 3}, {NULL}};
const Segment SGI142374[] = {{&S38003[81], 6}, {NULL}};
const Segment SGI142375[] = {{&S38004[42], 10}, {NULL}};
const Segment SGI142376[] = {{&S38004[30], 6}, {NULL}};
const Segment SGI142377[] = {{&S38004[31], 3}, {NULL}};
const Segment SGI142378[] = {{&S38004[78], 6}, {NULL}};
const Segment SGI142379[] = {{&S28487[34], 10}, {NULL}};
const Segment SGI142380[] = {{&S28487[27], 6}, {NULL}};
const Segment SGI142381[] = {{&S28487[97], 9}, {NULL}};
const Segment SGI142382[] = {{&S28487[70], 6}, {NULL}};
const Segment SGI142383[] = {{&S28489[34], 10}, {NULL}};
const Segment SGI142384[] = {{&S28489[27], 6}, {NULL}};
const Segment SGI142385[] = {{&S28489[97], 9}, {NULL}};
const Segment SGI142386[] = {{&S28489[70], 6}, {NULL}};
const Segment SGI142387[] = {{&S38008[44], 10}, {NULL}};
const Segment SGI142388[] = {{&S38008[44], 22}, {&S38008[208], 7}, {NULL}};
const Segment SGI142389[] = {{&S38008[30], 6}, {NULL}};
const Segment SGI142390[] = {{&S38008[31], 3}, {NULL}};
const Segment SGI142391[] = {{&S38008[80], 6}, {NULL}};
const Segment SGI142392[] = {{&S28493[34], 10}, {NULL}};
const Segment SGI142393[] = {{&S28493[27], 6}, {NULL}};
const Segment SGI142394[] = {{&S38009[65], 10}, {NULL}};
const Segment SGI142395[] = {{&S38009[65], 22}, {NULL}};
const Segment SGI142396[] = {{&S38009[51], 6}, {NULL}};
const Segment SGI142397[] = {{&S28493[97], 9}, {NULL}};
const Segment SGI142398[] = {{&S28493[70], 6}, {NULL}};
const Segment SGI142399[] = {{&S38010[44], 10}, {NULL}};
const Segment SGI142400[] = {{&S38010[44], 28}, {NULL}};
const Segment SGI142401[] = {{&S38010[30], 6}, {NULL}};
const Segment SGI142402[] = {{&S28496[34], 11}, {NULL}};
const Segment SGI142403[] = {{&S28496[27], 6}, {NULL}};
const Segment SGI142404[] = {{&S38011[72], 10}, {NULL}};
const Segment SGI142405[] = {{&S38011[72], 22}, {NULL}};
const Segment SGI142406[] = {{&S38011[50], 14}, {NULL}};
const Segment SGI142407[] = {{&S28496[98], 9}, {NULL}};
const Segment SGI142408[] = {{&S28496[71], 6}, {NULL}};
const Segment SGI142409[] = {{&S28497[34], 10}, {NULL}};
const Segment SGI142410[] = {{&S28497[27], 6}, {NULL}};
const Segment SGI142411[] = {{&S38012[44], 10}, {NULL}};
const Segment SGI142412[] = {{&S38012[44], 19}, {NULL}};
const Segment SGI142413[] = {{&S38012[30], 6}, {NULL}};
const Segment SGI142414[] = {{&S28497[97], 9}, {NULL}};
const Segment SGI142415[] = {{&S28497[70], 6}, {NULL}};
const Segment SGI142416[] = {{&S38012[31], 3}, {NULL}};
const Segment SGI142417[] = {{&S38012[77], 5}, {NULL}};
const Segment SGI142418[] = {{&S38013[46], 10}, {NULL}};
const Segment SGI142419[] = {{&S38013[46], 28}, {NULL}};
const Segment SGI142420[] = {{&S38013[32], 6}, {NULL}};
const Segment SGI142421[] = {{&S28499[34], 11}, {NULL}};
const Segment SGI142422[] = {{&S28499[27], 6}, {NULL}};
const Segment SGI142423[] = {{&S28499[98], 9}, {NULL}};
const Segment SGI142424[] = {{&S28499[71], 6}, {NULL}};
const Segment SGI142425[] = {{&S38014[44], 10}, {NULL}};
const Segment SGI142426[] = {{&S38014[44], 22}, {NULL}};
const Segment SGI142427[] = {{&S38014[30], 6}, {NULL}};
const Segment SGI142428[] = {{&S38014[83], 6}, {NULL}};
const Segment SGI142429[] = {{&S38015[44], 10}, {NULL}};
const Segment SGI142430[] = {{&S28502[34], 11}, {NULL}};
const Segment SGI142431[] = {{&S38015[44], 22}, {NULL}};
const Segment SGI142432[] = {{&S38015[30], 6}, {NULL}};
const Segment SGI142433[] = {{&S28502[27], 6}, {NULL}};
const Segment SGI142434[] = {{&S38015[31], 3}, {NULL}};
const Segment SGI142435[] = {{&S38015[80], 6}, {NULL}};
const Segment SGI142436[] = {{&S28502[98], 9}, {NULL}};
const Segment SGI142437[] = {{&S28502[71], 6}, {NULL}};
const Segment SGI142438[] = {{&S38016[99], 29}, {NULL}};
const Segment SGI142439[] = {{&S38017[44], 10}, {NULL}};
const Segment SGI142440[] = {{&S38017[44], 22}, {NULL}};
const Segment SGI142441[] = {{&S38017[30], 6}, {NULL}};
const Segment SGI142442[] = {{&S28506[34], 10}, {NULL}};
const Segment SGI142443[] = {{&S38017[31], 3}, {NULL}};
const Segment SGI142444[] = {{&S28506[27], 6}, {NULL}};
const Segment SGI142445[] = {{&S38017[80], 6}, {NULL}};
const Segment SGI142446[] = {{&S28506[97], 9}, {NULL}};
const Segment SGI142447[] = {{&S28506[70], 6}, {NULL}};
const Segment SGI142448[] = {{&S38018[44], 10}, {NULL}};
const Segment SGI142449[] = {{&S38018[44], 22}, {NULL}};
const Segment SGI142450[] = {{&S38018[30], 6}, {NULL}};
const Segment SGI142451[] = {{&S38018[125], 3}, {NULL}};
const Segment SGI142452[] = {{&S38018[90], 6}, {NULL}};
const Segment SGI142453[] = {{&S38018[145], 8}, {NULL}};
const Segment SGI142454[] = {{&S28508[34], 10}, {NULL}};
const Segment SGI142455[] = {{&S28508[27], 6}, {NULL}};
const Segment SGI142456[] = {{&S28508[97], 9}, {NULL}};
const Segment SGI142457[] = {{&S28508[70], 6}, {NULL}};
const Segment SGI142458[] = {{&S38020[44], 10}, {NULL}};
const Segment SGI142459[] = {{&S38020[44], 22}, {NULL}};
const Segment SGI142460[] = {{&S38020[30], 6}, {NULL}};
const Segment SGI142461[] = {{&S38020[80], 6}, {NULL}};
const Segment SGI142462[] = {{&S28510[34], 10}, {NULL}};
const Segment SGI142463[] = {{&S28510[27], 6}, {NULL}};
const Segment SGI142464[] = {{&S28510[97], 9}, {NULL}};
const Segment SGI142465[] = {{&S28510[70], 6}, {NULL}};
const Segment SGI142466[] = {{&S28512[34], 10}, {NULL}};
const Segment SGI142467[] = {{&S28512[27], 6}, {NULL}};
const Segment SGI142468[] = {{&S28512[97], 9}, {NULL}};
const Segment SGI142469[] = {{&S28512[70], 6}, {NULL}};
const Segment SGI142470[] = {{&S28516[34], 10}, {NULL}};
const Segment SGI142471[] = {{&S28516[27], 6}, {NULL}};
const Segment SGI142472[] = {{&S28516[97], 9}, {NULL}};
const Segment SGI142473[] = {{&S28516[70], 6}, {NULL}};
const Segment SGI142474[] = {{&S28518[34], 18}, {NULL}};
const Segment SGI142475[] = {{&S28518[27], 6}, {NULL}};
const Segment SGI142476[] = {{&S28518[105], 9}, {NULL}};
const Segment SGI142477[] = {{&S28518[78], 6}, {NULL}};
const Segment SGI142478[] = {{&S28519[34], 11}, {NULL}};
const Segment SGI142479[] = {{&S28519[27], 6}, {NULL}};
const Segment SGI142480[] = {{&S28519[98], 9}, {NULL}};
const Segment SGI142481[] = {{&S28519[71], 6}, {NULL}};
const Segment SGI142482[] = {{&S28520[34], 10}, {NULL}};
const Segment SGI142483[] = {{&S28520[27], 6}, {NULL}};
const Segment SGI142484[] = {{&S28520[97], 9}, {NULL}};
const Segment SGI142485[] = {{&S28520[70], 6}, {NULL}};
const Segment SGI142486[] = {{&S27667[44], 13}, {NULL}};
const Segment SGI142487[] = {{&S27667[44], 30}, {NULL}};
const Segment SGI142488[] = {{&S27667[30], 6}, {NULL}};
const Segment SGI142489[] = {{&S27667[123], 3}, {NULL}};
const Segment SGI142490[] = {{&S27667[88], 6}, {NULL}};
const Segment SGI142491[] = {{&S28522[34], 11}, {NULL}};
const Segment SGI142492[] = {{&S28522[27], 6}, {NULL}};
const Segment SGI142493[] = {{&S28522[98], 9}, {NULL}};
const Segment SGI142494[] = {{&S28522[71], 6}, {NULL}};
const Segment SGI142495[] = {{&S27668[38], 7}, {NULL}};
const Segment SGI142496[] = {{&S27668[31], 6}, {NULL}};
const Segment SGI142497[] = {{&S27669[44], 11}, {NULL}};
const Segment SGI142498[] = {{&S27669[44], 28}, {NULL}};
const Segment SGI142499[] = {{&S27669[30], 6}, {NULL}};
const Segment SGI142500[] = {{&S27669[121], 3}, {NULL}};
const Segment SGI142501[] = {{&S27669[86], 6}, {NULL}};
const Segment SGI142502[] = {{&S27670[44], 7}, {NULL}};
const Segment SGI142503[] = {{&S27670[44], 24}, {NULL}};
const Segment SGI142504[] = {{&S27670[30], 6}, {NULL}};
const Segment SGI142505[] = {{&S27670[117], 3}, {NULL}};
const Segment SGI142506[] = {{&S27670[82], 6}, {NULL}};
const Segment SGI142507[] = {{&S27671[44], 13}, {NULL}};
const Segment SGI142508[] = {{&S27671[44], 30}, {NULL}};
const Segment SGI142509[] = {{&S27671[30], 6}, {NULL}};
const Segment SGI142510[] = {{&S27671[123], 3}, {NULL}};
const Segment SGI142511[] = {{&S27671[88], 6}, {NULL}};
const Segment SGI142512[] = {{&S27672[44], 11}, {NULL}};
const Segment SGI142513[] = {{&S27672[44], 28}, {NULL}};
const Segment SGI142514[] = {{&S27672[30], 6}, {NULL}};
const Segment SGI142515[] = {{&S27672[121], 3}, {NULL}};
const Segment SGI142516[] = {{&S27672[86], 6}, {NULL}};
const Segment SGI142517[] = {{&S27673[44], 7}, {NULL}};
const Segment SGI142518[] = {{&S27673[44], 24}, {NULL}};
const Segment SGI142519[] = {{&S27673[30], 6}, {NULL}};
const Segment SGI142520[] = {{&S27673[117], 3}, {NULL}};
const Segment SGI142521[] = {{&S27673[82], 6}, {NULL}};
const Segment SGI142522[] = {{&S27674[44], 7}, {NULL}};
const Segment SGI142523[] = {{&S27674[44], 24}, {NULL}};
const Segment SGI142524[] = {{&S27674[30], 6}, {NULL}};
const Segment SGI142525[] = {{&S27674[117], 3}, {NULL}};
const Segment SGI142526[] = {{&S27674[82], 6}, {NULL}};
const Segment SGI142527[] = {{&S27676[44], 12}, {NULL}};
const Segment SGI142528[] = {{&S27676[44], 29}, {NULL}};
const Segment SGI142529[] = {{&S27676[30], 6}, {NULL}};
const Segment SGI142530[] = {{&S27676[122], 3}, {NULL}};
const Segment SGI142531[] = {{&S27676[87], 6}, {NULL}};
const Segment SGI142532[] = {{&S27677[44], 11}, {NULL}};
const Segment SGI142533[] = {{&S27677[44], 27}, {NULL}};
const Segment SGI142534[] = {{&S27677[30], 6}, {NULL}};
const Segment SGI142535[] = {{&S27677[120], 3}, {NULL}};
const Segment SGI142536[] = {{&S27677[85], 6}, {NULL}};
const Segment SGI142537[] = {{&S27678[44], 7}, {NULL}};
const Segment SGI142538[] = {{&S27678[44], 23}, {NULL}};
const Segment SGI142539[] = {{&S27678[30], 6}, {NULL}};
const Segment SGI142540[] = {{&S27678[116], 3}, {NULL}};
const Segment SGI142541[] = {{&S27678[81], 6}, {NULL}};
const Segment SGI142542[] = {{&S27679[44], 7}, {NULL}};
const Segment SGI142543[] = {{&S27679[44], 39}, {NULL}};
const Segment SGI142544[] = {{&S27679[30], 6}, {NULL}};
const Segment SGI142545[] = {{&S27679[132], 3}, {NULL}};
const Segment SGI142546[] = {{&S27679[97], 6}, {NULL}};
const Segment SGI142547[] = {{&S38282[44], 10}, {NULL}};
const Segment SGI142548[] = {{&S38282[44], 22}, {NULL}};
const Segment SGI142549[] = {{&S38282[30], 6}, {NULL}};
const Segment SGI142550[] = {{&S38282[31], 3}, {NULL}};
const Segment SGI142551[] = {{&S38282[80], 6}, {NULL}};
const Segment SGI142552[] = {{&S38283[38], 10}, {NULL}};
const Segment SGI142553[] = {{&S38283[31], 6}, {NULL}};
const Segment SGI142554[] = {{&S38284[42], 10}, {NULL}};
const Segment SGI142555[] = {{&S38284[30], 6}, {NULL}};
const Segment SGI142556[] = {{&S38284[31], 3}, {NULL}};
const Segment SGI142557[] = {{&S38284[78], 6}, {NULL}};
const Segment SGI142558[] = {{&S38285[44], 10}, {NULL}};
const Segment SGI142559[] = {{&S38285[44], 22}, {NULL}};
const Segment SGI142560[] = {{&S38285[30], 6}, {NULL}};
const Segment SGI142561[] = {{&S38285[31], 3}, {NULL}};
const Segment SGI142562[] = {{&S38285[80], 6}, {NULL}};
const Segment SGI142563[] = {{&S38286[38], 10}, {NULL}};
const Segment SGI142564[] = {{&S38286[31], 6}, {NULL}};
const Segment SGI142565[] = {{&S38287[42], 10}, {NULL}};
const Segment SGI142566[] = {{&S38287[30], 6}, {NULL}};
const Segment SGI142567[] = {{&S38287[31], 3}, {NULL}};
const Segment SGI142568[] = {{&S38287[78], 6}, {NULL}};
const Segment SGI142569[] = {{&S38288[44], 10}, {NULL}};
const Segment SGI142570[] = {{&S38288[44], 22}, {NULL}};
const Segment SGI142571[] = {{&S38288[30], 6}, {NULL}};
const Segment SGI142572[] = {{&S38288[125], 3}, {NULL}};
const Segment SGI142573[] = {{&S38288[90], 6}, {NULL}};
const Segment SGI142574[] = {{&S38288[145], 8}, {NULL}};
const Segment SGI142575[] = {{&S38290[44], 10}, {NULL}};
const Segment SGI142576[] = {{&S38290[44], 22}, {NULL}};
const Segment SGI142577[] = {{&S38290[30], 6}, {NULL}};
const Segment SGI142578[] = {{&S38290[83], 6}, {NULL}};
const Segment SGI142579[] = {{&S38291[44], 10}, {NULL}};
const Segment SGI142580[] = {{&S38291[44], 73}, {NULL}};
const Segment SGI142581[] = {{&S38291[30], 6}, {NULL}};
const Segment SGI142582[] = {{&S38292[44], 10}, {NULL}};
const Segment SGI142583[] = {{&S38292[44], 28}, {NULL}};
const Segment SGI142584[] = {{&S38292[30], 6}, {NULL}};
const Segment SGI142585[] = {{&S38293[44], 22}, {NULL}};
const Segment SGI142586[] = {{&S38293[30], 6}, {NULL}};
const Segment SGI142587[] = {{&S38294[44], 10}, {NULL}};
const Segment SGI142588[] = {{&S38294[44], 22}, {NULL}};
const Segment SGI142589[] = {{&S38294[30], 6}, {NULL}};
const Segment SGI142590[] = {{&S38294[31], 3}, {NULL}};
const Segment SGI142591[] = {{&S38294[80], 6}, {NULL}};
const Segment SGI142592[] = {{&S38295[30], 1}, {NULL}};
const Segment SGI142593[] = {{&S38295[63], 9}, {NULL}};
const Segment SGI142594[] = {{&S38295[49], 6}, {NULL}};
const Segment SGI142595[] = {{&S38296[44], 10}, {NULL}};
const Segment SGI142596[] = {{&S38296[44], 22}, {NULL}};
const Segment SGI142597[] = {{&S38296[30], 6}, {NULL}};
const Segment SGI142598[] = {{&S38296[80], 6}, {NULL}};
const Segment SGI142599[] = {{&S38297[44], 10}, {NULL}};
const Segment SGI142600[] = {{&S38297[44], 39}, {NULL}};
const Segment SGI142601[] = {{&S38297[30], 6}, {NULL}};
const Segment SGI142602[] = {{&S38297[31], 3}, {NULL}};
const Segment SGI142603[] = {{&S38297[97], 6}, {NULL}};
const Segment SGI142604[] = {{&S38298[44], 9}, {NULL}};
const Segment SGI142605[] = {{&S38298[30], 6}, {NULL}};
const Segment SGI142606[] = {{&S38299[44], 10}, {NULL}};
const Segment SGI142607[] = {{&S38299[44], 28}, {NULL}};
const Segment SGI142608[] = {{&S38299[30], 6}, {NULL}};
const Segment SGI142609[] = {{&S38300[44], 10}, {NULL}};
const Segment SGI142610[] = {{&S38300[44], 22}, {NULL}};
const Segment SGI142611[] = {{&S38300[30], 6}, {NULL}};
const Segment SGI142612[] = {{&S38300[83], 4}, {NULL}};
const Segment SGI142613[] = {{&S38300[83], 3}, {NULL}};
const Segment SGI142614[] = {{&S38301[47], 9}, {NULL}};
const Segment SGI142615[] = {{&S38301[30], 9}, {NULL}};
const Segment SGI142616[] = {{&S38301[106], 3}, {NULL}};
const Segment SGI142617[] = {{&S38301[70], 7}, {NULL}};
const Segment SGI142618[] = {{&S38301[126], 8}, {NULL}};
const Segment SGI142619[] = {{&S38302[55], 18}, {NULL}};
const Segment SGI142620[] = {{&S38302[48], 6}, {NULL}};
const Segment SGI142621[] = {{&S38303[44], 10}, {NULL}};
const Segment SGI142622[] = {{&S38303[44], 28}, {NULL}};
const Segment SGI142623[] = {{&S38303[30], 6}, {NULL}};
const Segment SGI142624[] = {{&S38304[44], 10}, {NULL}};
const Segment SGI142625[] = {{&S38304[44], 22}, {NULL}};
const Segment SGI142626[] = {{&S38304[30], 6}, {NULL}};
const Segment SGI142627[] = {{&S38304[116], 3}, {NULL}};
const Segment SGI142628[] = {{&S38304[80], 7}, {NULL}};
const Segment SGI142629[] = {{&S38304[136], 8}, {NULL}};
const Segment SGI142630[] = {{&S38305[44], 10}, {NULL}};
const Segment SGI142631[] = {{&S38305[44], 28}, {NULL}};
const Segment SGI142632[] = {{&S38305[30], 6}, {NULL}};
const Segment SGI142633[] = {{&S38307[44], 10}, {NULL}};
const Segment SGI142634[] = {{&S38307[44], 22}, {NULL}};
const Segment SGI142635[] = {{&S38307[30], 6}, {NULL}};
const Segment SGI142636[] = {{&S38308[44], 10}, {NULL}};
const Segment SGI142637[] = {{&S38308[44], 47}, {NULL}};
const Segment SGI142638[] = {{&S38308[30], 6}, {NULL}};
const Segment SGI142639[] = {{&S38309[46], 10}, {NULL}};
const Segment SGI142640[] = {{&S38309[46], 28}, {NULL}};
const Segment SGI142641[] = {{&S38309[32], 6}, {NULL}};
const Segment SGI142642[] = {{&S38310[99], 29}, {NULL}};
const Segment SGI142643[] = {{&S38311[46], 10}, {NULL}};
const Segment SGI142644[] = {{&S38311[46], 28}, {NULL}};
const Segment SGI142645[] = {{&S38311[32], 6}, {NULL}};
const Segment SGI142646[] = {{&S38312[44], 10}, {NULL}};
const Segment SGI142647[] = {{&S38312[44], 22}, {NULL}};
const Segment SGI142648[] = {{&S38312[30], 6}, {NULL}};
const Segment SGI142649[] = {{&S38312[31], 3}, {NULL}};
const Segment SGI142650[] = {{&S38312[80], 5}, {NULL}};
const Segment SGI142651[] = {{&S38313[44], 10}, {NULL}};
const Segment SGI142652[] = {{&S38313[44], 22}, {NULL}};
const Segment SGI142653[] = {{&S38313[30], 6}, {NULL}};
const Segment SGI142654[] = {{&S38313[31], 3}, {NULL}};
const Segment SGI142655[] = {{&S38313[80], 6}, {NULL}};
const Segment SGI142656[] = {{&S38314[44], 10}, {NULL}};
const Segment SGI142657[] = {{&S38314[44], 19}, {NULL}};
const Segment SGI142658[] = {{&S38314[30], 6}, {NULL}};
const Segment SGI142659[] = {{&S38314[31], 3}, {NULL}};
const Segment SGI142660[] = {{&S38314[77], 6}, {NULL}};
const Segment SGI142661[] = {{&S38315[44], 10}, {&S38315[44], 10}, {NULL}};
const Segment SGI142662[] = {{&S38315[44], 22}, {&S38315[44], 22}, {NULL}};
const Segment SGI142663[] = {{&S38315[30], 6}, {&S38315[30], 6}, {NULL}};
const Segment SGI142664[] = {{&S38315[31], 3}, {NULL}};
const Segment SGI142665[] = {{&S38315[80], 6}, {NULL}};
const Segment SGI142666[] = {{&S38316[44], 10}, {NULL}};
const Segment SGI142667[] = {{&S38316[44], 19}, {NULL}};
const Segment SGI142668[] = {{&S38316[30], 6}, {NULL}};
const Segment SGI142669[] = {{&S38316[31], 3}, {NULL}};
const Segment SGI142670[] = {{&S38316[77], 5}, {NULL}};
const Segment SGI142671[] = {{&S38386[44], 10}, {NULL}};
const Segment SGI142672[] = {{&S38386[44], 26}, {NULL}};
const Segment SGI142673[] = {{&S38386[30], 6}, {NULL}};
const Segment SGI142674[] = {{&S38386[31], 3}, {NULL}};
const Segment SGI142675[] = {{&S38386[84], 6}, {NULL}};
const Segment SGI142676[] = {{&S38387[44], 10}, {NULL}};
const Segment SGI142677[] = {{&S38387[44], 26}, {NULL}};
const Segment SGI142678[] = {{&S38387[30], 6}, {NULL}};
const Segment SGI142679[] = {{&S38387[31], 3}, {NULL}};
const Segment SGI142680[] = {{&S38387[84], 6}, {NULL}};
const Segment SGI142681[] = {{&S38388[38], 10}, {NULL}};
const Segment SGI142682[] = {{&S38388[31], 6}, {NULL}};
const Segment SGI142683[] = {{&S38389[44], 10}, {NULL}};
const Segment SGI142684[] = {{&S38389[44], 26}, {NULL}};
const Segment SGI142685[] = {{&S38389[30], 6}, {NULL}};
const Segment SGI142686[] = {{&S38389[31], 3}, {NULL}};
const Segment SGI142687[] = {{&S38389[84], 6}, {NULL}};
const Segment SGI142688[] = {{&S38390[44], 10}, {NULL}};
const Segment SGI142689[] = {{&S38390[44], 26}, {NULL}};
const Segment SGI142690[] = {{&S38390[30], 6}, {NULL}};
const Segment SGI142691[] = {{&S38390[31], 3}, {NULL}};
const Segment SGI142692[] = {{&S38390[84], 6}, {NULL}};
const Segment SGI142693[] = {{&S38432[44], 7}, {NULL}};
const Segment SGI142694[] = {{&S38432[44], 24}, {NULL}};
const Segment SGI142695[] = {{&S38432[30], 6}, {NULL}};
const Segment SGI142696[] = {{&S38432[31], 3}, {NULL}};
const Segment SGI142697[] = {{&S38432[82], 6}, {NULL}};
const Segment SGI142698[] = {{&S38434[44], 7}, {NULL}};
const Segment SGI142699[] = {{&S38434[44], 24}, {NULL}};
const Segment SGI142700[] = {{&S38434[30], 6}, {NULL}};
const Segment SGI142701[] = {{&S38434[31], 3}, {NULL}};
const Segment SGI142702[] = {{&S38434[82], 6}, {NULL}};
const Segment SGI142703[] = {{&S38435[38], 7}, {NULL}};
const Segment SGI142704[] = {{&S38435[31], 6}, {NULL}};
const Segment SGI142705[] = {{&S38438[44], 7}, {NULL}};
const Segment SGI142706[] = {{&S38438[44], 24}, {NULL}};
const Segment SGI142707[] = {{&S38438[30], 6}, {NULL}};
const Segment SGI142708[] = {{&S38438[31], 3}, {NULL}};
const Segment SGI142709[] = {{&S38438[82], 6}, {NULL}};
const Segment SGI142710[] = {{&S38439[44], 7}, {NULL}};
const Segment SGI142711[] = {{&S38439[44], 24}, {NULL}};
const Segment SGI142712[] = {{&S38439[30], 6}, {NULL}};
const Segment SGI142713[] = {{&S38439[31], 3}, {NULL}};
const Segment SGI142714[] = {{&S38439[82], 6}, {NULL}};
const Segment SGI142715[] = {{&S38533[34], 8}, {NULL}};
const Segment SGI142716[] = {{&S38533[27], 6}, {NULL}};
const Segment SGI142717[] = {{&S38533[100], 9}, {NULL}};
const Segment SGI142718[] = {{&S38533[73], 6}, {NULL}};
const Segment SGI142719[] = {{&S38534[34], 7}, {NULL}};
const Segment SGI142720[] = {{&S38534[27], 6}, {NULL}};
const Segment SGI142721[] = {{&S38534[98], 9}, {NULL}};
const Segment SGI142722[] = {{&S38534[71], 6}, {NULL}};
const Segment SGI142723[] = {{&S38535[34], 7}, {NULL}};
const Segment SGI142724[] = {{&S38535[27], 6}, {NULL}};
const Segment SGI142725[] = {{&S38535[97], 9}, {NULL}};
const Segment SGI142726[] = {{&S38535[70], 6}, {NULL}};
const Segment SGI142727[] = {{&S32077[32], 9}, {NULL}};
const Segment SGI142728[] = {{&S32077[27], 4}, {NULL}};
const Segment SGI142729[] = {{&S32077[94], 9}, {NULL}};
const Segment SGI142730[] = {{&S32077[67], 6}, {NULL}};
const Segment SGI142731[] = {{&S32082[32], 15}, {NULL}};
const Segment SGI142732[] = {{&S32082[27], 4}, {NULL}};
const Segment SGI142733[] = {{&S32082[100], 9}, {NULL}};
const Segment SGI142734[] = {{&S32082[73], 6}, {NULL}};
const Segment SGI142735[] = {{&S32087[39], 9}, {NULL}};
const Segment SGI142736[] = {{&S32087[39], 21}, {NULL}};
const Segment SGI142737[] = {{&S32087[27], 4}, {NULL}};
const Segment SGI142738[] = {{&S32087[101], 9}, {NULL}};
const Segment SGI142739[] = {{&S32087[74], 6}, {NULL}};
const Segment SGI142740[] = {{&S29025[44], 10}, {NULL}};
const Segment SGI142741[] = {{&S29025[44], 22}, {NULL}};
const Segment SGI142742[] = {{&S29025[30], 6}, {NULL}};
const Segment SGI142743[] = {{&S29025[115], 3}, {NULL}};
const Segment SGI142744[] = {{&S29025[80], 6}, {NULL}};
const Segment SGI142745[] = {{&S29029[38], 10}, {NULL}};
const Segment SGI142746[] = {{&S29029[31], 6}, {NULL}};
const Segment SGI142747[] = {{&S28373[44], 10}, {NULL}};
const Segment SGI142748[] = {{&S28373[44], 22}, {NULL}};
const Segment SGI142749[] = {{&S28373[30], 6}, {NULL}};
const Segment SGI142750[] = {{&S28373[115], 3}, {NULL}};
const Segment SGI142751[] = {{&S28373[80], 6}, {NULL}};
const Segment SGI142752[] = {{&S28375[44], 10}, {NULL}};
const Segment SGI142753[] = {{&S28375[44], 22}, {NULL}};
const Segment SGI142754[] = {{&S28375[30], 6}, {NULL}};
const Segment SGI142755[] = {{&S28375[115], 3}, {NULL}};
const Segment SGI142756[] = {{&S28375[80], 6}, {NULL}};
const Segment SGI142757[] = {{&S28377[38], 10}, {NULL}};
const Segment SGI142758[] = {{&S28377[31], 6}, {NULL}};
const Segment SGI142759[] = {{&S28378[42], 10}, {NULL}};
const Segment SGI142760[] = {{&S28378[30], 6}, {NULL}};
const Segment SGI142761[] = {{&S28378[113], 3}, {NULL}};
const Segment SGI142762[] = {{&S28378[78], 6}, {NULL}};
const Segment SGI142763[] = {{&S26060[30], 1}, {NULL}};
const Segment SGI142764[] = {{&S26060[61], 8}, {NULL}};
const Segment SGI142765[] = {{&S26060[49], 4}, {NULL}};
const Segment SGI142766[] = {{&S26061[61], 9}, {NULL}};
const Segment SGI142767[] = {{&S26061[61], 21}, {NULL}};
const Segment SGI142768[] = {{&S26061[49], 4}, {NULL}};
const Segment SGI142769[] = {{&S26062[42], 9}, {NULL}};
const Segment SGI142770[] = {{&S26062[42], 20}, {NULL}};
const Segment SGI142771[] = {{&S26062[30], 4}, {NULL}};
const Segment SGI142772[] = {{&S26062[111], 3}, {NULL}};
const Segment SGI142773[] = {{&S26062[76], 6}, {NULL}};
const Segment SGI142774[] = {{&S26063[42], 9}, {NULL}};
const Segment SGI142775[] = {{&S26063[42], 21}, {NULL}};
const Segment SGI142776[] = {{&S26063[30], 4}, {NULL}};
const Segment SGI142777[] = {{&S26063[112], 3}, {NULL}};
const Segment SGI142778[] = {{&S26063[77], 6}, {NULL}};
const Segment SGI142779[] = {{&S26064[42], 9}, {NULL}};
const Segment SGI142780[] = {{&S26064[42], 21}, {NULL}};
const Segment SGI142781[] = {{&S26064[30], 4}, {NULL}};
const Segment SGI142782[] = {{&S26064[112], 3}, {NULL}};
const Segment SGI142783[] = {{&S26064[77], 6}, {NULL}};
const Segment SGI142784[] = {{&S26065[42], 9}, {NULL}};
const Segment SGI142785[] = {{&S26065[42], 21}, {NULL}};
const Segment SGI142786[] = {{&S26065[30], 4}, {NULL}};
const Segment SGI142787[] = {{&S26065[112], 3}, {NULL}};
const Segment SGI142788[] = {{&S26065[77], 6}, {NULL}};
const Segment SGI142789[] = {{&S26066[42], 9}, {NULL}};
const Segment SGI142790[] = {{&S26066[42], 21}, {NULL}};
const Segment SGI142791[] = {{&S26066[30], 4}, {NULL}};
const Segment SGI142792[] = {{&S26066[112], 3}, {NULL}};
const Segment SGI142793[] = {{&S26066[77], 6}, {NULL}};
const Segment SGI142794[] = {{&S26067[42], 9}, {NULL}};
const Segment SGI142795[] = {{&S26067[42], 21}, {NULL}};
const Segment SGI142796[] = {{&S26067[30], 4}, {NULL}};
const Segment SGI142797[] = {{&S26067[112], 3}, {NULL}};
const Segment SGI142798[] = {{&S26067[77], 6}, {NULL}};
const Segment SGI142799[] = {{&S26068[36], 9}, {NULL}};
const Segment SGI142800[] = {{&S26068[31], 4}, {NULL}};
const Segment SGI142801[] = {{&S26071[40], 9}, {NULL}};
const Segment SGI142802[] = {{&S26071[30], 4}, {NULL}};
const Segment SGI142803[] = {{&S26071[110], 3}, {NULL}};
const Segment SGI142804[] = {{&S26071[75], 6}, {NULL}};
const Segment SGI142805[] = {{&S28479[34], 11}, {NULL}};
const Segment SGI142806[] = {{&S28479[27], 6}, {NULL}};
const Segment SGI142807[] = {{&S38773[34], 10}, {NULL}};
const Segment SGI142808[] = {{&S38773[27], 6}, {NULL}};
const Segment SGI142809[] = {{&S28479[98], 9}, {NULL}};
const Segment SGI142810[] = {{&S28479[71], 6}, {NULL}};
const Segment SGI142811[] = {{&S38773[97], 9}, {NULL}};
const Segment SGI142812[] = {{&S38773[70], 6}, {NULL}};
const Segment SGI142813[] = {{&S28481[34], 11}, {NULL}};
const Segment SGI142814[] = {{&S28481[27], 6}, {NULL}};
const Segment SGI142815[] = {{&S38775[34], 10}, {NULL}};
const Segment SGI142816[] = {{&S38775[27], 6}, {NULL}};
const Segment SGI142817[] = {{&S28481[98], 9}, {NULL}};
const Segment SGI142818[] = {{&S28481[71], 6}, {NULL}};
const Segment SGI142819[] = {{&S38775[97], 9}, {NULL}};
const Segment SGI142820[] = {{&S38775[70], 6}, {NULL}};
const Segment SGI142821[] = {{&S28483[34], 11}, {NULL}};
const Segment SGI142822[] = {{&S28483[27], 6}, {NULL}};
const Segment SGI142823[] = {{&S28483[98], 9}, {NULL}};
const Segment SGI142824[] = {{&S28483[71], 6}, {NULL}};
const Segment SGI142825[] = {{&S38778[34], 11}, {NULL}};
const Segment SGI142826[] = {{&S38778[27], 6}, {NULL}};
const Segment SGI142827[] = {{&S28484[34], 10}, {NULL}};
const Segment SGI142828[] = {{&S38778[98], 9}, {NULL}};
const Segment SGI142829[] = {{&S38778[71], 6}, {NULL}};
const Segment SGI142830[] = {{&S28484[27], 6}, {NULL}};
const Segment SGI142831[] = {{&S28484[97], 9}, {NULL}};
const Segment SGI142832[] = {{&S28484[70], 6}, {NULL}};
const Segment SGI142833[] = {{&S38779[34], 11}, {NULL}};
const Segment SGI142834[] = {{&S38779[27], 6}, {NULL}};
const Segment SGI142835[] = {{&S28485[34], 11}, {NULL}};
const Segment SGI142836[] = {{&S28485[27], 6}, {NULL}};
const Segment SGI142837[] = {{&S38779[98], 9}, {NULL}};
const Segment SGI142838[] = {{&S38779[71], 6}, {NULL}};
const Segment SGI142839[] = {{&S28485[98], 9}, {NULL}};
const Segment SGI142840[] = {{&S28485[71], 6}, {NULL}};
const Segment SGI142841[] = {{&S28486[34], 11}, {NULL}};
const Segment SGI142842[] = {{&S28486[27], 6}, {NULL}};
const Segment SGI142843[] = {{&S28486[98], 9}, {NULL}};
const Segment SGI142844[] = {{&S28486[71], 6}, {NULL}};
const Segment SGI142845[] = {{&S28488[34], 10}, {NULL}};
const Segment SGI142846[] = {{&S28488[27], 6}, {NULL}};
const Segment SGI142847[] = {{&S28488[97], 9}, {NULL}};
const Segment SGI142848[] = {{&S28488[70], 6}, {NULL}};
const Segment SGI142849[] = {{&S28490[34], 10}, {NULL}};
const Segment SGI142850[] = {{&S28490[27], 6}, {NULL}};
const Segment SGI142851[] = {{&S28490[97], 9}, {NULL}};
const Segment SGI142852[] = {{&S28490[70], 6}, {NULL}};
const Segment SGI142853[] = {{&S28491[34], 10}, {NULL}};
const Segment SGI142854[] = {{&S28491[27], 6}, {NULL}};
const Segment SGI142855[] = {{&S28491[97], 9}, {NULL}};
const Segment SGI142856[] = {{&S28491[70], 6}, {NULL}};
const Segment SGI142857[] = {{&S28492[34], 10}, {NULL}};
const Segment SGI142858[] = {{&S28492[27], 6}, {NULL}};
const Segment SGI142859[] = {{&S28492[97], 9}, {NULL}};
const Segment SGI142860[] = {{&S28492[70], 6}, {NULL}};
const Segment SGI142861[] = {{&S28494[34], 18}, {NULL}};
const Segment SGI142862[] = {{&S28494[27], 6}, {NULL}};
const Segment SGI142863[] = {{&S28494[105], 9}, {NULL}};
const Segment SGI142864[] = {{&S28494[78], 6}, {NULL}};
const Segment SGI142865[] = {{&S28495[34], 11}, {NULL}};
const Segment SGI142866[] = {{&S28495[27], 6}, {NULL}};
const Segment SGI142867[] = {{&S38796[44], 11}, {NULL}};
const Segment SGI142868[] = {{&S28495[98], 9}, {NULL}};
const Segment SGI142869[] = {{&S28495[71], 6}, {NULL}};
const Segment SGI142870[] = {{&S38796[44], 23}, {NULL}};
const Segment SGI142871[] = {{&S38796[30], 6}, {NULL}};
const Segment SGI142872[] = {{&S38796[31], 3}, {NULL}};
const Segment SGI142873[] = {{&S38796[81], 6}, {NULL}};
const Segment SGI142874[] = {{&S38797[38], 11}, {NULL}};
const Segment SGI142875[] = {{&S38797[31], 6}, {NULL}};
const Segment SGI142876[] = {{&S28498[34], 10}, {NULL}};
const Segment SGI142877[] = {{&S28498[27], 6}, {NULL}};
const Segment SGI142878[] = {{&S28498[97], 9}, {NULL}};
const Segment SGI142879[] = {{&S28498[70], 6}, {NULL}};
const Segment SGI142880[] = {{&S38798[44], 11}, {NULL}};
const Segment SGI142881[] = {{&S38798[44], 23}, {NULL}};
const Segment SGI142882[] = {{&S38798[30], 6}, {NULL}};
const Segment SGI142883[] = {{&S38798[31], 3}, {&S38798[176], 3}, {NULL}};
const Segment SGI142884[] = {{&S38798[81], 6}, {NULL}};
const Segment SGI142885[] = {{&S28500[34], 11}, {NULL}};
const Segment SGI142886[] = {{&S28500[27], 6}, {NULL}};
const Segment SGI142887[] = {{&S38799[44], 11}, {NULL}};
const Segment SGI142888[] = {{&S28500[98], 9}, {NULL}};
const Segment SGI142889[] = {{&S28500[71], 6}, {NULL}};
const Segment SGI142890[] = {{&S38799[44], 23}, {NULL}};
const Segment SGI142891[] = {{&S38799[30], 6}, {NULL}};
const Segment SGI142892[] = {{&S38799[31], 3}, {&S38799[159], 3}, {NULL}};
const Segment SGI142893[] = {{&S38799[81], 6}, {NULL}};
const Segment SGI142894[] = {{&S28501[34], 10}, {NULL}};
const Segment SGI142895[] = {{&S28501[27], 6}, {NULL}};
const Segment SGI142896[] = {{&S28501[97], 9}, {NULL}};
const Segment SGI142897[] = {{&S28501[70], 6}, {NULL}};
const Segment SGI142898[] = {{&S38800[44], 10}, {NULL}};
const Segment SGI142899[] = {{&S38800[44], 22}, {NULL}};
const Segment SGI142900[] = {{&S38800[30], 6}, {NULL}};
const Segment SGI142901[] = {{&S38800[31], 3}, {NULL}};
const Segment SGI142902[] = {{&S38800[95], 6}, {NULL}};
const Segment SGI142903[] = {{&S38800[68], 8}, {NULL}};
const Segment SGI142904[] = {{&S28503[34], 11}, {NULL}};
const Segment SGI142905[] = {{&S28503[27], 6}, {NULL}};
const Segment SGI142906[] = {{&S28503[98], 9}, {NULL}};
const Segment SGI142907[] = {{&S38801[44], 10}, {NULL}};
const Segment SGI142908[] = {{&S28503[71], 6}, {NULL}};
const Segment SGI142909[] = {{&S38801[44], 22}, {NULL}};
const Segment SGI142910[] = {{&S38801[30], 6}, {NULL}};
const Segment SGI142911[] = {{&S38801[31], 3}, {NULL}};
const Segment SGI142912[] = {{&S38801[95], 6}, {NULL}};
const Segment SGI142913[] = {{&S38801[68], 8}, {NULL}};
const Segment SGI142914[] = {{&S28504[34], 10}, {NULL}};
const Segment SGI142915[] = {{&S28504[27], 6}, {NULL}};
const Segment SGI142916[] = {{&S28504[97], 9}, {NULL}};
const Segment SGI142917[] = {{&S28504[70], 6}, {NULL}};
const Segment SGI142918[] = {{&S38802[44], 21}, {NULL}};
const Segment SGI142919[] = {{&S38802[44], 33}, {NULL}};
const Segment SGI142920[] = {{&S38802[30], 6}, {NULL}};
const Segment SGI142921[] = {{&S38802[31], 3}, {NULL}};
const Segment SGI142922[] = {{&S38802[106], 6}, {NULL}};
const Segment SGI142923[] = {{&S28505[34], 10}, {NULL}};
const Segment SGI142924[] = {{&S38802[79], 8}, {NULL}};
const Segment SGI142925[] = {{&S28505[27], 6}, {NULL}};
const Segment SGI142926[] = {{&S28505[97], 9}, {NULL}};
const Segment SGI142927[] = {{&S28505[70], 6}, {NULL}};
const Segment SGI142928[] = {{&S38803[44], 21}, {NULL}};
const Segment SGI142929[] = {{&S38803[44], 33}, {NULL}};
const Segment SGI142930[] = {{&S38803[30], 6}, {NULL}};
const Segment SGI142931[] = {{&S38803[31], 3}, {NULL}};
const Segment SGI142932[] = {{&S38803[106], 6}, {NULL}};
const Segment SGI142933[] = {{&S38803[79], 8}, {NULL}};
const Segment SGI142934[] = {{&S28507[34], 10}, {NULL}};
const Segment SGI142935[] = {{&S28507[27], 6}, {NULL}};
const Segment SGI142936[] = {{&S28507[97], 9}, {NULL}};
const Segment SGI142937[] = {{&S28507[70], 6}, {NULL}};
const Segment SGI142938[] = {{&S38804[44], 11}, {NULL}};
const Segment SGI142939[] = {{&S38804[44], 23}, {NULL}};
const Segment SGI142940[] = {{&S38804[30], 6}, {NULL}};
const Segment SGI142941[] = {{&S38804[31], 3}, {NULL}};
const Segment SGI142942[] = {{&S38804[96], 6}, {NULL}};
const Segment SGI142943[] = {{&S38804[69], 8}, {NULL}};
const Segment SGI142944[] = {{&S28509[34], 18}, {NULL}};
const Segment SGI142945[] = {{&S38805[44], 10}, {NULL}};
const Segment SGI142946[] = {{&S28509[27], 6}, {NULL}};
const Segment SGI142947[] = {{&S38805[44], 22}, {NULL}};
const Segment SGI142948[] = {{&S38805[30], 6}, {NULL}};
const Segment SGI142949[] = {{&S28509[105], 9}, {NULL}};
const Segment SGI142950[] = {{&S38805[31], 3}, {NULL}};
const Segment SGI142951[] = {{&S28509[78], 6}, {NULL}};
const Segment SGI142952[] = {{&S38805[95], 6}, {NULL}};
const Segment SGI142953[] = {{&S38805[68], 8}, {NULL}};
const Segment SGI142954[] = {{&S38806[44], 21}, {NULL}};
const Segment SGI142955[] = {{&S38806[44], 33}, {NULL}};
const Segment SGI142956[] = {{&S38806[30], 6}, {NULL}};
const Segment SGI142957[] = {{&S38806[31], 3}, {NULL}};
const Segment SGI142958[] = {{&S28511[34], 11}, {NULL}};
const Segment SGI142959[] = {{&S38806[106], 6}, {NULL}};
const Segment SGI142960[] = {{&S28511[27], 6}, {NULL}};
const Segment SGI142961[] = {{&S38806[79], 8}, {NULL}};
const Segment SGI142962[] = {{&S28511[98], 9}, {NULL}};
const Segment SGI142963[] = {{&S28511[71], 6}, {NULL}};
const Segment SGI142964[] = {{&S38807[44], 10}, {NULL}};
const Segment SGI142965[] = {{&S38807[44], 22}, {NULL}};
const Segment SGI142966[] = {{&S38807[30], 6}, {NULL}};
const Segment SGI142967[] = {{&S38807[31], 3}, {NULL}};
const Segment SGI142968[] = {{&S38807[99], 6}, {NULL}};
const Segment SGI142969[] = {{&S38807[68], 8}, {NULL}};
const Segment SGI142970[] = {{&S28513[34], 10}, {NULL}};
const Segment SGI142971[] = {{&S28513[27], 6}, {NULL}};
const Segment SGI142972[] = {{&S28513[97], 9}, {NULL}};
const Segment SGI142973[] = {{&S28513[70], 6}, {NULL}};
const Segment SGI142974[] = {{&S38808[44], 11}, {NULL}};
const Segment SGI142975[] = {{&S38808[44], 23}, {NULL}};
const Segment SGI142976[] = {{&S38808[30], 6}, {NULL}};
const Segment SGI142977[] = {{&S38808[31], 3}, {NULL}};
const Segment SGI142978[] = {{&S38808[96], 6}, {NULL}};
const Segment SGI142979[] = {{&S38808[69], 8}, {NULL}};
const Segment SGI142980[] = {{&S28514[34], 11}, {NULL}};
const Segment SGI142981[] = {{&S28514[27], 6}, {NULL}};
const Segment SGI142982[] = {{&S28514[98], 9}, {NULL}};
const Segment SGI142983[] = {{&S28514[71], 6}, {NULL}};
const Segment SGI142984[] = {{&S28515[34], 10}, {NULL}};
const Segment SGI142985[] = {{&S28515[27], 6}, {NULL}};
const Segment SGI142986[] = {{&S28515[97], 9}, {NULL}};
const Segment SGI142987[] = {{&S28515[70], 6}, {NULL}};
const Segment SGI142988[] = {{&S28521[34], 10}, {NULL}};
const Segment SGI142989[] = {{&S28521[27], 6}, {NULL}};
const Segment SGI142990[] = {{&S28521[97], 9}, {NULL}};
const Segment SGI142991[] = {{&S28521[70], 6}, {NULL}};
const Segment SGI142992[] = {{&S28523[34], 11}, {NULL}};
const Segment SGI142993[] = {{&S28523[27], 6}, {NULL}};
const Segment SGI142994[] = {{&S28523[98], 9}, {NULL}};
const Segment SGI142995[] = {{&S28523[71], 6}, {NULL}};
const Segment SGI142996[] = {{&S28524[34], 11}, {NULL}};
const Segment SGI142997[] = {{&S28524[27], 6}, {NULL}};
const Segment SGI142998[] = {{&S28524[98], 9}, {NULL}};
const Segment SGI142999[] = {{&S28524[71], 6}, {NULL}};
const Segment SGI143000[] = {{&S38832[34], 11}, {NULL}};
const Segment SGI143001[] = {{&S38832[27], 6}, {NULL}};
const Segment SGI143002[] = {{&S38832[98], 9}, {NULL}};
const Segment SGI143003[] = {{&S38832[71], 6}, {NULL}};
const Segment SGI143004[] = {{&S38833[34], 10}, {NULL}};
const Segment SGI143005[] = {{&S38833[27], 6}, {NULL}};
const Segment SGI143006[] = {{&S38833[97], 9}, {NULL}};
const Segment SGI143007[] = {{&S38833[70], 6}, {NULL}};
const Segment SGI143008[] = {{&S38836[34], 18}, {NULL}};
const Segment SGI143009[] = {{&S38836[27], 6}, {NULL}};
const Segment SGI143010[] = {{&S38836[105], 9}, {NULL}};
const Segment SGI143011[] = {{&S38836[78], 6}, {NULL}};
const Segment SGI143012[] = {{&S38838[34], 10}, {NULL}};
const Segment SGI143013[] = {{&S38838[27], 6}, {NULL}};
const Segment SGI143014[] = {{&S38838[97], 9}, {NULL}};
const Segment SGI143015[] = {{&S38838[70], 6}, {NULL}};
const Segment SGI143016[] = {{&S38839[34], 11}, {NULL}};
const Segment SGI143017[] = {{&S38839[27], 6}, {NULL}};
const Segment SGI143018[] = {{&S38839[98], 9}, {NULL}};
const Segment SGI143019[] = {{&S38839[71], 6}, {NULL}};
const Segment SGI143020[] = {{&S38841[44], 10}, {NULL}};
const Segment SGI143021[] = {{&S38841[44], 22}, {NULL}};
const Segment SGI143022[] = {{&S38841[30], 6}, {NULL}};
const Segment SGI143023[] = {{&S38841[31], 3}, {NULL}};
const Segment SGI143024[] = {{&S38841[80], 6}, {NULL}};
const Segment SGI143025[] = {{&S38842[44], 30}, {NULL}};
const Segment SGI143026[] = {{&S38842[44], 42}, {NULL}};
const Segment SGI143027[] = {{&S38842[30], 6}, {NULL}};
const Segment SGI143028[] = {{&S38842[31], 3}, {NULL}};
const Segment SGI143029[] = {{&S38842[100], 6}, {NULL}};
const Segment SGI143030[] = {{&S38843[38], 10}, {NULL}};
const Segment SGI143031[] = {{&S38843[31], 6}, {NULL}};
const Segment SGI143032[] = {{&S38844[44], 11}, {NULL}};
const Segment SGI143033[] = {{&S38844[44], 23}, {NULL}};
const Segment SGI143034[] = {{&S38844[30], 6}, {NULL}};
const Segment SGI143035[] = {{&S38844[31], 3}, {NULL}};
const Segment SGI143036[] = {{&S38844[81], 6}, {NULL}};
const Segment SGI143037[] = {{&S38845[44], 11}, {NULL}};
const Segment SGI143038[] = {{&S38845[44], 23}, {NULL}};
const Segment SGI143039[] = {{&S38845[30], 6}, {NULL}};
const Segment SGI143040[] = {{&S38845[31], 3}, {NULL}};
const Segment SGI143041[] = {{&S38845[81], 6}, {NULL}};
const Segment SGI143042[] = {{&S38846[42], 10}, {NULL}};
const Segment SGI143043[] = {{&S38846[30], 6}, {NULL}};
const Segment SGI143044[] = {{&S38846[31], 3}, {NULL}};
const Segment SGI143045[] = {{&S38846[78], 6}, {NULL}};
const Segment SGI143046[] = {{&S38872[44], 11}, {NULL}};
const Segment SGI143047[] = {{&S38872[44], 23}, {NULL}};
const Segment SGI143048[] = {{&S38872[30], 6}, {NULL}};
const Segment SGI143049[] = {{&S38872[31], 3}, {NULL}};
const Segment SGI143050[] = {{&S38872[81], 6}, {NULL}};
const Segment SGI143051[] = {{&S38873[44], 18}, {NULL}};
const Segment SGI143052[] = {{&S38873[44], 30}, {NULL}};
const Segment SGI143053[] = {{&S38873[30], 6}, {NULL}};
const Segment SGI143054[] = {{&S38873[31], 3}, {NULL}};
const Segment SGI143055[] = {{&S38873[88], 6}, {NULL}};
const Segment SGI143056[] = {{&S38874[38], 11}, {NULL}};
const Segment SGI143057[] = {{&S38874[31], 6}, {NULL}};
const Segment SGI143058[] = {{&S38875[42], 11}, {NULL}};
const Segment SGI143059[] = {{&S38875[30], 6}, {NULL}};
const Segment SGI143060[] = {{&S38875[31], 3}, {NULL}};
const Segment SGI143061[] = {{&S38875[79], 6}, {NULL}};
const Segment SGI143062[] = {{&S38908[34], 11}, {NULL}};
const Segment SGI143063[] = {{&S38908[27], 6}, {NULL}};
const Segment SGI143064[] = {{&S38908[98], 9}, {NULL}};
const Segment SGI143065[] = {{&S38908[71], 6}, {NULL}};
const Segment SGI143066[] = {{&S38911[34], 10}, {NULL}};
const Segment SGI143067[] = {{&S38911[27], 6}, {NULL}};
const Segment SGI143068[] = {{&S38911[97], 9}, {NULL}};
const Segment SGI143069[] = {{&S38911[70], 6}, {NULL}};
const Segment SGI143070[] = {{&S38914[34], 10}, {NULL}};
const Segment SGI143071[] = {{&S38914[27], 6}, {NULL}};
const Segment SGI143072[] = {{&S38914[97], 9}, {NULL}};
const Segment SGI143073[] = {{&S38914[70], 6}, {NULL}};
const Segment SGI143074[] = {{&S38916[34], 10}, {NULL}};
const Segment SGI143075[] = {{&S38916[27], 6}, {NULL}};
const Segment SGI143076[] = {{&S38916[97], 9}, {NULL}};
const Segment SGI143077[] = {{&S38916[70], 6}, {NULL}};
const Segment SGI143078[] = {{&S38917[34], 11}, {NULL}};
const Segment SGI143079[] = {{&S38917[27], 6}, {NULL}};
const Segment SGI143080[] = {{&S38917[98], 9}, {NULL}};
const Segment SGI143081[] = {{&S38917[71], 6}, {NULL}};
const Segment SGI143082[] = {{&S38918[34], 11}, {NULL}};
const Segment SGI143083[] = {{&S38918[27], 6}, {NULL}};
const Segment SGI143084[] = {{&S38918[98], 9}, {NULL}};
const Segment SGI143085[] = {{&S38918[71], 6}, {NULL}};
const Segment SGI143086[] = {{&S38919[34], 11}, {NULL}};
const Segment SGI143087[] = {{&S38919[27], 6}, {NULL}};
const Segment SGI143088[] = {{&S38919[98], 9}, {NULL}};
const Segment SGI143089[] = {{&S38919[71], 6}, {NULL}};
const Segment SGI143090[] = {{&S38920[34], 8}, {NULL}};
const Segment SGI143091[] = {{&S38920[27], 6}, {NULL}};
const Segment SGI143092[] = {{&S38920[95], 9}, {NULL}};
const Segment SGI143093[] = {{&S38920[68], 6}, {NULL}};
const Segment SGI143094[] = {{&S38921[34], 11}, {NULL}};
const Segment SGI143095[] = {{&S38921[27], 6}, {NULL}};
const Segment SGI143096[] = {{&S38921[98], 9}, {NULL}};
const Segment SGI143097[] = {{&S38921[71], 6}, {NULL}};
const Segment SGI143098[] = {{&S38922[34], 11}, {NULL}};
const Segment SGI143099[] = {{&S38922[27], 6}, {NULL}};
const Segment SGI143100[] = {{&S38922[98], 9}, {NULL}};
const Segment SGI143101[] = {{&S38922[71], 6}, {NULL}};
const Segment SGI143102[] = {{&S38923[34], 11}, {NULL}};
const Segment SGI143103[] = {{&S38923[27], 6}, {NULL}};
const Segment SGI143104[] = {{&S38923[98], 9}, {NULL}};
const Segment SGI143105[] = {{&S38923[71], 6}, {NULL}};
const Segment SGI143106[] = {{&S38924[34], 8}, {NULL}};
const Segment SGI143107[] = {{&S38924[27], 6}, {NULL}};
const Segment SGI143108[] = {{&S38924[95], 9}, {NULL}};
const Segment SGI143109[] = {{&S38924[68], 6}, {NULL}};
const Segment SGI143110[] = {{&S29568[34], 10}, {NULL}};
const Segment SGI143111[] = {{&S29568[27], 6}, {NULL}};
const Segment SGI143112[] = {{&S29568[97], 9}, {NULL}};
const Segment SGI143113[] = {{&S29568[70], 6}, {NULL}};
const Segment SGI143114[] = {{&S29569[34], 10}, {NULL}};
const Segment SGI143115[] = {{&S29569[27], 6}, {NULL}};
const Segment SGI143116[] = {{&S29569[97], 9}, {NULL}};
const Segment SGI143117[] = {{&S29569[70], 6}, {NULL}};
const Segment SGI143118[] = {{&S29570[34], 11}, {NULL}};
const Segment SGI143119[] = {{&S29570[27], 6}, {NULL}};
const Segment SGI143120[] = {{&S29570[98], 9}, {NULL}};
const Segment SGI143121[] = {{&S29570[71], 6}, {NULL}};
const Segment SGI143122[] = {{&S29572[34], 10}, {NULL}};
const Segment SGI143123[] = {{&S29572[27], 6}, {NULL}};
const Segment SGI143124[] = {{&S29572[97], 9}, {NULL}};
const Segment SGI143125[] = {{&S29572[70], 6}, {NULL}};
const Segment SGI143126[] = {{&S29575[34], 11}, {NULL}};
const Segment SGI143127[] = {{&S29575[27], 6}, {NULL}};
const Segment SGI143128[] = {{&S29575[98], 9}, {NULL}};
const Segment SGI143129[] = {{&S29575[71], 6}, {NULL}};
const Segment SGI143130[] = {{&S29576[34], 10}, {NULL}};
const Segment SGI143131[] = {{&S29576[27], 6}, {NULL}};
const Segment SGI143132[] = {{&S29576[97], 9}, {NULL}};
const Segment SGI143133[] = {{&S29576[70], 6}, {NULL}};
const Segment SGI143134[] = {{&S29577[34], 11}, {NULL}};
const Segment SGI143135[] = {{&S29577[27], 6}, {NULL}};
const Segment SGI143136[] = {{&S29577[98], 9}, {NULL}};
const Segment SGI143137[] = {{&S29577[71], 6}, {NULL}};
const Segment SGI143138[] = {{&S29579[34], 10}, {NULL}};
const Segment SGI143139[] = {{&S29579[27], 6}, {NULL}};
const Segment SGI143140[] = {{&S29579[97], 9}, {NULL}};
const Segment SGI143141[] = {{&S29579[70], 6}, {NULL}};
const Segment SGI143142[] = {{&S29581[34], 7}, {NULL}};
const Segment SGI143143[] = {{&S29581[27], 6}, {NULL}};
const Segment SGI143144[] = {{&S29581[94], 9}, {NULL}};
const Segment SGI143145[] = {{&S29581[67], 6}, {NULL}};
const Segment SGI143146[] = {{&S39054[34], 2}, {NULL}};
const Segment SGI143147[] = {{&S39054[27], 6}, {NULL}};
const Segment SGI143148[] = {{&S39054[89], 9}, {NULL}};
const Segment SGI143149[] = {{&S39054[62], 6}, {NULL}};
const Segment SGI143150[] = {{&S39056[34], 2}, {NULL}};
const Segment SGI143151[] = {{&S39056[27], 6}, {NULL}};
const Segment SGI143152[] = {{&S39056[89], 9}, {NULL}};
const Segment SGI143153[] = {{&S39056[62], 6}, {NULL}};
const Segment SGI143154[] = {{&S39057[11], 1}, {NULL}};
const Segment SGI143155[] = {{&S39057[27], 6}, {NULL}};
const Segment SGI143156[] = {{&S39057[88], 9}, {NULL}};
const Segment SGI143157[] = {{&S39057[61], 6}, {NULL}};
const Segment SGI143158[] = {{&S39060[11], 1}, {NULL}};
const Segment SGI143159[] = {{&S39060[27], 14}, {NULL}};
const Segment SGI143160[] = {{&S39060[96], 9}, {NULL}};
const Segment SGI143161[] = {{&S39060[69], 6}, {NULL}};
const Segment SGI143162[] = {{&S39061[11], 1}, {NULL}};
const Segment SGI143163[] = {{&S39061[27], 6}, {NULL}};
const Segment SGI143164[] = {{&S39061[88], 9}, {NULL}};
const Segment SGI143165[] = {{&S39061[61], 6}, {NULL}};
const Segment SGI143166[] = {{&S39063[34], 5}, {NULL}};
const Segment SGI143167[] = {{&S39063[27], 6}, {NULL}};
const Segment SGI143168[] = {{&S39063[92], 9}, {NULL}};
const Segment SGI143169[] = {{&S39063[65], 6}, {NULL}};
const Segment SGI143170[] = {{&S39072[34], 21}, {NULL}};
const Segment SGI143171[] = {{&S39072[27], 6}, {NULL}};
const Segment SGI143172[] = {{&S39072[108], 9}, {NULL}};
const Segment SGI143173[] = {{&S39072[81], 6}, {NULL}};
const Segment SGI143174[] = {{&S39073[34], 9}, {NULL}};
const Segment SGI143175[] = {{&S39073[27], 6}, {NULL}};
const Segment SGI143176[] = {{&S39073[96], 9}, {NULL}};
const Segment SGI143177[] = {{&S39073[69], 6}, {NULL}};
const Segment SGI143178[] = {{&S39075[34], 16}, {NULL}};
const Segment SGI143179[] = {{&S39075[27], 6}, {NULL}};
const Segment SGI143180[] = {{&S39075[103], 9}, {NULL}};
const Segment SGI143181[] = {{&S39075[76], 6}, {NULL}};
const Segment SGI143182[] = {{&S39077[34], 5}, {NULL}};
const Segment SGI143183[] = {{&S39077[27], 6}, {NULL}};
const Segment SGI143184[] = {{&S39077[92], 9}, {NULL}};
const Segment SGI143185[] = {{&S39077[65], 6}, {NULL}};
const Segment SGI143186[] = {{&S39086[34], 9}, {NULL}};
const Segment SGI143187[] = {{&S39086[27], 6}, {NULL}};
const Segment SGI143188[] = {{&S39086[96], 9}, {NULL}};
const Segment SGI143189[] = {{&S39086[69], 6}, {NULL}};
const Segment SGI143190[] = {{&S39090[34], 7}, {NULL}};
const Segment SGI143191[] = {{&S39090[27], 6}, {NULL}};
const Segment SGI143192[] = {{&S39090[94], 9}, {NULL}};
const Segment SGI143193[] = {{&S39090[67], 6}, {NULL}};
const Segment SGI143194[] = {{&S39092[34], 21}, {NULL}};
const Segment SGI143195[] = {{&S39092[27], 6}, {NULL}};
const Segment SGI143196[] = {{&S39092[108], 9}, {NULL}};
const Segment SGI143197[] = {{&S39092[81], 6}, {NULL}};
const Segment SGI143198[] = {{&S39097[27], 6}, {NULL}};
const Segment SGI143199[] = {{&S39097[100], 9}, {NULL}};
const Segment SGI143200[] = {{&S39097[73], 6}, {NULL}};
const Segment SGI143201[] = {{&S39100[34], 16}, {NULL}};
const Segment SGI143202[] = {{&S39100[27], 6}, {NULL}};
const Segment SGI143203[] = {{&S39100[103], 9}, {NULL}};
const Segment SGI143204[] = {{&S39100[76], 6}, {NULL}};
const Segment SGI143205[] = {{&S39101[34], 8}, {NULL}};
const Segment SGI143206[] = {{&S39101[27], 6}, {NULL}};
const Segment SGI143207[] = {{&S39101[95], 9}, {NULL}};
const Segment SGI143208[] = {{&S39101[68], 6}, {NULL}};
const Segment SGI143209[] = {{&S39102[34], 16}, {NULL}};
const Segment SGI143210[] = {{&S39102[27], 6}, {NULL}};
const Segment SGI143211[] = {{&S39102[103], 9}, {NULL}};
const Segment SGI143212[] = {{&S39102[76], 6}, {NULL}};
const Segment SGI143213[] = {{&S39103[34], 6}, {NULL}};
const Segment SGI143214[] = {{&S39103[27], 6}, {NULL}};
const Segment SGI143215[] = {{&S39103[93], 9}, {NULL}};
const Segment SGI143216[] = {{&S39103[66], 6}, {NULL}};
const Segment SGI143217[] = {{&S39105[34], 8}, {NULL}};
const Segment SGI143218[] = {{&S39105[27], 6}, {NULL}};
const Segment SGI143219[] = {{&S39105[95], 9}, {NULL}};
const Segment SGI143220[] = {{&S39105[68], 6}, {NULL}};
const Segment SGI143221[] = {{&S39106[34], 9}, {NULL}};
const Segment SGI143222[] = {{&S39106[27], 6}, {NULL}};
const Segment SGI143223[] = {{&S39106[96], 9}, {NULL}};
const Segment SGI143224[] = {{&S39106[69], 6}, {NULL}};
const Segment SGI143225[] = {{&S39108[34], 9}, {NULL}};
const Segment SGI143226[] = {{&S39108[27], 6}, {NULL}};
const Segment SGI143227[] = {{&S39108[96], 9}, {NULL}};
const Segment SGI143228[] = {{&S39108[69], 6}, {NULL}};
const Segment SGI143229[] = {{&S39111[34], 8}, {NULL}};
const Segment SGI143230[] = {{&S39111[27], 6}, {NULL}};
const Segment SGI143231[] = {{&S39111[95], 9}, {NULL}};
const Segment SGI143232[] = {{&S39111[68], 6}, {NULL}};
const Segment SGI143233[] = {{&S39113[34], 16}, {NULL}};
const Segment SGI143234[] = {{&S39113[27], 6}, {NULL}};
const Segment SGI143235[] = {{&S39113[103], 9}, {NULL}};
const Segment SGI143236[] = {{&S39113[76], 6}, {NULL}};
const Segment SGI143237[] = {{&S39114[34], 10}, {NULL}};
const Segment SGI143238[] = {{&S39114[27], 6}, {NULL}};
const Segment SGI143239[] = {{&S39114[97], 9}, {NULL}};
const Segment SGI143240[] = {{&S39114[70], 6}, {NULL}};
const Segment SGI143241[] = {{&S39116[34], 16}, {NULL}};
const Segment SGI143242[] = {{&S39116[27], 6}, {NULL}};
const Segment SGI143243[] = {{&S39116[103], 9}, {NULL}};
const Segment SGI143244[] = {{&S39116[76], 6}, {NULL}};
const Segment SGI143245[] = {{&S39119[34], 5}, {NULL}};
const Segment SGI143246[] = {{&S39119[27], 6}, {NULL}};
const Segment SGI143247[] = {{&S39119[92], 9}, {NULL}};
const Segment SGI143248[] = {{&S39119[65], 6}, {NULL}};
const Segment SGI143249[] = {{&S39120[34], 21}, {NULL}};
const Segment SGI143250[] = {{&S39120[27], 6}, {NULL}};
const Segment SGI143251[] = {{&S39120[108], 9}, {NULL}};
const Segment SGI143252[] = {{&S39120[81], 6}, {NULL}};
const Segment SGI143253[] = {{&S39121[34], 10}, {NULL}};
const Segment SGI143254[] = {{&S39121[27], 6}, {NULL}};
const Segment SGI143255[] = {{&S39121[97], 9}, {NULL}};
const Segment SGI143256[] = {{&S39121[70], 6}, {NULL}};
const Segment SGI143257[] = {{&S39122[34], 29}, {NULL}};
const Segment SGI143258[] = {{&S39122[27], 6}, {NULL}};
const Segment SGI143259[] = {{&S39122[116], 9}, {NULL}};
const Segment SGI143260[] = {{&S39122[89], 6}, {NULL}};
const Segment SGI143261[] = {{&S39123[34], 6}, {NULL}};
const Segment SGI143262[] = {{&S39123[27], 6}, {NULL}};
const Segment SGI143263[] = {{&S39123[93], 9}, {NULL}};
const Segment SGI143264[] = {{&S39123[66], 6}, {NULL}};
const Segment SGI143265[] = {{&S39125[34], 20}, {NULL}};
const Segment SGI143266[] = {{&S39125[27], 6}, {NULL}};
const Segment SGI143267[] = {{&S39125[107], 9}, {NULL}};
const Segment SGI143268[] = {{&S39125[80], 6}, {NULL}};
const Segment SGI143269[] = {{&S39132[34], 28}, {NULL}};
const Segment SGI143270[] = {{&S39132[27], 6}, {NULL}};
const Segment SGI143271[] = {{&S39132[115], 9}, {NULL}};
const Segment SGI143272[] = {{&S39132[88], 6}, {NULL}};
const Segment SGI143273[] = {{&S39133[34], 8}, {NULL}};
const Segment SGI143274[] = {{&S39133[27], 6}, {NULL}};
const Segment SGI143275[] = {{&S39133[95], 9}, {NULL}};
const Segment SGI143276[] = {{&S39133[68], 6}, {NULL}};
const Segment SGI143277[] = {{&S39135[34], 23}, {NULL}};
const Segment SGI143278[] = {{&S39135[27], 6}, {NULL}};
const Segment SGI143279[] = {{&S39135[110], 9}, {NULL}};
const Segment SGI143280[] = {{&S39135[83], 6}, {NULL}};
const Segment SGI143281[] = {{&S39142[44], 7}, {NULL}};
const Segment SGI143282[] = {{&S39142[44], 30}, {NULL}};
const Segment SGI143283[] = {{&S39142[30], 6}, {NULL}};
const Segment SGI143284[] = {{&S39142[31], 3}, {NULL}};
const Segment SGI143285[] = {{&S39142[88], 6}, {NULL}};
const Segment SGI143286[] = {{&S39143[44], 7}, {NULL}};
const Segment SGI143287[] = {{&S39143[44], 30}, {NULL}};
const Segment SGI143288[] = {{&S39143[30], 6}, {NULL}};
const Segment SGI143289[] = {{&S39143[31], 3}, {NULL}};
const Segment SGI143290[] = {{&S39143[88], 6}, {NULL}};
const Segment SGI143291[] = {{&S39144[44], 7}, {NULL}};
const Segment SGI143292[] = {{&S39144[44], 33}, {NULL}};
const Segment SGI143293[] = {{&S39144[30], 6}, {NULL}};
const Segment SGI143294[] = {{&S39144[31], 3}, {NULL}};
const Segment SGI143295[] = {{&S39144[91], 6}, {NULL}};
const Segment SGI143296[] = {{&S39145[44], 7}, {NULL}};
const Segment SGI143297[] = {{&S39145[44], 30}, {NULL}};
const Segment SGI143298[] = {{&S39145[30], 6}, {NULL}};
const Segment SGI143299[] = {{&S39145[31], 3}, {NULL}};
const Segment SGI143300[] = {{&S39145[88], 6}, {NULL}};
const Segment SGI143301[] = {{&S39146[44], 7}, {NULL}};
const Segment SGI143302[] = {{&S39146[44], 30}, {NULL}};
const Segment SGI143303[] = {{&S39146[30], 6}, {NULL}};
const Segment SGI143304[] = {{&S39146[31], 3}, {NULL}};
const Segment SGI143305[] = {{&S39146[88], 6}, {NULL}};
const Segment SGI143306[] = {{&S39147[44], 7}, {NULL}};
const Segment SGI143307[] = {{&S39147[44], 33}, {NULL}};
const Segment SGI143308[] = {{&S39147[30], 6}, {NULL}};
const Segment SGI143309[] = {{&S39147[31], 3}, {NULL}};
const Segment SGI143310[] = {{&S39147[91], 6}, {NULL}};
const Segment SGI143311[] = {{&S39148[44], 7}, {NULL}};
const Segment SGI143312[] = {{&S39148[44], 30}, {NULL}};
const Segment SGI143313[] = {{&S39148[30], 6}, {NULL}};
const Segment SGI143314[] = {{&S39148[31], 3}, {NULL}};
const Segment SGI143315[] = {{&S39148[88], 6}, {NULL}};
const Segment SGI143316[] = {{&S39149[44], 7}, {NULL}};
const Segment SGI143317[] = {{&S39149[44], 33}, {NULL}};
const Segment SGI143318[] = {{&S39149[30], 6}, {NULL}};
const Segment SGI143319[] = {{&S39149[31], 3}, {NULL}};
const Segment SGI143320[] = {{&S39149[91], 6}, {NULL}};
const Segment SGI143321[] = {{&S39150[44], 7}, {NULL}};
const Segment SGI143322[] = {{&S39150[44], 33}, {NULL}};
const Segment SGI143323[] = {{&S39150[30], 6}, {NULL}};
const Segment SGI143324[] = {{&S39150[31], 3}, {NULL}};
const Segment SGI143325[] = {{&S39150[91], 6}, {NULL}};
const Segment SGI143326[] = {{&S39151[44], 7}, {NULL}};
const Segment SGI143327[] = {{&S39151[44], 32}, {NULL}};
const Segment SGI143328[] = {{&S39151[30], 6}, {NULL}};
const Segment SGI143329[] = {{&S39151[31], 3}, {NULL}};
const Segment SGI143330[] = {{&S39151[90], 6}, {NULL}};
const Segment SGI143331[] = {{&S39152[44], 7}, {NULL}};
const Segment SGI143332[] = {{&S39152[44], 30}, {NULL}};
const Segment SGI143333[] = {{&S39152[30], 6}, {NULL}};
const Segment SGI143334[] = {{&S39152[31], 3}, {NULL}};
const Segment SGI143335[] = {{&S39152[88], 6}, {NULL}};
const Segment SGI143336[] = {{&S39153[44], 7}, {NULL}};
const Segment SGI143337[] = {{&S39153[44], 32}, {NULL}};
const Segment SGI143338[] = {{&S39153[30], 6}, {NULL}};
const Segment SGI143339[] = {{&S39153[31], 3}, {NULL}};
const Segment SGI143340[] = {{&S39153[90], 6}, {NULL}};
const Segment SGI143341[] = {{&S39154[38], 7}, {NULL}};
const Segment SGI143342[] = {{&S39154[31], 6}, {NULL}};
const Segment SGI143343[] = {{&S39155[44], 7}, {NULL}};
const Segment SGI143344[] = {{&S39155[44], 29}, {NULL}};
const Segment SGI143345[] = {{&S39155[30], 6}, {NULL}};
const Segment SGI143346[] = {{&S39155[31], 3}, {NULL}};
const Segment SGI143347[] = {{&S39155[87], 6}, {NULL}};
const Segment SGI143348[] = {{&S39156[44], 7}, {NULL}};
const Segment SGI143349[] = {{&S39156[44], 30}, {NULL}};
const Segment SGI143350[] = {{&S39156[30], 6}, {NULL}};
const Segment SGI143351[] = {{&S39156[31], 3}, {NULL}};
const Segment SGI143352[] = {{&S39156[88], 6}, {NULL}};
const Segment SGI143353[] = {{&S39157[44], 7}, {NULL}};
const Segment SGI143354[] = {{&S39157[44], 20}, {NULL}};
const Segment SGI143355[] = {{&S39157[30], 6}, {NULL}};
const Segment SGI143356[] = {{&S39157[31], 3}, {NULL}};
const Segment SGI143357[] = {{&S39157[78], 5}, {NULL}};
const Segment SGI143358[] = {{&S34168[34], 5}, {NULL}};
const Segment SGI143359[] = {{&S34168[27], 6}, {NULL}};
const Segment SGI143360[] = {{&S34168[65], 6}, {NULL}};
const Segment SGI143361[] = {{&S29801[34], 14}, {NULL}};
const Segment SGI143362[] = {{&S29801[27], 6}, {NULL}};
const Segment SGI143363[] = {{&S29801[101], 9}, {NULL}};
const Segment SGI143364[] = {{&S29801[74], 6}, {NULL}};
const Segment SGI143365[] = {{&S29803[34], 14}, {NULL}};
const Segment SGI143366[] = {{&S29803[27], 6}, {NULL}};
const Segment SGI143367[] = {{&S29803[101], 9}, {NULL}};
const Segment SGI143368[] = {{&S29803[74], 6}, {NULL}};
const Segment SGI143369[] = {{&S29806[34], 15}, {NULL}};
const Segment SGI143370[] = {{&S29806[27], 6}, {NULL}};
const Segment SGI143371[] = {{&S29806[102], 9}, {NULL}};
const Segment SGI143372[] = {{&S29806[75], 6}, {NULL}};
const Segment SGI143373[] = {{&S29807[34], 10}, {NULL}};
const Segment SGI143374[] = {{&S29807[27], 6}, {NULL}};
const Segment SGI143375[] = {{&S29807[97], 9}, {NULL}};
const Segment SGI143376[] = {{&S29807[70], 6}, {NULL}};
const Segment SGI143377[] = {{&S29860[44], 80}, {NULL}};
const Segment SGI143378[] = {{&S29860[44], 101}, {NULL}};
const Segment SGI143379[] = {{&S29860[30], 6}, {NULL}};
const Segment SGI143380[] = {{&S29860[192], 3}, {NULL}};
const Segment SGI143381[] = {{&S29860[118], 5}, {NULL}};
const Segment SGI143382[] = {{&S29918[44], 7}, {NULL}};
const Segment SGI143383[] = {{&S29918[44], 19}, {NULL}};
const Segment SGI143384[] = {{&S29918[30], 6}, {NULL}};
const Segment SGI143385[] = {{&S29918[112], 3}, {NULL}};
const Segment SGI143386[] = {{&S29918[77], 6}, {NULL}};
const Segment SGI143387[] = {{&S29935[34], 11}, {NULL}};
const Segment SGI143388[] = {{&S29935[27], 6}, {NULL}};
const Segment SGI143389[] = {{&S29935[98], 9}, {NULL}};
const Segment SGI143390[] = {{&S29935[71], 6}, {NULL}};
const Segment SGI143391[] = {{&S29936[34], 10}, {NULL}};
const Segment SGI143392[] = {{&S29936[27], 6}, {NULL}};
const Segment SGI143393[] = {{&S29936[97], 9}, {NULL}};
const Segment SGI143394[] = {{&S29936[70], 6}, {NULL}};
const Segment SGI143395[] = {{&S29937[34], 10}, {NULL}};
const Segment SGI143396[] = {{&S29937[27], 6}, {NULL}};
const Segment SGI143397[] = {{&S29937[97], 9}, {NULL}};
const Segment SGI143398[] = {{&S29937[70], 6}, {NULL}};
const Segment SGI143399[] = {{&S29938[34], 11}, {NULL}};
const Segment SGI143400[] = {{&S29938[27], 6}, {NULL}};
const Segment SGI143401[] = {{&S29938[98], 9}, {NULL}};
const Segment SGI143402[] = {{&S29938[71], 6}, {NULL}};
const Segment SGI143403[] = {{&S29939[34], 10}, {NULL}};
const Segment SGI143404[] = {{&S29939[27], 6}, {NULL}};
const Segment SGI143405[] = {{&S29939[97], 9}, {NULL}};
const Segment SGI143406[] = {{&S29939[70], 6}, {NULL}};
const Segment SGI143407[] = {{&S29941[34], 10}, {NULL}};
const Segment SGI143408[] = {{&S29941[27], 6}, {NULL}};
const Segment SGI143409[] = {{&S29941[97], 9}, {NULL}};
const Segment SGI143410[] = {{&S29941[70], 6}, {NULL}};
const Segment SGI143411[] = {{&S29942[34], 10}, {NULL}};
const Segment SGI143412[] = {{&S29942[27], 6}, {NULL}};
const Segment SGI143413[] = {{&S29942[97], 9}, {NULL}};
const Segment SGI143414[] = {{&S29942[70], 6}, {NULL}};
const Segment SGI143415[] = {{&S34210[34], 4}, {NULL}};
const Segment SGI143416[] = {{&S34210[27], 6}, {NULL}};
const Segment SGI143417[] = {{&S34210[64], 6}, {NULL}};
const Segment SGI143418[] = {{&S29943[34], 10}, {NULL}};
const Segment SGI143419[] = {{&S29943[27], 6}, {NULL}};
const Segment SGI143420[] = {{&S29943[97], 9}, {NULL}};
const Segment SGI143421[] = {{&S29943[70], 6}, {NULL}};
const Segment SGI143422[] = {{&S29949[34], 10}, {NULL}};
const Segment SGI143423[] = {{&S29949[27], 6}, {NULL}};
const Segment SGI143424[] = {{&S29949[97], 9}, {NULL}};
const Segment SGI143425[] = {{&S29949[70], 6}, {NULL}};
const Segment SGI143426[] = {{&S29950[34], 11}, {NULL}};
const Segment SGI143427[] = {{&S29950[27], 6}, {NULL}};
const Segment SGI143428[] = {{&S29950[98], 9}, {NULL}};
const Segment SGI143429[] = {{&S29950[71], 6}, {NULL}};
const Segment SGI143430[] = {{&S29953[34], 11}, {NULL}};
const Segment SGI143431[] = {{&S29953[27], 6}, {NULL}};
const Segment SGI143432[] = {{&S29953[98], 9}, {NULL}};
const Segment SGI143433[] = {{&S29953[71], 6}, {NULL}};
const Segment SGI143434[] = {{&S29954[34], 7}, {NULL}};
const Segment SGI143435[] = {{&S29954[27], 6}, {NULL}};
const Segment SGI143436[] = {{&S29954[94], 9}, {NULL}};
const Segment SGI143437[] = {{&S29954[67], 6}, {NULL}};
const Segment SGI143438[] = {{&S29026[44], 10}, {NULL}};
const Segment SGI143439[] = {{&S29026[44], 22}, {NULL}};
const Segment SGI143440[] = {{&S29026[30], 6}, {NULL}};
const Segment SGI143441[] = {{&S29026[115], 3}, {NULL}};
const Segment SGI143442[] = {{&S29026[80], 6}, {NULL}};
const Segment SGI143443[] = {{&S29027[38], 10}, {NULL}};
const Segment SGI143444[] = {{&S29027[31], 6}, {NULL}};
const Segment SGI143445[] = {{&S29028[44], 10}, {NULL}};
const Segment SGI143446[] = {{&S29028[44], 22}, {NULL}};
const Segment SGI143447[] = {{&S29028[30], 6}, {NULL}};
const Segment SGI143448[] = {{&S29028[115], 3}, {NULL}};
const Segment SGI143449[] = {{&S29028[80], 6}, {NULL}};
const Segment SGI143450[] = {{&S29030[42], 10}, {NULL}};
const Segment SGI143451[] = {{&S29030[30], 6}, {NULL}};
const Segment SGI143452[] = {{&S29030[113], 3}, {NULL}};
const Segment SGI143453[] = {{&S29030[78], 6}, {NULL}};
const Segment SGI143454[] = {{&S39491[44], 10}, {NULL}};
const Segment SGI143455[] = {{&S39491[44], 22}, {NULL}};
const Segment SGI143456[] = {{&S39491[30], 6}, {NULL}};
const Segment SGI143457[] = {{&S39491[31], 3}, {NULL}};
const Segment SGI143458[] = {{&S39491[80], 6}, {NULL}};
const Segment SGI143459[] = {{&S39492[38], 10}, {NULL}};
const Segment SGI143460[] = {{&S39492[31], 6}, {NULL}};
const Segment SGI143461[] = {{&S39525[44], 6}, {NULL}};
const Segment SGI143462[] = {{&S39525[44], 18}, {NULL}};
const Segment SGI143463[] = {{&S39525[30], 6}, {NULL}};
const Segment SGI143464[] = {{&S39525[31], 3}, {NULL}};
const Segment SGI143465[] = {{&S39525[76], 6}, {NULL}};
const Segment SGI143466[] = {{&S39526[38], 6}, {NULL}};
const Segment SGI143467[] = {{&S39526[31], 6}, {NULL}};
const Segment SGI143468[] = {{&S39535[34], 11}, {NULL}};
const Segment SGI143469[] = {{&S39535[27], 6}, {NULL}};
const Segment SGI143470[] = {{&S39535[98], 9}, {NULL}};
const Segment SGI143471[] = {{&S39535[71], 6}, {NULL}};
const Segment SGI143472[] = {{&S39536[34], 7}, {NULL}};
const Segment SGI143473[] = {{&S39536[27], 6}, {NULL}};
const Segment SGI143474[] = {{&S39536[94], 9}, {NULL}};
const Segment SGI143475[] = {{&S39536[67], 6}, {NULL}};
const Segment SGI143476[] = {{&S39538[34], 7}, {NULL}};
const Segment SGI143477[] = {{&S39538[27], 6}, {NULL}};
const Segment SGI143478[] = {{&S39538[94], 9}, {NULL}};
const Segment SGI143479[] = {{&S39538[67], 6}, {NULL}};
const Segment SGI143480[] = {{&S39539[34], 12}, {NULL}};
const Segment SGI143481[] = {{&S39539[27], 6}, {NULL}};
const Segment SGI143482[] = {{&S39539[98], 9}, {NULL}};
const Segment SGI143483[] = {{&S39539[72], 6}, {NULL}};
const Segment SGI143484[] = {{&S30224[34], 18}, {NULL}};
const Segment SGI143485[] = {{&S30224[27], 6}, {NULL}};
const Segment SGI143486[] = {{&S30224[105], 9}, {NULL}};
const Segment SGI143487[] = {{&S30224[78], 6}, {NULL}};
const Segment SGI143488[] = {{&S30226[34], 11}, {NULL}};
const Segment SGI143489[] = {{&S30226[27], 6}, {NULL}};
const Segment SGI143490[] = {{&S30226[98], 9}, {NULL}};
const Segment SGI143491[] = {{&S30226[71], 6}, {NULL}};
const Segment SGI143492[] = {{&S30228[34], 18}, {NULL}};
const Segment SGI143493[] = {{&S30228[27], 6}, {NULL}};
const Segment SGI143494[] = {{&S30228[105], 9}, {NULL}};
const Segment SGI143495[] = {{&S30228[78], 6}, {NULL}};
const Segment SGI143496[] = {{&S30231[34], 10}, {NULL}};
const Segment SGI143497[] = {{&S30231[27], 6}, {NULL}};
const Segment SGI143498[] = {{&S30231[97], 9}, {NULL}};
const Segment SGI143499[] = {{&S30231[70], 6}, {NULL}};
const Segment SGI143500[] = {{&S30234[34], 10}, {NULL}};
const Segment SGI143501[] = {{&S30234[27], 6}, {NULL}};
const Segment SGI143502[] = {{&S30234[97], 9}, {NULL}};
const Segment SGI143503[] = {{&S30234[70], 6}, {NULL}};
const Segment SGI143504[] = {{&S30237[34], 11}, {NULL}};
const Segment SGI143505[] = {{&S30237[27], 6}, {NULL}};
const Segment SGI143506[] = {{&S30237[98], 9}, {NULL}};
const Segment SGI143507[] = {{&S30237[71], 6}, {NULL}};
const Segment SGI143508[] = {{&S30240[34], 10}, {NULL}};
const Segment SGI143509[] = {{&S30240[27], 6}, {NULL}};
const Segment SGI143510[] = {{&S30240[97], 9}, {NULL}};
const Segment SGI143511[] = {{&S30240[70], 6}, {NULL}};
const Segment SGI143512[] = {{&S30241[34], 11}, {NULL}};
const Segment SGI143513[] = {{&S30241[27], 6}, {NULL}};
const Segment SGI143514[] = {{&S30241[98], 9}, {NULL}};
const Segment SGI143515[] = {{&S30241[71], 6}, {NULL}};
const Segment SGI143516[] = {{&S30242[34], 11}, {NULL}};
const Segment SGI143517[] = {{&S30242[27], 6}, {NULL}};
const Segment SGI143518[] = {{&S30242[98], 9}, {NULL}};
const Segment SGI143519[] = {{&S30242[71], 6}, {NULL}};
const Segment SGI143520[] = {{&S30281[44], 25}, {NULL}};
const Segment SGI143521[] = {{&S30281[44], 37}, {NULL}};
const Segment SGI143522[] = {{&S30281[30], 6}, {NULL}};
const Segment SGI143523[] = {{&S30281[130], 3}, {NULL}};
const Segment SGI143524[] = {{&S30281[95], 6}, {NULL}};
const Segment SGI143525[] = {{&S30282[44], 25}, {NULL}};
const Segment SGI143526[] = {{&S30282[44], 37}, {NULL}};
const Segment SGI143527[] = {{&S30282[30], 6}, {NULL}};
const Segment SGI143528[] = {{&S30282[130], 3}, {NULL}};
const Segment SGI143529[] = {{&S30282[95], 6}, {NULL}};
const Segment SGI143530[] = {{&S30283[38], 25}, {NULL}};
const Segment SGI143531[] = {{&S30283[31], 6}, {NULL}};
const Segment SGI143532[] = {{&S30285[44], 7}, {NULL}};
const Segment SGI143533[] = {{&S30285[44], 19}, {NULL}};
const Segment SGI143534[] = {{&S30285[30], 6}, {NULL}};
const Segment SGI143535[] = {{&S30285[112], 3}, {NULL}};
const Segment SGI143536[] = {{&S30285[77], 6}, {NULL}};
const Segment SGI143537[] = {{&S30286[42], 25}, {NULL}};
const Segment SGI143538[] = {{&S30286[30], 6}, {NULL}};
const Segment SGI143539[] = {{&S30286[128], 3}, {NULL}};
const Segment SGI143540[] = {{&S30286[93], 6}, {NULL}};
const Segment SGI143541[] = {{&S39630[44], 10}, {NULL}};
const Segment SGI143542[] = {{&S39630[44], 22}, {NULL}};
const Segment SGI143543[] = {{&S39630[30], 6}, {NULL}};
const Segment SGI143544[] = {{&S39630[31], 3}, {NULL}};
const Segment SGI143545[] = {{&S39630[80], 6}, {NULL}};
const Segment SGI143546[] = {{&S39631[42], 10}, {NULL}};
const Segment SGI143547[] = {{&S39631[30], 6}, {NULL}};
const Segment SGI143548[] = {{&S39631[31], 3}, {NULL}};
const Segment SGI143549[] = {{&S39631[78], 6}, {NULL}};
const Segment SGI143550[] = {{&S39632[38], 10}, {NULL}};
const Segment SGI143551[] = {{&S39632[31], 6}, {NULL}};
const Segment SGI143552[] = {{&S39633[44], 11}, {NULL}};
const Segment SGI143553[] = {{&S39633[44], 23}, {NULL}};
const Segment SGI143554[] = {{&S39633[30], 6}, {NULL}};
const Segment SGI143555[] = {{&S39633[31], 3}, {NULL}};
const Segment SGI143556[] = {{&S39633[81], 6}, {NULL}};
const Segment SGI143557[] = {{&S39634[44], 30}, {NULL}};
const Segment SGI143558[] = {{&S39634[44], 42}, {NULL}};
const Segment SGI143559[] = {{&S39634[30], 6}, {NULL}};
const Segment SGI143560[] = {{&S39634[31], 3}, {NULL}};
const Segment SGI143561[] = {{&S39634[100], 6}, {NULL}};
const Segment SGI143562[] = {{&S39635[44], 11}, {NULL}};
const Segment SGI143563[] = {{&S39635[44], 23}, {NULL}};
const Segment SGI143564[] = {{&S39635[30], 6}, {NULL}};
const Segment SGI143565[] = {{&S39635[31], 3}, {NULL}};
const Segment SGI143566[] = {{&S39635[81], 6}, {NULL}};
const Segment SGI143567[] = {{&S39636[44], 30}, {NULL}};
const Segment SGI143568[] = {{&S39636[44], 42}, {NULL}};
const Segment SGI143569[] = {{&S39636[30], 6}, {NULL}};
const Segment SGI143570[] = {{&S39636[31], 3}, {NULL}};
const Segment SGI143571[] = {{&S39636[100], 6}, {NULL}};
const Segment SGI143572[] = {{&S39637[44], 30}, {NULL}};
const Segment SGI143573[] = {{&S39637[44], 42}, {NULL}};
const Segment SGI143574[] = {{&S39637[30], 6}, {NULL}};
const Segment SGI143575[] = {{&S39637[31], 3}, {NULL}};
const Segment SGI143576[] = {{&S39637[100], 6}, {NULL}};
const Segment SGI143577[] = {{&S39638[38], 11}, {NULL}};
const Segment SGI143578[] = {{&S39638[31], 6}, {NULL}};
const Segment SGI143579[] = {{&S39647[34], 7}, {NULL}};
const Segment SGI143580[] = {{&S39647[27], 6}, {NULL}};
const Segment SGI143581[] = {{&S39647[98], 9}, {NULL}};
const Segment SGI143582[] = {{&S39647[71], 6}, {NULL}};
const Segment SGI143583[] = {{&S39648[34], 7}, {NULL}};
const Segment SGI143584[] = {{&S39648[27], 6}, {NULL}};
const Segment SGI143585[] = {{&S39648[99], 9}, {NULL}};
const Segment SGI143586[] = {{&S39648[72], 6}, {NULL}};
const Segment SGI143587[] = {{&S39671[34], 11}, {NULL}};
const Segment SGI143588[] = {{&S39671[27], 6}, {NULL}};
const Segment SGI143589[] = {{&S39671[98], 9}, {NULL}};
const Segment SGI143590[] = {{&S39671[71], 6}, {NULL}};
const Segment SGI143591[] = {{&S39672[34], 11}, {NULL}};
const Segment SGI143592[] = {{&S39672[27], 6}, {NULL}};
const Segment SGI143593[] = {{&S39672[98], 9}, {NULL}};
const Segment SGI143594[] = {{&S39672[71], 6}, {NULL}};
const Segment SGI143595[] = {{&S39673[34], 12}, {NULL}};
const Segment SGI143596[] = {{&S39673[27], 6}, {NULL}};
const Segment SGI143597[] = {{&S39673[99], 9}, {NULL}};
const Segment SGI143598[] = {{&S39673[72], 6}, {NULL}};
const Segment SGI143599[] = {{&S39674[34], 12}, {NULL}};
const Segment SGI143600[] = {{&S39674[27], 6}, {NULL}};
const Segment SGI143601[] = {{&S39674[99], 9}, {NULL}};
const Segment SGI143602[] = {{&S39674[72], 6}, {NULL}};
const Segment SGI143603[] = {{&S39675[34], 12}, {NULL}};
const Segment SGI143604[] = {{&S39675[27], 6}, {NULL}};
const Segment SGI143605[] = {{&S39675[99], 9}, {NULL}};
const Segment SGI143606[] = {{&S39675[72], 6}, {NULL}};
const Segment SGI143607[] = {{&S39682[44], 10}, {NULL}};
const Segment SGI143608[] = {{&S39682[44], 22}, {NULL}};
const Segment SGI143609[] = {{&S39682[30], 6}, {NULL}};
const Segment SGI143610[] = {{&S39682[31], 3}, {NULL}};
const Segment SGI143611[] = {{&S39682[80], 6}, {NULL}};
const Segment SGI143612[] = {{&S39683[38], 10}, {NULL}};
const Segment SGI143613[] = {{&S39683[31], 6}, {NULL}};
const Segment SGI143614[] = {{&S39702[44], 7}, {NULL}};
const Segment SGI143615[] = {{&S39702[44], 35}, {NULL}};
const Segment SGI143616[] = {{&S39702[30], 6}, {NULL}};
const Segment SGI143617[] = {{&S39702[31], 3}, {NULL}};
const Segment SGI143618[] = {{&S39702[93], 6}, {NULL}};
const Segment SGI143619[] = {{&S39703[44], 7}, {NULL}};
const Segment SGI143620[] = {{&S39703[44], 36}, {NULL}};
const Segment SGI143621[] = {{&S39703[30], 6}, {NULL}};
const Segment SGI143622[] = {{&S39703[31], 3}, {NULL}};
const Segment SGI143623[] = {{&S39703[94], 6}, {NULL}};
const Segment SGI143624[] = {{&S39704[44], 7}, {NULL}};
const Segment SGI143625[] = {{&S39704[44], 36}, {NULL}};
const Segment SGI143626[] = {{&S39704[30], 6}, {NULL}};
const Segment SGI143627[] = {{&S39704[31], 3}, {NULL}};
const Segment SGI143628[] = {{&S39704[94], 6}, {NULL}};
const Segment SGI143629[] = {{&S39705[44], 7}, {NULL}};
const Segment SGI143630[] = {{&S39705[44], 36}, {NULL}};
const Segment SGI143631[] = {{&S39705[30], 6}, {NULL}};
const Segment SGI143632[] = {{&S39705[31], 3}, {NULL}};
const Segment SGI143633[] = {{&S39705[94], 6}, {NULL}};
const Segment SGI143634[] = {{&S39706[44], 7}, {NULL}};
const Segment SGI143635[] = {{&S39706[44], 36}, {NULL}};
const Segment SGI143636[] = {{&S39706[30], 6}, {NULL}};
const Segment SGI143637[] = {{&S39706[31], 3}, {NULL}};
const Segment SGI143638[] = {{&S39706[94], 6}, {NULL}};
const Segment SGI143639[] = {{&S39707[38], 7}, {NULL}};
const Segment SGI143640[] = {{&S39707[31], 6}, {NULL}};
const Segment SGI143641[] = {{&S39709[44], 8}, {NULL}};
const Segment SGI143642[] = {{&S39709[44], 23}, {NULL}};
const Segment SGI143643[] = {{&S39709[30], 6}, {NULL}};
const Segment SGI143644[] = {{&S39709[31], 3}, {NULL}};
const Segment SGI143645[] = {{&S39709[81], 6}, {NULL}};
const Segment SGI143646[] = {{&S39710[44], 8}, {NULL}};
const Segment SGI143647[] = {{&S39710[44], 24}, {NULL}};
const Segment SGI143648[] = {{&S39710[30], 6}, {NULL}};
const Segment SGI143649[] = {{&S39710[85], 6}, {NULL}};
const Segment SGI143650[] = {{&S39711[38], 8}, {NULL}};
const Segment SGI143651[] = {{&S39711[31], 6}, {NULL}};
const Segment SGI143652[] = {{&S39712[44], 8}, {NULL}};
const Segment SGI143653[] = {{&S39712[44], 24}, {NULL}};
const Segment SGI143654[] = {{&S39712[30], 6}, {NULL}};
const Segment SGI143655[] = {{&S39712[31], 3}, {NULL}};
const Segment SGI143656[] = {{&S39712[82], 6}, {NULL}};
const Segment SGI143657[] = {{&S39713[44], 8}, {NULL}};
const Segment SGI143658[] = {{&S39713[44], 24}, {NULL}};
const Segment SGI143659[] = {{&S39713[30], 6}, {NULL}};
const Segment SGI143660[] = {{&S39713[31], 3}, {NULL}};
const Segment SGI143661[] = {{&S39713[82], 6}, {NULL}};
const Segment SGI143662[] = {{&S39714[44], 8}, {NULL}};
const Segment SGI143663[] = {{&S39714[44], 24}, {NULL}};
const Segment SGI143664[] = {{&S39714[30], 6}, {NULL}};
const Segment SGI143665[] = {{&S39714[31], 3}, {NULL}};
const Segment SGI143666[] = {{&S39714[82], 6}, {NULL}};
const Segment SGI143667[] = {{&S30443[34], 15}, {NULL}};
const Segment SGI143668[] = {{&S30443[27], 6}, {NULL}};
const Segment SGI143669[] = {{&S30443[105], 9}, {NULL}};
const Segment SGI143670[] = {{&S30443[78], 6}, {NULL}};
const Segment SGI143671[] = {{&S30444[34], 7}, {NULL}};
const Segment SGI143672[] = {{&S30444[27], 6}, {NULL}};
const Segment SGI143673[] = {{&S30444[110], 9}, {NULL}};
const Segment SGI143674[] = {{&S30444[83], 6}, {NULL}};
const Segment SGI143675[] = {{&S29564[34], 10}, {NULL}};
const Segment SGI143676[] = {{&S29564[27], 6}, {NULL}};
const Segment SGI143677[] = {{&S29564[97], 9}, {NULL}};
const Segment SGI143678[] = {{&S29564[70], 6}, {NULL}};
const Segment SGI143679[] = {{&S29565[34], 10}, {NULL}};
const Segment SGI143680[] = {{&S29565[27], 6}, {NULL}};
const Segment SGI143681[] = {{&S29565[97], 9}, {NULL}};
const Segment SGI143682[] = {{&S29565[70], 6}, {NULL}};
const Segment SGI143683[] = {{&S29566[34], 10}, {NULL}};
const Segment SGI143684[] = {{&S29566[27], 6}, {NULL}};
const Segment SGI143685[] = {{&S29566[97], 9}, {NULL}};
const Segment SGI143686[] = {{&S29566[70], 6}, {NULL}};
const Segment SGI143687[] = {{&S29567[34], 10}, {NULL}};
const Segment SGI143688[] = {{&S29567[27], 6}, {NULL}};
const Segment SGI143689[] = {{&S29567[97], 9}, {NULL}};
const Segment SGI143690[] = {{&S29567[70], 6}, {NULL}};
const Segment SGI143691[] = {{&S29571[34], 11}, {NULL}};
const Segment SGI143692[] = {{&S29571[27], 6}, {NULL}};
const Segment SGI143693[] = {{&S29571[98], 9}, {NULL}};
const Segment SGI143694[] = {{&S29571[71], 6}, {NULL}};
const Segment SGI143695[] = {{&S29574[34], 11}, {NULL}};
const Segment SGI143696[] = {{&S29574[27], 6}, {NULL}};
const Segment SGI143697[] = {{&S29574[98], 9}, {NULL}};
const Segment SGI143698[] = {{&S29574[71], 6}, {NULL}};
const Segment SGI143699[] = {{&S29578[34], 10}, {NULL}};
const Segment SGI143700[] = {{&S29578[27], 6}, {NULL}};
const Segment SGI143701[] = {{&S29578[97], 9}, {NULL}};
const Segment SGI143702[] = {{&S29578[70], 6}, {NULL}};
const Segment SGI143703[] = {{&S29580[34], 8}, {NULL}};
const Segment SGI143704[] = {{&S29580[27], 6}, {NULL}};
const Segment SGI143705[] = {{&S29580[95], 9}, {NULL}};
const Segment SGI143706[] = {{&S29580[68], 6}, {NULL}};
const Segment SGI143707[] = {{&S29583[34], 11}, {NULL}};
const Segment SGI143708[] = {{&S29583[27], 6}, {NULL}};
const Segment SGI143709[] = {{&S29583[98], 9}, {NULL}};
const Segment SGI143710[] = {{&S29583[71], 6}, {NULL}};
const Segment SGI143711[] = {{&S39893[44], 19}, {NULL}};
const Segment SGI143712[] = {{&S39893[44], 31}, {NULL}};
const Segment SGI143713[] = {{&S39893[30], 6}, {NULL}};
const Segment SGI143714[] = {{&S39893[31], 3}, {NULL}};
const Segment SGI143715[] = {{&S39893[89], 6}, {NULL}};
const Segment SGI143716[] = {{&S39894[38], 19}, {NULL}};
const Segment SGI143717[] = {{&S39894[31], 6}, {NULL}};
const Segment SGI143718[] = {{&S39895[44], 19}, {NULL}};
const Segment SGI143719[] = {{&S39895[44], 31}, {NULL}};
const Segment SGI143720[] = {{&S39895[30], 6}, {NULL}};
const Segment SGI143721[] = {{&S39895[31], 3}, {NULL}};
const Segment SGI143722[] = {{&S39895[89], 6}, {NULL}};
const Segment SGI143723[] = {{&S39896[38], 19}, {NULL}};
const Segment SGI143724[] = {{&S39896[31], 6}, {NULL}};
const Segment SGI143725[] = {{&S39954[44], 10}, {NULL}};
const Segment SGI143726[] = {{&S39954[44], 22}, {NULL}};
const Segment SGI143727[] = {{&S39954[30], 6}, {NULL}};
const Segment SGI143728[] = {{&S39954[31], 3}, {NULL}};
const Segment SGI143729[] = {{&S39954[80], 6}, {NULL}};
const Segment SGI143730[] = {{&S39955[38], 10}, {NULL}};
const Segment SGI143731[] = {{&S39955[31], 6}, {NULL}};
const Segment SGI143732[] = {{&S39956[44], 10}, {NULL}};
const Segment SGI143733[] = {{&S39956[44], 22}, {NULL}};
const Segment SGI143734[] = {{&S39956[30], 6}, {NULL}};
const Segment SGI143735[] = {{&S39956[31], 3}, {NULL}};
const Segment SGI143736[] = {{&S39956[80], 6}, {NULL}};
const Segment SGI143737[] = {{&S40091[34], 10}, {NULL}};
const Segment SGI143738[] = {{&S40091[27], 6}, {NULL}};
const Segment SGI143739[] = {{&S40091[97], 9}, {NULL}};
const Segment SGI143740[] = {{&S40091[70], 6}, {NULL}};
const Segment SGI143741[] = {{&S29804[34], 14}, {NULL}};
const Segment SGI143742[] = {{&S29804[27], 6}, {NULL}};
const Segment SGI143743[] = {{&S29804[101], 9}, {NULL}};
const Segment SGI143744[] = {{&S29804[74], 6}, {NULL}};
const Segment SGI143745[] = {{&S29805[34], 14}, {NULL}};
const Segment SGI143746[] = {{&S29805[27], 6}, {NULL}};
const Segment SGI143747[] = {{&S29805[101], 9}, {NULL}};
const Segment SGI143748[] = {{&S29805[74], 6}, {NULL}};
const Segment SGI143749[] = {{&S29808[34], 15}, {NULL}};
const Segment SGI143750[] = {{&S29808[27], 6}, {NULL}};
const Segment SGI143751[] = {{&S29808[102], 9}, {NULL}};
const Segment SGI143752[] = {{&S29808[75], 6}, {NULL}};
const Segment SGI143753[] = {{&S40205[34], 7}, {NULL}};
const Segment SGI143754[] = {{&S40205[27], 6}, {NULL}};
const Segment SGI143755[] = {{&S40205[98], 9}, {NULL}};
const Segment SGI143756[] = {{&S40205[71], 6}, {NULL}};
const Segment SGI143757[] = {{&S40206[34], 7}, {NULL}};
const Segment SGI143758[] = {{&S40206[27], 6}, {NULL}};
const Segment SGI143759[] = {{&S40206[98], 9}, {NULL}};
const Segment SGI143760[] = {{&S40206[71], 6}, {NULL}};
const Segment SGI143761[] = {{&S40211[34], 9}, {NULL}};
const Segment SGI143762[] = {{&S40211[27], 6}, {NULL}};
const Segment SGI143763[] = {{&S40211[96], 9}, {NULL}};
const Segment SGI143764[] = {{&S40211[69], 6}, {NULL}};
const Segment SGI143765[] = {{&S40212[34], 9}, {NULL}};
const Segment SGI143766[] = {{&S40212[27], 6}, {NULL}};
const Segment SGI143767[] = {{&S40212[96], 9}, {NULL}};
const Segment SGI143768[] = {{&S40212[69], 6}, {NULL}};
const Segment SGI143769[] = {{&S40213[34], 9}, {NULL}};
const Segment SGI143770[] = {{&S40213[27], 6}, {NULL}};
const Segment SGI143771[] = {{&S40213[96], 9}, {NULL}};
const Segment SGI143772[] = {{&S40213[69], 6}, {NULL}};
const Segment SGI143773[] = {{&S29852[44], 10}, {NULL}};
const Segment SGI143774[] = {{&S29852[44], 22}, {NULL}};
const Segment SGI143775[] = {{&S29852[30], 6}, {NULL}};
const Segment SGI143776[] = {{&S29852[115], 3}, {NULL}};
const Segment SGI143777[] = {{&S29852[80], 6}, {NULL}};
const Segment SGI143778[] = {{&S29853[38], 10}, {NULL}};
const Segment SGI143779[] = {{&S29853[31], 6}, {NULL}};
const Segment SGI143780[] = {{&S29854[44], 11}, {NULL}};
const Segment SGI143781[] = {{&S29854[44], 23}, {NULL}};
const Segment SGI143782[] = {{&S29854[30], 6}, {NULL}};
const Segment SGI143783[] = {{&S29854[116], 3}, {NULL}};
const Segment SGI143784[] = {{&S29854[81], 6}, {NULL}};
const Segment SGI143785[] = {{&S29855[42], 10}, {NULL}};
const Segment SGI143786[] = {{&S29855[30], 6}, {NULL}};
const Segment SGI143787[] = {{&S29855[113], 3}, {NULL}};
const Segment SGI143788[] = {{&S29855[78], 6}, {NULL}};
const Segment SGI143789[] = {{&S29856[44], 10}, {NULL}};
const Segment SGI143790[] = {{&S29856[44], 22}, {NULL}};
const Segment SGI143791[] = {{&S29856[30], 6}, {NULL}};
const Segment SGI143792[] = {{&S29856[127], 3}, {NULL}};
const Segment SGI143793[] = {{&S29856[80], 6}, {NULL}};
const Segment SGI143794[] = {{&S29857[38], 11}, {NULL}};
const Segment SGI143795[] = {{&S29857[31], 6}, {NULL}};
const Segment SGI143796[] = {{&S29858[44], 11}, {NULL}};
const Segment SGI143797[] = {{&S29858[44], 23}, {NULL}};
const Segment SGI143798[] = {{&S29858[30], 6}, {NULL}};
const Segment SGI143799[] = {{&S29858[116], 3}, {NULL}};
const Segment SGI143800[] = {{&S29858[81], 6}, {NULL}};
const Segment SGI143801[] = {{&S29859[44], 10}, {NULL}};
const Segment SGI143802[] = {{&S29859[44], 22}, {NULL}};
const Segment SGI143803[] = {{&S29859[30], 6}, {NULL}};
const Segment SGI143804[] = {{&S29859[83], 6}, {NULL}};
const Segment SGI143805[] = {{&S29919[44], 7}, {NULL}};
const Segment SGI143806[] = {{&S29919[44], 19}, {NULL}};
const Segment SGI143807[] = {{&S29919[30], 6}, {NULL}};
const Segment SGI143808[] = {{&S29919[112], 3}, {NULL}};
const Segment SGI143809[] = {{&S29919[77], 6}, {NULL}};
const Segment SGI143810[] = {{&S29920[44], 12}, {NULL}};
const Segment SGI143811[] = {{&S29920[44], 33}, {NULL}};
const Segment SGI143812[] = {{&S29920[30], 6}, {NULL}};
const Segment SGI143813[] = {{&S29920[126], 3}, {NULL}};
const Segment SGI143814[] = {{&S29920[91], 6}, {NULL}};
const Segment SGI143815[] = {{&S29921[44], 12}, {NULL}};
const Segment SGI143816[] = {{&S29921[44], 31}, {NULL}};
const Segment SGI143817[] = {{&S29921[30], 6}, {NULL}};
const Segment SGI143818[] = {{&S29921[124], 3}, {NULL}};
const Segment SGI143819[] = {{&S29921[89], 6}, {NULL}};
const Segment SGI143820[] = {{&S29922[44], 12}, {NULL}};
const Segment SGI143821[] = {{&S29922[44], 34}, {NULL}};
const Segment SGI143822[] = {{&S29922[30], 6}, {NULL}};
const Segment SGI143823[] = {{&S29922[127], 3}, {NULL}};
const Segment SGI143824[] = {{&S29922[92], 6}, {NULL}};
const Segment SGI143825[] = {{&S40305[44], 7}, {NULL}};
const Segment SGI143826[] = {{&S40305[44], 25}, {NULL}};
const Segment SGI143827[] = {{&S40305[30], 6}, {NULL}};
const Segment SGI143828[] = {{&S40305[31], 3}, {NULL}};
const Segment SGI143829[] = {{&S40305[83], 6}, {NULL}};
const Segment SGI143830[] = {{&S29923[38], 12}, {NULL}};
const Segment SGI143831[] = {{&S29923[31], 6}, {NULL}};
const Segment SGI143832[] = {{&S40306[38], 7}, {NULL}};
const Segment SGI143833[] = {{&S40306[31], 6}, {NULL}};
const Segment SGI143834[] = {{&S29924[44], 12}, {NULL}};
const Segment SGI143835[] = {{&S29924[44], 34}, {NULL}};
const Segment SGI143836[] = {{&S29924[30], 6}, {NULL}};
const Segment SGI143837[] = {{&S29924[127], 3}, {NULL}};
const Segment SGI143838[] = {{&S29924[92], 6}, {NULL}};
const Segment SGI143839[] = {{&S40308[44], 7}, {NULL}};
const Segment SGI143840[] = {{&S40308[44], 25}, {NULL}};
const Segment SGI143841[] = {{&S40308[30], 6}, {NULL}};
const Segment SGI143842[] = {{&S29925[44], 12}, {NULL}};
const Segment SGI143843[] = {{&S29925[44], 26}, {NULL}};
const Segment SGI143844[] = {{&S29925[30], 6}, {NULL}};
const Segment SGI143845[] = {{&S40308[31], 3}, {NULL}};
const Segment SGI143846[] = {{&S40308[83], 6}, {NULL}};
const Segment SGI143847[] = {{&S29925[119], 3}, {NULL}};
const Segment SGI143848[] = {{&S29925[84], 6}, {NULL}};
const Segment SGI143849[] = {{&S29926[44], 12}, {NULL}};
const Segment SGI143850[] = {{&S29926[44], 34}, {NULL}};
const Segment SGI143851[] = {{&S29926[30], 6}, {NULL}};
const Segment SGI143852[] = {{&S29926[127], 3}, {NULL}};
const Segment SGI143853[] = {{&S29926[92], 6}, {NULL}};
const Segment SGI143854[] = {{&S29927[42], 12}, {NULL}};
const Segment SGI143855[] = {{&S29927[30], 6}, {NULL}};
const Segment SGI143856[] = {{&S29927[125], 3}, {NULL}};
const Segment SGI143857[] = {{&S29927[90], 6}, {NULL}};
const Segment SGI143858[] = {{&S29928[44], 12}, {NULL}};
const Segment SGI143859[] = {{&S29928[44], 34}, {NULL}};
const Segment SGI143860[] = {{&S29928[30], 6}, {NULL}};
const Segment SGI143861[] = {{&S29928[92], 6}, {NULL}};
const Segment SGI143862[] = {{&S29929[44], 12}, {NULL}};
const Segment SGI143863[] = {{&S29929[44], 45}, {NULL}};
const Segment SGI143864[] = {{&S29929[30], 6}, {NULL}};
const Segment SGI143865[] = {{&S29929[139], 3}, {NULL}};
const Segment SGI143866[] = {{&S29929[104], 6}, {NULL}};
const Segment SGI143867[] = {{&S29930[44], 12}, {NULL}};
const Segment SGI143868[] = {{&S29930[44], 27}, {NULL}};
const Segment SGI143869[] = {{&S29930[30], 6}, {NULL}};
const Segment SGI143870[] = {{&S29930[120], 3}, {NULL}};
const Segment SGI143871[] = {{&S29930[85], 6}, {NULL}};
const Segment SGI143872[] = {{&S29940[34], 10}, {NULL}};
const Segment SGI143873[] = {{&S29940[27], 6}, {NULL}};
const Segment SGI143874[] = {{&S29940[97], 9}, {NULL}};
const Segment SGI143875[] = {{&S29940[70], 6}, {NULL}};
const Segment SGI143876[] = {{&S29944[34], 10}, {NULL}};
const Segment SGI143877[] = {{&S29944[27], 6}, {NULL}};
const Segment SGI143878[] = {{&S29944[97], 9}, {NULL}};
const Segment SGI143879[] = {{&S29944[70], 6}, {NULL}};
const Segment SGI143880[] = {{&S29945[34], 10}, {NULL}};
const Segment SGI143881[] = {{&S29945[27], 6}, {NULL}};
const Segment SGI143882[] = {{&S29945[97], 9}, {NULL}};
const Segment SGI143883[] = {{&S29945[70], 6}, {NULL}};
const Segment SGI143884[] = {{&S29946[34], 10}, {NULL}};
const Segment SGI143885[] = {{&S29946[27], 6}, {NULL}};
const Segment SGI143886[] = {{&S29946[97], 9}, {NULL}};
const Segment SGI143887[] = {{&S29946[70], 6}, {NULL}};
const Segment SGI143888[] = {{&S29947[34], 11}, {NULL}};
const Segment SGI143889[] = {{&S29947[27], 6}, {NULL}};
const Segment SGI143890[] = {{&S29947[98], 9}, {NULL}};
const Segment SGI143891[] = {{&S29947[71], 6}, {NULL}};
const Segment SGI143892[] = {{&S29948[34], 10}, {NULL}};
const Segment SGI143893[] = {{&S29948[27], 6}, {NULL}};
const Segment SGI143894[] = {{&S29948[97], 9}, {NULL}};
const Segment SGI143895[] = {{&S29948[70], 6}, {NULL}};
const Segment SGI143896[] = {{&S29951[34], 11}, {NULL}};
const Segment SGI143897[] = {{&S29951[27], 6}, {NULL}};
const Segment SGI143898[] = {{&S29951[98], 9}, {NULL}};
const Segment SGI143899[] = {{&S29951[71], 6}, {NULL}};
const Segment SGI143900[] = {{&S29955[34], 10}, {NULL}};
const Segment SGI143901[] = {{&S29955[27], 6}, {NULL}};
const Segment SGI143902[] = {{&S29955[97], 9}, {NULL}};
const Segment SGI143903[] = {{&S29955[70], 6}, {NULL}};
const Segment SGI143904[] = {{&S29958[34], 11}, {NULL}};
const Segment SGI143905[] = {{&S29958[27], 6}, {NULL}};
const Segment SGI143906[] = {{&S29958[98], 9}, {NULL}};
const Segment SGI143907[] = {{&S29958[71], 6}, {NULL}};
const Segment SGI143908[] = {{&S29959[34], 8}, {NULL}};
const Segment SGI143909[] = {{&S29959[27], 6}, {NULL}};
const Segment SGI143910[] = {{&S29959[95], 9}, {NULL}};
const Segment SGI143911[] = {{&S29959[68], 6}, {NULL}};
const Segment SGI143912[] = {{&S40369[44], 10}, {NULL}};
const Segment SGI143913[] = {{&S40369[44], 22}, {NULL}};
const Segment SGI143914[] = {{&S40369[30], 6}, {NULL}};
const Segment SGI143915[] = {{&S40369[31], 3}, {NULL}};
const Segment SGI143916[] = {{&S40369[80], 6}, {NULL}};
const Segment SGI143917[] = {{&S40370[44], 13}, {NULL}};
const Segment SGI143918[] = {{&S40370[44], 25}, {NULL}};
const Segment SGI143919[] = {{&S40370[30], 6}, {NULL}};
const Segment SGI143920[] = {{&S40370[31], 3}, {NULL}};
const Segment SGI143921[] = {{&S40370[83], 6}, {NULL}};
const Segment SGI143922[] = {{&S40371[38], 10}, {NULL}};
const Segment SGI143923[] = {{&S40371[31], 6}, {NULL}};
const Segment SGI143924[] = {{&S40372[38], 13}, {NULL}};
const Segment SGI143925[] = {{&S40372[31], 6}, {NULL}};
const Segment SGI143926[] = {{&S31114[34], 11}, {NULL}};
const Segment SGI143927[] = {{&S31114[27], 6}, {NULL}};
const Segment SGI143928[] = {{&S31114[98], 9}, {NULL}};
const Segment SGI143929[] = {{&S31114[71], 6}, {NULL}};
const Segment SGI143930[] = {{&S31115[34], 10}, {NULL}};
const Segment SGI143931[] = {{&S31115[27], 6}, {NULL}};
const Segment SGI143932[] = {{&S31115[97], 9}, {NULL}};
const Segment SGI143933[] = {{&S31115[70], 6}, {NULL}};
const Segment SGI143934[] = {{&S31118[34], 11}, {NULL}};
const Segment SGI143935[] = {{&S31118[27], 6}, {NULL}};
const Segment SGI143936[] = {{&S31118[98], 9}, {NULL}};
const Segment SGI143937[] = {{&S31118[71], 6}, {NULL}};
const Segment SGI143938[] = {{&S31119[34], 10}, {NULL}};
const Segment SGI143939[] = {{&S31119[27], 6}, {NULL}};
const Segment SGI143940[] = {{&S31119[97], 9}, {NULL}};
const Segment SGI143941[] = {{&S31119[70], 6}, {NULL}};
const Segment SGI143942[] = {{&S31123[34], 11}, {NULL}};
const Segment SGI143943[] = {{&S31123[27], 6}, {NULL}};
const Segment SGI143944[] = {{&S31123[98], 9}, {NULL}};
const Segment SGI143945[] = {{&S31123[71], 6}, {NULL}};
const Segment SGI143946[] = {{&S31127[34], 10}, {NULL}};
const Segment SGI143947[] = {{&S31127[27], 6}, {NULL}};
const Segment SGI143948[] = {{&S31127[97], 9}, {NULL}};
const Segment SGI143949[] = {{&S31127[70], 6}, {NULL}};
const Segment SGI143950[] = {{&S31128[34], 11}, {NULL}};
const Segment SGI143951[] = {{&S31128[27], 6}, {NULL}};
const Segment SGI143952[] = {{&S31128[98], 9}, {NULL}};
const Segment SGI143953[] = {{&S31128[71], 6}, {NULL}};
const Segment SGI143954[] = {{&S31129[34], 11}, {NULL}};
const Segment SGI143955[] = {{&S31129[27], 6}, {NULL}};
const Segment SGI143956[] = {{&S31129[98], 9}, {NULL}};
const Segment SGI143957[] = {{&S31129[71], 6}, {NULL}};
const Segment SGI143958[] = {{&S31130[34], 18}, {NULL}};
const Segment SGI143959[] = {{&S31130[27], 6}, {NULL}};
const Segment SGI143960[] = {{&S31130[105], 9}, {NULL}};
const Segment SGI143961[] = {{&S31130[78], 6}, {NULL}};
const Segment SGI143962[] = {{&S31132[34], 18}, {NULL}};
const Segment SGI143963[] = {{&S31132[27], 6}, {NULL}};
const Segment SGI143964[] = {{&S31132[105], 9}, {NULL}};
const Segment SGI143965[] = {{&S31132[78], 6}, {NULL}};
const Segment SGI143966[] = {{&S31136[34], 10}, {NULL}};
const Segment SGI143967[] = {{&S31136[27], 6}, {NULL}};
const Segment SGI143968[] = {{&S31136[97], 9}, {NULL}};
const Segment SGI143969[] = {{&S31136[70], 6}, {NULL}};
const Segment SGI143970[] = {{&S31138[34], 10}, {NULL}};
const Segment SGI143971[] = {{&S31138[27], 6}, {NULL}};
const Segment SGI143972[] = {{&S31138[97], 9}, {NULL}};
const Segment SGI143973[] = {{&S31138[70], 6}, {NULL}};
const Segment SGI143974[] = {{&S31140[34], 11}, {NULL}};
const Segment SGI143975[] = {{&S31140[27], 6}, {NULL}};
const Segment SGI143976[] = {{&S31140[98], 9}, {NULL}};
const Segment SGI143977[] = {{&S31140[71], 6}, {NULL}};
const Segment SGI143978[] = {{&S31145[34], 11}, {NULL}};
const Segment SGI143979[] = {{&S31145[27], 6}, {NULL}};
const Segment SGI143980[] = {{&S31145[98], 9}, {NULL}};
const Segment SGI143981[] = {{&S31145[71], 6}, {NULL}};
const Segment SGI143982[] = {{&S31153[44], 6}, {NULL}};
const Segment SGI143983[] = {{&S31153[44], 18}, {NULL}};
const Segment SGI143984[] = {{&S31153[30], 6}, {NULL}};
const Segment SGI143985[] = {{&S31153[111], 3}, {NULL}};
const Segment SGI143986[] = {{&S31153[76], 6}, {NULL}};
const Segment SGI143987[] = {{&S31154[44], 6}, {NULL}};
const Segment SGI143988[] = {{&S31154[44], 18}, {NULL}};
const Segment SGI143989[] = {{&S31154[30], 6}, {NULL}};
const Segment SGI143990[] = {{&S31154[111], 3}, {NULL}};
const Segment SGI143991[] = {{&S31154[76], 6}, {NULL}};
const Segment SGI143992[] = {{&S31155[38], 6}, {NULL}};
const Segment SGI143993[] = {{&S31155[31], 6}, {NULL}};
const Segment SGI143994[] = {{&S31156[44], 6}, {NULL}};
const Segment SGI143995[] = {{&S31156[44], 18}, {NULL}};
const Segment SGI143996[] = {{&S31156[30], 6}, {NULL}};
const Segment SGI143997[] = {{&S31156[111], 3}, {NULL}};
const Segment SGI143998[] = {{&S31156[76], 6}, {NULL}};
const Segment SGI143999[] = {{&S31157[44], 6}, {NULL}};
const Segment SGI144000[] = {{&S31157[44], 18}, {NULL}};
const Segment SGI144001[] = {{&S31157[30], 6}, {NULL}};
const Segment SGI144002[] = {{&S31157[111], 3}, {NULL}};
const Segment SGI144003[] = {{&S31157[76], 6}, {NULL}};
const Segment SGI144004[] = {{&S31158[44], 6}, {NULL}};
const Segment SGI144005[] = {{&S31158[44], 18}, {NULL}};
const Segment SGI144006[] = {{&S31158[30], 6}, {NULL}};
const Segment SGI144007[] = {{&S31158[111], 3}, {NULL}};
const Segment SGI144008[] = {{&S31158[76], 6}, {NULL}};
const Segment SGI144009[] = {{&S31163[44], 6}, {NULL}};
const Segment SGI144010[] = {{&S31163[44], 18}, {NULL}};
const Segment SGI144011[] = {{&S31163[30], 6}, {NULL}};
const Segment SGI144012[] = {{&S31163[111], 3}, {NULL}};
const Segment SGI144013[] = {{&S31163[76], 6}, {NULL}};
const Segment SGI144014[] = {{&S40496[44], 10}, {NULL}};
const Segment SGI144015[] = {{&S40496[44], 22}, {NULL}};
const Segment SGI144016[] = {{&S40496[30], 6}, {NULL}};
const Segment SGI144017[] = {{&S40496[31], 3}, {NULL}};
const Segment SGI144018[] = {{&S40496[80], 6}, {NULL}};
const Segment SGI144019[] = {{&S40500[44], 101}, {NULL}};
const Segment SGI144020[] = {{&S40500[44], 113}, {NULL}};
const Segment SGI144021[] = {{&S40500[30], 6}, {NULL}};
const Segment SGI144022[] = {{&S40500[31], 3}, {NULL}};
const Segment SGI144023[] = {{&S40500[171], 6}, {NULL}};
const Segment SGI144024[] = {{&S40501[44], 15}, {NULL}};
const Segment SGI144025[] = {{&S40501[44], 27}, {NULL}};
const Segment SGI144026[] = {{&S40501[30], 6}, {NULL}};
const Segment SGI144027[] = {{&S40501[31], 3}, {NULL}};
const Segment SGI144028[] = {{&S40501[85], 6}, {NULL}};
const Segment SGI144029[] = {{&S40502[38], 14}, {NULL}};
const Segment SGI144030[] = {{&S40502[31], 6}, {NULL}};
const Segment SGI144031[] = {{&S40503[44], 19}, {NULL}};
const Segment SGI144032[] = {{&S40503[44], 31}, {NULL}};
const Segment SGI144033[] = {{&S40503[30], 6}, {NULL}};
const Segment SGI144034[] = {{&S40503[31], 3}, {NULL}};
const Segment SGI144035[] = {{&S40503[89], 6}, {NULL}};
const Segment SGI144036[] = {{&S40504[44], 14}, {NULL}};
const Segment SGI144037[] = {{&S40504[44], 26}, {NULL}};
const Segment SGI144038[] = {{&S40504[30], 6}, {NULL}};
const Segment SGI144039[] = {{&S40504[87], 6}, {NULL}};
const Segment SGI144040[] = {{&S40505[44], 14}, {NULL}};
const Segment SGI144041[] = {{&S40505[44], 26}, {NULL}};
const Segment SGI144042[] = {{&S40505[30], 6}, {NULL}};
const Segment SGI144043[] = {{&S40505[87], 4}, {NULL}};
const Segment SGI144044[] = {{&S40505[87], 3}, {NULL}};
const Segment SGI144045[] = {{&S40506[44], 14}, {NULL}};
const Segment SGI144046[] = {{&S40506[44], 26}, {NULL}};
const Segment SGI144047[] = {{&S40506[30], 6}, {NULL}};
const Segment SGI144048[] = {{&S40506[31], 3}, {NULL}};
const Segment SGI144049[] = {{&S40506[84], 6}, {NULL}};
const Segment SGI144050[] = {{&S40507[44], 14}, {NULL}};
const Segment SGI144051[] = {{&S40507[44], 26}, {NULL}};
const Segment SGI144052[] = {{&S40507[30], 6}, {NULL}};
const Segment SGI144053[] = {{&S40507[31], 3}, {NULL}};
const Segment SGI144054[] = {{&S40507[84], 6}, {NULL}};
const Segment SGI144055[] = {{&S40508[44], 14}, {NULL}};
const Segment SGI144056[] = {{&S40508[44], 26}, {NULL}};
const Segment SGI144057[] = {{&S40508[30], 6}, {NULL}};
const Segment SGI144058[] = {{&S40508[31], 3}, {NULL}};
const Segment SGI144059[] = {{&S40508[84], 6}, {NULL}};
const Segment SGI144060[] = {{&S40509[44], 15}, {NULL}};
const Segment SGI144061[] = {{&S40509[44], 27}, {NULL}};
const Segment SGI144062[] = {{&S40509[30], 6}, {NULL}};
const Segment SGI144063[] = {{&S40509[31], 3}, {NULL}};
const Segment SGI144064[] = {{&S40509[85], 6}, {NULL}};
const Segment SGI144065[] = {{&S40510[44], 14}, {NULL}};
const Segment SGI144066[] = {{&S40510[44], 26}, {NULL}};
const Segment SGI144067[] = {{&S40510[30], 6}, {NULL}};
const Segment SGI144068[] = {{&S40510[31], 3}, {NULL}};
const Segment SGI144069[] = {{&S40510[84], 6}, {NULL}};
const Segment SGI144070[] = {{&S40511[38], 15}, {NULL}};
const Segment SGI144071[] = {{&S40511[31], 6}, {NULL}};
const Segment SGI144072[] = {{&S40512[44], 10}, {NULL}};
const Segment SGI144073[] = {{&S40512[44], 22}, {NULL}};
const Segment SGI144074[] = {{&S40512[30], 6}, {NULL}};
const Segment SGI144075[] = {{&S40512[31], 3}, {NULL}};
const Segment SGI144076[] = {{&S40512[80], 6}, {NULL}};
const Segment SGI144077[] = {{&S40514[44], 14}, {NULL}};
const Segment SGI144078[] = {{&S40514[44], 27}, {NULL}};
const Segment SGI144079[] = {{&S40514[30], 6}, {NULL}};
const Segment SGI144080[] = {{&S40514[31], 3}, {NULL}};
const Segment SGI144081[] = {{&S40514[85], 6}, {NULL}};
const Segment SGI144082[] = {{&S40515[42], 14}, {NULL}};
const Segment SGI144083[] = {{&S40515[30], 6}, {NULL}};
const Segment SGI144084[] = {{&S40515[31], 3}, {NULL}};
const Segment SGI144085[] = {{&S40515[82], 6}, {NULL}};
const Segment SGI144086[] = {{&S40516[44], 19}, {NULL}};
const Segment SGI144087[] = {{&S40516[44], 31}, {NULL}};
const Segment SGI144088[] = {{&S40516[30], 6}, {NULL}};
const Segment SGI144089[] = {{&S40516[31], 3}, {NULL}};
const Segment SGI144090[] = {{&S40516[89], 6}, {NULL}};
const Segment SGI144091[] = {{&S40551[44], 13}, {NULL}};
const Segment SGI144092[] = {{&S40551[44], 25}, {NULL}};
const Segment SGI144093[] = {{&S40551[30], 6}, {NULL}};
const Segment SGI144094[] = {{&S40551[31], 3}, {NULL}};
const Segment SGI144095[] = {{&S40551[83], 6}, {NULL}};
const Segment SGI144096[] = {{&S40552[38], 13}, {NULL}};
const Segment SGI144097[] = {{&S40552[31], 6}, {NULL}};
const Segment SGI144098[] = {{&S40553[44], 12}, {NULL}};
const Segment SGI144099[] = {{&S40553[44], 24}, {NULL}};
const Segment SGI144100[] = {{&S40553[30], 6}, {NULL}};
const Segment SGI144101[] = {{&S40553[31], 3}, {NULL}};
const Segment SGI144102[] = {{&S40553[82], 6}, {NULL}};
const Segment SGI144103[] = {{&S40554[44], 20}, {NULL}};
const Segment SGI144104[] = {{&S40554[44], 32}, {NULL}};
const Segment SGI144105[] = {{&S40554[30], 6}, {NULL}};
const Segment SGI144106[] = {{&S40554[31], 3}, {NULL}};
const Segment SGI144107[] = {{&S40554[90], 6}, {NULL}};
const Segment SGI144108[] = {{&S40555[38], 12}, {NULL}};
const Segment SGI144109[] = {{&S40555[31], 6}, {NULL}};
const Segment SGI144110[] = {{&S40556[44], 20}, {NULL}};
const Segment SGI144111[] = {{&S40556[44], 32}, {NULL}};
const Segment SGI144112[] = {{&S40556[30], 6}, {NULL}};
const Segment SGI144113[] = {{&S40556[31], 3}, {NULL}};
const Segment SGI144114[] = {{&S40556[90], 6}, {NULL}};
const Segment SGI144115[] = {{&S40557[44], 20}, {NULL}};
const Segment SGI144116[] = {{&S40557[44], 32}, {NULL}};
const Segment SGI144117[] = {{&S40557[30], 6}, {NULL}};
const Segment SGI144118[] = {{&S40557[31], 3}, {NULL}};
const Segment SGI144119[] = {{&S40557[90], 6}, {NULL}};
const Segment SGI144120[] = {{&S40561[44], 12}, {NULL}};
const Segment SGI144121[] = {{&S40561[44], 24}, {NULL}};
const Segment SGI144122[] = {{&S40561[30], 6}, {NULL}};
const Segment SGI144123[] = {{&S40561[31], 3}, {NULL}};
const Segment SGI144124[] = {{&S40561[82], 6}, {NULL}};
const Segment SGI144125[] = {{&S40562[44], 9}, {NULL}};
const Segment SGI144126[] = {{&S40562[44], 21}, {NULL}};
const Segment SGI144127[] = {{&S40562[30], 6}, {NULL}};
const Segment SGI144128[] = {{&S40562[31], 3}, {NULL}};
const Segment SGI144129[] = {{&S40562[79], 6}, {NULL}};
const Segment SGI144130[] = {{&S40563[38], 9}, {NULL}};
const Segment SGI144131[] = {{&S40563[31], 6}, {NULL}};
const Segment SGI144132[] = {{&S30223[34], 10}, {NULL}};
const Segment SGI144133[] = {{&S30223[27], 6}, {NULL}};
const Segment SGI144134[] = {{&S30223[97], 9}, {NULL}};
const Segment SGI144135[] = {{&S30223[70], 6}, {NULL}};
const Segment SGI144136[] = {{&S30225[34], 10}, {NULL}};
const Segment SGI144137[] = {{&S30225[27], 6}, {NULL}};
const Segment SGI144138[] = {{&S30225[97], 9}, {NULL}};
const Segment SGI144139[] = {{&S30225[70], 6}, {NULL}};
const Segment SGI144140[] = {{&S30227[34], 10}, {NULL}};
const Segment SGI144141[] = {{&S30227[27], 6}, {NULL}};
const Segment SGI144142[] = {{&S30227[97], 9}, {NULL}};
const Segment SGI144143[] = {{&S30227[70], 6}, {NULL}};
const Segment SGI144144[] = {{&S30229[34], 11}, {NULL}};
const Segment SGI144145[] = {{&S30229[27], 6}, {NULL}};
const Segment SGI144146[] = {{&S30229[98], 9}, {NULL}};
const Segment SGI144147[] = {{&S30229[71], 6}, {NULL}};
const Segment SGI144148[] = {{&S30230[34], 11}, {NULL}};
const Segment SGI144149[] = {{&S30230[27], 6}, {NULL}};
const Segment SGI144150[] = {{&S30230[98], 9}, {NULL}};
const Segment SGI144151[] = {{&S30230[71], 6}, {NULL}};
const Segment SGI144152[] = {{&S30232[34], 11}, {NULL}};
const Segment SGI144153[] = {{&S30232[27], 6}, {NULL}};
const Segment SGI144154[] = {{&S30232[98], 9}, {NULL}};
const Segment SGI144155[] = {{&S30232[71], 6}, {NULL}};
const Segment SGI144156[] = {{&S30233[34], 11}, {NULL}};
const Segment SGI144157[] = {{&S30233[27], 6}, {NULL}};
const Segment SGI144158[] = {{&S30233[98], 9}, {NULL}};
const Segment SGI144159[] = {{&S30233[71], 6}, {NULL}};
const Segment SGI144160[] = {{&S30235[34], 10}, {NULL}};
const Segment SGI144161[] = {{&S30235[27], 6}, {NULL}};
const Segment SGI144162[] = {{&S30235[97], 9}, {NULL}};
const Segment SGI144163[] = {{&S30235[70], 6}, {NULL}};
const Segment SGI144164[] = {{&S30236[34], 11}, {NULL}};
const Segment SGI144165[] = {{&S30236[27], 6}, {NULL}};
const Segment SGI144166[] = {{&S30236[98], 9}, {NULL}};
const Segment SGI144167[] = {{&S30236[71], 6}, {NULL}};
const Segment SGI144168[] = {{&S30238[34], 11}, {NULL}};
const Segment SGI144169[] = {{&S30238[27], 6}, {NULL}};
const Segment SGI144170[] = {{&S30238[98], 9}, {NULL}};
const Segment SGI144171[] = {{&S30238[71], 6}, {NULL}};
const Segment SGI144172[] = {{&S30239[34], 18}, {NULL}};
const Segment SGI144173[] = {{&S30239[27], 6}, {NULL}};
const Segment SGI144174[] = {{&S30239[105], 9}, {NULL}};
const Segment SGI144175[] = {{&S30239[78], 6}, {NULL}};
const Segment SGI144176[] = {{&S30243[34], 11}, {NULL}};
const Segment SGI144177[] = {{&S30243[27], 6}, {NULL}};
const Segment SGI144178[] = {{&S30243[98], 9}, {NULL}};
const Segment SGI144179[] = {{&S30243[71], 6}, {NULL}};
const Segment SGI144180[] = {{&S30244[34], 11}, {NULL}};
const Segment SGI144181[] = {{&S30244[27], 6}, {NULL}};
const Segment SGI144182[] = {{&S30244[98], 9}, {NULL}};
const Segment SGI144183[] = {{&S30244[71], 6}, {NULL}};
const Segment SGI144184[] = {{&S30284[44], 25}, {NULL}};
const Segment SGI144185[] = {{&S30284[44], 37}, {NULL}};
const Segment SGI144186[] = {{&S30284[30], 6}, {NULL}};
const Segment SGI144187[] = {{&S30284[130], 3}, {NULL}};
const Segment SGI144188[] = {{&S30284[95], 6}, {NULL}};
const Segment SGI144189[] = {{&S15803[34], 10}, {NULL}};
const Segment SGI144190[] = {{&S15803[27], 6}, {NULL}};
const Segment SGI144191[] = {{&S15803[96], 9}, {NULL}};
const Segment SGI144192[] = {{&S15803[69], 6}, {NULL}};
const Segment SGI144193[] = {{&S15804[41], 18}, {NULL}};
const Segment SGI144194[] = {{&S15804[41], 29}, {NULL}};
const Segment SGI144195[] = {{&S15804[27], 6}, {NULL}};
const Segment SGI144196[] = {{&S15804[119], 3}, {NULL}};
const Segment SGI144197[] = {{&S15804[123], 9}, {NULL}};
const Segment SGI144198[] = {{&S15804[84], 6}, {NULL}};
const Segment SGI144199[] = {{&S15805[41], 31}, {NULL}};
const Segment SGI144200[] = {{&S15805[41], 42}, {NULL}};
const Segment SGI144201[] = {{&S15805[27], 6}, {NULL}};
const Segment SGI144202[] = {{&S15805[132], 3}, {NULL}};
const Segment SGI144203[] = {{&S15805[136], 9}, {NULL}};
const Segment SGI144204[] = {{&S15805[97], 6}, {NULL}};
const Segment SGI144205[] = {{&S15806[41], 25}, {NULL}};
const Segment SGI144206[] = {{&S15806[41], 36}, {NULL}};
const Segment SGI144207[] = {{&S15806[27], 6}, {NULL}};
const Segment SGI144208[] = {{&S15806[126], 3}, {NULL}};
const Segment SGI144209[] = {{&S15806[130], 9}, {NULL}};
const Segment SGI144210[] = {{&S15806[91], 6}, {NULL}};
const Segment SGI144211[] = {{&S15807[41], 31}, {NULL}};
const Segment SGI144212[] = {{&S15807[41], 42}, {NULL}};
const Segment SGI144213[] = {{&S15807[27], 6}, {NULL}};
const Segment SGI144214[] = {{&S15807[132], 3}, {NULL}};
const Segment SGI144215[] = {{&S15807[136], 9}, {NULL}};
const Segment SGI144216[] = {{&S15807[97], 6}, {NULL}};
const Segment SGI144217[] = {{&S15808[41], 19}, {NULL}};
const Segment SGI144218[] = {{&S15808[41], 30}, {NULL}};
const Segment SGI144219[] = {{&S15808[27], 6}, {NULL}};
const Segment SGI144220[] = {{&S15808[120], 3}, {NULL}};
const Segment SGI144221[] = {{&S15808[124], 9}, {NULL}};
const Segment SGI144222[] = {{&S15808[85], 6}, {NULL}};
const Segment SGI144223[] = {{&S15809[41], 31}, {NULL}};
const Segment SGI144224[] = {{&S15809[41], 42}, {NULL}};
const Segment SGI144225[] = {{&S15809[27], 6}, {NULL}};
const Segment SGI144226[] = {{&S15809[132], 3}, {NULL}};
const Segment SGI144227[] = {{&S15809[136], 9}, {NULL}};
const Segment SGI144228[] = {{&S15809[97], 6}, {NULL}};
const Segment SGI144229[] = {{&S15810[34], 11}, {NULL}};
const Segment SGI144230[] = {{&S15810[27], 6}, {NULL}};
const Segment SGI144231[] = {{&S15810[97], 9}, {NULL}};
const Segment SGI144232[] = {{&S15810[70], 6}, {NULL}};
const Segment SGI144233[] = {{&S15811[41], 31}, {NULL}};
const Segment SGI144234[] = {{&S15811[41], 42}, {NULL}};
const Segment SGI144235[] = {{&S15811[27], 6}, {NULL}};
const Segment SGI144236[] = {{&S15811[132], 3}, {NULL}};
const Segment SGI144237[] = {{&S15811[136], 9}, {NULL}};
const Segment SGI144238[] = {{&S15811[97], 6}, {NULL}};
const Segment SGI144239[] = {{&S15812[41], 31}, {NULL}};
const Segment SGI144240[] = {{&S15812[41], 42}, {NULL}};
const Segment SGI144241[] = {{&S15812[27], 6}, {NULL}};
const Segment SGI144242[] = {{&S15812[132], 3}, {NULL}};
const Segment SGI144243[] = {{&S15812[136], 9}, {NULL}};
const Segment SGI144244[] = {{&S15812[97], 6}, {NULL}};
const Segment SGI144245[] = {{&S15813[41], 31}, {NULL}};
const Segment SGI144246[] = {{&S15813[41], 42}, {NULL}};
const Segment SGI144247[] = {{&S15813[27], 6}, {NULL}};
const Segment SGI144248[] = {{&S15813[132], 3}, {NULL}};
const Segment SGI144249[] = {{&S15813[136], 9}, {NULL}};
const Segment SGI144250[] = {{&S15813[97], 6}, {NULL}};
const Segment SGI144251[] = {{&S15814[41], 10}, {NULL}};
const Segment SGI144252[] = {{&S15814[41], 21}, {NULL}};
const Segment SGI144253[] = {{&S15814[27], 6}, {NULL}};
const Segment SGI144254[] = {{&S15814[111], 3}, {NULL}};
const Segment SGI144255[] = {{&S15814[115], 9}, {NULL}};
const Segment SGI144256[] = {{&S15814[76], 6}, {NULL}};
const Segment SGI144257[] = {{&S15815[41], 31}, {NULL}};
const Segment SGI144258[] = {{&S15815[41], 42}, {NULL}};
const Segment SGI144259[] = {{&S15815[27], 6}, {NULL}};
const Segment SGI144260[] = {{&S15815[132], 3}, {NULL}};
const Segment SGI144261[] = {{&S15815[136], 9}, {NULL}};
const Segment SGI144262[] = {{&S15815[97], 6}, {NULL}};
const Segment SGI144263[] = {{&S15816[41], 31}, {NULL}};
const Segment SGI144264[] = {{&S15816[41], 42}, {NULL}};
const Segment SGI144265[] = {{&S15816[27], 6}, {NULL}};
const Segment SGI144266[] = {{&S15816[132], 3}, {NULL}};
const Segment SGI144267[] = {{&S15816[136], 9}, {NULL}};
const Segment SGI144268[] = {{&S15816[97], 6}, {NULL}};
const Segment SGI144269[] = {{&S15817[41], 31}, {NULL}};
const Segment SGI144270[] = {{&S15817[41], 42}, {NULL}};
const Segment SGI144271[] = {{&S15817[27], 6}, {NULL}};
const Segment SGI144272[] = {{&S15817[132], 3}, {NULL}};
const Segment SGI144273[] = {{&S15817[136], 9}, {NULL}};
const Segment SGI144274[] = {{&S15817[97], 6}, {NULL}};
const Segment SGI144275[] = {{&S15818[41], 31}, {NULL}};
const Segment SGI144276[] = {{&S15818[41], 42}, {NULL}};
const Segment SGI144277[] = {{&S15818[27], 6}, {NULL}};
const Segment SGI144278[] = {{&S15818[132], 3}, {NULL}};
const Segment SGI144279[] = {{&S15818[136], 9}, {NULL}};
const Segment SGI144280[] = {{&S15818[97], 6}, {NULL}};
const Segment SGI144281[] = {{&S15819[41], 31}, {NULL}};
const Segment SGI144282[] = {{&S15819[41], 42}, {NULL}};
const Segment SGI144283[] = {{&S15819[27], 6}, {NULL}};
const Segment SGI144284[] = {{&S15819[132], 3}, {NULL}};
const Segment SGI144285[] = {{&S15819[136], 9}, {NULL}};
const Segment SGI144286[] = {{&S15819[97], 6}, {NULL}};
const Segment SGI144287[] = {{&S15820[41], 31}, {NULL}};
const Segment SGI144288[] = {{&S15820[41], 42}, {NULL}};
const Segment SGI144289[] = {{&S15820[27], 6}, {NULL}};
const Segment SGI144290[] = {{&S15820[132], 3}, {NULL}};
const Segment SGI144291[] = {{&S15820[136], 9}, {NULL}};
const Segment SGI144292[] = {{&S15820[97], 6}, {NULL}};
const Segment SGI144293[] = {{&S15821[41], 31}, {NULL}};
const Segment SGI144294[] = {{&S15821[41], 42}, {NULL}};
const Segment SGI144295[] = {{&S15821[27], 6}, {NULL}};
const Segment SGI144296[] = {{&S15821[132], 3}, {NULL}};
const Segment SGI144297[] = {{&S15821[136], 9}, {NULL}};
const Segment SGI144298[] = {{&S15821[97], 6}, {NULL}};
const Segment SGI144299[] = {{&S30427[44], 7}, {NULL}};
const Segment SGI144300[] = {{&S30427[44], 24}, {NULL}};
const Segment SGI144301[] = {{&S30427[30], 6}, {NULL}};
const Segment SGI144302[] = {{&S30427[117], 3}, {NULL}};
const Segment SGI144303[] = {{&S30427[82], 6}, {NULL}};
const Segment SGI144304[] = {{&S30428[44], 11}, {NULL}};
const Segment SGI144305[] = {{&S30428[44], 28}, {NULL}};
const Segment SGI144306[] = {{&S30428[30], 6}, {NULL}};
const Segment SGI144307[] = {{&S30428[121], 3}, {NULL}};
const Segment SGI144308[] = {{&S30428[86], 6}, {NULL}};
const Segment SGI144309[] = {{&S30429[44], 11}, {NULL}};
const Segment SGI144310[] = {{&S30429[44], 28}, {NULL}};
const Segment SGI144311[] = {{&S30429[30], 6}, {NULL}};
const Segment SGI144312[] = {{&S30429[121], 3}, {NULL}};
const Segment SGI144313[] = {{&S30429[86], 6}, {NULL}};
const Segment SGI144314[] = {{&S30430[44], 7}, {NULL}};
const Segment SGI144315[] = {{&S30430[44], 24}, {NULL}};
const Segment SGI144316[] = {{&S30430[30], 6}, {NULL}};
const Segment SGI144317[] = {{&S30430[117], 3}, {NULL}};
const Segment SGI144318[] = {{&S30430[82], 6}, {NULL}};
const Segment SGI144319[] = {{&S30431[44], 11}, {NULL}};
const Segment SGI144320[] = {{&S30431[44], 28}, {NULL}};
const Segment SGI144321[] = {{&S30431[30], 6}, {NULL}};
const Segment SGI144322[] = {{&S30431[121], 3}, {NULL}};
const Segment SGI144323[] = {{&S30431[86], 6}, {NULL}};
const Segment SGI144324[] = {{&S30432[44], 7}, {NULL}};
const Segment SGI144325[] = {{&S30432[44], 24}, {NULL}};
const Segment SGI144326[] = {{&S30432[30], 6}, {NULL}};
const Segment SGI144327[] = {{&S30432[117], 3}, {NULL}};
const Segment SGI144328[] = {{&S30432[82], 6}, {NULL}};
const Segment SGI144329[] = {{&S30433[44], 13}, {NULL}};
const Segment SGI144330[] = {{&S30433[44], 30}, {NULL}};
const Segment SGI144331[] = {{&S30433[30], 6}, {NULL}};
const Segment SGI144332[] = {{&S30433[123], 3}, {NULL}};
const Segment SGI144333[] = {{&S30433[88], 6}, {NULL}};
const Segment SGI144334[] = {{&S30434[38], 7}, {NULL}};
const Segment SGI144335[] = {{&S30434[31], 6}, {NULL}};
const Segment SGI144336[] = {{&S30435[44], 11}, {NULL}};
const Segment SGI144337[] = {{&S30435[44], 28}, {NULL}};
const Segment SGI144338[] = {{&S30435[30], 6}, {NULL}};
const Segment SGI144339[] = {{&S30435[121], 3}, {NULL}};
const Segment SGI144340[] = {{&S30435[86], 6}, {NULL}};
const Segment SGI144341[] = {{&S30436[44], 7}, {NULL}};
const Segment SGI144342[] = {{&S30436[44], 24}, {NULL}};
const Segment SGI144343[] = {{&S30436[30], 6}, {NULL}};
const Segment SGI144344[] = {{&S30436[117], 3}, {NULL}};
const Segment SGI144345[] = {{&S30436[82], 6}, {NULL}};
const Segment SGI144346[] = {{&S30437[44], 11}, {NULL}};
const Segment SGI144347[] = {{&S30437[44], 28}, {NULL}};
const Segment SGI144348[] = {{&S30437[30], 6}, {NULL}};
const Segment SGI144349[] = {{&S30437[121], 3}, {NULL}};
const Segment SGI144350[] = {{&S30437[86], 6}, {NULL}};
const Segment SGI144351[] = {{&S30438[44], 13}, {NULL}};
const Segment SGI144352[] = {{&S30438[44], 30}, {NULL}};
const Segment SGI144353[] = {{&S30438[30], 6}, {NULL}};
const Segment SGI144354[] = {{&S30438[123], 3}, {NULL}};
const Segment SGI144355[] = {{&S30438[88], 6}, {NULL}};
const Segment SGI144356[] = {{&S30440[44], 7}, {NULL}};
const Segment SGI144357[] = {{&S30440[44], 24}, {NULL}};
const Segment SGI144358[] = {{&S30440[30], 6}, {NULL}};
const Segment SGI144359[] = {{&S30440[117], 3}, {NULL}};
const Segment SGI144360[] = {{&S30440[82], 6}, {NULL}};
const Segment SGI144361[] = {{&S30442[42], 11}, {NULL}};
const Segment SGI144362[] = {{&S30442[30], 6}, {NULL}};
const Segment SGI144363[] = {{&S30442[119], 3}, {NULL}};
const Segment SGI144364[] = {{&S30442[84], 6}, {NULL}};
const Segment SGI144365[] = {{&S30445[34], 9}, {NULL}};
const Segment SGI144366[] = {{&S30445[27], 6}, {NULL}};
const Segment SGI144367[] = {{&S30445[110], 9}, {NULL}};
const Segment SGI144368[] = {{&S30445[83], 6}, {NULL}};
const Segment SGI144369[] = {{&S30446[34], 7}, {NULL}};
const Segment SGI144370[] = {{&S30446[27], 6}, {NULL}};
const Segment SGI144371[] = {{&S30446[111], 9}, {NULL}};
const Segment SGI144372[] = {{&S30446[84], 6}, {NULL}};
const Segment SGI144373[] = {{&S30447[34], 15}, {NULL}};
const Segment SGI144374[] = {{&S30447[27], 6}, {NULL}};
const Segment SGI144375[] = {{&S30447[114], 9}, {NULL}};
const Segment SGI144376[] = {{&S30447[87], 6}, {NULL}};
const Segment SGI144377[] = {{&S40768[44], 11}, {NULL}};
const Segment SGI144378[] = {{&S40768[44], 23}, {NULL}};
const Segment SGI144379[] = {{&S40768[30], 6}, {NULL}};
const Segment SGI144380[] = {{&S40768[31], 3}, {NULL}};
const Segment SGI144381[] = {{&S40768[81], 6}, {NULL}};
const Segment SGI144382[] = {{&S40769[44], 11}, {NULL}};
const Segment SGI144383[] = {{&S40769[44], 23}, {NULL}};
const Segment SGI144384[] = {{&S40769[30], 6}, {NULL}};
const Segment SGI144385[] = {{&S40769[31], 3}, {NULL}};
const Segment SGI144386[] = {{&S40769[81], 6}, {NULL}};
const Segment SGI144387[] = {{&S40770[38], 11}, {NULL}};
const Segment SGI144388[] = {{&S40770[31], 6}, {NULL}};
const Segment SGI144389[] = {{&S40771[44], 11}, {NULL}};
const Segment SGI144390[] = {{&S40771[44], 23}, {NULL}};
const Segment SGI144391[] = {{&S40771[30], 6}, {NULL}};
const Segment SGI144392[] = {{&S40771[31], 3}, {NULL}};
const Segment SGI144393[] = {{&S40771[81], 6}, {NULL}};
const Segment SGI144394[] = {{&S40774[38], 11}, {NULL}};
const Segment SGI144395[] = {{&S40774[31], 6}, {NULL}};
const Segment SGI144396[] = {{&S35888[34], 10}, {NULL}};
const Segment SGI144397[] = {{&S35888[27], 6}, {NULL}};
const Segment SGI144398[] = {{&S35888[70], 6}, {NULL}};
const Segment SGI144399[] = {{&S35889[34], 8}, {NULL}};
const Segment SGI144400[] = {{&S40776[38], 11}, {NULL}};
const Segment SGI144401[] = {{&S35889[27], 6}, {NULL}};
const Segment SGI144402[] = {{&S40776[31], 6}, {NULL}};
const Segment SGI144403[] = {{&S35889[68], 6}, {NULL}};
const Segment SGI144404[] = {{&S35890[34], 7}, {NULL}};
const Segment SGI144405[] = {{&S35890[27], 6}, {NULL}};
const Segment SGI144406[] = {{&S35890[67], 6}, {NULL}};
const Segment SGI144407[] = {{&S40778[44], 11}, {NULL}};
const Segment SGI144408[] = {{&S40778[44], 23}, {NULL}};
const Segment SGI144409[] = {{&S40778[30], 6}, {NULL}};
const Segment SGI144410[] = {{&S40778[84], 6}, {NULL}};
const Segment SGI144411[] = {{&S40779[44], 11}, {NULL}};
const Segment SGI144412[] = {{&S40779[44], 23}, {NULL}};
const Segment SGI144413[] = {{&S40779[30], 6}, {NULL}};
const Segment SGI144414[] = {{&S40779[31], 3}, {NULL}};
const Segment SGI144415[] = {{&S40779[81], 6}, {NULL}};
const Segment SGI144416[] = {{&S31708[34], 7}, {NULL}};
const Segment SGI144417[] = {{&S31708[27], 6}, {NULL}};
const Segment SGI144418[] = {{&S31708[99], 9}, {NULL}};
const Segment SGI144419[] = {{&S31708[72], 6}, {NULL}};
const Segment SGI144420[] = {{&S40827[44], 9}, {NULL}};
const Segment SGI144421[] = {{&S40827[44], 21}, {NULL}};
const Segment SGI144422[] = {{&S40827[30], 6}, {NULL}};
const Segment SGI144423[] = {{&S40827[79], 6}, {NULL}};
const Segment SGI144424[] = {{&S40830[44], 9}, {NULL}};
const Segment SGI144425[] = {{&S40830[44], 21}, {NULL}};
const Segment SGI144426[] = {{&S40830[30], 6}, {NULL}};
const Segment SGI144427[] = {{&S40830[31], 3}, {NULL}};
const Segment SGI144428[] = {{&S40830[79], 6}, {NULL}};
const Segment SGI144429[] = {{&S40831[44], 14}, {NULL}};
const Segment SGI144430[] = {{&S40831[44], 26}, {NULL}};
const Segment SGI144431[] = {{&S40831[30], 6}, {NULL}};
const Segment SGI144432[] = {{&S40831[31], 3}, {NULL}};
const Segment SGI144433[] = {{&S40831[84], 6}, {NULL}};
const Segment SGI144434[] = {{&S40832[44], 14}, {NULL}};
const Segment SGI144435[] = {{&S40832[44], 26}, {NULL}};
const Segment SGI144436[] = {{&S40832[30], 6}, {NULL}};
const Segment SGI144437[] = {{&S40832[31], 3}, {NULL}};
const Segment SGI144438[] = {{&S40832[84], 6}, {NULL}};
const Segment SGI144439[] = {{&S40833[38], 9}, {NULL}};
const Segment SGI144440[] = {{&S40833[31], 6}, {NULL}};
const Segment SGI144441[] = {{&S40834[44], 14}, {NULL}};
const Segment SGI144442[] = {{&S40834[44], 26}, {NULL}};
const Segment SGI144443[] = {{&S40834[30], 6}, {NULL}};
const Segment SGI144444[] = {{&S40834[31], 3}, {NULL}};
const Segment SGI144445[] = {{&S40834[84], 6}, {NULL}};
const Segment SGI144446[] = {{&S31788[34], 15}, {NULL}};
const Segment SGI144447[] = {{&S31788[27], 6}, {NULL}};
const Segment SGI144448[] = {{&S31788[105], 9}, {NULL}};
const Segment SGI144449[] = {{&S31788[78], 6}, {NULL}};
const Segment SGI144450[] = {{&S31791[34], 7}, {NULL}};
const Segment SGI144451[] = {{&S31791[27], 6}, {NULL}};
const Segment SGI144452[] = {{&S31791[110], 9}, {NULL}};
const Segment SGI144453[] = {{&S31791[83], 6}, {NULL}};
const Segment SGI144454[] = {{&S31811[44], 11}, {NULL}};
const Segment SGI144455[] = {{&S31811[44], 23}, {NULL}};
const Segment SGI144456[] = {{&S31811[30], 6}, {NULL}};
const Segment SGI144457[] = {{&S31811[116], 3}, {NULL}};
const Segment SGI144458[] = {{&S31811[81], 6}, {NULL}};
const Segment SGI144459[] = {{&S40976[44], 7}, {NULL}};
const Segment SGI144460[] = {{&S40976[44], 24}, {NULL}};
const Segment SGI144461[] = {{&S40976[30], 6}, {NULL}};
const Segment SGI144462[] = {{&S40976[31], 3}, {NULL}};
const Segment SGI144463[] = {{&S40976[82], 6}, {NULL}};
const Segment SGI144464[] = {{&S40977[44], 5}, {NULL}};
const Segment SGI144465[] = {{&S40977[44], 17}, {NULL}};
const Segment SGI144466[] = {{&S40977[30], 6}, {NULL}};
const Segment SGI144467[] = {{&S40977[31], 3}, {NULL}};
const Segment SGI144468[] = {{&S40977[75], 6}, {NULL}};
const Segment SGI144469[] = {{&S40978[38], 7}, {NULL}};
const Segment SGI144470[] = {{&S40978[31], 6}, {NULL}};
const Segment SGI144471[] = {{&S40979[42], 5}, {NULL}};
const Segment SGI144472[] = {{&S40979[30], 6}, {NULL}};
const Segment SGI144473[] = {{&S40979[31], 3}, {NULL}};
const Segment SGI144474[] = {{&S40979[73], 6}, {NULL}};
const Segment SGI144475[] = {{&S40980[38], 5}, {NULL}};
const Segment SGI144476[] = {{&S40980[31], 6}, {NULL}};
const Segment SGI144477[] = {{&S40981[44], 7}, {NULL}};
const Segment SGI144478[] = {{&S40981[44], 24}, {NULL}};
const Segment SGI144479[] = {{&S40981[30], 6}, {NULL}};
const Segment SGI144480[] = {{&S40981[31], 3}, {NULL}};
const Segment SGI144481[] = {{&S40981[82], 6}, {NULL}};
const Segment SGI144482[] = {{&S40982[44], 10}, {NULL}};
const Segment SGI144483[] = {{&S40982[44], 22}, {NULL}};
const Segment SGI144484[] = {{&S40982[30], 6}, {NULL}};
const Segment SGI144485[] = {{&S40982[31], 3}, {NULL}};
const Segment SGI144486[] = {{&S40982[80], 6}, {NULL}};
const Segment SGI144487[] = {{&S40983[38], 10}, {NULL}};
const Segment SGI144488[] = {{&S40983[31], 6}, {NULL}};
const Segment SGI144489[] = {{&S40984[42], 10}, {NULL}};
const Segment SGI144490[] = {{&S40984[30], 6}, {NULL}};
const Segment SGI144491[] = {{&S40984[31], 3}, {NULL}};
const Segment SGI144492[] = {{&S40984[78], 6}, {NULL}};
const Segment SGI144493[] = {{&S40987[44], 14}, {NULL}};
const Segment SGI144494[] = {{&S40987[44], 26}, {NULL}};
const Segment SGI144495[] = {{&S40987[30], 6}, {NULL}};
const Segment SGI144496[] = {{&S40987[31], 3}, {NULL}};
const Segment SGI144497[] = {{&S40987[84], 6}, {NULL}};
const Segment SGI144498[] = {{&S40988[38], 14}, {NULL}};
const Segment SGI144499[] = {{&S40988[31], 6}, {NULL}};
const Segment SGI144500[] = {{&S41036[44], 17}, {NULL}};
const Segment SGI144501[] = {{&S41036[44], 29}, {NULL}};
const Segment SGI144502[] = {{&S41036[30], 6}, {NULL}};
const Segment SGI144503[] = {{&S41036[31], 3}, {NULL}};
const Segment SGI144504[] = {{&S41036[87], 6}, {NULL}};
const Segment SGI144505[] = {{&S26069[38], 9}, {NULL}};
const Segment SGI144506[] = {{&S26069[31], 6}, {NULL}};
const Segment SGI144507[] = {{&S41037[44], 17}, {NULL}};
const Segment SGI144508[] = {{&S41037[44], 29}, {NULL}};
const Segment SGI144509[] = {{&S41037[30], 6}, {NULL}};
const Segment SGI144510[] = {{&S41037[31], 3}, {NULL}};
const Segment SGI144511[] = {{&S41037[87], 6}, {NULL}};
const Segment SGI144512[] = {{&S26070[44], 9}, {NULL}};
const Segment SGI144513[] = {{&S26070[44], 21}, {NULL}};
const Segment SGI144514[] = {{&S26070[30], 6}, {NULL}};
const Segment SGI144515[] = {{&S26070[114], 3}, {NULL}};
const Segment SGI144516[] = {{&S26070[79], 6}, {NULL}};
const Segment SGI144517[] = {{&S41038[44], 9}, {NULL}};
const Segment SGI144518[] = {{&S41038[44], 21}, {NULL}};
const Segment SGI144519[] = {{&S41038[30], 6}, {NULL}};
const Segment SGI144520[] = {{&S41038[31], 3}, {NULL}};
const Segment SGI144521[] = {{&S41038[79], 6}, {NULL}};
const Segment SGI144522[] = {{&S41039[44], 17}, {NULL}};
const Segment SGI144523[] = {{&S41039[44], 29}, {NULL}};
const Segment SGI144524[] = {{&S26072[42], 9}, {NULL}};
const Segment SGI144525[] = {{&S41039[30], 6}, {NULL}};
const Segment SGI144526[] = {{&S26072[30], 6}, {NULL}};
const Segment SGI144527[] = {{&S41039[31], 3}, {NULL}};
const Segment SGI144528[] = {{&S26072[112], 3}, {NULL}};
const Segment SGI144529[] = {{&S41039[87], 6}, {NULL}};
const Segment SGI144530[] = {{&S26072[77], 6}, {NULL}};
const Segment SGI144531[] = {{&S26073[44], 9}, {NULL}};
const Segment SGI144532[] = {{&S41040[38], 9}, {NULL}};
const Segment SGI144533[] = {{&S26073[44], 21}, {NULL}};
const Segment SGI144534[] = {{&S26073[30], 6}, {NULL}};
const Segment SGI144535[] = {{&S41040[31], 6}, {NULL}};
const Segment SGI144536[] = {{&S26073[114], 3}, {NULL}};
const Segment SGI144537[] = {{&S26073[79], 6}, {NULL}};
const Segment SGI144538[] = {{&S26074[44], 9}, {NULL}};
const Segment SGI144539[] = {{&S26074[44], 21}, {NULL}};
const Segment SGI144540[] = {{&S26074[30], 6}, {NULL}};
const Segment SGI144541[] = {{&S26074[82], 6}, {NULL}};
const Segment SGI144542[] = {{&S26075[44], 9}, {NULL}};
const Segment SGI144543[] = {{&S26075[44], 21}, {NULL}};
const Segment SGI144544[] = {{&S26075[30], 6}, {NULL}};
const Segment SGI144545[] = {{&S26075[114], 3}, {NULL}};
const Segment SGI144546[] = {{&S26075[79], 6}, {NULL}};
const Segment SGI144547[] = {{&S26082[44], 9}, {NULL}};
const Segment SGI144548[] = {{&S26082[44], 21}, {NULL}};
const Segment SGI144549[] = {{&S26082[30], 6}, {NULL}};
const Segment SGI144550[] = {{&S26082[114], 3}, {NULL}};
const Segment SGI144551[] = {{&S26082[79], 6}, {NULL}};
const Segment SGI144552[] = {{&S41115[44], 11}, {NULL}};
const Segment SGI144553[] = {{&S41115[44], 23}, {NULL}};
const Segment SGI144554[] = {{&S41115[30], 6}, {NULL}};
const Segment SGI144555[] = {{&S41115[31], 3}, {NULL}};
const Segment SGI144556[] = {{&S41115[81], 6}, {NULL}};
const Segment SGI144557[] = {{&S41116[44], 11}, {NULL}};
const Segment SGI144558[] = {{&S41116[44], 23}, {NULL}};
const Segment SGI144559[] = {{&S41116[30], 6}, {NULL}};
const Segment SGI144560[] = {{&S41116[31], 3}, {NULL}};
const Segment SGI144561[] = {{&S41116[81], 6}, {NULL}};
const Segment SGI144562[] = {{&S41117[38], 11}, {NULL}};
const Segment SGI144563[] = {{&S41117[31], 6}, {NULL}};
const Segment SGI144564[] = {{&S41118[44], 10}, {NULL}};
const Segment SGI144565[] = {{&S41118[44], 22}, {NULL}};
const Segment SGI144566[] = {{&S41118[30], 6}, {NULL}};
const Segment SGI144567[] = {{&S41118[31], 3}, {NULL}};
const Segment SGI144568[] = {{&S41118[80], 6}, {NULL}};
const Segment SGI144569[] = {{&S41119[38], 11}, {NULL}};
const Segment SGI144570[] = {{&S41119[31], 6}, {NULL}};
const Segment SGI144571[] = {{&S41120[42], 11}, {NULL}};
const Segment SGI144572[] = {{&S41120[30], 6}, {NULL}};
const Segment SGI144573[] = {{&S41120[31], 3}, {NULL}};
const Segment SGI144574[] = {{&S41120[79], 6}, {NULL}};
const Segment SGI144575[] = {{&S41121[44], 10}, {NULL}};
const Segment SGI144576[] = {{&S41121[44], 22}, {NULL}};
const Segment SGI144577[] = {{&S41121[30], 6}, {NULL}};
const Segment SGI144578[] = {{&S41121[31], 3}, {NULL}};
const Segment SGI144579[] = {{&S41121[80], 6}, {NULL}};
const Segment SGI144580[] = {{&S41122[44], 18}, {NULL}};
const Segment SGI144581[] = {{&S41122[44], 30}, {NULL}};
const Segment SGI144582[] = {{&S41122[30], 6}, {NULL}};
const Segment SGI144583[] = {{&S41122[31], 3}, {NULL}};
const Segment SGI144584[] = {{&S41122[88], 6}, {NULL}};
const Segment SGI144585[] = {{&S41124[38], 10}, {NULL}};
const Segment SGI144586[] = {{&S41124[31], 6}, {NULL}};
const Segment SGI144587[] = {{&S41142[44], 10}, {NULL}};
const Segment SGI144588[] = {{&S41142[44], 22}, {NULL}};
const Segment SGI144589[] = {{&S41142[30], 6}, {NULL}};
const Segment SGI144590[] = {{&S41142[31], 3}, {NULL}};
const Segment SGI144591[] = {{&S41142[80], 6}, {NULL}};
const Segment SGI144592[] = {{&S41143[38], 10}, {NULL}};
const Segment SGI144593[] = {{&S41143[31], 6}, {NULL}};
const Segment SGI144594[] = {{&S41203[34], 10}, {NULL}};
const Segment SGI144595[] = {{&S41203[27], 6}, {NULL}};
const Segment SGI144596[] = {{&S41203[97], 9}, {NULL}};
const Segment SGI144597[] = {{&S41203[70], 6}, {NULL}};
const Segment SGI144598[] = {{&S41205[34], 13}, {NULL}};
const Segment SGI144599[] = {{&S41205[27], 6}, {NULL}};
const Segment SGI144600[] = {{&S41205[100], 9}, {NULL}};
const Segment SGI144601[] = {{&S41205[73], 6}, {NULL}};
const Segment SGI144602[] = {{&S41206[34], 14}, {NULL}};
const Segment SGI144603[] = {{&S41206[27], 6}, {NULL}};
const Segment SGI144604[] = {{&S41206[101], 9}, {NULL}};
const Segment SGI144605[] = {{&S41206[74], 6}, {NULL}};
const Segment SGI144606[] = {{&S32078[34], 9}, {NULL}};
const Segment SGI144607[] = {{&S32078[27], 6}, {NULL}};
const Segment SGI144608[] = {{&S32078[96], 9}, {NULL}};
const Segment SGI144609[] = {{&S32078[69], 6}, {NULL}};
const Segment SGI144610[] = {{&S32079[34], 9}, {NULL}};
const Segment SGI144611[] = {{&S32079[27], 6}, {NULL}};
const Segment SGI144612[] = {{&S32079[96], 9}, {NULL}};
const Segment SGI144613[] = {{&S32079[69], 6}, {NULL}};
const Segment SGI144614[] = {{&S32081[34], 16}, {NULL}};
const Segment SGI144615[] = {{&S32081[27], 6}, {NULL}};
const Segment SGI144616[] = {{&S32081[103], 9}, {NULL}};
const Segment SGI144617[] = {{&S32081[76], 6}, {NULL}};
const Segment SGI144618[] = {{&S32083[27], 6}, {NULL}};
const Segment SGI144619[] = {{&S32083[111], 9}, {NULL}};
const Segment SGI144620[] = {{&S32083[84], 6}, {NULL}};
const Segment SGI144621[] = {{&S32084[30], 6}, {NULL}};
const Segment SGI144622[] = {{&S32084[85], 9}, {NULL}};
const Segment SGI144623[] = {{&S32084[59], 5}, {NULL}};
const Segment SGI144624[] = {{&S32085[34], 13}, {NULL}};
const Segment SGI144625[] = {{&S32085[27], 6}, {NULL}};
const Segment SGI144626[] = {{&S32085[100], 9}, {NULL}};
const Segment SGI144627[] = {{&S32085[73], 6}, {NULL}};
const Segment SGI144628[] = {{&S31112[34], 10}, {NULL}};
const Segment SGI144629[] = {{&S31112[27], 6}, {NULL}};
const Segment SGI144630[] = {{&S31112[97], 9}, {NULL}};
const Segment SGI144631[] = {{&S31112[70], 6}, {NULL}};
const Segment SGI144632[] = {{&S31113[34], 11}, {NULL}};
const Segment SGI144633[] = {{&S31113[27], 6}, {NULL}};
const Segment SGI144634[] = {{&S31113[98], 9}, {NULL}};
const Segment SGI144635[] = {{&S31113[71], 6}, {NULL}};
const Segment SGI144636[] = {{&S31116[34], 11}, {NULL}};
const Segment SGI144637[] = {{&S31116[27], 6}, {NULL}};
const Segment SGI144638[] = {{&S31116[98], 9}, {NULL}};
const Segment SGI144639[] = {{&S31116[71], 6}, {NULL}};
const Segment SGI144640[] = {{&S31117[34], 11}, {NULL}};
const Segment SGI144641[] = {{&S31117[27], 6}, {NULL}};
const Segment SGI144642[] = {{&S31117[98], 9}, {NULL}};
const Segment SGI144643[] = {{&S31117[71], 6}, {NULL}};
const Segment SGI144644[] = {{&S31120[34], 11}, {NULL}};
const Segment SGI144645[] = {{&S31120[27], 6}, {NULL}};
const Segment SGI144646[] = {{&S31120[98], 9}, {NULL}};
const Segment SGI144647[] = {{&S31120[71], 6}, {NULL}};
const Segment SGI144648[] = {{&S31121[34], 10}, {NULL}};
const Segment SGI144649[] = {{&S31121[27], 6}, {NULL}};
const Segment SGI144650[] = {{&S31121[97], 9}, {NULL}};
const Segment SGI144651[] = {{&S31121[70], 6}, {NULL}};
const Segment SGI144652[] = {{&S41219[34], 10}, {NULL}};
const Segment SGI144653[] = {{&S41219[27], 6}, {NULL}};
const Segment SGI144654[] = {{&S31122[34], 11}, {NULL}};
const Segment SGI144655[] = {{&S31122[27], 6}, {NULL}};
const Segment SGI144656[] = {{&S41219[97], 9}, {NULL}};
const Segment SGI144657[] = {{&S41219[70], 6}, {NULL}};
const Segment SGI144658[] = {{&S31122[98], 9}, {NULL}};
const Segment SGI144659[] = {{&S31122[71], 6}, {NULL}};
const Segment SGI144660[] = {{&S41221[34], 10}, {NULL}};
const Segment SGI144661[] = {{&S41221[27], 6}, {NULL}};
const Segment SGI144662[] = {{&S31124[34], 10}, {NULL}};
const Segment SGI144663[] = {{&S31124[27], 6}, {NULL}};
const Segment SGI144664[] = {{&S41221[97], 9}, {NULL}};
const Segment SGI144665[] = {{&S41221[70], 6}, {NULL}};
const Segment SGI144666[] = {{&S31124[97], 9}, {NULL}};
const Segment SGI144667[] = {{&S31124[70], 6}, {NULL}};
const Segment SGI144668[] = {{&S41222[34], 11}, {NULL}};
const Segment SGI144669[] = {{&S31125[34], 10}, {NULL}};
const Segment SGI144670[] = {{&S41222[27], 6}, {NULL}};
const Segment SGI144671[] = {{&S31125[27], 6}, {NULL}};
const Segment SGI144672[] = {{&S41222[98], 9}, {NULL}};
const Segment SGI144673[] = {{&S41222[71], 6}, {NULL}};
const Segment SGI144674[] = {{&S31125[97], 9}, {NULL}};
const Segment SGI144675[] = {{&S31125[70], 6}, {NULL}};
const Segment SGI144676[] = {{&S41223[42], 12}, {NULL}};
const Segment SGI144677[] = {{&S31126[34], 11}, {NULL}};
const Segment SGI144678[] = {{&S41223[30], 6}, {NULL}};
const Segment SGI144679[] = {{&S31126[27], 6}, {NULL}};
const Segment SGI144680[] = {{&S41223[31], 3}, {NULL}};
const Segment SGI144681[] = {{&S41223[80], 6}, {NULL}};
const Segment SGI144682[] = {{&S31126[98], 9}, {NULL}};
const Segment SGI144683[] = {{&S31126[71], 6}, {NULL}};
const Segment SGI144684[] = {{&S41224[44], 12}, {NULL}};
const Segment SGI144685[] = {{&S41224[44], 24}, {NULL}};
const Segment SGI144686[] = {{&S41224[30], 6}, {NULL}};
const Segment SGI144687[] = {{&S41224[31], 3}, {NULL}};
const Segment SGI144688[] = {{&S41224[82], 6}, {NULL}};
const Segment SGI144689[] = {{&S41225[44], 15}, {NULL}};
const Segment SGI144690[] = {{&S41225[44], 27}, {NULL}};
const Segment SGI144691[] = {{&S41225[30], 6}, {NULL}};
const Segment SGI144692[] = {{&S41225[31], 3}, {NULL}};
const Segment SGI144693[] = {{&S41225[85], 6}, {NULL}};
const Segment SGI144694[] = {{&S31131[34], 10}, {NULL}};
const Segment SGI144695[] = {{&S31131[27], 6}, {NULL}};
const Segment SGI144696[] = {{&S41226[44], 15}, {NULL}};
const Segment SGI144697[] = {{&S41226[44], 27}, {NULL}};
const Segment SGI144698[] = {{&S31131[97], 9}, {NULL}};
const Segment SGI144699[] = {{&S41226[30], 6}, {NULL}};
const Segment SGI144700[] = {{&S31131[70], 6}, {NULL}};
const Segment SGI144701[] = {{&S41226[31], 3}, {NULL}};
const Segment SGI144702[] = {{&S41226[85], 6}, {NULL}};
const Segment SGI144703[] = {{&S41227[44], 12}, {NULL}};
const Segment SGI144704[] = {{&S41227[44], 24}, {NULL}};
const Segment SGI144705[] = {{&S41227[30], 6}, {NULL}};
const Segment SGI144706[] = {{&S31133[34], 11}, {NULL}};
const Segment SGI144707[] = {{&S31133[27], 6}, {NULL}};
const Segment SGI144708[] = {{&S41227[31], 3}, {NULL}};
const Segment SGI144709[] = {{&S41227[82], 6}, {NULL}};
const Segment SGI144710[] = {{&S31133[98], 9}, {NULL}};
const Segment SGI144711[] = {{&S31133[71], 6}, {NULL}};
const Segment SGI144712[] = {{&S41228[38], 12}, {NULL}};
const Segment SGI144713[] = {{&S41228[31], 6}, {NULL}};
const Segment SGI144714[] = {{&S31134[34], 18}, {NULL}};
const Segment SGI144715[] = {{&S31134[27], 6}, {NULL}};
const Segment SGI144716[] = {{&S31134[105], 9}, {NULL}};
const Segment SGI144717[] = {{&S31134[78], 6}, {NULL}};
const Segment SGI144718[] = {{&S31135[34], 11}, {NULL}};
const Segment SGI144719[] = {{&S31135[27], 6}, {NULL}};
const Segment SGI144720[] = {{&S31135[98], 9}, {NULL}};
const Segment SGI144721[] = {{&S31135[71], 6}, {NULL}};
const Segment SGI144722[] = {{&S26436[44], 9}, {NULL}};
const Segment SGI144723[] = {{&S31137[34], 11}, {NULL}};
const Segment SGI144724[] = {{&S26436[44], 21}, {NULL}};
const Segment SGI144725[] = {{&S26436[30], 6}, {NULL}};
const Segment SGI144726[] = {{&S31137[27], 6}, {NULL}};
const Segment SGI144727[] = {{&S26436[114], 3}, {NULL}};
const Segment SGI144728[] = {{&S31137[98], 9}, {NULL}};
const Segment SGI144729[] = {{&S26436[79], 6}, {NULL}};
const Segment SGI144730[] = {{&S31137[71], 6}, {NULL}};
const Segment SGI144731[] = {{&S31139[34], 11}, {NULL}};
const Segment SGI144732[] = {{&S31139[27], 6}, {NULL}};
const Segment SGI144733[] = {{&S31139[98], 9}, {NULL}};
const Segment SGI144734[] = {{&S31139[71], 6}, {NULL}};
const Segment SGI144735[] = {{&S31141[34], 18}, {NULL}};
const Segment SGI144736[] = {{&S31141[27], 6}, {NULL}};
const Segment SGI144737[] = {{&S31141[105], 9}, {NULL}};
const Segment SGI144738[] = {{&S31141[78], 6}, {NULL}};
const Segment SGI144739[] = {{&S31142[34], 11}, {NULL}};
const Segment SGI144740[] = {{&S31142[27], 6}, {NULL}};
const Segment SGI144741[] = {{&S31142[98], 9}, {NULL}};
const Segment SGI144742[] = {{&S31142[71], 6}, {NULL}};
const Segment SGI144743[] = {{&S31143[34], 11}, {NULL}};
const Segment SGI144744[] = {{&S31143[27], 6}, {NULL}};
const Segment SGI144745[] = {{&S31143[98], 9}, {NULL}};
const Segment SGI144746[] = {{&S31143[71], 6}, {NULL}};
const Segment SGI144747[] = {{&S31144[34], 11}, {NULL}};
const Segment SGI144748[] = {{&S31144[27], 6}, {NULL}};
const Segment SGI144749[] = {{&S31144[98], 9}, {NULL}};
const Segment SGI144750[] = {{&S31144[71], 6}, {NULL}};
const Segment SGI144751[] = {{&S26456[44], 9}, {NULL}};
const Segment SGI144752[] = {{&S26456[44], 21}, {NULL}};
const Segment SGI144753[] = {{&S26456[30], 6}, {NULL}};
const Segment SGI144754[] = {{&S26456[114], 3}, {NULL}};
const Segment SGI144755[] = {{&S26456[79], 6}, {NULL}};
const Segment SGI144756[] = {{&S31159[44], 6}, {NULL}};
const Segment SGI144757[] = {{&S31159[44], 18}, {NULL}};
const Segment SGI144758[] = {{&S31159[30], 6}, {NULL}};
const Segment SGI144759[] = {{&S31159[111], 3}, {NULL}};
const Segment SGI144760[] = {{&S31159[76], 6}, {NULL}};
const Segment SGI144761[] = {{&S31160[38], 6}, {NULL}};
const Segment SGI144762[] = {{&S31160[31], 6}, {NULL}};
const Segment SGI144763[] = {{&S31162[44], 6}, {NULL}};
const Segment SGI144764[] = {{&S31162[44], 18}, {NULL}};
const Segment SGI144765[] = {{&S31162[30], 6}, {NULL}};
const Segment SGI144766[] = {{&S31162[111], 3}, {NULL}};
const Segment SGI144767[] = {{&S31162[76], 6}, {NULL}};
const Segment SGI144768[] = {{&S31164[44], 6}, {NULL}};
const Segment SGI144769[] = {{&S31164[44], 18}, {NULL}};
const Segment SGI144770[] = {{&S31164[30], 6}, {NULL}};
const Segment SGI144771[] = {{&S31164[111], 3}, {NULL}};
const Segment SGI144772[] = {{&S31164[76], 6}, {NULL}};
const Segment SGI144773[] = {{&S31166[42], 6}, {NULL}};
const Segment SGI144774[] = {{&S31166[30], 6}, {NULL}};
const Segment SGI144775[] = {{&S31166[109], 3}, {NULL}};
const Segment SGI144776[] = {{&S31166[74], 6}, {NULL}};
const Segment SGI144777[] = {{&S31167[44], 6}, {NULL}};
const Segment SGI144778[] = {{&S31167[44], 18}, {NULL}};
const Segment SGI144779[] = {{&S31167[30], 6}, {NULL}};
const Segment SGI144780[] = {{&S31167[111], 3}, {NULL}};
const Segment SGI144781[] = {{&S31167[76], 6}, {NULL}};
const Segment SGI144782[] = {{&S31168[44], 6}, {NULL}};
const Segment SGI144783[] = {{&S31168[44], 18}, {NULL}};
const Segment SGI144784[] = {{&S31168[30], 6}, {NULL}};
const Segment SGI144785[] = {{&S31168[111], 3}, {NULL}};
const Segment SGI144786[] = {{&S31168[76], 6}, {NULL}};
const Segment SGI144787[] = {{&S41350[44], 14}, {NULL}};
const Segment SGI144788[] = {{&S41350[44], 23}, {NULL}};
const Segment SGI144789[] = {{&S41350[30], 6}, {NULL}};
const Segment SGI144790[] = {{&S41350[31], 3}, {NULL}};
const Segment SGI144791[] = {{&S41350[81], 6}, {NULL}};
const Segment SGI144792[] = {{&S41351[44], 9}, {NULL}};
const Segment SGI144793[] = {{&S41351[44], 21}, {NULL}};
const Segment SGI144794[] = {{&S41351[30], 6}, {NULL}};
const Segment SGI144795[] = {{&S41351[31], 3}, {NULL}};
const Segment SGI144796[] = {{&S41351[79], 6}, {NULL}};
const Segment SGI144797[] = {{&S41352[38], 9}, {NULL}};
const Segment SGI144798[] = {{&S41352[31], 6}, {NULL}};
const Segment SGI144799[] = {{&S41363[44], 13}, {NULL}};
const Segment SGI144800[] = {{&S41363[44], 25}, {NULL}};
const Segment SGI144801[] = {{&S41363[30], 6}, {NULL}};
const Segment SGI144802[] = {{&S41363[31], 3}, {NULL}};
const Segment SGI144803[] = {{&S41363[83], 6}, {NULL}};
const Segment SGI144804[] = {{&S41364[44], 10}, {NULL}};
const Segment SGI144805[] = {{&S41364[44], 22}, {NULL}};
const Segment SGI144806[] = {{&S41364[30], 6}, {NULL}};
const Segment SGI144807[] = {{&S41364[31], 3}, {NULL}};
const Segment SGI144808[] = {{&S41364[80], 6}, {NULL}};
const Segment SGI144809[] = {{&S41365[38], 10}, {NULL}};
const Segment SGI144810[] = {{&S41365[31], 6}, {NULL}};
const Segment SGI144811[] = {{&S41485[44], 14}, {NULL}};
const Segment SGI144812[] = {{&S41485[44], 23}, {NULL}};
const Segment SGI144813[] = {{&S41485[30], 6}, {NULL}};
const Segment SGI144814[] = {{&S41485[31], 3}, {NULL}};
const Segment SGI144815[] = {{&S41485[81], 6}, {NULL}};
const Segment SGI144816[] = {{&S41486[44], 9}, {NULL}};
const Segment SGI144817[] = {{&S41486[44], 21}, {NULL}};
const Segment SGI144818[] = {{&S41486[30], 6}, {NULL}};
const Segment SGI144819[] = {{&S41486[31], 3}, {NULL}};
const Segment SGI144820[] = {{&S41486[79], 6}, {NULL}};
const Segment SGI144821[] = {{&S41487[38], 9}, {NULL}};
const Segment SGI144822[] = {{&S41487[31], 6}, {NULL}};
const Segment SGI144823[] = {{&S41490[34], 7}, {NULL}};
const Segment SGI144824[] = {{&S41490[27], 6}, {NULL}};
const Segment SGI144825[] = {{&S41490[98], 9}, {NULL}};
const Segment SGI144826[] = {{&S41490[71], 6}, {NULL}};
const Segment SGI144827[] = {{&S41492[34], 8}, {NULL}};
const Segment SGI144828[] = {{&S41492[27], 6}, {NULL}};
const Segment SGI144829[] = {{&S41492[99], 9}, {NULL}};
const Segment SGI144830[] = {{&S41492[72], 6}, {NULL}};
const Segment SGI144831[] = {{&S32533[34], 7}, {NULL}};
const Segment SGI144832[] = {{&S32533[27], 6}, {NULL}};
const Segment SGI144833[] = {{&S32533[99], 9}, {NULL}};
const Segment SGI144834[] = {{&S32533[72], 6}, {NULL}};
const Segment SGI144835[] = {{&S31709[34], 7}, {NULL}};
const Segment SGI144836[] = {{&S31709[27], 6}, {NULL}};
const Segment SGI144837[] = {{&S31709[103], 9}, {NULL}};
const Segment SGI144838[] = {{&S31709[76], 6}, {NULL}};
const Segment SGI144839[] = {{&S31710[34], 7}, {NULL}};
const Segment SGI144840[] = {{&S31710[27], 6}, {NULL}};
const Segment SGI144841[] = {{&S31710[97], 9}, {NULL}};
const Segment SGI144842[] = {{&S31710[70], 6}, {NULL}};
const Segment SGI144843[] = {{&S41635[44], 25}, {NULL}};
const Segment SGI144844[] = {{&S41635[44], 37}, {NULL}};
const Segment SGI144845[] = {{&S41635[30], 6}, {NULL}};
const Segment SGI144846[] = {{&S41635[31], 3}, {NULL}};
const Segment SGI144847[] = {{&S41635[95], 6}, {NULL}};
const Segment SGI144848[] = {{&S31789[34], 15}, {NULL}};
const Segment SGI144849[] = {{&S31789[27], 6}, {NULL}};
const Segment SGI144850[] = {{&S31789[105], 9}, {NULL}};
const Segment SGI144851[] = {{&S31789[78], 6}, {NULL}};
const Segment SGI144852[] = {{&S31790[34], 7}, {NULL}};
const Segment SGI144853[] = {{&S31790[27], 6}, {NULL}};
const Segment SGI144854[] = {{&S31790[111], 9}, {NULL}};
const Segment SGI144855[] = {{&S31790[84], 6}, {NULL}};
const Segment SGI144856[] = {{&S31808[44], 11}, {NULL}};
const Segment SGI144857[] = {{&S31808[44], 23}, {NULL}};
const Segment SGI144858[] = {{&S31808[30], 6}, {NULL}};
const Segment SGI144859[] = {{&S31808[116], 3}, {NULL}};
const Segment SGI144860[] = {{&S31808[81], 6}, {NULL}};
const Segment SGI144861[] = {{&S31809[44], 32}, {NULL}};
const Segment SGI144862[] = {{&S31809[44], 44}, {NULL}};
const Segment SGI144863[] = {{&S31809[30], 6}, {NULL}};
const Segment SGI144864[] = {{&S31809[137], 3}, {NULL}};
const Segment SGI144865[] = {{&S31809[102], 6}, {NULL}};
const Segment SGI144866[] = {{&S31810[44], 10}, {NULL}};
const Segment SGI144867[] = {{&S31810[44], 22}, {NULL}};
const Segment SGI144868[] = {{&S31810[30], 6}, {NULL}};
const Segment SGI144869[] = {{&S31810[115], 3}, {NULL}};
const Segment SGI144870[] = {{&S31810[80], 6}, {NULL}};
const Segment SGI144871[] = {{&S31812[38], 11}, {NULL}};
const Segment SGI144872[] = {{&S31812[31], 6}, {NULL}};
const Segment SGI144873[] = {{&S31813[38], 10}, {NULL}};
const Segment SGI144874[] = {{&S31813[31], 6}, {NULL}};
const Segment SGI144875[] = {{&S31814[44], 32}, {NULL}};
const Segment SGI144876[] = {{&S31814[44], 44}, {NULL}};
const Segment SGI144877[] = {{&S31814[30], 6}, {NULL}};
const Segment SGI144878[] = {{&S31814[137], 3}, {NULL}};
const Segment SGI144879[] = {{&S31814[102], 6}, {NULL}};
const Segment SGI144880[] = {{&S31815[44], 32}, {NULL}};
const Segment SGI144881[] = {{&S31815[44], 44}, {NULL}};
const Segment SGI144882[] = {{&S31815[30], 6}, {NULL}};
const Segment SGI144883[] = {{&S31815[137], 3}, {NULL}};
const Segment SGI144884[] = {{&S31815[102], 6}, {NULL}};
const Segment SGI144885[] = {{&S31816[44], 31}, {NULL}};
const Segment SGI144886[] = {{&S31816[44], 43}, {NULL}};
const Segment SGI144887[] = {{&S31816[30], 6}, {NULL}};
const Segment SGI144888[] = {{&S31816[136], 3}, {NULL}};
const Segment SGI144889[] = {{&S31816[101], 6}, {NULL}};
const Segment SGI144890[] = {{&S31817[44], 32}, {NULL}};
const Segment SGI144891[] = {{&S31817[44], 44}, {NULL}};
const Segment SGI144892[] = {{&S31817[30], 6}, {NULL}};
const Segment SGI144893[] = {{&S31817[137], 3}, {NULL}};
const Segment SGI144894[] = {{&S31817[102], 6}, {NULL}};
const Segment SGI144895[] = {{&S31818[44], 31}, {NULL}};
const Segment SGI144896[] = {{&S31818[44], 43}, {NULL}};
const Segment SGI144897[] = {{&S31818[30], 6}, {NULL}};
const Segment SGI144898[] = {{&S31818[136], 3}, {NULL}};
const Segment SGI144899[] = {{&S31818[101], 6}, {NULL}};
const Segment SGI144900[] = {{&S41744[44], 10}, {NULL}};
const Segment SGI144901[] = {{&S41744[44], 22}, {NULL}};
const Segment SGI144902[] = {{&S31819[48], 10}, {NULL}};
const Segment SGI144903[] = {{&S31819[48], 22}, {NULL}};
const Segment SGI144904[] = {{&S31819[30], 10}, {NULL}};
const Segment SGI144905[] = {{&S41744[30], 6}, {NULL}};
const Segment SGI144906[] = {{&S31819[119], 3}, {NULL}};
const Segment SGI144907[] = {{&S31819[84], 6}, {NULL}};
const Segment SGI144908[] = {{&S41744[80], 6}, {NULL}};
const Segment SGI144909[] = {{&S41745[44], 10}, {NULL}};
const Segment SGI144910[] = {{&S41745[44], 22}, {NULL}};
const Segment SGI144911[] = {{&S41745[30], 6}, {NULL}};
const Segment SGI144912[] = {{&S41745[31], 3}, {NULL}};
const Segment SGI144913[] = {{&S41745[80], 6}, {NULL}};
const Segment SGI144914[] = {{&S41746[44], 10}, {NULL}};
const Segment SGI144915[] = {{&S41746[44], 22}, {NULL}};
const Segment SGI144916[] = {{&S41746[30], 6}, {NULL}};
const Segment SGI144917[] = {{&S41746[31], 3}, {NULL}};
const Segment SGI144918[] = {{&S41746[80], 6}, {NULL}};
const Segment SGI144919[] = {{&S41747[44], 10}, {NULL}};
const Segment SGI144920[] = {{&S41747[44], 22}, {NULL}};
const Segment SGI144921[] = {{&S41747[30], 6}, {NULL}};
const Segment SGI144922[] = {{&S41747[31], 3}, {NULL}};
const Segment SGI144923[] = {{&S41747[80], 6}, {NULL}};
const Segment SGI144924[] = {{&S41749[44], 10}, {NULL}};
const Segment SGI144925[] = {{&S41749[44], 22}, {NULL}};
const Segment SGI144926[] = {{&S41749[30], 6}, {NULL}};
const Segment SGI144927[] = {{&S41749[31], 3}, {NULL}};
const Segment SGI144928[] = {{&S41749[80], 6}, {NULL}};
const Segment SGI144929[] = {{&S41750[44], 10}, {NULL}};
const Segment SGI144930[] = {{&S41750[44], 22}, {NULL}};
const Segment SGI144931[] = {{&S41750[30], 6}, {NULL}};
const Segment SGI144932[] = {{&S41750[31], 3}, {NULL}};
const Segment SGI144933[] = {{&S41750[80], 6}, {NULL}};
const Segment SGI144934[] = {{&S41751[38], 10}, {NULL}};
const Segment SGI144935[] = {{&S41751[31], 6}, {NULL}};
const Segment SGI144936[] = {{&S41754[44], 10}, {NULL}};
const Segment SGI144937[] = {{&S41754[44], 37}, {NULL}};
const Segment SGI144938[] = {{&S41754[30], 6}, {NULL}};
const Segment SGI144939[] = {{&S41754[31], 3}, {NULL}};
const Segment SGI144940[] = {{&S41754[95], 6}, {NULL}};
const Segment SGI144941[] = {{&S41756[44], 11}, {NULL}};
const Segment SGI144942[] = {{&S41756[44], 23}, {NULL}};
const Segment SGI144943[] = {{&S41756[30], 6}, {NULL}};
const Segment SGI144944[] = {{&S41756[31], 3}, {NULL}};
const Segment SGI144945[] = {{&S41756[81], 6}, {NULL}};
const Segment SGI144946[] = {{&S41757[44], 10}, {NULL}};
const Segment SGI144947[] = {{&S41757[44], 22}, {NULL}};
const Segment SGI144948[] = {{&S41757[30], 6}, {NULL}};
const Segment SGI144949[] = {{&S41757[31], 3}, {NULL}};
const Segment SGI144950[] = {{&S41757[80], 6}, {NULL}};
const Segment SGI144951[] = {{&S41758[44], 10}, {NULL}};
const Segment SGI144952[] = {{&S41758[44], 21}, {NULL}};
const Segment SGI144953[] = {{&S41758[30], 6}, {NULL}};
const Segment SGI144954[] = {{&S41758[31], 3}, {NULL}};
const Segment SGI144955[] = {{&S41758[79], 6}, {NULL}};
const Segment SGI144956[] = {{&S41759[44], 10}, {NULL}};
const Segment SGI144957[] = {{&S41759[44], 21}, {NULL}};
const Segment SGI144958[] = {{&S41759[30], 6}, {NULL}};
const Segment SGI144959[] = {{&S41759[31], 3}, {NULL}};
const Segment SGI144960[] = {{&S41759[79], 6}, {NULL}};
const Segment SGI144961[] = {{&S31914[44], 11}, {NULL}};
const Segment SGI144962[] = {{&S31914[44], 28}, {NULL}};
const Segment SGI144963[] = {{&S31914[30], 6}, {NULL}};
const Segment SGI144964[] = {{&S31914[121], 3}, {NULL}};
const Segment SGI144965[] = {{&S31914[86], 6}, {NULL}};
const Segment SGI144966[] = {{&S31915[44], 7}, {NULL}};
const Segment SGI144967[] = {{&S31915[44], 24}, {NULL}};
const Segment SGI144968[] = {{&S31915[30], 6}, {NULL}};
const Segment SGI144969[] = {{&S31915[117], 3}, {NULL}};
const Segment SGI144970[] = {{&S31915[82], 6}, {NULL}};
const Segment SGI144971[] = {{&S31916[44], 12}, {NULL}};
const Segment SGI144972[] = {{&S31916[44], 29}, {NULL}};
const Segment SGI144973[] = {{&S31916[30], 6}, {NULL}};
const Segment SGI144974[] = {{&S31916[122], 3}, {NULL}};
const Segment SGI144975[] = {{&S31916[87], 6}, {NULL}};
const Segment SGI144976[] = {{&S31917[44], 11}, {NULL}};
const Segment SGI144977[] = {{&S31917[44], 28}, {NULL}};
const Segment SGI144978[] = {{&S31917[30], 6}, {NULL}};
const Segment SGI144979[] = {{&S31917[121], 3}, {NULL}};
const Segment SGI144980[] = {{&S31917[86], 6}, {NULL}};
const Segment SGI144981[] = {{&S31918[44], 7}, {NULL}};
const Segment SGI144982[] = {{&S31918[44], 24}, {NULL}};
const Segment SGI144983[] = {{&S31918[30], 6}, {NULL}};
const Segment SGI144984[] = {{&S31918[117], 3}, {NULL}};
const Segment SGI144985[] = {{&S31918[82], 6}, {NULL}};
const Segment SGI144986[] = {{&S31919[38], 7}, {NULL}};
const Segment SGI144987[] = {{&S31919[31], 6}, {NULL}};
const Segment SGI144988[] = {{&S31920[44], 11}, {NULL}};
const Segment SGI144989[] = {{&S31920[44], 28}, {NULL}};
const Segment SGI144990[] = {{&S31920[30], 6}, {NULL}};
const Segment SGI144991[] = {{&S31920[121], 3}, {NULL}};
const Segment SGI144992[] = {{&S31920[86], 6}, {NULL}};
const Segment SGI144993[] = {{&S31921[44], 13}, {NULL}};
const Segment SGI144994[] = {{&S31921[44], 30}, {NULL}};
const Segment SGI144995[] = {{&S31921[30], 6}, {NULL}};
const Segment SGI144996[] = {{&S31921[123], 3}, {NULL}};
const Segment SGI144997[] = {{&S31921[88], 6}, {NULL}};
const Segment SGI144998[] = {{&S31922[44], 12}, {NULL}};
const Segment SGI144999[] = {{&S31922[44], 29}, {NULL}};
const Segment SGI145000[] = {{&S31922[30], 6}, {NULL}};
const Segment SGI145001[] = {{&S31922[122], 3}, {NULL}};
const Segment SGI145002[] = {{&S31922[87], 6}, {NULL}};
const Segment SGI145003[] = {{&S31923[44], 7}, {NULL}};
const Segment SGI145004[] = {{&S31923[44], 24}, {NULL}};
const Segment SGI145005[] = {{&S31923[30], 6}, {NULL}};
const Segment SGI145006[] = {{&S31923[117], 3}, {NULL}};
const Segment SGI145007[] = {{&S31923[82], 6}, {NULL}};
const Segment SGI145008[] = {{&S31925[44], 11}, {NULL}};
const Segment SGI145009[] = {{&S31925[44], 28}, {NULL}};
const Segment SGI145010[] = {{&S31925[30], 6}, {NULL}};
const Segment SGI145011[] = {{&S31925[121], 3}, {NULL}};
const Segment SGI145012[] = {{&S31925[86], 6}, {NULL}};
const Segment SGI145013[] = {{&S31926[38], 7}, {NULL}};
const Segment SGI145014[] = {{&S31926[31], 6}, {NULL}};
const Segment SGI145015[] = {{&S31928[44], 7}, {NULL}};
const Segment SGI145016[] = {{&S31928[44], 24}, {NULL}};
const Segment SGI145017[] = {{&S31928[30], 6}, {NULL}};
const Segment SGI145018[] = {{&S31928[117], 3}, {NULL}};
const Segment SGI145019[] = {{&S31928[82], 6}, {NULL}};
const Segment SGI145020[] = {{&S31929[44], 11}, {NULL}};
const Segment SGI145021[] = {{&S31929[44], 28}, {NULL}};
const Segment SGI145022[] = {{&S31929[30], 6}, {NULL}};
const Segment SGI145023[] = {{&S31929[121], 3}, {NULL}};
const Segment SGI145024[] = {{&S31929[86], 6}, {NULL}};
const Segment SGI145025[] = {{&S32844[44], 10}, {NULL}};
const Segment SGI145026[] = {{&S31930[44], 11}, {NULL}};
const Segment SGI145027[] = {{&S32844[44], 22}, {NULL}};
const Segment SGI145028[] = {{&S32844[30], 6}, {NULL}};
const Segment SGI145029[] = {{&S31930[44], 28}, {NULL}};
const Segment SGI145030[] = {{&S31930[30], 6}, {NULL}};
const Segment SGI145031[] = {{&S32844[115], 3}, {NULL}};
const Segment SGI145032[] = {{&S31930[121], 3}, {NULL}};
const Segment SGI145033[] = {{&S32844[80], 6}, {NULL}};
const Segment SGI145034[] = {{&S31930[86], 6}, {NULL}};
const Segment SGI145035[] = {{&S32845[44], 10}, {NULL}};
const Segment SGI145036[] = {{&S31931[44], 7}, {NULL}};
const Segment SGI145037[] = {{&S32845[44], 22}, {NULL}};
const Segment SGI145038[] = {{&S31931[44], 24}, {NULL}};
const Segment SGI145039[] = {{&S32845[30], 6}, {NULL}};
const Segment SGI145040[] = {{&S31931[30], 6}, {NULL}};
const Segment SGI145041[] = {{&S32845[115], 3}, {NULL}};
const Segment SGI145042[] = {{&S31931[117], 3}, {NULL}};
const Segment SGI145043[] = {{&S32845[80], 6}, {NULL}};
const Segment SGI145044[] = {{&S31931[82], 6}, {NULL}};
const Segment SGI145045[] = {{&S31932[44], 7}, {NULL}};
const Segment SGI145046[] = {{&S31932[44], 24}, {NULL}};
const Segment SGI145047[] = {{&S32846[44], 10}, {NULL}};
const Segment SGI145048[] = {{&S31932[30], 6}, {NULL}};
const Segment SGI145049[] = {{&S32846[44], 22}, {NULL}};
const Segment SGI145050[] = {{&S32846[30], 6}, {NULL}};
const Segment SGI145051[] = {{&S31932[117], 3}, {NULL}};
const Segment SGI145052[] = {{&S32846[115], 3}, {NULL}};
const Segment SGI145053[] = {{&S31932[82], 6}, {NULL}};
const Segment SGI145054[] = {{&S32846[80], 6}, {NULL}};
const Segment SGI145055[] = {{&S32848[38], 10}, {NULL}};
const Segment SGI145056[] = {{&S32848[31], 6}, {NULL}};
const Segment SGI145057[] = {{&S41781[34], 20}, {NULL}};
const Segment SGI145058[] = {{&S41781[27], 6}, {NULL}};
const Segment SGI145059[] = {{&S41781[107], 9}, {NULL}};
const Segment SGI145060[] = {{&S41781[80], 6}, {NULL}};
const Segment SGI145061[] = {{&S41783[34], 19}, {NULL}};
const Segment SGI145062[] = {{&S41783[27], 6}, {NULL}};
const Segment SGI145063[] = {{&S41783[106], 9}, {NULL}};
const Segment SGI145064[] = {{&S41783[79], 6}, {NULL}};
const Segment SGI145065[] = {{&S41784[34], 13}, {NULL}};
const Segment SGI145066[] = {{&S41784[27], 6}, {NULL}};
const Segment SGI145067[] = {{&S41784[100], 9}, {NULL}};
const Segment SGI145068[] = {{&S41784[73], 6}, {NULL}};
const Segment SGI145069[] = {{&S41786[44], 13}, {NULL}};
const Segment SGI145070[] = {{&S41786[44], 25}, {NULL}};
const Segment SGI145071[] = {{&S41786[30], 6}, {NULL}};
const Segment SGI145072[] = {{&S41786[31], 3}, {NULL}};
const Segment SGI145073[] = {{&S41786[83], 6}, {NULL}};
const Segment SGI145074[] = {{&S41787[44], 10}, {NULL}};
const Segment SGI145075[] = {{&S41787[44], 22}, {NULL}};
const Segment SGI145076[] = {{&S41787[30], 6}, {NULL}};
const Segment SGI145077[] = {{&S41787[31], 3}, {NULL}};
const Segment SGI145078[] = {{&S41787[80], 6}, {NULL}};
const Segment SGI145079[] = {{&S41788[38], 10}, {NULL}};
const Segment SGI145080[] = {{&S41788[31], 6}, {NULL}};
const Segment SGI145081[] = {{&S32931[34], 7}, {NULL}};
const Segment SGI145082[] = {{&S32931[27], 6}, {NULL}};
const Segment SGI145083[] = {{&S32931[99], 9}, {NULL}};
const Segment SGI145084[] = {{&S32931[72], 6}, {NULL}};
const Segment SGI145085[] = {{&S32932[34], 7}, {NULL}};
const Segment SGI145086[] = {{&S32932[27], 6}, {NULL}};
const Segment SGI145087[] = {{&S32932[99], 9}, {NULL}};
const Segment SGI145088[] = {{&S32932[72], 6}, {NULL}};
const Segment SGI145089[] = {{&S41827[44], 18}, {NULL}};
const Segment SGI145090[] = {{&S41827[44], 30}, {NULL}};
const Segment SGI145091[] = {{&S41827[30], 6}, {NULL}};
const Segment SGI145092[] = {{&S41827[31], 3}, {NULL}};
const Segment SGI145093[] = {{&S41827[88], 6}, {NULL}};
const Segment SGI145094[] = {{&S41828[42], 18}, {NULL}};
const Segment SGI145095[] = {{&S41828[30], 6}, {NULL}};
const Segment SGI145096[] = {{&S41828[31], 3}, {NULL}};
const Segment SGI145097[] = {{&S41828[86], 6}, {NULL}};
const Segment SGI145098[] = {{&S41829[38], 18}, {NULL}};
const Segment SGI145099[] = {{&S41829[31], 6}, {NULL}};
const Segment SGI145100[] = {{&S32080[34], 9}, {NULL}};
const Segment SGI145101[] = {{&S32080[27], 6}, {NULL}};
const Segment SGI145102[] = {{&S32080[96], 9}, {NULL}};
const Segment SGI145103[] = {{&S32080[69], 6}, {NULL}};
const Segment SGI145104[] = {{&S32086[34], 9}, {NULL}};
const Segment SGI145105[] = {{&S32086[27], 6}, {NULL}};
const Segment SGI145106[] = {{&S32086[96], 9}, {NULL}};
const Segment SGI145107[] = {{&S32086[69], 6}, {NULL}};
const Segment SGI145108[] = {{&S41868[44], 11}, {NULL}};
const Segment SGI145109[] = {{&S41868[44], 23}, {NULL}};
const Segment SGI145110[] = {{&S41868[30], 6}, {NULL}};
const Segment SGI145111[] = {{&S41868[31], 3}, {NULL}};
const Segment SGI145112[] = {{&S41868[81], 6}, {NULL}};
const Segment SGI145113[] = {{&S41869[44], 11}, {NULL}};
const Segment SGI145114[] = {{&S41869[44], 23}, {NULL}};
const Segment SGI145115[] = {{&S41869[30], 6}, {NULL}};
const Segment SGI145116[] = {{&S41869[31], 3}, {NULL}};
const Segment SGI145117[] = {{&S41869[81], 6}, {NULL}};
const Segment SGI145118[] = {{&S41870[44], 11}, {NULL}};
const Segment SGI145119[] = {{&S41870[44], 23}, {NULL}};
const Segment SGI145120[] = {{&S41870[30], 6}, {NULL}};
const Segment SGI145121[] = {{&S41870[31], 3}, {NULL}};
const Segment SGI145122[] = {{&S41870[81], 6}, {NULL}};
const Segment SGI145123[] = {{&S41871[44], 12}, {NULL}};
const Segment SGI145124[] = {{&S41871[44], 24}, {NULL}};
const Segment SGI145125[] = {{&S41871[30], 6}, {NULL}};
const Segment SGI145126[] = {{&S41871[31], 3}, {NULL}};
const Segment SGI145127[] = {{&S41871[82], 6}, {NULL}};
const Segment SGI145128[] = {{&S41872[38], 12}, {NULL}};
const Segment SGI145129[] = {{&S41872[31], 6}, {NULL}};
const Segment SGI145130[] = {{&S41873[44], 22}, {NULL}};
const Segment SGI145131[] = {{&S41873[44], 34}, {NULL}};
const Segment SGI145132[] = {{&S41873[30], 6}, {NULL}};
const Segment SGI145133[] = {{&S41873[31], 3}, {NULL}};
const Segment SGI145134[] = {{&S41873[92], 6}, {NULL}};
const Segment SGI145135[] = {{&S41874[44], 22}, {NULL}};
const Segment SGI145136[] = {{&S41874[44], 34}, {NULL}};
const Segment SGI145137[] = {{&S41874[30], 6}, {NULL}};
const Segment SGI145138[] = {{&S41874[31], 3}, {NULL}};
const Segment SGI145139[] = {{&S41874[92], 6}, {NULL}};
const Segment SGI145140[] = {{&S41875[44], 22}, {NULL}};
const Segment SGI145141[] = {{&S41875[44], 34}, {NULL}};
const Segment SGI145142[] = {{&S41875[30], 6}, {NULL}};
const Segment SGI145143[] = {{&S41875[31], 3}, {NULL}};
const Segment SGI145144[] = {{&S41875[92], 6}, {NULL}};
const Segment SGI145145[] = {{&S41876[44], 22}, {NULL}};
const Segment SGI145146[] = {{&S41876[44], 34}, {NULL}};
const Segment SGI145147[] = {{&S41876[30], 6}, {NULL}};
const Segment SGI145148[] = {{&S41876[31], 3}, {NULL}};
const Segment SGI145149[] = {{&S41876[92], 6}, {NULL}};
const Segment SGI145150[] = {{&S41877[44], 31}, {NULL}};
const Segment SGI145151[] = {{&S41877[44], 43}, {NULL}};
const Segment SGI145152[] = {{&S41877[30], 6}, {NULL}};
const Segment SGI145153[] = {{&S41877[31], 3}, {NULL}};
const Segment SGI145154[] = {{&S41877[101], 6}, {NULL}};
const Segment SGI145155[] = {{&S41878[44], 22}, {NULL}};
const Segment SGI145156[] = {{&S41878[44], 34}, {NULL}};
const Segment SGI145157[] = {{&S41878[30], 6}, {NULL}};
const Segment SGI145158[] = {{&S41878[31], 3}, {NULL}};
const Segment SGI145159[] = {{&S41878[92], 6}, {NULL}};
const Segment SGI145160[] = {{&S41879[44], 23}, {NULL}};
const Segment SGI145161[] = {{&S41879[44], 35}, {NULL}};
const Segment SGI145162[] = {{&S41879[30], 6}, {NULL}};
const Segment SGI145163[] = {{&S41879[31], 3}, {NULL}};
const Segment SGI145164[] = {{&S41879[93], 6}, {NULL}};
const Segment SGI145165[] = {{&S41880[44], 22}, {NULL}};
const Segment SGI145166[] = {{&S41880[44], 34}, {NULL}};
const Segment SGI145167[] = {{&S41880[30], 6}, {NULL}};
const Segment SGI145168[] = {{&S41880[31], 3}, {NULL}};
const Segment SGI145169[] = {{&S41880[92], 6}, {NULL}};
const Segment SGI145170[] = {{&S41881[44], 32}, {NULL}};
const Segment SGI145171[] = {{&S41881[44], 44}, {NULL}};
const Segment SGI145172[] = {{&S41881[30], 6}, {NULL}};
const Segment SGI145173[] = {{&S41881[31], 3}, {NULL}};
const Segment SGI145174[] = {{&S41881[102], 6}, {NULL}};
const Segment SGI145175[] = {{&S41882[44], 22}, {NULL}};
const Segment SGI145176[] = {{&S41882[44], 34}, {NULL}};
const Segment SGI145177[] = {{&S41882[30], 6}, {NULL}};
const Segment SGI145178[] = {{&S41882[31], 3}, {NULL}};
const Segment SGI145179[] = {{&S41882[92], 6}, {NULL}};
const Segment SGI145180[] = {{&S41889[44], 18}, {NULL}};
const Segment SGI145181[] = {{&S41889[44], 30}, {NULL}};
const Segment SGI145182[] = {{&S41889[30], 6}, {NULL}};
const Segment SGI145183[] = {{&S41889[31], 3}, {NULL}};
const Segment SGI145184[] = {{&S41889[88], 6}, {NULL}};
const Segment SGI145185[] = {{&S41890[38], 18}, {NULL}};
const Segment SGI145186[] = {{&S41890[31], 6}, {NULL}};
const Segment SGI145187[] = {{&S41891[44], 10}, {NULL}};
const Segment SGI145188[] = {{&S41891[44], 22}, {NULL}};
const Segment SGI145189[] = {{&S41891[30], 6}, {NULL}};
const Segment SGI145190[] = {{&S41891[31], 3}, {NULL}};
const Segment SGI145191[] = {{&S41891[80], 6}, {NULL}};
const Segment SGI145192[] = {{&S41926[44], 13}, {NULL}};
const Segment SGI145193[] = {{&S41926[44], 29}, {NULL}};
const Segment SGI145194[] = {{&S41926[30], 6}, {NULL}};
const Segment SGI145195[] = {{&S41926[31], 3}, {NULL}};
const Segment SGI145196[] = {{&S41926[87], 6}, {NULL}};
const Segment SGI145197[] = {{&S41927[38], 13}, {NULL}};
const Segment SGI145198[] = {{&S41927[31], 6}, {NULL}};
const Segment SGI145199[] = {{&S41928[44], 13}, {NULL}};
const Segment SGI145200[] = {{&S41928[44], 29}, {NULL}};
const Segment SGI145201[] = {{&S41928[30], 6}, {NULL}};
const Segment SGI145202[] = {{&S41928[31], 3}, {&S41928[165], 3}, {NULL}};
const Segment SGI145203[] = {{&S41928[87], 6}, {NULL}};
const Segment SGI145204[] = {{&S41986[34], 10}, {NULL}};
const Segment SGI145205[] = {{&S41986[27], 6}, {NULL}};
const Segment SGI145206[] = {{&S41986[97], 9}, {NULL}};
const Segment SGI145207[] = {{&S41986[70], 6}, {NULL}};
const Segment SGI145208[] = {{&S41987[34], 14}, {NULL}};
const Segment SGI145209[] = {{&S41987[27], 6}, {NULL}};
const Segment SGI145210[] = {{&S41987[101], 9}, {NULL}};
const Segment SGI145211[] = {{&S41987[74], 6}, {NULL}};
const Segment SGI145212[] = {{&S41988[34], 13}, {NULL}};
const Segment SGI145213[] = {{&S41988[27], 6}, {NULL}};
const Segment SGI145214[] = {{&S41988[100], 9}, {NULL}};
const Segment SGI145215[] = {{&S41988[73], 6}, {NULL}};
const Segment SGI145216[] = {{&S41989[34], 10}, {NULL}};
const Segment SGI145217[] = {{&S41989[27], 6}, {NULL}};
const Segment SGI145218[] = {{&S41989[97], 9}, {NULL}};
const Segment SGI145219[] = {{&S41989[70], 6}, {NULL}};
const Segment SGI145220[] = {{&S27555[44], 29}, {NULL}};
const Segment SGI145221[] = {{&S27555[44], 41}, {NULL}};
const Segment SGI145222[] = {{&S27555[30], 6}, {NULL}};
const Segment SGI145223[] = {{&S27555[134], 3}, {NULL}};
const Segment SGI145224[] = {{&S27555[99], 6}, {NULL}};
const Segment SGI145225[] = {{&S27556[38], 29}, {NULL}};
const Segment SGI145226[] = {{&S27556[31], 6}, {NULL}};
const Segment SGI145227[] = {{&S27557[38], 30}, {NULL}};
const Segment SGI145228[] = {{&S27557[31], 6}, {NULL}};
const Segment SGI145229[] = {{&S27558[44], 30}, {NULL}};
const Segment SGI145230[] = {{&S27558[44], 42}, {NULL}};
const Segment SGI145231[] = {{&S27558[30], 6}, {NULL}};
const Segment SGI145232[] = {{&S27558[135], 3}, {NULL}};
const Segment SGI145233[] = {{&S27558[100], 6}, {NULL}};
const Segment SGI145234[] = {{&S27559[44], 30}, {NULL}};
const Segment SGI145235[] = {{&S27559[44], 42}, {NULL}};
const Segment SGI145236[] = {{&S27559[30], 6}, {NULL}};
const Segment SGI145237[] = {{&S27559[135], 3}, {NULL}};
const Segment SGI145238[] = {{&S27559[100], 6}, {NULL}};
const Segment SGI145239[] = {{&S27560[44], 17}, {NULL}};
const Segment SGI145240[] = {{&S27560[44], 29}, {NULL}};
const Segment SGI145241[] = {{&S27560[30], 6}, {NULL}};
const Segment SGI145242[] = {{&S27560[122], 3}, {NULL}};
const Segment SGI145243[] = {{&S27560[87], 6}, {NULL}};
const Segment SGI145244[] = {{&S27561[44], 24}, {NULL}};
const Segment SGI145245[] = {{&S27561[44], 36}, {NULL}};
const Segment SGI145246[] = {{&S27561[30], 6}, {NULL}};
const Segment SGI145247[] = {{&S27561[129], 3}, {NULL}};
const Segment SGI145248[] = {{&S27561[94], 6}, {NULL}};
const Segment SGI145249[] = {{&S27563[44], 29}, {NULL}};
const Segment SGI145250[] = {{&S27563[44], 41}, {NULL}};
const Segment SGI145251[] = {{&S27563[30], 6}, {NULL}};
const Segment SGI145252[] = {{&S27563[134], 3}, {NULL}};
const Segment SGI145253[] = {{&S27563[99], 6}, {NULL}};
const Segment SGI145254[] = {{&S27564[38], 30}, {NULL}};
const Segment SGI145255[] = {{&S27564[31], 6}, {NULL}};
const Segment SGI145256[] = {{&S42102[44], 15}, {NULL}};
const Segment SGI145257[] = {{&S42102[44], 27}, {NULL}};
const Segment SGI145258[] = {{&S42102[30], 6}, {NULL}};
const Segment SGI145259[] = {{&S42102[31], 3}, {NULL}};
const Segment SGI145260[] = {{&S42102[85], 6}, {NULL}};
const Segment SGI145261[] = {{&S42103[38], 15}, {NULL}};
const Segment SGI145262[] = {{&S42103[31], 6}, {NULL}};
const Segment SGI145263[] = {{&S42104[44], 15}, {NULL}};
const Segment SGI145264[] = {{&S42104[44], 27}, {NULL}};
const Segment SGI145265[] = {{&S42104[30], 6}, {NULL}};
const Segment SGI145266[] = {{&S42104[88], 6}, {NULL}};
const Segment SGI145267[] = {{&S42155[34], 10}, {NULL}};
const Segment SGI145268[] = {{&S42155[27], 6}, {NULL}};
const Segment SGI145269[] = {{&S42155[97], 9}, {NULL}};
const Segment SGI145270[] = {{&S42155[70], 6}, {NULL}};
const Segment SGI145271[] = {{&S42157[34], 11}, {NULL}};
const Segment SGI145272[] = {{&S42157[27], 6}, {NULL}};
const Segment SGI145273[] = {{&S42157[98], 9}, {NULL}};
const Segment SGI145274[] = {{&S42157[71], 6}, {NULL}};
const Segment SGI145275[] = {{&S42158[44], 10}, {NULL}};
const Segment SGI145276[] = {{&S42158[44], 22}, {NULL}};
const Segment SGI145277[] = {{&S42158[30], 6}, {NULL}};
const Segment SGI145278[] = {{&S42158[31], 3}, {NULL}};
const Segment SGI145279[] = {{&S42158[80], 6}, {NULL}};
const Segment SGI145280[] = {{&S42159[38], 10}, {NULL}};
const Segment SGI145281[] = {{&S42159[31], 6}, {NULL}};
const Segment SGI145282[] = {{&S42167[42], 12}, {NULL}};
const Segment SGI145283[] = {{&S42167[30], 6}, {NULL}};
const Segment SGI145284[] = {{&S42167[31], 3}, {NULL}};
const Segment SGI145285[] = {{&S42167[87], 6}, {NULL}};
const Segment SGI145286[] = {{&S42167[66], 8}, {NULL}};
const Segment SGI145287[] = {{&S42168[44], 12}, {NULL}};
const Segment SGI145288[] = {{&S42168[44], 24}, {NULL}};
const Segment SGI145289[] = {{&S42168[30], 6}, {NULL}};
const Segment SGI145290[] = {{&S42168[31], 3}, {NULL}};
const Segment SGI145291[] = {{&S42168[89], 6}, {NULL}};
const Segment SGI145292[] = {{&S42168[68], 8}, {NULL}};
const Segment SGI145293[] = {{&S42169[44], 15}, {NULL}};
const Segment SGI145294[] = {{&S42169[44], 27}, {NULL}};
const Segment SGI145295[] = {{&S33371[34], 15}, {NULL}};
const Segment SGI145296[] = {{&S42169[30], 6}, {NULL}};
const Segment SGI145297[] = {{&S33371[27], 6}, {NULL}};
const Segment SGI145298[] = {{&S42169[31], 3}, {NULL}};
const Segment SGI145299[] = {{&S42169[92], 6}, {NULL}};
const Segment SGI145300[] = {{&S33371[105], 9}, {NULL}};
const Segment SGI145301[] = {{&S42169[71], 8}, {NULL}};
const Segment SGI145302[] = {{&S33371[78], 6}, {NULL}};
const Segment SGI145303[] = {{&S42170[44], 15}, {NULL}};
const Segment SGI145304[] = {{&S42170[44], 27}, {NULL}};
const Segment SGI145305[] = {{&S42170[30], 6}, {NULL}};
const Segment SGI145306[] = {{&S33372[34], 7}, {NULL}};
const Segment SGI145307[] = {{&S33372[27], 6}, {NULL}};
const Segment SGI145308[] = {{&S42170[31], 3}, {NULL}};
const Segment SGI145309[] = {{&S42170[92], 6}, {NULL}};
const Segment SGI145310[] = {{&S33372[110], 9}, {NULL}};
const Segment SGI145311[] = {{&S42170[71], 8}, {NULL}};
const Segment SGI145312[] = {{&S33372[83], 6}, {NULL}};
const Segment SGI145313[] = {{&S42171[44], 12}, {NULL}};
const Segment SGI145314[] = {{&S42171[44], 24}, {NULL}};
const Segment SGI145315[] = {{&S42171[30], 6}, {NULL}};
const Segment SGI145316[] = {{&S42171[31], 3}, {NULL}};
const Segment SGI145317[] = {{&S42171[89], 6}, {NULL}};
const Segment SGI145318[] = {{&S42171[68], 8}, {NULL}};
const Segment SGI145319[] = {{&S42172[44], 12}, {NULL}};
const Segment SGI145320[] = {{&S42172[44], 24}, {NULL}};
const Segment SGI145321[] = {{&S42172[30], 6}, {NULL}};
const Segment SGI145322[] = {{&S42172[31], 3}, {NULL}};
const Segment SGI145323[] = {{&S42172[82], 6}, {NULL}};
const Segment SGI145324[] = {{&S42172[135], 7}, {NULL}};
const Segment SGI145325[] = {{&S42176[44], 10}, {NULL}};
const Segment SGI145326[] = {{&S42176[44], 22}, {NULL}};
const Segment SGI145327[] = {{&S42176[30], 6}, {NULL}};
const Segment SGI145328[] = {{&S42176[31], 3}, {NULL}};
const Segment SGI145329[] = {{&S42176[80], 6}, {NULL}};
const Segment SGI145330[] = {{&S42177[38], 10}, {NULL}};
const Segment SGI145331[] = {{&S42177[31], 6}, {NULL}};
const Segment SGI145332[] = {{&S33418[34], 15}, {NULL}};
const Segment SGI145333[] = {{&S33418[27], 6}, {NULL}};
const Segment SGI145334[] = {{&S33418[105], 9}, {NULL}};
const Segment SGI145335[] = {{&S33418[78], 6}, {NULL}};
const Segment SGI145336[] = {{&S33422[34], 15}, {NULL}};
const Segment SGI145337[] = {{&S33422[27], 6}, {NULL}};
const Segment SGI145338[] = {{&S33422[105], 9}, {NULL}};
const Segment SGI145339[] = {{&S33422[78], 6}, {NULL}};
const Segment SGI145340[] = {{&S33423[34], 7}, {NULL}};
const Segment SGI145341[] = {{&S33423[27], 6}, {NULL}};
const Segment SGI145342[] = {{&S33423[110], 9}, {NULL}};
const Segment SGI145343[] = {{&S33423[83], 6}, {NULL}};
const Segment SGI145344[] = {{&S32531[34], 7}, {NULL}};
const Segment SGI145345[] = {{&S32531[27], 6}, {NULL}};
const Segment SGI145346[] = {{&S32531[97], 9}, {NULL}};
const Segment SGI145347[] = {{&S32531[70], 6}, {NULL}};
const Segment SGI145348[] = {{&S32532[34], 7}, {NULL}};
const Segment SGI145349[] = {{&S32532[27], 6}, {NULL}};
const Segment SGI145350[] = {{&S32532[103], 9}, {NULL}};
const Segment SGI145351[] = {{&S32532[76], 6}, {NULL}};
const Segment SGI145352[] = {{&S42241[44], 20}, {NULL}};
const Segment SGI145353[] = {{&S42241[44], 32}, {NULL}};
const Segment SGI145354[] = {{&S42241[30], 6}, {NULL}};
const Segment SGI145355[] = {{&S42241[31], 3}, {NULL}};
const Segment SGI145356[] = {{&S42241[90], 6}, {NULL}};
const Segment SGI145357[] = {{&S42242[44], 20}, {NULL}};
const Segment SGI145358[] = {{&S42242[44], 32}, {NULL}};
const Segment SGI145359[] = {{&S42242[30], 6}, {NULL}};
const Segment SGI145360[] = {{&S42242[31], 3}, {NULL}};
const Segment SGI145361[] = {{&S42242[90], 6}, {NULL}};
const Segment SGI145362[] = {{&S42244[51], 20}, {NULL}};
const Segment SGI145363[] = {{&S42244[51], 32}, {NULL}};
const Segment SGI145364[] = {{&S42244[30], 13}, {NULL}};
const Segment SGI145365[] = {{&S42244[31], 3}, {NULL}};
const Segment SGI145366[] = {{&S42244[97], 6}, {NULL}};
const Segment SGI145367[] = {{&S42246[38], 20}, {NULL}};
const Segment SGI145368[] = {{&S42246[31], 6}, {NULL}};
const Segment SGI145369[] = {{&S42248[45], 20}, {NULL}};
const Segment SGI145370[] = {{&S42248[45], 86}, {NULL}};
const Segment SGI145371[] = {{&S42248[31], 6}, {NULL}};
const Segment SGI145372[] = {{&S42253[44], 7}, {NULL}};
const Segment SGI145373[] = {{&S42253[44], 19}, {NULL}};
const Segment SGI145374[] = {{&S42253[30], 6}, {NULL}};
const Segment SGI145375[] = {{&S42253[31], 3}, {NULL}};
const Segment SGI145376[] = {{&S42253[77], 6}, {NULL}};
const Segment SGI145377[] = {{&S42310[44], 14}, {NULL}};
const Segment SGI145378[] = {{&S42310[44], 26}, {NULL}};
const Segment SGI145379[] = {{&S42310[30], 6}, {NULL}};
const Segment SGI145380[] = {{&S42310[31], 3}, {NULL}};
const Segment SGI145381[] = {{&S42310[84], 6}, {NULL}};
const Segment SGI145382[] = {{&S42311[38], 14}, {NULL}};
const Segment SGI145383[] = {{&S42311[31], 6}, {NULL}};
const Segment SGI145384[] = {{&S42312[44], 10}, {NULL}};
const Segment SGI145385[] = {{&S42312[44], 22}, {NULL}};
const Segment SGI145386[] = {{&S42312[30], 6}, {NULL}};
const Segment SGI145387[] = {{&S42312[31], 3}, {NULL}};
const Segment SGI145388[] = {{&S42312[80], 6}, {NULL}};
const Segment SGI145389[] = {{&S42313[38], 10}, {NULL}};
const Segment SGI145390[] = {{&S42313[31], 6}, {NULL}};
const Segment SGI145391[] = {{&S42314[44], 11}, {NULL}};
const Segment SGI145392[] = {{&S42314[44], 23}, {NULL}};
const Segment SGI145393[] = {{&S42314[30], 6}, {NULL}};
const Segment SGI145394[] = {{&S42314[31], 3}, {NULL}};
const Segment SGI145395[] = {{&S42314[81], 6}, {NULL}};
const Segment SGI145396[] = {{&S42315[42], 10}, {NULL}};
const Segment SGI145397[] = {{&S42315[30], 6}, {NULL}};
const Segment SGI145398[] = {{&S42315[31], 3}, {NULL}};
const Segment SGI145399[] = {{&S42315[78], 6}, {NULL}};
const Segment SGI145400[] = {{&S42318[44], 10}, {NULL}};
const Segment SGI145401[] = {{&S42318[44], 22}, {NULL}};
const Segment SGI145402[] = {{&S42318[30], 6}, {NULL}};
const Segment SGI145403[] = {{&S42318[83], 6}, {NULL}};
const Segment SGI145404[] = {{&S42319[44], 10}, {NULL}};
const Segment SGI145405[] = {{&S42319[44], 22}, {NULL}};
const Segment SGI145406[] = {{&S42319[30], 6}, {NULL}};
const Segment SGI145407[] = {{&S42319[125], 3}, {NULL}};
const Segment SGI145408[] = {{&S42319[90], 6}, {NULL}};
const Segment SGI145409[] = {{&S42319[145], 8}, {NULL}};
const Segment SGI145410[] = {{&S42320[44], 10}, {NULL}};
const Segment SGI145411[] = {{&S42320[44], 22}, {NULL}};
const Segment SGI145412[] = {{&S42320[30], 6}, {NULL}};
const Segment SGI145413[] = {{&S42320[31], 3}, {NULL}};
const Segment SGI145414[] = {{&S42320[80], 5}, {NULL}};
const Segment SGI145415[] = {{&S42321[44], 10}, {NULL}};
const Segment SGI145416[] = {{&S42321[44], 22}, {NULL}};
const Segment SGI145417[] = {{&S42321[30], 6}, {NULL}};
const Segment SGI145418[] = {{&S42321[31], 3}, {NULL}};
const Segment SGI145419[] = {{&S42321[80], 6}, {NULL}};
const Segment SGI145420[] = {{&S42322[44], 10}, {NULL}};
const Segment SGI145421[] = {{&S42322[44], 28}, {NULL}};
const Segment SGI145422[] = {{&S42322[30], 6}, {NULL}};
const Segment SGI145423[] = {{&S42323[44], 22}, {NULL}};
const Segment SGI145424[] = {{&S42323[30], 6}, {NULL}};
const Segment SGI145425[] = {{&S42324[38], 11}, {NULL}};
const Segment SGI145426[] = {{&S42324[31], 6}, {NULL}};
const Segment SGI145427[] = {{&S42325[44], 10}, {NULL}};
const Segment SGI145428[] = {{&S42325[44], 22}, {NULL}};
const Segment SGI145429[] = {{&S42325[30], 6}, {NULL}};
const Segment SGI145430[] = {{&S42325[80], 6}, {NULL}};
const Segment SGI145431[] = {{&S42326[44], 9}, {NULL}};
const Segment SGI145432[] = {{&S42326[30], 6}, {NULL}};
const Segment SGI145433[] = {{&S42327[44], 10}, {NULL}};
const Segment SGI145434[] = {{&S42327[44], 22}, {NULL}};
const Segment SGI145435[] = {{&S42327[30], 6}, {NULL}};
const Segment SGI145436[] = {{&S42327[31], 3}, {NULL}};
const Segment SGI145437[] = {{&S42327[80], 6}, {NULL}};
const Segment SGI145438[] = {{&S42328[44], 10}, {NULL}};
const Segment SGI145439[] = {{&S42328[44], 22}, {NULL}};
const Segment SGI145440[] = {{&S42328[30], 6}, {NULL}};
const Segment SGI145441[] = {{&S42328[31], 3}, {NULL}};
const Segment SGI145442[] = {{&S42328[80], 6}, {NULL}};
const Segment SGI145443[] = {{&S42329[44], 10}, {NULL}};
const Segment SGI145444[] = {{&S42329[44], 22}, {NULL}};
const Segment SGI145445[] = {{&S42329[30], 6}, {NULL}};
const Segment SGI145446[] = {{&S27803[34], 10}, {NULL}};
const Segment SGI145447[] = {{&S27803[27], 6}, {NULL}};
const Segment SGI145448[] = {{&S27803[97], 9}, {NULL}};
const Segment SGI145449[] = {{&S27803[70], 6}, {NULL}};
const Segment SGI145450[] = {{&S42330[44], 10}, {NULL}};
const Segment SGI145451[] = {{&S42330[44], 28}, {NULL}};
const Segment SGI145452[] = {{&S42330[30], 6}, {NULL}};
const Segment SGI145453[] = {{&S42331[44], 10}, {NULL}};
const Segment SGI145454[] = {{&S42331[44], 22}, {NULL}};
const Segment SGI145455[] = {{&S42331[30], 6}, {NULL}};
const Segment SGI145456[] = {{&S42331[31], 3}, {NULL}};
const Segment SGI145457[] = {{&S42331[80], 6}, {NULL}};
const Segment SGI145458[] = {{&S42332[46], 10}, {NULL}};
const Segment SGI145459[] = {{&S42332[46], 28}, {NULL}};
const Segment SGI145460[] = {{&S42332[32], 6}, {NULL}};
const Segment SGI145461[] = {{&S42333[45], 10}, {NULL}};
const Segment SGI145462[] = {{&S42333[45], 28}, {NULL}};
const Segment SGI145463[] = {{&S42333[31], 6}, {NULL}};
const Segment SGI145464[] = {{&S42334[44], 11}, {NULL}};
const Segment SGI145465[] = {{&S42334[44], 23}, {NULL}};
const Segment SGI145466[] = {{&S42334[30], 6}, {NULL}};
const Segment SGI145467[] = {{&S42334[31], 3}, {NULL}};
const Segment SGI145468[] = {{&S42334[81], 5}, {NULL}};
const Segment SGI145469[] = {{&S42335[44], 10}, {NULL}};
const Segment SGI145470[] = {{&S42335[44], 19}, {NULL}};
const Segment SGI145471[] = {{&S42335[30], 6}, {NULL}};
const Segment SGI145472[] = {{&S42335[31], 3}, {NULL}};
const Segment SGI145473[] = {{&S42335[77], 5}, {NULL}};
const Segment SGI145474[] = {{&S42336[42], 11}, {NULL}};
const Segment SGI145475[] = {{&S42336[30], 6}, {NULL}};
const Segment SGI145476[] = {{&S42336[31], 3}, {NULL}};
const Segment SGI145477[] = {{&S42336[79], 6}, {NULL}};
const Segment SGI145478[] = {{&S42337[44], 10}, {NULL}};
const Segment SGI145479[] = {{&S42337[44], 22}, {NULL}};
const Segment SGI145480[] = {{&S42337[30], 6}, {NULL}};
const Segment SGI145481[] = {{&S42337[31], 3}, {NULL}};
const Segment SGI145482[] = {{&S42337[80], 6}, {NULL}};
const Segment SGI145483[] = {{&S42340[34], 10}, {NULL}};
const Segment SGI145484[] = {{&S42340[27], 6}, {NULL}};
const Segment SGI145485[] = {{&S42340[97], 9}, {NULL}};
const Segment SGI145486[] = {{&S42340[70], 6}, {NULL}};
const Segment SGI145487[] = {{&S42343[34], 11}, {NULL}};
const Segment SGI145488[] = {{&S42343[27], 6}, {NULL}};
const Segment SGI145489[] = {{&S42343[98], 9}, {NULL}};
const Segment SGI145490[] = {{&S42343[71], 6}, {NULL}};
const Segment SGI145491[] = {{&S42344[34], 11}, {NULL}};
const Segment SGI145492[] = {{&S42344[27], 6}, {NULL}};
const Segment SGI145493[] = {{&S42344[98], 9}, {NULL}};
const Segment SGI145494[] = {{&S42344[71], 6}, {NULL}};
const Segment SGI145495[] = {{&S37887[44], 24}, {NULL}};
const Segment SGI145496[] = {{&S37887[44], 36}, {NULL}};
const Segment SGI145497[] = {{&S37887[30], 6}, {NULL}};
const Segment SGI145498[] = {{&S37887[31], 3}, {NULL}};
const Segment SGI145499[] = {{&S37887[94], 6}, {NULL}};
const Segment SGI145500[] = {{&S37904[44], 25}, {NULL}};
const Segment SGI145501[] = {{&S37904[44], 37}, {NULL}};
const Segment SGI145502[] = {{&S37904[30], 6}, {NULL}};
const Segment SGI145503[] = {{&S37904[31], 3}, {NULL}};
const Segment SGI145504[] = {{&S37904[95], 6}, {NULL}};
const Segment SGI145505[] = {{&S32847[44], 10}, {NULL}};
const Segment SGI145506[] = {{&S32847[44], 22}, {NULL}};
const Segment SGI145507[] = {{&S32847[30], 6}, {NULL}};
const Segment SGI145508[] = {{&S32847[115], 3}, {NULL}};
const Segment SGI145509[] = {{&S32847[80], 6}, {NULL}};
const Segment SGI145510[] = {{&S32849[38], 10}, {NULL}};
const Segment SGI145511[] = {{&S32849[31], 6}, {NULL}};
const Segment SGI145512[] = {{&S32850[44], 14}, {NULL}};
const Segment SGI145513[] = {{&S32850[44], 26}, {NULL}};
const Segment SGI145514[] = {{&S32850[30], 6}, {NULL}};
const Segment SGI145515[] = {{&S32850[119], 3}, {NULL}};
const Segment SGI145516[] = {{&S32850[84], 6}, {NULL}};
const Segment SGI145517[] = {{&S32851[44], 13}, {NULL}};
const Segment SGI145518[] = {{&S32851[44], 25}, {NULL}};
const Segment SGI145519[] = {{&S32851[30], 6}, {NULL}};
const Segment SGI145520[] = {{&S32851[118], 3}, {NULL}};
const Segment SGI145521[] = {{&S32851[83], 6}, {NULL}};
const Segment SGI145522[] = {{&S32852[38], 14}, {NULL}};
const Segment SGI145523[] = {{&S32852[31], 6}, {NULL}};
const Segment SGI145524[] = {{&S32853[38], 13}, {NULL}};
const Segment SGI145525[] = {{&S32853[31], 6}, {NULL}};
const Segment SGI145526[] = {{&S32854[42], 10}, {NULL}};
const Segment SGI145527[] = {{&S32854[30], 6}, {NULL}};
const Segment SGI145528[] = {{&S32854[113], 3}, {NULL}};
const Segment SGI145529[] = {{&S32854[78], 6}, {NULL}};
const Segment SGI145530[] = {{&S32855[44], 10}, {NULL}};
const Segment SGI145531[] = {{&S32855[44], 22}, {NULL}};
const Segment SGI145532[] = {{&S32855[30], 6}, {NULL}};
const Segment SGI145533[] = {{&S32855[115], 3}, {NULL}};
const Segment SGI145534[] = {{&S32855[80], 6}, {NULL}};
const Segment SGI145535[] = {{&S32876[44], 7}, {NULL}};
const Segment SGI145536[] = {{&S32876[44], 24}, {NULL}};
const Segment SGI145537[] = {{&S32876[30], 6}, {NULL}};
const Segment SGI145538[] = {{&S32876[117], 3}, {NULL}};
const Segment SGI145539[] = {{&S32876[82], 6}, {NULL}};
const Segment SGI145540[] = {{&S32877[44], 7}, {NULL}};
const Segment SGI145541[] = {{&S32877[44], 24}, {NULL}};
const Segment SGI145542[] = {{&S32877[30], 6}, {NULL}};
const Segment SGI145543[] = {{&S32877[117], 3}, {NULL}};
const Segment SGI145544[] = {{&S32877[82], 6}, {NULL}};
const Segment SGI145545[] = {{&S32878[44], 7}, {NULL}};
const Segment SGI145546[] = {{&S32878[44], 24}, {NULL}};
const Segment SGI145547[] = {{&S32878[30], 6}, {NULL}};
const Segment SGI145548[] = {{&S32878[117], 3}, {NULL}};
const Segment SGI145549[] = {{&S32878[82], 6}, {NULL}};
const Segment SGI145550[] = {{&S32879[44], 11}, {NULL}};
const Segment SGI145551[] = {{&S32879[44], 28}, {NULL}};
const Segment SGI145552[] = {{&S32879[30], 6}, {NULL}};
const Segment SGI145553[] = {{&S32879[121], 3}, {NULL}};
const Segment SGI145554[] = {{&S32879[86], 6}, {NULL}};
const Segment SGI145555[] = {{&S32880[38], 7}, {NULL}};
const Segment SGI145556[] = {{&S32880[31], 6}, {NULL}};
const Segment SGI145557[] = {{&S32881[44], 7}, {NULL}};
const Segment SGI145558[] = {{&S32881[44], 24}, {NULL}};
const Segment SGI145559[] = {{&S32881[30], 6}, {NULL}};
const Segment SGI145560[] = {{&S32881[117], 3}, {NULL}};
const Segment SGI145561[] = {{&S32881[82], 6}, {NULL}};
const Segment SGI145562[] = {{&S42430[44], 10}, {NULL}};
const Segment SGI145563[] = {{&S42430[44], 22}, {NULL}};
const Segment SGI145564[] = {{&S42430[30], 6}, {NULL}};
const Segment SGI145565[] = {{&S32882[44], 13}, {NULL}};
const Segment SGI145566[] = {{&S32882[44], 30}, {NULL}};
const Segment SGI145567[] = {{&S42430[31], 3}, {NULL}};
const Segment SGI145568[] = {{&S32882[30], 6}, {NULL}};
const Segment SGI145569[] = {{&S42430[95], 6}, {NULL}};
const Segment SGI145570[] = {{&S32882[123], 3}, {NULL}};
const Segment SGI145571[] = {{&S42430[68], 8}, {NULL}};
const Segment SGI145572[] = {{&S32882[88], 6}, {NULL}};
const Segment SGI145573[] = {{&S42431[44], 10}, {NULL}};
const Segment SGI145574[] = {{&S42431[44], 22}, {NULL}};
const Segment SGI145575[] = {{&S42431[30], 6}, {NULL}};
const Segment SGI145576[] = {{&S42431[31], 3}, {NULL}};
const Segment SGI145577[] = {{&S42431[95], 6}, {NULL}};
const Segment SGI145578[] = {{&S42431[68], 8}, {NULL}};
const Segment SGI145579[] = {{&S42432[44], 10}, {NULL}};
const Segment SGI145580[] = {{&S42432[44], 22}, {NULL}};
const Segment SGI145581[] = {{&S42432[30], 6}, {NULL}};
const Segment SGI145582[] = {{&S42432[31], 3}, {NULL}};
const Segment SGI145583[] = {{&S42432[99], 6}, {NULL}};
const Segment SGI145584[] = {{&S42432[68], 8}, {NULL}};
const Segment SGI145585[] = {{&S42433[44], 10}, {NULL}};
const Segment SGI145586[] = {{&S42433[44], 27}, {NULL}};
const Segment SGI145587[] = {{&S42433[30], 6}, {NULL}};
const Segment SGI145588[] = {{&S42433[31], 3}, {NULL}};
const Segment SGI145589[] = {{&S42433[105], 6}, {NULL}};
const Segment SGI145590[] = {{&S42433[73], 8}, {NULL}};
const Segment SGI145591[] = {{&S32930[34], 7}, {NULL}};
const Segment SGI145592[] = {{&S32930[27], 6}, {NULL}};
const Segment SGI145593[] = {{&S32930[103], 9}, {NULL}};
const Segment SGI145594[] = {{&S32930[76], 6}, {NULL}};
const Segment SGI145595[] = {{&S32933[34], 7}, {NULL}};
const Segment SGI145596[] = {{&S32933[27], 6}, {NULL}};
const Segment SGI145597[] = {{&S32933[103], 9}, {NULL}};
const Segment SGI145598[] = {{&S32933[76], 6}, {NULL}};
const Segment SGI145599[] = {{&S32934[34], 7}, {NULL}};
const Segment SGI145600[] = {{&S32934[27], 6}, {NULL}};
const Segment SGI145601[] = {{&S32934[97], 9}, {NULL}};
const Segment SGI145602[] = {{&S32934[70], 6}, {NULL}};
const Segment SGI145603[] = {{&S32935[34], 7}, {NULL}};
const Segment SGI145604[] = {{&S32935[27], 6}, {NULL}};
const Segment SGI145605[] = {{&S32935[97], 9}, {NULL}};
const Segment SGI145606[] = {{&S32935[70], 6}, {NULL}};
const Segment SGI145607[] = {{&S42490[44], 11}, {NULL}};
const Segment SGI145608[] = {{&S42490[44], 23}, {NULL}};
const Segment SGI145609[] = {{&S42490[30], 6}, {NULL}};
const Segment SGI145610[] = {{&S42490[31], 3}, {NULL}};
const Segment SGI145611[] = {{&S42490[81], 6}, {NULL}};
const Segment SGI145612[] = {{&S42491[38], 11}, {NULL}};
const Segment SGI145613[] = {{&S42491[31], 6}, {NULL}};
const Segment SGI145614[] = {{&S42492[42], 11}, {NULL}};
const Segment SGI145615[] = {{&S42492[30], 6}, {NULL}};
const Segment SGI145616[] = {{&S42492[31], 3}, {NULL}};
const Segment SGI145617[] = {{&S42492[79], 6}, {NULL}};
const Segment SGI145618[] = {{&S30641[42], 25}, {NULL}};
const Segment SGI145619[] = {{&S30641[42], 37}, {NULL}};
const Segment SGI145620[] = {{&S30641[30], 4}, {NULL}};
const Segment SGI145621[] = {{&S30641[128], 3}, {NULL}};
const Segment SGI145622[] = {{&S30641[93], 6}, {NULL}};
const Segment SGI145623[] = {{&S30658[42], 14}, {NULL}};
const Segment SGI145624[] = {{&S30658[42], 26}, {NULL}};
const Segment SGI145625[] = {{&S30658[30], 4}, {NULL}};
const Segment SGI145626[] = {{&S30658[117], 3}, {NULL}};
const Segment SGI145627[] = {{&S30658[82], 6}, {NULL}};
const Segment SGI145628[] = {{&S30712[42], 14}, {NULL}};
const Segment SGI145629[] = {{&S30712[42], 26}, {NULL}};
const Segment SGI145630[] = {{&S30712[30], 4}, {NULL}};
const Segment SGI145631[] = {{&S30712[117], 3}, {NULL}};
const Segment SGI145632[] = {{&S30712[82], 6}, {NULL}};
const Segment SGI145633[] = {{&S30785[42], 24}, {NULL}};
const Segment SGI145634[] = {{&S30785[42], 36}, {NULL}};
const Segment SGI145635[] = {{&S30785[30], 4}, {NULL}};
const Segment SGI145636[] = {{&S30785[127], 3}, {NULL}};
const Segment SGI145637[] = {{&S30785[92], 6}, {NULL}};
const Segment SGI145638[] = {{&S42805[44], 10}, {NULL}};
const Segment SGI145639[] = {{&S42805[44], 22}, {NULL}};
const Segment SGI145640[] = {{&S42805[30], 6}, {NULL}};
const Segment SGI145641[] = {{&S42805[31], 3}, {NULL}};
const Segment SGI145642[] = {{&S42805[80], 6}, {NULL}};
const Segment SGI145643[] = {{&S42807[38], 10}, {NULL}};
const Segment SGI145644[] = {{&S42807[31], 6}, {NULL}};
const Segment SGI145645[] = {{&S42812[44], 10}, {NULL}};
const Segment SGI145646[] = {{&S42812[44], 22}, {NULL}};
const Segment SGI145647[] = {{&S42812[30], 6}, {NULL}};
const Segment SGI145648[] = {{&S42812[116], 3}, {NULL}};
const Segment SGI145649[] = {{&S42812[80], 7}, {NULL}};
const Segment SGI145650[] = {{&S42812[136], 8}, {NULL}};
const Segment SGI145651[] = {{&S42813[34], 10}, {NULL}};
const Segment SGI145652[] = {{&S42813[34], 22}, {NULL}};
const Segment SGI145653[] = {{&S42814[44], 10}, {NULL}};
const Segment SGI145654[] = {{&S42814[44], 28}, {NULL}};
const Segment SGI145655[] = {{&S42814[30], 6}, {NULL}};
const Segment SGI145656[] = {{&S42815[44], 10}, {NULL}};
const Segment SGI145657[] = {{&S42815[44], 28}, {NULL}};
const Segment SGI145658[] = {{&S42815[30], 6}, {NULL}};
const Segment SGI145659[] = {{&S42816[44], 10}, {NULL}};
const Segment SGI145660[] = {{&S42816[44], 22}, {NULL}};
const Segment SGI145661[] = {{&S42816[30], 6}, {NULL}};
const Segment SGI145662[] = {{&S42816[80], 6}, {NULL}};
const Segment SGI145663[] = {{&S42818[44], 10}, {NULL}};
const Segment SGI145664[] = {{&S42818[44], 28}, {NULL}};
const Segment SGI145665[] = {{&S42818[30], 6}, {NULL}};
const Segment SGI145666[] = {{&S42819[46], 9}, {NULL}};
const Segment SGI145667[] = {{&S42819[32], 6}, {NULL}};
const Segment SGI145668[] = {{&S42820[44], 10}, {NULL}};
const Segment SGI145669[] = {{&S42820[44], 17}, {NULL}};
const Segment SGI145670[] = {{&S42820[30], 6}, {NULL}};
const Segment SGI145671[] = {{&S42820[31], 3}, {NULL}};
const Segment SGI145672[] = {{&S42820[75], 6}, {NULL}};
const Segment SGI145673[] = {{&S42821[44], 10}, {NULL}};
const Segment SGI145674[] = {{&S42821[44], 22}, {NULL}};
const Segment SGI145675[] = {{&S42821[30], 6}, {NULL}};
const Segment SGI145676[] = {{&S42821[83], 6}, {NULL}};
const Segment SGI145677[] = {{&S42822[30], 1}, {NULL}};
const Segment SGI145678[] = {{&S42822[63], 9}, {NULL}};
const Segment SGI145679[] = {{&S42822[49], 6}, {NULL}};
const Segment SGI145680[] = {{&S42823[64], 10}, {NULL}};
const Segment SGI145681[] = {{&S42823[64], 22}, {NULL}};
const Segment SGI145682[] = {{&S42823[50], 6}, {NULL}};
const Segment SGI145683[] = {{&S42824[42], 10}, {NULL}};
const Segment SGI145684[] = {{&S42824[30], 6}, {NULL}};
const Segment SGI145685[] = {{&S42824[31], 3}, {NULL}};
const Segment SGI145686[] = {{&S42824[78], 6}, {NULL}};
const Segment SGI145687[] = {{&S42832[34], 7}, {NULL}};
const Segment SGI145688[] = {{&S42832[27], 6}, {NULL}};
const Segment SGI145689[] = {{&S42832[99], 9}, {NULL}};
const Segment SGI145690[] = {{&S42832[72], 6}, {NULL}};
const Segment SGI145691[] = {{&S33369[34], 15}, {NULL}};
const Segment SGI145692[] = {{&S33369[27], 6}, {NULL}};
const Segment SGI145693[] = {{&S33369[105], 9}, {NULL}};
const Segment SGI145694[] = {{&S33369[78], 6}, {NULL}};
const Segment SGI145695[] = {{&S33370[34], 7}, {NULL}};
const Segment SGI145696[] = {{&S33370[27], 6}, {NULL}};
const Segment SGI145697[] = {{&S33370[111], 9}, {NULL}};
const Segment SGI145698[] = {{&S33370[84], 6}, {NULL}};
const Segment SGI145699[] = {{&S31006[32], 9}, {NULL}};
const Segment SGI145700[] = {{&S31006[27], 4}, {NULL}};
const Segment SGI145701[] = {{&S31006[94], 9}, {NULL}};
const Segment SGI145702[] = {{&S31006[67], 6}, {NULL}};
const Segment SGI145703[] = {{&S42883[44], 14}, {NULL}};
const Segment SGI145704[] = {{&S42883[44], 26}, {NULL}};
const Segment SGI145705[] = {{&S42883[30], 6}, {NULL}};
const Segment SGI145706[] = {{&S42883[31], 3}, {NULL}};
const Segment SGI145707[] = {{&S42883[84], 6}, {NULL}};
const Segment SGI145708[] = {{&S42884[38], 14}, {NULL}};
const Segment SGI145709[] = {{&S42884[31], 6}, {NULL}};
const Segment SGI145710[] = {{&S42887[34], 13}, {NULL}};
const Segment SGI145711[] = {{&S42887[27], 6}, {NULL}};
const Segment SGI145712[] = {{&S33395[44], 11}, {NULL}};
const Segment SGI145713[] = {{&S33395[44], 29}, {NULL}};
const Segment SGI145714[] = {{&S33395[30], 6}, {NULL}};
const Segment SGI145715[] = {{&S42887[100], 9}, {NULL}};
const Segment SGI145716[] = {{&S42887[73], 6}, {NULL}};
const Segment SGI145717[] = {{&S33395[122], 3}, {NULL}};
const Segment SGI145718[] = {{&S33395[87], 6}, {NULL}};
const Segment SGI145719[] = {{&S42888[34], 10}, {NULL}};
const Segment SGI145720[] = {{&S42888[27], 6}, {NULL}};
const Segment SGI145721[] = {{&S33396[44], 7}, {NULL}};
const Segment SGI145722[] = {{&S33396[44], 25}, {NULL}};
const Segment SGI145723[] = {{&S33396[30], 6}, {NULL}};
const Segment SGI145724[] = {{&S42888[97], 9}, {NULL}};
const Segment SGI145725[] = {{&S42888[70], 6}, {NULL}};
const Segment SGI145726[] = {{&S33396[118], 3}, {NULL}};
const Segment SGI145727[] = {{&S33396[83], 6}, {NULL}};
const Segment SGI145728[] = {{&S33397[38], 7}, {NULL}};
const Segment SGI145729[] = {{&S33397[31], 6}, {NULL}};
const Segment SGI145730[] = {{&S33398[44], 7}, {NULL}};
const Segment SGI145731[] = {{&S33398[44], 25}, {NULL}};
const Segment SGI145732[] = {{&S33398[30], 6}, {NULL}};
const Segment SGI145733[] = {{&S33398[118], 3}, {NULL}};
const Segment SGI145734[] = {{&S33398[83], 6}, {NULL}};
const Segment SGI145735[] = {{&S33399[44], 11}, {NULL}};
const Segment SGI145736[] = {{&S33399[44], 29}, {NULL}};
const Segment SGI145737[] = {{&S33399[30], 6}, {NULL}};
const Segment SGI145738[] = {{&S33399[122], 3}, {NULL}};
const Segment SGI145739[] = {{&S33399[87], 6}, {NULL}};
const Segment SGI145740[] = {{&S33400[44], 11}, {NULL}};
const Segment SGI145741[] = {{&S33400[44], 31}, {NULL}};
const Segment SGI145742[] = {{&S33400[30], 6}, {NULL}};
const Segment SGI145743[] = {{&S33400[124], 3}, {NULL}};
const Segment SGI145744[] = {{&S33400[89], 6}, {NULL}};
const Segment SGI145745[] = {{&S33401[44], 7}, {NULL}};
const Segment SGI145746[] = {{&S33401[44], 27}, {NULL}};
const Segment SGI145747[] = {{&S33401[30], 6}, {NULL}};
const Segment SGI145748[] = {{&S33401[120], 3}, {NULL}};
const Segment SGI145749[] = {{&S33401[85], 6}, {NULL}};
const Segment SGI145750[] = {{&S33402[44], 11}, {NULL}};
const Segment SGI145751[] = {{&S33402[44], 31}, {NULL}};
const Segment SGI145752[] = {{&S33402[30], 6}, {NULL}};
const Segment SGI145753[] = {{&S33402[124], 3}, {NULL}};
const Segment SGI145754[] = {{&S33402[89], 6}, {NULL}};
const Segment SGI145755[] = {{&S33404[44], 7}, {NULL}};
const Segment SGI145756[] = {{&S33404[44], 27}, {NULL}};
const Segment SGI145757[] = {{&S33404[30], 6}, {NULL}};
const Segment SGI145758[] = {{&S33404[120], 3}, {NULL}};
const Segment SGI145759[] = {{&S33404[85], 6}, {NULL}};
const Segment SGI145760[] = {{&S42905[44], 14}, {NULL}};
const Segment SGI145761[] = {{&S42905[44], 26}, {NULL}};
const Segment SGI145762[] = {{&S42905[30], 6}, {NULL}};
const Segment SGI145763[] = {{&S42905[31], 3}, {NULL}};
const Segment SGI145764[] = {{&S42905[84], 6}, {NULL}};
const Segment SGI145765[] = {{&S42906[44], 14}, {NULL}};
const Segment SGI145766[] = {{&S42906[44], 26}, {NULL}};
const Segment SGI145767[] = {{&S42906[30], 6}, {NULL}};
const Segment SGI145768[] = {{&S33406[44], 7}, {NULL}};
const Segment SGI145769[] = {{&S42906[31], 3}, {NULL}};
const Segment SGI145770[] = {{&S33406[44], 19}, {NULL}};
const Segment SGI145771[] = {{&S33406[30], 6}, {NULL}};
const Segment SGI145772[] = {{&S42906[84], 6}, {NULL}};
const Segment SGI145773[] = {{&S33406[112], 3}, {NULL}};
const Segment SGI145774[] = {{&S33406[77], 6}, {NULL}};
const Segment SGI145775[] = {{&S42907[38], 14}, {NULL}};
const Segment SGI145776[] = {{&S42907[31], 6}, {NULL}};
const Segment SGI145777[] = {{&S42908[44], 15}, {NULL}};
const Segment SGI145778[] = {{&S42908[44], 27}, {NULL}};
const Segment SGI145779[] = {{&S42908[30], 6}, {NULL}};
const Segment SGI145780[] = {{&S42908[31], 3}, {NULL}};
const Segment SGI145781[] = {{&S42908[85], 6}, {NULL}};
const Segment SGI145782[] = {{&S42909[44], 15}, {NULL}};
const Segment SGI145783[] = {{&S42909[44], 27}, {NULL}};
const Segment SGI145784[] = {{&S42909[30], 6}, {NULL}};
const Segment SGI145785[] = {{&S42909[31], 3}, {NULL}};
const Segment SGI145786[] = {{&S42909[85], 6}, {NULL}};
const Segment SGI145787[] = {{&S42910[44], 15}, {NULL}};
const Segment SGI145788[] = {{&S42910[44], 27}, {NULL}};
const Segment SGI145789[] = {{&S42910[30], 6}, {NULL}};
const Segment SGI145790[] = {{&S42910[31], 3}, {NULL}};
const Segment SGI145791[] = {{&S42910[85], 6}, {NULL}};
const Segment SGI145792[] = {{&S42911[44], 13}, {NULL}};
const Segment SGI145793[] = {{&S42911[44], 25}, {NULL}};
const Segment SGI145794[] = {{&S42911[30], 6}, {NULL}};
const Segment SGI145795[] = {{&S42911[31], 3}, {NULL}};
const Segment SGI145796[] = {{&S42911[83], 6}, {NULL}};
const Segment SGI145797[] = {{&S42912[44], 16}, {NULL}};
const Segment SGI145798[] = {{&S42912[44], 28}, {NULL}};
const Segment SGI145799[] = {{&S42912[30], 6}, {NULL}};
const Segment SGI145800[] = {{&S42912[31], 3}, {NULL}};
const Segment SGI145801[] = {{&S42912[86], 6}, {NULL}};
const Segment SGI145802[] = {{&S33417[34], 15}, {NULL}};
const Segment SGI145803[] = {{&S33417[27], 6}, {NULL}};
const Segment SGI145804[] = {{&S42913[44], 17}, {NULL}};
const Segment SGI145805[] = {{&S42913[44], 29}, {NULL}};
const Segment SGI145806[] = {{&S33417[114], 9}, {NULL}};
const Segment SGI145807[] = {{&S42913[30], 6}, {NULL}};
const Segment SGI145808[] = {{&S33417[87], 6}, {NULL}};
const Segment SGI145809[] = {{&S42913[31], 3}, {NULL}};
const Segment SGI145810[] = {{&S42913[87], 6}, {NULL}};
const Segment SGI145811[] = {{&S42914[44], 16}, {NULL}};
const Segment SGI145812[] = {{&S42914[44], 28}, {NULL}};
const Segment SGI145813[] = {{&S42914[30], 6}, {NULL}};
const Segment SGI145814[] = {{&S33419[34], 7}, {NULL}};
const Segment SGI145815[] = {{&S33419[27], 6}, {NULL}};
const Segment SGI145816[] = {{&S42914[31], 3}, {NULL}};
const Segment SGI145817[] = {{&S42914[86], 6}, {NULL}};
const Segment SGI145818[] = {{&S33419[111], 9}, {NULL}};
const Segment SGI145819[] = {{&S33419[84], 6}, {NULL}};
const Segment SGI145820[] = {{&S42915[38], 15}, {NULL}};
const Segment SGI145821[] = {{&S42915[31], 6}, {NULL}};
const Segment SGI145822[] = {{&S33420[34], 9}, {NULL}};
const Segment SGI145823[] = {{&S33420[27], 6}, {NULL}};
const Segment SGI145824[] = {{&S33420[110], 9}, {NULL}};
const Segment SGI145825[] = {{&S33420[83], 6}, {NULL}};
const Segment SGI145826[] = {{&S42916[44], 15}, {NULL}};
const Segment SGI145827[] = {{&S42916[44], 27}, {NULL}};
const Segment SGI145828[] = {{&S42916[30], 6}, {NULL}};
const Segment SGI145829[] = {{&S42916[31], 3}, {NULL}};
const Segment SGI145830[] = {{&S42916[85], 6}, {NULL}};
const Segment SGI145831[] = {{&S33421[34], 7}, {NULL}};
const Segment SGI145832[] = {{&S33421[27], 6}, {NULL}};
const Segment SGI145833[] = {{&S33421[111], 9}, {NULL}};
const Segment SGI145834[] = {{&S33421[84], 6}, {NULL}};
const Segment SGI145835[] = {{&S42917[44], 16}, {NULL}};
const Segment SGI145836[] = {{&S42917[44], 28}, {NULL}};
const Segment SGI145837[] = {{&S42917[30], 6}, {NULL}};
const Segment SGI145838[] = {{&S42917[31], 3}, {NULL}};
const Segment SGI145839[] = {{&S42917[86], 6}, {NULL}};
const Segment SGI145840[] = {{&S42918[38], 15}, {NULL}};
const Segment SGI145841[] = {{&S42918[31], 6}, {NULL}};
const Segment SGI145842[] = {{&S33424[34], 15}, {NULL}};
const Segment SGI145843[] = {{&S33424[27], 6}, {NULL}};
const Segment SGI145844[] = {{&S42932[34], 21}, {NULL}};
const Segment SGI145845[] = {{&S42932[27], 6}, {NULL}};
const Segment SGI145846[] = {{&S42932[108], 9}, {NULL}};
const Segment SGI145847[] = {{&S42932[81], 6}, {NULL}};
const Segment SGI145848[] = {{&S42933[34], 18}, {NULL}};
const Segment SGI145849[] = {{&S42933[27], 6}, {NULL}};
const Segment SGI145850[] = {{&S42933[105], 9}, {NULL}};
const Segment SGI145851[] = {{&S42933[78], 6}, {NULL}};
const Segment SGI145852[] = {{&S42934[34], 14}, {NULL}};
const Segment SGI145853[] = {{&S42934[27], 6}, {NULL}};
const Segment SGI145854[] = {{&S42934[101], 9}, {NULL}};
const Segment SGI145855[] = {{&S42934[74], 6}, {NULL}};
const Segment SGI145856[] = {{&S42953[34], 11}, {NULL}};
const Segment SGI145857[] = {{&S42953[27], 6}, {NULL}};
const Segment SGI145858[] = {{&S42953[98], 9}, {NULL}};
const Segment SGI145859[] = {{&S42953[71], 6}, {NULL}};
const Segment SGI145860[] = {{&S42954[34], 11}, {NULL}};
const Segment SGI145861[] = {{&S42954[27], 6}, {NULL}};
const Segment SGI145862[] = {{&S42954[98], 9}, {NULL}};
const Segment SGI145863[] = {{&S42954[71], 6}, {NULL}};
const Segment SGI145864[] = {{&S42956[34], 10}, {NULL}};
const Segment SGI145865[] = {{&S42956[27], 6}, {NULL}};
const Segment SGI145866[] = {{&S42956[97], 9}, {NULL}};
const Segment SGI145867[] = {{&S42956[70], 6}, {NULL}};
const Segment SGI145868[] = {{&S42959[34], 10}, {NULL}};
const Segment SGI145869[] = {{&S42959[27], 6}, {NULL}};
const Segment SGI145870[] = {{&S42959[97], 9}, {NULL}};
const Segment SGI145871[] = {{&S42959[70], 6}, {NULL}};
const Segment SGI145872[] = {{&S33424[105], 9}, {NULL}};
const Segment SGI145873[] = {{&S33424[78], 6}, {NULL}};
const Segment SGI145874[] = {{&S37324[42], 12}, {NULL}};
const Segment SGI145875[] = {{&S37324[42], 24}, {NULL}};
const Segment SGI145876[] = {{&S37324[30], 4}, {NULL}};
const Segment SGI145877[] = {{&S37324[115], 3}, {NULL}};
const Segment SGI145878[] = {{&S37324[80], 6}, {NULL}};
const Segment SGI145879[] = {{&S18671[44], 14}, {NULL}};
const Segment SGI145880[] = {{&S18671[44], 26}, {NULL}};
const Segment SGI145881[] = {{&S18671[30], 6}, {NULL}};
const Segment SGI145882[] = {{&S18671[119], 3}, {NULL}};
const Segment SGI145883[] = {{&S18671[84], 6}, {NULL}};
const Segment SGI145884[] = {{&S18672[44], 14}, {NULL}};
const Segment SGI145885[] = {{&S18672[44], 25}, {NULL}};
const Segment SGI145886[] = {{&S18672[30], 6}, {NULL}};
const Segment SGI145887[] = {{&S18672[118], 3}, {NULL}};
const Segment SGI145888[] = {{&S18672[83], 6}, {NULL}};
const Segment SGI145889[] = {{&S18675[44], 14}, {NULL}};
const Segment SGI145890[] = {{&S18675[44], 25}, {NULL}};
const Segment SGI145891[] = {{&S18675[30], 6}, {NULL}};
const Segment SGI145892[] = {{&S18675[118], 3}, {NULL}};
const Segment SGI145893[] = {{&S18675[83], 6}, {NULL}};
const Segment SGI145894[] = {{&S18676[38], 14}, {NULL}};
const Segment SGI145895[] = {{&S18676[31], 6}, {NULL}};
const Segment SGI145896[] = {{&S18677[44], 14}, {NULL}};
const Segment SGI145897[] = {{&S18677[44], 26}, {NULL}};
const Segment SGI145898[] = {{&S18677[30], 6}, {NULL}};
const Segment SGI145899[] = {{&S18677[119], 3}, {NULL}};
const Segment SGI145900[] = {{&S18677[84], 6}, {NULL}};
const Segment SGI145901[] = {{&S18678[44], 14}, {NULL}};
const Segment SGI145902[] = {{&S18678[44], 25}, {NULL}};
const Segment SGI145903[] = {{&S18678[30], 6}, {NULL}};
const Segment SGI145904[] = {{&S18678[86], 6}, {NULL}};
const Segment SGI145905[] = {{&S18681[42], 14}, {NULL}};
const Segment SGI145906[] = {{&S18681[30], 6}, {NULL}};
const Segment SGI145907[] = {{&S18681[116], 3}, {NULL}};
const Segment SGI145908[] = {{&S18681[81], 6}, {NULL}};
const Segment SGI145909[] = {{&S18682[44], 14}, {NULL}};
const Segment SGI145910[] = {{&S18682[44], 26}, {NULL}};
const Segment SGI145911[] = {{&S18682[30], 6}, {NULL}};
const Segment SGI145912[] = {{&S18682[119], 3}, {NULL}};
const Segment SGI145913[] = {{&S18682[84], 6}, {NULL}};
const Segment SGI145914[] = {{&S43221[44], 9}, {NULL}};
const Segment SGI145915[] = {{&S43221[44], 20}, {NULL}};
const Segment SGI145916[] = {{&S43221[30], 6}, {NULL}};
const Segment SGI145917[] = {{&S43221[31], 3}, {NULL}};
const Segment SGI145918[] = {{&S43221[78], 6}, {NULL}};
const Segment SGI145919[] = {{&S43222[44], 9}, {NULL}};
const Segment SGI145920[] = {{&S43222[44], 21}, {NULL}};
const Segment SGI145921[] = {{&S43222[30], 6}, {NULL}};
const Segment SGI145922[] = {{&S43222[31], 3}, {NULL}};
const Segment SGI145923[] = {{&S43222[79], 6}, {NULL}};
const Segment SGI145924[] = {{&S43223[44], 9}, {NULL}};
const Segment SGI145925[] = {{&S43223[44], 21}, {NULL}};
const Segment SGI145926[] = {{&S43223[30], 6}, {NULL}};
const Segment SGI145927[] = {{&S43223[79], 6}, {NULL}};
const Segment SGI145928[] = {{&S43224[44], 9}, {NULL}};
const Segment SGI145929[] = {{&S43224[44], 21}, {NULL}};
const Segment SGI145930[] = {{&S43224[30], 6}, {NULL}};
const Segment SGI145931[] = {{&S43224[79], 6}, {NULL}};
const Segment SGI145932[] = {{&S43225[44], 9}, {NULL}};
const Segment SGI145933[] = {{&S43225[44], 21}, {NULL}};
const Segment SGI145934[] = {{&S43225[30], 6}, {NULL}};
const Segment SGI145935[] = {{&S43225[31], 3}, {NULL}};
const Segment SGI145936[] = {{&S43225[79], 6}, {NULL}};
const Segment SGI145937[] = {{&S43226[44], 9}, {NULL}};
const Segment SGI145938[] = {{&S43226[44], 21}, {NULL}};
const Segment SGI145939[] = {{&S43226[30], 6}, {NULL}};
const Segment SGI145940[] = {{&S43226[82], 4}, {NULL}};
const Segment SGI145941[] = {{&S43226[82], 3}, {NULL}};
const Segment SGI145942[] = {{&S43227[38], 9}, {NULL}};
const Segment SGI145943[] = {{&S43227[31], 6}, {NULL}};
const Segment SGI145944[] = {{&S43228[44], 8}, {NULL}};
const Segment SGI145945[] = {{&S43228[44], 20}, {NULL}};
const Segment SGI145946[] = {{&S43228[30], 6}, {NULL}};
const Segment SGI145947[] = {{&S43228[31], 3}, {NULL}};
const Segment SGI145948[] = {{&S43228[78], 6}, {NULL}};
const Segment SGI145949[] = {{&S43229[44], 9}, {NULL}};
const Segment SGI145950[] = {{&S43229[44], 21}, {NULL}};
const Segment SGI145951[] = {{&S43229[30], 6}, {NULL}};
const Segment SGI145952[] = {{&S43229[31], 3}, {NULL}};
const Segment SGI145953[] = {{&S43229[79], 6}, {NULL}};
const Segment SGI145954[] = {{&S43230[44], 9}, {NULL}};
const Segment SGI145955[] = {{&S43230[44], 21}, {NULL}};
const Segment SGI145956[] = {{&S43230[30], 6}, {NULL}};
const Segment SGI145957[] = {{&S43230[31], 3}, {NULL}};
const Segment SGI145958[] = {{&S43230[79], 6}, {NULL}};
const Segment SGI145959[] = {{&S43231[44], 9}, {NULL}};
const Segment SGI145960[] = {{&S43231[44], 21}, {NULL}};
const Segment SGI145961[] = {{&S43231[30], 6}, {NULL}};
const Segment SGI145962[] = {{&S43231[31], 3}, {NULL}};
const Segment SGI145963[] = {{&S43231[79], 6}, {NULL}};
const Segment SGI145964[] = {{&S43232[44], 9}, {NULL}};
const Segment SGI145965[] = {{&S43232[44], 21}, {NULL}};
const Segment SGI145966[] = {{&S43232[30], 6}, {NULL}};
const Segment SGI145967[] = {{&S43232[31], 3}, {NULL}};
const Segment SGI145968[] = {{&S43232[79], 6}, {NULL}};
const Segment SGI145969[] = {{&S18684[44], 14}, {NULL}};
const Segment SGI145970[] = {{&S18684[44], 25}, {NULL}};
const Segment SGI145971[] = {{&S18684[30], 6}, {NULL}};
const Segment SGI145972[] = {{&S18684[118], 3}, {NULL}};
const Segment SGI145973[] = {{&S18684[83], 6}, {NULL}};
const Segment SGI145974[] = {{&S18685[44], 14}, {NULL}};
const Segment SGI145975[] = {{&S18685[44], 26}, {NULL}};
const Segment SGI145976[] = {{&S18685[30], 6}, {NULL}};
const Segment SGI145977[] = {{&S18685[119], 3}, {NULL}};
const Segment SGI145978[] = {{&S43235[44], 9}, {NULL}};
const Segment SGI145979[] = {{&S18685[84], 6}, {NULL}};
const Segment SGI145980[] = {{&S43235[44], 21}, {NULL}};
const Segment SGI145981[] = {{&S43235[30], 6}, {NULL}};
const Segment SGI145982[] = {{&S43235[31], 3}, {NULL}};
const Segment SGI145983[] = {{&S43235[79], 6}, {NULL}};
const Segment SGI145984[] = {{&S43236[44], 9}, {NULL}};
const Segment SGI145985[] = {{&S43236[44], 21}, {NULL}};
const Segment SGI145986[] = {{&S43236[30], 6}, {NULL}};
const Segment SGI145987[] = {{&S43236[31], 3}, {NULL}};
const Segment SGI145988[] = {{&S43236[79], 6}, {NULL}};
const Segment SGI145989[] = {{&S43238[44], 9}, {NULL}};
const Segment SGI145990[] = {{&S43238[44], 21}, {NULL}};
const Segment SGI145991[] = {{&S43238[30], 6}, {NULL}};
const Segment SGI145992[] = {{&S43238[82], 6}, {NULL}};
const Segment SGI145993[] = {{&S43239[44], 9}, {NULL}};
const Segment SGI145994[] = {{&S43239[44], 21}, {NULL}};
const Segment SGI145995[] = {{&S43239[30], 6}, {NULL}};
const Segment SGI145996[] = {{&S43239[82], 6}, {NULL}};
const Segment SGI145997[] = {{&S43241[44], 13}, {NULL}};
const Segment SGI145998[] = {{&S43241[44], 25}, {NULL}};
const Segment SGI145999[] = {{&S43241[30], 6}, {NULL}};
const Segment SGI146000[] = {{&S43241[31], 3}, {NULL}};
const Segment SGI146001[] = {{&S43241[83], 6}, {NULL}};
const Segment SGI146002[] = {{&S43242[44], 8}, {NULL}};
const Segment SGI146003[] = {{&S43242[44], 20}, {NULL}};
const Segment SGI146004[] = {{&S43242[30], 6}, {NULL}};
const Segment SGI146005[] = {{&S43242[31], 3}, {NULL}};
const Segment SGI146006[] = {{&S43242[78], 6}, {NULL}};
const Segment SGI146007[] = {{&S43243[38], 8}, {NULL}};
const Segment SGI146008[] = {{&S43243[31], 6}, {NULL}};
const Segment SGI146009[] = {{&S43248[34], 11}, {NULL}};
const Segment SGI146010[] = {{&S43248[27], 6}, {NULL}};
const Segment SGI146011[] = {{&S43248[98], 9}, {NULL}};
const Segment SGI146012[] = {{&S43248[71], 6}, {NULL}};
const Segment SGI146013[] = {{&S43251[34], 10}, {NULL}};
const Segment SGI146014[] = {{&S43251[27], 6}, {NULL}};
const Segment SGI146015[] = {{&S43251[97], 9}, {NULL}};
const Segment SGI146016[] = {{&S43251[70], 6}, {NULL}};
const Segment SGI146017[] = {{&S43252[34], 10}, {NULL}};
const Segment SGI146018[] = {{&S43252[27], 6}, {NULL}};
const Segment SGI146019[] = {{&S43252[97], 9}, {NULL}};
const Segment SGI146020[] = {{&S43252[70], 6}, {NULL}};
const Segment SGI146021[] = {{&S43253[34], 10}, {NULL}};
const Segment SGI146022[] = {{&S43253[27], 6}, {NULL}};
const Segment SGI146023[] = {{&S43253[97], 9}, {NULL}};
const Segment SGI146024[] = {{&S43253[70], 6}, {NULL}};
const Segment SGI146025[] = {{&S43267[34], 7}, {NULL}};
const Segment SGI146026[] = {{&S43267[27], 6}, {NULL}};
const Segment SGI146027[] = {{&S43267[110], 9}, {NULL}};
const Segment SGI146028[] = {{&S43267[83], 6}, {NULL}};
const Segment SGI146029[] = {{&S43280[34], 7}, {NULL}};
const Segment SGI146030[] = {{&S43280[27], 6}, {NULL}};
const Segment SGI146031[] = {{&S43280[94], 9}, {NULL}};
const Segment SGI146032[] = {{&S43280[67], 6}, {NULL}};
const Segment SGI146033[] = {{&S43282[34], 12}, {NULL}};
const Segment SGI146034[] = {{&S43282[27], 6}, {NULL}};
const Segment SGI146035[] = {{&S43282[115], 9}, {NULL}};
const Segment SGI146036[] = {{&S43282[88], 6}, {NULL}};
const Segment SGI146037[] = {{&S43283[34], 12}, {NULL}};
const Segment SGI146038[] = {{&S43283[27], 6}, {NULL}};
const Segment SGI146039[] = {{&S43283[115], 9}, {NULL}};
const Segment SGI146040[] = {{&S43283[88], 6}, {NULL}};
const Segment SGI146041[] = {{&S43284[34], 12}, {NULL}};
const Segment SGI146042[] = {{&S43284[27], 6}, {NULL}};
const Segment SGI146043[] = {{&S43284[99], 9}, {NULL}};
const Segment SGI146044[] = {{&S43284[72], 6}, {NULL}};
const Segment SGI146045[] = {{&S43285[44], 10}, {NULL}};
const Segment SGI146046[] = {{&S43285[44], 22}, {NULL}};
const Segment SGI146047[] = {{&S43285[30], 6}, {NULL}};
const Segment SGI146048[] = {{&S43285[31], 3}, {NULL}};
const Segment SGI146049[] = {{&S43285[80], 6}, {NULL}};
const Segment SGI146050[] = {{&S43286[38], 10}, {NULL}};
const Segment SGI146051[] = {{&S43286[31], 6}, {NULL}};
const Segment SGI146052[] = {{&S43300[34], 10}, {NULL}};
const Segment SGI146053[] = {{&S43300[27], 6}, {NULL}};
const Segment SGI146054[] = {{&S43300[97], 9}, {NULL}};
const Segment SGI146055[] = {{&S43300[70], 6}, {NULL}};
const Segment SGI146056[] = {{&S43304[34], 11}, {NULL}};
const Segment SGI146057[] = {{&S43304[27], 6}, {NULL}};
const Segment SGI146058[] = {{&S43304[98], 9}, {NULL}};
const Segment SGI146059[] = {{&S43304[71], 6}, {NULL}};
const Segment SGI146060[] = {{&S43305[34], 11}, {NULL}};
const Segment SGI146061[] = {{&S43305[27], 6}, {NULL}};
const Segment SGI146062[] = {{&S43305[98], 9}, {NULL}};
const Segment SGI146063[] = {{&S43305[71], 6}, {NULL}};
const Segment SGI146064[] = {{&S34561[34], 7}, {NULL}};
const Segment SGI146065[] = {{&S34561[27], 6}, {NULL}};
const Segment SGI146066[] = {{&S34561[99], 9}, {NULL}};
const Segment SGI146067[] = {{&S34561[72], 6}, {NULL}};
const Segment SGI146068[] = {{&S43374[34], 15}, {NULL}};
const Segment SGI146069[] = {{&S43374[27], 6}, {NULL}};
const Segment SGI146070[] = {{&S43374[102], 9}, {NULL}};
const Segment SGI146071[] = {{&S43374[75], 6}, {NULL}};
const Segment SGI146072[] = {{&S43375[34], 16}, {NULL}};
const Segment SGI146073[] = {{&S43375[27], 6}, {NULL}};
const Segment SGI146074[] = {{&S43375[103], 9}, {NULL}};
const Segment SGI146075[] = {{&S43375[76], 6}, {NULL}};
const Segment SGI146076[] = {{&S43413[44], 30}, {NULL}};
const Segment SGI146077[] = {{&S43413[44], 42}, {NULL}};
const Segment SGI146078[] = {{&S43413[30], 6}, {NULL}};
const Segment SGI146079[] = {{&S43413[31], 3}, {NULL}};
const Segment SGI146080[] = {{&S43413[100], 6}, {NULL}};
const Segment SGI146081[] = {{&S43432[34], 9}, {NULL}};
const Segment SGI146082[] = {{&S43432[27], 6}, {NULL}};
const Segment SGI146083[] = {{&S43432[96], 9}, {NULL}};
const Segment SGI146084[] = {{&S43432[69], 6}, {NULL}};
const Segment SGI146085[] = {{&S43488[34], 11}, {NULL}};
const Segment SGI146086[] = {{&S43488[27], 6}, {NULL}};
const Segment SGI146087[] = {{&S43488[98], 9}, {NULL}};
const Segment SGI146088[] = {{&S43488[71], 6}, {NULL}};
const Segment SGI146089[] = {{&S43491[34], 10}, {NULL}};
const Segment SGI146090[] = {{&S43491[27], 6}, {NULL}};
const Segment SGI146091[] = {{&S43491[97], 9}, {NULL}};
const Segment SGI146092[] = {{&S43491[70], 6}, {NULL}};
const Segment SGI146093[] = {{&S43492[34], 11}, {NULL}};
const Segment SGI146094[] = {{&S43492[27], 6}, {NULL}};
const Segment SGI146095[] = {{&S43492[98], 9}, {NULL}};
const Segment SGI146096[] = {{&S43492[71], 6}, {NULL}};
const Segment SGI146097[] = {{&S43550[44], 11}, {NULL}};
const Segment SGI146098[] = {{&S43550[44], 23}, {NULL}};
const Segment SGI146099[] = {{&S43550[30], 6}, {NULL}};
const Segment SGI146100[] = {{&S43550[31], 3}, {NULL}};
const Segment SGI146101[] = {{&S43550[81], 6}, {NULL}};
const Segment SGI146102[] = {{&S43551[44], 7}, {NULL}};
const Segment SGI146103[] = {{&S43551[44], 19}, {NULL}};
const Segment SGI146104[] = {{&S43551[30], 6}, {NULL}};
const Segment SGI146105[] = {{&S43551[31], 3}, {NULL}};
const Segment SGI146106[] = {{&S43551[77], 6}, {NULL}};
const Segment SGI146107[] = {{&S43552[38], 7}, {NULL}};
const Segment SGI146108[] = {{&S43552[31], 6}, {NULL}};
const Segment SGI146109[] = {{&S43572[34], 9}, {NULL}};
const Segment SGI146110[] = {{&S43572[27], 6}, {NULL}};
const Segment SGI146111[] = {{&S43572[96], 9}, {NULL}};
const Segment SGI146112[] = {{&S43572[69], 6}, {NULL}};
const Segment SGI146113[] = {{&S43574[44], 11}, {NULL}};
const Segment SGI146114[] = {{&S43574[44], 23}, {NULL}};
const Segment SGI146115[] = {{&S43574[30], 6}, {NULL}};
const Segment SGI146116[] = {{&S43574[31], 3}, {NULL}};
const Segment SGI146117[] = {{&S43574[81], 6}, {NULL}};
const Segment SGI146118[] = {{&S43575[38], 11}, {NULL}};
const Segment SGI146119[] = {{&S43575[31], 6}, {NULL}};
const Segment SGI146120[] = {{&S43639[44], 10}, {NULL}};
const Segment SGI146121[] = {{&S43639[44], 33}, {NULL}};
const Segment SGI146122[] = {{&S43639[30], 6}, {NULL}};
const Segment SGI146123[] = {{&S43639[31], 3}, {NULL}};
const Segment SGI146124[] = {{&S43639[91], 6}, {NULL}};
const Segment SGI146125[] = {{&S43640[44], 10}, {NULL}};
const Segment SGI146126[] = {{&S43640[44], 33}, {NULL}};
const Segment SGI146127[] = {{&S43640[30], 6}, {NULL}};
const Segment SGI146128[] = {{&S43640[31], 3}, {NULL}};
const Segment SGI146129[] = {{&S43640[91], 6}, {NULL}};
const Segment SGI146130[] = {{&S43641[44], 7}, {NULL}};
const Segment SGI146131[] = {{&S43641[44], 30}, {NULL}};
const Segment SGI146132[] = {{&S43641[30], 6}, {NULL}};
const Segment SGI146133[] = {{&S43641[31], 3}, {NULL}};
const Segment SGI146134[] = {{&S43641[88], 6}, {NULL}};
const Segment SGI146135[] = {{&S43642[38], 7}, {NULL}};
const Segment SGI146136[] = {{&S43642[31], 6}, {NULL}};
const Segment SGI146137[] = {{&S43643[44], 10}, {NULL}};
const Segment SGI146138[] = {{&S43643[44], 33}, {NULL}};
const Segment SGI146139[] = {{&S43643[30], 6}, {NULL}};
const Segment SGI146140[] = {{&S43643[31], 3}, {NULL}};
const Segment SGI146141[] = {{&S43643[91], 6}, {NULL}};
const Segment SGI146142[] = {{&S31972[32], 9}, {NULL}};
const Segment SGI146143[] = {{&S31972[27], 4}, {NULL}};
const Segment SGI146144[] = {{&S31972[94], 9}, {NULL}};
const Segment SGI146145[] = {{&S31972[67], 6}, {NULL}};
const Segment SGI146146[] = {{&S43683[34], 10}, {NULL}};
const Segment SGI146147[] = {{&S43683[27], 6}, {NULL}};
const Segment SGI146148[] = {{&S43683[97], 9}, {NULL}};
const Segment SGI146149[] = {{&S43683[70], 6}, {NULL}};
const Segment SGI146150[] = {{&S43686[34], 11}, {NULL}};
const Segment SGI146151[] = {{&S43686[27], 6}, {NULL}};
const Segment SGI146152[] = {{&S43686[98], 9}, {NULL}};
const Segment SGI146153[] = {{&S43686[71], 6}, {NULL}};
const Segment SGI146154[] = {{&S43687[34], 11}, {NULL}};
const Segment SGI146155[] = {{&S43687[27], 6}, {NULL}};
const Segment SGI146156[] = {{&S43687[98], 9}, {NULL}};
const Segment SGI146157[] = {{&S43687[71], 6}, {NULL}};
const Segment SGI146158[] = {{&S43689[34], 12}, {NULL}};
const Segment SGI146159[] = {{&S43689[27], 6}, {NULL}};
const Segment SGI146160[] = {{&S43689[99], 9}, {NULL}};
const Segment SGI146161[] = {{&S43689[72], 6}, {NULL}};
const Segment SGI146162[] = {{&S43693[34], 7}, {NULL}};
const Segment SGI146163[] = {{&S43693[27], 6}, {NULL}};
const Segment SGI146164[] = {{&S43693[111], 9}, {NULL}};
const Segment SGI146165[] = {{&S43693[84], 6}, {NULL}};
const Segment SGI146166[] = {{&S43792[34], 7}, {NULL}};
const Segment SGI146167[] = {{&S43792[27], 6}, {NULL}};
const Segment SGI146168[] = {{&S43792[110], 9}, {NULL}};
const Segment SGI146169[] = {{&S43792[83], 6}, {NULL}};
const Segment SGI146170[] = {{&S43793[34], 7}, {NULL}};
const Segment SGI146171[] = {{&S43793[27], 6}, {NULL}};
const Segment SGI146172[] = {{&S43793[103], 9}, {NULL}};
const Segment SGI146173[] = {{&S43793[76], 6}, {NULL}};
const Segment SGI146174[] = {{&S43794[44], 20}, {NULL}};
const Segment SGI146175[] = {{&S43794[44], 48}, {NULL}};
const Segment SGI146176[] = {{&S43794[30], 6}, {NULL}};
const Segment SGI146177[] = {{&S43794[31], 3}, {NULL}};
const Segment SGI146178[] = {{&S43794[106], 6}, {NULL}};
const Segment SGI146179[] = {{&S43795[44], 20}, {NULL}};
const Segment SGI146180[] = {{&S43795[44], 48}, {NULL}};
const Segment SGI146181[] = {{&S43795[30], 6}, {NULL}};
const Segment SGI146182[] = {{&S43795[31], 3}, {NULL}};
const Segment SGI146183[] = {{&S43795[106], 6}, {NULL}};
const Segment SGI146184[] = {{&S43796[44], 20}, {NULL}};
const Segment SGI146185[] = {{&S43796[44], 48}, {NULL}};
const Segment SGI146186[] = {{&S43796[30], 6}, {NULL}};
const Segment SGI146187[] = {{&S43796[31], 3}, {NULL}};
const Segment SGI146188[] = {{&S43796[106], 6}, {NULL}};
const Segment SGI146189[] = {{&S43797[38], 20}, {NULL}};
const Segment SGI146190[] = {{&S43797[31], 6}, {NULL}};
const Segment SGI146191[] = {{&S39096[34], 30}, {NULL}};
const Segment SGI146192[] = {{&S39096[27], 6}, {NULL}};
const Segment SGI146193[] = {{&S39096[90], 6}, {NULL}};
const Segment SGI146194[] = {{&S39110[34], 24}, {NULL}};
const Segment SGI146195[] = {{&S39110[27], 6}, {NULL}};
const Segment SGI146196[] = {{&S39110[84], 6}, {NULL}};
const Segment SGI146197[] = {{&S39112[34], 25}, {NULL}};
const Segment SGI146198[] = {{&S39112[27], 6}, {NULL}};
const Segment SGI146199[] = {{&S39112[85], 6}, {NULL}};
const Segment SGI146200[] = {{&S39118[34], 25}, {NULL}};
const Segment SGI146201[] = {{&S39118[27], 6}, {NULL}};
const Segment SGI146202[] = {{&S39118[85], 6}, {NULL}};
const Segment SGI146203[] = {{&S39127[34], 29}, {NULL}};
const Segment SGI146204[] = {{&S39127[27], 6}, {NULL}};
const Segment SGI146205[] = {{&S39127[89], 6}, {NULL}};
const Segment SGI146206[] = {{&S43857[44], 12}, {NULL}};
const Segment SGI146207[] = {{&S43857[44], 24}, {NULL}};
const Segment SGI146208[] = {{&S43857[30], 6}, {NULL}};
const Segment SGI146209[] = {{&S43857[31], 3}, {NULL}};
const Segment SGI146210[] = {{&S43857[82], 6}, {NULL}};
const Segment SGI146211[] = {{&S43858[44], 12}, {NULL}};
const Segment SGI146212[] = {{&S43858[44], 24}, {NULL}};
const Segment SGI146213[] = {{&S43858[30], 6}, {NULL}};
const Segment SGI146214[] = {{&S43858[31], 3}, {NULL}};
const Segment SGI146215[] = {{&S43858[82], 6}, {NULL}};
const Segment SGI146216[] = {{&S43859[38], 12}, {NULL}};
const Segment SGI146217[] = {{&S43859[31], 6}, {NULL}};
const Segment SGI146218[] = {{&S43860[44], 23}, {NULL}};
const Segment SGI146219[] = {{&S43860[44], 35}, {NULL}};
const Segment SGI146220[] = {{&S43860[30], 6}, {NULL}};
const Segment SGI146221[] = {{&S43860[31], 3}, {NULL}};
const Segment SGI146222[] = {{&S43860[93], 6}, {NULL}};
const Segment SGI146223[] = {{&S43862[44], 23}, {NULL}};
const Segment SGI146224[] = {{&S43862[44], 35}, {NULL}};
const Segment SGI146225[] = {{&S43862[30], 6}, {NULL}};
const Segment SGI146226[] = {{&S43862[31], 3}, {NULL}};
const Segment SGI146227[] = {{&S43862[93], 6}, {NULL}};
const Segment SGI146228[] = {{&S43863[44], 23}, {NULL}};
const Segment SGI146229[] = {{&S43863[44], 35}, {NULL}};
const Segment SGI146230[] = {{&S43863[30], 6}, {NULL}};
const Segment SGI146231[] = {{&S43863[31], 3}, {NULL}};
const Segment SGI146232[] = {{&S43863[93], 6}, {NULL}};
const Segment SGI146233[] = {{&S43864[44], 24}, {NULL}};
const Segment SGI146234[] = {{&S43864[44], 36}, {NULL}};
const Segment SGI146235[] = {{&S43864[30], 6}, {NULL}};
const Segment SGI146236[] = {{&S43864[31], 3}, {NULL}};
const Segment SGI146237[] = {{&S43864[94], 6}, {NULL}};
const Segment SGI146238[] = {{&S43865[44], 24}, {NULL}};
const Segment SGI146239[] = {{&S43865[44], 36}, {NULL}};
const Segment SGI146240[] = {{&S43865[30], 6}, {NULL}};
const Segment SGI146241[] = {{&S43865[31], 3}, {NULL}};
const Segment SGI146242[] = {{&S43865[94], 6}, {NULL}};
const Segment SGI146243[] = {{&S43866[44], 23}, {NULL}};
const Segment SGI146244[] = {{&S43866[44], 35}, {NULL}};
const Segment SGI146245[] = {{&S43866[30], 6}, {NULL}};
const Segment SGI146246[] = {{&S43866[31], 3}, {NULL}};
const Segment SGI146247[] = {{&S43866[93], 6}, {NULL}};
const Segment SGI146248[] = {{&S43867[44], 10}, {NULL}};
const Segment SGI146249[] = {{&S43867[44], 22}, {NULL}};
const Segment SGI146250[] = {{&S43867[30], 6}, {NULL}};
const Segment SGI146251[] = {{&S43867[31], 3}, {NULL}};
const Segment SGI146252[] = {{&S43867[80], 6}, {NULL}};
const Segment SGI146253[] = {{&S43868[44], 24}, {NULL}};
const Segment SGI146254[] = {{&S43868[44], 36}, {NULL}};
const Segment SGI146255[] = {{&S43868[30], 6}, {NULL}};
const Segment SGI146256[] = {{&S43868[31], 3}, {NULL}};
const Segment SGI146257[] = {{&S43868[94], 6}, {NULL}};
const Segment SGI146258[] = {{&S43869[44], 15}, {NULL}};
const Segment SGI146259[] = {{&S43869[44], 33}, {NULL}};
const Segment SGI146260[] = {{&S43869[30], 6}, {NULL}};
const Segment SGI146261[] = {{&S43869[31], 3}, {NULL}};
const Segment SGI146262[] = {{&S43869[91], 6}, {NULL}};
const Segment SGI146263[] = {{&S43870[44], 15}, {NULL}};
const Segment SGI146264[] = {{&S43870[44], 33}, {NULL}};
const Segment SGI146265[] = {{&S43870[30], 6}, {NULL}};
const Segment SGI146266[] = {{&S43870[94], 6}, {NULL}};
const Segment SGI146267[] = {{&S43871[44], 43}, {NULL}};
const Segment SGI146268[] = {{&S43871[30], 6}, {NULL}};
const Segment SGI146269[] = {{&S43871[119], 6}, {NULL}};
const Segment SGI146270[] = {{&S43872[44], 39}, {NULL}};
const Segment SGI146271[] = {{&S43872[30], 6}, {NULL}};
const Segment SGI146272[] = {{&S43872[115], 6}, {NULL}};
const Segment SGI146273[] = {{&S43873[44], 43}, {NULL}};
const Segment SGI146274[] = {{&S43873[30], 6}, {NULL}};
const Segment SGI146275[] = {{&S43873[119], 6}, {NULL}};
const Segment SGI146276[] = {{&S43874[44], 39}, {NULL}};
const Segment SGI146277[] = {{&S43874[30], 6}, {NULL}};
const Segment SGI146278[] = {{&S43874[115], 6}, {NULL}};
const Segment SGI146279[] = {{&S43875[44], 15}, {NULL}};
const Segment SGI146280[] = {{&S43875[44], 33}, {NULL}};
const Segment SGI146281[] = {{&S43875[30], 6}, {NULL}};
const Segment SGI146282[] = {{&S43875[31], 3}, {NULL}};
const Segment SGI146283[] = {{&S43875[91], 6}, {NULL}};
const Segment SGI146284[] = {{&S43876[44], 15}, {NULL}};
const Segment SGI146285[] = {{&S43876[44], 33}, {NULL}};
const Segment SGI146286[] = {{&S43876[30], 6}, {NULL}};
const Segment SGI146287[] = {{&S43876[94], 6}, {NULL}};
const Segment SGI146288[] = {{&S43877[44], 43}, {NULL}};
const Segment SGI146289[] = {{&S43877[30], 6}, {NULL}};
const Segment SGI146290[] = {{&S43877[119], 6}, {NULL}};
const Segment SGI146291[] = {{&S43878[38], 15}, {NULL}};
const Segment SGI146292[] = {{&S43878[31], 6}, {NULL}};
const Segment SGI146293[] = {{&S43879[44], 15}, {NULL}};
const Segment SGI146294[] = {{&S43879[44], 33}, {NULL}};
const Segment SGI146295[] = {{&S43879[30], 6}, {NULL}};
const Segment SGI146296[] = {{&S43879[94], 4}, {NULL}};
const Segment SGI146297[] = {{&S43879[94], 3}, {NULL}};
const Segment SGI146298[] = {{&S43880[44], 16}, {NULL}};
const Segment SGI146299[] = {{&S43880[44], 35}, {NULL}};
const Segment SGI146300[] = {{&S43880[30], 6}, {NULL}};
const Segment SGI146301[] = {{&S43880[31], 3}, {NULL}};
const Segment SGI146302[] = {{&S43880[93], 6}, {NULL}};
const Segment SGI146303[] = {{&S43881[44], 39}, {NULL}};
const Segment SGI146304[] = {{&S43881[30], 6}, {NULL}};
const Segment SGI146305[] = {{&S43881[115], 6}, {NULL}};
const Segment SGI146306[] = {{&S43882[44], 39}, {NULL}};
const Segment SGI146307[] = {{&S43882[30], 6}, {NULL}};
const Segment SGI146308[] = {{&S43882[115], 6}, {NULL}};
const Segment SGI146309[] = {{&S43883[38], 16}, {NULL}};
const Segment SGI146310[] = {{&S43883[31], 6}, {NULL}};
const Segment SGI146311[] = {{&S43884[38], 15}, {NULL}};
const Segment SGI146312[] = {{&S43884[31], 6}, {NULL}};
const Segment SGI146313[] = {{&S43885[44], 8}, {NULL}};
const Segment SGI146314[] = {{&S43885[44], 27}, {NULL}};
const Segment SGI146315[] = {{&S43885[30], 6}, {NULL}};
const Segment SGI146316[] = {{&S43885[31], 3}, {NULL}};
const Segment SGI146317[] = {{&S43885[85], 6}, {NULL}};
const Segment SGI146318[] = {{&S43886[44], 39}, {NULL}};
const Segment SGI146319[] = {{&S43886[30], 6}, {NULL}};
const Segment SGI146320[] = {{&S43886[115], 6}, {NULL}};
const Segment SGI146321[] = {{&S43887[44], 43}, {NULL}};
const Segment SGI146322[] = {{&S43887[30], 6}, {NULL}};
const Segment SGI146323[] = {{&S43887[119], 6}, {NULL}};
const Segment SGI146324[] = {{&S43888[44], 40}, {NULL}};
const Segment SGI146325[] = {{&S43888[30], 6}, {NULL}};
const Segment SGI146326[] = {{&S43888[116], 6}, {NULL}};
const Segment SGI146327[] = {{&S43889[44], 40}, {NULL}};
const Segment SGI146328[] = {{&S43889[30], 6}, {NULL}};
const Segment SGI146329[] = {{&S43889[116], 6}, {NULL}};
const Segment SGI146330[] = {{&S43890[44], 39}, {NULL}};
const Segment SGI146331[] = {{&S43890[30], 6}, {NULL}};
const Segment SGI146332[] = {{&S43890[115], 6}, {NULL}};
const Segment SGI146333[] = {{&S43891[44], 21}, {NULL}};
const Segment SGI146334[] = {{&S43891[44], 40}, {NULL}};
const Segment SGI146335[] = {{&S43891[30], 6}, {NULL}};
const Segment SGI146336[] = {{&S43891[31], 3}, {NULL}};
const Segment SGI146337[] = {{&S43891[98], 6}, {NULL}};
const Segment SGI146338[] = {{&S43896[44], 7}, {NULL}};
const Segment SGI146339[] = {{&S43896[44], 24}, {NULL}};
const Segment SGI146340[] = {{&S43896[30], 6}, {NULL}};
const Segment SGI146341[] = {{&S43896[31], 3}, {NULL}};
const Segment SGI146342[] = {{&S43896[82], 6}, {NULL}};
const Segment SGI146343[] = {{&S43898[38], 7}, {NULL}};
const Segment SGI146344[] = {{&S43898[31], 6}, {NULL}};
const Segment SGI146345[] = {{&S43899[44], 7}, {NULL}};
const Segment SGI146346[] = {{&S43899[44], 24}, {NULL}};
const Segment SGI146347[] = {{&S43899[30], 6}, {NULL}};
const Segment SGI146348[] = {{&S43899[31], 3}, {NULL}};
const Segment SGI146349[] = {{&S43899[82], 6}, {NULL}};
const Segment SGI146350[] = {{&S34559[34], 7}, {NULL}};
const Segment SGI146351[] = {{&S34559[27], 6}, {NULL}};
const Segment SGI146352[] = {{&S34559[97], 9}, {NULL}};
const Segment SGI146353[] = {{&S34559[70], 6}, {NULL}};
const Segment SGI146354[] = {{&S34560[34], 7}, {NULL}};
const Segment SGI146355[] = {{&S34560[27], 6}, {NULL}};
const Segment SGI146356[] = {{&S34560[103], 9}, {NULL}};
const Segment SGI146357[] = {{&S34560[76], 6}, {NULL}};
const Segment SGI146358[] = {{&S34830[44], 15}, {NULL}};
const Segment SGI146359[] = {{&S34830[44], 27}, {NULL}};
const Segment SGI146360[] = {{&S34830[30], 6}, {NULL}};
const Segment SGI146361[] = {{&S34830[120], 3}, {NULL}};
const Segment SGI146362[] = {{&S34830[85], 6}, {NULL}};
const Segment SGI146363[] = {{&S34831[44], 12}, {NULL}};
const Segment SGI146364[] = {{&S34831[44], 24}, {NULL}};
const Segment SGI146365[] = {{&S34831[30], 6}, {NULL}};
const Segment SGI146366[] = {{&S34831[117], 3}, {NULL}};
const Segment SGI146367[] = {{&S34831[82], 6}, {NULL}};
const Segment SGI146368[] = {{&S34832[38], 12}, {NULL}};
const Segment SGI146369[] = {{&S34832[31], 6}, {NULL}};
const Segment SGI146370[] = {{&S34836[42], 12}, {NULL}};
const Segment SGI146371[] = {{&S34836[30], 6}, {NULL}};
const Segment SGI146372[] = {{&S34836[115], 3}, {NULL}};
const Segment SGI146373[] = {{&S34836[80], 6}, {NULL}};
const Segment SGI146374[] = {{&S44018[34], 15}, {NULL}};
const Segment SGI146375[] = {{&S44018[27], 6}, {NULL}};
const Segment SGI146376[] = {{&S44018[102], 9}, {NULL}};
const Segment SGI146377[] = {{&S44018[75], 6}, {NULL}};
const Segment SGI146378[] = {{&S44019[34], 15}, {NULL}};
const Segment SGI146379[] = {{&S44019[27], 6}, {NULL}};
const Segment SGI146380[] = {{&S44019[102], 9}, {NULL}};
const Segment SGI146381[] = {{&S44019[75], 6}, {NULL}};
const Segment SGI146382[] = {{&S44022[34], 14}, {NULL}};
const Segment SGI146383[] = {{&S44022[27], 6}, {NULL}};
const Segment SGI146384[] = {{&S44022[101], 9}, {NULL}};
const Segment SGI146385[] = {{&S44022[74], 6}, {NULL}};
const Segment SGI146386[] = {{&S44024[34], 14}, {NULL}};
const Segment SGI146387[] = {{&S44024[27], 6}, {NULL}};
const Segment SGI146388[] = {{&S44024[101], 9}, {NULL}};
const Segment SGI146389[] = {{&S44024[74], 6}, {NULL}};
const Segment SGI146390[] = {{&S44025[34], 10}, {NULL}};
const Segment SGI146391[] = {{&S44025[27], 6}, {NULL}};
const Segment SGI146392[] = {{&S44025[97], 9}, {NULL}};
const Segment SGI146393[] = {{&S44025[70], 6}, {NULL}};
const Segment SGI146394[] = {{&S44026[34], 26}, {NULL}};
const Segment SGI146395[] = {{&S44026[27], 6}, {NULL}};
const Segment SGI146396[] = {{&S44026[113], 9}, {NULL}};
const Segment SGI146397[] = {{&S44026[86], 6}, {NULL}};
const Segment SGI146398[] = {{&S44027[41], 14}, {NULL}};
const Segment SGI146399[] = {{&S44027[27], 13}, {NULL}};
const Segment SGI146400[] = {{&S44027[108], 9}, {NULL}};
const Segment SGI146401[] = {{&S44027[81], 6}, {NULL}};
const Segment SGI146402[] = {{&S44044[34], 19}, {NULL}};
const Segment SGI146403[] = {{&S44044[27], 6}, {NULL}};
const Segment SGI146404[] = {{&S44044[106], 9}, {NULL}};
const Segment SGI146405[] = {{&S44044[79], 6}, {NULL}};
const Segment SGI146406[] = {{&S44045[34], 19}, {NULL}};
const Segment SGI146407[] = {{&S44045[27], 6}, {NULL}};
const Segment SGI146408[] = {{&S44045[106], 9}, {NULL}};
const Segment SGI146409[] = {{&S44045[79], 6}, {NULL}};
const Segment SGI146410[] = {{&S34919[44], 7}, {NULL}};
const Segment SGI146411[] = {{&S34919[44], 24}, {NULL}};
const Segment SGI146412[] = {{&S34919[30], 6}, {NULL}};
const Segment SGI146413[] = {{&S34919[117], 3}, {NULL}};
const Segment SGI146414[] = {{&S34919[82], 6}, {NULL}};
const Segment SGI146415[] = {{&S44064[34], 10}, {NULL}};
const Segment SGI146416[] = {{&S34920[38], 7}, {NULL}};
const Segment SGI146417[] = {{&S34920[31], 6}, {NULL}};
const Segment SGI146418[] = {{&S44064[27], 6}, {NULL}};
const Segment SGI146419[] = {{&S44064[97], 9}, {NULL}};
const Segment SGI146420[] = {{&S44064[70], 6}, {NULL}};
const Segment SGI146421[] = {{&S34921[44], 7}, {NULL}};
const Segment SGI146422[] = {{&S34921[44], 22}, {NULL}};
const Segment SGI146423[] = {{&S34921[30], 6}, {NULL}};
const Segment SGI146424[] = {{&S44065[34], 13}, {NULL}};
const Segment SGI146425[] = {{&S44065[27], 6}, {NULL}};
const Segment SGI146426[] = {{&S34921[115], 3}, {NULL}};
const Segment SGI146427[] = {{&S34921[80], 6}, {NULL}};
const Segment SGI146428[] = {{&S44065[100], 9}, {NULL}};
const Segment SGI146429[] = {{&S44065[73], 6}, {NULL}};
const Segment SGI146430[] = {{&S34922[37], 15}, {NULL}};
const Segment SGI146431[] = {{&S44066[34], 14}, {NULL}};
const Segment SGI146432[] = {{&S44066[27], 6}, {NULL}};
const Segment SGI146433[] = {{&S44066[101], 9}, {NULL}};
const Segment SGI146434[] = {{&S44066[74], 6}, {NULL}};
const Segment SGI146435[] = {{&S34926[55], 7}, {NULL}};
const Segment SGI146436[] = {{&S34926[55], 24}, {NULL}};
const Segment SGI146437[] = {{&S34926[41], 6}, {NULL}};
const Segment SGI146438[] = {{&S34926[132], 3}, {NULL}};
const Segment SGI146439[] = {{&S34926[98], 5}, {NULL}};
const Segment SGI146440[] = {{&S34928[42], 7}, {NULL}};
const Segment SGI146441[] = {{&S34928[30], 6}, {NULL}};
const Segment SGI146442[] = {{&S34928[115], 3}, {NULL}};
const Segment SGI146443[] = {{&S34928[80], 6}, {NULL}};
const Segment SGI146444[] = {{&S44215[38], 7}, {NULL}};
const Segment SGI146445[] = {{&S44215[31], 6}, {NULL}};
const Segment SGI146446[] = {{&S44216[44], 7}, {NULL}};
const Segment SGI146447[] = {{&S44216[44], 24}, {NULL}};
const Segment SGI146448[] = {{&S44216[30], 6}, {NULL}};
const Segment SGI146449[] = {{&S44216[31], 3}, {NULL}};
const Segment SGI146450[] = {{&S44216[82], 6}, {NULL}};
const Segment SGI146451[] = {{&S44217[44], 7}, {NULL}};
const Segment SGI146452[] = {{&S44217[44], 24}, {NULL}};
const Segment SGI146453[] = {{&S44217[30], 6}, {NULL}};
const Segment SGI146454[] = {{&S44217[31], 3}, {NULL}};
const Segment SGI146455[] = {{&S44217[82], 6}, {NULL}};
const Segment SGI146456[] = {{&S44218[44], 7}, {NULL}};
const Segment SGI146457[] = {{&S44218[44], 24}, {NULL}};
const Segment SGI146458[] = {{&S44218[30], 6}, {NULL}};
const Segment SGI146459[] = {{&S44218[31], 3}, {NULL}};
const Segment SGI146460[] = {{&S44218[82], 6}, {NULL}};
const Segment SGI146461[] = {{&S44224[44], 10}, {NULL}};
const Segment SGI146462[] = {{&S44224[44], 22}, {NULL}};
const Segment SGI146463[] = {{&S44224[30], 6}, {NULL}};
const Segment SGI146464[] = {{&S44224[31], 3}, {NULL}};
const Segment SGI146465[] = {{&S44224[80], 6}, {NULL}};
const Segment SGI146466[] = {{&S44227[44], 11}, {NULL}};
const Segment SGI146467[] = {{&S44227[44], 23}, {NULL}};
const Segment SGI146468[] = {{&S44227[30], 6}, {NULL}};
const Segment SGI146469[] = {{&S44227[31], 3}, {NULL}};
const Segment SGI146470[] = {{&S44227[81], 6}, {NULL}};
const Segment SGI146471[] = {{&S44228[38], 10}, {NULL}};
const Segment SGI146472[] = {{&S44228[31], 6}, {NULL}};
const Segment SGI146473[] = {{&S44231[38], 11}, {NULL}};
const Segment SGI146474[] = {{&S44231[31], 6}, {NULL}};
const Segment SGI146475[] = {{&S44232[44], 10}, {NULL}};
const Segment SGI146476[] = {{&S44232[44], 22}, {NULL}};
const Segment SGI146477[] = {{&S44232[30], 6}, {NULL}};
const Segment SGI146478[] = {{&S44232[31], 3}, {NULL}};
const Segment SGI146479[] = {{&S44232[80], 6}, {NULL}};
const Segment SGI146480[] = {{&S44249[34], 10}, {NULL}};
const Segment SGI146481[] = {{&S44249[27], 6}, {NULL}};
const Segment SGI146482[] = {{&S44249[97], 9}, {NULL}};
const Segment SGI146483[] = {{&S44249[70], 6}, {NULL}};
const Segment SGI146484[] = {{&S44250[34], 11}, {NULL}};
const Segment SGI146485[] = {{&S44250[27], 6}, {NULL}};
const Segment SGI146486[] = {{&S44250[98], 9}, {NULL}};
const Segment SGI146487[] = {{&S44250[71], 6}, {NULL}};
const Segment SGI146488[] = {{&S44252[34], 18}, {NULL}};
const Segment SGI146489[] = {{&S44252[27], 6}, {NULL}};
const Segment SGI146490[] = {{&S44252[105], 9}, {NULL}};
const Segment SGI146491[] = {{&S44252[78], 6}, {NULL}};
const Segment SGI146492[] = {{&S44254[34], 10}, {NULL}};
const Segment SGI146493[] = {{&S44254[27], 6}, {NULL}};
const Segment SGI146494[] = {{&S44254[97], 9}, {NULL}};
const Segment SGI146495[] = {{&S44254[70], 6}, {NULL}};
const Segment SGI146496[] = {{&S34833[44], 15}, {NULL}};
const Segment SGI146497[] = {{&S34833[44], 27}, {NULL}};
const Segment SGI146498[] = {{&S34833[30], 6}, {NULL}};
const Segment SGI146499[] = {{&S34833[120], 3}, {NULL}};
const Segment SGI146500[] = {{&S34833[85], 6}, {NULL}};
const Segment SGI146501[] = {{&S44279[34], 7}, {NULL}};
const Segment SGI146502[] = {{&S44279[27], 6}, {NULL}};
const Segment SGI146503[] = {{&S34834[44], 12}, {NULL}};
const Segment SGI146504[] = {{&S34834[44], 24}, {NULL}};
const Segment SGI146505[] = {{&S34834[30], 6}, {NULL}};
const Segment SGI146506[] = {{&S44279[100], 9}, {NULL}};
const Segment SGI146507[] = {{&S44279[73], 6}, {NULL}};
const Segment SGI146508[] = {{&S34834[117], 3}, {NULL}};
const Segment SGI146509[] = {{&S34834[82], 6}, {NULL}};
const Segment SGI146510[] = {{&S44280[34], 10}, {NULL}};
const Segment SGI146511[] = {{&S34835[38], 12}, {NULL}};
const Segment SGI146512[] = {{&S34835[31], 6}, {NULL}};
const Segment SGI146513[] = {{&S44280[27], 6}, {NULL}};
const Segment SGI146514[] = {{&S44280[97], 9}, {NULL}};
const Segment SGI146515[] = {{&S44280[70], 6}, {NULL}};
const Segment SGI146516[] = {{&S44282[34], 8}, {NULL}};
const Segment SGI146517[] = {{&S44282[27], 6}, {NULL}};
const Segment SGI146518[] = {{&S44282[100], 9}, {NULL}};
const Segment SGI146519[] = {{&S44282[73], 6}, {NULL}};
const Segment SGI146520[] = {{&S34923[44], 7}, {NULL}};
const Segment SGI146521[] = {{&S34923[44], 28}, {NULL}};
const Segment SGI146522[] = {{&S34923[30], 6}, {NULL}};
const Segment SGI146523[] = {{&S34923[121], 3}, {NULL}};
const Segment SGI146524[] = {{&S34923[86], 6}, {NULL}};
const Segment SGI146525[] = {{&S34924[44], 7}, {NULL}};
const Segment SGI146526[] = {{&S34924[44], 22}, {NULL}};
const Segment SGI146527[] = {{&S34924[30], 6}, {NULL}};
const Segment SGI146528[] = {{&S34924[115], 3}, {NULL}};
const Segment SGI146529[] = {{&S34924[80], 6}, {NULL}};
const Segment SGI146530[] = {{&S34925[38], 7}, {NULL}};
const Segment SGI146531[] = {{&S34925[31], 6}, {NULL}};
const Segment SGI146532[] = {{&S34927[44], 7}, {NULL}};
const Segment SGI146533[] = {{&S34927[44], 24}, {NULL}};
const Segment SGI146534[] = {{&S34927[30], 6}, {NULL}};
const Segment SGI146535[] = {{&S34927[117], 3}, {NULL}};
const Segment SGI146536[] = {{&S34927[82], 6}, {NULL}};
const Segment SGI146537[] = {{&S34929[55], 7}, {NULL}};
const Segment SGI146538[] = {{&S34929[55], 22}, {NULL}};
const Segment SGI146539[] = {{&S34929[41], 6}, {NULL}};
const Segment SGI146540[] = {{&S34929[130], 3}, {NULL}};
const Segment SGI146541[] = {{&S34929[96], 5}, {NULL}};
const Segment SGI146542[] = {{&S34930[44], 7}, {NULL}};
const Segment SGI146543[] = {{&S34930[44], 22}, {NULL}};
const Segment SGI146544[] = {{&S34930[30], 6}, {NULL}};
const Segment SGI146545[] = {{&S34930[115], 3}, {NULL}};
const Segment SGI146546[] = {{&S34930[80], 6}, {NULL}};
const Segment SGI146547[] = {{&S34931[44], 7}, {NULL}};
const Segment SGI146548[] = {{&S34931[44], 24}, {NULL}};
const Segment SGI146549[] = {{&S34931[30], 6}, {NULL}};
const Segment SGI146550[] = {{&S35252[44], 15}, {NULL}};
const Segment SGI146551[] = {{&S34931[117], 3}, {NULL}};
const Segment SGI146552[] = {{&S35252[44], 30}, {NULL}};
const Segment SGI146553[] = {{&S34931[82], 6}, {NULL}};
const Segment SGI146554[] = {{&S35252[30], 6}, {NULL}};
const Segment SGI146555[] = {{&S35252[123], 3}, {NULL}};
const Segment SGI146556[] = {{&S44349[44], 10}, {NULL}};
const Segment SGI146557[] = {{&S44349[44], 22}, {NULL}};
const Segment SGI146558[] = {{&S35252[88], 6}, {NULL}};
const Segment SGI146559[] = {{&S44349[30], 6}, {NULL}};
const Segment SGI146560[] = {{&S44349[31], 3}, {NULL}};
const Segment SGI146561[] = {{&S34932[42], 7}, {NULL}};
const Segment SGI146562[] = {{&S44349[80], 6}, {NULL}};
const Segment SGI146563[] = {{&S34932[30], 6}, {NULL}};
const Segment SGI146564[] = {{&S34932[113], 3}, {NULL}};
const Segment SGI146565[] = {{&S34932[78], 6}, {NULL}};
const Segment SGI146566[] = {{&S35253[44], 15}, {NULL}};
const Segment SGI146567[] = {{&S35253[44], 30}, {NULL}};
const Segment SGI146568[] = {{&S35253[30], 6}, {NULL}};
const Segment SGI146569[] = {{&S35253[123], 3}, {NULL}};
const Segment SGI146570[] = {{&S44350[38], 10}, {NULL}};
const Segment SGI146571[] = {{&S35253[88], 6}, {NULL}};
const Segment SGI146572[] = {{&S44350[31], 6}, {NULL}};
const Segment SGI146573[] = {{&S34933[38], 7}, {NULL}};
const Segment SGI146574[] = {{&S34933[31], 6}, {NULL}};
const Segment SGI146575[] = {{&S35254[38], 15}, {NULL}};
const Segment SGI146576[] = {{&S35254[31], 6}, {NULL}};
const Segment SGI146577[] = {{&S34934[37], 15}, {NULL}};
const Segment SGI146578[] = {{&S34935[55], 7}, {NULL}};
const Segment SGI146579[] = {{&S34935[55], 22}, {NULL}};
const Segment SGI146580[] = {{&S35256[37], 23}, {NULL}};
const Segment SGI146581[] = {{&S34935[41], 6}, {NULL}};
const Segment SGI146582[] = {{&S34935[130], 3}, {NULL}};
const Segment SGI146583[] = {{&S34935[96], 5}, {NULL}};
const Segment SGI146584[] = {{&S34937[55], 7}, {NULL}};
const Segment SGI146585[] = {{&S34937[55], 24}, {NULL}};
const Segment SGI146586[] = {{&S34937[41], 6}, {NULL}};
const Segment SGI146587[] = {{&S34937[132], 3}, {NULL}};
const Segment SGI146588[] = {{&S34937[98], 5}, {NULL}};
const Segment SGI146589[] = {{&S35262[44], 15}, {NULL}};
const Segment SGI146590[] = {{&S35262[44], 29}, {NULL}};
const Segment SGI146591[] = {{&S35262[30], 6}, {NULL}};
const Segment SGI146592[] = {{&S35262[122], 3}, {NULL}};
const Segment SGI146593[] = {{&S35262[87], 6}, {NULL}};
const Segment SGI146594[] = {{&S35270[44], 7}, {NULL}};
const Segment SGI146595[] = {{&S35270[44], 35}, {NULL}};
const Segment SGI146596[] = {{&S35270[30], 6}, {NULL}};
const Segment SGI146597[] = {{&S35270[128], 3}, {NULL}};
const Segment SGI146598[] = {{&S35270[93], 6}, {NULL}};
const Segment SGI146599[] = {{&S35343[44], 14}, {NULL}};
const Segment SGI146600[] = {{&S35343[44], 26}, {NULL}};
const Segment SGI146601[] = {{&S35343[30], 6}, {NULL}};
const Segment SGI146602[] = {{&S35343[119], 3}, {NULL}};
const Segment SGI146603[] = {{&S35343[84], 6}, {NULL}};
const Segment SGI146604[] = {{&S35344[44], 14}, {NULL}};
const Segment SGI146605[] = {{&S35344[44], 26}, {NULL}};
const Segment SGI146606[] = {{&S35344[30], 6}, {NULL}};
const Segment SGI146607[] = {{&S35344[119], 3}, {NULL}};
const Segment SGI146608[] = {{&S35344[84], 6}, {NULL}};
const Segment SGI146609[] = {{&S35345[38], 14}, {NULL}};
const Segment SGI146610[] = {{&S35345[31], 6}, {NULL}};
const Segment SGI146611[] = {{&S35346[44], 34}, {NULL}};
const Segment SGI146612[] = {{&S35346[44], 46}, {NULL}};
const Segment SGI146613[] = {{&S35346[30], 6}, {NULL}};
const Segment SGI146614[] = {{&S35346[139], 3}, {NULL}};
const Segment SGI146615[] = {{&S35346[104], 6}, {NULL}};
const Segment SGI146616[] = {{&S35347[44], 25}, {NULL}};
const Segment SGI146617[] = {{&S35347[44], 37}, {NULL}};
const Segment SGI146618[] = {{&S35347[30], 6}, {NULL}};
const Segment SGI146619[] = {{&S35347[130], 3}, {NULL}};
const Segment SGI146620[] = {{&S35347[95], 6}, {NULL}};
const Segment SGI146621[] = {{&S35348[44], 25}, {NULL}};
const Segment SGI146622[] = {{&S35348[44], 37}, {NULL}};
const Segment SGI146623[] = {{&S35348[30], 6}, {NULL}};
const Segment SGI146624[] = {{&S35348[130], 3}, {NULL}};
const Segment SGI146625[] = {{&S35348[95], 6}, {NULL}};
const Segment SGI146626[] = {{&S35349[44], 26}, {NULL}};
const Segment SGI146627[] = {{&S35349[44], 38}, {NULL}};
const Segment SGI146628[] = {{&S35349[30], 6}, {NULL}};
const Segment SGI146629[] = {{&S35349[131], 3}, {NULL}};
const Segment SGI146630[] = {{&S35349[96], 6}, {NULL}};
const Segment SGI146631[] = {{&S35350[44], 25}, {NULL}};
const Segment SGI146632[] = {{&S35350[44], 37}, {NULL}};
const Segment SGI146633[] = {{&S35350[30], 6}, {NULL}};
const Segment SGI146634[] = {{&S35350[130], 3}, {NULL}};
const Segment SGI146635[] = {{&S35350[95], 6}, {NULL}};
const Segment SGI146636[] = {{&S35351[44], 25}, {NULL}};
const Segment SGI146637[] = {{&S35351[44], 37}, {NULL}};
const Segment SGI146638[] = {{&S35351[30], 6}, {NULL}};
const Segment SGI146639[] = {{&S35351[130], 3}, {NULL}};
const Segment SGI146640[] = {{&S35351[95], 6}, {NULL}};
const Segment SGI146641[] = {{&S35352[44], 25}, {NULL}};
const Segment SGI146642[] = {{&S35352[44], 37}, {NULL}};
const Segment SGI146643[] = {{&S35352[30], 6}, {NULL}};
const Segment SGI146644[] = {{&S35352[130], 3}, {NULL}};
const Segment SGI146645[] = {{&S35352[95], 6}, {NULL}};
const Segment SGI146646[] = {{&S35353[44], 25}, {NULL}};
const Segment SGI146647[] = {{&S35353[44], 37}, {NULL}};
const Segment SGI146648[] = {{&S35353[30], 6}, {NULL}};
const Segment SGI146649[] = {{&S35353[130], 3}, {NULL}};
const Segment SGI146650[] = {{&S35353[95], 6}, {NULL}};
const Segment SGI146651[] = {{&S35354[44], 23}, {NULL}};
const Segment SGI146652[] = {{&S35354[44], 35}, {NULL}};
const Segment SGI146653[] = {{&S35354[30], 6}, {NULL}};
const Segment SGI146654[] = {{&S35354[128], 3}, {NULL}};
const Segment SGI146655[] = {{&S35354[93], 6}, {NULL}};
const Segment SGI146656[] = {{&S35355[44], 25}, {NULL}};
const Segment SGI146657[] = {{&S35355[44], 37}, {NULL}};
const Segment SGI146658[] = {{&S35355[30], 6}, {NULL}};
const Segment SGI146659[] = {{&S35355[130], 3}, {NULL}};
const Segment SGI146660[] = {{&S35355[95], 6}, {NULL}};
const Segment SGI146661[] = {{&S35356[44], 24}, {NULL}};
const Segment SGI146662[] = {{&S35356[44], 36}, {NULL}};
const Segment SGI146663[] = {{&S35356[30], 6}, {NULL}};
const Segment SGI146664[] = {{&S35356[129], 3}, {NULL}};
const Segment SGI146665[] = {{&S35356[94], 6}, {NULL}};
const Segment SGI146666[] = {{&S35357[44], 23}, {NULL}};
const Segment SGI146667[] = {{&S35357[44], 35}, {NULL}};
const Segment SGI146668[] = {{&S35357[30], 6}, {NULL}};
const Segment SGI146669[] = {{&S35357[128], 3}, {NULL}};
const Segment SGI146670[] = {{&S35357[93], 6}, {NULL}};
const Segment SGI146671[] = {{&S35358[44], 25}, {NULL}};
const Segment SGI146672[] = {{&S35358[44], 37}, {NULL}};
const Segment SGI146673[] = {{&S35358[30], 6}, {NULL}};
const Segment SGI146674[] = {{&S35358[130], 3}, {NULL}};
const Segment SGI146675[] = {{&S35358[95], 6}, {NULL}};
const Segment SGI146676[] = {{&S35359[44], 34}, {NULL}};
const Segment SGI146677[] = {{&S35359[44], 46}, {NULL}};
const Segment SGI146678[] = {{&S35359[30], 6}, {NULL}};
const Segment SGI146679[] = {{&S35359[139], 3}, {NULL}};
const Segment SGI146680[] = {{&S35359[104], 6}, {NULL}};
const Segment SGI146681[] = {{&S35360[44], 24}, {NULL}};
const Segment SGI146682[] = {{&S35360[44], 36}, {NULL}};
const Segment SGI146683[] = {{&S35360[30], 6}, {NULL}};
const Segment SGI146684[] = {{&S35360[129], 3}, {NULL}};
const Segment SGI146685[] = {{&S35360[94], 6}, {NULL}};
const Segment SGI146686[] = {{&S44505[34], 10}, {NULL}};
const Segment SGI146687[] = {{&S44505[27], 6}, {NULL}};
const Segment SGI146688[] = {{&S44505[97], 9}, {NULL}};
const Segment SGI146689[] = {{&S44505[70], 6}, {NULL}};
const Segment SGI146690[] = {{&S44506[34], 10}, {NULL}};
const Segment SGI146691[] = {{&S44506[27], 6}, {NULL}};
const Segment SGI146692[] = {{&S44506[97], 9}, {NULL}};
const Segment SGI146693[] = {{&S44506[70], 6}, {NULL}};
const Segment SGI146694[] = {{&S44573[44], 9}, {NULL}};
const Segment SGI146695[] = {{&S44573[44], 21}, {NULL}};
const Segment SGI146696[] = {{&S44573[30], 6}, {NULL}};
const Segment SGI146697[] = {{&S44573[31], 3}, {NULL}};
const Segment SGI146698[] = {{&S44573[79], 6}, {NULL}};
const Segment SGI146699[] = {{&S44574[44], 10}, {NULL}};
const Segment SGI146700[] = {{&S44574[44], 22}, {NULL}};
const Segment SGI146701[] = {{&S44574[30], 6}, {NULL}};
const Segment SGI146702[] = {{&S44574[31], 3}, {NULL}};
const Segment SGI146703[] = {{&S44574[80], 6}, {NULL}};
const Segment SGI146704[] = {{&S44575[44], 10}, {NULL}};
const Segment SGI146705[] = {{&S44575[44], 22}, {NULL}};
const Segment SGI146706[] = {{&S44575[30], 6}, {NULL}};
const Segment SGI146707[] = {{&S44575[31], 3}, {NULL}};
const Segment SGI146708[] = {{&S44575[80], 6}, {NULL}};
const Segment SGI146709[] = {{&S44576[44], 10}, {NULL}};
const Segment SGI146710[] = {{&S44576[44], 22}, {NULL}};
const Segment SGI146711[] = {{&S44576[30], 6}, {NULL}};
const Segment SGI146712[] = {{&S44576[31], 3}, {NULL}};
const Segment SGI146713[] = {{&S44576[80], 6}, {NULL}};
const Segment SGI146714[] = {{&S44577[44], 17}, {NULL}};
const Segment SGI146715[] = {{&S44577[44], 29}, {NULL}};
const Segment SGI146716[] = {{&S44577[30], 6}, {NULL}};
const Segment SGI146717[] = {{&S44577[31], 3}, {NULL}};
const Segment SGI146718[] = {{&S44577[87], 6}, {NULL}};
const Segment SGI146719[] = {{&S44578[44], 17}, {NULL}};
const Segment SGI146720[] = {{&S44578[44], 29}, {NULL}};
const Segment SGI146721[] = {{&S44578[30], 6}, {NULL}};
const Segment SGI146722[] = {{&S44578[31], 3}, {NULL}};
const Segment SGI146723[] = {{&S44578[87], 6}, {NULL}};
const Segment SGI146724[] = {{&S44579[38], 9}, {NULL}};
const Segment SGI146725[] = {{&S44579[31], 6}, {NULL}};
const Segment SGI146726[] = {{&S44580[38], 10}, {NULL}};
const Segment SGI146727[] = {{&S44580[31], 6}, {NULL}};
const Segment SGI146728[] = {{&S44581[38], 10}, {NULL}};
const Segment SGI146729[] = {{&S44581[31], 6}, {NULL}};
const Segment SGI146730[] = {{&S44582[44], 17}, {NULL}};
const Segment SGI146731[] = {{&S44582[44], 29}, {NULL}};
const Segment SGI146732[] = {{&S44582[30], 6}, {NULL}};
const Segment SGI146733[] = {{&S44582[31], 3}, {NULL}};
const Segment SGI146734[] = {{&S44582[87], 6}, {NULL}};
const Segment SGI146735[] = {{&S44614[44], 18}, {NULL}};
const Segment SGI146736[] = {{&S44614[44], 30}, {NULL}};
const Segment SGI146737[] = {{&S44614[30], 6}, {NULL}};
const Segment SGI146738[] = {{&S44614[31], 3}, {NULL}};
const Segment SGI146739[] = {{&S44614[88], 6}, {NULL}};
const Segment SGI146740[] = {{&S44615[38], 18}, {NULL}};
const Segment SGI146741[] = {{&S44615[31], 6}, {NULL}};
const Segment SGI146742[] = {{&S35361[44], 34}, {NULL}};
const Segment SGI146743[] = {{&S35361[44], 46}, {NULL}};
const Segment SGI146744[] = {{&S35361[30], 6}, {NULL}};
const Segment SGI146745[] = {{&S35361[139], 3}, {NULL}};
const Segment SGI146746[] = {{&S35361[104], 6}, {NULL}};
const Segment SGI146747[] = {{&S35255[44], 7}, {NULL}};
const Segment SGI146748[] = {{&S35255[44], 34}, {NULL}};
const Segment SGI146749[] = {{&S35255[30], 6}, {NULL}};
const Segment SGI146750[] = {{&S35255[127], 3}, {NULL}};
const Segment SGI146751[] = {{&S35255[92], 6}, {NULL}};
const Segment SGI146752[] = {{&S35257[44], 7}, {NULL}};
const Segment SGI146753[] = {{&S35257[44], 36}, {NULL}};
const Segment SGI146754[] = {{&S35257[30], 6}, {NULL}};
const Segment SGI146755[] = {{&S35257[129], 3}, {NULL}};
const Segment SGI146756[] = {{&S35257[94], 6}, {NULL}};
const Segment SGI146757[] = {{&S35258[44], 15}, {NULL}};
const Segment SGI146758[] = {{&S35258[44], 39}, {NULL}};
const Segment SGI146759[] = {{&S35258[30], 6}, {NULL}};
const Segment SGI146760[] = {{&S35258[132], 3}, {NULL}};
const Segment SGI146761[] = {{&S35258[97], 6}, {NULL}};
const Segment SGI146762[] = {{&S35259[38], 15}, {NULL}};
const Segment SGI146763[] = {{&S35259[31], 6}, {NULL}};
const Segment SGI146764[] = {{&S35260[44], 7}, {NULL}};
const Segment SGI146765[] = {{&S35260[44], 37}, {NULL}};
const Segment SGI146766[] = {{&S35260[30], 6}, {NULL}};
const Segment SGI146767[] = {{&S35260[130], 3}, {NULL}};
const Segment SGI146768[] = {{&S35260[95], 6}, {NULL}};
const Segment SGI146769[] = {{&S35261[38], 7}, {NULL}};
const Segment SGI146770[] = {{&S35261[31], 6}, {NULL}};
const Segment SGI146771[] = {{&S35263[44], 7}, {NULL}};
const Segment SGI146772[] = {{&S35263[44], 34}, {NULL}};
const Segment SGI146773[] = {{&S35263[30], 6}, {NULL}};
const Segment SGI146774[] = {{&S35263[127], 3}, {NULL}};
const Segment SGI146775[] = {{&S35263[92], 6}, {NULL}};
const Segment SGI146776[] = {{&S35264[44], 7}, {NULL}};
const Segment SGI146777[] = {{&S35264[44], 34}, {NULL}};
const Segment SGI146778[] = {{&S35264[30], 6}, {NULL}};
const Segment SGI146779[] = {{&S35264[127], 3}, {NULL}};
const Segment SGI146780[] = {{&S35264[92], 6}, {NULL}};
const Segment SGI146781[] = {{&S35265[44], 7}, {NULL}};
const Segment SGI146782[] = {{&S35265[44], 36}, {NULL}};
const Segment SGI146783[] = {{&S35265[30], 6}, {NULL}};
const Segment SGI146784[] = {{&S35265[129], 3}, {NULL}};
const Segment SGI146785[] = {{&S35265[94], 6}, {NULL}};
const Segment SGI146786[] = {{&S35266[42], 15}, {NULL}};
const Segment SGI146787[] = {{&S35266[30], 6}, {NULL}};
const Segment SGI146788[] = {{&S35266[130], 3}, {NULL}};
const Segment SGI146789[] = {{&S35266[95], 6}, {NULL}};
const Segment SGI146790[] = {{&S35267[44], 15}, {NULL}};
const Segment SGI146791[] = {{&S35267[44], 39}, {NULL}};
const Segment SGI146792[] = {{&S35267[30], 6}, {NULL}};
const Segment SGI146793[] = {{&S35267[132], 3}, {NULL}};
const Segment SGI146794[] = {{&S35267[97], 6}, {NULL}};
const Segment SGI146795[] = {{&S35268[44], 7}, {NULL}};
const Segment SGI146796[] = {{&S35268[44], 35}, {NULL}};
const Segment SGI146797[] = {{&S35268[30], 6}, {NULL}};
const Segment SGI146798[] = {{&S35268[128], 3}, {NULL}};
const Segment SGI146799[] = {{&S35268[93], 6}, {NULL}};
const Segment SGI146800[] = {{&S35269[44], 7}, {NULL}};
const Segment SGI146801[] = {{&S35269[44], 36}, {NULL}};
const Segment SGI146802[] = {{&S35269[30], 6}, {NULL}};
const Segment SGI146803[] = {{&S35269[129], 3}, {NULL}};
const Segment SGI146804[] = {{&S35269[94], 6}, {NULL}};
const Segment SGI146805[] = {{&S35271[44], 15}, {NULL}};
const Segment SGI146806[] = {{&S35271[44], 33}, {NULL}};
const Segment SGI146807[] = {{&S35271[30], 6}, {NULL}};
const Segment SGI146808[] = {{&S35271[126], 3}, {NULL}};
const Segment SGI146809[] = {{&S35271[91], 6}, {NULL}};
const Segment SGI146810[] = {{&S35272[44], 15}, {NULL}};
const Segment SGI146811[] = {{&S35272[44], 30}, {NULL}};
const Segment SGI146812[] = {{&S35272[30], 6}, {NULL}};
const Segment SGI146813[] = {{&S35272[123], 3}, {NULL}};
const Segment SGI146814[] = {{&S35272[88], 6}, {NULL}};
const Segment SGI146815[] = {{&S35273[44], 15}, {NULL}};
const Segment SGI146816[] = {{&S35273[44], 32}, {NULL}};
const Segment SGI146817[] = {{&S35273[30], 6}, {NULL}};
const Segment SGI146818[] = {{&S35273[125], 3}, {NULL}};
const Segment SGI146819[] = {{&S35273[90], 6}, {NULL}};
const Segment SGI146820[] = {{&S35274[44], 7}, {NULL}};
const Segment SGI146821[] = {{&S35274[44], 36}, {NULL}};
const Segment SGI146822[] = {{&S35274[30], 6}, {NULL}};
const Segment SGI146823[] = {{&S35274[129], 3}, {NULL}};
const Segment SGI146824[] = {{&S35274[94], 6}, {NULL}};
const Segment SGI146825[] = {{&S35275[37], 23}, {NULL}};
const Segment SGI146826[] = {{&S39245[42], 9}, {NULL}};
const Segment SGI146827[] = {{&S39245[42], 21}, {NULL}};
const Segment SGI146828[] = {{&S39245[30], 4}, {NULL}};
const Segment SGI146829[] = {{&S39245[112], 3}, {NULL}};
const Segment SGI146830[] = {{&S39245[77], 6}, {NULL}};
const Segment SGI146831[] = {{&S39249[42], 9}, {NULL}};
const Segment SGI146832[] = {{&S39249[42], 21}, {NULL}};
const Segment SGI146833[] = {{&S39249[30], 4}, {NULL}};
const Segment SGI146834[] = {{&S44741[44], 10}, {NULL}};
const Segment SGI146835[] = {{&S44741[44], 22}, {NULL}};
const Segment SGI146836[] = {{&S44741[30], 6}, {NULL}};
const Segment SGI146837[] = {{&S39249[112], 3}, {NULL}};
const Segment SGI146838[] = {{&S39249[77], 6}, {NULL}};
const Segment SGI146839[] = {{&S44741[31], 3}, {NULL}};
const Segment SGI146840[] = {{&S44741[80], 6}, {NULL}};
const Segment SGI146841[] = {{&S44742[38], 10}, {NULL}};
const Segment SGI146842[] = {{&S44742[31], 6}, {NULL}};
const Segment SGI146843[] = {{&S32944[32], 14}, {NULL}};
const Segment SGI146844[] = {{&S32944[27], 4}, {NULL}};
const Segment SGI146845[] = {{&S39252[36], 9}, {NULL}};
const Segment SGI146846[] = {{&S32944[99], 9}, {NULL}};
const Segment SGI146847[] = {{&S39252[31], 4}, {NULL}};
const Segment SGI146848[] = {{&S32944[72], 6}, {NULL}};
const Segment SGI146849[] = {{&S44747[44], 12}, {NULL}};
const Segment SGI146850[] = {{&S44747[44], 24}, {NULL}};
const Segment SGI146851[] = {{&S44747[30], 6}, {NULL}};
const Segment SGI146852[] = {{&S44747[31], 3}, {NULL}};
const Segment SGI146853[] = {{&S44747[82], 6}, {NULL}};
const Segment SGI146854[] = {{&S39294[42], 9}, {NULL}};
const Segment SGI146855[] = {{&S39294[42], 20}, {NULL}};
const Segment SGI146856[] = {{&S39294[30], 4}, {NULL}};
const Segment SGI146857[] = {{&S39294[111], 3}, {NULL}};
const Segment SGI146858[] = {{&S39294[76], 6}, {NULL}};
const Segment SGI146859[] = {{&S44785[44], 7}, {NULL}};
const Segment SGI146860[] = {{&S44785[44], 18}, {NULL}};
const Segment SGI146861[] = {{&S44785[30], 6}, {NULL}};
const Segment SGI146862[] = {{&S44785[31], 3}, {NULL}};
const Segment SGI146863[] = {{&S44785[76], 6}, {NULL}};
const Segment SGI146864[] = {{&S44786[38], 7}, {NULL}};
const Segment SGI146865[] = {{&S44786[31], 6}, {NULL}};
const Segment SGI146866[] = {{&S39298[42], 9}, {NULL}};
const Segment SGI146867[] = {{&S39298[42], 21}, {NULL}};
const Segment SGI146868[] = {{&S39298[30], 4}, {NULL}};
const Segment SGI146869[] = {{&S39298[80], 6}, {NULL}};
const Segment SGI146870[] = {{&S39312[42], 9}, {NULL}};
const Segment SGI146871[] = {{&S39312[42], 21}, {NULL}};
const Segment SGI146872[] = {{&S39312[30], 4}, {NULL}};
const Segment SGI146873[] = {{&S39312[112], 3}, {NULL}};
const Segment SGI146874[] = {{&S39312[77], 6}, {NULL}};
const Segment SGI146875[] = {{&S44853[44], 9}, {NULL}};
const Segment SGI146876[] = {{&S44853[44], 20}, {NULL}};
const Segment SGI146877[] = {{&S44853[30], 6}, {NULL}};
const Segment SGI146878[] = {{&S44853[31], 3}, {NULL}};
const Segment SGI146879[] = {{&S44853[78], 6}, {NULL}};
const Segment SGI146880[] = {{&S44854[44], 9}, {NULL}};
const Segment SGI146881[] = {{&S44854[44], 21}, {NULL}};
const Segment SGI146882[] = {{&S44854[30], 6}, {NULL}};
const Segment SGI146883[] = {{&S44854[31], 3}, {NULL}};
const Segment SGI146884[] = {{&S44854[79], 6}, {NULL}};
const Segment SGI146885[] = {{&S44855[38], 9}, {NULL}};
const Segment SGI146886[] = {{&S44855[31], 6}, {NULL}};
const Segment SGI146887[] = {{&S44856[44], 17}, {NULL}};
const Segment SGI146888[] = {{&S44856[44], 29}, {NULL}};
const Segment SGI146889[] = {{&S44856[30], 6}, {NULL}};
const Segment SGI146890[] = {{&S44856[31], 3}, {NULL}};
const Segment SGI146891[] = {{&S44856[87], 6}, {NULL}};
const Segment SGI146892[] = {{&S44857[44], 11}, {NULL}};
const Segment SGI146893[] = {{&S44857[44], 23}, {NULL}};
const Segment SGI146894[] = {{&S44857[30], 6}, {NULL}};
const Segment SGI146895[] = {{&S44857[31], 3}, {NULL}};
const Segment SGI146896[] = {{&S44857[81], 6}, {NULL}};
const Segment SGI146897[] = {{&S44858[41], 9}, {NULL}};
const Segment SGI146898[] = {{&S44858[30], 6}, {NULL}};
const Segment SGI146899[] = {{&S44858[31], 3}, {NULL}};
const Segment SGI146900[] = {{&S44858[76], 6}, {NULL}};
const Segment SGI146901[] = {{&S44859[44], 17}, {NULL}};
const Segment SGI146902[] = {{&S44859[44], 29}, {NULL}};
const Segment SGI146903[] = {{&S44859[30], 6}, {NULL}};
const Segment SGI146904[] = {{&S44859[31], 3}, {NULL}};
const Segment SGI146905[] = {{&S44859[87], 6}, {NULL}};
const Segment SGI146906[] = {{&S35444[44], 10}, {NULL}};
const Segment SGI146907[] = {{&S35444[44], 22}, {NULL}};
const Segment SGI146908[] = {{&S35444[30], 6}, {NULL}};
const Segment SGI146909[] = {{&S35444[115], 3}, {NULL}};
const Segment SGI146910[] = {{&S35444[80], 6}, {NULL}};
const Segment SGI146911[] = {{&S35445[38], 10}, {NULL}};
const Segment SGI146912[] = {{&S35445[31], 6}, {NULL}};
const Segment SGI146913[] = {{&S45000[44], 10}, {NULL}};
const Segment SGI146914[] = {{&S45000[44], 22}, {NULL}};
const Segment SGI146915[] = {{&S45000[30], 6}, {NULL}};
const Segment SGI146916[] = {{&S45000[31], 3}, {NULL}};
const Segment SGI146917[] = {{&S45000[80], 6}, {NULL}};
const Segment SGI146918[] = {{&S45001[38], 10}, {NULL}};
const Segment SGI146919[] = {{&S45001[31], 6}, {NULL}};
const Segment SGI146920[] = {{&S45025[44], 11}, {NULL}};
const Segment SGI146921[] = {{&S45025[44], 23}, {NULL}};
const Segment SGI146922[] = {{&S45025[30], 6}, {NULL}};
const Segment SGI146923[] = {{&S45025[31], 3}, {NULL}};
const Segment SGI146924[] = {{&S45025[81], 6}, {NULL}};
const Segment SGI146925[] = {{&S45026[38], 11}, {NULL}};
const Segment SGI146926[] = {{&S45026[31], 6}, {NULL}};
const Segment SGI146927[] = {{&S45027[42], 11}, {NULL}};
const Segment SGI146928[] = {{&S45027[30], 6}, {NULL}};
const Segment SGI146929[] = {{&S45027[31], 3}, {NULL}};
const Segment SGI146930[] = {{&S45027[79], 6}, {NULL}};
const Segment SGI146931[] = {{&S45052[34], 9}, {NULL}};
const Segment SGI146932[] = {{&S45052[27], 6}, {NULL}};
const Segment SGI146933[] = {{&S45052[96], 9}, {NULL}};
const Segment SGI146934[] = {{&S45052[69], 6}, {NULL}};
const Segment SGI146935[] = {{&S45144[34], 7}, {NULL}};
const Segment SGI146936[] = {{&S45144[27], 6}, {NULL}};
const Segment SGI146937[] = {{&S45144[100], 9}, {NULL}};
const Segment SGI146938[] = {{&S45144[73], 6}, {NULL}};
const Segment SGI146939[] = {{&S45145[34], 7}, {NULL}};
const Segment SGI146940[] = {{&S45145[27], 6}, {NULL}};
const Segment SGI146941[] = {{&S45145[100], 9}, {NULL}};
const Segment SGI146942[] = {{&S45145[73], 6}, {NULL}};
const Segment SGI146943[] = {{&S45146[34], 10}, {NULL}};
const Segment SGI146944[] = {{&S45146[27], 6}, {NULL}};
const Segment SGI146945[] = {{&S45146[97], 9}, {NULL}};
const Segment SGI146946[] = {{&S45146[70], 6}, {NULL}};
const Segment SGI146947[] = {{&S45147[34], 9}, {NULL}};
const Segment SGI146948[] = {{&S45147[27], 6}, {NULL}};
const Segment SGI146949[] = {{&S45147[96], 9}, {NULL}};
const Segment SGI146950[] = {{&S45147[69], 6}, {NULL}};
const Segment SGI146951[] = {{&S30637[44], 12}, {NULL}};
const Segment SGI146952[] = {{&S30637[44], 27}, {NULL}};
const Segment SGI146953[] = {{&S30637[30], 6}, {NULL}};
const Segment SGI146954[] = {{&S30637[119], 3}, {NULL}};
const Segment SGI146955[] = {{&S30637[85], 5}, {NULL}};
const Segment SGI146956[] = {{&S30638[44], 6}, {NULL}};
const Segment SGI146957[] = {{&S30638[44], 18}, {NULL}};
const Segment SGI146958[] = {{&S30638[30], 6}, {NULL}};
const Segment SGI146959[] = {{&S30638[111], 3}, {NULL}};
const Segment SGI146960[] = {{&S30638[76], 6}, {NULL}};
const Segment SGI146961[] = {{&S30640[44], 18}, {NULL}};
const Segment SGI146962[] = {{&S30640[44], 30}, {NULL}};
const Segment SGI146963[] = {{&S30640[30], 6}, {NULL}};
const Segment SGI146964[] = {{&S30640[123], 3}, {NULL}};
const Segment SGI146965[] = {{&S30640[88], 6}, {NULL}};
const Segment SGI146966[] = {{&S45191[34], 9}, {NULL}};
const Segment SGI146967[] = {{&S45191[27], 6}, {NULL}};
const Segment SGI146968[] = {{&S45191[96], 9}, {NULL}};
const Segment SGI146969[] = {{&S45191[69], 6}, {NULL}};
const Segment SGI146970[] = {{&S30644[44], 23}, {NULL}};
const Segment SGI146971[] = {{&S30644[44], 35}, {NULL}};
const Segment SGI146972[] = {{&S30644[30], 6}, {NULL}};
const Segment SGI146973[] = {{&S30644[128], 3}, {NULL}};
const Segment SGI146974[] = {{&S30644[93], 6}, {NULL}};
const Segment SGI146975[] = {{&S30645[44], 13}, {NULL}};
const Segment SGI146976[] = {{&S30645[44], 25}, {NULL}};
const Segment SGI146977[] = {{&S30645[30], 6}, {NULL}};
const Segment SGI146978[] = {{&S30645[118], 3}, {NULL}};
const Segment SGI146979[] = {{&S30645[83], 6}, {NULL}};
const Segment SGI146980[] = {{&S30646[44], 12}, {NULL}};
const Segment SGI146981[] = {{&S30646[44], 24}, {NULL}};
const Segment SGI146982[] = {{&S30646[30], 6}, {NULL}};
const Segment SGI146983[] = {{&S30646[117], 3}, {NULL}};
const Segment SGI146984[] = {{&S30646[82], 6}, {NULL}};
const Segment SGI146985[] = {{&S30648[44], 6}, {NULL}};
const Segment SGI146986[] = {{&S30648[44], 18}, {NULL}};
const Segment SGI146987[] = {{&S30648[30], 6}, {NULL}};
const Segment SGI146988[] = {{&S30648[111], 3}, {NULL}};
const Segment SGI146989[] = {{&S30648[76], 6}, {NULL}};
const Segment SGI146990[] = {{&S30650[44], 12}, {NULL}};
const Segment SGI146991[] = {{&S30650[44], 24}, {NULL}};
const Segment SGI146992[] = {{&S30650[30], 6}, {NULL}};
const Segment SGI146993[] = {{&S30650[117], 3}, {NULL}};
const Segment SGI146994[] = {{&S30650[82], 6}, {NULL}};
const Segment SGI146995[] = {{&S30651[44], 4}, {NULL}};
const Segment SGI146996[] = {{&S30651[44], 16}, {NULL}};
const Segment SGI146997[] = {{&S30651[30], 6}, {NULL}};
const Segment SGI146998[] = {{&S30651[109], 3}, {NULL}};
const Segment SGI146999[] = {{&S30651[74], 6}, {NULL}};
const Segment SGI147000[] = {{&S30652[44], 6}, {NULL}};
const Segment SGI147001[] = {{&S30652[44], 18}, {NULL}};
const Segment SGI147002[] = {{&S30652[30], 6}, {NULL}};
const Segment SGI147003[] = {{&S33510[32], 14}, {NULL}};
const Segment SGI147004[] = {{&S33510[27], 4}, {NULL}};
const Segment SGI147005[] = {{&S30652[111], 3}, {NULL}};
const Segment SGI147006[] = {{&S30652[76], 6}, {NULL}};
const Segment SGI147007[] = {{&S33510[99], 9}, {NULL}};
const Segment SGI147008[] = {{&S33510[72], 6}, {NULL}};
const Segment SGI147009[] = {{&S30653[44], 15}, {NULL}};
const Segment SGI147010[] = {{&S30653[44], 27}, {NULL}};
const Segment SGI147011[] = {{&S30653[30], 6}, {NULL}};
const Segment SGI147012[] = {{&S30653[120], 3}, {NULL}};
const Segment SGI147013[] = {{&S30653[85], 6}, {NULL}};
const Segment SGI147014[] = {{&S30656[44], 16}, {NULL}};
const Segment SGI147015[] = {{&S30656[44], 28}, {NULL}};
const Segment SGI147016[] = {{&S30656[30], 6}, {&S30656[52], 20}, {NULL}};
const Segment SGI147017[] = {{&S30656[121], 3}, {NULL}};
const Segment SGI147018[] = {{&S30656[86], 6}, {NULL}};
const Segment SGI147019[] = {{&S30657[44], 6}, {NULL}};
const Segment SGI147020[] = {{&S30657[44], 18}, {NULL}};
const Segment SGI147021[] = {{&S30657[30], 6}, {NULL}};
const Segment SGI147022[] = {{&S30657[111], 3}, {NULL}};
const Segment SGI147023[] = {{&S30657[76], 6}, {NULL}};
const Segment SGI147024[] = {{&S30659[44], 6}, {NULL}};
const Segment SGI147025[] = {{&S30659[44], 18}, {NULL}};
const Segment SGI147026[] = {{&S30659[30], 6}, {NULL}};
const Segment SGI147027[] = {{&S30659[111], 3}, {NULL}};
const Segment SGI147028[] = {{&S30659[76], 6}, {NULL}};
const Segment SGI147029[] = {{&S30660[80], 6}, {NULL}};
const Segment SGI147030[] = {{&S30661[44], 6}, {NULL}};
const Segment SGI147031[] = {{&S30661[44], 18}, {NULL}};
const Segment SGI147032[] = {{&S30661[30], 6}, {NULL}};
const Segment SGI147033[] = {{&S30661[111], 3}, {NULL}};
const Segment SGI147034[] = {{&S30661[76], 6}, {NULL}};
const Segment SGI147035[] = {{&S30663[44], 13}, {NULL}};
const Segment SGI147036[] = {{&S30663[44], 25}, {NULL}};
const Segment SGI147037[] = {{&S30663[30], 6}, {NULL}};
const Segment SGI147038[] = {{&S30663[118], 3}, {NULL}};
const Segment SGI147039[] = {{&S30663[83], 6}, {NULL}};
const Segment SGI147040[] = {{&S30665[44], 6}, {NULL}};
const Segment SGI147041[] = {{&S30665[44], 18}, {NULL}};
const Segment SGI147042[] = {{&S30665[30], 6}, {NULL}};
const Segment SGI147043[] = {{&S30665[111], 3}, {NULL}};
const Segment SGI147044[] = {{&S30665[76], 6}, {NULL}};
const Segment SGI147045[] = {{&S30667[44], 9}, {NULL}};
const Segment SGI147046[] = {{&S30667[44], 21}, {NULL}};
const Segment SGI147047[] = {{&S30667[30], 6}, {NULL}};
const Segment SGI147048[] = {{&S30667[114], 3}, {NULL}};
const Segment SGI147049[] = {{&S30667[79], 6}, {NULL}};
const Segment SGI147050[] = {{&S30668[44], 6}, {NULL}};
const Segment SGI147051[] = {{&S30668[44], 18}, {NULL}};
const Segment SGI147052[] = {{&S30668[30], 6}, {NULL}};
const Segment SGI147053[] = {{&S30668[111], 3}, {NULL}};
const Segment SGI147054[] = {{&S30668[76], 6}, {NULL}};
const Segment SGI147055[] = {{&S30670[44], 13}, {NULL}};
const Segment SGI147056[] = {{&S30670[44], 25}, {NULL}};
const Segment SGI147057[] = {{&S30670[30], 6}, {NULL}};
const Segment SGI147058[] = {{&S30670[118], 3}, {NULL}};
const Segment SGI147059[] = {{&S30670[83], 6}, {NULL}};
const Segment SGI147060[] = {{&S30671[43], 11}, {NULL}};
const Segment SGI147061[] = {{&S30671[43], 23}, {NULL}};
const Segment SGI147062[] = {{&S30671[30], 6}, {NULL}};
const Segment SGI147063[] = {{&S30671[115], 3}, {NULL}};
const Segment SGI147064[] = {{&S30671[80], 6}, {NULL}};
const Segment SGI147065[] = {{&S30672[44], 11}, {NULL}};
const Segment SGI147066[] = {{&S30672[44], 23}, {NULL}};
const Segment SGI147067[] = {{&S30672[30], 6}, {NULL}};
const Segment SGI147068[] = {{&S30672[116], 3}, {NULL}};
const Segment SGI147069[] = {{&S30672[81], 6}, {NULL}};
const Segment SGI147070[] = {{&S30673[44], 7}, {NULL}};
const Segment SGI147071[] = {{&S30673[44], 19}, {NULL}};
const Segment SGI147072[] = {{&S30673[30], 6}, {NULL}};
const Segment SGI147073[] = {{&S30673[111], 3}, {NULL}};
const Segment SGI147074[] = {{&S30673[77], 5}, {NULL}};
const Segment SGI147075[] = {{&S30674[44], 6}, {NULL}};
const Segment SGI147076[] = {{&S30674[44], 18}, {NULL}};
const Segment SGI147077[] = {{&S30674[30], 6}, {NULL}};
const Segment SGI147078[] = {{&S30674[111], 3}, {NULL}};
const Segment SGI147079[] = {{&S30674[76], 6}, {NULL}};
const Segment SGI147080[] = {{&S30676[44], 13}, {NULL}};
const Segment SGI147081[] = {{&S30676[44], 25}, {NULL}};
const Segment SGI147082[] = {{&S30676[30], 6}, {NULL}};
const Segment SGI147083[] = {{&S30676[118], 3}, {NULL}};
const Segment SGI147084[] = {{&S30676[83], 6}, {NULL}};
const Segment SGI147085[] = {{&S30677[44], 4}, {NULL}};
const Segment SGI147086[] = {{&S30677[44], 16}, {NULL}};
const Segment SGI147087[] = {{&S30677[30], 6}, {NULL}};
const Segment SGI147088[] = {{&S30677[109], 3}, {NULL}};
const Segment SGI147089[] = {{&S30677[74], 6}, {NULL}};
const Segment SGI147090[] = {{&S30678[44], 13}, {NULL}};
const Segment SGI147091[] = {{&S30678[44], 25}, {NULL}};
const Segment SGI147092[] = {{&S30678[30], 6}, {NULL}};
const Segment SGI147093[] = {{&S30678[118], 3}, {NULL}};
const Segment SGI147094[] = {{&S30678[83], 6}, {NULL}};
const Segment SGI147095[] = {{&S30679[44], 14}, {NULL}};
const Segment SGI147096[] = {{&S30679[44], 26}, {NULL}};
const Segment SGI147097[] = {{&S30679[30], 6}, {NULL}};
const Segment SGI147098[] = {{&S30679[119], 3}, {NULL}};
const Segment SGI147099[] = {{&S30679[84], 6}, {NULL}};
const Segment SGI147100[] = {{&S30682[44], 19}, {NULL}};
const Segment SGI147101[] = {{&S30682[30], 6}, {NULL}};
const Segment SGI147102[] = {{&S30682[126], 3}, {NULL}};
const Segment SGI147103[] = {{&S30682[91], 6}, {NULL}};
const Segment SGI147104[] = {{&S30683[44], 11}, {NULL}};
const Segment SGI147105[] = {{&S30683[44], 23}, {NULL}};
const Segment SGI147106[] = {{&S30683[30], 6}, {NULL}};
const Segment SGI147107[] = {{&S30683[116], 3}, {NULL}};
const Segment SGI147108[] = {{&S30683[81], 6}, {NULL}};
const Segment SGI147109[] = {{&S30684[44], 7}, {NULL}};
const Segment SGI147110[] = {{&S30684[44], 19}, {NULL}};
const Segment SGI147111[] = {{&S30684[30], 6}, {NULL}};
const Segment SGI147112[] = {{&S30684[112], 3}, {NULL}};
const Segment SGI147113[] = {{&S30684[77], 6}, {NULL}};
const Segment SGI147114[] = {{&S30685[44], 10}, {NULL}};
const Segment SGI147115[] = {{&S30685[44], 22}, {NULL}};
const Segment SGI147116[] = {{&S30685[30], 6}, {NULL}};
const Segment SGI147117[] = {{&S30685[115], 3}, {NULL}};
const Segment SGI147118[] = {{&S30685[80], 6}, {NULL}};
const Segment SGI147119[] = {{&S30686[44], 7}, {NULL}};
const Segment SGI147120[] = {{&S30686[44], 27}, {NULL}};
const Segment SGI147121[] = {{&S30686[30], 6}, {NULL}};
const Segment SGI147122[] = {{&S30686[120], 3}, {NULL}};
const Segment SGI147123[] = {{&S30686[85], 6}, {NULL}};
const Segment SGI147124[] = {{&S30687[44], 20}, {NULL}};
const Segment SGI147125[] = {{&S30687[44], 32}, {NULL}};
const Segment SGI147126[] = {{&S30687[30], 6}, {NULL}};
const Segment SGI147127[] = {{&S30687[125], 3}, {NULL}};
const Segment SGI147128[] = {{&S30687[90], 6}, {NULL}};
const Segment SGI147129[] = {{&S30688[44], 4}, {NULL}};
const Segment SGI147130[] = {{&S30688[44], 16}, {NULL}};
const Segment SGI147131[] = {{&S30688[30], 6}, {NULL}};
const Segment SGI147132[] = {{&S30688[109], 3}, {NULL}};
const Segment SGI147133[] = {{&S30688[74], 6}, {NULL}};
const Segment SGI147134[] = {{&S30689[44], 14}, {NULL}};
const Segment SGI147135[] = {{&S30689[44], 26}, {NULL}};
const Segment SGI147136[] = {{&S30689[30], 6}, {NULL}};
const Segment SGI147137[] = {{&S30689[119], 3}, {NULL}};
const Segment SGI147138[] = {{&S30689[84], 6}, {NULL}};
const Segment SGI147139[] = {{&S30690[44], 10}, {NULL}};
const Segment SGI147140[] = {{&S30690[44], 22}, {NULL}};
const Segment SGI147141[] = {{&S30690[30], 6}, {NULL}};
const Segment SGI147142[] = {{&S30690[115], 3}, {NULL}};
const Segment SGI147143[] = {{&S30690[80], 6}, {NULL}};
const Segment SGI147144[] = {{&S30691[44], 13}, {NULL}};
const Segment SGI147145[] = {{&S30691[44], 31}, {NULL}};
const Segment SGI147146[] = {{&S30691[30], 6}, {NULL}};
const Segment SGI147147[] = {{&S30691[123], 3}, {NULL}};
const Segment SGI147148[] = {{&S30691[89], 5}, {NULL}};
const Segment SGI147149[] = {{&S35982[34], 15}, {NULL}};
const Segment SGI147150[] = {{&S35982[27], 6}, {NULL}};
const Segment SGI147151[] = {{&S35982[105], 9}, {NULL}};
const Segment SGI147152[] = {{&S35982[78], 6}, {NULL}};
const Segment SGI147153[] = {{&S35984[34], 15}, {NULL}};
const Segment SGI147154[] = {{&S30695[44], 15}, {NULL}};
const Segment SGI147155[] = {{&S35984[27], 6}, {NULL}};
const Segment SGI147156[] = {{&S30695[44], 27}, {NULL}};
const Segment SGI147157[] = {{&S30695[30], 6}, {NULL}};
const Segment SGI147158[] = {{&S35984[105], 9}, {NULL}};
const Segment SGI147159[] = {{&S30695[120], 3}, {NULL}};
const Segment SGI147160[] = {{&S35984[78], 6}, {NULL}};
const Segment SGI147161[] = {{&S30695[85], 6}, {NULL}};
const Segment SGI147162[] = {{&S30696[44], 17}, {NULL}};
const Segment SGI147163[] = {{&S30696[44], 29}, {NULL}};
const Segment SGI147164[] = {{&S30696[30], 6}, {NULL}};
const Segment SGI147165[] = {{&S30696[122], 3}, {NULL}};
const Segment SGI147166[] = {{&S30696[87], 6}, {NULL}};
const Segment SGI147167[] = {{&S30698[44], 23}, {NULL}};
const Segment SGI147168[] = {{&S30698[44], 35}, {NULL}};
const Segment SGI147169[] = {{&S30698[30], 6}, {NULL}};
const Segment SGI147170[] = {{&S30698[128], 3}, {NULL}};
const Segment SGI147171[] = {{&S30698[93], 6}, {NULL}};
const Segment SGI147172[] = {{&S35989[34], 7}, {NULL}};
const Segment SGI147173[] = {{&S35989[27], 6}, {NULL}};
const Segment SGI147174[] = {{&S35989[110], 9}, {NULL}};
const Segment SGI147175[] = {{&S35989[83], 6}, {NULL}};
const Segment SGI147176[] = {{&S30700[44], 9}, {NULL}};
const Segment SGI147177[] = {{&S30700[44], 21}, {NULL}};
const Segment SGI147178[] = {{&S30700[30], 6}, {NULL}};
const Segment SGI147179[] = {{&S30700[114], 3}, {NULL}};
const Segment SGI147180[] = {{&S30700[79], 6}, {NULL}};
const Segment SGI147181[] = {{&S30701[44], 6}, {NULL}};
const Segment SGI147182[] = {{&S30701[44], 18}, {NULL}};
const Segment SGI147183[] = {{&S30701[30], 6}, {NULL}};
const Segment SGI147184[] = {{&S30701[111], 3}, {NULL}};
const Segment SGI147185[] = {{&S30701[76], 6}, {NULL}};
const Segment SGI147186[] = {{&S30703[44], 7}, {NULL}};
const Segment SGI147187[] = {{&S30703[44], 19}, {NULL}};
const Segment SGI147188[] = {{&S30703[30], 6}, {NULL}};
const Segment SGI147189[] = {{&S30703[112], 3}, {NULL}};
const Segment SGI147190[] = {{&S30703[77], 6}, {NULL}};
const Segment SGI147191[] = {{&S30704[44], 10}, {NULL}};
const Segment SGI147192[] = {{&S30704[44], 22}, {NULL}};
const Segment SGI147193[] = {{&S30704[30], 6}, {NULL}};
const Segment SGI147194[] = {{&S30704[115], 3}, {NULL}};
const Segment SGI147195[] = {{&S30704[80], 6}, {NULL}};
const Segment SGI147196[] = {{&S30706[44], 15}, {NULL}};
const Segment SGI147197[] = {{&S30706[44], 27}, {NULL}};
const Segment SGI147198[] = {{&S30706[30], 6}, {NULL}};
const Segment SGI147199[] = {{&S30706[120], 3}, {NULL}};
const Segment SGI147200[] = {{&S30706[85], 6}, {NULL}};
const Segment SGI147201[] = {{&S30708[44], 11}, {NULL}};
const Segment SGI147202[] = {{&S30708[44], 26}, {NULL}};
const Segment SGI147203[] = {{&S30708[30], 6}, {NULL}};
const Segment SGI147204[] = {{&S30708[119], 3}, {NULL}};
const Segment SGI147205[] = {{&S30708[84], 6}, {NULL}};
const Segment SGI147206[] = {{&S30709[44], 7}, {NULL}};
const Segment SGI147207[] = {{&S30709[44], 19}, {NULL}};
const Segment SGI147208[] = {{&S30709[30], 6}, {NULL}};
const Segment SGI147209[] = {{&S30709[112], 3}, {NULL}};
const Segment SGI147210[] = {{&S30709[77], 6}, {NULL}};
const Segment SGI147211[] = {{&S30710[44], 12}, {NULL}};
const Segment SGI147212[] = {{&S30710[44], 24}, {NULL}};
const Segment SGI147213[] = {{&S30710[30], 6}, {NULL}};
const Segment SGI147214[] = {{&S30710[117], 3}, {NULL}};
const Segment SGI147215[] = {{&S30710[82], 6}, {NULL}};
const Segment SGI147216[] = {{&S30711[44], 6}, {NULL}};
const Segment SGI147217[] = {{&S30711[44], 18}, {NULL}};
const Segment SGI147218[] = {{&S30711[30], 6}, {NULL}};
const Segment SGI147219[] = {{&S30711[111], 3}, {NULL}};
const Segment SGI147220[] = {{&S30711[76], 6}, {NULL}};
const Segment SGI147221[] = {{&S30713[44], 17}, {NULL}};
const Segment SGI147222[] = {{&S30713[44], 37}, {NULL}};
const Segment SGI147223[] = {{&S30713[30], 6}, {NULL}};
const Segment SGI147224[] = {{&S30713[130], 3}, {NULL}};
const Segment SGI147225[] = {{&S30713[95], 6}, {NULL}};
const Segment SGI147226[] = {{&S30714[44], 4}, {NULL}};
const Segment SGI147227[] = {{&S30714[44], 16}, {NULL}};
const Segment SGI147228[] = {{&S30714[30], 6}, {NULL}};
const Segment SGI147229[] = {{&S30714[109], 3}, {NULL}};
const Segment SGI147230[] = {{&S30714[74], 6}, {NULL}};
const Segment SGI147231[] = {{&S30715[44], 12}, {NULL}};
const Segment SGI147232[] = {{&S30715[44], 30}, {NULL}};
const Segment SGI147233[] = {{&S30715[30], 6}, {NULL}};
const Segment SGI147234[] = {{&S30715[122], 3}, {NULL}};
const Segment SGI147235[] = {{&S30715[88], 5}, {NULL}};
const Segment SGI147236[] = {{&S30716[44], 12}, {NULL}};
const Segment SGI147237[] = {{&S30716[44], 24}, {NULL}};
const Segment SGI147238[] = {{&S30716[30], 6}, {NULL}};
const Segment SGI147239[] = {{&S30716[117], 3}, {NULL}};
const Segment SGI147240[] = {{&S30716[82], 6}, {NULL}};
const Segment SGI147241[] = {{&S30717[43], 20}, {NULL}};
const Segment SGI147242[] = {{&S30717[30], 6}, {NULL}};
const Segment SGI147243[] = {{&S30717[31], 3}, {NULL}};
const Segment SGI147244[] = {{&S30717[77], 6}, {NULL}};
const Segment SGI147245[] = {{&S45322[44], 15}, {NULL}};
const Segment SGI147246[] = {{&S45322[44], 27}, {NULL}};
const Segment SGI147247[] = {{&S45322[30], 6}, {NULL}};
const Segment SGI147248[] = {{&S45322[31], 3}, {NULL}};
const Segment SGI147249[] = {{&S45322[85], 6}, {NULL}};
const Segment SGI147250[] = {{&S30719[44], 6}, {NULL}};
const Segment SGI147251[] = {{&S30719[44], 18}, {NULL}};
const Segment SGI147252[] = {{&S30719[30], 6}, {NULL}};
const Segment SGI147253[] = {{&S30719[111], 3}, {NULL}};
const Segment SGI147254[] = {{&S45323[44], 15}, {NULL}};
const Segment SGI147255[] = {{&S45323[44], 27}, {NULL}};
const Segment SGI147256[] = {{&S30719[76], 6}, {NULL}};
const Segment SGI147257[] = {{&S45323[30], 6}, {NULL}};
const Segment SGI147258[] = {{&S45323[31], 3}, {NULL}};
const Segment SGI147259[] = {{&S45323[85], 6}, {NULL}};
const Segment SGI147260[] = {{&S30720[44], 5}, {NULL}};
const Segment SGI147261[] = {{&S30720[44], 17}, {NULL}};
const Segment SGI147262[] = {{&S30720[30], 6}, {NULL}};
const Segment SGI147263[] = {{&S30720[110], 3}, {NULL}};
const Segment SGI147264[] = {{&S30720[75], 6}, {NULL}};
const Segment SGI147265[] = {{&S45324[44], 15}, {NULL}};
const Segment SGI147266[] = {{&S45324[44], 27}, {NULL}};
const Segment SGI147267[] = {{&S45324[30], 6}, {NULL}};
const Segment SGI147268[] = {{&S45324[31], 3}, {NULL}};
const Segment SGI147269[] = {{&S45324[85], 6}, {NULL}};
const Segment SGI147270[] = {{&S30721[43], 23}, {NULL}};
const Segment SGI147271[] = {{&S30721[30], 6}, {NULL}};
const Segment SGI147272[] = {{&S30721[31], 3}, {NULL}};
const Segment SGI147273[] = {{&S30721[80], 6}, {NULL}};
const Segment SGI147274[] = {{&S45325[44], 11}, {NULL}};
const Segment SGI147275[] = {{&S45325[44], 23}, {NULL}};
const Segment SGI147276[] = {{&S45325[30], 6}, {NULL}};
const Segment SGI147277[] = {{&S45325[31], 3}, {NULL}};
const Segment SGI147278[] = {{&S30722[44], 6}, {NULL}};
const Segment SGI147279[] = {{&S45325[81], 6}, {NULL}};
const Segment SGI147280[] = {{&S30722[44], 18}, {NULL}};
const Segment SGI147281[] = {{&S30722[30], 6}, {NULL}};
const Segment SGI147282[] = {{&S30722[111], 3}, {NULL}};
const Segment SGI147283[] = {{&S30722[76], 6}, {NULL}};
const Segment SGI147284[] = {{&S45326[44], 11}, {NULL}};
const Segment SGI147285[] = {{&S45326[44], 23}, {NULL}};
const Segment SGI147286[] = {{&S45326[30], 6}, {NULL}};
const Segment SGI147287[] = {{&S30723[44], 11}, {NULL}};
const Segment SGI147288[] = {{&S30723[44], 23}, {NULL}};
const Segment SGI147289[] = {{&S45326[31], 3}, {NULL}};
const Segment SGI147290[] = {{&S30723[30], 6}, {NULL}};
const Segment SGI147291[] = {{&S45326[81], 6}, {NULL}};
const Segment SGI147292[] = {{&S30723[116], 3}, {NULL}};
const Segment SGI147293[] = {{&S30723[81], 6}, {NULL}};
const Segment SGI147294[] = {{&S45327[44], 14}, {NULL}};
const Segment SGI147295[] = {{&S30724[44], 8}, {NULL}};
const Segment SGI147296[] = {{&S45327[44], 26}, {NULL}};
const Segment SGI147297[] = {{&S30724[44], 20}, {NULL}};
const Segment SGI147298[] = {{&S45327[30], 6}, {NULL}};
const Segment SGI147299[] = {{&S30724[30], 6}, {NULL}};
const Segment SGI147300[] = {{&S45327[31], 3}, {NULL}};
const Segment SGI147301[] = {{&S30724[81], 6}, {NULL}};
const Segment SGI147302[] = {{&S45327[84], 6}, {NULL}};
const Segment SGI147303[] = {{&S30725[44], 7}, {NULL}};
const Segment SGI147304[] = {{&S30725[44], 19}, {NULL}};
const Segment SGI147305[] = {{&S30725[30], 6}, {NULL}};
const Segment SGI147306[] = {{&S45328[38], 15}, {NULL}};
const Segment SGI147307[] = {{&S45328[31], 6}, {NULL}};
const Segment SGI147308[] = {{&S30725[112], 3}, {NULL}};
const Segment SGI147309[] = {{&S30725[77], 6}, {NULL}};
const Segment SGI147310[] = {{&S30726[44], 9}, {NULL}};
const Segment SGI147311[] = {{&S30726[44], 21}, {NULL}};
const Segment SGI147312[] = {{&S30726[30], 6}, {NULL}};
const Segment SGI147313[] = {{&S45329[38], 11}, {NULL}};
const Segment SGI147314[] = {{&S45329[31], 6}, {NULL}};
const Segment SGI147315[] = {{&S30726[114], 3}, {NULL}};
const Segment SGI147316[] = {{&S30726[79], 6}, {NULL}};
const Segment SGI147317[] = {{&S30727[44], 17}, {NULL}};
const Segment SGI147318[] = {{&S30727[44], 29}, {NULL}};
const Segment SGI147319[] = {{&S30727[30], 6}, {NULL}};
const Segment SGI147320[] = {{&S30727[122], 3}, {NULL}};
const Segment SGI147321[] = {{&S30727[87], 6}, {NULL}};
const Segment SGI147322[] = {{&S30728[44], 12}, {NULL}};
const Segment SGI147323[] = {{&S30728[44], 24}, {NULL}};
const Segment SGI147324[] = {{&S30728[30], 6}, {NULL}};
const Segment SGI147325[] = {{&S30728[117], 3}, {NULL}};
const Segment SGI147326[] = {{&S30728[82], 6}, {NULL}};
const Segment SGI147327[] = {{&S30729[44], 13}, {NULL}};
const Segment SGI147328[] = {{&S30729[44], 25}, {NULL}};
const Segment SGI147329[] = {{&S30729[30], 6}, {NULL}};
const Segment SGI147330[] = {{&S30729[118], 3}, {NULL}};
const Segment SGI147331[] = {{&S30729[83], 6}, {NULL}};
const Segment SGI147332[] = {{&S30731[43], 23}, {NULL}};
const Segment SGI147333[] = {{&S30731[30], 6}, {NULL}};
const Segment SGI147334[] = {{&S30731[31], 3}, {NULL}};
const Segment SGI147335[] = {{&S30731[80], 6}, {NULL}};
const Segment SGI147336[] = {{&S30732[44], 22}, {NULL}};
const Segment SGI147337[] = {{&S30732[44], 34}, {NULL}};
const Segment SGI147338[] = {{&S30732[30], 6}, {NULL}};
const Segment SGI147339[] = {{&S30732[127], 3}, {NULL}};
const Segment SGI147340[] = {{&S30732[92], 6}, {NULL}};
const Segment SGI147341[] = {{&S30733[44], 7}, {NULL}};
const Segment SGI147342[] = {{&S30733[44], 19}, {NULL}};
const Segment SGI147343[] = {{&S30733[30], 6}, {NULL}};
const Segment SGI147344[] = {{&S30733[112], 3}, {NULL}};
const Segment SGI147345[] = {{&S30733[77], 6}, {NULL}};
const Segment SGI147346[] = {{&S30736[43], 19}, {NULL}};
const Segment SGI147347[] = {{&S30736[30], 6}, {NULL}};
const Segment SGI147348[] = {{&S30736[31], 3}, {NULL}};
const Segment SGI147349[] = {{&S30736[76], 6}, {NULL}};
const Segment SGI147350[] = {{&S30737[44], 9}, {NULL}};
const Segment SGI147351[] = {{&S30737[44], 21}, {NULL}};
const Segment SGI147352[] = {{&S30737[30], 6}, {NULL}};
const Segment SGI147353[] = {{&S30737[114], 3}, {NULL}};
const Segment SGI147354[] = {{&S30737[79], 6}, {NULL}};
const Segment SGI147355[] = {{&S30738[44], 5}, {NULL}};
const Segment SGI147356[] = {{&S30738[44], 17}, {NULL}};
const Segment SGI147357[] = {{&S30738[30], 6}, {NULL}};
const Segment SGI147358[] = {{&S30738[110], 3}, {NULL}};
const Segment SGI147359[] = {{&S30738[75], 6}, {NULL}};
const Segment SGI147360[] = {{&S30739[44], 8}, {NULL}};
const Segment SGI147361[] = {{&S30739[44], 20}, {NULL}};
const Segment SGI147362[] = {{&S33697[32], 9}, {NULL}};
const Segment SGI147363[] = {{&S30739[30], 6}, {NULL}};
const Segment SGI147364[] = {{&S33697[27], 4}, {NULL}};
const Segment SGI147365[] = {{&S30739[113], 3}, {NULL}};
const Segment SGI147366[] = {{&S30739[78], 6}, {NULL}};
const Segment SGI147367[] = {{&S33697[94], 9}, {NULL}};
const Segment SGI147368[] = {{&S33697[67], 6}, {NULL}};
const Segment SGI147369[] = {{&S30742[44], 13}, {NULL}};
const Segment SGI147370[] = {{&S45358[44], 10}, {NULL}};
const Segment SGI147371[] = {{&S30742[44], 25}, {NULL}};
const Segment SGI147372[] = {{&S45358[44], 22}, {NULL}};
const Segment SGI147373[] = {{&S30742[30], 6}, {NULL}};
const Segment SGI147374[] = {{&S45358[30], 6}, {NULL}};
const Segment SGI147375[] = {{&S30742[118], 3}, {NULL}};
const Segment SGI147376[] = {{&S45358[31], 3}, {NULL}};
const Segment SGI147377[] = {{&S30742[83], 6}, {NULL}};
const Segment SGI147378[] = {{&S45358[80], 6}, {NULL}};
const Segment SGI147379[] = {{&S30743[44], 20}, {NULL}};
const Segment SGI147380[] = {{&S30743[44], 32}, {NULL}};
const Segment SGI147381[] = {{&S30743[30], 6}, {NULL}};
const Segment SGI147382[] = {{&S45359[44], 10}, {NULL}};
const Segment SGI147383[] = {{&S30743[125], 3}, {NULL}};
const Segment SGI147384[] = {{&S45359[44], 22}, {NULL}};
const Segment SGI147385[] = {{&S45359[30], 6}, {NULL}};
const Segment SGI147386[] = {{&S30743[90], 6}, {NULL}};
const Segment SGI147387[] = {{&S45359[31], 3}, {NULL}};
const Segment SGI147388[] = {{&S45359[80], 6}, {NULL}};
const Segment SGI147389[] = {{&S30744[44], 15}, {NULL}};
const Segment SGI147390[] = {{&S30744[44], 27}, {NULL}};
const Segment SGI147391[] = {{&S30744[30], 6}, {NULL}};
const Segment SGI147392[] = {{&S30744[120], 3}, {NULL}};
const Segment SGI147393[] = {{&S45360[38], 10}, {NULL}};
const Segment SGI147394[] = {{&S30744[85], 6}, {NULL}};
const Segment SGI147395[] = {{&S45360[31], 6}, {NULL}};
const Segment SGI147396[] = {{&S30745[44], 10}, {NULL}};
const Segment SGI147397[] = {{&S30745[44], 22}, {NULL}};
const Segment SGI147398[] = {{&S30745[30], 6}, {NULL}};
const Segment SGI147399[] = {{&S45361[44], 28}, {NULL}};
const Segment SGI147400[] = {{&S30745[115], 3}, {NULL}};
const Segment SGI147401[] = {{&S45361[44], 40}, {NULL}};
const Segment SGI147402[] = {{&S45361[30], 6}, {NULL}};
const Segment SGI147403[] = {{&S30745[80], 6}, {NULL}};
const Segment SGI147404[] = {{&S45361[31], 3}, {NULL}};
const Segment SGI147405[] = {{&S45361[98], 6}, {NULL}};
const Segment SGI147406[] = {{&S30748[44], 10}, {NULL}};
const Segment SGI147407[] = {{&S30748[44], 22}, {NULL}};
const Segment SGI147408[] = {{&S30748[30], 6}, {NULL}};
const Segment SGI147409[] = {{&S30748[115], 3}, {NULL}};
const Segment SGI147410[] = {{&S45364[44], 10}, {NULL}};
const Segment SGI147411[] = {{&S30748[80], 6}, {NULL}};
const Segment SGI147412[] = {{&S45364[44], 73}, {NULL}};
const Segment SGI147413[] = {{&S45364[30], 6}, {NULL}};
const Segment SGI147414[] = {{&S30749[44], 5}, {NULL}};
const Segment SGI147415[] = {{&S30749[44], 17}, {NULL}};
const Segment SGI147416[] = {{&S30749[30], 6}, {NULL}};
const Segment SGI147417[] = {{&S30749[110], 3}, {NULL}};
const Segment SGI147418[] = {{&S30749[75], 6}, {NULL}};
const Segment SGI147419[] = {{&S45365[44], 10}, {NULL}};
const Segment SGI147420[] = {{&S45365[44], 28}, {NULL}};
const Segment SGI147421[] = {{&S45365[30], 6}, {NULL}};
const Segment SGI147422[] = {{&S30750[44], 8}, {NULL}};
const Segment SGI147423[] = {{&S30750[44], 20}, {NULL}};
const Segment SGI147424[] = {{&S30750[30], 6}, {NULL}};
const Segment SGI147425[] = {{&S30750[113], 3}, {NULL}};
const Segment SGI147426[] = {{&S45366[44], 22}, {NULL}};
const Segment SGI147427[] = {{&S45366[30], 6}, {NULL}};
const Segment SGI147428[] = {{&S30750[78], 6}, {NULL}};
const Segment SGI147429[] = {{&S40775[44], 7}, {NULL}};
const Segment SGI147430[] = {{&S40775[44], 19}, {NULL}};
const Segment SGI147431[] = {{&S40775[30], 6}, {NULL}};
const Segment SGI147432[] = {{&S40775[31], 3}, {NULL}};
const Segment SGI147433[] = {{&S40775[77], 6}, {NULL}};
const Segment SGI147434[] = {{&S45367[44], 10}, {NULL}};
const Segment SGI147435[] = {{&S45367[44], 22}, {NULL}};
const Segment SGI147436[] = {{&S45367[30], 6}, {NULL}};
const Segment SGI147437[] = {{&S30752[44], 15}, {NULL}};
const Segment SGI147438[] = {{&S30752[44], 27}, {NULL}};
const Segment SGI147439[] = {{&S30752[30], 6}, {NULL}};
const Segment SGI147440[] = {{&S45367[80], 6}, {NULL}};
const Segment SGI147441[] = {{&S30752[120], 3}, {NULL}};
const Segment SGI147442[] = {{&S30752[85], 6}, {NULL}};
const Segment SGI147443[] = {{&S45368[44], 10}, {NULL}};
const Segment SGI147444[] = {{&S45368[44], 22}, {NULL}};
const Segment SGI147445[] = {{&S45368[30], 6}, {NULL}};
const Segment SGI147446[] = {{&S30753[44], 4}, {NULL}};
const Segment SGI147447[] = {{&S30753[44], 16}, {NULL}};
const Segment SGI147448[] = {{&S30753[30], 6}, {NULL}};
const Segment SGI147449[] = {{&S45368[31], 3}, {NULL}};
const Segment SGI147450[] = {{&S45368[80], 6}, {NULL}};
const Segment SGI147451[] = {{&S30753[109], 3}, {NULL}};
const Segment SGI147452[] = {{&S30753[74], 6}, {NULL}};
const Segment SGI147453[] = {{&S30754[44], 7}, {NULL}};
const Segment SGI147454[] = {{&S30754[44], 19}, {NULL}};
const Segment SGI147455[] = {{&S30754[30], 6}, {NULL}};
const Segment SGI147456[] = {{&S45369[44], 28}, {NULL}};
const Segment SGI147457[] = {{&S45369[44], 40}, {NULL}};
const Segment SGI147458[] = {{&S45369[30], 6}, {NULL}};
const Segment SGI147459[] = {{&S30754[112], 3}, {NULL}};
const Segment SGI147460[] = {{&S30754[77], 6}, {NULL}};
const Segment SGI147461[] = {{&S45369[31], 3}, {NULL}};
const Segment SGI147462[] = {{&S45369[98], 6}, {NULL}};
const Segment SGI147463[] = {{&S30755[44], 9}, {NULL}};
const Segment SGI147464[] = {{&S30755[44], 21}, {NULL}};
const Segment SGI147465[] = {{&S30755[30], 6}, {NULL}};
const Segment SGI147466[] = {{&S30755[114], 3}, {NULL}};
const Segment SGI147467[] = {{&S45370[65], 10}, {NULL}};
const Segment SGI147468[] = {{&S45370[65], 22}, {NULL}};
const Segment SGI147469[] = {{&S45370[51], 6}, {NULL}};
const Segment SGI147470[] = {{&S30755[79], 6}, {NULL}};
const Segment SGI147471[] = {{&S45371[44], 10}, {NULL}};
const Segment SGI147472[] = {{&S45371[44], 22}, {NULL}};
const Segment SGI147473[] = {{&S45371[30], 6}, {NULL}};
const Segment SGI147474[] = {{&S30757[44], 33}, {NULL}};
const Segment SGI147475[] = {{&S45371[83], 6}, {NULL}};
const Segment SGI147476[] = {{&S30757[44], 45}, {NULL}};
const Segment SGI147477[] = {{&S30757[30], 6}, {NULL}};
const Segment SGI147478[] = {{&S30757[138], 3}, {NULL}};
const Segment SGI147479[] = {{&S30757[103], 6}, {NULL}};
const Segment SGI147480[] = {{&S45372[42], 10}, {NULL}};
const Segment SGI147481[] = {{&S45372[30], 6}, {NULL}};
const Segment SGI147482[] = {{&S30758[44], 6}, {NULL}};
const Segment SGI147483[] = {{&S45372[31], 3}, {NULL}};
const Segment SGI147484[] = {{&S30758[44], 18}, {NULL}};
const Segment SGI147485[] = {{&S30758[30], 6}, {NULL}};
const Segment SGI147486[] = {{&S45372[78], 6}, {NULL}};
const Segment SGI147487[] = {{&S30758[111], 3}, {NULL}};
const Segment SGI147488[] = {{&S30758[76], 6}, {NULL}};
const Segment SGI147489[] = {{&S45373[46], 10}, {NULL}};
const Segment SGI147490[] = {{&S45373[46], 28}, {NULL}};
const Segment SGI147491[] = {{&S45373[32], 6}, {NULL}};
const Segment SGI147492[] = {{&S30760[44], 19}, {NULL}};
const Segment SGI147493[] = {{&S30760[44], 31}, {NULL}};
const Segment SGI147494[] = {{&S30760[30], 6}, {NULL}};
const Segment SGI147495[] = {{&S45374[45], 10}, {NULL}};
const Segment SGI147496[] = {{&S45374[45], 28}, {NULL}};
const Segment SGI147497[] = {{&S45374[31], 6}, {NULL}};
const Segment SGI147498[] = {{&S30760[124], 3}, {NULL}};
const Segment SGI147499[] = {{&S30760[89], 6}, {NULL}};
const Segment SGI147500[] = {{&S45375[44], 11}, {NULL}};
const Segment SGI147501[] = {{&S45375[44], 23}, {NULL}};
const Segment SGI147502[] = {{&S45375[30], 6}, {NULL}};
const Segment SGI147503[] = {{&S45375[31], 3}, {NULL}};
const Segment SGI147504[] = {{&S30762[44], 17}, {NULL}};
const Segment SGI147505[] = {{&S45375[81], 6}, {NULL}};
const Segment SGI147506[] = {{&S30762[44], 37}, {NULL}};
const Segment SGI147507[] = {{&S30762[30], 6}, {NULL}};
const Segment SGI147508[] = {{&S30762[130], 3}, {NULL}};
const Segment SGI147509[] = {{&S30762[95], 6}, {NULL}};
const Segment SGI147510[] = {{&S45376[44], 10}, {NULL}};
const Segment SGI147511[] = {{&S45376[44], 22}, {NULL}};
const Segment SGI147512[] = {{&S45376[30], 6}, {NULL}};
const Segment SGI147513[] = {{&S45376[116], 3}, {NULL}};
const Segment SGI147514[] = {{&S45376[80], 7}, {NULL}};
const Segment SGI147515[] = {{&S45376[136], 8}, {NULL}};
const Segment SGI147516[] = {{&S45377[38], 11}, {NULL}};
const Segment SGI147517[] = {{&S45377[31], 6}, {NULL}};
const Segment SGI147518[] = {{&S30766[44], 7}, {NULL}};
const Segment SGI147519[] = {{&S30766[44], 19}, {NULL}};
const Segment SGI147520[] = {{&S30766[30], 6}, {NULL}};
const Segment SGI147521[] = {{&S30766[112], 3}, {NULL}};
const Segment SGI147522[] = {{&S45378[44], 29}, {NULL}};
const Segment SGI147523[] = {{&S45378[44], 41}, {NULL}};
const Segment SGI147524[] = {{&S30766[77], 6}, {NULL}};
const Segment SGI147525[] = {{&S45378[30], 6}, {NULL}};
const Segment SGI147526[] = {{&S45378[31], 3}, {NULL}};
const Segment SGI147527[] = {{&S45378[99], 6}, {NULL}};
const Segment SGI147528[] = {{&S30769[44], 12}, {NULL}};
const Segment SGI147529[] = {{&S30769[44], 24}, {NULL}};
const Segment SGI147530[] = {{&S30769[30], 6}, {NULL}};
const Segment SGI147531[] = {{&S30769[117], 3}, {NULL}};
const Segment SGI147532[] = {{&S30769[82], 6}, {NULL}};
const Segment SGI147533[] = {{&S30772[44], 9}, {NULL}};
const Segment SGI147534[] = {{&S30772[44], 21}, {NULL}};
const Segment SGI147535[] = {{&S30772[30], 6}, {NULL}};
const Segment SGI147536[] = {{&S30772[114], 3}, {NULL}};
const Segment SGI147537[] = {{&S30772[79], 6}, {NULL}};
const Segment SGI147538[] = {{&S30773[44], 12}, {NULL}};
const Segment SGI147539[] = {{&S30773[44], 24}, {NULL}};
const Segment SGI147540[] = {{&S30773[30], 6}, {NULL}};
const Segment SGI147541[] = {{&S30773[117], 3}, {NULL}};
const Segment SGI147542[] = {{&S30773[82], 6}, {NULL}};
const Segment SGI147543[] = {{&S30778[44], 11}, {NULL}};
const Segment SGI147544[] = {{&S30778[44], 23}, {NULL}};
const Segment SGI147545[] = {{&S30778[30], 6}, {NULL}};
const Segment SGI147546[] = {{&S30778[116], 3}, {NULL}};
const Segment SGI147547[] = {{&S30778[81], 6}, {NULL}};
const Segment SGI147548[] = {{&S30779[44], 9}, {NULL}};
const Segment SGI147549[] = {{&S30779[44], 21}, {NULL}};
const Segment SGI147550[] = {{&S30779[30], 6}, {NULL}};
const Segment SGI147551[] = {{&S30779[114], 3}, {NULL}};
const Segment SGI147552[] = {{&S30779[79], 6}, {NULL}};
const Segment SGI147553[] = {{&S30782[44], 17}, {NULL}};
const Segment SGI147554[] = {{&S30782[44], 29}, {NULL}};
const Segment SGI147555[] = {{&S30782[30], 6}, {NULL}};
const Segment SGI147556[] = {{&S30782[122], 3}, {NULL}};
const Segment SGI147557[] = {{&S30782[87], 6}, {NULL}};
const Segment SGI147558[] = {{&S30783[44], 10}, {NULL}};
const Segment SGI147559[] = {{&S30783[44], 22}, {NULL}};
const Segment SGI147560[] = {{&S30783[30], 6}, {NULL}};
const Segment SGI147561[] = {{&S30783[115], 3}, {NULL}};
const Segment SGI147562[] = {{&S30783[80], 6}, {NULL}};
const Segment SGI147563[] = {{&S30786[44], 7}, {NULL}};
const Segment SGI147564[] = {{&S30786[44], 19}, {NULL}};
const Segment SGI147565[] = {{&S30786[30], 6}, {NULL}};
const Segment SGI147566[] = {{&S30786[112], 3}, {NULL}};
const Segment SGI147567[] = {{&S30786[77], 6}, {NULL}};
const Segment SGI147568[] = {{&S30787[44], 5}, {NULL}};
const Segment SGI147569[] = {{&S30787[44], 17}, {NULL}};
const Segment SGI147570[] = {{&S30787[30], 6}, {NULL}};
const Segment SGI147571[] = {{&S30787[110], 3}, {NULL}};
const Segment SGI147572[] = {{&S30787[75], 6}, {NULL}};
const Segment SGI147573[] = {{&S30788[44], 7}, {NULL}};
const Segment SGI147574[] = {{&S30788[44], 19}, {NULL}};
const Segment SGI147575[] = {{&S30788[30], 6}, {NULL}};
const Segment SGI147576[] = {{&S30788[112], 3}, {NULL}};
const Segment SGI147577[] = {{&S30788[77], 6}, {NULL}};
const Segment SGI147578[] = {{&S30789[44], 12}, {NULL}};
const Segment SGI147579[] = {{&S30789[44], 24}, {NULL}};
const Segment SGI147580[] = {{&S30789[30], 6}, {NULL}};
const Segment SGI147581[] = {{&S30789[117], 3}, {NULL}};
const Segment SGI147582[] = {{&S30789[82], 6}, {NULL}};
const Segment SGI147583[] = {{&S30790[44], 12}, {NULL}};
const Segment SGI147584[] = {{&S30790[44], 24}, {NULL}};
const Segment SGI147585[] = {{&S30790[30], 6}, {NULL}};
const Segment SGI147586[] = {{&S30790[117], 3}, {NULL}};
const Segment SGI147587[] = {{&S30790[82], 6}, {NULL}};
const Segment SGI147588[] = {{&S36108[44], 9}, {NULL}};
const Segment SGI147589[] = {{&S30791[44], 8}, {NULL}};
const Segment SGI147590[] = {{&S36108[44], 21}, {NULL}};
const Segment SGI147591[] = {{&S30791[44], 20}, {NULL}};
const Segment SGI147592[] = {{&S36108[30], 6}, {NULL}};
const Segment SGI147593[] = {{&S30791[30], 6}, {NULL}};
const Segment SGI147594[] = {{&S30791[113], 3}, {NULL}};
const Segment SGI147595[] = {{&S36108[114], 3}, {NULL}};
const Segment SGI147596[] = {{&S30791[78], 6}, {NULL}};
const Segment SGI147597[] = {{&S36108[79], 6}, {NULL}};
const Segment SGI147598[] = {{&S36109[44], 9}, {NULL}};
const Segment SGI147599[] = {{&S36109[44], 21}, {NULL}};
const Segment SGI147600[] = {{&S36109[30], 6}, {NULL}};
const Segment SGI147601[] = {{&S36109[114], 3}, {NULL}};
const Segment SGI147602[] = {{&S36109[79], 6}, {NULL}};
const Segment SGI147603[] = {{&S30793[44], 8}, {NULL}};
const Segment SGI147604[] = {{&S30793[44], 20}, {NULL}};
const Segment SGI147605[] = {{&S30793[30], 6}, {NULL}};
const Segment SGI147606[] = {{&S30793[113], 3}, {NULL}};
const Segment SGI147607[] = {{&S30793[78], 6}, {NULL}};
const Segment SGI147608[] = {{&S36110[44], 5}, {NULL}};
const Segment SGI147609[] = {{&S36110[44], 17}, {NULL}};
const Segment SGI147610[] = {{&S36110[30], 6}, {NULL}};
const Segment SGI147611[] = {{&S36110[110], 3}, {NULL}};
const Segment SGI147612[] = {{&S36110[75], 6}, {NULL}};
const Segment SGI147613[] = {{&S30794[44], 9}, {NULL}};
const Segment SGI147614[] = {{&S30794[44], 21}, {NULL}};
const Segment SGI147615[] = {{&S30794[30], 6}, {NULL}};
const Segment SGI147616[] = {{&S30794[114], 3}, {NULL}};
const Segment SGI147617[] = {{&S30794[79], 6}, {NULL}};
const Segment SGI147618[] = {{&S30795[44], 8}, {NULL}};
const Segment SGI147619[] = {{&S30795[44], 20}, {NULL}};
const Segment SGI147620[] = {{&S30795[30], 6}, {NULL}};
const Segment SGI147621[] = {{&S36112[44], 5}, {NULL}};
const Segment SGI147622[] = {{&S36112[44], 17}, {NULL}};
const Segment SGI147623[] = {{&S36112[30], 6}, {NULL}};
const Segment SGI147624[] = {{&S30795[113], 3}, {NULL}};
const Segment SGI147625[] = {{&S30795[78], 6}, {NULL}};
const Segment SGI147626[] = {{&S36112[110], 3}, {NULL}};
const Segment SGI147627[] = {{&S36112[75], 6}, {NULL}};
const Segment SGI147628[] = {{&S30796[44], 7}, {NULL}};
const Segment SGI147629[] = {{&S30796[44], 19}, {NULL}};
const Segment SGI147630[] = {{&S30796[30], 6}, {NULL}};
const Segment SGI147631[] = {{&S36113[38], 5}, {NULL}};
const Segment SGI147632[] = {{&S36113[31], 6}, {NULL}};
const Segment SGI147633[] = {{&S30796[112], 3}, {NULL}};
const Segment SGI147634[] = {{&S30796[77], 6}, {NULL}};
const Segment SGI147635[] = {{&S30797[44], 4}, {NULL}};
const Segment SGI147636[] = {{&S30797[44], 16}, {NULL}};
const Segment SGI147637[] = {{&S30797[30], 6}, {NULL}};
const Segment SGI147638[] = {{&S30797[109], 3}, {NULL}};
const Segment SGI147639[] = {{&S30797[74], 6}, {NULL}};
const Segment SGI147640[] = {{&S36115[44], 12}, {NULL}};
const Segment SGI147641[] = {{&S36115[44], 24}, {NULL}};
const Segment SGI147642[] = {{&S36115[30], 6}, {NULL}};
const Segment SGI147643[] = {{&S36115[117], 3}, {NULL}};
const Segment SGI147644[] = {{&S36115[82], 6}, {NULL}};
const Segment SGI147645[] = {{&S45431[34], 7}, {NULL}};
const Segment SGI147646[] = {{&S30798[44], 20}, {NULL}};
const Segment SGI147647[] = {{&S45431[27], 6}, {NULL}};
const Segment SGI147648[] = {{&S30798[44], 32}, {NULL}};
const Segment SGI147649[] = {{&S30798[30], 6}, {NULL}};
const Segment SGI147650[] = {{&S30798[125], 3}, {NULL}};
const Segment SGI147651[] = {{&S45431[112], 9}, {NULL}};
const Segment SGI147652[] = {{&S45431[85], 6}, {NULL}};
const Segment SGI147653[] = {{&S30798[90], 6}, {NULL}};
const Segment SGI147654[] = {{&S36116[44], 9}, {NULL}};
const Segment SGI147655[] = {{&S36116[44], 30}, {NULL}};
const Segment SGI147656[] = {{&S36116[30], 6}, {NULL}};
const Segment SGI147657[] = {{&S36116[123], 3}, {NULL}};
const Segment SGI147658[] = {{&S36116[88], 6}, {NULL}};
const Segment SGI147659[] = {{&S30799[44], 7}, {NULL}};
const Segment SGI147660[] = {{&S30799[44], 19}, {NULL}};
const Segment SGI147661[] = {{&S30799[30], 6}, {NULL}};
const Segment SGI147662[] = {{&S30799[112], 3}, {NULL}};
const Segment SGI147663[] = {{&S30799[77], 6}, {NULL}};
const Segment SGI147664[] = {{&S36117[42], 5}, {NULL}};
const Segment SGI147665[] = {{&S45433[34], 7}, {NULL}};
const Segment SGI147666[] = {{&S36117[30], 6}, {NULL}};
const Segment SGI147667[] = {{&S45433[27], 6}, {NULL}};
const Segment SGI147668[] = {{&S36117[108], 3}, {NULL}};
const Segment SGI147669[] = {{&S36117[73], 6}, {NULL}};
const Segment SGI147670[] = {{&S30800[44], 4}, {NULL}};
const Segment SGI147671[] = {{&S45433[112], 9}, {NULL}};
const Segment SGI147672[] = {{&S30800[44], 16}, {NULL}};
const Segment SGI147673[] = {{&S45433[85], 6}, {NULL}};
const Segment SGI147674[] = {{&S30800[30], 6}, {NULL}};
const Segment SGI147675[] = {{&S30800[109], 3}, {NULL}};
const Segment SGI147676[] = {{&S30800[74], 6}, {NULL}};
const Segment SGI147677[] = {{&S45434[34], 8}, {NULL}};
const Segment SGI147678[] = {{&S45434[27], 6}, {NULL}};
const Segment SGI147679[] = {{&S30801[44], 12}, {NULL}};
const Segment SGI147680[] = {{&S30801[44], 24}, {NULL}};
const Segment SGI147681[] = {{&S36119[44], 9}, {NULL}};
const Segment SGI147682[] = {{&S30801[30], 6}, {NULL}};
const Segment SGI147683[] = {{&S36119[44], 20}, {NULL}};
const Segment SGI147684[] = {{&S36119[30], 6}, {NULL}};
const Segment SGI147685[] = {{&S45434[103], 9}, {NULL}};
const Segment SGI147686[] = {{&S45434[76], 6}, {NULL}};
const Segment SGI147687[] = {{&S30801[117], 3}, {NULL}};
const Segment SGI147688[] = {{&S36119[113], 3}, {NULL}};
const Segment SGI147689[] = {{&S30801[82], 6}, {NULL}};
const Segment SGI147690[] = {{&S36119[78], 6}, {NULL}};
const Segment SGI147691[] = {{&S30802[44], 5}, {NULL}};
const Segment SGI147692[] = {{&S30802[44], 17}, {NULL}};
const Segment SGI147693[] = {{&S30802[30], 6}, {NULL}};
const Segment SGI147694[] = {{&S30802[110], 3}, {NULL}};
const Segment SGI147695[] = {{&S30802[75], 6}, {NULL}};
const Segment SGI147696[] = {{&S30803[44], 10}, {NULL}};
const Segment SGI147697[] = {{&S30803[44], 22}, {NULL}};
const Segment SGI147698[] = {{&S30803[30], 6}, {NULL}};
const Segment SGI147699[] = {{&S30803[115], 3}, {NULL}};
const Segment SGI147700[] = {{&S30803[80], 6}, {NULL}};
const Segment SGI147701[] = {{&S45440[34], 9}, {NULL}};
const Segment SGI147702[] = {{&S45440[27], 6}, {NULL}};
const Segment SGI147703[] = {{&S45440[96], 9}, {NULL}};
const Segment SGI147704[] = {{&S45440[69], 6}, {NULL}};
const Segment SGI147705[] = {{&S30806[44], 4}, {NULL}};
const Segment SGI147706[] = {{&S30806[44], 16}, {NULL}};
const Segment SGI147707[] = {{&S30806[30], 6}, {NULL}};
const Segment SGI147708[] = {{&S30806[109], 3}, {NULL}};
const Segment SGI147709[] = {{&S30806[74], 6}, {NULL}};
const Segment SGI147710[] = {{&S30807[44], 18}, {NULL}};
const Segment SGI147711[] = {{&S30807[44], 30}, {NULL}};
const Segment SGI147712[] = {{&S30807[30], 6}, {NULL}};
const Segment SGI147713[] = {{&S30807[123], 3}, {NULL}};
const Segment SGI147714[] = {{&S30807[88], 6}, {NULL}};
const Segment SGI147715[] = {{&S30808[44], 4}, {NULL}};
const Segment SGI147716[] = {{&S30808[44], 16}, {NULL}};
const Segment SGI147717[] = {{&S30808[30], 6}, {NULL}};
const Segment SGI147718[] = {{&S30808[109], 3}, {NULL}};
const Segment SGI147719[] = {{&S30808[74], 6}, {NULL}};
const Segment SGI147720[] = {{&S30810[43], 19}, {NULL}};
const Segment SGI147721[] = {{&S30810[30], 6}, {NULL}};
const Segment SGI147722[] = {{&S30810[31], 3}, {NULL}};
const Segment SGI147723[] = {{&S30810[76], 6}, {NULL}};
const Segment SGI147724[] = {{&S30815[44], 10}, {NULL}};
const Segment SGI147725[] = {{&S30815[44], 28}, {NULL}};
const Segment SGI147726[] = {{&S30815[30], 6}, {NULL}};
const Segment SGI147727[] = {{&S30815[121], 3}, {NULL}};
const Segment SGI147728[] = {{&S30815[86], 6}, {NULL}};
const Segment SGI147729[] = {{&S30817[44], 16}, {NULL}};
const Segment SGI147730[] = {{&S30817[44], 28}, {NULL}};
const Segment SGI147731[] = {{&S30817[30], 6}, {NULL}};
const Segment SGI147732[] = {{&S30817[121], 3}, {NULL}};
const Segment SGI147733[] = {{&S30817[86], 6}, {NULL}};
const Segment SGI147734[] = {{&S30818[44], 14}, {NULL}};
const Segment SGI147735[] = {{&S30818[44], 26}, {NULL}};
const Segment SGI147736[] = {{&S30818[30], 6}, {NULL}};
const Segment SGI147737[] = {{&S30818[119], 3}, {NULL}};
const Segment SGI147738[] = {{&S30818[84], 6}, {NULL}};
const Segment SGI147739[] = {{&S30819[44], 12}, {NULL}};
const Segment SGI147740[] = {{&S30819[44], 24}, {NULL}};
const Segment SGI147741[] = {{&S30819[30], 6}, {NULL}};
const Segment SGI147742[] = {{&S30819[117], 3}, {NULL}};
const Segment SGI147743[] = {{&S30819[82], 6}, {NULL}};
const Segment SGI147744[] = {{&S30820[44], 14}, {NULL}};
const Segment SGI147745[] = {{&S30820[44], 26}, {NULL}};
const Segment SGI147746[] = {{&S30820[30], 6}, {NULL}};
const Segment SGI147747[] = {{&S30820[119], 3}, {NULL}};
const Segment SGI147748[] = {{&S30820[84], 6}, {NULL}};
const Segment SGI147749[] = {{&S30822[44], 7}, {NULL}};
const Segment SGI147750[] = {{&S30822[44], 19}, {NULL}};
const Segment SGI147751[] = {{&S30822[30], 6}, {NULL}};
const Segment SGI147752[] = {{&S30822[112], 3}, {NULL}};
const Segment SGI147753[] = {{&S30822[77], 6}, {NULL}};
const Segment SGI147754[] = {{&S30823[44], 8}, {NULL}};
const Segment SGI147755[] = {{&S30823[44], 20}, {NULL}};
const Segment SGI147756[] = {{&S30823[30], 6}, {NULL}};
const Segment SGI147757[] = {{&S30823[113], 3}, {NULL}};
const Segment SGI147758[] = {{&S30823[78], 6}, {NULL}};
const Segment SGI147759[] = {{&S30826[44], 6}, {NULL}};
const Segment SGI147760[] = {{&S30826[44], 18}, {NULL}};
const Segment SGI147761[] = {{&S30826[30], 6}, {NULL}};
const Segment SGI147762[] = {{&S30826[111], 3}, {NULL}};
const Segment SGI147763[] = {{&S30826[76], 6}, {NULL}};
const Segment SGI147764[] = {{&S30827[44], 6}, {NULL}};
const Segment SGI147765[] = {{&S30827[44], 18}, {NULL}};
const Segment SGI147766[] = {{&S30827[30], 6}, {NULL}};
const Segment SGI147767[] = {{&S30827[111], 3}, {NULL}};
const Segment SGI147768[] = {{&S30827[76], 6}, {NULL}};
const Segment SGI147769[] = {{&S30829[44], 5}, {NULL}};
const Segment SGI147770[] = {{&S30829[44], 17}, {NULL}};
const Segment SGI147771[] = {{&S30829[30], 6}, {NULL}};
const Segment SGI147772[] = {{&S30829[110], 3}, {NULL}};
const Segment SGI147773[] = {{&S30829[75], 6}, {NULL}};
const Segment SGI147774[] = {{&S30831[44], 15}, {NULL}};
const Segment SGI147775[] = {{&S30831[44], 27}, {NULL}};
const Segment SGI147776[] = {{&S30831[30], 6}, {NULL}};
const Segment SGI147777[] = {{&S30831[120], 3}, {NULL}};
const Segment SGI147778[] = {{&S30831[85], 6}, {NULL}};
const Segment SGI147779[] = {{&S30833[44], 5}, {NULL}};
const Segment SGI147780[] = {{&S30833[44], 14}, {NULL}};
const Segment SGI147781[] = {{&S30833[30], 6}, {NULL}};
const Segment SGI147782[] = {{&S30833[107], 3}, {NULL}};
const Segment SGI147783[] = {{&S30833[72], 6}, {NULL}};
const Segment SGI147784[] = {{&S30840[44], 8}, {NULL}};
const Segment SGI147785[] = {{&S30840[44], 20}, {NULL}};
const Segment SGI147786[] = {{&S30840[30], 6}, {NULL}};
const Segment SGI147787[] = {{&S30840[113], 3}, {NULL}};
const Segment SGI147788[] = {{&S30840[78], 6}, {NULL}};
const Segment SGI147789[] = {{&S30841[44], 6}, {NULL}};
const Segment SGI147790[] = {{&S30841[44], 18}, {NULL}};
const Segment SGI147791[] = {{&S30841[30], 6}, {NULL}};
const Segment SGI147792[] = {{&S30841[111], 3}, {NULL}};
const Segment SGI147793[] = {{&S30841[76], 6}, {NULL}};
const Segment SGI147794[] = {{&S30843[44], 18}, {NULL}};
const Segment SGI147795[] = {{&S30843[44], 30}, {NULL}};
const Segment SGI147796[] = {{&S30843[30], 6}, {NULL}};
const Segment SGI147797[] = {{&S30843[123], 3}, {NULL}};
const Segment SGI147798[] = {{&S30843[88], 6}, {NULL}};
const Segment SGI147799[] = {{&S30844[44], 14}, {NULL}};
const Segment SGI147800[] = {{&S30844[44], 26}, {NULL}};
const Segment SGI147801[] = {{&S30844[30], 6}, {NULL}};
const Segment SGI147802[] = {{&S36186[44], 10}, {NULL}};
const Segment SGI147803[] = {{&S36186[44], 22}, {NULL}};
const Segment SGI147804[] = {{&S36186[30], 6}, {NULL}};
const Segment SGI147805[] = {{&S30844[119], 3}, {NULL}};
const Segment SGI147806[] = {{&S30844[84], 6}, {NULL}};
const Segment SGI147807[] = {{&S36186[115], 3}, {NULL}};
const Segment SGI147808[] = {{&S36186[80], 6}, {NULL}};
const Segment SGI147809[] = {{&S36187[44], 10}, {NULL}};
const Segment SGI147810[] = {{&S36187[44], 22}, {NULL}};
const Segment SGI147811[] = {{&S36187[30], 6}, {NULL}};
const Segment SGI147812[] = {{&S36187[115], 3}, {NULL}};
const Segment SGI147813[] = {{&S36187[80], 6}, {NULL}};
const Segment SGI147814[] = {{&S36188[44], 10}, {NULL}};
const Segment SGI147815[] = {{&S36188[44], 22}, {NULL}};
const Segment SGI147816[] = {{&S36188[30], 6}, {NULL}};
const Segment SGI147817[] = {{&S30848[44], 7}, {NULL}};
const Segment SGI147818[] = {{&S30848[44], 19}, {NULL}};
const Segment SGI147819[] = {{&S30848[30], 6}, {NULL}};
const Segment SGI147820[] = {{&S36188[115], 3}, {NULL}};
const Segment SGI147821[] = {{&S36188[80], 6}, {NULL}};
const Segment SGI147822[] = {{&S30848[112], 3}, {NULL}};
const Segment SGI147823[] = {{&S30848[77], 6}, {NULL}};
const Segment SGI147824[] = {{&S45506[34], 10}, {NULL}};
const Segment SGI147825[] = {{&S36189[44], 10}, {NULL}};
const Segment SGI147826[] = {{&S45506[27], 6}, {NULL}};
const Segment SGI147827[] = {{&S36189[44], 22}, {NULL}};
const Segment SGI147828[] = {{&S36189[30], 6}, {NULL}};
const Segment SGI147829[] = {{&S36189[115], 3}, {NULL}};
const Segment SGI147830[] = {{&S45506[97], 9}, {NULL}};
const Segment SGI147831[] = {{&S45506[70], 6}, {NULL}};
const Segment SGI147832[] = {{&S36189[80], 6}, {NULL}};
const Segment SGI147833[] = {{&S30852[44], 19}, {NULL}};
const Segment SGI147834[] = {{&S45508[34], 10}, {NULL}};
const Segment SGI147835[] = {{&S30852[44], 31}, {NULL}};
const Segment SGI147836[] = {{&S45508[27], 6}, {NULL}};
const Segment SGI147837[] = {{&S30852[30], 6}, {NULL}};
const Segment SGI147838[] = {{&S30852[124], 3}, {NULL}};
const Segment SGI147839[] = {{&S45508[97], 9}, {NULL}};
const Segment SGI147840[] = {{&S45508[70], 6}, {NULL}};
const Segment SGI147841[] = {{&S30852[89], 6}, {NULL}};
const Segment SGI147842[] = {{&S36192[44], 10}, {NULL}};
const Segment SGI147843[] = {{&S36192[44], 22}, {NULL}};
const Segment SGI147844[] = {{&S36192[30], 6}, {NULL}};
const Segment SGI147845[] = {{&S36192[115], 3}, {NULL}};
const Segment SGI147846[] = {{&S36192[80], 6}, {NULL}};
const Segment SGI147847[] = {{&S45509[34], 11}, {NULL}};
const Segment SGI147848[] = {{&S45509[27], 6}, {NULL}};
const Segment SGI147849[] = {{&S45509[98], 9}, {NULL}};
const Segment SGI147850[] = {{&S45509[71], 6}, {NULL}};
const Segment SGI147851[] = {{&S30854[44], 7}, {NULL}};
const Segment SGI147852[] = {{&S30854[44], 19}, {NULL}};
const Segment SGI147853[] = {{&S30854[30], 6}, {NULL}};
const Segment SGI147854[] = {{&S30854[112], 3}, {NULL}};
const Segment SGI147855[] = {{&S30854[77], 6}, {NULL}};
const Segment SGI147856[] = {{&S45510[34], 11}, {NULL}};
const Segment SGI147857[] = {{&S45510[27], 6}, {NULL}};
const Segment SGI147858[] = {{&S36195[44], 18}, {NULL}};
const Segment SGI147859[] = {{&S45510[98], 9}, {NULL}};
const Segment SGI147860[] = {{&S36195[44], 30}, {NULL}};
const Segment SGI147861[] = {{&S45510[71], 6}, {NULL}};
const Segment SGI147862[] = {{&S36195[30], 6}, {NULL}};
const Segment SGI147863[] = {{&S36195[123], 3}, {NULL}};
const Segment SGI147864[] = {{&S36195[88], 6}, {NULL}};
const Segment SGI147865[] = {{&S45511[34], 11}, {NULL}};
const Segment SGI147866[] = {{&S45511[27], 6}, {NULL}};
const Segment SGI147867[] = {{&S45511[98], 9}, {NULL}};
const Segment SGI147868[] = {{&S45511[71], 6}, {NULL}};
const Segment SGI147869[] = {{&S45512[34], 11}, {NULL}};
const Segment SGI147870[] = {{&S45512[27], 6}, {NULL}};
const Segment SGI147871[] = {{&S30859[44], 4}, {NULL}};
const Segment SGI147872[] = {{&S30859[44], 16}, {NULL}};
const Segment SGI147873[] = {{&S30859[30], 6}, {NULL}};
const Segment SGI147874[] = {{&S45512[98], 9}, {NULL}};
const Segment SGI147875[] = {{&S45512[71], 6}, {NULL}};
const Segment SGI147876[] = {{&S30859[109], 3}, {NULL}};
const Segment SGI147877[] = {{&S30859[74], 6}, {NULL}};
const Segment SGI147878[] = {{&S45513[34], 10}, {NULL}};
const Segment SGI147879[] = {{&S45513[27], 6}, {NULL}};
const Segment SGI147880[] = {{&S45513[97], 9}, {NULL}};
const Segment SGI147881[] = {{&S45513[70], 6}, {NULL}};
const Segment SGI147882[] = {{&S45514[34], 8}, {NULL}};
const Segment SGI147883[] = {{&S45514[27], 6}, {NULL}};
const Segment SGI147884[] = {{&S45514[95], 9}, {NULL}};
const Segment SGI147885[] = {{&S30863[44], 9}, {NULL}};
const Segment SGI147886[] = {{&S45514[68], 6}, {NULL}};
const Segment SGI147887[] = {{&S30863[44], 21}, {NULL}};
const Segment SGI147888[] = {{&S30863[30], 6}, {NULL}};
const Segment SGI147889[] = {{&S30863[114], 3}, {NULL}};
const Segment SGI147890[] = {{&S30863[79], 6}, {NULL}};
const Segment SGI147891[] = {{&S45515[34], 11}, {NULL}};
const Segment SGI147892[] = {{&S45515[27], 6}, {NULL}};
const Segment SGI147893[] = {{&S45515[98], 9}, {NULL}};
const Segment SGI147894[] = {{&S45515[71], 6}, {NULL}};
const Segment SGI147895[] = {{&S30865[44], 7}, {NULL}};
const Segment SGI147896[] = {{&S30865[44], 19}, {NULL}};
const Segment SGI147897[] = {{&S30865[30], 6}, {NULL}};
const Segment SGI147898[] = {{&S30865[112], 3}, {NULL}};
const Segment SGI147899[] = {{&S30865[77], 6}, {NULL}};
const Segment SGI147900[] = {{&S45519[34], 7}, {NULL}};
const Segment SGI147901[] = {{&S45519[27], 6}, {NULL}};
const Segment SGI147902[] = {{&S45519[94], 9}, {NULL}};
const Segment SGI147903[] = {{&S45519[67], 6}, {NULL}};
const Segment SGI147904[] = {{&S30870[44], 10}, {NULL}};
const Segment SGI147905[] = {{&S30870[44], 22}, {NULL}};
const Segment SGI147906[] = {{&S30870[30], 6}, {NULL}};
const Segment SGI147907[] = {{&S45520[34], 14}, {NULL}};
const Segment SGI147908[] = {{&S45520[27], 6}, {NULL}};
const Segment SGI147909[] = {{&S30870[115], 3}, {NULL}};
const Segment SGI147910[] = {{&S30870[80], 6}, {NULL}};
const Segment SGI147911[] = {{&S45520[114], 9}, {NULL}};
const Segment SGI147912[] = {{&S45520[87], 6}, {NULL}};
const Segment SGI147913[] = {{&S45521[34], 14}, {NULL}};
const Segment SGI147914[] = {{&S45521[27], 6}, {NULL}};
const Segment SGI147915[] = {{&S30872[55], 6}, {NULL}};
const Segment SGI147916[] = {{&S45521[114], 9}, {NULL}};
const Segment SGI147917[] = {{&S45521[87], 6}, {NULL}};
const Segment SGI147918[] = {{&S30872[117], 6}, {NULL}};
const Segment SGI147919[] = {{&S30874[44], 8}, {NULL}};
const Segment SGI147920[] = {{&S30874[44], 20}, {NULL}};
const Segment SGI147921[] = {{&S30874[30], 6}, {NULL}};
const Segment SGI147922[] = {{&S30874[113], 3}, {NULL}};
const Segment SGI147923[] = {{&S30874[78], 6}, {NULL}};
const Segment SGI147924[] = {{&S30875[44], 11}, {NULL}};
const Segment SGI147925[] = {{&S30875[44], 23}, {NULL}};
const Segment SGI147926[] = {{&S30875[30], 6}, {NULL}};
const Segment SGI147927[] = {{&S30875[116], 3}, {NULL}};
const Segment SGI147928[] = {{&S30875[81], 6}, {NULL}};
const Segment SGI147929[] = {{&S30876[44], 15}, {NULL}};
const Segment SGI147930[] = {{&S30876[44], 27}, {NULL}};
const Segment SGI147931[] = {{&S30876[30], 6}, {NULL}};
const Segment SGI147932[] = {{&S30876[120], 3}, {NULL}};
const Segment SGI147933[] = {{&S30876[85], 6}, {NULL}};
const Segment SGI147934[] = {{&S30879[44], 15}, {NULL}};
const Segment SGI147935[] = {{&S30879[44], 27}, {NULL}};
const Segment SGI147936[] = {{&S30879[30], 6}, {NULL}};
const Segment SGI147937[] = {{&S30879[120], 3}, {NULL}};
const Segment SGI147938[] = {{&S30879[85], 6}, {NULL}};
const Segment SGI147939[] = {{&S30880[44], 10}, {NULL}};
const Segment SGI147940[] = {{&S30880[44], 22}, {NULL}};
const Segment SGI147941[] = {{&S30880[30], 6}, {NULL}};
const Segment SGI147942[] = {{&S30880[115], 3}, {NULL}};
const Segment SGI147943[] = {{&S30880[80], 6}, {NULL}};
const Segment SGI147944[] = {{&S30881[44], 13}, {NULL}};
const Segment SGI147945[] = {{&S30881[44], 25}, {NULL}};
const Segment SGI147946[] = {{&S30881[30], 6}, {NULL}};
const Segment SGI147947[] = {{&S30881[118], 3}, {NULL}};
const Segment SGI147948[] = {{&S30881[83], 6}, {NULL}};
const Segment SGI147949[] = {{&S45537[34], 10}, {NULL}};
const Segment SGI147950[] = {{&S45537[27], 6}, {NULL}};
const Segment SGI147951[] = {{&S45537[97], 9}, {NULL}};
const Segment SGI147952[] = {{&S45537[70], 6}, {NULL}};
const Segment SGI147953[] = {{&S30883[44], 10}, {NULL}};
const Segment SGI147954[] = {{&S30883[44], 22}, {NULL}};
const Segment SGI147955[] = {{&S45538[34], 14}, {NULL}};
const Segment SGI147956[] = {{&S30883[30], 6}, {NULL}};
const Segment SGI147957[] = {{&S45538[27], 6}, {NULL}};
const Segment SGI147958[] = {{&S30883[115], 3}, {NULL}};
const Segment SGI147959[] = {{&S30883[80], 6}, {NULL}};
const Segment SGI147960[] = {{&S45538[101], 9}, {NULL}};
const Segment SGI147961[] = {{&S45538[74], 6}, {NULL}};
const Segment SGI147962[] = {{&S45539[34], 13}, {NULL}};
const Segment SGI147963[] = {{&S45539[27], 6}, {NULL}};
const Segment SGI147964[] = {{&S30885[44], 8}, {NULL}};
const Segment SGI147965[] = {{&S30885[44], 20}, {NULL}};
const Segment SGI147966[] = {{&S30885[30], 6}, {NULL}};
const Segment SGI147967[] = {{&S45539[100], 9}, {NULL}};
const Segment SGI147968[] = {{&S45539[73], 6}, {NULL}};
const Segment SGI147969[] = {{&S30885[113], 3}, {NULL}};
const Segment SGI147970[] = {{&S30885[78], 6}, {NULL}};
const Segment SGI147971[] = {{&S30887[44], 7}, {NULL}};
const Segment SGI147972[] = {{&S30887[44], 19}, {NULL}};
const Segment SGI147973[] = {{&S30887[30], 6}, {NULL}};
const Segment SGI147974[] = {{&S30887[112], 3}, {NULL}};
const Segment SGI147975[] = {{&S30887[77], 6}, {NULL}};
const Segment SGI147976[] = {{&S30890[44], 6}, {NULL}};
const Segment SGI147977[] = {{&S30890[44], 18}, {NULL}};
const Segment SGI147978[] = {{&S30890[30], 6}, {NULL}};
const Segment SGI147979[] = {{&S30890[111], 3}, {NULL}};
const Segment SGI147980[] = {{&S30890[76], 6}, {NULL}};
const Segment SGI147981[] = {{&S30892[44], 6}, {NULL}};
const Segment SGI147982[] = {{&S30892[44], 18}, {NULL}};
const Segment SGI147983[] = {{&S30892[30], 6}, {NULL}};
const Segment SGI147984[] = {{&S30892[111], 3}, {NULL}};
const Segment SGI147985[] = {{&S30892[76], 6}, {NULL}};
const Segment SGI147986[] = {{&S30893[44], 10}, {NULL}};
const Segment SGI147987[] = {{&S30893[44], 22}, {NULL}};
const Segment SGI147988[] = {{&S30893[30], 6}, {NULL}};
const Segment SGI147989[] = {{&S30893[115], 3}, {NULL}};
const Segment SGI147990[] = {{&S30893[80], 6}, {NULL}};
const Segment SGI147991[] = {{&S30894[44], 10}, {NULL}};
const Segment SGI147992[] = {{&S30894[44], 22}, {NULL}};
const Segment SGI147993[] = {{&S30894[30], 6}, {NULL}};
const Segment SGI147994[] = {{&S30894[115], 3}, {NULL}};
const Segment SGI147995[] = {{&S30894[80], 6}, {NULL}};
const Segment SGI147996[] = {{&S30897[44], 11}, {NULL}};
const Segment SGI147997[] = {{&S30897[44], 23}, {NULL}};
const Segment SGI147998[] = {{&S30897[30], 6}, {NULL}};
const Segment SGI147999[] = {{&S30897[116], 3}, {NULL}};
const Segment SGI148000[] = {{&S30897[81], 6}, {NULL}};
const Segment SGI148001[] = {{&S30899[44], 23}, {NULL}};
const Segment SGI148002[] = {{&S30899[44], 35}, {NULL}};
const Segment SGI148003[] = {{&S30899[30], 6}, {NULL}};
const Segment SGI148004[] = {{&S30899[128], 3}, {NULL}};
const Segment SGI148005[] = {{&S30899[93], 6}, {NULL}};
const Segment SGI148006[] = {{&S30903[44], 11}, {NULL}};
const Segment SGI148007[] = {{&S30903[44], 23}, {NULL}};
const Segment SGI148008[] = {{&S30903[30], 6}, {NULL}};
const Segment SGI148009[] = {{&S30903[116], 3}, {NULL}};
const Segment SGI148010[] = {{&S30903[81], 6}, {NULL}};
const Segment SGI148011[] = {{&S30904[44], 9}, {NULL}};
const Segment SGI148012[] = {{&S30904[44], 21}, {NULL}};
const Segment SGI148013[] = {{&S30904[30], 6}, {NULL}};
const Segment SGI148014[] = {{&S30904[114], 3}, {NULL}};
const Segment SGI148015[] = {{&S30904[79], 6}, {NULL}};
const Segment SGI148016[] = {{&S30906[44], 14}, {NULL}};
const Segment SGI148017[] = {{&S30906[44], 26}, {NULL}};
const Segment SGI148018[] = {{&S30906[30], 6}, {NULL}};
const Segment SGI148019[] = {{&S30906[119], 3}, {NULL}};
const Segment SGI148020[] = {{&S30906[84], 6}, {NULL}};
const Segment SGI148021[] = {{&S45569[34], 10}, {NULL}};
const Segment SGI148022[] = {{&S45569[27], 6}, {NULL}};
const Segment SGI148023[] = {{&S45569[97], 9}, {NULL}};
const Segment SGI148024[] = {{&S45569[70], 6}, {NULL}};
const Segment SGI148025[] = {{&S45570[34], 13}, {NULL}};
const Segment SGI148026[] = {{&S45570[27], 6}, {NULL}};
const Segment SGI148027[] = {{&S45570[100], 9}, {NULL}};
const Segment SGI148028[] = {{&S45570[73], 6}, {NULL}};
const Segment SGI148029[] = {{&S30912[44], 16}, {NULL}};
const Segment SGI148030[] = {{&S30912[44], 28}, {NULL}};
const Segment SGI148031[] = {{&S30912[30], 6}, {NULL}};
const Segment SGI148032[] = {{&S30912[121], 3}, {NULL}};
const Segment SGI148033[] = {{&S30912[86], 6}, {NULL}};
const Segment SGI148034[] = {{&S36272[34], 7}, {NULL}};
const Segment SGI148035[] = {{&S36272[27], 6}, {NULL}};
const Segment SGI148036[] = {{&S30913[44], 17}, {NULL}};
const Segment SGI148037[] = {{&S30913[44], 29}, {NULL}};
const Segment SGI148038[] = {{&S30913[30], 6}, {NULL}};
const Segment SGI148039[] = {{&S36272[99], 9}, {NULL}};
const Segment SGI148040[] = {{&S36272[72], 6}, {NULL}};
const Segment SGI148041[] = {{&S30913[122], 3}, {NULL}};
const Segment SGI148042[] = {{&S30913[87], 6}, {NULL}};
const Segment SGI148043[] = {{&S30914[44], 11}, {NULL}};
const Segment SGI148044[] = {{&S36273[34], 7}, {NULL}};
const Segment SGI148045[] = {{&S30914[44], 23}, {NULL}};
const Segment SGI148046[] = {{&S30914[30], 6}, {NULL}};
const Segment SGI148047[] = {{&S36273[27], 6}, {NULL}};
const Segment SGI148048[] = {{&S30914[116], 3}, {NULL}};
const Segment SGI148049[] = {{&S36273[99], 9}, {NULL}};
const Segment SGI148050[] = {{&S30914[81], 6}, {NULL}};
const Segment SGI148051[] = {{&S36273[72], 6}, {NULL}};
const Segment SGI148052[] = {{&S30916[44], 10}, {NULL}};
const Segment SGI148053[] = {{&S30916[44], 22}, {NULL}};
const Segment SGI148054[] = {{&S30916[30], 6}, {NULL}};
const Segment SGI148055[] = {{&S30916[115], 3}, {NULL}};
const Segment SGI148056[] = {{&S30916[80], 6}, {NULL}};
const Segment SGI148057[] = {{&S30917[44], 12}, {NULL}};
const Segment SGI148058[] = {{&S30917[44], 24}, {NULL}};
const Segment SGI148059[] = {{&S30917[30], 6}, {NULL}};
const Segment SGI148060[] = {{&S30917[117], 3}, {NULL}};
const Segment SGI148061[] = {{&S30917[82], 6}, {NULL}};
const Segment SGI148062[] = {{&S45582[44], 10}, {NULL}};
const Segment SGI148063[] = {{&S45582[44], 22}, {NULL}};
const Segment SGI148064[] = {{&S45582[30], 6}, {NULL}};
const Segment SGI148065[] = {{&S45582[31], 3}, {NULL}};
const Segment SGI148066[] = {{&S45582[99], 6}, {NULL}};
const Segment SGI148067[] = {{&S45582[68], 8}, {NULL}};
const Segment SGI148068[] = {{&S30918[44], 13}, {NULL}};
const Segment SGI148069[] = {{&S30918[44], 25}, {NULL}};
const Segment SGI148070[] = {{&S30918[30], 6}, {NULL}};
const Segment SGI148071[] = {{&S30918[118], 3}, {NULL}};
const Segment SGI148072[] = {{&S30918[83], 6}, {NULL}};
const Segment SGI148073[] = {{&S45583[44], 9}, {NULL}};
const Segment SGI148074[] = {{&S45583[44], 21}, {NULL}};
const Segment SGI148075[] = {{&S45583[30], 6}, {NULL}};
const Segment SGI148076[] = {{&S45583[31], 3}, {NULL}};
const Segment SGI148077[] = {{&S30919[44], 7}, {NULL}};
const Segment SGI148078[] = {{&S45583[103], 6}, {NULL}};
const Segment SGI148079[] = {{&S30919[44], 19}, {NULL}};
const Segment SGI148080[] = {{&S30919[30], 6}, {NULL}};
const Segment SGI148081[] = {{&S45583[67], 8}, {NULL}};
const Segment SGI148082[] = {{&S30919[112], 3}, {NULL}};
const Segment SGI148083[] = {{&S30919[77], 6}, {NULL}};
const Segment SGI148084[] = {{&S45584[44], 9}, {NULL}};
const Segment SGI148085[] = {{&S45584[44], 21}, {NULL}};
const Segment SGI148086[] = {{&S45584[30], 6}, {NULL}};
const Segment SGI148087[] = {{&S30920[44], 10}, {NULL}};
const Segment SGI148088[] = {{&S30920[44], 22}, {NULL}};
const Segment SGI148089[] = {{&S30920[30], 6}, {NULL}};
const Segment SGI148090[] = {{&S45584[31], 3}, {NULL}};
const Segment SGI148091[] = {{&S45584[94], 6}, {NULL}};
const Segment SGI148092[] = {{&S30920[115], 3}, {NULL}};
const Segment SGI148093[] = {{&S30920[80], 6}, {NULL}};
const Segment SGI148094[] = {{&S45584[67], 8}, {NULL}};
const Segment SGI148095[] = {{&S30921[44], 10}, {NULL}};
const Segment SGI148096[] = {{&S45585[44], 10}, {NULL}};
const Segment SGI148097[] = {{&S30921[44], 31}, {NULL}};
const Segment SGI148098[] = {{&S45585[44], 22}, {NULL}};
const Segment SGI148099[] = {{&S30921[30], 6}, {NULL}};
const Segment SGI148100[] = {{&S45585[30], 6}, {NULL}};
const Segment SGI148101[] = {{&S30921[124], 3}, {NULL}};
const Segment SGI148102[] = {{&S45585[31], 3}, {NULL}};
const Segment SGI148103[] = {{&S30921[89], 6}, {NULL}};
const Segment SGI148104[] = {{&S45585[101], 6}, {NULL}};
const Segment SGI148105[] = {{&S45585[68], 8}, {NULL}};
const Segment SGI148106[] = {{&S30922[44], 7}, {NULL}};
const Segment SGI148107[] = {{&S30922[44], 19}, {NULL}};
const Segment SGI148108[] = {{&S30922[30], 6}, {NULL}};
const Segment SGI148109[] = {{&S45586[44], 9}, {NULL}};
const Segment SGI148110[] = {{&S45586[44], 21}, {NULL}};
const Segment SGI148111[] = {{&S30922[112], 3}, {NULL}};
const Segment SGI148112[] = {{&S45586[30], 6}, {NULL}};
const Segment SGI148113[] = {{&S30922[77], 6}, {NULL}};
const Segment SGI148114[] = {{&S45586[31], 3}, {NULL}};
const Segment SGI148115[] = {{&S45586[114], 6}, {NULL}};
const Segment SGI148116[] = {{&S45586[67], 8}, {&S45586[67], 8}, {NULL}};
const Segment SGI148117[] = {{&S45587[44], 9}, {NULL}};
const Segment SGI148118[] = {{&S45587[44], 21}, {NULL}};
const Segment SGI148119[] = {{&S30924[44], 17}, {NULL}};
const Segment SGI148120[] = {{&S45587[30], 6}, {NULL}};
const Segment SGI148121[] = {{&S30924[44], 29}, {NULL}};
const Segment SGI148122[] = {{&S30924[30], 6}, {NULL}};
const Segment SGI148123[] = {{&S45587[31], 3}, {NULL}};
const Segment SGI148124[] = {{&S30924[122], 3}, {NULL}};
const Segment SGI148125[] = {{&S45587[110], 6}, {NULL}};
const Segment SGI148126[] = {{&S30924[87], 6}, {NULL}};
const Segment SGI148127[] = {{&S45587[67], 8}, {NULL}};
const Segment SGI148128[] = {{&S45588[44], 5}, {NULL}};
const Segment SGI148129[] = {{&S45588[44], 17}, {NULL}};
const Segment SGI148130[] = {{&S45588[30], 6}, {NULL}};
const Segment SGI148131[] = {{&S45588[31], 3}, {NULL}};
const Segment SGI148132[] = {{&S45588[100], 6}, {NULL}};
const Segment SGI148133[] = {{&S45588[63], 8}, {NULL}};
const Segment SGI148134[] = {{&S30927[44], 9}, {NULL}};
const Segment SGI148135[] = {{&S30927[44], 21}, {NULL}};
const Segment SGI148136[] = {{&S30927[30], 6}, {NULL}};
const Segment SGI148137[] = {{&S30927[114], 3}, {NULL}};
const Segment SGI148138[] = {{&S30927[79], 6}, {NULL}};
const Segment SGI148139[] = {{&S30928[44], 16}, {NULL}};
const Segment SGI148140[] = {{&S30928[44], 28}, {NULL}};
const Segment SGI148141[] = {{&S30928[30], 6}, {NULL}};
const Segment SGI148142[] = {{&S30928[121], 3}, {NULL}};
const Segment SGI148143[] = {{&S30928[86], 6}, {NULL}};
const Segment SGI148144[] = {{&S30929[38], 17}, {NULL}};
const Segment SGI148145[] = {{&S30929[31], 6}, {NULL}};
const Segment SGI148146[] = {{&S34006[32], 9}, {NULL}};
const Segment SGI148147[] = {{&S34006[27], 4}, {NULL}};
const Segment SGI148148[] = {{&S34006[94], 9}, {NULL}};
const Segment SGI148149[] = {{&S34006[67], 6}, {NULL}};
const Segment SGI148150[] = {{&S30934[44], 19}, {NULL}};
const Segment SGI148151[] = {{&S30934[44], 31}, {NULL}};
const Segment SGI148152[] = {{&S30934[30], 6}, {NULL}};
const Segment SGI148153[] = {{&S30934[124], 3}, {NULL}};
const Segment SGI148154[] = {{&S30934[89], 6}, {NULL}};
const Segment SGI148155[] = {{&S30937[44], 10}, {NULL}};
const Segment SGI148156[] = {{&S30937[44], 22}, {NULL}};
const Segment SGI148157[] = {{&S30937[30], 6}, {NULL}};
const Segment SGI148158[] = {{&S30937[115], 3}, {NULL}};
const Segment SGI148159[] = {{&S30937[80], 6}, {NULL}};
const Segment SGI148160[] = {{&S30938[44], 5}, {NULL}};
const Segment SGI148161[] = {{&S30938[44], 17}, {NULL}};
const Segment SGI148162[] = {{&S30938[30], 6}, {NULL}};
const Segment SGI148163[] = {{&S30938[110], 3}, {NULL}};
const Segment SGI148164[] = {{&S30938[75], 6}, {NULL}};
const Segment SGI148165[] = {{&S30939[44], 6}, {NULL}};
const Segment SGI148166[] = {{&S30939[44], 18}, {NULL}};
const Segment SGI148167[] = {{&S30939[30], 6}, {NULL}};
const Segment SGI148168[] = {{&S30939[111], 3}, {NULL}};
const Segment SGI148169[] = {{&S30939[76], 6}, {NULL}};
const Segment SGI148170[] = {{&S30940[44], 11}, {NULL}};
const Segment SGI148171[] = {{&S30940[44], 23}, {NULL}};
const Segment SGI148172[] = {{&S30940[30], 6}, {NULL}};
const Segment SGI148173[] = {{&S30940[116], 3}, {NULL}};
const Segment SGI148174[] = {{&S30940[81], 6}, {NULL}};
const Segment SGI148175[] = {{&S30942[44], 15}, {NULL}};
const Segment SGI148176[] = {{&S30942[44], 27}, {NULL}};
const Segment SGI148177[] = {{&S30942[30], 6}, {NULL}};
const Segment SGI148178[] = {{&S30942[120], 3}, {NULL}};
const Segment SGI148179[] = {{&S30942[85], 6}, {NULL}};
const Segment SGI148180[] = {{&S34033[32], 10}, {NULL}};
const Segment SGI148181[] = {{&S34033[27], 4}, {NULL}};
const Segment SGI148182[] = {{&S30948[44], 7}, {NULL}};
const Segment SGI148183[] = {{&S30948[44], 19}, {NULL}};
const Segment SGI148184[] = {{&S34033[95], 9}, {NULL}};
const Segment SGI148185[] = {{&S30948[30], 6}, {NULL}};
const Segment SGI148186[] = {{&S34033[68], 6}, {NULL}};
const Segment SGI148187[] = {{&S30948[112], 3}, {NULL}};
const Segment SGI148188[] = {{&S30948[77], 6}, {NULL}};
const Segment SGI148189[] = {{&S30953[44], 10}, {NULL}};
const Segment SGI148190[] = {{&S30953[44], 22}, {NULL}};
const Segment SGI148191[] = {{&S30953[30], 6}, {NULL}};
const Segment SGI148192[] = {{&S30953[115], 3}, {NULL}};
const Segment SGI148193[] = {{&S30953[80], 6}, {NULL}};
const Segment SGI148194[] = {{&S34040[32], 9}, {NULL}};
const Segment SGI148195[] = {{&S34040[27], 4}, {NULL}};
const Segment SGI148196[] = {{&S34040[94], 9}, {NULL}};
const Segment SGI148197[] = {{&S34040[67], 6}, {NULL}};
const Segment SGI148198[] = {{&S30960[44], 14}, {NULL}};
const Segment SGI148199[] = {{&S30960[44], 26}, {NULL}};
const Segment SGI148200[] = {{&S30960[30], 6}, {NULL}};
const Segment SGI148201[] = {{&S30960[119], 3}, {NULL}};
const Segment SGI148202[] = {{&S45634[34], 10}, {NULL}};
const Segment SGI148203[] = {{&S30960[84], 6}, {NULL}};
const Segment SGI148204[] = {{&S45634[27], 6}, {NULL}};
const Segment SGI148205[] = {{&S45634[97], 9}, {NULL}};
const Segment SGI148206[] = {{&S45634[70], 6}, {NULL}};
const Segment SGI148207[] = {{&S30961[44], 7}, {NULL}};
const Segment SGI148208[] = {{&S30961[44], 19}, {NULL}};
const Segment SGI148209[] = {{&S30961[30], 6}, {NULL}};
const Segment SGI148210[] = {{&S30961[112], 3}, {NULL}};
const Segment SGI148211[] = {{&S30961[77], 6}, {NULL}};
const Segment SGI148212[] = {{&S30962[44], 11}, {NULL}};
const Segment SGI148213[] = {{&S30962[44], 23}, {NULL}};
const Segment SGI148214[] = {{&S30962[30], 6}, {NULL}};
const Segment SGI148215[] = {{&S30962[116], 3}, {NULL}};
const Segment SGI148216[] = {{&S30962[81], 6}, {NULL}};
const Segment SGI148217[] = {{&S30964[44], 14}, {NULL}};
const Segment SGI148218[] = {{&S30964[44], 26}, {NULL}};
const Segment SGI148219[] = {{&S30964[30], 6}, {NULL}};
const Segment SGI148220[] = {{&S30964[119], 3}, {NULL}};
const Segment SGI148221[] = {{&S30964[84], 6}, {NULL}};
const Segment SGI148222[] = {{&S30965[44], 14}, {NULL}};
const Segment SGI148223[] = {{&S30965[44], 26}, {NULL}};
const Segment SGI148224[] = {{&S30965[30], 6}, {NULL}};
const Segment SGI148225[] = {{&S30965[119], 3}, {NULL}};
const Segment SGI148226[] = {{&S30965[84], 6}, {NULL}};
const Segment SGI148227[] = {{&S30966[44], 7}, {NULL}};
const Segment SGI148228[] = {{&S30966[44], 19}, {NULL}};
const Segment SGI148229[] = {{&S30966[30], 6}, {NULL}};
const Segment SGI148230[] = {{&S30966[112], 3}, {NULL}};
const Segment SGI148231[] = {{&S30966[77], 6}, {NULL}};
const Segment SGI148232[] = {{&S30967[44], 11}, {NULL}};
const Segment SGI148233[] = {{&S30967[44], 23}, {NULL}};
const Segment SGI148234[] = {{&S30967[30], 6}, {NULL}};
const Segment SGI148235[] = {{&S30967[116], 3}, {NULL}};
const Segment SGI148236[] = {{&S30967[81], 6}, {NULL}};
const Segment SGI148237[] = {{&S45646[34], 10}, {NULL}};
const Segment SGI148238[] = {{&S45646[27], 6}, {NULL}};
const Segment SGI148239[] = {{&S45646[97], 9}, {NULL}};
const Segment SGI148240[] = {{&S45646[70], 6}, {NULL}};
const Segment SGI148241[] = {{&S30970[44], 10}, {NULL}};
const Segment SGI148242[] = {{&S30970[44], 22}, {NULL}};
const Segment SGI148243[] = {{&S30970[30], 6}, {NULL}};
const Segment SGI148244[] = {{&S30970[80], 6}, {NULL}};
const Segment SGI148245[] = {{&S45648[34], 11}, {NULL}};
const Segment SGI148246[] = {{&S45648[27], 6}, {NULL}};
const Segment SGI148247[] = {{&S45648[98], 9}, {NULL}};
const Segment SGI148248[] = {{&S45648[71], 6}, {NULL}};
const Segment SGI148249[] = {{&S45650[34], 18}, {NULL}};
const Segment SGI148250[] = {{&S30973[44], 13}, {NULL}};
const Segment SGI148251[] = {{&S30973[44], 25}, {NULL}};
const Segment SGI148252[] = {{&S45650[27], 6}, {NULL}};
const Segment SGI148253[] = {{&S30973[30], 6}, {NULL}};
const Segment SGI148254[] = {{&S30973[118], 3}, {NULL}};
const Segment SGI148255[] = {{&S45650[105], 9}, {NULL}};
const Segment SGI148256[] = {{&S45650[78], 6}, {NULL}};
const Segment SGI148257[] = {{&S30973[83], 6}, {NULL}};
const Segment SGI148258[] = {{&S30975[44], 20}, {NULL}};
const Segment SGI148259[] = {{&S30975[44], 32}, {NULL}};
const Segment SGI148260[] = {{&S30975[30], 6}, {NULL}};
const Segment SGI148261[] = {{&S30975[125], 3}, {NULL}};
const Segment SGI148262[] = {{&S30975[90], 6}, {NULL}};
const Segment SGI148263[] = {{&S30984[44], 13}, {NULL}};
const Segment SGI148264[] = {{&S30984[44], 25}, {NULL}};
const Segment SGI148265[] = {{&S30984[30], 6}, {NULL}};
const Segment SGI148266[] = {{&S30984[118], 3}, {NULL}};
const Segment SGI148267[] = {{&S30984[83], 6}, {NULL}};
const Segment SGI148268[] = {{&S45665[44], 10}, {NULL}};
const Segment SGI148269[] = {{&S45665[44], 22}, {NULL}};
const Segment SGI148270[] = {{&S45665[30], 6}, {NULL}};
const Segment SGI148271[] = {{&S45665[31], 3}, {NULL}};
const Segment SGI148272[] = {{&S45665[80], 6}, {NULL}};
const Segment SGI148273[] = {{&S30987[44], 17}, {NULL}};
const Segment SGI148274[] = {{&S30987[44], 29}, {NULL}};
const Segment SGI148275[] = {{&S30987[30], 6}, {NULL}};
const Segment SGI148276[] = {{&S30987[122], 3}, {NULL}};
const Segment SGI148277[] = {{&S30987[87], 6}, {NULL}};
const Segment SGI148278[] = {{&S45666[38], 10}, {NULL}};
const Segment SGI148279[] = {{&S45666[31], 6}, {NULL}};
const Segment SGI148280[] = {{&S30991[44], 8}, {NULL}};
const Segment SGI148281[] = {{&S45669[42], 10}, {NULL}};
const Segment SGI148282[] = {{&S30991[44], 20}, {NULL}};
const Segment SGI148283[] = {{&S45669[30], 6}, {NULL}};
const Segment SGI148284[] = {{&S30991[30], 6}, {NULL}};
const Segment SGI148285[] = {{&S45669[31], 3}, {NULL}};
const Segment SGI148286[] = {{&S30991[113], 3}, {NULL}};
const Segment SGI148287[] = {{&S45669[78], 6}, {NULL}};
const Segment SGI148288[] = {{&S30991[78], 6}, {NULL}};
const Segment SGI148289[] = {{&S30992[44], 10}, {NULL}};
const Segment SGI148290[] = {{&S30992[44], 22}, {NULL}};
const Segment SGI148291[] = {{&S45670[44], 10}, {NULL}};
const Segment SGI148292[] = {{&S30992[30], 6}, {NULL}};
const Segment SGI148293[] = {{&S45670[44], 22}, {NULL}};
const Segment SGI148294[] = {{&S45670[30], 6}, {NULL}};
const Segment SGI148295[] = {{&S30992[115], 3}, {NULL}};
const Segment SGI148296[] = {{&S30992[80], 6}, {NULL}};
const Segment SGI148297[] = {{&S45670[80], 6}, {NULL}};
const Segment SGI148298[] = {{&S30993[44], 4}, {NULL}};
const Segment SGI148299[] = {{&S45671[45], 10}, {NULL}};
const Segment SGI148300[] = {{&S30993[44], 16}, {NULL}};
const Segment SGI148301[] = {{&S45671[45], 28}, {NULL}};
const Segment SGI148302[] = {{&S30993[30], 6}, {NULL}};
const Segment SGI148303[] = {{&S45671[31], 6}, {NULL}};
const Segment SGI148304[] = {{&S30993[109], 3}, {NULL}};
const Segment SGI148305[] = {{&S30993[74], 6}, {NULL}};
const Segment SGI148306[] = {{&S30994[44], 11}, {NULL}};
const Segment SGI148307[] = {{&S30994[44], 23}, {NULL}};
const Segment SGI148308[] = {{&S45672[44], 10}, {NULL}};
const Segment SGI148309[] = {{&S30994[30], 6}, {NULL}};
const Segment SGI148310[] = {{&S45672[44], 22}, {NULL}};
const Segment SGI148311[] = {{&S45672[30], 6}, {NULL}};
const Segment SGI148312[] = {{&S30994[116], 3}, {NULL}};
const Segment SGI148313[] = {{&S45672[31], 3}, {NULL}};
const Segment SGI148314[] = {{&S30994[81], 6}, {NULL}};
const Segment SGI148315[] = {{&S45672[80], 6}, {NULL}};
const Segment SGI148316[] = {{&S45673[44], 10}, {NULL}};
const Segment SGI148317[] = {{&S45673[44], 22}, {NULL}};
const Segment SGI148318[] = {{&S45673[30], 6}, {NULL}};
const Segment SGI148319[] = {{&S45673[31], 3}, {NULL}};
const Segment SGI148320[] = {{&S30996[44], 8}, {NULL}};
const Segment SGI148321[] = {{&S45673[80], 6}, {NULL}};
const Segment SGI148322[] = {{&S30996[44], 20}, {NULL}};
const Segment SGI148323[] = {{&S30996[30], 6}, {NULL}};
const Segment SGI148324[] = {{&S30996[113], 3}, {NULL}};
const Segment SGI148325[] = {{&S30996[78], 6}, {NULL}};
const Segment SGI148326[] = {{&S45674[44], 10}, {NULL}};
const Segment SGI148327[] = {{&S45674[44], 22}, {NULL}};
const Segment SGI148328[] = {{&S45674[30], 6}, {NULL}};
const Segment SGI148329[] = {{&S45674[31], 3}, {NULL}};
const Segment SGI148330[] = {{&S45674[80], 6}, {NULL}};
const Segment SGI148331[] = {{&S30999[44], 11}, {NULL}};
const Segment SGI148332[] = {{&S30999[44], 23}, {NULL}};
const Segment SGI148333[] = {{&S30999[30], 6}, {NULL}};
const Segment SGI148334[] = {{&S30999[116], 3}, {NULL}};
const Segment SGI148335[] = {{&S30999[81], 6}, {NULL}};
const Segment SGI148336[] = {{&S31004[44], 13}, {NULL}};
const Segment SGI148337[] = {{&S31004[44], 25}, {NULL}};
const Segment SGI148338[] = {{&S31004[30], 6}, {NULL}};
const Segment SGI148339[] = {{&S31004[118], 3}, {NULL}};
const Segment SGI148340[] = {{&S31004[83], 6}, {NULL}};
const Segment SGI148341[] = {{&S31007[34], 9}, {NULL}};
const Segment SGI148342[] = {{&S31007[27], 6}, {NULL}};
const Segment SGI148343[] = {{&S31007[96], 9}, {NULL}};
const Segment SGI148344[] = {{&S31007[69], 6}, {NULL}};
const Segment SGI148345[] = {{&S31008[34], 9}, {NULL}};
const Segment SGI148346[] = {{&S31008[27], 6}, {NULL}};
const Segment SGI148347[] = {{&S31008[96], 9}, {NULL}};
const Segment SGI148348[] = {{&S31008[69], 6}, {NULL}};
const Segment SGI148349[] = {{&S31011[41], 9}, {NULL}};
const Segment SGI148350[] = {{&S31011[41], 21}, {NULL}};
const Segment SGI148351[] = {{&S31011[27], 6}, {NULL}};
const Segment SGI148352[] = {{&S31011[102], 9}, {NULL}};
const Segment SGI148353[] = {{&S31011[76], 6}, {NULL}};
const Segment SGI148354[] = {{&S45717[44], 15}, {NULL}};
const Segment SGI148355[] = {{&S45717[44], 27}, {NULL}};
const Segment SGI148356[] = {{&S45717[30], 6}, {NULL}};
const Segment SGI148357[] = {{&S45717[31], 3}, {NULL}};
const Segment SGI148358[] = {{&S45717[85], 6}, {NULL}};
const Segment SGI148359[] = {{&S45718[38], 15}, {NULL}};
const Segment SGI148360[] = {{&S45718[31], 6}, {NULL}};
const Segment SGI148361[] = {{&S45719[42], 15}, {NULL}};
const Segment SGI148362[] = {{&S45719[30], 6}, {NULL}};
const Segment SGI148363[] = {{&S45719[31], 3}, {NULL}};
const Segment SGI148364[] = {{&S45719[83], 6}, {NULL}};
const Segment SGI148365[] = {{&S45720[44], 15}, {NULL}};
const Segment SGI148366[] = {{&S45720[44], 27}, {NULL}};
const Segment SGI148367[] = {{&S45720[30], 6}, {NULL}};
const Segment SGI148368[] = {{&S45720[88], 4}, {NULL}};
const Segment SGI148369[] = {{&S45720[88], 3}, {NULL}};
const Segment SGI148370[] = {{&S45734[44], 5}, {NULL}};
const Segment SGI148371[] = {{&S45734[44], 17}, {NULL}};
const Segment SGI148372[] = {{&S45734[30], 6}, {NULL}};
const Segment SGI148373[] = {{&S45734[31], 3}, {NULL}};
const Segment SGI148374[] = {{&S45734[75], 6}, {NULL}};
const Segment SGI148375[] = {{&S45736[44], 9}, {NULL}};
const Segment SGI148376[] = {{&S45736[44], 21}, {NULL}};
const Segment SGI148377[] = {{&S45736[30], 6}, {NULL}};
const Segment SGI148378[] = {{&S35983[34], 7}, {NULL}};
const Segment SGI148379[] = {{&S35983[27], 6}, {NULL}};
const Segment SGI148380[] = {{&S45736[31], 3}, {NULL}};
const Segment SGI148381[] = {{&S45736[79], 6}, {NULL}};
const Segment SGI148382[] = {{&S35983[111], 9}, {NULL}};
const Segment SGI148383[] = {{&S35983[84], 6}, {NULL}};
const Segment SGI148384[] = {{&S45737[44], 13}, {NULL}};
const Segment SGI148385[] = {{&S45737[44], 25}, {NULL}};
const Segment SGI148386[] = {{&S45737[30], 6}, {NULL}};
const Segment SGI148387[] = {{&S45737[31], 3}, {NULL}};
const Segment SGI148388[] = {{&S45737[83], 6}, {NULL}};
const Segment SGI148389[] = {{&S35985[34], 15}, {NULL}};
const Segment SGI148390[] = {{&S35985[27], 6}, {NULL}};
const Segment SGI148391[] = {{&S35985[114], 9}, {NULL}};
const Segment SGI148392[] = {{&S35985[87], 6}, {NULL}};
const Segment SGI148393[] = {{&S45739[34], 10}, {NULL}};
const Segment SGI148394[] = {{&S45739[27], 6}, {NULL}};
const Segment SGI148395[] = {{&S35986[34], 9}, {NULL}};
const Segment SGI148396[] = {{&S35986[27], 6}, {NULL}};
const Segment SGI148397[] = {{&S45739[97], 9}, {NULL}};
const Segment SGI148398[] = {{&S45739[70], 6}, {NULL}};
const Segment SGI148399[] = {{&S35986[110], 9}, {NULL}};
const Segment SGI148400[] = {{&S35986[83], 6}, {NULL}};
const Segment SGI148401[] = {{&S45740[34], 10}, {NULL}};
const Segment SGI148402[] = {{&S45740[27], 6}, {NULL}};
const Segment SGI148403[] = {{&S35987[34], 7}, {NULL}};
const Segment SGI148404[] = {{&S35987[27], 6}, {NULL}};
const Segment SGI148405[] = {{&S45740[97], 9}, {NULL}};
const Segment SGI148406[] = {{&S45740[70], 6}, {NULL}};
const Segment SGI148407[] = {{&S35987[111], 9}, {NULL}};
const Segment SGI148408[] = {{&S35987[84], 6}, {NULL}};
const Segment SGI148409[] = {{&S35988[34], 15}, {NULL}};
const Segment SGI148410[] = {{&S35988[27], 6}, {NULL}};
const Segment SGI148411[] = {{&S35988[114], 9}, {NULL}};
const Segment SGI148412[] = {{&S35988[87], 6}, {NULL}};
const Segment SGI148413[] = {{&S45811[44], 7}, {NULL}};
const Segment SGI148414[] = {{&S45811[44], 29}, {NULL}};
const Segment SGI148415[] = {{&S45811[30], 6}, {NULL}};
const Segment SGI148416[] = {{&S45811[31], 3}, {NULL}};
const Segment SGI148417[] = {{&S45811[87], 6}, {NULL}};
const Segment SGI148418[] = {{&S45812[44], 7}, {NULL}};
const Segment SGI148419[] = {{&S45812[44], 33}, {NULL}};
const Segment SGI148420[] = {{&S45812[30], 6}, {NULL}};
const Segment SGI148421[] = {{&S45812[31], 3}, {NULL}};
const Segment SGI148422[] = {{&S45812[91], 6}, {NULL}};
const Segment SGI148423[] = {{&S45813[44], 7}, {NULL}};
const Segment SGI148424[] = {{&S45813[44], 22}, {NULL}};
const Segment SGI148425[] = {{&S45813[30], 6}, {NULL}};
const Segment SGI148426[] = {{&S45813[31], 3}, {NULL}};
const Segment SGI148427[] = {{&S45813[80], 6}, {NULL}};
const Segment SGI148428[] = {{&S45814[38], 7}, {NULL}};
const Segment SGI148429[] = {{&S45814[31], 6}, {NULL}};
const Segment SGI148430[] = {{&S36111[44], 9}, {NULL}};
const Segment SGI148431[] = {{&S36111[44], 21}, {NULL}};
const Segment SGI148432[] = {{&S36111[30], 6}, {NULL}};
const Segment SGI148433[] = {{&S36111[114], 3}, {NULL}};
const Segment SGI148434[] = {{&S36111[79], 6}, {NULL}};
const Segment SGI148435[] = {{&S36114[38], 5}, {NULL}};
const Segment SGI148436[] = {{&S36114[31], 6}, {NULL}};
const Segment SGI148437[] = {{&S36118[44], 5}, {NULL}};
const Segment SGI148438[] = {{&S36118[44], 17}, {NULL}};
const Segment SGI148439[] = {{&S36118[30], 6}, {NULL}};
const Segment SGI148440[] = {{&S36118[110], 3}, {NULL}};
const Segment SGI148441[] = {{&S36118[75], 6}, {NULL}};
const Segment SGI148442[] = {{&S45901[34], 7}, {NULL}};
const Segment SGI148443[] = {{&S45901[27], 6}, {NULL}};
const Segment SGI148444[] = {{&S45901[93], 9}, {NULL}};
const Segment SGI148445[] = {{&S45901[66], 6}, {NULL}};
const Segment SGI148446[] = {{&S45902[34], 7}, {NULL}};
const Segment SGI148447[] = {{&S45902[27], 6}, {NULL}};
const Segment SGI148448[] = {{&S45902[94], 9}, {NULL}};
const Segment SGI148449[] = {{&S45902[67], 6}, {NULL}};
const Segment SGI148450[] = {{&S36190[44], 10}, {NULL}};
const Segment SGI148451[] = {{&S36190[44], 22}, {NULL}};
const Segment SGI148452[] = {{&S36190[30], 6}, {NULL}};
const Segment SGI148453[] = {{&S36190[115], 3}, {NULL}};
const Segment SGI148454[] = {{&S36190[80], 6}, {NULL}};
const Segment SGI148455[] = {{&S36191[44], 18}, {NULL}};
const Segment SGI148456[] = {{&S36191[44], 30}, {NULL}};
const Segment SGI148457[] = {{&S36191[30], 6}, {NULL}};
const Segment SGI148458[] = {{&S36191[123], 3}, {NULL}};
const Segment SGI148459[] = {{&S36191[88], 6}, {NULL}};
const Segment SGI148460[] = {{&S36193[44], 10}, {NULL}};
const Segment SGI148461[] = {{&S36193[44], 22}, {NULL}};
const Segment SGI148462[] = {{&S36193[30], 6}, {NULL}};
const Segment SGI148463[] = {{&S36193[115], 3}, {NULL}};
const Segment SGI148464[] = {{&S36193[80], 6}, {NULL}};
const Segment SGI148465[] = {{&S36194[38], 18}, {NULL}};
const Segment SGI148466[] = {{&S36194[31], 6}, {NULL}};
const Segment SGI148467[] = {{&S36196[44], 18}, {NULL}};
const Segment SGI148468[] = {{&S36196[44], 35}, {NULL}};
const Segment SGI148469[] = {{&S36196[30], 6}, {NULL}};
const Segment SGI148470[] = {{&S36196[128], 3}, {NULL}};
const Segment SGI148471[] = {{&S36196[93], 6}, {NULL}};
const Segment SGI148472[] = {{&S36197[38], 10}, {NULL}};
const Segment SGI148473[] = {{&S36197[31], 6}, {NULL}};
const Segment SGI148474[] = {{&S36198[44], 18}, {NULL}};
const Segment SGI148475[] = {{&S36198[44], 30}, {NULL}};
const Segment SGI148476[] = {{&S36198[30], 6}, {NULL}};
const Segment SGI148477[] = {{&S36198[123], 3}, {NULL}};
const Segment SGI148478[] = {{&S36198[88], 6}, {NULL}};
const Segment SGI148479[] = {{&S36199[44], 11}, {NULL}};
const Segment SGI148480[] = {{&S36199[44], 23}, {NULL}};
const Segment SGI148481[] = {{&S36199[30], 6}, {NULL}};
const Segment SGI148482[] = {{&S36199[116], 3}, {NULL}};
const Segment SGI148483[] = {{&S36199[81], 6}, {NULL}};
const Segment SGI148484[] = {{&S36200[44], 18}, {NULL}};
const Segment SGI148485[] = {{&S36200[44], 45}, {NULL}};
const Segment SGI148486[] = {{&S36200[30], 6}, {NULL}};
const Segment SGI148487[] = {{&S36200[138], 3}, {NULL}};
const Segment SGI148488[] = {{&S36200[103], 6}, {NULL}};
const Segment SGI148489[] = {{&S36201[38], 11}, {NULL}};
const Segment SGI148490[] = {{&S36201[31], 6}, {NULL}};
const Segment SGI148491[] = {{&S36202[42], 10}, {NULL}};
const Segment SGI148492[] = {{&S36202[30], 6}, {NULL}};
const Segment SGI148493[] = {{&S36202[113], 3}, {NULL}};
const Segment SGI148494[] = {{&S36202[78], 6}, {NULL}};
const Segment SGI148495[] = {{&S45953[44], 7}, {NULL}};
const Segment SGI148496[] = {{&S45953[44], 19}, {NULL}};
const Segment SGI148497[] = {{&S45953[30], 6}, {NULL}};
const Segment SGI148498[] = {{&S45953[31], 3}, {NULL}};
const Segment SGI148499[] = {{&S45953[77], 6}, {NULL}};
const Segment SGI148500[] = {{&S45954[38], 7}, {NULL}};
const Segment SGI148501[] = {{&S45954[31], 6}, {NULL}};
const Segment SGI148502[] = {{&S36274[34], 7}, {NULL}};
const Segment SGI148503[] = {{&S36274[27], 6}, {NULL}};
const Segment SGI148504[] = {{&S36274[103], 9}, {NULL}};
const Segment SGI148505[] = {{&S36274[76], 6}, {NULL}};
const Segment SGI148506[] = {{&S36275[34], 7}, {NULL}};
const Segment SGI148507[] = {{&S36275[27], 6}, {NULL}};
const Segment SGI148508[] = {{&S36275[103], 9}, {NULL}};
const Segment SGI148509[] = {{&S36275[76], 6}, {NULL}};
const Segment SGI148510[] = {{&S36276[34], 7}, {NULL}};
const Segment SGI148511[] = {{&S36276[27], 6}, {NULL}};
const Segment SGI148512[] = {{&S36276[97], 9}, {NULL}};
const Segment SGI148513[] = {{&S36276[70], 6}, {NULL}};
const Segment SGI148514[] = {{&S36277[34], 7}, {NULL}};
const Segment SGI148515[] = {{&S36277[27], 6}, {NULL}};
const Segment SGI148516[] = {{&S36277[97], 9}, {NULL}};
const Segment SGI148517[] = {{&S36277[70], 6}, {NULL}};
const Segment SGI148518[] = {{&S46057[44], 7}, {NULL}};
const Segment SGI148519[] = {{&S46057[44], 19}, {NULL}};
const Segment SGI148520[] = {{&S46057[30], 6}, {NULL}};
const Segment SGI148521[] = {{&S46057[31], 3}, {NULL}};
const Segment SGI148522[] = {{&S46057[77], 6}, {NULL}};
const Segment SGI148523[] = {{&S46058[38], 7}, {NULL}};
const Segment SGI148524[] = {{&S46058[31], 6}, {NULL}};
const Segment SGI148525[] = {{&S46059[44], 7}, {NULL}};
const Segment SGI148526[] = {{&S46059[44], 19}, {NULL}};
const Segment SGI148527[] = {{&S46059[30], 6}, {NULL}};
const Segment SGI148528[] = {{&S46059[31], 3}, {NULL}};
const Segment SGI148529[] = {{&S46059[77], 6}, {NULL}};
const Segment SGI148530[] = {{&S46060[42], 7}, {NULL}};
const Segment SGI148531[] = {{&S46060[30], 6}, {NULL}};
const Segment SGI148532[] = {{&S46060[31], 3}, {NULL}};
const Segment SGI148533[] = {{&S46060[75], 6}, {NULL}};
const Segment SGI148534[] = {{&S46076[44], 9}, {NULL}};
const Segment SGI148535[] = {{&S46076[44], 21}, {NULL}};
const Segment SGI148536[] = {{&S46076[30], 6}, {NULL}};
const Segment SGI148537[] = {{&S46076[31], 3}, {NULL}};
const Segment SGI148538[] = {{&S46076[79], 6}, {NULL}};
const Segment SGI148539[] = {{&S46077[38], 9}, {NULL}};
const Segment SGI148540[] = {{&S46077[31], 6}, {NULL}};
const Segment SGI148541[] = {{&S46078[44], 24}, {NULL}};
const Segment SGI148542[] = {{&S46078[44], 36}, {NULL}};
const Segment SGI148543[] = {{&S46078[30], 6}, {NULL}};
const Segment SGI148544[] = {{&S46078[31], 3}, {NULL}};
const Segment SGI148545[] = {{&S46078[94], 6}, {NULL}};
const Segment SGI148546[] = {{&S46079[44], 10}, {NULL}};
const Segment SGI148547[] = {{&S46079[44], 22}, {NULL}};
const Segment SGI148548[] = {{&S46079[30], 6}, {NULL}};
const Segment SGI148549[] = {{&S46079[31], 3}, {NULL}};
const Segment SGI148550[] = {{&S46079[80], 6}, {NULL}};
const Segment SGI148551[] = {{&S46080[44], 16}, {NULL}};
const Segment SGI148552[] = {{&S46080[44], 28}, {NULL}};
const Segment SGI148553[] = {{&S46080[30], 6}, {NULL}};
const Segment SGI148554[] = {{&S46080[31], 3}, {NULL}};
const Segment SGI148555[] = {{&S46080[86], 6}, {NULL}};
const Segment SGI148556[] = {{&S46081[44], 17}, {NULL}};
const Segment SGI148557[] = {{&S46081[44], 29}, {NULL}};
const Segment SGI148558[] = {{&S46081[30], 6}, {NULL}};
const Segment SGI148559[] = {{&S46081[31], 3}, {NULL}};
const Segment SGI148560[] = {{&S46081[87], 6}, {NULL}};
const Segment SGI148561[] = {{&S46082[44], 10}, {NULL}};
const Segment SGI148562[] = {{&S46082[44], 22}, {NULL}};
const Segment SGI148563[] = {{&S46082[30], 6}, {NULL}};
const Segment SGI148564[] = {{&S46082[31], 3}, {NULL}};
const Segment SGI148565[] = {{&S46082[80], 6}, {NULL}};
const Segment SGI148566[] = {{&S46083[44], 24}, {NULL}};
const Segment SGI148567[] = {{&S46083[44], 36}, {NULL}};
const Segment SGI148568[] = {{&S46083[30], 6}, {NULL}};
const Segment SGI148569[] = {{&S46083[31], 3}, {NULL}};
const Segment SGI148570[] = {{&S46083[94], 6}, {NULL}};
const Segment SGI148571[] = {{&S46084[44], 17}, {NULL}};
const Segment SGI148572[] = {{&S46084[44], 29}, {NULL}};
const Segment SGI148573[] = {{&S46084[30], 6}, {NULL}};
const Segment SGI148574[] = {{&S46084[31], 3}, {NULL}};
const Segment SGI148575[] = {{&S46084[87], 6}, {NULL}};
const Segment SGI148576[] = {{&S36349[44], 8}, {NULL}};
const Segment SGI148577[] = {{&S36349[44], 30}, {NULL}};
const Segment SGI148578[] = {{&S36349[30], 6}, {NULL}};
const Segment SGI148579[] = {{&S36349[123], 3}, {NULL}};
const Segment SGI148580[] = {{&S36349[88], 6}, {NULL}};
const Segment SGI148581[] = {{&S36350[38], 8}, {NULL}};
const Segment SGI148582[] = {{&S36350[31], 6}, {NULL}};
const Segment SGI148583[] = {{&S36351[44], 8}, {NULL}};
const Segment SGI148584[] = {{&S36351[44], 30}, {NULL}};
const Segment SGI148585[] = {{&S36351[30], 6}, {NULL}};
const Segment SGI148586[] = {{&S36351[123], 3}, {NULL}};
const Segment SGI148587[] = {{&S36351[88], 6}, {NULL}};
const Segment SGI148588[] = {{&S36352[44], 8}, {NULL}};
const Segment SGI148589[] = {{&S36352[44], 32}, {NULL}};
const Segment SGI148590[] = {{&S36352[30], 6}, {NULL}};
const Segment SGI148591[] = {{&S36352[125], 3}, {NULL}};
const Segment SGI148592[] = {{&S36352[90], 6}, {NULL}};
const Segment SGI148593[] = {{&S36353[42], 8}, {NULL}};
const Segment SGI148594[] = {{&S36353[30], 6}, {NULL}};
const Segment SGI148595[] = {{&S36353[121], 3}, {NULL}};
const Segment SGI148596[] = {{&S36353[86], 6}, {NULL}};
const Segment SGI148597[] = {{&S46111[34], 10}, {NULL}};
const Segment SGI148598[] = {{&S46111[27], 6}, {NULL}};
const Segment SGI148599[] = {{&S46111[97], 9}, {NULL}};
const Segment SGI148600[] = {{&S46111[70], 6}, {NULL}};
const Segment SGI148601[] = {{&S46113[44], 18}, {NULL}};
const Segment SGI148602[] = {{&S46113[44], 39}, {NULL}};
const Segment SGI148603[] = {{&S46113[30], 6}, {NULL}};
const Segment SGI148604[] = {{&S46113[31], 3}, {NULL}};
const Segment SGI148605[] = {{&S46113[97], 6}, {NULL}};
const Segment SGI148606[] = {{&S46114[44], 15}, {NULL}};
const Segment SGI148607[] = {{&S46114[44], 33}, {NULL}};
const Segment SGI148608[] = {{&S46114[30], 6}, {NULL}};
const Segment SGI148609[] = {{&S46114[31], 3}, {NULL}};
const Segment SGI148610[] = {{&S46114[91], 6}, {NULL}};
const Segment SGI148611[] = {{&S46115[44], 17}, {NULL}};
const Segment SGI148612[] = {{&S46115[44], 37}, {NULL}};
const Segment SGI148613[] = {{&S46115[30], 6}, {NULL}};
const Segment SGI148614[] = {{&S46115[31], 3}, {NULL}};
const Segment SGI148615[] = {{&S46115[95], 6}, {NULL}};
const Segment SGI148616[] = {{&S46116[44], 15}, {NULL}};
const Segment SGI148617[] = {{&S46116[44], 33}, {NULL}};
const Segment SGI148618[] = {{&S46116[30], 6}, {NULL}};
const Segment SGI148619[] = {{&S46116[94], 6}, {NULL}};
const Segment SGI148620[] = {{&S46117[44], 16}, {NULL}};
const Segment SGI148621[] = {{&S46117[44], 35}, {NULL}};
const Segment SGI148622[] = {{&S46117[30], 6}, {NULL}};
const Segment SGI148623[] = {{&S46117[31], 3}, {NULL}};
const Segment SGI148624[] = {{&S46117[93], 6}, {NULL}};
const Segment SGI148625[] = {{&S46118[44], 17}, {NULL}};
const Segment SGI148626[] = {{&S46118[44], 37}, {NULL}};
const Segment SGI148627[] = {{&S46118[30], 6}, {NULL}};
const Segment SGI148628[] = {{&S46118[31], 3}, {NULL}};
const Segment SGI148629[] = {{&S46118[95], 6}, {NULL}};
const Segment SGI148630[] = {{&S46119[38], 15}, {NULL}};
const Segment SGI148631[] = {{&S46119[31], 6}, {NULL}};
const Segment SGI148632[] = {{&S46120[38], 16}, {NULL}};
const Segment SGI148633[] = {{&S46120[31], 6}, {NULL}};
const Segment SGI148634[] = {{&S46121[38], 18}, {NULL}};
const Segment SGI148635[] = {{&S46121[31], 6}, {NULL}};
const Segment SGI148636[] = {{&S46123[34], 10}, {NULL}};
const Segment SGI148637[] = {{&S46123[27], 6}, {NULL}};
const Segment SGI148638[] = {{&S46123[97], 9}, {NULL}};
const Segment SGI148639[] = {{&S46123[70], 6}, {NULL}};
const Segment SGI148640[] = {{&S46124[34], 11}, {NULL}};
const Segment SGI148641[] = {{&S46124[27], 6}, {NULL}};
const Segment SGI148642[] = {{&S46124[98], 9}, {NULL}};
const Segment SGI148643[] = {{&S46124[71], 6}, {NULL}};
const Segment SGI148644[] = {{&S46125[34], 11}, {NULL}};
const Segment SGI148645[] = {{&S46125[27], 6}, {NULL}};
const Segment SGI148646[] = {{&S46125[98], 9}, {NULL}};
const Segment SGI148647[] = {{&S46125[71], 6}, {NULL}};
const Segment SGI148648[] = {{&S46126[34], 11}, {NULL}};
const Segment SGI148649[] = {{&S46126[27], 6}, {NULL}};
const Segment SGI148650[] = {{&S46126[98], 9}, {NULL}};
const Segment SGI148651[] = {{&S46126[71], 6}, {NULL}};
const Segment SGI148652[] = {{&S46128[34], 11}, {NULL}};
const Segment SGI148653[] = {{&S46128[27], 6}, {NULL}};
const Segment SGI148654[] = {{&S46128[98], 9}, {NULL}};
const Segment SGI148655[] = {{&S46128[71], 6}, {NULL}};
const Segment SGI148656[] = {{&S46129[34], 10}, {NULL}};
const Segment SGI148657[] = {{&S46129[27], 6}, {NULL}};
const Segment SGI148658[] = {{&S46129[97], 9}, {NULL}};
const Segment SGI148659[] = {{&S46129[70], 6}, {NULL}};
const Segment SGI148660[] = {{&S46130[34], 21}, {NULL}};
const Segment SGI148661[] = {{&S46130[27], 6}, {NULL}};
const Segment SGI148662[] = {{&S46130[108], 9}, {NULL}};
const Segment SGI148663[] = {{&S46130[81], 6}, {NULL}};
const Segment SGI148664[] = {{&S46131[34], 20}, {NULL}};
const Segment SGI148665[] = {{&S46131[27], 6}, {NULL}};
const Segment SGI148666[] = {{&S46131[107], 9}, {NULL}};
const Segment SGI148667[] = {{&S46131[80], 6}, {NULL}};
const Segment SGI148668[] = {{&S46141[44], 7}, {NULL}};
const Segment SGI148669[] = {{&S46141[44], 19}, {NULL}};
const Segment SGI148670[] = {{&S46141[30], 6}, {NULL}};
const Segment SGI148671[] = {{&S46141[31], 3}, {NULL}};
const Segment SGI148672[] = {{&S46141[77], 6}, {NULL}};
const Segment SGI148673[] = {{&S46143[44], 10}, {NULL}};
const Segment SGI148674[] = {{&S46143[44], 22}, {NULL}};
const Segment SGI148675[] = {{&S46143[30], 6}, {NULL}};
const Segment SGI148676[] = {{&S46143[31], 3}, {NULL}};
const Segment SGI148677[] = {{&S46143[80], 6}, {NULL}};
const Segment SGI148678[] = {{&S46146[44], 10}, {NULL}};
const Segment SGI148679[] = {{&S46146[44], 22}, {NULL}};
const Segment SGI148680[] = {{&S46146[30], 6}, {NULL}};
const Segment SGI148681[] = {{&S46146[83], 6}, {NULL}};
const Segment SGI148682[] = {{&S46155[38], 10}, {NULL}};
const Segment SGI148683[] = {{&S46155[31], 6}, {NULL}};
const Segment SGI148684[] = {{&S46156[44], 7}, {NULL}};
const Segment SGI148685[] = {{&S46156[44], 19}, {NULL}};
const Segment SGI148686[] = {{&S46156[30], 6}, {NULL}};
const Segment SGI148687[] = {{&S46156[31], 3}, {NULL}};
const Segment SGI148688[] = {{&S46156[77], 6}, {NULL}};
const Segment SGI148689[] = {{&S46159[44], 7}, {NULL}};
const Segment SGI148690[] = {{&S46159[44], 19}, {NULL}};
const Segment SGI148691[] = {{&S46159[30], 6}, {NULL}};
const Segment SGI148692[] = {{&S46159[31], 3}, {NULL}};
const Segment SGI148693[] = {{&S46159[77], 5}, {NULL}};
const Segment SGI148694[] = {{&S46161[38], 7}, {NULL}};
const Segment SGI148695[] = {{&S46161[31], 6}, {NULL}};
const Segment SGI148696[] = {{&S46167[44], 13}, {NULL}};
const Segment SGI148697[] = {{&S46167[44], 25}, {NULL}};
const Segment SGI148698[] = {{&S46167[30], 6}, {NULL}};
const Segment SGI148699[] = {{&S46167[31], 3}, {NULL}};
const Segment SGI148700[] = {{&S46167[83], 6}, {NULL}};
const Segment SGI148701[] = {{&S46168[44], 10}, {NULL}};
const Segment SGI148702[] = {{&S46168[44], 22}, {NULL}};
const Segment SGI148703[] = {{&S46168[30], 6}, {NULL}};
const Segment SGI148704[] = {{&S46168[31], 3}, {NULL}};
const Segment SGI148705[] = {{&S46168[80], 6}, {NULL}};
const Segment SGI148706[] = {{&S46169[38], 10}, {NULL}};
const Segment SGI148707[] = {{&S46169[31], 6}, {NULL}};
const Segment SGI148708[] = {{&S46170[44], 10}, {NULL}};
const Segment SGI148709[] = {{&S46170[44], 22}, {NULL}};
const Segment SGI148710[] = {{&S46170[30], 6}, {NULL}};
const Segment SGI148711[] = {{&S46170[31], 3}, {NULL}};
const Segment SGI148712[] = {{&S46170[80], 6}, {NULL}};
const Segment SGI148713[] = {{&S34471[42], 9}, {NULL}};
const Segment SGI148714[] = {{&S34471[42], 20}, {NULL}};
const Segment SGI148715[] = {{&S34471[30], 4}, {NULL}};
const Segment SGI148716[] = {{&S34471[111], 3}, {NULL}};
const Segment SGI148717[] = {{&S34471[76], 6}, {NULL}};
const Segment SGI148718[] = {{&S34472[42], 9}, {NULL}};
const Segment SGI148719[] = {{&S34472[42], 21}, {NULL}};
const Segment SGI148720[] = {{&S34472[30], 4}, {NULL}};
const Segment SGI148721[] = {{&S34472[112], 3}, {NULL}};
const Segment SGI148722[] = {{&S34472[77], 6}, {NULL}};
const Segment SGI148723[] = {{&S34473[42], 9}, {NULL}};
const Segment SGI148724[] = {{&S34473[42], 21}, {NULL}};
const Segment SGI148725[] = {{&S34473[30], 4}, {NULL}};
const Segment SGI148726[] = {{&S34473[112], 3}, {NULL}};
const Segment SGI148727[] = {{&S34473[77], 6}, {NULL}};
const Segment SGI148728[] = {{&S34474[42], 9}, {NULL}};
const Segment SGI148729[] = {{&S34474[42], 21}, {NULL}};
const Segment SGI148730[] = {{&S34474[30], 4}, {NULL}};
const Segment SGI148731[] = {{&S34474[112], 3}, {NULL}};
const Segment SGI148732[] = {{&S34474[77], 6}, {NULL}};
const Segment SGI148733[] = {{&S34475[42], 9}, {NULL}};
const Segment SGI148734[] = {{&S34475[42], 20}, {NULL}};
const Segment SGI148735[] = {{&S34475[30], 4}, {NULL}};
const Segment SGI148736[] = {{&S34475[111], 3}, {NULL}};
const Segment SGI148737[] = {{&S34475[76], 6}, {NULL}};
const Segment SGI148738[] = {{&S34476[42], 9}, {NULL}};
const Segment SGI148739[] = {{&S34476[42], 21}, {NULL}};
const Segment SGI148740[] = {{&S34476[30], 4}, {NULL}};
const Segment SGI148741[] = {{&S34476[112], 3}, {NULL}};
const Segment SGI148742[] = {{&S34476[77], 6}, {NULL}};
const Segment SGI148743[] = {{&S34477[42], 9}, {NULL}};
const Segment SGI148744[] = {{&S34477[42], 21}, {NULL}};
const Segment SGI148745[] = {{&S34477[30], 4}, {NULL}};
const Segment SGI148746[] = {{&S46192[34], 10}, {NULL}};
const Segment SGI148747[] = {{&S34477[112], 3}, {NULL}};
const Segment SGI148748[] = {{&S46192[27], 6}, {NULL}};
const Segment SGI148749[] = {{&S34477[77], 6}, {NULL}};
const Segment SGI148750[] = {{&S46192[97], 9}, {NULL}};
const Segment SGI148751[] = {{&S46192[70], 6}, {NULL}};
const Segment SGI148752[] = {{&S34478[45], 15}, {NULL}};
const Segment SGI148753[] = {{&S34478[45], 26}, {NULL}};
const Segment SGI148754[] = {{&S34478[33], 4}, {NULL}};
const Segment SGI148755[] = {{&S34479[42], 9}, {NULL}};
const Segment SGI148756[] = {{&S34479[42], 21}, {NULL}};
const Segment SGI148757[] = {{&S34479[30], 4}, {NULL}};
const Segment SGI148758[] = {{&S34479[112], 3}, {NULL}};
const Segment SGI148759[] = {{&S34479[77], 6}, {NULL}};
const Segment SGI148760[] = {{&S34480[36], 9}, {NULL}};
const Segment SGI148761[] = {{&S34480[31], 4}, {NULL}};
const Segment SGI148762[] = {{&S46198[44], 12}, {NULL}};
const Segment SGI148763[] = {{&S46198[44], 24}, {NULL}};
const Segment SGI148764[] = {{&S46198[30], 6}, {NULL}};
const Segment SGI148765[] = {{&S46198[31], 3}, {NULL}};
const Segment SGI148766[] = {{&S46198[82], 6}, {NULL}};
const Segment SGI148767[] = {{&S36479[34], 9}, {NULL}};
const Segment SGI148768[] = {{&S36479[27], 6}, {NULL}};
const Segment SGI148769[] = {{&S36479[96], 9}, {NULL}};
const Segment SGI148770[] = {{&S36479[69], 6}, {NULL}};
const Segment SGI148771[] = {{&S36480[34], 8}, {NULL}};
const Segment SGI148772[] = {{&S36480[27], 6}, {NULL}};
const Segment SGI148773[] = {{&S36480[95], 9}, {NULL}};
const Segment SGI148774[] = {{&S36480[68], 6}, {NULL}};
const Segment SGI148775[] = {{&S36481[34], 11}, {NULL}};
const Segment SGI148776[] = {{&S36481[27], 6}, {NULL}};
const Segment SGI148777[] = {{&S36481[98], 9}, {NULL}};
const Segment SGI148778[] = {{&S40758[32], 14}, {NULL}};
const Segment SGI148779[] = {{&S36481[71], 6}, {NULL}};
const Segment SGI148780[] = {{&S40758[27], 4}, {NULL}};
const Segment SGI148781[] = {{&S40758[98], 9}, {NULL}};
const Segment SGI148782[] = {{&S40758[72], 5}, {NULL}};
const Segment SGI148783[] = {{&S40758[131], 7}, {NULL}};
const Segment SGI148784[] = {{&S36482[34], 14}, {NULL}};
const Segment SGI148785[] = {{&S36482[27], 6}, {NULL}};
const Segment SGI148786[] = {{&S36482[101], 9}, {NULL}};
const Segment SGI148787[] = {{&S36482[74], 6}, {NULL}};
const Segment SGI148788[] = {{&S36483[34], 18}, {NULL}};
const Segment SGI148789[] = {{&S36483[27], 6}, {NULL}};
const Segment SGI148790[] = {{&S36483[105], 9}, {NULL}};
const Segment SGI148791[] = {{&S36483[78], 6}, {NULL}};
const Segment SGI148792[] = {{&S40763[32], 14}, {NULL}};
const Segment SGI148793[] = {{&S40763[27], 4}, {NULL}};
const Segment SGI148794[] = {{&S40763[99], 9}, {NULL}};
const Segment SGI148795[] = {{&S40763[72], 6}, {NULL}};
const Segment SGI148796[] = {{&S46239[34], 10}, {NULL}};
const Segment SGI148797[] = {{&S46239[27], 6}, {NULL}};
const Segment SGI148798[] = {{&S46239[97], 9}, {NULL}};
const Segment SGI148799[] = {{&S46239[70], 6}, {NULL}};
const Segment SGI148800[] = {{&S46240[34], 13}, {NULL}};
const Segment SGI148801[] = {{&S46240[27], 6}, {NULL}};
const Segment SGI148802[] = {{&S46240[100], 9}, {NULL}};
const Segment SGI148803[] = {{&S46240[73], 6}, {NULL}};
const Segment SGI148804[] = {{&S46241[34], 14}, {NULL}};
const Segment SGI148805[] = {{&S46241[27], 6}, {NULL}};
const Segment SGI148806[] = {{&S46241[101], 9}, {NULL}};
const Segment SGI148807[] = {{&S46241[74], 6}, {NULL}};
const Segment SGI148808[] = {{&S46252[34], 11}, {NULL}};
const Segment SGI148809[] = {{&S46252[27], 6}, {NULL}};
const Segment SGI148810[] = {{&S46252[98], 9}, {NULL}};
const Segment SGI148811[] = {{&S46252[71], 6}, {NULL}};
const Segment SGI148812[] = {{&S46253[34], 18}, {NULL}};
const Segment SGI148813[] = {{&S46253[27], 6}, {NULL}};
const Segment SGI148814[] = {{&S46253[105], 9}, {NULL}};
const Segment SGI148815[] = {{&S46253[78], 6}, {NULL}};
const Segment SGI148816[] = {{&S46254[34], 11}, {NULL}};
const Segment SGI148817[] = {{&S46254[27], 6}, {NULL}};
const Segment SGI148818[] = {{&S46254[98], 9}, {NULL}};
const Segment SGI148819[] = {{&S46254[71], 6}, {NULL}};
const Segment SGI148820[] = {{&S46255[34], 10}, {NULL}};
const Segment SGI148821[] = {{&S46255[27], 6}, {NULL}};
const Segment SGI148822[] = {{&S46255[97], 9}, {NULL}};
const Segment SGI148823[] = {{&S46255[70], 6}, {NULL}};
const Segment SGI148824[] = {{&S46256[34], 11}, {NULL}};
const Segment SGI148825[] = {{&S46256[27], 6}, {NULL}};
const Segment SGI148826[] = {{&S46256[98], 9}, {NULL}};
const Segment SGI148827[] = {{&S46256[71], 6}, {NULL}};
const Segment SGI148828[] = {{&S46257[34], 11}, {NULL}};
const Segment SGI148829[] = {{&S46257[27], 6}, {NULL}};
const Segment SGI148830[] = {{&S46257[98], 9}, {NULL}};
const Segment SGI148831[] = {{&S46257[71], 6}, {NULL}};
const Segment SGI148832[] = {{&S46258[34], 11}, {NULL}};
const Segment SGI148833[] = {{&S46258[27], 6}, {NULL}};
const Segment SGI148834[] = {{&S46258[98], 9}, {NULL}};
const Segment SGI148835[] = {{&S46258[71], 6}, {NULL}};
const Segment SGI148836[] = {{&S46259[34], 11}, {NULL}};
const Segment SGI148837[] = {{&S46259[27], 6}, {NULL}};
const Segment SGI148838[] = {{&S46259[98], 9}, {NULL}};
const Segment SGI148839[] = {{&S46259[71], 6}, {NULL}};
const Segment SGI148840[] = {{&S46260[34], 11}, {NULL}};
const Segment SGI148841[] = {{&S46260[27], 6}, {NULL}};
const Segment SGI148842[] = {{&S46260[98], 9}, {NULL}};
const Segment SGI148843[] = {{&S46260[71], 6}, {NULL}};
const Segment SGI148844[] = {{&S46261[34], 14}, {NULL}};
const Segment SGI148845[] = {{&S46261[27], 6}, {NULL}};
const Segment SGI148846[] = {{&S46261[101], 9}, {NULL}};
const Segment SGI148847[] = {{&S46261[74], 6}, {NULL}};
const Segment SGI148848[] = {{&S46262[34], 18}, {NULL}};
const Segment SGI148849[] = {{&S46262[27], 6}, {NULL}};
const Segment SGI148850[] = {{&S46262[105], 9}, {NULL}};
const Segment SGI148851[] = {{&S46262[78], 6}, {NULL}};
const Segment SGI148852[] = {{&S46319[44], 11}, {NULL}};
const Segment SGI148853[] = {{&S46319[44], 23}, {NULL}};
const Segment SGI148854[] = {{&S46319[30], 6}, {NULL}};
const Segment SGI148855[] = {{&S46319[31], 3}, {NULL}};
const Segment SGI148856[] = {{&S46319[81], 6}, {NULL}};
const Segment SGI148857[] = {{&S46321[44], 15}, {NULL}};
const Segment SGI148858[] = {{&S46321[44], 27}, {NULL}};
const Segment SGI148859[] = {{&S46321[30], 6}, {NULL}};
const Segment SGI148860[] = {{&S46321[31], 3}, {NULL}};
const Segment SGI148861[] = {{&S46321[85], 6}, {NULL}};
const Segment SGI148862[] = {{&S46322[44], 15}, {NULL}};
const Segment SGI148863[] = {{&S46322[44], 27}, {NULL}};
const Segment SGI148864[] = {{&S46322[30], 6}, {NULL}};
const Segment SGI148865[] = {{&S46322[31], 3}, {NULL}};
const Segment SGI148866[] = {{&S46322[85], 6}, {NULL}};
const Segment SGI148867[] = {{&S46323[38], 11}, {NULL}};
const Segment SGI148868[] = {{&S46323[31], 6}, {NULL}};
const Segment SGI148869[] = {{&S46325[44], 20}, {NULL}};
const Segment SGI148870[] = {{&S46325[44], 32}, {NULL}};
const Segment SGI148871[] = {{&S46325[30], 6}, {NULL}};
const Segment SGI148872[] = {{&S46325[31], 3}, {NULL}};
const Segment SGI148873[] = {{&S46325[90], 6}, {NULL}};
const Segment SGI148874[] = {{&S46326[38], 15}, {NULL}};
const Segment SGI148875[] = {{&S46326[31], 6}, {NULL}};
const Segment SGI148876[] = {{&S46327[44], 15}, {NULL}};
const Segment SGI148877[] = {{&S46327[44], 27}, {NULL}};
const Segment SGI148878[] = {{&S46327[30], 6}, {NULL}};
const Segment SGI148879[] = {{&S46327[31], 3}, {NULL}};
const Segment SGI148880[] = {{&S46327[85], 6}, {NULL}};
const Segment SGI148881[] = {{&S34663[32], 14}, {NULL}};
const Segment SGI148882[] = {{&S34663[27], 4}, {NULL}};
const Segment SGI148883[] = {{&S34663[99], 9}, {NULL}};
const Segment SGI148884[] = {{&S34663[72], 6}, {NULL}};
const Segment SGI148885[] = {{&S46329[44], 14}, {NULL}};
const Segment SGI148886[] = {{&S46329[44], 28}, {NULL}};
const Segment SGI148887[] = {{&S46329[30], 6}, {NULL}};
const Segment SGI148888[] = {{&S46329[31], 3}, {NULL}};
const Segment SGI148889[] = {{&S46329[86], 6}, {NULL}};
const Segment SGI148890[] = {{&S46331[44], 14}, {NULL}};
const Segment SGI148891[] = {{&S46331[44], 26}, {NULL}};
const Segment SGI148892[] = {{&S46331[30], 6}, {NULL}};
const Segment SGI148893[] = {{&S46331[31], 3}, {NULL}};
const Segment SGI148894[] = {{&S46331[84], 6}, {NULL}};
const Segment SGI148895[] = {{&S46332[44], 15}, {NULL}};
const Segment SGI148896[] = {{&S46332[44], 29}, {NULL}};
const Segment SGI148897[] = {{&S46332[30], 6}, {NULL}};
const Segment SGI148898[] = {{&S46332[31], 3}, {NULL}};
const Segment SGI148899[] = {{&S46332[87], 6}, {NULL}};
const Segment SGI148900[] = {{&S46333[44], 15}, {NULL}};
const Segment SGI148901[] = {{&S46333[44], 27}, {NULL}};
const Segment SGI148902[] = {{&S46333[30], 6}, {NULL}};
const Segment SGI148903[] = {{&S46333[31], 3}, {NULL}};
const Segment SGI148904[] = {{&S46333[85], 6}, {NULL}};
const Segment SGI148905[] = {{&S46335[44], 15}, {NULL}};
const Segment SGI148906[] = {{&S46335[44], 25}, {NULL}};
const Segment SGI148907[] = {{&S46335[30], 6}, {NULL}};
const Segment SGI148908[] = {{&S46335[31], 3}, {NULL}};
const Segment SGI148909[] = {{&S46335[83], 6}, {NULL}};
const Segment SGI148910[] = {{&S46336[44], 15}, {NULL}};
const Segment SGI148911[] = {{&S46336[44], 27}, {NULL}};
const Segment SGI148912[] = {{&S46336[30], 6}, {NULL}};
const Segment SGI148913[] = {{&S46336[31], 3}, {NULL}};
const Segment SGI148914[] = {{&S46336[85], 6}, {NULL}};
const Segment SGI148915[] = {{&S46337[44], 20}, {NULL}};
const Segment SGI148916[] = {{&S46337[44], 30}, {NULL}};
const Segment SGI148917[] = {{&S46337[30], 6}, {NULL}};
const Segment SGI148918[] = {{&S46337[31], 3}, {NULL}};
const Segment SGI148919[] = {{&S46337[88], 6}, {NULL}};
const Segment SGI148920[] = {{&S46338[44], 15}, {NULL}};
const Segment SGI148921[] = {{&S46338[44], 25}, {NULL}};
const Segment SGI148922[] = {{&S46338[30], 6}, {NULL}};
const Segment SGI148923[] = {{&S46338[31], 3}, {NULL}};
const Segment SGI148924[] = {{&S46338[83], 6}, {NULL}};
const Segment SGI148925[] = {{&S46340[34], 10}, {NULL}};
const Segment SGI148926[] = {{&S46340[27], 6}, {NULL}};
const Segment SGI148927[] = {{&S46340[97], 9}, {NULL}};
const Segment SGI148928[] = {{&S46340[70], 6}, {NULL}};
const Segment SGI148929[] = {{&S46344[44], 12}, {NULL}};
const Segment SGI148930[] = {{&S46344[44], 24}, {NULL}};
const Segment SGI148931[] = {{&S46344[30], 6}, {NULL}};
const Segment SGI148932[] = {{&S46344[31], 3}, {NULL}};
const Segment SGI148933[] = {{&S46344[82], 6}, {NULL}};
const Segment SGI148934[] = {{&S46345[44], 10}, {NULL}};
const Segment SGI148935[] = {{&S46345[44], 22}, {NULL}};
const Segment SGI148936[] = {{&S46345[30], 6}, {NULL}};
const Segment SGI148937[] = {{&S46345[31], 3}, {NULL}};
const Segment SGI148938[] = {{&S46345[80], 6}, {NULL}};
const Segment SGI148939[] = {{&S46346[38], 10}, {NULL}};
const Segment SGI148940[] = {{&S46346[31], 6}, {NULL}};
const Segment SGI148941[] = {{&S46354[34], 7}, {NULL}};
const Segment SGI148942[] = {{&S46354[27], 6}, {NULL}};
const Segment SGI148943[] = {{&S46354[111], 9}, {NULL}};
const Segment SGI148944[] = {{&S46354[84], 6}, {NULL}};
const Segment SGI148945[] = {{&S46363[44], 10}, {NULL}};
const Segment SGI148946[] = {{&S46363[44], 22}, {NULL}};
const Segment SGI148947[] = {{&S46363[30], 6}, {NULL}};
const Segment SGI148948[] = {{&S46363[31], 3}, {NULL}};
const Segment SGI148949[] = {{&S46363[80], 6}, {NULL}};
const Segment SGI148950[] = {{&S46364[38], 10}, {NULL}};
const Segment SGI148951[] = {{&S46364[31], 6}, {NULL}};
const Segment SGI148952[] = {{&S46365[44], 6}, {NULL}};
const Segment SGI148953[] = {{&S46365[44], 18}, {NULL}};
const Segment SGI148954[] = {{&S46365[30], 6}, {NULL}};
const Segment SGI148955[] = {{&S46365[31], 3}, {NULL}};
const Segment SGI148956[] = {{&S46365[76], 6}, {NULL}};
const Segment SGI148957[] = {{&S46371[44], 10}, {NULL}};
const Segment SGI148958[] = {{&S46371[44], 22}, {NULL}};
const Segment SGI148959[] = {{&S46371[30], 6}, {NULL}};
const Segment SGI148960[] = {{&S46371[31], 3}, {NULL}};
const Segment SGI148961[] = {{&S46371[80], 6}, {NULL}};
const Segment SGI148962[] = {{&S46372[44], 9}, {NULL}};
const Segment SGI148963[] = {{&S46372[44], 21}, {NULL}};
const Segment SGI148964[] = {{&S46372[30], 6}, {NULL}};
const Segment SGI148965[] = {{&S46372[31], 3}, {NULL}};
const Segment SGI148966[] = {{&S46372[79], 6}, {NULL}};
const Segment SGI148967[] = {{&S46373[38], 9}, {NULL}};
const Segment SGI148968[] = {{&S46373[31], 6}, {NULL}};
const Segment SGI148969[] = {{&S46374[38], 10}, {NULL}};
const Segment SGI148970[] = {{&S46374[31], 6}, {NULL}};
const Segment SGI148971[] = {{&S46379[34], 10}, {NULL}};
const Segment SGI148972[] = {{&S46379[27], 6}, {NULL}};
const Segment SGI148973[] = {{&S46379[97], 9}, {NULL}};
const Segment SGI148974[] = {{&S46379[70], 6}, {NULL}};
const Segment SGI148975[] = {{&S46380[34], 10}, {NULL}};
const Segment SGI148976[] = {{&S46380[27], 6}, {NULL}};
const Segment SGI148977[] = {{&S46380[97], 9}, {NULL}};
const Segment SGI148978[] = {{&S46380[70], 6}, {NULL}};
const Segment SGI148979[] = {{&S46394[44], 17}, {NULL}};
const Segment SGI148980[] = {{&S46394[44], 29}, {NULL}};
const Segment SGI148981[] = {{&S46394[30], 6}, {NULL}};
const Segment SGI148982[] = {{&S46394[31], 3}, {NULL}};
const Segment SGI148983[] = {{&S46394[87], 5}, {NULL}};
const Segment SGI148984[] = {{&S46395[38], 17}, {NULL}};
const Segment SGI148985[] = {{&S46395[31], 6}, {NULL}};
const Segment SGI148986[] = {{&S36784[44], 8}, {NULL}};
const Segment SGI148987[] = {{&S36784[44], 21}, {NULL}};
const Segment SGI148988[] = {{&S36784[30], 6}, {NULL}};
const Segment SGI148989[] = {{&S36784[114], 3}, {NULL}};
const Segment SGI148990[] = {{&S36784[79], 6}, {NULL}};
const Segment SGI148991[] = {{&S36785[44], 8}, {NULL}};
const Segment SGI148992[] = {{&S36785[44], 21}, {NULL}};
const Segment SGI148993[] = {{&S36785[30], 6}, {NULL}};
const Segment SGI148994[] = {{&S36785[114], 3}, {NULL}};
const Segment SGI148995[] = {{&S36785[79], 6}, {NULL}};
const Segment SGI148996[] = {{&S36786[38], 8}, {NULL}};
const Segment SGI148997[] = {{&S36786[31], 6}, {NULL}};
const Segment SGI148998[] = {{&S36787[38], 8}, {NULL}};
const Segment SGI148999[] = {{&S36787[31], 6}, {NULL}};
const Segment SGI149000[] = {{&S36788[44], 8}, {NULL}};
const Segment SGI149001[] = {{&S36788[44], 22}, {NULL}};
const Segment SGI149002[] = {{&S36788[30], 6}, {NULL}};
const Segment SGI149003[] = {{&S36788[115], 3}, {NULL}};
const Segment SGI149004[] = {{&S36788[80], 6}, {NULL}};
const Segment SGI149005[] = {{&S36789[44], 8}, {NULL}};
const Segment SGI149006[] = {{&S36789[44], 22}, {NULL}};
const Segment SGI149007[] = {{&S36789[30], 6}, {NULL}};
const Segment SGI149008[] = {{&S36789[115], 3}, {NULL}};
const Segment SGI149009[] = {{&S36789[80], 6}, {NULL}};
const Segment SGI149010[] = {{&S31973[34], 9}, {NULL}};
const Segment SGI149011[] = {{&S31973[27], 6}, {NULL}};
const Segment SGI149012[] = {{&S31973[96], 9}, {NULL}};
const Segment SGI149013[] = {{&S31973[69], 6}, {NULL}};
const Segment SGI149014[] = {{&S46515[44], 14}, {NULL}};
const Segment SGI149015[] = {{&S46515[44], 26}, {NULL}};
const Segment SGI149016[] = {{&S46515[30], 6}, {NULL}};
const Segment SGI149017[] = {{&S46515[31], 3}, {NULL}};
const Segment SGI149018[] = {{&S46515[84], 6}, {NULL}};
const Segment SGI149019[] = {{&S46516[38], 14}, {NULL}};
const Segment SGI149020[] = {{&S46516[31], 6}, {NULL}};
const Segment SGI149021[] = {{&S46565[44], 18}, {NULL}};
const Segment SGI149022[] = {{&S46565[44], 30}, {NULL}};
const Segment SGI149023[] = {{&S46565[30], 6}, {NULL}};
const Segment SGI149024[] = {{&S46565[31], 3}, {NULL}};
const Segment SGI149025[] = {{&S46565[88], 6}, {NULL}};
const Segment SGI149026[] = {{&S46566[44], 11}, {NULL}};
const Segment SGI149027[] = {{&S46566[44], 23}, {NULL}};
const Segment SGI149028[] = {{&S46566[30], 6}, {NULL}};
const Segment SGI149029[] = {{&S46566[31], 3}, {NULL}};
const Segment SGI149030[] = {{&S46566[81], 6}, {NULL}};
const Segment SGI149031[] = {{&S46567[44], 11}, {NULL}};
const Segment SGI149032[] = {{&S46567[44], 23}, {NULL}};
const Segment SGI149033[] = {{&S46567[30], 6}, {NULL}};
const Segment SGI149034[] = {{&S46567[31], 3}, {NULL}};
const Segment SGI149035[] = {{&S46567[81], 6}, {NULL}};
const Segment SGI149036[] = {{&S46568[44], 11}, {NULL}};
const Segment SGI149037[] = {{&S46568[44], 23}, {NULL}};
const Segment SGI149038[] = {{&S46568[30], 6}, {NULL}};
const Segment SGI149039[] = {{&S46568[31], 3}, {NULL}};
const Segment SGI149040[] = {{&S46568[81], 6}, {NULL}};
const Segment SGI149041[] = {{&S46569[44], 10}, {NULL}};
const Segment SGI149042[] = {{&S46569[44], 22}, {NULL}};
const Segment SGI149043[] = {{&S46569[30], 6}, {NULL}};
const Segment SGI149044[] = {{&S46569[31], 3}, {NULL}};
const Segment SGI149045[] = {{&S46569[80], 6}, {NULL}};
const Segment SGI149046[] = {{&S46570[38], 11}, {NULL}};
const Segment SGI149047[] = {{&S46570[31], 6}, {NULL}};
const Segment SGI149048[] = {{&S46571[38], 18}, {NULL}};
const Segment SGI149049[] = {{&S46571[31], 6}, {NULL}};
const Segment SGI149050[] = {{&S46595[44], 10}, {NULL}};
const Segment SGI149051[] = {{&S46595[44], 22}, {NULL}};
const Segment SGI149052[] = {{&S46595[30], 6}, {NULL}};
const Segment SGI149053[] = {{&S46595[31], 3}, {NULL}};
const Segment SGI149054[] = {{&S46595[80], 6}, {NULL}};
const Segment SGI149055[] = {{&S46598[44], 16}, {NULL}};
const Segment SGI149056[] = {{&S46598[44], 28}, {NULL}};
const Segment SGI149057[] = {{&S46598[30], 6}, {NULL}};
const Segment SGI149058[] = {{&S46598[31], 3}, {NULL}};
const Segment SGI149059[] = {{&S46598[86], 6}, {NULL}};
const Segment SGI149060[] = {{&S46601[44], 18}, {NULL}};
const Segment SGI149061[] = {{&S46601[44], 30}, {NULL}};
const Segment SGI149062[] = {{&S46601[30], 6}, {NULL}};
const Segment SGI149063[] = {{&S46601[31], 3}, {NULL}};
const Segment SGI149064[] = {{&S46601[88], 6}, {NULL}};
const Segment SGI149065[] = {{&S46602[44], 15}, {NULL}};
const Segment SGI149066[] = {{&S46602[44], 27}, {NULL}};
const Segment SGI149067[] = {{&S46602[30], 6}, {NULL}};
const Segment SGI149068[] = {{&S46602[31], 3}, {NULL}};
const Segment SGI149069[] = {{&S46602[85], 6}, {NULL}};
const Segment SGI149070[] = {{&S46603[44], 15}, {NULL}};
const Segment SGI149071[] = {{&S46603[44], 27}, {NULL}};
const Segment SGI149072[] = {{&S46603[30], 6}, {NULL}};
const Segment SGI149073[] = {{&S46603[31], 3}, {NULL}};
const Segment SGI149074[] = {{&S46603[85], 6}, {NULL}};
const Segment SGI149075[] = {{&S46604[44], 19}, {NULL}};
const Segment SGI149076[] = {{&S46604[44], 31}, {NULL}};
const Segment SGI149077[] = {{&S46604[30], 6}, {NULL}};
const Segment SGI149078[] = {{&S46604[31], 3}, {NULL}};
const Segment SGI149079[] = {{&S46604[89], 6}, {NULL}};
const Segment SGI149080[] = {{&S46605[44], 16}, {NULL}};
const Segment SGI149081[] = {{&S46605[44], 28}, {NULL}};
const Segment SGI149082[] = {{&S46605[30], 6}, {NULL}};
const Segment SGI149083[] = {{&S46605[31], 3}, {NULL}};
const Segment SGI149084[] = {{&S46605[86], 6}, {NULL}};
const Segment SGI149085[] = {{&S46606[44], 16}, {NULL}};
const Segment SGI149086[] = {{&S46606[44], 28}, {NULL}};
const Segment SGI149087[] = {{&S46606[30], 6}, {NULL}};
const Segment SGI149088[] = {{&S46606[31], 3}, {NULL}};
const Segment SGI149089[] = {{&S46606[86], 6}, {NULL}};
const Segment SGI149090[] = {{&S46608[44], 15}, {NULL}};
const Segment SGI149091[] = {{&S46608[44], 27}, {NULL}};
const Segment SGI149092[] = {{&S46608[30], 6}, {NULL}};
const Segment SGI149093[] = {{&S46608[31], 3}, {NULL}};
const Segment SGI149094[] = {{&S46608[85], 6}, {NULL}};
const Segment SGI149095[] = {{&S46609[44], 12}, {NULL}};
const Segment SGI149096[] = {{&S46609[44], 24}, {NULL}};
const Segment SGI149097[] = {{&S46609[30], 6}, {NULL}};
const Segment SGI149098[] = {{&S46609[31], 3}, {NULL}};
const Segment SGI149099[] = {{&S46609[82], 6}, {NULL}};
const Segment SGI149100[] = {{&S46610[44], 21}, {NULL}};
const Segment SGI149101[] = {{&S46610[44], 33}, {NULL}};
const Segment SGI149102[] = {{&S46610[30], 6}, {NULL}};
const Segment SGI149103[] = {{&S46610[31], 3}, {NULL}};
const Segment SGI149104[] = {{&S46610[91], 6}, {NULL}};
const Segment SGI149105[] = {{&S41235[42], 5}, {NULL}};
const Segment SGI149106[] = {{&S41235[42], 17}, {NULL}};
const Segment SGI149107[] = {{&S41235[30], 4}, {NULL}};
const Segment SGI149108[] = {{&S41235[108], 3}, {NULL}};
const Segment SGI149109[] = {{&S41235[73], 6}, {NULL}};
const Segment SGI149110[] = {{&S46611[44], 16}, {NULL}};
const Segment SGI149111[] = {{&S46611[44], 28}, {NULL}};
const Segment SGI149112[] = {{&S46611[30], 6}, {NULL}};
const Segment SGI149113[] = {{&S46611[31], 3}, {NULL}};
const Segment SGI149114[] = {{&S46611[86], 6}, {NULL}};
const Segment SGI149115[] = {{&S46612[44], 17}, {NULL}};
const Segment SGI149116[] = {{&S46612[44], 29}, {NULL}};
const Segment SGI149117[] = {{&S46612[30], 6}, {NULL}};
const Segment SGI149118[] = {{&S46612[31], 3}, {NULL}};
const Segment SGI149119[] = {{&S46612[87], 6}, {NULL}};
const Segment SGI149120[] = {{&S46613[44], 19}, {NULL}};
const Segment SGI149121[] = {{&S46613[44], 31}, {NULL}};
const Segment SGI149122[] = {{&S46613[30], 6}, {NULL}};
const Segment SGI149123[] = {{&S46613[31], 3}, {NULL}};
const Segment SGI149124[] = {{&S46613[89], 6}, {NULL}};
const Segment SGI149125[] = {{&S46614[44], 20}, {NULL}};
const Segment SGI149126[] = {{&S46614[44], 32}, {NULL}};
const Segment SGI149127[] = {{&S46614[30], 6}, {NULL}};
const Segment SGI149128[] = {{&S46614[31], 3}, {NULL}};
const Segment SGI149129[] = {{&S46614[90], 6}, {NULL}};
const Segment SGI149130[] = {{&S46615[38], 19}, {NULL}};
const Segment SGI149131[] = {{&S46615[31], 6}, {NULL}};
const Segment SGI149132[] = {{&S46616[44], 18}, {NULL}};
const Segment SGI149133[] = {{&S46616[44], 30}, {NULL}};
const Segment SGI149134[] = {{&S46616[30], 6}, {NULL}};
const Segment SGI149135[] = {{&S46616[31], 3}, {NULL}};
const Segment SGI149136[] = {{&S46616[88], 6}, {NULL}};
const Segment SGI149137[] = {{&S46617[38], 16}, {NULL}};
const Segment SGI149138[] = {{&S46617[31], 6}, {NULL}};
const Segment SGI149139[] = {{&S46618[38], 18}, {NULL}};
const Segment SGI149140[] = {{&S46618[31], 6}, {NULL}};
const Segment SGI149141[] = {{&S36972[34], 15}, {NULL}};
const Segment SGI149142[] = {{&S36972[27], 6}, {NULL}};
const Segment SGI149143[] = {{&S36972[105], 9}, {NULL}};
const Segment SGI149144[] = {{&S36972[78], 6}, {NULL}};
const Segment SGI149145[] = {{&S36974[34], 7}, {NULL}};
const Segment SGI149146[] = {{&S36974[27], 6}, {NULL}};
const Segment SGI149147[] = {{&S36974[110], 9}, {NULL}};
const Segment SGI149148[] = {{&S36974[83], 6}, {NULL}};
const Segment SGI149149[] = {{&S46651[34], 13}, {NULL}};
const Segment SGI149150[] = {{&S46651[27], 6}, {NULL}};
const Segment SGI149151[] = {{&S46651[100], 9}, {NULL}};
const Segment SGI149152[] = {{&S46651[73], 6}, {NULL}};
const Segment SGI149153[] = {{&S46652[34], 10}, {NULL}};
const Segment SGI149154[] = {{&S46652[27], 6}, {NULL}};
const Segment SGI149155[] = {{&S46652[97], 9}, {NULL}};
const Segment SGI149156[] = {{&S46652[70], 6}, {NULL}};
const Segment SGI149157[] = {{&S36998[44], 7}, {NULL}};
const Segment SGI149158[] = {{&S36998[44], 35}, {NULL}};
const Segment SGI149159[] = {{&S36998[30], 6}, {NULL}};
const Segment SGI149160[] = {{&S46653[34], 10}, {NULL}};
const Segment SGI149161[] = {{&S46653[27], 6}, {NULL}};
const Segment SGI149162[] = {{&S36998[128], 3}, {NULL}};
const Segment SGI149163[] = {{&S36998[93], 6}, {NULL}};
const Segment SGI149164[] = {{&S46653[97], 9}, {NULL}};
const Segment SGI149165[] = {{&S46653[70], 6}, {NULL}};
const Segment SGI149166[] = {{&S36999[38], 7}, {NULL}};
const Segment SGI149167[] = {{&S36999[31], 6}, {NULL}};
const Segment SGI149168[] = {{&S46654[34], 14}, {NULL}};
const Segment SGI149169[] = {{&S46654[27], 6}, {NULL}};
const Segment SGI149170[] = {{&S46654[101], 9}, {NULL}};
const Segment SGI149171[] = {{&S46654[74], 6}, {NULL}};
const Segment SGI149172[] = {{&S37001[44], 7}, {NULL}};
const Segment SGI149173[] = {{&S37001[44], 33}, {NULL}};
const Segment SGI149174[] = {{&S37001[30], 6}, {NULL}};
const Segment SGI149175[] = {{&S37001[126], 3}, {NULL}};
const Segment SGI149176[] = {{&S37001[91], 6}, {NULL}};
const Segment SGI149177[] = {{&S46658[44], 11}, {NULL}};
const Segment SGI149178[] = {{&S46658[44], 23}, {NULL}};
const Segment SGI149179[] = {{&S46658[30], 6}, {NULL}};
const Segment SGI149180[] = {{&S37002[37], 15}, {NULL}};
const Segment SGI149181[] = {{&S46658[31], 3}, {NULL}};
const Segment SGI149182[] = {{&S46658[81], 6}, {NULL}};
const Segment SGI149183[] = {{&S46659[38], 11}, {NULL}};
const Segment SGI149184[] = {{&S37003[42], 7}, {NULL}};
const Segment SGI149185[] = {{&S46659[31], 6}, {NULL}};
const Segment SGI149186[] = {{&S37003[30], 6}, {NULL}};
const Segment SGI149187[] = {{&S37003[126], 3}, {NULL}};
const Segment SGI149188[] = {{&S37003[91], 6}, {NULL}};
const Segment SGI149189[] = {{&S46660[42], 11}, {NULL}};
const Segment SGI149190[] = {{&S46660[30], 6}, {NULL}};
const Segment SGI149191[] = {{&S46660[31], 3}, {NULL}};
const Segment SGI149192[] = {{&S46660[79], 6}, {NULL}};
const Segment SGI149193[] = {{&S37005[55], 7}, {NULL}};
const Segment SGI149194[] = {{&S37005[55], 35}, {NULL}};
const Segment SGI149195[] = {{&S37005[41], 6}, {NULL}};
const Segment SGI149196[] = {{&S37005[143], 3}, {NULL}};
const Segment SGI149197[] = {{&S37005[109], 5}, {NULL}};
const Segment SGI149198[] = {{&S46722[44], 13}, {NULL}};
const Segment SGI149199[] = {{&S46722[44], 25}, {NULL}};
const Segment SGI149200[] = {{&S46722[30], 6}, {NULL}};
const Segment SGI149201[] = {{&S46722[31], 3}, {NULL}};
const Segment SGI149202[] = {{&S46722[83], 6}, {NULL}};
const Segment SGI149203[] = {{&S46723[44], 10}, {NULL}};
const Segment SGI149204[] = {{&S46723[44], 22}, {NULL}};
const Segment SGI149205[] = {{&S46723[30], 6}, {NULL}};
const Segment SGI149206[] = {{&S46723[31], 3}, {NULL}};
const Segment SGI149207[] = {{&S46723[80], 6}, {NULL}};
const Segment SGI149208[] = {{&S46724[38], 10}, {NULL}};
const Segment SGI149209[] = {{&S46724[31], 6}, {NULL}};
const Segment SGI149210[] = {{&S46735[44], 10}, {NULL}};
const Segment SGI149211[] = {{&S46735[44], 32}, {NULL}};
const Segment SGI149212[] = {{&S46735[30], 6}, {NULL}};
const Segment SGI149213[] = {{&S46735[31], 3}, {NULL}};
const Segment SGI149214[] = {{&S46735[90], 5}, {NULL}};
const Segment SGI149215[] = {{&S46739[44], 10}, {NULL}};
const Segment SGI149216[] = {{&S46739[44], 27}, {NULL}};
const Segment SGI149217[] = {{&S46739[30], 6}, {NULL}};
const Segment SGI149218[] = {{&S46739[31], 3}, {NULL}};
const Segment SGI149219[] = {{&S46739[85], 5}, {NULL}};
const Segment SGI149220[] = {{&S46742[44], 10}, {NULL}};
const Segment SGI149221[] = {{&S46742[44], 25}, {NULL}};
const Segment SGI149222[] = {{&S46742[30], 6}, {NULL}};
const Segment SGI149223[] = {{&S46742[31], 3}, {NULL}};
const Segment SGI149224[] = {{&S46742[83], 5}, {NULL}};
const Segment SGI149225[] = {{&S46743[44], 10}, {NULL}};
const Segment SGI149226[] = {{&S46743[44], 22}, {NULL}};
const Segment SGI149227[] = {{&S46743[30], 6}, {NULL}};
const Segment SGI149228[] = {{&S46743[31], 3}, {NULL}};
const Segment SGI149229[] = {{&S46743[80], 6}, {NULL}};
const Segment SGI149230[] = {{&S46745[44], 7}, {NULL}};
const Segment SGI149231[] = {{&S46745[44], 28}, {NULL}};
const Segment SGI149232[] = {{&S46745[30], 6}, {NULL}};
const Segment SGI149233[] = {{&S46745[31], 3}, {NULL}};
const Segment SGI149234[] = {{&S46745[86], 5}, {NULL}};
const Segment SGI149235[] = {{&S46786[34], 7}, {NULL}};
const Segment SGI149236[] = {{&S46786[27], 6}, {NULL}};
const Segment SGI149237[] = {{&S46786[105], 9}, {NULL}};
const Segment SGI149238[] = {{&S46786[78], 6}, {NULL}};
const Segment SGI149239[] = {{&S46805[44], 10}, {NULL}};
const Segment SGI149240[] = {{&S46805[44], 22}, {NULL}};
const Segment SGI149241[] = {{&S46805[30], 6}, {NULL}};
const Segment SGI149242[] = {{&S46805[31], 3}, {NULL}};
const Segment SGI149243[] = {{&S46805[80], 6}, {NULL}};
const Segment SGI149244[] = {{&S46806[44], 11}, {NULL}};
const Segment SGI149245[] = {{&S46806[44], 23}, {NULL}};
const Segment SGI149246[] = {{&S46806[30], 6}, {NULL}};
const Segment SGI149247[] = {{&S46806[31], 3}, {NULL}};
const Segment SGI149248[] = {{&S46806[81], 6}, {NULL}};
const Segment SGI149249[] = {{&S46807[38], 10}, {NULL}};
const Segment SGI149250[] = {{&S46807[31], 6}, {NULL}};
const Segment SGI149251[] = {{&S46827[34], 10}, {NULL}};
const Segment SGI149252[] = {{&S46827[27], 6}, {NULL}};
const Segment SGI149253[] = {{&S46827[97], 9}, {NULL}};
const Segment SGI149254[] = {{&S46827[70], 6}, {NULL}};
const Segment SGI149255[] = {{&S46829[34], 11}, {NULL}};
const Segment SGI149256[] = {{&S46829[27], 6}, {NULL}};
const Segment SGI149257[] = {{&S46829[98], 9}, {NULL}};
const Segment SGI149258[] = {{&S46829[71], 6}, {NULL}};
const Segment SGI149259[] = {{&S46835[44], 10}, {NULL}};
const Segment SGI149260[] = {{&S46835[44], 33}, {NULL}};
const Segment SGI149261[] = {{&S46835[30], 6}, {NULL}};
const Segment SGI149262[] = {{&S46835[31], 3}, {NULL}};
const Segment SGI149263[] = {{&S46835[91], 6}, {NULL}};
const Segment SGI149264[] = {{&S46836[44], 7}, {NULL}};
const Segment SGI149265[] = {{&S46836[44], 30}, {NULL}};
const Segment SGI149266[] = {{&S46836[30], 6}, {NULL}};
const Segment SGI149267[] = {{&S46836[31], 3}, {NULL}};
const Segment SGI149268[] = {{&S46836[88], 6}, {NULL}};
const Segment SGI149269[] = {{&S46837[38], 7}, {NULL}};
const Segment SGI149270[] = {{&S46837[31], 6}, {NULL}};
const Segment SGI149271[] = {{&S46838[44], 10}, {NULL}};
const Segment SGI149272[] = {{&S46838[44], 33}, {NULL}};
const Segment SGI149273[] = {{&S46838[30], 6}, {NULL}};
const Segment SGI149274[] = {{&S46838[31], 3}, {NULL}};
const Segment SGI149275[] = {{&S46838[91], 6}, {NULL}};
const Segment SGI149276[] = {{&S46839[44], 10}, {NULL}};
const Segment SGI149277[] = {{&S46839[44], 22}, {NULL}};
const Segment SGI149278[] = {{&S46839[30], 6}, {NULL}};
const Segment SGI149279[] = {{&S46839[31], 3}, {NULL}};
const Segment SGI149280[] = {{&S46839[80], 6}, {NULL}};
const Segment SGI149281[] = {{&S46840[38], 10}, {NULL}};
const Segment SGI149282[] = {{&S46840[31], 6}, {NULL}};
const Segment SGI149283[] = {{&S46858[34], 10}, {NULL}};
const Segment SGI149284[] = {{&S46858[27], 6}, {NULL}};
const Segment SGI149285[] = {{&S46858[97], 9}, {NULL}};
const Segment SGI149286[] = {{&S46858[70], 6}, {NULL}};
const Segment SGI149287[] = {{&S46864[44], 10}, {NULL}};
const Segment SGI149288[] = {{&S46864[44], 22}, {NULL}};
const Segment SGI149289[] = {{&S46864[30], 6}, {NULL}};
const Segment SGI149290[] = {{&S46864[31], 3}, {NULL}};
const Segment SGI149291[] = {{&S46864[80], 6}, {NULL}};
const Segment SGI149292[] = {{&S46865[44], 10}, {NULL}};
const Segment SGI149293[] = {{&S46865[44], 22}, {NULL}};
const Segment SGI149294[] = {{&S46865[30], 6}, {NULL}};
const Segment SGI149295[] = {{&S46865[31], 3}, {NULL}};
const Segment SGI149296[] = {{&S46865[80], 6}, {NULL}};
const Segment SGI149297[] = {{&S46866[38], 10}, {NULL}};
const Segment SGI149298[] = {{&S46866[31], 6}, {NULL}};
const Segment SGI149299[] = {{&S46867[38], 10}, {NULL}};
const Segment SGI149300[] = {{&S46867[31], 6}, {NULL}};
const Segment SGI149301[] = {{&S46868[42], 10}, {NULL}};
const Segment SGI149302[] = {{&S46868[30], 6}, {NULL}};
const Segment SGI149303[] = {{&S46868[31], 3}, {NULL}};
const Segment SGI149304[] = {{&S46868[78], 6}, {NULL}};
const Segment SGI149305[] = {{&S46869[42], 10}, {NULL}};
const Segment SGI149306[] = {{&S46869[30], 6}, {NULL}};
const Segment SGI149307[] = {{&S46869[31], 3}, {NULL}};
const Segment SGI149308[] = {{&S46869[78], 6}, {NULL}};
const Segment SGI149309[] = {{&S46870[44], 20}, {NULL}};
const Segment SGI149310[] = {{&S46870[44], 32}, {NULL}};
const Segment SGI149311[] = {{&S46870[30], 6}, {NULL}};
const Segment SGI149312[] = {{&S46870[31], 3}, {NULL}};
const Segment SGI149313[] = {{&S46870[90], 6}, {NULL}};
const Segment SGI149314[] = {{&S46871[44], 10}, {NULL}};
const Segment SGI149315[] = {{&S46871[44], 41}, {NULL}};
const Segment SGI149316[] = {{&S46871[30], 6}, {NULL}};
const Segment SGI149317[] = {{&S46871[31], 3}, {NULL}};
const Segment SGI149318[] = {{&S46871[99], 6}, {NULL}};
const Segment SGI149319[] = {{&S46872[44], 10}, {NULL}};
const Segment SGI149320[] = {{&S46872[44], 39}, {NULL}};
const Segment SGI149321[] = {{&S46872[30], 6}, {NULL}};
const Segment SGI149322[] = {{&S46872[31], 3}, {NULL}};
const Segment SGI149323[] = {{&S46872[97], 6}, {NULL}};
const Segment SGI149324[] = {{&S46873[44], 10}, {NULL}};
const Segment SGI149325[] = {{&S46873[44], 39}, {NULL}};
const Segment SGI149326[] = {{&S46873[30], 6}, {NULL}};
const Segment SGI149327[] = {{&S46873[31], 3}, {NULL}};
const Segment SGI149328[] = {{&S46873[97], 6}, {NULL}};
const Segment SGI149329[] = {{&S46874[42], 20}, {NULL}};
const Segment SGI149330[] = {{&S46874[30], 6}, {NULL}};
const Segment SGI149331[] = {{&S46874[31], 3}, {NULL}};
const Segment SGI149332[] = {{&S46874[88], 6}, {NULL}};
const Segment SGI149333[] = {{&S46875[44], 10}, {NULL}};
const Segment SGI149334[] = {{&S46875[44], 22}, {NULL}};
const Segment SGI149335[] = {{&S46875[30], 6}, {NULL}};
const Segment SGI149336[] = {{&S46875[31], 3}, {NULL}};
const Segment SGI149337[] = {{&S46875[80], 6}, {NULL}};
const Segment SGI149338[] = {{&S46876[44], 10}, {NULL}};
const Segment SGI149339[] = {{&S46876[44], 39}, {NULL}};
const Segment SGI149340[] = {{&S46876[30], 6}, {NULL}};
const Segment SGI149341[] = {{&S46876[31], 3}, {NULL}};
const Segment SGI149342[] = {{&S46876[97], 6}, {NULL}};
const Segment SGI149343[] = {{&S37247[38], 29}, {NULL}};
const Segment SGI149344[] = {{&S37247[31], 6}, {NULL}};
const Segment SGI149345[] = {{&S46877[44], 20}, {NULL}};
const Segment SGI149346[] = {{&S46877[44], 32}, {NULL}};
const Segment SGI149347[] = {{&S46877[30], 6}, {NULL}};
const Segment SGI149348[] = {{&S46877[31], 3}, {NULL}};
const Segment SGI149349[] = {{&S46877[90], 6}, {NULL}};
const Segment SGI149350[] = {{&S37248[44], 29}, {NULL}};
const Segment SGI149351[] = {{&S37248[44], 41}, {NULL}};
const Segment SGI149352[] = {{&S37248[30], 6}, {NULL}};
const Segment SGI149353[] = {{&S37248[134], 3}, {NULL}};
const Segment SGI149354[] = {{&S37248[99], 6}, {NULL}};
const Segment SGI149355[] = {{&S37249[44], 17}, {NULL}};
const Segment SGI149356[] = {{&S37249[44], 29}, {NULL}};
const Segment SGI149357[] = {{&S37249[30], 6}, {NULL}};
const Segment SGI149358[] = {{&S37249[122], 3}, {NULL}};
const Segment SGI149359[] = {{&S37249[87], 6}, {NULL}};
const Segment SGI149360[] = {{&S37250[44], 30}, {NULL}};
const Segment SGI149361[] = {{&S37250[44], 42}, {NULL}};
const Segment SGI149362[] = {{&S37250[30], 6}, {NULL}};
const Segment SGI149363[] = {{&S37250[135], 3}, {NULL}};
const Segment SGI149364[] = {{&S37250[100], 6}, {NULL}};
const Segment SGI149365[] = {{&S37251[44], 29}, {NULL}};
const Segment SGI149366[] = {{&S37251[44], 41}, {NULL}};
const Segment SGI149367[] = {{&S37251[30], 6}, {NULL}};
const Segment SGI149368[] = {{&S37251[134], 3}, {NULL}};
const Segment SGI149369[] = {{&S37251[99], 6}, {NULL}};
const Segment SGI149370[] = {{&S22811[34], 9}, {NULL}};
const Segment SGI149371[] = {{&S22811[27], 6}, {NULL}};
const Segment SGI149372[] = {{&S22811[95], 9}, {NULL}};
const Segment SGI149373[] = {{&S22811[68], 6}, {NULL}};
const Segment SGI149374[] = {{&S37325[44], 12}, {NULL}};
const Segment SGI149375[] = {{&S37325[44], 24}, {NULL}};
const Segment SGI149376[] = {{&S37325[30], 6}, {NULL}};
const Segment SGI149377[] = {{&S37325[117], 3}, {NULL}};
const Segment SGI149378[] = {{&S37325[82], 6}, {NULL}};
const Segment SGI149379[] = {{&S37326[44], 12}, {NULL}};
const Segment SGI149380[] = {{&S37326[44], 24}, {NULL}};
const Segment SGI149381[] = {{&S37326[30], 6}, {NULL}};
const Segment SGI149382[] = {{&S37326[117], 3}, {NULL}};
const Segment SGI149383[] = {{&S37326[82], 6}, {NULL}};
const Segment SGI149384[] = {{&S37327[44], 26}, {NULL}};
const Segment SGI149385[] = {{&S37327[44], 38}, {NULL}};
const Segment SGI149386[] = {{&S37327[30], 6}, {NULL}};
const Segment SGI149387[] = {{&S37327[131], 3}, {NULL}};
const Segment SGI149388[] = {{&S37327[96], 6}, {NULL}};
const Segment SGI149389[] = {{&S37328[44], 12}, {NULL}};
const Segment SGI149390[] = {{&S37328[44], 24}, {NULL}};
const Segment SGI149391[] = {{&S37328[30], 6}, {NULL}};
const Segment SGI149392[] = {{&S37328[117], 3}, {NULL}};
const Segment SGI149393[] = {{&S37328[82], 6}, {NULL}};
const Segment SGI149394[] = {{&S37329[38], 12}, {NULL}};
const Segment SGI149395[] = {{&S37329[31], 6}, {NULL}};
const Segment SGI149396[] = {{&S37330[44], 26}, {NULL}};
const Segment SGI149397[] = {{&S37330[44], 47}, {NULL}};
const Segment SGI149398[] = {{&S37330[30], 6}, {NULL}};
const Segment SGI149399[] = {{&S37330[140], 3}, {NULL}};
const Segment SGI149400[] = {{&S37330[105], 6}, {NULL}};
const Segment SGI149401[] = {{&S47006[44], 10}, {NULL}};
const Segment SGI149402[] = {{&S47006[44], 22}, {NULL}};
const Segment SGI149403[] = {{&S47006[30], 6}, {NULL}};
const Segment SGI149404[] = {{&S47006[31], 3}, {NULL}};
const Segment SGI149405[] = {{&S47006[80], 6}, {NULL}};
const Segment SGI149406[] = {{&S47007[38], 10}, {NULL}};
const Segment SGI149407[] = {{&S47007[31], 6}, {NULL}};
const Segment SGI149408[] = {{&S36971[34], 15}, {NULL}};
const Segment SGI149409[] = {{&S36971[27], 6}, {NULL}};
const Segment SGI149410[] = {{&S36971[114], 9}, {NULL}};
const Segment SGI149411[] = {{&S36971[87], 6}, {NULL}};
const Segment SGI149412[] = {{&S36973[34], 7}, {NULL}};
const Segment SGI149413[] = {{&S36973[27], 6}, {NULL}};
const Segment SGI149414[] = {{&S36973[111], 9}, {NULL}};
const Segment SGI149415[] = {{&S36973[84], 6}, {NULL}};
const Segment SGI149416[] = {{&S47043[44], 11}, {NULL}};
const Segment SGI149417[] = {{&S47043[44], 23}, {NULL}};
const Segment SGI149418[] = {{&S47043[30], 6}, {NULL}};
const Segment SGI149419[] = {{&S47043[31], 3}, {NULL}};
const Segment SGI149420[] = {{&S47043[81], 6}, {NULL}};
const Segment SGI149421[] = {{&S47044[38], 11}, {NULL}};
const Segment SGI149422[] = {{&S47044[31], 6}, {NULL}};
const Segment SGI149423[] = {{&S47045[42], 11}, {NULL}};
const Segment SGI149424[] = {{&S47045[30], 6}, {NULL}};
const Segment SGI149425[] = {{&S47045[31], 3}, {NULL}};
const Segment SGI149426[] = {{&S47045[79], 6}, {NULL}};
const Segment SGI149427[] = {{&S32442[44], 10}, {NULL}};
const Segment SGI149428[] = {{&S32442[44], 22}, {NULL}};
const Segment SGI149429[] = {{&S32442[30], 6}, {NULL}};
const Segment SGI149430[] = {{&S32442[133], 3}, {NULL}};
const Segment SGI149431[] = {{&S32442[98], 6}, {NULL}};
const Segment SGI149432[] = {{&S32442[68], 8}, {NULL}};
const Segment SGI149433[] = {{&S32443[44], 10}, {NULL}};
const Segment SGI149434[] = {{&S32443[44], 22}, {NULL}};
const Segment SGI149435[] = {{&S32443[30], 6}, {NULL}};
const Segment SGI149436[] = {{&S32443[133], 3}, {NULL}};
const Segment SGI149437[] = {{&S32443[98], 6}, {NULL}};
const Segment SGI149438[] = {{&S32443[68], 8}, {NULL}};
const Segment SGI149439[] = {{&S32444[44], 10}, {NULL}};
const Segment SGI149440[] = {{&S32444[44], 22}, {NULL}};
const Segment SGI149441[] = {{&S32444[30], 6}, {NULL}};
const Segment SGI149442[] = {{&S32444[133], 3}, {NULL}};
const Segment SGI149443[] = {{&S32444[98], 6}, {NULL}};
const Segment SGI149444[] = {{&S32444[68], 8}, {NULL}};
const Segment SGI149445[] = {{&S32445[44], 10}, {NULL}};
const Segment SGI149446[] = {{&S32445[44], 22}, {NULL}};
const Segment SGI149447[] = {{&S32445[30], 6}, {NULL}};
const Segment SGI149448[] = {{&S32445[133], 3}, {NULL}};
const Segment SGI149449[] = {{&S32445[98], 6}, {NULL}};
const Segment SGI149450[] = {{&S32445[68], 8}, {NULL}};
const Segment SGI149451[] = {{&S37000[44], 7}, {NULL}};
const Segment SGI149452[] = {{&S32446[44], 10}, {NULL}};
const Segment SGI149453[] = {{&S37000[44], 34}, {NULL}};
const Segment SGI149454[] = {{&S32446[44], 22}, {NULL}};
const Segment SGI149455[] = {{&S37000[30], 6}, {NULL}};
const Segment SGI149456[] = {{&S32446[30], 6}, {NULL}};
const Segment SGI149457[] = {{&S37000[127], 3}, {NULL}};
const Segment SGI149458[] = {{&S32446[133], 3}, {NULL}};
const Segment SGI149459[] = {{&S37000[92], 6}, {NULL}};
const Segment SGI149460[] = {{&S32446[98], 6}, {NULL}};
const Segment SGI149461[] = {{&S32446[68], 8}, {NULL}};
const Segment SGI149462[] = {{&S32447[44], 10}, {NULL}};
const Segment SGI149463[] = {{&S32447[44], 22}, {NULL}};
const Segment SGI149464[] = {{&S32447[30], 6}, {NULL}};
const Segment SGI149465[] = {{&S32447[133], 3}, {NULL}};
const Segment SGI149466[] = {{&S32447[98], 6}, {NULL}};
const Segment SGI149467[] = {{&S32447[68], 8}, {NULL}};
const Segment SGI149468[] = {{&S37004[38], 7}, {NULL}};
const Segment SGI149469[] = {{&S37004[31], 6}, {NULL}};
const Segment SGI149470[] = {{&S47065[34], 19}, {NULL}};
const Segment SGI149471[] = {{&S47065[27], 6}, {NULL}};
const Segment SGI149472[] = {{&S47065[106], 9}, {NULL}};
const Segment SGI149473[] = {{&S37006[44], 7}, {NULL}};
const Segment SGI149474[] = {{&S47065[79], 6}, {NULL}};
const Segment SGI149475[] = {{&S37006[44], 37}, {NULL}};
const Segment SGI149476[] = {{&S37006[30], 6}, {NULL}};
const Segment SGI149477[] = {{&S37006[130], 3}, {NULL}};
const Segment SGI149478[] = {{&S37006[95], 6}, {NULL}};
const Segment SGI149479[] = {{&S47066[34], 18}, {NULL}};
const Segment SGI149480[] = {{&S47066[27], 6}, {NULL}};
const Segment SGI149481[] = {{&S47066[105], 9}, {NULL}};
const Segment SGI149482[] = {{&S47066[78], 6}, {NULL}};
const Segment SGI149483[] = {{&S47068[34], 10}, {NULL}};
const Segment SGI149484[] = {{&S47068[27], 6}, {NULL}};
const Segment SGI149485[] = {{&S47068[97], 9}, {NULL}};
const Segment SGI149486[] = {{&S47068[70], 6}, {NULL}};
const Segment SGI149487[] = {{&S47088[44], 10}, {NULL}};
const Segment SGI149488[] = {{&S47088[44], 22}, {NULL}};
const Segment SGI149489[] = {{&S47088[30], 6}, {NULL}};
const Segment SGI149490[] = {{&S47088[31], 3}, {NULL}};
const Segment SGI149491[] = {{&S47088[80], 6}, {NULL}};
const Segment SGI149492[] = {{&S47089[38], 10}, {NULL}};
const Segment SGI149493[] = {{&S47089[31], 6}, {NULL}};
const Segment SGI149494[] = {{&S47091[42], 10}, {NULL}};
const Segment SGI149495[] = {{&S47091[30], 6}, {NULL}};
const Segment SGI149496[] = {{&S47091[31], 3}, {NULL}};
const Segment SGI149497[] = {{&S47091[78], 6}, {NULL}};
const Segment SGI149498[] = {{&S47092[34], 11}, {NULL}};
const Segment SGI149499[] = {{&S47092[27], 6}, {NULL}};
const Segment SGI149500[] = {{&S47092[98], 9}, {NULL}};
const Segment SGI149501[] = {{&S47092[71], 6}, {NULL}};
const Segment SGI149502[] = {{&S47095[34], 10}, {NULL}};
const Segment SGI149503[] = {{&S47095[27], 6}, {NULL}};
const Segment SGI149504[] = {{&S47095[97], 9}, {NULL}};
const Segment SGI149505[] = {{&S47095[70], 6}, {NULL}};
const Segment SGI149506[] = {{&S47096[34], 18}, {NULL}};
const Segment SGI149507[] = {{&S47096[27], 6}, {NULL}};
const Segment SGI149508[] = {{&S47096[105], 9}, {NULL}};
const Segment SGI149509[] = {{&S47096[78], 6}, {NULL}};
const Segment SGI149510[] = {{&S37442[34], 7}, {NULL}};
const Segment SGI149511[] = {{&S37442[27], 6}, {NULL}};
const Segment SGI149512[] = {{&S37442[100], 9}, {NULL}};
const Segment SGI149513[] = {{&S37442[73], 6}, {NULL}};
const Segment SGI149514[] = {{&S47140[44], 18}, {NULL}};
const Segment SGI149515[] = {{&S47140[44], 30}, {NULL}};
const Segment SGI149516[] = {{&S47140[30], 6}, {NULL}};
const Segment SGI149517[] = {{&S47140[31], 3}, {NULL}};
const Segment SGI149518[] = {{&S47140[88], 6}, {NULL}};
const Segment SGI149519[] = {{&S47141[44], 11}, {NULL}};
const Segment SGI149520[] = {{&S47141[44], 23}, {NULL}};
const Segment SGI149521[] = {{&S47141[30], 6}, {NULL}};
const Segment SGI149522[] = {{&S47141[31], 3}, {NULL}};
const Segment SGI149523[] = {{&S47141[81], 6}, {NULL}};
const Segment SGI149524[] = {{&S47142[38], 18}, {NULL}};
const Segment SGI149525[] = {{&S47142[31], 6}, {NULL}};
const Segment SGI149526[] = {{&S47143[38], 11}, {NULL}};
const Segment SGI149527[] = {{&S47143[31], 6}, {NULL}};
const Segment SGI149528[] = {{&S47153[34], 7}, {NULL}};
const Segment SGI149529[] = {{&S47153[27], 6}, {NULL}};
const Segment SGI149530[] = {{&S47153[98], 9}, {NULL}};
const Segment SGI149531[] = {{&S47153[71], 6}, {NULL}};
const Segment SGI149532[] = {{&S47154[34], 7}, {NULL}};
const Segment SGI149533[] = {{&S47154[27], 6}, {NULL}};
const Segment SGI149534[] = {{&S47154[98], 9}, {NULL}};
const Segment SGI149535[] = {{&S47154[71], 6}, {NULL}};
const Segment SGI149536[] = {{&S47162[44], 9}, {NULL}};
const Segment SGI149537[] = {{&S47162[44], 21}, {NULL}};
const Segment SGI149538[] = {{&S47162[30], 6}, {NULL}};
const Segment SGI149539[] = {{&S47162[31], 3}, {NULL}};
const Segment SGI149540[] = {{&S47162[79], 6}, {NULL}};
const Segment SGI149541[] = {{&S47163[44], 9}, {NULL}};
const Segment SGI149542[] = {{&S47163[44], 21}, {NULL}};
const Segment SGI149543[] = {{&S47163[30], 6}, {NULL}};
const Segment SGI149544[] = {{&S47163[31], 3}, {NULL}};
const Segment SGI149545[] = {{&S47163[79], 6}, {NULL}};
const Segment SGI149546[] = {{&S47167[38], 9}, {NULL}};
const Segment SGI149547[] = {{&S47167[31], 6}, {NULL}};
const Segment SGI149548[] = {{&S47170[44], 7}, {NULL}};
const Segment SGI149549[] = {{&S47170[44], 28}, {NULL}};
const Segment SGI149550[] = {{&S47170[30], 6}, {NULL}};
const Segment SGI149551[] = {{&S47170[31], 3}, {NULL}};
const Segment SGI149552[] = {{&S47170[86], 6}, {NULL}};
const Segment SGI149553[] = {{&S47171[44], 7}, {NULL}};
const Segment SGI149554[] = {{&S47171[44], 28}, {NULL}};
const Segment SGI149555[] = {{&S47171[30], 6}, {NULL}};
const Segment SGI149556[] = {{&S47171[31], 3}, {NULL}};
const Segment SGI149557[] = {{&S47171[86], 6}, {NULL}};
const Segment SGI149558[] = {{&S47172[38], 7}, {NULL}};
const Segment SGI149559[] = {{&S47172[31], 6}, {NULL}};
const Segment SGI149560[] = {{&S47173[44], 7}, {NULL}};
const Segment SGI149561[] = {{&S47173[44], 32}, {NULL}};
const Segment SGI149562[] = {{&S47173[30], 6}, {NULL}};
const Segment SGI149563[] = {{&S47173[31], 3}, {NULL}};
const Segment SGI149564[] = {{&S47173[90], 6}, {NULL}};
const Segment SGI149565[] = {{&S47174[44], 7}, {NULL}};
const Segment SGI149566[] = {{&S47174[44], 32}, {NULL}};
const Segment SGI149567[] = {{&S47174[30], 6}, {NULL}};
const Segment SGI149568[] = {{&S47174[31], 3}, {NULL}};
const Segment SGI149569[] = {{&S47174[90], 6}, {NULL}};
const Segment SGI149570[] = {{&S47175[44], 10}, {NULL}};
const Segment SGI149571[] = {{&S47175[44], 22}, {NULL}};
const Segment SGI149572[] = {{&S47175[30], 6}, {NULL}};
const Segment SGI149573[] = {{&S47175[31], 3}, {NULL}};
const Segment SGI149574[] = {{&S47175[80], 6}, {NULL}};
const Segment SGI149575[] = {{&S47176[38], 10}, {NULL}};
const Segment SGI149576[] = {{&S47176[31], 6}, {NULL}};
const Segment SGI149577[] = {{&S47177[42], 10}, {NULL}};
const Segment SGI149578[] = {{&S47177[30], 6}, {NULL}};
const Segment SGI149579[] = {{&S47177[31], 3}, {NULL}};
const Segment SGI149580[] = {{&S47177[78], 6}, {NULL}};
const Segment SGI149581[] = {{&S47178[44], 10}, {NULL}};
const Segment SGI149582[] = {{&S47178[44], 22}, {NULL}};
const Segment SGI149583[] = {{&S47178[30], 6}, {NULL}};
const Segment SGI149584[] = {{&S47178[31], 3}, {NULL}};
const Segment SGI149585[] = {{&S47178[80], 5}, {NULL}};
const Segment SGI149586[] = {{&S47179[44], 16}, {NULL}};
const Segment SGI149587[] = {{&S47179[44], 28}, {NULL}};
const Segment SGI149588[] = {{&S47179[30], 6}, {NULL}};
const Segment SGI149589[] = {{&S47179[31], 3}, {NULL}};
const Segment SGI149590[] = {{&S47179[86], 5}, {NULL}};
const Segment SGI149591[] = {{&S47180[44], 18}, {NULL}};
const Segment SGI149592[] = {{&S47180[44], 30}, {NULL}};
const Segment SGI149593[] = {{&S47180[30], 6}, {NULL}};
const Segment SGI149594[] = {{&S47180[31], 3}, {NULL}};
const Segment SGI149595[] = {{&S47180[88], 5}, {NULL}};
const Segment SGI149596[] = {{&S47182[44], 20}, {NULL}};
const Segment SGI149597[] = {{&S47182[44], 32}, {NULL}};
const Segment SGI149598[] = {{&S47182[30], 6}, {NULL}};
const Segment SGI149599[] = {{&S47182[31], 3}, {NULL}};
const Segment SGI149600[] = {{&S47182[90], 5}, {NULL}};
const Segment SGI149601[] = {{&S47183[44], 16}, {NULL}};
const Segment SGI149602[] = {{&S47183[44], 28}, {NULL}};
const Segment SGI149603[] = {{&S47183[30], 6}, {NULL}};
const Segment SGI149604[] = {{&S47183[31], 3}, {NULL}};
const Segment SGI149605[] = {{&S47183[86], 5}, {NULL}};
const Segment SGI149606[] = {{&S47253[34], 13}, {NULL}};
const Segment SGI149607[] = {{&S47253[27], 6}, {NULL}};
const Segment SGI149608[] = {{&S47253[104], 9}, {NULL}};
const Segment SGI149609[] = {{&S47253[77], 6}, {NULL}};
const Segment SGI149610[] = {{&S47317[34], 10}, {NULL}};
const Segment SGI149611[] = {{&S47317[27], 6}, {NULL}};
const Segment SGI149612[] = {{&S47317[97], 9}, {NULL}};
const Segment SGI149613[] = {{&S47317[70], 6}, {NULL}};
const Segment SGI149614[] = {{&S47347[34], 11}, {NULL}};
const Segment SGI149615[] = {{&S47347[27], 6}, {NULL}};
const Segment SGI149616[] = {{&S47347[98], 9}, {NULL}};
const Segment SGI149617[] = {{&S47347[71], 6}, {NULL}};
const Segment SGI149618[] = {{&S37331[44], 12}, {NULL}};
const Segment SGI149619[] = {{&S37331[44], 24}, {NULL}};
const Segment SGI149620[] = {{&S37331[30], 6}, {NULL}};
const Segment SGI149621[] = {{&S37331[117], 3}, {NULL}};
const Segment SGI149622[] = {{&S37331[82], 6}, {NULL}};
const Segment SGI149623[] = {{&S47354[34], 13}, {NULL}};
const Segment SGI149624[] = {{&S47354[27], 6}, {NULL}};
const Segment SGI149625[] = {{&S47354[100], 9}, {NULL}};
const Segment SGI149626[] = {{&S47354[73], 6}, {NULL}};
const Segment SGI149627[] = {{&S37716[34], 29}, {NULL}};
const Segment SGI149628[] = {{&S37716[27], 6}, {NULL}};
const Segment SGI149629[] = {{&S37716[116], 9}, {NULL}};
const Segment SGI149630[] = {{&S37716[89], 6}, {NULL}};
const Segment SGI149631[] = {{&S37717[34], 30}, {NULL}};
const Segment SGI149632[] = {{&S37717[27], 6}, {NULL}};
const Segment SGI149633[] = {{&S37717[117], 9}, {NULL}};
const Segment SGI149634[] = {{&S37717[90], 6}, {NULL}};
const Segment SGI149635[] = {{&S37718[34], 30}, {NULL}};
const Segment SGI149636[] = {{&S37718[27], 6}, {NULL}};
const Segment SGI149637[] = {{&S37718[117], 9}, {NULL}};
const Segment SGI149638[] = {{&S37718[90], 6}, {NULL}};
const Segment SGI149639[] = {{&S37719[34], 17}, {NULL}};
const Segment SGI149640[] = {{&S37719[27], 6}, {NULL}};
const Segment SGI149641[] = {{&S37719[104], 9}, {NULL}};
const Segment SGI149642[] = {{&S37719[77], 6}, {NULL}};
const Segment SGI149643[] = {{&S42058[11], 1}, {NULL}};
const Segment SGI149644[] = {{&S42058[27], 4}, {NULL}};
const Segment SGI149645[] = {{&S42058[86], 9}, {NULL}};
const Segment SGI149646[] = {{&S42058[59], 6}, {NULL}};
const Segment SGI149647[] = {{&S37721[34], 24}, {NULL}};
const Segment SGI149648[] = {{&S37721[27], 6}, {NULL}};
const Segment SGI149649[] = {{&S37721[111], 9}, {NULL}};
const Segment SGI149650[] = {{&S37721[84], 6}, {NULL}};
const Segment SGI149651[] = {{&S37443[34], 7}, {NULL}};
const Segment SGI149652[] = {{&S37443[27], 6}, {NULL}};
const Segment SGI149653[] = {{&S37443[100], 9}, {NULL}};
const Segment SGI149654[] = {{&S37443[73], 6}, {NULL}};
const Segment SGI149655[] = {{&S37797[44], 18}, {NULL}};
const Segment SGI149656[] = {{&S37797[44], 30}, {NULL}};
const Segment SGI149657[] = {{&S37797[30], 6}, {NULL}};
const Segment SGI149658[] = {{&S37797[123], 3}, {NULL}};
const Segment SGI149659[] = {{&S37797[88], 6}, {NULL}};
const Segment SGI149660[] = {{&S37798[38], 18}, {NULL}};
const Segment SGI149661[] = {{&S37798[31], 6}, {NULL}};
const Segment SGI149662[] = {{&S37834[44], 6}, {NULL}};
const Segment SGI149663[] = {{&S37834[44], 18}, {NULL}};
const Segment SGI149664[] = {{&S37834[30], 6}, {NULL}};
const Segment SGI149665[] = {{&S37834[111], 3}, {NULL}};
const Segment SGI149666[] = {{&S37834[76], 6}, {NULL}};
const Segment SGI149667[] = {{&S37835[38], 6}, {NULL}};
const Segment SGI149668[] = {{&S37835[31], 6}, {NULL}};
const Segment SGI149669[] = {{&S37837[44], 6}, {NULL}};
const Segment SGI149670[] = {{&S37837[44], 20}, {NULL}};
const Segment SGI149671[] = {{&S37837[30], 6}, {NULL}};
const Segment SGI149672[] = {{&S37837[113], 3}, {NULL}};
const Segment SGI149673[] = {{&S37837[78], 6}, {NULL}};
const Segment SGI149674[] = {{&S37501[44], 11}, {NULL}};
const Segment SGI149675[] = {{&S37501[44], 23}, {NULL}};
const Segment SGI149676[] = {{&S37501[30], 6}, {NULL}};
const Segment SGI149677[] = {{&S37501[116], 3}, {NULL}};
const Segment SGI149678[] = {{&S37501[81], 6}, {NULL}};
const Segment SGI149679[] = {{&S37502[44], 11}, {NULL}};
const Segment SGI149680[] = {{&S37502[44], 23}, {NULL}};
const Segment SGI149681[] = {{&S37502[30], 6}, {NULL}};
const Segment SGI149682[] = {{&S37502[116], 3}, {NULL}};
const Segment SGI149683[] = {{&S37502[81], 6}, {NULL}};
const Segment SGI149684[] = {{&S37503[38], 12}, {NULL}};
const Segment SGI149685[] = {{&S37503[31], 6}, {NULL}};
const Segment SGI149686[] = {{&S37504[44], 22}, {NULL}};
const Segment SGI149687[] = {{&S37504[44], 34}, {NULL}};
const Segment SGI149688[] = {{&S37504[30], 6}, {NULL}};
const Segment SGI149689[] = {{&S37504[127], 3}, {NULL}};
const Segment SGI149690[] = {{&S37504[92], 6}, {NULL}};
const Segment SGI149691[] = {{&S37505[44], 11}, {NULL}};
const Segment SGI149692[] = {{&S37505[44], 23}, {NULL}};
const Segment SGI149693[] = {{&S37505[30], 6}, {NULL}};
const Segment SGI149694[] = {{&S37505[116], 3}, {NULL}};
const Segment SGI149695[] = {{&S37505[81], 6}, {NULL}};
const Segment SGI149696[] = {{&S37506[44], 22}, {NULL}};
const Segment SGI149697[] = {{&S37506[44], 34}, {NULL}};
const Segment SGI149698[] = {{&S37506[30], 6}, {NULL}};
const Segment SGI149699[] = {{&S37506[127], 3}, {NULL}};
const Segment SGI149700[] = {{&S37506[92], 6}, {NULL}};
const Segment SGI149701[] = {{&S37507[44], 22}, {NULL}};
const Segment SGI149702[] = {{&S37507[44], 34}, {NULL}};
const Segment SGI149703[] = {{&S37507[30], 6}, {NULL}};
const Segment SGI149704[] = {{&S37507[127], 3}, {NULL}};
const Segment SGI149705[] = {{&S37507[92], 6}, {NULL}};
const Segment SGI149706[] = {{&S37508[44], 24}, {NULL}};
const Segment SGI149707[] = {{&S37508[44], 36}, {NULL}};
const Segment SGI149708[] = {{&S37508[30], 6}, {NULL}};
const Segment SGI149709[] = {{&S37508[129], 3}, {NULL}};
const Segment SGI149710[] = {{&S37508[94], 6}, {NULL}};
const Segment SGI149711[] = {{&S37509[44], 22}, {NULL}};
const Segment SGI149712[] = {{&S37509[44], 34}, {NULL}};
const Segment SGI149713[] = {{&S37509[30], 6}, {NULL}};
const Segment SGI149714[] = {{&S37509[127], 3}, {NULL}};
const Segment SGI149715[] = {{&S37509[92], 6}, {NULL}};
const Segment SGI149716[] = {{&S37510[44], 12}, {NULL}};
const Segment SGI149717[] = {{&S37510[44], 24}, {NULL}};
const Segment SGI149718[] = {{&S37510[30], 6}, {NULL}};
const Segment SGI149719[] = {{&S37510[117], 3}, {NULL}};
const Segment SGI149720[] = {{&S37510[82], 6}, {NULL}};
const Segment SGI149721[] = {{&S37511[44], 22}, {NULL}};
const Segment SGI149722[] = {{&S37511[44], 34}, {NULL}};
const Segment SGI149723[] = {{&S37511[30], 6}, {NULL}};
const Segment SGI149724[] = {{&S37511[127], 3}, {NULL}};
const Segment SGI149725[] = {{&S37511[92], 6}, {NULL}};
const Segment SGI149726[] = {{&S37512[44], 22}, {NULL}};
const Segment SGI149727[] = {{&S37512[44], 34}, {NULL}};
const Segment SGI149728[] = {{&S37512[30], 6}, {NULL}};
const Segment SGI149729[] = {{&S37512[127], 3}, {NULL}};
const Segment SGI149730[] = {{&S37512[92], 6}, {NULL}};
const Segment SGI149731[] = {{&S37513[44], 31}, {NULL}};
const Segment SGI149732[] = {{&S37513[44], 43}, {NULL}};
const Segment SGI149733[] = {{&S37513[30], 6}, {NULL}};
const Segment SGI149734[] = {{&S37513[136], 3}, {NULL}};
const Segment SGI149735[] = {{&S37513[101], 6}, {NULL}};
const Segment SGI149736[] = {{&S37514[44], 22}, {NULL}};
const Segment SGI149737[] = {{&S37514[44], 34}, {NULL}};
const Segment SGI149738[] = {{&S37514[30], 6}, {NULL}};
const Segment SGI149739[] = {{&S37514[127], 3}, {NULL}};
const Segment SGI149740[] = {{&S37514[92], 6}, {NULL}};
const Segment SGI149741[] = {{&S37539[34], 9}, {NULL}};
const Segment SGI149742[] = {{&S37539[27], 6}, {NULL}};
const Segment SGI149743[] = {{&S37539[96], 9}, {NULL}};
const Segment SGI149744[] = {{&S37539[69], 6}, {NULL}};
const Segment SGI149745[] = {{&S37540[34], 8}, {NULL}};
const Segment SGI149746[] = {{&S37540[27], 6}, {NULL}};
const Segment SGI149747[] = {{&S37540[95], 9}, {NULL}};
const Segment SGI149748[] = {{&S37540[68], 6}, {NULL}};
const Segment SGI149749[] = {{&S37541[34], 11}, {NULL}};
const Segment SGI149750[] = {{&S37541[27], 6}, {NULL}};
const Segment SGI149751[] = {{&S37541[98], 9}, {NULL}};
const Segment SGI149752[] = {{&S37541[71], 6}, {NULL}};
const Segment SGI149753[] = {{&S37542[34], 14}, {NULL}};
const Segment SGI149754[] = {{&S37542[27], 6}, {NULL}};
const Segment SGI149755[] = {{&S37542[101], 9}, {NULL}};
const Segment SGI149756[] = {{&S37542[74], 6}, {NULL}};
const Segment SGI149757[] = {{&S47427[34], 14}, {NULL}};
const Segment SGI149758[] = {{&S47427[27], 6}, {NULL}};
const Segment SGI149759[] = {{&S37543[34], 18}, {NULL}};
const Segment SGI149760[] = {{&S47427[101], 9}, {NULL}};
const Segment SGI149761[] = {{&S37543[27], 6}, {NULL}};
const Segment SGI149762[] = {{&S47427[74], 6}, {NULL}};
const Segment SGI149763[] = {{&S37543[105], 9}, {NULL}};
const Segment SGI149764[] = {{&S37543[78], 6}, {NULL}};
const Segment SGI149765[] = {{&S47429[34], 13}, {NULL}};
const Segment SGI149766[] = {{&S47429[27], 6}, {NULL}};
const Segment SGI149767[] = {{&S47429[100], 9}, {NULL}};
const Segment SGI149768[] = {{&S47429[73], 6}, {NULL}};
const Segment SGI149769[] = {{&S47430[34], 6}, {NULL}};
const Segment SGI149770[] = {{&S47430[27], 6}, {NULL}};
const Segment SGI149771[] = {{&S47430[93], 9}, {NULL}};
const Segment SGI149772[] = {{&S47430[66], 6}, {NULL}};
const Segment SGI149773[] = {{&S47432[34], 6}, {NULL}};
const Segment SGI149774[] = {{&S47432[27], 6}, {NULL}};
const Segment SGI149775[] = {{&S47432[93], 9}, {NULL}};
const Segment SGI149776[] = {{&S47432[66], 6}, {NULL}};
const Segment SGI149777[] = {{&S32945[34], 14}, {NULL}};
const Segment SGI149778[] = {{&S32945[27], 6}, {NULL}};
const Segment SGI149779[] = {{&S32945[101], 9}, {NULL}};
const Segment SGI149780[] = {{&S32945[74], 6}, {NULL}};
const Segment SGI149781[] = {{&S32946[34], 14}, {NULL}};
const Segment SGI149782[] = {{&S32946[27], 6}, {NULL}};
const Segment SGI149783[] = {{&S32946[101], 9}, {NULL}};
const Segment SGI149784[] = {{&S32946[74], 6}, {NULL}};
const Segment SGI149785[] = {{&S47450[44], 11}, {NULL}};
const Segment SGI149786[] = {{&S47450[44], 31}, {NULL}};
const Segment SGI149787[] = {{&S47450[30], 6}, {NULL}};
const Segment SGI149788[] = {{&S47450[31], 3}, {NULL}};
const Segment SGI149789[] = {{&S47450[89], 6}, {NULL}};
const Segment SGI149790[] = {{&S47451[44], 11}, {NULL}};
const Segment SGI149791[] = {{&S47451[44], 70}, {NULL}};
const Segment SGI149792[] = {{&S47451[30], 6}, {NULL}};
const Segment SGI149793[] = {{&S47451[31], 3}, {NULL}};
const Segment SGI149794[] = {{&S47451[128], 6}, {NULL}};
const Segment SGI149795[] = {{&S47452[44], 11}, {NULL}};
const Segment SGI149796[] = {{&S47452[44], 45}, {NULL}};
const Segment SGI149797[] = {{&S47452[30], 6}, {NULL}};
const Segment SGI149798[] = {{&S47452[31], 3}, {NULL}};
const Segment SGI149799[] = {{&S47452[103], 6}, {NULL}};
const Segment SGI149800[] = {{&S47453[44], 11}, {NULL}};
const Segment SGI149801[] = {{&S47453[44], 30}, {NULL}};
const Segment SGI149802[] = {{&S47453[30], 6}, {NULL}};
const Segment SGI149803[] = {{&S47453[31], 3}, {NULL}};
const Segment SGI149804[] = {{&S47453[88], 6}, {NULL}};
const Segment SGI149805[] = {{&S47454[44], 11}, {NULL}};
const Segment SGI149806[] = {{&S47454[44], 30}, {NULL}};
const Segment SGI149807[] = {{&S47454[30], 6}, {NULL}};
const Segment SGI149808[] = {{&S47454[31], 3}, {NULL}};
const Segment SGI149809[] = {{&S47454[88], 6}, {NULL}};
const Segment SGI149810[] = {{&S47455[44], 11}, {NULL}};
const Segment SGI149811[] = {{&S47455[44], 30}, {NULL}};
const Segment SGI149812[] = {{&S47455[30], 6}, {NULL}};
const Segment SGI149813[] = {{&S47455[31], 3}, {NULL}};
const Segment SGI149814[] = {{&S47455[88], 6}, {NULL}};
const Segment SGI149815[] = {{&S47456[44], 11}, {NULL}};
const Segment SGI149816[] = {{&S47456[44], 30}, {NULL}};
const Segment SGI149817[] = {{&S47456[30], 6}, {NULL}};
const Segment SGI149818[] = {{&S47456[31], 3}, {NULL}};
const Segment SGI149819[] = {{&S47456[88], 6}, {NULL}};
const Segment SGI149820[] = {{&S47457[44], 11}, {NULL}};
const Segment SGI149821[] = {{&S47457[44], 30}, {NULL}};
const Segment SGI149822[] = {{&S47457[30], 6}, {NULL}};
const Segment SGI149823[] = {{&S47457[31], 3}, {NULL}};
const Segment SGI149824[] = {{&S47457[88], 6}, {NULL}};
const Segment SGI149825[] = {{&S47458[44], 11}, {NULL}};
const Segment SGI149826[] = {{&S47458[44], 35}, {NULL}};
const Segment SGI149827[] = {{&S47458[30], 6}, {NULL}};
const Segment SGI149828[] = {{&S47458[31], 3}, {NULL}};
const Segment SGI149829[] = {{&S47458[93], 6}, {NULL}};
const Segment SGI149830[] = {{&S47459[44], 11}, {NULL}};
const Segment SGI149831[] = {{&S47459[44], 36}, {NULL}};
const Segment SGI149832[] = {{&S47459[30], 6}, {NULL}};
const Segment SGI149833[] = {{&S47459[31], 3}, {NULL}};
const Segment SGI149834[] = {{&S47459[94], 6}, {NULL}};
const Segment SGI149835[] = {{&S47460[44], 11}, {NULL}};
const Segment SGI149836[] = {{&S47460[44], 36}, {NULL}};
const Segment SGI149837[] = {{&S47460[30], 6}, {NULL}};
const Segment SGI149838[] = {{&S47460[31], 3}, {NULL}};
const Segment SGI149839[] = {{&S47460[94], 6}, {NULL}};
const Segment SGI149840[] = {{&S47461[38], 11}, {NULL}};
const Segment SGI149841[] = {{&S47461[31], 6}, {NULL}};
const Segment SGI149842[] = {{&S47464[34], 14}, {NULL}};
const Segment SGI149843[] = {{&S47464[27], 6}, {NULL}};
const Segment SGI149844[] = {{&S47464[101], 9}, {NULL}};
const Segment SGI149845[] = {{&S47464[74], 6}, {NULL}};
const Segment SGI149846[] = {{&S36040[32], 9}, {NULL}};
const Segment SGI149847[] = {{&S36040[27], 4}, {NULL}};
const Segment SGI149848[] = {{&S36040[94], 9}, {NULL}};
const Segment SGI149849[] = {{&S36040[67], 6}, {NULL}};
const Segment SGI149850[] = {{&S47520[34], 14}, {NULL}};
const Segment SGI149851[] = {{&S47520[27], 6}, {NULL}};
const Segment SGI149852[] = {{&S47520[101], 9}, {NULL}};
const Segment SGI149853[] = {{&S47520[74], 6}, {NULL}};
const Segment SGI149854[] = {{&S47558[44], 11}, {NULL}};
const Segment SGI149855[] = {{&S47558[44], 23}, {NULL}};
const Segment SGI149856[] = {{&S47558[30], 6}, {NULL}};
const Segment SGI149857[] = {{&S47558[31], 3}, {NULL}};
const Segment SGI149858[] = {{&S47558[81], 6}, {NULL}};
const Segment SGI149859[] = {{&S47559[38], 11}, {NULL}};
const Segment SGI149860[] = {{&S47559[31], 6}, {NULL}};
const Segment SGI149861[] = {{&S47560[42], 11}, {NULL}};
const Segment SGI149862[] = {{&S47560[30], 6}, {NULL}};
const Segment SGI149863[] = {{&S47560[31], 3}, {NULL}};
const Segment SGI149864[] = {{&S47560[79], 6}, {NULL}};
const Segment SGI149865[] = {{&S37546[44], 8}, {NULL}};
const Segment SGI149866[] = {{&S37546[44], 20}, {NULL}};
const Segment SGI149867[] = {{&S37546[30], 6}, {NULL}};
const Segment SGI149868[] = {{&S37546[31], 3}, {NULL}};
const Segment SGI149869[] = {{&S37546[78], 6}, {NULL}};
const Segment SGI149870[] = {{&S37547[38], 8}, {NULL}};
const Segment SGI149871[] = {{&S37547[31], 6}, {NULL}};
const Segment SGI149872[] = {{&S37548[44], 8}, {NULL}};
const Segment SGI149873[] = {{&S37548[44], 20}, {NULL}};
const Segment SGI149874[] = {{&S37548[30], 6}, {NULL}};
const Segment SGI149875[] = {{&S37548[113], 3}, {NULL}};
const Segment SGI149876[] = {{&S37548[78], 6}, {NULL}};
const Segment SGI149877[] = {{&S37614[44], 9}, {NULL}};
const Segment SGI149878[] = {{&S37614[44], 21}, {NULL}};
const Segment SGI149879[] = {{&S37614[30], 6}, {NULL}};
const Segment SGI149880[] = {{&S37614[114], 3}, {NULL}};
const Segment SGI149881[] = {{&S37614[79], 6}, {NULL}};
const Segment SGI149882[] = {{&S37615[44], 10}, {NULL}};
const Segment SGI149883[] = {{&S37615[44], 22}, {NULL}};
const Segment SGI149884[] = {{&S37615[30], 6}, {NULL}};
const Segment SGI149885[] = {{&S37615[115], 3}, {NULL}};
const Segment SGI149886[] = {{&S37615[80], 6}, {NULL}};
const Segment SGI149887[] = {{&S37616[38], 10}, {NULL}};
const Segment SGI149888[] = {{&S37616[31], 6}, {NULL}};
const Segment SGI149889[] = {{&S37617[38], 9}, {NULL}};
const Segment SGI149890[] = {{&S37617[31], 6}, {NULL}};
const Segment SGI149891[] = {{&S37641[44], 10}, {NULL}};
const Segment SGI149892[] = {{&S37641[44], 22}, {NULL}};
const Segment SGI149893[] = {{&S37641[30], 6}, {NULL}};
const Segment SGI149894[] = {{&S37641[115], 3}, {NULL}};
const Segment SGI149895[] = {{&S37641[80], 6}, {NULL}};
const Segment SGI149896[] = {{&S37642[38], 10}, {NULL}};
const Segment SGI149897[] = {{&S37642[31], 6}, {NULL}};
const Segment SGI149898[] = {{&S37643[42], 10}, {NULL}};
const Segment SGI149899[] = {{&S37643[30], 6}, {NULL}};
const Segment SGI149900[] = {{&S37643[113], 3}, {NULL}};
const Segment SGI149901[] = {{&S37643[78], 6}, {NULL}};
const Segment SGI149902[] = {{&S38178[34], 7}, {NULL}};
const Segment SGI149903[] = {{&S38178[27], 6}, {NULL}};
const Segment SGI149904[] = {{&S38178[99], 9}, {NULL}};
const Segment SGI149905[] = {{&S38178[72], 6}, {NULL}};
const Segment SGI149906[] = {{&S38213[44], 7}, {NULL}};
const Segment SGI149907[] = {{&S38213[44], 19}, {NULL}};
const Segment SGI149908[] = {{&S38213[30], 6}, {NULL}};
const Segment SGI149909[] = {{&S38213[112], 3}, {NULL}};
const Segment SGI149910[] = {{&S38213[77], 6}, {NULL}};
const Segment SGI149911[] = {{&S47601[34], 10}, {NULL}};
const Segment SGI149912[] = {{&S47601[27], 6}, {NULL}};
const Segment SGI149913[] = {{&S47601[97], 9}, {NULL}};
const Segment SGI149914[] = {{&S47601[70], 6}, {NULL}};
const Segment SGI149915[] = {{&S47620[44], 10}, {NULL}};
const Segment SGI149916[] = {{&S47620[44], 22}, {NULL}};
const Segment SGI149917[] = {{&S47620[30], 6}, {NULL}};
const Segment SGI149918[] = {{&S37720[34], 24}, {NULL}};
const Segment SGI149919[] = {{&S37720[27], 6}, {NULL}};
const Segment SGI149920[] = {{&S47620[31], 3}, {NULL}};
const Segment SGI149921[] = {{&S47620[80], 6}, {NULL}};
const Segment SGI149922[] = {{&S37720[111], 9}, {NULL}};
const Segment SGI149923[] = {{&S37720[84], 6}, {NULL}};
const Segment SGI149924[] = {{&S47621[44], 9}, {NULL}};
const Segment SGI149925[] = {{&S47621[44], 21}, {NULL}};
const Segment SGI149926[] = {{&S47621[30], 6}, {NULL}};
const Segment SGI149927[] = {{&S47621[31], 3}, {NULL}};
const Segment SGI149928[] = {{&S47621[79], 6}, {NULL}};
const Segment SGI149929[] = {{&S47622[44], 11}, {NULL}};
const Segment SGI149930[] = {{&S47622[44], 23}, {NULL}};
const Segment SGI149931[] = {{&S47622[30], 6}, {NULL}};
const Segment SGI149932[] = {{&S47622[31], 3}, {NULL}};
const Segment SGI149933[] = {{&S47622[81], 6}, {NULL}};
const Segment SGI149934[] = {{&S47623[44], 14}, {NULL}};
const Segment SGI149935[] = {{&S47623[44], 26}, {NULL}};
const Segment SGI149936[] = {{&S47623[30], 6}, {NULL}};
const Segment SGI149937[] = {{&S47623[31], 3}, {NULL}};
const Segment SGI149938[] = {{&S47623[84], 6}, {NULL}};
const Segment SGI149939[] = {{&S47624[44], 16}, {NULL}};
const Segment SGI149940[] = {{&S47624[44], 26}, {NULL}};
const Segment SGI149941[] = {{&S47624[30], 6}, {NULL}};
const Segment SGI149942[] = {{&S47624[31], 3}, {NULL}};
const Segment SGI149943[] = {{&S47624[84], 6}, {NULL}};
const Segment SGI149944[] = {{&S47625[38], 9}, {NULL}};
const Segment SGI149945[] = {{&S47625[31], 6}, {NULL}};
const Segment SGI149946[] = {{&S47626[38], 11}, {NULL}};
const Segment SGI149947[] = {{&S47626[31], 6}, {NULL}};
const Segment SGI149948[] = {{&S47627[44], 15}, {NULL}};
const Segment SGI149949[] = {{&S47627[44], 27}, {NULL}};
const Segment SGI149950[] = {{&S47627[30], 6}, {NULL}};
const Segment SGI149951[] = {{&S47627[31], 3}, {NULL}};
const Segment SGI149952[] = {{&S47627[85], 6}, {NULL}};
const Segment SGI149953[] = {{&S37799[44], 11}, {NULL}};
const Segment SGI149954[] = {{&S37799[44], 23}, {NULL}};
const Segment SGI149955[] = {{&S37799[30], 6}, {NULL}};
const Segment SGI149956[] = {{&S37799[116], 3}, {NULL}};
const Segment SGI149957[] = {{&S36372[32], 9}, {NULL}};
const Segment SGI149958[] = {{&S37799[81], 6}, {NULL}};
const Segment SGI149959[] = {{&S36372[27], 4}, {NULL}};
const Segment SGI149960[] = {{&S36372[94], 9}, {NULL}};
const Segment SGI149961[] = {{&S36372[67], 6}, {NULL}};
const Segment SGI149962[] = {{&S37800[38], 11}, {NULL}};
const Segment SGI149963[] = {{&S37800[31], 6}, {NULL}};
const Segment SGI149964[] = {{&S37801[44], 18}, {NULL}};
const Segment SGI149965[] = {{&S37801[44], 30}, {NULL}};
const Segment SGI149966[] = {{&S37801[30], 6}, {NULL}};
const Segment SGI149967[] = {{&S37801[123], 3}, {NULL}};
const Segment SGI149968[] = {{&S37801[88], 6}, {NULL}};
const Segment SGI149969[] = {{&S47695[34], 10}, {NULL}};
const Segment SGI149970[] = {{&S47695[27], 6}, {NULL}};
const Segment SGI149971[] = {{&S37802[38], 18}, {NULL}};
const Segment SGI149972[] = {{&S37802[31], 6}, {NULL}};
const Segment SGI149973[] = {{&S47695[103], 9}, {NULL}};
const Segment SGI149974[] = {{&S47695[76], 6}, {NULL}};
const Segment SGI149975[] = {{&S47698[34], 10}, {NULL}};
const Segment SGI149976[] = {{&S47698[27], 6}, {NULL}};
const Segment SGI149977[] = {{&S47698[103], 9}, {NULL}};
const Segment SGI149978[] = {{&S47698[76], 6}, {NULL}};
const Segment SGI149979[] = {{&S47699[34], 10}, {NULL}};
const Segment SGI149980[] = {{&S47699[27], 6}, {NULL}};
const Segment SGI149981[] = {{&S47699[103], 9}, {NULL}};
const Segment SGI149982[] = {{&S47699[76], 6}, {NULL}};
const Segment SGI149983[] = {{&S47724[34], 15}, {NULL}};
const Segment SGI149984[] = {{&S47724[27], 6}, {NULL}};
const Segment SGI149985[] = {{&S47724[102], 9}, {NULL}};
const Segment SGI149986[] = {{&S47724[75], 6}, {NULL}};
const Segment SGI149987[] = {{&S47732[34], 10}, {NULL}};
const Segment SGI149988[] = {{&S47732[27], 6}, {NULL}};
const Segment SGI149989[] = {{&S47732[97], 9}, {NULL}};
const Segment SGI149990[] = {{&S47732[70], 6}, {NULL}};
const Segment SGI149991[] = {{&S47733[34], 10}, {NULL}};
const Segment SGI149992[] = {{&S47733[27], 6}, {NULL}};
const Segment SGI149993[] = {{&S47733[97], 9}, {NULL}};
const Segment SGI149994[] = {{&S47733[70], 6}, {NULL}};
const Segment SGI149995[] = {{&S47734[34], 8}, {NULL}};
const Segment SGI149996[] = {{&S47734[27], 6}, {NULL}};
const Segment SGI149997[] = {{&S47734[95], 9}, {NULL}};
const Segment SGI149998[] = {{&S47734[68], 6}, {NULL}};
const Segment SGI149999[] = {{&S47735[34], 10}, {NULL}};
const Segment SGI150000[] = {{&S47735[27], 6}, {NULL}};
const Segment SGI150001[] = {{&S47735[97], 9}, {NULL}};
const Segment SGI150002[] = {{&S47735[70], 6}, {NULL}};
const Segment SGI150003[] = {{&S47736[34], 11}, {NULL}};
const Segment SGI150004[] = {{&S47736[27], 6}, {NULL}};
const Segment SGI150005[] = {{&S47736[98], 9}, {NULL}};
const Segment SGI150006[] = {{&S47736[71], 6}, {NULL}};
const Segment SGI150007[] = {{&S47737[34], 11}, {NULL}};
const Segment SGI150008[] = {{&S47737[27], 6}, {NULL}};
const Segment SGI150009[] = {{&S47737[98], 9}, {NULL}};
const Segment SGI150010[] = {{&S47737[71], 6}, {NULL}};
const Segment SGI150011[] = {{&S47738[34], 7}, {NULL}};
const Segment SGI150012[] = {{&S47738[27], 6}, {NULL}};
const Segment SGI150013[] = {{&S47738[94], 9}, {NULL}};
const Segment SGI150014[] = {{&S47738[67], 6}, {NULL}};
const Segment SGI150015[] = {{&S47739[34], 10}, {NULL}};
const Segment SGI150016[] = {{&S47739[27], 6}, {NULL}};
const Segment SGI150017[] = {{&S38382[44], 9}, {NULL}};
const Segment SGI150018[] = {{&S47739[97], 9}, {NULL}};
const Segment SGI150019[] = {{&S38382[44], 21}, {NULL}};
const Segment SGI150020[] = {{&S47739[70], 6}, {NULL}};
const Segment SGI150021[] = {{&S38382[30], 6}, {NULL}};
const Segment SGI150022[] = {{&S38382[114], 3}, {NULL}};
const Segment SGI150023[] = {{&S38382[79], 6}, {NULL}};
const Segment SGI150024[] = {{&S47740[34], 10}, {NULL}};
const Segment SGI150025[] = {{&S47740[27], 6}, {NULL}};
const Segment SGI150026[] = {{&S38383[44], 7}, {NULL}};
const Segment SGI150027[] = {{&S47740[97], 9}, {NULL}};
const Segment SGI150028[] = {{&S38383[44], 19}, {NULL}};
const Segment SGI150029[] = {{&S47740[70], 6}, {NULL}};
const Segment SGI150030[] = {{&S38383[30], 6}, {NULL}};
const Segment SGI150031[] = {{&S38383[112], 3}, {NULL}};
const Segment SGI150032[] = {{&S38383[77], 6}, {NULL}};
const Segment SGI150033[] = {{&S38384[38], 9}, {NULL}};
const Segment SGI150034[] = {{&S38384[31], 6}, {NULL}};
const Segment SGI150035[] = {{&S47743[44], 12}, {NULL}};
const Segment SGI150036[] = {{&S38385[38], 7}, {NULL}};
const Segment SGI150037[] = {{&S38385[31], 6}, {NULL}};
const Segment SGI150038[] = {{&S47743[44], 29}, {NULL}};
const Segment SGI150039[] = {{&S47743[30], 6}, {NULL}};
const Segment SGI150040[] = {{&S47743[31], 3}, {NULL}};
const Segment SGI150041[] = {{&S47743[87], 6}, {NULL}};
const Segment SGI150042[] = {{&S47744[44], 12}, {NULL}};
const Segment SGI150043[] = {{&S47744[44], 29}, {NULL}};
const Segment SGI150044[] = {{&S47744[30], 6}, {NULL}};
const Segment SGI150045[] = {{&S47744[31], 3}, {NULL}};
const Segment SGI150046[] = {{&S47744[87], 6}, {NULL}};
const Segment SGI150047[] = {{&S47745[44], 12}, {NULL}};
const Segment SGI150048[] = {{&S47745[44], 37}, {NULL}};
const Segment SGI150049[] = {{&S47745[30], 6}, {NULL}};
const Segment SGI150050[] = {{&S47745[31], 3}, {NULL}};
const Segment SGI150051[] = {{&S47745[95], 6}, {NULL}};
const Segment SGI150052[] = {{&S47746[44], 12}, {NULL}};
const Segment SGI150053[] = {{&S47746[44], 28}, {NULL}};
const Segment SGI150054[] = {{&S47746[30], 6}, {NULL}};
const Segment SGI150055[] = {{&S47746[31], 3}, {NULL}};
const Segment SGI150056[] = {{&S47746[86], 6}, {NULL}};
const Segment SGI150057[] = {{&S47747[44], 12}, {NULL}};
const Segment SGI150058[] = {{&S47747[44], 29}, {NULL}};
const Segment SGI150059[] = {{&S47747[30], 6}, {NULL}};
const Segment SGI150060[] = {{&S47747[31], 3}, {NULL}};
const Segment SGI150061[] = {{&S47747[87], 6}, {NULL}};
const Segment SGI150062[] = {{&S47748[44], 8}, {NULL}};
const Segment SGI150063[] = {{&S47748[44], 25}, {NULL}};
const Segment SGI150064[] = {{&S47748[30], 6}, {NULL}};
const Segment SGI150065[] = {{&S47748[31], 3}, {NULL}};
const Segment SGI150066[] = {{&S47748[83], 6}, {NULL}};
const Segment SGI150067[] = {{&S47749[38], 8}, {NULL}};
const Segment SGI150068[] = {{&S47749[31], 6}, {NULL}};
const Segment SGI150069[] = {{&S47783[44], 8}, {NULL}};
const Segment SGI150070[] = {{&S47783[44], 31}, {NULL}};
const Segment SGI150071[] = {{&S47783[30], 6}, {NULL}};
const Segment SGI150072[] = {{&S47783[31], 3}, {NULL}};
const Segment SGI150073[] = {{&S47783[89], 6}, {NULL}};
const Segment SGI150074[] = {{&S47784[44], 8}, {NULL}};
const Segment SGI150075[] = {{&S47784[44], 32}, {NULL}};
const Segment SGI150076[] = {{&S47784[30], 6}, {NULL}};
const Segment SGI150077[] = {{&S47784[31], 3}, {NULL}};
const Segment SGI150078[] = {{&S47784[90], 6}, {NULL}};
const Segment SGI150079[] = {{&S47785[44], 8}, {NULL}};
const Segment SGI150080[] = {{&S47785[44], 23}, {NULL}};
const Segment SGI150081[] = {{&S47785[30], 6}, {NULL}};
const Segment SGI150082[] = {{&S47785[31], 3}, {NULL}};
const Segment SGI150083[] = {{&S47785[81], 6}, {NULL}};
const Segment SGI150084[] = {{&S47786[38], 8}, {NULL}};
const Segment SGI150085[] = {{&S47786[31], 6}, {NULL}};
const Segment SGI150086[] = {{&S47787[44], 8}, {NULL}};
const Segment SGI150087[] = {{&S47787[44], 31}, {NULL}};
const Segment SGI150088[] = {{&S47787[30], 6}, {NULL}};
const Segment SGI150089[] = {{&S47787[31], 3}, {NULL}};
const Segment SGI150090[] = {{&S47787[89], 6}, {NULL}};
const Segment SGI150091[] = {{&S33506[34], 14}, {NULL}};
const Segment SGI150092[] = {{&S33506[27], 6}, {NULL}};
const Segment SGI150093[] = {{&S33506[101], 9}, {NULL}};
const Segment SGI150094[] = {{&S33506[74], 6}, {NULL}};
const Segment SGI150095[] = {{&S33508[34], 14}, {NULL}};
const Segment SGI150096[] = {{&S33508[27], 6}, {NULL}};
const Segment SGI150097[] = {{&S33508[101], 9}, {NULL}};
const Segment SGI150098[] = {{&S33508[74], 6}, {NULL}};
const Segment SGI150099[] = {{&S33511[34], 14}, {NULL}};
const Segment SGI150100[] = {{&S33511[27], 6}, {NULL}};
const Segment SGI150101[] = {{&S33511[101], 9}, {NULL}};
const Segment SGI150102[] = {{&S33511[74], 6}, {NULL}};
const Segment SGI150103[] = {{&S38473[34], 11}, {NULL}};
const Segment SGI150104[] = {{&S38473[27], 6}, {NULL}};
const Segment SGI150105[] = {{&S38473[98], 9}, {NULL}};
const Segment SGI150106[] = {{&S38473[71], 6}, {NULL}};
const Segment SGI150107[] = {{&S38474[34], 7}, {NULL}};
const Segment SGI150108[] = {{&S38474[27], 6}, {NULL}};
const Segment SGI150109[] = {{&S38474[94], 9}, {NULL}};
const Segment SGI150110[] = {{&S38474[67], 6}, {NULL}};
const Segment SGI150111[] = {{&S47820[34], 11}, {NULL}};
const Segment SGI150112[] = {{&S47820[27], 6}, {NULL}};
const Segment SGI150113[] = {{&S47820[98], 9}, {NULL}};
const Segment SGI150114[] = {{&S47820[71], 6}, {NULL}};
const Segment SGI150115[] = {{&S47821[34], 10}, {NULL}};
const Segment SGI150116[] = {{&S47821[27], 6}, {NULL}};
const Segment SGI150117[] = {{&S38477[34], 11}, {NULL}};
const Segment SGI150118[] = {{&S38477[27], 6}, {NULL}};
const Segment SGI150119[] = {{&S47821[97], 9}, {NULL}};
const Segment SGI150120[] = {{&S47821[70], 6}, {NULL}};
const Segment SGI150121[] = {{&S38477[98], 9}, {NULL}};
const Segment SGI150122[] = {{&S38477[71], 6}, {NULL}};
const Segment SGI150123[] = {{&S47824[34], 18}, {NULL}};
const Segment SGI150124[] = {{&S47824[27], 6}, {NULL}};
const Segment SGI150125[] = {{&S47824[105], 9}, {NULL}};
const Segment SGI150126[] = {{&S47824[78], 6}, {NULL}};
const Segment SGI150127[] = {{&S38482[34], 12}, {NULL}};
const Segment SGI150128[] = {{&S38482[27], 6}, {NULL}};
const Segment SGI150129[] = {{&S38482[99], 9}, {NULL}};
const Segment SGI150130[] = {{&S38482[72], 6}, {NULL}};
const Segment SGI150131[] = {{&S47846[34], 13}, {NULL}};
const Segment SGI150132[] = {{&S47846[27], 6}, {NULL}};
const Segment SGI150133[] = {{&S47846[100], 9}, {NULL}};
const Segment SGI150134[] = {{&S47846[73], 6}, {NULL}};
const Segment SGI150135[] = {{&S36561[32], 14}, {NULL}};
const Segment SGI150136[] = {{&S36561[27], 4}, {NULL}};
const Segment SGI150137[] = {{&S36561[99], 9}, {NULL}};
const Segment SGI150138[] = {{&S36561[72], 6}, {NULL}};
const Segment SGI150139[] = {{&S47888[34], 7}, {NULL}};
const Segment SGI150140[] = {{&S47888[27], 6}, {NULL}};
const Segment SGI150141[] = {{&S47888[94], 9}, {NULL}};
const Segment SGI150142[] = {{&S47888[67], 6}, {NULL}};
const Segment SGI150143[] = {{&S47890[34], 6}, {NULL}};
const Segment SGI150144[] = {{&S47890[27], 6}, {NULL}};
const Segment SGI150145[] = {{&S47890[93], 9}, {NULL}};
const Segment SGI150146[] = {{&S47890[66], 6}, {NULL}};
const Segment SGI150147[] = {{&S47892[34], 7}, {NULL}};
const Segment SGI150148[] = {{&S47892[27], 6}, {NULL}};
const Segment SGI150149[] = {{&S47892[98], 9}, {NULL}};
const Segment SGI150150[] = {{&S47892[71], 6}, {NULL}};
const Segment SGI150151[] = {{&S47908[44], 10}, {NULL}};
const Segment SGI150152[] = {{&S47908[44], 22}, {NULL}};
const Segment SGI150153[] = {{&S47908[30], 6}, {NULL}};
const Segment SGI150154[] = {{&S47908[31], 3}, {NULL}};
const Segment SGI150155[] = {{&S47908[80], 6}, {NULL}};
const Segment SGI150156[] = {{&S47909[38], 10}, {NULL}};
const Segment SGI150157[] = {{&S47909[31], 6}, {NULL}};
const Segment SGI150158[] = {{&S33699[34], 9}, {NULL}};
const Segment SGI150159[] = {{&S33699[27], 6}, {NULL}};
const Segment SGI150160[] = {{&S33699[96], 9}, {NULL}};
const Segment SGI150161[] = {{&S33699[69], 6}, {NULL}};
const Segment SGI150162[] = {{&S47974[44], 10}, {NULL}};
const Segment SGI150163[] = {{&S47974[44], 22}, {NULL}};
const Segment SGI150164[] = {{&S47974[30], 6}, {NULL}};
const Segment SGI150165[] = {{&S47974[31], 3}, {NULL}};
const Segment SGI150166[] = {{&S47974[80], 6}, {NULL}};
const Segment SGI150167[] = {{&S47975[44], 10}, {NULL}};
const Segment SGI150168[] = {{&S47975[44], 22}, {NULL}};
const Segment SGI150169[] = {{&S47975[30], 6}, {NULL}};
const Segment SGI150170[] = {{&S47975[31], 3}, {NULL}};
const Segment SGI150171[] = {{&S47975[80], 6}, {NULL}};
const Segment SGI150172[] = {{&S47976[44], 10}, {NULL}};
const Segment SGI150173[] = {{&S47976[44], 22}, {NULL}};
const Segment SGI150174[] = {{&S47976[30], 6}, {NULL}};
const Segment SGI150175[] = {{&S47976[31], 3}, {NULL}};
const Segment SGI150176[] = {{&S47976[80], 6}, {NULL}};
const Segment SGI150177[] = {{&S47977[38], 10}, {NULL}};
const Segment SGI150178[] = {{&S47977[31], 6}, {NULL}};
const Segment SGI150179[] = {{&S47978[38], 10}, {NULL}};
const Segment SGI150180[] = {{&S42998[32], 6}, {NULL}};
const Segment SGI150181[] = {{&S47978[31], 6}, {NULL}};
const Segment SGI150182[] = {{&S42998[27], 4}, {NULL}};
const Segment SGI150183[] = {{&S42998[91], 9}, {NULL}};
const Segment SGI150184[] = {{&S42998[64], 6}, {NULL}};
const Segment SGI150185[] = {{&S38652[34], 10}, {NULL}};
const Segment SGI150186[] = {{&S38652[27], 6}, {NULL}};
const Segment SGI150187[] = {{&S38652[97], 9}, {NULL}};
const Segment SGI150188[] = {{&S38652[70], 6}, {NULL}};
const Segment SGI150189[] = {{&S38654[46], 10}, {NULL}};
const Segment SGI150190[] = {{&S38654[27], 18}, {NULL}};
const Segment SGI150191[] = {{&S38654[109], 9}, {NULL}};
const Segment SGI150192[] = {{&S38654[82], 6}, {NULL}};
const Segment SGI150193[] = {{&S48000[44], 8}, {NULL}};
const Segment SGI150194[] = {{&S48000[44], 32}, {NULL}};
const Segment SGI150195[] = {{&S48000[30], 6}, {NULL}};
const Segment SGI150196[] = {{&S48000[31], 3}, {NULL}};
const Segment SGI150197[] = {{&S48000[90], 6}, {NULL}};
const Segment SGI150198[] = {{&S48001[44], 8}, {NULL}};
const Segment SGI150199[] = {{&S48001[44], 32}, {NULL}};
const Segment SGI150200[] = {{&S48001[30], 6}, {NULL}};
const Segment SGI150201[] = {{&S48001[31], 3}, {NULL}};
const Segment SGI150202[] = {{&S48001[90], 6}, {NULL}};
const Segment SGI150203[] = {{&S48002[44], 7}, {NULL}};
const Segment SGI150204[] = {{&S48002[44], 37}, {NULL}};
const Segment SGI150205[] = {{&S48002[30], 6}, {NULL}};
const Segment SGI150206[] = {{&S48002[31], 3}, {NULL}};
const Segment SGI150207[] = {{&S48002[95], 6}, {NULL}};
const Segment SGI150208[] = {{&S48003[38], 8}, {NULL}};
const Segment SGI150209[] = {{&S48003[31], 6}, {NULL}};
const Segment SGI150210[] = {{&S38176[34], 7}, {NULL}};
const Segment SGI150211[] = {{&S38176[27], 6}, {NULL}};
const Segment SGI150212[] = {{&S38176[97], 9}, {NULL}};
const Segment SGI150213[] = {{&S38176[70], 6}, {NULL}};
const Segment SGI150214[] = {{&S38177[34], 7}, {NULL}};
const Segment SGI150215[] = {{&S38177[27], 6}, {NULL}};
const Segment SGI150216[] = {{&S38177[103], 9}, {NULL}};
const Segment SGI150217[] = {{&S38177[76], 6}, {NULL}};
const Segment SGI150218[] = {{&S48032[34], 13}, {NULL}};
const Segment SGI150219[] = {{&S48032[27], 6}, {NULL}};
const Segment SGI150220[] = {{&S48032[100], 9}, {NULL}};
const Segment SGI150221[] = {{&S48032[73], 6}, {NULL}};
const Segment SGI150222[] = {{&S38214[44], 7}, {NULL}};
const Segment SGI150223[] = {{&S38214[44], 19}, {NULL}};
const Segment SGI150224[] = {{&S38214[30], 6}, {NULL}};
const Segment SGI150225[] = {{&S38214[112], 3}, {NULL}};
const Segment SGI150226[] = {{&S38214[77], 6}, {NULL}};
const Segment SGI150227[] = {{&S38215[44], 14}, {NULL}};
const Segment SGI150228[] = {{&S38215[44], 26}, {NULL}};
const Segment SGI150229[] = {{&S38215[30], 6}, {NULL}};
const Segment SGI150230[] = {{&S38215[119], 3}, {NULL}};
const Segment SGI150231[] = {{&S38215[84], 6}, {NULL}};
const Segment SGI150232[] = {{&S43084[32], 9}, {NULL}};
const Segment SGI150233[] = {{&S38216[44], 14}, {NULL}};
const Segment SGI150234[] = {{&S43084[27], 4}, {NULL}};
const Segment SGI150235[] = {{&S38216[44], 36}, {NULL}};
const Segment SGI150236[] = {{&S38216[30], 6}, {NULL}};
const Segment SGI150237[] = {{&S43084[94], 9}, {NULL}};
const Segment SGI150238[] = {{&S43084[67], 6}, {NULL}};
const Segment SGI150239[] = {{&S38216[129], 3}, {NULL}};
const Segment SGI150240[] = {{&S38216[94], 6}, {NULL}};
const Segment SGI150241[] = {{&S38217[38], 14}, {NULL}};
const Segment SGI150242[] = {{&S38217[31], 6}, {NULL}};
const Segment SGI150243[] = {{&S38218[44], 14}, {NULL}};
const Segment SGI150244[] = {{&S38218[44], 36}, {NULL}};
const Segment SGI150245[] = {{&S38218[30], 6}, {NULL}};
const Segment SGI150246[] = {{&S38218[129], 3}, {NULL}};
const Segment SGI150247[] = {{&S38218[94], 6}, {NULL}};
const Segment SGI150248[] = {{&S38219[42], 14}, {NULL}};
const Segment SGI150249[] = {{&S38219[30], 6}, {NULL}};
const Segment SGI150250[] = {{&S38219[127], 3}, {NULL}};
const Segment SGI150251[] = {{&S38219[92], 6}, {NULL}};
const Segment SGI150252[] = {{&S48060[44], 10}, {NULL}};
const Segment SGI150253[] = {{&S48060[44], 22}, {NULL}};
const Segment SGI150254[] = {{&S48060[30], 6}, {NULL}};
const Segment SGI150255[] = {{&S48060[31], 3}, {NULL}};
const Segment SGI150256[] = {{&S48060[80], 6}, {NULL}};
const Segment SGI150257[] = {{&S38220[44], 14}, {NULL}};
const Segment SGI150258[] = {{&S38220[44], 29}, {NULL}};
const Segment SGI150259[] = {{&S38220[30], 6}, {NULL}};
const Segment SGI150260[] = {{&S38220[122], 3}, {NULL}};
const Segment SGI150261[] = {{&S38220[87], 6}, {NULL}};
const Segment SGI150262[] = {{&S48061[38], 10}, {NULL}};
const Segment SGI150263[] = {{&S48061[31], 6}, {NULL}};
const Segment SGI150264[] = {{&S48079[44], 11}, {NULL}};
const Segment SGI150265[] = {{&S48079[44], 23}, {NULL}};
const Segment SGI150266[] = {{&S48079[30], 6}, {NULL}};
const Segment SGI150267[] = {{&S48079[31], 3}, {NULL}};
const Segment SGI150268[] = {{&S48079[81], 6}, {NULL}};
const Segment SGI150269[] = {{&S48080[38], 11}, {NULL}};
const Segment SGI150270[] = {{&S48080[31], 6}, {NULL}};
const Segment SGI150271[] = {{&S48081[42], 11}, {NULL}};
const Segment SGI150272[] = {{&S48081[30], 6}, {NULL}};
const Segment SGI150273[] = {{&S48081[31], 3}, {NULL}};
const Segment SGI150274[] = {{&S48081[79], 6}, {NULL}};
const Segment SGI150275[] = {{&S38245[44], 11}, {NULL}};
const Segment SGI150276[] = {{&S38245[44], 28}, {NULL}};
const Segment SGI150277[] = {{&S38245[30], 6}, {NULL}};
const Segment SGI150278[] = {{&S38245[121], 3}, {NULL}};
const Segment SGI150279[] = {{&S38245[86], 6}, {NULL}};
const Segment SGI150280[] = {{&S38246[44], 7}, {NULL}};
const Segment SGI150281[] = {{&S38246[44], 24}, {NULL}};
const Segment SGI150282[] = {{&S38246[30], 6}, {NULL}};
const Segment SGI150283[] = {{&S38246[117], 3}, {NULL}};
const Segment SGI150284[] = {{&S38246[82], 6}, {NULL}};
const Segment SGI150285[] = {{&S38247[44], 11}, {NULL}};
const Segment SGI150286[] = {{&S38247[44], 28}, {NULL}};
const Segment SGI150287[] = {{&S38247[30], 6}, {NULL}};
const Segment SGI150288[] = {{&S38247[121], 3}, {NULL}};
const Segment SGI150289[] = {{&S38247[86], 6}, {NULL}};
const Segment SGI150290[] = {{&S38248[38], 7}, {NULL}};
const Segment SGI150291[] = {{&S38248[31], 6}, {NULL}};
const Segment SGI150292[] = {{&S38249[44], 11}, {NULL}};
const Segment SGI150293[] = {{&S38249[44], 28}, {NULL}};
const Segment SGI150294[] = {{&S38249[30], 6}, {NULL}};
const Segment SGI150295[] = {{&S38249[121], 3}, {NULL}};
const Segment SGI150296[] = {{&S38249[86], 6}, {NULL}};
const Segment SGI150297[] = {{&S48092[44], 13}, {NULL}};
const Segment SGI150298[] = {{&S48092[44], 25}, {NULL}};
const Segment SGI150299[] = {{&S48092[30], 6}, {NULL}};
const Segment SGI150300[] = {{&S48092[86], 4}, {NULL}};
const Segment SGI150301[] = {{&S48092[86], 3}, {NULL}};
const Segment SGI150302[] = {{&S38250[44], 7}, {NULL}};
const Segment SGI150303[] = {{&S38250[44], 24}, {NULL}};
const Segment SGI150304[] = {{&S38250[30], 6}, {NULL}};
const Segment SGI150305[] = {{&S38250[117], 3}, {NULL}};
const Segment SGI150306[] = {{&S38250[82], 6}, {NULL}};
const Segment SGI150307[] = {{&S48094[44], 13}, {NULL}};
const Segment SGI150308[] = {{&S38251[44], 11}, {NULL}};
const Segment SGI150309[] = {{&S48094[44], 25}, {NULL}};
const Segment SGI150310[] = {{&S38251[44], 28}, {NULL}};
const Segment SGI150311[] = {{&S48094[30], 6}, {NULL}};
const Segment SGI150312[] = {{&S38251[30], 6}, {NULL}};
const Segment SGI150313[] = {{&S48094[31], 3}, {NULL}};
const Segment SGI150314[] = {{&S38251[121], 3}, {NULL}};
const Segment SGI150315[] = {{&S48094[83], 6}, {NULL}};
const Segment SGI150316[] = {{&S38251[86], 6}, {NULL}};
const Segment SGI150317[] = {{&S38252[44], 7}, {NULL}};
const Segment SGI150318[] = {{&S38252[44], 24}, {NULL}};
const Segment SGI150319[] = {{&S38252[30], 6}, {NULL}};
const Segment SGI150320[] = {{&S38252[117], 3}, {NULL}};
const Segment SGI150321[] = {{&S38252[82], 6}, {NULL}};
const Segment SGI150322[] = {{&S48096[44], 13}, {NULL}};
const Segment SGI150323[] = {{&S48096[44], 25}, {NULL}};
const Segment SGI150324[] = {{&S48096[30], 6}, {NULL}};
const Segment SGI150325[] = {{&S48096[86], 6}, {NULL}};
const Segment SGI150326[] = {{&S38253[44], 11}, {NULL}};
const Segment SGI150327[] = {{&S38253[44], 28}, {NULL}};
const Segment SGI150328[] = {{&S38253[30], 6}, {NULL}};
const Segment SGI150329[] = {{&S38253[121], 3}, {NULL}};
const Segment SGI150330[] = {{&S38253[86], 6}, {NULL}};
const Segment SGI150331[] = {{&S48097[44], 8}, {NULL}};
const Segment SGI150332[] = {{&S48097[44], 20}, {NULL}};
const Segment SGI150333[] = {{&S48097[30], 6}, {NULL}};
const Segment SGI150334[] = {{&S48097[31], 3}, {NULL}};
const Segment SGI150335[] = {{&S48097[78], 6}, {NULL}};
const Segment SGI150336[] = {{&S38254[44], 7}, {NULL}};
const Segment SGI150337[] = {{&S38254[44], 24}, {NULL}};
const Segment SGI150338[] = {{&S38254[30], 6}, {NULL}};
const Segment SGI150339[] = {{&S38254[117], 3}, {NULL}};
const Segment SGI150340[] = {{&S38254[82], 6}, {NULL}};
const Segment SGI150341[] = {{&S38255[44], 7}, {NULL}};
const Segment SGI150342[] = {{&S48099[44], 8}, {NULL}};
const Segment SGI150343[] = {{&S38255[44], 24}, {NULL}};
const Segment SGI150344[] = {{&S38255[30], 6}, {NULL}};
const Segment SGI150345[] = {{&S48099[44], 20}, {NULL}};
const Segment SGI150346[] = {{&S48099[30], 6}, {NULL}};
const Segment SGI150347[] = {{&S38255[117], 3}, {NULL}};
const Segment SGI150348[] = {{&S48099[81], 6}, {NULL}};
const Segment SGI150349[] = {{&S38255[82], 6}, {NULL}};
const Segment SGI150350[] = {{&S48102[44], 8}, {NULL}};
const Segment SGI150351[] = {{&S48102[44], 20}, {NULL}};
const Segment SGI150352[] = {{&S48102[30], 6}, {NULL}};
const Segment SGI150353[] = {{&S48102[81], 4}, {NULL}};
const Segment SGI150354[] = {{&S48102[81], 3}, {NULL}};
const Segment SGI150355[] = {{&S48103[38], 13}, {NULL}};
const Segment SGI150356[] = {{&S48103[31], 6}, {NULL}};
const Segment SGI150357[] = {{&S48105[44], 13}, {NULL}};
const Segment SGI150358[] = {{&S48105[44], 25}, {NULL}};
const Segment SGI150359[] = {{&S48105[30], 6}, {NULL}};
const Segment SGI150360[] = {{&S48105[31], 3}, {NULL}};
const Segment SGI150361[] = {{&S48105[83], 5}, {NULL}};
const Segment SGI150362[] = {{&S48106[31], 6}, {NULL}};
const Segment SGI150363[] = {{&S48135[44], 10}, {NULL}};
const Segment SGI150364[] = {{&S48135[44], 22}, {NULL}};
const Segment SGI150365[] = {{&S48135[30], 6}, {NULL}};
const Segment SGI150366[] = {{&S48135[31], 3}, {NULL}};
const Segment SGI150367[] = {{&S48135[80], 6}, {NULL}};
const Segment SGI150368[] = {{&S48136[38], 10}, {NULL}};
const Segment SGI150369[] = {{&S48136[31], 6}, {NULL}};
const Segment SGI150370[] = {{&S48139[34], 12}, {NULL}};
const Segment SGI150371[] = {{&S48139[27], 6}, {NULL}};
const Segment SGI150372[] = {{&S48139[99], 9}, {NULL}};
const Segment SGI150373[] = {{&S48139[72], 6}, {NULL}};
const Segment SGI150374[] = {{&S48160[34], 10}, {NULL}};
const Segment SGI150375[] = {{&S48160[27], 6}, {NULL}};
const Segment SGI150376[] = {{&S48160[97], 9}, {NULL}};
const Segment SGI150377[] = {{&S48160[70], 6}, {NULL}};
const Segment SGI150378[] = {{&S48162[34], 11}, {NULL}};
const Segment SGI150379[] = {{&S48162[27], 6}, {NULL}};
const Segment SGI150380[] = {{&S48162[98], 9}, {NULL}};
const Segment SGI150381[] = {{&S48162[71], 6}, {NULL}};
const Segment SGI150382[] = {{&S48165[44], 15}, {NULL}};
const Segment SGI150383[] = {{&S48165[44], 27}, {NULL}};
const Segment SGI150384[] = {{&S48165[30], 6}, {NULL}};
const Segment SGI150385[] = {{&S48165[31], 3}, {NULL}};
const Segment SGI150386[] = {{&S48165[85], 5}, {NULL}};
const Segment SGI150387[] = {{&S48166[38], 15}, {NULL}};
const Segment SGI150388[] = {{&S48166[31], 6}, {NULL}};
const Segment SGI150389[] = {{&S48224[34], 11}, {NULL}};
const Segment SGI150390[] = {{&S48224[27], 6}, {NULL}};
const Segment SGI150391[] = {{&S48224[98], 9}, {NULL}};
const Segment SGI150392[] = {{&S48224[71], 6}, {NULL}};
const Segment SGI150393[] = {{&S34007[34], 9}, {NULL}};
const Segment SGI150394[] = {{&S34007[27], 6}, {NULL}};
const Segment SGI150395[] = {{&S34007[96], 9}, {NULL}};
const Segment SGI150396[] = {{&S34007[69], 6}, {NULL}};
const Segment SGI150397[] = {{&S48250[34], 6}, {NULL}};
const Segment SGI150398[] = {{&S48250[27], 6}, {NULL}};
const Segment SGI150399[] = {{&S48250[93], 9}, {NULL}};
const Segment SGI150400[] = {{&S48250[66], 6}, {NULL}};
const Segment SGI150401[] = {{&S34034[34], 10}, {NULL}};
const Segment SGI150402[] = {{&S34034[27], 6}, {NULL}};
const Segment SGI150403[] = {{&S34034[97], 9}, {NULL}};
const Segment SGI150404[] = {{&S34034[70], 6}, {NULL}};
const Segment SGI150405[] = {{&S34036[41], 10}, {NULL}};
const Segment SGI150406[] = {{&S34036[41], 22}, {NULL}};
const Segment SGI150407[] = {{&S34036[27], 6}, {NULL}};
const Segment SGI150408[] = {{&S34036[103], 9}, {NULL}};
const Segment SGI150409[] = {{&S34036[77], 6}, {NULL}};
const Segment SGI150410[] = {{&S34039[34], 9}, {NULL}};
const Segment SGI150411[] = {{&S34039[27], 6}, {NULL}};
const Segment SGI150412[] = {{&S34039[96], 9}, {NULL}};
const Segment SGI150413[] = {{&S34039[69], 6}, {NULL}};
const Segment SGI150414[] = {{&S34041[34], 9}, {NULL}};
const Segment SGI150415[] = {{&S34041[27], 6}, {NULL}};
const Segment SGI150416[] = {{&S34041[96], 9}, {NULL}};
const Segment SGI150417[] = {{&S34041[69], 6}, {NULL}};
const Segment SGI150418[] = {{&S48287[34], 9}, {NULL}};
const Segment SGI150419[] = {{&S48287[27], 6}, {NULL}};
const Segment SGI150420[] = {{&S48287[96], 9}, {NULL}};
const Segment SGI150421[] = {{&S48287[69], 6}, {NULL}};
const Segment SGI150422[] = {{&S48295[34], 8}, {NULL}};
const Segment SGI150423[] = {{&S48295[27], 6}, {NULL}};
const Segment SGI150424[] = {{&S48295[100], 9}, {NULL}};
const Segment SGI150425[] = {{&S48295[73], 6}, {NULL}};
const Segment SGI150426[] = {{&S48296[34], 7}, {NULL}};
const Segment SGI150427[] = {{&S48296[27], 6}, {NULL}};
const Segment SGI150428[] = {{&S48296[97], 9}, {NULL}};
const Segment SGI150429[] = {{&S48296[70], 6}, {NULL}};
const Segment SGI150430[] = {{&S48297[34], 7}, {NULL}};
const Segment SGI150431[] = {{&S48297[27], 6}, {NULL}};
const Segment SGI150432[] = {{&S48297[98], 9}, {NULL}};
const Segment SGI150433[] = {{&S48297[71], 6}, {NULL}};
const Segment SGI150434[] = {{&S48301[34], 7}, {NULL}};
const Segment SGI150435[] = {{&S48301[27], 6}, {NULL}};
const Segment SGI150436[] = {{&S48301[93], 9}, {NULL}};
const Segment SGI150437[] = {{&S48301[66], 6}, {NULL}};
const Segment SGI150438[] = {{&S48302[34], 6}, {NULL}};
const Segment SGI150439[] = {{&S48302[27], 6}, {NULL}};
const Segment SGI150440[] = {{&S48302[93], 9}, {NULL}};
const Segment SGI150441[] = {{&S48302[66], 6}, {NULL}};
const Segment SGI150442[] = {{&S48303[34], 11}, {NULL}};
const Segment SGI150443[] = {{&S48303[27], 6}, {NULL}};
const Segment SGI150444[] = {{&S48303[98], 9}, {NULL}};
const Segment SGI150445[] = {{&S48303[71], 6}, {NULL}};
const Segment SGI150446[] = {{&S48305[34], 11}, {NULL}};
const Segment SGI150447[] = {{&S48305[27], 6}, {NULL}};
const Segment SGI150448[] = {{&S48305[98], 9}, {NULL}};
const Segment SGI150449[] = {{&S48305[71], 6}, {NULL}};
const Segment SGI150450[] = {{&S48309[34], 11}, {NULL}};
const Segment SGI150451[] = {{&S48309[27], 6}, {NULL}};
const Segment SGI150452[] = {{&S48309[98], 9}, {NULL}};
const Segment SGI150453[] = {{&S48309[71], 6}, {NULL}};
const Segment SGI150454[] = {{&S38475[34], 11}, {NULL}};
const Segment SGI150455[] = {{&S38475[27], 6}, {NULL}};
const Segment SGI150456[] = {{&S38475[98], 9}, {NULL}};
const Segment SGI150457[] = {{&S38475[71], 6}, {NULL}};
const Segment SGI150458[] = {{&S38476[34], 7}, {NULL}};
const Segment SGI150459[] = {{&S38476[27], 6}, {NULL}};
const Segment SGI150460[] = {{&S38476[94], 9}, {NULL}};
const Segment SGI150461[] = {{&S38476[67], 6}, {NULL}};
const Segment SGI150462[] = {{&S38479[34], 11}, {NULL}};
const Segment SGI150463[] = {{&S38479[27], 6}, {NULL}};
const Segment SGI150464[] = {{&S38479[98], 9}, {NULL}};
const Segment SGI150465[] = {{&S38479[71], 6}, {NULL}};
const Segment SGI150466[] = {{&S38481[34], 11}, {NULL}};
const Segment SGI150467[] = {{&S38481[27], 6}, {NULL}};
const Segment SGI150468[] = {{&S38481[98], 9}, {NULL}};
const Segment SGI150469[] = {{&S38481[71], 6}, {NULL}};
const Segment SGI150470[] = {{&S38483[34], 7}, {NULL}};
const Segment SGI150471[] = {{&S38483[27], 6}, {NULL}};
const Segment SGI150472[] = {{&S38483[94], 9}, {NULL}};
const Segment SGI150473[] = {{&S38483[67], 6}, {NULL}};
const Segment SGI150474[] = {{&S48335[44], 10}, {NULL}};
const Segment SGI150475[] = {{&S48335[44], 22}, {NULL}};
const Segment SGI150476[] = {{&S48335[30], 6}, {NULL}};
const Segment SGI150477[] = {{&S48335[31], 3}, {NULL}};
const Segment SGI150478[] = {{&S48335[80], 6}, {NULL}};
const Segment SGI150479[] = {{&S48336[38], 10}, {NULL}};
const Segment SGI150480[] = {{&S48336[31], 6}, {NULL}};
const Segment SGI150481[] = {{&S48361[34], 7}, {NULL}};
const Segment SGI150482[] = {{&S48361[27], 6}, {NULL}};
const Segment SGI150483[] = {{&S48361[94], 9}, {NULL}};
const Segment SGI150484[] = {{&S48361[67], 6}, {NULL}};
const Segment SGI150485[] = {{&S48362[34], 11}, {NULL}};
const Segment SGI150486[] = {{&S48362[27], 6}, {NULL}};
const Segment SGI150487[] = {{&S48362[98], 9}, {NULL}};
const Segment SGI150488[] = {{&S48362[71], 6}, {NULL}};
const Segment SGI150489[] = {{&S39137[34], 7}, {NULL}};
const Segment SGI150490[] = {{&S39137[27], 6}, {NULL}};
const Segment SGI150491[] = {{&S39137[94], 9}, {NULL}};
const Segment SGI150492[] = {{&S39137[67], 6}, {NULL}};
const Segment SGI150493[] = {{&S48434[34], 10}, {NULL}};
const Segment SGI150494[] = {{&S48434[27], 6}, {NULL}};
const Segment SGI150495[] = {{&S48434[97], 9}, {NULL}};
const Segment SGI150496[] = {{&S48434[70], 6}, {NULL}};
const Segment SGI150497[] = {{&S48435[34], 10}, {NULL}};
const Segment SGI150498[] = {{&S48435[27], 6}, {NULL}};
const Segment SGI150499[] = {{&S48435[97], 9}, {NULL}};
const Segment SGI150500[] = {{&S48435[70], 6}, {NULL}};
const Segment SGI150501[] = {{&S48436[34], 11}, {NULL}};
const Segment SGI150502[] = {{&S48436[27], 6}, {NULL}};
const Segment SGI150503[] = {{&S48436[98], 9}, {NULL}};
const Segment SGI150504[] = {{&S48436[71], 6}, {NULL}};
const Segment SGI150505[] = {{&S48437[44], 10}, {NULL}};
const Segment SGI150506[] = {{&S48437[44], 22}, {NULL}};
const Segment SGI150507[] = {{&S48437[30], 6}, {NULL}};
const Segment SGI150508[] = {{&S48437[31], 3}, {NULL}};
const Segment SGI150509[] = {{&S48437[80], 6}, {NULL}};
const Segment SGI150510[] = {{&S48438[38], 10}, {NULL}};
const Segment SGI150511[] = {{&S48438[31], 6}, {NULL}};
const Segment SGI150512[] = {{&S48439[42], 10}, {NULL}};
const Segment SGI150513[] = {{&S48439[30], 6}, {NULL}};
const Segment SGI150514[] = {{&S48439[31], 3}, {NULL}};
const Segment SGI150515[] = {{&S48439[78], 6}, {NULL}};
const Segment SGI150516[] = {{&S48440[44], 10}, {NULL}};
const Segment SGI150517[] = {{&S48440[44], 22}, {NULL}};
const Segment SGI150518[] = {{&S48440[30], 6}, {NULL}};
const Segment SGI150519[] = {{&S48440[31], 3}, {NULL}};
const Segment SGI150520[] = {{&S48440[80], 5}, {NULL}};
const Segment SGI150521[] = {{&S48441[44], 15}, {NULL}};
const Segment SGI150522[] = {{&S48441[44], 27}, {NULL}};
const Segment SGI150523[] = {{&S48441[30], 6}, {NULL}};
const Segment SGI150524[] = {{&S48441[31], 3}, {NULL}};
const Segment SGI150525[] = {{&S39162[44], 11}, {NULL}};
const Segment SGI150526[] = {{&S48441[85], 6}, {NULL}};
const Segment SGI150527[] = {{&S39162[44], 23}, {NULL}};
const Segment SGI150528[] = {{&S39162[30], 6}, {NULL}};
const Segment SGI150529[] = {{&S39162[116], 3}, {NULL}};
const Segment SGI150530[] = {{&S39162[81], 6}, {NULL}};
const Segment SGI150531[] = {{&S48442[42], 15}, {NULL}};
const Segment SGI150532[] = {{&S48442[30], 6}, {NULL}};
const Segment SGI150533[] = {{&S48442[31], 3}, {NULL}};
const Segment SGI150534[] = {{&S48442[83], 6}, {NULL}};
const Segment SGI150535[] = {{&S39164[38], 11}, {NULL}};
const Segment SGI150536[] = {{&S39164[31], 6}, {NULL}};
const Segment SGI150537[] = {{&S39246[44], 9}, {NULL}};
const Segment SGI150538[] = {{&S39246[44], 21}, {NULL}};
const Segment SGI150539[] = {{&S39246[30], 6}, {NULL}};
const Segment SGI150540[] = {{&S39246[114], 3}, {NULL}};
const Segment SGI150541[] = {{&S39246[79], 6}, {NULL}};
const Segment SGI150542[] = {{&S48534[44], 13}, {NULL}};
const Segment SGI150543[] = {{&S48534[44], 25}, {NULL}};
const Segment SGI150544[] = {{&S48534[30], 6}, {NULL}};
const Segment SGI150545[] = {{&S48534[31], 3}, {NULL}};
const Segment SGI150546[] = {{&S39247[44], 9}, {NULL}};
const Segment SGI150547[] = {{&S39247[44], 21}, {NULL}};
const Segment SGI150548[] = {{&S48534[83], 6}, {NULL}};
const Segment SGI150549[] = {{&S39247[30], 6}, {NULL}};
const Segment SGI150550[] = {{&S39247[114], 3}, {NULL}};
const Segment SGI150551[] = {{&S39247[79], 6}, {NULL}};
const Segment SGI150552[] = {{&S48535[44], 10}, {NULL}};
const Segment SGI150553[] = {{&S39248[38], 9}, {NULL}};
const Segment SGI150554[] = {{&S48535[44], 22}, {NULL}};
const Segment SGI150555[] = {{&S39248[31], 6}, {NULL}};
const Segment SGI150556[] = {{&S48535[30], 6}, {NULL}};
const Segment SGI150557[] = {{&S48535[31], 3}, {NULL}};
const Segment SGI150558[] = {{&S48535[80], 6}, {NULL}};
const Segment SGI150559[] = {{&S48536[38], 10}, {NULL}};
const Segment SGI150560[] = {{&S48536[31], 6}, {NULL}};
const Segment SGI150561[] = {{&S39250[44], 9}, {NULL}};
const Segment SGI150562[] = {{&S39250[44], 21}, {NULL}};
const Segment SGI150563[] = {{&S39250[30], 6}, {NULL}};
const Segment SGI150564[] = {{&S39250[114], 3}, {NULL}};
const Segment SGI150565[] = {{&S39250[79], 6}, {NULL}};
const Segment SGI150566[] = {{&S39251[44], 9}, {NULL}};
const Segment SGI150567[] = {{&S39251[44], 21}, {NULL}};
const Segment SGI150568[] = {{&S39251[30], 6}, {NULL}};
const Segment SGI150569[] = {{&S39251[114], 3}, {NULL}};
const Segment SGI150570[] = {{&S39251[79], 6}, {NULL}};
const Segment SGI150571[] = {{&S39253[44], 9}, {NULL}};
const Segment SGI150572[] = {{&S39253[44], 21}, {NULL}};
const Segment SGI150573[] = {{&S39253[30], 6}, {NULL}};
const Segment SGI150574[] = {{&S39253[82], 6}, {NULL}};
const Segment SGI150575[] = {{&S39254[44], 9}, {NULL}};
const Segment SGI150576[] = {{&S39254[44], 21}, {NULL}};
const Segment SGI150577[] = {{&S39254[30], 6}, {NULL}};
const Segment SGI150578[] = {{&S39254[114], 3}, {NULL}};
const Segment SGI150579[] = {{&S39254[79], 6}, {NULL}};
const Segment SGI150580[] = {{&S39256[44], 9}, {NULL}};
const Segment SGI150581[] = {{&S39256[44], 21}, {NULL}};
const Segment SGI150582[] = {{&S39256[30], 6}, {NULL}};
const Segment SGI150583[] = {{&S39256[79], 6}, {NULL}};
const Segment SGI150584[] = {{&S39258[44], 9}, {NULL}};
const Segment SGI150585[] = {{&S39258[44], 21}, {NULL}};
const Segment SGI150586[] = {{&S39258[30], 6}, {NULL}};
const Segment SGI150587[] = {{&S39258[114], 3}, {NULL}};
const Segment SGI150588[] = {{&S39258[79], 6}, {NULL}};
const Segment SGI150589[] = {{&S39259[44], 22}, {NULL}};
const Segment SGI150590[] = {{&S39259[30], 6}, {NULL}};
const Segment SGI150591[] = {{&S39259[129], 3}, {NULL}};
const Segment SGI150592[] = {{&S39259[94], 6}, {NULL}};
const Segment SGI150593[] = {{&S39260[42], 9}, {NULL}};
const Segment SGI150594[] = {{&S39260[30], 6}, {NULL}};
const Segment SGI150595[] = {{&S39260[112], 3}, {NULL}};
const Segment SGI150596[] = {{&S38653[34], 10}, {NULL}};
const Segment SGI150597[] = {{&S39260[77], 6}, {NULL}};
const Segment SGI150598[] = {{&S38653[27], 6}, {NULL}};
const Segment SGI150599[] = {{&S38653[97], 9}, {NULL}};
const Segment SGI150600[] = {{&S38653[70], 6}, {NULL}};
const Segment SGI150601[] = {{&S48556[34], 24}, {NULL}};
const Segment SGI150602[] = {{&S48556[27], 6}, {NULL}};
const Segment SGI150603[] = {{&S48556[111], 9}, {NULL}};
const Segment SGI150604[] = {{&S48556[84], 6}, {NULL}};
const Segment SGI150605[] = {{&S39261[44], 16}, {NULL}};
const Segment SGI150606[] = {{&S39261[44], 28}, {NULL}};
const Segment SGI150607[] = {{&S39261[30], 6}, {NULL}};
const Segment SGI150608[] = {{&S39261[121], 3}, {NULL}};
const Segment SGI150609[] = {{&S39261[86], 6}, {NULL}};
const Segment SGI150610[] = {{&S48557[34], 26}, {NULL}};
const Segment SGI150611[] = {{&S48557[27], 6}, {NULL}};
const Segment SGI150612[] = {{&S48557[113], 9}, {NULL}};
const Segment SGI150613[] = {{&S48557[86], 6}, {NULL}};
const Segment SGI150614[] = {{&S39262[44], 16}, {NULL}};
const Segment SGI150615[] = {{&S39262[44], 28}, {NULL}};
const Segment SGI150616[] = {{&S39262[30], 6}, {NULL}};
const Segment SGI150617[] = {{&S39262[121], 3}, {NULL}};
const Segment SGI150618[] = {{&S39262[86], 6}, {NULL}};
const Segment SGI150619[] = {{&S39263[44], 9}, {NULL}};
const Segment SGI150620[] = {{&S39263[44], 21}, {NULL}};
const Segment SGI150621[] = {{&S39263[30], 6}, {NULL}};
const Segment SGI150622[] = {{&S39263[114], 3}, {NULL}};
const Segment SGI150623[] = {{&S39263[79], 6}, {NULL}};
const Segment SGI150624[] = {{&S39264[44], 9}, {NULL}};
const Segment SGI150625[] = {{&S39264[44], 41}, {NULL}};
const Segment SGI150626[] = {{&S39264[30], 6}, {NULL}};
const Segment SGI150627[] = {{&S39264[134], 3}, {NULL}};
const Segment SGI150628[] = {{&S39264[99], 6}, {NULL}};
const Segment SGI150629[] = {{&S39265[44], 9}, {NULL}};
const Segment SGI150630[] = {{&S39265[44], 21}, {NULL}};
const Segment SGI150631[] = {{&S39265[30], 6}, {NULL}};
const Segment SGI150632[] = {{&S39265[79], 6}, {NULL}};
const Segment SGI150633[] = {{&S48563[44], 15}, {NULL}};
const Segment SGI150634[] = {{&S48563[44], 38}, {NULL}};
const Segment SGI150635[] = {{&S48563[30], 6}, {NULL}};
const Segment SGI150636[] = {{&S48563[31], 3}, {NULL}};
const Segment SGI150637[] = {{&S48563[96], 5}, {NULL}};
const Segment SGI150638[] = {{&S39266[44], 9}, {NULL}};
const Segment SGI150639[] = {{&S39266[44], 32}, {NULL}};
const Segment SGI150640[] = {{&S39266[30], 6}, {NULL}};
const Segment SGI150641[] = {{&S39266[125], 3}, {NULL}};
const Segment SGI150642[] = {{&S39266[90], 6}, {NULL}};
const Segment SGI150643[] = {{&S48564[44], 15}, {NULL}};
const Segment SGI150644[] = {{&S48564[44], 38}, {NULL}};
const Segment SGI150645[] = {{&S48564[30], 6}, {NULL}};
const Segment SGI150646[] = {{&S48564[31], 3}, {NULL}};
const Segment SGI150647[] = {{&S48564[96], 5}, {NULL}};
const Segment SGI150648[] = {{&S39267[44], 9}, {NULL}};
const Segment SGI150649[] = {{&S39267[44], 21}, {NULL}};
const Segment SGI150650[] = {{&S39267[30], 6}, {NULL}};
const Segment SGI150651[] = {{&S39267[79], 6}, {NULL}};
const Segment SGI150652[] = {{&S48565[44], 8}, {NULL}};
const Segment SGI150653[] = {{&S48565[44], 31}, {NULL}};
const Segment SGI150654[] = {{&S48565[30], 6}, {NULL}};
const Segment SGI150655[] = {{&S39268[44], 9}, {NULL}};
const Segment SGI150656[] = {{&S39268[44], 21}, {NULL}};
const Segment SGI150657[] = {{&S39268[30], 6}, {NULL}};
const Segment SGI150658[] = {{&S48565[31], 3}, {NULL}};
const Segment SGI150659[] = {{&S48565[89], 5}, {NULL}};
const Segment SGI150660[] = {{&S39268[114], 3}, {NULL}};
const Segment SGI150661[] = {{&S39268[79], 6}, {NULL}};
const Segment SGI150662[] = {{&S48566[44], 15}, {NULL}};
const Segment SGI150663[] = {{&S48566[44], 38}, {NULL}};
const Segment SGI150664[] = {{&S39269[44], 9}, {NULL}};
const Segment SGI150665[] = {{&S48566[30], 6}, {NULL}};
const Segment SGI150666[] = {{&S39269[44], 21}, {NULL}};
const Segment SGI150667[] = {{&S39269[30], 6}, {NULL}};
const Segment SGI150668[] = {{&S48566[31], 3}, {NULL}};
const Segment SGI150669[] = {{&S48566[96], 5}, {NULL}};
const Segment SGI150670[] = {{&S39269[79], 6}, {NULL}};
const Segment SGI150671[] = {{&S48567[44], 8}, {NULL}};
const Segment SGI150672[] = {{&S48567[44], 31}, {NULL}};
const Segment SGI150673[] = {{&S48567[30], 6}, {NULL}};
const Segment SGI150674[] = {{&S39270[44], 9}, {&S39270[174], 14}, {NULL}};
const Segment SGI150675[] = {{&S48567[31], 3}, {NULL}};
const Segment SGI150676[] = {{&S39270[44], 21}, {NULL}};
const Segment SGI150677[] = {{&S39270[30], 6}, {NULL}};
const Segment SGI150678[] = {{&S48567[89], 5}, {NULL}};
const Segment SGI150679[] = {{&S39270[114], 3}, {NULL}};
const Segment SGI150680[] = {{&S39270[79], 6}, {NULL}};
const Segment SGI150681[] = {{&S48568[38], 8}, {NULL}};
const Segment SGI150682[] = {{&S48568[31], 6}, {NULL}};
const Segment SGI150683[] = {{&S39271[44], 9}, {NULL}};
const Segment SGI150684[] = {{&S39271[44], 21}, {NULL}};
const Segment SGI150685[] = {{&S39271[30], 6}, {NULL}};
const Segment SGI150686[] = {{&S39271[114], 3}, {NULL}};
const Segment SGI150687[] = {{&S39271[79], 6}, {NULL}};
const Segment SGI150688[] = {{&S48569[44], 8}, {NULL}};
const Segment SGI150689[] = {{&S48569[44], 31}, {NULL}};
const Segment SGI150690[] = {{&S48569[30], 6}, {NULL}};
const Segment SGI150691[] = {{&S39272[44], 9}, {NULL}};
const Segment SGI150692[] = {{&S48569[31], 3}, {NULL}};
const Segment SGI150693[] = {{&S39272[44], 21}, {NULL}};
const Segment SGI150694[] = {{&S39272[30], 6}, {NULL}};
const Segment SGI150695[] = {{&S48569[89], 5}, {NULL}};
const Segment SGI150696[] = {{&S39272[115], 3}, {NULL}};
const Segment SGI150697[] = {{&S39272[79], 7}, {NULL}};
const Segment SGI150698[] = {{&S39272[135], 8}, {NULL}};
const Segment SGI150699[] = {{&S48570[44], 15}, {NULL}};
const Segment SGI150700[] = {{&S48570[44], 38}, {NULL}};
const Segment SGI150701[] = {{&S48570[30], 6}, {NULL}};
const Segment SGI150702[] = {{&S39273[45], 9}, {NULL}};
const Segment SGI150703[] = {{&S48570[31], 3}, {NULL}};
const Segment SGI150704[] = {{&S39273[45], 68}, {NULL}};
const Segment SGI150705[] = {{&S39273[31], 6}, {NULL}};
const Segment SGI150706[] = {{&S48570[96], 5}, {NULL}};
const Segment SGI150707[] = {{&S48571[44], 8}, {NULL}};
const Segment SGI150708[] = {{&S48571[44], 31}, {NULL}};
const Segment SGI150709[] = {{&S48571[30], 6}, {NULL}};
const Segment SGI150710[] = {{&S39274[44], 9}, {NULL}};
const Segment SGI150711[] = {{&S39274[44], 75}, {NULL}};
const Segment SGI150712[] = {{&S39274[30], 6}, {NULL}};
const Segment SGI150713[] = {{&S48571[31], 3}, {NULL}};
const Segment SGI150714[] = {{&S48571[89], 5}, {NULL}};
const Segment SGI150715[] = {{&S39275[44], 9}, {NULL}};
const Segment SGI150716[] = {{&S39275[44], 32}, {NULL}};
const Segment SGI150717[] = {{&S48572[44], 15}, {NULL}};
const Segment SGI150718[] = {{&S39275[30], 6}, {NULL}};
const Segment SGI150719[] = {{&S48572[44], 38}, {NULL}};
const Segment SGI150720[] = {{&S48572[30], 6}, {NULL}};
const Segment SGI150721[] = {{&S48572[31], 3}, {NULL}};
const Segment SGI150722[] = {{&S48572[96], 5}, {NULL}};
const Segment SGI150723[] = {{&S39276[44], 9}, {NULL}};
const Segment SGI150724[] = {{&S39276[44], 21}, {NULL}};
const Segment SGI150725[] = {{&S39276[30], 6}, {NULL}};
const Segment SGI150726[] = {{&S48573[44], 8}, {NULL}};
const Segment SGI150727[] = {{&S48573[44], 31}, {NULL}};
const Segment SGI150728[] = {{&S48573[30], 6}, {NULL}};
const Segment SGI150729[] = {{&S48573[31], 3}, {NULL}};
const Segment SGI150730[] = {{&S39277[44], 9}, {NULL}};
const Segment SGI150731[] = {{&S48573[89], 5}, {NULL}};
const Segment SGI150732[] = {{&S39277[44], 75}, {NULL}};
const Segment SGI150733[] = {{&S39277[30], 6}, {NULL}};
const Segment SGI150734[] = {{&S48574[34], 19}, {NULL}};
const Segment SGI150735[] = {{&S48574[27], 6}, {NULL}};
const Segment SGI150736[] = {{&S48574[106], 9}, {NULL}};
const Segment SGI150737[] = {{&S48574[79], 6}, {NULL}};
const Segment SGI150738[] = {{&S39280[59], 9}, {NULL}};
const Segment SGI150739[] = {{&S39280[59], 71}, {NULL}};
const Segment SGI150740[] = {{&S48575[34], 18}, {NULL}};
const Segment SGI150741[] = {{&S39280[45], 6}, {NULL}};
const Segment SGI150742[] = {{&S48575[27], 6}, {NULL}};
const Segment SGI150743[] = {{&S48575[105], 9}, {NULL}};
const Segment SGI150744[] = {{&S48575[78], 6}, {NULL}};
const Segment SGI150745[] = {{&S39281[52], 9}, {NULL}};
const Segment SGI150746[] = {{&S39281[52], 21}, {NULL}};
const Segment SGI150747[] = {{&S39281[38], 6}, {NULL}};
const Segment SGI150748[] = {{&S48577[34], 18}, {NULL}};
const Segment SGI150749[] = {{&S48577[27], 6}, {NULL}};
const Segment SGI150750[] = {{&S48577[105], 9}, {NULL}};
const Segment SGI150751[] = {{&S48577[78], 6}, {NULL}};
const Segment SGI150752[] = {{&S39283[30], 1}, {NULL}};
const Segment SGI150753[] = {{&S39283[63], 8}, {NULL}};
const Segment SGI150754[] = {{&S39283[49], 6}, {NULL}};
const Segment SGI150755[] = {{&S39285[64], 9}, {NULL}};
const Segment SGI150756[] = {{&S39285[64], 21}, {NULL}};
const Segment SGI150757[] = {{&S39285[50], 6}, {NULL}};
const Segment SGI150758[] = {{&S48581[34], 10}, {NULL}};
const Segment SGI150759[] = {{&S48581[27], 6}, {NULL}};
const Segment SGI150760[] = {{&S48581[97], 9}, {NULL}};
const Segment SGI150761[] = {{&S48581[70], 6}, {NULL}};
const Segment SGI150762[] = {{&S39290[38], 16}, {NULL}};
const Segment SGI150763[] = {{&S39290[31], 6}, {NULL}};
const Segment SGI150764[] = {{&S39291[31], 6}, {NULL}};
const Segment SGI150765[] = {{&S39292[44], 22}, {NULL}};
const Segment SGI150766[] = {{&S39292[30], 6}, {NULL}};
const Segment SGI150767[] = {{&S39292[94], 6}, {NULL}};
const Segment SGI150768[] = {{&S39293[44], 9}, {NULL}};
const Segment SGI150769[] = {{&S39293[44], 21}, {NULL}};
const Segment SGI150770[] = {{&S39293[30], 6}, {NULL}};
const Segment SGI150771[] = {{&S39293[79], 6}, {NULL}};
const Segment SGI150772[] = {{&S39296[44], 9}, {NULL}};
const Segment SGI150773[] = {{&S39296[44], 21}, {NULL}};
const Segment SGI150774[] = {{&S39296[30], 6}, {NULL}};
const Segment SGI150775[] = {{&S39296[113], 3}, {NULL}};
const Segment SGI150776[] = {{&S39296[79], 5}, {NULL}};
const Segment SGI150777[] = {{&S39299[44], 9}, {NULL}};
const Segment SGI150778[] = {{&S39299[44], 21}, {NULL}};
const Segment SGI150779[] = {{&S39299[30], 6}, {NULL}};
const Segment SGI150780[] = {{&S39299[82], 6}, {NULL}};
const Segment SGI150781[] = {{&S39300[44], 13}, {NULL}};
const Segment SGI150782[] = {{&S39300[44], 25}, {NULL}};
const Segment SGI150783[] = {{&S39300[30], 6}, {NULL}};
const Segment SGI150784[] = {{&S39300[118], 3}, {NULL}};
const Segment SGI150785[] = {{&S48601[44], 10}, {NULL}};
const Segment SGI150786[] = {{&S39300[83], 6}, {NULL}};
const Segment SGI150787[] = {{&S48601[44], 22}, {NULL}};
const Segment SGI150788[] = {{&S48601[30], 6}, {NULL}};
const Segment SGI150789[] = {{&S48601[31], 3}, {NULL}};
const Segment SGI150790[] = {{&S48601[80], 6}, {NULL}};
const Segment SGI150791[] = {{&S48602[38], 10}, {NULL}};
const Segment SGI150792[] = {{&S48602[31], 6}, {NULL}};
const Segment SGI150793[] = {{&S39305[44], 9}, {NULL}};
const Segment SGI150794[] = {{&S39305[44], 21}, {NULL}};
const Segment SGI150795[] = {{&S39305[30], 6}, {NULL}};
const Segment SGI150796[] = {{&S39305[82], 6}, {NULL}};
const Segment SGI150797[] = {{&S39310[44], 9}, {NULL}};
const Segment SGI150798[] = {{&S39310[44], 22}, {NULL}};
const Segment SGI150799[] = {{&S39310[30], 6}, {NULL}};
const Segment SGI150800[] = {{&S39310[115], 3}, {NULL}};
const Segment SGI150801[] = {{&S39310[80], 6}, {NULL}};
const Segment SGI150802[] = {{&S39314[34], 9}, {NULL}};
const Segment SGI150803[] = {{&S39314[34], 21}, {NULL}};
const Segment SGI150804[] = {{&S39317[70], 9}, {NULL}};
const Segment SGI150805[] = {{&S39317[70], 21}, {NULL}};
const Segment SGI150806[] = {{&S39317[30], 32}, {NULL}};
const Segment SGI150807[] = {{&S39317[140], 3}, {NULL}};
const Segment SGI150808[] = {{&S39317[105], 6}, {NULL}};
const Segment SGI150809[] = {{&S43559[32], 6}, {NULL}};
const Segment SGI150810[] = {{&S43559[27], 4}, {NULL}};
const Segment SGI150811[] = {{&S43559[90], 9}, {NULL}};
const Segment SGI150812[] = {{&S43559[63], 6}, {NULL}};
const Segment SGI150813[] = {{&S39318[44], 9}, {NULL}};
const Segment SGI150814[] = {{&S39318[44], 20}, {NULL}};
const Segment SGI150815[] = {{&S39318[30], 6}, {NULL}};
const Segment SGI150816[] = {{&S39318[113], 3}, {NULL}};
const Segment SGI150817[] = {{&S39318[78], 6}, {NULL}};
const Segment SGI150818[] = {{&S34427[44], 10}, {NULL}};
const Segment SGI150819[] = {{&S34427[44], 22}, {NULL}};
const Segment SGI150820[] = {{&S34427[30], 6}, {NULL}};
const Segment SGI150821[] = {{&S48622[34], 7}, {NULL}};
const Segment SGI150822[] = {{&S48622[27], 6}, {NULL}};
const Segment SGI150823[] = {{&S34427[133], 3}, {NULL}};
const Segment SGI150824[] = {{&S34427[98], 6}, {NULL}};
const Segment SGI150825[] = {{&S34427[68], 8}, {NULL}};
const Segment SGI150826[] = {{&S48622[93], 9}, {NULL}};
const Segment SGI150827[] = {{&S48622[66], 6}, {NULL}};
const Segment SGI150828[] = {{&S34428[44], 11}, {NULL}};
const Segment SGI150829[] = {{&S34428[44], 23}, {NULL}};
const Segment SGI150830[] = {{&S48623[34], 8}, {NULL}};
const Segment SGI150831[] = {{&S34428[30], 6}, {NULL}};
const Segment SGI150832[] = {{&S48623[27], 6}, {NULL}};
const Segment SGI150833[] = {{&S34428[134], 3}, {NULL}};
const Segment SGI150834[] = {{&S48623[99], 9}, {NULL}};
const Segment SGI150835[] = {{&S34428[99], 6}, {NULL}};
const Segment SGI150836[] = {{&S48623[72], 6}, {NULL}};
const Segment SGI150837[] = {{&S34428[69], 8}, {NULL}};
const Segment SGI150838[] = {{&S48624[34], 8}, {NULL}};
const Segment SGI150839[] = {{&S34429[44], 10}, {NULL}};
const Segment SGI150840[] = {{&S48624[27], 6}, {NULL}};
const Segment SGI150841[] = {{&S34429[44], 22}, {NULL}};
const Segment SGI150842[] = {{&S34429[30], 6}, {NULL}};
const Segment SGI150843[] = {{&S48624[99], 9}, {NULL}};
const Segment SGI150844[] = {{&S34429[133], 3}, {NULL}};
const Segment SGI150845[] = {{&S48624[72], 6}, {NULL}};
const Segment SGI150846[] = {{&S34429[98], 6}, {NULL}};
const Segment SGI150847[] = {{&S34429[68], 8}, {NULL}};
const Segment SGI150848[] = {{&S48625[34], 7}, {NULL}};
const Segment SGI150849[] = {{&S48625[27], 6}, {NULL}};
const Segment SGI150850[] = {{&S48625[99], 9}, {NULL}};
const Segment SGI150851[] = {{&S48625[72], 6}, {NULL}};
const Segment SGI150852[] = {{&S48626[34], 7}, {NULL}};
const Segment SGI150853[] = {{&S48626[27], 6}, {NULL}};
const Segment SGI150854[] = {{&S48626[99], 9}, {NULL}};
const Segment SGI150855[] = {{&S48626[72], 6}, {NULL}};
const Segment SGI150856[] = {{&S34433[44], 11}, {NULL}};
const Segment SGI150857[] = {{&S34433[44], 23}, {NULL}};
const Segment SGI150858[] = {{&S34433[30], 6}, {NULL}};
const Segment SGI150859[] = {{&S34433[134], 3}, {NULL}};
const Segment SGI150860[] = {{&S34433[99], 6}, {NULL}};
const Segment SGI150861[] = {{&S34433[69], 8}, {NULL}};
const Segment SGI150862[] = {{&S48634[34], 10}, {NULL}};
const Segment SGI150863[] = {{&S48634[27], 6}, {NULL}};
const Segment SGI150864[] = {{&S48634[97], 9}, {NULL}};
const Segment SGI150865[] = {{&S48634[70], 6}, {NULL}};
const Segment SGI150866[] = {{&S48635[34], 10}, {NULL}};
const Segment SGI150867[] = {{&S48635[27], 6}, {NULL}};
const Segment SGI150868[] = {{&S48635[97], 9}, {NULL}};
const Segment SGI150869[] = {{&S48635[70], 6}, {NULL}};
const Segment SGI150870[] = {{&S48643[44], 39}, {NULL}};
const Segment SGI150871[] = {{&S48643[30], 6}, {NULL}};
const Segment SGI150872[] = {{&S48643[115], 6}, {NULL}};
const Segment SGI150873[] = {{&S48644[44], 39}, {NULL}};
const Segment SGI150874[] = {{&S48644[30], 6}, {NULL}};
const Segment SGI150875[] = {{&S48644[115], 6}, {NULL}};
const Segment SGI150876[] = {{&S48645[44], 15}, {NULL}};
const Segment SGI150877[] = {{&S48645[44], 33}, {NULL}};
const Segment SGI150878[] = {{&S48645[30], 6}, {NULL}};
const Segment SGI150879[] = {{&S48645[94], 6}, {NULL}};
const Segment SGI150880[] = {{&S48646[44], 15}, {NULL}};
const Segment SGI150881[] = {{&S48646[44], 33}, {NULL}};
const Segment SGI150882[] = {{&S48646[30], 6}, {NULL}};
const Segment SGI150883[] = {{&S48646[31], 3}, {NULL}};
const Segment SGI150884[] = {{&S48646[91], 6}, {NULL}};
const Segment SGI150885[] = {{&S48647[44], 39}, {NULL}};
const Segment SGI150886[] = {{&S48647[30], 6}, {NULL}};
const Segment SGI150887[] = {{&S48647[115], 6}, {NULL}};
const Segment SGI150888[] = {{&S48648[44], 43}, {NULL}};
const Segment SGI150889[] = {{&S48648[30], 6}, {NULL}};
const Segment SGI150890[] = {{&S48648[119], 6}, {NULL}};
const Segment SGI150891[] = {{&S48649[44], 15}, {NULL}};
const Segment SGI150892[] = {{&S48649[44], 33}, {NULL}};
const Segment SGI150893[] = {{&S48649[30], 6}, {NULL}};
const Segment SGI150894[] = {{&S48649[94], 4}, {NULL}};
const Segment SGI150895[] = {{&S48649[94], 3}, {NULL}};
const Segment SGI150896[] = {{&S48650[44], 39}, {NULL}};
const Segment SGI150897[] = {{&S48650[30], 6}, {NULL}};
const Segment SGI150898[] = {{&S48650[115], 6}, {NULL}};
const Segment SGI150899[] = {{&S48651[44], 43}, {NULL}};
const Segment SGI150900[] = {{&S48651[30], 6}, {NULL}};
const Segment SGI150901[] = {{&S48651[119], 6}, {NULL}};
const Segment SGI150902[] = {{&S48652[38], 15}, {NULL}};
const Segment SGI150903[] = {{&S48652[31], 6}, {NULL}};
const Segment SGI150904[] = {{&S48653[44], 33}, {NULL}};
const Segment SGI150905[] = {{&S48653[30], 6}, {NULL}};
const Segment SGI150906[] = {{&S48654[44], 39}, {NULL}};
const Segment SGI150907[] = {{&S48654[30], 6}, {NULL}};
const Segment SGI150908[] = {{&S48654[115], 6}, {NULL}};
const Segment SGI150909[] = {{&S48655[44], 14}, {NULL}};
const Segment SGI150910[] = {{&S48655[44], 25}, {NULL}};
const Segment SGI150911[] = {{&S48655[30], 6}, {NULL}};
const Segment SGI150912[] = {{&S39382[44], 18}, {NULL}};
const Segment SGI150913[] = {{&S48655[31], 3}, {NULL}};
const Segment SGI150914[] = {{&S39382[44], 39}, {NULL}};
const Segment SGI150915[] = {{&S39382[30], 6}, {NULL}};
const Segment SGI150916[] = {{&S48655[83], 6}, {NULL}};
const Segment SGI150917[] = {{&S39382[132], 3}, {NULL}};
const Segment SGI150918[] = {{&S39382[97], 6}, {NULL}};
const Segment SGI150919[] = {{&S48656[44], 39}, {NULL}};
const Segment SGI150920[] = {{&S48656[30], 6}, {NULL}};
const Segment SGI150921[] = {{&S39383[44], 16}, {NULL}};
const Segment SGI150922[] = {{&S39383[44], 33}, {NULL}};
const Segment SGI150923[] = {{&S39383[30], 6}, {NULL}};
const Segment SGI150924[] = {{&S48656[115], 6}, {NULL}};
const Segment SGI150925[] = {{&S39383[126], 3}, {NULL}};
const Segment SGI150926[] = {{&S39383[91], 6}, {NULL}};
const Segment SGI150927[] = {{&S48657[44], 44}, {NULL}};
const Segment SGI150928[] = {{&S48657[30], 6}, {NULL}};
const Segment SGI150929[] = {{&S34481[44], 9}, {NULL}};
const Segment SGI150930[] = {{&S39384[44], 18}, {NULL}};
const Segment SGI150931[] = {{&S34481[44], 21}, {NULL}};
const Segment SGI150932[] = {{&S39384[44], 39}, {NULL}};
const Segment SGI150933[] = {{&S34481[30], 6}, {NULL}};
const Segment SGI150934[] = {{&S39384[30], 6}, {NULL}};
const Segment SGI150935[] = {{&S48657[120], 6}, {NULL}};
const Segment SGI150936[] = {{&S34481[114], 3}, {NULL}};
const Segment SGI150937[] = {{&S34481[79], 6}, {NULL}};
const Segment SGI150938[] = {{&S39384[100], 6}, {NULL}};
const Segment SGI150939[] = {{&S34482[38], 9}, {NULL}};
const Segment SGI150940[] = {{&S39385[44], 16}, {NULL}};
const Segment SGI150941[] = {{&S34482[31], 6}, {NULL}};
const Segment SGI150942[] = {{&S39385[44], 37}, {NULL}};
const Segment SGI150943[] = {{&S39385[30], 6}, {NULL}};
const Segment SGI150944[] = {{&S39385[130], 3}, {NULL}};
const Segment SGI150945[] = {{&S39385[95], 6}, {NULL}};
const Segment SGI150946[] = {{&S39386[44], 18}, {NULL}};
const Segment SGI150947[] = {{&S39386[44], 39}, {NULL}};
const Segment SGI150948[] = {{&S39386[30], 6}, {NULL}};
const Segment SGI150949[] = {{&S34484[44], 9}, {NULL}};
const Segment SGI150950[] = {{&S34484[44], 21}, {NULL}};
const Segment SGI150951[] = {{&S34484[30], 6}, {NULL}};
const Segment SGI150952[] = {{&S39386[132], 3}, {NULL}};
const Segment SGI150953[] = {{&S39386[97], 6}, {NULL}};
const Segment SGI150954[] = {{&S34484[114], 3}, {NULL}};
const Segment SGI150955[] = {{&S34484[79], 6}, {NULL}};
const Segment SGI150956[] = {{&S39387[44], 18}, {NULL}};
const Segment SGI150957[] = {{&S39387[44], 31}, {NULL}};
const Segment SGI150958[] = {{&S39387[30], 6}, {NULL}};
const Segment SGI150959[] = {{&S39387[124], 3}, {NULL}};
const Segment SGI150960[] = {{&S39387[89], 6}, {NULL}};
const Segment SGI150961[] = {{&S39388[38], 18}, {NULL}};
const Segment SGI150962[] = {{&S39388[31], 6}, {NULL}};
const Segment SGI150963[] = {{&S39389[44], 18}, {NULL}};
const Segment SGI150964[] = {{&S39389[44], 31}, {NULL}};
const Segment SGI150965[] = {{&S39389[30], 6}, {NULL}};
const Segment SGI150966[] = {{&S34489[38], 10}, {NULL}};
const Segment SGI150967[] = {{&S34489[31], 6}, {NULL}};
const Segment SGI150968[] = {{&S39389[124], 3}, {NULL}};
const Segment SGI150969[] = {{&S39389[89], 6}, {NULL}};
const Segment SGI150970[] = {{&S34490[44], 10}, {NULL}};
const Segment SGI150971[] = {{&S34490[44], 22}, {NULL}};
const Segment SGI150972[] = {{&S39390[44], 18}, {NULL}};
const Segment SGI150973[] = {{&S34490[30], 6}, {NULL}};
const Segment SGI150974[] = {{&S39390[44], 39}, {NULL}};
const Segment SGI150975[] = {{&S39390[30], 6}, {NULL}};
const Segment SGI150976[] = {{&S34490[115], 3}, {NULL}};
const Segment SGI150977[] = {{&S39390[132], 3}, {NULL}};
const Segment SGI150978[] = {{&S34490[80], 6}, {NULL}};
const Segment SGI150979[] = {{&S39390[97], 6}, {NULL}};
const Segment SGI150980[] = {{&S34491[44], 10}, {NULL}};
const Segment SGI150981[] = {{&S39391[38], 16}, {NULL}};
const Segment SGI150982[] = {{&S34491[44], 22}, {NULL}};
const Segment SGI150983[] = {{&S39391[31], 6}, {NULL}};
const Segment SGI150984[] = {{&S34491[30], 6}, {NULL}};
const Segment SGI150985[] = {{&S34491[115], 3}, {NULL}};
const Segment SGI150986[] = {{&S34491[80], 6}, {NULL}};
const Segment SGI150987[] = {{&S39392[44], 16}, {NULL}};
const Segment SGI150988[] = {{&S39392[44], 30}, {NULL}};
const Segment SGI150989[] = {{&S39392[30], 6}, {NULL}};
const Segment SGI150990[] = {{&S34492[44], 10}, {NULL}};
const Segment SGI150991[] = {{&S34492[44], 22}, {NULL}};
const Segment SGI150992[] = {{&S39392[123], 3}, {NULL}};
const Segment SGI150993[] = {{&S34492[30], 6}, {NULL}};
const Segment SGI150994[] = {{&S39392[88], 6}, {NULL}};
const Segment SGI150995[] = {{&S34492[115], 3}, {NULL}};
const Segment SGI150996[] = {{&S34492[80], 6}, {NULL}};
const Segment SGI150997[] = {{&S39393[38], 18}, {NULL}};
const Segment SGI150998[] = {{&S39393[31], 6}, {NULL}};
const Segment SGI150999[] = {{&S34493[44], 10}, {NULL}};
const Segment SGI151000[] = {{&S34493[44], 22}, {NULL}};
const Segment SGI151001[] = {{&S34493[30], 6}, {NULL}};
const Segment SGI151002[] = {{&S34493[115], 3}, {NULL}};
const Segment SGI151003[] = {{&S34493[80], 6}, {NULL}};
const Segment SGI151004[] = {{&S39394[44], 18}, {NULL}};
const Segment SGI151005[] = {{&S39394[44], 39}, {NULL}};
const Segment SGI151006[] = {{&S39394[30], 6}, {NULL}};
const Segment SGI151007[] = {{&S39394[132], 3}, {NULL}};
const Segment SGI151008[] = {{&S39394[97], 6}, {NULL}};
const Segment SGI151009[] = {{&S39395[38], 18}, {NULL}};
const Segment SGI151010[] = {{&S39395[31], 6}, {NULL}};
const Segment SGI151011[] = {{&S34496[44], 10}, {NULL}};
const Segment SGI151012[] = {{&S34496[44], 22}, {NULL}};
const Segment SGI151013[] = {{&S34496[30], 6}, {NULL}};
const Segment SGI151014[] = {{&S34496[115], 3}, {NULL}};
const Segment SGI151015[] = {{&S34496[80], 6}, {NULL}};
const Segment SGI151016[] = {{&S39396[44], 20}, {NULL}};
const Segment SGI151017[] = {{&S39396[44], 41}, {NULL}};
const Segment SGI151018[] = {{&S39396[30], 6}, {NULL}};
const Segment SGI151019[] = {{&S39396[134], 3}, {NULL}};
const Segment SGI151020[] = {{&S39396[99], 6}, {NULL}};
const Segment SGI151021[] = {{&S39397[38], 18}, {NULL}};
const Segment SGI151022[] = {{&S39397[31], 6}, {NULL}};
const Segment SGI151023[] = {{&S34499[44], 10}, {NULL}};
const Segment SGI151024[] = {{&S34499[44], 22}, {NULL}};
const Segment SGI151025[] = {{&S34499[30], 6}, {NULL}};
const Segment SGI151026[] = {{&S34499[115], 3}, {NULL}};
const Segment SGI151027[] = {{&S34499[80], 6}, {NULL}};
const Segment SGI151028[] = {{&S34504[44], 10}, {NULL}};
const Segment SGI151029[] = {{&S34504[44], 22}, {NULL}};
const Segment SGI151030[] = {{&S34504[30], 6}, {NULL}};
const Segment SGI151031[] = {{&S34504[115], 3}, {NULL}};
const Segment SGI151032[] = {{&S34504[80], 6}, {NULL}};
const Segment SGI151033[] = {{&S34506[38], 10}, {NULL}};
const Segment SGI151034[] = {{&S34506[31], 6}, {NULL}};
const Segment SGI151035[] = {{&S34509[38], 10}, {NULL}};
const Segment SGI151036[] = {{&S34509[31], 6}, {NULL}};
const Segment SGI151037[] = {{&S48713[34], 10}, {NULL}};
const Segment SGI151038[] = {{&S48713[27], 6}, {NULL}};
const Segment SGI151039[] = {{&S48713[97], 9}, {NULL}};
const Segment SGI151040[] = {{&S48713[70], 6}, {NULL}};
const Segment SGI151041[] = {{&S48714[34], 11}, {NULL}};
const Segment SGI151042[] = {{&S48714[27], 6}, {NULL}};
const Segment SGI151043[] = {{&S48714[98], 9}, {NULL}};
const Segment SGI151044[] = {{&S48714[71], 6}, {NULL}};
const Segment SGI151045[] = {{&S48716[34], 18}, {NULL}};
const Segment SGI151046[] = {{&S48716[27], 6}, {NULL}};
const Segment SGI151047[] = {{&S48716[105], 9}, {NULL}};
const Segment SGI151048[] = {{&S48716[78], 6}, {NULL}};
const Segment SGI151049[] = {{&S48723[44], 24}, {NULL}};
const Segment SGI151050[] = {{&S48723[44], 36}, {NULL}};
const Segment SGI151051[] = {{&S48723[30], 6}, {NULL}};
const Segment SGI151052[] = {{&S36989[32], 14}, {NULL}};
const Segment SGI151053[] = {{&S36989[27], 4}, {NULL}};
const Segment SGI151054[] = {{&S48723[31], 3}, {NULL}};
const Segment SGI151055[] = {{&S48723[94], 6}, {NULL}};
const Segment SGI151056[] = {{&S36989[99], 9}, {NULL}};
const Segment SGI151057[] = {{&S36989[72], 6}, {NULL}};
const Segment SGI151058[] = {{&S48724[44], 23}, {NULL}};
const Segment SGI151059[] = {{&S48724[44], 35}, {NULL}};
const Segment SGI151060[] = {{&S48724[30], 6}, {NULL}};
const Segment SGI151061[] = {{&S48724[31], 3}, {NULL}};
const Segment SGI151062[] = {{&S48724[93], 6}, {NULL}};
const Segment SGI151063[] = {{&S48725[38], 24}, {NULL}};
const Segment SGI151064[] = {{&S48725[31], 6}, {NULL}};
const Segment SGI151065[] = {{&S48727[34], 10}, {NULL}};
const Segment SGI151066[] = {{&S48727[27], 6}, {NULL}};
const Segment SGI151067[] = {{&S48727[97], 9}, {NULL}};
const Segment SGI151068[] = {{&S48727[70], 6}, {NULL}};
const Segment SGI151069[] = {{&S48728[34], 10}, {NULL}};
const Segment SGI151070[] = {{&S48728[27], 6}, {NULL}};
const Segment SGI151071[] = {{&S48728[97], 9}, {NULL}};
const Segment SGI151072[] = {{&S48728[70], 6}, {NULL}};
const Segment SGI151073[] = {{&S48736[34], 10}, {NULL}};
const Segment SGI151074[] = {{&S38906[34], 7}, {NULL}};
const Segment SGI151075[] = {{&S48736[27], 6}, {NULL}};
const Segment SGI151076[] = {{&S38906[27], 6}, {NULL}};
const Segment SGI151077[] = {{&S48736[97], 9}, {NULL}};
const Segment SGI151078[] = {{&S48736[70], 6}, {NULL}};
const Segment SGI151079[] = {{&S38906[104], 9}, {NULL}};
const Segment SGI151080[] = {{&S38906[77], 6}, {NULL}};
const Segment SGI151081[] = {{&S48737[34], 10}, {NULL}};
const Segment SGI151082[] = {{&S48737[27], 6}, {NULL}};
const Segment SGI151083[] = {{&S48737[97], 9}, {NULL}};
const Segment SGI151084[] = {{&S48737[70], 6}, {NULL}};
const Segment SGI151085[] = {{&S48744[34], 41}, {NULL}};
const Segment SGI151086[] = {{&S48744[27], 6}, {NULL}};
const Segment SGI151087[] = {{&S48744[128], 9}, {NULL}};
const Segment SGI151088[] = {{&S48744[101], 6}, {NULL}};
const Segment SGI151089[] = {{&S48761[34], 9}, {NULL}};
const Segment SGI151090[] = {{&S48761[27], 6}, {NULL}};
const Segment SGI151091[] = {{&S48761[96], 9}, {NULL}};
const Segment SGI151092[] = {{&S48761[69], 6}, {NULL}};
const Segment SGI151093[] = {{&S48762[34], 9}, {NULL}};
const Segment SGI151094[] = {{&S48762[27], 6}, {NULL}};
const Segment SGI151095[] = {{&S48762[96], 9}, {NULL}};
const Segment SGI151096[] = {{&S48762[69], 6}, {NULL}};
const Segment SGI151097[] = {{&S48763[34], 9}, {NULL}};
const Segment SGI151098[] = {{&S48763[27], 6}, {NULL}};
const Segment SGI151099[] = {{&S48763[96], 9}, {NULL}};
const Segment SGI151100[] = {{&S48763[69], 6}, {NULL}};
const Segment SGI151101[] = {{&S48764[34], 9}, {NULL}};
const Segment SGI151102[] = {{&S48764[27], 6}, {NULL}};
const Segment SGI151103[] = {{&S48764[96], 9}, {NULL}};
const Segment SGI151104[] = {{&S48764[69], 6}, {NULL}};
const Segment SGI151105[] = {{&S48787[34], 11}, {NULL}};
const Segment SGI151106[] = {{&S48787[27], 6}, {NULL}};
const Segment SGI151107[] = {{&S48787[98], 9}, {NULL}};
const Segment SGI151108[] = {{&S48787[71], 6}, {NULL}};
const Segment SGI151109[] = {{&S48788[34], 18}, {NULL}};
const Segment SGI151110[] = {{&S48788[27], 6}, {NULL}};
const Segment SGI151111[] = {{&S48788[105], 9}, {NULL}};
const Segment SGI151112[] = {{&S48788[78], 6}, {NULL}};
const Segment SGI151113[] = {{&S48789[34], 11}, {NULL}};
const Segment SGI151114[] = {{&S48789[27], 6}, {NULL}};
const Segment SGI151115[] = {{&S48789[98], 9}, {NULL}};
const Segment SGI151116[] = {{&S48789[71], 6}, {NULL}};
const Segment SGI151117[] = {{&S38977[44], 10}, {NULL}};
const Segment SGI151118[] = {{&S38977[44], 27}, {NULL}};
const Segment SGI151119[] = {{&S38977[30], 6}, {NULL}};
const Segment SGI151120[] = {{&S48790[34], 11}, {NULL}};
const Segment SGI151121[] = {{&S48790[27], 6}, {NULL}};
const Segment SGI151122[] = {{&S38977[119], 3}, {NULL}};
const Segment SGI151123[] = {{&S38977[85], 5}, {NULL}};
const Segment SGI151124[] = {{&S48790[98], 9}, {NULL}};
const Segment SGI151125[] = {{&S48790[71], 6}, {NULL}};
const Segment SGI151126[] = {{&S38978[44], 10}, {NULL}};
const Segment SGI151127[] = {{&S38978[44], 22}, {NULL}};
const Segment SGI151128[] = {{&S38978[30], 6}, {NULL}};
const Segment SGI151129[] = {{&S48791[34], 10}, {NULL}};
const Segment SGI151130[] = {{&S38978[115], 3}, {NULL}};
const Segment SGI151131[] = {{&S48791[27], 6}, {NULL}};
const Segment SGI151132[] = {{&S38978[80], 6}, {NULL}};
const Segment SGI151133[] = {{&S48791[97], 9}, {NULL}};
const Segment SGI151134[] = {{&S48791[70], 6}, {NULL}};
const Segment SGI151135[] = {{&S38979[38], 10}, {NULL}};
const Segment SGI151136[] = {{&S38979[31], 6}, {NULL}};
const Segment SGI151137[] = {{&S48792[34], 11}, {NULL}};
const Segment SGI151138[] = {{&S48792[27], 6}, {NULL}};
const Segment SGI151139[] = {{&S48792[98], 9}, {NULL}};
const Segment SGI151140[] = {{&S48792[71], 6}, {NULL}};
const Segment SGI151141[] = {{&S38980[44], 11}, {NULL}};
const Segment SGI151142[] = {{&S38980[44], 23}, {NULL}};
const Segment SGI151143[] = {{&S38980[30], 6}, {NULL}};
const Segment SGI151144[] = {{&S38980[116], 3}, {NULL}};
const Segment SGI151145[] = {{&S38980[81], 6}, {NULL}};
const Segment SGI151146[] = {{&S48793[34], 11}, {NULL}};
const Segment SGI151147[] = {{&S48793[27], 6}, {NULL}};
const Segment SGI151148[] = {{&S38981[44], 29}, {NULL}};
const Segment SGI151149[] = {{&S48793[98], 9}, {NULL}};
const Segment SGI151150[] = {{&S38981[44], 41}, {NULL}};
const Segment SGI151151[] = {{&S48793[71], 6}, {NULL}};
const Segment SGI151152[] = {{&S38981[30], 6}, {NULL}};
const Segment SGI151153[] = {{&S38981[134], 3}, {NULL}};
const Segment SGI151154[] = {{&S38981[99], 6}, {NULL}};
const Segment SGI151155[] = {{&S48794[34], 11}, {NULL}};
const Segment SGI151156[] = {{&S48794[27], 6}, {NULL}};
const Segment SGI151157[] = {{&S48794[98], 9}, {NULL}};
const Segment SGI151158[] = {{&S38982[38], 11}, {NULL}};
const Segment SGI151159[] = {{&S48794[71], 6}, {NULL}};
const Segment SGI151160[] = {{&S38982[31], 6}, {NULL}};
const Segment SGI151161[] = {{&S38983[42], 10}, {NULL}};
const Segment SGI151162[] = {{&S38983[30], 6}, {NULL}};
const Segment SGI151163[] = {{&S38983[113], 3}, {NULL}};
const Segment SGI151164[] = {{&S38983[78], 6}, {NULL}};
const Segment SGI151165[] = {{&S48812[44], 7}, {NULL}};
const Segment SGI151166[] = {{&S48812[44], 19}, {NULL}};
const Segment SGI151167[] = {{&S48812[30], 6}, {NULL}};
const Segment SGI151168[] = {{&S48812[31], 3}, {NULL}};
const Segment SGI151169[] = {{&S48812[77], 6}, {NULL}};
const Segment SGI151170[] = {{&S48813[38], 7}, {NULL}};
const Segment SGI151171[] = {{&S48813[31], 6}, {NULL}};
const Segment SGI151172[] = {{&S48823[34], 8}, {NULL}};
const Segment SGI151173[] = {{&S48823[27], 6}, {NULL}};
const Segment SGI151174[] = {{&S48823[100], 9}, {NULL}};
const Segment SGI151175[] = {{&S48823[73], 6}, {NULL}};
const Segment SGI151176[] = {{&S48824[34], 7}, {NULL}};
const Segment SGI151177[] = {{&S48824[27], 6}, {NULL}};
const Segment SGI151178[] = {{&S48824[97], 9}, {NULL}};
const Segment SGI151179[] = {{&S48824[70], 6}, {NULL}};
const Segment SGI151180[] = {{&S48825[34], 7}, {NULL}};
const Segment SGI151181[] = {{&S48825[27], 6}, {NULL}};
const Segment SGI151182[] = {{&S48825[98], 9}, {NULL}};
const Segment SGI151183[] = {{&S48825[71], 6}, {NULL}};
const Segment SGI151184[] = {{&S48852[34], 18}, {NULL}};
const Segment SGI151185[] = {{&S48852[27], 6}, {NULL}};
const Segment SGI151186[] = {{&S48852[113], 9}, {NULL}};
const Segment SGI151187[] = {{&S48852[86], 6}, {NULL}};
const Segment SGI151188[] = {{&S48853[34], 6}, {NULL}};
const Segment SGI151189[] = {{&S48853[27], 6}, {NULL}};
const Segment SGI151190[] = {{&S48853[101], 9}, {NULL}};
const Segment SGI151191[] = {{&S48853[74], 6}, {NULL}};
const Segment SGI151192[] = {{&S48858[34], 21}, {NULL}};
const Segment SGI151193[] = {{&S48858[27], 6}, {NULL}};
const Segment SGI151194[] = {{&S48858[108], 9}, {NULL}};
const Segment SGI151195[] = {{&S48858[81], 6}, {NULL}};
const Segment SGI151196[] = {{&S48860[34], 18}, {NULL}};
const Segment SGI151197[] = {{&S48860[27], 6}, {NULL}};
const Segment SGI151198[] = {{&S48860[105], 9}, {NULL}};
const Segment SGI151199[] = {{&S48860[78], 6}, {NULL}};
const Segment SGI151200[] = {{&S48870[34], 10}, {NULL}};
const Segment SGI151201[] = {{&S48870[27], 6}, {NULL}};
const Segment SGI151202[] = {{&S48870[97], 9}, {NULL}};
const Segment SGI151203[] = {{&S48870[70], 6}, {NULL}};
const Segment SGI151204[] = {{&S48872[34], 11}, {NULL}};
const Segment SGI151205[] = {{&S48872[27], 6}, {NULL}};
const Segment SGI151206[] = {{&S48872[98], 9}, {NULL}};
const Segment SGI151207[] = {{&S48872[71], 6}, {NULL}};
const Segment SGI151208[] = {{&S34662[34], 14}, {NULL}};
const Segment SGI151209[] = {{&S34662[27], 6}, {NULL}};
const Segment SGI151210[] = {{&S34662[101], 9}, {NULL}};
const Segment SGI151211[] = {{&S34662[74], 6}, {NULL}};
const Segment SGI151212[] = {{&S39138[34], 7}, {NULL}};
const Segment SGI151213[] = {{&S39138[27], 6}, {NULL}};
const Segment SGI151214[] = {{&S39138[94], 9}, {NULL}};
const Segment SGI151215[] = {{&S39138[67], 6}, {NULL}};
const Segment SGI151216[] = {{&S39139[34], 12}, {NULL}};
const Segment SGI151217[] = {{&S39139[27], 6}, {NULL}};
const Segment SGI151218[] = {{&S39139[109], 9}, {NULL}};
const Segment SGI151219[] = {{&S39139[82], 6}, {NULL}};
const Segment SGI151220[] = {{&S39140[34], 12}, {NULL}};
const Segment SGI151221[] = {{&S39140[27], 6}, {NULL}};
const Segment SGI151222[] = {{&S39140[109], 9}, {NULL}};
const Segment SGI151223[] = {{&S39140[82], 6}, {NULL}};
const Segment SGI151224[] = {{&S39158[44], 10}, {NULL}};
const Segment SGI151225[] = {{&S39158[44], 22}, {NULL}};
const Segment SGI151226[] = {{&S39158[30], 6}, {NULL}};
const Segment SGI151227[] = {{&S39158[115], 3}, {NULL}};
const Segment SGI151228[] = {{&S39158[80], 6}, {NULL}};
const Segment SGI151229[] = {{&S39159[38], 10}, {NULL}};
const Segment SGI151230[] = {{&S39159[31], 6}, {NULL}};
const Segment SGI151231[] = {{&S39160[42], 10}, {NULL}};
const Segment SGI151232[] = {{&S39160[30], 6}, {NULL}};
const Segment SGI151233[] = {{&S39160[113], 3}, {NULL}};
const Segment SGI151234[] = {{&S39160[78], 6}, {NULL}};
const Segment SGI151235[] = {{&S39161[44], 12}, {NULL}};
const Segment SGI151236[] = {{&S39161[44], 24}, {NULL}};
const Segment SGI151237[] = {{&S39161[30], 6}, {NULL}};
const Segment SGI151238[] = {{&S39161[117], 3}, {NULL}};
const Segment SGI151239[] = {{&S39161[82], 6}, {NULL}};
const Segment SGI151240[] = {{&S39163[44], 11}, {NULL}};
const Segment SGI151241[] = {{&S39163[44], 23}, {NULL}};
const Segment SGI151242[] = {{&S39163[30], 6}, {NULL}};
const Segment SGI151243[] = {{&S39163[116], 3}, {NULL}};
const Segment SGI151244[] = {{&S39163[81], 6}, {NULL}};
const Segment SGI151245[] = {{&S39165[38], 11}, {NULL}};
const Segment SGI151246[] = {{&S39165[31], 6}, {NULL}};
const Segment SGI151247[] = {{&S39740[44], 18}, {NULL}};
const Segment SGI151248[] = {{&S39740[44], 30}, {NULL}};
const Segment SGI151249[] = {{&S39740[30], 6}, {NULL}};
const Segment SGI151250[] = {{&S39740[123], 3}, {NULL}};
const Segment SGI151251[] = {{&S39740[88], 6}, {NULL}};
const Segment SGI151252[] = {{&S37369[32], 9}, {NULL}};
const Segment SGI151253[] = {{&S37369[27], 4}, {NULL}};
const Segment SGI151254[] = {{&S37369[94], 9}, {NULL}};
const Segment SGI151255[] = {{&S37369[67], 6}, {NULL}};
const Segment SGI151256[] = {{&S39255[44], 9}, {NULL}};
const Segment SGI151257[] = {{&S39255[44], 21}, {NULL}};
const Segment SGI151258[] = {{&S39255[30], 6}, {NULL}};
const Segment SGI151259[] = {{&S39255[114], 3}, {NULL}};
const Segment SGI151260[] = {{&S39255[79], 6}, {NULL}};
const Segment SGI151261[] = {{&S39257[38], 9}, {NULL}};
const Segment SGI151262[] = {{&S39257[31], 6}, {NULL}};
const Segment SGI151263[] = {{&S39278[44], 9}, {NULL}};
const Segment SGI151264[] = {{&S39278[44], 68}, {NULL}};
const Segment SGI151265[] = {{&S39278[30], 6}, {NULL}};
const Segment SGI151266[] = {{&S39279[44], 9}, {NULL}};
const Segment SGI151267[] = {{&S39279[44], 75}, {NULL}};
const Segment SGI151268[] = {{&S39279[30], 6}, {NULL}};
const Segment SGI151269[] = {{&S39282[30], 1}, {NULL}};
const Segment SGI151270[] = {{&S39282[63], 8}, {NULL}};
const Segment SGI151271[] = {{&S39282[49], 6}, {NULL}};
const Segment SGI151272[] = {{&S39284[64], 9}, {NULL}};
const Segment SGI151273[] = {{&S39284[64], 21}, {NULL}};
const Segment SGI151274[] = {{&S39284[50], 6}, {NULL}};
const Segment SGI151275[] = {{&S48955[44], 11}, {NULL}};
const Segment SGI151276[] = {{&S48955[44], 41}, {NULL}};
const Segment SGI151277[] = {{&S48955[30], 6}, {NULL}};
const Segment SGI151278[] = {{&S48955[31], 3}, {NULL}};
const Segment SGI151279[] = {{&S48955[99], 6}, {NULL}};
const Segment SGI151280[] = {{&S48956[44], 11}, {NULL}};
const Segment SGI151281[] = {{&S48956[44], 48}, {NULL}};
const Segment SGI151282[] = {{&S48956[30], 6}, {NULL}};
const Segment SGI151283[] = {{&S48956[31], 3}, {NULL}};
const Segment SGI151284[] = {{&S48956[106], 6}, {NULL}};
const Segment SGI151285[] = {{&S39288[32], 6}, {NULL}};
const Segment SGI151286[] = {{&S48957[44], 11}, {NULL}};
const Segment SGI151287[] = {{&S48957[44], 28}, {NULL}};
const Segment SGI151288[] = {{&S48957[30], 6}, {NULL}};
const Segment SGI151289[] = {{&S48957[31], 3}, {NULL}};
const Segment SGI151290[] = {{&S48957[86], 6}, {NULL}};
const Segment SGI151291[] = {{&S48959[44], 11}, {NULL}};
const Segment SGI151292[] = {{&S48959[44], 41}, {NULL}};
const Segment SGI151293[] = {{&S48959[30], 6}, {NULL}};
const Segment SGI151294[] = {{&S48959[31], 3}, {NULL}};
const Segment SGI151295[] = {{&S48959[99], 6}, {NULL}};
const Segment SGI151296[] = {{&S48960[44], 11}, {NULL}};
const Segment SGI151297[] = {{&S48960[44], 28}, {NULL}};
const Segment SGI151298[] = {{&S48960[30], 6}, {NULL}};
const Segment SGI151299[] = {{&S48960[31], 3}, {NULL}};
const Segment SGI151300[] = {{&S48960[86], 6}, {NULL}};
const Segment SGI151301[] = {{&S48961[44], 11}, {NULL}};
const Segment SGI151302[] = {{&S48961[44], 48}, {NULL}};
const Segment SGI151303[] = {{&S48961[30], 6}, {NULL}};
const Segment SGI151304[] = {{&S48961[31], 3}, {NULL}};
const Segment SGI151305[] = {{&S39297[44], 9}, {NULL}};
const Segment SGI151306[] = {{&S39297[44], 20}, {NULL}};
const Segment SGI151307[] = {{&S48961[106], 6}, {NULL}};
const Segment SGI151308[] = {{&S39297[30], 6}, {NULL}};
const Segment SGI151309[] = {{&S39297[113], 3}, {NULL}};
const Segment SGI151310[] = {{&S39297[78], 6}, {NULL}};
const Segment SGI151311[] = {{&S48962[44], 11}, {NULL}};
const Segment SGI151312[] = {{&S48962[44], 28}, {NULL}};
const Segment SGI151313[] = {{&S48962[30], 6}, {NULL}};
const Segment SGI151314[] = {{&S48962[31], 3}, {NULL}};
const Segment SGI151315[] = {{&S48962[86], 6}, {NULL}};
const Segment SGI151316[] = {{&S48963[44], 11}, {NULL}};
const Segment SGI151317[] = {{&S48963[44], 41}, {NULL}};
const Segment SGI151318[] = {{&S48963[30], 6}, {NULL}};
const Segment SGI151319[] = {{&S48963[31], 3}, {NULL}};
const Segment SGI151320[] = {{&S48963[99], 6}, {NULL}};
const Segment SGI151321[] = {{&S39301[44], 9}, {NULL}};
const Segment SGI151322[] = {{&S39301[44], 21}, {NULL}};
const Segment SGI151323[] = {{&S39301[30], 6}, {NULL}};
const Segment SGI151324[] = {{&S39301[114], 3}, {NULL}};
const Segment SGI151325[] = {{&S39301[79], 6}, {NULL}};
const Segment SGI151326[] = {{&S48964[44], 11}, {NULL}};
const Segment SGI151327[] = {{&S48964[44], 28}, {NULL}};
const Segment SGI151328[] = {{&S48964[30], 6}, {NULL}};
const Segment SGI151329[] = {{&S48964[31], 3}, {NULL}};
const Segment SGI151330[] = {{&S48964[86], 6}, {NULL}};
const Segment SGI151331[] = {{&S39302[44], 9}, {NULL}};
const Segment SGI151332[] = {{&S39302[44], 21}, {NULL}};
const Segment SGI151333[] = {{&S39302[30], 6}, {NULL}};
const Segment SGI151334[] = {{&S39302[114], 3}, {NULL}};
const Segment SGI151335[] = {{&S39302[79], 6}, {NULL}};
const Segment SGI151336[] = {{&S48965[44], 11}, {NULL}};
const Segment SGI151337[] = {{&S48965[44], 28}, {NULL}};
const Segment SGI151338[] = {{&S48965[30], 6}, {NULL}};
const Segment SGI151339[] = {{&S48965[31], 3}, {NULL}};
const Segment SGI151340[] = {{&S48965[86], 6}, {NULL}};
const Segment SGI151341[] = {{&S39303[44], 9}, {&S39303[174], 14}, {NULL}};
const Segment SGI151342[] = {{&S39303[44], 21}, {NULL}};
const Segment SGI151343[] = {{&S39303[30], 6}, {NULL}};
const Segment SGI151344[] = {{&S39303[114], 3}, {NULL}};
const Segment SGI151345[] = {{&S39303[79], 6}, {NULL}};
const Segment SGI151346[] = {{&S48966[44], 11}, {NULL}};
const Segment SGI151347[] = {{&S48966[44], 28}, {NULL}};
const Segment SGI151348[] = {{&S48966[30], 6}, {NULL}};
const Segment SGI151349[] = {{&S48966[31], 3}, {NULL}};
const Segment SGI151350[] = {{&S48966[86], 6}, {NULL}};
const Segment SGI151351[] = {{&S39304[44], 9}, {NULL}};
const Segment SGI151352[] = {{&S39304[44], 21}, {NULL}};
const Segment SGI151353[] = {{&S39304[30], 6}, {NULL}};
const Segment SGI151354[] = {{&S39304[82], 6}, {NULL}};
const Segment SGI151355[] = {{&S48968[44], 11}, {NULL}};
const Segment SGI151356[] = {{&S48968[44], 41}, {NULL}};
const Segment SGI151357[] = {{&S48968[30], 6}, {NULL}};
const Segment SGI151358[] = {{&S48968[31], 3}, {NULL}};
const Segment SGI151359[] = {{&S39306[44], 9}, {NULL}};
const Segment SGI151360[] = {{&S48968[99], 6}, {NULL}};
const Segment SGI151361[] = {{&S39306[44], 21}, {NULL}};
const Segment SGI151362[] = {{&S39306[30], 6}, {NULL}};
const Segment SGI151363[] = {{&S39306[114], 3}, {NULL}};
const Segment SGI151364[] = {{&S39306[79], 6}, {NULL}};
const Segment SGI151365[] = {{&S48969[44], 11}, {NULL}};
const Segment SGI151366[] = {{&S48969[44], 32}, {NULL}};
const Segment SGI151367[] = {{&S48969[30], 6}, {NULL}};
const Segment SGI151368[] = {{&S48969[31], 3}, {NULL}};
const Segment SGI151369[] = {{&S39307[42], 9}, {NULL}};
const Segment SGI151370[] = {{&S39307[30], 6}, {NULL}};
const Segment SGI151371[] = {{&S48969[90], 6}, {NULL}};
const Segment SGI151372[] = {{&S39307[112], 3}, {NULL}};
const Segment SGI151373[] = {{&S39307[77], 6}, {NULL}};
const Segment SGI151374[] = {{&S48970[44], 11}, {NULL}};
const Segment SGI151375[] = {{&S48970[44], 41}, {NULL}};
const Segment SGI151376[] = {{&S48970[30], 6}, {NULL}};
const Segment SGI151377[] = {{&S48970[31], 3}, {NULL}};
const Segment SGI151378[] = {{&S48970[99], 6}, {NULL}};
const Segment SGI151379[] = {{&S39308[44], 9}, {NULL}};
const Segment SGI151380[] = {{&S39308[44], 21}, {&S39308[211], 7}, {NULL}};
const Segment SGI151381[] = {{&S39308[30], 6}, {NULL}};
const Segment SGI151382[] = {{&S39308[114], 3}, {NULL}};
const Segment SGI151383[] = {{&S39308[79], 6}, {NULL}};
const Segment SGI151384[] = {{&S48971[44], 11}, {NULL}};
const Segment SGI151385[] = {{&S48971[44], 28}, {NULL}};
const Segment SGI151386[] = {{&S48971[30], 6}, {NULL}};
const Segment SGI151387[] = {{&S48971[31], 3}, {NULL}};
const Segment SGI151388[] = {{&S48971[86], 6}, {NULL}};
const Segment SGI151389[] = {{&S39309[44], 9}, {NULL}};
const Segment SGI151390[] = {{&S39309[44], 21}, {NULL}};
const Segment SGI151391[] = {{&S39309[30], 6}, {NULL}};
const Segment SGI151392[] = {{&S39309[79], 6}, {NULL}};
const Segment SGI151393[] = {{&S48972[44], 11}, {NULL}};
const Segment SGI151394[] = {{&S48972[44], 28}, {NULL}};
const Segment SGI151395[] = {{&S48972[30], 6}, {NULL}};
const Segment SGI151396[] = {{&S48972[31], 3}, {NULL}};
const Segment SGI151397[] = {{&S48972[86], 6}, {NULL}};
const Segment SGI151398[] = {{&S39311[44], 9}, {NULL}};
const Segment SGI151399[] = {{&S39311[44], 41}, {NULL}};
const Segment SGI151400[] = {{&S39311[30], 6}, {NULL}};
const Segment SGI151401[] = {{&S48973[44], 11}, {NULL}};
const Segment SGI151402[] = {{&S48973[44], 26}, {NULL}};
const Segment SGI151403[] = {{&S48973[30], 6}, {NULL}};
const Segment SGI151404[] = {{&S39311[134], 3}, {NULL}};
const Segment SGI151405[] = {{&S39311[99], 6}, {NULL}};
const Segment SGI151406[] = {{&S48973[31], 3}, {NULL}};
const Segment SGI151407[] = {{&S48973[84], 6}, {NULL}};
const Segment SGI151408[] = {{&S34874[38], 10}, {NULL}};
const Segment SGI151409[] = {{&S34874[31], 6}, {NULL}};
const Segment SGI151410[] = {{&S48974[44], 11}, {NULL}};
const Segment SGI151411[] = {{&S48974[44], 28}, {NULL}};
const Segment SGI151412[] = {{&S48974[30], 6}, {NULL}};
const Segment SGI151413[] = {{&S39313[44], 9}, {NULL}};
const Segment SGI151414[] = {{&S39313[44], 21}, {NULL}};
const Segment SGI151415[] = {{&S48974[31], 3}, {NULL}};
const Segment SGI151416[] = {{&S39313[30], 6}, {NULL}};
const Segment SGI151417[] = {{&S48974[86], 6}, {NULL}};
const Segment SGI151418[] = {{&S39313[115], 3}, {NULL}};
const Segment SGI151419[] = {{&S39313[79], 7}, {NULL}};
const Segment SGI151420[] = {{&S39313[135], 8}, {NULL}};
const Segment SGI151421[] = {{&S48975[44], 11}, {NULL}};
const Segment SGI151422[] = {{&S48975[44], 28}, {NULL}};
const Segment SGI151423[] = {{&S48975[30], 6}, {NULL}};
const Segment SGI151424[] = {{&S48975[31], 3}, {NULL}};
const Segment SGI151425[] = {{&S48975[86], 6}, {NULL}};
const Segment SGI151426[] = {{&S39315[53], 14}, {NULL}};
const Segment SGI151427[] = {{&S39315[46], 6}, {NULL}};
const Segment SGI151428[] = {{&S48976[44], 11}, {NULL}};
const Segment SGI151429[] = {{&S48976[44], 28}, {NULL}};
const Segment SGI151430[] = {{&S48976[30], 6}, {NULL}};
const Segment SGI151431[] = {{&S48976[31], 3}, {NULL}};
const Segment SGI151432[] = {{&S48976[86], 6}, {NULL}};
const Segment SGI151433[] = {{&S34879[44], 10}, {NULL}};
const Segment SGI151434[] = {{&S34879[44], 22}, {NULL}};
const Segment SGI151435[] = {{&S34879[30], 6}, {NULL}};
const Segment SGI151436[] = {{&S34879[115], 3}, {NULL}};
const Segment SGI151437[] = {{&S34879[80], 6}, {NULL}};
const Segment SGI151438[] = {{&S48977[44], 11}, {NULL}};
const Segment SGI151439[] = {{&S48977[44], 28}, {NULL}};
const Segment SGI151440[] = {{&S48977[30], 6}, {NULL}};
const Segment SGI151441[] = {{&S48977[31], 3}, {NULL}};
const Segment SGI151442[] = {{&S48977[86], 6}, {NULL}};
const Segment SGI151443[] = {{&S48978[38], 7}, {NULL}};
const Segment SGI151444[] = {{&S48978[31], 6}, {NULL}};
const Segment SGI151445[] = {{&S48979[44], 7}, {NULL}};
const Segment SGI151446[] = {{&S48979[44], 24}, {NULL}};
const Segment SGI151447[] = {{&S48979[30], 6}, {NULL}};
const Segment SGI151448[] = {{&S48979[31], 3}, {NULL}};
const Segment SGI151449[] = {{&S48979[82], 6}, {NULL}};
const Segment SGI151450[] = {{&S48980[44], 7}, {NULL}};
const Segment SGI151451[] = {{&S48980[44], 24}, {NULL}};
const Segment SGI151452[] = {{&S48980[30], 6}, {NULL}};
const Segment SGI151453[] = {{&S48980[85], 6}, {NULL}};
const Segment SGI151454[] = {{&S34888[44], 10}, {NULL}};
const Segment SGI151455[] = {{&S34888[44], 22}, {NULL}};
const Segment SGI151456[] = {{&S34888[30], 6}, {NULL}};
const Segment SGI151457[] = {{&S48981[44], 7}, {NULL}};
const Segment SGI151458[] = {{&S34888[115], 3}, {NULL}};
const Segment SGI151459[] = {{&S48981[44], 24}, {NULL}};
const Segment SGI151460[] = {{&S48981[30], 6}, {NULL}};
const Segment SGI151461[] = {{&S34888[80], 6}, {NULL}};
const Segment SGI151462[] = {{&S48981[31], 3}, {NULL}};
const Segment SGI151463[] = {{&S48981[82], 6}, {NULL}};
const Segment SGI151464[] = {{&S48982[44], 7}, {NULL}};
const Segment SGI151465[] = {{&S48982[44], 24}, {NULL}};
const Segment SGI151466[] = {{&S48982[30], 6}, {NULL}};
const Segment SGI151467[] = {{&S48982[31], 3}, {NULL}};
const Segment SGI151468[] = {{&S48982[82], 6}, {NULL}};
const Segment SGI151469[] = {{&S48983[44], 7}, {NULL}};
const Segment SGI151470[] = {{&S48983[44], 19}, {NULL}};
const Segment SGI151471[] = {{&S48983[30], 6}, {NULL}};
const Segment SGI151472[] = {{&S48983[31], 3}, {NULL}};
const Segment SGI151473[] = {{&S48983[77], 6}, {NULL}};
const Segment SGI151474[] = {{&S48984[44], 11}, {NULL}};
const Segment SGI151475[] = {{&S48984[44], 28}, {NULL}};
const Segment SGI151476[] = {{&S48984[30], 6}, {NULL}};
const Segment SGI151477[] = {{&S48984[31], 3}, {NULL}};
const Segment SGI151478[] = {{&S48984[86], 6}, {NULL}};
const Segment SGI151479[] = {{&S48985[44], 7}, {NULL}};
const Segment SGI151480[] = {{&S48985[44], 24}, {NULL}};
const Segment SGI151481[] = {{&S48985[30], 6}, {NULL}};
const Segment SGI151482[] = {{&S48985[31], 3}, {NULL}};
const Segment SGI151483[] = {{&S48985[82], 6}, {NULL}};
const Segment SGI151484[] = {{&S48999[34], 10}, {NULL}};
const Segment SGI151485[] = {{&S48999[27], 6}, {NULL}};
const Segment SGI151486[] = {{&S48999[97], 9}, {NULL}};
const Segment SGI151487[] = {{&S48999[70], 6}, {NULL}};
const Segment SGI151488[] = {{&S49000[34], 9}, {NULL}};
const Segment SGI151489[] = {{&S49000[27], 6}, {NULL}};
const Segment SGI151490[] = {{&S49000[96], 9}, {NULL}};
const Segment SGI151491[] = {{&S49000[69], 6}, {NULL}};
const Segment SGI151492[] = {{&S37374[32], 9}, {NULL}};
const Segment SGI151493[] = {{&S37374[27], 4}, {NULL}};
const Segment SGI151494[] = {{&S37374[94], 9}, {NULL}};
const Segment SGI151495[] = {{&S37374[67], 6}, {NULL}};
const Segment SGI151496[] = {{&S49105[34], 10}, {NULL}};
const Segment SGI151497[] = {{&S49105[27], 6}, {NULL}};
const Segment SGI151498[] = {{&S49105[97], 9}, {NULL}};
const Segment SGI151499[] = {{&S49105[70], 6}, {NULL}};
const Segment SGI151500[] = {{&S39944[34], 10}, {NULL}};
const Segment SGI151501[] = {{&S39944[27], 6}, {NULL}};
const Segment SGI151502[] = {{&S39944[97], 9}, {NULL}};
const Segment SGI151503[] = {{&S39944[70], 6}, {NULL}};
const Segment SGI151504[] = {{&S49178[34], 9}, {NULL}};
const Segment SGI151505[] = {{&S49178[27], 6}, {NULL}};
const Segment SGI151506[] = {{&S49178[96], 9}, {NULL}};
const Segment SGI151507[] = {{&S49178[69], 6}, {NULL}};
const Segment SGI151508[] = {{&S49217[44], 9}, {NULL}};
const Segment SGI151509[] = {{&S49217[44], 21}, {NULL}};
const Segment SGI151510[] = {{&S49217[30], 6}, {NULL}};
const Segment SGI151511[] = {{&S49217[31], 3}, {NULL}};
const Segment SGI151512[] = {{&S49217[79], 6}, {NULL}};
const Segment SGI151513[] = {{&S49218[44], 9}, {NULL}};
const Segment SGI151514[] = {{&S49218[44], 21}, {NULL}};
const Segment SGI151515[] = {{&S49218[30], 6}, {NULL}};
const Segment SGI151516[] = {{&S49218[31], 3}, {NULL}};
const Segment SGI151517[] = {{&S49218[79], 6}, {NULL}};
const Segment SGI151518[] = {{&S49220[38], 9}, {NULL}};
const Segment SGI151519[] = {{&S49220[31], 6}, {NULL}};
const Segment SGI151520[] = {{&S49221[44], 8}, {NULL}};
const Segment SGI151521[] = {{&S49221[44], 20}, {NULL}};
const Segment SGI151522[] = {{&S49221[30], 6}, {NULL}};
const Segment SGI151523[] = {{&S49221[31], 3}, {NULL}};
const Segment SGI151524[] = {{&S49221[78], 6}, {NULL}};
const Segment SGI151525[] = {{&S49222[44], 9}, {NULL}};
const Segment SGI151526[] = {{&S49222[44], 21}, {NULL}};
const Segment SGI151527[] = {{&S49222[30], 6}, {NULL}};
const Segment SGI151528[] = {{&S49222[31], 3}, {NULL}};
const Segment SGI151529[] = {{&S49222[79], 6}, {NULL}};
const Segment SGI151530[] = {{&S49223[44], 8}, {NULL}};
const Segment SGI151531[] = {{&S49223[44], 20}, {NULL}};
const Segment SGI151532[] = {{&S49223[30], 6}, {NULL}};
const Segment SGI151533[] = {{&S49223[31], 3}, {NULL}};
const Segment SGI151534[] = {{&S49223[78], 6}, {NULL}};
const Segment SGI151535[] = {{&S49224[38], 8}, {NULL}};
const Segment SGI151536[] = {{&S49224[31], 6}, {NULL}};
const Segment SGI151537[] = {{&S49225[38], 9}, {NULL}};
const Segment SGI151538[] = {{&S49225[31], 6}, {NULL}};
const Segment SGI151539[] = {{&S49226[44], 9}, {NULL}};
const Segment SGI151540[] = {{&S49226[44], 21}, {NULL}};
const Segment SGI151541[] = {{&S49226[30], 6}, {NULL}};
const Segment SGI151542[] = {{&S49226[31], 3}, {NULL}};
const Segment SGI151543[] = {{&S49226[79], 6}, {NULL}};
const Segment SGI151544[] = {{&S49227[44], 9}, {NULL}};
const Segment SGI151545[] = {{&S49227[44], 21}, {NULL}};
const Segment SGI151546[] = {{&S49227[30], 6}, {NULL}};
const Segment SGI151547[] = {{&S49227[31], 3}, {NULL}};
const Segment SGI151548[] = {{&S49227[79], 6}, {NULL}};
const Segment SGI151549[] = {{&S49228[44], 9}, {NULL}};
const Segment SGI151550[] = {{&S49228[44], 21}, {NULL}};
const Segment SGI151551[] = {{&S49228[30], 6}, {NULL}};
const Segment SGI151552[] = {{&S49228[31], 3}, {NULL}};
const Segment SGI151553[] = {{&S49228[79], 6}, {NULL}};
const Segment SGI151554[] = {{&S49234[34], 10}, {NULL}};
const Segment SGI151555[] = {{&S49234[27], 6}, {NULL}};
const Segment SGI151556[] = {{&S49234[97], 9}, {NULL}};
const Segment SGI151557[] = {{&S49234[70], 6}, {NULL}};
const Segment SGI151558[] = {{&S49235[34], 10}, {NULL}};
const Segment SGI151559[] = {{&S49235[27], 6}, {NULL}};
const Segment SGI151560[] = {{&S49235[97], 9}, {NULL}};
const Segment SGI151561[] = {{&S49235[70], 6}, {NULL}};
const Segment SGI151562[] = {{&S40065[34], 10}, {NULL}};
const Segment SGI151563[] = {{&S40065[27], 6}, {NULL}};
const Segment SGI151564[] = {{&S40065[97], 9}, {NULL}};
const Segment SGI151565[] = {{&S40065[70], 6}, {NULL}};
const Segment SGI151566[] = {{&S49252[34], 7}, {NULL}};
const Segment SGI151567[] = {{&S49252[27], 6}, {NULL}};
const Segment SGI151568[] = {{&S49252[106], 9}, {NULL}};
const Segment SGI151569[] = {{&S49252[79], 6}, {NULL}};
const Segment SGI151570[] = {{&S40070[34], 10}, {NULL}};
const Segment SGI151571[] = {{&S40070[27], 6}, {NULL}};
const Segment SGI151572[] = {{&S49253[34], 7}, {NULL}};
const Segment SGI151573[] = {{&S40070[97], 9}, {NULL}};
const Segment SGI151574[] = {{&S49253[27], 6}, {NULL}};
const Segment SGI151575[] = {{&S40070[70], 6}, {NULL}};
const Segment SGI151576[] = {{&S49253[106], 9}, {NULL}};
const Segment SGI151577[] = {{&S49253[79], 6}, {NULL}};
const Segment SGI151578[] = {{&S40071[34], 11}, {NULL}};
const Segment SGI151579[] = {{&S40071[27], 6}, {NULL}};
const Segment SGI151580[] = {{&S40071[98], 9}, {NULL}};
const Segment SGI151581[] = {{&S40071[71], 6}, {NULL}};
const Segment SGI151582[] = {{&S49254[34], 7}, {NULL}};
const Segment SGI151583[] = {{&S49254[27], 6}, {NULL}};
const Segment SGI151584[] = {{&S49254[106], 9}, {NULL}};
const Segment SGI151585[] = {{&S49254[79], 6}, {NULL}};
const Segment SGI151586[] = {{&S40075[34], 10}, {NULL}};
const Segment SGI151587[] = {{&S40075[27], 6}, {NULL}};
const Segment SGI151588[] = {{&S40075[97], 9}, {NULL}};
const Segment SGI151589[] = {{&S40075[70], 6}, {NULL}};
const Segment SGI151590[] = {{&S37536[32], 14}, {NULL}};
const Segment SGI151591[] = {{&S37536[27], 4}, {NULL}};
const Segment SGI151592[] = {{&S37536[99], 9}, {NULL}};
const Segment SGI151593[] = {{&S37536[72], 6}, {NULL}};
const Segment SGI151594[] = {{&S49310[44], 5}, {NULL}};
const Segment SGI151595[] = {{&S49310[44], 17}, {NULL}};
const Segment SGI151596[] = {{&S49310[30], 6}, {NULL}};
const Segment SGI151597[] = {{&S49310[31], 3}, {NULL}};
const Segment SGI151598[] = {{&S49310[75], 6}, {NULL}};
const Segment SGI151599[] = {{&S49311[44], 5}, {NULL}};
const Segment SGI151600[] = {{&S49311[44], 17}, {NULL}};
const Segment SGI151601[] = {{&S49311[30], 6}, {NULL}};
const Segment SGI151602[] = {{&S49311[31], 3}, {NULL}};
const Segment SGI151603[] = {{&S49311[75], 6}, {NULL}};
const Segment SGI151604[] = {{&S49312[38], 5}, {NULL}};
const Segment SGI151605[] = {{&S49312[31], 6}, {NULL}};
const Segment SGI151606[] = {{&S49313[44], 5}, {NULL}};
const Segment SGI151607[] = {{&S49313[44], 17}, {NULL}};
const Segment SGI151608[] = {{&S49313[30], 6}, {NULL}};
const Segment SGI151609[] = {{&S49313[31], 3}, {NULL}};
const Segment SGI151610[] = {{&S49313[75], 6}, {NULL}};
const Segment SGI151611[] = {{&S49314[44], 17}, {NULL}};
const Segment SGI151612[] = {{&S49314[44], 29}, {NULL}};
const Segment SGI151613[] = {{&S49314[30], 6}, {NULL}};
const Segment SGI151614[] = {{&S49314[31], 3}, {NULL}};
const Segment SGI151615[] = {{&S49314[87], 6}, {NULL}};
const Segment SGI151616[] = {{&S49315[44], 5}, {NULL}};
const Segment SGI151617[] = {{&S49315[44], 20}, {NULL}};
const Segment SGI151618[] = {{&S49315[30], 6}, {NULL}};
const Segment SGI151619[] = {{&S49315[31], 3}, {NULL}};
const Segment SGI151620[] = {{&S49315[78], 6}, {NULL}};
const Segment SGI151621[] = {{&S39741[44], 18}, {NULL}};
const Segment SGI151622[] = {{&S39741[44], 30}, {NULL}};
const Segment SGI151623[] = {{&S39741[30], 6}, {NULL}};
const Segment SGI151624[] = {{&S39741[123], 3}, {NULL}};
const Segment SGI151625[] = {{&S39741[88], 6}, {NULL}};
const Segment SGI151626[] = {{&S39742[38], 18}, {NULL}};
const Segment SGI151627[] = {{&S39742[31], 6}, {NULL}};
const Segment SGI151628[] = {{&S39743[44], 19}, {NULL}};
const Segment SGI151629[] = {{&S39743[44], 31}, {NULL}};
const Segment SGI151630[] = {{&S39743[30], 6}, {NULL}};
const Segment SGI151631[] = {{&S39743[124], 3}, {NULL}};
const Segment SGI151632[] = {{&S39743[89], 6}, {NULL}};
const Segment SGI151633[] = {{&S39744[44], 19}, {NULL}};
const Segment SGI151634[] = {{&S39744[44], 31}, {NULL}};
const Segment SGI151635[] = {{&S39744[30], 6}, {NULL}};
const Segment SGI151636[] = {{&S39744[124], 3}, {NULL}};
const Segment SGI151637[] = {{&S39744[89], 6}, {NULL}};
const Segment SGI151638[] = {{&S39745[38], 19}, {NULL}};
const Segment SGI151639[] = {{&S39745[31], 6}, {NULL}};
const Segment SGI151640[] = {{&S39747[44], 7}, {NULL}};
const Segment SGI151641[] = {{&S39747[44], 29}, {NULL}};
const Segment SGI151642[] = {{&S39747[30], 6}, {NULL}};
const Segment SGI151643[] = {{&S39747[122], 3}, {NULL}};
const Segment SGI151644[] = {{&S39747[87], 6}, {NULL}};
const Segment SGI151645[] = {{&S39748[44], 7}, {NULL}};
const Segment SGI151646[] = {{&S39748[44], 29}, {NULL}};
const Segment SGI151647[] = {{&S39748[30], 6}, {NULL}};
const Segment SGI151648[] = {{&S39748[122], 3}, {NULL}};
const Segment SGI151649[] = {{&S39748[87], 6}, {NULL}};
const Segment SGI151650[] = {{&S39749[38], 7}, {NULL}};
const Segment SGI151651[] = {{&S39749[31], 6}, {NULL}};
const Segment SGI151652[] = {{&S39750[44], 7}, {NULL}};
const Segment SGI151653[] = {{&S39750[44], 29}, {NULL}};
const Segment SGI151654[] = {{&S39750[30], 6}, {NULL}};
const Segment SGI151655[] = {{&S39750[122], 3}, {NULL}};
const Segment SGI151656[] = {{&S39750[87], 6}, {NULL}};
const Segment SGI151657[] = {{&S39751[44], 7}, {NULL}};
const Segment SGI151658[] = {{&S39751[44], 29}, {NULL}};
const Segment SGI151659[] = {{&S39751[30], 6}, {NULL}};
const Segment SGI151660[] = {{&S39751[122], 3}, {NULL}};
const Segment SGI151661[] = {{&S39751[87], 6}, {NULL}};
const Segment SGI151662[] = {{&S39752[44], 7}, {NULL}};
const Segment SGI151663[] = {{&S39752[44], 29}, {NULL}};
const Segment SGI151664[] = {{&S39752[30], 6}, {NULL}};
const Segment SGI151665[] = {{&S39752[122], 3}, {NULL}};
const Segment SGI151666[] = {{&S39752[87], 6}, {NULL}};
const Segment SGI151667[] = {{&S49391[34], 10}, {NULL}};
const Segment SGI151668[] = {{&S49391[27], 6}, {NULL}};
const Segment SGI151669[] = {{&S49391[97], 9}, {NULL}};
const Segment SGI151670[] = {{&S49391[70], 6}, {NULL}};
const Segment SGI151671[] = {{&S39753[44], 7}, {NULL}};
const Segment SGI151672[] = {{&S39753[44], 29}, {NULL}};
const Segment SGI151673[] = {{&S39753[30], 6}, {NULL}};
const Segment SGI151674[] = {{&S39753[122], 3}, {NULL}};
const Segment SGI151675[] = {{&S39753[87], 6}, {NULL}};
const Segment SGI151676[] = {{&S39754[44], 7}, {NULL}};
const Segment SGI151677[] = {{&S39754[44], 29}, {NULL}};
const Segment SGI151678[] = {{&S39754[30], 6}, {NULL}};
const Segment SGI151679[] = {{&S39754[122], 3}, {NULL}};
const Segment SGI151680[] = {{&S39754[87], 6}, {NULL}};
const Segment SGI151681[] = {{&S39755[44], 7}, {NULL}};
const Segment SGI151682[] = {{&S39755[44], 29}, {NULL}};
const Segment SGI151683[] = {{&S39755[30], 6}, {NULL}};
const Segment SGI151684[] = {{&S39755[122], 3}, {NULL}};
const Segment SGI151685[] = {{&S39755[87], 6}, {NULL}};
const Segment SGI151686[] = {{&S37709[32], 9}, {NULL}};
const Segment SGI151687[] = {{&S37709[27], 4}, {NULL}};
const Segment SGI151688[] = {{&S37709[94], 9}, {NULL}};
const Segment SGI151689[] = {{&S37709[67], 6}, {NULL}};
const Segment SGI151690[] = {{&S49482[44], 10}, {NULL}};
const Segment SGI151691[] = {{&S49482[44], 22}, {NULL}};
const Segment SGI151692[] = {{&S49482[30], 6}, {NULL}};
const Segment SGI151693[] = {{&S49482[31], 3}, {NULL}};
const Segment SGI151694[] = {{&S49482[80], 6}, {NULL}};
const Segment SGI151695[] = {{&S49483[44], 10}, {NULL}};
const Segment SGI151696[] = {{&S49483[44], 22}, {NULL}};
const Segment SGI151697[] = {{&S49483[30], 6}, {NULL}};
const Segment SGI151698[] = {{&S49483[31], 3}, {NULL}};
const Segment SGI151699[] = {{&S49483[80], 6}, {NULL}};
const Segment SGI151700[] = {{&S49484[44], 10}, {NULL}};
const Segment SGI151701[] = {{&S49484[44], 22}, {NULL}};
const Segment SGI151702[] = {{&S49484[30], 6}, {NULL}};
const Segment SGI151703[] = {{&S49484[31], 3}, {NULL}};
const Segment SGI151704[] = {{&S49484[80], 6}, {NULL}};
const Segment SGI151705[] = {{&S49485[38], 10}, {NULL}};
const Segment SGI151706[] = {{&S49485[31], 6}, {NULL}};
const Segment SGI151707[] = {{&S49486[38], 10}, {NULL}};
const Segment SGI151708[] = {{&S49486[31], 6}, {NULL}};
const Segment SGI151709[] = {{&S49487[38], 10}, {NULL}};
const Segment SGI151710[] = {{&S49487[31], 6}, {NULL}};
const Segment SGI151711[] = {{&S49488[42], 10}, {NULL}};
const Segment SGI151712[] = {{&S49488[30], 6}, {NULL}};
const Segment SGI151713[] = {{&S49488[31], 3}, {NULL}};
const Segment SGI151714[] = {{&S49488[78], 6}, {NULL}};
const Segment SGI151715[] = {{&S49489[44], 10}, {NULL}};
const Segment SGI151716[] = {{&S49489[44], 22}, {NULL}};
const Segment SGI151717[] = {{&S49489[30], 6}, {NULL}};
const Segment SGI151718[] = {{&S49489[31], 3}, {NULL}};
const Segment SGI151719[] = {{&S49489[80], 6}, {NULL}};
const Segment SGI151720[] = {{&S49490[42], 10}, {NULL}};
const Segment SGI151721[] = {{&S49490[30], 6}, {NULL}};
const Segment SGI151722[] = {{&S49490[31], 3}, {NULL}};
const Segment SGI151723[] = {{&S49490[78], 6}, {NULL}};
const Segment SGI151724[] = {{&S49491[42], 10}, {NULL}};
const Segment SGI151725[] = {{&S49491[30], 6}, {NULL}};
const Segment SGI151726[] = {{&S49491[31], 3}, {NULL}};
const Segment SGI151727[] = {{&S49491[78], 6}, {NULL}};
const Segment SGI151728[] = {{&S49492[44], 7}, {NULL}};
const Segment SGI151729[] = {{&S49492[44], 19}, {NULL}};
const Segment SGI151730[] = {{&S49492[30], 6}, {NULL}};
const Segment SGI151731[] = {{&S49492[31], 3}, {NULL}};
const Segment SGI151732[] = {{&S49492[77], 6}, {NULL}};
const Segment SGI151733[] = {{&S49493[38], 7}, {NULL}};
const Segment SGI151734[] = {{&S49493[31], 6}, {NULL}};
const Segment SGI151735[] = {{&S49501[44], 11}, {NULL}};
const Segment SGI151736[] = {{&S49501[44], 23}, {NULL}};
const Segment SGI151737[] = {{&S49501[30], 6}, {NULL}};
const Segment SGI151738[] = {{&S49501[31], 3}, {NULL}};
const Segment SGI151739[] = {{&S49501[81], 6}, {NULL}};
const Segment SGI151740[] = {{&S39899[44], 7}, {NULL}};
const Segment SGI151741[] = {{&S39899[44], 32}, {NULL}};
const Segment SGI151742[] = {{&S39899[30], 6}, {NULL}};
const Segment SGI151743[] = {{&S49502[38], 11}, {NULL}};
const Segment SGI151744[] = {{&S49502[31], 6}, {NULL}};
const Segment SGI151745[] = {{&S39899[125], 3}, {NULL}};
const Segment SGI151746[] = {{&S39899[90], 6}, {NULL}};
const Segment SGI151747[] = {{&S39900[38], 7}, {NULL}};
const Segment SGI151748[] = {{&S39900[31], 6}, {NULL}};
const Segment SGI151749[] = {{&S39901[44], 7}, {NULL}};
const Segment SGI151750[] = {{&S39901[44], 31}, {NULL}};
const Segment SGI151751[] = {{&S39901[30], 6}, {NULL}};
const Segment SGI151752[] = {{&S39901[124], 3}, {NULL}};
const Segment SGI151753[] = {{&S39901[89], 6}, {NULL}};
const Segment SGI151754[] = {{&S39902[44], 15}, {NULL}};
const Segment SGI151755[] = {{&S39902[44], 36}, {NULL}};
const Segment SGI151756[] = {{&S39902[30], 6}, {NULL}};
const Segment SGI151757[] = {{&S39902[129], 3}, {NULL}};
const Segment SGI151758[] = {{&S39902[94], 6}, {NULL}};
const Segment SGI151759[] = {{&S39903[44], 7}, {NULL}};
const Segment SGI151760[] = {{&S39903[44], 31}, {NULL}};
const Segment SGI151761[] = {{&S39903[30], 6}, {NULL}};
const Segment SGI151762[] = {{&S39903[124], 3}, {NULL}};
const Segment SGI151763[] = {{&S39903[89], 6}, {NULL}};
const Segment SGI151764[] = {{&S39904[44], 11}, {NULL}};
const Segment SGI151765[] = {{&S39904[44], 29}, {NULL}};
const Segment SGI151766[] = {{&S39904[30], 6}, {NULL}};
const Segment SGI151767[] = {{&S39904[122], 3}, {NULL}};
const Segment SGI151768[] = {{&S39904[87], 6}, {NULL}};
const Segment SGI151769[] = {{&S39905[44], 11}, {NULL}};
const Segment SGI151770[] = {{&S39905[44], 38}, {NULL}};
const Segment SGI151771[] = {{&S39905[30], 6}, {NULL}};
const Segment SGI151772[] = {{&S39905[131], 3}, {NULL}};
const Segment SGI151773[] = {{&S39905[96], 6}, {NULL}};
const Segment SGI151774[] = {{&S39906[44], 7}, {NULL}};
const Segment SGI151775[] = {{&S39906[44], 19}, {NULL}};
const Segment SGI151776[] = {{&S39906[30], 6}, {NULL}};
const Segment SGI151777[] = {{&S39906[112], 3}, {NULL}};
const Segment SGI151778[] = {{&S39906[77], 6}, {NULL}};
const Segment SGI151779[] = {{&S39907[44], 7}, {NULL}};
const Segment SGI151780[] = {{&S39907[44], 25}, {NULL}};
const Segment SGI151781[] = {{&S39907[30], 6}, {NULL}};
const Segment SGI151782[] = {{&S39907[118], 3}, {NULL}};
const Segment SGI151783[] = {{&S39907[83], 6}, {NULL}};
const Segment SGI151784[] = {{&S39908[44], 7}, {NULL}};
const Segment SGI151785[] = {{&S39908[44], 34}, {NULL}};
const Segment SGI151786[] = {{&S39908[30], 6}, {NULL}};
const Segment SGI151787[] = {{&S39908[127], 3}, {NULL}};
const Segment SGI151788[] = {{&S39908[92], 6}, {NULL}};
const Segment SGI151789[] = {{&S39909[44], 7}, {NULL}};
const Segment SGI151790[] = {{&S39909[44], 29}, {NULL}};
const Segment SGI151791[] = {{&S39909[30], 6}, {NULL}};
const Segment SGI151792[] = {{&S39909[122], 3}, {NULL}};
const Segment SGI151793[] = {{&S39909[87], 6}, {NULL}};
const Segment SGI151794[] = {{&S39936[34], 9}, {NULL}};
const Segment SGI151795[] = {{&S39936[27], 6}, {NULL}};
const Segment SGI151796[] = {{&S39936[96], 9}, {NULL}};
const Segment SGI151797[] = {{&S39936[69], 6}, {NULL}};
const Segment SGI151798[] = {{&S39937[34], 9}, {NULL}};
const Segment SGI151799[] = {{&S39937[27], 6}, {NULL}};
const Segment SGI151800[] = {{&S39937[96], 9}, {NULL}};
const Segment SGI151801[] = {{&S39937[69], 6}, {NULL}};
const Segment SGI151802[] = {{&S49560[44], 14}, {NULL}};
const Segment SGI151803[] = {{&S39938[34], 9}, {NULL}};
const Segment SGI151804[] = {{&S49560[44], 31}, {NULL}};
const Segment SGI151805[] = {{&S39938[27], 6}, {NULL}};
const Segment SGI151806[] = {{&S49560[30], 6}, {NULL}};
const Segment SGI151807[] = {{&S49560[31], 3}, {NULL}};
const Segment SGI151808[] = {{&S39938[96], 9}, {NULL}};
const Segment SGI151809[] = {{&S39938[69], 6}, {NULL}};
const Segment SGI151810[] = {{&S49560[89], 6}, {NULL}};
const Segment SGI151811[] = {{&S49561[38], 14}, {NULL}};
const Segment SGI151812[] = {{&S39939[34], 9}, {NULL}};
const Segment SGI151813[] = {{&S49561[31], 6}, {NULL}};
const Segment SGI151814[] = {{&S39939[27], 6}, {NULL}};
const Segment SGI151815[] = {{&S39939[96], 9}, {NULL}};
const Segment SGI151816[] = {{&S39939[69], 6}, {NULL}};
const Segment SGI151817[] = {{&S39940[34], 10}, {NULL}};
const Segment SGI151818[] = {{&S39940[27], 6}, {NULL}};
const Segment SGI151819[] = {{&S39940[97], 9}, {NULL}};
const Segment SGI151820[] = {{&S49563[44], 9}, {NULL}};
const Segment SGI151821[] = {{&S39940[70], 6}, {NULL}};
const Segment SGI151822[] = {{&S49563[44], 21}, {NULL}};
const Segment SGI151823[] = {{&S49563[30], 6}, {NULL}};
const Segment SGI151824[] = {{&S49563[31], 3}, {NULL}};
const Segment SGI151825[] = {{&S49563[79], 6}, {NULL}};
const Segment SGI151826[] = {{&S39941[34], 11}, {NULL}};
const Segment SGI151827[] = {{&S39941[27], 6}, {NULL}};
const Segment SGI151828[] = {{&S39941[98], 9}, {NULL}};
const Segment SGI151829[] = {{&S39941[71], 6}, {NULL}};
const Segment SGI151830[] = {{&S49564[38], 9}, {NULL}};
const Segment SGI151831[] = {{&S49564[31], 6}, {NULL}};
const Segment SGI151832[] = {{&S39942[34], 11}, {NULL}};
const Segment SGI151833[] = {{&S39942[27], 6}, {NULL}};
const Segment SGI151834[] = {{&S49565[44], 11}, {NULL}};
const Segment SGI151835[] = {{&S49565[44], 23}, {NULL}};
const Segment SGI151836[] = {{&S49565[30], 6}, {NULL}};
const Segment SGI151837[] = {{&S39942[98], 9}, {NULL}};
const Segment SGI151838[] = {{&S39942[71], 6}, {NULL}};
const Segment SGI151839[] = {{&S49565[31], 3}, {NULL}};
const Segment SGI151840[] = {{&S49565[81], 6}, {NULL}};
const Segment SGI151841[] = {{&S39943[34], 11}, {NULL}};
const Segment SGI151842[] = {{&S39943[27], 6}, {NULL}};
const Segment SGI151843[] = {{&S49566[34], 12}, {NULL}};
const Segment SGI151844[] = {{&S39943[98], 9}, {NULL}};
const Segment SGI151845[] = {{&S49566[27], 6}, {NULL}};
const Segment SGI151846[] = {{&S39943[71], 6}, {NULL}};
const Segment SGI151847[] = {{&S49566[115], 9}, {NULL}};
const Segment SGI151848[] = {{&S49566[88], 6}, {NULL}};
const Segment SGI151849[] = {{&S49567[34], 12}, {NULL}};
const Segment SGI151850[] = {{&S49567[27], 6}, {NULL}};
const Segment SGI151851[] = {{&S39945[34], 10}, {NULL}};
const Segment SGI151852[] = {{&S39945[27], 6}, {NULL}};
const Segment SGI151853[] = {{&S49567[115], 9}, {NULL}};
const Segment SGI151854[] = {{&S49567[88], 6}, {NULL}};
const Segment SGI151855[] = {{&S39945[97], 9}, {NULL}};
const Segment SGI151856[] = {{&S39945[70], 6}, {NULL}};
const Segment SGI151857[] = {{&S49589[34], 13}, {NULL}};
const Segment SGI151858[] = {{&S49589[27], 6}, {NULL}};
const Segment SGI151859[] = {{&S49589[99], 9}, {NULL}};
const Segment SGI151860[] = {{&S49589[72], 6}, {NULL}};
const Segment SGI151861[] = {{&S49592[34], 9}, {NULL}};
const Segment SGI151862[] = {{&S49592[27], 6}, {NULL}};
const Segment SGI151863[] = {{&S49592[96], 9}, {NULL}};
const Segment SGI151864[] = {{&S49592[69], 6}, {NULL}};
const Segment SGI151865[] = {{&S49593[34], 5}, {NULL}};
const Segment SGI151866[] = {{&S49593[27], 6}, {NULL}};
const Segment SGI151867[] = {{&S49593[92], 9}, {NULL}};
const Segment SGI151868[] = {{&S49593[65], 6}, {NULL}};
const Segment SGI151869[] = {{&S49594[34], 12}, {NULL}};
const Segment SGI151870[] = {{&S49594[27], 6}, {NULL}};
const Segment SGI151871[] = {{&S49594[98], 9}, {NULL}};
const Segment SGI151872[] = {{&S49594[71], 6}, {NULL}};
const Segment SGI151873[] = {{&S49596[6], 4}, {NULL}};
const Segment SGI151874[] = {{&S49596[83], 5}, {NULL}};
const Segment SGI151875[] = {{&S40473[44], 11}, {NULL}};
const Segment SGI151876[] = {{&S40473[44], 28}, {NULL}};
const Segment SGI151877[] = {{&S40473[30], 6}, {NULL}};
const Segment SGI151878[] = {{&S40473[121], 3}, {NULL}};
const Segment SGI151879[] = {{&S40473[86], 6}, {NULL}};
const Segment SGI151880[] = {{&S40474[44], 7}, {NULL}};
const Segment SGI151881[] = {{&S40474[44], 24}, {NULL}};
const Segment SGI151882[] = {{&S40474[30], 6}, {NULL}};
const Segment SGI151883[] = {{&S40474[117], 3}, {NULL}};
const Segment SGI151884[] = {{&S40474[82], 6}, {NULL}};
const Segment SGI151885[] = {{&S40475[44], 11}, {NULL}};
const Segment SGI151886[] = {{&S40475[44], 28}, {NULL}};
const Segment SGI151887[] = {{&S40475[30], 6}, {NULL}};
const Segment SGI151888[] = {{&S40475[121], 3}, {NULL}};
const Segment SGI151889[] = {{&S40475[86], 6}, {NULL}};
const Segment SGI151890[] = {{&S40476[44], 11}, {NULL}};
const Segment SGI151891[] = {{&S40476[44], 28}, {NULL}};
const Segment SGI151892[] = {{&S40476[30], 6}, {NULL}};
const Segment SGI151893[] = {{&S40476[121], 3}, {NULL}};
const Segment SGI151894[] = {{&S40476[86], 6}, {NULL}};
const Segment SGI151895[] = {{&S40477[44], 7}, {NULL}};
const Segment SGI151896[] = {{&S40477[44], 24}, {NULL}};
const Segment SGI151897[] = {{&S40477[30], 6}, {NULL}};
const Segment SGI151898[] = {{&S40477[117], 3}, {NULL}};
const Segment SGI151899[] = {{&S40477[82], 6}, {NULL}};
const Segment SGI151900[] = {{&S40478[38], 7}, {NULL}};
const Segment SGI151901[] = {{&S40478[31], 6}, {NULL}};
const Segment SGI151902[] = {{&S40479[44], 13}, {NULL}};
const Segment SGI151903[] = {{&S40479[44], 30}, {NULL}};
const Segment SGI151904[] = {{&S40479[30], 6}, {NULL}};
const Segment SGI151905[] = {{&S40479[123], 3}, {NULL}};
const Segment SGI151906[] = {{&S40479[88], 6}, {NULL}};
const Segment SGI151907[] = {{&S40480[44], 11}, {NULL}};
const Segment SGI151908[] = {{&S40480[44], 28}, {NULL}};
const Segment SGI151909[] = {{&S40480[30], 6}, {NULL}};
const Segment SGI151910[] = {{&S40480[121], 3}, {NULL}};
const Segment SGI151911[] = {{&S40480[86], 6}, {NULL}};
const Segment SGI151912[] = {{&S40481[44], 11}, {NULL}};
const Segment SGI151913[] = {{&S40481[44], 28}, {NULL}};
const Segment SGI151914[] = {{&S40481[30], 6}, {NULL}};
const Segment SGI151915[] = {{&S40481[121], 3}, {NULL}};
const Segment SGI151916[] = {{&S40481[86], 6}, {NULL}};
const Segment SGI151917[] = {{&S40482[44], 12}, {NULL}};
const Segment SGI151918[] = {{&S40482[44], 29}, {NULL}};
const Segment SGI151919[] = {{&S40482[30], 6}, {NULL}};
const Segment SGI151920[] = {{&S40482[122], 3}, {NULL}};
const Segment SGI151921[] = {{&S40482[87], 6}, {NULL}};
const Segment SGI151922[] = {{&S40483[44], 12}, {NULL}};
const Segment SGI151923[] = {{&S40483[44], 29}, {NULL}};
const Segment SGI151924[] = {{&S40483[30], 6}, {NULL}};
const Segment SGI151925[] = {{&S40483[122], 3}, {NULL}};
const Segment SGI151926[] = {{&S40483[87], 6}, {NULL}};
const Segment SGI151927[] = {{&S40484[44], 11}, {NULL}};
const Segment SGI151928[] = {{&S40484[44], 28}, {NULL}};
const Segment SGI151929[] = {{&S40484[30], 6}, {NULL}};
const Segment SGI151930[] = {{&S40484[121], 3}, {NULL}};
const Segment SGI151931[] = {{&S40484[86], 6}, {NULL}};
const Segment SGI151932[] = {{&S40485[44], 7}, {NULL}};
const Segment SGI151933[] = {{&S40485[44], 24}, {NULL}};
const Segment SGI151934[] = {{&S40485[30], 6}, {NULL}};
const Segment SGI151935[] = {{&S40485[117], 3}, {NULL}};
const Segment SGI151936[] = {{&S40485[82], 6}, {NULL}};
const Segment SGI151937[] = {{&S40008[44], 7}, {NULL}};
const Segment SGI151938[] = {{&S40008[44], 24}, {NULL}};
const Segment SGI151939[] = {{&S40008[30], 6}, {NULL}};
const Segment SGI151940[] = {{&S40008[117], 3}, {NULL}};
const Segment SGI151941[] = {{&S40008[82], 6}, {NULL}};
const Segment SGI151942[] = {{&S40486[44], 7}, {NULL}};
const Segment SGI151943[] = {{&S40486[44], 24}, {NULL}};
const Segment SGI151944[] = {{&S40486[30], 6}, {NULL}};
const Segment SGI151945[] = {{&S40486[117], 3}, {NULL}};
const Segment SGI151946[] = {{&S40486[82], 6}, {NULL}};
const Segment SGI151947[] = {{&S40009[44], 7}, {NULL}};
const Segment SGI151948[] = {{&S40009[44], 24}, {NULL}};
const Segment SGI151949[] = {{&S40009[30], 6}, {NULL}};
const Segment SGI151950[] = {{&S40009[117], 3}, {NULL}};
const Segment SGI151951[] = {{&S40009[82], 6}, {NULL}};
const Segment SGI151952[] = {{&S40487[44], 13}, {NULL}};
const Segment SGI151953[] = {{&S40487[44], 30}, {NULL}};
const Segment SGI151954[] = {{&S40487[30], 6}, {NULL}};
const Segment SGI151955[] = {{&S40487[123], 3}, {NULL}};
const Segment SGI151956[] = {{&S40487[88], 6}, {NULL}};
const Segment SGI151957[] = {{&S40010[44], 11}, {NULL}};
const Segment SGI151958[] = {{&S40010[44], 28}, {NULL}};
const Segment SGI151959[] = {{&S40010[30], 6}, {NULL}};
const Segment SGI151960[] = {{&S40010[121], 3}, {NULL}};
const Segment SGI151961[] = {{&S40010[86], 6}, {NULL}};
const Segment SGI151962[] = {{&S40488[44], 7}, {NULL}};
const Segment SGI151963[] = {{&S40488[44], 24}, {NULL}};
const Segment SGI151964[] = {{&S40488[30], 6}, {NULL}};
const Segment SGI151965[] = {{&S40488[117], 3}, {NULL}};
const Segment SGI151966[] = {{&S40488[82], 6}, {NULL}};
const Segment SGI151967[] = {{&S40011[38], 7}, {NULL}};
const Segment SGI151968[] = {{&S40011[31], 6}, {NULL}};
const Segment SGI151969[] = {{&S40489[44], 11}, {NULL}};
const Segment SGI151970[] = {{&S40489[44], 28}, {NULL}};
const Segment SGI151971[] = {{&S40489[30], 6}, {NULL}};
const Segment SGI151972[] = {{&S40489[121], 3}, {NULL}};
const Segment SGI151973[] = {{&S40012[44], 7}, {NULL}};
const Segment SGI151974[] = {{&S40012[44], 24}, {NULL}};
const Segment SGI151975[] = {{&S40489[86], 6}, {NULL}};
const Segment SGI151976[] = {{&S40012[30], 6}, {NULL}};
const Segment SGI151977[] = {{&S40012[117], 3}, {NULL}};
const Segment SGI151978[] = {{&S40012[82], 6}, {NULL}};
const Segment SGI151979[] = {{&S40490[44], 7}, {NULL}};
const Segment SGI151980[] = {{&S40490[44], 24}, {NULL}};
const Segment SGI151981[] = {{&S40490[30], 6}, {NULL}};
const Segment SGI151982[] = {{&S40013[44], 11}, {NULL}};
const Segment SGI151983[] = {{&S40013[44], 28}, {NULL}};
const Segment SGI151984[] = {{&S40490[117], 3}, {NULL}};
const Segment SGI151985[] = {{&S40013[30], 6}, {NULL}};
const Segment SGI151986[] = {{&S40490[82], 6}, {NULL}};
const Segment SGI151987[] = {{&S40013[121], 3}, {NULL}};
const Segment SGI151988[] = {{&S40013[86], 6}, {NULL}};
const Segment SGI151989[] = {{&S40014[38], 7}, {NULL}};
const Segment SGI151990[] = {{&S40014[31], 6}, {NULL}};
const Segment SGI151991[] = {{&S40015[44], 12}, {NULL}};
const Segment SGI151992[] = {{&S40015[44], 29}, {NULL}};
const Segment SGI151993[] = {{&S40015[30], 6}, {NULL}};
const Segment SGI151994[] = {{&S40015[122], 3}, {NULL}};
const Segment SGI151995[] = {{&S40015[87], 6}, {NULL}};
const Segment SGI151996[] = {{&S40016[44], 7}, {NULL}};
const Segment SGI151997[] = {{&S40016[44], 24}, {NULL}};
const Segment SGI151998[] = {{&S40016[30], 6}, {NULL}};
const Segment SGI151999[] = {{&S40016[117], 3}, {NULL}};
const Segment SGI152000[] = {{&S40016[82], 6}, {NULL}};
const Segment SGI152001[] = {{&S49645[34], 10}, {NULL}};
const Segment SGI152002[] = {{&S49645[27], 6}, {NULL}};
const Segment SGI152003[] = {{&S49645[97], 9}, {NULL}};
const Segment SGI152004[] = {{&S49645[70], 6}, {NULL}};
const Segment SGI152005[] = {{&S49646[34], 10}, {NULL}};
const Segment SGI152006[] = {{&S49646[27], 6}, {NULL}};
const Segment SGI152007[] = {{&S49646[97], 9}, {NULL}};
const Segment SGI152008[] = {{&S49646[70], 6}, {NULL}};
const Segment SGI152009[] = {{&S49647[34], 10}, {NULL}};
const Segment SGI152010[] = {{&S49647[27], 6}, {NULL}};
const Segment SGI152011[] = {{&S49647[97], 9}, {NULL}};
const Segment SGI152012[] = {{&S49647[70], 6}, {NULL}};
const Segment SGI152013[] = {{&S49648[34], 10}, {NULL}};
const Segment SGI152014[] = {{&S49648[27], 6}, {NULL}};
const Segment SGI152015[] = {{&S49648[97], 9}, {NULL}};
const Segment SGI152016[] = {{&S49648[70], 6}, {NULL}};
const Segment SGI152017[] = {{&S49649[34], 8}, {NULL}};
const Segment SGI152018[] = {{&S49649[27], 6}, {NULL}};
const Segment SGI152019[] = {{&S49649[95], 9}, {NULL}};
const Segment SGI152020[] = {{&S49649[68], 6}, {NULL}};
const Segment SGI152021[] = {{&S49650[34], 11}, {NULL}};
const Segment SGI152022[] = {{&S49650[27], 6}, {NULL}};
const Segment SGI152023[] = {{&S49650[98], 9}, {NULL}};
const Segment SGI152024[] = {{&S49650[71], 6}, {NULL}};
const Segment SGI152025[] = {{&S49651[34], 11}, {NULL}};
const Segment SGI152026[] = {{&S49651[27], 6}, {NULL}};
const Segment SGI152027[] = {{&S49651[98], 9}, {NULL}};
const Segment SGI152028[] = {{&S49651[71], 6}, {NULL}};
const Segment SGI152029[] = {{&S49652[34], 7}, {NULL}};
const Segment SGI152030[] = {{&S49652[27], 6}, {NULL}};
const Segment SGI152031[] = {{&S49652[94], 9}, {NULL}};
const Segment SGI152032[] = {{&S49652[67], 6}, {NULL}};
const Segment SGI152033[] = {{&S49653[44], 18}, {NULL}};
const Segment SGI152034[] = {{&S49653[44], 30}, {NULL}};
const Segment SGI152035[] = {{&S49653[30], 6}, {NULL}};
const Segment SGI152036[] = {{&S49653[31], 3}, {NULL}};
const Segment SGI152037[] = {{&S49653[88], 6}, {NULL}};
const Segment SGI152038[] = {{&S49654[38], 18}, {NULL}};
const Segment SGI152039[] = {{&S49654[31], 6}, {NULL}};
const Segment SGI152040[] = {{&S40061[34], 10}, {NULL}};
const Segment SGI152041[] = {{&S40061[27], 6}, {NULL}};
const Segment SGI152042[] = {{&S40061[97], 9}, {NULL}};
const Segment SGI152043[] = {{&S40061[70], 6}, {NULL}};
const Segment SGI152044[] = {{&S40062[34], 11}, {NULL}};
const Segment SGI152045[] = {{&S40062[27], 6}, {NULL}};
const Segment SGI152046[] = {{&S40062[98], 9}, {NULL}};
const Segment SGI152047[] = {{&S40062[71], 6}, {NULL}};
const Segment SGI152048[] = {{&S40063[34], 11}, {NULL}};
const Segment SGI152049[] = {{&S40063[27], 6}, {NULL}};
const Segment SGI152050[] = {{&S40063[98], 9}, {NULL}};
const Segment SGI152051[] = {{&S40063[71], 6}, {NULL}};
const Segment SGI152052[] = {{&S40064[34], 11}, {NULL}};
const Segment SGI152053[] = {{&S40064[27], 6}, {NULL}};
const Segment SGI152054[] = {{&S40064[98], 9}, {NULL}};
const Segment SGI152055[] = {{&S40064[71], 6}, {NULL}};
const Segment SGI152056[] = {{&S40072[34], 11}, {NULL}};
const Segment SGI152057[] = {{&S40072[27], 6}, {NULL}};
const Segment SGI152058[] = {{&S40072[98], 9}, {NULL}};
const Segment SGI152059[] = {{&S40072[71], 6}, {NULL}};
const Segment SGI152060[] = {{&S40073[34], 10}, {NULL}};
const Segment SGI152061[] = {{&S40073[27], 6}, {NULL}};
const Segment SGI152062[] = {{&S40073[97], 9}, {NULL}};
const Segment SGI152063[] = {{&S40073[70], 6}, {NULL}};
const Segment SGI152064[] = {{&S40074[34], 10}, {NULL}};
const Segment SGI152065[] = {{&S40074[27], 6}, {NULL}};
const Segment SGI152066[] = {{&S40074[97], 9}, {NULL}};
const Segment SGI152067[] = {{&S40074[70], 6}, {NULL}};
const Segment SGI152068[] = {{&S40076[34], 11}, {NULL}};
const Segment SGI152069[] = {{&S40076[27], 6}, {NULL}};
const Segment SGI152070[] = {{&S40076[98], 9}, {NULL}};
const Segment SGI152071[] = {{&S40076[71], 6}, {NULL}};
const Segment SGI152072[] = {{&S40077[34], 11}, {NULL}};
const Segment SGI152073[] = {{&S40077[27], 6}, {NULL}};
const Segment SGI152074[] = {{&S40077[98], 9}, {NULL}};
const Segment SGI152075[] = {{&S40077[71], 6}, {NULL}};
const Segment SGI152076[] = {{&S40078[34], 11}, {NULL}};
const Segment SGI152077[] = {{&S40078[27], 6}, {NULL}};
const Segment SGI152078[] = {{&S40078[98], 9}, {NULL}};
const Segment SGI152079[] = {{&S40078[71], 6}, {NULL}};
const Segment SGI152080[] = {{&S40079[44], 17}, {NULL}};
const Segment SGI152081[] = {{&S40079[44], 29}, {NULL}};
const Segment SGI152082[] = {{&S40079[30], 6}, {NULL}};
const Segment SGI152083[] = {{&S40079[122], 3}, {NULL}};
const Segment SGI152084[] = {{&S40079[87], 6}, {NULL}};
const Segment SGI152085[] = {{&S40080[44], 9}, {NULL}};
const Segment SGI152086[] = {{&S40080[44], 21}, {NULL}};
const Segment SGI152087[] = {{&S40080[30], 6}, {NULL}};
const Segment SGI152088[] = {{&S40080[114], 3}, {NULL}};
const Segment SGI152089[] = {{&S40080[79], 6}, {NULL}};
const Segment SGI152090[] = {{&S40081[44], 17}, {NULL}};
const Segment SGI152091[] = {{&S40081[44], 29}, {NULL}};
const Segment SGI152092[] = {{&S40081[30], 6}, {NULL}};
const Segment SGI152093[] = {{&S49709[44], 9}, {NULL}};
const Segment SGI152094[] = {{&S49709[44], 23}, {NULL}};
const Segment SGI152095[] = {{&S49709[30], 6}, {NULL}};
const Segment SGI152096[] = {{&S40081[122], 3}, {NULL}};
const Segment SGI152097[] = {{&S40081[87], 6}, {NULL}};
const Segment SGI152098[] = {{&S49709[31], 3}, {NULL}};
const Segment SGI152099[] = {{&S49709[81], 6}, {NULL}};
const Segment SGI152100[] = {{&S40082[38], 9}, {NULL}};
const Segment SGI152101[] = {{&S40082[31], 6}, {NULL}};
const Segment SGI152102[] = {{&S49710[44], 9}, {NULL}};
const Segment SGI152103[] = {{&S49710[44], 23}, {NULL}};
const Segment SGI152104[] = {{&S49710[30], 6}, {NULL}};
const Segment SGI152105[] = {{&S49710[31], 3}, {NULL}};
const Segment SGI152106[] = {{&S49710[81], 6}, {NULL}};
const Segment SGI152107[] = {{&S40083[44], 17}, {NULL}};
const Segment SGI152108[] = {{&S40083[44], 29}, {NULL}};
const Segment SGI152109[] = {{&S40083[30], 6}, {NULL}};
const Segment SGI152110[] = {{&S40083[122], 3}, {NULL}};
const Segment SGI152111[] = {{&S40083[87], 6}, {NULL}};
const Segment SGI152112[] = {{&S49711[38], 9}, {NULL}};
const Segment SGI152113[] = {{&S49711[31], 6}, {NULL}};
const Segment SGI152114[] = {{&S40084[44], 17}, {NULL}};
const Segment SGI152115[] = {{&S40084[44], 29}, {NULL}};
const Segment SGI152116[] = {{&S40084[30], 6}, {NULL}};
const Segment SGI152117[] = {{&S40084[122], 3}, {NULL}};
const Segment SGI152118[] = {{&S40084[87], 6}, {NULL}};
const Segment SGI152119[] = {{&S49712[44], 9}, {NULL}};
const Segment SGI152120[] = {{&S49712[44], 26}, {NULL}};
const Segment SGI152121[] = {{&S49712[30], 6}, {NULL}};
const Segment SGI152122[] = {{&S49712[31], 3}, {NULL}};
const Segment SGI152123[] = {{&S49712[84], 6}, {NULL}};
const Segment SGI152124[] = {{&S40085[44], 17}, {NULL}};
const Segment SGI152125[] = {{&S40085[44], 29}, {NULL}};
const Segment SGI152126[] = {{&S40085[30], 6}, {NULL}};
const Segment SGI152127[] = {{&S40085[122], 3}, {NULL}};
const Segment SGI152128[] = {{&S40085[87], 6}, {NULL}};
const Segment SGI152129[] = {{&S49714[34], 8}, {NULL}};
const Segment SGI152130[] = {{&S49714[27], 6}, {NULL}};
const Segment SGI152131[] = {{&S49714[100], 9}, {NULL}};
const Segment SGI152132[] = {{&S49714[73], 6}, {NULL}};
const Segment SGI152133[] = {{&S49715[34], 7}, {NULL}};
const Segment SGI152134[] = {{&S49715[27], 6}, {NULL}};
const Segment SGI152135[] = {{&S49715[98], 9}, {NULL}};
const Segment SGI152136[] = {{&S49715[71], 6}, {NULL}};
const Segment SGI152137[] = {{&S49716[34], 7}, {NULL}};
const Segment SGI152138[] = {{&S49716[27], 6}, {NULL}};
const Segment SGI152139[] = {{&S49716[97], 9}, {NULL}};
const Segment SGI152140[] = {{&S49716[70], 6}, {NULL}};
const Segment SGI152141[] = {{&S49728[34], 10}, {NULL}};
const Segment SGI152142[] = {{&S49728[27], 6}, {NULL}};
const Segment SGI152143[] = {{&S49728[97], 9}, {NULL}};
const Segment SGI152144[] = {{&S49728[70], 6}, {NULL}};
const Segment SGI152145[] = {{&S49730[34], 10}, {NULL}};
const Segment SGI152146[] = {{&S49730[27], 6}, {NULL}};
const Segment SGI152147[] = {{&S49730[97], 9}, {NULL}};
const Segment SGI152148[] = {{&S49730[70], 6}, {NULL}};
const Segment SGI152149[] = {{&S49731[34], 11}, {NULL}};
const Segment SGI152150[] = {{&S49731[27], 6}, {NULL}};
const Segment SGI152151[] = {{&S49731[98], 9}, {NULL}};
const Segment SGI152152[] = {{&S49731[71], 6}, {NULL}};
const Segment SGI152153[] = {{&S49732[34], 11}, {NULL}};
const Segment SGI152154[] = {{&S49732[27], 6}, {NULL}};
const Segment SGI152155[] = {{&S49732[98], 9}, {NULL}};
const Segment SGI152156[] = {{&S49732[71], 6}, {NULL}};
const Segment SGI152157[] = {{&S49733[34], 11}, {NULL}};
const Segment SGI152158[] = {{&S49733[27], 6}, {NULL}};
const Segment SGI152159[] = {{&S49733[98], 9}, {NULL}};
const Segment SGI152160[] = {{&S49733[71], 6}, {NULL}};
const Segment SGI152161[] = {{&S49734[34], 8}, {NULL}};
const Segment SGI152162[] = {{&S49734[27], 6}, {NULL}};
const Segment SGI152163[] = {{&S49734[95], 9}, {NULL}};
const Segment SGI152164[] = {{&S49734[68], 6}, {NULL}};
const Segment SGI152165[] = {{&S49735[34], 11}, {NULL}};
const Segment SGI152166[] = {{&S49735[27], 6}, {NULL}};
const Segment SGI152167[] = {{&S49735[98], 9}, {NULL}};
const Segment SGI152168[] = {{&S49735[71], 6}, {NULL}};
const Segment SGI152169[] = {{&S49736[34], 11}, {NULL}};
const Segment SGI152170[] = {{&S49736[27], 6}, {NULL}};
const Segment SGI152171[] = {{&S49736[98], 9}, {NULL}};
const Segment SGI152172[] = {{&S49736[71], 6}, {NULL}};
const Segment SGI152173[] = {{&S49767[34], 10}, {NULL}};
const Segment SGI152174[] = {{&S49767[27], 6}, {NULL}};
const Segment SGI152175[] = {{&S49767[97], 9}, {NULL}};
const Segment SGI152176[] = {{&S49767[70], 6}, {NULL}};
const Segment SGI152177[] = {{&S49768[34], 10}, {NULL}};
const Segment SGI152178[] = {{&S49768[27], 6}, {NULL}};
const Segment SGI152179[] = {{&S49768[97], 9}, {NULL}};
const Segment SGI152180[] = {{&S49768[70], 6}, {NULL}};
const Segment SGI152181[] = {{&S26076[44], 9}, {NULL}};
const Segment SGI152182[] = {{&S26076[44], 20}, {NULL}};
const Segment SGI152183[] = {{&S26076[30], 6}, {NULL}};
const Segment SGI152184[] = {{&S26076[113], 3}, {NULL}};
const Segment SGI152185[] = {{&S26076[78], 6}, {NULL}};
const Segment SGI152186[] = {{&S26077[38], 9}, {NULL}};
const Segment SGI152187[] = {{&S26077[31], 6}, {NULL}};
const Segment SGI152188[] = {{&S26078[44], 9}, {NULL}};
const Segment SGI152189[] = {{&S26078[44], 20}, {NULL}};
const Segment SGI152190[] = {{&S26078[30], 6}, {NULL}};
const Segment SGI152191[] = {{&S26078[113], 3}, {NULL}};
const Segment SGI152192[] = {{&S26078[78], 6}, {NULL}};
const Segment SGI152193[] = {{&S26079[42], 9}, {NULL}};
const Segment SGI152194[] = {{&S26079[30], 6}, {NULL}};
const Segment SGI152195[] = {{&S26079[111], 3}, {NULL}};
const Segment SGI152196[] = {{&S26079[76], 6}, {NULL}};
const Segment SGI152197[] = {{&S26080[44], 9}, {NULL}};
const Segment SGI152198[] = {{&S26080[44], 21}, {NULL}};
const Segment SGI152199[] = {{&S26080[30], 6}, {NULL}};
const Segment SGI152200[] = {{&S26080[114], 3}, {NULL}};
const Segment SGI152201[] = {{&S26080[79], 6}, {NULL}};
const Segment SGI152202[] = {{&S26081[44], 9}, {NULL}};
const Segment SGI152203[] = {{&S26081[44], 21}, {NULL}};
const Segment SGI152204[] = {{&S26081[30], 6}, {NULL}};
const Segment SGI152205[] = {{&S26081[114], 3}, {NULL}};
const Segment SGI152206[] = {{&S26081[79], 6}, {NULL}};
const Segment SGI152207[] = {{&S26083[44], 12}, {NULL}};
const Segment SGI152208[] = {{&S26083[44], 23}, {NULL}};
const Segment SGI152209[] = {{&S26083[30], 6}, {NULL}};
const Segment SGI152210[] = {{&S26083[116], 3}, {NULL}};
const Segment SGI152211[] = {{&S26083[81], 6}, {NULL}};
const Segment SGI152212[] = {{&S49815[34], 8}, {NULL}};
const Segment SGI152213[] = {{&S49815[27], 6}, {NULL}};
const Segment SGI152214[] = {{&S49815[95], 9}, {NULL}};
const Segment SGI152215[] = {{&S49815[68], 6}, {NULL}};
const Segment SGI152216[] = {{&S49816[34], 11}, {NULL}};
const Segment SGI152217[] = {{&S49816[27], 6}, {NULL}};
const Segment SGI152218[] = {{&S49816[98], 9}, {NULL}};
const Segment SGI152219[] = {{&S49816[71], 6}, {NULL}};
const Segment SGI152220[] = {{&S49817[34], 11}, {NULL}};
const Segment SGI152221[] = {{&S49817[27], 6}, {NULL}};
const Segment SGI152222[] = {{&S49817[98], 9}, {NULL}};
const Segment SGI152223[] = {{&S49817[71], 6}, {NULL}};
const Segment SGI152224[] = {{&S49818[34], 10}, {NULL}};
const Segment SGI152225[] = {{&S49818[27], 6}, {NULL}};
const Segment SGI152226[] = {{&S49818[97], 9}, {NULL}};
const Segment SGI152227[] = {{&S49818[70], 6}, {NULL}};
const Segment SGI152228[] = {{&S49819[34], 10}, {NULL}};
const Segment SGI152229[] = {{&S49819[27], 6}, {NULL}};
const Segment SGI152230[] = {{&S49819[97], 9}, {NULL}};
const Segment SGI152231[] = {{&S49819[70], 6}, {NULL}};
const Segment SGI152232[] = {{&S49821[34], 10}, {NULL}};
const Segment SGI152233[] = {{&S49821[27], 6}, {NULL}};
const Segment SGI152234[] = {{&S49821[97], 9}, {NULL}};
const Segment SGI152235[] = {{&S49821[70], 6}, {NULL}};
const Segment SGI152236[] = {{&S49822[34], 11}, {NULL}};
const Segment SGI152237[] = {{&S49822[27], 6}, {NULL}};
const Segment SGI152238[] = {{&S49822[98], 9}, {NULL}};
const Segment SGI152239[] = {{&S49822[71], 6}, {NULL}};
const Segment SGI152240[] = {{&S49823[34], 11}, {NULL}};
const Segment SGI152241[] = {{&S49823[27], 6}, {NULL}};
const Segment SGI152242[] = {{&S49823[98], 9}, {NULL}};
const Segment SGI152243[] = {{&S49823[71], 6}, {NULL}};
const Segment SGI152244[] = {{&S49825[34], 11}, {NULL}};
const Segment SGI152245[] = {{&S49825[27], 6}, {NULL}};
const Segment SGI152246[] = {{&S49825[98], 9}, {NULL}};
const Segment SGI152247[] = {{&S49825[71], 6}, {NULL}};
const Segment SGI152248[] = {{&S49826[44], 10}, {NULL}};
const Segment SGI152249[] = {{&S49826[44], 22}, {NULL}};
const Segment SGI152250[] = {{&S49826[30], 6}, {NULL}};
const Segment SGI152251[] = {{&S49826[31], 3}, {NULL}};
const Segment SGI152252[] = {{&S49826[95], 6}, {NULL}};
const Segment SGI152253[] = {{&S49826[68], 8}, {NULL}};
const Segment SGI152254[] = {{&S49827[44], 10}, {NULL}};
const Segment SGI152255[] = {{&S49827[44], 22}, {NULL}};
const Segment SGI152256[] = {{&S49827[30], 6}, {NULL}};
const Segment SGI152257[] = {{&S49827[31], 3}, {NULL}};
const Segment SGI152258[] = {{&S49827[99], 6}, {NULL}};
const Segment SGI152259[] = {{&S49827[68], 8}, {NULL}};
const Segment SGI152260[] = {{&S49828[44], 10}, {NULL}};
const Segment SGI152261[] = {{&S49828[44], 22}, {NULL}};
const Segment SGI152262[] = {{&S49828[30], 6}, {NULL}};
const Segment SGI152263[] = {{&S49828[31], 3}, {NULL}};
const Segment SGI152264[] = {{&S49828[99], 6}, {NULL}};
const Segment SGI152265[] = {{&S49828[68], 8}, {NULL}};
const Segment SGI152266[] = {{&S49829[44], 10}, {NULL}};
const Segment SGI152267[] = {{&S49829[44], 22}, {NULL}};
const Segment SGI152268[] = {{&S49829[30], 6}, {NULL}};
const Segment SGI152269[] = {{&S49829[31], 3}, {NULL}};
const Segment SGI152270[] = {{&S49829[95], 6}, {NULL}};
const Segment SGI152271[] = {{&S49829[68], 8}, {NULL}};
const Segment SGI152272[] = {{&S49830[44], 10}, {NULL}};
const Segment SGI152273[] = {{&S49830[44], 22}, {NULL}};
const Segment SGI152274[] = {{&S49830[30], 6}, {NULL}};
const Segment SGI152275[] = {{&S49830[31], 3}, {NULL}};
const Segment SGI152276[] = {{&S49830[95], 6}, {NULL}};
const Segment SGI152277[] = {{&S49830[68], 8}, {NULL}};
const Segment SGI152278[] = {{&S49831[44], 11}, {NULL}};
const Segment SGI152279[] = {{&S49831[44], 23}, {NULL}};
const Segment SGI152280[] = {{&S49831[30], 6}, {NULL}};
const Segment SGI152281[] = {{&S49831[31], 3}, {NULL}};
const Segment SGI152282[] = {{&S49831[96], 6}, {NULL}};
const Segment SGI152283[] = {{&S49831[69], 8}, {NULL}};
const Segment SGI152284[] = {{&S49832[44], 11}, {NULL}};
const Segment SGI152285[] = {{&S49832[44], 23}, {NULL}};
const Segment SGI152286[] = {{&S49832[30], 6}, {NULL}};
const Segment SGI152287[] = {{&S49832[31], 3}, {NULL}};
const Segment SGI152288[] = {{&S49832[96], 6}, {NULL}};
const Segment SGI152289[] = {{&S49832[69], 8}, {NULL}};
const Segment SGI152290[] = {{&S49833[44], 11}, {NULL}};
const Segment SGI152291[] = {{&S49833[44], 23}, {NULL}};
const Segment SGI152292[] = {{&S49833[30], 6}, {NULL}};
const Segment SGI152293[] = {{&S49833[31], 3}, {NULL}};
const Segment SGI152294[] = {{&S49833[96], 6}, {NULL}};
const Segment SGI152295[] = {{&S49833[69], 8}, {NULL}};
const Segment SGI152296[] = {{&S49834[44], 11}, {NULL}};
const Segment SGI152297[] = {{&S49834[44], 23}, {NULL}};
const Segment SGI152298[] = {{&S49834[30], 6}, {NULL}};
const Segment SGI152299[] = {{&S49834[31], 3}, {NULL}};
const Segment SGI152300[] = {{&S49834[100], 6}, {NULL}};
const Segment SGI152301[] = {{&S49834[69], 8}, {NULL}};
const Segment SGI152302[] = {{&S49835[44], 11}, {NULL}};
const Segment SGI152303[] = {{&S49835[44], 23}, {NULL}};
const Segment SGI152304[] = {{&S49835[30], 6}, {NULL}};
const Segment SGI152305[] = {{&S49835[31], 3}, {NULL}};
const Segment SGI152306[] = {{&S49835[96], 6}, {NULL}};
const Segment SGI152307[] = {{&S49835[69], 8}, {NULL}};
const Segment SGI152308[] = {{&S49836[44], 11}, {NULL}};
const Segment SGI152309[] = {{&S49836[44], 23}, {NULL}};
const Segment SGI152310[] = {{&S49836[30], 6}, {NULL}};
const Segment SGI152311[] = {{&S49836[31], 3}, {NULL}};
const Segment SGI152312[] = {{&S49836[96], 6}, {NULL}};
const Segment SGI152313[] = {{&S49836[69], 8}, {NULL}};
const Segment SGI152314[] = {{&S49837[44], 10}, {NULL}};
const Segment SGI152315[] = {{&S49837[44], 22}, {NULL}};
const Segment SGI152316[] = {{&S49837[30], 6}, {NULL}};
const Segment SGI152317[] = {{&S49837[31], 3}, {NULL}};
const Segment SGI152318[] = {{&S49837[99], 6}, {NULL}};
const Segment SGI152319[] = {{&S49837[68], 8}, {NULL}};
const Segment SGI152320[] = {{&S49838[44], 10}, {NULL}};
const Segment SGI152321[] = {{&S49838[44], 22}, {NULL}};
const Segment SGI152322[] = {{&S49838[30], 6}, {NULL}};
const Segment SGI152323[] = {{&S49838[31], 3}, {NULL}};
const Segment SGI152324[] = {{&S49838[96], 6}, {NULL}};
const Segment SGI152325[] = {{&S49838[68], 8}, {NULL}};
const Segment SGI152326[] = {{&S40253[34], 9}, {NULL}};
const Segment SGI152327[] = {{&S40253[27], 6}, {NULL}};
const Segment SGI152328[] = {{&S40253[96], 9}, {NULL}};
const Segment SGI152329[] = {{&S40253[69], 6}, {NULL}};
const Segment SGI152330[] = {{&S49841[34], 11}, {NULL}};
const Segment SGI152331[] = {{&S49841[27], 6}, {NULL}};
const Segment SGI152332[] = {{&S40254[34], 14}, {NULL}};
const Segment SGI152333[] = {{&S40759[34], 14}, {NULL}};
const Segment SGI152334[] = {{&S40254[27], 6}, {NULL}};
const Segment SGI152335[] = {{&S49841[98], 9}, {NULL}};
const Segment SGI152336[] = {{&S40759[27], 6}, {NULL}};
const Segment SGI152337[] = {{&S49841[71], 6}, {NULL}};
const Segment SGI152338[] = {{&S40254[101], 9}, {NULL}};
const Segment SGI152339[] = {{&S40254[74], 6}, {NULL}};
const Segment SGI152340[] = {{&S40759[101], 9}, {NULL}};
const Segment SGI152341[] = {{&S40759[74], 6}, {NULL}};
const Segment SGI152342[] = {{&S49842[34], 10}, {NULL}};
const Segment SGI152343[] = {{&S49842[27], 6}, {NULL}};
const Segment SGI152344[] = {{&S40255[34], 11}, {NULL}};
const Segment SGI152345[] = {{&S40255[27], 6}, {NULL}};
const Segment SGI152346[] = {{&S49842[97], 9}, {NULL}};
const Segment SGI152347[] = {{&S49842[70], 6}, {NULL}};
const Segment SGI152348[] = {{&S40760[34], 14}, {NULL}};
const Segment SGI152349[] = {{&S40255[98], 9}, {NULL}};
const Segment SGI152350[] = {{&S40255[71], 6}, {NULL}};
const Segment SGI152351[] = {{&S40760[27], 6}, {NULL}};
const Segment SGI152352[] = {{&S40760[101], 9}, {NULL}};
const Segment SGI152353[] = {{&S40760[74], 6}, {NULL}};
const Segment SGI152354[] = {{&S49843[34], 10}, {NULL}};
const Segment SGI152355[] = {{&S49843[27], 6}, {NULL}};
const Segment SGI152356[] = {{&S40256[34], 8}, {NULL}};
const Segment SGI152357[] = {{&S40256[27], 6}, {NULL}};
const Segment SGI152358[] = {{&S49843[97], 9}, {NULL}};
const Segment SGI152359[] = {{&S49843[70], 6}, {NULL}};
const Segment SGI152360[] = {{&S40256[95], 9}, {NULL}};
const Segment SGI152361[] = {{&S40256[68], 6}, {NULL}};
const Segment SGI152362[] = {{&S49844[34], 10}, {NULL}};
const Segment SGI152363[] = {{&S49844[27], 6}, {NULL}};
const Segment SGI152364[] = {{&S49844[97], 9}, {NULL}};
const Segment SGI152365[] = {{&S49844[70], 6}, {NULL}};
const Segment SGI152366[] = {{&S40764[34], 10}, {NULL}};
const Segment SGI152367[] = {{&S40764[27], 6}, {NULL}};
const Segment SGI152368[] = {{&S49845[34], 8}, {NULL}};
const Segment SGI152369[] = {{&S49845[27], 6}, {NULL}};
const Segment SGI152370[] = {{&S40764[97], 9}, {NULL}};
const Segment SGI152371[] = {{&S40764[70], 6}, {NULL}};
const Segment SGI152372[] = {{&S49845[95], 9}, {NULL}};
const Segment SGI152373[] = {{&S49845[68], 6}, {NULL}};
const Segment SGI152374[] = {{&S49846[34], 10}, {NULL}};
const Segment SGI152375[] = {{&S49846[27], 6}, {NULL}};
const Segment SGI152376[] = {{&S49846[97], 9}, {NULL}};
const Segment SGI152377[] = {{&S49846[70], 6}, {NULL}};
const Segment SGI152378[] = {{&S49847[34], 11}, {NULL}};
const Segment SGI152379[] = {{&S49847[27], 6}, {NULL}};
const Segment SGI152380[] = {{&S49847[98], 9}, {NULL}};
const Segment SGI152381[] = {{&S49847[71], 6}, {NULL}};
const Segment SGI152382[] = {{&S40309[34], 9}, {NULL}};
const Segment SGI152383[] = {{&S40309[27], 6}, {NULL}};
const Segment SGI152384[] = {{&S40309[96], 9}, {NULL}};
const Segment SGI152385[] = {{&S40309[69], 6}, {NULL}};
const Segment SGI152386[] = {{&S40310[34], 9}, {NULL}};
const Segment SGI152387[] = {{&S40310[27], 6}, {NULL}};
const Segment SGI152388[] = {{&S40310[96], 9}, {NULL}};
const Segment SGI152389[] = {{&S40310[69], 6}, {NULL}};
const Segment SGI152390[] = {{&S40311[34], 9}, {NULL}};
const Segment SGI152391[] = {{&S40311[27], 6}, {NULL}};
const Segment SGI152392[] = {{&S40311[96], 9}, {NULL}};
const Segment SGI152393[] = {{&S40311[69], 6}, {NULL}};
const Segment SGI152394[] = {{&S40312[34], 9}, {NULL}};
const Segment SGI152395[] = {{&S40312[27], 6}, {NULL}};
const Segment SGI152396[] = {{&S40312[96], 9}, {NULL}};
const Segment SGI152397[] = {{&S40312[69], 6}, {NULL}};
const Segment SGI152398[] = {{&S40825[44], 11}, {NULL}};
const Segment SGI152399[] = {{&S40825[44], 23}, {NULL}};
const Segment SGI152400[] = {{&S40825[30], 6}, {NULL}};
const Segment SGI152401[] = {{&S40825[116], 3}, {NULL}};
const Segment SGI152402[] = {{&S40825[81], 6}, {NULL}};
const Segment SGI152403[] = {{&S49910[44], 11}, {NULL}};
const Segment SGI152404[] = {{&S49910[44], 23}, {NULL}};
const Segment SGI152405[] = {{&S49910[30], 6}, {NULL}};
const Segment SGI152406[] = {{&S49910[31], 3}, {NULL}};
const Segment SGI152407[] = {{&S49910[81], 6}, {NULL}};
const Segment SGI152408[] = {{&S49911[44], 11}, {NULL}};
const Segment SGI152409[] = {{&S49911[44], 23}, {NULL}};
const Segment SGI152410[] = {{&S49911[30], 6}, {NULL}};
const Segment SGI152411[] = {{&S49911[84], 6}, {NULL}};
const Segment SGI152412[] = {{&S49912[38], 11}, {NULL}};
const Segment SGI152413[] = {{&S49912[31], 6}, {NULL}};
const Segment SGI152414[] = {{&S49913[44], 16}, {NULL}};
const Segment SGI152415[] = {{&S49913[44], 28}, {NULL}};
const Segment SGI152416[] = {{&S49913[30], 6}, {NULL}};
const Segment SGI152417[] = {{&S49913[31], 3}, {NULL}};
const Segment SGI152418[] = {{&S49913[86], 6}, {NULL}};
const Segment SGI152419[] = {{&S49914[44], 16}, {NULL}};
const Segment SGI152420[] = {{&S49914[44], 28}, {NULL}};
const Segment SGI152421[] = {{&S49914[30], 6}, {NULL}};
const Segment SGI152422[] = {{&S49914[31], 3}, {NULL}};
const Segment SGI152423[] = {{&S49914[86], 6}, {NULL}};
const Segment SGI152424[] = {{&S49915[44], 16}, {NULL}};
const Segment SGI152425[] = {{&S49915[44], 28}, {NULL}};
const Segment SGI152426[] = {{&S49915[30], 6}, {NULL}};
const Segment SGI152427[] = {{&S49915[31], 3}, {NULL}};
const Segment SGI152428[] = {{&S49915[86], 6}, {NULL}};
const Segment SGI152429[] = {{&S49916[44], 16}, {NULL}};
const Segment SGI152430[] = {{&S49916[44], 28}, {NULL}};
const Segment SGI152431[] = {{&S49916[30], 6}, {NULL}};
const Segment SGI152432[] = {{&S49916[31], 3}, {NULL}};
const Segment SGI152433[] = {{&S49916[86], 6}, {NULL}};
const Segment SGI152434[] = {{&S49917[44], 16}, {NULL}};
const Segment SGI152435[] = {{&S49917[44], 28}, {NULL}};
const Segment SGI152436[] = {{&S49917[30], 6}, {NULL}};
const Segment SGI152437[] = {{&S49917[31], 3}, {NULL}};
const Segment SGI152438[] = {{&S49917[86], 6}, {NULL}};
const Segment SGI152439[] = {{&S49919[44], 16}, {NULL}};
const Segment SGI152440[] = {{&S49919[44], 28}, {NULL}};
const Segment SGI152441[] = {{&S49919[30], 6}, {NULL}};
const Segment SGI152442[] = {{&S49919[31], 3}, {NULL}};
const Segment SGI152443[] = {{&S49919[86], 6}, {NULL}};
const Segment SGI152444[] = {{&S49920[44], 11}, {NULL}};
const Segment SGI152445[] = {{&S49920[44], 31}, {NULL}};
const Segment SGI152446[] = {{&S49920[30], 6}, {NULL}};
const Segment SGI152447[] = {{&S49920[31], 3}, {NULL}};
const Segment SGI152448[] = {{&S49920[89], 6}, {NULL}};
const Segment SGI152449[] = {{&S49921[44], 37}, {NULL}};
const Segment SGI152450[] = {{&S49921[44], 49}, {NULL}};
const Segment SGI152451[] = {{&S49921[30], 6}, {NULL}};
const Segment SGI152452[] = {{&S49921[31], 3}, {NULL}};
const Segment SGI152453[] = {{&S49921[107], 6}, {NULL}};
const Segment SGI152454[] = {{&S40899[34], 10}, {NULL}};
const Segment SGI152455[] = {{&S40899[27], 6}, {NULL}};
const Segment SGI152456[] = {{&S40899[97], 9}, {NULL}};
const Segment SGI152457[] = {{&S40899[70], 6}, {NULL}};
const Segment SGI152458[] = {{&S40900[34], 10}, {NULL}};
const Segment SGI152459[] = {{&S40900[27], 6}, {NULL}};
const Segment SGI152460[] = {{&S40900[97], 9}, {NULL}};
const Segment SGI152461[] = {{&S40900[70], 6}, {NULL}};
const Segment SGI152462[] = {{&S49957[34], 8}, {NULL}};
const Segment SGI152463[] = {{&S49957[27], 6}, {NULL}};
const Segment SGI152464[] = {{&S49957[95], 9}, {NULL}};
const Segment SGI152465[] = {{&S49957[68], 6}, {NULL}};
const Segment SGI152466[] = {{&S49965[34], 6}, {NULL}};
const Segment SGI152467[] = {{&S49965[27], 6}, {NULL}};
const Segment SGI152468[] = {{&S49965[93], 9}, {NULL}};
const Segment SGI152469[] = {{&S49965[66], 6}, {NULL}};
const Segment SGI152470[] = {{&S40427[34], 9}, {NULL}};
const Segment SGI152471[] = {{&S40427[27], 6}, {NULL}};
const Segment SGI152472[] = {{&S40427[96], 9}, {NULL}};
const Segment SGI152473[] = {{&S40427[69], 6}, {NULL}};
const Segment SGI152474[] = {{&S40428[34], 11}, {NULL}};
const Segment SGI152475[] = {{&S40428[27], 6}, {NULL}};
const Segment SGI152476[] = {{&S40428[98], 9}, {NULL}};
const Segment SGI152477[] = {{&S40428[71], 6}, {NULL}};
const Segment SGI152478[] = {{&S40429[34], 8}, {NULL}};
const Segment SGI152479[] = {{&S40429[27], 6}, {NULL}};
const Segment SGI152480[] = {{&S40429[95], 9}, {NULL}};
const Segment SGI152481[] = {{&S40429[68], 6}, {NULL}};
const Segment SGI152482[] = {{&S40430[34], 18}, {NULL}};
const Segment SGI152483[] = {{&S40430[27], 6}, {NULL}};
const Segment SGI152484[] = {{&S40430[105], 9}, {NULL}};
const Segment SGI152485[] = {{&S40430[78], 6}, {NULL}};
const Segment SGI152486[] = {{&S49989[44], 9}, {NULL}};
const Segment SGI152487[] = {{&S49989[44], 21}, {NULL}};
const Segment SGI152488[] = {{&S49989[30], 6}, {NULL}};
const Segment SGI152489[] = {{&S49989[31], 3}, {NULL}};
const Segment SGI152490[] = {{&S49989[79], 6}, {NULL}};
const Segment SGI152491[] = {{&S40431[34], 14}, {NULL}};
const Segment SGI152492[] = {{&S40431[27], 6}, {NULL}};
const Segment SGI152493[] = {{&S40431[101], 9}, {NULL}};
const Segment SGI152494[] = {{&S40431[74], 6}, {NULL}};
const Segment SGI152495[] = {{&S49990[44], 4}, {NULL}};
const Segment SGI152496[] = {{&S49990[44], 41}, {NULL}};
const Segment SGI152497[] = {{&S49990[30], 6}, {NULL}};
const Segment SGI152498[] = {{&S49990[31], 3}, {NULL}};
const Segment SGI152499[] = {{&S49990[99], 6}, {NULL}};
const Segment SGI152500[] = {{&S49991[44], 9}, {NULL}};
const Segment SGI152501[] = {{&S49991[44], 21}, {NULL}};
const Segment SGI152502[] = {{&S49991[30], 6}, {NULL}};
const Segment SGI152503[] = {{&S49991[82], 6}, {NULL}};
const Segment SGI152504[] = {{&S49992[44], 4}, {NULL}};
const Segment SGI152505[] = {{&S49992[44], 41}, {NULL}};
const Segment SGI152506[] = {{&S49992[30], 6}, {NULL}};
const Segment SGI152507[] = {{&S49992[31], 3}, {NULL}};
const Segment SGI152508[] = {{&S49992[99], 6}, {NULL}};
const Segment SGI152509[] = {{&S49993[44], 4}, {NULL}};
const Segment SGI152510[] = {{&S49993[44], 41}, {NULL}};
const Segment SGI152511[] = {{&S49993[30], 6}, {NULL}};
const Segment SGI152512[] = {{&S49993[31], 3}, {NULL}};
const Segment SGI152513[] = {{&S49993[99], 6}, {NULL}};
const Segment SGI152514[] = {{&S49995[44], 4}, {NULL}};
const Segment SGI152515[] = {{&S49995[44], 16}, {NULL}};
const Segment SGI152516[] = {{&S49995[30], 6}, {NULL}};
const Segment SGI152517[] = {{&S49995[31], 3}, {NULL}};
const Segment SGI152518[] = {{&S49995[74], 6}, {NULL}};
const Segment SGI152519[] = {{&S49997[44], 10}, {NULL}};
const Segment SGI152520[] = {{&S49997[44], 22}, {NULL}};
const Segment SGI152521[] = {{&S49997[30], 6}, {NULL}};
const Segment SGI152522[] = {{&S49997[31], 3}, {NULL}};
const Segment SGI152523[] = {{&S49997[80], 6}, {NULL}};
const Segment SGI152524[] = {{&S49998[38], 9}, {NULL}};
const Segment SGI152525[] = {{&S49998[31], 6}, {NULL}};
const Segment SGI152526[] = {{&S50000[34], 18}, {NULL}};
const Segment SGI152527[] = {{&S50000[27], 6}, {NULL}};
const Segment SGI152528[] = {{&S50000[105], 9}, {NULL}};
const Segment SGI152529[] = {{&S50000[78], 6}, {NULL}};
const Segment SGI152530[] = {{&S50001[34], 11}, {NULL}};
const Segment SGI152531[] = {{&S50001[27], 6}, {NULL}};
const Segment SGI152532[] = {{&S50001[98], 9}, {NULL}};
const Segment SGI152533[] = {{&S50001[71], 6}, {NULL}};
const Segment SGI152534[] = {{&S50002[34], 10}, {NULL}};
const Segment SGI152535[] = {{&S50002[27], 6}, {NULL}};
const Segment SGI152536[] = {{&S50002[97], 9}, {NULL}};
const Segment SGI152537[] = {{&S50002[70], 6}, {NULL}};
const Segment SGI152538[] = {{&S50003[34], 11}, {NULL}};
const Segment SGI152539[] = {{&S50003[27], 6}, {NULL}};
const Segment SGI152540[] = {{&S50003[98], 9}, {NULL}};
const Segment SGI152541[] = {{&S50003[71], 6}, {NULL}};
const Segment SGI152542[] = {{&S50004[34], 11}, {NULL}};
const Segment SGI152543[] = {{&S50004[27], 6}, {NULL}};
const Segment SGI152544[] = {{&S50004[98], 9}, {NULL}};
const Segment SGI152545[] = {{&S50004[71], 6}, {NULL}};
const Segment SGI152546[] = {{&S50005[34], 11}, {NULL}};
const Segment SGI152547[] = {{&S50005[27], 6}, {NULL}};
const Segment SGI152548[] = {{&S50005[98], 9}, {NULL}};
const Segment SGI152549[] = {{&S50005[71], 6}, {NULL}};
const Segment SGI152550[] = {{&S50006[34], 11}, {NULL}};
const Segment SGI152551[] = {{&S50006[27], 6}, {NULL}};
const Segment SGI152552[] = {{&S50006[98], 9}, {NULL}};
const Segment SGI152553[] = {{&S50006[71], 6}, {NULL}};
const Segment SGI152554[] = {{&S50007[34], 11}, {NULL}};
const Segment SGI152555[] = {{&S50007[27], 6}, {NULL}};
const Segment SGI152556[] = {{&S50007[98], 9}, {NULL}};
const Segment SGI152557[] = {{&S50007[71], 6}, {NULL}};
const Segment SGI152558[] = {{&S40989[44], 10}, {NULL}};
const Segment SGI152559[] = {{&S40989[44], 22}, {NULL}};
const Segment SGI152560[] = {{&S40989[30], 6}, {NULL}};
const Segment SGI152561[] = {{&S40989[131], 3}, {NULL}};
const Segment SGI152562[] = {{&S40989[96], 6}, {NULL}};
const Segment SGI152563[] = {{&S40989[68], 8}, {NULL}};
const Segment SGI152564[] = {{&S40990[44], 10}, {NULL}};
const Segment SGI152565[] = {{&S40990[44], 22}, {NULL}};
const Segment SGI152566[] = {{&S40990[30], 6}, {NULL}};
const Segment SGI152567[] = {{&S40990[131], 3}, {NULL}};
const Segment SGI152568[] = {{&S40990[96], 6}, {NULL}};
const Segment SGI152569[] = {{&S40990[68], 8}, {NULL}};
const Segment SGI152570[] = {{&S40991[44], 11}, {NULL}};
const Segment SGI152571[] = {{&S40991[44], 23}, {NULL}};
const Segment SGI152572[] = {{&S40991[30], 6}, {NULL}};
const Segment SGI152573[] = {{&S40991[132], 3}, {NULL}};
const Segment SGI152574[] = {{&S40991[97], 6}, {NULL}};
const Segment SGI152575[] = {{&S40991[69], 8}, {NULL}};
const Segment SGI152576[] = {{&S40992[44], 10}, {NULL}};
const Segment SGI152577[] = {{&S40992[44], 22}, {NULL}};
const Segment SGI152578[] = {{&S40992[30], 6}, {NULL}};
const Segment SGI152579[] = {{&S40992[131], 3}, {NULL}};
const Segment SGI152580[] = {{&S40992[96], 6}, {NULL}};
const Segment SGI152581[] = {{&S40992[68], 8}, {NULL}};
const Segment SGI152582[] = {{&S40995[44], 11}, {NULL}};
const Segment SGI152583[] = {{&S40995[44], 42}, {NULL}};
const Segment SGI152584[] = {{&S40995[30], 6}, {NULL}};
const Segment SGI152585[] = {{&S40995[156], 3}, {NULL}};
const Segment SGI152586[] = {{&S40995[121], 6}, {NULL}};
const Segment SGI152587[] = {{&S40995[88], 8}, {NULL}};
const Segment SGI152588[] = {{&S40996[44], 10}, {NULL}};
const Segment SGI152589[] = {{&S40996[44], 19}, {NULL}};
const Segment SGI152590[] = {{&S40996[30], 6}, {NULL}};
const Segment SGI152591[] = {{&S40996[128], 3}, {NULL}};
const Segment SGI152592[] = {{&S40996[93], 6}, {NULL}};
const Segment SGI152593[] = {{&S40996[65], 8}, {NULL}};
const Segment SGI152594[] = {{&S50038[44], 10}, {NULL}};
const Segment SGI152595[] = {{&S50038[44], 22}, {NULL}};
const Segment SGI152596[] = {{&S50038[30], 6}, {NULL}};
const Segment SGI152597[] = {{&S50038[31], 3}, {NULL}};
const Segment SGI152598[] = {{&S50038[80], 6}, {NULL}};
const Segment SGI152599[] = {{&S50039[38], 10}, {NULL}};
const Segment SGI152600[] = {{&S50039[31], 6}, {NULL}};
const Segment SGI152601[] = {{&S50040[44], 10}, {NULL}};
const Segment SGI152602[] = {{&S50040[44], 22}, {NULL}};
const Segment SGI152603[] = {{&S50040[30], 6}, {NULL}};
const Segment SGI152604[] = {{&S50040[31], 3}, {NULL}};
const Segment SGI152605[] = {{&S50040[80], 6}, {NULL}};
const Segment SGI152606[] = {{&S50041[44], 10}, {NULL}};
const Segment SGI152607[] = {{&S50041[44], 22}, {NULL}};
const Segment SGI152608[] = {{&S50041[30], 6}, {NULL}};
const Segment SGI152609[] = {{&S50041[80], 6}, {NULL}};
const Segment SGI152610[] = {{&S50042[30], 1}, {NULL}};
const Segment SGI152611[] = {{&S50042[63], 9}, {NULL}};
const Segment SGI152612[] = {{&S50042[49], 6}, {NULL}};
const Segment SGI152613[] = {{&S50043[60], 10}, {NULL}};
const Segment SGI152614[] = {{&S50043[60], 28}, {NULL}};
const Segment SGI152615[] = {{&S50043[46], 6}, {NULL}};
const Segment SGI152616[] = {{&S50044[59], 10}, {NULL}};
const Segment SGI152617[] = {{&S50044[59], 28}, {NULL}};
const Segment SGI152618[] = {{&S50044[45], 6}, {NULL}};
const Segment SGI152619[] = {{&S50045[42], 10}, {NULL}};
const Segment SGI152620[] = {{&S50045[30], 6}, {NULL}};
const Segment SGI152621[] = {{&S50045[31], 3}, {NULL}};
const Segment SGI152622[] = {{&S50045[78], 6}, {NULL}};
const Segment SGI152623[] = {{&S50046[44], 10}, {NULL}};
const Segment SGI152624[] = {{&S50046[44], 22}, {NULL}};
const Segment SGI152625[] = {{&S50046[30], 6}, {NULL}};
const Segment SGI152626[] = {{&S50046[83], 6}, {NULL}};
const Segment SGI152627[] = {{&S50047[44], 10}, {NULL}};
const Segment SGI152628[] = {{&S50047[44], 22}, {NULL}};
const Segment SGI152629[] = {{&S50047[30], 6}, {NULL}};
const Segment SGI152630[] = {{&S50047[31], 3}, {NULL}};
const Segment SGI152631[] = {{&S50047[80], 6}, {NULL}};
const Segment SGI152632[] = {{&S50048[44], 10}, {NULL}};
const Segment SGI152633[] = {{&S50048[44], 22}, {NULL}};
const Segment SGI152634[] = {{&S50048[30], 6}, {NULL}};
const Segment SGI152635[] = {{&S50048[115], 3}, {NULL}};
const Segment SGI152636[] = {{&S50048[80], 6}, {NULL}};
const Segment SGI152637[] = {{&S50048[135], 8}, {NULL}};
const Segment SGI152638[] = {{&S50060[34], 10}, {NULL}};
const Segment SGI152639[] = {{&S50060[27], 6}, {NULL}};
const Segment SGI152640[] = {{&S50060[97], 9}, {NULL}};
const Segment SGI152641[] = {{&S50060[70], 6}, {NULL}};
const Segment SGI152642[] = {{&S50062[34], 11}, {NULL}};
const Segment SGI152643[] = {{&S50062[27], 6}, {NULL}};
const Segment SGI152644[] = {{&S50062[98], 9}, {NULL}};
const Segment SGI152645[] = {{&S50062[71], 6}, {NULL}};
const Segment SGI152646[] = {{&S50063[34], 11}, {NULL}};
const Segment SGI152647[] = {{&S50063[27], 6}, {NULL}};
const Segment SGI152648[] = {{&S50063[98], 9}, {NULL}};
const Segment SGI152649[] = {{&S50063[71], 6}, {NULL}};
const Segment SGI152650[] = {{&S50064[34], 11}, {NULL}};
const Segment SGI152651[] = {{&S50064[27], 6}, {NULL}};
const Segment SGI152652[] = {{&S50064[98], 9}, {NULL}};
const Segment SGI152653[] = {{&S50064[71], 6}, {NULL}};
const Segment SGI152654[] = {{&S50065[34], 10}, {NULL}};
const Segment SGI152655[] = {{&S50065[27], 6}, {NULL}};
const Segment SGI152656[] = {{&S50065[97], 9}, {NULL}};
const Segment SGI152657[] = {{&S50065[70], 6}, {NULL}};
const Segment SGI152658[] = {{&S50066[34], 8}, {NULL}};
const Segment SGI152659[] = {{&S50066[27], 6}, {NULL}};
const Segment SGI152660[] = {{&S50066[95], 9}, {NULL}};
const Segment SGI152661[] = {{&S50066[68], 6}, {NULL}};
const Segment SGI152662[] = {{&S50067[34], 11}, {NULL}};
const Segment SGI152663[] = {{&S50067[27], 6}, {NULL}};
const Segment SGI152664[] = {{&S50067[98], 9}, {NULL}};
const Segment SGI152665[] = {{&S50067[71], 6}, {NULL}};
const Segment SGI152666[] = {{&S41073[34], 11}, {NULL}};
const Segment SGI152667[] = {{&S41073[27], 6}, {NULL}};
const Segment SGI152668[] = {{&S41073[98], 9}, {NULL}};
const Segment SGI152669[] = {{&S41073[71], 6}, {NULL}};
const Segment SGI152670[] = {{&S41074[34], 11}, {NULL}};
const Segment SGI152671[] = {{&S41074[27], 6}, {NULL}};
const Segment SGI152672[] = {{&S41074[98], 9}, {NULL}};
const Segment SGI152673[] = {{&S41074[71], 6}, {NULL}};
const Segment SGI152674[] = {{&S41075[34], 11}, {NULL}};
const Segment SGI152675[] = {{&S41075[27], 6}, {NULL}};
const Segment SGI152676[] = {{&S41075[98], 9}, {NULL}};
const Segment SGI152677[] = {{&S41075[71], 6}, {NULL}};
const Segment SGI152678[] = {{&S50107[34], 30}, {NULL}};
const Segment SGI152679[] = {{&S50107[27], 6}, {NULL}};
const Segment SGI152680[] = {{&S50107[117], 9}, {NULL}};
const Segment SGI152681[] = {{&S50107[90], 6}, {NULL}};
const Segment SGI152682[] = {{&S46144[44], 7}, {NULL}};
const Segment SGI152683[] = {{&S46144[44], 19}, {NULL}};
const Segment SGI152684[] = {{&S46144[30], 6}, {NULL}};
const Segment SGI152685[] = {{&S46144[31], 3}, {NULL}};
const Segment SGI152686[] = {{&S46144[77], 6}, {NULL}};
const Segment SGI152687[] = {{&S46145[44], 7}, {NULL}};
const Segment SGI152688[] = {{&S46145[44], 19}, {NULL}};
const Segment SGI152689[] = {{&S46145[30], 6}, {NULL}};
const Segment SGI152690[] = {{&S46145[31], 3}, {NULL}};
const Segment SGI152691[] = {{&S46145[77], 5}, {NULL}};
const Segment SGI152692[] = {{&S46147[44], 10}, {NULL}};
const Segment SGI152693[] = {{&S46147[44], 21}, {NULL}};
const Segment SGI152694[] = {{&S46147[30], 6}, {NULL}};
const Segment SGI152695[] = {{&S46147[31], 3}, {NULL}};
const Segment SGI152696[] = {{&S46147[79], 5}, {NULL}};
const Segment SGI152697[] = {{&S50184[34], 10}, {NULL}};
const Segment SGI152698[] = {{&S50184[27], 6}, {NULL}};
const Segment SGI152699[] = {{&S46148[44], 7}, {NULL}};
const Segment SGI152700[] = {{&S46148[44], 19}, {NULL}};
const Segment SGI152701[] = {{&S26431[38], 9}, {NULL}};
const Segment SGI152702[] = {{&S46148[30], 6}, {NULL}};
const Segment SGI152703[] = {{&S26431[31], 6}, {NULL}};
const Segment SGI152704[] = {{&S46148[31], 3}, {NULL}};
const Segment SGI152705[] = {{&S46148[77], 6}, {NULL}};
const Segment SGI152706[] = {{&S50184[97], 9}, {NULL}};
const Segment SGI152707[] = {{&S50184[70], 6}, {NULL}};
const Segment SGI152708[] = {{&S46149[44], 10}, {NULL}};
const Segment SGI152709[] = {{&S46149[44], 21}, {NULL}};
const Segment SGI152710[] = {{&S46149[30], 6}, {NULL}};
const Segment SGI152711[] = {{&S46149[31], 3}, {NULL}};
const Segment SGI152712[] = {{&S46149[79], 5}, {NULL}};
const Segment SGI152713[] = {{&S26432[44], 9}, {NULL}};
const Segment SGI152714[] = {{&S26432[44], 20}, {NULL}};
const Segment SGI152715[] = {{&S26432[30], 6}, {NULL}};
const Segment SGI152716[] = {{&S26432[113], 3}, {NULL}};
const Segment SGI152717[] = {{&S26432[78], 6}, {NULL}};
const Segment SGI152718[] = {{&S50186[34], 11}, {NULL}};
const Segment SGI152719[] = {{&S50186[27], 6}, {NULL}};
const Segment SGI152720[] = {{&S46151[44], 10}, {NULL}};
const Segment SGI152721[] = {{&S46151[44], 22}, {NULL}};
const Segment SGI152722[] = {{&S50186[98], 9}, {NULL}};
const Segment SGI152723[] = {{&S46151[30], 6}, {NULL}};
const Segment SGI152724[] = {{&S50186[71], 6}, {NULL}};
const Segment SGI152725[] = {{&S46151[31], 3}, {NULL}};
const Segment SGI152726[] = {{&S46151[80], 5}, {NULL}};
const Segment SGI152727[] = {{&S26433[44], 9}, {NULL}};
const Segment SGI152728[] = {{&S26433[44], 20}, {NULL}};
const Segment SGI152729[] = {{&S26433[30], 6}, {NULL}};
const Segment SGI152730[] = {{&S26433[113], 3}, {NULL}};
const Segment SGI152731[] = {{&S26433[78], 6}, {NULL}};
const Segment SGI152732[] = {{&S50187[34], 10}, {NULL}};
const Segment SGI152733[] = {{&S50187[27], 6}, {NULL}};
const Segment SGI152734[] = {{&S50187[97], 9}, {NULL}};
const Segment SGI152735[] = {{&S50187[70], 6}, {NULL}};
const Segment SGI152736[] = {{&S46153[38], 7}, {NULL}};
const Segment SGI152737[] = {{&S26434[44], 13}, {NULL}};
const Segment SGI152738[] = {{&S46153[31], 6}, {NULL}};
const Segment SGI152739[] = {{&S26434[44], 24}, {NULL}};
const Segment SGI152740[] = {{&S26434[30], 6}, {NULL}};
const Segment SGI152741[] = {{&S26434[117], 3}, {NULL}};
const Segment SGI152742[] = {{&S26434[82], 6}, {NULL}};
const Segment SGI152743[] = {{&S50188[34], 10}, {NULL}};
const Segment SGI152744[] = {{&S50188[27], 6}, {NULL}};
const Segment SGI152745[] = {{&S50188[97], 9}, {NULL}};
const Segment SGI152746[] = {{&S50188[70], 6}, {NULL}};
const Segment SGI152747[] = {{&S26435[44], 9}, {NULL}};
const Segment SGI152748[] = {{&S26435[44], 21}, {NULL}};
const Segment SGI152749[] = {{&S26435[30], 6}, {NULL}};
const Segment SGI152750[] = {{&S26435[114], 3}, {NULL}};
const Segment SGI152751[] = {{&S26435[79], 6}, {NULL}};
const Segment SGI152752[] = {{&S46157[44], 10}, {NULL}};
const Segment SGI152753[] = {{&S46157[44], 33}, {NULL}};
const Segment SGI152754[] = {{&S46157[30], 6}, {NULL}};
const Segment SGI152755[] = {{&S46157[31], 3}, {NULL}};
const Segment SGI152756[] = {{&S46157[91], 5}, {NULL}};
const Segment SGI152757[] = {{&S50190[34], 11}, {NULL}};
const Segment SGI152758[] = {{&S50190[27], 6}, {NULL}};
const Segment SGI152759[] = {{&S50190[98], 9}, {NULL}};
const Segment SGI152760[] = {{&S50190[71], 6}, {NULL}};
const Segment SGI152761[] = {{&S26437[44], 14}, {NULL}};
const Segment SGI152762[] = {{&S26437[44], 25}, {NULL}};
const Segment SGI152763[] = {{&S26437[30], 6}, {NULL}};
const Segment SGI152764[] = {{&S26437[118], 3}, {NULL}};
const Segment SGI152765[] = {{&S26437[83], 6}, {NULL}};
const Segment SGI152766[] = {{&S50191[34], 11}, {NULL}};
const Segment SGI152767[] = {{&S50191[27], 6}, {NULL}};
const Segment SGI152768[] = {{&S50191[98], 9}, {NULL}};
const Segment SGI152769[] = {{&S26438[38], 14}, {NULL}};
const Segment SGI152770[] = {{&S50191[71], 6}, {NULL}};
const Segment SGI152771[] = {{&S26438[31], 6}, {NULL}};
const Segment SGI152772[] = {{&S50192[34], 11}, {NULL}};
const Segment SGI152773[] = {{&S50192[27], 6}, {NULL}};
const Segment SGI152774[] = {{&S46162[44], 39}, {NULL}};
const Segment SGI152775[] = {{&S26439[44], 11}, {NULL}};
const Segment SGI152776[] = {{&S46162[44], 51}, {NULL}};
const Segment SGI152777[] = {{&S26439[44], 22}, {NULL}};
const Segment SGI152778[] = {{&S46162[30], 6}, {NULL}};
const Segment SGI152779[] = {{&S26439[30], 6}, {NULL}};
const Segment SGI152780[] = {{&S46162[31], 3}, {NULL}};
const Segment SGI152781[] = {{&S50192[98], 9}, {NULL}};
const Segment SGI152782[] = {{&S46162[109], 6}, {NULL}};
const Segment SGI152783[] = {{&S50192[71], 6}, {NULL}};
const Segment SGI152784[] = {{&S46163[44], 7}, {NULL}};
const Segment SGI152785[] = {{&S46163[44], 19}, {NULL}};
const Segment SGI152786[] = {{&S50193[34], 8}, {NULL}};
const Segment SGI152787[] = {{&S46163[30], 6}, {NULL}};
const Segment SGI152788[] = {{&S46163[31], 3}, {NULL}};
const Segment SGI152789[] = {{&S50193[27], 6}, {NULL}};
const Segment SGI152790[] = {{&S46163[77], 5}, {NULL}};
const Segment SGI152791[] = {{&S50193[95], 9}, {NULL}};
const Segment SGI152792[] = {{&S50193[68], 6}, {NULL}};
const Segment SGI152793[] = {{&S46164[44], 7}, {NULL}};
const Segment SGI152794[] = {{&S46164[44], 19}, {NULL}};
const Segment SGI152795[] = {{&S46164[30], 6}, {NULL}};
const Segment SGI152796[] = {{&S46164[31], 3}, {NULL}};
const Segment SGI152797[] = {{&S46164[77], 6}, {NULL}};
const Segment SGI152798[] = {{&S46165[38], 10}, {NULL}};
const Segment SGI152799[] = {{&S46165[31], 6}, {NULL}};
const Segment SGI152800[] = {{&S50195[34], 11}, {NULL}};
const Segment SGI152801[] = {{&S50195[27], 6}, {NULL}};
const Segment SGI152802[] = {{&S46166[44], 8}, {NULL}};
const Segment SGI152803[] = {{&S50195[98], 9}, {NULL}};
const Segment SGI152804[] = {{&S46166[44], 20}, {NULL}};
const Segment SGI152805[] = {{&S50195[71], 6}, {NULL}};
const Segment SGI152806[] = {{&S46166[30], 6}, {NULL}};
const Segment SGI152807[] = {{&S46166[31], 3}, {NULL}};
const Segment SGI152808[] = {{&S46166[78], 6}, {NULL}};
const Segment SGI152809[] = {{&S40706[44], 10}, {NULL}};
const Segment SGI152810[] = {{&S40706[44], 22}, {NULL}};
const Segment SGI152811[] = {{&S40706[30], 6}, {NULL}};
const Segment SGI152812[] = {{&S40706[115], 3}, {NULL}};
const Segment SGI152813[] = {{&S40706[80], 6}, {NULL}};
const Segment SGI152814[] = {{&S40707[38], 10}, {NULL}};
const Segment SGI152815[] = {{&S40707[31], 6}, {NULL}};
const Segment SGI152816[] = {{&S41236[44], 5}, {NULL}};
const Segment SGI152817[] = {{&S41236[44], 17}, {NULL}};
const Segment SGI152818[] = {{&S41236[30], 6}, {NULL}};
const Segment SGI152819[] = {{&S41236[110], 3}, {NULL}};
const Segment SGI152820[] = {{&S41236[75], 6}, {NULL}};
const Segment SGI152821[] = {{&S41237[44], 12}, {NULL}};
const Segment SGI152822[] = {{&S41237[44], 24}, {NULL}};
const Segment SGI152823[] = {{&S41237[30], 6}, {NULL}};
const Segment SGI152824[] = {{&S41237[117], 3}, {NULL}};
const Segment SGI152825[] = {{&S41237[82], 6}, {NULL}};
const Segment SGI152826[] = {{&S41238[44], 9}, {NULL}};
const Segment SGI152827[] = {{&S41238[44], 21}, {NULL}};
const Segment SGI152828[] = {{&S41238[30], 6}, {NULL}};
const Segment SGI152829[] = {{&S41238[114], 3}, {NULL}};
const Segment SGI152830[] = {{&S41238[79], 6}, {NULL}};
const Segment SGI152831[] = {{&S41240[38], 9}, {NULL}};
const Segment SGI152832[] = {{&S41240[31], 6}, {NULL}};
const Segment SGI152833[] = {{&S41242[44], 5}, {NULL}};
const Segment SGI152834[] = {{&S41242[44], 16}, {NULL}};
const Segment SGI152835[] = {{&S41242[30], 6}, {NULL}};
const Segment SGI152836[] = {{&S41242[108], 3}, {NULL}};
const Segment SGI152837[] = {{&S41242[74], 5}, {NULL}};
const Segment SGI152838[] = {{&S41245[44], 11}, {NULL}};
const Segment SGI152839[] = {{&S41245[44], 23}, {NULL}};
const Segment SGI152840[] = {{&S41245[30], 6}, {NULL}};
const Segment SGI152841[] = {{&S41245[116], 3}, {NULL}};
const Segment SGI152842[] = {{&S41245[81], 6}, {NULL}};
const Segment SGI152843[] = {{&S40761[34], 14}, {NULL}};
const Segment SGI152844[] = {{&S40761[27], 6}, {NULL}};
const Segment SGI152845[] = {{&S40761[101], 9}, {NULL}};
const Segment SGI152846[] = {{&S40761[74], 6}, {NULL}};
const Segment SGI152847[] = {{&S41246[44], 22}, {NULL}};
const Segment SGI152848[] = {{&S41246[44], 34}, {NULL}};
const Segment SGI152849[] = {{&S41246[30], 6}, {NULL}};
const Segment SGI152850[] = {{&S41246[127], 3}, {NULL}};
const Segment SGI152851[] = {{&S41246[92], 6}, {NULL}};
const Segment SGI152852[] = {{&S40762[34], 14}, {NULL}};
const Segment SGI152853[] = {{&S40762[27], 6}, {NULL}};
const Segment SGI152854[] = {{&S40762[101], 9}, {NULL}};
const Segment SGI152855[] = {{&S40762[74], 6}, {NULL}};
const Segment SGI152856[] = {{&S41248[44], 11}, {NULL}};
const Segment SGI152857[] = {{&S41248[44], 23}, {NULL}};
const Segment SGI152858[] = {{&S41248[30], 6}, {NULL}};
const Segment SGI152859[] = {{&S41248[116], 3}, {NULL}};
const Segment SGI152860[] = {{&S41248[81], 6}, {NULL}};
const Segment SGI152861[] = {{&S41249[44], 11}, {NULL}};
const Segment SGI152862[] = {{&S36041[34], 9}, {NULL}};
const Segment SGI152863[] = {{&S41249[44], 23}, {NULL}};
const Segment SGI152864[] = {{&S41249[30], 6}, {NULL}};
const Segment SGI152865[] = {{&S36041[27], 6}, {NULL}};
const Segment SGI152866[] = {{&S41249[116], 3}, {NULL}};
const Segment SGI152867[] = {{&S41249[81], 6}, {NULL}};
const Segment SGI152868[] = {{&S36041[96], 9}, {NULL}};
const Segment SGI152869[] = {{&S36041[69], 6}, {NULL}};
const Segment SGI152870[] = {{&S41250[44], 5}, {NULL}};
const Segment SGI152871[] = {{&S41250[44], 17}, {NULL}};
const Segment SGI152872[] = {{&S41250[30], 6}, {NULL}};
const Segment SGI152873[] = {{&S41250[110], 3}, {NULL}};
const Segment SGI152874[] = {{&S41250[75], 6}, {NULL}};
const Segment SGI152875[] = {{&S41251[38], 11}, {NULL}};
const Segment SGI152876[] = {{&S41251[31], 6}, {NULL}};
const Segment SGI152877[] = {{&S41252[44], 11}, {NULL}};
const Segment SGI152878[] = {{&S41252[44], 23}, {NULL}};
const Segment SGI152879[] = {{&S41252[30], 6}, {NULL}};
const Segment SGI152880[] = {{&S41252[116], 3}, {NULL}};
const Segment SGI152881[] = {{&S41252[81], 6}, {NULL}};
const Segment SGI152882[] = {{&S41253[44], 9}, {NULL}};
const Segment SGI152883[] = {{&S50272[34], 9}, {NULL}};
const Segment SGI152884[] = {{&S41253[44], 21}, {NULL}};
const Segment SGI152885[] = {{&S41253[30], 6}, {NULL}};
const Segment SGI152886[] = {{&S50272[27], 6}, {NULL}};
const Segment SGI152887[] = {{&S41253[114], 3}, {NULL}};
const Segment SGI152888[] = {{&S41253[79], 6}, {NULL}};
const Segment SGI152889[] = {{&S50272[96], 9}, {NULL}};
const Segment SGI152890[] = {{&S50272[69], 6}, {NULL}};
const Segment SGI152891[] = {{&S50290[34], 7}, {NULL}};
const Segment SGI152892[] = {{&S50290[27], 6}, {NULL}};
const Segment SGI152893[] = {{&S50290[97], 9}, {NULL}};
const Segment SGI152894[] = {{&S50290[70], 6}, {NULL}};
const Segment SGI152895[] = {{&S50291[34], 8}, {NULL}};
const Segment SGI152896[] = {{&S50291[27], 6}, {NULL}};
const Segment SGI152897[] = {{&S50291[100], 9}, {NULL}};
const Segment SGI152898[] = {{&S50291[73], 6}, {NULL}};
const Segment SGI152899[] = {{&S50292[34], 7}, {NULL}};
const Segment SGI152900[] = {{&S50292[27], 6}, {NULL}};
const Segment SGI152901[] = {{&S50292[97], 9}, {NULL}};
const Segment SGI152902[] = {{&S50292[70], 6}, {NULL}};
const Segment SGI152903[] = {{&S40818[34], 9}, {NULL}};
const Segment SGI152904[] = {{&S40818[27], 6}, {NULL}};
const Segment SGI152905[] = {{&S40818[96], 9}, {NULL}};
const Segment SGI152906[] = {{&S40818[69], 6}, {NULL}};
const Segment SGI152907[] = {{&S50307[34], 7}, {NULL}};
const Segment SGI152908[] = {{&S50307[27], 6}, {NULL}};
const Segment SGI152909[] = {{&S50307[94], 9}, {NULL}};
const Segment SGI152910[] = {{&S50307[67], 6}, {NULL}};
const Segment SGI152911[] = {{&S40819[34], 9}, {NULL}};
const Segment SGI152912[] = {{&S40819[27], 6}, {NULL}};
const Segment SGI152913[] = {{&S40819[96], 9}, {NULL}};
const Segment SGI152914[] = {{&S40819[69], 6}, {NULL}};
const Segment SGI152915[] = {{&S40820[34], 9}, {NULL}};
const Segment SGI152916[] = {{&S40820[27], 6}, {NULL}};
const Segment SGI152917[] = {{&S40820[96], 9}, {NULL}};
const Segment SGI152918[] = {{&S40820[69], 6}, {NULL}};
const Segment SGI152919[] = {{&S40821[34], 9}, {NULL}};
const Segment SGI152920[] = {{&S40821[27], 6}, {NULL}};
const Segment SGI152921[] = {{&S40821[96], 9}, {NULL}};
const Segment SGI152922[] = {{&S40821[69], 6}, {NULL}};
const Segment SGI152923[] = {{&S40822[44], 11}, {NULL}};
const Segment SGI152924[] = {{&S40822[44], 23}, {NULL}};
const Segment SGI152925[] = {{&S40822[30], 6}, {NULL}};
const Segment SGI152926[] = {{&S40822[116], 3}, {NULL}};
const Segment SGI152927[] = {{&S40822[81], 6}, {NULL}};
const Segment SGI152928[] = {{&S40823[38], 11}, {NULL}};
const Segment SGI152929[] = {{&S40823[31], 6}, {NULL}};
const Segment SGI152930[] = {{&S40824[42], 11}, {NULL}};
const Segment SGI152931[] = {{&S40824[30], 6}, {NULL}};
const Segment SGI152932[] = {{&S40824[114], 3}, {NULL}};
const Segment SGI152933[] = {{&S40824[79], 6}, {NULL}};
const Segment SGI152934[] = {{&S26439[115], 3}, {NULL}};
const Segment SGI152935[] = {{&S26439[80], 6}, {NULL}};
const Segment SGI152936[] = {{&S26440[44], 25}, {NULL}};
const Segment SGI152937[] = {{&S26440[44], 36}, {NULL}};
const Segment SGI152938[] = {{&S26440[30], 6}, {NULL}};
const Segment SGI152939[] = {{&S26440[129], 3}, {NULL}};
const Segment SGI152940[] = {{&S26440[94], 6}, {NULL}};
const Segment SGI152941[] = {{&S26441[38], 5}, {NULL}};
const Segment SGI152942[] = {{&S26441[31], 6}, {NULL}};
const Segment SGI152943[] = {{&S26442[44], 20}, {NULL}};
const Segment SGI152944[] = {{&S26442[44], 31}, {NULL}};
const Segment SGI152945[] = {{&S26442[30], 6}, {NULL}};
const Segment SGI152946[] = {{&S26442[124], 3}, {NULL}};
const Segment SGI152947[] = {{&S26442[89], 6}, {NULL}};
const Segment SGI152948[] = {{&S26443[44], 31}, {NULL}};
const Segment SGI152949[] = {{&S26443[44], 42}, {NULL}};
const Segment SGI152950[] = {{&S26443[30], 6}, {NULL}};
const Segment SGI152951[] = {{&S26443[135], 3}, {NULL}};
const Segment SGI152952[] = {{&S26443[100], 6}, {NULL}};
const Segment SGI152953[] = {{&S26444[38], 11}, {NULL}};
const Segment SGI152954[] = {{&S26444[31], 6}, {NULL}};
const Segment SGI152955[] = {{&S26445[44], 25}, {NULL}};
const Segment SGI152956[] = {{&S26445[44], 36}, {NULL}};
const Segment SGI152957[] = {{&S26445[30], 6}, {NULL}};
const Segment SGI152958[] = {{&S26445[129], 3}, {NULL}};
const Segment SGI152959[] = {{&S26445[94], 6}, {NULL}};
const Segment SGI152960[] = {{&S26446[44], 5}, {NULL}};
const Segment SGI152961[] = {{&S26446[44], 17}, {NULL}};
const Segment SGI152962[] = {{&S26446[30], 6}, {NULL}};
const Segment SGI152963[] = {{&S26446[110], 3}, {NULL}};
const Segment SGI152964[] = {{&S26446[75], 6}, {NULL}};
const Segment SGI152965[] = {{&S26447[44], 31}, {NULL}};
const Segment SGI152966[] = {{&S26447[44], 42}, {NULL}};
const Segment SGI152967[] = {{&S26447[30], 6}, {NULL}};
const Segment SGI152968[] = {{&S26447[135], 3}, {NULL}};
const Segment SGI152969[] = {{&S26447[100], 6}, {NULL}};
const Segment SGI152970[] = {{&S26448[44], 31}, {NULL}};
const Segment SGI152971[] = {{&S26448[44], 42}, {NULL}};
const Segment SGI152972[] = {{&S26448[30], 6}, {NULL}};
const Segment SGI152973[] = {{&S26448[135], 3}, {NULL}};
const Segment SGI152974[] = {{&S26448[100], 6}, {NULL}};
const Segment SGI152975[] = {{&S26449[44], 20}, {NULL}};
const Segment SGI152976[] = {{&S26449[44], 31}, {NULL}};
const Segment SGI152977[] = {{&S26449[30], 6}, {NULL}};
const Segment SGI152978[] = {{&S26449[124], 3}, {NULL}};
const Segment SGI152979[] = {{&S26449[89], 6}, {NULL}};
const Segment SGI152980[] = {{&S26450[44], 31}, {NULL}};
const Segment SGI152981[] = {{&S26450[44], 42}, {NULL}};
const Segment SGI152982[] = {{&S26450[30], 6}, {NULL}};
const Segment SGI152983[] = {{&S26450[135], 3}, {NULL}};
const Segment SGI152984[] = {{&S26450[100], 6}, {NULL}};
const Segment SGI152985[] = {{&S26451[44], 20}, {NULL}};
const Segment SGI152986[] = {{&S26451[44], 31}, {NULL}};
const Segment SGI152987[] = {{&S26451[30], 6}, {NULL}};
const Segment SGI152988[] = {{&S26451[124], 3}, {NULL}};
const Segment SGI152989[] = {{&S26451[89], 6}, {NULL}};
const Segment SGI152990[] = {{&S26452[42], 9}, {NULL}};
const Segment SGI152991[] = {{&S26452[30], 6}, {NULL}};
const Segment SGI152992[] = {{&S26452[111], 3}, {NULL}};
const Segment SGI152993[] = {{&S26452[76], 6}, {NULL}};
const Segment SGI152994[] = {{&S26453[44], 5}, {NULL}};
const Segment SGI152995[] = {{&S26453[44], 17}, {NULL}};
const Segment SGI152996[] = {{&S26453[30], 6}, {NULL}};
const Segment SGI152997[] = {{&S26453[110], 3}, {NULL}};
const Segment SGI152998[] = {{&S26453[75], 6}, {NULL}};
const Segment SGI152999[] = {{&S26454[44], 31}, {NULL}};
const Segment SGI153000[] = {{&S26454[44], 42}, {NULL}};
const Segment SGI153001[] = {{&S26454[30], 6}, {NULL}};
const Segment SGI153002[] = {{&S26454[135], 3}, {NULL}};
const Segment SGI153003[] = {{&S26454[100], 6}, {NULL}};
const Segment SGI153004[] = {{&S26455[44], 18}, {NULL}};
const Segment SGI153005[] = {{&S26455[44], 29}, {NULL}};
const Segment SGI153006[] = {{&S26455[30], 6}, {NULL}};
const Segment SGI153007[] = {{&S26455[122], 3}, {NULL}};
const Segment SGI153008[] = {{&S50372[34], 10}, {NULL}};
const Segment SGI153009[] = {{&S26455[87], 6}, {NULL}};
const Segment SGI153010[] = {{&S50372[27], 6}, {NULL}};
const Segment SGI153011[] = {{&S50372[97], 9}, {NULL}};
const Segment SGI153012[] = {{&S50372[70], 6}, {NULL}};
const Segment SGI153013[] = {{&S50373[34], 10}, {NULL}};
const Segment SGI153014[] = {{&S50373[27], 6}, {NULL}};
const Segment SGI153015[] = {{&S50373[97], 9}, {NULL}};
const Segment SGI153016[] = {{&S50373[70], 6}, {NULL}};
const Segment SGI153017[] = {{&S40901[34], 10}, {NULL}};
const Segment SGI153018[] = {{&S40901[27], 6}, {NULL}};
const Segment SGI153019[] = {{&S40901[97], 9}, {NULL}};
const Segment SGI153020[] = {{&S40901[70], 6}, {NULL}};
const Segment SGI153021[] = {{&S40902[34], 10}, {NULL}};
const Segment SGI153022[] = {{&S40902[27], 6}, {NULL}};
const Segment SGI153023[] = {{&S40902[97], 9}, {NULL}};
const Segment SGI153024[] = {{&S40902[70], 6}, {NULL}};
const Segment SGI153025[] = {{&S40903[34], 10}, {NULL}};
const Segment SGI153026[] = {{&S40903[27], 6}, {NULL}};
const Segment SGI153027[] = {{&S40903[97], 9}, {NULL}};
const Segment SGI153028[] = {{&S40903[70], 6}, {NULL}};
const Segment SGI153029[] = {{&S40904[34], 11}, {NULL}};
const Segment SGI153030[] = {{&S40904[27], 6}, {NULL}};
const Segment SGI153031[] = {{&S40904[98], 9}, {NULL}};
const Segment SGI153032[] = {{&S40904[71], 6}, {NULL}};
const Segment SGI153033[] = {{&S41326[44], 6}, {NULL}};
const Segment SGI153034[] = {{&S41326[44], 18}, {NULL}};
const Segment SGI153035[] = {{&S41326[30], 6}, {NULL}};
const Segment SGI153036[] = {{&S41326[111], 3}, {NULL}};
const Segment SGI153037[] = {{&S41326[76], 6}, {NULL}};
const Segment SGI153038[] = {{&S50459[34], 10}, {NULL}};
const Segment SGI153039[] = {{&S50459[27], 6}, {NULL}};
const Segment SGI153040[] = {{&S50459[97], 9}, {NULL}};
const Segment SGI153041[] = {{&S50459[70], 6}, {NULL}};
const Segment SGI153042[] = {{&S50460[34], 8}, {NULL}};
const Segment SGI153043[] = {{&S50460[27], 6}, {NULL}};
const Segment SGI153044[] = {{&S50460[95], 9}, {NULL}};
const Segment SGI153045[] = {{&S50460[68], 6}, {NULL}};
const Segment SGI153046[] = {{&S50461[34], 10}, {NULL}};
const Segment SGI153047[] = {{&S50461[27], 6}, {NULL}};
const Segment SGI153048[] = {{&S50461[97], 9}, {NULL}};
const Segment SGI153049[] = {{&S50461[70], 6}, {NULL}};
const Segment SGI153050[] = {{&S50462[34], 10}, {NULL}};
const Segment SGI153051[] = {{&S50462[27], 6}, {NULL}};
const Segment SGI153052[] = {{&S50462[97], 9}, {NULL}};
const Segment SGI153053[] = {{&S50462[70], 6}, {NULL}};
const Segment SGI153054[] = {{&S50463[34], 11}, {NULL}};
const Segment SGI153055[] = {{&S50463[27], 6}, {NULL}};
const Segment SGI153056[] = {{&S50463[98], 9}, {NULL}};
const Segment SGI153057[] = {{&S50463[71], 6}, {NULL}};
const Segment SGI153058[] = {{&S50464[34], 10}, {NULL}};
const Segment SGI153059[] = {{&S50464[27], 6}, {NULL}};
const Segment SGI153060[] = {{&S50464[97], 9}, {NULL}};
const Segment SGI153061[] = {{&S50464[70], 6}, {NULL}};
const Segment SGI153062[] = {{&S50465[34], 11}, {NULL}};
const Segment SGI153063[] = {{&S50465[27], 6}, {NULL}};
const Segment SGI153064[] = {{&S50465[98], 9}, {NULL}};
const Segment SGI153065[] = {{&S50465[71], 6}, {NULL}};
const Segment SGI153066[] = {{&S50466[34], 7}, {NULL}};
const Segment SGI153067[] = {{&S50466[27], 6}, {NULL}};
const Segment SGI153068[] = {{&S50466[94], 9}, {NULL}};
const Segment SGI153069[] = {{&S50466[67], 6}, {NULL}};
const Segment SGI153070[] = {{&S50468[34], 13}, {NULL}};
const Segment SGI153071[] = {{&S50468[27], 6}, {NULL}};
const Segment SGI153072[] = {{&S50468[100], 9}, {NULL}};
const Segment SGI153073[] = {{&S50468[73], 6}, {NULL}};
const Segment SGI153074[] = {{&S50475[34], 7}, {NULL}};
const Segment SGI153075[] = {{&S50475[27], 6}, {NULL}};
const Segment SGI153076[] = {{&S50475[96], 9}, {NULL}};
const Segment SGI153077[] = {{&S50475[69], 6}, {NULL}};
const Segment SGI153078[] = {{&S40993[44], 10}, {NULL}};
const Segment SGI153079[] = {{&S40993[44], 22}, {NULL}};
const Segment SGI153080[] = {{&S40993[30], 6}, {NULL}};
const Segment SGI153081[] = {{&S40993[131], 3}, {NULL}};
const Segment SGI153082[] = {{&S40993[96], 6}, {NULL}};
const Segment SGI153083[] = {{&S40993[68], 8}, {NULL}};
const Segment SGI153084[] = {{&S40994[44], 11}, {NULL}};
const Segment SGI153085[] = {{&S40994[44], 23}, {NULL}};
const Segment SGI153086[] = {{&S40994[30], 6}, {NULL}};
const Segment SGI153087[] = {{&S40994[136], 3}, {NULL}};
const Segment SGI153088[] = {{&S40994[101], 6}, {NULL}};
const Segment SGI153089[] = {{&S40994[69], 8}, {NULL}};
const Segment SGI153090[] = {{&S40997[44], 10}, {NULL}};
const Segment SGI153091[] = {{&S40997[44], 22}, {NULL}};
const Segment SGI153092[] = {{&S40997[30], 6}, {NULL}};
const Segment SGI153093[] = {{&S40997[131], 3}, {NULL}};
const Segment SGI153094[] = {{&S40997[96], 6}, {NULL}};
const Segment SGI153095[] = {{&S40997[68], 8}, {NULL}};
const Segment SGI153096[] = {{&S41033[34], 12}, {NULL}};
const Segment SGI153097[] = {{&S41033[27], 6}, {NULL}};
const Segment SGI153098[] = {{&S41033[109], 9}, {NULL}};
const Segment SGI153099[] = {{&S41033[82], 6}, {NULL}};
const Segment SGI153100[] = {{&S41034[34], 12}, {NULL}};
const Segment SGI153101[] = {{&S41034[27], 6}, {NULL}};
const Segment SGI153102[] = {{&S41034[109], 9}, {NULL}};
const Segment SGI153103[] = {{&S41034[82], 6}, {NULL}};
const Segment SGI153104[] = {{&S41035[34], 7}, {NULL}};
const Segment SGI153105[] = {{&S41035[27], 6}, {NULL}};
const Segment SGI153106[] = {{&S41035[94], 9}, {NULL}};
const Segment SGI153107[] = {{&S41035[67], 6}, {NULL}};
const Segment SGI153108[] = {{&S41417[44], 10}, {NULL}};
const Segment SGI153109[] = {{&S41417[44], 22}, {NULL}};
const Segment SGI153110[] = {{&S41417[30], 6}, {NULL}};
const Segment SGI153111[] = {{&S41417[131], 3}, {NULL}};
const Segment SGI153112[] = {{&S41417[96], 6}, {NULL}};
const Segment SGI153113[] = {{&S41417[68], 8}, {NULL}};
const Segment SGI153114[] = {{&S41418[44], 10}, {NULL}};
const Segment SGI153115[] = {{&S41418[44], 22}, {NULL}};
const Segment SGI153116[] = {{&S41418[30], 6}, {NULL}};
const Segment SGI153117[] = {{&S41418[148], 3}, {NULL}};
const Segment SGI153118[] = {{&S41418[113], 6}, {NULL}};
const Segment SGI153119[] = {{&S41418[68], 8}, {NULL}};
const Segment SGI153120[] = {{&S41419[44], 10}, {NULL}};
const Segment SGI153121[] = {{&S41419[44], 22}, {NULL}};
const Segment SGI153122[] = {{&S41419[30], 6}, {NULL}};
const Segment SGI153123[] = {{&S41419[131], 3}, {NULL}};
const Segment SGI153124[] = {{&S41419[96], 6}, {NULL}};
const Segment SGI153125[] = {{&S41419[68], 8}, {NULL}};
const Segment SGI153126[] = {{&S41062[44], 10}, {NULL}};
const Segment SGI153127[] = {{&S41062[44], 22}, {NULL}};
const Segment SGI153128[] = {{&S41062[30], 6}, {NULL}};
const Segment SGI153129[] = {{&S41062[115], 3}, {NULL}};
const Segment SGI153130[] = {{&S41062[80], 6}, {NULL}};
const Segment SGI153131[] = {{&S41063[38], 10}, {NULL}};
const Segment SGI153132[] = {{&S41063[31], 6}, {NULL}};
const Segment SGI153133[] = {{&S41064[42], 10}, {NULL}};
const Segment SGI153134[] = {{&S41064[30], 6}, {NULL}};
const Segment SGI153135[] = {{&S41064[113], 3}, {NULL}};
const Segment SGI153136[] = {{&S41064[78], 6}, {NULL}};
const Segment SGI153137[] = {{&S50571[34], 7}, {NULL}};
const Segment SGI153138[] = {{&S50571[27], 6}, {NULL}};
const Segment SGI153139[] = {{&S50571[100], 9}, {NULL}};
const Segment SGI153140[] = {{&S50571[73], 6}, {NULL}};
const Segment SGI153141[] = {{&S50572[34], 7}, {NULL}};
const Segment SGI153142[] = {{&S50572[27], 6}, {NULL}};
const Segment SGI153143[] = {{&S50572[100], 9}, {NULL}};
const Segment SGI153144[] = {{&S50572[73], 6}, {NULL}};
const Segment SGI153145[] = {{&S41076[34], 7}, {NULL}};
const Segment SGI153146[] = {{&S41076[27], 6}, {NULL}};
const Segment SGI153147[] = {{&S41076[94], 9}, {NULL}};
const Segment SGI153148[] = {{&S41076[67], 6}, {NULL}};
const Segment SGI153149[] = {{&S41450[34], 10}, {NULL}};
const Segment SGI153150[] = {{&S50573[34], 10}, {NULL}};
const Segment SGI153151[] = {{&S41450[27], 6}, {NULL}};
const Segment SGI153152[] = {{&S50573[27], 6}, {NULL}};
const Segment SGI153153[] = {{&S36373[34], 9}, {NULL}};
const Segment SGI153154[] = {{&S41450[97], 9}, {NULL}};
const Segment SGI153155[] = {{&S50573[97], 9}, {NULL}};
const Segment SGI153156[] = {{&S41450[70], 6}, {NULL}};
const Segment SGI153157[] = {{&S50573[70], 6}, {NULL}};
const Segment SGI153158[] = {{&S36373[27], 6}, {NULL}};
const Segment SGI153159[] = {{&S41077[34], 11}, {NULL}};
const Segment SGI153160[] = {{&S41077[27], 6}, {NULL}};
const Segment SGI153161[] = {{&S36373[96], 9}, {NULL}};
const Segment SGI153162[] = {{&S36373[69], 6}, {NULL}};
const Segment SGI153163[] = {{&S41077[98], 9}, {NULL}};
const Segment SGI153164[] = {{&S41077[71], 6}, {NULL}};
const Segment SGI153165[] = {{&S41452[34], 10}, {NULL}};
const Segment SGI153166[] = {{&S41078[34], 7}, {NULL}};
const Segment SGI153167[] = {{&S41452[27], 6}, {NULL}};
const Segment SGI153168[] = {{&S41078[27], 6}, {NULL}};
const Segment SGI153169[] = {{&S41452[97], 9}, {NULL}};
const Segment SGI153170[] = {{&S41078[94], 9}, {NULL}};
const Segment SGI153171[] = {{&S41452[70], 6}, {NULL}};
const Segment SGI153172[] = {{&S41078[67], 6}, {NULL}};
const Segment SGI153173[] = {{&S41464[34], 11}, {NULL}};
const Segment SGI153174[] = {{&S41464[27], 6}, {NULL}};
const Segment SGI153175[] = {{&S41464[98], 9}, {NULL}};
const Segment SGI153176[] = {{&S41464[71], 6}, {NULL}};
const Segment SGI153177[] = {{&S41465[34], 27}, {NULL}};
const Segment SGI153178[] = {{&S41465[27], 6}, {NULL}};
const Segment SGI153179[] = {{&S41465[114], 9}, {NULL}};
const Segment SGI153180[] = {{&S41465[87], 6}, {NULL}};
const Segment SGI153181[] = {{&S41466[34], 11}, {NULL}};
const Segment SGI153182[] = {{&S41466[27], 6}, {NULL}};
const Segment SGI153183[] = {{&S41466[98], 9}, {NULL}};
const Segment SGI153184[] = {{&S41466[71], 6}, {NULL}};
const Segment SGI153185[] = {{&S41472[34], 10}, {NULL}};
const Segment SGI153186[] = {{&S41472[27], 6}, {NULL}};
const Segment SGI153187[] = {{&S41472[97], 9}, {NULL}};
const Segment SGI153188[] = {{&S41472[70], 6}, {NULL}};
const Segment SGI153189[] = {{&S41474[34], 10}, {NULL}};
const Segment SGI153190[] = {{&S41474[27], 6}, {NULL}};
const Segment SGI153191[] = {{&S41474[97], 9}, {NULL}};
const Segment SGI153192[] = {{&S41474[70], 6}, {NULL}};
const Segment SGI153193[] = {{&S41481[34], 10}, {NULL}};
const Segment SGI153194[] = {{&S41481[27], 6}, {NULL}};
const Segment SGI153195[] = {{&S41481[97], 9}, {NULL}};
const Segment SGI153196[] = {{&S41481[70], 6}, {NULL}};
const Segment SGI153197[] = {{&S50620[44], 7}, {NULL}};
const Segment SGI153198[] = {{&S50620[44], 19}, {NULL}};
const Segment SGI153199[] = {{&S50620[30], 6}, {NULL}};
const Segment SGI153200[] = {{&S50620[31], 3}, {NULL}};
const Segment SGI153201[] = {{&S50620[77], 6}, {NULL}};
const Segment SGI153202[] = {{&S50621[44], 10}, {NULL}};
const Segment SGI153203[] = {{&S50621[44], 22}, {NULL}};
const Segment SGI153204[] = {{&S50621[30], 6}, {NULL}};
const Segment SGI153205[] = {{&S50621[31], 3}, {NULL}};
const Segment SGI153206[] = {{&S50621[80], 6}, {NULL}};
const Segment SGI153207[] = {{&S50623[44], 17}, {NULL}};
const Segment SGI153208[] = {{&S50623[44], 29}, {NULL}};
const Segment SGI153209[] = {{&S50623[30], 6}, {NULL}};
const Segment SGI153210[] = {{&S50623[31], 3}, {NULL}};
const Segment SGI153211[] = {{&S50623[87], 6}, {NULL}};
const Segment SGI153212[] = {{&S50624[44], 10}, {NULL}};
const Segment SGI153213[] = {{&S50624[44], 22}, {NULL}};
const Segment SGI153214[] = {{&S50624[30], 6}, {NULL}};
const Segment SGI153215[] = {{&S50624[31], 3}, {NULL}};
const Segment SGI153216[] = {{&S50624[80], 6}, {NULL}};
const Segment SGI153217[] = {{&S41508[34], 11}, {NULL}};
const Segment SGI153218[] = {{&S41508[27], 6}, {NULL}};
const Segment SGI153219[] = {{&S41508[98], 9}, {NULL}};
const Segment SGI153220[] = {{&S41508[71], 6}, {NULL}};
const Segment SGI153221[] = {{&S41510[34], 11}, {NULL}};
const Segment SGI153222[] = {{&S41510[27], 6}, {NULL}};
const Segment SGI153223[] = {{&S41510[98], 9}, {NULL}};
const Segment SGI153224[] = {{&S41510[71], 6}, {NULL}};
const Segment SGI153225[] = {{&S41521[34], 10}, {NULL}};
const Segment SGI153226[] = {{&S41521[27], 6}, {NULL}};
const Segment SGI153227[] = {{&S41521[97], 9}, {NULL}};
const Segment SGI153228[] = {{&S41521[70], 6}, {NULL}};
const Segment SGI153229[] = {{&S41522[34], 7}, {NULL}};
const Segment SGI153230[] = {{&S41522[27], 6}, {NULL}};
const Segment SGI153231[] = {{&S41522[110], 9}, {NULL}};
const Segment SGI153232[] = {{&S41522[83], 6}, {NULL}};
const Segment SGI153233[] = {{&S50683[44], 7}, {NULL}};
const Segment SGI153234[] = {{&S50683[44], 29}, {NULL}};
const Segment SGI153235[] = {{&S50683[30], 6}, {NULL}};
const Segment SGI153236[] = {{&S50683[31], 3}, {NULL}};
const Segment SGI153237[] = {{&S50683[87], 6}, {NULL}};
const Segment SGI153238[] = {{&S50684[44], 7}, {NULL}};
const Segment SGI153239[] = {{&S50684[44], 29}, {NULL}};
const Segment SGI153240[] = {{&S50684[30], 6}, {NULL}};
const Segment SGI153241[] = {{&S50684[31], 3}, {NULL}};
const Segment SGI153242[] = {{&S50684[87], 6}, {NULL}};
const Segment SGI153243[] = {{&S50685[44], 7}, {NULL}};
const Segment SGI153244[] = {{&S50685[44], 29}, {NULL}};
const Segment SGI153245[] = {{&S50685[30], 6}, {NULL}};
const Segment SGI153246[] = {{&S50685[31], 3}, {NULL}};
const Segment SGI153247[] = {{&S50685[87], 6}, {NULL}};
const Segment SGI153248[] = {{&S50686[44], 7}, {NULL}};
const Segment SGI153249[] = {{&S50686[44], 22}, {NULL}};
const Segment SGI153250[] = {{&S50686[30], 6}, {NULL}};
const Segment SGI153251[] = {{&S50686[31], 3}, {NULL}};
const Segment SGI153252[] = {{&S50686[80], 6}, {NULL}};
const Segment SGI153253[] = {{&S50687[38], 7}, {NULL}};
const Segment SGI153254[] = {{&S50687[31], 6}, {NULL}};
const Segment SGI153255[] = {{&S50699[44], 11}, {NULL}};
const Segment SGI153256[] = {{&S50699[44], 23}, {NULL}};
const Segment SGI153257[] = {{&S50699[30], 6}, {NULL}};
const Segment SGI153258[] = {{&S50699[31], 3}, {NULL}};
const Segment SGI153259[] = {{&S50699[81], 6}, {NULL}};
const Segment SGI153260[] = {{&S50700[38], 11}, {NULL}};
const Segment SGI153261[] = {{&S50700[31], 6}, {NULL}};
const Segment SGI153262[] = {{&S50701[44], 18}, {NULL}};
const Segment SGI153263[] = {{&S50701[44], 30}, {NULL}};
const Segment SGI153264[] = {{&S50701[30], 6}, {NULL}};
const Segment SGI153265[] = {{&S50701[31], 3}, {NULL}};
const Segment SGI153266[] = {{&S50701[88], 6}, {NULL}};
const Segment SGI153267[] = {{&S45724[32], 9}, {NULL}};
const Segment SGI153268[] = {{&S45724[27], 4}, {NULL}};
const Segment SGI153269[] = {{&S45724[94], 9}, {NULL}};
const Segment SGI153270[] = {{&S45724[67], 6}, {NULL}};
const Segment SGI153271[] = {{&S46744[44], 18}, {NULL}};
const Segment SGI153272[] = {{&S46744[44], 29}, {NULL}};
const Segment SGI153273[] = {{&S46744[30], 6}, {NULL}};
const Segment SGI153274[] = {{&S46744[31], 3}, {NULL}};
const Segment SGI153275[] = {{&S46744[87], 5}, {NULL}};
const Segment SGI153276[] = {{&S50739[34], 21}, {NULL}};
const Segment SGI153277[] = {{&S50739[27], 6}, {NULL}};
const Segment SGI153278[] = {{&S50739[108], 9}, {NULL}};
const Segment SGI153279[] = {{&S50739[81], 6}, {NULL}};
const Segment SGI153280[] = {{&S41239[44], 5}, {NULL}};
const Segment SGI153281[] = {{&S41239[44], 17}, {NULL}};
const Segment SGI153282[] = {{&S41239[30], 6}, {NULL}};
const Segment SGI153283[] = {{&S41239[110], 3}, {NULL}};
const Segment SGI153284[] = {{&S41239[75], 6}, {NULL}};
const Segment SGI153285[] = {{&S41241[44], 5}, {NULL}};
const Segment SGI153286[] = {{&S41241[44], 17}, {NULL}};
const Segment SGI153287[] = {{&S41241[30], 6}, {NULL}};
const Segment SGI153288[] = {{&S41241[110], 3}, {NULL}};
const Segment SGI153289[] = {{&S41241[75], 6}, {NULL}};
const Segment SGI153290[] = {{&S41627[44], 6}, {NULL}};
const Segment SGI153291[] = {{&S41627[44], 18}, {NULL}};
const Segment SGI153292[] = {{&S41627[30], 6}, {NULL}};
const Segment SGI153293[] = {{&S41627[111], 3}, {NULL}};
const Segment SGI153294[] = {{&S41627[76], 6}, {NULL}};
const Segment SGI153295[] = {{&S41243[44], 12}, {NULL}};
const Segment SGI153296[] = {{&S41243[44], 24}, {NULL}};
const Segment SGI153297[] = {{&S41243[30], 6}, {NULL}};
const Segment SGI153298[] = {{&S41243[117], 3}, {NULL}};
const Segment SGI153299[] = {{&S41243[82], 6}, {NULL}};
const Segment SGI153300[] = {{&S41628[38], 6}, {NULL}};
const Segment SGI153301[] = {{&S41628[31], 6}, {NULL}};
const Segment SGI153302[] = {{&S41244[44], 5}, {NULL}};
const Segment SGI153303[] = {{&S41244[44], 16}, {NULL}};
const Segment SGI153304[] = {{&S41244[30], 6}, {NULL}};
const Segment SGI153305[] = {{&S41244[108], 3}, {NULL}};
const Segment SGI153306[] = {{&S41244[74], 5}, {NULL}};
const Segment SGI153307[] = {{&S50791[44], 11}, {NULL}};
const Segment SGI153308[] = {{&S50791[44], 23}, {NULL}};
const Segment SGI153309[] = {{&S50791[30], 6}, {NULL}};
const Segment SGI153310[] = {{&S50791[31], 3}, {NULL}};
const Segment SGI153311[] = {{&S50791[81], 6}, {NULL}};
const Segment SGI153312[] = {{&S50792[44], 10}, {NULL}};
const Segment SGI153313[] = {{&S50792[44], 22}, {NULL}};
const Segment SGI153314[] = {{&S50792[30], 6}, {NULL}};
const Segment SGI153315[] = {{&S50792[31], 3}, {NULL}};
const Segment SGI153316[] = {{&S50792[80], 6}, {NULL}};
const Segment SGI153317[] = {{&S50793[38], 10}, {NULL}};
const Segment SGI153318[] = {{&S50793[31], 6}, {NULL}};
const Segment SGI153319[] = {{&S50794[38], 11}, {NULL}};
const Segment SGI153320[] = {{&S50794[31], 6}, {NULL}};
const Segment SGI153321[] = {{&S41275[44], 10}, {NULL}};
const Segment SGI153322[] = {{&S41275[44], 22}, {NULL}};
const Segment SGI153323[] = {{&S41275[30], 6}, {NULL}};
const Segment SGI153324[] = {{&S50795[44], 10}, {&S50795[176], 18}, {NULL}};
const Segment SGI153325[] = {{&S50795[44], 22}, {NULL}};
const Segment SGI153326[] = {{&S50795[30], 6}, {NULL}};
const Segment SGI153327[] = {{&S41275[115], 3}, {NULL}};
const Segment SGI153328[] = {{&S41275[80], 6}, {NULL}};
const Segment SGI153329[] = {{&S50795[31], 3}, {NULL}};
const Segment SGI153330[] = {{&S50795[80], 6}, {NULL}};
const Segment SGI153331[] = {{&S41276[44], 10}, {NULL}};
const Segment SGI153332[] = {{&S41276[44], 22}, {NULL}};
const Segment SGI153333[] = {{&S41276[30], 6}, {NULL}};
const Segment SGI153334[] = {{&S41276[115], 3}, {NULL}};
const Segment SGI153335[] = {{&S41276[80], 6}, {NULL}};
const Segment SGI153336[] = {{&S41277[44], 9}, {NULL}};
const Segment SGI153337[] = {{&S41277[44], 21}, {NULL}};
const Segment SGI153338[] = {{&S41277[30], 6}, {NULL}};
const Segment SGI153339[] = {{&S41277[114], 3}, {NULL}};
const Segment SGI153340[] = {{&S41277[79], 6}, {NULL}};
const Segment SGI153341[] = {{&S41278[44], 17}, {NULL}};
const Segment SGI153342[] = {{&S41278[44], 29}, {NULL}};
const Segment SGI153343[] = {{&S41278[30], 6}, {NULL}};
const Segment SGI153344[] = {{&S41278[122], 3}, {NULL}};
const Segment SGI153345[] = {{&S41278[87], 6}, {NULL}};
const Segment SGI153346[] = {{&S41279[44], 10}, {NULL}};
const Segment SGI153347[] = {{&S41279[44], 22}, {NULL}};
const Segment SGI153348[] = {{&S41279[30], 6}, {NULL}};
const Segment SGI153349[] = {{&S50802[34], 7}, {NULL}};
const Segment SGI153350[] = {{&S41279[115], 3}, {NULL}};
const Segment SGI153351[] = {{&S50802[27], 6}, {NULL}};
const Segment SGI153352[] = {{&S41279[80], 6}, {NULL}};
const Segment SGI153353[] = {{&S50802[98], 9}, {NULL}};
const Segment SGI153354[] = {{&S50802[71], 6}, {NULL}};
const Segment SGI153355[] = {{&S41280[38], 9}, {NULL}};
const Segment SGI153356[] = {{&S41280[31], 6}, {NULL}};
const Segment SGI153357[] = {{&S50803[34], 7}, {NULL}};
const Segment SGI153358[] = {{&S50803[27], 6}, {NULL}};
const Segment SGI153359[] = {{&S50803[98], 9}, {NULL}};
const Segment SGI153360[] = {{&S50803[71], 6}, {NULL}};
const Segment SGI153361[] = {{&S41281[38], 10}, {NULL}};
const Segment SGI153362[] = {{&S41281[31], 6}, {NULL}};
const Segment SGI153363[] = {{&S41282[38], 10}, {NULL}};
const Segment SGI153364[] = {{&S41282[31], 6}, {NULL}};
const Segment SGI153365[] = {{&S41283[38], 10}, {NULL}};
const Segment SGI153366[] = {{&S41283[31], 6}, {NULL}};
const Segment SGI153367[] = {{&S41284[44], 14}, {NULL}};
const Segment SGI153368[] = {{&S41284[44], 26}, {NULL}};
const Segment SGI153369[] = {{&S41284[30], 6}, {NULL}};
const Segment SGI153370[] = {{&S41284[119], 3}, {NULL}};
const Segment SGI153371[] = {{&S41284[84], 6}, {NULL}};
const Segment SGI153372[] = {{&S41285[44], 16}, {NULL}};
const Segment SGI153373[] = {{&S41285[44], 28}, {NULL}};
const Segment SGI153374[] = {{&S41285[30], 6}, {NULL}};
const Segment SGI153375[] = {{&S41285[121], 3}, {NULL}};
const Segment SGI153376[] = {{&S41285[86], 6}, {NULL}};
const Segment SGI153377[] = {{&S50819[44], 13}, {NULL}};
const Segment SGI153378[] = {{&S50819[44], 25}, {NULL}};
const Segment SGI153379[] = {{&S50819[30], 6}, {NULL}};
const Segment SGI153380[] = {{&S50819[31], 3}, {NULL}};
const Segment SGI153381[] = {{&S50819[83], 6}, {NULL}};
const Segment SGI153382[] = {{&S50820[44], 9}, {NULL}};
const Segment SGI153383[] = {{&S50820[44], 21}, {NULL}};
const Segment SGI153384[] = {{&S50820[30], 6}, {NULL}};
const Segment SGI153385[] = {{&S50820[31], 3}, {NULL}};
const Segment SGI153386[] = {{&S50820[79], 6}, {NULL}};
const Segment SGI153387[] = {{&S50821[44], 13}, {NULL}};
const Segment SGI153388[] = {{&S50821[44], 25}, {NULL}};
const Segment SGI153389[] = {{&S50821[30], 6}, {NULL}};
const Segment SGI153390[] = {{&S50821[31], 3}, {NULL}};
const Segment SGI153391[] = {{&S50821[83], 6}, {NULL}};
const Segment SGI153392[] = {{&S50822[38], 13}, {NULL}};
const Segment SGI153393[] = {{&S50822[31], 6}, {NULL}};
const Segment SGI153394[] = {{&S50823[38], 9}, {NULL}};
const Segment SGI153395[] = {{&S50823[31], 6}, {NULL}};
const Segment SGI153396[] = {{&S41324[44], 6}, {NULL}};
const Segment SGI153397[] = {{&S41324[44], 18}, {NULL}};
const Segment SGI153398[] = {{&S41324[30], 6}, {NULL}};
const Segment SGI153399[] = {{&S41324[111], 3}, {NULL}};
const Segment SGI153400[] = {{&S41324[76], 6}, {NULL}};
const Segment SGI153401[] = {{&S41325[54], 6}, {NULL}};
const Segment SGI153402[] = {{&S41325[47], 6}, {NULL}};
const Segment SGI153403[] = {{&S50871[44], 12}, {NULL}};
const Segment SGI153404[] = {{&S50871[44], 23}, {NULL}};
const Segment SGI153405[] = {{&S50871[30], 6}, {NULL}};
const Segment SGI153406[] = {{&S50871[31], 3}, {NULL}};
const Segment SGI153407[] = {{&S50871[81], 6}, {NULL}};
const Segment SGI153408[] = {{&S50873[44], 12}, {NULL}};
const Segment SGI153409[] = {{&S50873[44], 24}, {NULL}};
const Segment SGI153410[] = {{&S50873[30], 6}, {NULL}};
const Segment SGI153411[] = {{&S50873[31], 3}, {NULL}};
const Segment SGI153412[] = {{&S50873[82], 6}, {NULL}};
const Segment SGI153413[] = {{&S50874[38], 12}, {NULL}};
const Segment SGI153414[] = {{&S50874[31], 6}, {NULL}};
const Segment SGI153415[] = {{&S50876[44], 12}, {NULL}};
const Segment SGI153416[] = {{&S50876[44], 24}, {NULL}};
const Segment SGI153417[] = {{&S50876[30], 6}, {NULL}};
const Segment SGI153418[] = {{&S50876[31], 3}, {NULL}};
const Segment SGI153419[] = {{&S50876[82], 6}, {NULL}};
const Segment SGI153420[] = {{&S50877[44], 12}, {NULL}};
const Segment SGI153421[] = {{&S50877[44], 24}, {NULL}};
const Segment SGI153422[] = {{&S50877[30], 6}, {NULL}};
const Segment SGI153423[] = {{&S50877[31], 3}, {NULL}};
const Segment SGI153424[] = {{&S50877[82], 6}, {NULL}};
const Segment SGI153425[] = {{&S50878[44], 12}, {NULL}};
const Segment SGI153426[] = {{&S50878[44], 24}, {NULL}};
const Segment SGI153427[] = {{&S50878[30], 6}, {NULL}};
const Segment SGI153428[] = {{&S50878[31], 3}, {NULL}};
const Segment SGI153429[] = {{&S50878[82], 6}, {NULL}};
const Segment SGI153430[] = {{&S50879[44], 10}, {NULL}};
const Segment SGI153431[] = {{&S50879[44], 23}, {NULL}};
const Segment SGI153432[] = {{&S50879[30], 6}, {NULL}};
const Segment SGI153433[] = {{&S50879[31], 3}, {NULL}};
const Segment SGI153434[] = {{&S36558[34], 14}, {NULL}};
const Segment SGI153435[] = {{&S36558[27], 6}, {NULL}};
const Segment SGI153436[] = {{&S50879[81], 6}, {NULL}};
const Segment SGI153437[] = {{&S36558[101], 9}, {NULL}};
const Segment SGI153438[] = {{&S36558[74], 6}, {NULL}};
const Segment SGI153439[] = {{&S50880[44], 12}, {NULL}};
const Segment SGI153440[] = {{&S50880[44], 24}, {NULL}};
const Segment SGI153441[] = {{&S50880[30], 6}, {NULL}};
const Segment SGI153442[] = {{&S50880[31], 3}, {NULL}};
const Segment SGI153443[] = {{&S41366[44], 9}, {NULL}};
const Segment SGI153444[] = {{&S41366[44], 21}, {NULL}};
const Segment SGI153445[] = {{&S50880[82], 6}, {NULL}};
const Segment SGI153446[] = {{&S41366[30], 6}, {NULL}};
const Segment SGI153447[] = {{&S41366[114], 3}, {NULL}};
const Segment SGI153448[] = {{&S41366[79], 6}, {NULL}};
const Segment SGI153449[] = {{&S41367[38], 9}, {NULL}};
const Segment SGI153450[] = {{&S41367[31], 6}, {NULL}};
const Segment SGI153451[] = {{&S50882[42], 12}, {NULL}};
const Segment SGI153452[] = {{&S50882[30], 6}, {NULL}};
const Segment SGI153453[] = {{&S50882[31], 3}, {NULL}};
const Segment SGI153454[] = {{&S50882[80], 6}, {NULL}};
const Segment SGI153455[] = {{&S41368[44], 14}, {NULL}};
const Segment SGI153456[] = {{&S41368[44], 26}, {NULL}};
const Segment SGI153457[] = {{&S41368[30], 6}, {NULL}};
const Segment SGI153458[] = {{&S41368[119], 3}, {NULL}};
const Segment SGI153459[] = {{&S41368[84], 6}, {NULL}};
const Segment SGI153460[] = {{&S41369[44], 14}, {NULL}};
const Segment SGI153461[] = {{&S41369[44], 26}, {NULL}};
const Segment SGI153462[] = {{&S41369[30], 6}, {NULL}};
const Segment SGI153463[] = {{&S41369[119], 3}, {NULL}};
const Segment SGI153464[] = {{&S41369[84], 6}, {NULL}};
const Segment SGI153465[] = {{&S41370[44], 14}, {NULL}};
const Segment SGI153466[] = {{&S41370[44], 26}, {NULL}};
const Segment SGI153467[] = {{&S41370[30], 6}, {NULL}};
const Segment SGI153468[] = {{&S41370[119], 3}, {NULL}};
const Segment SGI153469[] = {{&S41370[84], 6}, {NULL}};
const Segment SGI153470[] = {{&S41371[65], 10}, {NULL}};
const Segment SGI153471[] = {{&S41371[65], 22}, {NULL}};
const Segment SGI153472[] = {{&S41371[51], 6}, {NULL}};
const Segment SGI153473[] = {{&S50889[44], 5}, {NULL}};
const Segment SGI153474[] = {{&S41371[136], 3}, {NULL}};
const Segment SGI153475[] = {{&S50889[44], 17}, {NULL}};
const Segment SGI153476[] = {{&S50889[30], 6}, {NULL}};
const Segment SGI153477[] = {{&S41371[101], 6}, {NULL}};
const Segment SGI153478[] = {{&S50889[31], 3}, {NULL}};
const Segment SGI153479[] = {{&S50889[75], 6}, {NULL}};
const Segment SGI153480[] = {{&S41372[44], 16}, {NULL}};
const Segment SGI153481[] = {{&S41372[44], 28}, {NULL}};
const Segment SGI153482[] = {{&S41372[30], 6}, {NULL}};
const Segment SGI153483[] = {{&S50890[38], 5}, {NULL}};
const Segment SGI153484[] = {{&S50890[31], 6}, {NULL}};
const Segment SGI153485[] = {{&S41372[121], 3}, {NULL}};
const Segment SGI153486[] = {{&S41372[86], 6}, {NULL}};
const Segment SGI153487[] = {{&S50903[44], 10}, {NULL}};
const Segment SGI153488[] = {{&S50903[44], 22}, {NULL}};
const Segment SGI153489[] = {{&S50903[30], 6}, {NULL}};
const Segment SGI153490[] = {{&S50903[31], 3}, {NULL}};
const Segment SGI153491[] = {{&S50903[80], 6}, {NULL}};
const Segment SGI153492[] = {{&S50911[44], 10}, {NULL}};
const Segment SGI153493[] = {{&S50911[44], 22}, {NULL}};
const Segment SGI153494[] = {{&S50911[30], 6}, {NULL}};
const Segment SGI153495[] = {{&S50911[31], 3}, {NULL}};
const Segment SGI153496[] = {{&S50911[80], 6}, {NULL}};
const Segment SGI153497[] = {{&S50928[43], 10}, {NULL}};
const Segment SGI153498[] = {{&S50928[43], 22}, {NULL}};
const Segment SGI153499[] = {{&S50928[30], 6}, {NULL}};
const Segment SGI153500[] = {{&S50928[31], 3}, {NULL}};
const Segment SGI153501[] = {{&S50928[79], 6}, {NULL}};
const Segment SGI153502[] = {{&S50929[44], 10}, {NULL}};
const Segment SGI153503[] = {{&S50929[44], 22}, {NULL}};
const Segment SGI153504[] = {{&S50929[30], 6}, {NULL}};
const Segment SGI153505[] = {{&S50929[83], 6}, {NULL}};
const Segment SGI153506[] = {{&S50930[43], 11}, {NULL}};
const Segment SGI153507[] = {{&S50930[43], 23}, {NULL}};
const Segment SGI153508[] = {{&S50930[30], 6}, {NULL}};
const Segment SGI153509[] = {{&S50930[31], 3}, {NULL}};
const Segment SGI153510[] = {{&S50930[80], 6}, {NULL}};
const Segment SGI153511[] = {{&S50932[38], 10}, {NULL}};
const Segment SGI153512[] = {{&S50932[31], 6}, {NULL}};
const Segment SGI153513[] = {{&S50933[44], 10}, {NULL}};
const Segment SGI153514[] = {{&S50933[30], 6}, {NULL}};
const Segment SGI153515[] = {{&S50933[31], 3}, {NULL}};
const Segment SGI153516[] = {{&S50933[82], 6}, {NULL}};
const Segment SGI153517[] = {{&S50934[44], 10}, {NULL}};
const Segment SGI153518[] = {{&S50934[44], 22}, {NULL}};
const Segment SGI153519[] = {{&S50934[30], 6}, {NULL}};
const Segment SGI153520[] = {{&S50934[83], 4}, {NULL}};
const Segment SGI153521[] = {{&S50934[83], 3}, {NULL}};
const Segment SGI153522[] = {{&S50935[44], 13}, {NULL}};
const Segment SGI153523[] = {{&S50935[44], 25}, {NULL}};
const Segment SGI153524[] = {{&S50935[30], 6}, {NULL}};
const Segment SGI153525[] = {{&S50935[31], 3}, {NULL}};
const Segment SGI153526[] = {{&S50935[83], 6}, {NULL}};
const Segment SGI153527[] = {{&S50936[44], 9}, {NULL}};
const Segment SGI153528[] = {{&S50936[44], 21}, {NULL}};
const Segment SGI153529[] = {{&S50936[30], 6}, {NULL}};
const Segment SGI153530[] = {{&S50936[31], 3}, {NULL}};
const Segment SGI153531[] = {{&S50936[79], 6}, {NULL}};
const Segment SGI153532[] = {{&S50950[34], 10}, {NULL}};
const Segment SGI153533[] = {{&S50950[27], 6}, {NULL}};
const Segment SGI153534[] = {{&S50950[97], 9}, {NULL}};
const Segment SGI153535[] = {{&S50950[70], 6}, {NULL}};
const Segment SGI153536[] = {{&S41846[44], 10}, {NULL}};
const Segment SGI153537[] = {{&S41846[44], 22}, {NULL}};
const Segment SGI153538[] = {{&S41846[30], 6}, {NULL}};
const Segment SGI153539[] = {{&S41846[115], 3}, {NULL}};
const Segment SGI153540[] = {{&S41846[80], 6}, {NULL}};
const Segment SGI153541[] = {{&S41847[44], 10}, {NULL}};
const Segment SGI153542[] = {{&S41847[44], 22}, {NULL}};
const Segment SGI153543[] = {{&S41847[30], 6}, {NULL}};
const Segment SGI153544[] = {{&S41847[115], 3}, {NULL}};
const Segment SGI153545[] = {{&S41847[80], 6}, {NULL}};
const Segment SGI153546[] = {{&S50954[44], 23}, {NULL}};
const Segment SGI153547[] = {{&S50954[44], 45}, {NULL}};
const Segment SGI153548[] = {{&S50954[30], 6}, {NULL}};
const Segment SGI153549[] = {{&S50954[31], 3}, {NULL}};
const Segment SGI153550[] = {{&S50954[103], 6}, {NULL}};
const Segment SGI153551[] = {{&S41848[44], 10}, {NULL}};
const Segment SGI153552[] = {{&S41848[44], 22}, {NULL}};
const Segment SGI153553[] = {{&S41848[30], 6}, {NULL}};
const Segment SGI153554[] = {{&S41848[115], 3}, {NULL}};
const Segment SGI153555[] = {{&S41848[80], 6}, {NULL}};
const Segment SGI153556[] = {{&S50955[38], 23}, {NULL}};
const Segment SGI153557[] = {{&S50955[31], 6}, {NULL}};
const Segment SGI153558[] = {{&S50956[44], 24}, {NULL}};
const Segment SGI153559[] = {{&S50956[44], 46}, {NULL}};
const Segment SGI153560[] = {{&S41850[44], 10}, {NULL}};
const Segment SGI153561[] = {{&S50956[30], 6}, {NULL}};
const Segment SGI153562[] = {{&S41850[44], 22}, {NULL}};
const Segment SGI153563[] = {{&S41850[30], 6}, {NULL}};
const Segment SGI153564[] = {{&S50956[31], 3}, {NULL}};
const Segment SGI153565[] = {{&S41850[115], 3}, {NULL}};
const Segment SGI153566[] = {{&S50956[104], 6}, {NULL}};
const Segment SGI153567[] = {{&S41850[80], 6}, {NULL}};
const Segment SGI153568[] = {{&S50957[44], 24}, {NULL}};
const Segment SGI153569[] = {{&S50957[44], 46}, {NULL}};
const Segment SGI153570[] = {{&S50957[30], 6}, {NULL}};
const Segment SGI153571[] = {{&S50957[31], 3}, {NULL}};
const Segment SGI153572[] = {{&S50957[104], 6}, {NULL}};
const Segment SGI153573[] = {{&S41854[38], 10}, {NULL}};
const Segment SGI153574[] = {{&S41854[31], 6}, {NULL}};
const Segment SGI153575[] = {{&S50958[44], 7}, {NULL}};
const Segment SGI153576[] = {{&S50958[44], 24}, {NULL}};
const Segment SGI153577[] = {{&S50958[30], 6}, {NULL}};
const Segment SGI153578[] = {{&S50958[31], 3}, {NULL}};
const Segment SGI153579[] = {{&S50958[82], 6}, {NULL}};
const Segment SGI153580[] = {{&S50959[44], 7}, {NULL}};
const Segment SGI153581[] = {{&S50959[44], 23}, {NULL}};
const Segment SGI153582[] = {{&S50959[30], 6}, {NULL}};
const Segment SGI153583[] = {{&S50959[31], 3}, {NULL}};
const Segment SGI153584[] = {{&S50959[81], 6}, {NULL}};
const Segment SGI153585[] = {{&S50961[44], 7}, {NULL}};
const Segment SGI153586[] = {{&S50961[44], 23}, {NULL}};
const Segment SGI153587[] = {{&S50961[30], 6}, {NULL}};
const Segment SGI153588[] = {{&S50961[31], 3}, {NULL}};
const Segment SGI153589[] = {{&S50961[81], 6}, {NULL}};
const Segment SGI153590[] = {{&S50962[44], 7}, {NULL}};
const Segment SGI153591[] = {{&S50962[44], 24}, {NULL}};
const Segment SGI153592[] = {{&S50962[30], 6}, {NULL}};
const Segment SGI153593[] = {{&S50962[31], 3}, {NULL}};
const Segment SGI153594[] = {{&S50962[82], 6}, {NULL}};
const Segment SGI153595[] = {{&S50963[38], 7}, {NULL}};
const Segment SGI153596[] = {{&S50963[31], 6}, {NULL}};
const Segment SGI153597[] = {{&S41866[44], 18}, {NULL}};
const Segment SGI153598[] = {{&S41866[44], 39}, {NULL}};
const Segment SGI153599[] = {{&S41866[30], 6}, {NULL}};
const Segment SGI153600[] = {{&S41866[132], 3}, {NULL}};
const Segment SGI153601[] = {{&S41866[97], 6}, {NULL}};
const Segment SGI153602[] = {{&S41867[38], 18}, {NULL}};
const Segment SGI153603[] = {{&S41867[31], 6}, {NULL}};
const Segment SGI153604[] = {{&S50982[44], 8}, {NULL}};
const Segment SGI153605[] = {{&S50982[44], 20}, {NULL}};
const Segment SGI153606[] = {{&S50982[30], 6}, {NULL}};
const Segment SGI153607[] = {{&S50982[31], 3}, {NULL}};
const Segment SGI153608[] = {{&S50982[78], 6}, {NULL}};
const Segment SGI153609[] = {{&S50983[38], 8}, {NULL}};
const Segment SGI153610[] = {{&S50983[31], 6}, {NULL}};
const Segment SGI153611[] = {{&S51001[34], 7}, {NULL}};
const Segment SGI153612[] = {{&S51001[27], 6}, {NULL}};
const Segment SGI153613[] = {{&S51001[94], 9}, {NULL}};
const Segment SGI153614[] = {{&S51001[67], 6}, {NULL}};
const Segment SGI153615[] = {{&S51004[34], 10}, {NULL}};
const Segment SGI153616[] = {{&S51004[27], 6}, {NULL}};
const Segment SGI153617[] = {{&S51004[97], 9}, {NULL}};
const Segment SGI153618[] = {{&S51004[70], 6}, {NULL}};
const Segment SGI153619[] = {{&S51006[44], 30}, {NULL}};
const Segment SGI153620[] = {{&S51006[44], 42}, {NULL}};
const Segment SGI153621[] = {{&S51006[30], 6}, {NULL}};
const Segment SGI153622[] = {{&S51006[31], 3}, {NULL}};
const Segment SGI153623[] = {{&S51006[100], 6}, {NULL}};
const Segment SGI153624[] = {{&S51007[44], 10}, {NULL}};
const Segment SGI153625[] = {{&S51007[44], 22}, {NULL}};
const Segment SGI153626[] = {{&S51007[30], 6}, {NULL}};
const Segment SGI153627[] = {{&S51007[31], 3}, {NULL}};
const Segment SGI153628[] = {{&S51007[80], 6}, {NULL}};
const Segment SGI153629[] = {{&S51008[38], 10}, {NULL}};
const Segment SGI153630[] = {{&S51008[31], 6}, {NULL}};
const Segment SGI153631[] = {{&S51009[44], 29}, {NULL}};
const Segment SGI153632[] = {{&S51009[44], 41}, {NULL}};
const Segment SGI153633[] = {{&S51009[30], 6}, {NULL}};
const Segment SGI153634[] = {{&S51009[31], 3}, {NULL}};
const Segment SGI153635[] = {{&S51009[99], 6}, {NULL}};
const Segment SGI153636[] = {{&S51010[44], 30}, {NULL}};
const Segment SGI153637[] = {{&S51010[44], 42}, {NULL}};
const Segment SGI153638[] = {{&S51010[30], 6}, {NULL}};
const Segment SGI153639[] = {{&S51010[31], 3}, {NULL}};
const Segment SGI153640[] = {{&S51010[100], 6}, {NULL}};
const Segment SGI153641[] = {{&S51114[34], 10}, {NULL}};
const Segment SGI153642[] = {{&S51114[27], 6}, {NULL}};
const Segment SGI153643[] = {{&S51114[97], 9}, {NULL}};
const Segment SGI153644[] = {{&S51114[70], 6}, {NULL}};
const Segment SGI153645[] = {{&S41373[38], 10}, {NULL}};
const Segment SGI153646[] = {{&S41373[31], 6}, {NULL}};
const Segment SGI153647[] = {{&S41374[44], 14}, {NULL}};
const Segment SGI153648[] = {{&S41374[44], 26}, {NULL}};
const Segment SGI153649[] = {{&S41374[30], 6}, {NULL}};
const Segment SGI153650[] = {{&S41374[119], 3}, {NULL}};
const Segment SGI153651[] = {{&S41374[84], 6}, {NULL}};
const Segment SGI153652[] = {{&S41375[44], 17}, {NULL}};
const Segment SGI153653[] = {{&S41375[44], 29}, {NULL}};
const Segment SGI153654[] = {{&S41375[30], 6}, {NULL}};
const Segment SGI153655[] = {{&S41375[122], 3}, {NULL}};
const Segment SGI153656[] = {{&S41375[87], 6}, {NULL}};
const Segment SGI153657[] = {{&S42059[34], 15}, {NULL}};
const Segment SGI153658[] = {{&S42059[27], 6}, {NULL}};
const Segment SGI153659[] = {{&S41376[44], 14}, {NULL}};
const Segment SGI153660[] = {{&S41376[44], 26}, {NULL}};
const Segment SGI153661[] = {{&S41376[30], 6}, {NULL}};
const Segment SGI153662[] = {{&S42059[102], 9}, {NULL}};
const Segment SGI153663[] = {{&S42059[75], 6}, {NULL}};
const Segment SGI153664[] = {{&S41376[119], 3}, {NULL}};
const Segment SGI153665[] = {{&S41376[84], 6}, {NULL}};
const Segment SGI153666[] = {{&S42060[11], 1}, {NULL}};
const Segment SGI153667[] = {{&S42060[27], 6}, {NULL}};
const Segment SGI153668[] = {{&S41377[44], 14}, {NULL}};
const Segment SGI153669[] = {{&S41377[44], 26}, {NULL}};
const Segment SGI153670[] = {{&S41377[30], 6}, {NULL}};
const Segment SGI153671[] = {{&S42060[88], 9}, {NULL}};
const Segment SGI153672[] = {{&S42060[61], 6}, {NULL}};
const Segment SGI153673[] = {{&S41377[119], 3}, {NULL}};
const Segment SGI153674[] = {{&S41377[84], 6}, {NULL}};
const Segment SGI153675[] = {{&S42063[11], 1}, {NULL}};
const Segment SGI153676[] = {{&S42063[27], 6}, {NULL}};
const Segment SGI153677[] = {{&S42063[88], 9}, {NULL}};
const Segment SGI153678[] = {{&S42063[61], 6}, {NULL}};
const Segment SGI153679[] = {{&S51160[44], 10}, {NULL}};
const Segment SGI153680[] = {{&S51160[44], 22}, {NULL}};
const Segment SGI153681[] = {{&S51160[30], 6}, {NULL}};
const Segment SGI153682[] = {{&S51160[31], 3}, {NULL}};
const Segment SGI153683[] = {{&S51160[80], 6}, {NULL}};
const Segment SGI153684[] = {{&S42065[34], 6}, {NULL}};
const Segment SGI153685[] = {{&S51161[38], 10}, {NULL}};
const Segment SGI153686[] = {{&S51161[31], 6}, {NULL}};
const Segment SGI153687[] = {{&S42065[27], 6}, {NULL}};
const Segment SGI153688[] = {{&S42065[93], 9}, {NULL}};
const Segment SGI153689[] = {{&S42065[66], 6}, {NULL}};
const Segment SGI153690[] = {{&S51162[42], 10}, {NULL}};
const Segment SGI153691[] = {{&S51162[30], 6}, {NULL}};
const Segment SGI153692[] = {{&S42066[34], 19}, {NULL}};
const Segment SGI153693[] = {{&S42066[27], 6}, {NULL}};
const Segment SGI153694[] = {{&S51162[31], 3}, {NULL}};
const Segment SGI153695[] = {{&S51162[78], 6}, {NULL}};
const Segment SGI153696[] = {{&S42066[106], 9}, {NULL}};
const Segment SGI153697[] = {{&S42066[79], 6}, {NULL}};
const Segment SGI153698[] = {{&S51163[44], 10}, {NULL}};
const Segment SGI153699[] = {{&S51163[44], 46}, {NULL}};
const Segment SGI153700[] = {{&S51163[30], 6}, {NULL}};
const Segment SGI153701[] = {{&S42067[34], 7}, {NULL}};
const Segment SGI153702[] = {{&S42067[27], 6}, {NULL}};
const Segment SGI153703[] = {{&S51163[31], 3}, {NULL}};
const Segment SGI153704[] = {{&S51163[104], 6}, {NULL}};
const Segment SGI153705[] = {{&S42067[94], 9}, {NULL}};
const Segment SGI153706[] = {{&S42067[67], 6}, {NULL}};
const Segment SGI153707[] = {{&S51164[44], 10}, {NULL}};
const Segment SGI153708[] = {{&S51164[44], 43}, {NULL}};
const Segment SGI153709[] = {{&S51164[30], 6}, {NULL}};
const Segment SGI153710[] = {{&S51164[31], 3}, {NULL}};
const Segment SGI153711[] = {{&S51164[101], 6}, {NULL}};
const Segment SGI153712[] = {{&S42070[34], 2}, {NULL}};
const Segment SGI153713[] = {{&S42070[27], 6}, {NULL}};
const Segment SGI153714[] = {{&S51165[44], 10}, {NULL}};
const Segment SGI153715[] = {{&S51165[44], 50}, {NULL}};
const Segment SGI153716[] = {{&S42070[89], 9}, {NULL}};
const Segment SGI153717[] = {{&S51165[30], 6}, {NULL}};
const Segment SGI153718[] = {{&S42070[62], 6}, {NULL}};
const Segment SGI153719[] = {{&S51165[31], 3}, {NULL}};
const Segment SGI153720[] = {{&S51165[108], 6}, {NULL}};
const Segment SGI153721[] = {{&S42071[34], 28}, {NULL}};
const Segment SGI153722[] = {{&S42071[27], 6}, {NULL}};
const Segment SGI153723[] = {{&S42071[115], 9}, {NULL}};
const Segment SGI153724[] = {{&S42071[88], 6}, {NULL}};
const Segment SGI153725[] = {{&S51166[44], 10}, {NULL}};
const Segment SGI153726[] = {{&S51166[44], 43}, {NULL}};
const Segment SGI153727[] = {{&S51166[30], 6}, {NULL}};
const Segment SGI153728[] = {{&S51166[31], 3}, {NULL}};
const Segment SGI153729[] = {{&S51166[101], 6}, {NULL}};
const Segment SGI153730[] = {{&S42073[34], 25}, {NULL}};
const Segment SGI153731[] = {{&S42073[27], 6}, {NULL}};
const Segment SGI153732[] = {{&S51167[44], 10}, {NULL}};
const Segment SGI153733[] = {{&S51167[44], 43}, {NULL}};
const Segment SGI153734[] = {{&S51167[30], 6}, {NULL}};
const Segment SGI153735[] = {{&S42073[112], 9}, {NULL}};
const Segment SGI153736[] = {{&S51167[31], 3}, {NULL}};
const Segment SGI153737[] = {{&S42073[85], 6}, {NULL}};
const Segment SGI153738[] = {{&S51167[101], 6}, {NULL}};
const Segment SGI153739[] = {{&S42074[34], 31}, {NULL}};
const Segment SGI153740[] = {{&S42074[27], 6}, {NULL}};
const Segment SGI153741[] = {{&S51168[44], 10}, {NULL}};
const Segment SGI153742[] = {{&S51168[44], 41}, {NULL}};
const Segment SGI153743[] = {{&S51168[30], 6}, {NULL}};
const Segment SGI153744[] = {{&S42074[118], 9}, {NULL}};
const Segment SGI153745[] = {{&S42074[91], 6}, {NULL}};
const Segment SGI153746[] = {{&S51168[31], 3}, {NULL}};
const Segment SGI153747[] = {{&S51168[99], 6}, {NULL}};
const Segment SGI153748[] = {{&S51169[44], 10}, {NULL}};
const Segment SGI153749[] = {{&S51169[44], 41}, {NULL}};
const Segment SGI153750[] = {{&S51169[30], 6}, {NULL}};
const Segment SGI153751[] = {{&S42076[34], 22}, {NULL}};
const Segment SGI153752[] = {{&S42076[27], 6}, {NULL}};
const Segment SGI153753[] = {{&S51169[31], 3}, {NULL}};
const Segment SGI153754[] = {{&S51169[99], 6}, {NULL}};
const Segment SGI153755[] = {{&S42076[109], 9}, {NULL}};
const Segment SGI153756[] = {{&S42076[82], 6}, {NULL}};
const Segment SGI153757[] = {{&S51170[44], 10}, {NULL}};
const Segment SGI153758[] = {{&S51170[44], 21}, {NULL}};
const Segment SGI153759[] = {{&S51170[30], 6}, {NULL}};
const Segment SGI153760[] = {{&S51170[31], 3}, {NULL}};
const Segment SGI153761[] = {{&S51170[79], 5}, {NULL}};
const Segment SGI153762[] = {{&S42078[34], 24}, {NULL}};
const Segment SGI153763[] = {{&S42078[27], 6}, {NULL}};
const Segment SGI153764[] = {{&S42078[111], 9}, {NULL}};
const Segment SGI153765[] = {{&S42078[84], 6}, {NULL}};
const Segment SGI153766[] = {{&S51171[44], 10}, {NULL}};
const Segment SGI153767[] = {{&S51171[44], 33}, {NULL}};
const Segment SGI153768[] = {{&S51171[30], 6}, {NULL}};
const Segment SGI153769[] = {{&S51171[31], 3}, {NULL}};
const Segment SGI153770[] = {{&S51171[91], 6}, {NULL}};
const Segment SGI153771[] = {{&S42079[34], 29}, {NULL}};
const Segment SGI153772[] = {{&S42079[27], 6}, {NULL}};
const Segment SGI153773[] = {{&S42079[116], 9}, {NULL}};
const Segment SGI153774[] = {{&S42079[89], 6}, {NULL}};
const Segment SGI153775[] = {{&S51172[44], 10}, {NULL}};
const Segment SGI153776[] = {{&S51172[44], 44}, {NULL}};
const Segment SGI153777[] = {{&S51172[30], 6}, {NULL}};
const Segment SGI153778[] = {{&S51172[31], 3}, {NULL}};
const Segment SGI153779[] = {{&S42080[34], 38}, {NULL}};
const Segment SGI153780[] = {{&S51172[102], 6}, {NULL}};
const Segment SGI153781[] = {{&S42080[27], 6}, {NULL}};
const Segment SGI153782[] = {{&S42080[125], 9}, {NULL}};
const Segment SGI153783[] = {{&S42080[98], 6}, {NULL}};
const Segment SGI153784[] = {{&S51173[44], 10}, {NULL}};
const Segment SGI153785[] = {{&S51173[44], 28}, {NULL}};
const Segment SGI153786[] = {{&S51173[30], 6}, {NULL}};
const Segment SGI153787[] = {{&S51173[31], 3}, {NULL}};
const Segment SGI153788[] = {{&S51173[86], 6}, {NULL}};
const Segment SGI153789[] = {{&S42081[34], 33}, {NULL}};
const Segment SGI153790[] = {{&S42081[27], 6}, {NULL}};
const Segment SGI153791[] = {{&S42081[120], 9}, {NULL}};
const Segment SGI153792[] = {{&S42081[93], 6}, {NULL}};
const Segment SGI153793[] = {{&S42082[34], 27}, {NULL}};
const Segment SGI153794[] = {{&S42082[27], 6}, {NULL}};
const Segment SGI153795[] = {{&S42082[114], 9}, {NULL}};
const Segment SGI153796[] = {{&S42082[87], 6}, {NULL}};
const Segment SGI153797[] = {{&S42083[34], 24}, {NULL}};
const Segment SGI153798[] = {{&S42083[27], 6}, {NULL}};
const Segment SGI153799[] = {{&S42083[111], 9}, {NULL}};
const Segment SGI153800[] = {{&S42083[84], 6}, {NULL}};
const Segment SGI153801[] = {{&S51178[44], 10}, {NULL}};
const Segment SGI153802[] = {{&S51178[44], 22}, {NULL}};
const Segment SGI153803[] = {{&S51178[30], 6}, {NULL}};
const Segment SGI153804[] = {{&S51178[31], 3}, {NULL}};
const Segment SGI153805[] = {{&S51178[80], 6}, {NULL}};
const Segment SGI153806[] = {{&S42085[34], 16}, {NULL}};
const Segment SGI153807[] = {{&S51179[44], 10}, {NULL}};
const Segment SGI153808[] = {{&S51179[44], 22}, {NULL}};
const Segment SGI153809[] = {{&S42085[27], 6}, {NULL}};
const Segment SGI153810[] = {{&S51179[30], 6}, {NULL}};
const Segment SGI153811[] = {{&S51179[31], 3}, {NULL}};
const Segment SGI153812[] = {{&S42085[103], 9}, {NULL}};
const Segment SGI153813[] = {{&S42085[76], 6}, {NULL}};
const Segment SGI153814[] = {{&S51179[80], 6}, {NULL}};
const Segment SGI153815[] = {{&S42086[34], 22}, {NULL}};
const Segment SGI153816[] = {{&S51180[38], 10}, {NULL}};
const Segment SGI153817[] = {{&S51180[31], 6}, {NULL}};
const Segment SGI153818[] = {{&S42086[27], 6}, {NULL}};
const Segment SGI153819[] = {{&S42086[109], 9}, {NULL}};
const Segment SGI153820[] = {{&S42086[82], 6}, {NULL}};
const Segment SGI153821[] = {{&S41420[44], 10}, {NULL}};
const Segment SGI153822[] = {{&S41420[44], 22}, {NULL}};
const Segment SGI153823[] = {{&S41420[30], 6}, {NULL}};
const Segment SGI153824[] = {{&S41420[135], 3}, {NULL}};
const Segment SGI153825[] = {{&S41420[100], 6}, {NULL}};
const Segment SGI153826[] = {{&S51181[42], 10}, {NULL}};
const Segment SGI153827[] = {{&S41420[68], 8}, {NULL}};
const Segment SGI153828[] = {{&S51181[30], 6}, {NULL}};
const Segment SGI153829[] = {{&S42087[34], 19}, {NULL}};
const Segment SGI153830[] = {{&S42087[27], 6}, {NULL}};
const Segment SGI153831[] = {{&S51181[31], 3}, {NULL}};
const Segment SGI153832[] = {{&S51181[78], 6}, {NULL}};
const Segment SGI153833[] = {{&S42087[106], 9}, {NULL}};
const Segment SGI153834[] = {{&S42087[79], 6}, {NULL}};
const Segment SGI153835[] = {{&S41421[44], 11}, {NULL}};
const Segment SGI153836[] = {{&S41421[44], 23}, {NULL}};
const Segment SGI153837[] = {{&S41421[30], 6}, {NULL}};
const Segment SGI153838[] = {{&S51182[44], 10}, {NULL}};
const Segment SGI153839[] = {{&S41421[136], 3}, {NULL}};
const Segment SGI153840[] = {{&S51182[44], 28}, {NULL}};
const Segment SGI153841[] = {{&S51182[30], 6}, {NULL}};
const Segment SGI153842[] = {{&S41421[101], 6}, {NULL}};
const Segment SGI153843[] = {{&S42088[34], 31}, {NULL}};
const Segment SGI153844[] = {{&S41421[69], 8}, {NULL}};
const Segment SGI153845[] = {{&S42088[27], 6}, {NULL}};
const Segment SGI153846[] = {{&S51182[31], 3}, {NULL}};
const Segment SGI153847[] = {{&S51182[86], 6}, {NULL}};
const Segment SGI153848[] = {{&S42088[118], 9}, {NULL}};
const Segment SGI153849[] = {{&S42088[91], 6}, {NULL}};
const Segment SGI153850[] = {{&S41422[44], 10}, {NULL}};
const Segment SGI153851[] = {{&S51183[44], 10}, {NULL}};
const Segment SGI153852[] = {{&S41422[44], 22}, {NULL}};
const Segment SGI153853[] = {{&S51183[44], 28}, {NULL}};
const Segment SGI153854[] = {{&S41422[30], 6}, {NULL}};
const Segment SGI153855[] = {{&S51183[30], 6}, {NULL}};
const Segment SGI153856[] = {{&S41422[115], 3}, {NULL}};
const Segment SGI153857[] = {{&S42089[34], 25}, {NULL}};
const Segment SGI153858[] = {{&S51183[31], 3}, {NULL}};
const Segment SGI153859[] = {{&S41422[80], 6}, {NULL}};
const Segment SGI153860[] = {{&S42089[27], 6}, {NULL}};
const Segment SGI153861[] = {{&S51183[86], 6}, {NULL}};
const Segment SGI153862[] = {{&S42089[112], 9}, {NULL}};
const Segment SGI153863[] = {{&S42089[85], 6}, {NULL}};
const Segment SGI153864[] = {{&S41423[44], 10}, {NULL}};
const Segment SGI153865[] = {{&S41423[44], 22}, {NULL}};
const Segment SGI153866[] = {{&S41423[30], 6}, {NULL}};
const Segment SGI153867[] = {{&S51184[44], 10}, {NULL}};
const Segment SGI153868[] = {{&S41423[115], 3}, {NULL}};
const Segment SGI153869[] = {{&S51184[44], 20}, {NULL}};
const Segment SGI153870[] = {{&S51184[30], 6}, {NULL}};
const Segment SGI153871[] = {{&S41423[80], 6}, {NULL}};
const Segment SGI153872[] = {{&S42090[34], 12}, {NULL}};
const Segment SGI153873[] = {{&S42090[27], 6}, {NULL}};
const Segment SGI153874[] = {{&S51184[31], 3}, {NULL}};
const Segment SGI153875[] = {{&S51184[78], 6}, {NULL}};
const Segment SGI153876[] = {{&S42090[99], 9}, {NULL}};
const Segment SGI153877[] = {{&S42090[72], 6}, {NULL}};
const Segment SGI153878[] = {{&S41424[44], 10}, {NULL}};
const Segment SGI153879[] = {{&S41424[44], 22}, {NULL}};
const Segment SGI153880[] = {{&S41424[30], 6}, {NULL}};
const Segment SGI153881[] = {{&S41424[115], 3}, {NULL}};
const Segment SGI153882[] = {{&S51185[44], 10}, {NULL}};
const Segment SGI153883[] = {{&S41424[80], 6}, {NULL}};
const Segment SGI153884[] = {{&S42091[34], 25}, {NULL}};
const Segment SGI153885[] = {{&S51185[44], 28}, {NULL}};
const Segment SGI153886[] = {{&S51185[30], 6}, {NULL}};
const Segment SGI153887[] = {{&S42091[27], 6}, {NULL}};
const Segment SGI153888[] = {{&S51185[31], 3}, {NULL}};
const Segment SGI153889[] = {{&S51185[86], 6}, {NULL}};
const Segment SGI153890[] = {{&S42091[112], 9}, {NULL}};
const Segment SGI153891[] = {{&S42091[85], 6}, {NULL}};
const Segment SGI153892[] = {{&S41425[44], 11}, {NULL}};
const Segment SGI153893[] = {{&S41425[44], 23}, {NULL}};
const Segment SGI153894[] = {{&S41425[30], 6}, {NULL}};
const Segment SGI153895[] = {{&S41425[116], 3}, {NULL}};
const Segment SGI153896[] = {{&S41425[81], 6}, {NULL}};
const Segment SGI153897[] = {{&S51186[44], 10}, {NULL}};
const Segment SGI153898[] = {{&S42092[34], 27}, {NULL}};
const Segment SGI153899[] = {{&S51186[44], 62}, {NULL}};
const Segment SGI153900[] = {{&S42092[27], 6}, {NULL}};
const Segment SGI153901[] = {{&S51186[30], 6}, {NULL}};
const Segment SGI153902[] = {{&S51186[31], 3}, {NULL}};
const Segment SGI153903[] = {{&S42092[114], 9}, {NULL}};
const Segment SGI153904[] = {{&S51186[120], 6}, {NULL}};
const Segment SGI153905[] = {{&S42092[87], 6}, {NULL}};
const Segment SGI153906[] = {{&S41426[38], 10}, {NULL}};
const Segment SGI153907[] = {{&S41426[31], 6}, {NULL}};
const Segment SGI153908[] = {{&S42093[19], 9}, {NULL}};
const Segment SGI153909[] = {{&S42093[27], 6}, {&S42093[42], 13}, {NULL}};
const Segment SGI153910[] = {{&S51187[44], 10}, {NULL}};
const Segment SGI153911[] = {{&S51187[44], 66}, {NULL}};
const Segment SGI153912[] = {{&S51187[30], 6}, {NULL}};
const Segment SGI153913[] = {{&S42093[96], 9}, {NULL}};
const Segment SGI153914[] = {{&S41427[38], 10}, {NULL}};
const Segment SGI153915[] = {{&S42093[69], 6}, {NULL}};
const Segment SGI153916[] = {{&S41427[31], 6}, {NULL}};
const Segment SGI153917[] = {{&S51187[31], 3}, {NULL}};
const Segment SGI153918[] = {{&S51187[124], 6}, {NULL}};
const Segment SGI153919[] = {{&S42094[34], 27}, {NULL}};
const Segment SGI153920[] = {{&S41428[38], 10}, {NULL}};
const Segment SGI153921[] = {{&S42094[27], 6}, {NULL}};
const Segment SGI153922[] = {{&S41428[31], 6}, {NULL}};
const Segment SGI153923[] = {{&S51188[44], 10}, {NULL}};
const Segment SGI153924[] = {{&S51188[44], 38}, {NULL}};
const Segment SGI153925[] = {{&S42094[114], 9}, {NULL}};
const Segment SGI153926[] = {{&S51188[30], 6}, {NULL}};
const Segment SGI153927[] = {{&S42094[87], 6}, {NULL}};
const Segment SGI153928[] = {{&S51188[31], 3}, {NULL}};
const Segment SGI153929[] = {{&S51188[96], 6}, {NULL}};
const Segment SGI153930[] = {{&S41429[44], 19}, {NULL}};
const Segment SGI153931[] = {{&S41429[44], 31}, {NULL}};
const Segment SGI153932[] = {{&S41429[30], 6}, {NULL}};
const Segment SGI153933[] = {{&S41429[124], 3}, {NULL}};
const Segment SGI153934[] = {{&S41429[89], 6}, {NULL}};
const Segment SGI153935[] = {{&S51189[44], 10}, {NULL}};
const Segment SGI153936[] = {{&S51189[44], 29}, {NULL}};
const Segment SGI153937[] = {{&S51189[30], 6}, {NULL}};
const Segment SGI153938[] = {{&S51189[31], 3}, {NULL}};
const Segment SGI153939[] = {{&S51189[87], 6}, {NULL}};
const Segment SGI153940[] = {{&S41430[44], 13}, {NULL}};
const Segment SGI153941[] = {{&S41430[44], 25}, {NULL}};
const Segment SGI153942[] = {{&S41430[30], 6}, {NULL}};
const Segment SGI153943[] = {{&S41430[118], 3}, {NULL}};
const Segment SGI153944[] = {{&S41430[83], 6}, {NULL}};
const Segment SGI153945[] = {{&S51190[44], 10}, {NULL}};
const Segment SGI153946[] = {{&S51190[44], 44}, {NULL}};
const Segment SGI153947[] = {{&S51190[30], 6}, {NULL}};
const Segment SGI153948[] = {{&S51190[31], 3}, {NULL}};
const Segment SGI153949[] = {{&S51190[102], 6}, {NULL}};
const Segment SGI153950[] = {{&S41431[44], 12}, {NULL}};
const Segment SGI153951[] = {{&S41431[44], 24}, {NULL}};
const Segment SGI153952[] = {{&S41431[30], 6}, {NULL}};
const Segment SGI153953[] = {{&S41431[117], 3}, {NULL}};
const Segment SGI153954[] = {{&S41431[82], 6}, {NULL}};
const Segment SGI153955[] = {{&S51191[44], 10}, {NULL}};
const Segment SGI153956[] = {{&S51191[44], 25}, {NULL}};
const Segment SGI153957[] = {{&S51191[30], 6}, {NULL}};
const Segment SGI153958[] = {{&S51191[31], 3}, {NULL}};
const Segment SGI153959[] = {{&S51191[83], 6}, {NULL}};
const Segment SGI153960[] = {{&S41432[44], 19}, {NULL}};
const Segment SGI153961[] = {{&S41432[44], 31}, {NULL}};
const Segment SGI153962[] = {{&S41432[30], 6}, {NULL}};
const Segment SGI153963[] = {{&S41432[124], 3}, {NULL}};
const Segment SGI153964[] = {{&S41432[89], 6}, {NULL}};
const Segment SGI153965[] = {{&S41433[38], 11}, {NULL}};
const Segment SGI153966[] = {{&S41433[31], 6}, {NULL}};
const Segment SGI153967[] = {{&S51194[44], 11}, {NULL}};
const Segment SGI153968[] = {{&S41434[44], 9}, {NULL}};
const Segment SGI153969[] = {{&S51194[44], 28}, {NULL}};
const Segment SGI153970[] = {{&S41434[44], 21}, {NULL}};
const Segment SGI153971[] = {{&S51194[30], 6}, {NULL}};
const Segment SGI153972[] = {{&S41434[30], 6}, {NULL}};
const Segment SGI153973[] = {{&S51194[31], 3}, {NULL}};
const Segment SGI153974[] = {{&S41434[114], 3}, {NULL}};
const Segment SGI153975[] = {{&S51194[86], 6}, {NULL}};
const Segment SGI153976[] = {{&S41434[79], 6}, {NULL}};
const Segment SGI153977[] = {{&S41435[44], 19}, {NULL}};
const Segment SGI153978[] = {{&S51195[44], 11}, {NULL}};
const Segment SGI153979[] = {{&S41435[44], 31}, {NULL}};
const Segment SGI153980[] = {{&S51195[44], 28}, {NULL}};
const Segment SGI153981[] = {{&S41435[30], 6}, {NULL}};
const Segment SGI153982[] = {{&S51195[30], 6}, {NULL}};
const Segment SGI153983[] = {{&S41435[124], 3}, {NULL}};
const Segment SGI153984[] = {{&S51195[31], 3}, {NULL}};
const Segment SGI153985[] = {{&S41435[89], 6}, {NULL}};
const Segment SGI153986[] = {{&S51195[86], 6}, {NULL}};
const Segment SGI153987[] = {{&S51196[44], 11}, {NULL}};
const Segment SGI153988[] = {{&S51196[44], 28}, {NULL}};
const Segment SGI153989[] = {{&S51196[30], 6}, {NULL}};
const Segment SGI153990[] = {{&S51196[31], 3}, {NULL}};
const Segment SGI153991[] = {{&S51196[86], 6}, {NULL}};
const Segment SGI153992[] = {{&S51197[44], 11}, {NULL}};
const Segment SGI153993[] = {{&S51197[44], 29}, {NULL}};
const Segment SGI153994[] = {{&S51197[30], 6}, {NULL}};
const Segment SGI153995[] = {{&S51197[31], 3}, {NULL}};
const Segment SGI153996[] = {{&S51197[87], 6}, {NULL}};
const Segment SGI153997[] = {{&S51198[38], 7}, {NULL}};
const Segment SGI153998[] = {{&S51198[31], 6}, {NULL}};
const Segment SGI153999[] = {{&S51199[44], 7}, {NULL}};
const Segment SGI154000[] = {{&S51199[44], 25}, {NULL}};
const Segment SGI154001[] = {{&S51199[30], 6}, {NULL}};
const Segment SGI154002[] = {{&S51199[31], 3}, {NULL}};
const Segment SGI154003[] = {{&S51199[83], 6}, {NULL}};
const Segment SGI154004[] = {{&S51200[44], 11}, {NULL}};
const Segment SGI154005[] = {{&S51200[44], 28}, {NULL}};
const Segment SGI154006[] = {{&S51200[30], 6}, {NULL}};
const Segment SGI154007[] = {{&S51200[31], 3}, {NULL}};
const Segment SGI154008[] = {{&S51200[86], 6}, {NULL}};
const Segment SGI154009[] = {{&S51201[44], 7}, {NULL}};
const Segment SGI154010[] = {{&S51201[44], 24}, {NULL}};
const Segment SGI154011[] = {{&S51201[30], 6}, {NULL}};
const Segment SGI154012[] = {{&S41449[34], 10}, {NULL}};
const Segment SGI154013[] = {{&S51201[31], 3}, {NULL}};
const Segment SGI154014[] = {{&S41449[27], 6}, {NULL}};
const Segment SGI154015[] = {{&S51201[82], 6}, {NULL}};
const Segment SGI154016[] = {{&S41449[97], 9}, {NULL}};
const Segment SGI154017[] = {{&S41449[70], 6}, {NULL}};
const Segment SGI154018[] = {{&S41451[34], 10}, {NULL}};
const Segment SGI154019[] = {{&S41451[27], 6}, {NULL}};
const Segment SGI154020[] = {{&S41451[97], 9}, {NULL}};
const Segment SGI154021[] = {{&S41451[70], 6}, {NULL}};
const Segment SGI154022[] = {{&S36986[34], 14}, {NULL}};
const Segment SGI154023[] = {{&S36986[27], 6}, {NULL}};
const Segment SGI154024[] = {{&S36986[101], 9}, {NULL}};
const Segment SGI154025[] = {{&S36986[74], 6}, {NULL}};
const Segment SGI154026[] = {{&S41453[34], 10}, {NULL}};
const Segment SGI154027[] = {{&S41453[27], 6}, {NULL}};
const Segment SGI154028[] = {{&S41453[97], 9}, {NULL}};
const Segment SGI154029[] = {{&S41453[70], 6}, {NULL}};
const Segment SGI154030[] = {{&S36988[34], 14}, {NULL}};
const Segment SGI154031[] = {{&S36988[27], 6}, {NULL}};
const Segment SGI154032[] = {{&S36988[101], 9}, {NULL}};
const Segment SGI154033[] = {{&S36988[74], 6}, {NULL}};
const Segment SGI154034[] = {{&S51214[44], 13}, {NULL}};
const Segment SGI154035[] = {{&S51214[44], 25}, {NULL}};
const Segment SGI154036[] = {{&S51214[30], 6}, {NULL}};
const Segment SGI154037[] = {{&S51214[31], 3}, {NULL}};
const Segment SGI154038[] = {{&S51214[83], 6}, {NULL}};
const Segment SGI154039[] = {{&S51215[44], 10}, {NULL}};
const Segment SGI154040[] = {{&S41463[34], 11}, {NULL}};
const Segment SGI154041[] = {{&S51215[44], 22}, {NULL}};
const Segment SGI154042[] = {{&S41463[27], 6}, {NULL}};
const Segment SGI154043[] = {{&S51215[30], 6}, {NULL}};
const Segment SGI154044[] = {{&S51215[31], 3}, {NULL}};
const Segment SGI154045[] = {{&S41463[98], 9}, {NULL}};
const Segment SGI154046[] = {{&S51215[80], 6}, {NULL}};
const Segment SGI154047[] = {{&S41463[71], 6}, {NULL}};
const Segment SGI154048[] = {{&S51216[38], 10}, {NULL}};
const Segment SGI154049[] = {{&S51216[31], 6}, {NULL}};
const Segment SGI154050[] = {{&S51218[44], 6}, {NULL}};
const Segment SGI154051[] = {{&S51218[44], 18}, {NULL}};
const Segment SGI154052[] = {{&S51218[30], 6}, {NULL}};
const Segment SGI154053[] = {{&S51218[31], 3}, {NULL}};
const Segment SGI154054[] = {{&S51218[76], 6}, {NULL}};
const Segment SGI154055[] = {{&S51219[38], 6}, {NULL}};
const Segment SGI154056[] = {{&S51219[31], 6}, {NULL}};
const Segment SGI154057[] = {{&S51220[44], 14}, {NULL}};
const Segment SGI154058[] = {{&S51220[44], 26}, {NULL}};
const Segment SGI154059[] = {{&S51220[30], 6}, {NULL}};
const Segment SGI154060[] = {{&S51220[31], 3}, {NULL}};
const Segment SGI154061[] = {{&S51220[84], 6}, {NULL}};
const Segment SGI154062[] = {{&S41473[34], 10}, {NULL}};
const Segment SGI154063[] = {{&S41473[27], 6}, {NULL}};
const Segment SGI154064[] = {{&S41473[97], 9}, {NULL}};
const Segment SGI154065[] = {{&S41473[70], 6}, {NULL}};
const Segment SGI154066[] = {{&S51223[34], 21}, {NULL}};
const Segment SGI154067[] = {{&S51223[27], 6}, {NULL}};
const Segment SGI154068[] = {{&S51223[108], 9}, {NULL}};
const Segment SGI154069[] = {{&S51223[81], 6}, {NULL}};
const Segment SGI154070[] = {{&S41475[34], 10}, {NULL}};
const Segment SGI154071[] = {{&S41475[27], 6}, {NULL}};
const Segment SGI154072[] = {{&S41475[97], 9}, {NULL}};
const Segment SGI154073[] = {{&S41475[70], 6}, {NULL}};
const Segment SGI154074[] = {{&S51225[34], 18}, {NULL}};
const Segment SGI154075[] = {{&S51225[27], 6}, {NULL}};
const Segment SGI154076[] = {{&S41476[34], 10}, {NULL}};
const Segment SGI154077[] = {{&S41476[27], 6}, {NULL}};
const Segment SGI154078[] = {{&S51225[105], 9}, {NULL}};
const Segment SGI154079[] = {{&S51225[78], 6}, {NULL}};
const Segment SGI154080[] = {{&S41476[97], 9}, {NULL}};
const Segment SGI154081[] = {{&S41476[70], 6}, {NULL}};
const Segment SGI154082[] = {{&S41477[34], 10}, {NULL}};
const Segment SGI154083[] = {{&S41477[27], 6}, {NULL}};
const Segment SGI154084[] = {{&S41477[97], 9}, {NULL}};
const Segment SGI154085[] = {{&S41477[70], 6}, {NULL}};
const Segment SGI154086[] = {{&S41478[34], 11}, {NULL}};
const Segment SGI154087[] = {{&S41478[27], 6}, {NULL}};
const Segment SGI154088[] = {{&S41478[98], 9}, {NULL}};
const Segment SGI154089[] = {{&S41478[71], 6}, {NULL}};
const Segment SGI154090[] = {{&S41479[34], 11}, {NULL}};
const Segment SGI154091[] = {{&S41479[27], 6}, {NULL}};
const Segment SGI154092[] = {{&S41479[98], 9}, {NULL}};
const Segment SGI154093[] = {{&S41479[71], 6}, {NULL}};
const Segment SGI154094[] = {{&S41480[34], 11}, {NULL}};
const Segment SGI154095[] = {{&S41480[27], 6}, {NULL}};
const Segment SGI154096[] = {{&S41480[98], 9}, {NULL}};
const Segment SGI154097[] = {{&S41480[71], 6}, {NULL}};
const Segment SGI154098[] = {{&S41482[34], 11}, {NULL}};
const Segment SGI154099[] = {{&S41482[27], 6}, {NULL}};
const Segment SGI154100[] = {{&S41482[98], 9}, {NULL}};
const Segment SGI154101[] = {{&S41482[71], 6}, {NULL}};
const Segment SGI154102[] = {{&S41483[34], 11}, {NULL}};
const Segment SGI154103[] = {{&S41483[27], 6}, {NULL}};
const Segment SGI154104[] = {{&S51238[34], 7}, {NULL}};
const Segment SGI154105[] = {{&S41483[98], 9}, {NULL}};
const Segment SGI154106[] = {{&S41483[71], 6}, {NULL}};
const Segment SGI154107[] = {{&S51238[27], 6}, {NULL}};
const Segment SGI154108[] = {{&S51238[108], 9}, {NULL}};
const Segment SGI154109[] = {{&S51238[81], 6}, {NULL}};
const Segment SGI154110[] = {{&S41484[34], 11}, {NULL}};
const Segment SGI154111[] = {{&S41484[27], 6}, {NULL}};
const Segment SGI154112[] = {{&S41484[98], 9}, {NULL}};
const Segment SGI154113[] = {{&S51239[34], 15}, {NULL}};
const Segment SGI154114[] = {{&S41484[71], 6}, {NULL}};
const Segment SGI154115[] = {{&S51239[27], 6}, {NULL}};
const Segment SGI154116[] = {{&S51239[118], 9}, {NULL}};
const Segment SGI154117[] = {{&S51239[91], 6}, {NULL}};
const Segment SGI154118[] = {{&S51240[34], 7}, {NULL}};
const Segment SGI154119[] = {{&S51240[27], 6}, {NULL}};
const Segment SGI154120[] = {{&S51240[108], 9}, {NULL}};
const Segment SGI154121[] = {{&S51240[81], 6}, {NULL}};
const Segment SGI154122[] = {{&S42173[38], 11}, {NULL}};
const Segment SGI154123[] = {{&S51241[34], 15}, {NULL}};
const Segment SGI154124[] = {{&S42173[31], 6}, {NULL}};
const Segment SGI154125[] = {{&S51241[27], 6}, {NULL}};
const Segment SGI154126[] = {{&S51241[118], 9}, {NULL}};
const Segment SGI154127[] = {{&S51241[91], 6}, {NULL}};
const Segment SGI154128[] = {{&S42174[44], 11}, {NULL}};
const Segment SGI154129[] = {{&S42174[44], 23}, {NULL}};
const Segment SGI154130[] = {{&S42174[30], 6}, {NULL}};
const Segment SGI154131[] = {{&S42174[116], 3}, {NULL}};
const Segment SGI154132[] = {{&S42174[81], 6}, {NULL}};
const Segment SGI154133[] = {{&S42175[44], 11}, {NULL}};
const Segment SGI154134[] = {{&S42175[44], 23}, {NULL}};
const Segment SGI154135[] = {{&S42175[30], 6}, {NULL}};
const Segment SGI154136[] = {{&S42175[116], 3}, {NULL}};
const Segment SGI154137[] = {{&S42175[81], 6}, {NULL}};
const Segment SGI154138[] = {{&S41506[34], 10}, {NULL}};
const Segment SGI154139[] = {{&S41506[27], 6}, {NULL}};
const Segment SGI154140[] = {{&S41506[97], 9}, {NULL}};
const Segment SGI154141[] = {{&S41506[70], 6}, {NULL}};
const Segment SGI154142[] = {{&S41507[34], 7}, {NULL}};
const Segment SGI154143[] = {{&S41507[27], 6}, {NULL}};
const Segment SGI154144[] = {{&S41507[99], 9}, {NULL}};
const Segment SGI154145[] = {{&S41507[72], 6}, {NULL}};
const Segment SGI154146[] = {{&S41509[34], 7}, {NULL}};
const Segment SGI154147[] = {{&S41509[27], 6}, {NULL}};
const Segment SGI154148[] = {{&S41509[94], 9}, {NULL}};
const Segment SGI154149[] = {{&S41509[67], 6}, {NULL}};
const Segment SGI154150[] = {{&S41511[34], 11}, {NULL}};
const Segment SGI154151[] = {{&S41511[27], 6}, {NULL}};
const Segment SGI154152[] = {{&S41511[98], 9}, {NULL}};
const Segment SGI154153[] = {{&S41511[71], 6}, {NULL}};
const Segment SGI154154[] = {{&S51269[44], 6}, {NULL}};
const Segment SGI154155[] = {{&S51269[44], 18}, {NULL}};
const Segment SGI154156[] = {{&S51269[30], 6}, {NULL}};
const Segment SGI154157[] = {{&S51269[31], 3}, {NULL}};
const Segment SGI154158[] = {{&S51269[76], 6}, {NULL}};
const Segment SGI154159[] = {{&S51270[38], 6}, {NULL}};
const Segment SGI154160[] = {{&S51270[31], 6}, {NULL}};
const Segment SGI154161[] = {{&S41520[34], 10}, {NULL}};
const Segment SGI154162[] = {{&S41520[27], 6}, {NULL}};
const Segment SGI154163[] = {{&S41520[97], 9}, {NULL}};
const Segment SGI154164[] = {{&S41520[70], 6}, {NULL}};
const Segment SGI154165[] = {{&S51277[44], 10}, {NULL}};
const Segment SGI154166[] = {{&S51277[44], 22}, {NULL}};
const Segment SGI154167[] = {{&S51277[30], 6}, {NULL}};
const Segment SGI154168[] = {{&S51277[31], 3}, {NULL}};
const Segment SGI154169[] = {{&S51277[80], 6}, {NULL}};
const Segment SGI154170[] = {{&S51278[38], 10}, {NULL}};
const Segment SGI154171[] = {{&S51278[31], 6}, {NULL}};
const Segment SGI154172[] = {{&S41523[34], 7}, {NULL}};
const Segment SGI154173[] = {{&S41523[27], 6}, {NULL}};
const Segment SGI154174[] = {{&S41523[109], 9}, {NULL}};
const Segment SGI154175[] = {{&S41523[82], 6}, {NULL}};
const Segment SGI154176[] = {{&S51279[44], 10}, {NULL}};
const Segment SGI154177[] = {{&S51279[44], 22}, {NULL}};
const Segment SGI154178[] = {{&S51279[30], 6}, {NULL}};
const Segment SGI154179[] = {{&S51279[31], 3}, {NULL}};
const Segment SGI154180[] = {{&S51279[80], 6}, {NULL}};
const Segment SGI154181[] = {{&S51280[42], 10}, {NULL}};
const Segment SGI154182[] = {{&S51280[30], 6}, {NULL}};
const Segment SGI154183[] = {{&S51280[31], 3}, {NULL}};
const Segment SGI154184[] = {{&S51280[78], 6}, {NULL}};
const Segment SGI154185[] = {{&S51281[44], 29}, {NULL}};
const Segment SGI154186[] = {{&S51281[44], 41}, {NULL}};
const Segment SGI154187[] = {{&S51281[30], 6}, {NULL}};
const Segment SGI154188[] = {{&S51281[31], 3}, {NULL}};
const Segment SGI154189[] = {{&S51281[99], 6}, {NULL}};
const Segment SGI154190[] = {{&S41534[44], 30}, {NULL}};
const Segment SGI154191[] = {{&S51286[44], 11}, {NULL}};
const Segment SGI154192[] = {{&S41534[44], 42}, {NULL}};
const Segment SGI154193[] = {{&S51286[44], 35}, {NULL}};
const Segment SGI154194[] = {{&S41534[30], 6}, {NULL}};
const Segment SGI154195[] = {{&S51286[30], 6}, {NULL}};
const Segment SGI154196[] = {{&S41534[135], 3}, {NULL}};
const Segment SGI154197[] = {{&S51286[31], 3}, {NULL}};
const Segment SGI154198[] = {{&S41534[100], 6}, {NULL}};
const Segment SGI154199[] = {{&S51286[93], 6}, {NULL}};
const Segment SGI154200[] = {{&S41535[44], 29}, {NULL}};
const Segment SGI154201[] = {{&S41535[44], 41}, {NULL}};
const Segment SGI154202[] = {{&S51287[44], 11}, {NULL}};
const Segment SGI154203[] = {{&S41535[30], 6}, {NULL}};
const Segment SGI154204[] = {{&S51287[44], 34}, {NULL}};
const Segment SGI154205[] = {{&S51287[30], 6}, {NULL}};
const Segment SGI154206[] = {{&S41535[134], 3}, {NULL}};
const Segment SGI154207[] = {{&S51287[31], 3}, {NULL}};
const Segment SGI154208[] = {{&S41535[99], 6}, {NULL}};
const Segment SGI154209[] = {{&S51287[92], 6}, {NULL}};
const Segment SGI154210[] = {{&S41536[44], 10}, {NULL}};
const Segment SGI154211[] = {{&S51288[38], 11}, {NULL}};
const Segment SGI154212[] = {{&S41536[44], 22}, {NULL}};
const Segment SGI154213[] = {{&S41536[30], 6}, {NULL}};
const Segment SGI154214[] = {{&S51288[31], 6}, {NULL}};
const Segment SGI154215[] = {{&S41536[115], 3}, {NULL}};
const Segment SGI154216[] = {{&S41536[80], 6}, {NULL}};
const Segment SGI154217[] = {{&S41537[38], 10}, {NULL}};
const Segment SGI154218[] = {{&S39681[32], 9}, {NULL}};
const Segment SGI154219[] = {{&S41537[31], 6}, {NULL}};
const Segment SGI154220[] = {{&S39681[27], 4}, {NULL}};
const Segment SGI154221[] = {{&S39681[94], 9}, {NULL}};
const Segment SGI154222[] = {{&S39681[67], 6}, {NULL}};
const Segment SGI154223[] = {{&S41538[44], 11}, {NULL}};
const Segment SGI154224[] = {{&S41538[44], 23}, {NULL}};
const Segment SGI154225[] = {{&S41538[30], 6}, {NULL}};
const Segment SGI154226[] = {{&S41538[116], 3}, {NULL}};
const Segment SGI154227[] = {{&S41538[81], 6}, {NULL}};
const Segment SGI154228[] = {{&S41539[38], 11}, {NULL}};
const Segment SGI154229[] = {{&S41539[31], 6}, {NULL}};
const Segment SGI154230[] = {{&S41540[44], 29}, {NULL}};
const Segment SGI154231[] = {{&S41540[44], 41}, {NULL}};
const Segment SGI154232[] = {{&S41540[30], 6}, {NULL}};
const Segment SGI154233[] = {{&S41540[134], 3}, {NULL}};
const Segment SGI154234[] = {{&S41540[99], 6}, {NULL}};
const Segment SGI154235[] = {{&S51297[34], 15}, {NULL}};
const Segment SGI154236[] = {{&S51297[27], 6}, {NULL}};
const Segment SGI154237[] = {{&S41541[44], 30}, {NULL}};
const Segment SGI154238[] = {{&S41541[44], 42}, {NULL}};
const Segment SGI154239[] = {{&S41541[30], 6}, {NULL}};
const Segment SGI154240[] = {{&S51297[102], 9}, {NULL}};
const Segment SGI154241[] = {{&S51297[75], 6}, {NULL}};
const Segment SGI154242[] = {{&S41541[135], 3}, {NULL}};
const Segment SGI154243[] = {{&S41541[100], 6}, {NULL}};
const Segment SGI154244[] = {{&S51298[34], 11}, {NULL}};
const Segment SGI154245[] = {{&S51298[27], 6}, {NULL}};
const Segment SGI154246[] = {{&S41542[44], 30}, {NULL}};
const Segment SGI154247[] = {{&S41542[44], 42}, {NULL}};
const Segment SGI154248[] = {{&S41542[30], 6}, {NULL}};
const Segment SGI154249[] = {{&S51298[98], 9}, {NULL}};
const Segment SGI154250[] = {{&S51298[71], 6}, {NULL}};
const Segment SGI154251[] = {{&S41542[135], 3}, {NULL}};
const Segment SGI154252[] = {{&S41542[100], 6}, {NULL}};
const Segment SGI154253[] = {{&S41543[44], 30}, {NULL}};
const Segment SGI154254[] = {{&S41543[44], 42}, {NULL}};
const Segment SGI154255[] = {{&S41543[30], 6}, {NULL}};
const Segment SGI154256[] = {{&S41543[135], 3}, {NULL}};
const Segment SGI154257[] = {{&S41543[100], 6}, {NULL}};
const Segment SGI154258[] = {{&S41544[44], 30}, {NULL}};
const Segment SGI154259[] = {{&S41544[44], 42}, {NULL}};
const Segment SGI154260[] = {{&S41544[30], 6}, {NULL}};
const Segment SGI154261[] = {{&S41544[135], 3}, {NULL}};
const Segment SGI154262[] = {{&S41544[100], 6}, {NULL}};
const Segment SGI154263[] = {{&S51349[34], 10}, {NULL}};
const Segment SGI154264[] = {{&S51349[27], 6}, {NULL}};
const Segment SGI154265[] = {{&S51349[97], 9}, {NULL}};
const Segment SGI154266[] = {{&S51349[70], 6}, {NULL}};
const Segment SGI154267[] = {{&S51351[44], 11}, {NULL}};
const Segment SGI154268[] = {{&S51351[44], 23}, {NULL}};
const Segment SGI154269[] = {{&S51351[30], 6}, {NULL}};
const Segment SGI154270[] = {{&S51351[31], 3}, {NULL}};
const Segment SGI154271[] = {{&S51351[81], 6}, {NULL}};
const Segment SGI154272[] = {{&S51352[38], 11}, {NULL}};
const Segment SGI154273[] = {{&S51352[31], 6}, {NULL}};
const Segment SGI154274[] = {{&S37228[44], 18}, {NULL}};
const Segment SGI154275[] = {{&S37228[44], 30}, {NULL}};
const Segment SGI154276[] = {{&S37228[30], 6}, {NULL}};
const Segment SGI154277[] = {{&S37228[141], 3}, {NULL}};
const Segment SGI154278[] = {{&S37228[106], 6}, {NULL}};
const Segment SGI154279[] = {{&S37228[76], 8}, {NULL}};
const Segment SGI154280[] = {{&S37229[44], 10}, {NULL}};
const Segment SGI154281[] = {{&S37229[44], 22}, {NULL}};
const Segment SGI154282[] = {{&S37229[30], 6}, {NULL}};
const Segment SGI154283[] = {{&S37229[133], 3}, {NULL}};
const Segment SGI154284[] = {{&S37229[98], 6}, {NULL}};
const Segment SGI154285[] = {{&S37229[68], 8}, {NULL}};
const Segment SGI154286[] = {{&S37230[44], 11}, {NULL}};
const Segment SGI154287[] = {{&S37230[44], 23}, {NULL}};
const Segment SGI154288[] = {{&S37230[30], 6}, {NULL}};
const Segment SGI154289[] = {{&S37230[134], 3}, {NULL}};
const Segment SGI154290[] = {{&S37230[99], 6}, {NULL}};
const Segment SGI154291[] = {{&S37230[69], 8}, {NULL}};
const Segment SGI154292[] = {{&S37231[44], 18}, {NULL}};
const Segment SGI154293[] = {{&S37231[44], 30}, {NULL}};
const Segment SGI154294[] = {{&S37231[30], 6}, {NULL}};
const Segment SGI154295[] = {{&S37231[141], 3}, {NULL}};
const Segment SGI154296[] = {{&S37231[106], 6}, {NULL}};
const Segment SGI154297[] = {{&S37231[76], 8}, {NULL}};
const Segment SGI154298[] = {{&S51438[34], 13}, {NULL}};
const Segment SGI154299[] = {{&S51438[27], 6}, {NULL}};
const Segment SGI154300[] = {{&S51438[104], 9}, {NULL}};
const Segment SGI154301[] = {{&S51438[77], 6}, {NULL}};
const Segment SGI154302[] = {{&S41672[34], 7}, {NULL}};
const Segment SGI154303[] = {{&S41672[27], 6}, {NULL}};
const Segment SGI154304[] = {{&S41672[102], 9}, {NULL}};
const Segment SGI154305[] = {{&S41672[75], 6}, {NULL}};
const Segment SGI154306[] = {{&S51480[34], 7}, {NULL}};
const Segment SGI154307[] = {{&S51480[27], 6}, {NULL}};
const Segment SGI154308[] = {{&S51480[99], 9}, {NULL}};
const Segment SGI154309[] = {{&S51480[72], 6}, {NULL}};
const Segment SGI154310[] = {{&S51482[34], 7}, {NULL}};
const Segment SGI154311[] = {{&S51482[27], 6}, {NULL}};
const Segment SGI154312[] = {{&S51482[99], 9}, {NULL}};
const Segment SGI154313[] = {{&S51482[72], 6}, {NULL}};
const Segment SGI154314[] = {{&S42441[34], 10}, {NULL}};
const Segment SGI154315[] = {{&S42441[27], 6}, {NULL}};
const Segment SGI154316[] = {{&S51483[34], 8}, {NULL}};
const Segment SGI154317[] = {{&S51483[27], 6}, {NULL}};
const Segment SGI154318[] = {{&S42441[97], 9}, {NULL}};
const Segment SGI154319[] = {{&S42441[70], 6}, {NULL}};
const Segment SGI154320[] = {{&S51483[100], 9}, {NULL}};
const Segment SGI154321[] = {{&S51483[73], 6}, {NULL}};
const Segment SGI154322[] = {{&S42442[34], 10}, {NULL}};
const Segment SGI154323[] = {{&S42442[27], 6}, {NULL}};
const Segment SGI154324[] = {{&S42442[97], 9}, {NULL}};
const Segment SGI154325[] = {{&S42442[70], 6}, {NULL}};
const Segment SGI154326[] = {{&S27562[44], 24}, {NULL}};
const Segment SGI154327[] = {{&S27562[44], 35}, {NULL}};
const Segment SGI154328[] = {{&S27562[30], 6}, {NULL}};
const Segment SGI154329[] = {{&S27562[128], 3}, {NULL}};
const Segment SGI154330[] = {{&S27562[93], 6}, {NULL}};
const Segment SGI154331[] = {{&S51515[44], 6}, {NULL}};
const Segment SGI154332[] = {{&S51515[44], 18}, {NULL}};
const Segment SGI154333[] = {{&S51515[30], 6}, {NULL}};
const Segment SGI154334[] = {{&S51515[31], 3}, {NULL}};
const Segment SGI154335[] = {{&S51515[76], 6}, {NULL}};
const Segment SGI154336[] = {{&S51517[38], 6}, {NULL}};
const Segment SGI154337[] = {{&S51517[31], 6}, {NULL}};
const Segment SGI154338[] = {{&S51520[34], 11}, {NULL}};
const Segment SGI154339[] = {{&S51520[27], 6}, {NULL}};
const Segment SGI154340[] = {{&S51520[98], 9}, {NULL}};
const Segment SGI154341[] = {{&S51520[71], 6}, {NULL}};
const Segment SGI154342[] = {{&S51521[34], 18}, {NULL}};
const Segment SGI154343[] = {{&S51521[27], 6}, {NULL}};
const Segment SGI154344[] = {{&S51521[105], 9}, {NULL}};
const Segment SGI154345[] = {{&S51521[78], 6}, {NULL}};
const Segment SGI154346[] = {{&S51522[34], 10}, {NULL}};
const Segment SGI154347[] = {{&S51522[27], 6}, {NULL}};
const Segment SGI154348[] = {{&S51522[97], 9}, {NULL}};
const Segment SGI154349[] = {{&S51522[70], 6}, {NULL}};
const Segment SGI154350[] = {{&S51523[34], 11}, {NULL}};
const Segment SGI154351[] = {{&S51523[27], 6}, {NULL}};
const Segment SGI154352[] = {{&S51523[98], 9}, {NULL}};
const Segment SGI154353[] = {{&S51523[71], 6}, {NULL}};
const Segment SGI154354[] = {{&S51524[34], 11}, {NULL}};
const Segment SGI154355[] = {{&S51524[27], 6}, {NULL}};
const Segment SGI154356[] = {{&S51524[98], 9}, {NULL}};
const Segment SGI154357[] = {{&S51524[71], 6}, {NULL}};
const Segment SGI154358[] = {{&S51525[34], 11}, {NULL}};
const Segment SGI154359[] = {{&S51525[27], 6}, {NULL}};
const Segment SGI154360[] = {{&S51525[98], 9}, {NULL}};
const Segment SGI154361[] = {{&S51525[71], 6}, {NULL}};
const Segment SGI154362[] = {{&S51526[34], 11}, {NULL}};
const Segment SGI154363[] = {{&S51526[27], 6}, {NULL}};
const Segment SGI154364[] = {{&S51526[98], 9}, {NULL}};
const Segment SGI154365[] = {{&S51526[71], 6}, {NULL}};
const Segment SGI154366[] = {{&S51527[34], 11}, {NULL}};
const Segment SGI154367[] = {{&S51527[27], 6}, {NULL}};
const Segment SGI154368[] = {{&S51527[98], 9}, {NULL}};
const Segment SGI154369[] = {{&S51527[71], 6}, {NULL}};
const Segment SGI154370[] = {{&S51534[44], 10}, {NULL}};
const Segment SGI154371[] = {{&S51534[44], 22}, {NULL}};
const Segment SGI154372[] = {{&S51534[30], 6}, {NULL}};
const Segment SGI154373[] = {{&S51534[146], 4}, {NULL}};
const Segment SGI154374[] = {{&S51534[172], 3}, {NULL}};
const Segment SGI154375[] = {{&S51534[31], 3}, {&S51534[221], 5}, {NULL}};
const Segment SGI154376[] = {{&S51534[80], 6}, {NULL}};
const Segment SGI154377[] = {{&S37232[44], 10}, {NULL}};
const Segment SGI154378[] = {{&S37232[44], 22}, {NULL}};
const Segment SGI154379[] = {{&S37232[30], 6}, {NULL}};
const Segment SGI154380[] = {{&S37232[133], 3}, {NULL}};
const Segment SGI154381[] = {{&S37232[98], 6}, {NULL}};
const Segment SGI154382[] = {{&S37232[68], 8}, {NULL}};
const Segment SGI154383[] = {{&S51545[44], 10}, {NULL}};
const Segment SGI154384[] = {{&S51545[44], 22}, {NULL}};
const Segment SGI154385[] = {{&S51545[30], 6}, {NULL}};
const Segment SGI154386[] = {{&S51545[31], 3}, {NULL}};
const Segment SGI154387[] = {{&S37235[44], 11}, {NULL}};
const Segment SGI154388[] = {{&S37235[44], 23}, {NULL}};
const Segment SGI154389[] = {{&S51545[80], 6}, {NULL}};
const Segment SGI154390[] = {{&S37235[30], 6}, {NULL}};
const Segment SGI154391[] = {{&S37235[134], 3}, {NULL}};
const Segment SGI154392[] = {{&S37235[99], 6}, {NULL}};
const Segment SGI154393[] = {{&S37235[69], 8}, {NULL}};
const Segment SGI154394[] = {{&S51546[38], 10}, {NULL}};
const Segment SGI154395[] = {{&S51546[31], 6}, {NULL}};
const Segment SGI154396[] = {{&S51547[42], 10}, {NULL}};
const Segment SGI154397[] = {{&S51547[30], 6}, {NULL}};
const Segment SGI154398[] = {{&S51547[31], 3}, {NULL}};
const Segment SGI154399[] = {{&S51547[78], 6}, {NULL}};
const Segment SGI154400[] = {{&S51548[44], 10}, {NULL}};
const Segment SGI154401[] = {{&S51548[44], 22}, {NULL}};
const Segment SGI154402[] = {{&S51548[30], 6}, {NULL}};
const Segment SGI154403[] = {{&S51548[31], 3}, {NULL}};
const Segment SGI154404[] = {{&S51548[80], 6}, {NULL}};
const Segment SGI154405[] = {{&S51549[44], 10}, {NULL}};
const Segment SGI154406[] = {{&S51549[44], 39}, {NULL}};
const Segment SGI154407[] = {{&S51549[30], 6}, {NULL}};
const Segment SGI154408[] = {{&S51549[31], 3}, {NULL}};
const Segment SGI154409[] = {{&S51549[97], 6}, {NULL}};
const Segment SGI154410[] = {{&S42523[44], 8}, {NULL}};
const Segment SGI154411[] = {{&S42523[44], 20}, {NULL}};
const Segment SGI154412[] = {{&S42523[30], 6}, {NULL}};
const Segment SGI154413[] = {{&S42523[113], 3}, {NULL}};
const Segment SGI154414[] = {{&S42523[78], 6}, {NULL}};
const Segment SGI154415[] = {{&S51550[44], 10}, {NULL}};
const Segment SGI154416[] = {{&S51550[44], 39}, {NULL}};
const Segment SGI154417[] = {{&S51550[30], 6}, {NULL}};
const Segment SGI154418[] = {{&S51550[31], 3}, {NULL}};
const Segment SGI154419[] = {{&S51550[97], 6}, {NULL}};
const Segment SGI154420[] = {{&S42524[38], 8}, {NULL}};
const Segment SGI154421[] = {{&S42524[31], 6}, {NULL}};
const Segment SGI154422[] = {{&S51551[44], 10}, {NULL}};
const Segment SGI154423[] = {{&S51551[44], 39}, {NULL}};
const Segment SGI154424[] = {{&S51551[30], 6}, {NULL}};
const Segment SGI154425[] = {{&S51551[31], 3}, {NULL}};
const Segment SGI154426[] = {{&S51551[97], 6}, {NULL}};
const Segment SGI154427[] = {{&S51552[44], 10}, {NULL}};
const Segment SGI154428[] = {{&S51552[44], 39}, {NULL}};
const Segment SGI154429[] = {{&S51552[30], 6}, {NULL}};
const Segment SGI154430[] = {{&S51552[31], 3}, {NULL}};
const Segment SGI154431[] = {{&S51552[97], 6}, {NULL}};
const Segment SGI154432[] = {{&S42529[34], 10}, {NULL}};
const Segment SGI154433[] = {{&S42529[27], 6}, {NULL}};
const Segment SGI154434[] = {{&S42529[97], 9}, {NULL}};
const Segment SGI154435[] = {{&S51553[44], 10}, {NULL}};
const Segment SGI154436[] = {{&S42529[70], 6}, {NULL}};
const Segment SGI154437[] = {{&S51553[44], 41}, {NULL}};
const Segment SGI154438[] = {{&S51553[30], 6}, {NULL}};
const Segment SGI154439[] = {{&S51553[31], 3}, {NULL}};
const Segment SGI154440[] = {{&S51553[99], 6}, {NULL}};
const Segment SGI154441[] = {{&S51555[34], 7}, {NULL}};
const Segment SGI154442[] = {{&S51555[27], 6}, {NULL}};
const Segment SGI154443[] = {{&S51555[98], 9}, {NULL}};
const Segment SGI154444[] = {{&S51555[71], 6}, {NULL}};
const Segment SGI154445[] = {{&S51556[34], 7}, {NULL}};
const Segment SGI154446[] = {{&S51556[27], 6}, {NULL}};
const Segment SGI154447[] = {{&S51556[97], 9}, {NULL}};
const Segment SGI154448[] = {{&S51556[70], 6}, {NULL}};
const Segment SGI154449[] = {{&S51557[34], 8}, {NULL}};
const Segment SGI154450[] = {{&S51557[27], 6}, {NULL}};
const Segment SGI154451[] = {{&S51557[100], 9}, {NULL}};
const Segment SGI154452[] = {{&S51557[73], 6}, {NULL}};
const Segment SGI154453[] = {{&S51558[34], 10}, {NULL}};
const Segment SGI154454[] = {{&S51558[27], 6}, {NULL}};
const Segment SGI154455[] = {{&S51558[97], 9}, {NULL}};
const Segment SGI154456[] = {{&S51558[70], 6}, {NULL}};
const Segment SGI154457[] = {{&S51559[34], 11}, {NULL}};
const Segment SGI154458[] = {{&S51559[27], 6}, {NULL}};
const Segment SGI154459[] = {{&S51559[98], 9}, {NULL}};
const Segment SGI154460[] = {{&S51559[71], 6}, {NULL}};
const Segment SGI154461[] = {{&S51560[34], 11}, {NULL}};
const Segment SGI154462[] = {{&S51560[27], 6}, {NULL}};
const Segment SGI154463[] = {{&S51560[98], 9}, {NULL}};
const Segment SGI154464[] = {{&S51560[71], 6}, {NULL}};
const Segment SGI154465[] = {{&S51571[34], 13}, {NULL}};
const Segment SGI154466[] = {{&S51571[27], 6}, {NULL}};
const Segment SGI154467[] = {{&S51571[100], 9}, {NULL}};
const Segment SGI154468[] = {{&S51571[73], 6}, {NULL}};
const Segment SGI154469[] = {{&S51594[34], 11}, {NULL}};
const Segment SGI154470[] = {{&S51594[27], 6}, {NULL}};
const Segment SGI154471[] = {{&S51594[98], 9}, {NULL}};
const Segment SGI154472[] = {{&S51594[71], 6}, {NULL}};
const Segment SGI154473[] = {{&S41849[44], 10}, {NULL}};
const Segment SGI154474[] = {{&S41849[44], 22}, {NULL}};
const Segment SGI154475[] = {{&S41849[30], 6}, {NULL}};
const Segment SGI154476[] = {{&S41849[115], 3}, {NULL}};
const Segment SGI154477[] = {{&S41849[80], 6}, {NULL}};
const Segment SGI154478[] = {{&S41851[44], 10}, {NULL}};
const Segment SGI154479[] = {{&S41851[44], 22}, {NULL}};
const Segment SGI154480[] = {{&S41851[30], 6}, {NULL}};
const Segment SGI154481[] = {{&S41851[115], 3}, {NULL}};
const Segment SGI154482[] = {{&S41851[80], 6}, {NULL}};
const Segment SGI154483[] = {{&S41852[38], 10}, {NULL}};
const Segment SGI154484[] = {{&S41852[31], 6}, {NULL}};
const Segment SGI154485[] = {{&S41853[44], 27}, {NULL}};
const Segment SGI154486[] = {{&S41853[44], 39}, {NULL}};
const Segment SGI154487[] = {{&S41853[30], 6}, {NULL}};
const Segment SGI154488[] = {{&S41853[132], 3}, {NULL}};
const Segment SGI154489[] = {{&S41853[97], 6}, {NULL}};
const Segment SGI154490[] = {{&S41855[44], 10}, {NULL}};
const Segment SGI154491[] = {{&S41855[44], 21}, {NULL}};
const Segment SGI154492[] = {{&S41855[30], 6}, {NULL}};
const Segment SGI154493[] = {{&S41855[114], 3}, {NULL}};
const Segment SGI154494[] = {{&S41855[79], 6}, {NULL}};
const Segment SGI154495[] = {{&S41856[44], 18}, {NULL}};
const Segment SGI154496[] = {{&S41856[44], 30}, {NULL}};
const Segment SGI154497[] = {{&S41856[30], 6}, {NULL}};
const Segment SGI154498[] = {{&S41856[123], 3}, {NULL}};
const Segment SGI154499[] = {{&S41856[88], 6}, {NULL}};
const Segment SGI154500[] = {{&S41857[38], 18}, {NULL}};
const Segment SGI154501[] = {{&S41857[31], 6}, {NULL}};
const Segment SGI154502[] = {{&S37372[34], 9}, {NULL}};
const Segment SGI154503[] = {{&S37372[27], 6}, {NULL}};
const Segment SGI154504[] = {{&S37372[96], 9}, {NULL}};
const Segment SGI154505[] = {{&S37372[69], 6}, {NULL}};
const Segment SGI154506[] = {{&S37373[34], 9}, {NULL}};
const Segment SGI154507[] = {{&S37373[27], 6}, {NULL}};
const Segment SGI154508[] = {{&S37373[96], 9}, {NULL}};
const Segment SGI154509[] = {{&S37373[69], 6}, {NULL}};
const Segment SGI154510[] = {{&S42562[44], 10}, {NULL}};
const Segment SGI154511[] = {{&S42562[44], 22}, {NULL}};
const Segment SGI154512[] = {{&S42562[30], 6}, {NULL}};
const Segment SGI154513[] = {{&S42562[131], 3}, {NULL}};
const Segment SGI154514[] = {{&S42562[96], 6}, {NULL}};
const Segment SGI154515[] = {{&S42562[68], 8}, {NULL}};
const Segment SGI154516[] = {{&S42563[44], 10}, {NULL}};
const Segment SGI154517[] = {{&S42563[44], 22}, {NULL}};
const Segment SGI154518[] = {{&S42563[30], 6}, {NULL}};
const Segment SGI154519[] = {{&S42563[131], 3}, {NULL}};
const Segment SGI154520[] = {{&S42563[96], 6}, {NULL}};
const Segment SGI154521[] = {{&S41892[44], 10}, {NULL}};
const Segment SGI154522[] = {{&S42563[68], 8}, {NULL}};
const Segment SGI154523[] = {{&S41892[44], 22}, {NULL}};
const Segment SGI154524[] = {{&S41892[30], 6}, {NULL}};
const Segment SGI154525[] = {{&S41892[115], 3}, {NULL}};
const Segment SGI154526[] = {{&S41892[80], 6}, {NULL}};
const Segment SGI154527[] = {{&S42564[44], 10}, {NULL}};
const Segment SGI154528[] = {{&S42564[44], 22}, {NULL}};
const Segment SGI154529[] = {{&S42564[30], 6}, {NULL}};
const Segment SGI154530[] = {{&S42564[131], 3}, {NULL}};
const Segment SGI154531[] = {{&S42564[96], 6}, {NULL}};
const Segment SGI154532[] = {{&S41893[44], 11}, {NULL}};
const Segment SGI154533[] = {{&S41893[44], 23}, {NULL}};
const Segment SGI154534[] = {{&S42564[68], 8}, {NULL}};
const Segment SGI154535[] = {{&S41893[30], 6}, {NULL}};
const Segment SGI154536[] = {{&S41893[116], 3}, {NULL}};
const Segment SGI154537[] = {{&S41893[81], 6}, {NULL}};
const Segment SGI154538[] = {{&S42565[44], 10}, {NULL}};
const Segment SGI154539[] = {{&S42565[44], 22}, {NULL}};
const Segment SGI154540[] = {{&S42565[30], 6}, {NULL}};
const Segment SGI154541[] = {{&S41894[38], 10}, {NULL}};
const Segment SGI154542[] = {{&S42565[131], 3}, {NULL}};
const Segment SGI154543[] = {{&S41894[31], 6}, {NULL}};
const Segment SGI154544[] = {{&S42565[96], 6}, {NULL}};
const Segment SGI154545[] = {{&S42565[68], 8}, {NULL}};
const Segment SGI154546[] = {{&S41895[44], 11}, {NULL}};
const Segment SGI154547[] = {{&S41895[44], 23}, {NULL}};
const Segment SGI154548[] = {{&S42566[44], 10}, {NULL}};
const Segment SGI154549[] = {{&S41895[30], 6}, {NULL}};
const Segment SGI154550[] = {{&S42566[44], 22}, {NULL}};
const Segment SGI154551[] = {{&S42566[30], 6}, {NULL}};
const Segment SGI154552[] = {{&S41895[116], 3}, {NULL}};
const Segment SGI154553[] = {{&S41895[81], 6}, {NULL}};
const Segment SGI154554[] = {{&S42566[131], 3}, {NULL}};
const Segment SGI154555[] = {{&S42566[96], 6}, {NULL}};
const Segment SGI154556[] = {{&S42566[68], 8}, {NULL}};
const Segment SGI154557[] = {{&S41896[38], 11}, {NULL}};
const Segment SGI154558[] = {{&S41896[31], 6}, {NULL}};
const Segment SGI154559[] = {{&S42567[44], 10}, {NULL}};
const Segment SGI154560[] = {{&S42567[44], 22}, {NULL}};
const Segment SGI154561[] = {{&S42567[30], 6}, {NULL}};
const Segment SGI154562[] = {{&S42567[131], 3}, {NULL}};
const Segment SGI154563[] = {{&S42567[96], 6}, {NULL}};
const Segment SGI154564[] = {{&S41897[38], 11}, {NULL}};
const Segment SGI154565[] = {{&S42567[68], 8}, {NULL}};
const Segment SGI154566[] = {{&S41897[31], 6}, {NULL}};
const Segment SGI154567[] = {{&S42568[44], 10}, {NULL}};
const Segment SGI154568[] = {{&S42568[44], 22}, {NULL}};
const Segment SGI154569[] = {{&S42568[30], 6}, {NULL}};
const Segment SGI154570[] = {{&S42568[131], 3}, {NULL}};
const Segment SGI154571[] = {{&S42568[96], 6}, {NULL}};
const Segment SGI154572[] = {{&S42568[68], 8}, {NULL}};
const Segment SGI154573[] = {{&S42569[44], 10}, {NULL}};
const Segment SGI154574[] = {{&S42569[44], 22}, {NULL}};
const Segment SGI154575[] = {{&S42569[30], 6}, {NULL}};
const Segment SGI154576[] = {{&S42569[131], 3}, {NULL}};
const Segment SGI154577[] = {{&S42569[96], 6}, {NULL}};
const Segment SGI154578[] = {{&S42569[68], 8}, {NULL}};
const Segment SGI154579[] = {{&S42570[44], 10}, {NULL}};
const Segment SGI154580[] = {{&S42570[44], 22}, {NULL}};
const Segment SGI154581[] = {{&S42570[30], 6}, {NULL}};
const Segment SGI154582[] = {{&S42570[131], 3}, {NULL}};
const Segment SGI154583[] = {{&S42570[96], 6}, {NULL}};
const Segment SGI154584[] = {{&S42570[68], 8}, {NULL}};
const Segment SGI154585[] = {{&S51641[34], 8}, {NULL}};
const Segment SGI154586[] = {{&S51641[27], 6}, {NULL}};
const Segment SGI154587[] = {{&S51641[95], 9}, {NULL}};
const Segment SGI154588[] = {{&S51641[68], 6}, {NULL}};
const Segment SGI154589[] = {{&S41909[34], 7}, {NULL}};
const Segment SGI154590[] = {{&S41909[27], 6}, {NULL}};
const Segment SGI154591[] = {{&S41909[100], 9}, {NULL}};
const Segment SGI154592[] = {{&S41909[73], 6}, {NULL}};
const Segment SGI154593[] = {{&S51642[34], 10}, {NULL}};
const Segment SGI154594[] = {{&S51642[27], 6}, {NULL}};
const Segment SGI154595[] = {{&S51642[97], 9}, {NULL}};
const Segment SGI154596[] = {{&S51642[70], 6}, {NULL}};
const Segment SGI154597[] = {{&S51643[34], 10}, {NULL}};
const Segment SGI154598[] = {{&S51643[27], 6}, {NULL}};
const Segment SGI154599[] = {{&S41911[44], 10}, {NULL}};
const Segment SGI154600[] = {{&S41911[44], 22}, {NULL}};
const Segment SGI154601[] = {{&S41911[30], 6}, {NULL}};
const Segment SGI154602[] = {{&S51643[97], 9}, {NULL}};
const Segment SGI154603[] = {{&S51643[70], 6}, {NULL}};
const Segment SGI154604[] = {{&S41911[115], 3}, {NULL}};
const Segment SGI154605[] = {{&S41911[80], 6}, {NULL}};
const Segment SGI154606[] = {{&S51644[34], 10}, {NULL}};
const Segment SGI154607[] = {{&S51644[27], 6}, {NULL}};
const Segment SGI154608[] = {{&S41912[38], 10}, {NULL}};
const Segment SGI154609[] = {{&S41912[31], 6}, {NULL}};
const Segment SGI154610[] = {{&S51644[97], 9}, {NULL}};
const Segment SGI154611[] = {{&S51644[70], 6}, {NULL}};
const Segment SGI154612[] = {{&S51645[34], 11}, {NULL}};
const Segment SGI154613[] = {{&S51645[27], 6}, {NULL}};
const Segment SGI154614[] = {{&S51645[98], 9}, {NULL}};
const Segment SGI154615[] = {{&S51645[71], 6}, {NULL}};
const Segment SGI154616[] = {{&S51646[34], 10}, {NULL}};
const Segment SGI154617[] = {{&S51646[27], 6}, {NULL}};
const Segment SGI154618[] = {{&S51646[97], 9}, {NULL}};
const Segment SGI154619[] = {{&S51646[70], 6}, {NULL}};
const Segment SGI154620[] = {{&S51648[34], 11}, {NULL}};
const Segment SGI154621[] = {{&S51648[27], 6}, {NULL}};
const Segment SGI154622[] = {{&S51648[98], 9}, {NULL}};
const Segment SGI154623[] = {{&S51648[71], 6}, {NULL}};
const Segment SGI154624[] = {{&S51649[34], 7}, {NULL}};
const Segment SGI154625[] = {{&S51649[27], 6}, {NULL}};
const Segment SGI154626[] = {{&S51649[94], 9}, {NULL}};
const Segment SGI154627[] = {{&S51649[67], 6}, {NULL}};
const Segment SGI154628[] = {{&S51650[34], 10}, {NULL}};
const Segment SGI154629[] = {{&S51650[27], 6}, {NULL}};
const Segment SGI154630[] = {{&S51650[97], 9}, {NULL}};
const Segment SGI154631[] = {{&S51650[70], 6}, {NULL}};
const Segment SGI154632[] = {{&S51651[34], 10}, {NULL}};
const Segment SGI154633[] = {{&S51651[27], 6}, {NULL}};
const Segment SGI154634[] = {{&S51651[97], 9}, {NULL}};
const Segment SGI154635[] = {{&S51651[70], 6}, {NULL}};
const Segment SGI154636[] = {{&S51661[34], 7}, {NULL}};
const Segment SGI154637[] = {{&S51661[27], 6}, {NULL}};
const Segment SGI154638[] = {{&S51661[98], 9}, {NULL}};
const Segment SGI154639[] = {{&S51661[71], 6}, {NULL}};
const Segment SGI154640[] = {{&S51662[34], 7}, {NULL}};
const Segment SGI154641[] = {{&S51662[27], 6}, {NULL}};
const Segment SGI154642[] = {{&S51662[99], 9}, {NULL}};
const Segment SGI154643[] = {{&S51662[72], 6}, {NULL}};
const Segment SGI154644[] = {{&S51680[44], 8}, {NULL}};
const Segment SGI154645[] = {{&S51680[44], 25}, {NULL}};
const Segment SGI154646[] = {{&S51680[30], 6}, {NULL}};
const Segment SGI154647[] = {{&S51680[31], 3}, {NULL}};
const Segment SGI154648[] = {{&S51680[83], 6}, {NULL}};
const Segment SGI154649[] = {{&S51681[38], 8}, {NULL}};
const Segment SGI154650[] = {{&S51681[31], 6}, {NULL}};
const Segment SGI154651[] = {{&S51682[44], 8}, {NULL}};
const Segment SGI154652[] = {{&S51682[44], 25}, {NULL}};
const Segment SGI154653[] = {{&S51682[30], 6}, {NULL}};
const Segment SGI154654[] = {{&S51682[31], 3}, {NULL}};
const Segment SGI154655[] = {{&S51682[83], 6}, {NULL}};
const Segment SGI154656[] = {{&S51683[34], 12}, {NULL}};
const Segment SGI154657[] = {{&S51683[27], 6}, {NULL}};
const Segment SGI154658[] = {{&S51683[99], 9}, {NULL}};
const Segment SGI154659[] = {{&S51683[72], 6}, {NULL}};
const Segment SGI154660[] = {{&S51684[44], 17}, {NULL}};
const Segment SGI154661[] = {{&S51684[44], 26}, {NULL}};
const Segment SGI154662[] = {{&S51684[30], 6}, {NULL}};
const Segment SGI154663[] = {{&S51684[31], 3}, {NULL}};
const Segment SGI154664[] = {{&S51684[84], 6}, {NULL}};
const Segment SGI154665[] = {{&S51685[44], 9}, {NULL}};
const Segment SGI154666[] = {{&S51685[44], 18}, {NULL}};
const Segment SGI154667[] = {{&S51685[30], 6}, {NULL}};
const Segment SGI154668[] = {{&S51685[31], 3}, {NULL}};
const Segment SGI154669[] = {{&S51685[76], 6}, {NULL}};
const Segment SGI154670[] = {{&S51686[38], 9}, {NULL}};
const Segment SGI154671[] = {{&S51686[31], 6}, {NULL}};
const Segment SGI154672[] = {{&S42644[34], 10}, {NULL}};
const Segment SGI154673[] = {{&S42644[27], 6}, {NULL}};
const Segment SGI154674[] = {{&S42644[97], 9}, {NULL}};
const Segment SGI154675[] = {{&S42644[70], 6}, {NULL}};
const Segment SGI154676[] = {{&S51703[44], 15}, {NULL}};
const Segment SGI154677[] = {{&S51703[44], 28}, {NULL}};
const Segment SGI154678[] = {{&S51703[30], 6}, {NULL}};
const Segment SGI154679[] = {{&S51703[31], 3}, {NULL}};
const Segment SGI154680[] = {{&S51703[86], 6}, {NULL}};
const Segment SGI154681[] = {{&S51704[44], 15}, {NULL}};
const Segment SGI154682[] = {{&S51704[44], 28}, {NULL}};
const Segment SGI154683[] = {{&S51704[30], 6}, {NULL}};
const Segment SGI154684[] = {{&S51704[31], 3}, {NULL}};
const Segment SGI154685[] = {{&S51704[86], 6}, {NULL}};
const Segment SGI154686[] = {{&S51705[44], 15}, {NULL}};
const Segment SGI154687[] = {{&S51705[44], 31}, {NULL}};
const Segment SGI154688[] = {{&S51705[30], 6}, {NULL}};
const Segment SGI154689[] = {{&S51705[31], 3}, {NULL}};
const Segment SGI154690[] = {{&S51705[89], 6}, {NULL}};
const Segment SGI154691[] = {{&S41992[44], 10}, {NULL}};
const Segment SGI154692[] = {{&S41992[44], 22}, {NULL}};
const Segment SGI154693[] = {{&S41992[30], 6}, {NULL}};
const Segment SGI154694[] = {{&S41992[115], 3}, {NULL}};
const Segment SGI154695[] = {{&S41992[80], 6}, {NULL}};
const Segment SGI154696[] = {{&S41993[38], 10}, {NULL}};
const Segment SGI154697[] = {{&S41993[31], 6}, {NULL}};
const Segment SGI154698[] = {{&S41994[44], 11}, {NULL}};
const Segment SGI154699[] = {{&S41994[44], 23}, {NULL}};
const Segment SGI154700[] = {{&S41994[30], 6}, {NULL}};
const Segment SGI154701[] = {{&S41994[116], 3}, {NULL}};
const Segment SGI154702[] = {{&S41994[81], 6}, {NULL}};
const Segment SGI154703[] = {{&S41995[38], 11}, {NULL}};
const Segment SGI154704[] = {{&S41995[31], 6}, {NULL}};
const Segment SGI154705[] = {{&S51721[34], 7}, {NULL}};
const Segment SGI154706[] = {{&S51721[27], 6}, {NULL}};
const Segment SGI154707[] = {{&S51721[98], 9}, {NULL}};
const Segment SGI154708[] = {{&S51721[71], 6}, {NULL}};
const Segment SGI154709[] = {{&S37537[34], 14}, {NULL}};
const Segment SGI154710[] = {{&S37537[27], 6}, {NULL}};
const Segment SGI154711[] = {{&S37537[101], 9}, {NULL}};
const Segment SGI154712[] = {{&S37537[74], 6}, {NULL}};
const Segment SGI154713[] = {{&S51743[34], 10}, {NULL}};
const Segment SGI154714[] = {{&S51743[27], 6}, {NULL}};
const Segment SGI154715[] = {{&S51743[97], 9}, {NULL}};
const Segment SGI154716[] = {{&S51743[70], 6}, {NULL}};
const Segment SGI154717[] = {{&S51744[34], 10}, {NULL}};
const Segment SGI154718[] = {{&S51744[27], 6}, {NULL}};
const Segment SGI154719[] = {{&S51744[97], 9}, {NULL}};
const Segment SGI154720[] = {{&S51744[70], 6}, {NULL}};
const Segment SGI154721[] = {{&S51745[34], 10}, {NULL}};
const Segment SGI154722[] = {{&S51745[27], 6}, {NULL}};
const Segment SGI154723[] = {{&S51745[97], 9}, {NULL}};
const Segment SGI154724[] = {{&S51745[70], 6}, {NULL}};
const Segment SGI154725[] = {{&S27804[41], 18}, {NULL}};
const Segment SGI154726[] = {{&S27804[41], 29}, {NULL}};
const Segment SGI154727[] = {{&S27804[27], 6}, {NULL}};
const Segment SGI154728[] = {{&S27804[119], 3}, {NULL}};
const Segment SGI154729[] = {{&S27804[123], 9}, {NULL}};
const Segment SGI154730[] = {{&S27804[84], 6}, {NULL}};
const Segment SGI154731[] = {{&S27805[34], 10}, {NULL}};
const Segment SGI154732[] = {{&S27805[27], 6}, {NULL}};
const Segment SGI154733[] = {{&S27805[96], 9}, {NULL}};
const Segment SGI154734[] = {{&S27805[69], 6}, {NULL}};
const Segment SGI154735[] = {{&S42061[34], 15}, {NULL}};
const Segment SGI154736[] = {{&S42061[27], 6}, {NULL}};
const Segment SGI154737[] = {{&S42061[102], 9}, {NULL}};
const Segment SGI154738[] = {{&S42061[75], 6}, {NULL}};
const Segment SGI154739[] = {{&S42062[34], 6}, {NULL}};
const Segment SGI154740[] = {{&S42062[27], 6}, {NULL}};
const Segment SGI154741[] = {{&S42062[93], 9}, {NULL}};
const Segment SGI154742[] = {{&S42062[66], 6}, {NULL}};
const Segment SGI154743[] = {{&S42064[34], 17}, {NULL}};
const Segment SGI154744[] = {{&S42064[27], 6}, {NULL}};
const Segment SGI154745[] = {{&S42064[104], 9}, {NULL}};
const Segment SGI154746[] = {{&S42064[77], 6}, {NULL}};
const Segment SGI154747[] = {{&S42068[34], 17}, {NULL}};
const Segment SGI154748[] = {{&S42068[27], 6}, {NULL}};
const Segment SGI154749[] = {{&S42068[104], 9}, {NULL}};
const Segment SGI154750[] = {{&S42068[77], 6}, {NULL}};
const Segment SGI154751[] = {{&S42069[34], 8}, {NULL}};
const Segment SGI154752[] = {{&S42069[27], 6}, {NULL}};
const Segment SGI154753[] = {{&S42069[95], 9}, {NULL}};
const Segment SGI154754[] = {{&S42069[68], 6}, {NULL}};
const Segment SGI154755[] = {{&S42072[34], 9}, {NULL}};
const Segment SGI154756[] = {{&S42072[27], 6}, {NULL}};
const Segment SGI154757[] = {{&S42072[96], 9}, {NULL}};
const Segment SGI154758[] = {{&S42072[69], 6}, {NULL}};
const Segment SGI154759[] = {{&S42075[34], 16}, {NULL}};
const Segment SGI154760[] = {{&S42075[27], 6}, {NULL}};
const Segment SGI154761[] = {{&S42075[103], 9}, {NULL}};
const Segment SGI154762[] = {{&S42075[76], 6}, {NULL}};
const Segment SGI154763[] = {{&S42077[34], 20}, {NULL}};
const Segment SGI154764[] = {{&S42077[27], 6}, {NULL}};
const Segment SGI154765[] = {{&S42077[116], 9}, {NULL}};
const Segment SGI154766[] = {{&S42077[89], 6}, {NULL}};
const Segment SGI154767[] = {{&S51803[34], 10}, {NULL}};
const Segment SGI154768[] = {{&S51803[27], 6}, {NULL}};
const Segment SGI154769[] = {{&S51803[97], 9}, {NULL}};
const Segment SGI154770[] = {{&S51803[70], 6}, {NULL}};
const Segment SGI154771[] = {{&S51804[34], 11}, {NULL}};
const Segment SGI154772[] = {{&S51804[27], 6}, {NULL}};
const Segment SGI154773[] = {{&S51804[98], 9}, {NULL}};
const Segment SGI154774[] = {{&S51804[71], 6}, {NULL}};
const Segment SGI154775[] = {{&S51805[34], 18}, {NULL}};
const Segment SGI154776[] = {{&S51805[27], 6}, {NULL}};
const Segment SGI154777[] = {{&S51805[105], 9}, {NULL}};
const Segment SGI154778[] = {{&S51805[78], 6}, {NULL}};
const Segment SGI154779[] = {{&S51806[34], 11}, {NULL}};
const Segment SGI154780[] = {{&S51806[27], 6}, {NULL}};
const Segment SGI154781[] = {{&S51806[98], 9}, {NULL}};
const Segment SGI154782[] = {{&S51806[71], 6}, {NULL}};
const Segment SGI154783[] = {{&S51807[34], 11}, {NULL}};
const Segment SGI154784[] = {{&S51807[27], 6}, {NULL}};
const Segment SGI154785[] = {{&S51807[98], 9}, {NULL}};
const Segment SGI154786[] = {{&S51807[71], 6}, {NULL}};
const Segment SGI154787[] = {{&S51808[34], 10}, {NULL}};
const Segment SGI154788[] = {{&S51808[27], 6}, {NULL}};
const Segment SGI154789[] = {{&S51808[97], 9}, {NULL}};
const Segment SGI154790[] = {{&S51808[70], 6}, {NULL}};
const Segment SGI154791[] = {{&S42119[44], 11}, {NULL}};
const Segment SGI154792[] = {{&S42119[44], 23}, {NULL}};
const Segment SGI154793[] = {{&S42119[30], 6}, {NULL}};
const Segment SGI154794[] = {{&S42119[116], 3}, {NULL}};
const Segment SGI154795[] = {{&S42119[81], 6}, {NULL}};
const Segment SGI154796[] = {{&S42120[44], 11}, {NULL}};
const Segment SGI154797[] = {{&S42120[44], 23}, {NULL}};
const Segment SGI154798[] = {{&S42120[30], 6}, {NULL}};
const Segment SGI154799[] = {{&S42120[116], 3}, {NULL}};
const Segment SGI154800[] = {{&S42120[81], 6}, {NULL}};
const Segment SGI154801[] = {{&S42121[38], 11}, {NULL}};
const Segment SGI154802[] = {{&S42121[31], 6}, {NULL}};
const Segment SGI154803[] = {{&S42122[38], 11}, {NULL}};
const Segment SGI154804[] = {{&S42122[31], 6}, {NULL}};
const Segment SGI154805[] = {{&S42123[44], 10}, {NULL}};
const Segment SGI154806[] = {{&S42123[44], 22}, {NULL}};
const Segment SGI154807[] = {{&S42123[30], 6}, {NULL}};
const Segment SGI154808[] = {{&S42123[115], 3}, {NULL}};
const Segment SGI154809[] = {{&S42123[80], 6}, {NULL}};
const Segment SGI154810[] = {{&S42124[42], 11}, {NULL}};
const Segment SGI154811[] = {{&S42124[30], 6}, {NULL}};
const Segment SGI154812[] = {{&S42124[114], 3}, {NULL}};
const Segment SGI154813[] = {{&S42124[79], 6}, {NULL}};
const Segment SGI154814[] = {{&S42132[34], 12}, {NULL}};
const Segment SGI154815[] = {{&S42132[27], 6}, {NULL}};
const Segment SGI154816[] = {{&S42132[109], 9}, {NULL}};
const Segment SGI154817[] = {{&S42132[82], 6}, {NULL}};
const Segment SGI154818[] = {{&S42133[34], 12}, {NULL}};
const Segment SGI154819[] = {{&S42133[27], 6}, {NULL}};
const Segment SGI154820[] = {{&S42133[109], 9}, {NULL}};
const Segment SGI154821[] = {{&S42133[82], 6}, {NULL}};
const Segment SGI154822[] = {{&S42134[44], 11}, {NULL}};
const Segment SGI154823[] = {{&S42134[44], 23}, {NULL}};
const Segment SGI154824[] = {{&S42134[30], 6}, {NULL}};
const Segment SGI154825[] = {{&S42134[116], 3}, {NULL}};
const Segment SGI154826[] = {{&S42134[81], 6}, {NULL}};
const Segment SGI154827[] = {{&S42135[44], 11}, {NULL}};
const Segment SGI154828[] = {{&S42135[44], 23}, {NULL}};
const Segment SGI154829[] = {{&S42135[30], 6}, {NULL}};
const Segment SGI154830[] = {{&S42135[116], 3}, {NULL}};
const Segment SGI154831[] = {{&S42135[81], 6}, {NULL}};
const Segment SGI154832[] = {{&S42136[44], 11}, {NULL}};
const Segment SGI154833[] = {{&S42136[44], 23}, {NULL}};
const Segment SGI154834[] = {{&S42136[30], 6}, {NULL}};
const Segment SGI154835[] = {{&S42136[116], 3}, {NULL}};
const Segment SGI154836[] = {{&S42136[81], 6}, {NULL}};
const Segment SGI154837[] = {{&S42137[38], 11}, {NULL}};
const Segment SGI154838[] = {{&S42137[31], 6}, {NULL}};
const Segment SGI154839[] = {{&S42138[38], 11}, {NULL}};
const Segment SGI154840[] = {{&S42138[31], 6}, {NULL}};
const Segment SGI154841[] = {{&S42139[44], 18}, {NULL}};
const Segment SGI154842[] = {{&S42139[44], 30}, {NULL}};
const Segment SGI154843[] = {{&S42139[30], 6}, {NULL}};
const Segment SGI154844[] = {{&S42139[123], 3}, {NULL}};
const Segment SGI154845[] = {{&S42139[88], 6}, {NULL}};
const Segment SGI154846[] = {{&S42140[38], 11}, {NULL}};
const Segment SGI154847[] = {{&S42140[31], 6}, {NULL}};
const Segment SGI154848[] = {{&S42141[44], 11}, {NULL}};
const Segment SGI154849[] = {{&S42141[44], 28}, {NULL}};
const Segment SGI154850[] = {{&S42141[30], 6}, {NULL}};
const Segment SGI154851[] = {{&S42141[120], 3}, {NULL}};
const Segment SGI154852[] = {{&S42141[86], 5}, {NULL}};
const Segment SGI154853[] = {{&S42142[44], 11}, {NULL}};
const Segment SGI154854[] = {{&S42142[44], 23}, {NULL}};
const Segment SGI154855[] = {{&S42142[30], 6}, {NULL}};
const Segment SGI154856[] = {{&S42142[115], 3}, {NULL}};
const Segment SGI154857[] = {{&S42142[81], 5}, {NULL}};
const Segment SGI154858[] = {{&S42143[44], 11}, {NULL}};
const Segment SGI154859[] = {{&S42143[44], 23}, {NULL}};
const Segment SGI154860[] = {{&S42143[30], 6}, {NULL}};
const Segment SGI154861[] = {{&S42143[115], 3}, {NULL}};
const Segment SGI154862[] = {{&S42143[81], 5}, {NULL}};
const Segment SGI154863[] = {{&S42144[44], 18}, {NULL}};
const Segment SGI154864[] = {{&S42144[44], 30}, {NULL}};
const Segment SGI154865[] = {{&S42144[30], 6}, {NULL}};
const Segment SGI154866[] = {{&S42144[123], 3}, {NULL}};
const Segment SGI154867[] = {{&S42144[88], 6}, {NULL}};
const Segment SGI154868[] = {{&S42164[34], 7}, {NULL}};
const Segment SGI154869[] = {{&S42164[27], 6}, {NULL}};
const Segment SGI154870[] = {{&S42164[100], 9}, {NULL}};
const Segment SGI154871[] = {{&S42164[73], 6}, {NULL}};
const Segment SGI154872[] = {{&S51906[34], 21}, {NULL}};
const Segment SGI154873[] = {{&S51906[27], 6}, {NULL}};
const Segment SGI154874[] = {{&S37708[34], 9}, {NULL}};
const Segment SGI154875[] = {{&S37708[27], 6}, {NULL}};
const Segment SGI154876[] = {{&S51906[108], 9}, {NULL}};
const Segment SGI154877[] = {{&S51906[81], 6}, {NULL}};
const Segment SGI154878[] = {{&S37708[96], 9}, {NULL}};
const Segment SGI154879[] = {{&S37708[69], 6}, {NULL}};
const Segment SGI154880[] = {{&S51927[34], 9}, {NULL}};
const Segment SGI154881[] = {{&S51927[27], 6}, {NULL}};
const Segment SGI154882[] = {{&S51927[96], 9}, {NULL}};
const Segment SGI154883[] = {{&S51927[69], 6}, {NULL}};
const Segment SGI154884[] = {{&S51928[34], 10}, {NULL}};
const Segment SGI154885[] = {{&S51928[27], 6}, {NULL}};
const Segment SGI154886[] = {{&S51928[97], 9}, {NULL}};
const Segment SGI154887[] = {{&S51928[70], 6}, {NULL}};
const Segment SGI154888[] = {{&S37737[38], 29}, {NULL}};
const Segment SGI154889[] = {{&S37737[31], 6}, {NULL}};
const Segment SGI154890[] = {{&S37738[44], 29}, {NULL}};
const Segment SGI154891[] = {{&S37738[44], 41}, {NULL}};
const Segment SGI154892[] = {{&S37738[30], 6}, {NULL}};
const Segment SGI154893[] = {{&S37738[134], 3}, {NULL}};
const Segment SGI154894[] = {{&S37738[99], 6}, {NULL}};
const Segment SGI154895[] = {{&S37740[44], 17}, {NULL}};
const Segment SGI154896[] = {{&S37740[44], 29}, {NULL}};
const Segment SGI154897[] = {{&S37740[30], 6}, {NULL}};
const Segment SGI154898[] = {{&S37740[122], 3}, {NULL}};
const Segment SGI154899[] = {{&S37740[87], 6}, {NULL}};
const Segment SGI154900[] = {{&S51955[34], 21}, {NULL}};
const Segment SGI154901[] = {{&S51955[27], 6}, {NULL}};
const Segment SGI154902[] = {{&S51955[108], 9}, {NULL}};
const Segment SGI154903[] = {{&S51955[81], 6}, {NULL}};
const Segment SGI154904[] = {{&S51957[34], 18}, {NULL}};
const Segment SGI154905[] = {{&S51957[27], 6}, {NULL}};
const Segment SGI154906[] = {{&S51957[105], 9}, {NULL}};
const Segment SGI154907[] = {{&S51957[78], 6}, {NULL}};
const Segment SGI154908[] = {{&S51971[44], 6}, {NULL}};
const Segment SGI154909[] = {{&S51971[44], 18}, {NULL}};
const Segment SGI154910[] = {{&S51971[30], 6}, {NULL}};
const Segment SGI154911[] = {{&S51971[31], 3}, {NULL}};
const Segment SGI154912[] = {{&S51971[76], 6}, {NULL}};
const Segment SGI154913[] = {{&S51972[44], 6}, {NULL}};
const Segment SGI154914[] = {{&S51972[44], 18}, {NULL}};
const Segment SGI154915[] = {{&S51972[30], 6}, {NULL}};
const Segment SGI154916[] = {{&S51972[31], 3}, {NULL}};
const Segment SGI154917[] = {{&S51972[76], 6}, {NULL}};
const Segment SGI154918[] = {{&S42834[44], 10}, {NULL}};
const Segment SGI154919[] = {{&S42834[44], 22}, {NULL}};
const Segment SGI154920[] = {{&S42834[30], 6}, {NULL}};
const Segment SGI154921[] = {{&S42834[115], 3}, {NULL}};
const Segment SGI154922[] = {{&S42834[80], 6}, {NULL}};
const Segment SGI154923[] = {{&S51973[38], 6}, {NULL}};
const Segment SGI154924[] = {{&S51973[31], 6}, {NULL}};
const Segment SGI154925[] = {{&S51996[34], 11}, {NULL}};
const Segment SGI154926[] = {{&S51996[27], 6}, {NULL}};
const Segment SGI154927[] = {{&S51996[98], 9}, {NULL}};
const Segment SGI154928[] = {{&S51996[71], 6}, {NULL}};
const Segment SGI154929[] = {{&S52027[44], 8}, {NULL}};
const Segment SGI154930[] = {{&S52027[44], 19}, {NULL}};
const Segment SGI154931[] = {{&S52027[30], 6}, {NULL}};
const Segment SGI154932[] = {{&S52027[31], 3}, {NULL}};
const Segment SGI154933[] = {{&S52027[77], 6}, {NULL}};
const Segment SGI154934[] = {{&S52028[44], 8}, {NULL}};
const Segment SGI154935[] = {{&S52028[44], 19}, {NULL}};
const Segment SGI154936[] = {{&S52028[30], 6}, {NULL}};
const Segment SGI154937[] = {{&S52028[31], 3}, {NULL}};
const Segment SGI154938[] = {{&S52028[77], 6}, {NULL}};
const Segment SGI154939[] = {{&S52029[44], 8}, {NULL}};
const Segment SGI154940[] = {{&S52029[44], 19}, {NULL}};
const Segment SGI154941[] = {{&S52029[30], 6}, {NULL}};
const Segment SGI154942[] = {{&S52029[31], 3}, {NULL}};
const Segment SGI154943[] = {{&S52029[77], 6}, {NULL}};
const Segment SGI154944[] = {{&S52030[38], 8}, {NULL}};
const Segment SGI154945[] = {{&S52030[31], 6}, {NULL}};
const Segment SGI154946[] = {{&S52031[44], 8}, {NULL}};
const Segment SGI154947[] = {{&S52031[44], 19}, {NULL}};
const Segment SGI154948[] = {{&S52031[30], 6}, {NULL}};
const Segment SGI154949[] = {{&S52031[31], 3}, {NULL}};
const Segment SGI154950[] = {{&S52031[77], 6}, {NULL}};
const Segment SGI154951[] = {{&S42892[44], 10}, {NULL}};
const Segment SGI154952[] = {{&S42892[44], 22}, {NULL}};
const Segment SGI154953[] = {{&S42892[30], 6}, {NULL}};
const Segment SGI154954[] = {{&S42892[115], 3}, {NULL}};
const Segment SGI154955[] = {{&S42892[80], 6}, {NULL}};
const Segment SGI154956[] = {{&S42893[44], 18}, {NULL}};
const Segment SGI154957[] = {{&S42893[44], 30}, {NULL}};
const Segment SGI154958[] = {{&S42893[30], 6}, {NULL}};
const Segment SGI154959[] = {{&S42893[123], 3}, {NULL}};
const Segment SGI154960[] = {{&S42893[88], 6}, {NULL}};
const Segment SGI154961[] = {{&S42894[44], 18}, {NULL}};
const Segment SGI154962[] = {{&S42894[44], 30}, {NULL}};
const Segment SGI154963[] = {{&S42894[30], 6}, {NULL}};
const Segment SGI154964[] = {{&S52040[34], 10}, {NULL}};
const Segment SGI154965[] = {{&S42894[123], 3}, {NULL}};
const Segment SGI154966[] = {{&S52040[27], 6}, {NULL}};
const Segment SGI154967[] = {{&S42894[88], 6}, {NULL}};
const Segment SGI154968[] = {{&S52040[97], 9}, {NULL}};
const Segment SGI154969[] = {{&S52040[70], 6}, {NULL}};
const Segment SGI154970[] = {{&S42895[44], 10}, {NULL}};
const Segment SGI154971[] = {{&S42895[44], 22}, {NULL}};
const Segment SGI154972[] = {{&S42895[30], 6}, {NULL}};
const Segment SGI154973[] = {{&S42895[115], 3}, {NULL}};
const Segment SGI154974[] = {{&S42895[80], 6}, {NULL}};
const Segment SGI154975[] = {{&S52044[34], 10}, {NULL}};
const Segment SGI154976[] = {{&S52044[27], 6}, {NULL}};
const Segment SGI154977[] = {{&S52044[97], 9}, {NULL}};
const Segment SGI154978[] = {{&S52044[70], 6}, {NULL}};
const Segment SGI154979[] = {{&S52046[34], 11}, {NULL}};
const Segment SGI154980[] = {{&S52046[27], 6}, {NULL}};
const Segment SGI154981[] = {{&S52046[98], 9}, {NULL}};
const Segment SGI154982[] = {{&S52046[71], 6}, {NULL}};
const Segment SGI154983[] = {{&S52047[34], 14}, {NULL}};
const Segment SGI154984[] = {{&S52047[27], 6}, {NULL}};
const Segment SGI154985[] = {{&S52047[101], 9}, {NULL}};
const Segment SGI154986[] = {{&S52047[74], 6}, {NULL}};
const Segment SGI154987[] = {{&S52048[34], 15}, {NULL}};
const Segment SGI154988[] = {{&S52048[27], 6}, {NULL}};
const Segment SGI154989[] = {{&S52048[102], 9}, {NULL}};
const Segment SGI154990[] = {{&S52048[75], 6}, {NULL}};
const Segment SGI154991[] = {{&S52056[30], 6}, {NULL}};
const Segment SGI154992[] = {{&S52056[31], 3}, {NULL}};
const Segment SGI154993[] = {{&S52056[82], 6}, {NULL}};
const Segment SGI154994[] = {{&S52057[44], 14}, {NULL}};
const Segment SGI154995[] = {{&S52057[44], 26}, {NULL}};
const Segment SGI154996[] = {{&S52057[30], 6}, {NULL}};
const Segment SGI154997[] = {{&S52057[31], 3}, {NULL}};
const Segment SGI154998[] = {{&S52057[84], 6}, {NULL}};
const Segment SGI154999[] = {{&S52058[38], 14}, {NULL}};
const Segment SGI155000[] = {{&S52058[31], 6}, {NULL}};
const Segment SGI155001[] = {{&S52059[44], 14}, {NULL}};
const Segment SGI155002[] = {{&S52059[44], 26}, {NULL}};
const Segment SGI155003[] = {{&S52059[30], 6}, {NULL}};
const Segment SGI155004[] = {{&S52059[87], 6}, {NULL}};
const Segment SGI155005[] = {{&S52060[44], 7}, {NULL}};
const Segment SGI155006[] = {{&S52060[44], 19}, {NULL}};
const Segment SGI155007[] = {{&S52060[30], 6}, {NULL}};
const Segment SGI155008[] = {{&S52060[31], 3}, {NULL}};
const Segment SGI155009[] = {{&S52060[77], 6}, {NULL}};
const Segment SGI155010[] = {{&S52091[34], 11}, {NULL}};
const Segment SGI155011[] = {{&S52091[27], 6}, {NULL}};
const Segment SGI155012[] = {{&S52091[98], 9}, {NULL}};
const Segment SGI155013[] = {{&S52091[71], 6}, {NULL}};
const Segment SGI155014[] = {{&S52092[34], 11}, {NULL}};
const Segment SGI155015[] = {{&S52092[27], 6}, {NULL}};
const Segment SGI155016[] = {{&S52092[98], 9}, {NULL}};
const Segment SGI155017[] = {{&S52092[71], 6}, {NULL}};
const Segment SGI155018[] = {{&S52093[34], 10}, {NULL}};
const Segment SGI155019[] = {{&S52093[27], 6}, {NULL}};
const Segment SGI155020[] = {{&S52093[97], 9}, {NULL}};
const Segment SGI155021[] = {{&S52093[70], 6}, {NULL}};
const Segment SGI155022[] = {{&S52094[34], 11}, {NULL}};
const Segment SGI155023[] = {{&S52094[27], 6}, {NULL}};
const Segment SGI155024[] = {{&S52094[98], 9}, {NULL}};
const Segment SGI155025[] = {{&S52094[71], 6}, {NULL}};
const Segment SGI155026[] = {{&S52095[34], 11}, {NULL}};
const Segment SGI155027[] = {{&S52095[27], 6}, {NULL}};
const Segment SGI155028[] = {{&S52095[98], 9}, {NULL}};
const Segment SGI155029[] = {{&S52095[71], 6}, {NULL}};
const Segment SGI155030[] = {{&S52096[34], 11}, {NULL}};
const Segment SGI155031[] = {{&S52096[27], 6}, {NULL}};
const Segment SGI155032[] = {{&S52096[98], 9}, {NULL}};
const Segment SGI155033[] = {{&S52096[71], 6}, {NULL}};
const Segment SGI155034[] = {{&S52097[34], 18}, {NULL}};
const Segment SGI155035[] = {{&S52097[27], 6}, {NULL}};
const Segment SGI155036[] = {{&S52097[105], 9}, {NULL}};
const Segment SGI155037[] = {{&S52097[78], 6}, {NULL}};
const Segment SGI155038[] = {{&S52098[34], 11}, {NULL}};
const Segment SGI155039[] = {{&S52098[27], 6}, {NULL}};
const Segment SGI155040[] = {{&S52098[98], 9}, {NULL}};
const Segment SGI155041[] = {{&S52098[71], 6}, {NULL}};
const Segment SGI155042[] = {{&S52103[34], 11}, {NULL}};
const Segment SGI155043[] = {{&S52103[27], 6}, {NULL}};
const Segment SGI155044[] = {{&S52103[98], 9}, {NULL}};
const Segment SGI155045[] = {{&S52103[71], 6}, {NULL}};
const Segment SGI155046[] = {{&S52105[34], 11}, {NULL}};
const Segment SGI155047[] = {{&S52105[27], 6}, {NULL}};
const Segment SGI155048[] = {{&S52105[98], 9}, {NULL}};
const Segment SGI155049[] = {{&S52105[71], 6}, {NULL}};
const Segment SGI155050[] = {{&S42984[34], 11}, {NULL}};
const Segment SGI155051[] = {{&S42984[27], 6}, {NULL}};
const Segment SGI155052[] = {{&S42984[98], 9}, {NULL}};
const Segment SGI155053[] = {{&S42984[71], 6}, {NULL}};
const Segment SGI155054[] = {{&S52112[34], 13}, {NULL}};
const Segment SGI155055[] = {{&S52112[27], 6}, {NULL}};
const Segment SGI155056[] = {{&S52112[100], 9}, {NULL}};
const Segment SGI155057[] = {{&S52112[73], 6}, {NULL}};
const Segment SGI155058[] = {{&S42985[34], 11}, {NULL}};
const Segment SGI155059[] = {{&S42985[27], 6}, {NULL}};
const Segment SGI155060[] = {{&S42356[44], 9}, {NULL}};
const Segment SGI155061[] = {{&S42356[44], 21}, {NULL}};
const Segment SGI155062[] = {{&S42356[30], 6}, {NULL}};
const Segment SGI155063[] = {{&S42985[98], 9}, {NULL}};
const Segment SGI155064[] = {{&S42985[71], 6}, {NULL}};
const Segment SGI155065[] = {{&S42356[114], 3}, {NULL}};
const Segment SGI155066[] = {{&S42356[79], 6}, {NULL}};
const Segment SGI155067[] = {{&S42357[44], 14}, {NULL}};
const Segment SGI155068[] = {{&S42357[44], 26}, {NULL}};
const Segment SGI155069[] = {{&S42357[30], 6}, {NULL}};
const Segment SGI155070[] = {{&S42357[119], 3}, {NULL}};
const Segment SGI155071[] = {{&S42987[34], 27}, {NULL}};
const Segment SGI155072[] = {{&S42357[84], 6}, {NULL}};
const Segment SGI155073[] = {{&S42987[27], 6}, {NULL}};
const Segment SGI155074[] = {{&S42987[114], 9}, {NULL}};
const Segment SGI155075[] = {{&S42987[87], 6}, {NULL}};
const Segment SGI155076[] = {{&S42358[38], 9}, {NULL}};
const Segment SGI155077[] = {{&S42358[31], 6}, {NULL}};
const Segment SGI155078[] = {{&S42359[44], 17}, {NULL}};
const Segment SGI155079[] = {{&S42359[44], 29}, {NULL}};
const Segment SGI155080[] = {{&S42359[30], 6}, {NULL}};
const Segment SGI155081[] = {{&S42359[122], 3}, {NULL}};
const Segment SGI155082[] = {{&S42359[87], 6}, {NULL}};
const Segment SGI155083[] = {{&S42992[34], 23}, {NULL}};
const Segment SGI155084[] = {{&S42992[27], 6}, {NULL}};
const Segment SGI155085[] = {{&S42992[110], 9}, {NULL}};
const Segment SGI155086[] = {{&S42992[83], 6}, {NULL}};
const Segment SGI155087[] = {{&S42993[34], 18}, {NULL}};
const Segment SGI155088[] = {{&S42993[27], 6}, {NULL}};
const Segment SGI155089[] = {{&S42993[105], 9}, {NULL}};
const Segment SGI155090[] = {{&S42993[78], 6}, {NULL}};
const Segment SGI155091[] = {{&S42994[34], 6}, {NULL}};
const Segment SGI155092[] = {{&S42994[27], 6}, {NULL}};
const Segment SGI155093[] = {{&S42994[98], 9}, {NULL}};
const Segment SGI155094[] = {{&S42994[71], 6}, {NULL}};
const Segment SGI155095[] = {{&S42995[34], 21}, {NULL}};
const Segment SGI155096[] = {{&S42995[27], 6}, {NULL}};
const Segment SGI155097[] = {{&S42995[121], 9}, {NULL}};
const Segment SGI155098[] = {{&S42995[94], 6}, {NULL}};
const Segment SGI155099[] = {{&S42996[34], 12}, {NULL}};
const Segment SGI155100[] = {{&S42996[27], 6}, {NULL}};
const Segment SGI155101[] = {{&S42996[99], 9}, {NULL}};
const Segment SGI155102[] = {{&S42996[72], 6}, {NULL}};
const Segment SGI155103[] = {{&S42999[34], 13}, {NULL}};
const Segment SGI155104[] = {{&S42999[27], 6}, {NULL}};
const Segment SGI155105[] = {{&S42374[44], 9}, {NULL}};
const Segment SGI155106[] = {{&S42374[44], 21}, {NULL}};
const Segment SGI155107[] = {{&S42374[30], 6}, {NULL}};
const Segment SGI155108[] = {{&S42999[100], 9}, {NULL}};
const Segment SGI155109[] = {{&S42374[114], 3}, {NULL}};
const Segment SGI155110[] = {{&S42999[73], 6}, {NULL}};
const Segment SGI155111[] = {{&S42374[79], 6}, {NULL}};
const Segment SGI155112[] = {{&S42375[38], 9}, {NULL}};
const Segment SGI155113[] = {{&S42375[31], 6}, {NULL}};
const Segment SGI155114[] = {{&S42376[44], 9}, {NULL}};
const Segment SGI155115[] = {{&S42376[44], 21}, {NULL}};
const Segment SGI155116[] = {{&S42376[30], 6}, {NULL}};
const Segment SGI155117[] = {{&S42376[114], 3}, {NULL}};
const Segment SGI155118[] = {{&S42376[79], 6}, {NULL}};
const Segment SGI155119[] = {{&S43002[34], 13}, {NULL}};
const Segment SGI155120[] = {{&S43002[27], 6}, {NULL}};
const Segment SGI155121[] = {{&S43002[100], 9}, {NULL}};
const Segment SGI155122[] = {{&S43002[73], 6}, {NULL}};
const Segment SGI155123[] = {{&S52145[34], 9}, {NULL}};
const Segment SGI155124[] = {{&S52145[27], 6}, {NULL}};
const Segment SGI155125[] = {{&S52145[96], 9}, {NULL}};
const Segment SGI155126[] = {{&S52145[69], 6}, {NULL}};
const Segment SGI155127[] = {{&S52146[34], 9}, {NULL}};
const Segment SGI155128[] = {{&S52146[27], 6}, {NULL}};
const Segment SGI155129[] = {{&S43005[34], 8}, {NULL}};
const Segment SGI155130[] = {{&S43005[27], 6}, {NULL}};
const Segment SGI155131[] = {{&S52146[96], 9}, {NULL}};
const Segment SGI155132[] = {{&S52146[69], 6}, {NULL}};
const Segment SGI155133[] = {{&S43005[95], 9}, {NULL}};
const Segment SGI155134[] = {{&S43005[68], 6}, {NULL}};
const Segment SGI155135[] = {{&S52147[34], 9}, {NULL}};
const Segment SGI155136[] = {{&S52147[27], 6}, {NULL}};
const Segment SGI155137[] = {{&S43006[34], 13}, {NULL}};
const Segment SGI155138[] = {{&S43006[27], 6}, {NULL}};
const Segment SGI155139[] = {{&S52147[96], 9}, {NULL}};
const Segment SGI155140[] = {{&S52147[69], 6}, {NULL}};
const Segment SGI155141[] = {{&S43006[100], 9}, {NULL}};
const Segment SGI155142[] = {{&S43006[73], 6}, {NULL}};
const Segment SGI155143[] = {{&S43007[34], 7}, {NULL}};
const Segment SGI155144[] = {{&S43007[27], 6}, {NULL}};
const Segment SGI155145[] = {{&S43007[94], 9}, {NULL}};
const Segment SGI155146[] = {{&S43007[67], 6}, {NULL}};
const Segment SGI155147[] = {{&S43009[34], 11}, {NULL}};
const Segment SGI155148[] = {{&S43009[27], 6}, {NULL}};
const Segment SGI155149[] = {{&S43009[98], 9}, {NULL}};
const Segment SGI155150[] = {{&S43009[71], 6}, {NULL}};
const Segment SGI155151[] = {{&S43011[34], 12}, {NULL}};
const Segment SGI155152[] = {{&S43011[27], 6}, {NULL}};
const Segment SGI155153[] = {{&S43011[99], 9}, {NULL}};
const Segment SGI155154[] = {{&S43011[72], 6}, {NULL}};
const Segment SGI155155[] = {{&S43012[34], 11}, {NULL}};
const Segment SGI155156[] = {{&S43012[27], 6}, {NULL}};
const Segment SGI155157[] = {{&S43012[98], 9}, {NULL}};
const Segment SGI155158[] = {{&S43012[71], 6}, {NULL}};
const Segment SGI155159[] = {{&S52159[44], 9}, {NULL}};
const Segment SGI155160[] = {{&S52159[44], 21}, {NULL}};
const Segment SGI155161[] = {{&S52159[30], 6}, {NULL}};
const Segment SGI155162[] = {{&S52159[31], 3}, {NULL}};
const Segment SGI155163[] = {{&S52159[79], 6}, {NULL}};
const Segment SGI155164[] = {{&S52160[38], 9}, {NULL}};
const Segment SGI155165[] = {{&S43014[34], 11}, {NULL}};
const Segment SGI155166[] = {{&S52160[31], 6}, {NULL}};
const Segment SGI155167[] = {{&S43014[27], 6}, {NULL}};
const Segment SGI155168[] = {{&S43014[98], 9}, {NULL}};
const Segment SGI155169[] = {{&S43014[71], 6}, {NULL}};
const Segment SGI155170[] = {{&S52161[44], 9}, {NULL}};
const Segment SGI155171[] = {{&S52161[44], 21}, {NULL}};
const Segment SGI155172[] = {{&S52161[30], 6}, {NULL}};
const Segment SGI155173[] = {{&S43015[34], 4}, {NULL}};
const Segment SGI155174[] = {{&S52161[124], 3}, {NULL}};
const Segment SGI155175[] = {{&S43015[27], 6}, {NULL}};
const Segment SGI155176[] = {{&S52161[89], 6}, {NULL}};
const Segment SGI155177[] = {{&S52161[144], 8}, {NULL}};
const Segment SGI155178[] = {{&S43015[91], 9}, {NULL}};
const Segment SGI155179[] = {{&S43015[64], 6}, {NULL}};
const Segment SGI155180[] = {{&S43016[34], 10}, {NULL}};
const Segment SGI155181[] = {{&S43016[27], 6}, {NULL}};
const Segment SGI155182[] = {{&S43016[97], 9}, {NULL}};
const Segment SGI155183[] = {{&S43016[70], 6}, {NULL}};
const Segment SGI155184[] = {{&S43017[34], 11}, {NULL}};
const Segment SGI155185[] = {{&S43017[27], 6}, {NULL}};
const Segment SGI155186[] = {{&S43017[98], 9}, {NULL}};
const Segment SGI155187[] = {{&S43017[71], 6}, {NULL}};
const Segment SGI155188[] = {{&S52166[44], 12}, {NULL}};
const Segment SGI155189[] = {{&S52166[44], 24}, {NULL}};
const Segment SGI155190[] = {{&S52166[30], 6}, {NULL}};
const Segment SGI155191[] = {{&S52166[31], 3}, {NULL}};
const Segment SGI155192[] = {{&S52166[82], 6}, {NULL}};
const Segment SGI155193[] = {{&S43018[27], 6}, {NULL}};
const Segment SGI155194[] = {{&S43018[102], 9}, {NULL}};
const Segment SGI155195[] = {{&S43018[75], 6}, {NULL}};
const Segment SGI155196[] = {{&S52167[44], 12}, {NULL}};
const Segment SGI155197[] = {{&S52167[44], 34}, {NULL}};
const Segment SGI155198[] = {{&S52167[30], 6}, {NULL}};
const Segment SGI155199[] = {{&S52167[31], 3}, {NULL}};
const Segment SGI155200[] = {{&S52167[92], 6}, {NULL}};
const Segment SGI155201[] = {{&S43019[34], 8}, {NULL}};
const Segment SGI155202[] = {{&S43019[27], 6}, {NULL}};
const Segment SGI155203[] = {{&S43019[95], 9}, {NULL}};
const Segment SGI155204[] = {{&S43019[68], 6}, {NULL}};
const Segment SGI155205[] = {{&S52168[44], 12}, {NULL}};
const Segment SGI155206[] = {{&S52168[44], 34}, {NULL}};
const Segment SGI155207[] = {{&S52168[30], 6}, {NULL}};
const Segment SGI155208[] = {{&S52168[31], 3}, {NULL}};
const Segment SGI155209[] = {{&S52168[92], 6}, {NULL}};
const Segment SGI155210[] = {{&S43020[34], 10}, {NULL}};
const Segment SGI155211[] = {{&S43020[27], 6}, {NULL}};
const Segment SGI155212[] = {{&S43020[97], 9}, {NULL}};
const Segment SGI155213[] = {{&S43020[70], 6}, {NULL}};
const Segment SGI155214[] = {{&S52169[44], 12}, {NULL}};
const Segment SGI155215[] = {{&S52169[44], 34}, {NULL}};
const Segment SGI155216[] = {{&S52169[30], 6}, {NULL}};
const Segment SGI155217[] = {{&S52169[31], 3}, {NULL}};
const Segment SGI155218[] = {{&S52169[92], 6}, {NULL}};
const Segment SGI155219[] = {{&S43021[34], 13}, {NULL}};
const Segment SGI155220[] = {{&S43021[27], 6}, {NULL}};
const Segment SGI155221[] = {{&S43021[100], 9}, {NULL}};
const Segment SGI155222[] = {{&S43021[73], 6}, {NULL}};
const Segment SGI155223[] = {{&S52170[44], 12}, {NULL}};
const Segment SGI155224[] = {{&S52170[44], 34}, {NULL}};
const Segment SGI155225[] = {{&S52170[30], 6}, {NULL}};
const Segment SGI155226[] = {{&S52170[31], 3}, {NULL}};
const Segment SGI155227[] = {{&S52170[92], 6}, {NULL}};
const Segment SGI155228[] = {{&S43022[34], 26}, {NULL}};
const Segment SGI155229[] = {{&S43022[27], 6}, {NULL}};
const Segment SGI155230[] = {{&S52171[44], 12}, {NULL}};
const Segment SGI155231[] = {{&S43022[113], 9}, {NULL}};
const Segment SGI155232[] = {{&S43022[86], 6}, {NULL}};
const Segment SGI155233[] = {{&S52171[44], 34}, {NULL}};
const Segment SGI155234[] = {{&S52171[30], 6}, {NULL}};
const Segment SGI155235[] = {{&S52171[31], 3}, {NULL}};
const Segment SGI155236[] = {{&S52171[92], 6}, {NULL}};
const Segment SGI155237[] = {{&S43023[34], 10}, {NULL}};
const Segment SGI155238[] = {{&S43023[27], 6}, {NULL}};
const Segment SGI155239[] = {{&S43023[92], 9}, {NULL}};
const Segment SGI155240[] = {{&S52172[44], 12}, {NULL}};
const Segment SGI155241[] = {{&S43023[65], 6}, {NULL}};
const Segment SGI155242[] = {{&S52172[44], 34}, {NULL}};
const Segment SGI155243[] = {{&S52172[30], 6}, {NULL}};
const Segment SGI155244[] = {{&S52172[31], 3}, {NULL}};
const Segment SGI155245[] = {{&S52172[92], 6}, {NULL}};
const Segment SGI155246[] = {{&S43024[34], 9}, {NULL}};
const Segment SGI155247[] = {{&S43024[27], 6}, {NULL}};
const Segment SGI155248[] = {{&S52173[44], 12}, {NULL}};
const Segment SGI155249[] = {{&S43024[91], 9}, {NULL}};
const Segment SGI155250[] = {{&S52173[44], 34}, {NULL}};
const Segment SGI155251[] = {{&S43024[64], 6}, {NULL}};
const Segment SGI155252[] = {{&S52173[30], 6}, {NULL}};
const Segment SGI155253[] = {{&S52173[31], 3}, {NULL}};
const Segment SGI155254[] = {{&S52173[92], 6}, {NULL}};
const Segment SGI155255[] = {{&S52174[44], 12}, {NULL}};
const Segment SGI155256[] = {{&S52174[44], 34}, {NULL}};
const Segment SGI155257[] = {{&S52174[30], 6}, {NULL}};
const Segment SGI155258[] = {{&S43026[34], 7}, {NULL}};
const Segment SGI155259[] = {{&S52174[31], 3}, {NULL}};
const Segment SGI155260[] = {{&S43026[27], 6}, {NULL}};
const Segment SGI155261[] = {{&S52174[92], 6}, {NULL}};
const Segment SGI155262[] = {{&S43026[94], 9}, {NULL}};
const Segment SGI155263[] = {{&S43026[67], 6}, {NULL}};
const Segment SGI155264[] = {{&S52175[44], 12}, {NULL}};
const Segment SGI155265[] = {{&S52175[44], 34}, {NULL}};
const Segment SGI155266[] = {{&S52175[30], 6}, {NULL}};
const Segment SGI155267[] = {{&S52175[31], 3}, {NULL}};
const Segment SGI155268[] = {{&S43027[34], 8}, {NULL}};
const Segment SGI155269[] = {{&S52175[92], 6}, {NULL}};
const Segment SGI155270[] = {{&S43027[27], 6}, {NULL}};
const Segment SGI155271[] = {{&S43027[95], 9}, {NULL}};
const Segment SGI155272[] = {{&S43027[68], 6}, {NULL}};
const Segment SGI155273[] = {{&S52176[44], 12}, {NULL}};
const Segment SGI155274[] = {{&S52176[44], 31}, {NULL}};
const Segment SGI155275[] = {{&S52176[30], 6}, {NULL}};
const Segment SGI155276[] = {{&S43028[34], 12}, {NULL}};
const Segment SGI155277[] = {{&S52176[31], 3}, {NULL}};
const Segment SGI155278[] = {{&S43028[27], 6}, {NULL}};
const Segment SGI155279[] = {{&S52176[89], 6}, {NULL}};
const Segment SGI155280[] = {{&S43028[99], 9}, {NULL}};
const Segment SGI155281[] = {{&S43028[72], 6}, {NULL}};
const Segment SGI155282[] = {{&S52177[44], 12}, {NULL}};
const Segment SGI155283[] = {{&S52177[44], 31}, {NULL}};
const Segment SGI155284[] = {{&S52177[30], 6}, {NULL}};
const Segment SGI155285[] = {{&S42438[34], 10}, {NULL}};
const Segment SGI155286[] = {{&S42438[27], 6}, {NULL}};
const Segment SGI155287[] = {{&S43029[34], 8}, {NULL}};
const Segment SGI155288[] = {{&S52177[31], 3}, {NULL}};
const Segment SGI155289[] = {{&S43029[27], 6}, {NULL}};
const Segment SGI155290[] = {{&S52177[89], 6}, {NULL}};
const Segment SGI155291[] = {{&S42438[97], 9}, {NULL}};
const Segment SGI155292[] = {{&S42438[70], 6}, {NULL}};
const Segment SGI155293[] = {{&S43029[95], 9}, {NULL}};
const Segment SGI155294[] = {{&S43029[68], 6}, {NULL}};
const Segment SGI155295[] = {{&S42439[34], 10}, {NULL}};
const Segment SGI155296[] = {{&S52178[44], 12}, {NULL}};
const Segment SGI155297[] = {{&S42439[27], 6}, {NULL}};
const Segment SGI155298[] = {{&S52178[44], 35}, {NULL}};
const Segment SGI155299[] = {{&S52178[30], 6}, {NULL}};
const Segment SGI155300[] = {{&S43030[34], 11}, {NULL}};
const Segment SGI155301[] = {{&S43030[27], 6}, {NULL}};
const Segment SGI155302[] = {{&S42439[97], 9}, {NULL}};
const Segment SGI155303[] = {{&S52178[31], 3}, {NULL}};
const Segment SGI155304[] = {{&S42439[70], 6}, {NULL}};
const Segment SGI155305[] = {{&S52178[93], 6}, {NULL}};
const Segment SGI155306[] = {{&S43030[98], 9}, {NULL}};
const Segment SGI155307[] = {{&S43030[71], 6}, {NULL}};
const Segment SGI155308[] = {{&S42440[34], 10}, {NULL}};
const Segment SGI155309[] = {{&S42440[27], 6}, {NULL}};
const Segment SGI155310[] = {{&S52179[44], 12}, {NULL}};
const Segment SGI155311[] = {{&S52179[44], 34}, {NULL}};
const Segment SGI155312[] = {{&S52179[30], 6}, {NULL}};
const Segment SGI155313[] = {{&S42440[97], 9}, {NULL}};
const Segment SGI155314[] = {{&S42440[70], 6}, {NULL}};
const Segment SGI155315[] = {{&S43031[34], 9}, {NULL}};
const Segment SGI155316[] = {{&S52179[31], 3}, {NULL}};
const Segment SGI155317[] = {{&S43031[27], 6}, {NULL}};
const Segment SGI155318[] = {{&S52179[92], 6}, {NULL}};
const Segment SGI155319[] = {{&S43031[91], 9}, {NULL}};
const Segment SGI155320[] = {{&S43031[64], 6}, {NULL}};
const Segment SGI155321[] = {{&S52180[44], 12}, {NULL}};
const Segment SGI155322[] = {{&S52180[44], 34}, {NULL}};
const Segment SGI155323[] = {{&S52180[30], 6}, {NULL}};
const Segment SGI155324[] = {{&S43032[34], 10}, {NULL}};
const Segment SGI155325[] = {{&S52180[31], 3}, {NULL}};
const Segment SGI155326[] = {{&S43032[27], 6}, {NULL}};
const Segment SGI155327[] = {{&S52180[92], 6}, {NULL}};
const Segment SGI155328[] = {{&S43032[97], 9}, {NULL}};
const Segment SGI155329[] = {{&S43032[70], 6}, {NULL}};
const Segment SGI155330[] = {{&S52181[44], 12}, {NULL}};
const Segment SGI155331[] = {{&S52181[44], 34}, {NULL}};
const Segment SGI155332[] = {{&S52181[30], 6}, {NULL}};
const Segment SGI155333[] = {{&S43033[34], 6}, {NULL}};
const Segment SGI155334[] = {{&S52181[31], 3}, {NULL}};
const Segment SGI155335[] = {{&S43033[27], 6}, {NULL}};
const Segment SGI155336[] = {{&S52181[92], 6}, {NULL}};
const Segment SGI155337[] = {{&S43033[93], 9}, {NULL}};
const Segment SGI155338[] = {{&S43033[66], 6}, {NULL}};
const Segment SGI155339[] = {{&S52182[44], 12}, {NULL}};
const Segment SGI155340[] = {{&S52182[44], 34}, {NULL}};
const Segment SGI155341[] = {{&S52182[30], 6}, {NULL}};
const Segment SGI155342[] = {{&S43034[34], 11}, {NULL}};
const Segment SGI155343[] = {{&S43034[27], 6}, {NULL}};
const Segment SGI155344[] = {{&S52182[31], 3}, {NULL}};
const Segment SGI155345[] = {{&S52182[92], 6}, {NULL}};
const Segment SGI155346[] = {{&S43034[98], 9}, {NULL}};
const Segment SGI155347[] = {{&S43034[71], 6}, {NULL}};
const Segment SGI155348[] = {{&S52183[44], 12}, {NULL}};
const Segment SGI155349[] = {{&S52183[44], 34}, {NULL}};
const Segment SGI155350[] = {{&S52183[30], 6}, {NULL}};
const Segment SGI155351[] = {{&S43035[34], 21}, {NULL}};
const Segment SGI155352[] = {{&S43035[27], 6}, {NULL}};
const Segment SGI155353[] = {{&S52183[31], 3}, {NULL}};
const Segment SGI155354[] = {{&S52183[92], 6}, {NULL}};
const Segment SGI155355[] = {{&S43035[108], 9}, {NULL}};
const Segment SGI155356[] = {{&S43035[81], 6}, {NULL}};
const Segment SGI155357[] = {{&S52184[44], 12}, {NULL}};
const Segment SGI155358[] = {{&S52184[44], 34}, {NULL}};
const Segment SGI155359[] = {{&S52184[30], 6}, {NULL}};
const Segment SGI155360[] = {{&S43036[34], 7}, {NULL}};
const Segment SGI155361[] = {{&S43036[27], 6}, {NULL}};
const Segment SGI155362[] = {{&S52184[31], 3}, {NULL}};
const Segment SGI155363[] = {{&S52184[92], 6}, {NULL}};
const Segment SGI155364[] = {{&S43036[94], 9}, {NULL}};
const Segment SGI155365[] = {{&S43036[67], 6}, {NULL}};
const Segment SGI155366[] = {{&S52185[44], 12}, {NULL}};
const Segment SGI155367[] = {{&S52185[44], 31}, {NULL}};
const Segment SGI155368[] = {{&S52185[30], 6}, {NULL}};
const Segment SGI155369[] = {{&S52185[31], 3}, {NULL}};
const Segment SGI155370[] = {{&S52185[89], 6}, {NULL}};
const Segment SGI155371[] = {{&S43038[34], 8}, {NULL}};
const Segment SGI155372[] = {{&S43038[27], 6}, {NULL}};
const Segment SGI155373[] = {{&S43038[95], 9}, {NULL}};
const Segment SGI155374[] = {{&S43038[68], 6}, {NULL}};
const Segment SGI155375[] = {{&S52186[44], 12}, {NULL}};
const Segment SGI155376[] = {{&S52186[44], 34}, {NULL}};
const Segment SGI155377[] = {{&S52186[30], 6}, {NULL}};
const Segment SGI155378[] = {{&S52186[31], 3}, {NULL}};
const Segment SGI155379[] = {{&S52186[92], 6}, {NULL}};
const Segment SGI155380[] = {{&S43039[34], 13}, {NULL}};
const Segment SGI155381[] = {{&S43039[27], 6}, {NULL}};
const Segment SGI155382[] = {{&S43039[100], 9}, {NULL}};
const Segment SGI155383[] = {{&S43039[73], 6}, {NULL}};
const Segment SGI155384[] = {{&S52187[44], 12}, {NULL}};
const Segment SGI155385[] = {{&S52187[44], 35}, {NULL}};
const Segment SGI155386[] = {{&S52187[30], 6}, {NULL}};
const Segment SGI155387[] = {{&S52187[31], 3}, {NULL}};
const Segment SGI155388[] = {{&S52187[93], 6}, {NULL}};
const Segment SGI155389[] = {{&S43040[34], 14}, {NULL}};
const Segment SGI155390[] = {{&S43040[27], 6}, {NULL}};
const Segment SGI155391[] = {{&S43040[101], 9}, {NULL}};
const Segment SGI155392[] = {{&S52188[44], 12}, {NULL}};
const Segment SGI155393[] = {{&S43040[74], 6}, {NULL}};
const Segment SGI155394[] = {{&S52188[44], 34}, {NULL}};
const Segment SGI155395[] = {{&S52188[30], 6}, {NULL}};
const Segment SGI155396[] = {{&S52188[31], 3}, {NULL}};
const Segment SGI155397[] = {{&S52188[92], 6}, {NULL}};
const Segment SGI155398[] = {{&S43041[34], 15}, {NULL}};
const Segment SGI155399[] = {{&S43041[27], 6}, {NULL}};
const Segment SGI155400[] = {{&S52189[44], 12}, {NULL}};
const Segment SGI155401[] = {{&S43041[102], 9}, {NULL}};
const Segment SGI155402[] = {{&S52189[44], 35}, {NULL}};
const Segment SGI155403[] = {{&S43041[75], 6}, {NULL}};
const Segment SGI155404[] = {{&S52189[30], 6}, {NULL}};
const Segment SGI155405[] = {{&S52189[31], 3}, {NULL}};
const Segment SGI155406[] = {{&S52189[93], 6}, {NULL}};
const Segment SGI155407[] = {{&S52190[44], 12}, {NULL}};
const Segment SGI155408[] = {{&S52190[44], 34}, {NULL}};
const Segment SGI155409[] = {{&S52190[30], 6}, {NULL}};
const Segment SGI155410[] = {{&S52190[31], 3}, {NULL}};
const Segment SGI155411[] = {{&S52190[92], 6}, {NULL}};
const Segment SGI155412[] = {{&S43044[34], 7}, {NULL}};
const Segment SGI155413[] = {{&S43044[27], 6}, {NULL}};
const Segment SGI155414[] = {{&S43044[94], 9}, {NULL}};
const Segment SGI155415[] = {{&S52191[44], 12}, {NULL}};
const Segment SGI155416[] = {{&S43044[67], 6}, {NULL}};
const Segment SGI155417[] = {{&S52191[44], 31}, {NULL}};
const Segment SGI155418[] = {{&S52191[30], 6}, {NULL}};
const Segment SGI155419[] = {{&S52191[31], 3}, {NULL}};
const Segment SGI155420[] = {{&S52191[89], 6}, {NULL}};
const Segment SGI155421[] = {{&S43048[34], 6}, {NULL}};
const Segment SGI155422[] = {{&S43048[27], 6}, {NULL}};
const Segment SGI155423[] = {{&S43048[93], 9}, {NULL}};
const Segment SGI155424[] = {{&S43048[66], 6}, {NULL}};
const Segment SGI155425[] = {{&S43049[34], 11}, {NULL}};
const Segment SGI155426[] = {{&S43049[27], 6}, {NULL}};
const Segment SGI155427[] = {{&S43049[101], 9}, {NULL}};
const Segment SGI155428[] = {{&S43049[74], 6}, {NULL}};
const Segment SGI155429[] = {{&S43050[34], 10}, {NULL}};
const Segment SGI155430[] = {{&S43050[27], 6}, {NULL}};
const Segment SGI155431[] = {{&S43050[97], 9}, {NULL}};
const Segment SGI155432[] = {{&S43050[70], 6}, {NULL}};
const Segment SGI155433[] = {{&S43051[34], 8}, {NULL}};
const Segment SGI155434[] = {{&S43051[27], 6}, {NULL}};
const Segment SGI155435[] = {{&S43051[95], 9}, {NULL}};
const Segment SGI155436[] = {{&S43051[68], 6}, {NULL}};
const Segment SGI155437[] = {{&S43054[34], 25}, {NULL}};
const Segment SGI155438[] = {{&S43054[27], 6}, {NULL}};
const Segment SGI155439[] = {{&S43054[112], 9}, {NULL}};
const Segment SGI155440[] = {{&S43054[85], 6}, {NULL}};
const Segment SGI155441[] = {{&S43056[34], 7}, {NULL}};
const Segment SGI155442[] = {{&S43056[27], 6}, {NULL}};
const Segment SGI155443[] = {{&S43056[94], 9}, {NULL}};
const Segment SGI155444[] = {{&S43056[67], 6}, {NULL}};
const Segment SGI155445[] = {{&S43059[34], 12}, {NULL}};
const Segment SGI155446[] = {{&S43059[27], 6}, {NULL}};
const Segment SGI155447[] = {{&S43059[99], 9}, {NULL}};
const Segment SGI155448[] = {{&S43059[72], 6}, {NULL}};
const Segment SGI155449[] = {{&S43061[34], 14}, {NULL}};
const Segment SGI155450[] = {{&S43061[27], 6}, {NULL}};
const Segment SGI155451[] = {{&S43061[101], 9}, {NULL}};
const Segment SGI155452[] = {{&S43061[74], 6}, {NULL}};
const Segment SGI155453[] = {{&S43064[34], 12}, {NULL}};
const Segment SGI155454[] = {{&S43064[27], 6}, {NULL}};
const Segment SGI155455[] = {{&S43064[101], 9}, {NULL}};
const Segment SGI155456[] = {{&S43064[74], 6}, {NULL}};
const Segment SGI155457[] = {{&S43065[34], 8}, {NULL}};
const Segment SGI155458[] = {{&S43065[27], 6}, {NULL}};
const Segment SGI155459[] = {{&S43065[95], 9}, {NULL}};
const Segment SGI155460[] = {{&S43065[68], 6}, {NULL}};
const Segment SGI155461[] = {{&S43067[34], 8}, {NULL}};
const Segment SGI155462[] = {{&S43067[27], 6}, {NULL}};
const Segment SGI155463[] = {{&S43067[95], 9}, {NULL}};
const Segment SGI155464[] = {{&S43067[68], 6}, {NULL}};
const Segment SGI155465[] = {{&S43068[34], 7}, {NULL}};
const Segment SGI155466[] = {{&S43068[27], 6}, {NULL}};
const Segment SGI155467[] = {{&S43068[94], 9}, {NULL}};
const Segment SGI155468[] = {{&S43068[67], 6}, {NULL}};
const Segment SGI155469[] = {{&S43069[34], 12}, {NULL}};
const Segment SGI155470[] = {{&S43069[27], 6}, {NULL}};
const Segment SGI155471[] = {{&S43069[99], 9}, {NULL}};
const Segment SGI155472[] = {{&S43069[72], 6}, {NULL}};
const Segment SGI155473[] = {{&S43070[34], 6}, {NULL}};
const Segment SGI155474[] = {{&S43070[27], 6}, {NULL}};
const Segment SGI155475[] = {{&S43070[93], 9}, {NULL}};
const Segment SGI155476[] = {{&S43070[66], 6}, {NULL}};
const Segment SGI155477[] = {{&S43071[34], 13}, {NULL}};
const Segment SGI155478[] = {{&S43071[27], 6}, {NULL}};
const Segment SGI155479[] = {{&S43071[100], 9}, {NULL}};
const Segment SGI155480[] = {{&S43071[73], 6}, {NULL}};
const Segment SGI155481[] = {{&S43072[34], 19}, {NULL}};
const Segment SGI155482[] = {{&S43072[27], 6}, {NULL}};
const Segment SGI155483[] = {{&S43072[106], 9}, {NULL}};
const Segment SGI155484[] = {{&S43072[79], 6}, {NULL}};
const Segment SGI155485[] = {{&S43073[34], 10}, {NULL}};
const Segment SGI155486[] = {{&S43073[27], 6}, {NULL}};
const Segment SGI155487[] = {{&S43073[97], 9}, {NULL}};
const Segment SGI155488[] = {{&S43073[70], 6}, {NULL}};
const Segment SGI155489[] = {{&S43074[34], 9}, {NULL}};
const Segment SGI155490[] = {{&S43074[27], 6}, {NULL}};
const Segment SGI155491[] = {{&S43074[91], 9}, {NULL}};
const Segment SGI155492[] = {{&S43074[64], 6}, {NULL}};
const Segment SGI155493[] = {{&S43078[34], 9}, {NULL}};
const Segment SGI155494[] = {{&S43078[27], 6}, {NULL}};
const Segment SGI155495[] = {{&S43078[96], 9}, {NULL}};
const Segment SGI155496[] = {{&S43078[69], 6}, {NULL}};
const Segment SGI155497[] = {{&S43082[34], 11}, {NULL}};
const Segment SGI155498[] = {{&S43082[27], 6}, {NULL}};
const Segment SGI155499[] = {{&S43082[98], 9}, {NULL}};
const Segment SGI155500[] = {{&S43082[71], 6}, {NULL}};
const Segment SGI155501[] = {{&S43083[34], 10}, {NULL}};
const Segment SGI155502[] = {{&S43083[27], 6}, {NULL}};
const Segment SGI155503[] = {{&S43083[97], 9}, {NULL}};
const Segment SGI155504[] = {{&S43083[70], 6}, {NULL}};
const Segment SGI155505[] = {{&S43087[34], 11}, {NULL}};
const Segment SGI155506[] = {{&S43087[27], 6}, {NULL}};
const Segment SGI155507[] = {{&S43087[98], 9}, {NULL}};
const Segment SGI155508[] = {{&S43087[71], 6}, {NULL}};
const Segment SGI155509[] = {{&S43090[34], 11}, {NULL}};
const Segment SGI155510[] = {{&S43090[27], 6}, {NULL}};
const Segment SGI155511[] = {{&S43090[98], 9}, {NULL}};
const Segment SGI155512[] = {{&S43090[71], 6}, {NULL}};
const Segment SGI155513[] = {{&S43092[34], 12}, {NULL}};
const Segment SGI155514[] = {{&S43092[27], 6}, {NULL}};
const Segment SGI155515[] = {{&S43092[99], 9}, {NULL}};
const Segment SGI155516[] = {{&S43092[72], 6}, {NULL}};
const Segment SGI155517[] = {{&S37848[44], 10}, {NULL}};
const Segment SGI155518[] = {{&S43093[34], 21}, {NULL}};
const Segment SGI155519[] = {{&S37848[44], 22}, {NULL}};
const Segment SGI155520[] = {{&S43093[27], 6}, {NULL}};
const Segment SGI155521[] = {{&S37848[30], 6}, {NULL}};
const Segment SGI155522[] = {{&S37848[133], 3}, {NULL}};
const Segment SGI155523[] = {{&S43093[108], 9}, {NULL}};
const Segment SGI155524[] = {{&S43093[81], 6}, {NULL}};
const Segment SGI155525[] = {{&S37848[98], 6}, {NULL}};
const Segment SGI155526[] = {{&S37848[68], 8}, {NULL}};
const Segment SGI155527[] = {{&S43094[34], 2}, {NULL}};
const Segment SGI155528[] = {{&S43094[27], 6}, {NULL}};
const Segment SGI155529[] = {{&S37849[44], 10}, {NULL}};
const Segment SGI155530[] = {{&S37849[44], 22}, {NULL}};
const Segment SGI155531[] = {{&S37849[30], 6}, {NULL}};
const Segment SGI155532[] = {{&S43094[89], 9}, {NULL}};
const Segment SGI155533[] = {{&S43094[62], 6}, {NULL}};
const Segment SGI155534[] = {{&S37849[133], 3}, {NULL}};
const Segment SGI155535[] = {{&S37849[98], 6}, {NULL}};
const Segment SGI155536[] = {{&S37849[68], 8}, {NULL}};
const Segment SGI155537[] = {{&S43095[34], 14}, {NULL}};
const Segment SGI155538[] = {{&S43095[27], 6}, {NULL}};
const Segment SGI155539[] = {{&S43095[101], 9}, {NULL}};
const Segment SGI155540[] = {{&S43095[74], 6}, {NULL}};
const Segment SGI155541[] = {{&S43096[34], 8}, {NULL}};
const Segment SGI155542[] = {{&S43096[27], 6}, {NULL}};
const Segment SGI155543[] = {{&S43096[101], 9}, {NULL}};
const Segment SGI155544[] = {{&S43096[74], 6}, {NULL}};
const Segment SGI155545[] = {{&S43099[34], 11}, {NULL}};
const Segment SGI155546[] = {{&S43099[27], 6}, {NULL}};
const Segment SGI155547[] = {{&S43099[98], 9}, {NULL}};
const Segment SGI155548[] = {{&S43099[71], 6}, {NULL}};
const Segment SGI155549[] = {{&S43101[34], 9}, {NULL}};
const Segment SGI155550[] = {{&S43101[27], 6}, {NULL}};
const Segment SGI155551[] = {{&S43101[96], 9}, {NULL}};
const Segment SGI155552[] = {{&S43101[69], 6}, {NULL}};
const Segment SGI155553[] = {{&S52285[44], 10}, {NULL}};
const Segment SGI155554[] = {{&S52285[44], 22}, {NULL}};
const Segment SGI155555[] = {{&S52285[30], 6}, {NULL}};
const Segment SGI155556[] = {{&S52285[31], 3}, {NULL}};
const Segment SGI155557[] = {{&S52285[80], 6}, {NULL}};
const Segment SGI155558[] = {{&S43103[38], 7}, {NULL}};
const Segment SGI155559[] = {{&S43103[31], 6}, {NULL}};
const Segment SGI155560[] = {{&S52288[44], 10}, {NULL}};
const Segment SGI155561[] = {{&S43105[44], 11}, {NULL}};
const Segment SGI155562[] = {{&S52288[44], 22}, {NULL}};
const Segment SGI155563[] = {{&S52288[30], 6}, {NULL}};
const Segment SGI155564[] = {{&S43105[44], 29}, {NULL}};
const Segment SGI155565[] = {{&S43105[30], 6}, {NULL}};
const Segment SGI155566[] = {{&S52288[31], 3}, {NULL}};
const Segment SGI155567[] = {{&S43105[122], 3}, {NULL}};
const Segment SGI155568[] = {{&S52288[80], 6}, {NULL}};
const Segment SGI155569[] = {{&S43105[87], 6}, {NULL}};
const Segment SGI155570[] = {{&S52289[44], 10}, {NULL}};
const Segment SGI155571[] = {{&S52289[44], 22}, {NULL}};
const Segment SGI155572[] = {{&S52289[30], 6}, {NULL}};
const Segment SGI155573[] = {{&S52289[31], 3}, {NULL}};
const Segment SGI155574[] = {{&S52289[80], 6}, {NULL}};
const Segment SGI155575[] = {{&S52290[44], 10}, {NULL}};
const Segment SGI155576[] = {{&S52290[44], 22}, {NULL}};
const Segment SGI155577[] = {{&S52290[30], 6}, {NULL}};
const Segment SGI155578[] = {{&S52290[31], 3}, {NULL}};
const Segment SGI155579[] = {{&S52290[80], 6}, {NULL}};
const Segment SGI155580[] = {{&S52291[44], 10}, {NULL}};
const Segment SGI155581[] = {{&S52291[44], 22}, {NULL}};
const Segment SGI155582[] = {{&S52291[30], 6}, {NULL}};
const Segment SGI155583[] = {{&S52291[31], 3}, {NULL}};
const Segment SGI155584[] = {{&S52291[80], 6}, {NULL}};
const Segment SGI155585[] = {{&S52292[44], 10}, {NULL}};
const Segment SGI155586[] = {{&S52292[44], 21}, {NULL}};
const Segment SGI155587[] = {{&S52292[30], 6}, {NULL}};
const Segment SGI155588[] = {{&S52292[31], 3}, {NULL}};
const Segment SGI155589[] = {{&S52292[79], 6}, {NULL}};
const Segment SGI155590[] = {{&S52298[44], 10}, {NULL}};
const Segment SGI155591[] = {{&S52298[44], 22}, {NULL}};
const Segment SGI155592[] = {{&S52298[30], 6}, {NULL}};
const Segment SGI155593[] = {{&S52298[31], 3}, {NULL}};
const Segment SGI155594[] = {{&S52298[80], 6}, {NULL}};
const Segment SGI155595[] = {{&S52299[44], 11}, {NULL}};
const Segment SGI155596[] = {{&S52299[44], 23}, {NULL}};
const Segment SGI155597[] = {{&S52299[30], 6}, {NULL}};
const Segment SGI155598[] = {{&S52299[31], 3}, {NULL}};
const Segment SGI155599[] = {{&S52299[81], 6}, {NULL}};
const Segment SGI155600[] = {{&S52300[38], 10}, {NULL}};
const Segment SGI155601[] = {{&S52300[31], 6}, {NULL}};
const Segment SGI155602[] = {{&S52306[44], 15}, {NULL}};
const Segment SGI155603[] = {{&S52306[44], 24}, {NULL}};
const Segment SGI155604[] = {{&S52306[30], 6}, {NULL}};
const Segment SGI155605[] = {{&S52306[31], 3}, {NULL}};
const Segment SGI155606[] = {{&S52306[82], 6}, {NULL}};
const Segment SGI155607[] = {{&S52307[38], 15}, {NULL}};
const Segment SGI155608[] = {{&S52307[31], 6}, {NULL}};
const Segment SGI155609[] = {{&S52308[44], 11}, {NULL}};
const Segment SGI155610[] = {{&S52308[44], 28}, {NULL}};
const Segment SGI155611[] = {{&S52308[30], 6}, {NULL}};
const Segment SGI155612[] = {{&S52308[31], 3}, {NULL}};
const Segment SGI155613[] = {{&S52308[86], 6}, {NULL}};
const Segment SGI155614[] = {{&S52309[44], 7}, {NULL}};
const Segment SGI155615[] = {{&S52309[44], 24}, {NULL}};
const Segment SGI155616[] = {{&S52309[30], 6}, {NULL}};
const Segment SGI155617[] = {{&S52309[31], 3}, {NULL}};
const Segment SGI155618[] = {{&S52309[82], 6}, {NULL}};
const Segment SGI155619[] = {{&S52310[44], 7}, {NULL}};
const Segment SGI155620[] = {{&S52310[44], 24}, {NULL}};
const Segment SGI155621[] = {{&S52310[30], 6}, {NULL}};
const Segment SGI155622[] = {{&S52310[31], 3}, {NULL}};
const Segment SGI155623[] = {{&S52310[82], 6}, {NULL}};
const Segment SGI155624[] = {{&S52311[38], 7}, {NULL}};
const Segment SGI155625[] = {{&S52311[31], 6}, {NULL}};
const Segment SGI155626[] = {{&S42525[34], 10}, {NULL}};
const Segment SGI155627[] = {{&S42525[27], 6}, {NULL}};
const Segment SGI155628[] = {{&S42525[97], 9}, {NULL}};
const Segment SGI155629[] = {{&S42525[70], 6}, {NULL}};
const Segment SGI155630[] = {{&S42526[34], 11}, {NULL}};
const Segment SGI155631[] = {{&S42526[27], 6}, {NULL}};
const Segment SGI155632[] = {{&S42526[98], 9}, {NULL}};
const Segment SGI155633[] = {{&S42526[71], 6}, {NULL}};
const Segment SGI155634[] = {{&S42527[34], 11}, {NULL}};
const Segment SGI155635[] = {{&S42527[27], 6}, {NULL}};
const Segment SGI155636[] = {{&S42527[98], 9}, {NULL}};
const Segment SGI155637[] = {{&S42527[71], 6}, {NULL}};
const Segment SGI155638[] = {{&S42528[34], 11}, {NULL}};
const Segment SGI155639[] = {{&S42528[27], 6}, {NULL}};
const Segment SGI155640[] = {{&S42528[98], 9}, {NULL}};
const Segment SGI155641[] = {{&S42528[71], 6}, {NULL}};
const Segment SGI155642[] = {{&S52343[34], 15}, {NULL}};
const Segment SGI155643[] = {{&S52343[27], 6}, {NULL}};
const Segment SGI155644[] = {{&S52343[102], 9}, {NULL}};
const Segment SGI155645[] = {{&S52343[75], 6}, {NULL}};
const Segment SGI155646[] = {{&S52353[44], 9}, {NULL}};
const Segment SGI155647[] = {{&S52353[44], 21}, {NULL}};
const Segment SGI155648[] = {{&S52353[30], 6}, {NULL}};
const Segment SGI155649[] = {{&S52353[31], 3}, {NULL}};
const Segment SGI155650[] = {{&S52353[79], 6}, {NULL}};
const Segment SGI155651[] = {{&S52355[44], 9}, {NULL}};
const Segment SGI155652[] = {{&S52355[44], 21}, {NULL}};
const Segment SGI155653[] = {{&S52355[30], 6}, {NULL}};
const Segment SGI155654[] = {{&S52355[31], 3}, {NULL}};
const Segment SGI155655[] = {{&S52355[79], 6}, {NULL}};
const Segment SGI155656[] = {{&S52356[44], 9}, {NULL}};
const Segment SGI155657[] = {{&S52356[44], 21}, {NULL}};
const Segment SGI155658[] = {{&S52356[30], 6}, {NULL}};
const Segment SGI155659[] = {{&S52356[31], 3}, {NULL}};
const Segment SGI155660[] = {{&S52356[79], 6}, {NULL}};
const Segment SGI155661[] = {{&S52358[38], 9}, {NULL}};
const Segment SGI155662[] = {{&S52358[31], 6}, {NULL}};
const Segment SGI155663[] = {{&S52359[38], 10}, {NULL}};
const Segment SGI155664[] = {{&S52359[31], 6}, {NULL}};
const Segment SGI155665[] = {{&S52360[44], 16}, {NULL}};
const Segment SGI155666[] = {{&S52360[44], 28}, {NULL}};
const Segment SGI155667[] = {{&S52360[30], 6}, {NULL}};
const Segment SGI155668[] = {{&S52360[31], 3}, {NULL}};
const Segment SGI155669[] = {{&S52360[86], 6}, {NULL}};
const Segment SGI155670[] = {{&S52364[46], 9}, {NULL}};
const Segment SGI155671[] = {{&S52364[46], 21}, {NULL}};
const Segment SGI155672[] = {{&S52364[30], 8}, {NULL}};
const Segment SGI155673[] = {{&S52364[31], 3}, {NULL}};
const Segment SGI155674[] = {{&S52364[81], 6}, {NULL}};
const Segment SGI155675[] = {{&S42571[44], 10}, {NULL}};
const Segment SGI155676[] = {{&S42571[44], 22}, {NULL}};
const Segment SGI155677[] = {{&S42571[30], 6}, {NULL}};
const Segment SGI155678[] = {{&S42571[135], 3}, {NULL}};
const Segment SGI155679[] = {{&S42571[100], 6}, {NULL}};
const Segment SGI155680[] = {{&S42571[68], 8}, {NULL}};
const Segment SGI155681[] = {{&S42572[44], 10}, {NULL}};
const Segment SGI155682[] = {{&S42572[44], 22}, {NULL}};
const Segment SGI155683[] = {{&S42572[30], 6}, {NULL}};
const Segment SGI155684[] = {{&S42572[131], 3}, {NULL}};
const Segment SGI155685[] = {{&S42572[96], 6}, {NULL}};
const Segment SGI155686[] = {{&S42572[68], 8}, {NULL}};
const Segment SGI155687[] = {{&S42573[44], 10}, {NULL}};
const Segment SGI155688[] = {{&S42573[44], 22}, {NULL}};
const Segment SGI155689[] = {{&S42573[30], 6}, {NULL}};
const Segment SGI155690[] = {{&S42573[135], 3}, {NULL}};
const Segment SGI155691[] = {{&S42573[100], 6}, {NULL}};
const Segment SGI155692[] = {{&S42573[68], 8}, {NULL}};
const Segment SGI155693[] = {{&S42574[44], 18}, {NULL}};
const Segment SGI155694[] = {{&S42574[44], 30}, {NULL}};
const Segment SGI155695[] = {{&S42574[30], 6}, {NULL}};
const Segment SGI155696[] = {{&S42574[143], 3}, {NULL}};
const Segment SGI155697[] = {{&S42574[108], 6}, {NULL}};
const Segment SGI155698[] = {{&S42574[76], 8}, {NULL}};
const Segment SGI155699[] = {{&S42575[44], 8}, {NULL}};
const Segment SGI155700[] = {{&S42575[44], 20}, {NULL}};
const Segment SGI155701[] = {{&S42575[30], 6}, {NULL}};
const Segment SGI155702[] = {{&S42575[129], 3}, {NULL}};
const Segment SGI155703[] = {{&S42575[94], 6}, {NULL}};
const Segment SGI155704[] = {{&S42575[66], 8}, {NULL}};
const Segment SGI155705[] = {{&S52393[44], 14}, {NULL}};
const Segment SGI155706[] = {{&S52393[44], 31}, {NULL}};
const Segment SGI155707[] = {{&S52393[30], 6}, {NULL}};
const Segment SGI155708[] = {{&S52393[92], 6}, {NULL}};
const Segment SGI155709[] = {{&S52394[44], 14}, {NULL}};
const Segment SGI155710[] = {{&S52394[44], 31}, {NULL}};
const Segment SGI155711[] = {{&S52394[30], 6}, {NULL}};
const Segment SGI155712[] = {{&S52394[31], 3}, {NULL}};
const Segment SGI155713[] = {{&S52394[89], 6}, {NULL}};
const Segment SGI155714[] = {{&S52395[38], 14}, {NULL}};
const Segment SGI155715[] = {{&S52395[31], 6}, {NULL}};
const Segment SGI155716[] = {{&S43244[34], 11}, {NULL}};
const Segment SGI155717[] = {{&S43244[27], 6}, {NULL}};
const Segment SGI155718[] = {{&S43244[98], 9}, {NULL}};
const Segment SGI155719[] = {{&S43244[71], 6}, {NULL}};
const Segment SGI155720[] = {{&S52399[34], 11}, {NULL}};
const Segment SGI155721[] = {{&S52399[27], 6}, {NULL}};
const Segment SGI155722[] = {{&S52399[98], 9}, {NULL}};
const Segment SGI155723[] = {{&S52399[71], 6}, {NULL}};
const Segment SGI155724[] = {{&S43245[34], 11}, {NULL}};
const Segment SGI155725[] = {{&S43245[27], 6}, {NULL}};
const Segment SGI155726[] = {{&S43245[98], 9}, {NULL}};
const Segment SGI155727[] = {{&S43245[71], 6}, {NULL}};
const Segment SGI155728[] = {{&S52400[34], 10}, {NULL}};
const Segment SGI155729[] = {{&S52400[27], 6}, {NULL}};
const Segment SGI155730[] = {{&S52400[97], 9}, {NULL}};
const Segment SGI155731[] = {{&S52400[70], 6}, {NULL}};
const Segment SGI155732[] = {{&S52401[34], 11}, {NULL}};
const Segment SGI155733[] = {{&S52401[27], 6}, {NULL}};
const Segment SGI155734[] = {{&S52401[98], 9}, {NULL}};
const Segment SGI155735[] = {{&S52401[71], 6}, {NULL}};
const Segment SGI155736[] = {{&S52416[34], 7}, {NULL}};
const Segment SGI155737[] = {{&S52416[27], 6}, {NULL}};
const Segment SGI155738[] = {{&S52416[94], 9}, {NULL}};
const Segment SGI155739[] = {{&S52416[67], 6}, {NULL}};
const Segment SGI155740[] = {{&S52417[34], 7}, {NULL}};
const Segment SGI155741[] = {{&S52417[27], 6}, {NULL}};
const Segment SGI155742[] = {{&S52417[94], 9}, {NULL}};
const Segment SGI155743[] = {{&S52417[67], 6}, {NULL}};
const Segment SGI155744[] = {{&S52443[34], 11}, {NULL}};
const Segment SGI155745[] = {{&S52443[27], 6}, {NULL}};
const Segment SGI155746[] = {{&S52443[98], 9}, {NULL}};
const Segment SGI155747[] = {{&S52443[71], 6}, {NULL}};
const Segment SGI155748[] = {{&S42643[34], 10}, {NULL}};
const Segment SGI155749[] = {{&S42643[27], 6}, {NULL}};
const Segment SGI155750[] = {{&S42643[97], 9}, {NULL}};
const Segment SGI155751[] = {{&S42643[70], 6}, {NULL}};
const Segment SGI155752[] = {{&S42672[34], 7}, {NULL}};
const Segment SGI155753[] = {{&S42672[27], 6}, {NULL}};
const Segment SGI155754[] = {{&S42672[99], 9}, {NULL}};
const Segment SGI155755[] = {{&S42672[72], 6}, {NULL}};
const Segment SGI155756[] = {{&S42673[34], 7}, {NULL}};
const Segment SGI155757[] = {{&S42673[27], 6}, {NULL}};
const Segment SGI155758[] = {{&S42673[99], 9}, {NULL}};
const Segment SGI155759[] = {{&S42673[72], 6}, {NULL}};
const Segment SGI155760[] = {{&S52526[34], 11}, {NULL}};
const Segment SGI155761[] = {{&S52526[27], 6}, {NULL}};
const Segment SGI155762[] = {{&S52526[98], 9}, {NULL}};
const Segment SGI155763[] = {{&S52526[71], 6}, {NULL}};
const Segment SGI155764[] = {{&S52538[44], 15}, {NULL}};
const Segment SGI155765[] = {{&S52538[44], 33}, {NULL}};
const Segment SGI155766[] = {{&S52538[30], 6}, {NULL}};
const Segment SGI155767[] = {{&S52538[31], 3}, {NULL}};
const Segment SGI155768[] = {{&S52538[91], 6}, {NULL}};
const Segment SGI155769[] = {{&S52539[44], 14}, {NULL}};
const Segment SGI155770[] = {{&S52539[44], 31}, {NULL}};
const Segment SGI155771[] = {{&S52539[30], 6}, {NULL}};
const Segment SGI155772[] = {{&S52539[31], 3}, {NULL}};
const Segment SGI155773[] = {{&S52539[89], 6}, {NULL}};
const Segment SGI155774[] = {{&S52540[44], 15}, {NULL}};
const Segment SGI155775[] = {{&S52540[44], 33}, {NULL}};
const Segment SGI155776[] = {{&S52540[30], 6}, {NULL}};
const Segment SGI155777[] = {{&S52540[94], 6}, {NULL}};
const Segment SGI155778[] = {{&S52541[38], 15}, {NULL}};
const Segment SGI155779[] = {{&S52541[31], 6}, {NULL}};
const Segment SGI155780[] = {{&S52542[44], 15}, {NULL}};
const Segment SGI155781[] = {{&S52542[44], 33}, {NULL}};
const Segment SGI155782[] = {{&S52542[30], 6}, {NULL}};
const Segment SGI155783[] = {{&S52542[91], 6}, {NULL}};
const Segment SGI155784[] = {{&S52554[34], 13}, {NULL}};
const Segment SGI155785[] = {{&S52554[27], 6}, {NULL}};
const Segment SGI155786[] = {{&S52554[100], 9}, {NULL}};
const Segment SGI155787[] = {{&S52554[73], 6}, {NULL}};
const Segment SGI155788[] = {{&S52555[34], 6}, {NULL}};
const Segment SGI155789[] = {{&S52555[27], 6}, {NULL}};
const Segment SGI155790[] = {{&S43370[34], 10}, {NULL}};
const Segment SGI155791[] = {{&S52555[93], 9}, {NULL}};
const Segment SGI155792[] = {{&S43370[27], 6}, {NULL}};
const Segment SGI155793[] = {{&S52555[66], 6}, {NULL}};
const Segment SGI155794[] = {{&S43370[97], 9}, {NULL}};
const Segment SGI155795[] = {{&S43370[70], 6}, {NULL}};
const Segment SGI155796[] = {{&S52557[34], 14}, {NULL}};
const Segment SGI155797[] = {{&S52557[27], 6}, {NULL}};
const Segment SGI155798[] = {{&S52557[114], 9}, {NULL}};
const Segment SGI155799[] = {{&S52557[87], 6}, {NULL}};
const Segment SGI155800[] = {{&S52558[34], 14}, {NULL}};
const Segment SGI155801[] = {{&S52558[27], 6}, {NULL}};
const Segment SGI155802[] = {{&S52558[114], 9}, {NULL}};
const Segment SGI155803[] = {{&S52558[87], 6}, {NULL}};
const Segment SGI155804[] = {{&S52562[34], 8}, {NULL}};
const Segment SGI155805[] = {{&S52562[27], 6}, {NULL}};
const Segment SGI155806[] = {{&S52562[100], 9}, {NULL}};
const Segment SGI155807[] = {{&S52562[73], 6}, {NULL}};
const Segment SGI155808[] = {{&S52580[34], 8}, {NULL}};
const Segment SGI155809[] = {{&S52580[27], 6}, {NULL}};
const Segment SGI155810[] = {{&S52580[111], 9}, {NULL}};
const Segment SGI155811[] = {{&S52580[84], 6}, {NULL}};
const Segment SGI155812[] = {{&S52581[34], 8}, {NULL}};
const Segment SGI155813[] = {{&S52581[27], 6}, {NULL}};
const Segment SGI155814[] = {{&S52581[111], 9}, {NULL}};
const Segment SGI155815[] = {{&S52581[84], 6}, {NULL}};
const Segment SGI155816[] = {{&S52608[34], 10}, {NULL}};
const Segment SGI155817[] = {{&S52608[27], 6}, {NULL}};
const Segment SGI155818[] = {{&S52608[97], 9}, {NULL}};
const Segment SGI155819[] = {{&S52608[70], 6}, {NULL}};
const Segment SGI155820[] = {{&S52610[44], 6}, {NULL}};
const Segment SGI155821[] = {{&S52610[44], 18}, {NULL}};
const Segment SGI155822[] = {{&S52610[30], 6}, {NULL}};
const Segment SGI155823[] = {{&S52610[31], 3}, {NULL}};
const Segment SGI155824[] = {{&S52610[76], 6}, {NULL}};
const Segment SGI155825[] = {{&S52611[58], 5}, {NULL}};
const Segment SGI155826[] = {{&S52611[41], 16}, {NULL}};
const Segment SGI155827[] = {{&S52611[29], 3}, {NULL}};
const Segment SGI155828[] = {{&S52611[86], 9}, {NULL}};
const Segment SGI155829[] = {{&S52612[62], 5}, {NULL}};
const Segment SGI155830[] = {{&S52612[44], 17}, {NULL}};
const Segment SGI155831[] = {{&S52612[30], 6}, {NULL}};
const Segment SGI155832[] = {{&S52612[31], 3}, {NULL}};
const Segment SGI155833[] = {{&S52612[90], 6}, {NULL}};
const Segment SGI155834[] = {{&S52614[34], 9}, {NULL}};
const Segment SGI155835[] = {{&S52614[27], 6}, {NULL}};
const Segment SGI155836[] = {{&S52614[96], 9}, {NULL}};
const Segment SGI155837[] = {{&S52614[69], 6}, {NULL}};
const Segment SGI155838[] = {{&S43441[34], 11}, {NULL}};
const Segment SGI155839[] = {{&S43441[27], 6}, {NULL}};
const Segment SGI155840[] = {{&S43441[98], 9}, {NULL}};
const Segment SGI155841[] = {{&S43441[71], 6}, {NULL}};
const Segment SGI155842[] = {{&S43442[34], 11}, {NULL}};
const Segment SGI155843[] = {{&S43442[27], 6}, {NULL}};
const Segment SGI155844[] = {{&S43442[98], 9}, {NULL}};
const Segment SGI155845[] = {{&S43442[71], 6}, {NULL}};
const Segment SGI155846[] = {{&S43444[34], 27}, {NULL}};
const Segment SGI155847[] = {{&S43444[27], 6}, {NULL}};
const Segment SGI155848[] = {{&S43444[114], 9}, {NULL}};
const Segment SGI155849[] = {{&S43444[87], 6}, {NULL}};
const Segment SGI155850[] = {{&S52626[34], 10}, {NULL}};
const Segment SGI155851[] = {{&S52626[27], 6}, {NULL}};
const Segment SGI155852[] = {{&S52626[97], 9}, {NULL}};
const Segment SGI155853[] = {{&S52626[70], 6}, {NULL}};
const Segment SGI155854[] = {{&S52627[34], 14}, {NULL}};
const Segment SGI155855[] = {{&S52627[27], 6}, {NULL}};
const Segment SGI155856[] = {{&S42833[44], 10}, {NULL}};
const Segment SGI155857[] = {{&S42833[44], 22}, {NULL}};
const Segment SGI155858[] = {{&S42833[30], 6}, {NULL}};
const Segment SGI155859[] = {{&S52627[101], 9}, {NULL}};
const Segment SGI155860[] = {{&S52627[74], 6}, {NULL}};
const Segment SGI155861[] = {{&S42833[115], 3}, {NULL}};
const Segment SGI155862[] = {{&S42833[80], 6}, {NULL}};
const Segment SGI155863[] = {{&S42835[38], 10}, {NULL}};
const Segment SGI155864[] = {{&S42835[31], 6}, {NULL}};
const Segment SGI155865[] = {{&S52633[44], 10}, {NULL}};
const Segment SGI155866[] = {{&S52633[44], 22}, {NULL}};
const Segment SGI155867[] = {{&S52633[30], 6}, {NULL}};
const Segment SGI155868[] = {{&S52633[31], 3}, {NULL}};
const Segment SGI155869[] = {{&S52633[80], 6}, {NULL}};
const Segment SGI155870[] = {{&S52644[34], 9}, {NULL}};
const Segment SGI155871[] = {{&S52644[27], 6}, {NULL}};
const Segment SGI155872[] = {{&S52644[96], 9}, {NULL}};
const Segment SGI155873[] = {{&S52644[69], 6}, {NULL}};
const Segment SGI155874[] = {{&S52645[34], 9}, {NULL}};
const Segment SGI155875[] = {{&S52645[27], 6}, {NULL}};
const Segment SGI155876[] = {{&S52645[96], 9}, {NULL}};
const Segment SGI155877[] = {{&S52645[69], 6}, {NULL}};
const Segment SGI155878[] = {{&S52646[34], 9}, {NULL}};
const Segment SGI155879[] = {{&S52646[27], 6}, {NULL}};
const Segment SGI155880[] = {{&S52646[96], 9}, {NULL}};
const Segment SGI155881[] = {{&S52646[69], 6}, {NULL}};
const Segment SGI155882[] = {{&S52647[34], 9}, {NULL}};
const Segment SGI155883[] = {{&S52647[27], 6}, {NULL}};
const Segment SGI155884[] = {{&S52647[96], 9}, {NULL}};
const Segment SGI155885[] = {{&S52647[69], 6}, {NULL}};
const Segment SGI155886[] = {{&S43481[74], 17}, {NULL}};
const Segment SGI155887[] = {{&S43481[44], 22}, {&S43481[74], 29}, {NULL}};
const Segment SGI155888[] = {{&S43481[30], 6}, {NULL}};
const Segment SGI155889[] = {{&S43481[151], 3}, {NULL}};
const Segment SGI155890[] = {{&S43481[117], 6}, {NULL}};
const Segment SGI155891[] = {{&S52654[34], 6}, {NULL}};
const Segment SGI155892[] = {{&S52654[27], 6}, {NULL}};
const Segment SGI155893[] = {{&S52654[93], 9}, {NULL}};
const Segment SGI155894[] = {{&S52654[66], 6}, {NULL}};
const Segment SGI155895[] = {{&S52655[34], 13}, {NULL}};
const Segment SGI155896[] = {{&S52655[27], 6}, {NULL}};
const Segment SGI155897[] = {{&S52655[100], 9}, {NULL}};
const Segment SGI155898[] = {{&S52655[73], 6}, {NULL}};
const Segment SGI155899[] = {{&S52656[34], 6}, {NULL}};
const Segment SGI155900[] = {{&S52656[27], 6}, {NULL}};
const Segment SGI155901[] = {{&S52656[93], 9}, {NULL}};
const Segment SGI155902[] = {{&S52656[66], 6}, {NULL}};
const Segment SGI155903[] = {{&S52658[34], 12}, {NULL}};
const Segment SGI155904[] = {{&S52658[27], 6}, {NULL}};
const Segment SGI155905[] = {{&S52658[99], 9}, {NULL}};
const Segment SGI155906[] = {{&S52658[72], 6}, {NULL}};
const Segment SGI155907[] = {{&S38040[38], 10}, {NULL}};
const Segment SGI155908[] = {{&S38040[31], 6}, {NULL}};
const Segment SGI155909[] = {{&S43494[34], 25}, {NULL}};
const Segment SGI155910[] = {{&S43494[27], 6}, {NULL}};
const Segment SGI155911[] = {{&S43494[112], 9}, {NULL}};
const Segment SGI155912[] = {{&S43494[85], 6}, {NULL}};
const Segment SGI155913[] = {{&S38041[44], 10}, {NULL}};
const Segment SGI155914[] = {{&S38041[44], 22}, {NULL}};
const Segment SGI155915[] = {{&S38041[30], 6}, {NULL}};
const Segment SGI155916[] = {{&S38041[115], 3}, {NULL}};
const Segment SGI155917[] = {{&S38041[80], 6}, {NULL}};
const Segment SGI155918[] = {{&S43496[34], 26}, {NULL}};
const Segment SGI155919[] = {{&S43496[27], 6}, {NULL}};
const Segment SGI155920[] = {{&S43496[113], 9}, {NULL}};
const Segment SGI155921[] = {{&S38042[44], 11}, {NULL}};
const Segment SGI155922[] = {{&S43496[86], 6}, {NULL}};
const Segment SGI155923[] = {{&S38042[44], 23}, {NULL}};
const Segment SGI155924[] = {{&S38042[30], 6}, {NULL}};
const Segment SGI155925[] = {{&S38042[116], 3}, {NULL}};
const Segment SGI155926[] = {{&S38042[81], 6}, {NULL}};
const Segment SGI155927[] = {{&S43497[34], 7}, {NULL}};
const Segment SGI155928[] = {{&S43497[27], 6}, {NULL}};
const Segment SGI155929[] = {{&S38043[44], 10}, {NULL}};
const Segment SGI155930[] = {{&S43497[94], 9}, {NULL}};
const Segment SGI155931[] = {{&S38043[44], 22}, {NULL}};
const Segment SGI155932[] = {{&S43497[67], 6}, {NULL}};
const Segment SGI155933[] = {{&S38043[30], 6}, {NULL}};
const Segment SGI155934[] = {{&S38043[115], 3}, {NULL}};
const Segment SGI155935[] = {{&S38043[80], 6}, {NULL}};
const Segment SGI155936[] = {{&S42896[44], 10}, {NULL}};
const Segment SGI155937[] = {{&S42896[44], 22}, {NULL}};
const Segment SGI155938[] = {{&S42896[30], 6}, {NULL}};
const Segment SGI155939[] = {{&S42896[115], 3}, {NULL}};
const Segment SGI155940[] = {{&S42896[80], 6}, {NULL}};
const Segment SGI155941[] = {{&S42897[44], 18}, {NULL}};
const Segment SGI155942[] = {{&S42897[44], 30}, {NULL}};
const Segment SGI155943[] = {{&S42897[30], 6}, {NULL}};
const Segment SGI155944[] = {{&S42897[123], 3}, {NULL}};
const Segment SGI155945[] = {{&S42897[88], 6}, {NULL}};
const Segment SGI155946[] = {{&S42898[38], 18}, {NULL}};
const Segment SGI155947[] = {{&S42898[31], 6}, {NULL}};
const Segment SGI155948[] = {{&S42899[44], 10}, {NULL}};
const Segment SGI155949[] = {{&S42899[44], 22}, {NULL}};
const Segment SGI155950[] = {{&S42899[30], 6}, {NULL}};
const Segment SGI155951[] = {{&S42899[115], 3}, {NULL}};
const Segment SGI155952[] = {{&S42899[80], 6}, {NULL}};
const Segment SGI155953[] = {{&S42900[42], 18}, {NULL}};
const Segment SGI155954[] = {{&S42900[30], 6}, {NULL}};
const Segment SGI155955[] = {{&S42900[121], 3}, {NULL}};
const Segment SGI155956[] = {{&S42900[86], 6}, {NULL}};
const Segment SGI155957[] = {{&S52697[44], 12}, {NULL}};
const Segment SGI155958[] = {{&S52697[44], 29}, {NULL}};
const Segment SGI155959[] = {{&S52697[30], 6}, {NULL}};
const Segment SGI155960[] = {{&S52697[31], 3}, {NULL}};
const Segment SGI155961[] = {{&S52697[87], 6}, {NULL}};
const Segment SGI155962[] = {{&S52698[44], 12}, {NULL}};
const Segment SGI155963[] = {{&S52698[44], 29}, {NULL}};
const Segment SGI155964[] = {{&S52698[30], 6}, {NULL}};
const Segment SGI155965[] = {{&S52698[31], 3}, {NULL}};
const Segment SGI155966[] = {{&S52698[87], 6}, {NULL}};
const Segment SGI155967[] = {{&S52699[44], 8}, {NULL}};
const Segment SGI155968[] = {{&S52699[44], 25}, {NULL}};
const Segment SGI155969[] = {{&S52699[30], 6}, {NULL}};
const Segment SGI155970[] = {{&S52699[31], 3}, {NULL}};
const Segment SGI155971[] = {{&S52699[83], 6}, {NULL}};
const Segment SGI155972[] = {{&S52700[44], 12}, {NULL}};
const Segment SGI155973[] = {{&S52700[44], 28}, {NULL}};
const Segment SGI155974[] = {{&S52700[30], 6}, {NULL}};
const Segment SGI155975[] = {{&S52700[31], 3}, {NULL}};
const Segment SGI155976[] = {{&S52700[86], 6}, {NULL}};
const Segment SGI155977[] = {{&S52701[44], 8}, {NULL}};
const Segment SGI155978[] = {{&S52701[44], 25}, {NULL}};
const Segment SGI155979[] = {{&S52701[30], 6}, {NULL}};
const Segment SGI155980[] = {{&S52701[31], 3}, {NULL}};
const Segment SGI155981[] = {{&S52701[83], 6}, {NULL}};
const Segment SGI155982[] = {{&S52702[44], 8}, {NULL}};
const Segment SGI155983[] = {{&S52702[44], 24}, {NULL}};
const Segment SGI155984[] = {{&S52702[30], 6}, {NULL}};
const Segment SGI155985[] = {{&S52702[31], 3}, {NULL}};
const Segment SGI155986[] = {{&S52702[82], 6}, {NULL}};
const Segment SGI155987[] = {{&S52703[44], 12}, {NULL}};
const Segment SGI155988[] = {{&S52703[44], 29}, {NULL}};
const Segment SGI155989[] = {{&S52703[30], 6}, {NULL}};
const Segment SGI155990[] = {{&S52703[31], 3}, {NULL}};
const Segment SGI155991[] = {{&S52703[87], 6}, {NULL}};
const Segment SGI155992[] = {{&S52704[44], 8}, {NULL}};
const Segment SGI155993[] = {{&S52704[44], 25}, {NULL}};
const Segment SGI155994[] = {{&S52704[30], 6}, {NULL}};
const Segment SGI155995[] = {{&S52704[31], 3}, {NULL}};
const Segment SGI155996[] = {{&S52704[83], 6}, {NULL}};
const Segment SGI155997[] = {{&S52705[38], 8}, {NULL}};
const Segment SGI155998[] = {{&S52705[31], 6}, {NULL}};
const Segment SGI155999[] = {{&S43555[34], 6}, {NULL}};
const Segment SGI156000[] = {{&S43555[27], 6}, {NULL}};
const Segment SGI156001[] = {{&S43555[93], 9}, {NULL}};
const Segment SGI156002[] = {{&S43555[66], 6}, {NULL}};
const Segment SGI156003[] = {{&S43556[34], 7}, {NULL}};
const Segment SGI156004[] = {{&S43556[27], 6}, {NULL}};
const Segment SGI156005[] = {{&S43556[94], 9}, {NULL}};
const Segment SGI156006[] = {{&S43556[67], 6}, {NULL}};
const Segment SGI156007[] = {{&S52713[44], 11}, {NULL}};
const Segment SGI156008[] = {{&S52713[44], 23}, {NULL}};
const Segment SGI156009[] = {{&S42935[44], 10}, {NULL}};
const Segment SGI156010[] = {{&S52713[30], 6}, {NULL}};
const Segment SGI156011[] = {{&S42935[44], 22}, {NULL}};
const Segment SGI156012[] = {{&S42935[30], 6}, {NULL}};
const Segment SGI156013[] = {{&S52713[31], 3}, {NULL}};
const Segment SGI156014[] = {{&S42935[115], 3}, {NULL}};
const Segment SGI156015[] = {{&S52713[81], 6}, {NULL}};
const Segment SGI156016[] = {{&S42935[80], 6}, {NULL}};
const Segment SGI156017[] = {{&S52714[38], 11}, {NULL}};
const Segment SGI156018[] = {{&S52714[31], 6}, {NULL}};
const Segment SGI156019[] = {{&S42936[38], 10}, {NULL}};
const Segment SGI156020[] = {{&S42936[31], 6}, {NULL}};
const Segment SGI156021[] = {{&S41043[32], 9}, {NULL}};
const Segment SGI156022[] = {{&S41043[27], 4}, {NULL}};
const Segment SGI156023[] = {{&S41043[94], 9}, {NULL}};
const Segment SGI156024[] = {{&S41043[67], 6}, {NULL}};
const Segment SGI156025[] = {{&S42965[44], 11}, {NULL}};
const Segment SGI156026[] = {{&S42965[44], 23}, {NULL}};
const Segment SGI156027[] = {{&S42965[30], 6}, {NULL}};
const Segment SGI156028[] = {{&S42965[116], 3}, {NULL}};
const Segment SGI156029[] = {{&S42965[81], 6}, {NULL}};
const Segment SGI156030[] = {{&S42966[44], 32}, {NULL}};
const Segment SGI156031[] = {{&S42966[44], 44}, {NULL}};
const Segment SGI156032[] = {{&S42966[30], 6}, {NULL}};
const Segment SGI156033[] = {{&S42966[137], 3}, {NULL}};
const Segment SGI156034[] = {{&S42966[102], 6}, {NULL}};
const Segment SGI156035[] = {{&S42967[38], 11}, {NULL}};
const Segment SGI156036[] = {{&S42967[31], 6}, {NULL}};
const Segment SGI156037[] = {{&S42968[44], 32}, {NULL}};
const Segment SGI156038[] = {{&S42968[44], 44}, {NULL}};
const Segment SGI156039[] = {{&S42968[30], 6}, {NULL}};
const Segment SGI156040[] = {{&S42968[137], 3}, {NULL}};
const Segment SGI156041[] = {{&S42968[102], 6}, {NULL}};
const Segment SGI156042[] = {{&S42969[44], 32}, {NULL}};
const Segment SGI156043[] = {{&S42969[44], 44}, {NULL}};
const Segment SGI156044[] = {{&S42969[30], 6}, {NULL}};
const Segment SGI156045[] = {{&S42969[137], 3}, {NULL}};
const Segment SGI156046[] = {{&S42969[102], 6}, {NULL}};
const Segment SGI156047[] = {{&S42970[44], 32}, {NULL}};
const Segment SGI156048[] = {{&S42970[44], 44}, {NULL}};
const Segment SGI156049[] = {{&S42970[30], 6}, {NULL}};
const Segment SGI156050[] = {{&S42970[137], 3}, {NULL}};
const Segment SGI156051[] = {{&S42970[102], 6}, {NULL}};
const Segment SGI156052[] = {{&S42971[44], 31}, {NULL}};
const Segment SGI156053[] = {{&S42971[44], 43}, {NULL}};
const Segment SGI156054[] = {{&S42971[30], 6}, {NULL}};
const Segment SGI156055[] = {{&S42971[136], 3}, {NULL}};
const Segment SGI156056[] = {{&S42971[101], 6}, {NULL}};
const Segment SGI156057[] = {{&S42986[34], 11}, {NULL}};
const Segment SGI156058[] = {{&S42986[27], 6}, {NULL}};
const Segment SGI156059[] = {{&S42986[98], 9}, {NULL}};
const Segment SGI156060[] = {{&S42986[71], 6}, {NULL}};
const Segment SGI156061[] = {{&S42988[34], 7}, {NULL}};
const Segment SGI156062[] = {{&S42988[27], 6}, {NULL}};
const Segment SGI156063[] = {{&S42988[99], 9}, {NULL}};
const Segment SGI156064[] = {{&S42988[72], 6}, {NULL}};
const Segment SGI156065[] = {{&S42989[34], 7}, {NULL}};
const Segment SGI156066[] = {{&S42989[27], 6}, {NULL}};
const Segment SGI156067[] = {{&S42989[98], 9}, {NULL}};
const Segment SGI156068[] = {{&S42989[71], 6}, {NULL}};
const Segment SGI156069[] = {{&S42997[34], 16}, {NULL}};
const Segment SGI156070[] = {{&S42997[27], 6}, {NULL}};
const Segment SGI156071[] = {{&S42997[103], 9}, {NULL}};
const Segment SGI156072[] = {{&S42997[76], 6}, {NULL}};
const Segment SGI156073[] = {{&S52789[44], 10}, {NULL}};
const Segment SGI156074[] = {{&S52789[44], 22}, {NULL}};
const Segment SGI156075[] = {{&S52789[30], 6}, {NULL}};
const Segment SGI156076[] = {{&S52789[31], 3}, {NULL}};
const Segment SGI156077[] = {{&S52789[80], 6}, {NULL}};
const Segment SGI156078[] = {{&S52790[44], 11}, {NULL}};
const Segment SGI156079[] = {{&S43000[34], 21}, {NULL}};
const Segment SGI156080[] = {{&S52790[44], 23}, {NULL}};
const Segment SGI156081[] = {{&S43000[27], 6}, {NULL}};
const Segment SGI156082[] = {{&S52790[30], 6}, {NULL}};
const Segment SGI156083[] = {{&S52790[31], 3}, {NULL}};
const Segment SGI156084[] = {{&S43000[108], 9}, {NULL}};
const Segment SGI156085[] = {{&S52790[81], 6}, {NULL}};
const Segment SGI156086[] = {{&S43000[81], 6}, {NULL}};
const Segment SGI156087[] = {{&S43001[34], 10}, {NULL}};
const Segment SGI156088[] = {{&S43001[27], 6}, {NULL}};
const Segment SGI156089[] = {{&S52791[42], 10}, {NULL}};
const Segment SGI156090[] = {{&S52791[30], 6}, {NULL}};
const Segment SGI156091[] = {{&S43001[97], 9}, {NULL}};
const Segment SGI156092[] = {{&S43001[70], 6}, {NULL}};
const Segment SGI156093[] = {{&S52791[31], 3}, {NULL}};
const Segment SGI156094[] = {{&S52791[78], 6}, {NULL}};
const Segment SGI156095[] = {{&S52792[38], 10}, {NULL}};
const Segment SGI156096[] = {{&S52792[31], 6}, {NULL}};
const Segment SGI156097[] = {{&S43003[34], 17}, {NULL}};
const Segment SGI156098[] = {{&S43003[27], 6}, {NULL}};
const Segment SGI156099[] = {{&S43003[104], 9}, {NULL}};
const Segment SGI156100[] = {{&S43003[77], 6}, {NULL}};
const Segment SGI156101[] = {{&S43004[34], 10}, {NULL}};
const Segment SGI156102[] = {{&S43004[27], 6}, {NULL}};
const Segment SGI156103[] = {{&S43004[97], 9}, {NULL}};
const Segment SGI156104[] = {{&S43004[70], 6}, {NULL}};
const Segment SGI156105[] = {{&S52795[34], 15}, {NULL}};
const Segment SGI156106[] = {{&S52795[27], 6}, {NULL}};
const Segment SGI156107[] = {{&S52795[102], 9}, {NULL}};
const Segment SGI156108[] = {{&S52795[75], 6}, {NULL}};
const Segment SGI156109[] = {{&S43008[34], 10}, {NULL}};
const Segment SGI156110[] = {{&S43008[27], 6}, {NULL}};
const Segment SGI156111[] = {{&S43008[97], 9}, {NULL}};
const Segment SGI156112[] = {{&S43008[70], 6}, {NULL}};
const Segment SGI156113[] = {{&S43025[34], 8}, {NULL}};
const Segment SGI156114[] = {{&S43025[27], 6}, {NULL}};
const Segment SGI156115[] = {{&S43025[95], 9}, {NULL}};
const Segment SGI156116[] = {{&S43025[68], 6}, {NULL}};
const Segment SGI156117[] = {{&S43665[44], 8}, {NULL}};
const Segment SGI156118[] = {{&S43665[44], 20}, {NULL}};
const Segment SGI156119[] = {{&S43665[30], 6}, {NULL}};
const Segment SGI156120[] = {{&S43665[113], 3}, {NULL}};
const Segment SGI156121[] = {{&S43665[78], 6}, {NULL}};
const Segment SGI156122[] = {{&S43666[38], 8}, {NULL}};
const Segment SGI156123[] = {{&S43666[31], 6}, {NULL}};
const Segment SGI156124[] = {{&S43037[34], 6}, {NULL}};
const Segment SGI156125[] = {{&S43037[27], 6}, {NULL}};
const Segment SGI156126[] = {{&S43037[93], 9}, {NULL}};
const Segment SGI156127[] = {{&S43037[66], 6}, {NULL}};
const Segment SGI156128[] = {{&S43042[34], 8}, {NULL}};
const Segment SGI156129[] = {{&S43042[27], 6}, {NULL}};
const Segment SGI156130[] = {{&S43042[95], 9}, {NULL}};
const Segment SGI156131[] = {{&S43042[68], 6}, {NULL}};
const Segment SGI156132[] = {{&S43043[34], 4}, {NULL}};
const Segment SGI156133[] = {{&S43043[27], 6}, {NULL}};
const Segment SGI156134[] = {{&S43043[91], 9}, {NULL}};
const Segment SGI156135[] = {{&S43043[64], 6}, {NULL}};
const Segment SGI156136[] = {{&S43045[34], 8}, {NULL}};
const Segment SGI156137[] = {{&S43045[27], 6}, {NULL}};
const Segment SGI156138[] = {{&S43045[100], 9}, {NULL}};
const Segment SGI156139[] = {{&S43045[73], 6}, {NULL}};
const Segment SGI156140[] = {{&S43046[34], 6}, {NULL}};
const Segment SGI156141[] = {{&S43046[27], 6}, {NULL}};
const Segment SGI156142[] = {{&S43046[93], 9}, {NULL}};
const Segment SGI156143[] = {{&S43046[66], 6}, {NULL}};
const Segment SGI156144[] = {{&S43047[34], 6}, {NULL}};
const Segment SGI156145[] = {{&S43047[27], 6}, {NULL}};
const Segment SGI156146[] = {{&S43047[93], 9}, {NULL}};
const Segment SGI156147[] = {{&S43047[66], 6}, {NULL}};
const Segment SGI156148[] = {{&S43052[34], 7}, {NULL}};
const Segment SGI156149[] = {{&S43052[27], 6}, {NULL}};
const Segment SGI156150[] = {{&S43052[94], 9}, {NULL}};
const Segment SGI156151[] = {{&S43052[67], 6}, {NULL}};
const Segment SGI156152[] = {{&S43053[34], 6}, {NULL}};
const Segment SGI156153[] = {{&S43053[27], 6}, {NULL}};
const Segment SGI156154[] = {{&S43053[93], 9}, {NULL}};
const Segment SGI156155[] = {{&S43053[66], 6}, {NULL}};
const Segment SGI156156[] = {{&S43055[34], 6}, {NULL}};
const Segment SGI156157[] = {{&S43055[27], 6}, {NULL}};
const Segment SGI156158[] = {{&S52862[34], 11}, {NULL}};
const Segment SGI156159[] = {{&S43055[93], 9}, {NULL}};
const Segment SGI156160[] = {{&S52862[27], 6}, {NULL}};
const Segment SGI156161[] = {{&S43055[66], 6}, {NULL}};
const Segment SGI156162[] = {{&S52862[98], 9}, {NULL}};
const Segment SGI156163[] = {{&S52862[71], 6}, {NULL}};
const Segment SGI156164[] = {{&S52863[34], 11}, {NULL}};
const Segment SGI156165[] = {{&S52863[27], 6}, {NULL}};
const Segment SGI156166[] = {{&S43057[34], 4}, {NULL}};
const Segment SGI156167[] = {{&S43057[27], 6}, {NULL}};
const Segment SGI156168[] = {{&S52863[98], 9}, {NULL}};
const Segment SGI156169[] = {{&S52863[71], 6}, {NULL}};
const Segment SGI156170[] = {{&S43057[91], 9}, {NULL}};
const Segment SGI156171[] = {{&S43057[64], 6}, {NULL}};
const Segment SGI156172[] = {{&S43058[34], 10}, {NULL}};
const Segment SGI156173[] = {{&S43058[27], 6}, {NULL}};
const Segment SGI156174[] = {{&S43058[97], 9}, {NULL}};
const Segment SGI156175[] = {{&S43058[70], 6}, {NULL}};
const Segment SGI156176[] = {{&S43060[34], 6}, {NULL}};
const Segment SGI156177[] = {{&S43060[27], 6}, {NULL}};
const Segment SGI156178[] = {{&S43060[93], 9}, {NULL}};
const Segment SGI156179[] = {{&S43060[66], 6}, {NULL}};
const Segment SGI156180[] = {{&S43062[34], 4}, {NULL}};
const Segment SGI156181[] = {{&S43062[27], 6}, {NULL}};
const Segment SGI156182[] = {{&S43062[91], 9}, {NULL}};
const Segment SGI156183[] = {{&S43062[64], 6}, {NULL}};
const Segment SGI156184[] = {{&S43063[34], 11}, {NULL}};
const Segment SGI156185[] = {{&S43063[27], 6}, {NULL}};
const Segment SGI156186[] = {{&S43063[98], 9}, {NULL}};
const Segment SGI156187[] = {{&S43063[71], 6}, {NULL}};
const Segment SGI156188[] = {{&S43066[34], 8}, {NULL}};
const Segment SGI156189[] = {{&S43066[27], 6}, {NULL}};
const Segment SGI156190[] = {{&S43066[95], 9}, {NULL}};
const Segment SGI156191[] = {{&S43066[68], 6}, {NULL}};
const Segment SGI156192[] = {{&S43075[34], 8}, {NULL}};
const Segment SGI156193[] = {{&S43075[27], 6}, {NULL}};
const Segment SGI156194[] = {{&S43075[103], 9}, {NULL}};
const Segment SGI156195[] = {{&S43075[76], 6}, {NULL}};
const Segment SGI156196[] = {{&S43076[34], 11}, {NULL}};
const Segment SGI156197[] = {{&S43076[27], 6}, {NULL}};
const Segment SGI156198[] = {{&S43076[98], 9}, {NULL}};
const Segment SGI156199[] = {{&S43076[71], 6}, {NULL}};
const Segment SGI156200[] = {{&S43077[34], 20}, {NULL}};
const Segment SGI156201[] = {{&S43077[27], 6}, {NULL}};
const Segment SGI156202[] = {{&S43077[107], 9}, {NULL}};
const Segment SGI156203[] = {{&S43077[80], 6}, {NULL}};
const Segment SGI156204[] = {{&S43079[27], 6}, {NULL}};
const Segment SGI156205[] = {{&S43079[117], 9}, {NULL}};
const Segment SGI156206[] = {{&S43079[90], 6}, {NULL}};
const Segment SGI156207[] = {{&S43080[34], 13}, {NULL}};
const Segment SGI156208[] = {{&S43080[27], 6}, {NULL}};
const Segment SGI156209[] = {{&S43080[121], 9}, {NULL}};
const Segment SGI156210[] = {{&S43080[94], 6}, {NULL}};
const Segment SGI156211[] = {{&S43081[34], 21}, {NULL}};
const Segment SGI156212[] = {{&S43081[27], 6}, {NULL}};
const Segment SGI156213[] = {{&S43081[108], 9}, {NULL}};
const Segment SGI156214[] = {{&S43081[81], 6}, {NULL}};
const Segment SGI156215[] = {{&S43085[34], 16}, {NULL}};
const Segment SGI156216[] = {{&S43085[27], 6}, {NULL}};
const Segment SGI156217[] = {{&S43085[103], 9}, {NULL}};
const Segment SGI156218[] = {{&S43085[76], 6}, {NULL}};
const Segment SGI156219[] = {{&S43747[34], 10}, {NULL}};
const Segment SGI156220[] = {{&S43747[27], 6}, {NULL}};
const Segment SGI156221[] = {{&S43747[97], 9}, {NULL}};
const Segment SGI156222[] = {{&S43747[70], 6}, {NULL}};
const Segment SGI156223[] = {{&S43086[34], 9}, {NULL}};
const Segment SGI156224[] = {{&S43086[27], 6}, {NULL}};
const Segment SGI156225[] = {{&S43086[96], 9}, {NULL}};
const Segment SGI156226[] = {{&S43086[69], 6}, {NULL}};
const Segment SGI156227[] = {{&S43088[34], 18}, {NULL}};
const Segment SGI156228[] = {{&S43088[27], 6}, {NULL}};
const Segment SGI156229[] = {{&S43088[105], 9}, {NULL}};
const Segment SGI156230[] = {{&S43088[78], 6}, {NULL}};
const Segment SGI156231[] = {{&S43089[34], 6}, {NULL}};
const Segment SGI156232[] = {{&S43089[27], 6}, {NULL}};
const Segment SGI156233[] = {{&S43089[98], 9}, {NULL}};
const Segment SGI156234[] = {{&S43089[71], 6}, {NULL}};
const Segment SGI156235[] = {{&S43091[34], 14}, {NULL}};
const Segment SGI156236[] = {{&S43091[27], 6}, {NULL}};
const Segment SGI156237[] = {{&S43091[101], 9}, {NULL}};
const Segment SGI156238[] = {{&S43091[74], 6}, {NULL}};
const Segment SGI156239[] = {{&S52925[34], 11}, {NULL}};
const Segment SGI156240[] = {{&S52925[27], 6}, {NULL}};
const Segment SGI156241[] = {{&S52925[98], 9}, {NULL}};
const Segment SGI156242[] = {{&S52925[71], 6}, {NULL}};
const Segment SGI156243[] = {{&S43097[34], 10}, {NULL}};
const Segment SGI156244[] = {{&S43097[27], 6}, {NULL}};
const Segment SGI156245[] = {{&S43097[97], 9}, {NULL}};
const Segment SGI156246[] = {{&S43097[70], 6}, {NULL}};
const Segment SGI156247[] = {{&S43098[34], 18}, {NULL}};
const Segment SGI156248[] = {{&S43098[27], 6}, {NULL}};
const Segment SGI156249[] = {{&S43098[105], 9}, {NULL}};
const Segment SGI156250[] = {{&S43098[78], 6}, {NULL}};
const Segment SGI156251[] = {{&S43100[34], 10}, {NULL}};
const Segment SGI156252[] = {{&S43100[27], 6}, {NULL}};
const Segment SGI156253[] = {{&S43100[97], 9}, {NULL}};
const Segment SGI156254[] = {{&S43100[70], 6}, {NULL}};
const Segment SGI156255[] = {{&S43104[44], 7}, {NULL}};
const Segment SGI156256[] = {{&S43104[44], 25}, {NULL}};
const Segment SGI156257[] = {{&S43104[30], 6}, {NULL}};
const Segment SGI156258[] = {{&S43104[118], 3}, {NULL}};
const Segment SGI156259[] = {{&S43104[83], 6}, {NULL}};
const Segment SGI156260[] = {{&S43106[44], 7}, {NULL}};
const Segment SGI156261[] = {{&S43106[44], 25}, {NULL}};
const Segment SGI156262[] = {{&S43106[30], 6}, {NULL}};
const Segment SGI156263[] = {{&S43106[118], 3}, {NULL}};
const Segment SGI156264[] = {{&S43106[83], 6}, {NULL}};
const Segment SGI156265[] = {{&S43107[44], 12}, {NULL}};
const Segment SGI156266[] = {{&S43107[44], 30}, {NULL}};
const Segment SGI156267[] = {{&S43107[30], 6}, {NULL}};
const Segment SGI156268[] = {{&S43107[123], 3}, {NULL}};
const Segment SGI156269[] = {{&S43107[88], 6}, {NULL}};
const Segment SGI156270[] = {{&S43108[44], 8}, {NULL}};
const Segment SGI156271[] = {{&S43108[44], 26}, {NULL}};
const Segment SGI156272[] = {{&S43108[30], 6}, {NULL}};
const Segment SGI156273[] = {{&S43108[119], 3}, {NULL}};
const Segment SGI156274[] = {{&S43108[84], 6}, {NULL}};
const Segment SGI156275[] = {{&S43109[44], 8}, {NULL}};
const Segment SGI156276[] = {{&S43109[44], 26}, {NULL}};
const Segment SGI156277[] = {{&S43109[30], 6}, {NULL}};
const Segment SGI156278[] = {{&S43109[119], 3}, {NULL}};
const Segment SGI156279[] = {{&S43109[84], 6}, {NULL}};
const Segment SGI156280[] = {{&S43110[44], 12}, {NULL}};
const Segment SGI156281[] = {{&S43110[44], 30}, {NULL}};
const Segment SGI156282[] = {{&S43110[30], 6}, {NULL}};
const Segment SGI156283[] = {{&S43110[123], 3}, {NULL}};
const Segment SGI156284[] = {{&S43110[88], 6}, {NULL}};
const Segment SGI156285[] = {{&S43111[44], 12}, {NULL}};
const Segment SGI156286[] = {{&S43111[44], 30}, {NULL}};
const Segment SGI156287[] = {{&S43111[30], 6}, {NULL}};
const Segment SGI156288[] = {{&S43111[123], 3}, {NULL}};
const Segment SGI156289[] = {{&S43111[88], 6}, {NULL}};
const Segment SGI156290[] = {{&S43112[44], 7}, {NULL}};
const Segment SGI156291[] = {{&S43112[44], 25}, {NULL}};
const Segment SGI156292[] = {{&S43112[30], 6}, {NULL}};
const Segment SGI156293[] = {{&S43112[118], 3}, {NULL}};
const Segment SGI156294[] = {{&S43112[83], 6}, {NULL}};
const Segment SGI156295[] = {{&S43113[38], 7}, {NULL}};
const Segment SGI156296[] = {{&S43113[31], 6}, {NULL}};
const Segment SGI156297[] = {{&S43114[44], 8}, {NULL}};
const Segment SGI156298[] = {{&S43114[44], 26}, {NULL}};
const Segment SGI156299[] = {{&S43114[30], 6}, {NULL}};
const Segment SGI156300[] = {{&S43114[119], 3}, {NULL}};
const Segment SGI156301[] = {{&S43114[84], 6}, {NULL}};
const Segment SGI156302[] = {{&S43115[44], 7}, {NULL}};
const Segment SGI156303[] = {{&S43115[44], 25}, {NULL}};
const Segment SGI156304[] = {{&S43115[30], 6}, {NULL}};
const Segment SGI156305[] = {{&S43115[118], 3}, {NULL}};
const Segment SGI156306[] = {{&S43115[83], 6}, {NULL}};
const Segment SGI156307[] = {{&S43116[44], 10}, {NULL}};
const Segment SGI156308[] = {{&S43116[44], 28}, {NULL}};
const Segment SGI156309[] = {{&S43116[30], 6}, {NULL}};
const Segment SGI156310[] = {{&S43116[121], 3}, {NULL}};
const Segment SGI156311[] = {{&S43116[86], 6}, {NULL}};
const Segment SGI156312[] = {{&S41289[42], 10}, {NULL}};
const Segment SGI156313[] = {{&S41289[42], 22}, {NULL}};
const Segment SGI156314[] = {{&S41289[30], 4}, {NULL}};
const Segment SGI156315[] = {{&S41289[113], 3}, {NULL}};
const Segment SGI156316[] = {{&S41289[78], 6}, {NULL}};
const Segment SGI156317[] = {{&S43117[44], 12}, {NULL}};
const Segment SGI156318[] = {{&S43117[44], 30}, {NULL}};
const Segment SGI156319[] = {{&S43117[30], 6}, {NULL}};
const Segment SGI156320[] = {{&S43117[123], 3}, {NULL}};
const Segment SGI156321[] = {{&S43117[88], 6}, {NULL}};
const Segment SGI156322[] = {{&S43118[44], 8}, {NULL}};
const Segment SGI156323[] = {{&S43118[44], 26}, {NULL}};
const Segment SGI156324[] = {{&S43118[30], 6}, {NULL}};
const Segment SGI156325[] = {{&S43798[44], 14}, {NULL}};
const Segment SGI156326[] = {{&S43118[119], 3}, {NULL}};
const Segment SGI156327[] = {{&S43798[44], 26}, {NULL}};
const Segment SGI156328[] = {{&S43118[84], 6}, {NULL}};
const Segment SGI156329[] = {{&S43798[30], 6}, {NULL}};
const Segment SGI156330[] = {{&S41292[42], 10}, {NULL}};
const Segment SGI156331[] = {{&S41292[42], 22}, {NULL}};
const Segment SGI156332[] = {{&S41292[30], 4}, {NULL}};
const Segment SGI156333[] = {{&S43798[135], 3}, {NULL}};
const Segment SGI156334[] = {{&S43798[100], 6}, {NULL}};
const Segment SGI156335[] = {{&S41292[113], 3}, {NULL}};
const Segment SGI156336[] = {{&S43798[72], 8}, {NULL}};
const Segment SGI156337[] = {{&S41292[78], 6}, {NULL}};
const Segment SGI156338[] = {{&S43119[38], 8}, {NULL}};
const Segment SGI156339[] = {{&S43119[31], 6}, {NULL}};
const Segment SGI156340[] = {{&S43799[44], 14}, {NULL}};
const Segment SGI156341[] = {{&S43799[44], 26}, {NULL}};
const Segment SGI156342[] = {{&S43799[30], 6}, {NULL}};
const Segment SGI156343[] = {{&S43799[135], 3}, {NULL}};
const Segment SGI156344[] = {{&S43120[44], 7}, {NULL}};
const Segment SGI156345[] = {{&S43799[100], 6}, {NULL}};
const Segment SGI156346[] = {{&S43120[44], 27}, {NULL}};
const Segment SGI156347[] = {{&S43120[30], 6}, {NULL}};
const Segment SGI156348[] = {{&S43799[72], 8}, {NULL}};
const Segment SGI156349[] = {{&S43120[120], 3}, {NULL}};
const Segment SGI156350[] = {{&S43120[85], 6}, {NULL}};
const Segment SGI156351[] = {{&S41295[36], 10}, {NULL}};
const Segment SGI156352[] = {{&S41295[31], 4}, {NULL}};
const Segment SGI156353[] = {{&S43800[44], 14}, {NULL}};
const Segment SGI156354[] = {{&S43800[44], 26}, {NULL}};
const Segment SGI156355[] = {{&S43800[30], 6}, {NULL}};
const Segment SGI156356[] = {{&S43800[135], 3}, {NULL}};
const Segment SGI156357[] = {{&S43121[44], 10}, {NULL}};
const Segment SGI156358[] = {{&S43121[44], 30}, {NULL}};
const Segment SGI156359[] = {{&S43800[100], 6}, {NULL}};
const Segment SGI156360[] = {{&S43121[30], 6}, {NULL}};
const Segment SGI156361[] = {{&S43800[72], 8}, {NULL}};
const Segment SGI156362[] = {{&S43121[123], 3}, {NULL}};
const Segment SGI156363[] = {{&S43121[88], 6}, {NULL}};
const Segment SGI156364[] = {{&S43122[44], 7}, {NULL}};
const Segment SGI156365[] = {{&S43122[44], 27}, {NULL}};
const Segment SGI156366[] = {{&S43122[30], 6}, {NULL}};
const Segment SGI156367[] = {{&S43802[44], 16}, {NULL}};
const Segment SGI156368[] = {{&S43802[44], 28}, {NULL}};
const Segment SGI156369[] = {{&S43122[120], 3}, {NULL}};
const Segment SGI156370[] = {{&S43802[30], 6}, {NULL}};
const Segment SGI156371[] = {{&S43122[85], 6}, {NULL}};
const Segment SGI156372[] = {{&S43802[137], 3}, {NULL}};
const Segment SGI156373[] = {{&S41299[42], 10}, {NULL}};
const Segment SGI156374[] = {{&S43802[102], 6}, {NULL}};
const Segment SGI156375[] = {{&S41299[42], 22}, {NULL}};
const Segment SGI156376[] = {{&S41299[30], 4}, {NULL}};
const Segment SGI156377[] = {{&S43802[74], 8}, {NULL}};
const Segment SGI156378[] = {{&S41299[113], 3}, {NULL}};
const Segment SGI156379[] = {{&S41299[78], 6}, {NULL}};
const Segment SGI156380[] = {{&S43123[44], 11}, {NULL}};
const Segment SGI156381[] = {{&S43123[44], 23}, {NULL}};
const Segment SGI156382[] = {{&S43123[30], 6}, {NULL}};
const Segment SGI156383[] = {{&S43123[116], 3}, {NULL}};
const Segment SGI156384[] = {{&S43123[81], 6}, {NULL}};
const Segment SGI156385[] = {{&S43124[38], 11}, {NULL}};
const Segment SGI156386[] = {{&S43124[31], 6}, {NULL}};
const Segment SGI156387[] = {{&S43125[44], 30}, {NULL}};
const Segment SGI156388[] = {{&S43125[44], 42}, {NULL}};
const Segment SGI156389[] = {{&S43125[30], 6}, {NULL}};
const Segment SGI156390[] = {{&S43125[135], 3}, {NULL}};
const Segment SGI156391[] = {{&S43125[100], 6}, {NULL}};
const Segment SGI156392[] = {{&S43126[44], 30}, {NULL}};
const Segment SGI156393[] = {{&S43126[44], 42}, {NULL}};
const Segment SGI156394[] = {{&S43126[30], 6}, {NULL}};
const Segment SGI156395[] = {{&S43126[135], 3}, {NULL}};
const Segment SGI156396[] = {{&S43126[100], 6}, {NULL}};
const Segment SGI156397[] = {{&S43127[44], 31}, {NULL}};
const Segment SGI156398[] = {{&S43127[44], 43}, {NULL}};
const Segment SGI156399[] = {{&S43127[30], 6}, {NULL}};
const Segment SGI156400[] = {{&S43127[136], 3}, {NULL}};
const Segment SGI156401[] = {{&S43127[101], 6}, {NULL}};
const Segment SGI156402[] = {{&S43128[44], 31}, {NULL}};
const Segment SGI156403[] = {{&S43128[44], 43}, {NULL}};
const Segment SGI156404[] = {{&S43128[30], 6}, {NULL}};
const Segment SGI156405[] = {{&S43128[136], 3}, {NULL}};
const Segment SGI156406[] = {{&S43128[101], 6}, {NULL}};
const Segment SGI156407[] = {{&S38391[44], 14}, {NULL}};
const Segment SGI156408[] = {{&S38391[44], 26}, {NULL}};
const Segment SGI156409[] = {{&S38391[30], 6}, {NULL}};
const Segment SGI156410[] = {{&S38391[137], 3}, {NULL}};
const Segment SGI156411[] = {{&S38391[102], 6}, {NULL}};
const Segment SGI156412[] = {{&S38391[72], 8}, {NULL}};
const Segment SGI156413[] = {{&S38392[44], 14}, {NULL}};
const Segment SGI156414[] = {{&S38392[44], 26}, {NULL}};
const Segment SGI156415[] = {{&S38392[30], 6}, {NULL}};
const Segment SGI156416[] = {{&S38392[137], 3}, {NULL}};
const Segment SGI156417[] = {{&S38392[102], 6}, {NULL}};
const Segment SGI156418[] = {{&S38392[72], 8}, {NULL}};
const Segment SGI156419[] = {{&S43839[44], 26}, {NULL}};
const Segment SGI156420[] = {{&S43839[44], 38}, {NULL}};
const Segment SGI156421[] = {{&S43839[30], 6}, {NULL}};
const Segment SGI156422[] = {{&S43839[131], 3}, {NULL}};
const Segment SGI156423[] = {{&S43839[96], 6}, {NULL}};
const Segment SGI156424[] = {{&S43840[44], 10}, {NULL}};
const Segment SGI156425[] = {{&S43840[44], 22}, {NULL}};
const Segment SGI156426[] = {{&S43840[30], 6}, {NULL}};
const Segment SGI156427[] = {{&S43840[115], 3}, {NULL}};
const Segment SGI156428[] = {{&S43840[80], 6}, {NULL}};
const Segment SGI156429[] = {{&S43841[44], 18}, {NULL}};
const Segment SGI156430[] = {{&S43841[44], 30}, {NULL}};
const Segment SGI156431[] = {{&S43841[30], 6}, {NULL}};
const Segment SGI156432[] = {{&S43841[123], 3}, {NULL}};
const Segment SGI156433[] = {{&S43841[88], 6}, {NULL}};
const Segment SGI156434[] = {{&S43842[44], 18}, {NULL}};
const Segment SGI156435[] = {{&S43842[44], 30}, {NULL}};
const Segment SGI156436[] = {{&S43842[30], 6}, {NULL}};
const Segment SGI156437[] = {{&S43842[123], 3}, {NULL}};
const Segment SGI156438[] = {{&S43842[88], 6}, {NULL}};
const Segment SGI156439[] = {{&S43845[44], 18}, {NULL}};
const Segment SGI156440[] = {{&S43845[44], 30}, {NULL}};
const Segment SGI156441[] = {{&S43845[30], 6}, {NULL}};
const Segment SGI156442[] = {{&S43845[123], 3}, {NULL}};
const Segment SGI156443[] = {{&S43845[88], 6}, {NULL}};
const Segment SGI156444[] = {{&S43846[44], 10}, {NULL}};
const Segment SGI156445[] = {{&S43846[44], 22}, {NULL}};
const Segment SGI156446[] = {{&S43846[30], 6}, {NULL}};
const Segment SGI156447[] = {{&S43846[115], 3}, {NULL}};
const Segment SGI156448[] = {{&S43846[80], 6}, {NULL}};
const Segment SGI156449[] = {{&S43246[34], 7}, {NULL}};
const Segment SGI156450[] = {{&S43246[27], 6}, {NULL}};
const Segment SGI156451[] = {{&S43246[94], 9}, {NULL}};
const Segment SGI156452[] = {{&S43246[67], 6}, {NULL}};
const Segment SGI156453[] = {{&S43247[34], 11}, {NULL}};
const Segment SGI156454[] = {{&S43247[27], 6}, {NULL}};
const Segment SGI156455[] = {{&S43247[98], 9}, {NULL}};
const Segment SGI156456[] = {{&S43247[71], 6}, {NULL}};
const Segment SGI156457[] = {{&S43900[34], 29}, {NULL}};
const Segment SGI156458[] = {{&S43900[27], 6}, {NULL}};
const Segment SGI156459[] = {{&S43900[116], 9}, {NULL}};
const Segment SGI156460[] = {{&S43900[89], 6}, {NULL}};
const Segment SGI156461[] = {{&S43902[34], 30}, {NULL}};
const Segment SGI156462[] = {{&S43902[27], 6}, {NULL}};
const Segment SGI156463[] = {{&S43902[117], 9}, {NULL}};
const Segment SGI156464[] = {{&S43902[90], 6}, {NULL}};
const Segment SGI156465[] = {{&S43903[34], 17}, {NULL}};
const Segment SGI156466[] = {{&S43903[27], 6}, {NULL}};
const Segment SGI156467[] = {{&S43903[104], 9}, {NULL}};
const Segment SGI156468[] = {{&S43903[77], 6}, {NULL}};
const Segment SGI156469[] = {{&S43917[44], 15}, {NULL}};
const Segment SGI156470[] = {{&S43917[44], 27}, {NULL}};
const Segment SGI156471[] = {{&S43917[30], 6}, {NULL}};
const Segment SGI156472[] = {{&S43917[120], 3}, {NULL}};
const Segment SGI156473[] = {{&S43917[85], 6}, {NULL}};
const Segment SGI156474[] = {{&S43918[38], 15}, {NULL}};
const Segment SGI156475[] = {{&S43918[31], 6}, {NULL}};
const Segment SGI156476[] = {{&S53016[34], 10}, {NULL}};
const Segment SGI156477[] = {{&S53016[27], 6}, {NULL}};
const Segment SGI156478[] = {{&S53016[97], 9}, {NULL}};
const Segment SGI156479[] = {{&S53016[70], 6}, {NULL}};
const Segment SGI156480[] = {{&S43287[44], 7}, {NULL}};
const Segment SGI156481[] = {{&S43287[44], 25}, {NULL}};
const Segment SGI156482[] = {{&S43287[30], 6}, {NULL}};
const Segment SGI156483[] = {{&S53017[34], 7}, {NULL}};
const Segment SGI156484[] = {{&S43287[118], 3}, {NULL}};
const Segment SGI156485[] = {{&S53017[27], 6}, {NULL}};
const Segment SGI156486[] = {{&S43287[83], 6}, {NULL}};
const Segment SGI156487[] = {{&S53017[100], 9}, {NULL}};
const Segment SGI156488[] = {{&S53017[73], 6}, {NULL}};
const Segment SGI156489[] = {{&S43288[38], 7}, {NULL}};
const Segment SGI156490[] = {{&S43288[31], 6}, {NULL}};
const Segment SGI156491[] = {{&S53018[34], 13}, {NULL}};
const Segment SGI156492[] = {{&S53018[27], 6}, {NULL}};
const Segment SGI156493[] = {{&S53018[100], 9}, {NULL}};
const Segment SGI156494[] = {{&S53018[73], 6}, {NULL}};
const Segment SGI156495[] = {{&S43289[44], 11}, {NULL}};
const Segment SGI156496[] = {{&S43289[44], 29}, {NULL}};
const Segment SGI156497[] = {{&S43289[30], 6}, {NULL}};
const Segment SGI156498[] = {{&S43289[122], 3}, {NULL}};
const Segment SGI156499[] = {{&S43289[87], 6}, {NULL}};
const Segment SGI156500[] = {{&S43290[44], 7}, {NULL}};
const Segment SGI156501[] = {{&S43290[44], 25}, {NULL}};
const Segment SGI156502[] = {{&S43290[30], 6}, {NULL}};
const Segment SGI156503[] = {{&S43290[118], 3}, {NULL}};
const Segment SGI156504[] = {{&S43290[83], 6}, {NULL}};
const Segment SGI156505[] = {{&S43291[44], 11}, {NULL}};
const Segment SGI156506[] = {{&S43291[44], 29}, {NULL}};
const Segment SGI156507[] = {{&S43291[30], 6}, {NULL}};
const Segment SGI156508[] = {{&S43291[122], 3}, {NULL}};
const Segment SGI156509[] = {{&S43291[87], 6}, {NULL}};
const Segment SGI156510[] = {{&S43292[44], 11}, {NULL}};
const Segment SGI156511[] = {{&S43292[44], 29}, {NULL}};
const Segment SGI156512[] = {{&S43292[30], 6}, {NULL}};
const Segment SGI156513[] = {{&S43292[122], 3}, {NULL}};
const Segment SGI156514[] = {{&S43292[87], 6}, {NULL}};
const Segment SGI156515[] = {{&S43293[44], 7}, {NULL}};
const Segment SGI156516[] = {{&S43293[44], 25}, {NULL}};
const Segment SGI156517[] = {{&S43293[30], 6}, {NULL}};
const Segment SGI156518[] = {{&S43293[118], 3}, {NULL}};
const Segment SGI156519[] = {{&S43293[83], 6}, {NULL}};
const Segment SGI156520[] = {{&S43294[44], 11}, {NULL}};
const Segment SGI156521[] = {{&S43294[44], 29}, {NULL}};
const Segment SGI156522[] = {{&S43294[30], 6}, {NULL}};
const Segment SGI156523[] = {{&S43294[122], 3}, {NULL}};
const Segment SGI156524[] = {{&S43294[87], 6}, {NULL}};
const Segment SGI156525[] = {{&S43295[44], 11}, {NULL}};
const Segment SGI156526[] = {{&S43295[44], 29}, {NULL}};
const Segment SGI156527[] = {{&S43295[30], 6}, {NULL}};
const Segment SGI156528[] = {{&S43295[122], 3}, {NULL}};
const Segment SGI156529[] = {{&S43295[87], 6}, {NULL}};
const Segment SGI156530[] = {{&S43296[44], 7}, {NULL}};
const Segment SGI156531[] = {{&S43296[44], 25}, {NULL}};
const Segment SGI156532[] = {{&S43296[30], 6}, {NULL}};
const Segment SGI156533[] = {{&S43296[118], 3}, {NULL}};
const Segment SGI156534[] = {{&S43296[83], 6}, {NULL}};
const Segment SGI156535[] = {{&S43964[44], 5}, {NULL}};
const Segment SGI156536[] = {{&S43964[44], 17}, {NULL}};
const Segment SGI156537[] = {{&S43964[30], 6}, {NULL}};
const Segment SGI156538[] = {{&S43964[110], 3}, {NULL}};
const Segment SGI156539[] = {{&S43964[75], 6}, {NULL}};
const Segment SGI156540[] = {{&S43965[42], 5}, {NULL}};
const Segment SGI156541[] = {{&S43965[30], 6}, {NULL}};
const Segment SGI156542[] = {{&S43965[108], 3}, {NULL}};
const Segment SGI156543[] = {{&S43965[73], 6}, {NULL}};
const Segment SGI156544[] = {{&S43966[38], 5}, {NULL}};
const Segment SGI156545[] = {{&S43966[31], 6}, {NULL}};
const Segment SGI156546[] = {{&S53050[44], 7}, {NULL}};
const Segment SGI156547[] = {{&S53050[44], 19}, {NULL}};
const Segment SGI156548[] = {{&S53050[30], 6}, {NULL}};
const Segment SGI156549[] = {{&S53050[31], 3}, {NULL}};
const Segment SGI156550[] = {{&S53050[77], 6}, {NULL}};
const Segment SGI156551[] = {{&S43972[34], 10}, {NULL}};
const Segment SGI156552[] = {{&S43972[27], 6}, {NULL}};
const Segment SGI156553[] = {{&S43972[97], 9}, {NULL}};
const Segment SGI156554[] = {{&S43972[70], 6}, {NULL}};
const Segment SGI156555[] = {{&S43315[44], 10}, {NULL}};
const Segment SGI156556[] = {{&S43315[44], 22}, {NULL}};
const Segment SGI156557[] = {{&S43315[30], 6}, {NULL}};
const Segment SGI156558[] = {{&S43315[115], 3}, {NULL}};
const Segment SGI156559[] = {{&S43315[80], 6}, {NULL}};
const Segment SGI156560[] = {{&S43316[44], 10}, {NULL}};
const Segment SGI156561[] = {{&S43316[44], 22}, {NULL}};
const Segment SGI156562[] = {{&S43316[30], 6}, {NULL}};
const Segment SGI156563[] = {{&S43316[115], 3}, {NULL}};
const Segment SGI156564[] = {{&S43316[80], 6}, {NULL}};
const Segment SGI156565[] = {{&S43317[38], 10}, {NULL}};
const Segment SGI156566[] = {{&S43317[31], 6}, {NULL}};
const Segment SGI156567[] = {{&S43318[44], 10}, {NULL}};
const Segment SGI156568[] = {{&S43318[44], 22}, {NULL}};
const Segment SGI156569[] = {{&S43318[30], 6}, {NULL}};
const Segment SGI156570[] = {{&S43318[115], 3}, {NULL}};
const Segment SGI156571[] = {{&S43318[80], 6}, {NULL}};
const Segment SGI156572[] = {{&S43319[38], 10}, {NULL}};
const Segment SGI156573[] = {{&S43319[31], 6}, {NULL}};
const Segment SGI156574[] = {{&S43320[44], 15}, {NULL}};
const Segment SGI156575[] = {{&S43320[44], 27}, {NULL}};
const Segment SGI156576[] = {{&S43320[30], 6}, {NULL}};
const Segment SGI156577[] = {{&S43320[120], 3}, {NULL}};
const Segment SGI156578[] = {{&S43320[85], 6}, {NULL}};
const Segment SGI156579[] = {{&S43321[38], 10}, {NULL}};
const Segment SGI156580[] = {{&S43321[31], 6}, {NULL}};
const Segment SGI156581[] = {{&S43986[34], 10}, {NULL}};
const Segment SGI156582[] = {{&S43986[27], 6}, {NULL}};
const Segment SGI156583[] = {{&S43986[97], 9}, {NULL}};
const Segment SGI156584[] = {{&S43986[70], 6}, {NULL}};
const Segment SGI156585[] = {{&S43322[44], 15}, {NULL}};
const Segment SGI156586[] = {{&S43322[44], 27}, {NULL}};
const Segment SGI156587[] = {{&S43322[30], 6}, {NULL}};
const Segment SGI156588[] = {{&S43322[120], 3}, {NULL}};
const Segment SGI156589[] = {{&S43322[85], 6}, {NULL}};
const Segment SGI156590[] = {{&S43323[44], 15}, {NULL}};
const Segment SGI156591[] = {{&S43323[44], 27}, {NULL}};
const Segment SGI156592[] = {{&S43323[30], 6}, {NULL}};
const Segment SGI156593[] = {{&S43323[120], 3}, {NULL}};
const Segment SGI156594[] = {{&S43989[34], 10}, {NULL}};
const Segment SGI156595[] = {{&S43323[85], 6}, {NULL}};
const Segment SGI156596[] = {{&S43989[27], 6}, {NULL}};
const Segment SGI156597[] = {{&S43989[97], 9}, {NULL}};
const Segment SGI156598[] = {{&S43989[70], 6}, {NULL}};
const Segment SGI156599[] = {{&S43324[44], 10}, {NULL}};
const Segment SGI156600[] = {{&S43324[44], 22}, {NULL}};
const Segment SGI156601[] = {{&S43324[30], 6}, {NULL}};
const Segment SGI156602[] = {{&S43990[34], 10}, {NULL}};
const Segment SGI156603[] = {{&S43324[115], 3}, {NULL}};
const Segment SGI156604[] = {{&S43990[27], 6}, {NULL}};
const Segment SGI156605[] = {{&S43324[80], 6}, {NULL}};
const Segment SGI156606[] = {{&S43990[97], 9}, {NULL}};
const Segment SGI156607[] = {{&S43990[70], 6}, {NULL}};
const Segment SGI156608[] = {{&S43325[44], 15}, {NULL}};
const Segment SGI156609[] = {{&S43325[44], 27}, {NULL}};
const Segment SGI156610[] = {{&S43325[30], 6}, {NULL}};
const Segment SGI156611[] = {{&S43325[120], 3}, {NULL}};
const Segment SGI156612[] = {{&S43325[85], 6}, {NULL}};
const Segment SGI156613[] = {{&S53107[34], 14}, {NULL}};
const Segment SGI156614[] = {{&S53107[27], 6}, {NULL}};
const Segment SGI156615[] = {{&S53107[106], 9}, {NULL}};
const Segment SGI156616[] = {{&S53107[79], 6}, {NULL}};
const Segment SGI156617[] = {{&S53123[44], 4}, {NULL}};
const Segment SGI156618[] = {{&S53123[44], 16}, {NULL}};
const Segment SGI156619[] = {{&S53123[30], 6}, {NULL}};
const Segment SGI156620[] = {{&S53123[31], 3}, {NULL}};
const Segment SGI156621[] = {{&S53123[74], 6}, {NULL}};
const Segment SGI156622[] = {{&S53124[44], 12}, {NULL}};
const Segment SGI156623[] = {{&S53124[44], 24}, {NULL}};
const Segment SGI156624[] = {{&S53124[30], 6}, {NULL}};
const Segment SGI156625[] = {{&S53124[31], 3}, {NULL}};
const Segment SGI156626[] = {{&S53124[82], 6}, {NULL}};
const Segment SGI156627[] = {{&S53125[38], 4}, {NULL}};
const Segment SGI156628[] = {{&S53125[31], 6}, {NULL}};
const Segment SGI156629[] = {{&S44050[38], 12}, {NULL}};
const Segment SGI156630[] = {{&S44050[31], 6}, {NULL}};
const Segment SGI156631[] = {{&S44051[44], 12}, {NULL}};
const Segment SGI156632[] = {{&S44051[44], 24}, {NULL}};
const Segment SGI156633[] = {{&S44051[30], 6}, {NULL}};
const Segment SGI156634[] = {{&S44051[117], 3}, {NULL}};
const Segment SGI156635[] = {{&S44051[82], 6}, {NULL}};
const Segment SGI156636[] = {{&S44052[44], 12}, {NULL}};
const Segment SGI156637[] = {{&S44052[44], 24}, {NULL}};
const Segment SGI156638[] = {{&S44052[30], 6}, {NULL}};
const Segment SGI156639[] = {{&S44052[117], 3}, {NULL}};
const Segment SGI156640[] = {{&S44052[82], 6}, {NULL}};
const Segment SGI156641[] = {{&S44053[44], 23}, {NULL}};
const Segment SGI156642[] = {{&S44053[44], 35}, {NULL}};
const Segment SGI156643[] = {{&S44053[30], 6}, {NULL}};
const Segment SGI156644[] = {{&S44053[128], 3}, {NULL}};
const Segment SGI156645[] = {{&S44053[93], 6}, {NULL}};
const Segment SGI156646[] = {{&S44054[44], 23}, {NULL}};
const Segment SGI156647[] = {{&S44054[44], 35}, {NULL}};
const Segment SGI156648[] = {{&S44054[30], 6}, {NULL}};
const Segment SGI156649[] = {{&S44054[128], 3}, {NULL}};
const Segment SGI156650[] = {{&S44054[93], 6}, {NULL}};
const Segment SGI156651[] = {{&S44055[44], 23}, {NULL}};
const Segment SGI156652[] = {{&S44055[44], 35}, {NULL}};
const Segment SGI156653[] = {{&S44055[30], 6}, {NULL}};
const Segment SGI156654[] = {{&S44055[128], 3}, {NULL}};
const Segment SGI156655[] = {{&S44055[93], 6}, {NULL}};
const Segment SGI156656[] = {{&S44056[44], 23}, {NULL}};
const Segment SGI156657[] = {{&S44056[44], 35}, {NULL}};
const Segment SGI156658[] = {{&S44056[30], 6}, {NULL}};
const Segment SGI156659[] = {{&S44056[128], 3}, {NULL}};
const Segment SGI156660[] = {{&S44056[93], 6}, {NULL}};
const Segment SGI156661[] = {{&S44057[44], 23}, {NULL}};
const Segment SGI156662[] = {{&S44057[44], 35}, {NULL}};
const Segment SGI156663[] = {{&S44057[30], 6}, {NULL}};
const Segment SGI156664[] = {{&S44057[128], 3}, {NULL}};
const Segment SGI156665[] = {{&S44057[93], 6}, {NULL}};
const Segment SGI156666[] = {{&S44058[44], 23}, {NULL}};
const Segment SGI156667[] = {{&S44058[44], 35}, {NULL}};
const Segment SGI156668[] = {{&S44058[30], 6}, {NULL}};
const Segment SGI156669[] = {{&S44058[128], 3}, {NULL}};
const Segment SGI156670[] = {{&S44058[93], 6}, {NULL}};
const Segment SGI156671[] = {{&S44060[44], 13}, {NULL}};
const Segment SGI156672[] = {{&S44060[44], 25}, {NULL}};
const Segment SGI156673[] = {{&S44060[30], 6}, {NULL}};
const Segment SGI156674[] = {{&S44060[118], 3}, {NULL}};
const Segment SGI156675[] = {{&S44060[83], 6}, {NULL}};
const Segment SGI156676[] = {{&S44062[44], 12}, {NULL}};
const Segment SGI156677[] = {{&S44062[44], 24}, {NULL}};
const Segment SGI156678[] = {{&S44062[30], 6}, {NULL}};
const Segment SGI156679[] = {{&S44062[117], 3}, {NULL}};
const Segment SGI156680[] = {{&S44062[82], 6}, {NULL}};
const Segment SGI156681[] = {{&S44063[38], 12}, {NULL}};
const Segment SGI156682[] = {{&S44063[31], 6}, {NULL}};
const Segment SGI156683[] = {{&S53170[34], 11}, {NULL}};
const Segment SGI156684[] = {{&S53170[27], 6}, {NULL}};
const Segment SGI156685[] = {{&S53170[98], 9}, {NULL}};
const Segment SGI156686[] = {{&S53170[71], 6}, {NULL}};
const Segment SGI156687[] = {{&S53171[34], 11}, {NULL}};
const Segment SGI156688[] = {{&S53171[27], 6}, {NULL}};
const Segment SGI156689[] = {{&S53171[98], 9}, {NULL}};
const Segment SGI156690[] = {{&S53171[71], 6}, {NULL}};
const Segment SGI156691[] = {{&S44094[34], 10}, {NULL}};
const Segment SGI156692[] = {{&S44094[27], 6}, {NULL}};
const Segment SGI156693[] = {{&S44094[97], 9}, {NULL}};
const Segment SGI156694[] = {{&S44094[70], 6}, {NULL}};
const Segment SGI156695[] = {{&S53231[34], 13}, {NULL}};
const Segment SGI156696[] = {{&S53231[27], 6}, {NULL}};
const Segment SGI156697[] = {{&S53231[104], 9}, {NULL}};
const Segment SGI156698[] = {{&S53231[77], 6}, {NULL}};
const Segment SGI156699[] = {{&S43326[44], 20}, {NULL}};
const Segment SGI156700[] = {{&S43326[44], 32}, {NULL}};
const Segment SGI156701[] = {{&S43326[30], 6}, {NULL}};
const Segment SGI156702[] = {{&S43326[125], 3}, {NULL}};
const Segment SGI156703[] = {{&S43326[90], 6}, {NULL}};
const Segment SGI156704[] = {{&S43327[44], 20}, {NULL}};
const Segment SGI156705[] = {{&S43327[44], 32}, {NULL}};
const Segment SGI156706[] = {{&S43327[30], 6}, {NULL}};
const Segment SGI156707[] = {{&S43327[125], 3}, {NULL}};
const Segment SGI156708[] = {{&S43327[90], 6}, {NULL}};
const Segment SGI156709[] = {{&S43328[44], 10}, {NULL}};
const Segment SGI156710[] = {{&S43328[44], 22}, {NULL}};
const Segment SGI156711[] = {{&S43328[30], 6}, {NULL}};
const Segment SGI156712[] = {{&S43328[115], 3}, {NULL}};
const Segment SGI156713[] = {{&S43328[80], 6}, {NULL}};
const Segment SGI156714[] = {{&S43329[44], 19}, {NULL}};
const Segment SGI156715[] = {{&S43329[44], 31}, {NULL}};
const Segment SGI156716[] = {{&S43329[30], 6}, {NULL}};
const Segment SGI156717[] = {{&S43329[124], 3}, {NULL}};
const Segment SGI156718[] = {{&S43329[89], 6}, {NULL}};
const Segment SGI156719[] = {{&S43330[42], 10}, {NULL}};
const Segment SGI156720[] = {{&S43330[30], 6}, {NULL}};
const Segment SGI156721[] = {{&S43330[113], 3}, {NULL}};
const Segment SGI156722[] = {{&S43330[78], 6}, {NULL}};
const Segment SGI156723[] = {{&S38770[34], 9}, {NULL}};
const Segment SGI156724[] = {{&S38770[27], 6}, {NULL}};
const Segment SGI156725[] = {{&S38770[96], 9}, {NULL}};
const Segment SGI156726[] = {{&S38770[69], 6}, {NULL}};
const Segment SGI156727[] = {{&S53259[44], 14}, {NULL}};
const Segment SGI156728[] = {{&S53259[44], 26}, {NULL}};
const Segment SGI156729[] = {{&S53259[30], 6}, {NULL}};
const Segment SGI156730[] = {{&S53259[31], 3}, {NULL}};
const Segment SGI156731[] = {{&S53259[103], 6}, {NULL}};
const Segment SGI156732[] = {{&S53259[72], 8}, {NULL}};
const Segment SGI156733[] = {{&S38781[44], 11}, {NULL}};
const Segment SGI156734[] = {{&S38781[44], 23}, {NULL}};
const Segment SGI156735[] = {{&S38781[30], 6}, {NULL}};
const Segment SGI156736[] = {{&S38781[116], 3}, {NULL}};
const Segment SGI156737[] = {{&S38781[81], 6}, {NULL}};
const Segment SGI156738[] = {{&S53260[44], 14}, {NULL}};
const Segment SGI156739[] = {{&S53260[44], 26}, {NULL}};
const Segment SGI156740[] = {{&S53260[30], 6}, {NULL}};
const Segment SGI156741[] = {{&S53260[31], 3}, {NULL}};
const Segment SGI156742[] = {{&S53260[103], 6}, {NULL}};
const Segment SGI156743[] = {{&S53260[72], 8}, {NULL}};
const Segment SGI156744[] = {{&S53261[44], 14}, {NULL}};
const Segment SGI156745[] = {{&S53261[44], 26}, {NULL}};
const Segment SGI156746[] = {{&S53261[30], 6}, {NULL}};
const Segment SGI156747[] = {{&S53261[31], 3}, {NULL}};
const Segment SGI156748[] = {{&S53261[103], 6}, {NULL}};
const Segment SGI156749[] = {{&S53261[72], 8}, {NULL}};
const Segment SGI156750[] = {{&S53262[44], 14}, {NULL}};
const Segment SGI156751[] = {{&S38786[44], 11}, {NULL}};
const Segment SGI156752[] = {{&S38786[44], 23}, {NULL}};
const Segment SGI156753[] = {{&S53262[44], 29}, {&S53262[74], 18}, {NULL}};
const Segment SGI156754[] = {{&S38786[30], 6}, {NULL}};
const Segment SGI156755[] = {{&S53262[30], 6}, {NULL}};
const Segment SGI156756[] = {{&S38786[116], 3}, {NULL}};
const Segment SGI156757[] = {{&S53262[31], 3}, {NULL}};
const Segment SGI156758[] = {{&S38786[81], 6}, {NULL}};
const Segment SGI156759[] = {{&S53262[106], 6}, {NULL}};
const Segment SGI156760[] = {{&S53262[75], 8}, {NULL}};
const Segment SGI156761[] = {{&S53263[44], 14}, {NULL}};
const Segment SGI156762[] = {{&S53263[44], 26}, {NULL}};
const Segment SGI156763[] = {{&S53263[30], 6}, {NULL}};
const Segment SGI156764[] = {{&S53263[31], 3}, {NULL}};
const Segment SGI156765[] = {{&S53263[110], 6}, {NULL}};
const Segment SGI156766[] = {{&S53263[72], 8}, {NULL}};
const Segment SGI156767[] = {{&S53278[34], 10}, {NULL}};
const Segment SGI156768[] = {{&S53278[27], 6}, {NULL}};
const Segment SGI156769[] = {{&S53278[97], 9}, {NULL}};
const Segment SGI156770[] = {{&S53278[70], 6}, {NULL}};
const Segment SGI156771[] = {{&S43369[34], 10}, {NULL}};
const Segment SGI156772[] = {{&S43369[27], 6}, {NULL}};
const Segment SGI156773[] = {{&S43369[97], 9}, {NULL}};
const Segment SGI156774[] = {{&S43369[70], 6}, {NULL}};
const Segment SGI156775[] = {{&S43371[34], 10}, {NULL}};
const Segment SGI156776[] = {{&S43371[27], 6}, {NULL}};
const Segment SGI156777[] = {{&S43371[97], 9}, {NULL}};
const Segment SGI156778[] = {{&S43371[70], 6}, {NULL}};
const Segment SGI156779[] = {{&S53311[34], 10}, {NULL}};
const Segment SGI156780[] = {{&S53311[27], 6}, {NULL}};
const Segment SGI156781[] = {{&S53311[97], 9}, {NULL}};
const Segment SGI156782[] = {{&S53311[70], 6}, {NULL}};
const Segment SGI156783[] = {{&S43407[44], 10}, {NULL}};
const Segment SGI156784[] = {{&S43407[44], 22}, {NULL}};
const Segment SGI156785[] = {{&S43407[30], 6}, {NULL}};
const Segment SGI156786[] = {{&S43407[115], 3}, {NULL}};
const Segment SGI156787[] = {{&S43407[80], 6}, {NULL}};
const Segment SGI156788[] = {{&S44199[34], 10}, {NULL}};
const Segment SGI156789[] = {{&S44199[27], 6}, {NULL}};
const Segment SGI156790[] = {{&S44199[97], 9}, {NULL}};
const Segment SGI156791[] = {{&S44199[70], 6}, {NULL}};
const Segment SGI156792[] = {{&S43408[38], 10}, {NULL}};
const Segment SGI156793[] = {{&S43408[31], 6}, {NULL}};
const Segment SGI156794[] = {{&S44199[138], 8}, {NULL}};
const Segment SGI156795[] = {{&S43409[44], 11}, {NULL}};
const Segment SGI156796[] = {{&S43409[44], 23}, {NULL}};
const Segment SGI156797[] = {{&S43409[30], 6}, {NULL}};
const Segment SGI156798[] = {{&S44201[34], 10}, {NULL}};
const Segment SGI156799[] = {{&S43409[116], 3}, {NULL}};
const Segment SGI156800[] = {{&S44201[27], 6}, {NULL}};
const Segment SGI156801[] = {{&S43409[81], 6}, {NULL}};
const Segment SGI156802[] = {{&S44201[97], 9}, {NULL}};
const Segment SGI156803[] = {{&S44201[70], 6}, {NULL}};
const Segment SGI156804[] = {{&S44201[138], 8}, {NULL}};
const Segment SGI156805[] = {{&S43410[44], 30}, {NULL}};
const Segment SGI156806[] = {{&S43410[44], 42}, {NULL}};
const Segment SGI156807[] = {{&S43410[30], 6}, {NULL}};
const Segment SGI156808[] = {{&S43410[135], 3}, {NULL}};
const Segment SGI156809[] = {{&S43410[100], 6}, {NULL}};
const Segment SGI156810[] = {{&S44203[34], 11}, {NULL}};
const Segment SGI156811[] = {{&S44203[27], 6}, {NULL}};
const Segment SGI156812[] = {{&S44203[98], 9}, {NULL}};
const Segment SGI156813[] = {{&S44203[71], 6}, {NULL}};
const Segment SGI156814[] = {{&S44203[139], 8}, {NULL}};
const Segment SGI156815[] = {{&S44204[34], 10}, {NULL}};
const Segment SGI156816[] = {{&S44204[27], 6}, {NULL}};
const Segment SGI156817[] = {{&S44204[97], 9}, {NULL}};
const Segment SGI156818[] = {{&S44204[70], 6}, {NULL}};
const Segment SGI156819[] = {{&S44204[138], 8}, {NULL}};
const Segment SGI156820[] = {{&S53355[34], 10}, {NULL}};
const Segment SGI156821[] = {{&S53355[27], 6}, {NULL}};
const Segment SGI156822[] = {{&S53355[97], 9}, {NULL}};
const Segment SGI156823[] = {{&S53355[70], 6}, {NULL}};
const Segment SGI156824[] = {{&S43439[34], 12}, {NULL}};
const Segment SGI156825[] = {{&S53356[34], 9}, {NULL}};
const Segment SGI156826[] = {{&S43439[27], 6}, {NULL}};
const Segment SGI156827[] = {{&S53356[27], 6}, {NULL}};
const Segment SGI156828[] = {{&S43439[109], 9}, {NULL}};
const Segment SGI156829[] = {{&S53356[96], 9}, {NULL}};
const Segment SGI156830[] = {{&S43439[82], 6}, {NULL}};
const Segment SGI156831[] = {{&S53356[69], 6}, {NULL}};
const Segment SGI156832[] = {{&S43440[34], 12}, {NULL}};
const Segment SGI156833[] = {{&S43440[27], 6}, {NULL}};
const Segment SGI156834[] = {{&S43440[109], 9}, {NULL}};
const Segment SGI156835[] = {{&S43440[82], 6}, {NULL}};
const Segment SGI156836[] = {{&S43443[34], 11}, {NULL}};
const Segment SGI156837[] = {{&S43443[27], 6}, {NULL}};
const Segment SGI156838[] = {{&S43443[98], 9}, {NULL}};
const Segment SGI156839[] = {{&S43443[71], 6}, {NULL}};
const Segment SGI156840[] = {{&S53387[34], 10}, {NULL}};
const Segment SGI156841[] = {{&S53387[27], 6}, {NULL}};
const Segment SGI156842[] = {{&S53387[97], 9}, {NULL}};
const Segment SGI156843[] = {{&S53387[70], 6}, {NULL}};
const Segment SGI156844[] = {{&S53389[44], 18}, {NULL}};
const Segment SGI156845[] = {{&S53389[44], 43}, {NULL}};
const Segment SGI156846[] = {{&S53389[30], 6}, {NULL}};
const Segment SGI156847[] = {{&S53389[31], 3}, {NULL}};
const Segment SGI156848[] = {{&S53389[101], 6}, {NULL}};
const Segment SGI156849[] = {{&S53390[44], 15}, {NULL}};
const Segment SGI156850[] = {{&S53390[44], 40}, {NULL}};
const Segment SGI156851[] = {{&S53390[30], 6}, {NULL}};
const Segment SGI156852[] = {{&S53390[31], 3}, {NULL}};
const Segment SGI156853[] = {{&S53390[98], 6}, {NULL}};
const Segment SGI156854[] = {{&S53391[44], 18}, {NULL}};
const Segment SGI156855[] = {{&S53391[44], 43}, {NULL}};
const Segment SGI156856[] = {{&S53391[30], 6}, {NULL}};
const Segment SGI156857[] = {{&S53391[31], 3}, {NULL}};
const Segment SGI156858[] = {{&S53391[101], 6}, {NULL}};
const Segment SGI156859[] = {{&S53392[44], 15}, {NULL}};
const Segment SGI156860[] = {{&S53392[44], 30}, {NULL}};
const Segment SGI156861[] = {{&S53392[30], 6}, {NULL}};
const Segment SGI156862[] = {{&S53392[31], 3}, {NULL}};
const Segment SGI156863[] = {{&S53392[88], 6}, {NULL}};
const Segment SGI156864[] = {{&S53393[44], 18}, {NULL}};
const Segment SGI156865[] = {{&S53393[44], 43}, {NULL}};
const Segment SGI156866[] = {{&S53393[30], 6}, {NULL}};
const Segment SGI156867[] = {{&S53393[31], 3}, {NULL}};
const Segment SGI156868[] = {{&S53393[101], 6}, {NULL}};
const Segment SGI156869[] = {{&S53394[44], 15}, {NULL}};
const Segment SGI156870[] = {{&S53394[44], 41}, {NULL}};
const Segment SGI156871[] = {{&S53394[30], 6}, {NULL}};
const Segment SGI156872[] = {{&S53394[31], 3}, {NULL}};
const Segment SGI156873[] = {{&S53394[99], 6}, {NULL}};
const Segment SGI156874[] = {{&S53395[44], 15}, {NULL}};
const Segment SGI156875[] = {{&S53395[44], 43}, {NULL}};
const Segment SGI156876[] = {{&S53395[30], 6}, {NULL}};
const Segment SGI156877[] = {{&S53395[31], 3}, {NULL}};
const Segment SGI156878[] = {{&S53395[101], 6}, {NULL}};
const Segment SGI156879[] = {{&S44283[44], 6}, {NULL}};
const Segment SGI156880[] = {{&S44283[44], 18}, {NULL}};
const Segment SGI156881[] = {{&S44283[30], 6}, {NULL}};
const Segment SGI156882[] = {{&S44283[111], 3}, {NULL}};
const Segment SGI156883[] = {{&S44283[76], 6}, {NULL}};
const Segment SGI156884[] = {{&S53396[44], 15}, {NULL}};
const Segment SGI156885[] = {{&S53396[44], 43}, {NULL}};
const Segment SGI156886[] = {{&S53396[30], 6}, {NULL}};
const Segment SGI156887[] = {{&S53396[31], 3}, {NULL}};
const Segment SGI156888[] = {{&S53396[101], 6}, {NULL}};
const Segment SGI156889[] = {{&S53397[44], 18}, {NULL}};
const Segment SGI156890[] = {{&S53397[44], 34}, {NULL}};
const Segment SGI156891[] = {{&S53397[30], 6}, {NULL}};
const Segment SGI156892[] = {{&S53397[31], 3}, {NULL}};
const Segment SGI156893[] = {{&S53397[92], 6}, {NULL}};
const Segment SGI156894[] = {{&S44286[44], 15}, {NULL}};
const Segment SGI156895[] = {{&S44286[44], 27}, {NULL}};
const Segment SGI156896[] = {{&S44286[30], 6}, {NULL}};
const Segment SGI156897[] = {{&S44286[120], 3}, {NULL}};
const Segment SGI156898[] = {{&S44286[85], 6}, {NULL}};
const Segment SGI156899[] = {{&S53398[44], 7}, {NULL}};
const Segment SGI156900[] = {{&S53398[44], 36}, {NULL}};
const Segment SGI156901[] = {{&S53398[30], 6}, {NULL}};
const Segment SGI156902[] = {{&S53398[31], 3}, {NULL}};
const Segment SGI156903[] = {{&S53398[94], 6}, {NULL}};
const Segment SGI156904[] = {{&S44287[44], 24}, {NULL}};
const Segment SGI156905[] = {{&S44287[30], 6}, {NULL}};
const Segment SGI156906[] = {{&S44287[131], 3}, {NULL}};
const Segment SGI156907[] = {{&S44287[96], 6}, {NULL}};
const Segment SGI156908[] = {{&S53399[44], 15}, {NULL}};
const Segment SGI156909[] = {{&S53399[44], 44}, {NULL}};
const Segment SGI156910[] = {{&S53399[30], 6}, {NULL}};
const Segment SGI156911[] = {{&S43495[34], 25}, {NULL}};
const Segment SGI156912[] = {{&S44288[38], 6}, {NULL}};
const Segment SGI156913[] = {{&S44288[31], 6}, {NULL}};
const Segment SGI156914[] = {{&S43495[27], 6}, {NULL}};
const Segment SGI156915[] = {{&S53399[31], 3}, {NULL}};
const Segment SGI156916[] = {{&S53399[102], 6}, {NULL}};
const Segment SGI156917[] = {{&S43495[112], 9}, {NULL}};
const Segment SGI156918[] = {{&S43495[85], 6}, {NULL}};
const Segment SGI156919[] = {{&S44289[38], 15}, {NULL}};
const Segment SGI156920[] = {{&S44289[31], 6}, {NULL}};
const Segment SGI156921[] = {{&S53400[38], 15}, {NULL}};
const Segment SGI156922[] = {{&S53400[31], 6}, {NULL}};
const Segment SGI156923[] = {{&S53401[44], 7}, {NULL}};
const Segment SGI156924[] = {{&S53401[44], 35}, {NULL}};
const Segment SGI156925[] = {{&S53401[30], 6}, {NULL}};
const Segment SGI156926[] = {{&S53401[31], 3}, {NULL}};
const Segment SGI156927[] = {{&S44291[44], 12}, {NULL}};
const Segment SGI156928[] = {{&S44291[44], 24}, {NULL}};
const Segment SGI156929[] = {{&S53401[93], 6}, {NULL}};
const Segment SGI156930[] = {{&S44291[30], 6}, {NULL}};
const Segment SGI156931[] = {{&S44291[117], 3}, {NULL}};
const Segment SGI156932[] = {{&S44291[82], 6}, {NULL}};
const Segment SGI156933[] = {{&S53402[44], 7}, {NULL}};
const Segment SGI156934[] = {{&S53402[44], 33}, {NULL}};
const Segment SGI156935[] = {{&S53402[30], 6}, {NULL}};
const Segment SGI156936[] = {{&S44292[44], 15}, {NULL}};
const Segment SGI156937[] = {{&S44292[44], 27}, {NULL}};
const Segment SGI156938[] = {{&S53402[31], 3}, {NULL}};
const Segment SGI156939[] = {{&S44292[30], 6}, {NULL}};
const Segment SGI156940[] = {{&S53402[91], 6}, {NULL}};
const Segment SGI156941[] = {{&S44292[120], 3}, {NULL}};
const Segment SGI156942[] = {{&S44292[85], 6}, {NULL}};
const Segment SGI156943[] = {{&S53403[44], 7}, {NULL}};
const Segment SGI156944[] = {{&S53403[44], 39}, {NULL}};
const Segment SGI156945[] = {{&S53403[30], 6}, {NULL}};
const Segment SGI156946[] = {{&S53403[31], 3}, {NULL}};
const Segment SGI156947[] = {{&S53403[97], 6}, {NULL}};
const Segment SGI156948[] = {{&S44295[42], 15}, {NULL}};
const Segment SGI156949[] = {{&S53404[44], 7}, {NULL}};
const Segment SGI156950[] = {{&S44295[30], 6}, {NULL}};
const Segment SGI156951[] = {{&S53404[44], 33}, {NULL}};
const Segment SGI156952[] = {{&S53404[30], 6}, {NULL}};
const Segment SGI156953[] = {{&S44295[118], 3}, {NULL}};
const Segment SGI156954[] = {{&S53404[31], 3}, {NULL}};
const Segment SGI156955[] = {{&S44295[83], 6}, {NULL}};
const Segment SGI156956[] = {{&S53404[91], 6}, {NULL}};
const Segment SGI156957[] = {{&S53405[38], 7}, {NULL}};
const Segment SGI156958[] = {{&S53405[31], 6}, {NULL}};
const Segment SGI156959[] = {{&S53406[44], 7}, {NULL}};
const Segment SGI156960[] = {{&S53406[44], 38}, {NULL}};
const Segment SGI156961[] = {{&S53406[30], 6}, {NULL}};
const Segment SGI156962[] = {{&S53406[31], 3}, {NULL}};
const Segment SGI156963[] = {{&S53406[96], 6}, {NULL}};
const Segment SGI156964[] = {{&S53407[44], 7}, {NULL}};
const Segment SGI156965[] = {{&S53407[44], 33}, {NULL}};
const Segment SGI156966[] = {{&S53407[30], 6}, {NULL}};
const Segment SGI156967[] = {{&S53407[31], 3}, {NULL}};
const Segment SGI156968[] = {{&S53407[91], 6}, {NULL}};
const Segment SGI156969[] = {{&S53408[37], 23}, {NULL}};
const Segment SGI156970[] = {{&S53414[44], 13}, {NULL}};
const Segment SGI156971[] = {{&S53414[44], 25}, {NULL}};
const Segment SGI156972[] = {{&S53414[30], 6}, {NULL}};
const Segment SGI156973[] = {{&S53414[31], 3}, {NULL}};
const Segment SGI156974[] = {{&S53414[83], 6}, {NULL}};
const Segment SGI156975[] = {{&S53417[34], 11}, {NULL}};
const Segment SGI156976[] = {{&S53417[27], 6}, {NULL}};
const Segment SGI156977[] = {{&S53417[98], 9}, {NULL}};
const Segment SGI156978[] = {{&S53417[71], 6}, {NULL}};
const Segment SGI156979[] = {{&S44313[34], 11}, {NULL}};
const Segment SGI156980[] = {{&S44313[27], 6}, {NULL}};
const Segment SGI156981[] = {{&S44313[98], 9}, {NULL}};
const Segment SGI156982[] = {{&S44313[71], 6}, {NULL}};
const Segment SGI156983[] = {{&S44315[34], 11}, {NULL}};
const Segment SGI156984[] = {{&S44315[27], 6}, {NULL}};
const Segment SGI156985[] = {{&S44315[98], 9}, {NULL}};
const Segment SGI156986[] = {{&S44315[71], 6}, {NULL}};
const Segment SGI156987[] = {{&S43531[44], 10}, {NULL}};
const Segment SGI156988[] = {{&S43531[44], 22}, {NULL}};
const Segment SGI156989[] = {{&S43531[30], 6}, {NULL}};
const Segment SGI156990[] = {{&S43531[115], 3}, {NULL}};
const Segment SGI156991[] = {{&S43531[80], 6}, {NULL}};
const Segment SGI156992[] = {{&S43532[38], 10}, {NULL}};
const Segment SGI156993[] = {{&S43532[31], 6}, {NULL}};
const Segment SGI156994[] = {{&S53427[44], 10}, {NULL}};
const Segment SGI156995[] = {{&S53427[44], 22}, {NULL}};
const Segment SGI156996[] = {{&S53427[30], 6}, {NULL}};
const Segment SGI156997[] = {{&S53427[31], 3}, {NULL}};
const Segment SGI156998[] = {{&S53427[80], 6}, {NULL}};
const Segment SGI156999[] = {{&S49599[83], 5}, {NULL}};
const Segment SGI157000[] = {{&S43534[44], 10}, {NULL}};
const Segment SGI157001[] = {{&S43534[44], 22}, {NULL}};
const Segment SGI157002[] = {{&S43534[30], 6}, {NULL}};
const Segment SGI157003[] = {{&S43534[115], 3}, {NULL}};
const Segment SGI157004[] = {{&S53428[38], 10}, {NULL}};
const Segment SGI157005[] = {{&S53428[31], 6}, {NULL}};
const Segment SGI157006[] = {{&S43534[80], 6}, {NULL}};
const Segment SGI157007[] = {{&S43535[44], 11}, {NULL}};
const Segment SGI157008[] = {{&S43535[44], 23}, {NULL}};
const Segment SGI157009[] = {{&S43535[30], 6}, {NULL}};
const Segment SGI157010[] = {{&S53429[34], 10}, {NULL}};
const Segment SGI157011[] = {{&S53429[27], 6}, {NULL}};
const Segment SGI157012[] = {{&S43535[116], 3}, {NULL}};
const Segment SGI157013[] = {{&S43535[81], 6}, {NULL}};
const Segment SGI157014[] = {{&S53429[97], 9}, {NULL}};
const Segment SGI157015[] = {{&S53429[70], 6}, {NULL}};
const Segment SGI157016[] = {{&S43536[44], 11}, {NULL}};
const Segment SGI157017[] = {{&S53430[34], 10}, {NULL}};
const Segment SGI157018[] = {{&S43536[44], 23}, {NULL}};
const Segment SGI157019[] = {{&S43536[30], 6}, {NULL}};
const Segment SGI157020[] = {{&S53430[27], 6}, {NULL}};
const Segment SGI157021[] = {{&S43536[116], 3}, {NULL}};
const Segment SGI157022[] = {{&S53430[97], 9}, {NULL}};
const Segment SGI157023[] = {{&S43536[81], 6}, {NULL}};
const Segment SGI157024[] = {{&S53430[70], 6}, {NULL}};
const Segment SGI157025[] = {{&S43537[44], 11}, {NULL}};
const Segment SGI157026[] = {{&S43537[44], 23}, {NULL}};
const Segment SGI157027[] = {{&S43537[30], 6}, {NULL}};
const Segment SGI157028[] = {{&S53431[34], 18}, {NULL}};
const Segment SGI157029[] = {{&S53431[27], 6}, {NULL}};
const Segment SGI157030[] = {{&S43537[116], 3}, {NULL}};
const Segment SGI157031[] = {{&S43537[81], 6}, {NULL}};
const Segment SGI157032[] = {{&S53431[105], 9}, {NULL}};
const Segment SGI157033[] = {{&S53431[78], 6}, {NULL}};
const Segment SGI157034[] = {{&S43538[44], 30}, {NULL}};
const Segment SGI157035[] = {{&S43538[44], 42}, {NULL}};
const Segment SGI157036[] = {{&S43538[30], 6}, {NULL}};
const Segment SGI157037[] = {{&S43538[135], 3}, {NULL}};
const Segment SGI157038[] = {{&S43538[100], 6}, {NULL}};
const Segment SGI157039[] = {{&S53433[34], 10}, {NULL}};
const Segment SGI157040[] = {{&S53433[27], 6}, {NULL}};
const Segment SGI157041[] = {{&S53433[97], 9}, {NULL}};
const Segment SGI157042[] = {{&S53433[70], 6}, {NULL}};
const Segment SGI157043[] = {{&S43539[38], 10}, {NULL}};
const Segment SGI157044[] = {{&S43539[31], 6}, {NULL}};
const Segment SGI157045[] = {{&S53434[34], 10}, {NULL}};
const Segment SGI157046[] = {{&S53434[27], 6}, {NULL}};
const Segment SGI157047[] = {{&S43540[44], 32}, {NULL}};
const Segment SGI157048[] = {{&S43540[44], 44}, {NULL}};
const Segment SGI157049[] = {{&S53434[97], 9}, {NULL}};
const Segment SGI157050[] = {{&S43540[30], 6}, {NULL}};
const Segment SGI157051[] = {{&S53434[70], 6}, {NULL}};
const Segment SGI157052[] = {{&S43540[137], 3}, {NULL}};
const Segment SGI157053[] = {{&S43540[102], 6}, {NULL}};
const Segment SGI157054[] = {{&S43541[44], 30}, {NULL}};
const Segment SGI157055[] = {{&S43541[44], 42}, {NULL}};
const Segment SGI157056[] = {{&S43541[30], 6}, {NULL}};
const Segment SGI157057[] = {{&S43541[135], 3}, {NULL}};
const Segment SGI157058[] = {{&S43541[100], 6}, {NULL}};
const Segment SGI157059[] = {{&S43542[38], 11}, {NULL}};
const Segment SGI157060[] = {{&S43542[31], 6}, {NULL}};
const Segment SGI157061[] = {{&S44343[34], 10}, {NULL}};
const Segment SGI157062[] = {{&S44343[27], 6}, {NULL}};
const Segment SGI157063[] = {{&S44343[97], 9}, {NULL}};
const Segment SGI157064[] = {{&S44343[70], 6}, {NULL}};
const Segment SGI157065[] = {{&S43543[44], 11}, {NULL}};
const Segment SGI157066[] = {{&S43543[44], 23}, {NULL}};
const Segment SGI157067[] = {{&S43543[30], 6}, {NULL}};
const Segment SGI157068[] = {{&S43543[116], 3}, {NULL}};
const Segment SGI157069[] = {{&S43543[81], 6}, {NULL}};
const Segment SGI157070[] = {{&S43544[44], 32}, {NULL}};
const Segment SGI157071[] = {{&S43544[44], 44}, {NULL}};
const Segment SGI157072[] = {{&S43544[30], 6}, {NULL}};
const Segment SGI157073[] = {{&S43544[137], 3}, {NULL}};
const Segment SGI157074[] = {{&S43544[102], 6}, {NULL}};
const Segment SGI157075[] = {{&S43545[44], 31}, {NULL}};
const Segment SGI157076[] = {{&S43545[44], 43}, {NULL}};
const Segment SGI157077[] = {{&S43545[30], 6}, {NULL}};
const Segment SGI157078[] = {{&S43545[136], 3}, {NULL}};
const Segment SGI157079[] = {{&S43545[101], 6}, {NULL}};
const Segment SGI157080[] = {{&S43546[38], 11}, {NULL}};
const Segment SGI157081[] = {{&S43546[31], 6}, {NULL}};
const Segment SGI157082[] = {{&S43547[44], 31}, {NULL}};
const Segment SGI157083[] = {{&S43547[44], 43}, {NULL}};
const Segment SGI157084[] = {{&S43547[30], 6}, {NULL}};
const Segment SGI157085[] = {{&S43547[136], 3}, {NULL}};
const Segment SGI157086[] = {{&S43547[101], 6}, {NULL}};
const Segment SGI157087[] = {{&S43548[38], 11}, {NULL}};
const Segment SGI157088[] = {{&S43548[31], 6}, {NULL}};
const Segment SGI157089[] = {{&S43549[44], 31}, {NULL}};
const Segment SGI157090[] = {{&S43549[44], 43}, {NULL}};
const Segment SGI157091[] = {{&S43549[30], 6}, {NULL}};
const Segment SGI157092[] = {{&S53452[34], 11}, {NULL}};
const Segment SGI157093[] = {{&S53452[27], 6}, {NULL}};
const Segment SGI157094[] = {{&S43549[136], 3}, {NULL}};
const Segment SGI157095[] = {{&S43549[101], 6}, {NULL}};
const Segment SGI157096[] = {{&S53452[98], 9}, {NULL}};
const Segment SGI157097[] = {{&S53452[71], 6}, {NULL}};
const Segment SGI157098[] = {{&S43557[34], 6}, {NULL}};
const Segment SGI157099[] = {{&S43557[27], 6}, {NULL}};
const Segment SGI157100[] = {{&S43557[93], 9}, {NULL}};
const Segment SGI157101[] = {{&S43557[66], 6}, {NULL}};
const Segment SGI157102[] = {{&S43558[34], 7}, {NULL}};
const Segment SGI157103[] = {{&S43558[27], 6}, {NULL}};
const Segment SGI157104[] = {{&S43558[94], 9}, {NULL}};
const Segment SGI157105[] = {{&S43558[67], 6}, {NULL}};
const Segment SGI157106[] = {{&S44370[34], 10}, {NULL}};
const Segment SGI157107[] = {{&S44370[27], 6}, {NULL}};
const Segment SGI157108[] = {{&S44370[97], 9}, {NULL}};
const Segment SGI157109[] = {{&S44370[70], 6}, {NULL}};
const Segment SGI157110[] = {{&S44370[138], 8}, {NULL}};
const Segment SGI157111[] = {{&S53466[44], 13}, {NULL}};
const Segment SGI157112[] = {{&S53466[44], 25}, {NULL}};
const Segment SGI157113[] = {{&S53466[30], 6}, {NULL}};
const Segment SGI157114[] = {{&S53466[31], 3}, {NULL}};
const Segment SGI157115[] = {{&S53466[83], 6}, {NULL}};
const Segment SGI157116[] = {{&S53467[44], 5}, {NULL}};
const Segment SGI157117[] = {{&S53467[44], 17}, {NULL}};
const Segment SGI157118[] = {{&S53467[30], 6}, {NULL}};
const Segment SGI157119[] = {{&S53467[31], 3}, {NULL}};
const Segment SGI157120[] = {{&S53467[75], 6}, {NULL}};
const Segment SGI157121[] = {{&S44374[34], 10}, {NULL}};
const Segment SGI157122[] = {{&S44374[27], 6}, {NULL}};
const Segment SGI157123[] = {{&S44374[97], 9}, {NULL}};
const Segment SGI157124[] = {{&S44374[70], 6}, {NULL}};
const Segment SGI157125[] = {{&S44374[138], 8}, {NULL}};
const Segment SGI157126[] = {{&S53469[44], 15}, {NULL}};
const Segment SGI157127[] = {{&S53469[30], 6}, {NULL}};
const Segment SGI157128[] = {{&S53469[31], 3}, {NULL}};
const Segment SGI157129[] = {{&S53469[87], 6}, {NULL}};
const Segment SGI157130[] = {{&S53471[44], 10}, {NULL}};
const Segment SGI157131[] = {{&S53471[44], 22}, {NULL}};
const Segment SGI157132[] = {{&S53471[30], 6}, {NULL}};
const Segment SGI157133[] = {{&S53471[31], 3}, {NULL}};
const Segment SGI157134[] = {{&S53471[80], 6}, {NULL}};
const Segment SGI157135[] = {{&S53473[38], 5}, {NULL}};
const Segment SGI157136[] = {{&S53473[31], 6}, {NULL}};
const Segment SGI157137[] = {{&S43576[34], 9}, {NULL}};
const Segment SGI157138[] = {{&S43576[27], 6}, {NULL}};
const Segment SGI157139[] = {{&S43576[96], 9}, {NULL}};
const Segment SGI157140[] = {{&S43576[69], 6}, {NULL}};
const Segment SGI157141[] = {{&S43577[34], 9}, {NULL}};
const Segment SGI157142[] = {{&S43577[27], 6}, {NULL}};
const Segment SGI157143[] = {{&S43577[96], 9}, {NULL}};
const Segment SGI157144[] = {{&S43577[69], 6}, {NULL}};
const Segment SGI157145[] = {{&S53479[34], 11}, {NULL}};
const Segment SGI157146[] = {{&S53479[27], 6}, {NULL}};
const Segment SGI157147[] = {{&S43578[34], 9}, {NULL}};
const Segment SGI157148[] = {{&S43578[27], 6}, {NULL}};
const Segment SGI157149[] = {{&S53479[98], 9}, {NULL}};
const Segment SGI157150[] = {{&S53479[71], 6}, {NULL}};
const Segment SGI157151[] = {{&S43578[96], 9}, {NULL}};
const Segment SGI157152[] = {{&S43578[69], 6}, {NULL}};
const Segment SGI157153[] = {{&S43579[34], 9}, {NULL}};
const Segment SGI157154[] = {{&S43579[27], 6}, {NULL}};
const Segment SGI157155[] = {{&S43579[96], 9}, {NULL}};
const Segment SGI157156[] = {{&S43579[69], 6}, {NULL}};
const Segment SGI157157[] = {{&S43580[34], 9}, {NULL}};
const Segment SGI157158[] = {{&S43580[27], 6}, {NULL}};
const Segment SGI157159[] = {{&S43580[96], 9}, {NULL}};
const Segment SGI157160[] = {{&S43580[69], 6}, {NULL}};
const Segment SGI157161[] = {{&S43581[38], 7}, {NULL}};
const Segment SGI157162[] = {{&S43581[31], 6}, {NULL}};
const Segment SGI157163[] = {{&S53487[44], 15}, {NULL}};
const Segment SGI157164[] = {{&S43582[44], 7}, {NULL}};
const Segment SGI157165[] = {{&S53487[44], 27}, {NULL}};
const Segment SGI157166[] = {{&S43582[44], 35}, {NULL}};
const Segment SGI157167[] = {{&S53487[30], 6}, {NULL}};
const Segment SGI157168[] = {{&S43582[30], 6}, {NULL}};
const Segment SGI157169[] = {{&S53487[31], 3}, {NULL}};
const Segment SGI157170[] = {{&S43582[128], 3}, {NULL}};
const Segment SGI157171[] = {{&S53487[104], 6}, {NULL}};
const Segment SGI157172[] = {{&S43582[93], 6}, {NULL}};
const Segment SGI157173[] = {{&S53487[73], 8}, {NULL}};
const Segment SGI157174[] = {{&S43583[44], 7}, {NULL}};
const Segment SGI157175[] = {{&S43583[44], 35}, {NULL}};
const Segment SGI157176[] = {{&S43583[30], 6}, {NULL}};
const Segment SGI157177[] = {{&S53488[44], 15}, {NULL}};
const Segment SGI157178[] = {{&S53488[44], 27}, {NULL}};
const Segment SGI157179[] = {{&S53488[30], 6}, {NULL}};
const Segment SGI157180[] = {{&S43583[128], 3}, {NULL}};
const Segment SGI157181[] = {{&S43583[93], 6}, {NULL}};
const Segment SGI157182[] = {{&S53488[31], 3}, {NULL}};
const Segment SGI157183[] = {{&S53488[100], 6}, {NULL}};
const Segment SGI157184[] = {{&S53488[73], 8}, {NULL}};
const Segment SGI157185[] = {{&S43584[44], 7}, {NULL}};
const Segment SGI157186[] = {{&S43584[44], 35}, {NULL}};
const Segment SGI157187[] = {{&S43584[30], 6}, {NULL}};
const Segment SGI157188[] = {{&S53489[44], 15}, {NULL}};
const Segment SGI157189[] = {{&S53489[44], 27}, {NULL}};
const Segment SGI157190[] = {{&S53489[30], 6}, {NULL}};
const Segment SGI157191[] = {{&S43584[128], 3}, {NULL}};
const Segment SGI157192[] = {{&S43584[93], 6}, {NULL}};
const Segment SGI157193[] = {{&S53489[31], 3}, {NULL}};
const Segment SGI157194[] = {{&S53489[104], 6}, {NULL}};
const Segment SGI157195[] = {{&S53489[73], 8}, {NULL}};
const Segment SGI157196[] = {{&S43585[37], 15}, {NULL}};
const Segment SGI157197[] = {{&S53490[44], 11}, {NULL}};
const Segment SGI157198[] = {{&S53490[44], 23}, {NULL}};
const Segment SGI157199[] = {{&S53490[30], 6}, {NULL}};
const Segment SGI157200[] = {{&S53490[31], 3}, {NULL}};
const Segment SGI157201[] = {{&S53490[96], 6}, {NULL}};
const Segment SGI157202[] = {{&S53490[69], 8}, {NULL}};
const Segment SGI157203[] = {{&S53509[34], 11}, {NULL}};
const Segment SGI157204[] = {{&S53509[27], 6}, {NULL}};
const Segment SGI157205[] = {{&S53509[98], 9}, {NULL}};
const Segment SGI157206[] = {{&S53509[71], 6}, {NULL}};
const Segment SGI157207[] = {{&S53548[34], 9}, {NULL}};
const Segment SGI157208[] = {{&S53548[27], 6}, {NULL}};
const Segment SGI157209[] = {{&S53548[96], 9}, {NULL}};
const Segment SGI157210[] = {{&S53548[69], 6}, {NULL}};
const Segment SGI157211[] = {{&S53549[34], 9}, {NULL}};
const Segment SGI157212[] = {{&S53549[27], 6}, {NULL}};
const Segment SGI157213[] = {{&S53549[96], 9}, {NULL}};
const Segment SGI157214[] = {{&S53549[69], 6}, {NULL}};
const Segment SGI157215[] = {{&S43644[34], 8}, {NULL}};
const Segment SGI157216[] = {{&S43644[27], 6}, {NULL}};
const Segment SGI157217[] = {{&S53550[34], 9}, {NULL}};
const Segment SGI157218[] = {{&S43644[105], 9}, {NULL}};
const Segment SGI157219[] = {{&S43644[78], 6}, {NULL}};
const Segment SGI157220[] = {{&S53550[27], 6}, {NULL}};
const Segment SGI157221[] = {{&S53550[96], 9}, {NULL}};
const Segment SGI157222[] = {{&S53550[69], 6}, {NULL}};
const Segment SGI157223[] = {{&S43645[34], 8}, {NULL}};
const Segment SGI157224[] = {{&S43645[27], 6}, {NULL}};
const Segment SGI157225[] = {{&S43645[105], 9}, {NULL}};
const Segment SGI157226[] = {{&S43645[78], 6}, {NULL}};
const Segment SGI157227[] = {{&S44467[44], 10}, {NULL}};
const Segment SGI157228[] = {{&S44467[44], 22}, {NULL}};
const Segment SGI157229[] = {{&S44467[30], 6}, {NULL}};
const Segment SGI157230[] = {{&S44467[115], 3}, {NULL}};
const Segment SGI157231[] = {{&S44467[80], 6}, {NULL}};
const Segment SGI157232[] = {{&S44468[44], 10}, {NULL}};
const Segment SGI157233[] = {{&S44468[44], 22}, {NULL}};
const Segment SGI157234[] = {{&S44468[30], 6}, {NULL}};
const Segment SGI157235[] = {{&S44468[83], 6}, {NULL}};
const Segment SGI157236[] = {{&S53554[34], 10}, {NULL}};
const Segment SGI157237[] = {{&S53554[27], 6}, {NULL}};
const Segment SGI157238[] = {{&S53554[97], 9}, {NULL}};
const Segment SGI157239[] = {{&S53554[70], 6}, {NULL}};
const Segment SGI157240[] = {{&S44469[38], 10}, {NULL}};
const Segment SGI157241[] = {{&S44469[31], 6}, {NULL}};
const Segment SGI157242[] = {{&S44474[44], 10}, {NULL}};
const Segment SGI157243[] = {{&S44474[44], 22}, {NULL}};
const Segment SGI157244[] = {{&S44474[30], 6}, {NULL}};
const Segment SGI157245[] = {{&S44474[115], 3}, {NULL}};
const Segment SGI157246[] = {{&S44474[80], 6}, {NULL}};
const Segment SGI157247[] = {{&S53565[44], 10}, {NULL}};
const Segment SGI157248[] = {{&S53565[44], 22}, {NULL}};
const Segment SGI157249[] = {{&S53565[30], 6}, {NULL}};
const Segment SGI157250[] = {{&S53565[31], 3}, {NULL}};
const Segment SGI157251[] = {{&S53565[80], 6}, {NULL}};
const Segment SGI157252[] = {{&S53566[38], 10}, {NULL}};
const Segment SGI157253[] = {{&S53566[31], 6}, {NULL}};
const Segment SGI157254[] = {{&S43667[34], 7}, {NULL}};
const Segment SGI157255[] = {{&S43667[27], 6}, {NULL}};
const Segment SGI157256[] = {{&S43667[104], 9}, {NULL}};
const Segment SGI157257[] = {{&S43667[77], 6}, {NULL}};
const Segment SGI157258[] = {{&S43668[34], 7}, {NULL}};
const Segment SGI157259[] = {{&S43668[27], 6}, {NULL}};
const Segment SGI157260[] = {{&S43668[104], 9}, {NULL}};
const Segment SGI157261[] = {{&S43668[77], 6}, {NULL}};
const Segment SGI157262[] = {{&S43699[34], 12}, {NULL}};
const Segment SGI157263[] = {{&S43699[27], 6}, {NULL}};
const Segment SGI157264[] = {{&S43699[109], 9}, {NULL}};
const Segment SGI157265[] = {{&S43699[82], 6}, {NULL}};
const Segment SGI157266[] = {{&S43700[34], 12}, {NULL}};
const Segment SGI157267[] = {{&S43700[27], 6}, {NULL}};
const Segment SGI157268[] = {{&S43700[109], 9}, {NULL}};
const Segment SGI157269[] = {{&S43700[82], 6}, {NULL}};
const Segment SGI157270[] = {{&S53595[34], 14}, {NULL}};
const Segment SGI157271[] = {{&S53595[27], 6}, {NULL}};
const Segment SGI157272[] = {{&S53595[101], 9}, {NULL}};
const Segment SGI157273[] = {{&S53595[74], 6}, {NULL}};
const Segment SGI157274[] = {{&S53648[44], 10}, {NULL}};
const Segment SGI157275[] = {{&S53648[44], 22}, {NULL}};
const Segment SGI157276[] = {{&S53648[30], 6}, {NULL}};
const Segment SGI157277[] = {{&S53648[31], 3}, {NULL}};
const Segment SGI157278[] = {{&S53648[80], 6}, {NULL}};
const Segment SGI157279[] = {{&S53662[44], 13}, {NULL}};
const Segment SGI157280[] = {{&S53662[44], 38}, {NULL}};
const Segment SGI157281[] = {{&S53662[30], 6}, {NULL}};
const Segment SGI157282[] = {{&S53662[31], 3}, {NULL}};
const Segment SGI157283[] = {{&S53662[96], 6}, {NULL}};
const Segment SGI157284[] = {{&S53663[38], 13}, {NULL}};
const Segment SGI157285[] = {{&S53663[31], 6}, {NULL}};
const Segment SGI157286[] = {{&S53675[34], 6}, {NULL}};
const Segment SGI157287[] = {{&S53675[27], 6}, {NULL}};
const Segment SGI157288[] = {{&S53675[93], 9}, {NULL}};
const Segment SGI157289[] = {{&S53675[66], 6}, {NULL}};
const Segment SGI157290[] = {{&S53676[34], 6}, {NULL}};
const Segment SGI157291[] = {{&S53676[27], 6}, {NULL}};
const Segment SGI157292[] = {{&S53676[93], 9}, {NULL}};
const Segment SGI157293[] = {{&S53676[66], 6}, {NULL}};
const Segment SGI157294[] = {{&S53677[34], 13}, {NULL}};
const Segment SGI157295[] = {{&S53677[27], 6}, {NULL}};
const Segment SGI157296[] = {{&S53677[100], 9}, {NULL}};
const Segment SGI157297[] = {{&S53677[73], 6}, {NULL}};
const Segment SGI157298[] = {{&S53684[34], 10}, {NULL}};
const Segment SGI157299[] = {{&S53684[27], 6}, {NULL}};
const Segment SGI157300[] = {{&S53684[97], 9}, {NULL}};
const Segment SGI157301[] = {{&S53684[70], 6}, {NULL}};
const Segment SGI157302[] = {{&S53684[139], 8}, {NULL}};
const Segment SGI157303[] = {{&S53686[34], 10}, {NULL}};
const Segment SGI157304[] = {{&S53686[27], 6}, {NULL}};
const Segment SGI157305[] = {{&S53686[97], 9}, {NULL}};
const Segment SGI157306[] = {{&S53686[70], 6}, {NULL}};
const Segment SGI157307[] = {{&S53686[138], 8}, {NULL}};
const Segment SGI157308[] = {{&S53690[34], 11}, {NULL}};
const Segment SGI157309[] = {{&S53690[27], 6}, {NULL}};
const Segment SGI157310[] = {{&S53690[98], 9}, {NULL}};
const Segment SGI157311[] = {{&S53690[71], 6}, {NULL}};
const Segment SGI157312[] = {{&S53690[139], 8}, {NULL}};
const Segment SGI157313[] = {{&S53697[34], 19}, {NULL}};
const Segment SGI157314[] = {{&S53697[27], 6}, {NULL}};
const Segment SGI157315[] = {{&S53697[106], 9}, {NULL}};
const Segment SGI157316[] = {{&S53697[79], 6}, {NULL}};
const Segment SGI157317[] = {{&S53698[34], 19}, {NULL}};
const Segment SGI157318[] = {{&S53698[27], 6}, {NULL}};
const Segment SGI157319[] = {{&S53698[106], 9}, {NULL}};
const Segment SGI157320[] = {{&S53698[79], 6}, {NULL}};
const Segment SGI157321[] = {{&S53721[34], 10}, {NULL}};
const Segment SGI157322[] = {{&S53721[27], 6}, {NULL}};
const Segment SGI157323[] = {{&S53721[120], 9}, {NULL}};
const Segment SGI157324[] = {{&S53721[93], 6}, {NULL}};
const Segment SGI157325[] = {{&S53735[44], 13}, {NULL}};
const Segment SGI157326[] = {{&S53735[44], 25}, {NULL}};
const Segment SGI157327[] = {{&S53735[30], 6}, {NULL}};
const Segment SGI157328[] = {{&S53735[31], 3}, {NULL}};
const Segment SGI157329[] = {{&S53735[83], 6}, {NULL}};
const Segment SGI157330[] = {{&S53736[44], 5}, {NULL}};
const Segment SGI157331[] = {{&S53736[44], 17}, {NULL}};
const Segment SGI157332[] = {{&S53736[30], 6}, {NULL}};
const Segment SGI157333[] = {{&S53736[31], 3}, {NULL}};
const Segment SGI157334[] = {{&S53736[75], 6}, {NULL}};
const Segment SGI157335[] = {{&S53737[44], 46}, {NULL}};
const Segment SGI157336[] = {{&S53737[44], 58}, {NULL}};
const Segment SGI157337[] = {{&S53737[30], 6}, {NULL}};
const Segment SGI157338[] = {{&S53737[31], 3}, {NULL}};
const Segment SGI157339[] = {{&S53737[116], 6}, {NULL}};
const Segment SGI157340[] = {{&S53738[38], 5}, {NULL}};
const Segment SGI157341[] = {{&S53738[31], 6}, {NULL}};
const Segment SGI157342[] = {{&S53752[11], 1}, {NULL}};
const Segment SGI157343[] = {{&S53752[29], 6}, {NULL}};
const Segment SGI157344[] = {{&S53752[30], 3}, {NULL}};
const Segment SGI157345[] = {{&S53752[69], 6}, {NULL}};
const Segment SGI157346[] = {{&S53753[44], 11}, {NULL}};
const Segment SGI157347[] = {{&S53753[44], 23}, {NULL}};
const Segment SGI157348[] = {{&S53753[30], 6}, {NULL}};
const Segment SGI157349[] = {{&S53753[31], 3}, {NULL}};
const Segment SGI157350[] = {{&S53753[81], 6}, {NULL}};
const Segment SGI157351[] = {{&S53754[42], 13}, {NULL}};
const Segment SGI157352[] = {{&S53754[29], 6}, {NULL}};
const Segment SGI157353[] = {{&S53754[30], 3}, {NULL}};
const Segment SGI157354[] = {{&S53754[81], 6}, {NULL}};
const Segment SGI157355[] = {{&S53793[34], 10}, {NULL}};
const Segment SGI157356[] = {{&S53793[27], 6}, {NULL}};
const Segment SGI157357[] = {{&S53793[97], 9}, {NULL}};
const Segment SGI157358[] = {{&S53793[70], 6}, {NULL}};
const Segment SGI157359[] = {{&S53794[34], 15}, {NULL}};
const Segment SGI157360[] = {{&S53794[27], 6}, {NULL}};
const Segment SGI157361[] = {{&S53794[118], 9}, {NULL}};
const Segment SGI157362[] = {{&S53794[91], 6}, {NULL}};
const Segment SGI157363[] = {{&S53795[34], 7}, {NULL}};
const Segment SGI157364[] = {{&S53795[27], 6}, {NULL}};
const Segment SGI157365[] = {{&S53795[108], 9}, {NULL}};
const Segment SGI157366[] = {{&S53795[81], 6}, {NULL}};
const Segment SGI157367[] = {{&S49999[44], 4}, {NULL}};
const Segment SGI157368[] = {{&S49999[44], 16}, {NULL}};
const Segment SGI157369[] = {{&S49999[30], 6}, {NULL}};
const Segment SGI157370[] = {{&S49999[31], 3}, {NULL}};
const Segment SGI157371[] = {{&S49999[74], 6}, {NULL}};
const Segment SGI157372[] = {{&S44828[34], 10}, {NULL}};
const Segment SGI157373[] = {{&S44828[27], 6}, {NULL}};
const Segment SGI157374[] = {{&S44828[97], 9}, {NULL}};
const Segment SGI157375[] = {{&S44828[70], 6}, {NULL}};
const Segment SGI157376[] = {{&S44830[34], 10}, {NULL}};
const Segment SGI157377[] = {{&S44830[27], 6}, {NULL}};
const Segment SGI157378[] = {{&S44830[97], 9}, {NULL}};
const Segment SGI157379[] = {{&S44830[70], 6}, {NULL}};
const Segment SGI157380[] = {{&S53844[34], 19}, {NULL}};
const Segment SGI157381[] = {{&S53844[27], 6}, {NULL}};
const Segment SGI157382[] = {{&S53844[106], 9}, {NULL}};
const Segment SGI157383[] = {{&S53844[79], 6}, {NULL}};
const Segment SGI157384[] = {{&S53845[34], 19}, {NULL}};
const Segment SGI157385[] = {{&S53845[27], 6}, {NULL}};
const Segment SGI157386[] = {{&S53845[106], 9}, {NULL}};
const Segment SGI157387[] = {{&S53845[79], 6}, {NULL}};
const Segment SGI157388[] = {{&S42162[32], 10}, {NULL}};
const Segment SGI157389[] = {{&S42162[27], 4}, {NULL}};
const Segment SGI157390[] = {{&S42162[95], 9}, {NULL}};
const Segment SGI157391[] = {{&S42162[68], 6}, {NULL}};
const Segment SGI157392[] = {{&S43748[34], 10}, {NULL}};
const Segment SGI157393[] = {{&S43748[27], 6}, {NULL}};
const Segment SGI157394[] = {{&S43748[97], 9}, {NULL}};
const Segment SGI157395[] = {{&S43748[70], 6}, {NULL}};
const Segment SGI157396[] = {{&S43801[44], 14}, {NULL}};
const Segment SGI157397[] = {{&S43801[44], 26}, {NULL}};
const Segment SGI157398[] = {{&S43801[30], 6}, {NULL}};
const Segment SGI157399[] = {{&S43801[139], 3}, {NULL}};
const Segment SGI157400[] = {{&S43801[104], 6}, {NULL}};
const Segment SGI157401[] = {{&S43801[72], 8}, {NULL}};
const Segment SGI157402[] = {{&S44938[44], 6}, {NULL}};
const Segment SGI157403[] = {{&S44938[44], 18}, {NULL}};
const Segment SGI157404[] = {{&S44938[30], 6}, {NULL}};
const Segment SGI157405[] = {{&S44938[111], 3}, {NULL}};
const Segment SGI157406[] = {{&S44938[76], 6}, {NULL}};
const Segment SGI157407[] = {{&S44938[143], 15}, {NULL}};
const Segment SGI157408[] = {{&S44939[44], 7}, {NULL}};
const Segment SGI157409[] = {{&S44939[44], 19}, {NULL}};
const Segment SGI157410[] = {{&S44939[30], 6}, {NULL}};
const Segment SGI157411[] = {{&S44939[112], 3}, {NULL}};
const Segment SGI157412[] = {{&S44939[77], 6}, {NULL}};
const Segment SGI157413[] = {{&S44939[144], 15}, {NULL}};
const Segment SGI157414[] = {{&S44940[44], 7}, {NULL}};
const Segment SGI157415[] = {{&S44940[44], 19}, {NULL}};
const Segment SGI157416[] = {{&S44940[30], 6}, {NULL}};
const Segment SGI157417[] = {{&S53927[44], 9}, {NULL}};
const Segment SGI157418[] = {{&S53927[44], 21}, {NULL}};
const Segment SGI157419[] = {{&S53927[30], 6}, {NULL}};
const Segment SGI157420[] = {{&S44940[112], 3}, {NULL}};
const Segment SGI157421[] = {{&S44940[77], 6}, {NULL}};
const Segment SGI157422[] = {{&S53927[31], 3}, {NULL}};
const Segment SGI157423[] = {{&S44940[144], 15}, {NULL}};
const Segment SGI157424[] = {{&S53927[79], 6}, {NULL}};
const Segment SGI157425[] = {{&S44941[44], 8}, {NULL}};
const Segment SGI157426[] = {{&S44941[44], 20}, {NULL}};
const Segment SGI157427[] = {{&S44941[30], 6}, {NULL}};
const Segment SGI157428[] = {{&S53928[38], 9}, {NULL}};
const Segment SGI157429[] = {{&S53928[31], 6}, {NULL}};
const Segment SGI157430[] = {{&S44941[113], 3}, {NULL}};
const Segment SGI157431[] = {{&S44941[78], 6}, {NULL}};
const Segment SGI157432[] = {{&S44941[145], 15}, {NULL}};
const Segment SGI157433[] = {{&S44942[44], 7}, {NULL}};
const Segment SGI157434[] = {{&S44942[44], 19}, {NULL}};
const Segment SGI157435[] = {{&S44942[30], 6}, {NULL}};
const Segment SGI157436[] = {{&S44942[112], 3}, {NULL}};
const Segment SGI157437[] = {{&S44942[77], 6}, {NULL}};
const Segment SGI157438[] = {{&S44942[144], 15}, {NULL}};
const Segment SGI157439[] = {{&S44943[44], 6}, {NULL}};
const Segment SGI157440[] = {{&S44943[44], 19}, {NULL}};
const Segment SGI157441[] = {{&S44943[30], 6}, {NULL}};
const Segment SGI157442[] = {{&S44943[112], 3}, {NULL}};
const Segment SGI157443[] = {{&S44943[77], 6}, {NULL}};
const Segment SGI157444[] = {{&S44943[144], 15}, {NULL}};
const Segment SGI157445[] = {{&S44944[44], 6}, {NULL}};
const Segment SGI157446[] = {{&S44944[44], 26}, {NULL}};
const Segment SGI157447[] = {{&S44944[30], 6}, {NULL}};
const Segment SGI157448[] = {{&S44944[119], 3}, {NULL}};
const Segment SGI157449[] = {{&S44944[84], 6}, {NULL}};
const Segment SGI157450[] = {{&S44944[151], 15}, {NULL}};
const Segment SGI157451[] = {{&S44945[44], 7}, {NULL}};
const Segment SGI157452[] = {{&S44945[44], 27}, {NULL}};
const Segment SGI157453[] = {{&S44945[30], 6}, {NULL}};
const Segment SGI157454[] = {{&S44945[120], 3}, {NULL}};
const Segment SGI157455[] = {{&S44945[85], 6}, {NULL}};
const Segment SGI157456[] = {{&S44945[152], 15}, {NULL}};
const Segment SGI157457[] = {{&S44946[44], 7}, {NULL}};
const Segment SGI157458[] = {{&S44946[44], 19}, {NULL}};
const Segment SGI157459[] = {{&S44946[30], 6}, {NULL}};
const Segment SGI157460[] = {{&S44946[77], 6}, {NULL}};
const Segment SGI157461[] = {{&S44946[143], 15}, {NULL}};
const Segment SGI157462[] = {{&S44947[44], 7}, {NULL}};
const Segment SGI157463[] = {{&S44947[44], 27}, {NULL}};
const Segment SGI157464[] = {{&S44947[30], 6}, {NULL}};
const Segment SGI157465[] = {{&S44947[120], 3}, {NULL}};
const Segment SGI157466[] = {{&S44947[85], 6}, {NULL}};
const Segment SGI157467[] = {{&S44947[152], 15}, {NULL}};
const Segment SGI157468[] = {{&S44948[44], 6}, {NULL}};
const Segment SGI157469[] = {{&S44948[44], 18}, {NULL}};
const Segment SGI157470[] = {{&S44948[30], 6}, {NULL}};
const Segment SGI157471[] = {{&S44948[76], 6}, {NULL}};
const Segment SGI157472[] = {{&S44948[142], 15}, {NULL}};
const Segment SGI157473[] = {{&S44949[44], 6}, {NULL}};
const Segment SGI157474[] = {{&S44949[44], 16}, {NULL}};
const Segment SGI157475[] = {{&S44949[30], 6}, {NULL}};
const Segment SGI157476[] = {{&S44949[109], 3}, {NULL}};
const Segment SGI157477[] = {{&S44949[74], 6}, {NULL}};
const Segment SGI157478[] = {{&S44949[141], 15}, {NULL}};
const Segment SGI157479[] = {{&S43843[44], 18}, {NULL}};
const Segment SGI157480[] = {{&S43843[44], 30}, {NULL}};
const Segment SGI157481[] = {{&S43843[30], 6}, {NULL}};
const Segment SGI157482[] = {{&S43843[123], 3}, {NULL}};
const Segment SGI157483[] = {{&S43843[88], 6}, {NULL}};
const Segment SGI157484[] = {{&S43844[44], 10}, {NULL}};
const Segment SGI157485[] = {{&S43844[44], 22}, {NULL}};
const Segment SGI157486[] = {{&S43844[30], 6}, {NULL}};
const Segment SGI157487[] = {{&S43844[115], 3}, {NULL}};
const Segment SGI157488[] = {{&S43844[80], 6}, {NULL}};
const Segment SGI157489[] = {{&S53952[34], 18}, {NULL}};
const Segment SGI157490[] = {{&S53952[27], 6}, {NULL}};
const Segment SGI157491[] = {{&S53952[105], 9}, {NULL}};
const Segment SGI157492[] = {{&S53952[78], 6}, {NULL}};
const Segment SGI157493[] = {{&S43847[38], 18}, {NULL}};
const Segment SGI157494[] = {{&S43847[31], 6}, {NULL}};
const Segment SGI157495[] = {{&S43848[42], 18}, {NULL}};
const Segment SGI157496[] = {{&S43848[30], 6}, {NULL}};
const Segment SGI157497[] = {{&S43848[121], 3}, {NULL}};
const Segment SGI157498[] = {{&S43848[86], 6}, {NULL}};
const Segment SGI157499[] = {{&S43849[44], 18}, {NULL}};
const Segment SGI157500[] = {{&S43849[44], 30}, {NULL}};
const Segment SGI157501[] = {{&S43849[30], 6}, {NULL}};
const Segment SGI157502[] = {{&S43849[123], 3}, {NULL}};
const Segment SGI157503[] = {{&S43849[88], 6}, {NULL}};
const Segment SGI157504[] = {{&S39680[34], 9}, {NULL}};
const Segment SGI157505[] = {{&S39680[27], 6}, {NULL}};
const Segment SGI157506[] = {{&S39680[96], 9}, {NULL}};
const Segment SGI157507[] = {{&S39680[69], 6}, {NULL}};
const Segment SGI157508[] = {{&S53995[44], 8}, {NULL}};
const Segment SGI157509[] = {{&S53995[44], 25}, {NULL}};
const Segment SGI157510[] = {{&S53995[30], 6}, {NULL}};
const Segment SGI157511[] = {{&S53995[31], 3}, {NULL}};
const Segment SGI157512[] = {{&S53995[83], 6}, {NULL}};
const Segment SGI157513[] = {{&S53996[44], 8}, {NULL}};
const Segment SGI157514[] = {{&S53996[44], 25}, {NULL}};
const Segment SGI157515[] = {{&S53996[30], 6}, {NULL}};
const Segment SGI157516[] = {{&S53996[31], 3}, {NULL}};
const Segment SGI157517[] = {{&S53996[83], 6}, {NULL}};
const Segment SGI157518[] = {{&S53997[44], 8}, {NULL}};
const Segment SGI157519[] = {{&S53997[44], 26}, {NULL}};
const Segment SGI157520[] = {{&S53997[30], 6}, {NULL}};
const Segment SGI157521[] = {{&S53997[31], 3}, {NULL}};
const Segment SGI157522[] = {{&S53997[84], 6}, {NULL}};
const Segment SGI157523[] = {{&S53998[44], 8}, {NULL}};
const Segment SGI157524[] = {{&S53998[44], 26}, {NULL}};
const Segment SGI157525[] = {{&S53998[30], 6}, {NULL}};
const Segment SGI157526[] = {{&S53998[31], 3}, {NULL}};
const Segment SGI157527[] = {{&S53998[84], 6}, {NULL}};
const Segment SGI157528[] = {{&S53999[44], 8}, {NULL}};
const Segment SGI157529[] = {{&S53999[44], 26}, {NULL}};
const Segment SGI157530[] = {{&S53999[30], 6}, {NULL}};
const Segment SGI157531[] = {{&S53999[31], 3}, {NULL}};
const Segment SGI157532[] = {{&S53999[84], 6}, {NULL}};
const Segment SGI157533[] = {{&S54000[44], 8}, {NULL}};
const Segment SGI157534[] = {{&S54000[44], 25}, {NULL}};
const Segment SGI157535[] = {{&S54000[30], 6}, {NULL}};
const Segment SGI157536[] = {{&S54000[31], 3}, {NULL}};
const Segment SGI157537[] = {{&S54000[83], 6}, {NULL}};
const Segment SGI157538[] = {{&S43901[34], 29}, {NULL}};
const Segment SGI157539[] = {{&S43901[27], 6}, {NULL}};
const Segment SGI157540[] = {{&S54008[44], 10}, {NULL}};
const Segment SGI157541[] = {{&S54008[44], 33}, {NULL}};
const Segment SGI157542[] = {{&S54008[30], 6}, {NULL}};
const Segment SGI157543[] = {{&S54008[31], 3}, {NULL}};
const Segment SGI157544[] = {{&S54008[91], 6}, {NULL}};
const Segment SGI157545[] = {{&S45053[34], 10}, {NULL}};
const Segment SGI157546[] = {{&S45053[27], 6}, {NULL}};
const Segment SGI157547[] = {{&S45053[97], 9}, {NULL}};
const Segment SGI157548[] = {{&S45053[70], 6}, {NULL}};
const Segment SGI157549[] = {{&S45055[34], 10}, {NULL}};
const Segment SGI157550[] = {{&S45055[27], 6}, {NULL}};
const Segment SGI157551[] = {{&S45055[97], 9}, {NULL}};
const Segment SGI157552[] = {{&S45055[70], 6}, {NULL}};
const Segment SGI157553[] = {{&S54100[34], 7}, {NULL}};
const Segment SGI157554[] = {{&S54100[27], 6}, {NULL}};
const Segment SGI157555[] = {{&S54100[96], 9}, {NULL}};
const Segment SGI157556[] = {{&S54100[69], 6}, {NULL}};
const Segment SGI157557[] = {{&S54116[34], 7}, {NULL}};
const Segment SGI157558[] = {{&S54116[27], 6}, {NULL}};
const Segment SGI157559[] = {{&S54116[94], 9}, {NULL}};
const Segment SGI157560[] = {{&S54116[67], 6}, {NULL}};
const Segment SGI157561[] = {{&S39823[34], 10}, {NULL}};
const Segment SGI157562[] = {{&S39823[27], 6}, {NULL}};
const Segment SGI157563[] = {{&S39823[97], 9}, {NULL}};
const Segment SGI157564[] = {{&S39823[70], 6}, {NULL}};
const Segment SGI157565[] = {{&S39825[34], 10}, {NULL}};
const Segment SGI157566[] = {{&S39825[27], 6}, {NULL}};
const Segment SGI157567[] = {{&S39825[97], 9}, {NULL}};
const Segment SGI157568[] = {{&S39825[70], 6}, {NULL}};
const Segment SGI157569[] = {{&S54145[34], 7}, {NULL}};
const Segment SGI157570[] = {{&S54145[27], 6}, {NULL}};
const Segment SGI157571[] = {{&S54145[100], 9}, {NULL}};
const Segment SGI157572[] = {{&S54145[73], 6}, {NULL}};
const Segment SGI157573[] = {{&S54146[34], 10}, {NULL}};
const Segment SGI157574[] = {{&S54146[27], 6}, {NULL}};
const Segment SGI157575[] = {{&S54146[97], 9}, {NULL}};
const Segment SGI157576[] = {{&S54146[70], 6}, {NULL}};
const Segment SGI157577[] = {{&S54147[34], 7}, {NULL}};
const Segment SGI157578[] = {{&S54147[27], 6}, {NULL}};
const Segment SGI157579[] = {{&S54147[100], 9}, {NULL}};
const Segment SGI157580[] = {{&S54147[73], 6}, {NULL}};
const Segment SGI157581[] = {{&S45163[34], 11}, {NULL}};
const Segment SGI157582[] = {{&S45163[27], 6}, {NULL}};
const Segment SGI157583[] = {{&S45163[98], 9}, {NULL}};
const Segment SGI157584[] = {{&S45163[71], 6}, {NULL}};
const Segment SGI157585[] = {{&S54159[34], 11}, {NULL}};
const Segment SGI157586[] = {{&S54159[27], 6}, {NULL}};
const Segment SGI157587[] = {{&S54159[98], 9}, {NULL}};
const Segment SGI157588[] = {{&S54159[71], 6}, {NULL}};
const Segment SGI157589[] = {{&S45165[34], 11}, {NULL}};
const Segment SGI157590[] = {{&S54160[34], 11}, {NULL}};
const Segment SGI157591[] = {{&S45165[27], 6}, {NULL}};
const Segment SGI157592[] = {{&S54160[27], 6}, {NULL}};
const Segment SGI157593[] = {{&S45165[98], 9}, {NULL}};
const Segment SGI157594[] = {{&S45165[71], 6}, {NULL}};
const Segment SGI157595[] = {{&S54160[98], 9}, {NULL}};
const Segment SGI157596[] = {{&S54160[71], 6}, {NULL}};
const Segment SGI157597[] = {{&S45166[34], 11}, {NULL}};
const Segment SGI157598[] = {{&S45166[27], 6}, {NULL}};
const Segment SGI157599[] = {{&S45166[98], 9}, {NULL}};
const Segment SGI157600[] = {{&S45166[71], 6}, {NULL}};
const Segment SGI157601[] = {{&S43901[116], 9}, {NULL}};
const Segment SGI157602[] = {{&S43901[89], 6}, {NULL}};
const Segment SGI157603[] = {{&S54177[34], 14}, {NULL}};
const Segment SGI157604[] = {{&S54177[27], 6}, {NULL}};
const Segment SGI157605[] = {{&S54177[101], 9}, {NULL}};
const Segment SGI157606[] = {{&S54177[74], 6}, {NULL}};
const Segment SGI157607[] = {{&S54178[34], 14}, {NULL}};
const Segment SGI157608[] = {{&S54178[27], 6}, {NULL}};
const Segment SGI157609[] = {{&S54178[101], 9}, {NULL}};
const Segment SGI157610[] = {{&S54178[74], 6}, {NULL}};
const Segment SGI157611[] = {{&S54221[34], 7}, {NULL}};
const Segment SGI157612[] = {{&S54221[27], 6}, {NULL}};
const Segment SGI157613[] = {{&S54221[94], 9}, {NULL}};
const Segment SGI157614[] = {{&S54221[67], 6}, {NULL}};
const Segment SGI157615[] = {{&S43971[34], 10}, {NULL}};
const Segment SGI157616[] = {{&S43971[27], 6}, {NULL}};
const Segment SGI157617[] = {{&S43971[97], 9}, {NULL}};
const Segment SGI157618[] = {{&S43971[70], 6}, {NULL}};
const Segment SGI157619[] = {{&S43973[34], 10}, {NULL}};
const Segment SGI157620[] = {{&S43973[27], 6}, {NULL}};
const Segment SGI157621[] = {{&S43973[97], 9}, {NULL}};
const Segment SGI157622[] = {{&S43973[70], 6}, {NULL}};
const Segment SGI157623[] = {{&S43974[44], 11}, {NULL}};
const Segment SGI157624[] = {{&S43974[44], 28}, {NULL}};
const Segment SGI157625[] = {{&S43974[30], 6}, {NULL}};
const Segment SGI157626[] = {{&S43974[121], 3}, {NULL}};
const Segment SGI157627[] = {{&S43974[86], 6}, {NULL}};
const Segment SGI157628[] = {{&S43975[38], 7}, {NULL}};
const Segment SGI157629[] = {{&S43975[31], 6}, {NULL}};
const Segment SGI157630[] = {{&S43976[44], 7}, {NULL}};
const Segment SGI157631[] = {{&S43976[44], 24}, {NULL}};
const Segment SGI157632[] = {{&S43976[30], 6}, {NULL}};
const Segment SGI157633[] = {{&S43976[117], 3}, {NULL}};
const Segment SGI157634[] = {{&S43976[82], 6}, {NULL}};
const Segment SGI157635[] = {{&S43977[44], 11}, {NULL}};
const Segment SGI157636[] = {{&S43977[44], 28}, {NULL}};
const Segment SGI157637[] = {{&S43977[30], 6}, {NULL}};
const Segment SGI157638[] = {{&S43977[121], 3}, {NULL}};
const Segment SGI157639[] = {{&S43977[86], 6}, {NULL}};
const Segment SGI157640[] = {{&S43978[44], 7}, {NULL}};
const Segment SGI157641[] = {{&S43978[44], 24}, {NULL}};
const Segment SGI157642[] = {{&S43978[30], 6}, {NULL}};
const Segment SGI157643[] = {{&S43978[117], 3}, {NULL}};
const Segment SGI157644[] = {{&S43978[82], 6}, {NULL}};
const Segment SGI157645[] = {{&S43979[44], 11}, {NULL}};
const Segment SGI157646[] = {{&S43979[44], 27}, {NULL}};
const Segment SGI157647[] = {{&S43979[30], 6}, {NULL}};
const Segment SGI157648[] = {{&S43979[120], 3}, {NULL}};
const Segment SGI157649[] = {{&S43979[85], 6}, {NULL}};
const Segment SGI157650[] = {{&S43980[44], 7}, {NULL}};
const Segment SGI157651[] = {{&S43980[44], 23}, {NULL}};
const Segment SGI157652[] = {{&S43980[30], 6}, {NULL}};
const Segment SGI157653[] = {{&S43980[116], 3}, {NULL}};
const Segment SGI157654[] = {{&S43980[81], 6}, {NULL}};
const Segment SGI157655[] = {{&S43981[34], 7}, {NULL}};
const Segment SGI157656[] = {{&S43981[27], 6}, {NULL}};
const Segment SGI157657[] = {{&S54255[44], 7}, {NULL}};
const Segment SGI157658[] = {{&S54255[44], 24}, {NULL}};
const Segment SGI157659[] = {{&S43981[100], 9}, {NULL}};
const Segment SGI157660[] = {{&S54255[30], 6}, {NULL}};
const Segment SGI157661[] = {{&S43981[73], 6}, {NULL}};
const Segment SGI157662[] = {{&S54255[31], 3}, {NULL}};
const Segment SGI157663[] = {{&S54255[82], 6}, {NULL}};
const Segment SGI157664[] = {{&S43982[34], 7}, {NULL}};
const Segment SGI157665[] = {{&S43982[27], 6}, {NULL}};
const Segment SGI157666[] = {{&S54256[44], 7}, {NULL}};
const Segment SGI157667[] = {{&S43982[100], 9}, {NULL}};
const Segment SGI157668[] = {{&S54256[44], 24}, {NULL}};
const Segment SGI157669[] = {{&S43982[73], 6}, {NULL}};
const Segment SGI157670[] = {{&S54256[30], 6}, {NULL}};
const Segment SGI157671[] = {{&S54256[31], 3}, {NULL}};
const Segment SGI157672[] = {{&S54256[82], 6}, {NULL}};
const Segment SGI157673[] = {{&S54257[38], 7}, {NULL}};
const Segment SGI157674[] = {{&S54257[31], 6}, {NULL}};
const Segment SGI157675[] = {{&S54259[34], 9}, {NULL}};
const Segment SGI157676[] = {{&S54259[27], 6}, {NULL}};
const Segment SGI157677[] = {{&S43987[34], 10}, {NULL}};
const Segment SGI157678[] = {{&S54259[96], 9}, {NULL}};
const Segment SGI157679[] = {{&S54259[69], 6}, {NULL}};
const Segment SGI157680[] = {{&S43987[27], 6}, {NULL}};
const Segment SGI157681[] = {{&S43987[97], 9}, {NULL}};
const Segment SGI157682[] = {{&S43987[70], 6}, {NULL}};
const Segment SGI157683[] = {{&S54260[34], 6}, {NULL}};
const Segment SGI157684[] = {{&S54260[27], 6}, {NULL}};
const Segment SGI157685[] = {{&S54260[93], 9}, {NULL}};
const Segment SGI157686[] = {{&S54260[66], 6}, {NULL}};
const Segment SGI157687[] = {{&S43988[34], 10}, {NULL}};
const Segment SGI157688[] = {{&S43988[27], 6}, {NULL}};
const Segment SGI157689[] = {{&S43988[97], 9}, {NULL}};
const Segment SGI157690[] = {{&S43988[70], 6}, {NULL}};
const Segment SGI157691[] = {{&S54261[34], 6}, {NULL}};
const Segment SGI157692[] = {{&S54261[27], 6}, {NULL}};
const Segment SGI157693[] = {{&S54261[93], 9}, {NULL}};
const Segment SGI157694[] = {{&S54261[66], 6}, {NULL}};
const Segment SGI157695[] = {{&S54262[34], 9}, {NULL}};
const Segment SGI157696[] = {{&S54262[27], 6}, {NULL}};
const Segment SGI157697[] = {{&S54262[96], 9}, {NULL}};
const Segment SGI157698[] = {{&S54262[69], 6}, {NULL}};
const Segment SGI157699[] = {{&S54265[34], 7}, {NULL}};
const Segment SGI157700[] = {{&S54265[27], 6}, {NULL}};
const Segment SGI157701[] = {{&S54265[94], 9}, {NULL}};
const Segment SGI157702[] = {{&S54265[67], 6}, {NULL}};
const Segment SGI157703[] = {{&S54266[34], 14}, {NULL}};
const Segment SGI157704[] = {{&S43997[44], 9}, {NULL}};
const Segment SGI157705[] = {{&S43997[44], 21}, {NULL}};
const Segment SGI157706[] = {{&S54266[27], 6}, {NULL}};
const Segment SGI157707[] = {{&S43997[30], 6}, {NULL}};
const Segment SGI157708[] = {{&S43997[114], 3}, {NULL}};
const Segment SGI157709[] = {{&S54266[101], 9}, {NULL}};
const Segment SGI157710[] = {{&S54266[74], 6}, {NULL}};
const Segment SGI157711[] = {{&S43997[79], 6}, {NULL}};
const Segment SGI157712[] = {{&S43998[38], 9}, {NULL}};
const Segment SGI157713[] = {{&S54267[34], 9}, {NULL}};
const Segment SGI157714[] = {{&S43998[31], 6}, {NULL}};
const Segment SGI157715[] = {{&S54267[27], 6}, {NULL}};
const Segment SGI157716[] = {{&S54267[96], 9}, {NULL}};
const Segment SGI157717[] = {{&S54267[69], 6}, {NULL}};
const Segment SGI157718[] = {{&S43999[44], 10}, {NULL}};
const Segment SGI157719[] = {{&S43999[44], 22}, {NULL}};
const Segment SGI157720[] = {{&S43999[30], 6}, {NULL}};
const Segment SGI157721[] = {{&S54268[34], 6}, {NULL}};
const Segment SGI157722[] = {{&S54268[27], 6}, {NULL}};
const Segment SGI157723[] = {{&S43999[115], 3}, {NULL}};
const Segment SGI157724[] = {{&S43999[80], 6}, {NULL}};
const Segment SGI157725[] = {{&S54268[93], 9}, {NULL}};
const Segment SGI157726[] = {{&S54268[66], 6}, {NULL}};
const Segment SGI157727[] = {{&S44000[44], 9}, {NULL}};
const Segment SGI157728[] = {{&S44000[44], 21}, {NULL}};
const Segment SGI157729[] = {{&S44000[30], 6}, {NULL}};
const Segment SGI157730[] = {{&S54269[34], 8}, {NULL}};
const Segment SGI157731[] = {{&S54269[27], 6}, {NULL}};
const Segment SGI157732[] = {{&S44000[114], 3}, {NULL}};
const Segment SGI157733[] = {{&S44000[79], 6}, {NULL}};
const Segment SGI157734[] = {{&S54269[95], 9}, {NULL}};
const Segment SGI157735[] = {{&S54269[68], 6}, {NULL}};
const Segment SGI157736[] = {{&S44001[38], 9}, {NULL}};
const Segment SGI157737[] = {{&S44001[31], 6}, {NULL}};
const Segment SGI157738[] = {{&S54270[34], 4}, {NULL}};
const Segment SGI157739[] = {{&S54270[27], 6}, {NULL}};
const Segment SGI157740[] = {{&S54270[91], 9}, {NULL}};
const Segment SGI157741[] = {{&S54270[64], 6}, {NULL}};
const Segment SGI157742[] = {{&S44002[44], 24}, {NULL}};
const Segment SGI157743[] = {{&S44002[44], 36}, {NULL}};
const Segment SGI157744[] = {{&S44002[30], 6}, {NULL}};
const Segment SGI157745[] = {{&S44002[129], 3}, {NULL}};
const Segment SGI157746[] = {{&S44002[94], 6}, {NULL}};
const Segment SGI157747[] = {{&S54271[34], 7}, {NULL}};
const Segment SGI157748[] = {{&S54271[27], 6}, {NULL}};
const Segment SGI157749[] = {{&S54271[94], 9}, {NULL}};
const Segment SGI157750[] = {{&S54271[67], 6}, {NULL}};
const Segment SGI157751[] = {{&S44003[44], 15}, {NULL}};
const Segment SGI157752[] = {{&S44003[44], 27}, {NULL}};
const Segment SGI157753[] = {{&S44003[30], 6}, {NULL}};
const Segment SGI157754[] = {{&S44003[120], 3}, {NULL}};
const Segment SGI157755[] = {{&S44003[85], 6}, {NULL}};
const Segment SGI157756[] = {{&S54272[34], 15}, {NULL}};
const Segment SGI157757[] = {{&S54272[27], 6}, {NULL}};
const Segment SGI157758[] = {{&S54272[102], 9}, {NULL}};
const Segment SGI157759[] = {{&S54272[75], 6}, {NULL}};
const Segment SGI157760[] = {{&S44004[44], 15}, {NULL}};
const Segment SGI157761[] = {{&S44004[44], 27}, {NULL}};
const Segment SGI157762[] = {{&S44004[30], 6}, {NULL}};
const Segment SGI157763[] = {{&S44004[120], 3}, {NULL}};
const Segment SGI157764[] = {{&S44004[85], 6}, {NULL}};
const Segment SGI157765[] = {{&S54273[34], 12}, {NULL}};
const Segment SGI157766[] = {{&S54273[27], 6}, {NULL}};
const Segment SGI157767[] = {{&S54273[99], 9}, {NULL}};
const Segment SGI157768[] = {{&S54273[72], 6}, {NULL}};
const Segment SGI157769[] = {{&S44005[44], 21}, {NULL}};
const Segment SGI157770[] = {{&S44005[44], 33}, {NULL}};
const Segment SGI157771[] = {{&S44005[30], 6}, {NULL}};
const Segment SGI157772[] = {{&S44005[126], 3}, {NULL}};
const Segment SGI157773[] = {{&S44005[91], 6}, {NULL}};
const Segment SGI157774[] = {{&S54274[34], 6}, {NULL}};
const Segment SGI157775[] = {{&S54274[27], 6}, {NULL}};
const Segment SGI157776[] = {{&S54274[93], 9}, {NULL}};
const Segment SGI157777[] = {{&S54274[66], 6}, {NULL}};
const Segment SGI157778[] = {{&S44006[44], 14}, {NULL}};
const Segment SGI157779[] = {{&S44006[44], 26}, {NULL}};
const Segment SGI157780[] = {{&S44006[30], 6}, {NULL}};
const Segment SGI157781[] = {{&S44006[119], 3}, {NULL}};
const Segment SGI157782[] = {{&S44006[84], 6}, {NULL}};
const Segment SGI157783[] = {{&S54275[34], 4}, {NULL}};
const Segment SGI157784[] = {{&S54275[27], 6}, {NULL}};
const Segment SGI157785[] = {{&S54275[91], 9}, {NULL}};
const Segment SGI157786[] = {{&S54275[64], 6}, {NULL}};
const Segment SGI157787[] = {{&S44007[38], 10}, {NULL}};
const Segment SGI157788[] = {{&S44007[31], 6}, {NULL}};
const Segment SGI157789[] = {{&S54276[34], 4}, {NULL}};
const Segment SGI157790[] = {{&S54276[27], 6}, {NULL}};
const Segment SGI157791[] = {{&S54276[91], 9}, {NULL}};
const Segment SGI157792[] = {{&S54276[64], 6}, {NULL}};
const Segment SGI157793[] = {{&S44008[41], 9}, {NULL}};
const Segment SGI157794[] = {{&S44008[30], 6}, {NULL}};
const Segment SGI157795[] = {{&S44008[111], 3}, {NULL}};
const Segment SGI157796[] = {{&S44008[76], 6}, {NULL}};
const Segment SGI157797[] = {{&S54277[34], 8}, {NULL}};
const Segment SGI157798[] = {{&S54277[27], 6}, {NULL}};
const Segment SGI157799[] = {{&S54277[95], 9}, {NULL}};
const Segment SGI157800[] = {{&S54277[68], 6}, {NULL}};
const Segment SGI157801[] = {{&S44009[44], 10}, {NULL}};
const Segment SGI157802[] = {{&S44009[44], 22}, {NULL}};
const Segment SGI157803[] = {{&S44009[30], 6}, {NULL}};
const Segment SGI157804[] = {{&S44009[115], 3}, {NULL}};
const Segment SGI157805[] = {{&S44009[80], 6}, {NULL}};
const Segment SGI157806[] = {{&S54278[34], 7}, {NULL}};
const Segment SGI157807[] = {{&S54278[27], 6}, {NULL}};
const Segment SGI157808[] = {{&S54278[94], 9}, {NULL}};
const Segment SGI157809[] = {{&S54278[67], 6}, {NULL}};
const Segment SGI157810[] = {{&S54282[34], 7}, {NULL}};
const Segment SGI157811[] = {{&S54282[27], 6}, {NULL}};
const Segment SGI157812[] = {{&S54282[94], 9}, {NULL}};
const Segment SGI157813[] = {{&S54282[67], 6}, {NULL}};
const Segment SGI157814[] = {{&S54283[34], 9}, {NULL}};
const Segment SGI157815[] = {{&S54283[27], 6}, {NULL}};
const Segment SGI157816[] = {{&S54283[96], 9}, {NULL}};
const Segment SGI157817[] = {{&S54283[69], 6}, {NULL}};
const Segment SGI157818[] = {{&S54284[34], 9}, {NULL}};
const Segment SGI157819[] = {{&S54284[27], 6}, {NULL}};
const Segment SGI157820[] = {{&S54284[96], 9}, {NULL}};
const Segment SGI157821[] = {{&S54284[69], 6}, {NULL}};
const Segment SGI157822[] = {{&S54285[34], 11}, {NULL}};
const Segment SGI157823[] = {{&S54285[27], 6}, {NULL}};
const Segment SGI157824[] = {{&S54285[98], 9}, {NULL}};
const Segment SGI157825[] = {{&S54285[71], 6}, {NULL}};
const Segment SGI157826[] = {{&S54287[34], 8}, {NULL}};
const Segment SGI157827[] = {{&S54287[27], 6}, {NULL}};
const Segment SGI157828[] = {{&S54287[95], 9}, {NULL}};
const Segment SGI157829[] = {{&S54287[68], 6}, {NULL}};
const Segment SGI157830[] = {{&S54290[34], 11}, {NULL}};
const Segment SGI157831[] = {{&S54290[27], 6}, {NULL}};
const Segment SGI157832[] = {{&S54290[98], 9}, {NULL}};
const Segment SGI157833[] = {{&S54290[71], 6}, {NULL}};
const Segment SGI157834[] = {{&S54291[27], 6}, {NULL}};
const Segment SGI157835[] = {{&S54291[111], 9}, {NULL}};
const Segment SGI157836[] = {{&S54291[84], 6}, {NULL}};
const Segment SGI157837[] = {{&S54292[34], 11}, {NULL}};
const Segment SGI157838[] = {{&S54292[27], 6}, {NULL}};
const Segment SGI157839[] = {{&S54292[98], 9}, {NULL}};
const Segment SGI157840[] = {{&S54292[71], 6}, {NULL}};
const Segment SGI157841[] = {{&S54293[34], 10}, {NULL}};
const Segment SGI157842[] = {{&S54293[27], 6}, {NULL}};
const Segment SGI157843[] = {{&S54293[97], 9}, {NULL}};
const Segment SGI157844[] = {{&S54293[70], 6}, {NULL}};
const Segment SGI157845[] = {{&S54294[34], 11}, {NULL}};
const Segment SGI157846[] = {{&S54294[27], 6}, {NULL}};
const Segment SGI157847[] = {{&S54294[106], 9}, {NULL}};
const Segment SGI157848[] = {{&S54294[79], 6}, {NULL}};
const Segment SGI157849[] = {{&S54295[34], 7}, {NULL}};
const Segment SGI157850[] = {{&S54295[27], 6}, {NULL}};
const Segment SGI157851[] = {{&S54295[94], 9}, {NULL}};
const Segment SGI157852[] = {{&S54295[67], 6}, {NULL}};
const Segment SGI157853[] = {{&S54296[34], 6}, {NULL}};
const Segment SGI157854[] = {{&S54296[27], 6}, {NULL}};
const Segment SGI157855[] = {{&S54296[93], 9}, {NULL}};
const Segment SGI157856[] = {{&S54296[66], 6}, {NULL}};
const Segment SGI157857[] = {{&S54297[34], 5}, {NULL}};
const Segment SGI157858[] = {{&S54297[27], 6}, {NULL}};
const Segment SGI157859[] = {{&S54297[92], 9}, {NULL}};
const Segment SGI157860[] = {{&S54297[65], 6}, {NULL}};
const Segment SGI157861[] = {{&S54298[34], 7}, {NULL}};
const Segment SGI157862[] = {{&S54298[27], 6}, {NULL}};
const Segment SGI157863[] = {{&S54298[94], 9}, {NULL}};
const Segment SGI157864[] = {{&S54298[67], 6}, {NULL}};
const Segment SGI157865[] = {{&S54299[34], 11}, {NULL}};
const Segment SGI157866[] = {{&S54299[27], 6}, {NULL}};
const Segment SGI157867[] = {{&S54299[98], 9}, {NULL}};
const Segment SGI157868[] = {{&S54299[71], 6}, {NULL}};
const Segment SGI157869[] = {{&S54300[34], 5}, {NULL}};
const Segment SGI157870[] = {{&S54300[27], 6}, {NULL}};
const Segment SGI157871[] = {{&S54300[92], 9}, {NULL}};
const Segment SGI157872[] = {{&S54300[65], 6}, {NULL}};
const Segment SGI157873[] = {{&S54301[34], 4}, {NULL}};
const Segment SGI157874[] = {{&S54301[27], 6}, {NULL}};
const Segment SGI157875[] = {{&S54301[91], 9}, {NULL}};
const Segment SGI157876[] = {{&S54301[64], 6}, {NULL}};
const Segment SGI157877[] = {{&S54302[34], 19}, {NULL}};
const Segment SGI157878[] = {{&S54302[27], 6}, {NULL}};
const Segment SGI157879[] = {{&S54302[106], 9}, {NULL}};
const Segment SGI157880[] = {{&S54302[79], 6}, {NULL}};
const Segment SGI157881[] = {{&S44059[44], 12}, {NULL}};
const Segment SGI157882[] = {{&S44059[44], 24}, {NULL}};
const Segment SGI157883[] = {{&S44059[30], 6}, {NULL}};
const Segment SGI157884[] = {{&S44059[117], 3}, {NULL}};
const Segment SGI157885[] = {{&S44059[82], 6}, {NULL}};
const Segment SGI157886[] = {{&S54304[34], 8}, {NULL}};
const Segment SGI157887[] = {{&S54304[27], 6}, {NULL}};
const Segment SGI157888[] = {{&S54304[95], 9}, {NULL}};
const Segment SGI157889[] = {{&S54304[68], 6}, {NULL}};
const Segment SGI157890[] = {{&S44061[44], 12}, {NULL}};
const Segment SGI157891[] = {{&S44061[44], 24}, {NULL}};
const Segment SGI157892[] = {{&S54305[34], 15}, {NULL}};
const Segment SGI157893[] = {{&S44061[30], 6}, {NULL}};
const Segment SGI157894[] = {{&S54305[27], 6}, {NULL}};
const Segment SGI157895[] = {{&S44061[117], 3}, {NULL}};
const Segment SGI157896[] = {{&S44061[82], 6}, {NULL}};
const Segment SGI157897[] = {{&S54305[102], 9}, {NULL}};
const Segment SGI157898[] = {{&S54305[75], 6}, {NULL}};
const Segment SGI157899[] = {{&S54306[34], 13}, {NULL}};
const Segment SGI157900[] = {{&S54306[27], 6}, {NULL}};
const Segment SGI157901[] = {{&S54306[100], 9}, {NULL}};
const Segment SGI157902[] = {{&S54306[73], 6}, {NULL}};
const Segment SGI157903[] = {{&S54307[34], 4}, {NULL}};
const Segment SGI157904[] = {{&S54307[27], 6}, {NULL}};
const Segment SGI157905[] = {{&S54307[91], 9}, {NULL}};
const Segment SGI157906[] = {{&S54307[64], 6}, {NULL}};
const Segment SGI157907[] = {{&S54308[34], 7}, {NULL}};
const Segment SGI157908[] = {{&S54308[27], 6}, {NULL}};
const Segment SGI157909[] = {{&S54308[94], 9}, {NULL}};
const Segment SGI157910[] = {{&S54308[67], 6}, {NULL}};
const Segment SGI157911[] = {{&S54311[34], 7}, {NULL}};
const Segment SGI157912[] = {{&S54311[27], 6}, {NULL}};
const Segment SGI157913[] = {{&S54311[94], 9}, {NULL}};
const Segment SGI157914[] = {{&S54311[67], 6}, {NULL}};
const Segment SGI157915[] = {{&S54312[34], 7}, {NULL}};
const Segment SGI157916[] = {{&S54312[27], 6}, {NULL}};
const Segment SGI157917[] = {{&S54312[94], 9}, {NULL}};
const Segment SGI157918[] = {{&S54312[67], 6}, {NULL}};
const Segment SGI157919[] = {{&S54313[34], 19}, {NULL}};
const Segment SGI157920[] = {{&S54313[27], 6}, {NULL}};
const Segment SGI157921[] = {{&S54313[106], 9}, {NULL}};
const Segment SGI157922[] = {{&S54313[79], 6}, {NULL}};
const Segment SGI157923[] = {{&S54314[34], 6}, {NULL}};
const Segment SGI157924[] = {{&S54314[27], 6}, {NULL}};
const Segment SGI157925[] = {{&S54314[93], 9}, {NULL}};
const Segment SGI157926[] = {{&S54314[66], 6}, {NULL}};
const Segment SGI157927[] = {{&S54315[34], 8}, {NULL}};
const Segment SGI157928[] = {{&S54315[27], 6}, {NULL}};
const Segment SGI157929[] = {{&S54315[95], 9}, {NULL}};
const Segment SGI157930[] = {{&S54315[68], 6}, {NULL}};
const Segment SGI157931[] = {{&S54316[34], 8}, {NULL}};
const Segment SGI157932[] = {{&S54316[27], 6}, {NULL}};
const Segment SGI157933[] = {{&S45396[34], 10}, {NULL}};
const Segment SGI157934[] = {{&S54316[95], 9}, {NULL}};
const Segment SGI157935[] = {{&S45396[27], 6}, {NULL}};
const Segment SGI157936[] = {{&S54316[68], 6}, {NULL}};
const Segment SGI157937[] = {{&S45396[97], 9}, {NULL}};
const Segment SGI157938[] = {{&S45396[70], 6}, {NULL}};
const Segment SGI157939[] = {{&S54320[34], 13}, {NULL}};
const Segment SGI157940[] = {{&S54320[27], 6}, {NULL}};
const Segment SGI157941[] = {{&S54320[100], 9}, {NULL}};
const Segment SGI157942[] = {{&S54320[73], 6}, {NULL}};
const Segment SGI157943[] = {{&S50625[44], 7}, {NULL}};
const Segment SGI157944[] = {{&S50625[44], 28}, {NULL}};
const Segment SGI157945[] = {{&S50625[30], 6}, {NULL}};
const Segment SGI157946[] = {{&S50625[31], 3}, {NULL}};
const Segment SGI157947[] = {{&S50625[86], 5}, {NULL}};
const Segment SGI157948[] = {{&S44091[34], 11}, {NULL}};
const Segment SGI157949[] = {{&S44091[27], 6}, {NULL}};
const Segment SGI157950[] = {{&S44091[98], 9}, {NULL}};
const Segment SGI157951[] = {{&S44091[71], 6}, {NULL}};
const Segment SGI157952[] = {{&S45404[34], 10}, {NULL}};
const Segment SGI157953[] = {{&S45404[27], 6}, {NULL}};
const Segment SGI157954[] = {{&S44092[34], 11}, {NULL}};
const Segment SGI157955[] = {{&S44092[27], 6}, {NULL}};
const Segment SGI157956[] = {{&S54324[34], 11}, {NULL}};
const Segment SGI157957[] = {{&S45404[97], 9}, {NULL}};
const Segment SGI157958[] = {{&S45404[70], 6}, {NULL}};
const Segment SGI157959[] = {{&S54324[27], 6}, {NULL}};
const Segment SGI157960[] = {{&S44092[98], 9}, {NULL}};
const Segment SGI157961[] = {{&S44092[71], 6}, {NULL}};
const Segment SGI157962[] = {{&S54324[98], 9}, {NULL}};
const Segment SGI157963[] = {{&S54324[71], 6}, {NULL}};
const Segment SGI157964[] = {{&S44093[34], 11}, {NULL}};
const Segment SGI157965[] = {{&S44093[27], 6}, {NULL}};
const Segment SGI157966[] = {{&S54325[34], 7}, {NULL}};
const Segment SGI157967[] = {{&S44093[98], 9}, {NULL}};
const Segment SGI157968[] = {{&S54325[27], 6}, {NULL}};
const Segment SGI157969[] = {{&S44093[71], 6}, {NULL}};
const Segment SGI157970[] = {{&S54325[98], 9}, {NULL}};
const Segment SGI157971[] = {{&S54325[71], 6}, {NULL}};
const Segment SGI157972[] = {{&S54326[34], 7}, {NULL}};
const Segment SGI157973[] = {{&S54326[27], 6}, {NULL}};
const Segment SGI157974[] = {{&S44095[34], 10}, {NULL}};
const Segment SGI157975[] = {{&S44095[27], 6}, {NULL}};
const Segment SGI157976[] = {{&S54326[99], 9}, {NULL}};
const Segment SGI157977[] = {{&S54326[72], 6}, {NULL}};
const Segment SGI157978[] = {{&S44095[97], 9}, {NULL}};
const Segment SGI157979[] = {{&S44095[70], 6}, {NULL}};
const Segment SGI157980[] = {{&S44096[34], 10}, {NULL}};
const Segment SGI157981[] = {{&S44096[27], 6}, {NULL}};
const Segment SGI157982[] = {{&S44096[97], 9}, {NULL}};
const Segment SGI157983[] = {{&S44096[70], 6}, {NULL}};
const Segment SGI157984[] = {{&S44097[34], 11}, {NULL}};
const Segment SGI157985[] = {{&S44097[27], 6}, {NULL}};
const Segment SGI157986[] = {{&S44097[98], 9}, {NULL}};
const Segment SGI157987[] = {{&S44097[71], 6}, {NULL}};
const Segment SGI157988[] = {{&S44098[34], 11}, {NULL}};
const Segment SGI157989[] = {{&S44098[27], 6}, {NULL}};
const Segment SGI157990[] = {{&S44098[98], 9}, {NULL}};
const Segment SGI157991[] = {{&S44098[71], 6}, {NULL}};
const Segment SGI157992[] = {{&S44099[34], 11}, {NULL}};
const Segment SGI157993[] = {{&S44099[27], 6}, {NULL}};
const Segment SGI157994[] = {{&S44099[98], 9}, {NULL}};
const Segment SGI157995[] = {{&S44099[71], 6}, {NULL}};
const Segment SGI157996[] = {{&S54360[34], 18}, {NULL}};
const Segment SGI157997[] = {{&S54360[27], 6}, {NULL}};
const Segment SGI157998[] = {{&S54360[105], 9}, {NULL}};
const Segment SGI157999[] = {{&S54360[78], 6}, {NULL}};
const Segment SGI158000[] = {{&S45448[34], 10}, {NULL}};
const Segment SGI158001[] = {{&S45448[27], 6}, {NULL}};
const Segment SGI158002[] = {{&S45448[97], 9}, {NULL}};
const Segment SGI158003[] = {{&S45448[70], 6}, {NULL}};
const Segment SGI158004[] = {{&S54361[34], 18}, {NULL}};
const Segment SGI158005[] = {{&S54361[27], 6}, {NULL}};
const Segment SGI158006[] = {{&S54361[105], 9}, {NULL}};
const Segment SGI158007[] = {{&S54361[78], 6}, {NULL}};
const Segment SGI158008[] = {{&S45450[34], 10}, {NULL}};
const Segment SGI158009[] = {{&S45450[27], 6}, {NULL}};
const Segment SGI158010[] = {{&S45450[97], 9}, {NULL}};
const Segment SGI158011[] = {{&S45450[70], 6}, {NULL}};
const Segment SGI158012[] = {{&S45456[34], 7}, {NULL}};
const Segment SGI158013[] = {{&S45456[27], 6}, {NULL}};
const Segment SGI158014[] = {{&S45456[94], 9}, {NULL}};
const Segment SGI158015[] = {{&S45456[67], 6}, {NULL}};
const Segment SGI158016[] = {{&S54388[34], 12}, {NULL}};
const Segment SGI158017[] = {{&S54388[27], 6}, {NULL}};
const Segment SGI158018[] = {{&S54388[99], 9}, {NULL}};
const Segment SGI158019[] = {{&S54388[72], 6}, {NULL}};
const Segment SGI158020[] = {{&S54391[44], 12}, {NULL}};
const Segment SGI158021[] = {{&S54391[44], 29}, {NULL}};
const Segment SGI158022[] = {{&S54391[30], 6}, {NULL}};
const Segment SGI158023[] = {{&S54391[31], 3}, {NULL}};
const Segment SGI158024[] = {{&S54391[87], 6}, {NULL}};
const Segment SGI158025[] = {{&S54392[44], 8}, {NULL}};
const Segment SGI158026[] = {{&S54392[44], 25}, {NULL}};
const Segment SGI158027[] = {{&S54392[30], 6}, {NULL}};
const Segment SGI158028[] = {{&S54392[31], 3}, {NULL}};
const Segment SGI158029[] = {{&S54392[83], 6}, {NULL}};
const Segment SGI158030[] = {{&S54393[38], 8}, {NULL}};
const Segment SGI158031[] = {{&S54393[31], 6}, {NULL}};
const Segment SGI158032[] = {{&S54394[44], 12}, {NULL}};
const Segment SGI158033[] = {{&S54394[44], 29}, {NULL}};
const Segment SGI158034[] = {{&S54394[30], 6}, {NULL}};
const Segment SGI158035[] = {{&S54394[31], 3}, {NULL}};
const Segment SGI158036[] = {{&S54394[87], 6}, {NULL}};
const Segment SGI158037[] = {{&S54395[44], 8}, {NULL}};
const Segment SGI158038[] = {{&S54395[44], 25}, {NULL}};
const Segment SGI158039[] = {{&S54395[30], 6}, {NULL}};
const Segment SGI158040[] = {{&S54395[31], 3}, {NULL}};
const Segment SGI158041[] = {{&S54395[83], 6}, {NULL}};
const Segment SGI158042[] = {{&S54396[44], 12}, {NULL}};
const Segment SGI158043[] = {{&S54396[44], 29}, {NULL}};
const Segment SGI158044[] = {{&S54396[30], 6}, {NULL}};
const Segment SGI158045[] = {{&S54396[31], 3}, {NULL}};
const Segment SGI158046[] = {{&S54396[87], 6}, {NULL}};
const Segment SGI158047[] = {{&S54397[44], 8}, {NULL}};
const Segment SGI158048[] = {{&S54397[44], 25}, {NULL}};
const Segment SGI158049[] = {{&S54397[30], 6}, {NULL}};
const Segment SGI158050[] = {{&S54397[31], 3}, {NULL}};
const Segment SGI158051[] = {{&S54397[83], 6}, {NULL}};
const Segment SGI158052[] = {{&S54401[44], 16}, {NULL}};
const Segment SGI158053[] = {{&S54401[44], 28}, {NULL}};
const Segment SGI158054[] = {{&S54401[30], 6}, {NULL}};
const Segment SGI158055[] = {{&S54401[31], 3}, {NULL}};
const Segment SGI158056[] = {{&S54401[86], 6}, {NULL}};
const Segment SGI158057[] = {{&S54402[38], 16}, {NULL}};
const Segment SGI158058[] = {{&S54402[31], 6}, {NULL}};
const Segment SGI158059[] = {{&S54403[34], 7}, {NULL}};
const Segment SGI158060[] = {{&S54403[27], 6}, {NULL}};
const Segment SGI158061[] = {{&S54403[100], 9}, {NULL}};
const Segment SGI158062[] = {{&S54403[73], 6}, {NULL}};
const Segment SGI158063[] = {{&S54404[34], 7}, {NULL}};
const Segment SGI158064[] = {{&S54404[27], 6}, {NULL}};
const Segment SGI158065[] = {{&S54404[100], 9}, {NULL}};
const Segment SGI158066[] = {{&S54404[73], 6}, {NULL}};
const Segment SGI158067[] = {{&S45528[34], 11}, {NULL}};
const Segment SGI158068[] = {{&S45528[27], 6}, {NULL}};
const Segment SGI158069[] = {{&S45528[98], 9}, {NULL}};
const Segment SGI158070[] = {{&S45528[71], 6}, {NULL}};
const Segment SGI158071[] = {{&S45529[34], 11}, {NULL}};
const Segment SGI158072[] = {{&S45529[27], 6}, {NULL}};
const Segment SGI158073[] = {{&S45529[98], 9}, {NULL}};
const Segment SGI158074[] = {{&S45529[71], 6}, {NULL}};
const Segment SGI158075[] = {{&S44195[34], 10}, {NULL}};
const Segment SGI158076[] = {{&S44195[27], 6}, {NULL}};
const Segment SGI158077[] = {{&S45531[34], 27}, {NULL}};
const Segment SGI158078[] = {{&S45531[27], 6}, {NULL}};
const Segment SGI158079[] = {{&S44195[97], 9}, {NULL}};
const Segment SGI158080[] = {{&S44195[70], 6}, {NULL}};
const Segment SGI158081[] = {{&S45531[114], 9}, {NULL}};
const Segment SGI158082[] = {{&S44195[139], 8}, {NULL}};
const Segment SGI158083[] = {{&S45531[87], 6}, {NULL}};
const Segment SGI158084[] = {{&S44196[34], 10}, {NULL}};
const Segment SGI158085[] = {{&S44196[27], 6}, {NULL}};
const Segment SGI158086[] = {{&S44196[97], 9}, {NULL}};
const Segment SGI158087[] = {{&S44196[70], 6}, {NULL}};
const Segment SGI158088[] = {{&S44196[138], 8}, {NULL}};
const Segment SGI158089[] = {{&S44197[34], 10}, {NULL}};
const Segment SGI158090[] = {{&S44197[27], 6}, {NULL}};
const Segment SGI158091[] = {{&S44197[97], 9}, {NULL}};
const Segment SGI158092[] = {{&S44197[70], 6}, {NULL}};
const Segment SGI158093[] = {{&S44197[138], 8}, {NULL}};
const Segment SGI158094[] = {{&S44198[34], 10}, {NULL}};
const Segment SGI158095[] = {{&S44198[27], 6}, {NULL}};
const Segment SGI158096[] = {{&S44198[97], 9}, {NULL}};
const Segment SGI158097[] = {{&S44198[70], 6}, {NULL}};
const Segment SGI158098[] = {{&S44198[138], 8}, {NULL}};
const Segment SGI158099[] = {{&S44200[34], 11}, {NULL}};
const Segment SGI158100[] = {{&S44200[27], 6}, {NULL}};
const Segment SGI158101[] = {{&S44200[98], 9}, {NULL}};
const Segment SGI158102[] = {{&S44200[71], 6}, {NULL}};
const Segment SGI158103[] = {{&S44200[139], 8}, {NULL}};
const Segment SGI158104[] = {{&S45541[34], 12}, {NULL}};
const Segment SGI158105[] = {{&S45541[27], 6}, {NULL}};
const Segment SGI158106[] = {{&S45541[99], 9}, {NULL}};
const Segment SGI158107[] = {{&S45541[72], 6}, {NULL}};
const Segment SGI158108[] = {{&S54447[34], 14}, {NULL}};
const Segment SGI158109[] = {{&S54447[27], 6}, {NULL}};
const Segment SGI158110[] = {{&S45542[34], 14}, {NULL}};
const Segment SGI158111[] = {{&S54447[101], 9}, {NULL}};
const Segment SGI158112[] = {{&S54447[74], 6}, {NULL}};
const Segment SGI158113[] = {{&S45542[27], 6}, {NULL}};
const Segment SGI158114[] = {{&S45542[101], 9}, {NULL}};
const Segment SGI158115[] = {{&S45542[74], 6}, {NULL}};
const Segment SGI158116[] = {{&S54455[44], 9}, {NULL}};
const Segment SGI158117[] = {{&S54455[44], 21}, {NULL}};
const Segment SGI158118[] = {{&S54455[30], 6}, {NULL}};
const Segment SGI158119[] = {{&S54455[31], 3}, {NULL}};
const Segment SGI158120[] = {{&S54455[79], 6}, {NULL}};
const Segment SGI158121[] = {{&S45552[44], 11}, {NULL}};
const Segment SGI158122[] = {{&S45552[44], 23}, {NULL}};
const Segment SGI158123[] = {{&S45552[30], 6}, {NULL}};
const Segment SGI158124[] = {{&S45552[116], 3}, {NULL}};
const Segment SGI158125[] = {{&S45552[81], 6}, {NULL}};
const Segment SGI158126[] = {{&S45555[38], 11}, {NULL}};
const Segment SGI158127[] = {{&S45555[31], 6}, {NULL}};
const Segment SGI158128[] = {{&S45558[44], 10}, {NULL}};
const Segment SGI158129[] = {{&S45558[44], 22}, {NULL}};
const Segment SGI158130[] = {{&S45558[30], 6}, {NULL}};
const Segment SGI158131[] = {{&S45558[115], 3}, {NULL}};
const Segment SGI158132[] = {{&S45558[80], 6}, {NULL}};
const Segment SGI158133[] = {{&S54486[34], 12}, {NULL}};
const Segment SGI158134[] = {{&S54486[27], 6}, {NULL}};
const Segment SGI158135[] = {{&S45579[34], 7}, {NULL}};
const Segment SGI158136[] = {{&S54486[99], 9}, {NULL}};
const Segment SGI158137[] = {{&S54486[72], 6}, {NULL}};
const Segment SGI158138[] = {{&S45579[27], 6}, {NULL}};
const Segment SGI158139[] = {{&S45579[99], 9}, {NULL}};
const Segment SGI158140[] = {{&S45579[72], 6}, {NULL}};
const Segment SGI158141[] = {{&S54489[34], 11}, {NULL}};
const Segment SGI158142[] = {{&S54489[27], 6}, {NULL}};
const Segment SGI158143[] = {{&S54489[98], 9}, {NULL}};
const Segment SGI158144[] = {{&S54489[71], 6}, {NULL}};
const Segment SGI158145[] = {{&S54490[34], 10}, {NULL}};
const Segment SGI158146[] = {{&S54490[27], 6}, {NULL}};
const Segment SGI158147[] = {{&S54490[97], 9}, {NULL}};
const Segment SGI158148[] = {{&S54490[70], 6}, {NULL}};
const Segment SGI158149[] = {{&S54491[34], 11}, {NULL}};
const Segment SGI158150[] = {{&S54491[27], 6}, {NULL}};
const Segment SGI158151[] = {{&S54491[98], 9}, {NULL}};
const Segment SGI158152[] = {{&S54491[71], 6}, {NULL}};
const Segment SGI158153[] = {{&S54492[34], 11}, {NULL}};
const Segment SGI158154[] = {{&S54492[27], 6}, {NULL}};
const Segment SGI158155[] = {{&S54492[98], 9}, {NULL}};
const Segment SGI158156[] = {{&S54492[71], 6}, {NULL}};
const Segment SGI158157[] = {{&S45591[34], 11}, {NULL}};
const Segment SGI158158[] = {{&S45591[27], 6}, {NULL}};
const Segment SGI158159[] = {{&S54493[44], 7}, {NULL}};
const Segment SGI158160[] = {{&S54493[44], 19}, {NULL}};
const Segment SGI158161[] = {{&S54493[30], 6}, {NULL}};
const Segment SGI158162[] = {{&S45591[98], 9}, {NULL}};
const Segment SGI158163[] = {{&S45591[71], 6}, {NULL}};
const Segment SGI158164[] = {{&S54493[80], 6}, {NULL}};
const Segment SGI158165[] = {{&S45592[34], 11}, {NULL}};
const Segment SGI158166[] = {{&S45592[27], 6}, {NULL}};
const Segment SGI158167[] = {{&S54494[44], 7}, {NULL}};
const Segment SGI158168[] = {{&S54494[44], 19}, {NULL}};
const Segment SGI158169[] = {{&S54494[30], 6}, {NULL}};
const Segment SGI158170[] = {{&S45592[98], 9}, {NULL}};
const Segment SGI158171[] = {{&S45592[71], 6}, {NULL}};
const Segment SGI158172[] = {{&S54494[31], 3}, {NULL}};
const Segment SGI158173[] = {{&S54494[77], 6}, {NULL}};
const Segment SGI158174[] = {{&S54495[44], 8}, {NULL}};
const Segment SGI158175[] = {{&S54495[44], 20}, {NULL}};
const Segment SGI158176[] = {{&S54495[30], 6}, {NULL}};
const Segment SGI158177[] = {{&S45594[34], 27}, {NULL}};
const Segment SGI158178[] = {{&S54495[81], 6}, {NULL}};
const Segment SGI158179[] = {{&S45594[27], 6}, {NULL}};
const Segment SGI158180[] = {{&S45594[114], 9}, {NULL}};
const Segment SGI158181[] = {{&S45594[87], 6}, {NULL}};
const Segment SGI158182[] = {{&S54496[44], 8}, {NULL}};
const Segment SGI158183[] = {{&S54496[44], 20}, {NULL}};
const Segment SGI158184[] = {{&S54496[30], 6}, {NULL}};
const Segment SGI158185[] = {{&S54496[31], 3}, {NULL}};
const Segment SGI158186[] = {{&S54496[78], 6}, {NULL}};
const Segment SGI158187[] = {{&S45595[34], 11}, {NULL}};
const Segment SGI158188[] = {{&S45595[27], 6}, {NULL}};
const Segment SGI158189[] = {{&S45595[98], 9}, {NULL}};
const Segment SGI158190[] = {{&S45595[71], 6}, {NULL}};
const Segment SGI158191[] = {{&S54497[44], 19}, {NULL}};
const Segment SGI158192[] = {{&S54497[30], 6}, {NULL}};
const Segment SGI158193[] = {{&S54498[44], 20}, {NULL}};
const Segment SGI158194[] = {{&S54498[30], 6}, {NULL}};
const Segment SGI158195[] = {{&S54499[44], 7}, {NULL}};
const Segment SGI158196[] = {{&S54499[44], 19}, {NULL}};
const Segment SGI158197[] = {{&S54499[30], 6}, {NULL}};
const Segment SGI158198[] = {{&S54499[80], 4}, {NULL}};
const Segment SGI158199[] = {{&S54499[80], 3}, {NULL}};
const Segment SGI158200[] = {{&S54500[38], 7}, {NULL}};
const Segment SGI158201[] = {{&S54500[31], 6}, {NULL}};
const Segment SGI158202[] = {{&S54501[31], 6}, {NULL}};
const Segment SGI158203[] = {{&S44284[44], 6}, {NULL}};
const Segment SGI158204[] = {{&S44284[44], 18}, {NULL}};
const Segment SGI158205[] = {{&S44284[30], 6}, {NULL}};
const Segment SGI158206[] = {{&S44284[111], 3}, {NULL}};
const Segment SGI158207[] = {{&S44284[76], 6}, {NULL}};
const Segment SGI158208[] = {{&S44285[38], 6}, {NULL}};
const Segment SGI158209[] = {{&S44285[31], 6}, {NULL}};
const Segment SGI158210[] = {{&S44290[44], 15}, {NULL}};
const Segment SGI158211[] = {{&S44290[44], 27}, {NULL}};
const Segment SGI158212[] = {{&S44290[30], 6}, {NULL}};
const Segment SGI158213[] = {{&S44290[120], 3}, {NULL}};
const Segment SGI158214[] = {{&S44290[85], 6}, {NULL}};
const Segment SGI158215[] = {{&S30639[44], 9}, {NULL}};
const Segment SGI158216[] = {{&S54519[34], 15}, {NULL}};
const Segment SGI158217[] = {{&S30639[44], 20}, {NULL}};
const Segment SGI158218[] = {{&S54519[27], 6}, {NULL}};
const Segment SGI158219[] = {{&S30639[30], 6}, {NULL}};
const Segment SGI158220[] = {{&S44293[44], 15}, {NULL}};
const Segment SGI158221[] = {{&S44293[44], 27}, {NULL}};
const Segment SGI158222[] = {{&S44293[30], 6}, {NULL}};
const Segment SGI158223[] = {{&S30639[113], 3}, {NULL}};
const Segment SGI158224[] = {{&S54519[102], 9}, {NULL}};
const Segment SGI158225[] = {{&S54519[75], 6}, {NULL}};
const Segment SGI158226[] = {{&S44293[120], 3}, {NULL}};
const Segment SGI158227[] = {{&S30639[78], 6}, {NULL}};
const Segment SGI158228[] = {{&S44293[85], 6}, {NULL}};
const Segment SGI158229[] = {{&S44294[44], 6}, {NULL}};
const Segment SGI158230[] = {{&S44294[44], 18}, {NULL}};
const Segment SGI158231[] = {{&S44294[30], 6}, {NULL}};
const Segment SGI158232[] = {{&S45626[34], 10}, {NULL}};
const Segment SGI158233[] = {{&S44294[111], 3}, {NULL}};
const Segment SGI158234[] = {{&S45626[27], 6}, {NULL}};
const Segment SGI158235[] = {{&S44294[76], 6}, {NULL}};
const Segment SGI158236[] = {{&S45626[97], 9}, {NULL}};
const Segment SGI158237[] = {{&S45626[70], 6}, {NULL}};
const Segment SGI158238[] = {{&S54522[44], 10}, {NULL}};
const Segment SGI158239[] = {{&S54522[44], 22}, {NULL}};
const Segment SGI158240[] = {{&S54522[30], 6}, {NULL}};
const Segment SGI158241[] = {{&S54522[31], 3}, {NULL}};
const Segment SGI158242[] = {{&S54522[80], 6}, {NULL}};
const Segment SGI158243[] = {{&S54523[38], 10}, {NULL}};
const Segment SGI158244[] = {{&S54523[31], 6}, {NULL}};
const Segment SGI158245[] = {{&S54526[34], 13}, {NULL}};
const Segment SGI158246[] = {{&S54526[27], 6}, {NULL}};
const Segment SGI158247[] = {{&S54526[100], 9}, {NULL}};
const Segment SGI158248[] = {{&S54526[73], 6}, {NULL}};
const Segment SGI158249[] = {{&S54528[34], 10}, {NULL}};
const Segment SGI158250[] = {{&S54528[27], 6}, {NULL}};
const Segment SGI158251[] = {{&S54528[97], 9}, {NULL}};
const Segment SGI158252[] = {{&S54528[70], 6}, {NULL}};
const Segment SGI158253[] = {{&S40340[44], 8}, {NULL}};
const Segment SGI158254[] = {{&S40340[44], 20}, {NULL}};
const Segment SGI158255[] = {{&S40340[30], 6}, {NULL}};
const Segment SGI158256[] = {{&S40340[113], 3}, {NULL}};
const Segment SGI158257[] = {{&S40340[78], 6}, {NULL}};
const Segment SGI158258[] = {{&S40341[38], 8}, {NULL}};
const Segment SGI158259[] = {{&S40341[31], 6}, {NULL}};
const Segment SGI158260[] = {{&S54532[44], 16}, {NULL}};
const Segment SGI158261[] = {{&S54532[44], 28}, {NULL}};
const Segment SGI158262[] = {{&S54532[30], 6}, {NULL}};
const Segment SGI158263[] = {{&S54532[31], 3}, {NULL}};
const Segment SGI158264[] = {{&S54532[86], 6}, {NULL}};
const Segment SGI158265[] = {{&S54533[44], 20}, {NULL}};
const Segment SGI158266[] = {{&S54533[44], 32}, {NULL}};
const Segment SGI158267[] = {{&S54533[30], 6}, {NULL}};
const Segment SGI158268[] = {{&S54533[31], 3}, {NULL}};
const Segment SGI158269[] = {{&S54533[90], 6}, {NULL}};
const Segment SGI158270[] = {{&S54534[44], 17}, {NULL}};
const Segment SGI158271[] = {{&S44314[34], 7}, {NULL}};
const Segment SGI158272[] = {{&S54534[44], 29}, {NULL}};
const Segment SGI158273[] = {{&S54534[30], 6}, {NULL}};
const Segment SGI158274[] = {{&S44314[27], 6}, {NULL}};
const Segment SGI158275[] = {{&S54534[31], 3}, {NULL}};
const Segment SGI158276[] = {{&S44314[94], 9}, {NULL}};
const Segment SGI158277[] = {{&S54534[87], 6}, {NULL}};
const Segment SGI158278[] = {{&S44314[67], 6}, {NULL}};
const Segment SGI158279[] = {{&S44316[34], 11}, {NULL}};
const Segment SGI158280[] = {{&S44316[27], 6}, {NULL}};
const Segment SGI158281[] = {{&S44316[98], 9}, {NULL}};
const Segment SGI158282[] = {{&S44316[71], 6}, {NULL}};
const Segment SGI158283[] = {{&S44342[34], 10}, {NULL}};
const Segment SGI158284[] = {{&S44342[27], 6}, {NULL}};
const Segment SGI158285[] = {{&S44342[97], 9}, {NULL}};
const Segment SGI158286[] = {{&S44342[70], 6}, {NULL}};
const Segment SGI158287[] = {{&S45683[34], 7}, {NULL}};
const Segment SGI158288[] = {{&S45683[27], 6}, {NULL}};
const Segment SGI158289[] = {{&S45683[110], 9}, {NULL}};
const Segment SGI158290[] = {{&S45683[83], 6}, {NULL}};
const Segment SGI158291[] = {{&S54571[34], 10}, {NULL}};
const Segment SGI158292[] = {{&S54571[27], 6}, {NULL}};
const Segment SGI158293[] = {{&S54571[97], 9}, {NULL}};
const Segment SGI158294[] = {{&S54571[70], 6}, {NULL}};
const Segment SGI158295[] = {{&S45684[34], 7}, {NULL}};
const Segment SGI158296[] = {{&S45684[27], 6}, {NULL}};
const Segment SGI158297[] = {{&S45684[110], 9}, {NULL}};
const Segment SGI158298[] = {{&S45684[83], 6}, {NULL}};
const Segment SGI158299[] = {{&S54572[34], 11}, {NULL}};
const Segment SGI158300[] = {{&S54572[27], 6}, {NULL}};
const Segment SGI158301[] = {{&S54572[98], 9}, {NULL}};
const Segment SGI158302[] = {{&S54572[71], 6}, {NULL}};
const Segment SGI158303[] = {{&S54573[34], 7}, {NULL}};
const Segment SGI158304[] = {{&S54573[27], 6}, {NULL}};
const Segment SGI158305[] = {{&S54573[100], 9}, {NULL}};
const Segment SGI158306[] = {{&S54573[73], 6}, {NULL}};
const Segment SGI158307[] = {{&S54574[34], 7}, {NULL}};
const Segment SGI158308[] = {{&S54574[27], 6}, {NULL}};
const Segment SGI158309[] = {{&S54574[100], 9}, {NULL}};
const Segment SGI158310[] = {{&S54574[73], 6}, {NULL}};
const Segment SGI158311[] = {{&S30642[44], 10}, {NULL}};
const Segment SGI158312[] = {{&S30642[44], 21}, {NULL}};
const Segment SGI158313[] = {{&S30642[30], 6}, {NULL}};
const Segment SGI158314[] = {{&S54579[34], 7}, {NULL}};
const Segment SGI158315[] = {{&S54579[27], 6}, {NULL}};
const Segment SGI158316[] = {{&S30642[114], 3}, {NULL}};
const Segment SGI158317[] = {{&S30642[79], 6}, {NULL}};
const Segment SGI158318[] = {{&S54579[99], 9}, {NULL}};
const Segment SGI158319[] = {{&S54579[72], 6}, {NULL}};
const Segment SGI158320[] = {{&S30643[44], 8}, {NULL}};
const Segment SGI158321[] = {{&S30643[44], 19}, {NULL}};
const Segment SGI158322[] = {{&S30643[30], 6}, {NULL}};
const Segment SGI158323[] = {{&S54580[34], 7}, {NULL}};
const Segment SGI158324[] = {{&S30643[112], 3}, {NULL}};
const Segment SGI158325[] = {{&S54580[27], 6}, {NULL}};
const Segment SGI158326[] = {{&S30643[77], 6}, {NULL}};
const Segment SGI158327[] = {{&S54580[99], 9}, {NULL}};
const Segment SGI158328[] = {{&S54580[72], 6}, {NULL}};
const Segment SGI158329[] = {{&S44368[34], 10}, {NULL}};
const Segment SGI158330[] = {{&S44368[27], 6}, {NULL}};
const Segment SGI158331[] = {{&S44368[97], 9}, {NULL}};
const Segment SGI158332[] = {{&S44368[70], 6}, {NULL}};
const Segment SGI158333[] = {{&S44368[138], 8}, {NULL}};
const Segment SGI158334[] = {{&S44369[34], 10}, {NULL}};
const Segment SGI158335[] = {{&S44369[27], 6}, {NULL}};
const Segment SGI158336[] = {{&S44369[97], 9}, {NULL}};
const Segment SGI158337[] = {{&S44369[70], 6}, {NULL}};
const Segment SGI158338[] = {{&S30647[44], 12}, {NULL}};
const Segment SGI158339[] = {{&S44369[138], 8}, {NULL}};
const Segment SGI158340[] = {{&S30647[44], 23}, {NULL}};
const Segment SGI158341[] = {{&S30647[30], 6}, {NULL}};
const Segment SGI158342[] = {{&S30647[116], 3}, {NULL}};
const Segment SGI158343[] = {{&S30647[81], 6}, {NULL}};
const Segment SGI158344[] = {{&S44371[34], 11}, {NULL}};
const Segment SGI158345[] = {{&S44371[27], 6}, {NULL}};
const Segment SGI158346[] = {{&S44371[98], 9}, {NULL}};
const Segment SGI158347[] = {{&S44371[71], 6}, {NULL}};
const Segment SGI158348[] = {{&S30649[44], 6}, {NULL}};
const Segment SGI158349[] = {{&S30649[44], 18}, {NULL}};
const Segment SGI158350[] = {{&S44371[139], 8}, {NULL}};
const Segment SGI158351[] = {{&S30649[30], 6}, {NULL}};
const Segment SGI158352[] = {{&S30649[111], 3}, {NULL}};
const Segment SGI158353[] = {{&S30649[76], 6}, {NULL}};
const Segment SGI158354[] = {{&S44372[34], 11}, {NULL}};
const Segment SGI158355[] = {{&S44372[27], 6}, {NULL}};
const Segment SGI158356[] = {{&S44372[98], 9}, {NULL}};
const Segment SGI158357[] = {{&S44372[71], 6}, {NULL}};
const Segment SGI158358[] = {{&S44372[139], 8}, {NULL}};
const Segment SGI158359[] = {{&S44373[34], 10}, {NULL}};
const Segment SGI158360[] = {{&S44373[27], 6}, {NULL}};
const Segment SGI158361[] = {{&S44373[97], 9}, {NULL}};
const Segment SGI158362[] = {{&S44373[70], 6}, {NULL}};
const Segment SGI158363[] = {{&S44373[138], 8}, {NULL}};
const Segment SGI158364[] = {{&S30654[44], 9}, {NULL}};
const Segment SGI158365[] = {{&S30654[44], 20}, {NULL}};
const Segment SGI158366[] = {{&S30654[30], 6}, {NULL}};
const Segment SGI158367[] = {{&S30654[113], 3}, {NULL}};
const Segment SGI158368[] = {{&S30654[78], 6}, {NULL}};
const Segment SGI158369[] = {{&S30655[44], 8}, {NULL}};
const Segment SGI158370[] = {{&S30655[44], 19}, {NULL}};
const Segment SGI158371[] = {{&S30655[30], 6}, {NULL}};
const Segment SGI158372[] = {{&S30655[112], 3}, {NULL}};
const Segment SGI158373[] = {{&S30655[77], 6}, {NULL}};
const Segment SGI158374[] = {{&S54599[34], 10}, {NULL}};
const Segment SGI158375[] = {{&S54599[27], 6}, {NULL}};
const Segment SGI158376[] = {{&S54599[97], 9}, {NULL}};
const Segment SGI158377[] = {{&S54599[70], 6}, {NULL}};
const Segment SGI158378[] = {{&S30664[23], 15}, {NULL}};
const Segment SGI158379[] = {{&S30664[31], 19}, {NULL}};
const Segment SGI158380[] = {{&S54607[34], 14}, {NULL}};
const Segment SGI158381[] = {{&S54607[27], 6}, {NULL}};
const Segment SGI158382[] = {{&S54607[101], 9}, {NULL}};
const Segment SGI158383[] = {{&S54607[74], 6}, {NULL}};
const Segment SGI158384[] = {{&S30669[44], 7}, {NULL}};
const Segment SGI158385[] = {{&S30669[44], 18}, {NULL}};
const Segment SGI158386[] = {{&S30669[30], 6}, {NULL}};
const Segment SGI158387[] = {{&S30669[111], 3}, {NULL}};
const Segment SGI158388[] = {{&S30669[76], 6}, {NULL}};
const Segment SGI158389[] = {{&S30675[44], 4}, {NULL}};
const Segment SGI158390[] = {{&S30675[44], 16}, {NULL}};
const Segment SGI158391[] = {{&S30675[30], 6}, {NULL}};
const Segment SGI158392[] = {{&S30675[109], 3}, {NULL}};
const Segment SGI158393[] = {{&S30675[74], 6}, {NULL}};
const Segment SGI158394[] = {{&S42721[42], 10}, {NULL}};
const Segment SGI158395[] = {{&S42721[42], 22}, {NULL}};
const Segment SGI158396[] = {{&S42721[30], 4}, {NULL}};
const Segment SGI158397[] = {{&S30680[44], 6}, {NULL}};
const Segment SGI158398[] = {{&S42721[113], 3}, {NULL}};
const Segment SGI158399[] = {{&S30680[44], 17}, {NULL}};
const Segment SGI158400[] = {{&S30680[30], 6}, {NULL}};
const Segment SGI158401[] = {{&S42721[78], 6}, {NULL}};
const Segment SGI158402[] = {{&S30680[110], 3}, {NULL}};
const Segment SGI158403[] = {{&S30680[75], 6}, {NULL}};
const Segment SGI158404[] = {{&S30681[44], 11}, {NULL}};
const Segment SGI158405[] = {{&S30681[44], 22}, {NULL}};
const Segment SGI158406[] = {{&S30681[30], 6}, {NULL}};
const Segment SGI158407[] = {{&S30681[115], 3}, {NULL}};
const Segment SGI158408[] = {{&S42723[42], 27}, {NULL}};
const Segment SGI158409[] = {{&S30681[80], 6}, {NULL}};
const Segment SGI158410[] = {{&S42723[42], 39}, {NULL}};
const Segment SGI158411[] = {{&S42723[30], 4}, {&S42723[50], 31}, {NULL}};
const Segment SGI158412[] = {{&S42723[130], 3}, {NULL}};
const Segment SGI158413[] = {{&S42723[95], 6}, {NULL}};
const Segment SGI158414[] = {{&S42730[11], 1}, {NULL}};
const Segment SGI158415[] = {{&S42730[42], 13}, {NULL}};
const Segment SGI158416[] = {{&S42730[30], 4}, {NULL}};
const Segment SGI158417[] = {{&S42730[104], 3}, {NULL}};
const Segment SGI158418[] = {{&S42730[69], 6}, {NULL}};
const Segment SGI158419[] = {{&S30692[44], 13}, {NULL}};
const Segment SGI158420[] = {{&S30692[44], 24}, {NULL}};
const Segment SGI158421[] = {{&S30692[30], 6}, {NULL}};
const Segment SGI158422[] = {{&S30692[117], 3}, {NULL}};
const Segment SGI158423[] = {{&S30692[82], 6}, {NULL}};
const Segment SGI158424[] = {{&S30694[44], 18}, {NULL}};
const Segment SGI158425[] = {{&S30694[44], 29}, {NULL}};
const Segment SGI158426[] = {{&S30694[30], 6}, {NULL}};
const Segment SGI158427[] = {{&S30694[122], 3}, {NULL}};
const Segment SGI158428[] = {{&S30694[87], 6}, {NULL}};
const Segment SGI158429[] = {{&S30697[53], 6}, {NULL}};
const Segment SGI158430[] = {{&S30697[115], 6}, {NULL}};
const Segment SGI158431[] = {{&S42741[42], 27}, {NULL}};
const Segment SGI158432[] = {{&S42741[42], 39}, {NULL}};
const Segment SGI158433[] = {{&S42741[30], 4}, {&S42741[50], 31}, {NULL}};
const Segment SGI158434[] = {{&S42741[130], 3}, {NULL}};
const Segment SGI158435[] = {{&S42741[95], 6}, {NULL}};
const Segment SGI158436[] = {{&S30699[44], 4}, {NULL}};
const Segment SGI158437[] = {{&S30699[44], 15}, {NULL}};
const Segment SGI158438[] = {{&S30699[30], 6}, {NULL}};
const Segment SGI158439[] = {{&S30699[108], 3}, {NULL}};
const Segment SGI158440[] = {{&S30699[73], 6}, {NULL}};
const Segment SGI158441[] = {{&S30702[44], 12}, {NULL}};
const Segment SGI158442[] = {{&S30702[44], 23}, {NULL}};
const Segment SGI158443[] = {{&S30702[30], 6}, {NULL}};
const Segment SGI158444[] = {{&S30702[116], 3}, {NULL}};
const Segment SGI158445[] = {{&S30702[81], 6}, {NULL}};
const Segment SGI158446[] = {{&S45723[34], 9}, {NULL}};
const Segment SGI158447[] = {{&S45723[27], 6}, {NULL}};
const Segment SGI158448[] = {{&S45723[96], 9}, {NULL}};
const Segment SGI158449[] = {{&S45723[69], 6}, {NULL}};
const Segment SGI158450[] = {{&S30705[44], 12}, {NULL}};
const Segment SGI158451[] = {{&S30705[44], 23}, {NULL}};
const Segment SGI158452[] = {{&S30705[30], 6}, {NULL}};
const Segment SGI158453[] = {{&S30705[116], 3}, {NULL}};
const Segment SGI158454[] = {{&S30705[81], 6}, {NULL}};
const Segment SGI158455[] = {{&S45725[34], 24}, {NULL}};
const Segment SGI158456[] = {{&S45725[27], 6}, {NULL}};
const Segment SGI158457[] = {{&S45725[111], 9}, {NULL}};
const Segment SGI158458[] = {{&S45725[84], 6}, {NULL}};
const Segment SGI158459[] = {{&S30707[44], 26}, {NULL}};
const Segment SGI158460[] = {{&S30707[44], 37}, {NULL}};
const Segment SGI158461[] = {{&S30707[30], 6}, {NULL}};
const Segment SGI158462[] = {{&S30707[130], 3}, {NULL}};
const Segment SGI158463[] = {{&S30707[95], 6}, {NULL}};
const Segment SGI158464[] = {{&S45726[34], 10}, {NULL}};
const Segment SGI158465[] = {{&S45726[27], 6}, {NULL}};
const Segment SGI158466[] = {{&S45726[97], 9}, {NULL}};
const Segment SGI158467[] = {{&S45726[70], 6}, {NULL}};
const Segment SGI158468[] = {{&S54668[34], 13}, {NULL}};
const Segment SGI158469[] = {{&S54668[27], 6}, {NULL}};
const Segment SGI158470[] = {{&S54668[104], 9}, {NULL}};
const Segment SGI158471[] = {{&S54668[77], 6}, {NULL}};
const Segment SGI158472[] = {{&S30718[44], 10}, {NULL}};
const Segment SGI158473[] = {{&S30718[44], 21}, {NULL}};
const Segment SGI158474[] = {{&S30718[30], 6}, {NULL}};
const Segment SGI158475[] = {{&S30718[114], 3}, {NULL}};
const Segment SGI158476[] = {{&S30718[79], 6}, {NULL}};
const Segment SGI158477[] = {{&S30730[44], 7}, {NULL}};
const Segment SGI158478[] = {{&S30730[44], 15}, {NULL}};
const Segment SGI158479[] = {{&S30730[30], 6}, {NULL}};
const Segment SGI158480[] = {{&S30730[108], 3}, {NULL}};
const Segment SGI158481[] = {{&S30730[73], 6}, {NULL}};
const Segment SGI158482[] = {{&S54685[34], 7}, {NULL}};
const Segment SGI158483[] = {{&S54685[27], 6}, {NULL}};
const Segment SGI158484[] = {{&S54685[94], 9}, {NULL}};
const Segment SGI158485[] = {{&S54685[67], 6}, {NULL}};
const Segment SGI158486[] = {{&S30734[44], 7}, {NULL}};
const Segment SGI158487[] = {{&S30734[44], 19}, {NULL}};
const Segment SGI158488[] = {{&S30734[30], 6}, {NULL}};
const Segment SGI158489[] = {{&S30734[112], 3}, {NULL}};
const Segment SGI158490[] = {{&S30734[77], 6}, {NULL}};
const Segment SGI158491[] = {{&S54686[34], 7}, {NULL}};
const Segment SGI158492[] = {{&S54686[27], 6}, {NULL}};
const Segment SGI158493[] = {{&S54686[93], 9}, {NULL}};
const Segment SGI158494[] = {{&S54686[66], 6}, {NULL}};
const Segment SGI158495[] = {{&S30735[44], 8}, {NULL}};
const Segment SGI158496[] = {{&S30735[44], 19}, {NULL}};
const Segment SGI158497[] = {{&S30735[30], 6}, {NULL}};
const Segment SGI158498[] = {{&S30735[112], 3}, {NULL}};
const Segment SGI158499[] = {{&S30735[77], 6}, {NULL}};
const Segment SGI158500[] = {{&S54687[34], 8}, {NULL}};
const Segment SGI158501[] = {{&S54687[27], 6}, {NULL}};
const Segment SGI158502[] = {{&S54687[95], 9}, {NULL}};
const Segment SGI158503[] = {{&S54687[68], 6}, {NULL}};
const Segment SGI158504[] = {{&S30740[44], 9}, {NULL}};
const Segment SGI158505[] = {{&S30740[44], 21}, {NULL}};
const Segment SGI158506[] = {{&S30740[30], 6}, {NULL}};
const Segment SGI158507[] = {{&S30740[114], 3}, {NULL}};
const Segment SGI158508[] = {{&S30740[79], 6}, {NULL}};
const Segment SGI158509[] = {{&S44477[44], 10}, {NULL}};
const Segment SGI158510[] = {{&S44477[44], 22}, {NULL}};
const Segment SGI158511[] = {{&S44477[30], 6}, {NULL}};
const Segment SGI158512[] = {{&S30741[44], 8}, {NULL}};
const Segment SGI158513[] = {{&S44477[115], 3}, {NULL}};
const Segment SGI158514[] = {{&S30741[44], 19}, {NULL}};
const Segment SGI158515[] = {{&S30741[30], 6}, {NULL}};
const Segment SGI158516[] = {{&S44477[80], 6}, {NULL}};
const Segment SGI158517[] = {{&S30741[112], 3}, {NULL}};
const Segment SGI158518[] = {{&S30741[77], 6}, {NULL}};
const Segment SGI158519[] = {{&S44479[38], 10}, {NULL}};
const Segment SGI158520[] = {{&S44479[31], 6}, {NULL}};
const Segment SGI158521[] = {{&S30746[44], 7}, {NULL}};
const Segment SGI158522[] = {{&S30746[44], 18}, {NULL}};
const Segment SGI158523[] = {{&S30746[30], 6}, {NULL}};
const Segment SGI158524[] = {{&S30746[111], 3}, {NULL}};
const Segment SGI158525[] = {{&S30746[76], 6}, {NULL}};
const Segment SGI158526[] = {{&S44483[44], 10}, {NULL}};
const Segment SGI158527[] = {{&S44483[44], 22}, {NULL}};
const Segment SGI158528[] = {{&S44483[30], 6}, {NULL}};
const Segment SGI158529[] = {{&S44483[83], 6}, {NULL}};
const Segment SGI158530[] = {{&S30756[44], 15}, {NULL}};
const Segment SGI158531[] = {{&S30756[44], 33}, {NULL}};
const Segment SGI158532[] = {{&S30756[30], 6}, {NULL}};
const Segment SGI158533[] = {{&S30756[126], 3}, {NULL}};
const Segment SGI158534[] = {{&S30756[91], 6}, {NULL}};
const Segment SGI158535[] = {{&S45785[34], 10}, {NULL}};
const Segment SGI158536[] = {{&S30759[44], 9}, {NULL}};
const Segment SGI158537[] = {{&S45785[27], 6}, {NULL}};
const Segment SGI158538[] = {{&S30759[44], 20}, {NULL}};
const Segment SGI158539[] = {{&S30759[30], 6}, {NULL}};
const Segment SGI158540[] = {{&S30759[113], 3}, {NULL}};
const Segment SGI158541[] = {{&S45785[97], 9}, {NULL}};
const Segment SGI158542[] = {{&S45785[70], 6}, {NULL}};
const Segment SGI158543[] = {{&S30759[78], 6}, {NULL}};
const Segment SGI158544[] = {{&S30761[44], 6}, {NULL}};
const Segment SGI158545[] = {{&S30761[44], 22}, {NULL}};
const Segment SGI158546[] = {{&S30761[30], 6}, {NULL}};
const Segment SGI158547[] = {{&S30761[115], 3}, {NULL}};
const Segment SGI158548[] = {{&S45788[34], 10}, {NULL}};
const Segment SGI158549[] = {{&S30761[80], 6}, {NULL}};
const Segment SGI158550[] = {{&S45788[27], 6}, {NULL}};
const Segment SGI158551[] = {{&S45788[97], 9}, {NULL}};
const Segment SGI158552[] = {{&S45788[70], 6}, {NULL}};
const Segment SGI158553[] = {{&S30763[44], 10}, {NULL}};
const Segment SGI158554[] = {{&S30763[44], 22}, {NULL}};
const Segment SGI158555[] = {{&S30763[30], 6}, {NULL}};
const Segment SGI158556[] = {{&S30763[115], 3}, {NULL}};
const Segment SGI158557[] = {{&S30763[80], 6}, {NULL}};
const Segment SGI158558[] = {{&S30764[44], 6}, {NULL}};
const Segment SGI158559[] = {{&S30764[44], 23}, {NULL}};
const Segment SGI158560[] = {{&S54722[34], 7}, {NULL}};
const Segment SGI158561[] = {{&S30764[30], 6}, {NULL}};
const Segment SGI158562[] = {{&S54722[27], 6}, {NULL}};
const Segment SGI158563[] = {{&S30764[116], 3}, {NULL}};
const Segment SGI158564[] = {{&S30764[81], 6}, {NULL}};
const Segment SGI158565[] = {{&S54722[99], 9}, {NULL}};
const Segment SGI158566[] = {{&S54722[72], 6}, {NULL}};
const Segment SGI158567[] = {{&S30765[44], 43}, {NULL}};
const Segment SGI158568[] = {{&S30765[30], 6}, {NULL}};
const Segment SGI158569[] = {{&S30765[136], 3}, {NULL}};
const Segment SGI158570[] = {{&S30765[101], 6}, {NULL}};
const Segment SGI158571[] = {{&S54725[34], 7}, {NULL}};
const Segment SGI158572[] = {{&S54725[27], 6}, {NULL}};
const Segment SGI158573[] = {{&S54725[98], 9}, {NULL}};
const Segment SGI158574[] = {{&S30767[44], 15}, {NULL}};
const Segment SGI158575[] = {{&S54725[71], 6}, {NULL}};
const Segment SGI158576[] = {{&S30767[44], 33}, {NULL}};
const Segment SGI158577[] = {{&S30767[30], 6}, {NULL}};
const Segment SGI158578[] = {{&S30767[126], 3}, {NULL}};
const Segment SGI158579[] = {{&S30767[91], 6}, {NULL}};
const Segment SGI158580[] = {{&S54726[34], 7}, {NULL}};
const Segment SGI158581[] = {{&S54726[27], 6}, {NULL}};
const Segment SGI158582[] = {{&S54726[98], 9}, {NULL}};
const Segment SGI158583[] = {{&S30768[44], 10}, {NULL}};
const Segment SGI158584[] = {{&S54726[71], 6}, {NULL}};
const Segment SGI158585[] = {{&S30768[44], 21}, {NULL}};
const Segment SGI158586[] = {{&S30768[30], 6}, {NULL}};
const Segment SGI158587[] = {{&S30768[114], 3}, {NULL}};
const Segment SGI158588[] = {{&S30768[79], 6}, {NULL}};
const Segment SGI158589[] = {{&S30771[44], 43}, {NULL}};
const Segment SGI158590[] = {{&S30771[30], 6}, {NULL}};
const Segment SGI158591[] = {{&S30771[136], 3}, {NULL}};
const Segment SGI158592[] = {{&S30771[101], 6}, {NULL}};
const Segment SGI158593[] = {{&S30774[44], 12}, {NULL}};
const Segment SGI158594[] = {{&S30774[44], 23}, {NULL}};
const Segment SGI158595[] = {{&S30774[30], 6}, {NULL}};
const Segment SGI158596[] = {{&S30774[116], 3}, {NULL}};
const Segment SGI158597[] = {{&S30774[81], 6}, {NULL}};
const Segment SGI158598[] = {{&S30775[78], 6}, {NULL}};
const Segment SGI158599[] = {{&S30777[44], 7}, {NULL}};
const Segment SGI158600[] = {{&S30777[44], 18}, {NULL}};
const Segment SGI158601[] = {{&S30777[30], 6}, {NULL}};
const Segment SGI158602[] = {{&S30777[111], 3}, {NULL}};
const Segment SGI158603[] = {{&S30777[76], 6}, {NULL}};
const Segment SGI158604[] = {{&S54743[34], 10}, {NULL}};
const Segment SGI158605[] = {{&S54743[27], 6}, {NULL}};
const Segment SGI158606[] = {{&S54743[97], 9}, {NULL}};
const Segment SGI158607[] = {{&S54743[70], 6}, {NULL}};
const Segment SGI158608[] = {{&S30780[44], 14}, {NULL}};
const Segment SGI158609[] = {{&S30780[44], 27}, {NULL}};
const Segment SGI158610[] = {{&S30780[30], 6}, {NULL}};
const Segment SGI158611[] = {{&S30780[120], 3}, {NULL}};
const Segment SGI158612[] = {{&S30780[85], 6}, {NULL}};
const Segment SGI158613[] = {{&S54744[34], 6}, {NULL}};
const Segment SGI158614[] = {{&S54744[27], 6}, {NULL}};
const Segment SGI158615[] = {{&S54744[93], 9}, {NULL}};
const Segment SGI158616[] = {{&S54744[66], 6}, {NULL}};
const Segment SGI158617[] = {{&S30781[44], 5}, {NULL}};
const Segment SGI158618[] = {{&S30781[44], 16}, {NULL}};
const Segment SGI158619[] = {{&S30781[30], 6}, {NULL}};
const Segment SGI158620[] = {{&S30781[109], 3}, {NULL}};
const Segment SGI158621[] = {{&S30781[74], 6}, {NULL}};
const Segment SGI158622[] = {{&S30784[44], 19}, {NULL}};
const Segment SGI158623[] = {{&S30784[44], 30}, {NULL}};
const Segment SGI158624[] = {{&S30784[30], 6}, {NULL}};
const Segment SGI158625[] = {{&S30784[123], 3}, {NULL}};
const Segment SGI158626[] = {{&S30784[88], 6}, {NULL}};
const Segment SGI158627[] = {{&S30792[44], 10}, {NULL}};
const Segment SGI158628[] = {{&S30792[44], 22}, {NULL}};
const Segment SGI158629[] = {{&S30792[30], 6}, {NULL}};
const Segment SGI158630[] = {{&S30792[31], 3}, {NULL}};
const Segment SGI158631[] = {{&S30792[80], 6}, {NULL}};
const Segment SGI158632[] = {{&S30804[44], 13}, {NULL}};
const Segment SGI158633[] = {{&S30804[44], 24}, {NULL}};
const Segment SGI158634[] = {{&S30804[30], 6}, {NULL}};
const Segment SGI158635[] = {{&S30804[117], 3}, {NULL}};
const Segment SGI158636[] = {{&S30804[82], 6}, {NULL}};
const Segment SGI158637[] = {{&S30809[38], 15}, {NULL}};
const Segment SGI158638[] = {{&S30809[31], 6}, {NULL}};
const Segment SGI158639[] = {{&S30811[44], 6}, {NULL}};
const Segment SGI158640[] = {{&S30811[44], 17}, {NULL}};
const Segment SGI158641[] = {{&S30811[30], 6}, {NULL}};
const Segment SGI158642[] = {{&S30811[110], 3}, {NULL}};
const Segment SGI158643[] = {{&S30811[75], 6}, {NULL}};
const Segment SGI158644[] = {{&S54786[34], 7}, {NULL}};
const Segment SGI158645[] = {{&S30812[44], 12}, {NULL}};
const Segment SGI158646[] = {{&S54786[27], 6}, {NULL}};
const Segment SGI158647[] = {{&S30812[44], 23}, {NULL}};
const Segment SGI158648[] = {{&S30812[30], 6}, {NULL}};
const Segment SGI158649[] = {{&S30812[116], 3}, {NULL}};
const Segment SGI158650[] = {{&S54786[98], 9}, {NULL}};
const Segment SGI158651[] = {{&S54786[71], 6}, {NULL}};
const Segment SGI158652[] = {{&S30812[81], 6}, {NULL}};
const Segment SGI158653[] = {{&S30813[44], 3}, {NULL}};
const Segment SGI158654[] = {{&S30813[44], 14}, {NULL}};
const Segment SGI158655[] = {{&S30813[30], 6}, {NULL}};
const Segment SGI158656[] = {{&S30813[107], 3}, {NULL}};
const Segment SGI158657[] = {{&S30813[72], 6}, {NULL}};
const Segment SGI158658[] = {{&S30814[44], 8}, {NULL}};
const Segment SGI158659[] = {{&S30814[44], 19}, {NULL}};
const Segment SGI158660[] = {{&S30814[30], 6}, {NULL}};
const Segment SGI158661[] = {{&S30814[112], 3}, {NULL}};
const Segment SGI158662[] = {{&S30814[77], 6}, {NULL}};
const Segment SGI158663[] = {{&S30816[44], 7}, {NULL}};
const Segment SGI158664[] = {{&S30816[44], 18}, {NULL}};
const Segment SGI158665[] = {{&S30816[30], 6}, {NULL}};
const Segment SGI158666[] = {{&S30816[111], 3}, {NULL}};
const Segment SGI158667[] = {{&S30816[76], 6}, {NULL}};
const Segment SGI158668[] = {{&S30821[44], 11}, {NULL}};
const Segment SGI158669[] = {{&S30821[44], 22}, {NULL}};
const Segment SGI158670[] = {{&S30821[30], 6}, {NULL}};
const Segment SGI158671[] = {{&S30821[115], 3}, {NULL}};
const Segment SGI158672[] = {{&S30821[80], 6}, {NULL}};
const Segment SGI158673[] = {{&S30824[44], 11}, {NULL}};
const Segment SGI158674[] = {{&S30824[44], 22}, {NULL}};
const Segment SGI158675[] = {{&S30824[30], 6}, {NULL}};
const Segment SGI158676[] = {{&S30824[115], 3}, {NULL}};
const Segment SGI158677[] = {{&S30824[80], 6}, {NULL}};
const Segment SGI158678[] = {{&S30825[44], 10}, {NULL}};
const Segment SGI158679[] = {{&S30825[44], 21}, {NULL}};
const Segment SGI158680[] = {{&S30825[30], 6}, {NULL}};
const Segment SGI158681[] = {{&S30825[114], 3}, {NULL}};
const Segment SGI158682[] = {{&S30825[79], 6}, {NULL}};
const Segment SGI158683[] = {{&S30828[44], 22}, {NULL}};
const Segment SGI158684[] = {{&S30828[44], 35}, {NULL}};
const Segment SGI158685[] = {{&S30828[30], 6}, {NULL}};
const Segment SGI158686[] = {{&S30828[128], 3}, {NULL}};
const Segment SGI158687[] = {{&S30828[93], 6}, {NULL}};
const Segment SGI158688[] = {{&S30830[44], 10}, {NULL}};
const Segment SGI158689[] = {{&S30830[44], 34}, {NULL}};
const Segment SGI158690[] = {{&S30830[30], 6}, {NULL}};
const Segment SGI158691[] = {{&S30830[127], 3}, {NULL}};
const Segment SGI158692[] = {{&S30830[92], 6}, {NULL}};
const Segment SGI158693[] = {{&S54813[44], 6}, {NULL}};
const Segment SGI158694[] = {{&S54813[44], 18}, {NULL}};
const Segment SGI158695[] = {{&S54813[30], 6}, {NULL}};
const Segment SGI158696[] = {{&S54813[31], 3}, {NULL}};
const Segment SGI158697[] = {{&S54813[76], 6}, {NULL}};
const Segment SGI158698[] = {{&S30832[44], 11}, {NULL}};
const Segment SGI158699[] = {{&S30832[44], 22}, {NULL}};
const Segment SGI158700[] = {{&S30832[30], 6}, {NULL}};
const Segment SGI158701[] = {{&S30832[115], 3}, {NULL}};
const Segment SGI158702[] = {{&S30832[80], 6}, {NULL}};
const Segment SGI158703[] = {{&S44597[44], 10}, {NULL}};
const Segment SGI158704[] = {{&S44597[44], 22}, {NULL}};
const Segment SGI158705[] = {{&S44597[30], 6}, {NULL}};
const Segment SGI158706[] = {{&S44597[115], 3}, {NULL}};
const Segment SGI158707[] = {{&S44597[80], 6}, {NULL}};
const Segment SGI158708[] = {{&S30834[44], 8}, {NULL}};
const Segment SGI158709[] = {{&S30834[44], 19}, {NULL}};
const Segment SGI158710[] = {{&S30834[30], 6}, {NULL}};
const Segment SGI158711[] = {{&S44598[38], 10}, {NULL}};
const Segment SGI158712[] = {{&S44598[31], 6}, {NULL}};
const Segment SGI158713[] = {{&S30834[112], 3}, {NULL}};
const Segment SGI158714[] = {{&S30834[77], 6}, {NULL}};
const Segment SGI158715[] = {{&S30836[44], 10}, {NULL}};
const Segment SGI158716[] = {{&S30836[44], 21}, {NULL}};
const Segment SGI158717[] = {{&S30836[30], 6}, {NULL}};
const Segment SGI158718[] = {{&S30836[114], 3}, {NULL}};
const Segment SGI158719[] = {{&S30836[79], 6}, {NULL}};
const Segment SGI158720[] = {{&S30837[44], 6}, {NULL}};
const Segment SGI158721[] = {{&S30837[44], 16}, {NULL}};
const Segment SGI158722[] = {{&S30837[30], 6}, {NULL}};
const Segment SGI158723[] = {{&S30837[109], 3}, {NULL}};
const Segment SGI158724[] = {{&S30837[74], 6}, {NULL}};
const Segment SGI158725[] = {{&S30838[44], 7}, {NULL}};
const Segment SGI158726[] = {{&S30838[44], 18}, {NULL}};
const Segment SGI158727[] = {{&S30838[30], 6}, {NULL}};
const Segment SGI158728[] = {{&S30838[111], 3}, {NULL}};
const Segment SGI158729[] = {{&S30838[76], 6}, {NULL}};
const Segment SGI158730[] = {{&S45903[44], 10}, {NULL}};
const Segment SGI158731[] = {{&S45903[44], 22}, {NULL}};
const Segment SGI158732[] = {{&S45903[30], 6}, {NULL}};
const Segment SGI158733[] = {{&S45903[131], 3}, {NULL}};
const Segment SGI158734[] = {{&S45903[96], 6}, {NULL}};
const Segment SGI158735[] = {{&S45903[68], 8}, {NULL}};
const Segment SGI158736[] = {{&S30845[44], 11}, {NULL}};
const Segment SGI158737[] = {{&S30845[44], 22}, {NULL}};
const Segment SGI158738[] = {{&S30845[30], 6}, {NULL}};
const Segment SGI158739[] = {{&S30845[115], 3}, {NULL}};
const Segment SGI158740[] = {{&S30845[80], 6}, {NULL}};
const Segment SGI158741[] = {{&S45904[44], 18}, {NULL}};
const Segment SGI158742[] = {{&S45904[44], 30}, {NULL}};
const Segment SGI158743[] = {{&S45904[30], 6}, {NULL}};
const Segment SGI158744[] = {{&S45904[139], 3}, {NULL}};
const Segment SGI158745[] = {{&S45904[104], 6}, {NULL}};
const Segment SGI158746[] = {{&S45904[76], 8}, {NULL}};
const Segment SGI158747[] = {{&S30847[44], 12}, {NULL}};
const Segment SGI158748[] = {{&S30847[44], 23}, {NULL}};
const Segment SGI158749[] = {{&S30847[30], 6}, {NULL}};
const Segment SGI158750[] = {{&S45905[44], 18}, {NULL}};
const Segment SGI158751[] = {{&S45905[44], 30}, {NULL}};
const Segment SGI158752[] = {{&S45905[30], 6}, {NULL}};
const Segment SGI158753[] = {{&S30847[116], 3}, {NULL}};
const Segment SGI158754[] = {{&S30847[81], 6}, {NULL}};
const Segment SGI158755[] = {{&S45905[139], 3}, {NULL}};
const Segment SGI158756[] = {{&S45905[104], 6}, {NULL}};
const Segment SGI158757[] = {{&S45905[76], 8}, {NULL}};
const Segment SGI158758[] = {{&S45906[44], 10}, {NULL}};
const Segment SGI158759[] = {{&S45906[44], 22}, {NULL}};
const Segment SGI158760[] = {{&S45906[30], 6}, {NULL}};
const Segment SGI158761[] = {{&S30849[44], 12}, {NULL}};
const Segment SGI158762[] = {{&S30849[44], 23}, {NULL}};
const Segment SGI158763[] = {{&S30849[30], 6}, {NULL}};
const Segment SGI158764[] = {{&S45906[131], 3}, {NULL}};
const Segment SGI158765[] = {{&S45906[96], 6}, {NULL}};
const Segment SGI158766[] = {{&S30849[116], 3}, {NULL}};
const Segment SGI158767[] = {{&S45906[68], 8}, {NULL}};
const Segment SGI158768[] = {{&S30849[81], 6}, {NULL}};
const Segment SGI158769[] = {{&S45907[44], 11}, {NULL}};
const Segment SGI158770[] = {{&S30850[44], 9}, {NULL}};
const Segment SGI158771[] = {{&S45907[44], 23}, {NULL}};
const Segment SGI158772[] = {{&S30850[44], 20}, {NULL}};
const Segment SGI158773[] = {{&S45907[30], 6}, {NULL}};
const Segment SGI158774[] = {{&S30850[30], 6}, {NULL}};
const Segment SGI158775[] = {{&S30850[113], 3}, {NULL}};
const Segment SGI158776[] = {{&S45907[132], 3}, {NULL}};
const Segment SGI158777[] = {{&S30850[78], 6}, {NULL}};
const Segment SGI158778[] = {{&S45907[97], 6}, {NULL}};
const Segment SGI158779[] = {{&S45907[69], 8}, {NULL}};
const Segment SGI158780[] = {{&S30851[44], 6}, {NULL}};
const Segment SGI158781[] = {{&S30851[44], 17}, {NULL}};
const Segment SGI158782[] = {{&S30851[30], 6}, {NULL}};
const Segment SGI158783[] = {{&S45908[44], 10}, {NULL}};
const Segment SGI158784[] = {{&S45908[44], 22}, {NULL}};
const Segment SGI158785[] = {{&S30851[110], 3}, {NULL}};
const Segment SGI158786[] = {{&S45908[30], 6}, {NULL}};
const Segment SGI158787[] = {{&S30851[75], 6}, {NULL}};
const Segment SGI158788[] = {{&S45908[131], 3}, {NULL}};
const Segment SGI158789[] = {{&S45908[96], 6}, {NULL}};
const Segment SGI158790[] = {{&S45908[68], 8}, {NULL}};
const Segment SGI158791[] = {{&S45909[44], 18}, {NULL}};
const Segment SGI158792[] = {{&S30853[44], 10}, {NULL}};
const Segment SGI158793[] = {{&S45909[44], 30}, {NULL}};
const Segment SGI158794[] = {{&S30853[44], 21}, {NULL}};
const Segment SGI158795[] = {{&S45909[30], 6}, {NULL}};
const Segment SGI158796[] = {{&S30853[30], 6}, {NULL}};
const Segment SGI158797[] = {{&S30853[31], 3}, {NULL}};
const Segment SGI158798[] = {{&S45909[139], 3}, {NULL}};
const Segment SGI158799[] = {{&S30853[79], 6}, {NULL}};
const Segment SGI158800[] = {{&S45909[104], 6}, {NULL}};
const Segment SGI158801[] = {{&S45909[76], 8}, {NULL}};
const Segment SGI158802[] = {{&S54856[44], 10}, {NULL}};
const Segment SGI158803[] = {{&S54856[44], 22}, {NULL}};
const Segment SGI158804[] = {{&S30855[44], 15}, {NULL}};
const Segment SGI158805[] = {{&S54856[30], 6}, {NULL}};
const Segment SGI158806[] = {{&S30855[44], 33}, {NULL}};
const Segment SGI158807[] = {{&S30855[30], 6}, {NULL}};
const Segment SGI158808[] = {{&S54856[31], 3}, {NULL}};
const Segment SGI158809[] = {{&S54856[80], 6}, {NULL}};
const Segment SGI158810[] = {{&S30855[94], 6}, {NULL}};
const Segment SGI158811[] = {{&S54857[44], 14}, {NULL}};
const Segment SGI158812[] = {{&S54857[44], 26}, {NULL}};
const Segment SGI158813[] = {{&S54857[30], 6}, {NULL}};
const Segment SGI158814[] = {{&S54857[31], 3}, {NULL}};
const Segment SGI158815[] = {{&S45913[34], 10}, {NULL}};
const Segment SGI158816[] = {{&S54857[84], 6}, {NULL}};
const Segment SGI158817[] = {{&S45913[27], 6}, {NULL}};
const Segment SGI158818[] = {{&S30857[44], 9}, {NULL}};
const Segment SGI158819[] = {{&S30857[44], 20}, {NULL}};
const Segment SGI158820[] = {{&S45913[97], 9}, {NULL}};
const Segment SGI158821[] = {{&S30857[30], 6}, {NULL}};
const Segment SGI158822[] = {{&S45913[70], 6}, {NULL}};
const Segment SGI158823[] = {{&S30857[113], 3}, {NULL}};
const Segment SGI158824[] = {{&S30857[78], 6}, {NULL}};
const Segment SGI158825[] = {{&S54858[38], 10}, {NULL}};
const Segment SGI158826[] = {{&S54858[31], 6}, {NULL}};
const Segment SGI158827[] = {{&S30858[44], 9}, {NULL}};
const Segment SGI158828[] = {{&S30858[44], 20}, {NULL}};
const Segment SGI158829[] = {{&S30858[30], 6}, {NULL}};
const Segment SGI158830[] = {{&S30858[113], 3}, {NULL}};
const Segment SGI158831[] = {{&S30858[78], 6}, {NULL}};
const Segment SGI158832[] = {{&S30860[44], 8}, {NULL}};
const Segment SGI158833[] = {{&S30860[44], 19}, {NULL}};
const Segment SGI158834[] = {{&S30860[30], 6}, {NULL}};
const Segment SGI158835[] = {{&S30860[112], 3}, {NULL}};
const Segment SGI158836[] = {{&S30860[77], 6}, {NULL}};
const Segment SGI158837[] = {{&S30861[44], 9}, {NULL}};
const Segment SGI158838[] = {{&S30861[44], 20}, {NULL}};
const Segment SGI158839[] = {{&S30861[30], 6}, {NULL}};
const Segment SGI158840[] = {{&S30861[113], 3}, {NULL}};
const Segment SGI158841[] = {{&S30861[78], 6}, {NULL}};
const Segment SGI158842[] = {{&S30862[44], 7}, {NULL}};
const Segment SGI158843[] = {{&S30862[44], 18}, {NULL}};
const Segment SGI158844[] = {{&S30862[30], 6}, {NULL}};
const Segment SGI158845[] = {{&S30862[111], 3}, {NULL}};
const Segment SGI158846[] = {{&S30862[76], 6}, {NULL}};
const Segment SGI158847[] = {{&S54867[44], 11}, {NULL}};
const Segment SGI158848[] = {{&S54867[44], 23}, {NULL}};
const Segment SGI158849[] = {{&S54867[30], 6}, {NULL}};
const Segment SGI158850[] = {{&S54867[31], 3}, {NULL}};
const Segment SGI158851[] = {{&S44649[44], 10}, {NULL}};
const Segment SGI158852[] = {{&S54867[81], 6}, {NULL}};
const Segment SGI158853[] = {{&S44649[44], 22}, {NULL}};
const Segment SGI158854[] = {{&S44649[30], 6}, {NULL}};
const Segment SGI158855[] = {{&S44649[115], 3}, {NULL}};
const Segment SGI158856[] = {{&S44649[80], 6}, {NULL}};
const Segment SGI158857[] = {{&S30864[44], 14}, {NULL}};
const Segment SGI158858[] = {{&S30864[44], 25}, {NULL}};
const Segment SGI158859[] = {{&S30864[30], 6}, {NULL}};
const Segment SGI158860[] = {{&S54868[44], 11}, {NULL}};
const Segment SGI158861[] = {{&S54868[44], 23}, {NULL}};
const Segment SGI158862[] = {{&S30864[118], 3}, {NULL}};
const Segment SGI158863[] = {{&S54868[30], 6}, {NULL}};
const Segment SGI158864[] = {{&S30864[83], 6}, {NULL}};
const Segment SGI158865[] = {{&S54868[31], 3}, {NULL}};
const Segment SGI158866[] = {{&S44650[44], 10}, {NULL}};
const Segment SGI158867[] = {{&S54868[81], 6}, {NULL}};
const Segment SGI158868[] = {{&S44650[44], 22}, {NULL}};
const Segment SGI158869[] = {{&S44650[30], 6}, {NULL}};
const Segment SGI158870[] = {{&S44650[115], 3}, {NULL}};
const Segment SGI158871[] = {{&S44650[80], 6}, {NULL}};
const Segment SGI158872[] = {{&S54869[44], 11}, {NULL}};
const Segment SGI158873[] = {{&S54869[44], 23}, {NULL}};
const Segment SGI158874[] = {{&S54869[30], 6}, {NULL}};
const Segment SGI158875[] = {{&S54869[31], 3}, {NULL}};
const Segment SGI158876[] = {{&S30866[44], 10}, {NULL}};
const Segment SGI158877[] = {{&S54869[81], 6}, {NULL}};
const Segment SGI158878[] = {{&S44651[44], 10}, {NULL}};
const Segment SGI158879[] = {{&S30866[44], 21}, {NULL}};
const Segment SGI158880[] = {{&S44651[44], 22}, {NULL}};
const Segment SGI158881[] = {{&S30866[30], 6}, {NULL}};
const Segment SGI158882[] = {{&S44651[30], 6}, {NULL}};
const Segment SGI158883[] = {{&S30866[114], 3}, {NULL}};
const Segment SGI158884[] = {{&S44651[115], 3}, {NULL}};
const Segment SGI158885[] = {{&S30866[79], 6}, {NULL}};
const Segment SGI158886[] = {{&S44651[80], 6}, {NULL}};
const Segment SGI158887[] = {{&S54870[38], 11}, {NULL}};
const Segment SGI158888[] = {{&S54870[31], 6}, {NULL}};
const Segment SGI158889[] = {{&S44652[38], 10}, {NULL}};
const Segment SGI158890[] = {{&S44652[31], 6}, {NULL}};
const Segment SGI158891[] = {{&S30867[44], 23}, {NULL}};
const Segment SGI158892[] = {{&S30867[44], 34}, {NULL}};
const Segment SGI158893[] = {{&S30867[30], 6}, {NULL}};
const Segment SGI158894[] = {{&S30867[127], 3}, {NULL}};
const Segment SGI158895[] = {{&S30867[92], 6}, {NULL}};
const Segment SGI158896[] = {{&S44653[38], 10}, {NULL}};
const Segment SGI158897[] = {{&S44653[31], 6}, {NULL}};
const Segment SGI158898[] = {{&S30868[44], 9}, {NULL}};
const Segment SGI158899[] = {{&S30868[44], 20}, {NULL}};
const Segment SGI158900[] = {{&S30868[30], 6}, {NULL}};
const Segment SGI158901[] = {{&S30868[113], 3}, {NULL}};
const Segment SGI158902[] = {{&S30868[78], 6}, {NULL}};
const Segment SGI158903[] = {{&S44654[44], 19}, {NULL}};
const Segment SGI158904[] = {{&S44654[44], 31}, {NULL}};
const Segment SGI158905[] = {{&S44654[30], 6}, {NULL}};
const Segment SGI158906[] = {{&S44654[124], 3}, {NULL}};
const Segment SGI158907[] = {{&S44654[89], 6}, {NULL}};
const Segment SGI158908[] = {{&S30869[44], 14}, {NULL}};
const Segment SGI158909[] = {{&S30869[44], 25}, {NULL}};
const Segment SGI158910[] = {{&S30869[30], 6}, {NULL}};
const Segment SGI158911[] = {{&S30869[118], 3}, {NULL}};
const Segment SGI158912[] = {{&S30869[83], 6}, {NULL}};
const Segment SGI158913[] = {{&S44655[38], 10}, {NULL}};
const Segment SGI158914[] = {{&S44655[31], 6}, {NULL}};
const Segment SGI158915[] = {{&S44656[44], 9}, {NULL}};
const Segment SGI158916[] = {{&S44656[44], 21}, {NULL}};
const Segment SGI158917[] = {{&S44656[30], 6}, {NULL}};
const Segment SGI158918[] = {{&S44656[114], 3}, {NULL}};
const Segment SGI158919[] = {{&S30871[44], 43}, {NULL}};
const Segment SGI158920[] = {{&S44656[79], 6}, {NULL}};
const Segment SGI158921[] = {{&S30871[30], 6}, {NULL}};
const Segment SGI158922[] = {{&S30871[136], 3}, {NULL}};
const Segment SGI158923[] = {{&S30871[101], 6}, {NULL}};
const Segment SGI158924[] = {{&S44657[44], 19}, {NULL}};
const Segment SGI158925[] = {{&S44657[44], 31}, {NULL}};
const Segment SGI158926[] = {{&S44657[30], 6}, {NULL}};
const Segment SGI158927[] = {{&S44657[124], 3}, {NULL}};
const Segment SGI158928[] = {{&S44657[89], 6}, {NULL}};
const Segment SGI158929[] = {{&S30873[44], 9}, {NULL}};
const Segment SGI158930[] = {{&S30873[44], 20}, {NULL}};
const Segment SGI158931[] = {{&S30873[30], 6}, {NULL}};
const Segment SGI158932[] = {{&S30873[113], 3}, {NULL}};
const Segment SGI158933[] = {{&S30873[78], 6}, {NULL}};
const Segment SGI158934[] = {{&S30877[44], 14}, {NULL}};
const Segment SGI158935[] = {{&S30877[44], 25}, {NULL}};
const Segment SGI158936[] = {{&S30877[30], 6}, {NULL}};
const Segment SGI158937[] = {{&S30877[118], 3}, {NULL}};
const Segment SGI158938[] = {{&S30877[83], 6}, {NULL}};
const Segment SGI158939[] = {{&S30878[44], 23}, {NULL}};
const Segment SGI158940[] = {{&S30878[44], 44}, {NULL}};
const Segment SGI158941[] = {{&S30878[30], 6}, {NULL}};
const Segment SGI158942[] = {{&S30878[137], 3}, {NULL}};
const Segment SGI158943[] = {{&S30878[102], 6}, {NULL}};
const Segment SGI158944[] = {{&S30882[44], 9}, {NULL}};
const Segment SGI158945[] = {{&S30882[44], 20}, {NULL}};
const Segment SGI158946[] = {{&S30882[30], 6}, {NULL}};
const Segment SGI158947[] = {{&S30882[113], 3}, {NULL}};
const Segment SGI158948[] = {{&S30882[78], 6}, {NULL}};
const Segment SGI158949[] = {{&S30884[44], 15}, {NULL}};
const Segment SGI158950[] = {{&S30884[44], 26}, {NULL}};
const Segment SGI158951[] = {{&S30884[30], 6}, {NULL}};
const Segment SGI158952[] = {{&S30884[119], 3}, {NULL}};
const Segment SGI158953[] = {{&S30884[84], 6}, {NULL}};
const Segment SGI158954[] = {{&S45956[34], 10}, {NULL}};
const Segment SGI158955[] = {{&S45956[27], 6}, {NULL}};
const Segment SGI158956[] = {{&S45956[97], 9}, {NULL}};
const Segment SGI158957[] = {{&S45956[70], 6}, {NULL}};
const Segment SGI158958[] = {{&S30886[44], 6}, {NULL}};
const Segment SGI158959[] = {{&S30886[44], 17}, {NULL}};
const Segment SGI158960[] = {{&S30886[30], 6}, {NULL}};
const Segment SGI158961[] = {{&S30886[110], 3}, {NULL}};
const Segment SGI158962[] = {{&S30886[75], 6}, {NULL}};
const Segment SGI158963[] = {{&S30888[44], 8}, {NULL}};
const Segment SGI158964[] = {{&S30888[44], 19}, {NULL}};
const Segment SGI158965[] = {{&S30888[30], 6}, {NULL}};
const Segment SGI158966[] = {{&S30888[112], 3}, {NULL}};
const Segment SGI158967[] = {{&S30888[77], 6}, {NULL}};
const Segment SGI158968[] = {{&S30889[44], 5}, {NULL}};
const Segment SGI158969[] = {{&S30889[44], 16}, {NULL}};
const Segment SGI158970[] = {{&S30889[30], 6}, {NULL}};
const Segment SGI158971[] = {{&S30889[109], 3}, {NULL}};
const Segment SGI158972[] = {{&S30889[74], 6}, {NULL}};
const Segment SGI158973[] = {{&S30891[44], 10}, {NULL}};
const Segment SGI158974[] = {{&S30891[44], 21}, {NULL}};
const Segment SGI158975[] = {{&S30891[30], 6}, {NULL}};
const Segment SGI158976[] = {{&S30891[114], 3}, {NULL}};
const Segment SGI158977[] = {{&S30891[79], 6}, {NULL}};
const Segment SGI158978[] = {{&S30895[44], 10}, {NULL}};
const Segment SGI158979[] = {{&S30895[44], 21}, {NULL}};
const Segment SGI158980[] = {{&S30895[30], 6}, {NULL}};
const Segment SGI158981[] = {{&S54918[34], 13}, {NULL}};
const Segment SGI158982[] = {{&S30895[114], 3}, {NULL}};
const Segment SGI158983[] = {{&S54918[27], 6}, {NULL}};
const Segment SGI158984[] = {{&S30895[79], 6}, {NULL}};
const Segment SGI158985[] = {{&S54918[100], 9}, {NULL}};
const Segment SGI158986[] = {{&S54918[73], 6}, {NULL}};
const Segment SGI158987[] = {{&S30896[44], 4}, {NULL}};
const Segment SGI158988[] = {{&S30896[44], 15}, {NULL}};
const Segment SGI158989[] = {{&S30896[30], 6}, {NULL}};
const Segment SGI158990[] = {{&S30896[108], 3}, {NULL}};
const Segment SGI158991[] = {{&S54919[34], 10}, {NULL}};
const Segment SGI158992[] = {{&S30896[73], 6}, {NULL}};
const Segment SGI158993[] = {{&S54919[27], 6}, {NULL}};
const Segment SGI158994[] = {{&S54919[97], 9}, {NULL}};
const Segment SGI158995[] = {{&S54919[70], 6}, {NULL}};
const Segment SGI158996[] = {{&S30898[44], 20}, {NULL}};
const Segment SGI158997[] = {{&S30898[44], 31}, {NULL}};
const Segment SGI158998[] = {{&S30898[30], 6}, {NULL}};
const Segment SGI158999[] = {{&S30898[124], 3}, {NULL}};
const Segment SGI159000[] = {{&S30898[89], 6}, {NULL}};
const Segment SGI159001[] = {{&S30900[44], 17}, {NULL}};
const Segment SGI159002[] = {{&S30900[44], 28}, {NULL}};
const Segment SGI159003[] = {{&S30900[30], 6}, {NULL}};
const Segment SGI159004[] = {{&S54925[34], 7}, {NULL}};
const Segment SGI159005[] = {{&S54925[27], 6}, {NULL}};
const Segment SGI159006[] = {{&S30900[121], 3}, {NULL}};
const Segment SGI159007[] = {{&S30900[86], 6}, {NULL}};
const Segment SGI159008[] = {{&S54925[100], 9}, {NULL}};
const Segment SGI159009[] = {{&S54925[73], 6}, {NULL}};
const Segment SGI159010[] = {{&S30901[44], 12}, {NULL}};
const Segment SGI159011[] = {{&S30901[44], 28}, {NULL}};
const Segment SGI159012[] = {{&S30901[30], 6}, {NULL}};
const Segment SGI159013[] = {{&S54926[34], 7}, {NULL}};
const Segment SGI159014[] = {{&S54926[27], 6}, {NULL}};
const Segment SGI159015[] = {{&S30901[121], 3}, {NULL}};
const Segment SGI159016[] = {{&S30901[86], 6}, {NULL}};
const Segment SGI159017[] = {{&S54926[100], 9}, {NULL}};
const Segment SGI159018[] = {{&S54926[73], 6}, {NULL}};
const Segment SGI159019[] = {{&S30905[44], 9}, {NULL}};
const Segment SGI159020[] = {{&S30905[44], 20}, {NULL}};
const Segment SGI159021[] = {{&S30905[30], 6}, {NULL}};
const Segment SGI159022[] = {{&S30905[113], 3}, {NULL}};
const Segment SGI159023[] = {{&S30905[78], 6}, {NULL}};
const Segment SGI159024[] = {{&S30907[44], 11}, {NULL}};
const Segment SGI159025[] = {{&S30907[44], 22}, {NULL}};
const Segment SGI159026[] = {{&S30907[30], 6}, {NULL}};
const Segment SGI159027[] = {{&S30907[115], 3}, {NULL}};
const Segment SGI159028[] = {{&S30907[80], 6}, {NULL}};
const Segment SGI159029[] = {{&S30908[44], 5}, {NULL}};
const Segment SGI159030[] = {{&S30908[44], 16}, {NULL}};
const Segment SGI159031[] = {{&S30908[30], 6}, {NULL}};
const Segment SGI159032[] = {{&S30908[109], 3}, {NULL}};
const Segment SGI159033[] = {{&S30908[74], 6}, {NULL}};
const Segment SGI159034[] = {{&S30909[44], 6}, {NULL}};
const Segment SGI159035[] = {{&S30909[44], 26}, {NULL}};
const Segment SGI159036[] = {{&S30909[30], 6}, {NULL}};
const Segment SGI159037[] = {{&S30909[119], 3}, {NULL}};
const Segment SGI159038[] = {{&S30909[84], 6}, {NULL}};
const Segment SGI159039[] = {{&S30910[44], 35}, {NULL}};
const Segment SGI159040[] = {{&S30910[44], 46}, {NULL}};
const Segment SGI159041[] = {{&S30910[30], 6}, {NULL}};
const Segment SGI159042[] = {{&S30910[139], 3}, {NULL}};
const Segment SGI159043[] = {{&S30910[104], 6}, {NULL}};
const Segment SGI159044[] = {{&S30911[44], 14}, {NULL}};
const Segment SGI159045[] = {{&S30911[44], 25}, {NULL}};
const Segment SGI159046[] = {{&S30911[30], 6}, {NULL}};
const Segment SGI159047[] = {{&S30911[118], 3}, {NULL}};
const Segment SGI159048[] = {{&S30911[83], 6}, {NULL}};
const Segment SGI159049[] = {{&S30915[44], 29}, {NULL}};
const Segment SGI159050[] = {{&S30915[44], 40}, {NULL}};
const Segment SGI159051[] = {{&S30915[30], 6}, {NULL}};
const Segment SGI159052[] = {{&S30915[133], 3}, {NULL}};
const Segment SGI159053[] = {{&S30915[98], 6}, {NULL}};
const Segment SGI159054[] = {{&S54948[44], 10}, {NULL}};
const Segment SGI159055[] = {{&S54948[44], 22}, {NULL}};
const Segment SGI159056[] = {{&S54948[30], 6}, {NULL}};
const Segment SGI159057[] = {{&S54948[31], 3}, {NULL}};
const Segment SGI159058[] = {{&S54948[80], 6}, {NULL}};
const Segment SGI159059[] = {{&S54949[44], 10}, {NULL}};
const Segment SGI159060[] = {{&S54949[44], 22}, {NULL}};
const Segment SGI159061[] = {{&S54949[30], 6}, {NULL}};
const Segment SGI159062[] = {{&S54949[31], 3}, {NULL}};
const Segment SGI159063[] = {{&S54949[80], 6}, {NULL}};
const Segment SGI159064[] = {{&S54950[44], 10}, {NULL}};
const Segment SGI159065[] = {{&S54950[44], 22}, {NULL}};
const Segment SGI159066[] = {{&S54950[30], 6}, {NULL}};
const Segment SGI159067[] = {{&S54950[31], 3}, {NULL}};
const Segment SGI159068[] = {{&S54950[80], 6}, {NULL}};
const Segment SGI159069[] = {{&S54952[44], 10}, {NULL}};
const Segment SGI159070[] = {{&S54952[44], 22}, {NULL}};
const Segment SGI159071[] = {{&S54952[30], 6}, {NULL}};
const Segment SGI159072[] = {{&S54952[31], 3}, {NULL}};
const Segment SGI159073[] = {{&S54952[80], 6}, {NULL}};
const Segment SGI159074[] = {{&S30923[44], 8}, {NULL}};
const Segment SGI159075[] = {{&S30923[44], 19}, {NULL}};
const Segment SGI159076[] = {{&S30923[30], 6}, {NULL}};
const Segment SGI159077[] = {{&S30923[112], 3}, {NULL}};
const Segment SGI159078[] = {{&S30923[77], 6}, {NULL}};
const Segment SGI159079[] = {{&S54953[44], 10}, {NULL}};
const Segment SGI159080[] = {{&S54953[44], 22}, {NULL}};
const Segment SGI159081[] = {{&S54953[30], 6}, {NULL}};
const Segment SGI159082[] = {{&S54953[31], 3}, {NULL}};
const Segment SGI159083[] = {{&S54953[80], 6}, {NULL}};
const Segment SGI159084[] = {{&S30925[44], 10}, {NULL}};
const Segment SGI159085[] = {{&S54954[38], 10}, {NULL}};
const Segment SGI159086[] = {{&S30925[44], 21}, {NULL}};
const Segment SGI159087[] = {{&S54954[31], 6}, {NULL}};
const Segment SGI159088[] = {{&S30925[30], 6}, {NULL}};
const Segment SGI159089[] = {{&S30925[114], 3}, {NULL}};
const Segment SGI159090[] = {{&S30925[79], 6}, {NULL}};
const Segment SGI159091[] = {{&S30926[44], 14}, {NULL}};
const Segment SGI159092[] = {{&S30926[44], 25}, {NULL}};
const Segment SGI159093[] = {{&S30926[30], 6}, {NULL}};
const Segment SGI159094[] = {{&S30926[118], 3}, {NULL}};
const Segment SGI159095[] = {{&S30926[83], 6}, {NULL}};
const Segment SGI159096[] = {{&S30930[44], 8}, {NULL}};
const Segment SGI159097[] = {{&S30930[44], 20}, {NULL}};
const Segment SGI159098[] = {{&S30930[30], 6}, {NULL}};
const Segment SGI159099[] = {{&S30930[113], 3}, {NULL}};
const Segment SGI159100[] = {{&S30930[78], 6}, {NULL}};
const Segment SGI159101[] = {{&S30931[44], 9}, {NULL}};
const Segment SGI159102[] = {{&S30931[44], 20}, {NULL}};
const Segment SGI159103[] = {{&S30931[30], 6}, {NULL}};
const Segment SGI159104[] = {{&S30931[113], 3}, {NULL}};
const Segment SGI159105[] = {{&S30931[78], 6}, {NULL}};
const Segment SGI159106[] = {{&S30932[44], 9}, {NULL}};
const Segment SGI159107[] = {{&S30932[44], 20}, {NULL}};
const Segment SGI159108[] = {{&S30932[30], 6}, {NULL}};
const Segment SGI159109[] = {{&S30932[113], 3}, {NULL}};
const Segment SGI159110[] = {{&S30932[78], 6}, {NULL}};
const Segment SGI159111[] = {{&S30933[38], 15}, {NULL}};
const Segment SGI159112[] = {{&S30933[31], 6}, {NULL}};
const Segment SGI159113[] = {{&S30936[71], 13}, {NULL}};
const Segment SGI159114[] = {{&S30936[71], 24}, {NULL}};
const Segment SGI159115[] = {{&S30936[57], 6}, {NULL}};
const Segment SGI159116[] = {{&S30936[144], 3}, {NULL}};
const Segment SGI159117[] = {{&S30936[109], 6}, {NULL}};
const Segment SGI159118[] = {{&S30941[44], 8}, {NULL}};
const Segment SGI159119[] = {{&S30941[44], 19}, {NULL}};
const Segment SGI159120[] = {{&S30941[30], 6}, {NULL}};
const Segment SGI159121[] = {{&S30941[112], 3}, {NULL}};
const Segment SGI159122[] = {{&S30941[77], 6}, {NULL}};
const Segment SGI159123[] = {{&S54983[44], 8}, {NULL}};
const Segment SGI159124[] = {{&S54983[44], 20}, {NULL}};
const Segment SGI159125[] = {{&S54983[30], 6}, {NULL}};
const Segment SGI159126[] = {{&S54983[31], 3}, {NULL}};
const Segment SGI159127[] = {{&S54983[78], 6}, {NULL}};
const Segment SGI159128[] = {{&S44767[34], 7}, {NULL}};
const Segment SGI159129[] = {{&S44767[27], 6}, {NULL}};
const Segment SGI159130[] = {{&S30943[44], 13}, {NULL}};
const Segment SGI159131[] = {{&S30943[44], 24}, {NULL}};
const Segment SGI159132[] = {{&S30943[30], 6}, {NULL}};
const Segment SGI159133[] = {{&S44767[99], 9}, {NULL}};
const Segment SGI159134[] = {{&S44767[72], 6}, {NULL}};
const Segment SGI159135[] = {{&S30943[117], 3}, {NULL}};
const Segment SGI159136[] = {{&S54984[38], 8}, {NULL}};
const Segment SGI159137[] = {{&S54984[31], 6}, {NULL}};
const Segment SGI159138[] = {{&S30943[82], 6}, {NULL}};
const Segment SGI159139[] = {{&S30944[44], 19}, {NULL}};
const Segment SGI159140[] = {{&S30944[44], 30}, {NULL}};
const Segment SGI159141[] = {{&S30944[30], 6}, {NULL}};
const Segment SGI159142[] = {{&S30944[123], 3}, {NULL}};
const Segment SGI159143[] = {{&S30944[88], 6}, {NULL}};
const Segment SGI159144[] = {{&S30945[44], 10}, {NULL}};
const Segment SGI159145[] = {{&S30945[44], 22}, {NULL}};
const Segment SGI159146[] = {{&S30945[30], 6}, {NULL}};
const Segment SGI159147[] = {{&S30945[115], 3}, {NULL}};
const Segment SGI159148[] = {{&S30945[80], 6}, {NULL}};
const Segment SGI159149[] = {{&S30946[44], 17}, {NULL}};
const Segment SGI159150[] = {{&S30946[44], 29}, {NULL}};
const Segment SGI159151[] = {{&S30946[30], 6}, {NULL}};
const Segment SGI159152[] = {{&S30946[122], 3}, {NULL}};
const Segment SGI159153[] = {{&S30946[87], 6}, {NULL}};
const Segment SGI159154[] = {{&S30947[44], 19}, {NULL}};
const Segment SGI159155[] = {{&S30947[30], 6}, {NULL}};
const Segment SGI159156[] = {{&S30947[125], 3}, {NULL}};
const Segment SGI159157[] = {{&S30947[90], 6}, {NULL}};
const Segment SGI159158[] = {{&S30949[44], 5}, {NULL}};
const Segment SGI159159[] = {{&S30949[44], 16}, {NULL}};
const Segment SGI159160[] = {{&S30949[30], 6}, {NULL}};
const Segment SGI159161[] = {{&S30949[109], 3}, {NULL}};
const Segment SGI159162[] = {{&S30949[74], 6}, {NULL}};
const Segment SGI159163[] = {{&S30950[44], 6}, {NULL}};
const Segment SGI159164[] = {{&S30950[44], 17}, {NULL}};
const Segment SGI159165[] = {{&S30950[30], 6}, {NULL}};
const Segment SGI159166[] = {{&S30950[110], 3}, {NULL}};
const Segment SGI159167[] = {{&S46068[44], 15}, {NULL}};
const Segment SGI159168[] = {{&S30950[75], 6}, {NULL}};
const Segment SGI159169[] = {{&S46068[44], 27}, {NULL}};
const Segment SGI159170[] = {{&S46068[30], 6}, {NULL}};
const Segment SGI159171[] = {{&S46068[136], 3}, {NULL}};
const Segment SGI159172[] = {{&S46068[101], 6}, {NULL}};
const Segment SGI159173[] = {{&S46068[73], 8}, {NULL}};
const Segment SGI159174[] = {{&S44782[44], 9}, {NULL}};
const Segment SGI159175[] = {{&S44782[44], 21}, {NULL}};
const Segment SGI159176[] = {{&S44782[30], 6}, {NULL}};
const Segment SGI159177[] = {{&S30951[44], 30}, {NULL}};
const Segment SGI159178[] = {{&S30951[44], 41}, {NULL}};
const Segment SGI159179[] = {{&S30951[30], 6}, {NULL}};
const Segment SGI159180[] = {{&S44782[114], 3}, {NULL}};
const Segment SGI159181[] = {{&S44782[79], 6}, {NULL}};
const Segment SGI159182[] = {{&S30951[134], 3}, {NULL}};
const Segment SGI159183[] = {{&S30951[99], 6}, {NULL}};
const Segment SGI159184[] = {{&S44783[44], 17}, {NULL}};
const Segment SGI159185[] = {{&S44783[44], 29}, {NULL}};
const Segment SGI159186[] = {{&S44783[30], 6}, {NULL}};
const Segment SGI159187[] = {{&S30952[44], 10}, {NULL}};
const Segment SGI159188[] = {{&S30952[44], 21}, {NULL}};
const Segment SGI159189[] = {{&S44783[122], 3}, {NULL}};
const Segment SGI159190[] = {{&S30952[30], 6}, {NULL}};
const Segment SGI159191[] = {{&S44783[87], 6}, {NULL}};
const Segment SGI159192[] = {{&S30952[114], 3}, {NULL}};
const Segment SGI159193[] = {{&S30952[79], 6}, {NULL}};
const Segment SGI159194[] = {{&S44784[38], 9}, {NULL}};
const Segment SGI159195[] = {{&S44784[31], 6}, {NULL}};
const Segment SGI159196[] = {{&S30954[44], 4}, {NULL}};
const Segment SGI159197[] = {{&S30954[44], 15}, {NULL}};
const Segment SGI159198[] = {{&S30954[30], 6}, {NULL}};
const Segment SGI159199[] = {{&S30954[108], 3}, {NULL}};
const Segment SGI159200[] = {{&S30954[73], 6}, {NULL}};
const Segment SGI159201[] = {{&S30955[44], 14}, {NULL}};
const Segment SGI159202[] = {{&S30955[44], 25}, {NULL}};
const Segment SGI159203[] = {{&S30955[30], 6}, {NULL}};
const Segment SGI159204[] = {{&S30955[118], 3}, {NULL}};
const Segment SGI159205[] = {{&S30955[83], 6}, {NULL}};
const Segment SGI159206[] = {{&S30956[44], 5}, {NULL}};
const Segment SGI159207[] = {{&S30956[44], 16}, {NULL}};
const Segment SGI159208[] = {{&S30956[30], 6}, {NULL}};
const Segment SGI159209[] = {{&S30956[109], 3}, {NULL}};
const Segment SGI159210[] = {{&S30956[74], 6}, {NULL}};
const Segment SGI159211[] = {{&S30957[44], 8}, {NULL}};
const Segment SGI159212[] = {{&S30957[44], 19}, {NULL}};
const Segment SGI159213[] = {{&S30957[30], 6}, {NULL}};
const Segment SGI159214[] = {{&S30957[112], 3}, {NULL}};
const Segment SGI159215[] = {{&S30957[77], 6}, {NULL}};
const Segment SGI159216[] = {{&S30958[44], 12}, {NULL}};
const Segment SGI159217[] = {{&S30958[44], 23}, {NULL}};
const Segment SGI159218[] = {{&S30958[30], 6}, {NULL}};
const Segment SGI159219[] = {{&S30958[116], 3}, {NULL}};
const Segment SGI159220[] = {{&S30958[81], 6}, {NULL}};
const Segment SGI159221[] = {{&S30959[44], 22}, {NULL}};
const Segment SGI159222[] = {{&S30959[44], 33}, {NULL}};
const Segment SGI159223[] = {{&S30959[30], 6}, {NULL}};
const Segment SGI159224[] = {{&S30959[126], 3}, {NULL}};
const Segment SGI159225[] = {{&S30959[91], 6}, {NULL}};
const Segment SGI159226[] = {{&S30963[44], 10}, {NULL}};
const Segment SGI159227[] = {{&S30963[44], 21}, {NULL}};
const Segment SGI159228[] = {{&S30963[30], 6}, {NULL}};
const Segment SGI159229[] = {{&S30963[114], 3}, {NULL}};
const Segment SGI159230[] = {{&S30963[79], 6}, {NULL}};
const Segment SGI159231[] = {{&S30968[44], 5}, {NULL}};
const Segment SGI159232[] = {{&S30968[44], 16}, {NULL}};
const Segment SGI159233[] = {{&S30968[30], 6}, {NULL}};
const Segment SGI159234[] = {{&S30968[109], 3}, {NULL}};
const Segment SGI159235[] = {{&S30968[74], 6}, {NULL}};
const Segment SGI159236[] = {{&S30969[44], 21}, {NULL}};
const Segment SGI159237[] = {{&S30969[44], 36}, {NULL}};
const Segment SGI159238[] = {{&S30969[30], 6}, {NULL}};
const Segment SGI159239[] = {{&S30969[129], 3}, {NULL}};
const Segment SGI159240[] = {{&S30969[94], 6}, {NULL}};
const Segment SGI159241[] = {{&S30971[44], 8}, {NULL}};
const Segment SGI159242[] = {{&S30971[44], 19}, {NULL}};
const Segment SGI159243[] = {{&S30971[30], 6}, {NULL}};
const Segment SGI159244[] = {{&S30971[112], 3}, {NULL}};
const Segment SGI159245[] = {{&S30971[77], 6}, {NULL}};
const Segment SGI159246[] = {{&S30972[44], 13}, {NULL}};
const Segment SGI159247[] = {{&S30972[44], 24}, {NULL}};
const Segment SGI159248[] = {{&S30972[30], 6}, {NULL}};
const Segment SGI159249[] = {{&S30972[117], 3}, {NULL}};
const Segment SGI159250[] = {{&S30972[82], 6}, {NULL}};
const Segment SGI159251[] = {{&S30974[44], 5}, {NULL}};
const Segment SGI159252[] = {{&S30974[44], 16}, {NULL}};
const Segment SGI159253[] = {{&S30974[30], 6}, {NULL}};
const Segment SGI159254[] = {{&S30974[109], 3}, {NULL}};
const Segment SGI159255[] = {{&S30974[74], 6}, {NULL}};
const Segment SGI159256[] = {{&S30976[44], 16}, {NULL}};
const Segment SGI159257[] = {{&S30976[44], 36}, {NULL}};
const Segment SGI159258[] = {{&S30976[30], 6}, {NULL}};
const Segment SGI159259[] = {{&S30976[129], 3}, {NULL}};
const Segment SGI159260[] = {{&S30976[94], 6}, {NULL}};
const Segment SGI159261[] = {{&S30977[44], 5}, {NULL}};
const Segment SGI159262[] = {{&S30977[44], 26}, {NULL}};
const Segment SGI159263[] = {{&S30977[30], 6}, {NULL}};
const Segment SGI159264[] = {{&S30977[118], 3}, {NULL}};
const Segment SGI159265[] = {{&S30977[84], 5}, {NULL}};
const Segment SGI159266[] = {{&S30978[44], 21}, {NULL}};
const Segment SGI159267[] = {{&S30978[44], 32}, {NULL}};
const Segment SGI159268[] = {{&S30978[30], 6}, {NULL}};
const Segment SGI159269[] = {{&S30978[125], 3}, {NULL}};
const Segment SGI159270[] = {{&S30978[90], 6}, {NULL}};
const Segment SGI159271[] = {{&S55044[44], 10}, {NULL}};
const Segment SGI159272[] = {{&S55044[44], 22}, {NULL}};
const Segment SGI159273[] = {{&S55044[30], 6}, {NULL}};
const Segment SGI159274[] = {{&S44827[34], 10}, {NULL}};
const Segment SGI159275[] = {{&S55044[31], 3}, {NULL}};
const Segment SGI159276[] = {{&S44827[27], 6}, {NULL}};
const Segment SGI159277[] = {{&S55044[80], 6}, {NULL}};
const Segment SGI159278[] = {{&S30979[44], 20}, {NULL}};
const Segment SGI159279[] = {{&S30979[44], 32}, {NULL}};
const Segment SGI159280[] = {{&S30979[30], 6}, {NULL}};
const Segment SGI159281[] = {{&S44827[97], 9}, {NULL}};
const Segment SGI159282[] = {{&S44827[70], 6}, {NULL}};
const Segment SGI159283[] = {{&S30979[125], 3}, {NULL}};
const Segment SGI159284[] = {{&S30979[90], 6}, {NULL}};
const Segment SGI159285[] = {{&S55045[38], 10}, {NULL}};
const Segment SGI159286[] = {{&S55045[31], 6}, {NULL}};
const Segment SGI159287[] = {{&S30980[44], 12}, {NULL}};
const Segment SGI159288[] = {{&S30980[44], 23}, {NULL}};
const Segment SGI159289[] = {{&S30980[30], 6}, {NULL}};
const Segment SGI159290[] = {{&S30980[116], 3}, {NULL}};
const Segment SGI159291[] = {{&S30980[81], 6}, {NULL}};
const Segment SGI159292[] = {{&S44829[34], 10}, {NULL}};
const Segment SGI159293[] = {{&S44829[27], 6}, {NULL}};
const Segment SGI159294[] = {{&S55046[34], 7}, {NULL}};
const Segment SGI159295[] = {{&S55046[27], 6}, {NULL}};
const Segment SGI159296[] = {{&S44829[97], 9}, {NULL}};
const Segment SGI159297[] = {{&S44829[70], 6}, {NULL}};
const Segment SGI159298[] = {{&S55046[112], 9}, {NULL}};
const Segment SGI159299[] = {{&S55046[85], 6}, {NULL}};
const Segment SGI159300[] = {{&S30981[44], 7}, {NULL}};
const Segment SGI159301[] = {{&S30981[44], 18}, {NULL}};
const Segment SGI159302[] = {{&S30981[30], 6}, {NULL}};
const Segment SGI159303[] = {{&S30981[111], 3}, {NULL}};
const Segment SGI159304[] = {{&S30981[76], 6}, {NULL}};
const Segment SGI159305[] = {{&S30982[44], 10}, {NULL}};
const Segment SGI159306[] = {{&S44831[34], 10}, {NULL}};
const Segment SGI159307[] = {{&S30982[44], 21}, {NULL}};
const Segment SGI159308[] = {{&S44831[27], 6}, {NULL}};
const Segment SGI159309[] = {{&S30982[30], 6}, {NULL}};
const Segment SGI159310[] = {{&S30982[114], 3}, {NULL}};
const Segment SGI159311[] = {{&S44831[97], 9}, {NULL}};
const Segment SGI159312[] = {{&S44831[70], 6}, {NULL}};
const Segment SGI159313[] = {{&S30982[79], 6}, {NULL}};
const Segment SGI159314[] = {{&S30983[44], 11}, {NULL}};
const Segment SGI159315[] = {{&S30983[44], 22}, {NULL}};
const Segment SGI159316[] = {{&S30983[30], 6}, {NULL}};
const Segment SGI159317[] = {{&S30983[115], 3}, {NULL}};
const Segment SGI159318[] = {{&S30983[80], 6}, {NULL}};
const Segment SGI159319[] = {{&S55053[34], 6}, {NULL}};
const Segment SGI159320[] = {{&S30985[43], 10}, {NULL}};
const Segment SGI159321[] = {{&S55053[27], 6}, {NULL}};
const Segment SGI159322[] = {{&S30985[43], 22}, {NULL}};
const Segment SGI159323[] = {{&S30985[30], 6}, {NULL}};
const Segment SGI159324[] = {{&S30985[114], 3}, {NULL}};
const Segment SGI159325[] = {{&S55053[93], 9}, {NULL}};
const Segment SGI159326[] = {{&S55053[66], 6}, {NULL}};
const Segment SGI159327[] = {{&S30985[79], 6}, {NULL}};
const Segment SGI159328[] = {{&S55054[34], 13}, {NULL}};
const Segment SGI159329[] = {{&S55054[27], 6}, {NULL}};
const Segment SGI159330[] = {{&S30986[44], 9}, {NULL}};
const Segment SGI159331[] = {{&S30986[44], 27}, {NULL}};
const Segment SGI159332[] = {{&S30986[30], 6}, {NULL}};
const Segment SGI159333[] = {{&S55054[100], 9}, {NULL}};
const Segment SGI159334[] = {{&S55054[73], 6}, {NULL}};
const Segment SGI159335[] = {{&S30986[120], 3}, {NULL}};
const Segment SGI159336[] = {{&S30986[85], 6}, {NULL}};
const Segment SGI159337[] = {{&S55055[34], 6}, {NULL}};
const Segment SGI159338[] = {{&S55055[27], 6}, {NULL}};
const Segment SGI159339[] = {{&S55055[93], 9}, {NULL}};
const Segment SGI159340[] = {{&S55055[66], 6}, {NULL}};
const Segment SGI159341[] = {{&S30988[44], 9}, {NULL}};
const Segment SGI159342[] = {{&S30988[44], 21}, {NULL}};
const Segment SGI159343[] = {{&S30988[30], 6}, {NULL}};
const Segment SGI159344[] = {{&S30988[114], 3}, {NULL}};
const Segment SGI159345[] = {{&S30988[79], 6}, {NULL}};
const Segment SGI159346[] = {{&S30989[44], 22}, {NULL}};
const Segment SGI159347[] = {{&S30989[44], 33}, {NULL}};
const Segment SGI159348[] = {{&S30989[30], 6}, {NULL}};
const Segment SGI159349[] = {{&S30989[126], 3}, {NULL}};
const Segment SGI159350[] = {{&S30989[91], 6}, {NULL}};
const Segment SGI159351[] = {{&S30995[44], 9}, {NULL}};
const Segment SGI159352[] = {{&S30995[44], 20}, {NULL}};
const Segment SGI159353[] = {{&S30995[30], 6}, {NULL}};
const Segment SGI159354[] = {{&S30995[113], 3}, {NULL}};
const Segment SGI159355[] = {{&S30995[78], 6}, {NULL}};
const Segment SGI159356[] = {{&S30997[44], 11}, {NULL}};
const Segment SGI159357[] = {{&S30997[30], 6}, {NULL}};
const Segment SGI159358[] = {{&S30997[117], 3}, {NULL}};
const Segment SGI159359[] = {{&S30997[82], 6}, {NULL}};
const Segment SGI159360[] = {{&S30998[44], 10}, {NULL}};
const Segment SGI159361[] = {{&S30998[44], 21}, {NULL}};
const Segment SGI159362[] = {{&S30998[30], 6}, {NULL}};
const Segment SGI159363[] = {{&S30998[114], 3}, {NULL}};
const Segment SGI159364[] = {{&S30998[79], 6}, {NULL}};
const Segment SGI159365[] = {{&S31000[44], 14}, {NULL}};
const Segment SGI159366[] = {{&S31000[44], 25}, {NULL}};
const Segment SGI159367[] = {{&S31000[30], 6}, {NULL}};
const Segment SGI159368[] = {{&S31000[118], 3}, {NULL}};
const Segment SGI159369[] = {{&S31000[83], 6}, {NULL}};
const Segment SGI159370[] = {{&S31001[44], 5}, {NULL}};
const Segment SGI159371[] = {{&S31001[44], 16}, {NULL}};
const Segment SGI159372[] = {{&S31001[30], 6}, {NULL}};
const Segment SGI159373[] = {{&S31001[109], 3}, {NULL}};
const Segment SGI159374[] = {{&S31001[74], 6}, {NULL}};
const Segment SGI159375[] = {{&S31002[44], 10}, {NULL}};
const Segment SGI159376[] = {{&S31002[44], 21}, {NULL}};
const Segment SGI159377[] = {{&S31002[30], 6}, {NULL}};
const Segment SGI159378[] = {{&S31002[114], 3}, {NULL}};
const Segment SGI159379[] = {{&S31002[79], 6}, {NULL}};
const Segment SGI159380[] = {{&S31003[44], 18}, {NULL}};
const Segment SGI159381[] = {{&S31003[44], 30}, {NULL}};
const Segment SGI159382[] = {{&S31003[30], 6}, {NULL}};
const Segment SGI159383[] = {{&S31003[31], 3}, {NULL}};
const Segment SGI159384[] = {{&S31003[88], 6}, {NULL}};
const Segment SGI159385[] = {{&S55080[44], 12}, {NULL}};
const Segment SGI159386[] = {{&S55080[44], 42}, {NULL}};
const Segment SGI159387[] = {{&S55080[30], 6}, {NULL}};
const Segment SGI159388[] = {{&S55080[31], 3}, {NULL}};
const Segment SGI159389[] = {{&S55080[100], 6}, {NULL}};
const Segment SGI159390[] = {{&S55081[44], 9}, {NULL}};
const Segment SGI159391[] = {{&S31005[44], 10}, {NULL}};
const Segment SGI159392[] = {{&S55081[44], 39}, {NULL}};
const Segment SGI159393[] = {{&S31005[44], 21}, {NULL}};
const Segment SGI159394[] = {{&S55081[30], 6}, {NULL}};
const Segment SGI159395[] = {{&S31005[30], 6}, {NULL}};
const Segment SGI159396[] = {{&S55081[31], 3}, {NULL}};
const Segment SGI159397[] = {{&S31005[114], 3}, {NULL}};
const Segment SGI159398[] = {{&S55081[97], 6}, {NULL}};
const Segment SGI159399[] = {{&S31005[79], 6}, {NULL}};
const Segment SGI159400[] = {{&S55082[44], 11}, {NULL}};
const Segment SGI159401[] = {{&S55082[44], 23}, {NULL}};
const Segment SGI159402[] = {{&S55082[30], 6}, {NULL}};
const Segment SGI159403[] = {{&S55082[31], 3}, {NULL}};
const Segment SGI159404[] = {{&S55082[81], 6}, {NULL}};
const Segment SGI159405[] = {{&S31009[34], 9}, {NULL}};
const Segment SGI159406[] = {{&S31009[27], 6}, {NULL}};
const Segment SGI159407[] = {{&S31009[95], 9}, {NULL}};
const Segment SGI159408[] = {{&S31009[68], 6}, {NULL}};
const Segment SGI159409[] = {{&S31010[34], 9}, {NULL}};
const Segment SGI159410[] = {{&S31010[27], 6}, {NULL}};
const Segment SGI159411[] = {{&S31010[95], 9}, {NULL}};
const Segment SGI159412[] = {{&S31010[68], 6}, {NULL}};
const Segment SGI159413[] = {{&S55093[34], 7}, {NULL}};
const Segment SGI159414[] = {{&S55093[27], 6}, {NULL}};
const Segment SGI159415[] = {{&S55093[94], 9}, {NULL}};
const Segment SGI159416[] = {{&S55093[67], 6}, {NULL}};
const Segment SGI159417[] = {{&S46194[34], 10}, {NULL}};
const Segment SGI159418[] = {{&S46194[27], 6}, {NULL}};
const Segment SGI159419[] = {{&S46194[97], 9}, {NULL}};
const Segment SGI159420[] = {{&S46194[70], 6}, {NULL}};
const Segment SGI159421[] = {{&S46209[34], 10}, {NULL}};
const Segment SGI159422[] = {{&S46209[27], 6}, {NULL}};
const Segment SGI159423[] = {{&S46209[97], 9}, {NULL}};
const Segment SGI159424[] = {{&S46209[70], 6}, {NULL}};
const Segment SGI159425[] = {{&S55136[44], 12}, {NULL}};
const Segment SGI159426[] = {{&S55136[44], 24}, {NULL}};
const Segment SGI159427[] = {{&S55136[30], 6}, {NULL}};
const Segment SGI159428[] = {{&S55136[31], 3}, {NULL}};
const Segment SGI159429[] = {{&S55136[82], 6}, {NULL}};
const Segment SGI159430[] = {{&S55137[44], 12}, {NULL}};
const Segment SGI159431[] = {{&S55137[44], 24}, {NULL}};
const Segment SGI159432[] = {{&S55137[30], 6}, {NULL}};
const Segment SGI159433[] = {{&S55137[31], 3}, {NULL}};
const Segment SGI159434[] = {{&S55137[82], 6}, {NULL}};
const Segment SGI159435[] = {{&S55138[44], 15}, {NULL}};
const Segment SGI159436[] = {{&S55138[44], 27}, {NULL}};
const Segment SGI159437[] = {{&S55138[30], 6}, {NULL}};
const Segment SGI159438[] = {{&S55138[31], 3}, {NULL}};
const Segment SGI159439[] = {{&S55138[85], 6}, {NULL}};
const Segment SGI159440[] = {{&S55139[44], 16}, {NULL}};
const Segment SGI159441[] = {{&S55139[44], 28}, {NULL}};
const Segment SGI159442[] = {{&S55139[30], 6}, {NULL}};
const Segment SGI159443[] = {{&S55139[31], 3}, {NULL}};
const Segment SGI159444[] = {{&S55139[86], 6}, {NULL}};
const Segment SGI159445[] = {{&S55141[44], 16}, {NULL}};
const Segment SGI159446[] = {{&S55141[44], 28}, {NULL}};
const Segment SGI159447[] = {{&S55141[30], 6}, {NULL}};
const Segment SGI159448[] = {{&S55141[31], 3}, {NULL}};
const Segment SGI159449[] = {{&S55141[86], 6}, {NULL}};
const Segment SGI159450[] = {{&S55142[38], 16}, {NULL}};
const Segment SGI159451[] = {{&S55142[31], 6}, {NULL}};
const Segment SGI159452[] = {{&S46243[34], 10}, {NULL}};
const Segment SGI159453[] = {{&S46243[27], 6}, {NULL}};
const Segment SGI159454[] = {{&S46243[97], 9}, {NULL}};
const Segment SGI159455[] = {{&S46243[70], 6}, {NULL}};
const Segment SGI159456[] = {{&S44987[44], 12}, {NULL}};
const Segment SGI159457[] = {{&S44987[44], 29}, {NULL}};
const Segment SGI159458[] = {{&S44987[30], 6}, {NULL}};
const Segment SGI159459[] = {{&S44987[122], 3}, {NULL}};
const Segment SGI159460[] = {{&S44987[87], 6}, {NULL}};
const Segment SGI159461[] = {{&S55188[34], 10}, {NULL}};
const Segment SGI159462[] = {{&S55188[27], 6}, {NULL}};
const Segment SGI159463[] = {{&S55188[97], 9}, {NULL}};
const Segment SGI159464[] = {{&S55188[70], 6}, {NULL}};
const Segment SGI159465[] = {{&S44988[44], 8}, {NULL}};
const Segment SGI159466[] = {{&S44988[44], 25}, {NULL}};
const Segment SGI159467[] = {{&S44988[30], 6}, {NULL}};
const Segment SGI159468[] = {{&S44988[118], 3}, {NULL}};
const Segment SGI159469[] = {{&S44988[83], 6}, {NULL}};
const Segment SGI159470[] = {{&S44989[38], 8}, {NULL}};
const Segment SGI159471[] = {{&S44989[31], 6}, {NULL}};
const Segment SGI159472[] = {{&S44990[44], 12}, {NULL}};
const Segment SGI159473[] = {{&S44990[44], 29}, {NULL}};
const Segment SGI159474[] = {{&S44990[30], 6}, {NULL}};
const Segment SGI159475[] = {{&S44990[122], 3}, {NULL}};
const Segment SGI159476[] = {{&S44990[87], 6}, {NULL}};
const Segment SGI159477[] = {{&S55193[34], 21}, {NULL}};
const Segment SGI159478[] = {{&S55193[27], 6}, {NULL}};
const Segment SGI159479[] = {{&S55193[108], 9}, {NULL}};
const Segment SGI159480[] = {{&S55193[81], 6}, {NULL}};
const Segment SGI159481[] = {{&S44991[42], 12}, {NULL}};
const Segment SGI159482[] = {{&S44991[30], 6}, {NULL}};
const Segment SGI159483[] = {{&S44991[120], 3}, {NULL}};
const Segment SGI159484[] = {{&S44991[85], 6}, {NULL}};
const Segment SGI159485[] = {{&S44993[34], 9}, {NULL}};
const Segment SGI159486[] = {{&S55197[34], 20}, {NULL}};
const Segment SGI159487[] = {{&S44993[27], 6}, {NULL}};
const Segment SGI159488[] = {{&S55197[27], 6}, {NULL}};
const Segment SGI159489[] = {{&S44993[96], 9}, {NULL}};
const Segment SGI159490[] = {{&S44993[69], 6}, {NULL}};
const Segment SGI159491[] = {{&S55197[107], 9}, {NULL}};
const Segment SGI159492[] = {{&S55197[80], 6}, {NULL}};
const Segment SGI159493[] = {{&S41042[34], 9}, {NULL}};
const Segment SGI159494[] = {{&S41042[27], 6}, {NULL}};
const Segment SGI159495[] = {{&S41042[96], 9}, {NULL}};
const Segment SGI159496[] = {{&S41042[69], 6}, {NULL}};
const Segment SGI159497[] = {{&S44994[34], 14}, {NULL}};
const Segment SGI159498[] = {{&S44994[27], 6}, {NULL}};
const Segment SGI159499[] = {{&S55198[34], 22}, {NULL}};
const Segment SGI159500[] = {{&S55198[27], 6}, {NULL}};
const Segment SGI159501[] = {{&S44994[101], 9}, {NULL}};
const Segment SGI159502[] = {{&S44994[74], 6}, {NULL}};
const Segment SGI159503[] = {{&S55198[109], 9}, {NULL}};
const Segment SGI159504[] = {{&S55198[82], 6}, {NULL}};
const Segment SGI159505[] = {{&S44995[34], 8}, {NULL}};
const Segment SGI159506[] = {{&S44995[27], 6}, {NULL}};
const Segment SGI159507[] = {{&S55199[34], 25}, {NULL}};
const Segment SGI159508[] = {{&S55199[27], 6}, {NULL}};
const Segment SGI159509[] = {{&S44995[95], 9}, {NULL}};
const Segment SGI159510[] = {{&S44995[68], 6}, {NULL}};
const Segment SGI159511[] = {{&S55199[112], 9}, {NULL}};
const Segment SGI159512[] = {{&S55199[85], 6}, {NULL}};
const Segment SGI159513[] = {{&S44996[34], 9}, {NULL}};
const Segment SGI159514[] = {{&S44996[27], 6}, {NULL}};
const Segment SGI159515[] = {{&S55200[34], 25}, {NULL}};
const Segment SGI159516[] = {{&S44996[96], 9}, {NULL}};
const Segment SGI159517[] = {{&S44996[69], 6}, {NULL}};
const Segment SGI159518[] = {{&S55200[27], 6}, {NULL}};
const Segment SGI159519[] = {{&S55200[112], 9}, {NULL}};
const Segment SGI159520[] = {{&S55200[85], 6}, {NULL}};
const Segment SGI159521[] = {{&S44997[34], 11}, {NULL}};
const Segment SGI159522[] = {{&S44997[27], 6}, {NULL}};
const Segment SGI159523[] = {{&S44997[98], 9}, {NULL}};
const Segment SGI159524[] = {{&S44997[71], 6}, {NULL}};
const Segment SGI159525[] = {{&S55201[34], 30}, {NULL}};
const Segment SGI159526[] = {{&S55201[27], 6}, {NULL}};
const Segment SGI159527[] = {{&S55201[117], 9}, {NULL}};
const Segment SGI159528[] = {{&S55201[90], 6}, {NULL}};
const Segment SGI159529[] = {{&S44998[34], 8}, {NULL}};
const Segment SGI159530[] = {{&S44998[27], 6}, {NULL}};
const Segment SGI159531[] = {{&S44998[95], 9}, {NULL}};
const Segment SGI159532[] = {{&S44998[68], 6}, {NULL}};
const Segment SGI159533[] = {{&S55202[34], 21}, {NULL}};
const Segment SGI159534[] = {{&S55202[27], 6}, {NULL}};
const Segment SGI159535[] = {{&S55202[108], 9}, {NULL}};
const Segment SGI159536[] = {{&S55202[81], 6}, {NULL}};
const Segment SGI159537[] = {{&S44999[34], 11}, {NULL}};
const Segment SGI159538[] = {{&S44999[27], 6}, {NULL}};
const Segment SGI159539[] = {{&S44999[98], 9}, {NULL}};
const Segment SGI159540[] = {{&S44999[71], 6}, {NULL}};
const Segment SGI159541[] = {{&S55204[34], 29}, {NULL}};
const Segment SGI159542[] = {{&S55204[27], 6}, {NULL}};
const Segment SGI159543[] = {{&S55204[116], 9}, {NULL}};
const Segment SGI159544[] = {{&S55204[89], 6}, {NULL}};
const Segment SGI159545[] = {{&S55206[34], 27}, {NULL}};
const Segment SGI159546[] = {{&S55206[27], 6}, {NULL}};
const Segment SGI159547[] = {{&S55206[114], 9}, {NULL}};
const Segment SGI159548[] = {{&S55206[87], 6}, {NULL}};
const Segment SGI159549[] = {{&S55211[44], 13}, {NULL}};
const Segment SGI159550[] = {{&S55211[44], 25}, {NULL}};
const Segment SGI159551[] = {{&S55211[30], 6}, {NULL}};
const Segment SGI159552[] = {{&S55211[31], 3}, {NULL}};
const Segment SGI159553[] = {{&S55211[83], 6}, {NULL}};
const Segment SGI159554[] = {{&S55212[44], 12}, {NULL}};
const Segment SGI159555[] = {{&S55212[44], 24}, {NULL}};
const Segment SGI159556[] = {{&S55212[30], 6}, {NULL}};
const Segment SGI159557[] = {{&S55212[31], 3}, {NULL}};
const Segment SGI159558[] = {{&S55212[82], 6}, {NULL}};
const Segment SGI159559[] = {{&S55213[38], 13}, {NULL}};
const Segment SGI159560[] = {{&S55213[31], 6}, {NULL}};
const Segment SGI159561[] = {{&S46313[34], 10}, {NULL}};
const Segment SGI159562[] = {{&S46313[27], 6}, {NULL}};
const Segment SGI159563[] = {{&S46313[97], 9}, {NULL}};
const Segment SGI159564[] = {{&S46313[70], 6}, {NULL}};
const Segment SGI159565[] = {{&S45054[34], 10}, {NULL}};
const Segment SGI159566[] = {{&S45054[27], 6}, {NULL}};
const Segment SGI159567[] = {{&S45054[97], 9}, {NULL}};
const Segment SGI159568[] = {{&S45054[70], 6}, {NULL}};
const Segment SGI159569[] = {{&S45056[34], 10}, {NULL}};
const Segment SGI159570[] = {{&S45056[27], 6}, {NULL}};
const Segment SGI159571[] = {{&S45056[97], 9}, {NULL}};
const Segment SGI159572[] = {{&S45056[70], 6}, {NULL}};
const Segment SGI159573[] = {{&S55272[34], 41}, {NULL}};
const Segment SGI159574[] = {{&S55272[27], 6}, {NULL}};
const Segment SGI159575[] = {{&S55272[128], 9}, {NULL}};
const Segment SGI159576[] = {{&S55272[101], 6}, {NULL}};
const Segment SGI159577[] = {{&S46370[34], 10}, {NULL}};
const Segment SGI159578[] = {{&S46370[27], 6}, {NULL}};
const Segment SGI159579[] = {{&S55274[34], 11}, {NULL}};
const Segment SGI159580[] = {{&S46370[97], 9}, {NULL}};
const Segment SGI159581[] = {{&S55274[27], 6}, {NULL}};
const Segment SGI159582[] = {{&S46370[70], 6}, {NULL}};
const Segment SGI159583[] = {{&S55274[98], 9}, {NULL}};
const Segment SGI159584[] = {{&S55274[71], 6}, {NULL}};
const Segment SGI159585[] = {{&S46376[34], 10}, {NULL}};
const Segment SGI159586[] = {{&S46376[27], 6}, {NULL}};
const Segment SGI159587[] = {{&S46376[97], 9}, {NULL}};
const Segment SGI159588[] = {{&S46376[70], 6}, {NULL}};
const Segment SGI159589[] = {{&S55302[44], 17}, {NULL}};
const Segment SGI159590[] = {{&S55302[44], 37}, {NULL}};
const Segment SGI159591[] = {{&S55302[30], 6}, {NULL}};
const Segment SGI159592[] = {{&S55302[31], 3}, {NULL}};
const Segment SGI159593[] = {{&S55302[95], 6}, {NULL}};
const Segment SGI159594[] = {{&S55303[38], 17}, {NULL}};
const Segment SGI159595[] = {{&S55303[31], 6}, {NULL}};
const Segment SGI159596[] = {{&S46399[34], 10}, {NULL}};
const Segment SGI159597[] = {{&S46399[27], 6}, {NULL}};
const Segment SGI159598[] = {{&S46399[97], 9}, {NULL}};
const Segment SGI159599[] = {{&S46399[70], 6}, {NULL}};
const Segment SGI159600[] = {{&S55316[34], 7}, {NULL}};
const Segment SGI159601[] = {{&S55316[27], 6}, {NULL}};
const Segment SGI159602[] = {{&S55316[105], 9}, {NULL}};
const Segment SGI159603[] = {{&S55316[78], 6}, {NULL}};
const Segment SGI159604[] = {{&S55317[34], 7}, {NULL}};
const Segment SGI159605[] = {{&S55317[27], 6}, {NULL}};
const Segment SGI159606[] = {{&S55317[105], 9}, {NULL}};
const Segment SGI159607[] = {{&S46413[34], 10}, {NULL}};
const Segment SGI159608[] = {{&S55317[78], 6}, {NULL}};
const Segment SGI159609[] = {{&S46413[27], 6}, {NULL}};
const Segment SGI159610[] = {{&S46413[97], 9}, {NULL}};
const Segment SGI159611[] = {{&S46413[70], 6}, {NULL}};
const Segment SGI159612[] = {{&S55341[44], 11}, {NULL}};
const Segment SGI159613[] = {{&S55341[44], 23}, {NULL}};
const Segment SGI159614[] = {{&S55341[30], 6}, {NULL}};
const Segment SGI159615[] = {{&S55341[31], 3}, {NULL}};
const Segment SGI159616[] = {{&S55341[81], 6}, {NULL}};
const Segment SGI159617[] = {{&S55342[44], 5}, {NULL}};
const Segment SGI159618[] = {{&S55342[44], 17}, {NULL}};
const Segment SGI159619[] = {{&S55342[30], 6}, {NULL}};
const Segment SGI159620[] = {{&S55342[31], 3}, {NULL}};
const Segment SGI159621[] = {{&S55342[75], 6}, {NULL}};
const Segment SGI159622[] = {{&S55343[44], 19}, {NULL}};
const Segment SGI159623[] = {{&S55343[44], 31}, {NULL}};
const Segment SGI159624[] = {{&S55343[30], 6}, {NULL}};
const Segment SGI159625[] = {{&S55343[31], 3}, {NULL}};
const Segment SGI159626[] = {{&S55343[89], 6}, {NULL}};
const Segment SGI159627[] = {{&S55344[44], 8}, {NULL}};
const Segment SGI159628[] = {{&S55344[44], 20}, {NULL}};
const Segment SGI159629[] = {{&S55344[30], 6}, {NULL}};
const Segment SGI159630[] = {{&S55344[31], 3}, {NULL}};
const Segment SGI159631[] = {{&S55344[78], 6}, {NULL}};
const Segment SGI159632[] = {{&S55346[44], 12}, {NULL}};
const Segment SGI159633[] = {{&S55346[44], 24}, {NULL}};
const Segment SGI159634[] = {{&S55346[30], 6}, {NULL}};
const Segment SGI159635[] = {{&S55346[31], 3}, {NULL}};
const Segment SGI159636[] = {{&S55346[82], 6}, {NULL}};
const Segment SGI159637[] = {{&S45148[34], 12}, {NULL}};
const Segment SGI159638[] = {{&S45148[27], 6}, {NULL}};
const Segment SGI159639[] = {{&S45148[109], 9}, {NULL}};
const Segment SGI159640[] = {{&S55347[44], 10}, {NULL}};
const Segment SGI159641[] = {{&S45148[82], 6}, {NULL}};
const Segment SGI159642[] = {{&S55347[44], 22}, {NULL}};
const Segment SGI159643[] = {{&S55347[30], 6}, {NULL}};
const Segment SGI159644[] = {{&S55347[31], 3}, {NULL}};
const Segment SGI159645[] = {{&S55347[80], 6}, {NULL}};
const Segment SGI159646[] = {{&S43502[32], 10}, {NULL}};
const Segment SGI159647[] = {{&S43502[27], 4}, {NULL}};
const Segment SGI159648[] = {{&S43502[95], 9}, {NULL}};
const Segment SGI159649[] = {{&S45149[34], 12}, {NULL}};
const Segment SGI159650[] = {{&S43502[68], 6}, {NULL}};
const Segment SGI159651[] = {{&S45149[27], 6}, {NULL}};
const Segment SGI159652[] = {{&S45149[109], 9}, {NULL}};
const Segment SGI159653[] = {{&S55348[44], 12}, {NULL}};
const Segment SGI159654[] = {{&S45149[82], 6}, {NULL}};
const Segment SGI159655[] = {{&S55348[44], 24}, {NULL}};
const Segment SGI159656[] = {{&S55348[30], 6}, {NULL}};
const Segment SGI159657[] = {{&S55348[31], 3}, {NULL}};
const Segment SGI159658[] = {{&S55348[82], 6}, {NULL}};
const Segment SGI159659[] = {{&S55349[44], 20}, {NULL}};
const Segment SGI159660[] = {{&S55349[44], 32}, {NULL}};
const Segment SGI159661[] = {{&S55349[30], 6}, {&S55349[52], 24}, {NULL}};
const Segment SGI159662[] = {{&S55349[31], 3}, {NULL}};
const Segment SGI159663[] = {{&S55349[90], 6}, {NULL}};
const Segment SGI159664[] = {{&S45155[34], 10}, {NULL}};
const Segment SGI159665[] = {{&S45155[27], 6}, {NULL}};
const Segment SGI159666[] = {{&S45155[97], 9}, {NULL}};
const Segment SGI159667[] = {{&S45155[70], 6}, {NULL}};
const Segment SGI159668[] = {{&S45156[34], 11}, {NULL}};
const Segment SGI159669[] = {{&S45156[27], 6}, {NULL}};
const Segment SGI159670[] = {{&S45156[98], 9}, {NULL}};
const Segment SGI159671[] = {{&S45156[71], 6}, {NULL}};
const Segment SGI159672[] = {{&S55362[44], 6}, {NULL}};
const Segment SGI159673[] = {{&S55362[44], 18}, {NULL}};
const Segment SGI159674[] = {{&S55362[30], 6}, {NULL}};
const Segment SGI159675[] = {{&S55362[31], 3}, {NULL}};
const Segment SGI159676[] = {{&S55362[76], 6}, {NULL}};
const Segment SGI159677[] = {{&S45164[34], 7}, {NULL}};
const Segment SGI159678[] = {{&S45164[27], 6}, {NULL}};
const Segment SGI159679[] = {{&S45164[94], 9}, {NULL}};
const Segment SGI159680[] = {{&S45164[67], 6}, {NULL}};
const Segment SGI159681[] = {{&S55363[38], 6}, {NULL}};
const Segment SGI159682[] = {{&S55363[31], 6}, {NULL}};
const Segment SGI159683[] = {{&S45167[34], 11}, {NULL}};
const Segment SGI159684[] = {{&S45167[27], 6}, {NULL}};
const Segment SGI159685[] = {{&S45167[98], 9}, {NULL}};
const Segment SGI159686[] = {{&S45167[71], 6}, {NULL}};
const Segment SGI159687[] = {{&S55368[44], 11}, {NULL}};
const Segment SGI159688[] = {{&S55368[44], 33}, {NULL}};
const Segment SGI159689[] = {{&S55368[30], 6}, {NULL}};
const Segment SGI159690[] = {{&S46473[44], 6}, {NULL}};
const Segment SGI159691[] = {{&S46473[44], 18}, {NULL}};
const Segment SGI159692[] = {{&S46473[30], 6}, {NULL}};
const Segment SGI159693[] = {{&S55368[31], 3}, {NULL}};
const Segment SGI159694[] = {{&S55368[91], 5}, {NULL}};
const Segment SGI159695[] = {{&S46473[111], 3}, {NULL}};
const Segment SGI159696[] = {{&S46473[76], 6}, {NULL}};
const Segment SGI159697[] = {{&S55369[44], 6}, {NULL}};
const Segment SGI159698[] = {{&S55369[44], 18}, {NULL}};
const Segment SGI159699[] = {{&S46474[44], 6}, {NULL}};
const Segment SGI159700[] = {{&S55369[30], 6}, {NULL}};
const Segment SGI159701[] = {{&S46474[44], 18}, {NULL}};
const Segment SGI159702[] = {{&S46474[30], 6}, {NULL}};
const Segment SGI159703[] = {{&S55369[31], 3}, {NULL}};
const Segment SGI159704[] = {{&S55369[76], 6}, {NULL}};
const Segment SGI159705[] = {{&S46474[79], 6}, {NULL}};
const Segment SGI159706[] = {{&S55370[44], 11}, {NULL}};
const Segment SGI159707[] = {{&S55370[44], 33}, {NULL}};
const Segment SGI159708[] = {{&S46475[44], 18}, {NULL}};
const Segment SGI159709[] = {{&S55370[30], 6}, {NULL}};
const Segment SGI159710[] = {{&S46475[30], 6}, {NULL}};
const Segment SGI159711[] = {{&S55370[31], 3}, {NULL}};
const Segment SGI159712[] = {{&S55370[91], 5}, {NULL}};
const Segment SGI159713[] = {{&S46476[44], 7}, {NULL}};
const Segment SGI159714[] = {{&S46476[44], 19}, {NULL}};
const Segment SGI159715[] = {{&S46476[30], 6}, {NULL}};
const Segment SGI159716[] = {{&S55371[44], 11}, {NULL}};
const Segment SGI159717[] = {{&S55371[44], 33}, {NULL}};
const Segment SGI159718[] = {{&S55371[30], 6}, {NULL}};
const Segment SGI159719[] = {{&S46476[80], 6}, {NULL}};
const Segment SGI159720[] = {{&S55371[31], 3}, {NULL}};
const Segment SGI159721[] = {{&S55371[91], 5}, {NULL}};
const Segment SGI159722[] = {{&S46477[44], 7}, {NULL}};
const Segment SGI159723[] = {{&S46477[44], 19}, {NULL}};
const Segment SGI159724[] = {{&S46477[30], 6}, {NULL}};
const Segment SGI159725[] = {{&S55372[44], 11}, {NULL}};
const Segment SGI159726[] = {{&S55372[44], 33}, {NULL}};
const Segment SGI159727[] = {{&S46477[112], 3}, {NULL}};
const Segment SGI159728[] = {{&S55372[30], 6}, {NULL}};
const Segment SGI159729[] = {{&S46477[77], 6}, {NULL}};
const Segment SGI159730[] = {{&S55372[31], 3}, {NULL}};
const Segment SGI159731[] = {{&S55372[91], 5}, {NULL}};
const Segment SGI159732[] = {{&S46478[44], 7}, {NULL}};
const Segment SGI159733[] = {{&S46478[44], 19}, {NULL}};
const Segment SGI159734[] = {{&S46478[30], 6}, {NULL}};
const Segment SGI159735[] = {{&S46478[112], 3}, {NULL}};
const Segment SGI159736[] = {{&S46478[77], 6}, {NULL}};
const Segment SGI159737[] = {{&S55374[34], 7}, {NULL}};
const Segment SGI159738[] = {{&S55374[27], 6}, {NULL}};
const Segment SGI159739[] = {{&S46479[44], 8}, {NULL}};
const Segment SGI159740[] = {{&S55374[93], 9}, {NULL}};
const Segment SGI159741[] = {{&S55374[66], 6}, {NULL}};
const Segment SGI159742[] = {{&S46479[44], 20}, {NULL}};
const Segment SGI159743[] = {{&S46479[30], 6}, {NULL}};
const Segment SGI159744[] = {{&S46479[113], 3}, {NULL}};
const Segment SGI159745[] = {{&S46479[78], 6}, {NULL}};
const Segment SGI159746[] = {{&S55375[34], 7}, {NULL}};
const Segment SGI159747[] = {{&S55375[27], 6}, {NULL}};
const Segment SGI159748[] = {{&S55375[94], 9}, {NULL}};
const Segment SGI159749[] = {{&S46480[44], 7}, {NULL}};
const Segment SGI159750[] = {{&S55375[67], 6}, {NULL}};
const Segment SGI159751[] = {{&S46480[44], 19}, {NULL}};
const Segment SGI159752[] = {{&S46480[30], 6}, {NULL}};
const Segment SGI159753[] = {{&S46480[112], 3}, {NULL}};
const Segment SGI159754[] = {{&S46480[77], 6}, {NULL}};
const Segment SGI159755[] = {{&S55376[34], 19}, {NULL}};
const Segment SGI159756[] = {{&S55376[27], 6}, {NULL}};
const Segment SGI159757[] = {{&S46481[54], 6}, {NULL}};
const Segment SGI159758[] = {{&S46481[47], 6}, {NULL}};
const Segment SGI159759[] = {{&S55376[106], 9}, {NULL}};
const Segment SGI159760[] = {{&S55376[79], 6}, {NULL}};
const Segment SGI159761[] = {{&S46482[44], 8}, {NULL}};
const Segment SGI159762[] = {{&S46482[44], 20}, {NULL}};
const Segment SGI159763[] = {{&S46482[30], 6}, {NULL}};
const Segment SGI159764[] = {{&S55377[34], 19}, {NULL}};
const Segment SGI159765[] = {{&S55377[27], 6}, {NULL}};
const Segment SGI159766[] = {{&S46482[81], 6}, {NULL}};
const Segment SGI159767[] = {{&S55377[106], 9}, {NULL}};
const Segment SGI159768[] = {{&S55377[79], 6}, {NULL}};
const Segment SGI159769[] = {{&S46483[44], 7}, {NULL}};
const Segment SGI159770[] = {{&S46483[44], 19}, {NULL}};
const Segment SGI159771[] = {{&S46483[30], 6}, {NULL}};
const Segment SGI159772[] = {{&S46483[80], 6}, {NULL}};
const Segment SGI159773[] = {{&S46484[31], 6}, {NULL}};
const Segment SGI159774[] = {{&S46485[44], 15}, {NULL}};
const Segment SGI159775[] = {{&S46485[44], 27}, {NULL}};
const Segment SGI159776[] = {{&S46485[30], 6}, {NULL}};
const Segment SGI159777[] = {{&S55381[34], 7}, {NULL}};
const Segment SGI159778[] = {{&S55381[27], 6}, {NULL}};
const Segment SGI159779[] = {{&S46485[120], 3}, {NULL}};
const Segment SGI159780[] = {{&S46485[85], 6}, {NULL}};
const Segment SGI159781[] = {{&S55381[100], 9}, {NULL}};
const Segment SGI159782[] = {{&S55381[73], 6}, {NULL}};
const Segment SGI159783[] = {{&S46486[44], 15}, {NULL}};
const Segment SGI159784[] = {{&S46486[44], 27}, {NULL}};
const Segment SGI159785[] = {{&S46486[30], 6}, {NULL}};
const Segment SGI159786[] = {{&S46486[120], 3}, {NULL}};
const Segment SGI159787[] = {{&S46486[85], 6}, {NULL}};
const Segment SGI159788[] = {{&S55383[34], 10}, {NULL}};
const Segment SGI159789[] = {{&S55383[27], 6}, {NULL}};
const Segment SGI159790[] = {{&S46487[38], 15}, {NULL}};
const Segment SGI159791[] = {{&S46487[31], 6}, {NULL}};
const Segment SGI159792[] = {{&S55383[97], 9}, {NULL}};
const Segment SGI159793[] = {{&S55383[70], 6}, {NULL}};
const Segment SGI159794[] = {{&S46488[44], 11}, {NULL}};
const Segment SGI159795[] = {{&S46488[44], 23}, {NULL}};
const Segment SGI159796[] = {{&S46488[30], 6}, {NULL}};
const Segment SGI159797[] = {{&S46488[116], 3}, {NULL}};
const Segment SGI159798[] = {{&S46488[81], 6}, {NULL}};
const Segment SGI159799[] = {{&S46489[44], 11}, {NULL}};
const Segment SGI159800[] = {{&S46489[44], 23}, {NULL}};
const Segment SGI159801[] = {{&S46489[30], 6}, {NULL}};
const Segment SGI159802[] = {{&S46489[116], 3}, {NULL}};
const Segment SGI159803[] = {{&S46489[81], 6}, {NULL}};
const Segment SGI159804[] = {{&S46490[38], 11}, {NULL}};
const Segment SGI159805[] = {{&S46490[31], 6}, {NULL}};
const Segment SGI159806[] = {{&S41187[44], 10}, {NULL}};
const Segment SGI159807[] = {{&S41187[44], 22}, {NULL}};
const Segment SGI159808[] = {{&S41187[30], 6}, {NULL}};
const Segment SGI159809[] = {{&S41187[115], 3}, {NULL}};
const Segment SGI159810[] = {{&S41187[80], 6}, {NULL}};
const Segment SGI159811[] = {{&S41188[44], 10}, {NULL}};
const Segment SGI159812[] = {{&S46500[34], 10}, {NULL}};
const Segment SGI159813[] = {{&S41188[44], 22}, {NULL}};
const Segment SGI159814[] = {{&S46500[27], 6}, {NULL}};
const Segment SGI159815[] = {{&S41188[30], 6}, {NULL}};
const Segment SGI159816[] = {{&S41188[115], 3}, {NULL}};
const Segment SGI159817[] = {{&S46500[97], 9}, {NULL}};
const Segment SGI159818[] = {{&S41188[80], 6}, {NULL}};
const Segment SGI159819[] = {{&S46500[70], 6}, {NULL}};
const Segment SGI159820[] = {{&S55412[34], 11}, {NULL}};
const Segment SGI159821[] = {{&S55412[27], 6}, {NULL}};
const Segment SGI159822[] = {{&S55412[98], 9}, {NULL}};
const Segment SGI159823[] = {{&S55412[71], 6}, {NULL}};
const Segment SGI159824[] = {{&S55446[34], 7}, {NULL}};
const Segment SGI159825[] = {{&S55446[27], 6}, {NULL}};
const Segment SGI159826[] = {{&S55446[99], 9}, {NULL}};
const Segment SGI159827[] = {{&S55446[72], 6}, {NULL}};
const Segment SGI159828[] = {{&S55447[34], 7}, {NULL}};
const Segment SGI159829[] = {{&S55447[27], 6}, {NULL}};
const Segment SGI159830[] = {{&S55447[99], 9}, {NULL}};
const Segment SGI159831[] = {{&S55447[72], 6}, {NULL}};
const Segment SGI159832[] = {{&S55448[34], 8}, {NULL}};
const Segment SGI159833[] = {{&S55448[27], 6}, {NULL}};
const Segment SGI159834[] = {{&S55448[99], 9}, {NULL}};
const Segment SGI159835[] = {{&S55448[72], 6}, {NULL}};
const Segment SGI159836[] = {{&S55449[34], 8}, {NULL}};
const Segment SGI159837[] = {{&S55449[27], 6}, {NULL}};
const Segment SGI159838[] = {{&S55449[99], 9}, {NULL}};
const Segment SGI159839[] = {{&S55449[72], 6}, {NULL}};
const Segment SGI159840[] = {{&S55452[44], 15}, {NULL}};
const Segment SGI159841[] = {{&S55452[44], 42}, {NULL}};
const Segment SGI159842[] = {{&S55452[30], 6}, {NULL}};
const Segment SGI159843[] = {{&S55452[31], 3}, {NULL}};
const Segment SGI159844[] = {{&S55452[100], 6}, {NULL}};
const Segment SGI159845[] = {{&S55453[44], 13}, {NULL}};
const Segment SGI159846[] = {{&S55453[44], 25}, {NULL}};
const Segment SGI159847[] = {{&S55453[30], 6}, {NULL}};
const Segment SGI159848[] = {{&S55453[31], 3}, {NULL}};
const Segment SGI159849[] = {{&S55453[83], 6}, {NULL}};
const Segment SGI159850[] = {{&S55459[34], 7}, {NULL}};
const Segment SGI159851[] = {{&S55459[27], 6}, {NULL}};
const Segment SGI159852[] = {{&S55459[106], 9}, {NULL}};
const Segment SGI159853[] = {{&S55459[79], 6}, {NULL}};
const Segment SGI159854[] = {{&S55460[34], 7}, {NULL}};
const Segment SGI159855[] = {{&S55460[27], 6}, {NULL}};
const Segment SGI159856[] = {{&S55460[106], 9}, {NULL}};
const Segment SGI159857[] = {{&S55460[79], 6}, {NULL}};
const Segment SGI159858[] = {{&S41263[34], 10}, {NULL}};
const Segment SGI159859[] = {{&S41263[27], 6}, {NULL}};
const Segment SGI159860[] = {{&S41263[97], 9}, {NULL}};
const Segment SGI159861[] = {{&S55461[34], 7}, {NULL}};
const Segment SGI159862[] = {{&S41263[70], 6}, {NULL}};
const Segment SGI159863[] = {{&S55461[27], 6}, {NULL}};
const Segment SGI159864[] = {{&S55461[106], 9}, {NULL}};
const Segment SGI159865[] = {{&S55461[79], 6}, {NULL}};
const Segment SGI159866[] = {{&S55463[34], 12}, {NULL}};
const Segment SGI159867[] = {{&S55463[27], 6}, {NULL}};
const Segment SGI159868[] = {{&S55463[99], 9}, {NULL}};
const Segment SGI159869[] = {{&S55463[72], 6}, {NULL}};
const Segment SGI159870[] = {{&S50390[32], 14}, {NULL}};
const Segment SGI159871[] = {{&S50390[27], 4}, {NULL}};
const Segment SGI159872[] = {{&S50390[99], 9}, {NULL}};
const Segment SGI159873[] = {{&S50390[72], 6}, {NULL}};
const Segment SGI159874[] = {{&S41290[44], 10}, {NULL}};
const Segment SGI159875[] = {{&S41290[44], 22}, {NULL}};
const Segment SGI159876[] = {{&S41290[30], 6}, {NULL}};
const Segment SGI159877[] = {{&S41290[115], 3}, {NULL}};
const Segment SGI159878[] = {{&S41290[80], 6}, {NULL}};
const Segment SGI159879[] = {{&S41291[38], 10}, {NULL}};
const Segment SGI159880[] = {{&S41291[31], 6}, {NULL}};
const Segment SGI159881[] = {{&S41293[44], 10}, {NULL}};
const Segment SGI159882[] = {{&S41293[44], 22}, {NULL}};
const Segment SGI159883[] = {{&S41293[30], 6}, {NULL}};
const Segment SGI159884[] = {{&S41293[115], 3}, {NULL}};
const Segment SGI159885[] = {{&S41293[80], 6}, {NULL}};
const Segment SGI159886[] = {{&S41297[44], 10}, {NULL}};
const Segment SGI159887[] = {{&S41297[44], 22}, {NULL}};
const Segment SGI159888[] = {{&S41297[30], 6}, {NULL}};
const Segment SGI159889[] = {{&S41297[115], 3}, {NULL}};
const Segment SGI159890[] = {{&S41297[80], 6}, {NULL}};
const Segment SGI159891[] = {{&S41354[44], 9}, {NULL}};
const Segment SGI159892[] = {{&S41354[44], 21}, {NULL}};
const Segment SGI159893[] = {{&S41354[30], 6}, {NULL}};
const Segment SGI159894[] = {{&S41354[132], 3}, {NULL}};
const Segment SGI159895[] = {{&S41354[97], 6}, {NULL}};
const Segment SGI159896[] = {{&S45397[34], 10}, {NULL}};
const Segment SGI159897[] = {{&S41354[67], 8}, {NULL}};
const Segment SGI159898[] = {{&S45397[27], 6}, {NULL}};
const Segment SGI159899[] = {{&S55560[44], 12}, {NULL}};
const Segment SGI159900[] = {{&S55560[44], 24}, {NULL}};
const Segment SGI159901[] = {{&S55560[30], 6}, {NULL}};
const Segment SGI159902[] = {{&S45397[97], 9}, {NULL}};
const Segment SGI159903[] = {{&S45397[70], 6}, {NULL}};
const Segment SGI159904[] = {{&S55560[31], 3}, {NULL}};
const Segment SGI159905[] = {{&S55560[101], 6}, {NULL}};
const Segment SGI159906[] = {{&S55560[70], 8}, {NULL}};
const Segment SGI159907[] = {{&S41355[44], 9}, {NULL}};
const Segment SGI159908[] = {{&S41355[44], 21}, {NULL}};
const Segment SGI159909[] = {{&S41355[30], 6}, {NULL}};
const Segment SGI159910[] = {{&S41355[132], 3}, {NULL}};
const Segment SGI159911[] = {{&S41355[97], 6}, {NULL}};
const Segment SGI159912[] = {{&S41355[67], 8}, {NULL}};
const Segment SGI159913[] = {{&S55561[44], 12}, {NULL}};
const Segment SGI159914[] = {{&S55561[44], 24}, {NULL}};
const Segment SGI159915[] = {{&S55561[30], 6}, {NULL}};
const Segment SGI159916[] = {{&S55561[31], 3}, {NULL}};
const Segment SGI159917[] = {{&S55561[111], 6}, {NULL}};
const Segment SGI159918[] = {{&S55561[70], 8}, {NULL}};
const Segment SGI159919[] = {{&S55562[44], 12}, {NULL}};
const Segment SGI159920[] = {{&S55562[44], 24}, {NULL}};
const Segment SGI159921[] = {{&S55562[30], 6}, {NULL}};
const Segment SGI159922[] = {{&S55562[31], 3}, {NULL}};
const Segment SGI159923[] = {{&S55562[103], 6}, {NULL}};
const Segment SGI159924[] = {{&S55562[70], 8}, {NULL}};
const Segment SGI159925[] = {{&S45403[34], 10}, {NULL}};
const Segment SGI159926[] = {{&S45403[27], 6}, {NULL}};
const Segment SGI159927[] = {{&S45403[97], 9}, {NULL}};
const Segment SGI159928[] = {{&S45403[70], 6}, {NULL}};
const Segment SGI159929[] = {{&S45412[34], 7}, {NULL}};
const Segment SGI159930[] = {{&S45412[27], 6}, {NULL}};
const Segment SGI159931[] = {{&S45412[100], 9}, {NULL}};
const Segment SGI159932[] = {{&S45412[73], 6}, {NULL}};
const Segment SGI159933[] = {{&S45413[34], 7}, {NULL}};
const Segment SGI159934[] = {{&S45413[27], 6}, {NULL}};
const Segment SGI159935[] = {{&S45413[100], 9}, {NULL}};
const Segment SGI159936[] = {{&S45413[73], 6}, {NULL}};
const Segment SGI159937[] = {{&S55586[34], 7}, {NULL}};
const Segment SGI159938[] = {{&S55586[27], 6}, {NULL}};
const Segment SGI159939[] = {{&S55586[98], 9}, {NULL}};
const Segment SGI159940[] = {{&S55586[71], 6}, {NULL}};
const Segment SGI159941[] = {{&S55597[34], 10}, {NULL}};
const Segment SGI159942[] = {{&S55597[27], 6}, {NULL}};
const Segment SGI159943[] = {{&S55597[97], 9}, {NULL}};
const Segment SGI159944[] = {{&S55597[70], 6}, {NULL}};
const Segment SGI159945[] = {{&S55598[34], 10}, {NULL}};
const Segment SGI159946[] = {{&S55598[27], 6}, {NULL}};
const Segment SGI159947[] = {{&S55598[97], 9}, {NULL}};
const Segment SGI159948[] = {{&S55598[70], 6}, {NULL}};
const Segment SGI159949[] = {{&S55599[34], 7}, {NULL}};
const Segment SGI159950[] = {{&S55599[27], 6}, {NULL}};
const Segment SGI159951[] = {{&S55599[100], 9}, {NULL}};
const Segment SGI159952[] = {{&S55599[73], 6}, {NULL}};
const Segment SGI159953[] = {{&S45449[34], 10}, {NULL}};
const Segment SGI159954[] = {{&S45449[27], 6}, {NULL}};
const Segment SGI159955[] = {{&S45449[97], 9}, {NULL}};
const Segment SGI159956[] = {{&S45449[70], 6}, {NULL}};
const Segment SGI159957[] = {{&S45451[34], 10}, {NULL}};
const Segment SGI159958[] = {{&S45451[27], 6}, {NULL}};
const Segment SGI159959[] = {{&S45451[97], 9}, {NULL}};
const Segment SGI159960[] = {{&S45451[70], 6}, {NULL}};
const Segment SGI159961[] = {{&S55626[44], 16}, {NULL}};
const Segment SGI159962[] = {{&S55626[44], 35}, {NULL}};
const Segment SGI159963[] = {{&S55626[30], 6}, {NULL}};
const Segment SGI159964[] = {{&S55626[31], 3}, {NULL}};
const Segment SGI159965[] = {{&S55626[93], 6}, {NULL}};
const Segment SGI159966[] = {{&S55627[44], 16}, {NULL}};
const Segment SGI159967[] = {{&S55627[44], 35}, {NULL}};
const Segment SGI159968[] = {{&S55627[30], 6}, {NULL}};
const Segment SGI159969[] = {{&S55627[31], 3}, {NULL}};
const Segment SGI159970[] = {{&S55627[93], 6}, {NULL}};
const Segment SGI159971[] = {{&S55628[44], 16}, {NULL}};
const Segment SGI159972[] = {{&S55628[44], 35}, {NULL}};
const Segment SGI159973[] = {{&S55628[30], 6}, {NULL}};
const Segment SGI159974[] = {{&S55628[31], 3}, {NULL}};
const Segment SGI159975[] = {{&S55628[93], 6}, {NULL}};
const Segment SGI159976[] = {{&S55629[44], 16}, {NULL}};
const Segment SGI159977[] = {{&S55629[44], 35}, {NULL}};
const Segment SGI159978[] = {{&S55629[30], 6}, {NULL}};
const Segment SGI159979[] = {{&S55629[31], 3}, {NULL}};
const Segment SGI159980[] = {{&S55629[93], 6}, {NULL}};
const Segment SGI159981[] = {{&S55630[44], 16}, {NULL}};
const Segment SGI159982[] = {{&S55630[44], 40}, {NULL}};
const Segment SGI159983[] = {{&S55630[30], 6}, {NULL}};
const Segment SGI159984[] = {{&S55630[31], 3}, {NULL}};
const Segment SGI159985[] = {{&S55630[98], 6}, {NULL}};
const Segment SGI159986[] = {{&S55631[44], 16}, {NULL}};
const Segment SGI159987[] = {{&S55631[44], 41}, {NULL}};
const Segment SGI159988[] = {{&S55631[30], 6}, {NULL}};
const Segment SGI159989[] = {{&S55631[31], 3}, {NULL}};
const Segment SGI159990[] = {{&S55631[99], 6}, {NULL}};
const Segment SGI159991[] = {{&S55632[44], 16}, {NULL}};
const Segment SGI159992[] = {{&S55632[44], 41}, {NULL}};
const Segment SGI159993[] = {{&S55632[30], 6}, {NULL}};
const Segment SGI159994[] = {{&S55632[31], 3}, {NULL}};
const Segment SGI159995[] = {{&S55632[99], 6}, {NULL}};
const Segment SGI159996[] = {{&S55633[44], 19}, {NULL}};
const Segment SGI159997[] = {{&S55633[44], 43}, {NULL}};
const Segment SGI159998[] = {{&S55633[30], 6}, {NULL}};
const Segment SGI159999[] = {{&S55633[31], 3}, {NULL}};
const Segment SGI160000[] = {{&S55633[101], 6}, {NULL}};
const Segment SGI160001[] = {{&S55634[44], 19}, {NULL}};
const Segment SGI160002[] = {{&S55634[44], 44}, {NULL}};
const Segment SGI160003[] = {{&S55634[30], 6}, {NULL}};
const Segment SGI160004[] = {{&S50578[42], 14}, {NULL}};
const Segment SGI160005[] = {{&S50578[42], 26}, {NULL}};
const Segment SGI160006[] = {{&S55634[31], 3}, {NULL}};
const Segment SGI160007[] = {{&S50578[30], 4}, {NULL}};
const Segment SGI160008[] = {{&S55634[102], 6}, {NULL}};
const Segment SGI160009[] = {{&S50578[117], 3}, {NULL}};
const Segment SGI160010[] = {{&S50578[82], 6}, {NULL}};
const Segment SGI160011[] = {{&S55635[44], 19}, {NULL}};
const Segment SGI160012[] = {{&S55635[44], 44}, {NULL}};
const Segment SGI160013[] = {{&S55635[30], 6}, {NULL}};
const Segment SGI160014[] = {{&S55635[31], 3}, {NULL}};
const Segment SGI160015[] = {{&S55635[102], 6}, {NULL}};
const Segment SGI160016[] = {{&S55636[34], 10}, {NULL}};
const Segment SGI160017[] = {{&S55636[27], 6}, {NULL}};
const Segment SGI160018[] = {{&S55636[97], 9}, {NULL}};
const Segment SGI160019[] = {{&S55636[70], 6}, {NULL}};
const Segment SGI160020[] = {{&S55637[34], 9}, {NULL}};
const Segment SGI160021[] = {{&S55637[27], 6}, {NULL}};
const Segment SGI160022[] = {{&S55637[96], 9}, {NULL}};
const Segment SGI160023[] = {{&S55637[69], 6}, {NULL}};
const Segment SGI160024[] = {{&S55670[34], 8}, {NULL}};
const Segment SGI160025[] = {{&S55670[27], 6}, {NULL}};
const Segment SGI160026[] = {{&S55670[95], 9}, {NULL}};
const Segment SGI160027[] = {{&S55670[68], 6}, {NULL}};
const Segment SGI160028[] = {{&S55685[44], 9}, {NULL}};
const Segment SGI160029[] = {{&S55685[44], 21}, {NULL}};
const Segment SGI160030[] = {{&S55685[30], 6}, {NULL}};
const Segment SGI160031[] = {{&S55685[31], 3}, {NULL}};
const Segment SGI160032[] = {{&S55685[79], 6}, {NULL}};
const Segment SGI160033[] = {{&S55686[38], 9}, {NULL}};
const Segment SGI160034[] = {{&S55686[31], 6}, {NULL}};
const Segment SGI160035[] = {{&S55687[42], 9}, {NULL}};
const Segment SGI160036[] = {{&S55687[30], 6}, {NULL}};
const Segment SGI160037[] = {{&S55687[31], 3}, {NULL}};
const Segment SGI160038[] = {{&S55687[77], 6}, {NULL}};
const Segment SGI160039[] = {{&S50662[32], 14}, {NULL}};
const Segment SGI160040[] = {{&S50662[27], 4}, {NULL}};
const Segment SGI160041[] = {{&S50662[99], 9}, {NULL}};
const Segment SGI160042[] = {{&S50662[72], 6}, {NULL}};
const Segment SGI160043[] = {{&S55703[44], 10}, {NULL}};
const Segment SGI160044[] = {{&S55703[44], 22}, {NULL}};
const Segment SGI160045[] = {{&S55703[30], 6}, {NULL}};
const Segment SGI160046[] = {{&S55703[139], 4}, {NULL}};
const Segment SGI160047[] = {{&S55703[165], 3}, {NULL}};
const Segment SGI160048[] = {{&S55703[31], 3}, {&S55703[214], 5}, {NULL}};
const Segment SGI160049[] = {{&S55703[80], 6}, {NULL}};
const Segment SGI160050[] = {{&S55705[42], 10}, {NULL}};
const Segment SGI160051[] = {{&S55705[30], 6}, {NULL}};
const Segment SGI160052[] = {{&S55705[137], 4}, {NULL}};
const Segment SGI160053[] = {{&S55705[163], 3}, {NULL}};
const Segment SGI160054[] = {{&S55705[31], 3}, {&S55705[212], 5}, {NULL}};
const Segment SGI160055[] = {{&S55705[78], 6}, {NULL}};
const Segment SGI160056[] = {{&S55710[44], 7}, {NULL}};
const Segment SGI160057[] = {{&S55710[44], 24}, {NULL}};
const Segment SGI160058[] = {{&S55710[30], 6}, {NULL}};
const Segment SGI160059[] = {{&S55710[31], 3}, {NULL}};
const Segment SGI160060[] = {{&S55710[82], 6}, {NULL}};
const Segment SGI160061[] = {{&S55711[44], 7}, {NULL}};
const Segment SGI160062[] = {{&S55711[44], 24}, {NULL}};
const Segment SGI160063[] = {{&S55711[30], 6}, {NULL}};
const Segment SGI160064[] = {{&S55711[31], 3}, {NULL}};
const Segment SGI160065[] = {{&S55711[82], 6}, {NULL}};
const Segment SGI160066[] = {{&S55712[38], 7}, {NULL}};
const Segment SGI160067[] = {{&S55712[31], 6}, {NULL}};
const Segment SGI160068[] = {{&S55755[34], 8}, {NULL}};
const Segment SGI160069[] = {{&S55755[27], 6}, {NULL}};
const Segment SGI160070[] = {{&S55755[95], 9}, {NULL}};
const Segment SGI160071[] = {{&S55755[68], 6}, {NULL}};
const Segment SGI160072[] = {{&S45457[34], 7}, {NULL}};
const Segment SGI160073[] = {{&S45457[27], 6}, {NULL}};
const Segment SGI160074[] = {{&S45457[94], 9}, {NULL}};
const Segment SGI160075[] = {{&S45457[67], 6}, {NULL}};
const Segment SGI160076[] = {{&S45458[34], 14}, {NULL}};
const Segment SGI160077[] = {{&S45458[27], 6}, {NULL}};
const Segment SGI160078[] = {{&S45458[111], 9}, {NULL}};
const Segment SGI160079[] = {{&S45458[84], 6}, {NULL}};
const Segment SGI160080[] = {{&S45459[34], 14}, {NULL}};
const Segment SGI160081[] = {{&S45459[27], 6}, {NULL}};
const Segment SGI160082[] = {{&S45459[111], 9}, {NULL}};
const Segment SGI160083[] = {{&S45459[84], 6}, {NULL}};
const Segment SGI160084[] = {{&S45530[34], 11}, {NULL}};
const Segment SGI160085[] = {{&S45530[27], 6}, {NULL}};
const Segment SGI160086[] = {{&S45530[98], 9}, {NULL}};
const Segment SGI160087[] = {{&S45530[71], 6}, {NULL}};
const Segment SGI160088[] = {{&S55803[34], 7}, {NULL}};
const Segment SGI160089[] = {{&S55803[27], 6}, {NULL}};
const Segment SGI160090[] = {{&S55803[108], 9}, {NULL}};
const Segment SGI160091[] = {{&S55803[81], 6}, {NULL}};
const Segment SGI160092[] = {{&S55804[34], 15}, {NULL}};
const Segment SGI160093[] = {{&S55804[27], 6}, {NULL}};
const Segment SGI160094[] = {{&S55804[118], 9}, {NULL}};
const Segment SGI160095[] = {{&S55804[91], 6}, {NULL}};
const Segment SGI160096[] = {{&S46797[34], 9}, {NULL}};
const Segment SGI160097[] = {{&S46797[27], 6}, {NULL}};
const Segment SGI160098[] = {{&S46797[96], 9}, {NULL}};
const Segment SGI160099[] = {{&S46797[69], 6}, {NULL}};
const Segment SGI160100[] = {{&S45553[44], 11}, {NULL}};
const Segment SGI160101[] = {{&S45553[44], 23}, {NULL}};
const Segment SGI160102[] = {{&S45553[30], 6}, {NULL}};
const Segment SGI160103[] = {{&S45553[116], 3}, {NULL}};
const Segment SGI160104[] = {{&S45553[81], 6}, {NULL}};
const Segment SGI160105[] = {{&S45554[38], 11}, {NULL}};
const Segment SGI160106[] = {{&S45554[31], 6}, {NULL}};
const Segment SGI160107[] = {{&S46800[34], 5}, {NULL}};
const Segment SGI160108[] = {{&S46800[27], 6}, {NULL}};
const Segment SGI160109[] = {{&S46800[92], 9}, {NULL}};
const Segment SGI160110[] = {{&S46800[65], 6}, {NULL}};
const Segment SGI160111[] = {{&S46801[34], 12}, {NULL}};
const Segment SGI160112[] = {{&S45556[44], 10}, {NULL}};
const Segment SGI160113[] = {{&S46801[27], 6}, {NULL}};
const Segment SGI160114[] = {{&S45556[44], 22}, {NULL}};
const Segment SGI160115[] = {{&S45556[30], 6}, {NULL}};
const Segment SGI160116[] = {{&S45556[115], 3}, {NULL}};
const Segment SGI160117[] = {{&S46801[99], 9}, {NULL}};
const Segment SGI160118[] = {{&S46801[72], 6}, {NULL}};
const Segment SGI160119[] = {{&S45556[80], 6}, {NULL}};
const Segment SGI160120[] = {{&S45557[44], 11}, {NULL}};
const Segment SGI160121[] = {{&S45557[44], 23}, {NULL}};
const Segment SGI160122[] = {{&S45557[30], 6}, {NULL}};
const Segment SGI160123[] = {{&S45557[116], 3}, {NULL}};
const Segment SGI160124[] = {{&S45557[81], 6}, {NULL}};
const Segment SGI160125[] = {{&S45559[38], 10}, {NULL}};
const Segment SGI160126[] = {{&S45559[31], 6}, {NULL}};
const Segment SGI160127[] = {{&S55855[44], 22}, {NULL}};
const Segment SGI160128[] = {{&S55855[44], 34}, {NULL}};
const Segment SGI160129[] = {{&S55855[30], 6}, {NULL}};
const Segment SGI160130[] = {{&S55855[31], 3}, {NULL}};
const Segment SGI160131[] = {{&S55855[92], 6}, {NULL}};
const Segment SGI160132[] = {{&S55856[42], 22}, {NULL}};
const Segment SGI160133[] = {{&S55856[30], 6}, {NULL}};
const Segment SGI160134[] = {{&S55856[31], 3}, {NULL}};
const Segment SGI160135[] = {{&S55856[90], 6}, {NULL}};
const Segment SGI160136[] = {{&S55857[38], 22}, {NULL}};
const Segment SGI160137[] = {{&S55857[31], 6}, {NULL}};
const Segment SGI160138[] = {{&S55862[34], 6}, {NULL}};
const Segment SGI160139[] = {{&S55862[27], 6}, {NULL}};
const Segment SGI160140[] = {{&S55862[93], 9}, {NULL}};
const Segment SGI160141[] = {{&S55862[66], 6}, {NULL}};
const Segment SGI160142[] = {{&S46854[34], 10}, {NULL}};
const Segment SGI160143[] = {{&S46854[27], 6}, {NULL}};
const Segment SGI160144[] = {{&S55885[34], 5}, {NULL}};
const Segment SGI160145[] = {{&S55885[27], 6}, {NULL}};
const Segment SGI160146[] = {{&S46854[97], 9}, {NULL}};
const Segment SGI160147[] = {{&S46854[70], 6}, {NULL}};
const Segment SGI160148[] = {{&S55885[92], 9}, {NULL}};
const Segment SGI160149[] = {{&S55885[65], 6}, {NULL}};
const Segment SGI160150[] = {{&S55886[34], 9}, {NULL}};
const Segment SGI160151[] = {{&S55886[27], 6}, {NULL}};
const Segment SGI160152[] = {{&S55886[96], 9}, {NULL}};
const Segment SGI160153[] = {{&S55886[69], 6}, {NULL}};
const Segment SGI160154[] = {{&S55888[34], 5}, {NULL}};
const Segment SGI160155[] = {{&S55888[27], 6}, {NULL}};
const Segment SGI160156[] = {{&S55888[92], 9}, {NULL}};
const Segment SGI160157[] = {{&S55888[65], 6}, {NULL}};
const Segment SGI160158[] = {{&S55891[11], 1}, {NULL}};
const Segment SGI160159[] = {{&S55891[27], 6}, {NULL}};
const Segment SGI160160[] = {{&S55891[88], 9}, {NULL}};
const Segment SGI160161[] = {{&S55891[61], 6}, {NULL}};
const Segment SGI160162[] = {{&S55892[34], 21}, {NULL}};
const Segment SGI160163[] = {{&S55892[27], 6}, {NULL}};
const Segment SGI160164[] = {{&S55892[108], 9}, {NULL}};
const Segment SGI160165[] = {{&S55892[81], 6}, {NULL}};
const Segment SGI160166[] = {{&S55893[34], 21}, {NULL}};
const Segment SGI160167[] = {{&S55893[27], 6}, {NULL}};
const Segment SGI160168[] = {{&S55893[108], 9}, {NULL}};
const Segment SGI160169[] = {{&S55893[81], 6}, {NULL}};
const Segment SGI160170[] = {{&S55895[34], 8}, {NULL}};
const Segment SGI160171[] = {{&S55895[27], 6}, {NULL}};
const Segment SGI160172[] = {{&S55895[95], 9}, {NULL}};
const Segment SGI160173[] = {{&S55895[68], 6}, {NULL}};
const Segment SGI160174[] = {{&S55896[11], 1}, {NULL}};
const Segment SGI160175[] = {{&S55896[27], 6}, {NULL}};
const Segment SGI160176[] = {{&S55896[88], 9}, {NULL}};
const Segment SGI160177[] = {{&S55896[61], 6}, {NULL}};
const Segment SGI160178[] = {{&S55897[34], 16}, {NULL}};
const Segment SGI160179[] = {{&S55897[27], 6}, {NULL}};
const Segment SGI160180[] = {{&S55897[103], 9}, {NULL}};
const Segment SGI160181[] = {{&S55897[76], 6}, {NULL}};
const Segment SGI160182[] = {{&S55899[34], 10}, {NULL}};
const Segment SGI160183[] = {{&S55899[27], 6}, {NULL}};
const Segment SGI160184[] = {{&S55899[97], 9}, {NULL}};
const Segment SGI160185[] = {{&S55899[70], 6}, {NULL}};
const Segment SGI160186[] = {{&S55901[34], 5}, {NULL}};
const Segment SGI160187[] = {{&S55901[27], 6}, {NULL}};
const Segment SGI160188[] = {{&S55901[92], 9}, {NULL}};
const Segment SGI160189[] = {{&S55901[65], 6}, {NULL}};
const Segment SGI160190[] = {{&S55902[34], 16}, {NULL}};
const Segment SGI160191[] = {{&S55902[27], 6}, {NULL}};
const Segment SGI160192[] = {{&S41792[34], 10}, {NULL}};
const Segment SGI160193[] = {{&S41792[27], 6}, {NULL}};
const Segment SGI160194[] = {{&S55902[103], 9}, {NULL}};
const Segment SGI160195[] = {{&S55902[76], 6}, {NULL}};
const Segment SGI160196[] = {{&S41792[97], 9}, {NULL}};
const Segment SGI160197[] = {{&S41792[70], 6}, {NULL}};
const Segment SGI160198[] = {{&S55903[34], 30}, {NULL}};
const Segment SGI160199[] = {{&S55903[27], 6}, {NULL}};
const Segment SGI160200[] = {{&S55903[117], 9}, {NULL}};
const Segment SGI160201[] = {{&S55903[90], 6}, {NULL}};
const Segment SGI160202[] = {{&S41794[34], 10}, {NULL}};
const Segment SGI160203[] = {{&S41794[27], 6}, {NULL}};
const Segment SGI160204[] = {{&S41794[97], 9}, {NULL}};
const Segment SGI160205[] = {{&S41794[70], 6}, {NULL}};
const Segment SGI160206[] = {{&S41795[34], 10}, {NULL}};
const Segment SGI160207[] = {{&S41795[27], 6}, {NULL}};
const Segment SGI160208[] = {{&S41795[97], 9}, {NULL}};
const Segment SGI160209[] = {{&S41795[70], 6}, {NULL}};
const Segment SGI160210[] = {{&S46912[34], 10}, {NULL}};
const Segment SGI160211[] = {{&S46912[27], 6}, {NULL}};
const Segment SGI160212[] = {{&S46912[97], 9}, {NULL}};
const Segment SGI160213[] = {{&S46912[70], 6}, {NULL}};
const Segment SGI160214[] = {{&S31971[34], 9}, {NULL}};
const Segment SGI160215[] = {{&S31971[27], 6}, {NULL}};
const Segment SGI160216[] = {{&S31971[95], 9}, {NULL}};
const Segment SGI160217[] = {{&S31971[68], 6}, {NULL}};
const Segment SGI160218[] = {{&S45580[34], 7}, {NULL}};
const Segment SGI160219[] = {{&S45580[27], 6}, {NULL}};
const Segment SGI160220[] = {{&S45580[100], 9}, {NULL}};
const Segment SGI160221[] = {{&S45580[73], 6}, {NULL}};
const Segment SGI160222[] = {{&S46918[34], 10}, {NULL}};
const Segment SGI160223[] = {{&S46918[27], 6}, {NULL}};
const Segment SGI160224[] = {{&S45581[34], 8}, {NULL}};
const Segment SGI160225[] = {{&S45581[27], 6}, {NULL}};
const Segment SGI160226[] = {{&S46918[97], 9}, {NULL}};
const Segment SGI160227[] = {{&S46918[70], 6}, {NULL}};
const Segment SGI160228[] = {{&S45581[101], 9}, {NULL}};
const Segment SGI160229[] = {{&S45581[74], 6}, {NULL}};
const Segment SGI160230[] = {{&S45589[44], 6}, {NULL}};
const Segment SGI160231[] = {{&S45589[44], 18}, {NULL}};
const Segment SGI160232[] = {{&S45589[30], 6}, {NULL}};
const Segment SGI160233[] = {{&S45589[111], 3}, {NULL}};
const Segment SGI160234[] = {{&S45589[76], 6}, {NULL}};
const Segment SGI160235[] = {{&S45590[38], 6}, {NULL}};
const Segment SGI160236[] = {{&S45590[31], 6}, {NULL}};
const Segment SGI160237[] = {{&S45593[34], 11}, {NULL}};
const Segment SGI160238[] = {{&S45593[27], 6}, {NULL}};
const Segment SGI160239[] = {{&S45593[98], 9}, {NULL}};
const Segment SGI160240[] = {{&S45593[71], 6}, {NULL}};
const Segment SGI160241[] = {{&S45598[44], 10}, {NULL}};
const Segment SGI160242[] = {{&S45598[44], 22}, {NULL}};
const Segment SGI160243[] = {{&S45598[30], 6}, {NULL}};
const Segment SGI160244[] = {{&S45598[115], 3}, {NULL}};
const Segment SGI160245[] = {{&S45598[80], 6}, {NULL}};
const Segment SGI160246[] = {{&S45599[38], 10}, {NULL}};
const Segment SGI160247[] = {{&S45599[31], 6}, {NULL}};
const Segment SGI160248[] = {{&S45600[44], 11}, {NULL}};
const Segment SGI160249[] = {{&S45600[44], 23}, {NULL}};
const Segment SGI160250[] = {{&S45600[30], 6}, {NULL}};
const Segment SGI160251[] = {{&S45600[116], 3}, {NULL}};
const Segment SGI160252[] = {{&S45600[81], 6}, {NULL}};
const Segment SGI160253[] = {{&S45601[44], 11}, {NULL}};
const Segment SGI160254[] = {{&S45601[44], 23}, {NULL}};
const Segment SGI160255[] = {{&S45601[30], 6}, {NULL}};
const Segment SGI160256[] = {{&S45601[116], 3}, {NULL}};
const Segment SGI160257[] = {{&S45601[81], 6}, {NULL}};
const Segment SGI160258[] = {{&S41859[34], 24}, {NULL}};
const Segment SGI160259[] = {{&S41859[27], 6}, {NULL}};
const Segment SGI160260[] = {{&S45602[38], 11}, {NULL}};
const Segment SGI160261[] = {{&S45602[31], 6}, {NULL}};
const Segment SGI160262[] = {{&S41859[111], 9}, {NULL}};
const Segment SGI160263[] = {{&S41859[84], 6}, {NULL}};
const Segment SGI160264[] = {{&S45603[38], 11}, {NULL}};
const Segment SGI160265[] = {{&S45603[31], 6}, {NULL}};
const Segment SGI160266[] = {{&S41860[34], 8}, {NULL}};
const Segment SGI160267[] = {{&S41860[27], 6}, {NULL}};
const Segment SGI160268[] = {{&S41860[95], 9}, {NULL}};
const Segment SGI160269[] = {{&S41860[68], 6}, {NULL}};
const Segment SGI160270[] = {{&S45604[34], 7}, {NULL}};
const Segment SGI160271[] = {{&S45604[27], 6}, {NULL}};
const Segment SGI160272[] = {{&S45604[104], 9}, {NULL}};
const Segment SGI160273[] = {{&S45604[77], 6}, {NULL}};
const Segment SGI160274[] = {{&S41862[34], 29}, {NULL}};
const Segment SGI160275[] = {{&S41862[27], 6}, {NULL}};
const Segment SGI160276[] = {{&S41862[116], 9}, {NULL}};
const Segment SGI160277[] = {{&S41862[89], 6}, {NULL}};
const Segment SGI160278[] = {{&S55979[44], 10}, {NULL}};
const Segment SGI160279[] = {{&S55979[44], 22}, {NULL}};
const Segment SGI160280[] = {{&S55979[30], 6}, {NULL}};
const Segment SGI160281[] = {{&S55979[31], 3}, {NULL}};
const Segment SGI160282[] = {{&S41863[34], 30}, {NULL}};
const Segment SGI160283[] = {{&S55979[80], 6}, {NULL}};
const Segment SGI160284[] = {{&S41863[27], 6}, {NULL}};
const Segment SGI160285[] = {{&S41863[117], 9}, {NULL}};
const Segment SGI160286[] = {{&S41863[90], 6}, {NULL}};
const Segment SGI160287[] = {{&S55980[38], 10}, {NULL}};
const Segment SGI160288[] = {{&S55980[31], 6}, {NULL}};
const Segment SGI160289[] = {{&S41864[34], 30}, {NULL}};
const Segment SGI160290[] = {{&S41864[27], 6}, {NULL}};
const Segment SGI160291[] = {{&S41864[117], 9}, {NULL}};
const Segment SGI160292[] = {{&S41864[90], 6}, {NULL}};
const Segment SGI160293[] = {{&S41865[34], 17}, {NULL}};
const Segment SGI160294[] = {{&S41865[27], 6}, {NULL}};
const Segment SGI160295[] = {{&S55983[34], 7}, {NULL}};
const Segment SGI160296[] = {{&S41865[104], 9}, {NULL}};
const Segment SGI160297[] = {{&S55983[27], 6}, {NULL}};
const Segment SGI160298[] = {{&S41865[77], 6}, {NULL}};
const Segment SGI160299[] = {{&S55983[98], 9}, {NULL}};
const Segment SGI160300[] = {{&S55983[71], 6}, {NULL}};
const Segment SGI160301[] = {{&S44247[32], 10}, {NULL}};
const Segment SGI160302[] = {{&S44247[27], 4}, {NULL}};
const Segment SGI160303[] = {{&S55984[34], 6}, {NULL}};
const Segment SGI160304[] = {{&S44247[95], 9}, {NULL}};
const Segment SGI160305[] = {{&S55984[27], 6}, {NULL}};
const Segment SGI160306[] = {{&S44247[68], 6}, {NULL}};
const Segment SGI160307[] = {{&S55984[93], 9}, {NULL}};
const Segment SGI160308[] = {{&S55984[66], 6}, {NULL}};
const Segment SGI160309[] = {{&S45622[34], 11}, {NULL}};
const Segment SGI160310[] = {{&S45622[27], 6}, {NULL}};
const Segment SGI160311[] = {{&S45622[98], 9}, {NULL}};
const Segment SGI160312[] = {{&S45622[71], 6}, {NULL}};
const Segment SGI160313[] = {{&S45623[34], 10}, {NULL}};
const Segment SGI160314[] = {{&S45623[27], 6}, {NULL}};
const Segment SGI160315[] = {{&S45623[97], 9}, {NULL}};
const Segment SGI160316[] = {{&S45623[70], 6}, {NULL}};
const Segment SGI160317[] = {{&S45624[34], 10}, {NULL}};
const Segment SGI160318[] = {{&S45624[27], 6}, {NULL}};
const Segment SGI160319[] = {{&S45624[97], 9}, {NULL}};
const Segment SGI160320[] = {{&S45624[70], 6}, {NULL}};
const Segment SGI160321[] = {{&S45625[34], 10}, {NULL}};
const Segment SGI160322[] = {{&S45625[27], 6}, {NULL}};
const Segment SGI160323[] = {{&S45625[97], 9}, {NULL}};
const Segment SGI160324[] = {{&S45625[70], 6}, {NULL}};
const Segment SGI160325[] = {{&S45627[34], 11}, {NULL}};
const Segment SGI160326[] = {{&S45627[27], 6}, {NULL}};
const Segment SGI160327[] = {{&S45627[98], 9}, {NULL}};
const Segment SGI160328[] = {{&S45627[71], 6}, {NULL}};
const Segment SGI160329[] = {{&S45628[34], 11}, {NULL}};
const Segment SGI160330[] = {{&S45628[27], 6}, {NULL}};
const Segment SGI160331[] = {{&S45628[98], 9}, {NULL}};
const Segment SGI160332[] = {{&S45628[71], 6}, {NULL}};
const Segment SGI160333[] = {{&S47013[44], 37}, {NULL}};
const Segment SGI160334[] = {{&S47013[30], 6}, {NULL}};
const Segment SGI160335[] = {{&S47013[113], 6}, {NULL}};
const Segment SGI160336[] = {{&S47014[44], 14}, {NULL}};
const Segment SGI160337[] = {{&S47014[44], 31}, {NULL}};
const Segment SGI160338[] = {{&S47014[30], 6}, {NULL}};
const Segment SGI160339[] = {{&S47014[92], 6}, {NULL}};
const Segment SGI160340[] = {{&S47015[44], 37}, {NULL}};
const Segment SGI160341[] = {{&S47015[30], 6}, {NULL}};
const Segment SGI160342[] = {{&S47015[113], 6}, {NULL}};
const Segment SGI160343[] = {{&S47016[44], 14}, {NULL}};
const Segment SGI160344[] = {{&S47016[44], 31}, {NULL}};
const Segment SGI160345[] = {{&S47016[30], 6}, {NULL}};
const Segment SGI160346[] = {{&S47016[124], 3}, {NULL}};
const Segment SGI160347[] = {{&S47016[89], 6}, {NULL}};
const Segment SGI160348[] = {{&S45660[44], 11}, {NULL}};
const Segment SGI160349[] = {{&S45660[44], 28}, {NULL}};
const Segment SGI160350[] = {{&S45660[30], 6}, {NULL}};
const Segment SGI160351[] = {{&S47017[44], 37}, {NULL}};
const Segment SGI160352[] = {{&S47017[30], 6}, {NULL}};
const Segment SGI160353[] = {{&S45660[121], 3}, {NULL}};
const Segment SGI160354[] = {{&S45660[86], 6}, {NULL}};
const Segment SGI160355[] = {{&S47017[113], 6}, {NULL}};
const Segment SGI160356[] = {{&S45661[44], 7}, {NULL}};
const Segment SGI160357[] = {{&S45661[44], 24}, {NULL}};
const Segment SGI160358[] = {{&S45661[30], 6}, {NULL}};
const Segment SGI160359[] = {{&S45661[117], 3}, {NULL}};
const Segment SGI160360[] = {{&S47018[44], 37}, {NULL}};
const Segment SGI160361[] = {{&S47018[30], 6}, {NULL}};
const Segment SGI160362[] = {{&S45661[82], 6}, {NULL}};
const Segment SGI160363[] = {{&S47018[113], 6}, {NULL}};
const Segment SGI160364[] = {{&S45662[38], 7}, {NULL}};
const Segment SGI160365[] = {{&S45662[31], 6}, {NULL}};
const Segment SGI160366[] = {{&S47019[44], 31}, {NULL}};
const Segment SGI160367[] = {{&S47019[30], 6}, {NULL}};
const Segment SGI160368[] = {{&S45663[44], 11}, {NULL}};
const Segment SGI160369[] = {{&S45663[44], 27}, {NULL}};
const Segment SGI160370[] = {{&S45663[30], 6}, {NULL}};
const Segment SGI160371[] = {{&S45663[120], 3}, {NULL}};
const Segment SGI160372[] = {{&S45663[85], 6}, {NULL}};
const Segment SGI160373[] = {{&S47020[44], 39}, {NULL}};
const Segment SGI160374[] = {{&S47020[30], 6}, {NULL}};
const Segment SGI160375[] = {{&S47020[115], 6}, {NULL}};
const Segment SGI160376[] = {{&S45664[44], 7}, {NULL}};
const Segment SGI160377[] = {{&S45664[44], 23}, {NULL}};
const Segment SGI160378[] = {{&S45664[30], 6}, {NULL}};
const Segment SGI160379[] = {{&S45664[116], 3}, {NULL}};
const Segment SGI160380[] = {{&S45664[81], 6}, {NULL}};
const Segment SGI160381[] = {{&S47021[44], 37}, {NULL}};
const Segment SGI160382[] = {{&S47021[30], 6}, {NULL}};
const Segment SGI160383[] = {{&S47021[113], 6}, {NULL}};
const Segment SGI160384[] = {{&S47022[38], 14}, {NULL}};
const Segment SGI160385[] = {{&S47022[31], 6}, {NULL}};
const Segment SGI160386[] = {{&S47023[44], 39}, {NULL}};
const Segment SGI160387[] = {{&S47023[30], 6}, {NULL}};
const Segment SGI160388[] = {{&S47023[115], 6}, {NULL}};
const Segment SGI160389[] = {{&S56054[34], 7}, {NULL}};
const Segment SGI160390[] = {{&S47024[44], 39}, {NULL}};
const Segment SGI160391[] = {{&S47024[30], 6}, {NULL}};
const Segment SGI160392[] = {{&S56054[27], 6}, {NULL}};
const Segment SGI160393[] = {{&S56054[108], 9}, {NULL}};
const Segment SGI160394[] = {{&S47024[115], 6}, {NULL}};
const Segment SGI160395[] = {{&S56054[81], 6}, {NULL}};
const Segment SGI160396[] = {{&S47025[44], 15}, {NULL}};
const Segment SGI160397[] = {{&S47025[44], 33}, {NULL}};
const Segment SGI160398[] = {{&S47025[30], 6}, {NULL}};
const Segment SGI160399[] = {{&S56055[34], 15}, {NULL}};
const Segment SGI160400[] = {{&S47025[126], 3}, {NULL}};
const Segment SGI160401[] = {{&S56055[27], 6}, {NULL}};
const Segment SGI160402[] = {{&S47025[91], 6}, {NULL}};
const Segment SGI160403[] = {{&S56055[118], 9}, {NULL}};
const Segment SGI160404[] = {{&S56055[91], 6}, {NULL}};
const Segment SGI160405[] = {{&S56056[34], 7}, {NULL}};
const Segment SGI160406[] = {{&S56056[27], 6}, {NULL}};
const Segment SGI160407[] = {{&S47027[38], 15}, {NULL}};
const Segment SGI160408[] = {{&S47027[31], 6}, {NULL}};
const Segment SGI160409[] = {{&S56056[108], 9}, {NULL}};
const Segment SGI160410[] = {{&S56056[81], 6}, {NULL}};
const Segment SGI160411[] = {{&S56057[34], 15}, {NULL}};
const Segment SGI160412[] = {{&S56057[27], 6}, {NULL}};
const Segment SGI160413[] = {{&S56057[118], 9}, {NULL}};
const Segment SGI160414[] = {{&S56057[91], 6}, {NULL}};
const Segment SGI160415[] = {{&S47029[44], 40}, {NULL}};
const Segment SGI160416[] = {{&S47029[30], 6}, {NULL}};
const Segment SGI160417[] = {{&S47029[116], 6}, {NULL}};
const Segment SGI160418[] = {{&S47030[44], 15}, {NULL}};
const Segment SGI160419[] = {{&S47030[44], 33}, {NULL}};
const Segment SGI160420[] = {{&S47030[30], 6}, {NULL}};
const Segment SGI160421[] = {{&S47030[94], 6}, {NULL}};
const Segment SGI160422[] = {{&S56063[34], 7}, {NULL}};
const Segment SGI160423[] = {{&S56063[27], 6}, {NULL}};
const Segment SGI160424[] = {{&S45685[34], 7}, {NULL}};
const Segment SGI160425[] = {{&S56063[94], 9}, {NULL}};
const Segment SGI160426[] = {{&S56063[67], 6}, {NULL}};
const Segment SGI160427[] = {{&S45685[27], 6}, {NULL}};
const Segment SGI160428[] = {{&S47033[44], 37}, {NULL}};
const Segment SGI160429[] = {{&S45685[109], 9}, {NULL}};
const Segment SGI160430[] = {{&S45685[82], 6}, {NULL}};
const Segment SGI160431[] = {{&S47033[30], 6}, {NULL}};
const Segment SGI160432[] = {{&S47033[113], 6}, {NULL}};
const Segment SGI160433[] = {{&S56064[34], 7}, {NULL}};
const Segment SGI160434[] = {{&S56064[27], 6}, {NULL}};
const Segment SGI160435[] = {{&S56064[93], 9}, {NULL}};
const Segment SGI160436[] = {{&S56064[66], 6}, {NULL}};
const Segment SGI160437[] = {{&S47039[44], 37}, {NULL}};
const Segment SGI160438[] = {{&S47039[30], 6}, {NULL}};
const Segment SGI160439[] = {{&S47039[113], 6}, {NULL}};
const Segment SGI160440[] = {{&S56071[34], 9}, {NULL}};
const Segment SGI160441[] = {{&S56071[27], 6}, {NULL}};
const Segment SGI160442[] = {{&S56071[96], 9}, {NULL}};
const Segment SGI160443[] = {{&S56071[69], 6}, {NULL}};
const Segment SGI160444[] = {{&S47040[44], 37}, {NULL}};
const Segment SGI160445[] = {{&S47040[30], 6}, {NULL}};
const Segment SGI160446[] = {{&S47040[113], 6}, {NULL}};
const Segment SGI160447[] = {{&S56072[34], 12}, {NULL}};
const Segment SGI160448[] = {{&S56072[27], 6}, {NULL}};
const Segment SGI160449[] = {{&S56072[99], 9}, {NULL}};
const Segment SGI160450[] = {{&S56072[72], 6}, {NULL}};
const Segment SGI160451[] = {{&S56082[34], 10}, {NULL}};
const Segment SGI160452[] = {{&S56082[27], 6}, {NULL}};
const Segment SGI160453[] = {{&S56082[97], 9}, {NULL}};
const Segment SGI160454[] = {{&S56082[70], 6}, {NULL}};
const Segment SGI160455[] = {{&S56083[34], 11}, {NULL}};
const Segment SGI160456[] = {{&S56083[27], 6}, {NULL}};
const Segment SGI160457[] = {{&S56083[98], 9}, {NULL}};
const Segment SGI160458[] = {{&S56083[71], 6}, {NULL}};
const Segment SGI160459[] = {{&S56084[34], 11}, {NULL}};
const Segment SGI160460[] = {{&S56084[27], 6}, {NULL}};
const Segment SGI160461[] = {{&S56084[98], 9}, {NULL}};
const Segment SGI160462[] = {{&S56084[71], 6}, {NULL}};
const Segment SGI160463[] = {{&S56097[44], 8}, {NULL}};
const Segment SGI160464[] = {{&S56097[44], 20}, {NULL}};
const Segment SGI160465[] = {{&S56097[30], 6}, {NULL}};
const Segment SGI160466[] = {{&S56097[31], 3}, {NULL}};
const Segment SGI160467[] = {{&S56097[78], 6}, {NULL}};
const Segment SGI160468[] = {{&S45727[34], 9}, {NULL}};
const Segment SGI160469[] = {{&S45727[27], 6}, {NULL}};
const Segment SGI160470[] = {{&S45727[96], 9}, {NULL}};
const Segment SGI160471[] = {{&S45727[69], 6}, {NULL}};
const Segment SGI160472[] = {{&S56098[38], 8}, {NULL}};
const Segment SGI160473[] = {{&S56098[31], 6}, {NULL}};
const Segment SGI160474[] = {{&S45728[34], 10}, {NULL}};
const Segment SGI160475[] = {{&S45728[27], 6}, {NULL}};
const Segment SGI160476[] = {{&S45728[97], 9}, {NULL}};
const Segment SGI160477[] = {{&S45728[70], 6}, {NULL}};
const Segment SGI160478[] = {{&S56107[44], 10}, {NULL}};
const Segment SGI160479[] = {{&S56107[44], 22}, {NULL}};
const Segment SGI160480[] = {{&S56107[30], 6}, {NULL}};
const Segment SGI160481[] = {{&S56107[31], 3}, {NULL}};
const Segment SGI160482[] = {{&S56107[95], 6}, {NULL}};
const Segment SGI160483[] = {{&S56107[68], 8}, {NULL}};
const Segment SGI160484[] = {{&S56108[44], 10}, {NULL}};
const Segment SGI160485[] = {{&S56108[44], 22}, {NULL}};
const Segment SGI160486[] = {{&S56108[30], 6}, {NULL}};
const Segment SGI160487[] = {{&S56108[31], 3}, {NULL}};
const Segment SGI160488[] = {{&S56108[95], 6}, {NULL}};
const Segment SGI160489[] = {{&S56108[68], 8}, {NULL}};
const Segment SGI160490[] = {{&S56109[44], 10}, {NULL}};
const Segment SGI160491[] = {{&S56109[44], 22}, {NULL}};
const Segment SGI160492[] = {{&S56109[30], 6}, {NULL}};
const Segment SGI160493[] = {{&S56109[31], 3}, {NULL}};
const Segment SGI160494[] = {{&S56109[99], 6}, {NULL}};
const Segment SGI160495[] = {{&S56109[68], 8}, {NULL}};
const Segment SGI160496[] = {{&S56111[34], 13}, {NULL}};
const Segment SGI160497[] = {{&S56111[27], 6}, {NULL}};
const Segment SGI160498[] = {{&S56111[100], 9}, {NULL}};
const Segment SGI160499[] = {{&S56111[73], 6}, {NULL}};
const Segment SGI160500[] = {{&S56119[34], 15}, {NULL}};
const Segment SGI160501[] = {{&S56119[27], 6}, {NULL}};
const Segment SGI160502[] = {{&S56119[102], 9}, {NULL}};
const Segment SGI160503[] = {{&S56119[75], 6}, {NULL}};
const Segment SGI160504[] = {{&S56123[44], 6}, {NULL}};
const Segment SGI160505[] = {{&S56123[44], 18}, {NULL}};
const Segment SGI160506[] = {{&S56123[30], 6}, {NULL}};
const Segment SGI160507[] = {{&S56123[31], 3}, {NULL}};
const Segment SGI160508[] = {{&S56123[76], 6}, {NULL}};
const Segment SGI160509[] = {{&S42039[34], 9}, {NULL}};
const Segment SGI160510[] = {{&S42039[27], 6}, {NULL}};
const Segment SGI160511[] = {{&S42039[96], 9}, {NULL}};
const Segment SGI160512[] = {{&S42039[69], 6}, {NULL}};
const Segment SGI160513[] = {{&S56137[34], 7}, {NULL}};
const Segment SGI160514[] = {{&S56137[27], 6}, {NULL}};
const Segment SGI160515[] = {{&S56137[100], 9}, {NULL}};
const Segment SGI160516[] = {{&S56137[73], 6}, {NULL}};
const Segment SGI160517[] = {{&S45784[34], 10}, {NULL}};
const Segment SGI160518[] = {{&S45784[27], 6}, {NULL}};
const Segment SGI160519[] = {{&S45784[97], 9}, {NULL}};
const Segment SGI160520[] = {{&S45784[70], 6}, {NULL}};
const Segment SGI160521[] = {{&S45786[34], 10}, {NULL}};
const Segment SGI160522[] = {{&S45786[27], 6}, {NULL}};
const Segment SGI160523[] = {{&S45786[97], 9}, {NULL}};
const Segment SGI160524[] = {{&S45786[70], 6}, {NULL}};
const Segment SGI160525[] = {{&S45787[34], 10}, {NULL}};
const Segment SGI160526[] = {{&S45787[27], 6}, {NULL}};
const Segment SGI160527[] = {{&S45787[97], 9}, {NULL}};
const Segment SGI160528[] = {{&S45787[70], 6}, {NULL}};
const Segment SGI160529[] = {{&S47145[34], 10}, {NULL}};
const Segment SGI160530[] = {{&S47145[27], 6}, {NULL}};
const Segment SGI160531[] = {{&S47145[97], 9}, {NULL}};
const Segment SGI160532[] = {{&S47145[70], 6}, {NULL}};
const Segment SGI160533[] = {{&S47147[34], 9}, {NULL}};
const Segment SGI160534[] = {{&S47147[27], 6}, {NULL}};
const Segment SGI160535[] = {{&S47147[96], 9}, {NULL}};
const Segment SGI160536[] = {{&S47147[69], 6}, {NULL}};
const Segment SGI160537[] = {{&S47148[34], 7}, {NULL}};
const Segment SGI160538[] = {{&S47148[27], 6}, {NULL}};
const Segment SGI160539[] = {{&S47148[94], 9}, {NULL}};
const Segment SGI160540[] = {{&S47148[67], 6}, {NULL}};
const Segment SGI160541[] = {{&S47150[34], 7}, {NULL}};
const Segment SGI160542[] = {{&S47150[27], 6}, {NULL}};
const Segment SGI160543[] = {{&S47150[110], 9}, {NULL}};
const Segment SGI160544[] = {{&S47150[83], 6}, {NULL}};
const Segment SGI160545[] = {{&S56176[34], 11}, {NULL}};
const Segment SGI160546[] = {{&S56176[27], 6}, {NULL}};
const Segment SGI160547[] = {{&S56176[98], 9}, {NULL}};
const Segment SGI160548[] = {{&S56176[71], 6}, {NULL}};
const Segment SGI160549[] = {{&S45838[34], 11}, {NULL}};
const Segment SGI160550[] = {{&S45838[27], 6}, {NULL}};
const Segment SGI160551[] = {{&S45838[98], 9}, {NULL}};
const Segment SGI160552[] = {{&S45838[71], 6}, {NULL}};
const Segment SGI160553[] = {{&S56214[34], 11}, {NULL}};
const Segment SGI160554[] = {{&S56214[27], 6}, {NULL}};
const Segment SGI160555[] = {{&S56214[98], 9}, {NULL}};
const Segment SGI160556[] = {{&S56214[71], 6}, {NULL}};
const Segment SGI160557[] = {{&S56244[34], 13}, {NULL}};
const Segment SGI160558[] = {{&S56244[27], 6}, {NULL}};
const Segment SGI160559[] = {{&S56244[100], 9}, {NULL}};
const Segment SGI160560[] = {{&S56244[73], 6}, {NULL}};
const Segment SGI160561[] = {{&S56264[34], 7}, {NULL}};
const Segment SGI160562[] = {{&S56264[27], 6}, {NULL}};
const Segment SGI160563[] = {{&S56264[94], 9}, {NULL}};
const Segment SGI160564[] = {{&S56264[67], 6}, {NULL}};
const Segment SGI160565[] = {{&S47245[44], 10}, {NULL}};
const Segment SGI160566[] = {{&S47245[44], 22}, {NULL}};
const Segment SGI160567[] = {{&S47245[30], 6}, {NULL}};
const Segment SGI160568[] = {{&S47245[115], 3}, {NULL}};
const Segment SGI160569[] = {{&S47245[80], 6}, {NULL}};
const Segment SGI160570[] = {{&S56266[34], 13}, {NULL}};
const Segment SGI160571[] = {{&S56266[27], 6}, {NULL}};
const Segment SGI160572[] = {{&S56266[100], 9}, {NULL}};
const Segment SGI160573[] = {{&S56266[73], 6}, {NULL}};
const Segment SGI160574[] = {{&S56267[34], 10}, {NULL}};
const Segment SGI160575[] = {{&S56267[27], 6}, {NULL}};
const Segment SGI160576[] = {{&S56267[97], 9}, {NULL}};
const Segment SGI160577[] = {{&S56267[70], 6}, {NULL}};
const Segment SGI160578[] = {{&S47248[44], 10}, {NULL}};
const Segment SGI160579[] = {{&S47248[44], 22}, {NULL}};
const Segment SGI160580[] = {{&S47248[30], 6}, {NULL}};
const Segment SGI160581[] = {{&S47248[115], 3}, {NULL}};
const Segment SGI160582[] = {{&S47248[80], 6}, {NULL}};
const Segment SGI160583[] = {{&S47250[34], 10}, {NULL}};
const Segment SGI160584[] = {{&S47250[27], 6}, {NULL}};
const Segment SGI160585[] = {{&S47250[97], 9}, {NULL}};
const Segment SGI160586[] = {{&S56271[34], 10}, {NULL}};
const Segment SGI160587[] = {{&S47250[70], 6}, {NULL}};
const Segment SGI160588[] = {{&S56271[27], 6}, {NULL}};
const Segment SGI160589[] = {{&S56271[97], 9}, {NULL}};
const Segment SGI160590[] = {{&S45910[44], 18}, {NULL}};
const Segment SGI160591[] = {{&S56271[70], 6}, {NULL}};
const Segment SGI160592[] = {{&S45910[44], 30}, {NULL}};
const Segment SGI160593[] = {{&S45910[30], 6}, {NULL}};
const Segment SGI160594[] = {{&S56271[138], 8}, {NULL}};
const Segment SGI160595[] = {{&S45910[143], 3}, {NULL}};
const Segment SGI160596[] = {{&S45910[108], 6}, {NULL}};
const Segment SGI160597[] = {{&S45910[76], 8}, {NULL}};
const Segment SGI160598[] = {{&S47252[34], 10}, {NULL}};
const Segment SGI160599[] = {{&S47252[27], 6}, {NULL}};
const Segment SGI160600[] = {{&S56272[34], 11}, {NULL}};
const Segment SGI160601[] = {{&S56272[27], 6}, {NULL}};
const Segment SGI160602[] = {{&S47252[97], 9}, {NULL}};
const Segment SGI160603[] = {{&S47252[70], 6}, {NULL}};
const Segment SGI160604[] = {{&S56272[98], 9}, {NULL}};
const Segment SGI160605[] = {{&S45911[44], 10}, {NULL}};
const Segment SGI160606[] = {{&S56272[71], 6}, {NULL}};
const Segment SGI160607[] = {{&S45911[44], 22}, {NULL}};
const Segment SGI160608[] = {{&S45911[30], 6}, {NULL}};
const Segment SGI160609[] = {{&S56272[139], 8}, {NULL}};
const Segment SGI160610[] = {{&S45911[135], 3}, {NULL}};
const Segment SGI160611[] = {{&S45911[100], 6}, {NULL}};
const Segment SGI160612[] = {{&S45911[68], 8}, {NULL}};
const Segment SGI160613[] = {{&S45912[44], 11}, {NULL}};
const Segment SGI160614[] = {{&S45912[44], 23}, {NULL}};
const Segment SGI160615[] = {{&S45912[30], 6}, {NULL}};
const Segment SGI160616[] = {{&S45912[136], 3}, {NULL}};
const Segment SGI160617[] = {{&S45912[101], 6}, {NULL}};
const Segment SGI160618[] = {{&S45912[69], 8}, {NULL}};
const Segment SGI160619[] = {{&S56276[44], 10}, {NULL}};
const Segment SGI160620[] = {{&S56276[44], 22}, {NULL}};
const Segment SGI160621[] = {{&S56276[30], 6}, {NULL}};
const Segment SGI160622[] = {{&S56276[31], 3}, {NULL}};
const Segment SGI160623[] = {{&S56276[80], 6}, {NULL}};
const Segment SGI160624[] = {{&S45914[34], 10}, {NULL}};
const Segment SGI160625[] = {{&S45914[27], 6}, {NULL}};
const Segment SGI160626[] = {{&S45914[97], 9}, {NULL}};
const Segment SGI160627[] = {{&S45914[70], 6}, {NULL}};
const Segment SGI160628[] = {{&S56277[38], 10}, {NULL}};
const Segment SGI160629[] = {{&S56277[31], 6}, {NULL}};
const Segment SGI160630[] = {{&S56278[44], 10}, {NULL}};
const Segment SGI160631[] = {{&S56278[44], 22}, {NULL}};
const Segment SGI160632[] = {{&S56278[30], 6}, {NULL}};
const Segment SGI160633[] = {{&S56278[83], 6}, {NULL}};
const Segment SGI160634[] = {{&S51229[32], 14}, {NULL}};
const Segment SGI160635[] = {{&S51229[27], 4}, {NULL}};
const Segment SGI160636[] = {{&S51229[99], 9}, {NULL}};
const Segment SGI160637[] = {{&S51229[72], 6}, {NULL}};
const Segment SGI160638[] = {{&S47300[34], 29}, {NULL}};
const Segment SGI160639[] = {{&S47300[27], 6}, {NULL}};
const Segment SGI160640[] = {{&S47300[116], 9}, {NULL}};
const Segment SGI160641[] = {{&S47300[89], 6}, {NULL}};
const Segment SGI160642[] = {{&S56310[34], 10}, {NULL}};
const Segment SGI160643[] = {{&S56310[27], 6}, {NULL}};
const Segment SGI160644[] = {{&S56310[97], 9}, {NULL}};
const Segment SGI160645[] = {{&S56310[70], 6}, {NULL}};
const Segment SGI160646[] = {{&S47301[34], 30}, {NULL}};
const Segment SGI160647[] = {{&S47301[27], 6}, {NULL}};
const Segment SGI160648[] = {{&S47301[117], 9}, {NULL}};
const Segment SGI160649[] = {{&S47301[90], 6}, {NULL}};
const Segment SGI160650[] = {{&S56311[34], 11}, {NULL}};
const Segment SGI160651[] = {{&S56311[27], 6}, {NULL}};
const Segment SGI160652[] = {{&S56311[98], 9}, {NULL}};
const Segment SGI160653[] = {{&S56311[71], 6}, {NULL}};
const Segment SGI160654[] = {{&S47302[34], 30}, {NULL}};
const Segment SGI160655[] = {{&S47302[27], 6}, {NULL}};
const Segment SGI160656[] = {{&S47302[117], 9}, {NULL}};
const Segment SGI160657[] = {{&S47302[90], 6}, {NULL}};
const Segment SGI160658[] = {{&S56313[27], 6}, {NULL}};
const Segment SGI160659[] = {{&S56313[99], 9}, {NULL}};
const Segment SGI160660[] = {{&S56313[72], 6}, {NULL}};
const Segment SGI160661[] = {{&S56314[34], 13}, {NULL}};
const Segment SGI160662[] = {{&S56314[27], 6}, {NULL}};
const Segment SGI160663[] = {{&S56314[100], 9}, {NULL}};
const Segment SGI160664[] = {{&S56314[73], 6}, {NULL}};
const Segment SGI160665[] = {{&S56315[34], 13}, {NULL}};
const Segment SGI160666[] = {{&S56315[27], 6}, {NULL}};
const Segment SGI160667[] = {{&S56315[100], 9}, {NULL}};
const Segment SGI160668[] = {{&S56315[73], 6}, {NULL}};
const Segment SGI160669[] = {{&S45955[34], 10}, {NULL}};
const Segment SGI160670[] = {{&S45955[27], 6}, {NULL}};
const Segment SGI160671[] = {{&S56316[34], 9}, {NULL}};
const Segment SGI160672[] = {{&S56316[27], 6}, {NULL}};
const Segment SGI160673[] = {{&S45955[97], 9}, {NULL}};
const Segment SGI160674[] = {{&S45955[70], 6}, {NULL}};
const Segment SGI160675[] = {{&S56316[96], 9}, {NULL}};
const Segment SGI160676[] = {{&S56316[69], 6}, {NULL}};
const Segment SGI160677[] = {{&S56321[34], 7}, {NULL}};
const Segment SGI160678[] = {{&S56321[27], 6}, {NULL}};
const Segment SGI160679[] = {{&S56321[100], 9}, {NULL}};
const Segment SGI160680[] = {{&S56321[73], 6}, {NULL}};
const Segment SGI160681[] = {{&S42131[34], 10}, {NULL}};
const Segment SGI160682[] = {{&S42131[27], 6}, {NULL}};
const Segment SGI160683[] = {{&S42131[97], 9}, {NULL}};
const Segment SGI160684[] = {{&S42131[70], 6}, {NULL}};
const Segment SGI160685[] = {{&S32448[44], 10}, {NULL}};
const Segment SGI160686[] = {{&S32448[44], 21}, {NULL}};
const Segment SGI160687[] = {{&S32448[30], 6}, {NULL}};
const Segment SGI160688[] = {{&S32448[132], 3}, {NULL}};
const Segment SGI160689[] = {{&S32448[97], 6}, {NULL}};
const Segment SGI160690[] = {{&S32448[67], 8}, {NULL}};
const Segment SGI160691[] = {{&S32449[44], 10}, {NULL}};
const Segment SGI160692[] = {{&S32449[44], 21}, {NULL}};
const Segment SGI160693[] = {{&S32449[30], 6}, {NULL}};
const Segment SGI160694[] = {{&S32449[132], 3}, {NULL}};
const Segment SGI160695[] = {{&S47362[34], 10}, {NULL}};
const Segment SGI160696[] = {{&S32449[97], 6}, {NULL}};
const Segment SGI160697[] = {{&S47362[27], 6}, {NULL}};
const Segment SGI160698[] = {{&S32449[67], 8}, {NULL}};
const Segment SGI160699[] = {{&S47362[97], 9}, {NULL}};
const Segment SGI160700[] = {{&S47362[70], 6}, {NULL}};
const Segment SGI160701[] = {{&S47362[138], 8}, {NULL}};
const Segment SGI160702[] = {{&S32450[44], 10}, {NULL}};
const Segment SGI160703[] = {{&S32450[44], 21}, {NULL}};
const Segment SGI160704[] = {{&S32450[30], 6}, {NULL}};
const Segment SGI160705[] = {{&S32450[132], 3}, {NULL}};
const Segment SGI160706[] = {{&S32450[97], 6}, {NULL}};
const Segment SGI160707[] = {{&S32450[67], 8}, {NULL}};
const Segment SGI160708[] = {{&S32451[44], 10}, {NULL}};
const Segment SGI160709[] = {{&S32451[44], 22}, {NULL}};
const Segment SGI160710[] = {{&S32451[30], 6}, {NULL}};
const Segment SGI160711[] = {{&S32451[133], 3}, {NULL}};
const Segment SGI160712[] = {{&S32451[98], 6}, {NULL}};
const Segment SGI160713[] = {{&S32451[68], 8}, {NULL}};
const Segment SGI160714[] = {{&S47366[34], 10}, {NULL}};
const Segment SGI160715[] = {{&S47366[27], 6}, {NULL}};
const Segment SGI160716[] = {{&S32452[44], 10}, {NULL}};
const Segment SGI160717[] = {{&S47366[97], 9}, {NULL}};
const Segment SGI160718[] = {{&S32452[44], 20}, {NULL}};
const Segment SGI160719[] = {{&S47366[70], 6}, {NULL}};
const Segment SGI160720[] = {{&S32452[30], 6}, {NULL}};
const Segment SGI160721[] = {{&S47366[138], 8}, {NULL}};
const Segment SGI160722[] = {{&S32452[131], 3}, {NULL}};
const Segment SGI160723[] = {{&S32452[96], 6}, {NULL}};
const Segment SGI160724[] = {{&S32452[66], 8}, {NULL}};
const Segment SGI160725[] = {{&S32453[44], 10}, {NULL}};
const Segment SGI160726[] = {{&S32453[44], 20}, {NULL}};
const Segment SGI160727[] = {{&S32453[30], 6}, {NULL}};
const Segment SGI160728[] = {{&S32453[131], 3}, {NULL}};
const Segment SGI160729[] = {{&S32453[96], 6}, {NULL}};
const Segment SGI160730[] = {{&S32453[66], 8}, {NULL}};
const Segment SGI160731[] = {{&S32454[44], 10}, {NULL}};
const Segment SGI160732[] = {{&S32454[44], 20}, {NULL}};
const Segment SGI160733[] = {{&S32454[30], 6}, {NULL}};
const Segment SGI160734[] = {{&S32454[131], 3}, {NULL}};
const Segment SGI160735[] = {{&S32454[96], 6}, {NULL}};
const Segment SGI160736[] = {{&S32454[66], 8}, {NULL}};
const Segment SGI160737[] = {{&S32455[44], 10}, {NULL}};
const Segment SGI160738[] = {{&S32455[44], 22}, {NULL}};
const Segment SGI160739[] = {{&S32455[30], 6}, {NULL}};
const Segment SGI160740[] = {{&S32455[115], 3}, {NULL}};
const Segment SGI160741[] = {{&S32455[80], 6}, {NULL}};
const Segment SGI160742[] = {{&S32455[140], 8}, {NULL}};
const Segment SGI160743[] = {{&S47374[34], 7}, {NULL}};
const Segment SGI160744[] = {{&S47374[27], 6}, {NULL}};
const Segment SGI160745[] = {{&S32456[44], 10}, {NULL}};
const Segment SGI160746[] = {{&S32456[44], 20}, {NULL}};
const Segment SGI160747[] = {{&S32456[30], 6}, {NULL}};
const Segment SGI160748[] = {{&S47374[99], 9}, {NULL}};
const Segment SGI160749[] = {{&S47374[72], 6}, {NULL}};
const Segment SGI160750[] = {{&S32456[113], 3}, {NULL}};
const Segment SGI160751[] = {{&S42161[34], 10}, {NULL}};
const Segment SGI160752[] = {{&S32456[78], 6}, {NULL}};
const Segment SGI160753[] = {{&S42161[27], 6}, {NULL}};
const Segment SGI160754[] = {{&S32456[138], 8}, {NULL}};
const Segment SGI160755[] = {{&S42161[97], 9}, {NULL}};
const Segment SGI160756[] = {{&S42161[70], 6}, {NULL}};
const Segment SGI160757[] = {{&S47375[34], 11}, {NULL}};
const Segment SGI160758[] = {{&S47375[27], 6}, {NULL}};
const Segment SGI160759[] = {{&S32457[44], 10}, {NULL}};
const Segment SGI160760[] = {{&S32457[44], 22}, {NULL}};
const Segment SGI160761[] = {{&S47375[98], 9}, {NULL}};
const Segment SGI160762[] = {{&S47375[71], 6}, {NULL}};
const Segment SGI160763[] = {{&S32457[30], 6}, {NULL}};
const Segment SGI160764[] = {{&S32457[115], 3}, {NULL}};
const Segment SGI160765[] = {{&S32457[80], 6}, {NULL}};
const Segment SGI160766[] = {{&S32457[140], 8}, {NULL}};
const Segment SGI160767[] = {{&S47376[34], 11}, {NULL}};
const Segment SGI160768[] = {{&S47376[27], 6}, {NULL}};
const Segment SGI160769[] = {{&S47376[98], 9}, {NULL}};
const Segment SGI160770[] = {{&S47376[71], 6}, {NULL}};
const Segment SGI160771[] = {{&S47377[34], 27}, {NULL}};
const Segment SGI160772[] = {{&S47377[27], 6}, {NULL}};
const Segment SGI160773[] = {{&S47377[114], 9}, {NULL}};
const Segment SGI160774[] = {{&S47377[87], 6}, {NULL}};
const Segment SGI160775[] = {{&S47379[34], 11}, {NULL}};
const Segment SGI160776[] = {{&S47379[27], 6}, {NULL}};
const Segment SGI160777[] = {{&S47379[98], 9}, {NULL}};
const Segment SGI160778[] = {{&S47379[71], 6}, {NULL}};
const Segment SGI160779[] = {{&S47380[34], 11}, {NULL}};
const Segment SGI160780[] = {{&S47380[27], 6}, {NULL}};
const Segment SGI160781[] = {{&S47380[98], 9}, {NULL}};
const Segment SGI160782[] = {{&S47380[71], 6}, {NULL}};
const Segment SGI160783[] = {{&S46051[44], 9}, {NULL}};
const Segment SGI160784[] = {{&S46051[44], 21}, {NULL}};
const Segment SGI160785[] = {{&S46051[30], 6}, {NULL}};
const Segment SGI160786[] = {{&S46051[114], 3}, {NULL}};
const Segment SGI160787[] = {{&S46051[79], 6}, {NULL}};
const Segment SGI160788[] = {{&S46052[38], 9}, {NULL}};
const Segment SGI160789[] = {{&S46052[31], 6}, {NULL}};
const Segment SGI160790[] = {{&S47395[34], 10}, {NULL}};
const Segment SGI160791[] = {{&S47395[27], 6}, {NULL}};
const Segment SGI160792[] = {{&S47395[97], 9}, {NULL}};
const Segment SGI160793[] = {{&S46053[44], 25}, {NULL}};
const Segment SGI160794[] = {{&S47395[70], 6}, {NULL}};
const Segment SGI160795[] = {{&S46053[44], 37}, {NULL}};
const Segment SGI160796[] = {{&S46053[30], 6}, {NULL}};
const Segment SGI160797[] = {{&S46053[130], 3}, {NULL}};
const Segment SGI160798[] = {{&S46053[95], 6}, {NULL}};
const Segment SGI160799[] = {{&S46054[44], 14}, {NULL}};
const Segment SGI160800[] = {{&S46054[44], 26}, {NULL}};
const Segment SGI160801[] = {{&S46054[30], 6}, {NULL}};
const Segment SGI160802[] = {{&S46054[119], 3}, {NULL}};
const Segment SGI160803[] = {{&S46054[84], 6}, {NULL}};
const Segment SGI160804[] = {{&S47398[34], 10}, {NULL}};
const Segment SGI160805[] = {{&S47398[27], 6}, {NULL}};
const Segment SGI160806[] = {{&S47398[97], 9}, {NULL}};
const Segment SGI160807[] = {{&S47398[70], 6}, {NULL}};
const Segment SGI160808[] = {{&S46055[44], 25}, {NULL}};
const Segment SGI160809[] = {{&S46055[44], 37}, {NULL}};
const Segment SGI160810[] = {{&S46055[30], 6}, {NULL}};
const Segment SGI160811[] = {{&S46055[130], 3}, {NULL}};
const Segment SGI160812[] = {{&S46055[95], 6}, {NULL}};
const Segment SGI160813[] = {{&S46069[44], 15}, {NULL}};
const Segment SGI160814[] = {{&S46069[44], 27}, {NULL}};
const Segment SGI160815[] = {{&S46069[30], 6}, {NULL}};
const Segment SGI160816[] = {{&S46069[136], 3}, {NULL}};
const Segment SGI160817[] = {{&S46069[101], 6}, {NULL}};
const Segment SGI160818[] = {{&S46069[73], 8}, {NULL}};
const Segment SGI160819[] = {{&S46070[44], 8}, {NULL}};
const Segment SGI160820[] = {{&S46070[44], 20}, {NULL}};
const Segment SGI160821[] = {{&S46070[30], 6}, {NULL}};
const Segment SGI160822[] = {{&S46070[129], 3}, {NULL}};
const Segment SGI160823[] = {{&S46070[94], 6}, {NULL}};
const Segment SGI160824[] = {{&S46070[66], 8}, {NULL}};
const Segment SGI160825[] = {{&S46071[44], 16}, {NULL}};
const Segment SGI160826[] = {{&S46071[44], 28}, {NULL}};
const Segment SGI160827[] = {{&S46071[30], 6}, {NULL}};
const Segment SGI160828[] = {{&S46071[137], 3}, {NULL}};
const Segment SGI160829[] = {{&S46071[102], 6}, {NULL}};
const Segment SGI160830[] = {{&S46071[74], 8}, {NULL}};
const Segment SGI160831[] = {{&S46072[44], 15}, {NULL}};
const Segment SGI160832[] = {{&S46072[44], 27}, {NULL}};
const Segment SGI160833[] = {{&S46072[30], 6}, {NULL}};
const Segment SGI160834[] = {{&S46072[140], 3}, {NULL}};
const Segment SGI160835[] = {{&S46072[105], 6}, {NULL}};
const Segment SGI160836[] = {{&S46072[73], 8}, {NULL}};
const Segment SGI160837[] = {{&S47448[34], 10}, {NULL}};
const Segment SGI160838[] = {{&S47448[27], 6}, {NULL}};
const Segment SGI160839[] = {{&S47448[97], 9}, {NULL}};
const Segment SGI160840[] = {{&S47448[70], 6}, {NULL}};
const Segment SGI160841[] = {{&S47449[34], 11}, {NULL}};
const Segment SGI160842[] = {{&S47449[27], 6}, {NULL}};
const Segment SGI160843[] = {{&S47449[98], 9}, {NULL}};
const Segment SGI160844[] = {{&S47449[71], 6}, {NULL}};
const Segment SGI160845[] = {{&S52362[44], 5}, {NULL}};
const Segment SGI160846[] = {{&S52362[44], 17}, {NULL}};
const Segment SGI160847[] = {{&S52362[30], 6}, {NULL}};
const Segment SGI160848[] = {{&S52362[31], 3}, {NULL}};
const Segment SGI160849[] = {{&S52362[75], 6}, {NULL}};
const Segment SGI160850[] = {{&S46136[44], 11}, {NULL}};
const Segment SGI160851[] = {{&S46136[44], 23}, {NULL}};
const Segment SGI160852[] = {{&S46136[30], 6}, {NULL}};
const Segment SGI160853[] = {{&S46136[116], 3}, {NULL}};
const Segment SGI160854[] = {{&S46136[81], 6}, {NULL}};
const Segment SGI160855[] = {{&S46137[44], 7}, {NULL}};
const Segment SGI160856[] = {{&S46137[44], 19}, {NULL}};
const Segment SGI160857[] = {{&S46137[30], 6}, {NULL}};
const Segment SGI160858[] = {{&S46137[112], 3}, {NULL}};
const Segment SGI160859[] = {{&S46137[77], 6}, {NULL}};
const Segment SGI160860[] = {{&S46138[38], 7}, {NULL}};
const Segment SGI160861[] = {{&S46138[31], 6}, {NULL}};
const Segment SGI160862[] = {{&S47491[44], 10}, {NULL}};
const Segment SGI160863[] = {{&S47491[44], 22}, {NULL}};
const Segment SGI160864[] = {{&S47491[30], 6}, {NULL}};
const Segment SGI160865[] = {{&S47491[115], 3}, {NULL}};
const Segment SGI160866[] = {{&S47491[80], 6}, {NULL}};
const Segment SGI160867[] = {{&S47492[44], 10}, {NULL}};
const Segment SGI160868[] = {{&S47492[44], 22}, {NULL}};
const Segment SGI160869[] = {{&S47492[30], 6}, {NULL}};
const Segment SGI160870[] = {{&S47492[115], 3}, {NULL}};
const Segment SGI160871[] = {{&S47492[80], 6}, {NULL}};
const Segment SGI160872[] = {{&S47493[44], 10}, {NULL}};
const Segment SGI160873[] = {{&S47493[44], 22}, {NULL}};
const Segment SGI160874[] = {{&S47493[30], 6}, {NULL}};
const Segment SGI160875[] = {{&S47493[115], 3}, {NULL}};
const Segment SGI160876[] = {{&S47493[80], 6}, {NULL}};
const Segment SGI160877[] = {{&S47494[38], 10}, {NULL}};
const Segment SGI160878[] = {{&S47494[31], 6}, {NULL}};
const Segment SGI160879[] = {{&S47495[44], 10}, {NULL}};
const Segment SGI160880[] = {{&S47495[44], 22}, {NULL}};
const Segment SGI160881[] = {{&S47495[30], 6}, {NULL}};
const Segment SGI160882[] = {{&S47495[115], 3}, {NULL}};
const Segment SGI160883[] = {{&S47495[80], 6}, {NULL}};
const Segment SGI160884[] = {{&S47497[44], 10}, {NULL}};
const Segment SGI160885[] = {{&S47497[44], 25}, {NULL}};
const Segment SGI160886[] = {{&S47497[30], 6}, {NULL}};
const Segment SGI160887[] = {{&S47497[118], 3}, {NULL}};
const Segment SGI160888[] = {{&S47497[83], 6}, {NULL}};
const Segment SGI160889[] = {{&S47498[44], 10}, {NULL}};
const Segment SGI160890[] = {{&S47498[44], 22}, {NULL}};
const Segment SGI160891[] = {{&S47498[30], 6}, {NULL}};
const Segment SGI160892[] = {{&S47498[115], 3}, {NULL}};
const Segment SGI160893[] = {{&S47498[80], 6}, {NULL}};
const Segment SGI160894[] = {{&S47501[44], 10}, {NULL}};
const Segment SGI160895[] = {{&S47501[44], 22}, {NULL}};
const Segment SGI160896[] = {{&S47501[30], 6}, {NULL}};
const Segment SGI160897[] = {{&S47501[115], 3}, {NULL}};
const Segment SGI160898[] = {{&S47501[80], 6}, {NULL}};
const Segment SGI160899[] = {{&S47502[44], 10}, {NULL}};
const Segment SGI160900[] = {{&S47502[44], 37}, {NULL}};
const Segment SGI160901[] = {{&S47502[30], 6}, {NULL}};
const Segment SGI160902[] = {{&S47502[130], 3}, {NULL}};
const Segment SGI160903[] = {{&S47502[95], 6}, {NULL}};
const Segment SGI160904[] = {{&S47503[44], 10}, {NULL}};
const Segment SGI160905[] = {{&S47503[44], 22}, {NULL}};
const Segment SGI160906[] = {{&S47503[30], 6}, {NULL}};
const Segment SGI160907[] = {{&S47503[83], 6}, {NULL}};
const Segment SGI160908[] = {{&S46195[34], 10}, {NULL}};
const Segment SGI160909[] = {{&S46195[27], 6}, {NULL}};
const Segment SGI160910[] = {{&S46195[97], 9}, {NULL}};
const Segment SGI160911[] = {{&S46195[70], 6}, {NULL}};
const Segment SGI160912[] = {{&S46200[44], 10}, {NULL}};
const Segment SGI160913[] = {{&S46200[44], 22}, {NULL}};
const Segment SGI160914[] = {{&S46200[30], 6}, {NULL}};
const Segment SGI160915[] = {{&S46200[115], 3}, {NULL}};
const Segment SGI160916[] = {{&S46200[80], 6}, {NULL}};
const Segment SGI160917[] = {{&S46201[38], 10}, {NULL}};
const Segment SGI160918[] = {{&S46201[31], 6}, {NULL}};
const Segment SGI160919[] = {{&S46202[44], 11}, {NULL}};
const Segment SGI160920[] = {{&S46202[44], 23}, {NULL}};
const Segment SGI160921[] = {{&S46202[30], 6}, {NULL}};
const Segment SGI160922[] = {{&S46202[116], 3}, {NULL}};
const Segment SGI160923[] = {{&S46202[81], 6}, {NULL}};
const Segment SGI160924[] = {{&S46203[42], 10}, {NULL}};
const Segment SGI160925[] = {{&S46203[30], 6}, {NULL}};
const Segment SGI160926[] = {{&S46203[113], 3}, {NULL}};
const Segment SGI160927[] = {{&S46203[78], 6}, {NULL}};
const Segment SGI160928[] = {{&S46204[38], 11}, {NULL}};
const Segment SGI160929[] = {{&S46204[31], 6}, {NULL}};
const Segment SGI160930[] = {{&S46205[34], 10}, {NULL}};
const Segment SGI160931[] = {{&S46205[27], 6}, {NULL}};
const Segment SGI160932[] = {{&S46205[97], 9}, {NULL}};
const Segment SGI160933[] = {{&S46205[70], 6}, {NULL}};
const Segment SGI160934[] = {{&S46206[34], 18}, {NULL}};
const Segment SGI160935[] = {{&S46206[27], 6}, {NULL}};
const Segment SGI160936[] = {{&S46206[105], 9}, {NULL}};
const Segment SGI160937[] = {{&S46206[78], 6}, {NULL}};
const Segment SGI160938[] = {{&S46207[34], 10}, {NULL}};
const Segment SGI160939[] = {{&S46207[27], 6}, {NULL}};
const Segment SGI160940[] = {{&S46207[97], 9}, {NULL}};
const Segment SGI160941[] = {{&S46207[70], 6}, {NULL}};
const Segment SGI160942[] = {{&S46208[34], 11}, {NULL}};
const Segment SGI160943[] = {{&S46208[27], 6}, {NULL}};
const Segment SGI160944[] = {{&S46208[98], 9}, {NULL}};
const Segment SGI160945[] = {{&S46208[71], 6}, {NULL}};
const Segment SGI160946[] = {{&S46210[34], 11}, {NULL}};
const Segment SGI160947[] = {{&S46210[27], 6}, {NULL}};
const Segment SGI160948[] = {{&S46210[98], 9}, {NULL}};
const Segment SGI160949[] = {{&S46210[71], 6}, {NULL}};
const Segment SGI160950[] = {{&S46211[34], 10}, {NULL}};
const Segment SGI160951[] = {{&S46211[27], 6}, {NULL}};
const Segment SGI160952[] = {{&S46211[97], 9}, {NULL}};
const Segment SGI160953[] = {{&S46211[70], 6}, {NULL}};
const Segment SGI160954[] = {{&S46212[34], 10}, {NULL}};
const Segment SGI160955[] = {{&S46212[27], 6}, {NULL}};
const Segment SGI160956[] = {{&S46212[97], 9}, {NULL}};
const Segment SGI160957[] = {{&S46212[70], 6}, {NULL}};
const Segment SGI160958[] = {{&S46213[34], 11}, {NULL}};
const Segment SGI160959[] = {{&S46213[27], 6}, {NULL}};
const Segment SGI160960[] = {{&S46213[98], 9}, {NULL}};
const Segment SGI160961[] = {{&S46213[71], 6}, {NULL}};
const Segment SGI160962[] = {{&S46214[34], 10}, {NULL}};
const Segment SGI160963[] = {{&S46214[27], 6}, {NULL}};
const Segment SGI160964[] = {{&S46214[97], 9}, {NULL}};
const Segment SGI160965[] = {{&S46214[70], 6}, {NULL}};
const Segment SGI160966[] = {{&S46215[34], 11}, {NULL}};
const Segment SGI160967[] = {{&S46215[27], 6}, {NULL}};
const Segment SGI160968[] = {{&S46215[98], 9}, {NULL}};
const Segment SGI160969[] = {{&S46215[71], 6}, {NULL}};
const Segment SGI160970[] = {{&S46216[34], 7}, {NULL}};
const Segment SGI160971[] = {{&S46216[27], 6}, {NULL}};
const Segment SGI160972[] = {{&S46216[104], 9}, {NULL}};
const Segment SGI160973[] = {{&S46216[77], 6}, {NULL}};
const Segment SGI160974[] = {{&S42266[34], 29}, {NULL}};
const Segment SGI160975[] = {{&S42266[27], 6}, {NULL}};
const Segment SGI160976[] = {{&S42266[116], 9}, {NULL}};
const Segment SGI160977[] = {{&S42266[89], 6}, {NULL}};
const Segment SGI160978[] = {{&S42267[34], 30}, {NULL}};
const Segment SGI160979[] = {{&S42267[27], 6}, {NULL}};
const Segment SGI160980[] = {{&S42267[117], 9}, {NULL}};
const Segment SGI160981[] = {{&S42267[90], 6}, {NULL}};
const Segment SGI160982[] = {{&S42268[34], 30}, {NULL}};
const Segment SGI160983[] = {{&S42268[27], 6}, {NULL}};
const Segment SGI160984[] = {{&S42268[117], 9}, {NULL}};
const Segment SGI160985[] = {{&S42268[90], 6}, {NULL}};
const Segment SGI160986[] = {{&S46242[34], 10}, {NULL}};
const Segment SGI160987[] = {{&S46242[27], 6}, {NULL}};
const Segment SGI160988[] = {{&S46242[97], 9}, {NULL}};
const Segment SGI160989[] = {{&S46242[70], 6}, {NULL}};
const Segment SGI160990[] = {{&S42270[34], 17}, {NULL}};
const Segment SGI160991[] = {{&S42270[27], 6}, {NULL}};
const Segment SGI160992[] = {{&S42270[104], 9}, {NULL}};
const Segment SGI160993[] = {{&S42270[77], 6}, {NULL}};
const Segment SGI160994[] = {{&S46305[34], 10}, {NULL}};
const Segment SGI160995[] = {{&S46305[27], 6}, {NULL}};
const Segment SGI160996[] = {{&S46305[97], 9}, {NULL}};
const Segment SGI160997[] = {{&S46305[70], 6}, {NULL}};
const Segment SGI160998[] = {{&S46306[34], 11}, {NULL}};
const Segment SGI160999[] = {{&S46306[27], 6}, {NULL}};
const Segment SGI161000[] = {{&S46306[98], 9}, {NULL}};
const Segment SGI161001[] = {{&S46306[71], 6}, {NULL}};
const Segment SGI161002[] = {{&S46307[34], 11}, {NULL}};
const Segment SGI161003[] = {{&S46307[27], 6}, {NULL}};
const Segment SGI161004[] = {{&S46307[98], 9}, {NULL}};
const Segment SGI161005[] = {{&S46307[71], 6}, {NULL}};
const Segment SGI161006[] = {{&S46308[34], 11}, {NULL}};
const Segment SGI161007[] = {{&S46308[27], 6}, {NULL}};
const Segment SGI161008[] = {{&S46308[98], 9}, {NULL}};
const Segment SGI161009[] = {{&S46308[71], 6}, {NULL}};
const Segment SGI161010[] = {{&S46309[34], 11}, {NULL}};
const Segment SGI161011[] = {{&S46309[27], 6}, {NULL}};
const Segment SGI161012[] = {{&S46309[98], 9}, {NULL}};
const Segment SGI161013[] = {{&S46309[71], 6}, {NULL}};
const Segment SGI161014[] = {{&S46310[34], 11}, {NULL}};
const Segment SGI161015[] = {{&S46310[27], 6}, {NULL}};
const Segment SGI161016[] = {{&S46310[98], 9}, {NULL}};
const Segment SGI161017[] = {{&S46310[71], 6}, {NULL}};
const Segment SGI161018[] = {{&S46311[34], 8}, {NULL}};
const Segment SGI161019[] = {{&S46311[27], 6}, {NULL}};
const Segment SGI161020[] = {{&S46311[95], 9}, {NULL}};
const Segment SGI161021[] = {{&S46311[68], 6}, {NULL}};
const Segment SGI161022[] = {{&S46312[34], 10}, {NULL}};
const Segment SGI161023[] = {{&S46312[27], 6}, {NULL}};
const Segment SGI161024[] = {{&S46312[97], 9}, {NULL}};
const Segment SGI161025[] = {{&S46312[70], 6}, {NULL}};
const Segment SGI161026[] = {{&S51666[32], 14}, {NULL}};
const Segment SGI161027[] = {{&S51666[27], 4}, {NULL}};
const Segment SGI161028[] = {{&S51666[99], 9}, {NULL}};
const Segment SGI161029[] = {{&S51666[72], 6}, {NULL}};
const Segment SGI161030[] = {{&S46369[34], 10}, {NULL}};
const Segment SGI161031[] = {{&S46369[27], 6}, {NULL}};
const Segment SGI161032[] = {{&S46369[97], 9}, {NULL}};
const Segment SGI161033[] = {{&S46369[70], 6}, {NULL}};
const Segment SGI161034[] = {{&S46375[34], 10}, {NULL}};
const Segment SGI161035[] = {{&S46375[27], 6}, {NULL}};
const Segment SGI161036[] = {{&S46375[97], 9}, {NULL}};
const Segment SGI161037[] = {{&S46375[70], 6}, {NULL}};
const Segment SGI161038[] = {{&S46377[34], 7}, {NULL}};
const Segment SGI161039[] = {{&S46377[27], 6}, {NULL}};
const Segment SGI161040[] = {{&S46377[100], 9}, {NULL}};
const Segment SGI161041[] = {{&S46377[73], 6}, {NULL}};
const Segment SGI161042[] = {{&S46383[34], 7}, {NULL}};
const Segment SGI161043[] = {{&S46383[27], 6}, {NULL}};
const Segment SGI161044[] = {{&S46383[99], 9}, {NULL}};
const Segment SGI161045[] = {{&S46383[72], 6}, {NULL}};
const Segment SGI161046[] = {{&S46384[34], 10}, {NULL}};
const Segment SGI161047[] = {{&S46384[27], 6}, {NULL}};
const Segment SGI161048[] = {{&S46384[97], 9}, {NULL}};
const Segment SGI161049[] = {{&S46384[70], 6}, {NULL}};
const Segment SGI161050[] = {{&S46398[34], 10}, {NULL}};
const Segment SGI161051[] = {{&S46398[27], 6}, {NULL}};
const Segment SGI161052[] = {{&S46398[97], 9}, {NULL}};
const Segment SGI161053[] = {{&S46398[70], 6}, {NULL}};
const Segment SGI161054[] = {{&S46400[34], 10}, {NULL}};
const Segment SGI161055[] = {{&S46400[27], 6}, {NULL}};
const Segment SGI161056[] = {{&S46400[97], 9}, {NULL}};
const Segment SGI161057[] = {{&S46400[70], 6}, {NULL}};
const Segment SGI161058[] = {{&S46401[34], 7}, {NULL}};
const Segment SGI161059[] = {{&S46401[27], 6}, {NULL}};
const Segment SGI161060[] = {{&S46401[104], 9}, {NULL}};
const Segment SGI161061[] = {{&S46401[77], 6}, {NULL}};
const Segment SGI161062[] = {{&S46406[34], 11}, {NULL}};
const Segment SGI161063[] = {{&S46406[27], 6}, {NULL}};
const Segment SGI161064[] = {{&S46406[98], 9}, {NULL}};
const Segment SGI161065[] = {{&S46406[71], 6}, {NULL}};
const Segment SGI161066[] = {{&S46407[34], 10}, {NULL}};
const Segment SGI161067[] = {{&S46407[27], 6}, {NULL}};
const Segment SGI161068[] = {{&S46407[97], 9}, {NULL}};
const Segment SGI161069[] = {{&S46407[70], 6}, {NULL}};
const Segment SGI161070[] = {{&S46408[34], 11}, {NULL}};
const Segment SGI161071[] = {{&S46408[27], 6}, {NULL}};
const Segment SGI161072[] = {{&S46408[98], 9}, {NULL}};
const Segment SGI161073[] = {{&S46408[71], 6}, {NULL}};
const Segment SGI161074[] = {{&S46409[34], 11}, {NULL}};
const Segment SGI161075[] = {{&S46409[27], 6}, {NULL}};
const Segment SGI161076[] = {{&S46409[98], 9}, {NULL}};
const Segment SGI161077[] = {{&S46409[71], 6}, {NULL}};
const Segment SGI161078[] = {{&S46410[34], 11}, {NULL}};
const Segment SGI161079[] = {{&S46410[27], 6}, {NULL}};
const Segment SGI161080[] = {{&S46410[98], 9}, {NULL}};
const Segment SGI161081[] = {{&S46410[71], 6}, {NULL}};
const Segment SGI161082[] = {{&S46411[34], 11}, {NULL}};
const Segment SGI161083[] = {{&S46411[27], 6}, {NULL}};
const Segment SGI161084[] = {{&S46411[98], 9}, {NULL}};
const Segment SGI161085[] = {{&S46411[71], 6}, {NULL}};
const Segment SGI161086[] = {{&S46412[34], 10}, {NULL}};
const Segment SGI161087[] = {{&S46412[27], 6}, {NULL}};
const Segment SGI161088[] = {{&S46412[97], 9}, {NULL}};
const Segment SGI161089[] = {{&S46412[70], 6}, {NULL}};
const Segment SGI161090[] = {{&S46414[34], 8}, {NULL}};
const Segment SGI161091[] = {{&S46414[27], 6}, {NULL}};
const Segment SGI161092[] = {{&S46414[95], 9}, {NULL}};
const Segment SGI161093[] = {{&S46414[68], 6}, {NULL}};
const Segment SGI161094[] = {{&S45102[42], 10}, {NULL}};
const Segment SGI161095[] = {{&S45102[42], 22}, {NULL}};
const Segment SGI161096[] = {{&S45102[30], 4}, {NULL}};
const Segment SGI161097[] = {{&S45102[113], 3}, {NULL}};
const Segment SGI161098[] = {{&S45102[78], 6}, {NULL}};
const Segment SGI161099[] = {{&S45104[36], 10}, {NULL}};
const Segment SGI161100[] = {{&S45104[31], 4}, {NULL}};
const Segment SGI161101[] = {{&S47684[34], 8}, {NULL}};
const Segment SGI161102[] = {{&S47684[27], 6}, {NULL}};
const Segment SGI161103[] = {{&S47684[95], 9}, {NULL}};
const Segment SGI161104[] = {{&S47684[68], 6}, {NULL}};
const Segment SGI161105[] = {{&S32947[34], 14}, {NULL}};
const Segment SGI161106[] = {{&S32947[27], 6}, {NULL}};
const Segment SGI161107[] = {{&S32947[100], 9}, {NULL}};
const Segment SGI161108[] = {{&S32947[73], 6}, {NULL}};
const Segment SGI161109[] = {{&S32948[34], 14}, {NULL}};
const Segment SGI161110[] = {{&S32948[27], 6}, {NULL}};
const Segment SGI161111[] = {{&S32948[100], 9}, {NULL}};
const Segment SGI161112[] = {{&S32948[73], 6}, {NULL}};
const Segment SGI161113[] = {{&S32949[41], 22}, {NULL}};
const Segment SGI161114[] = {{&S32949[41], 33}, {NULL}};
const Segment SGI161115[] = {{&S32949[27], 6}, {NULL}};
const Segment SGI161116[] = {{&S32949[123], 3}, {NULL}};
const Segment SGI161117[] = {{&S32949[127], 9}, {NULL}};
const Segment SGI161118[] = {{&S32949[88], 6}, {NULL}};
const Segment SGI161119[] = {{&S47701[34], 11}, {NULL}};
const Segment SGI161120[] = {{&S47701[27], 6}, {NULL}};
const Segment SGI161121[] = {{&S47701[98], 9}, {NULL}};
const Segment SGI161122[] = {{&S47701[71], 6}, {NULL}};
const Segment SGI161123[] = {{&S46496[34], 7}, {NULL}};
const Segment SGI161124[] = {{&S46496[27], 6}, {NULL}};
const Segment SGI161125[] = {{&S46496[104], 9}, {NULL}};
const Segment SGI161126[] = {{&S46496[77], 6}, {NULL}};
const Segment SGI161127[] = {{&S46497[44], 10}, {NULL}};
const Segment SGI161128[] = {{&S46497[44], 22}, {NULL}};
const Segment SGI161129[] = {{&S46497[30], 6}, {NULL}};
const Segment SGI161130[] = {{&S46497[115], 3}, {NULL}};
const Segment SGI161131[] = {{&S46497[80], 6}, {NULL}};
const Segment SGI161132[] = {{&S46498[38], 10}, {NULL}};
const Segment SGI161133[] = {{&S46498[31], 6}, {NULL}};
const Segment SGI161134[] = {{&S46499[34], 10}, {NULL}};
const Segment SGI161135[] = {{&S46499[27], 6}, {NULL}};
const Segment SGI161136[] = {{&S46499[97], 9}, {NULL}};
const Segment SGI161137[] = {{&S46499[70], 6}, {NULL}};
const Segment SGI161138[] = {{&S46501[44], 9}, {NULL}};
const Segment SGI161139[] = {{&S46501[44], 21}, {NULL}};
const Segment SGI161140[] = {{&S46501[30], 6}, {NULL}};
const Segment SGI161141[] = {{&S46501[114], 3}, {NULL}};
const Segment SGI161142[] = {{&S46501[79], 6}, {NULL}};
const Segment SGI161143[] = {{&S56702[34], 25}, {NULL}};
const Segment SGI161144[] = {{&S56702[27], 6}, {NULL}};
const Segment SGI161145[] = {{&S56702[112], 9}, {NULL}};
const Segment SGI161146[] = {{&S56702[85], 6}, {NULL}};
const Segment SGI161147[] = {{&S46502[44], 12}, {NULL}};
const Segment SGI161148[] = {{&S46502[44], 24}, {NULL}};
const Segment SGI161149[] = {{&S46502[30], 6}, {NULL}};
const Segment SGI161150[] = {{&S46502[117], 3}, {NULL}};
const Segment SGI161151[] = {{&S46502[82], 6}, {NULL}};
const Segment SGI161152[] = {{&S46503[38], 9}, {NULL}};
const Segment SGI161153[] = {{&S46503[31], 6}, {NULL}};
const Segment SGI161154[] = {{&S56704[44], 9}, {NULL}};
const Segment SGI161155[] = {{&S56704[44], 21}, {NULL}};
const Segment SGI161156[] = {{&S56704[30], 6}, {NULL}};
const Segment SGI161157[] = {{&S56704[31], 3}, {NULL}};
const Segment SGI161158[] = {{&S56704[79], 6}, {NULL}};
const Segment SGI161159[] = {{&S56705[44], 9}, {NULL}};
const Segment SGI161160[] = {{&S56705[44], 21}, {NULL}};
const Segment SGI161161[] = {{&S56705[30], 6}, {NULL}};
const Segment SGI161162[] = {{&S56705[31], 3}, {NULL}};
const Segment SGI161163[] = {{&S56705[79], 6}, {NULL}};
const Segment SGI161164[] = {{&S42581[34], 10}, {NULL}};
const Segment SGI161165[] = {{&S42581[27], 6}, {NULL}};
const Segment SGI161166[] = {{&S42581[97], 9}, {NULL}};
const Segment SGI161167[] = {{&S42581[70], 6}, {NULL}};
const Segment SGI161168[] = {{&S42583[34], 18}, {NULL}};
const Segment SGI161169[] = {{&S42583[27], 6}, {NULL}};
const Segment SGI161170[] = {{&S42583[105], 9}, {NULL}};
const Segment SGI161171[] = {{&S42583[78], 6}, {NULL}};
const Segment SGI161172[] = {{&S47764[34], 10}, {NULL}};
const Segment SGI161173[] = {{&S47764[27], 6}, {NULL}};
const Segment SGI161174[] = {{&S47764[97], 9}, {NULL}};
const Segment SGI161175[] = {{&S47764[70], 6}, {NULL}};
const Segment SGI161176[] = {{&S56767[34], 10}, {NULL}};
const Segment SGI161177[] = {{&S56767[27], 6}, {NULL}};
const Segment SGI161178[] = {{&S56767[97], 9}, {NULL}};
const Segment SGI161179[] = {{&S56767[70], 6}, {NULL}};
const Segment SGI161180[] = {{&S56768[34], 10}, {NULL}};
const Segment SGI161181[] = {{&S56768[27], 6}, {NULL}};
const Segment SGI161182[] = {{&S56768[97], 9}, {NULL}};
const Segment SGI161183[] = {{&S56768[70], 6}, {NULL}};
const Segment SGI161184[] = {{&S56783[34], 10}, {NULL}};
const Segment SGI161185[] = {{&S56783[27], 6}, {NULL}};
const Segment SGI161186[] = {{&S56783[97], 9}, {NULL}};
const Segment SGI161187[] = {{&S56783[70], 6}, {NULL}};
const Segment SGI161188[] = {{&S47843[34], 10}, {NULL}};
const Segment SGI161189[] = {{&S47843[27], 6}, {NULL}};
const Segment SGI161190[] = {{&S47843[97], 9}, {NULL}};
const Segment SGI161191[] = {{&S47843[70], 6}, {NULL}};
const Segment SGI161192[] = {{&S47867[44], 15}, {NULL}};
const Segment SGI161193[] = {{&S47867[44], 27}, {NULL}};
const Segment SGI161194[] = {{&S47867[30], 6}, {NULL}};
const Segment SGI161195[] = {{&S47867[136], 3}, {NULL}};
const Segment SGI161196[] = {{&S47867[101], 6}, {NULL}};
const Segment SGI161197[] = {{&S47867[73], 8}, {NULL}};
const Segment SGI161198[] = {{&S47868[44], 15}, {NULL}};
const Segment SGI161199[] = {{&S47868[44], 27}, {NULL}};
const Segment SGI161200[] = {{&S47868[30], 6}, {NULL}};
const Segment SGI161201[] = {{&S47868[136], 3}, {NULL}};
const Segment SGI161202[] = {{&S56834[34], 7}, {NULL}};
const Segment SGI161203[] = {{&S47868[101], 6}, {NULL}};
const Segment SGI161204[] = {{&S56834[27], 6}, {NULL}};
const Segment SGI161205[] = {{&S47868[73], 8}, {NULL}};
const Segment SGI161206[] = {{&S56834[106], 9}, {NULL}};
const Segment SGI161207[] = {{&S56834[79], 6}, {NULL}};
const Segment SGI161208[] = {{&S47869[44], 15}, {NULL}};
const Segment SGI161209[] = {{&S47869[44], 27}, {NULL}};
const Segment SGI161210[] = {{&S47869[30], 6}, {NULL}};
const Segment SGI161211[] = {{&S47869[136], 3}, {NULL}};
const Segment SGI161212[] = {{&S56835[34], 7}, {NULL}};
const Segment SGI161213[] = {{&S47869[101], 6}, {NULL}};
const Segment SGI161214[] = {{&S56835[27], 6}, {NULL}};
const Segment SGI161215[] = {{&S47869[73], 8}, {NULL}};
const Segment SGI161216[] = {{&S56835[106], 9}, {NULL}};
const Segment SGI161217[] = {{&S56835[79], 6}, {NULL}};
const Segment SGI161218[] = {{&S56836[34], 18}, {NULL}};
const Segment SGI161219[] = {{&S56836[27], 6}, {NULL}};
const Segment SGI161220[] = {{&S56836[105], 9}, {NULL}};
const Segment SGI161221[] = {{&S56836[78], 6}, {NULL}};
const Segment SGI161222[] = {{&S42724[44], 10}, {NULL}};
const Segment SGI161223[] = {{&S42724[44], 22}, {NULL}};
const Segment SGI161224[] = {{&S42724[30], 6}, {NULL}};
const Segment SGI161225[] = {{&S42724[114], 3}, {NULL}};
const Segment SGI161226[] = {{&S42724[80], 5}, {NULL}};
const Segment SGI161227[] = {{&S56857[44], 7}, {NULL}};
const Segment SGI161228[] = {{&S56857[44], 19}, {NULL}};
const Segment SGI161229[] = {{&S56857[30], 6}, {NULL}};
const Segment SGI161230[] = {{&S56857[31], 3}, {NULL}};
const Segment SGI161231[] = {{&S56857[77], 6}, {NULL}};
const Segment SGI161232[] = {{&S42726[44], 9}, {NULL}};
const Segment SGI161233[] = {{&S42726[44], 21}, {NULL}};
const Segment SGI161234[] = {{&S42726[30], 6}, {NULL}};
const Segment SGI161235[] = {{&S42726[114], 3}, {NULL}};
const Segment SGI161236[] = {{&S42726[79], 6}, {NULL}};
const Segment SGI161237[] = {{&S56858[44], 8}, {NULL}};
const Segment SGI161238[] = {{&S56858[44], 20}, {NULL}};
const Segment SGI161239[] = {{&S56858[30], 6}, {NULL}};
const Segment SGI161240[] = {{&S56858[31], 3}, {NULL}};
const Segment SGI161241[] = {{&S56858[78], 6}, {NULL}};
const Segment SGI161242[] = {{&S42728[44], 15}, {NULL}};
const Segment SGI161243[] = {{&S42728[44], 27}, {NULL}};
const Segment SGI161244[] = {{&S56859[38], 7}, {NULL}};
const Segment SGI161245[] = {{&S42728[30], 6}, {NULL}};
const Segment SGI161246[] = {{&S56859[31], 6}, {NULL}};
const Segment SGI161247[] = {{&S42728[120], 3}, {NULL}};
const Segment SGI161248[] = {{&S42728[85], 6}, {NULL}};
const Segment SGI161249[] = {{&S47901[44], 11}, {NULL}};
const Segment SGI161250[] = {{&S47901[44], 23}, {NULL}};
const Segment SGI161251[] = {{&S47901[30], 6}, {NULL}};
const Segment SGI161252[] = {{&S56860[34], 7}, {NULL}};
const Segment SGI161253[] = {{&S56860[27], 6}, {NULL}};
const Segment SGI161254[] = {{&S47901[116], 3}, {NULL}};
const Segment SGI161255[] = {{&S47901[81], 6}, {NULL}};
const Segment SGI161256[] = {{&S56860[111], 9}, {NULL}};
const Segment SGI161257[] = {{&S56860[84], 6}, {NULL}};
const Segment SGI161258[] = {{&S47901[151], 3}, {NULL}};
const Segment SGI161259[] = {{&S42731[44], 7}, {NULL}};
const Segment SGI161260[] = {{&S42731[44], 19}, {NULL}};
const Segment SGI161261[] = {{&S42731[30], 6}, {NULL}};
const Segment SGI161262[] = {{&S42731[112], 3}, {NULL}};
const Segment SGI161263[] = {{&S42731[77], 6}, {NULL}};
const Segment SGI161264[] = {{&S42732[44], 19}, {NULL}};
const Segment SGI161265[] = {{&S42732[44], 31}, {NULL}};
const Segment SGI161266[] = {{&S42732[30], 6}, {NULL}};
const Segment SGI161267[] = {{&S42732[124], 3}, {NULL}};
const Segment SGI161268[] = {{&S47904[44], 18}, {NULL}};
const Segment SGI161269[] = {{&S42732[89], 6}, {NULL}};
const Segment SGI161270[] = {{&S47904[44], 30}, {NULL}};
const Segment SGI161271[] = {{&S47904[30], 6}, {NULL}};
const Segment SGI161272[] = {{&S47904[123], 3}, {NULL}};
const Segment SGI161273[] = {{&S56864[34], 10}, {NULL}};
const Segment SGI161274[] = {{&S56864[27], 6}, {NULL}};
const Segment SGI161275[] = {{&S47904[88], 6}, {NULL}};
const Segment SGI161276[] = {{&S47904[158], 3}, {NULL}};
const Segment SGI161277[] = {{&S56864[97], 9}, {NULL}};
const Segment SGI161278[] = {{&S56864[70], 6}, {NULL}};
const Segment SGI161279[] = {{&S42733[44], 19}, {NULL}};
const Segment SGI161280[] = {{&S42733[44], 31}, {NULL}};
const Segment SGI161281[] = {{&S42733[30], 6}, {NULL}};
const Segment SGI161282[] = {{&S42733[124], 3}, {NULL}};
const Segment SGI161283[] = {{&S42733[89], 6}, {NULL}};
const Segment SGI161284[] = {{&S56865[34], 11}, {NULL}};
const Segment SGI161285[] = {{&S56865[27], 6}, {NULL}};
const Segment SGI161286[] = {{&S42734[44], 20}, {NULL}};
const Segment SGI161287[] = {{&S56865[98], 9}, {NULL}};
const Segment SGI161288[] = {{&S42734[44], 32}, {NULL}};
const Segment SGI161289[] = {{&S56865[71], 6}, {NULL}};
const Segment SGI161290[] = {{&S42734[30], 6}, {NULL}};
const Segment SGI161291[] = {{&S42734[125], 3}, {NULL}};
const Segment SGI161292[] = {{&S42734[90], 6}, {NULL}};
const Segment SGI161293[] = {{&S42735[44], 14}, {NULL}};
const Segment SGI161294[] = {{&S42735[44], 26}, {NULL}};
const Segment SGI161295[] = {{&S42735[30], 6}, {NULL}};
const Segment SGI161296[] = {{&S42735[119], 3}, {NULL}};
const Segment SGI161297[] = {{&S42735[84], 6}, {NULL}};
const Segment SGI161298[] = {{&S42739[44], 21}, {NULL}};
const Segment SGI161299[] = {{&S42739[44], 33}, {NULL}};
const Segment SGI161300[] = {{&S42739[30], 6}, {NULL}};
const Segment SGI161301[] = {{&S42739[126], 3}, {NULL}};
const Segment SGI161302[] = {{&S42739[91], 6}, {NULL}};
const Segment SGI161303[] = {{&S56873[44], 7}, {NULL}};
const Segment SGI161304[] = {{&S56873[44], 19}, {NULL}};
const Segment SGI161305[] = {{&S56873[30], 6}, {NULL}};
const Segment SGI161306[] = {{&S56873[31], 3}, {NULL}};
const Segment SGI161307[] = {{&S56873[77], 6}, {NULL}};
const Segment SGI161308[] = {{&S42742[44], 26}, {NULL}};
const Segment SGI161309[] = {{&S42742[44], 38}, {NULL}};
const Segment SGI161310[] = {{&S42742[30], 6}, {NULL}};
const Segment SGI161311[] = {{&S42742[131], 3}, {NULL}};
const Segment SGI161312[] = {{&S42742[96], 6}, {NULL}};
const Segment SGI161313[] = {{&S56875[34], 11}, {NULL}};
const Segment SGI161314[] = {{&S56875[27], 6}, {NULL}};
const Segment SGI161315[] = {{&S56875[98], 9}, {NULL}};
const Segment SGI161316[] = {{&S56875[71], 6}, {NULL}};
const Segment SGI161317[] = {{&S47919[34], 7}, {NULL}};
const Segment SGI161318[] = {{&S47919[27], 6}, {NULL}};
const Segment SGI161319[] = {{&S42744[44], 20}, {NULL}};
const Segment SGI161320[] = {{&S42744[44], 32}, {NULL}};
const Segment SGI161321[] = {{&S47919[110], 9}, {NULL}};
const Segment SGI161322[] = {{&S42744[30], 6}, {NULL}};
const Segment SGI161323[] = {{&S47919[83], 6}, {NULL}};
const Segment SGI161324[] = {{&S42744[125], 3}, {NULL}};
const Segment SGI161325[] = {{&S42744[90], 6}, {NULL}};
const Segment SGI161326[] = {{&S42747[44], 26}, {NULL}};
const Segment SGI161327[] = {{&S42747[44], 38}, {NULL}};
const Segment SGI161328[] = {{&S42747[30], 6}, {NULL}};
const Segment SGI161329[] = {{&S42747[131], 3}, {NULL}};
const Segment SGI161330[] = {{&S42747[96], 6}, {NULL}};
const Segment SGI161331[] = {{&S42748[44], 9}, {NULL}};
const Segment SGI161332[] = {{&S42748[44], 21}, {NULL}};
const Segment SGI161333[] = {{&S42748[30], 6}, {NULL}};
const Segment SGI161334[] = {{&S42748[113], 3}, {NULL}};
const Segment SGI161335[] = {{&S42748[79], 5}, {NULL}};
const Segment SGI161336[] = {{&S42750[38], 14}, {NULL}};
const Segment SGI161337[] = {{&S42750[31], 6}, {NULL}};
const Segment SGI161338[] = {{&S42755[44], 21}, {NULL}};
const Segment SGI161339[] = {{&S56891[34], 18}, {NULL}};
const Segment SGI161340[] = {{&S42755[44], 33}, {NULL}};
const Segment SGI161341[] = {{&S42755[30], 6}, {NULL}};
const Segment SGI161342[] = {{&S56891[27], 6}, {NULL}};
const Segment SGI161343[] = {{&S42755[126], 3}, {NULL}};
const Segment SGI161344[] = {{&S56891[105], 9}, {NULL}};
const Segment SGI161345[] = {{&S42755[91], 6}, {NULL}};
const Segment SGI161346[] = {{&S56891[78], 6}, {NULL}};
const Segment SGI161347[] = {{&S56893[34], 10}, {NULL}};
const Segment SGI161348[] = {{&S56893[27], 6}, {NULL}};
const Segment SGI161349[] = {{&S56893[97], 9}, {NULL}};
const Segment SGI161350[] = {{&S56893[70], 6}, {NULL}};
const Segment SGI161351[] = {{&S42761[83], 6}, {NULL}};
const Segment SGI161352[] = {{&S42763[44], 18}, {NULL}};
const Segment SGI161353[] = {{&S42763[44], 30}, {NULL}};
const Segment SGI161354[] = {{&S42763[30], 6}, {NULL}};
const Segment SGI161355[] = {{&S42763[123], 3}, {NULL}};
const Segment SGI161356[] = {{&S42763[88], 6}, {NULL}};
const Segment SGI161357[] = {{&S42765[44], 8}, {NULL}};
const Segment SGI161358[] = {{&S42765[44], 20}, {NULL}};
const Segment SGI161359[] = {{&S42765[30], 6}, {NULL}};
const Segment SGI161360[] = {{&S42765[113], 3}, {NULL}};
const Segment SGI161361[] = {{&S42765[78], 6}, {NULL}};
const Segment SGI161362[] = {{&S42767[44], 3}, {NULL}};
const Segment SGI161363[] = {{&S42767[44], 12}, {NULL}};
const Segment SGI161364[] = {{&S42767[30], 6}, {NULL}};
const Segment SGI161365[] = {{&S42767[105], 3}, {NULL}};
const Segment SGI161366[] = {{&S42767[70], 6}, {NULL}};
const Segment SGI161367[] = {{&S42768[22], 9}, {NULL}};
const Segment SGI161368[] = {{&S42768[44], 21}, {NULL}};
const Segment SGI161369[] = {{&S42768[30], 6}, {&S42768[52], 13}, {NULL}};
const Segment SGI161370[] = {{&S42768[114], 3}, {NULL}};
const Segment SGI161371[] = {{&S42768[79], 6}, {NULL}};
const Segment SGI161372[] = {{&S56909[34], 7}, {NULL}};
const Segment SGI161373[] = {{&S56909[27], 6}, {NULL}};
const Segment SGI161374[] = {{&S56909[99], 9}, {NULL}};
const Segment SGI161375[] = {{&S56909[72], 6}, {NULL}};
const Segment SGI161376[] = {{&S56910[34], 8}, {NULL}};
const Segment SGI161377[] = {{&S42770[44], 14}, {NULL}};
const Segment SGI161378[] = {{&S56910[27], 6}, {NULL}};
const Segment SGI161379[] = {{&S42770[44], 31}, {NULL}};
const Segment SGI161380[] = {{&S42770[30], 6}, {NULL}};
const Segment SGI161381[] = {{&S56910[99], 9}, {NULL}};
const Segment SGI161382[] = {{&S42770[124], 3}, {NULL}};
const Segment SGI161383[] = {{&S56910[72], 6}, {NULL}};
const Segment SGI161384[] = {{&S42770[89], 6}, {NULL}};
const Segment SGI161385[] = {{&S56911[34], 8}, {NULL}};
const Segment SGI161386[] = {{&S56911[27], 6}, {NULL}};
const Segment SGI161387[] = {{&S56911[99], 9}, {NULL}};
const Segment SGI161388[] = {{&S56911[72], 6}, {NULL}};
const Segment SGI161389[] = {{&S56912[34], 7}, {NULL}};
const Segment SGI161390[] = {{&S56912[27], 6}, {NULL}};
const Segment SGI161391[] = {{&S56912[99], 9}, {NULL}};
const Segment SGI161392[] = {{&S56912[72], 6}, {NULL}};
const Segment SGI161393[] = {{&S56918[34], 11}, {NULL}};
const Segment SGI161394[] = {{&S56918[27], 6}, {NULL}};
const Segment SGI161395[] = {{&S56918[98], 9}, {NULL}};
const Segment SGI161396[] = {{&S56918[71], 6}, {NULL}};
const Segment SGI161397[] = {{&S56920[34], 15}, {NULL}};
const Segment SGI161398[] = {{&S56920[27], 6}, {NULL}};
const Segment SGI161399[] = {{&S56920[102], 9}, {NULL}};
const Segment SGI161400[] = {{&S56920[75], 6}, {NULL}};
const Segment SGI161401[] = {{&S56962[34], 12}, {NULL}};
const Segment SGI161402[] = {{&S56962[27], 6}, {NULL}};
const Segment SGI161403[] = {{&S56962[99], 9}, {NULL}};
const Segment SGI161404[] = {{&S56962[72], 6}, {NULL}};
const Segment SGI161405[] = {{&S48009[34], 6}, {NULL}};
const Segment SGI161406[] = {{&S48009[27], 6}, {NULL}};
const Segment SGI161407[] = {{&S48009[93], 9}, {NULL}};
const Segment SGI161408[] = {{&S48009[66], 6}, {NULL}};
const Segment SGI161409[] = {{&S56964[34], 12}, {NULL}};
const Segment SGI161410[] = {{&S56964[27], 6}, {NULL}};
const Segment SGI161411[] = {{&S56964[99], 9}, {NULL}};
const Segment SGI161412[] = {{&S56964[72], 6}, {NULL}};
const Segment SGI161413[] = {{&S56965[34], 7}, {NULL}};
const Segment SGI161414[] = {{&S56965[27], 6}, {NULL}};
const Segment SGI161415[] = {{&S56965[94], 9}, {NULL}};
const Segment SGI161416[] = {{&S56965[67], 6}, {NULL}};
const Segment SGI161417[] = {{&S56966[34], 7}, {NULL}};
const Segment SGI161418[] = {{&S56966[27], 6}, {NULL}};
const Segment SGI161419[] = {{&S56966[93], 9}, {NULL}};
const Segment SGI161420[] = {{&S56966[66], 6}, {NULL}};
const Segment SGI161421[] = {{&S56967[44], 7}, {NULL}};
const Segment SGI161422[] = {{&S56967[44], 19}, {NULL}};
const Segment SGI161423[] = {{&S56967[30], 6}, {NULL}};
const Segment SGI161424[] = {{&S56967[31], 3}, {NULL}};
const Segment SGI161425[] = {{&S56967[77], 6}, {NULL}};
const Segment SGI161426[] = {{&S56990[34], 10}, {NULL}};
const Segment SGI161427[] = {{&S56990[27], 6}, {NULL}};
const Segment SGI161428[] = {{&S56990[97], 9}, {NULL}};
const Segment SGI161429[] = {{&S56990[70], 6}, {NULL}};
const Segment SGI161430[] = {{&S56991[34], 11}, {NULL}};
const Segment SGI161431[] = {{&S56991[27], 6}, {NULL}};
const Segment SGI161432[] = {{&S56991[98], 9}, {NULL}};
const Segment SGI161433[] = {{&S56991[71], 6}, {NULL}};
const Segment SGI161434[] = {{&S48044[34], 10}, {NULL}};
const Segment SGI161435[] = {{&S48044[27], 6}, {NULL}};
const Segment SGI161436[] = {{&S48044[97], 9}, {NULL}};
const Segment SGI161437[] = {{&S48044[70], 6}, {NULL}};
const Segment SGI161438[] = {{&S57009[34], 7}, {NULL}};
const Segment SGI161439[] = {{&S57009[27], 6}, {NULL}};
const Segment SGI161440[] = {{&S57009[94], 9}, {NULL}};
const Segment SGI161441[] = {{&S57009[67], 6}, {NULL}};
const Segment SGI161442[] = {{&S48068[34], 12}, {NULL}};
const Segment SGI161443[] = {{&S48068[27], 6}, {NULL}};
const Segment SGI161444[] = {{&S48068[99], 9}, {NULL}};
const Segment SGI161445[] = {{&S48068[72], 6}, {NULL}};
const Segment SGI161446[] = {{&S48071[34], 12}, {NULL}};
const Segment SGI161447[] = {{&S48071[27], 6}, {NULL}};
const Segment SGI161448[] = {{&S48071[99], 9}, {NULL}};
const Segment SGI161449[] = {{&S48071[72], 6}, {NULL}};
const Segment SGI161450[] = {{&S48072[34], 14}, {NULL}};
const Segment SGI161451[] = {{&S48072[27], 6}, {NULL}};
const Segment SGI161452[] = {{&S48072[101], 9}, {NULL}};
const Segment SGI161453[] = {{&S48072[74], 6}, {NULL}};
const Segment SGI161454[] = {{&S48074[34], 10}, {NULL}};
const Segment SGI161455[] = {{&S48074[27], 6}, {NULL}};
const Segment SGI161456[] = {{&S48074[97], 9}, {NULL}};
const Segment SGI161457[] = {{&S48074[70], 6}, {NULL}};
const Segment SGI161458[] = {{&S57031[44], 7}, {NULL}};
const Segment SGI161459[] = {{&S57031[44], 20}, {NULL}};
const Segment SGI161460[] = {{&S57031[30], 6}, {NULL}};
const Segment SGI161461[] = {{&S57031[31], 3}, {NULL}};
const Segment SGI161462[] = {{&S57031[78], 6}, {NULL}};
const Segment SGI161463[] = {{&S57032[44], 7}, {NULL}};
const Segment SGI161464[] = {{&S57032[44], 20}, {NULL}};
const Segment SGI161465[] = {{&S57032[30], 6}, {NULL}};
const Segment SGI161466[] = {{&S57032[31], 3}, {NULL}};
const Segment SGI161467[] = {{&S57032[78], 6}, {NULL}};
const Segment SGI161468[] = {{&S46692[44], 14}, {NULL}};
const Segment SGI161469[] = {{&S46692[44], 26}, {NULL}};
const Segment SGI161470[] = {{&S46692[30], 6}, {NULL}};
const Segment SGI161471[] = {{&S57033[44], 7}, {NULL}};
const Segment SGI161472[] = {{&S57033[44], 20}, {NULL}};
const Segment SGI161473[] = {{&S46692[119], 3}, {NULL}};
const Segment SGI161474[] = {{&S57033[30], 6}, {NULL}};
const Segment SGI161475[] = {{&S46692[84], 6}, {NULL}};
const Segment SGI161476[] = {{&S57033[31], 3}, {NULL}};
const Segment SGI161477[] = {{&S57033[78], 6}, {NULL}};
const Segment SGI161478[] = {{&S46693[38], 14}, {NULL}};
const Segment SGI161479[] = {{&S46693[31], 6}, {NULL}};
const Segment SGI161480[] = {{&S57034[44], 7}, {NULL}};
const Segment SGI161481[] = {{&S57034[44], 20}, {NULL}};
const Segment SGI161482[] = {{&S57034[30], 6}, {NULL}};
const Segment SGI161483[] = {{&S57034[31], 3}, {NULL}};
const Segment SGI161484[] = {{&S57034[78], 6}, {NULL}};
const Segment SGI161485[] = {{&S57035[38], 7}, {NULL}};
const Segment SGI161486[] = {{&S57035[31], 6}, {NULL}};
const Segment SGI161487[] = {{&S57036[44], 7}, {NULL}};
const Segment SGI161488[] = {{&S57036[44], 20}, {NULL}};
const Segment SGI161489[] = {{&S57036[30], 6}, {NULL}};
const Segment SGI161490[] = {{&S57036[31], 3}, {NULL}};
const Segment SGI161491[] = {{&S57036[78], 6}, {NULL}};
const Segment SGI161492[] = {{&S57043[34], 9}, {NULL}};
const Segment SGI161493[] = {{&S57043[27], 6}, {NULL}};
const Segment SGI161494[] = {{&S46704[34], 9}, {NULL}};
const Segment SGI161495[] = {{&S46704[27], 6}, {NULL}};
const Segment SGI161496[] = {{&S57043[96], 9}, {NULL}};
const Segment SGI161497[] = {{&S57043[69], 6}, {NULL}};
const Segment SGI161498[] = {{&S46704[96], 9}, {NULL}};
const Segment SGI161499[] = {{&S46704[69], 6}, {NULL}};
const Segment SGI161500[] = {{&S57044[34], 5}, {NULL}};
const Segment SGI161501[] = {{&S57044[27], 6}, {NULL}};
const Segment SGI161502[] = {{&S46705[34], 10}, {NULL}};
const Segment SGI161503[] = {{&S46705[27], 6}, {NULL}};
const Segment SGI161504[] = {{&S57044[92], 9}, {NULL}};
const Segment SGI161505[] = {{&S57044[65], 6}, {NULL}};
const Segment SGI161506[] = {{&S46705[97], 9}, {NULL}};
const Segment SGI161507[] = {{&S46705[70], 6}, {NULL}};
const Segment SGI161508[] = {{&S57045[34], 5}, {NULL}};
const Segment SGI161509[] = {{&S57045[27], 6}, {NULL}};
const Segment SGI161510[] = {{&S57045[92], 9}, {NULL}};
const Segment SGI161511[] = {{&S57045[65], 6}, {NULL}};
const Segment SGI161512[] = {{&S57046[34], 16}, {NULL}};
const Segment SGI161513[] = {{&S57046[27], 6}, {NULL}};
const Segment SGI161514[] = {{&S57046[103], 9}, {NULL}};
const Segment SGI161515[] = {{&S57046[76], 6}, {NULL}};
const Segment SGI161516[] = {{&S57047[34], 5}, {NULL}};
const Segment SGI161517[] = {{&S57047[27], 6}, {NULL}};
const Segment SGI161518[] = {{&S57047[92], 9}, {NULL}};
const Segment SGI161519[] = {{&S57047[65], 6}, {NULL}};
const Segment SGI161520[] = {{&S57048[34], 8}, {NULL}};
const Segment SGI161521[] = {{&S57048[27], 6}, {NULL}};
const Segment SGI161522[] = {{&S57048[95], 9}, {NULL}};
const Segment SGI161523[] = {{&S57048[68], 6}, {NULL}};
const Segment SGI161524[] = {{&S57049[11], 1}, {NULL}};
const Segment SGI161525[] = {{&S57049[27], 6}, {NULL}};
const Segment SGI161526[] = {{&S57049[88], 9}, {NULL}};
const Segment SGI161527[] = {{&S57049[61], 6}, {NULL}};
const Segment SGI161528[] = {{&S57052[34], 13}, {NULL}};
const Segment SGI161529[] = {{&S57052[27], 6}, {NULL}};
const Segment SGI161530[] = {{&S57052[100], 9}, {NULL}};
const Segment SGI161531[] = {{&S57052[73], 6}, {NULL}};
const Segment SGI161532[] = {{&S57053[34], 16}, {NULL}};
const Segment SGI161533[] = {{&S57053[27], 6}, {NULL}};
const Segment SGI161534[] = {{&S57053[103], 9}, {NULL}};
const Segment SGI161535[] = {{&S57053[76], 6}, {NULL}};
const Segment SGI161536[] = {{&S57054[34], 30}, {NULL}};
const Segment SGI161537[] = {{&S57054[27], 6}, {NULL}};
const Segment SGI161538[] = {{&S57054[117], 9}, {NULL}};
const Segment SGI161539[] = {{&S57054[90], 6}, {NULL}};
const Segment SGI161540[] = {{&S57055[34], 21}, {NULL}};
const Segment SGI161541[] = {{&S57055[27], 6}, {NULL}};
const Segment SGI161542[] = {{&S57055[108], 9}, {NULL}};
const Segment SGI161543[] = {{&S57055[81], 6}, {NULL}};
const Segment SGI161544[] = {{&S57057[11], 1}, {NULL}};
const Segment SGI161545[] = {{&S57057[27], 6}, {NULL}};
const Segment SGI161546[] = {{&S57057[88], 9}, {NULL}};
const Segment SGI161547[] = {{&S57057[61], 6}, {NULL}};
const Segment SGI161548[] = {{&S57059[34], 21}, {NULL}};
const Segment SGI161549[] = {{&S57059[27], 6}, {NULL}};
const Segment SGI161550[] = {{&S57059[108], 9}, {NULL}};
const Segment SGI161551[] = {{&S57059[81], 6}, {NULL}};
const Segment SGI161552[] = {{&S57061[34], 10}, {NULL}};
const Segment SGI161553[] = {{&S57061[27], 6}, {NULL}};
const Segment SGI161554[] = {{&S57061[97], 9}, {NULL}};
const Segment SGI161555[] = {{&S57061[70], 6}, {NULL}};
const Segment SGI161556[] = {{&S57067[34], 10}, {NULL}};
const Segment SGI161557[] = {{&S57067[27], 6}, {NULL}};
const Segment SGI161558[] = {{&S57067[97], 9}, {NULL}};
const Segment SGI161559[] = {{&S57067[70], 6}, {NULL}};
const Segment SGI161560[] = {{&S57068[34], 10}, {NULL}};
const Segment SGI161561[] = {{&S57068[27], 6}, {NULL}};
const Segment SGI161562[] = {{&S57068[97], 9}, {NULL}};
const Segment SGI161563[] = {{&S57068[70], 6}, {NULL}};
const Segment SGI161564[] = {{&S48142[38], 10}, {NULL}};
const Segment SGI161565[] = {{&S57069[34], 9}, {NULL}};
const Segment SGI161566[] = {{&S48142[31], 6}, {NULL}};
const Segment SGI161567[] = {{&S57069[27], 6}, {NULL}};
const Segment SGI161568[] = {{&S57069[96], 9}, {NULL}};
const Segment SGI161569[] = {{&S57069[69], 6}, {NULL}};
const Segment SGI161570[] = {{&S48143[44], 10}, {NULL}};
const Segment SGI161571[] = {{&S48143[44], 76}, {NULL}};
const Segment SGI161572[] = {{&S48143[30], 6}, {NULL}};
const Segment SGI161573[] = {{&S57070[34], 10}, {NULL}};
const Segment SGI161574[] = {{&S57070[27], 6}, {NULL}};
const Segment SGI161575[] = {{&S57070[97], 9}, {NULL}};
const Segment SGI161576[] = {{&S57070[70], 6}, {NULL}};
const Segment SGI161577[] = {{&S48144[44], 10}, {NULL}};
const Segment SGI161578[] = {{&S48144[44], 22}, {NULL}};
const Segment SGI161579[] = {{&S48144[30], 6}, {NULL}};
const Segment SGI161580[] = {{&S57071[34], 10}, {NULL}};
const Segment SGI161581[] = {{&S57071[27], 6}, {NULL}};
const Segment SGI161582[] = {{&S48144[115], 3}, {NULL}};
const Segment SGI161583[] = {{&S48144[80], 6}, {NULL}};
const Segment SGI161584[] = {{&S57071[97], 9}, {NULL}};
const Segment SGI161585[] = {{&S57071[70], 6}, {NULL}};
const Segment SGI161586[] = {{&S48145[44], 10}, {NULL}};
const Segment SGI161587[] = {{&S48145[44], 22}, {NULL}};
const Segment SGI161588[] = {{&S48145[30], 6}, {NULL}};
const Segment SGI161589[] = {{&S57072[44], 18}, {NULL}};
const Segment SGI161590[] = {{&S57072[44], 30}, {NULL}};
const Segment SGI161591[] = {{&S57072[30], 6}, {NULL}};
const Segment SGI161592[] = {{&S48145[125], 3}, {NULL}};
const Segment SGI161593[] = {{&S48145[90], 6}, {NULL}};
const Segment SGI161594[] = {{&S57072[31], 3}, {NULL}};
const Segment SGI161595[] = {{&S48145[145], 8}, {NULL}};
const Segment SGI161596[] = {{&S57072[88], 6}, {NULL}};
const Segment SGI161597[] = {{&S46756[34], 9}, {NULL}};
const Segment SGI161598[] = {{&S46756[27], 6}, {NULL}};
const Segment SGI161599[] = {{&S46756[96], 9}, {NULL}};
const Segment SGI161600[] = {{&S46756[69], 6}, {NULL}};
const Segment SGI161601[] = {{&S48146[44], 10}, {NULL}};
const Segment SGI161602[] = {{&S48146[44], 22}, {NULL}};
const Segment SGI161603[] = {{&S48146[30], 6}, {NULL}};
const Segment SGI161604[] = {{&S48146[115], 3}, {NULL}};
const Segment SGI161605[] = {{&S48146[80], 6}, {NULL}};
const Segment SGI161606[] = {{&S46757[34], 9}, {NULL}};
const Segment SGI161607[] = {{&S46757[27], 6}, {NULL}};
const Segment SGI161608[] = {{&S46757[96], 9}, {NULL}};
const Segment SGI161609[] = {{&S46757[69], 6}, {NULL}};
const Segment SGI161610[] = {{&S48147[44], 10}, {NULL}};
const Segment SGI161611[] = {{&S48147[44], 22}, {NULL}};
const Segment SGI161612[] = {{&S48147[30], 6}, {NULL}};
const Segment SGI161613[] = {{&S48147[80], 6}, {NULL}};
const Segment SGI161614[] = {{&S46758[34], 9}, {NULL}};
const Segment SGI161615[] = {{&S46758[27], 6}, {NULL}};
const Segment SGI161616[] = {{&S46758[96], 9}, {NULL}};
const Segment SGI161617[] = {{&S46758[69], 6}, {NULL}};
const Segment SGI161618[] = {{&S48148[44], 10}, {NULL}};
const Segment SGI161619[] = {{&S48148[44], 23}, {NULL}};
const Segment SGI161620[] = {{&S48148[30], 6}, {NULL}};
const Segment SGI161621[] = {{&S46759[34], 9}, {NULL}};
const Segment SGI161622[] = {{&S48148[116], 3}, {NULL}};
const Segment SGI161623[] = {{&S46759[27], 6}, {NULL}};
const Segment SGI161624[] = {{&S48148[81], 6}, {NULL}};
const Segment SGI161625[] = {{&S46759[96], 9}, {NULL}};
const Segment SGI161626[] = {{&S46759[69], 6}, {NULL}};
const Segment SGI161627[] = {{&S57081[34], 11}, {NULL}};
const Segment SGI161628[] = {{&S57081[27], 6}, {NULL}};
const Segment SGI161629[] = {{&S57081[98], 9}, {NULL}};
const Segment SGI161630[] = {{&S48151[52], 10}, {NULL}};
const Segment SGI161631[] = {{&S57081[71], 6}, {NULL}};
const Segment SGI161632[] = {{&S48151[52], 22}, {NULL}};
const Segment SGI161633[] = {{&S48151[38], 6}, {NULL}};
const Segment SGI161634[] = {{&S46762[44], 18}, {NULL}};
const Segment SGI161635[] = {{&S46762[44], 30}, {NULL}};
const Segment SGI161636[] = {{&S46762[30], 6}, {NULL}};
const Segment SGI161637[] = {{&S46762[123], 3}, {NULL}};
const Segment SGI161638[] = {{&S46762[88], 6}, {NULL}};
const Segment SGI161639[] = {{&S48152[44], 10}, {NULL}};
const Segment SGI161640[] = {{&S48152[44], 22}, {NULL}};
const Segment SGI161641[] = {{&S48152[30], 6}, {NULL}};
const Segment SGI161642[] = {{&S48152[83], 6}, {NULL}};
const Segment SGI161643[] = {{&S46763[38], 18}, {NULL}};
const Segment SGI161644[] = {{&S46763[31], 6}, {NULL}};
const Segment SGI161645[] = {{&S57089[34], 5}, {NULL}};
const Segment SGI161646[] = {{&S57089[27], 6}, {NULL}};
const Segment SGI161647[] = {{&S57089[92], 9}, {NULL}};
const Segment SGI161648[] = {{&S57089[65], 6}, {NULL}};
const Segment SGI161649[] = {{&S57090[34], 9}, {NULL}};
const Segment SGI161650[] = {{&S57090[27], 6}, {NULL}};
const Segment SGI161651[] = {{&S57090[96], 9}, {NULL}};
const Segment SGI161652[] = {{&S57090[69], 6}, {NULL}};
const Segment SGI161653[] = {{&S57091[34], 5}, {NULL}};
const Segment SGI161654[] = {{&S57091[27], 6}, {NULL}};
const Segment SGI161655[] = {{&S57091[92], 9}, {NULL}};
const Segment SGI161656[] = {{&S57091[65], 6}, {NULL}};
const Segment SGI161657[] = {{&S57093[34], 16}, {NULL}};
const Segment SGI161658[] = {{&S57093[27], 6}, {NULL}};
const Segment SGI161659[] = {{&S57093[103], 9}, {NULL}};
const Segment SGI161660[] = {{&S57093[76], 6}, {NULL}};
const Segment SGI161661[] = {{&S57095[34], 5}, {NULL}};
const Segment SGI161662[] = {{&S57095[27], 6}, {NULL}};
const Segment SGI161663[] = {{&S57095[92], 9}, {NULL}};
const Segment SGI161664[] = {{&S57095[65], 6}, {NULL}};
const Segment SGI161665[] = {{&S46779[34], 10}, {NULL}};
const Segment SGI161666[] = {{&S46779[27], 6}, {NULL}};
const Segment SGI161667[] = {{&S46779[97], 9}, {NULL}};
const Segment SGI161668[] = {{&S46779[70], 6}, {NULL}};
const Segment SGI161669[] = {{&S57096[34], 21}, {NULL}};
const Segment SGI161670[] = {{&S57096[27], 6}, {NULL}};
const Segment SGI161671[] = {{&S57096[108], 9}, {NULL}};
const Segment SGI161672[] = {{&S57096[81], 6}, {NULL}};
const Segment SGI161673[] = {{&S46780[34], 11}, {NULL}};
const Segment SGI161674[] = {{&S46780[27], 6}, {NULL}};
const Segment SGI161675[] = {{&S46780[98], 9}, {NULL}};
const Segment SGI161676[] = {{&S46780[71], 6}, {NULL}};
const Segment SGI161677[] = {{&S57097[34], 10}, {NULL}};
const Segment SGI161678[] = {{&S57097[27], 6}, {NULL}};
const Segment SGI161679[] = {{&S57097[97], 9}, {NULL}};
const Segment SGI161680[] = {{&S57097[70], 6}, {NULL}};
const Segment SGI161681[] = {{&S57098[34], 30}, {NULL}};
const Segment SGI161682[] = {{&S57098[27], 6}, {NULL}};
const Segment SGI161683[] = {{&S57098[117], 9}, {NULL}};
const Segment SGI161684[] = {{&S57098[90], 6}, {NULL}};
const Segment SGI161685[] = {{&S57100[11], 1}, {NULL}};
const Segment SGI161686[] = {{&S57100[27], 6}, {NULL}};
const Segment SGI161687[] = {{&S57100[88], 9}, {NULL}};
const Segment SGI161688[] = {{&S57100[61], 6}, {NULL}};
const Segment SGI161689[] = {{&S57101[11], 1}, {NULL}};
const Segment SGI161690[] = {{&S57101[27], 6}, {NULL}};
const Segment SGI161691[] = {{&S57101[88], 9}, {NULL}};
const Segment SGI161692[] = {{&S57101[61], 6}, {NULL}};
const Segment SGI161693[] = {{&S57102[34], 16}, {NULL}};
const Segment SGI161694[] = {{&S57102[27], 6}, {NULL}};
const Segment SGI161695[] = {{&S57102[103], 9}, {NULL}};
const Segment SGI161696[] = {{&S57102[76], 6}, {NULL}};
const Segment SGI161697[] = {{&S57104[34], 8}, {NULL}};
const Segment SGI161698[] = {{&S57104[27], 6}, {NULL}};
const Segment SGI161699[] = {{&S57104[95], 9}, {NULL}};
const Segment SGI161700[] = {{&S57104[68], 6}, {NULL}};
const Segment SGI161701[] = {{&S57106[34], 13}, {NULL}};
const Segment SGI161702[] = {{&S57106[27], 6}, {NULL}};
const Segment SGI161703[] = {{&S57106[100], 9}, {NULL}};
const Segment SGI161704[] = {{&S57106[73], 6}, {NULL}};
const Segment SGI161705[] = {{&S46798[34], 5}, {NULL}};
const Segment SGI161706[] = {{&S46798[27], 6}, {NULL}};
const Segment SGI161707[] = {{&S46798[92], 9}, {NULL}};
const Segment SGI161708[] = {{&S46798[65], 6}, {NULL}};
const Segment SGI161709[] = {{&S46799[34], 9}, {NULL}};
const Segment SGI161710[] = {{&S46799[27], 6}, {NULL}};
const Segment SGI161711[] = {{&S46799[96], 9}, {NULL}};
const Segment SGI161712[] = {{&S46799[69], 6}, {NULL}};
const Segment SGI161713[] = {{&S46802[44], 7}, {NULL}};
const Segment SGI161714[] = {{&S46802[44], 30}, {NULL}};
const Segment SGI161715[] = {{&S46802[30], 6}, {NULL}};
const Segment SGI161716[] = {{&S46802[123], 3}, {NULL}};
const Segment SGI161717[] = {{&S46802[88], 6}, {NULL}};
const Segment SGI161718[] = {{&S46803[38], 7}, {NULL}};
const Segment SGI161719[] = {{&S46803[31], 6}, {NULL}};
const Segment SGI161720[] = {{&S46804[44], 7}, {NULL}};
const Segment SGI161721[] = {{&S46804[44], 30}, {NULL}};
const Segment SGI161722[] = {{&S46804[30], 6}, {NULL}};
const Segment SGI161723[] = {{&S46804[123], 3}, {NULL}};
const Segment SGI161724[] = {{&S46804[88], 6}, {NULL}};
const Segment SGI161725[] = {{&S46855[34], 10}, {NULL}};
const Segment SGI161726[] = {{&S46855[27], 6}, {NULL}};
const Segment SGI161727[] = {{&S46855[97], 9}, {NULL}};
const Segment SGI161728[] = {{&S46855[70], 6}, {NULL}};
const Segment SGI161729[] = {{&S48248[34], 10}, {NULL}};
const Segment SGI161730[] = {{&S48248[27], 6}, {NULL}};
const Segment SGI161731[] = {{&S48248[97], 9}, {NULL}};
const Segment SGI161732[] = {{&S48248[70], 6}, {NULL}};
const Segment SGI161733[] = {{&S48249[34], 10}, {NULL}};
const Segment SGI161734[] = {{&S48249[27], 6}, {NULL}};
const Segment SGI161735[] = {{&S48249[97], 9}, {NULL}};
const Segment SGI161736[] = {{&S48249[70], 6}, {NULL}};
const Segment SGI161737[] = {{&S46911[34], 10}, {NULL}};
const Segment SGI161738[] = {{&S46911[27], 6}, {NULL}};
const Segment SGI161739[] = {{&S46911[97], 9}, {NULL}};
const Segment SGI161740[] = {{&S46911[70], 6}, {NULL}};
const Segment SGI161741[] = {{&S46919[34], 10}, {NULL}};
const Segment SGI161742[] = {{&S46919[27], 6}, {NULL}};
const Segment SGI161743[] = {{&S46919[97], 9}, {NULL}};
const Segment SGI161744[] = {{&S46919[70], 6}, {NULL}};
const Segment SGI161745[] = {{&S57121[34], 7}, {NULL}};
const Segment SGI161746[] = {{&S57121[27], 6}, {NULL}};
const Segment SGI161747[] = {{&S57121[100], 9}, {NULL}};
const Segment SGI161748[] = {{&S57121[73], 6}, {NULL}};
const Segment SGI161749[] = {{&S57122[34], 7}, {NULL}};
const Segment SGI161750[] = {{&S57122[27], 6}, {NULL}};
const Segment SGI161751[] = {{&S57122[100], 9}, {NULL}};
const Segment SGI161752[] = {{&S57122[73], 6}, {NULL}};
const Segment SGI161753[] = {{&S57123[34], 10}, {NULL}};
const Segment SGI161754[] = {{&S57123[27], 6}, {NULL}};
const Segment SGI161755[] = {{&S57123[97], 9}, {NULL}};
const Segment SGI161756[] = {{&S57123[70], 6}, {NULL}};
const Segment SGI161757[] = {{&S48348[34], 10}, {NULL}};
const Segment SGI161758[] = {{&S48348[27], 6}, {NULL}};
const Segment SGI161759[] = {{&S57151[44], 5}, {NULL}};
const Segment SGI161760[] = {{&S57151[44], 17}, {NULL}};
const Segment SGI161761[] = {{&S57151[30], 6}, {NULL}};
const Segment SGI161762[] = {{&S48348[97], 9}, {NULL}};
const Segment SGI161763[] = {{&S48348[70], 6}, {NULL}};
const Segment SGI161764[] = {{&S57151[31], 3}, {NULL}};
const Segment SGI161765[] = {{&S57151[75], 6}, {NULL}};
const Segment SGI161766[] = {{&S48348[139], 8}, {NULL}};
const Segment SGI161767[] = {{&S57152[44], 5}, {NULL}};
const Segment SGI161768[] = {{&S57152[44], 20}, {NULL}};
const Segment SGI161769[] = {{&S57152[30], 6}, {NULL}};
const Segment SGI161770[] = {{&S57152[31], 3}, {NULL}};
const Segment SGI161771[] = {{&S57152[78], 6}, {NULL}};
const Segment SGI161772[] = {{&S57153[44], 5}, {NULL}};
const Segment SGI161773[] = {{&S57153[44], 17}, {NULL}};
const Segment SGI161774[] = {{&S57153[30], 6}, {NULL}};
const Segment SGI161775[] = {{&S57153[31], 3}, {NULL}};
const Segment SGI161776[] = {{&S57153[75], 6}, {NULL}};
const Segment SGI161777[] = {{&S48353[34], 10}, {NULL}};
const Segment SGI161778[] = {{&S48353[27], 6}, {NULL}};
const Segment SGI161779[] = {{&S48353[97], 9}, {NULL}};
const Segment SGI161780[] = {{&S48353[70], 6}, {NULL}};
const Segment SGI161781[] = {{&S48353[138], 8}, {NULL}};
const Segment SGI161782[] = {{&S48355[34], 10}, {NULL}};
const Segment SGI161783[] = {{&S48355[27], 6}, {NULL}};
const Segment SGI161784[] = {{&S48355[97], 9}, {NULL}};
const Segment SGI161785[] = {{&S48355[70], 6}, {NULL}};
const Segment SGI161786[] = {{&S48355[138], 8}, {NULL}};
const Segment SGI161787[] = {{&S47003[34], 14}, {NULL}};
const Segment SGI161788[] = {{&S48380[34], 18}, {NULL}};
const Segment SGI161789[] = {{&S47003[27], 6}, {NULL}};
const Segment SGI161790[] = {{&S48380[27], 6}, {NULL}};
const Segment SGI161791[] = {{&S47003[101], 9}, {NULL}};
const Segment SGI161792[] = {{&S47003[74], 6}, {NULL}};
const Segment SGI161793[] = {{&S48380[105], 9}, {NULL}};
const Segment SGI161794[] = {{&S48380[78], 6}, {NULL}};
const Segment SGI161795[] = {{&S48380[146], 8}, {NULL}};
const Segment SGI161796[] = {{&S47004[34], 13}, {NULL}};
const Segment SGI161797[] = {{&S47004[27], 6}, {NULL}};
const Segment SGI161798[] = {{&S47004[100], 9}, {NULL}};
const Segment SGI161799[] = {{&S57186[34], 13}, {NULL}};
const Segment SGI161800[] = {{&S47004[73], 6}, {NULL}};
const Segment SGI161801[] = {{&S57186[27], 6}, {NULL}};
const Segment SGI161802[] = {{&S57186[100], 9}, {NULL}};
const Segment SGI161803[] = {{&S57186[73], 6}, {NULL}};
const Segment SGI161804[] = {{&S48382[34], 11}, {NULL}};
const Segment SGI161805[] = {{&S48382[27], 6}, {NULL}};
const Segment SGI161806[] = {{&S47005[34], 10}, {NULL}};
const Segment SGI161807[] = {{&S48382[98], 9}, {NULL}};
const Segment SGI161808[] = {{&S47005[27], 6}, {NULL}};
const Segment SGI161809[] = {{&S48382[71], 6}, {NULL}};
const Segment SGI161810[] = {{&S48382[139], 8}, {NULL}};
const Segment SGI161811[] = {{&S47005[97], 9}, {NULL}};
const Segment SGI161812[] = {{&S57187[34], 6}, {NULL}};
const Segment SGI161813[] = {{&S47005[70], 6}, {NULL}};
const Segment SGI161814[] = {{&S57187[27], 6}, {NULL}};
const Segment SGI161815[] = {{&S57187[93], 9}, {NULL}};
const Segment SGI161816[] = {{&S57187[66], 6}, {NULL}};
const Segment SGI161817[] = {{&S57188[34], 6}, {NULL}};
const Segment SGI161818[] = {{&S57188[27], 6}, {NULL}};
const Segment SGI161819[] = {{&S57188[93], 9}, {NULL}};
const Segment SGI161820[] = {{&S57188[66], 6}, {NULL}};
const Segment SGI161821[] = {{&S48389[34], 10}, {NULL}};
const Segment SGI161822[] = {{&S48389[27], 6}, {NULL}};
const Segment SGI161823[] = {{&S48389[97], 9}, {NULL}};
const Segment SGI161824[] = {{&S48389[70], 6}, {NULL}};
const Segment SGI161825[] = {{&S48389[138], 8}, {NULL}};
const Segment SGI161826[] = {{&S33507[34], 14}, {NULL}};
const Segment SGI161827[] = {{&S33507[27], 6}, {NULL}};
const Segment SGI161828[] = {{&S33507[100], 9}, {NULL}};
const Segment SGI161829[] = {{&S33507[73], 6}, {NULL}};
const Segment SGI161830[] = {{&S33509[34], 14}, {NULL}};
const Segment SGI161831[] = {{&S33509[27], 6}, {NULL}};
const Segment SGI161832[] = {{&S33509[100], 9}, {NULL}};
const Segment SGI161833[] = {{&S33509[73], 6}, {NULL}};
const Segment SGI161834[] = {{&S47026[44], 39}, {NULL}};
const Segment SGI161835[] = {{&S47026[30], 6}, {NULL}};
const Segment SGI161836[] = {{&S47026[115], 6}, {NULL}};
const Segment SGI161837[] = {{&S47028[44], 39}, {NULL}};
const Segment SGI161838[] = {{&S47028[30], 6}, {NULL}};
const Segment SGI161839[] = {{&S47028[115], 6}, {NULL}};
const Segment SGI161840[] = {{&S47031[44], 15}, {NULL}};
const Segment SGI161841[] = {{&S47031[44], 33}, {NULL}};
const Segment SGI161842[] = {{&S47031[30], 6}, {NULL}};
const Segment SGI161843[] = {{&S47031[126], 3}, {NULL}};
const Segment SGI161844[] = {{&S47031[91], 6}, {NULL}};
const Segment SGI161845[] = {{&S57230[34], 10}, {NULL}};
const Segment SGI161846[] = {{&S57230[27], 6}, {NULL}};
const Segment SGI161847[] = {{&S47032[44], 39}, {NULL}};
const Segment SGI161848[] = {{&S57230[97], 9}, {NULL}};
const Segment SGI161849[] = {{&S57230[70], 6}, {NULL}};
const Segment SGI161850[] = {{&S47032[30], 6}, {NULL}};
const Segment SGI161851[] = {{&S47032[115], 6}, {NULL}};
const Segment SGI161852[] = {{&S48427[44], 11}, {NULL}};
const Segment SGI161853[] = {{&S48427[44], 23}, {NULL}};
const Segment SGI161854[] = {{&S48427[30], 6}, {NULL}};
const Segment SGI161855[] = {{&S57232[34], 11}, {NULL}};
const Segment SGI161856[] = {{&S57232[27], 6}, {NULL}};
const Segment SGI161857[] = {{&S48427[116], 3}, {NULL}};
const Segment SGI161858[] = {{&S48427[81], 6}, {NULL}};
const Segment SGI161859[] = {{&S57232[98], 9}, {NULL}};
const Segment SGI161860[] = {{&S57232[71], 6}, {NULL}};
const Segment SGI161861[] = {{&S47034[44], 15}, {NULL}};
const Segment SGI161862[] = {{&S47034[44], 33}, {NULL}};
const Segment SGI161863[] = {{&S47034[30], 6}, {NULL}};
const Segment SGI161864[] = {{&S47034[94], 6}, {NULL}};
const Segment SGI161865[] = {{&S47035[38], 15}, {NULL}};
const Segment SGI161866[] = {{&S47035[31], 6}, {NULL}};
const Segment SGI161867[] = {{&S47036[44], 39}, {NULL}};
const Segment SGI161868[] = {{&S47036[30], 6}, {NULL}};
const Segment SGI161869[] = {{&S47036[115], 6}, {NULL}};
const Segment SGI161870[] = {{&S47037[44], 39}, {NULL}};
const Segment SGI161871[] = {{&S47037[30], 6}, {NULL}};
const Segment SGI161872[] = {{&S47037[115], 6}, {NULL}};
const Segment SGI161873[] = {{&S47038[44], 33}, {NULL}};
const Segment SGI161874[] = {{&S47038[30], 6}, {NULL}};
const Segment SGI161875[] = {{&S57242[6], 4}, {NULL}};
const Segment SGI161876[] = {{&S57242[76], 5}, {NULL}};
const Segment SGI161877[] = {{&S47046[34], 11}, {NULL}};
const Segment SGI161878[] = {{&S47046[27], 6}, {NULL}};
const Segment SGI161879[] = {{&S47046[98], 9}, {NULL}};
const Segment SGI161880[] = {{&S47046[71], 6}, {NULL}};
const Segment SGI161881[] = {{&S57277[34], 7}, {NULL}};
const Segment SGI161882[] = {{&S57277[27], 6}, {NULL}};
const Segment SGI161883[] = {{&S57277[105], 9}, {NULL}};
const Segment SGI161884[] = {{&S57277[78], 6}, {NULL}};
const Segment SGI161885[] = {{&S57281[34], 18}, {NULL}};
const Segment SGI161886[] = {{&S57281[27], 6}, {NULL}};
const Segment SGI161887[] = {{&S57281[114], 9}, {NULL}};
const Segment SGI161888[] = {{&S57281[87], 6}, {NULL}};
const Segment SGI161889[] = {{&S57282[34], 16}, {NULL}};
const Segment SGI161890[] = {{&S57282[27], 6}, {NULL}};
const Segment SGI161891[] = {{&S57282[110], 9}, {NULL}};
const Segment SGI161892[] = {{&S57282[83], 6}, {NULL}};
const Segment SGI161893[] = {{&S57283[34], 15}, {NULL}};
const Segment SGI161894[] = {{&S57283[27], 6}, {NULL}};
const Segment SGI161895[] = {{&S57283[108], 9}, {NULL}};
const Segment SGI161896[] = {{&S57283[81], 6}, {NULL}};
const Segment SGI161897[] = {{&S57284[34], 17}, {NULL}};
const Segment SGI161898[] = {{&S57284[27], 6}, {NULL}};
const Segment SGI161899[] = {{&S57284[112], 9}, {NULL}};
const Segment SGI161900[] = {{&S57284[85], 6}, {NULL}};
const Segment SGI161901[] = {{&S57285[34], 17}, {NULL}};
const Segment SGI161902[] = {{&S57285[27], 6}, {NULL}};
const Segment SGI161903[] = {{&S57285[112], 9}, {NULL}};
const Segment SGI161904[] = {{&S57285[85], 6}, {NULL}};
const Segment SGI161905[] = {{&S48538[34], 11}, {NULL}};
const Segment SGI161906[] = {{&S48538[27], 6}, {NULL}};
const Segment SGI161907[] = {{&S48538[98], 9}, {NULL}};
const Segment SGI161908[] = {{&S48538[71], 6}, {NULL}};
const Segment SGI161909[] = {{&S48543[34], 6}, {NULL}};
const Segment SGI161910[] = {{&S48543[27], 6}, {NULL}};
const Segment SGI161911[] = {{&S57296[34], 7}, {NULL}};
const Segment SGI161912[] = {{&S57296[27], 6}, {NULL}};
const Segment SGI161913[] = {{&S48543[93], 9}, {NULL}};
const Segment SGI161914[] = {{&S48543[66], 6}, {NULL}};
const Segment SGI161915[] = {{&S57296[93], 9}, {NULL}};
const Segment SGI161916[] = {{&S57296[66], 6}, {NULL}};
const Segment SGI161917[] = {{&S48545[34], 15}, {NULL}};
const Segment SGI161918[] = {{&S48545[27], 6}, {NULL}};
const Segment SGI161919[] = {{&S48545[102], 9}, {NULL}};
const Segment SGI161920[] = {{&S48545[75], 6}, {NULL}};
const Segment SGI161921[] = {{&S47146[34], 10}, {NULL}};
const Segment SGI161922[] = {{&S47146[27], 6}, {NULL}};
const Segment SGI161923[] = {{&S57299[44], 10}, {NULL}};
const Segment SGI161924[] = {{&S57299[44], 22}, {NULL}};
const Segment SGI161925[] = {{&S57299[30], 6}, {NULL}};
const Segment SGI161926[] = {{&S47146[97], 9}, {NULL}};
const Segment SGI161927[] = {{&S47146[70], 6}, {NULL}};
const Segment SGI161928[] = {{&S57299[31], 3}, {NULL}};
const Segment SGI161929[] = {{&S57299[95], 6}, {NULL}};
const Segment SGI161930[] = {{&S57299[68], 8}, {NULL}};
const Segment SGI161931[] = {{&S57300[44], 10}, {NULL}};
const Segment SGI161932[] = {{&S57300[44], 22}, {NULL}};
const Segment SGI161933[] = {{&S57300[30], 6}, {NULL}};
const Segment SGI161934[] = {{&S57300[31], 3}, {NULL}};
const Segment SGI161935[] = {{&S57300[108], 6}, {NULL}};
const Segment SGI161936[] = {{&S57300[68], 8}, {NULL}};
const Segment SGI161937[] = {{&S47149[34], 7}, {NULL}};
const Segment SGI161938[] = {{&S47149[27], 6}, {NULL}};
const Segment SGI161939[] = {{&S47149[109], 9}, {NULL}};
const Segment SGI161940[] = {{&S47149[82], 6}, {NULL}};
const Segment SGI161941[] = {{&S57301[44], 10}, {NULL}};
const Segment SGI161942[] = {{&S57301[44], 22}, {NULL}};
const Segment SGI161943[] = {{&S57301[30], 6}, {NULL}};
const Segment SGI161944[] = {{&S57301[31], 3}, {NULL}};
const Segment SGI161945[] = {{&S57301[95], 6}, {NULL}};
const Segment SGI161946[] = {{&S57301[68], 8}, {NULL}};
const Segment SGI161947[] = {{&S57302[44], 17}, {NULL}};
const Segment SGI161948[] = {{&S57302[44], 36}, {NULL}};
const Segment SGI161949[] = {{&S57302[30], 6}, {NULL}};
const Segment SGI161950[] = {{&S57302[31], 3}, {NULL}};
const Segment SGI161951[] = {{&S57302[94], 6}, {NULL}};
const Segment SGI161952[] = {{&S57303[44], 17}, {NULL}};
const Segment SGI161953[] = {{&S57303[44], 36}, {NULL}};
const Segment SGI161954[] = {{&S57303[30], 6}, {NULL}};
const Segment SGI161955[] = {{&S57303[31], 3}, {NULL}};
const Segment SGI161956[] = {{&S57303[94], 6}, {NULL}};
const Segment SGI161957[] = {{&S57304[44], 17}, {NULL}};
const Segment SGI161958[] = {{&S57304[44], 36}, {NULL}};
const Segment SGI161959[] = {{&S57304[30], 6}, {NULL}};
const Segment SGI161960[] = {{&S57304[31], 3}, {NULL}};
const Segment SGI161961[] = {{&S57304[94], 6}, {NULL}};
const Segment SGI161962[] = {{&S57305[34], 7}, {NULL}};
const Segment SGI161963[] = {{&S57305[27], 6}, {NULL}};
const Segment SGI161964[] = {{&S57305[106], 9}, {NULL}};
const Segment SGI161965[] = {{&S57305[79], 6}, {NULL}};
const Segment SGI161966[] = {{&S57306[34], 7}, {NULL}};
const Segment SGI161967[] = {{&S57306[27], 6}, {NULL}};
const Segment SGI161968[] = {{&S57306[106], 9}, {NULL}};
const Segment SGI161969[] = {{&S57306[79], 6}, {NULL}};
const Segment SGI161970[] = {{&S47169[34], 7}, {NULL}};
const Segment SGI161971[] = {{&S47169[27], 6}, {NULL}};
const Segment SGI161972[] = {{&S47169[99], 9}, {NULL}};
const Segment SGI161973[] = {{&S47169[72], 6}, {NULL}};
const Segment SGI161974[] = {{&S33698[34], 9}, {NULL}};
const Segment SGI161975[] = {{&S33698[27], 6}, {NULL}};
const Segment SGI161976[] = {{&S33698[95], 9}, {NULL}};
const Segment SGI161977[] = {{&S33698[68], 6}, {NULL}};
const Segment SGI161978[] = {{&S46005[32], 10}, {NULL}};
const Segment SGI161979[] = {{&S46005[27], 4}, {NULL}};
const Segment SGI161980[] = {{&S46005[95], 9}, {NULL}};
const Segment SGI161981[] = {{&S46005[68], 6}, {NULL}};
const Segment SGI161982[] = {{&S47198[34], 8}, {NULL}};
const Segment SGI161983[] = {{&S47198[27], 6}, {NULL}};
const Segment SGI161984[] = {{&S47198[105], 9}, {NULL}};
const Segment SGI161985[] = {{&S47198[78], 6}, {NULL}};
const Segment SGI161986[] = {{&S33727[44], 10}, {NULL}};
const Segment SGI161987[] = {{&S33727[44], 21}, {NULL}};
const Segment SGI161988[] = {{&S33727[30], 6}, {NULL}};
const Segment SGI161989[] = {{&S33727[114], 3}, {NULL}};
const Segment SGI161990[] = {{&S33727[79], 6}, {NULL}};
const Segment SGI161991[] = {{&S33728[38], 10}, {NULL}};
const Segment SGI161992[] = {{&S33728[31], 6}, {NULL}};
const Segment SGI161993[] = {{&S33729[44], 10}, {NULL}};
const Segment SGI161994[] = {{&S33729[44], 22}, {NULL}};
const Segment SGI161995[] = {{&S33729[30], 6}, {NULL}};
const Segment SGI161996[] = {{&S33729[115], 3}, {NULL}};
const Segment SGI161997[] = {{&S33729[80], 6}, {NULL}};
const Segment SGI161998[] = {{&S48614[44], 14}, {NULL}};
const Segment SGI161999[] = {{&S48614[44], 26}, {NULL}};
const Segment SGI162000[] = {{&S48614[30], 6}, {NULL}};
const Segment SGI162001[] = {{&S33730[42], 10}, {NULL}};
const Segment SGI162002[] = {{&S33730[30], 6}, {NULL}};
const Segment SGI162003[] = {{&S48614[31], 3}, {NULL}};
const Segment SGI162004[] = {{&S33730[112], 3}, {NULL}};
const Segment SGI162005[] = {{&S48614[84], 6}, {NULL}};
const Segment SGI162006[] = {{&S33730[77], 6}, {NULL}};
const Segment SGI162007[] = {{&S33731[44], 11}, {NULL}};
const Segment SGI162008[] = {{&S33731[44], 22}, {NULL}};
const Segment SGI162009[] = {{&S33731[30], 6}, {NULL}};
const Segment SGI162010[] = {{&S33731[115], 3}, {NULL}};
const Segment SGI162011[] = {{&S33731[80], 6}, {NULL}};
const Segment SGI162012[] = {{&S33732[38], 11}, {NULL}};
const Segment SGI162013[] = {{&S33732[31], 6}, {NULL}};
const Segment SGI162014[] = {{&S33733[44], 10}, {NULL}};
const Segment SGI162015[] = {{&S33733[44], 22}, {NULL}};
const Segment SGI162016[] = {{&S33733[30], 6}, {NULL}};
const Segment SGI162017[] = {{&S33733[115], 3}, {NULL}};
const Segment SGI162018[] = {{&S33733[80], 6}, {NULL}};
const Segment SGI162019[] = {{&S48627[44], 9}, {NULL}};
const Segment SGI162020[] = {{&S48627[44], 21}, {NULL}};
const Segment SGI162021[] = {{&S48627[30], 6}, {NULL}};
const Segment SGI162022[] = {{&S48627[130], 3}, {NULL}};
const Segment SGI162023[] = {{&S48627[95], 6}, {NULL}};
const Segment SGI162024[] = {{&S48627[67], 8}, {NULL}};
const Segment SGI162025[] = {{&S48628[44], 9}, {NULL}};
const Segment SGI162026[] = {{&S48628[44], 21}, {NULL}};
const Segment SGI162027[] = {{&S48628[30], 6}, {NULL}};
const Segment SGI162028[] = {{&S48628[134], 3}, {NULL}};
const Segment SGI162029[] = {{&S48628[99], 6}, {NULL}};
const Segment SGI162030[] = {{&S48628[67], 8}, {NULL}};
const Segment SGI162031[] = {{&S48630[44], 9}, {NULL}};
const Segment SGI162032[] = {{&S48630[44], 21}, {NULL}};
const Segment SGI162033[] = {{&S48630[30], 6}, {NULL}};
const Segment SGI162034[] = {{&S48630[134], 3}, {NULL}};
const Segment SGI162035[] = {{&S48630[99], 6}, {NULL}};
const Segment SGI162036[] = {{&S48630[67], 8}, {NULL}};
const Segment SGI162037[] = {{&S48631[44], 9}, {NULL}};
const Segment SGI162038[] = {{&S48631[44], 21}, {NULL}};
const Segment SGI162039[] = {{&S48631[30], 6}, {NULL}};
const Segment SGI162040[] = {{&S48631[148], 3}, {NULL}};
const Segment SGI162041[] = {{&S48631[113], 6}, {NULL}};
const Segment SGI162042[] = {{&S48631[67], 8}, {NULL}};
const Segment SGI162043[] = {{&S57379[34], 21}, {NULL}};
const Segment SGI162044[] = {{&S57379[27], 6}, {NULL}};
const Segment SGI162045[] = {{&S57379[108], 9}, {NULL}};
const Segment SGI162046[] = {{&S57379[81], 6}, {NULL}};
const Segment SGI162047[] = {{&S57381[44], 7}, {NULL}};
const Segment SGI162048[] = {{&S57381[44], 19}, {NULL}};
const Segment SGI162049[] = {{&S57381[30], 6}, {NULL}};
const Segment SGI162050[] = {{&S57381[31], 3}, {NULL}};
const Segment SGI162051[] = {{&S57381[77], 6}, {NULL}};
const Segment SGI162052[] = {{&S57382[38], 7}, {NULL}};
const Segment SGI162053[] = {{&S57382[31], 6}, {NULL}};
const Segment SGI162054[] = {{&S47246[44], 11}, {NULL}};
const Segment SGI162055[] = {{&S47246[44], 23}, {NULL}};
const Segment SGI162056[] = {{&S47246[30], 6}, {NULL}};
const Segment SGI162057[] = {{&S47246[116], 3}, {NULL}};
const Segment SGI162058[] = {{&S47246[81], 6}, {NULL}};
const Segment SGI162059[] = {{&S57390[44], 19}, {NULL}};
const Segment SGI162060[] = {{&S57390[44], 31}, {NULL}};
const Segment SGI162061[] = {{&S57390[30], 6}, {NULL}};
const Segment SGI162062[] = {{&S47247[38], 11}, {NULL}};
const Segment SGI162063[] = {{&S57390[31], 3}, {NULL}};
const Segment SGI162064[] = {{&S47247[31], 6}, {NULL}};
const Segment SGI162065[] = {{&S57390[89], 6}, {NULL}};
const Segment SGI162066[] = {{&S57391[44], 21}, {NULL}};
const Segment SGI162067[] = {{&S57391[44], 33}, {NULL}};
const Segment SGI162068[] = {{&S57391[30], 6}, {NULL}};
const Segment SGI162069[] = {{&S57391[31], 3}, {NULL}};
const Segment SGI162070[] = {{&S57391[91], 6}, {NULL}};
const Segment SGI162071[] = {{&S47249[34], 10}, {NULL}};
const Segment SGI162072[] = {{&S47249[27], 6}, {NULL}};
const Segment SGI162073[] = {{&S47249[97], 9}, {NULL}};
const Segment SGI162074[] = {{&S47249[70], 6}, {NULL}};
const Segment SGI162075[] = {{&S57393[44], 20}, {NULL}};
const Segment SGI162076[] = {{&S57393[44], 32}, {NULL}};
const Segment SGI162077[] = {{&S57393[30], 6}, {NULL}};
const Segment SGI162078[] = {{&S47251[34], 10}, {NULL}};
const Segment SGI162079[] = {{&S57393[31], 3}, {NULL}};
const Segment SGI162080[] = {{&S47251[27], 6}, {NULL}};
const Segment SGI162081[] = {{&S57393[90], 6}, {NULL}};
const Segment SGI162082[] = {{&S47251[97], 9}, {NULL}};
const Segment SGI162083[] = {{&S47251[70], 6}, {NULL}};
const Segment SGI162084[] = {{&S57395[44], 19}, {NULL}};
const Segment SGI162085[] = {{&S57395[44], 42}, {NULL}};
const Segment SGI162086[] = {{&S57395[30], 6}, {NULL}};
const Segment SGI162087[] = {{&S57395[31], 3}, {NULL}};
const Segment SGI162088[] = {{&S57395[100], 6}, {NULL}};
const Segment SGI162089[] = {{&S57396[34], 7}, {NULL}};
const Segment SGI162090[] = {{&S57396[27], 6}, {NULL}};
const Segment SGI162091[] = {{&S48659[34], 7}, {NULL}};
const Segment SGI162092[] = {{&S47256[34], 10}, {NULL}};
const Segment SGI162093[] = {{&S48659[27], 6}, {NULL}};
const Segment SGI162094[] = {{&S47256[27], 6}, {NULL}};
const Segment SGI162095[] = {{&S57396[94], 9}, {NULL}};
const Segment SGI162096[] = {{&S57396[67], 6}, {NULL}};
const Segment SGI162097[] = {{&S48659[99], 9}, {NULL}};
const Segment SGI162098[] = {{&S47256[97], 9}, {NULL}};
const Segment SGI162099[] = {{&S48659[72], 6}, {NULL}};
const Segment SGI162100[] = {{&S47256[70], 6}, {NULL}};
const Segment SGI162101[] = {{&S57397[34], 7}, {NULL}};
const Segment SGI162102[] = {{&S47257[34], 11}, {NULL}};
const Segment SGI162103[] = {{&S57397[27], 6}, {NULL}};
const Segment SGI162104[] = {{&S47257[27], 6}, {NULL}};
const Segment SGI162105[] = {{&S57397[94], 9}, {NULL}};
const Segment SGI162106[] = {{&S47257[98], 9}, {NULL}};
const Segment SGI162107[] = {{&S57397[67], 6}, {NULL}};
const Segment SGI162108[] = {{&S47257[71], 6}, {NULL}};
const Segment SGI162109[] = {{&S57398[34], 9}, {NULL}};
const Segment SGI162110[] = {{&S57398[27], 6}, {NULL}};
const Segment SGI162111[] = {{&S57398[96], 9}, {NULL}};
const Segment SGI162112[] = {{&S57398[69], 6}, {NULL}};
const Segment SGI162113[] = {{&S57399[34], 10}, {NULL}};
const Segment SGI162114[] = {{&S57399[27], 6}, {NULL}};
const Segment SGI162115[] = {{&S57399[97], 9}, {NULL}};
const Segment SGI162116[] = {{&S57399[70], 6}, {NULL}};
const Segment SGI162117[] = {{&S57400[34], 10}, {NULL}};
const Segment SGI162118[] = {{&S57400[27], 6}, {NULL}};
const Segment SGI162119[] = {{&S57400[97], 9}, {NULL}};
const Segment SGI162120[] = {{&S57400[70], 6}, {NULL}};
const Segment SGI162121[] = {{&S57401[34], 10}, {NULL}};
const Segment SGI162122[] = {{&S57401[27], 6}, {NULL}};
const Segment SGI162123[] = {{&S57401[97], 9}, {NULL}};
const Segment SGI162124[] = {{&S57401[70], 6}, {NULL}};
const Segment SGI162125[] = {{&S57415[34], 9}, {NULL}};
const Segment SGI162126[] = {{&S57415[27], 6}, {NULL}};
const Segment SGI162127[] = {{&S57415[96], 9}, {NULL}};
const Segment SGI162128[] = {{&S57415[69], 6}, {NULL}};
const Segment SGI162129[] = {{&S57416[34], 9}, {NULL}};
const Segment SGI162130[] = {{&S57416[27], 6}, {NULL}};
const Segment SGI162131[] = {{&S57416[96], 9}, {NULL}};
const Segment SGI162132[] = {{&S57416[69], 6}, {NULL}};
const Segment SGI162133[] = {{&S47303[34], 24}, {NULL}};
const Segment SGI162134[] = {{&S47303[27], 6}, {NULL}};
const Segment SGI162135[] = {{&S47303[111], 9}, {NULL}};
const Segment SGI162136[] = {{&S47303[84], 6}, {NULL}};
const Segment SGI162137[] = {{&S48704[34], 12}, {NULL}};
const Segment SGI162138[] = {{&S48704[27], 6}, {NULL}};
const Segment SGI162139[] = {{&S48704[99], 9}, {NULL}};
const Segment SGI162140[] = {{&S48704[72], 6}, {NULL}};
const Segment SGI162141[] = {{&S48705[34], 14}, {NULL}};
const Segment SGI162142[] = {{&S57432[34], 21}, {NULL}};
const Segment SGI162143[] = {{&S48705[27], 6}, {NULL}};
const Segment SGI162144[] = {{&S57432[27], 6}, {NULL}};
const Segment SGI162145[] = {{&S48705[101], 9}, {NULL}};
const Segment SGI162146[] = {{&S48705[74], 6}, {NULL}};
const Segment SGI162147[] = {{&S57432[108], 9}, {NULL}};
const Segment SGI162148[] = {{&S57432[81], 6}, {NULL}};
const Segment SGI162149[] = {{&S48706[34], 12}, {NULL}};
const Segment SGI162150[] = {{&S48706[27], 6}, {NULL}};
const Segment SGI162151[] = {{&S57433[34], 20}, {NULL}};
const Segment SGI162152[] = {{&S57433[27], 6}, {NULL}};
const Segment SGI162153[] = {{&S48706[99], 9}, {NULL}};
const Segment SGI162154[] = {{&S48706[72], 6}, {NULL}};
const Segment SGI162155[] = {{&S57433[107], 9}, {NULL}};
const Segment SGI162156[] = {{&S57433[80], 6}, {NULL}};
const Segment SGI162157[] = {{&S52778[32], 12}, {NULL}};
const Segment SGI162158[] = {{&S52778[27], 4}, {NULL}};
const Segment SGI162159[] = {{&S57434[34], 13}, {NULL}};
const Segment SGI162160[] = {{&S57434[27], 6}, {NULL}};
const Segment SGI162161[] = {{&S52778[97], 9}, {NULL}};
const Segment SGI162162[] = {{&S52778[70], 6}, {NULL}};
const Segment SGI162163[] = {{&S57434[100], 9}, {NULL}};
const Segment SGI162164[] = {{&S57434[73], 6}, {NULL}};
const Segment SGI162165[] = {{&S57436[41], 20}, {NULL}};
const Segment SGI162166[] = {{&S57436[27], 13}, {NULL}};
const Segment SGI162167[] = {{&S57436[114], 9}, {NULL}};
const Segment SGI162168[] = {{&S57436[87], 6}, {NULL}};
const Segment SGI162169[] = {{&S57437[34], 20}, {NULL}};
const Segment SGI162170[] = {{&S57437[27], 6}, {NULL}};
const Segment SGI162171[] = {{&S57437[107], 9}, {NULL}};
const Segment SGI162172[] = {{&S57437[80], 6}, {NULL}};
const Segment SGI162173[] = {{&S57439[34], 7}, {NULL}};
const Segment SGI162174[] = {{&S57439[27], 6}, {NULL}};
const Segment SGI162175[] = {{&S57439[94], 9}, {NULL}};
const Segment SGI162176[] = {{&S57439[67], 6}, {NULL}};
const Segment SGI162177[] = {{&S57450[44], 11}, {NULL}};
const Segment SGI162178[] = {{&S57450[44], 23}, {NULL}};
const Segment SGI162179[] = {{&S57450[30], 6}, {NULL}};
const Segment SGI162180[] = {{&S57450[31], 3}, {NULL}};
const Segment SGI162181[] = {{&S57450[81], 6}, {NULL}};
const Segment SGI162182[] = {{&S57451[44], 11}, {NULL}};
const Segment SGI162183[] = {{&S57451[44], 23}, {NULL}};
const Segment SGI162184[] = {{&S57451[30], 6}, {NULL}};
const Segment SGI162185[] = {{&S57451[31], 3}, {NULL}};
const Segment SGI162186[] = {{&S57451[81], 6}, {NULL}};
const Segment SGI162187[] = {{&S57452[44], 11}, {NULL}};
const Segment SGI162188[] = {{&S57452[44], 23}, {NULL}};
const Segment SGI162189[] = {{&S57452[30], 6}, {NULL}};
const Segment SGI162190[] = {{&S57452[31], 3}, {NULL}};
const Segment SGI162191[] = {{&S57452[81], 6}, {NULL}};
const Segment SGI162192[] = {{&S57453[38], 11}, {NULL}};
const Segment SGI162193[] = {{&S57453[31], 6}, {NULL}};
const Segment SGI162194[] = {{&S57454[34], 10}, {NULL}};
const Segment SGI162195[] = {{&S57454[27], 6}, {NULL}};
const Segment SGI162196[] = {{&S57454[97], 9}, {NULL}};
const Segment SGI162197[] = {{&S57454[70], 6}, {NULL}};
const Segment SGI162198[] = {{&S47361[34], 18}, {NULL}};
const Segment SGI162199[] = {{&S47361[27], 6}, {NULL}};
const Segment SGI162200[] = {{&S47361[105], 9}, {NULL}};
const Segment SGI162201[] = {{&S47361[78], 6}, {NULL}};
const Segment SGI162202[] = {{&S47361[146], 8}, {NULL}};
const Segment SGI162203[] = {{&S47363[34], 10}, {NULL}};
const Segment SGI162204[] = {{&S47363[27], 6}, {NULL}};
const Segment SGI162205[] = {{&S47363[97], 9}, {NULL}};
const Segment SGI162206[] = {{&S47363[70], 6}, {NULL}};
const Segment SGI162207[] = {{&S47363[138], 8}, {NULL}};
const Segment SGI162208[] = {{&S57472[34], 12}, {NULL}};
const Segment SGI162209[] = {{&S57472[27], 6}, {NULL}};
const Segment SGI162210[] = {{&S57472[99], 9}, {NULL}};
const Segment SGI162211[] = {{&S57472[72], 6}, {NULL}};
const Segment SGI162212[] = {{&S47364[34], 10}, {NULL}};
const Segment SGI162213[] = {{&S47364[27], 6}, {NULL}};
const Segment SGI162214[] = {{&S47364[97], 9}, {NULL}};
const Segment SGI162215[] = {{&S47364[70], 6}, {NULL}};
const Segment SGI162216[] = {{&S47364[138], 8}, {NULL}};
const Segment SGI162217[] = {{&S47365[34], 10}, {NULL}};
const Segment SGI162218[] = {{&S47365[27], 6}, {NULL}};
const Segment SGI162219[] = {{&S47365[97], 9}, {NULL}};
const Segment SGI162220[] = {{&S47365[70], 6}, {NULL}};
const Segment SGI162221[] = {{&S47365[138], 8}, {NULL}};
const Segment SGI162222[] = {{&S57475[34], 12}, {NULL}};
const Segment SGI162223[] = {{&S57475[27], 6}, {NULL}};
const Segment SGI162224[] = {{&S57475[99], 9}, {NULL}};
const Segment SGI162225[] = {{&S57475[72], 6}, {NULL}};
const Segment SGI162226[] = {{&S47367[34], 11}, {NULL}};
const Segment SGI162227[] = {{&S47367[27], 6}, {NULL}};
const Segment SGI162228[] = {{&S47367[98], 9}, {NULL}};
const Segment SGI162229[] = {{&S47367[71], 6}, {NULL}};
const Segment SGI162230[] = {{&S47367[139], 8}, {NULL}};
const Segment SGI162231[] = {{&S47368[34], 10}, {NULL}};
const Segment SGI162232[] = {{&S47368[27], 6}, {NULL}};
const Segment SGI162233[] = {{&S47368[97], 9}, {NULL}};
const Segment SGI162234[] = {{&S47368[70], 6}, {NULL}};
const Segment SGI162235[] = {{&S47368[138], 8}, {NULL}};
const Segment SGI162236[] = {{&S47369[34], 11}, {NULL}};
const Segment SGI162237[] = {{&S47369[27], 6}, {NULL}};
const Segment SGI162238[] = {{&S47369[98], 9}, {NULL}};
const Segment SGI162239[] = {{&S47369[71], 6}, {NULL}};
const Segment SGI162240[] = {{&S47369[139], 8}, {NULL}};
const Segment SGI162241[] = {{&S57484[44], 10}, {NULL}};
const Segment SGI162242[] = {{&S57484[44], 22}, {NULL}};
const Segment SGI162243[] = {{&S57484[30], 6}, {NULL}};
const Segment SGI162244[] = {{&S57484[31], 3}, {NULL}};
const Segment SGI162245[] = {{&S57484[80], 6}, {NULL}};
const Segment SGI162246[] = {{&S57484[150], 3}, {NULL}};
const Segment SGI162247[] = {{&S57485[44], 10}, {NULL}};
const Segment SGI162248[] = {{&S57485[44], 22}, {NULL}};
const Segment SGI162249[] = {{&S57485[30], 6}, {NULL}};
const Segment SGI162250[] = {{&S57485[31], 3}, {NULL}};
const Segment SGI162251[] = {{&S57485[80], 6}, {NULL}};
const Segment SGI162252[] = {{&S57485[150], 3}, {NULL}};
const Segment SGI162253[] = {{&S47378[34], 11}, {NULL}};
const Segment SGI162254[] = {{&S47378[27], 6}, {NULL}};
const Segment SGI162255[] = {{&S57486[44], 10}, {NULL}};
const Segment SGI162256[] = {{&S57486[44], 22}, {NULL}};
const Segment SGI162257[] = {{&S57486[30], 6}, {NULL}};
const Segment SGI162258[] = {{&S47378[98], 9}, {NULL}};
const Segment SGI162259[] = {{&S47378[71], 6}, {NULL}};
const Segment SGI162260[] = {{&S57486[31], 3}, {NULL}};
const Segment SGI162261[] = {{&S57486[80], 6}, {NULL}};
const Segment SGI162262[] = {{&S57486[150], 3}, {NULL}};
const Segment SGI162263[] = {{&S47396[34], 10}, {NULL}};
const Segment SGI162264[] = {{&S47396[27], 6}, {NULL}};
const Segment SGI162265[] = {{&S47396[97], 9}, {NULL}};
const Segment SGI162266[] = {{&S47396[70], 6}, {NULL}};
const Segment SGI162267[] = {{&S47397[34], 10}, {NULL}};
const Segment SGI162268[] = {{&S47397[27], 6}, {NULL}};
const Segment SGI162269[] = {{&S47397[97], 9}, {NULL}};
const Segment SGI162270[] = {{&S47397[70], 6}, {NULL}};
const Segment SGI162271[] = {{&S47399[34], 11}, {NULL}};
const Segment SGI162272[] = {{&S47399[27], 6}, {NULL}};
const Segment SGI162273[] = {{&S47399[98], 9}, {NULL}};
const Segment SGI162274[] = {{&S47399[71], 6}, {NULL}};
const Segment SGI162275[] = {{&S47400[34], 8}, {NULL}};
const Segment SGI162276[] = {{&S47400[27], 6}, {NULL}};
const Segment SGI162277[] = {{&S47400[95], 9}, {NULL}};
const Segment SGI162278[] = {{&S47400[68], 6}, {NULL}};
const Segment SGI162279[] = {{&S57511[34], 20}, {NULL}};
const Segment SGI162280[] = {{&S57511[27], 6}, {NULL}};
const Segment SGI162281[] = {{&S47401[34], 11}, {NULL}};
const Segment SGI162282[] = {{&S47401[27], 6}, {NULL}};
const Segment SGI162283[] = {{&S57511[123], 9}, {NULL}};
const Segment SGI162284[] = {{&S57511[96], 6}, {NULL}};
const Segment SGI162285[] = {{&S47401[98], 9}, {NULL}};
const Segment SGI162286[] = {{&S47401[71], 6}, {NULL}};
const Segment SGI162287[] = {{&S47402[34], 11}, {NULL}};
const Segment SGI162288[] = {{&S47402[27], 6}, {NULL}};
const Segment SGI162289[] = {{&S47402[98], 9}, {NULL}};
const Segment SGI162290[] = {{&S47402[71], 6}, {NULL}};
const Segment SGI162291[] = {{&S57513[34], 7}, {NULL}};
const Segment SGI162292[] = {{&S57513[27], 6}, {NULL}};
const Segment SGI162293[] = {{&S57513[96], 9}, {NULL}};
const Segment SGI162294[] = {{&S57513[69], 6}, {NULL}};
const Segment SGI162295[] = {{&S47403[34], 11}, {NULL}};
const Segment SGI162296[] = {{&S47403[27], 6}, {NULL}};
const Segment SGI162297[] = {{&S47403[98], 9}, {NULL}};
const Segment SGI162298[] = {{&S47403[71], 6}, {NULL}};
const Segment SGI162299[] = {{&S47404[34], 11}, {NULL}};
const Segment SGI162300[] = {{&S47404[27], 6}, {NULL}};
const Segment SGI162301[] = {{&S47404[98], 9}, {NULL}};
const Segment SGI162302[] = {{&S47404[71], 6}, {NULL}};
const Segment SGI162303[] = {{&S47405[34], 11}, {NULL}};
const Segment SGI162304[] = {{&S47405[27], 6}, {NULL}};
const Segment SGI162305[] = {{&S47405[98], 9}, {NULL}};
const Segment SGI162306[] = {{&S47405[71], 6}, {NULL}};
const Segment SGI162307[] = {{&S47406[34], 11}, {NULL}};
const Segment SGI162308[] = {{&S47406[27], 6}, {NULL}};
const Segment SGI162309[] = {{&S47406[98], 9}, {NULL}};
const Segment SGI162310[] = {{&S47406[71], 6}, {NULL}};
const Segment SGI162311[] = {{&S57522[44], 7}, {NULL}};
const Segment SGI162312[] = {{&S57522[44], 19}, {NULL}};
const Segment SGI162313[] = {{&S57522[30], 6}, {NULL}};
const Segment SGI162314[] = {{&S57522[31], 3}, {NULL}};
const Segment SGI162315[] = {{&S57522[77], 6}, {NULL}};
const Segment SGI162316[] = {{&S57523[38], 7}, {NULL}};
const Segment SGI162317[] = {{&S57523[31], 6}, {NULL}};
const Segment SGI162318[] = {{&S57533[44], 8}, {NULL}};
const Segment SGI162319[] = {{&S57533[44], 20}, {NULL}};
const Segment SGI162320[] = {{&S57533[30], 6}, {NULL}};
const Segment SGI162321[] = {{&S57533[31], 3}, {NULL}};
const Segment SGI162322[] = {{&S57533[78], 6}, {NULL}};
const Segment SGI162323[] = {{&S57541[44], 9}, {NULL}};
const Segment SGI162324[] = {{&S57541[44], 21}, {NULL}};
const Segment SGI162325[] = {{&S57541[30], 6}, {NULL}};
const Segment SGI162326[] = {{&S57541[31], 3}, {NULL}};
const Segment SGI162327[] = {{&S57541[79], 6}, {NULL}};
const Segment SGI162328[] = {{&S57542[38], 9}, {NULL}};
const Segment SGI162329[] = {{&S57542[31], 6}, {NULL}};
const Segment SGI162330[] = {{&S47446[34], 7}, {NULL}};
const Segment SGI162331[] = {{&S47446[27], 6}, {NULL}};
const Segment SGI162332[] = {{&S47446[99], 9}, {NULL}};
const Segment SGI162333[] = {{&S47446[72], 6}, {NULL}};
const Segment SGI162334[] = {{&S47447[34], 10}, {NULL}};
const Segment SGI162335[] = {{&S47447[27], 6}, {NULL}};
const Segment SGI162336[] = {{&S47447[97], 9}, {NULL}};
const Segment SGI162337[] = {{&S47447[70], 6}, {NULL}};
const Segment SGI162338[] = {{&S57570[34], 7}, {NULL}};
const Segment SGI162339[] = {{&S57570[27], 6}, {NULL}};
const Segment SGI162340[] = {{&S57570[94], 9}, {NULL}};
const Segment SGI162341[] = {{&S57570[67], 6}, {NULL}};
const Segment SGI162342[] = {{&S48902[44], 10}, {NULL}};
const Segment SGI162343[] = {{&S48902[44], 33}, {NULL}};
const Segment SGI162344[] = {{&S48902[30], 6}, {NULL}};
const Segment SGI162345[] = {{&S48902[126], 3}, {NULL}};
const Segment SGI162346[] = {{&S48902[91], 6}, {NULL}};
const Segment SGI162347[] = {{&S48903[44], 10}, {NULL}};
const Segment SGI162348[] = {{&S48903[44], 33}, {NULL}};
const Segment SGI162349[] = {{&S48903[30], 6}, {NULL}};
const Segment SGI162350[] = {{&S48903[126], 3}, {NULL}};
const Segment SGI162351[] = {{&S48903[91], 6}, {NULL}};
const Segment SGI162352[] = {{&S48904[44], 10}, {NULL}};
const Segment SGI162353[] = {{&S48904[44], 33}, {NULL}};
const Segment SGI162354[] = {{&S48904[30], 6}, {NULL}};
const Segment SGI162355[] = {{&S47496[44], 10}, {NULL}};
const Segment SGI162356[] = {{&S48904[126], 3}, {NULL}};
const Segment SGI162357[] = {{&S47496[44], 22}, {NULL}};
const Segment SGI162358[] = {{&S47496[30], 6}, {NULL}};
const Segment SGI162359[] = {{&S48904[91], 6}, {NULL}};
const Segment SGI162360[] = {{&S47496[115], 3}, {NULL}};
const Segment SGI162361[] = {{&S47496[80], 6}, {NULL}};
const Segment SGI162362[] = {{&S48905[38], 10}, {NULL}};
const Segment SGI162363[] = {{&S48905[31], 6}, {NULL}};
const Segment SGI162364[] = {{&S43463[44], 10}, {NULL}};
const Segment SGI162365[] = {{&S43463[44], 22}, {NULL}};
const Segment SGI162366[] = {{&S43463[30], 6}, {NULL}};
const Segment SGI162367[] = {{&S43463[115], 3}, {NULL}};
const Segment SGI162368[] = {{&S43463[80], 6}, {NULL}};
const Segment SGI162369[] = {{&S47499[38], 10}, {NULL}};
const Segment SGI162370[] = {{&S47499[31], 6}, {NULL}};
const Segment SGI162371[] = {{&S47500[44], 10}, {NULL}};
const Segment SGI162372[] = {{&S47500[44], 27}, {NULL}};
const Segment SGI162373[] = {{&S47500[30], 6}, {NULL}};
const Segment SGI162374[] = {{&S47500[120], 3}, {NULL}};
const Segment SGI162375[] = {{&S47500[85], 6}, {NULL}};
const Segment SGI162376[] = {{&S48911[44], 9}, {NULL}};
const Segment SGI162377[] = {{&S48911[44], 21}, {NULL}};
const Segment SGI162378[] = {{&S48911[30], 6}, {NULL}};
const Segment SGI162379[] = {{&S48911[114], 3}, {NULL}};
const Segment SGI162380[] = {{&S48911[79], 6}, {NULL}};
const Segment SGI162381[] = {{&S48912[38], 9}, {NULL}};
const Segment SGI162382[] = {{&S47504[34], 7}, {NULL}};
const Segment SGI162383[] = {{&S48912[31], 6}, {NULL}};
const Segment SGI162384[] = {{&S47504[27], 6}, {NULL}};
const Segment SGI162385[] = {{&S47504[99], 9}, {NULL}};
const Segment SGI162386[] = {{&S47504[72], 6}, {NULL}};
const Segment SGI162387[] = {{&S48913[44], 22}, {NULL}};
const Segment SGI162388[] = {{&S48913[44], 34}, {NULL}};
const Segment SGI162389[] = {{&S47505[34], 7}, {NULL}};
const Segment SGI162390[] = {{&S48913[30], 6}, {NULL}};
const Segment SGI162391[] = {{&S47505[27], 6}, {NULL}};
const Segment SGI162392[] = {{&S48913[127], 3}, {NULL}};
const Segment SGI162393[] = {{&S48913[92], 6}, {NULL}};
const Segment SGI162394[] = {{&S47505[99], 9}, {NULL}};
const Segment SGI162395[] = {{&S47505[72], 6}, {NULL}};
const Segment SGI162396[] = {{&S47506[34], 10}, {NULL}};
const Segment SGI162397[] = {{&S48914[44], 26}, {NULL}};
const Segment SGI162398[] = {{&S47506[27], 6}, {NULL}};
const Segment SGI162399[] = {{&S48914[44], 38}, {NULL}};
const Segment SGI162400[] = {{&S48914[30], 6}, {NULL}};
const Segment SGI162401[] = {{&S47506[97], 9}, {NULL}};
const Segment SGI162402[] = {{&S48914[131], 3}, {NULL}};
const Segment SGI162403[] = {{&S47506[70], 6}, {NULL}};
const Segment SGI162404[] = {{&S48914[96], 6}, {NULL}};
const Segment SGI162405[] = {{&S48915[44], 10}, {NULL}};
const Segment SGI162406[] = {{&S48915[44], 22}, {NULL}};
const Segment SGI162407[] = {{&S48915[30], 6}, {NULL}};
const Segment SGI162408[] = {{&S48915[115], 3}, {NULL}};
const Segment SGI162409[] = {{&S48915[80], 6}, {NULL}};
const Segment SGI162410[] = {{&S46389[42], 10}, {NULL}};
const Segment SGI162411[] = {{&S46389[42], 22}, {NULL}};
const Segment SGI162412[] = {{&S46389[30], 4}, {NULL}};
const Segment SGI162413[] = {{&S57643[34], 11}, {NULL}};
const Segment SGI162414[] = {{&S57643[27], 6}, {NULL}};
const Segment SGI162415[] = {{&S46389[113], 3}, {NULL}};
const Segment SGI162416[] = {{&S46389[78], 6}, {NULL}};
const Segment SGI162417[] = {{&S57643[98], 9}, {NULL}};
const Segment SGI162418[] = {{&S57643[71], 6}, {NULL}};
const Segment SGI162419[] = {{&S57644[34], 11}, {NULL}};
const Segment SGI162420[] = {{&S57644[27], 6}, {NULL}};
const Segment SGI162421[] = {{&S57644[98], 9}, {NULL}};
const Segment SGI162422[] = {{&S57644[71], 6}, {NULL}};
const Segment SGI162423[] = {{&S34005[34], 9}, {NULL}};
const Segment SGI162424[] = {{&S34005[27], 6}, {NULL}};
const Segment SGI162425[] = {{&S34005[95], 9}, {NULL}};
const Segment SGI162426[] = {{&S34005[68], 6}, {NULL}};
const Segment SGI162427[] = {{&S34008[34], 9}, {NULL}};
const Segment SGI162428[] = {{&S34008[27], 6}, {NULL}};
const Segment SGI162429[] = {{&S34008[95], 9}, {NULL}};
const Segment SGI162430[] = {{&S34008[68], 6}, {NULL}};
const Segment SGI162431[] = {{&S34035[34], 10}, {NULL}};
const Segment SGI162432[] = {{&S34035[27], 6}, {NULL}};
const Segment SGI162433[] = {{&S34035[96], 9}, {NULL}};
const Segment SGI162434[] = {{&S34035[69], 6}, {NULL}};
const Segment SGI162435[] = {{&S57731[44], 8}, {NULL}};
const Segment SGI162436[] = {{&S57731[44], 20}, {NULL}};
const Segment SGI162437[] = {{&S57731[30], 6}, {NULL}};
const Segment SGI162438[] = {{&S57731[31], 3}, {NULL}};
const Segment SGI162439[] = {{&S57731[78], 6}, {NULL}};
const Segment SGI162440[] = {{&S34037[34], 9}, {NULL}};
const Segment SGI162441[] = {{&S34037[27], 6}, {NULL}};
const Segment SGI162442[] = {{&S57732[44], 9}, {NULL}};
const Segment SGI162443[] = {{&S57732[44], 21}, {NULL}};
const Segment SGI162444[] = {{&S34037[95], 9}, {NULL}};
const Segment SGI162445[] = {{&S57732[30], 6}, {NULL}};
const Segment SGI162446[] = {{&S34037[68], 6}, {NULL}};
const Segment SGI162447[] = {{&S57732[31], 3}, {NULL}};
const Segment SGI162448[] = {{&S57732[79], 6}, {NULL}};
const Segment SGI162449[] = {{&S34038[34], 9}, {NULL}};
const Segment SGI162450[] = {{&S34038[27], 6}, {NULL}};
const Segment SGI162451[] = {{&S34038[95], 9}, {NULL}};
const Segment SGI162452[] = {{&S34038[68], 6}, {NULL}};
const Segment SGI162453[] = {{&S43464[44], 10}, {NULL}};
const Segment SGI162454[] = {{&S43464[44], 22}, {NULL}};
const Segment SGI162455[] = {{&S43464[30], 6}, {NULL}};
const Segment SGI162456[] = {{&S43464[115], 3}, {NULL}};
const Segment SGI162457[] = {{&S43464[80], 6}, {NULL}};
const Segment SGI162458[] = {{&S43465[44], 18}, {NULL}};
const Segment SGI162459[] = {{&S43465[44], 30}, {NULL}};
const Segment SGI162460[] = {{&S43465[30], 6}, {NULL}};
const Segment SGI162461[] = {{&S43465[123], 3}, {NULL}};
const Segment SGI162462[] = {{&S43465[88], 6}, {NULL}};
const Segment SGI162463[] = {{&S43466[44], 11}, {NULL}};
const Segment SGI162464[] = {{&S43466[44], 23}, {NULL}};
const Segment SGI162465[] = {{&S43466[30], 6}, {NULL}};
const Segment SGI162466[] = {{&S43466[116], 3}, {NULL}};
const Segment SGI162467[] = {{&S43466[81], 6}, {NULL}};
const Segment SGI162468[] = {{&S43467[44], 18}, {NULL}};
const Segment SGI162469[] = {{&S43467[44], 30}, {NULL}};
const Segment SGI162470[] = {{&S43467[30], 6}, {NULL}};
const Segment SGI162471[] = {{&S43467[123], 3}, {NULL}};
const Segment SGI162472[] = {{&S43467[88], 6}, {NULL}};
const Segment SGI162473[] = {{&S49058[34], 10}, {NULL}};
const Segment SGI162474[] = {{&S49058[27], 6}, {NULL}};
const Segment SGI162475[] = {{&S43468[44], 10}, {NULL}};
const Segment SGI162476[] = {{&S49058[97], 9}, {NULL}};
const Segment SGI162477[] = {{&S43468[44], 22}, {NULL}};
const Segment SGI162478[] = {{&S49058[70], 6}, {NULL}};
const Segment SGI162479[] = {{&S43468[30], 6}, {NULL}};
const Segment SGI162480[] = {{&S43468[115], 3}, {NULL}};
const Segment SGI162481[] = {{&S43468[80], 6}, {NULL}};
const Segment SGI162482[] = {{&S43470[44], 10}, {NULL}};
const Segment SGI162483[] = {{&S43470[44], 22}, {NULL}};
const Segment SGI162484[] = {{&S43470[30], 6}, {NULL}};
const Segment SGI162485[] = {{&S43470[115], 3}, {NULL}};
const Segment SGI162486[] = {{&S49061[34], 11}, {NULL}};
const Segment SGI162487[] = {{&S49061[27], 6}, {NULL}};
const Segment SGI162488[] = {{&S43470[80], 6}, {NULL}};
const Segment SGI162489[] = {{&S49061[98], 9}, {NULL}};
const Segment SGI162490[] = {{&S49061[71], 6}, {NULL}};
const Segment SGI162491[] = {{&S57759[34], 11}, {NULL}};
const Segment SGI162492[] = {{&S57759[27], 6}, {NULL}};
const Segment SGI162493[] = {{&S57759[98], 9}, {NULL}};
const Segment SGI162494[] = {{&S57759[71], 6}, {NULL}};
const Segment SGI162495[] = {{&S49067[34], 18}, {NULL}};
const Segment SGI162496[] = {{&S49067[27], 6}, {NULL}};
const Segment SGI162497[] = {{&S57760[34], 10}, {NULL}};
const Segment SGI162498[] = {{&S57760[27], 6}, {NULL}};
const Segment SGI162499[] = {{&S49067[105], 9}, {NULL}};
const Segment SGI162500[] = {{&S49067[78], 6}, {NULL}};
const Segment SGI162501[] = {{&S57760[97], 9}, {NULL}};
const Segment SGI162502[] = {{&S57760[70], 6}, {NULL}};
const Segment SGI162503[] = {{&S43480[38], 18}, {NULL}};
const Segment SGI162504[] = {{&S43480[31], 6}, {NULL}};
const Segment SGI162505[] = {{&S43485[38], 10}, {NULL}};
const Segment SGI162506[] = {{&S43485[31], 6}, {NULL}};
const Segment SGI162507[] = {{&S49075[34], 14}, {NULL}};
const Segment SGI162508[] = {{&S49075[27], 6}, {NULL}};
const Segment SGI162509[] = {{&S49075[101], 9}, {NULL}};
const Segment SGI162510[] = {{&S49075[74], 6}, {NULL}};
const Segment SGI162511[] = {{&S49076[34], 24}, {NULL}};
const Segment SGI162512[] = {{&S57772[34], 11}, {NULL}};
const Segment SGI162513[] = {{&S49076[27], 6}, {NULL}};
const Segment SGI162514[] = {{&S57772[27], 6}, {NULL}};
const Segment SGI162515[] = {{&S49076[111], 9}, {NULL}};
const Segment SGI162516[] = {{&S57772[98], 9}, {NULL}};
const Segment SGI162517[] = {{&S49076[84], 6}, {NULL}};
const Segment SGI162518[] = {{&S57772[71], 6}, {NULL}};
const Segment SGI162519[] = {{&S49077[34], 23}, {NULL}};
const Segment SGI162520[] = {{&S49077[27], 6}, {NULL}};
const Segment SGI162521[] = {{&S49077[110], 9}, {NULL}};
const Segment SGI162522[] = {{&S49077[83], 6}, {NULL}};
const Segment SGI162523[] = {{&S57775[34], 41}, {NULL}};
const Segment SGI162524[] = {{&S57775[27], 6}, {NULL}};
const Segment SGI162525[] = {{&S57775[128], 9}, {NULL}};
const Segment SGI162526[] = {{&S57775[101], 6}, {NULL}};
const Segment SGI162527[] = {{&S57777[34], 41}, {NULL}};
const Segment SGI162528[] = {{&S57777[27], 6}, {NULL}};
const Segment SGI162529[] = {{&S57777[128], 9}, {NULL}};
const Segment SGI162530[] = {{&S57777[101], 6}, {NULL}};
const Segment SGI162531[] = {{&S57778[34], 21}, {NULL}};
const Segment SGI162532[] = {{&S57778[27], 6}, {NULL}};
const Segment SGI162533[] = {{&S57778[108], 9}, {NULL}};
const Segment SGI162534[] = {{&S57778[81], 6}, {NULL}};
const Segment SGI162535[] = {{&S43501[34], 10}, {NULL}};
const Segment SGI162536[] = {{&S43501[27], 6}, {NULL}};
const Segment SGI162537[] = {{&S43501[97], 9}, {NULL}};
const Segment SGI162538[] = {{&S43501[70], 6}, {NULL}};
const Segment SGI162539[] = {{&S49089[34], 12}, {NULL}};
const Segment SGI162540[] = {{&S49089[27], 6}, {NULL}};
const Segment SGI162541[] = {{&S49089[99], 9}, {NULL}};
const Segment SGI162542[] = {{&S49089[72], 6}, {NULL}};
const Segment SGI162543[] = {{&S49090[34], 12}, {NULL}};
const Segment SGI162544[] = {{&S49090[27], 6}, {NULL}};
const Segment SGI162545[] = {{&S49090[99], 9}, {NULL}};
const Segment SGI162546[] = {{&S49090[72], 6}, {NULL}};
const Segment SGI162547[] = {{&S57788[34], 12}, {NULL}};
const Segment SGI162548[] = {{&S57788[27], 6}, {NULL}};
const Segment SGI162549[] = {{&S57788[99], 9}, {NULL}};
const Segment SGI162550[] = {{&S57788[72], 6}, {NULL}};
const Segment SGI162551[] = {{&S49098[44], 10}, {NULL}};
const Segment SGI162552[] = {{&S49098[44], 22}, {NULL}};
const Segment SGI162553[] = {{&S49098[30], 6}, {NULL}};
const Segment SGI162554[] = {{&S49098[115], 3}, {NULL}};
const Segment SGI162555[] = {{&S49098[80], 6}, {NULL}};
const Segment SGI162556[] = {{&S57794[34], 7}, {NULL}};
const Segment SGI162557[] = {{&S57794[27], 6}, {NULL}};
const Segment SGI162558[] = {{&S57794[104], 9}, {NULL}};
const Segment SGI162559[] = {{&S57794[77], 6}, {NULL}};
const Segment SGI162560[] = {{&S49100[44], 18}, {NULL}};
const Segment SGI162561[] = {{&S49100[44], 30}, {NULL}};
const Segment SGI162562[] = {{&S49100[30], 6}, {NULL}};
const Segment SGI162563[] = {{&S49100[123], 3}, {NULL}};
const Segment SGI162564[] = {{&S49100[88], 6}, {NULL}};
const Segment SGI162565[] = {{&S49101[38], 10}, {NULL}};
const Segment SGI162566[] = {{&S49101[31], 6}, {NULL}};
const Segment SGI162567[] = {{&S57848[34], 8}, {NULL}};
const Segment SGI162568[] = {{&S57848[27], 6}, {NULL}};
const Segment SGI162569[] = {{&S57848[95], 9}, {NULL}};
const Segment SGI162570[] = {{&S57848[68], 6}, {NULL}};
const Segment SGI162571[] = {{&S57890[34], 9}, {NULL}};
const Segment SGI162572[] = {{&S57890[27], 6}, {NULL}};
const Segment SGI162573[] = {{&S57890[96], 9}, {NULL}};
const Segment SGI162574[] = {{&S57890[69], 6}, {NULL}};
const Segment SGI162575[] = {{&S49232[34], 8}, {NULL}};
const Segment SGI162576[] = {{&S49232[27], 6}, {NULL}};
const Segment SGI162577[] = {{&S49232[96], 9}, {NULL}};
const Segment SGI162578[] = {{&S49232[69], 6}, {NULL}};
const Segment SGI162579[] = {{&S49233[34], 8}, {NULL}};
const Segment SGI162580[] = {{&S49233[27], 6}, {NULL}};
const Segment SGI162581[] = {{&S49233[96], 9}, {NULL}};
const Segment SGI162582[] = {{&S49233[69], 6}, {NULL}};
const Segment SGI162583[] = {{&S49236[34], 5}, {NULL}};
const Segment SGI162584[] = {{&S49236[27], 6}, {NULL}};
const Segment SGI162585[] = {{&S49236[92], 9}, {NULL}};
const Segment SGI162586[] = {{&S49236[65], 6}, {NULL}};
const Segment SGI162587[] = {{&S49237[34], 5}, {NULL}};
const Segment SGI162588[] = {{&S49237[27], 6}, {NULL}};
const Segment SGI162589[] = {{&S49237[92], 9}, {NULL}};
const Segment SGI162590[] = {{&S49237[65], 6}, {NULL}};
const Segment SGI162591[] = {{&S57937[44], 6}, {NULL}};
const Segment SGI162592[] = {{&S57937[44], 18}, {NULL}};
const Segment SGI162593[] = {{&S57937[30], 6}, {NULL}};
const Segment SGI162594[] = {{&S57937[31], 3}, {NULL}};
const Segment SGI162595[] = {{&S57937[76], 6}, {NULL}};
const Segment SGI162596[] = {{&S57942[34], 10}, {NULL}};
const Segment SGI162597[] = {{&S57942[27], 6}, {NULL}};
const Segment SGI162598[] = {{&S57942[97], 9}, {NULL}};
const Segment SGI162599[] = {{&S57942[70], 6}, {NULL}};
const Segment SGI162600[] = {{&S57947[44], 8}, {NULL}};
const Segment SGI162601[] = {{&S57947[44], 20}, {NULL}};
const Segment SGI162602[] = {{&S57947[30], 6}, {NULL}};
const Segment SGI162603[] = {{&S57947[31], 3}, {NULL}};
const Segment SGI162604[] = {{&S57947[78], 6}, {NULL}};
const Segment SGI162605[] = {{&S57960[34], 7}, {NULL}};
const Segment SGI162606[] = {{&S57960[27], 6}, {NULL}};
const Segment SGI162607[] = {{&S57960[93], 9}, {NULL}};
const Segment SGI162608[] = {{&S57960[66], 6}, {NULL}};
const Segment SGI162609[] = {{&S57961[34], 6}, {NULL}};
const Segment SGI162610[] = {{&S57961[27], 6}, {NULL}};
const Segment SGI162611[] = {{&S57961[93], 9}, {NULL}};
const Segment SGI162612[] = {{&S57961[66], 6}, {NULL}};
const Segment SGI162613[] = {{&S57962[34], 7}, {NULL}};
const Segment SGI162614[] = {{&S57962[27], 6}, {NULL}};
const Segment SGI162615[] = {{&S57962[99], 9}, {NULL}};
const Segment SGI162616[] = {{&S57962[72], 6}, {NULL}};
const Segment SGI162617[] = {{&S57963[34], 8}, {NULL}};
const Segment SGI162618[] = {{&S57963[27], 6}, {NULL}};
const Segment SGI162619[] = {{&S57963[99], 9}, {NULL}};
const Segment SGI162620[] = {{&S57963[72], 6}, {NULL}};
const Segment SGI162621[] = {{&S57964[34], 7}, {NULL}};
const Segment SGI162622[] = {{&S57964[27], 6}, {NULL}};
const Segment SGI162623[] = {{&S57964[99], 9}, {NULL}};
const Segment SGI162624[] = {{&S57964[72], 6}, {NULL}};
const Segment SGI162625[] = {{&S57965[34], 8}, {NULL}};
const Segment SGI162626[] = {{&S57965[27], 6}, {NULL}};
const Segment SGI162627[] = {{&S57965[99], 9}, {NULL}};
const Segment SGI162628[] = {{&S57965[72], 6}, {NULL}};
const Segment SGI162629[] = {{&S57970[44], 10}, {NULL}};
const Segment SGI162630[] = {{&S57970[44], 22}, {NULL}};
const Segment SGI162631[] = {{&S57970[30], 6}, {NULL}};
const Segment SGI162632[] = {{&S57970[83], 6}, {NULL}};
const Segment SGI162633[] = {{&S57971[44], 10}, {NULL}};
const Segment SGI162634[] = {{&S57971[44], 22}, {NULL}};
const Segment SGI162635[] = {{&S57971[30], 6}, {NULL}};
const Segment SGI162636[] = {{&S57971[31], 3}, {NULL}};
const Segment SGI162637[] = {{&S57971[80], 6}, {NULL}};
const Segment SGI162638[] = {{&S57972[38], 10}, {NULL}};
const Segment SGI162639[] = {{&S57972[31], 6}, {NULL}};
const Segment SGI162640[] = {{&S57974[61], 21}, {NULL}};
const Segment SGI162641[] = {{&S57974[48], 3}, {NULL}};
const Segment SGI162642[] = {{&S57974[95], 6}, {NULL}};
const Segment SGI162643[] = {{&S57977[44], 13}, {NULL}};
const Segment SGI162644[] = {{&S57977[44], 32}, {NULL}};
const Segment SGI162645[] = {{&S57977[30], 6}, {NULL}};
const Segment SGI162646[] = {{&S57977[31], 3}, {NULL}};
const Segment SGI162647[] = {{&S57977[90], 6}, {NULL}};
const Segment SGI162648[] = {{&S57978[44], 17}, {NULL}};
const Segment SGI162649[] = {{&S57978[44], 36}, {NULL}};
const Segment SGI162650[] = {{&S57978[30], 6}, {NULL}};
const Segment SGI162651[] = {{&S57978[31], 3}, {NULL}};
const Segment SGI162652[] = {{&S57978[94], 6}, {NULL}};
const Segment SGI162653[] = {{&S57979[38], 17}, {NULL}};
const Segment SGI162654[] = {{&S57979[31], 6}, {NULL}};
const Segment SGI162655[] = {{&S57982[44], 17}, {NULL}};
const Segment SGI162656[] = {{&S57982[44], 29}, {NULL}};
const Segment SGI162657[] = {{&S57982[30], 6}, {NULL}};
const Segment SGI162658[] = {{&S57982[31], 3}, {NULL}};
const Segment SGI162659[] = {{&S57982[87], 6}, {NULL}};
const Segment SGI162660[] = {{&S57984[45], 21}, {NULL}};
const Segment SGI162661[] = {{&S57984[32], 3}, {NULL}};
const Segment SGI162662[] = {{&S57984[80], 6}, {NULL}};
const Segment SGI162663[] = {{&S57985[44], 13}, {NULL}};
const Segment SGI162664[] = {{&S57985[44], 32}, {NULL}};
const Segment SGI162665[] = {{&S57985[30], 6}, {NULL}};
const Segment SGI162666[] = {{&S49332[34], 25}, {NULL}};
const Segment SGI162667[] = {{&S49332[27], 6}, {NULL}};
const Segment SGI162668[] = {{&S57985[93], 6}, {NULL}};
const Segment SGI162669[] = {{&S49332[112], 9}, {NULL}};
const Segment SGI162670[] = {{&S49332[85], 6}, {NULL}};
const Segment SGI162671[] = {{&S57986[60], 21}, {NULL}};
const Segment SGI162672[] = {{&S57986[46], 6}, {NULL}};
const Segment SGI162673[] = {{&S57986[47], 3}, {NULL}};
const Segment SGI162674[] = {{&S57986[94], 6}, {NULL}};
const Segment SGI162675[] = {{&S49334[34], 7}, {NULL}};
const Segment SGI162676[] = {{&S49334[27], 6}, {NULL}};
const Segment SGI162677[] = {{&S49334[94], 9}, {NULL}};
const Segment SGI162678[] = {{&S49334[67], 6}, {NULL}};
const Segment SGI162679[] = {{&S57989[44], 10}, {NULL}};
const Segment SGI162680[] = {{&S57989[44], 27}, {NULL}};
const Segment SGI162681[] = {{&S57989[30], 6}, {NULL}};
const Segment SGI162682[] = {{&S57989[31], 3}, {NULL}};
const Segment SGI162683[] = {{&S57989[85], 6}, {NULL}};
const Segment SGI162684[] = {{&S57991[44], 10}, {NULL}};
const Segment SGI162685[] = {{&S57991[44], 27}, {NULL}};
const Segment SGI162686[] = {{&S57991[30], 6}, {NULL}};
const Segment SGI162687[] = {{&S57991[31], 3}, {NULL}};
const Segment SGI162688[] = {{&S57991[85], 6}, {NULL}};
const Segment SGI162689[] = {{&S57992[38], 10}, {NULL}};
const Segment SGI162690[] = {{&S57992[31], 6}, {NULL}};
const Segment SGI162691[] = {{&S57994[44], 10}, {NULL}};
const Segment SGI162692[] = {{&S57994[44], 22}, {NULL}};
const Segment SGI162693[] = {{&S57994[30], 6}, {NULL}};
const Segment SGI162694[] = {{&S57994[31], 3}, {NULL}};
const Segment SGI162695[] = {{&S57994[80], 6}, {NULL}};
const Segment SGI162696[] = {{&S57997[46], 11}, {NULL}};
const Segment SGI162697[] = {{&S57997[46], 67}, {NULL}};
const Segment SGI162698[] = {{&S57997[32], 6}, {NULL}};
const Segment SGI162699[] = {{&S58000[38], 11}, {NULL}};
const Segment SGI162700[] = {{&S58000[31], 6}, {NULL}};
const Segment SGI162701[] = {{&S49355[34], 10}, {NULL}};
const Segment SGI162702[] = {{&S49355[27], 6}, {NULL}};
const Segment SGI162703[] = {{&S49355[97], 9}, {NULL}};
const Segment SGI162704[] = {{&S49355[70], 6}, {NULL}};
const Segment SGI162705[] = {{&S58002[44], 10}, {NULL}};
const Segment SGI162706[] = {{&S58002[44], 27}, {NULL}};
const Segment SGI162707[] = {{&S58002[30], 6}, {NULL}};
const Segment SGI162708[] = {{&S58002[85], 6}, {NULL}};
const Segment SGI162709[] = {{&S58004[44], 10}, {NULL}};
const Segment SGI162710[] = {{&S58004[44], 22}, {NULL}};
const Segment SGI162711[] = {{&S58004[30], 6}, {NULL}};
const Segment SGI162712[] = {{&S58004[31], 3}, {NULL}};
const Segment SGI162713[] = {{&S58004[80], 6}, {NULL}};
const Segment SGI162714[] = {{&S58005[44], 10}, {NULL}};
const Segment SGI162715[] = {{&S58005[44], 22}, {NULL}};
const Segment SGI162716[] = {{&S58005[30], 6}, {NULL}};
const Segment SGI162717[] = {{&S58005[83], 6}, {NULL}};
const Segment SGI162718[] = {{&S58006[44], 50}, {NULL}};
const Segment SGI162719[] = {{&S58006[30], 6}, {NULL}};
const Segment SGI162720[] = {{&S58006[31], 3}, {NULL}};
const Segment SGI162721[] = {{&S58006[69], 6}, {NULL}};
const Segment SGI162722[] = {{&S58007[44], 10}, {NULL}};
const Segment SGI162723[] = {{&S58007[44], 22}, {NULL}};
const Segment SGI162724[] = {{&S58007[30], 6}, {NULL}};
const Segment SGI162725[] = {{&S58007[80], 6}, {NULL}};
const Segment SGI162726[] = {{&S58008[44], 10}, {NULL}};
const Segment SGI162727[] = {{&S58008[44], 22}, {NULL}};
const Segment SGI162728[] = {{&S58008[30], 6}, {NULL}};
const Segment SGI162729[] = {{&S58010[44], 10}, {NULL}};
const Segment SGI162730[] = {{&S58010[44], 22}, {NULL}};
const Segment SGI162731[] = {{&S58010[30], 6}, {NULL}};
const Segment SGI162732[] = {{&S58010[83], 6}, {NULL}};
const Segment SGI162733[] = {{&S49371[34], 10}, {NULL}};
const Segment SGI162734[] = {{&S49371[27], 6}, {NULL}};
const Segment SGI162735[] = {{&S49371[97], 9}, {NULL}};
const Segment SGI162736[] = {{&S49371[70], 6}, {NULL}};
const Segment SGI162737[] = {{&S58011[44], 10}, {NULL}};
const Segment SGI162738[] = {{&S58011[44], 22}, {NULL}};
const Segment SGI162739[] = {{&S58011[30], 6}, {NULL}};
const Segment SGI162740[] = {{&S58011[83], 6}, {NULL}};
const Segment SGI162741[] = {{&S58012[44], 10}, {NULL}};
const Segment SGI162742[] = {{&S58012[44], 22}, {NULL}};
const Segment SGI162743[] = {{&S58012[30], 6}, {NULL}};
const Segment SGI162744[] = {{&S58013[46], 10}, {NULL}};
const Segment SGI162745[] = {{&S58013[46], 70}, {NULL}};
const Segment SGI162746[] = {{&S58013[32], 6}, {NULL}};
const Segment SGI162747[] = {{&S58016[44], 10}, {NULL}};
const Segment SGI162748[] = {{&S58016[44], 22}, {NULL}};
const Segment SGI162749[] = {{&S58016[30], 6}, {NULL}};
const Segment SGI162750[] = {{&S58016[116], 3}, {NULL}};
const Segment SGI162751[] = {{&S58016[80], 7}, {NULL}};
const Segment SGI162752[] = {{&S58016[136], 8}, {NULL}};
const Segment SGI162753[] = {{&S58017[44], 10}, {NULL}};
const Segment SGI162754[] = {{&S58017[44], 47}, {NULL}};
const Segment SGI162755[] = {{&S58017[30], 6}, {NULL}};
const Segment SGI162756[] = {{&S58017[31], 3}, {NULL}};
const Segment SGI162757[] = {{&S58017[105], 6}, {NULL}};
const Segment SGI162758[] = {{&S58018[44], 10}, {NULL}};
const Segment SGI162759[] = {{&S58018[44], 22}, {NULL}};
const Segment SGI162760[] = {{&S58018[30], 6}, {NULL}};
const Segment SGI162761[] = {{&S58018[31], 3}, {NULL}};
const Segment SGI162762[] = {{&S58018[80], 6}, {NULL}};
const Segment SGI162763[] = {{&S58019[44], 10}, {NULL}};
const Segment SGI162764[] = {{&S58019[44], 22}, {NULL}};
const Segment SGI162765[] = {{&S58019[30], 6}, {NULL}};
const Segment SGI162766[] = {{&S58019[31], 3}, {NULL}};
const Segment SGI162767[] = {{&S58019[80], 6}, {NULL}};
const Segment SGI162768[] = {{&S58021[99], 28}, {NULL}};
const Segment SGI162769[] = {{&S58022[44], 10}, {NULL}};
const Segment SGI162770[] = {{&S58022[44], 25}, {NULL}};
const Segment SGI162771[] = {{&S58022[30], 6}, {NULL}};
const Segment SGI162772[] = {{&S47702[34], 11}, {NULL}};
const Segment SGI162773[] = {{&S58022[31], 3}, {NULL}};
const Segment SGI162774[] = {{&S47702[27], 6}, {NULL}};
const Segment SGI162775[] = {{&S58022[83], 6}, {NULL}};
const Segment SGI162776[] = {{&S47702[98], 9}, {NULL}};
const Segment SGI162777[] = {{&S47702[71], 6}, {NULL}};
const Segment SGI162778[] = {{&S58027[44], 14}, {NULL}};
const Segment SGI162779[] = {{&S58027[44], 31}, {NULL}};
const Segment SGI162780[] = {{&S58027[30], 6}, {NULL}};
const Segment SGI162781[] = {{&S58027[31], 3}, {NULL}};
const Segment SGI162782[] = {{&S58027[89], 6}, {NULL}};
const Segment SGI162783[] = {{&S58034[44], 14}, {NULL}};
const Segment SGI162784[] = {{&S58034[44], 31}, {NULL}};
const Segment SGI162785[] = {{&S49404[34], 5}, {NULL}};
const Segment SGI162786[] = {{&S58034[30], 6}, {NULL}};
const Segment SGI162787[] = {{&S49404[27], 6}, {NULL}};
const Segment SGI162788[] = {{&S58034[31], 3}, {NULL}};
const Segment SGI162789[] = {{&S58034[89], 6}, {NULL}};
const Segment SGI162790[] = {{&S49404[92], 9}, {NULL}};
const Segment SGI162791[] = {{&S49404[65], 6}, {NULL}};
const Segment SGI162792[] = {{&S58035[38], 14}, {NULL}};
const Segment SGI162793[] = {{&S58035[31], 6}, {NULL}};
const Segment SGI162794[] = {{&S58053[44], 10}, {NULL}};
const Segment SGI162795[] = {{&S58053[44], 22}, {NULL}};
const Segment SGI162796[] = {{&S58053[30], 6}, {NULL}};
const Segment SGI162797[] = {{&S58053[31], 3}, {NULL}};
const Segment SGI162798[] = {{&S58053[95], 6}, {NULL}};
const Segment SGI162799[] = {{&S58053[68], 8}, {NULL}};
const Segment SGI162800[] = {{&S58054[44], 10}, {NULL}};
const Segment SGI162801[] = {{&S58054[44], 22}, {NULL}};
const Segment SGI162802[] = {{&S58054[30], 6}, {NULL}};
const Segment SGI162803[] = {{&S58054[31], 3}, {NULL}};
const Segment SGI162804[] = {{&S58054[99], 6}, {NULL}};
const Segment SGI162805[] = {{&S58054[68], 8}, {NULL}};
const Segment SGI162806[] = {{&S34430[44], 10}, {NULL}};
const Segment SGI162807[] = {{&S34430[44], 21}, {NULL}};
const Segment SGI162808[] = {{&S34430[30], 6}, {NULL}};
const Segment SGI162809[] = {{&S34430[132], 3}, {NULL}};
const Segment SGI162810[] = {{&S34430[97], 6}, {NULL}};
const Segment SGI162811[] = {{&S34430[67], 8}, {NULL}};
const Segment SGI162812[] = {{&S34431[44], 11}, {NULL}};
const Segment SGI162813[] = {{&S34431[44], 22}, {NULL}};
const Segment SGI162814[] = {{&S34431[30], 6}, {NULL}};
const Segment SGI162815[] = {{&S34431[133], 3}, {NULL}};
const Segment SGI162816[] = {{&S34431[98], 6}, {NULL}};
const Segment SGI162817[] = {{&S34431[68], 8}, {NULL}};
const Segment SGI162818[] = {{&S34432[44], 10}, {NULL}};
const Segment SGI162819[] = {{&S34432[44], 22}, {NULL}};
const Segment SGI162820[] = {{&S34432[30], 6}, {NULL}};
const Segment SGI162821[] = {{&S34432[133], 3}, {NULL}};
const Segment SGI162822[] = {{&S34432[98], 6}, {NULL}};
const Segment SGI162823[] = {{&S34432[68], 8}, {NULL}};
const Segment SGI162824[] = {{&S58067[44], 10}, {NULL}};
const Segment SGI162825[] = {{&S58067[44], 22}, {NULL}};
const Segment SGI162826[] = {{&S58067[30], 6}, {NULL}};
const Segment SGI162827[] = {{&S58067[31], 3}, {NULL}};
const Segment SGI162828[] = {{&S58067[80], 6}, {NULL}};
const Segment SGI162829[] = {{&S34434[44], 10}, {NULL}};
const Segment SGI162830[] = {{&S58068[38], 10}, {NULL}};
const Segment SGI162831[] = {{&S34434[44], 20}, {NULL}};
const Segment SGI162832[] = {{&S58068[31], 6}, {NULL}};
const Segment SGI162833[] = {{&S34434[30], 6}, {NULL}};
const Segment SGI162834[] = {{&S34434[131], 3}, {NULL}};
const Segment SGI162835[] = {{&S34434[96], 6}, {NULL}};
const Segment SGI162836[] = {{&S34434[66], 8}, {NULL}};
const Segment SGI162837[] = {{&S34435[44], 10}, {NULL}};
const Segment SGI162838[] = {{&S34435[44], 20}, {NULL}};
const Segment SGI162839[] = {{&S34435[30], 6}, {NULL}};
const Segment SGI162840[] = {{&S34435[131], 3}, {NULL}};
const Segment SGI162841[] = {{&S34435[96], 6}, {NULL}};
const Segment SGI162842[] = {{&S34435[66], 8}, {NULL}};
const Segment SGI162843[] = {{&S34436[44], 10}, {NULL}};
const Segment SGI162844[] = {{&S34436[44], 20}, {NULL}};
const Segment SGI162845[] = {{&S34436[30], 6}, {NULL}};
const Segment SGI162846[] = {{&S34436[113], 3}, {NULL}};
const Segment SGI162847[] = {{&S34436[78], 6}, {NULL}};
const Segment SGI162848[] = {{&S34436[138], 8}, {NULL}};
const Segment SGI162849[] = {{&S34437[44], 11}, {NULL}};
const Segment SGI162850[] = {{&S34437[44], 23}, {NULL}};
const Segment SGI162851[] = {{&S34437[30], 6}, {NULL}};
const Segment SGI162852[] = {{&S34437[116], 3}, {NULL}};
const Segment SGI162853[] = {{&S47763[34], 10}, {NULL}};
const Segment SGI162854[] = {{&S34437[81], 6}, {NULL}};
const Segment SGI162855[] = {{&S47763[27], 6}, {NULL}};
const Segment SGI162856[] = {{&S34437[141], 8}, {NULL}};
const Segment SGI162857[] = {{&S47763[97], 9}, {NULL}};
const Segment SGI162858[] = {{&S47763[70], 6}, {NULL}};
const Segment SGI162859[] = {{&S34438[44], 10}, {NULL}};
const Segment SGI162860[] = {{&S34438[44], 20}, {NULL}};
const Segment SGI162861[] = {{&S34438[30], 6}, {NULL}};
const Segment SGI162862[] = {{&S34438[131], 3}, {NULL}};
const Segment SGI162863[] = {{&S34438[96], 6}, {NULL}};
const Segment SGI162864[] = {{&S34438[66], 8}, {NULL}};
const Segment SGI162865[] = {{&S58105[34], 7}, {NULL}};
const Segment SGI162866[] = {{&S58105[27], 6}, {NULL}};
const Segment SGI162867[] = {{&S58105[98], 9}, {NULL}};
const Segment SGI162868[] = {{&S58105[71], 6}, {NULL}};
const Segment SGI162869[] = {{&S43808[34], 10}, {NULL}};
const Segment SGI162870[] = {{&S58134[34], 6}, {NULL}};
const Segment SGI162871[] = {{&S43808[27], 6}, {NULL}};
const Segment SGI162872[] = {{&S58134[27], 6}, {NULL}};
const Segment SGI162873[] = {{&S43808[97], 9}, {NULL}};
const Segment SGI162874[] = {{&S58134[93], 9}, {NULL}};
const Segment SGI162875[] = {{&S58134[66], 6}, {NULL}};
const Segment SGI162876[] = {{&S43808[70], 6}, {NULL}};
const Segment SGI162877[] = {{&S58135[34], 7}, {NULL}};
const Segment SGI162878[] = {{&S58135[27], 6}, {NULL}};
const Segment SGI162879[] = {{&S58135[98], 9}, {NULL}};
const Segment SGI162880[] = {{&S58135[71], 6}, {NULL}};
const Segment SGI162881[] = {{&S58137[44], 13}, {NULL}};
const Segment SGI162882[] = {{&S58137[44], 25}, {NULL}};
const Segment SGI162883[] = {{&S58137[30], 6}, {NULL}};
const Segment SGI162884[] = {{&S58137[31], 3}, {NULL}};
const Segment SGI162885[] = {{&S58137[109], 6}, {NULL}};
const Segment SGI162886[] = {{&S58137[71], 8}, {NULL}};
const Segment SGI162887[] = {{&S58138[44], 20}, {NULL}};
const Segment SGI162888[] = {{&S58138[44], 32}, {NULL}};
const Segment SGI162889[] = {{&S58138[30], 6}, {NULL}};
const Segment SGI162890[] = {{&S58138[31], 3}, {NULL}};
const Segment SGI162891[] = {{&S58138[127], 6}, {NULL}};
const Segment SGI162892[] = {{&S58138[78], 8}, {NULL}};
const Segment SGI162893[] = {{&S58139[44], 20}, {NULL}};
const Segment SGI162894[] = {{&S58139[44], 32}, {NULL}};
const Segment SGI162895[] = {{&S58139[30], 6}, {NULL}};
const Segment SGI162896[] = {{&S58139[31], 3}, {NULL}};
const Segment SGI162897[] = {{&S58139[109], 6}, {NULL}};
const Segment SGI162898[] = {{&S58139[78], 8}, {NULL}};
const Segment SGI162899[] = {{&S58140[44], 20}, {NULL}};
const Segment SGI162900[] = {{&S58140[44], 32}, {NULL}};
const Segment SGI162901[] = {{&S58140[30], 6}, {NULL}};
const Segment SGI162902[] = {{&S58140[31], 3}, {NULL}};
const Segment SGI162903[] = {{&S58140[114], 6}, {NULL}};
const Segment SGI162904[] = {{&S58140[78], 8}, {NULL}};
const Segment SGI162905[] = {{&S58141[44], 20}, {NULL}};
const Segment SGI162906[] = {{&S58141[44], 32}, {NULL}};
const Segment SGI162907[] = {{&S58141[30], 6}, {NULL}};
const Segment SGI162908[] = {{&S58141[31], 3}, {NULL}};
const Segment SGI162909[] = {{&S58141[105], 6}, {NULL}};
const Segment SGI162910[] = {{&S58141[78], 8}, {NULL}};
const Segment SGI162911[] = {{&S47842[34], 10}, {NULL}};
const Segment SGI162912[] = {{&S47842[27], 6}, {NULL}};
const Segment SGI162913[] = {{&S58142[44], 20}, {NULL}};
const Segment SGI162914[] = {{&S47842[97], 9}, {NULL}};
const Segment SGI162915[] = {{&S58142[44], 32}, {NULL}};
const Segment SGI162916[] = {{&S47842[70], 6}, {NULL}};
const Segment SGI162917[] = {{&S58142[30], 6}, {NULL}};
const Segment SGI162918[] = {{&S58142[31], 3}, {NULL}};
const Segment SGI162919[] = {{&S58142[105], 6}, {NULL}};
const Segment SGI162920[] = {{&S58142[78], 8}, {NULL}};
const Segment SGI162921[] = {{&S34483[44], 9}, {NULL}};
const Segment SGI162922[] = {{&S34483[44], 20}, {NULL}};
const Segment SGI162923[] = {{&S34483[30], 6}, {NULL}};
const Segment SGI162924[] = {{&S34483[113], 3}, {NULL}};
const Segment SGI162925[] = {{&S49504[44], 10}, {NULL}};
const Segment SGI162926[] = {{&S34483[78], 6}, {NULL}};
const Segment SGI162927[] = {{&S49504[44], 22}, {NULL}};
const Segment SGI162928[] = {{&S49504[30], 6}, {NULL}};
const Segment SGI162929[] = {{&S49504[115], 3}, {NULL}};
const Segment SGI162930[] = {{&S49504[80], 6}, {NULL}};
const Segment SGI162931[] = {{&S34485[44], 9}, {NULL}};
const Segment SGI162932[] = {{&S34485[44], 20}, {NULL}};
const Segment SGI162933[] = {{&S34485[30], 6}, {NULL}};
const Segment SGI162934[] = {{&S34485[113], 3}, {NULL}};
const Segment SGI162935[] = {{&S34485[78], 6}, {NULL}};
const Segment SGI162936[] = {{&S58147[34], 10}, {NULL}};
const Segment SGI162937[] = {{&S58147[27], 6}, {NULL}};
const Segment SGI162938[] = {{&S49507[44], 7}, {NULL}};
const Segment SGI162939[] = {{&S58147[97], 9}, {NULL}};
const Segment SGI162940[] = {{&S49507[44], 19}, {NULL}};
const Segment SGI162941[] = {{&S58147[70], 6}, {NULL}};
const Segment SGI162942[] = {{&S49507[30], 6}, {NULL}};
const Segment SGI162943[] = {{&S34486[38], 9}, {NULL}};
const Segment SGI162944[] = {{&S49507[112], 3}, {NULL}};
const Segment SGI162945[] = {{&S34486[31], 6}, {NULL}};
const Segment SGI162946[] = {{&S49507[77], 6}, {NULL}};
const Segment SGI162947[] = {{&S58148[34], 13}, {NULL}};
const Segment SGI162948[] = {{&S58148[27], 6}, {NULL}};
const Segment SGI162949[] = {{&S58148[100], 9}, {NULL}};
const Segment SGI162950[] = {{&S34487[42], 9}, {NULL}};
const Segment SGI162951[] = {{&S58148[73], 6}, {NULL}};
const Segment SGI162952[] = {{&S34487[30], 6}, {NULL}};
const Segment SGI162953[] = {{&S34487[111], 3}, {NULL}};
const Segment SGI162954[] = {{&S34487[76], 6}, {NULL}};
const Segment SGI162955[] = {{&S49509[44], 8}, {NULL}};
const Segment SGI162956[] = {{&S49509[44], 20}, {NULL}};
const Segment SGI162957[] = {{&S49509[30], 6}, {NULL}};
const Segment SGI162958[] = {{&S49509[113], 3}, {NULL}};
const Segment SGI162959[] = {{&S49509[78], 6}, {NULL}};
const Segment SGI162960[] = {{&S34488[44], 9}, {NULL}};
const Segment SGI162961[] = {{&S34488[44], 21}, {NULL}};
const Segment SGI162962[] = {{&S34488[30], 6}, {NULL}};
const Segment SGI162963[] = {{&S34488[114], 3}, {NULL}};
const Segment SGI162964[] = {{&S34488[79], 6}, {NULL}};
const Segment SGI162965[] = {{&S49510[44], 7}, {NULL}};
const Segment SGI162966[] = {{&S49510[44], 19}, {NULL}};
const Segment SGI162967[] = {{&S49510[30], 6}, {NULL}};
const Segment SGI162968[] = {{&S49510[112], 3}, {NULL}};
const Segment SGI162969[] = {{&S49510[77], 6}, {NULL}};
const Segment SGI162970[] = {{&S49515[44], 10}, {NULL}};
const Segment SGI162971[] = {{&S49515[44], 22}, {NULL}};
const Segment SGI162972[] = {{&S34494[44], 10}, {NULL}};
const Segment SGI162973[] = {{&S49515[30], 6}, {NULL}};
const Segment SGI162974[] = {{&S34494[44], 21}, {NULL}};
const Segment SGI162975[] = {{&S34494[30], 6}, {NULL}};
const Segment SGI162976[] = {{&S49515[114], 3}, {NULL}};
const Segment SGI162977[] = {{&S34494[114], 3}, {NULL}};
const Segment SGI162978[] = {{&S49515[80], 5}, {NULL}};
const Segment SGI162979[] = {{&S34494[79], 6}, {NULL}};
const Segment SGI162980[] = {{&S34495[38], 10}, {NULL}};
const Segment SGI162981[] = {{&S34495[31], 6}, {NULL}};
const Segment SGI162982[] = {{&S49518[44], 8}, {NULL}};
const Segment SGI162983[] = {{&S49518[44], 20}, {NULL}};
const Segment SGI162984[] = {{&S49518[30], 6}, {NULL}};
const Segment SGI162985[] = {{&S49518[113], 3}, {NULL}};
const Segment SGI162986[] = {{&S49518[78], 6}, {NULL}};
const Segment SGI162987[] = {{&S34497[44], 10}, {NULL}};
const Segment SGI162988[] = {{&S34497[44], 21}, {NULL}};
const Segment SGI162989[] = {{&S34497[30], 6}, {NULL}};
const Segment SGI162990[] = {{&S34497[114], 3}, {NULL}};
const Segment SGI162991[] = {{&S34497[79], 6}, {NULL}};
const Segment SGI162992[] = {{&S49519[44], 10}, {NULL}};
const Segment SGI162993[] = {{&S49519[44], 22}, {NULL}};
const Segment SGI162994[] = {{&S49519[30], 6}, {NULL}};
const Segment SGI162995[] = {{&S49519[115], 3}, {NULL}};
const Segment SGI162996[] = {{&S49519[80], 6}, {NULL}};
const Segment SGI162997[] = {{&S34498[44], 10}, {NULL}};
const Segment SGI162998[] = {{&S34498[44], 21}, {NULL}};
const Segment SGI162999[] = {{&S34498[30], 6}, {NULL}};
const Segment SGI163000[] = {{&S34498[114], 3}, {NULL}};
const Segment SGI163001[] = {{&S34498[79], 6}, {NULL}};
const Segment SGI163002[] = {{&S34500[44], 10}, {NULL}};
const Segment SGI163003[] = {{&S34500[44], 22}, {NULL}};
const Segment SGI163004[] = {{&S34500[30], 6}, {NULL}};
const Segment SGI163005[] = {{&S34500[115], 3}, {NULL}};
const Segment SGI163006[] = {{&S34500[80], 6}, {NULL}};
const Segment SGI163007[] = {{&S47870[44], 15}, {NULL}};
const Segment SGI163008[] = {{&S47870[44], 27}, {NULL}};
const Segment SGI163009[] = {{&S47870[30], 6}, {NULL}};
const Segment SGI163010[] = {{&S47870[136], 3}, {NULL}};
const Segment SGI163011[] = {{&S47870[101], 6}, {NULL}};
const Segment SGI163012[] = {{&S47870[73], 8}, {NULL}};
const Segment SGI163013[] = {{&S34501[44], 10}, {NULL}};
const Segment SGI163014[] = {{&S34501[44], 22}, {NULL}};
const Segment SGI163015[] = {{&S34501[30], 6}, {NULL}};
const Segment SGI163016[] = {{&S34501[115], 3}, {NULL}};
const Segment SGI163017[] = {{&S34501[80], 6}, {NULL}};
const Segment SGI163018[] = {{&S47871[44], 11}, {NULL}};
const Segment SGI163019[] = {{&S47871[44], 23}, {NULL}};
const Segment SGI163020[] = {{&S47871[30], 6}, {NULL}};
const Segment SGI163021[] = {{&S49526[44], 10}, {NULL}};
const Segment SGI163022[] = {{&S49526[44], 21}, {NULL}};
const Segment SGI163023[] = {{&S47871[132], 3}, {NULL}};
const Segment SGI163024[] = {{&S49526[30], 6}, {NULL}};
const Segment SGI163025[] = {{&S47871[97], 6}, {NULL}};
const Segment SGI163026[] = {{&S47871[69], 8}, {NULL}};
const Segment SGI163027[] = {{&S49526[113], 3}, {NULL}};
const Segment SGI163028[] = {{&S49526[79], 5}, {NULL}};
const Segment SGI163029[] = {{&S34502[38], 10}, {NULL}};
const Segment SGI163030[] = {{&S34502[31], 6}, {NULL}};
const Segment SGI163031[] = {{&S47872[44], 31}, {NULL}};
const Segment SGI163032[] = {{&S47872[44], 43}, {NULL}};
const Segment SGI163033[] = {{&S47872[30], 6}, {NULL}};
const Segment SGI163034[] = {{&S34503[44], 10}, {NULL}};
const Segment SGI163035[] = {{&S34503[44], 22}, {NULL}};
const Segment SGI163036[] = {{&S47872[156], 3}, {NULL}};
const Segment SGI163037[] = {{&S34503[30], 6}, {NULL}};
const Segment SGI163038[] = {{&S47872[121], 6}, {NULL}};
const Segment SGI163039[] = {{&S49528[44], 10}, {NULL}};
const Segment SGI163040[] = {{&S49528[44], 22}, {NULL}};
const Segment SGI163041[] = {{&S47872[89], 8}, {NULL}};
const Segment SGI163042[] = {{&S34503[115], 3}, {NULL}};
const Segment SGI163043[] = {{&S49528[30], 6}, {NULL}};
const Segment SGI163044[] = {{&S34503[80], 6}, {NULL}};
const Segment SGI163045[] = {{&S49528[115], 3}, {NULL}};
const Segment SGI163046[] = {{&S49528[80], 6}, {NULL}};
const Segment SGI163047[] = {{&S34505[38], 10}, {NULL}};
const Segment SGI163048[] = {{&S34505[31], 6}, {NULL}};
const Segment SGI163049[] = {{&S49531[34], 25}, {NULL}};
const Segment SGI163050[] = {{&S49531[27], 6}, {NULL}};
const Segment SGI163051[] = {{&S49531[112], 9}, {NULL}};
const Segment SGI163052[] = {{&S49531[85], 6}, {NULL}};
const Segment SGI163053[] = {{&S34508[44], 10}, {NULL}};
const Segment SGI163054[] = {{&S34508[44], 21}, {NULL}};
const Segment SGI163055[] = {{&S34508[30], 6}, {NULL}};
const Segment SGI163056[] = {{&S34508[114], 3}, {NULL}};
const Segment SGI163057[] = {{&S34508[79], 6}, {NULL}};
const Segment SGI163058[] = {{&S58187[34], 6}, {NULL}};
const Segment SGI163059[] = {{&S58187[27], 6}, {NULL}};
const Segment SGI163060[] = {{&S58187[93], 9}, {NULL}};
const Segment SGI163061[] = {{&S58187[66], 6}, {NULL}};
const Segment SGI163062[] = {{&S58188[34], 12}, {NULL}};
const Segment SGI163063[] = {{&S58188[27], 6}, {NULL}};
const Segment SGI163064[] = {{&S58188[99], 9}, {NULL}};
const Segment SGI163065[] = {{&S58188[72], 6}, {NULL}};
const Segment SGI163066[] = {{&S58189[34], 6}, {NULL}};
const Segment SGI163067[] = {{&S58189[27], 6}, {NULL}};
const Segment SGI163068[] = {{&S58189[93], 9}, {NULL}};
const Segment SGI163069[] = {{&S58189[66], 6}, {NULL}};
const Segment SGI163070[] = {{&S58190[34], 9}, {NULL}};
const Segment SGI163071[] = {{&S58190[27], 6}, {NULL}};
const Segment SGI163072[] = {{&S58190[96], 9}, {NULL}};
const Segment SGI163073[] = {{&S58190[69], 6}, {NULL}};
const Segment SGI163074[] = {{&S58191[34], 8}, {NULL}};
const Segment SGI163075[] = {{&S58191[27], 6}, {NULL}};
const Segment SGI163076[] = {{&S58191[95], 9}, {NULL}};
const Segment SGI163077[] = {{&S58191[68], 6}, {NULL}};
const Segment SGI163078[] = {{&S58192[34], 9}, {NULL}};
const Segment SGI163079[] = {{&S58192[27], 6}, {NULL}};
const Segment SGI163080[] = {{&S58192[96], 9}, {NULL}};
const Segment SGI163081[] = {{&S58192[69], 6}, {NULL}};
const Segment SGI163082[] = {{&S58193[34], 11}, {NULL}};
const Segment SGI163083[] = {{&S58193[27], 6}, {NULL}};
const Segment SGI163084[] = {{&S58193[98], 9}, {NULL}};
const Segment SGI163085[] = {{&S58193[71], 6}, {NULL}};
const Segment SGI163086[] = {{&S58194[34], 7}, {NULL}};
const Segment SGI163087[] = {{&S58194[27], 6}, {NULL}};
const Segment SGI163088[] = {{&S58194[94], 9}, {NULL}};
const Segment SGI163089[] = {{&S58194[67], 6}, {NULL}};
const Segment SGI163090[] = {{&S58195[34], 15}, {NULL}};
const Segment SGI163091[] = {{&S58195[27], 6}, {NULL}};
const Segment SGI163092[] = {{&S58195[102], 9}, {NULL}};
const Segment SGI163093[] = {{&S58195[75], 6}, {NULL}};
const Segment SGI163094[] = {{&S58196[34], 4}, {NULL}};
const Segment SGI163095[] = {{&S58196[27], 6}, {NULL}};
const Segment SGI163096[] = {{&S58196[91], 9}, {NULL}};
const Segment SGI163097[] = {{&S58196[64], 6}, {NULL}};
const Segment SGI163098[] = {{&S58197[34], 4}, {NULL}};
const Segment SGI163099[] = {{&S58197[27], 6}, {NULL}};
const Segment SGI163100[] = {{&S58197[91], 9}, {NULL}};
const Segment SGI163101[] = {{&S58197[64], 6}, {NULL}};
const Segment SGI163102[] = {{&S47902[44], 11}, {NULL}};
const Segment SGI163103[] = {{&S47902[44], 23}, {NULL}};
const Segment SGI163104[] = {{&S47902[30], 6}, {NULL}};
const Segment SGI163105[] = {{&S47902[116], 3}, {NULL}};
const Segment SGI163106[] = {{&S47902[81], 6}, {NULL}};
const Segment SGI163107[] = {{&S47902[151], 3}, {NULL}};
const Segment SGI163108[] = {{&S49556[44], 7}, {NULL}};
const Segment SGI163109[] = {{&S49556[44], 19}, {NULL}};
const Segment SGI163110[] = {{&S49556[30], 6}, {NULL}};
const Segment SGI163111[] = {{&S49556[112], 3}, {NULL}};
const Segment SGI163112[] = {{&S49556[77], 6}, {NULL}};
const Segment SGI163113[] = {{&S47903[44], 10}, {NULL}};
const Segment SGI163114[] = {{&S47903[44], 22}, {NULL}};
const Segment SGI163115[] = {{&S58200[34], 11}, {NULL}};
const Segment SGI163116[] = {{&S47903[30], 6}, {NULL}};
const Segment SGI163117[] = {{&S58200[27], 6}, {NULL}};
const Segment SGI163118[] = {{&S47903[115], 3}, {NULL}};
const Segment SGI163119[] = {{&S58200[98], 9}, {NULL}};
const Segment SGI163120[] = {{&S58200[71], 6}, {NULL}};
const Segment SGI163121[] = {{&S49557[44], 14}, {NULL}};
const Segment SGI163122[] = {{&S47903[80], 6}, {NULL}};
const Segment SGI163123[] = {{&S49557[44], 26}, {NULL}};
const Segment SGI163124[] = {{&S49557[30], 6}, {NULL}};
const Segment SGI163125[] = {{&S47903[150], 3}, {NULL}};
const Segment SGI163126[] = {{&S49557[119], 3}, {NULL}};
const Segment SGI163127[] = {{&S49557[84], 6}, {NULL}};
const Segment SGI163128[] = {{&S58201[34], 14}, {NULL}};
const Segment SGI163129[] = {{&S58201[27], 6}, {NULL}};
const Segment SGI163130[] = {{&S58201[101], 9}, {NULL}};
const Segment SGI163131[] = {{&S58201[74], 6}, {NULL}};
const Segment SGI163132[] = {{&S47905[44], 10}, {NULL}};
const Segment SGI163133[] = {{&S47905[44], 22}, {NULL}};
const Segment SGI163134[] = {{&S47905[30], 6}, {NULL}};
const Segment SGI163135[] = {{&S47905[115], 3}, {NULL}};
const Segment SGI163136[] = {{&S58202[34], 7}, {NULL}};
const Segment SGI163137[] = {{&S58202[27], 6}, {NULL}};
const Segment SGI163138[] = {{&S47905[80], 6}, {NULL}};
const Segment SGI163139[] = {{&S47905[150], 3}, {NULL}};
const Segment SGI163140[] = {{&S58202[94], 9}, {NULL}};
const Segment SGI163141[] = {{&S58202[67], 6}, {NULL}};
const Segment SGI163142[] = {{&S47906[44], 18}, {NULL}};
const Segment SGI163143[] = {{&S47906[44], 30}, {NULL}};
const Segment SGI163144[] = {{&S47906[30], 6}, {NULL}};
const Segment SGI163145[] = {{&S47906[123], 3}, {NULL}};
const Segment SGI163146[] = {{&S47906[88], 6}, {NULL}};
const Segment SGI163147[] = {{&S47906[158], 3}, {NULL}};
const Segment SGI163148[] = {{&S47907[44], 11}, {NULL}};
const Segment SGI163149[] = {{&S47907[44], 23}, {NULL}};
const Segment SGI163150[] = {{&S47907[30], 6}, {NULL}};
const Segment SGI163151[] = {{&S58206[34], 12}, {NULL}};
const Segment SGI163152[] = {{&S58206[27], 6}, {NULL}};
const Segment SGI163153[] = {{&S47907[116], 3}, {NULL}};
const Segment SGI163154[] = {{&S47907[81], 6}, {NULL}};
const Segment SGI163155[] = {{&S58206[99], 9}, {NULL}};
const Segment SGI163156[] = {{&S58206[72], 6}, {NULL}};
const Segment SGI163157[] = {{&S47907[151], 3}, {NULL}};
const Segment SGI163158[] = {{&S58207[34], 9}, {NULL}};
const Segment SGI163159[] = {{&S58207[27], 6}, {NULL}};
const Segment SGI163160[] = {{&S43905[34], 10}, {NULL}};
const Segment SGI163161[] = {{&S43905[27], 6}, {NULL}};
const Segment SGI163162[] = {{&S58207[96], 9}, {NULL}};
const Segment SGI163163[] = {{&S58207[69], 6}, {NULL}};
const Segment SGI163164[] = {{&S43905[97], 9}, {NULL}};
const Segment SGI163165[] = {{&S43905[70], 6}, {NULL}};
const Segment SGI163166[] = {{&S58209[34], 7}, {NULL}};
const Segment SGI163167[] = {{&S58209[27], 6}, {NULL}};
const Segment SGI163168[] = {{&S58209[94], 9}, {NULL}};
const Segment SGI163169[] = {{&S58209[67], 6}, {NULL}};
const Segment SGI163170[] = {{&S58210[34], 18}, {NULL}};
const Segment SGI163171[] = {{&S58210[27], 6}, {NULL}};
const Segment SGI163172[] = {{&S58210[105], 9}, {NULL}};
const Segment SGI163173[] = {{&S58210[78], 6}, {NULL}};
const Segment SGI163174[] = {{&S43910[34], 10}, {NULL}};
const Segment SGI163175[] = {{&S43910[27], 6}, {NULL}};
const Segment SGI163176[] = {{&S43910[97], 9}, {NULL}};
const Segment SGI163177[] = {{&S43910[70], 6}, {NULL}};
const Segment SGI163178[] = {{&S58211[34], 5}, {NULL}};
const Segment SGI163179[] = {{&S58211[27], 6}, {NULL}};
const Segment SGI163180[] = {{&S58211[92], 9}, {NULL}};
const Segment SGI163181[] = {{&S58211[65], 6}, {NULL}};
const Segment SGI163182[] = {{&S43912[34], 10}, {NULL}};
const Segment SGI163183[] = {{&S43912[27], 6}, {NULL}};
const Segment SGI163184[] = {{&S43912[97], 9}, {NULL}};
const Segment SGI163185[] = {{&S43912[70], 6}, {NULL}};
const Segment SGI163186[] = {{&S58213[34], 7}, {NULL}};
const Segment SGI163187[] = {{&S58213[27], 6}, {NULL}};
const Segment SGI163188[] = {{&S58213[94], 9}, {NULL}};
const Segment SGI163189[] = {{&S58213[67], 6}, {NULL}};
const Segment SGI163190[] = {{&S58214[34], 7}, {NULL}};
const Segment SGI163191[] = {{&S58214[27], 6}, {NULL}};
const Segment SGI163192[] = {{&S58214[94], 9}, {NULL}};
const Segment SGI163193[] = {{&S58214[67], 6}, {NULL}};
const Segment SGI163194[] = {{&S58215[34], 6}, {NULL}};
const Segment SGI163195[] = {{&S58215[27], 6}, {NULL}};
const Segment SGI163196[] = {{&S58215[93], 9}, {NULL}};
const Segment SGI163197[] = {{&S58215[66], 6}, {NULL}};
const Segment SGI163198[] = {{&S58216[34], 7}, {NULL}};
const Segment SGI163199[] = {{&S58216[27], 6}, {NULL}};
const Segment SGI163200[] = {{&S58216[94], 9}, {NULL}};
const Segment SGI163201[] = {{&S58216[67], 6}, {NULL}};
const Segment SGI163202[] = {{&S58218[34], 9}, {NULL}};
const Segment SGI163203[] = {{&S58218[27], 6}, {NULL}};
const Segment SGI163204[] = {{&S58218[96], 9}, {NULL}};
const Segment SGI163205[] = {{&S58218[69], 6}, {NULL}};
const Segment SGI163206[] = {{&S58219[34], 11}, {NULL}};
const Segment SGI163207[] = {{&S58219[27], 6}, {NULL}};
const Segment SGI163208[] = {{&S58219[98], 9}, {NULL}};
const Segment SGI163209[] = {{&S58219[71], 6}, {NULL}};
const Segment SGI163210[] = {{&S58220[34], 8}, {NULL}};
const Segment SGI163211[] = {{&S58220[27], 6}, {NULL}};
const Segment SGI163212[] = {{&S58220[95], 9}, {NULL}};
const Segment SGI163213[] = {{&S58220[68], 6}, {NULL}};
const Segment SGI163214[] = {{&S58221[34], 10}, {NULL}};
const Segment SGI163215[] = {{&S58221[27], 6}, {NULL}};
const Segment SGI163216[] = {{&S58221[97], 9}, {NULL}};
const Segment SGI163217[] = {{&S58221[70], 6}, {NULL}};
const Segment SGI163218[] = {{&S47008[42], 20}, {NULL}};
const Segment SGI163219[] = {{&S47008[42], 32}, {NULL}};
const Segment SGI163220[] = {{&S47008[30], 4}, {NULL}};
const Segment SGI163221[] = {{&S58222[34], 6}, {NULL}};
const Segment SGI163222[] = {{&S58222[27], 6}, {NULL}};
const Segment SGI163223[] = {{&S47008[123], 3}, {NULL}};
const Segment SGI163224[] = {{&S47008[88], 6}, {NULL}};
const Segment SGI163225[] = {{&S58222[93], 9}, {NULL}};
const Segment SGI163226[] = {{&S58222[66], 6}, {NULL}};
const Segment SGI163227[] = {{&S58223[34], 9}, {NULL}};
const Segment SGI163228[] = {{&S58223[27], 6}, {NULL}};
const Segment SGI163229[] = {{&S58223[98], 9}, {NULL}};
const Segment SGI163230[] = {{&S58223[71], 6}, {NULL}};
const Segment SGI163231[] = {{&S47011[36], 20}, {NULL}};
const Segment SGI163232[] = {{&S47011[31], 4}, {NULL}};
const Segment SGI163233[] = {{&S58243[44], 7}, {NULL}};
const Segment SGI163234[] = {{&S58243[44], 19}, {NULL}};
const Segment SGI163235[] = {{&S58243[30], 6}, {NULL}};
const Segment SGI163236[] = {{&S58243[31], 3}, {NULL}};
const Segment SGI163237[] = {{&S58243[77], 6}, {NULL}};
const Segment SGI163238[] = {{&S58244[38], 7}, {NULL}};
const Segment SGI163239[] = {{&S58244[31], 6}, {NULL}};
const Segment SGI163240[] = {{&S58250[34], 11}, {NULL}};
const Segment SGI163241[] = {{&S58250[27], 6}, {NULL}};
const Segment SGI163242[] = {{&S58250[98], 9}, {NULL}};
const Segment SGI163243[] = {{&S58250[71], 6}, {NULL}};
const Segment SGI163244[] = {{&S49635[34], 10}, {NULL}};
const Segment SGI163245[] = {{&S49635[27], 6}, {NULL}};
const Segment SGI163246[] = {{&S49635[97], 9}, {NULL}};
const Segment SGI163247[] = {{&S49635[70], 6}, {NULL}};
const Segment SGI163248[] = {{&S49637[34], 11}, {NULL}};
const Segment SGI163249[] = {{&S49637[27], 6}, {NULL}};
const Segment SGI163250[] = {{&S49637[98], 9}, {NULL}};
const Segment SGI163251[] = {{&S49637[71], 6}, {NULL}};
const Segment SGI163252[] = {{&S49639[34], 11}, {NULL}};
const Segment SGI163253[] = {{&S49639[27], 6}, {NULL}};
const Segment SGI163254[] = {{&S49639[98], 9}, {NULL}};
const Segment SGI163255[] = {{&S49639[71], 6}, {NULL}};
const Segment SGI163256[] = {{&S49642[34], 12}, {NULL}};
const Segment SGI163257[] = {{&S49642[27], 6}, {NULL}};
const Segment SGI163258[] = {{&S49642[99], 9}, {NULL}};
const Segment SGI163259[] = {{&S49642[72], 6}, {NULL}};
const Segment SGI163260[] = {{&S58282[34], 7}, {NULL}};
const Segment SGI163261[] = {{&S58282[27], 6}, {NULL}};
const Segment SGI163262[] = {{&S58282[96], 9}, {NULL}};
const Segment SGI163263[] = {{&S58282[69], 6}, {NULL}};
const Segment SGI163264[] = {{&S34661[34], 14}, {NULL}};
const Segment SGI163265[] = {{&S34661[27], 6}, {NULL}};
const Segment SGI163266[] = {{&S34661[100], 9}, {NULL}};
const Segment SGI163267[] = {{&S34661[73], 6}, {NULL}};
const Segment SGI163268[] = {{&S34664[34], 14}, {NULL}};
const Segment SGI163269[] = {{&S34664[27], 6}, {NULL}};
const Segment SGI163270[] = {{&S34664[100], 9}, {NULL}};
const Segment SGI163271[] = {{&S34664[73], 6}, {NULL}};
const Segment SGI163272[] = {{&S58302[34], 7}, {NULL}};
const Segment SGI163273[] = {{&S58302[27], 6}, {NULL}};
const Segment SGI163274[] = {{&S58302[96], 9}, {NULL}};
const Segment SGI163275[] = {{&S58302[69], 6}, {NULL}};
const Segment SGI163276[] = {{&S58324[44], 15}, {NULL}};
const Segment SGI163277[] = {{&S58324[44], 27}, {NULL}};
const Segment SGI163278[] = {{&S58324[30], 6}, {NULL}};
const Segment SGI163279[] = {{&S58324[31], 3}, {NULL}};
const Segment SGI163280[] = {{&S58324[85], 6}, {NULL}};
const Segment SGI163281[] = {{&S49700[34], 6}, {NULL}};
const Segment SGI163282[] = {{&S49700[27], 6}, {NULL}};
const Segment SGI163283[] = {{&S58325[34], 6}, {NULL}};
const Segment SGI163284[] = {{&S58325[27], 6}, {NULL}};
const Segment SGI163285[] = {{&S49700[93], 9}, {NULL}};
const Segment SGI163286[] = {{&S49700[66], 6}, {NULL}};
const Segment SGI163287[] = {{&S58325[93], 9}, {NULL}};
const Segment SGI163288[] = {{&S58325[66], 6}, {NULL}};
const Segment SGI163289[] = {{&S58326[34], 7}, {NULL}};
const Segment SGI163290[] = {{&S58326[27], 6}, {NULL}};
const Segment SGI163291[] = {{&S58326[98], 9}, {NULL}};
const Segment SGI163292[] = {{&S58326[71], 6}, {NULL}};
const Segment SGI163293[] = {{&S58331[34], 10}, {NULL}};
const Segment SGI163294[] = {{&S58331[27], 6}, {NULL}};
const Segment SGI163295[] = {{&S58331[103], 9}, {NULL}};
const Segment SGI163296[] = {{&S58331[76], 6}, {NULL}};
const Segment SGI163297[] = {{&S47920[34], 7}, {NULL}};
const Segment SGI163298[] = {{&S47920[27], 6}, {NULL}};
const Segment SGI163299[] = {{&S47920[112], 9}, {NULL}};
const Segment SGI163300[] = {{&S47920[85], 6}, {NULL}};
const Segment SGI163301[] = {{&S58369[34], 7}, {NULL}};
const Segment SGI163302[] = {{&S58369[27], 6}, {NULL}};
const Segment SGI163303[] = {{&S58369[99], 9}, {NULL}};
const Segment SGI163304[] = {{&S58369[72], 6}, {NULL}};
const Segment SGI163305[] = {{&S58370[34], 7}, {NULL}};
const Segment SGI163306[] = {{&S58370[27], 6}, {NULL}};
const Segment SGI163307[] = {{&S58370[93], 9}, {NULL}};
const Segment SGI163308[] = {{&S58370[66], 6}, {NULL}};
const Segment SGI163309[] = {{&S58379[44], 18}, {NULL}};
const Segment SGI163310[] = {{&S58379[44], 30}, {NULL}};
const Segment SGI163311[] = {{&S58379[30], 6}, {NULL}};
const Segment SGI163312[] = {{&S58379[31], 3}, {NULL}};
const Segment SGI163313[] = {{&S58379[88], 6}, {NULL}};
const Segment SGI163314[] = {{&S58380[44], 19}, {NULL}};
const Segment SGI163315[] = {{&S58380[44], 31}, {NULL}};
const Segment SGI163316[] = {{&S58380[30], 6}, {NULL}};
const Segment SGI163317[] = {{&S58380[31], 3}, {NULL}};
const Segment SGI163318[] = {{&S58380[89], 6}, {NULL}};
const Segment SGI163319[] = {{&S49764[34], 6}, {NULL}};
const Segment SGI163320[] = {{&S49764[27], 6}, {NULL}};
const Segment SGI163321[] = {{&S49764[93], 9}, {NULL}};
const Segment SGI163322[] = {{&S49764[66], 6}, {NULL}};
const Segment SGI163323[] = {{&S49799[34], 12}, {NULL}};
const Segment SGI163324[] = {{&S49799[27], 6}, {NULL}};
const Segment SGI163325[] = {{&S49799[99], 9}, {NULL}};
const Segment SGI163326[] = {{&S49799[72], 6}, {NULL}};
const Segment SGI163327[] = {{&S58422[34], 7}, {NULL}};
const Segment SGI163328[] = {{&S58422[27], 6}, {NULL}};
const Segment SGI163329[] = {{&S58422[99], 9}, {NULL}};
const Segment SGI163330[] = {{&S58422[72], 6}, {NULL}};
const Segment SGI163331[] = {{&S58423[34], 8}, {NULL}};
const Segment SGI163332[] = {{&S58423[27], 6}, {NULL}};
const Segment SGI163333[] = {{&S58423[99], 9}, {NULL}};
const Segment SGI163334[] = {{&S58423[72], 6}, {NULL}};
const Segment SGI163335[] = {{&S58424[34], 7}, {NULL}};
const Segment SGI163336[] = {{&S58424[27], 6}, {NULL}};
const Segment SGI163337[] = {{&S58424[99], 9}, {NULL}};
const Segment SGI163338[] = {{&S58424[72], 6}, {NULL}};
const Segment SGI163339[] = {{&S58425[34], 8}, {NULL}};
const Segment SGI163340[] = {{&S58425[27], 6}, {NULL}};
const Segment SGI163341[] = {{&S58425[99], 9}, {NULL}};
const Segment SGI163342[] = {{&S58425[72], 6}, {NULL}};
const Segment SGI163343[] = {{&S49814[34], 25}, {NULL}};
const Segment SGI163344[] = {{&S49814[27], 6}, {NULL}};
const Segment SGI163345[] = {{&S49814[112], 9}, {NULL}};
const Segment SGI163346[] = {{&S49814[85], 6}, {NULL}};
const Segment SGI163347[] = {{&S48008[34], 6}, {NULL}};
const Segment SGI163348[] = {{&S48008[27], 6}, {NULL}};
const Segment SGI163349[] = {{&S48008[93], 9}, {NULL}};
const Segment SGI163350[] = {{&S48008[66], 6}, {NULL}};
const Segment SGI163351[] = {{&S48020[34], 10}, {NULL}};
const Segment SGI163352[] = {{&S48020[27], 6}, {NULL}};
const Segment SGI163353[] = {{&S48020[97], 9}, {NULL}};
const Segment SGI163354[] = {{&S48020[70], 6}, {NULL}};
const Segment SGI163355[] = {{&S48021[34], 14}, {NULL}};
const Segment SGI163356[] = {{&S48021[27], 6}, {NULL}};
const Segment SGI163357[] = {{&S48021[101], 9}, {NULL}};
const Segment SGI163358[] = {{&S48021[74], 6}, {NULL}};
const Segment SGI163359[] = {{&S48022[34], 13}, {NULL}};
const Segment SGI163360[] = {{&S48022[27], 6}, {NULL}};
const Segment SGI163361[] = {{&S48022[100], 9}, {NULL}};
const Segment SGI163362[] = {{&S48022[73], 6}, {NULL}};
const Segment SGI163363[] = {{&S49839[34], 10}, {NULL}};
const Segment SGI163364[] = {{&S49839[27], 6}, {NULL}};
const Segment SGI163365[] = {{&S49839[97], 9}, {NULL}};
const Segment SGI163366[] = {{&S49839[70], 6}, {NULL}};
const Segment SGI163367[] = {{&S48045[34], 10}, {NULL}};
const Segment SGI163368[] = {{&S48045[27], 6}, {NULL}};
const Segment SGI163369[] = {{&S48045[97], 9}, {NULL}};
const Segment SGI163370[] = {{&S48045[70], 6}, {NULL}};
const Segment SGI163371[] = {{&S48050[34], 7}, {NULL}};
const Segment SGI163372[] = {{&S48050[27], 6}, {NULL}};
const Segment SGI163373[] = {{&S48050[104], 9}, {NULL}};
const Segment SGI163374[] = {{&S48050[77], 6}, {NULL}};
const Segment SGI163375[] = {{&S49876[34], 10}, {NULL}};
const Segment SGI163376[] = {{&S49876[27], 6}, {NULL}};
const Segment SGI163377[] = {{&S49876[97], 9}, {NULL}};
const Segment SGI163378[] = {{&S49876[70], 6}, {NULL}};
const Segment SGI163379[] = {{&S34875[44], 10}, {NULL}};
const Segment SGI163380[] = {{&S34875[44], 21}, {NULL}};
const Segment SGI163381[] = {{&S34875[30], 6}, {NULL}};
const Segment SGI163382[] = {{&S34875[114], 3}, {NULL}};
const Segment SGI163383[] = {{&S34875[79], 6}, {NULL}};
const Segment SGI163384[] = {{&S34876[38], 10}, {NULL}};
const Segment SGI163385[] = {{&S34876[31], 6}, {NULL}};
const Segment SGI163386[] = {{&S34877[44], 10}, {NULL}};
const Segment SGI163387[] = {{&S34877[44], 22}, {NULL}};
const Segment SGI163388[] = {{&S34877[30], 6}, {NULL}};
const Segment SGI163389[] = {{&S48069[34], 12}, {NULL}};
const Segment SGI163390[] = {{&S48069[27], 6}, {NULL}};
const Segment SGI163391[] = {{&S34877[115], 3}, {NULL}};
const Segment SGI163392[] = {{&S34877[80], 6}, {NULL}};
const Segment SGI163393[] = {{&S48069[99], 9}, {NULL}};
const Segment SGI163394[] = {{&S48069[72], 6}, {NULL}};
const Segment SGI163395[] = {{&S34878[44], 10}, {NULL}};
const Segment SGI163396[] = {{&S34878[44], 22}, {NULL}};
const Segment SGI163397[] = {{&S34878[30], 6}, {NULL}};
const Segment SGI163398[] = {{&S48070[34], 7}, {NULL}};
const Segment SGI163399[] = {{&S34878[114], 3}, {NULL}};
const Segment SGI163400[] = {{&S48070[27], 6}, {NULL}};
const Segment SGI163401[] = {{&S34878[80], 5}, {NULL}};
const Segment SGI163402[] = {{&S48070[99], 9}, {NULL}};
const Segment SGI163403[] = {{&S48070[72], 6}, {NULL}};
const Segment SGI163404[] = {{&S34880[44], 10}, {NULL}};
const Segment SGI163405[] = {{&S34880[44], 22}, {NULL}};
const Segment SGI163406[] = {{&S34880[30], 6}, {NULL}};
const Segment SGI163407[] = {{&S34880[115], 3}, {NULL}};
const Segment SGI163408[] = {{&S49887[34], 12}, {NULL}};
const Segment SGI163409[] = {{&S34880[80], 6}, {NULL}};
const Segment SGI163410[] = {{&S49887[27], 6}, {NULL}};
const Segment SGI163411[] = {{&S49887[99], 9}, {NULL}};
const Segment SGI163412[] = {{&S49887[72], 6}, {NULL}};
const Segment SGI163413[] = {{&S48073[34], 10}, {NULL}};
const Segment SGI163414[] = {{&S48073[27], 6}, {NULL}};
const Segment SGI163415[] = {{&S48073[97], 9}, {NULL}};
const Segment SGI163416[] = {{&S48073[70], 6}, {NULL}};
const Segment SGI163417[] = {{&S34885[42], 10}, {NULL}};
const Segment SGI163418[] = {{&S34885[30], 6}, {NULL}};
const Segment SGI163419[] = {{&S49891[34], 10}, {NULL}};
const Segment SGI163420[] = {{&S34885[112], 3}, {NULL}};
const Segment SGI163421[] = {{&S49891[27], 6}, {NULL}};
const Segment SGI163422[] = {{&S34885[77], 6}, {NULL}};
const Segment SGI163423[] = {{&S49891[97], 9}, {NULL}};
const Segment SGI163424[] = {{&S49891[70], 6}, {NULL}};
const Segment SGI163425[] = {{&S34887[44], 10}, {NULL}};
const Segment SGI163426[] = {{&S34887[44], 22}, {NULL}};
const Segment SGI163427[] = {{&S34887[30], 6}, {NULL}};
const Segment SGI163428[] = {{&S34887[115], 3}, {NULL}};
const Segment SGI163429[] = {{&S34887[80], 6}, {NULL}};
const Segment SGI163430[] = {{&S34889[44], 7}, {NULL}};
const Segment SGI163431[] = {{&S34889[44], 19}, {NULL}};
const Segment SGI163432[] = {{&S34889[30], 6}, {NULL}};
const Segment SGI163433[] = {{&S34889[112], 3}, {NULL}};
const Segment SGI163434[] = {{&S34889[77], 6}, {NULL}};
const Segment SGI163435[] = {{&S34891[44], 10}, {NULL}};
const Segment SGI163436[] = {{&S34891[44], 22}, {NULL}};
const Segment SGI163437[] = {{&S34891[30], 6}, {NULL}};
const Segment SGI163438[] = {{&S34891[115], 3}, {NULL}};
const Segment SGI163439[] = {{&S34891[80], 6}, {NULL}};
const Segment SGI163440[] = {{&S34897[44], 7}, {NULL}};
const Segment SGI163441[] = {{&S34897[44], 18}, {NULL}};
const Segment SGI163442[] = {{&S34897[30], 6}, {NULL}};
const Segment SGI163443[] = {{&S34897[110], 3}, {NULL}};
const Segment SGI163444[] = {{&S34897[76], 5}, {NULL}};
const Segment SGI163445[] = {{&S49906[34], 10}, {NULL}};
const Segment SGI163446[] = {{&S49906[27], 6}, {NULL}};
const Segment SGI163447[] = {{&S49906[97], 9}, {NULL}};
const Segment SGI163448[] = {{&S49906[70], 6}, {NULL}};
const Segment SGI163449[] = {{&S34898[44], 20}, {NULL}};
const Segment SGI163450[] = {{&S34898[44], 32}, {NULL}};
const Segment SGI163451[] = {{&S34898[30], 6}, {NULL}};
const Segment SGI163452[] = {{&S34898[125], 3}, {NULL}};
const Segment SGI163453[] = {{&S34898[90], 6}, {NULL}};
const Segment SGI163454[] = {{&S34902[44], 10}, {NULL}};
const Segment SGI163455[] = {{&S34902[44], 31}, {NULL}};
const Segment SGI163456[] = {{&S34902[30], 6}, {NULL}};
const Segment SGI163457[] = {{&S34902[123], 3}, {NULL}};
const Segment SGI163458[] = {{&S34902[89], 5}, {NULL}};
const Segment SGI163459[] = {{&S34904[44], 10}, {NULL}};
const Segment SGI163460[] = {{&S34904[44], 21}, {NULL}};
const Segment SGI163461[] = {{&S34904[30], 6}, {NULL}};
const Segment SGI163462[] = {{&S34904[114], 3}, {NULL}};
const Segment SGI163463[] = {{&S34904[79], 6}, {NULL}};
const Segment SGI163464[] = {{&S58464[44], 10}, {NULL}};
const Segment SGI163465[] = {{&S58464[44], 22}, {NULL}};
const Segment SGI163466[] = {{&S58464[30], 6}, {NULL}};
const Segment SGI163467[] = {{&S58464[31], 3}, {NULL}};
const Segment SGI163468[] = {{&S58464[80], 6}, {NULL}};
const Segment SGI163469[] = {{&S58493[34], 10}, {NULL}};
const Segment SGI163470[] = {{&S58493[27], 6}, {NULL}};
const Segment SGI163471[] = {{&S48149[38], 10}, {NULL}};
const Segment SGI163472[] = {{&S48149[31], 6}, {NULL}};
const Segment SGI163473[] = {{&S58493[97], 9}, {NULL}};
const Segment SGI163474[] = {{&S58493[70], 6}, {NULL}};
const Segment SGI163475[] = {{&S48150[44], 10}, {NULL}};
const Segment SGI163476[] = {{&S48150[44], 71}, {NULL}};
const Segment SGI163477[] = {{&S48150[30], 6}, {NULL}};
const Segment SGI163478[] = {{&S58494[34], 7}, {NULL}};
const Segment SGI163479[] = {{&S58494[27], 6}, {NULL}};
const Segment SGI163480[] = {{&S58494[110], 9}, {NULL}};
const Segment SGI163481[] = {{&S58494[83], 6}, {NULL}};
const Segment SGI163482[] = {{&S58495[34], 7}, {NULL}};
const Segment SGI163483[] = {{&S58495[27], 6}, {NULL}};
const Segment SGI163484[] = {{&S58495[110], 9}, {NULL}};
const Segment SGI163485[] = {{&S58495[83], 6}, {NULL}};
const Segment SGI163486[] = {{&S48153[44], 10}, {NULL}};
const Segment SGI163487[] = {{&S48153[44], 22}, {NULL}};
const Segment SGI163488[] = {{&S48153[30], 6}, {NULL}};
const Segment SGI163489[] = {{&S48153[115], 3}, {NULL}};
const Segment SGI163490[] = {{&S48153[80], 6}, {NULL}};
const Segment SGI163491[] = {{&S58496[34], 7}, {NULL}};
const Segment SGI163492[] = {{&S58496[27], 6}, {NULL}};
const Segment SGI163493[] = {{&S58496[98], 9}, {NULL}};
const Segment SGI163494[] = {{&S58496[71], 6}, {NULL}};
const Segment SGI163495[] = {{&S48154[44], 10}, {NULL}};
const Segment SGI163496[] = {{&S48154[44], 22}, {NULL}};
const Segment SGI163497[] = {{&S48154[30], 6}, {NULL}};
const Segment SGI163498[] = {{&S48154[83], 6}, {NULL}};
const Segment SGI163499[] = {{&S58497[34], 7}, {NULL}};
const Segment SGI163500[] = {{&S44246[34], 10}, {NULL}};
const Segment SGI163501[] = {{&S58497[27], 6}, {NULL}};
const Segment SGI163502[] = {{&S44246[27], 6}, {NULL}};
const Segment SGI163503[] = {{&S48155[44], 10}, {NULL}};
const Segment SGI163504[] = {{&S58497[99], 9}, {NULL}};
const Segment SGI163505[] = {{&S48155[44], 22}, {NULL}};
const Segment SGI163506[] = {{&S44246[97], 9}, {NULL}};
const Segment SGI163507[] = {{&S58497[72], 6}, {NULL}};
const Segment SGI163508[] = {{&S48155[30], 6}, {NULL}};
const Segment SGI163509[] = {{&S44246[70], 6}, {NULL}};
const Segment SGI163510[] = {{&S48155[115], 3}, {NULL}};
const Segment SGI163511[] = {{&S48155[80], 6}, {NULL}};
const Segment SGI163512[] = {{&S58506[44], 9}, {NULL}};
const Segment SGI163513[] = {{&S58506[44], 21}, {NULL}};
const Segment SGI163514[] = {{&S58506[30], 6}, {NULL}};
const Segment SGI163515[] = {{&S58506[31], 3}, {NULL}};
const Segment SGI163516[] = {{&S58506[79], 6}, {NULL}};
const Segment SGI163517[] = {{&S58507[38], 9}, {NULL}};
const Segment SGI163518[] = {{&S58507[31], 6}, {NULL}};
const Segment SGI163519[] = {{&S48167[34], 11}, {NULL}};
const Segment SGI163520[] = {{&S48167[27], 6}, {NULL}};
const Segment SGI163521[] = {{&S48167[98], 9}, {NULL}};
const Segment SGI163522[] = {{&S48167[71], 6}, {NULL}};
const Segment SGI163523[] = {{&S58528[44], 8}, {NULL}};
const Segment SGI163524[] = {{&S58528[44], 25}, {NULL}};
const Segment SGI163525[] = {{&S58528[30], 6}, {NULL}};
const Segment SGI163526[] = {{&S58528[31], 3}, {NULL}};
const Segment SGI163527[] = {{&S58528[83], 6}, {NULL}};
const Segment SGI163528[] = {{&S50009[34], 10}, {NULL}};
const Segment SGI163529[] = {{&S50009[27], 6}, {NULL}};
const Segment SGI163530[] = {{&S58529[44], 12}, {NULL}};
const Segment SGI163531[] = {{&S58529[44], 29}, {NULL}};
const Segment SGI163532[] = {{&S58529[30], 6}, {NULL}};
const Segment SGI163533[] = {{&S50009[97], 9}, {NULL}};
const Segment SGI163534[] = {{&S50009[70], 6}, {NULL}};
const Segment SGI163535[] = {{&S58529[31], 3}, {NULL}};
const Segment SGI163536[] = {{&S58529[87], 6}, {NULL}};
const Segment SGI163537[] = {{&S50010[34], 11}, {NULL}};
const Segment SGI163538[] = {{&S50010[27], 6}, {NULL}};
const Segment SGI163539[] = {{&S58530[44], 12}, {NULL}};
const Segment SGI163540[] = {{&S58530[44], 29}, {NULL}};
const Segment SGI163541[] = {{&S58530[30], 6}, {NULL}};
const Segment SGI163542[] = {{&S50010[98], 9}, {NULL}};
const Segment SGI163543[] = {{&S50010[71], 6}, {NULL}};
const Segment SGI163544[] = {{&S58530[31], 3}, {NULL}};
const Segment SGI163545[] = {{&S58530[87], 6}, {NULL}};
const Segment SGI163546[] = {{&S58531[44], 8}, {NULL}};
const Segment SGI163547[] = {{&S58531[44], 25}, {NULL}};
const Segment SGI163548[] = {{&S58531[30], 6}, {NULL}};
const Segment SGI163549[] = {{&S58531[31], 3}, {NULL}};
const Segment SGI163550[] = {{&S58531[83], 6}, {NULL}};
const Segment SGI163551[] = {{&S58532[44], 12}, {NULL}};
const Segment SGI163552[] = {{&S58532[44], 29}, {NULL}};
const Segment SGI163553[] = {{&S58532[30], 6}, {NULL}};
const Segment SGI163554[] = {{&S58532[31], 3}, {NULL}};
const Segment SGI163555[] = {{&S58532[87], 6}, {NULL}};
const Segment SGI163556[] = {{&S58533[44], 8}, {NULL}};
const Segment SGI163557[] = {{&S58533[44], 25}, {NULL}};
const Segment SGI163558[] = {{&S58533[30], 6}, {NULL}};
const Segment SGI163559[] = {{&S58533[31], 3}, {NULL}};
const Segment SGI163560[] = {{&S58533[83], 6}, {NULL}};
const Segment SGI163561[] = {{&S58534[44], 18}, {NULL}};
const Segment SGI163562[] = {{&S58534[44], 30}, {NULL}};
const Segment SGI163563[] = {{&S58534[30], 6}, {NULL}};
const Segment SGI163564[] = {{&S58534[31], 3}, {NULL}};
const Segment SGI163565[] = {{&S58534[88], 6}, {NULL}};
const Segment SGI163566[] = {{&S58535[38], 18}, {NULL}};
const Segment SGI163567[] = {{&S58535[31], 6}, {NULL}};
const Segment SGI163568[] = {{&S58537[34], 18}, {NULL}};
const Segment SGI163569[] = {{&S58537[27], 6}, {NULL}};
const Segment SGI163570[] = {{&S58537[105], 9}, {NULL}};
const Segment SGI163571[] = {{&S58537[78], 6}, {NULL}};
const Segment SGI163572[] = {{&S50024[34], 12}, {NULL}};
const Segment SGI163573[] = {{&S50024[27], 6}, {NULL}};
const Segment SGI163574[] = {{&S50024[99], 9}, {NULL}};
const Segment SGI163575[] = {{&S50024[72], 6}, {NULL}};
const Segment SGI163576[] = {{&S58540[44], 9}, {NULL}};
const Segment SGI163577[] = {{&S58540[44], 21}, {NULL}};
const Segment SGI163578[] = {{&S58540[30], 6}, {NULL}};
const Segment SGI163579[] = {{&S58540[31], 3}, {NULL}};
const Segment SGI163580[] = {{&S58540[79], 6}, {NULL}};
const Segment SGI163581[] = {{&S58541[38], 9}, {NULL}};
const Segment SGI163582[] = {{&S58541[31], 6}, {NULL}};
const Segment SGI163583[] = {{&S58542[34], 7}, {NULL}};
const Segment SGI163584[] = {{&S58542[27], 6}, {NULL}};
const Segment SGI163585[] = {{&S58542[108], 9}, {NULL}};
const Segment SGI163586[] = {{&S58542[81], 6}, {NULL}};
const Segment SGI163587[] = {{&S58543[34], 15}, {NULL}};
const Segment SGI163588[] = {{&S58543[27], 6}, {NULL}};
const Segment SGI163589[] = {{&S58543[118], 9}, {NULL}};
const Segment SGI163590[] = {{&S58543[91], 6}, {NULL}};
const Segment SGI163591[] = {{&S58546[34], 10}, {NULL}};
const Segment SGI163592[] = {{&S58546[27], 6}, {NULL}};
const Segment SGI163593[] = {{&S58546[97], 9}, {NULL}};
const Segment SGI163594[] = {{&S58546[70], 6}, {NULL}};
const Segment SGI163595[] = {{&S58547[44], 8}, {NULL}};
const Segment SGI163596[] = {{&S58547[44], 20}, {NULL}};
const Segment SGI163597[] = {{&S58547[30], 6}, {NULL}};
const Segment SGI163598[] = {{&S58547[31], 3}, {NULL}};
const Segment SGI163599[] = {{&S58547[78], 6}, {NULL}};
const Segment SGI163600[] = {{&S58556[44], 8}, {NULL}};
const Segment SGI163601[] = {{&S58556[44], 20}, {NULL}};
const Segment SGI163602[] = {{&S58556[30], 6}, {NULL}};
const Segment SGI163603[] = {{&S58556[31], 3}, {NULL}};
const Segment SGI163604[] = {{&S58556[78], 6}, {NULL}};
const Segment SGI163605[] = {{&S48241[44], 6}, {NULL}};
const Segment SGI163606[] = {{&S48241[44], 18}, {NULL}};
const Segment SGI163607[] = {{&S48241[30], 6}, {NULL}};
const Segment SGI163608[] = {{&S48241[111], 3}, {NULL}};
const Segment SGI163609[] = {{&S48241[76], 6}, {NULL}};
const Segment SGI163610[] = {{&S50059[34], 7}, {NULL}};
const Segment SGI163611[] = {{&S50059[27], 6}, {NULL}};
const Segment SGI163612[] = {{&S50059[110], 9}, {NULL}};
const Segment SGI163613[] = {{&S50059[83], 6}, {NULL}};
const Segment SGI163614[] = {{&S48245[34], 10}, {NULL}};
const Segment SGI163615[] = {{&S48245[27], 6}, {NULL}};
const Segment SGI163616[] = {{&S48245[97], 9}, {NULL}};
const Segment SGI163617[] = {{&S48245[70], 6}, {NULL}};
const Segment SGI163618[] = {{&S48246[34], 10}, {NULL}};
const Segment SGI163619[] = {{&S48246[27], 6}, {NULL}};
const Segment SGI163620[] = {{&S48246[97], 9}, {NULL}};
const Segment SGI163621[] = {{&S48246[70], 6}, {NULL}};
const Segment SGI163622[] = {{&S48247[34], 10}, {NULL}};
const Segment SGI163623[] = {{&S48247[27], 6}, {NULL}};
const Segment SGI163624[] = {{&S48247[97], 9}, {NULL}};
const Segment SGI163625[] = {{&S48247[70], 6}, {NULL}};
const Segment SGI163626[] = {{&S58583[44], 14}, {NULL}};
const Segment SGI163627[] = {{&S58583[44], 26}, {NULL}};
const Segment SGI163628[] = {{&S58583[30], 6}, {NULL}};
const Segment SGI163629[] = {{&S58583[31], 3}, {NULL}};
const Segment SGI163630[] = {{&S58583[84], 6}, {NULL}};
const Segment SGI163631[] = {{&S58584[44], 10}, {NULL}};
const Segment SGI163632[] = {{&S58584[44], 22}, {NULL}};
const Segment SGI163633[] = {{&S58584[30], 6}, {NULL}};
const Segment SGI163634[] = {{&S58584[31], 3}, {NULL}};
const Segment SGI163635[] = {{&S58584[80], 6}, {NULL}};
const Segment SGI163636[] = {{&S58585[38], 10}, {NULL}};
const Segment SGI163637[] = {{&S58585[31], 6}, {NULL}};
const Segment SGI163638[] = {{&S58594[44], 13}, {NULL}};
const Segment SGI163639[] = {{&S58594[44], 25}, {NULL}};
const Segment SGI163640[] = {{&S58594[30], 6}, {NULL}};
const Segment SGI163641[] = {{&S58594[31], 3}, {NULL}};
const Segment SGI163642[] = {{&S58594[83], 6}, {NULL}};
const Segment SGI163643[] = {{&S58601[44], 10}, {NULL}};
const Segment SGI163644[] = {{&S58601[44], 22}, {NULL}};
const Segment SGI163645[] = {{&S58601[30], 6}, {NULL}};
const Segment SGI163646[] = {{&S58601[31], 3}, {NULL}};
const Segment SGI163647[] = {{&S58601[95], 6}, {NULL}};
const Segment SGI163648[] = {{&S58601[68], 8}, {NULL}};
const Segment SGI163649[] = {{&S58602[44], 10}, {NULL}};
const Segment SGI163650[] = {{&S58602[44], 22}, {NULL}};
const Segment SGI163651[] = {{&S58602[30], 6}, {NULL}};
const Segment SGI163652[] = {{&S58602[31], 3}, {NULL}};
const Segment SGI163653[] = {{&S58602[95], 6}, {NULL}};
const Segment SGI163654[] = {{&S58602[68], 8}, {NULL}};
const Segment SGI163655[] = {{&S58603[44], 10}, {NULL}};
const Segment SGI163656[] = {{&S58603[44], 22}, {NULL}};
const Segment SGI163657[] = {{&S58603[30], 6}, {NULL}};
const Segment SGI163658[] = {{&S58603[31], 3}, {NULL}};
const Segment SGI163659[] = {{&S58603[99], 6}, {NULL}};
const Segment SGI163660[] = {{&S58603[68], 8}, {NULL}};
const Segment SGI163661[] = {{&S58609[34], 10}, {NULL}};
const Segment SGI163662[] = {{&S58609[27], 6}, {NULL}};
const Segment SGI163663[] = {{&S58609[103], 9}, {NULL}};
const Segment SGI163664[] = {{&S58609[76], 6}, {NULL}};
const Segment SGI163665[] = {{&S50109[34], 8}, {NULL}};
const Segment SGI163666[] = {{&S50109[27], 6}, {NULL}};
const Segment SGI163667[] = {{&S50109[95], 9}, {NULL}};
const Segment SGI163668[] = {{&S50109[68], 6}, {NULL}};
const Segment SGI163669[] = {{&S58613[34], 19}, {NULL}};
const Segment SGI163670[] = {{&S58613[27], 6}, {NULL}};
const Segment SGI163671[] = {{&S58613[106], 9}, {NULL}};
const Segment SGI163672[] = {{&S58613[79], 6}, {NULL}};
const Segment SGI163673[] = {{&S58614[34], 19}, {NULL}};
const Segment SGI163674[] = {{&S58614[27], 6}, {NULL}};
const Segment SGI163675[] = {{&S50113[34], 6}, {NULL}};
const Segment SGI163676[] = {{&S50113[27], 6}, {NULL}};
const Segment SGI163677[] = {{&S58614[106], 9}, {NULL}};
const Segment SGI163678[] = {{&S58614[79], 6}, {NULL}};
const Segment SGI163679[] = {{&S50113[93], 9}, {NULL}};
const Segment SGI163680[] = {{&S50113[66], 6}, {NULL}};
const Segment SGI163681[] = {{&S50116[34], 23}, {NULL}};
const Segment SGI163682[] = {{&S50116[27], 6}, {NULL}};
const Segment SGI163683[] = {{&S50116[110], 9}, {NULL}};
const Segment SGI163684[] = {{&S50116[83], 6}, {NULL}};
const Segment SGI163685[] = {{&S50117[34], 6}, {NULL}};
const Segment SGI163686[] = {{&S50117[27], 6}, {NULL}};
const Segment SGI163687[] = {{&S50117[93], 9}, {NULL}};
const Segment SGI163688[] = {{&S50117[66], 6}, {NULL}};
const Segment SGI163689[] = {{&S50120[34], 8}, {NULL}};
const Segment SGI163690[] = {{&S50120[27], 6}, {NULL}};
const Segment SGI163691[] = {{&S50120[95], 9}, {NULL}};
const Segment SGI163692[] = {{&S50120[68], 6}, {NULL}};
const Segment SGI163693[] = {{&S50121[34], 8}, {NULL}};
const Segment SGI163694[] = {{&S50121[27], 6}, {NULL}};
const Segment SGI163695[] = {{&S50121[98], 9}, {NULL}};
const Segment SGI163696[] = {{&S50121[71], 6}, {NULL}};
const Segment SGI163697[] = {{&S50124[34], 10}, {NULL}};
const Segment SGI163698[] = {{&S50124[27], 6}, {NULL}};
const Segment SGI163699[] = {{&S50124[97], 9}, {NULL}};
const Segment SGI163700[] = {{&S50124[70], 6}, {NULL}};
const Segment SGI163701[] = {{&S50126[34], 6}, {NULL}};
const Segment SGI163702[] = {{&S50126[27], 6}, {NULL}};
const Segment SGI163703[] = {{&S50126[93], 9}, {NULL}};
const Segment SGI163704[] = {{&S50126[66], 6}, {NULL}};
const Segment SGI163705[] = {{&S50127[34], 10}, {NULL}};
const Segment SGI163706[] = {{&S50127[27], 6}, {NULL}};
const Segment SGI163707[] = {{&S50127[97], 9}, {NULL}};
const Segment SGI163708[] = {{&S50127[70], 6}, {NULL}};
const Segment SGI163709[] = {{&S50129[34], 25}, {NULL}};
const Segment SGI163710[] = {{&S50129[27], 6}, {NULL}};
const Segment SGI163711[] = {{&S58640[34], 18}, {NULL}};
const Segment SGI163712[] = {{&S50129[112], 9}, {NULL}};
const Segment SGI163713[] = {{&S58640[27], 6}, {NULL}};
const Segment SGI163714[] = {{&S50129[85], 6}, {NULL}};
const Segment SGI163715[] = {{&S58640[105], 9}, {NULL}};
const Segment SGI163716[] = {{&S58640[78], 6}, {NULL}};
const Segment SGI163717[] = {{&S50130[34], 7}, {NULL}};
const Segment SGI163718[] = {{&S50130[27], 6}, {NULL}};
const Segment SGI163719[] = {{&S50130[94], 9}, {NULL}};
const Segment SGI163720[] = {{&S50130[67], 6}, {NULL}};
const Segment SGI163721[] = {{&S50131[34], 8}, {NULL}};
const Segment SGI163722[] = {{&S50131[27], 6}, {NULL}};
const Segment SGI163723[] = {{&S50131[95], 9}, {NULL}};
const Segment SGI163724[] = {{&S50131[68], 6}, {NULL}};
const Segment SGI163725[] = {{&S58646[34], 6}, {NULL}};
const Segment SGI163726[] = {{&S58646[27], 6}, {NULL}};
const Segment SGI163727[] = {{&S50133[34], 4}, {NULL}};
const Segment SGI163728[] = {{&S58646[93], 9}, {NULL}};
const Segment SGI163729[] = {{&S50133[27], 6}, {NULL}};
const Segment SGI163730[] = {{&S58646[66], 6}, {NULL}};
const Segment SGI163731[] = {{&S50133[91], 9}, {NULL}};
const Segment SGI163732[] = {{&S50133[64], 6}, {NULL}};
const Segment SGI163733[] = {{&S58647[44], 9}, {NULL}};
const Segment SGI163734[] = {{&S58647[44], 21}, {NULL}};
const Segment SGI163735[] = {{&S58647[30], 6}, {NULL}};
const Segment SGI163736[] = {{&S58647[31], 3}, {NULL}};
const Segment SGI163737[] = {{&S58647[79], 6}, {NULL}};
const Segment SGI163738[] = {{&S50135[34], 19}, {NULL}};
const Segment SGI163739[] = {{&S50135[27], 6}, {NULL}};
const Segment SGI163740[] = {{&S50135[106], 9}, {NULL}};
const Segment SGI163741[] = {{&S50135[79], 6}, {NULL}};
const Segment SGI163742[] = {{&S58648[38], 9}, {NULL}};
const Segment SGI163743[] = {{&S58648[31], 6}, {NULL}};
const Segment SGI163744[] = {{&S50137[34], 10}, {NULL}};
const Segment SGI163745[] = {{&S50137[27], 6}, {NULL}};
const Segment SGI163746[] = {{&S50137[97], 9}, {NULL}};
const Segment SGI163747[] = {{&S50137[70], 6}, {NULL}};
const Segment SGI163748[] = {{&S50138[34], 10}, {NULL}};
const Segment SGI163749[] = {{&S58652[34], 10}, {NULL}};
const Segment SGI163750[] = {{&S50138[27], 6}, {NULL}};
const Segment SGI163751[] = {{&S58652[27], 6}, {NULL}};
const Segment SGI163752[] = {{&S50138[97], 9}, {NULL}};
const Segment SGI163753[] = {{&S50138[70], 6}, {NULL}};
const Segment SGI163754[] = {{&S58652[97], 9}, {NULL}};
const Segment SGI163755[] = {{&S58652[70], 6}, {NULL}};
const Segment SGI163756[] = {{&S50139[34], 11}, {NULL}};
const Segment SGI163757[] = {{&S50139[27], 6}, {NULL}};
const Segment SGI163758[] = {{&S50139[98], 9}, {NULL}};
const Segment SGI163759[] = {{&S50139[71], 6}, {NULL}};
const Segment SGI163760[] = {{&S50140[34], 20}, {NULL}};
const Segment SGI163761[] = {{&S50140[27], 6}, {NULL}};
const Segment SGI163762[] = {{&S50140[107], 9}, {NULL}};
const Segment SGI163763[] = {{&S50140[80], 6}, {NULL}};
const Segment SGI163764[] = {{&S50141[34], 9}, {NULL}};
const Segment SGI163765[] = {{&S50141[27], 6}, {NULL}};
const Segment SGI163766[] = {{&S50141[96], 9}, {NULL}};
const Segment SGI163767[] = {{&S50141[69], 6}, {NULL}};
const Segment SGI163768[] = {{&S50143[34], 7}, {NULL}};
const Segment SGI163769[] = {{&S50143[27], 6}, {NULL}};
const Segment SGI163770[] = {{&S50143[94], 9}, {NULL}};
const Segment SGI163771[] = {{&S50143[67], 6}, {NULL}};
const Segment SGI163772[] = {{&S48347[34], 10}, {NULL}};
const Segment SGI163773[] = {{&S48347[27], 6}, {NULL}};
const Segment SGI163774[] = {{&S48347[97], 9}, {NULL}};
const Segment SGI163775[] = {{&S48347[70], 6}, {NULL}};
const Segment SGI163776[] = {{&S48347[139], 8}, {NULL}};
const Segment SGI163777[] = {{&S50148[34], 12}, {NULL}};
const Segment SGI163778[] = {{&S50148[27], 6}, {NULL}};
const Segment SGI163779[] = {{&S48349[34], 10}, {NULL}};
const Segment SGI163780[] = {{&S48349[27], 6}, {NULL}};
const Segment SGI163781[] = {{&S50148[99], 9}, {NULL}};
const Segment SGI163782[] = {{&S50148[72], 6}, {NULL}};
const Segment SGI163783[] = {{&S48349[97], 9}, {NULL}};
const Segment SGI163784[] = {{&S48349[70], 6}, {NULL}};
const Segment SGI163785[] = {{&S48349[138], 8}, {NULL}};
const Segment SGI163786[] = {{&S48350[34], 10}, {NULL}};
const Segment SGI163787[] = {{&S48350[27], 6}, {NULL}};
const Segment SGI163788[] = {{&S48350[97], 9}, {NULL}};
const Segment SGI163789[] = {{&S48350[70], 6}, {NULL}};
const Segment SGI163790[] = {{&S48350[138], 8}, {NULL}};
const Segment SGI163791[] = {{&S48351[34], 18}, {NULL}};
const Segment SGI163792[] = {{&S48351[27], 6}, {NULL}};
const Segment SGI163793[] = {{&S50152[34], 11}, {NULL}};
const Segment SGI163794[] = {{&S50152[27], 6}, {NULL}};
const Segment SGI163795[] = {{&S48351[105], 9}, {NULL}};
const Segment SGI163796[] = {{&S48351[78], 6}, {NULL}};
const Segment SGI163797[] = {{&S50152[98], 9}, {NULL}};
const Segment SGI163798[] = {{&S48351[146], 8}, {NULL}};
const Segment SGI163799[] = {{&S50152[71], 6}, {NULL}};
const Segment SGI163800[] = {{&S48352[34], 10}, {NULL}};
const Segment SGI163801[] = {{&S48352[27], 6}, {NULL}};
const Segment SGI163802[] = {{&S48352[97], 9}, {NULL}};
const Segment SGI163803[] = {{&S48352[70], 6}, {NULL}};
const Segment SGI163804[] = {{&S48352[138], 8}, {NULL}};
const Segment SGI163805[] = {{&S50154[34], 11}, {NULL}};
const Segment SGI163806[] = {{&S50154[27], 6}, {NULL}};
const Segment SGI163807[] = {{&S50154[98], 9}, {NULL}};
const Segment SGI163808[] = {{&S50154[71], 6}, {NULL}};
const Segment SGI163809[] = {{&S48354[34], 10}, {NULL}};
const Segment SGI163810[] = {{&S48354[27], 6}, {NULL}};
const Segment SGI163811[] = {{&S50155[34], 8}, {NULL}};
const Segment SGI163812[] = {{&S50155[27], 6}, {NULL}};
const Segment SGI163813[] = {{&S48354[97], 9}, {NULL}};
const Segment SGI163814[] = {{&S48354[70], 6}, {NULL}};
const Segment SGI163815[] = {{&S50155[95], 9}, {NULL}};
const Segment SGI163816[] = {{&S48354[138], 8}, {NULL}};
const Segment SGI163817[] = {{&S50155[68], 6}, {NULL}};
const Segment SGI163818[] = {{&S48356[34], 11}, {NULL}};
const Segment SGI163819[] = {{&S48356[27], 6}, {NULL}};
const Segment SGI163820[] = {{&S50157[34], 8}, {NULL}};
const Segment SGI163821[] = {{&S50157[27], 6}, {NULL}};
const Segment SGI163822[] = {{&S48356[98], 9}, {NULL}};
const Segment SGI163823[] = {{&S48356[71], 6}, {NULL}};
const Segment SGI163824[] = {{&S50157[95], 9}, {NULL}};
const Segment SGI163825[] = {{&S50157[68], 6}, {NULL}};
const Segment SGI163826[] = {{&S48356[139], 8}, {NULL}};
const Segment SGI163827[] = {{&S48357[34], 11}, {NULL}};
const Segment SGI163828[] = {{&S48357[27], 6}, {NULL}};
const Segment SGI163829[] = {{&S50158[34], 21}, {NULL}};
const Segment SGI163830[] = {{&S50158[27], 6}, {NULL}};
const Segment SGI163831[] = {{&S48357[98], 9}, {NULL}};
const Segment SGI163832[] = {{&S48357[71], 6}, {NULL}};
const Segment SGI163833[] = {{&S50158[108], 9}, {NULL}};
const Segment SGI163834[] = {{&S50158[81], 6}, {NULL}};
const Segment SGI163835[] = {{&S48357[139], 8}, {NULL}};
const Segment SGI163836[] = {{&S48358[34], 10}, {NULL}};
const Segment SGI163837[] = {{&S48358[27], 6}, {NULL}};
const Segment SGI163838[] = {{&S48358[97], 9}, {NULL}};
const Segment SGI163839[] = {{&S48358[70], 6}, {NULL}};
const Segment SGI163840[] = {{&S48358[138], 8}, {NULL}};
const Segment SGI163841[] = {{&S50160[34], 9}, {NULL}};
const Segment SGI163842[] = {{&S50160[27], 6}, {NULL}};
const Segment SGI163843[] = {{&S50160[96], 9}, {NULL}};
const Segment SGI163844[] = {{&S50160[69], 6}, {NULL}};
const Segment SGI163845[] = {{&S50161[34], 13}, {NULL}};
const Segment SGI163846[] = {{&S50161[27], 6}, {NULL}};
const Segment SGI163847[] = {{&S50161[100], 9}, {NULL}};
const Segment SGI163848[] = {{&S50161[73], 6}, {NULL}};
const Segment SGI163849[] = {{&S50162[34], 13}, {NULL}};
const Segment SGI163850[] = {{&S50162[27], 6}, {NULL}};
const Segment SGI163851[] = {{&S50162[100], 9}, {NULL}};
const Segment SGI163852[] = {{&S50162[73], 6}, {NULL}};
const Segment SGI163853[] = {{&S58696[34], 18}, {NULL}};
const Segment SGI163854[] = {{&S58696[27], 6}, {NULL}};
const Segment SGI163855[] = {{&S58696[105], 9}, {NULL}};
const Segment SGI163856[] = {{&S58696[78], 6}, {NULL}};
const Segment SGI163857[] = {{&S50164[34], 11}, {NULL}};
const Segment SGI163858[] = {{&S50164[27], 6}, {NULL}};
const Segment SGI163859[] = {{&S50164[98], 9}, {NULL}};
const Segment SGI163860[] = {{&S50164[71], 6}, {NULL}};
const Segment SGI163861[] = {{&S58698[34], 10}, {NULL}};
const Segment SGI163862[] = {{&S58698[27], 6}, {NULL}};
const Segment SGI163863[] = {{&S58698[97], 9}, {NULL}};
const Segment SGI163864[] = {{&S58698[70], 6}, {NULL}};
const Segment SGI163865[] = {{&S50167[34], 11}, {NULL}};
const Segment SGI163866[] = {{&S50167[27], 6}, {NULL}};
const Segment SGI163867[] = {{&S50167[98], 9}, {NULL}};
const Segment SGI163868[] = {{&S50167[71], 6}, {NULL}};
const Segment SGI163869[] = {{&S50168[34], 6}, {NULL}};
const Segment SGI163870[] = {{&S50168[27], 6}, {NULL}};
const Segment SGI163871[] = {{&S50168[93], 9}, {NULL}};
const Segment SGI163872[] = {{&S50168[66], 6}, {NULL}};
const Segment SGI163873[] = {{&S58704[34], 10}, {NULL}};
const Segment SGI163874[] = {{&S58704[27], 6}, {NULL}};
const Segment SGI163875[] = {{&S58704[97], 9}, {NULL}};
const Segment SGI163876[] = {{&S58704[70], 6}, {NULL}};
const Segment SGI163877[] = {{&S58705[34], 11}, {NULL}};
const Segment SGI163878[] = {{&S58705[27], 6}, {NULL}};
const Segment SGI163879[] = {{&S58705[98], 9}, {NULL}};
const Segment SGI163880[] = {{&S58705[71], 6}, {NULL}};
const Segment SGI163881[] = {{&S50172[34], 6}, {NULL}};
const Segment SGI163882[] = {{&S50172[27], 6}, {NULL}};
const Segment SGI163883[] = {{&S58706[34], 10}, {NULL}};
const Segment SGI163884[] = {{&S58706[27], 6}, {NULL}};
const Segment SGI163885[] = {{&S50172[93], 9}, {NULL}};
const Segment SGI163886[] = {{&S50172[66], 6}, {NULL}};
const Segment SGI163887[] = {{&S58706[97], 9}, {NULL}};
const Segment SGI163888[] = {{&S58706[70], 6}, {NULL}};
const Segment SGI163889[] = {{&S48381[34], 10}, {NULL}};
const Segment SGI163890[] = {{&S48381[27], 6}, {NULL}};
const Segment SGI163891[] = {{&S48381[97], 9}, {NULL}};
const Segment SGI163892[] = {{&S48381[70], 6}, {NULL}};
const Segment SGI163893[] = {{&S48381[138], 8}, {NULL}};
const Segment SGI163894[] = {{&S48383[34], 11}, {NULL}};
const Segment SGI163895[] = {{&S48383[27], 6}, {NULL}};
const Segment SGI163896[] = {{&S48383[98], 9}, {NULL}};
const Segment SGI163897[] = {{&S48383[71], 6}, {NULL}};
const Segment SGI163898[] = {{&S48383[139], 8}, {NULL}};
const Segment SGI163899[] = {{&S48384[34], 11}, {NULL}};
const Segment SGI163900[] = {{&S48384[27], 6}, {NULL}};
const Segment SGI163901[] = {{&S48384[98], 9}, {NULL}};
const Segment SGI163902[] = {{&S48384[71], 6}, {NULL}};
const Segment SGI163903[] = {{&S48384[139], 8}, {NULL}};
const Segment SGI163904[] = {{&S48385[34], 18}, {NULL}};
const Segment SGI163905[] = {{&S48385[27], 6}, {NULL}};
const Segment SGI163906[] = {{&S48385[105], 9}, {NULL}};
const Segment SGI163907[] = {{&S48385[78], 6}, {NULL}};
const Segment SGI163908[] = {{&S48385[146], 8}, {NULL}};
const Segment SGI163909[] = {{&S48386[34], 11}, {NULL}};
const Segment SGI163910[] = {{&S48386[27], 6}, {NULL}};
const Segment SGI163911[] = {{&S48386[98], 9}, {NULL}};
const Segment SGI163912[] = {{&S48386[71], 6}, {NULL}};
const Segment SGI163913[] = {{&S48386[139], 8}, {NULL}};
const Segment SGI163914[] = {{&S58719[44], 7}, {NULL}};
const Segment SGI163915[] = {{&S58719[44], 19}, {NULL}};
const Segment SGI163916[] = {{&S58719[30], 6}, {NULL}};
const Segment SGI163917[] = {{&S58719[31], 3}, {NULL}};
const Segment SGI163918[] = {{&S58719[77], 6}, {NULL}};
const Segment SGI163919[] = {{&S48387[34], 10}, {NULL}};
const Segment SGI163920[] = {{&S48387[27], 6}, {NULL}};
const Segment SGI163921[] = {{&S48387[97], 9}, {NULL}};
const Segment SGI163922[] = {{&S48387[70], 6}, {NULL}};
const Segment SGI163923[] = {{&S48387[138], 8}, {NULL}};
const Segment SGI163924[] = {{&S58720[38], 7}, {NULL}};
const Segment SGI163925[] = {{&S58720[31], 6}, {NULL}};
const Segment SGI163926[] = {{&S48388[34], 11}, {NULL}};
const Segment SGI163927[] = {{&S48388[27], 6}, {NULL}};
const Segment SGI163928[] = {{&S48388[98], 9}, {NULL}};
const Segment SGI163929[] = {{&S48388[71], 6}, {NULL}};
const Segment SGI163930[] = {{&S48388[139], 8}, {NULL}};
const Segment SGI163931[] = {{&S48390[34], 11}, {NULL}};
const Segment SGI163932[] = {{&S48390[27], 6}, {NULL}};
const Segment SGI163933[] = {{&S48390[98], 9}, {NULL}};
const Segment SGI163934[] = {{&S48390[71], 6}, {NULL}};
const Segment SGI163935[] = {{&S48390[139], 8}, {NULL}};
const Segment SGI163936[] = {{&S50196[34], 10}, {NULL}};
const Segment SGI163937[] = {{&S50196[27], 6}, {NULL}};
const Segment SGI163938[] = {{&S50196[97], 9}, {NULL}};
const Segment SGI163939[] = {{&S50196[70], 6}, {NULL}};
const Segment SGI163940[] = {{&S50198[34], 11}, {NULL}};
const Segment SGI163941[] = {{&S50198[27], 6}, {NULL}};
const Segment SGI163942[] = {{&S50198[98], 9}, {NULL}};
const Segment SGI163943[] = {{&S50198[71], 6}, {NULL}};
const Segment SGI163944[] = {{&S58741[44], 6}, {NULL}};
const Segment SGI163945[] = {{&S58741[44], 18}, {NULL}};
const Segment SGI163946[] = {{&S58741[30], 6}, {NULL}};
const Segment SGI163947[] = {{&S58741[31], 3}, {NULL}};
const Segment SGI163948[] = {{&S58741[76], 6}, {NULL}};
const Segment SGI163949[] = {{&S48420[44], 11}, {NULL}};
const Segment SGI163950[] = {{&S48420[44], 23}, {NULL}};
const Segment SGI163951[] = {{&S48420[30], 6}, {NULL}};
const Segment SGI163952[] = {{&S48420[116], 3}, {NULL}};
const Segment SGI163953[] = {{&S48420[81], 6}, {NULL}};
const Segment SGI163954[] = {{&S48421[44], 11}, {NULL}};
const Segment SGI163955[] = {{&S48421[44], 23}, {NULL}};
const Segment SGI163956[] = {{&S48421[30], 6}, {NULL}};
const Segment SGI163957[] = {{&S48421[116], 3}, {NULL}};
const Segment SGI163958[] = {{&S48421[81], 6}, {NULL}};
const Segment SGI163959[] = {{&S48422[44], 11}, {NULL}};
const Segment SGI163960[] = {{&S48422[44], 23}, {NULL}};
const Segment SGI163961[] = {{&S48422[30], 6}, {NULL}};
const Segment SGI163962[] = {{&S48422[116], 3}, {NULL}};
const Segment SGI163963[] = {{&S48422[81], 6}, {NULL}};
const Segment SGI163964[] = {{&S48423[38], 11}, {NULL}};
const Segment SGI163965[] = {{&S48423[31], 6}, {NULL}};
const Segment SGI163966[] = {{&S48424[44], 18}, {NULL}};
const Segment SGI163967[] = {{&S48424[44], 30}, {NULL}};
const Segment SGI163968[] = {{&S48424[30], 6}, {NULL}};
const Segment SGI163969[] = {{&S48424[123], 3}, {NULL}};
const Segment SGI163970[] = {{&S48424[88], 6}, {NULL}};
const Segment SGI163971[] = {{&S58763[44], 8}, {NULL}};
const Segment SGI163972[] = {{&S58763[44], 19}, {NULL}};
const Segment SGI163973[] = {{&S58763[30], 6}, {NULL}};
const Segment SGI163974[] = {{&S58763[31], 3}, {NULL}};
const Segment SGI163975[] = {{&S48425[38], 11}, {NULL}};
const Segment SGI163976[] = {{&S58763[77], 6}, {NULL}};
const Segment SGI163977[] = {{&S48425[31], 6}, {NULL}};
const Segment SGI163978[] = {{&S58764[44], 8}, {NULL}};
const Segment SGI163979[] = {{&S58764[44], 19}, {NULL}};
const Segment SGI163980[] = {{&S48426[38], 11}, {NULL}};
const Segment SGI163981[] = {{&S58764[30], 6}, {NULL}};
const Segment SGI163982[] = {{&S48426[31], 6}, {NULL}};
const Segment SGI163983[] = {{&S58764[31], 3}, {NULL}};
const Segment SGI163984[] = {{&S58764[77], 6}, {NULL}};
const Segment SGI163985[] = {{&S58765[44], 8}, {NULL}};
const Segment SGI163986[] = {{&S58765[44], 19}, {NULL}};
const Segment SGI163987[] = {{&S58765[30], 6}, {NULL}};
const Segment SGI163988[] = {{&S58765[31], 3}, {NULL}};
const Segment SGI163989[] = {{&S48428[44], 18}, {NULL}};
const Segment SGI163990[] = {{&S48428[44], 30}, {NULL}};
const Segment SGI163991[] = {{&S58765[77], 6}, {NULL}};
const Segment SGI163992[] = {{&S48428[30], 6}, {NULL}};
const Segment SGI163993[] = {{&S48428[123], 3}, {NULL}};
const Segment SGI163994[] = {{&S48428[88], 6}, {NULL}};
const Segment SGI163995[] = {{&S58766[44], 10}, {NULL}};
const Segment SGI163996[] = {{&S58766[44], 21}, {NULL}};
const Segment SGI163997[] = {{&S58766[30], 6}, {NULL}};
const Segment SGI163998[] = {{&S58766[31], 3}, {NULL}};
const Segment SGI163999[] = {{&S58766[79], 6}, {NULL}};
const Segment SGI164000[] = {{&S58767[44], 10}, {NULL}};
const Segment SGI164001[] = {{&S58767[44], 21}, {NULL}};
const Segment SGI164002[] = {{&S58767[30], 6}, {NULL}};
const Segment SGI164003[] = {{&S58767[31], 3}, {NULL}};
const Segment SGI164004[] = {{&S58767[79], 6}, {NULL}};
const Segment SGI164005[] = {{&S58768[44], 8}, {NULL}};
const Segment SGI164006[] = {{&S58768[44], 19}, {NULL}};
const Segment SGI164007[] = {{&S58768[30], 6}, {NULL}};
const Segment SGI164008[] = {{&S58768[31], 3}, {NULL}};
const Segment SGI164009[] = {{&S58768[77], 6}, {NULL}};
const Segment SGI164010[] = {{&S50242[34], 18}, {NULL}};
const Segment SGI164011[] = {{&S58769[44], 10}, {NULL}};
const Segment SGI164012[] = {{&S50242[27], 6}, {NULL}};
const Segment SGI164013[] = {{&S58769[44], 21}, {NULL}};
const Segment SGI164014[] = {{&S58769[30], 6}, {NULL}};
const Segment SGI164015[] = {{&S58769[31], 3}, {NULL}};
const Segment SGI164016[] = {{&S50242[105], 9}, {NULL}};
const Segment SGI164017[] = {{&S58769[79], 6}, {NULL}};
const Segment SGI164018[] = {{&S50242[78], 6}, {NULL}};
const Segment SGI164019[] = {{&S50242[146], 8}, {NULL}};
const Segment SGI164020[] = {{&S58770[44], 8}, {NULL}};
const Segment SGI164021[] = {{&S58770[44], 19}, {NULL}};
const Segment SGI164022[] = {{&S58770[30], 6}, {NULL}};
const Segment SGI164023[] = {{&S58770[31], 3}, {NULL}};
const Segment SGI164024[] = {{&S58770[77], 6}, {NULL}};
const Segment SGI164025[] = {{&S58771[44], 10}, {NULL}};
const Segment SGI164026[] = {{&S58771[44], 21}, {NULL}};
const Segment SGI164027[] = {{&S58771[30], 6}, {NULL}};
const Segment SGI164028[] = {{&S58771[31], 3}, {NULL}};
const Segment SGI164029[] = {{&S58771[79], 6}, {NULL}};
const Segment SGI164030[] = {{&S50246[34], 11}, {NULL}};
const Segment SGI164031[] = {{&S50246[27], 6}, {NULL}};
const Segment SGI164032[] = {{&S50246[98], 9}, {NULL}};
const Segment SGI164033[] = {{&S50246[71], 6}, {NULL}};
const Segment SGI164034[] = {{&S50246[139], 8}, {NULL}};
const Segment SGI164035[] = {{&S50248[34], 10}, {NULL}};
const Segment SGI164036[] = {{&S50248[27], 6}, {NULL}};
const Segment SGI164037[] = {{&S50248[97], 9}, {NULL}};
const Segment SGI164038[] = {{&S50248[70], 6}, {NULL}};
const Segment SGI164039[] = {{&S50248[138], 8}, {NULL}};
const Segment SGI164040[] = {{&S58780[44], 14}, {NULL}};
const Segment SGI164041[] = {{&S58780[44], 33}, {NULL}};
const Segment SGI164042[] = {{&S58780[30], 6}, {NULL}};
const Segment SGI164043[] = {{&S58780[31], 3}, {NULL}};
const Segment SGI164044[] = {{&S58780[91], 6}, {NULL}};
const Segment SGI164045[] = {{&S58781[44], 14}, {NULL}};
const Segment SGI164046[] = {{&S58781[44], 38}, {NULL}};
const Segment SGI164047[] = {{&S58781[30], 6}, {NULL}};
const Segment SGI164048[] = {{&S58781[31], 3}, {NULL}};
const Segment SGI164049[] = {{&S58781[96], 6}, {NULL}};
const Segment SGI164050[] = {{&S58782[44], 14}, {NULL}};
const Segment SGI164051[] = {{&S58782[44], 38}, {NULL}};
const Segment SGI164052[] = {{&S58782[30], 6}, {NULL}};
const Segment SGI164053[] = {{&S58782[31], 3}, {NULL}};
const Segment SGI164054[] = {{&S58782[96], 6}, {NULL}};
const Segment SGI164055[] = {{&S58783[44], 14}, {NULL}};
const Segment SGI164056[] = {{&S58783[44], 33}, {NULL}};
const Segment SGI164057[] = {{&S58783[30], 6}, {NULL}};
const Segment SGI164058[] = {{&S58783[31], 3}, {NULL}};
const Segment SGI164059[] = {{&S58783[91], 6}, {NULL}};
const Segment SGI164060[] = {{&S58784[44], 14}, {NULL}};
const Segment SGI164061[] = {{&S58784[44], 39}, {NULL}};
const Segment SGI164062[] = {{&S58784[30], 6}, {NULL}};
const Segment SGI164063[] = {{&S58784[31], 3}, {NULL}};
const Segment SGI164064[] = {{&S58784[97], 6}, {NULL}};
const Segment SGI164065[] = {{&S58785[44], 14}, {NULL}};
const Segment SGI164066[] = {{&S58785[44], 39}, {NULL}};
const Segment SGI164067[] = {{&S58785[30], 6}, {NULL}};
const Segment SGI164068[] = {{&S58785[31], 3}, {NULL}};
const Segment SGI164069[] = {{&S58785[97], 6}, {NULL}};
const Segment SGI164070[] = {{&S58786[44], 14}, {NULL}};
const Segment SGI164071[] = {{&S58786[44], 34}, {NULL}};
const Segment SGI164072[] = {{&S58786[30], 6}, {NULL}};
const Segment SGI164073[] = {{&S58786[31], 3}, {NULL}};
const Segment SGI164074[] = {{&S58786[92], 6}, {NULL}};
const Segment SGI164075[] = {{&S58787[44], 14}, {NULL}};
const Segment SGI164076[] = {{&S58787[44], 33}, {NULL}};
const Segment SGI164077[] = {{&S58787[30], 6}, {NULL}};
const Segment SGI164078[] = {{&S58787[31], 3}, {NULL}};
const Segment SGI164079[] = {{&S58787[91], 6}, {NULL}};
const Segment SGI164080[] = {{&S58790[34], 12}, {NULL}};
const Segment SGI164081[] = {{&S58790[27], 6}, {NULL}};
const Segment SGI164082[] = {{&S58790[99], 9}, {NULL}};
const Segment SGI164083[] = {{&S58790[72], 6}, {NULL}};
const Segment SGI164084[] = {{&S58791[34], 12}, {NULL}};
const Segment SGI164085[] = {{&S58791[27], 6}, {NULL}};
const Segment SGI164086[] = {{&S58791[99], 9}, {NULL}};
const Segment SGI164087[] = {{&S58791[72], 6}, {NULL}};
const Segment SGI164088[] = {{&S50277[34], 10}, {NULL}};
const Segment SGI164089[] = {{&S50277[27], 6}, {NULL}};
const Segment SGI164090[] = {{&S50277[97], 9}, {NULL}};
const Segment SGI164091[] = {{&S50277[70], 6}, {NULL}};
const Segment SGI164092[] = {{&S58805[44], 11}, {NULL}};
const Segment SGI164093[] = {{&S58805[44], 23}, {NULL}};
const Segment SGI164094[] = {{&S58805[30], 6}, {NULL}};
const Segment SGI164095[] = {{&S58805[31], 3}, {NULL}};
const Segment SGI164096[] = {{&S58805[81], 6}, {NULL}};
const Segment SGI164097[] = {{&S58806[44], 11}, {NULL}};
const Segment SGI164098[] = {{&S58806[44], 22}, {NULL}};
const Segment SGI164099[] = {{&S58806[30], 6}, {NULL}};
const Segment SGI164100[] = {{&S58806[31], 3}, {NULL}};
const Segment SGI164101[] = {{&S58806[80], 5}, {NULL}};
const Segment SGI164102[] = {{&S58807[44], 11}, {NULL}};
const Segment SGI164103[] = {{&S58807[44], 23}, {NULL}};
const Segment SGI164104[] = {{&S58807[30], 6}, {NULL}};
const Segment SGI164105[] = {{&S58807[31], 3}, {NULL}};
const Segment SGI164106[] = {{&S58807[81], 6}, {NULL}};
const Segment SGI164107[] = {{&S58808[44], 11}, {NULL}};
const Segment SGI164108[] = {{&S58808[44], 23}, {NULL}};
const Segment SGI164109[] = {{&S58808[30], 6}, {NULL}};
const Segment SGI164110[] = {{&S58808[31], 3}, {NULL}};
const Segment SGI164111[] = {{&S58808[81], 6}, {NULL}};
const Segment SGI164112[] = {{&S58809[44], 11}, {NULL}};
const Segment SGI164113[] = {{&S58809[44], 23}, {NULL}};
const Segment SGI164114[] = {{&S58809[30], 6}, {NULL}};
const Segment SGI164115[] = {{&S58809[31], 3}, {NULL}};
const Segment SGI164116[] = {{&S58809[81], 5}, {NULL}};
const Segment SGI164117[] = {{&S58813[34], 11}, {NULL}};
const Segment SGI164118[] = {{&S58813[27], 6}, {NULL}};
const Segment SGI164119[] = {{&S58813[98], 9}, {NULL}};
const Segment SGI164120[] = {{&S58813[71], 6}, {NULL}};
const Segment SGI164121[] = {{&S58825[44], 13}, {NULL}};
const Segment SGI164122[] = {{&S58825[44], 25}, {NULL}};
const Segment SGI164123[] = {{&S58825[30], 6}, {NULL}};
const Segment SGI164124[] = {{&S58825[31], 3}, {NULL}};
const Segment SGI164125[] = {{&S58825[83], 6}, {NULL}};
const Segment SGI164126[] = {{&S58827[34], 18}, {NULL}};
const Segment SGI164127[] = {{&S58827[27], 6}, {NULL}};
const Segment SGI164128[] = {{&S58827[105], 9}, {NULL}};
const Segment SGI164129[] = {{&S58827[78], 6}, {NULL}};
const Segment SGI164130[] = {{&S58829[34], 14}, {NULL}};
const Segment SGI164131[] = {{&S58829[27], 6}, {NULL}};
const Segment SGI164132[] = {{&S58829[101], 9}, {NULL}};
const Segment SGI164133[] = {{&S58829[74], 6}, {NULL}};
const Segment SGI164134[] = {{&S48526[34], 7}, {NULL}};
const Segment SGI164135[] = {{&S48526[27], 6}, {NULL}};
const Segment SGI164136[] = {{&S48526[99], 9}, {NULL}};
const Segment SGI164137[] = {{&S48526[72], 6}, {NULL}};
const Segment SGI164138[] = {{&S48527[44], 10}, {NULL}};
const Segment SGI164139[] = {{&S48527[44], 22}, {NULL}};
const Segment SGI164140[] = {{&S48527[30], 6}, {NULL}};
const Segment SGI164141[] = {{&S48527[115], 3}, {NULL}};
const Segment SGI164142[] = {{&S48527[80], 6}, {NULL}};
const Segment SGI164143[] = {{&S48528[38], 10}, {NULL}};
const Segment SGI164144[] = {{&S48528[31], 6}, {NULL}};
const Segment SGI164145[] = {{&S48529[42], 10}, {NULL}};
const Segment SGI164146[] = {{&S48529[30], 6}, {NULL}};
const Segment SGI164147[] = {{&S48529[113], 3}, {NULL}};
const Segment SGI164148[] = {{&S48529[78], 6}, {NULL}};
const Segment SGI164149[] = {{&S48532[44], 10}, {NULL}};
const Segment SGI164150[] = {{&S48532[44], 22}, {NULL}};
const Segment SGI164151[] = {{&S48532[30], 6}, {NULL}};
const Segment SGI164152[] = {{&S48532[115], 3}, {NULL}};
const Segment SGI164153[] = {{&S48532[80], 6}, {NULL}};
const Segment SGI164154[] = {{&S48533[38], 10}, {NULL}};
const Segment SGI164155[] = {{&S48533[31], 6}, {NULL}};
const Segment SGI164156[] = {{&S48537[34], 11}, {NULL}};
const Segment SGI164157[] = {{&S48537[27], 6}, {NULL}};
const Segment SGI164158[] = {{&S48537[98], 9}, {NULL}};
const Segment SGI164159[] = {{&S48537[71], 6}, {NULL}};
const Segment SGI164160[] = {{&S48544[34], 6}, {NULL}};
const Segment SGI164161[] = {{&S48544[27], 6}, {NULL}};
const Segment SGI164162[] = {{&S48544[93], 9}, {NULL}};
const Segment SGI164163[] = {{&S48544[66], 6}, {NULL}};
const Segment SGI164164[] = {{&S48546[44], 10}, {NULL}};
const Segment SGI164165[] = {{&S48546[44], 22}, {NULL}};
const Segment SGI164166[] = {{&S48546[30], 6}, {NULL}};
const Segment SGI164167[] = {{&S48546[115], 3}, {NULL}};
const Segment SGI164168[] = {{&S48546[80], 6}, {NULL}};
const Segment SGI164169[] = {{&S48547[38], 10}, {NULL}};
const Segment SGI164170[] = {{&S48547[31], 6}, {NULL}};
const Segment SGI164171[] = {{&S48548[34], 10}, {NULL}};
const Segment SGI164172[] = {{&S48548[27], 6}, {NULL}};
const Segment SGI164173[] = {{&S48548[97], 9}, {NULL}};
const Segment SGI164174[] = {{&S48548[70], 6}, {NULL}};
const Segment SGI164175[] = {{&S50366[34], 7}, {NULL}};
const Segment SGI164176[] = {{&S50366[27], 6}, {NULL}};
const Segment SGI164177[] = {{&S50366[110], 9}, {NULL}};
const Segment SGI164178[] = {{&S50366[83], 6}, {NULL}};
const Segment SGI164179[] = {{&S50368[34], 7}, {NULL}};
const Segment SGI164180[] = {{&S50368[27], 6}, {NULL}};
const Segment SGI164181[] = {{&S50368[110], 9}, {NULL}};
const Segment SGI164182[] = {{&S50368[83], 6}, {NULL}};
const Segment SGI164183[] = {{&S58880[44], 12}, {NULL}};
const Segment SGI164184[] = {{&S58880[44], 24}, {NULL}};
const Segment SGI164185[] = {{&S58880[30], 6}, {NULL}};
const Segment SGI164186[] = {{&S58880[31], 3}, {NULL}};
const Segment SGI164187[] = {{&S58880[82], 6}, {NULL}};
const Segment SGI164188[] = {{&S58882[34], 24}, {NULL}};
const Segment SGI164189[] = {{&S58882[27], 6}, {NULL}};
const Segment SGI164190[] = {{&S58882[111], 9}, {NULL}};
const Segment SGI164191[] = {{&S58882[84], 6}, {NULL}};
const Segment SGI164192[] = {{&S58883[34], 23}, {NULL}};
const Segment SGI164193[] = {{&S58883[27], 6}, {NULL}};
const Segment SGI164194[] = {{&S58883[110], 9}, {NULL}};
const Segment SGI164195[] = {{&S58883[83], 6}, {NULL}};
const Segment SGI164196[] = {{&S50386[34], 10}, {NULL}};
const Segment SGI164197[] = {{&S50386[27], 6}, {NULL}};
const Segment SGI164198[] = {{&S50386[97], 9}, {NULL}};
const Segment SGI164199[] = {{&S50386[70], 6}, {NULL}};
const Segment SGI164200[] = {{&S50387[34], 14}, {NULL}};
const Segment SGI164201[] = {{&S50387[27], 6}, {NULL}};
const Segment SGI164202[] = {{&S50387[101], 9}, {NULL}};
const Segment SGI164203[] = {{&S50387[74], 6}, {NULL}};
const Segment SGI164204[] = {{&S50391[34], 14}, {NULL}};
const Segment SGI164205[] = {{&S50391[27], 6}, {NULL}};
const Segment SGI164206[] = {{&S50391[101], 9}, {NULL}};
const Segment SGI164207[] = {{&S50391[74], 6}, {NULL}};
const Segment SGI164208[] = {{&S55194[34], 24}, {NULL}};
const Segment SGI164209[] = {{&S55194[27], 6}, {NULL}};
const Segment SGI164210[] = {{&S55194[84], 6}, {NULL}};
const Segment SGI164211[] = {{&S50419[34], 14}, {NULL}};
const Segment SGI164212[] = {{&S50419[27], 6}, {NULL}};
const Segment SGI164213[] = {{&S50419[101], 9}, {NULL}};
const Segment SGI164214[] = {{&S50419[74], 6}, {NULL}};
const Segment SGI164215[] = {{&S50421[34], 23}, {NULL}};
const Segment SGI164216[] = {{&S50421[27], 6}, {NULL}};
const Segment SGI164217[] = {{&S50421[110], 9}, {NULL}};
const Segment SGI164218[] = {{&S50421[83], 6}, {NULL}};
const Segment SGI164219[] = {{&S50422[34], 24}, {NULL}};
const Segment SGI164220[] = {{&S50422[27], 6}, {NULL}};
const Segment SGI164221[] = {{&S50422[111], 9}, {NULL}};
const Segment SGI164222[] = {{&S50422[84], 6}, {NULL}};
const Segment SGI164223[] = {{&S44762[34], 18}, {NULL}};
const Segment SGI164224[] = {{&S44762[27], 6}, {NULL}};
const Segment SGI164225[] = {{&S44762[105], 9}, {NULL}};
const Segment SGI164226[] = {{&S44762[78], 6}, {NULL}};
const Segment SGI164227[] = {{&S44764[34], 10}, {NULL}};
const Segment SGI164228[] = {{&S44764[27], 6}, {NULL}};
const Segment SGI164229[] = {{&S44764[97], 9}, {NULL}};
const Segment SGI164230[] = {{&S44764[70], 6}, {NULL}};
const Segment SGI164231[] = {{&S58931[34], 6}, {NULL}};
const Segment SGI164232[] = {{&S58931[27], 6}, {NULL}};
const Segment SGI164233[] = {{&S44765[34], 10}, {NULL}};
const Segment SGI164234[] = {{&S44765[27], 6}, {NULL}};
const Segment SGI164235[] = {{&S58931[93], 9}, {NULL}};
const Segment SGI164236[] = {{&S58931[66], 6}, {NULL}};
const Segment SGI164237[] = {{&S44765[97], 9}, {NULL}};
const Segment SGI164238[] = {{&S44765[70], 6}, {NULL}};
const Segment SGI164239[] = {{&S58932[34], 7}, {NULL}};
const Segment SGI164240[] = {{&S58932[27], 6}, {NULL}};
const Segment SGI164241[] = {{&S58932[93], 9}, {NULL}};
const Segment SGI164242[] = {{&S58932[66], 6}, {NULL}};
const Segment SGI164243[] = {{&S50431[44], 18}, {NULL}};
const Segment SGI164244[] = {{&S50431[44], 30}, {NULL}};
const Segment SGI164245[] = {{&S50431[30], 6}, {NULL}};
const Segment SGI164246[] = {{&S50431[123], 3}, {NULL}};
const Segment SGI164247[] = {{&S50431[88], 6}, {NULL}};
const Segment SGI164248[] = {{&S50432[44], 21}, {NULL}};
const Segment SGI164249[] = {{&S50432[44], 33}, {NULL}};
const Segment SGI164250[] = {{&S50432[30], 6}, {NULL}};
const Segment SGI164251[] = {{&S50432[126], 3}, {NULL}};
const Segment SGI164252[] = {{&S50432[91], 6}, {NULL}};
const Segment SGI164253[] = {{&S50433[38], 21}, {NULL}};
const Segment SGI164254[] = {{&S50433[31], 6}, {NULL}};
const Segment SGI164255[] = {{&S58938[34], 14}, {NULL}};
const Segment SGI164256[] = {{&S58938[27], 6}, {NULL}};
const Segment SGI164257[] = {{&S58938[101], 9}, {NULL}};
const Segment SGI164258[] = {{&S58938[74], 6}, {NULL}};
const Segment SGI164259[] = {{&S50434[38], 18}, {NULL}};
const Segment SGI164260[] = {{&S50434[31], 6}, {NULL}};
const Segment SGI164261[] = {{&S50435[38], 18}, {NULL}};
const Segment SGI164262[] = {{&S50435[31], 6}, {NULL}};
const Segment SGI164263[] = {{&S50436[44], 18}, {NULL}};
const Segment SGI164264[] = {{&S50436[44], 30}, {NULL}};
const Segment SGI164265[] = {{&S50436[30], 6}, {NULL}};
const Segment SGI164266[] = {{&S50436[123], 3}, {NULL}};
const Segment SGI164267[] = {{&S50436[88], 6}, {NULL}};
const Segment SGI164268[] = {{&S58967[34], 10}, {NULL}};
const Segment SGI164269[] = {{&S58967[27], 6}, {NULL}};
const Segment SGI164270[] = {{&S58967[97], 9}, {NULL}};
const Segment SGI164271[] = {{&S58967[70], 6}, {NULL}};
const Segment SGI164272[] = {{&S58968[34], 10}, {NULL}};
const Segment SGI164273[] = {{&S58968[27], 6}, {NULL}};
const Segment SGI164274[] = {{&S58968[97], 9}, {NULL}};
const Segment SGI164275[] = {{&S58968[70], 6}, {NULL}};
const Segment SGI164276[] = {{&S50472[34], 10}, {NULL}};
const Segment SGI164277[] = {{&S50472[27], 6}, {NULL}};
const Segment SGI164278[] = {{&S50472[97], 9}, {NULL}};
const Segment SGI164279[] = {{&S50472[70], 6}, {NULL}};
const Segment SGI164280[] = {{&S44841[34], 29}, {NULL}};
const Segment SGI164281[] = {{&S44841[27], 6}, {NULL}};
const Segment SGI164282[] = {{&S44841[116], 9}, {NULL}};
const Segment SGI164283[] = {{&S44841[89], 6}, {NULL}};
const Segment SGI164284[] = {{&S54681[32], 9}, {NULL}};
const Segment SGI164285[] = {{&S54681[27], 4}, {NULL}};
const Segment SGI164286[] = {{&S54681[94], 9}, {NULL}};
const Segment SGI164287[] = {{&S54681[67], 6}, {NULL}};
const Segment SGI164288[] = {{&S44843[34], 30}, {NULL}};
const Segment SGI164289[] = {{&S50500[34], 29}, {NULL}};
const Segment SGI164290[] = {{&S44843[27], 6}, {NULL}};
const Segment SGI164291[] = {{&S50500[27], 6}, {NULL}};
const Segment SGI164292[] = {{&S44843[117], 9}, {NULL}};
const Segment SGI164293[] = {{&S50500[116], 9}, {NULL}};
const Segment SGI164294[] = {{&S44843[90], 6}, {NULL}};
const Segment SGI164295[] = {{&S50500[89], 6}, {NULL}};
const Segment SGI164296[] = {{&S44844[41], 29}, {NULL}};
const Segment SGI164297[] = {{&S44844[41], 41}, {NULL}};
const Segment SGI164298[] = {{&S44844[27], 6}, {NULL}};
const Segment SGI164299[] = {{&S50501[34], 30}, {NULL}};
const Segment SGI164300[] = {{&S50501[27], 6}, {NULL}};
const Segment SGI164301[] = {{&S44844[122], 9}, {NULL}};
const Segment SGI164302[] = {{&S44844[96], 6}, {NULL}};
const Segment SGI164303[] = {{&S50501[117], 9}, {NULL}};
const Segment SGI164304[] = {{&S50501[90], 6}, {NULL}};
const Segment SGI164305[] = {{&S44845[34], 30}, {NULL}};
const Segment SGI164306[] = {{&S44845[27], 6}, {NULL}};
const Segment SGI164307[] = {{&S44845[117], 9}, {NULL}};
const Segment SGI164308[] = {{&S44845[90], 6}, {NULL}};
const Segment SGI164309[] = {{&S59017[44], 10}, {NULL}};
const Segment SGI164310[] = {{&S59017[44], 22}, {NULL}};
const Segment SGI164311[] = {{&S59017[30], 6}, {NULL}};
const Segment SGI164312[] = {{&S59017[31], 3}, {NULL}};
const Segment SGI164313[] = {{&S59017[95], 6}, {NULL}};
const Segment SGI164314[] = {{&S59017[68], 8}, {NULL}};
const Segment SGI164315[] = {{&S59018[44], 10}, {NULL}};
const Segment SGI164316[] = {{&S59018[44], 22}, {NULL}};
const Segment SGI164317[] = {{&S59018[30], 6}, {NULL}};
const Segment SGI164318[] = {{&S59018[31], 3}, {NULL}};
const Segment SGI164319[] = {{&S59018[95], 6}, {NULL}};
const Segment SGI164320[] = {{&S59018[68], 8}, {NULL}};
const Segment SGI164321[] = {{&S59019[44], 10}, {NULL}};
const Segment SGI164322[] = {{&S59019[44], 22}, {NULL}};
const Segment SGI164323[] = {{&S50511[34], 12}, {NULL}};
const Segment SGI164324[] = {{&S59019[30], 6}, {NULL}};
const Segment SGI164325[] = {{&S50511[27], 6}, {NULL}};
const Segment SGI164326[] = {{&S59019[31], 3}, {NULL}};
const Segment SGI164327[] = {{&S50511[99], 9}, {NULL}};
const Segment SGI164328[] = {{&S59019[99], 6}, {NULL}};
const Segment SGI164329[] = {{&S50511[72], 6}, {NULL}};
const Segment SGI164330[] = {{&S48562[34], 8}, {NULL}};
const Segment SGI164331[] = {{&S48562[27], 6}, {NULL}};
const Segment SGI164332[] = {{&S48562[95], 9}, {NULL}};
const Segment SGI164333[] = {{&S48562[68], 6}, {NULL}};
const Segment SGI164334[] = {{&S50512[34], 14}, {NULL}};
const Segment SGI164335[] = {{&S50512[27], 6}, {NULL}};
const Segment SGI164336[] = {{&S50512[101], 9}, {NULL}};
const Segment SGI164337[] = {{&S50512[74], 6}, {NULL}};
const Segment SGI164338[] = {{&S48600[34], 10}, {NULL}};
const Segment SGI164339[] = {{&S48600[27], 6}, {NULL}};
const Segment SGI164340[] = {{&S48600[97], 9}, {NULL}};
const Segment SGI164341[] = {{&S48600[70], 6}, {NULL}};
const Segment SGI164342[] = {{&S48605[34], 14}, {NULL}};
const Segment SGI164343[] = {{&S48605[27], 6}, {NULL}};
const Segment SGI164344[] = {{&S48605[111], 9}, {NULL}};
const Segment SGI164345[] = {{&S48605[84], 6}, {NULL}};
const Segment SGI164346[] = {{&S48606[34], 14}, {NULL}};
const Segment SGI164347[] = {{&S48606[27], 6}, {NULL}};
const Segment SGI164348[] = {{&S48606[111], 9}, {NULL}};
const Segment SGI164349[] = {{&S48606[84], 6}, {NULL}};
const Segment SGI164350[] = {{&S50577[44], 14}, {NULL}};
const Segment SGI164351[] = {{&S50577[44], 26}, {NULL}};
const Segment SGI164352[] = {{&S50577[30], 6}, {NULL}};
const Segment SGI164353[] = {{&S50577[119], 3}, {NULL}};
const Segment SGI164354[] = {{&S50577[84], 6}, {NULL}};
const Segment SGI164355[] = {{&S50579[38], 14}, {NULL}};
const Segment SGI164356[] = {{&S50579[31], 6}, {NULL}};
const Segment SGI164357[] = {{&S48629[44], 9}, {NULL}};
const Segment SGI164358[] = {{&S48629[44], 21}, {NULL}};
const Segment SGI164359[] = {{&S48629[30], 6}, {NULL}};
const Segment SGI164360[] = {{&S48629[130], 3}, {NULL}};
const Segment SGI164361[] = {{&S48629[95], 6}, {NULL}};
const Segment SGI164362[] = {{&S48629[67], 8}, {NULL}};
const Segment SGI164363[] = {{&S50580[44], 14}, {NULL}};
const Segment SGI164364[] = {{&S50580[44], 26}, {NULL}};
const Segment SGI164365[] = {{&S50580[30], 6}, {NULL}};
const Segment SGI164366[] = {{&S50580[119], 3}, {NULL}};
const Segment SGI164367[] = {{&S50580[84], 6}, {NULL}};
const Segment SGI164368[] = {{&S48632[44], 9}, {NULL}};
const Segment SGI164369[] = {{&S48632[44], 21}, {NULL}};
const Segment SGI164370[] = {{&S48632[30], 6}, {NULL}};
const Segment SGI164371[] = {{&S48632[130], 3}, {NULL}};
const Segment SGI164372[] = {{&S48632[95], 6}, {NULL}};
const Segment SGI164373[] = {{&S48632[67], 8}, {NULL}};
const Segment SGI164374[] = {{&S48658[34], 7}, {NULL}};
const Segment SGI164375[] = {{&S48658[27], 6}, {NULL}};
const Segment SGI164376[] = {{&S48658[99], 9}, {NULL}};
const Segment SGI164377[] = {{&S48658[72], 6}, {NULL}};
const Segment SGI164378[] = {{&S48661[34], 7}, {NULL}};
const Segment SGI164379[] = {{&S48661[27], 6}, {NULL}};
const Segment SGI164380[] = {{&S44962[34], 10}, {NULL}};
const Segment SGI164381[] = {{&S44962[27], 6}, {NULL}};
const Segment SGI164382[] = {{&S48661[104], 9}, {NULL}};
const Segment SGI164383[] = {{&S48661[77], 6}, {NULL}};
const Segment SGI164384[] = {{&S44962[97], 9}, {NULL}};
const Segment SGI164385[] = {{&S44962[70], 6}, {NULL}};
const Segment SGI164386[] = {{&S48662[34], 7}, {NULL}};
const Segment SGI164387[] = {{&S48662[27], 6}, {NULL}};
const Segment SGI164388[] = {{&S48662[104], 9}, {NULL}};
const Segment SGI164389[] = {{&S48662[77], 6}, {NULL}};
const Segment SGI164390[] = {{&S44964[34], 10}, {NULL}};
const Segment SGI164391[] = {{&S44964[27], 6}, {NULL}};
const Segment SGI164392[] = {{&S44964[97], 9}, {NULL}};
const Segment SGI164393[] = {{&S44964[70], 6}, {NULL}};
const Segment SGI164394[] = {{&S44965[34], 10}, {NULL}};
const Segment SGI164395[] = {{&S44965[27], 6}, {NULL}};
const Segment SGI164396[] = {{&S44965[97], 9}, {NULL}};
const Segment SGI164397[] = {{&S44965[70], 6}, {NULL}};
const Segment SGI164398[] = {{&S50654[34], 10}, {NULL}};
const Segment SGI164399[] = {{&S50654[27], 6}, {NULL}};
const Segment SGI164400[] = {{&S50654[97], 9}, {NULL}};
const Segment SGI164401[] = {{&S50654[70], 6}, {NULL}};
const Segment SGI164402[] = {{&S50656[44], 17}, {NULL}};
const Segment SGI164403[] = {{&S50656[44], 29}, {NULL}};
const Segment SGI164404[] = {{&S50656[30], 6}, {NULL}};
const Segment SGI164405[] = {{&S50656[122], 3}, {NULL}};
const Segment SGI164406[] = {{&S50656[87], 6}, {NULL}};
const Segment SGI164407[] = {{&S50657[44], 17}, {NULL}};
const Segment SGI164408[] = {{&S50657[44], 31}, {NULL}};
const Segment SGI164409[] = {{&S50657[30], 6}, {NULL}};
const Segment SGI164410[] = {{&S50657[124], 3}, {NULL}};
const Segment SGI164411[] = {{&S50657[89], 6}, {NULL}};
const Segment SGI164412[] = {{&S50658[38], 17}, {NULL}};
const Segment SGI164413[] = {{&S50658[31], 6}, {NULL}};
const Segment SGI164414[] = {{&S50659[34], 14}, {NULL}};
const Segment SGI164415[] = {{&S50659[27], 6}, {NULL}};
const Segment SGI164416[] = {{&S50659[101], 9}, {NULL}};
const Segment SGI164417[] = {{&S50659[74], 6}, {NULL}};
const Segment SGI164418[] = {{&S50661[34], 10}, {NULL}};
const Segment SGI164419[] = {{&S50661[27], 6}, {NULL}};
const Segment SGI164420[] = {{&S50661[97], 9}, {NULL}};
const Segment SGI164421[] = {{&S50661[70], 6}, {NULL}};
const Segment SGI164422[] = {{&S59019[68], 8}, {NULL}};
const Segment SGI164423[] = {{&S59022[34], 9}, {NULL}};
const Segment SGI164424[] = {{&S59022[27], 6}, {NULL}};
const Segment SGI164425[] = {{&S59022[96], 9}, {NULL}};
const Segment SGI164426[] = {{&S59022[69], 6}, {NULL}};
const Segment SGI164427[] = {{&S48718[34], 7}, {NULL}};
const Segment SGI164428[] = {{&S48718[27], 6}, {NULL}};
const Segment SGI164429[] = {{&S48718[110], 9}, {NULL}};
const Segment SGI164430[] = {{&S48718[83], 6}, {NULL}};
const Segment SGI164431[] = {{&S59023[34], 9}, {NULL}};
const Segment SGI164432[] = {{&S59023[27], 6}, {NULL}};
const Segment SGI164433[] = {{&S59023[96], 9}, {NULL}};
const Segment SGI164434[] = {{&S59023[69], 6}, {NULL}};
const Segment SGI164435[] = {{&S59024[34], 9}, {NULL}};
const Segment SGI164436[] = {{&S59024[27], 6}, {NULL}};
const Segment SGI164437[] = {{&S59024[96], 9}, {NULL}};
const Segment SGI164438[] = {{&S59024[69], 6}, {NULL}};
const Segment SGI164439[] = {{&S50670[44], 10}, {NULL}};
const Segment SGI164440[] = {{&S50670[44], 22}, {NULL}};
const Segment SGI164441[] = {{&S50670[30], 6}, {NULL}};
const Segment SGI164442[] = {{&S50670[131], 3}, {NULL}};
const Segment SGI164443[] = {{&S50670[96], 6}, {NULL}};
const Segment SGI164444[] = {{&S50670[68], 8}, {NULL}};
const Segment SGI164445[] = {{&S48726[34], 10}, {NULL}};
const Segment SGI164446[] = {{&S50672[44], 10}, {NULL}};
const Segment SGI164447[] = {{&S48726[27], 6}, {NULL}};
const Segment SGI164448[] = {{&S50672[44], 22}, {NULL}};
const Segment SGI164449[] = {{&S50672[30], 6}, {NULL}};
const Segment SGI164450[] = {{&S48726[97], 9}, {NULL}};
const Segment SGI164451[] = {{&S50672[134], 3}, {NULL}};
const Segment SGI164452[] = {{&S48726[70], 6}, {NULL}};
const Segment SGI164453[] = {{&S50672[99], 6}, {NULL}};
const Segment SGI164454[] = {{&S50672[68], 8}, {NULL}};
const Segment SGI164455[] = {{&S50673[44], 10}, {NULL}};
const Segment SGI164456[] = {{&S50673[44], 22}, {NULL}};
const Segment SGI164457[] = {{&S50673[30], 6}, {NULL}};
const Segment SGI164458[] = {{&S50673[131], 3}, {NULL}};
const Segment SGI164459[] = {{&S50673[96], 6}, {NULL}};
const Segment SGI164460[] = {{&S50673[68], 8}, {NULL}};
const Segment SGI164461[] = {{&S50674[44], 11}, {NULL}};
const Segment SGI164462[] = {{&S50674[44], 23}, {NULL}};
const Segment SGI164463[] = {{&S50674[30], 6}, {NULL}};
const Segment SGI164464[] = {{&S45028[44], 9}, {NULL}};
const Segment SGI164465[] = {{&S45028[44], 21}, {NULL}};
const Segment SGI164466[] = {{&S45028[30], 6}, {NULL}};
const Segment SGI164467[] = {{&S50674[132], 3}, {NULL}};
const Segment SGI164468[] = {{&S50674[97], 6}, {NULL}};
const Segment SGI164469[] = {{&S45028[132], 3}, {NULL}};
const Segment SGI164470[] = {{&S50674[69], 8}, {NULL}};
const Segment SGI164471[] = {{&S45028[97], 6}, {NULL}};
const Segment SGI164472[] = {{&S45028[67], 8}, {NULL}};
const Segment SGI164473[] = {{&S50675[44], 10}, {NULL}};
const Segment SGI164474[] = {{&S50675[44], 22}, {NULL}};
const Segment SGI164475[] = {{&S45029[44], 9}, {NULL}};
const Segment SGI164476[] = {{&S50675[30], 6}, {NULL}};
const Segment SGI164477[] = {{&S45029[44], 21}, {NULL}};
const Segment SGI164478[] = {{&S45029[30], 6}, {NULL}};
const Segment SGI164479[] = {{&S50675[134], 3}, {NULL}};
const Segment SGI164480[] = {{&S45029[132], 3}, {NULL}};
const Segment SGI164481[] = {{&S50675[99], 6}, {NULL}};
const Segment SGI164482[] = {{&S45029[97], 6}, {NULL}};
const Segment SGI164483[] = {{&S50675[68], 8}, {NULL}};
const Segment SGI164484[] = {{&S45029[67], 8}, {NULL}};
const Segment SGI164485[] = {{&S50676[44], 11}, {NULL}};
const Segment SGI164486[] = {{&S50676[44], 23}, {NULL}};
const Segment SGI164487[] = {{&S50676[30], 6}, {NULL}};
const Segment SGI164488[] = {{&S50676[132], 3}, {NULL}};
const Segment SGI164489[] = {{&S50676[97], 6}, {NULL}};
const Segment SGI164490[] = {{&S50676[69], 8}, {NULL}};
const Segment SGI164491[] = {{&S45032[44], 9}, {NULL}};
const Segment SGI164492[] = {{&S45032[44], 21}, {NULL}};
const Segment SGI164493[] = {{&S45032[30], 6}, {NULL}};
const Segment SGI164494[] = {{&S45032[132], 3}, {NULL}};
const Segment SGI164495[] = {{&S45032[97], 6}, {NULL}};
const Segment SGI164496[] = {{&S45032[67], 8}, {NULL}};
const Segment SGI164497[] = {{&S50678[44], 11}, {NULL}};
const Segment SGI164498[] = {{&S50678[44], 23}, {NULL}};
const Segment SGI164499[] = {{&S50678[30], 6}, {NULL}};
const Segment SGI164500[] = {{&S50678[132], 3}, {NULL}};
const Segment SGI164501[] = {{&S50678[97], 6}, {NULL}};
const Segment SGI164502[] = {{&S50678[69], 8}, {NULL}};
const Segment SGI164503[] = {{&S45034[44], 9}, {NULL}};
const Segment SGI164504[] = {{&S45034[44], 21}, {NULL}};
const Segment SGI164505[] = {{&S45034[30], 6}, {NULL}};
const Segment SGI164506[] = {{&S45034[132], 3}, {NULL}};
const Segment SGI164507[] = {{&S45034[97], 6}, {NULL}};
const Segment SGI164508[] = {{&S45034[67], 8}, {NULL}};
const Segment SGI164509[] = {{&S50682[44], 11}, {NULL}};
const Segment SGI164510[] = {{&S50682[44], 23}, {NULL}};
const Segment SGI164511[] = {{&S50682[30], 6}, {NULL}};
const Segment SGI164512[] = {{&S50682[132], 3}, {NULL}};
const Segment SGI164513[] = {{&S50682[97], 6}, {NULL}};
const Segment SGI164514[] = {{&S50682[69], 8}, {NULL}};
const Segment SGI164515[] = {{&S59052[44], 7}, {NULL}};
const Segment SGI164516[] = {{&S59052[44], 23}, {NULL}};
const Segment SGI164517[] = {{&S59052[30], 6}, {NULL}};
const Segment SGI164518[] = {{&S59052[31], 3}, {NULL}};
const Segment SGI164519[] = {{&S59052[81], 6}, {NULL}};
const Segment SGI164520[] = {{&S59053[44], 7}, {NULL}};
const Segment SGI164521[] = {{&S59053[44], 23}, {NULL}};
const Segment SGI164522[] = {{&S59053[30], 6}, {NULL}};
const Segment SGI164523[] = {{&S59053[31], 3}, {NULL}};
const Segment SGI164524[] = {{&S59053[81], 6}, {NULL}};
const Segment SGI164525[] = {{&S48765[44], 10}, {NULL}};
const Segment SGI164526[] = {{&S48765[44], 22}, {NULL}};
const Segment SGI164527[] = {{&S48765[30], 6}, {NULL}};
const Segment SGI164528[] = {{&S48765[115], 3}, {NULL}};
const Segment SGI164529[] = {{&S48765[80], 6}, {NULL}};
const Segment SGI164530[] = {{&S48766[38], 10}, {NULL}};
const Segment SGI164531[] = {{&S48766[31], 6}, {NULL}};
const Segment SGI164532[] = {{&S48767[44], 11}, {NULL}};
const Segment SGI164533[] = {{&S48767[44], 23}, {NULL}};
const Segment SGI164534[] = {{&S48767[30], 6}, {NULL}};
const Segment SGI164535[] = {{&S48767[116], 3}, {NULL}};
const Segment SGI164536[] = {{&S48767[81], 6}, {NULL}};
const Segment SGI164537[] = {{&S48768[42], 10}, {NULL}};
const Segment SGI164538[] = {{&S48768[30], 6}, {NULL}};
const Segment SGI164539[] = {{&S48768[113], 3}, {NULL}};
const Segment SGI164540[] = {{&S48768[78], 6}, {NULL}};
const Segment SGI164541[] = {{&S48769[44], 11}, {NULL}};
const Segment SGI164542[] = {{&S48769[44], 23}, {NULL}};
const Segment SGI164543[] = {{&S48769[30], 6}, {NULL}};
const Segment SGI164544[] = {{&S48769[116], 3}, {NULL}};
const Segment SGI164545[] = {{&S48769[81], 6}, {NULL}};
const Segment SGI164546[] = {{&S50723[44], 10}, {NULL}};
const Segment SGI164547[] = {{&S50723[44], 27}, {NULL}};
const Segment SGI164548[] = {{&S50723[30], 6}, {NULL}};
const Segment SGI164549[] = {{&S50723[120], 3}, {NULL}};
const Segment SGI164550[] = {{&S50723[85], 6}, {NULL}};
const Segment SGI164551[] = {{&S50724[38], 10}, {NULL}};
const Segment SGI164552[] = {{&S50724[31], 6}, {NULL}};
const Segment SGI164553[] = {{&S48795[44], 12}, {NULL}};
const Segment SGI164554[] = {{&S48795[44], 29}, {NULL}};
const Segment SGI164555[] = {{&S48795[30], 6}, {NULL}};
const Segment SGI164556[] = {{&S50734[34], 12}, {NULL}};
const Segment SGI164557[] = {{&S50734[27], 6}, {NULL}};
const Segment SGI164558[] = {{&S48795[122], 3}, {NULL}};
const Segment SGI164559[] = {{&S48795[87], 6}, {NULL}};
const Segment SGI164560[] = {{&S50734[99], 9}, {NULL}};
const Segment SGI164561[] = {{&S50734[72], 6}, {NULL}};
const Segment SGI164562[] = {{&S48796[44], 8}, {NULL}};
const Segment SGI164563[] = {{&S48796[44], 25}, {NULL}};
const Segment SGI164564[] = {{&S48796[30], 6}, {NULL}};
const Segment SGI164565[] = {{&S50735[34], 14}, {NULL}};
const Segment SGI164566[] = {{&S48796[118], 3}, {NULL}};
const Segment SGI164567[] = {{&S50735[27], 6}, {NULL}};
const Segment SGI164568[] = {{&S48796[83], 6}, {NULL}};
const Segment SGI164569[] = {{&S50735[101], 9}, {NULL}};
const Segment SGI164570[] = {{&S50735[74], 6}, {NULL}};
const Segment SGI164571[] = {{&S48797[44], 12}, {NULL}};
const Segment SGI164572[] = {{&S48797[44], 29}, {NULL}};
const Segment SGI164573[] = {{&S48797[30], 6}, {NULL}};
const Segment SGI164574[] = {{&S48797[122], 3}, {NULL}};
const Segment SGI164575[] = {{&S48797[87], 6}, {NULL}};
const Segment SGI164576[] = {{&S48798[38], 8}, {NULL}};
const Segment SGI164577[] = {{&S48798[31], 6}, {NULL}};
const Segment SGI164578[] = {{&S48799[44], 8}, {NULL}};
const Segment SGI164579[] = {{&S48799[44], 25}, {NULL}};
const Segment SGI164580[] = {{&S48799[30], 6}, {NULL}};
const Segment SGI164581[] = {{&S48799[118], 3}, {NULL}};
const Segment SGI164582[] = {{&S48799[83], 6}, {NULL}};
const Segment SGI164583[] = {{&S45103[38], 10}, {NULL}};
const Segment SGI164584[] = {{&S45103[31], 6}, {NULL}};
const Segment SGI164585[] = {{&S48800[44], 12}, {NULL}};
const Segment SGI164586[] = {{&S48800[44], 28}, {NULL}};
const Segment SGI164587[] = {{&S48800[30], 6}, {NULL}};
const Segment SGI164588[] = {{&S48800[121], 3}, {NULL}};
const Segment SGI164589[] = {{&S48800[86], 6}, {NULL}};
const Segment SGI164590[] = {{&S45105[46], 10}, {NULL}};
const Segment SGI164591[] = {{&S45105[46], 66}, {NULL}};
const Segment SGI164592[] = {{&S48801[44], 8}, {NULL}};
const Segment SGI164593[] = {{&S45105[32], 6}, {NULL}};
const Segment SGI164594[] = {{&S48801[44], 24}, {NULL}};
const Segment SGI164595[] = {{&S48801[30], 6}, {NULL}};
const Segment SGI164596[] = {{&S48801[117], 3}, {NULL}};
const Segment SGI164597[] = {{&S48801[82], 6}, {NULL}};
const Segment SGI164598[] = {{&S45107[44], 10}, {NULL}};
const Segment SGI164599[] = {{&S45107[44], 22}, {NULL}};
const Segment SGI164600[] = {{&S45107[30], 6}, {NULL}};
const Segment SGI164601[] = {{&S45107[115], 3}, {NULL}};
const Segment SGI164602[] = {{&S45107[80], 6}, {NULL}};
const Segment SGI164603[] = {{&S50754[34], 10}, {NULL}};
const Segment SGI164604[] = {{&S50754[27], 6}, {NULL}};
const Segment SGI164605[] = {{&S50754[97], 9}, {NULL}};
const Segment SGI164606[] = {{&S50754[70], 6}, {NULL}};
const Segment SGI164607[] = {{&S59136[34], 11}, {NULL}};
const Segment SGI164608[] = {{&S59136[27], 6}, {NULL}};
const Segment SGI164609[] = {{&S59136[98], 9}, {NULL}};
const Segment SGI164610[] = {{&S59136[71], 6}, {NULL}};
const Segment SGI164611[] = {{&S59138[34], 11}, {NULL}};
const Segment SGI164612[] = {{&S59138[27], 6}, {NULL}};
const Segment SGI164613[] = {{&S59138[98], 9}, {NULL}};
const Segment SGI164614[] = {{&S59138[71], 6}, {NULL}};
const Segment SGI164615[] = {{&S59139[34], 10}, {NULL}};
const Segment SGI164616[] = {{&S59139[27], 6}, {NULL}};
const Segment SGI164617[] = {{&S59139[97], 9}, {NULL}};
const Segment SGI164618[] = {{&S59139[70], 6}, {NULL}};
const Segment SGI164619[] = {{&S59152[34], 13}, {NULL}};
const Segment SGI164620[] = {{&S59152[27], 6}, {NULL}};
const Segment SGI164621[] = {{&S59152[100], 9}, {NULL}};
const Segment SGI164622[] = {{&S59152[73], 6}, {NULL}};
const Segment SGI164623[] = {{&S59153[34], 10}, {NULL}};
const Segment SGI164624[] = {{&S59153[27], 6}, {NULL}};
const Segment SGI164625[] = {{&S59153[97], 9}, {NULL}};
const Segment SGI164626[] = {{&S59153[70], 6}, {NULL}};
const Segment SGI164627[] = {{&S48854[34], 10}, {NULL}};
const Segment SGI164628[] = {{&S48854[27], 6}, {NULL}};
const Segment SGI164629[] = {{&S48854[97], 9}, {NULL}};
const Segment SGI164630[] = {{&S48854[70], 6}, {NULL}};
const Segment SGI164631[] = {{&S48855[34], 10}, {NULL}};
const Segment SGI164632[] = {{&S48855[27], 6}, {NULL}};
const Segment SGI164633[] = {{&S48855[97], 9}, {NULL}};
const Segment SGI164634[] = {{&S48855[70], 6}, {NULL}};
const Segment SGI164635[] = {{&S59160[34], 6}, {NULL}};
const Segment SGI164636[] = {{&S59160[27], 6}, {NULL}};
const Segment SGI164637[] = {{&S59160[93], 9}, {NULL}};
const Segment SGI164638[] = {{&S59160[66], 6}, {NULL}};
const Segment SGI164639[] = {{&S50804[34], 14}, {NULL}};
const Segment SGI164640[] = {{&S50804[27], 6}, {NULL}};
const Segment SGI164641[] = {{&S50804[101], 9}, {NULL}};
const Segment SGI164642[] = {{&S50804[74], 6}, {NULL}};
const Segment SGI164643[] = {{&S59170[34], 18}, {NULL}};
const Segment SGI164644[] = {{&S59170[27], 6}, {NULL}};
const Segment SGI164645[] = {{&S59170[105], 9}, {NULL}};
const Segment SGI164646[] = {{&S59170[78], 6}, {NULL}};
const Segment SGI164647[] = {{&S48889[34], 11}, {NULL}};
const Segment SGI164648[] = {{&S48889[27], 6}, {NULL}};
const Segment SGI164649[] = {{&S48889[98], 9}, {NULL}};
const Segment SGI164650[] = {{&S48889[71], 6}, {NULL}};
const Segment SGI164651[] = {{&S48890[34], 10}, {NULL}};
const Segment SGI164652[] = {{&S48890[27], 6}, {NULL}};
const Segment SGI164653[] = {{&S48890[97], 9}, {NULL}};
const Segment SGI164654[] = {{&S48890[70], 6}, {NULL}};
const Segment SGI164655[] = {{&S48894[44], 10}, {NULL}};
const Segment SGI164656[] = {{&S48894[44], 22}, {NULL}};
const Segment SGI164657[] = {{&S48894[30], 6}, {NULL}};
const Segment SGI164658[] = {{&S48894[115], 3}, {NULL}};
const Segment SGI164659[] = {{&S48894[80], 6}, {NULL}};
const Segment SGI164660[] = {{&S59200[34], 14}, {NULL}};
const Segment SGI164661[] = {{&S59200[27], 6}, {NULL}};
const Segment SGI164662[] = {{&S48895[38], 10}, {NULL}};
const Segment SGI164663[] = {{&S48895[31], 6}, {NULL}};
const Segment SGI164664[] = {{&S59200[114], 9}, {NULL}};
const Segment SGI164665[] = {{&S59200[87], 6}, {NULL}};
const Segment SGI164666[] = {{&S59201[34], 14}, {NULL}};
const Segment SGI164667[] = {{&S59201[27], 6}, {NULL}};
const Segment SGI164668[] = {{&S59201[114], 9}, {NULL}};
const Segment SGI164669[] = {{&S59201[87], 6}, {NULL}};
const Segment SGI164670[] = {{&S59202[34], 7}, {NULL}};
const Segment SGI164671[] = {{&S59202[27], 6}, {NULL}};
const Segment SGI164672[] = {{&S59202[100], 9}, {NULL}};
const Segment SGI164673[] = {{&S59202[73], 6}, {NULL}};
const Segment SGI164674[] = {{&S59203[34], 7}, {NULL}};
const Segment SGI164675[] = {{&S59203[27], 6}, {NULL}};
const Segment SGI164676[] = {{&S59203[100], 9}, {NULL}};
const Segment SGI164677[] = {{&S59203[73], 6}, {NULL}};
const Segment SGI164678[] = {{&S59204[34], 10}, {NULL}};
const Segment SGI164679[] = {{&S59204[27], 6}, {NULL}};
const Segment SGI164680[] = {{&S59204[97], 9}, {NULL}};
const Segment SGI164681[] = {{&S59204[70], 6}, {NULL}};
const Segment SGI164682[] = {{&S48907[44], 10}, {NULL}};
const Segment SGI164683[] = {{&S48907[44], 22}, {NULL}};
const Segment SGI164684[] = {{&S48907[30], 6}, {NULL}};
const Segment SGI164685[] = {{&S48907[115], 3}, {NULL}};
const Segment SGI164686[] = {{&S48907[80], 6}, {NULL}};
const Segment SGI164687[] = {{&S48908[38], 10}, {NULL}};
const Segment SGI164688[] = {{&S48908[31], 6}, {NULL}};
const Segment SGI164689[] = {{&S59249[44], 14}, {NULL}};
const Segment SGI164690[] = {{&S59249[44], 26}, {NULL}};
const Segment SGI164691[] = {{&S59249[30], 6}, {NULL}};
const Segment SGI164692[] = {{&S59249[31], 3}, {NULL}};
const Segment SGI164693[] = {{&S59249[84], 6}, {NULL}};
const Segment SGI164694[] = {{&S59250[44], 11}, {NULL}};
const Segment SGI164695[] = {{&S59250[44], 20}, {NULL}};
const Segment SGI164696[] = {{&S59250[30], 6}, {NULL}};
const Segment SGI164697[] = {{&S59250[31], 3}, {NULL}};
const Segment SGI164698[] = {{&S59250[78], 6}, {NULL}};
const Segment SGI164699[] = {{&S59251[44], 11}, {NULL}};
const Segment SGI164700[] = {{&S59251[44], 20}, {NULL}};
const Segment SGI164701[] = {{&S59251[30], 6}, {NULL}};
const Segment SGI164702[] = {{&S59251[31], 3}, {NULL}};
const Segment SGI164703[] = {{&S59251[78], 6}, {NULL}};
const Segment SGI164704[] = {{&S59252[44], 11}, {NULL}};
const Segment SGI164705[] = {{&S59252[44], 21}, {NULL}};
const Segment SGI164706[] = {{&S59252[30], 6}, {NULL}};
const Segment SGI164707[] = {{&S59252[31], 3}, {NULL}};
const Segment SGI164708[] = {{&S59252[79], 6}, {NULL}};
const Segment SGI164709[] = {{&S59253[44], 11}, {NULL}};
const Segment SGI164710[] = {{&S59253[44], 21}, {NULL}};
const Segment SGI164711[] = {{&S59253[30], 6}, {NULL}};
const Segment SGI164712[] = {{&S59253[31], 3}, {NULL}};
const Segment SGI164713[] = {{&S59253[79], 6}, {NULL}};
const Segment SGI164714[] = {{&S59254[34], 10}, {NULL}};
const Segment SGI164715[] = {{&S59254[27], 6}, {NULL}};
const Segment SGI164716[] = {{&S59254[97], 9}, {NULL}};
const Segment SGI164717[] = {{&S59254[70], 6}, {NULL}};
const Segment SGI164718[] = {{&S59255[34], 10}, {NULL}};
const Segment SGI164719[] = {{&S59255[27], 6}, {NULL}};
const Segment SGI164720[] = {{&S59255[97], 9}, {NULL}};
const Segment SGI164721[] = {{&S59255[70], 6}, {NULL}};
const Segment SGI164722[] = {{&S59256[34], 9}, {NULL}};
const Segment SGI164723[] = {{&S59256[27], 6}, {NULL}};
const Segment SGI164724[] = {{&S59256[96], 9}, {NULL}};
const Segment SGI164725[] = {{&S59256[69], 6}, {NULL}};
const Segment SGI164726[] = {{&S50917[34], 11}, {NULL}};
const Segment SGI164727[] = {{&S50917[27], 6}, {NULL}};
const Segment SGI164728[] = {{&S50917[98], 9}, {NULL}};
const Segment SGI164729[] = {{&S50917[71], 6}, {NULL}};
const Segment SGI164730[] = {{&S48987[34], 10}, {NULL}};
const Segment SGI164731[] = {{&S48987[27], 6}, {NULL}};
const Segment SGI164732[] = {{&S48987[97], 9}, {NULL}};
const Segment SGI164733[] = {{&S48987[70], 6}, {NULL}};
const Segment SGI164734[] = {{&S59312[34], 13}, {NULL}};
const Segment SGI164735[] = {{&S59312[27], 6}, {NULL}};
const Segment SGI164736[] = {{&S59312[100], 9}, {NULL}};
const Segment SGI164737[] = {{&S59312[73], 6}, {NULL}};
const Segment SGI164738[] = {{&S50964[38], 9}, {NULL}};
const Segment SGI164739[] = {{&S50964[31], 6}, {NULL}};
const Segment SGI164740[] = {{&S50965[44], 9}, {NULL}};
const Segment SGI164741[] = {{&S50965[44], 21}, {NULL}};
const Segment SGI164742[] = {{&S50965[30], 6}, {NULL}};
const Segment SGI164743[] = {{&S50965[114], 3}, {NULL}};
const Segment SGI164744[] = {{&S50965[79], 6}, {NULL}};
const Segment SGI164745[] = {{&S50966[44], 9}, {NULL}};
const Segment SGI164746[] = {{&S50966[44], 21}, {NULL}};
const Segment SGI164747[] = {{&S50966[30], 6}, {NULL}};
const Segment SGI164748[] = {{&S59317[44], 10}, {NULL}};
const Segment SGI164749[] = {{&S59317[44], 22}, {NULL}};
const Segment SGI164750[] = {{&S59317[30], 6}, {NULL}};
const Segment SGI164751[] = {{&S50966[114], 3}, {NULL}};
const Segment SGI164752[] = {{&S50966[79], 6}, {NULL}};
const Segment SGI164753[] = {{&S59317[31], 3}, {NULL}};
const Segment SGI164754[] = {{&S59317[80], 6}, {NULL}};
const Segment SGI164755[] = {{&S59318[38], 10}, {NULL}};
const Segment SGI164756[] = {{&S59318[31], 6}, {NULL}};
const Segment SGI164757[] = {{&S59322[44], 7}, {NULL}};
const Segment SGI164758[] = {{&S59322[44], 24}, {NULL}};
const Segment SGI164759[] = {{&S59322[30], 6}, {NULL}};
const Segment SGI164760[] = {{&S59322[31], 3}, {NULL}};
const Segment SGI164761[] = {{&S59322[82], 6}, {NULL}};
const Segment SGI164762[] = {{&S59323[44], 7}, {NULL}};
const Segment SGI164763[] = {{&S59323[44], 24}, {NULL}};
const Segment SGI164764[] = {{&S59323[30], 6}, {NULL}};
const Segment SGI164765[] = {{&S59323[31], 3}, {NULL}};
const Segment SGI164766[] = {{&S59323[82], 6}, {NULL}};
const Segment SGI164767[] = {{&S59324[44], 10}, {NULL}};
const Segment SGI164768[] = {{&S59324[44], 21}, {NULL}};
const Segment SGI164769[] = {{&S59324[30], 6}, {NULL}};
const Segment SGI164770[] = {{&S59324[31], 3}, {NULL}};
const Segment SGI164771[] = {{&S59324[79], 6}, {NULL}};
const Segment SGI164772[] = {{&S59325[44], 10}, {NULL}};
const Segment SGI164773[] = {{&S59325[44], 21}, {NULL}};
const Segment SGI164774[] = {{&S59325[30], 6}, {NULL}};
const Segment SGI164775[] = {{&S59325[31], 3}, {NULL}};
const Segment SGI164776[] = {{&S50981[34], 15}, {NULL}};
const Segment SGI164777[] = {{&S59325[79], 6}, {NULL}};
const Segment SGI164778[] = {{&S50981[27], 6}, {NULL}};
const Segment SGI164779[] = {{&S50981[102], 9}, {NULL}};
const Segment SGI164780[] = {{&S50981[75], 6}, {NULL}};
const Segment SGI164781[] = {{&S59326[38], 10}, {NULL}};
const Segment SGI164782[] = {{&S59326[31], 6}, {NULL}};
const Segment SGI164783[] = {{&S59327[44], 11}, {NULL}};
const Segment SGI164784[] = {{&S59327[44], 22}, {NULL}};
const Segment SGI164785[] = {{&S59327[30], 6}, {NULL}};
const Segment SGI164786[] = {{&S59327[31], 3}, {NULL}};
const Segment SGI164787[] = {{&S59327[80], 6}, {NULL}};
const Segment SGI164788[] = {{&S59328[44], 10}, {NULL}};
const Segment SGI164789[] = {{&S59328[44], 21}, {NULL}};
const Segment SGI164790[] = {{&S59328[30], 6}, {NULL}};
const Segment SGI164791[] = {{&S59328[31], 3}, {NULL}};
const Segment SGI164792[] = {{&S59328[79], 6}, {NULL}};
const Segment SGI164793[] = {{&S59329[44], 10}, {NULL}};
const Segment SGI164794[] = {{&S59329[44], 21}, {NULL}};
const Segment SGI164795[] = {{&S59329[30], 6}, {NULL}};
const Segment SGI164796[] = {{&S59329[31], 3}, {NULL}};
const Segment SGI164797[] = {{&S59329[79], 6}, {NULL}};
const Segment SGI164798[] = {{&S59330[34], 7}, {NULL}};
const Segment SGI164799[] = {{&S59330[27], 6}, {NULL}};
const Segment SGI164800[] = {{&S49059[34], 11}, {NULL}};
const Segment SGI164801[] = {{&S59330[106], 9}, {NULL}};
const Segment SGI164802[] = {{&S59330[79], 6}, {NULL}};
const Segment SGI164803[] = {{&S49059[27], 6}, {NULL}};
const Segment SGI164804[] = {{&S49059[98], 9}, {NULL}};
const Segment SGI164805[] = {{&S50992[44], 10}, {NULL}};
const Segment SGI164806[] = {{&S49059[71], 6}, {NULL}};
const Segment SGI164807[] = {{&S50992[44], 22}, {NULL}};
const Segment SGI164808[] = {{&S50992[30], 6}, {NULL}};
const Segment SGI164809[] = {{&S50992[115], 3}, {NULL}};
const Segment SGI164810[] = {{&S50992[80], 6}, {NULL}};
const Segment SGI164811[] = {{&S59331[34], 7}, {NULL}};
const Segment SGI164812[] = {{&S59331[27], 6}, {NULL}};
const Segment SGI164813[] = {{&S49060[34], 11}, {NULL}};
const Segment SGI164814[] = {{&S59331[106], 9}, {NULL}};
const Segment SGI164815[] = {{&S49060[27], 6}, {NULL}};
const Segment SGI164816[] = {{&S59331[79], 6}, {NULL}};
const Segment SGI164817[] = {{&S49060[98], 9}, {NULL}};
const Segment SGI164818[] = {{&S49060[71], 6}, {NULL}};
const Segment SGI164819[] = {{&S50994[44], 10}, {NULL}};
const Segment SGI164820[] = {{&S50994[44], 22}, {NULL}};
const Segment SGI164821[] = {{&S50994[30], 6}, {NULL}};
const Segment SGI164822[] = {{&S50994[115], 3}, {NULL}};
const Segment SGI164823[] = {{&S50994[80], 6}, {NULL}};
const Segment SGI164824[] = {{&S49062[34], 18}, {NULL}};
const Segment SGI164825[] = {{&S49062[27], 6}, {NULL}};
const Segment SGI164826[] = {{&S50995[38], 10}, {NULL}};
const Segment SGI164827[] = {{&S50995[31], 6}, {NULL}};
const Segment SGI164828[] = {{&S49062[105], 9}, {NULL}};
const Segment SGI164829[] = {{&S49062[78], 6}, {NULL}};
const Segment SGI164830[] = {{&S49063[34], 10}, {NULL}};
const Segment SGI164831[] = {{&S49063[27], 6}, {NULL}};
const Segment SGI164832[] = {{&S49063[97], 9}, {NULL}};
const Segment SGI164833[] = {{&S49063[70], 6}, {NULL}};
const Segment SGI164834[] = {{&S50997[44], 10}, {NULL}};
const Segment SGI164835[] = {{&S50997[44], 22}, {NULL}};
const Segment SGI164836[] = {{&S50997[30], 6}, {NULL}};
const Segment SGI164837[] = {{&S50997[115], 3}, {NULL}};
const Segment SGI164838[] = {{&S59337[34], 7}, {NULL}};
const Segment SGI164839[] = {{&S50997[80], 6}, {NULL}};
const Segment SGI164840[] = {{&S59337[27], 6}, {NULL}};
const Segment SGI164841[] = {{&S49064[34], 10}, {NULL}};
const Segment SGI164842[] = {{&S49064[27], 6}, {NULL}};
const Segment SGI164843[] = {{&S59337[106], 9}, {NULL}};
const Segment SGI164844[] = {{&S59337[79], 6}, {NULL}};
const Segment SGI164845[] = {{&S49064[97], 9}, {NULL}};
const Segment SGI164846[] = {{&S49064[70], 6}, {NULL}};
const Segment SGI164847[] = {{&S59338[34], 7}, {NULL}};
const Segment SGI164848[] = {{&S59338[27], 6}, {NULL}};
const Segment SGI164849[] = {{&S49065[34], 11}, {NULL}};
const Segment SGI164850[] = {{&S59338[106], 9}, {NULL}};
const Segment SGI164851[] = {{&S49065[27], 6}, {NULL}};
const Segment SGI164852[] = {{&S59338[79], 6}, {NULL}};
const Segment SGI164853[] = {{&S49065[98], 9}, {NULL}};
const Segment SGI164854[] = {{&S49065[71], 6}, {NULL}};
const Segment SGI164855[] = {{&S59339[34], 7}, {NULL}};
const Segment SGI164856[] = {{&S59339[27], 6}, {NULL}};
const Segment SGI164857[] = {{&S49066[34], 11}, {NULL}};
const Segment SGI164858[] = {{&S49066[27], 6}, {NULL}};
const Segment SGI164859[] = {{&S59339[99], 9}, {NULL}};
const Segment SGI164860[] = {{&S59339[72], 6}, {NULL}};
const Segment SGI164861[] = {{&S49066[98], 9}, {NULL}};
const Segment SGI164862[] = {{&S49066[71], 6}, {NULL}};
const Segment SGI164863[] = {{&S59343[44], 12}, {NULL}};
const Segment SGI164864[] = {{&S59343[44], 24}, {NULL}};
const Segment SGI164865[] = {{&S59343[30], 6}, {NULL}};
const Segment SGI164866[] = {{&S59343[31], 3}, {NULL}};
const Segment SGI164867[] = {{&S59343[82], 6}, {NULL}};
const Segment SGI164868[] = {{&S49074[34], 7}, {NULL}};
const Segment SGI164869[] = {{&S49074[27], 6}, {NULL}};
const Segment SGI164870[] = {{&S49074[99], 9}, {NULL}};
const Segment SGI164871[] = {{&S49074[72], 6}, {NULL}};
const Segment SGI164872[] = {{&S49078[34], 14}, {NULL}};
const Segment SGI164873[] = {{&S49078[27], 6}, {NULL}};
const Segment SGI164874[] = {{&S49078[101], 9}, {NULL}};
const Segment SGI164875[] = {{&S49078[74], 6}, {NULL}};
const Segment SGI164876[] = {{&S51026[34], 16}, {NULL}};
const Segment SGI164877[] = {{&S51026[27], 6}, {NULL}};
const Segment SGI164878[] = {{&S59358[44], 11}, {NULL}};
const Segment SGI164879[] = {{&S59358[44], 35}, {NULL}};
const Segment SGI164880[] = {{&S59358[30], 6}, {NULL}};
const Segment SGI164881[] = {{&S51026[112], 9}, {NULL}};
const Segment SGI164882[] = {{&S51026[85], 6}, {NULL}};
const Segment SGI164883[] = {{&S59358[31], 3}, {NULL}};
const Segment SGI164884[] = {{&S59358[93], 6}, {NULL}};
const Segment SGI164885[] = {{&S51027[34], 18}, {NULL}};
const Segment SGI164886[] = {{&S51027[27], 6}, {NULL}};
const Segment SGI164887[] = {{&S51027[114], 9}, {NULL}};
const Segment SGI164888[] = {{&S59359[44], 7}, {NULL}};
const Segment SGI164889[] = {{&S51027[87], 6}, {NULL}};
const Segment SGI164890[] = {{&S59359[44], 37}, {NULL}};
const Segment SGI164891[] = {{&S59359[30], 6}, {NULL}};
const Segment SGI164892[] = {{&S59359[31], 3}, {NULL}};
const Segment SGI164893[] = {{&S59359[95], 6}, {NULL}};
const Segment SGI164894[] = {{&S51028[34], 18}, {NULL}};
const Segment SGI164895[] = {{&S51028[27], 6}, {NULL}};
const Segment SGI164896[] = {{&S51028[114], 9}, {NULL}};
const Segment SGI164897[] = {{&S51028[87], 6}, {NULL}};
const Segment SGI164898[] = {{&S51029[34], 18}, {NULL}};
const Segment SGI164899[] = {{&S51029[27], 6}, {NULL}};
const Segment SGI164900[] = {{&S51029[114], 9}, {NULL}};
const Segment SGI164901[] = {{&S51029[87], 6}, {NULL}};
const Segment SGI164902[] = {{&S55894[34], 30}, {NULL}};
const Segment SGI164903[] = {{&S55894[27], 6}, {NULL}};
const Segment SGI164904[] = {{&S55894[90], 6}, {NULL}};
const Segment SGI164905[] = {{&S49091[34], 12}, {NULL}};
const Segment SGI164906[] = {{&S49091[27], 6}, {NULL}};
const Segment SGI164907[] = {{&S51030[34], 18}, {NULL}};
const Segment SGI164908[] = {{&S51030[27], 6}, {NULL}};
const Segment SGI164909[] = {{&S49091[99], 9}, {NULL}};
const Segment SGI164910[] = {{&S49091[72], 6}, {NULL}};
const Segment SGI164911[] = {{&S51030[114], 9}, {NULL}};
const Segment SGI164912[] = {{&S51030[87], 6}, {NULL}};
const Segment SGI164913[] = {{&S49095[34], 7}, {NULL}};
const Segment SGI164914[] = {{&S49095[27], 6}, {NULL}};
const Segment SGI164915[] = {{&S59378[44], 24}, {NULL}};
const Segment SGI164916[] = {{&S59378[44], 36}, {NULL}};
const Segment SGI164917[] = {{&S59378[30], 6}, {NULL}};
const Segment SGI164918[] = {{&S59378[31], 3}, {NULL}};
const Segment SGI164919[] = {{&S59378[94], 6}, {NULL}};
const Segment SGI164920[] = {{&S59379[38], 24}, {NULL}};
const Segment SGI164921[] = {{&S59379[31], 6}, {NULL}};
const Segment SGI164922[] = {{&S59382[34], 18}, {NULL}};
const Segment SGI164923[] = {{&S59382[27], 6}, {NULL}};
const Segment SGI164924[] = {{&S59382[105], 9}, {NULL}};
const Segment SGI164925[] = {{&S59382[78], 6}, {NULL}};
const Segment SGI164926[] = {{&S51061[44], 10}, {NULL}};
const Segment SGI164927[] = {{&S51061[44], 22}, {NULL}};
const Segment SGI164928[] = {{&S51061[30], 6}, {NULL}};
const Segment SGI164929[] = {{&S51061[115], 3}, {NULL}};
const Segment SGI164930[] = {{&S51061[80], 6}, {NULL}};
const Segment SGI164931[] = {{&S51061[150], 3}, {NULL}};
const Segment SGI164932[] = {{&S51062[44], 10}, {NULL}};
const Segment SGI164933[] = {{&S51062[44], 22}, {NULL}};
const Segment SGI164934[] = {{&S51062[30], 6}, {NULL}};
const Segment SGI164935[] = {{&S51062[115], 3}, {NULL}};
const Segment SGI164936[] = {{&S51062[80], 6}, {NULL}};
const Segment SGI164937[] = {{&S51062[150], 3}, {NULL}};
const Segment SGI164938[] = {{&S55181[42], 17}, {NULL}};
const Segment SGI164939[] = {{&S55181[30], 4}, {NULL}};
const Segment SGI164940[] = {{&S55181[31], 3}, {NULL}};
const Segment SGI164941[] = {{&S55181[73], 6}, {NULL}};
const Segment SGI164942[] = {{&S55185[42], 21}, {NULL}};
const Segment SGI164943[] = {{&S55185[30], 4}, {NULL}};
const Segment SGI164944[] = {{&S55185[31], 3}, {NULL}};
const Segment SGI164945[] = {{&S55185[77], 6}, {NULL}};
const Segment SGI164946[] = {{&S59411[34], 7}, {NULL}};
const Segment SGI164947[] = {{&S59411[27], 6}, {NULL}};
const Segment SGI164948[] = {{&S59411[94], 9}, {NULL}};
const Segment SGI164949[] = {{&S59411[67], 6}, {NULL}};
const Segment SGI164950[] = {{&S59412[34], 11}, {NULL}};
const Segment SGI164951[] = {{&S59412[27], 6}, {NULL}};
const Segment SGI164952[] = {{&S59412[106], 9}, {NULL}};
const Segment SGI164953[] = {{&S45465[34], 10}, {NULL}};
const Segment SGI164954[] = {{&S59412[79], 6}, {NULL}};
const Segment SGI164955[] = {{&S45465[27], 6}, {NULL}};
const Segment SGI164956[] = {{&S45465[97], 9}, {NULL}};
const Segment SGI164957[] = {{&S45465[70], 6}, {NULL}};
const Segment SGI164958[] = {{&S59413[34], 4}, {NULL}};
const Segment SGI164959[] = {{&S59413[27], 6}, {NULL}};
const Segment SGI164960[] = {{&S45466[34], 18}, {NULL}};
const Segment SGI164961[] = {{&S59413[91], 9}, {NULL}};
const Segment SGI164962[] = {{&S59413[64], 6}, {NULL}};
const Segment SGI164963[] = {{&S45466[27], 6}, {NULL}};
const Segment SGI164964[] = {{&S51082[34], 12}, {NULL}};
const Segment SGI164965[] = {{&S45466[105], 9}, {NULL}};
const Segment SGI164966[] = {{&S51082[27], 6}, {NULL}};
const Segment SGI164967[] = {{&S45466[78], 6}, {NULL}};
const Segment SGI164968[] = {{&S51082[99], 9}, {NULL}};
const Segment SGI164969[] = {{&S51082[72], 6}, {NULL}};
const Segment SGI164970[] = {{&S59415[34], 9}, {NULL}};
const Segment SGI164971[] = {{&S59415[27], 6}, {NULL}};
const Segment SGI164972[] = {{&S59415[96], 9}, {NULL}};
const Segment SGI164973[] = {{&S59415[69], 6}, {NULL}};
const Segment SGI164974[] = {{&S59416[34], 9}, {NULL}};
const Segment SGI164975[] = {{&S59416[27], 6}, {NULL}};
const Segment SGI164976[] = {{&S59416[103], 9}, {NULL}};
const Segment SGI164977[] = {{&S59416[76], 6}, {NULL}};
const Segment SGI164978[] = {{&S59417[34], 19}, {NULL}};
const Segment SGI164979[] = {{&S59417[27], 6}, {NULL}};
const Segment SGI164980[] = {{&S59417[106], 9}, {NULL}};
const Segment SGI164981[] = {{&S59417[79], 6}, {NULL}};
const Segment SGI164982[] = {{&S59418[34], 7}, {NULL}};
const Segment SGI164983[] = {{&S59418[27], 6}, {NULL}};
const Segment SGI164984[] = {{&S59418[94], 9}, {NULL}};
const Segment SGI164985[] = {{&S59418[67], 6}, {NULL}};
const Segment SGI164986[] = {{&S59420[34], 11}, {NULL}};
const Segment SGI164987[] = {{&S59420[27], 6}, {NULL}};
const Segment SGI164988[] = {{&S59420[98], 9}, {NULL}};
const Segment SGI164989[] = {{&S59420[71], 6}, {NULL}};
const Segment SGI164990[] = {{&S59421[34], 11}, {NULL}};
const Segment SGI164991[] = {{&S59421[27], 6}, {NULL}};
const Segment SGI164992[] = {{&S59421[98], 9}, {NULL}};
const Segment SGI164993[] = {{&S59421[71], 6}, {NULL}};
const Segment SGI164994[] = {{&S59422[34], 6}, {NULL}};
const Segment SGI164995[] = {{&S59422[27], 6}, {NULL}};
const Segment SGI164996[] = {{&S59422[93], 9}, {NULL}};
const Segment SGI164997[] = {{&S59422[66], 6}, {NULL}};
const Segment SGI164998[] = {{&S59423[34], 6}, {NULL}};
const Segment SGI164999[] = {{&S59423[27], 6}, {NULL}};
const Segment SGI165000[] = {{&S59423[93], 9}, {NULL}};
const Segment SGI165001[] = {{&S59423[66], 6}, {NULL}};
const Segment SGI165002[] = {{&S59424[34], 9}, {NULL}};
const Segment SGI165003[] = {{&S59424[27], 6}, {NULL}};
const Segment SGI165004[] = {{&S59424[96], 9}, {NULL}};
const Segment SGI165005[] = {{&S59424[69], 6}, {NULL}};
const Segment SGI165006[] = {{&S59425[34], 15}, {NULL}};
const Segment SGI165007[] = {{&S59425[27], 6}, {NULL}};
const Segment SGI165008[] = {{&S59425[102], 9}, {NULL}};
const Segment SGI165009[] = {{&S59425[75], 6}, {NULL}};
const Segment SGI165010[] = {{&S59426[34], 7}, {NULL}};
const Segment SGI165011[] = {{&S59426[27], 6}, {NULL}};
const Segment SGI165012[] = {{&S59426[94], 9}, {NULL}};
const Segment SGI165013[] = {{&S59426[67], 6}, {NULL}};
const Segment SGI165014[] = {{&S59427[34], 7}, {NULL}};
const Segment SGI165015[] = {{&S59427[27], 6}, {NULL}};
const Segment SGI165016[] = {{&S59427[94], 9}, {NULL}};
const Segment SGI165017[] = {{&S59427[67], 6}, {NULL}};
const Segment SGI165018[] = {{&S59428[34], 14}, {NULL}};
const Segment SGI165019[] = {{&S59428[27], 6}, {NULL}};
const Segment SGI165020[] = {{&S59428[101], 9}, {NULL}};
const Segment SGI165021[] = {{&S59428[74], 6}, {NULL}};
const Segment SGI165022[] = {{&S59429[34], 4}, {NULL}};
const Segment SGI165023[] = {{&S59429[27], 6}, {NULL}};
const Segment SGI165024[] = {{&S59429[91], 9}, {NULL}};
const Segment SGI165025[] = {{&S59429[64], 6}, {NULL}};
const Segment SGI165026[] = {{&S51115[44], 10}, {NULL}};
const Segment SGI165027[] = {{&S51115[44], 22}, {NULL}};
const Segment SGI165028[] = {{&S51115[30], 6}, {NULL}};
const Segment SGI165029[] = {{&S59430[34], 6}, {NULL}};
const Segment SGI165030[] = {{&S59430[27], 6}, {NULL}};
const Segment SGI165031[] = {{&S51115[115], 3}, {NULL}};
const Segment SGI165032[] = {{&S59430[93], 9}, {NULL}};
const Segment SGI165033[] = {{&S51115[80], 6}, {NULL}};
const Segment SGI165034[] = {{&S59430[66], 6}, {NULL}};
const Segment SGI165035[] = {{&S51115[150], 3}, {NULL}};
const Segment SGI165036[] = {{&S59431[34], 6}, {NULL}};
const Segment SGI165037[] = {{&S59431[27], 6}, {NULL}};
const Segment SGI165038[] = {{&S51116[44], 10}, {NULL}};
const Segment SGI165039[] = {{&S51116[44], 22}, {NULL}};
const Segment SGI165040[] = {{&S51116[30], 6}, {NULL}};
const Segment SGI165041[] = {{&S59431[93], 9}, {NULL}};
const Segment SGI165042[] = {{&S59431[66], 6}, {NULL}};
const Segment SGI165043[] = {{&S51116[115], 3}, {NULL}};
const Segment SGI165044[] = {{&S51116[80], 6}, {NULL}};
const Segment SGI165045[] = {{&S51116[150], 3}, {NULL}};
const Segment SGI165046[] = {{&S59432[34], 14}, {NULL}};
const Segment SGI165047[] = {{&S59432[27], 6}, {NULL}};
const Segment SGI165048[] = {{&S59432[101], 9}, {NULL}};
const Segment SGI165049[] = {{&S59432[74], 6}, {NULL}};
const Segment SGI165050[] = {{&S51117[44], 18}, {NULL}};
const Segment SGI165051[] = {{&S51117[44], 30}, {NULL}};
const Segment SGI165052[] = {{&S51117[30], 6}, {NULL}};
const Segment SGI165053[] = {{&S51117[123], 3}, {NULL}};
const Segment SGI165054[] = {{&S51117[88], 6}, {NULL}};
const Segment SGI165055[] = {{&S51117[158], 3}, {NULL}};
const Segment SGI165056[] = {{&S59433[34], 11}, {NULL}};
const Segment SGI165057[] = {{&S59433[27], 6}, {NULL}};
const Segment SGI165058[] = {{&S59433[98], 9}, {NULL}};
const Segment SGI165059[] = {{&S59433[71], 6}, {NULL}};
const Segment SGI165060[] = {{&S51118[44], 11}, {NULL}};
const Segment SGI165061[] = {{&S51118[44], 23}, {NULL}};
const Segment SGI165062[] = {{&S51118[30], 6}, {NULL}};
const Segment SGI165063[] = {{&S59434[34], 12}, {NULL}};
const Segment SGI165064[] = {{&S51118[116], 3}, {NULL}};
const Segment SGI165065[] = {{&S59434[27], 6}, {NULL}};
const Segment SGI165066[] = {{&S51118[81], 6}, {NULL}};
const Segment SGI165067[] = {{&S51118[151], 3}, {NULL}};
const Segment SGI165068[] = {{&S59434[99], 9}, {NULL}};
const Segment SGI165069[] = {{&S59434[72], 6}, {NULL}};
const Segment SGI165070[] = {{&S59435[34], 11}, {NULL}};
const Segment SGI165071[] = {{&S59435[27], 6}, {NULL}};
const Segment SGI165072[] = {{&S59435[98], 9}, {NULL}};
const Segment SGI165073[] = {{&S59435[71], 6}, {NULL}};
const Segment SGI165074[] = {{&S59436[34], 7}, {NULL}};
const Segment SGI165075[] = {{&S59436[27], 6}, {NULL}};
const Segment SGI165076[] = {{&S59436[94], 9}, {NULL}};
const Segment SGI165077[] = {{&S59436[67], 6}, {NULL}};
const Segment SGI165078[] = {{&S59437[34], 6}, {NULL}};
const Segment SGI165079[] = {{&S59437[27], 6}, {NULL}};
const Segment SGI165080[] = {{&S59437[93], 9}, {NULL}};
const Segment SGI165081[] = {{&S59437[66], 6}, {NULL}};
const Segment SGI165082[] = {{&S59438[34], 5}, {NULL}};
const Segment SGI165083[] = {{&S59438[27], 6}, {NULL}};
const Segment SGI165084[] = {{&S59438[92], 9}, {NULL}};
const Segment SGI165085[] = {{&S59438[65], 6}, {NULL}};
const Segment SGI165086[] = {{&S59439[34], 7}, {NULL}};
const Segment SGI165087[] = {{&S59439[27], 6}, {NULL}};
const Segment SGI165088[] = {{&S59439[94], 9}, {NULL}};
const Segment SGI165089[] = {{&S59439[67], 6}, {NULL}};
const Segment SGI165090[] = {{&S59440[34], 8}, {NULL}};
const Segment SGI165091[] = {{&S59440[27], 6}, {NULL}};
const Segment SGI165092[] = {{&S59440[95], 9}, {NULL}};
const Segment SGI165093[] = {{&S59440[68], 6}, {NULL}};
const Segment SGI165094[] = {{&S59441[34], 12}, {NULL}};
const Segment SGI165095[] = {{&S59441[27], 6}, {NULL}};
const Segment SGI165096[] = {{&S59441[99], 9}, {NULL}};
const Segment SGI165097[] = {{&S59441[72], 6}, {NULL}};
const Segment SGI165098[] = {{&S59442[34], 7}, {NULL}};
const Segment SGI165099[] = {{&S59442[27], 6}, {NULL}};
const Segment SGI165100[] = {{&S59442[94], 9}, {NULL}};
const Segment SGI165101[] = {{&S59442[67], 6}, {NULL}};
const Segment SGI165102[] = {{&S59443[34], 10}, {NULL}};
const Segment SGI165103[] = {{&S59443[27], 6}, {NULL}};
const Segment SGI165104[] = {{&S59443[97], 9}, {NULL}};
const Segment SGI165105[] = {{&S59443[70], 6}, {NULL}};
const Segment SGI165106[] = {{&S59444[34], 8}, {NULL}};
const Segment SGI165107[] = {{&S59444[27], 6}, {NULL}};
const Segment SGI165108[] = {{&S59444[95], 9}, {NULL}};
const Segment SGI165109[] = {{&S59444[68], 6}, {NULL}};
const Segment SGI165110[] = {{&S51140[44], 12}, {NULL}};
const Segment SGI165111[] = {{&S51140[44], 24}, {NULL}};
const Segment SGI165112[] = {{&S51140[30], 6}, {NULL}};
const Segment SGI165113[] = {{&S51140[117], 3}, {NULL}};
const Segment SGI165114[] = {{&S51140[82], 6}, {NULL}};
const Segment SGI165115[] = {{&S49095[100], 9}, {NULL}};
const Segment SGI165116[] = {{&S49095[73], 6}, {NULL}};
const Segment SGI165117[] = {{&S59446[34], 8}, {NULL}};
const Segment SGI165118[] = {{&S59446[27], 6}, {NULL}};
const Segment SGI165119[] = {{&S59446[95], 9}, {NULL}};
const Segment SGI165120[] = {{&S59446[68], 6}, {NULL}};
const Segment SGI165121[] = {{&S49096[34], 15}, {NULL}};
const Segment SGI165122[] = {{&S49096[27], 6}, {NULL}};
const Segment SGI165123[] = {{&S49096[111], 9}, {NULL}};
const Segment SGI165124[] = {{&S49096[84], 6}, {NULL}};
const Segment SGI165125[] = {{&S49097[34], 7}, {NULL}};
const Segment SGI165126[] = {{&S49097[27], 6}, {NULL}};
const Segment SGI165127[] = {{&S49097[107], 9}, {NULL}};
const Segment SGI165128[] = {{&S49097[80], 6}, {NULL}};
const Segment SGI165129[] = {{&S49099[44], 10}, {NULL}};
const Segment SGI165130[] = {{&S49099[44], 22}, {NULL}};
const Segment SGI165131[] = {{&S49099[30], 6}, {NULL}};
const Segment SGI165132[] = {{&S49099[115], 3}, {NULL}};
const Segment SGI165133[] = {{&S49099[80], 6}, {NULL}};
const Segment SGI165134[] = {{&S49102[38], 10}, {NULL}};
const Segment SGI165135[] = {{&S49102[31], 6}, {NULL}};
const Segment SGI165136[] = {{&S49103[44], 18}, {NULL}};
const Segment SGI165137[] = {{&S49103[44], 30}, {NULL}};
const Segment SGI165138[] = {{&S49103[30], 6}, {NULL}};
const Segment SGI165139[] = {{&S49103[123], 3}, {NULL}};
const Segment SGI165140[] = {{&S49103[88], 6}, {NULL}};
const Segment SGI165141[] = {{&S59468[34], 7}, {NULL}};
const Segment SGI165142[] = {{&S59468[27], 6}, {NULL}};
const Segment SGI165143[] = {{&S59468[99], 9}, {NULL}};
const Segment SGI165144[] = {{&S59468[72], 6}, {NULL}};
const Segment SGI165145[] = {{&S59478[44], 4}, {NULL}};
const Segment SGI165146[] = {{&S59478[44], 28}, {NULL}};
const Segment SGI165147[] = {{&S59478[30], 6}, {NULL}};
const Segment SGI165148[] = {{&S59478[31], 3}, {NULL}};
const Segment SGI165149[] = {{&S59478[86], 6}, {NULL}};
const Segment SGI165150[] = {{&S59479[44], 4}, {NULL}};
const Segment SGI165151[] = {{&S59479[44], 28}, {NULL}};
const Segment SGI165152[] = {{&S59479[30], 6}, {NULL}};
const Segment SGI165153[] = {{&S59479[31], 3}, {NULL}};
const Segment SGI165154[] = {{&S59479[86], 6}, {NULL}};
const Segment SGI165155[] = {{&S59480[44], 4}, {NULL}};
const Segment SGI165156[] = {{&S59480[44], 16}, {NULL}};
const Segment SGI165157[] = {{&S59480[30], 6}, {NULL}};
const Segment SGI165158[] = {{&S59480[31], 3}, {NULL}};
const Segment SGI165159[] = {{&S59480[74], 6}, {NULL}};
const Segment SGI165160[] = {{&S59481[44], 4}, {NULL}};
const Segment SGI165161[] = {{&S59481[44], 28}, {NULL}};
const Segment SGI165162[] = {{&S59481[30], 6}, {NULL}};
const Segment SGI165163[] = {{&S59481[31], 3}, {NULL}};
const Segment SGI165164[] = {{&S59481[86], 6}, {NULL}};
const Segment SGI165165[] = {{&S59491[34], 12}, {NULL}};
const Segment SGI165166[] = {{&S59491[27], 6}, {NULL}};
const Segment SGI165167[] = {{&S59491[99], 9}, {NULL}};
const Segment SGI165168[] = {{&S59491[72], 6}, {NULL}};
const Segment SGI165169[] = {{&S59492[34], 15}, {NULL}};
const Segment SGI165170[] = {{&S59492[27], 6}, {NULL}};
const Segment SGI165171[] = {{&S59492[102], 9}, {NULL}};
const Segment SGI165172[] = {{&S59492[75], 6}, {NULL}};
const Segment SGI165173[] = {{&S59493[34], 6}, {NULL}};
const Segment SGI165174[] = {{&S59493[27], 6}, {NULL}};
const Segment SGI165175[] = {{&S59493[93], 9}, {NULL}};
const Segment SGI165176[] = {{&S59493[66], 6}, {NULL}};
const Segment SGI165177[] = {{&S59494[34], 9}, {NULL}};
const Segment SGI165178[] = {{&S59494[27], 6}, {NULL}};
const Segment SGI165179[] = {{&S59494[96], 9}, {NULL}};
const Segment SGI165180[] = {{&S59494[69], 6}, {NULL}};
const Segment SGI165181[] = {{&S59495[34], 8}, {NULL}};
const Segment SGI165182[] = {{&S59495[27], 6}, {NULL}};
const Segment SGI165183[] = {{&S59495[95], 9}, {NULL}};
const Segment SGI165184[] = {{&S59495[68], 6}, {NULL}};
const Segment SGI165185[] = {{&S59496[34], 4}, {NULL}};
const Segment SGI165186[] = {{&S59496[27], 6}, {NULL}};
const Segment SGI165187[] = {{&S59496[91], 9}, {NULL}};
const Segment SGI165188[] = {{&S59496[64], 6}, {NULL}};
const Segment SGI165189[] = {{&S59497[34], 14}, {NULL}};
const Segment SGI165190[] = {{&S59497[27], 6}, {NULL}};
const Segment SGI165191[] = {{&S59497[101], 9}, {NULL}};
const Segment SGI165192[] = {{&S59497[74], 6}, {NULL}};
const Segment SGI165193[] = {{&S59498[34], 4}, {NULL}};
const Segment SGI165194[] = {{&S59498[27], 6}, {NULL}};
const Segment SGI165195[] = {{&S59498[91], 9}, {NULL}};
const Segment SGI165196[] = {{&S59498[64], 6}, {NULL}};
const Segment SGI165197[] = {{&S59500[34], 6}, {NULL}};
const Segment SGI165198[] = {{&S59500[27], 6}, {NULL}};
const Segment SGI165199[] = {{&S59500[93], 9}, {NULL}};
const Segment SGI165200[] = {{&S59500[66], 6}, {NULL}};
const Segment SGI165201[] = {{&S59501[34], 9}, {NULL}};
const Segment SGI165202[] = {{&S59501[27], 6}, {NULL}};
const Segment SGI165203[] = {{&S59501[96], 9}, {NULL}};
const Segment SGI165204[] = {{&S59501[69], 6}, {NULL}};
const Segment SGI165205[] = {{&S36039[34], 9}, {NULL}};
const Segment SGI165206[] = {{&S36039[27], 6}, {NULL}};
const Segment SGI165207[] = {{&S36039[95], 9}, {NULL}};
const Segment SGI165208[] = {{&S36039[68], 6}, {NULL}};
const Segment SGI165209[] = {{&S59504[34], 10}, {NULL}};
const Segment SGI165210[] = {{&S59504[27], 6}, {NULL}};
const Segment SGI165211[] = {{&S59504[97], 9}, {NULL}};
const Segment SGI165212[] = {{&S59504[70], 6}, {NULL}};
const Segment SGI165213[] = {{&S59505[34], 4}, {NULL}};
const Segment SGI165214[] = {{&S59505[27], 6}, {NULL}};
const Segment SGI165215[] = {{&S36042[34], 9}, {NULL}};
const Segment SGI165216[] = {{&S36042[27], 6}, {NULL}};
const Segment SGI165217[] = {{&S59505[91], 9}, {NULL}};
const Segment SGI165218[] = {{&S59505[64], 6}, {NULL}};
const Segment SGI165219[] = {{&S36042[95], 9}, {NULL}};
const Segment SGI165220[] = {{&S36042[68], 6}, {NULL}};
const Segment SGI165221[] = {{&S59506[34], 19}, {NULL}};
const Segment SGI165222[] = {{&S59506[27], 6}, {NULL}};
const Segment SGI165223[] = {{&S59506[106], 9}, {NULL}};
const Segment SGI165224[] = {{&S59506[79], 6}, {NULL}};
const Segment SGI165225[] = {{&S59507[34], 5}, {NULL}};
const Segment SGI165226[] = {{&S59507[27], 6}, {NULL}};
const Segment SGI165227[] = {{&S59507[92], 9}, {NULL}};
const Segment SGI165228[] = {{&S59507[65], 6}, {NULL}};
const Segment SGI165229[] = {{&S59508[34], 6}, {NULL}};
const Segment SGI165230[] = {{&S59508[27], 6}, {NULL}};
const Segment SGI165231[] = {{&S59508[93], 9}, {NULL}};
const Segment SGI165232[] = {{&S59508[66], 6}, {NULL}};
const Segment SGI165233[] = {{&S51227[34], 10}, {NULL}};
const Segment SGI165234[] = {{&S51227[27], 6}, {NULL}};
const Segment SGI165235[] = {{&S59510[34], 7}, {NULL}};
const Segment SGI165236[] = {{&S59510[27], 6}, {NULL}};
const Segment SGI165237[] = {{&S51227[97], 9}, {NULL}};
const Segment SGI165238[] = {{&S59510[94], 9}, {NULL}};
const Segment SGI165239[] = {{&S51227[70], 6}, {NULL}};
const Segment SGI165240[] = {{&S59510[67], 6}, {NULL}};
const Segment SGI165241[] = {{&S51228[34], 14}, {NULL}};
const Segment SGI165242[] = {{&S59511[34], 18}, {NULL}};
const Segment SGI165243[] = {{&S51228[27], 6}, {NULL}};
const Segment SGI165244[] = {{&S59511[27], 6}, {NULL}};
const Segment SGI165245[] = {{&S51228[101], 9}, {NULL}};
const Segment SGI165246[] = {{&S51228[74], 6}, {NULL}};
const Segment SGI165247[] = {{&S59511[105], 9}, {NULL}};
const Segment SGI165248[] = {{&S59511[78], 6}, {NULL}};
const Segment SGI165249[] = {{&S59512[34], 5}, {NULL}};
const Segment SGI165250[] = {{&S59512[27], 6}, {NULL}};
const Segment SGI165251[] = {{&S59512[100], 9}, {NULL}};
const Segment SGI165252[] = {{&S59512[73], 6}, {NULL}};
const Segment SGI165253[] = {{&S59513[34], 8}, {NULL}};
const Segment SGI165254[] = {{&S59513[27], 6}, {NULL}};
const Segment SGI165255[] = {{&S59513[95], 9}, {NULL}};
const Segment SGI165256[] = {{&S51232[44], 18}, {NULL}};
const Segment SGI165257[] = {{&S59513[68], 6}, {NULL}};
const Segment SGI165258[] = {{&S51232[44], 30}, {NULL}};
const Segment SGI165259[] = {{&S51232[30], 6}, {NULL}};
const Segment SGI165260[] = {{&S51232[139], 3}, {NULL}};
const Segment SGI165261[] = {{&S51232[104], 6}, {NULL}};
const Segment SGI165262[] = {{&S51232[76], 8}, {NULL}};
const Segment SGI165263[] = {{&S59514[34], 7}, {NULL}};
const Segment SGI165264[] = {{&S59514[27], 6}, {NULL}};
const Segment SGI165265[] = {{&S59514[94], 9}, {NULL}};
const Segment SGI165266[] = {{&S59514[67], 6}, {NULL}};
const Segment SGI165267[] = {{&S51233[44], 18}, {NULL}};
const Segment SGI165268[] = {{&S51233[44], 30}, {NULL}};
const Segment SGI165269[] = {{&S51233[30], 6}, {NULL}};
const Segment SGI165270[] = {{&S51233[139], 3}, {NULL}};
const Segment SGI165271[] = {{&S51233[104], 6}, {NULL}};
const Segment SGI165272[] = {{&S51233[76], 8}, {NULL}};
const Segment SGI165273[] = {{&S59515[34], 15}, {NULL}};
const Segment SGI165274[] = {{&S59515[27], 6}, {NULL}};
const Segment SGI165275[] = {{&S59515[102], 9}, {NULL}};
const Segment SGI165276[] = {{&S59515[75], 6}, {NULL}};
const Segment SGI165277[] = {{&S51234[44], 18}, {NULL}};
const Segment SGI165278[] = {{&S51234[44], 30}, {NULL}};
const Segment SGI165279[] = {{&S51234[30], 6}, {NULL}};
const Segment SGI165280[] = {{&S51234[139], 3}, {NULL}};
const Segment SGI165281[] = {{&S51234[104], 6}, {NULL}};
const Segment SGI165282[] = {{&S59516[34], 8}, {NULL}};
const Segment SGI165283[] = {{&S51234[76], 8}, {NULL}};
const Segment SGI165284[] = {{&S59516[27], 6}, {NULL}};
const Segment SGI165285[] = {{&S59516[95], 9}, {NULL}};
const Segment SGI165286[] = {{&S59516[68], 6}, {NULL}};
const Segment SGI165287[] = {{&S59517[34], 9}, {NULL}};
const Segment SGI165288[] = {{&S59517[27], 6}, {NULL}};
const Segment SGI165289[] = {{&S59517[96], 9}, {NULL}};
const Segment SGI165290[] = {{&S51236[34], 5}, {NULL}};
const Segment SGI165291[] = {{&S59517[69], 6}, {NULL}};
const Segment SGI165292[] = {{&S51236[27], 6}, {NULL}};
const Segment SGI165293[] = {{&S51236[92], 9}, {NULL}};
const Segment SGI165294[] = {{&S51236[65], 6}, {NULL}};
const Segment SGI165295[] = {{&S59518[34], 15}, {NULL}};
const Segment SGI165296[] = {{&S59518[27], 6}, {NULL}};
const Segment SGI165297[] = {{&S59518[102], 9}, {NULL}};
const Segment SGI165298[] = {{&S59518[75], 6}, {NULL}};
const Segment SGI165299[] = {{&S59519[34], 8}, {NULL}};
const Segment SGI165300[] = {{&S59519[27], 6}, {NULL}};
const Segment SGI165301[] = {{&S59519[95], 9}, {NULL}};
const Segment SGI165302[] = {{&S59519[68], 6}, {NULL}};
const Segment SGI165303[] = {{&S59520[34], 5}, {NULL}};
const Segment SGI165304[] = {{&S59520[27], 6}, {NULL}};
const Segment SGI165305[] = {{&S59520[92], 9}, {NULL}};
const Segment SGI165306[] = {{&S59520[65], 6}, {NULL}};
const Segment SGI165307[] = {{&S59521[34], 7}, {NULL}};
const Segment SGI165308[] = {{&S59521[27], 6}, {NULL}};
const Segment SGI165309[] = {{&S59521[94], 9}, {NULL}};
const Segment SGI165310[] = {{&S59521[67], 6}, {NULL}};
const Segment SGI165311[] = {{&S59523[34], 7}, {NULL}};
const Segment SGI165312[] = {{&S59523[27], 6}, {NULL}};
const Segment SGI165313[] = {{&S59523[94], 9}, {NULL}};
const Segment SGI165314[] = {{&S59523[67], 6}, {NULL}};
const Segment SGI165315[] = {{&S59524[34], 7}, {NULL}};
const Segment SGI165316[] = {{&S59524[27], 6}, {NULL}};
const Segment SGI165317[] = {{&S59524[94], 9}, {NULL}};
const Segment SGI165318[] = {{&S59524[67], 6}, {NULL}};
const Segment SGI165319[] = {{&S59525[34], 13}, {NULL}};
const Segment SGI165320[] = {{&S59525[27], 6}, {NULL}};
const Segment SGI165321[] = {{&S59525[100], 9}, {NULL}};
const Segment SGI165322[] = {{&S59525[73], 6}, {NULL}};
const Segment SGI165323[] = {{&S59526[34], 11}, {NULL}};
const Segment SGI165324[] = {{&S59526[27], 6}, {NULL}};
const Segment SGI165325[] = {{&S59526[98], 9}, {NULL}};
const Segment SGI165326[] = {{&S59526[71], 6}, {NULL}};
const Segment SGI165327[] = {{&S59527[34], 13}, {NULL}};
const Segment SGI165328[] = {{&S59527[27], 6}, {NULL}};
const Segment SGI165329[] = {{&S59527[100], 9}, {NULL}};
const Segment SGI165330[] = {{&S59527[73], 6}, {NULL}};
const Segment SGI165331[] = {{&S59528[34], 9}, {NULL}};
const Segment SGI165332[] = {{&S59528[27], 6}, {NULL}};
const Segment SGI165333[] = {{&S59528[96], 9}, {NULL}};
const Segment SGI165334[] = {{&S59528[69], 6}, {NULL}};
const Segment SGI165335[] = {{&S51264[44], 18}, {NULL}};
const Segment SGI165336[] = {{&S51264[44], 30}, {NULL}};
const Segment SGI165337[] = {{&S51264[30], 6}, {NULL}};
const Segment SGI165338[] = {{&S59532[44], 7}, {NULL}};
const Segment SGI165339[] = {{&S59532[44], 18}, {NULL}};
const Segment SGI165340[] = {{&S59532[30], 6}, {NULL}};
const Segment SGI165341[] = {{&S51264[139], 3}, {NULL}};
const Segment SGI165342[] = {{&S59532[31], 3}, {NULL}};
const Segment SGI165343[] = {{&S51264[104], 6}, {NULL}};
const Segment SGI165344[] = {{&S59532[76], 6}, {NULL}};
const Segment SGI165345[] = {{&S51264[76], 8}, {NULL}};
const Segment SGI165346[] = {{&S51265[44], 18}, {NULL}};
const Segment SGI165347[] = {{&S59533[44], 7}, {NULL}};
const Segment SGI165348[] = {{&S51265[44], 30}, {NULL}};
const Segment SGI165349[] = {{&S59533[44], 18}, {NULL}};
const Segment SGI165350[] = {{&S51265[30], 6}, {NULL}};
const Segment SGI165351[] = {{&S59533[30], 6}, {NULL}};
const Segment SGI165352[] = {{&S59533[31], 3}, {NULL}};
const Segment SGI165353[] = {{&S51265[139], 3}, {NULL}};
const Segment SGI165354[] = {{&S59533[76], 6}, {NULL}};
const Segment SGI165355[] = {{&S51265[104], 6}, {NULL}};
const Segment SGI165356[] = {{&S51265[76], 8}, {NULL}};
const Segment SGI165357[] = {{&S59534[44], 7}, {NULL}};
const Segment SGI165358[] = {{&S59534[44], 18}, {NULL}};
const Segment SGI165359[] = {{&S59534[30], 6}, {NULL}};
const Segment SGI165360[] = {{&S51266[44], 18}, {NULL}};
const Segment SGI165361[] = {{&S51266[44], 30}, {NULL}};
const Segment SGI165362[] = {{&S59534[31], 3}, {NULL}};
const Segment SGI165363[] = {{&S51266[30], 6}, {NULL}};
const Segment SGI165364[] = {{&S59534[76], 6}, {NULL}};
const Segment SGI165365[] = {{&S51266[139], 3}, {NULL}};
const Segment SGI165366[] = {{&S51266[104], 6}, {NULL}};
const Segment SGI165367[] = {{&S51266[76], 8}, {NULL}};
const Segment SGI165368[] = {{&S59535[44], 7}, {NULL}};
const Segment SGI165369[] = {{&S59535[44], 18}, {NULL}};
const Segment SGI165370[] = {{&S59535[30], 6}, {NULL}};
const Segment SGI165371[] = {{&S59535[31], 3}, {NULL}};
const Segment SGI165372[] = {{&S59535[76], 6}, {NULL}};
const Segment SGI165373[] = {{&S49229[44], 10}, {NULL}};
const Segment SGI165374[] = {{&S49229[44], 22}, {NULL}};
const Segment SGI165375[] = {{&S59536[44], 7}, {NULL}};
const Segment SGI165376[] = {{&S49229[30], 6}, {NULL}};
const Segment SGI165377[] = {{&S59536[44], 18}, {NULL}};
const Segment SGI165378[] = {{&S59536[30], 6}, {NULL}};
const Segment SGI165379[] = {{&S49229[115], 3}, {NULL}};
const Segment SGI165380[] = {{&S59536[31], 3}, {NULL}};
const Segment SGI165381[] = {{&S49229[80], 6}, {NULL}};
const Segment SGI165382[] = {{&S59536[76], 6}, {NULL}};
const Segment SGI165383[] = {{&S49230[38], 10}, {NULL}};
const Segment SGI165384[] = {{&S59537[38], 7}, {NULL}};
const Segment SGI165385[] = {{&S49230[31], 6}, {NULL}};
const Segment SGI165386[] = {{&S59537[31], 6}, {NULL}};
const Segment SGI165387[] = {{&S49231[42], 10}, {NULL}};
const Segment SGI165388[] = {{&S51272[34], 18}, {NULL}};
const Segment SGI165389[] = {{&S49231[30], 6}, {NULL}};
const Segment SGI165390[] = {{&S51272[27], 6}, {NULL}};
const Segment SGI165391[] = {{&S49231[113], 3}, {NULL}};
const Segment SGI165392[] = {{&S49231[78], 6}, {NULL}};
const Segment SGI165393[] = {{&S51272[105], 9}, {NULL}};
const Segment SGI165394[] = {{&S51272[78], 6}, {NULL}};
const Segment SGI165395[] = {{&S51273[34], 10}, {NULL}};
const Segment SGI165396[] = {{&S51273[27], 6}, {NULL}};
const Segment SGI165397[] = {{&S51273[97], 9}, {NULL}};
const Segment SGI165398[] = {{&S51273[70], 6}, {NULL}};
const Segment SGI165399[] = {{&S49238[34], 5}, {NULL}};
const Segment SGI165400[] = {{&S49238[27], 6}, {NULL}};
const Segment SGI165401[] = {{&S49238[92], 9}, {NULL}};
const Segment SGI165402[] = {{&S49238[65], 6}, {NULL}};
const Segment SGI165403[] = {{&S59547[34], 7}, {NULL}};
const Segment SGI165404[] = {{&S59547[27], 6}, {NULL}};
const Segment SGI165405[] = {{&S59547[93], 9}, {NULL}};
const Segment SGI165406[] = {{&S59547[66], 6}, {NULL}};
const Segment SGI165407[] = {{&S59548[34], 6}, {NULL}};
const Segment SGI165408[] = {{&S59548[27], 6}, {NULL}};
const Segment SGI165409[] = {{&S59548[93], 9}, {NULL}};
const Segment SGI165410[] = {{&S59548[66], 6}, {NULL}};
const Segment SGI165411[] = {{&S59561[34], 7}, {NULL}};
const Segment SGI165412[] = {{&S59561[27], 6}, {NULL}};
const Segment SGI165413[] = {{&S59561[94], 9}, {NULL}};
const Segment SGI165414[] = {{&S59561[67], 6}, {NULL}};
const Segment SGI165415[] = {{&S59562[34], 5}, {NULL}};
const Segment SGI165416[] = {{&S59562[27], 6}, {NULL}};
const Segment SGI165417[] = {{&S59562[92], 9}, {NULL}};
const Segment SGI165418[] = {{&S59562[65], 6}, {NULL}};
const Segment SGI165419[] = {{&S59563[34], 7}, {NULL}};
const Segment SGI165420[] = {{&S59563[27], 6}, {NULL}};
const Segment SGI165421[] = {{&S59563[99], 9}, {NULL}};
const Segment SGI165422[] = {{&S59563[72], 6}, {NULL}};
const Segment SGI165423[] = {{&S59578[44], 11}, {NULL}};
const Segment SGI165424[] = {{&S59578[44], 28}, {NULL}};
const Segment SGI165425[] = {{&S59578[30], 6}, {NULL}};
const Segment SGI165426[] = {{&S59578[31], 3}, {NULL}};
const Segment SGI165427[] = {{&S59578[86], 6}, {NULL}};
const Segment SGI165428[] = {{&S59579[44], 11}, {NULL}};
const Segment SGI165429[] = {{&S59579[44], 28}, {NULL}};
const Segment SGI165430[] = {{&S59579[30], 6}, {NULL}};
const Segment SGI165431[] = {{&S59579[31], 3}, {NULL}};
const Segment SGI165432[] = {{&S59579[86], 6}, {NULL}};
const Segment SGI165433[] = {{&S59580[44], 7}, {NULL}};
const Segment SGI165434[] = {{&S59580[44], 24}, {NULL}};
const Segment SGI165435[] = {{&S59580[30], 6}, {NULL}};
const Segment SGI165436[] = {{&S59580[31], 3}, {NULL}};
const Segment SGI165437[] = {{&S59580[82], 6}, {NULL}};
const Segment SGI165438[] = {{&S59581[44], 7}, {NULL}};
const Segment SGI165439[] = {{&S59581[44], 23}, {NULL}};
const Segment SGI165440[] = {{&S59581[30], 6}, {NULL}};
const Segment SGI165441[] = {{&S59581[31], 3}, {NULL}};
const Segment SGI165442[] = {{&S59581[81], 6}, {NULL}};
const Segment SGI165443[] = {{&S59582[44], 11}, {NULL}};
const Segment SGI165444[] = {{&S59582[44], 27}, {NULL}};
const Segment SGI165445[] = {{&S59582[30], 6}, {NULL}};
const Segment SGI165446[] = {{&S59582[31], 3}, {NULL}};
const Segment SGI165447[] = {{&S59582[85], 6}, {NULL}};
const Segment SGI165448[] = {{&S59583[44], 11}, {NULL}};
const Segment SGI165449[] = {{&S59583[44], 28}, {NULL}};
const Segment SGI165450[] = {{&S59583[30], 6}, {NULL}};
const Segment SGI165451[] = {{&S51329[44], 20}, {NULL}};
const Segment SGI165452[] = {{&S59583[31], 3}, {NULL}};
const Segment SGI165453[] = {{&S51329[44], 32}, {NULL}};
const Segment SGI165454[] = {{&S51329[30], 6}, {NULL}};
const Segment SGI165455[] = {{&S59583[86], 6}, {NULL}};
const Segment SGI165456[] = {{&S51329[125], 3}, {NULL}};
const Segment SGI165457[] = {{&S51329[90], 6}, {NULL}};
const Segment SGI165458[] = {{&S59584[44], 7}, {NULL}};
const Segment SGI165459[] = {{&S59584[44], 24}, {NULL}};
const Segment SGI165460[] = {{&S59584[30], 6}, {NULL}};
const Segment SGI165461[] = {{&S51330[38], 20}, {NULL}};
const Segment SGI165462[] = {{&S51330[31], 6}, {NULL}};
const Segment SGI165463[] = {{&S59584[31], 3}, {NULL}};
const Segment SGI165464[] = {{&S59584[82], 6}, {NULL}};
const Segment SGI165465[] = {{&S51331[44], 20}, {NULL}};
const Segment SGI165466[] = {{&S59585[44], 11}, {NULL}};
const Segment SGI165467[] = {{&S51331[44], 86}, {NULL}};
const Segment SGI165468[] = {{&S59585[44], 28}, {NULL}};
const Segment SGI165469[] = {{&S51331[30], 6}, {NULL}};
const Segment SGI165470[] = {{&S59585[30], 6}, {NULL}};
const Segment SGI165471[] = {{&S59585[31], 3}, {NULL}};
const Segment SGI165472[] = {{&S59585[86], 6}, {NULL}};
const Segment SGI165473[] = {{&S51332[44], 20}, {NULL}};
const Segment SGI165474[] = {{&S51332[44], 83}, {NULL}};
const Segment SGI165475[] = {{&S59586[44], 7}, {NULL}};
const Segment SGI165476[] = {{&S51332[30], 6}, {NULL}};
const Segment SGI165477[] = {{&S59586[44], 24}, {NULL}};
const Segment SGI165478[] = {{&S59586[30], 6}, {NULL}};
const Segment SGI165479[] = {{&S59586[31], 3}, {NULL}};
const Segment SGI165480[] = {{&S59586[82], 6}, {NULL}};
const Segment SGI165481[] = {{&S51333[44], 20}, {NULL}};
const Segment SGI165482[] = {{&S51333[44], 84}, {NULL}};
const Segment SGI165483[] = {{&S51333[30], 6}, {NULL}};
const Segment SGI165484[] = {{&S59587[44], 7}, {NULL}};
const Segment SGI165485[] = {{&S59587[44], 24}, {NULL}};
const Segment SGI165486[] = {{&S59587[30], 6}, {NULL}};
const Segment SGI165487[] = {{&S59587[31], 3}, {NULL}};
const Segment SGI165488[] = {{&S59587[82], 6}, {NULL}};
const Segment SGI165489[] = {{&S51334[44], 20}, {NULL}};
const Segment SGI165490[] = {{&S51334[44], 83}, {NULL}};
const Segment SGI165491[] = {{&S51334[30], 6}, {NULL}};
const Segment SGI165492[] = {{&S59588[34], 18}, {NULL}};
const Segment SGI165493[] = {{&S59588[27], 6}, {NULL}};
const Segment SGI165494[] = {{&S59588[105], 9}, {NULL}};
const Segment SGI165495[] = {{&S59588[78], 6}, {NULL}};
const Segment SGI165496[] = {{&S51335[44], 20}, {NULL}};
const Segment SGI165497[] = {{&S51335[44], 83}, {NULL}};
const Segment SGI165498[] = {{&S51335[30], 6}, {NULL}};
const Segment SGI165499[] = {{&S51336[45], 20}, {NULL}};
const Segment SGI165500[] = {{&S51336[45], 84}, {NULL}};
const Segment SGI165501[] = {{&S51336[31], 6}, {NULL}};
const Segment SGI165502[] = {{&S51337[44], 20}, {NULL}};
const Segment SGI165503[] = {{&S51337[44], 81}, {NULL}};
const Segment SGI165504[] = {{&S51337[30], 6}, {NULL}};
const Segment SGI165505[] = {{&S51338[44], 20}, {NULL}};
const Segment SGI165506[] = {{&S51338[44], 32}, {NULL}};
const Segment SGI165507[] = {{&S51338[30], 6}, {NULL}};
const Segment SGI165508[] = {{&S51338[93], 6}, {NULL}};
const Segment SGI165509[] = {{&S51339[44], 20}, {NULL}};
const Segment SGI165510[] = {{&S51339[44], 32}, {NULL}};
const Segment SGI165511[] = {{&S51339[30], 6}, {NULL}};
const Segment SGI165512[] = {{&S51339[125], 3}, {NULL}};
const Segment SGI165513[] = {{&S51339[90], 6}, {NULL}};
const Segment SGI165514[] = {{&S51340[44], 20}, {NULL}};
const Segment SGI165515[] = {{&S51340[44], 32}, {NULL}};
const Segment SGI165516[] = {{&S51340[30], 6}, {NULL}};
const Segment SGI165517[] = {{&S51340[90], 6}, {NULL}};
const Segment SGI165518[] = {{&S51341[44], 20}, {NULL}};
const Segment SGI165519[] = {{&S51341[44], 32}, {NULL}};
const Segment SGI165520[] = {{&S51341[30], 6}, {NULL}};
const Segment SGI165521[] = {{&S51341[125], 3}, {NULL}};
const Segment SGI165522[] = {{&S49285[34], 7}, {NULL}};
const Segment SGI165523[] = {{&S51341[90], 6}, {NULL}};
const Segment SGI165524[] = {{&S49285[27], 6}, {NULL}};
const Segment SGI165525[] = {{&S49285[99], 9}, {NULL}};
const Segment SGI165526[] = {{&S49285[72], 6}, {NULL}};
const Segment SGI165527[] = {{&S51343[44], 20}, {NULL}};
const Segment SGI165528[] = {{&S51343[44], 32}, {NULL}};
const Segment SGI165529[] = {{&S49286[34], 7}, {NULL}};
const Segment SGI165530[] = {{&S51343[30], 6}, {NULL}};
const Segment SGI165531[] = {{&S49286[27], 6}, {NULL}};
const Segment SGI165532[] = {{&S51343[90], 6}, {NULL}};
const Segment SGI165533[] = {{&S49286[104], 9}, {NULL}};
const Segment SGI165534[] = {{&S49286[77], 6}, {NULL}};
const Segment SGI165535[] = {{&S51344[59], 20}, {NULL}};
const Segment SGI165536[] = {{&S51344[59], 81}, {NULL}};
const Segment SGI165537[] = {{&S49287[34], 7}, {NULL}};
const Segment SGI165538[] = {{&S51344[45], 6}, {NULL}};
const Segment SGI165539[] = {{&S49287[27], 6}, {NULL}};
const Segment SGI165540[] = {{&S49287[104], 9}, {NULL}};
const Segment SGI165541[] = {{&S49287[77], 6}, {NULL}};
const Segment SGI165542[] = {{&S49323[34], 10}, {NULL}};
const Segment SGI165543[] = {{&S49323[27], 6}, {NULL}};
const Segment SGI165544[] = {{&S49323[97], 9}, {NULL}};
const Segment SGI165545[] = {{&S49323[70], 6}, {NULL}};
const Segment SGI165546[] = {{&S49324[44], 18}, {NULL}};
const Segment SGI165547[] = {{&S49324[44], 30}, {NULL}};
const Segment SGI165548[] = {{&S49324[30], 6}, {NULL}};
const Segment SGI165549[] = {{&S49324[123], 3}, {NULL}};
const Segment SGI165550[] = {{&S49324[88], 6}, {NULL}};
const Segment SGI165551[] = {{&S49325[44], 11}, {NULL}};
const Segment SGI165552[] = {{&S49325[44], 23}, {NULL}};
const Segment SGI165553[] = {{&S49325[30], 6}, {NULL}};
const Segment SGI165554[] = {{&S49325[116], 3}, {NULL}};
const Segment SGI165555[] = {{&S49325[81], 6}, {NULL}};
const Segment SGI165556[] = {{&S49326[38], 18}, {NULL}};
const Segment SGI165557[] = {{&S49326[31], 6}, {NULL}};
const Segment SGI165558[] = {{&S45815[34], 29}, {NULL}};
const Segment SGI165559[] = {{&S45815[27], 6}, {NULL}};
const Segment SGI165560[] = {{&S45815[116], 9}, {NULL}};
const Segment SGI165561[] = {{&S45815[89], 6}, {NULL}};
const Segment SGI165562[] = {{&S45816[34], 30}, {NULL}};
const Segment SGI165563[] = {{&S45816[27], 6}, {NULL}};
const Segment SGI165564[] = {{&S45816[117], 9}, {NULL}};
const Segment SGI165565[] = {{&S45816[90], 6}, {NULL}};
const Segment SGI165566[] = {{&S45817[34], 30}, {NULL}};
const Segment SGI165567[] = {{&S45817[27], 6}, {NULL}};
const Segment SGI165568[] = {{&S45817[117], 9}, {NULL}};
const Segment SGI165569[] = {{&S45817[90], 6}, {NULL}};
const Segment SGI165570[] = {{&S45819[34], 17}, {NULL}};
const Segment SGI165571[] = {{&S45819[27], 6}, {NULL}};
const Segment SGI165572[] = {{&S49333[34], 25}, {NULL}};
const Segment SGI165573[] = {{&S45819[104], 9}, {NULL}};
const Segment SGI165574[] = {{&S49333[27], 6}, {NULL}};
const Segment SGI165575[] = {{&S45819[77], 6}, {NULL}};
const Segment SGI165576[] = {{&S49333[112], 9}, {NULL}};
const Segment SGI165577[] = {{&S49333[85], 6}, {NULL}};
const Segment SGI165578[] = {{&S49356[34], 10}, {NULL}};
const Segment SGI165579[] = {{&S49356[27], 6}, {NULL}};
const Segment SGI165580[] = {{&S49356[97], 9}, {NULL}};
const Segment SGI165581[] = {{&S49356[70], 6}, {NULL}};
const Segment SGI165582[] = {{&S49372[34], 10}, {NULL}};
const Segment SGI165583[] = {{&S49372[27], 6}, {NULL}};
const Segment SGI165584[] = {{&S49372[97], 9}, {NULL}};
const Segment SGI165585[] = {{&S49372[70], 6}, {NULL}};
const Segment SGI165586[] = {{&S49401[44], 10}, {NULL}};
const Segment SGI165587[] = {{&S49401[44], 22}, {NULL}};
const Segment SGI165588[] = {{&S49401[30], 6}, {NULL}};
const Segment SGI165589[] = {{&S49401[115], 3}, {NULL}};
const Segment SGI165590[] = {{&S49401[80], 6}, {NULL}};
const Segment SGI165591[] = {{&S49401[150], 3}, {NULL}};
const Segment SGI165592[] = {{&S49402[44], 10}, {NULL}};
const Segment SGI165593[] = {{&S49402[44], 22}, {NULL}};
const Segment SGI165594[] = {{&S49402[30], 6}, {NULL}};
const Segment SGI165595[] = {{&S49402[115], 3}, {NULL}};
const Segment SGI165596[] = {{&S49402[80], 6}, {NULL}};
const Segment SGI165597[] = {{&S49402[150], 3}, {NULL}};
const Segment SGI165598[] = {{&S49403[44], 10}, {NULL}};
const Segment SGI165599[] = {{&S49403[44], 22}, {NULL}};
const Segment SGI165600[] = {{&S49403[30], 6}, {NULL}};
const Segment SGI165601[] = {{&S49403[115], 3}, {NULL}};
const Segment SGI165602[] = {{&S49403[80], 6}, {NULL}};
const Segment SGI165603[] = {{&S49403[150], 3}, {NULL}};
const Segment SGI165604[] = {{&S49405[34], 5}, {NULL}};
const Segment SGI165605[] = {{&S49405[27], 6}, {NULL}};
const Segment SGI165606[] = {{&S49405[92], 9}, {NULL}};
const Segment SGI165607[] = {{&S49405[65], 6}, {NULL}};
const Segment SGI165608[] = {{&S51484[34], 12}, {NULL}};
const Segment SGI165609[] = {{&S51484[27], 6}, {NULL}};
const Segment SGI165610[] = {{&S51484[99], 9}, {NULL}};
const Segment SGI165611[] = {{&S51484[72], 6}, {NULL}};
const Segment SGI165612[] = {{&S59611[44], 8}, {NULL}};
const Segment SGI165613[] = {{&S59611[44], 20}, {NULL}};
const Segment SGI165614[] = {{&S59611[30], 6}, {NULL}};
const Segment SGI165615[] = {{&S59611[31], 3}, {NULL}};
const Segment SGI165616[] = {{&S59611[78], 6}, {NULL}};
const Segment SGI165617[] = {{&S59612[38], 8}, {NULL}};
const Segment SGI165618[] = {{&S59612[31], 6}, {NULL}};
const Segment SGI165619[] = {{&S59614[34], 14}, {NULL}};
const Segment SGI165620[] = {{&S59614[27], 6}, {NULL}};
const Segment SGI165621[] = {{&S59614[101], 9}, {NULL}};
const Segment SGI165622[] = {{&S59614[74], 6}, {NULL}};
const Segment SGI165623[] = {{&S59615[44], 24}, {NULL}};
const Segment SGI165624[] = {{&S59615[44], 38}, {NULL}};
const Segment SGI165625[] = {{&S59615[30], 6}, {NULL}};
const Segment SGI165626[] = {{&S59615[31], 3}, {NULL}};
const Segment SGI165627[] = {{&S59615[96], 6}, {NULL}};
const Segment SGI165628[] = {{&S59616[44], 23}, {NULL}};
const Segment SGI165629[] = {{&S59616[44], 37}, {NULL}};
const Segment SGI165630[] = {{&S59616[30], 6}, {NULL}};
const Segment SGI165631[] = {{&S59616[31], 3}, {NULL}};
const Segment SGI165632[] = {{&S59616[95], 6}, {NULL}};
const Segment SGI165633[] = {{&S59617[38], 24}, {NULL}};
const Segment SGI165634[] = {{&S59617[31], 6}, {NULL}};
const Segment SGI165635[] = {{&S59630[44], 10}, {NULL}};
const Segment SGI165636[] = {{&S59630[44], 22}, {NULL}};
const Segment SGI165637[] = {{&S59630[30], 6}, {NULL}};
const Segment SGI165638[] = {{&S59630[31], 3}, {NULL}};
const Segment SGI165639[] = {{&S59630[95], 6}, {NULL}};
const Segment SGI165640[] = {{&S59630[68], 8}, {NULL}};
const Segment SGI165641[] = {{&S59631[44], 10}, {NULL}};
const Segment SGI165642[] = {{&S59631[44], 22}, {NULL}};
const Segment SGI165643[] = {{&S59631[30], 6}, {NULL}};
const Segment SGI165644[] = {{&S59631[31], 3}, {NULL}};
const Segment SGI165645[] = {{&S59631[95], 6}, {NULL}};
const Segment SGI165646[] = {{&S59631[68], 8}, {NULL}};
const Segment SGI165647[] = {{&S59632[44], 10}, {NULL}};
const Segment SGI165648[] = {{&S59632[44], 22}, {NULL}};
const Segment SGI165649[] = {{&S59632[30], 6}, {NULL}};
const Segment SGI165650[] = {{&S59632[31], 3}, {NULL}};
const Segment SGI165651[] = {{&S59632[99], 6}, {NULL}};
const Segment SGI165652[] = {{&S59632[68], 8}, {NULL}};
const Segment SGI165653[] = {{&S59633[44], 10}, {NULL}};
const Segment SGI165654[] = {{&S59633[44], 22}, {NULL}};
const Segment SGI165655[] = {{&S59633[30], 6}, {NULL}};
const Segment SGI165656[] = {{&S59633[31], 3}, {NULL}};
const Segment SGI165657[] = {{&S59633[99], 6}, {NULL}};
const Segment SGI165658[] = {{&S59633[68], 8}, {NULL}};
const Segment SGI165659[] = {{&S36370[34], 9}, {NULL}};
const Segment SGI165660[] = {{&S36370[27], 6}, {NULL}};
const Segment SGI165661[] = {{&S36370[95], 9}, {NULL}};
const Segment SGI165662[] = {{&S59634[44], 11}, {NULL}};
const Segment SGI165663[] = {{&S36370[68], 6}, {NULL}};
const Segment SGI165664[] = {{&S59634[44], 23}, {NULL}};
const Segment SGI165665[] = {{&S59634[30], 6}, {NULL}};
const Segment SGI165666[] = {{&S59634[31], 3}, {NULL}};
const Segment SGI165667[] = {{&S59634[100], 6}, {NULL}};
const Segment SGI165668[] = {{&S59634[69], 8}, {NULL}};
const Segment SGI165669[] = {{&S36371[34], 9}, {NULL}};
const Segment SGI165670[] = {{&S36371[27], 6}, {NULL}};
const Segment SGI165671[] = {{&S36371[95], 9}, {NULL}};
const Segment SGI165672[] = {{&S36371[68], 6}, {NULL}};
const Segment SGI165673[] = {{&S59635[34], 10}, {NULL}};
const Segment SGI165674[] = {{&S59635[27], 6}, {NULL}};
const Segment SGI165675[] = {{&S59635[97], 9}, {NULL}};
const Segment SGI165676[] = {{&S59635[70], 6}, {NULL}};
const Segment SGI165677[] = {{&S59638[34], 16}, {NULL}};
const Segment SGI165678[] = {{&S59638[27], 6}, {NULL}};
const Segment SGI165679[] = {{&S59638[103], 9}, {NULL}};
const Segment SGI165680[] = {{&S59638[76], 6}, {NULL}};
const Segment SGI165681[] = {{&S59641[34], 15}, {NULL}};
const Segment SGI165682[] = {{&S59641[27], 6}, {NULL}};
const Segment SGI165683[] = {{&S59641[102], 9}, {NULL}};
const Segment SGI165684[] = {{&S59641[75], 6}, {NULL}};
const Segment SGI165685[] = {{&S59642[34], 16}, {NULL}};
const Segment SGI165686[] = {{&S59642[27], 6}, {NULL}};
const Segment SGI165687[] = {{&S59642[103], 9}, {NULL}};
const Segment SGI165688[] = {{&S59642[76], 6}, {NULL}};
const Segment SGI165689[] = {{&S59643[34], 15}, {NULL}};
const Segment SGI165690[] = {{&S59643[27], 6}, {NULL}};
const Segment SGI165691[] = {{&S59643[102], 9}, {NULL}};
const Segment SGI165692[] = {{&S59643[75], 6}, {NULL}};
const Segment SGI165693[] = {{&S59644[34], 18}, {NULL}};
const Segment SGI165694[] = {{&S59644[27], 6}, {NULL}};
const Segment SGI165695[] = {{&S59644[105], 9}, {NULL}};
const Segment SGI165696[] = {{&S59644[78], 6}, {NULL}};
const Segment SGI165697[] = {{&S59645[34], 12}, {NULL}};
const Segment SGI165698[] = {{&S59645[27], 6}, {NULL}};
const Segment SGI165699[] = {{&S59645[99], 9}, {NULL}};
const Segment SGI165700[] = {{&S59645[72], 6}, {NULL}};
const Segment SGI165701[] = {{&S59646[34], 19}, {NULL}};
const Segment SGI165702[] = {{&S59646[27], 6}, {NULL}};
const Segment SGI165703[] = {{&S59646[106], 9}, {NULL}};
const Segment SGI165704[] = {{&S59646[79], 6}, {NULL}};
const Segment SGI165705[] = {{&S59647[34], 16}, {NULL}};
const Segment SGI165706[] = {{&S59647[27], 6}, {NULL}};
const Segment SGI165707[] = {{&S59647[103], 9}, {NULL}};
const Segment SGI165708[] = {{&S59647[76], 6}, {NULL}};
const Segment SGI165709[] = {{&S59648[34], 12}, {NULL}};
const Segment SGI165710[] = {{&S59648[27], 6}, {NULL}};
const Segment SGI165711[] = {{&S59648[99], 9}, {NULL}};
const Segment SGI165712[] = {{&S59648[72], 6}, {NULL}};
const Segment SGI165713[] = {{&S59649[34], 16}, {NULL}};
const Segment SGI165714[] = {{&S59649[27], 6}, {NULL}};
const Segment SGI165715[] = {{&S59649[103], 9}, {NULL}};
const Segment SGI165716[] = {{&S59649[76], 6}, {NULL}};
const Segment SGI165717[] = {{&S59650[44], 8}, {NULL}};
const Segment SGI165718[] = {{&S59650[44], 20}, {NULL}};
const Segment SGI165719[] = {{&S59650[30], 6}, {NULL}};
const Segment SGI165720[] = {{&S59650[31], 3}, {NULL}};
const Segment SGI165721[] = {{&S59650[78], 6}, {NULL}};
const Segment SGI165722[] = {{&S59651[42], 8}, {NULL}};
const Segment SGI165723[] = {{&S59651[30], 6}, {NULL}};
const Segment SGI165724[] = {{&S59651[31], 3}, {NULL}};
const Segment SGI165725[] = {{&S59651[76], 6}, {NULL}};
const Segment SGI165726[] = {{&S59652[38], 8}, {NULL}};
const Segment SGI165727[] = {{&S59652[31], 6}, {NULL}};
const Segment SGI165728[] = {{&S59655[34], 14}, {NULL}};
const Segment SGI165729[] = {{&S59655[27], 6}, {NULL}};
const Segment SGI165730[] = {{&S59655[101], 9}, {NULL}};
const Segment SGI165731[] = {{&S59655[74], 6}, {NULL}};
const Segment SGI165732[] = {{&S46006[34], 10}, {NULL}};
const Segment SGI165733[] = {{&S46006[27], 6}, {NULL}};
const Segment SGI165734[] = {{&S46006[97], 9}, {NULL}};
const Segment SGI165735[] = {{&S46006[70], 6}, {NULL}};
const Segment SGI165736[] = {{&S49512[44], 10}, {NULL}};
const Segment SGI165737[] = {{&S49512[44], 22}, {NULL}};
const Segment SGI165738[] = {{&S49512[30], 6}, {NULL}};
const Segment SGI165739[] = {{&S49512[115], 3}, {NULL}};
const Segment SGI165740[] = {{&S49512[80], 6}, {NULL}};
const Segment SGI165741[] = {{&S49520[44], 10}, {NULL}};
const Segment SGI165742[] = {{&S49520[44], 22}, {NULL}};
const Segment SGI165743[] = {{&S49520[30], 6}, {NULL}};
const Segment SGI165744[] = {{&S49520[83], 6}, {NULL}};
const Segment SGI165745[] = {{&S49521[38], 10}, {NULL}};
const Segment SGI165746[] = {{&S49521[31], 6}, {NULL}};
const Segment SGI165747[] = {{&S59713[44], 7}, {NULL}};
const Segment SGI165748[] = {{&S59713[44], 19}, {NULL}};
const Segment SGI165749[] = {{&S59713[30], 6}, {NULL}};
const Segment SGI165750[] = {{&S59713[31], 3}, {NULL}};
const Segment SGI165751[] = {{&S59713[77], 6}, {NULL}};
const Segment SGI165752[] = {{&S59714[44], 7}, {NULL}};
const Segment SGI165753[] = {{&S59714[44], 19}, {NULL}};
const Segment SGI165754[] = {{&S59714[30], 6}, {NULL}};
const Segment SGI165755[] = {{&S59714[80], 6}, {NULL}};
const Segment SGI165756[] = {{&S49535[38], 10}, {NULL}};
const Segment SGI165757[] = {{&S49535[31], 6}, {NULL}};
const Segment SGI165758[] = {{&S49536[44], 10}, {NULL}};
const Segment SGI165759[] = {{&S49536[44], 22}, {NULL}};
const Segment SGI165760[] = {{&S49536[30], 6}, {NULL}};
const Segment SGI165761[] = {{&S49536[115], 3}, {NULL}};
const Segment SGI165762[] = {{&S49536[80], 6}, {NULL}};
const Segment SGI165763[] = {{&S49537[44], 10}, {NULL}};
const Segment SGI165764[] = {{&S49537[44], 22}, {NULL}};
const Segment SGI165765[] = {{&S49537[30], 6}, {NULL}};
const Segment SGI165766[] = {{&S49537[115], 3}, {NULL}};
const Segment SGI165767[] = {{&S49537[80], 6}, {NULL}};
const Segment SGI165768[] = {{&S49538[44], 15}, {NULL}};
const Segment SGI165769[] = {{&S49538[44], 27}, {NULL}};
const Segment SGI165770[] = {{&S49538[30], 6}, {NULL}};
const Segment SGI165771[] = {{&S49538[120], 3}, {NULL}};
const Segment SGI165772[] = {{&S49538[85], 6}, {NULL}};
const Segment SGI165773[] = {{&S49539[38], 10}, {NULL}};
const Segment SGI165774[] = {{&S49539[31], 6}, {NULL}};
const Segment SGI165775[] = {{&S49540[44], 10}, {NULL}};
const Segment SGI165776[] = {{&S49540[44], 22}, {NULL}};
const Segment SGI165777[] = {{&S49540[30], 6}, {NULL}};
const Segment SGI165778[] = {{&S49540[115], 3}, {NULL}};
const Segment SGI165779[] = {{&S49540[80], 6}, {NULL}};
const Segment SGI165780[] = {{&S49541[44], 10}, {NULL}};
const Segment SGI165781[] = {{&S49541[44], 22}, {NULL}};
const Segment SGI165782[] = {{&S49541[30], 6}, {NULL}};
const Segment SGI165783[] = {{&S49541[115], 3}, {NULL}};
const Segment SGI165784[] = {{&S49541[80], 6}, {NULL}};
const Segment SGI165785[] = {{&S49542[38], 10}, {NULL}};
const Segment SGI165786[] = {{&S49542[31], 6}, {NULL}};
const Segment SGI165787[] = {{&S49543[44], 10}, {NULL}};
const Segment SGI165788[] = {{&S49543[44], 22}, {NULL}};
const Segment SGI165789[] = {{&S49543[30], 6}, {NULL}};
const Segment SGI165790[] = {{&S49543[115], 3}, {NULL}};
const Segment SGI165791[] = {{&S49543[80], 6}, {NULL}};
const Segment SGI165792[] = {{&S59739[34], 10}, {NULL}};
const Segment SGI165793[] = {{&S59739[27], 6}, {NULL}};
const Segment SGI165794[] = {{&S59739[101], 9}, {NULL}};
const Segment SGI165795[] = {{&S59739[74], 6}, {NULL}};
const Segment SGI165796[] = {{&S49544[44], 10}, {NULL}};
const Segment SGI165797[] = {{&S49544[44], 22}, {NULL}};
const Segment SGI165798[] = {{&S49544[30], 6}, {NULL}};
const Segment SGI165799[] = {{&S49544[115], 3}, {NULL}};
const Segment SGI165800[] = {{&S49544[80], 6}, {NULL}};
const Segment SGI165801[] = {{&S49545[44], 10}, {NULL}};
const Segment SGI165802[] = {{&S49545[44], 22}, {NULL}};
const Segment SGI165803[] = {{&S49545[30], 6}, {NULL}};
const Segment SGI165804[] = {{&S49545[115], 3}, {NULL}};
const Segment SGI165805[] = {{&S49545[80], 6}, {NULL}};
const Segment SGI165806[] = {{&S59743[34], 11}, {NULL}};
const Segment SGI165807[] = {{&S49546[44], 10}, {NULL}};
const Segment SGI165808[] = {{&S59743[27], 6}, {NULL}};
const Segment SGI165809[] = {{&S49546[44], 51}, {NULL}};
const Segment SGI165810[] = {{&S49546[30], 6}, {NULL}};
const Segment SGI165811[] = {{&S59743[98], 9}, {NULL}};
const Segment SGI165812[] = {{&S49546[144], 3}, {NULL}};
const Segment SGI165813[] = {{&S59743[71], 6}, {NULL}};
const Segment SGI165814[] = {{&S49546[109], 6}, {NULL}};
const Segment SGI165815[] = {{&S49547[44], 15}, {NULL}};
const Segment SGI165816[] = {{&S49547[44], 31}, {NULL}};
const Segment SGI165817[] = {{&S49547[30], 6}, {NULL}};
const Segment SGI165818[] = {{&S49547[124], 3}, {NULL}};
const Segment SGI165819[] = {{&S49547[89], 6}, {NULL}};
const Segment SGI165820[] = {{&S49548[44], 15}, {NULL}};
const Segment SGI165821[] = {{&S49548[44], 27}, {NULL}};
const Segment SGI165822[] = {{&S49548[30], 6}, {NULL}};
const Segment SGI165823[] = {{&S49548[120], 3}, {NULL}};
const Segment SGI165824[] = {{&S49548[85], 6}, {NULL}};
const Segment SGI165825[] = {{&S59748[34], 10}, {NULL}};
const Segment SGI165826[] = {{&S59748[27], 6}, {NULL}};
const Segment SGI165827[] = {{&S59748[97], 9}, {NULL}};
const Segment SGI165828[] = {{&S59748[70], 6}, {NULL}};
const Segment SGI165829[] = {{&S49549[44], 10}, {NULL}};
const Segment SGI165830[] = {{&S49549[44], 26}, {NULL}};
const Segment SGI165831[] = {{&S49549[30], 6}, {NULL}};
const Segment SGI165832[] = {{&S49549[119], 3}, {NULL}};
const Segment SGI165833[] = {{&S49549[84], 6}, {NULL}};
const Segment SGI165834[] = {{&S59749[34], 13}, {NULL}};
const Segment SGI165835[] = {{&S59749[27], 6}, {NULL}};
const Segment SGI165836[] = {{&S59749[100], 9}, {NULL}};
const Segment SGI165837[] = {{&S59749[73], 6}, {NULL}};
const Segment SGI165838[] = {{&S49550[44], 10}, {NULL}};
const Segment SGI165839[] = {{&S49550[44], 22}, {NULL}};
const Segment SGI165840[] = {{&S49550[30], 6}, {NULL}};
const Segment SGI165841[] = {{&S49550[115], 3}, {NULL}};
const Segment SGI165842[] = {{&S49550[80], 6}, {NULL}};
const Segment SGI165843[] = {{&S49551[44], 15}, {NULL}};
const Segment SGI165844[] = {{&S49551[44], 27}, {NULL}};
const Segment SGI165845[] = {{&S49551[30], 6}, {NULL}};
const Segment SGI165846[] = {{&S49551[120], 3}, {NULL}};
const Segment SGI165847[] = {{&S49551[85], 6}, {NULL}};
const Segment SGI165848[] = {{&S49552[44], 16}, {NULL}};
const Segment SGI165849[] = {{&S49552[44], 28}, {NULL}};
const Segment SGI165850[] = {{&S49552[30], 6}, {NULL}};
const Segment SGI165851[] = {{&S49552[121], 3}, {NULL}};
const Segment SGI165852[] = {{&S49552[86], 6}, {NULL}};
const Segment SGI165853[] = {{&S49553[44], 15}, {NULL}};
const Segment SGI165854[] = {{&S49553[44], 31}, {NULL}};
const Segment SGI165855[] = {{&S49553[30], 6}, {NULL}};
const Segment SGI165856[] = {{&S49553[124], 3}, {NULL}};
const Segment SGI165857[] = {{&S49553[89], 6}, {NULL}};
const Segment SGI165858[] = {{&S59756[34], 6}, {NULL}};
const Segment SGI165859[] = {{&S59756[27], 6}, {NULL}};
const Segment SGI165860[] = {{&S59756[93], 9}, {NULL}};
const Segment SGI165861[] = {{&S59756[66], 6}, {NULL}};
const Segment SGI165862[] = {{&S49554[44], 16}, {NULL}};
const Segment SGI165863[] = {{&S49554[44], 28}, {NULL}};
const Segment SGI165864[] = {{&S49554[30], 6}, {NULL}};
const Segment SGI165865[] = {{&S49554[121], 3}, {NULL}};
const Segment SGI165866[] = {{&S49554[86], 6}, {NULL}};
const Segment SGI165867[] = {{&S59757[34], 7}, {NULL}};
const Segment SGI165868[] = {{&S59757[27], 6}, {NULL}};
const Segment SGI165869[] = {{&S59757[99], 9}, {NULL}};
const Segment SGI165870[] = {{&S59757[72], 6}, {NULL}};
const Segment SGI165871[] = {{&S49555[44], 15}, {NULL}};
const Segment SGI165872[] = {{&S49555[44], 27}, {NULL}};
const Segment SGI165873[] = {{&S49555[30], 6}, {NULL}};
const Segment SGI165874[] = {{&S49555[120], 3}, {NULL}};
const Segment SGI165875[] = {{&S49555[85], 6}, {NULL}};
const Segment SGI165876[] = {{&S49562[34], 7}, {NULL}};
const Segment SGI165877[] = {{&S49562[27], 6}, {NULL}};
const Segment SGI165878[] = {{&S49562[104], 9}, {NULL}};
const Segment SGI165879[] = {{&S49562[77], 6}, {NULL}};
const Segment SGI165880[] = {{&S51586[44], 84}, {NULL}};
const Segment SGI165881[] = {{&S51586[44], 96}, {NULL}};
const Segment SGI165882[] = {{&S51586[30], 6}, {NULL}};
const Segment SGI165883[] = {{&S51586[189], 3}, {NULL}};
const Segment SGI165884[] = {{&S51586[121], 6}, {NULL}};
const Segment SGI165885[] = {{&S51587[44], 10}, {NULL}};
const Segment SGI165886[] = {{&S51587[44], 22}, {NULL}};
const Segment SGI165887[] = {{&S51587[30], 6}, {NULL}};
const Segment SGI165888[] = {{&S51587[83], 6}, {NULL}};
const Segment SGI165889[] = {{&S51588[44], 10}, {NULL}};
const Segment SGI165890[] = {{&S51588[44], 22}, {NULL}};
const Segment SGI165891[] = {{&S51588[30], 6}, {NULL}};
const Segment SGI165892[] = {{&S51588[115], 3}, {NULL}};
const Segment SGI165893[] = {{&S51588[80], 6}, {NULL}};
const Segment SGI165894[] = {{&S51589[38], 10}, {NULL}};
const Segment SGI165895[] = {{&S51589[31], 6}, {NULL}};
const Segment SGI165896[] = {{&S51611[34], 5}, {NULL}};
const Segment SGI165897[] = {{&S51611[27], 6}, {NULL}};
const Segment SGI165898[] = {{&S51611[92], 9}, {NULL}};
const Segment SGI165899[] = {{&S51611[65], 6}, {NULL}};
const Segment SGI165900[] = {{&S49636[34], 10}, {NULL}};
const Segment SGI165901[] = {{&S49636[27], 6}, {NULL}};
const Segment SGI165902[] = {{&S49636[97], 9}, {NULL}};
const Segment SGI165903[] = {{&S49636[70], 6}, {NULL}};
const Segment SGI165904[] = {{&S49638[34], 10}, {NULL}};
const Segment SGI165905[] = {{&S49638[27], 6}, {NULL}};
const Segment SGI165906[] = {{&S49638[97], 9}, {NULL}};
const Segment SGI165907[] = {{&S49638[70], 6}, {NULL}};
const Segment SGI165908[] = {{&S49641[34], 11}, {NULL}};
const Segment SGI165909[] = {{&S49641[27], 6}, {NULL}};
const Segment SGI165910[] = {{&S49641[98], 9}, {NULL}};
const Segment SGI165911[] = {{&S49641[71], 6}, {NULL}};
const Segment SGI165912[] = {{&S49643[45], 27}, {NULL}};
const Segment SGI165913[] = {{&S49643[27], 17}, {NULL}};
const Segment SGI165914[] = {{&S49643[125], 9}, {NULL}};
const Segment SGI165915[] = {{&S49643[98], 6}, {NULL}};
const Segment SGI165916[] = {{&S51664[34], 14}, {NULL}};
const Segment SGI165917[] = {{&S51664[27], 6}, {NULL}};
const Segment SGI165918[] = {{&S51664[101], 9}, {NULL}};
const Segment SGI165919[] = {{&S51664[74], 6}, {NULL}};
const Segment SGI165920[] = {{&S51665[34], 10}, {NULL}};
const Segment SGI165921[] = {{&S51665[27], 6}, {NULL}};
const Segment SGI165922[] = {{&S51665[97], 9}, {NULL}};
const Segment SGI165923[] = {{&S51665[70], 6}, {NULL}};
const Segment SGI165924[] = {{&S49699[34], 6}, {NULL}};
const Segment SGI165925[] = {{&S49699[27], 6}, {NULL}};
const Segment SGI165926[] = {{&S49699[93], 9}, {NULL}};
const Segment SGI165927[] = {{&S49699[66], 6}, {NULL}};
const Segment SGI165928[] = {{&S36559[34], 14}, {NULL}};
const Segment SGI165929[] = {{&S36559[27], 6}, {NULL}};
const Segment SGI165930[] = {{&S36559[100], 9}, {NULL}};
const Segment SGI165931[] = {{&S36559[73], 6}, {NULL}};
const Segment SGI165932[] = {{&S36560[34], 14}, {NULL}};
const Segment SGI165933[] = {{&S36560[27], 6}, {NULL}};
const Segment SGI165934[] = {{&S36560[100], 9}, {NULL}};
const Segment SGI165935[] = {{&S36560[73], 6}, {NULL}};
const Segment SGI165936[] = {{&S51725[44], 11}, {NULL}};
const Segment SGI165937[] = {{&S51725[44], 23}, {NULL}};
const Segment SGI165938[] = {{&S51725[30], 6}, {NULL}};
const Segment SGI165939[] = {{&S51725[132], 3}, {NULL}};
const Segment SGI165940[] = {{&S51725[97], 6}, {NULL}};
const Segment SGI165941[] = {{&S51725[69], 8}, {NULL}};
const Segment SGI165942[] = {{&S51726[44], 7}, {NULL}};
const Segment SGI165943[] = {{&S51726[44], 19}, {NULL}};
const Segment SGI165944[] = {{&S51726[30], 6}, {NULL}};
const Segment SGI165945[] = {{&S51726[128], 3}, {NULL}};
const Segment SGI165946[] = {{&S51726[93], 6}, {NULL}};
const Segment SGI165947[] = {{&S51726[65], 8}, {NULL}};
const Segment SGI165948[] = {{&S51727[44], 7}, {NULL}};
const Segment SGI165949[] = {{&S51727[44], 19}, {NULL}};
const Segment SGI165950[] = {{&S51727[30], 6}, {NULL}};
const Segment SGI165951[] = {{&S51727[128], 3}, {NULL}};
const Segment SGI165952[] = {{&S51727[93], 6}, {NULL}};
const Segment SGI165953[] = {{&S51727[65], 8}, {NULL}};
const Segment SGI165954[] = {{&S51728[44], 11}, {NULL}};
const Segment SGI165955[] = {{&S51728[44], 23}, {NULL}};
const Segment SGI165956[] = {{&S51728[30], 6}, {NULL}};
const Segment SGI165957[] = {{&S51728[132], 3}, {NULL}};
const Segment SGI165958[] = {{&S51728[97], 6}, {NULL}};
const Segment SGI165959[] = {{&S51728[69], 8}, {NULL}};
const Segment SGI165960[] = {{&S51731[44], 11}, {NULL}};
const Segment SGI165961[] = {{&S51731[44], 23}, {NULL}};
const Segment SGI165962[] = {{&S51731[30], 6}, {NULL}};
const Segment SGI165963[] = {{&S51731[131], 3}, {NULL}};
const Segment SGI165964[] = {{&S51731[97], 6}, {NULL}};
const Segment SGI165965[] = {{&S51731[69], 8}, {NULL}};
const Segment SGI165966[] = {{&S59825[34], 41}, {NULL}};
const Segment SGI165967[] = {{&S59825[27], 6}, {NULL}};
const Segment SGI165968[] = {{&S59825[128], 9}, {NULL}};
const Segment SGI165969[] = {{&S59825[101], 6}, {NULL}};
const Segment SGI165970[] = {{&S59828[34], 21}, {NULL}};
const Segment SGI165971[] = {{&S59828[27], 6}, {NULL}};
const Segment SGI165972[] = {{&S59828[108], 9}, {NULL}};
const Segment SGI165973[] = {{&S59828[81], 6}, {NULL}};
const Segment SGI165974[] = {{&S59851[44], 10}, {NULL}};
const Segment SGI165975[] = {{&S59851[44], 22}, {NULL}};
const Segment SGI165976[] = {{&S59851[30], 6}, {NULL}};
const Segment SGI165977[] = {{&S59851[31], 3}, {NULL}};
const Segment SGI165978[] = {{&S59851[80], 6}, {NULL}};
const Segment SGI165979[] = {{&S59852[42], 10}, {NULL}};
const Segment SGI165980[] = {{&S59852[30], 6}, {NULL}};
const Segment SGI165981[] = {{&S59852[31], 3}, {NULL}};
const Segment SGI165982[] = {{&S59852[78], 6}, {NULL}};
const Segment SGI165983[] = {{&S59853[44], 10}, {NULL}};
const Segment SGI165984[] = {{&S59853[44], 22}, {NULL}};
const Segment SGI165985[] = {{&S59853[30], 6}, {NULL}};
const Segment SGI165986[] = {{&S59853[31], 3}, {NULL}};
const Segment SGI165987[] = {{&S59853[80], 5}, {NULL}};
const Segment SGI165988[] = {{&S59854[38], 10}, {NULL}};
const Segment SGI165989[] = {{&S59854[31], 6}, {NULL}};
const Segment SGI165990[] = {{&S59855[44], 15}, {NULL}};
const Segment SGI165991[] = {{&S59855[44], 27}, {NULL}};
const Segment SGI165992[] = {{&S59855[30], 6}, {NULL}};
const Segment SGI165993[] = {{&S59855[31], 3}, {NULL}};
const Segment SGI165994[] = {{&S59855[85], 6}, {NULL}};
const Segment SGI165995[] = {{&S46385[44], 10}, {NULL}};
const Segment SGI165996[] = {{&S46385[44], 22}, {NULL}};
const Segment SGI165997[] = {{&S46385[30], 6}, {NULL}};
const Segment SGI165998[] = {{&S46385[115], 3}, {NULL}};
const Segment SGI165999[] = {{&S46385[80], 6}, {NULL}};
const Segment SGI166000[] = {{&S46386[38], 10}, {NULL}};
const Segment SGI166001[] = {{&S46386[31], 6}, {NULL}};
const Segment SGI166002[] = {{&S46387[44], 10}, {NULL}};
const Segment SGI166003[] = {{&S46387[44], 22}, {NULL}};
const Segment SGI166004[] = {{&S46387[30], 6}, {NULL}};
const Segment SGI166005[] = {{&S46387[115], 3}, {NULL}};
const Segment SGI166006[] = {{&S46387[80], 6}, {NULL}};
const Segment SGI166007[] = {{&S46392[44], 10}, {NULL}};
const Segment SGI166008[] = {{&S46392[44], 27}, {NULL}};
const Segment SGI166009[] = {{&S46392[30], 6}, {NULL}};
const Segment SGI166010[] = {{&S46392[119], 3}, {NULL}};
const Segment SGI166011[] = {{&S46392[85], 5}, {NULL}};
const Segment SGI166012[] = {{&S51812[34], 10}, {NULL}};
const Segment SGI166013[] = {{&S51812[27], 6}, {NULL}};
const Segment SGI166014[] = {{&S51812[97], 9}, {NULL}};
const Segment SGI166015[] = {{&S51812[70], 6}, {NULL}};
const Segment SGI166016[] = {{&S59883[34], 9}, {NULL}};
const Segment SGI166017[] = {{&S59883[27], 6}, {NULL}};
const Segment SGI166018[] = {{&S59883[96], 9}, {NULL}};
const Segment SGI166019[] = {{&S59883[69], 6}, {NULL}};
const Segment SGI166020[] = {{&S59884[34], 9}, {NULL}};
const Segment SGI166021[] = {{&S59884[27], 6}, {NULL}};
const Segment SGI166022[] = {{&S59884[96], 9}, {NULL}};
const Segment SGI166023[] = {{&S59884[69], 6}, {NULL}};
const Segment SGI166024[] = {{&S59888[44], 13}, {NULL}};
const Segment SGI166025[] = {{&S59888[44], 25}, {NULL}};
const Segment SGI166026[] = {{&S59888[30], 6}, {NULL}};
const Segment SGI166027[] = {{&S59888[31], 3}, {NULL}};
const Segment SGI166028[] = {{&S59888[83], 6}, {NULL}};
const Segment SGI166029[] = {{&S59889[34], 18}, {NULL}};
const Segment SGI166030[] = {{&S59889[27], 6}, {NULL}};
const Segment SGI166031[] = {{&S59889[105], 9}, {NULL}};
const Segment SGI166032[] = {{&S59889[78], 6}, {NULL}};
const Segment SGI166033[] = {{&S59891[34], 18}, {NULL}};
const Segment SGI166034[] = {{&S59891[27], 6}, {NULL}};
const Segment SGI166035[] = {{&S59891[105], 9}, {NULL}};
const Segment SGI166036[] = {{&S59891[78], 6}, {NULL}};
const Segment SGI166037[] = {{&S36725[44], 7}, {NULL}};
const Segment SGI166038[] = {{&S36725[44], 25}, {NULL}};
const Segment SGI166039[] = {{&S36725[30], 6}, {NULL}};
const Segment SGI166040[] = {{&S36725[118], 3}, {NULL}};
const Segment SGI166041[] = {{&S36725[83], 6}, {NULL}};
const Segment SGI166042[] = {{&S59927[34], 10}, {NULL}};
const Segment SGI166043[] = {{&S59927[27], 6}, {NULL}};
const Segment SGI166044[] = {{&S59927[97], 9}, {NULL}};
const Segment SGI166045[] = {{&S59927[70], 6}, {NULL}};
const Segment SGI166046[] = {{&S36726[44], 11}, {NULL}};
const Segment SGI166047[] = {{&S36726[44], 29}, {NULL}};
const Segment SGI166048[] = {{&S36726[30], 6}, {NULL}};
const Segment SGI166049[] = {{&S36726[122], 3}, {NULL}};
const Segment SGI166050[] = {{&S36726[87], 6}, {NULL}};
const Segment SGI166051[] = {{&S36728[38], 7}, {NULL}};
const Segment SGI166052[] = {{&S36728[31], 6}, {NULL}};
const Segment SGI166053[] = {{&S36729[44], 11}, {NULL}};
const Segment SGI166054[] = {{&S36729[44], 31}, {NULL}};
const Segment SGI166055[] = {{&S36729[30], 6}, {NULL}};
const Segment SGI166056[] = {{&S36729[124], 3}, {NULL}};
const Segment SGI166057[] = {{&S36729[89], 6}, {NULL}};
const Segment SGI166058[] = {{&S36730[44], 7}, {NULL}};
const Segment SGI166059[] = {{&S36730[44], 27}, {NULL}};
const Segment SGI166060[] = {{&S36730[30], 6}, {NULL}};
const Segment SGI166061[] = {{&S36730[120], 3}, {NULL}};
const Segment SGI166062[] = {{&S36730[85], 6}, {NULL}};
const Segment SGI166063[] = {{&S36732[44], 7}, {NULL}};
const Segment SGI166064[] = {{&S36732[44], 26}, {NULL}};
const Segment SGI166065[] = {{&S36732[30], 6}, {NULL}};
const Segment SGI166066[] = {{&S36732[119], 3}, {NULL}};
const Segment SGI166067[] = {{&S36732[84], 6}, {NULL}};
const Segment SGI166068[] = {{&S36733[42], 7}, {NULL}};
const Segment SGI166069[] = {{&S36733[30], 6}, {NULL}};
const Segment SGI166070[] = {{&S36733[118], 3}, {NULL}};
const Segment SGI166071[] = {{&S36733[83], 6}, {NULL}};
const Segment SGI166072[] = {{&S36734[44], 7}, {NULL}};
const Segment SGI166073[] = {{&S36734[44], 26}, {NULL}};
const Segment SGI166074[] = {{&S36734[30], 6}, {NULL}};
const Segment SGI166075[] = {{&S36734[119], 3}, {NULL}};
const Segment SGI166076[] = {{&S36734[84], 6}, {NULL}};
const Segment SGI166077[] = {{&S36735[44], 11}, {NULL}};
const Segment SGI166078[] = {{&S36735[44], 30}, {NULL}};
const Segment SGI166079[] = {{&S36735[30], 6}, {NULL}};
const Segment SGI166080[] = {{&S36735[123], 3}, {NULL}};
const Segment SGI166081[] = {{&S36735[88], 6}, {NULL}};
const Segment SGI166082[] = {{&S49761[34], 11}, {NULL}};
const Segment SGI166083[] = {{&S49761[27], 6}, {NULL}};
const Segment SGI166084[] = {{&S49761[98], 9}, {NULL}};
const Segment SGI166085[] = {{&S49761[71], 6}, {NULL}};
const Segment SGI166086[] = {{&S49762[34], 18}, {NULL}};
const Segment SGI166087[] = {{&S49762[27], 6}, {NULL}};
const Segment SGI166088[] = {{&S49762[105], 9}, {NULL}};
const Segment SGI166089[] = {{&S49762[78], 6}, {NULL}};
const Segment SGI166090[] = {{&S49763[34], 10}, {NULL}};
const Segment SGI166091[] = {{&S49763[27], 6}, {NULL}};
const Segment SGI166092[] = {{&S49763[97], 9}, {NULL}};
const Segment SGI166093[] = {{&S49763[70], 6}, {NULL}};
const Segment SGI166094[] = {{&S49792[34], 10}, {NULL}};
const Segment SGI166095[] = {{&S49792[27], 6}, {NULL}};
const Segment SGI166096[] = {{&S49792[97], 9}, {NULL}};
const Segment SGI166097[] = {{&S49792[70], 6}, {NULL}};
const Segment SGI166098[] = {{&S49793[34], 11}, {NULL}};
const Segment SGI166099[] = {{&S49793[27], 6}, {NULL}};
const Segment SGI166100[] = {{&S51929[44], 11}, {NULL}};
const Segment SGI166101[] = {{&S51929[44], 23}, {NULL}};
const Segment SGI166102[] = {{&S51929[30], 6}, {NULL}};
const Segment SGI166103[] = {{&S49793[98], 9}, {NULL}};
const Segment SGI166104[] = {{&S49793[71], 6}, {NULL}};
const Segment SGI166105[] = {{&S51929[132], 3}, {NULL}};
const Segment SGI166106[] = {{&S51929[97], 6}, {NULL}};
const Segment SGI166107[] = {{&S51929[69], 8}, {NULL}};
const Segment SGI166108[] = {{&S51930[44], 11}, {NULL}};
const Segment SGI166109[] = {{&S51930[44], 23}, {NULL}};
const Segment SGI166110[] = {{&S51930[30], 6}, {NULL}};
const Segment SGI166111[] = {{&S51930[160], 3}, {NULL}};
const Segment SGI166112[] = {{&S51930[125], 6}, {NULL}};
const Segment SGI166113[] = {{&S51930[69], 8}, {NULL}};
const Segment SGI166114[] = {{&S51932[44], 11}, {NULL}};
const Segment SGI166115[] = {{&S51932[44], 23}, {NULL}};
const Segment SGI166116[] = {{&S51932[30], 6}, {NULL}};
const Segment SGI166117[] = {{&S51932[145], 3}, {NULL}};
const Segment SGI166118[] = {{&S51932[110], 6}, {NULL}};
const Segment SGI166119[] = {{&S51932[69], 8}, {NULL}};
const Segment SGI166120[] = {{&S51940[34], 8}, {NULL}};
const Segment SGI166121[] = {{&S51940[27], 6}, {NULL}};
const Segment SGI166122[] = {{&S51940[95], 9}, {NULL}};
const Segment SGI166123[] = {{&S51940[68], 6}, {NULL}};
const Segment SGI166124[] = {{&S49840[34], 10}, {NULL}};
const Segment SGI166125[] = {{&S49840[27], 6}, {NULL}};
const Segment SGI166126[] = {{&S49840[97], 9}, {NULL}};
const Segment SGI166127[] = {{&S49840[70], 6}, {NULL}};
const Segment SGI166128[] = {{&S51997[44], 20}, {NULL}};
const Segment SGI166129[] = {{&S51997[44], 32}, {NULL}};
const Segment SGI166130[] = {{&S51997[30], 6}, {NULL}};
const Segment SGI166131[] = {{&S51997[141], 3}, {NULL}};
const Segment SGI166132[] = {{&S51997[106], 6}, {NULL}};
const Segment SGI166133[] = {{&S51997[78], 8}, {NULL}};
const Segment SGI166134[] = {{&S51998[44], 20}, {NULL}};
const Segment SGI166135[] = {{&S51998[44], 32}, {NULL}};
const Segment SGI166136[] = {{&S51998[30], 6}, {NULL}};
const Segment SGI166137[] = {{&S51998[141], 3}, {NULL}};
const Segment SGI166138[] = {{&S51998[106], 6}, {NULL}};
const Segment SGI166139[] = {{&S51998[78], 8}, {NULL}};
const Segment SGI166140[] = {{&S51999[44], 20}, {NULL}};
const Segment SGI166141[] = {{&S51999[44], 32}, {NULL}};
const Segment SGI166142[] = {{&S51999[30], 6}, {NULL}};
const Segment SGI166143[] = {{&S51999[141], 3}, {NULL}};
const Segment SGI166144[] = {{&S51999[106], 6}, {NULL}};
const Segment SGI166145[] = {{&S51999[78], 8}, {NULL}};
const Segment SGI166146[] = {{&S49875[34], 10}, {NULL}};
const Segment SGI166147[] = {{&S49875[27], 6}, {NULL}};
const Segment SGI166148[] = {{&S49875[97], 9}, {NULL}};
const Segment SGI166149[] = {{&S49875[70], 6}, {NULL}};
const Segment SGI166150[] = {{&S52001[44], 20}, {NULL}};
const Segment SGI166151[] = {{&S52001[44], 32}, {NULL}};
const Segment SGI166152[] = {{&S52001[30], 6}, {NULL}};
const Segment SGI166153[] = {{&S52001[140], 3}, {NULL}};
const Segment SGI166154[] = {{&S52001[106], 6}, {NULL}};
const Segment SGI166155[] = {{&S52001[78], 8}, {NULL}};
const Segment SGI166156[] = {{&S49886[34], 12}, {NULL}};
const Segment SGI166157[] = {{&S49886[27], 6}, {NULL}};
const Segment SGI166158[] = {{&S49886[99], 9}, {NULL}};
const Segment SGI166159[] = {{&S49886[72], 6}, {NULL}};
const Segment SGI166160[] = {{&S49890[34], 10}, {NULL}};
const Segment SGI166161[] = {{&S49890[27], 6}, {NULL}};
const Segment SGI166162[] = {{&S49890[97], 9}, {NULL}};
const Segment SGI166163[] = {{&S49890[70], 6}, {NULL}};
const Segment SGI166164[] = {{&S52015[34], 25}, {NULL}};
const Segment SGI166165[] = {{&S52015[27], 6}, {NULL}};
const Segment SGI166166[] = {{&S52015[112], 9}, {NULL}};
const Segment SGI166167[] = {{&S52015[85], 6}, {NULL}};
const Segment SGI166168[] = {{&S46619[34], 10}, {NULL}};
const Segment SGI166169[] = {{&S46619[27], 6}, {NULL}};
const Segment SGI166170[] = {{&S46619[97], 9}, {NULL}};
const Segment SGI166171[] = {{&S46619[70], 6}, {NULL}};
const Segment SGI166172[] = {{&S59928[34], 18}, {NULL}};
const Segment SGI166173[] = {{&S59928[27], 6}, {NULL}};
const Segment SGI166174[] = {{&S59928[105], 9}, {NULL}};
const Segment SGI166175[] = {{&S59928[78], 6}, {NULL}};
const Segment SGI166176[] = {{&S46627[34], 18}, {NULL}};
const Segment SGI166177[] = {{&S46627[27], 6}, {NULL}};
const Segment SGI166178[] = {{&S46627[105], 9}, {NULL}};
const Segment SGI166179[] = {{&S46627[78], 6}, {NULL}};
const Segment SGI166180[] = {{&S46628[34], 10}, {NULL}};
const Segment SGI166181[] = {{&S46628[27], 6}, {NULL}};
const Segment SGI166182[] = {{&S46628[97], 9}, {NULL}};
const Segment SGI166183[] = {{&S46628[70], 6}, {NULL}};
const Segment SGI166184[] = {{&S52034[34], 10}, {NULL}};
const Segment SGI166185[] = {{&S52034[27], 6}, {NULL}};
const Segment SGI166186[] = {{&S52034[97], 9}, {NULL}};
const Segment SGI166187[] = {{&S52034[70], 6}, {NULL}};
const Segment SGI166188[] = {{&S52042[34], 10}, {NULL}};
const Segment SGI166189[] = {{&S52042[27], 6}, {NULL}};
const Segment SGI166190[] = {{&S59953[34], 16}, {NULL}};
const Segment SGI166191[] = {{&S59953[27], 6}, {NULL}};
const Segment SGI166192[] = {{&S52042[97], 9}, {NULL}};
const Segment SGI166193[] = {{&S52042[70], 6}, {NULL}};
const Segment SGI166194[] = {{&S59953[110], 9}, {NULL}};
const Segment SGI166195[] = {{&S59953[83], 6}, {NULL}};
const Segment SGI166196[] = {{&S59954[34], 15}, {NULL}};
const Segment SGI166197[] = {{&S59954[27], 6}, {NULL}};
const Segment SGI166198[] = {{&S59954[108], 9}, {NULL}};
const Segment SGI166199[] = {{&S59954[81], 6}, {NULL}};
const Segment SGI166200[] = {{&S59955[34], 15}, {NULL}};
const Segment SGI166201[] = {{&S59955[27], 6}, {NULL}};
const Segment SGI166202[] = {{&S59955[108], 9}, {NULL}};
const Segment SGI166203[] = {{&S59955[81], 6}, {NULL}};
const Segment SGI166204[] = {{&S59956[34], 8}, {NULL}};
const Segment SGI166205[] = {{&S59956[27], 6}, {NULL}};
const Segment SGI166206[] = {{&S59956[102], 9}, {NULL}};
const Segment SGI166207[] = {{&S59956[75], 6}, {NULL}};
const Segment SGI166208[] = {{&S52049[44], 4}, {NULL}};
const Segment SGI166209[] = {{&S52049[44], 16}, {NULL}};
const Segment SGI166210[] = {{&S52049[30], 6}, {NULL}};
const Segment SGI166211[] = {{&S52049[109], 3}, {NULL}};
const Segment SGI166212[] = {{&S52049[74], 6}, {NULL}};
const Segment SGI166213[] = {{&S52050[38], 4}, {NULL}};
const Segment SGI166214[] = {{&S52050[31], 6}, {NULL}};
const Segment SGI166215[] = {{&S52051[44], 4}, {NULL}};
const Segment SGI166216[] = {{&S52051[44], 16}, {NULL}};
const Segment SGI166217[] = {{&S52051[30], 6}, {NULL}};
const Segment SGI166218[] = {{&S52051[109], 3}, {NULL}};
const Segment SGI166219[] = {{&S52051[74], 6}, {NULL}};
const Segment SGI166220[] = {{&S59971[44], 10}, {NULL}};
const Segment SGI166221[] = {{&S59971[44], 22}, {NULL}};
const Segment SGI166222[] = {{&S59971[30], 6}, {NULL}};
const Segment SGI166223[] = {{&S59971[31], 3}, {NULL}};
const Segment SGI166224[] = {{&S59971[95], 6}, {NULL}};
const Segment SGI166225[] = {{&S59971[68], 8}, {NULL}};
const Segment SGI166226[] = {{&S59972[44], 10}, {NULL}};
const Segment SGI166227[] = {{&S59972[44], 22}, {NULL}};
const Segment SGI166228[] = {{&S59972[30], 6}, {NULL}};
const Segment SGI166229[] = {{&S59972[31], 3}, {NULL}};
const Segment SGI166230[] = {{&S59972[95], 6}, {NULL}};
const Segment SGI166231[] = {{&S59972[68], 8}, {NULL}};
const Segment SGI166232[] = {{&S59973[44], 18}, {NULL}};
const Segment SGI166233[] = {{&S59973[44], 30}, {NULL}};
const Segment SGI166234[] = {{&S59973[30], 6}, {NULL}};
const Segment SGI166235[] = {{&S59973[31], 3}, {NULL}};
const Segment SGI166236[] = {{&S59973[107], 6}, {NULL}};
const Segment SGI166237[] = {{&S59973[76], 8}, {NULL}};
const Segment SGI166238[] = {{&S59978[44], 10}, {NULL}};
const Segment SGI166239[] = {{&S59978[44], 22}, {NULL}};
const Segment SGI166240[] = {{&S59978[30], 6}, {NULL}};
const Segment SGI166241[] = {{&S59978[31], 3}, {NULL}};
const Segment SGI166242[] = {{&S59978[95], 6}, {NULL}};
const Segment SGI166243[] = {{&S59978[68], 8}, {NULL}};
const Segment SGI166244[] = {{&S59979[44], 10}, {NULL}};
const Segment SGI166245[] = {{&S59979[44], 22}, {NULL}};
const Segment SGI166246[] = {{&S59979[30], 6}, {NULL}};
const Segment SGI166247[] = {{&S59979[31], 3}, {NULL}};
const Segment SGI166248[] = {{&S59979[95], 6}, {NULL}};
const Segment SGI166249[] = {{&S59979[68], 8}, {NULL}};
const Segment SGI166250[] = {{&S59980[44], 10}, {NULL}};
const Segment SGI166251[] = {{&S59980[44], 22}, {NULL}};
const Segment SGI166252[] = {{&S59980[30], 6}, {NULL}};
const Segment SGI166253[] = {{&S59980[31], 3}, {NULL}};
const Segment SGI166254[] = {{&S59980[95], 6}, {NULL}};
const Segment SGI166255[] = {{&S59980[68], 8}, {NULL}};
const Segment SGI166256[] = {{&S52099[44], 20}, {NULL}};
const Segment SGI166257[] = {{&S52099[44], 32}, {NULL}};
const Segment SGI166258[] = {{&S52099[30], 6}, {NULL}};
const Segment SGI166259[] = {{&S52099[125], 3}, {NULL}};
const Segment SGI166260[] = {{&S52099[90], 6}, {NULL}};
const Segment SGI166261[] = {{&S52100[38], 20}, {NULL}};
const Segment SGI166262[] = {{&S52100[31], 6}, {NULL}};
const Segment SGI166263[] = {{&S60011[34], 9}, {NULL}};
const Segment SGI166264[] = {{&S60011[27], 6}, {NULL}};
const Segment SGI166265[] = {{&S60011[96], 9}, {NULL}};
const Segment SGI166266[] = {{&S60011[69], 6}, {NULL}};
const Segment SGI166267[] = {{&S60012[34], 10}, {NULL}};
const Segment SGI166268[] = {{&S60012[27], 6}, {NULL}};
const Segment SGI166269[] = {{&S60012[97], 9}, {NULL}};
const Segment SGI166270[] = {{&S60012[70], 6}, {NULL}};
const Segment SGI166271[] = {{&S52114[44], 11}, {NULL}};
const Segment SGI166272[] = {{&S52114[44], 28}, {NULL}};
const Segment SGI166273[] = {{&S52114[30], 6}, {NULL}};
const Segment SGI166274[] = {{&S52114[121], 3}, {NULL}};
const Segment SGI166275[] = {{&S52114[86], 6}, {NULL}};
const Segment SGI166276[] = {{&S36985[34], 14}, {NULL}};
const Segment SGI166277[] = {{&S36985[27], 6}, {NULL}};
const Segment SGI166278[] = {{&S52115[44], 7}, {NULL}};
const Segment SGI166279[] = {{&S52115[44], 24}, {NULL}};
const Segment SGI166280[] = {{&S52115[30], 6}, {NULL}};
const Segment SGI166281[] = {{&S36985[100], 9}, {NULL}};
const Segment SGI166282[] = {{&S36985[73], 6}, {NULL}};
const Segment SGI166283[] = {{&S52115[117], 3}, {NULL}};
const Segment SGI166284[] = {{&S52115[82], 6}, {NULL}};
const Segment SGI166285[] = {{&S52116[44], 11}, {NULL}};
const Segment SGI166286[] = {{&S52116[44], 28}, {NULL}};
const Segment SGI166287[] = {{&S60023[34], 7}, {NULL}};
const Segment SGI166288[] = {{&S52116[30], 6}, {NULL}};
const Segment SGI166289[] = {{&S60023[27], 6}, {NULL}};
const Segment SGI166290[] = {{&S52116[121], 3}, {NULL}};
const Segment SGI166291[] = {{&S60023[94], 9}, {NULL}};
const Segment SGI166292[] = {{&S52116[86], 6}, {NULL}};
const Segment SGI166293[] = {{&S60023[67], 6}, {NULL}};
const Segment SGI166294[] = {{&S36987[34], 14}, {NULL}};
const Segment SGI166295[] = {{&S36987[27], 6}, {NULL}};
const Segment SGI166296[] = {{&S36987[100], 9}, {NULL}};
const Segment SGI166297[] = {{&S36987[73], 6}, {NULL}};
const Segment SGI166298[] = {{&S52117[44], 11}, {NULL}};
const Segment SGI166299[] = {{&S52117[44], 28}, {NULL}};
const Segment SGI166300[] = {{&S52117[30], 6}, {NULL}};
const Segment SGI166301[] = {{&S52117[121], 3}, {NULL}};
const Segment SGI166302[] = {{&S52117[86], 6}, {NULL}};
const Segment SGI166303[] = {{&S52118[38], 7}, {NULL}};
const Segment SGI166304[] = {{&S52118[31], 6}, {NULL}};
const Segment SGI166305[] = {{&S52119[44], 11}, {NULL}};
const Segment SGI166306[] = {{&S52119[44], 28}, {NULL}};
const Segment SGI166307[] = {{&S52119[30], 6}, {NULL}};
const Segment SGI166308[] = {{&S52119[121], 3}, {NULL}};
const Segment SGI166309[] = {{&S52119[86], 6}, {NULL}};
const Segment SGI166310[] = {{&S52120[44], 11}, {NULL}};
const Segment SGI166311[] = {{&S52120[44], 28}, {NULL}};
const Segment SGI166312[] = {{&S52120[30], 6}, {NULL}};
const Segment SGI166313[] = {{&S52120[121], 3}, {NULL}};
const Segment SGI166314[] = {{&S52120[86], 6}, {NULL}};
const Segment SGI166315[] = {{&S52121[44], 7}, {NULL}};
const Segment SGI166316[] = {{&S52121[44], 24}, {NULL}};
const Segment SGI166317[] = {{&S52121[30], 6}, {NULL}};
const Segment SGI166318[] = {{&S52121[117], 3}, {NULL}};
const Segment SGI166319[] = {{&S52121[82], 6}, {NULL}};
const Segment SGI166320[] = {{&S52122[44], 7}, {NULL}};
const Segment SGI166321[] = {{&S52122[44], 24}, {NULL}};
const Segment SGI166322[] = {{&S52122[30], 6}, {NULL}};
const Segment SGI166323[] = {{&S52122[117], 3}, {NULL}};
const Segment SGI166324[] = {{&S52122[82], 6}, {NULL}};
const Segment SGI166325[] = {{&S52123[44], 7}, {NULL}};
const Segment SGI166326[] = {{&S52123[44], 24}, {NULL}};
const Segment SGI166327[] = {{&S52123[30], 6}, {NULL}};
const Segment SGI166328[] = {{&S52123[117], 3}, {NULL}};
const Segment SGI166329[] = {{&S52123[82], 6}, {NULL}};
const Segment SGI166330[] = {{&S52143[34], 10}, {NULL}};
const Segment SGI166331[] = {{&S52143[27], 6}, {NULL}};
const Segment SGI166332[] = {{&S52143[97], 9}, {NULL}};
const Segment SGI166333[] = {{&S52143[70], 6}, {NULL}};
const Segment SGI166334[] = {{&S52144[34], 11}, {NULL}};
const Segment SGI166335[] = {{&S52144[27], 6}, {NULL}};
const Segment SGI166336[] = {{&S52144[98], 9}, {NULL}};
const Segment SGI166337[] = {{&S52144[71], 6}, {NULL}};
const Segment SGI166338[] = {{&S46788[34], 29}, {NULL}};
const Segment SGI166339[] = {{&S46788[27], 6}, {NULL}};
const Segment SGI166340[] = {{&S46788[116], 9}, {NULL}};
const Segment SGI166341[] = {{&S46788[89], 6}, {NULL}};
const Segment SGI166342[] = {{&S46790[34], 30}, {NULL}};
const Segment SGI166343[] = {{&S46790[27], 6}, {NULL}};
const Segment SGI166344[] = {{&S46790[117], 9}, {NULL}};
const Segment SGI166345[] = {{&S46790[90], 6}, {NULL}};
const Segment SGI166346[] = {{&S46791[34], 30}, {NULL}};
const Segment SGI166347[] = {{&S46791[27], 6}, {NULL}};
const Segment SGI166348[] = {{&S46791[117], 9}, {NULL}};
const Segment SGI166349[] = {{&S46791[90], 6}, {NULL}};
const Segment SGI166350[] = {{&S57058[34], 30}, {NULL}};
const Segment SGI166351[] = {{&S57058[27], 6}, {NULL}};
const Segment SGI166352[] = {{&S57058[90], 6}, {NULL}};
const Segment SGI166353[] = {{&S60110[44], 6}, {NULL}};
const Segment SGI166354[] = {{&S60110[44], 18}, {NULL}};
const Segment SGI166355[] = {{&S60110[30], 6}, {NULL}};
const Segment SGI166356[] = {{&S60110[31], 3}, {NULL}};
const Segment SGI166357[] = {{&S60110[76], 6}, {NULL}};
const Segment SGI166358[] = {{&S60111[38], 6}, {NULL}};
const Segment SGI166359[] = {{&S60111[31], 6}, {NULL}};
const Segment SGI166360[] = {{&S60112[44], 11}, {NULL}};
const Segment SGI166361[] = {{&S60112[44], 23}, {NULL}};
const Segment SGI166362[] = {{&S60112[30], 6}, {NULL}};
const Segment SGI166363[] = {{&S60112[31], 3}, {NULL}};
const Segment SGI166364[] = {{&S60112[81], 6}, {NULL}};
const Segment SGI166365[] = {{&S37077[41], 18}, {NULL}};
const Segment SGI166366[] = {{&S37077[41], 29}, {NULL}};
const Segment SGI166367[] = {{&S37077[27], 6}, {NULL}};
const Segment SGI166368[] = {{&S37077[119], 3}, {NULL}};
const Segment SGI166369[] = {{&S37077[123], 9}, {NULL}};
const Segment SGI166370[] = {{&S37077[84], 6}, {NULL}};
const Segment SGI166371[] = {{&S37078[34], 18}, {NULL}};
const Segment SGI166372[] = {{&S37078[27], 6}, {NULL}};
const Segment SGI166373[] = {{&S37078[104], 9}, {NULL}};
const Segment SGI166374[] = {{&S37078[77], 6}, {NULL}};
const Segment SGI166375[] = {{&S37079[41], 26}, {NULL}};
const Segment SGI166376[] = {{&S37079[41], 37}, {NULL}};
const Segment SGI166377[] = {{&S37079[27], 6}, {NULL}};
const Segment SGI166378[] = {{&S37079[127], 3}, {NULL}};
const Segment SGI166379[] = {{&S37079[131], 9}, {NULL}};
const Segment SGI166380[] = {{&S37079[92], 6}, {NULL}};
const Segment SGI166381[] = {{&S37080[41], 10}, {NULL}};
const Segment SGI166382[] = {{&S37080[41], 21}, {NULL}};
const Segment SGI166383[] = {{&S37080[27], 6}, {NULL}};
const Segment SGI166384[] = {{&S37080[111], 3}, {NULL}};
const Segment SGI166385[] = {{&S37080[115], 9}, {NULL}};
const Segment SGI166386[] = {{&S37080[76], 6}, {NULL}};
const Segment SGI166387[] = {{&S57105[34], 30}, {NULL}};
const Segment SGI166388[] = {{&S57105[27], 6}, {NULL}};
const Segment SGI166389[] = {{&S57105[90], 6}, {NULL}};
const Segment SGI166390[] = {{&S37081[41], 18}, {NULL}};
const Segment SGI166391[] = {{&S37081[41], 29}, {NULL}};
const Segment SGI166392[] = {{&S37081[27], 6}, {NULL}};
const Segment SGI166393[] = {{&S37081[119], 3}, {NULL}};
const Segment SGI166394[] = {{&S37081[123], 9}, {NULL}};
const Segment SGI166395[] = {{&S37081[84], 6}, {NULL}};
const Segment SGI166396[] = {{&S57109[34], 24}, {NULL}};
const Segment SGI166397[] = {{&S57109[27], 6}, {NULL}};
const Segment SGI166398[] = {{&S57109[84], 6}, {NULL}};
const Segment SGI166399[] = {{&S60133[34], 10}, {NULL}};
const Segment SGI166400[] = {{&S60133[27], 6}, {NULL}};
const Segment SGI166401[] = {{&S60133[97], 9}, {NULL}};
const Segment SGI166402[] = {{&S60133[70], 6}, {NULL}};
const Segment SGI166403[] = {{&S60158[34], 7}, {NULL}};
const Segment SGI166404[] = {{&S60158[27], 6}, {NULL}};
const Segment SGI166405[] = {{&S60158[94], 9}, {NULL}};
const Segment SGI166406[] = {{&S60158[67], 6}, {NULL}};
const Segment SGI166407[] = {{&S60160[44], 7}, {NULL}};
const Segment SGI166408[] = {{&S60160[44], 20}, {NULL}};
const Segment SGI166409[] = {{&S60160[30], 6}, {NULL}};
const Segment SGI166410[] = {{&S60160[31], 3}, {NULL}};
const Segment SGI166411[] = {{&S60160[78], 6}, {NULL}};
const Segment SGI166412[] = {{&S60161[44], 7}, {NULL}};
const Segment SGI166413[] = {{&S60161[44], 20}, {NULL}};
const Segment SGI166414[] = {{&S60161[30], 6}, {NULL}};
const Segment SGI166415[] = {{&S60161[31], 3}, {NULL}};
const Segment SGI166416[] = {{&S60161[78], 6}, {NULL}};
const Segment SGI166417[] = {{&S60162[44], 7}, {NULL}};
const Segment SGI166418[] = {{&S60162[44], 20}, {NULL}};
const Segment SGI166419[] = {{&S60162[30], 6}, {NULL}};
const Segment SGI166420[] = {{&S60162[31], 3}, {NULL}};
const Segment SGI166421[] = {{&S60162[78], 6}, {NULL}};
const Segment SGI166422[] = {{&S60163[44], 7}, {NULL}};
const Segment SGI166423[] = {{&S60163[44], 20}, {NULL}};
const Segment SGI166424[] = {{&S60163[30], 6}, {NULL}};
const Segment SGI166425[] = {{&S60163[31], 3}, {NULL}};
const Segment SGI166426[] = {{&S60163[78], 6}, {NULL}};
const Segment SGI166427[] = {{&S60164[38], 7}, {NULL}};
const Segment SGI166428[] = {{&S60164[31], 6}, {NULL}};
const Segment SGI166429[] = {{&S60165[44], 7}, {NULL}};
const Segment SGI166430[] = {{&S60165[44], 20}, {NULL}};
const Segment SGI166431[] = {{&S60165[30], 6}, {NULL}};
const Segment SGI166432[] = {{&S60165[31], 3}, {NULL}};
const Segment SGI166433[] = {{&S52261[44], 25}, {NULL}};
const Segment SGI166434[] = {{&S60165[78], 6}, {NULL}};
const Segment SGI166435[] = {{&S52261[44], 37}, {NULL}};
const Segment SGI166436[] = {{&S52261[30], 6}, {NULL}};
const Segment SGI166437[] = {{&S52261[98], 6}, {NULL}};
const Segment SGI166438[] = {{&S52262[44], 25}, {NULL}};
const Segment SGI166439[] = {{&S52262[44], 37}, {NULL}};
const Segment SGI166440[] = {{&S52262[30], 6}, {NULL}};
const Segment SGI166441[] = {{&S52262[31], 3}, {NULL}};
const Segment SGI166442[] = {{&S52262[95], 6}, {NULL}};
const Segment SGI166443[] = {{&S52263[44], 25}, {NULL}};
const Segment SGI166444[] = {{&S52263[44], 37}, {NULL}};
const Segment SGI166445[] = {{&S52263[30], 6}, {NULL}};
const Segment SGI166446[] = {{&S52263[31], 3}, {NULL}};
const Segment SGI166447[] = {{&S52263[95], 6}, {NULL}};
const Segment SGI166448[] = {{&S52264[44], 25}, {NULL}};
const Segment SGI166449[] = {{&S52264[44], 37}, {NULL}};
const Segment SGI166450[] = {{&S52264[30], 6}, {NULL}};
const Segment SGI166451[] = {{&S52264[31], 3}, {NULL}};
const Segment SGI166452[] = {{&S52264[95], 6}, {NULL}};
const Segment SGI166453[] = {{&S52265[44], 25}, {NULL}};
const Segment SGI166454[] = {{&S52265[44], 37}, {NULL}};
const Segment SGI166455[] = {{&S52265[30], 6}, {NULL}};
const Segment SGI166456[] = {{&S49902[34], 10}, {NULL}};
const Segment SGI166457[] = {{&S49902[27], 6}, {NULL}};
const Segment SGI166458[] = {{&S52265[31], 3}, {NULL}};
const Segment SGI166459[] = {{&S52265[95], 6}, {NULL}};
const Segment SGI166460[] = {{&S49902[97], 9}, {NULL}};
const Segment SGI166461[] = {{&S49902[70], 6}, {NULL}};
const Segment SGI166462[] = {{&S49903[34], 11}, {NULL}};
const Segment SGI166463[] = {{&S52266[44], 25}, {NULL}};
const Segment SGI166464[] = {{&S52266[44], 37}, {NULL}};
const Segment SGI166465[] = {{&S49903[27], 6}, {NULL}};
const Segment SGI166466[] = {{&S52266[30], 6}, {NULL}};
const Segment SGI166467[] = {{&S52266[31], 3}, {NULL}};
const Segment SGI166468[] = {{&S49903[98], 9}, {NULL}};
const Segment SGI166469[] = {{&S49903[71], 6}, {NULL}};
const Segment SGI166470[] = {{&S52266[95], 6}, {NULL}};
const Segment SGI166471[] = {{&S49904[34], 11}, {NULL}};
const Segment SGI166472[] = {{&S49904[27], 6}, {NULL}};
const Segment SGI166473[] = {{&S49904[98], 9}, {NULL}};
const Segment SGI166474[] = {{&S49904[71], 6}, {NULL}};
const Segment SGI166475[] = {{&S49905[34], 11}, {NULL}};
const Segment SGI166476[] = {{&S49905[27], 6}, {NULL}};
const Segment SGI166477[] = {{&S49905[98], 9}, {NULL}};
const Segment SGI166478[] = {{&S49905[71], 6}, {NULL}};
const Segment SGI166479[] = {{&S49907[34], 11}, {NULL}};
const Segment SGI166480[] = {{&S49907[27], 6}, {NULL}};
const Segment SGI166481[] = {{&S49907[98], 9}, {NULL}};
const Segment SGI166482[] = {{&S49907[71], 6}, {NULL}};
const Segment SGI166483[] = {{&S49908[34], 11}, {NULL}};
const Segment SGI166484[] = {{&S49908[27], 6}, {NULL}};
const Segment SGI166485[] = {{&S49908[98], 9}, {NULL}};
const Segment SGI166486[] = {{&S49908[71], 6}, {NULL}};
const Segment SGI166487[] = {{&S46920[34], 12}, {NULL}};
const Segment SGI166488[] = {{&S46920[27], 6}, {NULL}};
const Segment SGI166489[] = {{&S46920[99], 9}, {NULL}};
const Segment SGI166490[] = {{&S46920[72], 6}, {NULL}};
const Segment SGI166491[] = {{&S49909[34], 10}, {NULL}};
const Segment SGI166492[] = {{&S49909[27], 6}, {NULL}};
const Segment SGI166493[] = {{&S49909[97], 9}, {NULL}};
const Segment SGI166494[] = {{&S49909[70], 6}, {NULL}};
const Segment SGI166495[] = {{&S60200[44], 13}, {NULL}};
const Segment SGI166496[] = {{&S60200[44], 25}, {NULL}};
const Segment SGI166497[] = {{&S60200[30], 6}, {NULL}};
const Segment SGI166498[] = {{&S60200[31], 3}, {NULL}};
const Segment SGI166499[] = {{&S60200[83], 6}, {NULL}};
const Segment SGI166500[] = {{&S46923[34], 9}, {NULL}};
const Segment SGI166501[] = {{&S60201[44], 13}, {NULL}};
const Segment SGI166502[] = {{&S46923[27], 6}, {NULL}};
const Segment SGI166503[] = {{&S60201[44], 25}, {NULL}};
const Segment SGI166504[] = {{&S60201[30], 6}, {NULL}};
const Segment SGI166505[] = {{&S46923[96], 9}, {NULL}};
const Segment SGI166506[] = {{&S60201[31], 3}, {NULL}};
const Segment SGI166507[] = {{&S46923[69], 6}, {NULL}};
const Segment SGI166508[] = {{&S60201[83], 6}, {NULL}};
const Segment SGI166509[] = {{&S46924[34], 33}, {NULL}};
const Segment SGI166510[] = {{&S46924[27], 6}, {NULL}};
const Segment SGI166511[] = {{&S60202[44], 14}, {NULL}};
const Segment SGI166512[] = {{&S60202[44], 26}, {NULL}};
const Segment SGI166513[] = {{&S60202[30], 6}, {NULL}};
const Segment SGI166514[] = {{&S46924[120], 9}, {NULL}};
const Segment SGI166515[] = {{&S46924[93], 6}, {NULL}};
const Segment SGI166516[] = {{&S60202[31], 3}, {NULL}};
const Segment SGI166517[] = {{&S60202[84], 6}, {NULL}};
const Segment SGI166518[] = {{&S46925[34], 10}, {NULL}};
const Segment SGI166519[] = {{&S46925[27], 6}, {NULL}};
const Segment SGI166520[] = {{&S46925[97], 9}, {NULL}};
const Segment SGI166521[] = {{&S46925[70], 6}, {NULL}};
const Segment SGI166522[] = {{&S60204[44], 13}, {NULL}};
const Segment SGI166523[] = {{&S60204[44], 25}, {NULL}};
const Segment SGI166524[] = {{&S60204[30], 6}, {NULL}};
const Segment SGI166525[] = {{&S60204[31], 3}, {NULL}};
const Segment SGI166526[] = {{&S60204[83], 6}, {NULL}};
const Segment SGI166527[] = {{&S46927[34], 5}, {NULL}};
const Segment SGI166528[] = {{&S46927[27], 6}, {NULL}};
const Segment SGI166529[] = {{&S60205[44], 14}, {NULL}};
const Segment SGI166530[] = {{&S60205[44], 26}, {NULL}};
const Segment SGI166531[] = {{&S60205[30], 6}, {NULL}};
const Segment SGI166532[] = {{&S46927[92], 9}, {NULL}};
const Segment SGI166533[] = {{&S60205[31], 3}, {NULL}};
const Segment SGI166534[] = {{&S46927[65], 6}, {NULL}};
const Segment SGI166535[] = {{&S60205[84], 6}, {NULL}};
const Segment SGI166536[] = {{&S46928[34], 10}, {NULL}};
const Segment SGI166537[] = {{&S60206[38], 13}, {NULL}};
const Segment SGI166538[] = {{&S46928[27], 6}, {NULL}};
const Segment SGI166539[] = {{&S60206[31], 6}, {NULL}};
const Segment SGI166540[] = {{&S46928[97], 9}, {NULL}};
const Segment SGI166541[] = {{&S46928[70], 6}, {NULL}};
const Segment SGI166542[] = {{&S60207[34], 7}, {NULL}};
const Segment SGI166543[] = {{&S60207[27], 6}, {NULL}};
const Segment SGI166544[] = {{&S46929[34], 17}, {NULL}};
const Segment SGI166545[] = {{&S46929[27], 6}, {NULL}};
const Segment SGI166546[] = {{&S60207[94], 9}, {NULL}};
const Segment SGI166547[] = {{&S60207[67], 6}, {NULL}};
const Segment SGI166548[] = {{&S46929[104], 9}, {NULL}};
const Segment SGI166549[] = {{&S46929[77], 6}, {NULL}};
const Segment SGI166550[] = {{&S60208[34], 7}, {NULL}};
const Segment SGI166551[] = {{&S60208[27], 6}, {NULL}};
const Segment SGI166552[] = {{&S46930[34], 14}, {NULL}};
const Segment SGI166553[] = {{&S46930[27], 6}, {NULL}};
const Segment SGI166554[] = {{&S60208[93], 9}, {NULL}};
const Segment SGI166555[] = {{&S60208[66], 6}, {NULL}};
const Segment SGI166556[] = {{&S46930[101], 9}, {NULL}};
const Segment SGI166557[] = {{&S46930[74], 6}, {NULL}};
const Segment SGI166558[] = {{&S60209[34], 7}, {NULL}};
const Segment SGI166559[] = {{&S60209[27], 6}, {NULL}};
const Segment SGI166560[] = {{&S52296[34], 10}, {NULL}};
const Segment SGI166561[] = {{&S52296[27], 6}, {NULL}};
const Segment SGI166562[] = {{&S60209[93], 9}, {NULL}};
const Segment SGI166563[] = {{&S60209[66], 6}, {NULL}};
const Segment SGI166564[] = {{&S52296[97], 9}, {NULL}};
const Segment SGI166565[] = {{&S52296[70], 6}, {NULL}};
const Segment SGI166566[] = {{&S46932[34], 11}, {NULL}};
const Segment SGI166567[] = {{&S46932[27], 6}, {NULL}};
const Segment SGI166568[] = {{&S46932[98], 9}, {NULL}};
const Segment SGI166569[] = {{&S46932[71], 6}, {NULL}};
const Segment SGI166570[] = {{&S60210[34], 14}, {NULL}};
const Segment SGI166571[] = {{&S60210[27], 6}, {NULL}};
const Segment SGI166572[] = {{&S60210[101], 9}, {NULL}};
const Segment SGI166573[] = {{&S60210[74], 6}, {NULL}};
const Segment SGI166574[] = {{&S49932[44], 16}, {NULL}};
const Segment SGI166575[] = {{&S49932[44], 28}, {NULL}};
const Segment SGI166576[] = {{&S49932[30], 6}, {NULL}};
const Segment SGI166577[] = {{&S49932[121], 3}, {NULL}};
const Segment SGI166578[] = {{&S49932[86], 6}, {NULL}};
const Segment SGI166579[] = {{&S46935[34], 6}, {NULL}};
const Segment SGI166580[] = {{&S46935[27], 6}, {NULL}};
const Segment SGI166581[] = {{&S46935[93], 9}, {NULL}};
const Segment SGI166582[] = {{&S49933[44], 14}, {NULL}};
const Segment SGI166583[] = {{&S46935[66], 6}, {NULL}};
const Segment SGI166584[] = {{&S60212[34], 18}, {NULL}};
const Segment SGI166585[] = {{&S49933[44], 26}, {NULL}};
const Segment SGI166586[] = {{&S49933[30], 6}, {NULL}};
const Segment SGI166587[] = {{&S60212[27], 6}, {NULL}};
const Segment SGI166588[] = {{&S49933[119], 3}, {NULL}};
const Segment SGI166589[] = {{&S60212[105], 9}, {NULL}};
const Segment SGI166590[] = {{&S49933[84], 6}, {NULL}};
const Segment SGI166591[] = {{&S60212[78], 6}, {NULL}};
const Segment SGI166592[] = {{&S49934[52], 9}, {NULL}};
const Segment SGI166593[] = {{&S49934[52], 21}, {NULL}};
const Segment SGI166594[] = {{&S49934[38], 6}, {NULL}};
const Segment SGI166595[] = {{&S49934[122], 3}, {NULL}};
const Segment SGI166596[] = {{&S49934[87], 6}, {NULL}};
const Segment SGI166597[] = {{&S49935[38], 9}, {NULL}};
const Segment SGI166598[] = {{&S49935[31], 6}, {NULL}};
const Segment SGI166599[] = {{&S60217[34], 7}, {NULL}};
const Segment SGI166600[] = {{&S60217[27], 6}, {NULL}};
const Segment SGI166601[] = {{&S49936[44], 24}, {NULL}};
const Segment SGI166602[] = {{&S49936[44], 36}, {NULL}};
const Segment SGI166603[] = {{&S46941[34], 6}, {NULL}};
const Segment SGI166604[] = {{&S49936[30], 6}, {NULL}};
const Segment SGI166605[] = {{&S46941[27], 6}, {NULL}};
const Segment SGI166606[] = {{&S60217[94], 9}, {NULL}};
const Segment SGI166607[] = {{&S60217[67], 6}, {NULL}};
const Segment SGI166608[] = {{&S49936[129], 3}, {NULL}};
const Segment SGI166609[] = {{&S49936[94], 6}, {NULL}};
const Segment SGI166610[] = {{&S46941[93], 9}, {NULL}};
const Segment SGI166611[] = {{&S46941[66], 6}, {NULL}};
const Segment SGI166612[] = {{&S49937[44], 14}, {NULL}};
const Segment SGI166613[] = {{&S49937[44], 26}, {NULL}};
const Segment SGI166614[] = {{&S49937[30], 6}, {NULL}};
const Segment SGI166615[] = {{&S49937[119], 3}, {NULL}};
const Segment SGI166616[] = {{&S60219[34], 18}, {NULL}};
const Segment SGI166617[] = {{&S49937[84], 6}, {NULL}};
const Segment SGI166618[] = {{&S60219[27], 6}, {NULL}};
const Segment SGI166619[] = {{&S46943[34], 9}, {NULL}};
const Segment SGI166620[] = {{&S46943[27], 6}, {NULL}};
const Segment SGI166621[] = {{&S60219[105], 9}, {NULL}};
const Segment SGI166622[] = {{&S60219[78], 6}, {NULL}};
const Segment SGI166623[] = {{&S46943[96], 9}, {NULL}};
const Segment SGI166624[] = {{&S46943[69], 6}, {NULL}};
const Segment SGI166625[] = {{&S49938[44], 14}, {NULL}};
const Segment SGI166626[] = {{&S49938[44], 26}, {NULL}};
const Segment SGI166627[] = {{&S49938[30], 6}, {NULL}};
const Segment SGI166628[] = {{&S49938[119], 3}, {NULL}};
const Segment SGI166629[] = {{&S49938[84], 6}, {NULL}};
const Segment SGI166630[] = {{&S46944[34], 10}, {NULL}};
const Segment SGI166631[] = {{&S46944[27], 6}, {NULL}};
const Segment SGI166632[] = {{&S46944[97], 9}, {NULL}};
const Segment SGI166633[] = {{&S46944[70], 6}, {NULL}};
const Segment SGI166634[] = {{&S49939[44], 24}, {NULL}};
const Segment SGI166635[] = {{&S60222[34], 7}, {NULL}};
const Segment SGI166636[] = {{&S49939[44], 36}, {NULL}};
const Segment SGI166637[] = {{&S49939[30], 6}, {NULL}};
const Segment SGI166638[] = {{&S60222[27], 6}, {NULL}};
const Segment SGI166639[] = {{&S49939[129], 3}, {NULL}};
const Segment SGI166640[] = {{&S60222[93], 9}, {NULL}};
const Segment SGI166641[] = {{&S60222[66], 6}, {NULL}};
const Segment SGI166642[] = {{&S49939[94], 6}, {NULL}};
const Segment SGI166643[] = {{&S46946[34], 4}, {NULL}};
const Segment SGI166644[] = {{&S46946[27], 6}, {NULL}};
const Segment SGI166645[] = {{&S46946[91], 9}, {NULL}};
const Segment SGI166646[] = {{&S46946[64], 6}, {NULL}};
const Segment SGI166647[] = {{&S46947[34], 12}, {NULL}};
const Segment SGI166648[] = {{&S46947[27], 6}, {NULL}};
const Segment SGI166649[] = {{&S46947[108], 9}, {NULL}};
const Segment SGI166650[] = {{&S46947[81], 6}, {NULL}};
const Segment SGI166651[] = {{&S46948[34], 7}, {NULL}};
const Segment SGI166652[] = {{&S46948[27], 6}, {NULL}};
const Segment SGI166653[] = {{&S46948[94], 9}, {NULL}};
const Segment SGI166654[] = {{&S46948[67], 6}, {NULL}};
const Segment SGI166655[] = {{&S46950[34], 22}, {NULL}};
const Segment SGI166656[] = {{&S46950[27], 6}, {NULL}};
const Segment SGI166657[] = {{&S46950[109], 9}, {NULL}};
const Segment SGI166658[] = {{&S46950[82], 6}, {NULL}};
const Segment SGI166659[] = {{&S46951[34], 13}, {NULL}};
const Segment SGI166660[] = {{&S46951[27], 6}, {NULL}};
const Segment SGI166661[] = {{&S46951[100], 9}, {NULL}};
const Segment SGI166662[] = {{&S46951[73], 6}, {NULL}};
const Segment SGI166663[] = {{&S46952[34], 14}, {NULL}};
const Segment SGI166664[] = {{&S46952[27], 6}, {NULL}};
const Segment SGI166665[] = {{&S46952[101], 9}, {NULL}};
const Segment SGI166666[] = {{&S46952[74], 6}, {NULL}};
const Segment SGI166667[] = {{&S46953[34], 9}, {NULL}};
const Segment SGI166668[] = {{&S46953[27], 6}, {NULL}};
const Segment SGI166669[] = {{&S46953[96], 9}, {NULL}};
const Segment SGI166670[] = {{&S46953[69], 6}, {NULL}};
const Segment SGI166671[] = {{&S46954[34], 17}, {NULL}};
const Segment SGI166672[] = {{&S46954[27], 6}, {NULL}};
const Segment SGI166673[] = {{&S46954[104], 9}, {NULL}};
const Segment SGI166674[] = {{&S46954[77], 6}, {NULL}};
const Segment SGI166675[] = {{&S46956[34], 6}, {NULL}};
const Segment SGI166676[] = {{&S46956[27], 6}, {NULL}};
const Segment SGI166677[] = {{&S46956[93], 9}, {NULL}};
const Segment SGI166678[] = {{&S46956[66], 6}, {NULL}};
const Segment SGI166679[] = {{&S46958[34], 9}, {NULL}};
const Segment SGI166680[] = {{&S46958[27], 6}, {NULL}};
const Segment SGI166681[] = {{&S46958[96], 9}, {NULL}};
const Segment SGI166682[] = {{&S46958[69], 6}, {NULL}};
const Segment SGI166683[] = {{&S46959[34], 7}, {NULL}};
const Segment SGI166684[] = {{&S46959[27], 6}, {NULL}};
const Segment SGI166685[] = {{&S46959[94], 9}, {NULL}};
const Segment SGI166686[] = {{&S46959[67], 6}, {NULL}};
const Segment SGI166687[] = {{&S46960[34], 6}, {NULL}};
const Segment SGI166688[] = {{&S46960[27], 6}, {NULL}};
const Segment SGI166689[] = {{&S46960[93], 9}, {NULL}};
const Segment SGI166690[] = {{&S46960[66], 6}, {NULL}};
const Segment SGI166691[] = {{&S46961[34], 7}, {NULL}};
const Segment SGI166692[] = {{&S46961[27], 6}, {NULL}};
const Segment SGI166693[] = {{&S46961[94], 9}, {NULL}};
const Segment SGI166694[] = {{&S37233[44], 10}, {NULL}};
const Segment SGI166695[] = {{&S46961[67], 6}, {NULL}};
const Segment SGI166696[] = {{&S49966[44], 10}, {NULL}};
const Segment SGI166697[] = {{&S37233[44], 21}, {NULL}};
const Segment SGI166698[] = {{&S49966[44], 22}, {NULL}};
const Segment SGI166699[] = {{&S37233[30], 6}, {NULL}};
const Segment SGI166700[] = {{&S49966[30], 6}, {NULL}};
const Segment SGI166701[] = {{&S49966[115], 3}, {NULL}};
const Segment SGI166702[] = {{&S37233[132], 3}, {NULL}};
const Segment SGI166703[] = {{&S49966[80], 6}, {NULL}};
const Segment SGI166704[] = {{&S37233[97], 6}, {NULL}};
const Segment SGI166705[] = {{&S37233[67], 8}, {NULL}};
const Segment SGI166706[] = {{&S49967[44], 11}, {NULL}};
const Segment SGI166707[] = {{&S46963[34], 8}, {NULL}};
const Segment SGI166708[] = {{&S49967[44], 23}, {NULL}};
const Segment SGI166709[] = {{&S37234[44], 18}, {NULL}};
const Segment SGI166710[] = {{&S49967[30], 6}, {NULL}};
const Segment SGI166711[] = {{&S46963[27], 6}, {NULL}};
const Segment SGI166712[] = {{&S37234[44], 29}, {NULL}};
const Segment SGI166713[] = {{&S37234[30], 6}, {NULL}};
const Segment SGI166714[] = {{&S49967[116], 3}, {NULL}};
const Segment SGI166715[] = {{&S46963[95], 9}, {NULL}};
const Segment SGI166716[] = {{&S49967[81], 6}, {NULL}};
const Segment SGI166717[] = {{&S37234[140], 3}, {NULL}};
const Segment SGI166718[] = {{&S46963[68], 6}, {NULL}};
const Segment SGI166719[] = {{&S37234[105], 6}, {NULL}};
const Segment SGI166720[] = {{&S37234[75], 8}, {NULL}};
const Segment SGI166721[] = {{&S52346[44], 28}, {NULL}};
const Segment SGI166722[] = {{&S52346[44], 40}, {NULL}};
const Segment SGI166723[] = {{&S52346[30], 6}, {NULL}};
const Segment SGI166724[] = {{&S52346[149], 3}, {NULL}};
const Segment SGI166725[] = {{&S49968[38], 10}, {NULL}};
const Segment SGI166726[] = {{&S49968[31], 6}, {NULL}};
const Segment SGI166727[] = {{&S52346[114], 6}, {NULL}};
const Segment SGI166728[] = {{&S46964[34], 10}, {NULL}};
const Segment SGI166729[] = {{&S46964[27], 6}, {NULL}};
const Segment SGI166730[] = {{&S52346[86], 8}, {NULL}};
const Segment SGI166731[] = {{&S46964[97], 9}, {NULL}};
const Segment SGI166732[] = {{&S46964[70], 6}, {NULL}};
const Segment SGI166733[] = {{&S37236[44], 11}, {NULL}};
const Segment SGI166734[] = {{&S52347[44], 28}, {NULL}};
const Segment SGI166735[] = {{&S37236[44], 22}, {NULL}};
const Segment SGI166736[] = {{&S49969[44], 11}, {NULL}};
const Segment SGI166737[] = {{&S37236[30], 6}, {NULL}};
const Segment SGI166738[] = {{&S52347[44], 40}, {NULL}};
const Segment SGI166739[] = {{&S49969[44], 23}, {NULL}};
const Segment SGI166740[] = {{&S52347[30], 6}, {NULL}};
const Segment SGI166741[] = {{&S37236[133], 3}, {NULL}};
const Segment SGI166742[] = {{&S46965[34], 23}, {NULL}};
const Segment SGI166743[] = {{&S37236[98], 6}, {NULL}};
const Segment SGI166744[] = {{&S46965[27], 6}, {NULL}};
const Segment SGI166745[] = {{&S52347[149], 3}, {NULL}};
const Segment SGI166746[] = {{&S37236[68], 8}, {NULL}};
const Segment SGI166747[] = {{&S52347[114], 6}, {NULL}};
const Segment SGI166748[] = {{&S46965[110], 9}, {NULL}};
const Segment SGI166749[] = {{&S52347[86], 8}, {NULL}};
const Segment SGI166750[] = {{&S46965[83], 6}, {NULL}};
const Segment SGI166751[] = {{&S60259[44], 6}, {NULL}};
const Segment SGI166752[] = {{&S60259[44], 18}, {NULL}};
const Segment SGI166753[] = {{&S60259[30], 6}, {NULL}};
const Segment SGI166754[] = {{&S60259[31], 3}, {NULL}};
const Segment SGI166755[] = {{&S60259[76], 6}, {NULL}};
const Segment SGI166756[] = {{&S37237[44], 10}, {NULL}};
const Segment SGI166757[] = {{&S37237[44], 21}, {NULL}};
const Segment SGI166758[] = {{&S37237[30], 6}, {NULL}};
const Segment SGI166759[] = {{&S37237[132], 3}, {NULL}};
const Segment SGI166760[] = {{&S37237[97], 6}, {NULL}};
const Segment SGI166761[] = {{&S37237[67], 8}, {NULL}};
const Segment SGI166762[] = {{&S37238[44], 11}, {NULL}};
const Segment SGI166763[] = {{&S46968[34], 17}, {NULL}};
const Segment SGI166764[] = {{&S37238[44], 22}, {NULL}};
const Segment SGI166765[] = {{&S46968[27], 6}, {NULL}};
const Segment SGI166766[] = {{&S37238[30], 6}, {NULL}};
const Segment SGI166767[] = {{&S37238[133], 3}, {NULL}};
const Segment SGI166768[] = {{&S46968[112], 9}, {NULL}};
const Segment SGI166769[] = {{&S37238[98], 6}, {NULL}};
const Segment SGI166770[] = {{&S46968[85], 6}, {NULL}};
const Segment SGI166771[] = {{&S37238[68], 8}, {NULL}};
const Segment SGI166772[] = {{&S60263[34], 14}, {NULL}};
const Segment SGI166773[] = {{&S60263[27], 6}, {NULL}};
const Segment SGI166774[] = {{&S37239[44], 10}, {NULL}};
const Segment SGI166775[] = {{&S60263[101], 9}, {NULL}};
const Segment SGI166776[] = {{&S37239[44], 22}, {NULL}};
const Segment SGI166777[] = {{&S60263[74], 6}, {NULL}};
const Segment SGI166778[] = {{&S37239[30], 6}, {NULL}};
const Segment SGI166779[] = {{&S37239[133], 3}, {NULL}};
const Segment SGI166780[] = {{&S37239[98], 6}, {NULL}};
const Segment SGI166781[] = {{&S46970[34], 8}, {NULL}};
const Segment SGI166782[] = {{&S46970[27], 6}, {NULL}};
const Segment SGI166783[] = {{&S37239[68], 8}, {NULL}};
const Segment SGI166784[] = {{&S46970[95], 9}, {NULL}};
const Segment SGI166785[] = {{&S46970[68], 6}, {NULL}};
const Segment SGI166786[] = {{&S37240[44], 18}, {NULL}};
const Segment SGI166787[] = {{&S37240[44], 30}, {NULL}};
const Segment SGI166788[] = {{&S37240[30], 6}, {NULL}};
const Segment SGI166789[] = {{&S46971[34], 10}, {NULL}};
const Segment SGI166790[] = {{&S37240[143], 3}, {NULL}};
const Segment SGI166791[] = {{&S46971[27], 6}, {NULL}};
const Segment SGI166792[] = {{&S37240[108], 6}, {NULL}};
const Segment SGI166793[] = {{&S37240[76], 8}, {NULL}};
const Segment SGI166794[] = {{&S46971[97], 9}, {NULL}};
const Segment SGI166795[] = {{&S46971[70], 6}, {NULL}};
const Segment SGI166796[] = {{&S37241[44], 10}, {NULL}};
const Segment SGI166797[] = {{&S37241[44], 20}, {NULL}};
const Segment SGI166798[] = {{&S46972[34], 15}, {NULL}};
const Segment SGI166799[] = {{&S37241[30], 6}, {NULL}};
const Segment SGI166800[] = {{&S46972[27], 6}, {NULL}};
const Segment SGI166801[] = {{&S37241[131], 3}, {NULL}};
const Segment SGI166802[] = {{&S37241[96], 6}, {NULL}};
const Segment SGI166803[] = {{&S46972[102], 9}, {NULL}};
const Segment SGI166804[] = {{&S46972[75], 6}, {NULL}};
const Segment SGI166805[] = {{&S37241[66], 8}, {NULL}};
const Segment SGI166806[] = {{&S37242[44], 18}, {NULL}};
const Segment SGI166807[] = {{&S37242[44], 28}, {NULL}};
const Segment SGI166808[] = {{&S37242[30], 6}, {NULL}};
const Segment SGI166809[] = {{&S37242[139], 3}, {NULL}};
const Segment SGI166810[] = {{&S37242[104], 6}, {NULL}};
const Segment SGI166811[] = {{&S37242[74], 8}, {NULL}};
const Segment SGI166812[] = {{&S46975[34], 13}, {NULL}};
const Segment SGI166813[] = {{&S46975[27], 6}, {NULL}};
const Segment SGI166814[] = {{&S37243[44], 10}, {NULL}};
const Segment SGI166815[] = {{&S37243[44], 22}, {NULL}};
const Segment SGI166816[] = {{&S37243[30], 6}, {NULL}};
const Segment SGI166817[] = {{&S46975[100], 9}, {NULL}};
const Segment SGI166818[] = {{&S46975[73], 6}, {NULL}};
const Segment SGI166819[] = {{&S37243[115], 3}, {NULL}};
const Segment SGI166820[] = {{&S37243[80], 6}, {NULL}};
const Segment SGI166821[] = {{&S37243[140], 8}, {NULL}};
const Segment SGI166822[] = {{&S46976[34], 8}, {NULL}};
const Segment SGI166823[] = {{&S46976[27], 6}, {NULL}};
const Segment SGI166824[] = {{&S46976[95], 9}, {NULL}};
const Segment SGI166825[] = {{&S46976[68], 6}, {NULL}};
const Segment SGI166826[] = {{&S37244[44], 11}, {NULL}};
const Segment SGI166827[] = {{&S37244[44], 23}, {NULL}};
const Segment SGI166828[] = {{&S37244[30], 6}, {NULL}};
const Segment SGI166829[] = {{&S37244[116], 3}, {NULL}};
const Segment SGI166830[] = {{&S37244[81], 6}, {NULL}};
const Segment SGI166831[] = {{&S37244[141], 8}, {NULL}};
const Segment SGI166832[] = {{&S46977[34], 9}, {NULL}};
const Segment SGI166833[] = {{&S46977[27], 6}, {NULL}};
const Segment SGI166834[] = {{&S60276[34], 10}, {NULL}};
const Segment SGI166835[] = {{&S60276[27], 6}, {NULL}};
const Segment SGI166836[] = {{&S46977[96], 9}, {NULL}};
const Segment SGI166837[] = {{&S46977[69], 6}, {NULL}};
const Segment SGI166838[] = {{&S60276[97], 9}, {NULL}};
const Segment SGI166839[] = {{&S60276[70], 6}, {NULL}};
const Segment SGI166840[] = {{&S37245[44], 11}, {NULL}};
const Segment SGI166841[] = {{&S37245[44], 23}, {NULL}};
const Segment SGI166842[] = {{&S37245[30], 6}, {NULL}};
const Segment SGI166843[] = {{&S46978[34], 6}, {NULL}};
const Segment SGI166844[] = {{&S37245[116], 3}, {NULL}};
const Segment SGI166845[] = {{&S46978[27], 6}, {NULL}};
const Segment SGI166846[] = {{&S37245[81], 6}, {NULL}};
const Segment SGI166847[] = {{&S37245[141], 8}, {NULL}};
const Segment SGI166848[] = {{&S46978[93], 9}, {NULL}};
const Segment SGI166849[] = {{&S46978[66], 6}, {NULL}};
const Segment SGI166850[] = {{&S37246[44], 10}, {NULL}};
const Segment SGI166851[] = {{&S37246[44], 20}, {NULL}};
const Segment SGI166852[] = {{&S37246[30], 6}, {NULL}};
const Segment SGI166853[] = {{&S37246[113], 3}, {NULL}};
const Segment SGI166854[] = {{&S37246[78], 6}, {NULL}};
const Segment SGI166855[] = {{&S37246[138], 8}, {NULL}};
const Segment SGI166856[] = {{&S46981[34], 11}, {NULL}};
const Segment SGI166857[] = {{&S46981[27], 6}, {NULL}};
const Segment SGI166858[] = {{&S46981[98], 9}, {NULL}};
const Segment SGI166859[] = {{&S46981[71], 6}, {NULL}};
const Segment SGI166860[] = {{&S46983[34], 4}, {NULL}};
const Segment SGI166861[] = {{&S46983[27], 6}, {NULL}};
const Segment SGI166862[] = {{&S46983[91], 9}, {NULL}};
const Segment SGI166863[] = {{&S46983[64], 6}, {NULL}};
const Segment SGI166864[] = {{&S46986[34], 7}, {NULL}};
const Segment SGI166865[] = {{&S46986[27], 6}, {NULL}};
const Segment SGI166866[] = {{&S46986[94], 9}, {NULL}};
const Segment SGI166867[] = {{&S46986[67], 6}, {NULL}};
const Segment SGI166868[] = {{&S46987[34], 6}, {NULL}};
const Segment SGI166869[] = {{&S46987[27], 6}, {NULL}};
const Segment SGI166870[] = {{&S46987[93], 9}, {NULL}};
const Segment SGI166871[] = {{&S46987[66], 6}, {NULL}};
const Segment SGI166872[] = {{&S46989[34], 19}, {NULL}};
const Segment SGI166873[] = {{&S46989[27], 6}, {NULL}};
const Segment SGI166874[] = {{&S46989[106], 9}, {NULL}};
const Segment SGI166875[] = {{&S46989[79], 6}, {NULL}};
const Segment SGI166876[] = {{&S46992[34], 7}, {NULL}};
const Segment SGI166877[] = {{&S46992[27], 6}, {NULL}};
const Segment SGI166878[] = {{&S46992[94], 9}, {NULL}};
const Segment SGI166879[] = {{&S46992[67], 6}, {NULL}};
const Segment SGI166880[] = {{&S46994[34], 9}, {NULL}};
const Segment SGI166881[] = {{&S46994[27], 6}, {NULL}};
const Segment SGI166882[] = {{&S46994[96], 9}, {NULL}};
const Segment SGI166883[] = {{&S46994[69], 6}, {NULL}};
const Segment SGI166884[] = {{&S46995[34], 6}, {NULL}};
const Segment SGI166885[] = {{&S46995[27], 6}, {NULL}};
const Segment SGI166886[] = {{&S46995[93], 9}, {NULL}};
const Segment SGI166887[] = {{&S46995[66], 6}, {NULL}};
const Segment SGI166888[] = {{&S46997[34], 11}, {NULL}};
const Segment SGI166889[] = {{&S46997[27], 6}, {NULL}};
const Segment SGI166890[] = {{&S60308[44], 6}, {NULL}};
const Segment SGI166891[] = {{&S60308[44], 18}, {NULL}};
const Segment SGI166892[] = {{&S60308[30], 6}, {NULL}};
const Segment SGI166893[] = {{&S46997[98], 9}, {NULL}};
const Segment SGI166894[] = {{&S46997[71], 6}, {NULL}};
const Segment SGI166895[] = {{&S60308[31], 3}, {NULL}};
const Segment SGI166896[] = {{&S60308[76], 6}, {NULL}};
const Segment SGI166897[] = {{&S49969[30], 6}, {NULL}};
const Segment SGI166898[] = {{&S49969[116], 3}, {NULL}};
const Segment SGI166899[] = {{&S49969[81], 6}, {NULL}};
const Segment SGI166900[] = {{&S49970[38], 11}, {NULL}};
const Segment SGI166901[] = {{&S52410[34], 5}, {NULL}};
const Segment SGI166902[] = {{&S49970[31], 6}, {NULL}};
const Segment SGI166903[] = {{&S52410[27], 6}, {NULL}};
const Segment SGI166904[] = {{&S52410[92], 9}, {NULL}};
const Segment SGI166905[] = {{&S52410[65], 6}, {NULL}};
const Segment SGI166906[] = {{&S49971[44], 11}, {NULL}};
const Segment SGI166907[] = {{&S49971[44], 23}, {NULL}};
const Segment SGI166908[] = {{&S49971[30], 6}, {NULL}};
const Segment SGI166909[] = {{&S52411[34], 5}, {NULL}};
const Segment SGI166910[] = {{&S52411[27], 6}, {NULL}};
const Segment SGI166911[] = {{&S49971[116], 3}, {NULL}};
const Segment SGI166912[] = {{&S49971[81], 6}, {NULL}};
const Segment SGI166913[] = {{&S52411[92], 9}, {NULL}};
const Segment SGI166914[] = {{&S52411[65], 6}, {NULL}};
const Segment SGI166915[] = {{&S49972[38], 11}, {NULL}};
const Segment SGI166916[] = {{&S49972[31], 6}, {NULL}};
const Segment SGI166917[] = {{&S52412[34], 12}, {NULL}};
const Segment SGI166918[] = {{&S52412[27], 6}, {NULL}};
const Segment SGI166919[] = {{&S52412[99], 9}, {NULL}};
const Segment SGI166920[] = {{&S52412[72], 6}, {NULL}};
const Segment SGI166921[] = {{&S52413[34], 12}, {NULL}};
const Segment SGI166922[] = {{&S52413[27], 6}, {NULL}};
const Segment SGI166923[] = {{&S52413[99], 9}, {NULL}};
const Segment SGI166924[] = {{&S52413[72], 6}, {NULL}};
const Segment SGI166925[] = {{&S60324[34], 7}, {NULL}};
const Segment SGI166926[] = {{&S60324[27], 6}, {NULL}};
const Segment SGI166927[] = {{&S52414[34], 14}, {NULL}};
const Segment SGI166928[] = {{&S52414[27], 6}, {NULL}};
const Segment SGI166929[] = {{&S60324[98], 9}, {NULL}};
const Segment SGI166930[] = {{&S60324[71], 6}, {NULL}};
const Segment SGI166931[] = {{&S57244[76], 5}, {NULL}};
const Segment SGI166932[] = {{&S52414[101], 9}, {NULL}};
const Segment SGI166933[] = {{&S52414[74], 6}, {NULL}};
const Segment SGI166934[] = {{&S60325[34], 7}, {NULL}};
const Segment SGI166935[] = {{&S60325[27], 6}, {NULL}};
const Segment SGI166936[] = {{&S60325[98], 9}, {NULL}};
const Segment SGI166937[] = {{&S60325[71], 6}, {NULL}};
const Segment SGI166938[] = {{&S52429[34], 7}, {NULL}};
const Segment SGI166939[] = {{&S52429[27], 6}, {NULL}};
const Segment SGI166940[] = {{&S52429[94], 9}, {NULL}};
const Segment SGI166941[] = {{&S52429[67], 6}, {NULL}};
const Segment SGI166942[] = {{&S52430[34], 9}, {NULL}};
const Segment SGI166943[] = {{&S52430[27], 6}, {NULL}};
const Segment SGI166944[] = {{&S52430[96], 9}, {NULL}};
const Segment SGI166945[] = {{&S52430[69], 6}, {NULL}};
const Segment SGI166946[] = {{&S60344[34], 12}, {NULL}};
const Segment SGI166947[] = {{&S60344[27], 6}, {NULL}};
const Segment SGI166948[] = {{&S60344[99], 9}, {NULL}};
const Segment SGI166949[] = {{&S60344[72], 6}, {NULL}};
const Segment SGI166950[] = {{&S60345[34], 12}, {NULL}};
const Segment SGI166951[] = {{&S60345[27], 6}, {NULL}};
const Segment SGI166952[] = {{&S60345[99], 9}, {NULL}};
const Segment SGI166953[] = {{&S60345[72], 6}, {NULL}};
const Segment SGI166954[] = {{&S50008[34], 10}, {NULL}};
const Segment SGI166955[] = {{&S50008[27], 6}, {NULL}};
const Segment SGI166956[] = {{&S50008[97], 9}, {NULL}};
const Segment SGI166957[] = {{&S50008[70], 6}, {NULL}};
const Segment SGI166958[] = {{&S60381[44], 7}, {NULL}};
const Segment SGI166959[] = {{&S60381[44], 19}, {NULL}};
const Segment SGI166960[] = {{&S60381[30], 6}, {NULL}};
const Segment SGI166961[] = {{&S60381[31], 3}, {NULL}};
const Segment SGI166962[] = {{&S60381[77], 6}, {NULL}};
const Segment SGI166963[] = {{&S50058[34], 7}, {NULL}};
const Segment SGI166964[] = {{&S50058[27], 6}, {NULL}};
const Segment SGI166965[] = {{&S50058[109], 9}, {NULL}};
const Segment SGI166966[] = {{&S50058[82], 6}, {NULL}};
const Segment SGI166967[] = {{&S52505[34], 10}, {NULL}};
const Segment SGI166968[] = {{&S52505[27], 6}, {NULL}};
const Segment SGI166969[] = {{&S52505[97], 9}, {NULL}};
const Segment SGI166970[] = {{&S52505[70], 6}, {NULL}};
const Segment SGI166971[] = {{&S60416[30], 6}, {NULL}};
const Segment SGI166972[] = {{&S60416[31], 3}, {NULL}};
const Segment SGI166973[] = {{&S60416[81], 6}, {NULL}};
const Segment SGI166974[] = {{&S60417[44], 13}, {NULL}};
const Segment SGI166975[] = {{&S60417[44], 25}, {NULL}};
const Segment SGI166976[] = {{&S60417[30], 6}, {NULL}};
const Segment SGI166977[] = {{&S60417[31], 3}, {NULL}};
const Segment SGI166978[] = {{&S60417[83], 6}, {NULL}};
const Segment SGI166979[] = {{&S60418[38], 13}, {NULL}};
const Segment SGI166980[] = {{&S60418[31], 6}, {NULL}};
const Segment SGI166981[] = {{&S52512[44], 8}, {NULL}};
const Segment SGI166982[] = {{&S52512[44], 20}, {NULL}};
const Segment SGI166983[] = {{&S52512[30], 6}, {NULL}};
const Segment SGI166984[] = {{&S52512[113], 3}, {NULL}};
const Segment SGI166985[] = {{&S60419[44], 13}, {NULL}};
const Segment SGI166986[] = {{&S52512[78], 6}, {NULL}};
const Segment SGI166987[] = {{&S60419[44], 25}, {NULL}};
const Segment SGI166988[] = {{&S60419[30], 6}, {NULL}};
const Segment SGI166989[] = {{&S60419[31], 3}, {NULL}};
const Segment SGI166990[] = {{&S60419[83], 6}, {NULL}};
const Segment SGI166991[] = {{&S52513[38], 8}, {NULL}};
const Segment SGI166992[] = {{&S52513[31], 6}, {NULL}};
const Segment SGI166993[] = {{&S52523[34], 11}, {NULL}};
const Segment SGI166994[] = {{&S52523[27], 6}, {NULL}};
const Segment SGI166995[] = {{&S52523[98], 9}, {NULL}};
const Segment SGI166996[] = {{&S52523[71], 6}, {NULL}};
const Segment SGI166997[] = {{&S50087[44], 9}, {NULL}};
const Segment SGI166998[] = {{&S50087[44], 21}, {NULL}};
const Segment SGI166999[] = {{&S50087[30], 6}, {NULL}};
const Segment SGI167000[] = {{&S50087[114], 3}, {NULL}};
const Segment SGI167001[] = {{&S50087[79], 6}, {NULL}};
const Segment SGI167002[] = {{&S50088[38], 9}, {NULL}};
const Segment SGI167003[] = {{&S50088[31], 6}, {NULL}};
const Segment SGI167004[] = {{&S60441[44], 7}, {NULL}};
const Segment SGI167005[] = {{&S60441[44], 35}, {NULL}};
const Segment SGI167006[] = {{&S60441[30], 6}, {NULL}};
const Segment SGI167007[] = {{&S60441[31], 3}, {NULL}};
const Segment SGI167008[] = {{&S60441[93], 6}, {NULL}};
const Segment SGI167009[] = {{&S60442[38], 7}, {NULL}};
const Segment SGI167010[] = {{&S60442[31], 6}, {NULL}};
const Segment SGI167011[] = {{&S60443[37], 15}, {NULL}};
const Segment SGI167012[] = {{&S60444[44], 7}, {NULL}};
const Segment SGI167013[] = {{&S60444[44], 28}, {NULL}};
const Segment SGI167014[] = {{&S60444[30], 6}, {NULL}};
const Segment SGI167015[] = {{&S60444[31], 3}, {NULL}};
const Segment SGI167016[] = {{&S60444[86], 6}, {NULL}};
const Segment SGI167017[] = {{&S60445[38], 7}, {NULL}};
const Segment SGI167018[] = {{&S60445[31], 6}, {NULL}};
const Segment SGI167019[] = {{&S60446[44], 10}, {NULL}};
const Segment SGI167020[] = {{&S60446[44], 22}, {NULL}};
const Segment SGI167021[] = {{&S60446[30], 6}, {NULL}};
const Segment SGI167022[] = {{&S60446[80], 6}, {NULL}};
const Segment SGI167023[] = {{&S60446[107], 13}, {NULL}};
const Segment SGI167024[] = {{&S60447[44], 11}, {NULL}};
const Segment SGI167025[] = {{&S60447[44], 23}, {NULL}};
const Segment SGI167026[] = {{&S60447[30], 6}, {NULL}};
const Segment SGI167027[] = {{&S60447[81], 6}, {NULL}};
const Segment SGI167028[] = {{&S60447[108], 13}, {NULL}};
const Segment SGI167029[] = {{&S50105[34], 10}, {NULL}};
const Segment SGI167030[] = {{&S50105[27], 6}, {NULL}};
const Segment SGI167031[] = {{&S50105[97], 9}, {NULL}};
const Segment SGI167032[] = {{&S50105[70], 6}, {NULL}};
const Segment SGI167033[] = {{&S50110[34], 6}, {NULL}};
const Segment SGI167034[] = {{&S50110[27], 6}, {NULL}};
const Segment SGI167035[] = {{&S50110[93], 9}, {NULL}};
const Segment SGI167036[] = {{&S50110[66], 6}, {NULL}};
const Segment SGI167037[] = {{&S50111[34], 6}, {NULL}};
const Segment SGI167038[] = {{&S50111[27], 6}, {NULL}};
const Segment SGI167039[] = {{&S50111[93], 9}, {NULL}};
const Segment SGI167040[] = {{&S50111[66], 6}, {NULL}};
const Segment SGI167041[] = {{&S60458[44], 9}, {NULL}};
const Segment SGI167042[] = {{&S50112[34], 10}, {NULL}};
const Segment SGI167043[] = {{&S60458[44], 21}, {NULL}};
const Segment SGI167044[] = {{&S60458[30], 6}, {NULL}};
const Segment SGI167045[] = {{&S50112[27], 6}, {NULL}};
const Segment SGI167046[] = {{&S60458[31], 3}, {NULL}};
const Segment SGI167047[] = {{&S50112[97], 9}, {NULL}};
const Segment SGI167048[] = {{&S60458[79], 6}, {NULL}};
const Segment SGI167049[] = {{&S50112[70], 6}, {NULL}};
const Segment SGI167050[] = {{&S60459[44], 9}, {NULL}};
const Segment SGI167051[] = {{&S60459[44], 43}, {NULL}};
const Segment SGI167052[] = {{&S60459[30], 6}, {NULL}};
const Segment SGI167053[] = {{&S60459[31], 3}, {NULL}};
const Segment SGI167054[] = {{&S60459[101], 6}, {NULL}};
const Segment SGI167055[] = {{&S50114[34], 10}, {NULL}};
const Segment SGI167056[] = {{&S50114[27], 6}, {NULL}};
const Segment SGI167057[] = {{&S50114[105], 9}, {NULL}};
const Segment SGI167058[] = {{&S50114[78], 6}, {NULL}};
const Segment SGI167059[] = {{&S60460[44], 9}, {NULL}};
const Segment SGI167060[] = {{&S60460[44], 21}, {NULL}};
const Segment SGI167061[] = {{&S60460[30], 6}, {NULL}};
const Segment SGI167062[] = {{&S60460[31], 3}, {NULL}};
const Segment SGI167063[] = {{&S52567[34], 10}, {NULL}};
const Segment SGI167064[] = {{&S60460[79], 6}, {NULL}};
const Segment SGI167065[] = {{&S50115[34], 10}, {NULL}};
const Segment SGI167066[] = {{&S52567[27], 6}, {NULL}};
const Segment SGI167067[] = {{&S50115[27], 6}, {NULL}};
const Segment SGI167068[] = {{&S52567[97], 9}, {NULL}};
const Segment SGI167069[] = {{&S52567[70], 6}, {NULL}};
const Segment SGI167070[] = {{&S50115[97], 9}, {NULL}};
const Segment SGI167071[] = {{&S50115[70], 6}, {NULL}};
const Segment SGI167072[] = {{&S60461[44], 9}, {NULL}};
const Segment SGI167073[] = {{&S60461[44], 43}, {NULL}};
const Segment SGI167074[] = {{&S60461[30], 6}, {NULL}};
const Segment SGI167075[] = {{&S60461[31], 3}, {NULL}};
const Segment SGI167076[] = {{&S60461[101], 6}, {NULL}};
const Segment SGI167077[] = {{&S60462[44], 4}, {NULL}};
const Segment SGI167078[] = {{&S60462[44], 16}, {NULL}};
const Segment SGI167079[] = {{&S60462[30], 6}, {NULL}};
const Segment SGI167080[] = {{&S50118[34], 6}, {NULL}};
const Segment SGI167081[] = {{&S50118[27], 6}, {NULL}};
const Segment SGI167082[] = {{&S60462[31], 3}, {NULL}};
const Segment SGI167083[] = {{&S60462[74], 6}, {NULL}};
const Segment SGI167084[] = {{&S50118[93], 9}, {NULL}};
const Segment SGI167085[] = {{&S50118[66], 6}, {NULL}};
const Segment SGI167086[] = {{&S60463[44], 10}, {NULL}};
const Segment SGI167087[] = {{&S50119[34], 6}, {NULL}};
const Segment SGI167088[] = {{&S60463[44], 31}, {NULL}};
const Segment SGI167089[] = {{&S50119[27], 6}, {NULL}};
const Segment SGI167090[] = {{&S60463[30], 6}, {NULL}};
const Segment SGI167091[] = {{&S60463[31], 3}, {NULL}};
const Segment SGI167092[] = {{&S50119[93], 9}, {NULL}};
const Segment SGI167093[] = {{&S50119[66], 6}, {NULL}};
const Segment SGI167094[] = {{&S60463[89], 6}, {NULL}};
const Segment SGI167095[] = {{&S60464[44], 13}, {NULL}};
const Segment SGI167096[] = {{&S60464[44], 25}, {NULL}};
const Segment SGI167097[] = {{&S60464[30], 6}, {NULL}};
const Segment SGI167098[] = {{&S60464[31], 3}, {NULL}};
const Segment SGI167099[] = {{&S60464[83], 6}, {NULL}};
const Segment SGI167100[] = {{&S50122[34], 5}, {NULL}};
const Segment SGI167101[] = {{&S50122[27], 6}, {NULL}};
const Segment SGI167102[] = {{&S60465[38], 4}, {NULL}};
const Segment SGI167103[] = {{&S60465[31], 6}, {NULL}};
const Segment SGI167104[] = {{&S50122[92], 9}, {NULL}};
const Segment SGI167105[] = {{&S50122[65], 6}, {NULL}};
const Segment SGI167106[] = {{&S50123[34], 6}, {NULL}};
const Segment SGI167107[] = {{&S50123[27], 6}, {NULL}};
const Segment SGI167108[] = {{&S50123[93], 9}, {NULL}};
const Segment SGI167109[] = {{&S50123[66], 6}, {NULL}};
const Segment SGI167110[] = {{&S50125[34], 6}, {NULL}};
const Segment SGI167111[] = {{&S50125[27], 6}, {NULL}};
const Segment SGI167112[] = {{&S50125[93], 9}, {NULL}};
const Segment SGI167113[] = {{&S50125[66], 6}, {NULL}};
const Segment SGI167114[] = {{&S50128[34], 11}, {NULL}};
const Segment SGI167115[] = {{&S50128[27], 6}, {NULL}};
const Segment SGI167116[] = {{&S50128[98], 9}, {NULL}};
const Segment SGI167117[] = {{&S50128[71], 6}, {NULL}};
const Segment SGI167118[] = {{&S50132[34], 10}, {NULL}};
const Segment SGI167119[] = {{&S50132[27], 6}, {NULL}};
const Segment SGI167120[] = {{&S50132[105], 9}, {NULL}};
const Segment SGI167121[] = {{&S50132[78], 6}, {NULL}};
const Segment SGI167122[] = {{&S50134[34], 9}, {NULL}};
const Segment SGI167123[] = {{&S50134[27], 6}, {NULL}};
const Segment SGI167124[] = {{&S50134[96], 9}, {NULL}};
const Segment SGI167125[] = {{&S50134[69], 6}, {NULL}};
const Segment SGI167126[] = {{&S50136[34], 8}, {NULL}};
const Segment SGI167127[] = {{&S50136[27], 6}, {NULL}};
const Segment SGI167128[] = {{&S50136[95], 9}, {NULL}};
const Segment SGI167129[] = {{&S50136[68], 6}, {NULL}};
const Segment SGI167130[] = {{&S52606[34], 12}, {NULL}};
const Segment SGI167131[] = {{&S52606[27], 6}, {NULL}};
const Segment SGI167132[] = {{&S52606[99], 9}, {NULL}};
const Segment SGI167133[] = {{&S52606[72], 6}, {NULL}};
const Segment SGI167134[] = {{&S50142[34], 6}, {NULL}};
const Segment SGI167135[] = {{&S50142[27], 6}, {NULL}};
const Segment SGI167136[] = {{&S52607[34], 12}, {NULL}};
const Segment SGI167137[] = {{&S52607[27], 6}, {NULL}};
const Segment SGI167138[] = {{&S50142[93], 9}, {NULL}};
const Segment SGI167139[] = {{&S50142[66], 6}, {NULL}};
const Segment SGI167140[] = {{&S52607[99], 9}, {NULL}};
const Segment SGI167141[] = {{&S52607[72], 6}, {NULL}};
const Segment SGI167142[] = {{&S50144[34], 6}, {NULL}};
const Segment SGI167143[] = {{&S50144[27], 6}, {NULL}};
const Segment SGI167144[] = {{&S50144[93], 9}, {NULL}};
const Segment SGI167145[] = {{&S50144[66], 6}, {NULL}};
const Segment SGI167146[] = {{&S50145[34], 6}, {NULL}};
const Segment SGI167147[] = {{&S50145[27], 6}, {NULL}};
const Segment SGI167148[] = {{&S60499[34], 10}, {NULL}};
const Segment SGI167149[] = {{&S60499[27], 6}, {NULL}};
const Segment SGI167150[] = {{&S50145[93], 9}, {NULL}};
const Segment SGI167151[] = {{&S50145[66], 6}, {NULL}};
const Segment SGI167152[] = {{&S60499[97], 9}, {NULL}};
const Segment SGI167153[] = {{&S60499[70], 6}, {NULL}};
const Segment SGI167154[] = {{&S50146[34], 6}, {NULL}};
const Segment SGI167155[] = {{&S50146[27], 6}, {NULL}};
const Segment SGI167156[] = {{&S50146[93], 9}, {NULL}};
const Segment SGI167157[] = {{&S50146[66], 6}, {NULL}};
const Segment SGI167158[] = {{&S50147[34], 10}, {NULL}};
const Segment SGI167159[] = {{&S50147[27], 6}, {NULL}};
const Segment SGI167160[] = {{&S50147[97], 9}, {NULL}};
const Segment SGI167161[] = {{&S50147[70], 6}, {NULL}};
const Segment SGI167162[] = {{&S50149[34], 10}, {NULL}};
const Segment SGI167163[] = {{&S50149[27], 6}, {NULL}};
const Segment SGI167164[] = {{&S50149[97], 9}, {NULL}};
const Segment SGI167165[] = {{&S50149[70], 6}, {NULL}};
const Segment SGI167166[] = {{&S50150[34], 7}, {NULL}};
const Segment SGI167167[] = {{&S50150[27], 6}, {NULL}};
const Segment SGI167168[] = {{&S50150[94], 9}, {NULL}};
const Segment SGI167169[] = {{&S50150[67], 6}, {NULL}};
const Segment SGI167170[] = {{&S50151[34], 7}, {NULL}};
const Segment SGI167171[] = {{&S50151[27], 6}, {NULL}};
const Segment SGI167172[] = {{&S50151[94], 9}, {NULL}};
const Segment SGI167173[] = {{&S50151[67], 6}, {NULL}};
const Segment SGI167174[] = {{&S50153[34], 8}, {NULL}};
const Segment SGI167175[] = {{&S50153[27], 6}, {NULL}};
const Segment SGI167176[] = {{&S50153[103], 9}, {NULL}};
const Segment SGI167177[] = {{&S50153[76], 6}, {NULL}};
const Segment SGI167178[] = {{&S60517[44], 25}, {NULL}};
const Segment SGI167179[] = {{&S60517[44], 37}, {NULL}};
const Segment SGI167180[] = {{&S60517[30], 6}, {NULL}};
const Segment SGI167181[] = {{&S60517[31], 3}, {NULL}};
const Segment SGI167182[] = {{&S50156[34], 10}, {NULL}};
const Segment SGI167183[] = {{&S60517[95], 6}, {NULL}};
const Segment SGI167184[] = {{&S50156[27], 6}, {NULL}};
const Segment SGI167185[] = {{&S37370[34], 9}, {NULL}};
const Segment SGI167186[] = {{&S37370[27], 6}, {NULL}};
const Segment SGI167187[] = {{&S50156[97], 9}, {NULL}};
const Segment SGI167188[] = {{&S50156[70], 6}, {NULL}};
const Segment SGI167189[] = {{&S37370[95], 9}, {NULL}};
const Segment SGI167190[] = {{&S37370[68], 6}, {NULL}};
const Segment SGI167191[] = {{&S60518[44], 25}, {NULL}};
const Segment SGI167192[] = {{&S60518[44], 37}, {NULL}};
const Segment SGI167193[] = {{&S60518[30], 6}, {NULL}};
const Segment SGI167194[] = {{&S60518[31], 3}, {NULL}};
const Segment SGI167195[] = {{&S60518[95], 6}, {NULL}};
const Segment SGI167196[] = {{&S37371[34], 9}, {NULL}};
const Segment SGI167197[] = {{&S37371[27], 6}, {NULL}};
const Segment SGI167198[] = {{&S37371[95], 9}, {NULL}};
const Segment SGI167199[] = {{&S37371[68], 6}, {NULL}};
const Segment SGI167200[] = {{&S60519[44], 25}, {NULL}};
const Segment SGI167201[] = {{&S60519[44], 37}, {NULL}};
const Segment SGI167202[] = {{&S60519[30], 6}, {NULL}};
const Segment SGI167203[] = {{&S50159[34], 6}, {NULL}};
const Segment SGI167204[] = {{&S60519[31], 3}, {NULL}};
const Segment SGI167205[] = {{&S50159[27], 6}, {NULL}};
const Segment SGI167206[] = {{&S60519[95], 6}, {NULL}};
const Segment SGI167207[] = {{&S50159[93], 9}, {NULL}};
const Segment SGI167208[] = {{&S50159[66], 6}, {NULL}};
const Segment SGI167209[] = {{&S60523[50], 11}, {NULL}};
const Segment SGI167210[] = {{&S60523[42], 7}, {NULL}};
const Segment SGI167211[] = {{&S50163[34], 16}, {NULL}};
const Segment SGI167212[] = {{&S50163[27], 6}, {NULL}};
const Segment SGI167213[] = {{&S60523[29], 3}, {NULL}};
const Segment SGI167214[] = {{&S60523[84], 9}, {NULL}};
const Segment SGI167215[] = {{&S50163[103], 9}, {NULL}};
const Segment SGI167216[] = {{&S50163[76], 6}, {NULL}};
const Segment SGI167217[] = {{&S60524[52], 11}, {NULL}};
const Segment SGI167218[] = {{&S60524[44], 7}, {NULL}};
const Segment SGI167219[] = {{&S60524[30], 6}, {NULL}};
const Segment SGI167220[] = {{&S60524[31], 3}, {NULL}};
const Segment SGI167221[] = {{&S60524[86], 9}, {NULL}};
const Segment SGI167222[] = {{&S50165[34], 10}, {NULL}};
const Segment SGI167223[] = {{&S50165[27], 6}, {NULL}};
const Segment SGI167224[] = {{&S60525[44], 11}, {NULL}};
const Segment SGI167225[] = {{&S50165[97], 9}, {NULL}};
const Segment SGI167226[] = {{&S60525[44], 23}, {NULL}};
const Segment SGI167227[] = {{&S50165[70], 6}, {NULL}};
const Segment SGI167228[] = {{&S60525[30], 6}, {NULL}};
const Segment SGI167229[] = {{&S60525[31], 3}, {NULL}};
const Segment SGI167230[] = {{&S60525[81], 6}, {NULL}};
const Segment SGI167231[] = {{&S50166[34], 16}, {NULL}};
const Segment SGI167232[] = {{&S50166[27], 6}, {NULL}};
const Segment SGI167233[] = {{&S50166[110], 9}, {NULL}};
const Segment SGI167234[] = {{&S50166[83], 6}, {NULL}};
const Segment SGI167235[] = {{&S50169[34], 7}, {NULL}};
const Segment SGI167236[] = {{&S50169[27], 6}, {NULL}};
const Segment SGI167237[] = {{&S50169[94], 9}, {NULL}};
const Segment SGI167238[] = {{&S50169[67], 6}, {NULL}};
const Segment SGI167239[] = {{&S50170[34], 16}, {NULL}};
const Segment SGI167240[] = {{&S50170[27], 6}, {NULL}};
const Segment SGI167241[] = {{&S50170[103], 9}, {NULL}};
const Segment SGI167242[] = {{&S50170[76], 6}, {NULL}};
const Segment SGI167243[] = {{&S50171[34], 18}, {NULL}};
const Segment SGI167244[] = {{&S50171[27], 6}, {NULL}};
const Segment SGI167245[] = {{&S50171[105], 9}, {NULL}};
const Segment SGI167246[] = {{&S50171[78], 6}, {NULL}};
const Segment SGI167247[] = {{&S50173[34], 17}, {NULL}};
const Segment SGI167248[] = {{&S50173[27], 6}, {NULL}};
const Segment SGI167249[] = {{&S50173[112], 9}, {NULL}};
const Segment SGI167250[] = {{&S50173[85], 6}, {NULL}};
const Segment SGI167251[] = {{&S50174[34], 6}, {NULL}};
const Segment SGI167252[] = {{&S50174[27], 6}, {NULL}};
const Segment SGI167253[] = {{&S50174[93], 9}, {NULL}};
const Segment SGI167254[] = {{&S50174[66], 6}, {NULL}};
const Segment SGI167255[] = {{&S50175[34], 18}, {NULL}};
const Segment SGI167256[] = {{&S50175[27], 6}, {NULL}};
const Segment SGI167257[] = {{&S50175[105], 9}, {NULL}};
const Segment SGI167258[] = {{&S50175[78], 6}, {NULL}};
const Segment SGI167259[] = {{&S60544[44], 7}, {NULL}};
const Segment SGI167260[] = {{&S60544[44], 19}, {NULL}};
const Segment SGI167261[] = {{&S60544[30], 6}, {NULL}};
const Segment SGI167262[] = {{&S60544[31], 3}, {NULL}};
const Segment SGI167263[] = {{&S60544[77], 6}, {NULL}};
const Segment SGI167264[] = {{&S50176[34], 6}, {NULL}};
const Segment SGI167265[] = {{&S50176[27], 6}, {NULL}};
const Segment SGI167266[] = {{&S50176[93], 9}, {NULL}};
const Segment SGI167267[] = {{&S50176[66], 6}, {NULL}};
const Segment SGI167268[] = {{&S60545[38], 7}, {NULL}};
const Segment SGI167269[] = {{&S60545[31], 6}, {NULL}};
const Segment SGI167270[] = {{&S50177[34], 15}, {NULL}};
const Segment SGI167271[] = {{&S50177[27], 6}, {NULL}};
const Segment SGI167272[] = {{&S50177[108], 9}, {NULL}};
const Segment SGI167273[] = {{&S50177[81], 6}, {NULL}};
const Segment SGI167274[] = {{&S56870[32], 9}, {NULL}};
const Segment SGI167275[] = {{&S50197[34], 10}, {NULL}};
const Segment SGI167276[] = {{&S56870[27], 4}, {NULL}};
const Segment SGI167277[] = {{&S50197[27], 6}, {NULL}};
const Segment SGI167278[] = {{&S50197[97], 9}, {NULL}};
const Segment SGI167279[] = {{&S56870[94], 9}, {NULL}};
const Segment SGI167280[] = {{&S50197[70], 6}, {NULL}};
const Segment SGI167281[] = {{&S56870[67], 6}, {NULL}};
const Segment SGI167282[] = {{&S50199[34], 10}, {NULL}};
const Segment SGI167283[] = {{&S50199[27], 6}, {NULL}};
const Segment SGI167284[] = {{&S50199[97], 9}, {NULL}};
const Segment SGI167285[] = {{&S50199[70], 6}, {NULL}};
const Segment SGI167286[] = {{&S50200[34], 11}, {NULL}};
const Segment SGI167287[] = {{&S50200[27], 6}, {NULL}};
const Segment SGI167288[] = {{&S50200[98], 9}, {NULL}};
const Segment SGI167289[] = {{&S50200[71], 6}, {NULL}};
const Segment SGI167290[] = {{&S60579[44], 7}, {NULL}};
const Segment SGI167291[] = {{&S60579[44], 19}, {NULL}};
const Segment SGI167292[] = {{&S60579[30], 6}, {NULL}};
const Segment SGI167293[] = {{&S60579[31], 3}, {NULL}};
const Segment SGI167294[] = {{&S60579[77], 6}, {NULL}};
const Segment SGI167295[] = {{&S60580[38], 7}, {NULL}};
const Segment SGI167296[] = {{&S60580[31], 6}, {NULL}};
const Segment SGI167297[] = {{&S60599[44], 9}, {NULL}};
const Segment SGI167298[] = {{&S60599[44], 18}, {NULL}};
const Segment SGI167299[] = {{&S60599[30], 6}, {NULL}};
const Segment SGI167300[] = {{&S60599[31], 3}, {NULL}};
const Segment SGI167301[] = {{&S60599[76], 6}, {NULL}};
const Segment SGI167302[] = {{&S60600[38], 9}, {NULL}};
const Segment SGI167303[] = {{&S60600[31], 6}, {NULL}};
const Segment SGI167304[] = {{&S50243[34], 10}, {NULL}};
const Segment SGI167305[] = {{&S50243[27], 6}, {NULL}};
const Segment SGI167306[] = {{&S50243[97], 9}, {NULL}};
const Segment SGI167307[] = {{&S50243[70], 6}, {NULL}};
const Segment SGI167308[] = {{&S50243[138], 8}, {NULL}};
const Segment SGI167309[] = {{&S50244[34], 10}, {NULL}};
const Segment SGI167310[] = {{&S50244[27], 6}, {NULL}};
const Segment SGI167311[] = {{&S50244[97], 9}, {NULL}};
const Segment SGI167312[] = {{&S50244[70], 6}, {NULL}};
const Segment SGI167313[] = {{&S50244[138], 8}, {NULL}};
const Segment SGI167314[] = {{&S60619[44], 13}, {NULL}};
const Segment SGI167315[] = {{&S60619[44], 25}, {NULL}};
const Segment SGI167316[] = {{&S60619[30], 6}, {NULL}};
const Segment SGI167317[] = {{&S50245[34], 11}, {NULL}};
const Segment SGI167318[] = {{&S50245[27], 6}, {NULL}};
const Segment SGI167319[] = {{&S60619[31], 3}, {NULL}};
const Segment SGI167320[] = {{&S60619[83], 6}, {NULL}};
const Segment SGI167321[] = {{&S50245[98], 9}, {NULL}};
const Segment SGI167322[] = {{&S50245[71], 6}, {NULL}};
const Segment SGI167323[] = {{&S50245[139], 8}, {NULL}};
const Segment SGI167324[] = {{&S60620[44], 13}, {NULL}};
const Segment SGI167325[] = {{&S60620[44], 25}, {NULL}};
const Segment SGI167326[] = {{&S60620[30], 6}, {NULL}};
const Segment SGI167327[] = {{&S60620[86], 6}, {NULL}};
const Segment SGI167328[] = {{&S50247[34], 11}, {NULL}};
const Segment SGI167329[] = {{&S50247[27], 6}, {NULL}};
const Segment SGI167330[] = {{&S50247[98], 9}, {NULL}};
const Segment SGI167331[] = {{&S50247[71], 6}, {NULL}};
const Segment SGI167332[] = {{&S60621[38], 13}, {NULL}};
const Segment SGI167333[] = {{&S50247[139], 8}, {NULL}};
const Segment SGI167334[] = {{&S60621[31], 6}, {NULL}};
const Segment SGI167335[] = {{&S60622[44], 14}, {NULL}};
const Segment SGI167336[] = {{&S60622[44], 26}, {NULL}};
const Segment SGI167337[] = {{&S60622[30], 6}, {NULL}};
const Segment SGI167338[] = {{&S50249[34], 11}, {NULL}};
const Segment SGI167339[] = {{&S50249[27], 6}, {NULL}};
const Segment SGI167340[] = {{&S60622[31], 3}, {NULL}};
const Segment SGI167341[] = {{&S60622[84], 6}, {NULL}};
const Segment SGI167342[] = {{&S50249[98], 9}, {NULL}};
const Segment SGI167343[] = {{&S50249[71], 6}, {NULL}};
const Segment SGI167344[] = {{&S50249[139], 8}, {NULL}};
const Segment SGI167345[] = {{&S60623[44], 13}, {NULL}};
const Segment SGI167346[] = {{&S60623[44], 25}, {NULL}};
const Segment SGI167347[] = {{&S60623[30], 6}, {NULL}};
const Segment SGI167348[] = {{&S50250[34], 11}, {NULL}};
const Segment SGI167349[] = {{&S60623[86], 4}, {NULL}};
const Segment SGI167350[] = {{&S50250[27], 6}, {NULL}};
const Segment SGI167351[] = {{&S60623[86], 3}, {NULL}};
const Segment SGI167352[] = {{&S50250[98], 9}, {NULL}};
const Segment SGI167353[] = {{&S50250[71], 6}, {NULL}};
const Segment SGI167354[] = {{&S50250[139], 8}, {NULL}};
const Segment SGI167355[] = {{&S50251[34], 11}, {NULL}};
const Segment SGI167356[] = {{&S50251[27], 6}, {NULL}};
const Segment SGI167357[] = {{&S50251[98], 9}, {NULL}};
const Segment SGI167358[] = {{&S50251[71], 6}, {NULL}};
const Segment SGI167359[] = {{&S50251[139], 8}, {NULL}};
const Segment SGI167360[] = {{&S60634[44], 8}, {NULL}};
const Segment SGI167361[] = {{&S60634[44], 19}, {NULL}};
const Segment SGI167362[] = {{&S60634[30], 6}, {NULL}};
const Segment SGI167363[] = {{&S60634[31], 3}, {NULL}};
const Segment SGI167364[] = {{&S60634[77], 6}, {NULL}};
const Segment SGI167365[] = {{&S60635[44], 8}, {NULL}};
const Segment SGI167366[] = {{&S60635[44], 19}, {NULL}};
const Segment SGI167367[] = {{&S60635[30], 6}, {NULL}};
const Segment SGI167368[] = {{&S60635[31], 3}, {NULL}};
const Segment SGI167369[] = {{&S60635[77], 6}, {NULL}};
const Segment SGI167370[] = {{&S60636[44], 8}, {NULL}};
const Segment SGI167371[] = {{&S60636[44], 19}, {NULL}};
const Segment SGI167372[] = {{&S60636[30], 6}, {NULL}};
const Segment SGI167373[] = {{&S60636[31], 3}, {NULL}};
const Segment SGI167374[] = {{&S60636[77], 6}, {NULL}};
const Segment SGI167375[] = {{&S60637[44], 8}, {NULL}};
const Segment SGI167376[] = {{&S60637[44], 19}, {NULL}};
const Segment SGI167377[] = {{&S60637[30], 6}, {NULL}};
const Segment SGI167378[] = {{&S60637[31], 3}, {NULL}};
const Segment SGI167379[] = {{&S60637[77], 6}, {NULL}};
const Segment SGI167380[] = {{&S60638[44], 8}, {NULL}};
const Segment SGI167381[] = {{&S60638[44], 19}, {NULL}};
const Segment SGI167382[] = {{&S60638[30], 6}, {NULL}};
const Segment SGI167383[] = {{&S60638[31], 3}, {NULL}};
const Segment SGI167384[] = {{&S60638[77], 6}, {NULL}};
const Segment SGI167385[] = {{&S60639[38], 8}, {NULL}};
const Segment SGI167386[] = {{&S60639[31], 6}, {NULL}};
const Segment SGI167387[] = {{&S52779[34], 12}, {NULL}};
const Segment SGI167388[] = {{&S52779[27], 6}, {NULL}};
const Segment SGI167389[] = {{&S52779[99], 9}, {NULL}};
const Segment SGI167390[] = {{&S52779[72], 6}, {NULL}};
const Segment SGI167391[] = {{&S37515[34], 10}, {NULL}};
const Segment SGI167392[] = {{&S37515[27], 6}, {NULL}};
const Segment SGI167393[] = {{&S37515[96], 9}, {NULL}};
const Segment SGI167394[] = {{&S37515[69], 6}, {NULL}};
const Segment SGI167395[] = {{&S52781[34], 26}, {NULL}};
const Segment SGI167396[] = {{&S52781[27], 6}, {NULL}};
const Segment SGI167397[] = {{&S37516[34], 11}, {NULL}};
const Segment SGI167398[] = {{&S37516[27], 6}, {NULL}};
const Segment SGI167399[] = {{&S52781[122], 9}, {NULL}};
const Segment SGI167400[] = {{&S52781[95], 6}, {NULL}};
const Segment SGI167401[] = {{&S37516[97], 9}, {NULL}};
const Segment SGI167402[] = {{&S37516[70], 6}, {NULL}};
const Segment SGI167403[] = {{&S50275[34], 10}, {NULL}};
const Segment SGI167404[] = {{&S50275[27], 6}, {NULL}};
const Segment SGI167405[] = {{&S60645[34], 9}, {NULL}};
const Segment SGI167406[] = {{&S60645[27], 6}, {NULL}};
const Segment SGI167407[] = {{&S37517[34], 11}, {NULL}};
const Segment SGI167408[] = {{&S50275[97], 9}, {NULL}};
const Segment SGI167409[] = {{&S50275[70], 6}, {NULL}};
const Segment SGI167410[] = {{&S37517[27], 6}, {NULL}};
const Segment SGI167411[] = {{&S60645[96], 9}, {NULL}};
const Segment SGI167412[] = {{&S60645[69], 6}, {NULL}};
const Segment SGI167413[] = {{&S37517[97], 9}, {NULL}};
const Segment SGI167414[] = {{&S37517[70], 6}, {NULL}};
const Segment SGI167415[] = {{&S50276[34], 10}, {NULL}};
const Segment SGI167416[] = {{&S50276[27], 6}, {NULL}};
const Segment SGI167417[] = {{&S50276[97], 9}, {NULL}};
const Segment SGI167418[] = {{&S50276[70], 6}, {NULL}};
const Segment SGI167419[] = {{&S50278[34], 11}, {NULL}};
const Segment SGI167420[] = {{&S50278[27], 6}, {NULL}};
const Segment SGI167421[] = {{&S50278[98], 9}, {NULL}};
const Segment SGI167422[] = {{&S50278[71], 6}, {NULL}};
const Segment SGI167423[] = {{&S50279[34], 11}, {NULL}};
const Segment SGI167424[] = {{&S50279[27], 6}, {NULL}};
const Segment SGI167425[] = {{&S50279[98], 9}, {NULL}};
const Segment SGI167426[] = {{&S50279[71], 6}, {NULL}};
const Segment SGI167427[] = {{&S50280[34], 11}, {NULL}};
const Segment SGI167428[] = {{&S50280[27], 6}, {NULL}};
const Segment SGI167429[] = {{&S50280[98], 9}, {NULL}};
const Segment SGI167430[] = {{&S50280[71], 6}, {NULL}};
const Segment SGI167431[] = {{&S50281[34], 11}, {NULL}};
const Segment SGI167432[] = {{&S50281[27], 6}, {NULL}};
const Segment SGI167433[] = {{&S50281[98], 9}, {NULL}};
const Segment SGI167434[] = {{&S50281[71], 6}, {NULL}};
const Segment SGI167435[] = {{&S50282[34], 11}, {NULL}};
const Segment SGI167436[] = {{&S50282[27], 6}, {NULL}};
const Segment SGI167437[] = {{&S50282[98], 9}, {NULL}};
const Segment SGI167438[] = {{&S50282[71], 6}, {NULL}};
const Segment SGI167439[] = {{&S50284[44], 7}, {NULL}};
const Segment SGI167440[] = {{&S50284[44], 28}, {NULL}};
const Segment SGI167441[] = {{&S50284[30], 6}, {NULL}};
const Segment SGI167442[] = {{&S50284[121], 3}, {NULL}};
const Segment SGI167443[] = {{&S50284[86], 6}, {NULL}};
const Segment SGI167444[] = {{&S50285[44], 7}, {NULL}};
const Segment SGI167445[] = {{&S50285[44], 29}, {NULL}};
const Segment SGI167446[] = {{&S50285[30], 6}, {NULL}};
const Segment SGI167447[] = {{&S50285[122], 3}, {NULL}};
const Segment SGI167448[] = {{&S50285[87], 6}, {NULL}};
const Segment SGI167449[] = {{&S50286[38], 7}, {NULL}};
const Segment SGI167450[] = {{&S50286[31], 6}, {NULL}};
const Segment SGI167451[] = {{&S37538[34], 14}, {NULL}};
const Segment SGI167452[] = {{&S37538[27], 6}, {NULL}};
const Segment SGI167453[] = {{&S50287[44], 7}, {NULL}};
const Segment SGI167454[] = {{&S50287[44], 29}, {NULL}};
const Segment SGI167455[] = {{&S50287[30], 6}, {NULL}};
const Segment SGI167456[] = {{&S37538[100], 9}, {NULL}};
const Segment SGI167457[] = {{&S37538[73], 6}, {NULL}};
const Segment SGI167458[] = {{&S50287[122], 3}, {NULL}};
const Segment SGI167459[] = {{&S50287[87], 6}, {NULL}};
const Segment SGI167460[] = {{&S60666[44], 8}, {NULL}};
const Segment SGI167461[] = {{&S60666[44], 19}, {NULL}};
const Segment SGI167462[] = {{&S60666[30], 6}, {NULL}};
const Segment SGI167463[] = {{&S50288[37], 15}, {NULL}};
const Segment SGI167464[] = {{&S60666[31], 3}, {NULL}};
const Segment SGI167465[] = {{&S60666[77], 6}, {NULL}};
const Segment SGI167466[] = {{&S50289[34], 8}, {NULL}};
const Segment SGI167467[] = {{&S50289[27], 6}, {NULL}};
const Segment SGI167468[] = {{&S60667[44], 8}, {NULL}};
const Segment SGI167469[] = {{&S60667[44], 19}, {NULL}};
const Segment SGI167470[] = {{&S60667[30], 6}, {NULL}};
const Segment SGI167471[] = {{&S50289[105], 9}, {NULL}};
const Segment SGI167472[] = {{&S60667[31], 3}, {NULL}};
const Segment SGI167473[] = {{&S50289[78], 6}, {NULL}};
const Segment SGI167474[] = {{&S60667[77], 6}, {NULL}};
const Segment SGI167475[] = {{&S60668[44], 8}, {NULL}};
const Segment SGI167476[] = {{&S60668[44], 19}, {NULL}};
const Segment SGI167477[] = {{&S60668[30], 6}, {NULL}};
const Segment SGI167478[] = {{&S60668[31], 3}, {NULL}};
const Segment SGI167479[] = {{&S60668[77], 6}, {NULL}};
const Segment SGI167480[] = {{&S60669[44], 8}, {NULL}};
const Segment SGI167481[] = {{&S60669[44], 19}, {NULL}};
const Segment SGI167482[] = {{&S60669[30], 6}, {NULL}};
const Segment SGI167483[] = {{&S50293[34], 12}, {NULL}};
const Segment SGI167484[] = {{&S50293[27], 6}, {NULL}};
const Segment SGI167485[] = {{&S60669[31], 3}, {NULL}};
const Segment SGI167486[] = {{&S60669[77], 6}, {NULL}};
const Segment SGI167487[] = {{&S50293[99], 9}, {NULL}};
const Segment SGI167488[] = {{&S50293[72], 6}, {NULL}};
const Segment SGI167489[] = {{&S60670[44], 8}, {NULL}};
const Segment SGI167490[] = {{&S60670[44], 19}, {NULL}};
const Segment SGI167491[] = {{&S60670[30], 6}, {NULL}};
const Segment SGI167492[] = {{&S60670[31], 3}, {NULL}};
const Segment SGI167493[] = {{&S60670[77], 6}, {NULL}};
const Segment SGI167494[] = {{&S60671[44], 8}, {NULL}};
const Segment SGI167495[] = {{&S60671[44], 19}, {NULL}};
const Segment SGI167496[] = {{&S60671[30], 6}, {NULL}};
const Segment SGI167497[] = {{&S60671[31], 3}, {NULL}};
const Segment SGI167498[] = {{&S60671[77], 6}, {NULL}};
const Segment SGI167499[] = {{&S50312[34], 14}, {NULL}};
const Segment SGI167500[] = {{&S50312[27], 6}, {NULL}};
const Segment SGI167501[] = {{&S50312[101], 9}, {NULL}};
const Segment SGI167502[] = {{&S50312[74], 6}, {NULL}};
const Segment SGI167503[] = {{&S50313[34], 13}, {NULL}};
const Segment SGI167504[] = {{&S50313[27], 6}, {NULL}};
const Segment SGI167505[] = {{&S50313[100], 9}, {NULL}};
const Segment SGI167506[] = {{&S50313[73], 6}, {NULL}};
const Segment SGI167507[] = {{&S50314[34], 10}, {NULL}};
const Segment SGI167508[] = {{&S50314[27], 6}, {NULL}};
const Segment SGI167509[] = {{&S50314[97], 9}, {NULL}};
const Segment SGI167510[] = {{&S50314[70], 6}, {NULL}};
const Segment SGI167511[] = {{&S60702[34], 10}, {NULL}};
const Segment SGI167512[] = {{&S60702[27], 6}, {NULL}};
const Segment SGI167513[] = {{&S60702[97], 9}, {NULL}};
const Segment SGI167514[] = {{&S60702[70], 6}, {NULL}};
const Segment SGI167515[] = {{&S60705[34], 10}, {NULL}};
const Segment SGI167516[] = {{&S60705[27], 6}, {NULL}};
const Segment SGI167517[] = {{&S60705[97], 9}, {NULL}};
const Segment SGI167518[] = {{&S60705[70], 6}, {NULL}};
const Segment SGI167519[] = {{&S60707[34], 9}, {NULL}};
const Segment SGI167520[] = {{&S60707[27], 6}, {NULL}};
const Segment SGI167521[] = {{&S60707[96], 9}, {NULL}};
const Segment SGI167522[] = {{&S60707[69], 6}, {NULL}};
const Segment SGI167523[] = {{&S60709[34], 8}, {NULL}};
const Segment SGI167524[] = {{&S60709[27], 6}, {NULL}};
const Segment SGI167525[] = {{&S60709[95], 9}, {NULL}};
const Segment SGI167526[] = {{&S60709[68], 6}, {NULL}};
const Segment SGI167527[] = {{&S50337[52], 9}, {NULL}};
const Segment SGI167528[] = {{&S50337[52], 21}, {NULL}};
const Segment SGI167529[] = {{&S50337[38], 6}, {NULL}};
const Segment SGI167530[] = {{&S50337[122], 3}, {NULL}};
const Segment SGI167531[] = {{&S50337[87], 6}, {NULL}};
const Segment SGI167532[] = {{&S50338[44], 14}, {NULL}};
const Segment SGI167533[] = {{&S50338[44], 26}, {NULL}};
const Segment SGI167534[] = {{&S50338[30], 6}, {NULL}};
const Segment SGI167535[] = {{&S50338[119], 3}, {NULL}};
const Segment SGI167536[] = {{&S50338[84], 6}, {NULL}};
const Segment SGI167537[] = {{&S50339[38], 9}, {NULL}};
const Segment SGI167538[] = {{&S50339[31], 6}, {NULL}};
const Segment SGI167539[] = {{&S50340[44], 14}, {NULL}};
const Segment SGI167540[] = {{&S50340[44], 26}, {NULL}};
const Segment SGI167541[] = {{&S50340[30], 6}, {NULL}};
const Segment SGI167542[] = {{&S60715[34], 7}, {NULL}};
const Segment SGI167543[] = {{&S50340[119], 3}, {NULL}};
const Segment SGI167544[] = {{&S60715[27], 6}, {NULL}};
const Segment SGI167545[] = {{&S50340[84], 6}, {NULL}};
const Segment SGI167546[] = {{&S60715[112], 9}, {NULL}};
const Segment SGI167547[] = {{&S60715[85], 6}, {NULL}};
const Segment SGI167548[] = {{&S50341[44], 14}, {NULL}};
const Segment SGI167549[] = {{&S50341[44], 26}, {NULL}};
const Segment SGI167550[] = {{&S50341[30], 6}, {NULL}};
const Segment SGI167551[] = {{&S50341[119], 3}, {NULL}};
const Segment SGI167552[] = {{&S50341[84], 6}, {NULL}};
const Segment SGI167553[] = {{&S50342[44], 24}, {NULL}};
const Segment SGI167554[] = {{&S50342[44], 36}, {NULL}};
const Segment SGI167555[] = {{&S50342[30], 6}, {NULL}};
const Segment SGI167556[] = {{&S50342[129], 3}, {NULL}};
const Segment SGI167557[] = {{&S50342[94], 6}, {NULL}};
const Segment SGI167558[] = {{&S50343[44], 24}, {NULL}};
const Segment SGI167559[] = {{&S50343[44], 36}, {NULL}};
const Segment SGI167560[] = {{&S50343[30], 6}, {NULL}};
const Segment SGI167561[] = {{&S50343[129], 3}, {NULL}};
const Segment SGI167562[] = {{&S50343[94], 6}, {NULL}};
const Segment SGI167563[] = {{&S50344[44], 31}, {NULL}};
const Segment SGI167564[] = {{&S50344[44], 43}, {NULL}};
const Segment SGI167565[] = {{&S50344[30], 6}, {NULL}};
const Segment SGI167566[] = {{&S50344[136], 3}, {NULL}};
const Segment SGI167567[] = {{&S50344[101], 6}, {NULL}};
const Segment SGI167568[] = {{&S50345[41], 9}, {NULL}};
const Segment SGI167569[] = {{&S50345[30], 6}, {NULL}};
const Segment SGI167570[] = {{&S50345[111], 3}, {NULL}};
const Segment SGI167571[] = {{&S50345[76], 6}, {NULL}};
const Segment SGI167572[] = {{&S50346[44], 24}, {NULL}};
const Segment SGI167573[] = {{&S50346[44], 36}, {NULL}};
const Segment SGI167574[] = {{&S50346[30], 6}, {NULL}};
const Segment SGI167575[] = {{&S50346[129], 3}, {NULL}};
const Segment SGI167576[] = {{&S50346[94], 6}, {NULL}};
const Segment SGI167577[] = {{&S50347[44], 10}, {NULL}};
const Segment SGI167578[] = {{&S50347[44], 22}, {NULL}};
const Segment SGI167579[] = {{&S50347[30], 6}, {NULL}};
const Segment SGI167580[] = {{&S50347[115], 3}, {NULL}};
const Segment SGI167581[] = {{&S50347[80], 6}, {NULL}};
const Segment SGI167582[] = {{&S50348[44], 24}, {NULL}};
const Segment SGI167583[] = {{&S50348[44], 36}, {NULL}};
const Segment SGI167584[] = {{&S50348[30], 6}, {NULL}};
const Segment SGI167585[] = {{&S50348[129], 3}, {NULL}};
const Segment SGI167586[] = {{&S50348[94], 6}, {NULL}};
const Segment SGI167587[] = {{&S50349[44], 23}, {NULL}};
const Segment SGI167588[] = {{&S50349[44], 35}, {NULL}};
const Segment SGI167589[] = {{&S50349[30], 6}, {NULL}};
const Segment SGI167590[] = {{&S50349[128], 3}, {NULL}};
const Segment SGI167591[] = {{&S50349[93], 6}, {NULL}};
const Segment SGI167592[] = {{&S50350[44], 14}, {NULL}};
const Segment SGI167593[] = {{&S50350[44], 26}, {NULL}};
const Segment SGI167594[] = {{&S50350[30], 6}, {NULL}};
const Segment SGI167595[] = {{&S50350[119], 3}, {NULL}};
const Segment SGI167596[] = {{&S50350[84], 6}, {NULL}};
const Segment SGI167597[] = {{&S50351[44], 31}, {NULL}};
const Segment SGI167598[] = {{&S50351[44], 43}, {NULL}};
const Segment SGI167599[] = {{&S50351[30], 6}, {NULL}};
const Segment SGI167600[] = {{&S50351[136], 3}, {NULL}};
const Segment SGI167601[] = {{&S50351[101], 6}, {NULL}};
const Segment SGI167602[] = {{&S50367[34], 7}, {NULL}};
const Segment SGI167603[] = {{&S50367[27], 6}, {NULL}};
const Segment SGI167604[] = {{&S50367[109], 9}, {NULL}};
const Segment SGI167605[] = {{&S50367[82], 6}, {NULL}};
const Segment SGI167606[] = {{&S60759[44], 10}, {NULL}};
const Segment SGI167607[] = {{&S60759[44], 22}, {NULL}};
const Segment SGI167608[] = {{&S60759[30], 6}, {NULL}};
const Segment SGI167609[] = {{&S60759[31], 3}, {NULL}};
const Segment SGI167610[] = {{&S60759[80], 6}, {NULL}};
const Segment SGI167611[] = {{&S60760[44], 7}, {NULL}};
const Segment SGI167612[] = {{&S60760[44], 23}, {NULL}};
const Segment SGI167613[] = {{&S60760[30], 6}, {NULL}};
const Segment SGI167614[] = {{&S60760[31], 3}, {NULL}};
const Segment SGI167615[] = {{&S60760[81], 6}, {NULL}};
const Segment SGI167616[] = {{&S60761[44], 7}, {NULL}};
const Segment SGI167617[] = {{&S60761[44], 23}, {NULL}};
const Segment SGI167618[] = {{&S60761[30], 6}, {NULL}};
const Segment SGI167619[] = {{&S60761[31], 3}, {NULL}};
const Segment SGI167620[] = {{&S60761[81], 6}, {NULL}};
const Segment SGI167621[] = {{&S60762[44], 7}, {NULL}};
const Segment SGI167622[] = {{&S60762[44], 23}, {NULL}};
const Segment SGI167623[] = {{&S60762[30], 6}, {NULL}};
const Segment SGI167624[] = {{&S60762[31], 3}, {NULL}};
const Segment SGI167625[] = {{&S60762[81], 6}, {NULL}};
const Segment SGI167626[] = {{&S50383[44], 15}, {NULL}};
const Segment SGI167627[] = {{&S50383[44], 27}, {NULL}};
const Segment SGI167628[] = {{&S50383[30], 6}, {NULL}};
const Segment SGI167629[] = {{&S60763[44], 7}, {NULL}};
const Segment SGI167630[] = {{&S50383[120], 3}, {NULL}};
const Segment SGI167631[] = {{&S60763[44], 23}, {NULL}};
const Segment SGI167632[] = {{&S50383[85], 6}, {NULL}};
const Segment SGI167633[] = {{&S60763[30], 6}, {NULL}};
const Segment SGI167634[] = {{&S60763[31], 3}, {NULL}};
const Segment SGI167635[] = {{&S60763[81], 6}, {NULL}};
const Segment SGI167636[] = {{&S50384[38], 15}, {NULL}};
const Segment SGI167637[] = {{&S50384[31], 6}, {NULL}};
const Segment SGI167638[] = {{&S60764[38], 7}, {NULL}};
const Segment SGI167639[] = {{&S60764[31], 6}, {NULL}};
const Segment SGI167640[] = {{&S50385[42], 15}, {NULL}};
const Segment SGI167641[] = {{&S50385[30], 6}, {NULL}};
const Segment SGI167642[] = {{&S50385[118], 3}, {NULL}};
const Segment SGI167643[] = {{&S50385[83], 6}, {NULL}};
const Segment SGI167644[] = {{&S60767[34], 9}, {NULL}};
const Segment SGI167645[] = {{&S60767[27], 6}, {NULL}};
const Segment SGI167646[] = {{&S60767[96], 9}, {NULL}};
const Segment SGI167647[] = {{&S60767[69], 6}, {NULL}};
const Segment SGI167648[] = {{&S50388[34], 14}, {NULL}};
const Segment SGI167649[] = {{&S50388[27], 6}, {NULL}};
const Segment SGI167650[] = {{&S50388[101], 9}, {NULL}};
const Segment SGI167651[] = {{&S50388[74], 6}, {NULL}};
const Segment SGI167652[] = {{&S60768[34], 11}, {NULL}};
const Segment SGI167653[] = {{&S60768[27], 6}, {NULL}};
const Segment SGI167654[] = {{&S60768[98], 9}, {NULL}};
const Segment SGI167655[] = {{&S60768[71], 6}, {NULL}};
const Segment SGI167656[] = {{&S50389[34], 14}, {NULL}};
const Segment SGI167657[] = {{&S50389[27], 6}, {NULL}};
const Segment SGI167658[] = {{&S50389[101], 9}, {NULL}};
const Segment SGI167659[] = {{&S50389[74], 6}, {NULL}};
const Segment SGI167660[] = {{&S60770[34], 10}, {NULL}};
const Segment SGI167661[] = {{&S60770[27], 6}, {NULL}};
const Segment SGI167662[] = {{&S60770[97], 9}, {NULL}};
const Segment SGI167663[] = {{&S60770[70], 6}, {NULL}};
const Segment SGI167664[] = {{&S60771[34], 10}, {NULL}};
const Segment SGI167665[] = {{&S60771[27], 6}, {NULL}};
const Segment SGI167666[] = {{&S60771[97], 9}, {NULL}};
const Segment SGI167667[] = {{&S60771[70], 6}, {NULL}};
const Segment SGI167668[] = {{&S60783[44], 21}, {NULL}};
const Segment SGI167669[] = {{&S60783[44], 33}, {NULL}};
const Segment SGI167670[] = {{&S60783[30], 6}, {NULL}};
const Segment SGI167671[] = {{&S60783[31], 3}, {NULL}};
const Segment SGI167672[] = {{&S60783[91], 6}, {NULL}};
const Segment SGI167673[] = {{&S60784[44], 21}, {NULL}};
const Segment SGI167674[] = {{&S60784[44], 33}, {NULL}};
const Segment SGI167675[] = {{&S60784[30], 6}, {NULL}};
const Segment SGI167676[] = {{&S60784[31], 3}, {NULL}};
const Segment SGI167677[] = {{&S60784[91], 6}, {NULL}};
const Segment SGI167678[] = {{&S60785[44], 25}, {NULL}};
const Segment SGI167679[] = {{&S60785[44], 37}, {NULL}};
const Segment SGI167680[] = {{&S60785[30], 6}, {NULL}};
const Segment SGI167681[] = {{&S60785[31], 3}, {NULL}};
const Segment SGI167682[] = {{&S60785[95], 6}, {NULL}};
const Segment SGI167683[] = {{&S50418[34], 9}, {NULL}};
const Segment SGI167684[] = {{&S50418[27], 6}, {NULL}};
const Segment SGI167685[] = {{&S50418[96], 9}, {NULL}};
const Segment SGI167686[] = {{&S50418[69], 6}, {NULL}};
const Segment SGI167687[] = {{&S50420[34], 14}, {NULL}};
const Segment SGI167688[] = {{&S50420[27], 6}, {NULL}};
const Segment SGI167689[] = {{&S50420[101], 9}, {NULL}};
const Segment SGI167690[] = {{&S50420[74], 6}, {NULL}};
const Segment SGI167691[] = {{&S50426[34], 14}, {NULL}};
const Segment SGI167692[] = {{&S37707[34], 9}, {NULL}};
const Segment SGI167693[] = {{&S50426[27], 6}, {NULL}};
const Segment SGI167694[] = {{&S37707[27], 6}, {NULL}};
const Segment SGI167695[] = {{&S50426[111], 9}, {NULL}};
const Segment SGI167696[] = {{&S37707[95], 9}, {NULL}};
const Segment SGI167697[] = {{&S50426[84], 6}, {NULL}};
const Segment SGI167698[] = {{&S37707[68], 6}, {NULL}};
const Segment SGI167699[] = {{&S50427[34], 14}, {NULL}};
const Segment SGI167700[] = {{&S50427[27], 6}, {NULL}};
const Segment SGI167701[] = {{&S50427[111], 9}, {NULL}};
const Segment SGI167702[] = {{&S50427[84], 6}, {NULL}};
const Segment SGI167703[] = {{&S37710[34], 9}, {NULL}};
const Segment SGI167704[] = {{&S37710[27], 6}, {NULL}};
const Segment SGI167705[] = {{&S52990[34], 6}, {NULL}};
const Segment SGI167706[] = {{&S52990[27], 6}, {NULL}};
const Segment SGI167707[] = {{&S37710[95], 9}, {NULL}};
const Segment SGI167708[] = {{&S37710[68], 6}, {NULL}};
const Segment SGI167709[] = {{&S52990[93], 9}, {NULL}};
const Segment SGI167710[] = {{&S52990[66], 6}, {NULL}};
const Segment SGI167711[] = {{&S60786[38], 25}, {NULL}};
const Segment SGI167712[] = {{&S60786[31], 6}, {NULL}};
const Segment SGI167713[] = {{&S60790[34], 11}, {NULL}};
const Segment SGI167714[] = {{&S60790[27], 6}, {NULL}};
const Segment SGI167715[] = {{&S60790[98], 9}, {NULL}};
const Segment SGI167716[] = {{&S60790[71], 6}, {NULL}};
const Segment SGI167717[] = {{&S60791[34], 14}, {NULL}};
const Segment SGI167718[] = {{&S60791[27], 6}, {NULL}};
const Segment SGI167719[] = {{&S60791[101], 9}, {NULL}};
const Segment SGI167720[] = {{&S60791[74], 6}, {NULL}};
const Segment SGI167721[] = {{&S60792[34], 9}, {NULL}};
const Segment SGI167722[] = {{&S60792[27], 6}, {NULL}};
const Segment SGI167723[] = {{&S60792[96], 9}, {NULL}};
const Segment SGI167724[] = {{&S60792[69], 6}, {NULL}};
const Segment SGI167725[] = {{&S37739[44], 24}, {NULL}};
const Segment SGI167726[] = {{&S37739[44], 35}, {NULL}};
const Segment SGI167727[] = {{&S37739[30], 6}, {NULL}};
const Segment SGI167728[] = {{&S60793[34], 6}, {NULL}};
const Segment SGI167729[] = {{&S37739[128], 3}, {NULL}};
const Segment SGI167730[] = {{&S60793[27], 6}, {NULL}};
const Segment SGI167731[] = {{&S37739[93], 6}, {NULL}};
const Segment SGI167732[] = {{&S60793[93], 9}, {NULL}};
const Segment SGI167733[] = {{&S60793[66], 6}, {NULL}};
const Segment SGI167734[] = {{&S60794[34], 15}, {NULL}};
const Segment SGI167735[] = {{&S60794[27], 6}, {NULL}};
const Segment SGI167736[] = {{&S60794[102], 9}, {NULL}};
const Segment SGI167737[] = {{&S60794[75], 6}, {NULL}};
const Segment SGI167738[] = {{&S60795[34], 14}, {NULL}};
const Segment SGI167739[] = {{&S60795[27], 6}, {NULL}};
const Segment SGI167740[] = {{&S60795[101], 9}, {NULL}};
const Segment SGI167741[] = {{&S60795[74], 6}, {NULL}};
const Segment SGI167742[] = {{&S60796[34], 9}, {NULL}};
const Segment SGI167743[] = {{&S60796[27], 6}, {NULL}};
const Segment SGI167744[] = {{&S60796[96], 9}, {NULL}};
const Segment SGI167745[] = {{&S60796[69], 6}, {NULL}};
const Segment SGI167746[] = {{&S50470[34], 10}, {NULL}};
const Segment SGI167747[] = {{&S50470[27], 6}, {NULL}};
const Segment SGI167748[] = {{&S60797[34], 5}, {NULL}};
const Segment SGI167749[] = {{&S60797[27], 6}, {NULL}};
const Segment SGI167750[] = {{&S50470[97], 9}, {NULL}};
const Segment SGI167751[] = {{&S50470[70], 6}, {NULL}};
const Segment SGI167752[] = {{&S60797[92], 9}, {NULL}};
const Segment SGI167753[] = {{&S60797[65], 6}, {NULL}};
const Segment SGI167754[] = {{&S50471[34], 10}, {NULL}};
const Segment SGI167755[] = {{&S60798[34], 9}, {NULL}};
const Segment SGI167756[] = {{&S50471[27], 6}, {NULL}};
const Segment SGI167757[] = {{&S60798[27], 6}, {NULL}};
const Segment SGI167758[] = {{&S50471[97], 9}, {NULL}};
const Segment SGI167759[] = {{&S50471[70], 6}, {NULL}};
const Segment SGI167760[] = {{&S60798[96], 9}, {NULL}};
const Segment SGI167761[] = {{&S60798[69], 6}, {NULL}};
const Segment SGI167762[] = {{&S60799[34], 11}, {NULL}};
const Segment SGI167763[] = {{&S60799[27], 6}, {NULL}};
const Segment SGI167764[] = {{&S60799[98], 9}, {NULL}};
const Segment SGI167765[] = {{&S60799[71], 6}, {NULL}};
const Segment SGI167766[] = {{&S60800[34], 11}, {NULL}};
const Segment SGI167767[] = {{&S60800[27], 6}, {NULL}};
const Segment SGI167768[] = {{&S60800[98], 9}, {NULL}};
const Segment SGI167769[] = {{&S60800[71], 6}, {NULL}};
const Segment SGI167770[] = {{&S50476[34], 8}, {NULL}};
const Segment SGI167771[] = {{&S60801[34], 16}, {NULL}};
const Segment SGI167772[] = {{&S50476[27], 6}, {NULL}};
const Segment SGI167773[] = {{&S60801[27], 6}, {NULL}};
const Segment SGI167774[] = {{&S50476[101], 9}, {NULL}};
const Segment SGI167775[] = {{&S50476[74], 6}, {NULL}};
const Segment SGI167776[] = {{&S60801[103], 9}, {NULL}};
const Segment SGI167777[] = {{&S60801[76], 6}, {NULL}};
const Segment SGI167778[] = {{&S60802[34], 7}, {NULL}};
const Segment SGI167779[] = {{&S60802[27], 6}, {NULL}};
const Segment SGI167780[] = {{&S60802[94], 9}, {NULL}};
const Segment SGI167781[] = {{&S60802[67], 6}, {NULL}};
const Segment SGI167782[] = {{&S60803[34], 6}, {NULL}};
const Segment SGI167783[] = {{&S60803[27], 6}, {NULL}};
const Segment SGI167784[] = {{&S60803[93], 9}, {NULL}};
const Segment SGI167785[] = {{&S60803[66], 6}, {NULL}};
const Segment SGI167786[] = {{&S60804[34], 6}, {NULL}};
const Segment SGI167787[] = {{&S60804[27], 6}, {NULL}};
const Segment SGI167788[] = {{&S60804[93], 9}, {NULL}};
const Segment SGI167789[] = {{&S60804[66], 6}, {NULL}};
const Segment SGI167790[] = {{&S60805[34], 6}, {NULL}};
const Segment SGI167791[] = {{&S60805[27], 6}, {NULL}};
const Segment SGI167792[] = {{&S60805[93], 9}, {NULL}};
const Segment SGI167793[] = {{&S60805[66], 6}, {NULL}};
const Segment SGI167794[] = {{&S60806[34], 15}, {NULL}};
const Segment SGI167795[] = {{&S60806[27], 6}, {NULL}};
const Segment SGI167796[] = {{&S60806[102], 9}, {NULL}};
const Segment SGI167797[] = {{&S60806[75], 6}, {NULL}};
const Segment SGI167798[] = {{&S60808[34], 7}, {NULL}};
const Segment SGI167799[] = {{&S60808[27], 6}, {NULL}};
const Segment SGI167800[] = {{&S60808[94], 9}, {NULL}};
const Segment SGI167801[] = {{&S60808[67], 6}, {NULL}};
const Segment SGI167802[] = {{&S60809[34], 6}, {NULL}};
const Segment SGI167803[] = {{&S60809[27], 6}, {NULL}};
const Segment SGI167804[] = {{&S60809[93], 9}, {NULL}};
const Segment SGI167805[] = {{&S60809[66], 6}, {NULL}};
const Segment SGI167806[] = {{&S60810[34], 5}, {NULL}};
const Segment SGI167807[] = {{&S60810[27], 6}, {NULL}};
const Segment SGI167808[] = {{&S60810[92], 9}, {NULL}};
const Segment SGI167809[] = {{&S60810[65], 6}, {NULL}};
const Segment SGI167810[] = {{&S60811[34], 10}, {NULL}};
const Segment SGI167811[] = {{&S60811[27], 6}, {NULL}};
const Segment SGI167812[] = {{&S60811[97], 9}, {NULL}};
const Segment SGI167813[] = {{&S60811[70], 6}, {NULL}};
const Segment SGI167814[] = {{&S60812[34], 7}, {NULL}};
const Segment SGI167815[] = {{&S60812[27], 6}, {NULL}};
const Segment SGI167816[] = {{&S60812[94], 9}, {NULL}};
const Segment SGI167817[] = {{&S60812[67], 6}, {NULL}};
const Segment SGI167818[] = {{&S60813[34], 4}, {NULL}};
const Segment SGI167819[] = {{&S60813[27], 6}, {NULL}};
const Segment SGI167820[] = {{&S60813[91], 9}, {NULL}};
const Segment SGI167821[] = {{&S60813[64], 6}, {NULL}};
const Segment SGI167822[] = {{&S60814[34], 9}, {NULL}};
const Segment SGI167823[] = {{&S60814[27], 6}, {NULL}};
const Segment SGI167824[] = {{&S60814[96], 9}, {NULL}};
const Segment SGI167825[] = {{&S60814[69], 6}, {NULL}};
const Segment SGI167826[] = {{&S53066[44], 9}, {NULL}};
const Segment SGI167827[] = {{&S53066[44], 21}, {NULL}};
const Segment SGI167828[] = {{&S53066[30], 6}, {NULL}};
const Segment SGI167829[] = {{&S53066[139], 3}, {NULL}};
const Segment SGI167830[] = {{&S53066[104], 6}, {NULL}};
const Segment SGI167831[] = {{&S53066[67], 8}, {NULL}};
const Segment SGI167832[] = {{&S50508[44], 10}, {NULL}};
const Segment SGI167833[] = {{&S50508[44], 22}, {NULL}};
const Segment SGI167834[] = {{&S50508[30], 6}, {NULL}};
const Segment SGI167835[] = {{&S50508[115], 3}, {NULL}};
const Segment SGI167836[] = {{&S50508[80], 6}, {NULL}};
const Segment SGI167837[] = {{&S50509[38], 10}, {NULL}};
const Segment SGI167838[] = {{&S50509[31], 6}, {NULL}};
const Segment SGI167839[] = {{&S50510[42], 10}, {NULL}};
const Segment SGI167840[] = {{&S50510[30], 6}, {NULL}};
const Segment SGI167841[] = {{&S50510[113], 3}, {NULL}};
const Segment SGI167842[] = {{&S50510[78], 6}, {NULL}};
const Segment SGI167843[] = {{&S47465[44], 10}, {NULL}};
const Segment SGI167844[] = {{&S47465[44], 22}, {NULL}};
const Segment SGI167845[] = {{&S47465[30], 6}, {NULL}};
const Segment SGI167846[] = {{&S47465[133], 3}, {NULL}};
const Segment SGI167847[] = {{&S47465[98], 6}, {NULL}};
const Segment SGI167848[] = {{&S47465[68], 8}, {NULL}};
const Segment SGI167849[] = {{&S47466[44], 10}, {NULL}};
const Segment SGI167850[] = {{&S47466[44], 22}, {NULL}};
const Segment SGI167851[] = {{&S47466[30], 6}, {NULL}};
const Segment SGI167852[] = {{&S47466[133], 3}, {NULL}};
const Segment SGI167853[] = {{&S47466[98], 6}, {NULL}};
const Segment SGI167854[] = {{&S47466[68], 8}, {NULL}};
const Segment SGI167855[] = {{&S47467[44], 10}, {NULL}};
const Segment SGI167856[] = {{&S47467[44], 22}, {NULL}};
const Segment SGI167857[] = {{&S47467[30], 6}, {NULL}};
const Segment SGI167858[] = {{&S47467[133], 3}, {NULL}};
const Segment SGI167859[] = {{&S47467[98], 6}, {NULL}};
const Segment SGI167860[] = {{&S47467[68], 8}, {NULL}};
const Segment SGI167861[] = {{&S37804[41], 18}, {NULL}};
const Segment SGI167862[] = {{&S37804[41], 29}, {NULL}};
const Segment SGI167863[] = {{&S37804[27], 6}, {NULL}};
const Segment SGI167864[] = {{&S37804[119], 3}, {NULL}};
const Segment SGI167865[] = {{&S37804[123], 9}, {NULL}};
const Segment SGI167866[] = {{&S37804[84], 6}, {NULL}};
const Segment SGI167867[] = {{&S37805[41], 21}, {NULL}};
const Segment SGI167868[] = {{&S37805[41], 32}, {NULL}};
const Segment SGI167869[] = {{&S37805[27], 6}, {NULL}};
const Segment SGI167870[] = {{&S37805[122], 3}, {NULL}};
const Segment SGI167871[] = {{&S37805[126], 9}, {NULL}};
const Segment SGI167872[] = {{&S37805[87], 6}, {NULL}};
const Segment SGI167873[] = {{&S37806[34], 10}, {NULL}};
const Segment SGI167874[] = {{&S37806[27], 6}, {NULL}};
const Segment SGI167875[] = {{&S37806[96], 9}, {NULL}};
const Segment SGI167876[] = {{&S37806[69], 6}, {NULL}};
const Segment SGI167877[] = {{&S37807[41], 19}, {NULL}};
const Segment SGI167878[] = {{&S37807[41], 30}, {NULL}};
const Segment SGI167879[] = {{&S37807[27], 6}, {NULL}};
const Segment SGI167880[] = {{&S37807[120], 3}, {NULL}};
const Segment SGI167881[] = {{&S60825[34], 15}, {NULL}};
const Segment SGI167882[] = {{&S37807[124], 9}, {NULL}};
const Segment SGI167883[] = {{&S37807[85], 6}, {NULL}};
const Segment SGI167884[] = {{&S60825[27], 6}, {NULL}};
const Segment SGI167885[] = {{&S60825[108], 9}, {NULL}};
const Segment SGI167886[] = {{&S60825[81], 6}, {NULL}};
const Segment SGI167887[] = {{&S60826[34], 14}, {NULL}};
const Segment SGI167888[] = {{&S60826[27], 6}, {NULL}};
const Segment SGI167889[] = {{&S60826[106], 9}, {NULL}};
const Segment SGI167890[] = {{&S60826[79], 6}, {NULL}};
const Segment SGI167891[] = {{&S60827[34], 18}, {NULL}};
const Segment SGI167892[] = {{&S60827[27], 6}, {NULL}};
const Segment SGI167893[] = {{&S60827[105], 9}, {NULL}};
const Segment SGI167894[] = {{&S60827[78], 6}, {NULL}};
const Segment SGI167895[] = {{&S60829[44], 9}, {NULL}};
const Segment SGI167896[] = {{&S60829[44], 20}, {NULL}};
const Segment SGI167897[] = {{&S60829[30], 6}, {NULL}};
const Segment SGI167898[] = {{&S60829[31], 3}, {NULL}};
const Segment SGI167899[] = {{&S60829[78], 6}, {NULL}};
const Segment SGI167900[] = {{&S60830[44], 9}, {NULL}};
const Segment SGI167901[] = {{&S60830[44], 20}, {NULL}};
const Segment SGI167902[] = {{&S60830[30], 6}, {NULL}};
const Segment SGI167903[] = {{&S60830[31], 3}, {NULL}};
const Segment SGI167904[] = {{&S60830[78], 6}, {NULL}};
const Segment SGI167905[] = {{&S60831[44], 9}, {NULL}};
const Segment SGI167906[] = {{&S60831[44], 21}, {NULL}};
const Segment SGI167907[] = {{&S60831[30], 6}, {NULL}};
const Segment SGI167908[] = {{&S60831[31], 3}, {NULL}};
const Segment SGI167909[] = {{&S60831[79], 6}, {NULL}};
const Segment SGI167910[] = {{&S60833[44], 9}, {NULL}};
const Segment SGI167911[] = {{&S60833[44], 21}, {NULL}};
const Segment SGI167912[] = {{&S60833[30], 6}, {NULL}};
const Segment SGI167913[] = {{&S60833[31], 3}, {NULL}};
const Segment SGI167914[] = {{&S60833[79], 6}, {NULL}};
const Segment SGI167915[] = {{&S60834[38], 9}, {NULL}};
const Segment SGI167916[] = {{&S60834[31], 6}, {NULL}};
const Segment SGI167917[] = {{&S37832[41], 18}, {NULL}};
const Segment SGI167918[] = {{&S37832[41], 29}, {NULL}};
const Segment SGI167919[] = {{&S37832[27], 6}, {NULL}};
const Segment SGI167920[] = {{&S37832[119], 3}, {NULL}};
const Segment SGI167921[] = {{&S37832[123], 9}, {NULL}};
const Segment SGI167922[] = {{&S37832[84], 6}, {NULL}};
const Segment SGI167923[] = {{&S37833[41], 10}, {NULL}};
const Segment SGI167924[] = {{&S37833[41], 21}, {NULL}};
const Segment SGI167925[] = {{&S37833[27], 6}, {NULL}};
const Segment SGI167926[] = {{&S37833[111], 3}, {NULL}};
const Segment SGI167927[] = {{&S37833[115], 9}, {NULL}};
const Segment SGI167928[] = {{&S37833[76], 6}, {NULL}};
const Segment SGI167929[] = {{&S60848[34], 7}, {NULL}};
const Segment SGI167930[] = {{&S60848[27], 6}, {NULL}};
const Segment SGI167931[] = {{&S60848[99], 9}, {NULL}};
const Segment SGI167932[] = {{&S60848[72], 6}, {NULL}};
const Segment SGI167933[] = {{&S60849[34], 8}, {NULL}};
const Segment SGI167934[] = {{&S60849[27], 6}, {NULL}};
const Segment SGI167935[] = {{&S60849[99], 9}, {NULL}};
const Segment SGI167936[] = {{&S60849[72], 6}, {NULL}};
const Segment SGI167937[] = {{&S60850[34], 7}, {NULL}};
const Segment SGI167938[] = {{&S60850[27], 6}, {NULL}};
const Segment SGI167939[] = {{&S50574[34], 11}, {NULL}};
const Segment SGI167940[] = {{&S60850[99], 9}, {NULL}};
const Segment SGI167941[] = {{&S60850[72], 6}, {NULL}};
const Segment SGI167942[] = {{&S50574[27], 6}, {NULL}};
const Segment SGI167943[] = {{&S50574[98], 9}, {NULL}};
const Segment SGI167944[] = {{&S50574[71], 6}, {NULL}};
const Segment SGI167945[] = {{&S60851[34], 8}, {NULL}};
const Segment SGI167946[] = {{&S60851[27], 6}, {NULL}};
const Segment SGI167947[] = {{&S60851[99], 9}, {NULL}};
const Segment SGI167948[] = {{&S60851[72], 6}, {NULL}};
const Segment SGI167949[] = {{&S37850[44], 10}, {NULL}};
const Segment SGI167950[] = {{&S37850[44], 21}, {NULL}};
const Segment SGI167951[] = {{&S37850[30], 6}, {NULL}};
const Segment SGI167952[] = {{&S37850[132], 3}, {NULL}};
const Segment SGI167953[] = {{&S37850[97], 6}, {NULL}};
const Segment SGI167954[] = {{&S37850[67], 8}, {NULL}};
const Segment SGI167955[] = {{&S37851[44], 10}, {NULL}};
const Segment SGI167956[] = {{&S37851[44], 22}, {NULL}};
const Segment SGI167957[] = {{&S37851[30], 6}, {NULL}};
const Segment SGI167958[] = {{&S37851[115], 3}, {NULL}};
const Segment SGI167959[] = {{&S37851[80], 6}, {NULL}};
const Segment SGI167960[] = {{&S37851[140], 8}, {NULL}};
const Segment SGI167961[] = {{&S60865[34], 8}, {NULL}};
const Segment SGI167962[] = {{&S60865[27], 6}, {NULL}};
const Segment SGI167963[] = {{&S60865[99], 9}, {NULL}};
const Segment SGI167964[] = {{&S60865[72], 6}, {NULL}};
const Segment SGI167965[] = {{&S60866[52], 6}, {NULL}};
const Segment SGI167966[] = {{&S60866[114], 6}, {NULL}};
const Segment SGI167967[] = {{&S60867[44], 15}, {NULL}};
const Segment SGI167968[] = {{&S60867[44], 33}, {NULL}};
const Segment SGI167969[] = {{&S60867[30], 6}, {NULL}};
const Segment SGI167970[] = {{&S60867[94], 6}, {NULL}};
const Segment SGI167971[] = {{&S60868[44], 15}, {NULL}};
const Segment SGI167972[] = {{&S60868[44], 33}, {NULL}};
const Segment SGI167973[] = {{&S60868[30], 6}, {NULL}};
const Segment SGI167974[] = {{&S60868[31], 3}, {NULL}};
const Segment SGI167975[] = {{&S60868[91], 6}, {NULL}};
const Segment SGI167976[] = {{&S60869[44], 43}, {NULL}};
const Segment SGI167977[] = {{&S60869[30], 6}, {NULL}};
const Segment SGI167978[] = {{&S60869[119], 6}, {NULL}};
const Segment SGI167979[] = {{&S60870[44], 14}, {NULL}};
const Segment SGI167980[] = {{&S60870[44], 31}, {NULL}};
const Segment SGI167981[] = {{&S60870[30], 6}, {NULL}};
const Segment SGI167982[] = {{&S53164[44], 10}, {NULL}};
const Segment SGI167983[] = {{&S53164[44], 22}, {NULL}};
const Segment SGI167984[] = {{&S53164[30], 6}, {NULL}};
const Segment SGI167985[] = {{&S60870[92], 6}, {NULL}};
const Segment SGI167986[] = {{&S53164[115], 3}, {NULL}};
const Segment SGI167987[] = {{&S53164[80], 6}, {NULL}};
const Segment SGI167988[] = {{&S53164[140], 7}, {NULL}};
const Segment SGI167989[] = {{&S37867[41], 18}, {NULL}};
const Segment SGI167990[] = {{&S37867[41], 29}, {NULL}};
const Segment SGI167991[] = {{&S37867[27], 6}, {NULL}};
const Segment SGI167992[] = {{&S37867[119], 3}, {NULL}};
const Segment SGI167993[] = {{&S60871[44], 43}, {NULL}};
const Segment SGI167994[] = {{&S37867[123], 9}, {NULL}};
const Segment SGI167995[] = {{&S60871[30], 6}, {NULL}};
const Segment SGI167996[] = {{&S37867[84], 6}, {NULL}};
const Segment SGI167997[] = {{&S60871[119], 6}, {NULL}};
const Segment SGI167998[] = {{&S37868[41], 31}, {NULL}};
const Segment SGI167999[] = {{&S37868[41], 42}, {NULL}};
const Segment SGI168000[] = {{&S37868[27], 6}, {NULL}};
const Segment SGI168001[] = {{&S60872[38], 15}, {NULL}};
const Segment SGI168002[] = {{&S60872[31], 6}, {NULL}};
const Segment SGI168003[] = {{&S37868[132], 3}, {NULL}};
const Segment SGI168004[] = {{&S37868[136], 9}, {NULL}};
const Segment SGI168005[] = {{&S37868[97], 6}, {NULL}};
const Segment SGI168006[] = {{&S60873[44], 39}, {NULL}};
const Segment SGI168007[] = {{&S37869[41], 31}, {NULL}};
const Segment SGI168008[] = {{&S60873[30], 6}, {NULL}};
const Segment SGI168009[] = {{&S37869[41], 42}, {NULL}};
const Segment SGI168010[] = {{&S37869[27], 6}, {NULL}};
const Segment SGI168011[] = {{&S60873[115], 6}, {NULL}};
const Segment SGI168012[] = {{&S37869[132], 3}, {NULL}};
const Segment SGI168013[] = {{&S37869[136], 9}, {NULL}};
const Segment SGI168014[] = {{&S37869[97], 6}, {NULL}};
const Segment SGI168015[] = {{&S60874[44], 14}, {NULL}};
const Segment SGI168016[] = {{&S60874[44], 31}, {NULL}};
const Segment SGI168017[] = {{&S60874[30], 6}, {NULL}};
const Segment SGI168018[] = {{&S37870[41], 19}, {NULL}};
const Segment SGI168019[] = {{&S60874[31], 3}, {NULL}};
const Segment SGI168020[] = {{&S37870[41], 30}, {NULL}};
const Segment SGI168021[] = {{&S37870[27], 6}, {NULL}};
const Segment SGI168022[] = {{&S60874[89], 6}, {NULL}};
const Segment SGI168023[] = {{&S37870[120], 3}, {NULL}};
const Segment SGI168024[] = {{&S37870[124], 9}, {NULL}};
const Segment SGI168025[] = {{&S37870[85], 6}, {NULL}};
const Segment SGI168026[] = {{&S60875[44], 39}, {NULL}};
const Segment SGI168027[] = {{&S60875[30], 6}, {NULL}};
const Segment SGI168028[] = {{&S37871[41], 31}, {NULL}};
const Segment SGI168029[] = {{&S37871[41], 42}, {NULL}};
const Segment SGI168030[] = {{&S37871[27], 6}, {NULL}};
const Segment SGI168031[] = {{&S60875[115], 6}, {NULL}};
const Segment SGI168032[] = {{&S37871[132], 3}, {NULL}};
const Segment SGI168033[] = {{&S37871[136], 9}, {NULL}};
const Segment SGI168034[] = {{&S37871[97], 6}, {NULL}};
const Segment SGI168035[] = {{&S60876[44], 39}, {NULL}};
const Segment SGI168036[] = {{&S60876[30], 6}, {NULL}};
const Segment SGI168037[] = {{&S37872[41], 19}, {NULL}};
const Segment SGI168038[] = {{&S37872[41], 30}, {NULL}};
const Segment SGI168039[] = {{&S60876[115], 6}, {NULL}};
const Segment SGI168040[] = {{&S37872[27], 6}, {NULL}};
const Segment SGI168041[] = {{&S37872[120], 3}, {NULL}};
const Segment SGI168042[] = {{&S37872[124], 9}, {NULL}};
const Segment SGI168043[] = {{&S37872[85], 6}, {NULL}};
const Segment SGI168044[] = {{&S60877[38], 14}, {NULL}};
const Segment SGI168045[] = {{&S60877[31], 6}, {NULL}};
const Segment SGI168046[] = {{&S37873[41], 31}, {NULL}};
const Segment SGI168047[] = {{&S37873[41], 42}, {NULL}};
const Segment SGI168048[] = {{&S37873[27], 6}, {NULL}};
const Segment SGI168049[] = {{&S37873[132], 3}, {NULL}};
const Segment SGI168050[] = {{&S37873[136], 9}, {NULL}};
const Segment SGI168051[] = {{&S37873[97], 6}, {NULL}};
const Segment SGI168052[] = {{&S60878[34], 10}, {NULL}};
const Segment SGI168053[] = {{&S60878[27], 6}, {NULL}};
const Segment SGI168054[] = {{&S60878[97], 9}, {NULL}};
const Segment SGI168055[] = {{&S60878[70], 6}, {NULL}};
const Segment SGI168056[] = {{&S37874[41], 19}, {NULL}};
const Segment SGI168057[] = {{&S37874[41], 30}, {NULL}};
const Segment SGI168058[] = {{&S37874[27], 6}, {NULL}};
const Segment SGI168059[] = {{&S37874[120], 3}, {NULL}};
const Segment SGI168060[] = {{&S37874[124], 9}, {NULL}};
const Segment SGI168061[] = {{&S37874[85], 6}, {NULL}};
const Segment SGI168062[] = {{&S37875[41], 25}, {NULL}};
const Segment SGI168063[] = {{&S37875[41], 36}, {NULL}};
const Segment SGI168064[] = {{&S37875[27], 6}, {NULL}};
const Segment SGI168065[] = {{&S37875[126], 3}, {NULL}};
const Segment SGI168066[] = {{&S37875[130], 9}, {NULL}};
const Segment SGI168067[] = {{&S37875[91], 6}, {NULL}};
const Segment SGI168068[] = {{&S60887[44], 10}, {NULL}};
const Segment SGI168069[] = {{&S60887[44], 22}, {NULL}};
const Segment SGI168070[] = {{&S60887[30], 6}, {NULL}};
const Segment SGI168071[] = {{&S60887[31], 3}, {NULL}};
const Segment SGI168072[] = {{&S60887[80], 6}, {NULL}};
const Segment SGI168073[] = {{&S60888[38], 10}, {NULL}};
const Segment SGI168074[] = {{&S60888[31], 6}, {NULL}};
const Segment SGI168075[] = {{&S60902[34], 7}, {NULL}};
const Segment SGI168076[] = {{&S60902[27], 6}, {NULL}};
const Segment SGI168077[] = {{&S60902[96], 9}, {NULL}};
const Segment SGI168078[] = {{&S60902[69], 6}, {NULL}};
const Segment SGI168079[] = {{&S60904[34], 7}, {NULL}};
const Segment SGI168080[] = {{&S60904[27], 6}, {NULL}};
const Segment SGI168081[] = {{&S60904[100], 9}, {NULL}};
const Segment SGI168082[] = {{&S60904[73], 6}, {NULL}};
const Segment SGI168083[] = {{&S50655[34], 10}, {NULL}};
const Segment SGI168084[] = {{&S50655[27], 6}, {NULL}};
const Segment SGI168085[] = {{&S50655[97], 9}, {NULL}};
const Segment SGI168086[] = {{&S50655[70], 6}, {NULL}};
const Segment SGI168087[] = {{&S60905[34], 7}, {NULL}};
const Segment SGI168088[] = {{&S60905[27], 6}, {NULL}};
const Segment SGI168089[] = {{&S60905[100], 9}, {NULL}};
const Segment SGI168090[] = {{&S60905[73], 6}, {NULL}};
const Segment SGI168091[] = {{&S53214[44], 12}, {NULL}};
const Segment SGI168092[] = {{&S53214[44], 24}, {NULL}};
const Segment SGI168093[] = {{&S53214[30], 6}, {NULL}};
const Segment SGI168094[] = {{&S53214[117], 3}, {NULL}};
const Segment SGI168095[] = {{&S53214[82], 6}, {NULL}};
const Segment SGI168096[] = {{&S50660[34], 14}, {NULL}};
const Segment SGI168097[] = {{&S53216[44], 12}, {NULL}};
const Segment SGI168098[] = {{&S53216[44], 24}, {NULL}};
const Segment SGI168099[] = {{&S50660[27], 6}, {NULL}};
const Segment SGI168100[] = {{&S53216[30], 6}, {NULL}};
const Segment SGI168101[] = {{&S53216[117], 3}, {NULL}};
const Segment SGI168102[] = {{&S50660[101], 9}, {NULL}};
const Segment SGI168103[] = {{&S50660[74], 6}, {NULL}};
const Segment SGI168104[] = {{&S53216[82], 6}, {NULL}};
const Segment SGI168105[] = {{&S53217[44], 12}, {NULL}};
const Segment SGI168106[] = {{&S53217[44], 24}, {NULL}};
const Segment SGI168107[] = {{&S53217[30], 6}, {NULL}};
const Segment SGI168108[] = {{&S53217[117], 3}, {NULL}};
const Segment SGI168109[] = {{&S53217[82], 6}, {NULL}};
const Segment SGI168110[] = {{&S53219[44], 12}, {NULL}};
const Segment SGI168111[] = {{&S53219[44], 25}, {NULL}};
const Segment SGI168112[] = {{&S53219[30], 6}, {NULL}};
const Segment SGI168113[] = {{&S53219[118], 3}, {NULL}};
const Segment SGI168114[] = {{&S53219[83], 6}, {NULL}};
const Segment SGI168115[] = {{&S53220[44], 17}, {NULL}};
const Segment SGI168116[] = {{&S53220[44], 29}, {NULL}};
const Segment SGI168117[] = {{&S53220[30], 6}, {NULL}};
const Segment SGI168118[] = {{&S53220[122], 3}, {NULL}};
const Segment SGI168119[] = {{&S53220[87], 6}, {NULL}};
const Segment SGI168120[] = {{&S60914[34], 7}, {NULL}};
const Segment SGI168121[] = {{&S60914[27], 6}, {NULL}};
const Segment SGI168122[] = {{&S60914[112], 9}, {NULL}};
const Segment SGI168123[] = {{&S60914[85], 6}, {NULL}};
const Segment SGI168124[] = {{&S50669[34], 11}, {NULL}};
const Segment SGI168125[] = {{&S50669[27], 6}, {NULL}};
const Segment SGI168126[] = {{&S50669[98], 9}, {NULL}};
const Segment SGI168127[] = {{&S50669[71], 6}, {NULL}};
const Segment SGI168128[] = {{&S50671[44], 10}, {NULL}};
const Segment SGI168129[] = {{&S58003[44], 10}, {NULL}};
const Segment SGI168130[] = {{&S50671[44], 22}, {NULL}};
const Segment SGI168131[] = {{&S58003[44], 22}, {NULL}};
const Segment SGI168132[] = {{&S50671[30], 6}, {NULL}};
const Segment SGI168133[] = {{&S58003[30], 6}, {NULL}};
const Segment SGI168134[] = {{&S58003[31], 3}, {NULL}};
const Segment SGI168135[] = {{&S58003[80], 6}, {NULL}};
const Segment SGI168136[] = {{&S50671[131], 3}, {NULL}};
const Segment SGI168137[] = {{&S50671[96], 6}, {NULL}};
const Segment SGI168138[] = {{&S50671[68], 8}, {NULL}};
const Segment SGI168139[] = {{&S50677[44], 11}, {NULL}};
const Segment SGI168140[] = {{&S50677[44], 23}, {NULL}};
const Segment SGI168141[] = {{&S50677[30], 6}, {NULL}};
const Segment SGI168142[] = {{&S50677[132], 3}, {NULL}};
const Segment SGI168143[] = {{&S50677[97], 6}, {NULL}};
const Segment SGI168144[] = {{&S50677[69], 8}, {NULL}};
const Segment SGI168145[] = {{&S50679[44], 10}, {NULL}};
const Segment SGI168146[] = {{&S50679[44], 22}, {NULL}};
const Segment SGI168147[] = {{&S50679[30], 6}, {NULL}};
const Segment SGI168148[] = {{&S50679[134], 3}, {NULL}};
const Segment SGI168149[] = {{&S50679[99], 6}, {NULL}};
const Segment SGI168150[] = {{&S50679[68], 8}, {NULL}};
const Segment SGI168151[] = {{&S50680[44], 11}, {NULL}};
const Segment SGI168152[] = {{&S50680[44], 23}, {NULL}};
const Segment SGI168153[] = {{&S50680[30], 6}, {NULL}};
const Segment SGI168154[] = {{&S50680[132], 3}, {NULL}};
const Segment SGI168155[] = {{&S50680[97], 6}, {NULL}};
const Segment SGI168156[] = {{&S50680[69], 8}, {NULL}};
const Segment SGI168157[] = {{&S50681[42], 11}, {NULL}};
const Segment SGI168158[] = {{&S50681[30], 6}, {NULL}};
const Segment SGI168159[] = {{&S50681[130], 3}, {NULL}};
const Segment SGI168160[] = {{&S50681[95], 6}, {NULL}};
const Segment SGI168161[] = {{&S50681[67], 8}, {NULL}};
const Segment SGI168162[] = {{&S60943[44], 8}, {NULL}};
const Segment SGI168163[] = {{&S60943[44], 20}, {NULL}};
const Segment SGI168164[] = {{&S60943[30], 6}, {NULL}};
const Segment SGI168165[] = {{&S60943[31], 3}, {NULL}};
const Segment SGI168166[] = {{&S60943[78], 6}, {NULL}};
const Segment SGI168167[] = {{&S60945[38], 8}, {NULL}};
const Segment SGI168168[] = {{&S60945[31], 6}, {NULL}};
const Segment SGI168169[] = {{&S60959[44], 7}, {NULL}};
const Segment SGI168170[] = {{&S60959[44], 19}, {NULL}};
const Segment SGI168171[] = {{&S60959[30], 6}, {NULL}};
const Segment SGI168172[] = {{&S60959[31], 3}, {NULL}};
const Segment SGI168173[] = {{&S60959[77], 6}, {NULL}};
const Segment SGI168174[] = {{&S60960[38], 7}, {NULL}};
const Segment SGI168175[] = {{&S60960[31], 6}, {NULL}};
const Segment SGI168176[] = {{&S50717[44], 8}, {NULL}};
const Segment SGI168177[] = {{&S50717[44], 20}, {NULL}};
const Segment SGI168178[] = {{&S50717[30], 6}, {NULL}};
const Segment SGI168179[] = {{&S50717[31], 3}, {NULL}};
const Segment SGI168180[] = {{&S50717[78], 6}, {NULL}};
const Segment SGI168181[] = {{&S50718[44], 8}, {NULL}};
const Segment SGI168182[] = {{&S50718[44], 20}, {NULL}};
const Segment SGI168183[] = {{&S50718[30], 6}, {NULL}};
const Segment SGI168184[] = {{&S50718[113], 3}, {NULL}};
const Segment SGI168185[] = {{&S50718[78], 6}, {NULL}};
const Segment SGI168186[] = {{&S50719[38], 8}, {NULL}};
const Segment SGI168187[] = {{&S50719[31], 6}, {NULL}};
const Segment SGI168188[] = {{&S53288[34], 10}, {NULL}};
const Segment SGI168189[] = {{&S53288[27], 6}, {NULL}};
const Segment SGI168190[] = {{&S53288[97], 9}, {NULL}};
const Segment SGI168191[] = {{&S53288[70], 6}, {NULL}};
const Segment SGI168192[] = {{&S53290[34], 11}, {NULL}};
const Segment SGI168193[] = {{&S53290[27], 6}, {NULL}};
const Segment SGI168194[] = {{&S53290[98], 9}, {NULL}};
const Segment SGI168195[] = {{&S53290[71], 6}, {NULL}};
const Segment SGI168196[] = {{&S60989[44], 13}, {NULL}};
const Segment SGI168197[] = {{&S60989[44], 29}, {NULL}};
const Segment SGI168198[] = {{&S60989[30], 6}, {NULL}};
const Segment SGI168199[] = {{&S60989[31], 3}, {NULL}};
const Segment SGI168200[] = {{&S60989[87], 6}, {NULL}};
const Segment SGI168201[] = {{&S60998[44], 28}, {NULL}};
const Segment SGI168202[] = {{&S60998[44], 40}, {NULL}};
const Segment SGI168203[] = {{&S60998[30], 6}, {NULL}};
const Segment SGI168204[] = {{&S60998[31], 3}, {NULL}};
const Segment SGI168205[] = {{&S60998[117], 6}, {NULL}};
const Segment SGI168206[] = {{&S60998[86], 8}, {NULL}};
const Segment SGI168207[] = {{&S60999[44], 10}, {NULL}};
const Segment SGI168208[] = {{&S60999[44], 22}, {NULL}};
const Segment SGI168209[] = {{&S50751[34], 14}, {NULL}};
const Segment SGI168210[] = {{&S60999[30], 6}, {NULL}};
const Segment SGI168211[] = {{&S50751[27], 6}, {NULL}};
const Segment SGI168212[] = {{&S60999[31], 3}, {NULL}};
const Segment SGI168213[] = {{&S60999[95], 6}, {NULL}};
const Segment SGI168214[] = {{&S50751[111], 9}, {NULL}};
const Segment SGI168215[] = {{&S50751[84], 6}, {NULL}};
const Segment SGI168216[] = {{&S60999[68], 8}, {NULL}};
const Segment SGI168217[] = {{&S50752[34], 14}, {NULL}};
const Segment SGI168218[] = {{&S61000[44], 10}, {NULL}};
const Segment SGI168219[] = {{&S61000[44], 22}, {NULL}};
const Segment SGI168220[] = {{&S50752[27], 6}, {NULL}};
const Segment SGI168221[] = {{&S61000[30], 6}, {NULL}};
const Segment SGI168222[] = {{&S61000[31], 3}, {NULL}};
const Segment SGI168223[] = {{&S50752[111], 9}, {NULL}};
const Segment SGI168224[] = {{&S50752[84], 6}, {NULL}};
const Segment SGI168225[] = {{&S61000[99], 6}, {NULL}};
const Segment SGI168226[] = {{&S61000[68], 8}, {NULL}};
const Segment SGI168227[] = {{&S61001[44], 10}, {NULL}};
const Segment SGI168228[] = {{&S61001[44], 22}, {NULL}};
const Segment SGI168229[] = {{&S61001[30], 6}, {NULL}};
const Segment SGI168230[] = {{&S61001[31], 3}, {NULL}};
const Segment SGI168231[] = {{&S61001[95], 6}, {NULL}};
const Segment SGI168232[] = {{&S61001[68], 8}, {NULL}};
const Segment SGI168233[] = {{&S50755[34], 10}, {NULL}};
const Segment SGI168234[] = {{&S50755[27], 6}, {NULL}};
const Segment SGI168235[] = {{&S61002[44], 5}, {NULL}};
const Segment SGI168236[] = {{&S50755[97], 9}, {NULL}};
const Segment SGI168237[] = {{&S61002[44], 17}, {NULL}};
const Segment SGI168238[] = {{&S50755[70], 6}, {NULL}};
const Segment SGI168239[] = {{&S61002[30], 6}, {NULL}};
const Segment SGI168240[] = {{&S61002[31], 3}, {NULL}};
const Segment SGI168241[] = {{&S61002[94], 6}, {NULL}};
const Segment SGI168242[] = {{&S61002[63], 8}, {NULL}};
const Segment SGI168243[] = {{&S61003[44], 6}, {NULL}};
const Segment SGI168244[] = {{&S61003[44], 18}, {NULL}};
const Segment SGI168245[] = {{&S61003[30], 6}, {NULL}};
const Segment SGI168246[] = {{&S61003[31], 3}, {NULL}};
const Segment SGI168247[] = {{&S61003[95], 6}, {NULL}};
const Segment SGI168248[] = {{&S61003[64], 8}, {NULL}};
const Segment SGI168249[] = {{&S61004[44], 5}, {NULL}};
const Segment SGI168250[] = {{&S61004[44], 17}, {NULL}};
const Segment SGI168251[] = {{&S61004[30], 6}, {NULL}};
const Segment SGI168252[] = {{&S61004[31], 3}, {NULL}};
const Segment SGI168253[] = {{&S61004[90], 6}, {NULL}};
const Segment SGI168254[] = {{&S61004[63], 8}, {NULL}};
const Segment SGI168255[] = {{&S61005[44], 7}, {NULL}};
const Segment SGI168256[] = {{&S61005[44], 19}, {NULL}};
const Segment SGI168257[] = {{&S61005[30], 6}, {NULL}};
const Segment SGI168258[] = {{&S61005[31], 3}, {NULL}};
const Segment SGI168259[] = {{&S61005[96], 6}, {NULL}};
const Segment SGI168260[] = {{&S61005[65], 8}, {NULL}};
const Segment SGI168261[] = {{&S61006[44], 16}, {NULL}};
const Segment SGI168262[] = {{&S61006[44], 28}, {NULL}};
const Segment SGI168263[] = {{&S61006[30], 6}, {NULL}};
const Segment SGI168264[] = {{&S61006[31], 3}, {NULL}};
const Segment SGI168265[] = {{&S61006[128], 6}, {NULL}};
const Segment SGI168266[] = {{&S61006[74], 8}, {NULL}};
const Segment SGI168267[] = {{&S61007[44], 7}, {NULL}};
const Segment SGI168268[] = {{&S61007[44], 19}, {NULL}};
const Segment SGI168269[] = {{&S61007[30], 6}, {NULL}};
const Segment SGI168270[] = {{&S61007[31], 3}, {NULL}};
const Segment SGI168271[] = {{&S61007[92], 6}, {NULL}};
const Segment SGI168272[] = {{&S61007[65], 8}, {NULL}};
const Segment SGI168273[] = {{&S61008[44], 6}, {NULL}};
const Segment SGI168274[] = {{&S61008[44], 18}, {NULL}};
const Segment SGI168275[] = {{&S61008[30], 6}, {NULL}};
const Segment SGI168276[] = {{&S61008[31], 3}, {NULL}};
const Segment SGI168277[] = {{&S61008[95], 6}, {NULL}};
const Segment SGI168278[] = {{&S61008[64], 8}, {NULL}};
const Segment SGI168279[] = {{&S61009[44], 5}, {NULL}};
const Segment SGI168280[] = {{&S61009[44], 17}, {NULL}};
const Segment SGI168281[] = {{&S61009[30], 6}, {NULL}};
const Segment SGI168282[] = {{&S61009[31], 3}, {NULL}};
const Segment SGI168283[] = {{&S61009[94], 6}, {NULL}};
const Segment SGI168284[] = {{&S61009[63], 8}, {NULL}};
const Segment SGI168285[] = {{&S47471[44], 10}, {NULL}};
const Segment SGI168286[] = {{&S47471[44], 22}, {NULL}};
const Segment SGI168287[] = {{&S61010[44], 8}, {NULL}};
const Segment SGI168288[] = {{&S61010[44], 20}, {NULL}};
const Segment SGI168289[] = {{&S61010[30], 6}, {NULL}};
const Segment SGI168290[] = {{&S61010[31], 3}, {NULL}};
const Segment SGI168291[] = {{&S61010[93], 6}, {NULL}};
const Segment SGI168292[] = {{&S61010[66], 8}, {NULL}};
const Segment SGI168293[] = {{&S61011[34], 10}, {NULL}};
const Segment SGI168294[] = {{&S61011[27], 6}, {NULL}};
const Segment SGI168295[] = {{&S61011[97], 9}, {NULL}};
const Segment SGI168296[] = {{&S61011[70], 6}, {NULL}};
const Segment SGI168297[] = {{&S38044[38], 10}, {NULL}};
const Segment SGI168298[] = {{&S38044[31], 6}, {NULL}};
const Segment SGI168299[] = {{&S38045[44], 10}, {NULL}};
const Segment SGI168300[] = {{&S38045[44], 21}, {NULL}};
const Segment SGI168301[] = {{&S38045[30], 6}, {NULL}};
const Segment SGI168302[] = {{&S38045[114], 3}, {NULL}};
const Segment SGI168303[] = {{&S38045[79], 6}, {NULL}};
const Segment SGI168304[] = {{&S61023[44], 8}, {NULL}};
const Segment SGI168305[] = {{&S61023[44], 20}, {NULL}};
const Segment SGI168306[] = {{&S61023[30], 6}, {NULL}};
const Segment SGI168307[] = {{&S38046[44], 11}, {NULL}};
const Segment SGI168308[] = {{&S38046[44], 22}, {NULL}};
const Segment SGI168309[] = {{&S38046[30], 6}, {NULL}};
const Segment SGI168310[] = {{&S61023[31], 3}, {NULL}};
const Segment SGI168311[] = {{&S61023[78], 6}, {NULL}};
const Segment SGI168312[] = {{&S38046[115], 3}, {NULL}};
const Segment SGI168313[] = {{&S38046[80], 6}, {NULL}};
const Segment SGI168314[] = {{&S61024[44], 14}, {NULL}};
const Segment SGI168315[] = {{&S38047[44], 10}, {NULL}};
const Segment SGI168316[] = {{&S38047[44], 22}, {NULL}};
const Segment SGI168317[] = {{&S61024[44], 26}, {NULL}};
const Segment SGI168318[] = {{&S61024[30], 6}, {NULL}};
const Segment SGI168319[] = {{&S38047[30], 6}, {NULL}};
const Segment SGI168320[] = {{&S61024[31], 3}, {NULL}};
const Segment SGI168321[] = {{&S38047[115], 3}, {NULL}};
const Segment SGI168322[] = {{&S61024[84], 6}, {NULL}};
const Segment SGI168323[] = {{&S38047[80], 6}, {NULL}};
const Segment SGI168324[] = {{&S38048[44], 11}, {NULL}};
const Segment SGI168325[] = {{&S38048[44], 23}, {NULL}};
const Segment SGI168326[] = {{&S38048[30], 6}, {NULL}};
const Segment SGI168327[] = {{&S38048[116], 3}, {NULL}};
const Segment SGI168328[] = {{&S38048[81], 6}, {NULL}};
const Segment SGI168329[] = {{&S57445[6], 4}, {NULL}};
const Segment SGI168330[] = {{&S57445[81], 5}, {NULL}};
const Segment SGI168331[] = {{&S38049[44], 10}, {NULL}};
const Segment SGI168332[] = {{&S38049[44], 21}, {NULL}};
const Segment SGI168333[] = {{&S38049[30], 6}, {NULL}};
const Segment SGI168334[] = {{&S38049[114], 3}, {NULL}};
const Segment SGI168335[] = {{&S38049[79], 6}, {NULL}};
const Segment SGI168336[] = {{&S38050[38], 11}, {NULL}};
const Segment SGI168337[] = {{&S38050[31], 6}, {NULL}};
const Segment SGI168338[] = {{&S53362[11], 1}, {NULL}};
const Segment SGI168339[] = {{&S61030[44], 10}, {NULL}};
const Segment SGI168340[] = {{&S61030[44], 21}, {NULL}};
const Segment SGI168341[] = {{&S53362[27], 6}, {NULL}};
const Segment SGI168342[] = {{&S61030[30], 6}, {NULL}};
const Segment SGI168343[] = {{&S61030[31], 3}, {NULL}};
const Segment SGI168344[] = {{&S53362[88], 9}, {NULL}};
const Segment SGI168345[] = {{&S53362[61], 6}, {NULL}};
const Segment SGI168346[] = {{&S61030[79], 6}, {NULL}};
const Segment SGI168347[] = {{&S38051[44], 10}, {NULL}};
const Segment SGI168348[] = {{&S38051[44], 56}, {NULL}};
const Segment SGI168349[] = {{&S38051[30], 6}, {NULL}};
const Segment SGI168350[] = {{&S38051[149], 3}, {NULL}};
const Segment SGI168351[] = {{&S38051[114], 6}, {NULL}};
const Segment SGI168352[] = {{&S61031[44], 10}, {NULL}};
const Segment SGI168353[] = {{&S61031[44], 21}, {NULL}};
const Segment SGI168354[] = {{&S61031[30], 6}, {NULL}};
const Segment SGI168355[] = {{&S61031[31], 3}, {NULL}};
const Segment SGI168356[] = {{&S61031[79], 6}, {NULL}};
const Segment SGI168357[] = {{&S53364[34], 25}, {NULL}};
const Segment SGI168358[] = {{&S53364[27], 6}, {NULL}};
const Segment SGI168359[] = {{&S53364[112], 9}, {NULL}};
const Segment SGI168360[] = {{&S53364[85], 6}, {NULL}};
const Segment SGI168361[] = {{&S61032[38], 10}, {NULL}};
const Segment SGI168362[] = {{&S61032[31], 6}, {NULL}};
const Segment SGI168363[] = {{&S61033[44], 10}, {NULL}};
const Segment SGI168364[] = {{&S61033[44], 21}, {NULL}};
const Segment SGI168365[] = {{&S53366[11], 1}, {NULL}};
const Segment SGI168366[] = {{&S61033[30], 6}, {NULL}};
const Segment SGI168367[] = {{&S53366[27], 6}, {NULL}};
const Segment SGI168368[] = {{&S61033[31], 3}, {NULL}};
const Segment SGI168369[] = {{&S61033[79], 6}, {NULL}};
const Segment SGI168370[] = {{&S53366[88], 9}, {NULL}};
const Segment SGI168371[] = {{&S53366[61], 6}, {NULL}};
const Segment SGI168372[] = {{&S57455[41], 10}, {NULL}};
const Segment SGI168373[] = {{&S57455[41], 25}, {NULL}};
const Segment SGI168374[] = {{&S57455[29], 4}, {NULL}};
const Segment SGI168375[] = {{&S57455[114], 3}, {NULL}};
const Segment SGI168376[] = {{&S57455[80], 6}, {NULL}};
const Segment SGI168377[] = {{&S53367[34], 7}, {NULL}};
const Segment SGI168378[] = {{&S61034[44], 13}, {NULL}};
const Segment SGI168379[] = {{&S61034[44], 25}, {NULL}};
const Segment SGI168380[] = {{&S53367[27], 6}, {NULL}};
const Segment SGI168381[] = {{&S61034[30], 6}, {NULL}};
const Segment SGI168382[] = {{&S61034[31], 3}, {NULL}};
const Segment SGI168383[] = {{&S53367[94], 9}, {NULL}};
const Segment SGI168384[] = {{&S53367[67], 6}, {NULL}};
const Segment SGI168385[] = {{&S61034[83], 6}, {NULL}};
const Segment SGI168386[] = {{&S53368[34], 31}, {NULL}};
const Segment SGI168387[] = {{&S61035[44], 13}, {NULL}};
const Segment SGI168388[] = {{&S61035[44], 25}, {NULL}};
const Segment SGI168389[] = {{&S53368[27], 6}, {NULL}};
const Segment SGI168390[] = {{&S61035[30], 6}, {NULL}};
const Segment SGI168391[] = {{&S61035[31], 3}, {NULL}};
const Segment SGI168392[] = {{&S53368[118], 9}, {NULL}};
const Segment SGI168393[] = {{&S61035[83], 6}, {NULL}};
const Segment SGI168394[] = {{&S53368[91], 6}, {NULL}};
const Segment SGI168395[] = {{&S53369[34], 16}, {NULL}};
const Segment SGI168396[] = {{&S53369[27], 6}, {NULL}};
const Segment SGI168397[] = {{&S53369[103], 9}, {NULL}};
const Segment SGI168398[] = {{&S53369[76], 6}, {NULL}};
const Segment SGI168399[] = {{&S53370[34], 27}, {NULL}};
const Segment SGI168400[] = {{&S53370[27], 6}, {NULL}};
const Segment SGI168401[] = {{&S53370[114], 9}, {NULL}};
const Segment SGI168402[] = {{&S53370[87], 6}, {NULL}};
const Segment SGI168403[] = {{&S61039[34], 7}, {NULL}};
const Segment SGI168404[] = {{&S61039[27], 6}, {NULL}};
const Segment SGI168405[] = {{&S61039[96], 9}, {NULL}};
const Segment SGI168406[] = {{&S61039[69], 6}, {NULL}};
const Segment SGI168407[] = {{&S53371[34], 27}, {NULL}};
const Segment SGI168408[] = {{&S53371[27], 6}, {NULL}};
const Segment SGI168409[] = {{&S53371[114], 9}, {NULL}};
const Segment SGI168410[] = {{&S53371[87], 6}, {NULL}};
const Segment SGI168411[] = {{&S53372[34], 25}, {NULL}};
const Segment SGI168412[] = {{&S53372[27], 6}, {NULL}};
const Segment SGI168413[] = {{&S53372[112], 9}, {NULL}};
const Segment SGI168414[] = {{&S53372[85], 6}, {NULL}};
const Segment SGI168415[] = {{&S53373[34], 20}, {NULL}};
const Segment SGI168416[] = {{&S53373[27], 6}, {NULL}};
const Segment SGI168417[] = {{&S53373[107], 9}, {NULL}};
const Segment SGI168418[] = {{&S53373[80], 6}, {NULL}};
const Segment SGI168419[] = {{&S53376[34], 31}, {NULL}};
const Segment SGI168420[] = {{&S53376[27], 6}, {NULL}};
const Segment SGI168421[] = {{&S53376[118], 9}, {NULL}};
const Segment SGI168422[] = {{&S53376[91], 6}, {NULL}};
const Segment SGI168423[] = {{&S53377[34], 18}, {NULL}};
const Segment SGI168424[] = {{&S53377[27], 6}, {NULL}};
const Segment SGI168425[] = {{&S53377[105], 9}, {NULL}};
const Segment SGI168426[] = {{&S53377[78], 6}, {NULL}};
const Segment SGI168427[] = {{&S53378[34], 21}, {NULL}};
const Segment SGI168428[] = {{&S53378[27], 6}, {NULL}};
const Segment SGI168429[] = {{&S53378[108], 9}, {NULL}};
const Segment SGI168430[] = {{&S53378[81], 6}, {NULL}};
const Segment SGI168431[] = {{&S57478[32], 9}, {NULL}};
const Segment SGI168432[] = {{&S57478[27], 4}, {NULL}};
const Segment SGI168433[] = {{&S57478[94], 9}, {NULL}};
const Segment SGI168434[] = {{&S57478[67], 6}, {NULL}};
const Segment SGI168435[] = {{&S61065[44], 6}, {NULL}};
const Segment SGI168436[] = {{&S61065[44], 18}, {NULL}};
const Segment SGI168437[] = {{&S61065[30], 6}, {NULL}};
const Segment SGI168438[] = {{&S61065[31], 3}, {NULL}};
const Segment SGI168439[] = {{&S61065[76], 6}, {NULL}};
const Segment SGI168440[] = {{&S61066[44], 10}, {NULL}};
const Segment SGI168441[] = {{&S61066[44], 22}, {NULL}};
const Segment SGI168442[] = {{&S61066[30], 6}, {NULL}};
const Segment SGI168443[] = {{&S61066[31], 3}, {NULL}};
const Segment SGI168444[] = {{&S61066[80], 6}, {NULL}};
const Segment SGI168445[] = {{&S61067[64], 6}, {NULL}};
const Segment SGI168446[] = {{&S61086[44], 11}, {NULL}};
const Segment SGI168447[] = {{&S61086[44], 23}, {NULL}};
const Segment SGI168448[] = {{&S61086[30], 6}, {NULL}};
const Segment SGI168449[] = {{&S61086[31], 3}, {NULL}};
const Segment SGI168450[] = {{&S61086[81], 6}, {NULL}};
const Segment SGI168451[] = {{&S53415[34], 8}, {NULL}};
const Segment SGI168452[] = {{&S53415[27], 6}, {NULL}};
const Segment SGI168453[] = {{&S53415[95], 9}, {NULL}};
const Segment SGI168454[] = {{&S53415[68], 6}, {NULL}};
const Segment SGI168455[] = {{&S61087[44], 5}, {NULL}};
const Segment SGI168456[] = {{&S61087[44], 17}, {NULL}};
const Segment SGI168457[] = {{&S61087[30], 6}, {NULL}};
const Segment SGI168458[] = {{&S61087[31], 3}, {NULL}};
const Segment SGI168459[] = {{&S61087[75], 6}, {NULL}};
const Segment SGI168460[] = {{&S61088[44], 10}, {NULL}};
const Segment SGI168461[] = {{&S61088[44], 22}, {NULL}};
const Segment SGI168462[] = {{&S61088[30], 6}, {NULL}};
const Segment SGI168463[] = {{&S61088[31], 3}, {NULL}};
const Segment SGI168464[] = {{&S61088[80], 6}, {NULL}};
const Segment SGI168465[] = {{&S61089[38], 5}, {NULL}};
const Segment SGI168466[] = {{&S61089[31], 6}, {NULL}};
const Segment SGI168467[] = {{&S47471[30], 6}, {NULL}};
const Segment SGI168468[] = {{&S47471[133], 3}, {NULL}};
const Segment SGI168469[] = {{&S47471[98], 6}, {NULL}};
const Segment SGI168470[] = {{&S61099[44], 9}, {NULL}};
const Segment SGI168471[] = {{&S61099[44], 21}, {NULL}};
const Segment SGI168472[] = {{&S47471[68], 8}, {NULL}};
const Segment SGI168473[] = {{&S61099[30], 6}, {NULL}};
const Segment SGI168474[] = {{&S61099[31], 3}, {NULL}};
const Segment SGI168475[] = {{&S61099[79], 6}, {NULL}};
const Segment SGI168476[] = {{&S61100[44], 19}, {NULL}};
const Segment SGI168477[] = {{&S61100[44], 31}, {NULL}};
const Segment SGI168478[] = {{&S61100[30], 6}, {NULL}};
const Segment SGI168479[] = {{&S61100[31], 3}, {NULL}};
const Segment SGI168480[] = {{&S61100[89], 6}, {NULL}};
const Segment SGI168481[] = {{&S61101[38], 9}, {NULL}};
const Segment SGI168482[] = {{&S61101[31], 6}, {NULL}};
const Segment SGI168483[] = {{&S61103[34], 10}, {NULL}};
const Segment SGI168484[] = {{&S61103[27], 6}, {NULL}};
const Segment SGI168485[] = {{&S61103[97], 9}, {NULL}};
const Segment SGI168486[] = {{&S61103[70], 6}, {NULL}};
const Segment SGI168487[] = {{&S47489[34], 29}, {NULL}};
const Segment SGI168488[] = {{&S47489[27], 6}, {NULL}};
const Segment SGI168489[] = {{&S47489[116], 9}, {NULL}};
const Segment SGI168490[] = {{&S47489[89], 6}, {NULL}};
const Segment SGI168491[] = {{&S53451[34], 10}, {NULL}};
const Segment SGI168492[] = {{&S53451[27], 6}, {NULL}};
const Segment SGI168493[] = {{&S61116[34], 10}, {NULL}};
const Segment SGI168494[] = {{&S61116[27], 6}, {NULL}};
const Segment SGI168495[] = {{&S53451[97], 9}, {NULL}};
const Segment SGI168496[] = {{&S53451[70], 6}, {NULL}};
const Segment SGI168497[] = {{&S61116[103], 9}, {NULL}};
const Segment SGI168498[] = {{&S61116[76], 6}, {NULL}};
const Segment SGI168499[] = {{&S61118[34], 12}, {NULL}};
const Segment SGI168500[] = {{&S61118[27], 6}, {NULL}};
const Segment SGI168501[] = {{&S61118[99], 9}, {NULL}};
const Segment SGI168502[] = {{&S61118[72], 6}, {NULL}};
const Segment SGI168503[] = {{&S61120[34], 12}, {NULL}};
const Segment SGI168504[] = {{&S61120[27], 6}, {NULL}};
const Segment SGI168505[] = {{&S61120[99], 9}, {NULL}};
const Segment SGI168506[] = {{&S61120[72], 6}, {NULL}};
const Segment SGI168507[] = {{&S53459[34], 7}, {NULL}};
const Segment SGI168508[] = {{&S53459[27], 6}, {NULL}};
const Segment SGI168509[] = {{&S53459[99], 9}, {NULL}};
const Segment SGI168510[] = {{&S53459[72], 6}, {NULL}};
const Segment SGI168511[] = {{&S53462[44], 7}, {NULL}};
const Segment SGI168512[] = {{&S53462[44], 19}, {NULL}};
const Segment SGI168513[] = {{&S53462[30], 6}, {NULL}};
const Segment SGI168514[] = {{&S53462[128], 3}, {NULL}};
const Segment SGI168515[] = {{&S53462[93], 6}, {NULL}};
const Segment SGI168516[] = {{&S53462[65], 8}, {NULL}};
const Segment SGI168517[] = {{&S53463[44], 7}, {NULL}};
const Segment SGI168518[] = {{&S53463[44], 19}, {NULL}};
const Segment SGI168519[] = {{&S53463[30], 6}, {NULL}};
const Segment SGI168520[] = {{&S53463[128], 3}, {NULL}};
const Segment SGI168521[] = {{&S53463[93], 6}, {NULL}};
const Segment SGI168522[] = {{&S53463[65], 8}, {NULL}};
const Segment SGI168523[] = {{&S53464[44], 7}, {NULL}};
const Segment SGI168524[] = {{&S53464[44], 19}, {NULL}};
const Segment SGI168525[] = {{&S53464[30], 6}, {NULL}};
const Segment SGI168526[] = {{&S53464[128], 3}, {NULL}};
const Segment SGI168527[] = {{&S53464[93], 6}, {NULL}};
const Segment SGI168528[] = {{&S53464[65], 8}, {NULL}};
const Segment SGI168529[] = {{&S53475[34], 18}, {NULL}};
const Segment SGI168530[] = {{&S53475[27], 6}, {NULL}};
const Segment SGI168531[] = {{&S53475[105], 9}, {NULL}};
const Segment SGI168532[] = {{&S53475[78], 6}, {NULL}};
const Segment SGI168533[] = {{&S53476[34], 10}, {NULL}};
const Segment SGI168534[] = {{&S53476[27], 6}, {NULL}};
const Segment SGI168535[] = {{&S53476[97], 9}, {NULL}};
const Segment SGI168536[] = {{&S53476[70], 6}, {NULL}};
const Segment SGI168537[] = {{&S61148[34], 14}, {NULL}};
const Segment SGI168538[] = {{&S61148[27], 6}, {NULL}};
const Segment SGI168539[] = {{&S61148[101], 9}, {NULL}};
const Segment SGI168540[] = {{&S61148[74], 6}, {NULL}};
const Segment SGI168541[] = {{&S61150[44], 10}, {NULL}};
const Segment SGI168542[] = {{&S61150[44], 22}, {NULL}};
const Segment SGI168543[] = {{&S61150[30], 6}, {NULL}};
const Segment SGI168544[] = {{&S61150[31], 3}, {NULL}};
const Segment SGI168545[] = {{&S61150[80], 6}, {NULL}};
const Segment SGI168546[] = {{&S61151[38], 10}, {NULL}};
const Segment SGI168547[] = {{&S61151[31], 6}, {NULL}};
const Segment SGI168548[] = {{&S61166[44], 4}, {NULL}};
const Segment SGI168549[] = {{&S61166[44], 16}, {NULL}};
const Segment SGI168550[] = {{&S61166[30], 6}, {NULL}};
const Segment SGI168551[] = {{&S61166[31], 3}, {NULL}};
const Segment SGI168552[] = {{&S61166[74], 6}, {NULL}};
const Segment SGI168553[] = {{&S61182[34], 10}, {NULL}};
const Segment SGI168554[] = {{&S61182[27], 6}, {NULL}};
const Segment SGI168555[] = {{&S61182[97], 9}, {NULL}};
const Segment SGI168556[] = {{&S61182[70], 6}, {NULL}};
const Segment SGI168557[] = {{&S61191[44], 10}, {NULL}};
const Segment SGI168558[] = {{&S61191[44], 22}, {NULL}};
const Segment SGI168559[] = {{&S61191[30], 6}, {NULL}};
const Segment SGI168560[] = {{&S61191[31], 3}, {NULL}};
const Segment SGI168561[] = {{&S61191[95], 6}, {NULL}};
const Segment SGI168562[] = {{&S61191[68], 8}, {NULL}};
const Segment SGI168563[] = {{&S53545[44], 12}, {NULL}};
const Segment SGI168564[] = {{&S53545[44], 24}, {NULL}};
const Segment SGI168565[] = {{&S53545[30], 6}, {NULL}};
const Segment SGI168566[] = {{&S53545[117], 3}, {NULL}};
const Segment SGI168567[] = {{&S53545[82], 6}, {NULL}};
const Segment SGI168568[] = {{&S53546[38], 12}, {NULL}};
const Segment SGI168569[] = {{&S53546[31], 6}, {NULL}};
const Segment SGI168570[] = {{&S57658[32], 9}, {NULL}};
const Segment SGI168571[] = {{&S57658[27], 4}, {NULL}};
const Segment SGI168572[] = {{&S57658[94], 9}, {NULL}};
const Segment SGI168573[] = {{&S57658[67], 6}, {NULL}};
const Segment SGI168574[] = {{&S61229[34], 14}, {NULL}};
const Segment SGI168575[] = {{&S61229[27], 6}, {NULL}};
const Segment SGI168576[] = {{&S61229[101], 9}, {NULL}};
const Segment SGI168577[] = {{&S61229[74], 6}, {NULL}};
const Segment SGI168578[] = {{&S53556[34], 25}, {NULL}};
const Segment SGI168579[] = {{&S53556[27], 6}, {NULL}};
const Segment SGI168580[] = {{&S53556[112], 9}, {NULL}};
const Segment SGI168581[] = {{&S53556[85], 6}, {NULL}};
const Segment SGI168582[] = {{&S61278[44], 8}, {NULL}};
const Segment SGI168583[] = {{&S61278[44], 24}, {NULL}};
const Segment SGI168584[] = {{&S61278[30], 6}, {NULL}};
const Segment SGI168585[] = {{&S61278[82], 6}, {NULL}};
const Segment SGI168586[] = {{&S61278[109], 13}, {NULL}};
const Segment SGI168587[] = {{&S61279[44], 7}, {NULL}};
const Segment SGI168588[] = {{&S61279[44], 24}, {NULL}};
const Segment SGI168589[] = {{&S61279[30], 6}, {NULL}};
const Segment SGI168590[] = {{&S61279[82], 6}, {NULL}};
const Segment SGI168591[] = {{&S61279[109], 13}, {NULL}};
const Segment SGI168592[] = {{&S61280[44], 8}, {NULL}};
const Segment SGI168593[] = {{&S61280[44], 24}, {NULL}};
const Segment SGI168594[] = {{&S61280[30], 6}, {NULL}};
const Segment SGI168595[] = {{&S61280[82], 6}, {NULL}};
const Segment SGI168596[] = {{&S61280[109], 13}, {NULL}};
const Segment SGI168597[] = {{&S61281[44], 7}, {NULL}};
const Segment SGI168598[] = {{&S61281[44], 24}, {NULL}};
const Segment SGI168599[] = {{&S61281[30], 6}, {NULL}};
const Segment SGI168600[] = {{&S61281[82], 6}, {NULL}};
const Segment SGI168601[] = {{&S61281[109], 13}, {NULL}};
const Segment SGI168602[] = {{&S61282[44], 8}, {NULL}};
const Segment SGI168603[] = {{&S61282[44], 24}, {NULL}};
const Segment SGI168604[] = {{&S61282[30], 6}, {NULL}};
const Segment SGI168605[] = {{&S61282[82], 6}, {NULL}};
const Segment SGI168606[] = {{&S61282[109], 13}, {NULL}};
const Segment SGI168607[] = {{&S61283[44], 7}, {NULL}};
const Segment SGI168608[] = {{&S61283[44], 24}, {NULL}};
const Segment SGI168609[] = {{&S61283[30], 6}, {NULL}};
const Segment SGI168610[] = {{&S61283[82], 6}, {NULL}};
const Segment SGI168611[] = {{&S61283[109], 13}, {NULL}};
const Segment SGI168612[] = {{&S47666[34], 10}, {NULL}};
const Segment SGI168613[] = {{&S47666[27], 6}, {NULL}};
const Segment SGI168614[] = {{&S47666[97], 9}, {NULL}};
const Segment SGI168615[] = {{&S47666[70], 6}, {NULL}};
const Segment SGI168616[] = {{&S47667[34], 10}, {NULL}};
const Segment SGI168617[] = {{&S47667[27], 6}, {NULL}};
const Segment SGI168618[] = {{&S47667[97], 9}, {NULL}};
const Segment SGI168619[] = {{&S47667[70], 6}, {NULL}};
const Segment SGI168620[] = {{&S47668[34], 10}, {NULL}};
const Segment SGI168621[] = {{&S47668[27], 6}, {NULL}};
const Segment SGI168622[] = {{&S47668[97], 9}, {NULL}};
const Segment SGI168623[] = {{&S47668[70], 6}, {NULL}};
const Segment SGI168624[] = {{&S61300[34], 9}, {NULL}};
const Segment SGI168625[] = {{&S61300[27], 6}, {NULL}};
const Segment SGI168626[] = {{&S61300[96], 9}, {NULL}};
const Segment SGI168627[] = {{&S61300[69], 6}, {NULL}};
const Segment SGI168628[] = {{&S61302[34], 13}, {NULL}};
const Segment SGI168629[] = {{&S61302[27], 6}, {NULL}};
const Segment SGI168630[] = {{&S61302[100], 9}, {NULL}};
const Segment SGI168631[] = {{&S61302[73], 6}, {NULL}};
const Segment SGI168632[] = {{&S47676[44], 10}, {NULL}};
const Segment SGI168633[] = {{&S47676[44], 22}, {NULL}};
const Segment SGI168634[] = {{&S47676[30], 6}, {NULL}};
const Segment SGI168635[] = {{&S47676[144], 3}, {NULL}};
const Segment SGI168636[] = {{&S47676[109], 6}, {NULL}};
const Segment SGI168637[] = {{&S61304[34], 10}, {NULL}};
const Segment SGI168638[] = {{&S61304[27], 6}, {NULL}};
const Segment SGI168639[] = {{&S47677[44], 10}, {NULL}};
const Segment SGI168640[] = {{&S47677[44], 22}, {NULL}};
const Segment SGI168641[] = {{&S47677[30], 6}, {NULL}};
const Segment SGI168642[] = {{&S61304[97], 9}, {NULL}};
const Segment SGI168643[] = {{&S61304[70], 6}, {NULL}};
const Segment SGI168644[] = {{&S47677[144], 3}, {NULL}};
const Segment SGI168645[] = {{&S47677[109], 6}, {NULL}};
const Segment SGI168646[] = {{&S61305[34], 23}, {NULL}};
const Segment SGI168647[] = {{&S61305[27], 6}, {NULL}};
const Segment SGI168648[] = {{&S61305[120], 9}, {NULL}};
const Segment SGI168649[] = {{&S61305[93], 6}, {NULL}};
const Segment SGI168650[] = {{&S61306[34], 24}, {NULL}};
const Segment SGI168651[] = {{&S61306[27], 6}, {NULL}};
const Segment SGI168652[] = {{&S61306[121], 9}, {NULL}};
const Segment SGI168653[] = {{&S61306[94], 6}, {NULL}};
const Segment SGI168654[] = {{&S61307[34], 24}, {NULL}};
const Segment SGI168655[] = {{&S61307[27], 6}, {NULL}};
const Segment SGI168656[] = {{&S61307[121], 9}, {NULL}};
const Segment SGI168657[] = {{&S61307[94], 6}, {NULL}};
const Segment SGI168658[] = {{&S61308[44], 9}, {NULL}};
const Segment SGI168659[] = {{&S61308[44], 21}, {NULL}};
const Segment SGI168660[] = {{&S61308[30], 6}, {NULL}};
const Segment SGI168661[] = {{&S61308[31], 3}, {NULL}};
const Segment SGI168662[] = {{&S61308[94], 6}, {NULL}};
const Segment SGI168663[] = {{&S61308[67], 8}, {NULL}};
const Segment SGI168664[] = {{&S53643[44], 11}, {NULL}};
const Segment SGI168665[] = {{&S53643[44], 23}, {NULL}};
const Segment SGI168666[] = {{&S53643[30], 6}, {NULL}};
const Segment SGI168667[] = {{&S53643[116], 3}, {NULL}};
const Segment SGI168668[] = {{&S53643[81], 6}, {NULL}};
const Segment SGI168669[] = {{&S53644[44], 6}, {NULL}};
const Segment SGI168670[] = {{&S53644[44], 18}, {NULL}};
const Segment SGI168671[] = {{&S53644[30], 6}, {NULL}};
const Segment SGI168672[] = {{&S53644[111], 3}, {NULL}};
const Segment SGI168673[] = {{&S53644[76], 6}, {NULL}};
const Segment SGI168674[] = {{&S53645[54], 11}, {NULL}};
const Segment SGI168675[] = {{&S53645[47], 6}, {NULL}};
const Segment SGI168676[] = {{&S53646[44], 9}, {NULL}};
const Segment SGI168677[] = {{&S53646[44], 21}, {NULL}};
const Segment SGI168678[] = {{&S53646[30], 6}, {NULL}};
const Segment SGI168679[] = {{&S53646[114], 3}, {NULL}};
const Segment SGI168680[] = {{&S53646[79], 6}, {NULL}};
const Segment SGI168681[] = {{&S61330[44], 11}, {NULL}};
const Segment SGI168682[] = {{&S61330[44], 23}, {NULL}};
const Segment SGI168683[] = {{&S61330[30], 6}, {NULL}};
const Segment SGI168684[] = {{&S61330[31], 3}, {NULL}};
const Segment SGI168685[] = {{&S61330[81], 6}, {NULL}};
const Segment SGI168686[] = {{&S61354[44], 7}, {NULL}};
const Segment SGI168687[] = {{&S61354[44], 20}, {NULL}};
const Segment SGI168688[] = {{&S61354[30], 6}, {NULL}};
const Segment SGI168689[] = {{&S61354[31], 3}, {NULL}};
const Segment SGI168690[] = {{&S61354[78], 6}, {NULL}};
const Segment SGI168691[] = {{&S61355[44], 7}, {NULL}};
const Segment SGI168692[] = {{&S61355[44], 20}, {NULL}};
const Segment SGI168693[] = {{&S61355[30], 6}, {NULL}};
const Segment SGI168694[] = {{&S61355[31], 3}, {NULL}};
const Segment SGI168695[] = {{&S61355[78], 6}, {NULL}};
const Segment SGI168696[] = {{&S61356[44], 7}, {NULL}};
const Segment SGI168697[] = {{&S61356[44], 20}, {NULL}};
const Segment SGI168698[] = {{&S61356[30], 6}, {NULL}};
const Segment SGI168699[] = {{&S61356[31], 3}, {NULL}};
const Segment SGI168700[] = {{&S61356[78], 6}, {NULL}};
const Segment SGI168701[] = {{&S61357[44], 7}, {NULL}};
const Segment SGI168702[] = {{&S61357[44], 20}, {NULL}};
const Segment SGI168703[] = {{&S61357[30], 6}, {NULL}};
const Segment SGI168704[] = {{&S61357[31], 3}, {NULL}};
const Segment SGI168705[] = {{&S61357[78], 6}, {NULL}};
const Segment SGI168706[] = {{&S38393[44], 14}, {NULL}};
const Segment SGI168707[] = {{&S38393[44], 25}, {NULL}};
const Segment SGI168708[] = {{&S38393[30], 6}, {NULL}};
const Segment SGI168709[] = {{&S61358[44], 10}, {NULL}};
const Segment SGI168710[] = {{&S61358[44], 22}, {NULL}};
const Segment SGI168711[] = {{&S61358[30], 6}, {NULL}};
const Segment SGI168712[] = {{&S38393[136], 3}, {NULL}};
const Segment SGI168713[] = {{&S38393[101], 6}, {NULL}};
const Segment SGI168714[] = {{&S38393[71], 8}, {NULL}};
const Segment SGI168715[] = {{&S61358[80], 6}, {NULL}};
const Segment SGI168716[] = {{&S61358[107], 13}, {NULL}};
const Segment SGI168717[] = {{&S38394[44], 14}, {NULL}};
const Segment SGI168718[] = {{&S38394[44], 26}, {NULL}};
const Segment SGI168719[] = {{&S38394[30], 6}, {NULL}};
const Segment SGI168720[] = {{&S38394[139], 3}, {NULL}};
const Segment SGI168721[] = {{&S38394[104], 6}, {NULL}};
const Segment SGI168722[] = {{&S38394[72], 8}, {NULL}};
const Segment SGI168723[] = {{&S61361[44], 11}, {NULL}};
const Segment SGI168724[] = {{&S61361[44], 23}, {NULL}};
const Segment SGI168725[] = {{&S61361[30], 6}, {NULL}};
const Segment SGI168726[] = {{&S61361[81], 6}, {NULL}};
const Segment SGI168727[] = {{&S61361[108], 13}, {NULL}};
const Segment SGI168728[] = {{&S38403[34], 10}, {NULL}};
const Segment SGI168729[] = {{&S38403[27], 6}, {NULL}};
const Segment SGI168730[] = {{&S38403[96], 9}, {NULL}};
const Segment SGI168731[] = {{&S38403[69], 6}, {NULL}};
const Segment SGI168732[] = {{&S38404[34], 11}, {NULL}};
const Segment SGI168733[] = {{&S38404[27], 6}, {NULL}};
const Segment SGI168734[] = {{&S38404[97], 9}, {NULL}};
const Segment SGI168735[] = {{&S38404[70], 6}, {NULL}};
const Segment SGI168736[] = {{&S53707[34], 10}, {NULL}};
const Segment SGI168737[] = {{&S53707[27], 6}, {NULL}};
const Segment SGI168738[] = {{&S61379[34], 8}, {NULL}};
const Segment SGI168739[] = {{&S53707[97], 9}, {NULL}};
const Segment SGI168740[] = {{&S53707[70], 6}, {NULL}};
const Segment SGI168741[] = {{&S61379[27], 6}, {NULL}};
const Segment SGI168742[] = {{&S61379[100], 9}, {NULL}};
const Segment SGI168743[] = {{&S61379[73], 6}, {NULL}};
const Segment SGI168744[] = {{&S61380[34], 7}, {NULL}};
const Segment SGI168745[] = {{&S61380[27], 6}, {NULL}};
const Segment SGI168746[] = {{&S61380[99], 9}, {NULL}};
const Segment SGI168747[] = {{&S61380[72], 6}, {NULL}};
const Segment SGI168748[] = {{&S61381[34], 7}, {NULL}};
const Segment SGI168749[] = {{&S61381[27], 6}, {NULL}};
const Segment SGI168750[] = {{&S61381[99], 9}, {NULL}};
const Segment SGI168751[] = {{&S61381[72], 6}, {NULL}};
const Segment SGI168752[] = {{&S61382[34], 7}, {NULL}};
const Segment SGI168753[] = {{&S61382[27], 6}, {NULL}};
const Segment SGI168754[] = {{&S61382[98], 9}, {NULL}};
const Segment SGI168755[] = {{&S61382[71], 6}, {NULL}};
const Segment SGI168756[] = {{&S50916[34], 11}, {NULL}};
const Segment SGI168757[] = {{&S50916[27], 6}, {NULL}};
const Segment SGI168758[] = {{&S50916[98], 9}, {NULL}};
const Segment SGI168759[] = {{&S50916[71], 6}, {NULL}};
const Segment SGI168760[] = {{&S50921[34], 8}, {NULL}};
const Segment SGI168761[] = {{&S61420[34], 7}, {NULL}};
const Segment SGI168762[] = {{&S50921[27], 6}, {NULL}};
const Segment SGI168763[] = {{&S61420[27], 6}, {NULL}};
const Segment SGI168764[] = {{&S50921[105], 9}, {NULL}};
const Segment SGI168765[] = {{&S50921[78], 6}, {NULL}};
const Segment SGI168766[] = {{&S61420[94], 9}, {NULL}};
const Segment SGI168767[] = {{&S61420[67], 6}, {NULL}};
const Segment SGI168768[] = {{&S50922[34], 8}, {NULL}};
const Segment SGI168769[] = {{&S50922[27], 6}, {NULL}};
const Segment SGI168770[] = {{&S50922[105], 9}, {NULL}};
const Segment SGI168771[] = {{&S50922[78], 6}, {NULL}};
const Segment SGI168772[] = {{&S47847[34], 29}, {NULL}};
const Segment SGI168773[] = {{&S47847[27], 6}, {NULL}};
const Segment SGI168774[] = {{&S47847[116], 9}, {NULL}};
const Segment SGI168775[] = {{&S47847[89], 6}, {NULL}};
const Segment SGI168776[] = {{&S61464[34], 10}, {NULL}};
const Segment SGI168777[] = {{&S61464[27], 6}, {NULL}};
const Segment SGI168778[] = {{&S61464[97], 9}, {NULL}};
const Segment SGI168779[] = {{&S61464[70], 6}, {NULL}};
const Segment SGI168780[] = {{&S61473[34], 7}, {NULL}};
const Segment SGI168781[] = {{&S61473[27], 6}, {NULL}};
const Segment SGI168782[] = {{&S61473[99], 9}, {NULL}};
const Segment SGI168783[] = {{&S61473[72], 6}, {NULL}};
const Segment SGI168784[] = {{&S61474[34], 7}, {NULL}};
const Segment SGI168785[] = {{&S61474[27], 6}, {NULL}};
const Segment SGI168786[] = {{&S61474[99], 9}, {NULL}};
const Segment SGI168787[] = {{&S61474[72], 6}, {NULL}};
const Segment SGI168788[] = {{&S47879[34], 18}, {NULL}};
const Segment SGI168789[] = {{&S47879[27], 6}, {NULL}};
const Segment SGI168790[] = {{&S47879[105], 9}, {NULL}};
const Segment SGI168791[] = {{&S47879[78], 6}, {NULL}};
const Segment SGI168792[] = {{&S47880[34], 10}, {NULL}};
const Segment SGI168793[] = {{&S47880[27], 6}, {NULL}};
const Segment SGI168794[] = {{&S47880[97], 9}, {NULL}};
const Segment SGI168795[] = {{&S47880[70], 6}, {NULL}};
const Segment SGI168796[] = {{&S50989[34], 7}, {NULL}};
const Segment SGI168797[] = {{&S50989[27], 6}, {NULL}};
const Segment SGI168798[] = {{&S50989[99], 9}, {NULL}};
const Segment SGI168799[] = {{&S50989[72], 6}, {NULL}};
const Segment SGI168800[] = {{&S47881[34], 10}, {NULL}};
const Segment SGI168801[] = {{&S47881[27], 6}, {NULL}};
const Segment SGI168802[] = {{&S47881[97], 9}, {NULL}};
const Segment SGI168803[] = {{&S47881[70], 6}, {NULL}};
const Segment SGI168804[] = {{&S50990[34], 7}, {NULL}};
const Segment SGI168805[] = {{&S50990[27], 6}, {NULL}};
const Segment SGI168806[] = {{&S50990[99], 9}, {NULL}};
const Segment SGI168807[] = {{&S50990[72], 6}, {NULL}};
const Segment SGI168808[] = {{&S47882[34], 11}, {NULL}};
const Segment SGI168809[] = {{&S47882[27], 6}, {NULL}};
const Segment SGI168810[] = {{&S61487[44], 11}, {NULL}};
const Segment SGI168811[] = {{&S61487[44], 28}, {NULL}};
const Segment SGI168812[] = {{&S61487[30], 6}, {NULL}};
const Segment SGI168813[] = {{&S47882[98], 9}, {NULL}};
const Segment SGI168814[] = {{&S47882[71], 6}, {NULL}};
const Segment SGI168815[] = {{&S61487[31], 3}, {NULL}};
const Segment SGI168816[] = {{&S50991[34], 10}, {NULL}};
const Segment SGI168817[] = {{&S61487[86], 6}, {NULL}};
const Segment SGI168818[] = {{&S50991[27], 6}, {NULL}};
const Segment SGI168819[] = {{&S50991[97], 9}, {NULL}};
const Segment SGI168820[] = {{&S50991[70], 6}, {NULL}};
const Segment SGI168821[] = {{&S61488[44], 11}, {NULL}};
const Segment SGI168822[] = {{&S61488[44], 28}, {NULL}};
const Segment SGI168823[] = {{&S61488[30], 6}, {NULL}};
const Segment SGI168824[] = {{&S61488[31], 3}, {NULL}};
const Segment SGI168825[] = {{&S61488[86], 6}, {NULL}};
const Segment SGI168826[] = {{&S50993[38], 10}, {NULL}};
const Segment SGI168827[] = {{&S50993[31], 6}, {NULL}};
const Segment SGI168828[] = {{&S61489[44], 11}, {NULL}};
const Segment SGI168829[] = {{&S61489[44], 28}, {NULL}};
const Segment SGI168830[] = {{&S61489[30], 6}, {NULL}};
const Segment SGI168831[] = {{&S61489[31], 3}, {NULL}};
const Segment SGI168832[] = {{&S61489[86], 6}, {NULL}};
const Segment SGI168833[] = {{&S61490[44], 7}, {NULL}};
const Segment SGI168834[] = {{&S61490[44], 24}, {NULL}};
const Segment SGI168835[] = {{&S61490[30], 6}, {NULL}};
const Segment SGI168836[] = {{&S50996[46], 10}, {NULL}};
const Segment SGI168837[] = {{&S61490[31], 3}, {NULL}};
const Segment SGI168838[] = {{&S50996[46], 71}, {NULL}};
const Segment SGI168839[] = {{&S50996[32], 6}, {NULL}};
const Segment SGI168840[] = {{&S61490[82], 6}, {NULL}};
const Segment SGI168841[] = {{&S61491[38], 7}, {NULL}};
const Segment SGI168842[] = {{&S61491[31], 6}, {NULL}};
const Segment SGI168843[] = {{&S50998[44], 10}, {NULL}};
const Segment SGI168844[] = {{&S50998[44], 22}, {NULL}};
const Segment SGI168845[] = {{&S50998[30], 6}, {NULL}};
const Segment SGI168846[] = {{&S50998[115], 3}, {NULL}};
const Segment SGI168847[] = {{&S50998[80], 6}, {NULL}};
const Segment SGI168848[] = {{&S50999[44], 11}, {NULL}};
const Segment SGI168849[] = {{&S50999[44], 23}, {NULL}};
const Segment SGI168850[] = {{&S50999[30], 6}, {NULL}};
const Segment SGI168851[] = {{&S50999[116], 3}, {NULL}};
const Segment SGI168852[] = {{&S50999[81], 6}, {NULL}};
const Segment SGI168853[] = {{&S51000[38], 11}, {NULL}};
const Segment SGI168854[] = {{&S51000[31], 6}, {NULL}};
const Segment SGI168855[] = {{&S61498[44], 5}, {NULL}};
const Segment SGI168856[] = {{&S61498[44], 17}, {NULL}};
const Segment SGI168857[] = {{&S61498[30], 6}, {NULL}};
const Segment SGI168858[] = {{&S61498[31], 3}, {NULL}};
const Segment SGI168859[] = {{&S61498[75], 6}, {NULL}};
const Segment SGI168860[] = {{&S51002[34], 14}, {NULL}};
const Segment SGI168861[] = {{&S51002[27], 6}, {NULL}};
const Segment SGI168862[] = {{&S53856[34], 16}, {NULL}};
const Segment SGI168863[] = {{&S51002[111], 9}, {NULL}};
const Segment SGI168864[] = {{&S53856[27], 6}, {NULL}};
const Segment SGI168865[] = {{&S51002[84], 6}, {NULL}};
const Segment SGI168866[] = {{&S61499[38], 5}, {NULL}};
const Segment SGI168867[] = {{&S61499[31], 6}, {NULL}};
const Segment SGI168868[] = {{&S53856[112], 9}, {NULL}};
const Segment SGI168869[] = {{&S53856[85], 6}, {NULL}};
const Segment SGI168870[] = {{&S51003[34], 14}, {NULL}};
const Segment SGI168871[] = {{&S51003[27], 6}, {NULL}};
const Segment SGI168872[] = {{&S53857[34], 18}, {NULL}};
const Segment SGI168873[] = {{&S51003[111], 9}, {NULL}};
const Segment SGI168874[] = {{&S53857[27], 6}, {NULL}};
const Segment SGI168875[] = {{&S51003[84], 6}, {NULL}};
const Segment SGI168876[] = {{&S53857[114], 9}, {NULL}};
const Segment SGI168877[] = {{&S53857[87], 6}, {NULL}};
const Segment SGI168878[] = {{&S53858[34], 18}, {NULL}};
const Segment SGI168879[] = {{&S53858[27], 6}, {NULL}};
const Segment SGI168880[] = {{&S53858[114], 9}, {NULL}};
const Segment SGI168881[] = {{&S53858[87], 6}, {NULL}};
const Segment SGI168882[] = {{&S53859[34], 18}, {NULL}};
const Segment SGI168883[] = {{&S53859[27], 6}, {NULL}};
const Segment SGI168884[] = {{&S53859[114], 9}, {NULL}};
const Segment SGI168885[] = {{&S53859[87], 6}, {NULL}};
const Segment SGI168886[] = {{&S53860[34], 18}, {NULL}};
const Segment SGI168887[] = {{&S53860[27], 6}, {NULL}};
const Segment SGI168888[] = {{&S53860[114], 9}, {NULL}};
const Segment SGI168889[] = {{&S53860[87], 6}, {NULL}};
const Segment SGI168890[] = {{&S53865[34], 12}, {NULL}};
const Segment SGI168891[] = {{&S53865[27], 6}, {NULL}};
const Segment SGI168892[] = {{&S53865[99], 9}, {NULL}};
const Segment SGI168893[] = {{&S53865[72], 6}, {NULL}};
const Segment SGI168894[] = {{&S53866[34], 13}, {NULL}};
const Segment SGI168895[] = {{&S53866[27], 6}, {NULL}};
const Segment SGI168896[] = {{&S53866[100], 9}, {NULL}};
const Segment SGI168897[] = {{&S53866[73], 6}, {NULL}};
const Segment SGI168898[] = {{&S53867[34], 9}, {NULL}};
const Segment SGI168899[] = {{&S53867[27], 6}, {NULL}};
const Segment SGI168900[] = {{&S53867[96], 9}, {NULL}};
const Segment SGI168901[] = {{&S53867[69], 6}, {NULL}};
const Segment SGI168902[] = {{&S53868[34], 7}, {NULL}};
const Segment SGI168903[] = {{&S53868[27], 6}, {NULL}};
const Segment SGI168904[] = {{&S53868[94], 9}, {NULL}};
const Segment SGI168905[] = {{&S53868[67], 6}, {NULL}};
const Segment SGI168906[] = {{&S51031[34], 11}, {NULL}};
const Segment SGI168907[] = {{&S51031[27], 6}, {NULL}};
const Segment SGI168908[] = {{&S53874[44], 8}, {NULL}};
const Segment SGI168909[] = {{&S51031[98], 9}, {NULL}};
const Segment SGI168910[] = {{&S53874[44], 20}, {NULL}};
const Segment SGI168911[] = {{&S51031[71], 6}, {NULL}};
const Segment SGI168912[] = {{&S53874[30], 6}, {NULL}};
const Segment SGI168913[] = {{&S53874[113], 3}, {NULL}};
const Segment SGI168914[] = {{&S53874[78], 6}, {NULL}};
const Segment SGI168915[] = {{&S51032[34], 10}, {NULL}};
const Segment SGI168916[] = {{&S51032[27], 6}, {NULL}};
const Segment SGI168917[] = {{&S53875[54], 8}, {NULL}};
const Segment SGI168918[] = {{&S53875[47], 6}, {NULL}};
const Segment SGI168919[] = {{&S51032[97], 9}, {NULL}};
const Segment SGI168920[] = {{&S51032[70], 6}, {NULL}};
const Segment SGI168921[] = {{&S53876[44], 8}, {NULL}};
const Segment SGI168922[] = {{&S53876[44], 20}, {NULL}};
const Segment SGI168923[] = {{&S53876[30], 6}, {NULL}};
const Segment SGI168924[] = {{&S53876[113], 3}, {NULL}};
const Segment SGI168925[] = {{&S53876[78], 6}, {NULL}};
const Segment SGI168926[] = {{&S53877[44], 8}, {NULL}};
const Segment SGI168927[] = {{&S53877[44], 20}, {NULL}};
const Segment SGI168928[] = {{&S53877[30], 6}, {NULL}};
const Segment SGI168929[] = {{&S53877[113], 3}, {NULL}};
const Segment SGI168930[] = {{&S53877[78], 6}, {NULL}};
const Segment SGI168931[] = {{&S61547[34], 7}, {NULL}};
const Segment SGI168932[] = {{&S61547[27], 6}, {NULL}};
const Segment SGI168933[] = {{&S61547[99], 9}, {NULL}};
const Segment SGI168934[] = {{&S61547[72], 6}, {NULL}};
const Segment SGI168935[] = {{&S61548[34], 7}, {NULL}};
const Segment SGI168936[] = {{&S61548[27], 6}, {NULL}};
const Segment SGI168937[] = {{&S61548[99], 9}, {NULL}};
const Segment SGI168938[] = {{&S61548[72], 6}, {NULL}};
const Segment SGI168939[] = {{&S47957[34], 29}, {NULL}};
const Segment SGI168940[] = {{&S47957[27], 6}, {NULL}};
const Segment SGI168941[] = {{&S47957[116], 9}, {NULL}};
const Segment SGI168942[] = {{&S47957[89], 6}, {NULL}};
const Segment SGI168943[] = {{&S61559[44], 8}, {NULL}};
const Segment SGI168944[] = {{&S61559[44], 19}, {NULL}};
const Segment SGI168945[] = {{&S61559[30], 6}, {NULL}};
const Segment SGI168946[] = {{&S61559[31], 3}, {NULL}};
const Segment SGI168947[] = {{&S61559[77], 6}, {NULL}};
const Segment SGI168948[] = {{&S61560[44], 8}, {NULL}};
const Segment SGI168949[] = {{&S61560[44], 19}, {NULL}};
const Segment SGI168950[] = {{&S61560[30], 6}, {NULL}};
const Segment SGI168951[] = {{&S61560[31], 3}, {NULL}};
const Segment SGI168952[] = {{&S61560[77], 6}, {NULL}};
const Segment SGI168953[] = {{&S51063[44], 10}, {NULL}};
const Segment SGI168954[] = {{&S51063[44], 22}, {NULL}};
const Segment SGI168955[] = {{&S51063[30], 6}, {NULL}};
const Segment SGI168956[] = {{&S61561[38], 8}, {NULL}};
const Segment SGI168957[] = {{&S61561[31], 6}, {NULL}};
const Segment SGI168958[] = {{&S51063[115], 3}, {NULL}};
const Segment SGI168959[] = {{&S51063[80], 6}, {NULL}};
const Segment SGI168960[] = {{&S51063[150], 3}, {NULL}};
const Segment SGI168961[] = {{&S61562[44], 8}, {NULL}};
const Segment SGI168962[] = {{&S61562[44], 19}, {NULL}};
const Segment SGI168963[] = {{&S61562[30], 6}, {NULL}};
const Segment SGI168964[] = {{&S51064[44], 10}, {NULL}};
const Segment SGI168965[] = {{&S61562[31], 3}, {NULL}};
const Segment SGI168966[] = {{&S51064[44], 22}, {NULL}};
const Segment SGI168967[] = {{&S51064[30], 6}, {NULL}};
const Segment SGI168968[] = {{&S61562[77], 6}, {NULL}};
const Segment SGI168969[] = {{&S51064[115], 3}, {NULL}};
const Segment SGI168970[] = {{&S51064[80], 6}, {NULL}};
const Segment SGI168971[] = {{&S51064[150], 3}, {NULL}};
const Segment SGI168972[] = {{&S51083[34], 12}, {NULL}};
const Segment SGI168973[] = {{&S51083[27], 6}, {NULL}};
const Segment SGI168974[] = {{&S51083[99], 9}, {NULL}};
const Segment SGI168975[] = {{&S51083[72], 6}, {NULL}};
const Segment SGI168976[] = {{&S53938[34], 11}, {NULL}};
const Segment SGI168977[] = {{&S53938[27], 6}, {NULL}};
const Segment SGI168978[] = {{&S53938[98], 9}, {NULL}};
const Segment SGI168979[] = {{&S53938[71], 6}, {NULL}};
const Segment SGI168980[] = {{&S53939[34], 11}, {NULL}};
const Segment SGI168981[] = {{&S53939[27], 6}, {NULL}};
const Segment SGI168982[] = {{&S53939[98], 9}, {NULL}};
const Segment SGI168983[] = {{&S53939[71], 6}, {NULL}};
const Segment SGI168984[] = {{&S53945[34], 10}, {NULL}};
const Segment SGI168985[] = {{&S48011[34], 10}, {NULL}};
const Segment SGI168986[] = {{&S53945[27], 6}, {NULL}};
const Segment SGI168987[] = {{&S48011[27], 6}, {NULL}};
const Segment SGI168988[] = {{&S53945[97], 9}, {NULL}};
const Segment SGI168989[] = {{&S53945[70], 6}, {NULL}};
const Segment SGI168990[] = {{&S48011[97], 9}, {NULL}};
const Segment SGI168991[] = {{&S48011[70], 6}, {NULL}};
const Segment SGI168992[] = {{&S53954[44], 7}, {NULL}};
const Segment SGI168993[] = {{&S53954[44], 19}, {NULL}};
const Segment SGI168994[] = {{&S53954[30], 6}, {NULL}};
const Segment SGI168995[] = {{&S53954[112], 3}, {NULL}};
const Segment SGI168996[] = {{&S53954[77], 6}, {NULL}};
const Segment SGI168997[] = {{&S53955[44], 10}, {NULL}};
const Segment SGI168998[] = {{&S53955[44], 22}, {NULL}};
const Segment SGI168999[] = {{&S53955[30], 6}, {NULL}};
const Segment SGI169000[] = {{&S53955[115], 3}, {NULL}};
const Segment SGI169001[] = {{&S53955[80], 6}, {NULL}};
const Segment SGI169002[] = {{&S53956[44], 7}, {NULL}};
const Segment SGI169003[] = {{&S53956[44], 19}, {NULL}};
const Segment SGI169004[] = {{&S53956[30], 6}, {NULL}};
const Segment SGI169005[] = {{&S53956[112], 3}, {NULL}};
const Segment SGI169006[] = {{&S53956[77], 6}, {NULL}};
const Segment SGI169007[] = {{&S53957[38], 10}, {NULL}};
const Segment SGI169008[] = {{&S53957[31], 6}, {NULL}};
const Segment SGI169009[] = {{&S53958[38], 7}, {NULL}};
const Segment SGI169010[] = {{&S53958[31], 6}, {NULL}};
const Segment SGI169011[] = {{&S51119[44], 10}, {NULL}};
const Segment SGI169012[] = {{&S51119[44], 22}, {NULL}};
const Segment SGI169013[] = {{&S51119[30], 6}, {NULL}};
const Segment SGI169014[] = {{&S51119[115], 3}, {NULL}};
const Segment SGI169015[] = {{&S51119[80], 6}, {NULL}};
const Segment SGI169016[] = {{&S51119[150], 3}, {NULL}};
const Segment SGI169017[] = {{&S51120[44], 10}, {NULL}};
const Segment SGI169018[] = {{&S51120[44], 22}, {NULL}};
const Segment SGI169019[] = {{&S51120[30], 6}, {NULL}};
const Segment SGI169020[] = {{&S51120[115], 3}, {NULL}};
const Segment SGI169021[] = {{&S51120[80], 6}, {NULL}};
const Segment SGI169022[] = {{&S51120[150], 3}, {NULL}};
const Segment SGI169023[] = {{&S51121[44], 11}, {NULL}};
const Segment SGI169024[] = {{&S51121[44], 23}, {NULL}};
const Segment SGI169025[] = {{&S51121[30], 6}, {NULL}};
const Segment SGI169026[] = {{&S51121[116], 3}, {NULL}};
const Segment SGI169027[] = {{&S51121[81], 6}, {NULL}};
const Segment SGI169028[] = {{&S51121[151], 3}, {NULL}};
const Segment SGI169029[] = {{&S51127[34], 8}, {NULL}};
const Segment SGI169030[] = {{&S51127[27], 6}, {NULL}};
const Segment SGI169031[] = {{&S51127[103], 9}, {NULL}};
const Segment SGI169032[] = {{&S51127[76], 6}, {NULL}};
const Segment SGI169033[] = {{&S51141[44], 16}, {NULL}};
const Segment SGI169034[] = {{&S51141[44], 28}, {NULL}};
const Segment SGI169035[] = {{&S51141[30], 6}, {NULL}};
const Segment SGI169036[] = {{&S61650[44], 10}, {NULL}};
const Segment SGI169037[] = {{&S61650[44], 22}, {NULL}};
const Segment SGI169038[] = {{&S61650[30], 6}, {NULL}};
const Segment SGI169039[] = {{&S51141[121], 3}, {NULL}};
const Segment SGI169040[] = {{&S51141[86], 6}, {NULL}};
const Segment SGI169041[] = {{&S61650[80], 6}, {NULL}};
const Segment SGI169042[] = {{&S61650[107], 13}, {NULL}};
const Segment SGI169043[] = {{&S51142[44], 16}, {NULL}};
const Segment SGI169044[] = {{&S51142[44], 28}, {NULL}};
const Segment SGI169045[] = {{&S51142[30], 6}, {NULL}};
const Segment SGI169046[] = {{&S61651[44], 11}, {NULL}};
const Segment SGI169047[] = {{&S61651[44], 23}, {NULL}};
const Segment SGI169048[] = {{&S61651[30], 6}, {NULL}};
const Segment SGI169049[] = {{&S51142[121], 3}, {NULL}};
const Segment SGI169050[] = {{&S51142[86], 6}, {NULL}};
const Segment SGI169051[] = {{&S61651[81], 6}, {NULL}};
const Segment SGI169052[] = {{&S61651[108], 13}, {NULL}};
const Segment SGI169053[] = {{&S51143[38], 16}, {NULL}};
const Segment SGI169054[] = {{&S51143[31], 6}, {NULL}};
const Segment SGI169055[] = {{&S51144[44], 16}, {NULL}};
const Segment SGI169056[] = {{&S51144[44], 28}, {NULL}};
const Segment SGI169057[] = {{&S51144[30], 6}, {NULL}};
const Segment SGI169058[] = {{&S51144[121], 3}, {NULL}};
const Segment SGI169059[] = {{&S51144[86], 6}, {NULL}};
const Segment SGI169060[] = {{&S51145[38], 16}, {NULL}};
const Segment SGI169061[] = {{&S51145[31], 6}, {NULL}};
const Segment SGI169062[] = {{&S51146[34], 8}, {NULL}};
const Segment SGI169063[] = {{&S61657[34], 6}, {NULL}};
const Segment SGI169064[] = {{&S51146[27], 6}, {NULL}};
const Segment SGI169065[] = {{&S61657[27], 6}, {NULL}};
const Segment SGI169066[] = {{&S51146[105], 9}, {NULL}};
const Segment SGI169067[] = {{&S51146[78], 6}, {NULL}};
const Segment SGI169068[] = {{&S61657[93], 9}, {NULL}};
const Segment SGI169069[] = {{&S61657[66], 6}, {NULL}};
const Segment SGI169070[] = {{&S51147[34], 8}, {NULL}};
const Segment SGI169071[] = {{&S61658[34], 7}, {NULL}};
const Segment SGI169072[] = {{&S51147[27], 6}, {NULL}};
const Segment SGI169073[] = {{&S61658[27], 6}, {NULL}};
const Segment SGI169074[] = {{&S51147[105], 9}, {NULL}};
const Segment SGI169075[] = {{&S51147[78], 6}, {NULL}};
const Segment SGI169076[] = {{&S61658[98], 9}, {NULL}};
const Segment SGI169077[] = {{&S61658[71], 6}, {NULL}};
const Segment SGI169078[] = {{&S51150[44], 12}, {NULL}};
const Segment SGI169079[] = {{&S51150[44], 32}, {NULL}};
const Segment SGI169080[] = {{&S51150[30], 6}, {NULL}};
const Segment SGI169081[] = {{&S51150[125], 3}, {NULL}};
const Segment SGI169082[] = {{&S51150[90], 6}, {NULL}};
const Segment SGI169083[] = {{&S51151[44], 8}, {NULL}};
const Segment SGI169084[] = {{&S51151[44], 28}, {NULL}};
const Segment SGI169085[] = {{&S51151[30], 6}, {NULL}};
const Segment SGI169086[] = {{&S51151[121], 3}, {NULL}};
const Segment SGI169087[] = {{&S51151[86], 6}, {NULL}};
const Segment SGI169088[] = {{&S51152[44], 12}, {NULL}};
const Segment SGI169089[] = {{&S51152[44], 31}, {NULL}};
const Segment SGI169090[] = {{&S51152[30], 6}, {NULL}};
const Segment SGI169091[] = {{&S51152[124], 3}, {NULL}};
const Segment SGI169092[] = {{&S51152[89], 6}, {NULL}};
const Segment SGI169093[] = {{&S51153[44], 12}, {NULL}};
const Segment SGI169094[] = {{&S51153[44], 31}, {NULL}};
const Segment SGI169095[] = {{&S51153[30], 6}, {NULL}};
const Segment SGI169096[] = {{&S51153[124], 3}, {NULL}};
const Segment SGI169097[] = {{&S51153[89], 6}, {NULL}};
const Segment SGI169098[] = {{&S51154[38], 8}, {NULL}};
const Segment SGI169099[] = {{&S51154[31], 6}, {NULL}};
const Segment SGI169100[] = {{&S51155[44], 8}, {NULL}};
const Segment SGI169101[] = {{&S51155[44], 27}, {NULL}};
const Segment SGI169102[] = {{&S51155[30], 6}, {NULL}};
const Segment SGI169103[] = {{&S51155[120], 3}, {NULL}};
const Segment SGI169104[] = {{&S51155[85], 6}, {NULL}};
const Segment SGI169105[] = {{&S51156[44], 8}, {NULL}};
const Segment SGI169106[] = {{&S51156[44], 27}, {NULL}};
const Segment SGI169107[] = {{&S51156[30], 6}, {NULL}};
const Segment SGI169108[] = {{&S51156[120], 3}, {NULL}};
const Segment SGI169109[] = {{&S51156[85], 6}, {NULL}};
const Segment SGI169110[] = {{&S61680[34], 18}, {NULL}};
const Segment SGI169111[] = {{&S61680[27], 6}, {NULL}};
const Segment SGI169112[] = {{&S61680[105], 9}, {NULL}};
const Segment SGI169113[] = {{&S61680[78], 6}, {NULL}};
const Segment SGI169114[] = {{&S61684[34], 12}, {NULL}};
const Segment SGI169115[] = {{&S61684[27], 6}, {NULL}};
const Segment SGI169116[] = {{&S61684[99], 9}, {NULL}};
const Segment SGI169117[] = {{&S61684[72], 6}, {NULL}};
const Segment SGI169118[] = {{&S54044[34], 7}, {NULL}};
const Segment SGI169119[] = {{&S54044[27], 6}, {NULL}};
const Segment SGI169120[] = {{&S54044[99], 9}, {NULL}};
const Segment SGI169121[] = {{&S54044[72], 6}, {NULL}};
const Segment SGI169122[] = {{&S38760[44], 10}, {NULL}};
const Segment SGI169123[] = {{&S38760[44], 21}, {NULL}};
const Segment SGI169124[] = {{&S38760[30], 6}, {NULL}};
const Segment SGI169125[] = {{&S38760[114], 3}, {NULL}};
const Segment SGI169126[] = {{&S38760[79], 6}, {NULL}};
const Segment SGI169127[] = {{&S38761[38], 10}, {NULL}};
const Segment SGI169128[] = {{&S38761[31], 6}, {NULL}};
const Segment SGI169129[] = {{&S61711[44], 4}, {NULL}};
const Segment SGI169130[] = {{&S38762[44], 13}, {NULL}};
const Segment SGI169131[] = {{&S61711[44], 13}, {NULL}};
const Segment SGI169132[] = {{&S38762[44], 24}, {NULL}};
const Segment SGI169133[] = {{&S61711[30], 6}, {NULL}};
const Segment SGI169134[] = {{&S38762[30], 6}, {NULL}};
const Segment SGI169135[] = {{&S61711[31], 3}, {NULL}};
const Segment SGI169136[] = {{&S38762[117], 3}, {NULL}};
const Segment SGI169137[] = {{&S61711[71], 6}, {NULL}};
const Segment SGI169138[] = {{&S38762[82], 6}, {NULL}};
const Segment SGI169139[] = {{&S61712[44], 12}, {NULL}};
const Segment SGI169140[] = {{&S61712[44], 21}, {NULL}};
const Segment SGI169141[] = {{&S61712[30], 6}, {NULL}};
const Segment SGI169142[] = {{&S61712[31], 3}, {NULL}};
const Segment SGI169143[] = {{&S61712[79], 6}, {NULL}};
const Segment SGI169144[] = {{&S38765[34], 9}, {NULL}};
const Segment SGI169145[] = {{&S38765[27], 6}, {NULL}};
const Segment SGI169146[] = {{&S61713[42], 4}, {NULL}};
const Segment SGI169147[] = {{&S61713[30], 6}, {NULL}};
const Segment SGI169148[] = {{&S38765[95], 9}, {NULL}};
const Segment SGI169149[] = {{&S38765[68], 6}, {NULL}};
const Segment SGI169150[] = {{&S61713[31], 3}, {NULL}};
const Segment SGI169151[] = {{&S61713[69], 6}, {NULL}};
const Segment SGI169152[] = {{&S38766[34], 11}, {NULL}};
const Segment SGI169153[] = {{&S38766[27], 6}, {NULL}};
const Segment SGI169154[] = {{&S61714[34], 11}, {NULL}};
const Segment SGI169155[] = {{&S61714[27], 6}, {NULL}};
const Segment SGI169156[] = {{&S38766[97], 9}, {NULL}};
const Segment SGI169157[] = {{&S38766[70], 6}, {NULL}};
const Segment SGI169158[] = {{&S61714[98], 9}, {NULL}};
const Segment SGI169159[] = {{&S61714[71], 6}, {NULL}};
const Segment SGI169160[] = {{&S38767[34], 14}, {NULL}};
const Segment SGI169161[] = {{&S38767[27], 6}, {NULL}};
const Segment SGI169162[] = {{&S61715[34], 11}, {NULL}};
const Segment SGI169163[] = {{&S61715[27], 6}, {NULL}};
const Segment SGI169164[] = {{&S38767[100], 9}, {NULL}};
const Segment SGI169165[] = {{&S38767[73], 6}, {NULL}};
const Segment SGI169166[] = {{&S61715[98], 9}, {NULL}};
const Segment SGI169167[] = {{&S61715[71], 6}, {NULL}};
const Segment SGI169168[] = {{&S38768[34], 5}, {NULL}};
const Segment SGI169169[] = {{&S38768[27], 6}, {NULL}};
const Segment SGI169170[] = {{&S38768[92], 9}, {NULL}};
const Segment SGI169171[] = {{&S38768[65], 6}, {NULL}};
const Segment SGI169172[] = {{&S38769[34], 18}, {NULL}};
const Segment SGI169173[] = {{&S38769[27], 6}, {NULL}};
const Segment SGI169174[] = {{&S38769[104], 9}, {NULL}};
const Segment SGI169175[] = {{&S38769[77], 6}, {NULL}};
const Segment SGI169176[] = {{&S38771[34], 8}, {NULL}};
const Segment SGI169177[] = {{&S38771[27], 6}, {NULL}};
const Segment SGI169178[] = {{&S38771[94], 9}, {NULL}};
const Segment SGI169179[] = {{&S38771[67], 6}, {NULL}};
const Segment SGI169180[] = {{&S61723[34], 10}, {NULL}};
const Segment SGI169181[] = {{&S61723[27], 6}, {NULL}};
const Segment SGI169182[] = {{&S61723[97], 9}, {NULL}};
const Segment SGI169183[] = {{&S61723[70], 6}, {NULL}};
const Segment SGI169184[] = {{&S61724[34], 10}, {NULL}};
const Segment SGI169185[] = {{&S61724[27], 6}, {NULL}};
const Segment SGI169186[] = {{&S61724[97], 9}, {NULL}};
const Segment SGI169187[] = {{&S61724[70], 6}, {NULL}};
const Segment SGI169188[] = {{&S61728[44], 13}, {NULL}};
const Segment SGI169189[] = {{&S61728[44], 25}, {NULL}};
const Segment SGI169190[] = {{&S61728[30], 6}, {NULL}};
const Segment SGI169191[] = {{&S61728[31], 3}, {NULL}};
const Segment SGI169192[] = {{&S61728[83], 6}, {NULL}};
const Segment SGI169193[] = {{&S61729[44], 6}, {NULL}};
const Segment SGI169194[] = {{&S61729[44], 18}, {NULL}};
const Segment SGI169195[] = {{&S38782[44], 11}, {NULL}};
const Segment SGI169196[] = {{&S61729[30], 6}, {NULL}};
const Segment SGI169197[] = {{&S38782[44], 22}, {NULL}};
const Segment SGI169198[] = {{&S38782[30], 6}, {NULL}};
const Segment SGI169199[] = {{&S61729[31], 3}, {NULL}};
const Segment SGI169200[] = {{&S61729[76], 6}, {NULL}};
const Segment SGI169201[] = {{&S38782[115], 3}, {NULL}};
const Segment SGI169202[] = {{&S38782[80], 6}, {NULL}};
const Segment SGI169203[] = {{&S51226[34], 14}, {NULL}};
const Segment SGI169204[] = {{&S51226[27], 6}, {NULL}};
const Segment SGI169205[] = {{&S51226[101], 9}, {NULL}};
const Segment SGI169206[] = {{&S51226[74], 6}, {NULL}};
const Segment SGI169207[] = {{&S38783[44], 7}, {NULL}};
const Segment SGI169208[] = {{&S38783[44], 18}, {NULL}};
const Segment SGI169209[] = {{&S38783[30], 6}, {NULL}};
const Segment SGI169210[] = {{&S61730[30], 6}, {NULL}};
const Segment SGI169211[] = {{&S38783[111], 3}, {NULL}};
const Segment SGI169212[] = {{&S61730[31], 3}, {NULL}};
const Segment SGI169213[] = {{&S38783[76], 6}, {NULL}};
const Segment SGI169214[] = {{&S61730[81], 6}, {NULL}};
const Segment SGI169215[] = {{&S38784[38], 11}, {NULL}};
const Segment SGI169216[] = {{&S38784[31], 6}, {NULL}};
const Segment SGI169217[] = {{&S61731[38], 13}, {NULL}};
const Segment SGI169218[] = {{&S61731[31], 6}, {NULL}};
const Segment SGI169219[] = {{&S38785[38], 7}, {NULL}};
const Segment SGI169220[] = {{&S38785[31], 6}, {NULL}};
const Segment SGI169221[] = {{&S61734[34], 14}, {NULL}};
const Segment SGI169222[] = {{&S61734[27], 6}, {NULL}};
const Segment SGI169223[] = {{&S38787[44], 11}, {NULL}};
const Segment SGI169224[] = {{&S61734[101], 9}, {NULL}};
const Segment SGI169225[] = {{&S38787[44], 23}, {NULL}};
const Segment SGI169226[] = {{&S61734[74], 6}, {NULL}};
const Segment SGI169227[] = {{&S38787[30], 6}, {NULL}};
const Segment SGI169228[] = {{&S38787[116], 3}, {NULL}};
const Segment SGI169229[] = {{&S38787[81], 6}, {NULL}};
const Segment SGI169230[] = {{&S61735[34], 15}, {NULL}};
const Segment SGI169231[] = {{&S61735[27], 6}, {NULL}};
const Segment SGI169232[] = {{&S38788[44], 7}, {NULL}};
const Segment SGI169233[] = {{&S61735[102], 9}, {NULL}};
const Segment SGI169234[] = {{&S38788[44], 19}, {NULL}};
const Segment SGI169235[] = {{&S61735[75], 6}, {NULL}};
const Segment SGI169236[] = {{&S38788[30], 6}, {NULL}};
const Segment SGI169237[] = {{&S38788[112], 3}, {NULL}};
const Segment SGI169238[] = {{&S38788[77], 6}, {NULL}};
const Segment SGI169239[] = {{&S51235[44], 18}, {NULL}};
const Segment SGI169240[] = {{&S51235[44], 30}, {NULL}};
const Segment SGI169241[] = {{&S51235[30], 6}, {NULL}};
const Segment SGI169242[] = {{&S51235[139], 3}, {NULL}};
const Segment SGI169243[] = {{&S51235[104], 6}, {NULL}};
const Segment SGI169244[] = {{&S51235[76], 8}, {NULL}};
const Segment SGI169245[] = {{&S38789[44], 11}, {NULL}};
const Segment SGI169246[] = {{&S38789[44], 22}, {NULL}};
const Segment SGI169247[] = {{&S38789[30], 6}, {NULL}};
const Segment SGI169248[] = {{&S38789[115], 3}, {NULL}};
const Segment SGI169249[] = {{&S38789[80], 6}, {NULL}};
const Segment SGI169250[] = {{&S51237[34], 5}, {NULL}};
const Segment SGI169251[] = {{&S51237[27], 6}, {NULL}};
const Segment SGI169252[] = {{&S38790[44], 12}, {NULL}};
const Segment SGI169253[] = {{&S38790[44], 23}, {NULL}};
const Segment SGI169254[] = {{&S38790[30], 6}, {NULL}};
const Segment SGI169255[] = {{&S51237[92], 9}, {NULL}};
const Segment SGI169256[] = {{&S51237[65], 6}, {NULL}};
const Segment SGI169257[] = {{&S38790[116], 3}, {NULL}};
const Segment SGI169258[] = {{&S38790[81], 6}, {NULL}};
const Segment SGI169259[] = {{&S38791[44], 22}, {NULL}};
const Segment SGI169260[] = {{&S38791[44], 33}, {NULL}};
const Segment SGI169261[] = {{&S38791[30], 6}, {NULL}};
const Segment SGI169262[] = {{&S38791[126], 3}, {NULL}};
const Segment SGI169263[] = {{&S38791[91], 6}, {NULL}};
const Segment SGI169264[] = {{&S38792[44], 22}, {NULL}};
const Segment SGI169265[] = {{&S38792[44], 33}, {NULL}};
const Segment SGI169266[] = {{&S38792[30], 6}, {NULL}};
const Segment SGI169267[] = {{&S38792[126], 3}, {NULL}};
const Segment SGI169268[] = {{&S38792[91], 6}, {NULL}};
const Segment SGI169269[] = {{&S38793[44], 22}, {NULL}};
const Segment SGI169270[] = {{&S38793[44], 33}, {NULL}};
const Segment SGI169271[] = {{&S38793[30], 6}, {NULL}};
const Segment SGI169272[] = {{&S38793[126], 3}, {NULL}};
const Segment SGI169273[] = {{&S38793[91], 6}, {NULL}};
const Segment SGI169274[] = {{&S38794[44], 11}, {NULL}};
const Segment SGI169275[] = {{&S38794[44], 22}, {NULL}};
const Segment SGI169276[] = {{&S38794[30], 6}, {NULL}};
const Segment SGI169277[] = {{&S38794[115], 3}, {NULL}};
const Segment SGI169278[] = {{&S38794[80], 6}, {NULL}};
const Segment SGI169279[] = {{&S38795[44], 22}, {NULL}};
const Segment SGI169280[] = {{&S38795[44], 33}, {NULL}};
const Segment SGI169281[] = {{&S38795[30], 6}, {NULL}};
const Segment SGI169282[] = {{&S38795[126], 3}, {NULL}};
const Segment SGI169283[] = {{&S61750[34], 19}, {NULL}};
const Segment SGI169284[] = {{&S38795[91], 6}, {NULL}};
const Segment SGI169285[] = {{&S61750[27], 6}, {NULL}};
const Segment SGI169286[] = {{&S61750[106], 9}, {NULL}};
const Segment SGI169287[] = {{&S61750[79], 6}, {NULL}};
const Segment SGI169288[] = {{&S61751[34], 19}, {NULL}};
const Segment SGI169289[] = {{&S61751[27], 6}, {NULL}};
const Segment SGI169290[] = {{&S61751[106], 9}, {NULL}};
const Segment SGI169291[] = {{&S61751[79], 6}, {NULL}};
const Segment SGI169292[] = {{&S54124[44], 8}, {NULL}};
const Segment SGI169293[] = {{&S54124[44], 20}, {NULL}};
const Segment SGI169294[] = {{&S54124[30], 6}, {NULL}};
const Segment SGI169295[] = {{&S54124[113], 3}, {NULL}};
const Segment SGI169296[] = {{&S54124[78], 6}, {NULL}};
const Segment SGI169297[] = {{&S54125[44], 8}, {NULL}};
const Segment SGI169298[] = {{&S54125[44], 20}, {NULL}};
const Segment SGI169299[] = {{&S54125[30], 6}, {NULL}};
const Segment SGI169300[] = {{&S54125[113], 3}, {NULL}};
const Segment SGI169301[] = {{&S54125[78], 6}, {NULL}};
const Segment SGI169302[] = {{&S54126[54], 8}, {NULL}};
const Segment SGI169303[] = {{&S54126[47], 6}, {NULL}};
const Segment SGI169304[] = {{&S51267[44], 18}, {NULL}};
const Segment SGI169305[] = {{&S51267[44], 30}, {NULL}};
const Segment SGI169306[] = {{&S51267[30], 6}, {NULL}};
const Segment SGI169307[] = {{&S51267[143], 3}, {NULL}};
const Segment SGI169308[] = {{&S51267[108], 6}, {NULL}};
const Segment SGI169309[] = {{&S51267[76], 8}, {NULL}};
const Segment SGI169310[] = {{&S61769[34], 14}, {NULL}};
const Segment SGI169311[] = {{&S61769[27], 6}, {NULL}};
const Segment SGI169312[] = {{&S51274[34], 10}, {NULL}};
const Segment SGI169313[] = {{&S51274[27], 6}, {NULL}};
const Segment SGI169314[] = {{&S61769[101], 9}, {NULL}};
const Segment SGI169315[] = {{&S54140[34], 6}, {NULL}};
const Segment SGI169316[] = {{&S61769[74], 6}, {NULL}};
const Segment SGI169317[] = {{&S54140[27], 6}, {NULL}};
const Segment SGI169318[] = {{&S51274[97], 9}, {NULL}};
const Segment SGI169319[] = {{&S51274[70], 6}, {NULL}};
const Segment SGI169320[] = {{&S54140[93], 9}, {NULL}};
const Segment SGI169321[] = {{&S54140[66], 6}, {NULL}};
const Segment SGI169322[] = {{&S51275[34], 18}, {NULL}};
const Segment SGI169323[] = {{&S51275[27], 6}, {NULL}};
const Segment SGI169324[] = {{&S51275[105], 9}, {NULL}};
const Segment SGI169325[] = {{&S51275[78], 6}, {NULL}};
const Segment SGI169326[] = {{&S51276[34], 11}, {NULL}};
const Segment SGI169327[] = {{&S51276[27], 6}, {NULL}};
const Segment SGI169328[] = {{&S51276[98], 9}, {NULL}};
const Segment SGI169329[] = {{&S51276[71], 6}, {NULL}};
const Segment SGI169330[] = {{&S61775[44], 7}, {NULL}};
const Segment SGI169331[] = {{&S61775[44], 19}, {NULL}};
const Segment SGI169332[] = {{&S61775[30], 6}, {NULL}};
const Segment SGI169333[] = {{&S61775[31], 3}, {NULL}};
const Segment SGI169334[] = {{&S61775[77], 6}, {NULL}};
const Segment SGI169335[] = {{&S61776[38], 7}, {NULL}};
const Segment SGI169336[] = {{&S61776[31], 6}, {NULL}};
const Segment SGI169337[] = {{&S54148[44], 15}, {NULL}};
const Segment SGI169338[] = {{&S54148[44], 27}, {NULL}};
const Segment SGI169339[] = {{&S54148[30], 6}, {NULL}};
const Segment SGI169340[] = {{&S54148[120], 3}, {NULL}};
const Segment SGI169341[] = {{&S54148[85], 6}, {NULL}};
const Segment SGI169342[] = {{&S51282[34], 11}, {NULL}};
const Segment SGI169343[] = {{&S51282[27], 6}, {NULL}};
const Segment SGI169344[] = {{&S54149[38], 15}, {NULL}};
const Segment SGI169345[] = {{&S54149[31], 6}, {NULL}};
const Segment SGI169346[] = {{&S51282[98], 9}, {NULL}};
const Segment SGI169347[] = {{&S51282[71], 6}, {NULL}};
const Segment SGI169348[] = {{&S61789[44], 8}, {NULL}};
const Segment SGI169349[] = {{&S61789[44], 20}, {NULL}};
const Segment SGI169350[] = {{&S61789[30], 6}, {NULL}};
const Segment SGI169351[] = {{&S61789[31], 3}, {NULL}};
const Segment SGI169352[] = {{&S61789[78], 6}, {NULL}};
const Segment SGI169353[] = {{&S61794[34], 14}, {NULL}};
const Segment SGI169354[] = {{&S61794[27], 6}, {NULL}};
const Segment SGI169355[] = {{&S61794[101], 9}, {NULL}};
const Segment SGI169356[] = {{&S61794[74], 6}, {NULL}};
const Segment SGI169357[] = {{&S61801[34], 10}, {NULL}};
const Segment SGI169358[] = {{&S61801[27], 6}, {NULL}};
const Segment SGI169359[] = {{&S61801[97], 9}, {NULL}};
const Segment SGI169360[] = {{&S61801[70], 6}, {NULL}};
const Segment SGI169361[] = {{&S61810[44], 9}, {NULL}};
const Segment SGI169362[] = {{&S61810[44], 21}, {NULL}};
const Segment SGI169363[] = {{&S61810[30], 6}, {NULL}};
const Segment SGI169364[] = {{&S61810[31], 3}, {NULL}};
const Segment SGI169365[] = {{&S61810[98], 6}, {NULL}};
const Segment SGI169366[] = {{&S61810[67], 8}, {NULL}};
const Segment SGI169367[] = {{&S61811[44], 9}, {NULL}};
const Segment SGI169368[] = {{&S61811[44], 21}, {NULL}};
const Segment SGI169369[] = {{&S61811[30], 6}, {NULL}};
const Segment SGI169370[] = {{&S61811[31], 3}, {NULL}};
const Segment SGI169371[] = {{&S61811[98], 6}, {NULL}};
const Segment SGI169372[] = {{&S61811[67], 8}, {NULL}};
const Segment SGI169373[] = {{&S61812[44], 9}, {NULL}};
const Segment SGI169374[] = {{&S61812[44], 21}, {NULL}};
const Segment SGI169375[] = {{&S51323[34], 10}, {NULL}};
const Segment SGI169376[] = {{&S61812[30], 6}, {NULL}};
const Segment SGI169377[] = {{&S51323[27], 6}, {NULL}};
const Segment SGI169378[] = {{&S61812[31], 3}, {NULL}};
const Segment SGI169379[] = {{&S61812[100], 6}, {NULL}};
const Segment SGI169380[] = {{&S51323[97], 9}, {NULL}};
const Segment SGI169381[] = {{&S51323[70], 6}, {NULL}};
const Segment SGI169382[] = {{&S61812[67], 8}, {NULL}};
const Segment SGI169383[] = {{&S61813[44], 9}, {NULL}};
const Segment SGI169384[] = {{&S51324[34], 11}, {NULL}};
const Segment SGI169385[] = {{&S61813[44], 27}, {NULL}};
const Segment SGI169386[] = {{&S61813[30], 6}, {NULL}};
const Segment SGI169387[] = {{&S51324[27], 6}, {NULL}};
const Segment SGI169388[] = {{&S61813[31], 3}, {NULL}};
const Segment SGI169389[] = {{&S51324[98], 9}, {NULL}};
const Segment SGI169390[] = {{&S61813[104], 6}, {NULL}};
const Segment SGI169391[] = {{&S51324[71], 6}, {NULL}};
const Segment SGI169392[] = {{&S61813[73], 8}, {NULL}};
const Segment SGI169393[] = {{&S51325[34], 18}, {NULL}};
const Segment SGI169394[] = {{&S51325[27], 6}, {NULL}};
const Segment SGI169395[] = {{&S51325[105], 9}, {NULL}};
const Segment SGI169396[] = {{&S51325[78], 6}, {NULL}};
const Segment SGI169397[] = {{&S61820[44], 18}, {NULL}};
const Segment SGI169398[] = {{&S61820[44], 30}, {NULL}};
const Segment SGI169399[] = {{&S61820[30], 6}, {NULL}};
const Segment SGI169400[] = {{&S61820[31], 3}, {NULL}};
const Segment SGI169401[] = {{&S61820[103], 6}, {NULL}};
const Segment SGI169402[] = {{&S61820[76], 8}, {NULL}};
const Segment SGI169403[] = {{&S61821[44], 18}, {NULL}};
const Segment SGI169404[] = {{&S61821[44], 30}, {NULL}};
const Segment SGI169405[] = {{&S61821[30], 6}, {NULL}};
const Segment SGI169406[] = {{&S61821[31], 3}, {NULL}};
const Segment SGI169407[] = {{&S61821[107], 6}, {NULL}};
const Segment SGI169408[] = {{&S61821[76], 8}, {NULL}};
const Segment SGI169409[] = {{&S61822[44], 11}, {NULL}};
const Segment SGI169410[] = {{&S61822[44], 23}, {NULL}};
const Segment SGI169411[] = {{&S61822[30], 6}, {NULL}};
const Segment SGI169412[] = {{&S61822[31], 3}, {NULL}};
const Segment SGI169413[] = {{&S61822[96], 6}, {NULL}};
const Segment SGI169414[] = {{&S61822[69], 8}, {NULL}};
const Segment SGI169415[] = {{&S61823[44], 11}, {NULL}};
const Segment SGI169416[] = {{&S61823[44], 26}, {&S61823[71], 14}, {NULL}};
const Segment SGI169417[] = {{&S61823[30], 6}, {NULL}};
const Segment SGI169418[] = {{&S61823[31], 3}, {NULL}};
const Segment SGI169419[] = {{&S61823[99], 6}, {NULL}};
const Segment SGI169420[] = {{&S61823[72], 8}, {NULL}};
const Segment SGI169421[] = {{&S51342[44], 20}, {NULL}};
const Segment SGI169422[] = {{&S51342[44], 32}, {NULL}};
const Segment SGI169423[] = {{&S51342[30], 6}, {NULL}};
const Segment SGI169424[] = {{&S51342[125], 3}, {NULL}};
const Segment SGI169425[] = {{&S51342[90], 6}, {NULL}};
const Segment SGI169426[] = {{&S51345[38], 20}, {NULL}};
const Segment SGI169427[] = {{&S51345[31], 6}, {NULL}};
const Segment SGI169428[] = {{&S38903[41], 11}, {NULL}};
const Segment SGI169429[] = {{&S38903[41], 22}, {NULL}};
const Segment SGI169430[] = {{&S38903[27], 6}, {NULL}};
const Segment SGI169431[] = {{&S38903[112], 3}, {NULL}};
const Segment SGI169432[] = {{&S38903[116], 9}, {NULL}};
const Segment SGI169433[] = {{&S38903[77], 6}, {NULL}};
const Segment SGI169434[] = {{&S38904[41], 19}, {NULL}};
const Segment SGI169435[] = {{&S38904[41], 30}, {NULL}};
const Segment SGI169436[] = {{&S38904[27], 6}, {NULL}};
const Segment SGI169437[] = {{&S38904[120], 3}, {NULL}};
const Segment SGI169438[] = {{&S38904[124], 9}, {NULL}};
const Segment SGI169439[] = {{&S38904[85], 6}, {NULL}};
const Segment SGI169440[] = {{&S38905[41], 18}, {NULL}};
const Segment SGI169441[] = {{&S38905[41], 29}, {NULL}};
const Segment SGI169442[] = {{&S38905[27], 6}, {NULL}};
const Segment SGI169443[] = {{&S38905[119], 3}, {NULL}};
const Segment SGI169444[] = {{&S38905[123], 9}, {NULL}};
const Segment SGI169445[] = {{&S38905[84], 6}, {NULL}};
const Segment SGI169446[] = {{&S61852[44], 7}, {NULL}};
const Segment SGI169447[] = {{&S61852[44], 19}, {NULL}};
const Segment SGI169448[] = {{&S61852[30], 6}, {NULL}};
const Segment SGI169449[] = {{&S51362[34], 12}, {NULL}};
const Segment SGI169450[] = {{&S51362[27], 6}, {NULL}};
const Segment SGI169451[] = {{&S61852[31], 3}, {NULL}};
const Segment SGI169452[] = {{&S61852[77], 6}, {NULL}};
const Segment SGI169453[] = {{&S51362[99], 9}, {NULL}};
const Segment SGI169454[] = {{&S51362[72], 6}, {NULL}};
const Segment SGI169455[] = {{&S61853[38], 7}, {NULL}};
const Segment SGI169456[] = {{&S61853[31], 6}, {NULL}};
const Segment SGI169457[] = {{&S61854[44], 7}, {NULL}};
const Segment SGI169458[] = {{&S61854[44], 19}, {NULL}};
const Segment SGI169459[] = {{&S61854[30], 6}, {NULL}};
const Segment SGI169460[] = {{&S61854[31], 3}, {NULL}};
const Segment SGI169461[] = {{&S61854[77], 6}, {NULL}};
const Segment SGI169462[] = {{&S61855[34], 10}, {NULL}};
const Segment SGI169463[] = {{&S61855[27], 6}, {NULL}};
const Segment SGI169464[] = {{&S61855[97], 9}, {NULL}};
const Segment SGI169465[] = {{&S61855[70], 6}, {NULL}};
const Segment SGI169466[] = {{&S61856[34], 11}, {NULL}};
const Segment SGI169467[] = {{&S61856[27], 6}, {NULL}};
const Segment SGI169468[] = {{&S61856[98], 9}, {NULL}};
const Segment SGI169469[] = {{&S61856[71], 6}, {NULL}};
const Segment SGI169470[] = {{&S61859[34], 7}, {NULL}};
const Segment SGI169471[] = {{&S61859[27], 6}, {NULL}};
const Segment SGI169472[] = {{&S61859[96], 9}, {NULL}};
const Segment SGI169473[] = {{&S61859[69], 6}, {NULL}};
const Segment SGI169474[] = {{&S61860[34], 18}, {NULL}};
const Segment SGI169475[] = {{&S61860[27], 6}, {NULL}};
const Segment SGI169476[] = {{&S61860[105], 9}, {NULL}};
const Segment SGI169477[] = {{&S61860[78], 6}, {NULL}};
const Segment SGI169478[] = {{&S61862[34], 10}, {NULL}};
const Segment SGI169479[] = {{&S61862[27], 6}, {NULL}};
const Segment SGI169480[] = {{&S61862[97], 9}, {NULL}};
const Segment SGI169481[] = {{&S61862[70], 6}, {NULL}};
const Segment SGI169482[] = {{&S48337[44], 10}, {NULL}};
const Segment SGI169483[] = {{&S48337[44], 22}, {NULL}};
const Segment SGI169484[] = {{&S48337[30], 6}, {NULL}};
const Segment SGI169485[] = {{&S61875[44], 10}, {NULL}};
const Segment SGI169486[] = {{&S61875[44], 22}, {NULL}};
const Segment SGI169487[] = {{&S61875[30], 6}, {NULL}};
const Segment SGI169488[] = {{&S48337[144], 3}, {NULL}};
const Segment SGI169489[] = {{&S61875[31], 3}, {NULL}};
const Segment SGI169490[] = {{&S48337[109], 6}, {NULL}};
const Segment SGI169491[] = {{&S61875[80], 6}, {NULL}};
const Segment SGI169492[] = {{&S61876[38], 10}, {NULL}};
const Segment SGI169493[] = {{&S48338[44], 10}, {NULL}};
const Segment SGI169494[] = {{&S61876[31], 6}, {NULL}};
const Segment SGI169495[] = {{&S48338[44], 22}, {NULL}};
const Segment SGI169496[] = {{&S48338[30], 6}, {NULL}};
const Segment SGI169497[] = {{&S48338[144], 3}, {NULL}};
const Segment SGI169498[] = {{&S48338[109], 6}, {NULL}};
const Segment SGI169499[] = {{&S48344[44], 11}, {NULL}};
const Segment SGI169500[] = {{&S48344[44], 23}, {NULL}};
const Segment SGI169501[] = {{&S48344[30], 6}, {NULL}};
const Segment SGI169502[] = {{&S48344[145], 3}, {NULL}};
const Segment SGI169503[] = {{&S48344[110], 6}, {NULL}};
const Segment SGI169504[] = {{&S61895[44], 9}, {NULL}};
const Segment SGI169505[] = {{&S61895[44], 28}, {NULL}};
const Segment SGI169506[] = {{&S61895[30], 6}, {NULL}};
const Segment SGI169507[] = {{&S61895[31], 3}, {NULL}};
const Segment SGI169508[] = {{&S61895[86], 6}, {NULL}};
const Segment SGI169509[] = {{&S61911[44], 7}, {NULL}};
const Segment SGI169510[] = {{&S61911[44], 19}, {NULL}};
const Segment SGI169511[] = {{&S61911[30], 6}, {NULL}};
const Segment SGI169512[] = {{&S61911[31], 3}, {NULL}};
const Segment SGI169513[] = {{&S61911[77], 6}, {NULL}};
const Segment SGI169514[] = {{&S54321[44], 7}, {NULL}};
const Segment SGI169515[] = {{&S54321[44], 19}, {NULL}};
const Segment SGI169516[] = {{&S54321[30], 6}, {NULL}};
const Segment SGI169517[] = {{&S54321[128], 3}, {NULL}};
const Segment SGI169518[] = {{&S54321[93], 6}, {NULL}};
const Segment SGI169519[] = {{&S54321[65], 8}, {NULL}};
const Segment SGI169520[] = {{&S61942[34], 14}, {NULL}};
const Segment SGI169521[] = {{&S61942[27], 6}, {NULL}};
const Segment SGI169522[] = {{&S61942[101], 9}, {NULL}};
const Segment SGI169523[] = {{&S61942[74], 6}, {NULL}};
const Segment SGI169524[] = {{&S61943[34], 14}, {NULL}};
const Segment SGI169525[] = {{&S61943[27], 6}, {NULL}};
const Segment SGI169526[] = {{&S61943[101], 9}, {NULL}};
const Segment SGI169527[] = {{&S61943[74], 6}, {NULL}};
const Segment SGI169528[] = {{&S61945[34], 15}, {NULL}};
const Segment SGI169529[] = {{&S61945[27], 6}, {NULL}};
const Segment SGI169530[] = {{&S61945[102], 9}, {NULL}};
const Segment SGI169531[] = {{&S61945[75], 6}, {NULL}};
const Segment SGI169532[] = {{&S61946[34], 15}, {NULL}};
const Segment SGI169533[] = {{&S61946[27], 6}, {NULL}};
const Segment SGI169534[] = {{&S61946[102], 9}, {NULL}};
const Segment SGI169535[] = {{&S61946[75], 6}, {NULL}};
const Segment SGI169536[] = {{&S61947[34], 15}, {NULL}};
const Segment SGI169537[] = {{&S61947[27], 6}, {NULL}};
const Segment SGI169538[] = {{&S61947[102], 9}, {NULL}};
const Segment SGI169539[] = {{&S61947[75], 6}, {NULL}};
const Segment SGI169540[] = {{&S54353[44], 10}, {NULL}};
const Segment SGI169541[] = {{&S54353[44], 22}, {NULL}};
const Segment SGI169542[] = {{&S54353[30], 6}, {NULL}};
const Segment SGI169543[] = {{&S54353[115], 3}, {NULL}};
const Segment SGI169544[] = {{&S54353[80], 6}, {NULL}};
const Segment SGI169545[] = {{&S54353[140], 11}, {NULL}};
const Segment SGI169546[] = {{&S54354[44], 10}, {NULL}};
const Segment SGI169547[] = {{&S54354[44], 22}, {NULL}};
const Segment SGI169548[] = {{&S54354[30], 6}, {NULL}};
const Segment SGI169549[] = {{&S54354[115], 3}, {NULL}};
const Segment SGI169550[] = {{&S54354[80], 6}, {NULL}};
const Segment SGI169551[] = {{&S54354[140], 11}, {NULL}};
const Segment SGI169552[] = {{&S61958[34], 11}, {NULL}};
const Segment SGI169553[] = {{&S61958[27], 6}, {NULL}};
const Segment SGI169554[] = {{&S61958[98], 9}, {NULL}};
const Segment SGI169555[] = {{&S61958[71], 6}, {NULL}};
const Segment SGI169556[] = {{&S61959[34], 14}, {NULL}};
const Segment SGI169557[] = {{&S61959[27], 6}, {NULL}};
const Segment SGI169558[] = {{&S61959[101], 9}, {NULL}};
const Segment SGI169559[] = {{&S61959[74], 6}, {NULL}};
const Segment SGI169560[] = {{&S54382[34], 29}, {NULL}};
const Segment SGI169561[] = {{&S54382[27], 6}, {NULL}};
const Segment SGI169562[] = {{&S54382[116], 9}, {NULL}};
const Segment SGI169563[] = {{&S54382[89], 6}, {NULL}};
const Segment SGI169564[] = {{&S54383[34], 30}, {NULL}};
const Segment SGI169565[] = {{&S54383[27], 6}, {NULL}};
const Segment SGI169566[] = {{&S54383[117], 9}, {NULL}};
const Segment SGI169567[] = {{&S54383[90], 6}, {NULL}};
const Segment SGI169568[] = {{&S54384[34], 30}, {NULL}};
const Segment SGI169569[] = {{&S54384[27], 6}, {NULL}};
const Segment SGI169570[] = {{&S54384[117], 9}, {NULL}};
const Segment SGI169571[] = {{&S54384[90], 6}, {NULL}};
const Segment SGI169572[] = {{&S54415[44], 10}, {NULL}};
const Segment SGI169573[] = {{&S54415[44], 22}, {NULL}};
const Segment SGI169574[] = {{&S54415[30], 6}, {NULL}};
const Segment SGI169575[] = {{&S54415[80], 6}, {NULL}};
const Segment SGI169576[] = {{&S54415[107], 13}, {NULL}};
const Segment SGI169577[] = {{&S54418[44], 10}, {NULL}};
const Segment SGI169578[] = {{&S54418[44], 22}, {NULL}};
const Segment SGI169579[] = {{&S54418[30], 6}, {NULL}};
const Segment SGI169580[] = {{&S54418[80], 6}, {NULL}};
const Segment SGI169581[] = {{&S54418[107], 13}, {NULL}};
const Segment SGI169582[] = {{&S54421[34], 10}, {NULL}};
const Segment SGI169583[] = {{&S54421[27], 6}, {NULL}};
const Segment SGI169584[] = {{&S54421[97], 9}, {NULL}};
const Segment SGI169585[] = {{&S54421[70], 6}, {NULL}};
const Segment SGI169586[] = {{&S61990[34], 14}, {NULL}};
const Segment SGI169587[] = {{&S61990[27], 6}, {NULL}};
const Segment SGI169588[] = {{&S61990[101], 9}, {NULL}};
const Segment SGI169589[] = {{&S61990[74], 6}, {NULL}};
const Segment SGI169590[] = {{&S62014[44], 12}, {NULL}};
const Segment SGI169591[] = {{&S62014[44], 22}, {NULL}};
const Segment SGI169592[] = {{&S62014[30], 6}, {NULL}};
const Segment SGI169593[] = {{&S62014[31], 3}, {NULL}};
const Segment SGI169594[] = {{&S62014[80], 6}, {NULL}};
const Segment SGI169595[] = {{&S62015[44], 12}, {NULL}};
const Segment SGI169596[] = {{&S62015[44], 22}, {NULL}};
const Segment SGI169597[] = {{&S62015[30], 6}, {NULL}};
const Segment SGI169598[] = {{&S62015[31], 3}, {NULL}};
const Segment SGI169599[] = {{&S62015[80], 6}, {NULL}};
const Segment SGI169600[] = {{&S62020[44], 6}, {NULL}};
const Segment SGI169601[] = {{&S62020[44], 18}, {NULL}};
const Segment SGI169602[] = {{&S62020[30], 6}, {NULL}};
const Segment SGI169603[] = {{&S62020[31], 3}, {NULL}};
const Segment SGI169604[] = {{&S62020[76], 6}, {NULL}};
const Segment SGI169605[] = {{&S62021[44], 11}, {NULL}};
const Segment SGI169606[] = {{&S62021[44], 23}, {NULL}};
const Segment SGI169607[] = {{&S62021[30], 6}, {NULL}};
const Segment SGI169608[] = {{&S62021[31], 3}, {NULL}};
const Segment SGI169609[] = {{&S62021[81], 6}, {NULL}};
const Segment SGI169610[] = {{&S62022[44], 6}, {NULL}};
const Segment SGI169611[] = {{&S62022[44], 18}, {NULL}};
const Segment SGI169612[] = {{&S62022[30], 6}, {NULL}};
const Segment SGI169613[] = {{&S62022[79], 6}, {NULL}};
const Segment SGI169614[] = {{&S62023[44], 7}, {NULL}};
const Segment SGI169615[] = {{&S62023[44], 19}, {NULL}};
const Segment SGI169616[] = {{&S62023[30], 6}, {NULL}};
const Segment SGI169617[] = {{&S62023[31], 3}, {NULL}};
const Segment SGI169618[] = {{&S62023[77], 6}, {NULL}};
const Segment SGI169619[] = {{&S62024[38], 6}, {NULL}};
const Segment SGI169620[] = {{&S62024[31], 6}, {NULL}};
const Segment SGI169621[] = {{&S62025[38], 7}, {NULL}};
const Segment SGI169622[] = {{&S62025[31], 6}, {NULL}};
const Segment SGI169623[] = {{&S62026[44], 7}, {NULL}};
const Segment SGI169624[] = {{&S62026[44], 19}, {NULL}};
const Segment SGI169625[] = {{&S62026[30], 6}, {NULL}};
const Segment SGI169626[] = {{&S62026[31], 3}, {NULL}};
const Segment SGI169627[] = {{&S62026[77], 6}, {NULL}};
const Segment SGI169628[] = {{&S62044[44], 9}, {NULL}};
const Segment SGI169629[] = {{&S62044[44], 21}, {NULL}};
const Segment SGI169630[] = {{&S62044[30], 6}, {NULL}};
const Segment SGI169631[] = {{&S62044[31], 3}, {NULL}};
const Segment SGI169632[] = {{&S62044[98], 6}, {NULL}};
const Segment SGI169633[] = {{&S62044[67], 8}, {NULL}};
const Segment SGI169634[] = {{&S62045[44], 9}, {NULL}};
const Segment SGI169635[] = {{&S62045[44], 21}, {NULL}};
const Segment SGI169636[] = {{&S62045[30], 6}, {NULL}};
const Segment SGI169637[] = {{&S62045[31], 3}, {NULL}};
const Segment SGI169638[] = {{&S62045[94], 6}, {NULL}};
const Segment SGI169639[] = {{&S62045[67], 8}, {NULL}};
const Segment SGI169640[] = {{&S62046[44], 9}, {NULL}};
const Segment SGI169641[] = {{&S62046[44], 21}, {NULL}};
const Segment SGI169642[] = {{&S62046[30], 6}, {NULL}};
const Segment SGI169643[] = {{&S62046[31], 3}, {NULL}};
const Segment SGI169644[] = {{&S62046[98], 6}, {NULL}};
const Segment SGI169645[] = {{&S62046[67], 8}, {NULL}};
const Segment SGI169646[] = {{&S54515[34], 11}, {NULL}};
const Segment SGI169647[] = {{&S54515[27], 6}, {NULL}};
const Segment SGI169648[] = {{&S48590[34], 10}, {NULL}};
const Segment SGI169649[] = {{&S48590[27], 6}, {NULL}};
const Segment SGI169650[] = {{&S54515[98], 9}, {NULL}};
const Segment SGI169651[] = {{&S54515[71], 6}, {NULL}};
const Segment SGI169652[] = {{&S48590[97], 9}, {NULL}};
const Segment SGI169653[] = {{&S48590[70], 6}, {NULL}};
const Segment SGI169654[] = {{&S54516[34], 11}, {NULL}};
const Segment SGI169655[] = {{&S54516[27], 6}, {NULL}};
const Segment SGI169656[] = {{&S54516[98], 9}, {NULL}};
const Segment SGI169657[] = {{&S54516[71], 6}, {NULL}};
const Segment SGI169658[] = {{&S48593[34], 18}, {NULL}};
const Segment SGI169659[] = {{&S48593[27], 6}, {NULL}};
const Segment SGI169660[] = {{&S48593[105], 9}, {NULL}};
const Segment SGI169661[] = {{&S48593[78], 6}, {NULL}};
const Segment SGI169662[] = {{&S62065[44], 14}, {NULL}};
const Segment SGI169663[] = {{&S62065[44], 26}, {NULL}};
const Segment SGI169664[] = {{&S62065[30], 6}, {NULL}};
const Segment SGI169665[] = {{&S62065[31], 3}, {NULL}};
const Segment SGI169666[] = {{&S62065[84], 6}, {NULL}};
const Segment SGI169667[] = {{&S62066[44], 10}, {NULL}};
const Segment SGI169668[] = {{&S48596[34], 10}, {NULL}};
const Segment SGI169669[] = {{&S62066[44], 22}, {NULL}};
const Segment SGI169670[] = {{&S62066[30], 6}, {NULL}};
const Segment SGI169671[] = {{&S48596[27], 6}, {NULL}};
const Segment SGI169672[] = {{&S62066[31], 3}, {NULL}};
const Segment SGI169673[] = {{&S48596[97], 9}, {NULL}};
const Segment SGI169674[] = {{&S62066[80], 6}, {NULL}};
const Segment SGI169675[] = {{&S48596[70], 6}, {NULL}};
const Segment SGI169676[] = {{&S62091[45], 10}, {NULL}};
const Segment SGI169677[] = {{&S62091[30], 6}, {NULL}};
const Segment SGI169678[] = {{&S62091[31], 3}, {NULL}};
const Segment SGI169679[] = {{&S62091[81], 5}, {NULL}};
const Segment SGI169680[] = {{&S62091[139], 13}, {NULL}};
const Segment SGI169681[] = {{&S54552[11], 1}, {NULL}};
const Segment SGI169682[] = {{&S54552[27], 6}, {NULL}};
const Segment SGI169683[] = {{&S54552[88], 9}, {NULL}};
const Segment SGI169684[] = {{&S54552[61], 6}, {NULL}};
const Segment SGI169685[] = {{&S62092[44], 10}, {NULL}};
const Segment SGI169686[] = {{&S62092[44], 22}, {NULL}};
const Segment SGI169687[] = {{&S62092[30], 6}, {NULL}};
const Segment SGI169688[] = {{&S62092[115], 3}, {NULL}};
const Segment SGI169689[] = {{&S62092[80], 6}, {NULL}};
const Segment SGI169690[] = {{&S62092[135], 8}, {NULL}};
const Segment SGI169691[] = {{&S62092[144], 13}, {NULL}};
const Segment SGI169692[] = {{&S54553[34], 7}, {NULL}};
const Segment SGI169693[] = {{&S54553[27], 6}, {NULL}};
const Segment SGI169694[] = {{&S54553[94], 9}, {NULL}};
const Segment SGI169695[] = {{&S54553[67], 6}, {NULL}};
const Segment SGI169696[] = {{&S54554[34], 16}, {NULL}};
const Segment SGI169697[] = {{&S54554[27], 6}, {NULL}};
const Segment SGI169698[] = {{&S54554[103], 9}, {NULL}};
const Segment SGI169699[] = {{&S54554[76], 6}, {NULL}};
const Segment SGI169700[] = {{&S54556[34], 31}, {NULL}};
const Segment SGI169701[] = {{&S54556[27], 6}, {NULL}};
const Segment SGI169702[] = {{&S54556[118], 9}, {NULL}};
const Segment SGI169703[] = {{&S54556[91], 6}, {NULL}};
const Segment SGI169704[] = {{&S54557[34], 25}, {NULL}};
const Segment SGI169705[] = {{&S54557[27], 6}, {NULL}};
const Segment SGI169706[] = {{&S54557[112], 9}, {NULL}};
const Segment SGI169707[] = {{&S54557[85], 6}, {NULL}};
const Segment SGI169708[] = {{&S54558[34], 31}, {NULL}};
const Segment SGI169709[] = {{&S54558[27], 6}, {NULL}};
const Segment SGI169710[] = {{&S54558[118], 9}, {NULL}};
const Segment SGI169711[] = {{&S54558[91], 6}, {NULL}};
const Segment SGI169712[] = {{&S54559[34], 27}, {NULL}};
const Segment SGI169713[] = {{&S54559[27], 6}, {NULL}};
const Segment SGI169714[] = {{&S54559[114], 9}, {NULL}};
const Segment SGI169715[] = {{&S62105[44], 8}, {NULL}};
const Segment SGI169716[] = {{&S54559[87], 6}, {NULL}};
const Segment SGI169717[] = {{&S62105[44], 20}, {NULL}};
const Segment SGI169718[] = {{&S62105[30], 6}, {NULL}};
const Segment SGI169719[] = {{&S62105[31], 3}, {NULL}};
const Segment SGI169720[] = {{&S62105[78], 6}, {NULL}};
const Segment SGI169721[] = {{&S54560[34], 27}, {NULL}};
const Segment SGI169722[] = {{&S54560[27], 6}, {NULL}};
const Segment SGI169723[] = {{&S62106[38], 8}, {NULL}};
const Segment SGI169724[] = {{&S62106[31], 6}, {NULL}};
const Segment SGI169725[] = {{&S54560[114], 9}, {NULL}};
const Segment SGI169726[] = {{&S54560[87], 6}, {NULL}};
const Segment SGI169727[] = {{&S54561[11], 1}, {NULL}};
const Segment SGI169728[] = {{&S54561[27], 6}, {NULL}};
const Segment SGI169729[] = {{&S54561[88], 9}, {NULL}};
const Segment SGI169730[] = {{&S54561[61], 6}, {NULL}};
const Segment SGI169731[] = {{&S54562[34], 25}, {NULL}};
const Segment SGI169732[] = {{&S54562[27], 6}, {NULL}};
const Segment SGI169733[] = {{&S54562[112], 9}, {NULL}};
const Segment SGI169734[] = {{&S54562[85], 6}, {NULL}};
const Segment SGI169735[] = {{&S54563[34], 18}, {NULL}};
const Segment SGI169736[] = {{&S54563[27], 6}, {NULL}};
const Segment SGI169737[] = {{&S54563[105], 9}, {NULL}};
const Segment SGI169738[] = {{&S54563[78], 6}, {NULL}};
const Segment SGI169739[] = {{&S62113[34], 10}, {NULL}};
const Segment SGI169740[] = {{&S62113[27], 6}, {NULL}};
const Segment SGI169741[] = {{&S62113[97], 9}, {NULL}};
const Segment SGI169742[] = {{&S62113[70], 6}, {NULL}};
const Segment SGI169743[] = {{&S54565[34], 25}, {NULL}};
const Segment SGI169744[] = {{&S54565[27], 6}, {NULL}};
const Segment SGI169745[] = {{&S54565[112], 9}, {NULL}};
const Segment SGI169746[] = {{&S54565[85], 6}, {NULL}};
const Segment SGI169747[] = {{&S62114[34], 13}, {NULL}};
const Segment SGI169748[] = {{&S62114[27], 6}, {NULL}};
const Segment SGI169749[] = {{&S62114[100], 9}, {NULL}};
const Segment SGI169750[] = {{&S62114[73], 6}, {NULL}};
const Segment SGI169751[] = {{&S54566[34], 22}, {NULL}};
const Segment SGI169752[] = {{&S54566[27], 6}, {NULL}};
const Segment SGI169753[] = {{&S54566[109], 9}, {NULL}};
const Segment SGI169754[] = {{&S54566[82], 6}, {NULL}};
const Segment SGI169755[] = {{&S54568[34], 21}, {NULL}};
const Segment SGI169756[] = {{&S54568[27], 6}, {NULL}};
const Segment SGI169757[] = {{&S54568[108], 9}, {NULL}};
const Segment SGI169758[] = {{&S54568[81], 6}, {NULL}};
const Segment SGI169759[] = {{&S54569[34], 16}, {NULL}};
const Segment SGI169760[] = {{&S54569[27], 6}, {NULL}};
const Segment SGI169761[] = {{&S54569[103], 9}, {NULL}};
const Segment SGI169762[] = {{&S54569[76], 6}, {NULL}};
const Segment SGI169763[] = {{&S54570[34], 25}, {NULL}};
const Segment SGI169764[] = {{&S54570[27], 6}, {NULL}};
const Segment SGI169765[] = {{&S54570[112], 9}, {NULL}};
const Segment SGI169766[] = {{&S54570[85], 6}, {NULL}};
const Segment SGI169767[] = {{&S62121[44], 18}, {NULL}};
const Segment SGI169768[] = {{&S62121[44], 27}, {NULL}};
const Segment SGI169769[] = {{&S62121[30], 6}, {NULL}};
const Segment SGI169770[] = {{&S62121[31], 3}, {NULL}};
const Segment SGI169771[] = {{&S62121[85], 6}, {NULL}};
const Segment SGI169772[] = {{&S51443[44], 16}, {NULL}};
const Segment SGI169773[] = {{&S51443[44], 35}, {NULL}};
const Segment SGI169774[] = {{&S51443[30], 6}, {NULL}};
const Segment SGI169775[] = {{&S51443[128], 3}, {NULL}};
const Segment SGI169776[] = {{&S51443[93], 6}, {NULL}};
const Segment SGI169777[] = {{&S51444[38], 16}, {NULL}};
const Segment SGI169778[] = {{&S51444[31], 6}, {NULL}};
const Segment SGI169779[] = {{&S51445[44], 16}, {NULL}};
const Segment SGI169780[] = {{&S51445[44], 35}, {NULL}};
const Segment SGI169781[] = {{&S51445[30], 6}, {NULL}};
const Segment SGI169782[] = {{&S51445[128], 3}, {NULL}};
const Segment SGI169783[] = {{&S51445[93], 6}, {NULL}};
const Segment SGI169784[] = {{&S51446[44], 16}, {NULL}};
const Segment SGI169785[] = {{&S51446[44], 42}, {NULL}};
const Segment SGI169786[] = {{&S51446[30], 6}, {NULL}};
const Segment SGI169787[] = {{&S51446[135], 3}, {NULL}};
const Segment SGI169788[] = {{&S51446[100], 6}, {NULL}};
const Segment SGI169789[] = {{&S51447[44], 9}, {NULL}};
const Segment SGI169790[] = {{&S51447[44], 21}, {NULL}};
const Segment SGI169791[] = {{&S51447[30], 6}, {NULL}};
const Segment SGI169792[] = {{&S51447[114], 3}, {NULL}};
const Segment SGI169793[] = {{&S51447[79], 6}, {NULL}};
const Segment SGI169794[] = {{&S51448[38], 9}, {NULL}};
const Segment SGI169795[] = {{&S51448[31], 6}, {NULL}};
const Segment SGI169796[] = {{&S51449[34], 10}, {NULL}};
const Segment SGI169797[] = {{&S51449[27], 6}, {NULL}};
const Segment SGI169798[] = {{&S51449[97], 9}, {NULL}};
const Segment SGI169799[] = {{&S51449[70], 6}, {NULL}};
const Segment SGI169800[] = {{&S51450[34], 11}, {NULL}};
const Segment SGI169801[] = {{&S51450[27], 6}, {NULL}};
const Segment SGI169802[] = {{&S51450[98], 9}, {NULL}};
const Segment SGI169803[] = {{&S51450[71], 6}, {NULL}};
const Segment SGI169804[] = {{&S62149[44], 10}, {NULL}};
const Segment SGI169805[] = {{&S62149[44], 22}, {NULL}};
const Segment SGI169806[] = {{&S62149[30], 6}, {NULL}};
const Segment SGI169807[] = {{&S62149[31], 3}, {NULL}};
const Segment SGI169808[] = {{&S62149[80], 6}, {NULL}};
const Segment SGI169809[] = {{&S62149[150], 3}, {NULL}};
const Segment SGI169810[] = {{&S62150[44], 18}, {NULL}};
const Segment SGI169811[] = {{&S62150[44], 30}, {NULL}};
const Segment SGI169812[] = {{&S62150[30], 6}, {NULL}};
const Segment SGI169813[] = {{&S62150[31], 3}, {NULL}};
const Segment SGI169814[] = {{&S62150[88], 6}, {NULL}};
const Segment SGI169815[] = {{&S62150[158], 3}, {NULL}};
const Segment SGI169816[] = {{&S62151[44], 11}, {NULL}};
const Segment SGI169817[] = {{&S62151[44], 23}, {NULL}};
const Segment SGI169818[] = {{&S62151[30], 6}, {NULL}};
const Segment SGI169819[] = {{&S62151[31], 3}, {NULL}};
const Segment SGI169820[] = {{&S62151[81], 6}, {NULL}};
const Segment SGI169821[] = {{&S62151[151], 3}, {NULL}};
const Segment SGI169822[] = {{&S62164[44], 11}, {NULL}};
const Segment SGI169823[] = {{&S62164[44], 23}, {NULL}};
const Segment SGI169824[] = {{&S62164[30], 6}, {NULL}};
const Segment SGI169825[] = {{&S62164[31], 3}, {NULL}};
const Segment SGI169826[] = {{&S62164[81], 6}, {NULL}};
const Segment SGI169827[] = {{&S62166[44], 10}, {NULL}};
const Segment SGI169828[] = {{&S62166[44], 31}, {NULL}};
const Segment SGI169829[] = {{&S62166[30], 6}, {NULL}};
const Segment SGI169830[] = {{&S62166[31], 3}, {NULL}};
const Segment SGI169831[] = {{&S62166[89], 5}, {NULL}};
const Segment SGI169832[] = {{&S62167[44], 77}, {NULL}};
const Segment SGI169833[] = {{&S62167[44], 98}, {NULL}};
const Segment SGI169834[] = {{&S62167[30], 6}, {NULL}};
const Segment SGI169835[] = {{&S62167[31], 3}, {NULL}};
const Segment SGI169836[] = {{&S62167[115], 5}, {NULL}};
const Segment SGI169837[] = {{&S62168[38], 10}, {NULL}};
const Segment SGI169838[] = {{&S62168[31], 6}, {NULL}};
const Segment SGI169839[] = {{&S62169[44], 77}, {NULL}};
const Segment SGI169840[] = {{&S62169[44], 98}, {NULL}};
const Segment SGI169841[] = {{&S62169[30], 6}, {NULL}};
const Segment SGI169842[] = {{&S62169[31], 3}, {NULL}};
const Segment SGI169843[] = {{&S62169[115], 5}, {NULL}};
const Segment SGI169844[] = {{&S62170[44], 77}, {NULL}};
const Segment SGI169845[] = {{&S62170[44], 98}, {NULL}};
const Segment SGI169846[] = {{&S62170[30], 6}, {NULL}};
const Segment SGI169847[] = {{&S62170[31], 3}, {NULL}};
const Segment SGI169848[] = {{&S62170[115], 5}, {NULL}};
const Segment SGI169849[] = {{&S62171[44], 77}, {NULL}};
const Segment SGI169850[] = {{&S62171[44], 98}, {NULL}};
const Segment SGI169851[] = {{&S62171[30], 6}, {NULL}};
const Segment SGI169852[] = {{&S62171[31], 3}, {NULL}};
const Segment SGI169853[] = {{&S62171[115], 5}, {NULL}};
const Segment SGI169854[] = {{&S62172[44], 77}, {NULL}};
const Segment SGI169855[] = {{&S62172[44], 98}, {NULL}};
const Segment SGI169856[] = {{&S62172[30], 6}, {NULL}};
const Segment SGI169857[] = {{&S62172[31], 3}, {NULL}};
const Segment SGI169858[] = {{&S62172[115], 5}, {NULL}};
const Segment SGI169859[] = {{&S62173[44], 77}, {NULL}};
const Segment SGI169860[] = {{&S62173[44], 98}, {NULL}};
const Segment SGI169861[] = {{&S62173[30], 6}, {NULL}};
const Segment SGI169862[] = {{&S62173[31], 3}, {NULL}};
const Segment SGI169863[] = {{&S62173[115], 5}, {NULL}};
const Segment SGI169864[] = {{&S62174[44], 77}, {NULL}};
const Segment SGI169865[] = {{&S62174[44], 98}, {NULL}};
const Segment SGI169866[] = {{&S62174[30], 6}, {NULL}};
const Segment SGI169867[] = {{&S62174[31], 3}, {NULL}};
const Segment SGI169868[] = {{&S62174[115], 5}, {NULL}};
const Segment SGI169869[] = {{&S62175[44], 77}, {NULL}};
const Segment SGI169870[] = {{&S54649[44], 8}, {NULL}};
const Segment SGI169871[] = {{&S62175[44], 98}, {NULL}};
const Segment SGI169872[] = {{&S54649[44], 20}, {NULL}};
const Segment SGI169873[] = {{&S54649[30], 6}, {NULL}};
const Segment SGI169874[] = {{&S62175[30], 6}, {NULL}};
const Segment SGI169875[] = {{&S54649[113], 3}, {NULL}};
const Segment SGI169876[] = {{&S62175[31], 3}, {NULL}};
const Segment SGI169877[] = {{&S54649[78], 6}, {NULL}};
const Segment SGI169878[] = {{&S62175[115], 5}, {NULL}};
const Segment SGI169879[] = {{&S54650[54], 8}, {NULL}};
const Segment SGI169880[] = {{&S54650[47], 6}, {NULL}};
const Segment SGI169881[] = {{&S54651[44], 9}, {NULL}};
const Segment SGI169882[] = {{&S54651[44], 21}, {NULL}};
const Segment SGI169883[] = {{&S54651[30], 6}, {NULL}};
const Segment SGI169884[] = {{&S54651[114], 3}, {NULL}};
const Segment SGI169885[] = {{&S54651[79], 6}, {NULL}};
const Segment SGI169886[] = {{&S54652[38], 9}, {NULL}};
const Segment SGI169887[] = {{&S54652[31], 6}, {NULL}};
const Segment SGI169888[] = {{&S62182[34], 10}, {NULL}};
const Segment SGI169889[] = {{&S62182[27], 6}, {NULL}};
const Segment SGI169890[] = {{&S62182[97], 9}, {NULL}};
const Segment SGI169891[] = {{&S62182[70], 6}, {NULL}};
const Segment SGI169892[] = {{&S62183[34], 10}, {NULL}};
const Segment SGI169893[] = {{&S62183[27], 6}, {NULL}};
const Segment SGI169894[] = {{&S62183[97], 9}, {NULL}};
const Segment SGI169895[] = {{&S62183[70], 6}, {NULL}};
const Segment SGI169896[] = {{&S62186[44], 12}, {NULL}};
const Segment SGI169897[] = {{&S62186[44], 24}, {NULL}};
const Segment SGI169898[] = {{&S62186[30], 6}, {NULL}};
const Segment SGI169899[] = {{&S62186[31], 3}, {NULL}};
const Segment SGI169900[] = {{&S62186[82], 6}, {NULL}};
const Segment SGI169901[] = {{&S62187[44], 14}, {NULL}};
const Segment SGI169902[] = {{&S62187[44], 26}, {NULL}};
const Segment SGI169903[] = {{&S62187[30], 6}, {NULL}};
const Segment SGI169904[] = {{&S62187[31], 3}, {NULL}};
const Segment SGI169905[] = {{&S62187[84], 6}, {NULL}};
const Segment SGI169906[] = {{&S54680[34], 9}, {NULL}};
const Segment SGI169907[] = {{&S54680[27], 6}, {NULL}};
const Segment SGI169908[] = {{&S54680[96], 9}, {NULL}};
const Segment SGI169909[] = {{&S54680[69], 6}, {NULL}};
const Segment SGI169910[] = {{&S54683[34], 11}, {NULL}};
const Segment SGI169911[] = {{&S54683[27], 6}, {NULL}};
const Segment SGI169912[] = {{&S54683[98], 9}, {NULL}};
const Segment SGI169913[] = {{&S54683[71], 6}, {NULL}};
const Segment SGI169914[] = {{&S51541[34], 11}, {NULL}};
const Segment SGI169915[] = {{&S51541[27], 6}, {NULL}};
const Segment SGI169916[] = {{&S54684[34], 11}, {NULL}};
const Segment SGI169917[] = {{&S54684[27], 6}, {NULL}};
const Segment SGI169918[] = {{&S51541[98], 9}, {NULL}};
const Segment SGI169919[] = {{&S51541[71], 6}, {NULL}};
const Segment SGI169920[] = {{&S54684[98], 9}, {NULL}};
const Segment SGI169921[] = {{&S54684[71], 6}, {NULL}};
const Segment SGI169922[] = {{&S51561[34], 10}, {NULL}};
const Segment SGI169923[] = {{&S51561[27], 6}, {NULL}};
const Segment SGI169924[] = {{&S51561[97], 9}, {NULL}};
const Segment SGI169925[] = {{&S51561[70], 6}, {NULL}};
const Segment SGI169926[] = {{&S54716[44], 10}, {NULL}};
const Segment SGI169927[] = {{&S51575[44], 10}, {NULL}};
const Segment SGI169928[] = {{&S54716[44], 22}, {NULL}};
const Segment SGI169929[] = {{&S51575[44], 22}, {NULL}};
const Segment SGI169930[] = {{&S54716[30], 6}, {NULL}};
const Segment SGI169931[] = {{&S51575[30], 6}, {NULL}};
const Segment SGI169932[] = {{&S51575[115], 3}, {NULL}};
const Segment SGI169933[] = {{&S54716[80], 6}, {NULL}};
const Segment SGI169934[] = {{&S51575[80], 6}, {NULL}};
const Segment SGI169935[] = {{&S54716[107], 13}, {NULL}};
const Segment SGI169936[] = {{&S51576[38], 10}, {NULL}};
const Segment SGI169937[] = {{&S51576[31], 6}, {NULL}};
const Segment SGI169938[] = {{&S51577[44], 10}, {NULL}};
const Segment SGI169939[] = {{&S51577[44], 22}, {NULL}};
const Segment SGI169940[] = {{&S51577[30], 6}, {NULL}};
const Segment SGI169941[] = {{&S51577[115], 3}, {NULL}};
const Segment SGI169942[] = {{&S51577[80], 6}, {NULL}};
const Segment SGI169943[] = {{&S51578[44], 11}, {NULL}};
const Segment SGI169944[] = {{&S51578[44], 23}, {NULL}};
const Segment SGI169945[] = {{&S51578[30], 6}, {NULL}};
const Segment SGI169946[] = {{&S51578[116], 3}, {NULL}};
const Segment SGI169947[] = {{&S54721[44], 10}, {NULL}};
const Segment SGI169948[] = {{&S51578[81], 6}, {NULL}};
const Segment SGI169949[] = {{&S54721[44], 22}, {NULL}};
const Segment SGI169950[] = {{&S54721[30], 6}, {NULL}};
const Segment SGI169951[] = {{&S54721[80], 6}, {NULL}};
const Segment SGI169952[] = {{&S54721[107], 13}, {NULL}};
const Segment SGI169953[] = {{&S51579[44], 10}, {NULL}};
const Segment SGI169954[] = {{&S51579[44], 22}, {NULL}};
const Segment SGI169955[] = {{&S51579[30], 6}, {NULL}};
const Segment SGI169956[] = {{&S51579[115], 3}, {NULL}};
const Segment SGI169957[] = {{&S51579[80], 6}, {NULL}};
const Segment SGI169958[] = {{&S51580[42], 10}, {NULL}};
const Segment SGI169959[] = {{&S62256[34], 10}, {NULL}};
const Segment SGI169960[] = {{&S51580[30], 6}, {NULL}};
const Segment SGI169961[] = {{&S62256[27], 6}, {NULL}};
const Segment SGI169962[] = {{&S51580[113], 3}, {NULL}};
const Segment SGI169963[] = {{&S51580[78], 6}, {NULL}};
const Segment SGI169964[] = {{&S62256[97], 9}, {NULL}};
const Segment SGI169965[] = {{&S62256[70], 6}, {NULL}};
const Segment SGI169966[] = {{&S51581[44], 11}, {NULL}};
const Segment SGI169967[] = {{&S62257[34], 6}, {NULL}};
const Segment SGI169968[] = {{&S51581[44], 23}, {NULL}};
const Segment SGI169969[] = {{&S51581[30], 6}, {NULL}};
const Segment SGI169970[] = {{&S62257[27], 6}, {NULL}};
const Segment SGI169971[] = {{&S51581[116], 3}, {NULL}};
const Segment SGI169972[] = {{&S62257[93], 9}, {NULL}};
const Segment SGI169973[] = {{&S51581[81], 6}, {NULL}};
const Segment SGI169974[] = {{&S62257[66], 6}, {NULL}};
const Segment SGI169975[] = {{&S62258[34], 7}, {NULL}};
const Segment SGI169976[] = {{&S51582[44], 10}, {NULL}};
const Segment SGI169977[] = {{&S51582[44], 29}, {NULL}};
const Segment SGI169978[] = {{&S62258[27], 6}, {NULL}};
const Segment SGI169979[] = {{&S51582[30], 6}, {NULL}};
const Segment SGI169980[] = {{&S51582[122], 3}, {NULL}};
const Segment SGI169981[] = {{&S62258[94], 9}, {NULL}};
const Segment SGI169982[] = {{&S62258[67], 6}, {NULL}};
const Segment SGI169983[] = {{&S51582[87], 6}, {NULL}};
const Segment SGI169984[] = {{&S54734[34], 10}, {NULL}};
const Segment SGI169985[] = {{&S54734[27], 6}, {NULL}};
const Segment SGI169986[] = {{&S54734[97], 9}, {NULL}};
const Segment SGI169987[] = {{&S54734[70], 6}, {NULL}};
const Segment SGI169988[] = {{&S51590[44], 10}, {NULL}};
const Segment SGI169989[] = {{&S51590[44], 22}, {NULL}};
const Segment SGI169990[] = {{&S51590[30], 6}, {NULL}};
const Segment SGI169991[] = {{&S51590[115], 3}, {NULL}};
const Segment SGI169992[] = {{&S51590[80], 6}, {NULL}};
const Segment SGI169993[] = {{&S51591[38], 10}, {NULL}};
const Segment SGI169994[] = {{&S51591[31], 6}, {NULL}};
const Segment SGI169995[] = {{&S51592[44], 10}, {NULL}};
const Segment SGI169996[] = {{&S51592[44], 22}, {NULL}};
const Segment SGI169997[] = {{&S51592[30], 6}, {NULL}};
const Segment SGI169998[] = {{&S51592[83], 6}, {NULL}};
const Segment SGI169999[] = {{&S51612[34], 5}, {NULL}};
const Segment SGI170000[] = {{&S51612[27], 6}, {NULL}};
const Segment SGI170001[] = {{&S51612[92], 9}, {NULL}};
const Segment SGI170002[] = {{&S51612[65], 6}, {NULL}};
const Segment SGI170003[] = {{&S62302[44], 4}, {NULL}};
const Segment SGI170004[] = {{&S62302[44], 26}, {NULL}};
const Segment SGI170005[] = {{&S62302[30], 6}, {NULL}};
const Segment SGI170006[] = {{&S62302[31], 3}, {NULL}};
const Segment SGI170007[] = {{&S62302[84], 6}, {NULL}};
const Segment SGI170008[] = {{&S62303[44], 12}, {NULL}};
const Segment SGI170009[] = {{&S62303[44], 42}, {NULL}};
const Segment SGI170010[] = {{&S62303[30], 6}, {NULL}};
const Segment SGI170011[] = {{&S62303[31], 3}, {NULL}};
const Segment SGI170012[] = {{&S62303[100], 6}, {NULL}};
const Segment SGI170013[] = {{&S62307[34], 7}, {NULL}};
const Segment SGI170014[] = {{&S62307[27], 6}, {NULL}};
const Segment SGI170015[] = {{&S62307[98], 9}, {NULL}};
const Segment SGI170016[] = {{&S62307[71], 6}, {NULL}};
const Segment SGI170017[] = {{&S62307[139], 8}, {NULL}};
const Segment SGI170018[] = {{&S62314[34], 8}, {NULL}};
const Segment SGI170019[] = {{&S62314[27], 6}, {NULL}};
const Segment SGI170020[] = {{&S62314[111], 9}, {NULL}};
const Segment SGI170021[] = {{&S62314[84], 6}, {NULL}};
const Segment SGI170022[] = {{&S62315[34], 10}, {NULL}};
const Segment SGI170023[] = {{&S62315[27], 6}, {NULL}};
const Segment SGI170024[] = {{&S62315[97], 9}, {NULL}};
const Segment SGI170025[] = {{&S62315[70], 6}, {NULL}};
const Segment SGI170026[] = {{&S62318[44], 13}, {NULL}};
const Segment SGI170027[] = {{&S62318[44], 25}, {NULL}};
const Segment SGI170028[] = {{&S62318[30], 6}, {NULL}};
const Segment SGI170029[] = {{&S62318[31], 3}, {NULL}};
const Segment SGI170030[] = {{&S62318[83], 6}, {NULL}};
const Segment SGI170031[] = {{&S62321[44], 7}, {NULL}};
const Segment SGI170032[] = {{&S62321[44], 19}, {NULL}};
const Segment SGI170033[] = {{&S62321[30], 6}, {NULL}};
const Segment SGI170034[] = {{&S62321[31], 3}, {NULL}};
const Segment SGI170035[] = {{&S62321[77], 6}, {NULL}};
const Segment SGI170036[] = {{&S62322[44], 9}, {NULL}};
const Segment SGI170037[] = {{&S62322[44], 21}, {NULL}};
const Segment SGI170038[] = {{&S62322[30], 6}, {NULL}};
const Segment SGI170039[] = {{&S62322[31], 3}, {NULL}};
const Segment SGI170040[] = {{&S62322[79], 6}, {NULL}};
const Segment SGI170041[] = {{&S62323[44], 7}, {NULL}};
const Segment SGI170042[] = {{&S62323[44], 19}, {NULL}};
const Segment SGI170043[] = {{&S62323[30], 6}, {NULL}};
const Segment SGI170044[] = {{&S62323[31], 3}, {NULL}};
const Segment SGI170045[] = {{&S62323[77], 6}, {NULL}};
const Segment SGI170046[] = {{&S51663[34], 14}, {NULL}};
const Segment SGI170047[] = {{&S51663[27], 6}, {NULL}};
const Segment SGI170048[] = {{&S51663[101], 9}, {NULL}};
const Segment SGI170049[] = {{&S51663[74], 6}, {NULL}};
const Segment SGI170050[] = {{&S62325[44], 8}, {NULL}};
const Segment SGI170051[] = {{&S62325[44], 20}, {NULL}};
const Segment SGI170052[] = {{&S62325[30], 6}, {NULL}};
const Segment SGI170053[] = {{&S62325[31], 3}, {NULL}};
const Segment SGI170054[] = {{&S62325[78], 6}, {NULL}};
const Segment SGI170055[] = {{&S62327[44], 13}, {NULL}};
const Segment SGI170056[] = {{&S62327[44], 25}, {NULL}};
const Segment SGI170057[] = {{&S62327[30], 6}, {NULL}};
const Segment SGI170058[] = {{&S62327[31], 3}, {NULL}};
const Segment SGI170059[] = {{&S62327[83], 6}, {NULL}};
const Segment SGI170060[] = {{&S62348[44], 16}, {NULL}};
const Segment SGI170061[] = {{&S62348[44], 28}, {NULL}};
const Segment SGI170062[] = {{&S62348[30], 6}, {NULL}};
const Segment SGI170063[] = {{&S62348[31], 3}, {NULL}};
const Segment SGI170064[] = {{&S62348[86], 6}, {NULL}};
const Segment SGI170065[] = {{&S62349[44], 9}, {NULL}};
const Segment SGI170066[] = {{&S62349[44], 21}, {NULL}};
const Segment SGI170067[] = {{&S62349[30], 6}, {NULL}};
const Segment SGI170068[] = {{&S62349[31], 3}, {NULL}};
const Segment SGI170069[] = {{&S62349[79], 6}, {NULL}};
const Segment SGI170070[] = {{&S62350[44], 10}, {NULL}};
const Segment SGI170071[] = {{&S62350[44], 22}, {NULL}};
const Segment SGI170072[] = {{&S62350[30], 6}, {NULL}};
const Segment SGI170073[] = {{&S62350[31], 3}, {NULL}};
const Segment SGI170074[] = {{&S62350[80], 6}, {NULL}};
const Segment SGI170075[] = {{&S62352[34], 8}, {NULL}};
const Segment SGI170076[] = {{&S62352[27], 6}, {NULL}};
const Segment SGI170077[] = {{&S62352[100], 9}, {NULL}};
const Segment SGI170078[] = {{&S62352[73], 6}, {NULL}};
const Segment SGI170079[] = {{&S54872[34], 6}, {NULL}};
const Segment SGI170080[] = {{&S54872[27], 6}, {NULL}};
const Segment SGI170081[] = {{&S54872[93], 9}, {NULL}};
const Segment SGI170082[] = {{&S54872[66], 6}, {NULL}};
const Segment SGI170083[] = {{&S54873[34], 6}, {NULL}};
const Segment SGI170084[] = {{&S54873[27], 6}, {NULL}};
const Segment SGI170085[] = {{&S54873[93], 9}, {NULL}};
const Segment SGI170086[] = {{&S54873[66], 6}, {NULL}};
const Segment SGI170087[] = {{&S54875[34], 7}, {NULL}};
const Segment SGI170088[] = {{&S54875[27], 6}, {NULL}};
const Segment SGI170089[] = {{&S54875[94], 9}, {NULL}};
const Segment SGI170090[] = {{&S54875[67], 6}, {NULL}};
const Segment SGI170091[] = {{&S62381[44], 13}, {NULL}};
const Segment SGI170092[] = {{&S62381[44], 25}, {NULL}};
const Segment SGI170093[] = {{&S62381[30], 6}, {NULL}};
const Segment SGI170094[] = {{&S62381[31], 3}, {NULL}};
const Segment SGI170095[] = {{&S62381[83], 6}, {NULL}};
const Segment SGI170096[] = {{&S62382[44], 13}, {NULL}};
const Segment SGI170097[] = {{&S62382[44], 25}, {NULL}};
const Segment SGI170098[] = {{&S62382[30], 6}, {NULL}};
const Segment SGI170099[] = {{&S62382[86], 6}, {NULL}};
const Segment SGI170100[] = {{&S54878[34], 10}, {NULL}};
const Segment SGI170101[] = {{&S54878[27], 6}, {NULL}};
const Segment SGI170102[] = {{&S54878[97], 9}, {NULL}};
const Segment SGI170103[] = {{&S54878[70], 6}, {NULL}};
const Segment SGI170104[] = {{&S62383[38], 13}, {NULL}};
const Segment SGI170105[] = {{&S62383[31], 6}, {NULL}};
const Segment SGI170106[] = {{&S62384[30], 6}, {NULL}};
const Segment SGI170107[] = {{&S62384[31], 3}, {NULL}};
const Segment SGI170108[] = {{&S62384[81], 6}, {NULL}};
const Segment SGI170109[] = {{&S54881[34], 7}, {NULL}};
const Segment SGI170110[] = {{&S54881[27], 6}, {NULL}};
const Segment SGI170111[] = {{&S54881[94], 9}, {NULL}};
const Segment SGI170112[] = {{&S54881[67], 6}, {NULL}};
const Segment SGI170113[] = {{&S54882[34], 6}, {NULL}};
const Segment SGI170114[] = {{&S54882[27], 6}, {NULL}};
const Segment SGI170115[] = {{&S54882[93], 9}, {NULL}};
const Segment SGI170116[] = {{&S54882[66], 6}, {NULL}};
const Segment SGI170117[] = {{&S54883[34], 17}, {NULL}};
const Segment SGI170118[] = {{&S54883[27], 6}, {NULL}};
const Segment SGI170119[] = {{&S54883[104], 9}, {NULL}};
const Segment SGI170120[] = {{&S54883[77], 6}, {NULL}};
const Segment SGI170121[] = {{&S62390[44], 15}, {NULL}};
const Segment SGI170122[] = {{&S62390[44], 27}, {NULL}};
const Segment SGI170123[] = {{&S62390[30], 6}, {NULL}};
const Segment SGI170124[] = {{&S62390[31], 3}, {NULL}};
const Segment SGI170125[] = {{&S54884[34], 4}, {NULL}};
const Segment SGI170126[] = {{&S62390[85], 6}, {NULL}};
const Segment SGI170127[] = {{&S54884[27], 6}, {NULL}};
const Segment SGI170128[] = {{&S54884[91], 9}, {NULL}};
const Segment SGI170129[] = {{&S54884[64], 6}, {NULL}};
const Segment SGI170130[] = {{&S62391[34], 14}, {NULL}};
const Segment SGI170131[] = {{&S62391[27], 6}, {NULL}};
const Segment SGI170132[] = {{&S54885[34], 8}, {NULL}};
const Segment SGI170133[] = {{&S62391[101], 9}, {NULL}};
const Segment SGI170134[] = {{&S62391[74], 6}, {NULL}};
const Segment SGI170135[] = {{&S54885[27], 6}, {NULL}};
const Segment SGI170136[] = {{&S54885[95], 9}, {NULL}};
const Segment SGI170137[] = {{&S54885[68], 6}, {NULL}};
const Segment SGI170138[] = {{&S54887[34], 5}, {NULL}};
const Segment SGI170139[] = {{&S54887[27], 6}, {NULL}};
const Segment SGI170140[] = {{&S54887[92], 9}, {NULL}};
const Segment SGI170141[] = {{&S54887[65], 6}, {NULL}};
const Segment SGI170142[] = {{&S54888[34], 7}, {NULL}};
const Segment SGI170143[] = {{&S54888[27], 6}, {NULL}};
const Segment SGI170144[] = {{&S54888[94], 9}, {NULL}};
const Segment SGI170145[] = {{&S54888[67], 6}, {NULL}};
const Segment SGI170146[] = {{&S54889[34], 6}, {NULL}};
const Segment SGI170147[] = {{&S54889[27], 6}, {NULL}};
const Segment SGI170148[] = {{&S54889[93], 9}, {NULL}};
const Segment SGI170149[] = {{&S54889[66], 6}, {NULL}};
const Segment SGI170150[] = {{&S62399[34], 9}, {NULL}};
const Segment SGI170151[] = {{&S62399[27], 6}, {NULL}};
const Segment SGI170152[] = {{&S62399[96], 9}, {NULL}};
const Segment SGI170153[] = {{&S62399[69], 6}, {NULL}};
const Segment SGI170154[] = {{&S54890[34], 11}, {NULL}};
const Segment SGI170155[] = {{&S54890[27], 6}, {NULL}};
const Segment SGI170156[] = {{&S54890[98], 9}, {NULL}};
const Segment SGI170157[] = {{&S54890[71], 6}, {NULL}};
const Segment SGI170158[] = {{&S54892[34], 9}, {NULL}};
const Segment SGI170159[] = {{&S54892[27], 6}, {NULL}};
const Segment SGI170160[] = {{&S54892[96], 9}, {NULL}};
const Segment SGI170161[] = {{&S54892[69], 6}, {NULL}};
const Segment SGI170162[] = {{&S54893[34], 10}, {NULL}};
const Segment SGI170163[] = {{&S54893[27], 6}, {NULL}};
const Segment SGI170164[] = {{&S54893[97], 9}, {NULL}};
const Segment SGI170165[] = {{&S54893[70], 6}, {NULL}};
const Segment SGI170166[] = {{&S54894[34], 19}, {NULL}};
const Segment SGI170167[] = {{&S54894[27], 6}, {NULL}};
const Segment SGI170168[] = {{&S54894[106], 9}, {NULL}};
const Segment SGI170169[] = {{&S54894[79], 6}, {NULL}};
const Segment SGI170170[] = {{&S54895[34], 10}, {NULL}};
const Segment SGI170171[] = {{&S54895[27], 6}, {NULL}};
const Segment SGI170172[] = {{&S54895[97], 9}, {NULL}};
const Segment SGI170173[] = {{&S54895[70], 6}, {NULL}};
const Segment SGI170174[] = {{&S54896[34], 10}, {NULL}};
const Segment SGI170175[] = {{&S54896[27], 6}, {NULL}};
const Segment SGI170176[] = {{&S54896[97], 9}, {NULL}};
const Segment SGI170177[] = {{&S54896[70], 6}, {NULL}};
const Segment SGI170178[] = {{&S54900[34], 19}, {NULL}};
const Segment SGI170179[] = {{&S54900[27], 6}, {NULL}};
const Segment SGI170180[] = {{&S54900[106], 9}, {NULL}};
const Segment SGI170181[] = {{&S54900[79], 6}, {NULL}};
const Segment SGI170182[] = {{&S54901[34], 8}, {NULL}};
const Segment SGI170183[] = {{&S54901[27], 6}, {NULL}};
const Segment SGI170184[] = {{&S54901[95], 9}, {NULL}};
const Segment SGI170185[] = {{&S54901[68], 6}, {NULL}};
const Segment SGI170186[] = {{&S54904[34], 23}, {NULL}};
const Segment SGI170187[] = {{&S54904[27], 6}, {NULL}};
const Segment SGI170188[] = {{&S54904[110], 9}, {NULL}};
const Segment SGI170189[] = {{&S54904[83], 6}, {NULL}};
const Segment SGI170190[] = {{&S62425[34], 7}, {NULL}};
const Segment SGI170191[] = {{&S62425[27], 6}, {NULL}};
const Segment SGI170192[] = {{&S62425[112], 9}, {NULL}};
const Segment SGI170193[] = {{&S62425[85], 6}, {NULL}};
const Segment SGI170194[] = {{&S54908[34], 20}, {NULL}};
const Segment SGI170195[] = {{&S54908[27], 6}, {NULL}};
const Segment SGI170196[] = {{&S54908[107], 9}, {NULL}};
const Segment SGI170197[] = {{&S54908[80], 6}, {NULL}};
const Segment SGI170198[] = {{&S48884[44], 15}, {NULL}};
const Segment SGI170199[] = {{&S48884[44], 27}, {NULL}};
const Segment SGI170200[] = {{&S48884[30], 6}, {NULL}};
const Segment SGI170201[] = {{&S48884[138], 3}, {NULL}};
const Segment SGI170202[] = {{&S48884[103], 6}, {NULL}};
const Segment SGI170203[] = {{&S48884[73], 8}, {NULL}};
const Segment SGI170204[] = {{&S48885[44], 15}, {NULL}};
const Segment SGI170205[] = {{&S48885[44], 27}, {NULL}};
const Segment SGI170206[] = {{&S48885[30], 6}, {NULL}};
const Segment SGI170207[] = {{&S48885[138], 3}, {NULL}};
const Segment SGI170208[] = {{&S54911[34], 14}, {NULL}};
const Segment SGI170209[] = {{&S48885[103], 6}, {NULL}};
const Segment SGI170210[] = {{&S54911[27], 6}, {NULL}};
const Segment SGI170211[] = {{&S48885[73], 8}, {NULL}};
const Segment SGI170212[] = {{&S54911[101], 9}, {NULL}};
const Segment SGI170213[] = {{&S54911[74], 6}, {NULL}};
const Segment SGI170214[] = {{&S54912[34], 23}, {NULL}};
const Segment SGI170215[] = {{&S54912[27], 6}, {NULL}};
const Segment SGI170216[] = {{&S54912[110], 9}, {NULL}};
const Segment SGI170217[] = {{&S54912[83], 6}, {NULL}};
const Segment SGI170218[] = {{&S62442[34], 6}, {NULL}};
const Segment SGI170219[] = {{&S62442[27], 6}, {NULL}};
const Segment SGI170220[] = {{&S62442[93], 9}, {NULL}};
const Segment SGI170221[] = {{&S62442[66], 6}, {NULL}};
const Segment SGI170222[] = {{&S62443[34], 7}, {NULL}};
const Segment SGI170223[] = {{&S62443[27], 6}, {NULL}};
const Segment SGI170224[] = {{&S62443[93], 9}, {NULL}};
const Segment SGI170225[] = {{&S62443[66], 6}, {NULL}};
const Segment SGI170226[] = {{&S39679[34], 9}, {NULL}};
const Segment SGI170227[] = {{&S39679[27], 6}, {NULL}};
const Segment SGI170228[] = {{&S39679[95], 9}, {NULL}};
const Segment SGI170229[] = {{&S39679[68], 6}, {NULL}};
const Segment SGI170230[] = {{&S54928[44], 10}, {NULL}};
const Segment SGI170231[] = {{&S54928[44], 22}, {NULL}};
const Segment SGI170232[] = {{&S54928[30], 6}, {NULL}};
const Segment SGI170233[] = {{&S54928[131], 3}, {NULL}};
const Segment SGI170234[] = {{&S54928[96], 6}, {NULL}};
const Segment SGI170235[] = {{&S54928[68], 8}, {NULL}};
const Segment SGI170236[] = {{&S54929[44], 10}, {NULL}};
const Segment SGI170237[] = {{&S54929[44], 22}, {NULL}};
const Segment SGI170238[] = {{&S54929[30], 6}, {NULL}};
const Segment SGI170239[] = {{&S54929[135], 3}, {NULL}};
const Segment SGI170240[] = {{&S54929[100], 6}, {NULL}};
const Segment SGI170241[] = {{&S54929[68], 8}, {NULL}};
const Segment SGI170242[] = {{&S62463[34], 10}, {NULL}};
const Segment SGI170243[] = {{&S62463[27], 6}, {NULL}};
const Segment SGI170244[] = {{&S62463[97], 9}, {NULL}};
const Segment SGI170245[] = {{&S62463[70], 6}, {NULL}};
const Segment SGI170246[] = {{&S54958[44], 10}, {NULL}};
const Segment SGI170247[] = {{&S54958[44], 22}, {NULL}};
const Segment SGI170248[] = {{&S54958[30], 6}, {NULL}};
const Segment SGI170249[] = {{&S54958[131], 3}, {NULL}};
const Segment SGI170250[] = {{&S54958[96], 6}, {NULL}};
const Segment SGI170251[] = {{&S54958[68], 8}, {NULL}};
const Segment SGI170252[] = {{&S54959[44], 10}, {NULL}};
const Segment SGI170253[] = {{&S54959[44], 22}, {NULL}};
const Segment SGI170254[] = {{&S54959[30], 6}, {NULL}};
const Segment SGI170255[] = {{&S54959[131], 3}, {NULL}};
const Segment SGI170256[] = {{&S54959[96], 6}, {NULL}};
const Segment SGI170257[] = {{&S54959[68], 8}, {NULL}};
const Segment SGI170258[] = {{&S54961[44], 10}, {NULL}};
const Segment SGI170259[] = {{&S54961[44], 22}, {NULL}};
const Segment SGI170260[] = {{&S54961[30], 6}, {NULL}};
const Segment SGI170261[] = {{&S54961[135], 3}, {NULL}};
const Segment SGI170262[] = {{&S54961[100], 6}, {NULL}};
const Segment SGI170263[] = {{&S54961[68], 8}, {NULL}};
const Segment SGI170264[] = {{&S62500[34], 6}, {NULL}};
const Segment SGI170265[] = {{&S62500[27], 6}, {NULL}};
const Segment SGI170266[] = {{&S62500[93], 9}, {NULL}};
const Segment SGI170267[] = {{&S62500[66], 6}, {NULL}};
const Segment SGI170268[] = {{&S54991[34], 11}, {NULL}};
const Segment SGI170269[] = {{&S54991[27], 6}, {NULL}};
const Segment SGI170270[] = {{&S54991[98], 9}, {NULL}};
const Segment SGI170271[] = {{&S54991[71], 6}, {NULL}};
const Segment SGI170272[] = {{&S54992[34], 8}, {NULL}};
const Segment SGI170273[] = {{&S54992[27], 6}, {NULL}};
const Segment SGI170274[] = {{&S54992[95], 9}, {NULL}};
const Segment SGI170275[] = {{&S54992[68], 6}, {NULL}};
const Segment SGI170276[] = {{&S54994[34], 13}, {NULL}};
const Segment SGI170277[] = {{&S54994[27], 6}, {NULL}};
const Segment SGI170278[] = {{&S54994[100], 9}, {NULL}};
const Segment SGI170279[] = {{&S54994[73], 6}, {NULL}};
const Segment SGI170280[] = {{&S54995[34], 8}, {NULL}};
const Segment SGI170281[] = {{&S54995[27], 6}, {NULL}};
const Segment SGI170282[] = {{&S54995[95], 9}, {NULL}};
const Segment SGI170283[] = {{&S54995[68], 6}, {NULL}};
const Segment SGI170284[] = {{&S54998[34], 6}, {NULL}};
const Segment SGI170285[] = {{&S54998[27], 6}, {NULL}};
const Segment SGI170286[] = {{&S54998[93], 9}, {NULL}};
const Segment SGI170287[] = {{&S54998[66], 6}, {NULL}};
const Segment SGI170288[] = {{&S55000[34], 9}, {NULL}};
const Segment SGI170289[] = {{&S55000[27], 6}, {NULL}};
const Segment SGI170290[] = {{&S55000[96], 9}, {NULL}};
const Segment SGI170291[] = {{&S55000[69], 6}, {NULL}};
const Segment SGI170292[] = {{&S55002[34], 6}, {NULL}};
const Segment SGI170293[] = {{&S55002[27], 6}, {NULL}};
const Segment SGI170294[] = {{&S55002[93], 9}, {NULL}};
const Segment SGI170295[] = {{&S55002[66], 6}, {NULL}};
const Segment SGI170296[] = {{&S55004[34], 8}, {NULL}};
const Segment SGI170297[] = {{&S55004[27], 6}, {NULL}};
const Segment SGI170298[] = {{&S55004[95], 9}, {NULL}};
const Segment SGI170299[] = {{&S55004[68], 6}, {NULL}};
const Segment SGI170300[] = {{&S62544[34], 14}, {NULL}};
const Segment SGI170301[] = {{&S62544[27], 6}, {NULL}};
const Segment SGI170302[] = {{&S62544[101], 9}, {NULL}};
const Segment SGI170303[] = {{&S55006[34], 10}, {NULL}};
const Segment SGI170304[] = {{&S62544[74], 6}, {NULL}};
const Segment SGI170305[] = {{&S55006[27], 6}, {NULL}};
const Segment SGI170306[] = {{&S55006[97], 9}, {NULL}};
const Segment SGI170307[] = {{&S55006[70], 6}, {NULL}};
const Segment SGI170308[] = {{&S55007[34], 10}, {NULL}};
const Segment SGI170309[] = {{&S55007[27], 6}, {NULL}};
const Segment SGI170310[] = {{&S55007[97], 9}, {NULL}};
const Segment SGI170311[] = {{&S55007[70], 6}, {NULL}};
const Segment SGI170312[] = {{&S55008[34], 10}, {NULL}};
const Segment SGI170313[] = {{&S55008[27], 6}, {NULL}};
const Segment SGI170314[] = {{&S55008[97], 9}, {NULL}};
const Segment SGI170315[] = {{&S55008[70], 6}, {NULL}};
const Segment SGI170316[] = {{&S55009[34], 7}, {NULL}};
const Segment SGI170317[] = {{&S55009[27], 6}, {NULL}};
const Segment SGI170318[] = {{&S55009[94], 9}, {NULL}};
const Segment SGI170319[] = {{&S55009[67], 6}, {NULL}};
const Segment SGI170320[] = {{&S55010[34], 12}, {NULL}};
const Segment SGI170321[] = {{&S55010[27], 6}, {NULL}};
const Segment SGI170322[] = {{&S55010[99], 9}, {NULL}};
const Segment SGI170323[] = {{&S55010[72], 6}, {NULL}};
const Segment SGI170324[] = {{&S55011[34], 8}, {NULL}};
const Segment SGI170325[] = {{&S55011[27], 6}, {NULL}};
const Segment SGI170326[] = {{&S55011[95], 9}, {NULL}};
const Segment SGI170327[] = {{&S55011[68], 6}, {NULL}};
const Segment SGI170328[] = {{&S55013[34], 25}, {NULL}};
const Segment SGI170329[] = {{&S55013[27], 6}, {NULL}};
const Segment SGI170330[] = {{&S62558[44], 7}, {NULL}};
const Segment SGI170331[] = {{&S62558[44], 19}, {NULL}};
const Segment SGI170332[] = {{&S62558[30], 6}, {NULL}};
const Segment SGI170333[] = {{&S55013[112], 9}, {NULL}};
const Segment SGI170334[] = {{&S55013[85], 6}, {NULL}};
const Segment SGI170335[] = {{&S62558[31], 3}, {NULL}};
const Segment SGI170336[] = {{&S62558[92], 6}, {NULL}};
const Segment SGI170337[] = {{&S62558[65], 8}, {NULL}};
const Segment SGI170338[] = {{&S55014[34], 8}, {NULL}};
const Segment SGI170339[] = {{&S55014[27], 6}, {NULL}};
const Segment SGI170340[] = {{&S62559[44], 7}, {NULL}};
const Segment SGI170341[] = {{&S62559[44], 19}, {NULL}};
const Segment SGI170342[] = {{&S62559[30], 6}, {NULL}};
const Segment SGI170343[] = {{&S55014[95], 9}, {NULL}};
const Segment SGI170344[] = {{&S55014[68], 6}, {NULL}};
const Segment SGI170345[] = {{&S62559[31], 3}, {NULL}};
const Segment SGI170346[] = {{&S62559[92], 6}, {NULL}};
const Segment SGI170347[] = {{&S62559[65], 8}, {NULL}};
const Segment SGI170348[] = {{&S62560[44], 7}, {NULL}};
const Segment SGI170349[] = {{&S62560[44], 19}, {NULL}};
const Segment SGI170350[] = {{&S62560[30], 6}, {NULL}};
const Segment SGI170351[] = {{&S62560[31], 3}, {NULL}};
const Segment SGI170352[] = {{&S62560[92], 6}, {NULL}};
const Segment SGI170353[] = {{&S55016[34], 17}, {NULL}};
const Segment SGI170354[] = {{&S62560[65], 8}, {NULL}};
const Segment SGI170355[] = {{&S55016[27], 6}, {NULL}};
const Segment SGI170356[] = {{&S55016[104], 9}, {NULL}};
const Segment SGI170357[] = {{&S55016[77], 6}, {NULL}};
const Segment SGI170358[] = {{&S62561[44], 8}, {NULL}};
const Segment SGI170359[] = {{&S62561[44], 20}, {NULL}};
const Segment SGI170360[] = {{&S62561[30], 6}, {NULL}};
const Segment SGI170361[] = {{&S62561[31], 3}, {NULL}};
const Segment SGI170362[] = {{&S55017[34], 6}, {NULL}};
const Segment SGI170363[] = {{&S62561[114], 6}, {NULL}};
const Segment SGI170364[] = {{&S55017[27], 6}, {NULL}};
const Segment SGI170365[] = {{&S62561[66], 8}, {NULL}};
const Segment SGI170366[] = {{&S55017[93], 9}, {NULL}};
const Segment SGI170367[] = {{&S55017[66], 6}, {NULL}};
const Segment SGI170368[] = {{&S62562[34], 10}, {NULL}};
const Segment SGI170369[] = {{&S62562[27], 6}, {NULL}};
const Segment SGI170370[] = {{&S55018[34], 10}, {NULL}};
const Segment SGI170371[] = {{&S55018[27], 6}, {NULL}};
const Segment SGI170372[] = {{&S62562[97], 9}, {NULL}};
const Segment SGI170373[] = {{&S62562[70], 6}, {NULL}};
const Segment SGI170374[] = {{&S55018[97], 9}, {NULL}};
const Segment SGI170375[] = {{&S55018[70], 6}, {NULL}};
const Segment SGI170376[] = {{&S55019[34], 4}, {NULL}};
const Segment SGI170377[] = {{&S55019[27], 6}, {NULL}};
const Segment SGI170378[] = {{&S55019[91], 9}, {NULL}};
const Segment SGI170379[] = {{&S55019[64], 6}, {NULL}};
const Segment SGI170380[] = {{&S62565[34], 11}, {NULL}};
const Segment SGI170381[] = {{&S62565[27], 6}, {NULL}};
const Segment SGI170382[] = {{&S62565[98], 9}, {NULL}};
const Segment SGI170383[] = {{&S62565[71], 6}, {NULL}};
const Segment SGI170384[] = {{&S55022[34], 6}, {NULL}};
const Segment SGI170385[] = {{&S55022[27], 6}, {NULL}};
const Segment SGI170386[] = {{&S62567[34], 11}, {NULL}};
const Segment SGI170387[] = {{&S62567[27], 6}, {NULL}};
const Segment SGI170388[] = {{&S55022[93], 9}, {NULL}};
const Segment SGI170389[] = {{&S55022[66], 6}, {NULL}};
const Segment SGI170390[] = {{&S62567[98], 9}, {NULL}};
const Segment SGI170391[] = {{&S62567[71], 6}, {NULL}};
const Segment SGI170392[] = {{&S55023[34], 11}, {NULL}};
const Segment SGI170393[] = {{&S55023[27], 6}, {NULL}};
const Segment SGI170394[] = {{&S62568[34], 10}, {NULL}};
const Segment SGI170395[] = {{&S62568[27], 6}, {NULL}};
const Segment SGI170396[] = {{&S55023[98], 9}, {NULL}};
const Segment SGI170397[] = {{&S55023[71], 6}, {NULL}};
const Segment SGI170398[] = {{&S62568[97], 9}, {NULL}};
const Segment SGI170399[] = {{&S62568[70], 6}, {NULL}};
const Segment SGI170400[] = {{&S62569[34], 11}, {NULL}};
const Segment SGI170401[] = {{&S62569[27], 6}, {NULL}};
const Segment SGI170402[] = {{&S62569[98], 9}, {NULL}};
const Segment SGI170403[] = {{&S62569[71], 6}, {NULL}};
const Segment SGI170404[] = {{&S55025[34], 8}, {NULL}};
const Segment SGI170405[] = {{&S55025[27], 6}, {NULL}};
const Segment SGI170406[] = {{&S55025[95], 9}, {NULL}};
const Segment SGI170407[] = {{&S55025[68], 6}, {NULL}};
const Segment SGI170408[] = {{&S62571[34], 7}, {NULL}};
const Segment SGI170409[] = {{&S62571[27], 6}, {NULL}};
const Segment SGI170410[] = {{&S62571[96], 9}, {NULL}};
const Segment SGI170411[] = {{&S62571[69], 6}, {NULL}};
const Segment SGI170412[] = {{&S55027[34], 7}, {NULL}};
const Segment SGI170413[] = {{&S55027[27], 6}, {NULL}};
const Segment SGI170414[] = {{&S55027[94], 9}, {NULL}};
const Segment SGI170415[] = {{&S55027[67], 6}, {NULL}};
const Segment SGI170416[] = {{&S39820[34], 10}, {NULL}};
const Segment SGI170417[] = {{&S39820[27], 6}, {NULL}};
const Segment SGI170418[] = {{&S62575[44], 14}, {NULL}};
const Segment SGI170419[] = {{&S62575[44], 26}, {NULL}};
const Segment SGI170420[] = {{&S62575[30], 6}, {NULL}};
const Segment SGI170421[] = {{&S39820[97], 9}, {NULL}};
const Segment SGI170422[] = {{&S39820[70], 6}, {NULL}};
const Segment SGI170423[] = {{&S62575[31], 3}, {NULL}};
const Segment SGI170424[] = {{&S62575[84], 6}, {NULL}};
const Segment SGI170425[] = {{&S39821[34], 10}, {NULL}};
const Segment SGI170426[] = {{&S39821[27], 6}, {NULL}};
const Segment SGI170427[] = {{&S62576[44], 9}, {NULL}};
const Segment SGI170428[] = {{&S62576[44], 21}, {NULL}};
const Segment SGI170429[] = {{&S62576[30], 6}, {NULL}};
const Segment SGI170430[] = {{&S39821[96], 9}, {NULL}};
const Segment SGI170431[] = {{&S39821[69], 6}, {NULL}};
const Segment SGI170432[] = {{&S62576[31], 3}, {NULL}};
const Segment SGI170433[] = {{&S62576[79], 6}, {NULL}};
const Segment SGI170434[] = {{&S55034[34], 6}, {NULL}};
const Segment SGI170435[] = {{&S55034[27], 6}, {NULL}};
const Segment SGI170436[] = {{&S55034[93], 9}, {NULL}};
const Segment SGI170437[] = {{&S39822[34], 10}, {NULL}};
const Segment SGI170438[] = {{&S55034[66], 6}, {NULL}};
const Segment SGI170439[] = {{&S39822[27], 6}, {NULL}};
const Segment SGI170440[] = {{&S62577[34], 10}, {NULL}};
const Segment SGI170441[] = {{&S39822[97], 9}, {NULL}};
const Segment SGI170442[] = {{&S62577[27], 6}, {NULL}};
const Segment SGI170443[] = {{&S39822[70], 6}, {NULL}};
const Segment SGI170444[] = {{&S62577[97], 9}, {NULL}};
const Segment SGI170445[] = {{&S62577[70], 6}, {NULL}};
const Segment SGI170446[] = {{&S62578[34], 7}, {NULL}};
const Segment SGI170447[] = {{&S62578[27], 6}, {NULL}};
const Segment SGI170448[] = {{&S39824[34], 10}, {NULL}};
const Segment SGI170449[] = {{&S39824[27], 6}, {NULL}};
const Segment SGI170450[] = {{&S62578[96], 9}, {NULL}};
const Segment SGI170451[] = {{&S62578[69], 6}, {NULL}};
const Segment SGI170452[] = {{&S39824[96], 9}, {NULL}};
const Segment SGI170453[] = {{&S39824[69], 6}, {NULL}};
const Segment SGI170454[] = {{&S62582[34], 10}, {NULL}};
const Segment SGI170455[] = {{&S62582[27], 6}, {NULL}};
const Segment SGI170456[] = {{&S62582[97], 9}, {NULL}};
const Segment SGI170457[] = {{&S62582[70], 6}, {NULL}};
const Segment SGI170458[] = {{&S49068[44], 10}, {NULL}};
const Segment SGI170459[] = {{&S49068[44], 22}, {NULL}};
const Segment SGI170460[] = {{&S49068[30], 6}, {NULL}};
const Segment SGI170461[] = {{&S49068[133], 3}, {NULL}};
const Segment SGI170462[] = {{&S49068[98], 6}, {NULL}};
const Segment SGI170463[] = {{&S49068[68], 8}, {NULL}};
const Segment SGI170464[] = {{&S49069[44], 10}, {NULL}};
const Segment SGI170465[] = {{&S49069[44], 22}, {NULL}};
const Segment SGI170466[] = {{&S49069[30], 6}, {NULL}};
const Segment SGI170467[] = {{&S49069[133], 3}, {NULL}};
const Segment SGI170468[] = {{&S49069[98], 6}, {NULL}};
const Segment SGI170469[] = {{&S49069[68], 8}, {NULL}};
const Segment SGI170470[] = {{&S62586[34], 10}, {NULL}};
const Segment SGI170471[] = {{&S62586[27], 6}, {NULL}};
const Segment SGI170472[] = {{&S62586[97], 9}, {NULL}};
const Segment SGI170473[] = {{&S62586[70], 6}, {NULL}};
const Segment SGI170474[] = {{&S62587[34], 11}, {NULL}};
const Segment SGI170475[] = {{&S62587[27], 6}, {NULL}};
const Segment SGI170476[] = {{&S49071[44], 10}, {NULL}};
const Segment SGI170477[] = {{&S49071[44], 22}, {NULL}};
const Segment SGI170478[] = {{&S49071[30], 6}, {NULL}};
const Segment SGI170479[] = {{&S62587[98], 9}, {NULL}};
const Segment SGI170480[] = {{&S62587[71], 6}, {NULL}};
const Segment SGI170481[] = {{&S49071[133], 3}, {NULL}};
const Segment SGI170482[] = {{&S49071[98], 6}, {NULL}};
const Segment SGI170483[] = {{&S49071[68], 8}, {NULL}};
const Segment SGI170484[] = {{&S51729[44], 11}, {NULL}};
const Segment SGI170485[] = {{&S51729[44], 23}, {NULL}};
const Segment SGI170486[] = {{&S51729[30], 6}, {NULL}};
const Segment SGI170487[] = {{&S51729[132], 3}, {NULL}};
const Segment SGI170488[] = {{&S51729[97], 6}, {NULL}};
const Segment SGI170489[] = {{&S51729[69], 8}, {NULL}};
const Segment SGI170490[] = {{&S51730[44], 7}, {NULL}};
const Segment SGI170491[] = {{&S51730[44], 19}, {NULL}};
const Segment SGI170492[] = {{&S51730[30], 6}, {NULL}};
const Segment SGI170493[] = {{&S51730[132], 3}, {NULL}};
const Segment SGI170494[] = {{&S51730[97], 6}, {NULL}};
const Segment SGI170495[] = {{&S51730[65], 8}, {NULL}};
const Segment SGI170496[] = {{&S62617[44], 24}, {NULL}};
const Segment SGI170497[] = {{&S62617[30], 6}, {NULL}};
const Segment SGI170498[] = {{&S62617[31], 3}, {NULL}};
const Segment SGI170499[] = {{&S62617[101], 6}, {NULL}};
const Segment SGI170500[] = {{&S51732[44], 11}, {NULL}};
const Segment SGI170501[] = {{&S51732[44], 23}, {NULL}};
const Segment SGI170502[] = {{&S51732[30], 6}, {NULL}};
const Segment SGI170503[] = {{&S51732[132], 3}, {NULL}};
const Segment SGI170504[] = {{&S51732[97], 6}, {NULL}};
const Segment SGI170505[] = {{&S51732[69], 8}, {NULL}};
const Segment SGI170506[] = {{&S62618[44], 24}, {NULL}};
const Segment SGI170507[] = {{&S62618[30], 6}, {NULL}};
const Segment SGI170508[] = {{&S62618[31], 3}, {NULL}};
const Segment SGI170509[] = {{&S62618[101], 6}, {NULL}};
const Segment SGI170510[] = {{&S51758[44], 10}, {NULL}};
const Segment SGI170511[] = {{&S51758[44], 22}, {NULL}};
const Segment SGI170512[] = {{&S51758[30], 6}, {NULL}};
const Segment SGI170513[] = {{&S51758[115], 3}, {NULL}};
const Segment SGI170514[] = {{&S51758[80], 6}, {NULL}};
const Segment SGI170515[] = {{&S51759[38], 10}, {NULL}};
const Segment SGI170516[] = {{&S51759[31], 6}, {NULL}};
const Segment SGI170517[] = {{&S62649[34], 14}, {NULL}};
const Segment SGI170518[] = {{&S62649[27], 6}, {NULL}};
const Segment SGI170519[] = {{&S62649[101], 9}, {NULL}};
const Segment SGI170520[] = {{&S62649[74], 6}, {NULL}};
const Segment SGI170521[] = {{&S62666[34], 10}, {NULL}};
const Segment SGI170522[] = {{&S62666[27], 6}, {NULL}};
const Segment SGI170523[] = {{&S62666[97], 9}, {NULL}};
const Segment SGI170524[] = {{&S62666[70], 6}, {NULL}};
const Segment SGI170525[] = {{&S62667[34], 10}, {NULL}};
const Segment SGI170526[] = {{&S62667[27], 6}, {NULL}};
const Segment SGI170527[] = {{&S62667[97], 9}, {NULL}};
const Segment SGI170528[] = {{&S62667[70], 6}, {NULL}};
const Segment SGI170529[] = {{&S62668[34], 14}, {NULL}};
const Segment SGI170530[] = {{&S62668[27], 6}, {NULL}};
const Segment SGI170531[] = {{&S62668[101], 9}, {NULL}};
const Segment SGI170532[] = {{&S62668[74], 6}, {NULL}};
const Segment SGI170533[] = {{&S62669[34], 15}, {NULL}};
const Segment SGI170534[] = {{&S62669[27], 6}, {NULL}};
const Segment SGI170535[] = {{&S62669[102], 9}, {NULL}};
const Segment SGI170536[] = {{&S62669[75], 6}, {NULL}};
const Segment SGI170537[] = {{&S39930[41], 18}, {NULL}};
const Segment SGI170538[] = {{&S39930[41], 29}, {NULL}};
const Segment SGI170539[] = {{&S39930[27], 6}, {NULL}};
const Segment SGI170540[] = {{&S39930[119], 3}, {NULL}};
const Segment SGI170541[] = {{&S39930[123], 9}, {NULL}};
const Segment SGI170542[] = {{&S39930[84], 6}, {NULL}};
const Segment SGI170543[] = {{&S62674[34], 14}, {NULL}};
const Segment SGI170544[] = {{&S62674[27], 6}, {NULL}};
const Segment SGI170545[] = {{&S62674[101], 9}, {NULL}};
const Segment SGI170546[] = {{&S62674[74], 6}, {NULL}};
const Segment SGI170547[] = {{&S62675[34], 15}, {NULL}};
const Segment SGI170548[] = {{&S62675[27], 6}, {NULL}};
const Segment SGI170549[] = {{&S62675[102], 9}, {NULL}};
const Segment SGI170550[] = {{&S62675[75], 6}, {NULL}};
const Segment SGI170551[] = {{&S62677[34], 10}, {NULL}};
const Segment SGI170552[] = {{&S62677[27], 6}, {NULL}};
const Segment SGI170553[] = {{&S62677[97], 9}, {NULL}};
const Segment SGI170554[] = {{&S62677[70], 6}, {NULL}};
const Segment SGI170555[] = {{&S62683[44], 5}, {NULL}};
const Segment SGI170556[] = {{&S62683[44], 17}, {NULL}};
const Segment SGI170557[] = {{&S62683[30], 6}, {NULL}};
const Segment SGI170558[] = {{&S62683[31], 3}, {NULL}};
const Segment SGI170559[] = {{&S62683[75], 6}, {NULL}};
const Segment SGI170560[] = {{&S62684[44], 10}, {NULL}};
const Segment SGI170561[] = {{&S62684[44], 22}, {NULL}};
const Segment SGI170562[] = {{&S62684[30], 6}, {NULL}};
const Segment SGI170563[] = {{&S62684[31], 3}, {NULL}};
const Segment SGI170564[] = {{&S62684[80], 6}, {NULL}};
const Segment SGI170565[] = {{&S62685[38], 5}, {NULL}};
const Segment SGI170566[] = {{&S62685[31], 6}, {NULL}};
const Segment SGI170567[] = {{&S62686[44], 7}, {NULL}};
const Segment SGI170568[] = {{&S62686[44], 19}, {NULL}};
const Segment SGI170569[] = {{&S62686[30], 6}, {NULL}};
const Segment SGI170570[] = {{&S62686[31], 3}, {NULL}};
const Segment SGI170571[] = {{&S62686[77], 6}, {NULL}};
const Segment SGI170572[] = {{&S51811[34], 10}, {NULL}};
const Segment SGI170573[] = {{&S51811[27], 6}, {NULL}};
const Segment SGI170574[] = {{&S51811[97], 9}, {NULL}};
const Segment SGI170575[] = {{&S51811[70], 6}, {NULL}};
const Segment SGI170576[] = {{&S62694[34], 10}, {NULL}};
const Segment SGI170577[] = {{&S62694[27], 6}, {NULL}};
const Segment SGI170578[] = {{&S62694[97], 9}, {NULL}};
const Segment SGI170579[] = {{&S62694[70], 6}, {NULL}};
const Segment SGI170580[] = {{&S62695[34], 13}, {NULL}};
const Segment SGI170581[] = {{&S62695[27], 6}, {NULL}};
const Segment SGI170582[] = {{&S51817[34], 11}, {NULL}};
const Segment SGI170583[] = {{&S62695[100], 9}, {NULL}};
const Segment SGI170584[] = {{&S51817[27], 6}, {NULL}};
const Segment SGI170585[] = {{&S62695[73], 6}, {NULL}};
const Segment SGI170586[] = {{&S51817[98], 9}, {NULL}};
const Segment SGI170587[] = {{&S51817[71], 6}, {NULL}};
const Segment SGI170588[] = {{&S51817[139], 8}, {NULL}};
const Segment SGI170589[] = {{&S51818[34], 10}, {NULL}};
const Segment SGI170590[] = {{&S51818[27], 6}, {NULL}};
const Segment SGI170591[] = {{&S51818[97], 9}, {NULL}};
const Segment SGI170592[] = {{&S51818[70], 6}, {NULL}};
const Segment SGI170593[] = {{&S51818[138], 8}, {NULL}};
const Segment SGI170594[] = {{&S51819[34], 11}, {NULL}};
const Segment SGI170595[] = {{&S51819[27], 6}, {NULL}};
const Segment SGI170596[] = {{&S55037[34], 11}, {NULL}};
const Segment SGI170597[] = {{&S55037[27], 6}, {NULL}};
const Segment SGI170598[] = {{&S55037[98], 9}, {NULL}};
const Segment SGI170599[] = {{&S55037[71], 6}, {NULL}};
const Segment SGI170600[] = {{&S55038[34], 13}, {NULL}};
const Segment SGI170601[] = {{&S55038[27], 6}, {NULL}};
const Segment SGI170602[] = {{&S55038[100], 9}, {NULL}};
const Segment SGI170603[] = {{&S55038[73], 6}, {NULL}};
const Segment SGI170604[] = {{&S55041[34], 9}, {NULL}};
const Segment SGI170605[] = {{&S55041[27], 6}, {NULL}};
const Segment SGI170606[] = {{&S55041[96], 9}, {NULL}};
const Segment SGI170607[] = {{&S55041[69], 6}, {NULL}};
const Segment SGI170608[] = {{&S62718[34], 14}, {NULL}};
const Segment SGI170609[] = {{&S62718[27], 6}, {NULL}};
const Segment SGI170610[] = {{&S62718[101], 9}, {NULL}};
const Segment SGI170611[] = {{&S62718[74], 6}, {NULL}};
const Segment SGI170612[] = {{&S62785[44], 7}, {NULL}};
const Segment SGI170613[] = {{&S62785[44], 35}, {NULL}};
const Segment SGI170614[] = {{&S62785[30], 6}, {NULL}};
const Segment SGI170615[] = {{&S62785[31], 3}, {NULL}};
const Segment SGI170616[] = {{&S62785[93], 6}, {NULL}};
const Segment SGI170617[] = {{&S62786[44], 7}, {NULL}};
const Segment SGI170618[] = {{&S62786[44], 35}, {NULL}};
const Segment SGI170619[] = {{&S62786[30], 6}, {NULL}};
const Segment SGI170620[] = {{&S62786[31], 3}, {NULL}};
const Segment SGI170621[] = {{&S62786[93], 6}, {NULL}};
const Segment SGI170622[] = {{&S62787[38], 7}, {NULL}};
const Segment SGI170623[] = {{&S62787[31], 6}, {NULL}};
const Segment SGI170624[] = {{&S55129[34], 6}, {NULL}};
const Segment SGI170625[] = {{&S55129[27], 6}, {NULL}};
const Segment SGI170626[] = {{&S55129[93], 9}, {NULL}};
const Segment SGI170627[] = {{&S55129[66], 6}, {NULL}};
const Segment SGI170628[] = {{&S62814[44], 7}, {NULL}};
const Segment SGI170629[] = {{&S62814[44], 19}, {NULL}};
const Segment SGI170630[] = {{&S62814[30], 6}, {NULL}};
const Segment SGI170631[] = {{&S62814[31], 3}, {NULL}};
const Segment SGI170632[] = {{&S62814[77], 6}, {NULL}};
const Segment SGI170633[] = {{&S62815[44], 11}, {NULL}};
const Segment SGI170634[] = {{&S62815[44], 23}, {NULL}};
const Segment SGI170635[] = {{&S51819[98], 9}, {NULL}};
const Segment SGI170636[] = {{&S62815[30], 6}, {NULL}};
const Segment SGI170637[] = {{&S51819[71], 6}, {NULL}};
const Segment SGI170638[] = {{&S62815[31], 3}, {NULL}};
const Segment SGI170639[] = {{&S51819[139], 8}, {NULL}};
const Segment SGI170640[] = {{&S62815[81], 6}, {NULL}};
const Segment SGI170641[] = {{&S51820[34], 11}, {NULL}};
const Segment SGI170642[] = {{&S51820[27], 6}, {NULL}};
const Segment SGI170643[] = {{&S62816[38], 11}, {NULL}};
const Segment SGI170644[] = {{&S62816[31], 6}, {NULL}};
const Segment SGI170645[] = {{&S51820[98], 9}, {NULL}};
const Segment SGI170646[] = {{&S51820[71], 6}, {NULL}};
const Segment SGI170647[] = {{&S51820[139], 8}, {NULL}};
const Segment SGI170648[] = {{&S62820[34], 7}, {NULL}};
const Segment SGI170649[] = {{&S62820[27], 6}, {NULL}};
const Segment SGI170650[] = {{&S62820[105], 9}, {NULL}};
const Segment SGI170651[] = {{&S62820[78], 6}, {NULL}};
const Segment SGI170652[] = {{&S49373[34], 8}, {NULL}};
const Segment SGI170653[] = {{&S49373[27], 6}, {NULL}};
const Segment SGI170654[] = {{&S49373[95], 9}, {NULL}};
const Segment SGI170655[] = {{&S49373[68], 6}, {NULL}};
const Segment SGI170656[] = {{&S55176[44], 9}, {NULL}};
const Segment SGI170657[] = {{&S55176[44], 21}, {NULL}};
const Segment SGI170658[] = {{&S55176[30], 6}, {NULL}};
const Segment SGI170659[] = {{&S55176[130], 3}, {NULL}};
const Segment SGI170660[] = {{&S55176[95], 6}, {NULL}};
const Segment SGI170661[] = {{&S55176[67], 8}, {NULL}};
const Segment SGI170662[] = {{&S55177[44], 9}, {NULL}};
const Segment SGI170663[] = {{&S55177[44], 21}, {NULL}};
const Segment SGI170664[] = {{&S55177[30], 6}, {NULL}};
const Segment SGI170665[] = {{&S55177[130], 3}, {NULL}};
const Segment SGI170666[] = {{&S55177[95], 6}, {NULL}};
const Segment SGI170667[] = {{&S55177[67], 8}, {NULL}};
const Segment SGI170668[] = {{&S55178[44], 9}, {NULL}};
const Segment SGI170669[] = {{&S55178[44], 21}, {NULL}};
const Segment SGI170670[] = {{&S55178[30], 6}, {NULL}};
const Segment SGI170671[] = {{&S55178[130], 3}, {NULL}};
const Segment SGI170672[] = {{&S55178[95], 6}, {NULL}};
const Segment SGI170673[] = {{&S55178[67], 8}, {NULL}};
const Segment SGI170674[] = {{&S55182[44], 13}, {NULL}};
const Segment SGI170675[] = {{&S55182[44], 25}, {NULL}};
const Segment SGI170676[] = {{&S55182[30], 6}, {NULL}};
const Segment SGI170677[] = {{&S55182[118], 3}, {NULL}};
const Segment SGI170678[] = {{&S55182[83], 6}, {NULL}};
const Segment SGI170679[] = {{&S55183[44], 13}, {NULL}};
const Segment SGI170680[] = {{&S55183[44], 25}, {NULL}};
const Segment SGI170681[] = {{&S55183[30], 6}, {NULL}};
const Segment SGI170682[] = {{&S55183[86], 6}, {NULL}};
const Segment SGI170683[] = {{&S55184[38], 13}, {NULL}};
const Segment SGI170684[] = {{&S55184[31], 6}, {NULL}};
const Segment SGI170685[] = {{&S51860[44], 10}, {NULL}};
const Segment SGI170686[] = {{&S51860[44], 22}, {NULL}};
const Segment SGI170687[] = {{&S51860[30], 6}, {NULL}};
const Segment SGI170688[] = {{&S51860[115], 3}, {NULL}};
const Segment SGI170689[] = {{&S51860[80], 6}, {NULL}};
const Segment SGI170690[] = {{&S51861[38], 10}, {NULL}};
const Segment SGI170691[] = {{&S51861[31], 6}, {NULL}};
const Segment SGI170692[] = {{&S62871[34], 10}, {NULL}};
const Segment SGI170693[] = {{&S62871[27], 6}, {NULL}};
const Segment SGI170694[] = {{&S62871[97], 9}, {NULL}};
const Segment SGI170695[] = {{&S62871[70], 6}, {NULL}};
const Segment SGI170696[] = {{&S40152[34], 7}, {NULL}};
const Segment SGI170697[] = {{&S40152[27], 6}, {NULL}};
const Segment SGI170698[] = {{&S40152[100], 9}, {NULL}};
const Segment SGI170699[] = {{&S40152[73], 6}, {NULL}};
const Segment SGI170700[] = {{&S40153[34], 7}, {NULL}};
const Segment SGI170701[] = {{&S40153[27], 6}, {NULL}};
const Segment SGI170702[] = {{&S40153[94], 9}, {NULL}};
const Segment SGI170703[] = {{&S40153[67], 6}, {NULL}};
const Segment SGI170704[] = {{&S62894[34], 15}, {NULL}};
const Segment SGI170705[] = {{&S62894[27], 6}, {NULL}};
const Segment SGI170706[] = {{&S62894[102], 9}, {NULL}};
const Segment SGI170707[] = {{&S62894[75], 6}, {NULL}};
const Segment SGI170708[] = {{&S62908[44], 10}, {NULL}};
const Segment SGI170709[] = {{&S62908[44], 22}, {NULL}};
const Segment SGI170710[] = {{&S62908[30], 6}, {NULL}};
const Segment SGI170711[] = {{&S62908[31], 3}, {NULL}};
const Segment SGI170712[] = {{&S62908[80], 6}, {NULL}};
const Segment SGI170713[] = {{&S62911[34], 11}, {NULL}};
const Segment SGI170714[] = {{&S62911[27], 6}, {NULL}};
const Segment SGI170715[] = {{&S62911[98], 9}, {NULL}};
const Segment SGI170716[] = {{&S62911[71], 6}, {NULL}};
const Segment SGI170717[] = {{&S52914[32], 10}, {NULL}};
const Segment SGI170718[] = {{&S52914[27], 4}, {NULL}};
const Segment SGI170719[] = {{&S52914[95], 9}, {NULL}};
const Segment SGI170720[] = {{&S52914[68], 6}, {NULL}};
const Segment SGI170721[] = {{&S51931[44], 11}, {NULL}};
const Segment SGI170722[] = {{&S51931[44], 23}, {NULL}};
const Segment SGI170723[] = {{&S51931[30], 6}, {NULL}};
const Segment SGI170724[] = {{&S51931[143], 3}, {NULL}};
const Segment SGI170725[] = {{&S51931[108], 6}, {NULL}};
const Segment SGI170726[] = {{&S51931[69], 8}, {NULL}};
const Segment SGI170727[] = {{&S62950[34], 7}, {NULL}};
const Segment SGI170728[] = {{&S62950[27], 6}, {NULL}};
const Segment SGI170729[] = {{&S62950[94], 9}, {NULL}};
const Segment SGI170730[] = {{&S62950[67], 6}, {NULL}};
const Segment SGI170731[] = {{&S62960[44], 7}, {NULL}};
const Segment SGI170732[] = {{&S62960[44], 19}, {NULL}};
const Segment SGI170733[] = {{&S62960[30], 6}, {NULL}};
const Segment SGI170734[] = {{&S62960[31], 3}, {NULL}};
const Segment SGI170735[] = {{&S62960[77], 6}, {NULL}};
const Segment SGI170736[] = {{&S62961[38], 7}, {NULL}};
const Segment SGI170737[] = {{&S62961[31], 6}, {NULL}};
const Segment SGI170738[] = {{&S62966[34], 7}, {NULL}};
const Segment SGI170739[] = {{&S62966[27], 6}, {NULL}};
const Segment SGI170740[] = {{&S62966[93], 9}, {NULL}};
const Segment SGI170741[] = {{&S62966[66], 6}, {NULL}};
const Segment SGI170742[] = {{&S62967[34], 6}, {NULL}};
const Segment SGI170743[] = {{&S62967[27], 6}, {NULL}};
const Segment SGI170744[] = {{&S62967[93], 9}, {NULL}};
const Segment SGI170745[] = {{&S62967[66], 6}, {NULL}};
const Segment SGI170746[] = {{&S51992[34], 10}, {NULL}};
const Segment SGI170747[] = {{&S51992[27], 6}, {NULL}};
const Segment SGI170748[] = {{&S51992[97], 9}, {NULL}};
const Segment SGI170749[] = {{&S51992[70], 6}, {NULL}};
const Segment SGI170750[] = {{&S51993[34], 11}, {NULL}};
const Segment SGI170751[] = {{&S51993[27], 6}, {NULL}};
const Segment SGI170752[] = {{&S51993[98], 9}, {NULL}};
const Segment SGI170753[] = {{&S51993[71], 6}, {NULL}};
const Segment SGI170754[] = {{&S51994[34], 10}, {NULL}};
const Segment SGI170755[] = {{&S51994[27], 6}, {NULL}};
const Segment SGI170756[] = {{&S51994[97], 9}, {NULL}};
const Segment SGI170757[] = {{&S51994[70], 6}, {NULL}};
const Segment SGI170758[] = {{&S51995[34], 12}, {NULL}};
const Segment SGI170759[] = {{&S51995[27], 6}, {NULL}};
const Segment SGI170760[] = {{&S51995[99], 9}, {NULL}};
const Segment SGI170761[] = {{&S51995[72], 6}, {NULL}};
const Segment SGI170762[] = {{&S52000[44], 20}, {NULL}};
const Segment SGI170763[] = {{&S52000[44], 32}, {NULL}};
const Segment SGI170764[] = {{&S52000[30], 6}, {NULL}};
const Segment SGI170765[] = {{&S52000[141], 3}, {NULL}};
const Segment SGI170766[] = {{&S52000[106], 6}, {NULL}};
const Segment SGI170767[] = {{&S52000[78], 8}, {NULL}};
const Segment SGI170768[] = {{&S52006[44], 14}, {NULL}};
const Segment SGI170769[] = {{&S52006[44], 26}, {NULL}};
const Segment SGI170770[] = {{&S52006[30], 6}, {NULL}};
const Segment SGI170771[] = {{&S52006[119], 3}, {NULL}};
const Segment SGI170772[] = {{&S52006[84], 6}, {NULL}};
const Segment SGI170773[] = {{&S52007[44], 14}, {NULL}};
const Segment SGI170774[] = {{&S52007[44], 26}, {NULL}};
const Segment SGI170775[] = {{&S52007[30], 6}, {NULL}};
const Segment SGI170776[] = {{&S52007[119], 3}, {NULL}};
const Segment SGI170777[] = {{&S52007[84], 6}, {NULL}};
const Segment SGI170778[] = {{&S52008[44], 14}, {NULL}};
const Segment SGI170779[] = {{&S52008[44], 26}, {NULL}};
const Segment SGI170780[] = {{&S52008[30], 6}, {NULL}};
const Segment SGI170781[] = {{&S52008[119], 3}, {NULL}};
const Segment SGI170782[] = {{&S52008[84], 6}, {NULL}};
const Segment SGI170783[] = {{&S52009[44], 10}, {NULL}};
const Segment SGI170784[] = {{&S52009[44], 22}, {NULL}};
const Segment SGI170785[] = {{&S52009[30], 6}, {NULL}};
const Segment SGI170786[] = {{&S52009[115], 3}, {NULL}};
const Segment SGI170787[] = {{&S52009[80], 6}, {NULL}};
const Segment SGI170788[] = {{&S52010[44], 14}, {NULL}};
const Segment SGI170789[] = {{&S52010[44], 26}, {NULL}};
const Segment SGI170790[] = {{&S52010[30], 6}, {NULL}};
const Segment SGI170791[] = {{&S52010[119], 3}, {NULL}};
const Segment SGI170792[] = {{&S52010[84], 6}, {NULL}};
const Segment SGI170793[] = {{&S52011[44], 14}, {NULL}};
const Segment SGI170794[] = {{&S52011[44], 26}, {NULL}};
const Segment SGI170795[] = {{&S52011[30], 6}, {NULL}};
const Segment SGI170796[] = {{&S52011[119], 3}, {NULL}};
const Segment SGI170797[] = {{&S52011[84], 6}, {NULL}};
const Segment SGI170798[] = {{&S52012[44], 14}, {NULL}};
const Segment SGI170799[] = {{&S52012[44], 26}, {NULL}};
const Segment SGI170800[] = {{&S52012[30], 6}, {NULL}};
const Segment SGI170801[] = {{&S52012[119], 3}, {NULL}};
const Segment SGI170802[] = {{&S52012[84], 6}, {NULL}};
const Segment SGI170803[] = {{&S52013[38], 10}, {NULL}};
const Segment SGI170804[] = {{&S52013[31], 6}, {NULL}};
const Segment SGI170805[] = {{&S52014[44], 14}, {NULL}};
const Segment SGI170806[] = {{&S52014[44], 26}, {NULL}};
const Segment SGI170807[] = {{&S52014[30], 6}, {NULL}};
const Segment SGI170808[] = {{&S52014[119], 3}, {NULL}};
const Segment SGI170809[] = {{&S52014[84], 6}, {NULL}};
const Segment SGI170810[] = {{&S52016[34], 25}, {NULL}};
const Segment SGI170811[] = {{&S52016[27], 6}, {NULL}};
const Segment SGI170812[] = {{&S52016[112], 9}, {NULL}};
const Segment SGI170813[] = {{&S52016[85], 6}, {NULL}};
const Segment SGI170814[] = {{&S62979[34], 11}, {NULL}};
const Segment SGI170815[] = {{&S62979[27], 6}, {NULL}};
const Segment SGI170816[] = {{&S62979[98], 9}, {NULL}};
const Segment SGI170817[] = {{&S62979[71], 6}, {NULL}};
const Segment SGI170818[] = {{&S52032[34], 10}, {NULL}};
const Segment SGI170819[] = {{&S52032[27], 6}, {NULL}};
const Segment SGI170820[] = {{&S52032[97], 9}, {NULL}};
const Segment SGI170821[] = {{&S52032[70], 6}, {NULL}};
const Segment SGI170822[] = {{&S52033[34], 11}, {NULL}};
const Segment SGI170823[] = {{&S52033[27], 6}, {NULL}};
const Segment SGI170824[] = {{&S52033[98], 9}, {NULL}};
const Segment SGI170825[] = {{&S52033[71], 6}, {NULL}};
const Segment SGI170826[] = {{&S52035[34], 10}, {NULL}};
const Segment SGI170827[] = {{&S52035[27], 6}, {NULL}};
const Segment SGI170828[] = {{&S40322[34], 10}, {NULL}};
const Segment SGI170829[] = {{&S40322[27], 6}, {NULL}};
const Segment SGI170830[] = {{&S52035[97], 9}, {NULL}};
const Segment SGI170831[] = {{&S52035[70], 6}, {NULL}};
const Segment SGI170832[] = {{&S40322[96], 9}, {NULL}};
const Segment SGI170833[] = {{&S40322[69], 6}, {NULL}};
const Segment SGI170834[] = {{&S52036[34], 11}, {NULL}};
const Segment SGI170835[] = {{&S52036[27], 6}, {NULL}};
const Segment SGI170836[] = {{&S40323[34], 11}, {NULL}};
const Segment SGI170837[] = {{&S40323[27], 6}, {NULL}};
const Segment SGI170838[] = {{&S52036[98], 9}, {NULL}};
const Segment SGI170839[] = {{&S52036[71], 6}, {NULL}};
const Segment SGI170840[] = {{&S40323[97], 9}, {NULL}};
const Segment SGI170841[] = {{&S40323[70], 6}, {NULL}};
const Segment SGI170842[] = {{&S52037[34], 11}, {NULL}};
const Segment SGI170843[] = {{&S52037[27], 6}, {NULL}};
const Segment SGI170844[] = {{&S52037[98], 9}, {NULL}};
const Segment SGI170845[] = {{&S52037[71], 6}, {NULL}};
const Segment SGI170846[] = {{&S52038[34], 11}, {NULL}};
const Segment SGI170847[] = {{&S52038[27], 6}, {NULL}};
const Segment SGI170848[] = {{&S52038[98], 9}, {NULL}};
const Segment SGI170849[] = {{&S52038[71], 6}, {NULL}};
const Segment SGI170850[] = {{&S62992[34], 10}, {NULL}};
const Segment SGI170851[] = {{&S62992[27], 6}, {NULL}};
const Segment SGI170852[] = {{&S62992[97], 9}, {NULL}};
const Segment SGI170853[] = {{&S62992[70], 6}, {NULL}};
const Segment SGI170854[] = {{&S52039[34], 11}, {NULL}};
const Segment SGI170855[] = {{&S52039[27], 6}, {NULL}};
const Segment SGI170856[] = {{&S52039[98], 9}, {NULL}};
const Segment SGI170857[] = {{&S52039[71], 6}, {NULL}};
const Segment SGI170858[] = {{&S62993[34], 7}, {NULL}};
const Segment SGI170859[] = {{&S62993[27], 6}, {NULL}};
const Segment SGI170860[] = {{&S62993[98], 9}, {NULL}};
const Segment SGI170861[] = {{&S62993[71], 6}, {NULL}};
const Segment SGI170862[] = {{&S52041[34], 8}, {NULL}};
const Segment SGI170863[] = {{&S52041[27], 6}, {NULL}};
const Segment SGI170864[] = {{&S62994[34], 7}, {NULL}};
const Segment SGI170865[] = {{&S62994[27], 6}, {NULL}};
const Segment SGI170866[] = {{&S52041[100], 9}, {NULL}};
const Segment SGI170867[] = {{&S52041[73], 6}, {NULL}};
const Segment SGI170868[] = {{&S62994[98], 9}, {NULL}};
const Segment SGI170869[] = {{&S62994[71], 6}, {NULL}};
const Segment SGI170870[] = {{&S63002[34], 14}, {NULL}};
const Segment SGI170871[] = {{&S63002[27], 6}, {NULL}};
const Segment SGI170872[] = {{&S63002[106], 9}, {NULL}};
const Segment SGI170873[] = {{&S63002[79], 6}, {NULL}};
const Segment SGI170874[] = {{&S55283[44], 7}, {NULL}};
const Segment SGI170875[] = {{&S55283[44], 19}, {NULL}};
const Segment SGI170876[] = {{&S55283[30], 6}, {NULL}};
const Segment SGI170877[] = {{&S55283[112], 3}, {NULL}};
const Segment SGI170878[] = {{&S55283[77], 6}, {NULL}};
const Segment SGI170879[] = {{&S55284[38], 7}, {NULL}};
const Segment SGI170880[] = {{&S55284[31], 6}, {NULL}};
const Segment SGI170881[] = {{&S63010[34], 12}, {NULL}};
const Segment SGI170882[] = {{&S63010[27], 6}, {NULL}};
const Segment SGI170883[] = {{&S63010[99], 9}, {NULL}};
const Segment SGI170884[] = {{&S63010[72], 6}, {NULL}};
const Segment SGI170885[] = {{&S55285[44], 7}, {NULL}};
const Segment SGI170886[] = {{&S55285[44], 19}, {NULL}};
const Segment SGI170887[] = {{&S55285[30], 6}, {NULL}};
const Segment SGI170888[] = {{&S55285[112], 3}, {NULL}};
const Segment SGI170889[] = {{&S55285[77], 6}, {NULL}};
const Segment SGI170890[] = {{&S63014[34], 14}, {NULL}};
const Segment SGI170891[] = {{&S63014[27], 6}, {NULL}};
const Segment SGI170892[] = {{&S63014[101], 9}, {NULL}};
const Segment SGI170893[] = {{&S63014[74], 6}, {NULL}};
const Segment SGI170894[] = {{&S55304[34], 10}, {NULL}};
const Segment SGI170895[] = {{&S55304[27], 6}, {NULL}};
const Segment SGI170896[] = {{&S55304[97], 9}, {NULL}};
const Segment SGI170897[] = {{&S55304[70], 6}, {NULL}};
const Segment SGI170898[] = {{&S55307[34], 10}, {NULL}};
const Segment SGI170899[] = {{&S55307[27], 6}, {NULL}};
const Segment SGI170900[] = {{&S55307[97], 9}, {NULL}};
const Segment SGI170901[] = {{&S55307[70], 6}, {NULL}};
const Segment SGI170902[] = {{&S52089[34], 7}, {NULL}};
const Segment SGI170903[] = {{&S52089[27], 6}, {NULL}};
const Segment SGI170904[] = {{&S52089[99], 9}, {NULL}};
const Segment SGI170905[] = {{&S52089[72], 6}, {NULL}};
const Segment SGI170906[] = {{&S52090[34], 7}, {NULL}};
const Segment SGI170907[] = {{&S52090[27], 6}, {NULL}};
const Segment SGI170908[] = {{&S52090[99], 9}, {NULL}};
const Segment SGI170909[] = {{&S52090[72], 6}, {NULL}};
const Segment SGI170910[] = {{&S52101[34], 11}, {NULL}};
const Segment SGI170911[] = {{&S52101[27], 6}, {NULL}};
const Segment SGI170912[] = {{&S52101[98], 9}, {NULL}};
const Segment SGI170913[] = {{&S52101[71], 6}, {NULL}};
const Segment SGI170914[] = {{&S52102[34], 11}, {NULL}};
const Segment SGI170915[] = {{&S52102[27], 6}, {NULL}};
const Segment SGI170916[] = {{&S52102[98], 9}, {NULL}};
const Segment SGI170917[] = {{&S52102[71], 6}, {NULL}};
const Segment SGI170918[] = {{&S52108[34], 18}, {NULL}};
const Segment SGI170919[] = {{&S52108[27], 6}, {NULL}};
const Segment SGI170920[] = {{&S52108[105], 9}, {NULL}};
const Segment SGI170921[] = {{&S52108[78], 6}, {NULL}};
const Segment SGI170922[] = {{&S52109[34], 10}, {NULL}};
const Segment SGI170923[] = {{&S52109[27], 6}, {NULL}};
const Segment SGI170924[] = {{&S52109[97], 9}, {NULL}};
const Segment SGI170925[] = {{&S52109[70], 6}, {NULL}};
const Segment SGI170926[] = {{&S52110[34], 11}, {NULL}};
const Segment SGI170927[] = {{&S52110[27], 6}, {NULL}};
const Segment SGI170928[] = {{&S52110[98], 9}, {NULL}};
const Segment SGI170929[] = {{&S52110[71], 6}, {NULL}};
const Segment SGI170930[] = {{&S63065[44], 8}, {NULL}};
const Segment SGI170931[] = {{&S63065[44], 20}, {NULL}};
const Segment SGI170932[] = {{&S63065[30], 6}, {NULL}};
const Segment SGI170933[] = {{&S63065[31], 3}, {NULL}};
const Segment SGI170934[] = {{&S63065[78], 6}, {NULL}};
const Segment SGI170935[] = {{&S63066[38], 8}, {NULL}};
const Segment SGI170936[] = {{&S63066[31], 6}, {NULL}};
const Segment SGI170937[] = {{&S63071[44], 10}, {NULL}};
const Segment SGI170938[] = {{&S63071[44], 21}, {NULL}};
const Segment SGI170939[] = {{&S63071[30], 6}, {NULL}};
const Segment SGI170940[] = {{&S63071[31], 3}, {NULL}};
const Segment SGI170941[] = {{&S63071[79], 6}, {NULL}};
const Segment SGI170942[] = {{&S63072[44], 10}, {NULL}};
const Segment SGI170943[] = {{&S63072[44], 21}, {NULL}};
const Segment SGI170944[] = {{&S63072[30], 6}, {NULL}};
const Segment SGI170945[] = {{&S63072[31], 3}, {NULL}};
const Segment SGI170946[] = {{&S63072[79], 6}, {NULL}};
const Segment SGI170947[] = {{&S63073[44], 10}, {NULL}};
const Segment SGI170948[] = {{&S63073[44], 21}, {NULL}};
const Segment SGI170949[] = {{&S63073[30], 6}, {NULL}};
const Segment SGI170950[] = {{&S63073[31], 3}, {NULL}};
const Segment SGI170951[] = {{&S63073[79], 6}, {NULL}};
const Segment SGI170952[] = {{&S63074[38], 10}, {NULL}};
const Segment SGI170953[] = {{&S63074[31], 6}, {NULL}};
const Segment SGI170954[] = {{&S55360[34], 8}, {NULL}};
const Segment SGI170955[] = {{&S55360[27], 6}, {NULL}};
const Segment SGI170956[] = {{&S55360[95], 9}, {NULL}};
const Segment SGI170957[] = {{&S55360[68], 6}, {NULL}};
const Segment SGI170958[] = {{&S52142[34], 10}, {NULL}};
const Segment SGI170959[] = {{&S52142[27], 6}, {NULL}};
const Segment SGI170960[] = {{&S52142[97], 9}, {NULL}};
const Segment SGI170961[] = {{&S52142[70], 6}, {NULL}};
const Segment SGI170962[] = {{&S52149[34], 10}, {NULL}};
const Segment SGI170963[] = {{&S52149[27], 6}, {NULL}};
const Segment SGI170964[] = {{&S52149[97], 9}, {NULL}};
const Segment SGI170965[] = {{&S52149[70], 6}, {NULL}};
const Segment SGI170966[] = {{&S52150[34], 10}, {NULL}};
const Segment SGI170967[] = {{&S52150[27], 6}, {NULL}};
const Segment SGI170968[] = {{&S52150[97], 9}, {NULL}};
const Segment SGI170969[] = {{&S52150[70], 6}, {NULL}};
const Segment SGI170970[] = {{&S63100[44], 9}, {NULL}};
const Segment SGI170971[] = {{&S63100[44], 21}, {NULL}};
const Segment SGI170972[] = {{&S63100[30], 6}, {NULL}};
const Segment SGI170973[] = {{&S63100[31], 3}, {NULL}};
const Segment SGI170974[] = {{&S63100[98], 6}, {NULL}};
const Segment SGI170975[] = {{&S63100[67], 8}, {NULL}};
const Segment SGI170976[] = {{&S63101[44], 9}, {&S63101[85], 8}, {NULL}};
const Segment SGI170977[] = {{&S63101[44], 21}, {NULL}};
const Segment SGI170978[] = {{&S63101[30], 6}, {NULL}};
const Segment SGI170979[] = {{&S63101[31], 3}, {NULL}};
const Segment SGI170980[] = {{&S63101[119], 5}, {NULL}};
const Segment SGI170981[] = {{&S63101[67], 8}, {NULL}};
const Segment SGI170982[] = {{&S63102[44], 9}, {NULL}};
const Segment SGI170983[] = {{&S63102[44], 21}, {NULL}};
const Segment SGI170984[] = {{&S63102[30], 6}, {NULL}};
const Segment SGI170985[] = {{&S63102[31], 3}, {NULL}};
const Segment SGI170986[] = {{&S63102[98], 6}, {NULL}};
const Segment SGI170987[] = {{&S63102[67], 8}, {NULL}};
const Segment SGI170988[] = {{&S63103[44], 9}, {NULL}};
const Segment SGI170989[] = {{&S63103[44], 20}, {NULL}};
const Segment SGI170990[] = {{&S63103[30], 6}, {NULL}};
const Segment SGI170991[] = {{&S63103[31], 3}, {NULL}};
const Segment SGI170992[] = {{&S63103[97], 6}, {NULL}};
const Segment SGI170993[] = {{&S63103[66], 8}, {NULL}};
const Segment SGI170994[] = {{&S63104[44], 9}, {&S63104[85], 8}, {NULL}};
const Segment SGI170995[] = {{&S63104[44], 21}, {NULL}};
const Segment SGI170996[] = {{&S63104[30], 6}, {NULL}};
const Segment SGI170997[] = {{&S63104[31], 3}, {NULL}};
const Segment SGI170998[] = {{&S63104[119], 5}, {NULL}};
const Segment SGI170999[] = {{&S63104[67], 8}, {NULL}};
const Segment SGI171000[] = {{&S63105[44], 9}, {NULL}};
const Segment SGI171001[] = {{&S63105[44], 20}, {NULL}};
const Segment SGI171002[] = {{&S63105[30], 6}, {NULL}};
const Segment SGI171003[] = {{&S63105[31], 3}, {NULL}};
const Segment SGI171004[] = {{&S63105[97], 6}, {NULL}};
const Segment SGI171005[] = {{&S63105[66], 8}, {NULL}};
const Segment SGI171006[] = {{&S63106[44], 9}, {NULL}};
const Segment SGI171007[] = {{&S63106[44], 20}, {NULL}};
const Segment SGI171008[] = {{&S63106[30], 6}, {NULL}};
const Segment SGI171009[] = {{&S63106[31], 3}, {NULL}};
const Segment SGI171010[] = {{&S63106[97], 6}, {NULL}};
const Segment SGI171011[] = {{&S63106[66], 8}, {NULL}};
const Segment SGI171012[] = {{&S55407[44], 10}, {NULL}};
const Segment SGI171013[] = {{&S55407[44], 22}, {NULL}};
const Segment SGI171014[] = {{&S55407[30], 6}, {NULL}};
const Segment SGI171015[] = {{&S55407[115], 3}, {NULL}};
const Segment SGI171016[] = {{&S55407[80], 6}, {NULL}};
const Segment SGI171017[] = {{&S55408[38], 10}, {NULL}};
const Segment SGI171018[] = {{&S55408[31], 6}, {NULL}};
const Segment SGI171019[] = {{&S52204[34], 10}, {NULL}};
const Segment SGI171020[] = {{&S52204[27], 6}, {NULL}};
const Segment SGI171021[] = {{&S52204[97], 9}, {NULL}};
const Segment SGI171022[] = {{&S52204[70], 6}, {NULL}};
const Segment SGI171023[] = {{&S52205[34], 11}, {NULL}};
const Segment SGI171024[] = {{&S52205[27], 6}, {NULL}};
const Segment SGI171025[] = {{&S52205[98], 9}, {NULL}};
const Segment SGI171026[] = {{&S52205[71], 6}, {NULL}};
const Segment SGI171027[] = {{&S52206[34], 11}, {NULL}};
const Segment SGI171028[] = {{&S52206[27], 6}, {NULL}};
const Segment SGI171029[] = {{&S52206[98], 9}, {NULL}};
const Segment SGI171030[] = {{&S52206[71], 6}, {NULL}};
const Segment SGI171031[] = {{&S55443[44], 18}, {NULL}};
const Segment SGI171032[] = {{&S55443[44], 30}, {NULL}};
const Segment SGI171033[] = {{&S55443[30], 6}, {NULL}};
const Segment SGI171034[] = {{&S55443[139], 3}, {NULL}};
const Segment SGI171035[] = {{&S55443[104], 6}, {NULL}};
const Segment SGI171036[] = {{&S55443[76], 8}, {NULL}};
const Segment SGI171037[] = {{&S55444[44], 18}, {NULL}};
const Segment SGI171038[] = {{&S55444[44], 30}, {NULL}};
const Segment SGI171039[] = {{&S55444[30], 6}, {NULL}};
const Segment SGI171040[] = {{&S55444[139], 3}, {NULL}};
const Segment SGI171041[] = {{&S55444[104], 6}, {NULL}};
const Segment SGI171042[] = {{&S55444[76], 8}, {NULL}};
const Segment SGI171043[] = {{&S52226[34], 7}, {NULL}};
const Segment SGI171044[] = {{&S52226[27], 6}, {NULL}};
const Segment SGI171045[] = {{&S52226[102], 9}, {NULL}};
const Segment SGI171046[] = {{&S52226[75], 6}, {NULL}};
const Segment SGI171047[] = {{&S63180[34], 11}, {NULL}};
const Segment SGI171048[] = {{&S63180[27], 6}, {NULL}};
const Segment SGI171049[] = {{&S63180[98], 9}, {NULL}};
const Segment SGI171050[] = {{&S63180[71], 6}, {NULL}};
const Segment SGI171051[] = {{&S63181[34], 11}, {NULL}};
const Segment SGI171052[] = {{&S63181[27], 6}, {NULL}};
const Segment SGI171053[] = {{&S63181[98], 9}, {NULL}};
const Segment SGI171054[] = {{&S63181[71], 6}, {NULL}};
const Segment SGI171055[] = {{&S63182[34], 11}, {NULL}};
const Segment SGI171056[] = {{&S63182[27], 6}, {NULL}};
const Segment SGI171057[] = {{&S63182[98], 9}, {NULL}};
const Segment SGI171058[] = {{&S63182[71], 6}, {NULL}};
const Segment SGI171059[] = {{&S63183[34], 18}, {NULL}};
const Segment SGI171060[] = {{&S63183[27], 6}, {NULL}};
const Segment SGI171061[] = {{&S63183[105], 9}, {NULL}};
const Segment SGI171062[] = {{&S63183[78], 6}, {NULL}};
const Segment SGI171063[] = {{&S63184[34], 10}, {NULL}};
const Segment SGI171064[] = {{&S63184[27], 6}, {NULL}};
const Segment SGI171065[] = {{&S63184[97], 9}, {NULL}};
const Segment SGI171066[] = {{&S63184[70], 6}, {NULL}};
const Segment SGI171067[] = {{&S55481[44], 10}, {NULL}};
const Segment SGI171068[] = {{&S55481[44], 22}, {NULL}};
const Segment SGI171069[] = {{&S55481[30], 6}, {NULL}};
const Segment SGI171070[] = {{&S55481[131], 3}, {NULL}};
const Segment SGI171071[] = {{&S55481[96], 6}, {NULL}};
const Segment SGI171072[] = {{&S55481[68], 8}, {NULL}};
const Segment SGI171073[] = {{&S55482[44], 10}, {NULL}};
const Segment SGI171074[] = {{&S55482[44], 22}, {NULL}};
const Segment SGI171075[] = {{&S55482[30], 6}, {NULL}};
const Segment SGI171076[] = {{&S55482[145], 3}, {NULL}};
const Segment SGI171077[] = {{&S55482[110], 6}, {NULL}};
const Segment SGI171078[] = {{&S55482[68], 8}, {NULL}};
const Segment SGI171079[] = {{&S55483[44], 10}, {NULL}};
const Segment SGI171080[] = {{&S55483[44], 22}, {NULL}};
const Segment SGI171081[] = {{&S55483[30], 6}, {NULL}};
const Segment SGI171082[] = {{&S55483[131], 3}, {NULL}};
const Segment SGI171083[] = {{&S55483[96], 6}, {NULL}};
const Segment SGI171084[] = {{&S55483[68], 8}, {NULL}};
const Segment SGI171085[] = {{&S63207[34], 7}, {NULL}};
const Segment SGI171086[] = {{&S63207[27], 6}, {NULL}};
const Segment SGI171087[] = {{&S63207[103], 9}, {NULL}};
const Segment SGI171088[] = {{&S63207[76], 6}, {NULL}};
const Segment SGI171089[] = {{&S63222[34], 8}, {NULL}};
const Segment SGI171090[] = {{&S63222[27], 6}, {NULL}};
const Segment SGI171091[] = {{&S63222[107], 9}, {NULL}};
const Segment SGI171092[] = {{&S63222[80], 6}, {NULL}};
const Segment SGI171093[] = {{&S63223[34], 10}, {NULL}};
const Segment SGI171094[] = {{&S63223[27], 6}, {NULL}};
const Segment SGI171095[] = {{&S63223[97], 9}, {NULL}};
const Segment SGI171096[] = {{&S63223[70], 6}, {NULL}};
const Segment SGI171097[] = {{&S52297[34], 10}, {NULL}};
const Segment SGI171098[] = {{&S52297[27], 6}, {NULL}};
const Segment SGI171099[] = {{&S52297[97], 9}, {NULL}};
const Segment SGI171100[] = {{&S52297[70], 6}, {NULL}};
const Segment SGI171101[] = {{&S63233[34], 7}, {NULL}};
const Segment SGI171102[] = {{&S63233[27], 6}, {NULL}};
const Segment SGI171103[] = {{&S63233[94], 9}, {NULL}};
const Segment SGI171104[] = {{&S63233[67], 6}, {NULL}};
const Segment SGI171105[] = {{&S63234[34], 24}, {NULL}};
const Segment SGI171106[] = {{&S63234[27], 6}, {NULL}};
const Segment SGI171107[] = {{&S63234[111], 9}, {NULL}};
const Segment SGI171108[] = {{&S63234[84], 6}, {NULL}};
const Segment SGI171109[] = {{&S63235[34], 23}, {NULL}};
const Segment SGI171110[] = {{&S63235[27], 6}, {NULL}};
const Segment SGI171111[] = {{&S63235[110], 9}, {NULL}};
const Segment SGI171112[] = {{&S63235[83], 6}, {NULL}};
const Segment SGI171113[] = {{&S52312[34], 10}, {NULL}};
const Segment SGI171114[] = {{&S52312[27], 6}, {NULL}};
const Segment SGI171115[] = {{&S52312[97], 9}, {NULL}};
const Segment SGI171116[] = {{&S52312[70], 6}, {NULL}};
const Segment SGI171117[] = {{&S52312[139], 8}, {NULL}};
const Segment SGI171118[] = {{&S52313[34], 10}, {NULL}};
const Segment SGI171119[] = {{&S52313[27], 6}, {NULL}};
const Segment SGI171120[] = {{&S52313[97], 9}, {NULL}};
const Segment SGI171121[] = {{&S52313[70], 6}, {NULL}};
const Segment SGI171122[] = {{&S52313[138], 8}, {NULL}};
const Segment SGI171123[] = {{&S63240[34], 11}, {NULL}};
const Segment SGI171124[] = {{&S63240[27], 6}, {NULL}};
const Segment SGI171125[] = {{&S63240[98], 9}, {NULL}};
const Segment SGI171126[] = {{&S52314[34], 11}, {NULL}};
const Segment SGI171127[] = {{&S63240[71], 6}, {NULL}};
const Segment SGI171128[] = {{&S52314[27], 6}, {NULL}};
const Segment SGI171129[] = {{&S52314[98], 9}, {NULL}};
const Segment SGI171130[] = {{&S52314[71], 6}, {NULL}};
const Segment SGI171131[] = {{&S52314[139], 8}, {NULL}};
const Segment SGI171132[] = {{&S63241[34], 11}, {NULL}};
const Segment SGI171133[] = {{&S63241[27], 6}, {NULL}};
const Segment SGI171134[] = {{&S63241[98], 9}, {NULL}};
const Segment SGI171135[] = {{&S63241[71], 6}, {NULL}};
const Segment SGI171136[] = {{&S52315[34], 11}, {NULL}};
const Segment SGI171137[] = {{&S52315[27], 6}, {NULL}};
const Segment SGI171138[] = {{&S52315[98], 9}, {NULL}};
const Segment SGI171139[] = {{&S52315[71], 6}, {NULL}};
const Segment SGI171140[] = {{&S52315[139], 8}, {NULL}};
const Segment SGI171141[] = {{&S63242[34], 11}, {NULL}};
const Segment SGI171142[] = {{&S63242[27], 6}, {NULL}};
const Segment SGI171143[] = {{&S63242[98], 9}, {NULL}};
const Segment SGI171144[] = {{&S63242[71], 6}, {NULL}};
const Segment SGI171145[] = {{&S52316[34], 11}, {NULL}};
const Segment SGI171146[] = {{&S52316[27], 6}, {NULL}};
const Segment SGI171147[] = {{&S52316[98], 9}, {NULL}};
const Segment SGI171148[] = {{&S52316[71], 6}, {NULL}};
const Segment SGI171149[] = {{&S52316[139], 8}, {NULL}};
const Segment SGI171150[] = {{&S63244[34], 10}, {NULL}};
const Segment SGI171151[] = {{&S63244[27], 6}, {NULL}};
const Segment SGI171152[] = {{&S63244[97], 9}, {NULL}};
const Segment SGI171153[] = {{&S63244[70], 6}, {NULL}};
const Segment SGI171154[] = {{&S63247[44], 8}, {NULL}};
const Segment SGI171155[] = {{&S63247[44], 19}, {NULL}};
const Segment SGI171156[] = {{&S63247[30], 6}, {NULL}};
const Segment SGI171157[] = {{&S63247[31], 3}, {NULL}};
const Segment SGI171158[] = {{&S63247[77], 6}, {NULL}};
const Segment SGI171159[] = {{&S63248[44], 12}, {NULL}};
const Segment SGI171160[] = {{&S63248[44], 23}, {NULL}};
const Segment SGI171161[] = {{&S63248[30], 6}, {NULL}};
const Segment SGI171162[] = {{&S63248[31], 3}, {NULL}};
const Segment SGI171163[] = {{&S63248[81], 6}, {NULL}};
const Segment SGI171164[] = {{&S63249[44], 8}, {NULL}};
const Segment SGI171165[] = {{&S63249[44], 19}, {NULL}};
const Segment SGI171166[] = {{&S63249[30], 6}, {NULL}};
const Segment SGI171167[] = {{&S63249[31], 3}, {NULL}};
const Segment SGI171168[] = {{&S63249[77], 6}, {NULL}};
const Segment SGI171169[] = {{&S63250[44], 12}, {NULL}};
const Segment SGI171170[] = {{&S63250[44], 23}, {NULL}};
const Segment SGI171171[] = {{&S63250[30], 6}, {NULL}};
const Segment SGI171172[] = {{&S63250[31], 3}, {NULL}};
const Segment SGI171173[] = {{&S63250[81], 6}, {NULL}};
const Segment SGI171174[] = {{&S63251[38], 8}, {NULL}};
const Segment SGI171175[] = {{&S63251[31], 6}, {NULL}};
const Segment SGI171176[] = {{&S63252[44], 8}, {NULL}};
const Segment SGI171177[] = {{&S63252[44], 19}, {NULL}};
const Segment SGI171178[] = {{&S63252[30], 6}, {NULL}};
const Segment SGI171179[] = {{&S63252[31], 3}, {NULL}};
const Segment SGI171180[] = {{&S63252[77], 6}, {NULL}};
const Segment SGI171181[] = {{&S63263[34], 10}, {NULL}};
const Segment SGI171182[] = {{&S63263[27], 6}, {NULL}};
const Segment SGI171183[] = {{&S63263[97], 9}, {NULL}};
const Segment SGI171184[] = {{&S52345[44], 28}, {NULL}};
const Segment SGI171185[] = {{&S63263[70], 6}, {NULL}};
const Segment SGI171186[] = {{&S52345[44], 40}, {NULL}};
const Segment SGI171187[] = {{&S52345[30], 6}, {NULL}};
const Segment SGI171188[] = {{&S52345[149], 3}, {NULL}};
const Segment SGI171189[] = {{&S52345[114], 6}, {NULL}};
const Segment SGI171190[] = {{&S52345[86], 8}, {NULL}};
const Segment SGI171191[] = {{&S63265[34], 11}, {NULL}};
const Segment SGI171192[] = {{&S63265[27], 6}, {NULL}};
const Segment SGI171193[] = {{&S63265[98], 9}, {NULL}};
const Segment SGI171194[] = {{&S63265[71], 6}, {NULL}};
const Segment SGI171195[] = {{&S52348[44], 12}, {NULL}};
const Segment SGI171196[] = {{&S52348[44], 24}, {NULL}};
const Segment SGI171197[] = {{&S52348[30], 6}, {NULL}};
const Segment SGI171198[] = {{&S52348[133], 3}, {NULL}};
const Segment SGI171199[] = {{&S52348[98], 6}, {NULL}};
const Segment SGI171200[] = {{&S52348[70], 8}, {NULL}};
const Segment SGI171201[] = {{&S63269[44], 9}, {NULL}};
const Segment SGI171202[] = {{&S63269[44], 20}, {NULL}};
const Segment SGI171203[] = {{&S63269[30], 6}, {NULL}};
const Segment SGI171204[] = {{&S63269[31], 3}, {NULL}};
const Segment SGI171205[] = {{&S63269[78], 6}, {NULL}};
const Segment SGI171206[] = {{&S55590[44], 18}, {NULL}};
const Segment SGI171207[] = {{&S55590[44], 39}, {NULL}};
const Segment SGI171208[] = {{&S55590[30], 6}, {NULL}};
const Segment SGI171209[] = {{&S55590[132], 3}, {NULL}};
const Segment SGI171210[] = {{&S55590[97], 6}, {NULL}};
const Segment SGI171211[] = {{&S63270[44], 9}, {NULL}};
const Segment SGI171212[] = {{&S63270[44], 20}, {NULL}};
const Segment SGI171213[] = {{&S63270[30], 6}, {NULL}};
const Segment SGI171214[] = {{&S63270[31], 3}, {NULL}};
const Segment SGI171215[] = {{&S63270[78], 6}, {NULL}};
const Segment SGI171216[] = {{&S55591[38], 18}, {NULL}};
const Segment SGI171217[] = {{&S55591[31], 6}, {NULL}};
const Segment SGI171218[] = {{&S63271[44], 9}, {NULL}};
const Segment SGI171219[] = {{&S63271[44], 20}, {NULL}};
const Segment SGI171220[] = {{&S63271[30], 6}, {NULL}};
const Segment SGI171221[] = {{&S63271[31], 3}, {NULL}};
const Segment SGI171222[] = {{&S63271[78], 6}, {NULL}};
const Segment SGI171223[] = {{&S63272[44], 9}, {NULL}};
const Segment SGI171224[] = {{&S63272[44], 20}, {NULL}};
const Segment SGI171225[] = {{&S63272[30], 6}, {NULL}};
const Segment SGI171226[] = {{&S63272[31], 3}, {NULL}};
const Segment SGI171227[] = {{&S63272[78], 6}, {NULL}};
const Segment SGI171228[] = {{&S63273[44], 9}, {NULL}};
const Segment SGI171229[] = {{&S63273[44], 20}, {NULL}};
const Segment SGI171230[] = {{&S63273[30], 6}, {NULL}};
const Segment SGI171231[] = {{&S63273[31], 3}, {NULL}};
const Segment SGI171232[] = {{&S63273[78], 6}, {NULL}};
const Segment SGI171233[] = {{&S63274[44], 9}, {NULL}};
const Segment SGI171234[] = {{&S63274[44], 20}, {NULL}};
const Segment SGI171235[] = {{&S63274[30], 6}, {NULL}};
const Segment SGI171236[] = {{&S63274[31], 3}, {NULL}};
const Segment SGI171237[] = {{&S63274[78], 6}, {NULL}};
const Segment SGI171238[] = {{&S52365[44], 11}, {NULL}};
const Segment SGI171239[] = {{&S52365[44], 28}, {NULL}};
const Segment SGI171240[] = {{&S52365[30], 6}, {NULL}};
const Segment SGI171241[] = {{&S63275[44], 9}, {NULL}};
const Segment SGI171242[] = {{&S63275[44], 20}, {NULL}};
const Segment SGI171243[] = {{&S52365[121], 3}, {NULL}};
const Segment SGI171244[] = {{&S63275[30], 6}, {NULL}};
const Segment SGI171245[] = {{&S52365[86], 6}, {NULL}};
const Segment SGI171246[] = {{&S63275[31], 3}, {NULL}};
const Segment SGI171247[] = {{&S63275[78], 6}, {NULL}};
const Segment SGI171248[] = {{&S52366[44], 7}, {NULL}};
const Segment SGI171249[] = {{&S52366[44], 24}, {NULL}};
const Segment SGI171250[] = {{&S52366[30], 6}, {NULL}};
const Segment SGI171251[] = {{&S52366[117], 3}, {NULL}};
const Segment SGI171252[] = {{&S63276[44], 9}, {NULL}};
const Segment SGI171253[] = {{&S63276[44], 20}, {NULL}};
const Segment SGI171254[] = {{&S52366[82], 6}, {NULL}};
const Segment SGI171255[] = {{&S63276[30], 6}, {NULL}};
const Segment SGI171256[] = {{&S63276[31], 3}, {NULL}};
const Segment SGI171257[] = {{&S63276[78], 6}, {NULL}};
const Segment SGI171258[] = {{&S52367[38], 7}, {NULL}};
const Segment SGI171259[] = {{&S52367[31], 6}, {NULL}};
const Segment SGI171260[] = {{&S63277[38], 9}, {NULL}};
const Segment SGI171261[] = {{&S63277[31], 6}, {NULL}};
const Segment SGI171262[] = {{&S52368[44], 11}, {NULL}};
const Segment SGI171263[] = {{&S52368[44], 28}, {NULL}};
const Segment SGI171264[] = {{&S52368[30], 6}, {NULL}};
const Segment SGI171265[] = {{&S52368[121], 3}, {NULL}};
const Segment SGI171266[] = {{&S52368[86], 6}, {NULL}};
const Segment SGI171267[] = {{&S52369[44], 7}, {NULL}};
const Segment SGI171268[] = {{&S52369[44], 24}, {NULL}};
const Segment SGI171269[] = {{&S52369[30], 6}, {NULL}};
const Segment SGI171270[] = {{&S52369[117], 3}, {NULL}};
const Segment SGI171271[] = {{&S52369[82], 6}, {NULL}};
const Segment SGI171272[] = {{&S52370[44], 11}, {NULL}};
const Segment SGI171273[] = {{&S52370[44], 28}, {NULL}};
const Segment SGI171274[] = {{&S52370[30], 6}, {NULL}};
const Segment SGI171275[] = {{&S52370[121], 3}, {NULL}};
const Segment SGI171276[] = {{&S52370[86], 6}, {NULL}};
const Segment SGI171277[] = {{&S52371[44], 11}, {NULL}};
const Segment SGI171278[] = {{&S52371[44], 28}, {NULL}};
const Segment SGI171279[] = {{&S52371[30], 6}, {NULL}};
const Segment SGI171280[] = {{&S52371[121], 3}, {NULL}};
const Segment SGI171281[] = {{&S52371[86], 6}, {NULL}};
const Segment SGI171282[] = {{&S63286[44], 7}, {NULL}};
const Segment SGI171283[] = {{&S63286[44], 19}, {NULL}};
const Segment SGI171284[] = {{&S63286[30], 6}, {NULL}};
const Segment SGI171285[] = {{&S63286[31], 3}, {NULL}};
const Segment SGI171286[] = {{&S63286[77], 6}, {NULL}};
const Segment SGI171287[] = {{&S63287[38], 7}, {NULL}};
const Segment SGI171288[] = {{&S63287[31], 6}, {NULL}};
const Segment SGI171289[] = {{&S63288[44], 14}, {NULL}};
const Segment SGI171290[] = {{&S63288[44], 26}, {NULL}};
const Segment SGI171291[] = {{&S63288[30], 6}, {NULL}};
const Segment SGI171292[] = {{&S63288[31], 3}, {NULL}};
const Segment SGI171293[] = {{&S63288[84], 6}, {NULL}};
const Segment SGI171294[] = {{&S63289[44], 7}, {NULL}};
const Segment SGI171295[] = {{&S63289[44], 19}, {NULL}};
const Segment SGI171296[] = {{&S63289[30], 6}, {NULL}};
const Segment SGI171297[] = {{&S63289[80], 6}, {NULL}};
const Segment SGI171298[] = {{&S63290[44], 13}, {NULL}};
const Segment SGI171299[] = {{&S63290[44], 25}, {NULL}};
const Segment SGI171300[] = {{&S63290[30], 6}, {NULL}};
const Segment SGI171301[] = {{&S52396[34], 7}, {NULL}};
const Segment SGI171302[] = {{&S52396[27], 6}, {NULL}};
const Segment SGI171303[] = {{&S52396[110], 9}, {NULL}};
const Segment SGI171304[] = {{&S52396[83], 6}, {NULL}};
const Segment SGI171305[] = {{&S55646[34], 25}, {NULL}};
const Segment SGI171306[] = {{&S55646[27], 6}, {NULL}};
const Segment SGI171307[] = {{&S55646[112], 9}, {NULL}};
const Segment SGI171308[] = {{&S55646[85], 6}, {NULL}};
const Segment SGI171309[] = {{&S52409[34], 5}, {NULL}};
const Segment SGI171310[] = {{&S52409[27], 6}, {NULL}};
const Segment SGI171311[] = {{&S52409[92], 9}, {NULL}};
const Segment SGI171312[] = {{&S52409[65], 6}, {NULL}};
const Segment SGI171313[] = {{&S55649[11], 1}, {NULL}};
const Segment SGI171314[] = {{&S55649[27], 6}, {NULL}};
const Segment SGI171315[] = {{&S55649[88], 9}, {NULL}};
const Segment SGI171316[] = {{&S55649[61], 6}, {NULL}};
const Segment SGI171317[] = {{&S55650[34], 16}, {NULL}};
const Segment SGI171318[] = {{&S55650[27], 6}, {NULL}};
const Segment SGI171319[] = {{&S55650[103], 9}, {NULL}};
const Segment SGI171320[] = {{&S55650[76], 6}, {NULL}};
const Segment SGI171321[] = {{&S40751[41], 23}, {NULL}};
const Segment SGI171322[] = {{&S40751[41], 34}, {NULL}};
const Segment SGI171323[] = {{&S55651[34], 7}, {NULL}};
const Segment SGI171324[] = {{&S40751[27], 6}, {NULL}};
const Segment SGI171325[] = {{&S55651[27], 6}, {NULL}};
const Segment SGI171326[] = {{&S40751[124], 3}, {NULL}};
const Segment SGI171327[] = {{&S55651[94], 9}, {NULL}};
const Segment SGI171328[] = {{&S40751[128], 9}, {NULL}};
const Segment SGI171329[] = {{&S55651[67], 6}, {NULL}};
const Segment SGI171330[] = {{&S40751[89], 6}, {NULL}};
const Segment SGI171331[] = {{&S40752[41], 19}, {NULL}};
const Segment SGI171332[] = {{&S40752[41], 30}, {NULL}};
const Segment SGI171333[] = {{&S40752[27], 6}, {NULL}};
const Segment SGI171334[] = {{&S40752[120], 3}, {NULL}};
const Segment SGI171335[] = {{&S40752[124], 9}, {NULL}};
const Segment SGI171336[] = {{&S55653[34], 31}, {NULL}};
const Segment SGI171337[] = {{&S40752[85], 6}, {NULL}};
const Segment SGI171338[] = {{&S55653[27], 6}, {NULL}};
const Segment SGI171339[] = {{&S55653[118], 9}, {NULL}};
const Segment SGI171340[] = {{&S55653[91], 6}, {NULL}};
const Segment SGI171341[] = {{&S40753[41], 19}, {NULL}};
const Segment SGI171342[] = {{&S40753[41], 30}, {NULL}};
const Segment SGI171343[] = {{&S40753[27], 6}, {NULL}};
const Segment SGI171344[] = {{&S40753[120], 3}, {NULL}};
const Segment SGI171345[] = {{&S40753[124], 9}, {NULL}};
const Segment SGI171346[] = {{&S55654[34], 27}, {NULL}};
const Segment SGI171347[] = {{&S40753[85], 6}, {NULL}};
const Segment SGI171348[] = {{&S55654[27], 6}, {NULL}};
const Segment SGI171349[] = {{&S55654[114], 9}, {NULL}};
const Segment SGI171350[] = {{&S55654[87], 6}, {NULL}};
const Segment SGI171351[] = {{&S40754[41], 28}, {NULL}};
const Segment SGI171352[] = {{&S40754[41], 39}, {NULL}};
const Segment SGI171353[] = {{&S40754[27], 6}, {NULL}};
const Segment SGI171354[] = {{&S40754[129], 3}, {NULL}};
const Segment SGI171355[] = {{&S55655[34], 27}, {NULL}};
const Segment SGI171356[] = {{&S40754[135], 9}, {NULL}};
const Segment SGI171357[] = {{&S40754[94], 6}, {NULL}};
const Segment SGI171358[] = {{&S55655[27], 6}, {NULL}};
const Segment SGI171359[] = {{&S55655[114], 9}, {NULL}};
const Segment SGI171360[] = {{&S55655[87], 6}, {NULL}};
const Segment SGI171361[] = {{&S59901[42], 17}, {NULL}};
const Segment SGI171362[] = {{&S59901[30], 4}, {NULL}};
const Segment SGI171363[] = {{&S55656[11], 1}, {NULL}};
const Segment SGI171364[] = {{&S59901[31], 3}, {NULL}};
const Segment SGI171365[] = {{&S55656[27], 6}, {NULL}};
const Segment SGI171366[] = {{&S59901[73], 6}, {NULL}};
const Segment SGI171367[] = {{&S55656[88], 9}, {NULL}};
const Segment SGI171368[] = {{&S55656[61], 6}, {NULL}};
const Segment SGI171369[] = {{&S40757[34], 10}, {NULL}};
const Segment SGI171370[] = {{&S40757[27], 6}, {NULL}};
const Segment SGI171371[] = {{&S55657[34], 31}, {NULL}};
const Segment SGI171372[] = {{&S55657[27], 6}, {NULL}};
const Segment SGI171373[] = {{&S40757[96], 9}, {NULL}};
const Segment SGI171374[] = {{&S40757[69], 6}, {NULL}};
const Segment SGI171375[] = {{&S55657[118], 9}, {NULL}};
const Segment SGI171376[] = {{&S55657[91], 6}, {NULL}};
const Segment SGI171377[] = {{&S63290[31], 3}, {NULL}};
const Segment SGI171378[] = {{&S63290[83], 6}, {NULL}};
const Segment SGI171379[] = {{&S55658[34], 20}, {NULL}};
const Segment SGI171380[] = {{&S52428[34], 9}, {NULL}};
const Segment SGI171381[] = {{&S55658[27], 6}, {NULL}};
const Segment SGI171382[] = {{&S52428[27], 6}, {NULL}};
const Segment SGI171383[] = {{&S55658[107], 9}, {NULL}};
const Segment SGI171384[] = {{&S52428[96], 9}, {NULL}};
const Segment SGI171385[] = {{&S55658[80], 6}, {NULL}};
const Segment SGI171386[] = {{&S52428[69], 6}, {NULL}};
const Segment SGI171387[] = {{&S63292[44], 15}, {NULL}};
const Segment SGI171388[] = {{&S63292[44], 27}, {NULL}};
const Segment SGI171389[] = {{&S63292[30], 6}, {NULL}};
const Segment SGI171390[] = {{&S63292[31], 3}, {NULL}};
const Segment SGI171391[] = {{&S63292[85], 6}, {NULL}};
const Segment SGI171392[] = {{&S55660[34], 25}, {NULL}};
const Segment SGI171393[] = {{&S55660[27], 6}, {NULL}};
const Segment SGI171394[] = {{&S55660[112], 9}, {NULL}};
const Segment SGI171395[] = {{&S55660[85], 6}, {NULL}};
const Segment SGI171396[] = {{&S55662[34], 16}, {NULL}};
const Segment SGI171397[] = {{&S55662[27], 6}, {NULL}};
const Segment SGI171398[] = {{&S55662[103], 9}, {NULL}};
const Segment SGI171399[] = {{&S55662[76], 6}, {NULL}};
const Segment SGI171400[] = {{&S55663[34], 21}, {NULL}};
const Segment SGI171401[] = {{&S55663[27], 6}, {NULL}};
const Segment SGI171402[] = {{&S55663[108], 9}, {NULL}};
const Segment SGI171403[] = {{&S55663[81], 6}, {NULL}};
const Segment SGI171404[] = {{&S55664[34], 28}, {NULL}};
const Segment SGI171405[] = {{&S55664[27], 6}, {NULL}};
const Segment SGI171406[] = {{&S55664[115], 9}, {NULL}};
const Segment SGI171407[] = {{&S55664[88], 6}, {NULL}};
const Segment SGI171408[] = {{&S55666[44], 6}, {NULL}};
const Segment SGI171409[] = {{&S55666[44], 18}, {NULL}};
const Segment SGI171410[] = {{&S55666[30], 6}, {NULL}};
const Segment SGI171411[] = {{&S55666[111], 3}, {NULL}};
const Segment SGI171412[] = {{&S55666[76], 6}, {NULL}};
const Segment SGI171413[] = {{&S55667[44], 6}, {NULL}};
const Segment SGI171414[] = {{&S55667[44], 18}, {NULL}};
const Segment SGI171415[] = {{&S55667[30], 6}, {NULL}};
const Segment SGI171416[] = {{&S55667[79], 6}, {NULL}};
const Segment SGI171417[] = {{&S55668[38], 6}, {NULL}};
const Segment SGI171418[] = {{&S55668[31], 6}, {NULL}};
const Segment SGI171419[] = {{&S49602[34], 10}, {NULL}};
const Segment SGI171420[] = {{&S49602[27], 6}, {NULL}};
const Segment SGI171421[] = {{&S55669[44], 6}, {NULL}};
const Segment SGI171422[] = {{&S55669[44], 18}, {NULL}};
const Segment SGI171423[] = {{&S55669[30], 6}, {NULL}};
const Segment SGI171424[] = {{&S49602[97], 9}, {NULL}};
const Segment SGI171425[] = {{&S49602[70], 6}, {NULL}};
const Segment SGI171426[] = {{&S55669[123], 3}, {NULL}};
const Segment SGI171427[] = {{&S55669[76], 6}, {NULL}};
const Segment SGI171428[] = {{&S55694[44], 10}, {NULL}};
const Segment SGI171429[] = {{&S55694[44], 22}, {NULL}};
const Segment SGI171430[] = {{&S55694[30], 6}, {NULL}};
const Segment SGI171431[] = {{&S55694[131], 3}, {NULL}};
const Segment SGI171432[] = {{&S55694[96], 6}, {NULL}};
const Segment SGI171433[] = {{&S55694[68], 8}, {NULL}};
const Segment SGI171434[] = {{&S55695[44], 10}, {NULL}};
const Segment SGI171435[] = {{&S55695[44], 22}, {NULL}};
const Segment SGI171436[] = {{&S55695[30], 6}, {NULL}};
const Segment SGI171437[] = {{&S55695[131], 3}, {NULL}};
const Segment SGI171438[] = {{&S55695[96], 6}, {NULL}};
const Segment SGI171439[] = {{&S55695[68], 8}, {NULL}};
const Segment SGI171440[] = {{&S55696[44], 10}, {NULL}};
const Segment SGI171441[] = {{&S55696[44], 22}, {NULL}};
const Segment SGI171442[] = {{&S55696[30], 6}, {NULL}};
const Segment SGI171443[] = {{&S55696[131], 3}, {NULL}};
const Segment SGI171444[] = {{&S55696[96], 6}, {NULL}};
const Segment SGI171445[] = {{&S55696[68], 8}, {NULL}};
const Segment SGI171446[] = {{&S55707[44], 7}, {NULL}};
const Segment SGI171447[] = {{&S55707[44], 19}, {NULL}};
const Segment SGI171448[] = {{&S55707[30], 6}, {NULL}};
const Segment SGI171449[] = {{&S55707[112], 3}, {NULL}};
const Segment SGI171450[] = {{&S55707[77], 6}, {NULL}};
const Segment SGI171451[] = {{&S55708[38], 7}, {NULL}};
const Segment SGI171452[] = {{&S55708[31], 6}, {NULL}};
const Segment SGI171453[] = {{&S52506[34], 10}, {NULL}};
const Segment SGI171454[] = {{&S52506[27], 6}, {NULL}};
const Segment SGI171455[] = {{&S52506[97], 9}, {NULL}};
const Segment SGI171456[] = {{&S52506[70], 6}, {NULL}};
const Segment SGI171457[] = {{&S59988[32], 9}, {NULL}};
const Segment SGI171458[] = {{&S59988[27], 4}, {NULL}};
const Segment SGI171459[] = {{&S59988[94], 9}, {NULL}};
const Segment SGI171460[] = {{&S59988[67], 6}, {NULL}};
const Segment SGI171461[] = {{&S63348[34], 8}, {NULL}};
const Segment SGI171462[] = {{&S63348[27], 6}, {NULL}};
const Segment SGI171463[] = {{&S63348[107], 9}, {NULL}};
const Segment SGI171464[] = {{&S63348[80], 6}, {NULL}};
const Segment SGI171465[] = {{&S63349[34], 7}, {NULL}};
const Segment SGI171466[] = {{&S63349[27], 6}, {NULL}};
const Segment SGI171467[] = {{&S63349[112], 9}, {NULL}};
const Segment SGI171468[] = {{&S63349[85], 6}, {NULL}};
const Segment SGI171469[] = {{&S63350[44], 15}, {NULL}};
const Segment SGI171470[] = {{&S63350[44], 27}, {NULL}};
const Segment SGI171471[] = {{&S63350[30], 6}, {NULL}};
const Segment SGI171472[] = {{&S52522[34], 11}, {NULL}};
const Segment SGI171473[] = {{&S63350[31], 3}, {NULL}};
const Segment SGI171474[] = {{&S52522[27], 6}, {NULL}};
const Segment SGI171475[] = {{&S63350[85], 6}, {NULL}};
const Segment SGI171476[] = {{&S52522[98], 9}, {NULL}};
const Segment SGI171477[] = {{&S52522[71], 6}, {NULL}};
const Segment SGI171478[] = {{&S63351[38], 15}, {NULL}};
const Segment SGI171479[] = {{&S63351[31], 6}, {NULL}};
const Segment SGI171480[] = {{&S63369[34], 7}, {NULL}};
const Segment SGI171481[] = {{&S63369[27], 6}, {NULL}};
const Segment SGI171482[] = {{&S63369[111], 9}, {NULL}};
const Segment SGI171483[] = {{&S63369[84], 6}, {NULL}};
const Segment SGI171484[] = {{&S49695[34], 10}, {NULL}};
const Segment SGI171485[] = {{&S49695[27], 6}, {NULL}};
const Segment SGI171486[] = {{&S63370[34], 7}, {NULL}};
const Segment SGI171487[] = {{&S63370[27], 6}, {NULL}};
const Segment SGI171488[] = {{&S49695[97], 9}, {NULL}};
const Segment SGI171489[] = {{&S49695[70], 6}, {NULL}};
const Segment SGI171490[] = {{&S63370[111], 9}, {NULL}};
const Segment SGI171491[] = {{&S63370[84], 6}, {NULL}};
const Segment SGI171492[] = {{&S49698[34], 11}, {NULL}};
const Segment SGI171493[] = {{&S49698[27], 6}, {NULL}};
const Segment SGI171494[] = {{&S49698[98], 9}, {NULL}};
const Segment SGI171495[] = {{&S49698[71], 6}, {NULL}};
const Segment SGI171496[] = {{&S52551[34], 8}, {NULL}};
const Segment SGI171497[] = {{&S52551[27], 6}, {NULL}};
const Segment SGI171498[] = {{&S63374[34], 14}, {NULL}};
const Segment SGI171499[] = {{&S63374[27], 6}, {NULL}};
const Segment SGI171500[] = {{&S52551[101], 9}, {NULL}};
const Segment SGI171501[] = {{&S52551[74], 6}, {NULL}};
const Segment SGI171502[] = {{&S63374[101], 9}, {NULL}};
const Segment SGI171503[] = {{&S63374[74], 6}, {NULL}};
const Segment SGI171504[] = {{&S52552[34], 7}, {NULL}};
const Segment SGI171505[] = {{&S52552[27], 6}, {NULL}};
const Segment SGI171506[] = {{&S63375[34], 14}, {NULL}};
const Segment SGI171507[] = {{&S63375[27], 6}, {NULL}};
const Segment SGI171508[] = {{&S52552[102], 9}, {NULL}};
const Segment SGI171509[] = {{&S52552[75], 6}, {NULL}};
const Segment SGI171510[] = {{&S63375[101], 9}, {NULL}};
const Segment SGI171511[] = {{&S63375[74], 6}, {NULL}};
const Segment SGI171512[] = {{&S49704[44], 11}, {NULL}};
const Segment SGI171513[] = {{&S49704[44], 23}, {NULL}};
const Segment SGI171514[] = {{&S49704[30], 6}, {NULL}};
const Segment SGI171515[] = {{&S49704[134], 3}, {NULL}};
const Segment SGI171516[] = {{&S49704[99], 6}, {NULL}};
const Segment SGI171517[] = {{&S49704[69], 8}, {NULL}};
const Segment SGI171518[] = {{&S49705[44], 11}, {NULL}};
const Segment SGI171519[] = {{&S49705[44], 23}, {NULL}};
const Segment SGI171520[] = {{&S49705[30], 6}, {NULL}};
const Segment SGI171521[] = {{&S49705[134], 3}, {NULL}};
const Segment SGI171522[] = {{&S49705[99], 6}, {NULL}};
const Segment SGI171523[] = {{&S49705[69], 8}, {NULL}};
const Segment SGI171524[] = {{&S52560[34], 9}, {NULL}};
const Segment SGI171525[] = {{&S52560[27], 6}, {NULL}};
const Segment SGI171526[] = {{&S52560[96], 9}, {NULL}};
const Segment SGI171527[] = {{&S52560[69], 6}, {NULL}};
const Segment SGI171528[] = {{&S52561[34], 10}, {NULL}};
const Segment SGI171529[] = {{&S52561[27], 6}, {NULL}};
const Segment SGI171530[] = {{&S52561[97], 9}, {NULL}};
const Segment SGI171531[] = {{&S52561[70], 6}, {NULL}};
const Segment SGI171532[] = {{&S55782[34], 20}, {NULL}};
const Segment SGI171533[] = {{&S55782[27], 6}, {NULL}};
const Segment SGI171534[] = {{&S55782[107], 9}, {NULL}};
const Segment SGI171535[] = {{&S55782[80], 6}, {NULL}};
const Segment SGI171536[] = {{&S52566[34], 10}, {NULL}};
const Segment SGI171537[] = {{&S52566[27], 6}, {NULL}};
const Segment SGI171538[] = {{&S52566[97], 9}, {NULL}};
const Segment SGI171539[] = {{&S55783[34], 20}, {NULL}};
const Segment SGI171540[] = {{&S52566[70], 6}, {NULL}};
const Segment SGI171541[] = {{&S55783[27], 6}, {NULL}};
const Segment SGI171542[] = {{&S55783[107], 9}, {NULL}};
const Segment SGI171543[] = {{&S55783[80], 6}, {NULL}};
const Segment SGI171544[] = {{&S63415[34], 18}, {NULL}};
const Segment SGI171545[] = {{&S63415[27], 6}, {NULL}};
const Segment SGI171546[] = {{&S63415[105], 9}, {NULL}};
const Segment SGI171547[] = {{&S55805[44], 10}, {NULL}};
const Segment SGI171548[] = {{&S63415[78], 6}, {NULL}};
const Segment SGI171549[] = {{&S55805[44], 22}, {NULL}};
const Segment SGI171550[] = {{&S55805[30], 6}, {NULL}};
const Segment SGI171551[] = {{&S55805[80], 6}, {NULL}};
const Segment SGI171552[] = {{&S55805[107], 13}, {NULL}};
const Segment SGI171553[] = {{&S52605[34], 12}, {NULL}};
const Segment SGI171554[] = {{&S52605[27], 6}, {NULL}};
const Segment SGI171555[] = {{&S52605[99], 9}, {NULL}};
const Segment SGI171556[] = {{&S52605[72], 6}, {NULL}};
const Segment SGI171557[] = {{&S63435[34], 7}, {NULL}};
const Segment SGI171558[] = {{&S63435[27], 6}, {NULL}};
const Segment SGI171559[] = {{&S63435[98], 9}, {NULL}};
const Segment SGI171560[] = {{&S63435[71], 6}, {NULL}};
const Segment SGI171561[] = {{&S63436[34], 10}, {NULL}};
const Segment SGI171562[] = {{&S63436[27], 6}, {NULL}};
const Segment SGI171563[] = {{&S63436[97], 9}, {NULL}};
const Segment SGI171564[] = {{&S63436[70], 6}, {NULL}};
const Segment SGI171565[] = {{&S63437[34], 10}, {NULL}};
const Segment SGI171566[] = {{&S63437[27], 6}, {NULL}};
const Segment SGI171567[] = {{&S63437[97], 9}, {NULL}};
const Segment SGI171568[] = {{&S63437[70], 6}, {NULL}};
const Segment SGI171569[] = {{&S52628[34], 7}, {NULL}};
const Segment SGI171570[] = {{&S52628[27], 6}, {NULL}};
const Segment SGI171571[] = {{&S52628[104], 9}, {NULL}};
const Segment SGI171572[] = {{&S52628[77], 6}, {NULL}};
const Segment SGI171573[] = {{&S63453[44], 10}, {NULL}};
const Segment SGI171574[] = {{&S63453[44], 22}, {NULL}};
const Segment SGI171575[] = {{&S63453[30], 6}, {NULL}};
const Segment SGI171576[] = {{&S63453[31], 3}, {NULL}};
const Segment SGI171577[] = {{&S63453[80], 6}, {NULL}};
const Segment SGI171578[] = {{&S63454[38], 10}, {NULL}};
const Segment SGI171579[] = {{&S63454[31], 6}, {NULL}};
const Segment SGI171580[] = {{&S52634[34], 10}, {NULL}};
const Segment SGI171581[] = {{&S52634[27], 6}, {NULL}};
const Segment SGI171582[] = {{&S52634[97], 9}, {NULL}};
const Segment SGI171583[] = {{&S52634[70], 6}, {NULL}};
const Segment SGI171584[] = {{&S52639[44], 6}, {NULL}};
const Segment SGI171585[] = {{&S52639[44], 18}, {NULL}};
const Segment SGI171586[] = {{&S52639[30], 6}, {NULL}};
const Segment SGI171587[] = {{&S52639[111], 3}, {NULL}};
const Segment SGI171588[] = {{&S52639[76], 6}, {NULL}};
const Segment SGI171589[] = {{&S52640[38], 6}, {NULL}};
const Segment SGI171590[] = {{&S63464[34], 10}, {NULL}};
const Segment SGI171591[] = {{&S52640[31], 6}, {NULL}};
const Segment SGI171592[] = {{&S63464[27], 6}, {NULL}};
const Segment SGI171593[] = {{&S63464[97], 9}, {NULL}};
const Segment SGI171594[] = {{&S63464[70], 6}, {NULL}};
const Segment SGI171595[] = {{&S55870[34], 18}, {NULL}};
const Segment SGI171596[] = {{&S55870[27], 6}, {NULL}};
const Segment SGI171597[] = {{&S55870[114], 9}, {NULL}};
const Segment SGI171598[] = {{&S55870[87], 6}, {NULL}};
const Segment SGI171599[] = {{&S63473[44], 7}, {NULL}};
const Segment SGI171600[] = {{&S63473[44], 19}, {NULL}};
const Segment SGI171601[] = {{&S63473[30], 6}, {NULL}};
const Segment SGI171602[] = {{&S63473[31], 3}, {NULL}};
const Segment SGI171603[] = {{&S63473[77], 6}, {NULL}};
const Segment SGI171604[] = {{&S63476[34], 7}, {NULL}};
const Segment SGI171605[] = {{&S63476[27], 6}, {NULL}};
const Segment SGI171606[] = {{&S63476[105], 9}, {NULL}};
const Segment SGI171607[] = {{&S63476[78], 6}, {NULL}};
const Segment SGI171608[] = {{&S63478[34], 9}, {NULL}};
const Segment SGI171609[] = {{&S63478[27], 6}, {NULL}};
const Segment SGI171610[] = {{&S63478[96], 9}, {NULL}};
const Segment SGI171611[] = {{&S63478[69], 6}, {NULL}};
const Segment SGI171612[] = {{&S41001[34], 18}, {NULL}};
const Segment SGI171613[] = {{&S41001[27], 6}, {NULL}};
const Segment SGI171614[] = {{&S41001[104], 9}, {NULL}};
const Segment SGI171615[] = {{&S41001[77], 6}, {NULL}};
const Segment SGI171616[] = {{&S41002[34], 10}, {NULL}};
const Segment SGI171617[] = {{&S41002[27], 6}, {NULL}};
const Segment SGI171618[] = {{&S41002[97], 9}, {NULL}};
const Segment SGI171619[] = {{&S41002[70], 6}, {NULL}};
const Segment SGI171620[] = {{&S63506[44], 13}, {NULL}};
const Segment SGI171621[] = {{&S63506[44], 25}, {NULL}};
const Segment SGI171622[] = {{&S63506[30], 6}, {NULL}};
const Segment SGI171623[] = {{&S63506[86], 6}, {NULL}};
const Segment SGI171624[] = {{&S63507[44], 13}, {NULL}};
const Segment SGI171625[] = {{&S63507[44], 25}, {NULL}};
const Segment SGI171626[] = {{&S63507[30], 6}, {NULL}};
const Segment SGI171627[] = {{&S63507[31], 3}, {NULL}};
const Segment SGI171628[] = {{&S63507[83], 6}, {NULL}};
const Segment SGI171629[] = {{&S63508[38], 13}, {NULL}};
const Segment SGI171630[] = {{&S63508[31], 6}, {NULL}};
const Segment SGI171631[] = {{&S63509[44], 13}, {NULL}};
const Segment SGI171632[] = {{&S63509[44], 25}, {NULL}};
const Segment SGI171633[] = {{&S63509[30], 6}, {NULL}};
const Segment SGI171634[] = {{&S63509[31], 3}, {NULL}};
const Segment SGI171635[] = {{&S63509[83], 6}, {NULL}};
const Segment SGI171636[] = {{&S63510[30], 6}, {NULL}};
const Segment SGI171637[] = {{&S63510[31], 3}, {NULL}};
const Segment SGI171638[] = {{&S63510[81], 6}, {NULL}};
const Segment SGI171639[] = {{&S63511[44], 20}, {NULL}};
const Segment SGI171640[] = {{&S63511[44], 32}, {NULL}};
const Segment SGI171641[] = {{&S63511[30], 6}, {NULL}};
const Segment SGI171642[] = {{&S63511[31], 3}, {NULL}};
const Segment SGI171643[] = {{&S63511[90], 6}, {NULL}};
const Segment SGI171644[] = {{&S49852[34], 10}, {NULL}};
const Segment SGI171645[] = {{&S49852[27], 6}, {NULL}};
const Segment SGI171646[] = {{&S49852[97], 9}, {NULL}};
const Segment SGI171647[] = {{&S49852[70], 6}, {NULL}};
const Segment SGI171648[] = {{&S63512[44], 20}, {NULL}};
const Segment SGI171649[] = {{&S63512[44], 32}, {NULL}};
const Segment SGI171650[] = {{&S63512[30], 6}, {NULL}};
const Segment SGI171651[] = {{&S63512[31], 3}, {NULL}};
const Segment SGI171652[] = {{&S63512[90], 6}, {NULL}};
const Segment SGI171653[] = {{&S63513[44], 13}, {NULL}};
const Segment SGI171654[] = {{&S63513[44], 25}, {NULL}};
const Segment SGI171655[] = {{&S63513[30], 6}, {NULL}};
const Segment SGI171656[] = {{&S63513[31], 3}, {NULL}};
const Segment SGI171657[] = {{&S63513[83], 6}, {NULL}};
const Segment SGI171658[] = {{&S41041[34], 9}, {NULL}};
const Segment SGI171659[] = {{&S41041[27], 6}, {NULL}};
const Segment SGI171660[] = {{&S41041[95], 9}, {NULL}};
const Segment SGI171661[] = {{&S41041[68], 6}, {NULL}};
const Segment SGI171662[] = {{&S41044[34], 9}, {NULL}};
const Segment SGI171663[] = {{&S41044[27], 6}, {NULL}};
const Segment SGI171664[] = {{&S41044[95], 9}, {NULL}};
const Segment SGI171665[] = {{&S41044[68], 6}, {NULL}};
const Segment SGI171666[] = {{&S63526[44], 9}, {NULL}};
const Segment SGI171667[] = {{&S63526[44], 21}, {NULL}};
const Segment SGI171668[] = {{&S63526[30], 6}, {NULL}};
const Segment SGI171669[] = {{&S63526[31], 3}, {NULL}};
const Segment SGI171670[] = {{&S63526[79], 6}, {NULL}};
const Segment SGI171671[] = {{&S63527[44], 9}, {NULL}};
const Segment SGI171672[] = {{&S63527[44], 21}, {NULL}};
const Segment SGI171673[] = {{&S63527[30], 6}, {NULL}};
const Segment SGI171674[] = {{&S63527[31], 3}, {NULL}};
const Segment SGI171675[] = {{&S63527[79], 6}, {NULL}};
const Segment SGI171676[] = {{&S63528[38], 9}, {NULL}};
const Segment SGI171677[] = {{&S63528[31], 6}, {NULL}};
const Segment SGI171678[] = {{&S63538[34], 15}, {NULL}};
const Segment SGI171679[] = {{&S63538[27], 6}, {NULL}};
const Segment SGI171680[] = {{&S63538[102], 9}, {NULL}};
const Segment SGI171681[] = {{&S63538[75], 6}, {NULL}};
const Segment SGI171682[] = {{&S55957[44], 17}, {NULL}};
const Segment SGI171683[] = {{&S55957[44], 29}, {NULL}};
const Segment SGI171684[] = {{&S55957[30], 6}, {NULL}};
const Segment SGI171685[] = {{&S55957[122], 3}, {NULL}};
const Segment SGI171686[] = {{&S55957[87], 6}, {NULL}};
const Segment SGI171687[] = {{&S55958[38], 17}, {NULL}};
const Segment SGI171688[] = {{&S55958[31], 6}, {NULL}};
const Segment SGI171689[] = {{&S55959[44], 19}, {NULL}};
const Segment SGI171690[] = {{&S55959[44], 31}, {NULL}};
const Segment SGI171691[] = {{&S55959[30], 6}, {NULL}};
const Segment SGI171692[] = {{&S55959[124], 3}, {NULL}};
const Segment SGI171693[] = {{&S55959[89], 6}, {NULL}};
const Segment SGI171694[] = {{&S63545[34], 10}, {NULL}};
const Segment SGI171695[] = {{&S63545[27], 6}, {NULL}};
const Segment SGI171696[] = {{&S63545[97], 9}, {NULL}};
const Segment SGI171697[] = {{&S63545[70], 6}, {NULL}};
const Segment SGI171698[] = {{&S63546[34], 13}, {NULL}};
const Segment SGI171699[] = {{&S63546[27], 6}, {NULL}};
const Segment SGI171700[] = {{&S63546[100], 9}, {NULL}};
const Segment SGI171701[] = {{&S63546[73], 6}, {NULL}};
const Segment SGI171702[] = {{&S52752[34], 11}, {NULL}};
const Segment SGI171703[] = {{&S52752[27], 6}, {NULL}};
const Segment SGI171704[] = {{&S52752[98], 9}, {NULL}};
const Segment SGI171705[] = {{&S52752[71], 6}, {NULL}};
const Segment SGI171706[] = {{&S52753[34], 11}, {NULL}};
const Segment SGI171707[] = {{&S52753[27], 6}, {NULL}};
const Segment SGI171708[] = {{&S63552[44], 13}, {NULL}};
const Segment SGI171709[] = {{&S63552[44], 25}, {NULL}};
const Segment SGI171710[] = {{&S52753[98], 9}, {NULL}};
const Segment SGI171711[] = {{&S63552[30], 6}, {NULL}};
const Segment SGI171712[] = {{&S52753[71], 6}, {NULL}};
const Segment SGI171713[] = {{&S63552[31], 3}, {NULL}};
const Segment SGI171714[] = {{&S63552[83], 6}, {NULL}};
const Segment SGI171715[] = {{&S55982[34], 6}, {NULL}};
const Segment SGI171716[] = {{&S55982[27], 6}, {NULL}};
const Segment SGI171717[] = {{&S55982[93], 9}, {NULL}};
const Segment SGI171718[] = {{&S55982[66], 6}, {NULL}};
const Segment SGI171719[] = {{&S63569[34], 14}, {NULL}};
const Segment SGI171720[] = {{&S63569[27], 6}, {NULL}};
const Segment SGI171721[] = {{&S63569[101], 9}, {NULL}};
const Segment SGI171722[] = {{&S63569[74], 6}, {NULL}};
const Segment SGI171723[] = {{&S63571[44], 8}, {NULL}};
const Segment SGI171724[] = {{&S63571[44], 20}, {NULL}};
const Segment SGI171725[] = {{&S63571[30], 6}, {NULL}};
const Segment SGI171726[] = {{&S63571[31], 3}, {NULL}};
const Segment SGI171727[] = {{&S63571[78], 6}, {NULL}};
const Segment SGI171728[] = {{&S52780[34], 12}, {NULL}};
const Segment SGI171729[] = {{&S52780[27], 6}, {NULL}};
const Segment SGI171730[] = {{&S52780[99], 9}, {NULL}};
const Segment SGI171731[] = {{&S52780[72], 6}, {NULL}};
const Segment SGI171732[] = {{&S52793[44], 11}, {NULL}};
const Segment SGI171733[] = {{&S52793[44], 23}, {NULL}};
const Segment SGI171734[] = {{&S52793[30], 6}, {NULL}};
const Segment SGI171735[] = {{&S52793[116], 3}, {NULL}};
const Segment SGI171736[] = {{&S52793[81], 6}, {NULL}};
const Segment SGI171737[] = {{&S52794[38], 11}, {NULL}};
const Segment SGI171738[] = {{&S52794[31], 6}, {NULL}};
const Segment SGI171739[] = {{&S63593[44], 17}, {NULL}};
const Segment SGI171740[] = {{&S63593[44], 29}, {NULL}};
const Segment SGI171741[] = {{&S63593[30], 6}, {NULL}};
const Segment SGI171742[] = {{&S63593[31], 3}, {NULL}};
const Segment SGI171743[] = {{&S63593[87], 6}, {NULL}};
const Segment SGI171744[] = {{&S63594[44], 17}, {NULL}};
const Segment SGI171745[] = {{&S63594[44], 29}, {NULL}};
const Segment SGI171746[] = {{&S63594[30], 6}, {NULL}};
const Segment SGI171747[] = {{&S63594[31], 3}, {NULL}};
const Segment SGI171748[] = {{&S63594[87], 6}, {NULL}};
const Segment SGI171749[] = {{&S52806[34], 7}, {NULL}};
const Segment SGI171750[] = {{&S52806[27], 6}, {NULL}};
const Segment SGI171751[] = {{&S52806[99], 9}, {NULL}};
const Segment SGI171752[] = {{&S52806[72], 6}, {NULL}};
const Segment SGI171753[] = {{&S56020[44], 10}, {NULL}};
const Segment SGI171754[] = {{&S56020[44], 22}, {NULL}};
const Segment SGI171755[] = {{&S56020[30], 6}, {NULL}};
const Segment SGI171756[] = {{&S56020[80], 6}, {NULL}};
const Segment SGI171757[] = {{&S56020[107], 13}, {NULL}};
const Segment SGI171758[] = {{&S63604[34], 7}, {NULL}};
const Segment SGI171759[] = {{&S63604[27], 6}, {NULL}};
const Segment SGI171760[] = {{&S63604[98], 9}, {NULL}};
const Segment SGI171761[] = {{&S63604[71], 6}, {NULL}};
const Segment SGI171762[] = {{&S63607[34], 15}, {NULL}};
const Segment SGI171763[] = {{&S63607[27], 6}, {NULL}};
const Segment SGI171764[] = {{&S63607[102], 9}, {NULL}};
const Segment SGI171765[] = {{&S63607[75], 6}, {NULL}};
const Segment SGI171766[] = {{&S63613[34], 13}, {NULL}};
const Segment SGI171767[] = {{&S63613[27], 6}, {NULL}};
const Segment SGI171768[] = {{&S63613[100], 9}, {NULL}};
const Segment SGI171769[] = {{&S63613[73], 6}, {NULL}};
const Segment SGI171770[] = {{&S63623[44], 7}, {NULL}};
const Segment SGI171771[] = {{&S63623[44], 24}, {NULL}};
const Segment SGI171772[] = {{&S63623[30], 6}, {NULL}};
const Segment SGI171773[] = {{&S63623[82], 6}, {NULL}};
const Segment SGI171774[] = {{&S63623[109], 13}, {NULL}};
const Segment SGI171775[] = {{&S63624[44], 8}, {NULL}};
const Segment SGI171776[] = {{&S63624[44], 24}, {NULL}};
const Segment SGI171777[] = {{&S63624[30], 6}, {NULL}};
const Segment SGI171778[] = {{&S63624[82], 6}, {NULL}};
const Segment SGI171779[] = {{&S63624[109], 13}, {NULL}};
const Segment SGI171780[] = {{&S63625[44], 7}, {NULL}};
const Segment SGI171781[] = {{&S63625[44], 24}, {NULL}};
const Segment SGI171782[] = {{&S63625[30], 6}, {NULL}};
const Segment SGI171783[] = {{&S63625[82], 6}, {NULL}};
const Segment SGI171784[] = {{&S63625[109], 13}, {NULL}};
const Segment SGI171785[] = {{&S63626[44], 8}, {NULL}};
const Segment SGI171786[] = {{&S63626[44], 24}, {NULL}};
const Segment SGI171787[] = {{&S63626[30], 6}, {NULL}};
const Segment SGI171788[] = {{&S63626[82], 6}, {NULL}};
const Segment SGI171789[] = {{&S63626[109], 13}, {NULL}};
const Segment SGI171790[] = {{&S63627[44], 8}, {NULL}};
const Segment SGI171791[] = {{&S63627[44], 24}, {NULL}};
const Segment SGI171792[] = {{&S63627[30], 6}, {NULL}};
const Segment SGI171793[] = {{&S63627[82], 6}, {NULL}};
const Segment SGI171794[] = {{&S63627[109], 13}, {NULL}};
const Segment SGI171795[] = {{&S63628[44], 7}, {NULL}};
const Segment SGI171796[] = {{&S63628[44], 24}, {NULL}};
const Segment SGI171797[] = {{&S63628[30], 6}, {NULL}};
const Segment SGI171798[] = {{&S63628[82], 6}, {NULL}};
const Segment SGI171799[] = {{&S63628[109], 13}, {NULL}};
const Segment SGI171800[] = {{&S52866[44], 9}, {NULL}};
const Segment SGI171801[] = {{&S52866[44], 21}, {NULL}};
const Segment SGI171802[] = {{&S52866[30], 6}, {NULL}};
const Segment SGI171803[] = {{&S52866[114], 3}, {NULL}};
const Segment SGI171804[] = {{&S52866[79], 6}, {NULL}};
const Segment SGI171805[] = {{&S52867[44], 14}, {NULL}};
const Segment SGI171806[] = {{&S52867[44], 26}, {NULL}};
const Segment SGI171807[] = {{&S52867[30], 6}, {NULL}};
const Segment SGI171808[] = {{&S52867[119], 3}, {NULL}};
const Segment SGI171809[] = {{&S52867[84], 6}, {NULL}};
const Segment SGI171810[] = {{&S52868[38], 9}, {NULL}};
const Segment SGI171811[] = {{&S52868[31], 6}, {NULL}};
const Segment SGI171812[] = {{&S52869[44], 16}, {NULL}};
const Segment SGI171813[] = {{&S52869[44], 28}, {NULL}};
const Segment SGI171814[] = {{&S52869[30], 6}, {NULL}};
const Segment SGI171815[] = {{&S52869[121], 3}, {NULL}};
const Segment SGI171816[] = {{&S52869[86], 6}, {NULL}};
const Segment SGI171817[] = {{&S52870[44], 14}, {NULL}};
const Segment SGI171818[] = {{&S52870[44], 26}, {NULL}};
const Segment SGI171819[] = {{&S52870[30], 6}, {NULL}};
const Segment SGI171820[] = {{&S52870[119], 3}, {NULL}};
const Segment SGI171821[] = {{&S52870[84], 6}, {NULL}};
const Segment SGI171822[] = {{&S56086[34], 29}, {NULL}};
const Segment SGI171823[] = {{&S56086[27], 6}, {NULL}};
const Segment SGI171824[] = {{&S56086[116], 9}, {NULL}};
const Segment SGI171825[] = {{&S56086[89], 6}, {NULL}};
const Segment SGI171826[] = {{&S56087[34], 30}, {NULL}};
const Segment SGI171827[] = {{&S56087[27], 6}, {NULL}};
const Segment SGI171828[] = {{&S56087[117], 9}, {NULL}};
const Segment SGI171829[] = {{&S56087[90], 6}, {NULL}};
const Segment SGI171830[] = {{&S56088[34], 30}, {NULL}};
const Segment SGI171831[] = {{&S56088[27], 6}, {NULL}};
const Segment SGI171832[] = {{&S56088[117], 9}, {NULL}};
const Segment SGI171833[] = {{&S56088[90], 6}, {NULL}};
const Segment SGI171834[] = {{&S56099[34], 30}, {NULL}};
const Segment SGI171835[] = {{&S56099[27], 6}, {NULL}};
const Segment SGI171836[] = {{&S56099[117], 9}, {NULL}};
const Segment SGI171837[] = {{&S56099[90], 6}, {NULL}};
const Segment SGI171838[] = {{&S56100[34], 29}, {NULL}};
const Segment SGI171839[] = {{&S56100[27], 6}, {NULL}};
const Segment SGI171840[] = {{&S56100[116], 9}, {NULL}};
const Segment SGI171841[] = {{&S56100[89], 6}, {NULL}};
const Segment SGI171842[] = {{&S56101[34], 30}, {NULL}};
const Segment SGI171843[] = {{&S56101[27], 6}, {NULL}};
const Segment SGI171844[] = {{&S56101[117], 9}, {NULL}};
const Segment SGI171845[] = {{&S56101[90], 6}, {NULL}};
const Segment SGI171846[] = {{&S56104[34], 9}, {NULL}};
const Segment SGI171847[] = {{&S56104[27], 6}, {NULL}};
const Segment SGI171848[] = {{&S56104[96], 9}, {NULL}};
const Segment SGI171849[] = {{&S56104[69], 6}, {NULL}};
const Segment SGI171850[] = {{&S63679[44], 7}, {NULL}};
const Segment SGI171851[] = {{&S63679[44], 19}, {NULL}};
const Segment SGI171852[] = {{&S63679[30], 6}, {NULL}};
const Segment SGI171853[] = {{&S63679[31], 3}, {NULL}};
const Segment SGI171854[] = {{&S63679[77], 6}, {NULL}};
const Segment SGI171855[] = {{&S56105[34], 7}, {NULL}};
const Segment SGI171856[] = {{&S56105[27], 6}, {NULL}};
const Segment SGI171857[] = {{&S56105[94], 9}, {NULL}};
const Segment SGI171858[] = {{&S56105[67], 6}, {NULL}};
const Segment SGI171859[] = {{&S56106[34], 12}, {NULL}};
const Segment SGI171860[] = {{&S56106[27], 6}, {NULL}};
const Segment SGI171861[] = {{&S56106[99], 9}, {NULL}};
const Segment SGI171862[] = {{&S56106[72], 6}, {NULL}};
const Segment SGI171863[] = {{&S52926[34], 7}, {NULL}};
const Segment SGI171864[] = {{&S52926[27], 6}, {NULL}};
const Segment SGI171865[] = {{&S52926[99], 9}, {NULL}};
const Segment SGI171866[] = {{&S52926[72], 6}, {NULL}};
const Segment SGI171867[] = {{&S63692[34], 7}, {NULL}};
const Segment SGI171868[] = {{&S63692[27], 6}, {NULL}};
const Segment SGI171869[] = {{&S63692[110], 9}, {NULL}};
const Segment SGI171870[] = {{&S63692[83], 6}, {NULL}};
const Segment SGI171871[] = {{&S63693[34], 7}, {NULL}};
const Segment SGI171872[] = {{&S63693[27], 6}, {NULL}};
const Segment SGI171873[] = {{&S63693[103], 9}, {NULL}};
const Segment SGI171874[] = {{&S63693[76], 6}, {NULL}};
const Segment SGI171875[] = {{&S63694[34], 7}, {NULL}};
const Segment SGI171876[] = {{&S63694[27], 6}, {NULL}};
const Segment SGI171877[] = {{&S63694[110], 9}, {NULL}};
const Segment SGI171878[] = {{&S63694[83], 6}, {NULL}};
const Segment SGI171879[] = {{&S63695[34], 7}, {NULL}};
const Segment SGI171880[] = {{&S63695[27], 6}, {NULL}};
const Segment SGI171881[] = {{&S63695[103], 9}, {NULL}};
const Segment SGI171882[] = {{&S63695[76], 6}, {NULL}};
const Segment SGI171883[] = {{&S63696[44], 7}, {NULL}};
const Segment SGI171884[] = {{&S63696[44], 24}, {NULL}};
const Segment SGI171885[] = {{&S63696[30], 6}, {NULL}};
const Segment SGI171886[] = {{&S63696[31], 3}, {NULL}};
const Segment SGI171887[] = {{&S63696[82], 6}, {NULL}};
const Segment SGI171888[] = {{&S63697[44], 11}, {NULL}};
const Segment SGI171889[] = {{&S63697[44], 28}, {NULL}};
const Segment SGI171890[] = {{&S63697[30], 6}, {NULL}};
const Segment SGI171891[] = {{&S56129[34], 11}, {NULL}};
const Segment SGI171892[] = {{&S63697[31], 3}, {NULL}};
const Segment SGI171893[] = {{&S56129[27], 6}, {NULL}};
const Segment SGI171894[] = {{&S63697[86], 6}, {NULL}};
const Segment SGI171895[] = {{&S56129[98], 9}, {NULL}};
const Segment SGI171896[] = {{&S56129[71], 6}, {NULL}};
const Segment SGI171897[] = {{&S63698[44], 7}, {NULL}};
const Segment SGI171898[] = {{&S63698[44], 24}, {NULL}};
const Segment SGI171899[] = {{&S63698[30], 6}, {NULL}};
const Segment SGI171900[] = {{&S63698[31], 3}, {NULL}};
const Segment SGI171901[] = {{&S63698[82], 6}, {NULL}};
const Segment SGI171902[] = {{&S56131[34], 10}, {NULL}};
const Segment SGI171903[] = {{&S56131[27], 6}, {NULL}};
const Segment SGI171904[] = {{&S56131[97], 9}, {NULL}};
const Segment SGI171905[] = {{&S56131[70], 6}, {NULL}};
const Segment SGI171906[] = {{&S63699[44], 11}, {NULL}};
const Segment SGI171907[] = {{&S63699[44], 28}, {NULL}};
const Segment SGI171908[] = {{&S63699[30], 6}, {NULL}};
const Segment SGI171909[] = {{&S63699[31], 3}, {NULL}};
const Segment SGI171910[] = {{&S63699[86], 6}, {NULL}};
const Segment SGI171911[] = {{&S56135[34], 10}, {NULL}};
const Segment SGI171912[] = {{&S56135[27], 6}, {NULL}};
const Segment SGI171913[] = {{&S56135[97], 9}, {NULL}};
const Segment SGI171914[] = {{&S56135[70], 6}, {NULL}};
const Segment SGI171915[] = {{&S56135[138], 8}, {NULL}};
const Segment SGI171916[] = {{&S63716[34], 12}, {NULL}};
const Segment SGI171917[] = {{&S63716[27], 6}, {NULL}};
const Segment SGI171918[] = {{&S63716[99], 9}, {NULL}};
const Segment SGI171919[] = {{&S63716[72], 6}, {NULL}};
const Segment SGI171920[] = {{&S63717[44], 6}, {NULL}};
const Segment SGI171921[] = {{&S63717[44], 18}, {NULL}};
const Segment SGI171922[] = {{&S63717[30], 6}, {NULL}};
const Segment SGI171923[] = {{&S63717[31], 3}, {NULL}};
const Segment SGI171924[] = {{&S63717[76], 6}, {NULL}};
const Segment SGI171925[] = {{&S63718[44], 6}, {NULL}};
const Segment SGI171926[] = {{&S63718[44], 18}, {NULL}};
const Segment SGI171927[] = {{&S63718[30], 6}, {NULL}};
const Segment SGI171928[] = {{&S63718[31], 3}, {NULL}};
const Segment SGI171929[] = {{&S63718[76], 6}, {NULL}};
const Segment SGI171930[] = {{&S56159[44], 6}, {NULL}};
const Segment SGI171931[] = {{&S56159[44], 18}, {NULL}};
const Segment SGI171932[] = {{&S63719[38], 6}, {NULL}};
const Segment SGI171933[] = {{&S56159[30], 6}, {NULL}};
const Segment SGI171934[] = {{&S63719[31], 6}, {NULL}};
const Segment SGI171935[] = {{&S56159[111], 3}, {NULL}};
const Segment SGI171936[] = {{&S56159[76], 6}, {NULL}};
const Segment SGI171937[] = {{&S63720[44], 9}, {NULL}};
const Segment SGI171938[] = {{&S56160[38], 6}, {NULL}};
const Segment SGI171939[] = {{&S63720[44], 21}, {NULL}};
const Segment SGI171940[] = {{&S56160[31], 6}, {NULL}};
const Segment SGI171941[] = {{&S63720[30], 6}, {NULL}};
const Segment SGI171942[] = {{&S63720[31], 3}, {NULL}};
const Segment SGI171943[] = {{&S63720[79], 6}, {NULL}};
const Segment SGI171944[] = {{&S63723[44], 4}, {NULL}};
const Segment SGI171945[] = {{&S63723[44], 16}, {NULL}};
const Segment SGI171946[] = {{&S63723[30], 6}, {NULL}};
const Segment SGI171947[] = {{&S63723[31], 3}, {NULL}};
const Segment SGI171948[] = {{&S63723[74], 6}, {NULL}};
const Segment SGI171949[] = {{&S63724[44], 4}, {NULL}};
const Segment SGI171950[] = {{&S63724[44], 16}, {NULL}};
const Segment SGI171951[] = {{&S63724[30], 6}, {NULL}};
const Segment SGI171952[] = {{&S63724[31], 3}, {NULL}};
const Segment SGI171953[] = {{&S63724[74], 6}, {NULL}};
const Segment SGI171954[] = {{&S63725[44], 4}, {NULL}};
const Segment SGI171955[] = {{&S63725[44], 26}, {NULL}};
const Segment SGI171956[] = {{&S63725[30], 6}, {NULL}};
const Segment SGI171957[] = {{&S63725[31], 3}, {NULL}};
const Segment SGI171958[] = {{&S63725[84], 6}, {NULL}};
const Segment SGI171959[] = {{&S41189[44], 10}, {NULL}};
const Segment SGI171960[] = {{&S41189[44], 21}, {NULL}};
const Segment SGI171961[] = {{&S41189[30], 6}, {NULL}};
const Segment SGI171962[] = {{&S41189[114], 3}, {NULL}};
const Segment SGI171963[] = {{&S41189[79], 6}, {NULL}};
const Segment SGI171964[] = {{&S41190[38], 10}, {NULL}};
const Segment SGI171965[] = {{&S41190[31], 6}, {NULL}};
const Segment SGI171966[] = {{&S63744[34], 10}, {NULL}};
const Segment SGI171967[] = {{&S63744[27], 6}, {NULL}};
const Segment SGI171968[] = {{&S63744[97], 9}, {NULL}};
const Segment SGI171969[] = {{&S63744[70], 6}, {NULL}};
const Segment SGI171970[] = {{&S63744[139], 8}, {NULL}};
const Segment SGI171971[] = {{&S63746[34], 10}, {NULL}};
const Segment SGI171972[] = {{&S63746[27], 6}, {NULL}};
const Segment SGI171973[] = {{&S63746[97], 9}, {NULL}};
const Segment SGI171974[] = {{&S63746[70], 6}, {NULL}};
const Segment SGI171975[] = {{&S63746[138], 8}, {NULL}};
const Segment SGI171976[] = {{&S63747[44], 7}, {NULL}};
const Segment SGI171977[] = {{&S63747[44], 19}, {NULL}};
const Segment SGI171978[] = {{&S63747[30], 6}, {NULL}};
const Segment SGI171979[] = {{&S63747[31], 3}, {NULL}};
const Segment SGI171980[] = {{&S63747[77], 6}, {NULL}};
const Segment SGI171981[] = {{&S63748[44], 6}, {NULL}};
const Segment SGI171982[] = {{&S63748[44], 18}, {NULL}};
const Segment SGI171983[] = {{&S63748[30], 6}, {NULL}};
const Segment SGI171984[] = {{&S63748[31], 3}, {NULL}};
const Segment SGI171985[] = {{&S63748[76], 6}, {NULL}};
const Segment SGI171986[] = {{&S63756[44], 8}, {NULL}};
const Segment SGI171987[] = {{&S63756[44], 19}, {NULL}};
const Segment SGI171988[] = {{&S63756[30], 6}, {NULL}};
const Segment SGI171989[] = {{&S63756[31], 3}, {NULL}};
const Segment SGI171990[] = {{&S63756[77], 6}, {NULL}};
const Segment SGI171991[] = {{&S63757[44], 8}, {NULL}};
const Segment SGI171992[] = {{&S63757[44], 19}, {NULL}};
const Segment SGI171993[] = {{&S63757[30], 6}, {NULL}};
const Segment SGI171994[] = {{&S63757[31], 3}, {NULL}};
const Segment SGI171995[] = {{&S63757[77], 6}, {NULL}};
const Segment SGI171996[] = {{&S63758[44], 8}, {NULL}};
const Segment SGI171997[] = {{&S63758[44], 19}, {NULL}};
const Segment SGI171998[] = {{&S63758[30], 6}, {NULL}};
const Segment SGI171999[] = {{&S63758[31], 3}, {NULL}};
const Segment SGI172000[] = {{&S63758[77], 6}, {NULL}};
const Segment SGI172001[] = {{&S56220[44], 11}, {NULL}};
const Segment SGI172002[] = {{&S56220[44], 22}, {NULL}};
const Segment SGI172003[] = {{&S56220[30], 6}, {NULL}};
const Segment SGI172004[] = {{&S56220[115], 3}, {NULL}};
const Segment SGI172005[] = {{&S56220[80], 6}, {NULL}};
const Segment SGI172006[] = {{&S56221[44], 17}, {NULL}};
const Segment SGI172007[] = {{&S56221[44], 38}, {NULL}};
const Segment SGI172008[] = {{&S56221[30], 6}, {NULL}};
const Segment SGI172009[] = {{&S56221[131], 3}, {NULL}};
const Segment SGI172010[] = {{&S56221[96], 6}, {NULL}};
const Segment SGI172011[] = {{&S56222[54], 17}, {NULL}};
const Segment SGI172012[] = {{&S56222[47], 6}, {NULL}};
const Segment SGI172013[] = {{&S56229[44], 15}, {NULL}};
const Segment SGI172014[] = {{&S56229[44], 27}, {NULL}};
const Segment SGI172015[] = {{&S56229[30], 6}, {NULL}};
const Segment SGI172016[] = {{&S56229[120], 3}, {NULL}};
const Segment SGI172017[] = {{&S56229[85], 6}, {NULL}};
const Segment SGI172018[] = {{&S56230[38], 15}, {NULL}};
const Segment SGI172019[] = {{&S56230[31], 6}, {NULL}};
const Segment SGI172020[] = {{&S63797[34], 10}, {NULL}};
const Segment SGI172021[] = {{&S63797[27], 6}, {NULL}};
const Segment SGI172022[] = {{&S63797[97], 9}, {NULL}};
const Segment SGI172023[] = {{&S63797[70], 6}, {NULL}};
const Segment SGI172024[] = {{&S63798[34], 13}, {NULL}};
const Segment SGI172025[] = {{&S63798[27], 6}, {NULL}};
const Segment SGI172026[] = {{&S63798[100], 9}, {NULL}};
const Segment SGI172027[] = {{&S63798[73], 6}, {NULL}};
const Segment SGI172028[] = {{&S63801[34], 7}, {NULL}};
const Segment SGI172029[] = {{&S63801[27], 6}, {NULL}};
const Segment SGI172030[] = {{&S63801[112], 9}, {NULL}};
const Segment SGI172031[] = {{&S63801[85], 6}, {NULL}};
const Segment SGI172032[] = {{&S41259[34], 10}, {NULL}};
const Segment SGI172033[] = {{&S41259[27], 6}, {NULL}};
const Segment SGI172034[] = {{&S41259[96], 9}, {NULL}};
const Segment SGI172035[] = {{&S41259[69], 6}, {NULL}};
const Segment SGI172036[] = {{&S41260[34], 10}, {NULL}};
const Segment SGI172037[] = {{&S41260[27], 6}, {NULL}};
const Segment SGI172038[] = {{&S41260[96], 9}, {NULL}};
const Segment SGI172039[] = {{&S41260[69], 6}, {NULL}};
const Segment SGI172040[] = {{&S41261[34], 10}, {NULL}};
const Segment SGI172041[] = {{&S41261[27], 6}, {NULL}};
const Segment SGI172042[] = {{&S41261[97], 9}, {NULL}};
const Segment SGI172043[] = {{&S41261[70], 6}, {NULL}};
const Segment SGI172044[] = {{&S41262[34], 10}, {NULL}};
const Segment SGI172045[] = {{&S41262[27], 6}, {NULL}};
const Segment SGI172046[] = {{&S63809[34], 17}, {NULL}};
const Segment SGI172047[] = {{&S63809[27], 6}, {NULL}};
const Segment SGI172048[] = {{&S41262[97], 9}, {NULL}};
const Segment SGI172049[] = {{&S41262[70], 6}, {NULL}};
const Segment SGI172050[] = {{&S63809[104], 9}, {NULL}};
const Segment SGI172051[] = {{&S63809[77], 6}, {NULL}};
const Segment SGI172052[] = {{&S63832[34], 11}, {NULL}};
const Segment SGI172053[] = {{&S63832[27], 6}, {NULL}};
const Segment SGI172054[] = {{&S63832[105], 9}, {NULL}};
const Segment SGI172055[] = {{&S63832[78], 6}, {NULL}};
const Segment SGI172056[] = {{&S63833[34], 11}, {NULL}};
const Segment SGI172057[] = {{&S63833[27], 6}, {NULL}};
const Segment SGI172058[] = {{&S63833[120], 9}, {NULL}};
const Segment SGI172059[] = {{&S63833[93], 6}, {NULL}};
const Segment SGI172060[] = {{&S63835[45], 10}, {NULL}};
const Segment SGI172061[] = {{&S63835[30], 6}, {NULL}};
const Segment SGI172062[] = {{&S63835[31], 3}, {NULL}};
const Segment SGI172063[] = {{&S63835[81], 5}, {NULL}};
const Segment SGI172064[] = {{&S63835[139], 13}, {NULL}};
const Segment SGI172065[] = {{&S63836[45], 11}, {NULL}};
const Segment SGI172066[] = {{&S63836[30], 6}, {NULL}};
const Segment SGI172067[] = {{&S63836[31], 3}, {NULL}};
const Segment SGI172068[] = {{&S63836[82], 5}, {NULL}};
const Segment SGI172069[] = {{&S63836[140], 13}, {NULL}};
const Segment SGI172070[] = {{&S41294[44], 10}, {NULL}};
const Segment SGI172071[] = {{&S41294[44], 21}, {NULL}};
const Segment SGI172072[] = {{&S41294[30], 6}, {NULL}};
const Segment SGI172073[] = {{&S41294[114], 3}, {NULL}};
const Segment SGI172074[] = {{&S41294[79], 6}, {NULL}};
const Segment SGI172075[] = {{&S41296[38], 10}, {NULL}};
const Segment SGI172076[] = {{&S41296[31], 6}, {NULL}};
const Segment SGI172077[] = {{&S63839[34], 18}, {NULL}};
const Segment SGI172078[] = {{&S63839[27], 6}, {NULL}};
const Segment SGI172079[] = {{&S63839[105], 9}, {NULL}};
const Segment SGI172080[] = {{&S63839[78], 6}, {NULL}};
const Segment SGI172081[] = {{&S41298[44], 10}, {NULL}};
const Segment SGI172082[] = {{&S41298[44], 22}, {NULL}};
const Segment SGI172083[] = {{&S41298[30], 6}, {NULL}};
const Segment SGI172084[] = {{&S41298[115], 3}, {NULL}};
const Segment SGI172085[] = {{&S41298[80], 6}, {NULL}};
const Segment SGI172086[] = {{&S63861[44], 11}, {NULL}};
const Segment SGI172087[] = {{&S63861[44], 23}, {NULL}};
const Segment SGI172088[] = {{&S63861[30], 6}, {NULL}};
const Segment SGI172089[] = {{&S63861[31], 3}, {NULL}};
const Segment SGI172090[] = {{&S63861[81], 6}, {NULL}};
const Segment SGI172091[] = {{&S52985[34], 10}, {NULL}};
const Segment SGI172092[] = {{&S52985[27], 6}, {NULL}};
const Segment SGI172093[] = {{&S52985[97], 9}, {NULL}};
const Segment SGI172094[] = {{&S52985[70], 6}, {NULL}};
const Segment SGI172095[] = {{&S63862[44], 11}, {NULL}};
const Segment SGI172096[] = {{&S63862[44], 23}, {NULL}};
const Segment SGI172097[] = {{&S63862[30], 6}, {NULL}};
const Segment SGI172098[] = {{&S63862[31], 3}, {NULL}};
const Segment SGI172099[] = {{&S63862[81], 6}, {NULL}};
const Segment SGI172100[] = {{&S52986[34], 11}, {NULL}};
const Segment SGI172101[] = {{&S52986[27], 6}, {NULL}};
const Segment SGI172102[] = {{&S52986[98], 9}, {NULL}};
const Segment SGI172103[] = {{&S52986[71], 6}, {NULL}};
const Segment SGI172104[] = {{&S63863[44], 11}, {NULL}};
const Segment SGI172105[] = {{&S63863[44], 23}, {NULL}};
const Segment SGI172106[] = {{&S63863[30], 6}, {NULL}};
const Segment SGI172107[] = {{&S63863[31], 3}, {NULL}};
const Segment SGI172108[] = {{&S52987[34], 11}, {NULL}};
const Segment SGI172109[] = {{&S63863[81], 6}, {NULL}};
const Segment SGI172110[] = {{&S52987[27], 6}, {NULL}};
const Segment SGI172111[] = {{&S52987[98], 9}, {NULL}};
const Segment SGI172112[] = {{&S52987[71], 6}, {NULL}};
const Segment SGI172113[] = {{&S63864[44], 11}, {NULL}};
const Segment SGI172114[] = {{&S63864[44], 22}, {NULL}};
const Segment SGI172115[] = {{&S63864[30], 6}, {NULL}};
const Segment SGI172116[] = {{&S52988[34], 11}, {NULL}};
const Segment SGI172117[] = {{&S63864[31], 3}, {NULL}};
const Segment SGI172118[] = {{&S52988[27], 6}, {NULL}};
const Segment SGI172119[] = {{&S63864[80], 6}, {NULL}};
const Segment SGI172120[] = {{&S52988[98], 9}, {NULL}};
const Segment SGI172121[] = {{&S52988[71], 6}, {NULL}};
const Segment SGI172122[] = {{&S63865[38], 11}, {NULL}};
const Segment SGI172123[] = {{&S63865[31], 6}, {NULL}};
const Segment SGI172124[] = {{&S52989[34], 6}, {NULL}};
const Segment SGI172125[] = {{&S52989[27], 6}, {NULL}};
const Segment SGI172126[] = {{&S52989[93], 9}, {NULL}};
const Segment SGI172127[] = {{&S52989[66], 6}, {NULL}};
const Segment SGI172128[] = {{&S63881[44], 10}, {NULL}};
const Segment SGI172129[] = {{&S63881[44], 21}, {NULL}};
const Segment SGI172130[] = {{&S63881[30], 6}, {NULL}};
const Segment SGI172131[] = {{&S63881[31], 3}, {NULL}};
const Segment SGI172132[] = {{&S63881[79], 6}, {NULL}};
const Segment SGI172133[] = {{&S63882[44], 11}, {NULL}};
const Segment SGI172134[] = {{&S63882[44], 22}, {NULL}};
const Segment SGI172135[] = {{&S63882[30], 6}, {NULL}};
const Segment SGI172136[] = {{&S63882[31], 3}, {NULL}};
const Segment SGI172137[] = {{&S63882[80], 6}, {NULL}};
const Segment SGI172138[] = {{&S63883[38], 10}, {NULL}};
const Segment SGI172139[] = {{&S63883[31], 6}, {NULL}};
const Segment SGI172140[] = {{&S63884[38], 11}, {NULL}};
const Segment SGI172141[] = {{&S63884[31], 6}, {NULL}};
const Segment SGI172142[] = {{&S63885[44], 10}, {NULL}};
const Segment SGI172143[] = {{&S63885[44], 21}, {NULL}};
const Segment SGI172144[] = {{&S63885[30], 6}, {NULL}};
const Segment SGI172145[] = {{&S63885[31], 3}, {NULL}};
const Segment SGI172146[] = {{&S63885[79], 6}, {NULL}};
const Segment SGI172147[] = {{&S63886[44], 11}, {NULL}};
const Segment SGI172148[] = {{&S63886[44], 22}, {NULL}};
const Segment SGI172149[] = {{&S63886[30], 6}, {NULL}};
const Segment SGI172150[] = {{&S63886[31], 3}, {NULL}};
const Segment SGI172151[] = {{&S63886[80], 6}, {NULL}};
const Segment SGI172152[] = {{&S63887[44], 10}, {NULL}};
const Segment SGI172153[] = {{&S63887[44], 21}, {NULL}};
const Segment SGI172154[] = {{&S63887[30], 6}, {NULL}};
const Segment SGI172155[] = {{&S63887[31], 3}, {NULL}};
const Segment SGI172156[] = {{&S63887[79], 6}, {NULL}};
const Segment SGI172157[] = {{&S63888[44], 11}, {NULL}};
const Segment SGI172158[] = {{&S63888[44], 22}, {NULL}};
const Segment SGI172159[] = {{&S63888[30], 6}, {NULL}};
const Segment SGI172160[] = {{&S63888[31], 3}, {NULL}};
const Segment SGI172161[] = {{&S63888[80], 6}, {NULL}};
const Segment SGI172162[] = {{&S41356[44], 9}, {NULL}};
const Segment SGI172163[] = {{&S41356[44], 20}, {NULL}};
const Segment SGI172164[] = {{&S41356[30], 6}, {NULL}};
const Segment SGI172165[] = {{&S41356[131], 3}, {NULL}};
const Segment SGI172166[] = {{&S41356[96], 6}, {NULL}};
const Segment SGI172167[] = {{&S41356[66], 8}, {NULL}};
const Segment SGI172168[] = {{&S41357[44], 9}, {NULL}};
const Segment SGI172169[] = {{&S41357[44], 21}, {NULL}};
const Segment SGI172170[] = {{&S41357[30], 6}, {NULL}};
const Segment SGI172171[] = {{&S41357[114], 3}, {NULL}};
const Segment SGI172172[] = {{&S41357[79], 6}, {NULL}};
const Segment SGI172173[] = {{&S41357[139], 8}, {NULL}};
const Segment SGI172174[] = {{&S63892[44], 12}, {NULL}};
const Segment SGI172175[] = {{&S63892[44], 24}, {NULL}};
const Segment SGI172176[] = {{&S63892[30], 6}, {NULL}};
const Segment SGI172177[] = {{&S63892[31], 3}, {NULL}};
const Segment SGI172178[] = {{&S63892[82], 6}, {NULL}};
const Segment SGI172179[] = {{&S41358[44], 9}, {NULL}};
const Segment SGI172180[] = {{&S41358[44], 19}, {NULL}};
const Segment SGI172181[] = {{&S41358[30], 6}, {NULL}};
const Segment SGI172182[] = {{&S41358[130], 3}, {NULL}};
const Segment SGI172183[] = {{&S41358[95], 6}, {NULL}};
const Segment SGI172184[] = {{&S41358[65], 8}, {NULL}};
const Segment SGI172185[] = {{&S41361[34], 10}, {NULL}};
const Segment SGI172186[] = {{&S41361[27], 6}, {NULL}};
const Segment SGI172187[] = {{&S41361[97], 9}, {NULL}};
const Segment SGI172188[] = {{&S41361[70], 6}, {NULL}};
const Segment SGI172189[] = {{&S41362[34], 18}, {NULL}};
const Segment SGI172190[] = {{&S41362[27], 6}, {NULL}};
const Segment SGI172191[] = {{&S41362[104], 9}, {NULL}};
const Segment SGI172192[] = {{&S41362[77], 6}, {NULL}};
const Segment SGI172193[] = {{&S63904[34], 7}, {NULL}};
const Segment SGI172194[] = {{&S63904[27], 6}, {NULL}};
const Segment SGI172195[] = {{&S63904[99], 9}, {NULL}};
const Segment SGI172196[] = {{&S63904[72], 6}, {NULL}};
const Segment SGI172197[] = {{&S63905[34], 8}, {NULL}};
const Segment SGI172198[] = {{&S63905[27], 6}, {NULL}};
const Segment SGI172199[] = {{&S63905[100], 9}, {NULL}};
const Segment SGI172200[] = {{&S63905[73], 6}, {NULL}};
const Segment SGI172201[] = {{&S63912[34], 16}, {NULL}};
const Segment SGI172202[] = {{&S63912[27], 6}, {NULL}};
const Segment SGI172203[] = {{&S63912[110], 9}, {NULL}};
const Segment SGI172204[] = {{&S63912[83], 6}, {NULL}};
const Segment SGI172205[] = {{&S63913[34], 17}, {NULL}};
const Segment SGI172206[] = {{&S63913[27], 6}, {NULL}};
const Segment SGI172207[] = {{&S63913[112], 9}, {NULL}};
const Segment SGI172208[] = {{&S63913[85], 6}, {NULL}};
const Segment SGI172209[] = {{&S63914[34], 18}, {NULL}};
const Segment SGI172210[] = {{&S63914[27], 6}, {NULL}};
const Segment SGI172211[] = {{&S63914[114], 9}, {NULL}};
const Segment SGI172212[] = {{&S63914[87], 6}, {NULL}};
const Segment SGI172213[] = {{&S63915[34], 17}, {NULL}};
const Segment SGI172214[] = {{&S63915[27], 6}, {NULL}};
const Segment SGI172215[] = {{&S63915[112], 9}, {NULL}};
const Segment SGI172216[] = {{&S63915[85], 6}, {NULL}};
const Segment SGI172217[] = {{&S41386[44], 18}, {NULL}};
const Segment SGI172218[] = {{&S41386[44], 29}, {NULL}};
const Segment SGI172219[] = {{&S41386[30], 6}, {NULL}};
const Segment SGI172220[] = {{&S41386[122], 3}, {NULL}};
const Segment SGI172221[] = {{&S41386[87], 6}, {NULL}};
const Segment SGI172222[] = {{&S53064[44], 9}, {NULL}};
const Segment SGI172223[] = {{&S53064[44], 21}, {NULL}};
const Segment SGI172224[] = {{&S53064[30], 6}, {NULL}};
const Segment SGI172225[] = {{&S53064[134], 3}, {NULL}};
const Segment SGI172226[] = {{&S53064[99], 6}, {NULL}};
const Segment SGI172227[] = {{&S53064[67], 8}, {NULL}};
const Segment SGI172228[] = {{&S41387[38], 18}, {NULL}};
const Segment SGI172229[] = {{&S41387[31], 6}, {NULL}};
const Segment SGI172230[] = {{&S53065[44], 10}, {NULL}};
const Segment SGI172231[] = {{&S53065[44], 22}, {NULL}};
const Segment SGI172232[] = {{&S53065[30], 6}, {NULL}};
const Segment SGI172233[] = {{&S41388[44], 10}, {NULL}};
const Segment SGI172234[] = {{&S41388[44], 21}, {NULL}};
const Segment SGI172235[] = {{&S53065[135], 3}, {NULL}};
const Segment SGI172236[] = {{&S41388[30], 6}, {NULL}};
const Segment SGI172237[] = {{&S53065[100], 6}, {NULL}};
const Segment SGI172238[] = {{&S53065[68], 8}, {NULL}};
const Segment SGI172239[] = {{&S41388[114], 3}, {NULL}};
const Segment SGI172240[] = {{&S41388[79], 6}, {NULL}};
const Segment SGI172241[] = {{&S41389[44], 10}, {NULL}};
const Segment SGI172242[] = {{&S41389[44], 22}, {NULL}};
const Segment SGI172243[] = {{&S41389[30], 6}, {NULL}};
const Segment SGI172244[] = {{&S41389[115], 3}, {NULL}};
const Segment SGI172245[] = {{&S53067[44], 13}, {NULL}};
const Segment SGI172246[] = {{&S41389[80], 6}, {NULL}};
const Segment SGI172247[] = {{&S53067[44], 25}, {NULL}};
const Segment SGI172248[] = {{&S53067[30], 6}, {NULL}};
const Segment SGI172249[] = {{&S53067[145], 3}, {NULL}};
const Segment SGI172250[] = {{&S53067[110], 6}, {NULL}};
const Segment SGI172251[] = {{&S53067[71], 8}, {NULL}};
const Segment SGI172252[] = {{&S41390[42], 18}, {NULL}};
const Segment SGI172253[] = {{&S41390[30], 6}, {NULL}};
const Segment SGI172254[] = {{&S41390[120], 3}, {NULL}};
const Segment SGI172255[] = {{&S41390[85], 6}, {NULL}};
const Segment SGI172256[] = {{&S41391[38], 10}, {NULL}};
const Segment SGI172257[] = {{&S41391[31], 6}, {NULL}};
const Segment SGI172258[] = {{&S53069[34], 9}, {NULL}};
const Segment SGI172259[] = {{&S53069[27], 6}, {NULL}};
const Segment SGI172260[] = {{&S53069[96], 9}, {NULL}};
const Segment SGI172261[] = {{&S53069[69], 6}, {NULL}};
const Segment SGI172262[] = {{&S63928[44], 6}, {NULL}};
const Segment SGI172263[] = {{&S63928[44], 19}, {NULL}};
const Segment SGI172264[] = {{&S63928[30], 6}, {NULL}};
const Segment SGI172265[] = {{&S63928[31], 3}, {NULL}};
const Segment SGI172266[] = {{&S53070[34], 10}, {NULL}};
const Segment SGI172267[] = {{&S53070[27], 6}, {NULL}};
const Segment SGI172268[] = {{&S63928[77], 6}, {NULL}};
const Segment SGI172269[] = {{&S53070[97], 9}, {NULL}};
const Segment SGI172270[] = {{&S53070[70], 6}, {NULL}};
const Segment SGI172271[] = {{&S63929[44], 6}, {NULL}};
const Segment SGI172272[] = {{&S63929[44], 19}, {NULL}};
const Segment SGI172273[] = {{&S63929[30], 6}, {NULL}};
const Segment SGI172274[] = {{&S63929[31], 3}, {NULL}};
const Segment SGI172275[] = {{&S63929[77], 6}, {NULL}};
const Segment SGI172276[] = {{&S54106[42], 9}, {NULL}};
const Segment SGI172277[] = {{&S54106[42], 21}, {NULL}};
const Segment SGI172278[] = {{&S54106[30], 4}, {NULL}};
const Segment SGI172279[] = {{&S54106[112], 3}, {NULL}};
const Segment SGI172280[] = {{&S54106[77], 6}, {NULL}};
const Segment SGI172281[] = {{&S54106[140], 3}, {NULL}};
const Segment SGI172282[] = {{&S63930[44], 6}, {NULL}};
const Segment SGI172283[] = {{&S63930[44], 19}, {NULL}};
const Segment SGI172284[] = {{&S63930[30], 6}, {NULL}};
const Segment SGI172285[] = {{&S63930[31], 3}, {NULL}};
const Segment SGI172286[] = {{&S63930[77], 6}, {NULL}};
const Segment SGI172287[] = {{&S63931[44], 6}, {NULL}};
const Segment SGI172288[] = {{&S63931[44], 19}, {NULL}};
const Segment SGI172289[] = {{&S63931[30], 6}, {NULL}};
const Segment SGI172290[] = {{&S63931[31], 3}, {NULL}};
const Segment SGI172291[] = {{&S63931[77], 6}, {NULL}};
const Segment SGI172292[] = {{&S63932[44], 6}, {NULL}};
const Segment SGI172293[] = {{&S63932[44], 19}, {NULL}};
const Segment SGI172294[] = {{&S63932[30], 6}, {NULL}};
const Segment SGI172295[] = {{&S63932[31], 3}, {NULL}};
const Segment SGI172296[] = {{&S63932[77], 6}, {NULL}};
const Segment SGI172297[] = {{&S63933[38], 6}, {NULL}};
const Segment SGI172298[] = {{&S63933[31], 6}, {NULL}};
const Segment SGI172299[] = {{&S63934[34], 24}, {NULL}};
const Segment SGI172300[] = {{&S63934[27], 6}, {NULL}};
const Segment SGI172301[] = {{&S63934[113], 9}, {NULL}};
const Segment SGI172302[] = {{&S63934[86], 6}, {NULL}};
const Segment SGI172303[] = {{&S63935[34], 23}, {NULL}};
const Segment SGI172304[] = {{&S63935[27], 6}, {NULL}};
const Segment SGI172305[] = {{&S63935[112], 9}, {NULL}};
const Segment SGI172306[] = {{&S63935[85], 6}, {NULL}};
const Segment SGI172307[] = {{&S63936[34], 7}, {NULL}};
const Segment SGI172308[] = {{&S63936[27], 6}, {NULL}};
const Segment SGI172309[] = {{&S63936[110], 9}, {NULL}};
const Segment SGI172310[] = {{&S63936[83], 6}, {NULL}};
const Segment SGI172311[] = {{&S63965[34], 14}, {NULL}};
const Segment SGI172312[] = {{&S63965[27], 6}, {NULL}};
const Segment SGI172313[] = {{&S63965[101], 9}, {NULL}};
const Segment SGI172314[] = {{&S63965[74], 6}, {NULL}};
const Segment SGI172315[] = {{&S53121[34], 9}, {NULL}};
const Segment SGI172316[] = {{&S53121[27], 6}, {NULL}};
const Segment SGI172317[] = {{&S53121[96], 9}, {NULL}};
const Segment SGI172318[] = {{&S53121[69], 6}, {NULL}};
const Segment SGI172319[] = {{&S53149[34], 7}, {NULL}};
const Segment SGI172320[] = {{&S53149[27], 6}, {NULL}};
const Segment SGI172321[] = {{&S53149[110], 9}, {NULL}};
const Segment SGI172322[] = {{&S53149[83], 6}, {NULL}};
const Segment SGI172323[] = {{&S64000[34], 14}, {NULL}};
const Segment SGI172324[] = {{&S64000[27], 6}, {NULL}};
const Segment SGI172325[] = {{&S64000[106], 9}, {NULL}};
const Segment SGI172326[] = {{&S64000[79], 6}, {NULL}};
const Segment SGI172327[] = {{&S64001[34], 9}, {NULL}};
const Segment SGI172328[] = {{&S64001[27], 6}, {NULL}};
const Segment SGI172329[] = {{&S64001[96], 9}, {NULL}};
const Segment SGI172330[] = {{&S64001[69], 6}, {NULL}};
const Segment SGI172331[] = {{&S53161[44], 10}, {NULL}};
const Segment SGI172332[] = {{&S53161[44], 22}, {NULL}};
const Segment SGI172333[] = {{&S53161[30], 6}, {NULL}};
const Segment SGI172334[] = {{&S53161[115], 3}, {NULL}};
const Segment SGI172335[] = {{&S53161[80], 6}, {NULL}};
const Segment SGI172336[] = {{&S53161[140], 7}, {NULL}};
const Segment SGI172337[] = {{&S53162[44], 10}, {NULL}};
const Segment SGI172338[] = {{&S53162[44], 22}, {NULL}};
const Segment SGI172339[] = {{&S53162[30], 6}, {NULL}};
const Segment SGI172340[] = {{&S53162[115], 3}, {NULL}};
const Segment SGI172341[] = {{&S53162[80], 6}, {NULL}};
const Segment SGI172342[] = {{&S53162[140], 7}, {NULL}};
const Segment SGI172343[] = {{&S53163[44], 10}, {NULL}};
const Segment SGI172344[] = {{&S53163[44], 22}, {NULL}};
const Segment SGI172345[] = {{&S53163[30], 6}, {NULL}};
const Segment SGI172346[] = {{&S53163[115], 3}, {NULL}};
const Segment SGI172347[] = {{&S53163[80], 6}, {NULL}};
const Segment SGI172348[] = {{&S53163[140], 7}, {NULL}};
const Segment SGI172349[] = {{&S53165[44], 11}, {NULL}};
const Segment SGI172350[] = {{&S53165[44], 23}, {NULL}};
const Segment SGI172351[] = {{&S53165[30], 6}, {NULL}};
const Segment SGI172352[] = {{&S53165[116], 3}, {NULL}};
const Segment SGI172353[] = {{&S53165[81], 6}, {NULL}};
const Segment SGI172354[] = {{&S53165[141], 7}, {NULL}};
const Segment SGI172355[] = {{&S53166[44], 11}, {NULL}};
const Segment SGI172356[] = {{&S53166[44], 23}, {NULL}};
const Segment SGI172357[] = {{&S53166[30], 6}, {NULL}};
const Segment SGI172358[] = {{&S53166[116], 3}, {NULL}};
const Segment SGI172359[] = {{&S53166[81], 6}, {NULL}};
const Segment SGI172360[] = {{&S53166[141], 7}, {NULL}};
const Segment SGI172361[] = {{&S53168[34], 9}, {NULL}};
const Segment SGI172362[] = {{&S53168[27], 6}, {NULL}};
const Segment SGI172363[] = {{&S53168[96], 9}, {NULL}};
const Segment SGI172364[] = {{&S53168[69], 6}, {NULL}};
const Segment SGI172365[] = {{&S50502[44], 10}, {NULL}};
const Segment SGI172366[] = {{&S50502[44], 22}, {NULL}};
const Segment SGI172367[] = {{&S50502[30], 6}, {NULL}};
const Segment SGI172368[] = {{&S50502[133], 3}, {NULL}};
const Segment SGI172369[] = {{&S50502[98], 6}, {NULL}};
const Segment SGI172370[] = {{&S50502[68], 8}, {NULL}};
const Segment SGI172371[] = {{&S50503[44], 10}, {NULL}};
const Segment SGI172372[] = {{&S50503[44], 22}, {NULL}};
const Segment SGI172373[] = {{&S50503[30], 6}, {NULL}};
const Segment SGI172374[] = {{&S50503[133], 3}, {NULL}};
const Segment SGI172375[] = {{&S50503[98], 6}, {NULL}};
const Segment SGI172376[] = {{&S50503[68], 8}, {NULL}};
const Segment SGI172377[] = {{&S50514[44], 10}, {NULL}};
const Segment SGI172378[] = {{&S50514[44], 22}, {NULL}};
const Segment SGI172379[] = {{&S50514[30], 6}, {NULL}};
const Segment SGI172380[] = {{&S50514[115], 3}, {NULL}};
const Segment SGI172381[] = {{&S50514[80], 6}, {NULL}};
const Segment SGI172382[] = {{&S50515[44], 10}, {NULL}};
const Segment SGI172383[] = {{&S50515[44], 22}, {NULL}};
const Segment SGI172384[] = {{&S50515[30], 6}, {NULL}};
const Segment SGI172385[] = {{&S50515[115], 3}, {NULL}};
const Segment SGI172386[] = {{&S50515[80], 6}, {NULL}};
const Segment SGI172387[] = {{&S50517[44], 10}, {NULL}};
const Segment SGI172388[] = {{&S50517[44], 22}, {NULL}};
const Segment SGI172389[] = {{&S50517[30], 6}, {NULL}};
const Segment SGI172390[] = {{&S50517[115], 3}, {NULL}};
const Segment SGI172391[] = {{&S50517[80], 6}, {NULL}};
const Segment SGI172392[] = {{&S64057[34], 7}, {NULL}};
const Segment SGI172393[] = {{&S64057[27], 6}, {NULL}};
const Segment SGI172394[] = {{&S64057[105], 9}, {NULL}};
const Segment SGI172395[] = {{&S64057[78], 6}, {NULL}};
const Segment SGI172396[] = {{&S64058[34], 15}, {NULL}};
const Segment SGI172397[] = {{&S64058[27], 6}, {NULL}};
const Segment SGI172398[] = {{&S64058[102], 9}, {NULL}};
const Segment SGI172399[] = {{&S64058[75], 6}, {NULL}};
const Segment SGI172400[] = {{&S50524[44], 10}, {NULL}};
const Segment SGI172401[] = {{&S50524[44], 22}, {NULL}};
const Segment SGI172402[] = {{&S64059[34], 14}, {NULL}};
const Segment SGI172403[] = {{&S50524[30], 6}, {NULL}};
const Segment SGI172404[] = {{&S64059[27], 6}, {NULL}};
const Segment SGI172405[] = {{&S50524[115], 3}, {NULL}};
const Segment SGI172406[] = {{&S50524[80], 6}, {NULL}};
const Segment SGI172407[] = {{&S64059[101], 9}, {NULL}};
const Segment SGI172408[] = {{&S64059[74], 6}, {NULL}};
const Segment SGI172409[] = {{&S50527[44], 10}, {NULL}};
const Segment SGI172410[] = {{&S50527[44], 22}, {NULL}};
const Segment SGI172411[] = {{&S50527[30], 6}, {NULL}};
const Segment SGI172412[] = {{&S50527[115], 3}, {NULL}};
const Segment SGI172413[] = {{&S50527[80], 6}, {NULL}};
const Segment SGI172414[] = {{&S64063[44], 13}, {NULL}};
const Segment SGI172415[] = {{&S64063[44], 25}, {NULL}};
const Segment SGI172416[] = {{&S64063[30], 6}, {NULL}};
const Segment SGI172417[] = {{&S50528[44], 8}, {NULL}};
const Segment SGI172418[] = {{&S50528[44], 20}, {NULL}};
const Segment SGI172419[] = {{&S50528[30], 6}, {NULL}};
const Segment SGI172420[] = {{&S64063[31], 3}, {NULL}};
const Segment SGI172421[] = {{&S64063[83], 6}, {NULL}};
const Segment SGI172422[] = {{&S50528[113], 3}, {NULL}};
const Segment SGI172423[] = {{&S50528[78], 6}, {NULL}};
const Segment SGI172424[] = {{&S64064[44], 13}, {NULL}};
const Segment SGI172425[] = {{&S64064[44], 25}, {NULL}};
const Segment SGI172426[] = {{&S50529[44], 10}, {NULL}};
const Segment SGI172427[] = {{&S64064[30], 6}, {NULL}};
const Segment SGI172428[] = {{&S50529[44], 22}, {NULL}};
const Segment SGI172429[] = {{&S50529[30], 6}, {NULL}};
const Segment SGI172430[] = {{&S64064[31], 3}, {NULL}};
const Segment SGI172431[] = {{&S50529[115], 3}, {NULL}};
const Segment SGI172432[] = {{&S64064[83], 6}, {NULL}};
const Segment SGI172433[] = {{&S50529[80], 6}, {NULL}};
const Segment SGI172434[] = {{&S64065[44], 13}, {NULL}};
const Segment SGI172435[] = {{&S64065[44], 25}, {NULL}};
const Segment SGI172436[] = {{&S64065[30], 6}, {NULL}};
const Segment SGI172437[] = {{&S64065[31], 3}, {NULL}};
const Segment SGI172438[] = {{&S50531[44], 7}, {NULL}};
const Segment SGI172439[] = {{&S64065[83], 6}, {NULL}};
const Segment SGI172440[] = {{&S50531[44], 19}, {NULL}};
const Segment SGI172441[] = {{&S50531[30], 6}, {NULL}};
const Segment SGI172442[] = {{&S50531[112], 3}, {NULL}};
const Segment SGI172443[] = {{&S50531[77], 6}, {NULL}};
const Segment SGI172444[] = {{&S64066[44], 13}, {NULL}};
const Segment SGI172445[] = {{&S64066[44], 25}, {NULL}};
const Segment SGI172446[] = {{&S64066[30], 6}, {NULL}};
const Segment SGI172447[] = {{&S64066[31], 3}, {NULL}};
const Segment SGI172448[] = {{&S64066[83], 6}, {NULL}};
const Segment SGI172449[] = {{&S64067[34], 8}, {NULL}};
const Segment SGI172450[] = {{&S64067[27], 6}, {NULL}};
const Segment SGI172451[] = {{&S64067[99], 9}, {NULL}};
const Segment SGI172452[] = {{&S64067[72], 6}, {NULL}};
const Segment SGI172453[] = {{&S64068[34], 18}, {NULL}};
const Segment SGI172454[] = {{&S64068[27], 6}, {NULL}};
const Segment SGI172455[] = {{&S64068[105], 9}, {NULL}};
const Segment SGI172456[] = {{&S64068[78], 6}, {NULL}};
const Segment SGI172457[] = {{&S64070[34], 10}, {NULL}};
const Segment SGI172458[] = {{&S64070[27], 6}, {NULL}};
const Segment SGI172459[] = {{&S64070[97], 9}, {NULL}};
const Segment SGI172460[] = {{&S64070[70], 6}, {NULL}};
const Segment SGI172461[] = {{&S64074[34], 17}, {NULL}};
const Segment SGI172462[] = {{&S64074[27], 6}, {NULL}};
const Segment SGI172463[] = {{&S64074[104], 9}, {NULL}};
const Segment SGI172464[] = {{&S64074[77], 6}, {NULL}};
const Segment SGI172465[] = {{&S64077[34], 10}, {NULL}};
const Segment SGI172466[] = {{&S64077[27], 6}, {NULL}};
const Segment SGI172467[] = {{&S64077[97], 9}, {NULL}};
const Segment SGI172468[] = {{&S64077[70], 6}, {NULL}};
const Segment SGI172469[] = {{&S64088[34], 7}, {NULL}};
const Segment SGI172470[] = {{&S64088[27], 6}, {NULL}};
const Segment SGI172471[] = {{&S64088[96], 9}, {NULL}};
const Segment SGI172472[] = {{&S64088[69], 6}, {NULL}};
const Segment SGI172473[] = {{&S64110[30], 6}, {NULL}};
const Segment SGI172474[] = {{&S64110[31], 3}, {NULL}};
const Segment SGI172475[] = {{&S64110[81], 6}, {NULL}};
const Segment SGI172476[] = {{&S64111[44], 13}, {NULL}};
const Segment SGI172477[] = {{&S64111[44], 25}, {NULL}};
const Segment SGI172478[] = {{&S64111[30], 6}, {NULL}};
const Segment SGI172479[] = {{&S64111[31], 3}, {NULL}};
const Segment SGI172480[] = {{&S64111[83], 6}, {NULL}};
const Segment SGI172481[] = {{&S64112[38], 13}, {NULL}};
const Segment SGI172482[] = {{&S64112[31], 6}, {NULL}};
const Segment SGI172483[] = {{&S50594[34], 10}, {NULL}};
const Segment SGI172484[] = {{&S50594[27], 6}, {NULL}};
const Segment SGI172485[] = {{&S50594[97], 9}, {NULL}};
const Segment SGI172486[] = {{&S50594[70], 6}, {NULL}};
const Segment SGI172487[] = {{&S50596[34], 11}, {NULL}};
const Segment SGI172488[] = {{&S50596[27], 6}, {NULL}};
const Segment SGI172489[] = {{&S64116[44], 23}, {NULL}};
const Segment SGI172490[] = {{&S64116[44], 35}, {NULL}};
const Segment SGI172491[] = {{&S64116[30], 6}, {NULL}};
const Segment SGI172492[] = {{&S50596[98], 9}, {NULL}};
const Segment SGI172493[] = {{&S50596[71], 6}, {NULL}};
const Segment SGI172494[] = {{&S64116[31], 3}, {NULL}};
const Segment SGI172495[] = {{&S64116[93], 6}, {NULL}};
const Segment SGI172496[] = {{&S64122[34], 15}, {NULL}};
const Segment SGI172497[] = {{&S41620[34], 18}, {NULL}};
const Segment SGI172498[] = {{&S64122[27], 6}, {NULL}};
const Segment SGI172499[] = {{&S41620[27], 6}, {NULL}};
const Segment SGI172500[] = {{&S64122[102], 9}, {NULL}};
const Segment SGI172501[] = {{&S64122[75], 6}, {NULL}};
const Segment SGI172502[] = {{&S41620[104], 9}, {NULL}};
const Segment SGI172503[] = {{&S41620[77], 6}, {NULL}};
const Segment SGI172504[] = {{&S41621[34], 18}, {NULL}};
const Segment SGI172505[] = {{&S41621[27], 6}, {NULL}};
const Segment SGI172506[] = {{&S41621[104], 9}, {NULL}};
const Segment SGI172507[] = {{&S41621[77], 6}, {NULL}};
const Segment SGI172508[] = {{&S64175[34], 12}, {NULL}};
const Segment SGI172509[] = {{&S64175[27], 6}, {NULL}};
const Segment SGI172510[] = {{&S64175[115], 9}, {NULL}};
const Segment SGI172511[] = {{&S64175[88], 6}, {NULL}};
const Segment SGI172512[] = {{&S41678[34], 10}, {NULL}};
const Segment SGI172513[] = {{&S41678[27], 6}, {NULL}};
const Segment SGI172514[] = {{&S41678[96], 9}, {NULL}};
const Segment SGI172515[] = {{&S41678[69], 6}, {NULL}};
const Segment SGI172516[] = {{&S41679[34], 10}, {NULL}};
const Segment SGI172517[] = {{&S41679[27], 6}, {NULL}};
const Segment SGI172518[] = {{&S41679[96], 9}, {NULL}};
const Segment SGI172519[] = {{&S41679[69], 6}, {NULL}};
const Segment SGI172520[] = {{&S64180[34], 11}, {NULL}};
const Segment SGI172521[] = {{&S64180[27], 6}, {NULL}};
const Segment SGI172522[] = {{&S64180[98], 9}, {NULL}};
const Segment SGI172523[] = {{&S64180[71], 6}, {NULL}};
const Segment SGI172524[] = {{&S64182[44], 13}, {NULL}};
const Segment SGI172525[] = {{&S64182[44], 25}, {NULL}};
const Segment SGI172526[] = {{&S64182[30], 6}, {NULL}};
const Segment SGI172527[] = {{&S64182[31], 3}, {NULL}};
const Segment SGI172528[] = {{&S64182[83], 6}, {NULL}};
const Segment SGI172529[] = {{&S53215[44], 12}, {NULL}};
const Segment SGI172530[] = {{&S53215[44], 24}, {NULL}};
const Segment SGI172531[] = {{&S53215[30], 6}, {NULL}};
const Segment SGI172532[] = {{&S53215[117], 3}, {NULL}};
const Segment SGI172533[] = {{&S53215[82], 6}, {NULL}};
const Segment SGI172534[] = {{&S53218[38], 12}, {NULL}};
const Segment SGI172535[] = {{&S53218[31], 6}, {NULL}};
const Segment SGI172536[] = {{&S53221[44], 25}, {NULL}};
const Segment SGI172537[] = {{&S53221[44], 37}, {NULL}};
const Segment SGI172538[] = {{&S53221[30], 6}, {NULL}};
const Segment SGI172539[] = {{&S53221[130], 3}, {NULL}};
const Segment SGI172540[] = {{&S53221[95], 6}, {NULL}};
const Segment SGI172541[] = {{&S53222[44], 16}, {NULL}};
const Segment SGI172542[] = {{&S53222[44], 28}, {NULL}};
const Segment SGI172543[] = {{&S53222[30], 6}, {NULL}};
const Segment SGI172544[] = {{&S53222[121], 3}, {NULL}};
const Segment SGI172545[] = {{&S53222[86], 6}, {NULL}};
const Segment SGI172546[] = {{&S64206[34], 7}, {NULL}};
const Segment SGI172547[] = {{&S64206[27], 6}, {NULL}};
const Segment SGI172548[] = {{&S53223[44], 16}, {NULL}};
const Segment SGI172549[] = {{&S53223[44], 28}, {NULL}};
const Segment SGI172550[] = {{&S53223[30], 6}, {NULL}};
const Segment SGI172551[] = {{&S64206[110], 9}, {NULL}};
const Segment SGI172552[] = {{&S64206[83], 6}, {NULL}};
const Segment SGI172553[] = {{&S53223[121], 3}, {NULL}};
const Segment SGI172554[] = {{&S53223[86], 6}, {NULL}};
const Segment SGI172555[] = {{&S53232[34], 9}, {NULL}};
const Segment SGI172556[] = {{&S53232[27], 6}, {NULL}};
const Segment SGI172557[] = {{&S53232[96], 9}, {NULL}};
const Segment SGI172558[] = {{&S53232[69], 6}, {NULL}};
const Segment SGI172559[] = {{&S64234[44], 9}, {NULL}};
const Segment SGI172560[] = {{&S64234[44], 21}, {NULL}};
const Segment SGI172561[] = {{&S64234[30], 6}, {NULL}};
const Segment SGI172562[] = {{&S64234[31], 3}, {NULL}};
const Segment SGI172563[] = {{&S64234[79], 6}, {NULL}};
const Segment SGI172564[] = {{&S64235[34], 10}, {NULL}};
const Segment SGI172565[] = {{&S64235[27], 6}, {NULL}};
const Segment SGI172566[] = {{&S64235[97], 9}, {NULL}};
const Segment SGI172567[] = {{&S64235[70], 6}, {NULL}};
const Segment SGI172568[] = {{&S50725[44], 10}, {NULL}};
const Segment SGI172569[] = {{&S50725[44], 22}, {NULL}};
const Segment SGI172570[] = {{&S50725[30], 6}, {NULL}};
const Segment SGI172571[] = {{&S50725[115], 3}, {NULL}};
const Segment SGI172572[] = {{&S50725[80], 6}, {NULL}};
const Segment SGI172573[] = {{&S50726[44], 10}, {NULL}};
const Segment SGI172574[] = {{&S50726[44], 22}, {NULL}};
const Segment SGI172575[] = {{&S50726[30], 6}, {NULL}};
const Segment SGI172576[] = {{&S50726[115], 3}, {NULL}};
const Segment SGI172577[] = {{&S50726[80], 6}, {NULL}};
const Segment SGI172578[] = {{&S50729[44], 10}, {NULL}};
const Segment SGI172579[] = {{&S50729[44], 22}, {NULL}};
const Segment SGI172580[] = {{&S50729[30], 6}, {NULL}};
const Segment SGI172581[] = {{&S50729[114], 3}, {NULL}};
const Segment SGI172582[] = {{&S50729[80], 6}, {NULL}};
const Segment SGI172583[] = {{&S64245[34], 18}, {NULL}};
const Segment SGI172584[] = {{&S64245[27], 6}, {NULL}};
const Segment SGI172585[] = {{&S64245[105], 9}, {NULL}};
const Segment SGI172586[] = {{&S64245[78], 6}, {NULL}};
const Segment SGI172587[] = {{&S53289[34], 10}, {NULL}};
const Segment SGI172588[] = {{&S53289[27], 6}, {NULL}};
const Segment SGI172589[] = {{&S53289[97], 9}, {NULL}};
const Segment SGI172590[] = {{&S53289[70], 6}, {NULL}};
const Segment SGI172591[] = {{&S53298[34], 10}, {NULL}};
const Segment SGI172592[] = {{&S53298[27], 6}, {NULL}};
const Segment SGI172593[] = {{&S53298[97], 9}, {NULL}};
const Segment SGI172594[] = {{&S53298[70], 6}, {NULL}};
const Segment SGI172595[] = {{&S64285[34], 10}, {NULL}};
const Segment SGI172596[] = {{&S64285[27], 6}, {NULL}};
const Segment SGI172597[] = {{&S53299[34], 10}, {NULL}};
const Segment SGI172598[] = {{&S64285[97], 9}, {NULL}};
const Segment SGI172599[] = {{&S64285[70], 6}, {NULL}};
const Segment SGI172600[] = {{&S53299[27], 6}, {NULL}};
const Segment SGI172601[] = {{&S53299[97], 9}, {NULL}};
const Segment SGI172602[] = {{&S53299[70], 6}, {NULL}};
const Segment SGI172603[] = {{&S53300[34], 11}, {NULL}};
const Segment SGI172604[] = {{&S53300[27], 6}, {NULL}};
const Segment SGI172605[] = {{&S53300[98], 9}, {NULL}};
const Segment SGI172606[] = {{&S53300[71], 6}, {NULL}};
const Segment SGI172607[] = {{&S41789[34], 10}, {NULL}};
const Segment SGI172608[] = {{&S41789[27], 6}, {NULL}};
const Segment SGI172609[] = {{&S41789[96], 9}, {NULL}};
const Segment SGI172610[] = {{&S41789[69], 6}, {NULL}};
const Segment SGI172611[] = {{&S53301[34], 11}, {NULL}};
const Segment SGI172612[] = {{&S53301[27], 6}, {NULL}};
const Segment SGI172613[] = {{&S53301[98], 9}, {NULL}};
const Segment SGI172614[] = {{&S53301[71], 6}, {NULL}};
const Segment SGI172615[] = {{&S41790[34], 10}, {NULL}};
const Segment SGI172616[] = {{&S41790[27], 6}, {NULL}};
const Segment SGI172617[] = {{&S41790[97], 9}, {NULL}};
const Segment SGI172618[] = {{&S41790[70], 6}, {NULL}};
const Segment SGI172619[] = {{&S41791[34], 10}, {NULL}};
const Segment SGI172620[] = {{&S41791[27], 6}, {NULL}};
const Segment SGI172621[] = {{&S41791[97], 9}, {NULL}};
const Segment SGI172622[] = {{&S41791[70], 6}, {NULL}};
const Segment SGI172623[] = {{&S64294[44], 21}, {NULL}};
const Segment SGI172624[] = {{&S64294[44], 33}, {NULL}};
const Segment SGI172625[] = {{&S64294[30], 6}, {NULL}};
const Segment SGI172626[] = {{&S64294[31], 3}, {NULL}};
const Segment SGI172627[] = {{&S64294[91], 6}, {NULL}};
const Segment SGI172628[] = {{&S41793[34], 10}, {NULL}};
const Segment SGI172629[] = {{&S41793[27], 6}, {NULL}};
const Segment SGI172630[] = {{&S41793[96], 9}, {NULL}};
const Segment SGI172631[] = {{&S41793[69], 6}, {NULL}};
const Segment SGI172632[] = {{&S64295[44], 22}, {NULL}};
const Segment SGI172633[] = {{&S64295[44], 34}, {NULL}};
const Segment SGI172634[] = {{&S64295[30], 6}, {NULL}};
const Segment SGI172635[] = {{&S64295[31], 3}, {NULL}};
const Segment SGI172636[] = {{&S64295[92], 6}, {NULL}};
const Segment SGI172637[] = {{&S64296[38], 21}, {NULL}};
const Segment SGI172638[] = {{&S64296[31], 6}, {NULL}};
const Segment SGI172639[] = {{&S41796[34], 8}, {NULL}};
const Segment SGI172640[] = {{&S41796[27], 6}, {NULL}};
const Segment SGI172641[] = {{&S41796[94], 9}, {NULL}};
const Segment SGI172642[] = {{&S41796[67], 6}, {NULL}};
const Segment SGI172643[] = {{&S64307[34], 15}, {NULL}};
const Segment SGI172644[] = {{&S64307[27], 6}, {NULL}};
const Segment SGI172645[] = {{&S64307[102], 9}, {NULL}};
const Segment SGI172646[] = {{&S64307[75], 6}, {NULL}};
const Segment SGI172647[] = {{&S61910[44], 7}, {NULL}};
const Segment SGI172648[] = {{&S61910[44], 19}, {NULL}};
const Segment SGI172649[] = {{&S61910[30], 6}, {NULL}};
const Segment SGI172650[] = {{&S61910[31], 3}, {NULL}};
const Segment SGI172651[] = {{&S61910[77], 6}, {NULL}};
const Segment SGI172652[] = {{&S61912[38], 7}, {NULL}};
const Segment SGI172653[] = {{&S61912[31], 6}, {NULL}};
const Segment SGI172654[] = {{&S53328[44], 12}, {NULL}};
const Segment SGI172655[] = {{&S53328[44], 24}, {NULL}};
const Segment SGI172656[] = {{&S53328[30], 6}, {NULL}};
const Segment SGI172657[] = {{&S53328[117], 3}, {NULL}};
const Segment SGI172658[] = {{&S53328[82], 6}, {NULL}};
const Segment SGI172659[] = {{&S53329[34], 9}, {NULL}};
const Segment SGI172660[] = {{&S53329[27], 6}, {NULL}};
const Segment SGI172661[] = {{&S53329[96], 9}, {NULL}};
const Segment SGI172662[] = {{&S53329[69], 6}, {NULL}};
const Segment SGI172663[] = {{&S53330[34], 10}, {NULL}};
const Segment SGI172664[] = {{&S53330[27], 6}, {NULL}};
const Segment SGI172665[] = {{&S53330[97], 9}, {NULL}};
const Segment SGI172666[] = {{&S53330[70], 6}, {NULL}};
const Segment SGI172667[] = {{&S53331[34], 10}, {NULL}};
const Segment SGI172668[] = {{&S53331[27], 6}, {NULL}};
const Segment SGI172669[] = {{&S53331[97], 9}, {NULL}};
const Segment SGI172670[] = {{&S53331[70], 6}, {NULL}};
const Segment SGI172671[] = {{&S64319[44], 5}, {NULL}};
const Segment SGI172672[] = {{&S64319[44], 17}, {NULL}};
const Segment SGI172673[] = {{&S64319[30], 6}, {NULL}};
const Segment SGI172674[] = {{&S64319[31], 3}, {NULL}};
const Segment SGI172675[] = {{&S64319[75], 6}, {NULL}};
const Segment SGI172676[] = {{&S64335[44], 4}, {NULL}};
const Segment SGI172677[] = {{&S64335[44], 16}, {NULL}};
const Segment SGI172678[] = {{&S64335[30], 6}, {NULL}};
const Segment SGI172679[] = {{&S64335[31], 3}, {NULL}};
const Segment SGI172680[] = {{&S64335[74], 6}, {NULL}};
const Segment SGI172681[] = {{&S64336[38], 4}, {NULL}};
const Segment SGI172682[] = {{&S64336[31], 6}, {NULL}};
const Segment SGI172683[] = {{&S64337[44], 9}, {NULL}};
const Segment SGI172684[] = {{&S64337[44], 21}, {NULL}};
const Segment SGI172685[] = {{&S64337[30], 6}, {NULL}};
const Segment SGI172686[] = {{&S64337[31], 3}, {NULL}};
const Segment SGI172687[] = {{&S64337[79], 6}, {NULL}};
const Segment SGI172688[] = {{&S53363[34], 15}, {NULL}};
const Segment SGI172689[] = {{&S53363[27], 6}, {NULL}};
const Segment SGI172690[] = {{&S53363[102], 9}, {NULL}};
const Segment SGI172691[] = {{&S53363[75], 6}, {NULL}};
const Segment SGI172692[] = {{&S53365[34], 16}, {NULL}};
const Segment SGI172693[] = {{&S64350[34], 10}, {NULL}};
const Segment SGI172694[] = {{&S53365[27], 6}, {NULL}};
const Segment SGI172695[] = {{&S64350[27], 6}, {NULL}};
const Segment SGI172696[] = {{&S53365[103], 9}, {NULL}};
const Segment SGI172697[] = {{&S53365[76], 6}, {NULL}};
const Segment SGI172698[] = {{&S64350[97], 9}, {NULL}};
const Segment SGI172699[] = {{&S64350[70], 6}, {NULL}};
const Segment SGI172700[] = {{&S64351[34], 13}, {NULL}};
const Segment SGI172701[] = {{&S64351[27], 6}, {NULL}};
const Segment SGI172702[] = {{&S64351[104], 9}, {NULL}};
const Segment SGI172703[] = {{&S64351[77], 6}, {NULL}};
const Segment SGI172704[] = {{&S64352[34], 13}, {NULL}};
const Segment SGI172705[] = {{&S64352[27], 6}, {NULL}};
const Segment SGI172706[] = {{&S64352[104], 9}, {NULL}};
const Segment SGI172707[] = {{&S64352[77], 6}, {NULL}};
const Segment SGI172708[] = {{&S41861[34], 8}, {NULL}};
const Segment SGI172709[] = {{&S41861[27], 6}, {NULL}};
const Segment SGI172710[] = {{&S41861[94], 9}, {NULL}};
const Segment SGI172711[] = {{&S41861[67], 6}, {NULL}};
const Segment SGI172712[] = {{&S53375[34], 20}, {NULL}};
const Segment SGI172713[] = {{&S53375[27], 6}, {NULL}};
const Segment SGI172714[] = {{&S53375[116], 9}, {NULL}};
const Segment SGI172715[] = {{&S53375[89], 6}, {NULL}};
const Segment SGI172716[] = {{&S53379[34], 19}, {NULL}};
const Segment SGI172717[] = {{&S53379[27], 6}, {NULL}};
const Segment SGI172718[] = {{&S53379[106], 9}, {NULL}};
const Segment SGI172719[] = {{&S53379[79], 6}, {NULL}};
const Segment SGI172720[] = {{&S53380[34], 20}, {NULL}};
const Segment SGI172721[] = {{&S53380[27], 6}, {NULL}};
const Segment SGI172722[] = {{&S53380[107], 9}, {NULL}};
const Segment SGI172723[] = {{&S53380[80], 6}, {NULL}};
const Segment SGI172724[] = {{&S64365[44], 12}, {NULL}};
const Segment SGI172725[] = {{&S64365[44], 24}, {NULL}};
const Segment SGI172726[] = {{&S64365[30], 6}, {NULL}};
const Segment SGI172727[] = {{&S64365[31], 3}, {NULL}};
const Segment SGI172728[] = {{&S64365[82], 6}, {NULL}};
const Segment SGI172729[] = {{&S64366[52], 6}, {NULL}};
const Segment SGI172730[] = {{&S64366[114], 6}, {NULL}};
const Segment SGI172731[] = {{&S53384[34], 7}, {NULL}};
const Segment SGI172732[] = {{&S53384[27], 6}, {NULL}};
const Segment SGI172733[] = {{&S53384[98], 9}, {NULL}};
const Segment SGI172734[] = {{&S53384[71], 6}, {NULL}};
const Segment SGI172735[] = {{&S64367[44], 14}, {NULL}};
const Segment SGI172736[] = {{&S64367[44], 31}, {NULL}};
const Segment SGI172737[] = {{&S64367[30], 6}, {NULL}};
const Segment SGI172738[] = {{&S64367[92], 6}, {NULL}};
const Segment SGI172739[] = {{&S64368[44], 14}, {NULL}};
const Segment SGI172740[] = {{&S64368[44], 31}, {NULL}};
const Segment SGI172741[] = {{&S64368[30], 6}, {NULL}};
const Segment SGI172742[] = {{&S64368[31], 3}, {NULL}};
const Segment SGI172743[] = {{&S64368[89], 6}, {NULL}};
const Segment SGI172744[] = {{&S64369[38], 14}, {NULL}};
const Segment SGI172745[] = {{&S64369[31], 6}, {NULL}};
const Segment SGI172746[] = {{&S56710[34], 6}, {NULL}};
const Segment SGI172747[] = {{&S56710[27], 6}, {NULL}};
const Segment SGI172748[] = {{&S64376[34], 10}, {NULL}};
const Segment SGI172749[] = {{&S64376[27], 6}, {NULL}};
const Segment SGI172750[] = {{&S56710[93], 9}, {NULL}};
const Segment SGI172751[] = {{&S56710[66], 6}, {NULL}};
const Segment SGI172752[] = {{&S64376[97], 9}, {NULL}};
const Segment SGI172753[] = {{&S64376[70], 6}, {NULL}};
const Segment SGI172754[] = {{&S56722[34], 10}, {NULL}};
const Segment SGI172755[] = {{&S56722[27], 6}, {NULL}};
const Segment SGI172756[] = {{&S56722[97], 9}, {NULL}};
const Segment SGI172757[] = {{&S56722[70], 6}, {NULL}};
const Segment SGI172758[] = {{&S64419[44], 10}, {NULL}};
const Segment SGI172759[] = {{&S64419[44], 22}, {NULL}};
const Segment SGI172760[] = {{&S64419[30], 6}, {NULL}};
const Segment SGI172761[] = {{&S64419[139], 4}, {NULL}};
const Segment SGI172762[] = {{&S64419[165], 3}, {NULL}};
const Segment SGI172763[] = {{&S64419[31], 3}, {&S64419[214], 5}, {NULL}};
const Segment SGI172764[] = {{&S64419[80], 6}, {NULL}};
const Segment SGI172765[] = {{&S56755[44], 6}, {NULL}};
const Segment SGI172766[] = {{&S56755[44], 18}, {NULL}};
const Segment SGI172767[] = {{&S56755[30], 6}, {NULL}};
const Segment SGI172768[] = {{&S56755[111], 3}, {NULL}};
const Segment SGI172769[] = {{&S56755[76], 6}, {NULL}};
const Segment SGI172770[] = {{&S64421[42], 10}, {NULL}};
const Segment SGI172771[] = {{&S64421[30], 6}, {NULL}};
const Segment SGI172772[] = {{&S64421[137], 4}, {NULL}};
const Segment SGI172773[] = {{&S56756[38], 6}, {NULL}};
const Segment SGI172774[] = {{&S64421[163], 3}, {NULL}};
const Segment SGI172775[] = {{&S56756[31], 6}, {NULL}};
const Segment SGI172776[] = {{&S64421[31], 3}, {&S64421[212], 5}, {NULL}};
const Segment SGI172777[] = {{&S64421[78], 6}, {NULL}};
const Segment SGI172778[] = {{&S53450[34], 10}, {NULL}};
const Segment SGI172779[] = {{&S53450[27], 6}, {NULL}};
const Segment SGI172780[] = {{&S53450[97], 9}, {NULL}};
const Segment SGI172781[] = {{&S53450[70], 6}, {NULL}};
const Segment SGI172782[] = {{&S64430[44], 13}, {NULL}};
const Segment SGI172783[] = {{&S64430[44], 25}, {NULL}};
const Segment SGI172784[] = {{&S64430[30], 6}, {NULL}};
const Segment SGI172785[] = {{&S64430[31], 3}, {NULL}};
const Segment SGI172786[] = {{&S64430[83], 6}, {NULL}};
const Segment SGI172787[] = {{&S41950[34], 10}, {NULL}};
const Segment SGI172788[] = {{&S41950[27], 6}, {NULL}};
const Segment SGI172789[] = {{&S64431[38], 13}, {NULL}};
const Segment SGI172790[] = {{&S64431[31], 6}, {NULL}};
const Segment SGI172791[] = {{&S41950[96], 9}, {NULL}};
const Segment SGI172792[] = {{&S41950[69], 6}, {NULL}};
const Segment SGI172793[] = {{&S41951[34], 11}, {NULL}};
const Segment SGI172794[] = {{&S41951[27], 6}, {NULL}};
const Segment SGI172795[] = {{&S41951[97], 9}, {NULL}};
const Segment SGI172796[] = {{&S41951[70], 6}, {NULL}};
const Segment SGI172797[] = {{&S41952[34], 10}, {NULL}};
const Segment SGI172798[] = {{&S41952[27], 6}, {NULL}};
const Segment SGI172799[] = {{&S41952[96], 9}, {NULL}};
const Segment SGI172800[] = {{&S41952[69], 6}, {NULL}};
const Segment SGI172801[] = {{&S53465[44], 7}, {NULL}};
const Segment SGI172802[] = {{&S53465[44], 19}, {NULL}};
const Segment SGI172803[] = {{&S53465[30], 6}, {NULL}};
const Segment SGI172804[] = {{&S53465[128], 3}, {NULL}};
const Segment SGI172805[] = {{&S53465[93], 6}, {NULL}};
const Segment SGI172806[] = {{&S53465[65], 8}, {NULL}};
const Segment SGI172807[] = {{&S50951[44], 18}, {NULL}};
const Segment SGI172808[] = {{&S50951[44], 30}, {NULL}};
const Segment SGI172809[] = {{&S50951[30], 6}, {NULL}};
const Segment SGI172810[] = {{&S50951[141], 3}, {NULL}};
const Segment SGI172811[] = {{&S50951[106], 6}, {NULL}};
const Segment SGI172812[] = {{&S50951[76], 8}, {NULL}};
const Segment SGI172813[] = {{&S56779[34], 19}, {NULL}};
const Segment SGI172814[] = {{&S56779[27], 6}, {NULL}};
const Segment SGI172815[] = {{&S56779[106], 9}, {NULL}};
const Segment SGI172816[] = {{&S56779[79], 6}, {NULL}};
const Segment SGI172817[] = {{&S56780[34], 13}, {NULL}};
const Segment SGI172818[] = {{&S56780[27], 6}, {NULL}};
const Segment SGI172819[] = {{&S56780[100], 9}, {NULL}};
const Segment SGI172820[] = {{&S56780[73], 6}, {NULL}};
const Segment SGI172821[] = {{&S64446[34], 16}, {NULL}};
const Segment SGI172822[] = {{&S64446[27], 6}, {NULL}};
const Segment SGI172823[] = {{&S64446[103], 9}, {NULL}};
const Segment SGI172824[] = {{&S64446[76], 6}, {NULL}};
const Segment SGI172825[] = {{&S64447[44], 11}, {NULL}};
const Segment SGI172826[] = {{&S64447[44], 23}, {NULL}};
const Segment SGI172827[] = {{&S64447[30], 6}, {NULL}};
const Segment SGI172828[] = {{&S64447[31], 3}, {NULL}};
const Segment SGI172829[] = {{&S53477[34], 18}, {NULL}};
const Segment SGI172830[] = {{&S64447[81], 6}, {NULL}};
const Segment SGI172831[] = {{&S53477[27], 6}, {NULL}};
const Segment SGI172832[] = {{&S53477[105], 9}, {NULL}};
const Segment SGI172833[] = {{&S53477[78], 6}, {NULL}};
const Segment SGI172834[] = {{&S64448[38], 11}, {NULL}};
const Segment SGI172835[] = {{&S64448[31], 6}, {NULL}};
const Segment SGI172836[] = {{&S53478[34], 10}, {NULL}};
const Segment SGI172837[] = {{&S53478[27], 6}, {NULL}};
const Segment SGI172838[] = {{&S53478[97], 9}, {NULL}};
const Segment SGI172839[] = {{&S53478[70], 6}, {NULL}};
const Segment SGI172840[] = {{&S64464[34], 10}, {NULL}};
const Segment SGI172841[] = {{&S56803[34], 10}, {NULL}};
const Segment SGI172842[] = {{&S64464[27], 6}, {NULL}};
const Segment SGI172843[] = {{&S56803[27], 6}, {NULL}};
const Segment SGI172844[] = {{&S64464[97], 9}, {NULL}};
const Segment SGI172845[] = {{&S56803[97], 9}, {NULL}};
const Segment SGI172846[] = {{&S64464[70], 6}, {NULL}};
const Segment SGI172847[] = {{&S56803[70], 6}, {NULL}};
const Segment SGI172848[] = {{&S56804[34], 18}, {NULL}};
const Segment SGI172849[] = {{&S56804[27], 6}, {NULL}};
const Segment SGI172850[] = {{&S53499[34], 8}, {NULL}};
const Segment SGI172851[] = {{&S53499[27], 6}, {NULL}};
const Segment SGI172852[] = {{&S56804[105], 9}, {NULL}};
const Segment SGI172853[] = {{&S56804[78], 6}, {NULL}};
const Segment SGI172854[] = {{&S53499[101], 9}, {NULL}};
const Segment SGI172855[] = {{&S53499[74], 6}, {NULL}};
const Segment SGI172856[] = {{&S50986[34], 10}, {NULL}};
const Segment SGI172857[] = {{&S50986[27], 6}, {NULL}};
const Segment SGI172858[] = {{&S50986[97], 9}, {NULL}};
const Segment SGI172859[] = {{&S50986[70], 6}, {NULL}};
const Segment SGI172860[] = {{&S50988[34], 11}, {NULL}};
const Segment SGI172861[] = {{&S50988[27], 6}, {NULL}};
const Segment SGI172862[] = {{&S50988[98], 9}, {NULL}};
const Segment SGI172863[] = {{&S50988[71], 6}, {NULL}};
const Segment SGI172864[] = {{&S56810[34], 7}, {NULL}};
const Segment SGI172865[] = {{&S56810[27], 6}, {NULL}};
const Segment SGI172866[] = {{&S56810[100], 9}, {NULL}};
const Segment SGI172867[] = {{&S56810[73], 6}, {NULL}};
const Segment SGI172868[] = {{&S64474[45], 10}, {NULL}};
const Segment SGI172869[] = {{&S64474[30], 6}, {NULL}};
const Segment SGI172870[] = {{&S64474[31], 3}, {NULL}};
const Segment SGI172871[] = {{&S64474[81], 5}, {NULL}};
const Segment SGI172872[] = {{&S64474[139], 13}, {NULL}};
const Segment SGI172873[] = {{&S64475[45], 11}, {NULL}};
const Segment SGI172874[] = {{&S64475[30], 6}, {NULL}};
const Segment SGI172875[] = {{&S64475[31], 3}, {NULL}};
const Segment SGI172876[] = {{&S64475[82], 5}, {NULL}};
const Segment SGI172877[] = {{&S64475[140], 13}, {NULL}};
const Segment SGI172878[] = {{&S53519[44], 10}, {NULL}};
const Segment SGI172879[] = {{&S53519[44], 22}, {NULL}};
const Segment SGI172880[] = {{&S53519[30], 6}, {NULL}};
const Segment SGI172881[] = {{&S53519[115], 3}, {NULL}};
const Segment SGI172882[] = {{&S53519[80], 6}, {NULL}};
const Segment SGI172883[] = {{&S56823[44], 10}, {NULL}};
const Segment SGI172884[] = {{&S56823[44], 22}, {NULL}};
const Segment SGI172885[] = {{&S56823[30], 6}, {NULL}};
const Segment SGI172886[] = {{&S56823[115], 3}, {NULL}};
const Segment SGI172887[] = {{&S56823[80], 6}, {NULL}};
const Segment SGI172888[] = {{&S53520[38], 10}, {NULL}};
const Segment SGI172889[] = {{&S53520[31], 6}, {NULL}};
const Segment SGI172890[] = {{&S56824[38], 6}, {NULL}};
const Segment SGI172891[] = {{&S56824[31], 6}, {NULL}};
const Segment SGI172892[] = {{&S56850[34], 18}, {NULL}};
const Segment SGI172893[] = {{&S56850[27], 6}, {NULL}};
const Segment SGI172894[] = {{&S56850[114], 9}, {NULL}};
const Segment SGI172895[] = {{&S56850[87], 6}, {NULL}};
const Segment SGI172896[] = {{&S42036[34], 14}, {NULL}};
const Segment SGI172897[] = {{&S42036[27], 6}, {NULL}};
const Segment SGI172898[] = {{&S42036[100], 9}, {NULL}};
const Segment SGI172899[] = {{&S42036[73], 6}, {NULL}};
const Segment SGI172900[] = {{&S42037[34], 9}, {NULL}};
const Segment SGI172901[] = {{&S42037[27], 6}, {NULL}};
const Segment SGI172902[] = {{&S42037[95], 9}, {NULL}};
const Segment SGI172903[] = {{&S42037[68], 6}, {NULL}};
const Segment SGI172904[] = {{&S53555[34], 25}, {NULL}};
const Segment SGI172905[] = {{&S53555[27], 6}, {NULL}};
const Segment SGI172906[] = {{&S53555[112], 9}, {NULL}};
const Segment SGI172907[] = {{&S53555[85], 6}, {NULL}};
const Segment SGI172908[] = {{&S42038[34], 5}, {NULL}};
const Segment SGI172909[] = {{&S42038[27], 6}, {NULL}};
const Segment SGI172910[] = {{&S42038[92], 9}, {NULL}};
const Segment SGI172911[] = {{&S42038[65], 6}, {NULL}};
const Segment SGI172912[] = {{&S42040[34], 18}, {NULL}};
const Segment SGI172913[] = {{&S42040[27], 6}, {NULL}};
const Segment SGI172914[] = {{&S42040[104], 9}, {NULL}};
const Segment SGI172915[] = {{&S42040[77], 6}, {NULL}};
const Segment SGI172916[] = {{&S53559[34], 11}, {NULL}};
const Segment SGI172917[] = {{&S53559[27], 6}, {NULL}};
const Segment SGI172918[] = {{&S53559[98], 9}, {NULL}};
const Segment SGI172919[] = {{&S53559[71], 6}, {NULL}};
const Segment SGI172920[] = {{&S42041[34], 35}, {NULL}};
const Segment SGI172921[] = {{&S42041[27], 6}, {NULL}};
const Segment SGI172922[] = {{&S42041[121], 9}, {NULL}};
const Segment SGI172923[] = {{&S42041[94], 6}, {NULL}};
const Segment SGI172924[] = {{&S53560[34], 11}, {NULL}};
const Segment SGI172925[] = {{&S53560[27], 6}, {NULL}};
const Segment SGI172926[] = {{&S53560[98], 9}, {NULL}};
const Segment SGI172927[] = {{&S53560[71], 6}, {NULL}};
const Segment SGI172928[] = {{&S42042[34], 11}, {NULL}};
const Segment SGI172929[] = {{&S42042[27], 6}, {NULL}};
const Segment SGI172930[] = {{&S42042[97], 9}, {NULL}};
const Segment SGI172931[] = {{&S42042[70], 6}, {NULL}};
const Segment SGI172932[] = {{&S56869[34], 9}, {NULL}};
const Segment SGI172933[] = {{&S56869[27], 6}, {NULL}};
const Segment SGI172934[] = {{&S56869[96], 9}, {NULL}};
const Segment SGI172935[] = {{&S56869[69], 6}, {NULL}};
const Segment SGI172936[] = {{&S51056[34], 29}, {NULL}};
const Segment SGI172937[] = {{&S51056[27], 6}, {NULL}};
const Segment SGI172938[] = {{&S51056[116], 9}, {NULL}};
const Segment SGI172939[] = {{&S51056[89], 6}, {NULL}};
const Segment SGI172940[] = {{&S64538[34], 18}, {NULL}};
const Segment SGI172941[] = {{&S64538[27], 6}, {NULL}};
const Segment SGI172942[] = {{&S64538[105], 9}, {NULL}};
const Segment SGI172943[] = {{&S64538[78], 6}, {NULL}};
const Segment SGI172944[] = {{&S64573[34], 10}, {NULL}};
const Segment SGI172945[] = {{&S64573[27], 6}, {NULL}};
const Segment SGI172946[] = {{&S64573[97], 9}, {NULL}};
const Segment SGI172947[] = {{&S64573[70], 6}, {NULL}};
const Segment SGI172948[] = {{&S42105[44], 10}, {NULL}};
const Segment SGI172949[] = {{&S42105[44], 21}, {NULL}};
const Segment SGI172950[] = {{&S42105[30], 6}, {NULL}};
const Segment SGI172951[] = {{&S42105[114], 3}, {NULL}};
const Segment SGI172952[] = {{&S42105[79], 6}, {NULL}};
const Segment SGI172953[] = {{&S42106[38], 10}, {NULL}};
const Segment SGI172954[] = {{&S42106[31], 6}, {NULL}};
const Segment SGI172955[] = {{&S42107[42], 10}, {NULL}};
const Segment SGI172956[] = {{&S42107[30], 6}, {NULL}};
const Segment SGI172957[] = {{&S42107[112], 3}, {NULL}};
const Segment SGI172958[] = {{&S42107[77], 6}, {NULL}};
const Segment SGI172959[] = {{&S42108[44], 10}, {NULL}};
const Segment SGI172960[] = {{&S42108[44], 22}, {NULL}};
const Segment SGI172961[] = {{&S42108[30], 6}, {NULL}};
const Segment SGI172962[] = {{&S42108[115], 3}, {NULL}};
const Segment SGI172963[] = {{&S42108[80], 6}, {NULL}};
const Segment SGI172964[] = {{&S42109[44], 11}, {NULL}};
const Segment SGI172965[] = {{&S42109[44], 22}, {NULL}};
const Segment SGI172966[] = {{&S42109[30], 6}, {NULL}};
const Segment SGI172967[] = {{&S64599[34], 17}, {NULL}};
const Segment SGI172968[] = {{&S42109[115], 3}, {NULL}};
const Segment SGI172969[] = {{&S64599[27], 6}, {NULL}};
const Segment SGI172970[] = {{&S42109[80], 6}, {NULL}};
const Segment SGI172971[] = {{&S64599[112], 9}, {NULL}};
const Segment SGI172972[] = {{&S64599[85], 6}, {NULL}};
const Segment SGI172973[] = {{&S64604[44], 13}, {NULL}};
const Segment SGI172974[] = {{&S64604[44], 25}, {NULL}};
const Segment SGI172975[] = {{&S64604[30], 6}, {NULL}};
const Segment SGI172976[] = {{&S64604[31], 3}, {NULL}};
const Segment SGI172977[] = {{&S64604[83], 6}, {NULL}};
const Segment SGI172978[] = {{&S64605[34], 8}, {NULL}};
const Segment SGI172979[] = {{&S64605[27], 6}, {NULL}};
const Segment SGI172980[] = {{&S64605[95], 9}, {NULL}};
const Segment SGI172981[] = {{&S64605[68], 6}, {NULL}};
const Segment SGI172982[] = {{&S53639[34], 7}, {NULL}};
const Segment SGI172983[] = {{&S53639[27], 6}, {NULL}};
const Segment SGI172984[] = {{&S53639[107], 9}, {NULL}};
const Segment SGI172985[] = {{&S53639[80], 6}, {NULL}};
const Segment SGI172986[] = {{&S53640[44], 8}, {NULL}};
const Segment SGI172987[] = {{&S53640[44], 20}, {NULL}};
const Segment SGI172988[] = {{&S53640[30], 6}, {NULL}};
const Segment SGI172989[] = {{&S64609[44], 6}, {NULL}};
const Segment SGI172990[] = {{&S64609[44], 18}, {NULL}};
const Segment SGI172991[] = {{&S64609[30], 6}, {NULL}};
const Segment SGI172992[] = {{&S53640[113], 3}, {NULL}};
const Segment SGI172993[] = {{&S53640[78], 6}, {NULL}};
const Segment SGI172994[] = {{&S64609[31], 3}, {NULL}};
const Segment SGI172995[] = {{&S64609[76], 6}, {NULL}};
const Segment SGI172996[] = {{&S53641[38], 8}, {NULL}};
const Segment SGI172997[] = {{&S53641[31], 6}, {NULL}};
const Segment SGI172998[] = {{&S64611[38], 6}, {NULL}};
const Segment SGI172999[] = {{&S42127[34], 10}, {NULL}};
const Segment SGI173000[] = {{&S64611[31], 6}, {NULL}};
const Segment SGI173001[] = {{&S42127[27], 6}, {NULL}};
const Segment SGI173002[] = {{&S42127[96], 9}, {NULL}};
const Segment SGI173003[] = {{&S42127[69], 6}, {NULL}};
const Segment SGI173004[] = {{&S64615[34], 10}, {NULL}};
const Segment SGI173005[] = {{&S64615[27], 6}, {NULL}};
const Segment SGI173006[] = {{&S64615[97], 9}, {NULL}};
const Segment SGI173007[] = {{&S64615[70], 6}, {NULL}};
const Segment SGI173008[] = {{&S51139[34], 8}, {NULL}};
const Segment SGI173009[] = {{&S51139[27], 6}, {NULL}};
const Segment SGI173010[] = {{&S51139[95], 9}, {NULL}};
const Segment SGI173011[] = {{&S51139[68], 6}, {NULL}};
const Segment SGI173012[] = {{&S64620[44], 5}, {NULL}};
const Segment SGI173013[] = {{&S64620[44], 17}, {NULL}};
const Segment SGI173014[] = {{&S64620[30], 6}, {NULL}};
const Segment SGI173015[] = {{&S64620[31], 3}, {NULL}};
const Segment SGI173016[] = {{&S64620[75], 6}, {NULL}};
const Segment SGI173017[] = {{&S64621[44], 13}, {NULL}};
const Segment SGI173018[] = {{&S64621[44], 24}, {NULL}};
const Segment SGI173019[] = {{&S64621[30], 6}, {NULL}};
const Segment SGI173020[] = {{&S64621[31], 3}, {NULL}};
const Segment SGI173021[] = {{&S64621[82], 6}, {NULL}};
const Segment SGI173022[] = {{&S53655[34], 10}, {NULL}};
const Segment SGI173023[] = {{&S53655[27], 6}, {NULL}};
const Segment SGI173024[] = {{&S64622[34], 8}, {NULL}};
const Segment SGI173025[] = {{&S64622[27], 6}, {NULL}};
const Segment SGI173026[] = {{&S53655[97], 9}, {NULL}};
const Segment SGI173027[] = {{&S53655[70], 6}, {NULL}};
const Segment SGI173028[] = {{&S64622[101], 9}, {NULL}};
const Segment SGI173029[] = {{&S64622[74], 6}, {NULL}};
const Segment SGI173030[] = {{&S51148[34], 8}, {NULL}};
const Segment SGI173031[] = {{&S51148[27], 6}, {NULL}};
const Segment SGI173032[] = {{&S51148[95], 9}, {NULL}};
const Segment SGI173033[] = {{&S51148[68], 6}, {NULL}};
const Segment SGI173034[] = {{&S53656[34], 11}, {NULL}};
const Segment SGI173035[] = {{&S53656[27], 6}, {NULL}};
const Segment SGI173036[] = {{&S53656[98], 9}, {NULL}};
const Segment SGI173037[] = {{&S53656[71], 6}, {NULL}};
const Segment SGI173038[] = {{&S56992[34], 7}, {NULL}};
const Segment SGI173039[] = {{&S56992[27], 6}, {NULL}};
const Segment SGI173040[] = {{&S56992[99], 9}, {NULL}};
const Segment SGI173041[] = {{&S56992[72], 6}, {NULL}};
const Segment SGI173042[] = {{&S42160[34], 10}, {NULL}};
const Segment SGI173043[] = {{&S42160[27], 6}, {NULL}};
const Segment SGI173044[] = {{&S42160[96], 9}, {NULL}};
const Segment SGI173045[] = {{&S42160[69], 6}, {NULL}};
const Segment SGI173046[] = {{&S53702[34], 13}, {NULL}};
const Segment SGI173047[] = {{&S53702[27], 6}, {NULL}};
const Segment SGI173048[] = {{&S53702[100], 9}, {NULL}};
const Segment SGI173049[] = {{&S53702[73], 6}, {NULL}};
const Segment SGI173050[] = {{&S53703[34], 10}, {NULL}};
const Segment SGI173051[] = {{&S53703[27], 6}, {NULL}};
const Segment SGI173052[] = {{&S53703[97], 9}, {NULL}};
const Segment SGI173053[] = {{&S64679[44], 7}, {NULL}};
const Segment SGI173054[] = {{&S53703[70], 6}, {NULL}};
const Segment SGI173055[] = {{&S64679[44], 20}, {NULL}};
const Segment SGI173056[] = {{&S64679[30], 6}, {NULL}};
const Segment SGI173057[] = {{&S64679[31], 3}, {NULL}};
const Segment SGI173058[] = {{&S64679[78], 6}, {NULL}};
const Segment SGI173059[] = {{&S64680[44], 7}, {NULL}};
const Segment SGI173060[] = {{&S64680[44], 20}, {NULL}};
const Segment SGI173061[] = {{&S64680[30], 6}, {NULL}};
const Segment SGI173062[] = {{&S64680[31], 3}, {NULL}};
const Segment SGI173063[] = {{&S64680[78], 6}, {NULL}};
const Segment SGI173064[] = {{&S64681[44], 7}, {NULL}};
const Segment SGI173065[] = {{&S42196[44], 9}, {NULL}};
const Segment SGI173066[] = {{&S64681[44], 20}, {NULL}};
const Segment SGI173067[] = {{&S42196[44], 20}, {NULL}};
const Segment SGI173068[] = {{&S64681[30], 6}, {NULL}};
const Segment SGI173069[] = {{&S42196[30], 6}, {NULL}};
const Segment SGI173070[] = {{&S64681[31], 3}, {NULL}};
const Segment SGI173071[] = {{&S42196[113], 3}, {NULL}};
const Segment SGI173072[] = {{&S64681[78], 6}, {NULL}};
const Segment SGI173073[] = {{&S42196[78], 6}, {NULL}};
const Segment SGI173074[] = {{&S42197[38], 9}, {NULL}};
const Segment SGI173075[] = {{&S42197[31], 6}, {NULL}};
const Segment SGI173076[] = {{&S64682[44], 7}, {NULL}};
const Segment SGI173077[] = {{&S64682[44], 20}, {NULL}};
const Segment SGI173078[] = {{&S64682[30], 6}, {NULL}};
const Segment SGI173079[] = {{&S64682[31], 3}, {NULL}};
const Segment SGI173080[] = {{&S64682[78], 6}, {NULL}};
const Segment SGI173081[] = {{&S42198[44], 10}, {NULL}};
const Segment SGI173082[] = {{&S42198[44], 21}, {NULL}};
const Segment SGI173083[] = {{&S42198[30], 6}, {NULL}};
const Segment SGI173084[] = {{&S42198[114], 3}, {NULL}};
const Segment SGI173085[] = {{&S64683[44], 7}, {NULL}};
const Segment SGI173086[] = {{&S64683[44], 20}, {NULL}};
const Segment SGI173087[] = {{&S42198[79], 6}, {NULL}};
const Segment SGI173088[] = {{&S64683[30], 6}, {NULL}};
const Segment SGI173089[] = {{&S64683[31], 3}, {NULL}};
const Segment SGI173090[] = {{&S64683[78], 6}, {NULL}};
const Segment SGI173091[] = {{&S64684[44], 7}, {NULL}};
const Segment SGI173092[] = {{&S64684[44], 20}, {NULL}};
const Segment SGI173093[] = {{&S64684[30], 6}, {NULL}};
const Segment SGI173094[] = {{&S64684[31], 3}, {NULL}};
const Segment SGI173095[] = {{&S64684[78], 6}, {NULL}};
const Segment SGI173096[] = {{&S57065[34], 9}, {NULL}};
const Segment SGI173097[] = {{&S57065[27], 6}, {NULL}};
const Segment SGI173098[] = {{&S57065[96], 9}, {NULL}};
const Segment SGI173099[] = {{&S57065[69], 6}, {NULL}};
const Segment SGI173100[] = {{&S64733[44], 18}, {NULL}};
const Segment SGI173101[] = {{&S64733[44], 30}, {NULL}};
const Segment SGI173102[] = {{&S64733[30], 6}, {NULL}};
const Segment SGI173103[] = {{&S64733[31], 3}, {NULL}};
const Segment SGI173104[] = {{&S64733[112], 6}, {NULL}};
const Segment SGI173105[] = {{&S64733[76], 8}, {NULL}};
const Segment SGI173106[] = {{&S57078[34], 8}, {NULL}};
const Segment SGI173107[] = {{&S57078[27], 6}, {NULL}};
const Segment SGI173108[] = {{&S64734[44], 18}, {NULL}};
const Segment SGI173109[] = {{&S57078[96], 9}, {NULL}};
const Segment SGI173110[] = {{&S64734[44], 30}, {NULL}};
const Segment SGI173111[] = {{&S57078[69], 6}, {NULL}};
const Segment SGI173112[] = {{&S64734[30], 6}, {NULL}};
const Segment SGI173113[] = {{&S64734[31], 3}, {NULL}};
const Segment SGI173114[] = {{&S64734[107], 6}, {NULL}};
const Segment SGI173115[] = {{&S64734[76], 8}, {NULL}};
const Segment SGI173116[] = {{&S57079[34], 8}, {NULL}};
const Segment SGI173117[] = {{&S57079[27], 6}, {NULL}};
const Segment SGI173118[] = {{&S57079[96], 9}, {NULL}};
const Segment SGI173119[] = {{&S64735[34], 10}, {NULL}};
const Segment SGI173120[] = {{&S57079[69], 6}, {NULL}};
const Segment SGI173121[] = {{&S64735[27], 6}, {NULL}};
const Segment SGI173122[] = {{&S64735[97], 9}, {NULL}};
const Segment SGI173123[] = {{&S64735[70], 6}, {NULL}};
const Segment SGI173124[] = {{&S64736[44], 10}, {NULL}};
const Segment SGI173125[] = {{&S64736[44], 22}, {NULL}};
const Segment SGI173126[] = {{&S64736[30], 6}, {NULL}};
const Segment SGI173127[] = {{&S64736[31], 3}, {NULL}};
const Segment SGI173128[] = {{&S64736[80], 6}, {NULL}};
const Segment SGI173129[] = {{&S64743[34], 7}, {NULL}};
const Segment SGI173130[] = {{&S64743[27], 6}, {NULL}};
const Segment SGI173131[] = {{&S64743[99], 9}, {NULL}};
const Segment SGI173132[] = {{&S64743[72], 6}, {NULL}};
const Segment SGI173133[] = {{&S64744[34], 7}, {NULL}};
const Segment SGI173134[] = {{&S64744[27], 6}, {NULL}};
const Segment SGI173135[] = {{&S64744[99], 9}, {NULL}};
const Segment SGI173136[] = {{&S64744[72], 6}, {NULL}};
const Segment SGI173137[] = {{&S64745[34], 7}, {NULL}};
const Segment SGI173138[] = {{&S64745[27], 6}, {NULL}};
const Segment SGI173139[] = {{&S64745[99], 9}, {NULL}};
const Segment SGI173140[] = {{&S64745[72], 6}, {NULL}};
const Segment SGI173141[] = {{&S42269[34], 8}, {NULL}};
const Segment SGI173142[] = {{&S42269[27], 6}, {NULL}};
const Segment SGI173143[] = {{&S42269[95], 9}, {NULL}};
const Segment SGI173144[] = {{&S42269[68], 6}, {NULL}};
const Segment SGI173145[] = {{&S42271[34], 24}, {NULL}};
const Segment SGI173146[] = {{&S42271[27], 6}, {NULL}};
const Segment SGI173147[] = {{&S42271[110], 9}, {NULL}};
const Segment SGI173148[] = {{&S42271[83], 6}, {NULL}};
const Segment SGI173149[] = {{&S51306[44], 12}, {NULL}};
const Segment SGI173150[] = {{&S51306[44], 24}, {NULL}};
const Segment SGI173151[] = {{&S51306[30], 6}, {NULL}};
const Segment SGI173152[] = {{&S51306[135], 3}, {NULL}};
const Segment SGI173153[] = {{&S51306[100], 6}, {NULL}};
const Segment SGI173154[] = {{&S51306[70], 8}, {NULL}};
const Segment SGI173155[] = {{&S51307[44], 28}, {NULL}};
const Segment SGI173156[] = {{&S51307[44], 40}, {NULL}};
const Segment SGI173157[] = {{&S51307[30], 6}, {NULL}};
const Segment SGI173158[] = {{&S51307[151], 3}, {NULL}};
const Segment SGI173159[] = {{&S51307[116], 6}, {NULL}};
const Segment SGI173160[] = {{&S51307[86], 8}, {NULL}};
const Segment SGI173161[] = {{&S51308[44], 12}, {NULL}};
const Segment SGI173162[] = {{&S51308[44], 24}, {NULL}};
const Segment SGI173163[] = {{&S51308[30], 6}, {NULL}};
const Segment SGI173164[] = {{&S51308[135], 3}, {NULL}};
const Segment SGI173165[] = {{&S51308[100], 6}, {NULL}};
const Segment SGI173166[] = {{&S51308[70], 8}, {NULL}};
const Segment SGI173167[] = {{&S64781[34], 7}, {NULL}};
const Segment SGI173168[] = {{&S64781[27], 6}, {NULL}};
const Segment SGI173169[] = {{&S64781[100], 9}, {NULL}};
const Segment SGI173170[] = {{&S64781[73], 6}, {NULL}};
const Segment SGI173171[] = {{&S51312[44], 28}, {NULL}};
const Segment SGI173172[] = {{&S51312[44], 40}, {NULL}};
const Segment SGI173173[] = {{&S51312[30], 6}, {NULL}};
const Segment SGI173174[] = {{&S51312[150], 3}, {NULL}};
const Segment SGI173175[] = {{&S51312[116], 6}, {NULL}};
const Segment SGI173176[] = {{&S64782[34], 7}, {NULL}};
const Segment SGI173177[] = {{&S51312[86], 8}, {NULL}};
const Segment SGI173178[] = {{&S64782[27], 6}, {NULL}};
const Segment SGI173179[] = {{&S64782[100], 9}, {NULL}};
const Segment SGI173180[] = {{&S64782[73], 6}, {NULL}};
const Segment SGI173181[] = {{&S51326[41], 18}, {NULL}};
const Segment SGI173182[] = {{&S51326[41], 30}, {NULL}};
const Segment SGI173183[] = {{&S51326[27], 6}, {NULL}};
const Segment SGI173184[] = {{&S51326[120], 3}, {NULL}};
const Segment SGI173185[] = {{&S51326[126], 9}, {NULL}};
const Segment SGI173186[] = {{&S51326[85], 6}, {NULL}};
const Segment SGI173187[] = {{&S53704[34], 14}, {NULL}};
const Segment SGI173188[] = {{&S53704[27], 6}, {NULL}};
const Segment SGI173189[] = {{&S53704[101], 9}, {NULL}};
const Segment SGI173190[] = {{&S53704[74], 6}, {NULL}};
const Segment SGI173191[] = {{&S53705[34], 10}, {NULL}};
const Segment SGI173192[] = {{&S53705[27], 6}, {NULL}};
const Segment SGI173193[] = {{&S53705[97], 9}, {NULL}};
const Segment SGI173194[] = {{&S53705[70], 6}, {NULL}};
const Segment SGI173195[] = {{&S53706[34], 14}, {NULL}};
const Segment SGI173196[] = {{&S53706[27], 6}, {NULL}};
const Segment SGI173197[] = {{&S53706[101], 9}, {NULL}};
const Segment SGI173198[] = {{&S53706[74], 6}, {NULL}};
const Segment SGI173199[] = {{&S53715[34], 7}, {NULL}};
const Segment SGI173200[] = {{&S53715[27], 6}, {NULL}};
const Segment SGI173201[] = {{&S53715[102], 9}, {NULL}};
const Segment SGI173202[] = {{&S53715[75], 6}, {NULL}};
const Segment SGI173203[] = {{&S53716[34], 8}, {NULL}};
const Segment SGI173204[] = {{&S53716[27], 6}, {NULL}};
const Segment SGI173205[] = {{&S53716[101], 9}, {NULL}};
const Segment SGI173206[] = {{&S53716[74], 6}, {NULL}};
const Segment SGI173207[] = {{&S64837[44], 13}, {NULL}};
const Segment SGI173208[] = {{&S64837[44], 25}, {NULL}};
const Segment SGI173209[] = {{&S64837[30], 6}, {&S64837[52], 17}, {NULL}};
const Segment SGI173210[] = {{&S64837[31], 3}, {NULL}};
const Segment SGI173211[] = {{&S64837[83], 6}, {NULL}};
const Segment SGI173212[] = {{&S64838[44], 14}, {NULL}};
const Segment SGI173213[] = {{&S64838[44], 26}, {NULL}};
const Segment SGI173214[] = {{&S64838[30], 6}, {NULL}};
const Segment SGI173215[] = {{&S64838[31], 3}, {NULL}};
const Segment SGI173216[] = {{&S64838[84], 6}, {NULL}};
const Segment SGI173217[] = {{&S64845[34], 7}, {NULL}};
const Segment SGI173218[] = {{&S64845[27], 6}, {NULL}};
const Segment SGI173219[] = {{&S64845[99], 9}, {NULL}};
const Segment SGI173220[] = {{&S64845[72], 6}, {NULL}};
const Segment SGI173221[] = {{&S64850[34], 12}, {NULL}};
const Segment SGI173222[] = {{&S64850[27], 6}, {NULL}};
const Segment SGI173223[] = {{&S64850[115], 9}, {NULL}};
const Segment SGI173224[] = {{&S64850[88], 6}, {NULL}};
const Segment SGI173225[] = {{&S64863[34], 9}, {NULL}};
const Segment SGI173226[] = {{&S64863[27], 6}, {NULL}};
const Segment SGI173227[] = {{&S64863[96], 9}, {NULL}};
const Segment SGI173228[] = {{&S64863[69], 6}, {NULL}};
const Segment SGI173229[] = {{&S53770[44], 10}, {NULL}};
const Segment SGI173230[] = {{&S53770[44], 22}, {NULL}};
const Segment SGI173231[] = {{&S53770[30], 6}, {NULL}};
const Segment SGI173232[] = {{&S64885[34], 18}, {NULL}};
const Segment SGI173233[] = {{&S64885[27], 6}, {NULL}};
const Segment SGI173234[] = {{&S53770[115], 3}, {NULL}};
const Segment SGI173235[] = {{&S53770[80], 6}, {NULL}};
const Segment SGI173236[] = {{&S64885[113], 9}, {NULL}};
const Segment SGI173237[] = {{&S64885[86], 6}, {NULL}};
const Segment SGI173238[] = {{&S53771[44], 30}, {NULL}};
const Segment SGI173239[] = {{&S53771[44], 42}, {NULL}};
const Segment SGI173240[] = {{&S53771[30], 6}, {NULL}};
const Segment SGI173241[] = {{&S53771[135], 3}, {NULL}};
const Segment SGI173242[] = {{&S53771[100], 6}, {NULL}};
const Segment SGI173243[] = {{&S53772[38], 10}, {NULL}};
const Segment SGI173244[] = {{&S53772[31], 6}, {NULL}};
const Segment SGI173245[] = {{&S53773[44], 11}, {NULL}};
const Segment SGI173246[] = {{&S53773[44], 23}, {NULL}};
const Segment SGI173247[] = {{&S53773[30], 6}, {NULL}};
const Segment SGI173248[] = {{&S53773[116], 3}, {NULL}};
const Segment SGI173249[] = {{&S53773[81], 6}, {NULL}};
const Segment SGI173250[] = {{&S53774[44], 11}, {NULL}};
const Segment SGI173251[] = {{&S53774[44], 23}, {NULL}};
const Segment SGI173252[] = {{&S53774[30], 6}, {NULL}};
const Segment SGI173253[] = {{&S53774[116], 3}, {NULL}};
const Segment SGI173254[] = {{&S53774[81], 6}, {NULL}};
const Segment SGI173255[] = {{&S57247[44], 10}, {NULL}};
const Segment SGI173256[] = {{&S57247[44], 22}, {NULL}};
const Segment SGI173257[] = {{&S57247[30], 6}, {NULL}};
const Segment SGI173258[] = {{&S57247[115], 3}, {NULL}};
const Segment SGI173259[] = {{&S53775[44], 30}, {NULL}};
const Segment SGI173260[] = {{&S57247[80], 6}, {NULL}};
const Segment SGI173261[] = {{&S53775[44], 42}, {NULL}};
const Segment SGI173262[] = {{&S53775[30], 6}, {NULL}};
const Segment SGI173263[] = {{&S53775[135], 3}, {NULL}};
const Segment SGI173264[] = {{&S53775[100], 6}, {NULL}};
const Segment SGI173265[] = {{&S57248[54], 10}, {NULL}};
const Segment SGI173266[] = {{&S57248[47], 6}, {NULL}};
const Segment SGI173267[] = {{&S53776[44], 11}, {NULL}};
const Segment SGI173268[] = {{&S53776[44], 23}, {NULL}};
const Segment SGI173269[] = {{&S53776[30], 6}, {NULL}};
const Segment SGI173270[] = {{&S53776[116], 3}, {NULL}};
const Segment SGI173271[] = {{&S53776[81], 6}, {NULL}};
const Segment SGI173272[] = {{&S53777[44], 30}, {NULL}};
const Segment SGI173273[] = {{&S53777[44], 42}, {NULL}};
const Segment SGI173274[] = {{&S53777[30], 6}, {NULL}};
const Segment SGI173275[] = {{&S53777[135], 3}, {NULL}};
const Segment SGI173276[] = {{&S62570[34], 10}, {NULL}};
const Segment SGI173277[] = {{&S53777[100], 6}, {NULL}};
const Segment SGI173278[] = {{&S62570[27], 6}, {NULL}};
const Segment SGI173279[] = {{&S62570[70], 6}, {NULL}};
const Segment SGI173280[] = {{&S53783[34], 10}, {NULL}};
const Segment SGI173281[] = {{&S53783[27], 6}, {NULL}};
const Segment SGI173282[] = {{&S53783[97], 9}, {NULL}};
const Segment SGI173283[] = {{&S53783[70], 6}, {NULL}};
const Segment SGI173284[] = {{&S53792[34], 7}, {NULL}};
const Segment SGI173285[] = {{&S53792[27], 6}, {NULL}};
const Segment SGI173286[] = {{&S53792[107], 9}, {NULL}};
const Segment SGI173287[] = {{&S53792[80], 6}, {NULL}};
const Segment SGI173288[] = {{&S57278[44], 7}, {NULL}};
const Segment SGI173289[] = {{&S57278[44], 19}, {NULL}};
const Segment SGI173290[] = {{&S57278[30], 6}, {NULL}};
const Segment SGI173291[] = {{&S64928[44], 8}, {NULL}};
const Segment SGI173292[] = {{&S64928[44], 20}, {NULL}};
const Segment SGI173293[] = {{&S64928[30], 6}, {NULL}};
const Segment SGI173294[] = {{&S57278[112], 3}, {NULL}};
const Segment SGI173295[] = {{&S57278[77], 6}, {NULL}};
const Segment SGI173296[] = {{&S64928[31], 3}, {NULL}};
const Segment SGI173297[] = {{&S64928[78], 6}, {NULL}};
const Segment SGI173298[] = {{&S57279[38], 7}, {NULL}};
const Segment SGI173299[] = {{&S57279[31], 6}, {NULL}};
const Segment SGI173300[] = {{&S53815[34], 7}, {NULL}};
const Segment SGI173301[] = {{&S53815[27], 6}, {NULL}};
const Segment SGI173302[] = {{&S53815[98], 9}, {NULL}};
const Segment SGI173303[] = {{&S53815[71], 6}, {NULL}};
const Segment SGI173304[] = {{&S57292[44], 13}, {NULL}};
const Segment SGI173305[] = {{&S57292[44], 25}, {NULL}};
const Segment SGI173306[] = {{&S57292[30], 6}, {NULL}};
const Segment SGI173307[] = {{&S57292[118], 3}, {NULL}};
const Segment SGI173308[] = {{&S57292[83], 6}, {NULL}};
const Segment SGI173309[] = {{&S57293[44], 13}, {NULL}};
const Segment SGI173310[] = {{&S57293[44], 20}, {NULL}};
const Segment SGI173311[] = {{&S57293[30], 6}, {NULL}};
const Segment SGI173312[] = {{&S57293[113], 3}, {NULL}};
const Segment SGI173313[] = {{&S57293[78], 6}, {NULL}};
const Segment SGI173314[] = {{&S57294[38], 13}, {NULL}};
const Segment SGI173315[] = {{&S57294[31], 6}, {NULL}};
const Segment SGI173316[] = {{&S64961[44], 11}, {NULL}};
const Segment SGI173317[] = {{&S64961[44], 23}, {NULL}};
const Segment SGI173318[] = {{&S64961[30], 6}, {NULL}};
const Segment SGI173319[] = {{&S64961[31], 3}, {NULL}};
const Segment SGI173320[] = {{&S64961[81], 6}, {NULL}};
const Segment SGI173321[] = {{&S64974[34], 18}, {NULL}};
const Segment SGI173322[] = {{&S64974[27], 6}, {NULL}};
const Segment SGI173323[] = {{&S64974[105], 9}, {NULL}};
const Segment SGI173324[] = {{&S64974[78], 6}, {NULL}};
const Segment SGI173325[] = {{&S65002[34], 10}, {NULL}};
const Segment SGI173326[] = {{&S65002[27], 6}, {NULL}};
const Segment SGI173327[] = {{&S65002[97], 9}, {NULL}};
const Segment SGI173328[] = {{&S65002[70], 6}, {NULL}};
const Segment SGI173329[] = {{&S65014[34], 14}, {NULL}};
const Segment SGI173330[] = {{&S65014[27], 6}, {NULL}};
const Segment SGI173331[] = {{&S65014[106], 9}, {NULL}};
const Segment SGI173332[] = {{&S65014[79], 6}, {NULL}};
const Segment SGI173333[] = {{&S65015[44], 11}, {NULL}};
const Segment SGI173334[] = {{&S65015[44], 23}, {NULL}};
const Segment SGI173335[] = {{&S65015[30], 6}, {NULL}};
const Segment SGI173336[] = {{&S65015[31], 3}, {NULL}};
const Segment SGI173337[] = {{&S65015[81], 6}, {NULL}};
const Segment SGI173338[] = {{&S53929[34], 10}, {NULL}};
const Segment SGI173339[] = {{&S53929[27], 6}, {NULL}};
const Segment SGI173340[] = {{&S53929[97], 9}, {NULL}};
const Segment SGI173341[] = {{&S53929[70], 6}, {NULL}};
const Segment SGI173342[] = {{&S51584[38], 8}, {NULL}};
const Segment SGI173343[] = {{&S51584[31], 6}, {NULL}};
const Segment SGI173344[] = {{&S51585[44], 8}, {NULL}};
const Segment SGI173345[] = {{&S51585[44], 20}, {NULL}};
const Segment SGI173346[] = {{&S51585[30], 6}, {NULL}};
const Segment SGI173347[] = {{&S51585[113], 3}, {NULL}};
const Segment SGI173348[] = {{&S51585[78], 6}, {NULL}};
const Segment SGI173349[] = {{&S65046[44], 10}, {NULL}};
const Segment SGI173350[] = {{&S65046[44], 22}, {NULL}};
const Segment SGI173351[] = {{&S65046[30], 6}, {NULL}};
const Segment SGI173352[] = {{&S65046[31], 3}, {NULL}};
const Segment SGI173353[] = {{&S65046[80], 6}, {NULL}};
const Segment SGI173354[] = {{&S53936[34], 7}, {NULL}};
const Segment SGI173355[] = {{&S53936[27], 6}, {NULL}};
const Segment SGI173356[] = {{&S53936[99], 9}, {NULL}};
const Segment SGI173357[] = {{&S53936[72], 6}, {NULL}};
const Segment SGI173358[] = {{&S53937[34], 7}, {NULL}};
const Segment SGI173359[] = {{&S53937[27], 6}, {NULL}};
const Segment SGI173360[] = {{&S53937[99], 9}, {NULL}};
const Segment SGI173361[] = {{&S53937[72], 6}, {NULL}};
const Segment SGI173362[] = {{&S53946[34], 10}, {NULL}};
const Segment SGI173363[] = {{&S53946[27], 6}, {NULL}};
const Segment SGI173364[] = {{&S53946[97], 9}, {NULL}};
const Segment SGI173365[] = {{&S53946[70], 6}, {NULL}};
const Segment SGI173366[] = {{&S53947[34], 14}, {NULL}};
const Segment SGI173367[] = {{&S53947[27], 6}, {NULL}};
const Segment SGI173368[] = {{&S53947[101], 9}, {NULL}};
const Segment SGI173369[] = {{&S53947[74], 6}, {NULL}};
const Segment SGI173370[] = {{&S53948[34], 13}, {NULL}};
const Segment SGI173371[] = {{&S53948[27], 6}, {NULL}};
const Segment SGI173372[] = {{&S53948[100], 9}, {NULL}};
const Segment SGI173373[] = {{&S53948[73], 6}, {NULL}};
const Segment SGI173374[] = {{&S53949[44], 9}, {NULL}};
const Segment SGI173375[] = {{&S53949[44], 21}, {NULL}};
const Segment SGI173376[] = {{&S53949[30], 6}, {NULL}};
const Segment SGI173377[] = {{&S53949[114], 3}, {NULL}};
const Segment SGI173378[] = {{&S53949[79], 6}, {NULL}};
const Segment SGI173379[] = {{&S53950[38], 9}, {NULL}};
const Segment SGI173380[] = {{&S53950[31], 6}, {NULL}};
const Segment SGI173381[] = {{&S53961[34], 10}, {NULL}};
const Segment SGI173382[] = {{&S53961[27], 6}, {NULL}};
const Segment SGI173383[] = {{&S53961[97], 9}, {NULL}};
const Segment SGI173384[] = {{&S53961[70], 6}, {NULL}};
const Segment SGI173385[] = {{&S65085[44], 7}, {NULL}};
const Segment SGI173386[] = {{&S65085[44], 19}, {NULL}};
const Segment SGI173387[] = {{&S65085[30], 6}, {NULL}};
const Segment SGI173388[] = {{&S65085[31], 3}, {NULL}};
const Segment SGI173389[] = {{&S65085[77], 6}, {NULL}};
const Segment SGI173390[] = {{&S65086[44], 10}, {NULL}};
const Segment SGI173391[] = {{&S65086[44], 29}, {NULL}};
const Segment SGI173392[] = {{&S65086[30], 6}, {NULL}};
const Segment SGI173393[] = {{&S65086[31], 3}, {NULL}};
const Segment SGI173394[] = {{&S65086[87], 6}, {NULL}};
const Segment SGI173395[] = {{&S65087[38], 10}, {NULL}};
const Segment SGI173396[] = {{&S65087[31], 6}, {NULL}};
const Segment SGI173397[] = {{&S65090[34], 10}, {NULL}};
const Segment SGI173398[] = {{&S65090[27], 6}, {NULL}};
const Segment SGI173399[] = {{&S65090[97], 9}, {NULL}};
const Segment SGI173400[] = {{&S65090[70], 6}, {NULL}};
const Segment SGI173401[] = {{&S65091[34], 9}, {NULL}};
const Segment SGI173402[] = {{&S65091[27], 6}, {NULL}};
const Segment SGI173403[] = {{&S65091[96], 9}, {NULL}};
const Segment SGI173404[] = {{&S65091[69], 6}, {NULL}};
const Segment SGI173405[] = {{&S65106[44], 11}, {NULL}};
const Segment SGI173406[] = {{&S65106[44], 23}, {NULL}};
const Segment SGI173407[] = {{&S65106[30], 6}, {NULL}};
const Segment SGI173408[] = {{&S65106[31], 3}, {NULL}};
const Segment SGI173409[] = {{&S65106[81], 6}, {NULL}};
const Segment SGI173410[] = {{&S54011[44], 10}, {NULL}};
const Segment SGI173411[] = {{&S54011[44], 22}, {NULL}};
const Segment SGI173412[] = {{&S54011[30], 6}, {NULL}};
const Segment SGI173413[] = {{&S54011[115], 3}, {NULL}};
const Segment SGI173414[] = {{&S54011[80], 6}, {NULL}};
const Segment SGI173415[] = {{&S54012[44], 10}, {NULL}};
const Segment SGI173416[] = {{&S54012[44], 22}, {NULL}};
const Segment SGI173417[] = {{&S54012[30], 6}, {NULL}};
const Segment SGI173418[] = {{&S54012[115], 3}, {NULL}};
const Segment SGI173419[] = {{&S54012[80], 6}, {NULL}};
const Segment SGI173420[] = {{&S54013[44], 15}, {NULL}};
const Segment SGI173421[] = {{&S54013[44], 27}, {NULL}};
const Segment SGI173422[] = {{&S54013[30], 6}, {NULL}};
const Segment SGI173423[] = {{&S54013[120], 3}, {NULL}};
const Segment SGI173424[] = {{&S54013[85], 6}, {NULL}};
const Segment SGI173425[] = {{&S54014[44], 15}, {NULL}};
const Segment SGI173426[] = {{&S54014[44], 27}, {NULL}};
const Segment SGI173427[] = {{&S54014[30], 6}, {NULL}};
const Segment SGI173428[] = {{&S54014[120], 3}, {NULL}};
const Segment SGI173429[] = {{&S54014[85], 6}, {NULL}};
const Segment SGI173430[] = {{&S54015[38], 10}, {NULL}};
const Segment SGI173431[] = {{&S54015[31], 6}, {NULL}};
const Segment SGI173432[] = {{&S54016[44], 15}, {NULL}};
const Segment SGI173433[] = {{&S54016[44], 27}, {NULL}};
const Segment SGI173434[] = {{&S54016[30], 6}, {NULL}};
const Segment SGI173435[] = {{&S54016[120], 3}, {NULL}};
const Segment SGI173436[] = {{&S57337[34], 9}, {NULL}};
const Segment SGI173437[] = {{&S54016[85], 6}, {NULL}};
const Segment SGI173438[] = {{&S57337[27], 6}, {NULL}};
const Segment SGI173439[] = {{&S57337[96], 9}, {NULL}};
const Segment SGI173440[] = {{&S57337[69], 6}, {NULL}};
const Segment SGI173441[] = {{&S54017[44], 10}, {NULL}};
const Segment SGI173442[] = {{&S54017[44], 22}, {NULL}};
const Segment SGI173443[] = {{&S54017[30], 6}, {NULL}};
const Segment SGI173444[] = {{&S57338[34], 7}, {NULL}};
const Segment SGI173445[] = {{&S57338[27], 6}, {NULL}};
const Segment SGI173446[] = {{&S54017[115], 3}, {NULL}};
const Segment SGI173447[] = {{&S54017[80], 6}, {NULL}};
const Segment SGI173448[] = {{&S57338[94], 9}, {NULL}};
const Segment SGI173449[] = {{&S57338[67], 6}, {NULL}};
const Segment SGI173450[] = {{&S54018[44], 10}, {NULL}};
const Segment SGI173451[] = {{&S54018[44], 22}, {NULL}};
const Segment SGI173452[] = {{&S54018[30], 6}, {NULL}};
const Segment SGI173453[] = {{&S54018[115], 3}, {NULL}};
const Segment SGI173454[] = {{&S54018[80], 6}, {NULL}};
const Segment SGI173455[] = {{&S54019[44], 10}, {NULL}};
const Segment SGI173456[] = {{&S54019[44], 22}, {NULL}};
const Segment SGI173457[] = {{&S54019[30], 6}, {NULL}};
const Segment SGI173458[] = {{&S54019[115], 3}, {NULL}};
const Segment SGI173459[] = {{&S54019[80], 6}, {NULL}};
const Segment SGI173460[] = {{&S54020[38], 10}, {NULL}};
const Segment SGI173461[] = {{&S54020[31], 6}, {NULL}};
const Segment SGI173462[] = {{&S54021[38], 10}, {NULL}};
const Segment SGI173463[] = {{&S54021[31], 6}, {NULL}};
const Segment SGI173464[] = {{&S54022[44], 15}, {NULL}};
const Segment SGI173465[] = {{&S54022[44], 27}, {NULL}};
const Segment SGI173466[] = {{&S54022[30], 6}, {NULL}};
const Segment SGI173467[] = {{&S54022[120], 3}, {NULL}};
const Segment SGI173468[] = {{&S54022[85], 6}, {NULL}};
const Segment SGI173469[] = {{&S65148[44], 14}, {NULL}};
const Segment SGI173470[] = {{&S65148[44], 26}, {NULL}};
const Segment SGI173471[] = {{&S65148[30], 6}, {NULL}};
const Segment SGI173472[] = {{&S65148[31], 3}, {NULL}};
const Segment SGI173473[] = {{&S65148[84], 6}, {NULL}};
const Segment SGI173474[] = {{&S65153[34], 9}, {NULL}};
const Segment SGI173475[] = {{&S65153[27], 6}, {NULL}};
const Segment SGI173476[] = {{&S65153[96], 9}, {NULL}};
const Segment SGI173477[] = {{&S65153[69], 6}, {NULL}};
const Segment SGI173478[] = {{&S54043[34], 11}, {NULL}};
const Segment SGI173479[] = {{&S54043[27], 6}, {NULL}};
const Segment SGI173480[] = {{&S54043[98], 9}, {NULL}};
const Segment SGI173481[] = {{&S54043[71], 6}, {NULL}};
const Segment SGI173482[] = {{&S65166[30], 6}, {NULL}};
const Segment SGI173483[] = {{&S65166[85], 9}, {NULL}};
const Segment SGI173484[] = {{&S65166[57], 6}, {NULL}};
const Segment SGI173485[] = {{&S65169[44], 15}, {NULL}};
const Segment SGI173486[] = {{&S65169[44], 27}, {NULL}};
const Segment SGI173487[] = {{&S65169[30], 6}, {NULL}};
const Segment SGI173488[] = {{&S65169[31], 3}, {NULL}};
const Segment SGI173489[] = {{&S65169[85], 6}, {NULL}};
const Segment SGI173490[] = {{&S65170[38], 15}, {NULL}};
const Segment SGI173491[] = {{&S65170[31], 6}, {NULL}};
const Segment SGI173492[] = {{&S51736[44], 10}, {NULL}};
const Segment SGI173493[] = {{&S51736[44], 22}, {NULL}};
const Segment SGI173494[] = {{&S51736[30], 6}, {NULL}};
const Segment SGI173495[] = {{&S57383[44], 6}, {NULL}};
const Segment SGI173496[] = {{&S57383[44], 18}, {NULL}};
const Segment SGI173497[] = {{&S57383[30], 6}, {NULL}};
const Segment SGI173498[] = {{&S51736[133], 3}, {NULL}};
const Segment SGI173499[] = {{&S51736[98], 6}, {NULL}};
const Segment SGI173500[] = {{&S57383[111], 3}, {NULL}};
const Segment SGI173501[] = {{&S51736[68], 8}, {NULL}};
const Segment SGI173502[] = {{&S57383[76], 6}, {NULL}};
const Segment SGI173503[] = {{&S51737[44], 10}, {NULL}};
const Segment SGI173504[] = {{&S51737[44], 22}, {NULL}};
const Segment SGI173505[] = {{&S51737[30], 6}, {NULL}};
const Segment SGI173506[] = {{&S51737[133], 3}, {NULL}};
const Segment SGI173507[] = {{&S51737[98], 6}, {NULL}};
const Segment SGI173508[] = {{&S51737[68], 8}, {NULL}};
const Segment SGI173509[] = {{&S51739[44], 10}, {NULL}};
const Segment SGI173510[] = {{&S51739[44], 22}, {NULL}};
const Segment SGI173511[] = {{&S51739[30], 6}, {NULL}};
const Segment SGI173512[] = {{&S51739[133], 3}, {NULL}};
const Segment SGI173513[] = {{&S51739[98], 6}, {NULL}};
const Segment SGI173514[] = {{&S51739[68], 8}, {NULL}};
const Segment SGI173515[] = {{&S51740[42], 10}, {NULL}};
const Segment SGI173516[] = {{&S51740[30], 6}, {NULL}};
const Segment SGI173517[] = {{&S51740[131], 3}, {NULL}};
const Segment SGI173518[] = {{&S51740[96], 6}, {NULL}};
const Segment SGI173519[] = {{&S51740[66], 8}, {NULL}};
const Segment SGI173520[] = {{&S42576[34], 10}, {NULL}};
const Segment SGI173521[] = {{&S42576[27], 6}, {NULL}};
const Segment SGI173522[] = {{&S42576[97], 9}, {NULL}};
const Segment SGI173523[] = {{&S42576[70], 6}, {NULL}};
const Segment SGI173524[] = {{&S57402[34], 10}, {NULL}};
const Segment SGI173525[] = {{&S57402[27], 6}, {NULL}};
const Segment SGI173526[] = {{&S57402[97], 9}, {NULL}};
const Segment SGI173527[] = {{&S57402[70], 6}, {NULL}};
const Segment SGI173528[] = {{&S42577[34], 10}, {NULL}};
const Segment SGI173529[] = {{&S42577[27], 6}, {NULL}};
const Segment SGI173530[] = {{&S42577[97], 9}, {NULL}};
const Segment SGI173531[] = {{&S57403[34], 18}, {NULL}};
const Segment SGI173532[] = {{&S42577[70], 6}, {NULL}};
const Segment SGI173533[] = {{&S57403[27], 6}, {NULL}};
const Segment SGI173534[] = {{&S57403[105], 9}, {NULL}};
const Segment SGI173535[] = {{&S57403[78], 6}, {NULL}};
const Segment SGI173536[] = {{&S42578[34], 18}, {NULL}};
const Segment SGI173537[] = {{&S42578[27], 6}, {NULL}};
const Segment SGI173538[] = {{&S42578[105], 9}, {NULL}};
const Segment SGI173539[] = {{&S42578[78], 6}, {NULL}};
const Segment SGI173540[] = {{&S42579[34], 10}, {NULL}};
const Segment SGI173541[] = {{&S42579[27], 6}, {NULL}};
const Segment SGI173542[] = {{&S42579[96], 9}, {NULL}};
const Segment SGI173543[] = {{&S42579[69], 6}, {NULL}};
const Segment SGI173544[] = {{&S42580[34], 11}, {NULL}};
const Segment SGI173545[] = {{&S42580[27], 6}, {NULL}};
const Segment SGI173546[] = {{&S42580[98], 9}, {NULL}};
const Segment SGI173547[] = {{&S42580[71], 6}, {NULL}};
const Segment SGI173548[] = {{&S42582[34], 11}, {NULL}};
const Segment SGI173549[] = {{&S42582[27], 6}, {NULL}};
const Segment SGI173550[] = {{&S42582[98], 9}, {NULL}};
const Segment SGI173551[] = {{&S42582[71], 6}, {NULL}};
const Segment SGI173552[] = {{&S65217[44], 7}, {NULL}};
const Segment SGI173553[] = {{&S65217[44], 19}, {NULL}};
const Segment SGI173554[] = {{&S65217[30], 6}, {NULL}};
const Segment SGI173555[] = {{&S65217[31], 3}, {NULL}};
const Segment SGI173556[] = {{&S65217[77], 6}, {NULL}};
const Segment SGI173557[] = {{&S65218[38], 7}, {NULL}};
const Segment SGI173558[] = {{&S65218[31], 6}, {NULL}};
const Segment SGI173559[] = {{&S65220[34], 10}, {NULL}};
const Segment SGI173560[] = {{&S65220[27], 6}, {NULL}};
const Segment SGI173561[] = {{&S65220[97], 9}, {NULL}};
const Segment SGI173562[] = {{&S65220[70], 6}, {NULL}};
const Segment SGI173563[] = {{&S54110[34], 11}, {NULL}};
const Segment SGI173564[] = {{&S54110[27], 6}, {NULL}};
const Segment SGI173565[] = {{&S54110[98], 9}, {NULL}};
const Segment SGI173566[] = {{&S54110[71], 6}, {NULL}};
const Segment SGI173567[] = {{&S65225[44], 10}, {NULL}};
const Segment SGI173568[] = {{&S65225[44], 22}, {NULL}};
const Segment SGI173569[] = {{&S65225[30], 6}, {NULL}};
const Segment SGI173570[] = {{&S65225[31], 3}, {NULL}};
const Segment SGI173571[] = {{&S65225[80], 6}, {NULL}};
const Segment SGI173572[] = {{&S65226[44], 10}, {NULL}};
const Segment SGI173573[] = {{&S65226[44], 22}, {NULL}};
const Segment SGI173574[] = {{&S65226[30], 6}, {NULL}};
const Segment SGI173575[] = {{&S65226[31], 3}, {NULL}};
const Segment SGI173576[] = {{&S65226[80], 6}, {NULL}};
const Segment SGI173577[] = {{&S65228[38], 10}, {NULL}};
const Segment SGI173578[] = {{&S65228[31], 6}, {NULL}};
const Segment SGI173579[] = {{&S65233[44], 15}, {NULL}};
const Segment SGI173580[] = {{&S65233[44], 27}, {NULL}};
const Segment SGI173581[] = {{&S65233[30], 6}, {NULL}};
const Segment SGI173582[] = {{&S65233[31], 3}, {NULL}};
const Segment SGI173583[] = {{&S65233[85], 6}, {NULL}};
const Segment SGI173584[] = {{&S65234[44], 16}, {NULL}};
const Segment SGI173585[] = {{&S65234[44], 28}, {NULL}};
const Segment SGI173586[] = {{&S65234[30], 6}, {NULL}};
const Segment SGI173587[] = {{&S57444[6], 4}, {NULL}};
const Segment SGI173588[] = {{&S65234[31], 3}, {NULL}};
const Segment SGI173589[] = {{&S57444[83], 5}, {NULL}};
const Segment SGI173590[] = {{&S65234[86], 6}, {NULL}};
const Segment SGI173591[] = {{&S65235[38], 15}, {NULL}};
const Segment SGI173592[] = {{&S65235[31], 6}, {NULL}};
const Segment SGI173593[] = {{&S51799[44], 9}, {NULL}};
const Segment SGI173594[] = {{&S51799[44], 21}, {NULL}};
const Segment SGI173595[] = {{&S51799[30], 6}, {NULL}};
const Segment SGI173596[] = {{&S51799[132], 3}, {NULL}};
const Segment SGI173597[] = {{&S51799[97], 6}, {NULL}};
const Segment SGI173598[] = {{&S51799[67], 8}, {NULL}};
const Segment SGI173599[] = {{&S65241[34], 6}, {NULL}};
const Segment SGI173600[] = {{&S51801[44], 9}, {NULL}};
const Segment SGI173601[] = {{&S51801[44], 21}, {NULL}};
const Segment SGI173602[] = {{&S65241[27], 6}, {NULL}};
const Segment SGI173603[] = {{&S51801[30], 6}, {NULL}};
const Segment SGI173604[] = {{&S51801[132], 3}, {NULL}};
const Segment SGI173605[] = {{&S65241[93], 9}, {NULL}};
const Segment SGI173606[] = {{&S65241[66], 6}, {NULL}};
const Segment SGI173607[] = {{&S51801[97], 6}, {NULL}};
const Segment SGI173608[] = {{&S51801[67], 8}, {NULL}};
const Segment SGI173609[] = {{&S42622[34], 10}, {NULL}};
const Segment SGI173610[] = {{&S42622[27], 6}, {NULL}};
const Segment SGI173611[] = {{&S42622[96], 9}, {NULL}};
const Segment SGI173612[] = {{&S42622[69], 6}, {NULL}};
const Segment SGI173613[] = {{&S65245[34], 10}, {NULL}};
const Segment SGI173614[] = {{&S65245[27], 6}, {NULL}};
const Segment SGI173615[] = {{&S65245[97], 9}, {NULL}};
const Segment SGI173616[] = {{&S65245[70], 6}, {NULL}};
const Segment SGI173617[] = {{&S42623[34], 10}, {NULL}};
const Segment SGI173618[] = {{&S42623[27], 6}, {NULL}};
const Segment SGI173619[] = {{&S42623[96], 9}, {NULL}};
const Segment SGI173620[] = {{&S42623[69], 6}, {NULL}};
const Segment SGI173621[] = {{&S65246[34], 10}, {NULL}};
const Segment SGI173622[] = {{&S65246[27], 6}, {NULL}};
const Segment SGI173623[] = {{&S65246[97], 9}, {NULL}};
const Segment SGI173624[] = {{&S65246[70], 6}, {NULL}};
const Segment SGI173625[] = {{&S65247[34], 10}, {NULL}};
const Segment SGI173626[] = {{&S65247[27], 6}, {NULL}};
const Segment SGI173627[] = {{&S65247[97], 9}, {NULL}};
const Segment SGI173628[] = {{&S65247[70], 6}, {NULL}};
const Segment SGI173629[] = {{&S42632[44], 8}, {NULL}};
const Segment SGI173630[] = {{&S42632[44], 19}, {NULL}};
const Segment SGI173631[] = {{&S42632[30], 6}, {NULL}};
const Segment SGI173632[] = {{&S42632[112], 3}, {NULL}};
const Segment SGI173633[] = {{&S42632[77], 6}, {NULL}};
const Segment SGI173634[] = {{&S42633[38], 8}, {NULL}};
const Segment SGI173635[] = {{&S42633[31], 6}, {NULL}};
const Segment SGI173636[] = {{&S54139[34], 6}, {NULL}};
const Segment SGI173637[] = {{&S54139[27], 6}, {NULL}};
const Segment SGI173638[] = {{&S54139[93], 9}, {NULL}};
const Segment SGI173639[] = {{&S54139[66], 6}, {NULL}};
const Segment SGI173640[] = {{&S65260[34], 11}, {NULL}};
const Segment SGI173641[] = {{&S65260[27], 6}, {NULL}};
const Segment SGI173642[] = {{&S65260[98], 9}, {NULL}};
const Segment SGI173643[] = {{&S65260[71], 6}, {NULL}};
const Segment SGI173644[] = {{&S65261[34], 11}, {NULL}};
const Segment SGI173645[] = {{&S65261[27], 6}, {NULL}};
const Segment SGI173646[] = {{&S65261[98], 9}, {NULL}};
const Segment SGI173647[] = {{&S65261[71], 6}, {NULL}};
const Segment SGI173648[] = {{&S54154[34], 10}, {NULL}};
const Segment SGI173649[] = {{&S54154[27], 6}, {NULL}};
const Segment SGI173650[] = {{&S54154[97], 9}, {NULL}};
const Segment SGI173651[] = {{&S54154[70], 6}, {NULL}};
const Segment SGI173652[] = {{&S54155[34], 11}, {NULL}};
const Segment SGI173653[] = {{&S54155[27], 6}, {NULL}};
const Segment SGI173654[] = {{&S54155[98], 9}, {NULL}};
const Segment SGI173655[] = {{&S54155[71], 6}, {NULL}};
const Segment SGI173656[] = {{&S65280[34], 10}, {NULL}};
const Segment SGI173657[] = {{&S65280[27], 6}, {NULL}};
const Segment SGI173658[] = {{&S65280[101], 9}, {NULL}};
const Segment SGI173659[] = {{&S65280[74], 6}, {NULL}};
const Segment SGI173660[] = {{&S54179[34], 7}, {NULL}};
const Segment SGI173661[] = {{&S54179[27], 6}, {NULL}};
const Segment SGI173662[] = {{&S54179[100], 9}, {NULL}};
const Segment SGI173663[] = {{&S54179[73], 6}, {NULL}};
const Segment SGI173664[] = {{&S42727[11], 1}, {NULL}};
const Segment SGI173665[] = {{&S42727[58], 17}, {NULL}};
const Segment SGI173666[] = {{&S42727[30], 20}, {NULL}};
const Segment SGI173667[] = {{&S42727[124], 3}, {NULL}};
const Segment SGI173668[] = {{&S42727[89], 6}, {NULL}};
const Segment SGI173669[] = {{&S42729[11], 1}, {NULL}};
const Segment SGI173670[] = {{&S42729[44], 13}, {NULL}};
const Segment SGI173671[] = {{&S42729[30], 6}, {NULL}};
const Segment SGI173672[] = {{&S42729[106], 3}, {NULL}};
const Segment SGI173673[] = {{&S42729[71], 6}, {NULL}};
const Segment SGI173674[] = {{&S42743[44], 16}, {NULL}};
const Segment SGI173675[] = {{&S42743[44], 44}, {NULL}};
const Segment SGI173676[] = {{&S42743[30], 6}, {NULL}};
const Segment SGI173677[] = {{&S42743[137], 3}, {NULL}};
const Segment SGI173678[] = {{&S42743[102], 6}, {NULL}};
const Segment SGI173679[] = {{&S42745[44], 9}, {NULL}};
const Segment SGI173680[] = {{&S42745[44], 21}, {NULL}};
const Segment SGI173681[] = {{&S42745[30], 6}, {NULL}};
const Segment SGI173682[] = {{&S42745[114], 3}, {NULL}};
const Segment SGI173683[] = {{&S42745[79], 6}, {NULL}};
const Segment SGI173684[] = {{&S42751[38], 9}, {NULL}};
const Segment SGI173685[] = {{&S42751[31], 6}, {NULL}};
const Segment SGI173686[] = {{&S51948[44], 10}, {NULL}};
const Segment SGI173687[] = {{&S51948[44], 22}, {NULL}};
const Segment SGI173688[] = {{&S65378[34], 10}, {NULL}};
const Segment SGI173689[] = {{&S51948[30], 6}, {NULL}};
const Segment SGI173690[] = {{&S65378[27], 6}, {NULL}};
const Segment SGI173691[] = {{&S51948[133], 3}, {NULL}};
const Segment SGI173692[] = {{&S65378[97], 9}, {NULL}};
const Segment SGI173693[] = {{&S51948[98], 6}, {NULL}};
const Segment SGI173694[] = {{&S65378[70], 6}, {NULL}};
const Segment SGI173695[] = {{&S51948[68], 8}, {NULL}};
const Segment SGI173696[] = {{&S42753[11], 1}, {NULL}};
const Segment SGI173697[] = {{&S42753[44], 13}, {NULL}};
const Segment SGI173698[] = {{&S42753[30], 6}, {NULL}};
const Segment SGI173699[] = {{&S42753[106], 3}, {NULL}};
const Segment SGI173700[] = {{&S51949[44], 10}, {NULL}};
const Segment SGI173701[] = {{&S42753[71], 6}, {NULL}};
const Segment SGI173702[] = {{&S51949[44], 22}, {NULL}};
const Segment SGI173703[] = {{&S51949[30], 6}, {NULL}};
const Segment SGI173704[] = {{&S51949[133], 3}, {NULL}};
const Segment SGI173705[] = {{&S51949[98], 6}, {NULL}};
const Segment SGI173706[] = {{&S51949[68], 8}, {NULL}};
const Segment SGI173707[] = {{&S42754[11], 1}, {NULL}};
const Segment SGI173708[] = {{&S42754[44], 13}, {NULL}};
const Segment SGI173709[] = {{&S42754[30], 6}, {NULL}};
const Segment SGI173710[] = {{&S42754[106], 3}, {NULL}};
const Segment SGI173711[] = {{&S42754[71], 6}, {NULL}};
const Segment SGI173712[] = {{&S54258[34], 10}, {NULL}};
const Segment SGI173713[] = {{&S54258[27], 6}, {NULL}};
const Segment SGI173714[] = {{&S42756[44], 9}, {NULL}};
const Segment SGI173715[] = {{&S42756[44], 20}, {NULL}};
const Segment SGI173716[] = {{&S42756[30], 6}, {NULL}};
const Segment SGI173717[] = {{&S54258[97], 9}, {NULL}};
const Segment SGI173718[] = {{&S54258[70], 6}, {NULL}};
const Segment SGI173719[] = {{&S42756[113], 3}, {NULL}};
const Segment SGI173720[] = {{&S42756[78], 6}, {NULL}};
const Segment SGI173721[] = {{&S42757[11], 1}, {NULL}};
const Segment SGI173722[] = {{&S42757[44], 13}, {NULL}};
const Segment SGI173723[] = {{&S42757[30], 6}, {NULL}};
const Segment SGI173724[] = {{&S42757[106], 3}, {NULL}};
const Segment SGI173725[] = {{&S42757[71], 6}, {NULL}};
const Segment SGI173726[] = {{&S42758[11], 1}, {NULL}};
const Segment SGI173727[] = {{&S57456[38], 11}, {NULL}};
const Segment SGI173728[] = {{&S42758[44], 13}, {NULL}};
const Segment SGI173729[] = {{&S57456[31], 6}, {NULL}};
const Segment SGI173730[] = {{&S42758[30], 6}, {NULL}};
const Segment SGI173731[] = {{&S42758[106], 3}, {NULL}};
const Segment SGI173732[] = {{&S42758[71], 6}, {NULL}};
const Segment SGI173733[] = {{&S57457[44], 11}, {NULL}};
const Segment SGI173734[] = {{&S57457[44], 23}, {NULL}};
const Segment SGI173735[] = {{&S42759[22], 9}, {NULL}};
const Segment SGI173736[] = {{&S57457[30], 6}, {NULL}};
const Segment SGI173737[] = {{&S42759[44], 20}, {NULL}};
const Segment SGI173738[] = {{&S42759[30], 6}, {&S42759[52], 12}, {NULL}};
const Segment SGI173739[] = {{&S57457[116], 3}, {NULL}};
const Segment SGI173740[] = {{&S42759[113], 3}, {NULL}};
const Segment SGI173741[] = {{&S57457[81], 6}, {NULL}};
const Segment SGI173742[] = {{&S42759[78], 6}, {NULL}};
const Segment SGI173743[] = {{&S42760[44], 5}, {NULL}};
const Segment SGI173744[] = {{&S42760[44], 16}, {NULL}};
const Segment SGI173745[] = {{&S42760[30], 6}, {NULL}};
const Segment SGI173746[] = {{&S42760[109], 3}, {NULL}};
const Segment SGI173747[] = {{&S42760[74], 6}, {NULL}};
const Segment SGI173748[] = {{&S57461[44], 19}, {NULL}};
const Segment SGI173749[] = {{&S57461[44], 31}, {NULL}};
const Segment SGI173750[] = {{&S57461[30], 6}, {NULL}};
const Segment SGI173751[] = {{&S57461[124], 3}, {NULL}};
const Segment SGI173752[] = {{&S57461[89], 6}, {NULL}};
const Segment SGI173753[] = {{&S57462[44], 16}, {NULL}};
const Segment SGI173754[] = {{&S57462[44], 28}, {NULL}};
const Segment SGI173755[] = {{&S57462[30], 6}, {NULL}};
const Segment SGI173756[] = {{&S42764[44], 9}, {NULL}};
const Segment SGI173757[] = {{&S42764[44], 20}, {NULL}};
const Segment SGI173758[] = {{&S42764[30], 6}, {NULL}};
const Segment SGI173759[] = {{&S57462[121], 3}, {NULL}};
const Segment SGI173760[] = {{&S57462[86], 6}, {NULL}};
const Segment SGI173761[] = {{&S42764[113], 3}, {NULL}};
const Segment SGI173762[] = {{&S42764[78], 6}, {NULL}};
const Segment SGI173763[] = {{&S57463[44], 9}, {NULL}};
const Segment SGI173764[] = {{&S57463[44], 21}, {NULL}};
const Segment SGI173765[] = {{&S57463[30], 6}, {NULL}};
const Segment SGI173766[] = {{&S57463[114], 3}, {NULL}};
const Segment SGI173767[] = {{&S57463[79], 6}, {NULL}};
const Segment SGI173768[] = {{&S57464[38], 9}, {NULL}};
const Segment SGI173769[] = {{&S57464[31], 6}, {NULL}};
const Segment SGI173770[] = {{&S57465[44], 19}, {NULL}};
const Segment SGI173771[] = {{&S57465[44], 42}, {NULL}};
const Segment SGI173772[] = {{&S57465[30], 6}, {NULL}};
const Segment SGI173773[] = {{&S57465[135], 3}, {NULL}};
const Segment SGI173774[] = {{&S57465[100], 6}, {NULL}};
const Segment SGI173775[] = {{&S57466[44], 20}, {NULL}};
const Segment SGI173776[] = {{&S57466[44], 32}, {NULL}};
const Segment SGI173777[] = {{&S57466[30], 6}, {NULL}};
const Segment SGI173778[] = {{&S57466[125], 3}, {NULL}};
const Segment SGI173779[] = {{&S57466[90], 6}, {NULL}};
const Segment SGI173780[] = {{&S65417[34], 7}, {NULL}};
const Segment SGI173781[] = {{&S65417[27], 6}, {NULL}};
const Segment SGI173782[] = {{&S65417[99], 9}, {NULL}};
const Segment SGI173783[] = {{&S65417[72], 6}, {NULL}};
const Segment SGI173784[] = {{&S57477[34], 9}, {NULL}};
const Segment SGI173785[] = {{&S57477[27], 6}, {NULL}};
const Segment SGI173786[] = {{&S57477[96], 9}, {NULL}};
const Segment SGI173787[] = {{&S57477[69], 6}, {NULL}};
const Segment SGI173788[] = {{&S65419[34], 6}, {NULL}};
const Segment SGI173789[] = {{&S65419[27], 6}, {NULL}};
const Segment SGI173790[] = {{&S65419[93], 9}, {NULL}};
const Segment SGI173791[] = {{&S65419[66], 6}, {NULL}};
const Segment SGI173792[] = {{&S57482[34], 10}, {NULL}};
const Segment SGI173793[] = {{&S57482[27], 6}, {NULL}};
const Segment SGI173794[] = {{&S57482[97], 9}, {NULL}};
const Segment SGI173795[] = {{&S57482[70], 6}, {NULL}};
const Segment SGI173796[] = {{&S65437[34], 15}, {NULL}};
const Segment SGI173797[] = {{&S65437[27], 6}, {NULL}};
const Segment SGI173798[] = {{&S65437[102], 9}, {NULL}};
const Segment SGI173799[] = {{&S65437[75], 6}, {NULL}};
const Segment SGI173800[] = {{&S54317[34], 7}, {NULL}};
const Segment SGI173801[] = {{&S54317[27], 6}, {NULL}};
const Segment SGI173802[] = {{&S54317[98], 9}, {NULL}};
const Segment SGI173803[] = {{&S54317[71], 6}, {NULL}};
const Segment SGI173804[] = {{&S54322[44], 7}, {NULL}};
const Segment SGI173805[] = {{&S54322[44], 19}, {NULL}};
const Segment SGI173806[] = {{&S54322[30], 6}, {NULL}};
const Segment SGI173807[] = {{&S54322[128], 3}, {NULL}};
const Segment SGI173808[] = {{&S54322[93], 6}, {NULL}};
const Segment SGI173809[] = {{&S54322[65], 8}, {NULL}};
const Segment SGI173810[] = {{&S65466[34], 10}, {NULL}};
const Segment SGI173811[] = {{&S65466[27], 6}, {NULL}};
const Segment SGI173812[] = {{&S65466[97], 9}, {NULL}};
const Segment SGI173813[] = {{&S65466[70], 6}, {NULL}};
const Segment SGI173814[] = {{&S57530[44], 11}, {NULL}};
const Segment SGI173815[] = {{&S57530[44], 23}, {NULL}};
const Segment SGI173816[] = {{&S57530[30], 6}, {NULL}};
const Segment SGI173817[] = {{&S57530[132], 3}, {NULL}};
const Segment SGI173818[] = {{&S57530[97], 6}, {NULL}};
const Segment SGI173819[] = {{&S57530[69], 8}, {NULL}};
const Segment SGI173820[] = {{&S57531[44], 11}, {NULL}};
const Segment SGI173821[] = {{&S65468[44], 7}, {NULL}};
const Segment SGI173822[] = {{&S57531[44], 23}, {NULL}};
const Segment SGI173823[] = {{&S65468[44], 19}, {NULL}};
const Segment SGI173824[] = {{&S57531[30], 6}, {NULL}};
const Segment SGI173825[] = {{&S65468[30], 6}, {NULL}};
const Segment SGI173826[] = {{&S57531[132], 3}, {NULL}};
const Segment SGI173827[] = {{&S65468[31], 3}, {NULL}};
const Segment SGI173828[] = {{&S57531[97], 6}, {NULL}};
const Segment SGI173829[] = {{&S65468[77], 6}, {NULL}};
const Segment SGI173830[] = {{&S57531[69], 8}, {NULL}};
const Segment SGI173831[] = {{&S65469[38], 7}, {NULL}};
const Segment SGI173832[] = {{&S65469[31], 6}, {NULL}};
const Segment SGI173833[] = {{&S65474[45], 10}, {NULL}};
const Segment SGI173834[] = {{&S65474[30], 6}, {NULL}};
const Segment SGI173835[] = {{&S65474[31], 3}, {NULL}};
const Segment SGI173836[] = {{&S65474[81], 5}, {NULL}};
const Segment SGI173837[] = {{&S65474[139], 13}, {NULL}};
const Segment SGI173838[] = {{&S54355[44], 10}, {NULL}};
const Segment SGI173839[] = {{&S54355[44], 22}, {NULL}};
const Segment SGI173840[] = {{&S54355[30], 6}, {NULL}};
const Segment SGI173841[] = {{&S54355[115], 3}, {NULL}};
const Segment SGI173842[] = {{&S54355[80], 6}, {NULL}};
const Segment SGI173843[] = {{&S54355[140], 11}, {NULL}};
const Segment SGI173844[] = {{&S54356[44], 10}, {NULL}};
const Segment SGI173845[] = {{&S54356[44], 22}, {NULL}};
const Segment SGI173846[] = {{&S54356[30], 6}, {NULL}};
const Segment SGI173847[] = {{&S54356[115], 3}, {NULL}};
const Segment SGI173848[] = {{&S54356[80], 6}, {NULL}};
const Segment SGI173849[] = {{&S54356[140], 11}, {NULL}};
const Segment SGI173850[] = {{&S54357[44], 10}, {NULL}};
const Segment SGI173851[] = {{&S54357[44], 22}, {NULL}};
const Segment SGI173852[] = {{&S54357[30], 6}, {NULL}};
const Segment SGI173853[] = {{&S54357[115], 3}, {NULL}};
const Segment SGI173854[] = {{&S54357[80], 6}, {NULL}};
const Segment SGI173855[] = {{&S54357[140], 11}, {NULL}};
const Segment SGI173856[] = {{&S54358[44], 11}, {NULL}};
const Segment SGI173857[] = {{&S54358[44], 23}, {NULL}};
const Segment SGI173858[] = {{&S54358[30], 6}, {NULL}};
const Segment SGI173859[] = {{&S54358[116], 3}, {NULL}};
const Segment SGI173860[] = {{&S54358[81], 6}, {NULL}};
const Segment SGI173861[] = {{&S54358[141], 11}, {NULL}};
const Segment SGI173862[] = {{&S54359[44], 10}, {NULL}};
const Segment SGI173863[] = {{&S54359[44], 22}, {NULL}};
const Segment SGI173864[] = {{&S54359[30], 6}, {NULL}};
const Segment SGI173865[] = {{&S65484[34], 20}, {NULL}};
const Segment SGI173866[] = {{&S54359[115], 3}, {NULL}};
const Segment SGI173867[] = {{&S65484[27], 6}, {NULL}};
const Segment SGI173868[] = {{&S54359[80], 6}, {NULL}};
const Segment SGI173869[] = {{&S54359[141], 12}, {NULL}};
const Segment SGI173870[] = {{&S65484[123], 9}, {NULL}};
const Segment SGI173871[] = {{&S65484[96], 6}, {NULL}};
const Segment SGI173872[] = {{&S57567[44], 17}, {NULL}};
const Segment SGI173873[] = {{&S57567[44], 29}, {NULL}};
const Segment SGI173874[] = {{&S57567[30], 6}, {NULL}};
const Segment SGI173875[] = {{&S57567[122], 3}, {NULL}};
const Segment SGI173876[] = {{&S57567[87], 6}, {NULL}};
const Segment SGI173877[] = {{&S57568[44], 19}, {NULL}};
const Segment SGI173878[] = {{&S57568[44], 31}, {NULL}};
const Segment SGI173879[] = {{&S57568[30], 6}, {NULL}};
const Segment SGI173880[] = {{&S57568[124], 3}, {NULL}};
const Segment SGI173881[] = {{&S57568[89], 6}, {NULL}};
const Segment SGI173882[] = {{&S57569[38], 17}, {NULL}};
const Segment SGI173883[] = {{&S57569[31], 6}, {NULL}};
const Segment SGI173884[] = {{&S57575[34], 11}, {NULL}};
const Segment SGI173885[] = {{&S57575[27], 6}, {NULL}};
const Segment SGI173886[] = {{&S57575[98], 9}, {NULL}};
const Segment SGI173887[] = {{&S57575[71], 6}, {NULL}};
const Segment SGI173888[] = {{&S54385[34], 24}, {NULL}};
const Segment SGI173889[] = {{&S54385[27], 6}, {NULL}};
const Segment SGI173890[] = {{&S54385[111], 9}, {NULL}};
const Segment SGI173891[] = {{&S54385[84], 6}, {NULL}};
const Segment SGI173892[] = {{&S65514[34], 14}, {NULL}};
const Segment SGI173893[] = {{&S65514[27], 6}, {NULL}};
const Segment SGI173894[] = {{&S65514[101], 9}, {NULL}};
const Segment SGI173895[] = {{&S65514[74], 6}, {NULL}};
const Segment SGI173896[] = {{&S65521[6], 4}, {NULL}};
const Segment SGI173897[] = {{&S65521[82], 5}, {NULL}};
const Segment SGI173898[] = {{&S54399[34], 8}, {NULL}};
const Segment SGI173899[] = {{&S54399[27], 6}, {NULL}};
const Segment SGI173900[] = {{&S54399[103], 9}, {NULL}};
const Segment SGI173901[] = {{&S54399[76], 6}, {NULL}};
const Segment SGI173902[] = {{&S65529[34], 5}, {NULL}};
const Segment SGI173903[] = {{&S65529[27], 6}, {NULL}};
const Segment SGI173904[] = {{&S65529[92], 9}, {NULL}};
const Segment SGI173905[] = {{&S65529[65], 6}, {NULL}};
const Segment SGI173906[] = {{&S54405[34], 18}, {NULL}};
const Segment SGI173907[] = {{&S54405[27], 6}, {NULL}};
const Segment SGI173908[] = {{&S54405[105], 9}, {NULL}};
const Segment SGI173909[] = {{&S54405[78], 6}, {NULL}};
const Segment SGI173910[] = {{&S57602[34], 7}, {NULL}};
const Segment SGI173911[] = {{&S57602[27], 6}, {NULL}};
const Segment SGI173912[] = {{&S57602[100], 9}, {NULL}};
const Segment SGI173913[] = {{&S57602[73], 6}, {NULL}};
const Segment SGI173914[] = {{&S54408[44], 8}, {NULL}};
const Segment SGI173915[] = {{&S54408[44], 28}, {NULL}};
const Segment SGI173916[] = {{&S54408[30], 6}, {NULL}};
const Segment SGI173917[] = {{&S54408[121], 3}, {NULL}};
const Segment SGI173918[] = {{&S54408[86], 6}, {NULL}};
const Segment SGI173919[] = {{&S54409[44], 18}, {NULL}};
const Segment SGI173920[] = {{&S54409[44], 38}, {NULL}};
const Segment SGI173921[] = {{&S54409[30], 6}, {NULL}};
const Segment SGI173922[] = {{&S54409[131], 3}, {NULL}};
const Segment SGI173923[] = {{&S54409[96], 6}, {NULL}};
const Segment SGI173924[] = {{&S52124[44], 18}, {NULL}};
const Segment SGI173925[] = {{&S52124[44], 30}, {NULL}};
const Segment SGI173926[] = {{&S52124[30], 6}, {NULL}};
const Segment SGI173927[] = {{&S52124[141], 3}, {NULL}};
const Segment SGI173928[] = {{&S52124[106], 6}, {NULL}};
const Segment SGI173929[] = {{&S52124[76], 8}, {NULL}};
const Segment SGI173930[] = {{&S54410[44], 18}, {NULL}};
const Segment SGI173931[] = {{&S54410[44], 38}, {NULL}};
const Segment SGI173932[] = {{&S54410[30], 6}, {NULL}};
const Segment SGI173933[] = {{&S54410[131], 3}, {NULL}};
const Segment SGI173934[] = {{&S54410[96], 6}, {NULL}};
const Segment SGI173935[] = {{&S65540[55], 6}, {NULL}};
const Segment SGI173936[] = {{&S65540[117], 6}, {NULL}};
const Segment SGI173937[] = {{&S54411[44], 18}, {NULL}};
const Segment SGI173938[] = {{&S54411[44], 38}, {NULL}};
const Segment SGI173939[] = {{&S54411[30], 6}, {NULL}};
const Segment SGI173940[] = {{&S54411[131], 3}, {NULL}};
const Segment SGI173941[] = {{&S54411[96], 6}, {NULL}};
const Segment SGI173942[] = {{&S65541[44], 10}, {NULL}};
const Segment SGI173943[] = {{&S65541[44], 30}, {NULL}};
const Segment SGI173944[] = {{&S65541[30], 6}, {NULL}};
const Segment SGI173945[] = {{&S65541[91], 6}, {NULL}};
const Segment SGI173946[] = {{&S54412[44], 15}, {NULL}};
const Segment SGI173947[] = {{&S54412[44], 35}, {NULL}};
const Segment SGI173948[] = {{&S54412[30], 6}, {NULL}};
const Segment SGI173949[] = {{&S54412[128], 3}, {NULL}};
const Segment SGI173950[] = {{&S54412[93], 6}, {NULL}};
const Segment SGI173951[] = {{&S65542[44], 10}, {NULL}};
const Segment SGI173952[] = {{&S65542[44], 30}, {NULL}};
const Segment SGI173953[] = {{&S65542[30], 6}, {NULL}};
const Segment SGI173954[] = {{&S65542[31], 3}, {NULL}};
const Segment SGI173955[] = {{&S54413[38], 8}, {NULL}};
const Segment SGI173956[] = {{&S65542[88], 6}, {NULL}};
const Segment SGI173957[] = {{&S54413[31], 6}, {NULL}};
const Segment SGI173958[] = {{&S65543[38], 10}, {NULL}};
const Segment SGI173959[] = {{&S54414[38], 18}, {NULL}};
const Segment SGI173960[] = {{&S65543[31], 6}, {NULL}};
const Segment SGI173961[] = {{&S54414[31], 6}, {NULL}};
const Segment SGI173962[] = {{&S54416[44], 10}, {NULL}};
const Segment SGI173963[] = {{&S54416[44], 22}, {NULL}};
const Segment SGI173964[] = {{&S54416[30], 6}, {NULL}};
const Segment SGI173965[] = {{&S54416[80], 6}, {NULL}};
const Segment SGI173966[] = {{&S54416[107], 13}, {NULL}};
const Segment SGI173967[] = {{&S54417[44], 10}, {NULL}};
const Segment SGI173968[] = {{&S54417[44], 22}, {NULL}};
const Segment SGI173969[] = {{&S54417[30], 6}, {NULL}};
const Segment SGI173970[] = {{&S54417[80], 6}, {NULL}};
const Segment SGI173971[] = {{&S54417[107], 13}, {NULL}};
const Segment SGI173972[] = {{&S54419[44], 10}, {NULL}};
const Segment SGI173973[] = {{&S54419[44], 22}, {NULL}};
const Segment SGI173974[] = {{&S54419[30], 6}, {NULL}};
const Segment SGI173975[] = {{&S54419[80], 6}, {NULL}};
const Segment SGI173976[] = {{&S54419[107], 13}, {NULL}};
const Segment SGI173977[] = {{&S54422[34], 10}, {NULL}};
const Segment SGI173978[] = {{&S54422[27], 6}, {NULL}};
const Segment SGI173979[] = {{&S54422[97], 9}, {NULL}};
const Segment SGI173980[] = {{&S54422[70], 6}, {NULL}};
const Segment SGI173981[] = {{&S54423[34], 10}, {NULL}};
const Segment SGI173982[] = {{&S54423[27], 6}, {NULL}};
const Segment SGI173983[] = {{&S54423[97], 9}, {NULL}};
const Segment SGI173984[] = {{&S54423[70], 6}, {NULL}};
const Segment SGI173985[] = {{&S65558[44], 24}, {NULL}};
const Segment SGI173986[] = {{&S65558[44], 36}, {NULL}};
const Segment SGI173987[] = {{&S65558[30], 6}, {NULL}};
const Segment SGI173988[] = {{&S65558[31], 3}, {NULL}};
const Segment SGI173989[] = {{&S65558[94], 6}, {NULL}};
const Segment SGI173990[] = {{&S65578[34], 10}, {NULL}};
const Segment SGI173991[] = {{&S65578[27], 6}, {NULL}};
const Segment SGI173992[] = {{&S65578[97], 9}, {NULL}};
const Segment SGI173993[] = {{&S65578[70], 6}, {NULL}};
const Segment SGI173994[] = {{&S65578[138], 8}, {NULL}};
const Segment SGI173995[] = {{&S65579[34], 14}, {NULL}};
const Segment SGI173996[] = {{&S65579[27], 6}, {NULL}};
const Segment SGI173997[] = {{&S65579[101], 9}, {NULL}};
const Segment SGI173998[] = {{&S65579[74], 6}, {NULL}};
const Segment SGI173999[] = {{&S65579[142], 8}, {NULL}};
const Segment SGI174000[] = {{&S65580[34], 13}, {NULL}};
const Segment SGI174001[] = {{&S65580[27], 6}, {NULL}};
const Segment SGI174002[] = {{&S57657[34], 9}, {NULL}};
const Segment SGI174003[] = {{&S57657[27], 6}, {NULL}};
const Segment SGI174004[] = {{&S65580[100], 9}, {NULL}};
const Segment SGI174005[] = {{&S65580[73], 6}, {NULL}};
const Segment SGI174006[] = {{&S57657[96], 9}, {NULL}};
const Segment SGI174007[] = {{&S57657[69], 6}, {NULL}};
const Segment SGI174008[] = {{&S65580[141], 8}, {NULL}};
const Segment SGI174009[] = {{&S65581[41], 7}, {NULL}};
const Segment SGI174010[] = {{&S65581[41], 19}, {NULL}};
const Segment SGI174011[] = {{&S65581[101], 9}, {NULL}};
const Segment SGI174012[] = {{&S65581[74], 6}, {NULL}};
const Segment SGI174013[] = {{&S65583[34], 10}, {NULL}};
const Segment SGI174014[] = {{&S65583[27], 6}, {NULL}};
const Segment SGI174015[] = {{&S65583[97], 9}, {NULL}};
const Segment SGI174016[] = {{&S65583[70], 6}, {NULL}};
const Segment SGI174017[] = {{&S65585[44], 8}, {NULL}};
const Segment SGI174018[] = {{&S65585[44], 20}, {NULL}};
const Segment SGI174019[] = {{&S65585[30], 6}, {NULL}};
const Segment SGI174020[] = {{&S65585[31], 3}, {NULL}};
const Segment SGI174021[] = {{&S65585[78], 6}, {NULL}};
const Segment SGI174022[] = {{&S65587[44], 8}, {NULL}};
const Segment SGI174023[] = {{&S65587[44], 20}, {NULL}};
const Segment SGI174024[] = {{&S65587[30], 6}, {NULL}};
const Segment SGI174025[] = {{&S65587[31], 3}, {NULL}};
const Segment SGI174026[] = {{&S65587[78], 6}, {NULL}};
const Segment SGI174027[] = {{&S65589[44], 8}, {NULL}};
const Segment SGI174028[] = {{&S65589[44], 20}, {NULL}};
const Segment SGI174029[] = {{&S65589[30], 6}, {NULL}};
const Segment SGI174030[] = {{&S65589[31], 3}, {NULL}};
const Segment SGI174031[] = {{&S65589[78], 6}, {NULL}};
const Segment SGI174032[] = {{&S65590[44], 9}, {NULL}};
const Segment SGI174033[] = {{&S65590[44], 21}, {NULL}};
const Segment SGI174034[] = {{&S65590[30], 6}, {NULL}};
const Segment SGI174035[] = {{&S65590[31], 3}, {NULL}};
const Segment SGI174036[] = {{&S65590[79], 6}, {NULL}};
const Segment SGI174037[] = {{&S65592[34], 10}, {NULL}};
const Segment SGI174038[] = {{&S65592[27], 6}, {NULL}};
const Segment SGI174039[] = {{&S65592[97], 9}, {NULL}};
const Segment SGI174040[] = {{&S65592[70], 6}, {NULL}};
const Segment SGI174041[] = {{&S54475[34], 10}, {NULL}};
const Segment SGI174042[] = {{&S54475[27], 6}, {NULL}};
const Segment SGI174043[] = {{&S54475[97], 9}, {NULL}};
const Segment SGI174044[] = {{&S54475[70], 6}, {NULL}};
const Segment SGI174045[] = {{&S54476[34], 10}, {NULL}};
const Segment SGI174046[] = {{&S54476[27], 6}, {NULL}};
const Segment SGI174047[] = {{&S54476[97], 9}, {NULL}};
const Segment SGI174048[] = {{&S54476[70], 6}, {NULL}};
const Segment SGI174049[] = {{&S65599[34], 10}, {NULL}};
const Segment SGI174050[] = {{&S65599[27], 6}, {NULL}};
const Segment SGI174051[] = {{&S65599[97], 9}, {NULL}};
const Segment SGI174052[] = {{&S65599[70], 6}, {NULL}};
const Segment SGI174053[] = {{&S65600[34], 6}, {NULL}};
const Segment SGI174054[] = {{&S65600[27], 6}, {NULL}};
const Segment SGI174055[] = {{&S65600[93], 9}, {NULL}};
const Segment SGI174056[] = {{&S65600[66], 6}, {NULL}};
const Segment SGI174057[] = {{&S54479[44], 11}, {NULL}};
const Segment SGI174058[] = {{&S54479[44], 28}, {NULL}};
const Segment SGI174059[] = {{&S54479[30], 6}, {NULL}};
const Segment SGI174060[] = {{&S54479[121], 3}, {NULL}};
const Segment SGI174061[] = {{&S54479[86], 6}, {NULL}};
const Segment SGI174062[] = {{&S54480[44], 7}, {NULL}};
const Segment SGI174063[] = {{&S54480[44], 24}, {NULL}};
const Segment SGI174064[] = {{&S54480[30], 6}, {NULL}};
const Segment SGI174065[] = {{&S54480[117], 3}, {NULL}};
const Segment SGI174066[] = {{&S54480[82], 6}, {NULL}};
const Segment SGI174067[] = {{&S54481[44], 11}, {NULL}};
const Segment SGI174068[] = {{&S54481[44], 28}, {NULL}};
const Segment SGI174069[] = {{&S54481[30], 6}, {NULL}};
const Segment SGI174070[] = {{&S54481[121], 3}, {NULL}};
const Segment SGI174071[] = {{&S65605[34], 14}, {NULL}};
const Segment SGI174072[] = {{&S54481[86], 6}, {NULL}};
const Segment SGI174073[] = {{&S65605[27], 6}, {NULL}};
const Segment SGI174074[] = {{&S65605[101], 9}, {NULL}};
const Segment SGI174075[] = {{&S65605[74], 6}, {NULL}};
const Segment SGI174076[] = {{&S54482[38], 7}, {NULL}};
const Segment SGI174077[] = {{&S54482[31], 6}, {NULL}};
const Segment SGI174078[] = {{&S54483[44], 7}, {NULL}};
const Segment SGI174079[] = {{&S54483[44], 24}, {NULL}};
const Segment SGI174080[] = {{&S54483[30], 6}, {NULL}};
const Segment SGI174081[] = {{&S54483[117], 3}, {NULL}};
const Segment SGI174082[] = {{&S54483[82], 6}, {NULL}};
const Segment SGI174083[] = {{&S54484[38], 7}, {NULL}};
const Segment SGI174084[] = {{&S54484[31], 6}, {NULL}};
const Segment SGI174085[] = {{&S54487[34], 8}, {NULL}};
const Segment SGI174086[] = {{&S54487[27], 6}, {NULL}};
const Segment SGI174087[] = {{&S54487[101], 9}, {NULL}};
const Segment SGI174088[] = {{&S54487[74], 6}, {NULL}};
const Segment SGI174089[] = {{&S65614[44], 12}, {NULL}};
const Segment SGI174090[] = {{&S65614[44], 28}, {NULL}};
const Segment SGI174091[] = {{&S65614[30], 6}, {NULL}};
const Segment SGI174092[] = {{&S65614[31], 3}, {NULL}};
const Segment SGI174093[] = {{&S65614[86], 6}, {NULL}};
const Segment SGI174094[] = {{&S65618[44], 12}, {NULL}};
const Segment SGI174095[] = {{&S65618[44], 24}, {NULL}};
const Segment SGI174096[] = {{&S65618[30], 6}, {NULL}};
const Segment SGI174097[] = {{&S65618[31], 3}, {NULL}};
const Segment SGI174098[] = {{&S65618[82], 6}, {NULL}};
const Segment SGI174099[] = {{&S65619[44], 11}, {NULL}};
const Segment SGI174100[] = {{&S65619[44], 23}, {NULL}};
const Segment SGI174101[] = {{&S65619[30], 6}, {NULL}};
const Segment SGI174102[] = {{&S65619[31], 3}, {NULL}};
const Segment SGI174103[] = {{&S65619[81], 6}, {NULL}};
const Segment SGI174104[] = {{&S65620[44], 16}, {NULL}};
const Segment SGI174105[] = {{&S65620[44], 28}, {NULL}};
const Segment SGI174106[] = {{&S65620[30], 6}, {NULL}};
const Segment SGI174107[] = {{&S65620[31], 3}, {NULL}};
const Segment SGI174108[] = {{&S65620[86], 6}, {NULL}};
const Segment SGI174109[] = {{&S65621[44], 16}, {NULL}};
const Segment SGI174110[] = {{&S65621[44], 28}, {NULL}};
const Segment SGI174111[] = {{&S65621[30], 6}, {NULL}};
const Segment SGI174112[] = {{&S65621[31], 3}, {NULL}};
const Segment SGI174113[] = {{&S65621[86], 6}, {NULL}};
const Segment SGI174114[] = {{&S57717[34], 9}, {NULL}};
const Segment SGI174115[] = {{&S57717[27], 6}, {NULL}};
const Segment SGI174116[] = {{&S57717[96], 9}, {NULL}};
const Segment SGI174117[] = {{&S57717[69], 6}, {NULL}};
const Segment SGI174118[] = {{&S54509[44], 9}, {NULL}};
const Segment SGI174119[] = {{&S54509[44], 21}, {NULL}};
const Segment SGI174120[] = {{&S54509[30], 6}, {NULL}};
const Segment SGI174121[] = {{&S54509[114], 3}, {NULL}};
const Segment SGI174122[] = {{&S54509[79], 6}, {NULL}};
const Segment SGI174123[] = {{&S65637[44], 17}, {NULL}};
const Segment SGI174124[] = {{&S65637[44], 29}, {NULL}};
const Segment SGI174125[] = {{&S54510[65], 10}, {NULL}};
const Segment SGI174126[] = {{&S65637[30], 6}, {NULL}};
const Segment SGI174127[] = {{&S54510[65], 22}, {NULL}};
const Segment SGI174128[] = {{&S54510[51], 6}, {NULL}};
const Segment SGI174129[] = {{&S65637[31], 3}, {NULL}};
const Segment SGI174130[] = {{&S65637[87], 6}, {NULL}};
const Segment SGI174131[] = {{&S54510[136], 3}, {NULL}};
const Segment SGI174132[] = {{&S54510[101], 6}, {NULL}};
const Segment SGI174133[] = {{&S65638[38], 17}, {NULL}};
const Segment SGI174134[] = {{&S65638[31], 6}, {NULL}};
const Segment SGI174135[] = {{&S54511[44], 10}, {NULL}};
const Segment SGI174136[] = {{&S54511[44], 22}, {NULL}};
const Segment SGI174137[] = {{&S54511[30], 6}, {NULL}};
const Segment SGI174138[] = {{&S54511[115], 3}, {NULL}};
const Segment SGI174139[] = {{&S54511[80], 6}, {NULL}};
const Segment SGI174140[] = {{&S65639[44], 17}, {NULL}};
const Segment SGI174141[] = {{&S65639[44], 29}, {NULL}};
const Segment SGI174142[] = {{&S65639[30], 6}, {NULL}};
const Segment SGI174143[] = {{&S65639[90], 6}, {NULL}};
const Segment SGI174144[] = {{&S54512[38], 9}, {NULL}};
const Segment SGI174145[] = {{&S54512[31], 6}, {NULL}};
const Segment SGI174146[] = {{&S54513[38], 10}, {NULL}};
const Segment SGI174147[] = {{&S54513[31], 6}, {NULL}};
const Segment SGI174148[] = {{&S54514[38], 10}, {NULL}};
const Segment SGI174149[] = {{&S54514[31], 6}, {NULL}};
const Segment SGI174150[] = {{&S65644[34], 10}, {NULL}};
const Segment SGI174151[] = {{&S65644[27], 6}, {NULL}};
const Segment SGI174152[] = {{&S65644[97], 9}, {NULL}};
const Segment SGI174153[] = {{&S65644[70], 6}, {NULL}};
const Segment SGI174154[] = {{&S57746[44], 7}, {NULL}};
const Segment SGI174155[] = {{&S57746[44], 19}, {NULL}};
const Segment SGI174156[] = {{&S57746[30], 6}, {NULL}};
const Segment SGI174157[] = {{&S57746[112], 3}, {NULL}};
const Segment SGI174158[] = {{&S57746[77], 6}, {NULL}};
const Segment SGI174159[] = {{&S57747[44], 7}, {NULL}};
const Segment SGI174160[] = {{&S57747[44], 19}, {NULL}};
const Segment SGI174161[] = {{&S57747[30], 6}, {NULL}};
const Segment SGI174162[] = {{&S57747[112], 3}, {NULL}};
const Segment SGI174163[] = {{&S57747[77], 6}, {NULL}};
const Segment SGI174164[] = {{&S54520[34], 10}, {NULL}};
const Segment SGI174165[] = {{&S54520[27], 6}, {NULL}};
const Segment SGI174166[] = {{&S54520[97], 9}, {NULL}};
const Segment SGI174167[] = {{&S54520[70], 6}, {NULL}};
const Segment SGI174168[] = {{&S57748[44], 7}, {NULL}};
const Segment SGI174169[] = {{&S57748[44], 19}, {NULL}};
const Segment SGI174170[] = {{&S57748[30], 6}, {NULL}};
const Segment SGI174171[] = {{&S57748[112], 3}, {NULL}};
const Segment SGI174172[] = {{&S57748[77], 6}, {NULL}};
const Segment SGI174173[] = {{&S54521[34], 11}, {NULL}};
const Segment SGI174174[] = {{&S54521[27], 6}, {NULL}};
const Segment SGI174175[] = {{&S54521[98], 9}, {NULL}};
const Segment SGI174176[] = {{&S54521[71], 6}, {NULL}};
const Segment SGI174177[] = {{&S54530[34], 7}, {NULL}};
const Segment SGI174178[] = {{&S54530[27], 6}, {NULL}};
const Segment SGI174179[] = {{&S54530[105], 9}, {NULL}};
const Segment SGI174180[] = {{&S54530[78], 6}, {NULL}};
const Segment SGI174181[] = {{&S65676[44], 7}, {NULL}};
const Segment SGI174182[] = {{&S65676[44], 19}, {NULL}};
const Segment SGI174183[] = {{&S65676[30], 6}, {NULL}};
const Segment SGI174184[] = {{&S65676[31], 3}, {NULL}};
const Segment SGI174185[] = {{&S65676[77], 6}, {NULL}};
const Segment SGI174186[] = {{&S65677[44], 7}, {NULL}};
const Segment SGI174187[] = {{&S65677[44], 19}, {NULL}};
const Segment SGI174188[] = {{&S65677[30], 6}, {NULL}};
const Segment SGI174189[] = {{&S65677[31], 3}, {NULL}};
const Segment SGI174190[] = {{&S65677[77], 6}, {NULL}};
const Segment SGI174191[] = {{&S65678[38], 7}, {NULL}};
const Segment SGI174192[] = {{&S65678[31], 6}, {NULL}};
const Segment SGI174193[] = {{&S54550[34], 15}, {NULL}};
const Segment SGI174194[] = {{&S54550[27], 6}, {NULL}};
const Segment SGI174195[] = {{&S54550[102], 9}, {NULL}};
const Segment SGI174196[] = {{&S54550[75], 6}, {NULL}};
const Segment SGI174197[] = {{&S54551[34], 16}, {NULL}};
const Segment SGI174198[] = {{&S54551[27], 6}, {NULL}};
const Segment SGI174199[] = {{&S54551[103], 9}, {NULL}};
const Segment SGI174200[] = {{&S54551[76], 6}, {NULL}};
const Segment SGI174201[] = {{&S54555[34], 20}, {NULL}};
const Segment SGI174202[] = {{&S54555[27], 6}, {NULL}};
const Segment SGI174203[] = {{&S54555[116], 9}, {NULL}};
const Segment SGI174204[] = {{&S54555[89], 6}, {NULL}};
const Segment SGI174205[] = {{&S65698[44], 25}, {NULL}};
const Segment SGI174206[] = {{&S65698[44], 37}, {NULL}};
const Segment SGI174207[] = {{&S65698[30], 6}, {NULL}};
const Segment SGI174208[] = {{&S65698[31], 3}, {NULL}};
const Segment SGI174209[] = {{&S65698[95], 6}, {NULL}};
const Segment SGI174210[] = {{&S54567[34], 19}, {NULL}};
const Segment SGI174211[] = {{&S54567[27], 6}, {NULL}};
const Segment SGI174212[] = {{&S54567[106], 9}, {NULL}};
const Segment SGI174213[] = {{&S54567[79], 6}, {NULL}};
const Segment SGI174214[] = {{&S65704[34], 14}, {NULL}};
const Segment SGI174215[] = {{&S65704[27], 6}, {NULL}};
const Segment SGI174216[] = {{&S65704[106], 9}, {NULL}};
const Segment SGI174217[] = {{&S65704[79], 6}, {NULL}};
const Segment SGI174218[] = {{&S65708[34], 14}, {NULL}};
const Segment SGI174219[] = {{&S65708[27], 6}, {NULL}};
const Segment SGI174220[] = {{&S65708[106], 9}, {NULL}};
const Segment SGI174221[] = {{&S65708[79], 6}, {NULL}};
const Segment SGI174222[] = {{&S65709[34], 10}, {NULL}};
const Segment SGI174223[] = {{&S65709[27], 6}, {NULL}};
const Segment SGI174224[] = {{&S54575[34], 7}, {NULL}};
const Segment SGI174225[] = {{&S54575[27], 6}, {NULL}};
const Segment SGI174226[] = {{&S65709[97], 9}, {NULL}};
const Segment SGI174227[] = {{&S65709[70], 6}, {NULL}};
const Segment SGI174228[] = {{&S54575[99], 9}, {NULL}};
const Segment SGI174229[] = {{&S54575[72], 6}, {NULL}};
const Segment SGI174230[] = {{&S54576[34], 7}, {NULL}};
const Segment SGI174231[] = {{&S54576[27], 6}, {NULL}};
const Segment SGI174232[] = {{&S54576[99], 9}, {NULL}};
const Segment SGI174233[] = {{&S54576[72], 6}, {NULL}};
const Segment SGI174234[] = {{&S65723[34], 18}, {NULL}};
const Segment SGI174235[] = {{&S65723[27], 6}, {NULL}};
const Segment SGI174236[] = {{&S65723[105], 9}, {NULL}};
const Segment SGI174237[] = {{&S65723[78], 6}, {NULL}};
const Segment SGI174238[] = {{&S65724[44], 6}, {NULL}};
const Segment SGI174239[] = {{&S65724[44], 30}, {NULL}};
const Segment SGI174240[] = {{&S65724[30], 6}, {NULL}};
const Segment SGI174241[] = {{&S65724[31], 3}, {NULL}};
const Segment SGI174242[] = {{&S65724[88], 6}, {NULL}};
const Segment SGI174243[] = {{&S65725[44], 6}, {NULL}};
const Segment SGI174244[] = {{&S65725[44], 30}, {NULL}};
const Segment SGI174245[] = {{&S65725[30], 6}, {NULL}};
const Segment SGI174246[] = {{&S65725[31], 3}, {NULL}};
const Segment SGI174247[] = {{&S65725[88], 6}, {NULL}};
const Segment SGI174248[] = {{&S65726[44], 6}, {NULL}};
const Segment SGI174249[] = {{&S65726[44], 30}, {NULL}};
const Segment SGI174250[] = {{&S65726[30], 6}, {NULL}};
const Segment SGI174251[] = {{&S65726[31], 3}, {NULL}};
const Segment SGI174252[] = {{&S65726[88], 6}, {NULL}};
const Segment SGI174253[] = {{&S65727[38], 6}, {NULL}};
const Segment SGI174254[] = {{&S65727[31], 6}, {NULL}};
const Segment SGI174255[] = {{&S54621[44], 11}, {NULL}};
const Segment SGI174256[] = {{&S54621[44], 29}, {NULL}};
const Segment SGI174257[] = {{&S54621[30], 6}, {NULL}};
const Segment SGI174258[] = {{&S54621[122], 3}, {NULL}};
const Segment SGI174259[] = {{&S54621[87], 6}, {NULL}};
const Segment SGI174260[] = {{&S54622[38], 7}, {NULL}};
const Segment SGI174261[] = {{&S54622[31], 6}, {NULL}};
const Segment SGI174262[] = {{&S54623[44], 7}, {NULL}};
const Segment SGI174263[] = {{&S54623[44], 25}, {NULL}};
const Segment SGI174264[] = {{&S54623[30], 6}, {NULL}};
const Segment SGI174265[] = {{&S54623[118], 3}, {NULL}};
const Segment SGI174266[] = {{&S54623[83], 6}, {NULL}};
const Segment SGI174267[] = {{&S54624[44], 7}, {NULL}};
const Segment SGI174268[] = {{&S54624[44], 25}, {NULL}};
const Segment SGI174269[] = {{&S54624[30], 6}, {NULL}};
const Segment SGI174270[] = {{&S54624[118], 3}, {NULL}};
const Segment SGI174271[] = {{&S65758[44], 8}, {NULL}};
const Segment SGI174272[] = {{&S65758[44], 20}, {NULL}};
const Segment SGI174273[] = {{&S54624[83], 6}, {NULL}};
const Segment SGI174274[] = {{&S65758[30], 6}, {NULL}};
const Segment SGI174275[] = {{&S65758[31], 3}, {NULL}};
const Segment SGI174276[] = {{&S65758[78], 6}, {NULL}};
const Segment SGI174277[] = {{&S54625[44], 11}, {NULL}};
const Segment SGI174278[] = {{&S54625[44], 29}, {NULL}};
const Segment SGI174279[] = {{&S54625[30], 6}, {NULL}};
const Segment SGI174280[] = {{&S54625[122], 3}, {NULL}};
const Segment SGI174281[] = {{&S54625[87], 6}, {NULL}};
const Segment SGI174282[] = {{&S54626[44], 7}, {NULL}};
const Segment SGI174283[] = {{&S54626[44], 27}, {NULL}};
const Segment SGI174284[] = {{&S54626[30], 6}, {NULL}};
const Segment SGI174285[] = {{&S54626[120], 3}, {NULL}};
const Segment SGI174286[] = {{&S54626[85], 6}, {NULL}};
const Segment SGI174287[] = {{&S54627[44], 7}, {NULL}};
const Segment SGI174288[] = {{&S54627[44], 25}, {NULL}};
const Segment SGI174289[] = {{&S54627[30], 6}, {NULL}};
const Segment SGI174290[] = {{&S54627[118], 3}, {NULL}};
const Segment SGI174291[] = {{&S54627[83], 6}, {NULL}};
const Segment SGI174292[] = {{&S54628[44], 9}, {NULL}};
const Segment SGI174293[] = {{&S54628[44], 21}, {NULL}};
const Segment SGI174294[] = {{&S54628[30], 6}, {NULL}};
const Segment SGI174295[] = {{&S54628[114], 3}, {NULL}};
const Segment SGI174296[] = {{&S54628[79], 6}, {NULL}};
const Segment SGI174297[] = {{&S54629[38], 9}, {NULL}};
const Segment SGI174298[] = {{&S54629[31], 6}, {NULL}};
const Segment SGI174299[] = {{&S65790[34], 10}, {NULL}};
const Segment SGI174300[] = {{&S65790[27], 6}, {NULL}};
const Segment SGI174301[] = {{&S65790[97], 9}, {NULL}};
const Segment SGI174302[] = {{&S65790[70], 6}, {NULL}};
const Segment SGI174303[] = {{&S54653[34], 7}, {NULL}};
const Segment SGI174304[] = {{&S54653[27], 6}, {NULL}};
const Segment SGI174305[] = {{&S54653[110], 9}, {NULL}};
const Segment SGI174306[] = {{&S54653[83], 6}, {NULL}};
const Segment SGI174307[] = {{&S65805[44], 6}, {NULL}};
const Segment SGI174308[] = {{&S65805[44], 18}, {NULL}};
const Segment SGI174309[] = {{&S65805[30], 6}, {NULL}};
const Segment SGI174310[] = {{&S65805[31], 3}, {NULL}};
const Segment SGI174311[] = {{&S65805[76], 6}, {NULL}};
const Segment SGI174312[] = {{&S54666[44], 10}, {NULL}};
const Segment SGI174313[] = {{&S54666[44], 22}, {NULL}};
const Segment SGI174314[] = {{&S54666[30], 6}, {NULL}};
const Segment SGI174315[] = {{&S54666[115], 3}, {NULL}};
const Segment SGI174316[] = {{&S54666[80], 6}, {NULL}};
const Segment SGI174317[] = {{&S54667[38], 10}, {NULL}};
const Segment SGI174318[] = {{&S54667[31], 6}, {NULL}};
const Segment SGI174319[] = {{&S65808[44], 4}, {NULL}};
const Segment SGI174320[] = {{&S65808[44], 16}, {NULL}};
const Segment SGI174321[] = {{&S65808[30], 6}, {NULL}};
const Segment SGI174322[] = {{&S65808[31], 3}, {NULL}};
const Segment SGI174323[] = {{&S65808[74], 6}, {NULL}};
const Segment SGI174324[] = {{&S65809[38], 4}, {NULL}};
const Segment SGI174325[] = {{&S65809[31], 6}, {NULL}};
const Segment SGI174326[] = {{&S65810[34], 8}, {NULL}};
const Segment SGI174327[] = {{&S65810[27], 6}, {NULL}};
const Segment SGI174328[] = {{&S65810[100], 9}, {NULL}};
const Segment SGI174329[] = {{&S65810[73], 6}, {NULL}};
const Segment SGI174330[] = {{&S54679[34], 9}, {NULL}};
const Segment SGI174331[] = {{&S54679[27], 6}, {NULL}};
const Segment SGI174332[] = {{&S54679[96], 9}, {NULL}};
const Segment SGI174333[] = {{&S54679[69], 6}, {NULL}};
const Segment SGI174334[] = {{&S65818[34], 12}, {NULL}};
const Segment SGI174335[] = {{&S65818[27], 6}, {NULL}};
const Segment SGI174336[] = {{&S65818[115], 9}, {NULL}};
const Segment SGI174337[] = {{&S65818[88], 6}, {NULL}};
const Segment SGI174338[] = {{&S57953[44], 10}, {NULL}};
const Segment SGI174339[] = {{&S57953[44], 22}, {NULL}};
const Segment SGI174340[] = {{&S57953[30], 6}, {NULL}};
const Segment SGI174341[] = {{&S57953[131], 3}, {NULL}};
const Segment SGI174342[] = {{&S57953[96], 6}, {NULL}};
const Segment SGI174343[] = {{&S57953[68], 8}, {NULL}};
const Segment SGI174344[] = {{&S57954[44], 10}, {NULL}};
const Segment SGI174345[] = {{&S57954[44], 22}, {NULL}};
const Segment SGI174346[] = {{&S57954[30], 6}, {NULL}};
const Segment SGI174347[] = {{&S57954[131], 3}, {NULL}};
const Segment SGI174348[] = {{&S57954[96], 6}, {NULL}};
const Segment SGI174349[] = {{&S65837[44], 10}, {NULL}};
const Segment SGI174350[] = {{&S65837[44], 22}, {NULL}};
const Segment SGI174351[] = {{&S57954[68], 8}, {NULL}};
const Segment SGI174352[] = {{&S65837[30], 6}, {NULL}};
const Segment SGI174353[] = {{&S65837[31], 3}, {NULL}};
const Segment SGI174354[] = {{&S65837[80], 6}, {NULL}};
const Segment SGI174355[] = {{&S57956[44], 10}, {NULL}};
const Segment SGI174356[] = {{&S57956[44], 22}, {NULL}};
const Segment SGI174357[] = {{&S57956[30], 6}, {NULL}};
const Segment SGI174358[] = {{&S57956[131], 3}, {NULL}};
const Segment SGI174359[] = {{&S57956[96], 6}, {NULL}};
const Segment SGI174360[] = {{&S57956[68], 8}, {NULL}};
const Segment SGI174361[] = {{&S65851[34], 10}, {NULL}};
const Segment SGI174362[] = {{&S65851[27], 6}, {NULL}};
const Segment SGI174363[] = {{&S65851[97], 9}, {NULL}};
const Segment SGI174364[] = {{&S65851[70], 6}, {NULL}};
const Segment SGI174365[] = {{&S65882[44], 5}, {NULL}};
const Segment SGI174366[] = {{&S65882[30], 6}, {NULL}};
const Segment SGI174367[] = {{&S65882[31], 3}, {NULL}};
const Segment SGI174368[] = {{&S65882[63], 6}, {NULL}};
const Segment SGI174369[] = {{&S65889[34], 10}, {NULL}};
const Segment SGI174370[] = {{&S65889[27], 6}, {NULL}};
const Segment SGI174371[] = {{&S65889[97], 9}, {NULL}};
const Segment SGI174372[] = {{&S65889[70], 6}, {NULL}};
const Segment SGI174373[] = {{&S65890[34], 10}, {NULL}};
const Segment SGI174374[] = {{&S65890[27], 6}, {NULL}};
const Segment SGI174375[] = {{&S65890[97], 9}, {NULL}};
const Segment SGI174376[] = {{&S65890[70], 6}, {NULL}};
const Segment SGI174377[] = {{&S54717[44], 10}, {NULL}};
const Segment SGI174378[] = {{&S54717[44], 22}, {NULL}};
const Segment SGI174379[] = {{&S54717[30], 6}, {NULL}};
const Segment SGI174380[] = {{&S54717[80], 6}, {NULL}};
const Segment SGI174381[] = {{&S54717[107], 13}, {NULL}};
const Segment SGI174382[] = {{&S54718[44], 10}, {NULL}};
const Segment SGI174383[] = {{&S54718[44], 22}, {NULL}};
const Segment SGI174384[] = {{&S54718[30], 6}, {NULL}};
const Segment SGI174385[] = {{&S54718[80], 6}, {NULL}};
const Segment SGI174386[] = {{&S54718[107], 13}, {NULL}};
const Segment SGI174387[] = {{&S54719[44], 10}, {NULL}};
const Segment SGI174388[] = {{&S54719[44], 22}, {NULL}};
const Segment SGI174389[] = {{&S54719[30], 6}, {NULL}};
const Segment SGI174390[] = {{&S54719[80], 6}, {NULL}};
const Segment SGI174391[] = {{&S54719[107], 13}, {NULL}};
const Segment SGI174392[] = {{&S54720[44], 10}, {NULL}};
const Segment SGI174393[] = {{&S54720[44], 22}, {NULL}};
const Segment SGI174394[] = {{&S54720[30], 6}, {NULL}};
const Segment SGI174395[] = {{&S54720[80], 6}, {NULL}};
const Segment SGI174396[] = {{&S54720[107], 13}, {NULL}};
const Segment SGI174397[] = {{&S54727[34], 6}, {NULL}};
const Segment SGI174398[] = {{&S54727[27], 6}, {NULL}};
const Segment SGI174399[] = {{&S54727[93], 9}, {NULL}};
const Segment SGI174400[] = {{&S54727[66], 6}, {NULL}};
const Segment SGI174401[] = {{&S65931[44], 10}, {NULL}};
const Segment SGI174402[] = {{&S65931[44], 22}, {NULL}};
const Segment SGI174403[] = {{&S65931[30], 6}, {NULL}};
const Segment SGI174404[] = {{&S54735[34], 10}, {NULL}};
const Segment SGI174405[] = {{&S65931[31], 3}, {NULL}};
const Segment SGI174406[] = {{&S54735[27], 6}, {NULL}};
const Segment SGI174407[] = {{&S65931[99], 6}, {NULL}};
const Segment SGI174408[] = {{&S65931[68], 8}, {NULL}};
const Segment SGI174409[] = {{&S54735[97], 9}, {NULL}};
const Segment SGI174410[] = {{&S54735[70], 6}, {NULL}};
const Segment SGI174411[] = {{&S65932[44], 10}, {NULL}};
const Segment SGI174412[] = {{&S65932[44], 22}, {NULL}};
const Segment SGI174413[] = {{&S65932[30], 6}, {NULL}};
const Segment SGI174414[] = {{&S54736[44], 12}, {NULL}};
const Segment SGI174415[] = {{&S54736[44], 24}, {NULL}};
const Segment SGI174416[] = {{&S54736[30], 6}, {NULL}};
const Segment SGI174417[] = {{&S65932[31], 3}, {NULL}};
const Segment SGI174418[] = {{&S54736[117], 3}, {NULL}};
const Segment SGI174419[] = {{&S65932[99], 6}, {NULL}};
const Segment SGI174420[] = {{&S54736[82], 6}, {NULL}};
const Segment SGI174421[] = {{&S65932[68], 8}, {NULL}};
const Segment SGI174422[] = {{&S54737[38], 12}, {NULL}};
const Segment SGI174423[] = {{&S54737[31], 6}, {NULL}};
const Segment SGI174424[] = {{&S65933[44], 10}, {NULL}};
const Segment SGI174425[] = {{&S65933[44], 22}, {NULL}};
const Segment SGI174426[] = {{&S65933[30], 6}, {NULL}};
const Segment SGI174427[] = {{&S65933[31], 3}, {NULL}};
const Segment SGI174428[] = {{&S65933[95], 6}, {NULL}};
const Segment SGI174429[] = {{&S65933[68], 8}, {NULL}};
const Segment SGI174430[] = {{&S54741[34], 18}, {NULL}};
const Segment SGI174431[] = {{&S54741[27], 6}, {NULL}};
const Segment SGI174432[] = {{&S54741[105], 9}, {NULL}};
const Segment SGI174433[] = {{&S54741[78], 6}, {NULL}};
const Segment SGI174434[] = {{&S54742[34], 11}, {NULL}};
const Segment SGI174435[] = {{&S54742[27], 6}, {NULL}};
const Segment SGI174436[] = {{&S54742[98], 9}, {NULL}};
const Segment SGI174437[] = {{&S54742[71], 6}, {NULL}};
const Segment SGI174438[] = {{&S65949[34], 10}, {NULL}};
const Segment SGI174439[] = {{&S65949[27], 6}, {NULL}};
const Segment SGI174440[] = {{&S65949[97], 9}, {NULL}};
const Segment SGI174441[] = {{&S65949[70], 6}, {NULL}};
const Segment SGI174442[] = {{&S58075[34], 18}, {NULL}};
const Segment SGI174443[] = {{&S58075[27], 6}, {NULL}};
const Segment SGI174444[] = {{&S58075[114], 9}, {NULL}};
const Segment SGI174445[] = {{&S58075[87], 6}, {NULL}};
const Segment SGI174446[] = {{&S58076[34], 16}, {NULL}};
const Segment SGI174447[] = {{&S58076[27], 6}, {NULL}};
const Segment SGI174448[] = {{&S58076[112], 9}, {NULL}};
const Segment SGI174449[] = {{&S58076[85], 6}, {NULL}};
const Segment SGI174450[] = {{&S58077[34], 18}, {NULL}};
const Segment SGI174451[] = {{&S58077[27], 6}, {NULL}};
const Segment SGI174452[] = {{&S58077[114], 9}, {NULL}};
const Segment SGI174453[] = {{&S58077[87], 6}, {NULL}};
const Segment SGI174454[] = {{&S58078[34], 18}, {NULL}};
const Segment SGI174455[] = {{&S58078[27], 6}, {NULL}};
const Segment SGI174456[] = {{&S58078[114], 9}, {NULL}};
const Segment SGI174457[] = {{&S58078[87], 6}, {NULL}};
const Segment SGI174458[] = {{&S58079[34], 18}, {NULL}};
const Segment SGI174459[] = {{&S58079[27], 6}, {NULL}};
const Segment SGI174460[] = {{&S58079[114], 9}, {NULL}};
const Segment SGI174461[] = {{&S58079[87], 6}, {NULL}};
const Segment SGI174462[] = {{&S58080[44], 13}, {NULL}};
const Segment SGI174463[] = {{&S58080[44], 25}, {NULL}};
const Segment SGI174464[] = {{&S58080[30], 6}, {NULL}};
const Segment SGI174465[] = {{&S58080[134], 3}, {NULL}};
const Segment SGI174466[] = {{&S65964[44], 8}, {NULL}};
const Segment SGI174467[] = {{&S58080[99], 6}, {NULL}};
const Segment SGI174468[] = {{&S65964[44], 20}, {NULL}};
const Segment SGI174469[] = {{&S65964[30], 6}, {NULL}};
const Segment SGI174470[] = {{&S58080[71], 8}, {NULL}};
const Segment SGI174471[] = {{&S65964[31], 3}, {NULL}};
const Segment SGI174472[] = {{&S65964[78], 6}, {NULL}};
const Segment SGI174473[] = {{&S65965[38], 8}, {NULL}};
const Segment SGI174474[] = {{&S65965[31], 6}, {NULL}};
const Segment SGI174475[] = {{&S58082[44], 13}, {NULL}};
const Segment SGI174476[] = {{&S58082[44], 25}, {NULL}};
const Segment SGI174477[] = {{&S58082[30], 6}, {NULL}};
const Segment SGI174478[] = {{&S58082[134], 3}, {NULL}};
const Segment SGI174479[] = {{&S58082[99], 6}, {NULL}};
const Segment SGI174480[] = {{&S58082[71], 8}, {NULL}};
const Segment SGI174481[] = {{&S65967[34], 12}, {NULL}};
const Segment SGI174482[] = {{&S65967[27], 6}, {NULL}};
const Segment SGI174483[] = {{&S65967[99], 9}, {NULL}};
const Segment SGI174484[] = {{&S65967[72], 6}, {NULL}};
const Segment SGI174485[] = {{&S54794[44], 7}, {NULL}};
const Segment SGI174486[] = {{&S54794[44], 19}, {NULL}};
const Segment SGI174487[] = {{&S54794[30], 6}, {NULL}};
const Segment SGI174488[] = {{&S54794[112], 3}, {NULL}};
const Segment SGI174489[] = {{&S54794[77], 6}, {NULL}};
const Segment SGI174490[] = {{&S54795[38], 7}, {NULL}};
const Segment SGI174491[] = {{&S54795[31], 6}, {NULL}};
const Segment SGI174492[] = {{&S54797[44], 9}, {NULL}};
const Segment SGI174493[] = {{&S54797[44], 21}, {NULL}};
const Segment SGI174494[] = {{&S54797[30], 6}, {NULL}};
const Segment SGI174495[] = {{&S54797[114], 3}, {NULL}};
const Segment SGI174496[] = {{&S54797[79], 6}, {NULL}};
const Segment SGI174497[] = {{&S54798[44], 10}, {NULL}};
const Segment SGI174498[] = {{&S54798[44], 22}, {NULL}};
const Segment SGI174499[] = {{&S54798[30], 6}, {NULL}};
const Segment SGI174500[] = {{&S54798[115], 3}, {NULL}};
const Segment SGI174501[] = {{&S54798[80], 6}, {NULL}};
const Segment SGI174502[] = {{&S54799[44], 10}, {NULL}};
const Segment SGI174503[] = {{&S54799[44], 22}, {NULL}};
const Segment SGI174504[] = {{&S54799[30], 6}, {NULL}};
const Segment SGI174505[] = {{&S54799[115], 3}, {NULL}};
const Segment SGI174506[] = {{&S54799[80], 6}, {NULL}};
const Segment SGI174507[] = {{&S54800[38], 10}, {NULL}};
const Segment SGI174508[] = {{&S54800[31], 6}, {NULL}};
const Segment SGI174509[] = {{&S54801[38], 9}, {NULL}};
const Segment SGI174510[] = {{&S54801[31], 6}, {NULL}};
const Segment SGI174511[] = {{&S54802[44], 26}, {NULL}};
const Segment SGI174512[] = {{&S54802[44], 38}, {NULL}};
const Segment SGI174513[] = {{&S54802[30], 6}, {NULL}};
const Segment SGI174514[] = {{&S54802[131], 3}, {NULL}};
const Segment SGI174515[] = {{&S54802[96], 6}, {NULL}};
const Segment SGI174516[] = {{&S54803[44], 26}, {NULL}};
const Segment SGI174517[] = {{&S54803[44], 38}, {NULL}};
const Segment SGI174518[] = {{&S54803[30], 6}, {NULL}};
const Segment SGI174519[] = {{&S54803[131], 3}, {NULL}};
const Segment SGI174520[] = {{&S54803[96], 6}, {NULL}};
const Segment SGI174521[] = {{&S65995[44], 11}, {NULL}};
const Segment SGI174522[] = {{&S65995[44], 23}, {NULL}};
const Segment SGI174523[] = {{&S54804[38], 10}, {NULL}};
const Segment SGI174524[] = {{&S65995[30], 6}, {NULL}};
const Segment SGI174525[] = {{&S54804[31], 6}, {NULL}};
const Segment SGI174526[] = {{&S65995[31], 3}, {NULL}};
const Segment SGI174527[] = {{&S65995[81], 6}, {NULL}};
const Segment SGI174528[] = {{&S52501[44], 10}, {NULL}};
const Segment SGI174529[] = {{&S52501[44], 22}, {NULL}};
const Segment SGI174530[] = {{&S52501[30], 6}, {NULL}};
const Segment SGI174531[] = {{&S52501[133], 3}, {NULL}};
const Segment SGI174532[] = {{&S54805[44], 10}, {NULL}};
const Segment SGI174533[] = {{&S54805[44], 22}, {NULL}};
const Segment SGI174534[] = {{&S52501[98], 6}, {NULL}};
const Segment SGI174535[] = {{&S58118[34], 10}, {NULL}};
const Segment SGI174536[] = {{&S54805[30], 6}, {NULL}};
const Segment SGI174537[] = {{&S58118[27], 6}, {NULL}};
const Segment SGI174538[] = {{&S52501[68], 8}, {NULL}};
const Segment SGI174539[] = {{&S54805[115], 3}, {NULL}};
const Segment SGI174540[] = {{&S54805[80], 6}, {NULL}};
const Segment SGI174541[] = {{&S58118[97], 9}, {NULL}};
const Segment SGI174542[] = {{&S58118[70], 6}, {NULL}};
const Segment SGI174543[] = {{&S52502[44], 10}, {NULL}};
const Segment SGI174544[] = {{&S52502[44], 22}, {NULL}};
const Segment SGI174545[] = {{&S52502[30], 6}, {NULL}};
const Segment SGI174546[] = {{&S52502[133], 3}, {NULL}};
const Segment SGI174547[] = {{&S52502[98], 6}, {NULL}};
const Segment SGI174548[] = {{&S52502[68], 8}, {NULL}};
const Segment SGI174549[] = {{&S58121[34], 6}, {NULL}};
const Segment SGI174550[] = {{&S58121[27], 6}, {NULL}};
const Segment SGI174551[] = {{&S58121[93], 9}, {NULL}};
const Segment SGI174552[] = {{&S58121[66], 6}, {NULL}};
const Segment SGI174553[] = {{&S66015[44], 8}, {NULL}};
const Segment SGI174554[] = {{&S66015[44], 20}, {NULL}};
const Segment SGI174555[] = {{&S66015[30], 6}, {NULL}};
const Segment SGI174556[] = {{&S66015[31], 3}, {NULL}};
const Segment SGI174557[] = {{&S66015[78], 6}, {NULL}};
const Segment SGI174558[] = {{&S66020[34], 15}, {NULL}};
const Segment SGI174559[] = {{&S66020[27], 6}, {NULL}};
const Segment SGI174560[] = {{&S66020[102], 9}, {NULL}};
const Segment SGI174561[] = {{&S66020[75], 6}, {NULL}};
const Segment SGI174562[] = {{&S52529[44], 20}, {NULL}};
const Segment SGI174563[] = {{&S52529[44], 32}, {NULL}};
const Segment SGI174564[] = {{&S52529[30], 6}, {NULL}};
const Segment SGI174565[] = {{&S52529[143], 3}, {NULL}};
const Segment SGI174566[] = {{&S52529[108], 6}, {NULL}};
const Segment SGI174567[] = {{&S52529[78], 8}, {NULL}};
const Segment SGI174568[] = {{&S52530[44], 20}, {NULL}};
const Segment SGI174569[] = {{&S52530[44], 32}, {NULL}};
const Segment SGI174570[] = {{&S52530[30], 6}, {NULL}};
const Segment SGI174571[] = {{&S52530[143], 3}, {NULL}};
const Segment SGI174572[] = {{&S52530[108], 6}, {NULL}};
const Segment SGI174573[] = {{&S52530[78], 8}, {NULL}};
const Segment SGI174574[] = {{&S52532[44], 21}, {NULL}};
const Segment SGI174575[] = {{&S52532[44], 33}, {NULL}};
const Segment SGI174576[] = {{&S52532[30], 6}, {NULL}};
const Segment SGI174577[] = {{&S52532[144], 3}, {NULL}};
const Segment SGI174578[] = {{&S52532[109], 6}, {NULL}};
const Segment SGI174579[] = {{&S52532[79], 8}, {NULL}};
const Segment SGI174580[] = {{&S66036[44], 12}, {NULL}};
const Segment SGI174581[] = {{&S66036[44], 24}, {NULL}};
const Segment SGI174582[] = {{&S66036[30], 6}, {NULL}};
const Segment SGI174583[] = {{&S66036[31], 3}, {NULL}};
const Segment SGI174584[] = {{&S66036[82], 6}, {NULL}};
const Segment SGI174585[] = {{&S66037[44], 13}, {NULL}};
const Segment SGI174586[] = {{&S66037[44], 25}, {NULL}};
const Segment SGI174587[] = {{&S66037[30], 6}, {NULL}};
const Segment SGI174588[] = {{&S66037[31], 3}, {NULL}};
const Segment SGI174589[] = {{&S66037[83], 6}, {NULL}};
const Segment SGI174590[] = {{&S66038[38], 12}, {NULL}};
const Segment SGI174591[] = {{&S66038[31], 6}, {NULL}};
const Segment SGI174592[] = {{&S54874[34], 16}, {NULL}};
const Segment SGI174593[] = {{&S54874[27], 6}, {NULL}};
const Segment SGI174594[] = {{&S54874[103], 9}, {NULL}};
const Segment SGI174595[] = {{&S54874[76], 6}, {NULL}};
const Segment SGI174596[] = {{&S54876[34], 6}, {NULL}};
const Segment SGI174597[] = {{&S54876[27], 6}, {NULL}};
const Segment SGI174598[] = {{&S54876[93], 9}, {NULL}};
const Segment SGI174599[] = {{&S54876[66], 6}, {NULL}};
const Segment SGI174600[] = {{&S54877[34], 7}, {NULL}};
const Segment SGI174601[] = {{&S54877[27], 6}, {NULL}};
const Segment SGI174602[] = {{&S54877[94], 9}, {NULL}};
const Segment SGI174603[] = {{&S54877[67], 6}, {NULL}};
const Segment SGI174604[] = {{&S66058[34], 10}, {NULL}};
const Segment SGI174605[] = {{&S66058[27], 6}, {NULL}};
const Segment SGI174606[] = {{&S66058[97], 9}, {NULL}};
const Segment SGI174607[] = {{&S66058[70], 6}, {NULL}};
const Segment SGI174608[] = {{&S66058[138], 8}, {NULL}};
const Segment SGI174609[] = {{&S54879[34], 4}, {NULL}};
const Segment SGI174610[] = {{&S54879[27], 6}, {NULL}};
const Segment SGI174611[] = {{&S66059[34], 13}, {NULL}};
const Segment SGI174612[] = {{&S54879[91], 9}, {NULL}};
const Segment SGI174613[] = {{&S54879[64], 6}, {NULL}};
const Segment SGI174614[] = {{&S66059[27], 6}, {NULL}};
const Segment SGI174615[] = {{&S66059[100], 9}, {NULL}};
const Segment SGI174616[] = {{&S66059[73], 6}, {NULL}};
const Segment SGI174617[] = {{&S66059[141], 8}, {NULL}};
const Segment SGI174618[] = {{&S66060[34], 14}, {NULL}};
const Segment SGI174619[] = {{&S66060[27], 6}, {NULL}};
const Segment SGI174620[] = {{&S66060[101], 9}, {NULL}};
const Segment SGI174621[] = {{&S66060[74], 6}, {NULL}};
const Segment SGI174622[] = {{&S66060[142], 8}, {NULL}};
const Segment SGI174623[] = {{&S54886[34], 4}, {NULL}};
const Segment SGI174624[] = {{&S54886[27], 6}, {NULL}};
const Segment SGI174625[] = {{&S54886[91], 9}, {NULL}};
const Segment SGI174626[] = {{&S54886[64], 6}, {NULL}};
const Segment SGI174627[] = {{&S52582[44], 9}, {NULL}};
const Segment SGI174628[] = {{&S52582[44], 21}, {NULL}};
const Segment SGI174629[] = {{&S54891[34], 17}, {NULL}};
const Segment SGI174630[] = {{&S52582[30], 6}, {NULL}};
const Segment SGI174631[] = {{&S54891[27], 6}, {NULL}};
const Segment SGI174632[] = {{&S52582[114], 3}, {NULL}};
const Segment SGI174633[] = {{&S52582[79], 6}, {NULL}};
const Segment SGI174634[] = {{&S54891[104], 9}, {NULL}};
const Segment SGI174635[] = {{&S54891[77], 6}, {NULL}};
const Segment SGI174636[] = {{&S66069[44], 7}, {NULL}};
const Segment SGI174637[] = {{&S66069[44], 19}, {NULL}};
const Segment SGI174638[] = {{&S66069[30], 6}, {NULL}};
const Segment SGI174639[] = {{&S66069[31], 3}, {NULL}};
const Segment SGI174640[] = {{&S66069[77], 6}, {NULL}};
const Segment SGI174641[] = {{&S52583[38], 9}, {NULL}};
const Segment SGI174642[] = {{&S52583[31], 6}, {NULL}};
const Segment SGI174643[] = {{&S66070[44], 7}, {NULL}};
const Segment SGI174644[] = {{&S66070[44], 19}, {NULL}};
const Segment SGI174645[] = {{&S66070[30], 6}, {NULL}};
const Segment SGI174646[] = {{&S66070[31], 3}, {NULL}};
const Segment SGI174647[] = {{&S66070[77], 6}, {NULL}};
const Segment SGI174648[] = {{&S66072[44], 9}, {NULL}};
const Segment SGI174649[] = {{&S66072[44], 21}, {NULL}};
const Segment SGI174650[] = {{&S66072[30], 6}, {NULL}};
const Segment SGI174651[] = {{&S66072[31], 3}, {NULL}};
const Segment SGI174652[] = {{&S54897[34], 11}, {NULL}};
const Segment SGI174653[] = {{&S66072[79], 6}, {NULL}};
const Segment SGI174654[] = {{&S54897[27], 6}, {NULL}};
const Segment SGI174655[] = {{&S54897[98], 9}, {NULL}};
const Segment SGI174656[] = {{&S54897[71], 6}, {NULL}};
const Segment SGI174657[] = {{&S54898[34], 10}, {NULL}};
const Segment SGI174658[] = {{&S54898[27], 6}, {NULL}};
const Segment SGI174659[] = {{&S66074[34], 16}, {NULL}};
const Segment SGI174660[] = {{&S54898[105], 9}, {NULL}};
const Segment SGI174661[] = {{&S54898[78], 6}, {NULL}};
const Segment SGI174662[] = {{&S66074[27], 6}, {NULL}};
const Segment SGI174663[] = {{&S66074[110], 9}, {NULL}};
const Segment SGI174664[] = {{&S66074[83], 6}, {NULL}};
const Segment SGI174665[] = {{&S54899[34], 8}, {NULL}};
const Segment SGI174666[] = {{&S54899[27], 6}, {NULL}};
const Segment SGI174667[] = {{&S54899[95], 9}, {NULL}};
const Segment SGI174668[] = {{&S54899[68], 6}, {NULL}};
const Segment SGI174669[] = {{&S66075[34], 18}, {NULL}};
const Segment SGI174670[] = {{&S66075[27], 6}, {NULL}};
const Segment SGI174671[] = {{&S66075[114], 9}, {NULL}};
const Segment SGI174672[] = {{&S66075[87], 6}, {NULL}};
const Segment SGI174673[] = {{&S66076[34], 17}, {NULL}};
const Segment SGI174674[] = {{&S66076[27], 6}, {NULL}};
const Segment SGI174675[] = {{&S66076[112], 9}, {NULL}};
const Segment SGI174676[] = {{&S66076[85], 6}, {NULL}};
const Segment SGI174677[] = {{&S54902[34], 10}, {NULL}};
const Segment SGI174678[] = {{&S54902[27], 6}, {NULL}};
const Segment SGI174679[] = {{&S54902[97], 9}, {NULL}};
const Segment SGI174680[] = {{&S54902[70], 6}, {NULL}};
const Segment SGI174681[] = {{&S66077[34], 15}, {NULL}};
const Segment SGI174682[] = {{&S66077[27], 6}, {NULL}};
const Segment SGI174683[] = {{&S54903[34], 10}, {NULL}};
const Segment SGI174684[] = {{&S66077[108], 9}, {NULL}};
const Segment SGI174685[] = {{&S54903[27], 6}, {NULL}};
const Segment SGI174686[] = {{&S66077[81], 6}, {NULL}};
const Segment SGI174687[] = {{&S54903[97], 9}, {NULL}};
const Segment SGI174688[] = {{&S54903[70], 6}, {NULL}};
const Segment SGI174689[] = {{&S66078[34], 17}, {NULL}};
const Segment SGI174690[] = {{&S66078[27], 6}, {NULL}};
const Segment SGI174691[] = {{&S66078[112], 9}, {NULL}};
const Segment SGI174692[] = {{&S66078[85], 6}, {NULL}};
const Segment SGI174693[] = {{&S54905[34], 6}, {NULL}};
const Segment SGI174694[] = {{&S54905[27], 6}, {NULL}};
const Segment SGI174695[] = {{&S54905[93], 9}, {NULL}};
const Segment SGI174696[] = {{&S54905[66], 6}, {NULL}};
const Segment SGI174697[] = {{&S54906[34], 7}, {NULL}};
const Segment SGI174698[] = {{&S54906[27], 6}, {NULL}};
const Segment SGI174699[] = {{&S54906[94], 9}, {NULL}};
const Segment SGI174700[] = {{&S54906[67], 6}, {NULL}};
const Segment SGI174701[] = {{&S54907[34], 11}, {NULL}};
const Segment SGI174702[] = {{&S54907[27], 6}, {NULL}};
const Segment SGI174703[] = {{&S54907[98], 9}, {NULL}};
const Segment SGI174704[] = {{&S54907[71], 6}, {NULL}};
const Segment SGI174705[] = {{&S54909[34], 15}, {NULL}};
const Segment SGI174706[] = {{&S54909[27], 6}, {NULL}};
const Segment SGI174707[] = {{&S54909[108], 9}, {NULL}};
const Segment SGI174708[] = {{&S54909[81], 6}, {NULL}};
const Segment SGI174709[] = {{&S54910[34], 6}, {NULL}};
const Segment SGI174710[] = {{&S54910[27], 6}, {NULL}};
const Segment SGI174711[] = {{&S54910[93], 9}, {NULL}};
const Segment SGI174712[] = {{&S54910[66], 6}, {NULL}};
const Segment SGI174713[] = {{&S58236[44], 25}, {NULL}};
const Segment SGI174714[] = {{&S58236[44], 37}, {NULL}};
const Segment SGI174715[] = {{&S58236[30], 6}, {NULL}};
const Segment SGI174716[] = {{&S58236[31], 3}, {NULL}};
const Segment SGI174717[] = {{&S58236[95], 6}, {NULL}};
const Segment SGI174718[] = {{&S58237[38], 25}, {NULL}};
const Segment SGI174719[] = {{&S58237[31], 6}, {NULL}};
const Segment SGI174720[] = {{&S66095[44], 10}, {NULL}};
const Segment SGI174721[] = {{&S66095[44], 22}, {NULL}};
const Segment SGI174722[] = {{&S66095[30], 6}, {NULL}};
const Segment SGI174723[] = {{&S58238[44], 6}, {NULL}};
const Segment SGI174724[] = {{&S58238[44], 18}, {NULL}};
const Segment SGI174725[] = {{&S58238[30], 6}, {NULL}};
const Segment SGI174726[] = {{&S66095[31], 3}, {NULL}};
const Segment SGI174727[] = {{&S66095[80], 6}, {NULL}};
const Segment SGI174728[] = {{&S58238[127], 3}, {NULL}};
const Segment SGI174729[] = {{&S58238[92], 6}, {NULL}};
const Segment SGI174730[] = {{&S58238[64], 8}, {NULL}};
const Segment SGI174731[] = {{&S58239[44], 13}, {NULL}};
const Segment SGI174732[] = {{&S58239[44], 25}, {NULL}};
const Segment SGI174733[] = {{&S58239[30], 6}, {NULL}};
const Segment SGI174734[] = {{&S58239[169], 3}, {NULL}};
const Segment SGI174735[] = {{&S58239[134], 6}, {NULL}};
const Segment SGI174736[] = {{&S54920[44], 10}, {NULL}};
const Segment SGI174737[] = {{&S58239[71], 8}, {NULL}};
const Segment SGI174738[] = {{&S54920[44], 22}, {NULL}};
const Segment SGI174739[] = {{&S54920[30], 6}, {NULL}};
const Segment SGI174740[] = {{&S54920[115], 3}, {NULL}};
const Segment SGI174741[] = {{&S54920[80], 6}, {NULL}};
const Segment SGI174742[] = {{&S58240[44], 6}, {NULL}};
const Segment SGI174743[] = {{&S58240[44], 18}, {NULL}};
const Segment SGI174744[] = {{&S58240[30], 6}, {NULL}};
const Segment SGI174745[] = {{&S58240[127], 3}, {NULL}};
const Segment SGI174746[] = {{&S54921[38], 10}, {NULL}};
const Segment SGI174747[] = {{&S58240[92], 6}, {NULL}};
const Segment SGI174748[] = {{&S54921[31], 6}, {NULL}};
const Segment SGI174749[] = {{&S58240[64], 8}, {NULL}};
const Segment SGI174750[] = {{&S58241[44], 13}, {NULL}};
const Segment SGI174751[] = {{&S58241[44], 25}, {NULL}};
const Segment SGI174752[] = {{&S58241[30], 6}, {NULL}};
const Segment SGI174753[] = {{&S58241[170], 3}, {NULL}};
const Segment SGI174754[] = {{&S58241[135], 6}, {NULL}};
const Segment SGI174755[] = {{&S58241[71], 8}, {NULL}};
const Segment SGI174756[] = {{&S66109[44], 7}, {NULL}};
const Segment SGI174757[] = {{&S66109[44], 19}, {NULL}};
const Segment SGI174758[] = {{&S66109[30], 6}, {NULL}};
const Segment SGI174759[] = {{&S66109[31], 3}, {NULL}};
const Segment SGI174760[] = {{&S66109[77], 6}, {NULL}};
const Segment SGI174761[] = {{&S54930[44], 10}, {NULL}};
const Segment SGI174762[] = {{&S54930[44], 22}, {NULL}};
const Segment SGI174763[] = {{&S54930[30], 6}, {NULL}};
const Segment SGI174764[] = {{&S54930[131], 3}, {NULL}};
const Segment SGI174765[] = {{&S54930[96], 6}, {NULL}};
const Segment SGI174766[] = {{&S54930[68], 8}, {NULL}};
const Segment SGI174767[] = {{&S66111[34], 18}, {NULL}};
const Segment SGI174768[] = {{&S66111[27], 6}, {NULL}};
const Segment SGI174769[] = {{&S66111[105], 9}, {NULL}};
const Segment SGI174770[] = {{&S66111[78], 6}, {NULL}};
const Segment SGI174771[] = {{&S58261[44], 14}, {NULL}};
const Segment SGI174772[] = {{&S58261[44], 26}, {NULL}};
const Segment SGI174773[] = {{&S58261[30], 6}, {NULL}};
const Segment SGI174774[] = {{&S58261[31], 3}, {NULL}};
const Segment SGI174775[] = {{&S58261[84], 6}, {NULL}};
const Segment SGI174776[] = {{&S54944[34], 10}, {NULL}};
const Segment SGI174777[] = {{&S54944[27], 6}, {NULL}};
const Segment SGI174778[] = {{&S54944[97], 9}, {NULL}};
const Segment SGI174779[] = {{&S54944[70], 6}, {NULL}};
const Segment SGI174780[] = {{&S54956[34], 11}, {NULL}};
const Segment SGI174781[] = {{&S54956[27], 6}, {NULL}};
const Segment SGI174782[] = {{&S54956[98], 9}, {NULL}};
const Segment SGI174783[] = {{&S54956[71], 6}, {NULL}};
const Segment SGI174784[] = {{&S54957[34], 10}, {NULL}};
const Segment SGI174785[] = {{&S54957[27], 6}, {NULL}};
const Segment SGI174786[] = {{&S54957[97], 9}, {NULL}};
const Segment SGI174787[] = {{&S54957[70], 6}, {NULL}};
const Segment SGI174788[] = {{&S54960[44], 10}, {NULL}};
const Segment SGI174789[] = {{&S54960[44], 22}, {NULL}};
const Segment SGI174790[] = {{&S54960[30], 6}, {NULL}};
const Segment SGI174791[] = {{&S54960[135], 3}, {NULL}};
const Segment SGI174792[] = {{&S54960[100], 6}, {NULL}};
const Segment SGI174793[] = {{&S54960[68], 8}, {NULL}};
const Segment SGI174794[] = {{&S66142[44], 9}, {NULL}};
const Segment SGI174795[] = {{&S66142[44], 21}, {NULL}};
const Segment SGI174796[] = {{&S66142[30], 6}, {NULL}};
const Segment SGI174797[] = {{&S66142[31], 3}, {NULL}};
const Segment SGI174798[] = {{&S66142[79], 6}, {NULL}};
const Segment SGI174799[] = {{&S54962[44], 8}, {NULL}};
const Segment SGI174800[] = {{&S54962[44], 20}, {NULL}};
const Segment SGI174801[] = {{&S54962[30], 6}, {NULL}};
const Segment SGI174802[] = {{&S54962[129], 3}, {NULL}};
const Segment SGI174803[] = {{&S54962[94], 6}, {NULL}};
const Segment SGI174804[] = {{&S54962[66], 8}, {NULL}};
const Segment SGI174805[] = {{&S66143[38], 9}, {NULL}};
const Segment SGI174806[] = {{&S66143[31], 6}, {NULL}};
const Segment SGI174807[] = {{&S66150[34], 14}, {NULL}};
const Segment SGI174808[] = {{&S66150[27], 6}, {NULL}};
const Segment SGI174809[] = {{&S66150[101], 9}, {NULL}};
const Segment SGI174810[] = {{&S66150[74], 6}, {NULL}};
const Segment SGI174811[] = {{&S54985[34], 9}, {NULL}};
const Segment SGI174812[] = {{&S54985[27], 6}, {NULL}};
const Segment SGI174813[] = {{&S54985[96], 9}, {NULL}};
const Segment SGI174814[] = {{&S54985[69], 6}, {NULL}};
const Segment SGI174815[] = {{&S58304[34], 11}, {NULL}};
const Segment SGI174816[] = {{&S58304[27], 6}, {NULL}};
const Segment SGI174817[] = {{&S58304[98], 9}, {NULL}};
const Segment SGI174818[] = {{&S58304[71], 6}, {NULL}};
const Segment SGI174819[] = {{&S66160[34], 7}, {NULL}};
const Segment SGI174820[] = {{&S66160[27], 6}, {NULL}};
const Segment SGI174821[] = {{&S66160[110], 9}, {NULL}};
const Segment SGI174822[] = {{&S66160[83], 6}, {NULL}};
const Segment SGI174823[] = {{&S58305[34], 11}, {NULL}};
const Segment SGI174824[] = {{&S58305[27], 6}, {NULL}};
const Segment SGI174825[] = {{&S58305[98], 9}, {NULL}};
const Segment SGI174826[] = {{&S58305[71], 6}, {NULL}};
const Segment SGI174827[] = {{&S66161[34], 7}, {NULL}};
const Segment SGI174828[] = {{&S66161[27], 6}, {NULL}};
const Segment SGI174829[] = {{&S66161[110], 9}, {NULL}};
const Segment SGI174830[] = {{&S66161[83], 6}, {NULL}};
const Segment SGI174831[] = {{&S58306[44], 25}, {NULL}};
const Segment SGI174832[] = {{&S58306[44], 37}, {NULL}};
const Segment SGI174833[] = {{&S58306[30], 6}, {NULL}};
const Segment SGI174834[] = {{&S58306[31], 3}, {NULL}};
const Segment SGI174835[] = {{&S58306[95], 6}, {NULL}};
const Segment SGI174836[] = {{&S58307[38], 25}, {NULL}};
const Segment SGI174837[] = {{&S58307[31], 6}, {NULL}};
const Segment SGI174838[] = {{&S54993[34], 6}, {NULL}};
const Segment SGI174839[] = {{&S54993[27], 6}, {NULL}};
const Segment SGI174840[] = {{&S54993[93], 9}, {NULL}};
const Segment SGI174841[] = {{&S54993[66], 6}, {NULL}};
const Segment SGI174842[] = {{&S54996[34], 6}, {NULL}};
const Segment SGI174843[] = {{&S54996[27], 6}, {NULL}};
const Segment SGI174844[] = {{&S54996[93], 9}, {NULL}};
const Segment SGI174845[] = {{&S54996[66], 6}, {NULL}};
const Segment SGI174846[] = {{&S54997[34], 6}, {NULL}};
const Segment SGI174847[] = {{&S54997[27], 6}, {NULL}};
const Segment SGI174848[] = {{&S54997[93], 9}, {NULL}};
const Segment SGI174849[] = {{&S54997[66], 6}, {NULL}};
const Segment SGI174850[] = {{&S54999[34], 7}, {NULL}};
const Segment SGI174851[] = {{&S54999[27], 6}, {NULL}};
const Segment SGI174852[] = {{&S54999[94], 9}, {NULL}};
const Segment SGI174853[] = {{&S54999[67], 6}, {NULL}};
const Segment SGI174854[] = {{&S66174[34], 10}, {NULL}};
const Segment SGI174855[] = {{&S66174[27], 6}, {NULL}};
const Segment SGI174856[] = {{&S66174[97], 9}, {NULL}};
const Segment SGI174857[] = {{&S66174[70], 6}, {NULL}};
const Segment SGI174858[] = {{&S55001[34], 10}, {NULL}};
const Segment SGI174859[] = {{&S55001[27], 6}, {NULL}};
const Segment SGI174860[] = {{&S55001[97], 9}, {NULL}};
const Segment SGI174861[] = {{&S55001[70], 6}, {NULL}};
const Segment SGI174862[] = {{&S66176[34], 7}, {NULL}};
const Segment SGI174863[] = {{&S66176[27], 6}, {NULL}};
const Segment SGI174864[] = {{&S55003[34], 11}, {NULL}};
const Segment SGI174865[] = {{&S55003[27], 6}, {NULL}};
const Segment SGI174866[] = {{&S66176[96], 9}, {NULL}};
const Segment SGI174867[] = {{&S66176[69], 6}, {NULL}};
const Segment SGI174868[] = {{&S55003[98], 9}, {NULL}};
const Segment SGI174869[] = {{&S55003[71], 6}, {NULL}};
const Segment SGI174870[] = {{&S66178[34], 10}, {NULL}};
const Segment SGI174871[] = {{&S55005[34], 7}, {NULL}};
const Segment SGI174872[] = {{&S66178[27], 6}, {NULL}};
const Segment SGI174873[] = {{&S55005[27], 6}, {NULL}};
const Segment SGI174874[] = {{&S66178[101], 9}, {NULL}};
const Segment SGI174875[] = {{&S55005[94], 9}, {NULL}};
const Segment SGI174876[] = {{&S66178[74], 6}, {NULL}};
const Segment SGI174877[] = {{&S55005[67], 6}, {NULL}};
const Segment SGI174878[] = {{&S58329[34], 15}, {NULL}};
const Segment SGI174879[] = {{&S58329[27], 6}, {NULL}};
const Segment SGI174880[] = {{&S66181[44], 11}, {NULL}};
const Segment SGI174881[] = {{&S58329[102], 9}, {NULL}};
const Segment SGI174882[] = {{&S66181[44], 23}, {NULL}};
const Segment SGI174883[] = {{&S58329[75], 6}, {NULL}};
const Segment SGI174884[] = {{&S66181[30], 6}, {NULL}};
const Segment SGI174885[] = {{&S66181[31], 3}, {NULL}};
const Segment SGI174886[] = {{&S66181[81], 6}, {NULL}};
const Segment SGI174887[] = {{&S58332[34], 6}, {NULL}};
const Segment SGI174888[] = {{&S58332[27], 6}, {NULL}};
const Segment SGI174889[] = {{&S55012[34], 5}, {NULL}};
const Segment SGI174890[] = {{&S58332[93], 9}, {NULL}};
const Segment SGI174891[] = {{&S58332[66], 6}, {NULL}};
const Segment SGI174892[] = {{&S55012[27], 6}, {NULL}};
const Segment SGI174893[] = {{&S55012[92], 9}, {NULL}};
const Segment SGI174894[] = {{&S55012[65], 6}, {NULL}};
const Segment SGI174895[] = {{&S58333[34], 17}, {NULL}};
const Segment SGI174896[] = {{&S58333[27], 6}, {NULL}};
const Segment SGI174897[] = {{&S58333[104], 9}, {NULL}};
const Segment SGI174898[] = {{&S58333[77], 6}, {NULL}};
const Segment SGI174899[] = {{&S55015[34], 10}, {NULL}};
const Segment SGI174900[] = {{&S55015[27], 6}, {NULL}};
const Segment SGI174901[] = {{&S55015[97], 9}, {NULL}};
const Segment SGI174902[] = {{&S55015[70], 6}, {NULL}};
const Segment SGI174903[] = {{&S58336[34], 6}, {NULL}};
const Segment SGI174904[] = {{&S58336[27], 6}, {NULL}};
const Segment SGI174905[] = {{&S58336[93], 9}, {NULL}};
const Segment SGI174906[] = {{&S58336[66], 6}, {NULL}};
const Segment SGI174907[] = {{&S58337[34], 13}, {NULL}};
const Segment SGI174908[] = {{&S58337[27], 6}, {NULL}};
const Segment SGI174909[] = {{&S58337[100], 9}, {NULL}};
const Segment SGI174910[] = {{&S58337[73], 6}, {NULL}};
const Segment SGI174911[] = {{&S55020[34], 16}, {NULL}};
const Segment SGI174912[] = {{&S58338[34], 7}, {NULL}};
const Segment SGI174913[] = {{&S55020[27], 6}, {NULL}};
const Segment SGI174914[] = {{&S58338[27], 6}, {NULL}};
const Segment SGI174915[] = {{&S55020[103], 9}, {NULL}};
const Segment SGI174916[] = {{&S55020[76], 6}, {NULL}};
const Segment SGI174917[] = {{&S58338[94], 9}, {NULL}};
const Segment SGI174918[] = {{&S58338[67], 6}, {NULL}};
const Segment SGI174919[] = {{&S66194[34], 14}, {NULL}};
const Segment SGI174920[] = {{&S66194[27], 6}, {NULL}};
const Segment SGI174921[] = {{&S55021[34], 10}, {NULL}};
const Segment SGI174922[] = {{&S55021[27], 6}, {NULL}};
const Segment SGI174923[] = {{&S66194[101], 9}, {NULL}};
const Segment SGI174924[] = {{&S66194[74], 6}, {NULL}};
const Segment SGI174925[] = {{&S55021[97], 9}, {NULL}};
const Segment SGI174926[] = {{&S55021[70], 6}, {NULL}};
const Segment SGI174927[] = {{&S58340[34], 4}, {NULL}};
const Segment SGI174928[] = {{&S58340[27], 6}, {NULL}};
const Segment SGI174929[] = {{&S58340[91], 9}, {NULL}};
const Segment SGI174930[] = {{&S58340[64], 6}, {NULL}};
const Segment SGI174931[] = {{&S55024[34], 4}, {NULL}};
const Segment SGI174932[] = {{&S55024[27], 6}, {NULL}};
const Segment SGI174933[] = {{&S58342[34], 8}, {NULL}};
const Segment SGI174934[] = {{&S58342[27], 6}, {NULL}};
const Segment SGI174935[] = {{&S55024[91], 9}, {NULL}};
const Segment SGI174936[] = {{&S55024[64], 6}, {NULL}};
const Segment SGI174937[] = {{&S58342[95], 9}, {NULL}};
const Segment SGI174938[] = {{&S58342[68], 6}, {NULL}};
const Segment SGI174939[] = {{&S58343[34], 9}, {NULL}};
const Segment SGI174940[] = {{&S58343[27], 6}, {NULL}};
const Segment SGI174941[] = {{&S58343[96], 9}, {NULL}};
const Segment SGI174942[] = {{&S55026[34], 6}, {NULL}};
const Segment SGI174943[] = {{&S58343[69], 6}, {NULL}};
const Segment SGI174944[] = {{&S55026[27], 6}, {NULL}};
const Segment SGI174945[] = {{&S55026[93], 9}, {NULL}};
const Segment SGI174946[] = {{&S55026[66], 6}, {NULL}};
const Segment SGI174947[] = {{&S58344[34], 13}, {NULL}};
const Segment SGI174948[] = {{&S58344[27], 6}, {NULL}};
const Segment SGI174949[] = {{&S58344[100], 9}, {NULL}};
const Segment SGI174950[] = {{&S58344[73], 6}, {NULL}};
const Segment SGI174951[] = {{&S55028[34], 6}, {NULL}};
const Segment SGI174952[] = {{&S55028[27], 6}, {NULL}};
const Segment SGI174953[] = {{&S58345[34], 10}, {NULL}};
const Segment SGI174954[] = {{&S58345[27], 6}, {NULL}};
const Segment SGI174955[] = {{&S55028[93], 9}, {NULL}};
const Segment SGI174956[] = {{&S55028[66], 6}, {NULL}};
const Segment SGI174957[] = {{&S58345[97], 9}, {NULL}};
const Segment SGI174958[] = {{&S58345[70], 6}, {NULL}};
const Segment SGI174959[] = {{&S55029[34], 6}, {NULL}};
const Segment SGI174960[] = {{&S55029[27], 6}, {NULL}};
const Segment SGI174961[] = {{&S58346[34], 8}, {NULL}};
const Segment SGI174962[] = {{&S55029[93], 9}, {NULL}};
const Segment SGI174963[] = {{&S58346[27], 6}, {NULL}};
const Segment SGI174964[] = {{&S55029[66], 6}, {NULL}};
const Segment SGI174965[] = {{&S58346[95], 9}, {NULL}};
const Segment SGI174966[] = {{&S58346[68], 6}, {NULL}};
const Segment SGI174967[] = {{&S55030[34], 7}, {NULL}};
const Segment SGI174968[] = {{&S55030[27], 6}, {NULL}};
const Segment SGI174969[] = {{&S55030[94], 9}, {NULL}};
const Segment SGI174970[] = {{&S55030[67], 6}, {NULL}};
const Segment SGI174971[] = {{&S55031[34], 18}, {NULL}};
const Segment SGI174972[] = {{&S55031[27], 6}, {NULL}};
const Segment SGI174973[] = {{&S55031[105], 9}, {NULL}};
const Segment SGI174974[] = {{&S55031[78], 6}, {NULL}};
const Segment SGI174975[] = {{&S58351[34], 11}, {NULL}};
const Segment SGI174976[] = {{&S58351[27], 6}, {NULL}};
const Segment SGI174977[] = {{&S55032[34], 10}, {NULL}};
const Segment SGI174978[] = {{&S55032[27], 6}, {NULL}};
const Segment SGI174979[] = {{&S58351[98], 9}, {NULL}};
const Segment SGI174980[] = {{&S58351[71], 6}, {NULL}};
const Segment SGI174981[] = {{&S55032[105], 9}, {NULL}};
const Segment SGI174982[] = {{&S55032[78], 6}, {NULL}};
const Segment SGI174983[] = {{&S66212[45], 8}, {NULL}};
const Segment SGI174984[] = {{&S66212[30], 6}, {NULL}};
const Segment SGI174985[] = {{&S66212[31], 3}, {NULL}};
const Segment SGI174986[] = {{&S66212[83], 5}, {NULL}};
const Segment SGI174987[] = {{&S55033[34], 8}, {NULL}};
const Segment SGI174988[] = {{&S66212[141], 13}, {NULL}};
const Segment SGI174989[] = {{&S55033[27], 6}, {NULL}};
const Segment SGI174990[] = {{&S55033[95], 9}, {NULL}};
const Segment SGI174991[] = {{&S55033[68], 6}, {NULL}};
const Segment SGI174992[] = {{&S58353[34], 7}, {NULL}};
const Segment SGI174993[] = {{&S58353[27], 6}, {NULL}};
const Segment SGI174994[] = {{&S66213[45], 7}, {NULL}};
const Segment SGI174995[] = {{&S66213[30], 6}, {NULL}};
const Segment SGI174996[] = {{&S58353[94], 9}, {NULL}};
const Segment SGI174997[] = {{&S58353[67], 6}, {NULL}};
const Segment SGI174998[] = {{&S66213[31], 3}, {NULL}};
const Segment SGI174999[] = {{&S66213[83], 5}, {NULL}};
const Segment SGI175000[] = {{&S66213[141], 13}, {NULL}};
const Segment SGI175001[] = {{&S58354[34], 14}, {NULL}};
const Segment SGI175002[] = {{&S58354[27], 6}, {NULL}};
const Segment SGI175003[] = {{&S58354[101], 9}, {NULL}};
const Segment SGI175004[] = {{&S66214[44], 8}, {NULL}};
const Segment SGI175005[] = {{&S58354[74], 6}, {NULL}};
const Segment SGI175006[] = {{&S66214[44], 24}, {NULL}};
const Segment SGI175007[] = {{&S66214[30], 6}, {NULL}};
const Segment SGI175008[] = {{&S66214[117], 3}, {NULL}};
const Segment SGI175009[] = {{&S66214[82], 6}, {NULL}};
const Segment SGI175010[] = {{&S66214[137], 8}, {NULL}};
const Segment SGI175011[] = {{&S66214[146], 13}, {NULL}};
const Segment SGI175012[] = {{&S58355[34], 12}, {NULL}};
const Segment SGI175013[] = {{&S58355[27], 6}, {NULL}};
const Segment SGI175014[] = {{&S58355[94], 9}, {NULL}};
const Segment SGI175015[] = {{&S58355[67], 6}, {NULL}};
const Segment SGI175016[] = {{&S66215[44], 7}, {NULL}};
const Segment SGI175017[] = {{&S66215[44], 24}, {NULL}};
const Segment SGI175018[] = {{&S66215[30], 6}, {NULL}};
const Segment SGI175019[] = {{&S66215[117], 3}, {NULL}};
const Segment SGI175020[] = {{&S66215[82], 6}, {NULL}};
const Segment SGI175021[] = {{&S66215[137], 8}, {NULL}};
const Segment SGI175022[] = {{&S66215[146], 13}, {NULL}};
const Segment SGI175023[] = {{&S58356[34], 4}, {NULL}};
const Segment SGI175024[] = {{&S58356[27], 6}, {NULL}};
const Segment SGI175025[] = {{&S58356[91], 9}, {NULL}};
const Segment SGI175026[] = {{&S58356[64], 6}, {NULL}};
const Segment SGI175027[] = {{&S66218[34], 12}, {NULL}};
const Segment SGI175028[] = {{&S66218[27], 6}, {NULL}};
const Segment SGI175029[] = {{&S66218[99], 9}, {NULL}};
const Segment SGI175030[] = {{&S66218[72], 6}, {NULL}};
const Segment SGI175031[] = {{&S58359[34], 13}, {NULL}};
const Segment SGI175032[] = {{&S58359[27], 6}, {NULL}};
const Segment SGI175033[] = {{&S58359[100], 9}, {NULL}};
const Segment SGI175034[] = {{&S58359[73], 6}, {NULL}};
const Segment SGI175035[] = {{&S58361[34], 8}, {NULL}};
const Segment SGI175036[] = {{&S58361[27], 6}, {NULL}};
const Segment SGI175037[] = {{&S58361[95], 9}, {NULL}};
const Segment SGI175038[] = {{&S58361[68], 6}, {NULL}};
const Segment SGI175039[] = {{&S58362[34], 11}, {NULL}};
const Segment SGI175040[] = {{&S58362[27], 6}, {NULL}};
const Segment SGI175041[] = {{&S58362[98], 9}, {NULL}};
const Segment SGI175042[] = {{&S58362[71], 6}, {NULL}};
const Segment SGI175043[] = {{&S58364[34], 10}, {NULL}};
const Segment SGI175044[] = {{&S58364[27], 6}, {NULL}};
const Segment SGI175045[] = {{&S58364[99], 9}, {NULL}};
const Segment SGI175046[] = {{&S58364[72], 6}, {NULL}};
const Segment SGI175047[] = {{&S66240[34], 9}, {NULL}};
const Segment SGI175048[] = {{&S66240[27], 6}, {NULL}};
const Segment SGI175049[] = {{&S66240[96], 9}, {NULL}};
const Segment SGI175050[] = {{&S66240[69], 6}, {NULL}};
const Segment SGI175051[] = {{&S58377[44], 10}, {NULL}};
const Segment SGI175052[] = {{&S58377[44], 27}, {NULL}};
const Segment SGI175053[] = {{&S58377[30], 6}, {NULL}};
const Segment SGI175054[] = {{&S58377[119], 3}, {NULL}};
const Segment SGI175055[] = {{&S58377[85], 5}, {NULL}};
const Segment SGI175056[] = {{&S58378[38], 10}, {NULL}};
const Segment SGI175057[] = {{&S58378[31], 6}, {NULL}};
const Segment SGI175058[] = {{&S66279[44], 6}, {NULL}};
const Segment SGI175059[] = {{&S66279[44], 18}, {NULL}};
const Segment SGI175060[] = {{&S66279[30], 6}, {NULL}};
const Segment SGI175061[] = {{&S66279[31], 3}, {NULL}};
const Segment SGI175062[] = {{&S66279[76], 6}, {NULL}};
const Segment SGI175063[] = {{&S66281[34], 10}, {NULL}};
const Segment SGI175064[] = {{&S66281[27], 6}, {NULL}};
const Segment SGI175065[] = {{&S66281[97], 9}, {NULL}};
const Segment SGI175066[] = {{&S66281[70], 6}, {NULL}};
const Segment SGI175067[] = {{&S43469[44], 10}, {NULL}};
const Segment SGI175068[] = {{&S43469[44], 21}, {NULL}};
const Segment SGI175069[] = {{&S43469[30], 6}, {NULL}};
const Segment SGI175070[] = {{&S43469[114], 3}, {NULL}};
const Segment SGI175071[] = {{&S43469[79], 6}, {NULL}};
const Segment SGI175072[] = {{&S66283[34], 11}, {NULL}};
const Segment SGI175073[] = {{&S66283[27], 6}, {NULL}};
const Segment SGI175074[] = {{&S66283[98], 9}, {NULL}};
const Segment SGI175075[] = {{&S66283[71], 6}, {NULL}};
const Segment SGI175076[] = {{&S43471[44], 10}, {NULL}};
const Segment SGI175077[] = {{&S43471[44], 21}, {NULL}};
const Segment SGI175078[] = {{&S43471[30], 6}, {NULL}};
const Segment SGI175079[] = {{&S43471[114], 3}, {NULL}};
const Segment SGI175080[] = {{&S43471[79], 6}, {NULL}};
const Segment SGI175081[] = {{&S66285[44], 10}, {NULL}};
const Segment SGI175082[] = {{&S66285[44], 22}, {NULL}};
const Segment SGI175083[] = {{&S66285[30], 6}, {NULL}};
const Segment SGI175084[] = {{&S43472[44], 18}, {NULL}};
const Segment SGI175085[] = {{&S66285[31], 3}, {NULL}};
const Segment SGI175086[] = {{&S43472[44], 29}, {NULL}};
const Segment SGI175087[] = {{&S43472[30], 6}, {NULL}};
const Segment SGI175088[] = {{&S66285[80], 6}, {NULL}};
const Segment SGI175089[] = {{&S43472[122], 3}, {NULL}};
const Segment SGI175090[] = {{&S43472[87], 6}, {NULL}};
const Segment SGI175091[] = {{&S66286[38], 10}, {NULL}};
const Segment SGI175092[] = {{&S66286[31], 6}, {NULL}};
const Segment SGI175093[] = {{&S43473[44], 11}, {NULL}};
const Segment SGI175094[] = {{&S43473[44], 22}, {NULL}};
const Segment SGI175095[] = {{&S43473[30], 6}, {NULL}};
const Segment SGI175096[] = {{&S43473[115], 3}, {NULL}};
const Segment SGI175097[] = {{&S43473[80], 6}, {NULL}};
const Segment SGI175098[] = {{&S43474[38], 10}, {NULL}};
const Segment SGI175099[] = {{&S43474[31], 6}, {NULL}};
const Segment SGI175100[] = {{&S43475[44], 18}, {NULL}};
const Segment SGI175101[] = {{&S43475[44], 30}, {NULL}};
const Segment SGI175102[] = {{&S43475[30], 6}, {NULL}};
const Segment SGI175103[] = {{&S43475[123], 3}, {NULL}};
const Segment SGI175104[] = {{&S43475[88], 6}, {NULL}};
const Segment SGI175105[] = {{&S43476[44], 10}, {NULL}};
const Segment SGI175106[] = {{&S43476[44], 22}, {NULL}};
const Segment SGI175107[] = {{&S43476[30], 6}, {NULL}};
const Segment SGI175108[] = {{&S43476[115], 3}, {NULL}};
const Segment SGI175109[] = {{&S43476[80], 6}, {NULL}};
const Segment SGI175110[] = {{&S43477[38], 18}, {NULL}};
const Segment SGI175111[] = {{&S43477[31], 6}, {NULL}};
const Segment SGI175112[] = {{&S43478[38], 10}, {NULL}};
const Segment SGI175113[] = {{&S43478[31], 6}, {NULL}};
const Segment SGI175114[] = {{&S43479[44], 10}, {NULL}};
const Segment SGI175115[] = {{&S43479[44], 21}, {NULL}};
const Segment SGI175116[] = {{&S43479[30], 6}, {NULL}};
const Segment SGI175117[] = {{&S43479[114], 3}, {NULL}};
const Segment SGI175118[] = {{&S43479[79], 6}, {NULL}};
const Segment SGI175119[] = {{&S66300[44], 12}, {NULL}};
const Segment SGI175120[] = {{&S66300[44], 24}, {NULL}};
const Segment SGI175121[] = {{&S66300[30], 6}, {NULL}};
const Segment SGI175122[] = {{&S66300[31], 3}, {NULL}};
const Segment SGI175123[] = {{&S66300[82], 6}, {NULL}};
const Segment SGI175124[] = {{&S43482[44], 10}, {NULL}};
const Segment SGI175125[] = {{&S43482[44], 22}, {NULL}};
const Segment SGI175126[] = {{&S43482[30], 6}, {NULL}};
const Segment SGI175127[] = {{&S43482[115], 3}, {NULL}};
const Segment SGI175128[] = {{&S43482[80], 6}, {NULL}};
const Segment SGI175129[] = {{&S43483[44], 11}, {NULL}};
const Segment SGI175130[] = {{&S43483[44], 23}, {NULL}};
const Segment SGI175131[] = {{&S43483[30], 6}, {NULL}};
const Segment SGI175132[] = {{&S43483[116], 3}, {NULL}};
const Segment SGI175133[] = {{&S43483[81], 6}, {NULL}};
const Segment SGI175134[] = {{&S43484[38], 10}, {NULL}};
const Segment SGI175135[] = {{&S43484[31], 6}, {NULL}};
const Segment SGI175136[] = {{&S66313[34], 7}, {NULL}};
const Segment SGI175137[] = {{&S66313[27], 6}, {NULL}};
const Segment SGI175138[] = {{&S66313[105], 9}, {NULL}};
const Segment SGI175139[] = {{&S66313[78], 6}, {NULL}};
const Segment SGI175140[] = {{&S43500[34], 10}, {NULL}};
const Segment SGI175141[] = {{&S43500[27], 6}, {NULL}};
const Segment SGI175142[] = {{&S58467[44], 10}, {NULL}};
const Segment SGI175143[] = {{&S58467[44], 22}, {NULL}};
const Segment SGI175144[] = {{&S58467[30], 6}, {NULL}};
const Segment SGI175145[] = {{&S43500[96], 9}, {NULL}};
const Segment SGI175146[] = {{&S43500[69], 6}, {NULL}};
const Segment SGI175147[] = {{&S55035[34], 6}, {NULL}};
const Segment SGI175148[] = {{&S55035[27], 6}, {NULL}};
const Segment SGI175149[] = {{&S55035[93], 9}, {NULL}};
const Segment SGI175150[] = {{&S55035[66], 6}, {NULL}};
const Segment SGI175151[] = {{&S66323[34], 10}, {NULL}};
const Segment SGI175152[] = {{&S66323[27], 6}, {NULL}};
const Segment SGI175153[] = {{&S55036[34], 16}, {NULL}};
const Segment SGI175154[] = {{&S66323[97], 9}, {NULL}};
const Segment SGI175155[] = {{&S66323[70], 6}, {NULL}};
const Segment SGI175156[] = {{&S55036[27], 6}, {NULL}};
const Segment SGI175157[] = {{&S55036[103], 9}, {NULL}};
const Segment SGI175158[] = {{&S55036[76], 6}, {NULL}};
const Segment SGI175159[] = {{&S55040[34], 10}, {NULL}};
const Segment SGI175160[] = {{&S52915[34], 10}, {NULL}};
const Segment SGI175161[] = {{&S55040[27], 6}, {NULL}};
const Segment SGI175162[] = {{&S52915[27], 6}, {NULL}};
const Segment SGI175163[] = {{&S55040[97], 9}, {NULL}};
const Segment SGI175164[] = {{&S52915[97], 9}, {NULL}};
const Segment SGI175165[] = {{&S55040[70], 6}, {NULL}};
const Segment SGI175166[] = {{&S52915[70], 6}, {NULL}};
const Segment SGI175167[] = {{&S66329[44], 14}, {NULL}};
const Segment SGI175168[] = {{&S66329[44], 26}, {NULL}};
const Segment SGI175169[] = {{&S66329[30], 6}, {NULL}};
const Segment SGI175170[] = {{&S66329[31], 3}, {NULL}};
const Segment SGI175171[] = {{&S66329[84], 6}, {NULL}};
const Segment SGI175172[] = {{&S55042[34], 7}, {NULL}};
const Segment SGI175173[] = {{&S55042[27], 6}, {NULL}};
const Segment SGI175174[] = {{&S55042[100], 9}, {NULL}};
const Segment SGI175175[] = {{&S55042[73], 6}, {NULL}};
const Segment SGI175176[] = {{&S55047[44], 12}, {NULL}};
const Segment SGI175177[] = {{&S55047[44], 24}, {NULL}};
const Segment SGI175178[] = {{&S55047[30], 6}, {NULL}};
const Segment SGI175179[] = {{&S55047[117], 3}, {NULL}};
const Segment SGI175180[] = {{&S55047[82], 6}, {NULL}};
const Segment SGI175181[] = {{&S55048[38], 12}, {NULL}};
const Segment SGI175182[] = {{&S55048[31], 6}, {NULL}};
const Segment SGI175183[] = {{&S55049[44], 16}, {NULL}};
const Segment SGI175184[] = {{&S55049[44], 28}, {NULL}};
const Segment SGI175185[] = {{&S55049[30], 6}, {NULL}};
const Segment SGI175186[] = {{&S55049[121], 3}, {NULL}};
const Segment SGI175187[] = {{&S55049[86], 6}, {NULL}};
const Segment SGI175188[] = {{&S52930[34], 9}, {NULL}};
const Segment SGI175189[] = {{&S52930[27], 6}, {NULL}};
const Segment SGI175190[] = {{&S52930[96], 9}, {NULL}};
const Segment SGI175191[] = {{&S52930[69], 6}, {NULL}};
const Segment SGI175192[] = {{&S66342[34], 24}, {NULL}};
const Segment SGI175193[] = {{&S66342[27], 6}, {NULL}};
const Segment SGI175194[] = {{&S66342[111], 9}, {NULL}};
const Segment SGI175195[] = {{&S66342[84], 6}, {NULL}};
const Segment SGI175196[] = {{&S66343[34], 23}, {NULL}};
const Segment SGI175197[] = {{&S66343[27], 6}, {NULL}};
const Segment SGI175198[] = {{&S66343[110], 9}, {NULL}};
const Segment SGI175199[] = {{&S66343[83], 6}, {NULL}};
const Segment SGI175200[] = {{&S66344[34], 15}, {NULL}};
const Segment SGI175201[] = {{&S66344[27], 6}, {NULL}};
const Segment SGI175202[] = {{&S66344[102], 9}, {NULL}};
const Segment SGI175203[] = {{&S66344[75], 6}, {NULL}};
const Segment SGI175204[] = {{&S43533[34], 10}, {NULL}};
const Segment SGI175205[] = {{&S43533[27], 6}, {NULL}};
const Segment SGI175206[] = {{&S43533[96], 9}, {NULL}};
const Segment SGI175207[] = {{&S43533[69], 6}, {NULL}};
const Segment SGI175208[] = {{&S66350[44], 13}, {NULL}};
const Segment SGI175209[] = {{&S66350[44], 25}, {NULL}};
const Segment SGI175210[] = {{&S66350[30], 6}, {NULL}};
const Segment SGI175211[] = {{&S66350[31], 3}, {NULL}};
const Segment SGI175212[] = {{&S66350[137], 6}, {NULL}};
const Segment SGI175213[] = {{&S66350[71], 8}, {NULL}};
const Segment SGI175214[] = {{&S66351[44], 13}, {NULL}};
const Segment SGI175215[] = {{&S66351[44], 25}, {NULL}};
const Segment SGI175216[] = {{&S66351[30], 6}, {NULL}};
const Segment SGI175217[] = {{&S66351[31], 3}, {NULL}};
const Segment SGI175218[] = {{&S66351[137], 6}, {NULL}};
const Segment SGI175219[] = {{&S66351[71], 8}, {NULL}};
const Segment SGI175220[] = {{&S66352[44], 13}, {NULL}};
const Segment SGI175221[] = {{&S66352[44], 25}, {NULL}};
const Segment SGI175222[] = {{&S66352[30], 6}, {NULL}};
const Segment SGI175223[] = {{&S66352[31], 3}, {NULL}};
const Segment SGI175224[] = {{&S66352[137], 6}, {NULL}};
const Segment SGI175225[] = {{&S66352[71], 8}, {NULL}};
const Segment SGI175226[] = {{&S66353[44], 6}, {NULL}};
const Segment SGI175227[] = {{&S66353[44], 18}, {NULL}};
const Segment SGI175228[] = {{&S66353[30], 6}, {NULL}};
const Segment SGI175229[] = {{&S66353[31], 3}, {NULL}};
const Segment SGI175230[] = {{&S66353[91], 6}, {NULL}};
const Segment SGI175231[] = {{&S66353[64], 8}, {NULL}};
const Segment SGI175232[] = {{&S66354[44], 6}, {NULL}};
const Segment SGI175233[] = {{&S66354[44], 17}, {NULL}};
const Segment SGI175234[] = {{&S66354[30], 6}, {NULL}};
const Segment SGI175235[] = {{&S66354[31], 3}, {NULL}};
const Segment SGI175236[] = {{&S66354[96], 6}, {NULL}};
const Segment SGI175237[] = {{&S66354[63], 8}, {NULL}};
const Segment SGI175238[] = {{&S66357[34], 15}, {NULL}};
const Segment SGI175239[] = {{&S66357[27], 6}, {NULL}};
const Segment SGI175240[] = {{&S66357[102], 9}, {NULL}};
const Segment SGI175241[] = {{&S66357[75], 6}, {NULL}};
const Segment SGI175242[] = {{&S43563[38], 10}, {NULL}};
const Segment SGI175243[] = {{&S43563[31], 6}, {NULL}};
const Segment SGI175244[] = {{&S43564[44], 10}, {NULL}};
const Segment SGI175245[] = {{&S43564[44], 21}, {NULL}};
const Segment SGI175246[] = {{&S43564[30], 6}, {NULL}};
const Segment SGI175247[] = {{&S43564[114], 3}, {NULL}};
const Segment SGI175248[] = {{&S43564[79], 6}, {NULL}};
const Segment SGI175249[] = {{&S43565[42], 10}, {NULL}};
const Segment SGI175250[] = {{&S43565[30], 6}, {NULL}};
const Segment SGI175251[] = {{&S43565[112], 3}, {NULL}};
const Segment SGI175252[] = {{&S43565[77], 6}, {NULL}};
const Segment SGI175253[] = {{&S66371[44], 17}, {NULL}};
const Segment SGI175254[] = {{&S66371[44], 38}, {NULL}};
const Segment SGI175255[] = {{&S66371[30], 6}, {NULL}};
const Segment SGI175256[] = {{&S43566[44], 10}, {NULL}};
const Segment SGI175257[] = {{&S66371[31], 3}, {NULL}};
const Segment SGI175258[] = {{&S43566[44], 22}, {NULL}};
const Segment SGI175259[] = {{&S43566[30], 6}, {NULL}};
const Segment SGI175260[] = {{&S66371[96], 5}, {NULL}};
const Segment SGI175261[] = {{&S43566[115], 3}, {NULL}};
const Segment SGI175262[] = {{&S43566[80], 6}, {NULL}};
const Segment SGI175263[] = {{&S52971[44], 19}, {NULL}};
const Segment SGI175264[] = {{&S52971[44], 31}, {NULL}};
const Segment SGI175265[] = {{&S52971[30], 6}, {NULL}};
const Segment SGI175266[] = {{&S52971[124], 3}, {NULL}};
const Segment SGI175267[] = {{&S52971[89], 6}, {NULL}};
const Segment SGI175268[] = {{&S52972[44], 19}, {NULL}};
const Segment SGI175269[] = {{&S52972[44], 33}, {NULL}};
const Segment SGI175270[] = {{&S52972[30], 6}, {NULL}};
const Segment SGI175271[] = {{&S66374[44], 17}, {NULL}};
const Segment SGI175272[] = {{&S66374[44], 35}, {NULL}};
const Segment SGI175273[] = {{&S66374[30], 6}, {NULL}};
const Segment SGI175274[] = {{&S52972[126], 3}, {NULL}};
const Segment SGI175275[] = {{&S52972[91], 6}, {NULL}};
const Segment SGI175276[] = {{&S66374[31], 3}, {NULL}};
const Segment SGI175277[] = {{&S66374[93], 5}, {NULL}};
const Segment SGI175278[] = {{&S58467[115], 3}, {NULL}};
const Segment SGI175279[] = {{&S58467[80], 6}, {NULL}};
const Segment SGI175280[] = {{&S58471[38], 10}, {NULL}};
const Segment SGI175281[] = {{&S58471[31], 6}, {NULL}};
const Segment SGI175282[] = {{&S58472[44], 10}, {NULL}};
const Segment SGI175283[] = {{&S58472[44], 22}, {NULL}};
const Segment SGI175284[] = {{&S58472[30], 6}, {NULL}};
const Segment SGI175285[] = {{&S58472[83], 6}, {NULL}};
const Segment SGI175286[] = {{&S55109[34], 10}, {NULL}};
const Segment SGI175287[] = {{&S55109[27], 6}, {NULL}};
const Segment SGI175288[] = {{&S55109[97], 9}, {NULL}};
const Segment SGI175289[] = {{&S55109[70], 6}, {NULL}};
const Segment SGI175290[] = {{&S55111[34], 7}, {NULL}};
const Segment SGI175291[] = {{&S55111[27], 6}, {NULL}};
const Segment SGI175292[] = {{&S55111[99], 9}, {NULL}};
const Segment SGI175293[] = {{&S55111[72], 6}, {NULL}};
const Segment SGI175294[] = {{&S55112[34], 7}, {NULL}};
const Segment SGI175295[] = {{&S55112[27], 6}, {NULL}};
const Segment SGI175296[] = {{&S55112[99], 9}, {NULL}};
const Segment SGI175297[] = {{&S55112[72], 6}, {NULL}};
const Segment SGI175298[] = {{&S52994[44], 10}, {NULL}};
const Segment SGI175299[] = {{&S52994[44], 22}, {NULL}};
const Segment SGI175300[] = {{&S52994[30], 6}, {NULL}};
const Segment SGI175301[] = {{&S52994[115], 3}, {NULL}};
const Segment SGI175302[] = {{&S52994[80], 6}, {NULL}};
const Segment SGI175303[] = {{&S52995[38], 10}, {NULL}};
const Segment SGI175304[] = {{&S52995[31], 6}, {NULL}};
const Segment SGI175305[] = {{&S52996[44], 10}, {NULL}};
const Segment SGI175306[] = {{&S52996[44], 22}, {NULL}};
const Segment SGI175307[] = {{&S52996[30], 6}, {NULL}};
const Segment SGI175308[] = {{&S52996[115], 3}, {NULL}};
const Segment SGI175309[] = {{&S52996[80], 6}, {NULL}};
const Segment SGI175310[] = {{&S55128[34], 7}, {NULL}};
const Segment SGI175311[] = {{&S55128[27], 6}, {NULL}};
const Segment SGI175312[] = {{&S55128[98], 9}, {NULL}};
const Segment SGI175313[] = {{&S55128[71], 6}, {NULL}};
const Segment SGI175314[] = {{&S58498[34], 10}, {NULL}};
const Segment SGI175315[] = {{&S58498[27], 6}, {NULL}};
const Segment SGI175316[] = {{&S58498[102], 9}, {NULL}};
const Segment SGI175317[] = {{&S58498[75], 6}, {NULL}};
const Segment SGI175318[] = {{&S55130[34], 6}, {NULL}};
const Segment SGI175319[] = {{&S55130[27], 6}, {NULL}};
const Segment SGI175320[] = {{&S66416[34], 10}, {NULL}};
const Segment SGI175321[] = {{&S66416[27], 6}, {NULL}};
const Segment SGI175322[] = {{&S55130[93], 9}, {NULL}};
const Segment SGI175323[] = {{&S55130[66], 6}, {NULL}};
const Segment SGI175324[] = {{&S66416[97], 9}, {NULL}};
const Segment SGI175325[] = {{&S66416[70], 6}, {NULL}};
const Segment SGI175326[] = {{&S66421[34], 7}, {NULL}};
const Segment SGI175327[] = {{&S66421[27], 6}, {NULL}};
const Segment SGI175328[] = {{&S66421[99], 9}, {NULL}};
const Segment SGI175329[] = {{&S66421[72], 6}, {NULL}};
const Segment SGI175330[] = {{&S66452[44], 18}, {NULL}};
const Segment SGI175331[] = {{&S66452[44], 30}, {NULL}};
const Segment SGI175332[] = {{&S66452[30], 6}, {NULL}};
const Segment SGI175333[] = {{&S66452[31], 3}, {NULL}};
const Segment SGI175334[] = {{&S66452[107], 6}, {NULL}};
const Segment SGI175335[] = {{&S66452[76], 8}, {NULL}};
const Segment SGI175336[] = {{&S66453[44], 18}, {NULL}};
const Segment SGI175337[] = {{&S66453[44], 30}, {NULL}};
const Segment SGI175338[] = {{&S66453[30], 6}, {NULL}};
const Segment SGI175339[] = {{&S66453[31], 3}, {NULL}};
const Segment SGI175340[] = {{&S66453[103], 6}, {NULL}};
const Segment SGI175341[] = {{&S66453[76], 8}, {NULL}};
const Segment SGI175342[] = {{&S66455[44], 8}, {NULL}};
const Segment SGI175343[] = {{&S66455[44], 20}, {NULL}};
const Segment SGI175344[] = {{&S66455[30], 6}, {NULL}};
const Segment SGI175345[] = {{&S66455[31], 3}, {NULL}};
const Segment SGI175346[] = {{&S66455[78], 6}, {NULL}};
const Segment SGI175347[] = {{&S66456[44], 12}, {NULL}};
const Segment SGI175348[] = {{&S66456[44], 37}, {NULL}};
const Segment SGI175349[] = {{&S66456[30], 6}, {NULL}};
const Segment SGI175350[] = {{&S66456[31], 3}, {NULL}};
const Segment SGI175351[] = {{&S66456[95], 6}, {NULL}};
const Segment SGI175352[] = {{&S55179[44], 9}, {NULL}};
const Segment SGI175353[] = {{&S55179[44], 21}, {NULL}};
const Segment SGI175354[] = {{&S55179[30], 6}, {NULL}};
const Segment SGI175355[] = {{&S55179[134], 3}, {NULL}};
const Segment SGI175356[] = {{&S55179[99], 6}, {NULL}};
const Segment SGI175357[] = {{&S55179[67], 8}, {NULL}};
const Segment SGI175358[] = {{&S66464[44], 10}, {NULL}};
const Segment SGI175359[] = {{&S66464[44], 22}, {NULL}};
const Segment SGI175360[] = {{&S66464[30], 6}, {NULL}};
const Segment SGI175361[] = {{&S66464[31], 3}, {NULL}};
const Segment SGI175362[] = {{&S66464[80], 6}, {NULL}};
const Segment SGI175363[] = {{&S58553[44], 14}, {NULL}};
const Segment SGI175364[] = {{&S58553[44], 26}, {NULL}};
const Segment SGI175365[] = {{&S58553[30], 6}, {NULL}};
const Segment SGI175366[] = {{&S58553[119], 3}, {NULL}};
const Segment SGI175367[] = {{&S58553[84], 6}, {NULL}};
const Segment SGI175368[] = {{&S55186[44], 13}, {NULL}};
const Segment SGI175369[] = {{&S55186[44], 25}, {NULL}};
const Segment SGI175370[] = {{&S55186[30], 6}, {NULL}};
const Segment SGI175371[] = {{&S55186[118], 3}, {NULL}};
const Segment SGI175372[] = {{&S55186[83], 6}, {NULL}};
const Segment SGI175373[] = {{&S58577[44], 10}, {NULL}};
const Segment SGI175374[] = {{&S58577[44], 22}, {NULL}};
const Segment SGI175375[] = {{&S58577[30], 6}, {NULL}};
const Segment SGI175376[] = {{&S66491[44], 9}, {NULL}};
const Segment SGI175377[] = {{&S66491[44], 21}, {NULL}};
const Segment SGI175378[] = {{&S66491[30], 6}, {NULL}};
const Segment SGI175379[] = {{&S58577[80], 6}, {NULL}};
const Segment SGI175380[] = {{&S66491[31], 3}, {NULL}};
const Segment SGI175381[] = {{&S58577[107], 13}, {NULL}};
const Segment SGI175382[] = {{&S66491[79], 6}, {NULL}};
const Segment SGI175383[] = {{&S58579[44], 10}, {NULL}};
const Segment SGI175384[] = {{&S58579[44], 22}, {NULL}};
const Segment SGI175385[] = {{&S58579[30], 6}, {NULL}};
const Segment SGI175386[] = {{&S58579[80], 6}, {NULL}};
const Segment SGI175387[] = {{&S58579[107], 13}, {NULL}};
const Segment SGI175388[] = {{&S58582[44], 10}, {NULL}};
const Segment SGI175389[] = {{&S58582[44], 22}, {NULL}};
const Segment SGI175390[] = {{&S58582[30], 6}, {NULL}};
const Segment SGI175391[] = {{&S58582[80], 6}, {NULL}};
const Segment SGI175392[] = {{&S58582[107], 13}, {NULL}};
const Segment SGI175393[] = {{&S66502[34], 9}, {NULL}};
const Segment SGI175394[] = {{&S66502[27], 6}, {NULL}};
const Segment SGI175395[] = {{&S66502[96], 9}, {NULL}};
const Segment SGI175396[] = {{&S66502[69], 6}, {NULL}};
const Segment SGI175397[] = {{&S66505[34], 10}, {NULL}};
const Segment SGI175398[] = {{&S66505[27], 6}, {NULL}};
const Segment SGI175399[] = {{&S66505[97], 9}, {NULL}};
const Segment SGI175400[] = {{&S66505[70], 6}, {NULL}};
const Segment SGI175401[] = {{&S66507[44], 9}, {NULL}};
const Segment SGI175402[] = {{&S66507[44], 21}, {NULL}};
const Segment SGI175403[] = {{&S66507[30], 6}, {NULL}};
const Segment SGI175404[] = {{&S66507[31], 3}, {NULL}};
const Segment SGI175405[] = {{&S66507[79], 6}, {NULL}};
const Segment SGI175406[] = {{&S53116[34], 11}, {NULL}};
const Segment SGI175407[] = {{&S53116[27], 6}, {NULL}};
const Segment SGI175408[] = {{&S53116[98], 9}, {NULL}};
const Segment SGI175409[] = {{&S53116[71], 6}, {NULL}};
const Segment SGI175410[] = {{&S53117[34], 10}, {NULL}};
const Segment SGI175411[] = {{&S53117[27], 6}, {NULL}};
const Segment SGI175412[] = {{&S53117[97], 9}, {NULL}};
const Segment SGI175413[] = {{&S53117[70], 6}, {NULL}};
const Segment SGI175414[] = {{&S55239[34], 11}, {NULL}};
const Segment SGI175415[] = {{&S55239[27], 6}, {NULL}};
const Segment SGI175416[] = {{&S55239[98], 9}, {NULL}};
const Segment SGI175417[] = {{&S55239[71], 6}, {NULL}};
const Segment SGI175418[] = {{&S55240[34], 11}, {NULL}};
const Segment SGI175419[] = {{&S55240[27], 6}, {NULL}};
const Segment SGI175420[] = {{&S55240[98], 9}, {NULL}};
const Segment SGI175421[] = {{&S55240[71], 6}, {NULL}};
const Segment SGI175422[] = {{&S66534[44], 10}, {NULL}};
const Segment SGI175423[] = {{&S66534[44], 22}, {NULL}};
const Segment SGI175424[] = {{&S66534[30], 6}, {NULL}};
const Segment SGI175425[] = {{&S66534[80], 6}, {NULL}};
const Segment SGI175426[] = {{&S66534[107], 13}, {NULL}};
const Segment SGI175427[] = {{&S66535[44], 11}, {NULL}};
const Segment SGI175428[] = {{&S66535[44], 23}, {NULL}};
const Segment SGI175429[] = {{&S66535[30], 6}, {NULL}};
const Segment SGI175430[] = {{&S66535[81], 6}, {NULL}};
const Segment SGI175431[] = {{&S66535[108], 13}, {NULL}};
const Segment SGI175432[] = {{&S58623[34], 12}, {NULL}};
const Segment SGI175433[] = {{&S58623[27], 6}, {NULL}};
const Segment SGI175434[] = {{&S58623[99], 9}, {NULL}};
const Segment SGI175435[] = {{&S58623[72], 6}, {NULL}};
const Segment SGI175436[] = {{&S66536[44], 11}, {NULL}};
const Segment SGI175437[] = {{&S66536[44], 23}, {NULL}};
const Segment SGI175438[] = {{&S66536[30], 6}, {NULL}};
const Segment SGI175439[] = {{&S58624[34], 13}, {NULL}};
const Segment SGI175440[] = {{&S66536[81], 6}, {NULL}};
const Segment SGI175441[] = {{&S58624[27], 6}, {NULL}};
const Segment SGI175442[] = {{&S66536[108], 13}, {NULL}};
const Segment SGI175443[] = {{&S58624[100], 9}, {NULL}};
const Segment SGI175444[] = {{&S58624[73], 6}, {NULL}};
const Segment SGI175445[] = {{&S66537[44], 10}, {NULL}};
const Segment SGI175446[] = {{&S66537[44], 22}, {NULL}};
const Segment SGI175447[] = {{&S66537[30], 6}, {NULL}};
const Segment SGI175448[] = {{&S66537[80], 6}, {NULL}};
const Segment SGI175449[] = {{&S66537[107], 13}, {NULL}};
const Segment SGI175450[] = {{&S43762[34], 10}, {NULL}};
const Segment SGI175451[] = {{&S43762[27], 6}, {NULL}};
const Segment SGI175452[] = {{&S43762[96], 9}, {NULL}};
const Segment SGI175453[] = {{&S43762[69], 6}, {NULL}};
const Segment SGI175454[] = {{&S53174[44], 18}, {NULL}};
const Segment SGI175455[] = {{&S53174[44], 30}, {NULL}};
const Segment SGI175456[] = {{&S53174[30], 6}, {NULL}};
const Segment SGI175457[] = {{&S53174[141], 3}, {NULL}};
const Segment SGI175458[] = {{&S53174[106], 6}, {NULL}};
const Segment SGI175459[] = {{&S53174[76], 8}, {NULL}};
const Segment SGI175460[] = {{&S55267[34], 7}, {NULL}};
const Segment SGI175461[] = {{&S55267[27], 6}, {NULL}};
const Segment SGI175462[] = {{&S55267[99], 9}, {NULL}};
const Segment SGI175463[] = {{&S55267[72], 6}, {NULL}};
const Segment SGI175464[] = {{&S55268[34], 7}, {NULL}};
const Segment SGI175465[] = {{&S55268[27], 6}, {NULL}};
const Segment SGI175466[] = {{&S55268[99], 9}, {NULL}};
const Segment SGI175467[] = {{&S55268[72], 6}, {NULL}};
const Segment SGI175468[] = {{&S66571[34], 11}, {NULL}};
const Segment SGI175469[] = {{&S66571[27], 6}, {NULL}};
const Segment SGI175470[] = {{&S66571[98], 9}, {NULL}};
const Segment SGI175471[] = {{&S66571[71], 6}, {NULL}};
const Segment SGI175472[] = {{&S66583[44], 11}, {NULL}};
const Segment SGI175473[] = {{&S66583[44], 23}, {NULL}};
const Segment SGI175474[] = {{&S66583[30], 6}, {NULL}};
const Segment SGI175475[] = {{&S66583[31], 3}, {NULL}};
const Segment SGI175476[] = {{&S66583[81], 6}, {NULL}};
const Segment SGI175477[] = {{&S66584[44], 11}, {NULL}};
const Segment SGI175478[] = {{&S66584[44], 23}, {NULL}};
const Segment SGI175479[] = {{&S66584[30], 6}, {NULL}};
const Segment SGI175480[] = {{&S66584[31], 3}, {NULL}};
const Segment SGI175481[] = {{&S66584[81], 6}, {NULL}};
const Segment SGI175482[] = {{&S66585[38], 11}, {NULL}};
const Segment SGI175483[] = {{&S66585[31], 6}, {NULL}};
const Segment SGI175484[] = {{&S66586[38], 11}, {NULL}};
const Segment SGI175485[] = {{&S66586[31], 6}, {NULL}};
const Segment SGI175486[] = {{&S66588[34], 13}, {NULL}};
const Segment SGI175487[] = {{&S66588[27], 6}, {NULL}};
const Segment SGI175488[] = {{&S66588[100], 9}, {NULL}};
const Segment SGI175489[] = {{&S66588[73], 6}, {NULL}};
const Segment SGI175490[] = {{&S66589[34], 10}, {NULL}};
const Segment SGI175491[] = {{&S66589[27], 6}, {NULL}};
const Segment SGI175492[] = {{&S66589[97], 9}, {NULL}};
const Segment SGI175493[] = {{&S66589[70], 6}, {NULL}};
const Segment SGI175494[] = {{&S55296[34], 10}, {NULL}};
const Segment SGI175495[] = {{&S55296[27], 6}, {NULL}};
const Segment SGI175496[] = {{&S55296[97], 9}, {NULL}};
const Segment SGI175497[] = {{&S55296[70], 6}, {NULL}};
const Segment SGI175498[] = {{&S55297[34], 10}, {NULL}};
const Segment SGI175499[] = {{&S55297[27], 6}, {NULL}};
const Segment SGI175500[] = {{&S55297[97], 9}, {NULL}};
const Segment SGI175501[] = {{&S55297[70], 6}, {NULL}};
const Segment SGI175502[] = {{&S43805[34], 10}, {NULL}};
const Segment SGI175503[] = {{&S55298[34], 18}, {NULL}};
const Segment SGI175504[] = {{&S43805[27], 6}, {NULL}};
const Segment SGI175505[] = {{&S55298[27], 6}, {NULL}};
const Segment SGI175506[] = {{&S43805[96], 9}, {NULL}};
const Segment SGI175507[] = {{&S43805[69], 6}, {NULL}};
const Segment SGI175508[] = {{&S55298[105], 9}, {NULL}};
const Segment SGI175509[] = {{&S55298[78], 6}, {NULL}};
const Segment SGI175510[] = {{&S58691[44], 6}, {NULL}};
const Segment SGI175511[] = {{&S58691[44], 18}, {NULL}};
const Segment SGI175512[] = {{&S58691[30], 6}, {NULL}};
const Segment SGI175513[] = {{&S58691[111], 3}, {NULL}};
const Segment SGI175514[] = {{&S58691[76], 6}, {NULL}};
const Segment SGI175515[] = {{&S55299[34], 11}, {NULL}};
const Segment SGI175516[] = {{&S55299[27], 6}, {NULL}};
const Segment SGI175517[] = {{&S43807[34], 7}, {NULL}};
const Segment SGI175518[] = {{&S55299[98], 9}, {NULL}};
const Segment SGI175519[] = {{&S43807[27], 6}, {NULL}};
const Segment SGI175520[] = {{&S55299[71], 6}, {NULL}};
const Segment SGI175521[] = {{&S43807[94], 9}, {NULL}};
const Segment SGI175522[] = {{&S43807[67], 6}, {NULL}};
const Segment SGI175523[] = {{&S58692[38], 6}, {NULL}};
const Segment SGI175524[] = {{&S58692[31], 6}, {NULL}};
const Segment SGI175525[] = {{&S55300[34], 18}, {NULL}};
const Segment SGI175526[] = {{&S55300[27], 6}, {NULL}};
const Segment SGI175527[] = {{&S55300[105], 9}, {NULL}};
const Segment SGI175528[] = {{&S55300[78], 6}, {NULL}};
const Segment SGI175529[] = {{&S43810[34], 17}, {NULL}};
const Segment SGI175530[] = {{&S55301[34], 11}, {NULL}};
const Segment SGI175531[] = {{&S43810[27], 6}, {NULL}};
const Segment SGI175532[] = {{&S55301[27], 6}, {NULL}};
const Segment SGI175533[] = {{&S43810[104], 9}, {NULL}};
const Segment SGI175534[] = {{&S55301[98], 9}, {NULL}};
const Segment SGI175535[] = {{&S43810[77], 6}, {NULL}};
const Segment SGI175536[] = {{&S55301[71], 6}, {NULL}};
const Segment SGI175537[] = {{&S53224[34], 29}, {NULL}};
const Segment SGI175538[] = {{&S53224[27], 6}, {NULL}};
const Segment SGI175539[] = {{&S53224[116], 9}, {NULL}};
const Segment SGI175540[] = {{&S53224[89], 6}, {NULL}};
const Segment SGI175541[] = {{&S55305[34], 10}, {NULL}};
const Segment SGI175542[] = {{&S55305[27], 6}, {NULL}};
const Segment SGI175543[] = {{&S55305[97], 9}, {NULL}};
const Segment SGI175544[] = {{&S55305[70], 6}, {NULL}};
const Segment SGI175545[] = {{&S55306[34], 10}, {NULL}};
const Segment SGI175546[] = {{&S55306[27], 6}, {NULL}};
const Segment SGI175547[] = {{&S55306[97], 9}, {NULL}};
const Segment SGI175548[] = {{&S55306[70], 6}, {NULL}};
const Segment SGI175549[] = {{&S55308[34], 10}, {NULL}};
const Segment SGI175550[] = {{&S55308[27], 6}, {NULL}};
const Segment SGI175551[] = {{&S66612[44], 15}, {NULL}};
const Segment SGI175552[] = {{&S66612[44], 34}, {NULL}};
const Segment SGI175553[] = {{&S55308[97], 9}, {NULL}};
const Segment SGI175554[] = {{&S66612[30], 6}, {NULL}};
const Segment SGI175555[] = {{&S55308[70], 6}, {NULL}};
const Segment SGI175556[] = {{&S66612[31], 3}, {NULL}};
const Segment SGI175557[] = {{&S66612[92], 6}, {NULL}};
const Segment SGI175558[] = {{&S66613[44], 15}, {NULL}};
const Segment SGI175559[] = {{&S66613[44], 34}, {NULL}};
const Segment SGI175560[] = {{&S66613[30], 6}, {NULL}};
const Segment SGI175561[] = {{&S66613[31], 3}, {NULL}};
const Segment SGI175562[] = {{&S66613[92], 6}, {NULL}};
const Segment SGI175563[] = {{&S66614[44], 15}, {NULL}};
const Segment SGI175564[] = {{&S66614[44], 34}, {NULL}};
const Segment SGI175565[] = {{&S66614[30], 6}, {NULL}};
const Segment SGI175566[] = {{&S66614[31], 3}, {NULL}};
const Segment SGI175567[] = {{&S66614[92], 6}, {NULL}};
const Segment SGI175568[] = {{&S66615[44], 15}, {NULL}};
const Segment SGI175569[] = {{&S66615[44], 34}, {NULL}};
const Segment SGI175570[] = {{&S55315[34], 9}, {NULL}};
const Segment SGI175571[] = {{&S66615[30], 6}, {NULL}};
const Segment SGI175572[] = {{&S55315[27], 6}, {NULL}};
const Segment SGI175573[] = {{&S66615[31], 3}, {NULL}};
const Segment SGI175574[] = {{&S66615[92], 6}, {NULL}};
const Segment SGI175575[] = {{&S55315[96], 9}, {NULL}};
const Segment SGI175576[] = {{&S55315[69], 6}, {NULL}};
const Segment SGI175577[] = {{&S55318[34], 11}, {NULL}};
const Segment SGI175578[] = {{&S55318[27], 6}, {NULL}};
const Segment SGI175579[] = {{&S55318[98], 9}, {NULL}};
const Segment SGI175580[] = {{&S66618[44], 7}, {NULL}};
const Segment SGI175581[] = {{&S55318[71], 6}, {NULL}};
const Segment SGI175582[] = {{&S66618[44], 19}, {NULL}};
const Segment SGI175583[] = {{&S66618[30], 6}, {NULL}};
const Segment SGI175584[] = {{&S66618[31], 3}, {NULL}};
const Segment SGI175585[] = {{&S66618[77], 6}, {NULL}};
const Segment SGI175586[] = {{&S55319[34], 10}, {NULL}};
const Segment SGI175587[] = {{&S55319[27], 6}, {NULL}};
const Segment SGI175588[] = {{&S55319[97], 9}, {NULL}};
const Segment SGI175589[] = {{&S55319[70], 6}, {NULL}};
const Segment SGI175590[] = {{&S66620[44], 8}, {NULL}};
const Segment SGI175591[] = {{&S66620[44], 20}, {NULL}};
const Segment SGI175592[] = {{&S66620[30], 6}, {NULL}};
const Segment SGI175593[] = {{&S55320[34], 11}, {NULL}};
const Segment SGI175594[] = {{&S55320[27], 6}, {NULL}};
const Segment SGI175595[] = {{&S66620[31], 3}, {NULL}};
const Segment SGI175596[] = {{&S66620[78], 6}, {NULL}};
const Segment SGI175597[] = {{&S55320[98], 9}, {NULL}};
const Segment SGI175598[] = {{&S55320[71], 6}, {NULL}};
const Segment SGI175599[] = {{&S55321[34], 11}, {NULL}};
const Segment SGI175600[] = {{&S55321[27], 6}, {NULL}};
const Segment SGI175601[] = {{&S55321[98], 9}, {NULL}};
const Segment SGI175602[] = {{&S55321[71], 6}, {NULL}};
const Segment SGI175603[] = {{&S55322[34], 11}, {NULL}};
const Segment SGI175604[] = {{&S55322[27], 6}, {NULL}};
const Segment SGI175605[] = {{&S55322[98], 9}, {NULL}};
const Segment SGI175606[] = {{&S55322[71], 6}, {NULL}};
const Segment SGI175607[] = {{&S66638[44], 9}, {NULL}};
const Segment SGI175608[] = {{&S66638[44], 21}, {NULL}};
const Segment SGI175609[] = {{&S66638[30], 6}, {NULL}};
const Segment SGI175610[] = {{&S66638[31], 3}, {NULL}};
const Segment SGI175611[] = {{&S66638[79], 6}, {NULL}};
const Segment SGI175612[] = {{&S66640[44], 9}, {NULL}};
const Segment SGI175613[] = {{&S66640[44], 21}, {NULL}};
const Segment SGI175614[] = {{&S66640[30], 6}, {NULL}};
const Segment SGI175615[] = {{&S66640[31], 3}, {NULL}};
const Segment SGI175616[] = {{&S66640[79], 6}, {NULL}};
const Segment SGI175617[] = {{&S66646[34], 20}, {NULL}};
const Segment SGI175618[] = {{&S66646[27], 6}, {NULL}};
const Segment SGI175619[] = {{&S66646[123], 9}, {NULL}};
const Segment SGI175620[] = {{&S66646[96], 6}, {NULL}};
const Segment SGI175621[] = {{&S66647[34], 10}, {NULL}};
const Segment SGI175622[] = {{&S66647[27], 6}, {NULL}};
const Segment SGI175623[] = {{&S66647[97], 9}, {NULL}};
const Segment SGI175624[] = {{&S66647[70], 6}, {NULL}};
const Segment SGI175625[] = {{&S66649[34], 15}, {NULL}};
const Segment SGI175626[] = {{&S66649[27], 6}, {NULL}};
const Segment SGI175627[] = {{&S66649[102], 9}, {NULL}};
const Segment SGI175628[] = {{&S66649[75], 6}, {NULL}};
const Segment SGI175629[] = {{&S66669[34], 7}, {NULL}};
const Segment SGI175630[] = {{&S66669[27], 6}, {NULL}};
const Segment SGI175631[] = {{&S66669[94], 9}, {NULL}};
const Segment SGI175632[] = {{&S66669[67], 6}, {NULL}};
const Segment SGI175633[] = {{&S43904[34], 10}, {NULL}};
const Segment SGI175634[] = {{&S43904[27], 6}, {NULL}};
const Segment SGI175635[] = {{&S43904[96], 9}, {NULL}};
const Segment SGI175636[] = {{&S43904[69], 6}, {NULL}};
const Segment SGI175637[] = {{&S43906[34], 10}, {NULL}};
const Segment SGI175638[] = {{&S43906[27], 6}, {NULL}};
const Segment SGI175639[] = {{&S43906[96], 9}, {NULL}};
const Segment SGI175640[] = {{&S43906[69], 6}, {NULL}};
const Segment SGI175641[] = {{&S43907[34], 10}, {NULL}};
const Segment SGI175642[] = {{&S43907[27], 6}, {NULL}};
const Segment SGI175643[] = {{&S43907[96], 9}, {NULL}};
const Segment SGI175644[] = {{&S43907[69], 6}, {NULL}};
const Segment SGI175645[] = {{&S43908[34], 10}, {NULL}};
const Segment SGI175646[] = {{&S43908[27], 6}, {NULL}};
const Segment SGI175647[] = {{&S43908[96], 9}, {NULL}};
const Segment SGI175648[] = {{&S43908[69], 6}, {NULL}};
const Segment SGI175649[] = {{&S55402[34], 10}, {NULL}};
const Segment SGI175650[] = {{&S55402[27], 6}, {NULL}};
const Segment SGI175651[] = {{&S55402[97], 9}, {NULL}};
const Segment SGI175652[] = {{&S55402[70], 6}, {NULL}};
const Segment SGI175653[] = {{&S43909[34], 10}, {NULL}};
const Segment SGI175654[] = {{&S43909[27], 6}, {NULL}};
const Segment SGI175655[] = {{&S55403[34], 11}, {NULL}};
const Segment SGI175656[] = {{&S43909[96], 9}, {NULL}};
const Segment SGI175657[] = {{&S43909[69], 6}, {NULL}};
const Segment SGI175658[] = {{&S55403[27], 6}, {NULL}};
const Segment SGI175659[] = {{&S55403[98], 9}, {NULL}};
const Segment SGI175660[] = {{&S55403[71], 6}, {NULL}};
const Segment SGI175661[] = {{&S43911[34], 10}, {NULL}};
const Segment SGI175662[] = {{&S43911[27], 6}, {NULL}};
const Segment SGI175663[] = {{&S43911[96], 9}, {NULL}};
const Segment SGI175664[] = {{&S43911[69], 6}, {NULL}};
const Segment SGI175665[] = {{&S66707[34], 18}, {NULL}};
const Segment SGI175666[] = {{&S66707[27], 6}, {NULL}};
const Segment SGI175667[] = {{&S66707[105], 9}, {NULL}};
const Segment SGI175668[] = {{&S66707[78], 6}, {NULL}};
const Segment SGI175669[] = {{&S53343[44], 10}, {NULL}};
const Segment SGI175670[] = {{&S53343[44], 22}, {NULL}};
const Segment SGI175671[] = {{&S53343[30], 6}, {NULL}};
const Segment SGI175672[] = {{&S53343[115], 3}, {NULL}};
const Segment SGI175673[] = {{&S53343[80], 6}, {NULL}};
const Segment SGI175674[] = {{&S66719[44], 5}, {NULL}};
const Segment SGI175675[] = {{&S66719[44], 17}, {NULL}};
const Segment SGI175676[] = {{&S66719[30], 6}, {NULL}};
const Segment SGI175677[] = {{&S66719[31], 3}, {NULL}};
const Segment SGI175678[] = {{&S66719[75], 6}, {NULL}};
const Segment SGI175679[] = {{&S66720[34], 15}, {NULL}};
const Segment SGI175680[] = {{&S66720[27], 6}, {NULL}};
const Segment SGI175681[] = {{&S66720[102], 9}, {NULL}};
const Segment SGI175682[] = {{&S66720[75], 6}, {NULL}};
const Segment SGI175683[] = {{&S66721[34], 14}, {NULL}};
const Segment SGI175684[] = {{&S66721[27], 6}, {NULL}};
const Segment SGI175685[] = {{&S66721[101], 9}, {NULL}};
const Segment SGI175686[] = {{&S66721[74], 6}, {NULL}};
const Segment SGI175687[] = {{&S66726[34], 10}, {NULL}};
const Segment SGI175688[] = {{&S66726[27], 6}, {NULL}};
const Segment SGI175689[] = {{&S66726[97], 9}, {NULL}};
const Segment SGI175690[] = {{&S66726[70], 6}, {NULL}};
const Segment SGI175691[] = {{&S66726[138], 8}, {NULL}};
const Segment SGI175692[] = {{&S66736[44], 9}, {NULL}};
const Segment SGI175693[] = {{&S66736[44], 21}, {NULL}};
const Segment SGI175694[] = {{&S66736[30], 6}, {NULL}};
const Segment SGI175695[] = {{&S66736[31], 3}, {NULL}};
const Segment SGI175696[] = {{&S66736[79], 6}, {NULL}};
const Segment SGI175697[] = {{&S66737[44], 9}, {NULL}};
const Segment SGI175698[] = {{&S66737[44], 21}, {NULL}};
const Segment SGI175699[] = {{&S66737[30], 6}, {NULL}};
const Segment SGI175700[] = {{&S66737[31], 3}, {NULL}};
const Segment SGI175701[] = {{&S55445[44], 18}, {NULL}};
const Segment SGI175702[] = {{&S55445[44], 30}, {NULL}};
const Segment SGI175703[] = {{&S66737[79], 6}, {NULL}};
const Segment SGI175704[] = {{&S55445[30], 6}, {NULL}};
const Segment SGI175705[] = {{&S55445[143], 3}, {NULL}};
const Segment SGI175706[] = {{&S55445[108], 6}, {NULL}};
const Segment SGI175707[] = {{&S55445[76], 8}, {NULL}};
const Segment SGI175708[] = {{&S66740[11], 1}, {NULL}};
const Segment SGI175709[] = {{&S66740[44], 10}, {NULL}};
const Segment SGI175710[] = {{&S66740[30], 6}, {NULL}};
const Segment SGI175711[] = {{&S66740[31], 3}, {NULL}};
const Segment SGI175712[] = {{&S66740[82], 6}, {NULL}};
const Segment SGI175713[] = {{&S66741[44], 7}, {NULL}};
const Segment SGI175714[] = {{&S66741[44], 19}, {NULL}};
const Segment SGI175715[] = {{&S66741[30], 6}, {NULL}};
const Segment SGI175716[] = {{&S66741[31], 3}, {NULL}};
const Segment SGI175717[] = {{&S66741[77], 6}, {NULL}};
const Segment SGI175718[] = {{&S66742[38], 7}, {NULL}};
const Segment SGI175719[] = {{&S66742[31], 6}, {NULL}};
const Segment SGI175720[] = {{&S43967[44], 10}, {NULL}};
const Segment SGI175721[] = {{&S43967[44], 21}, {NULL}};
const Segment SGI175722[] = {{&S43967[30], 6}, {NULL}};
const Segment SGI175723[] = {{&S66746[44], 9}, {NULL}};
const Segment SGI175724[] = {{&S66746[44], 21}, {NULL}};
const Segment SGI175725[] = {{&S43967[114], 3}, {NULL}};
const Segment SGI175726[] = {{&S66746[30], 6}, {NULL}};
const Segment SGI175727[] = {{&S43967[79], 6}, {NULL}};
const Segment SGI175728[] = {{&S66746[31], 3}, {NULL}};
const Segment SGI175729[] = {{&S66746[98], 6}, {NULL}};
const Segment SGI175730[] = {{&S66746[67], 8}, {NULL}};
const Segment SGI175731[] = {{&S43968[38], 10}, {NULL}};
const Segment SGI175732[] = {{&S43968[31], 6}, {NULL}};
const Segment SGI175733[] = {{&S66747[44], 9}, {NULL}};
const Segment SGI175734[] = {{&S66747[44], 21}, {NULL}};
const Segment SGI175735[] = {{&S66747[30], 6}, {NULL}};
const Segment SGI175736[] = {{&S66747[31], 3}, {NULL}};
const Segment SGI175737[] = {{&S66747[94], 6}, {NULL}};
const Segment SGI175738[] = {{&S66747[67], 8}, {NULL}};
const Segment SGI175739[] = {{&S43969[42], 10}, {NULL}};
const Segment SGI175740[] = {{&S43969[30], 6}, {NULL}};
const Segment SGI175741[] = {{&S43969[112], 3}, {NULL}};
const Segment SGI175742[] = {{&S43969[77], 6}, {NULL}};
const Segment SGI175743[] = {{&S66748[53], 9}, {NULL}};
const Segment SGI175744[] = {{&S66748[53], 21}, {NULL}};
const Segment SGI175745[] = {{&S66748[30], 15}, {NULL}};
const Segment SGI175746[] = {{&S66748[31], 3}, {NULL}};
const Segment SGI175747[] = {{&S66748[112], 6}, {NULL}};
const Segment SGI175748[] = {{&S66748[76], 8}, {NULL}};
const Segment SGI175749[] = {{&S66752[34], 10}, {NULL}};
const Segment SGI175750[] = {{&S66752[27], 6}, {NULL}};
const Segment SGI175751[] = {{&S66752[97], 9}, {NULL}};
const Segment SGI175752[] = {{&S66752[70], 6}, {NULL}};
const Segment SGI175753[] = {{&S66752[132], 8}, {NULL}};
const Segment SGI175754[] = {{&S66753[34], 11}, {NULL}};
const Segment SGI175755[] = {{&S66753[27], 6}, {NULL}};
const Segment SGI175756[] = {{&S66753[98], 9}, {NULL}};
const Segment SGI175757[] = {{&S66753[71], 6}, {NULL}};
const Segment SGI175758[] = {{&S66753[133], 8}, {NULL}};
const Segment SGI175759[] = {{&S66754[34], 10}, {NULL}};
const Segment SGI175760[] = {{&S66754[27], 6}, {NULL}};
const Segment SGI175761[] = {{&S66754[97], 9}, {NULL}};
const Segment SGI175762[] = {{&S66754[70], 6}, {NULL}};
const Segment SGI175763[] = {{&S66754[131], 8}, {NULL}};
const Segment SGI175764[] = {{&S55484[44], 10}, {NULL}};
const Segment SGI175765[] = {{&S55484[44], 22}, {NULL}};
const Segment SGI175766[] = {{&S55484[30], 6}, {NULL}};
const Segment SGI175767[] = {{&S55484[135], 3}, {NULL}};
const Segment SGI175768[] = {{&S55484[100], 6}, {NULL}};
const Segment SGI175769[] = {{&S55484[68], 8}, {NULL}};
const Segment SGI175770[] = {{&S66777[34], 6}, {NULL}};
const Segment SGI175771[] = {{&S66777[27], 6}, {NULL}};
const Segment SGI175772[] = {{&S66777[93], 9}, {NULL}};
const Segment SGI175773[] = {{&S66777[66], 6}, {NULL}};
const Segment SGI175774[] = {{&S66778[34], 10}, {NULL}};
const Segment SGI175775[] = {{&S66778[27], 6}, {NULL}};
const Segment SGI175776[] = {{&S66778[97], 9}, {NULL}};
const Segment SGI175777[] = {{&S66778[70], 6}, {NULL}};
const Segment SGI175778[] = {{&S66780[44], 5}, {NULL}};
const Segment SGI175779[] = {{&S66780[44], 17}, {NULL}};
const Segment SGI175780[] = {{&S66780[30], 6}, {NULL}};
const Segment SGI175781[] = {{&S66780[31], 3}, {NULL}};
const Segment SGI175782[] = {{&S66780[75], 6}, {NULL}};
const Segment SGI175783[] = {{&S66781[44], 5}, {NULL}};
const Segment SGI175784[] = {{&S66781[44], 17}, {NULL}};
const Segment SGI175785[] = {{&S66781[30], 6}, {NULL}};
const Segment SGI175786[] = {{&S66781[31], 3}, {NULL}};
const Segment SGI175787[] = {{&S66781[75], 6}, {NULL}};
const Segment SGI175788[] = {{&S66782[44], 5}, {NULL}};
const Segment SGI175789[] = {{&S66782[44], 17}, {NULL}};
const Segment SGI175790[] = {{&S66782[30], 6}, {NULL}};
const Segment SGI175791[] = {{&S66782[31], 3}, {NULL}};
const Segment SGI175792[] = {{&S66782[75], 6}, {NULL}};
const Segment SGI175793[] = {{&S66788[34], 20}, {NULL}};
const Segment SGI175794[] = {{&S66788[27], 6}, {NULL}};
const Segment SGI175795[] = {{&S66788[123], 9}, {NULL}};
const Segment SGI175796[] = {{&S66788[96], 6}, {NULL}};
const Segment SGI175797[] = {{&S44029[34], 10}, {NULL}};
const Segment SGI175798[] = {{&S44029[27], 6}, {NULL}};
const Segment SGI175799[] = {{&S44029[96], 9}, {NULL}};
const Segment SGI175800[] = {{&S44029[69], 6}, {NULL}};
const Segment SGI175801[] = {{&S56757[42], 5}, {NULL}};
const Segment SGI175802[] = {{&S56757[42], 17}, {NULL}};
const Segment SGI175803[] = {{&S66803[34], 12}, {NULL}};
const Segment SGI175804[] = {{&S56757[30], 4}, {NULL}};
const Segment SGI175805[] = {{&S66803[27], 6}, {NULL}};
const Segment SGI175806[] = {{&S56757[108], 3}, {NULL}};
const Segment SGI175807[] = {{&S56757[73], 6}, {NULL}};
const Segment SGI175808[] = {{&S66803[99], 9}, {NULL}};
const Segment SGI175809[] = {{&S66803[72], 6}, {NULL}};
const Segment SGI175810[] = {{&S56758[42], 5}, {NULL}};
const Segment SGI175811[] = {{&S56758[42], 17}, {NULL}};
const Segment SGI175812[] = {{&S56758[30], 4}, {NULL}};
const Segment SGI175813[] = {{&S56758[108], 3}, {NULL}};
const Segment SGI175814[] = {{&S56758[73], 6}, {NULL}};
const Segment SGI175815[] = {{&S66805[34], 16}, {NULL}};
const Segment SGI175816[] = {{&S66805[27], 6}, {NULL}};
const Segment SGI175817[] = {{&S56759[42], 12}, {NULL}};
const Segment SGI175818[] = {{&S56759[42], 24}, {NULL}};
const Segment SGI175819[] = {{&S56759[30], 4}, {NULL}};
const Segment SGI175820[] = {{&S66805[103], 9}, {NULL}};
const Segment SGI175821[] = {{&S66805[76], 6}, {NULL}};
const Segment SGI175822[] = {{&S56759[115], 3}, {NULL}};
const Segment SGI175823[] = {{&S56759[80], 6}, {NULL}};
const Segment SGI175824[] = {{&S66807[44], 10}, {NULL}};
const Segment SGI175825[] = {{&S66807[44], 22}, {NULL}};
const Segment SGI175826[] = {{&S66807[30], 6}, {NULL}};
const Segment SGI175827[] = {{&S66807[31], 3}, {NULL}};
const Segment SGI175828[] = {{&S66807[80], 6}, {NULL}};
const Segment SGI175829[] = {{&S55520[34], 10}, {NULL}};
const Segment SGI175830[] = {{&S55520[27], 6}, {NULL}};
const Segment SGI175831[] = {{&S55520[97], 9}, {NULL}};
const Segment SGI175832[] = {{&S55520[70], 6}, {NULL}};
const Segment SGI175833[] = {{&S66818[34], 18}, {NULL}};
const Segment SGI175834[] = {{&S55521[34], 11}, {NULL}};
const Segment SGI175835[] = {{&S66818[27], 6}, {NULL}};
const Segment SGI175836[] = {{&S55521[27], 6}, {NULL}};
const Segment SGI175837[] = {{&S55521[98], 9}, {NULL}};
const Segment SGI175838[] = {{&S66818[113], 9}, {NULL}};
const Segment SGI175839[] = {{&S55521[71], 6}, {NULL}};
const Segment SGI175840[] = {{&S66818[86], 6}, {NULL}};
const Segment SGI175841[] = {{&S66823[34], 11}, {NULL}};
const Segment SGI175842[] = {{&S66823[27], 6}, {NULL}};
const Segment SGI175843[] = {{&S66823[98], 9}, {NULL}};
const Segment SGI175844[] = {{&S66823[71], 6}, {NULL}};
const Segment SGI175845[] = {{&S58934[34], 12}, {NULL}};
const Segment SGI175846[] = {{&S58934[27], 6}, {NULL}};
const Segment SGI175847[] = {{&S66831[44], 18}, {NULL}};
const Segment SGI175848[] = {{&S66831[44], 30}, {NULL}};
const Segment SGI175849[] = {{&S66831[30], 6}, {NULL}};
const Segment SGI175850[] = {{&S58934[99], 9}, {NULL}};
const Segment SGI175851[] = {{&S58934[72], 6}, {NULL}};
const Segment SGI175852[] = {{&S66831[31], 3}, {NULL}};
const Segment SGI175853[] = {{&S66831[88], 6}, {NULL}};
const Segment SGI175854[] = {{&S56801[39], 19}, {NULL}};
const Segment SGI175855[] = {{&S56801[39], 31}, {NULL}};
const Segment SGI175856[] = {{&S56801[27], 4}, {NULL}};
const Segment SGI175857[] = {{&S56801[110], 9}, {NULL}};
const Segment SGI175858[] = {{&S56801[84], 6}, {NULL}};
const Segment SGI175859[] = {{&S66845[34], 7}, {NULL}};
const Segment SGI175860[] = {{&S66845[27], 6}, {NULL}};
const Segment SGI175861[] = {{&S66845[100], 9}, {NULL}};
const Segment SGI175862[] = {{&S66845[73], 6}, {NULL}};
const Segment SGI175863[] = {{&S53532[44], 7}, {NULL}};
const Segment SGI175864[] = {{&S53532[44], 19}, {NULL}};
const Segment SGI175865[] = {{&S53532[30], 6}, {NULL}};
const Segment SGI175866[] = {{&S53532[130], 3}, {NULL}};
const Segment SGI175867[] = {{&S53532[95], 6}, {NULL}};
const Segment SGI175868[] = {{&S53532[65], 8}, {NULL}};
const Segment SGI175869[] = {{&S53534[44], 11}, {NULL}};
const Segment SGI175870[] = {{&S53534[44], 23}, {NULL}};
const Segment SGI175871[] = {{&S53534[30], 6}, {NULL}};
const Segment SGI175872[] = {{&S53534[134], 3}, {NULL}};
const Segment SGI175873[] = {{&S53534[99], 6}, {NULL}};
const Segment SGI175874[] = {{&S53534[69], 8}, {NULL}};
const Segment SGI175875[] = {{&S53558[34], 29}, {NULL}};
const Segment SGI175876[] = {{&S53558[27], 6}, {NULL}};
const Segment SGI175877[] = {{&S53558[116], 9}, {NULL}};
const Segment SGI175878[] = {{&S53558[89], 6}, {NULL}};
const Segment SGI175879[] = {{&S55601[44], 12}, {NULL}};
const Segment SGI175880[] = {{&S55601[44], 24}, {NULL}};
const Segment SGI175881[] = {{&S55601[30], 6}, {NULL}};
const Segment SGI175882[] = {{&S55601[117], 3}, {NULL}};
const Segment SGI175883[] = {{&S55601[82], 6}, {NULL}};
const Segment SGI175884[] = {{&S55602[38], 12}, {NULL}};
const Segment SGI175885[] = {{&S55602[31], 6}, {NULL}};
const Segment SGI175886[] = {{&S66904[34], 13}, {NULL}};
const Segment SGI175887[] = {{&S66904[27], 6}, {NULL}};
const Segment SGI175888[] = {{&S66904[100], 9}, {NULL}};
const Segment SGI175889[] = {{&S66904[73], 6}, {NULL}};
const Segment SGI175890[] = {{&S66905[34], 10}, {NULL}};
const Segment SGI175891[] = {{&S66905[27], 6}, {NULL}};
const Segment SGI175892[] = {{&S66905[97], 9}, {NULL}};
const Segment SGI175893[] = {{&S66905[70], 6}, {NULL}};
const Segment SGI175894[] = {{&S44030[34], 10}, {NULL}};
const Segment SGI175895[] = {{&S44030[27], 6}, {NULL}};
const Segment SGI175896[] = {{&S44030[96], 9}, {NULL}};
const Segment SGI175897[] = {{&S44030[69], 6}, {NULL}};
const Segment SGI175898[] = {{&S66936[34], 7}, {NULL}};
const Segment SGI175899[] = {{&S66936[27], 6}, {NULL}};
const Segment SGI175900[] = {{&S66936[94], 9}, {NULL}};
const Segment SGI175901[] = {{&S66936[67], 6}, {NULL}};
const Segment SGI175902[] = {{&S66940[34], 10}, {NULL}};
const Segment SGI175903[] = {{&S66940[27], 6}, {NULL}};
const Segment SGI175904[] = {{&S66940[97], 9}, {NULL}};
const Segment SGI175905[] = {{&S66940[70], 6}, {NULL}};
const Segment SGI175906[] = {{&S66941[34], 7}, {NULL}};
const Segment SGI175907[] = {{&S66941[27], 6}, {NULL}};
const Segment SGI175908[] = {{&S66941[94], 9}, {NULL}};
const Segment SGI175909[] = {{&S66941[67], 6}, {NULL}};
const Segment SGI175910[] = {{&S66943[34], 24}, {NULL}};
const Segment SGI175911[] = {{&S66943[27], 6}, {NULL}};
const Segment SGI175912[] = {{&S66943[111], 9}, {NULL}};
const Segment SGI175913[] = {{&S66943[84], 6}, {NULL}};
const Segment SGI175914[] = {{&S66948[34], 10}, {NULL}};
const Segment SGI175915[] = {{&S66948[27], 6}, {NULL}};
const Segment SGI175916[] = {{&S66948[97], 9}, {NULL}};
const Segment SGI175917[] = {{&S66948[70], 6}, {NULL}};
const Segment SGI175918[] = {{&S66956[44], 9}, {NULL}};
const Segment SGI175919[] = {{&S66956[44], 21}, {NULL}};
const Segment SGI175920[] = {{&S66956[30], 6}, {NULL}};
const Segment SGI175921[] = {{&S66956[31], 3}, {NULL}};
const Segment SGI175922[] = {{&S66956[79], 6}, {NULL}};
const Segment SGI175923[] = {{&S66957[44], 9}, {NULL}};
const Segment SGI175924[] = {{&S66957[44], 21}, {NULL}};
const Segment SGI175925[] = {{&S66957[30], 6}, {NULL}};
const Segment SGI175926[] = {{&S66957[31], 3}, {NULL}};
const Segment SGI175927[] = {{&S66957[79], 6}, {NULL}};
const Segment SGI175928[] = {{&S66960[44], 11}, {NULL}};
const Segment SGI175929[] = {{&S66960[44], 23}, {NULL}};
const Segment SGI175930[] = {{&S66960[30], 6}, {NULL}};
const Segment SGI175931[] = {{&S66960[31], 3}, {NULL}};
const Segment SGI175932[] = {{&S66960[81], 6}, {NULL}};
const Segment SGI175933[] = {{&S55647[34], 15}, {NULL}};
const Segment SGI175934[] = {{&S55647[27], 6}, {NULL}};
const Segment SGI175935[] = {{&S55647[102], 9}, {NULL}};
const Segment SGI175936[] = {{&S66961[44], 11}, {NULL}};
const Segment SGI175937[] = {{&S55647[75], 6}, {NULL}};
const Segment SGI175938[] = {{&S66961[44], 23}, {NULL}};
const Segment SGI175939[] = {{&S66961[30], 6}, {NULL}};
const Segment SGI175940[] = {{&S66961[31], 3}, {NULL}};
const Segment SGI175941[] = {{&S66961[81], 6}, {NULL}};
const Segment SGI175942[] = {{&S55648[34], 16}, {NULL}};
const Segment SGI175943[] = {{&S55648[27], 6}, {NULL}};
const Segment SGI175944[] = {{&S55648[103], 9}, {NULL}};
const Segment SGI175945[] = {{&S55648[76], 6}, {NULL}};
const Segment SGI175946[] = {{&S59084[44], 10}, {NULL}};
const Segment SGI175947[] = {{&S59084[44], 22}, {NULL}};
const Segment SGI175948[] = {{&S59084[30], 6}, {NULL}};
const Segment SGI175949[] = {{&S59084[131], 3}, {NULL}};
const Segment SGI175950[] = {{&S59084[96], 6}, {NULL}};
const Segment SGI175951[] = {{&S59084[68], 8}, {NULL}};
const Segment SGI175952[] = {{&S55652[34], 20}, {NULL}};
const Segment SGI175953[] = {{&S55652[27], 6}, {NULL}};
const Segment SGI175954[] = {{&S59085[44], 10}, {NULL}};
const Segment SGI175955[] = {{&S59085[44], 22}, {NULL}};
const Segment SGI175956[] = {{&S59085[30], 6}, {NULL}};
const Segment SGI175957[] = {{&S66966[34], 10}, {NULL}};
const Segment SGI175958[] = {{&S55652[116], 9}, {NULL}};
const Segment SGI175959[] = {{&S55652[89], 6}, {NULL}};
const Segment SGI175960[] = {{&S66966[27], 6}, {NULL}};
const Segment SGI175961[] = {{&S59085[131], 3}, {NULL}};
const Segment SGI175962[] = {{&S59085[96], 6}, {NULL}};
const Segment SGI175963[] = {{&S66966[97], 9}, {NULL}};
const Segment SGI175964[] = {{&S59085[68], 8}, {NULL}};
const Segment SGI175965[] = {{&S66966[70], 6}, {NULL}};
const Segment SGI175966[] = {{&S63212[6], 4}, {NULL}};
const Segment SGI175967[] = {{&S63212[74], 5}, {NULL}};
const Segment SGI175968[] = {{&S59086[44], 10}, {NULL}};
const Segment SGI175969[] = {{&S59086[44], 22}, {NULL}};
const Segment SGI175970[] = {{&S59086[30], 6}, {NULL}};
const Segment SGI175971[] = {{&S59086[131], 3}, {NULL}};
const Segment SGI175972[] = {{&S59086[96], 6}, {NULL}};
const Segment SGI175973[] = {{&S59086[68], 8}, {NULL}};
const Segment SGI175974[] = {{&S55659[34], 19}, {NULL}};
const Segment SGI175975[] = {{&S55659[27], 6}, {NULL}};
const Segment SGI175976[] = {{&S55659[106], 9}, {NULL}};
const Segment SGI175977[] = {{&S55659[79], 6}, {NULL}};
const Segment SGI175978[] = {{&S59093[44], 10}, {NULL}};
const Segment SGI175979[] = {{&S59093[44], 22}, {NULL}};
const Segment SGI175980[] = {{&S59093[30], 6}, {NULL}};
const Segment SGI175981[] = {{&S59093[115], 3}, {NULL}};
const Segment SGI175982[] = {{&S59093[80], 6}, {NULL}};
const Segment SGI175983[] = {{&S59093[150], 3}, {NULL}};
const Segment SGI175984[] = {{&S59102[34], 6}, {NULL}};
const Segment SGI175985[] = {{&S59102[27], 6}, {NULL}};
const Segment SGI175986[] = {{&S59102[93], 9}, {NULL}};
const Segment SGI175987[] = {{&S59102[66], 6}, {NULL}};
const Segment SGI175988[] = {{&S55683[44], 7}, {NULL}};
const Segment SGI175989[] = {{&S55683[44], 19}, {NULL}};
const Segment SGI175990[] = {{&S67000[44], 18}, {NULL}};
const Segment SGI175991[] = {{&S55683[30], 6}, {NULL}};
const Segment SGI175992[] = {{&S67000[44], 30}, {NULL}};
const Segment SGI175993[] = {{&S67000[30], 6}, {NULL}};
const Segment SGI175994[] = {{&S55683[112], 3}, {NULL}};
const Segment SGI175995[] = {{&S55683[77], 6}, {NULL}};
const Segment SGI175996[] = {{&S67000[31], 3}, {NULL}};
const Segment SGI175997[] = {{&S67000[88], 6}, {NULL}};
const Segment SGI175998[] = {{&S55684[38], 7}, {NULL}};
const Segment SGI175999[] = {{&S55684[31], 6}, {NULL}};
const Segment SGI176000[] = {{&S67001[44], 11}, {NULL}};
const Segment SGI176001[] = {{&S67001[44], 23}, {NULL}};
const Segment SGI176002[] = {{&S67001[30], 6}, {NULL}};
const Segment SGI176003[] = {{&S67001[31], 3}, {NULL}};
const Segment SGI176004[] = {{&S67001[81], 6}, {NULL}};
const Segment SGI176005[] = {{&S67002[44], 7}, {NULL}};
const Segment SGI176006[] = {{&S67002[44], 19}, {NULL}};
const Segment SGI176007[] = {{&S67002[30], 6}, {NULL}};
const Segment SGI176008[] = {{&S67002[31], 3}, {NULL}};
const Segment SGI176009[] = {{&S67002[77], 6}, {NULL}};
const Segment SGI176010[] = {{&S67003[44], 10}, {NULL}};
const Segment SGI176011[] = {{&S67003[44], 22}, {NULL}};
const Segment SGI176012[] = {{&S67003[30], 6}, {NULL}};
const Segment SGI176013[] = {{&S67003[31], 3}, {NULL}};
const Segment SGI176014[] = {{&S67003[80], 6}, {NULL}};
const Segment SGI176015[] = {{&S67004[41], 7}, {NULL}};
const Segment SGI176016[] = {{&S67004[30], 6}, {NULL}};
const Segment SGI176017[] = {{&S67004[31], 3}, {NULL}};
const Segment SGI176018[] = {{&S67004[74], 6}, {NULL}};
const Segment SGI176019[] = {{&S67005[44], 7}, {NULL}};
const Segment SGI176020[] = {{&S67005[44], 19}, {NULL}};
const Segment SGI176021[] = {{&S67005[30], 6}, {NULL}};
const Segment SGI176022[] = {{&S67005[80], 6}, {NULL}};
const Segment SGI176023[] = {{&S67006[44], 11}, {NULL}};
const Segment SGI176024[] = {{&S67006[44], 23}, {NULL}};
const Segment SGI176025[] = {{&S67006[30], 6}, {NULL}};
const Segment SGI176026[] = {{&S67006[31], 3}, {NULL}};
const Segment SGI176027[] = {{&S67006[81], 6}, {NULL}};
const Segment SGI176028[] = {{&S55697[44], 10}, {NULL}};
const Segment SGI176029[] = {{&S55697[44], 22}, {NULL}};
const Segment SGI176030[] = {{&S55697[30], 6}, {NULL}};
const Segment SGI176031[] = {{&S55697[135], 3}, {NULL}};
const Segment SGI176032[] = {{&S55697[100], 6}, {NULL}};
const Segment SGI176033[] = {{&S55697[68], 8}, {NULL}};
const Segment SGI176034[] = {{&S55698[44], 10}, {NULL}};
const Segment SGI176035[] = {{&S55698[44], 22}, {NULL}};
const Segment SGI176036[] = {{&S55698[30], 6}, {NULL}};
const Segment SGI176037[] = {{&S55698[135], 3}, {NULL}};
const Segment SGI176038[] = {{&S55698[100], 6}, {NULL}};
const Segment SGI176039[] = {{&S55698[68], 8}, {NULL}};
const Segment SGI176040[] = {{&S67009[44], 7}, {NULL}};
const Segment SGI176041[] = {{&S67009[44], 14}, {NULL}};
const Segment SGI176042[] = {{&S67009[30], 6}, {NULL}};
const Segment SGI176043[] = {{&S67009[31], 3}, {NULL}};
const Segment SGI176044[] = {{&S67009[72], 6}, {NULL}};
const Segment SGI176045[] = {{&S67010[44], 7}, {NULL}};
const Segment SGI176046[] = {{&S67010[44], 19}, {NULL}};
const Segment SGI176047[] = {{&S67010[30], 6}, {NULL}};
const Segment SGI176048[] = {{&S67010[31], 3}, {NULL}};
const Segment SGI176049[] = {{&S67010[77], 6}, {NULL}};
const Segment SGI176050[] = {{&S67012[44], 6}, {NULL}};
const Segment SGI176051[] = {{&S67012[44], 18}, {NULL}};
const Segment SGI176052[] = {{&S67012[30], 6}, {NULL}};
const Segment SGI176053[] = {{&S67012[31], 3}, {NULL}};
const Segment SGI176054[] = {{&S67012[76], 6}, {NULL}};
const Segment SGI176055[] = {{&S67013[44], 7}, {NULL}};
const Segment SGI176056[] = {{&S59141[34], 10}, {NULL}};
const Segment SGI176057[] = {{&S67013[44], 19}, {NULL}};
const Segment SGI176058[] = {{&S59141[27], 6}, {NULL}};
const Segment SGI176059[] = {{&S67013[30], 6}, {NULL}};
const Segment SGI176060[] = {{&S67013[31], 3}, {NULL}};
const Segment SGI176061[] = {{&S59141[97], 9}, {NULL}};
const Segment SGI176062[] = {{&S67013[77], 6}, {NULL}};
const Segment SGI176063[] = {{&S59141[70], 6}, {NULL}};
const Segment SGI176064[] = {{&S67015[44], 11}, {NULL}};
const Segment SGI176065[] = {{&S67015[44], 23}, {NULL}};
const Segment SGI176066[] = {{&S67015[30], 6}, {NULL}};
const Segment SGI176067[] = {{&S67015[31], 3}, {NULL}};
const Segment SGI176068[] = {{&S67015[81], 6}, {NULL}};
const Segment SGI176069[] = {{&S67016[38], 11}, {NULL}};
const Segment SGI176070[] = {{&S67016[31], 6}, {NULL}};
const Segment SGI176071[] = {{&S67017[44], 11}, {NULL}};
const Segment SGI176072[] = {{&S67017[44], 23}, {NULL}};
const Segment SGI176073[] = {{&S67017[30], 6}, {NULL}};
const Segment SGI176074[] = {{&S67017[31], 3}, {NULL}};
const Segment SGI176075[] = {{&S67017[81], 6}, {NULL}};
const Segment SGI176076[] = {{&S67018[34], 10}, {NULL}};
const Segment SGI176077[] = {{&S67018[27], 6}, {NULL}};
const Segment SGI176078[] = {{&S67018[97], 9}, {NULL}};
const Segment SGI176079[] = {{&S67018[70], 6}, {NULL}};
const Segment SGI176080[] = {{&S67019[34], 14}, {NULL}};
const Segment SGI176081[] = {{&S67019[27], 6}, {NULL}};
const Segment SGI176082[] = {{&S67019[106], 9}, {NULL}};
const Segment SGI176083[] = {{&S67019[79], 6}, {NULL}};
const Segment SGI176084[] = {{&S67029[34], 11}, {NULL}};
const Segment SGI176085[] = {{&S67029[27], 6}, {NULL}};
const Segment SGI176086[] = {{&S67029[98], 9}, {NULL}};
const Segment SGI176087[] = {{&S67029[71], 6}, {NULL}};
const Segment SGI176088[] = {{&S53711[44], 10}, {NULL}};
const Segment SGI176089[] = {{&S53711[44], 22}, {NULL}};
const Segment SGI176090[] = {{&S53711[30], 6}, {NULL}};
const Segment SGI176091[] = {{&S53711[133], 3}, {NULL}};
const Segment SGI176092[] = {{&S53711[98], 6}, {NULL}};
const Segment SGI176093[] = {{&S53711[68], 8}, {NULL}};
const Segment SGI176094[] = {{&S67030[34], 12}, {NULL}};
const Segment SGI176095[] = {{&S67030[27], 6}, {NULL}};
const Segment SGI176096[] = {{&S67030[99], 9}, {NULL}};
const Segment SGI176097[] = {{&S67030[72], 6}, {NULL}};
const Segment SGI176098[] = {{&S53713[44], 10}, {NULL}};
const Segment SGI176099[] = {{&S53713[44], 22}, {NULL}};
const Segment SGI176100[] = {{&S53713[30], 6}, {NULL}};
const Segment SGI176101[] = {{&S53713[133], 3}, {NULL}};
const Segment SGI176102[] = {{&S53713[98], 6}, {NULL}};
const Segment SGI176103[] = {{&S53713[68], 8}, {NULL}};
const Segment SGI176104[] = {{&S53723[44], 10}, {NULL}};
const Segment SGI176105[] = {{&S53723[44], 22}, {NULL}};
const Segment SGI176106[] = {{&S53723[30], 6}, {NULL}};
const Segment SGI176107[] = {{&S53723[133], 3}, {NULL}};
const Segment SGI176108[] = {{&S53723[98], 6}, {NULL}};
const Segment SGI176109[] = {{&S53723[68], 8}, {NULL}};
const Segment SGI176110[] = {{&S55753[44], 10}, {NULL}};
const Segment SGI176111[] = {{&S55753[44], 22}, {NULL}};
const Segment SGI176112[] = {{&S55753[30], 6}, {NULL}};
const Segment SGI176113[] = {{&S55753[115], 3}, {NULL}};
const Segment SGI176114[] = {{&S53724[44], 10}, {NULL}};
const Segment SGI176115[] = {{&S55753[80], 6}, {NULL}};
const Segment SGI176116[] = {{&S53724[44], 22}, {NULL}};
const Segment SGI176117[] = {{&S53724[30], 6}, {NULL}};
const Segment SGI176118[] = {{&S53724[133], 3}, {NULL}};
const Segment SGI176119[] = {{&S53724[98], 6}, {NULL}};
const Segment SGI176120[] = {{&S53724[68], 8}, {NULL}};
const Segment SGI176121[] = {{&S55754[38], 10}, {NULL}};
const Segment SGI176122[] = {{&S55754[31], 6}, {NULL}};
const Segment SGI176123[] = {{&S56977[32], 14}, {NULL}};
const Segment SGI176124[] = {{&S56977[27], 4}, {NULL}};
const Segment SGI176125[] = {{&S56977[98], 9}, {NULL}};
const Segment SGI176126[] = {{&S56977[72], 5}, {NULL}};
const Segment SGI176127[] = {{&S56977[131], 7}, {NULL}};
const Segment SGI176128[] = {{&S55784[34], 20}, {NULL}};
const Segment SGI176129[] = {{&S55784[27], 6}, {NULL}};
const Segment SGI176130[] = {{&S55784[107], 9}, {NULL}};
const Segment SGI176131[] = {{&S55784[80], 6}, {NULL}};
const Segment SGI176132[] = {{&S55785[34], 11}, {NULL}};
const Segment SGI176133[] = {{&S55785[27], 6}, {NULL}};
const Segment SGI176134[] = {{&S55785[98], 9}, {NULL}};
const Segment SGI176135[] = {{&S55785[71], 6}, {NULL}};
const Segment SGI176136[] = {{&S59234[44], 20}, {NULL}};
const Segment SGI176137[] = {{&S59234[44], 32}, {NULL}};
const Segment SGI176138[] = {{&S59234[30], 6}, {NULL}};
const Segment SGI176139[] = {{&S59234[125], 3}, {NULL}};
const Segment SGI176140[] = {{&S59234[90], 6}, {NULL}};
const Segment SGI176141[] = {{&S59235[38], 20}, {NULL}};
const Segment SGI176142[] = {{&S59235[31], 6}, {NULL}};
const Segment SGI176143[] = {{&S55806[44], 10}, {NULL}};
const Segment SGI176144[] = {{&S55806[44], 22}, {NULL}};
const Segment SGI176145[] = {{&S55806[30], 6}, {NULL}};
const Segment SGI176146[] = {{&S55806[80], 6}, {NULL}};
const Segment SGI176147[] = {{&S55806[107], 13}, {NULL}};
const Segment SGI176148[] = {{&S55807[44], 10}, {NULL}};
const Segment SGI176149[] = {{&S55807[44], 22}, {NULL}};
const Segment SGI176150[] = {{&S55807[30], 6}, {NULL}};
const Segment SGI176151[] = {{&S55807[80], 6}, {NULL}};
const Segment SGI176152[] = {{&S55807[107], 13}, {NULL}};
const Segment SGI176153[] = {{&S55808[44], 10}, {NULL}};
const Segment SGI176154[] = {{&S55808[44], 22}, {NULL}};
const Segment SGI176155[] = {{&S55808[30], 6}, {NULL}};
const Segment SGI176156[] = {{&S55808[80], 6}, {NULL}};
const Segment SGI176157[] = {{&S55808[107], 13}, {NULL}};
const Segment SGI176158[] = {{&S44245[34], 10}, {NULL}};
const Segment SGI176159[] = {{&S44245[27], 6}, {NULL}};
const Segment SGI176160[] = {{&S44245[96], 9}, {NULL}};
const Segment SGI176161[] = {{&S44245[69], 6}, {NULL}};
const Segment SGI176162[] = {{&S67037[44], 12}, {NULL}};
const Segment SGI176163[] = {{&S67037[44], 24}, {NULL}};
const Segment SGI176164[] = {{&S67037[30], 6}, {NULL}};
const Segment SGI176165[] = {{&S67037[31], 3}, {NULL}};
const Segment SGI176166[] = {{&S67037[82], 6}, {NULL}};
const Segment SGI176167[] = {{&S67038[34], 7}, {NULL}};
const Segment SGI176168[] = {{&S67038[27], 6}, {NULL}};
const Segment SGI176169[] = {{&S67038[103], 9}, {NULL}};
const Segment SGI176170[] = {{&S67038[76], 6}, {NULL}};
const Segment SGI176171[] = {{&S67039[34], 7}, {NULL}};
const Segment SGI176172[] = {{&S67039[27], 6}, {NULL}};
const Segment SGI176173[] = {{&S67039[110], 9}, {NULL}};
const Segment SGI176174[] = {{&S67039[83], 6}, {NULL}};
const Segment SGI176175[] = {{&S67040[34], 9}, {NULL}};
const Segment SGI176176[] = {{&S67040[27], 6}, {NULL}};
const Segment SGI176177[] = {{&S67040[96], 9}, {NULL}};
const Segment SGI176178[] = {{&S67040[69], 6}, {NULL}};
const Segment SGI176179[] = {{&S67044[44], 10}, {NULL}};
const Segment SGI176180[] = {{&S67044[44], 22}, {NULL}};
const Segment SGI176181[] = {{&S67044[30], 6}, {NULL}};
const Segment SGI176182[] = {{&S67044[31], 3}, {NULL}};
const Segment SGI176183[] = {{&S67044[95], 6}, {NULL}};
const Segment SGI176184[] = {{&S67044[68], 8}, {NULL}};
const Segment SGI176185[] = {{&S67045[44], 10}, {NULL}};
const Segment SGI176186[] = {{&S67045[44], 22}, {NULL}};
const Segment SGI176187[] = {{&S67045[30], 6}, {NULL}};
const Segment SGI176188[] = {{&S67045[31], 3}, {NULL}};
const Segment SGI176189[] = {{&S67045[127], 6}, {NULL}};
const Segment SGI176190[] = {{&S67045[68], 8}, {&S67045[68], 8}, {NULL}};
const Segment SGI176191[] = {{&S67057[44], 19}, {NULL}};
const Segment SGI176192[] = {{&S67057[44], 31}, {NULL}};
const Segment SGI176193[] = {{&S67057[30], 6}, {NULL}};
const Segment SGI176194[] = {{&S67057[31], 3}, {NULL}};
const Segment SGI176195[] = {{&S67057[104], 6}, {NULL}};
const Segment SGI176196[] = {{&S67057[77], 8}, {NULL}};
const Segment SGI176197[] = {{&S67058[44], 20}, {NULL}};
const Segment SGI176198[] = {{&S67058[44], 32}, {NULL}};
const Segment SGI176199[] = {{&S67058[30], 6}, {NULL}};
const Segment SGI176200[] = {{&S67058[31], 3}, {NULL}};
const Segment SGI176201[] = {{&S67058[105], 6}, {NULL}};
const Segment SGI176202[] = {{&S67058[78], 8}, {NULL}};
const Segment SGI176203[] = {{&S67059[44], 19}, {NULL}};
const Segment SGI176204[] = {{&S67059[44], 31}, {NULL}};
const Segment SGI176205[] = {{&S67059[30], 6}, {NULL}};
const Segment SGI176206[] = {{&S67059[31], 3}, {NULL}};
const Segment SGI176207[] = {{&S67059[104], 6}, {NULL}};
const Segment SGI176208[] = {{&S67059[77], 8}, {NULL}};
const Segment SGI176209[] = {{&S53870[44], 10}, {NULL}};
const Segment SGI176210[] = {{&S53870[44], 22}, {NULL}};
const Segment SGI176211[] = {{&S53870[30], 6}, {NULL}};
const Segment SGI176212[] = {{&S53870[133], 3}, {NULL}};
const Segment SGI176213[] = {{&S53870[98], 6}, {NULL}};
const Segment SGI176214[] = {{&S53870[68], 8}, {NULL}};
const Segment SGI176215[] = {{&S53871[44], 10}, {NULL}};
const Segment SGI176216[] = {{&S53871[44], 22}, {NULL}};
const Segment SGI176217[] = {{&S53871[30], 6}, {NULL}};
const Segment SGI176218[] = {{&S53871[133], 3}, {NULL}};
const Segment SGI176219[] = {{&S53871[98], 6}, {NULL}};
const Segment SGI176220[] = {{&S53871[68], 8}, {NULL}};
const Segment SGI176221[] = {{&S59298[44], 4}, {NULL}};
const Segment SGI176222[] = {{&S59298[44], 16}, {NULL}};
const Segment SGI176223[] = {{&S59298[30], 6}, {NULL}};
const Segment SGI176224[] = {{&S59298[109], 3}, {NULL}};
const Segment SGI176225[] = {{&S59298[74], 6}, {NULL}};
const Segment SGI176226[] = {{&S59299[38], 4}, {NULL}};
const Segment SGI176227[] = {{&S59299[31], 6}, {NULL}};
const Segment SGI176228[] = {{&S67068[34], 10}, {NULL}};
const Segment SGI176229[] = {{&S67068[27], 6}, {NULL}};
const Segment SGI176230[] = {{&S67068[101], 9}, {NULL}};
const Segment SGI176231[] = {{&S67068[74], 6}, {NULL}};
const Segment SGI176232[] = {{&S67072[34], 10}, {NULL}};
const Segment SGI176233[] = {{&S67072[27], 6}, {NULL}};
const Segment SGI176234[] = {{&S55867[34], 7}, {NULL}};
const Segment SGI176235[] = {{&S67072[97], 9}, {NULL}};
const Segment SGI176236[] = {{&S67072[70], 6}, {NULL}};
const Segment SGI176237[] = {{&S55867[27], 6}, {NULL}};
const Segment SGI176238[] = {{&S55867[99], 9}, {NULL}};
const Segment SGI176239[] = {{&S55867[72], 6}, {NULL}};
const Segment SGI176240[] = {{&S55881[34], 10}, {NULL}};
const Segment SGI176241[] = {{&S55881[27], 6}, {NULL}};
const Segment SGI176242[] = {{&S55881[97], 9}, {NULL}};
const Segment SGI176243[] = {{&S55881[70], 6}, {NULL}};
const Segment SGI176244[] = {{&S55882[34], 9}, {NULL}};
const Segment SGI176245[] = {{&S55882[27], 6}, {NULL}};
const Segment SGI176246[] = {{&S55882[96], 9}, {NULL}};
const Segment SGI176247[] = {{&S55882[69], 6}, {NULL}};
const Segment SGI176248[] = {{&S67097[34], 10}, {NULL}};
const Segment SGI176249[] = {{&S67097[27], 6}, {NULL}};
const Segment SGI176250[] = {{&S67097[97], 9}, {NULL}};
const Segment SGI176251[] = {{&S67097[70], 6}, {NULL}};
const Segment SGI176252[] = {{&S67098[34], 15}, {NULL}};
const Segment SGI176253[] = {{&S67098[27], 6}, {NULL}};
const Segment SGI176254[] = {{&S67098[102], 9}, {NULL}};
const Segment SGI176255[] = {{&S67098[75], 6}, {NULL}};
const Segment SGI176256[] = {{&S67099[34], 11}, {NULL}};
const Segment SGI176257[] = {{&S67099[27], 6}, {NULL}};
const Segment SGI176258[] = {{&S67099[98], 9}, {NULL}};
const Segment SGI176259[] = {{&S67099[71], 6}, {NULL}};
const Segment SGI176260[] = {{&S67100[34], 15}, {NULL}};
const Segment SGI176261[] = {{&S67100[27], 6}, {NULL}};
const Segment SGI176262[] = {{&S67100[102], 9}, {NULL}};
const Segment SGI176263[] = {{&S67100[75], 6}, {NULL}};
const Segment SGI176264[] = {{&S67101[34], 14}, {NULL}};
const Segment SGI176265[] = {{&S67101[27], 6}, {NULL}};
const Segment SGI176266[] = {{&S67101[101], 9}, {NULL}};
const Segment SGI176267[] = {{&S67101[74], 6}, {NULL}};
const Segment SGI176268[] = {{&S67103[44], 10}, {NULL}};
const Segment SGI176269[] = {{&S67103[44], 22}, {NULL}};
const Segment SGI176270[] = {{&S67103[30], 6}, {NULL}};
const Segment SGI176271[] = {{&S67103[31], 3}, {NULL}};
const Segment SGI176272[] = {{&S67103[80], 6}, {NULL}};
const Segment SGI176273[] = {{&S67112[34], 12}, {NULL}};
const Segment SGI176274[] = {{&S67112[27], 6}, {NULL}};
const Segment SGI176275[] = {{&S67112[99], 9}, {NULL}};
const Segment SGI176276[] = {{&S67112[72], 6}, {NULL}};
const Segment SGI176277[] = {{&S67142[44], 10}, {NULL}};
const Segment SGI176278[] = {{&S67142[44], 22}, {NULL}};
const Segment SGI176279[] = {{&S67142[30], 6}, {NULL}};
const Segment SGI176280[] = {{&S67142[31], 3}, {NULL}};
const Segment SGI176281[] = {{&S67142[80], 6}, {NULL}};
const Segment SGI176282[] = {{&S67142[140], 11}, {NULL}};
const Segment SGI176283[] = {{&S55960[34], 7}, {NULL}};
const Segment SGI176284[] = {{&S55960[27], 6}, {NULL}};
const Segment SGI176285[] = {{&S59405[44], 13}, {NULL}};
const Segment SGI176286[] = {{&S55960[99], 9}, {NULL}};
const Segment SGI176287[] = {{&S59405[44], 20}, {NULL}};
const Segment SGI176288[] = {{&S59405[30], 6}, {NULL}};
const Segment SGI176289[] = {{&S55960[72], 6}, {NULL}};
const Segment SGI176290[] = {{&S59405[113], 3}, {NULL}};
const Segment SGI176291[] = {{&S59405[78], 6}, {NULL}};
const Segment SGI176292[] = {{&S55961[34], 7}, {NULL}};
const Segment SGI176293[] = {{&S59406[38], 23}, {NULL}};
const Segment SGI176294[] = {{&S59406[31], 6}, {NULL}};
const Segment SGI176295[] = {{&S55961[27], 6}, {NULL}};
const Segment SGI176296[] = {{&S55961[99], 9}, {NULL}};
const Segment SGI176297[] = {{&S55961[72], 6}, {NULL}};
const Segment SGI176298[] = {{&S59407[38], 13}, {NULL}};
const Segment SGI176299[] = {{&S59407[31], 6}, {NULL}};
const Segment SGI176300[] = {{&S59408[44], 23}, {NULL}};
const Segment SGI176301[] = {{&S59408[44], 35}, {NULL}};
const Segment SGI176302[] = {{&S59408[30], 6}, {NULL}};
const Segment SGI176303[] = {{&S59408[128], 3}, {NULL}};
const Segment SGI176304[] = {{&S59408[93], 6}, {NULL}};
const Segment SGI176305[] = {{&S55981[34], 6}, {NULL}};
const Segment SGI176306[] = {{&S55981[27], 6}, {NULL}};
const Segment SGI176307[] = {{&S55981[93], 9}, {NULL}};
const Segment SGI176308[] = {{&S55981[66], 6}, {NULL}};
const Segment SGI176309[] = {{&S67199[34], 7}, {NULL}};
const Segment SGI176310[] = {{&S67199[27], 6}, {NULL}};
const Segment SGI176311[] = {{&S67199[105], 9}, {NULL}};
const Segment SGI176312[] = {{&S67199[78], 6}, {NULL}};
const Segment SGI176313[] = {{&S56002[34], 7}, {NULL}};
const Segment SGI176314[] = {{&S56002[27], 6}, {NULL}};
const Segment SGI176315[] = {{&S56002[107], 9}, {NULL}};
const Segment SGI176316[] = {{&S56002[80], 6}, {NULL}};
const Segment SGI176317[] = {{&S67217[34], 7}, {NULL}};
const Segment SGI176318[] = {{&S67217[27], 6}, {NULL}};
const Segment SGI176319[] = {{&S67217[100], 9}, {NULL}};
const Segment SGI176320[] = {{&S67217[73], 6}, {NULL}};
const Segment SGI176321[] = {{&S56021[44], 10}, {NULL}};
const Segment SGI176322[] = {{&S56021[44], 22}, {NULL}};
const Segment SGI176323[] = {{&S56021[30], 6}, {NULL}};
const Segment SGI176324[] = {{&S56021[80], 6}, {NULL}};
const Segment SGI176325[] = {{&S56021[107], 13}, {NULL}};
const Segment SGI176326[] = {{&S67227[34], 10}, {NULL}};
const Segment SGI176327[] = {{&S67227[27], 6}, {NULL}};
const Segment SGI176328[] = {{&S67227[97], 9}, {NULL}};
const Segment SGI176329[] = {{&S67227[70], 6}, {NULL}};
const Segment SGI176330[] = {{&S56022[44], 10}, {NULL}};
const Segment SGI176331[] = {{&S56022[44], 22}, {NULL}};
const Segment SGI176332[] = {{&S56022[30], 6}, {NULL}};
const Segment SGI176333[] = {{&S56022[80], 6}, {NULL}};
const Segment SGI176334[] = {{&S56022[107], 13}, {NULL}};
const Segment SGI176335[] = {{&S56023[44], 10}, {NULL}};
const Segment SGI176336[] = {{&S56023[44], 22}, {NULL}};
const Segment SGI176337[] = {{&S56023[30], 6}, {NULL}};
const Segment SGI176338[] = {{&S56023[80], 6}, {NULL}};
const Segment SGI176339[] = {{&S56023[107], 13}, {NULL}};
const Segment SGI176340[] = {{&S56024[44], 10}, {NULL}};
const Segment SGI176341[] = {{&S56024[44], 22}, {NULL}};
const Segment SGI176342[] = {{&S56024[30], 6}, {NULL}};
const Segment SGI176343[] = {{&S56024[80], 6}, {NULL}};
const Segment SGI176344[] = {{&S56024[107], 13}, {NULL}};
const Segment SGI176345[] = {{&S56025[44], 11}, {NULL}};
const Segment SGI176346[] = {{&S56025[44], 23}, {NULL}};
const Segment SGI176347[] = {{&S56025[30], 6}, {NULL}};
const Segment SGI176348[] = {{&S56025[81], 6}, {NULL}};
const Segment SGI176349[] = {{&S56025[108], 13}, {NULL}};
const Segment SGI176350[] = {{&S67254[34], 7}, {NULL}};
const Segment SGI176351[] = {{&S67254[27], 6}, {NULL}};
const Segment SGI176352[] = {{&S67254[99], 9}, {NULL}};
const Segment SGI176353[] = {{&S67254[72], 6}, {NULL}};
const Segment SGI176354[] = {{&S67255[34], 7}, {NULL}};
const Segment SGI176355[] = {{&S67255[27], 6}, {NULL}};
const Segment SGI176356[] = {{&S67255[99], 9}, {NULL}};
const Segment SGI176357[] = {{&S67255[72], 6}, {NULL}};
const Segment SGI176358[] = {{&S59464[34], 12}, {NULL}};
const Segment SGI176359[] = {{&S59464[27], 6}, {NULL}};
const Segment SGI176360[] = {{&S59464[99], 9}, {NULL}};
const Segment SGI176361[] = {{&S59464[72], 6}, {NULL}};
const Segment SGI176362[] = {{&S59465[34], 13}, {NULL}};
const Segment SGI176363[] = {{&S59465[27], 6}, {NULL}};
const Segment SGI176364[] = {{&S59465[100], 9}, {NULL}};
const Segment SGI176365[] = {{&S59465[73], 6}, {NULL}};
const Segment SGI176366[] = {{&S67264[44], 10}, {NULL}};
const Segment SGI176367[] = {{&S67264[44], 27}, {NULL}};
const Segment SGI176368[] = {{&S67264[30], 6}, {NULL}};
const Segment SGI176369[] = {{&S67264[31], 3}, {NULL}};
const Segment SGI176370[] = {{&S67264[85], 6}, {NULL}};
const Segment SGI176371[] = {{&S56089[34], 24}, {NULL}};
const Segment SGI176372[] = {{&S56089[27], 6}, {NULL}};
const Segment SGI176373[] = {{&S56089[111], 9}, {NULL}};
const Segment SGI176374[] = {{&S56089[84], 6}, {NULL}};
const Segment SGI176375[] = {{&S56090[34], 10}, {NULL}};
const Segment SGI176376[] = {{&S56090[27], 6}, {NULL}};
const Segment SGI176377[] = {{&S56090[97], 9}, {NULL}};
const Segment SGI176378[] = {{&S56090[70], 6}, {NULL}};
const Segment SGI176379[] = {{&S67301[44], 12}, {NULL}};
const Segment SGI176380[] = {{&S67301[44], 24}, {NULL}};
const Segment SGI176381[] = {{&S67301[30], 6}, {NULL}};
const Segment SGI176382[] = {{&S67301[31], 3}, {NULL}};
const Segment SGI176383[] = {{&S67301[82], 6}, {NULL}};
const Segment SGI176384[] = {{&S67303[44], 7}, {NULL}};
const Segment SGI176385[] = {{&S67303[44], 19}, {NULL}};
const Segment SGI176386[] = {{&S67303[30], 6}, {NULL}};
const Segment SGI176387[] = {{&S67303[108], 2}, {NULL}};
const Segment SGI176388[] = {{&S67303[77], 5}, {NULL}};
const Segment SGI176389[] = {{&S56102[34], 24}, {NULL}};
const Segment SGI176390[] = {{&S56102[27], 6}, {NULL}};
const Segment SGI176391[] = {{&S56102[111], 9}, {NULL}};
const Segment SGI176392[] = {{&S56102[84], 6}, {NULL}};
const Segment SGI176393[] = {{&S56103[34], 7}, {NULL}};
const Segment SGI176394[] = {{&S56103[27], 6}, {NULL}};
const Segment SGI176395[] = {{&S56103[98], 9}, {NULL}};
const Segment SGI176396[] = {{&S56103[71], 6}, {NULL}};
const Segment SGI176397[] = {{&S67316[44], 8}, {NULL}};
const Segment SGI176398[] = {{&S67316[44], 20}, {NULL}};
const Segment SGI176399[] = {{&S67316[30], 6}, {NULL}};
const Segment SGI176400[] = {{&S67316[31], 3}, {NULL}};
const Segment SGI176401[] = {{&S67316[78], 6}, {NULL}};
const Segment SGI176402[] = {{&S67317[34], 10}, {NULL}};
const Segment SGI176403[] = {{&S67317[27], 6}, {NULL}};
const Segment SGI176404[] = {{&S67317[97], 9}, {NULL}};
const Segment SGI176405[] = {{&S67317[70], 6}, {NULL}};
const Segment SGI176406[] = {{&S67327[44], 21}, {NULL}};
const Segment SGI176407[] = {{&S67327[44], 33}, {NULL}};
const Segment SGI176408[] = {{&S67327[30], 6}, {NULL}};
const Segment SGI176409[] = {{&S67327[31], 3}, {NULL}};
const Segment SGI176410[] = {{&S67327[91], 6}, {NULL}};
const Segment SGI176411[] = {{&S56130[34], 10}, {NULL}};
const Segment SGI176412[] = {{&S56130[27], 6}, {NULL}};
const Segment SGI176413[] = {{&S56130[97], 9}, {NULL}};
const Segment SGI176414[] = {{&S56130[70], 6}, {NULL}};
const Segment SGI176415[] = {{&S67329[44], 12}, {NULL}};
const Segment SGI176416[] = {{&S67329[44], 24}, {NULL}};
const Segment SGI176417[] = {{&S67329[30], 6}, {NULL}};
const Segment SGI176418[] = {{&S67329[31], 3}, {NULL}};
const Segment SGI176419[] = {{&S67329[82], 6}, {NULL}};
const Segment SGI176420[] = {{&S67330[44], 9}, {NULL}};
const Segment SGI176421[] = {{&S67330[44], 21}, {NULL}};
const Segment SGI176422[] = {{&S56132[34], 10}, {NULL}};
const Segment SGI176423[] = {{&S67330[30], 6}, {NULL}};
const Segment SGI176424[] = {{&S56132[27], 6}, {NULL}};
const Segment SGI176425[] = {{&S67330[31], 3}, {NULL}};
const Segment SGI176426[] = {{&S67330[79], 6}, {NULL}};
const Segment SGI176427[] = {{&S56132[97], 9}, {NULL}};
const Segment SGI176428[] = {{&S56132[70], 6}, {NULL}};
const Segment SGI176429[] = {{&S56132[138], 8}, {NULL}};
const Segment SGI176430[] = {{&S56133[34], 10}, {NULL}};
const Segment SGI176431[] = {{&S56133[27], 6}, {NULL}};
const Segment SGI176432[] = {{&S56133[97], 9}, {NULL}};
const Segment SGI176433[] = {{&S56133[70], 6}, {NULL}};
const Segment SGI176434[] = {{&S67332[34], 11}, {NULL}};
const Segment SGI176435[] = {{&S67332[27], 6}, {NULL}};
const Segment SGI176436[] = {{&S56133[138], 8}, {NULL}};
const Segment SGI176437[] = {{&S67332[98], 9}, {NULL}};
const Segment SGI176438[] = {{&S67332[71], 6}, {NULL}};
const Segment SGI176439[] = {{&S56134[34], 10}, {NULL}};
const Segment SGI176440[] = {{&S56134[27], 6}, {NULL}};
const Segment SGI176441[] = {{&S56134[97], 9}, {NULL}};
const Segment SGI176442[] = {{&S67333[34], 11}, {NULL}};
const Segment SGI176443[] = {{&S56134[70], 6}, {NULL}};
const Segment SGI176444[] = {{&S67333[27], 6}, {NULL}};
const Segment SGI176445[] = {{&S56134[138], 8}, {NULL}};
const Segment SGI176446[] = {{&S67333[98], 9}, {NULL}};
const Segment SGI176447[] = {{&S67333[71], 6}, {NULL}};
const Segment SGI176448[] = {{&S67334[34], 11}, {NULL}};
const Segment SGI176449[] = {{&S56136[34], 11}, {NULL}};
const Segment SGI176450[] = {{&S67334[27], 6}, {NULL}};
const Segment SGI176451[] = {{&S56136[27], 6}, {NULL}};
const Segment SGI176452[] = {{&S67334[98], 9}, {NULL}};
const Segment SGI176453[] = {{&S67334[71], 6}, {NULL}};
const Segment SGI176454[] = {{&S56136[98], 9}, {NULL}};
const Segment SGI176455[] = {{&S56136[71], 6}, {NULL}};
const Segment SGI176456[] = {{&S56136[139], 8}, {NULL}};
const Segment SGI176457[] = {{&S67337[34], 11}, {NULL}};
const Segment SGI176458[] = {{&S67337[27], 6}, {NULL}};
const Segment SGI176459[] = {{&S67337[110], 9}, {NULL}};
const Segment SGI176460[] = {{&S67337[83], 6}, {NULL}};
const Segment SGI176461[] = {{&S67338[44], 9}, {NULL}};
const Segment SGI176462[] = {{&S67338[44], 27}, {NULL}};
const Segment SGI176463[] = {{&S67338[30], 6}, {NULL}};
const Segment SGI176464[] = {{&S67338[31], 3}, {NULL}};
const Segment SGI176465[] = {{&S67338[85], 6}, {NULL}};
const Segment SGI176466[] = {{&S67339[44], 9}, {NULL}};
const Segment SGI176467[] = {{&S67339[44], 27}, {NULL}};
const Segment SGI176468[] = {{&S67339[30], 6}, {NULL}};
const Segment SGI176469[] = {{&S67339[31], 3}, {NULL}};
const Segment SGI176470[] = {{&S67339[85], 6}, {NULL}};
const Segment SGI176471[] = {{&S67340[44], 9}, {NULL}};
const Segment SGI176472[] = {{&S67340[44], 27}, {NULL}};
const Segment SGI176473[] = {{&S67340[30], 6}, {NULL}};
const Segment SGI176474[] = {{&S67340[31], 3}, {NULL}};
const Segment SGI176475[] = {{&S67340[85], 6}, {NULL}};
const Segment SGI176476[] = {{&S56156[44], 12}, {NULL}};
const Segment SGI176477[] = {{&S56156[44], 24}, {NULL}};
const Segment SGI176478[] = {{&S56156[30], 6}, {NULL}};
const Segment SGI176479[] = {{&S56156[117], 3}, {NULL}};
const Segment SGI176480[] = {{&S56156[82], 6}, {NULL}};
const Segment SGI176481[] = {{&S56157[44], 9}, {NULL}};
const Segment SGI176482[] = {{&S56157[44], 21}, {NULL}};
const Segment SGI176483[] = {{&S56157[30], 6}, {NULL}};
const Segment SGI176484[] = {{&S56157[114], 3}, {NULL}};
const Segment SGI176485[] = {{&S56157[79], 6}, {NULL}};
const Segment SGI176486[] = {{&S67347[34], 10}, {NULL}};
const Segment SGI176487[] = {{&S67347[27], 6}, {NULL}};
const Segment SGI176488[] = {{&S67347[97], 9}, {NULL}};
const Segment SGI176489[] = {{&S67347[70], 6}, {NULL}};
const Segment SGI176490[] = {{&S56158[38], 9}, {NULL}};
const Segment SGI176491[] = {{&S56158[31], 6}, {NULL}};
const Segment SGI176492[] = {{&S67350[44], 15}, {NULL}};
const Segment SGI176493[] = {{&S67350[44], 27}, {NULL}};
const Segment SGI176494[] = {{&S67350[30], 6}, {NULL}};
const Segment SGI176495[] = {{&S67350[31], 3}, {NULL}};
const Segment SGI176496[] = {{&S67350[85], 6}, {NULL}};
const Segment SGI176497[] = {{&S67351[44], 8}, {NULL}};
const Segment SGI176498[] = {{&S67351[44], 19}, {NULL}};
const Segment SGI176499[] = {{&S67351[30], 6}, {NULL}};
const Segment SGI176500[] = {{&S67351[31], 3}, {NULL}};
const Segment SGI176501[] = {{&S67351[77], 6}, {NULL}};
const Segment SGI176502[] = {{&S67352[44], 8}, {NULL}};
const Segment SGI176503[] = {{&S67352[44], 19}, {NULL}};
const Segment SGI176504[] = {{&S67352[30], 6}, {NULL}};
const Segment SGI176505[] = {{&S67352[31], 3}, {NULL}};
const Segment SGI176506[] = {{&S67352[77], 6}, {NULL}};
const Segment SGI176507[] = {{&S67353[44], 8}, {NULL}};
const Segment SGI176508[] = {{&S67353[44], 19}, {NULL}};
const Segment SGI176509[] = {{&S67353[30], 6}, {NULL}};
const Segment SGI176510[] = {{&S67353[31], 3}, {NULL}};
const Segment SGI176511[] = {{&S67353[77], 6}, {NULL}};
const Segment SGI176512[] = {{&S67354[44], 8}, {NULL}};
const Segment SGI176513[] = {{&S67354[44], 19}, {NULL}};
const Segment SGI176514[] = {{&S67354[30], 6}, {NULL}};
const Segment SGI176515[] = {{&S67354[31], 3}, {NULL}};
const Segment SGI176516[] = {{&S67354[77], 6}, {NULL}};
const Segment SGI176517[] = {{&S67360[34], 8}, {NULL}};
const Segment SGI176518[] = {{&S56180[44], 14}, {NULL}};
const Segment SGI176519[] = {{&S67360[27], 6}, {NULL}};
const Segment SGI176520[] = {{&S56180[44], 26}, {NULL}};
const Segment SGI176521[] = {{&S56180[30], 6}, {NULL}};
const Segment SGI176522[] = {{&S67360[95], 9}, {NULL}};
const Segment SGI176523[] = {{&S56180[119], 3}, {NULL}};
const Segment SGI176524[] = {{&S67360[68], 6}, {NULL}};
const Segment SGI176525[] = {{&S56180[84], 6}, {NULL}};
const Segment SGI176526[] = {{&S56181[38], 14}, {NULL}};
const Segment SGI176527[] = {{&S56181[31], 6}, {NULL}};
const Segment SGI176528[] = {{&S56199[44], 11}, {NULL}};
const Segment SGI176529[] = {{&S56199[44], 23}, {NULL}};
const Segment SGI176530[] = {{&S56199[30], 6}, {NULL}};
const Segment SGI176531[] = {{&S56199[116], 3}, {NULL}};
const Segment SGI176532[] = {{&S56199[81], 6}, {NULL}};
const Segment SGI176533[] = {{&S56200[44], 18}, {NULL}};
const Segment SGI176534[] = {{&S56200[44], 30}, {NULL}};
const Segment SGI176535[] = {{&S56200[30], 6}, {NULL}};
const Segment SGI176536[] = {{&S56200[123], 3}, {NULL}};
const Segment SGI176537[] = {{&S56200[88], 6}, {NULL}};
const Segment SGI176538[] = {{&S56201[44], 18}, {NULL}};
const Segment SGI176539[] = {{&S56201[44], 30}, {NULL}};
const Segment SGI176540[] = {{&S56201[30], 6}, {NULL}};
const Segment SGI176541[] = {{&S56201[123], 3}, {NULL}};
const Segment SGI176542[] = {{&S56201[88], 6}, {NULL}};
const Segment SGI176543[] = {{&S56202[38], 11}, {NULL}};
const Segment SGI176544[] = {{&S56202[31], 6}, {NULL}};
const Segment SGI176545[] = {{&S56203[44], 18}, {NULL}};
const Segment SGI176546[] = {{&S56203[44], 30}, {NULL}};
const Segment SGI176547[] = {{&S56203[30], 6}, {NULL}};
const Segment SGI176548[] = {{&S56203[123], 3}, {NULL}};
const Segment SGI176549[] = {{&S56203[88], 6}, {NULL}};
const Segment SGI176550[] = {{&S56204[44], 18}, {NULL}};
const Segment SGI176551[] = {{&S56204[44], 30}, {NULL}};
const Segment SGI176552[] = {{&S56204[30], 6}, {NULL}};
const Segment SGI176553[] = {{&S56204[123], 3}, {NULL}};
const Segment SGI176554[] = {{&S56204[88], 6}, {NULL}};
const Segment SGI176555[] = {{&S67391[44], 5}, {NULL}};
const Segment SGI176556[] = {{&S67391[44], 17}, {NULL}};
const Segment SGI176557[] = {{&S67391[30], 6}, {NULL}};
const Segment SGI176558[] = {{&S67391[31], 3}, {NULL}};
const Segment SGI176559[] = {{&S67391[75], 6}, {NULL}};
const Segment SGI176560[] = {{&S44706[41], 18}, {NULL}};
const Segment SGI176561[] = {{&S44706[41], 29}, {NULL}};
const Segment SGI176562[] = {{&S44706[27], 6}, {NULL}};
const Segment SGI176563[] = {{&S67392[44], 11}, {NULL}};
const Segment SGI176564[] = {{&S67392[44], 23}, {NULL}};
const Segment SGI176565[] = {{&S67392[30], 6}, {NULL}};
const Segment SGI176566[] = {{&S44706[119], 3}, {NULL}};
const Segment SGI176567[] = {{&S67392[31], 3}, {NULL}};
const Segment SGI176568[] = {{&S44706[123], 9}, {NULL}};
const Segment SGI176569[] = {{&S44706[84], 6}, {NULL}};
const Segment SGI176570[] = {{&S67392[81], 6}, {NULL}};
const Segment SGI176571[] = {{&S67393[44], 11}, {NULL}};
const Segment SGI176572[] = {{&S67393[44], 23}, {NULL}};
const Segment SGI176573[] = {{&S67393[30], 6}, {NULL}};
const Segment SGI176574[] = {{&S67393[84], 6}, {NULL}};
const Segment SGI176575[] = {{&S56210[34], 10}, {NULL}};
const Segment SGI176576[] = {{&S56210[27], 6}, {NULL}};
const Segment SGI176577[] = {{&S56210[97], 9}, {NULL}};
const Segment SGI176578[] = {{&S56210[70], 6}, {NULL}};
const Segment SGI176579[] = {{&S67394[44], 11}, {NULL}};
const Segment SGI176580[] = {{&S67394[44], 23}, {NULL}};
const Segment SGI176581[] = {{&S67394[30], 6}, {NULL}};
const Segment SGI176582[] = {{&S67394[31], 3}, {NULL}};
const Segment SGI176583[] = {{&S67394[81], 6}, {NULL}};
const Segment SGI176584[] = {{&S56211[34], 10}, {NULL}};
const Segment SGI176585[] = {{&S56211[27], 6}, {NULL}};
const Segment SGI176586[] = {{&S56211[97], 9}, {NULL}};
const Segment SGI176587[] = {{&S56211[70], 6}, {NULL}};
const Segment SGI176588[] = {{&S67395[38], 11}, {NULL}};
const Segment SGI176589[] = {{&S67395[31], 6}, {NULL}};
const Segment SGI176590[] = {{&S56212[34], 11}, {NULL}};
const Segment SGI176591[] = {{&S56212[27], 6}, {NULL}};
const Segment SGI176592[] = {{&S56212[98], 9}, {NULL}};
const Segment SGI176593[] = {{&S56212[71], 6}, {NULL}};
const Segment SGI176594[] = {{&S56213[34], 11}, {NULL}};
const Segment SGI176595[] = {{&S56213[27], 6}, {NULL}};
const Segment SGI176596[] = {{&S56213[98], 9}, {NULL}};
const Segment SGI176597[] = {{&S56213[71], 6}, {NULL}};
const Segment SGI176598[] = {{&S56225[44], 10}, {NULL}};
const Segment SGI176599[] = {{&S56225[44], 22}, {NULL}};
const Segment SGI176600[] = {{&S56225[30], 6}, {NULL}};
const Segment SGI176601[] = {{&S56225[131], 3}, {NULL}};
const Segment SGI176602[] = {{&S56225[96], 6}, {NULL}};
const Segment SGI176603[] = {{&S56225[68], 8}, {NULL}};
const Segment SGI176604[] = {{&S56226[44], 10}, {NULL}};
const Segment SGI176605[] = {{&S56226[44], 22}, {NULL}};
const Segment SGI176606[] = {{&S56226[30], 6}, {NULL}};
const Segment SGI176607[] = {{&S56226[146], 3}, {NULL}};
const Segment SGI176608[] = {{&S56226[111], 6}, {NULL}};
const Segment SGI176609[] = {{&S56226[68], 8}, {NULL}};
const Segment SGI176610[] = {{&S44744[34], 10}, {NULL}};
const Segment SGI176611[] = {{&S44744[27], 6}, {NULL}};
const Segment SGI176612[] = {{&S44744[96], 9}, {NULL}};
const Segment SGI176613[] = {{&S44744[69], 6}, {NULL}};
const Segment SGI176614[] = {{&S44745[34], 10}, {NULL}};
const Segment SGI176615[] = {{&S44745[27], 6}, {NULL}};
const Segment SGI176616[] = {{&S44745[97], 9}, {NULL}};
const Segment SGI176617[] = {{&S44745[70], 6}, {NULL}};
const Segment SGI176618[] = {{&S44746[34], 11}, {NULL}};
const Segment SGI176619[] = {{&S44746[27], 6}, {NULL}};
const Segment SGI176620[] = {{&S44746[97], 9}, {NULL}};
const Segment SGI176621[] = {{&S44746[70], 6}, {NULL}};
const Segment SGI176622[] = {{&S67430[44], 9}, {NULL}};
const Segment SGI176623[] = {{&S67430[44], 21}, {NULL}};
const Segment SGI176624[] = {{&S67430[30], 6}, {NULL}};
const Segment SGI176625[] = {{&S54091[44], 10}, {NULL}};
const Segment SGI176626[] = {{&S54091[44], 22}, {NULL}};
const Segment SGI176627[] = {{&S54091[30], 6}, {NULL}};
const Segment SGI176628[] = {{&S67430[31], 3}, {NULL}};
const Segment SGI176629[] = {{&S67430[79], 6}, {NULL}};
const Segment SGI176630[] = {{&S54091[133], 3}, {NULL}};
const Segment SGI176631[] = {{&S54091[98], 6}, {NULL}};
const Segment SGI176632[] = {{&S54091[68], 8}, {NULL}};
const Segment SGI176633[] = {{&S67431[34], 7}, {NULL}};
const Segment SGI176634[] = {{&S67431[27], 6}, {NULL}};
const Segment SGI176635[] = {{&S54092[44], 10}, {NULL}};
const Segment SGI176636[] = {{&S54092[44], 22}, {NULL}};
const Segment SGI176637[] = {{&S67431[110], 9}, {NULL}};
const Segment SGI176638[] = {{&S54092[30], 6}, {NULL}};
const Segment SGI176639[] = {{&S67431[83], 6}, {NULL}};
const Segment SGI176640[] = {{&S54092[133], 3}, {NULL}};
const Segment SGI176641[] = {{&S54092[98], 6}, {NULL}};
const Segment SGI176642[] = {{&S54092[68], 8}, {NULL}};
const Segment SGI176643[] = {{&S44758[34], 18}, {NULL}};
const Segment SGI176644[] = {{&S44758[27], 6}, {NULL}};
const Segment SGI176645[] = {{&S44758[105], 9}, {NULL}};
const Segment SGI176646[] = {{&S44758[78], 6}, {NULL}};
const Segment SGI176647[] = {{&S67439[44], 10}, {NULL}};
const Segment SGI176648[] = {{&S67439[44], 22}, {NULL}};
const Segment SGI176649[] = {{&S67439[30], 6}, {NULL}};
const Segment SGI176650[] = {{&S67439[139], 4}, {NULL}};
const Segment SGI176651[] = {{&S67439[165], 3}, {NULL}};
const Segment SGI176652[] = {{&S44759[34], 10}, {NULL}};
const Segment SGI176653[] = {{&S67439[31], 3}, {&S67439[214], 5}, {NULL}};
const Segment SGI176654[] = {{&S44759[27], 6}, {NULL}};
const Segment SGI176655[] = {{&S67439[80], 6}, {NULL}};
const Segment SGI176656[] = {{&S44759[96], 9}, {NULL}};
const Segment SGI176657[] = {{&S44759[69], 6}, {NULL}};
const Segment SGI176658[] = {{&S67440[59], 6}, {NULL}};
const Segment SGI176659[] = {{&S44760[34], 10}, {NULL}};
const Segment SGI176660[] = {{&S67440[59], 18}, {NULL}};
const Segment SGI176661[] = {{&S44760[27], 6}, {NULL}};
const Segment SGI176662[] = {{&S67440[45], 6}, {NULL}};
const Segment SGI176663[] = {{&S67440[46], 3}, {NULL}};
const Segment SGI176664[] = {{&S44760[97], 9}, {NULL}};
const Segment SGI176665[] = {{&S44760[70], 6}, {NULL}};
const Segment SGI176666[] = {{&S67440[91], 6}, {NULL}};
const Segment SGI176667[] = {{&S67440[0], 14}, {NULL}};
const Segment SGI176668[] = {{&S44761[34], 10}, {NULL}};
const Segment SGI176669[] = {{&S44761[27], 6}, {NULL}};
const Segment SGI176670[] = {{&S67441[59], 5}, {NULL}};
const Segment SGI176671[] = {{&S67441[59], 25}, {NULL}};
const Segment SGI176672[] = {{&S67441[45], 6}, {NULL}};
const Segment SGI176673[] = {{&S44761[97], 9}, {NULL}};
const Segment SGI176674[] = {{&S44761[70], 6}, {NULL}};
const Segment SGI176675[] = {{&S67441[46], 3}, {NULL}};
const Segment SGI176676[] = {{&S67441[98], 6}, {NULL}};
const Segment SGI176677[] = {{&S67441[0], 14}, {NULL}};
const Segment SGI176678[] = {{&S67442[59], 6}, {NULL}};
const Segment SGI176679[] = {{&S67442[59], 18}, {NULL}};
const Segment SGI176680[] = {{&S54107[44], 9}, {NULL}};
const Segment SGI176681[] = {{&S67442[45], 6}, {NULL}};
const Segment SGI176682[] = {{&S54107[44], 21}, {NULL}};
const Segment SGI176683[] = {{&S44763[34], 11}, {NULL}};
const Segment SGI176684[] = {{&S54107[30], 6}, {NULL}};
const Segment SGI176685[] = {{&S44763[27], 6}, {NULL}};
const Segment SGI176686[] = {{&S67442[46], 3}, {NULL}};
const Segment SGI176687[] = {{&S67442[91], 6}, {NULL}};
const Segment SGI176688[] = {{&S54107[114], 3}, {NULL}};
const Segment SGI176689[] = {{&S44763[98], 9}, {NULL}};
const Segment SGI176690[] = {{&S44763[71], 6}, {NULL}};
const Segment SGI176691[] = {{&S67442[0], 14}, {NULL}};
const Segment SGI176692[] = {{&S54107[79], 6}, {NULL}};
const Segment SGI176693[] = {{&S54107[142], 3}, {NULL}};
const Segment SGI176694[] = {{&S67443[59], 12}, {NULL}};
const Segment SGI176695[] = {{&S67443[59], 24}, {NULL}};
const Segment SGI176696[] = {{&S67443[45], 6}, {NULL}};
const Segment SGI176697[] = {{&S67443[46], 3}, {NULL}};
const Segment SGI176698[] = {{&S67443[97], 6}, {NULL}};
const Segment SGI176699[] = {{&S67443[0], 14}, {NULL}};
const Segment SGI176700[] = {{&S44766[34], 11}, {NULL}};
const Segment SGI176701[] = {{&S44766[27], 6}, {NULL}};
const Segment SGI176702[] = {{&S67444[59], 11}, {NULL}};
const Segment SGI176703[] = {{&S67444[59], 23}, {NULL}};
const Segment SGI176704[] = {{&S67444[45], 6}, {NULL}};
const Segment SGI176705[] = {{&S44766[98], 9}, {NULL}};
const Segment SGI176706[] = {{&S44766[71], 6}, {NULL}};
const Segment SGI176707[] = {{&S67444[46], 3}, {NULL}};
const Segment SGI176708[] = {{&S67444[96], 6}, {NULL}};
const Segment SGI176709[] = {{&S67444[0], 14}, {NULL}};
const Segment SGI176710[] = {{&S67445[59], 5}, {NULL}};
const Segment SGI176711[] = {{&S67445[59], 17}, {NULL}};
const Segment SGI176712[] = {{&S67445[45], 6}, {NULL}};
const Segment SGI176713[] = {{&S67445[46], 3}, {NULL}};
const Segment SGI176714[] = {{&S67445[90], 6}, {NULL}};
const Segment SGI176715[] = {{&S67445[0], 14}, {NULL}};
const Segment SGI176716[] = {{&S67446[59], 5}, {NULL}};
const Segment SGI176717[] = {{&S67446[59], 17}, {NULL}};
const Segment SGI176718[] = {{&S67446[45], 6}, {NULL}};
const Segment SGI176719[] = {{&S67446[46], 3}, {NULL}};
const Segment SGI176720[] = {{&S67446[90], 6}, {NULL}};
const Segment SGI176721[] = {{&S67446[0], 14}, {NULL}};
const Segment SGI176722[] = {{&S67447[59], 16}, {NULL}};
const Segment SGI176723[] = {{&S67447[59], 28}, {NULL}};
const Segment SGI176724[] = {{&S67447[45], 6}, {NULL}};
const Segment SGI176725[] = {{&S67447[46], 3}, {NULL}};
const Segment SGI176726[] = {{&S67447[101], 6}, {NULL}};
const Segment SGI176727[] = {{&S67447[0], 14}, {NULL}};
const Segment SGI176728[] = {{&S67448[59], 12}, {NULL}};
const Segment SGI176729[] = {{&S67448[59], 24}, {NULL}};
const Segment SGI176730[] = {{&S67448[45], 6}, {NULL}};
const Segment SGI176731[] = {{&S67448[46], 3}, {NULL}};
const Segment SGI176732[] = {{&S67448[97], 6}, {NULL}};
const Segment SGI176733[] = {{&S67448[0], 14}, {NULL}};
const Segment SGI176734[] = {{&S67450[34], 14}, {NULL}};
const Segment SGI176735[] = {{&S67450[27], 6}, {NULL}};
const Segment SGI176736[] = {{&S67450[101], 9}, {NULL}};
const Segment SGI176737[] = {{&S67450[74], 6}, {NULL}};
const Segment SGI176738[] = {{&S67452[34], 14}, {NULL}};
const Segment SGI176739[] = {{&S67452[27], 6}, {NULL}};
const Segment SGI176740[] = {{&S67452[101], 9}, {NULL}};
const Segment SGI176741[] = {{&S67452[74], 6}, {NULL}};
const Segment SGI176742[] = {{&S67472[44], 11}, {NULL}};
const Segment SGI176743[] = {{&S67472[44], 23}, {NULL}};
const Segment SGI176744[] = {{&S67472[30], 6}, {NULL}};
const Segment SGI176745[] = {{&S67472[31], 3}, {NULL}};
const Segment SGI176746[] = {{&S67472[81], 6}, {NULL}};
const Segment SGI176747[] = {{&S67480[34], 9}, {NULL}};
const Segment SGI176748[] = {{&S67480[27], 6}, {NULL}};
const Segment SGI176749[] = {{&S67480[96], 9}, {NULL}};
const Segment SGI176750[] = {{&S67480[69], 6}, {NULL}};
const Segment SGI176751[] = {{&S67481[34], 10}, {NULL}};
const Segment SGI176752[] = {{&S67481[27], 6}, {NULL}};
const Segment SGI176753[] = {{&S67481[97], 9}, {NULL}};
const Segment SGI176754[] = {{&S67481[70], 6}, {NULL}};
const Segment SGI176755[] = {{&S67482[34], 10}, {NULL}};
const Segment SGI176756[] = {{&S67482[27], 6}, {NULL}};
const Segment SGI176757[] = {{&S59751[34], 25}, {NULL}};
const Segment SGI176758[] = {{&S67482[97], 9}, {NULL}};
const Segment SGI176759[] = {{&S59751[27], 6}, {NULL}};
const Segment SGI176760[] = {{&S67482[70], 6}, {NULL}};
const Segment SGI176761[] = {{&S59751[112], 9}, {NULL}};
const Segment SGI176762[] = {{&S59751[85], 6}, {NULL}};
const Segment SGI176763[] = {{&S59752[34], 25}, {NULL}};
const Segment SGI176764[] = {{&S59752[27], 6}, {NULL}};
const Segment SGI176765[] = {{&S59752[112], 9}, {NULL}};
const Segment SGI176766[] = {{&S59752[85], 6}, {NULL}};
const Segment SGI176767[] = {{&S59753[34], 25}, {NULL}};
const Segment SGI176768[] = {{&S59753[27], 6}, {NULL}};
const Segment SGI176769[] = {{&S59753[112], 9}, {NULL}};
const Segment SGI176770[] = {{&S59753[85], 6}, {NULL}};
const Segment SGI176771[] = {{&S59754[34], 25}, {NULL}};
const Segment SGI176772[] = {{&S59754[27], 6}, {NULL}};
const Segment SGI176773[] = {{&S59754[112], 9}, {NULL}};
const Segment SGI176774[] = {{&S59754[85], 6}, {NULL}};
const Segment SGI176775[] = {{&S59755[34], 25}, {NULL}};
const Segment SGI176776[] = {{&S59755[27], 6}, {NULL}};
const Segment SGI176777[] = {{&S59755[112], 9}, {NULL}};
const Segment SGI176778[] = {{&S59755[85], 6}, {NULL}};
const Segment SGI176779[] = {{&S67493[44], 8}, {NULL}};
const Segment SGI176780[] = {{&S67493[44], 20}, {NULL}};
const Segment SGI176781[] = {{&S67493[30], 6}, {NULL}};
const Segment SGI176782[] = {{&S67493[31], 3}, {NULL}};
const Segment SGI176783[] = {{&S67493[78], 6}, {NULL}};
const Segment SGI176784[] = {{&S67494[34], 10}, {NULL}};
const Segment SGI176785[] = {{&S67494[27], 6}, {NULL}};
const Segment SGI176786[] = {{&S67494[97], 9}, {NULL}};
const Segment SGI176787[] = {{&S67494[70], 6}, {NULL}};
const Segment SGI176788[] = {{&S67495[34], 11}, {NULL}};
const Segment SGI176789[] = {{&S67495[27], 6}, {NULL}};
const Segment SGI176790[] = {{&S67495[98], 9}, {NULL}};
const Segment SGI176791[] = {{&S67495[71], 6}, {NULL}};
const Segment SGI176792[] = {{&S57219[32], 9}, {NULL}};
const Segment SGI176793[] = {{&S57219[27], 4}, {NULL}};
const Segment SGI176794[] = {{&S57219[94], 9}, {NULL}};
const Segment SGI176795[] = {{&S57219[67], 6}, {NULL}};
const Segment SGI176796[] = {{&S67504[34], 7}, {NULL}};
const Segment SGI176797[] = {{&S67504[27], 6}, {NULL}};
const Segment SGI176798[] = {{&S67504[98], 9}, {NULL}};
const Segment SGI176799[] = {{&S67504[71], 6}, {NULL}};
const Segment SGI176800[] = {{&S67510[34], 10}, {NULL}};
const Segment SGI176801[] = {{&S67510[27], 6}, {NULL}};
const Segment SGI176802[] = {{&S67510[97], 9}, {NULL}};
const Segment SGI176803[] = {{&S67510[70], 6}, {NULL}};
const Segment SGI176804[] = {{&S67533[44], 6}, {NULL}};
const Segment SGI176805[] = {{&S67533[44], 18}, {NULL}};
const Segment SGI176806[] = {{&S67533[30], 6}, {NULL}};
const Segment SGI176807[] = {{&S67533[31], 3}, {NULL}};
const Segment SGI176808[] = {{&S67533[76], 6}, {NULL}};
const Segment SGI176809[] = {{&S67534[44], 6}, {NULL}};
const Segment SGI176810[] = {{&S67534[44], 18}, {NULL}};
const Segment SGI176811[] = {{&S67534[30], 6}, {NULL}};
const Segment SGI176812[] = {{&S67534[31], 3}, {NULL}};
const Segment SGI176813[] = {{&S67534[76], 6}, {NULL}};
const Segment SGI176814[] = {{&S67570[44], 9}, {NULL}};
const Segment SGI176815[] = {{&S67570[44], 21}, {NULL}};
const Segment SGI176816[] = {{&S67570[30], 6}, {NULL}};
const Segment SGI176817[] = {{&S67570[31], 3}, {NULL}};
const Segment SGI176818[] = {{&S67570[79], 6}, {NULL}};
const Segment SGI176819[] = {{&S67583[34], 8}, {NULL}};
const Segment SGI176820[] = {{&S67583[27], 6}, {NULL}};
const Segment SGI176821[] = {{&S67583[99], 9}, {NULL}};
const Segment SGI176822[] = {{&S67583[72], 6}, {NULL}};
const Segment SGI176823[] = {{&S67587[34], 7}, {NULL}};
const Segment SGI176824[] = {{&S67587[27], 6}, {NULL}};
const Segment SGI176825[] = {{&S67587[105], 9}, {NULL}};
const Segment SGI176826[] = {{&S67587[78], 6}, {NULL}};
const Segment SGI176827[] = {{&S67597[44], 7}, {NULL}};
const Segment SGI176828[] = {{&S67597[44], 19}, {NULL}};
const Segment SGI176829[] = {{&S67597[30], 6}, {NULL}};
const Segment SGI176830[] = {{&S67597[31], 3}, {NULL}};
const Segment SGI176831[] = {{&S67597[77], 6}, {NULL}};
const Segment SGI176832[] = {{&S67598[44], 7}, {NULL}};
const Segment SGI176833[] = {{&S67598[44], 19}, {NULL}};
const Segment SGI176834[] = {{&S67598[30], 6}, {NULL}};
const Segment SGI176835[] = {{&S67598[31], 3}, {NULL}};
const Segment SGI176836[] = {{&S67598[77], 6}, {NULL}};
const Segment SGI176837[] = {{&S67600[44], 8}, {NULL}};
const Segment SGI176838[] = {{&S67600[44], 20}, {NULL}};
const Segment SGI176839[] = {{&S67600[30], 6}, {NULL}};
const Segment SGI176840[] = {{&S67600[31], 3}, {NULL}};
const Segment SGI176841[] = {{&S67600[78], 6}, {NULL}};
const Segment SGI176842[] = {{&S67601[44], 7}, {NULL}};
const Segment SGI176843[] = {{&S67601[44], 19}, {NULL}};
const Segment SGI176844[] = {{&S67601[30], 6}, {NULL}};
const Segment SGI176845[] = {{&S67601[31], 3}, {NULL}};
const Segment SGI176846[] = {{&S67601[77], 6}, {NULL}};
const Segment SGI176847[] = {{&S67603[44], 9}, {NULL}};
const Segment SGI176848[] = {{&S67603[44], 21}, {NULL}};
const Segment SGI176849[] = {{&S67603[30], 6}, {NULL}};
const Segment SGI176850[] = {{&S67603[31], 3}, {NULL}};
const Segment SGI176851[] = {{&S67603[79], 6}, {NULL}};
const Segment SGI176852[] = {{&S56289[34], 7}, {NULL}};
const Segment SGI176853[] = {{&S56289[27], 6}, {NULL}};
const Segment SGI176854[] = {{&S67608[34], 10}, {NULL}};
const Segment SGI176855[] = {{&S67608[27], 6}, {NULL}};
const Segment SGI176856[] = {{&S56289[100], 9}, {NULL}};
const Segment SGI176857[] = {{&S56289[73], 6}, {NULL}};
const Segment SGI176858[] = {{&S67608[97], 9}, {NULL}};
const Segment SGI176859[] = {{&S67608[70], 6}, {NULL}};
const Segment SGI176860[] = {{&S56290[34], 7}, {NULL}};
const Segment SGI176861[] = {{&S56290[27], 6}, {NULL}};
const Segment SGI176862[] = {{&S56290[100], 9}, {NULL}};
const Segment SGI176863[] = {{&S56290[73], 6}, {NULL}};
const Segment SGI176864[] = {{&S56291[34], 8}, {NULL}};
const Segment SGI176865[] = {{&S56291[27], 6}, {NULL}};
const Segment SGI176866[] = {{&S59885[34], 15}, {NULL}};
const Segment SGI176867[] = {{&S56291[101], 9}, {NULL}};
const Segment SGI176868[] = {{&S59885[27], 6}, {NULL}};
const Segment SGI176869[] = {{&S56291[74], 6}, {NULL}};
const Segment SGI176870[] = {{&S67612[34], 7}, {NULL}};
const Segment SGI176871[] = {{&S59885[102], 9}, {NULL}};
const Segment SGI176872[] = {{&S59885[75], 6}, {NULL}};
const Segment SGI176873[] = {{&S67612[27], 6}, {NULL}};
const Segment SGI176874[] = {{&S67612[99], 9}, {NULL}};
const Segment SGI176875[] = {{&S67612[72], 6}, {NULL}};
const Segment SGI176876[] = {{&S67618[44], 10}, {NULL}};
const Segment SGI176877[] = {{&S67618[44], 22}, {NULL}};
const Segment SGI176878[] = {{&S67618[30], 6}, {NULL}};
const Segment SGI176879[] = {{&S67618[31], 3}, {NULL}};
const Segment SGI176880[] = {{&S67618[99], 6}, {NULL}};
const Segment SGI176881[] = {{&S67618[68], 8}, {NULL}};
const Segment SGI176882[] = {{&S67619[44], 10}, {NULL}};
const Segment SGI176883[] = {{&S67619[44], 22}, {NULL}};
const Segment SGI176884[] = {{&S67619[30], 6}, {NULL}};
const Segment SGI176885[] = {{&S67619[31], 3}, {NULL}};
const Segment SGI176886[] = {{&S67619[95], 6}, {NULL}};
const Segment SGI176887[] = {{&S67619[68], 8}, {NULL}};
const Segment SGI176888[] = {{&S67620[44], 10}, {NULL}};
const Segment SGI176889[] = {{&S67620[44], 22}, {NULL}};
const Segment SGI176890[] = {{&S67620[30], 6}, {NULL}};
const Segment SGI176891[] = {{&S67620[31], 3}, {NULL}};
const Segment SGI176892[] = {{&S67620[99], 6}, {NULL}};
const Segment SGI176893[] = {{&S67620[68], 8}, {NULL}};
const Segment SGI176894[] = {{&S59902[44], 13}, {NULL}};
const Segment SGI176895[] = {{&S59902[44], 25}, {NULL}};
const Segment SGI176896[] = {{&S59902[30], 6}, {NULL}};
const Segment SGI176897[] = {{&S59902[118], 3}, {NULL}};
const Segment SGI176898[] = {{&S59902[83], 6}, {NULL}};
const Segment SGI176899[] = {{&S59903[38], 13}, {NULL}};
const Segment SGI176900[] = {{&S59903[31], 6}, {NULL}};
const Segment SGI176901[] = {{&S59912[34], 29}, {NULL}};
const Segment SGI176902[] = {{&S59912[27], 6}, {NULL}};
const Segment SGI176903[] = {{&S59912[116], 9}, {NULL}};
const Segment SGI176904[] = {{&S59912[89], 6}, {NULL}};
const Segment SGI176905[] = {{&S59913[34], 30}, {NULL}};
const Segment SGI176906[] = {{&S59913[27], 6}, {NULL}};
const Segment SGI176907[] = {{&S59913[117], 9}, {NULL}};
const Segment SGI176908[] = {{&S59913[90], 6}, {NULL}};
const Segment SGI176909[] = {{&S67650[44], 12}, {NULL}};
const Segment SGI176910[] = {{&S67650[44], 24}, {NULL}};
const Segment SGI176911[] = {{&S67650[30], 6}, {NULL}};
const Segment SGI176912[] = {{&S67650[31], 3}, {NULL}};
const Segment SGI176913[] = {{&S67650[101], 6}, {NULL}};
const Segment SGI176914[] = {{&S67650[70], 8}, {NULL}};
const Segment SGI176915[] = {{&S67651[44], 12}, {NULL}};
const Segment SGI176916[] = {{&S67651[44], 24}, {NULL}};
const Segment SGI176917[] = {{&S67651[30], 6}, {NULL}};
const Segment SGI176918[] = {{&S67651[31], 3}, {NULL}};
const Segment SGI176919[] = {{&S67651[101], 6}, {NULL}};
const Segment SGI176920[] = {{&S67651[70], 8}, {NULL}};
const Segment SGI176921[] = {{&S59925[44], 21}, {NULL}};
const Segment SGI176922[] = {{&S59925[44], 33}, {NULL}};
const Segment SGI176923[] = {{&S59925[30], 6}, {NULL}};
const Segment SGI176924[] = {{&S59925[126], 3}, {NULL}};
const Segment SGI176925[] = {{&S67652[44], 17}, {NULL}};
const Segment SGI176926[] = {{&S59925[91], 6}, {NULL}};
const Segment SGI176927[] = {{&S67652[44], 29}, {NULL}};
const Segment SGI176928[] = {{&S67652[30], 6}, {NULL}};
const Segment SGI176929[] = {{&S67652[31], 3}, {NULL}};
const Segment SGI176930[] = {{&S67652[106], 6}, {NULL}};
const Segment SGI176931[] = {{&S67652[75], 8}, {NULL}};
const Segment SGI176932[] = {{&S59926[38], 21}, {NULL}};
const Segment SGI176933[] = {{&S59926[31], 6}, {NULL}};
const Segment SGI176934[] = {{&S67656[44], 9}, {NULL}};
const Segment SGI176935[] = {{&S67656[44], 21}, {NULL}};
const Segment SGI176936[] = {{&S67656[30], 6}, {NULL}};
const Segment SGI176937[] = {{&S67656[31], 3}, {NULL}};
const Segment SGI176938[] = {{&S67656[79], 6}, {NULL}};
const Segment SGI176939[] = {{&S59934[44], 5}, {NULL}};
const Segment SGI176940[] = {{&S59934[44], 17}, {NULL}};
const Segment SGI176941[] = {{&S59934[30], 6}, {NULL}};
const Segment SGI176942[] = {{&S59934[126], 3}, {NULL}};
const Segment SGI176943[] = {{&S59934[91], 6}, {NULL}};
const Segment SGI176944[] = {{&S59934[63], 8}, {NULL}};
const Segment SGI176945[] = {{&S59935[44], 9}, {NULL}};
const Segment SGI176946[] = {{&S59935[44], 21}, {NULL}};
const Segment SGI176947[] = {{&S59935[30], 6}, {NULL}};
const Segment SGI176948[] = {{&S59935[130], 3}, {NULL}};
const Segment SGI176949[] = {{&S59935[95], 6}, {NULL}};
const Segment SGI176950[] = {{&S59935[67], 8}, {NULL}};
const Segment SGI176951[] = {{&S59939[44], 9}, {NULL}};
const Segment SGI176952[] = {{&S59939[44], 21}, {NULL}};
const Segment SGI176953[] = {{&S59939[30], 6}, {NULL}};
const Segment SGI176954[] = {{&S59939[147], 3}, {NULL}};
const Segment SGI176955[] = {{&S59939[112], 6}, {NULL}};
const Segment SGI176956[] = {{&S59939[67], 8}, {NULL}};
const Segment SGI176957[] = {{&S67675[44], 5}, {NULL}};
const Segment SGI176958[] = {{&S67675[44], 44}, {NULL}};
const Segment SGI176959[] = {{&S67675[30], 6}, {NULL}};
const Segment SGI176960[] = {{&S67675[31], 3}, {NULL}};
const Segment SGI176961[] = {{&S67675[102], 6}, {NULL}};
const Segment SGI176962[] = {{&S67677[44], 5}, {NULL}};
const Segment SGI176963[] = {{&S67677[44], 50}, {NULL}};
const Segment SGI176964[] = {{&S67677[30], 6}, {NULL}};
const Segment SGI176965[] = {{&S67677[31], 3}, {NULL}};
const Segment SGI176966[] = {{&S67677[108], 6}, {NULL}};
const Segment SGI176967[] = {{&S67682[44], 12}, {NULL}};
const Segment SGI176968[] = {{&S67682[44], 24}, {NULL}};
const Segment SGI176969[] = {{&S67682[30], 6}, {NULL}};
const Segment SGI176970[] = {{&S67682[31], 3}, {NULL}};
const Segment SGI176971[] = {{&S67682[82], 6}, {NULL}};
const Segment SGI176972[] = {{&S67693[34], 8}, {NULL}};
const Segment SGI176973[] = {{&S67693[27], 6}, {NULL}};
const Segment SGI176974[] = {{&S67693[99], 9}, {NULL}};
const Segment SGI176975[] = {{&S67693[72], 6}, {NULL}};
const Segment SGI176976[] = {{&S67702[44], 8}, {NULL}};
const Segment SGI176977[] = {{&S67702[44], 20}, {NULL}};
const Segment SGI176978[] = {{&S67702[30], 6}, {NULL}};
const Segment SGI176979[] = {{&S67702[31], 3}, {NULL}};
const Segment SGI176980[] = {{&S67702[78], 6}, {NULL}};
const Segment SGI176981[] = {{&S67703[38], 8}, {NULL}};
const Segment SGI176982[] = {{&S67703[31], 6}, {NULL}};
const Segment SGI176983[] = {{&S67707[44], 8}, {NULL}};
const Segment SGI176984[] = {{&S67707[44], 20}, {NULL}};
const Segment SGI176985[] = {{&S67707[30], 6}, {NULL}};
const Segment SGI176986[] = {{&S67707[31], 3}, {NULL}};
const Segment SGI176987[] = {{&S67707[78], 6}, {NULL}};
const Segment SGI176988[] = {{&S59987[34], 9}, {NULL}};
const Segment SGI176989[] = {{&S59987[27], 6}, {NULL}};
const Segment SGI176990[] = {{&S59987[96], 9}, {NULL}};
const Segment SGI176991[] = {{&S59987[69], 6}, {NULL}};
const Segment SGI176992[] = {{&S67731[44], 19}, {NULL}};
const Segment SGI176993[] = {{&S67731[44], 41}, {NULL}};
const Segment SGI176994[] = {{&S67731[30], 6}, {NULL}};
const Segment SGI176995[] = {{&S67731[31], 3}, {NULL}};
const Segment SGI176996[] = {{&S67731[99], 6}, {NULL}};
const Segment SGI176997[] = {{&S67739[34], 14}, {NULL}};
const Segment SGI176998[] = {{&S67739[27], 6}, {NULL}};
const Segment SGI176999[] = {{&S67739[101], 9}, {NULL}};
const Segment SGI177000[] = {{&S67739[74], 6}, {NULL}};
const Segment SGI177001[] = {{&S67740[44], 15}, {NULL}};
const Segment SGI177002[] = {{&S67740[44], 27}, {NULL}};
const Segment SGI177003[] = {{&S67740[30], 6}, {NULL}};
const Segment SGI177004[] = {{&S67740[85], 6}, {NULL}};
const Segment SGI177005[] = {{&S67740[112], 13}, {NULL}};
const Segment SGI177006[] = {{&S60020[34], 15}, {NULL}};
const Segment SGI177007[] = {{&S60020[27], 6}, {NULL}};
const Segment SGI177008[] = {{&S60020[102], 9}, {NULL}};
const Segment SGI177009[] = {{&S60020[75], 6}, {NULL}};
const Segment SGI177010[] = {{&S44842[34], 8}, {NULL}};
const Segment SGI177011[] = {{&S44842[27], 6}, {NULL}};
const Segment SGI177012[] = {{&S44842[94], 9}, {NULL}};
const Segment SGI177013[] = {{&S44842[67], 6}, {NULL}};
const Segment SGI177014[] = {{&S67790[34], 11}, {NULL}};
const Segment SGI177015[] = {{&S67790[27], 6}, {NULL}};
const Segment SGI177016[] = {{&S67790[98], 9}, {NULL}};
const Segment SGI177017[] = {{&S67790[71], 6}, {NULL}};
const Segment SGI177018[] = {{&S67797[44], 10}, {NULL}};
const Segment SGI177019[] = {{&S67797[44], 22}, {NULL}};
const Segment SGI177020[] = {{&S67797[30], 6}, {NULL}};
const Segment SGI177021[] = {{&S67797[31], 3}, {NULL}};
const Segment SGI177022[] = {{&S67797[80], 6}, {NULL}};
const Segment SGI177023[] = {{&S67803[34], 12}, {NULL}};
const Segment SGI177024[] = {{&S67803[27], 6}, {NULL}};
const Segment SGI177025[] = {{&S67803[99], 9}, {NULL}};
const Segment SGI177026[] = {{&S67803[72], 6}, {NULL}};
const Segment SGI177027[] = {{&S60086[44], 10}, {NULL}};
const Segment SGI177028[] = {{&S60086[44], 22}, {NULL}};
const Segment SGI177029[] = {{&S60086[30], 6}, {NULL}};
const Segment SGI177030[] = {{&S60086[115], 3}, {NULL}};
const Segment SGI177031[] = {{&S60086[80], 6}, {NULL}};
const Segment SGI177032[] = {{&S60087[44], 10}, {NULL}};
const Segment SGI177033[] = {{&S60087[44], 22}, {NULL}};
const Segment SGI177034[] = {{&S60087[30], 6}, {NULL}};
const Segment SGI177035[] = {{&S60087[115], 3}, {NULL}};
const Segment SGI177036[] = {{&S60087[80], 6}, {NULL}};
const Segment SGI177037[] = {{&S60088[44], 10}, {NULL}};
const Segment SGI177038[] = {{&S60088[44], 22}, {NULL}};
const Segment SGI177039[] = {{&S60088[30], 6}, {NULL}};
const Segment SGI177040[] = {{&S60088[115], 3}, {NULL}};
const Segment SGI177041[] = {{&S60088[80], 6}, {NULL}};
const Segment SGI177042[] = {{&S60092[44], 10}, {NULL}};
const Segment SGI177043[] = {{&S60092[44], 71}, {NULL}};
const Segment SGI177044[] = {{&S60092[30], 6}, {NULL}};
const Segment SGI177045[] = {{&S60093[44], 10}, {NULL}};
const Segment SGI177046[] = {{&S60093[44], 79}, {NULL}};
const Segment SGI177047[] = {{&S60093[30], 6}, {NULL}};
const Segment SGI177048[] = {{&S60094[45], 10}, {NULL}};
const Segment SGI177049[] = {{&S60094[45], 71}, {NULL}};
const Segment SGI177050[] = {{&S60094[31], 6}, {NULL}};
const Segment SGI177051[] = {{&S60095[38], 10}, {NULL}};
const Segment SGI177052[] = {{&S60095[31], 6}, {NULL}};
const Segment SGI177053[] = {{&S60096[44], 10}, {NULL}};
const Segment SGI177054[] = {{&S60096[44], 75}, {NULL}};
const Segment SGI177055[] = {{&S60096[30], 6}, {NULL}};
const Segment SGI177056[] = {{&S67829[34], 10}, {NULL}};
const Segment SGI177057[] = {{&S67829[27], 6}, {NULL}};
const Segment SGI177058[] = {{&S60097[97], 28}, {NULL}};
const Segment SGI177059[] = {{&S67829[101], 9}, {NULL}};
const Segment SGI177060[] = {{&S67829[74], 6}, {NULL}};
const Segment SGI177061[] = {{&S60098[44], 10}, {NULL}};
const Segment SGI177062[] = {{&S60098[44], 22}, {&S60098[208], 7}, {NULL}};
const Segment SGI177063[] = {{&S60098[30], 6}, {NULL}};
const Segment SGI177064[] = {{&S60098[115], 3}, {NULL}};
const Segment SGI177065[] = {{&S60098[80], 6}, {NULL}};
const Segment SGI177066[] = {{&S60099[65], 10}, {NULL}};
const Segment SGI177067[] = {{&S60099[65], 22}, {NULL}};
const Segment SGI177068[] = {{&S60099[51], 6}, {NULL}};
const Segment SGI177069[] = {{&S60105[60], 10}, {NULL}};
const Segment SGI177070[] = {{&S60105[60], 74}, {NULL}};
const Segment SGI177071[] = {{&S60105[46], 6}, {NULL}};
const Segment SGI177072[] = {{&S67844[44], 8}, {NULL}};
const Segment SGI177073[] = {{&S67844[44], 24}, {NULL}};
const Segment SGI177074[] = {{&S67844[30], 6}, {NULL}};
const Segment SGI177075[] = {{&S67844[82], 6}, {NULL}};
const Segment SGI177076[] = {{&S67844[109], 13}, {NULL}};
const Segment SGI177077[] = {{&S67845[44], 7}, {NULL}};
const Segment SGI177078[] = {{&S67845[44], 24}, {NULL}};
const Segment SGI177079[] = {{&S67845[30], 6}, {NULL}};
const Segment SGI177080[] = {{&S67845[82], 6}, {NULL}};
const Segment SGI177081[] = {{&S67845[109], 13}, {NULL}};
const Segment SGI177082[] = {{&S67846[44], 7}, {NULL}};
const Segment SGI177083[] = {{&S67846[44], 24}, {NULL}};
const Segment SGI177084[] = {{&S67846[30], 6}, {NULL}};
const Segment SGI177085[] = {{&S67846[82], 6}, {NULL}};
const Segment SGI177086[] = {{&S67846[109], 13}, {NULL}};
const Segment SGI177087[] = {{&S67847[44], 8}, {NULL}};
const Segment SGI177088[] = {{&S67847[44], 24}, {NULL}};
const Segment SGI177089[] = {{&S67847[30], 6}, {NULL}};
const Segment SGI177090[] = {{&S67847[82], 6}, {NULL}};
const Segment SGI177091[] = {{&S67847[109], 13}, {NULL}};
const Segment SGI177092[] = {{&S67848[44], 7}, {NULL}};
const Segment SGI177093[] = {{&S67848[44], 24}, {NULL}};
const Segment SGI177094[] = {{&S67848[30], 6}, {NULL}};
const Segment SGI177095[] = {{&S67848[82], 6}, {NULL}};
const Segment SGI177096[] = {{&S67848[109], 13}, {NULL}};
const Segment SGI177097[] = {{&S67849[44], 8}, {NULL}};
const Segment SGI177098[] = {{&S67849[44], 24}, {NULL}};
const Segment SGI177099[] = {{&S67849[30], 6}, {NULL}};
const Segment SGI177100[] = {{&S67849[82], 6}, {NULL}};
const Segment SGI177101[] = {{&S67849[109], 13}, {NULL}};
const Segment SGI177102[] = {{&S44960[34], 10}, {NULL}};
const Segment SGI177103[] = {{&S44960[27], 6}, {NULL}};
const Segment SGI177104[] = {{&S44960[96], 9}, {NULL}};
const Segment SGI177105[] = {{&S44960[69], 6}, {NULL}};
const Segment SGI177106[] = {{&S44961[34], 10}, {NULL}};
const Segment SGI177107[] = {{&S44961[27], 6}, {NULL}};
const Segment SGI177108[] = {{&S44961[96], 9}, {NULL}};
const Segment SGI177109[] = {{&S44961[69], 6}, {NULL}};
const Segment SGI177110[] = {{&S44963[34], 10}, {NULL}};
const Segment SGI177111[] = {{&S44963[27], 6}, {NULL}};
const Segment SGI177112[] = {{&S44963[96], 9}, {NULL}};
const Segment SGI177113[] = {{&S44963[69], 6}, {NULL}};
const Segment SGI177114[] = {{&S44966[34], 10}, {NULL}};
const Segment SGI177115[] = {{&S44966[27], 6}, {NULL}};
const Segment SGI177116[] = {{&S44966[96], 9}, {NULL}};
const Segment SGI177117[] = {{&S44966[69], 6}, {NULL}};
const Segment SGI177118[] = {{&S67877[44], 15}, {NULL}};
const Segment SGI177119[] = {{&S44967[34], 10}, {NULL}};
const Segment SGI177120[] = {{&S67877[44], 29}, {NULL}};
const Segment SGI177121[] = {{&S44967[27], 6}, {NULL}};
const Segment SGI177122[] = {{&S67877[30], 6}, {NULL}};
const Segment SGI177123[] = {{&S67877[31], 3}, {NULL}};
const Segment SGI177124[] = {{&S44967[96], 9}, {NULL}};
const Segment SGI177125[] = {{&S44967[69], 6}, {NULL}};
const Segment SGI177126[] = {{&S67877[87], 6}, {NULL}};
const Segment SGI177127[] = {{&S44968[34], 10}, {NULL}};
const Segment SGI177128[] = {{&S44968[27], 6}, {NULL}};
const Segment SGI177129[] = {{&S67878[44], 15}, {NULL}};
const Segment SGI177130[] = {{&S67878[44], 29}, {NULL}};
const Segment SGI177131[] = {{&S67878[30], 6}, {NULL}};
const Segment SGI177132[] = {{&S44968[96], 9}, {NULL}};
const Segment SGI177133[] = {{&S44968[69], 6}, {NULL}};
const Segment SGI177134[] = {{&S67878[31], 3}, {NULL}};
const Segment SGI177135[] = {{&S67878[87], 6}, {NULL}};
const Segment SGI177136[] = {{&S60169[34], 11}, {NULL}};
const Segment SGI177137[] = {{&S60169[27], 6}, {NULL}};
const Segment SGI177138[] = {{&S60169[98], 9}, {NULL}};
const Segment SGI177139[] = {{&S60169[71], 6}, {NULL}};
const Segment SGI177140[] = {{&S60172[34], 11}, {NULL}};
const Segment SGI177141[] = {{&S60172[27], 6}, {NULL}};
const Segment SGI177142[] = {{&S60172[98], 9}, {NULL}};
const Segment SGI177143[] = {{&S60172[71], 6}, {NULL}};
const Segment SGI177144[] = {{&S67902[34], 7}, {NULL}};
const Segment SGI177145[] = {{&S67902[27], 6}, {NULL}};
const Segment SGI177146[] = {{&S67902[94], 9}, {NULL}};
const Segment SGI177147[] = {{&S67902[67], 6}, {NULL}};
const Segment SGI177148[] = {{&S60214[44], 11}, {NULL}};
const Segment SGI177149[] = {{&S60214[44], 23}, {NULL}};
const Segment SGI177150[] = {{&S60214[30], 6}, {NULL}};
const Segment SGI177151[] = {{&S60214[116], 3}, {NULL}};
const Segment SGI177152[] = {{&S60214[81], 6}, {NULL}};
const Segment SGI177153[] = {{&S45030[44], 9}, {NULL}};
const Segment SGI177154[] = {{&S45030[44], 20}, {NULL}};
const Segment SGI177155[] = {{&S45030[30], 6}, {NULL}};
const Segment SGI177156[] = {{&S45030[131], 3}, {NULL}};
const Segment SGI177157[] = {{&S45030[96], 6}, {NULL}};
const Segment SGI177158[] = {{&S45030[66], 8}, {NULL}};
const Segment SGI177159[] = {{&S45031[44], 9}, {NULL}};
const Segment SGI177160[] = {{&S45031[44], 20}, {NULL}};
const Segment SGI177161[] = {{&S45031[30], 6}, {NULL}};
const Segment SGI177162[] = {{&S45031[131], 3}, {NULL}};
const Segment SGI177163[] = {{&S45031[96], 6}, {NULL}};
const Segment SGI177164[] = {{&S45031[66], 8}, {NULL}};
const Segment SGI177165[] = {{&S45033[44], 9}, {NULL}};
const Segment SGI177166[] = {{&S45033[44], 21}, {NULL}};
const Segment SGI177167[] = {{&S45033[30], 6}, {NULL}};
const Segment SGI177168[] = {{&S45033[132], 3}, {NULL}};
const Segment SGI177169[] = {{&S45033[97], 6}, {NULL}};
const Segment SGI177170[] = {{&S45033[67], 8}, {NULL}};
const Segment SGI177171[] = {{&S67939[34], 7}, {NULL}};
const Segment SGI177172[] = {{&S67939[27], 6}, {NULL}};
const Segment SGI177173[] = {{&S67939[99], 9}, {NULL}};
const Segment SGI177174[] = {{&S67939[72], 6}, {NULL}};
const Segment SGI177175[] = {{&S45035[44], 9}, {NULL}};
const Segment SGI177176[] = {{&S45035[44], 21}, {NULL}};
const Segment SGI177177[] = {{&S45035[30], 6}, {NULL}};
const Segment SGI177178[] = {{&S45035[132], 3}, {NULL}};
const Segment SGI177179[] = {{&S67940[44], 11}, {NULL}};
const Segment SGI177180[] = {{&S45035[97], 6}, {NULL}};
const Segment SGI177181[] = {{&S67940[44], 23}, {NULL}};
const Segment SGI177182[] = {{&S67940[30], 6}, {NULL}};
const Segment SGI177183[] = {{&S45035[67], 8}, {NULL}};
const Segment SGI177184[] = {{&S67940[31], 3}, {NULL}};
const Segment SGI177185[] = {{&S67940[81], 6}, {NULL}};
const Segment SGI177186[] = {{&S45036[44], 9}, {NULL}};
const Segment SGI177187[] = {{&S45036[44], 19}, {NULL}};
const Segment SGI177188[] = {{&S45036[30], 6}, {NULL}};
const Segment SGI177189[] = {{&S45036[130], 3}, {NULL}};
const Segment SGI177190[] = {{&S45036[95], 6}, {NULL}};
const Segment SGI177191[] = {{&S45036[65], 8}, {NULL}};
const Segment SGI177192[] = {{&S67945[44], 6}, {NULL}};
const Segment SGI177193[] = {{&S67945[44], 18}, {NULL}};
const Segment SGI177194[] = {{&S67945[30], 6}, {NULL}};
const Segment SGI177195[] = {{&S67945[31], 3}, {NULL}};
const Segment SGI177196[] = {{&S67945[76], 6}, {NULL}};
const Segment SGI177197[] = {{&S60232[34], 11}, {NULL}};
const Segment SGI177198[] = {{&S60232[27], 6}, {NULL}};
const Segment SGI177199[] = {{&S60232[98], 9}, {NULL}};
const Segment SGI177200[] = {{&S60232[71], 6}, {NULL}};
const Segment SGI177201[] = {{&S60233[34], 11}, {NULL}};
const Segment SGI177202[] = {{&S60233[27], 6}, {NULL}};
const Segment SGI177203[] = {{&S60233[98], 9}, {NULL}};
const Segment SGI177204[] = {{&S60233[71], 6}, {NULL}};
const Segment SGI177205[] = {{&S67949[44], 14}, {NULL}};
const Segment SGI177206[] = {{&S67949[44], 26}, {NULL}};
const Segment SGI177207[] = {{&S67949[30], 6}, {NULL}};
const Segment SGI177208[] = {{&S67949[31], 3}, {NULL}};
const Segment SGI177209[] = {{&S67949[84], 6}, {NULL}};
const Segment SGI177210[] = {{&S67950[34], 14}, {NULL}};
const Segment SGI177211[] = {{&S67950[27], 6}, {NULL}};
const Segment SGI177212[] = {{&S67950[101], 9}, {NULL}};
const Segment SGI177213[] = {{&S67950[74], 6}, {NULL}};
const Segment SGI177214[] = {{&S67969[44], 5}, {NULL}};
const Segment SGI177215[] = {{&S67969[44], 17}, {NULL}};
const Segment SGI177216[] = {{&S67969[30], 6}, {NULL}};
const Segment SGI177217[] = {{&S67969[31], 3}, {NULL}};
const Segment SGI177218[] = {{&S67969[75], 6}, {NULL}};
const Segment SGI177219[] = {{&S54546[44], 10}, {NULL}};
const Segment SGI177220[] = {{&S54546[44], 22}, {NULL}};
const Segment SGI177221[] = {{&S54546[30], 6}, {NULL}};
const Segment SGI177222[] = {{&S54546[131], 3}, {NULL}};
const Segment SGI177223[] = {{&S54546[96], 6}, {NULL}};
const Segment SGI177224[] = {{&S54546[68], 8}, {NULL}};
const Segment SGI177225[] = {{&S67978[34], 10}, {NULL}};
const Segment SGI177226[] = {{&S67978[27], 6}, {NULL}};
const Segment SGI177227[] = {{&S54547[44], 10}, {NULL}};
const Segment SGI177228[] = {{&S54547[44], 22}, {NULL}};
const Segment SGI177229[] = {{&S54547[30], 6}, {NULL}};
const Segment SGI177230[] = {{&S67978[97], 9}, {NULL}};
const Segment SGI177231[] = {{&S67978[70], 6}, {NULL}};
const Segment SGI177232[] = {{&S54547[133], 3}, {NULL}};
const Segment SGI177233[] = {{&S54547[98], 6}, {NULL}};
const Segment SGI177234[] = {{&S54547[68], 8}, {NULL}};
const Segment SGI177235[] = {{&S54548[44], 10}, {NULL}};
const Segment SGI177236[] = {{&S54548[44], 22}, {NULL}};
const Segment SGI177237[] = {{&S54548[30], 6}, {NULL}};
const Segment SGI177238[] = {{&S54548[132], 3}, {NULL}};
const Segment SGI177239[] = {{&S54548[98], 6}, {NULL}};
const Segment SGI177240[] = {{&S54548[68], 8}, {NULL}};
const Segment SGI177241[] = {{&S68002[44], 7}, {NULL}};
const Segment SGI177242[] = {{&S68002[44], 19}, {NULL}};
const Segment SGI177243[] = {{&S68002[30], 6}, {NULL}};
const Segment SGI177244[] = {{&S68002[31], 3}, {NULL}};
const Segment SGI177245[] = {{&S68002[77], 6}, {NULL}};
const Segment SGI177246[] = {{&S45106[38], 10}, {NULL}};
const Segment SGI177247[] = {{&S45106[31], 6}, {NULL}};
const Segment SGI177248[] = {{&S45108[44], 10}, {NULL}};
const Segment SGI177249[] = {{&S45108[44], 21}, {NULL}};
const Segment SGI177250[] = {{&S45108[30], 6}, {NULL}};
const Segment SGI177251[] = {{&S45108[114], 3}, {NULL}};
const Segment SGI177252[] = {{&S45108[79], 6}, {NULL}};
const Segment SGI177253[] = {{&S45109[44], 10}, {NULL}};
const Segment SGI177254[] = {{&S45109[44], 22}, {NULL}};
const Segment SGI177255[] = {{&S45109[30], 6}, {NULL}};
const Segment SGI177256[] = {{&S45109[115], 3}, {NULL}};
const Segment SGI177257[] = {{&S45109[80], 6}, {NULL}};
const Segment SGI177258[] = {{&S45110[44], 10}, {NULL}};
const Segment SGI177259[] = {{&S45110[44], 21}, {NULL}};
const Segment SGI177260[] = {{&S45110[30], 6}, {NULL}};
const Segment SGI177261[] = {{&S45110[114], 3}, {NULL}};
const Segment SGI177262[] = {{&S45110[79], 6}, {NULL}};
const Segment SGI177263[] = {{&S68037[44], 11}, {NULL}};
const Segment SGI177264[] = {{&S68037[44], 23}, {NULL}};
const Segment SGI177265[] = {{&S68037[30], 6}, {NULL}};
const Segment SGI177266[] = {{&S68037[31], 3}, {NULL}};
const Segment SGI177267[] = {{&S68037[81], 6}, {NULL}};
const Segment SGI177268[] = {{&S68045[34], 7}, {NULL}};
const Segment SGI177269[] = {{&S68045[27], 6}, {NULL}};
const Segment SGI177270[] = {{&S68045[100], 9}, {NULL}};
const Segment SGI177271[] = {{&S68045[73], 6}, {NULL}};
const Segment SGI177272[] = {{&S68071[34], 11}, {NULL}};
const Segment SGI177273[] = {{&S68071[27], 6}, {NULL}};
const Segment SGI177274[] = {{&S68071[98], 9}, {NULL}};
const Segment SGI177275[] = {{&S68071[71], 6}, {NULL}};
const Segment SGI177276[] = {{&S56711[34], 6}, {NULL}};
const Segment SGI177277[] = {{&S56711[27], 6}, {NULL}};
const Segment SGI177278[] = {{&S56711[93], 9}, {NULL}};
const Segment SGI177279[] = {{&S56711[66], 6}, {NULL}};
const Segment SGI177280[] = {{&S60378[44], 13}, {NULL}};
const Segment SGI177281[] = {{&S60378[44], 25}, {NULL}};
const Segment SGI177282[] = {{&S60378[30], 6}, {NULL}};
const Segment SGI177283[] = {{&S60378[118], 3}, {NULL}};
const Segment SGI177284[] = {{&S60378[83], 6}, {NULL}};
const Segment SGI177285[] = {{&S60379[38], 13}, {NULL}};
const Segment SGI177286[] = {{&S60379[31], 6}, {NULL}};
const Segment SGI177287[] = {{&S56720[34], 10}, {NULL}};
const Segment SGI177288[] = {{&S56720[27], 6}, {NULL}};
const Segment SGI177289[] = {{&S56720[97], 9}, {NULL}};
const Segment SGI177290[] = {{&S56720[70], 6}, {NULL}};
const Segment SGI177291[] = {{&S56721[34], 10}, {NULL}};
const Segment SGI177292[] = {{&S56721[27], 6}, {NULL}};
const Segment SGI177293[] = {{&S56721[97], 9}, {NULL}};
const Segment SGI177294[] = {{&S56721[70], 6}, {NULL}};
const Segment SGI177295[] = {{&S56723[34], 14}, {NULL}};
const Segment SGI177296[] = {{&S56723[27], 6}, {NULL}};
const Segment SGI177297[] = {{&S56723[101], 9}, {NULL}};
const Segment SGI177298[] = {{&S56723[74], 6}, {NULL}};
const Segment SGI177299[] = {{&S68091[34], 10}, {NULL}};
const Segment SGI177300[] = {{&S56724[34], 13}, {NULL}};
const Segment SGI177301[] = {{&S68091[27], 6}, {NULL}};
const Segment SGI177302[] = {{&S56724[27], 6}, {NULL}};
const Segment SGI177303[] = {{&S68091[97], 9}, {NULL}};
const Segment SGI177304[] = {{&S68091[70], 6}, {NULL}};
const Segment SGI177305[] = {{&S56724[100], 9}, {NULL}};
const Segment SGI177306[] = {{&S56724[73], 6}, {NULL}};
const Segment SGI177307[] = {{&S68093[34], 11}, {NULL}};
const Segment SGI177308[] = {{&S68093[27], 6}, {NULL}};
const Segment SGI177309[] = {{&S68093[98], 9}, {NULL}};
const Segment SGI177310[] = {{&S68093[71], 6}, {NULL}};
const Segment SGI177311[] = {{&S60408[44], 9}, {NULL}};
const Segment SGI177312[] = {{&S60408[44], 20}, {NULL}};
const Segment SGI177313[] = {{&S60408[30], 6}, {NULL}};
const Segment SGI177314[] = {{&S60408[132], 3}, {NULL}};
const Segment SGI177315[] = {{&S60408[97], 6}, {NULL}};
const Segment SGI177316[] = {{&S60408[66], 8}, {NULL}};
const Segment SGI177317[] = {{&S68116[44], 9}, {NULL}};
const Segment SGI177318[] = {{&S68116[44], 21}, {NULL}};
const Segment SGI177319[] = {{&S68116[30], 6}, {NULL}};
const Segment SGI177320[] = {{&S68116[31], 3}, {NULL}};
const Segment SGI177321[] = {{&S68116[79], 6}, {NULL}};
const Segment SGI177322[] = {{&S60425[44], 6}, {NULL}};
const Segment SGI177323[] = {{&S60425[44], 18}, {NULL}};
const Segment SGI177324[] = {{&S60425[30], 6}, {NULL}};
const Segment SGI177325[] = {{&S60425[111], 3}, {NULL}};
const Segment SGI177326[] = {{&S60425[76], 6}, {NULL}};
const Segment SGI177327[] = {{&S60426[38], 6}, {NULL}};
const Segment SGI177328[] = {{&S60426[31], 6}, {NULL}};
const Segment SGI177329[] = {{&S56769[44], 7}, {NULL}};
const Segment SGI177330[] = {{&S56769[44], 24}, {NULL}};
const Segment SGI177331[] = {{&S56769[30], 6}, {NULL}};
const Segment SGI177332[] = {{&S56769[85], 6}, {NULL}};
const Segment SGI177333[] = {{&S56770[44], 7}, {NULL}};
const Segment SGI177334[] = {{&S56770[44], 24}, {NULL}};
const Segment SGI177335[] = {{&S56770[30], 6}, {NULL}};
const Segment SGI177336[] = {{&S56770[117], 3}, {NULL}};
const Segment SGI177337[] = {{&S56770[82], 6}, {NULL}};
const Segment SGI177338[] = {{&S56771[44], 11}, {NULL}};
const Segment SGI177339[] = {{&S56771[44], 28}, {NULL}};
const Segment SGI177340[] = {{&S56771[30], 6}, {NULL}};
const Segment SGI177341[] = {{&S56771[121], 3}, {NULL}};
const Segment SGI177342[] = {{&S56771[86], 6}, {NULL}};
const Segment SGI177343[] = {{&S56772[44], 7}, {NULL}};
const Segment SGI177344[] = {{&S56772[44], 24}, {NULL}};
const Segment SGI177345[] = {{&S56772[30], 6}, {NULL}};
const Segment SGI177346[] = {{&S56772[117], 3}, {NULL}};
const Segment SGI177347[] = {{&S56772[82], 6}, {NULL}};
const Segment SGI177348[] = {{&S56773[38], 7}, {NULL}};
const Segment SGI177349[] = {{&S56773[31], 6}, {NULL}};
const Segment SGI177350[] = {{&S56774[44], 7}, {NULL}};
const Segment SGI177351[] = {{&S56774[44], 24}, {NULL}};
const Segment SGI177352[] = {{&S56774[30], 6}, {NULL}};
const Segment SGI177353[] = {{&S56774[117], 3}, {NULL}};
const Segment SGI177354[] = {{&S56774[82], 6}, {NULL}};
const Segment SGI177355[] = {{&S56775[44], 11}, {NULL}};
const Segment SGI177356[] = {{&S56775[44], 28}, {NULL}};
const Segment SGI177357[] = {{&S56775[30], 6}, {NULL}};
const Segment SGI177358[] = {{&S56775[121], 3}, {NULL}};
const Segment SGI177359[] = {{&S56775[86], 6}, {NULL}};
const Segment SGI177360[] = {{&S68149[34], 11}, {NULL}};
const Segment SGI177361[] = {{&S68149[27], 6}, {NULL}};
const Segment SGI177362[] = {{&S68149[108], 9}, {NULL}};
const Segment SGI177363[] = {{&S68149[81], 6}, {NULL}};
const Segment SGI177364[] = {{&S56781[34], 19}, {NULL}};
const Segment SGI177365[] = {{&S56781[27], 6}, {NULL}};
const Segment SGI177366[] = {{&S56781[106], 9}, {NULL}};
const Segment SGI177367[] = {{&S56781[79], 6}, {NULL}};
const Segment SGI177368[] = {{&S56782[34], 13}, {NULL}};
const Segment SGI177369[] = {{&S56782[27], 6}, {NULL}};
const Segment SGI177370[] = {{&S56782[100], 9}, {NULL}};
const Segment SGI177371[] = {{&S56782[73], 6}, {NULL}};
const Segment SGI177372[] = {{&S56784[44], 9}, {NULL}};
const Segment SGI177373[] = {{&S56784[44], 21}, {NULL}};
const Segment SGI177374[] = {{&S56784[30], 6}, {NULL}};
const Segment SGI177375[] = {{&S56784[114], 3}, {NULL}};
const Segment SGI177376[] = {{&S56784[79], 6}, {NULL}};
const Segment SGI177377[] = {{&S56785[38], 9}, {NULL}};
const Segment SGI177378[] = {{&S56785[31], 6}, {NULL}};
const Segment SGI177379[] = {{&S56786[44], 24}, {NULL}};
const Segment SGI177380[] = {{&S56786[44], 36}, {NULL}};
const Segment SGI177381[] = {{&S56786[30], 6}, {NULL}};
const Segment SGI177382[] = {{&S56786[129], 3}, {NULL}};
const Segment SGI177383[] = {{&S56786[94], 6}, {NULL}};
const Segment SGI177384[] = {{&S56787[44], 14}, {NULL}};
const Segment SGI177385[] = {{&S56787[44], 26}, {NULL}};
const Segment SGI177386[] = {{&S56787[30], 6}, {NULL}};
const Segment SGI177387[] = {{&S56787[119], 3}, {NULL}};
const Segment SGI177388[] = {{&S56787[84], 6}, {NULL}};
const Segment SGI177389[] = {{&S56788[44], 24}, {NULL}};
const Segment SGI177390[] = {{&S56788[44], 36}, {NULL}};
const Segment SGI177391[] = {{&S56788[30], 6}, {NULL}};
const Segment SGI177392[] = {{&S56788[129], 3}, {NULL}};
const Segment SGI177393[] = {{&S56788[94], 6}, {NULL}};
const Segment SGI177394[] = {{&S56789[44], 14}, {NULL}};
const Segment SGI177395[] = {{&S56789[44], 26}, {NULL}};
const Segment SGI177396[] = {{&S56789[30], 6}, {NULL}};
const Segment SGI177397[] = {{&S56789[119], 3}, {NULL}};
const Segment SGI177398[] = {{&S56789[84], 6}, {NULL}};
const Segment SGI177399[] = {{&S56790[44], 24}, {NULL}};
const Segment SGI177400[] = {{&S56790[44], 36}, {NULL}};
const Segment SGI177401[] = {{&S56790[30], 6}, {NULL}};
const Segment SGI177402[] = {{&S56790[129], 3}, {NULL}};
const Segment SGI177403[] = {{&S56790[94], 6}, {NULL}};
const Segment SGI177404[] = {{&S56791[44], 24}, {NULL}};
const Segment SGI177405[] = {{&S56791[44], 36}, {NULL}};
const Segment SGI177406[] = {{&S56791[30], 6}, {NULL}};
const Segment SGI177407[] = {{&S56791[129], 3}, {NULL}};
const Segment SGI177408[] = {{&S56791[94], 6}, {NULL}};
const Segment SGI177409[] = {{&S60483[34], 12}, {NULL}};
const Segment SGI177410[] = {{&S60483[27], 6}, {NULL}};
const Segment SGI177411[] = {{&S60483[99], 9}, {NULL}};
const Segment SGI177412[] = {{&S60483[72], 6}, {NULL}};
const Segment SGI177413[] = {{&S60484[34], 12}, {NULL}};
const Segment SGI177414[] = {{&S60484[27], 6}, {NULL}};
const Segment SGI177415[] = {{&S68176[44], 12}, {NULL}};
const Segment SGI177416[] = {{&S68176[44], 24}, {NULL}};
const Segment SGI177417[] = {{&S60484[99], 9}, {NULL}};
const Segment SGI177418[] = {{&S68176[30], 6}, {NULL}};
const Segment SGI177419[] = {{&S60484[72], 6}, {NULL}};
const Segment SGI177420[] = {{&S68176[31], 3}, {NULL}};
const Segment SGI177421[] = {{&S68176[82], 6}, {NULL}};
const Segment SGI177422[] = {{&S56805[34], 18}, {NULL}};
const Segment SGI177423[] = {{&S56805[27], 6}, {NULL}};
const Segment SGI177424[] = {{&S56805[105], 9}, {NULL}};
const Segment SGI177425[] = {{&S56805[78], 6}, {NULL}};
const Segment SGI177426[] = {{&S56806[34], 10}, {NULL}};
const Segment SGI177427[] = {{&S56806[27], 6}, {NULL}};
const Segment SGI177428[] = {{&S56806[97], 9}, {NULL}};
const Segment SGI177429[] = {{&S56806[70], 6}, {NULL}};
const Segment SGI177430[] = {{&S56807[34], 18}, {NULL}};
const Segment SGI177431[] = {{&S56807[27], 6}, {NULL}};
const Segment SGI177432[] = {{&S56807[105], 9}, {NULL}};
const Segment SGI177433[] = {{&S56807[78], 6}, {NULL}};
const Segment SGI177434[] = {{&S56811[34], 7}, {NULL}};
const Segment SGI177435[] = {{&S56811[27], 6}, {NULL}};
const Segment SGI177436[] = {{&S56811[100], 9}, {NULL}};
const Segment SGI177437[] = {{&S56811[73], 6}, {NULL}};
const Segment SGI177438[] = {{&S68194[44], 8}, {NULL}};
const Segment SGI177439[] = {{&S68194[44], 26}, {NULL}};
const Segment SGI177440[] = {{&S68194[30], 6}, {NULL}};
const Segment SGI177441[] = {{&S68194[31], 3}, {NULL}};
const Segment SGI177442[] = {{&S56812[34], 9}, {NULL}};
const Segment SGI177443[] = {{&S68194[84], 6}, {NULL}};
const Segment SGI177444[] = {{&S56812[27], 6}, {NULL}};
const Segment SGI177445[] = {{&S56812[96], 9}, {NULL}};
const Segment SGI177446[] = {{&S56812[69], 6}, {NULL}};
const Segment SGI177447[] = {{&S68195[44], 8}, {NULL}};
const Segment SGI177448[] = {{&S68195[44], 26}, {NULL}};
const Segment SGI177449[] = {{&S68195[30], 6}, {NULL}};
const Segment SGI177450[] = {{&S68195[87], 6}, {NULL}};
const Segment SGI177451[] = {{&S68196[44], 8}, {NULL}};
const Segment SGI177452[] = {{&S68196[44], 20}, {NULL}};
const Segment SGI177453[] = {{&S68196[30], 6}, {NULL}};
const Segment SGI177454[] = {{&S68196[31], 3}, {NULL}};
const Segment SGI177455[] = {{&S68196[78], 6}, {NULL}};
const Segment SGI177456[] = {{&S68197[11], 1}, {NULL}};
const Segment SGI177457[] = {{&S68197[44], 10}, {NULL}};
const Segment SGI177458[] = {{&S68197[30], 6}, {NULL}};
const Segment SGI177459[] = {{&S68197[31], 3}, {NULL}};
const Segment SGI177460[] = {{&S68197[82], 6}, {NULL}};
const Segment SGI177461[] = {{&S68198[44], 7}, {NULL}};
const Segment SGI177462[] = {{&S68198[44], 19}, {NULL}};
const Segment SGI177463[] = {{&S68198[30], 6}, {NULL}};
const Segment SGI177464[] = {{&S68198[31], 3}, {NULL}};
const Segment SGI177465[] = {{&S68198[77], 6}, {NULL}};
const Segment SGI177466[] = {{&S68199[38], 7}, {NULL}};
const Segment SGI177467[] = {{&S68199[31], 6}, {NULL}};
const Segment SGI177468[] = {{&S60520[44], 11}, {NULL}};
const Segment SGI177469[] = {{&S60520[44], 23}, {NULL}};
const Segment SGI177470[] = {{&S60520[30], 6}, {NULL}};
const Segment SGI177471[] = {{&S56829[44], 10}, {NULL}};
const Segment SGI177472[] = {{&S56829[44], 22}, {NULL}};
const Segment SGI177473[] = {{&S56829[30], 6}, {NULL}};
const Segment SGI177474[] = {{&S60520[116], 3}, {NULL}};
const Segment SGI177475[] = {{&S60520[81], 6}, {NULL}};
const Segment SGI177476[] = {{&S56829[115], 3}, {NULL}};
const Segment SGI177477[] = {{&S56829[80], 6}, {NULL}};
const Segment SGI177478[] = {{&S56830[44], 9}, {NULL}};
const Segment SGI177479[] = {{&S56830[44], 21}, {NULL}};
const Segment SGI177480[] = {{&S56830[30], 6}, {NULL}};
const Segment SGI177481[] = {{&S56830[114], 3}, {NULL}};
const Segment SGI177482[] = {{&S56830[79], 6}, {NULL}};
const Segment SGI177483[] = {{&S56831[44], 10}, {NULL}};
const Segment SGI177484[] = {{&S56831[44], 22}, {NULL}};
const Segment SGI177485[] = {{&S56831[30], 6}, {NULL}};
const Segment SGI177486[] = {{&S56831[115], 3}, {NULL}};
const Segment SGI177487[] = {{&S56831[80], 6}, {NULL}};
const Segment SGI177488[] = {{&S56832[38], 9}, {NULL}};
const Segment SGI177489[] = {{&S56832[31], 6}, {NULL}};
const Segment SGI177490[] = {{&S56833[38], 10}, {NULL}};
const Segment SGI177491[] = {{&S56833[31], 6}, {NULL}};
const Segment SGI177492[] = {{&S60531[34], 7}, {NULL}};
const Segment SGI177493[] = {{&S60531[27], 6}, {NULL}};
const Segment SGI177494[] = {{&S60531[94], 9}, {NULL}};
const Segment SGI177495[] = {{&S60531[67], 6}, {NULL}};
const Segment SGI177496[] = {{&S60532[34], 9}, {NULL}};
const Segment SGI177497[] = {{&S60532[27], 6}, {NULL}};
const Segment SGI177498[] = {{&S60532[96], 9}, {NULL}};
const Segment SGI177499[] = {{&S60532[69], 6}, {NULL}};
const Segment SGI177500[] = {{&S68227[44], 11}, {NULL}};
const Segment SGI177501[] = {{&S68227[44], 23}, {NULL}};
const Segment SGI177502[] = {{&S68227[30], 6}, {NULL}};
const Segment SGI177503[] = {{&S68227[31], 3}, {NULL}};
const Segment SGI177504[] = {{&S68227[81], 6}, {NULL}};
const Segment SGI177505[] = {{&S56853[34], 18}, {NULL}};
const Segment SGI177506[] = {{&S56853[27], 6}, {NULL}};
const Segment SGI177507[] = {{&S56853[105], 9}, {NULL}};
const Segment SGI177508[] = {{&S56853[78], 6}, {NULL}};
const Segment SGI177509[] = {{&S68237[34], 20}, {NULL}};
const Segment SGI177510[] = {{&S68237[27], 6}, {NULL}};
const Segment SGI177511[] = {{&S68237[123], 9}, {NULL}};
const Segment SGI177512[] = {{&S68237[96], 6}, {NULL}};
const Segment SGI177513[] = {{&S56868[34], 9}, {NULL}};
const Segment SGI177514[] = {{&S56868[27], 6}, {NULL}};
const Segment SGI177515[] = {{&S56868[96], 9}, {NULL}};
const Segment SGI177516[] = {{&S56868[69], 6}, {NULL}};
const Segment SGI177517[] = {{&S54820[34], 9}, {NULL}};
const Segment SGI177518[] = {{&S54820[27], 6}, {NULL}};
const Segment SGI177519[] = {{&S54820[96], 9}, {NULL}};
const Segment SGI177520[] = {{&S54820[69], 6}, {NULL}};
const Segment SGI177521[] = {{&S68277[34], 10}, {NULL}};
const Segment SGI177522[] = {{&S68277[27], 6}, {NULL}};
const Segment SGI177523[] = {{&S68277[97], 9}, {NULL}};
const Segment SGI177524[] = {{&S68277[70], 6}, {NULL}};
const Segment SGI177525[] = {{&S45401[34], 10}, {NULL}};
const Segment SGI177526[] = {{&S45401[27], 6}, {NULL}};
const Segment SGI177527[] = {{&S45401[96], 9}, {NULL}};
const Segment SGI177528[] = {{&S45401[69], 6}, {NULL}};
const Segment SGI177529[] = {{&S68279[34], 11}, {NULL}};
const Segment SGI177530[] = {{&S68279[27], 6}, {NULL}};
const Segment SGI177531[] = {{&S68279[98], 9}, {NULL}};
const Segment SGI177532[] = {{&S68279[71], 6}, {NULL}};
const Segment SGI177533[] = {{&S45402[34], 10}, {NULL}};
const Segment SGI177534[] = {{&S45402[27], 6}, {NULL}};
const Segment SGI177535[] = {{&S45402[96], 9}, {NULL}};
const Segment SGI177536[] = {{&S45402[69], 6}, {NULL}};
const Segment SGI177537[] = {{&S60604[44], 18}, {NULL}};
const Segment SGI177538[] = {{&S60604[44], 30}, {NULL}};
const Segment SGI177539[] = {{&S60604[30], 6}, {NULL}};
const Segment SGI177540[] = {{&S60604[123], 3}, {NULL}};
const Segment SGI177541[] = {{&S60604[88], 6}, {NULL}};
const Segment SGI177542[] = {{&S60604[148], 7}, {NULL}};
const Segment SGI177543[] = {{&S60606[44], 11}, {NULL}};
const Segment SGI177544[] = {{&S60606[44], 23}, {NULL}};
const Segment SGI177545[] = {{&S60606[30], 6}, {NULL}};
const Segment SGI177546[] = {{&S60606[116], 3}, {NULL}};
const Segment SGI177547[] = {{&S60606[81], 6}, {NULL}};
const Segment SGI177548[] = {{&S60606[141], 7}, {NULL}};
const Segment SGI177549[] = {{&S68301[34], 18}, {NULL}};
const Segment SGI177550[] = {{&S68301[27], 6}, {NULL}};
const Segment SGI177551[] = {{&S60614[34], 18}, {NULL}};
const Segment SGI177552[] = {{&S60614[27], 6}, {NULL}};
const Segment SGI177553[] = {{&S68301[105], 9}, {NULL}};
const Segment SGI177554[] = {{&S68301[78], 6}, {NULL}};
const Segment SGI177555[] = {{&S60614[105], 9}, {NULL}};
const Segment SGI177556[] = {{&S60614[78], 6}, {NULL}};
const Segment SGI177557[] = {{&S68311[34], 7}, {NULL}};
const Segment SGI177558[] = {{&S68311[27], 6}, {NULL}};
const Segment SGI177559[] = {{&S68311[110], 9}, {NULL}};
const Segment SGI177560[] = {{&S68311[83], 6}, {NULL}};
const Segment SGI177561[] = {{&S68312[34], 7}, {NULL}};
const Segment SGI177562[] = {{&S68312[27], 6}, {NULL}};
const Segment SGI177563[] = {{&S68312[103], 9}, {NULL}};
const Segment SGI177564[] = {{&S68312[76], 6}, {NULL}};
const Segment SGI177565[] = {{&S60647[34], 9}, {NULL}};
const Segment SGI177566[] = {{&S60647[27], 6}, {NULL}};
const Segment SGI177567[] = {{&S60647[96], 9}, {NULL}};
const Segment SGI177568[] = {{&S60647[69], 6}, {NULL}};
const Segment SGI177569[] = {{&S68330[34], 7}, {NULL}};
const Segment SGI177570[] = {{&S68330[27], 6}, {NULL}};
const Segment SGI177571[] = {{&S68330[94], 9}, {NULL}};
const Segment SGI177572[] = {{&S68330[67], 6}, {NULL}};
const Segment SGI177573[] = {{&S68331[34], 7}, {NULL}};
const Segment SGI177574[] = {{&S68331[27], 6}, {NULL}};
const Segment SGI177575[] = {{&S68331[100], 9}, {NULL}};
const Segment SGI177576[] = {{&S68331[73], 6}, {NULL}};
const Segment SGI177577[] = {{&S45460[34], 18}, {NULL}};
const Segment SGI177578[] = {{&S45460[27], 6}, {NULL}};
const Segment SGI177579[] = {{&S45460[105], 9}, {NULL}};
const Segment SGI177580[] = {{&S45460[78], 6}, {NULL}};
const Segment SGI177581[] = {{&S68332[34], 7}, {NULL}};
const Segment SGI177582[] = {{&S60651[44], 9}, {NULL}};
const Segment SGI177583[] = {{&S60651[44], 21}, {NULL}};
const Segment SGI177584[] = {{&S68332[27], 6}, {NULL}};
const Segment SGI177585[] = {{&S60651[30], 6}, {NULL}};
const Segment SGI177586[] = {{&S60651[114], 3}, {NULL}};
const Segment SGI177587[] = {{&S68332[100], 9}, {NULL}};
const Segment SGI177588[] = {{&S60651[79], 6}, {NULL}};
const Segment SGI177589[] = {{&S68332[73], 6}, {NULL}};
const Segment SGI177590[] = {{&S45461[34], 10}, {NULL}};
const Segment SGI177591[] = {{&S45461[27], 6}, {NULL}};
const Segment SGI177592[] = {{&S45461[97], 9}, {NULL}};
const Segment SGI177593[] = {{&S45461[70], 6}, {NULL}};
const Segment SGI177594[] = {{&S60652[44], 10}, {NULL}};
const Segment SGI177595[] = {{&S68333[34], 11}, {NULL}};
const Segment SGI177596[] = {{&S60652[44], 22}, {NULL}};
const Segment SGI177597[] = {{&S68333[27], 6}, {NULL}};
const Segment SGI177598[] = {{&S60652[30], 6}, {NULL}};
const Segment SGI177599[] = {{&S60652[115], 3}, {NULL}};
const Segment SGI177600[] = {{&S68333[98], 9}, {NULL}};
const Segment SGI177601[] = {{&S60652[80], 6}, {NULL}};
const Segment SGI177602[] = {{&S68333[71], 6}, {NULL}};
const Segment SGI177603[] = {{&S45462[34], 10}, {NULL}};
const Segment SGI177604[] = {{&S45462[27], 6}, {NULL}};
const Segment SGI177605[] = {{&S45462[96], 9}, {NULL}};
const Segment SGI177606[] = {{&S45462[69], 6}, {NULL}};
const Segment SGI177607[] = {{&S60653[44], 9}, {NULL}};
const Segment SGI177608[] = {{&S60653[44], 21}, {NULL}};
const Segment SGI177609[] = {{&S60653[30], 6}, {NULL}};
const Segment SGI177610[] = {{&S60653[114], 3}, {NULL}};
const Segment SGI177611[] = {{&S60653[79], 6}, {NULL}};
const Segment SGI177612[] = {{&S45463[34], 10}, {NULL}};
const Segment SGI177613[] = {{&S45463[27], 6}, {NULL}};
const Segment SGI177614[] = {{&S45463[97], 9}, {NULL}};
const Segment SGI177615[] = {{&S45463[70], 6}, {NULL}};
const Segment SGI177616[] = {{&S60654[44], 9}, {NULL}};
const Segment SGI177617[] = {{&S60654[44], 22}, {NULL}};
const Segment SGI177618[] = {{&S60654[30], 6}, {NULL}};
const Segment SGI177619[] = {{&S60654[115], 3}, {NULL}};
const Segment SGI177620[] = {{&S60654[80], 6}, {NULL}};
const Segment SGI177621[] = {{&S45464[34], 11}, {NULL}};
const Segment SGI177622[] = {{&S45464[27], 6}, {NULL}};
const Segment SGI177623[] = {{&S45464[98], 9}, {NULL}};
const Segment SGI177624[] = {{&S45464[71], 6}, {NULL}};
const Segment SGI177625[] = {{&S60655[44], 9}, {NULL}};
const Segment SGI177626[] = {{&S60655[44], 21}, {NULL}};
const Segment SGI177627[] = {{&S60655[30], 6}, {NULL}};
const Segment SGI177628[] = {{&S60655[114], 3}, {NULL}};
const Segment SGI177629[] = {{&S60655[79], 6}, {NULL}};
const Segment SGI177630[] = {{&S60656[44], 25}, {NULL}};
const Segment SGI177631[] = {{&S60656[44], 37}, {NULL}};
const Segment SGI177632[] = {{&S60656[30], 6}, {NULL}};
const Segment SGI177633[] = {{&S60656[31], 3}, {NULL}};
const Segment SGI177634[] = {{&S60656[95], 6}, {NULL}};
const Segment SGI177635[] = {{&S60657[44], 32}, {NULL}};
const Segment SGI177636[] = {{&S60657[44], 44}, {NULL}};
const Segment SGI177637[] = {{&S60657[30], 6}, {NULL}};
const Segment SGI177638[] = {{&S60657[31], 3}, {NULL}};
const Segment SGI177639[] = {{&S60657[102], 6}, {NULL}};
const Segment SGI177640[] = {{&S56999[34], 8}, {NULL}};
const Segment SGI177641[] = {{&S56999[27], 6}, {NULL}};
const Segment SGI177642[] = {{&S56999[100], 9}, {NULL}};
const Segment SGI177643[] = {{&S56999[73], 6}, {NULL}};
const Segment SGI177644[] = {{&S68352[34], 7}, {NULL}};
const Segment SGI177645[] = {{&S68352[27], 6}, {NULL}};
const Segment SGI177646[] = {{&S68352[99], 9}, {NULL}};
const Segment SGI177647[] = {{&S68352[72], 6}, {NULL}};
const Segment SGI177648[] = {{&S68364[34], 8}, {NULL}};
const Segment SGI177649[] = {{&S68364[27], 6}, {NULL}};
const Segment SGI177650[] = {{&S68364[106], 9}, {NULL}};
const Segment SGI177651[] = {{&S68364[79], 6}, {NULL}};
const Segment SGI177652[] = {{&S68365[34], 8}, {NULL}};
const Segment SGI177653[] = {{&S68365[27], 6}, {NULL}};
const Segment SGI177654[] = {{&S68365[106], 9}, {NULL}};
const Segment SGI177655[] = {{&S68365[79], 6}, {NULL}};
const Segment SGI177656[] = {{&S60694[44], 13}, {NULL}};
const Segment SGI177657[] = {{&S60694[44], 25}, {NULL}};
const Segment SGI177658[] = {{&S60694[30], 6}, {NULL}};
const Segment SGI177659[] = {{&S60694[118], 3}, {NULL}};
const Segment SGI177660[] = {{&S60694[83], 6}, {NULL}};
const Segment SGI177661[] = {{&S60695[44], 13}, {NULL}};
const Segment SGI177662[] = {{&S60695[44], 25}, {NULL}};
const Segment SGI177663[] = {{&S60695[30], 6}, {NULL}};
const Segment SGI177664[] = {{&S60695[118], 3}, {NULL}};
const Segment SGI177665[] = {{&S60695[83], 6}, {NULL}};
const Segment SGI177666[] = {{&S60696[38], 13}, {NULL}};
const Segment SGI177667[] = {{&S60696[31], 6}, {NULL}};
const Segment SGI177668[] = {{&S68390[44], 11}, {NULL}};
const Segment SGI177669[] = {{&S68390[44], 23}, {NULL}};
const Segment SGI177670[] = {{&S68390[30], 6}, {NULL}};
const Segment SGI177671[] = {{&S68390[31], 3}, {NULL}};
const Segment SGI177672[] = {{&S68390[81], 6}, {NULL}};
const Segment SGI177673[] = {{&S68391[44], 11}, {NULL}};
const Segment SGI177674[] = {{&S68391[44], 23}, {NULL}};
const Segment SGI177675[] = {{&S68391[30], 6}, {NULL}};
const Segment SGI177676[] = {{&S68391[31], 3}, {NULL}};
const Segment SGI177677[] = {{&S68391[81], 6}, {NULL}};
const Segment SGI177678[] = {{&S68392[44], 13}, {NULL}};
const Segment SGI177679[] = {{&S60714[34], 6}, {NULL}};
const Segment SGI177680[] = {{&S68392[44], 48}, {NULL}};
const Segment SGI177681[] = {{&S68392[30], 6}, {NULL}};
const Segment SGI177682[] = {{&S60714[27], 6}, {NULL}};
const Segment SGI177683[] = {{&S68392[31], 3}, {NULL}};
const Segment SGI177684[] = {{&S60714[93], 9}, {NULL}};
const Segment SGI177685[] = {{&S60714[66], 6}, {NULL}};
const Segment SGI177686[] = {{&S68392[106], 6}, {NULL}};
const Segment SGI177687[] = {{&S68393[44], 13}, {NULL}};
const Segment SGI177688[] = {{&S68393[44], 23}, {NULL}};
const Segment SGI177689[] = {{&S68393[30], 6}, {NULL}};
const Segment SGI177690[] = {{&S68393[31], 3}, {NULL}};
const Segment SGI177691[] = {{&S68393[81], 6}, {NULL}};
const Segment SGI177692[] = {{&S68394[44], 13}, {NULL}};
const Segment SGI177693[] = {{&S68394[44], 23}, {NULL}};
const Segment SGI177694[] = {{&S68394[30], 6}, {NULL}};
const Segment SGI177695[] = {{&S60718[34], 10}, {NULL}};
const Segment SGI177696[] = {{&S60718[27], 6}, {NULL}};
const Segment SGI177697[] = {{&S68394[31], 3}, {NULL}};
const Segment SGI177698[] = {{&S68394[81], 6}, {NULL}};
const Segment SGI177699[] = {{&S60718[97], 9}, {NULL}};
const Segment SGI177700[] = {{&S60718[70], 6}, {NULL}};
const Segment SGI177701[] = {{&S60718[138], 8}, {NULL}};
const Segment SGI177702[] = {{&S57064[34], 9}, {NULL}};
const Segment SGI177703[] = {{&S57064[27], 6}, {NULL}};
const Segment SGI177704[] = {{&S57064[96], 9}, {NULL}};
const Segment SGI177705[] = {{&S57064[69], 6}, {NULL}};
const Segment SGI177706[] = {{&S57066[34], 9}, {NULL}};
const Segment SGI177707[] = {{&S57066[27], 6}, {NULL}};
const Segment SGI177708[] = {{&S57066[96], 9}, {NULL}};
const Segment SGI177709[] = {{&S57066[69], 6}, {NULL}};
const Segment SGI177710[] = {{&S68401[34], 7}, {NULL}};
const Segment SGI177711[] = {{&S68401[27], 6}, {NULL}};
const Segment SGI177712[] = {{&S68401[105], 9}, {NULL}};
const Segment SGI177713[] = {{&S68401[78], 6}, {NULL}};
const Segment SGI177714[] = {{&S57073[34], 7}, {NULL}};
const Segment SGI177715[] = {{&S57073[27], 6}, {NULL}};
const Segment SGI177716[] = {{&S57073[100], 9}, {NULL}};
const Segment SGI177717[] = {{&S57073[73], 6}, {NULL}};
const Segment SGI177718[] = {{&S57112[44], 16}, {NULL}};
const Segment SGI177719[] = {{&S57112[44], 28}, {NULL}};
const Segment SGI177720[] = {{&S57112[30], 6}, {NULL}};
const Segment SGI177721[] = {{&S57112[121], 3}, {NULL}};
const Segment SGI177722[] = {{&S57112[86], 6}, {NULL}};
const Segment SGI177723[] = {{&S57113[44], 13}, {NULL}};
const Segment SGI177724[] = {{&S57113[44], 25}, {NULL}};
const Segment SGI177725[] = {{&S57113[30], 6}, {NULL}};
const Segment SGI177726[] = {{&S57113[118], 3}, {NULL}};
const Segment SGI177727[] = {{&S57113[83], 6}, {NULL}};
const Segment SGI177728[] = {{&S57114[38], 16}, {NULL}};
const Segment SGI177729[] = {{&S57114[31], 6}, {NULL}};
const Segment SGI177730[] = {{&S57124[34], 8}, {NULL}};
const Segment SGI177731[] = {{&S57124[27], 6}, {NULL}};
const Segment SGI177732[] = {{&S57124[95], 9}, {NULL}};
const Segment SGI177733[] = {{&S57124[68], 6}, {NULL}};
const Segment SGI177734[] = {{&S57125[34], 9}, {NULL}};
const Segment SGI177735[] = {{&S57125[27], 6}, {NULL}};
const Segment SGI177736[] = {{&S57125[96], 9}, {NULL}};
const Segment SGI177737[] = {{&S57125[69], 6}, {NULL}};
const Segment SGI177738[] = {{&S57144[34], 8}, {NULL}};
const Segment SGI177739[] = {{&S57144[27], 6}, {NULL}};
const Segment SGI177740[] = {{&S57144[95], 9}, {NULL}};
const Segment SGI177741[] = {{&S57144[68], 6}, {NULL}};
const Segment SGI177742[] = {{&S57854[32], 10}, {NULL}};
const Segment SGI177743[] = {{&S57854[27], 4}, {NULL}};
const Segment SGI177744[] = {{&S57854[95], 9}, {NULL}};
const Segment SGI177745[] = {{&S57854[68], 6}, {NULL}};
const Segment SGI177746[] = {{&S45635[41], 18}, {NULL}};
const Segment SGI177747[] = {{&S57858[32], 20}, {NULL}};
const Segment SGI177748[] = {{&S45635[41], 29}, {NULL}};
const Segment SGI177749[] = {{&S57858[27], 4}, {NULL}};
const Segment SGI177750[] = {{&S45635[27], 6}, {NULL}};
const Segment SGI177751[] = {{&S45635[119], 3}, {NULL}};
const Segment SGI177752[] = {{&S57858[105], 9}, {NULL}};
const Segment SGI177753[] = {{&S57858[78], 6}, {NULL}};
const Segment SGI177754[] = {{&S45635[123], 9}, {NULL}};
const Segment SGI177755[] = {{&S45635[84], 6}, {NULL}};
const Segment SGI177756[] = {{&S60820[44], 10}, {NULL}};
const Segment SGI177757[] = {{&S60820[44], 22}, {NULL}};
const Segment SGI177758[] = {{&S60820[30], 6}, {NULL}};
const Segment SGI177759[] = {{&S45636[41], 31}, {NULL}};
const Segment SGI177760[] = {{&S60820[80], 6}, {NULL}};
const Segment SGI177761[] = {{&S45636[41], 42}, {NULL}};
const Segment SGI177762[] = {{&S45636[27], 6}, {NULL}};
const Segment SGI177763[] = {{&S60820[107], 13}, {NULL}};
const Segment SGI177764[] = {{&S45636[132], 3}, {NULL}};
const Segment SGI177765[] = {{&S45636[136], 9}, {NULL}};
const Segment SGI177766[] = {{&S45636[97], 6}, {NULL}};
const Segment SGI177767[] = {{&S45637[41], 31}, {NULL}};
const Segment SGI177768[] = {{&S45637[41], 42}, {NULL}};
const Segment SGI177769[] = {{&S45637[27], 6}, {NULL}};
const Segment SGI177770[] = {{&S45637[132], 3}, {NULL}};
const Segment SGI177771[] = {{&S45637[136], 9}, {NULL}};
const Segment SGI177772[] = {{&S45637[97], 6}, {NULL}};
const Segment SGI177773[] = {{&S57171[34], 10}, {NULL}};
const Segment SGI177774[] = {{&S57171[27], 6}, {NULL}};
const Segment SGI177775[] = {{&S57171[97], 9}, {NULL}};
const Segment SGI177776[] = {{&S57171[70], 6}, {NULL}};
const Segment SGI177777[] = {{&S57172[34], 10}, {NULL}};
const Segment SGI177778[] = {{&S57172[27], 6}, {NULL}};
const Segment SGI177779[] = {{&S57172[97], 9}, {NULL}};
const Segment SGI177780[] = {{&S57172[70], 6}, {NULL}};
const Segment SGI177781[] = {{&S57876[32], 24}, {NULL}};
const Segment SGI177782[] = {{&S57876[27], 4}, {NULL}};
const Segment SGI177783[] = {{&S57876[109], 9}, {NULL}};
const Segment SGI177784[] = {{&S57876[82], 6}, {NULL}};
const Segment SGI177785[] = {{&S60839[44], 12}, {NULL}};
const Segment SGI177786[] = {{&S60839[44], 24}, {NULL}};
const Segment SGI177787[] = {{&S60839[30], 6}, {NULL}};
const Segment SGI177788[] = {{&S60839[137], 3}, {NULL}};
const Segment SGI177789[] = {{&S60839[102], 6}, {NULL}};
const Segment SGI177790[] = {{&S60839[70], 8}, {NULL}};
const Segment SGI177791[] = {{&S60880[34], 14}, {NULL}};
const Segment SGI177792[] = {{&S60880[27], 6}, {NULL}};
const Segment SGI177793[] = {{&S60880[101], 9}, {NULL}};
const Segment SGI177794[] = {{&S60880[74], 6}, {NULL}};
const Segment SGI177795[] = {{&S60881[34], 14}, {NULL}};
const Segment SGI177796[] = {{&S60881[27], 6}, {NULL}};
const Segment SGI177797[] = {{&S60881[101], 9}, {NULL}};
const Segment SGI177798[] = {{&S60881[74], 6}, {NULL}};
const Segment SGI177799[] = {{&S60882[34], 23}, {NULL}};
const Segment SGI177800[] = {{&S60882[27], 6}, {NULL}};
const Segment SGI177801[] = {{&S60882[110], 9}, {NULL}};
const Segment SGI177802[] = {{&S60882[83], 6}, {NULL}};
const Segment SGI177803[] = {{&S60883[34], 24}, {NULL}};
const Segment SGI177804[] = {{&S60883[27], 6}, {NULL}};
const Segment SGI177805[] = {{&S60883[111], 9}, {NULL}};
const Segment SGI177806[] = {{&S60883[84], 6}, {NULL}};
const Segment SGI177807[] = {{&S57233[34], 10}, {NULL}};
const Segment SGI177808[] = {{&S57233[27], 6}, {NULL}};
const Segment SGI177809[] = {{&S57233[97], 9}, {NULL}};
const Segment SGI177810[] = {{&S57233[70], 6}, {NULL}};
const Segment SGI177811[] = {{&S57235[34], 18}, {NULL}};
const Segment SGI177812[] = {{&S57235[27], 6}, {NULL}};
const Segment SGI177813[] = {{&S57235[105], 9}, {NULL}};
const Segment SGI177814[] = {{&S57235[78], 6}, {NULL}};
const Segment SGI177815[] = {{&S57236[34], 11}, {NULL}};
const Segment SGI177816[] = {{&S57236[27], 6}, {NULL}};
const Segment SGI177817[] = {{&S60892[34], 15}, {NULL}};
const Segment SGI177818[] = {{&S60892[27], 6}, {NULL}};
const Segment SGI177819[] = {{&S57236[98], 9}, {NULL}};
const Segment SGI177820[] = {{&S57236[71], 6}, {NULL}};
const Segment SGI177821[] = {{&S60892[102], 9}, {NULL}};
const Segment SGI177822[] = {{&S60892[75], 6}, {NULL}};
const Segment SGI177823[] = {{&S60893[34], 6}, {NULL}};
const Segment SGI177824[] = {{&S60893[27], 6}, {NULL}};
const Segment SGI177825[] = {{&S60893[93], 9}, {NULL}};
const Segment SGI177826[] = {{&S60893[66], 6}, {NULL}};
const Segment SGI177827[] = {{&S45721[34], 10}, {NULL}};
const Segment SGI177828[] = {{&S45721[27], 6}, {NULL}};
const Segment SGI177829[] = {{&S45721[96], 9}, {NULL}};
const Segment SGI177830[] = {{&S45721[69], 6}, {NULL}};
const Segment SGI177831[] = {{&S45722[34], 10}, {NULL}};
const Segment SGI177832[] = {{&S45722[27], 6}, {NULL}};
const Segment SGI177833[] = {{&S45722[96], 9}, {NULL}};
const Segment SGI177834[] = {{&S45722[69], 6}, {NULL}};
const Segment SGI177835[] = {{&S68427[34], 7}, {NULL}};
const Segment SGI177836[] = {{&S68427[27], 6}, {NULL}};
const Segment SGI177837[] = {{&S68427[100], 9}, {NULL}};
const Segment SGI177838[] = {{&S68427[73], 6}, {NULL}};
const Segment SGI177839[] = {{&S68431[34], 25}, {NULL}};
const Segment SGI177840[] = {{&S68431[27], 6}, {NULL}};
const Segment SGI177841[] = {{&S68431[112], 9}, {NULL}};
const Segment SGI177842[] = {{&S68431[85], 6}, {NULL}};
const Segment SGI177843[] = {{&S68432[34], 25}, {NULL}};
const Segment SGI177844[] = {{&S68432[27], 6}, {NULL}};
const Segment SGI177845[] = {{&S68432[112], 9}, {NULL}};
const Segment SGI177846[] = {{&S68432[85], 6}, {NULL}};
const Segment SGI177847[] = {{&S60916[34], 11}, {NULL}};
const Segment SGI177848[] = {{&S60916[27], 6}, {NULL}};
const Segment SGI177849[] = {{&S60916[98], 9}, {NULL}};
const Segment SGI177850[] = {{&S60916[71], 6}, {NULL}};
const Segment SGI177851[] = {{&S60916[139], 8}, {NULL}};
const Segment SGI177852[] = {{&S68433[34], 25}, {NULL}};
const Segment SGI177853[] = {{&S68433[27], 6}, {NULL}};
const Segment SGI177854[] = {{&S68433[112], 9}, {NULL}};
const Segment SGI177855[] = {{&S68433[85], 6}, {NULL}};
const Segment SGI177856[] = {{&S68441[34], 7}, {NULL}};
const Segment SGI177857[] = {{&S68441[27], 6}, {NULL}};
const Segment SGI177858[] = {{&S68441[111], 9}, {NULL}};
const Segment SGI177859[] = {{&S68441[84], 6}, {NULL}};
const Segment SGI177860[] = {{&S60927[44], 25}, {NULL}};
const Segment SGI177861[] = {{&S60927[44], 37}, {NULL}};
const Segment SGI177862[] = {{&S60927[30], 6}, {NULL}};
const Segment SGI177863[] = {{&S60927[31], 3}, {NULL}};
const Segment SGI177864[] = {{&S60927[95], 6}, {NULL}};
const Segment SGI177865[] = {{&S68456[44], 12}, {NULL}};
const Segment SGI177866[] = {{&S68456[44], 24}, {NULL}};
const Segment SGI177867[] = {{&S68456[30], 6}, {NULL}};
const Segment SGI177868[] = {{&S68456[31], 3}, {NULL}};
const Segment SGI177869[] = {{&S68456[82], 6}, {NULL}};
const Segment SGI177870[] = {{&S60946[44], 10}, {NULL}};
const Segment SGI177871[] = {{&S60946[44], 22}, {NULL}};
const Segment SGI177872[] = {{&S60946[30], 6}, {NULL}};
const Segment SGI177873[] = {{&S60946[115], 3}, {NULL}};
const Segment SGI177874[] = {{&S60946[80], 6}, {NULL}};
const Segment SGI177875[] = {{&S68463[34], 11}, {NULL}};
const Segment SGI177876[] = {{&S68463[27], 6}, {NULL}};
const Segment SGI177877[] = {{&S60947[44], 10}, {NULL}};
const Segment SGI177878[] = {{&S60947[44], 22}, {NULL}};
const Segment SGI177879[] = {{&S60947[30], 6}, {NULL}};
const Segment SGI177880[] = {{&S68463[98], 9}, {NULL}};
const Segment SGI177881[] = {{&S68463[71], 6}, {NULL}};
const Segment SGI177882[] = {{&S60947[115], 3}, {NULL}};
const Segment SGI177883[] = {{&S60947[80], 6}, {NULL}};
const Segment SGI177884[] = {{&S60948[44], 10}, {NULL}};
const Segment SGI177885[] = {{&S60948[44], 22}, {NULL}};
const Segment SGI177886[] = {{&S60948[30], 6}, {NULL}};
const Segment SGI177887[] = {{&S60948[115], 3}, {NULL}};
const Segment SGI177888[] = {{&S60948[80], 6}, {NULL}};
const Segment SGI177889[] = {{&S60949[44], 10}, {NULL}};
const Segment SGI177890[] = {{&S60949[44], 22}, {NULL}};
const Segment SGI177891[] = {{&S60949[30], 6}, {NULL}};
const Segment SGI177892[] = {{&S60949[115], 3}, {NULL}};
const Segment SGI177893[] = {{&S60949[80], 6}, {NULL}};
const Segment SGI177894[] = {{&S60950[44], 10}, {NULL}};
const Segment SGI177895[] = {{&S60950[44], 22}, {NULL}};
const Segment SGI177896[] = {{&S60950[30], 6}, {NULL}};
const Segment SGI177897[] = {{&S60950[115], 3}, {NULL}};
const Segment SGI177898[] = {{&S60950[80], 6}, {NULL}};
const Segment SGI177899[] = {{&S45818[34], 8}, {NULL}};
const Segment SGI177900[] = {{&S45818[27], 6}, {NULL}};
const Segment SGI177901[] = {{&S45818[95], 9}, {NULL}};
const Segment SGI177902[] = {{&S45818[68], 6}, {NULL}};
const Segment SGI177903[] = {{&S68503[45], 15}, {NULL}};
const Segment SGI177904[] = {{&S68503[30], 6}, {NULL}};
const Segment SGI177905[] = {{&S68503[31], 3}, {NULL}};
const Segment SGI177906[] = {{&S68503[86], 5}, {NULL}};
const Segment SGI177907[] = {{&S68503[144], 13}, {NULL}};
const Segment SGI177908[] = {{&S68504[44], 15}, {NULL}};
const Segment SGI177909[] = {{&S68504[44], 27}, {NULL}};
const Segment SGI177910[] = {{&S68504[30], 6}, {NULL}};
const Segment SGI177911[] = {{&S68504[120], 3}, {NULL}};
const Segment SGI177912[] = {{&S68504[85], 6}, {NULL}};
const Segment SGI177913[] = {{&S68504[140], 8}, {NULL}};
const Segment SGI177914[] = {{&S68504[149], 13}, {NULL}};
const Segment SGI177915[] = {{&S60992[34], 10}, {NULL}};
const Segment SGI177916[] = {{&S60992[27], 6}, {NULL}};
const Segment SGI177917[] = {{&S60992[97], 9}, {NULL}};
const Segment SGI177918[] = {{&S60992[70], 6}, {NULL}};
const Segment SGI177919[] = {{&S60992[131], 8}, {NULL}};
const Segment SGI177920[] = {{&S68511[34], 7}, {NULL}};
const Segment SGI177921[] = {{&S68511[27], 6}, {NULL}};
const Segment SGI177922[] = {{&S68511[93], 9}, {NULL}};
const Segment SGI177923[] = {{&S68511[66], 6}, {NULL}};
const Segment SGI177924[] = {{&S68514[34], 7}, {NULL}};
const Segment SGI177925[] = {{&S68514[27], 6}, {NULL}};
const Segment SGI177926[] = {{&S68514[99], 9}, {NULL}};
const Segment SGI177927[] = {{&S68514[72], 6}, {NULL}};
const Segment SGI177928[] = {{&S68516[34], 7}, {NULL}};
const Segment SGI177929[] = {{&S68516[27], 6}, {NULL}};
const Segment SGI177930[] = {{&S68516[99], 9}, {NULL}};
const Segment SGI177931[] = {{&S68516[72], 6}, {NULL}};
const Segment SGI177932[] = {{&S68525[34], 14}, {NULL}};
const Segment SGI177933[] = {{&S68525[27], 6}, {NULL}};
const Segment SGI177934[] = {{&S68525[101], 9}, {NULL}};
const Segment SGI177935[] = {{&S68525[74], 6}, {NULL}};
const Segment SGI177936[] = {{&S68529[34], 7}, {NULL}};
const Segment SGI177937[] = {{&S68529[27], 6}, {NULL}};
const Segment SGI177938[] = {{&S68529[98], 9}, {NULL}};
const Segment SGI177939[] = {{&S68529[71], 6}, {NULL}};
const Segment SGI177940[] = {{&S68530[34], 6}, {NULL}};
const Segment SGI177941[] = {{&S68530[27], 6}, {NULL}};
const Segment SGI177942[] = {{&S68530[93], 9}, {NULL}};
const Segment SGI177943[] = {{&S68530[66], 6}, {NULL}};
const Segment SGI177944[] = {{&S68539[34], 7}, {NULL}};
const Segment SGI177945[] = {{&S68539[27], 6}, {NULL}};
const Segment SGI177946[] = {{&S68539[99], 9}, {NULL}};
const Segment SGI177947[] = {{&S68539[72], 6}, {NULL}};
const Segment SGI177948[] = {{&S68540[44], 10}, {NULL}};
const Segment SGI177949[] = {{&S68540[44], 22}, {NULL}};
const Segment SGI177950[] = {{&S68540[30], 6}, {NULL}};
const Segment SGI177951[] = {{&S68540[139], 4}, {NULL}};
const Segment SGI177952[] = {{&S68540[165], 3}, {NULL}};
const Segment SGI177953[] = {{&S68540[31], 3}, {&S68540[214], 5}, {NULL}};
const Segment SGI177954[] = {{&S68540[80], 6}, {NULL}};
const Segment SGI177955[] = {{&S68541[44], 10}, {NULL}};
const Segment SGI177956[] = {{&S68541[44], 22}, {NULL}};
const Segment SGI177957[] = {{&S68541[30], 6}, {NULL}};
const Segment SGI177958[] = {{&S68541[80], 6}, {NULL}};
const Segment SGI177959[] = {{&S68541[107], 13}, {NULL}};
const Segment SGI177960[] = {{&S68542[34], 18}, {NULL}};
const Segment SGI177961[] = {{&S68542[27], 6}, {NULL}};
const Segment SGI177962[] = {{&S68542[105], 9}, {NULL}};
const Segment SGI177963[] = {{&S68542[78], 6}, {NULL}};
const Segment SGI177964[] = {{&S68544[34], 7}, {NULL}};
const Segment SGI177965[] = {{&S68544[27], 6}, {NULL}};
const Segment SGI177966[] = {{&S68544[98], 9}, {NULL}};
const Segment SGI177967[] = {{&S68544[71], 6}, {NULL}};
const Segment SGI177968[] = {{&S57404[34], 10}, {NULL}};
const Segment SGI177969[] = {{&S57404[27], 6}, {NULL}};
const Segment SGI177970[] = {{&S57404[97], 9}, {NULL}};
const Segment SGI177971[] = {{&S57404[70], 6}, {NULL}};
const Segment SGI177972[] = {{&S57405[34], 18}, {NULL}};
const Segment SGI177973[] = {{&S57405[27], 6}, {NULL}};
const Segment SGI177974[] = {{&S57405[105], 9}, {NULL}};
const Segment SGI177975[] = {{&S57405[78], 6}, {NULL}};
const Segment SGI177976[] = {{&S57440[34], 9}, {NULL}};
const Segment SGI177977[] = {{&S57440[27], 6}, {NULL}};
const Segment SGI177978[] = {{&S57440[96], 9}, {NULL}};
const Segment SGI177979[] = {{&S57440[69], 6}, {NULL}};
const Segment SGI177980[] = {{&S57441[34], 10}, {NULL}};
const Segment SGI177981[] = {{&S57441[27], 6}, {NULL}};
const Segment SGI177982[] = {{&S57441[97], 9}, {NULL}};
const Segment SGI177983[] = {{&S57441[70], 6}, {NULL}};
const Segment SGI177984[] = {{&S61090[34], 12}, {NULL}};
const Segment SGI177985[] = {{&S61090[27], 6}, {NULL}};
const Segment SGI177986[] = {{&S57442[34], 10}, {NULL}};
const Segment SGI177987[] = {{&S57442[27], 6}, {NULL}};
const Segment SGI177988[] = {{&S61090[99], 9}, {NULL}};
const Segment SGI177989[] = {{&S61090[72], 6}, {NULL}};
const Segment SGI177990[] = {{&S57442[97], 9}, {NULL}};
const Segment SGI177991[] = {{&S57442[70], 6}, {NULL}};
const Segment SGI177992[] = {{&S57443[34], 10}, {NULL}};
const Segment SGI177993[] = {{&S57443[27], 6}, {NULL}};
const Segment SGI177994[] = {{&S57443[97], 9}, {NULL}};
const Segment SGI177995[] = {{&S57443[70], 6}, {NULL}};
const Segment SGI177996[] = {{&S61092[34], 13}, {NULL}};
const Segment SGI177997[] = {{&S61092[27], 6}, {NULL}};
const Segment SGI177998[] = {{&S61092[100], 9}, {NULL}};
const Segment SGI177999[] = {{&S61092[73], 6}, {NULL}};
const Segment SGI178000[] = {{&S57446[6], 4}, {NULL}};
const Segment SGI178001[] = {{&S57446[83], 5}, {NULL}};
const Segment SGI178002[] = {{&S68604[34], 13}, {NULL}};
const Segment SGI178003[] = {{&S68604[27], 6}, {NULL}};
const Segment SGI178004[] = {{&S68604[100], 9}, {NULL}};
const Segment SGI178005[] = {{&S68604[73], 6}, {NULL}};
const Segment SGI178006[] = {{&S68617[44], 7}, {NULL}};
const Segment SGI178007[] = {{&S68617[44], 19}, {NULL}};
const Segment SGI178008[] = {{&S68617[30], 6}, {NULL}};
const Segment SGI178009[] = {{&S68617[31], 3}, {NULL}};
const Segment SGI178010[] = {{&S68617[77], 6}, {NULL}};
const Segment SGI178011[] = {{&S57476[34], 9}, {NULL}};
const Segment SGI178012[] = {{&S57476[27], 6}, {NULL}};
const Segment SGI178013[] = {{&S57476[96], 9}, {NULL}};
const Segment SGI178014[] = {{&S57476[69], 6}, {NULL}};
const Segment SGI178015[] = {{&S57483[34], 10}, {NULL}};
const Segment SGI178016[] = {{&S57483[27], 6}, {NULL}};
const Segment SGI178017[] = {{&S57483[97], 9}, {NULL}};
const Segment SGI178018[] = {{&S57483[70], 6}, {NULL}};
const Segment SGI178019[] = {{&S68628[34], 10}, {NULL}};
const Segment SGI178020[] = {{&S68628[27], 6}, {NULL}};
const Segment SGI178021[] = {{&S68628[97], 9}, {NULL}};
const Segment SGI178022[] = {{&S68628[70], 6}, {NULL}};
const Segment SGI178023[] = {{&S57488[34], 10}, {NULL}};
const Segment SGI178024[] = {{&S68629[34], 14}, {NULL}};
const Segment SGI178025[] = {{&S57488[27], 6}, {NULL}};
const Segment SGI178026[] = {{&S68629[27], 6}, {NULL}};
const Segment SGI178027[] = {{&S57488[97], 9}, {NULL}};
const Segment SGI178028[] = {{&S57488[70], 6}, {NULL}};
const Segment SGI178029[] = {{&S68629[101], 9}, {NULL}};
const Segment SGI178030[] = {{&S68629[74], 6}, {NULL}};
const Segment SGI178031[] = {{&S68645[44], 10}, {NULL}};
const Segment SGI178032[] = {{&S68645[44], 22}, {NULL}};
const Segment SGI178033[] = {{&S68645[30], 6}, {NULL}};
const Segment SGI178034[] = {{&S68645[31], 3}, {NULL}};
const Segment SGI178035[] = {{&S68645[95], 6}, {NULL}};
const Segment SGI178036[] = {{&S68645[68], 8}, {NULL}};
const Segment SGI178037[] = {{&S61162[44], 6}, {NULL}};
const Segment SGI178038[] = {{&S61162[44], 18}, {NULL}};
const Segment SGI178039[] = {{&S61162[30], 6}, {NULL}};
const Segment SGI178040[] = {{&S61162[111], 3}, {NULL}};
const Segment SGI178041[] = {{&S61162[76], 6}, {NULL}};
const Segment SGI178042[] = {{&S68646[44], 10}, {NULL}};
const Segment SGI178043[] = {{&S68646[44], 22}, {NULL}};
const Segment SGI178044[] = {{&S68646[30], 6}, {NULL}};
const Segment SGI178045[] = {{&S68646[31], 3}, {NULL}};
const Segment SGI178046[] = {{&S61163[38], 6}, {NULL}};
const Segment SGI178047[] = {{&S68646[99], 6}, {NULL}};
const Segment SGI178048[] = {{&S61163[31], 6}, {NULL}};
const Segment SGI178049[] = {{&S68646[68], 8}, {NULL}};
const Segment SGI178050[] = {{&S46004[34], 10}, {NULL}};
const Segment SGI178051[] = {{&S46004[27], 6}, {NULL}};
const Segment SGI178052[] = {{&S46004[96], 9}, {NULL}};
const Segment SGI178053[] = {{&S46004[69], 6}, {NULL}};
const Segment SGI178054[] = {{&S61168[34], 11}, {NULL}};
const Segment SGI178055[] = {{&S61168[27], 6}, {NULL}};
const Segment SGI178056[] = {{&S61168[98], 9}, {NULL}};
const Segment SGI178057[] = {{&S61168[71], 6}, {NULL}};
const Segment SGI178058[] = {{&S61169[34], 11}, {NULL}};
const Segment SGI178059[] = {{&S61169[27], 6}, {NULL}};
const Segment SGI178060[] = {{&S61169[98], 9}, {NULL}};
const Segment SGI178061[] = {{&S61169[71], 6}, {NULL}};
const Segment SGI178062[] = {{&S61170[34], 11}, {NULL}};
const Segment SGI178063[] = {{&S61170[27], 6}, {NULL}};
const Segment SGI178064[] = {{&S61170[98], 9}, {NULL}};
const Segment SGI178065[] = {{&S61170[71], 6}, {NULL}};
const Segment SGI178066[] = {{&S61170[140], 8}, {NULL}};
const Segment SGI178067[] = {{&S61171[34], 11}, {NULL}};
const Segment SGI178068[] = {{&S61171[27], 6}, {NULL}};
const Segment SGI178069[] = {{&S61171[98], 9}, {NULL}};
const Segment SGI178070[] = {{&S61171[71], 6}, {NULL}};
const Segment SGI178071[] = {{&S61171[139], 8}, {NULL}};
const Segment SGI178072[] = {{&S61172[34], 9}, {NULL}};
const Segment SGI178073[] = {{&S61172[27], 6}, {NULL}};
const Segment SGI178074[] = {{&S61172[96], 9}, {NULL}};
const Segment SGI178075[] = {{&S61172[69], 6}, {NULL}};
const Segment SGI178076[] = {{&S68663[44], 15}, {NULL}};
const Segment SGI178077[] = {{&S68663[44], 27}, {NULL}};
const Segment SGI178078[] = {{&S68663[30], 6}, {NULL}};
const Segment SGI178079[] = {{&S68663[31], 3}, {NULL}};
const Segment SGI178080[] = {{&S68663[85], 6}, {NULL}};
const Segment SGI178081[] = {{&S61176[44], 6}, {NULL}};
const Segment SGI178082[] = {{&S61176[44], 18}, {NULL}};
const Segment SGI178083[] = {{&S61176[30], 6}, {NULL}};
const Segment SGI178084[] = {{&S61176[76], 6}, {NULL}};
const Segment SGI178085[] = {{&S61176[103], 13}, {NULL}};
const Segment SGI178086[] = {{&S68664[38], 15}, {NULL}};
const Segment SGI178087[] = {{&S68664[31], 6}, {NULL}};
const Segment SGI178088[] = {{&S57532[44], 11}, {NULL}};
const Segment SGI178089[] = {{&S61177[44], 7}, {NULL}};
const Segment SGI178090[] = {{&S57532[44], 23}, {NULL}};
const Segment SGI178091[] = {{&S61177[44], 19}, {NULL}};
const Segment SGI178092[] = {{&S57532[30], 6}, {NULL}};
const Segment SGI178093[] = {{&S61177[30], 6}, {NULL}};
const Segment SGI178094[] = {{&S57532[136], 3}, {NULL}};
const Segment SGI178095[] = {{&S61177[77], 6}, {NULL}};
const Segment SGI178096[] = {{&S57532[101], 6}, {NULL}};
const Segment SGI178097[] = {{&S57532[69], 8}, {NULL}};
const Segment SGI178098[] = {{&S61177[104], 13}, {NULL}};
const Segment SGI178099[] = {{&S61178[44], 7}, {NULL}};
const Segment SGI178100[] = {{&S61178[44], 19}, {NULL}};
const Segment SGI178101[] = {{&S61178[30], 6}, {NULL}};
const Segment SGI178102[] = {{&S61178[77], 6}, {NULL}};
const Segment SGI178103[] = {{&S61178[104], 13}, {NULL}};
const Segment SGI178104[] = {{&S61179[44], 8}, {NULL}};
const Segment SGI178105[] = {{&S61179[44], 20}, {NULL}};
const Segment SGI178106[] = {{&S61179[30], 6}, {NULL}};
const Segment SGI178107[] = {{&S61179[78], 6}, {NULL}};
const Segment SGI178108[] = {{&S61179[105], 13}, {NULL}};
const Segment SGI178109[] = {{&S61180[44], 7}, {NULL}};
const Segment SGI178110[] = {{&S61180[44], 19}, {NULL}};
const Segment SGI178111[] = {{&S61180[30], 6}, {NULL}};
const Segment SGI178112[] = {{&S61180[77], 6}, {NULL}};
const Segment SGI178113[] = {{&S61180[104], 13}, {NULL}};
const Segment SGI178114[] = {{&S57543[34], 7}, {NULL}};
const Segment SGI178115[] = {{&S57543[27], 6}, {NULL}};
const Segment SGI178116[] = {{&S57543[99], 9}, {NULL}};
const Segment SGI178117[] = {{&S57543[72], 6}, {NULL}};
const Segment SGI178118[] = {{&S57544[34], 7}, {NULL}};
const Segment SGI178119[] = {{&S57544[27], 6}, {NULL}};
const Segment SGI178120[] = {{&S57544[99], 9}, {NULL}};
const Segment SGI178121[] = {{&S57544[72], 6}, {NULL}};
const Segment SGI178122[] = {{&S68684[34], 10}, {NULL}};
const Segment SGI178123[] = {{&S68684[27], 6}, {NULL}};
const Segment SGI178124[] = {{&S68684[97], 9}, {NULL}};
const Segment SGI178125[] = {{&S68684[70], 6}, {NULL}};
const Segment SGI178126[] = {{&S68687[34], 9}, {NULL}};
const Segment SGI178127[] = {{&S68687[27], 6}, {NULL}};
const Segment SGI178128[] = {{&S68687[96], 9}, {NULL}};
const Segment SGI178129[] = {{&S68687[69], 6}, {NULL}};
const Segment SGI178130[] = {{&S68688[34], 11}, {NULL}};
const Segment SGI178131[] = {{&S68688[27], 6}, {NULL}};
const Segment SGI178132[] = {{&S68688[98], 9}, {NULL}};
const Segment SGI178133[] = {{&S68688[71], 6}, {NULL}};
const Segment SGI178134[] = {{&S57557[34], 7}, {NULL}};
const Segment SGI178135[] = {{&S57557[27], 6}, {NULL}};
const Segment SGI178136[] = {{&S57557[99], 9}, {NULL}};
const Segment SGI178137[] = {{&S57557[72], 6}, {NULL}};
const Segment SGI178138[] = {{&S57558[34], 7}, {NULL}};
const Segment SGI178139[] = {{&S57558[27], 6}, {NULL}};
const Segment SGI178140[] = {{&S57558[99], 9}, {NULL}};
const Segment SGI178141[] = {{&S57558[72], 6}, {NULL}};
const Segment SGI178142[] = {{&S57559[34], 7}, {NULL}};
const Segment SGI178143[] = {{&S57559[27], 6}, {NULL}};
const Segment SGI178144[] = {{&S57559[99], 9}, {NULL}};
const Segment SGI178145[] = {{&S57559[72], 6}, {NULL}};
const Segment SGI178146[] = {{&S68693[44], 22}, {NULL}};
const Segment SGI178147[] = {{&S68693[30], 6}, {NULL}};
const Segment SGI178148[] = {{&S68693[31], 3}, {NULL}};
const Segment SGI178149[] = {{&S68693[94], 6}, {NULL}};
const Segment SGI178150[] = {{&S57571[44], 5}, {NULL}};
const Segment SGI178151[] = {{&S57571[44], 17}, {NULL}};
const Segment SGI178152[] = {{&S57571[30], 6}, {NULL}};
const Segment SGI178153[] = {{&S57571[110], 3}, {NULL}};
const Segment SGI178154[] = {{&S57571[75], 6}, {NULL}};
const Segment SGI178155[] = {{&S57572[38], 5}, {NULL}};
const Segment SGI178156[] = {{&S57572[31], 6}, {NULL}};
const Segment SGI178157[] = {{&S61225[34], 18}, {NULL}};
const Segment SGI178158[] = {{&S61225[27], 6}, {NULL}};
const Segment SGI178159[] = {{&S61225[114], 9}, {NULL}};
const Segment SGI178160[] = {{&S61225[87], 6}, {NULL}};
const Segment SGI178161[] = {{&S57574[34], 11}, {NULL}};
const Segment SGI178162[] = {{&S57574[27], 6}, {NULL}};
const Segment SGI178163[] = {{&S57574[98], 9}, {NULL}};
const Segment SGI178164[] = {{&S57574[71], 6}, {NULL}};
const Segment SGI178165[] = {{&S61230[44], 10}, {NULL}};
const Segment SGI178166[] = {{&S61230[44], 22}, {NULL}};
const Segment SGI178167[] = {{&S61230[30], 6}, {NULL}};
const Segment SGI178168[] = {{&S61230[131], 3}, {NULL}};
const Segment SGI178169[] = {{&S61230[96], 6}, {NULL}};
const Segment SGI178170[] = {{&S61230[68], 8}, {NULL}};
const Segment SGI178171[] = {{&S61231[44], 10}, {NULL}};
const Segment SGI178172[] = {{&S61231[44], 22}, {NULL}};
const Segment SGI178173[] = {{&S61231[30], 6}, {NULL}};
const Segment SGI178174[] = {{&S61231[131], 3}, {NULL}};
const Segment SGI178175[] = {{&S61231[96], 6}, {NULL}};
const Segment SGI178176[] = {{&S61231[68], 8}, {NULL}};
const Segment SGI178177[] = {{&S57581[34], 7}, {NULL}};
const Segment SGI178178[] = {{&S57581[27], 6}, {NULL}};
const Segment SGI178179[] = {{&S57581[94], 9}, {NULL}};
const Segment SGI178180[] = {{&S68717[44], 10}, {NULL}};
const Segment SGI178181[] = {{&S57581[67], 6}, {NULL}};
const Segment SGI178182[] = {{&S68717[44], 22}, {NULL}};
const Segment SGI178183[] = {{&S68717[30], 6}, {NULL}};
const Segment SGI178184[] = {{&S61232[44], 10}, {NULL}};
const Segment SGI178185[] = {{&S61232[44], 22}, {NULL}};
const Segment SGI178186[] = {{&S68717[31], 3}, {NULL}};
const Segment SGI178187[] = {{&S61232[30], 6}, {NULL}};
const Segment SGI178188[] = {{&S68717[80], 6}, {NULL}};
const Segment SGI178189[] = {{&S61232[131], 3}, {NULL}};
const Segment SGI178190[] = {{&S61232[96], 6}, {NULL}};
const Segment SGI178191[] = {{&S61232[68], 8}, {NULL}};
const Segment SGI178192[] = {{&S68718[44], 5}, {NULL}};
const Segment SGI178193[] = {{&S68718[44], 21}, {NULL}};
const Segment SGI178194[] = {{&S68718[30], 6}, {NULL}};
const Segment SGI178195[] = {{&S68718[31], 3}, {NULL}};
const Segment SGI178196[] = {{&S68718[79], 6}, {NULL}};
const Segment SGI178197[] = {{&S68719[34], 14}, {NULL}};
const Segment SGI178198[] = {{&S68719[27], 6}, {NULL}};
const Segment SGI178199[] = {{&S61235[44], 9}, {NULL}};
const Segment SGI178200[] = {{&S68719[101], 9}, {NULL}};
const Segment SGI178201[] = {{&S61235[44], 21}, {NULL}};
const Segment SGI178202[] = {{&S68719[74], 6}, {NULL}};
const Segment SGI178203[] = {{&S61235[30], 6}, {NULL}};
const Segment SGI178204[] = {{&S61235[114], 3}, {NULL}};
const Segment SGI178205[] = {{&S61235[79], 6}, {NULL}};
const Segment SGI178206[] = {{&S57589[34], 11}, {NULL}};
const Segment SGI178207[] = {{&S57589[27], 6}, {NULL}};
const Segment SGI178208[] = {{&S61236[44], 11}, {NULL}};
const Segment SGI178209[] = {{&S61236[44], 23}, {NULL}};
const Segment SGI178210[] = {{&S61236[30], 6}, {NULL}};
const Segment SGI178211[] = {{&S57589[98], 9}, {NULL}};
const Segment SGI178212[] = {{&S57589[71], 6}, {NULL}};
const Segment SGI178213[] = {{&S61236[116], 3}, {NULL}};
const Segment SGI178214[] = {{&S61236[81], 6}, {NULL}};
const Segment SGI178215[] = {{&S57590[34], 10}, {NULL}};
const Segment SGI178216[] = {{&S57590[27], 6}, {NULL}};
const Segment SGI178217[] = {{&S57590[97], 9}, {NULL}};
const Segment SGI178218[] = {{&S57590[70], 6}, {NULL}};
const Segment SGI178219[] = {{&S57591[34], 11}, {NULL}};
const Segment SGI178220[] = {{&S57591[27], 6}, {NULL}};
const Segment SGI178221[] = {{&S57591[98], 9}, {NULL}};
const Segment SGI178222[] = {{&S57591[71], 6}, {NULL}};
const Segment SGI178223[] = {{&S68730[44], 10}, {NULL}};
const Segment SGI178224[] = {{&S68730[44], 19}, {NULL}};
const Segment SGI178225[] = {{&S68730[30], 6}, {NULL}};
const Segment SGI178226[] = {{&S68730[31], 3}, {NULL}};
const Segment SGI178227[] = {{&S57597[44], 12}, {NULL}};
const Segment SGI178228[] = {{&S57597[44], 24}, {NULL}};
const Segment SGI178229[] = {{&S68730[77], 6}, {NULL}};
const Segment SGI178230[] = {{&S57597[30], 6}, {NULL}};
const Segment SGI178231[] = {{&S57597[117], 3}, {NULL}};
const Segment SGI178232[] = {{&S57597[82], 6}, {NULL}};
const Segment SGI178233[] = {{&S68731[44], 10}, {NULL}};
const Segment SGI178234[] = {{&S68731[44], 22}, {NULL}};
const Segment SGI178235[] = {{&S68731[30], 6}, {NULL}};
const Segment SGI178236[] = {{&S68731[31], 3}, {NULL}};
const Segment SGI178237[] = {{&S57598[44], 10}, {NULL}};
const Segment SGI178238[] = {{&S57598[44], 22}, {NULL}};
const Segment SGI178239[] = {{&S68731[80], 6}, {NULL}};
const Segment SGI178240[] = {{&S57598[30], 6}, {NULL}};
const Segment SGI178241[] = {{&S57598[115], 3}, {NULL}};
const Segment SGI178242[] = {{&S57598[80], 6}, {NULL}};
const Segment SGI178243[] = {{&S68732[38], 10}, {NULL}};
const Segment SGI178244[] = {{&S68732[31], 6}, {NULL}};
const Segment SGI178245[] = {{&S57599[38], 10}, {NULL}};
const Segment SGI178246[] = {{&S57599[31], 6}, {NULL}};
const Segment SGI178247[] = {{&S57603[34], 7}, {NULL}};
const Segment SGI178248[] = {{&S57603[27], 6}, {NULL}};
const Segment SGI178249[] = {{&S57603[100], 9}, {NULL}};
const Segment SGI178250[] = {{&S57603[73], 6}, {NULL}};
const Segment SGI178251[] = {{&S57604[34], 18}, {NULL}};
const Segment SGI178252[] = {{&S57604[27], 6}, {NULL}};
const Segment SGI178253[] = {{&S57604[105], 9}, {NULL}};
const Segment SGI178254[] = {{&S57604[78], 6}, {NULL}};
const Segment SGI178255[] = {{&S57616[44], 11}, {NULL}};
const Segment SGI178256[] = {{&S57616[44], 23}, {NULL}};
const Segment SGI178257[] = {{&S57616[30], 6}, {NULL}};
const Segment SGI178258[] = {{&S57616[116], 3}, {NULL}};
const Segment SGI178259[] = {{&S57616[81], 6}, {NULL}};
const Segment SGI178260[] = {{&S57617[38], 11}, {NULL}};
const Segment SGI178261[] = {{&S57617[31], 6}, {NULL}};
const Segment SGI178262[] = {{&S68756[34], 16}, {NULL}};
const Segment SGI178263[] = {{&S68756[27], 6}, {NULL}};
const Segment SGI178264[] = {{&S68756[110], 9}, {NULL}};
const Segment SGI178265[] = {{&S68756[83], 6}, {NULL}};
const Segment SGI178266[] = {{&S68757[34], 18}, {NULL}};
const Segment SGI178267[] = {{&S68757[27], 6}, {NULL}};
const Segment SGI178268[] = {{&S68757[114], 9}, {NULL}};
const Segment SGI178269[] = {{&S68757[87], 6}, {NULL}};
const Segment SGI178270[] = {{&S61284[34], 15}, {NULL}};
const Segment SGI178271[] = {{&S61284[27], 6}, {NULL}};
const Segment SGI178272[] = {{&S61284[102], 9}, {NULL}};
const Segment SGI178273[] = {{&S61284[75], 6}, {NULL}};
const Segment SGI178274[] = {{&S68758[34], 17}, {NULL}};
const Segment SGI178275[] = {{&S68758[27], 6}, {NULL}};
const Segment SGI178276[] = {{&S68758[112], 9}, {NULL}};
const Segment SGI178277[] = {{&S68758[85], 6}, {NULL}};
const Segment SGI178278[] = {{&S68759[34], 17}, {NULL}};
const Segment SGI178279[] = {{&S61286[34], 6}, {NULL}};
const Segment SGI178280[] = {{&S61286[27], 6}, {NULL}};
const Segment SGI178281[] = {{&S68759[27], 6}, {NULL}};
const Segment SGI178282[] = {{&S61286[93], 9}, {NULL}};
const Segment SGI178283[] = {{&S68759[112], 9}, {NULL}};
const Segment SGI178284[] = {{&S61286[66], 6}, {NULL}};
const Segment SGI178285[] = {{&S68759[85], 6}, {NULL}};
const Segment SGI178286[] = {{&S68760[34], 15}, {NULL}};
const Segment SGI178287[] = {{&S61287[27], 6}, {NULL}};
const Segment SGI178288[] = {{&S68760[27], 6}, {NULL}};
const Segment SGI178289[] = {{&S61287[113], 9}, {NULL}};
const Segment SGI178290[] = {{&S68760[108], 9}, {NULL}};
const Segment SGI178291[] = {{&S68760[81], 6}, {NULL}};
const Segment SGI178292[] = {{&S61287[86], 6}, {NULL}};
const Segment SGI178293[] = {{&S68787[34], 12}, {NULL}};
const Segment SGI178294[] = {{&S68787[27], 6}, {NULL}};
const Segment SGI178295[] = {{&S68787[115], 9}, {NULL}};
const Segment SGI178296[] = {{&S68787[88], 6}, {NULL}};
const Segment SGI178297[] = {{&S57656[34], 9}, {NULL}};
const Segment SGI178298[] = {{&S57656[27], 6}, {NULL}};
const Segment SGI178299[] = {{&S57656[96], 9}, {NULL}};
const Segment SGI178300[] = {{&S57656[69], 6}, {NULL}};
const Segment SGI178301[] = {{&S57659[34], 9}, {NULL}};
const Segment SGI178302[] = {{&S57659[27], 6}, {NULL}};
const Segment SGI178303[] = {{&S57659[96], 9}, {NULL}};
const Segment SGI178304[] = {{&S57659[69], 6}, {NULL}};
const Segment SGI178305[] = {{&S68803[44], 8}, {NULL}};
const Segment SGI178306[] = {{&S68803[44], 25}, {NULL}};
const Segment SGI178307[] = {{&S68803[30], 6}, {NULL}};
const Segment SGI178308[] = {{&S68803[31], 3}, {NULL}};
const Segment SGI178309[] = {{&S68803[83], 6}, {NULL}};
const Segment SGI178310[] = {{&S68804[44], 8}, {NULL}};
const Segment SGI178311[] = {{&S68804[44], 25}, {NULL}};
const Segment SGI178312[] = {{&S68804[30], 6}, {NULL}};
const Segment SGI178313[] = {{&S68804[31], 3}, {NULL}};
const Segment SGI178314[] = {{&S68804[83], 6}, {NULL}};
const Segment SGI178315[] = {{&S55242[34], 11}, {NULL}};
const Segment SGI178316[] = {{&S55242[27], 6}, {NULL}};
const Segment SGI178317[] = {{&S55242[98], 9}, {NULL}};
const Segment SGI178318[] = {{&S55242[71], 6}, {NULL}};
const Segment SGI178319[] = {{&S61385[34], 18}, {NULL}};
const Segment SGI178320[] = {{&S61385[27], 6}, {NULL}};
const Segment SGI178321[] = {{&S61385[105], 9}, {NULL}};
const Segment SGI178322[] = {{&S61385[78], 6}, {NULL}};
const Segment SGI178323[] = {{&S61392[44], 25}, {NULL}};
const Segment SGI178324[] = {{&S61392[44], 37}, {NULL}};
const Segment SGI178325[] = {{&S61392[30], 6}, {NULL}};
const Segment SGI178326[] = {{&S61392[31], 3}, {NULL}};
const Segment SGI178327[] = {{&S61392[95], 6}, {NULL}};
const Segment SGI178328[] = {{&S61393[38], 25}, {NULL}};
const Segment SGI178329[] = {{&S61393[31], 6}, {NULL}};
const Segment SGI178330[] = {{&S57726[44], 10}, {NULL}};
const Segment SGI178331[] = {{&S57726[44], 22}, {NULL}};
const Segment SGI178332[] = {{&S57726[30], 6}, {NULL}};
const Segment SGI178333[] = {{&S57726[115], 3}, {NULL}};
const Segment SGI178334[] = {{&S57726[80], 6}, {NULL}};
const Segment SGI178335[] = {{&S57727[38], 10}, {NULL}};
const Segment SGI178336[] = {{&S57727[31], 6}, {NULL}};
const Segment SGI178337[] = {{&S46273[34], 10}, {NULL}};
const Segment SGI178338[] = {{&S46273[27], 6}, {NULL}};
const Segment SGI178339[] = {{&S46273[96], 9}, {NULL}};
const Segment SGI178340[] = {{&S46273[69], 6}, {NULL}};
const Segment SGI178341[] = {{&S46274[34], 11}, {NULL}};
const Segment SGI178342[] = {{&S46274[27], 6}, {NULL}};
const Segment SGI178343[] = {{&S46274[97], 9}, {NULL}};
const Segment SGI178344[] = {{&S46274[70], 6}, {NULL}};
const Segment SGI178345[] = {{&S57749[44], 7}, {NULL}};
const Segment SGI178346[] = {{&S57749[44], 19}, {NULL}};
const Segment SGI178347[] = {{&S57749[30], 6}, {NULL}};
const Segment SGI178348[] = {{&S57749[112], 3}, {NULL}};
const Segment SGI178349[] = {{&S57749[77], 6}, {NULL}};
const Segment SGI178350[] = {{&S46275[34], 11}, {NULL}};
const Segment SGI178351[] = {{&S46275[27], 6}, {NULL}};
const Segment SGI178352[] = {{&S46275[97], 9}, {NULL}};
const Segment SGI178353[] = {{&S46275[70], 6}, {NULL}};
const Segment SGI178354[] = {{&S57750[38], 7}, {NULL}};
const Segment SGI178355[] = {{&S57750[31], 6}, {NULL}};
const Segment SGI178356[] = {{&S57751[44], 7}, {NULL}};
const Segment SGI178357[] = {{&S57751[44], 19}, {NULL}};
const Segment SGI178358[] = {{&S57751[30], 6}, {NULL}};
const Segment SGI178359[] = {{&S57751[112], 3}, {NULL}};
const Segment SGI178360[] = {{&S57751[77], 6}, {NULL}};
const Segment SGI178361[] = {{&S66939[34], 7}, {NULL}};
const Segment SGI178362[] = {{&S66939[27], 6}, {NULL}};
const Segment SGI178363[] = {{&S66939[67], 6}, {NULL}};
const Segment SGI178364[] = {{&S66942[34], 7}, {NULL}};
const Segment SGI178365[] = {{&S66942[27], 6}, {NULL}};
const Segment SGI178366[] = {{&S66942[67], 6}, {NULL}};
const Segment SGI178367[] = {{&S55332[44], 6}, {NULL}};
const Segment SGI178368[] = {{&S55332[44], 18}, {NULL}};
const Segment SGI178369[] = {{&S55332[30], 6}, {NULL}};
const Segment SGI178370[] = {{&S55332[111], 3}, {NULL}};
const Segment SGI178371[] = {{&S55332[76], 6}, {NULL}};
const Segment SGI178372[] = {{&S55333[38], 6}, {NULL}};
const Segment SGI178373[] = {{&S55333[31], 6}, {NULL}};
const Segment SGI178374[] = {{&S61458[44], 11}, {NULL}};
const Segment SGI178375[] = {{&S61458[44], 30}, {NULL}};
const Segment SGI178376[] = {{&S61458[30], 6}, {NULL}};
const Segment SGI178377[] = {{&S61458[123], 3}, {NULL}};
const Segment SGI178378[] = {{&S61458[88], 6}, {NULL}};
const Segment SGI178379[] = {{&S61459[44], 11}, {NULL}};
const Segment SGI178380[] = {{&S61459[44], 30}, {NULL}};
const Segment SGI178381[] = {{&S61459[30], 6}, {NULL}};
const Segment SGI178382[] = {{&S61459[123], 3}, {NULL}};
const Segment SGI178383[] = {{&S61459[88], 6}, {NULL}};
const Segment SGI178384[] = {{&S61460[38], 11}, {NULL}};
const Segment SGI178385[] = {{&S61460[31], 6}, {NULL}};
const Segment SGI178386[] = {{&S61461[44], 13}, {NULL}};
const Segment SGI178387[] = {{&S61461[44], 32}, {NULL}};
const Segment SGI178388[] = {{&S61461[30], 6}, {NULL}};
const Segment SGI178389[] = {{&S61461[125], 3}, {NULL}};
const Segment SGI178390[] = {{&S61461[90], 6}, {NULL}};
const Segment SGI178391[] = {{&S61462[44], 13}, {NULL}};
const Segment SGI178392[] = {{&S61462[44], 32}, {NULL}};
const Segment SGI178393[] = {{&S61462[30], 6}, {NULL}};
const Segment SGI178394[] = {{&S61462[125], 3}, {NULL}};
const Segment SGI178395[] = {{&S61462[90], 6}, {NULL}};
const Segment SGI178396[] = {{&S61463[38], 13}, {NULL}};
const Segment SGI178397[] = {{&S61463[31], 6}, {NULL}};
const Segment SGI178398[] = {{&S61475[34], 11}, {NULL}};
const Segment SGI178399[] = {{&S61475[27], 6}, {NULL}};
const Segment SGI178400[] = {{&S61475[98], 9}, {NULL}};
const Segment SGI178401[] = {{&S61475[71], 6}, {NULL}};
const Segment SGI178402[] = {{&S61478[34], 11}, {NULL}};
const Segment SGI178403[] = {{&S61478[27], 6}, {NULL}};
const Segment SGI178404[] = {{&S61478[98], 9}, {NULL}};
const Segment SGI178405[] = {{&S61478[71], 6}, {NULL}};
const Segment SGI178406[] = {{&S68806[44], 9}, {NULL}};
const Segment SGI178407[] = {{&S68806[44], 28}, {NULL}};
const Segment SGI178408[] = {{&S68806[30], 6}, {NULL}};
const Segment SGI178409[] = {{&S68806[31], 3}, {NULL}};
const Segment SGI178410[] = {{&S68806[86], 6}, {NULL}};
const Segment SGI178411[] = {{&S68807[38], 9}, {NULL}};
const Segment SGI178412[] = {{&S68807[31], 6}, {NULL}};
const Segment SGI178413[] = {{&S68813[34], 14}, {NULL}};
const Segment SGI178414[] = {{&S68813[27], 6}, {NULL}};
const Segment SGI178415[] = {{&S68813[101], 9}, {NULL}};
const Segment SGI178416[] = {{&S68813[74], 6}, {NULL}};
const Segment SGI178417[] = {{&S46356[34], 18}, {NULL}};
const Segment SGI178418[] = {{&S46356[27], 6}, {NULL}};
const Segment SGI178419[] = {{&S46356[104], 9}, {NULL}};
const Segment SGI178420[] = {{&S68814[34], 14}, {NULL}};
const Segment SGI178421[] = {{&S46356[77], 6}, {NULL}};
const Segment SGI178422[] = {{&S68814[27], 6}, {NULL}};
const Segment SGI178423[] = {{&S68814[101], 9}, {NULL}};
const Segment SGI178424[] = {{&S68814[74], 6}, {NULL}};
const Segment SGI178425[] = {{&S46357[34], 10}, {NULL}};
const Segment SGI178426[] = {{&S46357[27], 6}, {NULL}};
const Segment SGI178427[] = {{&S46357[96], 9}, {NULL}};
const Segment SGI178428[] = {{&S46357[69], 6}, {NULL}};
const Segment SGI178429[] = {{&S46366[34], 10}, {NULL}};
const Segment SGI178430[] = {{&S46366[27], 6}, {NULL}};
const Segment SGI178431[] = {{&S46366[96], 9}, {NULL}};
const Segment SGI178432[] = {{&S46366[69], 6}, {NULL}};
const Segment SGI178433[] = {{&S46367[34], 11}, {NULL}};
const Segment SGI178434[] = {{&S46367[27], 6}, {NULL}};
const Segment SGI178435[] = {{&S46367[97], 9}, {NULL}};
const Segment SGI178436[] = {{&S46367[70], 6}, {NULL}};
const Segment SGI178437[] = {{&S46368[34], 11}, {NULL}};
const Segment SGI178438[] = {{&S46368[27], 6}, {NULL}};
const Segment SGI178439[] = {{&S46368[97], 9}, {NULL}};
const Segment SGI178440[] = {{&S46368[70], 6}, {NULL}};
const Segment SGI178441[] = {{&S61516[44], 15}, {NULL}};
const Segment SGI178442[] = {{&S61516[44], 27}, {NULL}};
const Segment SGI178443[] = {{&S61516[30], 6}, {NULL}};
const Segment SGI178444[] = {{&S61516[120], 3}, {NULL}};
const Segment SGI178445[] = {{&S61516[85], 6}, {NULL}};
const Segment SGI178446[] = {{&S61517[44], 16}, {NULL}};
const Segment SGI178447[] = {{&S61517[44], 28}, {NULL}};
const Segment SGI178448[] = {{&S61517[30], 6}, {NULL}};
const Segment SGI178449[] = {{&S61517[121], 3}, {NULL}};
const Segment SGI178450[] = {{&S61517[86], 6}, {NULL}};
const Segment SGI178451[] = {{&S61518[38], 15}, {NULL}};
const Segment SGI178452[] = {{&S61518[31], 6}, {NULL}};
const Segment SGI178453[] = {{&S68836[44], 14}, {NULL}};
const Segment SGI178454[] = {{&S68836[44], 26}, {NULL}};
const Segment SGI178455[] = {{&S68836[30], 6}, {NULL}};
const Segment SGI178456[] = {{&S68836[31], 3}, {NULL}};
const Segment SGI178457[] = {{&S68836[84], 6}, {NULL}};
const Segment SGI178458[] = {{&S61520[34], 9}, {NULL}};
const Segment SGI178459[] = {{&S61520[27], 6}, {NULL}};
const Segment SGI178460[] = {{&S68837[38], 14}, {NULL}};
const Segment SGI178461[] = {{&S68837[31], 6}, {NULL}};
const Segment SGI178462[] = {{&S61520[96], 9}, {NULL}};
const Segment SGI178463[] = {{&S61520[69], 6}, {NULL}};
const Segment SGI178464[] = {{&S68843[34], 7}, {NULL}};
const Segment SGI178465[] = {{&S68843[27], 6}, {NULL}};
const Segment SGI178466[] = {{&S68843[103], 9}, {NULL}};
const Segment SGI178467[] = {{&S68843[76], 6}, {NULL}};
const Segment SGI178468[] = {{&S46388[44], 10}, {NULL}};
const Segment SGI178469[] = {{&S46388[44], 21}, {NULL}};
const Segment SGI178470[] = {{&S46388[30], 6}, {NULL}};
const Segment SGI178471[] = {{&S68844[34], 7}, {NULL}};
const Segment SGI178472[] = {{&S68844[27], 6}, {NULL}};
const Segment SGI178473[] = {{&S46388[114], 3}, {NULL}};
const Segment SGI178474[] = {{&S46388[79], 6}, {NULL}};
const Segment SGI178475[] = {{&S68844[110], 9}, {NULL}};
const Segment SGI178476[] = {{&S68844[83], 6}, {NULL}};
const Segment SGI178477[] = {{&S68845[34], 10}, {NULL}};
const Segment SGI178478[] = {{&S68845[27], 6}, {NULL}};
const Segment SGI178479[] = {{&S46390[38], 10}, {NULL}};
const Segment SGI178480[] = {{&S46390[31], 6}, {NULL}};
const Segment SGI178481[] = {{&S68845[97], 9}, {NULL}};
const Segment SGI178482[] = {{&S68845[70], 6}, {NULL}};
const Segment SGI178483[] = {{&S61532[44], 10}, {NULL}};
const Segment SGI178484[] = {{&S61532[44], 22}, {NULL}};
const Segment SGI178485[] = {{&S46391[44], 10}, {NULL}};
const Segment SGI178486[] = {{&S61532[30], 6}, {NULL}};
const Segment SGI178487[] = {{&S46391[44], 22}, {NULL}};
const Segment SGI178488[] = {{&S46391[30], 6}, {NULL}};
const Segment SGI178489[] = {{&S61532[115], 3}, {NULL}};
const Segment SGI178490[] = {{&S46391[115], 3}, {NULL}};
const Segment SGI178491[] = {{&S61532[80], 6}, {NULL}};
const Segment SGI178492[] = {{&S46391[80], 6}, {NULL}};
const Segment SGI178493[] = {{&S61532[140], 11}, {NULL}};
const Segment SGI178494[] = {{&S68848[34], 10}, {NULL}};
const Segment SGI178495[] = {{&S68848[27], 6}, {NULL}};
const Segment SGI178496[] = {{&S68848[97], 9}, {NULL}};
const Segment SGI178497[] = {{&S68848[70], 6}, {NULL}};
const Segment SGI178498[] = {{&S61534[44], 18}, {NULL}};
const Segment SGI178499[] = {{&S61534[44], 30}, {NULL}};
const Segment SGI178500[] = {{&S61534[30], 6}, {NULL}};
const Segment SGI178501[] = {{&S61534[123], 3}, {NULL}};
const Segment SGI178502[] = {{&S61534[88], 6}, {NULL}};
const Segment SGI178503[] = {{&S61534[148], 11}, {NULL}};
const Segment SGI178504[] = {{&S68849[34], 10}, {NULL}};
const Segment SGI178505[] = {{&S68849[27], 6}, {NULL}};
const Segment SGI178506[] = {{&S68849[97], 9}, {NULL}};
const Segment SGI178507[] = {{&S68849[70], 6}, {NULL}};
const Segment SGI178508[] = {{&S61536[44], 11}, {NULL}};
const Segment SGI178509[] = {{&S68850[34], 10}, {NULL}};
const Segment SGI178510[] = {{&S61536[44], 23}, {NULL}};
const Segment SGI178511[] = {{&S68850[27], 6}, {NULL}};
const Segment SGI178512[] = {{&S61536[30], 6}, {NULL}};
const Segment SGI178513[] = {{&S61536[116], 3}, {NULL}};
const Segment SGI178514[] = {{&S68850[97], 9}, {NULL}};
const Segment SGI178515[] = {{&S68850[70], 6}, {NULL}};
const Segment SGI178516[] = {{&S61536[81], 6}, {NULL}};
const Segment SGI178517[] = {{&S61536[141], 11}, {NULL}};
const Segment SGI178518[] = {{&S61563[44], 17}, {NULL}};
const Segment SGI178519[] = {{&S61563[44], 29}, {NULL}};
const Segment SGI178520[] = {{&S61563[30], 6}, {NULL}};
const Segment SGI178521[] = {{&S61563[122], 3}, {NULL}};
const Segment SGI178522[] = {{&S61563[87], 6}, {NULL}};
const Segment SGI178523[] = {{&S61564[38], 17}, {NULL}};
const Segment SGI178524[] = {{&S61564[31], 6}, {NULL}};
const Segment SGI178525[] = {{&S61565[34], 5}, {NULL}};
const Segment SGI178526[] = {{&S61565[27], 6}, {NULL}};
const Segment SGI178527[] = {{&S61565[92], 9}, {NULL}};
const Segment SGI178528[] = {{&S61565[65], 6}, {NULL}};
const Segment SGI178529[] = {{&S68885[44], 29}, {NULL}};
const Segment SGI178530[] = {{&S68885[44], 41}, {NULL}};
const Segment SGI178531[] = {{&S68885[30], 6}, {NULL}};
const Segment SGI178532[] = {{&S68885[31], 3}, {NULL}};
const Segment SGI178533[] = {{&S68885[99], 6}, {NULL}};
const Segment SGI178534[] = {{&S68886[44], 24}, {NULL}};
const Segment SGI178535[] = {{&S68886[44], 36}, {NULL}};
const Segment SGI178536[] = {{&S68886[30], 6}, {NULL}};
const Segment SGI178537[] = {{&S68886[31], 3}, {NULL}};
const Segment SGI178538[] = {{&S68886[94], 6}, {NULL}};
const Segment SGI178539[] = {{&S46468[34], 11}, {NULL}};
const Segment SGI178540[] = {{&S46468[27], 6}, {NULL}};
const Segment SGI178541[] = {{&S46468[97], 9}, {NULL}};
const Segment SGI178542[] = {{&S46468[70], 6}, {NULL}};
const Segment SGI178543[] = {{&S46469[34], 7}, {NULL}};
const Segment SGI178544[] = {{&S46469[27], 6}, {NULL}};
const Segment SGI178545[] = {{&S46469[94], 9}, {NULL}};
const Segment SGI178546[] = {{&S46469[67], 6}, {NULL}};
const Segment SGI178547[] = {{&S68921[34], 9}, {NULL}};
const Segment SGI178548[] = {{&S68921[27], 6}, {NULL}};
const Segment SGI178549[] = {{&S68921[96], 9}, {NULL}};
const Segment SGI178550[] = {{&S68921[69], 6}, {NULL}};
const Segment SGI178551[] = {{&S68922[34], 9}, {NULL}};
const Segment SGI178552[] = {{&S68922[27], 6}, {NULL}};
const Segment SGI178553[] = {{&S68922[96], 9}, {NULL}};
const Segment SGI178554[] = {{&S68922[69], 6}, {NULL}};
const Segment SGI178555[] = {{&S68923[34], 9}, {NULL}};
const Segment SGI178556[] = {{&S68923[27], 6}, {NULL}};
const Segment SGI178557[] = {{&S68923[96], 9}, {NULL}};
const Segment SGI178558[] = {{&S68923[69], 6}, {NULL}};
const Segment SGI178559[] = {{&S68924[34], 8}, {NULL}};
const Segment SGI178560[] = {{&S68924[27], 6}, {NULL}};
const Segment SGI178561[] = {{&S68924[95], 9}, {NULL}};
const Segment SGI178562[] = {{&S68924[68], 6}, {NULL}};
const Segment SGI178563[] = {{&S57838[34], 10}, {NULL}};
const Segment SGI178564[] = {{&S57838[27], 6}, {NULL}};
const Segment SGI178565[] = {{&S57838[97], 9}, {NULL}};
const Segment SGI178566[] = {{&S57838[70], 6}, {NULL}};
const Segment SGI178567[] = {{&S68943[44], 8}, {NULL}};
const Segment SGI178568[] = {{&S68943[44], 33}, {NULL}};
const Segment SGI178569[] = {{&S68943[30], 6}, {NULL}};
const Segment SGI178570[] = {{&S68943[31], 3}, {NULL}};
const Segment SGI178571[] = {{&S68943[91], 6}, {NULL}};
const Segment SGI178572[] = {{&S68944[44], 8}, {NULL}};
const Segment SGI178573[] = {{&S68944[44], 33}, {NULL}};
const Segment SGI178574[] = {{&S68944[30], 6}, {NULL}};
const Segment SGI178575[] = {{&S68944[31], 3}, {NULL}};
const Segment SGI178576[] = {{&S68944[91], 6}, {NULL}};
const Segment SGI178577[] = {{&S68945[44], 8}, {NULL}};
const Segment SGI178578[] = {{&S68945[44], 33}, {NULL}};
const Segment SGI178579[] = {{&S68945[30], 6}, {NULL}};
const Segment SGI178580[] = {{&S68945[31], 3}, {NULL}};
const Segment SGI178581[] = {{&S68945[91], 6}, {NULL}};
const Segment SGI178582[] = {{&S68946[44], 8}, {NULL}};
const Segment SGI178583[] = {{&S68946[44], 33}, {NULL}};
const Segment SGI178584[] = {{&S68946[30], 6}, {NULL}};
const Segment SGI178585[] = {{&S68946[31], 3}, {NULL}};
const Segment SGI178586[] = {{&S68946[91], 6}, {NULL}};
const Segment SGI178587[] = {{&S68947[44], 8}, {NULL}};
const Segment SGI178588[] = {{&S68947[44], 33}, {NULL}};
const Segment SGI178589[] = {{&S68947[30], 6}, {NULL}};
const Segment SGI178590[] = {{&S68947[31], 3}, {NULL}};
const Segment SGI178591[] = {{&S68947[91], 6}, {NULL}};
const Segment SGI178592[] = {{&S68950[34], 15}, {NULL}};
const Segment SGI178593[] = {{&S68950[27], 6}, {NULL}};
const Segment SGI178594[] = {{&S68950[102], 9}, {NULL}};
const Segment SGI178595[] = {{&S68950[75], 6}, {NULL}};
const Segment SGI178596[] = {{&S57883[34], 8}, {NULL}};
const Segment SGI178597[] = {{&S57883[27], 6}, {NULL}};
const Segment SGI178598[] = {{&S57883[103], 9}, {NULL}};
const Segment SGI178599[] = {{&S57883[76], 6}, {NULL}};
const Segment SGI178600[] = {{&S69012[34], 10}, {NULL}};
const Segment SGI178601[] = {{&S69012[27], 6}, {NULL}};
const Segment SGI178602[] = {{&S69012[97], 9}, {NULL}};
const Segment SGI178603[] = {{&S69012[70], 6}, {NULL}};
const Segment SGI178604[] = {{&S69028[44], 22}, {NULL}};
const Segment SGI178605[] = {{&S69028[30], 6}, {NULL}};
const Segment SGI178606[] = {{&S69028[31], 3}, {NULL}};
const Segment SGI178607[] = {{&S69028[94], 6}, {NULL}};
const Segment SGI178608[] = {{&S57944[34], 10}, {NULL}};
const Segment SGI178609[] = {{&S57944[27], 6}, {NULL}};
const Segment SGI178610[] = {{&S57944[97], 9}, {NULL}};
const Segment SGI178611[] = {{&S57944[70], 6}, {NULL}};
const Segment SGI178612[] = {{&S69029[34], 10}, {NULL}};
const Segment SGI178613[] = {{&S69029[27], 6}, {NULL}};
const Segment SGI178614[] = {{&S69029[97], 9}, {NULL}};
const Segment SGI178615[] = {{&S69029[70], 6}, {NULL}};
const Segment SGI178616[] = {{&S69029[138], 8}, {NULL}};
const Segment SGI178617[] = {{&S69033[34], 9}, {NULL}};
const Segment SGI178618[] = {{&S69033[27], 6}, {NULL}};
const Segment SGI178619[] = {{&S69033[96], 9}, {NULL}};
const Segment SGI178620[] = {{&S69033[69], 6}, {NULL}};
const Segment SGI178621[] = {{&S69034[34], 10}, {NULL}};
const Segment SGI178622[] = {{&S69034[27], 6}, {NULL}};
const Segment SGI178623[] = {{&S69034[97], 9}, {NULL}};
const Segment SGI178624[] = {{&S69034[70], 6}, {NULL}};
const Segment SGI178625[] = {{&S69035[34], 11}, {NULL}};
const Segment SGI178626[] = {{&S69035[27], 6}, {NULL}};
const Segment SGI178627[] = {{&S69035[98], 9}, {NULL}};
const Segment SGI178628[] = {{&S69035[71], 6}, {NULL}};
const Segment SGI178629[] = {{&S69036[27], 6}, {NULL}};
const Segment SGI178630[] = {{&S57955[44], 10}, {NULL}};
const Segment SGI178631[] = {{&S57955[44], 22}, {NULL}};
const Segment SGI178632[] = {{&S57955[30], 6}, {NULL}};
const Segment SGI178633[] = {{&S69036[99], 9}, {NULL}};
const Segment SGI178634[] = {{&S69036[72], 6}, {NULL}};
const Segment SGI178635[] = {{&S57955[131], 3}, {NULL}};
const Segment SGI178636[] = {{&S57955[96], 6}, {NULL}};
const Segment SGI178637[] = {{&S57955[68], 8}, {NULL}};
const Segment SGI178638[] = {{&S69037[34], 13}, {NULL}};
const Segment SGI178639[] = {{&S69037[27], 6}, {NULL}};
const Segment SGI178640[] = {{&S69037[100], 9}, {NULL}};
const Segment SGI178641[] = {{&S69037[73], 6}, {NULL}};
const Segment SGI178642[] = {{&S57957[44], 10}, {NULL}};
const Segment SGI178643[] = {{&S57957[44], 22}, {NULL}};
const Segment SGI178644[] = {{&S57957[30], 6}, {NULL}};
const Segment SGI178645[] = {{&S57957[152], 3}, {NULL}};
const Segment SGI178646[] = {{&S57957[117], 6}, {NULL}};
const Segment SGI178647[] = {{&S57957[68], 8}, {NULL}};
const Segment SGI178648[] = {{&S57958[44], 10}, {NULL}};
const Segment SGI178649[] = {{&S57958[44], 22}, {NULL}};
const Segment SGI178650[] = {{&S57958[30], 6}, {NULL}};
const Segment SGI178651[] = {{&S57958[131], 3}, {NULL}};
const Segment SGI178652[] = {{&S57958[96], 6}, {NULL}};
const Segment SGI178653[] = {{&S57958[68], 8}, {NULL}};
const Segment SGI178654[] = {{&S61621[45], 10}, {NULL}};
const Segment SGI178655[] = {{&S61621[30], 6}, {NULL}};
const Segment SGI178656[] = {{&S61621[115], 3}, {NULL}};
const Segment SGI178657[] = {{&S61621[81], 5}, {NULL}};
const Segment SGI178658[] = {{&S61621[139], 13}, {NULL}};
const Segment SGI178659[] = {{&S61622[45], 10}, {NULL}};
const Segment SGI178660[] = {{&S61622[30], 6}, {NULL}};
const Segment SGI178661[] = {{&S61622[115], 3}, {NULL}};
const Segment SGI178662[] = {{&S61622[81], 5}, {NULL}};
const Segment SGI178663[] = {{&S61622[139], 13}, {NULL}};
const Segment SGI178664[] = {{&S46620[34], 10}, {NULL}};
const Segment SGI178665[] = {{&S46620[27], 6}, {NULL}};
const Segment SGI178666[] = {{&S65640[42], 10}, {NULL}};
const Segment SGI178667[] = {{&S65640[42], 22}, {NULL}};
const Segment SGI178668[] = {{&S65640[30], 4}, {NULL}};
const Segment SGI178669[] = {{&S46620[96], 9}, {NULL}};
const Segment SGI178670[] = {{&S46620[69], 6}, {NULL}};
const Segment SGI178671[] = {{&S61624[45], 10}, {NULL}};
const Segment SGI178672[] = {{&S61624[30], 6}, {NULL}};
const Segment SGI178673[] = {{&S65640[113], 3}, {NULL}};
const Segment SGI178674[] = {{&S65640[78], 6}, {NULL}};
const Segment SGI178675[] = {{&S61624[115], 3}, {NULL}};
const Segment SGI178676[] = {{&S61624[81], 5}, {NULL}};
const Segment SGI178677[] = {{&S61624[139], 13}, {NULL}};
const Segment SGI178678[] = {{&S46621[34], 18}, {NULL}};
const Segment SGI178679[] = {{&S46621[27], 6}, {NULL}};
const Segment SGI178680[] = {{&S46621[104], 9}, {NULL}};
const Segment SGI178681[] = {{&S46621[77], 6}, {NULL}};
const Segment SGI178682[] = {{&S46622[34], 10}, {NULL}};
const Segment SGI178683[] = {{&S46622[27], 6}, {NULL}};
const Segment SGI178684[] = {{&S46622[96], 9}, {NULL}};
const Segment SGI178685[] = {{&S46622[69], 6}, {NULL}};
const Segment SGI178686[] = {{&S46623[34], 18}, {NULL}};
const Segment SGI178687[] = {{&S46623[27], 6}, {NULL}};
const Segment SGI178688[] = {{&S46623[104], 9}, {NULL}};
const Segment SGI178689[] = {{&S46623[77], 6}, {NULL}};
const Segment SGI178690[] = {{&S46624[34], 10}, {NULL}};
const Segment SGI178691[] = {{&S46624[27], 6}, {NULL}};
const Segment SGI178692[] = {{&S46624[96], 9}, {NULL}};
const Segment SGI178693[] = {{&S46624[69], 6}, {NULL}};
const Segment SGI178694[] = {{&S46625[34], 10}, {NULL}};
const Segment SGI178695[] = {{&S46625[27], 6}, {NULL}};
const Segment SGI178696[] = {{&S46625[96], 9}, {NULL}};
const Segment SGI178697[] = {{&S46625[69], 6}, {NULL}};
const Segment SGI178698[] = {{&S46626[34], 11}, {NULL}};
const Segment SGI178699[] = {{&S46626[27], 6}, {NULL}};
const Segment SGI178700[] = {{&S46626[97], 9}, {NULL}};
const Segment SGI178701[] = {{&S46626[70], 6}, {NULL}};
const Segment SGI178702[] = {{&S46629[34], 10}, {NULL}};
const Segment SGI178703[] = {{&S46629[27], 6}, {NULL}};
const Segment SGI178704[] = {{&S46629[97], 9}, {NULL}};
const Segment SGI178705[] = {{&S46629[70], 6}, {NULL}};
const Segment SGI178706[] = {{&S46630[34], 11}, {NULL}};
const Segment SGI178707[] = {{&S46630[27], 6}, {NULL}};
const Segment SGI178708[] = {{&S46630[97], 9}, {NULL}};
const Segment SGI178709[] = {{&S46630[70], 6}, {NULL}};
const Segment SGI178710[] = {{&S69088[44], 9}, {NULL}};
const Segment SGI178711[] = {{&S69088[44], 21}, {NULL}};
const Segment SGI178712[] = {{&S69088[30], 6}, {NULL}};
const Segment SGI178713[] = {{&S69088[31], 3}, {NULL}};
const Segment SGI178714[] = {{&S69088[79], 6}, {NULL}};
const Segment SGI178715[] = {{&S67336[34], 7}, {NULL}};
const Segment SGI178716[] = {{&S67336[27], 6}, {NULL}};
const Segment SGI178717[] = {{&S67336[67], 6}, {NULL}};
const Segment SGI178718[] = {{&S61661[34], 14}, {NULL}};
const Segment SGI178719[] = {{&S61661[27], 6}, {NULL}};
const Segment SGI178720[] = {{&S61661[101], 9}, {NULL}};
const Segment SGI178721[] = {{&S61661[74], 6}, {NULL}};
const Segment SGI178722[] = {{&S61662[34], 14}, {NULL}};
const Segment SGI178723[] = {{&S61662[27], 6}, {NULL}};
const Segment SGI178724[] = {{&S61662[101], 9}, {NULL}};
const Segment SGI178725[] = {{&S61662[74], 6}, {NULL}};
const Segment SGI178726[] = {{&S46661[34], 7}, {NULL}};
const Segment SGI178727[] = {{&S46661[27], 6}, {NULL}};
const Segment SGI178728[] = {{&S46661[100], 9}, {NULL}};
const Segment SGI178729[] = {{&S46661[73], 6}, {NULL}};
const Segment SGI178730[] = {{&S46662[34], 7}, {NULL}};
const Segment SGI178731[] = {{&S46662[27], 6}, {NULL}};
const Segment SGI178732[] = {{&S46662[102], 9}, {NULL}};
const Segment SGI178733[] = {{&S46662[75], 6}, {NULL}};
const Segment SGI178734[] = {{&S61674[34], 14}, {NULL}};
const Segment SGI178735[] = {{&S61674[27], 6}, {NULL}};
const Segment SGI178736[] = {{&S61674[101], 9}, {NULL}};
const Segment SGI178737[] = {{&S61674[74], 6}, {NULL}};
const Segment SGI178738[] = {{&S69109[44], 7}, {NULL}};
const Segment SGI178739[] = {{&S69109[44], 19}, {NULL}};
const Segment SGI178740[] = {{&S69109[30], 6}, {NULL}};
const Segment SGI178741[] = {{&S69109[31], 3}, {NULL}};
const Segment SGI178742[] = {{&S61675[34], 14}, {NULL}};
const Segment SGI178743[] = {{&S61675[27], 6}, {NULL}};
const Segment SGI178744[] = {{&S69109[96], 6}, {NULL}};
const Segment SGI178745[] = {{&S69109[65], 8}, {NULL}};
const Segment SGI178746[] = {{&S61675[101], 9}, {NULL}};
const Segment SGI178747[] = {{&S61675[74], 6}, {NULL}};
const Segment SGI178748[] = {{&S58028[44], 10}, {NULL}};
const Segment SGI178749[] = {{&S58028[44], 22}, {NULL}};
const Segment SGI178750[] = {{&S58028[30], 6}, {NULL}};
const Segment SGI178751[] = {{&S58028[115], 3}, {NULL}};
const Segment SGI178752[] = {{&S58028[80], 6}, {NULL}};
const Segment SGI178753[] = {{&S58028[140], 7}, {NULL}};
const Segment SGI178754[] = {{&S58029[44], 10}, {NULL}};
const Segment SGI178755[] = {{&S58029[44], 22}, {NULL}};
const Segment SGI178756[] = {{&S58029[30], 6}, {NULL}};
const Segment SGI178757[] = {{&S58029[115], 3}, {NULL}};
const Segment SGI178758[] = {{&S58029[80], 6}, {NULL}};
const Segment SGI178759[] = {{&S58029[140], 7}, {NULL}};
const Segment SGI178760[] = {{&S58030[44], 18}, {NULL}};
const Segment SGI178761[] = {{&S58030[44], 30}, {NULL}};
const Segment SGI178762[] = {{&S58030[30], 6}, {NULL}};
const Segment SGI178763[] = {{&S58030[123], 3}, {NULL}};
const Segment SGI178764[] = {{&S58030[88], 6}, {NULL}};
const Segment SGI178765[] = {{&S58030[148], 7}, {NULL}};
const Segment SGI178766[] = {{&S58031[44], 10}, {NULL}};
const Segment SGI178767[] = {{&S58031[44], 22}, {NULL}};
const Segment SGI178768[] = {{&S58031[30], 6}, {NULL}};
const Segment SGI178769[] = {{&S58031[115], 3}, {NULL}};
const Segment SGI178770[] = {{&S58031[80], 6}, {NULL}};
const Segment SGI178771[] = {{&S58031[140], 7}, {NULL}};
const Segment SGI178772[] = {{&S58032[44], 10}, {NULL}};
const Segment SGI178773[] = {{&S58032[44], 22}, {NULL}};
const Segment SGI178774[] = {{&S58032[30], 6}, {NULL}};
const Segment SGI178775[] = {{&S58032[115], 3}, {NULL}};
const Segment SGI178776[] = {{&S58032[80], 6}, {NULL}};
const Segment SGI178777[] = {{&S58032[140], 7}, {NULL}};
const Segment SGI178778[] = {{&S58033[44], 11}, {NULL}};
const Segment SGI178779[] = {{&S58033[44], 23}, {NULL}};
const Segment SGI178780[] = {{&S58033[30], 6}, {NULL}};
const Segment SGI178781[] = {{&S58033[116], 3}, {NULL}};
const Segment SGI178782[] = {{&S58033[81], 6}, {NULL}};
const Segment SGI178783[] = {{&S58033[141], 7}, {NULL}};
const Segment SGI178784[] = {{&S58036[34], 14}, {NULL}};
const Segment SGI178785[] = {{&S58036[27], 6}, {NULL}};
const Segment SGI178786[] = {{&S58036[101], 9}, {NULL}};
const Segment SGI178787[] = {{&S58036[74], 6}, {NULL}};
const Segment SGI178788[] = {{&S69135[6], 4}, {NULL}};
const Segment SGI178789[] = {{&S69135[82], 5}, {NULL}};
const Segment SGI178790[] = {{&S58046[34], 7}, {NULL}};
const Segment SGI178791[] = {{&S58046[27], 6}, {NULL}};
const Segment SGI178792[] = {{&S58046[97], 9}, {NULL}};
const Segment SGI178793[] = {{&S58046[70], 6}, {NULL}};
const Segment SGI178794[] = {{&S69144[44], 22}, {NULL}};
const Segment SGI178795[] = {{&S69144[30], 6}, {NULL}};
const Segment SGI178796[] = {{&S69144[31], 3}, {NULL}};
const Segment SGI178797[] = {{&S69144[82], 6}, {NULL}};
const Segment SGI178798[] = {{&S69145[44], 11}, {NULL}};
const Segment SGI178799[] = {{&S69145[44], 23}, {NULL}};
const Segment SGI178800[] = {{&S69145[30], 6}, {NULL}};
const Segment SGI178801[] = {{&S69145[84], 6}, {NULL}};
const Segment SGI178802[] = {{&S61716[44], 15}, {NULL}};
const Segment SGI178803[] = {{&S61716[44], 27}, {NULL}};
const Segment SGI178804[] = {{&S61716[30], 6}, {NULL}};
const Segment SGI178805[] = {{&S61716[120], 3}, {NULL}};
const Segment SGI178806[] = {{&S69146[44], 22}, {NULL}};
const Segment SGI178807[] = {{&S61716[85], 6}, {NULL}};
const Segment SGI178808[] = {{&S69146[30], 6}, {NULL}};
const Segment SGI178809[] = {{&S69146[31], 3}, {NULL}};
const Segment SGI178810[] = {{&S69146[82], 6}, {NULL}};
const Segment SGI178811[] = {{&S58055[34], 10}, {NULL}};
const Segment SGI178812[] = {{&S58055[27], 6}, {NULL}};
const Segment SGI178813[] = {{&S61717[44], 15}, {NULL}};
const Segment SGI178814[] = {{&S61717[44], 27}, {NULL}};
const Segment SGI178815[] = {{&S61717[30], 6}, {NULL}};
const Segment SGI178816[] = {{&S58055[97], 9}, {NULL}};
const Segment SGI178817[] = {{&S58055[70], 6}, {NULL}};
const Segment SGI178818[] = {{&S69147[44], 10}, {NULL}};
const Segment SGI178819[] = {{&S69147[30], 6}, {NULL}};
const Segment SGI178820[] = {{&S61717[120], 3}, {NULL}};
const Segment SGI178821[] = {{&S61717[85], 6}, {NULL}};
const Segment SGI178822[] = {{&S69147[31], 3}, {NULL}};
const Segment SGI178823[] = {{&S69147[70], 6}, {NULL}};
const Segment SGI178824[] = {{&S58056[34], 11}, {NULL}};
const Segment SGI178825[] = {{&S58056[27], 6}, {NULL}};
const Segment SGI178826[] = {{&S58056[98], 9}, {NULL}};
const Segment SGI178827[] = {{&S58056[71], 6}, {NULL}};
const Segment SGI178828[] = {{&S69148[44], 22}, {NULL}};
const Segment SGI178829[] = {{&S69148[30], 6}, {NULL}};
const Segment SGI178830[] = {{&S69148[31], 3}, {NULL}};
const Segment SGI178831[] = {{&S69148[82], 6}, {NULL}};
const Segment SGI178832[] = {{&S61719[44], 15}, {NULL}};
const Segment SGI178833[] = {{&S61719[44], 27}, {NULL}};
const Segment SGI178834[] = {{&S61719[30], 6}, {NULL}};
const Segment SGI178835[] = {{&S61719[120], 3}, {NULL}};
const Segment SGI178836[] = {{&S61719[85], 6}, {NULL}};
const Segment SGI178837[] = {{&S69149[44], 22}, {NULL}};
const Segment SGI178838[] = {{&S69149[30], 6}, {NULL}};
const Segment SGI178839[] = {{&S69149[31], 3}, {NULL}};
const Segment SGI178840[] = {{&S69149[82], 6}, {NULL}};
const Segment SGI178841[] = {{&S69150[44], 22}, {NULL}};
const Segment SGI178842[] = {{&S69150[30], 6}, {NULL}};
const Segment SGI178843[] = {{&S69150[31], 3}, {NULL}};
const Segment SGI178844[] = {{&S69150[82], 6}, {NULL}};
const Segment SGI178845[] = {{&S69151[44], 22}, {NULL}};
const Segment SGI178846[] = {{&S69151[30], 6}, {NULL}};
const Segment SGI178847[] = {{&S69151[31], 3}, {NULL}};
const Segment SGI178848[] = {{&S69151[82], 6}, {NULL}};
const Segment SGI178849[] = {{&S69152[38], 11}, {NULL}};
const Segment SGI178850[] = {{&S69152[31], 6}, {NULL}};
const Segment SGI178851[] = {{&S58073[34], 7}, {NULL}};
const Segment SGI178852[] = {{&S69160[34], 8}, {NULL}};
const Segment SGI178853[] = {{&S58073[27], 6}, {NULL}};
const Segment SGI178854[] = {{&S69160[27], 6}, {NULL}};
const Segment SGI178855[] = {{&S69160[95], 9}, {NULL}};
const Segment SGI178856[] = {{&S58073[100], 9}, {NULL}};
const Segment SGI178857[] = {{&S69160[68], 6}, {NULL}};
const Segment SGI178858[] = {{&S58073[73], 6}, {NULL}};
const Segment SGI178859[] = {{&S58074[34], 7}, {NULL}};
const Segment SGI178860[] = {{&S58074[27], 6}, {NULL}};
const Segment SGI178861[] = {{&S58074[100], 9}, {NULL}};
const Segment SGI178862[] = {{&S58074[73], 6}, {NULL}};
const Segment SGI178863[] = {{&S58081[44], 13}, {NULL}};
const Segment SGI178864[] = {{&S69170[34], 7}, {NULL}};
const Segment SGI178865[] = {{&S58081[44], 25}, {NULL}};
const Segment SGI178866[] = {{&S58081[30], 6}, {NULL}};
const Segment SGI178867[] = {{&S69170[27], 6}, {NULL}};
const Segment SGI178868[] = {{&S58081[134], 3}, {NULL}};
const Segment SGI178869[] = {{&S58081[99], 6}, {NULL}};
const Segment SGI178870[] = {{&S69170[104], 9}, {NULL}};
const Segment SGI178871[] = {{&S69170[77], 6}, {NULL}};
const Segment SGI178872[] = {{&S58081[71], 8}, {NULL}};
const Segment SGI178873[] = {{&S46764[34], 10}, {NULL}};
const Segment SGI178874[] = {{&S46764[27], 6}, {NULL}};
const Segment SGI178875[] = {{&S46764[96], 9}, {NULL}};
const Segment SGI178876[] = {{&S46764[69], 6}, {NULL}};
const Segment SGI178877[] = {{&S61761[44], 10}, {NULL}};
const Segment SGI178878[] = {{&S61761[44], 22}, {NULL}};
const Segment SGI178879[] = {{&S61761[30], 6}, {NULL}};
const Segment SGI178880[] = {{&S46765[34], 10}, {NULL}};
const Segment SGI178881[] = {{&S61761[115], 3}, {NULL}};
const Segment SGI178882[] = {{&S46765[27], 6}, {NULL}};
const Segment SGI178883[] = {{&S61761[80], 6}, {NULL}};
const Segment SGI178884[] = {{&S69184[34], 10}, {NULL}};
const Segment SGI178885[] = {{&S69184[27], 6}, {NULL}};
const Segment SGI178886[] = {{&S61761[147], 15}, {NULL}};
const Segment SGI178887[] = {{&S46765[96], 9}, {NULL}};
const Segment SGI178888[] = {{&S46765[69], 6}, {NULL}};
const Segment SGI178889[] = {{&S69184[97], 9}, {NULL}};
const Segment SGI178890[] = {{&S69184[70], 6}, {NULL}};
const Segment SGI178891[] = {{&S69184[131], 8}, {NULL}};
const Segment SGI178892[] = {{&S61763[44], 10}, {NULL}};
const Segment SGI178893[] = {{&S61763[44], 30}, {NULL}};
const Segment SGI178894[] = {{&S61763[30], 6}, {NULL}};
const Segment SGI178895[] = {{&S61763[123], 3}, {NULL}};
const Segment SGI178896[] = {{&S61763[88], 6}, {NULL}};
const Segment SGI178897[] = {{&S61763[155], 15}, {NULL}};
const Segment SGI178898[] = {{&S58104[34], 6}, {NULL}};
const Segment SGI178899[] = {{&S58104[27], 6}, {NULL}};
const Segment SGI178900[] = {{&S58104[93], 9}, {NULL}};
const Segment SGI178901[] = {{&S58104[66], 6}, {NULL}};
const Segment SGI178902[] = {{&S61764[44], 10}, {NULL}};
const Segment SGI178903[] = {{&S61764[44], 24}, {NULL}};
const Segment SGI178904[] = {{&S61764[30], 6}, {NULL}};
const Segment SGI178905[] = {{&S61764[117], 3}, {NULL}};
const Segment SGI178906[] = {{&S61764[82], 6}, {NULL}};
const Segment SGI178907[] = {{&S61764[149], 15}, {NULL}};
const Segment SGI178908[] = {{&S61765[44], 10}, {NULL}};
const Segment SGI178909[] = {{&S61765[44], 22}, {NULL}};
const Segment SGI178910[] = {{&S61765[30], 6}, {NULL}};
const Segment SGI178911[] = {{&S61765[80], 6}, {NULL}};
const Segment SGI178912[] = {{&S61765[146], 15}, {NULL}};
const Segment SGI178913[] = {{&S69196[34], 11}, {NULL}};
const Segment SGI178914[] = {{&S69196[27], 6}, {NULL}};
const Segment SGI178915[] = {{&S69196[98], 9}, {NULL}};
const Segment SGI178916[] = {{&S69196[71], 6}, {NULL}};
const Segment SGI178917[] = {{&S69197[34], 11}, {NULL}};
const Segment SGI178918[] = {{&S69197[27], 6}, {NULL}};
const Segment SGI178919[] = {{&S69197[98], 9}, {NULL}};
const Segment SGI178920[] = {{&S69197[71], 6}, {NULL}};
const Segment SGI178921[] = {{&S69200[34], 7}, {NULL}};
const Segment SGI178922[] = {{&S58119[34], 10}, {NULL}};
const Segment SGI178923[] = {{&S69200[27], 6}, {NULL}};
const Segment SGI178924[] = {{&S58119[27], 6}, {NULL}};
const Segment SGI178925[] = {{&S69200[94], 9}, {NULL}};
const Segment SGI178926[] = {{&S69200[67], 6}, {NULL}};
const Segment SGI178927[] = {{&S58119[97], 9}, {NULL}};
const Segment SGI178928[] = {{&S58119[70], 6}, {NULL}};
const Segment SGI178929[] = {{&S58120[34], 18}, {NULL}};
const Segment SGI178930[] = {{&S58120[27], 6}, {NULL}};
const Segment SGI178931[] = {{&S58120[105], 9}, {NULL}};
const Segment SGI178932[] = {{&S58120[78], 6}, {NULL}};
const Segment SGI178933[] = {{&S46789[34], 8}, {NULL}};
const Segment SGI178934[] = {{&S46789[27], 6}, {NULL}};
const Segment SGI178935[] = {{&S46789[94], 9}, {NULL}};
const Segment SGI178936[] = {{&S46789[67], 6}, {NULL}};
const Segment SGI178937[] = {{&S69210[44], 6}, {NULL}};
const Segment SGI178938[] = {{&S69210[44], 18}, {NULL}};
const Segment SGI178939[] = {{&S69210[30], 6}, {NULL}};
const Segment SGI178940[] = {{&S61786[34], 8}, {NULL}};
const Segment SGI178941[] = {{&S61786[27], 6}, {NULL}};
const Segment SGI178942[] = {{&S69210[31], 3}, {NULL}};
const Segment SGI178943[] = {{&S69210[76], 6}, {NULL}};
const Segment SGI178944[] = {{&S61786[96], 9}, {NULL}};
const Segment SGI178945[] = {{&S61786[69], 6}, {NULL}};
const Segment SGI178946[] = {{&S61787[34], 8}, {NULL}};
const Segment SGI178947[] = {{&S61787[27], 6}, {NULL}};
const Segment SGI178948[] = {{&S61787[96], 9}, {NULL}};
const Segment SGI178949[] = {{&S61787[69], 6}, {NULL}};
const Segment SGI178950[] = {{&S69215[44], 8}, {NULL}};
const Segment SGI178951[] = {{&S69215[44], 20}, {NULL}};
const Segment SGI178952[] = {{&S69215[30], 6}, {NULL}};
const Segment SGI178953[] = {{&S58133[34], 18}, {NULL}};
const Segment SGI178954[] = {{&S58133[27], 6}, {NULL}};
const Segment SGI178955[] = {{&S69215[31], 3}, {NULL}};
const Segment SGI178956[] = {{&S69215[78], 6}, {NULL}};
const Segment SGI178957[] = {{&S58133[105], 9}, {NULL}};
const Segment SGI178958[] = {{&S58133[78], 6}, {NULL}};
const Segment SGI178959[] = {{&S61795[34], 6}, {NULL}};
const Segment SGI178960[] = {{&S61795[27], 6}, {NULL}};
const Segment SGI178961[] = {{&S61795[93], 9}, {NULL}};
const Segment SGI178962[] = {{&S61795[66], 6}, {NULL}};
const Segment SGI178963[] = {{&S61796[34], 7}, {NULL}};
const Segment SGI178964[] = {{&S61796[27], 6}, {NULL}};
const Segment SGI178965[] = {{&S61796[94], 9}, {NULL}};
const Segment SGI178966[] = {{&S61796[67], 6}, {NULL}};
const Segment SGI178967[] = {{&S58149[34], 7}, {NULL}};
const Segment SGI178968[] = {{&S58149[27], 6}, {NULL}};
const Segment SGI178969[] = {{&S58149[99], 9}, {NULL}};
const Segment SGI178970[] = {{&S58149[72], 6}, {NULL}};
const Segment SGI178971[] = {{&S58150[34], 6}, {NULL}};
const Segment SGI178972[] = {{&S58150[27], 6}, {NULL}};
const Segment SGI178973[] = {{&S58150[93], 9}, {NULL}};
const Segment SGI178974[] = {{&S58150[66], 6}, {NULL}};
const Segment SGI178975[] = {{&S58151[34], 6}, {NULL}};
const Segment SGI178976[] = {{&S58151[27], 6}, {NULL}};
const Segment SGI178977[] = {{&S58151[93], 9}, {NULL}};
const Segment SGI178978[] = {{&S58151[66], 6}, {NULL}};
const Segment SGI178979[] = {{&S58152[34], 7}, {NULL}};
const Segment SGI178980[] = {{&S58152[27], 6}, {NULL}};
const Segment SGI178981[] = {{&S58152[98], 9}, {NULL}};
const Segment SGI178982[] = {{&S58152[71], 6}, {NULL}};
const Segment SGI178983[] = {{&S69245[44], 8}, {NULL}};
const Segment SGI178984[] = {{&S69245[44], 20}, {NULL}};
const Segment SGI178985[] = {{&S69245[30], 6}, {NULL}};
const Segment SGI178986[] = {{&S69245[31], 3}, {NULL}};
const Segment SGI178987[] = {{&S69245[78], 6}, {NULL}};
const Segment SGI178988[] = {{&S69247[44], 10}, {NULL}};
const Segment SGI178989[] = {{&S69247[44], 22}, {NULL}};
const Segment SGI178990[] = {{&S69247[30], 6}, {NULL}};
const Segment SGI178991[] = {{&S69247[31], 3}, {NULL}};
const Segment SGI178992[] = {{&S69247[80], 6}, {NULL}};
const Segment SGI178993[] = {{&S69273[34], 10}, {NULL}};
const Segment SGI178994[] = {{&S69273[27], 6}, {NULL}};
const Segment SGI178995[] = {{&S69273[97], 9}, {NULL}};
const Segment SGI178996[] = {{&S69273[70], 6}, {NULL}};
const Segment SGI178997[] = {{&S69274[34], 9}, {NULL}};
const Segment SGI178998[] = {{&S69274[27], 6}, {NULL}};
const Segment SGI178999[] = {{&S69274[96], 9}, {NULL}};
const Segment SGI179000[] = {{&S69274[69], 6}, {NULL}};
const Segment SGI179001[] = {{&S55907[34], 10}, {NULL}};
const Segment SGI179002[] = {{&S55907[27], 6}, {NULL}};
const Segment SGI179003[] = {{&S55907[97], 9}, {NULL}};
const Segment SGI179004[] = {{&S55907[70], 6}, {NULL}};
const Segment SGI179005[] = {{&S69277[44], 12}, {NULL}};
const Segment SGI179006[] = {{&S69277[44], 24}, {NULL}};
const Segment SGI179007[] = {{&S69277[30], 6}, {NULL}};
const Segment SGI179008[] = {{&S55910[34], 6}, {NULL}};
const Segment SGI179009[] = {{&S55910[27], 6}, {NULL}};
const Segment SGI179010[] = {{&S69277[31], 3}, {NULL}};
const Segment SGI179011[] = {{&S69277[82], 6}, {NULL}};
const Segment SGI179012[] = {{&S55910[93], 9}, {NULL}};
const Segment SGI179013[] = {{&S55910[66], 6}, {NULL}};
const Segment SGI179014[] = {{&S55911[34], 4}, {NULL}};
const Segment SGI179015[] = {{&S55911[27], 6}, {NULL}};
const Segment SGI179016[] = {{&S55911[91], 9}, {NULL}};
const Segment SGI179017[] = {{&S55911[64], 6}, {NULL}};
const Segment SGI179018[] = {{&S55912[34], 20}, {NULL}};
const Segment SGI179019[] = {{&S55912[27], 6}, {NULL}};
const Segment SGI179020[] = {{&S55912[107], 9}, {NULL}};
const Segment SGI179021[] = {{&S55912[80], 6}, {NULL}};
const Segment SGI179022[] = {{&S46878[34], 10}, {NULL}};
const Segment SGI179023[] = {{&S46878[27], 6}, {NULL}};
const Segment SGI179024[] = {{&S46878[96], 9}, {NULL}};
const Segment SGI179025[] = {{&S46878[69], 6}, {NULL}};
const Segment SGI179026[] = {{&S55914[34], 14}, {NULL}};
const Segment SGI179027[] = {{&S55914[27], 6}, {NULL}};
const Segment SGI179028[] = {{&S55914[101], 9}, {NULL}};
const Segment SGI179029[] = {{&S55914[74], 6}, {NULL}};
const Segment SGI179030[] = {{&S46879[34], 10}, {NULL}};
const Segment SGI179031[] = {{&S46879[27], 6}, {NULL}};
const Segment SGI179032[] = {{&S46879[96], 9}, {NULL}};
const Segment SGI179033[] = {{&S46879[69], 6}, {NULL}};
const Segment SGI179034[] = {{&S55915[34], 4}, {NULL}};
const Segment SGI179035[] = {{&S55915[27], 6}, {NULL}};
const Segment SGI179036[] = {{&S55915[91], 9}, {NULL}};
const Segment SGI179037[] = {{&S55915[64], 6}, {NULL}};
const Segment SGI179038[] = {{&S55916[34], 6}, {NULL}};
const Segment SGI179039[] = {{&S55916[27], 6}, {NULL}};
const Segment SGI179040[] = {{&S55916[93], 9}, {NULL}};
const Segment SGI179041[] = {{&S55916[66], 6}, {NULL}};
const Segment SGI179042[] = {{&S55917[34], 13}, {NULL}};
const Segment SGI179043[] = {{&S55917[27], 6}, {NULL}};
const Segment SGI179044[] = {{&S55917[100], 9}, {NULL}};
const Segment SGI179045[] = {{&S55917[73], 6}, {NULL}};
const Segment SGI179046[] = {{&S55918[34], 17}, {NULL}};
const Segment SGI179047[] = {{&S55918[27], 6}, {NULL}};
const Segment SGI179048[] = {{&S55918[104], 9}, {NULL}};
const Segment SGI179049[] = {{&S55918[77], 6}, {NULL}};
const Segment SGI179050[] = {{&S55919[34], 10}, {NULL}};
const Segment SGI179051[] = {{&S55919[27], 6}, {NULL}};
const Segment SGI179052[] = {{&S55919[97], 9}, {NULL}};
const Segment SGI179053[] = {{&S55919[70], 6}, {NULL}};
const Segment SGI179054[] = {{&S55920[34], 9}, {NULL}};
const Segment SGI179055[] = {{&S55920[27], 6}, {NULL}};
const Segment SGI179056[] = {{&S55920[96], 9}, {NULL}};
const Segment SGI179057[] = {{&S55920[69], 6}, {NULL}};
const Segment SGI179058[] = {{&S55921[34], 6}, {NULL}};
const Segment SGI179059[] = {{&S55921[27], 6}, {NULL}};
const Segment SGI179060[] = {{&S55921[93], 9}, {NULL}};
const Segment SGI179061[] = {{&S55921[66], 6}, {NULL}};
const Segment SGI179062[] = {{&S55925[34], 12}, {NULL}};
const Segment SGI179063[] = {{&S55925[27], 6}, {NULL}};
const Segment SGI179064[] = {{&S55925[99], 9}, {NULL}};
const Segment SGI179065[] = {{&S55925[72], 6}, {NULL}};
const Segment SGI179066[] = {{&S55926[34], 23}, {NULL}};
const Segment SGI179067[] = {{&S55926[27], 6}, {NULL}};
const Segment SGI179068[] = {{&S55926[110], 9}, {NULL}};
const Segment SGI179069[] = {{&S55926[83], 6}, {NULL}};
const Segment SGI179070[] = {{&S55927[34], 6}, {NULL}};
const Segment SGI179071[] = {{&S55927[27], 6}, {NULL}};
const Segment SGI179072[] = {{&S55927[93], 9}, {NULL}};
const Segment SGI179073[] = {{&S55927[66], 6}, {NULL}};
const Segment SGI179074[] = {{&S55929[34], 7}, {NULL}};
const Segment SGI179075[] = {{&S55929[27], 6}, {NULL}};
const Segment SGI179076[] = {{&S69292[44], 7}, {NULL}};
const Segment SGI179077[] = {{&S55929[94], 9}, {NULL}};
const Segment SGI179078[] = {{&S55929[67], 6}, {NULL}};
const Segment SGI179079[] = {{&S69292[44], 19}, {NULL}};
const Segment SGI179080[] = {{&S69292[30], 6}, {NULL}};
const Segment SGI179081[] = {{&S69292[31], 3}, {NULL}};
const Segment SGI179082[] = {{&S69292[96], 6}, {NULL}};
const Segment SGI179083[] = {{&S69292[65], 8}, {NULL}};
const Segment SGI179084[] = {{&S55930[34], 8}, {NULL}};
const Segment SGI179085[] = {{&S55930[27], 6}, {NULL}};
const Segment SGI179086[] = {{&S55930[95], 9}, {NULL}};
const Segment SGI179087[] = {{&S55930[68], 6}, {NULL}};
const Segment SGI179088[] = {{&S69293[44], 7}, {NULL}};
const Segment SGI179089[] = {{&S69293[44], 19}, {NULL}};
const Segment SGI179090[] = {{&S69293[30], 6}, {NULL}};
const Segment SGI179091[] = {{&S61892[34], 5}, {NULL}};
const Segment SGI179092[] = {{&S69293[31], 3}, {NULL}};
const Segment SGI179093[] = {{&S61892[27], 6}, {NULL}};
const Segment SGI179094[] = {{&S69293[96], 6}, {NULL}};
const Segment SGI179095[] = {{&S55931[34], 7}, {NULL}};
const Segment SGI179096[] = {{&S69293[65], 8}, {NULL}};
const Segment SGI179097[] = {{&S55931[27], 6}, {NULL}};
const Segment SGI179098[] = {{&S61892[92], 9}, {NULL}};
const Segment SGI179099[] = {{&S61892[65], 6}, {NULL}};
const Segment SGI179100[] = {{&S55931[94], 9}, {NULL}};
const Segment SGI179101[] = {{&S55931[67], 6}, {NULL}};
const Segment SGI179102[] = {{&S69294[44], 7}, {NULL}};
const Segment SGI179103[] = {{&S69294[44], 19}, {NULL}};
const Segment SGI179104[] = {{&S69294[30], 6}, {NULL}};
const Segment SGI179105[] = {{&S69294[31], 3}, {NULL}};
const Segment SGI179106[] = {{&S69294[92], 6}, {NULL}};
const Segment SGI179107[] = {{&S69294[65], 8}, {NULL}};
const Segment SGI179108[] = {{&S69295[44], 7}, {NULL}};
const Segment SGI179109[] = {{&S55934[34], 13}, {NULL}};
const Segment SGI179110[] = {{&S69295[44], 17}, {NULL}};
const Segment SGI179111[] = {{&S69295[30], 6}, {NULL}};
const Segment SGI179112[] = {{&S55934[27], 6}, {NULL}};
const Segment SGI179113[] = {{&S69295[31], 3}, {NULL}};
const Segment SGI179114[] = {{&S69295[90], 6}, {NULL}};
const Segment SGI179115[] = {{&S55934[100], 9}, {NULL}};
const Segment SGI179116[] = {{&S55934[73], 6}, {NULL}};
const Segment SGI179117[] = {{&S69295[63], 8}, {NULL}};
const Segment SGI179118[] = {{&S55935[34], 11}, {NULL}};
const Segment SGI179119[] = {{&S55935[27], 6}, {NULL}};
const Segment SGI179120[] = {{&S55935[98], 9}, {NULL}};
const Segment SGI179121[] = {{&S55935[71], 6}, {NULL}};
const Segment SGI179122[] = {{&S55936[34], 9}, {NULL}};
const Segment SGI179123[] = {{&S55936[27], 6}, {NULL}};
const Segment SGI179124[] = {{&S61900[34], 10}, {NULL}};
const Segment SGI179125[] = {{&S61900[27], 6}, {NULL}};
const Segment SGI179126[] = {{&S55936[96], 9}, {NULL}};
const Segment SGI179127[] = {{&S55936[69], 6}, {NULL}};
const Segment SGI179128[] = {{&S61900[97], 9}, {NULL}};
const Segment SGI179129[] = {{&S61900[70], 6}, {NULL}};
const Segment SGI179130[] = {{&S61900[138], 8}, {NULL}};
const Segment SGI179131[] = {{&S55937[34], 4}, {NULL}};
const Segment SGI179132[] = {{&S55937[27], 6}, {NULL}};
const Segment SGI179133[] = {{&S55937[91], 9}, {NULL}};
const Segment SGI179134[] = {{&S55937[64], 6}, {NULL}};
const Segment SGI179135[] = {{&S55938[34], 8}, {NULL}};
const Segment SGI179136[] = {{&S55938[27], 6}, {NULL}};
const Segment SGI179137[] = {{&S55938[95], 9}, {NULL}};
const Segment SGI179138[] = {{&S55938[68], 6}, {NULL}};
const Segment SGI179139[] = {{&S46921[34], 6}, {NULL}};
const Segment SGI179140[] = {{&S46921[27], 6}, {NULL}};
const Segment SGI179141[] = {{&S55940[34], 33}, {NULL}};
const Segment SGI179142[] = {{&S46921[92], 9}, {NULL}};
const Segment SGI179143[] = {{&S46921[65], 6}, {NULL}};
const Segment SGI179144[] = {{&S55940[27], 6}, {NULL}};
const Segment SGI179145[] = {{&S55940[120], 9}, {NULL}};
const Segment SGI179146[] = {{&S55940[93], 6}, {NULL}};
const Segment SGI179147[] = {{&S46922[34], 8}, {NULL}};
const Segment SGI179148[] = {{&S46922[27], 6}, {NULL}};
const Segment SGI179149[] = {{&S55941[34], 17}, {NULL}};
const Segment SGI179150[] = {{&S46922[94], 9}, {NULL}};
const Segment SGI179151[] = {{&S46922[67], 6}, {NULL}};
const Segment SGI179152[] = {{&S55941[27], 6}, {NULL}};
const Segment SGI179153[] = {{&S55941[104], 9}, {NULL}};
const Segment SGI179154[] = {{&S55941[77], 6}, {NULL}};
const Segment SGI179155[] = {{&S46926[34], 6}, {NULL}};
const Segment SGI179156[] = {{&S46926[27], 6}, {NULL}};
const Segment SGI179157[] = {{&S55945[34], 17}, {NULL}};
const Segment SGI179158[] = {{&S46926[92], 9}, {NULL}};
const Segment SGI179159[] = {{&S55945[27], 6}, {NULL}};
const Segment SGI179160[] = {{&S46926[65], 6}, {NULL}};
const Segment SGI179161[] = {{&S55945[104], 9}, {NULL}};
const Segment SGI179162[] = {{&S55945[77], 6}, {NULL}};
const Segment SGI179163[] = {{&S61914[44], 25}, {NULL}};
const Segment SGI179164[] = {{&S61914[44], 37}, {NULL}};
const Segment SGI179165[] = {{&S61914[30], 6}, {NULL}};
const Segment SGI179166[] = {{&S61914[31], 3}, {NULL}};
const Segment SGI179167[] = {{&S61914[95], 6}, {NULL}};
const Segment SGI179168[] = {{&S61915[38], 25}, {NULL}};
const Segment SGI179169[] = {{&S61915[31], 6}, {NULL}};
const Segment SGI179170[] = {{&S55949[34], 10}, {NULL}};
const Segment SGI179171[] = {{&S46931[34], 4}, {NULL}};
const Segment SGI179172[] = {{&S55949[27], 6}, {NULL}};
const Segment SGI179173[] = {{&S46931[27], 6}, {NULL}};
const Segment SGI179174[] = {{&S55949[97], 9}, {NULL}};
const Segment SGI179175[] = {{&S46931[90], 9}, {NULL}};
const Segment SGI179176[] = {{&S55949[70], 6}, {NULL}};
const Segment SGI179177[] = {{&S46931[63], 6}, {NULL}};
const Segment SGI179178[] = {{&S55950[34], 17}, {NULL}};
const Segment SGI179179[] = {{&S55950[27], 6}, {NULL}};
const Segment SGI179180[] = {{&S69320[34], 24}, {NULL}};
const Segment SGI179181[] = {{&S69320[27], 6}, {NULL}};
const Segment SGI179182[] = {{&S55950[112], 9}, {NULL}};
const Segment SGI179183[] = {{&S55950[85], 6}, {NULL}};
const Segment SGI179184[] = {{&S46933[34], 9}, {NULL}};
const Segment SGI179185[] = {{&S69320[111], 9}, {NULL}};
const Segment SGI179186[] = {{&S46933[27], 6}, {NULL}};
const Segment SGI179187[] = {{&S69320[84], 6}, {NULL}};
const Segment SGI179188[] = {{&S46933[95], 9}, {NULL}};
const Segment SGI179189[] = {{&S46933[68], 6}, {NULL}};
const Segment SGI179190[] = {{&S69321[34], 23}, {NULL}};
const Segment SGI179191[] = {{&S69321[27], 6}, {NULL}};
const Segment SGI179192[] = {{&S46934[34], 6}, {NULL}};
const Segment SGI179193[] = {{&S46934[27], 6}, {NULL}};
const Segment SGI179194[] = {{&S69321[110], 9}, {NULL}};
const Segment SGI179195[] = {{&S69321[83], 6}, {NULL}};
const Segment SGI179196[] = {{&S46934[91], 9}, {NULL}};
const Segment SGI179197[] = {{&S46934[64], 6}, {NULL}};
const Segment SGI179198[] = {{&S46936[34], 7}, {NULL}};
const Segment SGI179199[] = {{&S46936[27], 6}, {NULL}};
const Segment SGI179200[] = {{&S69323[34], 12}, {NULL}};
const Segment SGI179201[] = {{&S69323[27], 6}, {NULL}};
const Segment SGI179202[] = {{&S46936[93], 9}, {NULL}};
const Segment SGI179203[] = {{&S46936[66], 6}, {NULL}};
const Segment SGI179204[] = {{&S69323[99], 9}, {NULL}};
const Segment SGI179205[] = {{&S69323[72], 6}, {NULL}};
const Segment SGI179206[] = {{&S46937[34], 13}, {NULL}};
const Segment SGI179207[] = {{&S46937[27], 6}, {NULL}};
const Segment SGI179208[] = {{&S46937[99], 9}, {NULL}};
const Segment SGI179209[] = {{&S46937[72], 6}, {NULL}};
const Segment SGI179210[] = {{&S46938[34], 6}, {NULL}};
const Segment SGI179211[] = {{&S46938[27], 6}, {NULL}};
const Segment SGI179212[] = {{&S46938[93], 9}, {NULL}};
const Segment SGI179213[] = {{&S46938[66], 6}, {NULL}};
const Segment SGI179214[] = {{&S46939[34], 10}, {NULL}};
const Segment SGI179215[] = {{&S46939[27], 6}, {NULL}};
const Segment SGI179216[] = {{&S46939[96], 9}, {NULL}};
const Segment SGI179217[] = {{&S46939[69], 6}, {NULL}};
const Segment SGI179218[] = {{&S69330[44], 10}, {NULL}};
const Segment SGI179219[] = {{&S69330[44], 22}, {NULL}};
const Segment SGI179220[] = {{&S69330[30], 6}, {NULL}};
const Segment SGI179221[] = {{&S46940[34], 5}, {NULL}};
const Segment SGI179222[] = {{&S69330[31], 3}, {NULL}};
const Segment SGI179223[] = {{&S46940[27], 6}, {NULL}};
const Segment SGI179224[] = {{&S69330[80], 6}, {NULL}};
const Segment SGI179225[] = {{&S46940[91], 9}, {NULL}};
const Segment SGI179226[] = {{&S46940[64], 6}, {NULL}};
const Segment SGI179227[] = {{&S69332[34], 12}, {NULL}};
const Segment SGI179228[] = {{&S46942[34], 18}, {NULL}};
const Segment SGI179229[] = {{&S69332[27], 6}, {NULL}};
const Segment SGI179230[] = {{&S46942[27], 6}, {NULL}};
const Segment SGI179231[] = {{&S69332[99], 9}, {NULL}};
const Segment SGI179232[] = {{&S69332[72], 6}, {NULL}};
const Segment SGI179233[] = {{&S46942[104], 9}, {NULL}};
const Segment SGI179234[] = {{&S46942[77], 6}, {NULL}};
const Segment SGI179235[] = {{&S46945[34], 35}, {NULL}};
const Segment SGI179236[] = {{&S46945[27], 6}, {NULL}};
const Segment SGI179237[] = {{&S46945[121], 9}, {NULL}};
const Segment SGI179238[] = {{&S46945[94], 6}, {NULL}};
const Segment SGI179239[] = {{&S69337[44], 7}, {NULL}};
const Segment SGI179240[] = {{&S69337[44], 19}, {NULL}};
const Segment SGI179241[] = {{&S69337[30], 6}, {NULL}};
const Segment SGI179242[] = {{&S69337[77], 6}, {NULL}};
const Segment SGI179243[] = {{&S69337[104], 13}, {NULL}};
const Segment SGI179244[] = {{&S69338[44], 8}, {NULL}};
const Segment SGI179245[] = {{&S69338[44], 20}, {NULL}};
const Segment SGI179246[] = {{&S46949[34], 8}, {NULL}};
const Segment SGI179247[] = {{&S69338[30], 6}, {NULL}};
const Segment SGI179248[] = {{&S46949[27], 6}, {NULL}};
const Segment SGI179249[] = {{&S69338[78], 6}, {NULL}};
const Segment SGI179250[] = {{&S46949[94], 9}, {NULL}};
const Segment SGI179251[] = {{&S46949[67], 6}, {NULL}};
const Segment SGI179252[] = {{&S69338[105], 13}, {NULL}};
const Segment SGI179253[] = {{&S46955[34], 8}, {NULL}};
const Segment SGI179254[] = {{&S46955[27], 6}, {NULL}};
const Segment SGI179255[] = {{&S46955[94], 9}, {NULL}};
const Segment SGI179256[] = {{&S46955[67], 6}, {NULL}};
const Segment SGI179257[] = {{&S69344[44], 9}, {NULL}};
const Segment SGI179258[] = {{&S69344[44], 20}, {NULL}};
const Segment SGI179259[] = {{&S69344[30], 6}, {NULL}};
const Segment SGI179260[] = {{&S69344[31], 3}, {NULL}};
const Segment SGI179261[] = {{&S69344[78], 6}, {NULL}};
const Segment SGI179262[] = {{&S46957[34], 11}, {NULL}};
const Segment SGI179263[] = {{&S46957[27], 6}, {NULL}};
const Segment SGI179264[] = {{&S46957[97], 9}, {NULL}};
const Segment SGI179265[] = {{&S46957[70], 6}, {NULL}};
const Segment SGI179266[] = {{&S69346[44], 9}, {NULL}};
const Segment SGI179267[] = {{&S69346[44], 21}, {NULL}};
const Segment SGI179268[] = {{&S69346[30], 6}, {NULL}};
const Segment SGI179269[] = {{&S69346[31], 3}, {NULL}};
const Segment SGI179270[] = {{&S69346[79], 6}, {NULL}};
const Segment SGI179271[] = {{&S69347[38], 9}, {NULL}};
const Segment SGI179272[] = {{&S69347[31], 6}, {NULL}};
const Segment SGI179273[] = {{&S46962[34], 10}, {NULL}};
const Segment SGI179274[] = {{&S46962[27], 6}, {NULL}};
const Segment SGI179275[] = {{&S46962[96], 9}, {NULL}};
const Segment SGI179276[] = {{&S46962[69], 6}, {NULL}};
const Segment SGI179277[] = {{&S55989[44], 10}, {NULL}};
const Segment SGI179278[] = {{&S55989[44], 22}, {NULL}};
const Segment SGI179279[] = {{&S69350[43], 8}, {NULL}};
const Segment SGI179280[] = {{&S55989[30], 6}, {NULL}};
const Segment SGI179281[] = {{&S69350[43], 20}, {NULL}};
const Segment SGI179282[] = {{&S69350[29], 6}, {NULL}};
const Segment SGI179283[] = {{&S55989[133], 3}, {NULL}};
const Segment SGI179284[] = {{&S69350[30], 3}, {NULL}};
const Segment SGI179285[] = {{&S55989[98], 6}, {NULL}};
const Segment SGI179286[] = {{&S69350[77], 6}, {NULL}};
const Segment SGI179287[] = {{&S55989[68], 8}, {NULL}};
const Segment SGI179288[] = {{&S46966[34], 8}, {NULL}};
const Segment SGI179289[] = {{&S46966[27], 6}, {NULL}};
const Segment SGI179290[] = {{&S55991[44], 10}, {NULL}};
const Segment SGI179291[] = {{&S55991[44], 22}, {NULL}};
const Segment SGI179292[] = {{&S55991[30], 6}, {NULL}};
const Segment SGI179293[] = {{&S46966[94], 9}, {NULL}};
const Segment SGI179294[] = {{&S46966[67], 6}, {NULL}};
const Segment SGI179295[] = {{&S55991[133], 3}, {NULL}};
const Segment SGI179296[] = {{&S55991[98], 6}, {NULL}};
const Segment SGI179297[] = {{&S55991[68], 8}, {NULL}};
const Segment SGI179298[] = {{&S61966[44], 11}, {NULL}};
const Segment SGI179299[] = {{&S61966[44], 23}, {NULL}};
const Segment SGI179300[] = {{&S61966[30], 6}, {NULL}};
const Segment SGI179301[] = {{&S46967[34], 10}, {NULL}};
const Segment SGI179302[] = {{&S61966[116], 3}, {NULL}};
const Segment SGI179303[] = {{&S46967[27], 6}, {NULL}};
const Segment SGI179304[] = {{&S61966[81], 6}, {NULL}};
const Segment SGI179305[] = {{&S46967[96], 9}, {NULL}};
const Segment SGI179306[] = {{&S46967[69], 6}, {NULL}};
const Segment SGI179307[] = {{&S55993[44], 6}, {NULL}};
const Segment SGI179308[] = {{&S55993[44], 18}, {NULL}};
const Segment SGI179309[] = {{&S55993[30], 6}, {NULL}};
const Segment SGI179310[] = {{&S61967[44], 11}, {NULL}};
const Segment SGI179311[] = {{&S61967[44], 23}, {NULL}};
const Segment SGI179312[] = {{&S61967[30], 6}, {NULL}};
const Segment SGI179313[] = {{&S55993[111], 3}, {NULL}};
const Segment SGI179314[] = {{&S55993[76], 6}, {NULL}};
const Segment SGI179315[] = {{&S61967[116], 3}, {NULL}};
const Segment SGI179316[] = {{&S61967[81], 6}, {NULL}};
const Segment SGI179317[] = {{&S46969[34], 10}, {NULL}};
const Segment SGI179318[] = {{&S46969[27], 6}, {NULL}};
const Segment SGI179319[] = {{&S46969[96], 9}, {NULL}};
const Segment SGI179320[] = {{&S55994[44], 6}, {NULL}};
const Segment SGI179321[] = {{&S46969[69], 6}, {NULL}};
const Segment SGI179322[] = {{&S55994[44], 18}, {NULL}};
const Segment SGI179323[] = {{&S55994[30], 6}, {NULL}};
const Segment SGI179324[] = {{&S61968[44], 11}, {NULL}};
const Segment SGI179325[] = {{&S55994[111], 3}, {NULL}};
const Segment SGI179326[] = {{&S61968[44], 23}, {NULL}};
const Segment SGI179327[] = {{&S61968[30], 6}, {NULL}};
const Segment SGI179328[] = {{&S55994[76], 6}, {NULL}};
const Segment SGI179329[] = {{&S61968[116], 3}, {NULL}};
const Segment SGI179330[] = {{&S61968[81], 6}, {NULL}};
const Segment SGI179331[] = {{&S55996[44], 6}, {NULL}};
const Segment SGI179332[] = {{&S55996[44], 20}, {NULL}};
const Segment SGI179333[] = {{&S55996[30], 6}, {NULL}};
const Segment SGI179334[] = {{&S55996[113], 3}, {NULL}};
const Segment SGI179335[] = {{&S46973[34], 11}, {NULL}};
const Segment SGI179336[] = {{&S55996[78], 6}, {NULL}};
const Segment SGI179337[] = {{&S46973[27], 6}, {NULL}};
const Segment SGI179338[] = {{&S46973[98], 9}, {NULL}};
const Segment SGI179339[] = {{&S46973[71], 6}, {NULL}};
const Segment SGI179340[] = {{&S46974[34], 9}, {NULL}};
const Segment SGI179341[] = {{&S46974[27], 6}, {NULL}};
const Segment SGI179342[] = {{&S46974[95], 9}, {NULL}};
const Segment SGI179343[] = {{&S46974[68], 6}, {NULL}};
const Segment SGI179344[] = {{&S46979[34], 19}, {NULL}};
const Segment SGI179345[] = {{&S46979[27], 6}, {NULL}};
const Segment SGI179346[] = {{&S46979[105], 9}, {NULL}};
const Segment SGI179347[] = {{&S46979[78], 6}, {NULL}};
const Segment SGI179348[] = {{&S46980[34], 8}, {NULL}};
const Segment SGI179349[] = {{&S46980[27], 6}, {NULL}};
const Segment SGI179350[] = {{&S46980[95], 9}, {NULL}};
const Segment SGI179351[] = {{&S46980[68], 6}, {NULL}};
const Segment SGI179352[] = {{&S46982[34], 10}, {NULL}};
const Segment SGI179353[] = {{&S46982[27], 6}, {NULL}};
const Segment SGI179354[] = {{&S46982[96], 9}, {NULL}};
const Segment SGI179355[] = {{&S46982[69], 6}, {NULL}};
const Segment SGI179356[] = {{&S46984[34], 10}, {NULL}};
const Segment SGI179357[] = {{&S46984[27], 6}, {NULL}};
const Segment SGI179358[] = {{&S46984[96], 9}, {NULL}};
const Segment SGI179359[] = {{&S46984[69], 6}, {NULL}};
const Segment SGI179360[] = {{&S69378[34], 10}, {NULL}};
const Segment SGI179361[] = {{&S69378[27], 6}, {NULL}};
const Segment SGI179362[] = {{&S69378[97], 9}, {NULL}};
const Segment SGI179363[] = {{&S69378[70], 6}, {NULL}};
const Segment SGI179364[] = {{&S46985[34], 9}, {NULL}};
const Segment SGI179365[] = {{&S46985[27], 6}, {NULL}};
const Segment SGI179366[] = {{&S46985[95], 9}, {NULL}};
const Segment SGI179367[] = {{&S46985[68], 6}, {NULL}};
const Segment SGI179368[] = {{&S46988[34], 7}, {NULL}};
const Segment SGI179369[] = {{&S46988[27], 6}, {NULL}};
const Segment SGI179370[] = {{&S46988[93], 9}, {NULL}};
const Segment SGI179371[] = {{&S46988[66], 6}, {NULL}};
const Segment SGI179372[] = {{&S46990[34], 25}, {NULL}};
const Segment SGI179373[] = {{&S46990[27], 6}, {NULL}};
const Segment SGI179374[] = {{&S46990[111], 9}, {NULL}};
const Segment SGI179375[] = {{&S46990[84], 6}, {NULL}};
const Segment SGI179376[] = {{&S69385[34], 15}, {NULL}};
const Segment SGI179377[] = {{&S69385[27], 6}, {NULL}};
const Segment SGI179378[] = {{&S69385[102], 9}, {NULL}};
const Segment SGI179379[] = {{&S69385[75], 6}, {NULL}};
const Segment SGI179380[] = {{&S46991[34], 25}, {NULL}};
const Segment SGI179381[] = {{&S46991[27], 6}, {NULL}};
const Segment SGI179382[] = {{&S46991[111], 9}, {NULL}};
const Segment SGI179383[] = {{&S46991[84], 6}, {NULL}};
const Segment SGI179384[] = {{&S46993[34], 6}, {NULL}};
const Segment SGI179385[] = {{&S46993[27], 6}, {NULL}};
const Segment SGI179386[] = {{&S46993[92], 9}, {NULL}};
const Segment SGI179387[] = {{&S46993[65], 6}, {NULL}};
const Segment SGI179388[] = {{&S46996[34], 12}, {NULL}};
const Segment SGI179389[] = {{&S46996[27], 6}, {NULL}};
const Segment SGI179390[] = {{&S46996[98], 9}, {NULL}};
const Segment SGI179391[] = {{&S46996[71], 6}, {NULL}};
const Segment SGI179392[] = {{&S69394[34], 14}, {NULL}};
const Segment SGI179393[] = {{&S69394[27], 6}, {NULL}};
const Segment SGI179394[] = {{&S69394[101], 9}, {NULL}};
const Segment SGI179395[] = {{&S69394[74], 6}, {NULL}};
const Segment SGI179396[] = {{&S69395[34], 14}, {NULL}};
const Segment SGI179397[] = {{&S69395[27], 6}, {NULL}};
const Segment SGI179398[] = {{&S69395[101], 9}, {NULL}};
const Segment SGI179399[] = {{&S69395[74], 6}, {NULL}};
const Segment SGI179400[] = {{&S69400[45], 10}, {NULL}};
const Segment SGI179401[] = {{&S69400[30], 6}, {NULL}};
const Segment SGI179402[] = {{&S69400[31], 3}, {NULL}};
const Segment SGI179403[] = {{&S69400[81], 5}, {NULL}};
const Segment SGI179404[] = {{&S69400[139], 13}, {NULL}};
const Segment SGI179405[] = {{&S47009[44], 20}, {NULL}};
const Segment SGI179406[] = {{&S47009[44], 31}, {NULL}};
const Segment SGI179407[] = {{&S47009[30], 6}, {NULL}};
const Segment SGI179408[] = {{&S47009[124], 3}, {NULL}};
const Segment SGI179409[] = {{&S47009[89], 6}, {NULL}};
const Segment SGI179410[] = {{&S47010[44], 21}, {NULL}};
const Segment SGI179411[] = {{&S47010[44], 32}, {NULL}};
const Segment SGI179412[] = {{&S47010[30], 6}, {NULL}};
const Segment SGI179413[] = {{&S47010[125], 3}, {NULL}};
const Segment SGI179414[] = {{&S47010[90], 6}, {NULL}};
const Segment SGI179415[] = {{&S47012[38], 20}, {NULL}};
const Segment SGI179416[] = {{&S47012[31], 6}, {NULL}};
const Segment SGI179417[] = {{&S69418[43], 7}, {NULL}};
const Segment SGI179418[] = {{&S69418[43], 19}, {NULL}};
const Segment SGI179419[] = {{&S69418[29], 6}, {NULL}};
const Segment SGI179420[] = {{&S69418[30], 3}, {NULL}};
const Segment SGI179421[] = {{&S69418[76], 6}, {NULL}};
const Segment SGI179422[] = {{&S69420[34], 25}, {NULL}};
const Segment SGI179423[] = {{&S69420[27], 6}, {NULL}};
const Segment SGI179424[] = {{&S69420[112], 9}, {NULL}};
const Segment SGI179425[] = {{&S69420[85], 6}, {NULL}};
const Segment SGI179426[] = {{&S69421[34], 25}, {NULL}};
const Segment SGI179427[] = {{&S69421[27], 6}, {NULL}};
const Segment SGI179428[] = {{&S69421[112], 9}, {NULL}};
const Segment SGI179429[] = {{&S69421[85], 6}, {NULL}};
const Segment SGI179430[] = {{&S69422[34], 25}, {NULL}};
const Segment SGI179431[] = {{&S69422[27], 6}, {NULL}};
const Segment SGI179432[] = {{&S69422[112], 9}, {NULL}};
const Segment SGI179433[] = {{&S69422[85], 6}, {NULL}};
const Segment SGI179434[] = {{&S69423[34], 25}, {NULL}};
const Segment SGI179435[] = {{&S69423[27], 6}, {NULL}};
const Segment SGI179436[] = {{&S69423[112], 9}, {NULL}};
const Segment SGI179437[] = {{&S69423[85], 6}, {NULL}};
const Segment SGI179438[] = {{&S69428[34], 10}, {NULL}};
const Segment SGI179439[] = {{&S69428[27], 6}, {NULL}};
const Segment SGI179440[] = {{&S69428[97], 9}, {NULL}};
const Segment SGI179441[] = {{&S69428[70], 6}, {NULL}};
const Segment SGI179442[] = {{&S47052[34], 18}, {NULL}};
const Segment SGI179443[] = {{&S47052[27], 6}, {NULL}};
const Segment SGI179444[] = {{&S47052[104], 9}, {NULL}};
const Segment SGI179445[] = {{&S47052[77], 6}, {NULL}};
const Segment SGI179446[] = {{&S69440[34], 11}, {NULL}};
const Segment SGI179447[] = {{&S69440[27], 6}, {NULL}};
const Segment SGI179448[] = {{&S69440[98], 9}, {NULL}};
const Segment SGI179449[] = {{&S69440[71], 6}, {NULL}};
const Segment SGI179450[] = {{&S62063[44], 24}, {NULL}};
const Segment SGI179451[] = {{&S62063[44], 36}, {NULL}};
const Segment SGI179452[] = {{&S62063[30], 6}, {NULL}};
const Segment SGI179453[] = {{&S62063[129], 3}, {NULL}};
const Segment SGI179454[] = {{&S62063[94], 6}, {NULL}};
const Segment SGI179455[] = {{&S69441[34], 10}, {NULL}};
const Segment SGI179456[] = {{&S69441[27], 6}, {NULL}};
const Segment SGI179457[] = {{&S69441[97], 9}, {NULL}};
const Segment SGI179458[] = {{&S69441[70], 6}, {NULL}};
const Segment SGI179459[] = {{&S62064[38], 24}, {NULL}};
const Segment SGI179460[] = {{&S62064[31], 6}, {NULL}};
const Segment SGI179461[] = {{&S62067[44], 10}, {NULL}};
const Segment SGI179462[] = {{&S62067[44], 27}, {NULL}};
const Segment SGI179463[] = {{&S62067[30], 6}, {NULL}};
const Segment SGI179464[] = {{&S62067[126], 3}, {NULL}};
const Segment SGI179465[] = {{&S62067[92], 5}, {NULL}};
const Segment SGI179466[] = {{&S62067[71], 8}, {NULL}};
const Segment SGI179467[] = {{&S69447[44], 6}, {NULL}};
const Segment SGI179468[] = {{&S69447[44], 19}, {NULL}};
const Segment SGI179469[] = {{&S69447[30], 6}, {NULL}};
const Segment SGI179470[] = {{&S69447[31], 3}, {NULL}};
const Segment SGI179471[] = {{&S69447[77], 6}, {NULL}};
const Segment SGI179472[] = {{&S69448[44], 6}, {NULL}};
const Segment SGI179473[] = {{&S69448[44], 18}, {NULL}};
const Segment SGI179474[] = {{&S69448[30], 6}, {NULL}};
const Segment SGI179475[] = {{&S69448[31], 3}, {NULL}};
const Segment SGI179476[] = {{&S69448[76], 6}, {NULL}};
const Segment SGI179477[] = {{&S69449[44], 6}, {NULL}};
const Segment SGI179478[] = {{&S69449[44], 16}, {NULL}};
const Segment SGI179479[] = {{&S69449[30], 6}, {NULL}};
const Segment SGI179480[] = {{&S69449[31], 3}, {NULL}};
const Segment SGI179481[] = {{&S69449[74], 6}, {NULL}};
const Segment SGI179482[] = {{&S62073[44], 6}, {NULL}};
const Segment SGI179483[] = {{&S62073[44], 18}, {NULL}};
const Segment SGI179484[] = {{&S62073[30], 6}, {NULL}};
const Segment SGI179485[] = {{&S62073[111], 3}, {NULL}};
const Segment SGI179486[] = {{&S62073[76], 6}, {NULL}};
const Segment SGI179487[] = {{&S69450[44], 6}, {NULL}};
const Segment SGI179488[] = {{&S69450[44], 18}, {NULL}};
const Segment SGI179489[] = {{&S69450[30], 6}, {NULL}};
const Segment SGI179490[] = {{&S69450[31], 3}, {NULL}};
const Segment SGI179491[] = {{&S69450[76], 6}, {NULL}};
const Segment SGI179492[] = {{&S62074[44], 7}, {NULL}};
const Segment SGI179493[] = {{&S62074[44], 19}, {NULL}};
const Segment SGI179494[] = {{&S62074[30], 6}, {NULL}};
const Segment SGI179495[] = {{&S62074[112], 3}, {NULL}};
const Segment SGI179496[] = {{&S62074[77], 6}, {NULL}};
const Segment SGI179497[] = {{&S69451[34], 10}, {NULL}};
const Segment SGI179498[] = {{&S69451[27], 6}, {NULL}};
const Segment SGI179499[] = {{&S69451[97], 9}, {NULL}};
const Segment SGI179500[] = {{&S69451[70], 6}, {NULL}};
const Segment SGI179501[] = {{&S69452[44], 13}, {NULL}};
const Segment SGI179502[] = {{&S69452[44], 25}, {NULL}};
const Segment SGI179503[] = {{&S69452[30], 6}, {NULL}};
const Segment SGI179504[] = {{&S69452[31], 3}, {NULL}};
const Segment SGI179505[] = {{&S69452[83], 6}, {NULL}};
const Segment SGI179506[] = {{&S69453[44], 13}, {NULL}};
const Segment SGI179507[] = {{&S69453[44], 25}, {NULL}};
const Segment SGI179508[] = {{&S69453[30], 6}, {NULL}};
const Segment SGI179509[] = {{&S69453[31], 3}, {NULL}};
const Segment SGI179510[] = {{&S69453[83], 6}, {NULL}};
const Segment SGI179511[] = {{&S69454[34], 10}, {NULL}};
const Segment SGI179512[] = {{&S69454[27], 6}, {NULL}};
const Segment SGI179513[] = {{&S69454[97], 9}, {NULL}};
const Segment SGI179514[] = {{&S69454[70], 6}, {NULL}};
const Segment SGI179515[] = {{&S69469[44], 10}, {NULL}};
const Segment SGI179516[] = {{&S69469[44], 22}, {NULL}};
const Segment SGI179517[] = {{&S69469[30], 6}, {NULL}};
const Segment SGI179518[] = {{&S69469[31], 3}, {NULL}};
const Segment SGI179519[] = {{&S69469[99], 6}, {NULL}};
const Segment SGI179520[] = {{&S69469[68], 8}, {NULL}};
const Segment SGI179521[] = {{&S69473[34], 7}, {NULL}};
const Segment SGI179522[] = {{&S69473[27], 6}, {NULL}};
const Segment SGI179523[] = {{&S69473[110], 9}, {NULL}};
const Segment SGI179524[] = {{&S69473[83], 6}, {NULL}};
const Segment SGI179525[] = {{&S69474[34], 7}, {NULL}};
const Segment SGI179526[] = {{&S69474[27], 6}, {NULL}};
const Segment SGI179527[] = {{&S69474[110], 9}, {NULL}};
const Segment SGI179528[] = {{&S69474[83], 6}, {NULL}};
const Segment SGI179529[] = {{&S69475[34], 7}, {NULL}};
const Segment SGI179530[] = {{&S69475[27], 6}, {NULL}};
const Segment SGI179531[] = {{&S69475[103], 9}, {NULL}};
const Segment SGI179532[] = {{&S69475[76], 6}, {NULL}};
const Segment SGI179533[] = {{&S69476[34], 7}, {NULL}};
const Segment SGI179534[] = {{&S69476[27], 6}, {NULL}};
const Segment SGI179535[] = {{&S69476[103], 9}, {NULL}};
const Segment SGI179536[] = {{&S69476[76], 6}, {NULL}};
const Segment SGI179537[] = {{&S62118[44], 16}, {NULL}};
const Segment SGI179538[] = {{&S62118[44], 28}, {NULL}};
const Segment SGI179539[] = {{&S62118[30], 6}, {NULL}};
const Segment SGI179540[] = {{&S62118[121], 3}, {NULL}};
const Segment SGI179541[] = {{&S62118[86], 6}, {NULL}};
const Segment SGI179542[] = {{&S62119[44], 10}, {NULL}};
const Segment SGI179543[] = {{&S62119[44], 21}, {NULL}};
const Segment SGI179544[] = {{&S62119[30], 6}, {NULL}};
const Segment SGI179545[] = {{&S62119[114], 3}, {NULL}};
const Segment SGI179546[] = {{&S62119[79], 6}, {NULL}};
const Segment SGI179547[] = {{&S62122[44], 6}, {NULL}};
const Segment SGI179548[] = {{&S62122[44], 18}, {NULL}};
const Segment SGI179549[] = {{&S62122[30], 6}, {NULL}};
const Segment SGI179550[] = {{&S62122[76], 6}, {NULL}};
const Segment SGI179551[] = {{&S62122[103], 13}, {NULL}};
const Segment SGI179552[] = {{&S62123[44], 7}, {NULL}};
const Segment SGI179553[] = {{&S62123[44], 19}, {NULL}};
const Segment SGI179554[] = {{&S62123[30], 6}, {NULL}};
const Segment SGI179555[] = {{&S62123[77], 6}, {NULL}};
const Segment SGI179556[] = {{&S62123[104], 13}, {NULL}};
const Segment SGI179557[] = {{&S62124[44], 8}, {NULL}};
const Segment SGI179558[] = {{&S62124[44], 20}, {NULL}};
const Segment SGI179559[] = {{&S62124[30], 6}, {NULL}};
const Segment SGI179560[] = {{&S62124[78], 6}, {NULL}};
const Segment SGI179561[] = {{&S62124[105], 13}, {NULL}};
const Segment SGI179562[] = {{&S62125[44], 7}, {NULL}};
const Segment SGI179563[] = {{&S62125[44], 19}, {NULL}};
const Segment SGI179564[] = {{&S62125[30], 6}, {NULL}};
const Segment SGI179565[] = {{&S62125[77], 6}, {NULL}};
const Segment SGI179566[] = {{&S62125[104], 13}, {NULL}};
const Segment SGI179567[] = {{&S62126[44], 7}, {NULL}};
const Segment SGI179568[] = {{&S62126[44], 19}, {NULL}};
const Segment SGI179569[] = {{&S62126[30], 6}, {NULL}};
const Segment SGI179570[] = {{&S62126[77], 6}, {NULL}};
const Segment SGI179571[] = {{&S62126[104], 13}, {NULL}};
const Segment SGI179572[] = {{&S47137[34], 10}, {NULL}};
const Segment SGI179573[] = {{&S47137[27], 6}, {NULL}};
const Segment SGI179574[] = {{&S47137[96], 9}, {NULL}};
const Segment SGI179575[] = {{&S47137[69], 6}, {NULL}};
const Segment SGI179576[] = {{&S69514[44], 11}, {NULL}};
const Segment SGI179577[] = {{&S69514[44], 23}, {NULL}};
const Segment SGI179578[] = {{&S69514[30], 6}, {NULL}};
const Segment SGI179579[] = {{&S69514[31], 3}, {NULL}};
const Segment SGI179580[] = {{&S69514[81], 6}, {NULL}};
const Segment SGI179581[] = {{&S69515[38], 11}, {NULL}};
const Segment SGI179582[] = {{&S69515[31], 6}, {NULL}};
const Segment SGI179583[] = {{&S47157[34], 10}, {NULL}};
const Segment SGI179584[] = {{&S47157[27], 6}, {NULL}};
const Segment SGI179585[] = {{&S47157[96], 9}, {NULL}};
const Segment SGI179586[] = {{&S47157[69], 6}, {NULL}};
const Segment SGI179587[] = {{&S47158[34], 10}, {NULL}};
const Segment SGI179588[] = {{&S47158[27], 6}, {NULL}};
const Segment SGI179589[] = {{&S69520[34], 18}, {NULL}};
const Segment SGI179590[] = {{&S69520[27], 6}, {NULL}};
const Segment SGI179591[] = {{&S47158[96], 9}, {NULL}};
const Segment SGI179592[] = {{&S47158[69], 6}, {NULL}};
const Segment SGI179593[] = {{&S69520[105], 9}, {NULL}};
const Segment SGI179594[] = {{&S69520[78], 6}, {NULL}};
const Segment SGI179595[] = {{&S47159[34], 11}, {NULL}};
const Segment SGI179596[] = {{&S47159[27], 6}, {NULL}};
const Segment SGI179597[] = {{&S47159[97], 9}, {NULL}};
const Segment SGI179598[] = {{&S47159[70], 6}, {NULL}};
const Segment SGI179599[] = {{&S69527[34], 14}, {NULL}};
const Segment SGI179600[] = {{&S69527[27], 6}, {NULL}};
const Segment SGI179601[] = {{&S69527[114], 9}, {NULL}};
const Segment SGI179602[] = {{&S69527[87], 6}, {NULL}};
const Segment SGI179603[] = {{&S69529[34], 10}, {NULL}};
const Segment SGI179604[] = {{&S69529[27], 6}, {NULL}};
const Segment SGI179605[] = {{&S69529[97], 9}, {NULL}};
const Segment SGI179606[] = {{&S69529[70], 6}, {NULL}};
const Segment SGI179607[] = {{&S69530[34], 6}, {NULL}};
const Segment SGI179608[] = {{&S69530[27], 6}, {NULL}};
const Segment SGI179609[] = {{&S69530[93], 9}, {NULL}};
const Segment SGI179610[] = {{&S69530[66], 6}, {NULL}};
const Segment SGI179611[] = {{&S69536[44], 10}, {NULL}};
const Segment SGI179612[] = {{&S69536[44], 22}, {NULL}};
const Segment SGI179613[] = {{&S69536[30], 6}, {NULL}};
const Segment SGI179614[] = {{&S69536[80], 6}, {NULL}};
const Segment SGI179615[] = {{&S69536[107], 13}, {NULL}};
const Segment SGI179616[] = {{&S69546[44], 11}, {NULL}};
const Segment SGI179617[] = {{&S69546[44], 23}, {NULL}};
const Segment SGI179618[] = {{&S69546[30], 6}, {NULL}};
const Segment SGI179619[] = {{&S69546[31], 3}, {NULL}};
const Segment SGI179620[] = {{&S69546[81], 6}, {NULL}};
const Segment SGI179621[] = {{&S69550[34], 7}, {NULL}};
const Segment SGI179622[] = {{&S69550[27], 6}, {NULL}};
const Segment SGI179623[] = {{&S69550[100], 9}, {NULL}};
const Segment SGI179624[] = {{&S69550[73], 6}, {NULL}};
const Segment SGI179625[] = {{&S69551[34], 7}, {NULL}};
const Segment SGI179626[] = {{&S69551[27], 6}, {NULL}};
const Segment SGI179627[] = {{&S69551[100], 9}, {NULL}};
const Segment SGI179628[] = {{&S69551[73], 6}, {NULL}};
const Segment SGI179629[] = {{&S62206[44], 5}, {NULL}};
const Segment SGI179630[] = {{&S62206[44], 17}, {NULL}};
const Segment SGI179631[] = {{&S62206[30], 6}, {NULL}};
const Segment SGI179632[] = {{&S62206[110], 3}, {NULL}};
const Segment SGI179633[] = {{&S62206[75], 6}, {NULL}};
const Segment SGI179634[] = {{&S62207[38], 5}, {NULL}};
const Segment SGI179635[] = {{&S62207[31], 6}, {NULL}};
const Segment SGI179636[] = {{&S69568[34], 9}, {NULL}};
const Segment SGI179637[] = {{&S69568[27], 6}, {NULL}};
const Segment SGI179638[] = {{&S69568[96], 9}, {NULL}};
const Segment SGI179639[] = {{&S69568[69], 6}, {NULL}};
const Segment SGI179640[] = {{&S69569[34], 10}, {NULL}};
const Segment SGI179641[] = {{&S69569[27], 6}, {NULL}};
const Segment SGI179642[] = {{&S69569[97], 9}, {NULL}};
const Segment SGI179643[] = {{&S69569[70], 6}, {NULL}};
const Segment SGI179644[] = {{&S58242[44], 6}, {NULL}};
const Segment SGI179645[] = {{&S58242[44], 18}, {NULL}};
const Segment SGI179646[] = {{&S58242[30], 6}, {NULL}};
const Segment SGI179647[] = {{&S58242[149], 3}, {NULL}};
const Segment SGI179648[] = {{&S58242[114], 6}, {NULL}};
const Segment SGI179649[] = {{&S58242[64], 8}, {NULL}};
const Segment SGI179650[] = {{&S69597[34], 10}, {NULL}};
const Segment SGI179651[] = {{&S69597[27], 6}, {NULL}};
const Segment SGI179652[] = {{&S69597[97], 9}, {NULL}};
const Segment SGI179653[] = {{&S69597[70], 6}, {NULL}};
const Segment SGI179654[] = {{&S69597[138], 8}, {NULL}};
const Segment SGI179655[] = {{&S56292[38], 10}, {NULL}};
const Segment SGI179656[] = {{&S69598[34], 10}, {NULL}};
const Segment SGI179657[] = {{&S56292[31], 6}, {NULL}};
const Segment SGI179658[] = {{&S69598[27], 6}, {NULL}};
const Segment SGI179659[] = {{&S69598[97], 9}, {NULL}};
const Segment SGI179660[] = {{&S69598[70], 6}, {NULL}};
const Segment SGI179661[] = {{&S69598[138], 8}, {NULL}};
const Segment SGI179662[] = {{&S56293[44], 10}, {NULL}};
const Segment SGI179663[] = {{&S56293[44], 22}, {NULL}};
const Segment SGI179664[] = {{&S56293[30], 6}, {NULL}};
const Segment SGI179665[] = {{&S56293[114], 3}, {NULL}};
const Segment SGI179666[] = {{&S56293[80], 6}, {NULL}};
const Segment SGI179667[] = {{&S69613[34], 8}, {NULL}};
const Segment SGI179668[] = {{&S69613[27], 6}, {NULL}};
const Segment SGI179669[] = {{&S69613[100], 9}, {NULL}};
const Segment SGI179670[] = {{&S69613[73], 6}, {NULL}};
const Segment SGI179671[] = {{&S58278[34], 9}, {NULL}};
const Segment SGI179672[] = {{&S58278[27], 6}, {NULL}};
const Segment SGI179673[] = {{&S58278[96], 9}, {NULL}};
const Segment SGI179674[] = {{&S58278[69], 6}, {NULL}};
const Segment SGI179675[] = {{&S56322[34], 8}, {NULL}};
const Segment SGI179676[] = {{&S56322[27], 6}, {NULL}};
const Segment SGI179677[] = {{&S56322[95], 9}, {NULL}};
const Segment SGI179678[] = {{&S69625[44], 10}, {NULL}};
const Segment SGI179679[] = {{&S56322[68], 6}, {NULL}};
const Segment SGI179680[] = {{&S69625[44], 22}, {NULL}};
const Segment SGI179681[] = {{&S69625[30], 6}, {NULL}};
const Segment SGI179682[] = {{&S69625[31], 3}, {NULL}};
const Segment SGI179683[] = {{&S69625[80], 6}, {NULL}};
const Segment SGI179684[] = {{&S69628[34], 13}, {NULL}};
const Segment SGI179685[] = {{&S69628[27], 6}, {NULL}};
const Segment SGI179686[] = {{&S69628[100], 9}, {NULL}};
const Segment SGI179687[] = {{&S69628[73], 6}, {NULL}};
const Segment SGI179688[] = {{&S62289[44], 10}, {NULL}};
const Segment SGI179689[] = {{&S62289[44], 22}, {NULL}};
const Segment SGI179690[] = {{&S62289[30], 6}, {NULL}};
const Segment SGI179691[] = {{&S62289[115], 3}, {NULL}};
const Segment SGI179692[] = {{&S69630[34], 10}, {NULL}};
const Segment SGI179693[] = {{&S62289[80], 6}, {NULL}};
const Segment SGI179694[] = {{&S69630[27], 6}, {NULL}};
const Segment SGI179695[] = {{&S62289[139], 10}, {NULL}};
const Segment SGI179696[] = {{&S69630[97], 9}, {NULL}};
const Segment SGI179697[] = {{&S69630[70], 6}, {NULL}};
const Segment SGI179698[] = {{&S62290[44], 10}, {NULL}};
const Segment SGI179699[] = {{&S62290[44], 22}, {NULL}};
const Segment SGI179700[] = {{&S62290[30], 6}, {NULL}};
const Segment SGI179701[] = {{&S62290[115], 3}, {NULL}};
const Segment SGI179702[] = {{&S62290[80], 6}, {NULL}};
const Segment SGI179703[] = {{&S62290[139], 10}, {NULL}};
const Segment SGI179704[] = {{&S69637[34], 7}, {NULL}};
const Segment SGI179705[] = {{&S69637[27], 6}, {NULL}};
const Segment SGI179706[] = {{&S69637[99], 9}, {NULL}};
const Segment SGI179707[] = {{&S69637[72], 6}, {NULL}};
const Segment SGI179708[] = {{&S69638[34], 7}, {NULL}};
const Segment SGI179709[] = {{&S69638[27], 6}, {NULL}};
const Segment SGI179710[] = {{&S69638[99], 9}, {NULL}};
const Segment SGI179711[] = {{&S69638[72], 6}, {NULL}};
const Segment SGI179712[] = {{&S69641[44], 11}, {NULL}};
const Segment SGI179713[] = {{&S69641[44], 23}, {NULL}};
const Segment SGI179714[] = {{&S69641[30], 6}, {NULL}};
const Segment SGI179715[] = {{&S69641[31], 3}, {NULL}};
const Segment SGI179716[] = {{&S69641[81], 6}, {NULL}};
const Segment SGI179717[] = {{&S69643[44], 7}, {NULL}};
const Segment SGI179718[] = {{&S69643[44], 15}, {NULL}};
const Segment SGI179719[] = {{&S69643[30], 6}, {NULL}};
const Segment SGI179720[] = {{&S69643[31], 3}, {NULL}};
const Segment SGI179721[] = {{&S69643[73], 6}, {NULL}};
const Segment SGI179722[] = {{&S69644[38], 7}, {NULL}};
const Segment SGI179723[] = {{&S69644[31], 6}, {NULL}};
const Segment SGI179724[] = {{&S69649[52], 6}, {NULL}};
const Segment SGI179725[] = {{&S69649[114], 6}, {NULL}};
const Segment SGI179726[] = {{&S69650[44], 14}, {NULL}};
const Segment SGI179727[] = {{&S69650[44], 31}, {NULL}};
const Segment SGI179728[] = {{&S62313[34], 10}, {NULL}};
const Segment SGI179729[] = {{&S69650[30], 6}, {NULL}};
const Segment SGI179730[] = {{&S62313[27], 6}, {NULL}};
const Segment SGI179731[] = {{&S69650[31], 3}, {NULL}};
const Segment SGI179732[] = {{&S69650[89], 6}, {NULL}};
const Segment SGI179733[] = {{&S62313[97], 9}, {NULL}};
const Segment SGI179734[] = {{&S62313[70], 6}, {NULL}};
const Segment SGI179735[] = {{&S69651[44], 14}, {NULL}};
const Segment SGI179736[] = {{&S69651[44], 31}, {NULL}};
const Segment SGI179737[] = {{&S69651[30], 6}, {NULL}};
const Segment SGI179738[] = {{&S69651[92], 6}, {NULL}};
const Segment SGI179739[] = {{&S69652[38], 14}, {NULL}};
const Segment SGI179740[] = {{&S69652[31], 6}, {NULL}};
const Segment SGI179741[] = {{&S58328[34], 9}, {NULL}};
const Segment SGI179742[] = {{&S58328[27], 6}, {NULL}};
const Segment SGI179743[] = {{&S58328[96], 9}, {NULL}};
const Segment SGI179744[] = {{&S58328[69], 6}, {NULL}};
const Segment SGI179745[] = {{&S58334[34], 16}, {NULL}};
const Segment SGI179746[] = {{&S58334[27], 6}, {NULL}};
const Segment SGI179747[] = {{&S58334[103], 9}, {NULL}};
const Segment SGI179748[] = {{&S58334[76], 6}, {NULL}};
const Segment SGI179749[] = {{&S58335[34], 7}, {NULL}};
const Segment SGI179750[] = {{&S58335[27], 6}, {NULL}};
const Segment SGI179751[] = {{&S58335[94], 9}, {NULL}};
const Segment SGI179752[] = {{&S58335[67], 6}, {NULL}};
const Segment SGI179753[] = {{&S58339[34], 6}, {NULL}};
const Segment SGI179754[] = {{&S58339[27], 6}, {NULL}};
const Segment SGI179755[] = {{&S58339[93], 9}, {NULL}};
const Segment SGI179756[] = {{&S58339[66], 6}, {NULL}};
const Segment SGI179757[] = {{&S58341[34], 8}, {NULL}};
const Segment SGI179758[] = {{&S58341[27], 6}, {NULL}};
const Segment SGI179759[] = {{&S58341[95], 9}, {NULL}};
const Segment SGI179760[] = {{&S58341[68], 6}, {NULL}};
const Segment SGI179761[] = {{&S69679[44], 13}, {NULL}};
const Segment SGI179762[] = {{&S69679[44], 29}, {NULL}};
const Segment SGI179763[] = {{&S69679[30], 6}, {NULL}};
const Segment SGI179764[] = {{&S58347[34], 7}, {NULL}};
const Segment SGI179765[] = {{&S58347[27], 6}, {NULL}};
const Segment SGI179766[] = {{&S69679[31], 3}, {NULL}};
const Segment SGI179767[] = {{&S69679[87], 6}, {NULL}};
const Segment SGI179768[] = {{&S58347[94], 9}, {NULL}};
const Segment SGI179769[] = {{&S58347[67], 6}, {NULL}};
const Segment SGI179770[] = {{&S58348[34], 4}, {NULL}};
const Segment SGI179771[] = {{&S58348[27], 6}, {NULL}};
const Segment SGI179772[] = {{&S58348[91], 9}, {NULL}};
const Segment SGI179773[] = {{&S58348[64], 6}, {NULL}};
const Segment SGI179774[] = {{&S58350[34], 8}, {NULL}};
const Segment SGI179775[] = {{&S58350[27], 6}, {NULL}};
const Segment SGI179776[] = {{&S58350[95], 9}, {NULL}};
const Segment SGI179777[] = {{&S58350[68], 6}, {NULL}};
const Segment SGI179778[] = {{&S58352[34], 6}, {NULL}};
const Segment SGI179779[] = {{&S58352[27], 6}, {NULL}};
const Segment SGI179780[] = {{&S62353[44], 14}, {NULL}};
const Segment SGI179781[] = {{&S62353[44], 26}, {NULL}};
const Segment SGI179782[] = {{&S62353[30], 6}, {NULL}};
const Segment SGI179783[] = {{&S58352[93], 9}, {NULL}};
const Segment SGI179784[] = {{&S58352[66], 6}, {NULL}};
const Segment SGI179785[] = {{&S62353[119], 3}, {NULL}};
const Segment SGI179786[] = {{&S62353[84], 6}, {NULL}};
const Segment SGI179787[] = {{&S69689[34], 9}, {NULL}};
const Segment SGI179788[] = {{&S69689[27], 6}, {NULL}};
const Segment SGI179789[] = {{&S69689[96], 9}, {NULL}};
const Segment SGI179790[] = {{&S69689[69], 6}, {NULL}};
const Segment SGI179791[] = {{&S62354[44], 25}, {NULL}};
const Segment SGI179792[] = {{&S62354[44], 37}, {NULL}};
const Segment SGI179793[] = {{&S62354[30], 6}, {NULL}};
const Segment SGI179794[] = {{&S62354[31], 3}, {NULL}};
const Segment SGI179795[] = {{&S62354[95], 6}, {NULL}};
const Segment SGI179796[] = {{&S69690[34], 11}, {NULL}};
const Segment SGI179797[] = {{&S69690[27], 6}, {NULL}};
const Segment SGI179798[] = {{&S69690[98], 9}, {NULL}};
const Segment SGI179799[] = {{&S69690[71], 6}, {NULL}};
const Segment SGI179800[] = {{&S58357[34], 6}, {NULL}};
const Segment SGI179801[] = {{&S58357[27], 6}, {NULL}};
const Segment SGI179802[] = {{&S58357[93], 9}, {NULL}};
const Segment SGI179803[] = {{&S58357[66], 6}, {NULL}};
const Segment SGI179804[] = {{&S58358[34], 6}, {NULL}};
const Segment SGI179805[] = {{&S58358[27], 6}, {NULL}};
const Segment SGI179806[] = {{&S58358[93], 9}, {NULL}};
const Segment SGI179807[] = {{&S58358[66], 6}, {NULL}};
const Segment SGI179808[] = {{&S58360[34], 6}, {NULL}};
const Segment SGI179809[] = {{&S58360[27], 6}, {NULL}};
const Segment SGI179810[] = {{&S58360[93], 9}, {NULL}};
const Segment SGI179811[] = {{&S58360[66], 6}, {NULL}};
const Segment SGI179812[] = {{&S69708[44], 9}, {NULL}};
const Segment SGI179813[] = {{&S69708[44], 28}, {NULL}};
const Segment SGI179814[] = {{&S69708[30], 6}, {NULL}};
const Segment SGI179815[] = {{&S69708[31], 3}, {NULL}};
const Segment SGI179816[] = {{&S69708[86], 6}, {NULL}};
const Segment SGI179817[] = {{&S69730[34], 10}, {NULL}};
const Segment SGI179818[] = {{&S69730[27], 6}, {NULL}};
const Segment SGI179819[] = {{&S69730[97], 9}, {NULL}};
const Segment SGI179820[] = {{&S69730[70], 6}, {NULL}};
const Segment SGI179821[] = {{&S62401[34], 8}, {NULL}};
const Segment SGI179822[] = {{&S62401[27], 6}, {NULL}};
const Segment SGI179823[] = {{&S62401[95], 9}, {NULL}};
const Segment SGI179824[] = {{&S62401[68], 6}, {NULL}};
const Segment SGI179825[] = {{&S69737[34], 7}, {NULL}};
const Segment SGI179826[] = {{&S69737[27], 6}, {NULL}};
const Segment SGI179827[] = {{&S69737[105], 9}, {NULL}};
const Segment SGI179828[] = {{&S69737[78], 6}, {NULL}};
const Segment SGI179829[] = {{&S69754[34], 10}, {NULL}};
const Segment SGI179830[] = {{&S69754[27], 6}, {NULL}};
const Segment SGI179831[] = {{&S69754[97], 9}, {NULL}};
const Segment SGI179832[] = {{&S69754[70], 6}, {NULL}};
const Segment SGI179833[] = {{&S69759[34], 10}, {NULL}};
const Segment SGI179834[] = {{&S69759[27], 6}, {NULL}};
const Segment SGI179835[] = {{&S69759[97], 9}, {NULL}};
const Segment SGI179836[] = {{&S69759[70], 6}, {NULL}};
const Segment SGI179837[] = {{&S62440[44], 16}, {NULL}};
const Segment SGI179838[] = {{&S62440[44], 28}, {NULL}};
const Segment SGI179839[] = {{&S62440[30], 6}, {NULL}};
const Segment SGI179840[] = {{&S62440[121], 3}, {NULL}};
const Segment SGI179841[] = {{&S62440[86], 6}, {NULL}};
const Segment SGI179842[] = {{&S62441[38], 16}, {NULL}};
const Segment SGI179843[] = {{&S62441[31], 6}, {NULL}};
const Segment SGI179844[] = {{&S69782[34], 10}, {NULL}};
const Segment SGI179845[] = {{&S69782[27], 6}, {NULL}};
const Segment SGI179846[] = {{&S69782[97], 9}, {NULL}};
const Segment SGI179847[] = {{&S69782[70], 6}, {NULL}};
const Segment SGI179848[] = {{&S47468[44], 10}, {NULL}};
const Segment SGI179849[] = {{&S47468[44], 21}, {NULL}};
const Segment SGI179850[] = {{&S47468[30], 6}, {NULL}};
const Segment SGI179851[] = {{&S47468[132], 3}, {NULL}};
const Segment SGI179852[] = {{&S69783[34], 9}, {NULL}};
const Segment SGI179853[] = {{&S47468[97], 6}, {NULL}};
const Segment SGI179854[] = {{&S69783[27], 6}, {NULL}};
const Segment SGI179855[] = {{&S47468[67], 8}, {NULL}};
const Segment SGI179856[] = {{&S69783[98], 9}, {NULL}};
const Segment SGI179857[] = {{&S69783[71], 6}, {NULL}};
const Segment SGI179858[] = {{&S47469[44], 10}, {NULL}};
const Segment SGI179859[] = {{&S47469[44], 21}, {NULL}};
const Segment SGI179860[] = {{&S47469[30], 6}, {NULL}};
const Segment SGI179861[] = {{&S47469[132], 3}, {NULL}};
const Segment SGI179862[] = {{&S47469[97], 6}, {NULL}};
const Segment SGI179863[] = {{&S47469[67], 8}, {NULL}};
const Segment SGI179864[] = {{&S47470[44], 10}, {NULL}};
const Segment SGI179865[] = {{&S47470[44], 21}, {NULL}};
const Segment SGI179866[] = {{&S47470[30], 6}, {NULL}};
const Segment SGI179867[] = {{&S47470[132], 3}, {NULL}};
const Segment SGI179868[] = {{&S47470[97], 6}, {NULL}};
const Segment SGI179869[] = {{&S47470[67], 8}, {NULL}};
const Segment SGI179870[] = {{&S47472[44], 10}, {NULL}};
const Segment SGI179871[] = {{&S47472[44], 22}, {NULL}};
const Segment SGI179872[] = {{&S47472[30], 6}, {NULL}};
const Segment SGI179873[] = {{&S47472[133], 3}, {NULL}};
const Segment SGI179874[] = {{&S47472[98], 6}, {NULL}};
const Segment SGI179875[] = {{&S47472[68], 8}, {NULL}};
const Segment SGI179876[] = {{&S47473[44], 10}, {NULL}};
const Segment SGI179877[] = {{&S47473[44], 22}, {NULL}};
const Segment SGI179878[] = {{&S47473[30], 6}, {NULL}};
const Segment SGI179879[] = {{&S47473[133], 3}, {NULL}};
const Segment SGI179880[] = {{&S47473[98], 6}, {NULL}};
const Segment SGI179881[] = {{&S47473[68], 8}, {NULL}};
const Segment SGI179882[] = {{&S47474[44], 10}, {NULL}};
const Segment SGI179883[] = {{&S47474[44], 21}, {NULL}};
const Segment SGI179884[] = {{&S47474[30], 6}, {NULL}};
const Segment SGI179885[] = {{&S47474[134], 3}, {NULL}};
const Segment SGI179886[] = {{&S47474[99], 6}, {NULL}};
const Segment SGI179887[] = {{&S47474[67], 8}, {NULL}};
const Segment SGI179888[] = {{&S47475[44], 18}, {NULL}};
const Segment SGI179889[] = {{&S47475[44], 29}, {NULL}};
const Segment SGI179890[] = {{&S47475[30], 6}, {NULL}};
const Segment SGI179891[] = {{&S47475[142], 3}, {NULL}};
const Segment SGI179892[] = {{&S47475[107], 6}, {NULL}};
const Segment SGI179893[] = {{&S47475[75], 8}, {NULL}};
const Segment SGI179894[] = {{&S47476[44], 18}, {NULL}};
const Segment SGI179895[] = {{&S47476[44], 30}, {NULL}};
const Segment SGI179896[] = {{&S47476[30], 6}, {NULL}};
const Segment SGI179897[] = {{&S47476[123], 3}, {NULL}};
const Segment SGI179898[] = {{&S47476[88], 6}, {NULL}};
const Segment SGI179899[] = {{&S47476[148], 8}, {NULL}};
const Segment SGI179900[] = {{&S69799[44], 17}, {NULL}};
const Segment SGI179901[] = {{&S69799[44], 29}, {NULL}};
const Segment SGI179902[] = {{&S69799[30], 6}, {NULL}};
const Segment SGI179903[] = {{&S69799[31], 3}, {NULL}};
const Segment SGI179904[] = {{&S69799[87], 6}, {NULL}};
const Segment SGI179905[] = {{&S47477[44], 10}, {NULL}};
const Segment SGI179906[] = {{&S62473[44], 11}, {NULL}};
const Segment SGI179907[] = {{&S47477[44], 22}, {NULL}};
const Segment SGI179908[] = {{&S62473[44], 23}, {NULL}};
const Segment SGI179909[] = {{&S47477[30], 6}, {NULL}};
const Segment SGI179910[] = {{&S62473[30], 6}, {NULL}};
const Segment SGI179911[] = {{&S47477[115], 3}, {NULL}};
const Segment SGI179912[] = {{&S62473[116], 3}, {NULL}};
const Segment SGI179913[] = {{&S47477[80], 6}, {NULL}};
const Segment SGI179914[] = {{&S62473[81], 6}, {NULL}};
const Segment SGI179915[] = {{&S47477[140], 8}, {NULL}};
const Segment SGI179916[] = {{&S47478[44], 10}, {NULL}};
const Segment SGI179917[] = {{&S47478[44], 22}, {NULL}};
const Segment SGI179918[] = {{&S47478[30], 6}, {NULL}};
const Segment SGI179919[] = {{&S47478[115], 3}, {NULL}};
const Segment SGI179920[] = {{&S47478[80], 6}, {NULL}};
const Segment SGI179921[] = {{&S47478[140], 8}, {NULL}};
const Segment SGI179922[] = {{&S47479[44], 10}, {NULL}};
const Segment SGI179923[] = {{&S47479[44], 22}, {NULL}};
const Segment SGI179924[] = {{&S47479[30], 6}, {NULL}};
const Segment SGI179925[] = {{&S47479[135], 3}, {NULL}};
const Segment SGI179926[] = {{&S47479[100], 6}, {NULL}};
const Segment SGI179927[] = {{&S47479[68], 8}, {NULL}};
const Segment SGI179928[] = {{&S47490[34], 29}, {NULL}};
const Segment SGI179929[] = {{&S47490[27], 6}, {NULL}};
const Segment SGI179930[] = {{&S47490[115], 9}, {NULL}};
const Segment SGI179931[] = {{&S47490[88], 6}, {NULL}};
const Segment SGI179932[] = {{&S62516[44], 11}, {NULL}};
const Segment SGI179933[] = {{&S62516[44], 23}, {NULL}};
const Segment SGI179934[] = {{&S62516[30], 6}, {NULL}};
const Segment SGI179935[] = {{&S62516[116], 3}, {NULL}};
const Segment SGI179936[] = {{&S62516[81], 6}, {NULL}};
const Segment SGI179937[] = {{&S58363[34], 10}, {NULL}};
const Segment SGI179938[] = {{&S58363[27], 6}, {NULL}};
const Segment SGI179939[] = {{&S58363[97], 9}, {NULL}};
const Segment SGI179940[] = {{&S58363[70], 6}, {NULL}};
const Segment SGI179941[] = {{&S62518[38], 11}, {NULL}};
const Segment SGI179942[] = {{&S62518[31], 6}, {NULL}};
const Segment SGI179943[] = {{&S58365[34], 6}, {NULL}};
const Segment SGI179944[] = {{&S58365[27], 6}, {NULL}};
const Segment SGI179945[] = {{&S58365[93], 9}, {NULL}};
const Segment SGI179946[] = {{&S58365[66], 6}, {NULL}};
const Segment SGI179947[] = {{&S69833[44], 9}, {NULL}};
const Segment SGI179948[] = {{&S69833[44], 21}, {NULL}};
const Segment SGI179949[] = {{&S69833[30], 6}, {NULL}};
const Segment SGI179950[] = {{&S69833[31], 3}, {NULL}};
const Segment SGI179951[] = {{&S69833[79], 6}, {NULL}};
const Segment SGI179952[] = {{&S58366[34], 16}, {NULL}};
const Segment SGI179953[] = {{&S58366[27], 6}, {NULL}};
const Segment SGI179954[] = {{&S58366[110], 9}, {NULL}};
const Segment SGI179955[] = {{&S58366[83], 6}, {NULL}};
const Segment SGI179956[] = {{&S69845[44], 7}, {NULL}};
const Segment SGI179957[] = {{&S69845[44], 23}, {NULL}};
const Segment SGI179958[] = {{&S69845[30], 6}, {NULL}};
const Segment SGI179959[] = {{&S69845[81], 6}, {NULL}};
const Segment SGI179960[] = {{&S69845[108], 13}, {NULL}};
const Segment SGI179961[] = {{&S69846[44], 7}, {NULL}};
const Segment SGI179962[] = {{&S69846[44], 23}, {NULL}};
const Segment SGI179963[] = {{&S69846[30], 6}, {NULL}};
const Segment SGI179964[] = {{&S69846[81], 6}, {NULL}};
const Segment SGI179965[] = {{&S58381[44], 19}, {NULL}};
const Segment SGI179966[] = {{&S58381[44], 31}, {NULL}};
const Segment SGI179967[] = {{&S69846[108], 13}, {NULL}};
const Segment SGI179968[] = {{&S58381[30], 6}, {NULL}};
const Segment SGI179969[] = {{&S58381[124], 3}, {NULL}};
const Segment SGI179970[] = {{&S58381[89], 6}, {NULL}};
const Segment SGI179971[] = {{&S69847[44], 7}, {NULL}};
const Segment SGI179972[] = {{&S69847[44], 24}, {NULL}};
const Segment SGI179973[] = {{&S69847[30], 6}, {NULL}};
const Segment SGI179974[] = {{&S58382[38], 19}, {NULL}};
const Segment SGI179975[] = {{&S58382[31], 6}, {NULL}};
const Segment SGI179976[] = {{&S69847[82], 6}, {NULL}};
const Segment SGI179977[] = {{&S69847[109], 13}, {NULL}};
const Segment SGI179978[] = {{&S58386[44], 10}, {NULL}};
const Segment SGI179979[] = {{&S58386[44], 22}, {NULL}};
const Segment SGI179980[] = {{&S58386[30], 6}, {NULL}};
const Segment SGI179981[] = {{&S58386[115], 3}, {NULL}};
const Segment SGI179982[] = {{&S58386[80], 6}, {NULL}};
const Segment SGI179983[] = {{&S58387[44], 20}, {NULL}};
const Segment SGI179984[] = {{&S58387[44], 32}, {NULL}};
const Segment SGI179985[] = {{&S58387[30], 6}, {NULL}};
const Segment SGI179986[] = {{&S58387[125], 3}, {NULL}};
const Segment SGI179987[] = {{&S58387[90], 6}, {NULL}};
const Segment SGI179988[] = {{&S58388[44], 15}, {NULL}};
const Segment SGI179989[] = {{&S58388[44], 27}, {NULL}};
const Segment SGI179990[] = {{&S58388[30], 6}, {NULL}};
const Segment SGI179991[] = {{&S58388[120], 3}, {NULL}};
const Segment SGI179992[] = {{&S58388[85], 6}, {NULL}};
const Segment SGI179993[] = {{&S58389[44], 15}, {NULL}};
const Segment SGI179994[] = {{&S58389[44], 27}, {NULL}};
const Segment SGI179995[] = {{&S58389[30], 6}, {NULL}};
const Segment SGI179996[] = {{&S69859[59], 7}, {NULL}};
const Segment SGI179997[] = {{&S69859[59], 19}, {NULL}};
const Segment SGI179998[] = {{&S69859[45], 6}, {NULL}};
const Segment SGI179999[] = {{&S58389[120], 3}, {NULL}};
const Segment SGI180000[] = {{&S58389[85], 6}, {NULL}};
const Segment SGI180001[] = {{&S69859[46], 3}, {NULL}};
const Segment SGI180002[] = {{&S69859[92], 6}, {NULL}};
const Segment SGI180003[] = {{&S69859[0], 14}, {NULL}};
const Segment SGI180004[] = {{&S58390[38], 10}, {NULL}};
const Segment SGI180005[] = {{&S58390[31], 6}, {NULL}};
const Segment SGI180006[] = {{&S58391[44], 15}, {NULL}};
const Segment SGI180007[] = {{&S58391[44], 27}, {NULL}};
const Segment SGI180008[] = {{&S58391[30], 6}, {NULL}};
const Segment SGI180009[] = {{&S58391[120], 3}, {NULL}};
const Segment SGI180010[] = {{&S58391[85], 6}, {NULL}};
const Segment SGI180011[] = {{&S58392[34], 11}, {NULL}};
const Segment SGI180012[] = {{&S58392[27], 6}, {NULL}};
const Segment SGI180013[] = {{&S58392[98], 9}, {NULL}};
const Segment SGI180014[] = {{&S58392[71], 6}, {NULL}};
const Segment SGI180015[] = {{&S58396[44], 11}, {NULL}};
const Segment SGI180016[] = {{&S58396[44], 23}, {NULL}};
const Segment SGI180017[] = {{&S58396[30], 6}, {NULL}};
const Segment SGI180018[] = {{&S58396[132], 3}, {NULL}};
const Segment SGI180019[] = {{&S58396[97], 6}, {NULL}};
const Segment SGI180020[] = {{&S58396[69], 8}, {NULL}};
const Segment SGI180021[] = {{&S69883[34], 13}, {NULL}};
const Segment SGI180022[] = {{&S69883[27], 6}, {NULL}};
const Segment SGI180023[] = {{&S69883[100], 9}, {NULL}};
const Segment SGI180024[] = {{&S69883[73], 6}, {NULL}};
const Segment SGI180025[] = {{&S69884[34], 9}, {NULL}};
const Segment SGI180026[] = {{&S69884[27], 6}, {NULL}};
const Segment SGI180027[] = {{&S69884[96], 9}, {NULL}};
const Segment SGI180028[] = {{&S69884[69], 6}, {NULL}};
const Segment SGI180029[] = {{&S69885[44], 14}, {NULL}};
const Segment SGI180030[] = {{&S69885[44], 26}, {NULL}};
const Segment SGI180031[] = {{&S69885[30], 6}, {NULL}};
const Segment SGI180032[] = {{&S69885[31], 3}, {NULL}};
const Segment SGI180033[] = {{&S69885[84], 6}, {NULL}};
const Segment SGI180034[] = {{&S69888[44], 7}, {NULL}};
const Segment SGI180035[] = {{&S69888[44], 19}, {NULL}};
const Segment SGI180036[] = {{&S69888[30], 6}, {NULL}};
const Segment SGI180037[] = {{&S69888[31], 3}, {NULL}};
const Segment SGI180038[] = {{&S69888[77], 6}, {NULL}};
const Segment SGI180039[] = {{&S69889[44], 11}, {NULL}};
const Segment SGI180040[] = {{&S69889[44], 23}, {NULL}};
const Segment SGI180041[] = {{&S69889[30], 6}, {NULL}};
const Segment SGI180042[] = {{&S69889[31], 3}, {NULL}};
const Segment SGI180043[] = {{&S69889[81], 6}, {NULL}};
const Segment SGI180044[] = {{&S69890[44], 11}, {NULL}};
const Segment SGI180045[] = {{&S69890[44], 23}, {NULL}};
const Segment SGI180046[] = {{&S69890[30], 6}, {NULL}};
const Segment SGI180047[] = {{&S69890[31], 3}, {NULL}};
const Segment SGI180048[] = {{&S69890[81], 6}, {NULL}};
const Segment SGI180049[] = {{&S62591[44], 10}, {NULL}};
const Segment SGI180050[] = {{&S62591[44], 22}, {NULL}};
const Segment SGI180051[] = {{&S62591[30], 6}, {NULL}};
const Segment SGI180052[] = {{&S62591[131], 3}, {NULL}};
const Segment SGI180053[] = {{&S62591[96], 6}, {NULL}};
const Segment SGI180054[] = {{&S69891[44], 11}, {NULL}};
const Segment SGI180055[] = {{&S62591[68], 8}, {NULL}};
const Segment SGI180056[] = {{&S69891[44], 23}, {NULL}};
const Segment SGI180057[] = {{&S69891[30], 6}, {NULL}};
const Segment SGI180058[] = {{&S69891[31], 3}, {NULL}};
const Segment SGI180059[] = {{&S69891[81], 6}, {NULL}};
const Segment SGI180060[] = {{&S62592[44], 10}, {NULL}};
const Segment SGI180061[] = {{&S62592[44], 22}, {NULL}};
const Segment SGI180062[] = {{&S62592[30], 6}, {NULL}};
const Segment SGI180063[] = {{&S62592[131], 3}, {NULL}};
const Segment SGI180064[] = {{&S62592[96], 6}, {NULL}};
const Segment SGI180065[] = {{&S69892[44], 16}, {NULL}};
const Segment SGI180066[] = {{&S69892[44], 28}, {NULL}};
const Segment SGI180067[] = {{&S62592[68], 8}, {NULL}};
const Segment SGI180068[] = {{&S69892[30], 6}, {NULL}};
const Segment SGI180069[] = {{&S69892[31], 3}, {NULL}};
const Segment SGI180070[] = {{&S69892[86], 6}, {NULL}};
const Segment SGI180071[] = {{&S69893[44], 20}, {NULL}};
const Segment SGI180072[] = {{&S69893[44], 32}, {NULL}};
const Segment SGI180073[] = {{&S69893[30], 6}, {NULL}};
const Segment SGI180074[] = {{&S62594[44], 10}, {NULL}};
const Segment SGI180075[] = {{&S62594[44], 22}, {NULL}};
const Segment SGI180076[] = {{&S62594[30], 6}, {NULL}};
const Segment SGI180077[] = {{&S69893[31], 3}, {NULL}};
const Segment SGI180078[] = {{&S69893[90], 6}, {NULL}};
const Segment SGI180079[] = {{&S62594[131], 3}, {NULL}};
const Segment SGI180080[] = {{&S62594[96], 6}, {NULL}};
const Segment SGI180081[] = {{&S62594[68], 8}, {NULL}};
const Segment SGI180082[] = {{&S62595[44], 10}, {NULL}};
const Segment SGI180083[] = {{&S62595[44], 22}, {NULL}};
const Segment SGI180084[] = {{&S62595[30], 6}, {NULL}};
const Segment SGI180085[] = {{&S62595[115], 3}, {NULL}};
const Segment SGI180086[] = {{&S62595[80], 6}, {NULL}};
const Segment SGI180087[] = {{&S62595[139], 10}, {NULL}};
const Segment SGI180088[] = {{&S62597[44], 18}, {NULL}};
const Segment SGI180089[] = {{&S62597[44], 30}, {NULL}};
const Segment SGI180090[] = {{&S62597[30], 6}, {NULL}};
const Segment SGI180091[] = {{&S62597[123], 3}, {NULL}};
const Segment SGI180092[] = {{&S62597[88], 6}, {NULL}};
const Segment SGI180093[] = {{&S62597[147], 10}, {NULL}};
const Segment SGI180094[] = {{&S62598[44], 11}, {NULL}};
const Segment SGI180095[] = {{&S62598[44], 23}, {NULL}};
const Segment SGI180096[] = {{&S62598[30], 6}, {NULL}};
const Segment SGI180097[] = {{&S62598[116], 3}, {NULL}};
const Segment SGI180098[] = {{&S62598[81], 6}, {NULL}};
const Segment SGI180099[] = {{&S62598[140], 10}, {NULL}};
const Segment SGI180100[] = {{&S62602[34], 9}, {NULL}};
const Segment SGI180101[] = {{&S62602[27], 6}, {NULL}};
const Segment SGI180102[] = {{&S62602[96], 9}, {NULL}};
const Segment SGI180103[] = {{&S62602[69], 6}, {NULL}};
const Segment SGI180104[] = {{&S69908[44], 9}, {NULL}};
const Segment SGI180105[] = {{&S69908[44], 21}, {NULL}};
const Segment SGI180106[] = {{&S69908[30], 6}, {NULL}};
const Segment SGI180107[] = {{&S69908[31], 3}, {NULL}};
const Segment SGI180108[] = {{&S69908[79], 6}, {NULL}};
const Segment SGI180109[] = {{&S69916[34], 9}, {NULL}};
const Segment SGI180110[] = {{&S69916[27], 6}, {NULL}};
const Segment SGI180111[] = {{&S69916[96], 9}, {NULL}};
const Segment SGI180112[] = {{&S69916[69], 6}, {NULL}};
const Segment SGI180113[] = {{&S69928[44], 19}, {NULL}};
const Segment SGI180114[] = {{&S58468[44], 10}, {NULL}};
const Segment SGI180115[] = {{&S69928[44], 31}, {NULL}};
const Segment SGI180116[] = {{&S58468[44], 22}, {NULL}};
const Segment SGI180117[] = {{&S69928[30], 6}, {NULL}};
const Segment SGI180118[] = {{&S58468[30], 6}, {NULL}};
const Segment SGI180119[] = {{&S69928[31], 3}, {NULL}};
const Segment SGI180120[] = {{&S58468[115], 3}, {NULL}};
const Segment SGI180121[] = {{&S69928[108], 6}, {NULL}};
const Segment SGI180122[] = {{&S58468[80], 6}, {NULL}};
const Segment SGI180123[] = {{&S69928[77], 8}, {NULL}};
const Segment SGI180124[] = {{&S58469[38], 10}, {NULL}};
const Segment SGI180125[] = {{&S58469[31], 6}, {NULL}};
const Segment SGI180126[] = {{&S69929[44], 19}, {NULL}};
const Segment SGI180127[] = {{&S69929[44], 31}, {NULL}};
const Segment SGI180128[] = {{&S69929[30], 6}, {NULL}};
const Segment SGI180129[] = {{&S69929[31], 3}, {NULL}};
const Segment SGI180130[] = {{&S69929[108], 6}, {NULL}};
const Segment SGI180131[] = {{&S69929[77], 8}, {NULL}};
const Segment SGI180132[] = {{&S58470[44], 10}, {NULL}};
const Segment SGI180133[] = {{&S58470[44], 22}, {NULL}};
const Segment SGI180134[] = {{&S58470[30], 6}, {NULL}};
const Segment SGI180135[] = {{&S58470[115], 3}, {NULL}};
const Segment SGI180136[] = {{&S58470[80], 6}, {NULL}};
const Segment SGI180137[] = {{&S69930[44], 13}, {NULL}};
const Segment SGI180138[] = {{&S69930[44], 25}, {NULL}};
const Segment SGI180139[] = {{&S69930[30], 6}, {NULL}};
const Segment SGI180140[] = {{&S69930[31], 3}, {NULL}};
const Segment SGI180141[] = {{&S69930[102], 6}, {NULL}};
const Segment SGI180142[] = {{&S69930[71], 8}, {NULL}};
const Segment SGI180143[] = {{&S58477[34], 18}, {NULL}};
const Segment SGI180144[] = {{&S62636[44], 7}, {NULL}};
const Segment SGI180145[] = {{&S62636[44], 19}, {NULL}};
const Segment SGI180146[] = {{&S58477[27], 6}, {NULL}};
const Segment SGI180147[] = {{&S62636[30], 6}, {NULL}};
const Segment SGI180148[] = {{&S62636[112], 3}, {NULL}};
const Segment SGI180149[] = {{&S58477[105], 9}, {NULL}};
const Segment SGI180150[] = {{&S58477[78], 6}, {NULL}};
const Segment SGI180151[] = {{&S62636[77], 6}, {NULL}};
const Segment SGI180152[] = {{&S62637[38], 7}, {NULL}};
const Segment SGI180153[] = {{&S58478[34], 10}, {NULL}};
const Segment SGI180154[] = {{&S62637[31], 6}, {NULL}};
const Segment SGI180155[] = {{&S58478[27], 6}, {NULL}};
const Segment SGI180156[] = {{&S58478[97], 9}, {NULL}};
const Segment SGI180157[] = {{&S58478[70], 6}, {NULL}};
const Segment SGI180158[] = {{&S62641[34], 29}, {NULL}};
const Segment SGI180159[] = {{&S62641[27], 6}, {NULL}};
const Segment SGI180160[] = {{&S62641[116], 9}, {NULL}};
const Segment SGI180161[] = {{&S62641[89], 6}, {NULL}};
const Segment SGI180162[] = {{&S62642[34], 30}, {NULL}};
const Segment SGI180163[] = {{&S62642[27], 6}, {NULL}};
const Segment SGI180164[] = {{&S62642[117], 9}, {NULL}};
const Segment SGI180165[] = {{&S62642[90], 6}, {NULL}};
const Segment SGI180166[] = {{&S62646[34], 8}, {NULL}};
const Segment SGI180167[] = {{&S62646[27], 6}, {NULL}};
const Segment SGI180168[] = {{&S62646[96], 9}, {NULL}};
const Segment SGI180169[] = {{&S62646[69], 6}, {NULL}};
const Segment SGI180170[] = {{&S62647[34], 8}, {NULL}};
const Segment SGI180171[] = {{&S62647[27], 6}, {NULL}};
const Segment SGI180172[] = {{&S69952[44], 10}, {NULL}};
const Segment SGI180173[] = {{&S69952[44], 22}, {NULL}};
const Segment SGI180174[] = {{&S69952[30], 6}, {NULL}};
const Segment SGI180175[] = {{&S62647[96], 9}, {NULL}};
const Segment SGI180176[] = {{&S62647[69], 6}, {NULL}};
const Segment SGI180177[] = {{&S69952[31], 3}, {NULL}};
const Segment SGI180178[] = {{&S69952[80], 6}, {NULL}};
const Segment SGI180179[] = {{&S62648[34], 18}, {NULL}};
const Segment SGI180180[] = {{&S62648[27], 6}, {NULL}};
const Segment SGI180181[] = {{&S62648[114], 9}, {NULL}};
const Segment SGI180182[] = {{&S62648[87], 6}, {NULL}};
const Segment SGI180183[] = {{&S69958[34], 15}, {NULL}};
const Segment SGI180184[] = {{&S69958[27], 6}, {NULL}};
const Segment SGI180185[] = {{&S69958[102], 9}, {NULL}};
const Segment SGI180186[] = {{&S69958[75], 6}, {NULL}};
const Segment SGI180187[] = {{&S69985[34], 18}, {NULL}};
const Segment SGI180188[] = {{&S69985[27], 6}, {NULL}};
const Segment SGI180189[] = {{&S69985[105], 9}, {NULL}};
const Segment SGI180190[] = {{&S69985[78], 6}, {NULL}};
const Segment SGI180191[] = {{&S69988[34], 9}, {NULL}};
const Segment SGI180192[] = {{&S69988[27], 6}, {NULL}};
const Segment SGI180193[] = {{&S69988[96], 9}, {NULL}};
const Segment SGI180194[] = {{&S69988[69], 6}, {NULL}};
const Segment SGI180195[] = {{&S69991[34], 12}, {NULL}};
const Segment SGI180196[] = {{&S69991[27], 6}, {NULL}};
const Segment SGI180197[] = {{&S69991[99], 9}, {NULL}};
const Segment SGI180198[] = {{&S69991[72], 6}, {NULL}};
const Segment SGI180199[] = {{&S58536[34], 12}, {NULL}};
const Segment SGI180200[] = {{&S58536[27], 6}, {NULL}};
const Segment SGI180201[] = {{&S58536[99], 9}, {NULL}};
const Segment SGI180202[] = {{&S58536[72], 6}, {NULL}};
const Segment SGI180203[] = {{&S70004[44], 10}, {NULL}};
const Segment SGI180204[] = {{&S70004[44], 22}, {NULL}};
const Segment SGI180205[] = {{&S70004[30], 6}, {NULL}};
const Segment SGI180206[] = {{&S70004[31], 3}, {NULL}};
const Segment SGI180207[] = {{&S70004[99], 6}, {NULL}};
const Segment SGI180208[] = {{&S70004[68], 8}, {NULL}};
const Segment SGI180209[] = {{&S62702[34], 7}, {NULL}};
const Segment SGI180210[] = {{&S62702[27], 6}, {NULL}};
const Segment SGI180211[] = {{&S70005[44], 10}, {NULL}};
const Segment SGI180212[] = {{&S70005[44], 22}, {NULL}};
const Segment SGI180213[] = {{&S70005[30], 6}, {NULL}};
const Segment SGI180214[] = {{&S62702[99], 9}, {NULL}};
const Segment SGI180215[] = {{&S62702[72], 6}, {NULL}};
const Segment SGI180216[] = {{&S70005[31], 3}, {NULL}};
const Segment SGI180217[] = {{&S70005[95], 6}, {NULL}};
const Segment SGI180218[] = {{&S70005[68], 8}, {NULL}};
const Segment SGI180219[] = {{&S70006[44], 10}, {NULL}};
const Segment SGI180220[] = {{&S70006[44], 22}, {NULL}};
const Segment SGI180221[] = {{&S70006[30], 6}, {NULL}};
const Segment SGI180222[] = {{&S70006[31], 3}, {NULL}};
const Segment SGI180223[] = {{&S70006[95], 6}, {NULL}};
const Segment SGI180224[] = {{&S70006[68], 8}, {NULL}};
const Segment SGI180225[] = {{&S70007[44], 10}, {NULL}};
const Segment SGI180226[] = {{&S70007[44], 22}, {NULL}};
const Segment SGI180227[] = {{&S70007[30], 6}, {NULL}};
const Segment SGI180228[] = {{&S70007[31], 3}, {NULL}};
const Segment SGI180229[] = {{&S70007[95], 6}, {NULL}};
const Segment SGI180230[] = {{&S70007[68], 8}, {NULL}};
const Segment SGI180231[] = {{&S70008[44], 10}, {&S70008[190], 18}, {NULL}};
const Segment SGI180232[] = {{&S70008[44], 22}, {NULL}};
const Segment SGI180233[] = {{&S70008[30], 6}, {NULL}};
const Segment SGI180234[] = {{&S70008[31], 3}, {NULL}};
const Segment SGI180235[] = {{&S70008[95], 6}, {NULL}};
const Segment SGI180236[] = {{&S70008[68], 8}, {NULL}};
const Segment SGI180237[] = {{&S70009[44], 16}, {NULL}};
const Segment SGI180238[] = {{&S70009[44], 28}, {NULL}};
const Segment SGI180239[] = {{&S70009[30], 6}, {NULL}};
const Segment SGI180240[] = {{&S70009[31], 3}, {NULL}};
const Segment SGI180241[] = {{&S70009[86], 6}, {NULL}};
const Segment SGI180242[] = {{&S56714[44], 11}, {NULL}};
const Segment SGI180243[] = {{&S56714[44], 23}, {NULL}};
const Segment SGI180244[] = {{&S56714[30], 6}, {NULL}};
const Segment SGI180245[] = {{&S56714[116], 3}, {NULL}};
const Segment SGI180246[] = {{&S56714[81], 6}, {NULL}};
const Segment SGI180247[] = {{&S70011[34], 10}, {NULL}};
const Segment SGI180248[] = {{&S70011[27], 6}, {NULL}};
const Segment SGI180249[] = {{&S70011[97], 9}, {NULL}};
const Segment SGI180250[] = {{&S70011[70], 6}, {NULL}};
const Segment SGI180251[] = {{&S56715[44], 11}, {NULL}};
const Segment SGI180252[] = {{&S56715[44], 23}, {NULL}};
const Segment SGI180253[] = {{&S56715[30], 6}, {NULL}};
const Segment SGI180254[] = {{&S56715[116], 3}, {NULL}};
const Segment SGI180255[] = {{&S56715[81], 6}, {NULL}};
const Segment SGI180256[] = {{&S56716[38], 11}, {NULL}};
const Segment SGI180257[] = {{&S56716[31], 6}, {NULL}};
const Segment SGI180258[] = {{&S58576[44], 10}, {NULL}};
const Segment SGI180259[] = {{&S58576[44], 22}, {NULL}};
const Segment SGI180260[] = {{&S58576[30], 6}, {NULL}};
const Segment SGI180261[] = {{&S70022[34], 8}, {NULL}};
const Segment SGI180262[] = {{&S70022[27], 6}, {NULL}};
const Segment SGI180263[] = {{&S58576[80], 6}, {NULL}};
const Segment SGI180264[] = {{&S70022[95], 9}, {NULL}};
const Segment SGI180265[] = {{&S58576[107], 13}, {NULL}};
const Segment SGI180266[] = {{&S70022[68], 6}, {NULL}};
const Segment SGI180267[] = {{&S70024[34], 12}, {NULL}};
const Segment SGI180268[] = {{&S58578[44], 10}, {NULL}};
const Segment SGI180269[] = {{&S70024[27], 6}, {NULL}};
const Segment SGI180270[] = {{&S58578[44], 22}, {NULL}};
const Segment SGI180271[] = {{&S58578[30], 6}, {NULL}};
const Segment SGI180272[] = {{&S70024[99], 9}, {NULL}};
const Segment SGI180273[] = {{&S70024[72], 6}, {NULL}};
const Segment SGI180274[] = {{&S58578[80], 6}, {NULL}};
const Segment SGI180275[] = {{&S58578[107], 13}, {NULL}};
const Segment SGI180276[] = {{&S58580[44], 10}, {NULL}};
const Segment SGI180277[] = {{&S58580[44], 22}, {NULL}};
const Segment SGI180278[] = {{&S58580[30], 6}, {NULL}};
const Segment SGI180279[] = {{&S58580[80], 6}, {NULL}};
const Segment SGI180280[] = {{&S58580[107], 13}, {NULL}};
const Segment SGI180281[] = {{&S58581[44], 10}, {NULL}};
const Segment SGI180282[] = {{&S58581[44], 22}, {NULL}};
const Segment SGI180283[] = {{&S58581[30], 6}, {NULL}};
const Segment SGI180284[] = {{&S58581[80], 6}, {NULL}};
const Segment SGI180285[] = {{&S58581[107], 13}, {NULL}};
const Segment SGI180286[] = {{&S58589[34], 10}, {NULL}};
const Segment SGI180287[] = {{&S58589[27], 6}, {NULL}};
const Segment SGI180288[] = {{&S58589[97], 9}, {NULL}};
const Segment SGI180289[] = {{&S58589[70], 6}, {NULL}};
const Segment SGI180290[] = {{&S58590[34], 11}, {NULL}};
const Segment SGI180291[] = {{&S58590[27], 6}, {NULL}};
const Segment SGI180292[] = {{&S58590[98], 9}, {NULL}};
const Segment SGI180293[] = {{&S58590[71], 6}, {NULL}};
const Segment SGI180294[] = {{&S58591[34], 10}, {NULL}};
const Segment SGI180295[] = {{&S58591[27], 6}, {NULL}};
const Segment SGI180296[] = {{&S58591[97], 9}, {NULL}};
const Segment SGI180297[] = {{&S58591[70], 6}, {NULL}};
const Segment SGI180298[] = {{&S58592[34], 11}, {NULL}};
const Segment SGI180299[] = {{&S58592[27], 6}, {NULL}};
const Segment SGI180300[] = {{&S58592[98], 9}, {NULL}};
const Segment SGI180301[] = {{&S58592[71], 6}, {NULL}};
const Segment SGI180302[] = {{&S70051[44], 7}, {NULL}};
const Segment SGI180303[] = {{&S70051[44], 19}, {NULL}};
const Segment SGI180304[] = {{&S70051[30], 6}, {NULL}};
const Segment SGI180305[] = {{&S70051[31], 3}, {NULL}};
const Segment SGI180306[] = {{&S70051[77], 6}, {NULL}};
const Segment SGI180307[] = {{&S70052[44], 7}, {NULL}};
const Segment SGI180308[] = {{&S70052[44], 17}, {NULL}};
const Segment SGI180309[] = {{&S70052[30], 6}, {NULL}};
const Segment SGI180310[] = {{&S70052[31], 3}, {NULL}};
const Segment SGI180311[] = {{&S70052[75], 6}, {NULL}};
const Segment SGI180312[] = {{&S70054[44], 7}, {NULL}};
const Segment SGI180313[] = {{&S70054[44], 19}, {NULL}};
const Segment SGI180314[] = {{&S70054[30], 6}, {NULL}};
const Segment SGI180315[] = {{&S70054[31], 3}, {NULL}};
const Segment SGI180316[] = {{&S70054[77], 6}, {NULL}};
const Segment SGI180317[] = {{&S62772[34], 15}, {NULL}};
const Segment SGI180318[] = {{&S62772[27], 6}, {NULL}};
const Segment SGI180319[] = {{&S62772[102], 9}, {NULL}};
const Segment SGI180320[] = {{&S62772[75], 6}, {NULL}};
const Segment SGI180321[] = {{&S70059[34], 7}, {NULL}};
const Segment SGI180322[] = {{&S70059[27], 6}, {NULL}};
const Segment SGI180323[] = {{&S70059[100], 9}, {NULL}};
const Segment SGI180324[] = {{&S70059[73], 6}, {NULL}};
const Segment SGI180325[] = {{&S58617[34], 18}, {NULL}};
const Segment SGI180326[] = {{&S58617[27], 6}, {NULL}};
const Segment SGI180327[] = {{&S58617[105], 9}, {NULL}};
const Segment SGI180328[] = {{&S58617[78], 6}, {NULL}};
const Segment SGI180329[] = {{&S58618[34], 18}, {NULL}};
const Segment SGI180330[] = {{&S58618[27], 6}, {NULL}};
const Segment SGI180331[] = {{&S58618[105], 9}, {NULL}};
const Segment SGI180332[] = {{&S58618[78], 6}, {NULL}};
const Segment SGI180333[] = {{&S58619[34], 10}, {NULL}};
const Segment SGI180334[] = {{&S58619[27], 6}, {NULL}};
const Segment SGI180335[] = {{&S58619[97], 9}, {NULL}};
const Segment SGI180336[] = {{&S58619[70], 6}, {NULL}};
const Segment SGI180337[] = {{&S58625[34], 12}, {NULL}};
const Segment SGI180338[] = {{&S58625[27], 6}, {NULL}};
const Segment SGI180339[] = {{&S58625[99], 9}, {NULL}};
const Segment SGI180340[] = {{&S58625[72], 6}, {NULL}};
const Segment SGI180341[] = {{&S70090[44], 18}, {NULL}};
const Segment SGI180342[] = {{&S70090[44], 30}, {NULL}};
const Segment SGI180343[] = {{&S70090[30], 6}, {NULL}};
const Segment SGI180344[] = {{&S70090[31], 3}, {NULL}};
const Segment SGI180345[] = {{&S70090[88], 6}, {NULL}};
const Segment SGI180346[] = {{&S70091[34], 10}, {NULL}};
const Segment SGI180347[] = {{&S70091[27], 6}, {NULL}};
const Segment SGI180348[] = {{&S70091[97], 9}, {NULL}};
const Segment SGI180349[] = {{&S70091[70], 6}, {NULL}};
const Segment SGI180350[] = {{&S70113[34], 8}, {NULL}};
const Segment SGI180351[] = {{&S70113[27], 6}, {NULL}};
const Segment SGI180352[] = {{&S70113[95], 9}, {NULL}};
const Segment SGI180353[] = {{&S70113[68], 6}, {NULL}};
const Segment SGI180354[] = {{&S70114[34], 10}, {NULL}};
const Segment SGI180355[] = {{&S70114[27], 6}, {NULL}};
const Segment SGI180356[] = {{&S70114[97], 9}, {NULL}};
const Segment SGI180357[] = {{&S70114[70], 6}, {NULL}};
const Segment SGI180358[] = {{&S70124[44], 9}, {NULL}};
const Segment SGI180359[] = {{&S70124[44], 21}, {NULL}};
const Segment SGI180360[] = {{&S70124[30], 6}, {NULL}};
const Segment SGI180361[] = {{&S70124[31], 3}, {NULL}};
const Segment SGI180362[] = {{&S70124[79], 6}, {NULL}};
const Segment SGI180363[] = {{&S58682[34], 8}, {NULL}};
const Segment SGI180364[] = {{&S58682[27], 6}, {NULL}};
const Segment SGI180365[] = {{&S58682[100], 9}, {NULL}};
const Segment SGI180366[] = {{&S58682[73], 6}, {NULL}};
const Segment SGI180367[] = {{&S70132[34], 10}, {NULL}};
const Segment SGI180368[] = {{&S70132[27], 6}, {NULL}};
const Segment SGI180369[] = {{&S70132[101], 9}, {NULL}};
const Segment SGI180370[] = {{&S70132[74], 6}, {NULL}};
const Segment SGI180371[] = {{&S58693[44], 11}, {NULL}};
const Segment SGI180372[] = {{&S58693[44], 23}, {NULL}};
const Segment SGI180373[] = {{&S58693[30], 6}, {NULL}};
const Segment SGI180374[] = {{&S58693[116], 3}, {NULL}};
const Segment SGI180375[] = {{&S58693[81], 6}, {NULL}};
const Segment SGI180376[] = {{&S58694[38], 11}, {NULL}};
const Segment SGI180377[] = {{&S58694[31], 6}, {NULL}};
const Segment SGI180378[] = {{&S70142[34], 10}, {NULL}};
const Segment SGI180379[] = {{&S70142[27], 6}, {NULL}};
const Segment SGI180380[] = {{&S70142[97], 9}, {NULL}};
const Segment SGI180381[] = {{&S70142[70], 6}, {NULL}};
const Segment SGI180382[] = {{&S70152[44], 9}, {NULL}};
const Segment SGI180383[] = {{&S70152[44], 21}, {NULL}};
const Segment SGI180384[] = {{&S70152[30], 6}, {NULL}};
const Segment SGI180385[] = {{&S70152[31], 3}, {NULL}};
const Segment SGI180386[] = {{&S70152[79], 6}, {NULL}};
const Segment SGI180387[] = {{&S58714[34], 9}, {NULL}};
const Segment SGI180388[] = {{&S58714[27], 6}, {NULL}};
const Segment SGI180389[] = {{&S58714[96], 9}, {NULL}};
const Segment SGI180390[] = {{&S58714[69], 6}, {NULL}};
const Segment SGI180391[] = {{&S58715[34], 9}, {NULL}};
const Segment SGI180392[] = {{&S58715[27], 6}, {NULL}};
const Segment SGI180393[] = {{&S58715[96], 9}, {NULL}};
const Segment SGI180394[] = {{&S58715[69], 6}, {NULL}};
const Segment SGI180395[] = {{&S70167[45], 8}, {NULL}};
const Segment SGI180396[] = {{&S70167[30], 6}, {NULL}};
const Segment SGI180397[] = {{&S70167[31], 3}, {NULL}};
const Segment SGI180398[] = {{&S70167[83], 5}, {NULL}};
const Segment SGI180399[] = {{&S70167[141], 13}, {NULL}};
const Segment SGI180400[] = {{&S70168[45], 7}, {NULL}};
const Segment SGI180401[] = {{&S70168[30], 6}, {NULL}};
const Segment SGI180402[] = {{&S70168[31], 3}, {NULL}};
const Segment SGI180403[] = {{&S70168[83], 5}, {NULL}};
const Segment SGI180404[] = {{&S70168[141], 13}, {NULL}};
const Segment SGI180405[] = {{&S70169[34], 8}, {NULL}};
const Segment SGI180406[] = {{&S70169[27], 6}, {NULL}};
const Segment SGI180407[] = {{&S70169[94], 9}, {NULL}};
const Segment SGI180408[] = {{&S70169[67], 6}, {NULL}};
const Segment SGI180409[] = {{&S70170[34], 11}, {NULL}};
const Segment SGI180410[] = {{&S70170[27], 6}, {NULL}};
const Segment SGI180411[] = {{&S70170[98], 9}, {NULL}};
const Segment SGI180412[] = {{&S70170[71], 6}, {NULL}};
const Segment SGI180413[] = {{&S58734[34], 10}, {NULL}};
const Segment SGI180414[] = {{&S58734[27], 6}, {NULL}};
const Segment SGI180415[] = {{&S58734[97], 9}, {NULL}};
const Segment SGI180416[] = {{&S58734[70], 6}, {NULL}};
const Segment SGI180417[] = {{&S58735[34], 11}, {NULL}};
const Segment SGI180418[] = {{&S58735[27], 6}, {NULL}};
const Segment SGI180419[] = {{&S58735[98], 9}, {NULL}};
const Segment SGI180420[] = {{&S58735[71], 6}, {NULL}};
const Segment SGI180421[] = {{&S70178[34], 10}, {NULL}};
const Segment SGI180422[] = {{&S70178[27], 6}, {NULL}};
const Segment SGI180423[] = {{&S58736[34], 11}, {NULL}};
const Segment SGI180424[] = {{&S58736[27], 6}, {NULL}};
const Segment SGI180425[] = {{&S70178[97], 9}, {NULL}};
const Segment SGI180426[] = {{&S70178[70], 6}, {NULL}};
const Segment SGI180427[] = {{&S58736[98], 9}, {NULL}};
const Segment SGI180428[] = {{&S58736[71], 6}, {NULL}};
const Segment SGI180429[] = {{&S70179[34], 11}, {NULL}};
const Segment SGI180430[] = {{&S70179[27], 6}, {NULL}};
const Segment SGI180431[] = {{&S58737[34], 11}, {NULL}};
const Segment SGI180432[] = {{&S58737[27], 6}, {NULL}};
const Segment SGI180433[] = {{&S70179[98], 9}, {NULL}};
const Segment SGI180434[] = {{&S70179[71], 6}, {NULL}};
const Segment SGI180435[] = {{&S58737[98], 9}, {NULL}};
const Segment SGI180436[] = {{&S58737[71], 6}, {NULL}};
const Segment SGI180437[] = {{&S58738[34], 11}, {NULL}};
const Segment SGI180438[] = {{&S70180[27], 6}, {NULL}};
const Segment SGI180439[] = {{&S58738[27], 6}, {NULL}};
const Segment SGI180440[] = {{&S70180[99], 9}, {NULL}};
const Segment SGI180441[] = {{&S58738[98], 9}, {NULL}};
const Segment SGI180442[] = {{&S70180[72], 6}, {NULL}};
const Segment SGI180443[] = {{&S58738[71], 6}, {NULL}};
const Segment SGI180444[] = {{&S70181[34], 10}, {NULL}};
const Segment SGI180445[] = {{&S70181[27], 6}, {NULL}};
const Segment SGI180446[] = {{&S70181[97], 9}, {NULL}};
const Segment SGI180447[] = {{&S70181[70], 6}, {NULL}};
const Segment SGI180448[] = {{&S70183[34], 13}, {NULL}};
const Segment SGI180449[] = {{&S70183[27], 6}, {NULL}};
const Segment SGI180450[] = {{&S70183[100], 9}, {NULL}};
const Segment SGI180451[] = {{&S70183[73], 6}, {NULL}};
const Segment SGI180452[] = {{&S70184[34], 9}, {NULL}};
const Segment SGI180453[] = {{&S70184[27], 6}, {NULL}};
const Segment SGI180454[] = {{&S70184[96], 9}, {NULL}};
const Segment SGI180455[] = {{&S70184[69], 6}, {NULL}};
const Segment SGI180456[] = {{&S62929[44], 10}, {NULL}};
const Segment SGI180457[] = {{&S62929[44], 26}, {NULL}};
const Segment SGI180458[] = {{&S62929[30], 6}, {NULL}};
const Segment SGI180459[] = {{&S70185[34], 11}, {NULL}};
const Segment SGI180460[] = {{&S70185[27], 6}, {NULL}};
const Segment SGI180461[] = {{&S62929[118], 3}, {NULL}};
const Segment SGI180462[] = {{&S62929[84], 5}, {NULL}};
const Segment SGI180463[] = {{&S70185[98], 9}, {NULL}};
const Segment SGI180464[] = {{&S70185[71], 6}, {NULL}};
const Segment SGI180465[] = {{&S62930[44], 11}, {NULL}};
const Segment SGI180466[] = {{&S62930[44], 28}, {NULL}};
const Segment SGI180467[] = {{&S62930[30], 6}, {NULL}};
const Segment SGI180468[] = {{&S62930[120], 3}, {NULL}};
const Segment SGI180469[] = {{&S62930[86], 5}, {NULL}};
const Segment SGI180470[] = {{&S70187[44], 7}, {NULL}};
const Segment SGI180471[] = {{&S70187[44], 19}, {NULL}};
const Segment SGI180472[] = {{&S70187[30], 6}, {NULL}};
const Segment SGI180473[] = {{&S70187[31], 3}, {NULL}};
const Segment SGI180474[] = {{&S70187[77], 6}, {NULL}};
const Segment SGI180475[] = {{&S70190[34], 11}, {NULL}};
const Segment SGI180476[] = {{&S70190[27], 6}, {NULL}};
const Segment SGI180477[] = {{&S70190[110], 9}, {NULL}};
const Segment SGI180478[] = {{&S70190[83], 6}, {NULL}};
const Segment SGI180479[] = {{&S70191[34], 7}, {NULL}};
const Segment SGI180480[] = {{&S70191[27], 6}, {NULL}};
const Segment SGI180481[] = {{&S70191[112], 9}, {NULL}};
const Segment SGI180482[] = {{&S70191[85], 6}, {NULL}};
const Segment SGI180483[] = {{&S70196[44], 16}, {NULL}};
const Segment SGI180484[] = {{&S70196[44], 28}, {NULL}};
const Segment SGI180485[] = {{&S70196[30], 6}, {NULL}};
const Segment SGI180486[] = {{&S70196[31], 3}, {NULL}};
const Segment SGI180487[] = {{&S70196[86], 6}, {NULL}};
const Segment SGI180488[] = {{&S70197[44], 7}, {NULL}};
const Segment SGI180489[] = {{&S70197[44], 19}, {NULL}};
const Segment SGI180490[] = {{&S70197[30], 6}, {NULL}};
const Segment SGI180491[] = {{&S70197[31], 3}, {NULL}};
const Segment SGI180492[] = {{&S70197[77], 6}, {NULL}};
const Segment SGI180493[] = {{&S70199[34], 7}, {NULL}};
const Segment SGI180494[] = {{&S70199[27], 6}, {NULL}};
const Segment SGI180495[] = {{&S70199[94], 9}, {NULL}};
const Segment SGI180496[] = {{&S70199[67], 6}, {NULL}};
const Segment SGI180497[] = {{&S70204[44], 7}, {NULL}};
const Segment SGI180498[] = {{&S70204[44], 19}, {NULL}};
const Segment SGI180499[] = {{&S70204[30], 6}, {NULL}};
const Segment SGI180500[] = {{&S70204[31], 3}, {NULL}};
const Segment SGI180501[] = {{&S70204[77], 6}, {NULL}};
const Segment SGI180502[] = {{&S70205[44], 11}, {NULL}};
const Segment SGI180503[] = {{&S70205[44], 23}, {NULL}};
const Segment SGI180504[] = {{&S70205[30], 6}, {NULL}};
const Segment SGI180505[] = {{&S70205[31], 3}, {NULL}};
const Segment SGI180506[] = {{&S70205[81], 6}, {NULL}};
const Segment SGI180507[] = {{&S66973[42], 20}, {NULL}};
const Segment SGI180508[] = {{&S70216[34], 14}, {NULL}};
const Segment SGI180509[] = {{&S66973[30], 4}, {NULL}};
const Segment SGI180510[] = {{&S70216[27], 6}, {NULL}};
const Segment SGI180511[] = {{&S66973[111], 3}, {NULL}};
const Segment SGI180512[] = {{&S66973[76], 6}, {NULL}};
const Segment SGI180513[] = {{&S70216[101], 9}, {NULL}};
const Segment SGI180514[] = {{&S70216[74], 6}, {NULL}};
const Segment SGI180515[] = {{&S70218[34], 10}, {NULL}};
const Segment SGI180516[] = {{&S70218[27], 6}, {NULL}};
const Segment SGI180517[] = {{&S70218[97], 9}, {NULL}};
const Segment SGI180518[] = {{&S70218[70], 6}, {NULL}};
const Segment SGI180519[] = {{&S70221[34], 7}, {NULL}};
const Segment SGI180520[] = {{&S70221[27], 6}, {NULL}};
const Segment SGI180521[] = {{&S70221[99], 9}, {NULL}};
const Segment SGI180522[] = {{&S70221[72], 6}, {NULL}};
const Segment SGI180523[] = {{&S70221[140], 8}, {NULL}};
const Segment SGI180524[] = {{&S70222[34], 8}, {NULL}};
const Segment SGI180525[] = {{&S70222[27], 6}, {NULL}};
const Segment SGI180526[] = {{&S70222[99], 9}, {NULL}};
const Segment SGI180527[] = {{&S70222[72], 6}, {NULL}};
const Segment SGI180528[] = {{&S70222[140], 8}, {NULL}};
const Segment SGI180529[] = {{&S70226[34], 18}, {NULL}};
const Segment SGI180530[] = {{&S70226[27], 6}, {NULL}};
const Segment SGI180531[] = {{&S70226[105], 9}, {NULL}};
const Segment SGI180532[] = {{&S70226[78], 6}, {NULL}};
const Segment SGI180533[] = {{&S56979[34], 14}, {NULL}};
const Segment SGI180534[] = {{&S56979[27], 6}, {NULL}};
const Segment SGI180535[] = {{&S56979[101], 9}, {NULL}};
const Segment SGI180536[] = {{&S56979[74], 6}, {NULL}};
const Segment SGI180537[] = {{&S70227[34], 11}, {NULL}};
const Segment SGI180538[] = {{&S70227[27], 6}, {NULL}};
const Segment SGI180539[] = {{&S70227[98], 9}, {NULL}};
const Segment SGI180540[] = {{&S70227[71], 6}, {NULL}};
const Segment SGI180541[] = {{&S70246[44], 10}, {NULL}};
const Segment SGI180542[] = {{&S70246[44], 22}, {NULL}};
const Segment SGI180543[] = {{&S70246[30], 6}, {NULL}};
const Segment SGI180544[] = {{&S70246[31], 3}, {NULL}};
const Segment SGI180545[] = {{&S70246[80], 6}, {NULL}};
const Segment SGI180546[] = {{&S70247[44], 6}, {NULL}};
const Segment SGI180547[] = {{&S70247[44], 18}, {NULL}};
const Segment SGI180548[] = {{&S70247[30], 6}, {NULL}};
const Segment SGI180549[] = {{&S70247[31], 3}, {NULL}};
const Segment SGI180550[] = {{&S70247[76], 6}, {NULL}};
const Segment SGI180551[] = {{&S70249[34], 14}, {NULL}};
const Segment SGI180552[] = {{&S70249[27], 6}, {NULL}};
const Segment SGI180553[] = {{&S70249[106], 9}, {NULL}};
const Segment SGI180554[] = {{&S70249[79], 6}, {NULL}};
const Segment SGI180555[] = {{&S70250[34], 8}, {NULL}};
const Segment SGI180556[] = {{&S70250[27], 6}, {NULL}};
const Segment SGI180557[] = {{&S70250[107], 9}, {NULL}};
const Segment SGI180558[] = {{&S70250[80], 6}, {NULL}};
const Segment SGI180559[] = {{&S47629[34], 10}, {NULL}};
const Segment SGI180560[] = {{&S47629[27], 6}, {NULL}};
const Segment SGI180561[] = {{&S47629[96], 9}, {NULL}};
const Segment SGI180562[] = {{&S47629[69], 6}, {NULL}};
const Segment SGI180563[] = {{&S57015[44], 10}, {NULL}};
const Segment SGI180564[] = {{&S57015[44], 22}, {NULL}};
const Segment SGI180565[] = {{&S57015[30], 6}, {NULL}};
const Segment SGI180566[] = {{&S57015[133], 3}, {NULL}};
const Segment SGI180567[] = {{&S57015[98], 6}, {NULL}};
const Segment SGI180568[] = {{&S57015[68], 8}, {NULL}};
const Segment SGI180569[] = {{&S47630[34], 11}, {NULL}};
const Segment SGI180570[] = {{&S47630[27], 6}, {NULL}};
const Segment SGI180571[] = {{&S47630[97], 9}, {NULL}};
const Segment SGI180572[] = {{&S47630[70], 6}, {NULL}};
const Segment SGI180573[] = {{&S57016[44], 10}, {NULL}};
const Segment SGI180574[] = {{&S57016[44], 22}, {NULL}};
const Segment SGI180575[] = {{&S57016[30], 6}, {NULL}};
const Segment SGI180576[] = {{&S57016[133], 3}, {NULL}};
const Segment SGI180577[] = {{&S57016[98], 6}, {NULL}};
const Segment SGI180578[] = {{&S57016[68], 8}, {NULL}};
const Segment SGI180579[] = {{&S47631[34], 11}, {NULL}};
const Segment SGI180580[] = {{&S47631[27], 6}, {NULL}};
const Segment SGI180581[] = {{&S47631[97], 9}, {NULL}};
const Segment SGI180582[] = {{&S47631[70], 6}, {NULL}};
const Segment SGI180583[] = {{&S63023[34], 6}, {NULL}};
const Segment SGI180584[] = {{&S63023[27], 6}, {NULL}};
const Segment SGI180585[] = {{&S63023[93], 9}, {NULL}};
const Segment SGI180586[] = {{&S63023[66], 6}, {NULL}};
const Segment SGI180587[] = {{&S63024[34], 7}, {NULL}};
const Segment SGI180588[] = {{&S63024[27], 6}, {NULL}};
const Segment SGI180589[] = {{&S63024[94], 9}, {NULL}};
const Segment SGI180590[] = {{&S63024[67], 6}, {NULL}};
const Segment SGI180591[] = {{&S70276[44], 10}, {NULL}};
const Segment SGI180592[] = {{&S70276[44], 22}, {NULL}};
const Segment SGI180593[] = {{&S70276[30], 6}, {NULL}};
const Segment SGI180594[] = {{&S70276[31], 3}, {NULL}};
const Segment SGI180595[] = {{&S70276[95], 6}, {NULL}};
const Segment SGI180596[] = {{&S70276[68], 8}, {NULL}};
const Segment SGI180597[] = {{&S70278[44], 10}, {NULL}};
const Segment SGI180598[] = {{&S70278[44], 22}, {NULL}};
const Segment SGI180599[] = {{&S70278[30], 6}, {NULL}};
const Segment SGI180600[] = {{&S70278[31], 3}, {NULL}};
const Segment SGI180601[] = {{&S70278[80], 6}, {NULL}};
const Segment SGI180602[] = {{&S70279[44], 16}, {NULL}};
const Segment SGI180603[] = {{&S70279[44], 28}, {NULL}};
const Segment SGI180604[] = {{&S70279[30], 6}, {NULL}};
const Segment SGI180605[] = {{&S70279[31], 3}, {NULL}};
const Segment SGI180606[] = {{&S70279[86], 6}, {NULL}};
const Segment SGI180607[] = {{&S70280[38], 16}, {NULL}};
const Segment SGI180608[] = {{&S70280[31], 6}, {NULL}};
const Segment SGI180609[] = {{&S63045[44], 6}, {NULL}};
const Segment SGI180610[] = {{&S63045[44], 18}, {NULL}};
const Segment SGI180611[] = {{&S63045[30], 6}, {NULL}};
const Segment SGI180612[] = {{&S63045[111], 3}, {NULL}};
const Segment SGI180613[] = {{&S63045[76], 6}, {NULL}};
const Segment SGI180614[] = {{&S63046[38], 6}, {NULL}};
const Segment SGI180615[] = {{&S63046[31], 6}, {NULL}};
const Segment SGI180616[] = {{&S63047[44], 16}, {NULL}};
const Segment SGI180617[] = {{&S63047[44], 28}, {NULL}};
const Segment SGI180618[] = {{&S63047[30], 6}, {NULL}};
const Segment SGI180619[] = {{&S63047[121], 3}, {NULL}};
const Segment SGI180620[] = {{&S63047[86], 6}, {NULL}};
const Segment SGI180621[] = {{&S70293[34], 10}, {NULL}};
const Segment SGI180622[] = {{&S70293[27], 6}, {NULL}};
const Segment SGI180623[] = {{&S70293[97], 9}, {NULL}};
const Segment SGI180624[] = {{&S70293[70], 6}, {NULL}};
const Segment SGI180625[] = {{&S47669[34], 10}, {NULL}};
const Segment SGI180626[] = {{&S47669[27], 6}, {NULL}};
const Segment SGI180627[] = {{&S47669[96], 9}, {NULL}};
const Segment SGI180628[] = {{&S47669[69], 6}, {NULL}};
const Segment SGI180629[] = {{&S47670[34], 10}, {NULL}};
const Segment SGI180630[] = {{&S47670[27], 6}, {NULL}};
const Segment SGI180631[] = {{&S47670[96], 9}, {NULL}};
const Segment SGI180632[] = {{&S47670[69], 6}, {NULL}};
const Segment SGI180633[] = {{&S47671[34], 10}, {NULL}};
const Segment SGI180634[] = {{&S47671[27], 6}, {NULL}};
const Segment SGI180635[] = {{&S47671[96], 9}, {NULL}};
const Segment SGI180636[] = {{&S47671[69], 6}, {NULL}};
const Segment SGI180637[] = {{&S58748[34], 7}, {NULL}};
const Segment SGI180638[] = {{&S58748[27], 6}, {NULL}};
const Segment SGI180639[] = {{&S58748[94], 9}, {NULL}};
const Segment SGI180640[] = {{&S58748[67], 6}, {NULL}};
const Segment SGI180641[] = {{&S47672[41], 18}, {NULL}};
const Segment SGI180642[] = {{&S47672[41], 29}, {NULL}};
const Segment SGI180643[] = {{&S47672[27], 6}, {NULL}};
const Segment SGI180644[] = {{&S47672[119], 3}, {NULL}};
const Segment SGI180645[] = {{&S47672[123], 9}, {NULL}};
const Segment SGI180646[] = {{&S47672[84], 6}, {NULL}};
const Segment SGI180647[] = {{&S70307[44], 8}, {NULL}};
const Segment SGI180648[] = {{&S70307[44], 19}, {NULL}};
const Segment SGI180649[] = {{&S70307[30], 6}, {NULL}};
const Segment SGI180650[] = {{&S70307[77], 6}, {NULL}};
const Segment SGI180651[] = {{&S70308[44], 8}, {NULL}};
const Segment SGI180652[] = {{&S70308[44], 19}, {NULL}};
const Segment SGI180653[] = {{&S70308[30], 6}, {NULL}};
const Segment SGI180654[] = {{&S70308[31], 3}, {NULL}};
const Segment SGI180655[] = {{&S63069[44], 9}, {NULL}};
const Segment SGI180656[] = {{&S70308[77], 6}, {NULL}};
const Segment SGI180657[] = {{&S63069[44], 21}, {NULL}};
const Segment SGI180658[] = {{&S63069[30], 6}, {NULL}};
const Segment SGI180659[] = {{&S63069[114], 3}, {NULL}};
const Segment SGI180660[] = {{&S63069[79], 6}, {NULL}};
const Segment SGI180661[] = {{&S70309[44], 8}, {NULL}};
const Segment SGI180662[] = {{&S70309[44], 19}, {NULL}};
const Segment SGI180663[] = {{&S70309[30], 6}, {NULL}};
const Segment SGI180664[] = {{&S47678[44], 10}, {NULL}};
const Segment SGI180665[] = {{&S47678[44], 21}, {NULL}};
const Segment SGI180666[] = {{&S47678[30], 6}, {NULL}};
const Segment SGI180667[] = {{&S70309[31], 3}, {NULL}};
const Segment SGI180668[] = {{&S70309[77], 6}, {NULL}};
const Segment SGI180669[] = {{&S47678[143], 3}, {NULL}};
const Segment SGI180670[] = {{&S63070[44], 9}, {NULL}};
const Segment SGI180671[] = {{&S63070[44], 21}, {NULL}};
const Segment SGI180672[] = {{&S47678[108], 6}, {NULL}};
const Segment SGI180673[] = {{&S63070[30], 6}, {NULL}};
const Segment SGI180674[] = {{&S63070[114], 3}, {NULL}};
const Segment SGI180675[] = {{&S63070[79], 6}, {NULL}};
const Segment SGI180676[] = {{&S70310[44], 8}, {NULL}};
const Segment SGI180677[] = {{&S70310[44], 19}, {NULL}};
const Segment SGI180678[] = {{&S70310[30], 6}, {NULL}};
const Segment SGI180679[] = {{&S70310[31], 3}, {NULL}};
const Segment SGI180680[] = {{&S70310[77], 6}, {NULL}};
const Segment SGI180681[] = {{&S47679[44], 10}, {NULL}};
const Segment SGI180682[] = {{&S47679[44], 21}, {NULL}};
const Segment SGI180683[] = {{&S47679[30], 6}, {NULL}};
const Segment SGI180684[] = {{&S47679[143], 3}, {NULL}};
const Segment SGI180685[] = {{&S47679[108], 6}, {NULL}};
const Segment SGI180686[] = {{&S70311[44], 8}, {NULL}};
const Segment SGI180687[] = {{&S70311[44], 19}, {NULL}};
const Segment SGI180688[] = {{&S70311[30], 6}, {NULL}};
const Segment SGI180689[] = {{&S70311[31], 3}, {NULL}};
const Segment SGI180690[] = {{&S70311[77], 6}, {NULL}};
const Segment SGI180691[] = {{&S47680[44], 10}, {NULL}};
const Segment SGI180692[] = {{&S47680[44], 21}, {NULL}};
const Segment SGI180693[] = {{&S47680[30], 6}, {NULL}};
const Segment SGI180694[] = {{&S47680[143], 3}, {NULL}};
const Segment SGI180695[] = {{&S47680[108], 6}, {NULL}};
const Segment SGI180696[] = {{&S47681[44], 10}, {NULL}};
const Segment SGI180697[] = {{&S47681[44], 22}, {NULL}};
const Segment SGI180698[] = {{&S47681[30], 6}, {NULL}};
const Segment SGI180699[] = {{&S47681[115], 3}, {NULL}};
const Segment SGI180700[] = {{&S47681[80], 6}, {NULL}};
const Segment SGI180701[] = {{&S70315[44], 7}, {NULL}};
const Segment SGI180702[] = {{&S70315[44], 19}, {NULL}};
const Segment SGI180703[] = {{&S70315[30], 6}, {NULL}};
const Segment SGI180704[] = {{&S70315[31], 3}, {NULL}};
const Segment SGI180705[] = {{&S70315[77], 6}, {NULL}};
const Segment SGI180706[] = {{&S47682[44], 10}, {NULL}};
const Segment SGI180707[] = {{&S47682[44], 22}, {NULL}};
const Segment SGI180708[] = {{&S47682[30], 6}, {NULL}};
const Segment SGI180709[] = {{&S47682[115], 3}, {NULL}};
const Segment SGI180710[] = {{&S47682[80], 6}, {NULL}};
const Segment SGI180711[] = {{&S47683[44], 10}, {NULL}};
const Segment SGI180712[] = {{&S47683[44], 22}, {NULL}};
const Segment SGI180713[] = {{&S47683[30], 6}, {NULL}};
const Segment SGI180714[] = {{&S47683[115], 3}, {NULL}};
const Segment SGI180715[] = {{&S47683[80], 6}, {NULL}};
const Segment SGI180716[] = {{&S70318[44], 24}, {NULL}};
const Segment SGI180717[] = {{&S70318[44], 36}, {NULL}};
const Segment SGI180718[] = {{&S70318[30], 6}, {NULL}};
const Segment SGI180719[] = {{&S70318[31], 3}, {NULL}};
const Segment SGI180720[] = {{&S70318[94], 6}, {NULL}};
const Segment SGI180721[] = {{&S70319[44], 13}, {NULL}};
const Segment SGI180722[] = {{&S70319[44], 25}, {NULL}};
const Segment SGI180723[] = {{&S70319[30], 6}, {NULL}};
const Segment SGI180724[] = {{&S70319[31], 3}, {NULL}};
const Segment SGI180725[] = {{&S70319[83], 6}, {NULL}};
const Segment SGI180726[] = {{&S58794[44], 11}, {NULL}};
const Segment SGI180727[] = {{&S58794[44], 23}, {NULL}};
const Segment SGI180728[] = {{&S58794[30], 6}, {NULL}};
const Segment SGI180729[] = {{&S58794[116], 3}, {NULL}};
const Segment SGI180730[] = {{&S58794[81], 6}, {NULL}};
const Segment SGI180731[] = {{&S63109[44], 6}, {NULL}};
const Segment SGI180732[] = {{&S63109[44], 18}, {NULL}};
const Segment SGI180733[] = {{&S63109[30], 6}, {NULL}};
const Segment SGI180734[] = {{&S58795[38], 11}, {NULL}};
const Segment SGI180735[] = {{&S58795[31], 6}, {NULL}};
const Segment SGI180736[] = {{&S63109[127], 3}, {NULL}};
const Segment SGI180737[] = {{&S63109[92], 6}, {NULL}};
const Segment SGI180738[] = {{&S63109[64], 8}, {NULL}};
const Segment SGI180739[] = {{&S70344[44], 22}, {NULL}};
const Segment SGI180740[] = {{&S70344[44], 33}, {NULL}};
const Segment SGI180741[] = {{&S70344[30], 6}, {NULL}};
const Segment SGI180742[] = {{&S70344[31], 3}, {NULL}};
const Segment SGI180743[] = {{&S70344[137], 6}, {NULL}};
const Segment SGI180744[] = {{&S70344[79], 8}, {NULL}};
const Segment SGI180745[] = {{&S63110[44], 6}, {NULL}};
const Segment SGI180746[] = {{&S63110[44], 18}, {NULL}};
const Segment SGI180747[] = {{&S63110[30], 6}, {NULL}};
const Segment SGI180748[] = {{&S63110[127], 3}, {NULL}};
const Segment SGI180749[] = {{&S63110[92], 6}, {NULL}};
const Segment SGI180750[] = {{&S63110[64], 8}, {NULL}};
const Segment SGI180751[] = {{&S70345[44], 18}, {NULL}};
const Segment SGI180752[] = {{&S70345[44], 28}, {NULL}};
const Segment SGI180753[] = {{&S70345[30], 6}, {NULL}};
const Segment SGI180754[] = {{&S70345[31], 3}, {NULL}};
const Segment SGI180755[] = {{&S70345[105], 6}, {NULL}};
const Segment SGI180756[] = {{&S70345[74], 8}, {NULL}};
const Segment SGI180757[] = {{&S58800[44], 9}, {NULL}};
const Segment SGI180758[] = {{&S58800[44], 21}, {NULL}};
const Segment SGI180759[] = {{&S70346[44], 9}, {NULL}};
const Segment SGI180760[] = {{&S58800[30], 6}, {NULL}};
const Segment SGI180761[] = {{&S70346[44], 19}, {NULL}};
const Segment SGI180762[] = {{&S70346[30], 6}, {NULL}};
const Segment SGI180763[] = {{&S58800[114], 3}, {NULL}};
const Segment SGI180764[] = {{&S58800[79], 6}, {NULL}};
const Segment SGI180765[] = {{&S70346[31], 3}, {NULL}};
const Segment SGI180766[] = {{&S70346[96], 6}, {NULL}};
const Segment SGI180767[] = {{&S70346[65], 8}, {NULL}};
const Segment SGI180768[] = {{&S58801[38], 9}, {NULL}};
const Segment SGI180769[] = {{&S58801[31], 6}, {NULL}};
const Segment SGI180770[] = {{&S58802[34], 9}, {NULL}};
const Segment SGI180771[] = {{&S58802[27], 6}, {NULL}};
const Segment SGI180772[] = {{&S58802[96], 9}, {NULL}};
const Segment SGI180773[] = {{&S58802[69], 6}, {NULL}};
const Segment SGI180774[] = {{&S58803[34], 10}, {NULL}};
const Segment SGI180775[] = {{&S58803[27], 6}, {NULL}};
const Segment SGI180776[] = {{&S58803[97], 9}, {NULL}};
const Segment SGI180777[] = {{&S58803[70], 6}, {NULL}};
const Segment SGI180778[] = {{&S70355[44], 9}, {NULL}};
const Segment SGI180779[] = {{&S70355[44], 21}, {NULL}};
const Segment SGI180780[] = {{&S70355[30], 6}, {NULL}};
const Segment SGI180781[] = {{&S70355[31], 3}, {NULL}};
const Segment SGI180782[] = {{&S70355[79], 6}, {NULL}};
const Segment SGI180783[] = {{&S63124[44], 10}, {NULL}};
const Segment SGI180784[] = {{&S63124[44], 22}, {NULL}};
const Segment SGI180785[] = {{&S63124[30], 6}, {NULL}};
const Segment SGI180786[] = {{&S63124[115], 3}, {NULL}};
const Segment SGI180787[] = {{&S63124[80], 6}, {NULL}};
const Segment SGI180788[] = {{&S63124[140], 11}, {NULL}};
const Segment SGI180789[] = {{&S57130[44], 14}, {NULL}};
const Segment SGI180790[] = {{&S57130[44], 26}, {NULL}};
const Segment SGI180791[] = {{&S57130[30], 6}, {NULL}};
const Segment SGI180792[] = {{&S63125[44], 10}, {NULL}};
const Segment SGI180793[] = {{&S63125[44], 22}, {NULL}};
const Segment SGI180794[] = {{&S57130[136], 3}, {NULL}};
const Segment SGI180795[] = {{&S63125[30], 6}, {NULL}};
const Segment SGI180796[] = {{&S57130[102], 6}, {NULL}};
const Segment SGI180797[] = {{&S57130[72], 8}, {NULL}};
const Segment SGI180798[] = {{&S63125[115], 3}, {NULL}};
const Segment SGI180799[] = {{&S63125[80], 6}, {NULL}};
const Segment SGI180800[] = {{&S63125[140], 11}, {NULL}};
const Segment SGI180801[] = {{&S57134[44], 7}, {NULL}};
const Segment SGI180802[] = {{&S57134[44], 19}, {NULL}};
const Segment SGI180803[] = {{&S57134[30], 6}, {NULL}};
const Segment SGI180804[] = {{&S57134[112], 3}, {NULL}};
const Segment SGI180805[] = {{&S57134[77], 6}, {NULL}};
const Segment SGI180806[] = {{&S58816[34], 9}, {NULL}};
const Segment SGI180807[] = {{&S58816[27], 6}, {NULL}};
const Segment SGI180808[] = {{&S63131[44], 10}, {NULL}};
const Segment SGI180809[] = {{&S58816[96], 9}, {NULL}};
const Segment SGI180810[] = {{&S63131[44], 22}, {NULL}};
const Segment SGI180811[] = {{&S57135[38], 7}, {NULL}};
const Segment SGI180812[] = {{&S58816[69], 6}, {NULL}};
const Segment SGI180813[] = {{&S63131[30], 6}, {NULL}};
const Segment SGI180814[] = {{&S57135[31], 6}, {NULL}};
const Segment SGI180815[] = {{&S63131[115], 3}, {NULL}};
const Segment SGI180816[] = {{&S63131[80], 6}, {NULL}};
const Segment SGI180817[] = {{&S63131[140], 11}, {NULL}};
const Segment SGI180818[] = {{&S58817[34], 10}, {NULL}};
const Segment SGI180819[] = {{&S58817[27], 6}, {NULL}};
const Segment SGI180820[] = {{&S58817[97], 9}, {NULL}};
const Segment SGI180821[] = {{&S58817[70], 6}, {NULL}};
const Segment SGI180822[] = {{&S58818[34], 10}, {NULL}};
const Segment SGI180823[] = {{&S58818[27], 6}, {NULL}};
const Segment SGI180824[] = {{&S58818[97], 9}, {NULL}};
const Segment SGI180825[] = {{&S58818[70], 6}, {NULL}};
const Segment SGI180826[] = {{&S58819[34], 10}, {NULL}};
const Segment SGI180827[] = {{&S58819[27], 6}, {NULL}};
const Segment SGI180828[] = {{&S58819[97], 9}, {NULL}};
const Segment SGI180829[] = {{&S58819[70], 6}, {NULL}};
const Segment SGI180830[] = {{&S70379[44], 13}, {NULL}};
const Segment SGI180831[] = {{&S70379[44], 25}, {NULL}};
const Segment SGI180832[] = {{&S70379[30], 6}, {NULL}};
const Segment SGI180833[] = {{&S70379[31], 3}, {NULL}};
const Segment SGI180834[] = {{&S70379[83], 6}, {NULL}};
const Segment SGI180835[] = {{&S70380[44], 14}, {NULL}};
const Segment SGI180836[] = {{&S70380[44], 26}, {NULL}};
const Segment SGI180837[] = {{&S70380[30], 6}, {NULL}};
const Segment SGI180838[] = {{&S70380[31], 3}, {NULL}};
const Segment SGI180839[] = {{&S70380[84], 6}, {NULL}};
const Segment SGI180840[] = {{&S70381[42], 13}, {NULL}};
const Segment SGI180841[] = {{&S70381[30], 6}, {NULL}};
const Segment SGI180842[] = {{&S70381[31], 3}, {NULL}};
const Segment SGI180843[] = {{&S70381[81], 6}, {NULL}};
const Segment SGI180844[] = {{&S63150[34], 6}, {NULL}};
const Segment SGI180845[] = {{&S63150[27], 6}, {NULL}};
const Segment SGI180846[] = {{&S63150[93], 9}, {NULL}};
const Segment SGI180847[] = {{&S63150[66], 6}, {NULL}};
const Segment SGI180848[] = {{&S70382[38], 13}, {NULL}};
const Segment SGI180849[] = {{&S70382[31], 6}, {NULL}};
const Segment SGI180850[] = {{&S70404[44], 10}, {NULL}};
const Segment SGI180851[] = {{&S70404[44], 22}, {NULL}};
const Segment SGI180852[] = {{&S70404[30], 6}, {NULL}};
const Segment SGI180853[] = {{&S70404[31], 3}, {NULL}};
const Segment SGI180854[] = {{&S70404[80], 6}, {NULL}};
const Segment SGI180855[] = {{&S70405[44], 6}, {NULL}};
const Segment SGI180856[] = {{&S70405[44], 18}, {NULL}};
const Segment SGI180857[] = {{&S70405[30], 6}, {NULL}};
const Segment SGI180858[] = {{&S70405[79], 6}, {NULL}};
const Segment SGI180859[] = {{&S70406[44], 7}, {NULL}};
const Segment SGI180860[] = {{&S70406[44], 19}, {NULL}};
const Segment SGI180861[] = {{&S70406[30], 6}, {NULL}};
const Segment SGI180862[] = {{&S70406[31], 3}, {NULL}};
const Segment SGI180863[] = {{&S70406[77], 6}, {NULL}};
const Segment SGI180864[] = {{&S70407[38], 6}, {NULL}};
const Segment SGI180865[] = {{&S70407[31], 6}, {NULL}};
const Segment SGI180866[] = {{&S70408[38], 7}, {NULL}};
const Segment SGI180867[] = {{&S70408[31], 6}, {NULL}};
const Segment SGI180868[] = {{&S70409[34], 10}, {NULL}};
const Segment SGI180869[] = {{&S70409[27], 6}, {NULL}};
const Segment SGI180870[] = {{&S63185[44], 12}, {NULL}};
const Segment SGI180871[] = {{&S63185[44], 24}, {NULL}};
const Segment SGI180872[] = {{&S63185[30], 6}, {NULL}};
const Segment SGI180873[] = {{&S70409[97], 9}, {NULL}};
const Segment SGI180874[] = {{&S70409[70], 6}, {NULL}};
const Segment SGI180875[] = {{&S63185[117], 3}, {NULL}};
const Segment SGI180876[] = {{&S63185[82], 6}, {NULL}};
const Segment SGI180877[] = {{&S70410[34], 6}, {NULL}};
const Segment SGI180878[] = {{&S70410[27], 6}, {NULL}};
const Segment SGI180879[] = {{&S63186[44], 12}, {NULL}};
const Segment SGI180880[] = {{&S63186[44], 24}, {NULL}};
const Segment SGI180881[] = {{&S63186[30], 6}, {NULL}};
const Segment SGI180882[] = {{&S70410[93], 9}, {NULL}};
const Segment SGI180883[] = {{&S70410[66], 6}, {NULL}};
const Segment SGI180884[] = {{&S63186[117], 3}, {NULL}};
const Segment SGI180885[] = {{&S63186[82], 6}, {NULL}};
const Segment SGI180886[] = {{&S70415[44], 9}, {NULL}};
const Segment SGI180887[] = {{&S70415[44], 21}, {NULL}};
const Segment SGI180888[] = {{&S70415[30], 6}, {NULL}};
const Segment SGI180889[] = {{&S70415[31], 3}, {NULL}};
const Segment SGI180890[] = {{&S70415[79], 6}, {NULL}};
const Segment SGI180891[] = {{&S70416[34], 10}, {NULL}};
const Segment SGI180892[] = {{&S70416[27], 6}, {NULL}};
const Segment SGI180893[] = {{&S70416[97], 9}, {NULL}};
const Segment SGI180894[] = {{&S70416[70], 6}, {NULL}};
const Segment SGI180895[] = {{&S58874[34], 10}, {NULL}};
const Segment SGI180896[] = {{&S58874[27], 6}, {NULL}};
const Segment SGI180897[] = {{&S58874[97], 9}, {NULL}};
const Segment SGI180898[] = {{&S58874[70], 6}, {NULL}};
const Segment SGI180899[] = {{&S58875[34], 11}, {NULL}};
const Segment SGI180900[] = {{&S58875[27], 6}, {NULL}};
const Segment SGI180901[] = {{&S58875[98], 9}, {NULL}};
const Segment SGI180902[] = {{&S58875[71], 6}, {NULL}};
const Segment SGI180903[] = {{&S58881[34], 10}, {NULL}};
const Segment SGI180904[] = {{&S58881[27], 6}, {NULL}};
const Segment SGI180905[] = {{&S58881[97], 9}, {NULL}};
const Segment SGI180906[] = {{&S58881[70], 6}, {NULL}};
const Segment SGI180907[] = {{&S63208[6], 4}, {NULL}};
const Segment SGI180908[] = {{&S63208[76], 5}, {NULL}};
const Segment SGI180909[] = {{&S57213[34], 20}, {NULL}};
const Segment SGI180910[] = {{&S57213[27], 6}, {NULL}};
const Segment SGI180911[] = {{&S57213[107], 9}, {NULL}};
const Segment SGI180912[] = {{&S57213[80], 6}, {NULL}};
const Segment SGI180913[] = {{&S57216[34], 19}, {NULL}};
const Segment SGI180914[] = {{&S57216[27], 6}, {NULL}};
const Segment SGI180915[] = {{&S57216[106], 9}, {NULL}};
const Segment SGI180916[] = {{&S57216[79], 6}, {NULL}};
const Segment SGI180917[] = {{&S57217[34], 21}, {NULL}};
const Segment SGI180918[] = {{&S57217[27], 6}, {NULL}};
const Segment SGI180919[] = {{&S57217[108], 9}, {NULL}};
const Segment SGI180920[] = {{&S57217[81], 6}, {NULL}};
const Segment SGI180921[] = {{&S57218[34], 14}, {NULL}};
const Segment SGI180922[] = {{&S57218[27], 6}, {NULL}};
const Segment SGI180923[] = {{&S57218[101], 9}, {NULL}};
const Segment SGI180924[] = {{&S57218[74], 6}, {NULL}};
const Segment SGI180925[] = {{&S57220[34], 26}, {NULL}};
const Segment SGI180926[] = {{&S57220[27], 6}, {NULL}};
const Segment SGI180927[] = {{&S57220[113], 9}, {NULL}};
const Segment SGI180928[] = {{&S57220[86], 6}, {NULL}};
const Segment SGI180929[] = {{&S57221[34], 26}, {NULL}};
const Segment SGI180930[] = {{&S57221[27], 6}, {NULL}};
const Segment SGI180931[] = {{&S57221[113], 9}, {NULL}};
const Segment SGI180932[] = {{&S57221[86], 6}, {NULL}};
const Segment SGI180933[] = {{&S63231[34], 12}, {NULL}};
const Segment SGI180934[] = {{&S63231[27], 6}, {NULL}};
const Segment SGI180935[] = {{&S63231[99], 9}, {NULL}};
const Segment SGI180936[] = {{&S63231[72], 6}, {NULL}};
const Segment SGI180937[] = {{&S63232[34], 12}, {NULL}};
const Segment SGI180938[] = {{&S63232[27], 6}, {NULL}};
const Segment SGI180939[] = {{&S63232[99], 9}, {NULL}};
const Segment SGI180940[] = {{&S63232[72], 6}, {NULL}};
const Segment SGI180941[] = {{&S47848[34], 29}, {NULL}};
const Segment SGI180942[] = {{&S47848[27], 6}, {NULL}};
const Segment SGI180943[] = {{&S47848[116], 9}, {NULL}};
const Segment SGI180944[] = {{&S47848[89], 6}, {NULL}};
const Segment SGI180945[] = {{&S70470[34], 18}, {NULL}};
const Segment SGI180946[] = {{&S70470[27], 6}, {NULL}};
const Segment SGI180947[] = {{&S70470[105], 9}, {NULL}};
const Segment SGI180948[] = {{&S70470[78], 6}, {NULL}};
const Segment SGI180949[] = {{&S58935[34], 16}, {NULL}};
const Segment SGI180950[] = {{&S58935[27], 6}, {NULL}};
const Segment SGI180951[] = {{&S58935[103], 9}, {NULL}};
const Segment SGI180952[] = {{&S58935[76], 6}, {NULL}};
const Segment SGI180953[] = {{&S58936[34], 16}, {NULL}};
const Segment SGI180954[] = {{&S58936[27], 6}, {NULL}};
const Segment SGI180955[] = {{&S58936[103], 9}, {NULL}};
const Segment SGI180956[] = {{&S58936[76], 6}, {NULL}};
const Segment SGI180957[] = {{&S70485[44], 9}, {NULL}};
const Segment SGI180958[] = {{&S70485[44], 21}, {NULL}};
const Segment SGI180959[] = {{&S70485[30], 6}, {NULL}};
const Segment SGI180960[] = {{&S70485[31], 3}, {NULL}};
const Segment SGI180961[] = {{&S70485[79], 6}, {NULL}};
const Segment SGI180962[] = {{&S70487[44], 10}, {NULL}};
const Segment SGI180963[] = {{&S70487[44], 22}, {NULL}};
const Segment SGI180964[] = {{&S70487[30], 6}, {NULL}};
const Segment SGI180965[] = {{&S70487[31], 3}, {NULL}};
const Segment SGI180966[] = {{&S70487[80], 6}, {NULL}};
const Segment SGI180967[] = {{&S70487[143], 3}, {NULL}};
const Segment SGI180968[] = {{&S70488[44], 10}, {NULL}};
const Segment SGI180969[] = {{&S70488[44], 22}, {NULL}};
const Segment SGI180970[] = {{&S70488[30], 6}, {NULL}};
const Segment SGI180971[] = {{&S70488[31], 3}, {NULL}};
const Segment SGI180972[] = {{&S70488[80], 6}, {NULL}};
const Segment SGI180973[] = {{&S70488[143], 3}, {NULL}};
const Segment SGI180974[] = {{&S57268[34], 11}, {NULL}};
const Segment SGI180975[] = {{&S57268[27], 6}, {NULL}};
const Segment SGI180976[] = {{&S57268[98], 9}, {NULL}};
const Segment SGI180977[] = {{&S57268[71], 6}, {NULL}};
const Segment SGI180978[] = {{&S47883[34], 10}, {NULL}};
const Segment SGI180979[] = {{&S47883[27], 6}, {NULL}};
const Segment SGI180980[] = {{&S47883[96], 9}, {NULL}};
const Segment SGI180981[] = {{&S47883[69], 6}, {NULL}};
const Segment SGI180982[] = {{&S47884[34], 10}, {NULL}};
const Segment SGI180983[] = {{&S47884[27], 6}, {NULL}};
const Segment SGI180984[] = {{&S47884[96], 9}, {NULL}};
const Segment SGI180985[] = {{&S47884[69], 6}, {NULL}};
const Segment SGI180986[] = {{&S47885[34], 18}, {NULL}};
const Segment SGI180987[] = {{&S47885[27], 6}, {NULL}};
const Segment SGI180988[] = {{&S47885[104], 9}, {NULL}};
const Segment SGI180989[] = {{&S47885[77], 6}, {NULL}};
const Segment SGI180990[] = {{&S47886[34], 11}, {NULL}};
const Segment SGI180991[] = {{&S70499[34], 10}, {NULL}};
const Segment SGI180992[] = {{&S47886[27], 6}, {NULL}};
const Segment SGI180993[] = {{&S70499[27], 6}, {NULL}};
const Segment SGI180994[] = {{&S47886[97], 9}, {NULL}};
const Segment SGI180995[] = {{&S70499[97], 9}, {NULL}};
const Segment SGI180996[] = {{&S47886[70], 6}, {NULL}};
const Segment SGI180997[] = {{&S70499[70], 6}, {NULL}};
const Segment SGI180998[] = {{&S47887[34], 10}, {NULL}};
const Segment SGI180999[] = {{&S47887[27], 6}, {NULL}};
const Segment SGI181000[] = {{&S47887[96], 9}, {NULL}};
const Segment SGI181001[] = {{&S47887[69], 6}, {NULL}};
const Segment SGI181002[] = {{&S70501[44], 9}, {NULL}};
const Segment SGI181003[] = {{&S70501[44], 21}, {NULL}};
const Segment SGI181004[] = {{&S70501[30], 6}, {NULL}};
const Segment SGI181005[] = {{&S70501[31], 3}, {NULL}};
const Segment SGI181006[] = {{&S70501[94], 6}, {NULL}};
const Segment SGI181007[] = {{&S70501[67], 8}, {NULL}};
const Segment SGI181008[] = {{&S70502[44], 9}, {NULL}};
const Segment SGI181009[] = {{&S63293[44], 11}, {NULL}};
const Segment SGI181010[] = {{&S70502[44], 21}, {NULL}};
const Segment SGI181011[] = {{&S63293[44], 23}, {NULL}};
const Segment SGI181012[] = {{&S70502[30], 6}, {NULL}};
const Segment SGI181013[] = {{&S63293[30], 6}, {NULL}};
const Segment SGI181014[] = {{&S70502[31], 3}, {NULL}};
const Segment SGI181015[] = {{&S63293[116], 3}, {NULL}};
const Segment SGI181016[] = {{&S63293[81], 6}, {NULL}};
const Segment SGI181017[] = {{&S70502[98], 6}, {NULL}};
const Segment SGI181018[] = {{&S70502[67], 8}, {NULL}};
const Segment SGI181019[] = {{&S63294[44], 12}, {NULL}};
const Segment SGI181020[] = {{&S63294[44], 24}, {NULL}};
const Segment SGI181021[] = {{&S63294[30], 6}, {NULL}};
const Segment SGI181022[] = {{&S63294[117], 3}, {NULL}};
const Segment SGI181023[] = {{&S63294[82], 6}, {NULL}};
const Segment SGI181024[] = {{&S58969[44], 10}, {NULL}};
const Segment SGI181025[] = {{&S58969[44], 22}, {NULL}};
const Segment SGI181026[] = {{&S58969[30], 6}, {NULL}};
const Segment SGI181027[] = {{&S58969[115], 3}, {NULL}};
const Segment SGI181028[] = {{&S58969[80], 6}, {NULL}};
const Segment SGI181029[] = {{&S70507[34], 10}, {NULL}};
const Segment SGI181030[] = {{&S70507[27], 6}, {NULL}};
const Segment SGI181031[] = {{&S70507[97], 9}, {NULL}};
const Segment SGI181032[] = {{&S70507[70], 6}, {NULL}};
const Segment SGI181033[] = {{&S58970[44], 9}, {NULL}};
const Segment SGI181034[] = {{&S58970[44], 21}, {NULL}};
const Segment SGI181035[] = {{&S58970[30], 6}, {NULL}};
const Segment SGI181036[] = {{&S58970[114], 3}, {NULL}};
const Segment SGI181037[] = {{&S58970[79], 6}, {NULL}};
const Segment SGI181038[] = {{&S70508[44], 23}, {NULL}};
const Segment SGI181039[] = {{&S70508[44], 34}, {NULL}};
const Segment SGI181040[] = {{&S70508[30], 6}, {NULL}};
const Segment SGI181041[] = {{&S58971[38], 9}, {NULL}};
const Segment SGI181042[] = {{&S58971[31], 6}, {NULL}};
const Segment SGI181043[] = {{&S70508[31], 3}, {NULL}};
const Segment SGI181044[] = {{&S70508[92], 6}, {NULL}};
const Segment SGI181045[] = {{&S58972[44], 10}, {NULL}};
const Segment SGI181046[] = {{&S58972[44], 22}, {NULL}};
const Segment SGI181047[] = {{&S58972[30], 6}, {NULL}};
const Segment SGI181048[] = {{&S58972[115], 3}, {NULL}};
const Segment SGI181049[] = {{&S58972[80], 6}, {NULL}};
const Segment SGI181050[] = {{&S58973[38], 10}, {NULL}};
const Segment SGI181051[] = {{&S58973[31], 6}, {NULL}};
const Segment SGI181052[] = {{&S63309[44], 7}, {NULL}};
const Segment SGI181053[] = {{&S63309[44], 19}, {NULL}};
const Segment SGI181054[] = {{&S63309[30], 6}, {NULL}};
const Segment SGI181055[] = {{&S63309[112], 3}, {NULL}};
const Segment SGI181056[] = {{&S63309[77], 6}, {NULL}};
const Segment SGI181057[] = {{&S63310[38], 7}, {NULL}};
const Segment SGI181058[] = {{&S63310[31], 6}, {NULL}};
const Segment SGI181059[] = {{&S63311[44], 16}, {NULL}};
const Segment SGI181060[] = {{&S63311[44], 28}, {NULL}};
const Segment SGI181061[] = {{&S63311[30], 6}, {NULL}};
const Segment SGI181062[] = {{&S63311[121], 3}, {NULL}};
const Segment SGI181063[] = {{&S63311[86], 6}, {NULL}};
const Segment SGI181064[] = {{&S63312[38], 16}, {NULL}};
const Segment SGI181065[] = {{&S63312[31], 6}, {NULL}};
const Segment SGI181066[] = {{&S63330[34], 29}, {NULL}};
const Segment SGI181067[] = {{&S63330[27], 6}, {NULL}};
const Segment SGI181068[] = {{&S63330[116], 9}, {NULL}};
const Segment SGI181069[] = {{&S63330[89], 6}, {NULL}};
const Segment SGI181070[] = {{&S63331[34], 30}, {NULL}};
const Segment SGI181071[] = {{&S63331[27], 6}, {NULL}};
const Segment SGI181072[] = {{&S63331[117], 9}, {NULL}};
const Segment SGI181073[] = {{&S63331[90], 6}, {NULL}};
const Segment SGI181074[] = {{&S57330[44], 9}, {NULL}};
const Segment SGI181075[] = {{&S57330[44], 21}, {NULL}};
const Segment SGI181076[] = {{&S57330[30], 6}, {NULL}};
const Segment SGI181077[] = {{&S57330[114], 3}, {NULL}};
const Segment SGI181078[] = {{&S57330[79], 6}, {NULL}};
const Segment SGI181079[] = {{&S57331[38], 9}, {NULL}};
const Segment SGI181080[] = {{&S57331[31], 6}, {NULL}};
const Segment SGI181081[] = {{&S57332[44], 9}, {NULL}};
const Segment SGI181082[] = {{&S57332[44], 21}, {NULL}};
const Segment SGI181083[] = {{&S57332[30], 6}, {NULL}};
const Segment SGI181084[] = {{&S57332[114], 3}, {NULL}};
const Segment SGI181085[] = {{&S57332[79], 6}, {NULL}};
const Segment SGI181086[] = {{&S57334[44], 8}, {NULL}};
const Segment SGI181087[] = {{&S57334[44], 20}, {NULL}};
const Segment SGI181088[] = {{&S57334[30], 6}, {NULL}};
const Segment SGI181089[] = {{&S57334[113], 3}, {NULL}};
const Segment SGI181090[] = {{&S57334[78], 6}, {NULL}};
const Segment SGI181091[] = {{&S59020[44], 11}, {NULL}};
const Segment SGI181092[] = {{&S59020[44], 23}, {NULL}};
const Segment SGI181093[] = {{&S59020[30], 6}, {NULL}};
const Segment SGI181094[] = {{&S59020[116], 3}, {NULL}};
const Segment SGI181095[] = {{&S59020[81], 6}, {NULL}};
const Segment SGI181096[] = {{&S59021[38], 11}, {NULL}};
const Segment SGI181097[] = {{&S59021[31], 6}, {NULL}};
const Segment SGI181098[] = {{&S47958[34], 29}, {NULL}};
const Segment SGI181099[] = {{&S47958[27], 6}, {NULL}};
const Segment SGI181100[] = {{&S47958[116], 9}, {NULL}};
const Segment SGI181101[] = {{&S47958[89], 6}, {NULL}};
const Segment SGI181102[] = {{&S63359[44], 10}, {NULL}};
const Segment SGI181103[] = {{&S63359[44], 22}, {NULL}};
const Segment SGI181104[] = {{&S63359[30], 6}, {NULL}};
const Segment SGI181105[] = {{&S63359[131], 3}, {NULL}};
const Segment SGI181106[] = {{&S63359[96], 6}, {NULL}};
const Segment SGI181107[] = {{&S63359[68], 8}, {NULL}};
const Segment SGI181108[] = {{&S63360[44], 10}, {NULL}};
const Segment SGI181109[] = {{&S63360[44], 22}, {NULL}};
const Segment SGI181110[] = {{&S63360[30], 6}, {NULL}};
const Segment SGI181111[] = {{&S63360[131], 3}, {NULL}};
const Segment SGI181112[] = {{&S63360[96], 6}, {NULL}};
const Segment SGI181113[] = {{&S63360[68], 8}, {NULL}};
const Segment SGI181114[] = {{&S63361[44], 10}, {NULL}};
const Segment SGI181115[] = {{&S63361[44], 22}, {NULL}};
const Segment SGI181116[] = {{&S63361[30], 6}, {NULL}};
const Segment SGI181117[] = {{&S63361[131], 3}, {NULL}};
const Segment SGI181118[] = {{&S63361[96], 6}, {NULL}};
const Segment SGI181119[] = {{&S63361[68], 8}, {NULL}};
const Segment SGI181120[] = {{&S63363[44], 17}, {NULL}};
const Segment SGI181121[] = {{&S63363[44], 29}, {NULL}};
const Segment SGI181122[] = {{&S63363[30], 6}, {NULL}};
const Segment SGI181123[] = {{&S63363[122], 3}, {NULL}};
const Segment SGI181124[] = {{&S63363[87], 6}, {NULL}};
const Segment SGI181125[] = {{&S63364[44], 18}, {NULL}};
const Segment SGI181126[] = {{&S63364[44], 30}, {NULL}};
const Segment SGI181127[] = {{&S63364[30], 6}, {NULL}};
const Segment SGI181128[] = {{&S63364[123], 3}, {NULL}};
const Segment SGI181129[] = {{&S63364[88], 6}, {NULL}};
const Segment SGI181130[] = {{&S63365[44], 8}, {NULL}};
const Segment SGI181131[] = {{&S63365[44], 20}, {NULL}};
const Segment SGI181132[] = {{&S63365[30], 6}, {NULL}};
const Segment SGI181133[] = {{&S63365[113], 3}, {NULL}};
const Segment SGI181134[] = {{&S63365[78], 6}, {NULL}};
const Segment SGI181135[] = {{&S63379[34], 18}, {NULL}};
const Segment SGI181136[] = {{&S63379[27], 6}, {NULL}};
const Segment SGI181137[] = {{&S63379[114], 9}, {NULL}};
const Segment SGI181138[] = {{&S63379[87], 6}, {NULL}};
const Segment SGI181139[] = {{&S63382[44], 7}, {NULL}};
const Segment SGI181140[] = {{&S63382[44], 19}, {NULL}};
const Segment SGI181141[] = {{&S63382[30], 6}, {NULL}};
const Segment SGI181142[] = {{&S63382[112], 3}, {NULL}};
const Segment SGI181143[] = {{&S63382[77], 6}, {NULL}};
const Segment SGI181144[] = {{&S63385[38], 7}, {NULL}};
const Segment SGI181145[] = {{&S63385[31], 6}, {NULL}};
const Segment SGI181146[] = {{&S47998[34], 11}, {NULL}};
const Segment SGI181147[] = {{&S47998[27], 6}, {NULL}};
const Segment SGI181148[] = {{&S47998[97], 9}, {NULL}};
const Segment SGI181149[] = {{&S47998[70], 6}, {NULL}};
const Segment SGI181150[] = {{&S63392[34], 9}, {NULL}};
const Segment SGI181151[] = {{&S63392[27], 6}, {NULL}};
const Segment SGI181152[] = {{&S63392[96], 9}, {NULL}};
const Segment SGI181153[] = {{&S63392[69], 6}, {NULL}};
const Segment SGI181154[] = {{&S63393[34], 9}, {NULL}};
const Segment SGI181155[] = {{&S63393[27], 6}, {NULL}};
const Segment SGI181156[] = {{&S63393[96], 9}, {NULL}};
const Segment SGI181157[] = {{&S63393[69], 6}, {NULL}};
const Segment SGI181158[] = {{&S48010[34], 10}, {NULL}};
const Segment SGI181159[] = {{&S48010[27], 6}, {NULL}};
const Segment SGI181160[] = {{&S59087[44], 10}, {NULL}};
const Segment SGI181161[] = {{&S59087[44], 22}, {NULL}};
const Segment SGI181162[] = {{&S59087[30], 6}, {NULL}};
const Segment SGI181163[] = {{&S48010[96], 9}, {NULL}};
const Segment SGI181164[] = {{&S48010[69], 6}, {NULL}};
const Segment SGI181165[] = {{&S59087[135], 3}, {NULL}};
const Segment SGI181166[] = {{&S59087[100], 6}, {NULL}};
const Segment SGI181167[] = {{&S59087[68], 8}, {NULL}};
const Segment SGI181168[] = {{&S59088[44], 10}, {NULL}};
const Segment SGI181169[] = {{&S59088[44], 22}, {NULL}};
const Segment SGI181170[] = {{&S59088[30], 6}, {NULL}};
const Segment SGI181171[] = {{&S59088[131], 3}, {NULL}};
const Segment SGI181172[] = {{&S48012[34], 10}, {NULL}};
const Segment SGI181173[] = {{&S48012[27], 6}, {NULL}};
const Segment SGI181174[] = {{&S59088[96], 6}, {NULL}};
const Segment SGI181175[] = {{&S59088[68], 8}, {NULL}};
const Segment SGI181176[] = {{&S48012[97], 9}, {NULL}};
const Segment SGI181177[] = {{&S48012[70], 6}, {NULL}};
const Segment SGI181178[] = {{&S59089[34], 10}, {NULL}};
const Segment SGI181179[] = {{&S59089[27], 6}, {NULL}};
const Segment SGI181180[] = {{&S48013[34], 11}, {NULL}};
const Segment SGI181181[] = {{&S48013[27], 6}, {NULL}};
const Segment SGI181182[] = {{&S59089[97], 9}, {NULL}};
const Segment SGI181183[] = {{&S59089[70], 6}, {NULL}};
const Segment SGI181184[] = {{&S48013[98], 9}, {NULL}};
const Segment SGI181185[] = {{&S48013[71], 6}, {NULL}};
const Segment SGI181186[] = {{&S59090[34], 10}, {NULL}};
const Segment SGI181187[] = {{&S59090[27], 6}, {NULL}};
const Segment SGI181188[] = {{&S48014[34], 11}, {NULL}};
const Segment SGI181189[] = {{&S48014[27], 6}, {NULL}};
const Segment SGI181190[] = {{&S59090[97], 9}, {NULL}};
const Segment SGI181191[] = {{&S59090[70], 6}, {NULL}};
const Segment SGI181192[] = {{&S48014[97], 9}, {NULL}};
const Segment SGI181193[] = {{&S48014[70], 6}, {NULL}};
const Segment SGI181194[] = {{&S59091[34], 10}, {NULL}};
const Segment SGI181195[] = {{&S59091[27], 6}, {NULL}};
const Segment SGI181196[] = {{&S59091[97], 9}, {NULL}};
const Segment SGI181197[] = {{&S59091[70], 6}, {NULL}};
const Segment SGI181198[] = {{&S59094[44], 10}, {NULL}};
const Segment SGI181199[] = {{&S59094[44], 22}, {NULL}};
const Segment SGI181200[] = {{&S59094[30], 6}, {NULL}};
const Segment SGI181201[] = {{&S59094[115], 3}, {NULL}};
const Segment SGI181202[] = {{&S59094[80], 6}, {NULL}};
const Segment SGI181203[] = {{&S59094[150], 3}, {NULL}};
const Segment SGI181204[] = {{&S59095[44], 18}, {NULL}};
const Segment SGI181205[] = {{&S59095[44], 30}, {NULL}};
const Segment SGI181206[] = {{&S59095[30], 6}, {NULL}};
const Segment SGI181207[] = {{&S59095[123], 3}, {NULL}};
const Segment SGI181208[] = {{&S59095[88], 6}, {NULL}};
const Segment SGI181209[] = {{&S59095[158], 3}, {NULL}};
const Segment SGI181210[] = {{&S59096[44], 11}, {NULL}};
const Segment SGI181211[] = {{&S59096[44], 23}, {NULL}};
const Segment SGI181212[] = {{&S59096[30], 6}, {NULL}};
const Segment SGI181213[] = {{&S59096[116], 3}, {NULL}};
const Segment SGI181214[] = {{&S59096[81], 6}, {NULL}};
const Segment SGI181215[] = {{&S59096[151], 3}, {NULL}};
const Segment SGI181216[] = {{&S59097[44], 10}, {NULL}};
const Segment SGI181217[] = {{&S59097[44], 22}, {NULL}};
const Segment SGI181218[] = {{&S59097[30], 6}, {NULL}};
const Segment SGI181219[] = {{&S59097[115], 3}, {NULL}};
const Segment SGI181220[] = {{&S59097[80], 6}, {NULL}};
const Segment SGI181221[] = {{&S59097[150], 3}, {NULL}};
const Segment SGI181222[] = {{&S59098[44], 10}, {NULL}};
const Segment SGI181223[] = {{&S59098[44], 22}, {NULL}};
const Segment SGI181224[] = {{&S59098[30], 6}, {NULL}};
const Segment SGI181225[] = {{&S59098[115], 3}, {NULL}};
const Segment SGI181226[] = {{&S59098[80], 6}, {NULL}};
const Segment SGI181227[] = {{&S59098[150], 3}, {NULL}};
const Segment SGI181228[] = {{&S59099[44], 10}, {NULL}};
const Segment SGI181229[] = {{&S59099[44], 22}, {NULL}};
const Segment SGI181230[] = {{&S59099[30], 6}, {NULL}};
const Segment SGI181231[] = {{&S59099[115], 3}, {NULL}};
const Segment SGI181232[] = {{&S59099[80], 6}, {NULL}};
const Segment SGI181233[] = {{&S59099[150], 3}, {NULL}};
const Segment SGI181234[] = {{&S63427[44], 4}, {NULL}};
const Segment SGI181235[] = {{&S63427[44], 16}, {NULL}};
const Segment SGI181236[] = {{&S63427[30], 6}, {NULL}};
const Segment SGI181237[] = {{&S63427[109], 3}, {NULL}};
const Segment SGI181238[] = {{&S63427[74], 6}, {NULL}};
const Segment SGI181239[] = {{&S63428[44], 4}, {NULL}};
const Segment SGI181240[] = {{&S63428[44], 16}, {NULL}};
const Segment SGI181241[] = {{&S63428[30], 6}, {NULL}};
const Segment SGI181242[] = {{&S63428[109], 3}, {NULL}};
const Segment SGI181243[] = {{&S63428[74], 6}, {NULL}};
const Segment SGI181244[] = {{&S63429[44], 4}, {NULL}};
const Segment SGI181245[] = {{&S63429[44], 16}, {NULL}};
const Segment SGI181246[] = {{&S63429[30], 6}, {NULL}};
const Segment SGI181247[] = {{&S63429[109], 3}, {NULL}};
const Segment SGI181248[] = {{&S63429[74], 6}, {NULL}};
const Segment SGI181249[] = {{&S63430[38], 4}, {NULL}};
const Segment SGI181250[] = {{&S63430[31], 6}, {NULL}};
const Segment SGI181251[] = {{&S59140[34], 10}, {NULL}};
const Segment SGI181252[] = {{&S59140[27], 6}, {NULL}};
const Segment SGI181253[] = {{&S59140[97], 9}, {NULL}};
const Segment SGI181254[] = {{&S59140[70], 6}, {NULL}};
const Segment SGI181255[] = {{&S48077[41], 18}, {NULL}};
const Segment SGI181256[] = {{&S48077[41], 29}, {NULL}};
const Segment SGI181257[] = {{&S48077[27], 6}, {NULL}};
const Segment SGI181258[] = {{&S48077[119], 3}, {NULL}};
const Segment SGI181259[] = {{&S48077[123], 9}, {NULL}};
const Segment SGI181260[] = {{&S48077[84], 6}, {NULL}};
const Segment SGI181261[] = {{&S59142[34], 10}, {NULL}};
const Segment SGI181262[] = {{&S59142[27], 6}, {NULL}};
const Segment SGI181263[] = {{&S59142[97], 9}, {NULL}};
const Segment SGI181264[] = {{&S59142[70], 6}, {NULL}};
const Segment SGI181265[] = {{&S48078[41], 31}, {NULL}};
const Segment SGI181266[] = {{&S48078[41], 42}, {NULL}};
const Segment SGI181267[] = {{&S48078[27], 6}, {NULL}};
const Segment SGI181268[] = {{&S48078[132], 3}, {NULL}};
const Segment SGI181269[] = {{&S48078[136], 9}, {NULL}};
const Segment SGI181270[] = {{&S48078[97], 6}, {NULL}};
const Segment SGI181271[] = {{&S63475[34], 10}, {NULL}};
const Segment SGI181272[] = {{&S63475[27], 6}, {NULL}};
const Segment SGI181273[] = {{&S63475[97], 9}, {NULL}};
const Segment SGI181274[] = {{&S63475[70], 6}, {NULL}};
const Segment SGI181275[] = {{&S63475[138], 8}, {NULL}};
const Segment SGI181276[] = {{&S59163[34], 12}, {NULL}};
const Segment SGI181277[] = {{&S59163[27], 6}, {NULL}};
const Segment SGI181278[] = {{&S59163[99], 9}, {NULL}};
const Segment SGI181279[] = {{&S59163[72], 6}, {NULL}};
const Segment SGI181280[] = {{&S63503[34], 10}, {NULL}};
const Segment SGI181281[] = {{&S63503[27], 6}, {NULL}};
const Segment SGI181282[] = {{&S63503[97], 9}, {NULL}};
const Segment SGI181283[] = {{&S63503[70], 6}, {NULL}};
const Segment SGI181284[] = {{&S63514[44], 10}, {NULL}};
const Segment SGI181285[] = {{&S63514[44], 22}, {NULL}};
const Segment SGI181286[] = {{&S63514[30], 6}, {NULL}};
const Segment SGI181287[] = {{&S63514[131], 3}, {NULL}};
const Segment SGI181288[] = {{&S63514[96], 6}, {NULL}};
const Segment SGI181289[] = {{&S63514[68], 8}, {NULL}};
const Segment SGI181290[] = {{&S63515[44], 10}, {NULL}};
const Segment SGI181291[] = {{&S63515[44], 22}, {NULL}};
const Segment SGI181292[] = {{&S63515[30], 6}, {NULL}};
const Segment SGI181293[] = {{&S63515[131], 3}, {NULL}};
const Segment SGI181294[] = {{&S63515[96], 6}, {NULL}};
const Segment SGI181295[] = {{&S63515[68], 8}, {NULL}};
const Segment SGI181296[] = {{&S59191[34], 7}, {NULL}};
const Segment SGI181297[] = {{&S59191[27], 6}, {NULL}};
const Segment SGI181298[] = {{&S59191[99], 9}, {NULL}};
const Segment SGI181299[] = {{&S59191[72], 6}, {NULL}};
const Segment SGI181300[] = {{&S48156[34], 18}, {NULL}};
const Segment SGI181301[] = {{&S48156[27], 6}, {NULL}};
const Segment SGI181302[] = {{&S48156[104], 9}, {NULL}};
const Segment SGI181303[] = {{&S48156[77], 6}, {NULL}};
const Segment SGI181304[] = {{&S63558[44], 9}, {NULL}};
const Segment SGI181305[] = {{&S63558[44], 21}, {NULL}};
const Segment SGI181306[] = {{&S63558[30], 6}, {NULL}};
const Segment SGI181307[] = {{&S63558[114], 3}, {NULL}};
const Segment SGI181308[] = {{&S63558[79], 6}, {NULL}};
const Segment SGI181309[] = {{&S63559[44], 12}, {NULL}};
const Segment SGI181310[] = {{&S63559[44], 24}, {NULL}};
const Segment SGI181311[] = {{&S63559[30], 6}, {NULL}};
const Segment SGI181312[] = {{&S63559[117], 3}, {NULL}};
const Segment SGI181313[] = {{&S63559[82], 6}, {NULL}};
const Segment SGI181314[] = {{&S63560[38], 12}, {NULL}};
const Segment SGI181315[] = {{&S63560[31], 6}, {NULL}};
const Segment SGI181316[] = {{&S59236[44], 10}, {NULL}};
const Segment SGI181317[] = {{&S59236[44], 22}, {NULL}};
const Segment SGI181318[] = {{&S59236[30], 6}, {NULL}};
const Segment SGI181319[] = {{&S59236[115], 3}, {NULL}};
const Segment SGI181320[] = {{&S59236[80], 6}, {NULL}};
const Segment SGI181321[] = {{&S59237[38], 10}, {NULL}};
const Segment SGI181322[] = {{&S59237[31], 6}, {NULL}};
const Segment SGI181323[] = {{&S59241[34], 15}, {NULL}};
const Segment SGI181324[] = {{&S59241[27], 6}, {NULL}};
const Segment SGI181325[] = {{&S59241[105], 9}, {NULL}};
const Segment SGI181326[] = {{&S59241[78], 6}, {NULL}};
const Segment SGI181327[] = {{&S59242[34], 7}, {NULL}};
const Segment SGI181328[] = {{&S59242[27], 6}, {NULL}};
const Segment SGI181329[] = {{&S59242[111], 9}, {NULL}};
const Segment SGI181330[] = {{&S59242[84], 6}, {NULL}};
const Segment SGI181331[] = {{&S63574[34], 10}, {NULL}};
const Segment SGI181332[] = {{&S63574[27], 6}, {NULL}};
const Segment SGI181333[] = {{&S63574[97], 9}, {NULL}};
const Segment SGI181334[] = {{&S63574[70], 6}, {NULL}};
const Segment SGI181335[] = {{&S63574[131], 8}, {NULL}};
const Segment SGI181336[] = {{&S63576[34], 15}, {NULL}};
const Segment SGI181337[] = {{&S63576[27], 6}, {NULL}};
const Segment SGI181338[] = {{&S63576[102], 9}, {NULL}};
const Segment SGI181339[] = {{&S63576[75], 6}, {NULL}};
const Segment SGI181340[] = {{&S48196[34], 11}, {NULL}};
const Segment SGI181341[] = {{&S48196[27], 6}, {NULL}};
const Segment SGI181342[] = {{&S63582[34], 10}, {NULL}};
const Segment SGI181343[] = {{&S63582[27], 6}, {NULL}};
const Segment SGI181344[] = {{&S48196[97], 9}, {NULL}};
const Segment SGI181345[] = {{&S48196[70], 6}, {NULL}};
const Segment SGI181346[] = {{&S63582[97], 9}, {NULL}};
const Segment SGI181347[] = {{&S63582[70], 6}, {NULL}};
const Segment SGI181348[] = {{&S63582[138], 8}, {NULL}};
const Segment SGI181349[] = {{&S57608[34], 9}, {NULL}};
const Segment SGI181350[] = {{&S57608[27], 6}, {NULL}};
const Segment SGI181351[] = {{&S57608[96], 9}, {NULL}};
const Segment SGI181352[] = {{&S57608[69], 6}, {NULL}};
const Segment SGI181353[] = {{&S57608[130], 8}, {NULL}};
const Segment SGI181354[] = {{&S70509[44], 23}, {NULL}};
const Segment SGI181355[] = {{&S70509[44], 34}, {NULL}};
const Segment SGI181356[] = {{&S70509[30], 6}, {NULL}};
const Segment SGI181357[] = {{&S70509[31], 3}, {NULL}};
const Segment SGI181358[] = {{&S70509[92], 6}, {NULL}};
const Segment SGI181359[] = {{&S63589[34], 18}, {NULL}};
const Segment SGI181360[] = {{&S63589[27], 6}, {NULL}};
const Segment SGI181361[] = {{&S63589[105], 9}, {NULL}};
const Segment SGI181362[] = {{&S63589[78], 6}, {NULL}};
const Segment SGI181363[] = {{&S63589[146], 8}, {NULL}};
const Segment SGI181364[] = {{&S48197[34], 11}, {NULL}};
const Segment SGI181365[] = {{&S48197[27], 6}, {NULL}};
const Segment SGI181366[] = {{&S48197[97], 9}, {NULL}};
const Segment SGI181367[] = {{&S48197[70], 6}, {NULL}};
const Segment SGI181368[] = {{&S57618[44], 17}, {NULL}};
const Segment SGI181369[] = {{&S57618[44], 29}, {NULL}};
const Segment SGI181370[] = {{&S57618[30], 6}, {NULL}};
const Segment SGI181371[] = {{&S57618[122], 3}, {NULL}};
const Segment SGI181372[] = {{&S57618[87], 6}, {NULL}};
const Segment SGI181373[] = {{&S57619[38], 17}, {NULL}};
const Segment SGI181374[] = {{&S57619[31], 6}, {NULL}};
const Segment SGI181375[] = {{&S70528[34], 10}, {NULL}};
const Segment SGI181376[] = {{&S70528[27], 6}, {NULL}};
const Segment SGI181377[] = {{&S70528[97], 9}, {NULL}};
const Segment SGI181378[] = {{&S70528[70], 6}, {NULL}};
const Segment SGI181379[] = {{&S63616[44], 12}, {NULL}};
const Segment SGI181380[] = {{&S63616[44], 24}, {NULL}};
const Segment SGI181381[] = {{&S63616[30], 6}, {NULL}};
const Segment SGI181382[] = {{&S63616[117], 3}, {NULL}};
const Segment SGI181383[] = {{&S63616[82], 6}, {NULL}};
const Segment SGI181384[] = {{&S63617[38], 12}, {NULL}};
const Segment SGI181385[] = {{&S63617[31], 6}, {NULL}};
const Segment SGI181386[] = {{&S70547[34], 10}, {NULL}};
const Segment SGI181387[] = {{&S70547[27], 6}, {NULL}};
const Segment SGI181388[] = {{&S63618[44], 15}, {NULL}};
const Segment SGI181389[] = {{&S70547[97], 9}, {NULL}};
const Segment SGI181390[] = {{&S63618[44], 27}, {NULL}};
const Segment SGI181391[] = {{&S70547[70], 6}, {NULL}};
const Segment SGI181392[] = {{&S63618[30], 6}, {NULL}};
const Segment SGI181393[] = {{&S70547[138], 8}, {NULL}};
const Segment SGI181394[] = {{&S63618[120], 3}, {NULL}};
const Segment SGI181395[] = {{&S63618[85], 6}, {NULL}};
const Segment SGI181396[] = {{&S70549[34], 11}, {NULL}};
const Segment SGI181397[] = {{&S70549[27], 6}, {NULL}};
const Segment SGI181398[] = {{&S70549[98], 9}, {NULL}};
const Segment SGI181399[] = {{&S70549[71], 6}, {NULL}};
const Segment SGI181400[] = {{&S70549[139], 8}, {NULL}};
const Segment SGI181401[] = {{&S70551[34], 15}, {NULL}};
const Segment SGI181402[] = {{&S70551[27], 6}, {NULL}};
const Segment SGI181403[] = {{&S70551[102], 9}, {NULL}};
const Segment SGI181404[] = {{&S70551[75], 6}, {NULL}};
const Segment SGI181405[] = {{&S70558[34], 9}, {NULL}};
const Segment SGI181406[] = {{&S70558[27], 6}, {NULL}};
const Segment SGI181407[] = {{&S70558[96], 9}, {NULL}};
const Segment SGI181408[] = {{&S70558[69], 6}, {NULL}};
const Segment SGI181409[] = {{&S70559[34], 11}, {NULL}};
const Segment SGI181410[] = {{&S70559[27], 6}, {NULL}};
const Segment SGI181411[] = {{&S70559[98], 9}, {NULL}};
const Segment SGI181412[] = {{&S70559[71], 6}, {NULL}};
const Segment SGI181413[] = {{&S70560[34], 10}, {NULL}};
const Segment SGI181414[] = {{&S70560[27], 6}, {NULL}};
const Segment SGI181415[] = {{&S70560[97], 9}, {NULL}};
const Segment SGI181416[] = {{&S70560[70], 6}, {NULL}};
const Segment SGI181417[] = {{&S63649[44], 19}, {NULL}};
const Segment SGI181418[] = {{&S63649[44], 31}, {NULL}};
const Segment SGI181419[] = {{&S63649[30], 6}, {NULL}};
const Segment SGI181420[] = {{&S63649[124], 3}, {NULL}};
const Segment SGI181421[] = {{&S63649[89], 6}, {NULL}};
const Segment SGI181422[] = {{&S63650[44], 19}, {NULL}};
const Segment SGI181423[] = {{&S63650[44], 31}, {NULL}};
const Segment SGI181424[] = {{&S63650[30], 6}, {NULL}};
const Segment SGI181425[] = {{&S70575[34], 9}, {NULL}};
const Segment SGI181426[] = {{&S63650[124], 3}, {NULL}};
const Segment SGI181427[] = {{&S70575[27], 6}, {NULL}};
const Segment SGI181428[] = {{&S63650[89], 6}, {NULL}};
const Segment SGI181429[] = {{&S70575[96], 9}, {NULL}};
const Segment SGI181430[] = {{&S70575[69], 6}, {NULL}};
const Segment SGI181431[] = {{&S63651[44], 19}, {NULL}};
const Segment SGI181432[] = {{&S70576[34], 10}, {NULL}};
const Segment SGI181433[] = {{&S63651[44], 31}, {NULL}};
const Segment SGI181434[] = {{&S63651[30], 6}, {NULL}};
const Segment SGI181435[] = {{&S70576[27], 6}, {NULL}};
const Segment SGI181436[] = {{&S63651[124], 3}, {NULL}};
const Segment SGI181437[] = {{&S70576[97], 9}, {NULL}};
const Segment SGI181438[] = {{&S63651[89], 6}, {NULL}};
const Segment SGI181439[] = {{&S70576[70], 6}, {NULL}};
const Segment SGI181440[] = {{&S63652[44], 13}, {NULL}};
const Segment SGI181441[] = {{&S63652[44], 25}, {NULL}};
const Segment SGI181442[] = {{&S63652[30], 6}, {NULL}};
const Segment SGI181443[] = {{&S63652[118], 3}, {NULL}};
const Segment SGI181444[] = {{&S63652[83], 6}, {NULL}};
const Segment SGI181445[] = {{&S70578[34], 10}, {NULL}};
const Segment SGI181446[] = {{&S70578[27], 6}, {NULL}};
const Segment SGI181447[] = {{&S70578[97], 9}, {NULL}};
const Segment SGI181448[] = {{&S70578[70], 6}, {NULL}};
const Segment SGI181449[] = {{&S70579[34], 7}, {NULL}};
const Segment SGI181450[] = {{&S70579[27], 6}, {NULL}};
const Segment SGI181451[] = {{&S70579[100], 9}, {NULL}};
const Segment SGI181452[] = {{&S70579[73], 6}, {NULL}};
const Segment SGI181453[] = {{&S63656[34], 15}, {NULL}};
const Segment SGI181454[] = {{&S63656[27], 6}, {NULL}};
const Segment SGI181455[] = {{&S70580[34], 7}, {NULL}};
const Segment SGI181456[] = {{&S70580[27], 6}, {NULL}};
const Segment SGI181457[] = {{&S63656[102], 9}, {NULL}};
const Segment SGI181458[] = {{&S63656[75], 6}, {NULL}};
const Segment SGI181459[] = {{&S70580[100], 9}, {NULL}};
const Segment SGI181460[] = {{&S70580[73], 6}, {NULL}};
const Segment SGI181461[] = {{&S63657[34], 6}, {NULL}};
const Segment SGI181462[] = {{&S63657[27], 6}, {NULL}};
const Segment SGI181463[] = {{&S63657[93], 9}, {NULL}};
const Segment SGI181464[] = {{&S63657[66], 6}, {NULL}};
const Segment SGI181465[] = {{&S70582[34], 9}, {NULL}};
const Segment SGI181466[] = {{&S70582[27], 6}, {NULL}};
const Segment SGI181467[] = {{&S70582[96], 9}, {NULL}};
const Segment SGI181468[] = {{&S70582[69], 6}, {NULL}};
const Segment SGI181469[] = {{&S70594[44], 6}, {NULL}};
const Segment SGI181470[] = {{&S70594[44], 18}, {NULL}};
const Segment SGI181471[] = {{&S70594[30], 6}, {NULL}};
const Segment SGI181472[] = {{&S70594[31], 3}, {NULL}};
const Segment SGI181473[] = {{&S70594[76], 6}, {NULL}};
const Segment SGI181474[] = {{&S63673[44], 9}, {NULL}};
const Segment SGI181475[] = {{&S63673[44], 21}, {NULL}};
const Segment SGI181476[] = {{&S63673[30], 6}, {NULL}};
const Segment SGI181477[] = {{&S63673[114], 3}, {NULL}};
const Segment SGI181478[] = {{&S63673[79], 6}, {NULL}};
const Segment SGI181479[] = {{&S63674[38], 9}, {NULL}};
const Segment SGI181480[] = {{&S63674[31], 6}, {NULL}};
const Segment SGI181481[] = {{&S63683[44], 10}, {NULL}};
const Segment SGI181482[] = {{&S63683[44], 22}, {NULL}};
const Segment SGI181483[] = {{&S63683[30], 6}, {NULL}};
const Segment SGI181484[] = {{&S63683[115], 3}, {NULL}};
const Segment SGI181485[] = {{&S63683[80], 6}, {NULL}};
const Segment SGI181486[] = {{&S63684[44], 10}, {NULL}};
const Segment SGI181487[] = {{&S63684[44], 22}, {NULL}};
const Segment SGI181488[] = {{&S63684[30], 6}, {NULL}};
const Segment SGI181489[] = {{&S63684[115], 3}, {NULL}};
const Segment SGI181490[] = {{&S63684[80], 6}, {NULL}};
const Segment SGI181491[] = {{&S70612[44], 10}, {NULL}};
const Segment SGI181492[] = {{&S70612[44], 22}, {NULL}};
const Segment SGI181493[] = {{&S70612[30], 6}, {NULL}};
const Segment SGI181494[] = {{&S70612[31], 3}, {NULL}};
const Segment SGI181495[] = {{&S70612[80], 6}, {NULL}};
const Segment SGI181496[] = {{&S63685[44], 10}, {NULL}};
const Segment SGI181497[] = {{&S63685[44], 22}, {NULL}};
const Segment SGI181498[] = {{&S63685[30], 6}, {NULL}};
const Segment SGI181499[] = {{&S70613[44], 10}, {NULL}};
const Segment SGI181500[] = {{&S70613[44], 22}, {NULL}};
const Segment SGI181501[] = {{&S63685[115], 3}, {NULL}};
const Segment SGI181502[] = {{&S70613[30], 6}, {NULL}};
const Segment SGI181503[] = {{&S63685[80], 6}, {NULL}};
const Segment SGI181504[] = {{&S70613[31], 3}, {NULL}};
const Segment SGI181505[] = {{&S70613[80], 6}, {NULL}};
const Segment SGI181506[] = {{&S63686[44], 10}, {NULL}};
const Segment SGI181507[] = {{&S63686[44], 22}, {NULL}};
const Segment SGI181508[] = {{&S70614[44], 10}, {NULL}};
const Segment SGI181509[] = {{&S63686[30], 6}, {NULL}};
const Segment SGI181510[] = {{&S70614[44], 17}, {NULL}};
const Segment SGI181511[] = {{&S70614[30], 6}, {NULL}};
const Segment SGI181512[] = {{&S63686[115], 3}, {NULL}};
const Segment SGI181513[] = {{&S70614[31], 3}, {NULL}};
const Segment SGI181514[] = {{&S63686[80], 6}, {NULL}};
const Segment SGI181515[] = {{&S70614[75], 6}, {NULL}};
const Segment SGI181516[] = {{&S48316[34], 10}, {NULL}};
const Segment SGI181517[] = {{&S48316[27], 6}, {NULL}};
const Segment SGI181518[] = {{&S48316[96], 9}, {NULL}};
const Segment SGI181519[] = {{&S48316[69], 6}, {NULL}};
const Segment SGI181520[] = {{&S70615[44], 10}, {NULL}};
const Segment SGI181521[] = {{&S70615[44], 22}, {NULL}};
const Segment SGI181522[] = {{&S70615[30], 6}, {NULL}};
const Segment SGI181523[] = {{&S70615[31], 3}, {NULL}};
const Segment SGI181524[] = {{&S70615[80], 6}, {NULL}};
const Segment SGI181525[] = {{&S70616[34], 10}, {NULL}};
const Segment SGI181526[] = {{&S70616[27], 6}, {NULL}};
const Segment SGI181527[] = {{&S70616[97], 9}, {NULL}};
const Segment SGI181528[] = {{&S70616[70], 6}, {NULL}};
const Segment SGI181529[] = {{&S70617[34], 9}, {NULL}};
const Segment SGI181530[] = {{&S63691[44], 22}, {NULL}};
const Segment SGI181531[] = {{&S70617[27], 6}, {NULL}};
const Segment SGI181532[] = {{&S63691[44], 34}, {NULL}};
const Segment SGI181533[] = {{&S63691[30], 6}, {NULL}};
const Segment SGI181534[] = {{&S70617[96], 9}, {NULL}};
const Segment SGI181535[] = {{&S63691[127], 3}, {NULL}};
const Segment SGI181536[] = {{&S70617[69], 6}, {NULL}};
const Segment SGI181537[] = {{&S63691[92], 6}, {NULL}};
const Segment SGI181538[] = {{&S70619[34], 16}, {NULL}};
const Segment SGI181539[] = {{&S70619[27], 6}, {NULL}};
const Segment SGI181540[] = {{&S70619[103], 9}, {NULL}};
const Segment SGI181541[] = {{&S70619[76], 6}, {NULL}};
const Segment SGI181542[] = {{&S70623[44], 10}, {NULL}};
const Segment SGI181543[] = {{&S70623[44], 22}, {NULL}};
const Segment SGI181544[] = {{&S70623[30], 6}, {NULL}};
const Segment SGI181545[] = {{&S70623[31], 3}, {NULL}};
const Segment SGI181546[] = {{&S70623[80], 6}, {NULL}};
const Segment SGI181547[] = {{&S70624[44], 8}, {NULL}};
const Segment SGI181548[] = {{&S70624[44], 20}, {NULL}};
const Segment SGI181549[] = {{&S70624[30], 6}, {NULL}};
const Segment SGI181550[] = {{&S70624[31], 3}, {NULL}};
const Segment SGI181551[] = {{&S70624[78], 6}, {NULL}};
const Segment SGI181552[] = {{&S63700[34], 18}, {NULL}};
const Segment SGI181553[] = {{&S63700[27], 6}, {NULL}};
const Segment SGI181554[] = {{&S63700[105], 9}, {NULL}};
const Segment SGI181555[] = {{&S63700[78], 6}, {NULL}};
const Segment SGI181556[] = {{&S70625[38], 10}, {NULL}};
const Segment SGI181557[] = {{&S70625[31], 6}, {NULL}};
const Segment SGI181558[] = {{&S70626[65], 10}, {NULL}};
const Segment SGI181559[] = {{&S70626[65], 22}, {NULL}};
const Segment SGI181560[] = {{&S70626[51], 6}, {NULL}};
const Segment SGI181561[] = {{&S63703[34], 6}, {NULL}};
const Segment SGI181562[] = {{&S63703[27], 6}, {NULL}};
const Segment SGI181563[] = {{&S63703[93], 9}, {NULL}};
const Segment SGI181564[] = {{&S63703[66], 6}, {NULL}};
const Segment SGI181565[] = {{&S70627[42], 10}, {NULL}};
const Segment SGI181566[] = {{&S70627[30], 6}, {NULL}};
const Segment SGI181567[] = {{&S70627[31], 3}, {NULL}};
const Segment SGI181568[] = {{&S70627[78], 6}, {NULL}};
const Segment SGI181569[] = {{&S70628[44], 20}, {NULL}};
const Segment SGI181570[] = {{&S70628[44], 32}, {NULL}};
const Segment SGI181571[] = {{&S70628[30], 6}, {NULL}};
const Segment SGI181572[] = {{&S48339[44], 10}, {NULL}};
const Segment SGI181573[] = {{&S48339[44], 21}, {NULL}};
const Segment SGI181574[] = {{&S70628[31], 3}, {NULL}};
const Segment SGI181575[] = {{&S48339[30], 6}, {NULL}};
const Segment SGI181576[] = {{&S70628[90], 6}, {NULL}};
const Segment SGI181577[] = {{&S48339[143], 3}, {NULL}};
const Segment SGI181578[] = {{&S48339[108], 6}, {NULL}};
const Segment SGI181579[] = {{&S48340[44], 10}, {NULL}};
const Segment SGI181580[] = {{&S48340[44], 21}, {NULL}};
const Segment SGI181581[] = {{&S48340[30], 6}, {NULL}};
const Segment SGI181582[] = {{&S48340[143], 3}, {NULL}};
const Segment SGI181583[] = {{&S48340[108], 6}, {NULL}};
const Segment SGI181584[] = {{&S48341[44], 11}, {NULL}};
const Segment SGI181585[] = {{&S63711[34], 15}, {NULL}};
const Segment SGI181586[] = {{&S48341[44], 22}, {NULL}};
const Segment SGI181587[] = {{&S63711[27], 6}, {NULL}};
const Segment SGI181588[] = {{&S48341[30], 6}, {NULL}};
const Segment SGI181589[] = {{&S48341[144], 3}, {NULL}};
const Segment SGI181590[] = {{&S63711[102], 9}, {NULL}};
const Segment SGI181591[] = {{&S63711[75], 6}, {NULL}};
const Segment SGI181592[] = {{&S48341[109], 6}, {NULL}};
const Segment SGI181593[] = {{&S48342[44], 10}, {NULL}};
const Segment SGI181594[] = {{&S48342[44], 21}, {NULL}};
const Segment SGI181595[] = {{&S48342[30], 6}, {NULL}};
const Segment SGI181596[] = {{&S48342[143], 3}, {NULL}};
const Segment SGI181597[] = {{&S48342[108], 6}, {NULL}};
const Segment SGI181598[] = {{&S48343[44], 10}, {NULL}};
const Segment SGI181599[] = {{&S48343[44], 22}, {NULL}};
const Segment SGI181600[] = {{&S48343[30], 6}, {NULL}};
const Segment SGI181601[] = {{&S48343[115], 3}, {NULL}};
const Segment SGI181602[] = {{&S48343[80], 6}, {NULL}};
const Segment SGI181603[] = {{&S48345[44], 10}, {NULL}};
const Segment SGI181604[] = {{&S48345[44], 22}, {NULL}};
const Segment SGI181605[] = {{&S48345[30], 6}, {NULL}};
const Segment SGI181606[] = {{&S48345[115], 3}, {NULL}};
const Segment SGI181607[] = {{&S48345[80], 6}, {NULL}};
const Segment SGI181608[] = {{&S48346[44], 10}, {NULL}};
const Segment SGI181609[] = {{&S48346[44], 22}, {NULL}};
const Segment SGI181610[] = {{&S48346[30], 6}, {NULL}};
const Segment SGI181611[] = {{&S48346[115], 3}, {NULL}};
const Segment SGI181612[] = {{&S48346[80], 6}, {NULL}};
const Segment SGI181613[] = {{&S70652[34], 10}, {NULL}};
const Segment SGI181614[] = {{&S70652[27], 6}, {NULL}};
const Segment SGI181615[] = {{&S70652[97], 9}, {NULL}};
const Segment SGI181616[] = {{&S70652[70], 6}, {NULL}};
const Segment SGI181617[] = {{&S70652[138], 8}, {NULL}};
const Segment SGI181618[] = {{&S70653[34], 10}, {NULL}};
const Segment SGI181619[] = {{&S70653[27], 6}, {NULL}};
const Segment SGI181620[] = {{&S70653[97], 9}, {NULL}};
const Segment SGI181621[] = {{&S70653[70], 6}, {NULL}};
const Segment SGI181622[] = {{&S70653[138], 8}, {NULL}};
const Segment SGI181623[] = {{&S70658[44], 7}, {NULL}};
const Segment SGI181624[] = {{&S70658[44], 19}, {NULL}};
const Segment SGI181625[] = {{&S70658[30], 6}, {NULL}};
const Segment SGI181626[] = {{&S70658[31], 3}, {NULL}};
const Segment SGI181627[] = {{&S70658[92], 6}, {NULL}};
const Segment SGI181628[] = {{&S70658[65], 8}, {NULL}};
const Segment SGI181629[] = {{&S70659[44], 7}, {NULL}};
const Segment SGI181630[] = {{&S70659[44], 19}, {NULL}};
const Segment SGI181631[] = {{&S70659[30], 6}, {NULL}};
const Segment SGI181632[] = {{&S70659[31], 3}, {NULL}};
const Segment SGI181633[] = {{&S70659[92], 6}, {NULL}};
const Segment SGI181634[] = {{&S70659[65], 8}, {NULL}};
const Segment SGI181635[] = {{&S70660[44], 7}, {NULL}};
const Segment SGI181636[] = {{&S70660[44], 19}, {NULL}};
const Segment SGI181637[] = {{&S70660[30], 6}, {NULL}};
const Segment SGI181638[] = {{&S70660[31], 3}, {NULL}};
const Segment SGI181639[] = {{&S70660[92], 6}, {NULL}};
const Segment SGI181640[] = {{&S70660[65], 8}, {NULL}};
const Segment SGI181641[] = {{&S70661[44], 11}, {NULL}};
const Segment SGI181642[] = {{&S70661[44], 23}, {NULL}};
const Segment SGI181643[] = {{&S70661[30], 6}, {NULL}};
const Segment SGI181644[] = {{&S70661[31], 3}, {NULL}};
const Segment SGI181645[] = {{&S70661[96], 6}, {NULL}};
const Segment SGI181646[] = {{&S70661[69], 8}, {NULL}};
const Segment SGI181647[] = {{&S70662[44], 11}, {NULL}};
const Segment SGI181648[] = {{&S70662[44], 23}, {NULL}};
const Segment SGI181649[] = {{&S70662[30], 6}, {NULL}};
const Segment SGI181650[] = {{&S70662[31], 3}, {NULL}};
const Segment SGI181651[] = {{&S70662[98], 6}, {NULL}};
const Segment SGI181652[] = {{&S70662[69], 8}, {NULL}};
const Segment SGI181653[] = {{&S70672[44], 13}, {NULL}};
const Segment SGI181654[] = {{&S70672[44], 25}, {NULL}};
const Segment SGI181655[] = {{&S70672[30], 6}, {NULL}};
const Segment SGI181656[] = {{&S70672[31], 3}, {NULL}};
const Segment SGI181657[] = {{&S70672[83], 6}, {NULL}};
const Segment SGI181658[] = {{&S70673[44], 16}, {NULL}};
const Segment SGI181659[] = {{&S70673[44], 28}, {NULL}};
const Segment SGI181660[] = {{&S70673[30], 6}, {NULL}};
const Segment SGI181661[] = {{&S70673[31], 3}, {NULL}};
const Segment SGI181662[] = {{&S70673[86], 6}, {NULL}};
const Segment SGI181663[] = {{&S70679[44], 8}, {NULL}};
const Segment SGI181664[] = {{&S70679[44], 20}, {NULL}};
const Segment SGI181665[] = {{&S70679[30], 6}, {NULL}};
const Segment SGI181666[] = {{&S70679[31], 3}, {NULL}};
const Segment SGI181667[] = {{&S70679[78], 6}, {NULL}};
const Segment SGI181668[] = {{&S70682[44], 4}, {NULL}};
const Segment SGI181669[] = {{&S70682[44], 16}, {NULL}};
const Segment SGI181670[] = {{&S70682[30], 6}, {NULL}};
const Segment SGI181671[] = {{&S70682[31], 3}, {NULL}};
const Segment SGI181672[] = {{&S70682[74], 6}, {NULL}};
const Segment SGI181673[] = {{&S70683[44], 9}, {NULL}};
const Segment SGI181674[] = {{&S70683[44], 21}, {NULL}};
const Segment SGI181675[] = {{&S70683[30], 6}, {NULL}};
const Segment SGI181676[] = {{&S70683[31], 3}, {NULL}};
const Segment SGI181677[] = {{&S70683[79], 6}, {NULL}};
const Segment SGI181678[] = {{&S70689[34], 9}, {NULL}};
const Segment SGI181679[] = {{&S70689[27], 6}, {NULL}};
const Segment SGI181680[] = {{&S70689[96], 9}, {NULL}};
const Segment SGI181681[] = {{&S70689[69], 6}, {NULL}};
const Segment SGI181682[] = {{&S63785[34], 9}, {NULL}};
const Segment SGI181683[] = {{&S63785[27], 6}, {NULL}};
const Segment SGI181684[] = {{&S63785[96], 9}, {NULL}};
const Segment SGI181685[] = {{&S63785[69], 6}, {NULL}};
const Segment SGI181686[] = {{&S70696[44], 10}, {NULL}};
const Segment SGI181687[] = {{&S70696[44], 22}, {NULL}};
const Segment SGI181688[] = {{&S70696[30], 6}, {NULL}};
const Segment SGI181689[] = {{&S70696[31], 3}, {NULL}};
const Segment SGI181690[] = {{&S70696[95], 6}, {NULL}};
const Segment SGI181691[] = {{&S70696[68], 8}, {NULL}};
const Segment SGI181692[] = {{&S70697[44], 10}, {NULL}};
const Segment SGI181693[] = {{&S70697[44], 22}, {NULL}};
const Segment SGI181694[] = {{&S70697[30], 6}, {NULL}};
const Segment SGI181695[] = {{&S70697[31], 3}, {NULL}};
const Segment SGI181696[] = {{&S70697[95], 6}, {NULL}};
const Segment SGI181697[] = {{&S70697[68], 8}, {NULL}};
const Segment SGI181698[] = {{&S70698[44], 10}, {NULL}};
const Segment SGI181699[] = {{&S70698[44], 22}, {NULL}};
const Segment SGI181700[] = {{&S70698[30], 6}, {NULL}};
const Segment SGI181701[] = {{&S70698[31], 3}, {NULL}};
const Segment SGI181702[] = {{&S70698[95], 6}, {NULL}};
const Segment SGI181703[] = {{&S70698[68], 8}, {NULL}};
const Segment SGI181704[] = {{&S63794[34], 18}, {NULL}};
const Segment SGI181705[] = {{&S63794[27], 6}, {NULL}};
const Segment SGI181706[] = {{&S63794[105], 9}, {NULL}};
const Segment SGI181707[] = {{&S63794[78], 6}, {NULL}};
const Segment SGI181708[] = {{&S57855[34], 6}, {NULL}};
const Segment SGI181709[] = {{&S57855[27], 6}, {NULL}};
const Segment SGI181710[] = {{&S57855[93], 9}, {NULL}};
const Segment SGI181711[] = {{&S57855[66], 6}, {NULL}};
const Segment SGI181712[] = {{&S57856[34], 13}, {NULL}};
const Segment SGI181713[] = {{&S57856[27], 6}, {NULL}};
const Segment SGI181714[] = {{&S57856[113], 9}, {NULL}};
const Segment SGI181715[] = {{&S57856[86], 6}, {NULL}};
const Segment SGI181716[] = {{&S57857[34], 10}, {NULL}};
const Segment SGI181717[] = {{&S57857[27], 6}, {NULL}};
const Segment SGI181718[] = {{&S57857[97], 9}, {NULL}};
const Segment SGI181719[] = {{&S57857[70], 6}, {NULL}};
const Segment SGI181720[] = {{&S57864[34], 9}, {NULL}};
const Segment SGI181721[] = {{&S57864[27], 6}, {NULL}};
const Segment SGI181722[] = {{&S57864[96], 9}, {NULL}};
const Segment SGI181723[] = {{&S57864[69], 6}, {NULL}};
const Segment SGI181724[] = {{&S57869[34], 9}, {NULL}};
const Segment SGI181725[] = {{&S57869[27], 6}, {NULL}};
const Segment SGI181726[] = {{&S57869[96], 9}, {NULL}};
const Segment SGI181727[] = {{&S57869[69], 6}, {NULL}};
const Segment SGI181728[] = {{&S57874[34], 11}, {NULL}};
const Segment SGI181729[] = {{&S57874[27], 6}, {NULL}};
const Segment SGI181730[] = {{&S57874[98], 9}, {NULL}};
const Segment SGI181731[] = {{&S57874[71], 6}, {NULL}};
const Segment SGI181732[] = {{&S70746[44], 8}, {NULL}};
const Segment SGI181733[] = {{&S70746[44], 20}, {NULL}};
const Segment SGI181734[] = {{&S70746[30], 6}, {NULL}};
const Segment SGI181735[] = {{&S70746[31], 3}, {NULL}};
const Segment SGI181736[] = {{&S70746[78], 6}, {NULL}};
const Segment SGI181737[] = {{&S70753[45], 10}, {NULL}};
const Segment SGI181738[] = {{&S70753[30], 6}, {NULL}};
const Segment SGI181739[] = {{&S70753[31], 3}, {NULL}};
const Segment SGI181740[] = {{&S70753[81], 5}, {NULL}};
const Segment SGI181741[] = {{&S70753[139], 13}, {NULL}};
const Segment SGI181742[] = {{&S63878[34], 15}, {NULL}};
const Segment SGI181743[] = {{&S63878[27], 6}, {NULL}};
const Segment SGI181744[] = {{&S63878[102], 9}, {NULL}};
const Segment SGI181745[] = {{&S63878[75], 6}, {NULL}};
const Segment SGI181746[] = {{&S63880[34], 6}, {NULL}};
const Segment SGI181747[] = {{&S63880[27], 6}, {NULL}};
const Segment SGI181748[] = {{&S63880[93], 9}, {NULL}};
const Segment SGI181749[] = {{&S63880[66], 6}, {NULL}};
const Segment SGI181750[] = {{&S59341[34], 7}, {NULL}};
const Segment SGI181751[] = {{&S59341[27], 6}, {NULL}};
const Segment SGI181752[] = {{&S59341[99], 9}, {NULL}};
const Segment SGI181753[] = {{&S59341[72], 6}, {NULL}};
const Segment SGI181754[] = {{&S59344[38], 7}, {NULL}};
const Segment SGI181755[] = {{&S59344[31], 6}, {NULL}};
const Segment SGI181756[] = {{&S59345[44], 7}, {NULL}};
const Segment SGI181757[] = {{&S59345[44], 25}, {NULL}};
const Segment SGI181758[] = {{&S59345[30], 6}, {NULL}};
const Segment SGI181759[] = {{&S59345[118], 3}, {NULL}};
const Segment SGI181760[] = {{&S59345[83], 6}, {NULL}};
const Segment SGI181761[] = {{&S59346[44], 7}, {NULL}};
const Segment SGI181762[] = {{&S70813[44], 7}, {NULL}};
const Segment SGI181763[] = {{&S59346[44], 25}, {NULL}};
const Segment SGI181764[] = {{&S70813[44], 19}, {NULL}};
const Segment SGI181765[] = {{&S59346[30], 6}, {NULL}};
const Segment SGI181766[] = {{&S70813[30], 6}, {NULL}};
const Segment SGI181767[] = {{&S59346[118], 3}, {NULL}};
const Segment SGI181768[] = {{&S70813[31], 3}, {NULL}};
const Segment SGI181769[] = {{&S59346[83], 6}, {NULL}};
const Segment SGI181770[] = {{&S70813[92], 6}, {NULL}};
const Segment SGI181771[] = {{&S70813[65], 8}, {NULL}};
const Segment SGI181772[] = {{&S59347[44], 7}, {NULL}};
const Segment SGI181773[] = {{&S70814[44], 7}, {NULL}};
const Segment SGI181774[] = {{&S59347[44], 25}, {NULL}};
const Segment SGI181775[] = {{&S59347[30], 6}, {NULL}};
const Segment SGI181776[] = {{&S70814[44], 19}, {NULL}};
const Segment SGI181777[] = {{&S70814[30], 6}, {NULL}};
const Segment SGI181778[] = {{&S59347[118], 3}, {NULL}};
const Segment SGI181779[] = {{&S70814[31], 3}, {NULL}};
const Segment SGI181780[] = {{&S59347[83], 6}, {NULL}};
const Segment SGI181781[] = {{&S70814[92], 6}, {NULL}};
const Segment SGI181782[] = {{&S70814[65], 8}, {NULL}};
const Segment SGI181783[] = {{&S59348[44], 7}, {NULL}};
const Segment SGI181784[] = {{&S59348[44], 25}, {NULL}};
const Segment SGI181785[] = {{&S59348[30], 6}, {NULL}};
const Segment SGI181786[] = {{&S63908[44], 11}, {NULL}};
const Segment SGI181787[] = {{&S63908[44], 23}, {NULL}};
const Segment SGI181788[] = {{&S63908[30], 6}, {NULL}};
const Segment SGI181789[] = {{&S59348[118], 3}, {NULL}};
const Segment SGI181790[] = {{&S59348[83], 6}, {NULL}};
const Segment SGI181791[] = {{&S63908[116], 3}, {NULL}};
const Segment SGI181792[] = {{&S63908[81], 6}, {NULL}};
const Segment SGI181793[] = {{&S59349[44], 7}, {NULL}};
const Segment SGI181794[] = {{&S63909[38], 11}, {NULL}};
const Segment SGI181795[] = {{&S70817[34], 12}, {NULL}};
const Segment SGI181796[] = {{&S59349[44], 25}, {NULL}};
const Segment SGI181797[] = {{&S63909[31], 6}, {NULL}};
const Segment SGI181798[] = {{&S59349[30], 6}, {NULL}};
const Segment SGI181799[] = {{&S70817[27], 6}, {NULL}};
const Segment SGI181800[] = {{&S59349[118], 3}, {NULL}};
const Segment SGI181801[] = {{&S70817[109], 9}, {NULL}};
const Segment SGI181802[] = {{&S59349[83], 6}, {NULL}};
const Segment SGI181803[] = {{&S70817[82], 6}, {NULL}};
const Segment SGI181804[] = {{&S59350[34], 7}, {NULL}};
const Segment SGI181805[] = {{&S59350[27], 6}, {NULL}};
const Segment SGI181806[] = {{&S59350[107], 9}, {NULL}};
const Segment SGI181807[] = {{&S59350[80], 6}, {NULL}};
const Segment SGI181808[] = {{&S59351[34], 7}, {NULL}};
const Segment SGI181809[] = {{&S59351[27], 6}, {NULL}};
const Segment SGI181810[] = {{&S59351[107], 9}, {NULL}};
const Segment SGI181811[] = {{&S59351[80], 6}, {NULL}};
const Segment SGI181812[] = {{&S70824[44], 10}, {NULL}};
const Segment SGI181813[] = {{&S70824[44], 22}, {NULL}};
const Segment SGI181814[] = {{&S70824[30], 6}, {NULL}};
const Segment SGI181815[] = {{&S70824[31], 3}, {NULL}};
const Segment SGI181816[] = {{&S70824[80], 6}, {NULL}};
const Segment SGI181817[] = {{&S70824[143], 3}, {NULL}};
const Segment SGI181818[] = {{&S70825[44], 11}, {NULL}};
const Segment SGI181819[] = {{&S70825[44], 23}, {NULL}};
const Segment SGI181820[] = {{&S70825[30], 6}, {NULL}};
const Segment SGI181821[] = {{&S70825[31], 3}, {NULL}};
const Segment SGI181822[] = {{&S70825[81], 6}, {NULL}};
const Segment SGI181823[] = {{&S70825[144], 3}, {NULL}};
const Segment SGI181824[] = {{&S59360[34], 7}, {NULL}};
const Segment SGI181825[] = {{&S59360[27], 6}, {NULL}};
const Segment SGI181826[] = {{&S59360[110], 9}, {NULL}};
const Segment SGI181827[] = {{&S59360[83], 6}, {NULL}};
const Segment SGI181828[] = {{&S70826[44], 10}, {NULL}};
const Segment SGI181829[] = {{&S70826[44], 22}, {NULL}};
const Segment SGI181830[] = {{&S70826[30], 6}, {NULL}};
const Segment SGI181831[] = {{&S59361[34], 7}, {NULL}};
const Segment SGI181832[] = {{&S70826[31], 3}, {NULL}};
const Segment SGI181833[] = {{&S59361[27], 6}, {NULL}};
const Segment SGI181834[] = {{&S70826[80], 6}, {NULL}};
const Segment SGI181835[] = {{&S70826[143], 3}, {NULL}};
const Segment SGI181836[] = {{&S59361[110], 9}, {NULL}};
const Segment SGI181837[] = {{&S59361[83], 6}, {NULL}};
const Segment SGI181838[] = {{&S59366[34], 9}, {NULL}};
const Segment SGI181839[] = {{&S59366[27], 6}, {NULL}};
const Segment SGI181840[] = {{&S59366[96], 9}, {NULL}};
const Segment SGI181841[] = {{&S59366[69], 6}, {NULL}};
const Segment SGI181842[] = {{&S70832[44], 11}, {NULL}};
const Segment SGI181843[] = {{&S70832[44], 23}, {NULL}};
const Segment SGI181844[] = {{&S70832[30], 6}, {NULL}};
const Segment SGI181845[] = {{&S70832[81], 6}, {NULL}};
const Segment SGI181846[] = {{&S59367[34], 10}, {NULL}};
const Segment SGI181847[] = {{&S70832[108], 13}, {NULL}};
const Segment SGI181848[] = {{&S59367[27], 6}, {NULL}};
const Segment SGI181849[] = {{&S59367[97], 9}, {NULL}};
const Segment SGI181850[] = {{&S59367[70], 6}, {NULL}};
const Segment SGI181851[] = {{&S70833[44], 10}, {NULL}};
const Segment SGI181852[] = {{&S70833[44], 22}, {NULL}};
const Segment SGI181853[] = {{&S70833[30], 6}, {NULL}};
const Segment SGI181854[] = {{&S70833[80], 6}, {NULL}};
const Segment SGI181855[] = {{&S70833[107], 13}, {NULL}};
const Segment SGI181856[] = {{&S63945[44], 13}, {NULL}};
const Segment SGI181857[] = {{&S63945[44], 39}, {NULL}};
const Segment SGI181858[] = {{&S63945[30], 6}, {NULL}};
const Segment SGI181859[] = {{&S63945[132], 3}, {NULL}};
const Segment SGI181860[] = {{&S63945[97], 6}, {NULL}};
const Segment SGI181861[] = {{&S63946[44], 12}, {NULL}};
const Segment SGI181862[] = {{&S63946[44], 37}, {NULL}};
const Segment SGI181863[] = {{&S63946[30], 6}, {NULL}};
const Segment SGI181864[] = {{&S63946[130], 3}, {NULL}};
const Segment SGI181865[] = {{&S63946[95], 6}, {NULL}};
const Segment SGI181866[] = {{&S59380[38], 9}, {NULL}};
const Segment SGI181867[] = {{&S59380[31], 6}, {NULL}};
const Segment SGI181868[] = {{&S59381[44], 9}, {NULL}};
const Segment SGI181869[] = {{&S59381[44], 21}, {NULL}};
const Segment SGI181870[] = {{&S59381[30], 6}, {NULL}};
const Segment SGI181871[] = {{&S59381[114], 3}, {NULL}};
const Segment SGI181872[] = {{&S59381[79], 6}, {NULL}};
const Segment SGI181873[] = {{&S59383[34], 8}, {NULL}};
const Segment SGI181874[] = {{&S59383[27], 6}, {NULL}};
const Segment SGI181875[] = {{&S48587[34], 18}, {NULL}};
const Segment SGI181876[] = {{&S59383[100], 9}, {NULL}};
const Segment SGI181877[] = {{&S48587[27], 6}, {NULL}};
const Segment SGI181878[] = {{&S59383[73], 6}, {NULL}};
const Segment SGI181879[] = {{&S48587[104], 9}, {NULL}};
const Segment SGI181880[] = {{&S48587[77], 6}, {NULL}};
const Segment SGI181881[] = {{&S48588[34], 10}, {NULL}};
const Segment SGI181882[] = {{&S48588[27], 6}, {NULL}};
const Segment SGI181883[] = {{&S48588[96], 9}, {NULL}};
const Segment SGI181884[] = {{&S48588[69], 6}, {NULL}};
const Segment SGI181885[] = {{&S48589[34], 10}, {NULL}};
const Segment SGI181886[] = {{&S48589[27], 6}, {NULL}};
const Segment SGI181887[] = {{&S48589[96], 9}, {NULL}};
const Segment SGI181888[] = {{&S48589[69], 6}, {NULL}};
const Segment SGI181889[] = {{&S70858[34], 13}, {NULL}};
const Segment SGI181890[] = {{&S70858[27], 6}, {NULL}};
const Segment SGI181891[] = {{&S70858[100], 9}, {NULL}};
const Segment SGI181892[] = {{&S70858[73], 6}, {NULL}};
const Segment SGI181893[] = {{&S48591[34], 11}, {NULL}};
const Segment SGI181894[] = {{&S48591[27], 6}, {NULL}};
const Segment SGI181895[] = {{&S48591[97], 9}, {NULL}};
const Segment SGI181896[] = {{&S70859[34], 11}, {NULL}};
const Segment SGI181897[] = {{&S48591[70], 6}, {NULL}};
const Segment SGI181898[] = {{&S70859[27], 6}, {NULL}};
const Segment SGI181899[] = {{&S70859[98], 9}, {NULL}};
const Segment SGI181900[] = {{&S70859[71], 6}, {NULL}};
const Segment SGI181901[] = {{&S48592[34], 10}, {NULL}};
const Segment SGI181902[] = {{&S48592[27], 6}, {NULL}};
const Segment SGI181903[] = {{&S48592[96], 9}, {NULL}};
const Segment SGI181904[] = {{&S48592[69], 6}, {NULL}};
const Segment SGI181905[] = {{&S70860[34], 10}, {NULL}};
const Segment SGI181906[] = {{&S70860[27], 6}, {NULL}};
const Segment SGI181907[] = {{&S70860[97], 9}, {NULL}};
const Segment SGI181908[] = {{&S70860[70], 6}, {NULL}};
const Segment SGI181909[] = {{&S70860[132], 8}, {NULL}};
const Segment SGI181910[] = {{&S48594[34], 18}, {NULL}};
const Segment SGI181911[] = {{&S48594[27], 6}, {NULL}};
const Segment SGI181912[] = {{&S48594[104], 9}, {NULL}};
const Segment SGI181913[] = {{&S48594[77], 6}, {NULL}};
const Segment SGI181914[] = {{&S70861[34], 10}, {NULL}};
const Segment SGI181915[] = {{&S70861[27], 6}, {NULL}};
const Segment SGI181916[] = {{&S70861[97], 9}, {NULL}};
const Segment SGI181917[] = {{&S70861[70], 6}, {NULL}};
const Segment SGI181918[] = {{&S48595[34], 10}, {NULL}};
const Segment SGI181919[] = {{&S70861[131], 8}, {NULL}};
const Segment SGI181920[] = {{&S48595[27], 6}, {NULL}};
const Segment SGI181921[] = {{&S48595[96], 9}, {NULL}};
const Segment SGI181922[] = {{&S48595[69], 6}, {NULL}};
const Segment SGI181923[] = {{&S70862[34], 20}, {NULL}};
const Segment SGI181924[] = {{&S70862[27], 6}, {NULL}};
const Segment SGI181925[] = {{&S70862[123], 9}, {NULL}};
const Segment SGI181926[] = {{&S70862[96], 6}, {NULL}};
const Segment SGI181927[] = {{&S48597[34], 11}, {NULL}};
const Segment SGI181928[] = {{&S48597[27], 6}, {NULL}};
const Segment SGI181929[] = {{&S70863[34], 10}, {NULL}};
const Segment SGI181930[] = {{&S70863[27], 6}, {NULL}};
const Segment SGI181931[] = {{&S48597[97], 9}, {NULL}};
const Segment SGI181932[] = {{&S48597[70], 6}, {NULL}};
const Segment SGI181933[] = {{&S70863[97], 9}, {NULL}};
const Segment SGI181934[] = {{&S70863[70], 6}, {NULL}};
const Segment SGI181935[] = {{&S70863[138], 8}, {NULL}};
const Segment SGI181936[] = {{&S48598[34], 10}, {NULL}};
const Segment SGI181937[] = {{&S48598[27], 6}, {NULL}};
const Segment SGI181938[] = {{&S48598[96], 9}, {NULL}};
const Segment SGI181939[] = {{&S48598[69], 6}, {NULL}};
const Segment SGI181940[] = {{&S70864[34], 10}, {NULL}};
const Segment SGI181941[] = {{&S70864[27], 6}, {NULL}};
const Segment SGI181942[] = {{&S70864[97], 9}, {NULL}};
const Segment SGI181943[] = {{&S70864[70], 6}, {NULL}};
const Segment SGI181944[] = {{&S70864[139], 8}, {NULL}};
const Segment SGI181945[] = {{&S48599[34], 11}, {NULL}};
const Segment SGI181946[] = {{&S48599[27], 6}, {NULL}};
const Segment SGI181947[] = {{&S48599[97], 9}, {NULL}};
const Segment SGI181948[] = {{&S48599[70], 6}, {NULL}};
const Segment SGI181949[] = {{&S48603[34], 18}, {NULL}};
const Segment SGI181950[] = {{&S70869[34], 14}, {NULL}};
const Segment SGI181951[] = {{&S48603[27], 6}, {NULL}};
const Segment SGI181952[] = {{&S70869[27], 6}, {NULL}};
const Segment SGI181953[] = {{&S48603[104], 9}, {NULL}};
const Segment SGI181954[] = {{&S48603[77], 6}, {NULL}};
const Segment SGI181955[] = {{&S70869[114], 9}, {NULL}};
const Segment SGI181956[] = {{&S70869[87], 6}, {NULL}};
const Segment SGI181957[] = {{&S48604[34], 18}, {NULL}};
const Segment SGI181958[] = {{&S48604[27], 6}, {NULL}};
const Segment SGI181959[] = {{&S48604[104], 9}, {NULL}};
const Segment SGI181960[] = {{&S48604[77], 6}, {NULL}};
const Segment SGI181961[] = {{&S70873[44], 10}, {NULL}};
const Segment SGI181962[] = {{&S70873[44], 22}, {NULL}};
const Segment SGI181963[] = {{&S70873[30], 6}, {NULL}};
const Segment SGI181964[] = {{&S70873[31], 3}, {NULL}};
const Segment SGI181965[] = {{&S70873[95], 6}, {NULL}};
const Segment SGI181966[] = {{&S70873[68], 8}, {NULL}};
const Segment SGI181967[] = {{&S70874[44], 10}, {NULL}};
const Segment SGI181968[] = {{&S70874[44], 22}, {NULL}};
const Segment SGI181969[] = {{&S70874[30], 6}, {NULL}};
const Segment SGI181970[] = {{&S70874[31], 3}, {NULL}};
const Segment SGI181971[] = {{&S70874[95], 6}, {NULL}};
const Segment SGI181972[] = {{&S70874[68], 8}, {NULL}};
const Segment SGI181973[] = {{&S70877[44], 33}, {NULL}};
const Segment SGI181974[] = {{&S70877[30], 6}, {NULL}};
const Segment SGI181975[] = {{&S70878[44], 7}, {NULL}};
const Segment SGI181976[] = {{&S70878[44], 19}, {NULL}};
const Segment SGI181977[] = {{&S70878[30], 6}, {NULL}};
const Segment SGI181978[] = {{&S70878[31], 3}, {NULL}};
const Segment SGI181979[] = {{&S70878[77], 6}, {NULL}};
const Segment SGI181980[] = {{&S70879[38], 7}, {NULL}};
const Segment SGI181981[] = {{&S70879[31], 6}, {NULL}};
const Segment SGI181982[] = {{&S64004[34], 29}, {NULL}};
const Segment SGI181983[] = {{&S64004[27], 6}, {NULL}};
const Segment SGI181984[] = {{&S64004[116], 9}, {NULL}};
const Segment SGI181985[] = {{&S70880[44], 7}, {NULL}};
const Segment SGI181986[] = {{&S64004[89], 6}, {NULL}};
const Segment SGI181987[] = {{&S70880[44], 19}, {NULL}};
const Segment SGI181988[] = {{&S70880[30], 6}, {NULL}};
const Segment SGI181989[] = {{&S70880[80], 6}, {NULL}};
const Segment SGI181990[] = {{&S64005[34], 30}, {NULL}};
const Segment SGI181991[] = {{&S64005[27], 6}, {NULL}};
const Segment SGI181992[] = {{&S70881[44], 6}, {NULL}};
const Segment SGI181993[] = {{&S70881[44], 18}, {NULL}};
const Segment SGI181994[] = {{&S70881[30], 6}, {NULL}};
const Segment SGI181995[] = {{&S64005[117], 9}, {NULL}};
const Segment SGI181996[] = {{&S64005[90], 6}, {NULL}};
const Segment SGI181997[] = {{&S70881[31], 3}, {NULL}};
const Segment SGI181998[] = {{&S70881[76], 6}, {NULL}};
const Segment SGI181999[] = {{&S70882[38], 6}, {NULL}};
const Segment SGI182000[] = {{&S70882[31], 6}, {NULL}};
const Segment SGI182001[] = {{&S64007[34], 9}, {NULL}};
const Segment SGI182002[] = {{&S64007[27], 6}, {NULL}};
const Segment SGI182003[] = {{&S64007[96], 9}, {NULL}};
const Segment SGI182004[] = {{&S64007[69], 6}, {NULL}};
const Segment SGI182005[] = {{&S64008[34], 9}, {NULL}};
const Segment SGI182006[] = {{&S64008[27], 6}, {NULL}};
const Segment SGI182007[] = {{&S64008[96], 9}, {NULL}};
const Segment SGI182008[] = {{&S64008[69], 6}, {NULL}};
const Segment SGI182009[] = {{&S70885[44], 13}, {NULL}};
const Segment SGI182010[] = {{&S70885[44], 25}, {NULL}};
const Segment SGI182011[] = {{&S70885[30], 6}, {NULL}};
const Segment SGI182012[] = {{&S70885[31], 3}, {NULL}};
const Segment SGI182013[] = {{&S70885[83], 6}, {NULL}};
const Segment SGI182014[] = {{&S59447[34], 7}, {NULL}};
const Segment SGI182015[] = {{&S59447[27], 6}, {NULL}};
const Segment SGI182016[] = {{&S59447[99], 9}, {NULL}};
const Segment SGI182017[] = {{&S59447[72], 6}, {NULL}};
const Segment SGI182018[] = {{&S59448[34], 7}, {NULL}};
const Segment SGI182019[] = {{&S59448[27], 6}, {NULL}};
const Segment SGI182020[] = {{&S59448[99], 9}, {NULL}};
const Segment SGI182021[] = {{&S59448[72], 6}, {NULL}};
const Segment SGI182022[] = {{&S70902[44], 11}, {NULL}};
const Segment SGI182023[] = {{&S70902[44], 23}, {NULL}};
const Segment SGI182024[] = {{&S70902[30], 6}, {NULL}};
const Segment SGI182025[] = {{&S70902[31], 3}, {NULL}};
const Segment SGI182026[] = {{&S70902[81], 6}, {NULL}};
const Segment SGI182027[] = {{&S70903[34], 6}, {NULL}};
const Segment SGI182028[] = {{&S70903[27], 6}, {NULL}};
const Segment SGI182029[] = {{&S70903[105], 9}, {NULL}};
const Segment SGI182030[] = {{&S70903[78], 6}, {NULL}};
const Segment SGI182031[] = {{&S70910[34], 10}, {NULL}};
const Segment SGI182032[] = {{&S70910[27], 6}, {NULL}};
const Segment SGI182033[] = {{&S70910[97], 9}, {NULL}};
const Segment SGI182034[] = {{&S70910[70], 6}, {NULL}};
const Segment SGI182035[] = {{&S70949[34], 10}, {NULL}};
const Segment SGI182036[] = {{&S70949[27], 6}, {NULL}};
const Segment SGI182037[] = {{&S70949[97], 9}, {NULL}};
const Segment SGI182038[] = {{&S70949[70], 6}, {NULL}};
const Segment SGI182039[] = {{&S70950[34], 11}, {NULL}};
const Segment SGI182040[] = {{&S64078[44], 5}, {NULL}};
const Segment SGI182041[] = {{&S64078[44], 17}, {NULL}};
const Segment SGI182042[] = {{&S70950[27], 6}, {NULL}};
const Segment SGI182043[] = {{&S64078[30], 6}, {NULL}};
const Segment SGI182044[] = {{&S64078[110], 3}, {NULL}};
const Segment SGI182045[] = {{&S70950[98], 9}, {NULL}};
const Segment SGI182046[] = {{&S70950[71], 6}, {NULL}};
const Segment SGI182047[] = {{&S64078[75], 6}, {NULL}};
const Segment SGI182048[] = {{&S64079[38], 5}, {NULL}};
const Segment SGI182049[] = {{&S64079[31], 6}, {NULL}};
const Segment SGI182050[] = {{&S70954[44], 15}, {NULL}};
const Segment SGI182051[] = {{&S70954[44], 27}, {NULL}};
const Segment SGI182052[] = {{&S70954[30], 6}, {NULL}};
const Segment SGI182053[] = {{&S70954[31], 3}, {NULL}};
const Segment SGI182054[] = {{&S70954[85], 6}, {NULL}};
const Segment SGI182055[] = {{&S61403[32], 9}, {NULL}};
const Segment SGI182056[] = {{&S61403[27], 4}, {NULL}};
const Segment SGI182057[] = {{&S61403[94], 9}, {NULL}};
const Segment SGI182058[] = {{&S61403[67], 6}, {NULL}};
const Segment SGI182059[] = {{&S59529[34], 8}, {NULL}};
const Segment SGI182060[] = {{&S59529[27], 6}, {NULL}};
const Segment SGI182061[] = {{&S59529[101], 9}, {NULL}};
const Segment SGI182062[] = {{&S59529[74], 6}, {NULL}};
const Segment SGI182063[] = {{&S70960[34], 14}, {NULL}};
const Segment SGI182064[] = {{&S70960[27], 6}, {NULL}};
const Segment SGI182065[] = {{&S70960[106], 9}, {NULL}};
const Segment SGI182066[] = {{&S70960[79], 6}, {NULL}};
const Segment SGI182067[] = {{&S59531[44], 18}, {NULL}};
const Segment SGI182068[] = {{&S59531[44], 30}, {NULL}};
const Segment SGI182069[] = {{&S59531[30], 6}, {NULL}};
const Segment SGI182070[] = {{&S57877[34], 16}, {NULL}};
const Segment SGI182071[] = {{&S59531[139], 3}, {NULL}};
const Segment SGI182072[] = {{&S57877[27], 6}, {NULL}};
const Segment SGI182073[] = {{&S59531[104], 6}, {NULL}};
const Segment SGI182074[] = {{&S59531[76], 8}, {NULL}};
const Segment SGI182075[] = {{&S57877[103], 9}, {NULL}};
const Segment SGI182076[] = {{&S57877[76], 6}, {NULL}};
const Segment SGI182077[] = {{&S64093[44], 10}, {NULL}};
const Segment SGI182078[] = {{&S64093[44], 22}, {NULL}};
const Segment SGI182079[] = {{&S64093[30], 6}, {NULL}};
const Segment SGI182080[] = {{&S64093[146], 4}, {NULL}};
const Segment SGI182081[] = {{&S64093[172], 3}, {NULL}};
const Segment SGI182082[] = {{&S64093[115], 3}, {&S64093[221], 5}, {NULL}};
const Segment SGI182083[] = {{&S64093[80], 6}, {NULL}};
const Segment SGI182084[] = {{&S70964[44], 9}, {NULL}};
const Segment SGI182085[] = {{&S70964[44], 21}, {NULL}};
const Segment SGI182086[] = {{&S70964[30], 6}, {NULL}};
const Segment SGI182087[] = {{&S70964[31], 3}, {NULL}};
const Segment SGI182088[] = {{&S70964[108], 6}, {NULL}};
const Segment SGI182089[] = {{&S70964[67], 8}, {NULL}};
const Segment SGI182090[] = {{&S70964[79], 15}, {NULL}};
const Segment SGI182091[] = {{&S70965[44], 9}, {NULL}};
const Segment SGI182092[] = {{&S70965[44], 21}, {NULL}};
const Segment SGI182093[] = {{&S70965[30], 6}, {NULL}};
const Segment SGI182094[] = {{&S70965[31], 3}, {NULL}};
const Segment SGI182095[] = {{&S70965[108], 6}, {NULL}};
const Segment SGI182096[] = {{&S70965[67], 8}, {NULL}};
const Segment SGI182097[] = {{&S70965[79], 15}, {NULL}};
const Segment SGI182098[] = {{&S70966[44], 13}, {NULL}};
const Segment SGI182099[] = {{&S70966[44], 25}, {NULL}};
const Segment SGI182100[] = {{&S70966[30], 6}, {NULL}};
const Segment SGI182101[] = {{&S70966[31], 3}, {NULL}};
const Segment SGI182102[] = {{&S70966[112], 6}, {NULL}};
const Segment SGI182103[] = {{&S70966[71], 8}, {NULL}};
const Segment SGI182104[] = {{&S70966[83], 15}, {NULL}};
const Segment SGI182105[] = {{&S48732[34], 10}, {NULL}};
const Segment SGI182106[] = {{&S48732[27], 6}, {NULL}};
const Segment SGI182107[] = {{&S48732[96], 9}, {NULL}};
const Segment SGI182108[] = {{&S48732[69], 6}, {NULL}};
const Segment SGI182109[] = {{&S64113[44], 11}, {NULL}};
const Segment SGI182110[] = {{&S64113[44], 28}, {NULL}};
const Segment SGI182111[] = {{&S64113[30], 6}, {NULL}};
const Segment SGI182112[] = {{&S64113[120], 3}, {NULL}};
const Segment SGI182113[] = {{&S64113[86], 5}, {NULL}};
const Segment SGI182114[] = {{&S64114[38], 11}, {NULL}};
const Segment SGI182115[] = {{&S64114[31], 6}, {NULL}};
const Segment SGI182116[] = {{&S70988[44], 11}, {NULL}};
const Segment SGI182117[] = {{&S70988[44], 23}, {NULL}};
const Segment SGI182118[] = {{&S70988[30], 6}, {NULL}};
const Segment SGI182119[] = {{&S70988[31], 3}, {NULL}};
const Segment SGI182120[] = {{&S70988[81], 6}, {NULL}};
const Segment SGI182121[] = {{&S70989[38], 11}, {NULL}};
const Segment SGI182122[] = {{&S70989[31], 6}, {NULL}};
const Segment SGI182123[] = {{&S70990[34], 11}, {NULL}};
const Segment SGI182124[] = {{&S70990[27], 6}, {NULL}};
const Segment SGI182125[] = {{&S70990[98], 9}, {NULL}};
const Segment SGI182126[] = {{&S70990[71], 6}, {NULL}};
const Segment SGI182127[] = {{&S71017[44], 10}, {NULL}};
const Segment SGI182128[] = {{&S71017[44], 22}, {NULL}};
const Segment SGI182129[] = {{&S71017[30], 6}, {NULL}};
const Segment SGI182130[] = {{&S71017[31], 3}, {NULL}};
const Segment SGI182131[] = {{&S71017[80], 6}, {NULL}};
const Segment SGI182132[] = {{&S59598[34], 9}, {NULL}};
const Segment SGI182133[] = {{&S59598[27], 6}, {NULL}};
const Segment SGI182134[] = {{&S59598[96], 9}, {NULL}};
const Segment SGI182135[] = {{&S59598[69], 6}, {NULL}};
const Segment SGI182136[] = {{&S71023[34], 10}, {NULL}};
const Segment SGI182137[] = {{&S71023[27], 6}, {NULL}};
const Segment SGI182138[] = {{&S71023[97], 9}, {NULL}};
const Segment SGI182139[] = {{&S71023[70], 6}, {NULL}};
const Segment SGI182140[] = {{&S64178[34], 11}, {NULL}};
const Segment SGI182141[] = {{&S64178[27], 6}, {NULL}};
const Segment SGI182142[] = {{&S64178[98], 9}, {NULL}};
const Segment SGI182143[] = {{&S64178[71], 6}, {NULL}};
const Segment SGI182144[] = {{&S64178[139], 8}, {NULL}};
const Segment SGI182145[] = {{&S71036[44], 6}, {NULL}};
const Segment SGI182146[] = {{&S71036[44], 18}, {NULL}};
const Segment SGI182147[] = {{&S71036[30], 6}, {NULL}};
const Segment SGI182148[] = {{&S64179[34], 11}, {NULL}};
const Segment SGI182149[] = {{&S71036[31], 3}, {NULL}};
const Segment SGI182150[] = {{&S64179[27], 6}, {NULL}};
const Segment SGI182151[] = {{&S71036[91], 6}, {NULL}};
const Segment SGI182152[] = {{&S71036[64], 8}, {NULL}};
const Segment SGI182153[] = {{&S64179[98], 9}, {NULL}};
const Segment SGI182154[] = {{&S64179[71], 6}, {NULL}};
const Segment SGI182155[] = {{&S64179[139], 8}, {NULL}};
const Segment SGI182156[] = {{&S71037[44], 6}, {NULL}};
const Segment SGI182157[] = {{&S71037[44], 18}, {NULL}};
const Segment SGI182158[] = {{&S71037[30], 6}, {NULL}};
const Segment SGI182159[] = {{&S71037[31], 3}, {NULL}};
const Segment SGI182160[] = {{&S71037[91], 6}, {NULL}};
const Segment SGI182161[] = {{&S71037[64], 8}, {NULL}};
const Segment SGI182162[] = {{&S71038[44], 6}, {NULL}};
const Segment SGI182163[] = {{&S71038[44], 18}, {NULL}};
const Segment SGI182164[] = {{&S71038[30], 6}, {NULL}};
const Segment SGI182165[] = {{&S71038[31], 3}, {NULL}};
const Segment SGI182166[] = {{&S71038[91], 6}, {NULL}};
const Segment SGI182167[] = {{&S71038[64], 8}, {NULL}};
const Segment SGI182168[] = {{&S64183[44], 7}, {NULL}};
const Segment SGI182169[] = {{&S64183[44], 19}, {NULL}};
const Segment SGI182170[] = {{&S64183[30], 6}, {NULL}};
const Segment SGI182171[] = {{&S64183[112], 3}, {NULL}};
const Segment SGI182172[] = {{&S64183[77], 6}, {NULL}};
const Segment SGI182173[] = {{&S71049[34], 10}, {NULL}};
const Segment SGI182174[] = {{&S71049[27], 6}, {NULL}};
const Segment SGI182175[] = {{&S71049[97], 9}, {NULL}};
const Segment SGI182176[] = {{&S71049[70], 6}, {NULL}};
const Segment SGI182177[] = {{&S64214[34], 12}, {NULL}};
const Segment SGI182178[] = {{&S64214[27], 6}, {NULL}};
const Segment SGI182179[] = {{&S64214[99], 9}, {NULL}};
const Segment SGI182180[] = {{&S64214[72], 6}, {NULL}};
const Segment SGI182181[] = {{&S59653[34], 7}, {NULL}};
const Segment SGI182182[] = {{&S59653[27], 6}, {NULL}};
const Segment SGI182183[] = {{&S59653[100], 9}, {NULL}};
const Segment SGI182184[] = {{&S59653[73], 6}, {NULL}};
const Segment SGI182185[] = {{&S68076[32], 6}, {NULL}};
const Segment SGI182186[] = {{&S68076[27], 4}, {NULL}};
const Segment SGI182187[] = {{&S68076[91], 9}, {NULL}};
const Segment SGI182188[] = {{&S68076[64], 6}, {NULL}};
const Segment SGI182189[] = {{&S64237[34], 12}, {NULL}};
const Segment SGI182190[] = {{&S64237[27], 6}, {NULL}};
const Segment SGI182191[] = {{&S64237[99], 9}, {NULL}};
const Segment SGI182192[] = {{&S64237[72], 6}, {NULL}};
const Segment SGI182193[] = {{&S64239[34], 12}, {NULL}};
const Segment SGI182194[] = {{&S64239[27], 6}, {NULL}};
const Segment SGI182195[] = {{&S64239[99], 9}, {NULL}};
const Segment SGI182196[] = {{&S64239[72], 6}, {NULL}};
const Segment SGI182197[] = {{&S71100[44], 10}, {NULL}};
const Segment SGI182198[] = {{&S71100[44], 22}, {NULL}};
const Segment SGI182199[] = {{&S71100[30], 6}, {NULL}};
const Segment SGI182200[] = {{&S71100[31], 3}, {NULL}};
const Segment SGI182201[] = {{&S71100[99], 6}, {NULL}};
const Segment SGI182202[] = {{&S71100[68], 8}, {NULL}};
const Segment SGI182203[] = {{&S71101[44], 10}, {NULL}};
const Segment SGI182204[] = {{&S71101[44], 22}, {NULL}};
const Segment SGI182205[] = {{&S71101[30], 6}, {NULL}};
const Segment SGI182206[] = {{&S71101[31], 3}, {NULL}};
const Segment SGI182207[] = {{&S71101[99], 6}, {NULL}};
const Segment SGI182208[] = {{&S71101[68], 8}, {NULL}};
const Segment SGI182209[] = {{&S71102[44], 10}, {NULL}};
const Segment SGI182210[] = {{&S71102[44], 22}, {NULL}};
const Segment SGI182211[] = {{&S71102[30], 6}, {NULL}};
const Segment SGI182212[] = {{&S71102[31], 3}, {NULL}};
const Segment SGI182213[] = {{&S71102[95], 6}, {NULL}};
const Segment SGI182214[] = {{&S71102[68], 8}, {NULL}};
const Segment SGI182215[] = {{&S57895[34], 6}, {NULL}};
const Segment SGI182216[] = {{&S57895[27], 6}, {NULL}};
const Segment SGI182217[] = {{&S57895[93], 9}, {NULL}};
const Segment SGI182218[] = {{&S57895[66], 6}, {NULL}};
const Segment SGI182219[] = {{&S48886[44], 11}, {NULL}};
const Segment SGI182220[] = {{&S48886[44], 22}, {NULL}};
const Segment SGI182221[] = {{&S48886[30], 6}, {NULL}};
const Segment SGI182222[] = {{&S48886[133], 3}, {NULL}};
const Segment SGI182223[] = {{&S48886[98], 6}, {NULL}};
const Segment SGI182224[] = {{&S48886[68], 8}, {NULL}};
const Segment SGI182225[] = {{&S48887[44], 15}, {NULL}};
const Segment SGI182226[] = {{&S48887[44], 26}, {NULL}};
const Segment SGI182227[] = {{&S48887[30], 6}, {NULL}};
const Segment SGI182228[] = {{&S48887[137], 3}, {NULL}};
const Segment SGI182229[] = {{&S48887[102], 6}, {NULL}};
const Segment SGI182230[] = {{&S48887[72], 8}, {NULL}};
const Segment SGI182231[] = {{&S71115[44], 12}, {NULL}};
const Segment SGI182232[] = {{&S71115[44], 24}, {NULL}};
const Segment SGI182233[] = {{&S71115[30], 6}, {NULL}};
const Segment SGI182234[] = {{&S48888[44], 15}, {NULL}};
const Segment SGI182235[] = {{&S48888[44], 27}, {NULL}};
const Segment SGI182236[] = {{&S48888[30], 6}, {NULL}};
const Segment SGI182237[] = {{&S71115[31], 3}, {NULL}};
const Segment SGI182238[] = {{&S71115[82], 6}, {NULL}};
const Segment SGI182239[] = {{&S48888[138], 3}, {NULL}};
const Segment SGI182240[] = {{&S48888[103], 6}, {NULL}};
const Segment SGI182241[] = {{&S48888[73], 8}, {NULL}};
const Segment SGI182242[] = {{&S71129[44], 16}, {NULL}};
const Segment SGI182243[] = {{&S71129[44], 38}, {NULL}};
const Segment SGI182244[] = {{&S71129[30], 6}, {NULL}};
const Segment SGI182245[] = {{&S71129[31], 3}, {NULL}};
const Segment SGI182246[] = {{&S71129[96], 6}, {NULL}};
const Segment SGI182247[] = {{&S71133[34], 14}, {NULL}};
const Segment SGI182248[] = {{&S71133[27], 6}, {NULL}};
const Segment SGI182249[] = {{&S71133[101], 9}, {NULL}};
const Segment SGI182250[] = {{&S71133[74], 6}, {NULL}};
const Segment SGI182251[] = {{&S71134[34], 14}, {NULL}};
const Segment SGI182252[] = {{&S71134[27], 6}, {NULL}};
const Segment SGI182253[] = {{&S71134[101], 9}, {NULL}};
const Segment SGI182254[] = {{&S71134[74], 6}, {NULL}};
const Segment SGI182255[] = {{&S71138[34], 17}, {NULL}};
const Segment SGI182256[] = {{&S71138[27], 6}, {NULL}};
const Segment SGI182257[] = {{&S71138[104], 9}, {NULL}};
const Segment SGI182258[] = {{&S71138[77], 6}, {NULL}};
const Segment SGI182259[] = {{&S71143[44], 10}, {NULL}};
const Segment SGI182260[] = {{&S71143[44], 39}, {NULL}};
const Segment SGI182261[] = {{&S71143[30], 6}, {NULL}};
const Segment SGI182262[] = {{&S59735[34], 11}, {NULL}};
const Segment SGI182263[] = {{&S71143[97], 6}, {NULL}};
const Segment SGI182264[] = {{&S59735[27], 6}, {NULL}};
const Segment SGI182265[] = {{&S59735[98], 9}, {NULL}};
const Segment SGI182266[] = {{&S59735[71], 6}, {NULL}};
const Segment SGI182267[] = {{&S71144[44], 10}, {NULL}};
const Segment SGI182268[] = {{&S71144[44], 39}, {NULL}};
const Segment SGI182269[] = {{&S71144[30], 6}, {NULL}};
const Segment SGI182270[] = {{&S71144[31], 3}, {NULL}};
const Segment SGI182271[] = {{&S71144[97], 6}, {NULL}};
const Segment SGI182272[] = {{&S59736[34], 12}, {NULL}};
const Segment SGI182273[] = {{&S59736[27], 6}, {NULL}};
const Segment SGI182274[] = {{&S59736[99], 9}, {NULL}};
const Segment SGI182275[] = {{&S59736[72], 6}, {NULL}};
const Segment SGI182276[] = {{&S71145[38], 10}, {NULL}};
const Segment SGI182277[] = {{&S71145[31], 6}, {NULL}};
const Segment SGI182278[] = {{&S59737[34], 12}, {NULL}};
const Segment SGI182279[] = {{&S59737[27], 6}, {NULL}};
const Segment SGI182280[] = {{&S59737[99], 9}, {NULL}};
const Segment SGI182281[] = {{&S59737[72], 6}, {NULL}};
const Segment SGI182282[] = {{&S59740[44], 14}, {NULL}};
const Segment SGI182283[] = {{&S59740[44], 26}, {NULL}};
const Segment SGI182284[] = {{&S59740[30], 6}, {NULL}};
const Segment SGI182285[] = {{&S59740[135], 3}, {NULL}};
const Segment SGI182286[] = {{&S59740[100], 6}, {NULL}};
const Segment SGI182287[] = {{&S59740[72], 8}, {NULL}};
const Segment SGI182288[] = {{&S59741[44], 11}, {NULL}};
const Segment SGI182289[] = {{&S59741[44], 23}, {NULL}};
const Segment SGI182290[] = {{&S59741[30], 6}, {NULL}};
const Segment SGI182291[] = {{&S59741[116], 3}, {NULL}};
const Segment SGI182292[] = {{&S59741[81], 6}, {NULL}};
const Segment SGI182293[] = {{&S59742[38], 11}, {NULL}};
const Segment SGI182294[] = {{&S59742[31], 6}, {NULL}};
const Segment SGI182295[] = {{&S71155[34], 11}, {NULL}};
const Segment SGI182296[] = {{&S57948[44], 12}, {NULL}};
const Segment SGI182297[] = {{&S71155[27], 6}, {NULL}};
const Segment SGI182298[] = {{&S57948[44], 27}, {NULL}};
const Segment SGI182299[] = {{&S57948[30], 6}, {NULL}};
const Segment SGI182300[] = {{&S71155[98], 9}, {NULL}};
const Segment SGI182301[] = {{&S57948[138], 3}, {NULL}};
const Segment SGI182302[] = {{&S71155[71], 6}, {NULL}};
const Segment SGI182303[] = {{&S57948[103], 6}, {NULL}};
const Segment SGI182304[] = {{&S57948[73], 8}, {NULL}};
const Segment SGI182305[] = {{&S59744[34], 9}, {NULL}};
const Segment SGI182306[] = {{&S59744[27], 6}, {NULL}};
const Segment SGI182307[] = {{&S59744[96], 9}, {NULL}};
const Segment SGI182308[] = {{&S59744[69], 6}, {NULL}};
const Segment SGI182309[] = {{&S64324[44], 10}, {NULL}};
const Segment SGI182310[] = {{&S64324[44], 29}, {NULL}};
const Segment SGI182311[] = {{&S64324[30], 6}, {NULL}};
const Segment SGI182312[] = {{&S64324[122], 3}, {NULL}};
const Segment SGI182313[] = {{&S64324[87], 6}, {NULL}};
const Segment SGI182314[] = {{&S59747[34], 11}, {NULL}};
const Segment SGI182315[] = {{&S59747[27], 6}, {NULL}};
const Segment SGI182316[] = {{&S59747[98], 9}, {NULL}};
const Segment SGI182317[] = {{&S59747[71], 6}, {NULL}};
const Segment SGI182318[] = {{&S64325[44], 10}, {NULL}};
const Segment SGI182319[] = {{&S64325[44], 29}, {NULL}};
const Segment SGI182320[] = {{&S64325[30], 6}, {NULL}};
const Segment SGI182321[] = {{&S64325[122], 3}, {NULL}};
const Segment SGI182322[] = {{&S64325[87], 6}, {NULL}};
const Segment SGI182323[] = {{&S61660[73], 10}, {NULL}};
const Segment SGI182324[] = {{&S64326[44], 10}, {NULL}};
const Segment SGI182325[] = {{&S64326[44], 29}, {NULL}};
const Segment SGI182326[] = {{&S61660[30], 34}, {NULL}};
const Segment SGI182327[] = {{&S64326[30], 6}, {NULL}};
const Segment SGI182328[] = {{&S64326[122], 3}, {NULL}};
const Segment SGI182329[] = {{&S59750[34], 10}, {NULL}};
const Segment SGI182330[] = {{&S61660[147], 3}, {NULL}};
const Segment SGI182331[] = {{&S59750[27], 6}, {NULL}};
const Segment SGI182332[] = {{&S64326[87], 6}, {NULL}};
const Segment SGI182333[] = {{&S61660[114], 5}, {NULL}};
const Segment SGI182334[] = {{&S59750[97], 9}, {NULL}};
const Segment SGI182335[] = {{&S59750[70], 6}, {NULL}};
const Segment SGI182336[] = {{&S64327[44], 10}, {NULL}};
const Segment SGI182337[] = {{&S64327[44], 29}, {NULL}};
const Segment SGI182338[] = {{&S64327[30], 6}, {NULL}};
const Segment SGI182339[] = {{&S64327[122], 3}, {NULL}};
const Segment SGI182340[] = {{&S64327[87], 6}, {NULL}};
const Segment SGI182341[] = {{&S64328[44], 10}, {NULL}};
const Segment SGI182342[] = {{&S64328[44], 29}, {NULL}};
const Segment SGI182343[] = {{&S64328[30], 6}, {NULL}};
const Segment SGI182344[] = {{&S64328[122], 3}, {NULL}};
const Segment SGI182345[] = {{&S64328[87], 6}, {NULL}};
const Segment SGI182346[] = {{&S64329[38], 10}, {NULL}};
const Segment SGI182347[] = {{&S64329[31], 6}, {NULL}};
const Segment SGI182348[] = {{&S64330[44], 10}, {NULL}};
const Segment SGI182349[] = {{&S64330[44], 29}, {NULL}};
const Segment SGI182350[] = {{&S64330[30], 6}, {NULL}};
const Segment SGI182351[] = {{&S64330[122], 3}, {NULL}};
const Segment SGI182352[] = {{&S64330[87], 6}, {NULL}};
const Segment SGI182353[] = {{&S64331[44], 10}, {NULL}};
const Segment SGI182354[] = {{&S68177[42], 13}, {NULL}};
const Segment SGI182355[] = {{&S64331[44], 29}, {NULL}};
const Segment SGI182356[] = {{&S68177[42], 29}, {NULL}};
const Segment SGI182357[] = {{&S64331[30], 6}, {NULL}};
const Segment SGI182358[] = {{&S68177[30], 4}, {NULL}};
const Segment SGI182359[] = {{&S64331[122], 3}, {NULL}};
const Segment SGI182360[] = {{&S71177[44], 6}, {NULL}};
const Segment SGI182361[] = {{&S68177[120], 3}, {NULL}};
const Segment SGI182362[] = {{&S64331[87], 6}, {NULL}};
const Segment SGI182363[] = {{&S71177[44], 18}, {NULL}};
const Segment SGI182364[] = {{&S71177[30], 6}, {NULL}};
const Segment SGI182365[] = {{&S68177[85], 6}, {NULL}};
const Segment SGI182366[] = {{&S71177[31], 3}, {NULL}};
const Segment SGI182367[] = {{&S71177[76], 6}, {NULL}};
const Segment SGI182368[] = {{&S64332[44], 12}, {NULL}};
const Segment SGI182369[] = {{&S64332[44], 31}, {NULL}};
const Segment SGI182370[] = {{&S64332[30], 6}, {NULL}};
const Segment SGI182371[] = {{&S64332[124], 3}, {NULL}};
const Segment SGI182372[] = {{&S64332[89], 6}, {NULL}};
const Segment SGI182373[] = {{&S71180[34], 16}, {NULL}};
const Segment SGI182374[] = {{&S71180[27], 6}, {NULL}};
const Segment SGI182375[] = {{&S64333[44], 15}, {NULL}};
const Segment SGI182376[] = {{&S64333[44], 34}, {NULL}};
const Segment SGI182377[] = {{&S64333[30], 6}, {NULL}};
const Segment SGI182378[] = {{&S71180[110], 9}, {NULL}};
const Segment SGI182379[] = {{&S71180[83], 6}, {NULL}};
const Segment SGI182380[] = {{&S64333[127], 3}, {NULL}};
const Segment SGI182381[] = {{&S64333[92], 6}, {NULL}};
const Segment SGI182382[] = {{&S71181[34], 18}, {NULL}};
const Segment SGI182383[] = {{&S71181[27], 6}, {NULL}};
const Segment SGI182384[] = {{&S64334[44], 15}, {NULL}};
const Segment SGI182385[] = {{&S64334[44], 34}, {NULL}};
const Segment SGI182386[] = {{&S64334[30], 6}, {NULL}};
const Segment SGI182387[] = {{&S71181[114], 9}, {NULL}};
const Segment SGI182388[] = {{&S71181[87], 6}, {NULL}};
const Segment SGI182389[] = {{&S64334[127], 3}, {NULL}};
const Segment SGI182390[] = {{&S64334[92], 6}, {NULL}};
const Segment SGI182391[] = {{&S71182[34], 15}, {NULL}};
const Segment SGI182392[] = {{&S71182[27], 6}, {NULL}};
const Segment SGI182393[] = {{&S71182[108], 9}, {NULL}};
const Segment SGI182394[] = {{&S71182[81], 6}, {NULL}};
const Segment SGI182395[] = {{&S71183[34], 17}, {NULL}};
const Segment SGI182396[] = {{&S71183[27], 6}, {NULL}};
const Segment SGI182397[] = {{&S71183[112], 9}, {NULL}};
const Segment SGI182398[] = {{&S71183[85], 6}, {NULL}};
const Segment SGI182399[] = {{&S71184[34], 17}, {NULL}};
const Segment SGI182400[] = {{&S71184[27], 6}, {NULL}};
const Segment SGI182401[] = {{&S71184[112], 9}, {NULL}};
const Segment SGI182402[] = {{&S71184[85], 6}, {NULL}};
const Segment SGI182403[] = {{&S71186[44], 10}, {NULL}};
const Segment SGI182404[] = {{&S71186[44], 22}, {NULL}};
const Segment SGI182405[] = {{&S71186[30], 6}, {NULL}};
const Segment SGI182406[] = {{&S71186[31], 3}, {NULL}};
const Segment SGI182407[] = {{&S71186[80], 6}, {NULL}};
const Segment SGI182408[] = {{&S71186[143], 3}, {NULL}};
const Segment SGI182409[] = {{&S71187[44], 13}, {NULL}};
const Segment SGI182410[] = {{&S71187[44], 38}, {NULL}};
const Segment SGI182411[] = {{&S71187[30], 6}, {NULL}};
const Segment SGI182412[] = {{&S71187[31], 3}, {NULL}};
const Segment SGI182413[] = {{&S71187[96], 6}, {NULL}};
const Segment SGI182414[] = {{&S71190[34], 11}, {NULL}};
const Segment SGI182415[] = {{&S71190[27], 6}, {NULL}};
const Segment SGI182416[] = {{&S71190[98], 9}, {NULL}};
const Segment SGI182417[] = {{&S71190[71], 6}, {NULL}};
const Segment SGI182418[] = {{&S64353[44], 6}, {NULL}};
const Segment SGI182419[] = {{&S64353[44], 18}, {NULL}};
const Segment SGI182420[] = {{&S64353[30], 6}, {NULL}};
const Segment SGI182421[] = {{&S48988[44], 12}, {NULL}};
const Segment SGI182422[] = {{&S64353[76], 6}, {NULL}};
const Segment SGI182423[] = {{&S48988[44], 23}, {NULL}};
const Segment SGI182424[] = {{&S48988[30], 6}, {NULL}};
const Segment SGI182425[] = {{&S64353[103], 13}, {NULL}};
const Segment SGI182426[] = {{&S48988[116], 3}, {NULL}};
const Segment SGI182427[] = {{&S48988[81], 6}, {NULL}};
const Segment SGI182428[] = {{&S64354[44], 7}, {NULL}};
const Segment SGI182429[] = {{&S64354[44], 19}, {NULL}};
const Segment SGI182430[] = {{&S64354[30], 6}, {NULL}};
const Segment SGI182431[] = {{&S48989[38], 12}, {NULL}};
const Segment SGI182432[] = {{&S48989[31], 6}, {NULL}};
const Segment SGI182433[] = {{&S64354[77], 6}, {NULL}};
const Segment SGI182434[] = {{&S64354[104], 13}, {NULL}};
const Segment SGI182435[] = {{&S48990[44], 14}, {NULL}};
const Segment SGI182436[] = {{&S48990[44], 25}, {NULL}};
const Segment SGI182437[] = {{&S64355[44], 8}, {NULL}};
const Segment SGI182438[] = {{&S48990[30], 6}, {NULL}};
const Segment SGI182439[] = {{&S64355[44], 20}, {NULL}};
const Segment SGI182440[] = {{&S64355[30], 6}, {NULL}};
const Segment SGI182441[] = {{&S48990[118], 3}, {NULL}};
const Segment SGI182442[] = {{&S48990[83], 6}, {NULL}};
const Segment SGI182443[] = {{&S48991[44], 25}, {NULL}};
const Segment SGI182444[] = {{&S48991[44], 36}, {NULL}};
const Segment SGI182445[] = {{&S48991[30], 6}, {NULL}};
const Segment SGI182446[] = {{&S48991[129], 3}, {NULL}};
const Segment SGI182447[] = {{&S48991[94], 6}, {NULL}};
const Segment SGI182448[] = {{&S71203[44], 4}, {NULL}};
const Segment SGI182449[] = {{&S71203[44], 31}, {NULL}};
const Segment SGI182450[] = {{&S71203[30], 6}, {NULL}};
const Segment SGI182451[] = {{&S71203[31], 3}, {NULL}};
const Segment SGI182452[] = {{&S71203[89], 6}, {NULL}};
const Segment SGI182453[] = {{&S71204[44], 4}, {NULL}};
const Segment SGI182454[] = {{&S71204[44], 16}, {NULL}};
const Segment SGI182455[] = {{&S71204[30], 6}, {NULL}};
const Segment SGI182456[] = {{&S71204[31], 3}, {NULL}};
const Segment SGI182457[] = {{&S71204[74], 6}, {NULL}};
const Segment SGI182458[] = {{&S71205[38], 4}, {NULL}};
const Segment SGI182459[] = {{&S71205[31], 6}, {NULL}};
const Segment SGI182460[] = {{&S71208[34], 7}, {NULL}};
const Segment SGI182461[] = {{&S71208[27], 6}, {NULL}};
const Segment SGI182462[] = {{&S71208[111], 9}, {NULL}};
const Segment SGI182463[] = {{&S71208[84], 6}, {NULL}};
const Segment SGI182464[] = {{&S61749[32], 10}, {NULL}};
const Segment SGI182465[] = {{&S61749[27], 4}, {NULL}};
const Segment SGI182466[] = {{&S61749[95], 9}, {NULL}};
const Segment SGI182467[] = {{&S61749[68], 6}, {NULL}};
const Segment SGI182468[] = {{&S71239[34], 7}, {NULL}};
const Segment SGI182469[] = {{&S71239[27], 6}, {NULL}};
const Segment SGI182470[] = {{&S71239[94], 9}, {NULL}};
const Segment SGI182471[] = {{&S71239[67], 6}, {NULL}};
const Segment SGI182472[] = {{&S71240[34], 14}, {NULL}};
const Segment SGI182473[] = {{&S71240[27], 6}, {NULL}};
const Segment SGI182474[] = {{&S71240[101], 9}, {NULL}};
const Segment SGI182475[] = {{&S71240[74], 6}, {NULL}};
const Segment SGI182476[] = {{&S71258[34], 10}, {NULL}};
const Segment SGI182477[] = {{&S71258[27], 6}, {NULL}};
const Segment SGI182478[] = {{&S71258[97], 9}, {NULL}};
const Segment SGI182479[] = {{&S71258[70], 6}, {NULL}};
const Segment SGI182480[] = {{&S71259[34], 11}, {NULL}};
const Segment SGI182481[] = {{&S71259[27], 6}, {NULL}};
const Segment SGI182482[] = {{&S71259[98], 9}, {NULL}};
const Segment SGI182483[] = {{&S71259[71], 6}, {NULL}};
const Segment SGI182484[] = {{&S49070[44], 10}, {NULL}};
const Segment SGI182485[] = {{&S49070[44], 21}, {NULL}};
const Segment SGI182486[] = {{&S49070[30], 6}, {NULL}};
const Segment SGI182487[] = {{&S49070[132], 3}, {NULL}};
const Segment SGI182488[] = {{&S49070[97], 6}, {NULL}};
const Segment SGI182489[] = {{&S49070[67], 8}, {NULL}};
const Segment SGI182490[] = {{&S49072[42], 10}, {NULL}};
const Segment SGI182491[] = {{&S49072[30], 6}, {NULL}};
const Segment SGI182492[] = {{&S49072[130], 3}, {NULL}};
const Segment SGI182493[] = {{&S49072[95], 6}, {NULL}};
const Segment SGI182494[] = {{&S49072[65], 8}, {NULL}};
const Segment SGI182495[] = {{&S59857[34], 10}, {NULL}};
const Segment SGI182496[] = {{&S49073[44], 10}, {NULL}};
const Segment SGI182497[] = {{&S59857[27], 6}, {NULL}};
const Segment SGI182498[] = {{&S49073[44], 22}, {NULL}};
const Segment SGI182499[] = {{&S49073[30], 6}, {NULL}};
const Segment SGI182500[] = {{&S59857[97], 9}, {NULL}};
const Segment SGI182501[] = {{&S49073[115], 3}, {NULL}};
const Segment SGI182502[] = {{&S59857[70], 6}, {NULL}};
const Segment SGI182503[] = {{&S49073[80], 6}, {NULL}};
const Segment SGI182504[] = {{&S49073[133], 8}, {NULL}};
const Segment SGI182505[] = {{&S71278[34], 13}, {NULL}};
const Segment SGI182506[] = {{&S71278[27], 6}, {NULL}};
const Segment SGI182507[] = {{&S71278[100], 9}, {NULL}};
const Segment SGI182508[] = {{&S71278[73], 6}, {NULL}};
const Segment SGI182509[] = {{&S71287[34], 24}, {NULL}};
const Segment SGI182510[] = {{&S71287[27], 6}, {NULL}};
const Segment SGI182511[] = {{&S59879[44], 25}, {NULL}};
const Segment SGI182512[] = {{&S59879[44], 37}, {NULL}};
const Segment SGI182513[] = {{&S71287[111], 9}, {NULL}};
const Segment SGI182514[] = {{&S59879[30], 6}, {NULL}};
const Segment SGI182515[] = {{&S71287[84], 6}, {NULL}};
const Segment SGI182516[] = {{&S59879[146], 3}, {NULL}};
const Segment SGI182517[] = {{&S59879[111], 6}, {NULL}};
const Segment SGI182518[] = {{&S59879[83], 8}, {NULL}};
const Segment SGI182519[] = {{&S71288[34], 23}, {NULL}};
const Segment SGI182520[] = {{&S71288[27], 6}, {NULL}};
const Segment SGI182521[] = {{&S59880[44], 25}, {NULL}};
const Segment SGI182522[] = {{&S59880[44], 37}, {NULL}};
const Segment SGI182523[] = {{&S71288[110], 9}, {NULL}};
const Segment SGI182524[] = {{&S59880[30], 6}, {NULL}};
const Segment SGI182525[] = {{&S71288[83], 6}, {NULL}};
const Segment SGI182526[] = {{&S59880[146], 3}, {NULL}};
const Segment SGI182527[] = {{&S59880[111], 6}, {NULL}};
const Segment SGI182528[] = {{&S59880[83], 8}, {NULL}};
const Segment SGI182529[] = {{&S71292[44], 15}, {NULL}};
const Segment SGI182530[] = {{&S71292[30], 6}, {NULL}};
const Segment SGI182531[] = {{&S71292[31], 3}, {NULL}};
const Segment SGI182532[] = {{&S71292[86], 6}, {NULL}};
const Segment SGI182533[] = {{&S71294[34], 10}, {NULL}};
const Segment SGI182534[] = {{&S71294[27], 6}, {NULL}};
const Segment SGI182535[] = {{&S71294[97], 9}, {NULL}};
const Segment SGI182536[] = {{&S71294[70], 6}, {NULL}};
const Segment SGI182537[] = {{&S71295[34], 13}, {NULL}};
const Segment SGI182538[] = {{&S71295[27], 6}, {NULL}};
const Segment SGI182539[] = {{&S71295[100], 9}, {NULL}};
const Segment SGI182540[] = {{&S71295[73], 6}, {NULL}};
const Segment SGI182541[] = {{&S59904[44], 13}, {NULL}};
const Segment SGI182542[] = {{&S59904[44], 25}, {NULL}};
const Segment SGI182543[] = {{&S59904[30], 6}, {NULL}};
const Segment SGI182544[] = {{&S59904[118], 3}, {NULL}};
const Segment SGI182545[] = {{&S59904[83], 6}, {NULL}};
const Segment SGI182546[] = {{&S71314[44], 13}, {NULL}};
const Segment SGI182547[] = {{&S71314[30], 6}, {NULL}};
const Segment SGI182548[] = {{&S71314[31], 3}, {NULL}};
const Segment SGI182549[] = {{&S71314[82], 6}, {NULL}};
const Segment SGI182550[] = {{&S71316[34], 9}, {NULL}};
const Segment SGI182551[] = {{&S71316[27], 6}, {NULL}};
const Segment SGI182552[] = {{&S71316[96], 9}, {NULL}};
const Segment SGI182553[] = {{&S71316[69], 6}, {NULL}};
const Segment SGI182554[] = {{&S59914[34], 29}, {NULL}};
const Segment SGI182555[] = {{&S59914[27], 6}, {NULL}};
const Segment SGI182556[] = {{&S59914[116], 9}, {NULL}};
const Segment SGI182557[] = {{&S59914[89], 6}, {NULL}};
const Segment SGI182558[] = {{&S59915[34], 14}, {NULL}};
const Segment SGI182559[] = {{&S59915[27], 6}, {NULL}};
const Segment SGI182560[] = {{&S59915[101], 9}, {NULL}};
const Segment SGI182561[] = {{&S59915[74], 6}, {NULL}};
const Segment SGI182562[] = {{&S71319[34], 6}, {NULL}};
const Segment SGI182563[] = {{&S71319[27], 6}, {NULL}};
const Segment SGI182564[] = {{&S71319[93], 9}, {NULL}};
const Segment SGI182565[] = {{&S71319[66], 6}, {NULL}};
const Segment SGI182566[] = {{&S71325[44], 11}, {NULL}};
const Segment SGI182567[] = {{&S71325[44], 18}, {NULL}};
const Segment SGI182568[] = {{&S71325[30], 6}, {NULL}};
const Segment SGI182569[] = {{&S71325[31], 3}, {NULL}};
const Segment SGI182570[] = {{&S71325[76], 6}, {NULL}};
const Segment SGI182571[] = {{&S71327[44], 11}, {NULL}};
const Segment SGI182572[] = {{&S71327[44], 32}, {NULL}};
const Segment SGI182573[] = {{&S71327[30], 6}, {NULL}};
const Segment SGI182574[] = {{&S71327[31], 3}, {NULL}};
const Segment SGI182575[] = {{&S71327[90], 6}, {NULL}};
const Segment SGI182576[] = {{&S64355[78], 6}, {NULL}};
const Segment SGI182577[] = {{&S64355[105], 13}, {NULL}};
const Segment SGI182578[] = {{&S64356[44], 7}, {NULL}};
const Segment SGI182579[] = {{&S64356[44], 19}, {NULL}};
const Segment SGI182580[] = {{&S64356[30], 6}, {NULL}};
const Segment SGI182581[] = {{&S64356[77], 6}, {NULL}};
const Segment SGI182582[] = {{&S64356[104], 13}, {NULL}};
const Segment SGI182583[] = {{&S64357[44], 7}, {NULL}};
const Segment SGI182584[] = {{&S59936[44], 5}, {NULL}};
const Segment SGI182585[] = {{&S64357[44], 19}, {NULL}};
const Segment SGI182586[] = {{&S59936[44], 17}, {NULL}};
const Segment SGI182587[] = {{&S64357[30], 6}, {NULL}};
const Segment SGI182588[] = {{&S59936[30], 6}, {NULL}};
const Segment SGI182589[] = {{&S59936[126], 3}, {NULL}};
const Segment SGI182590[] = {{&S64357[77], 6}, {NULL}};
const Segment SGI182591[] = {{&S59936[91], 6}, {NULL}};
const Segment SGI182592[] = {{&S64357[104], 13}, {NULL}};
const Segment SGI182593[] = {{&S59936[63], 8}, {NULL}};
const Segment SGI182594[] = {{&S59937[44], 5}, {NULL}};
const Segment SGI182595[] = {{&S59937[44], 17}, {NULL}};
const Segment SGI182596[] = {{&S59937[30], 6}, {NULL}};
const Segment SGI182597[] = {{&S59937[130], 3}, {NULL}};
const Segment SGI182598[] = {{&S59937[95], 6}, {NULL}};
const Segment SGI182599[] = {{&S59937[63], 8}, {NULL}};
const Segment SGI182600[] = {{&S59938[44], 9}, {NULL}};
const Segment SGI182601[] = {{&S59938[44], 21}, {NULL}};
const Segment SGI182602[] = {{&S59938[30], 6}, {NULL}};
const Segment SGI182603[] = {{&S49179[34], 10}, {NULL}};
const Segment SGI182604[] = {{&S49179[27], 6}, {NULL}};
const Segment SGI182605[] = {{&S59938[130], 3}, {NULL}};
const Segment SGI182606[] = {{&S59938[95], 6}, {NULL}};
const Segment SGI182607[] = {{&S59938[67], 8}, {NULL}};
const Segment SGI182608[] = {{&S49179[96], 9}, {NULL}};
const Segment SGI182609[] = {{&S49179[69], 6}, {NULL}};
const Segment SGI182610[] = {{&S71345[44], 9}, {NULL}};
const Segment SGI182611[] = {{&S71345[44], 21}, {NULL}};
const Segment SGI182612[] = {{&S71345[30], 6}, {NULL}};
const Segment SGI182613[] = {{&S71345[31], 3}, {NULL}};
const Segment SGI182614[] = {{&S71345[98], 6}, {NULL}};
const Segment SGI182615[] = {{&S71345[67], 8}, {NULL}};
const Segment SGI182616[] = {{&S71346[44], 9}, {NULL}};
const Segment SGI182617[] = {{&S71346[44], 20}, {NULL}};
const Segment SGI182618[] = {{&S71346[30], 6}, {NULL}};
const Segment SGI182619[] = {{&S71346[31], 3}, {NULL}};
const Segment SGI182620[] = {{&S71346[97], 6}, {NULL}};
const Segment SGI182621[] = {{&S71346[66], 8}, {NULL}};
const Segment SGI182622[] = {{&S71347[44], 9}, {&S71347[85], 8}, {NULL}};
const Segment SGI182623[] = {{&S71347[44], 21}, {NULL}};
const Segment SGI182624[] = {{&S71347[30], 6}, {NULL}};
const Segment SGI182625[] = {{&S71347[31], 3}, {NULL}};
const Segment SGI182626[] = {{&S71347[119], 5}, {NULL}};
const Segment SGI182627[] = {{&S71347[67], 8}, {NULL}};
const Segment SGI182628[] = {{&S71351[34], 7}, {NULL}};
const Segment SGI182629[] = {{&S71351[27], 6}, {NULL}};
const Segment SGI182630[] = {{&S71351[94], 9}, {NULL}};
const Segment SGI182631[] = {{&S71351[67], 6}, {NULL}};
const Segment SGI182632[] = {{&S71354[34], 10}, {NULL}};
const Segment SGI182633[] = {{&S71354[27], 6}, {NULL}};
const Segment SGI182634[] = {{&S71354[97], 9}, {NULL}};
const Segment SGI182635[] = {{&S71354[70], 6}, {NULL}};
const Segment SGI182636[] = {{&S64384[34], 7}, {NULL}};
const Segment SGI182637[] = {{&S64384[27], 6}, {NULL}};
const Segment SGI182638[] = {{&S64384[94], 9}, {NULL}};
const Segment SGI182639[] = {{&S64384[67], 6}, {NULL}};
const Segment SGI182640[] = {{&S64386[44], 7}, {NULL}};
const Segment SGI182641[] = {{&S64386[44], 19}, {NULL}};
const Segment SGI182642[] = {{&S64386[30], 6}, {NULL}};
const Segment SGI182643[] = {{&S64386[112], 3}, {NULL}};
const Segment SGI182644[] = {{&S64386[77], 6}, {NULL}};
const Segment SGI182645[] = {{&S64387[38], 7}, {NULL}};
const Segment SGI182646[] = {{&S64387[31], 6}, {NULL}};
const Segment SGI182647[] = {{&S71363[34], 15}, {NULL}};
const Segment SGI182648[] = {{&S71363[27], 6}, {NULL}};
const Segment SGI182649[] = {{&S71363[102], 9}, {NULL}};
const Segment SGI182650[] = {{&S71363[75], 6}, {NULL}};
const Segment SGI182651[] = {{&S64388[44], 10}, {NULL}};
const Segment SGI182652[] = {{&S64388[44], 33}, {NULL}};
const Segment SGI182653[] = {{&S64388[30], 6}, {NULL}};
const Segment SGI182654[] = {{&S64388[126], 3}, {NULL}};
const Segment SGI182655[] = {{&S64388[91], 6}, {NULL}};
const Segment SGI182656[] = {{&S64389[44], 13}, {NULL}};
const Segment SGI182657[] = {{&S64389[44], 39}, {NULL}};
const Segment SGI182658[] = {{&S64389[30], 6}, {NULL}};
const Segment SGI182659[] = {{&S64389[132], 3}, {NULL}};
const Segment SGI182660[] = {{&S64389[97], 6}, {NULL}};
const Segment SGI182661[] = {{&S59974[34], 10}, {NULL}};
const Segment SGI182662[] = {{&S59974[27], 6}, {NULL}};
const Segment SGI182663[] = {{&S59974[97], 9}, {NULL}};
const Segment SGI182664[] = {{&S59974[70], 6}, {NULL}};
const Segment SGI182665[] = {{&S59975[34], 18}, {NULL}};
const Segment SGI182666[] = {{&S59975[27], 6}, {NULL}};
const Segment SGI182667[] = {{&S59975[105], 9}, {NULL}};
const Segment SGI182668[] = {{&S59975[78], 6}, {NULL}};
const Segment SGI182669[] = {{&S59976[34], 19}, {NULL}};
const Segment SGI182670[] = {{&S59976[27], 6}, {NULL}};
const Segment SGI182671[] = {{&S59976[106], 9}, {NULL}};
const Segment SGI182672[] = {{&S59976[79], 6}, {NULL}};
const Segment SGI182673[] = {{&S71373[44], 16}, {NULL}};
const Segment SGI182674[] = {{&S71373[44], 28}, {NULL}};
const Segment SGI182675[] = {{&S71373[30], 6}, {NULL}};
const Segment SGI182676[] = {{&S71373[31], 3}, {NULL}};
const Segment SGI182677[] = {{&S71373[86], 6}, {NULL}};
const Segment SGI182678[] = {{&S59977[34], 19}, {NULL}};
const Segment SGI182679[] = {{&S59977[27], 6}, {NULL}};
const Segment SGI182680[] = {{&S59977[106], 9}, {NULL}};
const Segment SGI182681[] = {{&S59977[79], 6}, {NULL}};
const Segment SGI182682[] = {{&S71377[34], 7}, {NULL}};
const Segment SGI182683[] = {{&S71377[27], 6}, {NULL}};
const Segment SGI182684[] = {{&S71377[94], 9}, {NULL}};
const Segment SGI182685[] = {{&S71377[67], 6}, {NULL}};
const Segment SGI182686[] = {{&S58224[44], 7}, {NULL}};
const Segment SGI182687[] = {{&S58224[44], 19}, {NULL}};
const Segment SGI182688[] = {{&S58224[30], 6}, {NULL}};
const Segment SGI182689[] = {{&S58224[112], 3}, {NULL}};
const Segment SGI182690[] = {{&S58224[77], 6}, {NULL}};
const Segment SGI182691[] = {{&S58225[38], 7}, {NULL}};
const Segment SGI182692[] = {{&S58225[31], 6}, {NULL}};
const Segment SGI182693[] = {{&S59986[34], 9}, {NULL}};
const Segment SGI182694[] = {{&S59986[27], 6}, {NULL}};
const Segment SGI182695[] = {{&S59986[96], 9}, {NULL}};
const Segment SGI182696[] = {{&S59986[69], 6}, {NULL}};
const Segment SGI182697[] = {{&S58226[44], 8}, {NULL}};
const Segment SGI182698[] = {{&S58226[44], 20}, {NULL}};
const Segment SGI182699[] = {{&S58226[30], 6}, {NULL}};
const Segment SGI182700[] = {{&S58226[113], 3}, {NULL}};
const Segment SGI182701[] = {{&S58226[78], 6}, {NULL}};
const Segment SGI182702[] = {{&S59989[34], 9}, {NULL}};
const Segment SGI182703[] = {{&S59989[27], 6}, {NULL}};
const Segment SGI182704[] = {{&S59989[96], 9}, {NULL}};
const Segment SGI182705[] = {{&S59989[69], 6}, {NULL}};
const Segment SGI182706[] = {{&S71387[34], 10}, {NULL}};
const Segment SGI182707[] = {{&S71387[27], 6}, {NULL}};
const Segment SGI182708[] = {{&S71387[97], 9}, {NULL}};
const Segment SGI182709[] = {{&S71387[70], 6}, {NULL}};
const Segment SGI182710[] = {{&S71388[34], 11}, {NULL}};
const Segment SGI182711[] = {{&S71388[27], 6}, {NULL}};
const Segment SGI182712[] = {{&S58232[34], 11}, {NULL}};
const Segment SGI182713[] = {{&S58232[27], 6}, {NULL}};
const Segment SGI182714[] = {{&S71388[98], 9}, {NULL}};
const Segment SGI182715[] = {{&S71388[71], 6}, {NULL}};
const Segment SGI182716[] = {{&S58232[98], 9}, {NULL}};
const Segment SGI182717[] = {{&S58232[71], 6}, {NULL}};
const Segment SGI182718[] = {{&S60001[34], 8}, {NULL}};
const Segment SGI182719[] = {{&S60001[27], 6}, {NULL}};
const Segment SGI182720[] = {{&S60001[95], 9}, {NULL}};
const Segment SGI182721[] = {{&S60001[68], 6}, {NULL}};
const Segment SGI182722[] = {{&S71412[44], 14}, {NULL}};
const Segment SGI182723[] = {{&S71412[44], 26}, {NULL}};
const Segment SGI182724[] = {{&S71412[30], 6}, {NULL}};
const Segment SGI182725[] = {{&S71412[31], 3}, {NULL}};
const Segment SGI182726[] = {{&S71412[103], 6}, {NULL}};
const Segment SGI182727[] = {{&S71412[72], 8}, {NULL}};
const Segment SGI182728[] = {{&S71413[44], 14}, {NULL}};
const Segment SGI182729[] = {{&S60019[34], 10}, {NULL}};
const Segment SGI182730[] = {{&S71413[44], 26}, {NULL}};
const Segment SGI182731[] = {{&S60019[27], 6}, {NULL}};
const Segment SGI182732[] = {{&S71413[30], 6}, {NULL}};
const Segment SGI182733[] = {{&S71413[31], 3}, {NULL}};
const Segment SGI182734[] = {{&S60019[97], 9}, {NULL}};
const Segment SGI182735[] = {{&S60019[70], 6}, {NULL}};
const Segment SGI182736[] = {{&S71413[103], 6}, {NULL}};
const Segment SGI182737[] = {{&S71413[72], 8}, {NULL}};
const Segment SGI182738[] = {{&S64440[34], 12}, {NULL}};
const Segment SGI182739[] = {{&S64440[27], 6}, {NULL}};
const Segment SGI182740[] = {{&S64440[99], 9}, {NULL}};
const Segment SGI182741[] = {{&S71414[44], 14}, {NULL}};
const Segment SGI182742[] = {{&S64440[72], 6}, {NULL}};
const Segment SGI182743[] = {{&S71414[44], 26}, {NULL}};
const Segment SGI182744[] = {{&S71414[30], 6}, {NULL}};
const Segment SGI182745[] = {{&S71414[31], 3}, {NULL}};
const Segment SGI182746[] = {{&S71414[99], 6}, {NULL}};
const Segment SGI182747[] = {{&S60021[34], 7}, {NULL}};
const Segment SGI182748[] = {{&S60021[27], 6}, {NULL}};
const Segment SGI182749[] = {{&S71414[72], 8}, {NULL}};
const Segment SGI182750[] = {{&S60021[99], 9}, {NULL}};
const Segment SGI182751[] = {{&S60021[72], 6}, {NULL}};
const Segment SGI182752[] = {{&S60022[34], 7}, {NULL}};
const Segment SGI182753[] = {{&S60022[27], 6}, {NULL}};
const Segment SGI182754[] = {{&S60022[99], 9}, {NULL}};
const Segment SGI182755[] = {{&S60022[72], 6}, {NULL}};
const Segment SGI182756[] = {{&S71417[34], 7}, {NULL}};
const Segment SGI182757[] = {{&S71417[27], 6}, {NULL}};
const Segment SGI182758[] = {{&S71417[98], 9}, {NULL}};
const Segment SGI182759[] = {{&S71417[71], 6}, {NULL}};
const Segment SGI182760[] = {{&S71424[34], 10}, {NULL}};
const Segment SGI182761[] = {{&S71424[27], 6}, {NULL}};
const Segment SGI182762[] = {{&S71424[97], 9}, {NULL}};
const Segment SGI182763[] = {{&S71424[70], 6}, {NULL}};
const Segment SGI182764[] = {{&S64453[44], 10}, {NULL}};
const Segment SGI182765[] = {{&S64453[44], 22}, {NULL}};
const Segment SGI182766[] = {{&S64453[30], 6}, {NULL}};
const Segment SGI182767[] = {{&S64453[80], 6}, {NULL}};
const Segment SGI182768[] = {{&S64453[107], 13}, {NULL}};
const Segment SGI182769[] = {{&S64459[44], 25}, {NULL}};
const Segment SGI182770[] = {{&S64459[44], 37}, {NULL}};
const Segment SGI182771[] = {{&S64459[30], 6}, {NULL}};
const Segment SGI182772[] = {{&S64459[31], 3}, {NULL}};
const Segment SGI182773[] = {{&S64459[95], 6}, {NULL}};
const Segment SGI182774[] = {{&S64460[44], 25}, {NULL}};
const Segment SGI182775[] = {{&S64460[44], 37}, {NULL}};
const Segment SGI182776[] = {{&S64460[30], 6}, {NULL}};
const Segment SGI182777[] = {{&S64460[31], 3}, {NULL}};
const Segment SGI182778[] = {{&S64460[95], 6}, {NULL}};
const Segment SGI182779[] = {{&S64461[44], 25}, {NULL}};
const Segment SGI182780[] = {{&S64461[44], 37}, {NULL}};
const Segment SGI182781[] = {{&S64461[30], 6}, {NULL}};
const Segment SGI182782[] = {{&S64461[31], 3}, {NULL}};
const Segment SGI182783[] = {{&S64461[95], 6}, {NULL}};
const Segment SGI182784[] = {{&S64462[44], 10}, {NULL}};
const Segment SGI182785[] = {{&S64462[44], 22}, {NULL}};
const Segment SGI182786[] = {{&S64462[30], 6}, {NULL}};
const Segment SGI182787[] = {{&S64462[115], 3}, {NULL}};
const Segment SGI182788[] = {{&S64462[80], 6}, {NULL}};
const Segment SGI182789[] = {{&S71441[44], 10}, {NULL}};
const Segment SGI182790[] = {{&S71441[44], 22}, {NULL}};
const Segment SGI182791[] = {{&S71441[30], 6}, {NULL}};
const Segment SGI182792[] = {{&S71441[31], 3}, {NULL}};
const Segment SGI182793[] = {{&S71441[80], 6}, {NULL}};
const Segment SGI182794[] = {{&S71442[44], 10}, {NULL}};
const Segment SGI182795[] = {{&S71442[44], 22}, {NULL}};
const Segment SGI182796[] = {{&S71442[30], 6}, {NULL}};
const Segment SGI182797[] = {{&S71442[31], 3}, {NULL}};
const Segment SGI182798[] = {{&S71442[80], 6}, {NULL}};
const Segment SGI182799[] = {{&S71443[44], 10}, {NULL}};
const Segment SGI182800[] = {{&S71443[44], 22}, {NULL}};
const Segment SGI182801[] = {{&S71443[30], 6}, {NULL}};
const Segment SGI182802[] = {{&S71443[31], 3}, {NULL}};
const Segment SGI182803[] = {{&S71443[80], 6}, {NULL}};
const Segment SGI182804[] = {{&S71445[38], 10}, {NULL}};
const Segment SGI182805[] = {{&S71445[31], 6}, {NULL}};
const Segment SGI182806[] = {{&S71446[44], 8}, {NULL}};
const Segment SGI182807[] = {{&S71446[44], 20}, {NULL}};
const Segment SGI182808[] = {{&S71446[30], 6}, {NULL}};
const Segment SGI182809[] = {{&S71446[31], 3}, {NULL}};
const Segment SGI182810[] = {{&S71446[97], 6}, {NULL}};
const Segment SGI182811[] = {{&S71446[66], 8}, {NULL}};
const Segment SGI182812[] = {{&S71447[44], 8}, {NULL}};
const Segment SGI182813[] = {{&S71447[44], 20}, {NULL}};
const Segment SGI182814[] = {{&S71447[30], 6}, {NULL}};
const Segment SGI182815[] = {{&S71447[31], 3}, {NULL}};
const Segment SGI182816[] = {{&S71447[97], 6}, {NULL}};
const Segment SGI182817[] = {{&S71447[66], 8}, {NULL}};
const Segment SGI182818[] = {{&S71448[44], 8}, {NULL}};
const Segment SGI182819[] = {{&S71448[44], 20}, {NULL}};
const Segment SGI182820[] = {{&S71448[30], 6}, {NULL}};
const Segment SGI182821[] = {{&S71448[31], 3}, {NULL}};
const Segment SGI182822[] = {{&S71448[93], 6}, {NULL}};
const Segment SGI182823[] = {{&S71448[66], 8}, {NULL}};
const Segment SGI182824[] = {{&S71449[44], 5}, {NULL}};
const Segment SGI182825[] = {{&S71449[44], 17}, {NULL}};
const Segment SGI182826[] = {{&S71449[30], 6}, {NULL}};
const Segment SGI182827[] = {{&S71449[31], 3}, {NULL}};
const Segment SGI182828[] = {{&S71449[75], 6}, {NULL}};
const Segment SGI182829[] = {{&S71450[38], 5}, {NULL}};
const Segment SGI182830[] = {{&S71450[31], 6}, {NULL}};
const Segment SGI182831[] = {{&S71453[44], 14}, {NULL}};
const Segment SGI182832[] = {{&S71453[44], 26}, {NULL}};
const Segment SGI182833[] = {{&S71453[30], 6}, {NULL}};
const Segment SGI182834[] = {{&S71453[31], 3}, {NULL}};
const Segment SGI182835[] = {{&S71453[84], 6}, {NULL}};
const Segment SGI182836[] = {{&S71454[44], 14}, {NULL}};
const Segment SGI182837[] = {{&S71454[44], 26}, {NULL}};
const Segment SGI182838[] = {{&S71454[30], 6}, {NULL}};
const Segment SGI182839[] = {{&S71454[31], 3}, {NULL}};
const Segment SGI182840[] = {{&S71454[84], 6}, {NULL}};
const Segment SGI182841[] = {{&S64486[44], 10}, {NULL}};
const Segment SGI182842[] = {{&S64486[44], 22}, {NULL}};
const Segment SGI182843[] = {{&S64486[30], 6}, {NULL}};
const Segment SGI182844[] = {{&S64486[80], 6}, {NULL}};
const Segment SGI182845[] = {{&S64486[107], 13}, {NULL}};
const Segment SGI182846[] = {{&S71456[44], 10}, {NULL}};
const Segment SGI182847[] = {{&S71456[44], 22}, {NULL}};
const Segment SGI182848[] = {{&S71456[30], 6}, {NULL}};
const Segment SGI182849[] = {{&S71456[31], 3}, {NULL}};
const Segment SGI182850[] = {{&S71456[80], 6}, {NULL}};
const Segment SGI182851[] = {{&S60082[34], 9}, {NULL}};
const Segment SGI182852[] = {{&S60082[27], 6}, {NULL}};
const Segment SGI182853[] = {{&S60082[96], 9}, {NULL}};
const Segment SGI182854[] = {{&S60082[69], 6}, {NULL}};
const Segment SGI182855[] = {{&S60083[34], 10}, {NULL}};
const Segment SGI182856[] = {{&S60083[27], 6}, {NULL}};
const Segment SGI182857[] = {{&S60083[97], 9}, {NULL}};
const Segment SGI182858[] = {{&S60083[70], 6}, {NULL}};
const Segment SGI182859[] = {{&S60084[34], 10}, {NULL}};
const Segment SGI182860[] = {{&S60084[27], 6}, {NULL}};
const Segment SGI182861[] = {{&S60084[97], 9}, {NULL}};
const Segment SGI182862[] = {{&S60084[70], 6}, {NULL}};
const Segment SGI182863[] = {{&S60085[34], 10}, {NULL}};
const Segment SGI182864[] = {{&S60085[27], 6}, {NULL}};
const Segment SGI182865[] = {{&S60085[97], 9}, {NULL}};
const Segment SGI182866[] = {{&S60085[70], 6}, {NULL}};
const Segment SGI182867[] = {{&S60089[44], 10}, {NULL}};
const Segment SGI182868[] = {{&S60089[44], 22}, {NULL}};
const Segment SGI182869[] = {{&S60089[30], 6}, {NULL}};
const Segment SGI182870[] = {{&S60089[115], 3}, {NULL}};
const Segment SGI182871[] = {{&S60089[80], 6}, {NULL}};
const Segment SGI182872[] = {{&S60090[38], 10}, {NULL}};
const Segment SGI182873[] = {{&S60090[31], 6}, {NULL}};
const Segment SGI182874[] = {{&S60091[44], 10}, {NULL}};
const Segment SGI182875[] = {{&S60091[44], 22}, {NULL}};
const Segment SGI182876[] = {{&S60091[30], 6}, {NULL}};
const Segment SGI182877[] = {{&S60091[115], 3}, {NULL}};
const Segment SGI182878[] = {{&S60091[80], 6}, {NULL}};
const Segment SGI182879[] = {{&S60100[44], 10}, {NULL}};
const Segment SGI182880[] = {{&S60100[44], 22}, {NULL}};
const Segment SGI182881[] = {{&S60100[30], 6}, {NULL}};
const Segment SGI182882[] = {{&S60100[80], 6}, {NULL}};
const Segment SGI182883[] = {{&S64515[44], 17}, {NULL}};
const Segment SGI182884[] = {{&S64515[44], 29}, {NULL}};
const Segment SGI182885[] = {{&S64515[30], 6}, {NULL}};
const Segment SGI182886[] = {{&S60101[42], 10}, {NULL}};
const Segment SGI182887[] = {{&S60101[30], 6}, {NULL}};
const Segment SGI182888[] = {{&S64515[31], 3}, {NULL}};
const Segment SGI182889[] = {{&S64515[87], 6}, {NULL}};
const Segment SGI182890[] = {{&S60101[113], 3}, {NULL}};
const Segment SGI182891[] = {{&S60101[78], 6}, {NULL}};
const Segment SGI182892[] = {{&S64516[38], 17}, {NULL}};
const Segment SGI182893[] = {{&S64516[31], 6}, {NULL}};
const Segment SGI182894[] = {{&S60102[58], 10}, {NULL}};
const Segment SGI182895[] = {{&S60102[58], 71}, {NULL}};
const Segment SGI182896[] = {{&S60102[44], 6}, {NULL}};
const Segment SGI182897[] = {{&S60103[30], 1}, {NULL}};
const Segment SGI182898[] = {{&S60103[63], 9}, {NULL}};
const Segment SGI182899[] = {{&S60103[49], 6}, {NULL}};
const Segment SGI182900[] = {{&S60104[44], 10}, {NULL}};
const Segment SGI182901[] = {{&S60104[44], 42}, {NULL}};
const Segment SGI182902[] = {{&S60104[30], 6}, {NULL}};
const Segment SGI182903[] = {{&S60106[44], 10}, {NULL}};
const Segment SGI182904[] = {{&S60106[44], 22}, {NULL}};
const Segment SGI182905[] = {{&S60106[30], 6}, {NULL}};
const Segment SGI182906[] = {{&S60106[115], 3}, {NULL}};
const Segment SGI182907[] = {{&S60106[80], 6}, {NULL}};
const Segment SGI182908[] = {{&S60107[44], 10}, {NULL}};
const Segment SGI182909[] = {{&S60107[44], 22}, {NULL}};
const Segment SGI182910[] = {{&S60107[30], 6}, {NULL}};
const Segment SGI182911[] = {{&S60113[44], 13}, {NULL}};
const Segment SGI182912[] = {{&S60113[44], 25}, {NULL}};
const Segment SGI182913[] = {{&S60113[30], 6}, {NULL}};
const Segment SGI182914[] = {{&S60113[118], 3}, {NULL}};
const Segment SGI182915[] = {{&S60113[83], 6}, {NULL}};
const Segment SGI182916[] = {{&S60114[38], 13}, {NULL}};
const Segment SGI182917[] = {{&S60114[31], 6}, {NULL}};
const Segment SGI182918[] = {{&S71464[34], 5}, {NULL}};
const Segment SGI182919[] = {{&S71464[27], 6}, {NULL}};
const Segment SGI182920[] = {{&S71464[92], 9}, {NULL}};
const Segment SGI182921[] = {{&S71464[65], 6}, {NULL}};
const Segment SGI182922[] = {{&S60129[34], 14}, {NULL}};
const Segment SGI182923[] = {{&S60129[27], 6}, {NULL}};
const Segment SGI182924[] = {{&S60129[111], 9}, {NULL}};
const Segment SGI182925[] = {{&S60129[84], 6}, {NULL}};
const Segment SGI182926[] = {{&S64549[44], 10}, {NULL}};
const Segment SGI182927[] = {{&S64549[44], 22}, {NULL}};
const Segment SGI182928[] = {{&S64549[30], 6}, {NULL}};
const Segment SGI182929[] = {{&S64549[115], 3}, {NULL}};
const Segment SGI182930[] = {{&S64549[80], 6}, {NULL}};
const Segment SGI182931[] = {{&S60130[34], 14}, {NULL}};
const Segment SGI182932[] = {{&S60130[27], 6}, {NULL}};
const Segment SGI182933[] = {{&S60130[111], 9}, {NULL}};
const Segment SGI182934[] = {{&S60130[84], 6}, {NULL}};
const Segment SGI182935[] = {{&S64550[38], 10}, {NULL}};
const Segment SGI182936[] = {{&S64550[31], 6}, {NULL}};
const Segment SGI182937[] = {{&S60131[34], 7}, {NULL}};
const Segment SGI182938[] = {{&S60131[27], 6}, {NULL}};
const Segment SGI182939[] = {{&S60131[94], 9}, {NULL}};
const Segment SGI182940[] = {{&S60131[67], 6}, {NULL}};
const Segment SGI182941[] = {{&S64551[44], 10}, {NULL}};
const Segment SGI182942[] = {{&S64551[44], 22}, {NULL}};
const Segment SGI182943[] = {{&S64551[30], 6}, {NULL}};
const Segment SGI182944[] = {{&S64551[115], 3}, {NULL}};
const Segment SGI182945[] = {{&S64551[80], 6}, {NULL}};
const Segment SGI182946[] = {{&S60137[34], 18}, {NULL}};
const Segment SGI182947[] = {{&S60137[27], 6}, {NULL}};
const Segment SGI182948[] = {{&S60137[105], 9}, {NULL}};
const Segment SGI182949[] = {{&S60137[78], 6}, {NULL}};
const Segment SGI182950[] = {{&S71503[34], 9}, {NULL}};
const Segment SGI182951[] = {{&S71503[27], 6}, {NULL}};
const Segment SGI182952[] = {{&S71503[98], 9}, {NULL}};
const Segment SGI182953[] = {{&S71503[71], 6}, {NULL}};
const Segment SGI182954[] = {{&S71517[44], 11}, {NULL}};
const Segment SGI182955[] = {{&S71517[44], 23}, {NULL}};
const Segment SGI182956[] = {{&S71517[30], 6}, {NULL}};
const Segment SGI182957[] = {{&S71517[31], 3}, {NULL}};
const Segment SGI182958[] = {{&S71517[81], 6}, {NULL}};
const Segment SGI182959[] = {{&S71519[44], 30}, {NULL}};
const Segment SGI182960[] = {{&S71519[44], 42}, {NULL}};
const Segment SGI182961[] = {{&S71519[30], 6}, {NULL}};
const Segment SGI182962[] = {{&S71519[31], 3}, {NULL}};
const Segment SGI182963[] = {{&S71519[100], 6}, {NULL}};
const Segment SGI182964[] = {{&S71520[44], 25}, {NULL}};
const Segment SGI182965[] = {{&S71520[44], 37}, {NULL}};
const Segment SGI182966[] = {{&S71520[30], 6}, {NULL}};
const Segment SGI182967[] = {{&S71520[31], 3}, {NULL}};
const Segment SGI182968[] = {{&S71520[95], 6}, {NULL}};
const Segment SGI182969[] = {{&S60170[34], 11}, {NULL}};
const Segment SGI182970[] = {{&S60170[27], 6}, {NULL}};
const Segment SGI182971[] = {{&S60170[98], 9}, {NULL}};
const Segment SGI182972[] = {{&S60170[71], 6}, {NULL}};
const Segment SGI182973[] = {{&S71521[42], 30}, {NULL}};
const Segment SGI182974[] = {{&S71521[30], 6}, {NULL}};
const Segment SGI182975[] = {{&S71521[31], 3}, {NULL}};
const Segment SGI182976[] = {{&S71521[98], 6}, {NULL}};
const Segment SGI182977[] = {{&S60171[34], 11}, {NULL}};
const Segment SGI182978[] = {{&S60171[27], 6}, {NULL}};
const Segment SGI182979[] = {{&S60171[98], 9}, {NULL}};
const Segment SGI182980[] = {{&S60171[71], 6}, {NULL}};
const Segment SGI182981[] = {{&S60175[34], 10}, {NULL}};
const Segment SGI182982[] = {{&S60175[27], 6}, {NULL}};
const Segment SGI182983[] = {{&S60175[97], 9}, {NULL}};
const Segment SGI182984[] = {{&S60175[70], 6}, {NULL}};
const Segment SGI182985[] = {{&S60176[34], 11}, {NULL}};
const Segment SGI182986[] = {{&S60176[27], 6}, {NULL}};
const Segment SGI182987[] = {{&S60176[98], 9}, {NULL}};
const Segment SGI182988[] = {{&S60176[71], 6}, {NULL}};
const Segment SGI182989[] = {{&S71528[34], 13}, {NULL}};
const Segment SGI182990[] = {{&S71528[27], 6}, {NULL}};
const Segment SGI182991[] = {{&S71528[100], 9}, {NULL}};
const Segment SGI182992[] = {{&S71528[73], 6}, {NULL}};
const Segment SGI182993[] = {{&S71549[34], 7}, {NULL}};
const Segment SGI182994[] = {{&S71549[27], 6}, {NULL}};
const Segment SGI182995[] = {{&S71549[100], 9}, {NULL}};
const Segment SGI182996[] = {{&S71549[73], 6}, {NULL}};
const Segment SGI182997[] = {{&S71550[34], 7}, {NULL}};
const Segment SGI182998[] = {{&S71550[27], 6}, {NULL}};
const Segment SGI182999[] = {{&S71550[100], 9}, {NULL}};
const Segment SGI183000[] = {{&S71550[73], 6}, {NULL}};
const Segment SGI183001[] = {{&S58411[44], 15}, {NULL}};
const Segment SGI183002[] = {{&S58411[44], 27}, {NULL}};
const Segment SGI183003[] = {{&S58411[30], 6}, {NULL}};
const Segment SGI183004[] = {{&S58411[120], 3}, {NULL}};
const Segment SGI183005[] = {{&S58411[85], 6}, {NULL}};
const Segment SGI183006[] = {{&S71559[44], 10}, {NULL}};
const Segment SGI183007[] = {{&S71559[44], 22}, {NULL}};
const Segment SGI183008[] = {{&S71559[30], 6}, {NULL}};
const Segment SGI183009[] = {{&S71559[31], 3}, {NULL}};
const Segment SGI183010[] = {{&S71559[99], 6}, {NULL}};
const Segment SGI183011[] = {{&S71559[68], 8}, {NULL}};
const Segment SGI183012[] = {{&S71560[44], 10}, {NULL}};
const Segment SGI183013[] = {{&S71560[44], 22}, {NULL}};
const Segment SGI183014[] = {{&S71560[30], 6}, {NULL}};
const Segment SGI183015[] = {{&S71560[31], 3}, {NULL}};
const Segment SGI183016[] = {{&S71560[99], 6}, {NULL}};
const Segment SGI183017[] = {{&S71560[68], 8}, {NULL}};
const Segment SGI183018[] = {{&S71561[34], 13}, {NULL}};
const Segment SGI183019[] = {{&S71561[27], 6}, {NULL}};
const Segment SGI183020[] = {{&S71561[100], 9}, {NULL}};
const Segment SGI183021[] = {{&S71561[73], 6}, {NULL}};
const Segment SGI183022[] = {{&S71562[34], 10}, {NULL}};
const Segment SGI183023[] = {{&S71562[27], 6}, {NULL}};
const Segment SGI183024[] = {{&S71562[97], 9}, {NULL}};
const Segment SGI183025[] = {{&S71562[70], 6}, {NULL}};
const Segment SGI183026[] = {{&S71563[34], 7}, {NULL}};
const Segment SGI183027[] = {{&S71563[27], 6}, {NULL}};
const Segment SGI183028[] = {{&S60226[34], 8}, {NULL}};
const Segment SGI183029[] = {{&S60226[27], 6}, {NULL}};
const Segment SGI183030[] = {{&S71563[105], 9}, {NULL}};
const Segment SGI183031[] = {{&S71563[78], 6}, {NULL}};
const Segment SGI183032[] = {{&S60226[101], 9}, {NULL}};
const Segment SGI183033[] = {{&S60226[74], 6}, {NULL}};
const Segment SGI183034[] = {{&S64653[34], 9}, {NULL}};
const Segment SGI183035[] = {{&S64653[27], 6}, {NULL}};
const Segment SGI183036[] = {{&S71567[34], 13}, {NULL}};
const Segment SGI183037[] = {{&S71567[27], 6}, {NULL}};
const Segment SGI183038[] = {{&S64653[96], 9}, {NULL}};
const Segment SGI183039[] = {{&S64653[69], 6}, {NULL}};
const Segment SGI183040[] = {{&S71567[100], 9}, {NULL}};
const Segment SGI183041[] = {{&S71567[73], 6}, {NULL}};
const Segment SGI183042[] = {{&S71568[34], 18}, {NULL}};
const Segment SGI183043[] = {{&S71568[27], 6}, {NULL}};
const Segment SGI183044[] = {{&S71568[105], 9}, {NULL}};
const Segment SGI183045[] = {{&S71568[78], 6}, {NULL}};
const Segment SGI183046[] = {{&S49388[44], 10}, {NULL}};
const Segment SGI183047[] = {{&S49388[44], 21}, {NULL}};
const Segment SGI183048[] = {{&S49388[30], 6}, {NULL}};
const Segment SGI183049[] = {{&S49388[114], 3}, {NULL}};
const Segment SGI183050[] = {{&S49388[79], 6}, {NULL}};
const Segment SGI183051[] = {{&S49389[38], 10}, {NULL}};
const Segment SGI183052[] = {{&S49389[31], 6}, {NULL}};
const Segment SGI183053[] = {{&S58443[34], 14}, {NULL}};
const Segment SGI183054[] = {{&S58443[27], 6}, {NULL}};
const Segment SGI183055[] = {{&S58443[101], 9}, {NULL}};
const Segment SGI183056[] = {{&S58443[74], 6}, {NULL}};
const Segment SGI183057[] = {{&S58443[142], 8}, {NULL}};
const Segment SGI183058[] = {{&S71587[34], 10}, {NULL}};
const Segment SGI183059[] = {{&S71587[27], 6}, {NULL}};
const Segment SGI183060[] = {{&S71587[97], 9}, {NULL}};
const Segment SGI183061[] = {{&S71587[70], 6}, {NULL}};
const Segment SGI183062[] = {{&S60260[34], 18}, {NULL}};
const Segment SGI183063[] = {{&S60260[27], 6}, {NULL}};
const Segment SGI183064[] = {{&S60260[105], 9}, {NULL}};
const Segment SGI183065[] = {{&S60260[78], 6}, {NULL}};
const Segment SGI183066[] = {{&S60261[34], 19}, {NULL}};
const Segment SGI183067[] = {{&S60261[27], 6}, {NULL}};
const Segment SGI183068[] = {{&S60261[106], 9}, {NULL}};
const Segment SGI183069[] = {{&S60261[79], 6}, {NULL}};
const Segment SGI183070[] = {{&S71595[44], 12}, {NULL}};
const Segment SGI183071[] = {{&S71595[44], 24}, {NULL}};
const Segment SGI183072[] = {{&S71595[30], 6}, {NULL}};
const Segment SGI183073[] = {{&S71595[31], 3}, {NULL}};
const Segment SGI183074[] = {{&S71595[82], 6}, {NULL}};
const Segment SGI183075[] = {{&S71600[34], 10}, {NULL}};
const Segment SGI183076[] = {{&S71600[27], 6}, {NULL}};
const Segment SGI183077[] = {{&S60270[34], 7}, {NULL}};
const Segment SGI183078[] = {{&S71600[97], 9}, {NULL}};
const Segment SGI183079[] = {{&S71600[70], 6}, {NULL}};
const Segment SGI183080[] = {{&S60270[27], 6}, {NULL}};
const Segment SGI183081[] = {{&S71600[138], 8}, {NULL}};
const Segment SGI183082[] = {{&S60270[98], 9}, {NULL}};
const Segment SGI183083[] = {{&S60270[71], 6}, {NULL}};
const Segment SGI183084[] = {{&S64701[44], 10}, {NULL}};
const Segment SGI183085[] = {{&S64701[44], 22}, {NULL}};
const Segment SGI183086[] = {{&S64701[30], 6}, {NULL}};
const Segment SGI183087[] = {{&S64701[115], 3}, {NULL}};
const Segment SGI183088[] = {{&S64701[80], 6}, {NULL}};
const Segment SGI183089[] = {{&S71602[34], 7}, {NULL}};
const Segment SGI183090[] = {{&S71602[27], 6}, {NULL}};
const Segment SGI183091[] = {{&S64702[38], 10}, {NULL}};
const Segment SGI183092[] = {{&S64702[31], 6}, {NULL}};
const Segment SGI183093[] = {{&S71602[98], 9}, {NULL}};
const Segment SGI183094[] = {{&S71602[71], 6}, {NULL}};
const Segment SGI183095[] = {{&S60273[34], 9}, {NULL}};
const Segment SGI183096[] = {{&S60273[27], 6}, {NULL}};
const Segment SGI183097[] = {{&S71602[139], 8}, {NULL}};
const Segment SGI183098[] = {{&S60273[96], 9}, {NULL}};
const Segment SGI183099[] = {{&S60273[69], 6}, {NULL}};
const Segment SGI183100[] = {{&S60274[34], 10}, {NULL}};
const Segment SGI183101[] = {{&S60274[27], 6}, {NULL}};
const Segment SGI183102[] = {{&S60274[97], 9}, {NULL}};
const Segment SGI183103[] = {{&S60274[70], 6}, {NULL}};
const Segment SGI183104[] = {{&S60275[34], 10}, {NULL}};
const Segment SGI183105[] = {{&S60275[27], 6}, {NULL}};
const Segment SGI183106[] = {{&S60275[97], 9}, {NULL}};
const Segment SGI183107[] = {{&S71606[34], 10}, {NULL}};
const Segment SGI183108[] = {{&S60275[70], 6}, {NULL}};
const Segment SGI183109[] = {{&S71606[27], 6}, {NULL}};
const Segment SGI183110[] = {{&S71606[97], 9}, {NULL}};
const Segment SGI183111[] = {{&S71606[70], 6}, {NULL}};
const Segment SGI183112[] = {{&S58488[44], 14}, {NULL}};
const Segment SGI183113[] = {{&S58488[44], 26}, {NULL}};
const Segment SGI183114[] = {{&S58488[30], 6}, {NULL}};
const Segment SGI183115[] = {{&S58488[148], 3}, {NULL}};
const Segment SGI183116[] = {{&S58488[113], 6}, {NULL}};
const Segment SGI183117[] = {{&S60293[44], 6}, {NULL}};
const Segment SGI183118[] = {{&S60293[44], 18}, {NULL}};
const Segment SGI183119[] = {{&S60293[30], 6}, {NULL}};
const Segment SGI183120[] = {{&S60293[111], 3}, {NULL}};
const Segment SGI183121[] = {{&S60293[76], 6}, {NULL}};
const Segment SGI183122[] = {{&S60294[38], 6}, {NULL}};
const Segment SGI183123[] = {{&S60294[31], 6}, {NULL}};
const Segment SGI183124[] = {{&S71629[44], 11}, {NULL}};
const Segment SGI183125[] = {{&S71629[44], 34}, {NULL}};
const Segment SGI183126[] = {{&S71629[30], 6}, {NULL}};
const Segment SGI183127[] = {{&S71629[31], 3}, {NULL}};
const Segment SGI183128[] = {{&S71629[92], 6}, {NULL}};
const Segment SGI183129[] = {{&S71630[44], 11}, {NULL}};
const Segment SGI183130[] = {{&S71630[44], 32}, {NULL}};
const Segment SGI183131[] = {{&S71630[30], 6}, {NULL}};
const Segment SGI183132[] = {{&S71630[31], 3}, {NULL}};
const Segment SGI183133[] = {{&S71630[90], 6}, {NULL}};
const Segment SGI183134[] = {{&S71632[34], 9}, {NULL}};
const Segment SGI183135[] = {{&S71632[27], 6}, {NULL}};
const Segment SGI183136[] = {{&S60303[34], 11}, {NULL}};
const Segment SGI183137[] = {{&S60303[27], 6}, {NULL}};
const Segment SGI183138[] = {{&S71632[103], 9}, {NULL}};
const Segment SGI183139[] = {{&S71632[76], 6}, {NULL}};
const Segment SGI183140[] = {{&S60303[98], 9}, {NULL}};
const Segment SGI183141[] = {{&S60303[71], 6}, {NULL}};
const Segment SGI183142[] = {{&S60304[34], 11}, {NULL}};
const Segment SGI183143[] = {{&S60304[27], 6}, {NULL}};
const Segment SGI183144[] = {{&S60304[98], 9}, {NULL}};
const Segment SGI183145[] = {{&S60304[71], 6}, {NULL}};
const Segment SGI183146[] = {{&S60305[34], 10}, {NULL}};
const Segment SGI183147[] = {{&S60305[27], 6}, {NULL}};
const Segment SGI183148[] = {{&S60305[97], 9}, {NULL}};
const Segment SGI183149[] = {{&S60305[70], 6}, {NULL}};
const Segment SGI183150[] = {{&S58519[44], 10}, {NULL}};
const Segment SGI183151[] = {{&S58519[44], 22}, {NULL}};
const Segment SGI183152[] = {{&S58519[30], 6}, {NULL}};
const Segment SGI183153[] = {{&S58519[133], 3}, {NULL}};
const Segment SGI183154[] = {{&S58519[98], 6}, {NULL}};
const Segment SGI183155[] = {{&S58519[68], 8}, {NULL}};
const Segment SGI183156[] = {{&S58521[44], 10}, {NULL}};
const Segment SGI183157[] = {{&S58521[44], 22}, {NULL}};
const Segment SGI183158[] = {{&S58521[30], 6}, {NULL}};
const Segment SGI183159[] = {{&S58521[133], 3}, {NULL}};
const Segment SGI183160[] = {{&S58521[98], 6}, {NULL}};
const Segment SGI183161[] = {{&S58521[68], 8}, {NULL}};
const Segment SGI183162[] = {{&S60326[34], 7}, {NULL}};
const Segment SGI183163[] = {{&S60326[27], 6}, {NULL}};
const Segment SGI183164[] = {{&S60326[99], 9}, {NULL}};
const Segment SGI183165[] = {{&S60326[72], 6}, {NULL}};
const Segment SGI183166[] = {{&S60327[34], 7}, {NULL}};
const Segment SGI183167[] = {{&S60327[27], 6}, {NULL}};
const Segment SGI183168[] = {{&S60327[99], 9}, {NULL}};
const Segment SGI183169[] = {{&S60327[72], 6}, {NULL}};
const Segment SGI183170[] = {{&S71683[45], 10}, {NULL}};
const Segment SGI183171[] = {{&S71683[30], 6}, {NULL}};
const Segment SGI183172[] = {{&S71683[31], 3}, {NULL}};
const Segment SGI183173[] = {{&S71683[81], 5}, {NULL}};
const Segment SGI183174[] = {{&S71683[139], 13}, {NULL}};
const Segment SGI183175[] = {{&S64803[44], 10}, {NULL}};
const Segment SGI183176[] = {{&S64803[44], 22}, {NULL}};
const Segment SGI183177[] = {{&S64803[30], 6}, {NULL}};
const Segment SGI183178[] = {{&S64803[115], 3}, {NULL}};
const Segment SGI183179[] = {{&S64803[80], 6}, {NULL}};
const Segment SGI183180[] = {{&S64803[150], 3}, {NULL}};
const Segment SGI183181[] = {{&S64804[44], 18}, {NULL}};
const Segment SGI183182[] = {{&S64804[44], 30}, {NULL}};
const Segment SGI183183[] = {{&S64804[30], 6}, {NULL}};
const Segment SGI183184[] = {{&S64804[123], 3}, {NULL}};
const Segment SGI183185[] = {{&S64804[88], 6}, {NULL}};
const Segment SGI183186[] = {{&S71690[44], 24}, {NULL}};
const Segment SGI183187[] = {{&S64804[158], 3}, {NULL}};
const Segment SGI183188[] = {{&S71690[44], 38}, {NULL}};
const Segment SGI183189[] = {{&S71690[30], 6}, {NULL}};
const Segment SGI183190[] = {{&S71690[31], 3}, {NULL}};
const Segment SGI183191[] = {{&S71690[96], 6}, {NULL}};
const Segment SGI183192[] = {{&S64805[44], 11}, {NULL}};
const Segment SGI183193[] = {{&S64805[44], 23}, {NULL}};
const Segment SGI183194[] = {{&S64805[30], 6}, {NULL}};
const Segment SGI183195[] = {{&S64805[116], 3}, {NULL}};
const Segment SGI183196[] = {{&S64805[81], 6}, {NULL}};
const Segment SGI183197[] = {{&S64805[151], 3}, {NULL}};
const Segment SGI183198[] = {{&S64806[44], 10}, {NULL}};
const Segment SGI183199[] = {{&S64806[44], 22}, {NULL}};
const Segment SGI183200[] = {{&S64806[30], 6}, {NULL}};
const Segment SGI183201[] = {{&S71693[34], 10}, {NULL}};
const Segment SGI183202[] = {{&S71693[27], 6}, {NULL}};
const Segment SGI183203[] = {{&S64806[115], 3}, {NULL}};
const Segment SGI183204[] = {{&S64806[80], 6}, {NULL}};
const Segment SGI183205[] = {{&S71693[97], 9}, {NULL}};
const Segment SGI183206[] = {{&S64806[150], 3}, {NULL}};
const Segment SGI183207[] = {{&S60367[34], 11}, {NULL}};
const Segment SGI183208[] = {{&S71693[70], 6}, {NULL}};
const Segment SGI183209[] = {{&S60367[27], 6}, {NULL}};
const Segment SGI183210[] = {{&S60367[98], 9}, {NULL}};
const Segment SGI183211[] = {{&S60367[71], 6}, {NULL}};
const Segment SGI183212[] = {{&S64807[44], 10}, {NULL}};
const Segment SGI183213[] = {{&S64807[44], 22}, {NULL}};
const Segment SGI183214[] = {{&S64807[30], 6}, {NULL}};
const Segment SGI183215[] = {{&S71694[34], 11}, {NULL}};
const Segment SGI183216[] = {{&S71694[27], 6}, {NULL}};
const Segment SGI183217[] = {{&S64807[115], 3}, {NULL}};
const Segment SGI183218[] = {{&S64807[80], 6}, {NULL}};
const Segment SGI183219[] = {{&S60368[34], 10}, {NULL}};
const Segment SGI183220[] = {{&S71694[98], 9}, {NULL}};
const Segment SGI183221[] = {{&S60368[27], 6}, {NULL}};
const Segment SGI183222[] = {{&S64807[150], 3}, {NULL}};
const Segment SGI183223[] = {{&S71694[71], 6}, {NULL}};
const Segment SGI183224[] = {{&S60368[97], 9}, {NULL}};
const Segment SGI183225[] = {{&S60368[70], 6}, {NULL}};
const Segment SGI183226[] = {{&S60384[34], 18}, {NULL}};
const Segment SGI183227[] = {{&S60384[27], 6}, {NULL}};
const Segment SGI183228[] = {{&S60384[105], 9}, {NULL}};
const Segment SGI183229[] = {{&S60384[78], 6}, {NULL}};
const Segment SGI183230[] = {{&S71716[34], 14}, {NULL}};
const Segment SGI183231[] = {{&S71716[27], 6}, {NULL}};
const Segment SGI183232[] = {{&S71716[101], 9}, {NULL}};
const Segment SGI183233[] = {{&S64832[44], 10}, {NULL}};
const Segment SGI183234[] = {{&S71716[74], 6}, {NULL}};
const Segment SGI183235[] = {{&S64832[44], 22}, {NULL}};
const Segment SGI183236[] = {{&S64832[30], 6}, {NULL}};
const Segment SGI183237[] = {{&S64832[136], 3}, {NULL}};
const Segment SGI183238[] = {{&S64832[101], 6}, {NULL}};
const Segment SGI183239[] = {{&S71717[34], 14}, {NULL}};
const Segment SGI183240[] = {{&S71717[27], 6}, {NULL}};
const Segment SGI183241[] = {{&S71717[101], 9}, {NULL}};
const Segment SGI183242[] = {{&S71717[74], 6}, {NULL}};
const Segment SGI183243[] = {{&S71718[34], 15}, {NULL}};
const Segment SGI183244[] = {{&S71718[27], 6}, {NULL}};
const Segment SGI183245[] = {{&S71718[102], 9}, {NULL}};
const Segment SGI183246[] = {{&S71718[75], 6}, {NULL}};
const Segment SGI183247[] = {{&S64835[44], 10}, {NULL}};
const Segment SGI183248[] = {{&S64835[44], 22}, {NULL}};
const Segment SGI183249[] = {{&S64835[30], 6}, {NULL}};
const Segment SGI183250[] = {{&S64835[136], 3}, {NULL}};
const Segment SGI183251[] = {{&S64835[101], 6}, {NULL}};
const Segment SGI183252[] = {{&S49529[34], 10}, {NULL}};
const Segment SGI183253[] = {{&S49529[27], 6}, {NULL}};
const Segment SGI183254[] = {{&S49529[96], 9}, {NULL}};
const Segment SGI183255[] = {{&S49529[69], 6}, {NULL}};
const Segment SGI183256[] = {{&S49530[34], 10}, {NULL}};
const Segment SGI183257[] = {{&S49530[27], 6}, {NULL}};
const Segment SGI183258[] = {{&S71722[44], 7}, {NULL}};
const Segment SGI183259[] = {{&S71722[44], 19}, {NULL}};
const Segment SGI183260[] = {{&S71722[30], 6}, {NULL}};
const Segment SGI183261[] = {{&S49530[96], 9}, {NULL}};
const Segment SGI183262[] = {{&S49530[69], 6}, {NULL}};
const Segment SGI183263[] = {{&S71722[31], 3}, {NULL}};
const Segment SGI183264[] = {{&S71722[77], 6}, {NULL}};
const Segment SGI183265[] = {{&S64841[44], 12}, {NULL}};
const Segment SGI183266[] = {{&S64841[44], 24}, {NULL}};
const Segment SGI183267[] = {{&S64841[30], 6}, {NULL}};
const Segment SGI183268[] = {{&S64841[117], 3}, {NULL}};
const Segment SGI183269[] = {{&S64841[82], 6}, {NULL}};
const Segment SGI183270[] = {{&S64842[44], 12}, {NULL}};
const Segment SGI183271[] = {{&S64842[44], 24}, {NULL}};
const Segment SGI183272[] = {{&S64842[30], 6}, {NULL}};
const Segment SGI183273[] = {{&S64842[117], 3}, {NULL}};
const Segment SGI183274[] = {{&S64842[82], 6}, {NULL}};
const Segment SGI183275[] = {{&S64843[38], 12}, {NULL}};
const Segment SGI183276[] = {{&S64843[31], 6}, {NULL}};
const Segment SGI183277[] = {{&S71728[34], 18}, {NULL}};
const Segment SGI183278[] = {{&S71728[27], 6}, {NULL}};
const Segment SGI183279[] = {{&S71728[113], 9}, {NULL}};
const Segment SGI183280[] = {{&S71728[86], 6}, {NULL}};
const Segment SGI183281[] = {{&S60409[44], 9}, {NULL}};
const Segment SGI183282[] = {{&S60409[44], 20}, {NULL}};
const Segment SGI183283[] = {{&S60409[30], 6}, {NULL}};
const Segment SGI183284[] = {{&S60409[129], 3}, {NULL}};
const Segment SGI183285[] = {{&S64844[34], 6}, {NULL}};
const Segment SGI183286[] = {{&S60409[94], 6}, {NULL}};
const Segment SGI183287[] = {{&S64844[27], 6}, {NULL}};
const Segment SGI183288[] = {{&S60409[66], 8}, {NULL}};
const Segment SGI183289[] = {{&S64844[93], 9}, {NULL}};
const Segment SGI183290[] = {{&S64844[66], 6}, {NULL}};
const Segment SGI183291[] = {{&S60410[44], 9}, {NULL}};
const Segment SGI183292[] = {{&S60410[44], 21}, {NULL}};
const Segment SGI183293[] = {{&S60410[30], 6}, {NULL}};
const Segment SGI183294[] = {{&S60410[134], 3}, {NULL}};
const Segment SGI183295[] = {{&S71730[34], 10}, {NULL}};
const Segment SGI183296[] = {{&S60410[99], 6}, {NULL}};
const Segment SGI183297[] = {{&S71730[27], 6}, {NULL}};
const Segment SGI183298[] = {{&S60410[67], 8}, {NULL}};
const Segment SGI183299[] = {{&S71730[97], 9}, {NULL}};
const Segment SGI183300[] = {{&S71730[70], 6}, {NULL}};
const Segment SGI183301[] = {{&S71730[138], 8}, {NULL}};
const Segment SGI183302[] = {{&S71731[34], 14}, {NULL}};
const Segment SGI183303[] = {{&S71731[27], 6}, {NULL}};
const Segment SGI183304[] = {{&S71731[101], 9}, {NULL}};
const Segment SGI183305[] = {{&S71731[74], 6}, {NULL}};
const Segment SGI183306[] = {{&S60414[34], 9}, {NULL}};
const Segment SGI183307[] = {{&S60414[27], 6}, {NULL}};
const Segment SGI183308[] = {{&S71732[34], 10}, {NULL}};
const Segment SGI183309[] = {{&S60414[96], 9}, {NULL}};
const Segment SGI183310[] = {{&S71732[27], 6}, {NULL}};
const Segment SGI183311[] = {{&S60414[69], 6}, {NULL}};
const Segment SGI183312[] = {{&S71732[97], 9}, {NULL}};
const Segment SGI183313[] = {{&S71732[70], 6}, {NULL}};
const Segment SGI183314[] = {{&S60415[34], 10}, {NULL}};
const Segment SGI183315[] = {{&S60415[27], 6}, {NULL}};
const Segment SGI183316[] = {{&S60415[97], 9}, {NULL}};
const Segment SGI183317[] = {{&S60415[70], 6}, {NULL}};
const Segment SGI183318[] = {{&S60427[34], 18}, {NULL}};
const Segment SGI183319[] = {{&S60427[27], 6}, {NULL}};
const Segment SGI183320[] = {{&S62416[32], 20}, {NULL}};
const Segment SGI183321[] = {{&S62416[27], 4}, {NULL}};
const Segment SGI183322[] = {{&S60427[105], 9}, {NULL}};
const Segment SGI183323[] = {{&S60427[78], 6}, {NULL}};
const Segment SGI183324[] = {{&S62416[105], 9}, {NULL}};
const Segment SGI183325[] = {{&S62416[78], 6}, {NULL}};
const Segment SGI183326[] = {{&S71749[34], 10}, {NULL}};
const Segment SGI183327[] = {{&S71749[27], 6}, {NULL}};
const Segment SGI183328[] = {{&S71749[97], 9}, {NULL}};
const Segment SGI183329[] = {{&S71749[70], 6}, {NULL}};
const Segment SGI183330[] = {{&S60428[34], 10}, {NULL}};
const Segment SGI183331[] = {{&S60428[27], 6}, {NULL}};
const Segment SGI183332[] = {{&S60428[97], 9}, {NULL}};
const Segment SGI183333[] = {{&S60428[70], 6}, {NULL}};
const Segment SGI183334[] = {{&S64869[44], 7}, {NULL}};
const Segment SGI183335[] = {{&S60429[34], 18}, {NULL}};
const Segment SGI183336[] = {{&S64869[44], 19}, {NULL}};
const Segment SGI183337[] = {{&S64869[30], 6}, {NULL}};
const Segment SGI183338[] = {{&S60429[27], 6}, {NULL}};
const Segment SGI183339[] = {{&S64869[112], 3}, {NULL}};
const Segment SGI183340[] = {{&S60429[105], 9}, {NULL}};
const Segment SGI183341[] = {{&S64869[77], 6}, {NULL}};
const Segment SGI183342[] = {{&S60429[78], 6}, {NULL}};
const Segment SGI183343[] = {{&S60430[34], 10}, {NULL}};
const Segment SGI183344[] = {{&S60430[27], 6}, {NULL}};
const Segment SGI183345[] = {{&S60430[97], 9}, {NULL}};
const Segment SGI183346[] = {{&S60430[70], 6}, {NULL}};
const Segment SGI183347[] = {{&S71754[44], 5}, {NULL}};
const Segment SGI183348[] = {{&S71754[44], 17}, {NULL}};
const Segment SGI183349[] = {{&S71754[30], 6}, {NULL}};
const Segment SGI183350[] = {{&S64872[38], 7}, {NULL}};
const Segment SGI183351[] = {{&S64872[31], 6}, {NULL}};
const Segment SGI183352[] = {{&S60431[34], 11}, {NULL}};
const Segment SGI183353[] = {{&S71754[31], 3}, {NULL}};
const Segment SGI183354[] = {{&S60431[27], 6}, {NULL}};
const Segment SGI183355[] = {{&S71754[75], 6}, {NULL}};
const Segment SGI183356[] = {{&S60431[98], 9}, {NULL}};
const Segment SGI183357[] = {{&S60431[71], 6}, {NULL}};
const Segment SGI183358[] = {{&S64875[44], 15}, {NULL}};
const Segment SGI183359[] = {{&S64875[44], 27}, {NULL}};
const Segment SGI183360[] = {{&S64875[30], 6}, {NULL}};
const Segment SGI183361[] = {{&S64875[120], 3}, {NULL}};
const Segment SGI183362[] = {{&S64875[85], 6}, {NULL}};
const Segment SGI183363[] = {{&S71758[44], 8}, {NULL}};
const Segment SGI183364[] = {{&S71758[44], 20}, {NULL}};
const Segment SGI183365[] = {{&S71758[30], 6}, {NULL}};
const Segment SGI183366[] = {{&S71758[31], 3}, {NULL}};
const Segment SGI183367[] = {{&S71758[78], 6}, {NULL}};
const Segment SGI183368[] = {{&S71769[59], 10}, {NULL}};
const Segment SGI183369[] = {{&S71769[59], 22}, {NULL}};
const Segment SGI183370[] = {{&S71769[45], 6}, {NULL}};
const Segment SGI183371[] = {{&S71769[46], 3}, {NULL}};
const Segment SGI183372[] = {{&S71769[95], 6}, {NULL}};
const Segment SGI183373[] = {{&S71769[0], 14}, {NULL}};
const Segment SGI183374[] = {{&S60448[34], 9}, {NULL}};
const Segment SGI183375[] = {{&S60448[27], 6}, {NULL}};
const Segment SGI183376[] = {{&S60448[96], 9}, {NULL}};
const Segment SGI183377[] = {{&S60448[69], 6}, {NULL}};
const Segment SGI183378[] = {{&S71770[59], 5}, {NULL}};
const Segment SGI183379[] = {{&S71770[59], 17}, {NULL}};
const Segment SGI183380[] = {{&S71770[45], 6}, {NULL}};
const Segment SGI183381[] = {{&S71770[46], 3}, {NULL}};
const Segment SGI183382[] = {{&S71770[90], 6}, {NULL}};
const Segment SGI183383[] = {{&S71770[0], 14}, {NULL}};
const Segment SGI183384[] = {{&S60449[34], 10}, {NULL}};
const Segment SGI183385[] = {{&S60449[27], 6}, {NULL}};
const Segment SGI183386[] = {{&S60449[97], 9}, {NULL}};
const Segment SGI183387[] = {{&S60449[70], 6}, {NULL}};
const Segment SGI183388[] = {{&S71771[59], 5}, {NULL}};
const Segment SGI183389[] = {{&S71771[59], 17}, {NULL}};
const Segment SGI183390[] = {{&S71771[45], 6}, {NULL}};
const Segment SGI183391[] = {{&S71771[46], 3}, {NULL}};
const Segment SGI183392[] = {{&S71771[90], 6}, {NULL}};
const Segment SGI183393[] = {{&S60450[34], 10}, {NULL}};
const Segment SGI183394[] = {{&S60450[27], 6}, {NULL}};
const Segment SGI183395[] = {{&S71771[0], 14}, {NULL}};
const Segment SGI183396[] = {{&S60450[97], 9}, {NULL}};
const Segment SGI183397[] = {{&S60450[70], 6}, {NULL}};
const Segment SGI183398[] = {{&S71772[59], 13}, {NULL}};
const Segment SGI183399[] = {{&S71772[59], 25}, {NULL}};
const Segment SGI183400[] = {{&S71772[45], 6}, {NULL}};
const Segment SGI183401[] = {{&S60451[34], 10}, {NULL}};
const Segment SGI183402[] = {{&S71772[46], 3}, {NULL}};
const Segment SGI183403[] = {{&S60451[27], 6}, {NULL}};
const Segment SGI183404[] = {{&S71772[98], 6}, {NULL}};
const Segment SGI183405[] = {{&S71772[0], 14}, {NULL}};
const Segment SGI183406[] = {{&S60451[97], 9}, {NULL}};
const Segment SGI183407[] = {{&S60451[70], 6}, {NULL}};
const Segment SGI183408[] = {{&S64897[44], 7}, {NULL}};
const Segment SGI183409[] = {{&S64897[44], 19}, {NULL}};
const Segment SGI183410[] = {{&S64897[30], 6}, {NULL}};
const Segment SGI183411[] = {{&S64897[128], 3}, {NULL}};
const Segment SGI183412[] = {{&S49600[34], 10}, {NULL}};
const Segment SGI183413[] = {{&S64897[93], 6}, {NULL}};
const Segment SGI183414[] = {{&S49600[27], 6}, {NULL}};
const Segment SGI183415[] = {{&S60452[44], 10}, {NULL}};
const Segment SGI183416[] = {{&S64897[65], 8}, {NULL}};
const Segment SGI183417[] = {{&S60452[44], 21}, {NULL}};
const Segment SGI183418[] = {{&S60452[30], 6}, {NULL}};
const Segment SGI183419[] = {{&S49600[97], 9}, {NULL}};
const Segment SGI183420[] = {{&S49600[70], 6}, {NULL}};
const Segment SGI183421[] = {{&S60452[114], 3}, {NULL}};
const Segment SGI183422[] = {{&S60452[79], 6}, {NULL}};
const Segment SGI183423[] = {{&S64898[44], 7}, {NULL}};
const Segment SGI183424[] = {{&S64898[44], 19}, {NULL}};
const Segment SGI183425[] = {{&S64898[30], 6}, {NULL}};
const Segment SGI183426[] = {{&S64898[128], 3}, {NULL}};
const Segment SGI183427[] = {{&S49601[34], 11}, {NULL}};
const Segment SGI183428[] = {{&S64898[93], 6}, {NULL}};
const Segment SGI183429[] = {{&S60453[44], 10}, {NULL}};
const Segment SGI183430[] = {{&S49601[27], 6}, {NULL}};
const Segment SGI183431[] = {{&S60453[44], 21}, {NULL}};
const Segment SGI183432[] = {{&S60453[30], 6}, {NULL}};
const Segment SGI183433[] = {{&S64898[65], 8}, {NULL}};
const Segment SGI183434[] = {{&S58695[34], 8}, {NULL}};
const Segment SGI183435[] = {{&S49601[98], 9}, {NULL}};
const Segment SGI183436[] = {{&S60453[114], 3}, {NULL}};
const Segment SGI183437[] = {{&S58695[27], 6}, {NULL}};
const Segment SGI183438[] = {{&S49601[71], 6}, {NULL}};
const Segment SGI183439[] = {{&S60453[79], 6}, {NULL}};
const Segment SGI183440[] = {{&S58695[95], 9}, {NULL}};
const Segment SGI183441[] = {{&S58695[68], 6}, {NULL}};
const Segment SGI183442[] = {{&S60454[38], 10}, {NULL}};
const Segment SGI183443[] = {{&S60454[31], 6}, {NULL}};
const Segment SGI183444[] = {{&S49603[34], 11}, {NULL}};
const Segment SGI183445[] = {{&S49603[27], 6}, {NULL}};
const Segment SGI183446[] = {{&S49603[97], 9}, {NULL}};
const Segment SGI183447[] = {{&S49603[70], 6}, {NULL}};
const Segment SGI183448[] = {{&S60455[44], 10}, {NULL}};
const Segment SGI183449[] = {{&S60455[44], 21}, {NULL}};
const Segment SGI183450[] = {{&S60455[30], 6}, {NULL}};
const Segment SGI183451[] = {{&S60455[114], 3}, {NULL}};
const Segment SGI183452[] = {{&S60455[79], 6}, {NULL}};
const Segment SGI183453[] = {{&S60456[44], 10}, {NULL}};
const Segment SGI183454[] = {{&S60456[44], 21}, {NULL}};
const Segment SGI183455[] = {{&S60456[30], 6}, {NULL}};
const Segment SGI183456[] = {{&S60456[114], 3}, {NULL}};
const Segment SGI183457[] = {{&S60456[79], 6}, {NULL}};
const Segment SGI183458[] = {{&S70618[34], 5}, {NULL}};
const Segment SGI183459[] = {{&S70618[27], 6}, {NULL}};
const Segment SGI183460[] = {{&S70618[65], 6}, {NULL}};
const Segment SGI183461[] = {{&S58721[44], 10}, {NULL}};
const Segment SGI183462[] = {{&S58721[44], 22}, {NULL}};
const Segment SGI183463[] = {{&S58721[30], 6}, {NULL}};
const Segment SGI183464[] = {{&S58721[115], 3}, {NULL}};
const Segment SGI183465[] = {{&S58721[80], 6}, {NULL}};
const Segment SGI183466[] = {{&S58722[38], 10}, {NULL}};
const Segment SGI183467[] = {{&S58722[31], 6}, {NULL}};
const Segment SGI183468[] = {{&S58724[65], 10}, {NULL}};
const Segment SGI183469[] = {{&S58724[65], 22}, {NULL}};
const Segment SGI183470[] = {{&S58724[51], 6}, {NULL}};
const Segment SGI183471[] = {{&S60492[34], 8}, {NULL}};
const Segment SGI183472[] = {{&S60492[27], 6}, {NULL}};
const Segment SGI183473[] = {{&S60492[95], 9}, {NULL}};
const Segment SGI183474[] = {{&S60492[68], 6}, {NULL}};
const Segment SGI183475[] = {{&S60521[44], 10}, {NULL}};
const Segment SGI183476[] = {{&S60521[44], 22}, {NULL}};
const Segment SGI183477[] = {{&S60521[30], 6}, {NULL}};
const Segment SGI183478[] = {{&S60521[115], 3}, {NULL}};
const Segment SGI183479[] = {{&S60521[80], 6}, {NULL}};
const Segment SGI183480[] = {{&S60522[38], 10}, {NULL}};
const Segment SGI183481[] = {{&S60522[31], 6}, {NULL}};
const Segment SGI183482[] = {{&S60533[56], 6}, {NULL}};
const Segment SGI183483[] = {{&S60533[118], 6}, {NULL}};
const Segment SGI183484[] = {{&S60534[44], 18}, {NULL}};
const Segment SGI183485[] = {{&S60534[44], 39}, {NULL}};
const Segment SGI183486[] = {{&S60534[30], 6}, {NULL}};
const Segment SGI183487[] = {{&S60534[132], 3}, {NULL}};
const Segment SGI183488[] = {{&S60534[97], 6}, {NULL}};
const Segment SGI183489[] = {{&S60535[44], 18}, {NULL}};
const Segment SGI183490[] = {{&S60535[44], 39}, {NULL}};
const Segment SGI183491[] = {{&S60535[30], 6}, {NULL}};
const Segment SGI183492[] = {{&S60535[132], 3}, {NULL}};
const Segment SGI183493[] = {{&S60535[97], 6}, {NULL}};
const Segment SGI183494[] = {{&S49694[34], 10}, {NULL}};
const Segment SGI183495[] = {{&S49694[27], 6}, {NULL}};
const Segment SGI183496[] = {{&S60536[38], 18}, {NULL}};
const Segment SGI183497[] = {{&S60536[31], 6}, {NULL}};
const Segment SGI183498[] = {{&S49694[97], 9}, {NULL}};
const Segment SGI183499[] = {{&S49694[70], 6}, {NULL}};
const Segment SGI183500[] = {{&S60537[38], 18}, {NULL}};
const Segment SGI183501[] = {{&S60537[31], 6}, {NULL}};
const Segment SGI183502[] = {{&S49696[34], 11}, {NULL}};
const Segment SGI183503[] = {{&S49696[27], 6}, {NULL}};
const Segment SGI183504[] = {{&S49696[98], 9}, {NULL}};
const Segment SGI183505[] = {{&S49696[71], 6}, {NULL}};
const Segment SGI183506[] = {{&S60538[44], 18}, {NULL}};
const Segment SGI183507[] = {{&S60538[44], 39}, {NULL}};
const Segment SGI183508[] = {{&S60538[30], 6}, {NULL}};
const Segment SGI183509[] = {{&S60538[100], 6}, {NULL}};
const Segment SGI183510[] = {{&S49697[34], 10}, {NULL}};
const Segment SGI183511[] = {{&S49697[27], 6}, {NULL}};
const Segment SGI183512[] = {{&S49697[97], 9}, {NULL}};
const Segment SGI183513[] = {{&S49697[70], 6}, {NULL}};
const Segment SGI183514[] = {{&S60539[44], 18}, {NULL}};
const Segment SGI183515[] = {{&S60539[44], 39}, {NULL}};
const Segment SGI183516[] = {{&S60539[30], 6}, {NULL}};
const Segment SGI183517[] = {{&S60539[100], 6}, {NULL}};
const Segment SGI183518[] = {{&S65009[34], 14}, {NULL}};
const Segment SGI183519[] = {{&S65009[27], 6}, {NULL}};
const Segment SGI183520[] = {{&S65009[101], 9}, {NULL}};
const Segment SGI183521[] = {{&S65009[74], 6}, {NULL}};
const Segment SGI183522[] = {{&S49706[44], 11}, {NULL}};
const Segment SGI183523[] = {{&S49706[44], 22}, {NULL}};
const Segment SGI183524[] = {{&S49706[30], 6}, {NULL}};
const Segment SGI183525[] = {{&S49706[133], 3}, {NULL}};
const Segment SGI183526[] = {{&S49706[98], 6}, {NULL}};
const Segment SGI183527[] = {{&S49706[68], 8}, {NULL}};
const Segment SGI183528[] = {{&S49707[44], 11}, {NULL}};
const Segment SGI183529[] = {{&S49707[44], 23}, {NULL}};
const Segment SGI183530[] = {{&S49707[30], 6}, {NULL}};
const Segment SGI183531[] = {{&S49707[134], 3}, {NULL}};
const Segment SGI183532[] = {{&S49707[99], 6}, {NULL}};
const Segment SGI183533[] = {{&S49707[69], 8}, {NULL}};
const Segment SGI183534[] = {{&S65024[44], 15}, {NULL}};
const Segment SGI183535[] = {{&S65024[44], 27}, {NULL}};
const Segment SGI183536[] = {{&S65024[30], 6}, {NULL}};
const Segment SGI183537[] = {{&S65024[120], 3}, {NULL}};
const Segment SGI183538[] = {{&S65024[85], 6}, {NULL}};
const Segment SGI183539[] = {{&S65025[44], 17}, {NULL}};
const Segment SGI183540[] = {{&S65025[44], 29}, {NULL}};
const Segment SGI183541[] = {{&S65025[30], 6}, {NULL}};
const Segment SGI183542[] = {{&S65025[122], 3}, {NULL}};
const Segment SGI183543[] = {{&S65025[87], 6}, {NULL}};
const Segment SGI183544[] = {{&S71773[59], 13}, {NULL}};
const Segment SGI183545[] = {{&S71773[59], 25}, {NULL}};
const Segment SGI183546[] = {{&S71773[45], 6}, {NULL}};
const Segment SGI183547[] = {{&S71773[46], 3}, {NULL}};
const Segment SGI183548[] = {{&S71773[98], 6}, {NULL}};
const Segment SGI183549[] = {{&S71773[0], 14}, {NULL}};
const Segment SGI183550[] = {{&S71775[34], 5}, {NULL}};
const Segment SGI183551[] = {{&S71775[27], 6}, {NULL}};
const Segment SGI183552[] = {{&S71775[92], 9}, {NULL}};
const Segment SGI183553[] = {{&S71775[65], 6}, {NULL}};
const Segment SGI183554[] = {{&S65034[34], 7}, {NULL}};
const Segment SGI183555[] = {{&S65034[27], 6}, {NULL}};
const Segment SGI183556[] = {{&S65034[99], 9}, {NULL}};
const Segment SGI183557[] = {{&S65034[72], 6}, {NULL}};
const Segment SGI183558[] = {{&S71780[34], 14}, {NULL}};
const Segment SGI183559[] = {{&S71780[27], 6}, {NULL}};
const Segment SGI183560[] = {{&S71780[101], 9}, {NULL}};
const Segment SGI183561[] = {{&S71780[74], 6}, {NULL}};
const Segment SGI183562[] = {{&S71804[44], 7}, {NULL}};
const Segment SGI183563[] = {{&S71804[44], 21}, {NULL}};
const Segment SGI183564[] = {{&S71804[30], 6}, {NULL}};
const Segment SGI183565[] = {{&S71804[31], 3}, {NULL}};
const Segment SGI183566[] = {{&S71804[79], 6}, {NULL}};
const Segment SGI183567[] = {{&S71805[44], 7}, {NULL}};
const Segment SGI183568[] = {{&S71805[44], 21}, {NULL}};
const Segment SGI183569[] = {{&S71805[30], 6}, {NULL}};
const Segment SGI183570[] = {{&S71805[31], 3}, {NULL}};
const Segment SGI183571[] = {{&S71805[79], 6}, {NULL}};
const Segment SGI183572[] = {{&S71806[44], 7}, {NULL}};
const Segment SGI183573[] = {{&S71806[44], 20}, {NULL}};
const Segment SGI183574[] = {{&S71806[30], 6}, {NULL}};
const Segment SGI183575[] = {{&S71806[31], 3}, {NULL}};
const Segment SGI183576[] = {{&S71806[78], 6}, {NULL}};
const Segment SGI183577[] = {{&S71807[44], 7}, {NULL}};
const Segment SGI183578[] = {{&S71807[44], 20}, {NULL}};
const Segment SGI183579[] = {{&S71807[30], 6}, {NULL}};
const Segment SGI183580[] = {{&S60605[44], 10}, {NULL}};
const Segment SGI183581[] = {{&S60605[44], 22}, {NULL}};
const Segment SGI183582[] = {{&S60605[30], 6}, {NULL}};
const Segment SGI183583[] = {{&S71807[31], 3}, {NULL}};
const Segment SGI183584[] = {{&S71807[78], 6}, {NULL}};
const Segment SGI183585[] = {{&S60605[115], 3}, {NULL}};
const Segment SGI183586[] = {{&S60605[80], 6}, {NULL}};
const Segment SGI183587[] = {{&S60605[140], 7}, {NULL}};
const Segment SGI183588[] = {{&S71808[44], 7}, {NULL}};
const Segment SGI183589[] = {{&S71808[44], 21}, {NULL}};
const Segment SGI183590[] = {{&S71808[30], 6}, {NULL}};
const Segment SGI183591[] = {{&S71808[31], 3}, {NULL}};
const Segment SGI183592[] = {{&S71808[79], 6}, {NULL}};
const Segment SGI183593[] = {{&S60607[44], 11}, {NULL}};
const Segment SGI183594[] = {{&S60607[44], 23}, {NULL}};
const Segment SGI183595[] = {{&S60607[30], 6}, {NULL}};
const Segment SGI183596[] = {{&S60607[116], 3}, {NULL}};
const Segment SGI183597[] = {{&S60607[81], 6}, {NULL}};
const Segment SGI183598[] = {{&S60607[141], 7}, {NULL}};
const Segment SGI183599[] = {{&S71809[44], 7}, {NULL}};
const Segment SGI183600[] = {{&S71809[44], 20}, {NULL}};
const Segment SGI183601[] = {{&S71809[30], 6}, {NULL}};
const Segment SGI183602[] = {{&S71809[31], 3}, {NULL}};
const Segment SGI183603[] = {{&S71809[78], 6}, {NULL}};
const Segment SGI183604[] = {{&S60608[44], 11}, {NULL}};
const Segment SGI183605[] = {{&S60608[44], 23}, {NULL}};
const Segment SGI183606[] = {{&S60608[30], 6}, {NULL}};
const Segment SGI183607[] = {{&S60608[116], 3}, {NULL}};
const Segment SGI183608[] = {{&S60608[81], 6}, {NULL}};
const Segment SGI183609[] = {{&S60608[141], 7}, {NULL}};
const Segment SGI183610[] = {{&S71810[44], 7}, {NULL}};
const Segment SGI183611[] = {{&S71810[44], 20}, {NULL}};
const Segment SGI183612[] = {{&S71810[30], 6}, {NULL}};
const Segment SGI183613[] = {{&S71810[31], 3}, {NULL}};
const Segment SGI183614[] = {{&S71810[78], 6}, {NULL}};
const Segment SGI183615[] = {{&S60609[44], 10}, {NULL}};
const Segment SGI183616[] = {{&S60609[44], 22}, {NULL}};
const Segment SGI183617[] = {{&S60609[30], 6}, {NULL}};
const Segment SGI183618[] = {{&S60609[115], 3}, {NULL}};
const Segment SGI183619[] = {{&S60609[80], 6}, {NULL}};
const Segment SGI183620[] = {{&S60609[140], 7}, {NULL}};
const Segment SGI183621[] = {{&S71811[44], 7}, {NULL}};
const Segment SGI183622[] = {{&S71811[44], 21}, {NULL}};
const Segment SGI183623[] = {{&S71811[30], 6}, {NULL}};
const Segment SGI183624[] = {{&S71811[31], 3}, {NULL}};
const Segment SGI183625[] = {{&S71811[79], 6}, {NULL}};
const Segment SGI183626[] = {{&S60610[44], 11}, {NULL}};
const Segment SGI183627[] = {{&S60610[44], 23}, {NULL}};
const Segment SGI183628[] = {{&S60610[30], 6}, {NULL}};
const Segment SGI183629[] = {{&S60610[116], 3}, {NULL}};
const Segment SGI183630[] = {{&S60610[81], 6}, {NULL}};
const Segment SGI183631[] = {{&S60610[141], 7}, {NULL}};
const Segment SGI183632[] = {{&S60611[44], 18}, {NULL}};
const Segment SGI183633[] = {{&S60611[44], 30}, {NULL}};
const Segment SGI183634[] = {{&S60611[30], 6}, {NULL}};
const Segment SGI183635[] = {{&S60611[123], 3}, {NULL}};
const Segment SGI183636[] = {{&S60611[88], 6}, {NULL}};
const Segment SGI183637[] = {{&S60611[148], 7}, {NULL}};
const Segment SGI183638[] = {{&S71814[34], 9}, {NULL}};
const Segment SGI183639[] = {{&S71814[27], 6}, {NULL}};
const Segment SGI183640[] = {{&S71814[103], 9}, {NULL}};
const Segment SGI183641[] = {{&S71814[76], 6}, {NULL}};
const Segment SGI183642[] = {{&S60612[34], 18}, {NULL}};
const Segment SGI183643[] = {{&S60612[27], 6}, {NULL}};
const Segment SGI183644[] = {{&S60612[105], 9}, {NULL}};
const Segment SGI183645[] = {{&S60612[78], 6}, {NULL}};
const Segment SGI183646[] = {{&S60613[34], 11}, {NULL}};
const Segment SGI183647[] = {{&S60613[27], 6}, {NULL}};
const Segment SGI183648[] = {{&S60613[98], 9}, {NULL}};
const Segment SGI183649[] = {{&S60613[71], 6}, {NULL}};
const Segment SGI183650[] = {{&S60616[34], 7}, {NULL}};
const Segment SGI183651[] = {{&S60616[27], 6}, {NULL}};
const Segment SGI183652[] = {{&S60616[110], 9}, {NULL}};
const Segment SGI183653[] = {{&S60616[83], 6}, {NULL}};
const Segment SGI183654[] = {{&S60617[34], 7}, {NULL}};
const Segment SGI183655[] = {{&S60617[27], 6}, {NULL}};
const Segment SGI183656[] = {{&S60617[110], 9}, {NULL}};
const Segment SGI183657[] = {{&S60617[83], 6}, {NULL}};
const Segment SGI183658[] = {{&S60648[34], 9}, {NULL}};
const Segment SGI183659[] = {{&S60648[27], 6}, {NULL}};
const Segment SGI183660[] = {{&S60648[96], 9}, {NULL}};
const Segment SGI183661[] = {{&S60648[69], 6}, {NULL}};
const Segment SGI183662[] = {{&S71861[44], 5}, {NULL}};
const Segment SGI183663[] = {{&S71861[44], 17}, {NULL}};
const Segment SGI183664[] = {{&S71861[30], 6}, {NULL}};
const Segment SGI183665[] = {{&S71861[31], 3}, {NULL}};
const Segment SGI183666[] = {{&S71861[75], 6}, {NULL}};
const Segment SGI183667[] = {{&S71863[44], 5}, {NULL}};
const Segment SGI183668[] = {{&S71863[44], 17}, {NULL}};
const Segment SGI183669[] = {{&S71863[30], 6}, {NULL}};
const Segment SGI183670[] = {{&S71863[31], 3}, {NULL}};
const Segment SGI183671[] = {{&S71863[75], 6}, {NULL}};
const Segment SGI183672[] = {{&S71864[44], 5}, {NULL}};
const Segment SGI183673[] = {{&S71864[44], 17}, {NULL}};
const Segment SGI183674[] = {{&S71864[30], 6}, {NULL}};
const Segment SGI183675[] = {{&S71864[31], 3}, {NULL}};
const Segment SGI183676[] = {{&S65134[44], 10}, {NULL}};
const Segment SGI183677[] = {{&S65134[44], 33}, {NULL}};
const Segment SGI183678[] = {{&S71864[75], 6}, {NULL}};
const Segment SGI183679[] = {{&S65134[30], 6}, {NULL}};
const Segment SGI183680[] = {{&S65134[126], 3}, {NULL}};
const Segment SGI183681[] = {{&S65134[91], 6}, {NULL}};
const Segment SGI183682[] = {{&S71865[44], 5}, {NULL}};
const Segment SGI183683[] = {{&S71865[44], 17}, {NULL}};
const Segment SGI183684[] = {{&S71865[30], 6}, {NULL}};
const Segment SGI183685[] = {{&S71865[31], 3}, {NULL}};
const Segment SGI183686[] = {{&S71865[75], 6}, {NULL}};
const Segment SGI183687[] = {{&S65135[44], 10}, {NULL}};
const Segment SGI183688[] = {{&S65135[44], 33}, {NULL}};
const Segment SGI183689[] = {{&S65135[30], 6}, {NULL}};
const Segment SGI183690[] = {{&S65135[126], 3}, {NULL}};
const Segment SGI183691[] = {{&S65135[91], 6}, {NULL}};
const Segment SGI183692[] = {{&S65136[38], 10}, {NULL}};
const Segment SGI183693[] = {{&S65136[31], 6}, {NULL}};
const Segment SGI183694[] = {{&S71873[34], 12}, {NULL}};
const Segment SGI183695[] = {{&S71873[27], 6}, {NULL}};
const Segment SGI183696[] = {{&S71873[99], 9}, {NULL}};
const Segment SGI183697[] = {{&S71873[72], 6}, {NULL}};
const Segment SGI183698[] = {{&S71874[34], 10}, {NULL}};
const Segment SGI183699[] = {{&S71874[27], 6}, {NULL}};
const Segment SGI183700[] = {{&S71874[97], 9}, {NULL}};
const Segment SGI183701[] = {{&S71874[70], 6}, {NULL}};
const Segment SGI183702[] = {{&S65149[44], 11}, {NULL}};
const Segment SGI183703[] = {{&S65149[44], 23}, {NULL}};
const Segment SGI183704[] = {{&S65149[30], 6}, {NULL}};
const Segment SGI183705[] = {{&S65149[116], 3}, {NULL}};
const Segment SGI183706[] = {{&S65149[81], 6}, {NULL}};
const Segment SGI183707[] = {{&S65150[44], 4}, {NULL}};
const Segment SGI183708[] = {{&S65150[44], 41}, {NULL}};
const Segment SGI183709[] = {{&S65150[30], 6}, {NULL}};
const Segment SGI183710[] = {{&S65150[134], 3}, {NULL}};
const Segment SGI183711[] = {{&S65150[99], 6}, {NULL}};
const Segment SGI183712[] = {{&S49851[34], 10}, {NULL}};
const Segment SGI183713[] = {{&S65151[44], 4}, {NULL}};
const Segment SGI183714[] = {{&S65151[44], 16}, {NULL}};
const Segment SGI183715[] = {{&S49851[27], 6}, {NULL}};
const Segment SGI183716[] = {{&S65151[30], 6}, {NULL}};
const Segment SGI183717[] = {{&S65151[109], 3}, {NULL}};
const Segment SGI183718[] = {{&S49851[96], 9}, {NULL}};
const Segment SGI183719[] = {{&S65151[74], 6}, {NULL}};
const Segment SGI183720[] = {{&S49851[69], 6}, {NULL}};
const Segment SGI183721[] = {{&S65152[44], 10}, {NULL}};
const Segment SGI183722[] = {{&S65152[44], 22}, {NULL}};
const Segment SGI183723[] = {{&S65152[30], 6}, {NULL}};
const Segment SGI183724[] = {{&S65152[115], 3}, {NULL}};
const Segment SGI183725[] = {{&S65152[80], 6}, {NULL}};
const Segment SGI183726[] = {{&S49854[34], 7}, {NULL}};
const Segment SGI183727[] = {{&S49854[27], 6}, {NULL}};
const Segment SGI183728[] = {{&S49854[94], 9}, {NULL}};
const Segment SGI183729[] = {{&S49854[67], 6}, {NULL}};
const Segment SGI183730[] = {{&S71888[34], 8}, {NULL}};
const Segment SGI183731[] = {{&S71888[27], 6}, {NULL}};
const Segment SGI183732[] = {{&S71888[107], 9}, {NULL}};
const Segment SGI183733[] = {{&S71888[80], 6}, {NULL}};
const Segment SGI183734[] = {{&S71889[34], 7}, {NULL}};
const Segment SGI183735[] = {{&S71889[27], 6}, {NULL}};
const Segment SGI183736[] = {{&S65156[34], 16}, {NULL}};
const Segment SGI183737[] = {{&S71889[112], 9}, {NULL}};
const Segment SGI183738[] = {{&S65156[27], 6}, {NULL}};
const Segment SGI183739[] = {{&S71889[85], 6}, {NULL}};
const Segment SGI183740[] = {{&S65156[112], 9}, {NULL}};
const Segment SGI183741[] = {{&S65156[85], 6}, {NULL}};
const Segment SGI183742[] = {{&S65157[34], 18}, {NULL}};
const Segment SGI183743[] = {{&S65157[27], 6}, {NULL}};
const Segment SGI183744[] = {{&S60700[34], 10}, {NULL}};
const Segment SGI183745[] = {{&S60700[27], 6}, {NULL}};
const Segment SGI183746[] = {{&S65157[114], 9}, {NULL}};
const Segment SGI183747[] = {{&S65157[87], 6}, {NULL}};
const Segment SGI183748[] = {{&S60700[97], 9}, {NULL}};
const Segment SGI183749[] = {{&S60700[70], 6}, {NULL}};
const Segment SGI183750[] = {{&S65158[34], 18}, {NULL}};
const Segment SGI183751[] = {{&S65158[27], 6}, {NULL}};
const Segment SGI183752[] = {{&S65158[114], 9}, {NULL}};
const Segment SGI183753[] = {{&S65158[87], 6}, {NULL}};
const Segment SGI183754[] = {{&S65159[34], 18}, {NULL}};
const Segment SGI183755[] = {{&S65159[27], 6}, {NULL}};
const Segment SGI183756[] = {{&S65159[114], 9}, {NULL}};
const Segment SGI183757[] = {{&S65159[87], 6}, {NULL}};
const Segment SGI183758[] = {{&S65160[34], 18}, {NULL}};
const Segment SGI183759[] = {{&S65160[27], 6}, {NULL}};
const Segment SGI183760[] = {{&S65160[114], 9}, {NULL}};
const Segment SGI183761[] = {{&S65160[87], 6}, {NULL}};
const Segment SGI183762[] = {{&S71902[34], 9}, {NULL}};
const Segment SGI183763[] = {{&S71902[27], 6}, {NULL}};
const Segment SGI183764[] = {{&S71902[96], 9}, {NULL}};
const Segment SGI183765[] = {{&S71902[69], 6}, {NULL}};
const Segment SGI183766[] = {{&S60716[34], 10}, {NULL}};
const Segment SGI183767[] = {{&S60716[27], 6}, {NULL}};
const Segment SGI183768[] = {{&S60716[97], 9}, {NULL}};
const Segment SGI183769[] = {{&S60716[70], 6}, {NULL}};
const Segment SGI183770[] = {{&S60716[138], 8}, {NULL}};
const Segment SGI183771[] = {{&S60717[34], 10}, {NULL}};
const Segment SGI183772[] = {{&S60717[27], 6}, {NULL}};
const Segment SGI183773[] = {{&S60717[97], 9}, {NULL}};
const Segment SGI183774[] = {{&S60717[70], 6}, {NULL}};
const Segment SGI183775[] = {{&S60717[138], 8}, {NULL}};
const Segment SGI183776[] = {{&S60719[34], 18}, {NULL}};
const Segment SGI183777[] = {{&S60719[27], 6}, {NULL}};
const Segment SGI183778[] = {{&S60719[105], 9}, {NULL}};
const Segment SGI183779[] = {{&S60719[78], 6}, {NULL}};
const Segment SGI183780[] = {{&S60719[146], 8}, {NULL}};
const Segment SGI183781[] = {{&S60720[34], 11}, {NULL}};
const Segment SGI183782[] = {{&S60720[27], 6}, {NULL}};
const Segment SGI183783[] = {{&S60720[98], 9}, {NULL}};
const Segment SGI183784[] = {{&S60720[71], 6}, {NULL}};
const Segment SGI183785[] = {{&S60720[139], 8}, {NULL}};
const Segment SGI183786[] = {{&S60721[34], 10}, {NULL}};
const Segment SGI183787[] = {{&S60721[27], 6}, {NULL}};
const Segment SGI183788[] = {{&S60721[97], 9}, {NULL}};
const Segment SGI183789[] = {{&S60721[70], 6}, {NULL}};
const Segment SGI183790[] = {{&S60721[138], 8}, {NULL}};
const Segment SGI183791[] = {{&S60722[34], 10}, {NULL}};
const Segment SGI183792[] = {{&S60722[27], 6}, {NULL}};
const Segment SGI183793[] = {{&S60722[97], 9}, {NULL}};
const Segment SGI183794[] = {{&S60722[70], 6}, {NULL}};
const Segment SGI183795[] = {{&S71920[34], 10}, {NULL}};
const Segment SGI183796[] = {{&S60722[138], 8}, {NULL}};
const Segment SGI183797[] = {{&S71920[27], 6}, {NULL}};
const Segment SGI183798[] = {{&S71920[97], 9}, {NULL}};
const Segment SGI183799[] = {{&S71920[70], 6}, {NULL}};
const Segment SGI183800[] = {{&S71921[44], 11}, {NULL}};
const Segment SGI183801[] = {{&S71921[44], 23}, {NULL}};
const Segment SGI183802[] = {{&S71921[30], 6}, {NULL}};
const Segment SGI183803[] = {{&S71921[31], 3}, {NULL}};
const Segment SGI183804[] = {{&S71921[81], 6}, {NULL}};
const Segment SGI183805[] = {{&S71922[34], 10}, {NULL}};
const Segment SGI183806[] = {{&S71922[27], 6}, {NULL}};
const Segment SGI183807[] = {{&S71922[97], 9}, {NULL}};
const Segment SGI183808[] = {{&S71922[70], 6}, {NULL}};
const Segment SGI183809[] = {{&S71923[34], 16}, {NULL}};
const Segment SGI183810[] = {{&S71923[27], 6}, {NULL}};
const Segment SGI183811[] = {{&S71923[110], 9}, {NULL}};
const Segment SGI183812[] = {{&S71923[83], 6}, {NULL}};
const Segment SGI183813[] = {{&S71924[34], 17}, {NULL}};
const Segment SGI183814[] = {{&S71924[27], 6}, {NULL}};
const Segment SGI183815[] = {{&S71924[112], 9}, {NULL}};
const Segment SGI183816[] = {{&S71924[85], 6}, {NULL}};
const Segment SGI183817[] = {{&S71925[34], 18}, {NULL}};
const Segment SGI183818[] = {{&S71925[27], 6}, {NULL}};
const Segment SGI183819[] = {{&S71925[114], 9}, {NULL}};
const Segment SGI183820[] = {{&S71925[87], 6}, {NULL}};
const Segment SGI183821[] = {{&S71926[34], 15}, {NULL}};
const Segment SGI183822[] = {{&S71926[27], 6}, {NULL}};
const Segment SGI183823[] = {{&S71926[108], 9}, {NULL}};
const Segment SGI183824[] = {{&S71926[81], 6}, {NULL}};
const Segment SGI183825[] = {{&S71927[34], 17}, {NULL}};
const Segment SGI183826[] = {{&S71927[27], 6}, {NULL}};
const Segment SGI183827[] = {{&S71927[112], 9}, {NULL}};
const Segment SGI183828[] = {{&S71927[85], 6}, {NULL}};
const Segment SGI183829[] = {{&S60769[34], 10}, {NULL}};
const Segment SGI183830[] = {{&S60769[27], 6}, {NULL}};
const Segment SGI183831[] = {{&S60769[97], 9}, {NULL}};
const Segment SGI183832[] = {{&S60769[70], 6}, {NULL}};
const Segment SGI183833[] = {{&S71956[34], 11}, {NULL}};
const Segment SGI183834[] = {{&S71956[27], 6}, {NULL}};
const Segment SGI183835[] = {{&S71956[98], 9}, {NULL}};
const Segment SGI183836[] = {{&S71956[71], 6}, {NULL}};
const Segment SGI183837[] = {{&S60782[34], 10}, {NULL}};
const Segment SGI183838[] = {{&S60782[27], 6}, {NULL}};
const Segment SGI183839[] = {{&S60782[97], 9}, {NULL}};
const Segment SGI183840[] = {{&S60782[70], 6}, {NULL}};
const Segment SGI183841[] = {{&S71965[34], 10}, {NULL}};
const Segment SGI183842[] = {{&S71965[27], 6}, {NULL}};
const Segment SGI183843[] = {{&S71965[97], 9}, {NULL}};
const Segment SGI183844[] = {{&S71965[70], 6}, {NULL}};
const Segment SGI183845[] = {{&S71990[34], 14}, {NULL}};
const Segment SGI183846[] = {{&S71990[27], 6}, {NULL}};
const Segment SGI183847[] = {{&S71990[101], 9}, {NULL}};
const Segment SGI183848[] = {{&S71990[74], 6}, {NULL}};
const Segment SGI183849[] = {{&S71991[34], 16}, {NULL}};
const Segment SGI183850[] = {{&S71991[27], 6}, {NULL}};
const Segment SGI183851[] = {{&S71991[103], 9}, {NULL}};
const Segment SGI183852[] = {{&S71991[76], 6}, {NULL}};
const Segment SGI183853[] = {{&S71992[34], 14}, {NULL}};
const Segment SGI183854[] = {{&S71992[27], 6}, {NULL}};
const Segment SGI183855[] = {{&S71992[101], 9}, {NULL}};
const Segment SGI183856[] = {{&S71992[74], 6}, {NULL}};
const Segment SGI183857[] = {{&S71993[34], 15}, {NULL}};
const Segment SGI183858[] = {{&S71993[27], 6}, {NULL}};
const Segment SGI183859[] = {{&S71993[102], 9}, {NULL}};
const Segment SGI183860[] = {{&S71993[75], 6}, {NULL}};
const Segment SGI183861[] = {{&S71994[34], 15}, {NULL}};
const Segment SGI183862[] = {{&S71994[27], 6}, {NULL}};
const Segment SGI183863[] = {{&S60819[44], 10}, {NULL}};
const Segment SGI183864[] = {{&S60819[44], 22}, {NULL}};
const Segment SGI183865[] = {{&S71994[102], 9}, {NULL}};
const Segment SGI183866[] = {{&S60819[30], 6}, {NULL}};
const Segment SGI183867[] = {{&S71994[75], 6}, {NULL}};
const Segment SGI183868[] = {{&S60819[80], 6}, {NULL}};
const Segment SGI183869[] = {{&S60819[107], 13}, {NULL}};
const Segment SGI183870[] = {{&S71995[34], 7}, {NULL}};
const Segment SGI183871[] = {{&S71995[27], 6}, {NULL}};
const Segment SGI183872[] = {{&S71995[111], 9}, {NULL}};
const Segment SGI183873[] = {{&S71995[84], 6}, {NULL}};
const Segment SGI183874[] = {{&S60821[44], 10}, {NULL}};
const Segment SGI183875[] = {{&S60821[44], 22}, {NULL}};
const Segment SGI183876[] = {{&S60821[30], 6}, {NULL}};
const Segment SGI183877[] = {{&S60821[80], 6}, {NULL}};
const Segment SGI183878[] = {{&S60821[107], 13}, {NULL}};
const Segment SGI183879[] = {{&S71996[34], 8}, {NULL}};
const Segment SGI183880[] = {{&S71996[27], 6}, {NULL}};
const Segment SGI183881[] = {{&S71996[107], 9}, {NULL}};
const Segment SGI183882[] = {{&S71996[80], 6}, {NULL}};
const Segment SGI183883[] = {{&S60822[44], 10}, {NULL}};
const Segment SGI183884[] = {{&S60822[44], 22}, {NULL}};
const Segment SGI183885[] = {{&S60822[30], 6}, {NULL}};
const Segment SGI183886[] = {{&S60822[80], 6}, {NULL}};
const Segment SGI183887[] = {{&S60822[107], 13}, {NULL}};
const Segment SGI183888[] = {{&S59101[34], 9}, {NULL}};
const Segment SGI183889[] = {{&S59101[27], 6}, {NULL}};
const Segment SGI183890[] = {{&S60823[44], 10}, {NULL}};
const Segment SGI183891[] = {{&S60823[44], 22}, {NULL}};
const Segment SGI183892[] = {{&S60823[30], 6}, {NULL}};
const Segment SGI183893[] = {{&S59101[96], 9}, {NULL}};
const Segment SGI183894[] = {{&S59101[69], 6}, {NULL}};
const Segment SGI183895[] = {{&S60823[80], 6}, {NULL}};
const Segment SGI183896[] = {{&S60823[107], 13}, {NULL}};
const Segment SGI183897[] = {{&S60824[44], 11}, {NULL}};
const Segment SGI183898[] = {{&S60824[44], 23}, {NULL}};
const Segment SGI183899[] = {{&S60824[30], 6}, {NULL}};
const Segment SGI183900[] = {{&S60824[81], 6}, {NULL}};
const Segment SGI183901[] = {{&S60824[108], 13}, {NULL}};
const Segment SGI183902[] = {{&S72014[44], 9}, {NULL}};
const Segment SGI183903[] = {{&S72014[44], 21}, {NULL}};
const Segment SGI183904[] = {{&S72014[30], 6}, {NULL}};
const Segment SGI183905[] = {{&S72014[31], 3}, {NULL}};
const Segment SGI183906[] = {{&S72014[79], 6}, {NULL}};
const Segment SGI183907[] = {{&S72015[44], 16}, {NULL}};
const Segment SGI183908[] = {{&S72015[44], 28}, {NULL}};
const Segment SGI183909[] = {{&S72015[30], 6}, {NULL}};
const Segment SGI183910[] = {{&S72015[31], 3}, {NULL}};
const Segment SGI183911[] = {{&S72015[86], 6}, {NULL}};
const Segment SGI183912[] = {{&S60840[44], 12}, {NULL}};
const Segment SGI183913[] = {{&S60840[44], 24}, {NULL}};
const Segment SGI183914[] = {{&S60840[30], 6}, {NULL}};
const Segment SGI183915[] = {{&S60840[137], 3}, {NULL}};
const Segment SGI183916[] = {{&S60840[102], 6}, {NULL}};
const Segment SGI183917[] = {{&S60840[70], 8}, {NULL}};
const Segment SGI183918[] = {{&S60841[44], 12}, {NULL}};
const Segment SGI183919[] = {{&S60841[44], 22}, {NULL}};
const Segment SGI183920[] = {{&S60841[30], 6}, {NULL}};
const Segment SGI183921[] = {{&S60841[135], 3}, {NULL}};
const Segment SGI183922[] = {{&S60841[100], 6}, {NULL}};
const Segment SGI183923[] = {{&S60841[68], 8}, {NULL}};
const Segment SGI183924[] = {{&S60852[34], 8}, {NULL}};
const Segment SGI183925[] = {{&S60852[27], 6}, {NULL}};
const Segment SGI183926[] = {{&S60852[100], 9}, {NULL}};
const Segment SGI183927[] = {{&S60852[73], 6}, {NULL}};
const Segment SGI183928[] = {{&S60853[34], 8}, {NULL}};
const Segment SGI183929[] = {{&S60853[27], 6}, {NULL}};
const Segment SGI183930[] = {{&S60853[100], 9}, {NULL}};
const Segment SGI183931[] = {{&S60853[73], 6}, {NULL}};
const Segment SGI183932[] = {{&S60855[34], 12}, {NULL}};
const Segment SGI183933[] = {{&S60855[27], 6}, {NULL}};
const Segment SGI183934[] = {{&S60855[99], 9}, {NULL}};
const Segment SGI183935[] = {{&S60855[72], 6}, {NULL}};
const Segment SGI183936[] = {{&S72038[34], 10}, {NULL}};
const Segment SGI183937[] = {{&S72038[27], 6}, {NULL}};
const Segment SGI183938[] = {{&S72038[97], 9}, {NULL}};
const Segment SGI183939[] = {{&S72038[70], 6}, {NULL}};
const Segment SGI183940[] = {{&S72039[34], 11}, {NULL}};
const Segment SGI183941[] = {{&S72039[27], 6}, {NULL}};
const Segment SGI183942[] = {{&S72039[98], 9}, {NULL}};
const Segment SGI183943[] = {{&S72039[71], 6}, {NULL}};
const Segment SGI183944[] = {{&S72041[34], 18}, {NULL}};
const Segment SGI183945[] = {{&S72041[27], 6}, {NULL}};
const Segment SGI183946[] = {{&S72041[105], 9}, {NULL}};
const Segment SGI183947[] = {{&S72041[78], 6}, {NULL}};
const Segment SGI183948[] = {{&S72044[34], 9}, {NULL}};
const Segment SGI183949[] = {{&S72044[27], 6}, {NULL}};
const Segment SGI183950[] = {{&S72044[96], 9}, {NULL}};
const Segment SGI183951[] = {{&S72044[69], 6}, {NULL}};
const Segment SGI183952[] = {{&S59154[34], 9}, {NULL}};
const Segment SGI183953[] = {{&S59154[27], 6}, {NULL}};
const Segment SGI183954[] = {{&S59154[96], 9}, {NULL}};
const Segment SGI183955[] = {{&S59154[69], 6}, {NULL}};
const Segment SGI183956[] = {{&S72058[44], 14}, {NULL}};
const Segment SGI183957[] = {{&S72058[44], 42}, {NULL}};
const Segment SGI183958[] = {{&S72058[30], 6}, {NULL}};
const Segment SGI183959[] = {{&S72058[31], 3}, {NULL}};
const Segment SGI183960[] = {{&S72058[100], 6}, {NULL}};
const Segment SGI183961[] = {{&S72059[44], 14}, {NULL}};
const Segment SGI183962[] = {{&S72059[44], 26}, {NULL}};
const Segment SGI183963[] = {{&S72059[30], 6}, {NULL}};
const Segment SGI183964[] = {{&S72059[31], 3}, {NULL}};
const Segment SGI183965[] = {{&S72059[84], 6}, {NULL}};
const Segment SGI183966[] = {{&S72060[44], 14}, {NULL}};
const Segment SGI183967[] = {{&S72060[44], 26}, {NULL}};
const Segment SGI183968[] = {{&S72060[30], 6}, {NULL}};
const Segment SGI183969[] = {{&S60890[34], 7}, {NULL}};
const Segment SGI183970[] = {{&S72060[31], 3}, {NULL}};
const Segment SGI183971[] = {{&S60890[27], 6}, {NULL}};
const Segment SGI183972[] = {{&S72060[84], 6}, {NULL}};
const Segment SGI183973[] = {{&S60890[98], 9}, {NULL}};
const Segment SGI183974[] = {{&S60890[71], 6}, {NULL}};
const Segment SGI183975[] = {{&S72061[44], 17}, {NULL}};
const Segment SGI183976[] = {{&S72061[44], 45}, {NULL}};
const Segment SGI183977[] = {{&S72061[30], 6}, {NULL}};
const Segment SGI183978[] = {{&S60891[34], 6}, {NULL}};
const Segment SGI183979[] = {{&S60891[27], 6}, {NULL}};
const Segment SGI183980[] = {{&S72061[31], 3}, {NULL}};
const Segment SGI183981[] = {{&S72061[103], 6}, {NULL}};
const Segment SGI183982[] = {{&S60891[93], 9}, {NULL}};
const Segment SGI183983[] = {{&S60891[66], 6}, {NULL}};
const Segment SGI183984[] = {{&S72062[44], 7}, {NULL}};
const Segment SGI183985[] = {{&S72062[44], 34}, {NULL}};
const Segment SGI183986[] = {{&S72062[30], 6}, {NULL}};
const Segment SGI183987[] = {{&S72062[31], 3}, {NULL}};
const Segment SGI183988[] = {{&S72062[92], 6}, {NULL}};
const Segment SGI183989[] = {{&S72063[44], 7}, {NULL}};
const Segment SGI183990[] = {{&S72063[44], 34}, {NULL}};
const Segment SGI183991[] = {{&S72063[30], 6}, {NULL}};
const Segment SGI183992[] = {{&S72063[95], 6}, {NULL}};
const Segment SGI183993[] = {{&S72064[44], 7}, {NULL}};
const Segment SGI183994[] = {{&S72064[44], 33}, {NULL}};
const Segment SGI183995[] = {{&S72064[30], 6}, {NULL}};
const Segment SGI183996[] = {{&S72064[31], 3}, {NULL}};
const Segment SGI183997[] = {{&S72064[91], 6}, {NULL}};
const Segment SGI183998[] = {{&S60898[34], 10}, {NULL}};
const Segment SGI183999[] = {{&S60898[27], 6}, {NULL}};
const Segment SGI184000[] = {{&S60898[97], 9}, {NULL}};
const Segment SGI184001[] = {{&S60898[70], 6}, {NULL}};
const Segment SGI184002[] = {{&S72065[44], 7}, {NULL}};
const Segment SGI184003[] = {{&S72065[44], 33}, {NULL}};
const Segment SGI184004[] = {{&S72065[30], 6}, {NULL}};
const Segment SGI184005[] = {{&S72065[31], 3}, {NULL}};
const Segment SGI184006[] = {{&S60899[34], 11}, {NULL}};
const Segment SGI184007[] = {{&S72065[91], 6}, {NULL}};
const Segment SGI184008[] = {{&S60899[27], 6}, {NULL}};
const Segment SGI184009[] = {{&S60899[98], 9}, {NULL}};
const Segment SGI184010[] = {{&S60899[71], 6}, {NULL}};
const Segment SGI184011[] = {{&S72066[44], 7}, {NULL}};
const Segment SGI184012[] = {{&S72066[44], 33}, {NULL}};
const Segment SGI184013[] = {{&S72066[30], 6}, {NULL}};
const Segment SGI184014[] = {{&S72066[94], 6}, {NULL}};
const Segment SGI184015[] = {{&S72067[34], 5}, {NULL}};
const Segment SGI184016[] = {{&S72067[27], 6}, {NULL}};
const Segment SGI184017[] = {{&S72067[92], 9}, {NULL}};
const Segment SGI184018[] = {{&S72067[65], 6}, {NULL}};
const Segment SGI184019[] = {{&S72068[34], 5}, {NULL}};
const Segment SGI184020[] = {{&S72068[27], 6}, {NULL}};
const Segment SGI184021[] = {{&S72068[92], 9}, {NULL}};
const Segment SGI184022[] = {{&S72068[65], 6}, {NULL}};
const Segment SGI184023[] = {{&S72071[44], 10}, {NULL}};
const Segment SGI184024[] = {{&S72071[44], 22}, {NULL}};
const Segment SGI184025[] = {{&S72071[30], 6}, {NULL}};
const Segment SGI184026[] = {{&S72071[31], 3}, {NULL}};
const Segment SGI184027[] = {{&S72071[80], 6}, {NULL}};
const Segment SGI184028[] = {{&S72072[34], 4}, {NULL}};
const Segment SGI184029[] = {{&S72072[27], 6}, {NULL}};
const Segment SGI184030[] = {{&S72072[91], 9}, {NULL}};
const Segment SGI184031[] = {{&S72072[64], 6}, {NULL}};
const Segment SGI184032[] = {{&S50100[41], 18}, {NULL}};
const Segment SGI184033[] = {{&S50100[41], 29}, {NULL}};
const Segment SGI184034[] = {{&S50100[27], 6}, {NULL}};
const Segment SGI184035[] = {{&S50100[119], 3}, {NULL}};
const Segment SGI184036[] = {{&S50100[123], 9}, {NULL}};
const Segment SGI184037[] = {{&S50100[84], 6}, {NULL}};
const Segment SGI184038[] = {{&S72079[34], 7}, {NULL}};
const Segment SGI184039[] = {{&S72079[27], 6}, {NULL}};
const Segment SGI184040[] = {{&S72079[111], 9}, {NULL}};
const Segment SGI184041[] = {{&S72079[84], 6}, {NULL}};
const Segment SGI184042[] = {{&S62846[32], 10}, {NULL}};
const Segment SGI184043[] = {{&S62846[27], 4}, {NULL}};
const Segment SGI184044[] = {{&S62846[95], 9}, {NULL}};
const Segment SGI184045[] = {{&S62846[68], 6}, {NULL}};
const Segment SGI184046[] = {{&S59227[44], 8}, {NULL}};
const Segment SGI184047[] = {{&S59227[44], 20}, {NULL}};
const Segment SGI184048[] = {{&S59227[30], 6}, {NULL}};
const Segment SGI184049[] = {{&S59227[113], 3}, {NULL}};
const Segment SGI184050[] = {{&S59227[78], 6}, {NULL}};
const Segment SGI184051[] = {{&S59229[38], 8}, {NULL}};
const Segment SGI184052[] = {{&S59229[31], 6}, {NULL}};
const Segment SGI184053[] = {{&S59230[44], 9}, {NULL}};
const Segment SGI184054[] = {{&S59230[44], 21}, {NULL}};
const Segment SGI184055[] = {{&S59230[30], 6}, {NULL}};
const Segment SGI184056[] = {{&S72099[44], 16}, {NULL}};
const Segment SGI184057[] = {{&S59230[114], 3}, {NULL}};
const Segment SGI184058[] = {{&S72099[44], 36}, {NULL}};
const Segment SGI184059[] = {{&S72099[30], 6}, {NULL}};
const Segment SGI184060[] = {{&S59230[79], 6}, {NULL}};
const Segment SGI184061[] = {{&S72099[31], 3}, {NULL}};
const Segment SGI184062[] = {{&S72099[94], 6}, {NULL}};
const Segment SGI184063[] = {{&S72106[44], 5}, {NULL}};
const Segment SGI184064[] = {{&S72106[44], 26}, {NULL}};
const Segment SGI184065[] = {{&S72106[30], 6}, {NULL}};
const Segment SGI184066[] = {{&S72106[31], 3}, {NULL}};
const Segment SGI184067[] = {{&S72106[84], 6}, {NULL}};
const Segment SGI184068[] = {{&S60951[44], 10}, {NULL}};
const Segment SGI184069[] = {{&S60951[44], 22}, {NULL}};
const Segment SGI184070[] = {{&S60951[30], 6}, {NULL}};
const Segment SGI184071[] = {{&S60951[115], 3}, {NULL}};
const Segment SGI184072[] = {{&S60951[80], 6}, {NULL}};
const Segment SGI184073[] = {{&S60952[38], 10}, {NULL}};
const Segment SGI184074[] = {{&S60952[31], 6}, {NULL}};
const Segment SGI184075[] = {{&S60953[44], 10}, {NULL}};
const Segment SGI184076[] = {{&S60953[44], 22}, {NULL}};
const Segment SGI184077[] = {{&S60953[30], 6}, {NULL}};
const Segment SGI184078[] = {{&S60953[115], 3}, {NULL}};
const Segment SGI184079[] = {{&S60953[80], 6}, {NULL}};
const Segment SGI184080[] = {{&S60983[34], 10}, {NULL}};
const Segment SGI184081[] = {{&S60983[27], 6}, {NULL}};
const Segment SGI184082[] = {{&S60983[97], 9}, {NULL}};
const Segment SGI184083[] = {{&S60983[70], 6}, {NULL}};
const Segment SGI184084[] = {{&S60984[34], 11}, {NULL}};
const Segment SGI184085[] = {{&S60984[27], 6}, {NULL}};
const Segment SGI184086[] = {{&S72144[34], 10}, {NULL}};
const Segment SGI184087[] = {{&S60984[98], 9}, {NULL}};
const Segment SGI184088[] = {{&S60984[71], 6}, {NULL}};
const Segment SGI184089[] = {{&S72144[27], 6}, {NULL}};
const Segment SGI184090[] = {{&S72144[97], 9}, {NULL}};
const Segment SGI184091[] = {{&S72144[70], 6}, {NULL}};
const Segment SGI184092[] = {{&S72144[138], 8}, {NULL}};
const Segment SGI184093[] = {{&S72146[34], 11}, {NULL}};
const Segment SGI184094[] = {{&S72146[27], 6}, {NULL}};
const Segment SGI184095[] = {{&S72146[98], 9}, {NULL}};
const Segment SGI184096[] = {{&S72146[71], 6}, {NULL}};
const Segment SGI184097[] = {{&S72146[139], 8}, {NULL}};
const Segment SGI184098[] = {{&S60991[34], 10}, {NULL}};
const Segment SGI184099[] = {{&S60991[27], 6}, {NULL}};
const Segment SGI184100[] = {{&S60991[97], 9}, {NULL}};
const Segment SGI184101[] = {{&S60991[70], 6}, {NULL}};
const Segment SGI184102[] = {{&S60991[131], 8}, {NULL}};
const Segment SGI184103[] = {{&S60993[34], 10}, {NULL}};
const Segment SGI184104[] = {{&S60993[27], 6}, {NULL}};
const Segment SGI184105[] = {{&S60993[97], 9}, {NULL}};
const Segment SGI184106[] = {{&S60993[70], 6}, {NULL}};
const Segment SGI184107[] = {{&S60993[131], 8}, {NULL}};
const Segment SGI184108[] = {{&S69333[41], 11}, {NULL}};
const Segment SGI184109[] = {{&S69333[41], 23}, {NULL}};
const Segment SGI184110[] = {{&S69333[30], 4}, {NULL}};
const Segment SGI184111[] = {{&S69333[113], 3}, {NULL}};
const Segment SGI184112[] = {{&S69333[78], 6}, {NULL}};
const Segment SGI184113[] = {{&S72161[34], 7}, {NULL}};
const Segment SGI184114[] = {{&S72161[27], 6}, {NULL}};
const Segment SGI184115[] = {{&S72161[100], 9}, {NULL}};
const Segment SGI184116[] = {{&S72161[73], 6}, {NULL}};
const Segment SGI184117[] = {{&S72161[141], 8}, {NULL}};
const Segment SGI184118[] = {{&S72164[34], 11}, {NULL}};
const Segment SGI184119[] = {{&S72164[27], 6}, {NULL}};
const Segment SGI184120[] = {{&S72164[98], 9}, {NULL}};
const Segment SGI184121[] = {{&S72164[71], 6}, {NULL}};
const Segment SGI184122[] = {{&S72165[34], 10}, {NULL}};
const Segment SGI184123[] = {{&S72165[27], 6}, {NULL}};
const Segment SGI184124[] = {{&S72165[97], 9}, {NULL}};
const Segment SGI184125[] = {{&S72165[70], 6}, {NULL}};
const Segment SGI184126[] = {{&S72166[27], 6}, {NULL}};
const Segment SGI184127[] = {{&S72166[99], 9}, {NULL}};
const Segment SGI184128[] = {{&S72166[72], 6}, {NULL}};
const Segment SGI184129[] = {{&S65214[34], 28}, {NULL}};
const Segment SGI184130[] = {{&S72168[34], 13}, {NULL}};
const Segment SGI184131[] = {{&S72168[27], 6}, {NULL}};
const Segment SGI184132[] = {{&S65214[27], 6}, {NULL}};
const Segment SGI184133[] = {{&S72168[100], 9}, {NULL}};
const Segment SGI184134[] = {{&S72168[73], 6}, {NULL}};
const Segment SGI184135[] = {{&S65214[115], 9}, {NULL}};
const Segment SGI184136[] = {{&S65214[88], 6}, {NULL}};
const Segment SGI184137[] = {{&S61012[34], 7}, {NULL}};
const Segment SGI184138[] = {{&S61012[27], 6}, {NULL}};
const Segment SGI184139[] = {{&S61012[99], 9}, {NULL}};
const Segment SGI184140[] = {{&S61012[72], 6}, {NULL}};
const Segment SGI184141[] = {{&S72169[34], 9}, {NULL}};
const Segment SGI184142[] = {{&S72169[27], 6}, {NULL}};
const Segment SGI184143[] = {{&S72169[96], 9}, {NULL}};
const Segment SGI184144[] = {{&S72169[69], 6}, {NULL}};
const Segment SGI184145[] = {{&S61013[34], 7}, {NULL}};
const Segment SGI184146[] = {{&S61013[27], 6}, {NULL}};
const Segment SGI184147[] = {{&S61013[99], 9}, {NULL}};
const Segment SGI184148[] = {{&S61013[72], 6}, {NULL}};
const Segment SGI184149[] = {{&S72170[34], 7}, {NULL}};
const Segment SGI184150[] = {{&S72170[27], 6}, {NULL}};
const Segment SGI184151[] = {{&S72170[104], 9}, {NULL}};
const Segment SGI184152[] = {{&S72170[77], 6}, {NULL}};
const Segment SGI184153[] = {{&S61025[34], 11}, {NULL}};
const Segment SGI184154[] = {{&S61025[27], 6}, {NULL}};
const Segment SGI184155[] = {{&S61025[98], 9}, {NULL}};
const Segment SGI184156[] = {{&S61025[71], 6}, {NULL}};
const Segment SGI184157[] = {{&S61026[34], 11}, {NULL}};
const Segment SGI184158[] = {{&S72183[34], 15}, {NULL}};
const Segment SGI184159[] = {{&S61026[27], 6}, {NULL}};
const Segment SGI184160[] = {{&S72183[27], 6}, {NULL}};
const Segment SGI184161[] = {{&S61026[98], 9}, {NULL}};
const Segment SGI184162[] = {{&S61026[71], 6}, {NULL}};
const Segment SGI184163[] = {{&S72183[102], 9}, {NULL}};
const Segment SGI184164[] = {{&S72183[75], 6}, {NULL}};
const Segment SGI184165[] = {{&S72185[34], 16}, {NULL}};
const Segment SGI184166[] = {{&S72185[27], 6}, {NULL}};
const Segment SGI184167[] = {{&S72185[116], 9}, {NULL}};
const Segment SGI184168[] = {{&S72185[89], 6}, {NULL}};
const Segment SGI184169[] = {{&S72186[34], 19}, {NULL}};
const Segment SGI184170[] = {{&S72186[27], 6}, {NULL}};
const Segment SGI184171[] = {{&S72186[119], 9}, {NULL}};
const Segment SGI184172[] = {{&S72186[92], 6}, {NULL}};
const Segment SGI184173[] = {{&S72196[34], 18}, {NULL}};
const Segment SGI184174[] = {{&S72196[27], 6}, {NULL}};
const Segment SGI184175[] = {{&S72196[105], 9}, {NULL}};
const Segment SGI184176[] = {{&S72196[78], 6}, {NULL}};
const Segment SGI184177[] = {{&S72197[34], 11}, {NULL}};
const Segment SGI184178[] = {{&S72197[27], 6}, {NULL}};
const Segment SGI184179[] = {{&S72197[98], 9}, {NULL}};
const Segment SGI184180[] = {{&S72197[71], 6}, {NULL}};
const Segment SGI184181[] = {{&S72198[34], 11}, {NULL}};
const Segment SGI184182[] = {{&S72198[27], 6}, {NULL}};
const Segment SGI184183[] = {{&S72198[98], 9}, {NULL}};
const Segment SGI184184[] = {{&S72198[71], 6}, {NULL}};
const Segment SGI184185[] = {{&S72199[34], 11}, {NULL}};
const Segment SGI184186[] = {{&S72199[27], 6}, {NULL}};
const Segment SGI184187[] = {{&S72199[98], 9}, {NULL}};
const Segment SGI184188[] = {{&S72199[71], 6}, {NULL}};
const Segment SGI184189[] = {{&S71255[44], 5}, {NULL}};
const Segment SGI184190[] = {{&S71255[44], 17}, {NULL}};
const Segment SGI184191[] = {{&S71255[30], 6}, {NULL}};
const Segment SGI184192[] = {{&S71255[31], 3}, {NULL}};
const Segment SGI184193[] = {{&S71255[75], 6}, {NULL}};
const Segment SGI184194[] = {{&S61069[34], 10}, {NULL}};
const Segment SGI184195[] = {{&S61069[27], 6}, {NULL}};
const Segment SGI184196[] = {{&S61069[97], 9}, {NULL}};
const Segment SGI184197[] = {{&S61069[70], 6}, {NULL}};
const Segment SGI184198[] = {{&S61070[34], 10}, {NULL}};
const Segment SGI184199[] = {{&S61070[27], 6}, {NULL}};
const Segment SGI184200[] = {{&S61070[97], 9}, {NULL}};
const Segment SGI184201[] = {{&S61070[70], 6}, {NULL}};
const Segment SGI184202[] = {{&S61071[34], 9}, {NULL}};
const Segment SGI184203[] = {{&S61071[27], 6}, {NULL}};
const Segment SGI184204[] = {{&S61071[96], 9}, {NULL}};
const Segment SGI184205[] = {{&S61071[69], 6}, {NULL}};
const Segment SGI184206[] = {{&S61072[34], 7}, {NULL}};
const Segment SGI184207[] = {{&S61072[27], 6}, {NULL}};
const Segment SGI184208[] = {{&S61072[100], 9}, {NULL}};
const Segment SGI184209[] = {{&S61072[73], 6}, {NULL}};
const Segment SGI184210[] = {{&S72238[34], 18}, {NULL}};
const Segment SGI184211[] = {{&S72238[27], 6}, {NULL}};
const Segment SGI184212[] = {{&S72238[105], 9}, {NULL}};
const Segment SGI184213[] = {{&S72238[78], 6}, {NULL}};
const Segment SGI184214[] = {{&S61082[34], 11}, {NULL}};
const Segment SGI184215[] = {{&S61082[27], 6}, {NULL}};
const Segment SGI184216[] = {{&S72239[34], 11}, {NULL}};
const Segment SGI184217[] = {{&S61082[98], 9}, {NULL}};
const Segment SGI184218[] = {{&S72239[27], 6}, {NULL}};
const Segment SGI184219[] = {{&S61082[71], 6}, {NULL}};
const Segment SGI184220[] = {{&S72239[98], 9}, {NULL}};
const Segment SGI184221[] = {{&S72239[71], 6}, {NULL}};
const Segment SGI184222[] = {{&S61083[34], 11}, {NULL}};
const Segment SGI184223[] = {{&S61083[27], 6}, {NULL}};
const Segment SGI184224[] = {{&S72240[34], 11}, {NULL}};
const Segment SGI184225[] = {{&S72240[27], 6}, {NULL}};
const Segment SGI184226[] = {{&S61083[98], 9}, {NULL}};
const Segment SGI184227[] = {{&S61083[71], 6}, {NULL}};
const Segment SGI184228[] = {{&S72240[98], 9}, {NULL}};
const Segment SGI184229[] = {{&S72240[71], 6}, {NULL}};
const Segment SGI184230[] = {{&S59402[44], 10}, {NULL}};
const Segment SGI184231[] = {{&S59402[44], 22}, {NULL}};
const Segment SGI184232[] = {{&S59402[30], 6}, {NULL}};
const Segment SGI184233[] = {{&S61084[34], 11}, {NULL}};
const Segment SGI184234[] = {{&S61084[27], 6}, {NULL}};
const Segment SGI184235[] = {{&S59402[115], 3}, {NULL}};
const Segment SGI184236[] = {{&S72241[34], 11}, {NULL}};
const Segment SGI184237[] = {{&S59402[80], 6}, {NULL}};
const Segment SGI184238[] = {{&S72241[27], 6}, {NULL}};
const Segment SGI184239[] = {{&S61084[98], 9}, {NULL}};
const Segment SGI184240[] = {{&S61084[71], 6}, {NULL}};
const Segment SGI184241[] = {{&S72241[98], 9}, {NULL}};
const Segment SGI184242[] = {{&S72241[71], 6}, {NULL}};
const Segment SGI184243[] = {{&S59403[38], 10}, {NULL}};
const Segment SGI184244[] = {{&S59403[31], 6}, {NULL}};
const Segment SGI184245[] = {{&S72242[34], 18}, {NULL}};
const Segment SGI184246[] = {{&S72242[27], 6}, {NULL}};
const Segment SGI184247[] = {{&S72242[105], 9}, {NULL}};
const Segment SGI184248[] = {{&S72242[78], 6}, {NULL}};
const Segment SGI184249[] = {{&S61091[34], 12}, {NULL}};
const Segment SGI184250[] = {{&S61091[27], 6}, {NULL}};
const Segment SGI184251[] = {{&S65305[44], 7}, {NULL}};
const Segment SGI184252[] = {{&S65305[44], 19}, {NULL}};
const Segment SGI184253[] = {{&S65305[30], 6}, {NULL}};
const Segment SGI184254[] = {{&S61091[99], 9}, {NULL}};
const Segment SGI184255[] = {{&S61091[72], 6}, {NULL}};
const Segment SGI184256[] = {{&S65305[112], 3}, {NULL}};
const Segment SGI184257[] = {{&S65305[77], 6}, {NULL}};
const Segment SGI184258[] = {{&S72261[34], 7}, {NULL}};
const Segment SGI184259[] = {{&S72261[27], 6}, {NULL}};
const Segment SGI184260[] = {{&S72261[94], 9}, {NULL}};
const Segment SGI184261[] = {{&S72261[67], 6}, {NULL}};
const Segment SGI184262[] = {{&S61107[34], 7}, {NULL}};
const Segment SGI184263[] = {{&S61107[27], 6}, {NULL}};
const Segment SGI184264[] = {{&S61107[94], 9}, {NULL}};
const Segment SGI184265[] = {{&S61107[67], 6}, {NULL}};
const Segment SGI184266[] = {{&S72286[44], 6}, {NULL}};
const Segment SGI184267[] = {{&S72286[44], 18}, {NULL}};
const Segment SGI184268[] = {{&S72286[30], 6}, {NULL}};
const Segment SGI184269[] = {{&S72286[31], 3}, {NULL}};
const Segment SGI184270[] = {{&S72286[76], 6}, {NULL}};
const Segment SGI184271[] = {{&S72290[34], 15}, {NULL}};
const Segment SGI184272[] = {{&S72290[27], 6}, {NULL}};
const Segment SGI184273[] = {{&S72290[102], 9}, {NULL}};
const Segment SGI184274[] = {{&S72290[75], 6}, {NULL}};
const Segment SGI184275[] = {{&S71348[44], 7}, {NULL}};
const Segment SGI184276[] = {{&S71348[44], 19}, {NULL}};
const Segment SGI184277[] = {{&S71348[30], 6}, {NULL}};
const Segment SGI184278[] = {{&S71348[31], 3}, {NULL}};
const Segment SGI184279[] = {{&S71348[77], 6}, {NULL}};
const Segment SGI184280[] = {{&S61149[34], 8}, {NULL}};
const Segment SGI184281[] = {{&S61149[27], 6}, {NULL}};
const Segment SGI184282[] = {{&S61149[95], 9}, {NULL}};
const Segment SGI184283[] = {{&S61149[68], 6}, {NULL}};
const Segment SGI184284[] = {{&S72306[44], 12}, {NULL}};
const Segment SGI184285[] = {{&S72306[44], 24}, {NULL}};
const Segment SGI184286[] = {{&S72306[30], 6}, {NULL}};
const Segment SGI184287[] = {{&S72306[31], 3}, {NULL}};
const Segment SGI184288[] = {{&S72306[82], 6}, {NULL}};
const Segment SGI184289[] = {{&S72308[34], 7}, {NULL}};
const Segment SGI184290[] = {{&S72308[27], 6}, {NULL}};
const Segment SGI184291[] = {{&S72308[94], 9}, {NULL}};
const Segment SGI184292[] = {{&S61153[34], 10}, {NULL}};
const Segment SGI184293[] = {{&S72308[67], 6}, {NULL}};
const Segment SGI184294[] = {{&S61153[27], 6}, {NULL}};
const Segment SGI184295[] = {{&S61153[97], 9}, {NULL}};
const Segment SGI184296[] = {{&S61153[70], 6}, {NULL}};
const Segment SGI184297[] = {{&S72309[34], 20}, {NULL}};
const Segment SGI184298[] = {{&S72309[27], 6}, {NULL}};
const Segment SGI184299[] = {{&S72309[107], 9}, {NULL}};
const Segment SGI184300[] = {{&S72309[80], 6}, {NULL}};
const Segment SGI184301[] = {{&S72312[44], 6}, {NULL}};
const Segment SGI184302[] = {{&S72312[44], 18}, {NULL}};
const Segment SGI184303[] = {{&S72312[30], 6}, {NULL}};
const Segment SGI184304[] = {{&S72312[31], 3}, {NULL}};
const Segment SGI184305[] = {{&S72312[76], 6}, {NULL}};
const Segment SGI184306[] = {{&S72313[34], 9}, {NULL}};
const Segment SGI184307[] = {{&S72313[27], 6}, {NULL}};
const Segment SGI184308[] = {{&S72313[96], 9}, {NULL}};
const Segment SGI184309[] = {{&S72313[69], 6}, {NULL}};
const Segment SGI184310[] = {{&S72314[34], 10}, {NULL}};
const Segment SGI184311[] = {{&S72314[27], 6}, {NULL}};
const Segment SGI184312[] = {{&S72314[97], 9}, {NULL}};
const Segment SGI184313[] = {{&S72314[70], 6}, {NULL}};
const Segment SGI184314[] = {{&S61164[34], 7}, {NULL}};
const Segment SGI184315[] = {{&S61164[27], 6}, {NULL}};
const Segment SGI184316[] = {{&S61164[99], 9}, {NULL}};
const Segment SGI184317[] = {{&S61164[72], 6}, {NULL}};
const Segment SGI184318[] = {{&S61165[34], 7}, {NULL}};
const Segment SGI184319[] = {{&S61165[27], 6}, {NULL}};
const Segment SGI184320[] = {{&S61165[99], 9}, {NULL}};
const Segment SGI184321[] = {{&S61165[72], 6}, {NULL}};
const Segment SGI184322[] = {{&S72323[34], 7}, {NULL}};
const Segment SGI184323[] = {{&S72323[27], 6}, {NULL}};
const Segment SGI184324[] = {{&S72323[99], 9}, {NULL}};
const Segment SGI184325[] = {{&S72323[72], 6}, {NULL}};
const Segment SGI184326[] = {{&S72324[44], 14}, {NULL}};
const Segment SGI184327[] = {{&S72324[44], 26}, {NULL}};
const Segment SGI184328[] = {{&S72324[30], 6}, {NULL}};
const Segment SGI184329[] = {{&S72324[31], 3}, {NULL}};
const Segment SGI184330[] = {{&S61173[34], 9}, {NULL}};
const Segment SGI184331[] = {{&S72324[84], 6}, {NULL}};
const Segment SGI184332[] = {{&S61173[27], 6}, {NULL}};
const Segment SGI184333[] = {{&S61173[96], 9}, {NULL}};
const Segment SGI184334[] = {{&S61173[69], 6}, {NULL}};
const Segment SGI184335[] = {{&S61174[34], 9}, {NULL}};
const Segment SGI184336[] = {{&S61174[27], 6}, {NULL}};
const Segment SGI184337[] = {{&S61174[96], 9}, {NULL}};
const Segment SGI184338[] = {{&S61174[69], 6}, {NULL}};
const Segment SGI184339[] = {{&S72327[34], 9}, {NULL}};
const Segment SGI184340[] = {{&S72327[27], 6}, {NULL}};
const Segment SGI184341[] = {{&S72327[102], 9}, {NULL}};
const Segment SGI184342[] = {{&S72327[75], 6}, {NULL}};
const Segment SGI184343[] = {{&S72329[34], 12}, {NULL}};
const Segment SGI184344[] = {{&S72329[27], 6}, {NULL}};
const Segment SGI184345[] = {{&S72329[109], 9}, {NULL}};
const Segment SGI184346[] = {{&S72329[82], 6}, {NULL}};
const Segment SGI184347[] = {{&S72332[34], 8}, {NULL}};
const Segment SGI184348[] = {{&S72332[27], 6}, {NULL}};
const Segment SGI184349[] = {{&S72332[95], 9}, {NULL}};
const Segment SGI184350[] = {{&S72332[68], 6}, {NULL}};
const Segment SGI184351[] = {{&S72335[34], 7}, {NULL}};
const Segment SGI184352[] = {{&S72335[27], 6}, {NULL}};
const Segment SGI184353[] = {{&S72335[99], 9}, {NULL}};
const Segment SGI184354[] = {{&S72335[72], 6}, {NULL}};
const Segment SGI184355[] = {{&S72337[44], 15}, {NULL}};
const Segment SGI184356[] = {{&S72337[44], 27}, {NULL}};
const Segment SGI184357[] = {{&S72337[30], 6}, {NULL}};
const Segment SGI184358[] = {{&S72337[85], 6}, {NULL}};
const Segment SGI184359[] = {{&S72337[112], 13}, {NULL}};
const Segment SGI184360[] = {{&S72352[44], 14}, {NULL}};
const Segment SGI184361[] = {{&S72352[44], 26}, {NULL}};
const Segment SGI184362[] = {{&S72352[30], 6}, {NULL}};
const Segment SGI184363[] = {{&S72352[31], 3}, {NULL}};
const Segment SGI184364[] = {{&S72352[84], 6}, {NULL}};
const Segment SGI184365[] = {{&S72354[34], 13}, {NULL}};
const Segment SGI184366[] = {{&S72354[27], 6}, {NULL}};
const Segment SGI184367[] = {{&S72354[100], 9}, {NULL}};
const Segment SGI184368[] = {{&S72354[73], 6}, {NULL}};
const Segment SGI184369[] = {{&S72359[34], 10}, {NULL}};
const Segment SGI184370[] = {{&S72359[27], 6}, {NULL}};
const Segment SGI184371[] = {{&S72359[97], 9}, {NULL}};
const Segment SGI184372[] = {{&S72359[70], 6}, {NULL}};
const Segment SGI184373[] = {{&S72362[30], 6}, {NULL}};
const Segment SGI184374[] = {{&S72362[85], 9}, {NULL}};
const Segment SGI184375[] = {{&S72362[57], 6}, {NULL}};
const Segment SGI184376[] = {{&S61227[34], 10}, {NULL}};
const Segment SGI184377[] = {{&S61227[27], 6}, {NULL}};
const Segment SGI184378[] = {{&S61227[97], 9}, {NULL}};
const Segment SGI184379[] = {{&S61227[70], 6}, {NULL}};
const Segment SGI184380[] = {{&S61228[34], 10}, {NULL}};
const Segment SGI184381[] = {{&S61228[27], 6}, {NULL}};
const Segment SGI184382[] = {{&S61228[97], 9}, {NULL}};
const Segment SGI184383[] = {{&S61228[70], 6}, {NULL}};
const Segment SGI184384[] = {{&S59566[44], 10}, {NULL}};
const Segment SGI184385[] = {{&S59566[44], 22}, {NULL}};
const Segment SGI184386[] = {{&S59566[30], 6}, {NULL}};
const Segment SGI184387[] = {{&S59566[133], 3}, {NULL}};
const Segment SGI184388[] = {{&S59566[98], 6}, {NULL}};
const Segment SGI184389[] = {{&S59566[68], 8}, {NULL}};
const Segment SGI184390[] = {{&S59567[44], 11}, {NULL}};
const Segment SGI184391[] = {{&S59567[44], 23}, {NULL}};
const Segment SGI184392[] = {{&S59567[30], 6}, {NULL}};
const Segment SGI184393[] = {{&S59567[134], 3}, {NULL}};
const Segment SGI184394[] = {{&S59567[99], 6}, {NULL}};
const Segment SGI184395[] = {{&S59567[69], 8}, {NULL}};
const Segment SGI184396[] = {{&S61233[44], 10}, {NULL}};
const Segment SGI184397[] = {{&S61233[44], 22}, {NULL}};
const Segment SGI184398[] = {{&S61233[30], 6}, {NULL}};
const Segment SGI184399[] = {{&S61233[131], 3}, {NULL}};
const Segment SGI184400[] = {{&S59568[44], 10}, {NULL}};
const Segment SGI184401[] = {{&S61233[96], 6}, {NULL}};
const Segment SGI184402[] = {{&S59568[44], 22}, {NULL}};
const Segment SGI184403[] = {{&S61233[68], 8}, {NULL}};
const Segment SGI184404[] = {{&S59568[30], 6}, {NULL}};
const Segment SGI184405[] = {{&S59568[133], 3}, {NULL}};
const Segment SGI184406[] = {{&S59568[98], 6}, {NULL}};
const Segment SGI184407[] = {{&S59568[68], 8}, {NULL}};
const Segment SGI184408[] = {{&S59569[44], 10}, {NULL}};
const Segment SGI184409[] = {{&S59569[44], 22}, {NULL}};
const Segment SGI184410[] = {{&S59569[30], 6}, {NULL}};
const Segment SGI184411[] = {{&S59569[133], 3}, {NULL}};
const Segment SGI184412[] = {{&S59569[98], 6}, {NULL}};
const Segment SGI184413[] = {{&S59569[68], 8}, {NULL}};
const Segment SGI184414[] = {{&S61237[44], 9}, {NULL}};
const Segment SGI184415[] = {{&S61237[44], 21}, {NULL}};
const Segment SGI184416[] = {{&S61237[30], 6}, {NULL}};
const Segment SGI184417[] = {{&S61237[114], 3}, {NULL}};
const Segment SGI184418[] = {{&S61237[79], 6}, {NULL}};
const Segment SGI184419[] = {{&S61238[44], 9}, {NULL}};
const Segment SGI184420[] = {{&S61238[44], 21}, {NULL}};
const Segment SGI184421[] = {{&S61238[30], 6}, {NULL}};
const Segment SGI184422[] = {{&S61238[114], 3}, {NULL}};
const Segment SGI184423[] = {{&S61238[79], 6}, {NULL}};
const Segment SGI184424[] = {{&S61239[38], 9}, {NULL}};
const Segment SGI184425[] = {{&S61239[31], 6}, {NULL}};
const Segment SGI184426[] = {{&S61285[34], 6}, {NULL}};
const Segment SGI184427[] = {{&S61285[27], 6}, {NULL}};
const Segment SGI184428[] = {{&S61285[93], 9}, {NULL}};
const Segment SGI184429[] = {{&S61285[66], 6}, {NULL}};
const Segment SGI184430[] = {{&S65401[34], 6}, {NULL}};
const Segment SGI184431[] = {{&S65401[27], 6}, {NULL}};
const Segment SGI184432[] = {{&S65401[93], 9}, {NULL}};
const Segment SGI184433[] = {{&S65401[66], 6}, {NULL}};
const Segment SGI184434[] = {{&S61331[44], 8}, {NULL}};
const Segment SGI184435[] = {{&S61331[44], 19}, {NULL}};
const Segment SGI184436[] = {{&S61331[30], 6}, {NULL}};
const Segment SGI184437[] = {{&S61331[112], 3}, {NULL}};
const Segment SGI184438[] = {{&S61331[77], 6}, {NULL}};
const Segment SGI184439[] = {{&S61332[44], 8}, {NULL}};
const Segment SGI184440[] = {{&S61332[44], 19}, {NULL}};
const Segment SGI184441[] = {{&S61332[30], 6}, {NULL}};
const Segment SGI184442[] = {{&S61332[112], 3}, {NULL}};
const Segment SGI184443[] = {{&S61332[77], 6}, {NULL}};
const Segment SGI184444[] = {{&S61333[44], 8}, {NULL}};
const Segment SGI184445[] = {{&S61333[44], 19}, {NULL}};
const Segment SGI184446[] = {{&S61333[30], 6}, {NULL}};
const Segment SGI184447[] = {{&S61333[112], 3}, {NULL}};
const Segment SGI184448[] = {{&S61333[77], 6}, {NULL}};
const Segment SGI184449[] = {{&S61334[44], 8}, {NULL}};
const Segment SGI184450[] = {{&S61334[44], 19}, {NULL}};
const Segment SGI184451[] = {{&S61334[30], 6}, {NULL}};
const Segment SGI184452[] = {{&S61334[112], 3}, {NULL}};
const Segment SGI184453[] = {{&S61334[77], 6}, {NULL}};
const Segment SGI184454[] = {{&S61335[38], 8}, {NULL}};
const Segment SGI184455[] = {{&S61335[31], 6}, {NULL}};
const Segment SGI184456[] = {{&S61336[44], 18}, {NULL}};
const Segment SGI184457[] = {{&S61336[44], 30}, {NULL}};
const Segment SGI184458[] = {{&S61336[30], 6}, {NULL}};
const Segment SGI184459[] = {{&S61336[123], 3}, {NULL}};
const Segment SGI184460[] = {{&S61336[88], 6}, {NULL}};
const Segment SGI184461[] = {{&S61337[44], 18}, {NULL}};
const Segment SGI184462[] = {{&S61337[44], 30}, {NULL}};
const Segment SGI184463[] = {{&S61337[30], 6}, {NULL}};
const Segment SGI184464[] = {{&S61337[123], 3}, {NULL}};
const Segment SGI184465[] = {{&S61337[88], 6}, {NULL}};
const Segment SGI184466[] = {{&S61338[38], 18}, {NULL}};
const Segment SGI184467[] = {{&S61338[31], 6}, {NULL}};
const Segment SGI184468[] = {{&S61348[44], 11}, {NULL}};
const Segment SGI184469[] = {{&S61348[44], 29}, {NULL}};
const Segment SGI184470[] = {{&S61348[30], 6}, {NULL}};
const Segment SGI184471[] = {{&S61348[122], 3}, {NULL}};
const Segment SGI184472[] = {{&S61348[87], 6}, {NULL}};
const Segment SGI184473[] = {{&S61349[44], 7}, {NULL}};
const Segment SGI184474[] = {{&S61349[44], 25}, {NULL}};
const Segment SGI184475[] = {{&S61349[30], 6}, {NULL}};
const Segment SGI184476[] = {{&S61349[118], 3}, {NULL}};
const Segment SGI184477[] = {{&S61349[83], 6}, {NULL}};
const Segment SGI184478[] = {{&S61350[38], 7}, {NULL}};
const Segment SGI184479[] = {{&S61350[31], 6}, {NULL}};
const Segment SGI184480[] = {{&S65470[44], 4}, {NULL}};
const Segment SGI184481[] = {{&S65470[44], 16}, {NULL}};
const Segment SGI184482[] = {{&S65470[30], 6}, {NULL}};
const Segment SGI184483[] = {{&S65470[109], 3}, {NULL}};
const Segment SGI184484[] = {{&S65470[74], 6}, {NULL}};
const Segment SGI184485[] = {{&S65471[38], 4}, {NULL}};
const Segment SGI184486[] = {{&S65471[31], 6}, {NULL}};
const Segment SGI184487[] = {{&S61383[34], 18}, {NULL}};
const Segment SGI184488[] = {{&S61383[27], 6}, {NULL}};
const Segment SGI184489[] = {{&S61383[105], 9}, {NULL}};
const Segment SGI184490[] = {{&S61383[78], 6}, {NULL}};
const Segment SGI184491[] = {{&S65477[44], 10}, {NULL}};
const Segment SGI184492[] = {{&S65477[44], 22}, {NULL}};
const Segment SGI184493[] = {{&S65477[30], 6}, {NULL}};
const Segment SGI184494[] = {{&S65477[131], 3}, {NULL}};
const Segment SGI184495[] = {{&S65477[96], 6}, {NULL}};
const Segment SGI184496[] = {{&S61384[34], 11}, {NULL}};
const Segment SGI184497[] = {{&S61384[27], 6}, {NULL}};
const Segment SGI184498[] = {{&S65477[68], 8}, {NULL}};
const Segment SGI184499[] = {{&S61384[98], 9}, {NULL}};
const Segment SGI184500[] = {{&S61384[71], 6}, {NULL}};
const Segment SGI184501[] = {{&S65478[44], 10}, {NULL}};
const Segment SGI184502[] = {{&S65478[44], 22}, {NULL}};
const Segment SGI184503[] = {{&S65478[30], 6}, {NULL}};
const Segment SGI184504[] = {{&S65478[145], 3}, {NULL}};
const Segment SGI184505[] = {{&S65478[110], 6}, {NULL}};
const Segment SGI184506[] = {{&S65478[68], 8}, {NULL}};
const Segment SGI184507[] = {{&S65479[44], 10}, {NULL}};
const Segment SGI184508[] = {{&S65479[44], 22}, {NULL}};
const Segment SGI184509[] = {{&S65479[30], 6}, {NULL}};
const Segment SGI184510[] = {{&S65479[131], 3}, {NULL}};
const Segment SGI184511[] = {{&S65479[96], 6}, {NULL}};
const Segment SGI184512[] = {{&S65479[68], 8}, {NULL}};
const Segment SGI184513[] = {{&S61389[44], 10}, {NULL}};
const Segment SGI184514[] = {{&S61389[44], 22}, {NULL}};
const Segment SGI184515[] = {{&S61389[30], 6}, {NULL}};
const Segment SGI184516[] = {{&S61389[115], 3}, {NULL}};
const Segment SGI184517[] = {{&S61389[80], 6}, {NULL}};
const Segment SGI184518[] = {{&S61390[38], 10}, {NULL}};
const Segment SGI184519[] = {{&S61390[31], 6}, {NULL}};
const Segment SGI184520[] = {{&S61391[44], 10}, {NULL}};
const Segment SGI184521[] = {{&S61391[44], 22}, {NULL}};
const Segment SGI184522[] = {{&S61391[30], 6}, {NULL}};
const Segment SGI184523[] = {{&S61391[115], 3}, {NULL}};
const Segment SGI184524[] = {{&S61391[80], 6}, {NULL}};
const Segment SGI184525[] = {{&S61407[44], 11}, {NULL}};
const Segment SGI184526[] = {{&S61407[44], 28}, {NULL}};
const Segment SGI184527[] = {{&S61407[30], 6}, {NULL}};
const Segment SGI184528[] = {{&S61407[121], 3}, {NULL}};
const Segment SGI184529[] = {{&S61407[86], 6}, {NULL}};
const Segment SGI184530[] = {{&S61408[44], 7}, {NULL}};
const Segment SGI184531[] = {{&S61408[44], 24}, {NULL}};
const Segment SGI184532[] = {{&S61408[30], 6}, {NULL}};
const Segment SGI184533[] = {{&S61408[117], 3}, {NULL}};
const Segment SGI184534[] = {{&S72413[34], 10}, {NULL}};
const Segment SGI184535[] = {{&S61408[82], 6}, {NULL}};
const Segment SGI184536[] = {{&S72413[27], 6}, {NULL}};
const Segment SGI184537[] = {{&S72413[97], 9}, {NULL}};
const Segment SGI184538[] = {{&S72413[70], 6}, {NULL}};
const Segment SGI184539[] = {{&S61409[44], 11}, {NULL}};
const Segment SGI184540[] = {{&S61409[44], 28}, {NULL}};
const Segment SGI184541[] = {{&S61409[30], 6}, {NULL}};
const Segment SGI184542[] = {{&S61409[121], 3}, {NULL}};
const Segment SGI184543[] = {{&S61409[86], 6}, {NULL}};
const Segment SGI184544[] = {{&S61410[44], 7}, {NULL}};
const Segment SGI184545[] = {{&S61410[44], 24}, {NULL}};
const Segment SGI184546[] = {{&S61410[30], 6}, {NULL}};
const Segment SGI184547[] = {{&S61410[117], 3}, {NULL}};
const Segment SGI184548[] = {{&S61410[82], 6}, {NULL}};
const Segment SGI184549[] = {{&S61411[38], 7}, {NULL}};
const Segment SGI184550[] = {{&S61411[31], 6}, {NULL}};
const Segment SGI184551[] = {{&S72422[44], 7}, {NULL}};
const Segment SGI184552[] = {{&S72422[44], 19}, {NULL}};
const Segment SGI184553[] = {{&S72422[30], 6}, {NULL}};
const Segment SGI184554[] = {{&S72422[31], 3}, {NULL}};
const Segment SGI184555[] = {{&S72422[77], 6}, {NULL}};
const Segment SGI184556[] = {{&S61418[34], 15}, {NULL}};
const Segment SGI184557[] = {{&S61418[27], 6}, {NULL}};
const Segment SGI184558[] = {{&S61418[102], 9}, {NULL}};
const Segment SGI184559[] = {{&S61418[75], 6}, {NULL}};
const Segment SGI184560[] = {{&S61419[34], 7}, {NULL}};
const Segment SGI184561[] = {{&S61419[27], 6}, {NULL}};
const Segment SGI184562[] = {{&S61419[99], 9}, {NULL}};
const Segment SGI184563[] = {{&S61419[72], 6}, {NULL}};
const Segment SGI184564[] = {{&S72431[44], 10}, {NULL}};
const Segment SGI184565[] = {{&S72431[44], 22}, {NULL}};
const Segment SGI184566[] = {{&S72431[30], 6}, {NULL}};
const Segment SGI184567[] = {{&S72431[80], 6}, {NULL}};
const Segment SGI184568[] = {{&S50504[44], 10}, {NULL}};
const Segment SGI184569[] = {{&S72431[107], 13}, {NULL}};
const Segment SGI184570[] = {{&S50504[44], 21}, {NULL}};
const Segment SGI184571[] = {{&S50504[30], 6}, {NULL}};
const Segment SGI184572[] = {{&S50504[132], 3}, {NULL}};
const Segment SGI184573[] = {{&S50504[97], 6}, {NULL}};
const Segment SGI184574[] = {{&S50504[67], 8}, {NULL}};
const Segment SGI184575[] = {{&S50505[44], 18}, {NULL}};
const Segment SGI184576[] = {{&S50505[44], 29}, {NULL}};
const Segment SGI184577[] = {{&S50505[30], 6}, {NULL}};
const Segment SGI184578[] = {{&S50505[140], 3}, {NULL}};
const Segment SGI184579[] = {{&S50505[105], 6}, {NULL}};
const Segment SGI184580[] = {{&S50505[75], 8}, {NULL}};
const Segment SGI184581[] = {{&S50506[44], 10}, {NULL}};
const Segment SGI184582[] = {{&S50506[44], 22}, {NULL}};
const Segment SGI184583[] = {{&S50506[30], 6}, {NULL}};
const Segment SGI184584[] = {{&S50506[133], 3}, {NULL}};
const Segment SGI184585[] = {{&S50506[98], 6}, {NULL}};
const Segment SGI184586[] = {{&S50506[68], 8}, {NULL}};
const Segment SGI184587[] = {{&S65544[44], 6}, {NULL}};
const Segment SGI184588[] = {{&S65544[44], 18}, {NULL}};
const Segment SGI184589[] = {{&S65544[30], 6}, {NULL}};
const Segment SGI184590[] = {{&S65544[111], 3}, {NULL}};
const Segment SGI184591[] = {{&S50516[44], 10}, {NULL}};
const Segment SGI184592[] = {{&S50516[44], 21}, {NULL}};
const Segment SGI184593[] = {{&S65544[76], 6}, {NULL}};
const Segment SGI184594[] = {{&S50516[30], 6}, {NULL}};
const Segment SGI184595[] = {{&S50516[114], 3}, {NULL}};
const Segment SGI184596[] = {{&S50516[79], 6}, {NULL}};
const Segment SGI184597[] = {{&S50518[44], 10}, {NULL}};
const Segment SGI184598[] = {{&S50518[44], 21}, {NULL}};
const Segment SGI184599[] = {{&S50518[30], 6}, {NULL}};
const Segment SGI184600[] = {{&S50518[114], 3}, {NULL}};
const Segment SGI184601[] = {{&S50518[79], 6}, {NULL}};
const Segment SGI184602[] = {{&S50519[44], 10}, {NULL}};
const Segment SGI184603[] = {{&S50519[44], 21}, {NULL}};
const Segment SGI184604[] = {{&S50519[30], 6}, {NULL}};
const Segment SGI184605[] = {{&S50519[114], 3}, {NULL}};
const Segment SGI184606[] = {{&S50519[79], 6}, {NULL}};
const Segment SGI184607[] = {{&S50520[44], 10}, {NULL}};
const Segment SGI184608[] = {{&S50520[44], 21}, {NULL}};
const Segment SGI184609[] = {{&S50520[30], 6}, {NULL}};
const Segment SGI184610[] = {{&S50520[114], 3}, {NULL}};
const Segment SGI184611[] = {{&S50520[79], 6}, {NULL}};
const Segment SGI184612[] = {{&S50521[44], 16}, {NULL}};
const Segment SGI184613[] = {{&S50521[44], 27}, {NULL}};
const Segment SGI184614[] = {{&S50521[30], 6}, {NULL}};
const Segment SGI184615[] = {{&S50521[120], 3}, {NULL}};
const Segment SGI184616[] = {{&S50521[85], 6}, {NULL}};
const Segment SGI184617[] = {{&S50522[44], 10}, {NULL}};
const Segment SGI184618[] = {{&S50522[44], 21}, {NULL}};
const Segment SGI184619[] = {{&S50522[30], 6}, {NULL}};
const Segment SGI184620[] = {{&S50522[114], 3}, {NULL}};
const Segment SGI184621[] = {{&S50522[79], 6}, {NULL}};
const Segment SGI184622[] = {{&S50523[44], 10}, {NULL}};
const Segment SGI184623[] = {{&S50523[44], 22}, {NULL}};
const Segment SGI184624[] = {{&S50523[30], 6}, {NULL}};
const Segment SGI184625[] = {{&S50523[115], 3}, {NULL}};
const Segment SGI184626[] = {{&S50523[80], 6}, {NULL}};
const Segment SGI184627[] = {{&S72461[45], 10}, {NULL}};
const Segment SGI184628[] = {{&S72461[30], 6}, {NULL}};
const Segment SGI184629[] = {{&S50525[38], 10}, {NULL}};
const Segment SGI184630[] = {{&S50525[31], 6}, {NULL}};
const Segment SGI184631[] = {{&S72461[31], 3}, {NULL}};
const Segment SGI184632[] = {{&S72461[81], 5}, {NULL}};
const Segment SGI184633[] = {{&S72461[139], 13}, {NULL}};
const Segment SGI184634[] = {{&S50526[44], 10}, {NULL}};
const Segment SGI184635[] = {{&S50526[44], 22}, {NULL}};
const Segment SGI184636[] = {{&S50526[30], 6}, {NULL}};
const Segment SGI184637[] = {{&S50526[115], 3}, {NULL}};
const Segment SGI184638[] = {{&S50526[80], 6}, {NULL}};
const Segment SGI184639[] = {{&S50530[44], 18}, {NULL}};
const Segment SGI184640[] = {{&S50530[44], 29}, {NULL}};
const Segment SGI184641[] = {{&S50530[30], 6}, {NULL}};
const Segment SGI184642[] = {{&S50530[122], 3}, {NULL}};
const Segment SGI184643[] = {{&S50530[87], 6}, {NULL}};
const Segment SGI184644[] = {{&S61468[44], 9}, {NULL}};
const Segment SGI184645[] = {{&S61468[44], 21}, {NULL}};
const Segment SGI184646[] = {{&S61468[30], 6}, {NULL}};
const Segment SGI184647[] = {{&S50532[38], 10}, {NULL}};
const Segment SGI184648[] = {{&S50532[31], 6}, {NULL}};
const Segment SGI184649[] = {{&S61468[114], 3}, {NULL}};
const Segment SGI184650[] = {{&S61468[79], 6}, {NULL}};
const Segment SGI184651[] = {{&S50533[38], 18}, {NULL}};
const Segment SGI184652[] = {{&S50533[31], 6}, {NULL}};
const Segment SGI184653[] = {{&S61469[44], 11}, {NULL}};
const Segment SGI184654[] = {{&S61469[44], 23}, {NULL}};
const Segment SGI184655[] = {{&S61469[30], 6}, {NULL}};
const Segment SGI184656[] = {{&S61469[116], 3}, {NULL}};
const Segment SGI184657[] = {{&S61469[81], 6}, {NULL}};
const Segment SGI184658[] = {{&S50534[44], 10}, {NULL}};
const Segment SGI184659[] = {{&S50534[44], 22}, {NULL}};
const Segment SGI184660[] = {{&S50534[30], 6}, {NULL}};
const Segment SGI184661[] = {{&S61470[38], 9}, {NULL}};
const Segment SGI184662[] = {{&S50534[115], 3}, {NULL}};
const Segment SGI184663[] = {{&S61470[31], 6}, {NULL}};
const Segment SGI184664[] = {{&S50534[80], 6}, {NULL}};
const Segment SGI184665[] = {{&S65574[44], 25}, {NULL}};
const Segment SGI184666[] = {{&S65574[44], 37}, {NULL}};
const Segment SGI184667[] = {{&S65574[30], 6}, {NULL}};
const Segment SGI184668[] = {{&S65574[31], 3}, {NULL}};
const Segment SGI184669[] = {{&S65574[95], 6}, {NULL}};
const Segment SGI184670[] = {{&S50535[44], 10}, {NULL}};
const Segment SGI184671[] = {{&S50535[44], 22}, {NULL}};
const Segment SGI184672[] = {{&S50535[30], 6}, {NULL}};
const Segment SGI184673[] = {{&S50535[115], 3}, {NULL}};
const Segment SGI184674[] = {{&S50535[80], 6}, {NULL}};
const Segment SGI184675[] = {{&S65575[38], 25}, {NULL}};
const Segment SGI184676[] = {{&S65575[31], 6}, {NULL}};
const Segment SGI184677[] = {{&S50536[44], 18}, {NULL}};
const Segment SGI184678[] = {{&S50536[44], 30}, {NULL}};
const Segment SGI184679[] = {{&S50536[30], 6}, {NULL}};
const Segment SGI184680[] = {{&S50536[123], 3}, {NULL}};
const Segment SGI184681[] = {{&S50536[88], 6}, {NULL}};
const Segment SGI184682[] = {{&S50537[44], 10}, {NULL}};
const Segment SGI184683[] = {{&S50537[44], 22}, {NULL}};
const Segment SGI184684[] = {{&S50537[30], 6}, {NULL}};
const Segment SGI184685[] = {{&S50537[115], 3}, {NULL}};
const Segment SGI184686[] = {{&S50537[80], 6}, {NULL}};
const Segment SGI184687[] = {{&S61476[34], 11}, {NULL}};
const Segment SGI184688[] = {{&S61476[27], 6}, {NULL}};
const Segment SGI184689[] = {{&S61476[98], 9}, {NULL}};
const Segment SGI184690[] = {{&S61476[71], 6}, {NULL}};
const Segment SGI184691[] = {{&S61477[34], 11}, {NULL}};
const Segment SGI184692[] = {{&S61477[27], 6}, {NULL}};
const Segment SGI184693[] = {{&S61477[98], 9}, {NULL}};
const Segment SGI184694[] = {{&S61477[71], 6}, {NULL}};
const Segment SGI184695[] = {{&S72483[44], 9}, {NULL}};
const Segment SGI184696[] = {{&S72483[44], 21}, {NULL}};
const Segment SGI184697[] = {{&S72483[30], 6}, {NULL}};
const Segment SGI184698[] = {{&S72483[31], 3}, {NULL}};
const Segment SGI184699[] = {{&S72483[79], 6}, {NULL}};
const Segment SGI184700[] = {{&S61479[44], 8}, {NULL}};
const Segment SGI184701[] = {{&S61479[44], 26}, {NULL}};
const Segment SGI184702[] = {{&S61479[30], 6}, {NULL}};
const Segment SGI184703[] = {{&S72484[44], 12}, {NULL}};
const Segment SGI184704[] = {{&S72484[44], 24}, {NULL}};
const Segment SGI184705[] = {{&S61479[119], 3}, {NULL}};
const Segment SGI184706[] = {{&S72484[30], 6}, {NULL}};
const Segment SGI184707[] = {{&S61479[84], 6}, {NULL}};
const Segment SGI184708[] = {{&S72484[31], 3}, {NULL}};
const Segment SGI184709[] = {{&S72484[82], 6}, {NULL}};
const Segment SGI184710[] = {{&S61480[44], 8}, {NULL}};
const Segment SGI184711[] = {{&S61480[44], 26}, {NULL}};
const Segment SGI184712[] = {{&S61480[30], 6}, {NULL}};
const Segment SGI184713[] = {{&S61480[119], 3}, {NULL}};
const Segment SGI184714[] = {{&S61480[84], 6}, {NULL}};
const Segment SGI184715[] = {{&S61481[38], 8}, {NULL}};
const Segment SGI184716[] = {{&S61481[31], 6}, {NULL}};
const Segment SGI184717[] = {{&S72487[44], 23}, {NULL}};
const Segment SGI184718[] = {{&S72487[44], 34}, {NULL}};
const Segment SGI184719[] = {{&S72487[30], 6}, {NULL}};
const Segment SGI184720[] = {{&S72487[31], 3}, {NULL}};
const Segment SGI184721[] = {{&S72487[92], 6}, {NULL}};
const Segment SGI184722[] = {{&S72488[44], 23}, {NULL}};
const Segment SGI184723[] = {{&S72488[44], 34}, {NULL}};
const Segment SGI184724[] = {{&S72488[30], 6}, {NULL}};
const Segment SGI184725[] = {{&S72488[31], 3}, {NULL}};
const Segment SGI184726[] = {{&S72488[92], 6}, {NULL}};
const Segment SGI184727[] = {{&S72489[44], 12}, {NULL}};
const Segment SGI184728[] = {{&S72489[44], 37}, {NULL}};
const Segment SGI184729[] = {{&S72489[30], 6}, {NULL}};
const Segment SGI184730[] = {{&S72489[31], 3}, {NULL}};
const Segment SGI184731[] = {{&S72489[95], 6}, {NULL}};
const Segment SGI184732[] = {{&S65596[44], 18}, {NULL}};
const Segment SGI184733[] = {{&S65596[44], 39}, {NULL}};
const Segment SGI184734[] = {{&S65596[30], 6}, {NULL}};
const Segment SGI184735[] = {{&S65596[132], 3}, {NULL}};
const Segment SGI184736[] = {{&S65596[97], 6}, {NULL}};
const Segment SGI184737[] = {{&S50557[34], 10}, {NULL}};
const Segment SGI184738[] = {{&S50557[27], 6}, {NULL}};
const Segment SGI184739[] = {{&S50557[97], 9}, {NULL}};
const Segment SGI184740[] = {{&S50557[70], 6}, {NULL}};
const Segment SGI184741[] = {{&S65597[38], 18}, {NULL}};
const Segment SGI184742[] = {{&S65597[31], 6}, {NULL}};
const Segment SGI184743[] = {{&S72493[34], 11}, {NULL}};
const Segment SGI184744[] = {{&S72493[27], 6}, {NULL}};
const Segment SGI184745[] = {{&S50558[34], 10}, {NULL}};
const Segment SGI184746[] = {{&S50558[27], 6}, {NULL}};
const Segment SGI184747[] = {{&S72493[98], 9}, {NULL}};
const Segment SGI184748[] = {{&S72493[71], 6}, {NULL}};
const Segment SGI184749[] = {{&S50558[97], 9}, {NULL}};
const Segment SGI184750[] = {{&S50558[70], 6}, {NULL}};
const Segment SGI184751[] = {{&S50559[34], 18}, {NULL}};
const Segment SGI184752[] = {{&S50559[27], 6}, {NULL}};
const Segment SGI184753[] = {{&S50559[105], 9}, {NULL}};
const Segment SGI184754[] = {{&S72495[34], 25}, {NULL}};
const Segment SGI184755[] = {{&S50559[78], 6}, {NULL}};
const Segment SGI184756[] = {{&S72495[27], 6}, {NULL}};
const Segment SGI184757[] = {{&S72495[112], 9}, {NULL}};
const Segment SGI184758[] = {{&S72495[85], 6}, {NULL}};
const Segment SGI184759[] = {{&S50560[34], 10}, {NULL}};
const Segment SGI184760[] = {{&S50560[27], 6}, {NULL}};
const Segment SGI184761[] = {{&S72496[34], 25}, {NULL}};
const Segment SGI184762[] = {{&S50560[97], 9}, {NULL}};
const Segment SGI184763[] = {{&S50560[70], 6}, {NULL}};
const Segment SGI184764[] = {{&S72496[27], 6}, {NULL}};
const Segment SGI184765[] = {{&S72496[112], 9}, {NULL}};
const Segment SGI184766[] = {{&S72496[85], 6}, {NULL}};
const Segment SGI184767[] = {{&S50561[34], 10}, {NULL}};
const Segment SGI184768[] = {{&S50561[27], 6}, {NULL}};
const Segment SGI184769[] = {{&S50561[97], 9}, {NULL}};
const Segment SGI184770[] = {{&S72497[34], 25}, {NULL}};
const Segment SGI184771[] = {{&S50561[70], 6}, {NULL}};
const Segment SGI184772[] = {{&S72497[27], 6}, {NULL}};
const Segment SGI184773[] = {{&S72497[112], 9}, {NULL}};
const Segment SGI184774[] = {{&S72497[85], 6}, {NULL}};
const Segment SGI184775[] = {{&S50562[34], 10}, {NULL}};
const Segment SGI184776[] = {{&S50562[27], 6}, {NULL}};
const Segment SGI184777[] = {{&S50562[97], 9}, {NULL}};
const Segment SGI184778[] = {{&S50562[70], 6}, {NULL}};
const Segment SGI184779[] = {{&S50563[34], 16}, {NULL}};
const Segment SGI184780[] = {{&S50563[27], 6}, {NULL}};
const Segment SGI184781[] = {{&S50563[102], 9}, {NULL}};
const Segment SGI184782[] = {{&S50563[75], 6}, {NULL}};
const Segment SGI184783[] = {{&S50564[34], 16}, {NULL}};
const Segment SGI184784[] = {{&S50564[27], 6}, {NULL}};
const Segment SGI184785[] = {{&S50564[102], 9}, {NULL}};
const Segment SGI184786[] = {{&S50564[75], 6}, {NULL}};
const Segment SGI184787[] = {{&S50565[34], 10}, {NULL}};
const Segment SGI184788[] = {{&S50565[27], 6}, {NULL}};
const Segment SGI184789[] = {{&S50565[97], 9}, {NULL}};
const Segment SGI184790[] = {{&S50565[70], 6}, {NULL}};
const Segment SGI184791[] = {{&S50566[34], 10}, {NULL}};
const Segment SGI184792[] = {{&S50566[27], 6}, {NULL}};
const Segment SGI184793[] = {{&S50566[97], 9}, {NULL}};
const Segment SGI184794[] = {{&S50566[70], 6}, {NULL}};
const Segment SGI184795[] = {{&S61519[34], 9}, {NULL}};
const Segment SGI184796[] = {{&S61519[27], 6}, {NULL}};
const Segment SGI184797[] = {{&S61519[96], 9}, {NULL}};
const Segment SGI184798[] = {{&S61519[69], 6}, {NULL}};
const Segment SGI184799[] = {{&S65629[34], 13}, {NULL}};
const Segment SGI184800[] = {{&S65629[27], 6}, {NULL}};
const Segment SGI184801[] = {{&S65629[100], 9}, {NULL}};
const Segment SGI184802[] = {{&S65629[73], 6}, {NULL}};
const Segment SGI184803[] = {{&S65630[34], 17}, {NULL}};
const Segment SGI184804[] = {{&S65630[27], 6}, {NULL}};
const Segment SGI184805[] = {{&S65630[104], 9}, {NULL}};
const Segment SGI184806[] = {{&S65630[77], 6}, {NULL}};
const Segment SGI184807[] = {{&S61533[44], 10}, {NULL}};
const Segment SGI184808[] = {{&S61533[44], 22}, {NULL}};
const Segment SGI184809[] = {{&S61533[30], 6}, {NULL}};
const Segment SGI184810[] = {{&S61533[115], 3}, {NULL}};
const Segment SGI184811[] = {{&S61533[80], 6}, {NULL}};
const Segment SGI184812[] = {{&S61533[140], 11}, {NULL}};
const Segment SGI184813[] = {{&S50592[34], 10}, {NULL}};
const Segment SGI184814[] = {{&S61535[44], 10}, {NULL}};
const Segment SGI184815[] = {{&S50592[27], 6}, {NULL}};
const Segment SGI184816[] = {{&S61535[44], 22}, {NULL}};
const Segment SGI184817[] = {{&S61535[30], 6}, {NULL}};
const Segment SGI184818[] = {{&S50592[96], 9}, {NULL}};
const Segment SGI184819[] = {{&S61535[115], 3}, {NULL}};
const Segment SGI184820[] = {{&S50592[69], 6}, {NULL}};
const Segment SGI184821[] = {{&S61535[80], 6}, {NULL}};
const Segment SGI184822[] = {{&S61535[140], 11}, {NULL}};
const Segment SGI184823[] = {{&S50593[34], 11}, {NULL}};
const Segment SGI184824[] = {{&S50593[27], 6}, {NULL}};
const Segment SGI184825[] = {{&S50593[97], 9}, {NULL}};
const Segment SGI184826[] = {{&S50593[70], 6}, {NULL}};
const Segment SGI184827[] = {{&S61537[44], 11}, {NULL}};
const Segment SGI184828[] = {{&S61537[44], 23}, {NULL}};
const Segment SGI184829[] = {{&S61537[30], 6}, {NULL}};
const Segment SGI184830[] = {{&S61537[116], 3}, {NULL}};
const Segment SGI184831[] = {{&S61537[81], 6}, {NULL}};
const Segment SGI184832[] = {{&S61537[141], 11}, {NULL}};
const Segment SGI184833[] = {{&S72541[45], 10}, {NULL}};
const Segment SGI184834[] = {{&S72541[30], 6}, {NULL}};
const Segment SGI184835[] = {{&S72541[31], 3}, {NULL}};
const Segment SGI184836[] = {{&S50595[34], 10}, {NULL}};
const Segment SGI184837[] = {{&S72541[81], 5}, {NULL}};
const Segment SGI184838[] = {{&S50595[27], 6}, {NULL}};
const Segment SGI184839[] = {{&S72541[139], 13}, {NULL}};
const Segment SGI184840[] = {{&S50595[96], 9}, {NULL}};
const Segment SGI184841[] = {{&S50595[69], 6}, {NULL}};
const Segment SGI184842[] = {{&S72548[34], 10}, {NULL}};
const Segment SGI184843[] = {{&S72548[27], 6}, {NULL}};
const Segment SGI184844[] = {{&S72548[97], 9}, {NULL}};
const Segment SGI184845[] = {{&S72548[70], 6}, {NULL}};
const Segment SGI184846[] = {{&S72553[34], 8}, {NULL}};
const Segment SGI184847[] = {{&S72553[27], 6}, {NULL}};
const Segment SGI184848[] = {{&S72553[107], 9}, {NULL}};
const Segment SGI184849[] = {{&S72553[80], 6}, {NULL}};
const Segment SGI184850[] = {{&S72554[34], 7}, {NULL}};
const Segment SGI184851[] = {{&S72554[27], 6}, {NULL}};
const Segment SGI184852[] = {{&S72554[112], 9}, {NULL}};
const Segment SGI184853[] = {{&S72554[85], 6}, {NULL}};
const Segment SGI184854[] = {{&S72574[44], 10}, {NULL}};
const Segment SGI184855[] = {{&S72574[44], 22}, {NULL}};
const Segment SGI184856[] = {{&S72574[30], 6}, {NULL}};
const Segment SGI184857[] = {{&S72574[31], 3}, {NULL}};
const Segment SGI184858[] = {{&S72574[99], 6}, {NULL}};
const Segment SGI184859[] = {{&S72574[68], 8}, {NULL}};
const Segment SGI184860[] = {{&S72575[44], 10}, {NULL}};
const Segment SGI184861[] = {{&S72575[44], 22}, {NULL}};
const Segment SGI184862[] = {{&S72575[30], 6}, {NULL}};
const Segment SGI184863[] = {{&S72575[31], 3}, {NULL}};
const Segment SGI184864[] = {{&S72575[95], 6}, {NULL}};
const Segment SGI184865[] = {{&S72575[68], 8}, {NULL}};
const Segment SGI184866[] = {{&S72576[44], 10}, {NULL}};
const Segment SGI184867[] = {{&S72576[44], 22}, {NULL}};
const Segment SGI184868[] = {{&S72576[30], 6}, {NULL}};
const Segment SGI184869[] = {{&S72576[31], 3}, {NULL}};
const Segment SGI184870[] = {{&S72576[99], 6}, {NULL}};
const Segment SGI184871[] = {{&S72576[68], 8}, {NULL}};
const Segment SGI184872[] = {{&S61598[38], 8}, {NULL}};
const Segment SGI184873[] = {{&S61598[31], 6}, {NULL}};
const Segment SGI184874[] = {{&S61599[44], 12}, {NULL}};
const Segment SGI184875[] = {{&S61599[44], 32}, {NULL}};
const Segment SGI184876[] = {{&S61599[30], 6}, {NULL}};
const Segment SGI184877[] = {{&S61599[125], 3}, {NULL}};
const Segment SGI184878[] = {{&S61599[90], 6}, {NULL}};
const Segment SGI184879[] = {{&S61600[44], 8}, {NULL}};
const Segment SGI184880[] = {{&S61600[44], 28}, {NULL}};
const Segment SGI184881[] = {{&S61600[30], 6}, {NULL}};
const Segment SGI184882[] = {{&S61600[121], 3}, {NULL}};
const Segment SGI184883[] = {{&S61600[86], 6}, {NULL}};
const Segment SGI184884[] = {{&S61601[44], 12}, {NULL}};
const Segment SGI184885[] = {{&S61601[44], 32}, {NULL}};
const Segment SGI184886[] = {{&S61601[30], 6}, {NULL}};
const Segment SGI184887[] = {{&S61601[125], 3}, {NULL}};
const Segment SGI184888[] = {{&S61601[90], 6}, {NULL}};
const Segment SGI184889[] = {{&S61602[44], 8}, {NULL}};
const Segment SGI184890[] = {{&S61602[44], 28}, {NULL}};
const Segment SGI184891[] = {{&S61602[30], 6}, {NULL}};
const Segment SGI184892[] = {{&S61602[121], 3}, {NULL}};
const Segment SGI184893[] = {{&S61602[86], 6}, {NULL}};
const Segment SGI184894[] = {{&S72610[34], 15}, {NULL}};
const Segment SGI184895[] = {{&S72610[27], 6}, {NULL}};
const Segment SGI184896[] = {{&S65728[44], 4}, {NULL}};
const Segment SGI184897[] = {{&S72610[108], 9}, {NULL}};
const Segment SGI184898[] = {{&S65728[44], 16}, {NULL}};
const Segment SGI184899[] = {{&S72610[81], 6}, {NULL}};
const Segment SGI184900[] = {{&S65728[30], 6}, {NULL}};
const Segment SGI184901[] = {{&S65728[109], 3}, {NULL}};
const Segment SGI184902[] = {{&S65728[74], 6}, {NULL}};
const Segment SGI184903[] = {{&S65729[44], 4}, {NULL}};
const Segment SGI184904[] = {{&S65729[44], 16}, {NULL}};
const Segment SGI184905[] = {{&S65729[30], 6}, {NULL}};
const Segment SGI184906[] = {{&S65729[109], 3}, {NULL}};
const Segment SGI184907[] = {{&S65729[74], 6}, {NULL}};
const Segment SGI184908[] = {{&S65730[38], 4}, {NULL}};
const Segment SGI184909[] = {{&S65730[31], 6}, {NULL}};
const Segment SGI184910[] = {{&S61620[45], 10}, {NULL}};
const Segment SGI184911[] = {{&S59830[44], 5}, {NULL}};
const Segment SGI184912[] = {{&S61620[30], 6}, {NULL}};
const Segment SGI184913[] = {{&S59830[44], 17}, {NULL}};
const Segment SGI184914[] = {{&S59830[30], 6}, {NULL}};
const Segment SGI184915[] = {{&S61620[115], 3}, {NULL}};
const Segment SGI184916[] = {{&S59830[128], 3}, {NULL}};
const Segment SGI184917[] = {{&S61620[81], 5}, {NULL}};
const Segment SGI184918[] = {{&S59830[93], 6}, {NULL}};
const Segment SGI184919[] = {{&S61620[139], 13}, {NULL}};
const Segment SGI184920[] = {{&S59830[63], 8}, {NULL}};
const Segment SGI184921[] = {{&S59831[44], 5}, {NULL}};
const Segment SGI184922[] = {{&S59831[44], 17}, {NULL}};
const Segment SGI184923[] = {{&S59831[30], 6}, {NULL}};
const Segment SGI184924[] = {{&S72617[44], 11}, {NULL}};
const Segment SGI184925[] = {{&S72617[44], 23}, {NULL}};
const Segment SGI184926[] = {{&S72617[30], 6}, {NULL}};
const Segment SGI184927[] = {{&S59831[128], 3}, {NULL}};
const Segment SGI184928[] = {{&S59831[93], 6}, {NULL}};
const Segment SGI184929[] = {{&S72617[31], 3}, {NULL}};
const Segment SGI184930[] = {{&S59831[63], 8}, {NULL}};
const Segment SGI184931[] = {{&S72617[81], 6}, {NULL}};
const Segment SGI184932[] = {{&S61623[45], 10}, {NULL}};
const Segment SGI184933[] = {{&S61623[30], 6}, {NULL}};
const Segment SGI184934[] = {{&S61623[115], 3}, {NULL}};
const Segment SGI184935[] = {{&S61623[81], 5}, {NULL}};
const Segment SGI184936[] = {{&S61623[139], 13}, {NULL}};
const Segment SGI184937[] = {{&S61625[45], 10}, {NULL}};
const Segment SGI184938[] = {{&S61625[30], 6}, {NULL}};
const Segment SGI184939[] = {{&S61625[115], 3}, {NULL}};
const Segment SGI184940[] = {{&S61625[81], 5}, {NULL}};
const Segment SGI184941[] = {{&S61625[139], 13}, {NULL}};
const Segment SGI184942[] = {{&S61626[44], 10}, {NULL}};
const Segment SGI184943[] = {{&S61626[44], 22}, {NULL}};
const Segment SGI184944[] = {{&S61626[30], 6}, {NULL}};
const Segment SGI184945[] = {{&S61626[115], 3}, {NULL}};
const Segment SGI184946[] = {{&S61626[80], 6}, {NULL}};
const Segment SGI184947[] = {{&S61626[135], 8}, {NULL}};
const Segment SGI184948[] = {{&S61626[144], 13}, {NULL}};
const Segment SGI184949[] = {{&S72628[34], 9}, {NULL}};
const Segment SGI184950[] = {{&S72628[27], 6}, {NULL}};
const Segment SGI184951[] = {{&S72628[96], 9}, {NULL}};
const Segment SGI184952[] = {{&S72628[69], 6}, {NULL}};
const Segment SGI184953[] = {{&S72629[34], 9}, {NULL}};
const Segment SGI184954[] = {{&S72629[27], 6}, {NULL}};
const Segment SGI184955[] = {{&S72629[96], 9}, {NULL}};
const Segment SGI184956[] = {{&S72629[69], 6}, {NULL}};
const Segment SGI184957[] = {{&S72631[34], 11}, {NULL}};
const Segment SGI184958[] = {{&S72631[27], 6}, {NULL}};
const Segment SGI184959[] = {{&S72631[98], 9}, {NULL}};
const Segment SGI184960[] = {{&S72631[71], 6}, {NULL}};
const Segment SGI184961[] = {{&S72632[50], 13}, {NULL}};
const Segment SGI184962[] = {{&S72632[50], 25}, {NULL}};
const Segment SGI184963[] = {{&S72632[30], 11}, {NULL}};
const Segment SGI184964[] = {{&S72632[31], 3}, {NULL}};
const Segment SGI184965[] = {{&S72632[89], 6}, {NULL}};
const Segment SGI184966[] = {{&S72633[44], 11}, {NULL}};
const Segment SGI184967[] = {{&S72633[44], 23}, {NULL}};
const Segment SGI184968[] = {{&S72633[30], 6}, {NULL}};
const Segment SGI184969[] = {{&S72633[31], 3}, {NULL}};
const Segment SGI184970[] = {{&S72633[81], 6}, {NULL}};
const Segment SGI184971[] = {{&S72634[44], 5}, {NULL}};
const Segment SGI184972[] = {{&S72634[44], 48}, {NULL}};
const Segment SGI184973[] = {{&S72634[30], 6}, {NULL}};
const Segment SGI184974[] = {{&S72634[31], 3}, {NULL}};
const Segment SGI184975[] = {{&S72634[106], 6}, {NULL}};
const Segment SGI184976[] = {{&S72640[34], 7}, {NULL}};
const Segment SGI184977[] = {{&S72640[27], 6}, {NULL}};
const Segment SGI184978[] = {{&S72640[94], 9}, {NULL}};
const Segment SGI184979[] = {{&S72640[67], 6}, {NULL}};
const Segment SGI184980[] = {{&S50727[44], 10}, {NULL}};
const Segment SGI184981[] = {{&S50727[44], 21}, {NULL}};
const Segment SGI184982[] = {{&S50727[30], 6}, {NULL}};
const Segment SGI184983[] = {{&S50727[114], 3}, {NULL}};
const Segment SGI184984[] = {{&S50727[79], 6}, {NULL}};
const Segment SGI184985[] = {{&S50728[38], 10}, {NULL}};
const Segment SGI184986[] = {{&S50728[31], 6}, {NULL}};
const Segment SGI184987[] = {{&S65793[44], 11}, {NULL}};
const Segment SGI184988[] = {{&S65793[44], 23}, {NULL}};
const Segment SGI184989[] = {{&S65793[30], 6}, {NULL}};
const Segment SGI184990[] = {{&S65793[116], 3}, {NULL}};
const Segment SGI184991[] = {{&S50730[44], 10}, {NULL}};
const Segment SGI184992[] = {{&S50730[44], 22}, {NULL}};
const Segment SGI184993[] = {{&S65793[81], 6}, {NULL}};
const Segment SGI184994[] = {{&S50730[30], 6}, {NULL}};
const Segment SGI184995[] = {{&S50730[115], 3}, {NULL}};
const Segment SGI184996[] = {{&S50730[80], 6}, {NULL}};
const Segment SGI184997[] = {{&S72660[44], 16}, {NULL}};
const Segment SGI184998[] = {{&S72660[44], 36}, {NULL}};
const Segment SGI184999[] = {{&S72660[30], 6}, {NULL}};
const Segment SGI185000[] = {{&S72660[31], 3}, {NULL}};
const Segment SGI185001[] = {{&S72660[94], 6}, {NULL}};
const Segment SGI185002[] = {{&S72661[34], 10}, {NULL}};
const Segment SGI185003[] = {{&S72661[27], 6}, {NULL}};
const Segment SGI185004[] = {{&S72661[97], 9}, {NULL}};
const Segment SGI185005[] = {{&S72661[70], 6}, {NULL}};
const Segment SGI185006[] = {{&S72668[44], 9}, {NULL}};
const Segment SGI185007[] = {{&S72668[44], 34}, {NULL}};
const Segment SGI185008[] = {{&S72668[30], 6}, {NULL}};
const Segment SGI185009[] = {{&S72668[31], 3}, {NULL}};
const Segment SGI185010[] = {{&S72668[92], 6}, {NULL}};
const Segment SGI185011[] = {{&S61685[34], 10}, {NULL}};
const Segment SGI185012[] = {{&S61685[27], 6}, {NULL}};
const Segment SGI185013[] = {{&S61685[97], 9}, {NULL}};
const Segment SGI185014[] = {{&S61685[70], 6}, {NULL}};
const Segment SGI185015[] = {{&S72669[44], 9}, {NULL}};
const Segment SGI185016[] = {{&S72669[44], 34}, {NULL}};
const Segment SGI185017[] = {{&S72669[30], 6}, {NULL}};
const Segment SGI185018[] = {{&S72669[31], 3}, {NULL}};
const Segment SGI185019[] = {{&S72669[92], 6}, {NULL}};
const Segment SGI185020[] = {{&S61686[34], 10}, {NULL}};
const Segment SGI185021[] = {{&S61686[27], 6}, {NULL}};
const Segment SGI185022[] = {{&S61686[97], 9}, {NULL}};
const Segment SGI185023[] = {{&S61686[70], 6}, {NULL}};
const Segment SGI185024[] = {{&S65811[44], 9}, {NULL}};
const Segment SGI185025[] = {{&S65811[44], 21}, {NULL}};
const Segment SGI185026[] = {{&S65811[30], 6}, {NULL}};
const Segment SGI185027[] = {{&S72670[44], 9}, {NULL}};
const Segment SGI185028[] = {{&S65811[114], 3}, {NULL}};
const Segment SGI185029[] = {{&S72670[44], 34}, {NULL}};
const Segment SGI185030[] = {{&S72670[30], 6}, {NULL}};
const Segment SGI185031[] = {{&S65811[79], 6}, {NULL}};
const Segment SGI185032[] = {{&S72670[31], 3}, {NULL}};
const Segment SGI185033[] = {{&S72670[92], 6}, {NULL}};
const Segment SGI185034[] = {{&S61687[34], 10}, {NULL}};
const Segment SGI185035[] = {{&S61687[27], 6}, {NULL}};
const Segment SGI185036[] = {{&S61687[97], 9}, {NULL}};
const Segment SGI185037[] = {{&S61687[70], 6}, {NULL}};
const Segment SGI185038[] = {{&S72671[44], 9}, {NULL}};
const Segment SGI185039[] = {{&S72671[44], 34}, {NULL}};
const Segment SGI185040[] = {{&S72671[30], 6}, {NULL}};
const Segment SGI185041[] = {{&S72671[31], 3}, {NULL}};
const Segment SGI185042[] = {{&S72671[92], 6}, {NULL}};
const Segment SGI185043[] = {{&S61688[34], 11}, {NULL}};
const Segment SGI185044[] = {{&S61688[27], 6}, {NULL}};
const Segment SGI185045[] = {{&S61688[98], 9}, {NULL}};
const Segment SGI185046[] = {{&S61688[71], 6}, {NULL}};
const Segment SGI185047[] = {{&S61689[34], 10}, {NULL}};
const Segment SGI185048[] = {{&S61689[27], 6}, {NULL}};
const Segment SGI185049[] = {{&S61689[97], 9}, {NULL}};
const Segment SGI185050[] = {{&S61689[70], 6}, {NULL}};
const Segment SGI185051[] = {{&S61690[34], 11}, {NULL}};
const Segment SGI185052[] = {{&S61690[27], 6}, {NULL}};
const Segment SGI185053[] = {{&S61690[98], 9}, {NULL}};
const Segment SGI185054[] = {{&S61690[71], 6}, {NULL}};
const Segment SGI185055[] = {{&S61695[34], 9}, {NULL}};
const Segment SGI185056[] = {{&S61695[27], 6}, {NULL}};
const Segment SGI185057[] = {{&S61695[96], 9}, {NULL}};
const Segment SGI185058[] = {{&S61695[69], 6}, {NULL}};
const Segment SGI185059[] = {{&S65824[34], 18}, {NULL}};
const Segment SGI185060[] = {{&S65824[27], 6}, {NULL}};
const Segment SGI185061[] = {{&S65824[105], 9}, {NULL}};
const Segment SGI185062[] = {{&S65824[78], 6}, {NULL}};
const Segment SGI185063[] = {{&S65831[44], 12}, {NULL}};
const Segment SGI185064[] = {{&S65831[44], 24}, {NULL}};
const Segment SGI185065[] = {{&S65831[30], 6}, {NULL}};
const Segment SGI185066[] = {{&S65831[133], 3}, {NULL}};
const Segment SGI185067[] = {{&S65831[98], 6}, {NULL}};
const Segment SGI185068[] = {{&S65831[70], 8}, {NULL}};
const Segment SGI185069[] = {{&S50771[34], 10}, {NULL}};
const Segment SGI185070[] = {{&S50771[27], 6}, {NULL}};
const Segment SGI185071[] = {{&S50771[96], 9}, {NULL}};
const Segment SGI185072[] = {{&S65832[44], 12}, {NULL}};
const Segment SGI185073[] = {{&S50771[69], 6}, {NULL}};
const Segment SGI185074[] = {{&S65832[44], 24}, {NULL}};
const Segment SGI185075[] = {{&S65832[30], 6}, {NULL}};
const Segment SGI185076[] = {{&S65832[133], 3}, {NULL}};
const Segment SGI185077[] = {{&S65832[98], 6}, {NULL}};
const Segment SGI185078[] = {{&S65832[70], 8}, {NULL}};
const Segment SGI185079[] = {{&S50772[34], 10}, {NULL}};
const Segment SGI185080[] = {{&S50772[27], 6}, {NULL}};
const Segment SGI185081[] = {{&S65833[44], 12}, {NULL}};
const Segment SGI185082[] = {{&S50772[96], 9}, {NULL}};
const Segment SGI185083[] = {{&S50772[69], 6}, {NULL}};
const Segment SGI185084[] = {{&S65833[44], 24}, {NULL}};
const Segment SGI185085[] = {{&S65833[30], 6}, {NULL}};
const Segment SGI185086[] = {{&S65833[133], 3}, {NULL}};
const Segment SGI185087[] = {{&S65833[98], 6}, {NULL}};
const Segment SGI185088[] = {{&S65833[70], 8}, {NULL}};
const Segment SGI185089[] = {{&S50773[34], 11}, {NULL}};
const Segment SGI185090[] = {{&S50773[27], 6}, {NULL}};
const Segment SGI185091[] = {{&S50773[97], 9}, {NULL}};
const Segment SGI185092[] = {{&S50773[70], 6}, {NULL}};
const Segment SGI185093[] = {{&S50774[34], 11}, {NULL}};
const Segment SGI185094[] = {{&S50774[27], 6}, {NULL}};
const Segment SGI185095[] = {{&S50774[97], 9}, {NULL}};
const Segment SGI185096[] = {{&S50774[70], 6}, {NULL}};
const Segment SGI185097[] = {{&S50775[34], 11}, {NULL}};
const Segment SGI185098[] = {{&S50775[27], 6}, {NULL}};
const Segment SGI185099[] = {{&S50775[97], 9}, {NULL}};
const Segment SGI185100[] = {{&S50775[70], 6}, {NULL}};
const Segment SGI185101[] = {{&S65838[44], 11}, {NULL}};
const Segment SGI185102[] = {{&S65838[44], 23}, {NULL}};
const Segment SGI185103[] = {{&S65838[30], 6}, {NULL}};
const Segment SGI185104[] = {{&S65838[116], 3}, {NULL}};
const Segment SGI185105[] = {{&S65838[81], 6}, {NULL}};
const Segment SGI185106[] = {{&S50776[34], 11}, {NULL}};
const Segment SGI185107[] = {{&S50776[27], 6}, {NULL}};
const Segment SGI185108[] = {{&S61718[44], 15}, {NULL}};
const Segment SGI185109[] = {{&S61718[44], 27}, {NULL}};
const Segment SGI185110[] = {{&S50776[97], 9}, {NULL}};
const Segment SGI185111[] = {{&S61718[30], 6}, {NULL}};
const Segment SGI185112[] = {{&S50776[70], 6}, {NULL}};
const Segment SGI185113[] = {{&S61718[120], 3}, {NULL}};
const Segment SGI185114[] = {{&S65839[38], 11}, {NULL}};
const Segment SGI185115[] = {{&S72704[44], 13}, {NULL}};
const Segment SGI185116[] = {{&S61718[85], 6}, {NULL}};
const Segment SGI185117[] = {{&S65839[31], 6}, {NULL}};
const Segment SGI185118[] = {{&S72704[44], 25}, {NULL}};
const Segment SGI185119[] = {{&S72704[30], 6}, {NULL}};
const Segment SGI185120[] = {{&S72704[31], 3}, {NULL}};
const Segment SGI185121[] = {{&S72704[83], 6}, {NULL}};
const Segment SGI185122[] = {{&S61720[38], 15}, {NULL}};
const Segment SGI185123[] = {{&S61720[31], 6}, {NULL}};
const Segment SGI185124[] = {{&S61721[38], 11}, {NULL}};
const Segment SGI185125[] = {{&S61721[31], 6}, {NULL}};
const Segment SGI185126[] = {{&S65843[44], 9}, {NULL}};
const Segment SGI185127[] = {{&S65843[44], 21}, {NULL}};
const Segment SGI185128[] = {{&S65843[30], 6}, {NULL}};
const Segment SGI185129[] = {{&S65843[114], 3}, {NULL}};
const Segment SGI185130[] = {{&S65843[79], 6}, {NULL}};
const Segment SGI185131[] = {{&S65844[44], 9}, {NULL}};
const Segment SGI185132[] = {{&S65844[44], 21}, {NULL}};
const Segment SGI185133[] = {{&S65844[30], 6}, {NULL}};
const Segment SGI185134[] = {{&S65844[82], 6}, {NULL}};
const Segment SGI185135[] = {{&S72711[44], 9}, {NULL}};
const Segment SGI185136[] = {{&S72711[44], 21}, {NULL}};
const Segment SGI185137[] = {{&S65845[38], 9}, {NULL}};
const Segment SGI185138[] = {{&S72711[30], 6}, {NULL}};
const Segment SGI185139[] = {{&S65845[31], 6}, {NULL}};
const Segment SGI185140[] = {{&S72711[31], 3}, {NULL}};
const Segment SGI185141[] = {{&S72711[79], 6}, {NULL}};
const Segment SGI185142[] = {{&S65846[44], 10}, {NULL}};
const Segment SGI185143[] = {{&S65846[44], 22}, {NULL}};
const Segment SGI185144[] = {{&S65846[30], 6}, {NULL}};
const Segment SGI185145[] = {{&S65846[115], 3}, {NULL}};
const Segment SGI185146[] = {{&S72712[44], 10}, {NULL}};
const Segment SGI185147[] = {{&S72712[44], 22}, {NULL}};
const Segment SGI185148[] = {{&S65846[80], 6}, {NULL}};
const Segment SGI185149[] = {{&S72712[30], 6}, {NULL}};
const Segment SGI185150[] = {{&S72712[31], 3}, {NULL}};
const Segment SGI185151[] = {{&S72712[80], 6}, {NULL}};
const Segment SGI185152[] = {{&S72713[38], 9}, {NULL}};
const Segment SGI185153[] = {{&S72713[31], 6}, {NULL}};
const Segment SGI185154[] = {{&S72717[44], 6}, {NULL}};
const Segment SGI185155[] = {{&S72717[44], 18}, {NULL}};
const Segment SGI185156[] = {{&S72717[30], 6}, {NULL}};
const Segment SGI185157[] = {{&S72717[31], 3}, {NULL}};
const Segment SGI185158[] = {{&S72717[76], 6}, {NULL}};
const Segment SGI185159[] = {{&S72718[38], 6}, {NULL}};
const Segment SGI185160[] = {{&S72718[31], 6}, {NULL}};
const Segment SGI185161[] = {{&S72719[34], 11}, {NULL}};
const Segment SGI185162[] = {{&S72719[27], 6}, {NULL}};
const Segment SGI185163[] = {{&S72719[98], 9}, {NULL}};
const Segment SGI185164[] = {{&S72719[71], 6}, {NULL}};
const Segment SGI185165[] = {{&S72723[34], 15}, {NULL}};
const Segment SGI185166[] = {{&S72723[27], 6}, {NULL}};
const Segment SGI185167[] = {{&S72723[102], 9}, {NULL}};
const Segment SGI185168[] = {{&S72723[75], 6}, {NULL}};
const Segment SGI185169[] = {{&S72729[34], 18}, {NULL}};
const Segment SGI185170[] = {{&S72729[27], 6}, {NULL}};
const Segment SGI185171[] = {{&S72729[105], 9}, {NULL}};
const Segment SGI185172[] = {{&S72729[78], 6}, {NULL}};
const Segment SGI185173[] = {{&S72730[34], 11}, {NULL}};
const Segment SGI185174[] = {{&S72730[27], 6}, {NULL}};
const Segment SGI185175[] = {{&S72730[98], 9}, {NULL}};
const Segment SGI185176[] = {{&S72730[71], 6}, {NULL}};
const Segment SGI185177[] = {{&S72731[34], 11}, {NULL}};
const Segment SGI185178[] = {{&S72731[27], 6}, {NULL}};
const Segment SGI185179[] = {{&S72731[98], 9}, {NULL}};
const Segment SGI185180[] = {{&S72731[71], 6}, {NULL}};
const Segment SGI185181[] = {{&S65883[34], 21}, {NULL}};
const Segment SGI185182[] = {{&S65883[27], 6}, {NULL}};
const Segment SGI185183[] = {{&S65883[108], 9}, {NULL}};
const Segment SGI185184[] = {{&S65883[81], 6}, {NULL}};
const Segment SGI185185[] = {{&S50825[44], 15}, {NULL}};
const Segment SGI185186[] = {{&S50825[44], 33}, {NULL}};
const Segment SGI185187[] = {{&S50825[30], 6}, {NULL}};
const Segment SGI185188[] = {{&S50825[126], 3}, {NULL}};
const Segment SGI185189[] = {{&S50825[91], 6}, {NULL}};
const Segment SGI185190[] = {{&S50826[44], 8}, {NULL}};
const Segment SGI185191[] = {{&S50826[44], 26}, {NULL}};
const Segment SGI185192[] = {{&S50826[30], 6}, {NULL}};
const Segment SGI185193[] = {{&S50826[119], 3}, {NULL}};
const Segment SGI185194[] = {{&S50826[84], 6}, {NULL}};
const Segment SGI185195[] = {{&S72739[44], 19}, {NULL}};
const Segment SGI185196[] = {{&S72739[44], 31}, {NULL}};
const Segment SGI185197[] = {{&S72739[30], 6}, {NULL}};
const Segment SGI185198[] = {{&S72739[31], 3}, {NULL}};
const Segment SGI185199[] = {{&S50827[38], 15}, {NULL}};
const Segment SGI185200[] = {{&S50827[31], 6}, {NULL}};
const Segment SGI185201[] = {{&S72739[89], 6}, {NULL}};
const Segment SGI185202[] = {{&S72740[34], 7}, {NULL}};
const Segment SGI185203[] = {{&S50828[44], 15}, {NULL}};
const Segment SGI185204[] = {{&S72740[27], 6}, {NULL}};
const Segment SGI185205[] = {{&S50828[44], 33}, {NULL}};
const Segment SGI185206[] = {{&S50828[30], 6}, {NULL}};
const Segment SGI185207[] = {{&S72740[99], 9}, {NULL}};
const Segment SGI185208[] = {{&S72740[72], 6}, {NULL}};
const Segment SGI185209[] = {{&S50828[94], 6}, {NULL}};
const Segment SGI185210[] = {{&S50829[44], 22}, {NULL}};
const Segment SGI185211[] = {{&S50829[44], 40}, {NULL}};
const Segment SGI185212[] = {{&S72741[34], 7}, {NULL}};
const Segment SGI185213[] = {{&S50829[30], 6}, {NULL}};
const Segment SGI185214[] = {{&S72741[27], 6}, {NULL}};
const Segment SGI185215[] = {{&S50829[133], 3}, {NULL}};
const Segment SGI185216[] = {{&S50829[98], 6}, {NULL}};
const Segment SGI185217[] = {{&S72741[99], 9}, {NULL}};
const Segment SGI185218[] = {{&S72741[72], 6}, {NULL}};
const Segment SGI185219[] = {{&S72743[44], 11}, {NULL}};
const Segment SGI185220[] = {{&S72743[44], 26}, {NULL}};
const Segment SGI185221[] = {{&S72743[30], 6}, {NULL}};
const Segment SGI185222[] = {{&S50832[41], 18}, {NULL}};
const Segment SGI185223[] = {{&S50832[41], 29}, {NULL}};
const Segment SGI185224[] = {{&S50832[27], 6}, {NULL}};
const Segment SGI185225[] = {{&S72743[31], 3}, {NULL}};
const Segment SGI185226[] = {{&S72743[84], 6}, {NULL}};
const Segment SGI185227[] = {{&S50832[119], 3}, {NULL}};
const Segment SGI185228[] = {{&S50832[123], 9}, {NULL}};
const Segment SGI185229[] = {{&S50832[84], 6}, {NULL}};
const Segment SGI185230[] = {{&S65916[34], 11}, {NULL}};
const Segment SGI185231[] = {{&S65916[27], 6}, {NULL}};
const Segment SGI185232[] = {{&S65916[98], 9}, {NULL}};
const Segment SGI185233[] = {{&S65916[71], 6}, {NULL}};
const Segment SGI185234[] = {{&S65916[139], 8}, {NULL}};
const Segment SGI185235[] = {{&S65917[34], 11}, {NULL}};
const Segment SGI185236[] = {{&S65917[27], 6}, {NULL}};
const Segment SGI185237[] = {{&S65917[98], 9}, {NULL}};
const Segment SGI185238[] = {{&S72759[34], 7}, {NULL}};
const Segment SGI185239[] = {{&S65917[71], 6}, {NULL}};
const Segment SGI185240[] = {{&S72759[27], 6}, {NULL}};
const Segment SGI185241[] = {{&S65917[140], 8}, {NULL}};
const Segment SGI185242[] = {{&S72759[110], 9}, {NULL}};
const Segment SGI185243[] = {{&S72759[83], 6}, {NULL}};
const Segment SGI185244[] = {{&S65918[34], 11}, {NULL}};
const Segment SGI185245[] = {{&S65918[27], 6}, {NULL}};
const Segment SGI185246[] = {{&S65918[98], 9}, {NULL}};
const Segment SGI185247[] = {{&S72760[34], 8}, {NULL}};
const Segment SGI185248[] = {{&S65918[71], 6}, {NULL}};
const Segment SGI185249[] = {{&S72760[27], 6}, {NULL}};
const Segment SGI185250[] = {{&S65918[139], 8}, {NULL}};
const Segment SGI185251[] = {{&S72760[106], 9}, {NULL}};
const Segment SGI185252[] = {{&S72760[79], 6}, {NULL}};
const Segment SGI185253[] = {{&S72761[34], 8}, {NULL}};
const Segment SGI185254[] = {{&S72761[27], 6}, {NULL}};
const Segment SGI185255[] = {{&S72761[106], 9}, {NULL}};
const Segment SGI185256[] = {{&S72761[79], 6}, {NULL}};
const Segment SGI185257[] = {{&S69939[42], 10}, {NULL}};
const Segment SGI185258[] = {{&S69939[42], 22}, {NULL}};
const Segment SGI185259[] = {{&S69939[30], 4}, {NULL}};
const Segment SGI185260[] = {{&S69939[113], 3}, {NULL}};
const Segment SGI185261[] = {{&S69939[78], 6}, {NULL}};
const Segment SGI185262[] = {{&S50866[34], 8}, {NULL}};
const Segment SGI185263[] = {{&S50866[27], 6}, {NULL}};
const Segment SGI185264[] = {{&S50866[94], 9}, {NULL}};
const Segment SGI185265[] = {{&S50866[67], 6}, {NULL}};
const Segment SGI185266[] = {{&S65935[44], 7}, {NULL}};
const Segment SGI185267[] = {{&S65935[44], 19}, {NULL}};
const Segment SGI185268[] = {{&S65935[30], 6}, {NULL}};
const Segment SGI185269[] = {{&S65935[112], 3}, {NULL}};
const Segment SGI185270[] = {{&S65935[77], 6}, {NULL}};
const Segment SGI185271[] = {{&S65936[44], 14}, {NULL}};
const Segment SGI185272[] = {{&S65936[44], 26}, {NULL}};
const Segment SGI185273[] = {{&S65936[30], 6}, {NULL}};
const Segment SGI185274[] = {{&S65936[119], 3}, {NULL}};
const Segment SGI185275[] = {{&S65936[84], 6}, {NULL}};
const Segment SGI185276[] = {{&S65937[59], 10}, {NULL}};
const Segment SGI185277[] = {{&S65937[59], 22}, {NULL}};
const Segment SGI185278[] = {{&S65937[45], 6}, {NULL}};
const Segment SGI185279[] = {{&S65937[130], 3}, {NULL}};
const Segment SGI185280[] = {{&S65937[95], 6}, {NULL}};
const Segment SGI185281[] = {{&S65937[0], 14}, {NULL}};
const Segment SGI185282[] = {{&S72793[34], 10}, {NULL}};
const Segment SGI185283[] = {{&S72793[27], 6}, {NULL}};
const Segment SGI185284[] = {{&S72793[97], 9}, {NULL}};
const Segment SGI185285[] = {{&S72793[70], 6}, {NULL}};
const Segment SGI185286[] = {{&S72795[44], 10}, {NULL}};
const Segment SGI185287[] = {{&S72795[44], 22}, {NULL}};
const Segment SGI185288[] = {{&S72795[30], 6}, {NULL}};
const Segment SGI185289[] = {{&S72795[31], 3}, {NULL}};
const Segment SGI185290[] = {{&S72795[99], 6}, {NULL}};
const Segment SGI185291[] = {{&S72795[68], 8}, {NULL}};
const Segment SGI185292[] = {{&S72823[44], 14}, {NULL}};
const Segment SGI185293[] = {{&S72823[44], 26}, {NULL}};
const Segment SGI185294[] = {{&S72823[30], 6}, {NULL}};
const Segment SGI185295[] = {{&S72823[31], 3}, {NULL}};
const Segment SGI185296[] = {{&S72823[84], 6}, {NULL}};
const Segment SGI185297[] = {{&S72825[45], 10}, {NULL}};
const Segment SGI185298[] = {{&S72825[30], 6}, {NULL}};
const Segment SGI185299[] = {{&S72825[31], 3}, {NULL}};
const Segment SGI185300[] = {{&S72825[81], 5}, {NULL}};
const Segment SGI185301[] = {{&S72825[139], 13}, {NULL}};
const Segment SGI185302[] = {{&S72827[44], 10}, {NULL}};
const Segment SGI185303[] = {{&S72827[44], 22}, {NULL}};
const Segment SGI185304[] = {{&S72827[30], 6}, {NULL}};
const Segment SGI185305[] = {{&S72827[115], 3}, {NULL}};
const Segment SGI185306[] = {{&S72827[80], 6}, {NULL}};
const Segment SGI185307[] = {{&S72827[135], 8}, {NULL}};
const Segment SGI185308[] = {{&S72827[144], 13}, {NULL}};
const Segment SGI185309[] = {{&S72828[44], 6}, {NULL}};
const Segment SGI185310[] = {{&S72828[44], 18}, {NULL}};
const Segment SGI185311[] = {{&S72828[30], 6}, {NULL}};
const Segment SGI185312[] = {{&S72828[31], 3}, {NULL}};
const Segment SGI185313[] = {{&S72828[76], 6}, {NULL}};
const Segment SGI185314[] = {{&S72829[44], 6}, {NULL}};
const Segment SGI185315[] = {{&S72829[44], 18}, {NULL}};
const Segment SGI185316[] = {{&S72829[30], 6}, {NULL}};
const Segment SGI185317[] = {{&S72829[31], 3}, {NULL}};
const Segment SGI185318[] = {{&S72829[76], 6}, {NULL}};
const Segment SGI185319[] = {{&S61732[34], 12}, {NULL}};
const Segment SGI185320[] = {{&S61732[27], 6}, {NULL}};
const Segment SGI185321[] = {{&S66000[44], 14}, {NULL}};
const Segment SGI185322[] = {{&S66000[44], 26}, {NULL}};
const Segment SGI185323[] = {{&S66000[30], 6}, {NULL}};
const Segment SGI185324[] = {{&S61732[99], 9}, {NULL}};
const Segment SGI185325[] = {{&S61732[72], 6}, {NULL}};
const Segment SGI185326[] = {{&S66000[119], 3}, {NULL}};
const Segment SGI185327[] = {{&S66000[84], 6}, {NULL}};
const Segment SGI185328[] = {{&S61733[34], 12}, {NULL}};
const Segment SGI185329[] = {{&S66001[38], 14}, {NULL}};
const Segment SGI185330[] = {{&S61733[27], 6}, {NULL}};
const Segment SGI185331[] = {{&S66001[31], 6}, {NULL}};
const Segment SGI185332[] = {{&S61733[99], 9}, {NULL}};
const Segment SGI185333[] = {{&S61733[72], 6}, {NULL}};
const Segment SGI185334[] = {{&S72835[34], 41}, {NULL}};
const Segment SGI185335[] = {{&S72835[27], 6}, {NULL}};
const Segment SGI185336[] = {{&S72835[138], 9}, {NULL}};
const Segment SGI185337[] = {{&S72835[111], 6}, {NULL}};
const Segment SGI185338[] = {{&S72836[34], 20}, {NULL}};
const Segment SGI185339[] = {{&S61736[34], 8}, {NULL}};
const Segment SGI185340[] = {{&S72836[27], 6}, {NULL}};
const Segment SGI185341[] = {{&S61736[27], 6}, {NULL}};
const Segment SGI185342[] = {{&S72836[117], 9}, {NULL}};
const Segment SGI185343[] = {{&S61736[101], 9}, {NULL}};
const Segment SGI185344[] = {{&S61736[74], 6}, {NULL}};
const Segment SGI185345[] = {{&S72836[90], 6}, {NULL}};
const Segment SGI185346[] = {{&S72839[34], 9}, {NULL}};
const Segment SGI185347[] = {{&S72839[27], 6}, {NULL}};
const Segment SGI185348[] = {{&S72839[96], 9}, {NULL}};
const Segment SGI185349[] = {{&S72839[69], 6}, {NULL}};
const Segment SGI185350[] = {{&S50952[44], 18}, {NULL}};
const Segment SGI185351[] = {{&S72840[34], 10}, {NULL}};
const Segment SGI185352[] = {{&S50952[44], 29}, {NULL}};
const Segment SGI185353[] = {{&S72840[27], 6}, {NULL}};
const Segment SGI185354[] = {{&S50952[30], 6}, {NULL}};
const Segment SGI185355[] = {{&S50952[140], 3}, {NULL}};
const Segment SGI185356[] = {{&S72840[97], 9}, {NULL}};
const Segment SGI185357[] = {{&S72840[70], 6}, {NULL}};
const Segment SGI185358[] = {{&S50952[105], 6}, {NULL}};
const Segment SGI185359[] = {{&S50952[75], 8}, {NULL}};
const Segment SGI185360[] = {{&S50953[44], 18}, {NULL}};
const Segment SGI185361[] = {{&S50953[44], 30}, {NULL}};
const Segment SGI185362[] = {{&S50953[30], 6}, {NULL}};
const Segment SGI185363[] = {{&S50953[123], 3}, {NULL}};
const Segment SGI185364[] = {{&S50953[88], 6}, {NULL}};
const Segment SGI185365[] = {{&S50953[148], 8}, {NULL}};
const Segment SGI185366[] = {{&S72843[34], 10}, {NULL}};
const Segment SGI185367[] = {{&S72843[27], 6}, {NULL}};
const Segment SGI185368[] = {{&S72843[97], 9}, {NULL}};
const Segment SGI185369[] = {{&S72843[70], 6}, {NULL}};
const Segment SGI185370[] = {{&S66017[44], 9}, {NULL}};
const Segment SGI185371[] = {{&S66017[44], 21}, {NULL}};
const Segment SGI185372[] = {{&S66017[30], 6}, {NULL}};
const Segment SGI185373[] = {{&S66017[114], 3}, {NULL}};
const Segment SGI185374[] = {{&S66017[79], 6}, {NULL}};
const Segment SGI185375[] = {{&S72844[34], 7}, {NULL}};
const Segment SGI185376[] = {{&S72844[27], 6}, {NULL}};
const Segment SGI185377[] = {{&S66018[38], 9}, {NULL}};
const Segment SGI185378[] = {{&S72844[111], 9}, {NULL}};
const Segment SGI185379[] = {{&S72844[84], 6}, {NULL}};
const Segment SGI185380[] = {{&S66018[31], 6}, {NULL}};
const Segment SGI185381[] = {{&S72849[34], 10}, {NULL}};
const Segment SGI185382[] = {{&S72849[27], 6}, {NULL}};
const Segment SGI185383[] = {{&S72849[97], 9}, {NULL}};
const Segment SGI185384[] = {{&S72849[70], 6}, {NULL}};
const Segment SGI185385[] = {{&S72849[138], 8}, {NULL}};
const Segment SGI185386[] = {{&S61762[44], 10}, {NULL}};
const Segment SGI185387[] = {{&S61762[44], 37}, {NULL}};
const Segment SGI185388[] = {{&S61762[30], 6}, {NULL}};
const Segment SGI185389[] = {{&S61762[130], 3}, {NULL}};
const Segment SGI185390[] = {{&S61762[95], 6}, {NULL}};
const Segment SGI185391[] = {{&S61762[162], 15}, {NULL}};
const Segment SGI185392[] = {{&S66040[34], 5}, {NULL}};
const Segment SGI185393[] = {{&S66040[27], 6}, {NULL}};
const Segment SGI185394[] = {{&S66040[92], 9}, {NULL}};
const Segment SGI185395[] = {{&S66040[65], 6}, {NULL}};
const Segment SGI185396[] = {{&S50984[34], 11}, {NULL}};
const Segment SGI185397[] = {{&S50984[27], 6}, {NULL}};
const Segment SGI185398[] = {{&S50984[97], 9}, {NULL}};
const Segment SGI185399[] = {{&S50984[70], 6}, {NULL}};
const Segment SGI185400[] = {{&S50985[34], 10}, {NULL}};
const Segment SGI185401[] = {{&S50985[27], 6}, {NULL}};
const Segment SGI185402[] = {{&S50985[96], 9}, {NULL}};
const Segment SGI185403[] = {{&S50985[69], 6}, {NULL}};
const Segment SGI185404[] = {{&S50987[34], 10}, {NULL}};
const Segment SGI185405[] = {{&S50987[27], 6}, {NULL}};
const Segment SGI185406[] = {{&S50987[96], 9}, {NULL}};
const Segment SGI185407[] = {{&S61777[34], 8}, {NULL}};
const Segment SGI185408[] = {{&S50987[69], 6}, {NULL}};
const Segment SGI185409[] = {{&S61777[27], 6}, {NULL}};
const Segment SGI185410[] = {{&S61777[101], 9}, {NULL}};
const Segment SGI185411[] = {{&S61777[74], 6}, {NULL}};
const Segment SGI185412[] = {{&S61778[34], 7}, {NULL}};
const Segment SGI185413[] = {{&S61778[27], 6}, {NULL}};
const Segment SGI185414[] = {{&S61778[100], 9}, {NULL}};
const Segment SGI185415[] = {{&S61778[73], 6}, {NULL}};
const Segment SGI185416[] = {{&S61779[34], 7}, {NULL}};
const Segment SGI185417[] = {{&S61779[27], 6}, {NULL}};
const Segment SGI185418[] = {{&S61779[100], 9}, {NULL}};
const Segment SGI185419[] = {{&S61779[73], 6}, {NULL}};
const Segment SGI185420[] = {{&S61788[34], 10}, {NULL}};
const Segment SGI185421[] = {{&S61788[27], 6}, {NULL}};
const Segment SGI185422[] = {{&S61788[97], 9}, {NULL}};
const Segment SGI185423[] = {{&S61788[70], 6}, {NULL}};
const Segment SGI185424[] = {{&S61790[34], 10}, {NULL}};
const Segment SGI185425[] = {{&S61790[27], 6}, {NULL}};
const Segment SGI185426[] = {{&S61790[97], 9}, {NULL}};
const Segment SGI185427[] = {{&S61790[70], 6}, {NULL}};
const Segment SGI185428[] = {{&S61791[34], 10}, {NULL}};
const Segment SGI185429[] = {{&S61791[27], 6}, {NULL}};
const Segment SGI185430[] = {{&S61791[97], 9}, {NULL}};
const Segment SGI185431[] = {{&S61791[70], 6}, {NULL}};
const Segment SGI185432[] = {{&S61792[34], 11}, {NULL}};
const Segment SGI185433[] = {{&S61792[27], 6}, {NULL}};
const Segment SGI185434[] = {{&S61792[98], 9}, {NULL}};
const Segment SGI185435[] = {{&S61792[71], 6}, {NULL}};
const Segment SGI185436[] = {{&S61793[34], 11}, {NULL}};
const Segment SGI185437[] = {{&S61793[27], 6}, {NULL}};
const Segment SGI185438[] = {{&S61793[98], 9}, {NULL}};
const Segment SGI185439[] = {{&S61793[71], 6}, {NULL}};
const Segment SGI185440[] = {{&S72910[34], 5}, {NULL}};
const Segment SGI185441[] = {{&S72910[27], 6}, {NULL}};
const Segment SGI185442[] = {{&S72910[92], 9}, {NULL}};
const Segment SGI185443[] = {{&S72910[65], 6}, {NULL}};
const Segment SGI185444[] = {{&S61815[44], 7}, {NULL}};
const Segment SGI185445[] = {{&S61815[44], 19}, {NULL}};
const Segment SGI185446[] = {{&S61815[30], 6}, {NULL}};
const Segment SGI185447[] = {{&S61815[112], 3}, {NULL}};
const Segment SGI185448[] = {{&S61815[77], 6}, {NULL}};
const Segment SGI185449[] = {{&S61816[44], 8}, {NULL}};
const Segment SGI185450[] = {{&S61816[44], 20}, {NULL}};
const Segment SGI185451[] = {{&S61816[30], 6}, {NULL}};
const Segment SGI185452[] = {{&S61816[113], 3}, {NULL}};
const Segment SGI185453[] = {{&S61816[78], 6}, {NULL}};
const Segment SGI185454[] = {{&S61817[44], 8}, {NULL}};
const Segment SGI185455[] = {{&S61817[44], 20}, {NULL}};
const Segment SGI185456[] = {{&S61817[30], 6}, {NULL}};
const Segment SGI185457[] = {{&S61817[113], 3}, {NULL}};
const Segment SGI185458[] = {{&S61817[78], 6}, {NULL}};
const Segment SGI185459[] = {{&S61818[34], 18}, {NULL}};
const Segment SGI185460[] = {{&S61818[27], 6}, {NULL}};
const Segment SGI185461[] = {{&S61818[105], 9}, {NULL}};
const Segment SGI185462[] = {{&S61818[78], 6}, {NULL}};
const Segment SGI185463[] = {{&S61819[34], 10}, {NULL}};
const Segment SGI185464[] = {{&S61819[27], 6}, {NULL}};
const Segment SGI185465[] = {{&S61819[97], 9}, {NULL}};
const Segment SGI185466[] = {{&S61819[70], 6}, {NULL}};
const Segment SGI185467[] = {{&S72938[34], 8}, {NULL}};
const Segment SGI185468[] = {{&S72938[27], 6}, {NULL}};
const Segment SGI185469[] = {{&S72938[107], 9}, {NULL}};
const Segment SGI185470[] = {{&S72938[80], 6}, {NULL}};
const Segment SGI185471[] = {{&S72939[34], 7}, {NULL}};
const Segment SGI185472[] = {{&S72939[27], 6}, {NULL}};
const Segment SGI185473[] = {{&S72939[112], 9}, {NULL}};
const Segment SGI185474[] = {{&S72939[85], 6}, {NULL}};
const Segment SGI185475[] = {{&S51057[34], 29}, {NULL}};
const Segment SGI185476[] = {{&S51057[27], 6}, {NULL}};
const Segment SGI185477[] = {{&S51057[116], 9}, {NULL}};
const Segment SGI185478[] = {{&S51057[89], 6}, {NULL}};
const Segment SGI185479[] = {{&S72947[44], 9}, {NULL}};
const Segment SGI185480[] = {{&S72947[44], 21}, {NULL}};
const Segment SGI185481[] = {{&S72947[30], 6}, {NULL}};
const Segment SGI185482[] = {{&S72947[31], 3}, {NULL}};
const Segment SGI185483[] = {{&S72947[98], 6}, {NULL}};
const Segment SGI185484[] = {{&S72947[67], 8}, {NULL}};
const Segment SGI185485[] = {{&S72948[44], 9}, {NULL}};
const Segment SGI185486[] = {{&S72948[44], 20}, {NULL}};
const Segment SGI185487[] = {{&S72948[30], 6}, {NULL}};
const Segment SGI185488[] = {{&S72948[31], 3}, {NULL}};
const Segment SGI185489[] = {{&S72948[97], 6}, {NULL}};
const Segment SGI185490[] = {{&S72948[66], 8}, {NULL}};
const Segment SGI185491[] = {{&S72951[34], 7}, {NULL}};
const Segment SGI185492[] = {{&S72951[27], 6}, {NULL}};
const Segment SGI185493[] = {{&S72951[99], 9}, {NULL}};
const Segment SGI185494[] = {{&S72951[72], 6}, {NULL}};
const Segment SGI185495[] = {{&S66145[44], 10}, {NULL}};
const Segment SGI185496[] = {{&S66145[44], 22}, {NULL}};
const Segment SGI185497[] = {{&S66145[30], 6}, {NULL}};
const Segment SGI185498[] = {{&S66145[80], 6}, {NULL}};
const Segment SGI185499[] = {{&S66145[107], 13}, {NULL}};
const Segment SGI185500[] = {{&S66147[44], 10}, {NULL}};
const Segment SGI185501[] = {{&S66147[44], 22}, {NULL}};
const Segment SGI185502[] = {{&S66147[30], 6}, {NULL}};
const Segment SGI185503[] = {{&S66147[80], 6}, {NULL}};
const Segment SGI185504[] = {{&S66147[107], 13}, {NULL}};
const Segment SGI185505[] = {{&S51084[44], 11}, {NULL}};
const Segment SGI185506[] = {{&S51084[44], 22}, {NULL}};
const Segment SGI185507[] = {{&S51084[30], 6}, {NULL}};
const Segment SGI185508[] = {{&S51084[115], 3}, {NULL}};
const Segment SGI185509[] = {{&S51084[80], 6}, {NULL}};
const Segment SGI185510[] = {{&S72966[34], 9}, {NULL}};
const Segment SGI185511[] = {{&S72966[27], 6}, {NULL}};
const Segment SGI185512[] = {{&S51085[38], 11}, {NULL}};
const Segment SGI185513[] = {{&S51085[31], 6}, {NULL}};
const Segment SGI185514[] = {{&S61864[34], 10}, {NULL}};
const Segment SGI185515[] = {{&S72966[96], 9}, {NULL}};
const Segment SGI185516[] = {{&S72966[69], 6}, {NULL}};
const Segment SGI185517[] = {{&S61864[27], 6}, {NULL}};
const Segment SGI185518[] = {{&S61864[97], 9}, {NULL}};
const Segment SGI185519[] = {{&S61864[70], 6}, {NULL}};
const Segment SGI185520[] = {{&S51086[42], 11}, {NULL}};
const Segment SGI185521[] = {{&S51086[30], 6}, {NULL}};
const Segment SGI185522[] = {{&S72967[34], 7}, {NULL}};
const Segment SGI185523[] = {{&S72967[27], 6}, {NULL}};
const Segment SGI185524[] = {{&S51086[113], 3}, {NULL}};
const Segment SGI185525[] = {{&S51086[78], 6}, {NULL}};
const Segment SGI185526[] = {{&S72967[95], 9}, {NULL}};
const Segment SGI185527[] = {{&S72967[68], 6}, {NULL}};
const Segment SGI185528[] = {{&S66155[44], 8}, {NULL}};
const Segment SGI185529[] = {{&S66155[44], 20}, {NULL}};
const Segment SGI185530[] = {{&S66155[30], 6}, {NULL}};
const Segment SGI185531[] = {{&S66155[113], 3}, {NULL}};
const Segment SGI185532[] = {{&S66155[78], 6}, {NULL}};
const Segment SGI185533[] = {{&S66156[44], 7}, {NULL}};
const Segment SGI185534[] = {{&S66156[44], 19}, {NULL}};
const Segment SGI185535[] = {{&S66156[30], 6}, {NULL}};
const Segment SGI185536[] = {{&S66156[112], 3}, {NULL}};
const Segment SGI185537[] = {{&S66156[77], 6}, {NULL}};
const Segment SGI185538[] = {{&S72974[34], 10}, {NULL}};
const Segment SGI185539[] = {{&S72974[27], 6}, {NULL}};
const Segment SGI185540[] = {{&S72974[97], 9}, {NULL}};
const Segment SGI185541[] = {{&S72974[70], 6}, {NULL}};
const Segment SGI185542[] = {{&S72975[34], 7}, {NULL}};
const Segment SGI185543[] = {{&S72975[27], 6}, {NULL}};
const Segment SGI185544[] = {{&S72975[104], 9}, {NULL}};
const Segment SGI185545[] = {{&S72975[77], 6}, {NULL}};
const Segment SGI185546[] = {{&S61877[34], 7}, {NULL}};
const Segment SGI185547[] = {{&S61877[27], 6}, {NULL}};
const Segment SGI185548[] = {{&S61877[99], 9}, {NULL}};
const Segment SGI185549[] = {{&S61877[72], 6}, {NULL}};
const Segment SGI185550[] = {{&S66165[34], 8}, {NULL}};
const Segment SGI185551[] = {{&S66165[27], 6}, {NULL}};
const Segment SGI185552[] = {{&S66165[95], 9}, {NULL}};
const Segment SGI185553[] = {{&S66165[68], 6}, {NULL}};
const Segment SGI185554[] = {{&S66167[44], 14}, {NULL}};
const Segment SGI185555[] = {{&S66167[44], 26}, {NULL}};
const Segment SGI185556[] = {{&S66167[30], 6}, {NULL}};
const Segment SGI185557[] = {{&S66167[140], 3}, {NULL}};
const Segment SGI185558[] = {{&S66167[105], 6}, {NULL}};
const Segment SGI185559[] = {{&S61882[34], 10}, {NULL}};
const Segment SGI185560[] = {{&S61882[27], 6}, {NULL}};
const Segment SGI185561[] = {{&S61882[97], 9}, {NULL}};
const Segment SGI185562[] = {{&S61882[70], 6}, {NULL}};
const Segment SGI185563[] = {{&S61888[44], 16}, {NULL}};
const Segment SGI185564[] = {{&S61888[44], 35}, {NULL}};
const Segment SGI185565[] = {{&S61888[30], 6}, {NULL}};
const Segment SGI185566[] = {{&S61888[128], 3}, {NULL}};
const Segment SGI185567[] = {{&S61888[93], 6}, {NULL}};
const Segment SGI185568[] = {{&S61889[38], 16}, {NULL}};
const Segment SGI185569[] = {{&S61889[31], 6}, {NULL}};
const Segment SGI185570[] = {{&S61890[44], 23}, {NULL}};
const Segment SGI185571[] = {{&S61890[44], 46}, {NULL}};
const Segment SGI185572[] = {{&S61890[30], 6}, {NULL}};
const Segment SGI185573[] = {{&S61890[139], 3}, {NULL}};
const Segment SGI185574[] = {{&S61890[104], 6}, {NULL}};
const Segment SGI185575[] = {{&S61891[34], 10}, {NULL}};
const Segment SGI185576[] = {{&S61891[27], 6}, {NULL}};
const Segment SGI185577[] = {{&S61891[97], 9}, {NULL}};
const Segment SGI185578[] = {{&S61891[70], 6}, {NULL}};
const Segment SGI185579[] = {{&S73007[34], 8}, {NULL}};
const Segment SGI185580[] = {{&S73007[27], 6}, {NULL}};
const Segment SGI185581[] = {{&S73007[107], 9}, {NULL}};
const Segment SGI185582[] = {{&S61901[34], 10}, {NULL}};
const Segment SGI185583[] = {{&S73007[80], 6}, {NULL}};
const Segment SGI185584[] = {{&S61901[27], 6}, {NULL}};
const Segment SGI185585[] = {{&S61901[97], 9}, {NULL}};
const Segment SGI185586[] = {{&S61901[70], 6}, {NULL}};
const Segment SGI185587[] = {{&S61901[138], 8}, {NULL}};
const Segment SGI185588[] = {{&S51132[44], 10}, {NULL}};
const Segment SGI185589[] = {{&S51132[44], 21}, {NULL}};
const Segment SGI185590[] = {{&S51132[30], 6}, {NULL}};
const Segment SGI185591[] = {{&S73008[44], 10}, {NULL}};
const Segment SGI185592[] = {{&S51132[114], 3}, {NULL}};
const Segment SGI185593[] = {{&S73008[44], 22}, {NULL}};
const Segment SGI185594[] = {{&S73008[30], 6}, {NULL}};
const Segment SGI185595[] = {{&S51132[79], 6}, {NULL}};
const Segment SGI185596[] = {{&S73008[31], 3}, {NULL}};
const Segment SGI185597[] = {{&S73008[80], 6}, {NULL}};
const Segment SGI185598[] = {{&S51133[38], 10}, {NULL}};
const Segment SGI185599[] = {{&S51133[31], 6}, {NULL}};
const Segment SGI185600[] = {{&S73009[38], 10}, {NULL}};
const Segment SGI185601[] = {{&S73009[31], 6}, {NULL}};
const Segment SGI185602[] = {{&S51134[44], 11}, {NULL}};
const Segment SGI185603[] = {{&S51134[44], 22}, {NULL}};
const Segment SGI185604[] = {{&S51134[30], 6}, {NULL}};
const Segment SGI185605[] = {{&S73010[34], 9}, {NULL}};
const Segment SGI185606[] = {{&S51134[115], 3}, {NULL}};
const Segment SGI185607[] = {{&S73010[27], 6}, {NULL}};
const Segment SGI185608[] = {{&S51134[80], 6}, {NULL}};
const Segment SGI185609[] = {{&S73010[96], 9}, {NULL}};
const Segment SGI185610[] = {{&S73010[69], 6}, {NULL}};
const Segment SGI185611[] = {{&S51135[38], 11}, {NULL}};
const Segment SGI185612[] = {{&S51135[31], 6}, {NULL}};
const Segment SGI185613[] = {{&S73011[44], 5}, {NULL}};
const Segment SGI185614[] = {{&S73011[44], 17}, {NULL}};
const Segment SGI185615[] = {{&S73011[30], 6}, {NULL}};
const Segment SGI185616[] = {{&S73011[75], 6}, {NULL}};
const Segment SGI185617[] = {{&S73011[102], 13}, {NULL}};
const Segment SGI185618[] = {{&S51136[44], 11}, {NULL}};
const Segment SGI185619[] = {{&S51136[44], 22}, {NULL}};
const Segment SGI185620[] = {{&S51136[30], 6}, {NULL}};
const Segment SGI185621[] = {{&S51136[115], 3}, {NULL}};
const Segment SGI185622[] = {{&S51136[80], 6}, {NULL}};
const Segment SGI185623[] = {{&S51137[38], 11}, {NULL}};
const Segment SGI185624[] = {{&S51137[31], 6}, {NULL}};
const Segment SGI185625[] = {{&S66210[44], 21}, {NULL}};
const Segment SGI185626[] = {{&S66210[44], 33}, {NULL}};
const Segment SGI185627[] = {{&S66210[30], 6}, {NULL}};
const Segment SGI185628[] = {{&S66210[126], 3}, {NULL}};
const Segment SGI185629[] = {{&S66210[91], 6}, {NULL}};
const Segment SGI185630[] = {{&S51149[34], 8}, {NULL}};
const Segment SGI185631[] = {{&S51149[27], 6}, {NULL}};
const Segment SGI185632[] = {{&S51149[94], 9}, {NULL}};
const Segment SGI185633[] = {{&S51149[67], 6}, {NULL}};
const Segment SGI185634[] = {{&S73027[44], 7}, {NULL}};
const Segment SGI185635[] = {{&S73027[44], 19}, {NULL}};
const Segment SGI185636[] = {{&S73027[30], 6}, {NULL}};
const Segment SGI185637[] = {{&S73027[31], 3}, {NULL}};
const Segment SGI185638[] = {{&S73027[77], 6}, {NULL}};
const Segment SGI185639[] = {{&S66219[44], 15}, {NULL}};
const Segment SGI185640[] = {{&S66219[44], 27}, {NULL}};
const Segment SGI185641[] = {{&S66219[30], 6}, {NULL}};
const Segment SGI185642[] = {{&S66219[31], 3}, {NULL}};
const Segment SGI185643[] = {{&S66219[85], 6}, {NULL}};
const Segment SGI185644[] = {{&S60329[44], 10}, {NULL}};
const Segment SGI185645[] = {{&S60329[44], 22}, {NULL}};
const Segment SGI185646[] = {{&S60329[30], 6}, {NULL}};
const Segment SGI185647[] = {{&S60329[133], 3}, {NULL}};
const Segment SGI185648[] = {{&S60329[98], 6}, {NULL}};
const Segment SGI185649[] = {{&S60329[68], 8}, {NULL}};
const Segment SGI185650[] = {{&S73032[44], 10}, {NULL}};
const Segment SGI185651[] = {{&S73032[44], 27}, {NULL}};
const Segment SGI185652[] = {{&S73032[30], 6}, {NULL}};
const Segment SGI185653[] = {{&S73032[31], 3}, {NULL}};
const Segment SGI185654[] = {{&S73032[85], 5}, {NULL}};
const Segment SGI185655[] = {{&S73034[44], 7}, {NULL}};
const Segment SGI185656[] = {{&S73034[44], 19}, {NULL}};
const Segment SGI185657[] = {{&S73034[30], 6}, {NULL}};
const Segment SGI185658[] = {{&S73034[31], 3}, {NULL}};
const Segment SGI185659[] = {{&S73034[77], 6}, {NULL}};
const Segment SGI185660[] = {{&S73037[44], 13}, {NULL}};
const Segment SGI185661[] = {{&S73037[44], 25}, {NULL}};
const Segment SGI185662[] = {{&S73037[30], 6}, {NULL}};
const Segment SGI185663[] = {{&S73037[31], 3}, {NULL}};
const Segment SGI185664[] = {{&S73037[83], 6}, {NULL}};
const Segment SGI185665[] = {{&S73038[44], 13}, {NULL}};
const Segment SGI185666[] = {{&S73038[44], 25}, {NULL}};
const Segment SGI185667[] = {{&S73038[30], 6}, {NULL}};
const Segment SGI185668[] = {{&S73038[31], 3}, {NULL}};
const Segment SGI185669[] = {{&S73038[83], 6}, {NULL}};
const Segment SGI185670[] = {{&S66241[44], 11}, {NULL}};
const Segment SGI185671[] = {{&S66241[44], 22}, {NULL}};
const Segment SGI185672[] = {{&S66241[30], 6}, {NULL}};
const Segment SGI185673[] = {{&S66241[115], 3}, {NULL}};
const Segment SGI185674[] = {{&S66241[80], 6}, {NULL}};
const Segment SGI185675[] = {{&S61949[34], 10}, {NULL}};
const Segment SGI185676[] = {{&S61949[27], 6}, {NULL}};
const Segment SGI185677[] = {{&S61949[97], 9}, {NULL}};
const Segment SGI185678[] = {{&S61949[70], 6}, {NULL}};
const Segment SGI185679[] = {{&S66247[34], 25}, {NULL}};
const Segment SGI185680[] = {{&S66247[27], 6}, {NULL}};
const Segment SGI185681[] = {{&S66247[112], 9}, {NULL}};
const Segment SGI185682[] = {{&S66247[85], 6}, {NULL}};
const Segment SGI185683[] = {{&S66248[34], 25}, {NULL}};
const Segment SGI185684[] = {{&S66248[27], 6}, {NULL}};
const Segment SGI185685[] = {{&S66248[112], 9}, {NULL}};
const Segment SGI185686[] = {{&S66248[85], 6}, {NULL}};
const Segment SGI185687[] = {{&S66249[34], 25}, {NULL}};
const Segment SGI185688[] = {{&S66249[27], 6}, {NULL}};
const Segment SGI185689[] = {{&S66249[112], 9}, {NULL}};
const Segment SGI185690[] = {{&S66249[85], 6}, {NULL}};
const Segment SGI185691[] = {{&S66250[34], 25}, {NULL}};
const Segment SGI185692[] = {{&S66250[27], 6}, {NULL}};
const Segment SGI185693[] = {{&S66250[112], 9}, {NULL}};
const Segment SGI185694[] = {{&S66250[85], 6}, {NULL}};
const Segment SGI185695[] = {{&S66251[34], 25}, {NULL}};
const Segment SGI185696[] = {{&S66251[27], 6}, {NULL}};
const Segment SGI185697[] = {{&S66251[112], 9}, {NULL}};
const Segment SGI185698[] = {{&S66251[85], 6}, {NULL}};
const Segment SGI185699[] = {{&S66252[34], 25}, {NULL}};
const Segment SGI185700[] = {{&S51192[34], 7}, {NULL}};
const Segment SGI185701[] = {{&S66252[27], 6}, {NULL}};
const Segment SGI185702[] = {{&S51192[27], 6}, {NULL}};
const Segment SGI185703[] = {{&S66252[112], 9}, {NULL}};
const Segment SGI185704[] = {{&S51192[94], 9}, {NULL}};
const Segment SGI185705[] = {{&S51192[67], 6}, {NULL}};
const Segment SGI185706[] = {{&S66252[85], 6}, {NULL}};
const Segment SGI185707[] = {{&S51193[34], 7}, {NULL}};
const Segment SGI185708[] = {{&S51193[27], 6}, {NULL}};
const Segment SGI185709[] = {{&S51193[100], 9}, {NULL}};
const Segment SGI185710[] = {{&S51193[73], 6}, {NULL}};
const Segment SGI185711[] = {{&S61969[44], 11}, {NULL}};
const Segment SGI185712[] = {{&S61969[44], 23}, {NULL}};
const Segment SGI185713[] = {{&S61969[30], 6}, {NULL}};
const Segment SGI185714[] = {{&S61969[116], 3}, {NULL}};
const Segment SGI185715[] = {{&S61969[81], 6}, {NULL}};
const Segment SGI185716[] = {{&S60370[44], 6}, {NULL}};
const Segment SGI185717[] = {{&S60370[44], 18}, {NULL}};
const Segment SGI185718[] = {{&S61970[44], 11}, {NULL}};
const Segment SGI185719[] = {{&S60370[30], 6}, {NULL}};
const Segment SGI185720[] = {{&S61970[44], 23}, {NULL}};
const Segment SGI185721[] = {{&S61970[30], 6}, {NULL}};
const Segment SGI185722[] = {{&S60370[129], 3}, {NULL}};
const Segment SGI185723[] = {{&S61970[116], 3}, {NULL}};
const Segment SGI185724[] = {{&S60370[94], 6}, {NULL}};
const Segment SGI185725[] = {{&S61970[81], 6}, {NULL}};
const Segment SGI185726[] = {{&S60370[64], 8}, {NULL}};
const Segment SGI185727[] = {{&S61971[44], 11}, {NULL}};
const Segment SGI185728[] = {{&S61971[44], 23}, {NULL}};
const Segment SGI185729[] = {{&S61971[30], 6}, {NULL}};
const Segment SGI185730[] = {{&S61971[116], 3}, {NULL}};
const Segment SGI185731[] = {{&S61971[81], 6}, {NULL}};
const Segment SGI185732[] = {{&S60372[44], 6}, {NULL}};
const Segment SGI185733[] = {{&S60372[44], 18}, {NULL}};
const Segment SGI185734[] = {{&S60372[30], 6}, {NULL}};
const Segment SGI185735[] = {{&S60372[129], 3}, {NULL}};
const Segment SGI185736[] = {{&S60372[94], 6}, {NULL}};
const Segment SGI185737[] = {{&S60372[64], 8}, {NULL}};
const Segment SGI185738[] = {{&S61972[44], 18}, {NULL}};
const Segment SGI185739[] = {{&S61972[44], 30}, {NULL}};
const Segment SGI185740[] = {{&S61972[30], 6}, {NULL}};
const Segment SGI185741[] = {{&S61972[123], 3}, {NULL}};
const Segment SGI185742[] = {{&S61972[88], 6}, {NULL}};
const Segment SGI185743[] = {{&S61973[44], 18}, {NULL}};
const Segment SGI185744[] = {{&S61973[44], 30}, {NULL}};
const Segment SGI185745[] = {{&S61973[30], 6}, {NULL}};
const Segment SGI185746[] = {{&S61973[123], 3}, {NULL}};
const Segment SGI185747[] = {{&S73074[44], 14}, {NULL}};
const Segment SGI185748[] = {{&S73074[44], 28}, {NULL}};
const Segment SGI185749[] = {{&S61973[88], 6}, {NULL}};
const Segment SGI185750[] = {{&S73074[30], 6}, {NULL}};
const Segment SGI185751[] = {{&S73074[31], 3}, {NULL}};
const Segment SGI185752[] = {{&S73074[86], 6}, {NULL}};
const Segment SGI185753[] = {{&S61974[34], 18}, {NULL}};
const Segment SGI185754[] = {{&S61974[27], 6}, {NULL}};
const Segment SGI185755[] = {{&S61974[105], 9}, {NULL}};
const Segment SGI185756[] = {{&S61974[78], 6}, {NULL}};
const Segment SGI185757[] = {{&S73076[44], 14}, {NULL}};
const Segment SGI185758[] = {{&S73076[44], 26}, {NULL}};
const Segment SGI185759[] = {{&S73076[30], 6}, {NULL}};
const Segment SGI185760[] = {{&S61975[34], 18}, {NULL}};
const Segment SGI185761[] = {{&S73076[31], 3}, {NULL}};
const Segment SGI185762[] = {{&S61975[27], 6}, {NULL}};
const Segment SGI185763[] = {{&S73076[84], 6}, {NULL}};
const Segment SGI185764[] = {{&S61975[105], 9}, {NULL}};
const Segment SGI185765[] = {{&S61975[78], 6}, {NULL}};
const Segment SGI185766[] = {{&S73077[34], 15}, {NULL}};
const Segment SGI185767[] = {{&S73077[27], 6}, {NULL}};
const Segment SGI185768[] = {{&S61976[34], 18}, {NULL}};
const Segment SGI185769[] = {{&S61976[27], 6}, {NULL}};
const Segment SGI185770[] = {{&S73077[99], 9}, {NULL}};
const Segment SGI185771[] = {{&S73077[72], 6}, {NULL}};
const Segment SGI185772[] = {{&S61976[113], 9}, {NULL}};
const Segment SGI185773[] = {{&S61976[86], 6}, {NULL}};
const Segment SGI185774[] = {{&S61977[34], 18}, {NULL}};
const Segment SGI185775[] = {{&S61977[27], 6}, {NULL}};
const Segment SGI185776[] = {{&S61977[113], 9}, {NULL}};
const Segment SGI185777[] = {{&S61977[86], 6}, {NULL}};
const Segment SGI185778[] = {{&S73079[45], 7}, {NULL}};
const Segment SGI185779[] = {{&S73079[30], 6}, {NULL}};
const Segment SGI185780[] = {{&S73079[31], 3}, {NULL}};
const Segment SGI185781[] = {{&S73079[83], 5}, {NULL}};
const Segment SGI185782[] = {{&S73079[141], 13}, {NULL}};
const Segment SGI185783[] = {{&S61978[34], 15}, {NULL}};
const Segment SGI185784[] = {{&S61978[27], 6}, {NULL}};
const Segment SGI185785[] = {{&S61978[110], 9}, {NULL}};
const Segment SGI185786[] = {{&S61978[83], 6}, {NULL}};
const Segment SGI185787[] = {{&S73080[45], 8}, {NULL}};
const Segment SGI185788[] = {{&S73080[30], 6}, {NULL}};
const Segment SGI185789[] = {{&S73080[31], 3}, {NULL}};
const Segment SGI185790[] = {{&S73080[83], 5}, {NULL}};
const Segment SGI185791[] = {{&S61979[34], 8}, {NULL}};
const Segment SGI185792[] = {{&S73080[141], 13}, {NULL}};
const Segment SGI185793[] = {{&S61979[27], 6}, {NULL}};
const Segment SGI185794[] = {{&S61979[103], 9}, {NULL}};
const Segment SGI185795[] = {{&S61979[76], 6}, {NULL}};
const Segment SGI185796[] = {{&S73081[44], 17}, {NULL}};
const Segment SGI185797[] = {{&S73081[44], 29}, {NULL}};
const Segment SGI185798[] = {{&S73081[30], 6}, {NULL}};
const Segment SGI185799[] = {{&S73081[31], 3}, {NULL}};
const Segment SGI185800[] = {{&S61980[34], 18}, {NULL}};
const Segment SGI185801[] = {{&S73081[87], 6}, {NULL}};
const Segment SGI185802[] = {{&S61980[27], 6}, {NULL}};
const Segment SGI185803[] = {{&S61980[113], 9}, {NULL}};
const Segment SGI185804[] = {{&S61980[86], 6}, {NULL}};
const Segment SGI185805[] = {{&S73082[44], 18}, {NULL}};
const Segment SGI185806[] = {{&S73082[44], 30}, {NULL}};
const Segment SGI185807[] = {{&S73082[30], 6}, {NULL}};
const Segment SGI185808[] = {{&S73082[31], 3}, {NULL}};
const Segment SGI185809[] = {{&S73082[88], 6}, {NULL}};
const Segment SGI185810[] = {{&S73083[59], 4}, {NULL}};
const Segment SGI185811[] = {{&S73083[59], 16}, {NULL}};
const Segment SGI185812[] = {{&S73083[45], 6}, {NULL}};
const Segment SGI185813[] = {{&S73083[46], 3}, {NULL}};
const Segment SGI185814[] = {{&S73083[89], 6}, {NULL}};
const Segment SGI185815[] = {{&S73083[0], 14}, {NULL}};
const Segment SGI185816[] = {{&S73084[59], 6}, {NULL}};
const Segment SGI185817[] = {{&S73084[59], 18}, {NULL}};
const Segment SGI185818[] = {{&S73084[45], 6}, {NULL}};
const Segment SGI185819[] = {{&S73084[46], 3}, {NULL}};
const Segment SGI185820[] = {{&S73084[91], 6}, {NULL}};
const Segment SGI185821[] = {{&S73084[0], 14}, {NULL}};
const Segment SGI185822[] = {{&S73085[59], 8}, {NULL}};
const Segment SGI185823[] = {{&S73085[59], 20}, {NULL}};
const Segment SGI185824[] = {{&S73085[45], 6}, {NULL}};
const Segment SGI185825[] = {{&S73085[46], 3}, {NULL}};
const Segment SGI185826[] = {{&S73085[93], 6}, {NULL}};
const Segment SGI185827[] = {{&S73085[0], 14}, {NULL}};
const Segment SGI185828[] = {{&S73086[59], 11}, {NULL}};
const Segment SGI185829[] = {{&S73086[59], 23}, {NULL}};
const Segment SGI185830[] = {{&S73086[45], 6}, {NULL}};
const Segment SGI185831[] = {{&S73086[46], 3}, {NULL}};
const Segment SGI185832[] = {{&S73086[96], 6}, {NULL}};
const Segment SGI185833[] = {{&S73086[0], 14}, {NULL}};
const Segment SGI185834[] = {{&S66291[34], 10}, {NULL}};
const Segment SGI185835[] = {{&S66291[27], 6}, {NULL}};
const Segment SGI185836[] = {{&S73087[59], 11}, {NULL}};
const Segment SGI185837[] = {{&S73087[59], 23}, {NULL}};
const Segment SGI185838[] = {{&S73087[45], 6}, {NULL}};
const Segment SGI185839[] = {{&S66291[102], 9}, {NULL}};
const Segment SGI185840[] = {{&S66291[75], 6}, {NULL}};
const Segment SGI185841[] = {{&S73087[46], 3}, {NULL}};
const Segment SGI185842[] = {{&S73087[96], 6}, {NULL}};
const Segment SGI185843[] = {{&S73087[0], 14}, {NULL}};
const Segment SGI185844[] = {{&S73108[34], 11}, {NULL}};
const Segment SGI185845[] = {{&S73108[27], 6}, {NULL}};
const Segment SGI185846[] = {{&S73108[98], 9}, {NULL}};
const Segment SGI185847[] = {{&S73108[71], 6}, {NULL}};
const Segment SGI185848[] = {{&S66318[34], 11}, {NULL}};
const Segment SGI185849[] = {{&S66318[27], 6}, {NULL}};
const Segment SGI185850[] = {{&S66318[98], 9}, {NULL}};
const Segment SGI185851[] = {{&S66318[71], 6}, {NULL}};
const Segment SGI185852[] = {{&S66318[133], 8}, {NULL}};
const Segment SGI185853[] = {{&S66320[34], 10}, {NULL}};
const Segment SGI185854[] = {{&S66320[27], 6}, {NULL}};
const Segment SGI185855[] = {{&S66320[97], 9}, {NULL}};
const Segment SGI185856[] = {{&S66320[70], 6}, {NULL}};
const Segment SGI185857[] = {{&S66320[132], 8}, {NULL}};
const Segment SGI185858[] = {{&S73121[44], 11}, {NULL}};
const Segment SGI185859[] = {{&S73121[44], 23}, {NULL}};
const Segment SGI185860[] = {{&S73121[30], 6}, {NULL}};
const Segment SGI185861[] = {{&S73121[31], 3}, {NULL}};
const Segment SGI185862[] = {{&S73121[81], 6}, {NULL}};
const Segment SGI185863[] = {{&S66322[34], 10}, {NULL}};
const Segment SGI185864[] = {{&S66322[27], 6}, {NULL}};
const Segment SGI185865[] = {{&S66322[97], 9}, {NULL}};
const Segment SGI185866[] = {{&S66322[70], 6}, {NULL}};
const Segment SGI185867[] = {{&S66322[131], 8}, {NULL}};
const Segment SGI185868[] = {{&S73124[44], 41}, {NULL}};
const Segment SGI185869[] = {{&S73124[44], 63}, {NULL}};
const Segment SGI185870[] = {{&S73124[30], 6}, {NULL}};
const Segment SGI185871[] = {{&S73124[31], 3}, {NULL}};
const Segment SGI185872[] = {{&S73124[121], 6}, {NULL}};
const Segment SGI185873[] = {{&S73125[44], 20}, {NULL}};
const Segment SGI185874[] = {{&S73125[44], 42}, {NULL}};
const Segment SGI185875[] = {{&S73125[30], 6}, {NULL}};
const Segment SGI185876[] = {{&S73125[31], 3}, {NULL}};
const Segment SGI185877[] = {{&S73125[100], 6}, {NULL}};
const Segment SGI185878[] = {{&S73126[44], 20}, {NULL}};
const Segment SGI185879[] = {{&S73126[44], 42}, {NULL}};
const Segment SGI185880[] = {{&S73126[30], 6}, {NULL}};
const Segment SGI185881[] = {{&S73126[31], 3}, {NULL}};
const Segment SGI185882[] = {{&S73126[100], 6}, {NULL}};
const Segment SGI185883[] = {{&S73127[34], 10}, {NULL}};
const Segment SGI185884[] = {{&S73127[27], 6}, {NULL}};
const Segment SGI185885[] = {{&S73127[97], 9}, {NULL}};
const Segment SGI185886[] = {{&S73127[70], 6}, {NULL}};
const Segment SGI185887[] = {{&S51309[44], 12}, {NULL}};
const Segment SGI185888[] = {{&S51309[44], 23}, {NULL}};
const Segment SGI185889[] = {{&S51309[30], 6}, {NULL}};
const Segment SGI185890[] = {{&S73153[34], 10}, {NULL}};
const Segment SGI185891[] = {{&S73153[27], 6}, {NULL}};
const Segment SGI185892[] = {{&S51309[134], 3}, {NULL}};
const Segment SGI185893[] = {{&S51309[99], 6}, {NULL}};
const Segment SGI185894[] = {{&S73153[97], 9}, {NULL}};
const Segment SGI185895[] = {{&S51309[69], 8}, {NULL}};
const Segment SGI185896[] = {{&S73153[70], 6}, {NULL}};
const Segment SGI185897[] = {{&S51310[44], 28}, {NULL}};
const Segment SGI185898[] = {{&S51310[44], 39}, {NULL}};
const Segment SGI185899[] = {{&S51310[30], 6}, {NULL}};
const Segment SGI185900[] = {{&S73154[44], 8}, {NULL}};
const Segment SGI185901[] = {{&S73154[44], 20}, {NULL}};
const Segment SGI185902[] = {{&S73154[30], 6}, {NULL}};
const Segment SGI185903[] = {{&S51310[150], 3}, {NULL}};
const Segment SGI185904[] = {{&S51310[115], 6}, {NULL}};
const Segment SGI185905[] = {{&S73154[31], 3}, {NULL}};
const Segment SGI185906[] = {{&S51310[85], 8}, {NULL}};
const Segment SGI185907[] = {{&S73154[78], 6}, {NULL}};
const Segment SGI185908[] = {{&S51311[44], 28}, {NULL}};
const Segment SGI185909[] = {{&S51311[44], 40}, {NULL}};
const Segment SGI185910[] = {{&S51311[30], 6}, {NULL}};
const Segment SGI185911[] = {{&S73155[44], 12}, {NULL}};
const Segment SGI185912[] = {{&S73155[44], 24}, {NULL}};
const Segment SGI185913[] = {{&S73155[30], 6}, {NULL}};
const Segment SGI185914[] = {{&S51311[153], 3}, {NULL}};
const Segment SGI185915[] = {{&S51311[118], 6}, {NULL}};
const Segment SGI185916[] = {{&S73155[31], 3}, {NULL}};
const Segment SGI185917[] = {{&S51311[86], 8}, {NULL}};
const Segment SGI185918[] = {{&S73155[82], 6}, {NULL}};
const Segment SGI185919[] = {{&S66360[44], 10}, {NULL}};
const Segment SGI185920[] = {{&S66360[44], 22}, {NULL}};
const Segment SGI185921[] = {{&S66360[30], 6}, {NULL}};
const Segment SGI185922[] = {{&S66360[144], 3}, {NULL}};
const Segment SGI185923[] = {{&S66360[109], 6}, {NULL}};
const Segment SGI185924[] = {{&S66360[68], 8}, {NULL}};
const Segment SGI185925[] = {{&S73156[44], 9}, {NULL}};
const Segment SGI185926[] = {{&S73156[44], 22}, {NULL}};
const Segment SGI185927[] = {{&S73156[30], 6}, {NULL}};
const Segment SGI185928[] = {{&S73156[31], 3}, {NULL}};
const Segment SGI185929[] = {{&S73156[80], 6}, {NULL}};
const Segment SGI185930[] = {{&S51313[44], 12}, {NULL}};
const Segment SGI185931[] = {{&S51313[44], 24}, {NULL}};
const Segment SGI185932[] = {{&S51313[30], 6}, {NULL}};
const Segment SGI185933[] = {{&S51313[160], 3}, {NULL}};
const Segment SGI185934[] = {{&S51313[125], 6}, {NULL}};
const Segment SGI185935[] = {{&S51313[70], 8}, {NULL}};
const Segment SGI185936[] = {{&S73160[34], 10}, {NULL}};
const Segment SGI185937[] = {{&S73160[27], 6}, {NULL}};
const Segment SGI185938[] = {{&S73160[97], 9}, {NULL}};
const Segment SGI185939[] = {{&S73160[70], 6}, {NULL}};
const Segment SGI185940[] = {{&S51327[41], 18}, {NULL}};
const Segment SGI185941[] = {{&S51327[41], 29}, {NULL}};
const Segment SGI185942[] = {{&S51327[27], 6}, {NULL}};
const Segment SGI185943[] = {{&S51327[119], 3}, {NULL}};
const Segment SGI185944[] = {{&S51327[123], 9}, {NULL}};
const Segment SGI185945[] = {{&S51327[84], 6}, {NULL}};
const Segment SGI185946[] = {{&S62101[34], 11}, {NULL}};
const Segment SGI185947[] = {{&S62101[27], 6}, {NULL}};
const Segment SGI185948[] = {{&S62101[98], 9}, {NULL}};
const Segment SGI185949[] = {{&S62101[71], 6}, {NULL}};
const Segment SGI185950[] = {{&S73181[44], 7}, {NULL}};
const Segment SGI185951[] = {{&S73181[44], 19}, {NULL}};
const Segment SGI185952[] = {{&S73181[30], 6}, {NULL}};
const Segment SGI185953[] = {{&S73181[31], 3}, {NULL}};
const Segment SGI185954[] = {{&S73181[77], 6}, {NULL}};
const Segment SGI185955[] = {{&S62102[34], 10}, {NULL}};
const Segment SGI185956[] = {{&S62102[27], 6}, {NULL}};
const Segment SGI185957[] = {{&S62102[97], 9}, {NULL}};
const Segment SGI185958[] = {{&S62102[70], 6}, {NULL}};
const Segment SGI185959[] = {{&S62103[34], 11}, {NULL}};
const Segment SGI185960[] = {{&S62103[27], 6}, {NULL}};
const Segment SGI185961[] = {{&S62103[98], 9}, {NULL}};
const Segment SGI185962[] = {{&S62103[71], 6}, {NULL}};
const Segment SGI185963[] = {{&S73185[44], 7}, {NULL}};
const Segment SGI185964[] = {{&S73185[44], 19}, {NULL}};
const Segment SGI185965[] = {{&S73185[30], 6}, {NULL}};
const Segment SGI185966[] = {{&S73185[31], 3}, {NULL}};
const Segment SGI185967[] = {{&S73185[77], 6}, {NULL}};
const Segment SGI185968[] = {{&S62104[34], 9}, {NULL}};
const Segment SGI185969[] = {{&S62104[27], 6}, {NULL}};
const Segment SGI185970[] = {{&S62104[96], 9}, {NULL}};
const Segment SGI185971[] = {{&S62104[69], 6}, {NULL}};
const Segment SGI185972[] = {{&S73186[44], 7}, {NULL}};
const Segment SGI185973[] = {{&S73186[44], 19}, {NULL}};
const Segment SGI185974[] = {{&S73186[30], 6}, {NULL}};
const Segment SGI185975[] = {{&S73186[31], 3}, {NULL}};
const Segment SGI185976[] = {{&S73186[77], 6}, {NULL}};
const Segment SGI185977[] = {{&S73187[44], 7}, {NULL}};
const Segment SGI185978[] = {{&S73187[44], 19}, {NULL}};
const Segment SGI185979[] = {{&S73187[30], 6}, {NULL}};
const Segment SGI185980[] = {{&S73187[31], 3}, {NULL}};
const Segment SGI185981[] = {{&S73187[77], 6}, {NULL}};
const Segment SGI185982[] = {{&S73188[44], 7}, {NULL}};
const Segment SGI185983[] = {{&S73188[44], 21}, {NULL}};
const Segment SGI185984[] = {{&S73188[30], 6}, {NULL}};
const Segment SGI185985[] = {{&S73188[31], 3}, {NULL}};
const Segment SGI185986[] = {{&S73188[79], 6}, {NULL}};
const Segment SGI185987[] = {{&S73189[38], 7}, {NULL}};
const Segment SGI185988[] = {{&S73189[31], 6}, {NULL}};
const Segment SGI185989[] = {{&S64298[6], 4}, {NULL}};
const Segment SGI185990[] = {{&S64298[74], 5}, {NULL}};
const Segment SGI185991[] = {{&S62115[34], 7}, {NULL}};
const Segment SGI185992[] = {{&S62115[27], 6}, {NULL}};
const Segment SGI185993[] = {{&S62115[98], 9}, {NULL}};
const Segment SGI185994[] = {{&S62115[71], 6}, {NULL}};
const Segment SGI185995[] = {{&S66415[34], 15}, {NULL}};
const Segment SGI185996[] = {{&S66415[27], 6}, {NULL}};
const Segment SGI185997[] = {{&S73199[44], 9}, {NULL}};
const Segment SGI185998[] = {{&S73199[44], 21}, {NULL}};
const Segment SGI185999[] = {{&S73199[30], 6}, {NULL}};
const Segment SGI186000[] = {{&S66415[102], 9}, {NULL}};
const Segment SGI186001[] = {{&S66415[75], 6}, {NULL}};
const Segment SGI186002[] = {{&S73199[31], 3}, {NULL}};
const Segment SGI186003[] = {{&S73199[79], 6}, {NULL}};
const Segment SGI186004[] = {{&S60541[44], 9}, {NULL}};
const Segment SGI186005[] = {{&S60541[44], 21}, {NULL}};
const Segment SGI186006[] = {{&S60541[30], 6}, {NULL}};
const Segment SGI186007[] = {{&S60541[132], 3}, {NULL}};
const Segment SGI186008[] = {{&S60541[97], 6}, {NULL}};
const Segment SGI186009[] = {{&S60541[67], 8}, {NULL}};
const Segment SGI186010[] = {{&S66423[44], 11}, {NULL}};
const Segment SGI186011[] = {{&S66423[44], 23}, {NULL}};
const Segment SGI186012[] = {{&S66423[30], 6}, {NULL}};
const Segment SGI186013[] = {{&S66423[116], 3}, {NULL}};
const Segment SGI186014[] = {{&S66423[81], 6}, {NULL}};
const Segment SGI186015[] = {{&S66424[44], 16}, {NULL}};
const Segment SGI186016[] = {{&S66424[44], 28}, {NULL}};
const Segment SGI186017[] = {{&S66424[30], 6}, {NULL}};
const Segment SGI186018[] = {{&S66424[121], 3}, {NULL}};
const Segment SGI186019[] = {{&S66424[86], 6}, {NULL}};
const Segment SGI186020[] = {{&S66425[44], 5}, {NULL}};
const Segment SGI186021[] = {{&S66425[44], 17}, {NULL}};
const Segment SGI186022[] = {{&S66425[30], 6}, {NULL}};
const Segment SGI186023[] = {{&S66425[110], 3}, {NULL}};
const Segment SGI186024[] = {{&S66425[75], 6}, {NULL}};
const Segment SGI186025[] = {{&S73237[34], 7}, {NULL}};
const Segment SGI186026[] = {{&S73237[27], 6}, {NULL}};
const Segment SGI186027[] = {{&S73237[98], 9}, {NULL}};
const Segment SGI186028[] = {{&S73237[71], 6}, {NULL}};
const Segment SGI186029[] = {{&S73245[34], 10}, {NULL}};
const Segment SGI186030[] = {{&S73245[27], 6}, {NULL}};
const Segment SGI186031[] = {{&S73245[97], 9}, {NULL}};
const Segment SGI186032[] = {{&S73245[70], 6}, {NULL}};
const Segment SGI186033[] = {{&S73246[34], 11}, {NULL}};
const Segment SGI186034[] = {{&S73246[27], 6}, {NULL}};
const Segment SGI186035[] = {{&S62179[34], 7}, {NULL}};
const Segment SGI186036[] = {{&S62179[27], 6}, {NULL}};
const Segment SGI186037[] = {{&S73246[98], 9}, {NULL}};
const Segment SGI186038[] = {{&S73246[71], 6}, {NULL}};
const Segment SGI186039[] = {{&S62179[98], 9}, {NULL}};
const Segment SGI186040[] = {{&S62179[71], 6}, {NULL}};
const Segment SGI186041[] = {{&S73247[44], 10}, {NULL}};
const Segment SGI186042[] = {{&S73247[44], 22}, {NULL}};
const Segment SGI186043[] = {{&S73247[30], 6}, {NULL}};
const Segment SGI186044[] = {{&S73247[80], 6}, {NULL}};
const Segment SGI186045[] = {{&S73247[107], 13}, {NULL}};
const Segment SGI186046[] = {{&S66468[44], 12}, {NULL}};
const Segment SGI186047[] = {{&S66468[44], 24}, {NULL}};
const Segment SGI186048[] = {{&S66468[30], 6}, {NULL}};
const Segment SGI186049[] = {{&S66468[117], 3}, {NULL}};
const Segment SGI186050[] = {{&S66468[82], 6}, {NULL}};
const Segment SGI186051[] = {{&S66469[38], 12}, {NULL}};
const Segment SGI186052[] = {{&S66469[31], 6}, {NULL}};
const Segment SGI186053[] = {{&S66470[44], 15}, {NULL}};
const Segment SGI186054[] = {{&S66470[44], 27}, {NULL}};
const Segment SGI186055[] = {{&S66470[30], 6}, {NULL}};
const Segment SGI186056[] = {{&S66470[120], 3}, {NULL}};
const Segment SGI186057[] = {{&S66470[85], 6}, {NULL}};
const Segment SGI186058[] = {{&S73252[45], 10}, {NULL}};
const Segment SGI186059[] = {{&S73252[30], 6}, {NULL}};
const Segment SGI186060[] = {{&S73252[31], 3}, {NULL}};
const Segment SGI186061[] = {{&S73252[81], 5}, {NULL}};
const Segment SGI186062[] = {{&S73252[139], 13}, {NULL}};
const Segment SGI186063[] = {{&S73263[34], 15}, {NULL}};
const Segment SGI186064[] = {{&S73263[27], 6}, {NULL}};
const Segment SGI186065[] = {{&S73263[108], 9}, {NULL}};
const Segment SGI186066[] = {{&S73263[81], 6}, {NULL}};
const Segment SGI186067[] = {{&S66500[44], 6}, {NULL}};
const Segment SGI186068[] = {{&S66500[44], 18}, {NULL}};
const Segment SGI186069[] = {{&S66500[30], 6}, {NULL}};
const Segment SGI186070[] = {{&S66500[111], 3}, {NULL}};
const Segment SGI186071[] = {{&S66500[76], 6}, {NULL}};
const Segment SGI186072[] = {{&S66501[38], 6}, {NULL}};
const Segment SGI186073[] = {{&S66501[31], 6}, {NULL}};
const Segment SGI186074[] = {{&S66503[44], 7}, {NULL}};
const Segment SGI186075[] = {{&S66503[44], 19}, {NULL}};
const Segment SGI186076[] = {{&S66503[30], 6}, {NULL}};
const Segment SGI186077[] = {{&S73282[44], 16}, {NULL}};
const Segment SGI186078[] = {{&S73282[44], 28}, {NULL}};
const Segment SGI186079[] = {{&S66503[112], 3}, {NULL}};
const Segment SGI186080[] = {{&S73282[30], 6}, {NULL}};
const Segment SGI186081[] = {{&S66503[77], 6}, {NULL}};
const Segment SGI186082[] = {{&S73282[31], 3}, {NULL}};
const Segment SGI186083[] = {{&S73282[86], 6}, {NULL}};
const Segment SGI186084[] = {{&S66504[38], 7}, {NULL}};
const Segment SGI186085[] = {{&S66504[31], 6}, {NULL}};
const Segment SGI186086[] = {{&S73283[44], 16}, {NULL}};
const Segment SGI186087[] = {{&S73283[44], 28}, {NULL}};
const Segment SGI186088[] = {{&S73283[30], 6}, {NULL}};
const Segment SGI186089[] = {{&S73283[31], 3}, {NULL}};
const Segment SGI186090[] = {{&S73283[86], 6}, {NULL}};
const Segment SGI186091[] = {{&S73284[44], 7}, {NULL}};
const Segment SGI186092[] = {{&S73284[44], 19}, {NULL}};
const Segment SGI186093[] = {{&S73284[30], 6}, {NULL}};
const Segment SGI186094[] = {{&S73284[31], 3}, {NULL}};
const Segment SGI186095[] = {{&S73284[77], 6}, {NULL}};
const Segment SGI186096[] = {{&S66508[44], 7}, {NULL}};
const Segment SGI186097[] = {{&S66508[44], 19}, {NULL}};
const Segment SGI186098[] = {{&S66508[30], 6}, {NULL}};
const Segment SGI186099[] = {{&S66508[112], 3}, {NULL}};
const Segment SGI186100[] = {{&S66508[77], 6}, {NULL}};
const Segment SGI186101[] = {{&S73285[38], 7}, {NULL}};
const Segment SGI186102[] = {{&S73285[31], 6}, {NULL}};
const Segment SGI186103[] = {{&S66509[38], 7}, {NULL}};
const Segment SGI186104[] = {{&S66509[31], 6}, {NULL}};
const Segment SGI186105[] = {{&S62208[44], 5}, {NULL}};
const Segment SGI186106[] = {{&S62208[44], 17}, {NULL}};
const Segment SGI186107[] = {{&S62208[30], 6}, {NULL}};
const Segment SGI186108[] = {{&S62208[110], 3}, {NULL}};
const Segment SGI186109[] = {{&S62208[75], 6}, {NULL}};
const Segment SGI186110[] = {{&S66510[34], 15}, {NULL}};
const Segment SGI186111[] = {{&S66510[27], 6}, {NULL}};
const Segment SGI186112[] = {{&S66510[102], 9}, {NULL}};
const Segment SGI186113[] = {{&S66510[75], 6}, {NULL}};
const Segment SGI186114[] = {{&S73309[34], 10}, {NULL}};
const Segment SGI186115[] = {{&S73309[27], 6}, {NULL}};
const Segment SGI186116[] = {{&S73309[97], 9}, {NULL}};
const Segment SGI186117[] = {{&S73309[70], 6}, {NULL}};
const Segment SGI186118[] = {{&S62230[44], 10}, {NULL}};
const Segment SGI186119[] = {{&S62230[44], 22}, {NULL}};
const Segment SGI186120[] = {{&S62230[30], 6}, {NULL}};
const Segment SGI186121[] = {{&S62230[115], 3}, {NULL}};
const Segment SGI186122[] = {{&S62230[80], 6}, {NULL}};
const Segment SGI186123[] = {{&S66530[44], 17}, {NULL}};
const Segment SGI186124[] = {{&S66530[44], 29}, {NULL}};
const Segment SGI186125[] = {{&S66530[30], 6}, {NULL}};
const Segment SGI186126[] = {{&S62231[44], 11}, {NULL}};
const Segment SGI186127[] = {{&S66530[122], 3}, {NULL}};
const Segment SGI186128[] = {{&S62231[44], 23}, {NULL}};
const Segment SGI186129[] = {{&S62231[30], 6}, {NULL}};
const Segment SGI186130[] = {{&S66530[87], 6}, {NULL}};
const Segment SGI186131[] = {{&S62231[116], 3}, {NULL}};
const Segment SGI186132[] = {{&S62231[81], 6}, {NULL}};
const Segment SGI186133[] = {{&S62232[38], 10}, {NULL}};
const Segment SGI186134[] = {{&S66531[44], 12}, {NULL}};
const Segment SGI186135[] = {{&S62232[31], 6}, {NULL}};
const Segment SGI186136[] = {{&S66531[44], 24}, {NULL}};
const Segment SGI186137[] = {{&S66531[30], 6}, {NULL}};
const Segment SGI186138[] = {{&S66531[117], 3}, {NULL}};
const Segment SGI186139[] = {{&S73314[34], 10}, {NULL}};
const Segment SGI186140[] = {{&S73314[27], 6}, {NULL}};
const Segment SGI186141[] = {{&S66531[82], 6}, {NULL}};
const Segment SGI186142[] = {{&S73314[97], 9}, {NULL}};
const Segment SGI186143[] = {{&S73314[70], 6}, {NULL}};
const Segment SGI186144[] = {{&S62233[44], 11}, {NULL}};
const Segment SGI186145[] = {{&S62233[44], 23}, {NULL}};
const Segment SGI186146[] = {{&S62233[30], 6}, {NULL}};
const Segment SGI186147[] = {{&S62233[116], 3}, {NULL}};
const Segment SGI186148[] = {{&S62233[81], 6}, {NULL}};
const Segment SGI186149[] = {{&S73315[45], 10}, {NULL}};
const Segment SGI186150[] = {{&S73315[30], 6}, {NULL}};
const Segment SGI186151[] = {{&S73315[31], 3}, {NULL}};
const Segment SGI186152[] = {{&S73315[81], 5}, {NULL}};
const Segment SGI186153[] = {{&S73315[139], 13}, {NULL}};
const Segment SGI186154[] = {{&S73318[34], 11}, {NULL}};
const Segment SGI186155[] = {{&S73318[27], 6}, {NULL}};
const Segment SGI186156[] = {{&S73318[98], 9}, {NULL}};
const Segment SGI186157[] = {{&S73318[71], 6}, {NULL}};
const Segment SGI186158[] = {{&S73319[27], 6}, {NULL}};
const Segment SGI186159[] = {{&S73319[99], 9}, {NULL}};
const Segment SGI186160[] = {{&S73319[72], 6}, {NULL}};
const Segment SGI186161[] = {{&S73320[34], 10}, {NULL}};
const Segment SGI186162[] = {{&S73320[27], 6}, {NULL}};
const Segment SGI186163[] = {{&S73320[97], 9}, {NULL}};
const Segment SGI186164[] = {{&S73320[70], 6}, {NULL}};
const Segment SGI186165[] = {{&S73328[52], 10}, {NULL}};
const Segment SGI186166[] = {{&S73328[52], 22}, {NULL}};
const Segment SGI186167[] = {{&S73328[38], 6}, {NULL}};
const Segment SGI186168[] = {{&S73328[88], 5}, {NULL}};
const Segment SGI186169[] = {{&S73328[0], 14}, {NULL}};
const Segment SGI186170[] = {{&S66563[44], 10}, {NULL}};
const Segment SGI186171[] = {{&S66563[44], 17}, {NULL}};
const Segment SGI186172[] = {{&S66563[30], 6}, {NULL}};
const Segment SGI186173[] = {{&S66563[110], 3}, {NULL}};
const Segment SGI186174[] = {{&S66563[75], 6}, {NULL}};
const Segment SGI186175[] = {{&S66564[44], 8}, {NULL}};
const Segment SGI186176[] = {{&S66564[44], 20}, {NULL}};
const Segment SGI186177[] = {{&S66564[30], 6}, {NULL}};
const Segment SGI186178[] = {{&S73342[44], 11}, {NULL}};
const Segment SGI186179[] = {{&S66564[113], 3}, {NULL}};
const Segment SGI186180[] = {{&S73342[44], 23}, {NULL}};
const Segment SGI186181[] = {{&S73342[30], 6}, {NULL}};
const Segment SGI186182[] = {{&S66564[78], 6}, {NULL}};
const Segment SGI186183[] = {{&S73342[31], 3}, {NULL}};
const Segment SGI186184[] = {{&S73342[81], 6}, {NULL}};
const Segment SGI186185[] = {{&S66565[44], 8}, {NULL}};
const Segment SGI186186[] = {{&S66565[44], 20}, {NULL}};
const Segment SGI186187[] = {{&S66565[30], 6}, {NULL}};
const Segment SGI186188[] = {{&S73343[38], 11}, {NULL}};
const Segment SGI186189[] = {{&S73343[31], 6}, {NULL}};
const Segment SGI186190[] = {{&S66565[113], 3}, {NULL}};
const Segment SGI186191[] = {{&S66565[78], 6}, {NULL}};
const Segment SGI186192[] = {{&S62271[34], 18}, {NULL}};
const Segment SGI186193[] = {{&S62271[27], 6}, {NULL}};
const Segment SGI186194[] = {{&S62271[105], 9}, {NULL}};
const Segment SGI186195[] = {{&S62271[78], 6}, {NULL}};
const Segment SGI186196[] = {{&S66566[44], 8}, {NULL}};
const Segment SGI186197[] = {{&S66566[44], 20}, {NULL}};
const Segment SGI186198[] = {{&S66566[30], 6}, {NULL}};
const Segment SGI186199[] = {{&S66566[113], 3}, {NULL}};
const Segment SGI186200[] = {{&S66566[78], 6}, {NULL}};
const Segment SGI186201[] = {{&S62272[34], 18}, {NULL}};
const Segment SGI186202[] = {{&S62272[27], 6}, {NULL}};
const Segment SGI186203[] = {{&S62272[105], 9}, {NULL}};
const Segment SGI186204[] = {{&S62272[78], 6}, {NULL}};
const Segment SGI186205[] = {{&S66567[38], 8}, {NULL}};
const Segment SGI186206[] = {{&S66567[31], 6}, {NULL}};
const Segment SGI186207[] = {{&S66568[44], 7}, {NULL}};
const Segment SGI186208[] = {{&S66568[44], 19}, {NULL}};
const Segment SGI186209[] = {{&S66568[30], 6}, {NULL}};
const Segment SGI186210[] = {{&S66568[112], 3}, {NULL}};
const Segment SGI186211[] = {{&S66568[77], 6}, {NULL}};
const Segment SGI186212[] = {{&S73351[44], 7}, {NULL}};
const Segment SGI186213[] = {{&S73351[44], 19}, {NULL}};
const Segment SGI186214[] = {{&S73351[30], 6}, {NULL}};
const Segment SGI186215[] = {{&S73351[31], 3}, {NULL}};
const Segment SGI186216[] = {{&S73351[77], 6}, {NULL}};
const Segment SGI186217[] = {{&S73356[34], 8}, {NULL}};
const Segment SGI186218[] = {{&S73356[27], 6}, {NULL}};
const Segment SGI186219[] = {{&S73356[107], 9}, {NULL}};
const Segment SGI186220[] = {{&S73356[80], 6}, {NULL}};
const Segment SGI186221[] = {{&S73358[34], 6}, {NULL}};
const Segment SGI186222[] = {{&S73358[27], 6}, {NULL}};
const Segment SGI186223[] = {{&S73358[93], 9}, {NULL}};
const Segment SGI186224[] = {{&S73358[66], 6}, {NULL}};
const Segment SGI186225[] = {{&S62291[44], 10}, {NULL}};
const Segment SGI186226[] = {{&S62291[44], 22}, {NULL}};
const Segment SGI186227[] = {{&S62291[30], 6}, {NULL}};
const Segment SGI186228[] = {{&S62291[115], 3}, {NULL}};
const Segment SGI186229[] = {{&S62291[80], 6}, {NULL}};
const Segment SGI186230[] = {{&S62291[139], 10}, {NULL}};
const Segment SGI186231[] = {{&S62292[44], 10}, {NULL}};
const Segment SGI186232[] = {{&S62292[44], 22}, {NULL}};
const Segment SGI186233[] = {{&S62292[30], 6}, {NULL}};
const Segment SGI186234[] = {{&S62292[115], 3}, {NULL}};
const Segment SGI186235[] = {{&S62292[80], 6}, {NULL}};
const Segment SGI186236[] = {{&S62292[139], 10}, {NULL}};
const Segment SGI186237[] = {{&S62293[44], 10}, {NULL}};
const Segment SGI186238[] = {{&S62293[44], 22}, {NULL}};
const Segment SGI186239[] = {{&S62293[30], 6}, {NULL}};
const Segment SGI186240[] = {{&S62293[115], 3}, {NULL}};
const Segment SGI186241[] = {{&S62293[80], 6}, {NULL}};
const Segment SGI186242[] = {{&S62293[139], 10}, {NULL}};
const Segment SGI186243[] = {{&S62294[44], 8}, {NULL}};
const Segment SGI186244[] = {{&S62294[44], 20}, {NULL}};
const Segment SGI186245[] = {{&S62294[30], 6}, {NULL}};
const Segment SGI186246[] = {{&S62294[113], 3}, {NULL}};
const Segment SGI186247[] = {{&S62294[78], 6}, {NULL}};
const Segment SGI186248[] = {{&S62295[38], 8}, {NULL}};
const Segment SGI186249[] = {{&S62295[31], 6}, {NULL}};
const Segment SGI186250[] = {{&S62297[34], 10}, {NULL}};
const Segment SGI186251[] = {{&S62297[27], 6}, {NULL}};
const Segment SGI186252[] = {{&S62297[97], 9}, {NULL}};
const Segment SGI186253[] = {{&S62297[70], 6}, {NULL}};
const Segment SGI186254[] = {{&S62298[34], 10}, {NULL}};
const Segment SGI186255[] = {{&S62298[27], 6}, {NULL}};
const Segment SGI186256[] = {{&S62298[97], 9}, {NULL}};
const Segment SGI186257[] = {{&S62298[70], 6}, {NULL}};
const Segment SGI186258[] = {{&S62299[34], 10}, {NULL}};
const Segment SGI186259[] = {{&S62299[27], 6}, {NULL}};
const Segment SGI186260[] = {{&S62299[97], 9}, {NULL}};
const Segment SGI186261[] = {{&S62299[70], 6}, {NULL}};
const Segment SGI186262[] = {{&S62300[34], 9}, {NULL}};
const Segment SGI186263[] = {{&S62300[27], 6}, {NULL}};
const Segment SGI186264[] = {{&S62300[96], 9}, {NULL}};
const Segment SGI186265[] = {{&S62300[69], 6}, {NULL}};
const Segment SGI186266[] = {{&S73383[44], 6}, {NULL}};
const Segment SGI186267[] = {{&S73383[44], 18}, {NULL}};
const Segment SGI186268[] = {{&S73383[30], 6}, {NULL}};
const Segment SGI186269[] = {{&S73383[31], 3}, {NULL}};
const Segment SGI186270[] = {{&S73383[76], 6}, {NULL}};
const Segment SGI186271[] = {{&S73385[44], 13}, {NULL}};
const Segment SGI186272[] = {{&S73385[44], 24}, {NULL}};
const Segment SGI186273[] = {{&S73385[30], 6}, {NULL}};
const Segment SGI186274[] = {{&S73385[31], 3}, {NULL}};
const Segment SGI186275[] = {{&S73385[82], 6}, {NULL}};
const Segment SGI186276[] = {{&S73386[44], 13}, {NULL}};
const Segment SGI186277[] = {{&S73386[44], 24}, {NULL}};
const Segment SGI186278[] = {{&S73386[30], 6}, {NULL}};
const Segment SGI186279[] = {{&S73386[31], 3}, {NULL}};
const Segment SGI186280[] = {{&S73386[82], 6}, {NULL}};
const Segment SGI186281[] = {{&S73387[44], 13}, {NULL}};
const Segment SGI186282[] = {{&S73387[44], 24}, {NULL}};
const Segment SGI186283[] = {{&S73387[30], 6}, {NULL}};
const Segment SGI186284[] = {{&S73387[31], 3}, {NULL}};
const Segment SGI186285[] = {{&S73387[82], 6}, {NULL}};
const Segment SGI186286[] = {{&S62311[34], 10}, {NULL}};
const Segment SGI186287[] = {{&S62311[27], 6}, {NULL}};
const Segment SGI186288[] = {{&S73389[44], 6}, {NULL}};
const Segment SGI186289[] = {{&S62311[97], 9}, {NULL}};
const Segment SGI186290[] = {{&S62311[70], 6}, {NULL}};
const Segment SGI186291[] = {{&S73389[44], 18}, {NULL}};
const Segment SGI186292[] = {{&S73389[30], 6}, {NULL}};
const Segment SGI186293[] = {{&S73389[31], 3}, {NULL}};
const Segment SGI186294[] = {{&S73389[76], 6}, {NULL}};
const Segment SGI186295[] = {{&S62312[34], 11}, {NULL}};
const Segment SGI186296[] = {{&S62312[27], 6}, {NULL}};
const Segment SGI186297[] = {{&S62312[98], 9}, {NULL}};
const Segment SGI186298[] = {{&S62312[71], 6}, {NULL}};
const Segment SGI186299[] = {{&S73390[44], 20}, {NULL}};
const Segment SGI186300[] = {{&S73390[44], 32}, {NULL}};
const Segment SGI186301[] = {{&S73390[30], 6}, {NULL}};
const Segment SGI186302[] = {{&S73390[31], 3}, {NULL}};
const Segment SGI186303[] = {{&S73390[90], 6}, {NULL}};
const Segment SGI186304[] = {{&S66622[44], 10}, {NULL}};
const Segment SGI186305[] = {{&S66622[44], 22}, {NULL}};
const Segment SGI186306[] = {{&S66622[30], 6}, {NULL}};
const Segment SGI186307[] = {{&S66622[115], 3}, {NULL}};
const Segment SGI186308[] = {{&S66622[80], 6}, {NULL}};
const Segment SGI186309[] = {{&S62340[34], 10}, {NULL}};
const Segment SGI186310[] = {{&S62340[27], 6}, {NULL}};
const Segment SGI186311[] = {{&S62340[97], 9}, {NULL}};
const Segment SGI186312[] = {{&S62340[70], 6}, {NULL}};
const Segment SGI186313[] = {{&S73415[34], 9}, {NULL}};
const Segment SGI186314[] = {{&S73415[27], 6}, {NULL}};
const Segment SGI186315[] = {{&S73415[96], 9}, {NULL}};
const Segment SGI186316[] = {{&S73415[69], 6}, {NULL}};
const Segment SGI186317[] = {{&S73416[34], 11}, {NULL}};
const Segment SGI186318[] = {{&S73416[27], 6}, {NULL}};
const Segment SGI186319[] = {{&S73416[98], 9}, {NULL}};
const Segment SGI186320[] = {{&S73416[71], 6}, {NULL}};
const Segment SGI186321[] = {{&S73425[34], 14}, {NULL}};
const Segment SGI186322[] = {{&S73425[27], 6}, {NULL}};
const Segment SGI186323[] = {{&S73425[101], 9}, {NULL}};
const Segment SGI186324[] = {{&S73425[74], 6}, {NULL}};
const Segment SGI186325[] = {{&S73432[44], 12}, {NULL}};
const Segment SGI186326[] = {{&S73432[44], 24}, {NULL}};
const Segment SGI186327[] = {{&S73432[30], 6}, {NULL}};
const Segment SGI186328[] = {{&S73432[31], 3}, {NULL}};
const Segment SGI186329[] = {{&S73432[82], 6}, {NULL}};
const Segment SGI186330[] = {{&S73433[44], 9}, {NULL}};
const Segment SGI186331[] = {{&S73433[44], 21}, {NULL}};
const Segment SGI186332[] = {{&S73433[30], 6}, {NULL}};
const Segment SGI186333[] = {{&S66670[44], 11}, {NULL}};
const Segment SGI186334[] = {{&S66670[44], 23}, {NULL}};
const Segment SGI186335[] = {{&S66670[30], 6}, {NULL}};
const Segment SGI186336[] = {{&S73433[31], 3}, {NULL}};
const Segment SGI186337[] = {{&S73433[79], 6}, {NULL}};
const Segment SGI186338[] = {{&S66670[116], 3}, {NULL}};
const Segment SGI186339[] = {{&S66670[81], 6}, {NULL}};
const Segment SGI186340[] = {{&S62366[34], 7}, {NULL}};
const Segment SGI186341[] = {{&S62366[27], 6}, {NULL}};
const Segment SGI186342[] = {{&S62366[98], 9}, {NULL}};
const Segment SGI186343[] = {{&S62366[71], 6}, {NULL}};
const Segment SGI186344[] = {{&S62367[34], 7}, {NULL}};
const Segment SGI186345[] = {{&S62367[27], 6}, {NULL}};
const Segment SGI186346[] = {{&S62367[98], 9}, {NULL}};
const Segment SGI186347[] = {{&S62367[71], 6}, {NULL}};
const Segment SGI186348[] = {{&S62376[34], 16}, {NULL}};
const Segment SGI186349[] = {{&S62376[27], 6}, {NULL}};
const Segment SGI186350[] = {{&S62376[110], 9}, {NULL}};
const Segment SGI186351[] = {{&S62376[83], 6}, {NULL}};
const Segment SGI186352[] = {{&S66687[30], 6}, {NULL}};
const Segment SGI186353[] = {{&S66687[85], 9}, {NULL}};
const Segment SGI186354[] = {{&S66687[57], 6}, {NULL}};
const Segment SGI186355[] = {{&S62385[34], 9}, {NULL}};
const Segment SGI186356[] = {{&S62385[27], 6}, {NULL}};
const Segment SGI186357[] = {{&S62385[96], 9}, {NULL}};
const Segment SGI186358[] = {{&S62385[69], 6}, {NULL}};
const Segment SGI186359[] = {{&S66693[34], 15}, {NULL}};
const Segment SGI186360[] = {{&S66693[27], 6}, {NULL}};
const Segment SGI186361[] = {{&S66693[102], 9}, {NULL}};
const Segment SGI186362[] = {{&S66693[75], 6}, {NULL}};
const Segment SGI186363[] = {{&S73459[34], 10}, {NULL}};
const Segment SGI186364[] = {{&S73459[27], 6}, {NULL}};
const Segment SGI186365[] = {{&S73459[97], 9}, {NULL}};
const Segment SGI186366[] = {{&S73459[70], 6}, {NULL}};
const Segment SGI186367[] = {{&S73464[34], 14}, {NULL}};
const Segment SGI186368[] = {{&S73464[27], 6}, {NULL}};
const Segment SGI186369[] = {{&S73464[101], 9}, {NULL}};
const Segment SGI186370[] = {{&S73464[74], 6}, {NULL}};
const Segment SGI186371[] = {{&S62400[34], 8}, {NULL}};
const Segment SGI186372[] = {{&S62400[27], 6}, {NULL}};
const Segment SGI186373[] = {{&S66708[44], 9}, {NULL}};
const Segment SGI186374[] = {{&S62400[95], 9}, {NULL}};
const Segment SGI186375[] = {{&S66708[44], 21}, {NULL}};
const Segment SGI186376[] = {{&S62400[68], 6}, {NULL}};
const Segment SGI186377[] = {{&S66708[30], 6}, {NULL}};
const Segment SGI186378[] = {{&S66708[114], 3}, {NULL}};
const Segment SGI186379[] = {{&S66708[79], 6}, {NULL}};
const Segment SGI186380[] = {{&S66709[44], 9}, {NULL}};
const Segment SGI186381[] = {{&S66709[44], 21}, {NULL}};
const Segment SGI186382[] = {{&S66709[30], 6}, {NULL}};
const Segment SGI186383[] = {{&S73470[44], 10}, {NULL}};
const Segment SGI186384[] = {{&S66709[130], 3}, {NULL}};
const Segment SGI186385[] = {{&S73470[44], 22}, {NULL}};
const Segment SGI186386[] = {{&S66709[95], 6}, {NULL}};
const Segment SGI186387[] = {{&S73470[30], 6}, {NULL}};
const Segment SGI186388[] = {{&S66709[67], 8}, {NULL}};
const Segment SGI186389[] = {{&S73470[31], 3}, {NULL}};
const Segment SGI186390[] = {{&S73470[80], 6}, {NULL}};
const Segment SGI186391[] = {{&S73470[150], 3}, {NULL}};
const Segment SGI186392[] = {{&S66711[44], 10}, {NULL}};
const Segment SGI186393[] = {{&S66711[44], 22}, {NULL}};
const Segment SGI186394[] = {{&S66711[30], 6}, {NULL}};
const Segment SGI186395[] = {{&S73471[44], 10}, {NULL}};
const Segment SGI186396[] = {{&S73471[44], 22}, {NULL}};
const Segment SGI186397[] = {{&S73471[30], 6}, {NULL}};
const Segment SGI186398[] = {{&S66711[131], 3}, {NULL}};
const Segment SGI186399[] = {{&S66711[96], 6}, {NULL}};
const Segment SGI186400[] = {{&S73471[31], 3}, {NULL}};
const Segment SGI186401[] = {{&S66711[68], 8}, {NULL}};
const Segment SGI186402[] = {{&S73471[80], 6}, {NULL}};
const Segment SGI186403[] = {{&S73471[150], 3}, {NULL}};
const Segment SGI186404[] = {{&S73472[44], 11}, {NULL}};
const Segment SGI186405[] = {{&S73472[44], 23}, {NULL}};
const Segment SGI186406[] = {{&S73472[30], 6}, {NULL}};
const Segment SGI186407[] = {{&S73472[31], 3}, {NULL}};
const Segment SGI186408[] = {{&S73472[81], 6}, {NULL}};
const Segment SGI186409[] = {{&S73472[151], 3}, {NULL}};
const Segment SGI186410[] = {{&S73473[44], 10}, {NULL}};
const Segment SGI186411[] = {{&S73473[44], 22}, {NULL}};
const Segment SGI186412[] = {{&S73473[30], 6}, {NULL}};
const Segment SGI186413[] = {{&S73473[31], 3}, {NULL}};
const Segment SGI186414[] = {{&S73473[80], 6}, {NULL}};
const Segment SGI186415[] = {{&S73473[150], 3}, {NULL}};
const Segment SGI186416[] = {{&S73479[44], 10}, {NULL}};
const Segment SGI186417[] = {{&S73479[44], 22}, {NULL}};
const Segment SGI186418[] = {{&S73479[30], 6}, {NULL}};
const Segment SGI186419[] = {{&S73479[31], 3}, {NULL}};
const Segment SGI186420[] = {{&S73479[80], 6}, {NULL}};
const Segment SGI186421[] = {{&S73480[34], 7}, {NULL}};
const Segment SGI186422[] = {{&S73480[27], 6}, {NULL}};
const Segment SGI186423[] = {{&S73480[94], 9}, {NULL}};
const Segment SGI186424[] = {{&S73480[67], 6}, {NULL}};
const Segment SGI186425[] = {{&S73481[34], 8}, {NULL}};
const Segment SGI186426[] = {{&S73481[27], 6}, {NULL}};
const Segment SGI186427[] = {{&S73481[95], 9}, {NULL}};
const Segment SGI186428[] = {{&S73481[68], 6}, {NULL}};
const Segment SGI186429[] = {{&S73486[44], 7}, {NULL}};
const Segment SGI186430[] = {{&S73486[44], 19}, {NULL}};
const Segment SGI186431[] = {{&S73486[30], 6}, {NULL}};
const Segment SGI186432[] = {{&S73486[31], 3}, {NULL}};
const Segment SGI186433[] = {{&S73486[77], 6}, {NULL}};
const Segment SGI186434[] = {{&S73487[42], 7}, {NULL}};
const Segment SGI186435[] = {{&S73487[30], 6}, {NULL}};
const Segment SGI186436[] = {{&S73487[31], 3}, {NULL}};
const Segment SGI186437[] = {{&S73487[75], 6}, {NULL}};
const Segment SGI186438[] = {{&S73492[44], 14}, {NULL}};
const Segment SGI186439[] = {{&S73492[44], 26}, {NULL}};
const Segment SGI186440[] = {{&S73492[30], 6}, {NULL}};
const Segment SGI186441[] = {{&S73492[31], 3}, {NULL}};
const Segment SGI186442[] = {{&S73492[84], 6}, {NULL}};
const Segment SGI186443[] = {{&S73497[44], 10}, {NULL}};
const Segment SGI186444[] = {{&S73497[44], 22}, {NULL}};
const Segment SGI186445[] = {{&S73497[30], 6}, {NULL}};
const Segment SGI186446[] = {{&S73497[31], 3}, {NULL}};
const Segment SGI186447[] = {{&S73497[80], 6}, {NULL}};
const Segment SGI186448[] = {{&S73497[140], 11}, {NULL}};
const Segment SGI186449[] = {{&S73498[44], 13}, {NULL}};
const Segment SGI186450[] = {{&S73498[44], 25}, {NULL}};
const Segment SGI186451[] = {{&S51738[44], 10}, {NULL}};
const Segment SGI186452[] = {{&S73498[30], 6}, {NULL}};
const Segment SGI186453[] = {{&S51738[44], 21}, {NULL}};
const Segment SGI186454[] = {{&S51738[30], 6}, {NULL}};
const Segment SGI186455[] = {{&S73498[31], 3}, {NULL}};
const Segment SGI186456[] = {{&S51738[132], 3}, {NULL}};
const Segment SGI186457[] = {{&S73498[83], 6}, {NULL}};
const Segment SGI186458[] = {{&S51738[97], 6}, {NULL}};
const Segment SGI186459[] = {{&S73498[143], 11}, {NULL}};
const Segment SGI186460[] = {{&S51738[67], 8}, {NULL}};
const Segment SGI186461[] = {{&S73499[34], 14}, {NULL}};
const Segment SGI186462[] = {{&S73499[27], 6}, {NULL}};
const Segment SGI186463[] = {{&S73499[101], 9}, {NULL}};
const Segment SGI186464[] = {{&S73499[74], 6}, {NULL}};
const Segment SGI186465[] = {{&S51741[42], 10}, {NULL}};
const Segment SGI186466[] = {{&S51741[30], 6}, {NULL}};
const Segment SGI186467[] = {{&S51741[130], 3}, {NULL}};
const Segment SGI186468[] = {{&S51741[95], 6}, {NULL}};
const Segment SGI186469[] = {{&S51741[65], 8}, {NULL}};
const Segment SGI186470[] = {{&S51742[44], 10}, {NULL}};
const Segment SGI186471[] = {{&S51742[44], 22}, {NULL}};
const Segment SGI186472[] = {{&S51742[30], 6}, {NULL}};
const Segment SGI186473[] = {{&S51742[115], 3}, {NULL}};
const Segment SGI186474[] = {{&S51742[80], 6}, {NULL}};
const Segment SGI186475[] = {{&S51742[133], 8}, {NULL}};
const Segment SGI186476[] = {{&S73503[34], 10}, {NULL}};
const Segment SGI186477[] = {{&S73503[27], 6}, {NULL}};
const Segment SGI186478[] = {{&S73503[97], 9}, {NULL}};
const Segment SGI186479[] = {{&S73503[70], 6}, {NULL}};
const Segment SGI186480[] = {{&S73518[34], 24}, {NULL}};
const Segment SGI186481[] = {{&S73518[27], 6}, {NULL}};
const Segment SGI186482[] = {{&S73518[111], 9}, {NULL}};
const Segment SGI186483[] = {{&S73518[84], 6}, {NULL}};
const Segment SGI186484[] = {{&S73519[34], 23}, {NULL}};
const Segment SGI186485[] = {{&S73519[27], 6}, {NULL}};
const Segment SGI186486[] = {{&S73519[110], 9}, {NULL}};
const Segment SGI186487[] = {{&S73519[83], 6}, {NULL}};
const Segment SGI186488[] = {{&S66785[34], 10}, {NULL}};
const Segment SGI186489[] = {{&S66785[27], 6}, {NULL}};
const Segment SGI186490[] = {{&S66785[97], 9}, {NULL}};
const Segment SGI186491[] = {{&S66785[70], 6}, {NULL}};
const Segment SGI186492[] = {{&S66785[132], 8}, {NULL}};
const Segment SGI186493[] = {{&S66786[34], 10}, {NULL}};
const Segment SGI186494[] = {{&S66786[27], 6}, {NULL}};
const Segment SGI186495[] = {{&S66786[97], 9}, {NULL}};
const Segment SGI186496[] = {{&S66786[70], 6}, {NULL}};
const Segment SGI186497[] = {{&S66786[131], 8}, {NULL}};
const Segment SGI186498[] = {{&S62501[34], 9}, {NULL}};
const Segment SGI186499[] = {{&S62501[27], 6}, {NULL}};
const Segment SGI186500[] = {{&S62501[96], 9}, {NULL}};
const Segment SGI186501[] = {{&S62501[69], 6}, {NULL}};
const Segment SGI186502[] = {{&S62502[44], 7}, {NULL}};
const Segment SGI186503[] = {{&S62502[44], 27}, {NULL}};
const Segment SGI186504[] = {{&S62502[30], 6}, {NULL}};
const Segment SGI186505[] = {{&S62502[120], 3}, {NULL}};
const Segment SGI186506[] = {{&S62502[85], 6}, {NULL}};
const Segment SGI186507[] = {{&S62503[38], 7}, {NULL}};
const Segment SGI186508[] = {{&S62503[31], 6}, {NULL}};
const Segment SGI186509[] = {{&S62517[44], 11}, {NULL}};
const Segment SGI186510[] = {{&S62517[44], 23}, {NULL}};
const Segment SGI186511[] = {{&S62517[30], 6}, {NULL}};
const Segment SGI186512[] = {{&S62517[116], 3}, {NULL}};
const Segment SGI186513[] = {{&S62517[81], 6}, {NULL}};
const Segment SGI186514[] = {{&S62519[44], 13}, {NULL}};
const Segment SGI186515[] = {{&S62519[44], 25}, {NULL}};
const Segment SGI186516[] = {{&S62519[30], 6}, {NULL}};
const Segment SGI186517[] = {{&S62519[118], 3}, {NULL}};
const Segment SGI186518[] = {{&S62519[83], 6}, {NULL}};
const Segment SGI186519[] = {{&S66826[34], 6}, {NULL}};
const Segment SGI186520[] = {{&S66826[27], 6}, {NULL}};
const Segment SGI186521[] = {{&S62531[34], 9}, {NULL}};
const Segment SGI186522[] = {{&S62531[27], 6}, {NULL}};
const Segment SGI186523[] = {{&S66826[93], 9}, {NULL}};
const Segment SGI186524[] = {{&S66826[66], 6}, {NULL}};
const Segment SGI186525[] = {{&S62531[96], 9}, {NULL}};
const Segment SGI186526[] = {{&S62531[69], 6}, {NULL}};
const Segment SGI186527[] = {{&S66827[34], 7}, {NULL}};
const Segment SGI186528[] = {{&S66827[27], 6}, {NULL}};
const Segment SGI186529[] = {{&S66827[94], 9}, {NULL}};
const Segment SGI186530[] = {{&S66827[67], 6}, {NULL}};
const Segment SGI186531[] = {{&S66830[34], 10}, {NULL}};
const Segment SGI186532[] = {{&S66830[27], 6}, {NULL}};
const Segment SGI186533[] = {{&S66830[97], 9}, {NULL}};
const Segment SGI186534[] = {{&S66830[70], 6}, {NULL}};
const Segment SGI186535[] = {{&S66830[131], 8}, {NULL}};
const Segment SGI186536[] = {{&S73583[44], 7}, {NULL}};
const Segment SGI186537[] = {{&S73583[44], 19}, {NULL}};
const Segment SGI186538[] = {{&S73583[30], 6}, {NULL}};
const Segment SGI186539[] = {{&S73583[31], 3}, {NULL}};
const Segment SGI186540[] = {{&S73583[77], 6}, {NULL}};
const Segment SGI186541[] = {{&S73584[38], 7}, {NULL}};
const Segment SGI186542[] = {{&S73584[31], 6}, {NULL}};
const Segment SGI186543[] = {{&S66841[44], 9}, {NULL}};
const Segment SGI186544[] = {{&S66841[44], 20}, {NULL}};
const Segment SGI186545[] = {{&S66841[30], 6}, {NULL}};
const Segment SGI186546[] = {{&S66841[132], 3}, {NULL}};
const Segment SGI186547[] = {{&S66841[97], 6}, {NULL}};
const Segment SGI186548[] = {{&S66841[66], 8}, {NULL}};
const Segment SGI186549[] = {{&S73594[34], 8}, {NULL}};
const Segment SGI186550[] = {{&S73594[27], 6}, {NULL}};
const Segment SGI186551[] = {{&S73594[95], 9}, {NULL}};
const Segment SGI186552[] = {{&S73594[68], 6}, {NULL}};
const Segment SGI186553[] = {{&S73615[44], 16}, {NULL}};
const Segment SGI186554[] = {{&S73615[30], 6}, {NULL}};
const Segment SGI186555[] = {{&S73615[31], 3}, {NULL}};
const Segment SGI186556[] = {{&S73615[113], 6}, {NULL}};
const Segment SGI186557[] = {{&S73617[44], 13}, {NULL}};
const Segment SGI186558[] = {{&S73617[44], 25}, {NULL}};
const Segment SGI186559[] = {{&S73617[30], 6}, {NULL}};
const Segment SGI186560[] = {{&S73617[31], 3}, {NULL}};
const Segment SGI186561[] = {{&S73617[83], 6}, {NULL}};
const Segment SGI186562[] = {{&S73620[44], 6}, {NULL}};
const Segment SGI186563[] = {{&S73620[44], 18}, {NULL}};
const Segment SGI186564[] = {{&S73620[30], 6}, {NULL}};
const Segment SGI186565[] = {{&S73620[31], 3}, {NULL}};
const Segment SGI186566[] = {{&S73620[76], 6}, {NULL}};
const Segment SGI186567[] = {{&S73621[34], 8}, {NULL}};
const Segment SGI186568[] = {{&S73621[27], 6}, {NULL}};
const Segment SGI186569[] = {{&S73621[100], 9}, {NULL}};
const Segment SGI186570[] = {{&S73621[73], 6}, {NULL}};
const Segment SGI186571[] = {{&S62593[44], 10}, {NULL}};
const Segment SGI186572[] = {{&S62593[44], 22}, {NULL}};
const Segment SGI186573[] = {{&S62593[30], 6}, {NULL}};
const Segment SGI186574[] = {{&S62593[131], 3}, {NULL}};
const Segment SGI186575[] = {{&S62593[96], 6}, {NULL}};
const Segment SGI186576[] = {{&S62593[68], 8}, {NULL}};
const Segment SGI186577[] = {{&S62596[44], 10}, {NULL}};
const Segment SGI186578[] = {{&S62596[44], 22}, {NULL}};
const Segment SGI186579[] = {{&S62596[30], 6}, {NULL}};
const Segment SGI186580[] = {{&S62596[115], 3}, {NULL}};
const Segment SGI186581[] = {{&S62596[80], 6}, {NULL}};
const Segment SGI186582[] = {{&S62596[139], 10}, {NULL}};
const Segment SGI186583[] = {{&S62599[44], 11}, {NULL}};
const Segment SGI186584[] = {{&S62599[44], 23}, {NULL}};
const Segment SGI186585[] = {{&S62599[30], 6}, {NULL}};
const Segment SGI186586[] = {{&S62599[116], 3}, {NULL}};
const Segment SGI186587[] = {{&S62599[81], 6}, {NULL}};
const Segment SGI186588[] = {{&S62599[140], 10}, {NULL}};
const Segment SGI186589[] = {{&S62600[44], 11}, {NULL}};
const Segment SGI186590[] = {{&S62600[44], 23}, {NULL}};
const Segment SGI186591[] = {{&S62600[30], 6}, {NULL}};
const Segment SGI186592[] = {{&S62600[116], 3}, {NULL}};
const Segment SGI186593[] = {{&S62600[81], 6}, {NULL}};
const Segment SGI186594[] = {{&S62600[140], 10}, {NULL}};
const Segment SGI186595[] = {{&S62601[44], 18}, {NULL}};
const Segment SGI186596[] = {{&S62601[44], 30}, {NULL}};
const Segment SGI186597[] = {{&S62601[30], 6}, {NULL}};
const Segment SGI186598[] = {{&S62601[123], 3}, {NULL}};
const Segment SGI186599[] = {{&S62601[88], 6}, {NULL}};
const Segment SGI186600[] = {{&S62601[147], 10}, {NULL}};
const Segment SGI186601[] = {{&S73658[44], 8}, {NULL}};
const Segment SGI186602[] = {{&S73658[44], 20}, {NULL}};
const Segment SGI186603[] = {{&S73658[30], 6}, {NULL}};
const Segment SGI186604[] = {{&S73658[31], 3}, {NULL}};
const Segment SGI186605[] = {{&S73658[78], 6}, {NULL}};
const Segment SGI186606[] = {{&S73660[34], 9}, {NULL}};
const Segment SGI186607[] = {{&S73660[27], 6}, {NULL}};
const Segment SGI186608[] = {{&S73660[96], 9}, {NULL}};
const Segment SGI186609[] = {{&S73660[69], 6}, {NULL}};
const Segment SGI186610[] = {{&S62603[44], 10}, {NULL}};
const Segment SGI186611[] = {{&S62603[44], 22}, {NULL}};
const Segment SGI186612[] = {{&S62603[30], 6}, {NULL}};
const Segment SGI186613[] = {{&S62603[115], 3}, {NULL}};
const Segment SGI186614[] = {{&S62603[80], 6}, {NULL}};
const Segment SGI186615[] = {{&S62603[147], 15}, {NULL}};
const Segment SGI186616[] = {{&S62604[44], 10}, {NULL}};
const Segment SGI186617[] = {{&S62604[44], 30}, {NULL}};
const Segment SGI186618[] = {{&S62604[30], 6}, {NULL}};
const Segment SGI186619[] = {{&S62604[123], 3}, {NULL}};
const Segment SGI186620[] = {{&S62604[88], 6}, {NULL}};
const Segment SGI186621[] = {{&S62604[155], 15}, {NULL}};
const Segment SGI186622[] = {{&S62605[44], 10}, {NULL}};
const Segment SGI186623[] = {{&S62605[44], 28}, {NULL}};
const Segment SGI186624[] = {{&S62605[30], 6}, {NULL}};
const Segment SGI186625[] = {{&S62605[121], 3}, {NULL}};
const Segment SGI186626[] = {{&S62605[86], 6}, {NULL}};
const Segment SGI186627[] = {{&S62605[153], 15}, {NULL}};
const Segment SGI186628[] = {{&S62606[44], 11}, {NULL}};
const Segment SGI186629[] = {{&S62606[44], 23}, {NULL}};
const Segment SGI186630[] = {{&S62606[30], 6}, {NULL}};
const Segment SGI186631[] = {{&S62606[116], 3}, {NULL}};
const Segment SGI186632[] = {{&S62606[81], 6}, {NULL}};
const Segment SGI186633[] = {{&S62606[148], 15}, {NULL}};
const Segment SGI186634[] = {{&S73687[44], 14}, {NULL}};
const Segment SGI186635[] = {{&S73687[44], 37}, {NULL}};
const Segment SGI186636[] = {{&S73687[30], 6}, {NULL}};
const Segment SGI186637[] = {{&S73687[31], 3}, {NULL}};
const Segment SGI186638[] = {{&S73687[95], 6}, {NULL}};
const Segment SGI186639[] = {{&S73688[44], 14}, {NULL}};
const Segment SGI186640[] = {{&S73688[44], 26}, {NULL}};
const Segment SGI186641[] = {{&S73688[30], 6}, {NULL}};
const Segment SGI186642[] = {{&S73688[87], 4}, {NULL}};
const Segment SGI186643[] = {{&S73688[87], 3}, {NULL}};
const Segment SGI186644[] = {{&S73689[44], 14}, {NULL}};
const Segment SGI186645[] = {{&S73689[44], 26}, {NULL}};
const Segment SGI186646[] = {{&S73689[30], 6}, {NULL}};
const Segment SGI186647[] = {{&S73689[87], 6}, {NULL}};
const Segment SGI186648[] = {{&S51800[44], 9}, {NULL}};
const Segment SGI186649[] = {{&S51800[44], 20}, {NULL}};
const Segment SGI186650[] = {{&S51800[30], 6}, {NULL}};
const Segment SGI186651[] = {{&S51800[131], 3}, {NULL}};
const Segment SGI186652[] = {{&S51800[96], 6}, {NULL}};
const Segment SGI186653[] = {{&S51800[66], 8}, {NULL}};
const Segment SGI186654[] = {{&S73690[44], 14}, {NULL}};
const Segment SGI186655[] = {{&S73690[44], 26}, {NULL}};
const Segment SGI186656[] = {{&S73690[30], 6}, {NULL}};
const Segment SGI186657[] = {{&S73690[31], 3}, {NULL}};
const Segment SGI186658[] = {{&S73690[84], 6}, {NULL}};
const Segment SGI186659[] = {{&S73691[38], 14}, {NULL}};
const Segment SGI186660[] = {{&S51802[44], 9}, {NULL}};
const Segment SGI186661[] = {{&S73691[31], 6}, {NULL}};
const Segment SGI186662[] = {{&S51802[44], 21}, {NULL}};
const Segment SGI186663[] = {{&S51802[30], 6}, {NULL}};
const Segment SGI186664[] = {{&S51802[114], 3}, {NULL}};
const Segment SGI186665[] = {{&S51802[79], 6}, {NULL}};
const Segment SGI186666[] = {{&S51802[139], 8}, {NULL}};
const Segment SGI186667[] = {{&S73692[44], 12}, {NULL}};
const Segment SGI186668[] = {{&S73692[44], 24}, {NULL}};
const Segment SGI186669[] = {{&S73692[30], 6}, {NULL}};
const Segment SGI186670[] = {{&S73692[31], 3}, {NULL}};
const Segment SGI186671[] = {{&S73692[82], 6}, {NULL}};
const Segment SGI186672[] = {{&S73694[34], 11}, {NULL}};
const Segment SGI186673[] = {{&S73694[27], 6}, {NULL}};
const Segment SGI186674[] = {{&S73694[98], 9}, {NULL}};
const Segment SGI186675[] = {{&S73694[71], 6}, {NULL}};
const Segment SGI186676[] = {{&S73695[34], 10}, {NULL}};
const Segment SGI186677[] = {{&S73695[27], 6}, {NULL}};
const Segment SGI186678[] = {{&S73695[97], 9}, {NULL}};
const Segment SGI186679[] = {{&S73695[70], 6}, {NULL}};
const Segment SGI186680[] = {{&S73696[34], 11}, {NULL}};
const Segment SGI186681[] = {{&S73696[27], 6}, {NULL}};
const Segment SGI186682[] = {{&S73696[98], 9}, {NULL}};
const Segment SGI186683[] = {{&S73696[71], 6}, {NULL}};
const Segment SGI186684[] = {{&S73705[34], 10}, {NULL}};
const Segment SGI186685[] = {{&S73705[27], 6}, {NULL}};
const Segment SGI186686[] = {{&S73705[97], 9}, {NULL}};
const Segment SGI186687[] = {{&S73705[70], 6}, {NULL}};
const Segment SGI186688[] = {{&S73714[44], 10}, {NULL}};
const Segment SGI186689[] = {{&S73714[44], 22}, {NULL}};
const Segment SGI186690[] = {{&S73714[30], 6}, {NULL}};
const Segment SGI186691[] = {{&S73714[31], 3}, {NULL}};
const Segment SGI186692[] = {{&S73714[109], 6}, {NULL}};
const Segment SGI186693[] = {{&S73714[68], 8}, {NULL}};
const Segment SGI186694[] = {{&S73714[80], 15}, {NULL}};
const Segment SGI186695[] = {{&S73715[44], 10}, {NULL}};
const Segment SGI186696[] = {{&S73715[44], 22}, {NULL}};
const Segment SGI186697[] = {{&S73715[30], 6}, {NULL}};
const Segment SGI186698[] = {{&S73715[31], 3}, {NULL}};
const Segment SGI186699[] = {{&S73715[109], 6}, {NULL}};
const Segment SGI186700[] = {{&S73715[68], 8}, {NULL}};
const Segment SGI186701[] = {{&S73715[80], 15}, {NULL}};
const Segment SGI186702[] = {{&S73721[44], 19}, {NULL}};
const Segment SGI186703[] = {{&S73721[44], 35}, {NULL}};
const Segment SGI186704[] = {{&S73721[30], 6}, {NULL}};
const Segment SGI186705[] = {{&S73721[31], 3}, {NULL}};
const Segment SGI186706[] = {{&S73721[93], 6}, {NULL}};
const Segment SGI186707[] = {{&S73726[44], 14}, {NULL}};
const Segment SGI186708[] = {{&S73726[44], 26}, {NULL}};
const Segment SGI186709[] = {{&S73726[30], 6}, {NULL}};
const Segment SGI186710[] = {{&S73726[31], 3}, {NULL}};
const Segment SGI186711[] = {{&S73726[84], 6}, {NULL}};
const Segment SGI186712[] = {{&S73728[44], 7}, {NULL}};
const Segment SGI186713[] = {{&S73728[44], 19}, {NULL}};
const Segment SGI186714[] = {{&S73728[30], 6}, {NULL}};
const Segment SGI186715[] = {{&S73728[31], 3}, {NULL}};
const Segment SGI186716[] = {{&S73728[77], 6}, {NULL}};
const Segment SGI186717[] = {{&S73729[44], 13}, {NULL}};
const Segment SGI186718[] = {{&S73729[44], 25}, {NULL}};
const Segment SGI186719[] = {{&S73729[30], 6}, {NULL}};
const Segment SGI186720[] = {{&S73729[31], 3}, {NULL}};
const Segment SGI186721[] = {{&S73729[83], 6}, {NULL}};
const Segment SGI186722[] = {{&S73738[34], 16}, {NULL}};
const Segment SGI186723[] = {{&S73738[27], 6}, {NULL}};
const Segment SGI186724[] = {{&S73738[103], 9}, {NULL}};
const Segment SGI186725[] = {{&S73738[76], 6}, {NULL}};
const Segment SGI186726[] = {{&S61175[34], 8}, {NULL}};
const Segment SGI186727[] = {{&S61175[27], 6}, {NULL}};
const Segment SGI186728[] = {{&S61175[95], 9}, {NULL}};
const Segment SGI186729[] = {{&S61175[68], 6}, {NULL}};
const Segment SGI186730[] = {{&S73757[44], 7}, {NULL}};
const Segment SGI186731[] = {{&S73757[44], 18}, {NULL}};
const Segment SGI186732[] = {{&S73757[30], 6}, {NULL}};
const Segment SGI186733[] = {{&S73757[31], 3}, {NULL}};
const Segment SGI186734[] = {{&S73757[103], 6}, {NULL}};
const Segment SGI186735[] = {{&S73757[64], 8}, {NULL}};
const Segment SGI186736[] = {{&S62720[34], 7}, {NULL}};
const Segment SGI186737[] = {{&S62720[27], 6}, {NULL}};
const Segment SGI186738[] = {{&S73758[44], 7}, {NULL}};
const Segment SGI186739[] = {{&S73758[44], 19}, {NULL}};
const Segment SGI186740[] = {{&S73758[30], 6}, {NULL}};
const Segment SGI186741[] = {{&S62720[98], 9}, {NULL}};
const Segment SGI186742[] = {{&S62720[71], 6}, {NULL}};
const Segment SGI186743[] = {{&S73758[31], 3}, {NULL}};
const Segment SGI186744[] = {{&S73758[92], 6}, {NULL}};
const Segment SGI186745[] = {{&S73758[65], 8}, {NULL}};
const Segment SGI186746[] = {{&S62721[44], 8}, {NULL}};
const Segment SGI186747[] = {{&S62721[44], 25}, {NULL}};
const Segment SGI186748[] = {{&S62721[30], 6}, {NULL}};
const Segment SGI186749[] = {{&S73759[44], 7}, {NULL}};
const Segment SGI186750[] = {{&S62721[118], 3}, {NULL}};
const Segment SGI186751[] = {{&S73759[44], 19}, {NULL}};
const Segment SGI186752[] = {{&S73759[30], 6}, {NULL}};
const Segment SGI186753[] = {{&S62721[83], 6}, {NULL}};
const Segment SGI186754[] = {{&S73759[31], 3}, {NULL}};
const Segment SGI186755[] = {{&S73759[77], 6}, {NULL}};
const Segment SGI186756[] = {{&S62722[44], 12}, {NULL}};
const Segment SGI186757[] = {{&S62722[44], 29}, {NULL}};
const Segment SGI186758[] = {{&S62722[30], 6}, {NULL}};
const Segment SGI186759[] = {{&S62722[122], 3}, {NULL}};
const Segment SGI186760[] = {{&S62722[87], 6}, {NULL}};
const Segment SGI186761[] = {{&S62723[44], 12}, {NULL}};
const Segment SGI186762[] = {{&S62723[44], 29}, {NULL}};
const Segment SGI186763[] = {{&S62723[30], 6}, {NULL}};
const Segment SGI186764[] = {{&S62723[122], 3}, {NULL}};
const Segment SGI186765[] = {{&S62723[87], 6}, {NULL}};
const Segment SGI186766[] = {{&S73762[44], 10}, {NULL}};
const Segment SGI186767[] = {{&S73762[44], 22}, {NULL}};
const Segment SGI186768[] = {{&S73762[30], 6}, {NULL}};
const Segment SGI186769[] = {{&S73762[31], 3}, {NULL}};
const Segment SGI186770[] = {{&S73762[80], 6}, {NULL}};
const Segment SGI186771[] = {{&S62724[44], 8}, {NULL}};
const Segment SGI186772[] = {{&S62724[44], 25}, {NULL}};
const Segment SGI186773[] = {{&S62724[30], 6}, {NULL}};
const Segment SGI186774[] = {{&S62724[118], 3}, {NULL}};
const Segment SGI186775[] = {{&S62724[83], 6}, {NULL}};
const Segment SGI186776[] = {{&S73763[34], 10}, {NULL}};
const Segment SGI186777[] = {{&S73763[27], 6}, {NULL}};
const Segment SGI186778[] = {{&S73763[108], 9}, {NULL}};
const Segment SGI186779[] = {{&S73763[81], 6}, {NULL}};
const Segment SGI186780[] = {{&S62725[38], 8}, {NULL}};
const Segment SGI186781[] = {{&S62725[31], 6}, {NULL}};
const Segment SGI186782[] = {{&S73764[44], 10}, {NULL}};
const Segment SGI186783[] = {{&S73764[44], 22}, {NULL}};
const Segment SGI186784[] = {{&S73764[30], 6}, {NULL}};
const Segment SGI186785[] = {{&S62726[44], 12}, {NULL}};
const Segment SGI186786[] = {{&S62726[44], 29}, {NULL}};
const Segment SGI186787[] = {{&S62726[30], 6}, {NULL}};
const Segment SGI186788[] = {{&S73764[80], 6}, {NULL}};
const Segment SGI186789[] = {{&S62726[122], 3}, {NULL}};
const Segment SGI186790[] = {{&S73764[107], 13}, {NULL}};
const Segment SGI186791[] = {{&S62726[87], 6}, {NULL}};
const Segment SGI186792[] = {{&S62727[44], 8}, {NULL}};
const Segment SGI186793[] = {{&S62727[44], 25}, {NULL}};
const Segment SGI186794[] = {{&S62727[30], 6}, {NULL}};
const Segment SGI186795[] = {{&S62727[118], 3}, {NULL}};
const Segment SGI186796[] = {{&S62727[83], 6}, {NULL}};
const Segment SGI186797[] = {{&S67065[34], 9}, {NULL}};
const Segment SGI186798[] = {{&S67065[27], 6}, {NULL}};
const Segment SGI186799[] = {{&S67065[96], 9}, {NULL}};
const Segment SGI186800[] = {{&S67065[69], 6}, {NULL}};
const Segment SGI186801[] = {{&S67066[34], 9}, {NULL}};
const Segment SGI186802[] = {{&S67066[27], 6}, {NULL}};
const Segment SGI186803[] = {{&S67066[96], 9}, {NULL}};
const Segment SGI186804[] = {{&S67066[69], 6}, {NULL}};
const Segment SGI186805[] = {{&S62765[44], 11}, {NULL}};
const Segment SGI186806[] = {{&S62765[44], 23}, {NULL}};
const Segment SGI186807[] = {{&S62765[30], 6}, {NULL}};
const Segment SGI186808[] = {{&S62765[116], 3}, {NULL}};
const Segment SGI186809[] = {{&S73802[44], 7}, {NULL}};
const Segment SGI186810[] = {{&S73802[44], 18}, {NULL}};
const Segment SGI186811[] = {{&S73802[30], 6}, {NULL}};
const Segment SGI186812[] = {{&S62765[81], 6}, {NULL}};
const Segment SGI186813[] = {{&S73802[31], 3}, {NULL}};
const Segment SGI186814[] = {{&S73802[76], 6}, {NULL}};
const Segment SGI186815[] = {{&S62766[38], 6}, {NULL}};
const Segment SGI186816[] = {{&S62766[31], 6}, {NULL}};
const Segment SGI186817[] = {{&S73803[44], 16}, {NULL}};
const Segment SGI186818[] = {{&S73803[44], 28}, {NULL}};
const Segment SGI186819[] = {{&S73803[30], 6}, {NULL}};
const Segment SGI186820[] = {{&S73803[31], 3}, {NULL}};
const Segment SGI186821[] = {{&S73803[86], 6}, {NULL}};
const Segment SGI186822[] = {{&S62767[44], 9}, {NULL}};
const Segment SGI186823[] = {{&S62767[44], 21}, {NULL}};
const Segment SGI186824[] = {{&S62767[30], 6}, {NULL}};
const Segment SGI186825[] = {{&S62767[114], 3}, {NULL}};
const Segment SGI186826[] = {{&S62767[79], 6}, {NULL}};
const Segment SGI186827[] = {{&S73804[38], 16}, {NULL}};
const Segment SGI186828[] = {{&S73804[31], 6}, {NULL}};
const Segment SGI186829[] = {{&S62768[38], 9}, {NULL}};
const Segment SGI186830[] = {{&S62768[31], 6}, {NULL}};
const Segment SGI186831[] = {{&S51950[44], 10}, {NULL}};
const Segment SGI186832[] = {{&S51950[44], 21}, {NULL}};
const Segment SGI186833[] = {{&S51950[30], 6}, {NULL}};
const Segment SGI186834[] = {{&S51950[132], 3}, {NULL}};
const Segment SGI186835[] = {{&S51950[97], 6}, {NULL}};
const Segment SGI186836[] = {{&S51950[67], 8}, {NULL}};
const Segment SGI186837[] = {{&S51951[42], 10}, {NULL}};
const Segment SGI186838[] = {{&S51951[30], 6}, {NULL}};
const Segment SGI186839[] = {{&S51951[130], 3}, {NULL}};
const Segment SGI186840[] = {{&S51951[95], 6}, {NULL}};
const Segment SGI186841[] = {{&S51951[65], 8}, {NULL}};
const Segment SGI186842[] = {{&S51952[44], 10}, {NULL}};
const Segment SGI186843[] = {{&S51952[44], 22}, {NULL}};
const Segment SGI186844[] = {{&S51952[30], 6}, {NULL}};
const Segment SGI186845[] = {{&S51952[133], 3}, {NULL}};
const Segment SGI186846[] = {{&S51952[98], 6}, {NULL}};
const Segment SGI186847[] = {{&S51952[68], 8}, {NULL}};
const Segment SGI186848[] = {{&S67116[34], 15}, {NULL}};
const Segment SGI186849[] = {{&S67116[27], 6}, {NULL}};
const Segment SGI186850[] = {{&S67116[102], 9}, {NULL}};
const Segment SGI186851[] = {{&S67116[75], 6}, {NULL}};
const Segment SGI186852[] = {{&S73829[44], 10}, {NULL}};
const Segment SGI186853[] = {{&S73829[44], 22}, {NULL}};
const Segment SGI186854[] = {{&S73829[30], 6}, {NULL}};
const Segment SGI186855[] = {{&S73829[31], 3}, {NULL}};
const Segment SGI186856[] = {{&S73829[80], 6}, {NULL}};
const Segment SGI186857[] = {{&S73829[140], 13}, {NULL}};
const Segment SGI186858[] = {{&S73831[44], 10}, {NULL}};
const Segment SGI186859[] = {{&S73831[44], 22}, {NULL}};
const Segment SGI186860[] = {{&S73831[30], 6}, {NULL}};
const Segment SGI186861[] = {{&S73831[80], 6}, {NULL}};
const Segment SGI186862[] = {{&S73831[107], 13}, {NULL}};
const Segment SGI186863[] = {{&S73837[34], 11}, {NULL}};
const Segment SGI186864[] = {{&S73837[27], 6}, {NULL}};
const Segment SGI186865[] = {{&S73837[98], 9}, {NULL}};
const Segment SGI186866[] = {{&S73837[71], 6}, {NULL}};
const Segment SGI186867[] = {{&S73839[44], 11}, {NULL}};
const Segment SGI186868[] = {{&S73839[44], 23}, {NULL}};
const Segment SGI186869[] = {{&S73839[30], 6}, {NULL}};
const Segment SGI186870[] = {{&S73839[31], 3}, {NULL}};
const Segment SGI186871[] = {{&S73839[81], 6}, {NULL}};
const Segment SGI186872[] = {{&S62818[34], 10}, {NULL}};
const Segment SGI186873[] = {{&S62818[27], 6}, {NULL}};
const Segment SGI186874[] = {{&S62818[97], 9}, {NULL}};
const Segment SGI186875[] = {{&S62818[70], 6}, {NULL}};
const Segment SGI186876[] = {{&S62819[34], 18}, {NULL}};
const Segment SGI186877[] = {{&S62819[27], 6}, {NULL}};
const Segment SGI186878[] = {{&S62819[105], 9}, {NULL}};
const Segment SGI186879[] = {{&S62819[78], 6}, {NULL}};
const Segment SGI186880[] = {{&S67157[34], 10}, {NULL}};
const Segment SGI186881[] = {{&S67157[27], 6}, {NULL}};
const Segment SGI186882[] = {{&S67157[97], 9}, {NULL}};
const Segment SGI186883[] = {{&S67157[70], 6}, {NULL}};
const Segment SGI186884[] = {{&S73862[44], 13}, {NULL}};
const Segment SGI186885[] = {{&S73862[44], 25}, {NULL}};
const Segment SGI186886[] = {{&S73862[30], 6}, {NULL}};
const Segment SGI186887[] = {{&S73862[31], 3}, {NULL}};
const Segment SGI186888[] = {{&S73862[98], 6}, {NULL}};
const Segment SGI186889[] = {{&S73862[71], 8}, {NULL}};
const Segment SGI186890[] = {{&S73863[34], 7}, {NULL}};
const Segment SGI186891[] = {{&S73863[27], 6}, {NULL}};
const Segment SGI186892[] = {{&S73863[94], 9}, {NULL}};
const Segment SGI186893[] = {{&S73863[67], 6}, {NULL}};
const Segment SGI186894[] = {{&S73865[34], 8}, {NULL}};
const Segment SGI186895[] = {{&S73865[27], 6}, {NULL}};
const Segment SGI186896[] = {{&S73865[107], 9}, {NULL}};
const Segment SGI186897[] = {{&S73865[80], 6}, {NULL}};
const Segment SGI186898[] = {{&S73866[34], 7}, {NULL}};
const Segment SGI186899[] = {{&S73866[27], 6}, {NULL}};
const Segment SGI186900[] = {{&S73866[112], 9}, {NULL}};
const Segment SGI186901[] = {{&S73866[85], 6}, {NULL}};
const Segment SGI186902[] = {{&S73868[34], 10}, {NULL}};
const Segment SGI186903[] = {{&S73868[27], 6}, {NULL}};
const Segment SGI186904[] = {{&S73868[97], 9}, {NULL}};
const Segment SGI186905[] = {{&S73868[70], 6}, {NULL}};
const Segment SGI186906[] = {{&S67189[44], 18}, {NULL}};
const Segment SGI186907[] = {{&S67189[44], 30}, {NULL}};
const Segment SGI186908[] = {{&S67189[30], 6}, {NULL}};
const Segment SGI186909[] = {{&S67189[144], 3}, {NULL}};
const Segment SGI186910[] = {{&S67189[109], 6}, {NULL}};
const Segment SGI186911[] = {{&S67191[44], 18}, {NULL}};
const Segment SGI186912[] = {{&S67191[44], 30}, {NULL}};
const Segment SGI186913[] = {{&S67191[30], 6}, {NULL}};
const Segment SGI186914[] = {{&S67191[144], 3}, {NULL}};
const Segment SGI186915[] = {{&S67191[109], 6}, {NULL}};
const Segment SGI186916[] = {{&S73890[34], 14}, {NULL}};
const Segment SGI186917[] = {{&S73890[27], 6}, {NULL}};
const Segment SGI186918[] = {{&S73890[101], 9}, {NULL}};
const Segment SGI186919[] = {{&S73890[74], 6}, {NULL}};
const Segment SGI186920[] = {{&S73892[34], 10}, {NULL}};
const Segment SGI186921[] = {{&S73892[27], 6}, {NULL}};
const Segment SGI186922[] = {{&S73892[97], 9}, {NULL}};
const Segment SGI186923[] = {{&S73892[70], 6}, {NULL}};
const Segment SGI186924[] = {{&S67204[34], 8}, {NULL}};
const Segment SGI186925[] = {{&S67204[27], 6}, {NULL}};
const Segment SGI186926[] = {{&S67204[95], 9}, {NULL}};
const Segment SGI186927[] = {{&S67204[68], 6}, {NULL}};
const Segment SGI186928[] = {{&S67211[44], 10}, {NULL}};
const Segment SGI186929[] = {{&S67211[44], 22}, {NULL}};
const Segment SGI186930[] = {{&S67211[30], 6}, {NULL}};
const Segment SGI186931[] = {{&S67211[131], 3}, {NULL}};
const Segment SGI186932[] = {{&S67211[96], 6}, {NULL}};
const Segment SGI186933[] = {{&S67211[68], 8}, {NULL}};
const Segment SGI186934[] = {{&S67212[44], 10}, {NULL}};
const Segment SGI186935[] = {{&S67212[44], 22}, {NULL}};
const Segment SGI186936[] = {{&S67212[30], 6}, {NULL}};
const Segment SGI186937[] = {{&S67212[131], 3}, {NULL}};
const Segment SGI186938[] = {{&S67212[96], 6}, {NULL}};
const Segment SGI186939[] = {{&S67212[68], 8}, {NULL}};
const Segment SGI186940[] = {{&S67213[44], 10}, {NULL}};
const Segment SGI186941[] = {{&S67213[44], 22}, {NULL}};
const Segment SGI186942[] = {{&S67213[30], 6}, {NULL}};
const Segment SGI186943[] = {{&S67213[131], 3}, {NULL}};
const Segment SGI186944[] = {{&S67213[96], 6}, {NULL}};
const Segment SGI186945[] = {{&S67213[68], 8}, {NULL}};
const Segment SGI186946[] = {{&S67214[44], 10}, {NULL}};
const Segment SGI186947[] = {{&S67214[44], 22}, {NULL}};
const Segment SGI186948[] = {{&S67214[30], 6}, {NULL}};
const Segment SGI186949[] = {{&S67214[131], 3}, {NULL}};
const Segment SGI186950[] = {{&S67214[96], 6}, {NULL}};
const Segment SGI186951[] = {{&S67214[68], 8}, {NULL}};
const Segment SGI186952[] = {{&S73916[44], 9}, {NULL}};
const Segment SGI186953[] = {{&S73916[44], 21}, {NULL}};
const Segment SGI186954[] = {{&S73916[30], 6}, {NULL}};
const Segment SGI186955[] = {{&S73916[31], 3}, {NULL}};
const Segment SGI186956[] = {{&S73916[79], 6}, {NULL}};
const Segment SGI186957[] = {{&S73917[34], 12}, {NULL}};
const Segment SGI186958[] = {{&S73917[27], 6}, {NULL}};
const Segment SGI186959[] = {{&S73917[99], 9}, {NULL}};
const Segment SGI186960[] = {{&S73917[72], 6}, {NULL}};
const Segment SGI186961[] = {{&S73918[34], 15}, {NULL}};
const Segment SGI186962[] = {{&S73918[27], 6}, {NULL}};
const Segment SGI186963[] = {{&S73918[102], 9}, {NULL}};
const Segment SGI186964[] = {{&S73918[75], 6}, {NULL}};
const Segment SGI186965[] = {{&S73919[34], 15}, {NULL}};
const Segment SGI186966[] = {{&S73919[27], 6}, {NULL}};
const Segment SGI186967[] = {{&S73919[102], 9}, {NULL}};
const Segment SGI186968[] = {{&S73919[75], 6}, {NULL}};
const Segment SGI186969[] = {{&S73920[44], 7}, {NULL}};
const Segment SGI186970[] = {{&S73920[44], 19}, {NULL}};
const Segment SGI186971[] = {{&S73920[30], 6}, {NULL}};
const Segment SGI186972[] = {{&S73920[31], 3}, {NULL}};
const Segment SGI186973[] = {{&S73920[77], 6}, {NULL}};
const Segment SGI186974[] = {{&S73921[34], 8}, {NULL}};
const Segment SGI186975[] = {{&S73921[27], 6}, {NULL}};
const Segment SGI186976[] = {{&S61396[34], 19}, {NULL}};
const Segment SGI186977[] = {{&S61396[27], 6}, {NULL}};
const Segment SGI186978[] = {{&S73921[95], 9}, {NULL}};
const Segment SGI186979[] = {{&S73921[68], 6}, {NULL}};
const Segment SGI186980[] = {{&S61396[106], 9}, {NULL}};
const Segment SGI186981[] = {{&S61396[79], 6}, {NULL}};
const Segment SGI186982[] = {{&S73922[34], 9}, {NULL}};
const Segment SGI186983[] = {{&S61397[34], 20}, {NULL}};
const Segment SGI186984[] = {{&S73922[27], 6}, {NULL}};
const Segment SGI186985[] = {{&S61397[27], 6}, {NULL}};
const Segment SGI186986[] = {{&S73922[96], 9}, {NULL}};
const Segment SGI186987[] = {{&S73922[69], 6}, {NULL}};
const Segment SGI186988[] = {{&S61397[107], 9}, {NULL}};
const Segment SGI186989[] = {{&S61397[80], 6}, {NULL}};
const Segment SGI186990[] = {{&S61398[34], 14}, {NULL}};
const Segment SGI186991[] = {{&S61398[27], 6}, {NULL}};
const Segment SGI186992[] = {{&S61398[101], 9}, {NULL}};
const Segment SGI186993[] = {{&S61398[74], 6}, {NULL}};
const Segment SGI186994[] = {{&S67236[34], 7}, {NULL}};
const Segment SGI186995[] = {{&S67236[27], 6}, {NULL}};
const Segment SGI186996[] = {{&S61401[34], 19}, {NULL}};
const Segment SGI186997[] = {{&S61401[27], 6}, {NULL}};
const Segment SGI186998[] = {{&S67236[99], 9}, {NULL}};
const Segment SGI186999[] = {{&S67236[72], 6}, {NULL}};
const Segment SGI187000[] = {{&S61401[106], 9}, {NULL}};
const Segment SGI187001[] = {{&S61401[79], 6}, {NULL}};
const Segment SGI187002[] = {{&S61402[34], 21}, {NULL}};
const Segment SGI187003[] = {{&S61402[27], 6}, {NULL}};
const Segment SGI187004[] = {{&S61402[108], 9}, {NULL}};
const Segment SGI187005[] = {{&S61402[81], 6}, {NULL}};
const Segment SGI187006[] = {{&S73930[34], 10}, {NULL}};
const Segment SGI187007[] = {{&S73930[27], 6}, {NULL}};
const Segment SGI187008[] = {{&S73930[97], 9}, {NULL}};
const Segment SGI187009[] = {{&S73930[70], 6}, {NULL}};
const Segment SGI187010[] = {{&S61404[34], 18}, {NULL}};
const Segment SGI187011[] = {{&S61404[27], 6}, {NULL}};
const Segment SGI187012[] = {{&S61404[105], 9}, {NULL}};
const Segment SGI187013[] = {{&S61404[78], 6}, {NULL}};
const Segment SGI187014[] = {{&S67243[34], 10}, {NULL}};
const Segment SGI187015[] = {{&S67243[27], 6}, {NULL}};
const Segment SGI187016[] = {{&S67243[97], 9}, {NULL}};
const Segment SGI187017[] = {{&S67243[70], 6}, {NULL}};
const Segment SGI187018[] = {{&S67245[34], 10}, {NULL}};
const Segment SGI187019[] = {{&S67245[27], 6}, {NULL}};
const Segment SGI187020[] = {{&S67245[97], 9}, {NULL}};
const Segment SGI187021[] = {{&S67245[70], 6}, {NULL}};
const Segment SGI187022[] = {{&S67246[34], 11}, {NULL}};
const Segment SGI187023[] = {{&S67246[27], 6}, {NULL}};
const Segment SGI187024[] = {{&S67246[98], 9}, {NULL}};
const Segment SGI187025[] = {{&S67246[71], 6}, {NULL}};
const Segment SGI187026[] = {{&S67248[34], 11}, {NULL}};
const Segment SGI187027[] = {{&S67248[27], 6}, {NULL}};
const Segment SGI187028[] = {{&S67248[98], 9}, {NULL}};
const Segment SGI187029[] = {{&S67248[71], 6}, {NULL}};
const Segment SGI187030[] = {{&S67252[44], 16}, {NULL}};
const Segment SGI187031[] = {{&S67252[44], 28}, {NULL}};
const Segment SGI187032[] = {{&S67252[30], 6}, {NULL}};
const Segment SGI187033[] = {{&S67252[121], 3}, {NULL}};
const Segment SGI187034[] = {{&S67252[86], 6}, {NULL}};
const Segment SGI187035[] = {{&S73952[44], 7}, {NULL}};
const Segment SGI187036[] = {{&S73952[44], 19}, {NULL}};
const Segment SGI187037[] = {{&S73952[30], 6}, {NULL}};
const Segment SGI187038[] = {{&S73952[77], 6}, {NULL}};
const Segment SGI187039[] = {{&S73952[104], 13}, {NULL}};
const Segment SGI187040[] = {{&S73953[44], 8}, {NULL}};
const Segment SGI187041[] = {{&S73953[44], 20}, {NULL}};
const Segment SGI187042[] = {{&S73953[30], 6}, {NULL}};
const Segment SGI187043[] = {{&S73953[78], 6}, {NULL}};
const Segment SGI187044[] = {{&S73953[105], 13}, {NULL}};
const Segment SGI187045[] = {{&S73954[44], 15}, {NULL}};
const Segment SGI187046[] = {{&S73954[44], 27}, {NULL}};
const Segment SGI187047[] = {{&S73954[30], 6}, {NULL}};
const Segment SGI187048[] = {{&S73954[31], 3}, {NULL}};
const Segment SGI187049[] = {{&S73954[85], 6}, {NULL}};
const Segment SGI187050[] = {{&S73955[44], 10}, {NULL}};
const Segment SGI187051[] = {{&S73955[44], 22}, {NULL}};
const Segment SGI187052[] = {{&S73955[30], 6}, {NULL}};
const Segment SGI187053[] = {{&S73955[31], 3}, {NULL}};
const Segment SGI187054[] = {{&S73955[80], 6}, {NULL}};
const Segment SGI187055[] = {{&S73955[143], 3}, {NULL}};
const Segment SGI187056[] = {{&S52125[44], 18}, {NULL}};
const Segment SGI187057[] = {{&S52125[44], 29}, {NULL}};
const Segment SGI187058[] = {{&S52125[30], 6}, {NULL}};
const Segment SGI187059[] = {{&S52125[140], 3}, {NULL}};
const Segment SGI187060[] = {{&S52125[105], 6}, {NULL}};
const Segment SGI187061[] = {{&S52125[75], 8}, {NULL}};
const Segment SGI187062[] = {{&S73960[34], 10}, {NULL}};
const Segment SGI187063[] = {{&S73960[27], 6}, {NULL}};
const Segment SGI187064[] = {{&S52126[44], 18}, {NULL}};
const Segment SGI187065[] = {{&S52126[44], 30}, {NULL}};
const Segment SGI187066[] = {{&S73960[97], 9}, {NULL}};
const Segment SGI187067[] = {{&S52126[30], 6}, {NULL}};
const Segment SGI187068[] = {{&S73960[70], 6}, {NULL}};
const Segment SGI187069[] = {{&S52126[123], 3}, {NULL}};
const Segment SGI187070[] = {{&S52126[88], 6}, {NULL}};
const Segment SGI187071[] = {{&S52126[148], 8}, {NULL}};
const Segment SGI187072[] = {{&S67277[34], 11}, {NULL}};
const Segment SGI187073[] = {{&S67277[27], 6}, {NULL}};
const Segment SGI187074[] = {{&S67277[98], 9}, {NULL}};
const Segment SGI187075[] = {{&S67277[71], 6}, {NULL}};
const Segment SGI187076[] = {{&S73966[34], 8}, {NULL}};
const Segment SGI187077[] = {{&S73966[27], 6}, {NULL}};
const Segment SGI187078[] = {{&S73966[101], 9}, {NULL}};
const Segment SGI187079[] = {{&S73966[74], 6}, {NULL}};
const Segment SGI187080[] = {{&S67291[44], 8}, {NULL}};
const Segment SGI187081[] = {{&S67291[44], 20}, {NULL}};
const Segment SGI187082[] = {{&S67291[30], 6}, {NULL}};
const Segment SGI187083[] = {{&S67291[113], 3}, {NULL}};
const Segment SGI187084[] = {{&S67291[78], 6}, {NULL}};
const Segment SGI187085[] = {{&S67292[44], 9}, {NULL}};
const Segment SGI187086[] = {{&S67292[44], 21}, {NULL}};
const Segment SGI187087[] = {{&S67292[30], 6}, {NULL}};
const Segment SGI187088[] = {{&S67292[114], 3}, {NULL}};
const Segment SGI187089[] = {{&S67292[79], 6}, {NULL}};
const Segment SGI187090[] = {{&S73986[44], 8}, {NULL}};
const Segment SGI187091[] = {{&S73986[44], 20}, {NULL}};
const Segment SGI187092[] = {{&S73986[30], 6}, {NULL}};
const Segment SGI187093[] = {{&S73986[31], 3}, {NULL}};
const Segment SGI187094[] = {{&S73986[78], 6}, {NULL}};
const Segment SGI187095[] = {{&S73988[44], 14}, {NULL}};
const Segment SGI187096[] = {{&S73988[44], 26}, {NULL}};
const Segment SGI187097[] = {{&S73988[30], 6}, {NULL}};
const Segment SGI187098[] = {{&S73988[31], 3}, {NULL}};
const Segment SGI187099[] = {{&S73988[84], 6}, {NULL}};
const Segment SGI187100[] = {{&S73989[44], 14}, {NULL}};
const Segment SGI187101[] = {{&S73989[44], 26}, {NULL}};
const Segment SGI187102[] = {{&S73989[30], 6}, {NULL}};
const Segment SGI187103[] = {{&S73989[31], 3}, {NULL}};
const Segment SGI187104[] = {{&S73989[84], 6}, {NULL}};
const Segment SGI187105[] = {{&S73991[44], 14}, {NULL}};
const Segment SGI187106[] = {{&S73991[44], 38}, {NULL}};
const Segment SGI187107[] = {{&S73991[30], 6}, {NULL}};
const Segment SGI187108[] = {{&S73991[31], 3}, {NULL}};
const Segment SGI187109[] = {{&S73991[96], 6}, {NULL}};
const Segment SGI187110[] = {{&S73992[44], 14}, {NULL}};
const Segment SGI187111[] = {{&S73992[44], 26}, {NULL}};
const Segment SGI187112[] = {{&S73992[30], 6}, {NULL}};
const Segment SGI187113[] = {{&S73992[31], 3}, {NULL}};
const Segment SGI187114[] = {{&S73992[84], 6}, {NULL}};
const Segment SGI187115[] = {{&S73993[44], 18}, {NULL}};
const Segment SGI187116[] = {{&S73993[44], 30}, {NULL}};
const Segment SGI187117[] = {{&S73993[30], 6}, {NULL}};
const Segment SGI187118[] = {{&S73993[31], 3}, {NULL}};
const Segment SGI187119[] = {{&S73993[88], 6}, {NULL}};
const Segment SGI187120[] = {{&S73995[44], 14}, {NULL}};
const Segment SGI187121[] = {{&S73995[44], 38}, {NULL}};
const Segment SGI187122[] = {{&S73995[30], 6}, {NULL}};
const Segment SGI187123[] = {{&S73995[31], 3}, {NULL}};
const Segment SGI187124[] = {{&S73995[96], 6}, {NULL}};
const Segment SGI187125[] = {{&S74000[34], 8}, {NULL}};
const Segment SGI187126[] = {{&S74000[27], 6}, {NULL}};
const Segment SGI187127[] = {{&S74000[95], 9}, {NULL}};
const Segment SGI187128[] = {{&S74000[68], 6}, {NULL}};
const Segment SGI187129[] = {{&S74003[44], 15}, {NULL}};
const Segment SGI187130[] = {{&S74003[44], 27}, {NULL}};
const Segment SGI187131[] = {{&S74003[30], 6}, {NULL}};
const Segment SGI187132[] = {{&S74003[31], 3}, {NULL}};
const Segment SGI187133[] = {{&S74003[85], 6}, {NULL}};
const Segment SGI187134[] = {{&S74006[34], 11}, {NULL}};
const Segment SGI187135[] = {{&S74006[27], 6}, {NULL}};
const Segment SGI187136[] = {{&S74006[98], 9}, {NULL}};
const Segment SGI187137[] = {{&S74006[71], 6}, {NULL}};
const Segment SGI187138[] = {{&S74007[34], 14}, {NULL}};
const Segment SGI187139[] = {{&S74007[27], 6}, {NULL}};
const Segment SGI187140[] = {{&S74007[101], 9}, {NULL}};
const Segment SGI187141[] = {{&S74007[74], 6}, {NULL}};
const Segment SGI187142[] = {{&S73033[44], 10}, {NULL}};
const Segment SGI187143[] = {{&S73033[44], 27}, {NULL}};
const Segment SGI187144[] = {{&S73033[30], 6}, {NULL}};
const Segment SGI187145[] = {{&S73033[31], 3}, {NULL}};
const Segment SGI187146[] = {{&S73033[85], 5}, {NULL}};
const Segment SGI187147[] = {{&S74008[34], 15}, {NULL}};
const Segment SGI187148[] = {{&S74008[27], 6}, {NULL}};
const Segment SGI187149[] = {{&S74008[102], 9}, {NULL}};
const Segment SGI187150[] = {{&S74008[75], 6}, {NULL}};
const Segment SGI187151[] = {{&S74009[34], 14}, {NULL}};
const Segment SGI187152[] = {{&S74009[27], 6}, {NULL}};
const Segment SGI187153[] = {{&S74009[101], 9}, {NULL}};
const Segment SGI187154[] = {{&S74009[74], 6}, {NULL}};
const Segment SGI187155[] = {{&S74010[34], 15}, {NULL}};
const Segment SGI187156[] = {{&S74010[27], 6}, {NULL}};
const Segment SGI187157[] = {{&S74010[102], 9}, {NULL}};
const Segment SGI187158[] = {{&S74010[75], 6}, {NULL}};
const Segment SGI187159[] = {{&S67342[45], 10}, {NULL}};
const Segment SGI187160[] = {{&S67342[30], 6}, {NULL}};
const Segment SGI187161[] = {{&S67342[115], 3}, {NULL}};
const Segment SGI187162[] = {{&S67342[81], 5}, {NULL}};
const Segment SGI187163[] = {{&S67342[139], 13}, {NULL}};
const Segment SGI187164[] = {{&S74029[44], 7}, {NULL}};
const Segment SGI187165[] = {{&S74029[44], 19}, {NULL}};
const Segment SGI187166[] = {{&S74029[30], 6}, {NULL}};
const Segment SGI187167[] = {{&S74029[31], 3}, {NULL}};
const Segment SGI187168[] = {{&S74029[77], 6}, {NULL}};
const Segment SGI187169[] = {{&S74030[44], 15}, {NULL}};
const Segment SGI187170[] = {{&S74030[44], 27}, {NULL}};
const Segment SGI187171[] = {{&S74030[30], 6}, {NULL}};
const Segment SGI187172[] = {{&S74030[31], 3}, {NULL}};
const Segment SGI187173[] = {{&S74030[85], 6}, {NULL}};
const Segment SGI187174[] = {{&S61538[44], 6}, {NULL}};
const Segment SGI187175[] = {{&S61538[44], 18}, {NULL}};
const Segment SGI187176[] = {{&S74031[44], 11}, {NULL}};
const Segment SGI187177[] = {{&S61538[30], 6}, {NULL}};
const Segment SGI187178[] = {{&S74031[44], 23}, {NULL}};
const Segment SGI187179[] = {{&S74031[30], 6}, {NULL}};
const Segment SGI187180[] = {{&S61538[111], 3}, {NULL}};
const Segment SGI187181[] = {{&S67359[34], 6}, {NULL}};
const Segment SGI187182[] = {{&S74031[31], 3}, {NULL}};
const Segment SGI187183[] = {{&S61538[76], 6}, {NULL}};
const Segment SGI187184[] = {{&S67359[27], 6}, {NULL}};
const Segment SGI187185[] = {{&S74031[81], 6}, {NULL}};
const Segment SGI187186[] = {{&S67359[93], 9}, {NULL}};
const Segment SGI187187[] = {{&S67359[66], 6}, {NULL}};
const Segment SGI187188[] = {{&S61539[38], 6}, {NULL}};
const Segment SGI187189[] = {{&S61539[31], 6}, {NULL}};
const Segment SGI187190[] = {{&S67399[44], 32}, {NULL}};
const Segment SGI187191[] = {{&S67399[44], 44}, {NULL}};
const Segment SGI187192[] = {{&S67399[30], 6}, {NULL}};
const Segment SGI187193[] = {{&S67399[31], 3}, {NULL}};
const Segment SGI187194[] = {{&S67399[102], 6}, {NULL}};
const Segment SGI187195[] = {{&S52257[44], 7}, {NULL}};
const Segment SGI187196[] = {{&S52257[44], 25}, {NULL}};
const Segment SGI187197[] = {{&S52257[30], 6}, {NULL}};
const Segment SGI187198[] = {{&S52257[118], 3}, {NULL}};
const Segment SGI187199[] = {{&S52257[83], 6}, {NULL}};
const Segment SGI187200[] = {{&S67400[44], 25}, {NULL}};
const Segment SGI187201[] = {{&S67400[44], 37}, {NULL}};
const Segment SGI187202[] = {{&S67400[30], 6}, {NULL}};
const Segment SGI187203[] = {{&S67400[31], 3}, {NULL}};
const Segment SGI187204[] = {{&S67400[95], 6}, {NULL}};
const Segment SGI187205[] = {{&S52258[38], 7}, {NULL}};
const Segment SGI187206[] = {{&S52258[31], 6}, {NULL}};
const Segment SGI187207[] = {{&S52259[44], 7}, {NULL}};
const Segment SGI187208[] = {{&S52259[44], 27}, {NULL}};
const Segment SGI187209[] = {{&S52259[30], 6}, {NULL}};
const Segment SGI187210[] = {{&S52259[120], 3}, {NULL}};
const Segment SGI187211[] = {{&S52259[85], 6}, {NULL}};
const Segment SGI187212[] = {{&S52260[44], 10}, {NULL}};
const Segment SGI187213[] = {{&S52260[44], 30}, {NULL}};
const Segment SGI187214[] = {{&S52260[30], 6}, {NULL}};
const Segment SGI187215[] = {{&S52260[123], 3}, {NULL}};
const Segment SGI187216[] = {{&S52260[88], 6}, {NULL}};
const Segment SGI187217[] = {{&S74083[44], 8}, {NULL}};
const Segment SGI187218[] = {{&S74083[44], 20}, {NULL}};
const Segment SGI187219[] = {{&S74083[30], 6}, {NULL}};
const Segment SGI187220[] = {{&S74083[31], 3}, {NULL}};
const Segment SGI187221[] = {{&S74083[78], 6}, {NULL}};
const Segment SGI187222[] = {{&S74084[44], 8}, {NULL}};
const Segment SGI187223[] = {{&S74084[44], 20}, {NULL}};
const Segment SGI187224[] = {{&S74084[30], 6}, {NULL}};
const Segment SGI187225[] = {{&S74084[81], 6}, {NULL}};
const Segment SGI187226[] = {{&S74085[38], 8}, {NULL}};
const Segment SGI187227[] = {{&S74085[31], 6}, {NULL}};
const Segment SGI187228[] = {{&S74086[44], 25}, {NULL}};
const Segment SGI187229[] = {{&S74086[44], 37}, {NULL}};
const Segment SGI187230[] = {{&S74086[30], 6}, {NULL}};
const Segment SGI187231[] = {{&S74086[31], 3}, {NULL}};
const Segment SGI187232[] = {{&S74086[94], 6}, {NULL}};
const Segment SGI187233[] = {{&S74087[44], 25}, {NULL}};
const Segment SGI187234[] = {{&S74087[44], 37}, {NULL}};
const Segment SGI187235[] = {{&S74087[30], 6}, {NULL}};
const Segment SGI187236[] = {{&S74087[31], 3}, {NULL}};
const Segment SGI187237[] = {{&S74087[94], 6}, {NULL}};
const Segment SGI187238[] = {{&S74088[44], 25}, {NULL}};
const Segment SGI187239[] = {{&S74088[44], 37}, {NULL}};
const Segment SGI187240[] = {{&S74088[30], 6}, {NULL}};
const Segment SGI187241[] = {{&S74088[31], 3}, {NULL}};
const Segment SGI187242[] = {{&S74088[94], 6}, {NULL}};
const Segment SGI187243[] = {{&S74089[59], 10}, {NULL}};
const Segment SGI187244[] = {{&S74089[59], 22}, {NULL}};
const Segment SGI187245[] = {{&S74089[45], 6}, {NULL}};
const Segment SGI187246[] = {{&S74089[46], 3}, {NULL}};
const Segment SGI187247[] = {{&S74089[95], 6}, {NULL}};
const Segment SGI187248[] = {{&S74089[0], 14}, {NULL}};
const Segment SGI187249[] = {{&S74090[59], 11}, {NULL}};
const Segment SGI187250[] = {{&S74090[59], 23}, {NULL}};
const Segment SGI187251[] = {{&S74090[45], 6}, {NULL}};
const Segment SGI187252[] = {{&S74090[46], 3}, {NULL}};
const Segment SGI187253[] = {{&S74090[96], 6}, {NULL}};
const Segment SGI187254[] = {{&S74090[0], 14}, {NULL}};
const Segment SGI187255[] = {{&S74094[44], 13}, {NULL}};
const Segment SGI187256[] = {{&S74094[44], 25}, {NULL}};
const Segment SGI187257[] = {{&S74094[30], 6}, {NULL}};
const Segment SGI187258[] = {{&S74094[31], 3}, {NULL}};
const Segment SGI187259[] = {{&S74094[83], 6}, {NULL}};
const Segment SGI187260[] = {{&S74097[34], 10}, {NULL}};
const Segment SGI187261[] = {{&S74097[27], 6}, {NULL}};
const Segment SGI187262[] = {{&S74097[97], 9}, {NULL}};
const Segment SGI187263[] = {{&S74097[70], 6}, {NULL}};
const Segment SGI187264[] = {{&S52295[41], 19}, {NULL}};
const Segment SGI187265[] = {{&S74104[44], 8}, {NULL}};
const Segment SGI187266[] = {{&S52295[41], 30}, {NULL}};
const Segment SGI187267[] = {{&S74104[44], 20}, {NULL}};
const Segment SGI187268[] = {{&S74104[30], 6}, {NULL}};
const Segment SGI187269[] = {{&S52295[27], 6}, {NULL}};
const Segment SGI187270[] = {{&S74104[31], 3}, {NULL}};
const Segment SGI187271[] = {{&S52295[120], 3}, {NULL}};
const Segment SGI187272[] = {{&S74104[78], 6}, {NULL}};
const Segment SGI187273[] = {{&S52295[124], 9}, {NULL}};
const Segment SGI187274[] = {{&S52295[85], 6}, {NULL}};
const Segment SGI187275[] = {{&S74115[44], 11}, {NULL}};
const Segment SGI187276[] = {{&S74115[44], 23}, {NULL}};
const Segment SGI187277[] = {{&S74115[30], 6}, {NULL}};
const Segment SGI187278[] = {{&S74115[31], 3}, {NULL}};
const Segment SGI187279[] = {{&S74115[81], 6}, {NULL}};
const Segment SGI187280[] = {{&S74129[44], 15}, {NULL}};
const Segment SGI187281[] = {{&S74129[44], 27}, {NULL}};
const Segment SGI187282[] = {{&S74129[30], 6}, {NULL}};
const Segment SGI187283[] = {{&S74129[31], 3}, {NULL}};
const Segment SGI187284[] = {{&S74129[85], 6}, {NULL}};
const Segment SGI187285[] = {{&S74134[34], 10}, {NULL}};
const Segment SGI187286[] = {{&S74134[27], 6}, {NULL}};
const Segment SGI187287[] = {{&S74134[97], 9}, {NULL}};
const Segment SGI187288[] = {{&S74134[70], 6}, {NULL}};
const Segment SGI187289[] = {{&S74135[34], 11}, {NULL}};
const Segment SGI187290[] = {{&S74135[27], 6}, {NULL}};
const Segment SGI187291[] = {{&S74135[98], 9}, {NULL}};
const Segment SGI187292[] = {{&S74135[71], 6}, {NULL}};
const Segment SGI187293[] = {{&S67485[34], 13}, {NULL}};
const Segment SGI187294[] = {{&S67485[27], 6}, {NULL}};
const Segment SGI187295[] = {{&S67485[100], 9}, {NULL}};
const Segment SGI187296[] = {{&S67485[73], 6}, {NULL}};
const Segment SGI187297[] = {{&S67486[34], 6}, {NULL}};
const Segment SGI187298[] = {{&S67486[27], 6}, {NULL}};
const Segment SGI187299[] = {{&S67486[93], 9}, {NULL}};
const Segment SGI187300[] = {{&S67486[66], 6}, {NULL}};
const Segment SGI187301[] = {{&S67488[44], 7}, {NULL}};
const Segment SGI187302[] = {{&S67488[44], 19}, {NULL}};
const Segment SGI187303[] = {{&S67488[30], 6}, {NULL}};
const Segment SGI187304[] = {{&S67488[128], 3}, {NULL}};
const Segment SGI187305[] = {{&S67488[93], 6}, {NULL}};
const Segment SGI187306[] = {{&S74142[34], 11}, {NULL}};
const Segment SGI187307[] = {{&S74142[27], 6}, {NULL}};
const Segment SGI187308[] = {{&S67488[65], 8}, {NULL}};
const Segment SGI187309[] = {{&S74142[98], 9}, {NULL}};
const Segment SGI187310[] = {{&S74142[71], 6}, {NULL}};
const Segment SGI187311[] = {{&S74143[34], 10}, {NULL}};
const Segment SGI187312[] = {{&S74143[27], 6}, {NULL}};
const Segment SGI187313[] = {{&S74143[97], 9}, {NULL}};
const Segment SGI187314[] = {{&S74143[70], 6}, {NULL}};
const Segment SGI187315[] = {{&S67491[34], 6}, {NULL}};
const Segment SGI187316[] = {{&S67491[27], 6}, {NULL}};
const Segment SGI187317[] = {{&S67491[93], 9}, {NULL}};
const Segment SGI187318[] = {{&S67491[66], 6}, {NULL}};
const Segment SGI187319[] = {{&S61691[34], 17}, {NULL}};
const Segment SGI187320[] = {{&S61691[27], 6}, {NULL}};
const Segment SGI187321[] = {{&S61691[104], 9}, {NULL}};
const Segment SGI187322[] = {{&S61691[77], 6}, {NULL}};
const Segment SGI187323[] = {{&S74149[44], 19}, {NULL}};
const Segment SGI187324[] = {{&S74149[44], 31}, {NULL}};
const Segment SGI187325[] = {{&S74149[30], 6}, {NULL}};
const Segment SGI187326[] = {{&S74149[31], 3}, {NULL}};
const Segment SGI187327[] = {{&S74149[89], 6}, {NULL}};
const Segment SGI187328[] = {{&S67498[34], 15}, {NULL}};
const Segment SGI187329[] = {{&S67498[27], 6}, {NULL}};
const Segment SGI187330[] = {{&S67498[102], 9}, {NULL}};
const Segment SGI187331[] = {{&S67498[75], 6}, {NULL}};
const Segment SGI187332[] = {{&S67500[34], 11}, {NULL}};
const Segment SGI187333[] = {{&S67500[27], 6}, {NULL}};
const Segment SGI187334[] = {{&S67500[98], 9}, {NULL}};
const Segment SGI187335[] = {{&S67500[71], 6}, {NULL}};
const Segment SGI187336[] = {{&S74157[34], 10}, {NULL}};
const Segment SGI187337[] = {{&S74157[27], 6}, {NULL}};
const Segment SGI187338[] = {{&S74157[97], 9}, {NULL}};
const Segment SGI187339[] = {{&S74157[70], 6}, {NULL}};
const Segment SGI187340[] = {{&S67508[34], 6}, {NULL}};
const Segment SGI187341[] = {{&S74159[34], 15}, {NULL}};
const Segment SGI187342[] = {{&S67508[27], 6}, {NULL}};
const Segment SGI187343[] = {{&S74159[27], 6}, {NULL}};
const Segment SGI187344[] = {{&S67508[93], 9}, {NULL}};
const Segment SGI187345[] = {{&S67508[66], 6}, {NULL}};
const Segment SGI187346[] = {{&S74159[102], 9}, {NULL}};
const Segment SGI187347[] = {{&S74159[75], 6}, {NULL}};
const Segment SGI187348[] = {{&S74163[34], 16}, {NULL}};
const Segment SGI187349[] = {{&S74163[27], 6}, {NULL}};
const Segment SGI187350[] = {{&S74163[103], 9}, {NULL}};
const Segment SGI187351[] = {{&S74163[76], 6}, {NULL}};
const Segment SGI187352[] = {{&S74173[45], 15}, {NULL}};
const Segment SGI187353[] = {{&S74173[30], 6}, {NULL}};
const Segment SGI187354[] = {{&S74173[31], 3}, {NULL}};
const Segment SGI187355[] = {{&S74173[86], 5}, {NULL}};
const Segment SGI187356[] = {{&S74173[144], 13}, {NULL}};
const Segment SGI187357[] = {{&S74175[44], 17}, {NULL}};
const Segment SGI187358[] = {{&S74175[44], 29}, {NULL}};
const Segment SGI187359[] = {{&S74175[30], 6}, {NULL}};
const Segment SGI187360[] = {{&S74175[31], 3}, {NULL}};
const Segment SGI187361[] = {{&S74175[87], 6}, {NULL}};
const Segment SGI187362[] = {{&S74176[44], 9}, {NULL}};
const Segment SGI187363[] = {{&S74176[44], 21}, {NULL}};
const Segment SGI187364[] = {{&S74176[30], 6}, {NULL}};
const Segment SGI187365[] = {{&S74176[31], 3}, {NULL}};
const Segment SGI187366[] = {{&S74176[79], 6}, {NULL}};
const Segment SGI187367[] = {{&S74177[34], 10}, {NULL}};
const Segment SGI187368[] = {{&S74177[27], 6}, {NULL}};
const Segment SGI187369[] = {{&S74177[97], 9}, {NULL}};
const Segment SGI187370[] = {{&S74177[70], 6}, {NULL}};
const Segment SGI187371[] = {{&S74178[34], 10}, {NULL}};
const Segment SGI187372[] = {{&S74178[27], 6}, {NULL}};
const Segment SGI187373[] = {{&S74178[97], 9}, {NULL}};
const Segment SGI187374[] = {{&S74178[70], 6}, {NULL}};
const Segment SGI187375[] = {{&S65497[32], 10}, {NULL}};
const Segment SGI187376[] = {{&S65497[27], 4}, {NULL}};
const Segment SGI187377[] = {{&S65497[95], 9}, {NULL}};
const Segment SGI187378[] = {{&S65497[68], 6}, {NULL}};
const Segment SGI187379[] = {{&S74179[34], 13}, {NULL}};
const Segment SGI187380[] = {{&S74179[27], 6}, {NULL}};
const Segment SGI187381[] = {{&S74179[100], 9}, {NULL}};
const Segment SGI187382[] = {{&S74179[73], 6}, {NULL}};
const Segment SGI187383[] = {{&S67546[44], 11}, {NULL}};
const Segment SGI187384[] = {{&S67546[44], 27}, {NULL}};
const Segment SGI187385[] = {{&S67546[30], 6}, {NULL}};
const Segment SGI187386[] = {{&S74182[44], 13}, {NULL}};
const Segment SGI187387[] = {{&S74182[44], 25}, {NULL}};
const Segment SGI187388[] = {{&S74182[30], 6}, {NULL}};
const Segment SGI187389[] = {{&S67546[120], 3}, {NULL}};
const Segment SGI187390[] = {{&S67546[85], 6}, {NULL}};
const Segment SGI187391[] = {{&S74182[31], 3}, {NULL}};
const Segment SGI187392[] = {{&S74182[83], 6}, {NULL}};
const Segment SGI187393[] = {{&S67547[44], 7}, {NULL}};
const Segment SGI187394[] = {{&S67547[44], 23}, {NULL}};
const Segment SGI187395[] = {{&S67547[30], 6}, {NULL}};
const Segment SGI187396[] = {{&S74183[44], 15}, {NULL}};
const Segment SGI187397[] = {{&S74183[44], 27}, {NULL}};
const Segment SGI187398[] = {{&S74183[30], 6}, {NULL}};
const Segment SGI187399[] = {{&S67547[116], 3}, {NULL}};
const Segment SGI187400[] = {{&S67547[81], 6}, {NULL}};
const Segment SGI187401[] = {{&S74183[31], 3}, {NULL}};
const Segment SGI187402[] = {{&S74183[85], 6}, {NULL}};
const Segment SGI187403[] = {{&S67548[38], 7}, {NULL}};
const Segment SGI187404[] = {{&S61747[34], 10}, {NULL}};
const Segment SGI187405[] = {{&S67548[31], 6}, {NULL}};
const Segment SGI187406[] = {{&S61747[27], 6}, {NULL}};
const Segment SGI187407[] = {{&S61747[97], 9}, {NULL}};
const Segment SGI187408[] = {{&S61747[70], 6}, {NULL}};
const Segment SGI187409[] = {{&S52418[44], 18}, {NULL}};
const Segment SGI187410[] = {{&S52418[44], 29}, {NULL}};
const Segment SGI187411[] = {{&S52418[30], 6}, {NULL}};
const Segment SGI187412[] = {{&S52418[122], 3}, {NULL}};
const Segment SGI187413[] = {{&S52418[87], 6}, {NULL}};
const Segment SGI187414[] = {{&S52419[38], 18}, {NULL}};
const Segment SGI187415[] = {{&S52419[31], 6}, {NULL}};
const Segment SGI187416[] = {{&S74202[44], 18}, {NULL}};
const Segment SGI187417[] = {{&S74202[44], 27}, {NULL}};
const Segment SGI187418[] = {{&S74202[30], 6}, {NULL}};
const Segment SGI187419[] = {{&S74202[31], 3}, {NULL}};
const Segment SGI187420[] = {{&S74202[85], 6}, {NULL}};
const Segment SGI187421[] = {{&S74203[44], 18}, {NULL}};
const Segment SGI187422[] = {{&S74203[44], 27}, {NULL}};
const Segment SGI187423[] = {{&S74203[30], 6}, {NULL}};
const Segment SGI187424[] = {{&S74203[31], 3}, {NULL}};
const Segment SGI187425[] = {{&S74203[85], 6}, {NULL}};
const Segment SGI187426[] = {{&S74204[44], 18}, {NULL}};
const Segment SGI187427[] = {{&S74204[44], 27}, {NULL}};
const Segment SGI187428[] = {{&S74204[30], 6}, {NULL}};
const Segment SGI187429[] = {{&S74204[88], 6}, {NULL}};
const Segment SGI187430[] = {{&S67572[34], 15}, {NULL}};
const Segment SGI187431[] = {{&S67572[27], 6}, {NULL}};
const Segment SGI187432[] = {{&S67572[102], 9}, {NULL}};
const Segment SGI187433[] = {{&S67572[75], 6}, {NULL}};
const Segment SGI187434[] = {{&S74205[38], 18}, {NULL}};
const Segment SGI187435[] = {{&S74205[31], 6}, {NULL}};
const Segment SGI187436[] = {{&S74209[34], 8}, {NULL}};
const Segment SGI187437[] = {{&S74209[27], 6}, {NULL}};
const Segment SGI187438[] = {{&S74209[100], 9}, {NULL}};
const Segment SGI187439[] = {{&S74209[73], 6}, {NULL}};
const Segment SGI187440[] = {{&S74212[34], 7}, {NULL}};
const Segment SGI187441[] = {{&S74212[27], 6}, {NULL}};
const Segment SGI187442[] = {{&S74212[105], 9}, {NULL}};
const Segment SGI187443[] = {{&S74212[78], 6}, {NULL}};
const Segment SGI187444[] = {{&S62910[34], 7}, {NULL}};
const Segment SGI187445[] = {{&S62910[27], 6}, {NULL}};
const Segment SGI187446[] = {{&S62910[105], 9}, {NULL}};
const Segment SGI187447[] = {{&S62910[78], 6}, {NULL}};
const Segment SGI187448[] = {{&S74251[34], 8}, {NULL}};
const Segment SGI187449[] = {{&S74251[27], 6}, {NULL}};
const Segment SGI187450[] = {{&S74251[100], 9}, {NULL}};
const Segment SGI187451[] = {{&S74251[73], 6}, {NULL}};
const Segment SGI187452[] = {{&S62928[44], 10}, {NULL}};
const Segment SGI187453[] = {{&S62928[44], 21}, {NULL}};
const Segment SGI187454[] = {{&S62928[30], 6}, {NULL}};
const Segment SGI187455[] = {{&S62928[113], 3}, {NULL}};
const Segment SGI187456[] = {{&S62928[79], 5}, {NULL}};
const Segment SGI187457[] = {{&S62931[44], 10}, {NULL}};
const Segment SGI187458[] = {{&S62931[44], 22}, {NULL}};
const Segment SGI187459[] = {{&S62931[30], 6}, {NULL}};
const Segment SGI187460[] = {{&S62931[115], 3}, {NULL}};
const Segment SGI187461[] = {{&S62931[80], 6}, {NULL}};
const Segment SGI187462[] = {{&S74256[34], 9}, {NULL}};
const Segment SGI187463[] = {{&S74256[27], 6}, {NULL}};
const Segment SGI187464[] = {{&S74256[96], 9}, {NULL}};
const Segment SGI187465[] = {{&S74256[69], 6}, {NULL}};
const Segment SGI187466[] = {{&S62932[38], 10}, {NULL}};
const Segment SGI187467[] = {{&S62932[31], 6}, {NULL}};
const Segment SGI187468[] = {{&S74257[34], 11}, {NULL}};
const Segment SGI187469[] = {{&S74257[27], 6}, {NULL}};
const Segment SGI187470[] = {{&S74257[98], 9}, {NULL}};
const Segment SGI187471[] = {{&S74257[71], 6}, {NULL}};
const Segment SGI187472[] = {{&S74258[34], 11}, {NULL}};
const Segment SGI187473[] = {{&S74258[27], 6}, {NULL}};
const Segment SGI187474[] = {{&S74258[98], 9}, {NULL}};
const Segment SGI187475[] = {{&S74258[71], 6}, {NULL}};
const Segment SGI187476[] = {{&S74260[34], 9}, {NULL}};
const Segment SGI187477[] = {{&S74260[27], 6}, {NULL}};
const Segment SGI187478[] = {{&S74260[96], 9}, {NULL}};
const Segment SGI187479[] = {{&S74260[69], 6}, {NULL}};
const Segment SGI187480[] = {{&S52503[44], 10}, {NULL}};
const Segment SGI187481[] = {{&S52503[44], 21}, {NULL}};
const Segment SGI187482[] = {{&S52503[30], 6}, {NULL}};
const Segment SGI187483[] = {{&S52503[132], 3}, {NULL}};
const Segment SGI187484[] = {{&S52503[97], 6}, {NULL}};
const Segment SGI187485[] = {{&S52503[67], 8}, {NULL}};
const Segment SGI187486[] = {{&S52504[44], 10}, {NULL}};
const Segment SGI187487[] = {{&S52504[44], 22}, {NULL}};
const Segment SGI187488[] = {{&S52504[30], 6}, {NULL}};
const Segment SGI187489[] = {{&S52504[133], 3}, {NULL}};
const Segment SGI187490[] = {{&S52504[98], 6}, {NULL}};
const Segment SGI187491[] = {{&S52504[68], 8}, {NULL}};
const Segment SGI187492[] = {{&S62952[34], 10}, {NULL}};
const Segment SGI187493[] = {{&S62952[27], 6}, {NULL}};
const Segment SGI187494[] = {{&S62952[97], 9}, {NULL}};
const Segment SGI187495[] = {{&S62952[70], 6}, {NULL}};
const Segment SGI187496[] = {{&S62963[44], 12}, {NULL}};
const Segment SGI187497[] = {{&S62963[44], 24}, {NULL}};
const Segment SGI187498[] = {{&S62963[30], 6}, {NULL}};
const Segment SGI187499[] = {{&S62963[117], 3}, {NULL}};
const Segment SGI187500[] = {{&S62963[82], 6}, {NULL}};
const Segment SGI187501[] = {{&S67662[34], 7}, {NULL}};
const Segment SGI187502[] = {{&S67662[27], 6}, {NULL}};
const Segment SGI187503[] = {{&S67662[94], 9}, {NULL}};
const Segment SGI187504[] = {{&S67662[67], 6}, {NULL}};
const Segment SGI187505[] = {{&S62964[44], 20}, {NULL}};
const Segment SGI187506[] = {{&S62964[44], 32}, {NULL}};
const Segment SGI187507[] = {{&S62964[30], 6}, {NULL}};
const Segment SGI187508[] = {{&S62964[125], 3}, {NULL}};
const Segment SGI187509[] = {{&S62964[90], 6}, {NULL}};
const Segment SGI187510[] = {{&S62965[38], 12}, {NULL}};
const Segment SGI187511[] = {{&S62965[31], 6}, {NULL}};
const Segment SGI187512[] = {{&S67668[44], 30}, {NULL}};
const Segment SGI187513[] = {{&S67668[44], 42}, {NULL}};
const Segment SGI187514[] = {{&S67668[30], 6}, {NULL}};
const Segment SGI187515[] = {{&S67668[135], 3}, {NULL}};
const Segment SGI187516[] = {{&S67668[100], 6}, {NULL}};
const Segment SGI187517[] = {{&S67669[38], 30}, {NULL}};
const Segment SGI187518[] = {{&S67669[31], 6}, {NULL}};
const Segment SGI187519[] = {{&S74295[44], 7}, {NULL}};
const Segment SGI187520[] = {{&S74295[44], 26}, {NULL}};
const Segment SGI187521[] = {{&S74295[30], 6}, {NULL}};
const Segment SGI187522[] = {{&S74295[31], 3}, {NULL}};
const Segment SGI187523[] = {{&S74295[84], 6}, {NULL}};
const Segment SGI187524[] = {{&S67670[44], 31}, {NULL}};
const Segment SGI187525[] = {{&S67670[44], 43}, {NULL}};
const Segment SGI187526[] = {{&S67670[30], 6}, {NULL}};
const Segment SGI187527[] = {{&S67670[136], 3}, {NULL}};
const Segment SGI187528[] = {{&S52531[44], 20}, {NULL}};
const Segment SGI187529[] = {{&S67670[101], 6}, {NULL}};
const Segment SGI187530[] = {{&S52531[44], 31}, {NULL}};
const Segment SGI187531[] = {{&S52531[30], 6}, {NULL}};
const Segment SGI187532[] = {{&S52531[142], 3}, {NULL}};
const Segment SGI187533[] = {{&S52531[107], 6}, {NULL}};
const Segment SGI187534[] = {{&S74296[44], 7}, {NULL}};
const Segment SGI187535[] = {{&S74296[44], 28}, {NULL}};
const Segment SGI187536[] = {{&S52531[77], 8}, {NULL}};
const Segment SGI187537[] = {{&S74296[30], 6}, {NULL}};
const Segment SGI187538[] = {{&S74296[31], 3}, {NULL}};
const Segment SGI187539[] = {{&S74296[86], 6}, {NULL}};
const Segment SGI187540[] = {{&S74297[44], 10}, {NULL}};
const Segment SGI187541[] = {{&S74297[44], 22}, {NULL}};
const Segment SGI187542[] = {{&S52533[44], 21}, {NULL}};
const Segment SGI187543[] = {{&S74297[30], 6}, {NULL}};
const Segment SGI187544[] = {{&S52533[44], 32}, {NULL}};
const Segment SGI187545[] = {{&S52533[30], 6}, {NULL}};
const Segment SGI187546[] = {{&S74297[31], 3}, {NULL}};
const Segment SGI187547[] = {{&S52533[143], 3}, {NULL}};
const Segment SGI187548[] = {{&S74297[80], 6}, {NULL}};
const Segment SGI187549[] = {{&S52533[108], 6}, {NULL}};
const Segment SGI187550[] = {{&S52533[78], 8}, {NULL}};
const Segment SGI187551[] = {{&S74297[150], 3}, {NULL}};
const Segment SGI187552[] = {{&S52534[44], 20}, {NULL}};
const Segment SGI187553[] = {{&S52534[44], 32}, {NULL}};
const Segment SGI187554[] = {{&S52534[30], 6}, {NULL}};
const Segment SGI187555[] = {{&S52534[143], 3}, {NULL}};
const Segment SGI187556[] = {{&S52534[108], 6}, {NULL}};
const Segment SGI187557[] = {{&S52534[78], 8}, {NULL}};
const Segment SGI187558[] = {{&S52535[44], 20}, {NULL}};
const Segment SGI187559[] = {{&S52535[44], 31}, {NULL}};
const Segment SGI187560[] = {{&S52535[30], 6}, {NULL}};
const Segment SGI187561[] = {{&S52535[142], 3}, {NULL}};
const Segment SGI187562[] = {{&S67678[44], 12}, {NULL}};
const Segment SGI187563[] = {{&S52535[107], 6}, {NULL}};
const Segment SGI187564[] = {{&S67678[44], 24}, {NULL}};
const Segment SGI187565[] = {{&S52535[77], 8}, {NULL}};
const Segment SGI187566[] = {{&S67678[30], 6}, {NULL}};
const Segment SGI187567[] = {{&S67678[133], 3}, {NULL}};
const Segment SGI187568[] = {{&S67678[98], 6}, {NULL}};
const Segment SGI187569[] = {{&S67678[70], 8}, {NULL}};
const Segment SGI187570[] = {{&S52536[44], 21}, {NULL}};
const Segment SGI187571[] = {{&S52536[44], 33}, {NULL}};
const Segment SGI187572[] = {{&S52536[30], 6}, {NULL}};
const Segment SGI187573[] = {{&S52536[144], 3}, {NULL}};
const Segment SGI187574[] = {{&S67679[44], 12}, {NULL}};
const Segment SGI187575[] = {{&S52536[109], 6}, {NULL}};
const Segment SGI187576[] = {{&S67679[44], 24}, {NULL}};
const Segment SGI187577[] = {{&S52536[79], 8}, {NULL}};
const Segment SGI187578[] = {{&S67679[30], 6}, {NULL}};
const Segment SGI187579[] = {{&S67679[144], 3}, {NULL}};
const Segment SGI187580[] = {{&S67679[109], 6}, {NULL}};
const Segment SGI187581[] = {{&S67679[70], 8}, {NULL}};
const Segment SGI187582[] = {{&S74304[34], 18}, {NULL}};
const Segment SGI187583[] = {{&S74304[27], 6}, {NULL}};
const Segment SGI187584[] = {{&S74304[105], 9}, {NULL}};
const Segment SGI187585[] = {{&S74304[78], 6}, {NULL}};
const Segment SGI187586[] = {{&S62980[44], 8}, {NULL}};
const Segment SGI187587[] = {{&S62980[44], 19}, {NULL}};
const Segment SGI187588[] = {{&S62980[30], 6}, {NULL}};
const Segment SGI187589[] = {{&S62980[112], 3}, {NULL}};
const Segment SGI187590[] = {{&S62980[77], 6}, {NULL}};
const Segment SGI187591[] = {{&S74305[34], 19}, {NULL}};
const Segment SGI187592[] = {{&S74305[27], 6}, {NULL}};
const Segment SGI187593[] = {{&S67681[44], 12}, {NULL}};
const Segment SGI187594[] = {{&S67681[44], 24}, {NULL}};
const Segment SGI187595[] = {{&S67681[30], 6}, {NULL}};
const Segment SGI187596[] = {{&S74305[106], 9}, {NULL}};
const Segment SGI187597[] = {{&S74305[79], 6}, {NULL}};
const Segment SGI187598[] = {{&S67681[146], 3}, {NULL}};
const Segment SGI187599[] = {{&S67681[111], 6}, {NULL}};
const Segment SGI187600[] = {{&S62981[38], 8}, {NULL}};
const Segment SGI187601[] = {{&S62981[31], 6}, {NULL}};
const Segment SGI187602[] = {{&S67681[70], 8}, {NULL}};
const Segment SGI187603[] = {{&S74306[34], 16}, {NULL}};
const Segment SGI187604[] = {{&S74306[27], 6}, {NULL}};
const Segment SGI187605[] = {{&S62982[44], 8}, {NULL}};
const Segment SGI187606[] = {{&S74306[103], 9}, {NULL}};
const Segment SGI187607[] = {{&S62982[44], 19}, {NULL}};
const Segment SGI187608[] = {{&S74306[76], 6}, {NULL}};
const Segment SGI187609[] = {{&S62982[30], 6}, {NULL}};
const Segment SGI187610[] = {{&S62982[112], 3}, {NULL}};
const Segment SGI187611[] = {{&S62982[77], 6}, {NULL}};
const Segment SGI187612[] = {{&S74307[34], 17}, {NULL}};
const Segment SGI187613[] = {{&S74307[27], 6}, {NULL}};
const Segment SGI187614[] = {{&S74307[104], 9}, {NULL}};
const Segment SGI187615[] = {{&S74307[77], 6}, {NULL}};
const Segment SGI187616[] = {{&S74308[34], 19}, {NULL}};
const Segment SGI187617[] = {{&S74308[27], 6}, {NULL}};
const Segment SGI187618[] = {{&S74308[106], 9}, {NULL}};
const Segment SGI187619[] = {{&S74308[79], 6}, {NULL}};
const Segment SGI187620[] = {{&S62990[44], 7}, {NULL}};
const Segment SGI187621[] = {{&S62990[44], 19}, {NULL}};
const Segment SGI187622[] = {{&S62990[30], 6}, {NULL}};
const Segment SGI187623[] = {{&S62990[112], 3}, {NULL}};
const Segment SGI187624[] = {{&S62990[77], 6}, {NULL}};
const Segment SGI187625[] = {{&S62991[38], 7}, {NULL}};
const Segment SGI187626[] = {{&S62991[31], 6}, {NULL}};
const Segment SGI187627[] = {{&S67695[34], 8}, {NULL}};
const Segment SGI187628[] = {{&S67695[27], 6}, {NULL}};
const Segment SGI187629[] = {{&S67695[95], 9}, {NULL}};
const Segment SGI187630[] = {{&S67695[68], 6}, {NULL}};
const Segment SGI187631[] = {{&S74318[44], 18}, {NULL}};
const Segment SGI187632[] = {{&S74318[44], 30}, {NULL}};
const Segment SGI187633[] = {{&S74318[30], 6}, {NULL}};
const Segment SGI187634[] = {{&S74318[31], 3}, {NULL}};
const Segment SGI187635[] = {{&S74318[88], 6}, {NULL}};
const Segment SGI187636[] = {{&S74318[158], 3}, {NULL}};
const Segment SGI187637[] = {{&S74319[44], 11}, {NULL}};
const Segment SGI187638[] = {{&S74319[44], 23}, {NULL}};
const Segment SGI187639[] = {{&S74319[30], 6}, {NULL}};
const Segment SGI187640[] = {{&S74319[31], 3}, {NULL}};
const Segment SGI187641[] = {{&S74319[81], 6}, {NULL}};
const Segment SGI187642[] = {{&S74319[151], 3}, {NULL}};
const Segment SGI187643[] = {{&S74320[44], 10}, {NULL}};
const Segment SGI187644[] = {{&S74320[44], 22}, {NULL}};
const Segment SGI187645[] = {{&S74320[30], 6}, {NULL}};
const Segment SGI187646[] = {{&S74320[31], 3}, {NULL}};
const Segment SGI187647[] = {{&S74320[80], 6}, {NULL}};
const Segment SGI187648[] = {{&S74320[150], 3}, {NULL}};
const Segment SGI187649[] = {{&S74321[44], 11}, {NULL}};
const Segment SGI187650[] = {{&S74321[44], 23}, {NULL}};
const Segment SGI187651[] = {{&S74321[30], 6}, {NULL}};
const Segment SGI187652[] = {{&S74321[31], 3}, {NULL}};
const Segment SGI187653[] = {{&S74321[81], 6}, {NULL}};
const Segment SGI187654[] = {{&S74321[151], 3}, {NULL}};
const Segment SGI187655[] = {{&S74325[34], 6}, {NULL}};
const Segment SGI187656[] = {{&S74325[27], 6}, {NULL}};
const Segment SGI187657[] = {{&S74325[93], 9}, {NULL}};
const Segment SGI187658[] = {{&S74325[66], 6}, {NULL}};
const Segment SGI187659[] = {{&S67723[44], 10}, {NULL}};
const Segment SGI187660[] = {{&S67723[44], 22}, {NULL}};
const Segment SGI187661[] = {{&S67723[30], 6}, {NULL}};
const Segment SGI187662[] = {{&S67723[115], 3}, {NULL}};
const Segment SGI187663[] = {{&S67723[80], 6}, {NULL}};
const Segment SGI187664[] = {{&S67724[44], 10}, {NULL}};
const Segment SGI187665[] = {{&S67724[44], 22}, {NULL}};
const Segment SGI187666[] = {{&S67724[30], 6}, {NULL}};
const Segment SGI187667[] = {{&S67724[115], 3}, {NULL}};
const Segment SGI187668[] = {{&S67724[80], 6}, {NULL}};
const Segment SGI187669[] = {{&S74339[34], 8}, {NULL}};
const Segment SGI187670[] = {{&S74339[27], 6}, {NULL}};
const Segment SGI187671[] = {{&S74339[100], 9}, {NULL}};
const Segment SGI187672[] = {{&S74339[73], 6}, {NULL}};
const Segment SGI187673[] = {{&S67725[38], 10}, {NULL}};
const Segment SGI187674[] = {{&S67725[31], 6}, {NULL}};
const Segment SGI187675[] = {{&S67726[44], 10}, {NULL}};
const Segment SGI187676[] = {{&S67726[44], 22}, {NULL}};
const Segment SGI187677[] = {{&S67726[30], 6}, {NULL}};
const Segment SGI187678[] = {{&S67726[115], 3}, {NULL}};
const Segment SGI187679[] = {{&S67726[80], 6}, {NULL}};
const Segment SGI187680[] = {{&S67728[44], 10}, {NULL}};
const Segment SGI187681[] = {{&S67728[44], 22}, {NULL}};
const Segment SGI187682[] = {{&S67728[30], 6}, {NULL}};
const Segment SGI187683[] = {{&S67728[115], 3}, {NULL}};
const Segment SGI187684[] = {{&S67728[80], 6}, {NULL}};
const Segment SGI187685[] = {{&S74359[34], 10}, {NULL}};
const Segment SGI187686[] = {{&S74359[27], 6}, {NULL}};
const Segment SGI187687[] = {{&S74359[97], 9}, {NULL}};
const Segment SGI187688[] = {{&S74359[70], 6}, {NULL}};
const Segment SGI187689[] = {{&S74364[34], 8}, {NULL}};
const Segment SGI187690[] = {{&S74364[27], 6}, {NULL}};
const Segment SGI187691[] = {{&S74364[107], 9}, {NULL}};
const Segment SGI187692[] = {{&S74364[80], 6}, {NULL}};
const Segment SGI187693[] = {{&S52615[34], 18}, {NULL}};
const Segment SGI187694[] = {{&S63057[34], 7}, {NULL}};
const Segment SGI187695[] = {{&S52615[27], 6}, {NULL}};
const Segment SGI187696[] = {{&S63057[27], 6}, {NULL}};
const Segment SGI187697[] = {{&S52615[105], 9}, {NULL}};
const Segment SGI187698[] = {{&S63057[100], 9}, {NULL}};
const Segment SGI187699[] = {{&S52615[78], 6}, {NULL}};
const Segment SGI187700[] = {{&S74365[34], 7}, {NULL}};
const Segment SGI187701[] = {{&S63057[73], 6}, {NULL}};
const Segment SGI187702[] = {{&S74365[27], 6}, {NULL}};
const Segment SGI187703[] = {{&S74365[112], 9}, {NULL}};
const Segment SGI187704[] = {{&S74365[85], 6}, {NULL}};
const Segment SGI187705[] = {{&S52616[34], 10}, {NULL}};
const Segment SGI187706[] = {{&S52616[27], 6}, {NULL}};
const Segment SGI187707[] = {{&S63058[44], 12}, {NULL}};
const Segment SGI187708[] = {{&S63058[44], 30}, {NULL}};
const Segment SGI187709[] = {{&S63058[30], 6}, {NULL}};
const Segment SGI187710[] = {{&S52616[97], 9}, {NULL}};
const Segment SGI187711[] = {{&S74366[34], 10}, {NULL}};
const Segment SGI187712[] = {{&S52616[70], 6}, {NULL}};
const Segment SGI187713[] = {{&S63058[123], 3}, {NULL}};
const Segment SGI187714[] = {{&S74366[27], 6}, {NULL}};
const Segment SGI187715[] = {{&S63058[88], 6}, {NULL}};
const Segment SGI187716[] = {{&S74366[97], 9}, {NULL}};
const Segment SGI187717[] = {{&S74366[70], 6}, {NULL}};
const Segment SGI187718[] = {{&S52617[34], 10}, {NULL}};
const Segment SGI187719[] = {{&S52617[27], 6}, {NULL}};
const Segment SGI187720[] = {{&S63059[44], 12}, {NULL}};
const Segment SGI187721[] = {{&S63059[44], 30}, {NULL}};
const Segment SGI187722[] = {{&S63059[30], 6}, {NULL}};
const Segment SGI187723[] = {{&S52617[97], 9}, {NULL}};
const Segment SGI187724[] = {{&S52617[70], 6}, {NULL}};
const Segment SGI187725[] = {{&S63059[123], 3}, {NULL}};
const Segment SGI187726[] = {{&S63059[88], 6}, {NULL}};
const Segment SGI187727[] = {{&S52618[34], 10}, {NULL}};
const Segment SGI187728[] = {{&S52618[27], 6}, {NULL}};
const Segment SGI187729[] = {{&S63060[44], 12}, {NULL}};
const Segment SGI187730[] = {{&S63060[44], 30}, {NULL}};
const Segment SGI187731[] = {{&S52618[97], 9}, {NULL}};
const Segment SGI187732[] = {{&S63060[30], 6}, {NULL}};
const Segment SGI187733[] = {{&S52618[70], 6}, {NULL}};
const Segment SGI187734[] = {{&S63060[123], 3}, {NULL}};
const Segment SGI187735[] = {{&S63060[88], 6}, {NULL}};
const Segment SGI187736[] = {{&S74370[34], 7}, {NULL}};
const Segment SGI187737[] = {{&S74370[27], 6}, {NULL}};
const Segment SGI187738[] = {{&S74370[94], 9}, {NULL}};
const Segment SGI187739[] = {{&S52619[34], 16}, {NULL}};
const Segment SGI187740[] = {{&S74370[67], 6}, {NULL}};
const Segment SGI187741[] = {{&S52619[27], 6}, {NULL}};
const Segment SGI187742[] = {{&S52619[102], 9}, {NULL}};
const Segment SGI187743[] = {{&S63061[44], 12}, {NULL}};
const Segment SGI187744[] = {{&S52619[75], 6}, {NULL}};
const Segment SGI187745[] = {{&S63061[44], 30}, {NULL}};
const Segment SGI187746[] = {{&S63061[30], 6}, {NULL}};
const Segment SGI187747[] = {{&S63061[123], 3}, {NULL}};
const Segment SGI187748[] = {{&S63061[88], 6}, {NULL}};
const Segment SGI187749[] = {{&S52620[34], 10}, {NULL}};
const Segment SGI187750[] = {{&S52620[27], 6}, {NULL}};
const Segment SGI187751[] = {{&S52620[97], 9}, {NULL}};
const Segment SGI187752[] = {{&S52620[70], 6}, {NULL}};
const Segment SGI187753[] = {{&S63062[44], 8}, {NULL}};
const Segment SGI187754[] = {{&S63062[44], 26}, {NULL}};
const Segment SGI187755[] = {{&S63062[30], 6}, {NULL}};
const Segment SGI187756[] = {{&S63062[119], 3}, {NULL}};
const Segment SGI187757[] = {{&S63062[84], 6}, {NULL}};
const Segment SGI187758[] = {{&S52621[34], 10}, {NULL}};
const Segment SGI187759[] = {{&S52621[27], 6}, {NULL}};
const Segment SGI187760[] = {{&S63063[38], 8}, {NULL}};
const Segment SGI187761[] = {{&S63063[31], 6}, {NULL}};
const Segment SGI187762[] = {{&S52621[96], 9}, {NULL}};
const Segment SGI187763[] = {{&S52621[69], 6}, {NULL}};
const Segment SGI187764[] = {{&S63064[44], 12}, {NULL}};
const Segment SGI187765[] = {{&S52622[34], 10}, {NULL}};
const Segment SGI187766[] = {{&S63064[44], 30}, {NULL}};
const Segment SGI187767[] = {{&S63064[30], 6}, {NULL}};
const Segment SGI187768[] = {{&S52622[27], 6}, {NULL}};
const Segment SGI187769[] = {{&S63064[123], 3}, {NULL}};
const Segment SGI187770[] = {{&S52622[97], 9}, {NULL}};
const Segment SGI187771[] = {{&S63064[88], 6}, {NULL}};
const Segment SGI187772[] = {{&S52622[70], 6}, {NULL}};
const Segment SGI187773[] = {{&S52623[34], 10}, {NULL}};
const Segment SGI187774[] = {{&S52623[27], 6}, {NULL}};
const Segment SGI187775[] = {{&S52623[97], 9}, {NULL}};
const Segment SGI187776[] = {{&S52623[70], 6}, {NULL}};
const Segment SGI187777[] = {{&S63067[34], 8}, {NULL}};
const Segment SGI187778[] = {{&S63067[27], 6}, {NULL}};
const Segment SGI187779[] = {{&S52624[34], 10}, {NULL}};
const Segment SGI187780[] = {{&S52624[27], 6}, {NULL}};
const Segment SGI187781[] = {{&S63067[100], 9}, {NULL}};
const Segment SGI187782[] = {{&S63067[73], 6}, {NULL}};
const Segment SGI187783[] = {{&S52624[97], 9}, {NULL}};
const Segment SGI187784[] = {{&S52624[70], 6}, {NULL}};
const Segment SGI187785[] = {{&S63068[34], 8}, {NULL}};
const Segment SGI187786[] = {{&S63068[27], 6}, {NULL}};
const Segment SGI187787[] = {{&S63068[100], 9}, {NULL}};
const Segment SGI187788[] = {{&S63068[73], 6}, {NULL}};
const Segment SGI187789[] = {{&S74392[44], 11}, {NULL}};
const Segment SGI187790[] = {{&S74392[44], 23}, {NULL}};
const Segment SGI187791[] = {{&S74392[30], 6}, {NULL}};
const Segment SGI187792[] = {{&S74392[31], 3}, {NULL}};
const Segment SGI187793[] = {{&S74392[81], 6}, {NULL}};
const Segment SGI187794[] = {{&S63076[34], 7}, {NULL}};
const Segment SGI187795[] = {{&S63076[27], 6}, {NULL}};
const Segment SGI187796[] = {{&S63076[98], 9}, {NULL}};
const Segment SGI187797[] = {{&S63076[71], 6}, {NULL}};
const Segment SGI187798[] = {{&S74407[34], 11}, {NULL}};
const Segment SGI187799[] = {{&S74407[27], 6}, {NULL}};
const Segment SGI187800[] = {{&S74407[98], 9}, {NULL}};
const Segment SGI187801[] = {{&S74407[71], 6}, {NULL}};
const Segment SGI187802[] = {{&S74408[34], 11}, {NULL}};
const Segment SGI187803[] = {{&S74408[27], 6}, {NULL}};
const Segment SGI187804[] = {{&S74408[98], 9}, {NULL}};
const Segment SGI187805[] = {{&S74408[71], 6}, {NULL}};
const Segment SGI187806[] = {{&S74415[34], 7}, {NULL}};
const Segment SGI187807[] = {{&S74415[27], 6}, {NULL}};
const Segment SGI187808[] = {{&S74415[94], 9}, {NULL}};
const Segment SGI187809[] = {{&S74415[67], 6}, {NULL}};
const Segment SGI187810[] = {{&S74416[44], 11}, {NULL}};
const Segment SGI187811[] = {{&S74416[44], 23}, {NULL}};
const Segment SGI187812[] = {{&S74416[30], 6}, {NULL}};
const Segment SGI187813[] = {{&S74416[31], 3}, {NULL}};
const Segment SGI187814[] = {{&S74416[81], 6}, {NULL}};
const Segment SGI187815[] = {{&S74417[44], 11}, {NULL}};
const Segment SGI187816[] = {{&S74417[44], 23}, {NULL}};
const Segment SGI187817[] = {{&S74417[30], 6}, {NULL}};
const Segment SGI187818[] = {{&S74417[31], 3}, {NULL}};
const Segment SGI187819[] = {{&S74417[81], 6}, {NULL}};
const Segment SGI187820[] = {{&S74418[44], 6}, {NULL}};
const Segment SGI187821[] = {{&S74418[44], 18}, {NULL}};
const Segment SGI187822[] = {{&S74418[30], 6}, {NULL}};
const Segment SGI187823[] = {{&S74418[31], 3}, {NULL}};
const Segment SGI187824[] = {{&S74418[76], 6}, {NULL}};
const Segment SGI187825[] = {{&S63111[44], 6}, {NULL}};
const Segment SGI187826[] = {{&S63111[44], 18}, {NULL}};
const Segment SGI187827[] = {{&S63111[30], 6}, {NULL}};
const Segment SGI187828[] = {{&S63111[131], 3}, {NULL}};
const Segment SGI187829[] = {{&S63111[96], 6}, {NULL}};
const Segment SGI187830[] = {{&S63111[64], 8}, {NULL}};
const Segment SGI187831[] = {{&S63112[44], 7}, {NULL}};
const Segment SGI187832[] = {{&S63112[44], 20}, {NULL}};
const Segment SGI187833[] = {{&S63112[30], 6}, {NULL}};
const Segment SGI187834[] = {{&S74424[34], 9}, {NULL}};
const Segment SGI187835[] = {{&S74424[27], 6}, {NULL}};
const Segment SGI187836[] = {{&S63112[113], 3}, {NULL}};
const Segment SGI187837[] = {{&S63112[78], 6}, {NULL}};
const Segment SGI187838[] = {{&S74424[96], 9}, {NULL}};
const Segment SGI187839[] = {{&S74424[69], 6}, {NULL}};
const Segment SGI187840[] = {{&S63113[44], 7}, {NULL}};
const Segment SGI187841[] = {{&S63113[44], 20}, {NULL}};
const Segment SGI187842[] = {{&S63113[30], 6}, {NULL}};
const Segment SGI187843[] = {{&S63113[113], 3}, {NULL}};
const Segment SGI187844[] = {{&S63113[78], 6}, {NULL}};
const Segment SGI187845[] = {{&S63114[38], 7}, {NULL}};
const Segment SGI187846[] = {{&S63114[31], 6}, {NULL}};
const Segment SGI187847[] = {{&S63116[34], 11}, {NULL}};
const Segment SGI187848[] = {{&S63116[27], 6}, {NULL}};
const Segment SGI187849[] = {{&S63116[98], 9}, {NULL}};
const Segment SGI187850[] = {{&S63116[71], 6}, {NULL}};
const Segment SGI187851[] = {{&S74431[34], 7}, {NULL}};
const Segment SGI187852[] = {{&S74431[27], 6}, {NULL}};
const Segment SGI187853[] = {{&S63117[34], 10}, {NULL}};
const Segment SGI187854[] = {{&S63117[27], 6}, {NULL}};
const Segment SGI187855[] = {{&S74431[99], 9}, {NULL}};
const Segment SGI187856[] = {{&S74431[72], 6}, {NULL}};
const Segment SGI187857[] = {{&S63117[97], 9}, {NULL}};
const Segment SGI187858[] = {{&S63117[70], 6}, {NULL}};
const Segment SGI187859[] = {{&S74431[140], 8}, {NULL}};
const Segment SGI187860[] = {{&S63118[34], 11}, {NULL}};
const Segment SGI187861[] = {{&S63118[27], 6}, {NULL}};
const Segment SGI187862[] = {{&S63118[98], 9}, {NULL}};
const Segment SGI187863[] = {{&S63118[71], 6}, {NULL}};
const Segment SGI187864[] = {{&S63119[34], 11}, {NULL}};
const Segment SGI187865[] = {{&S63119[27], 6}, {NULL}};
const Segment SGI187866[] = {{&S63119[98], 9}, {NULL}};
const Segment SGI187867[] = {{&S63119[71], 6}, {NULL}};
const Segment SGI187868[] = {{&S63126[44], 10}, {NULL}};
const Segment SGI187869[] = {{&S63126[44], 22}, {NULL}};
const Segment SGI187870[] = {{&S63126[30], 6}, {NULL}};
const Segment SGI187871[] = {{&S63126[115], 3}, {NULL}};
const Segment SGI187872[] = {{&S63126[80], 6}, {NULL}};
const Segment SGI187873[] = {{&S63126[140], 11}, {NULL}};
const Segment SGI187874[] = {{&S63127[44], 10}, {NULL}};
const Segment SGI187875[] = {{&S63127[44], 22}, {NULL}};
const Segment SGI187876[] = {{&S63127[30], 6}, {NULL}};
const Segment SGI187877[] = {{&S63127[115], 3}, {NULL}};
const Segment SGI187878[] = {{&S63127[80], 6}, {NULL}};
const Segment SGI187879[] = {{&S63127[140], 11}, {NULL}};
const Segment SGI187880[] = {{&S63128[44], 10}, {NULL}};
const Segment SGI187881[] = {{&S63128[44], 22}, {NULL}};
const Segment SGI187882[] = {{&S63128[30], 6}, {NULL}};
const Segment SGI187883[] = {{&S63128[115], 3}, {NULL}};
const Segment SGI187884[] = {{&S63128[80], 6}, {NULL}};
const Segment SGI187885[] = {{&S63128[140], 11}, {NULL}};
const Segment SGI187886[] = {{&S63129[44], 18}, {NULL}};
const Segment SGI187887[] = {{&S63129[44], 30}, {NULL}};
const Segment SGI187888[] = {{&S63129[30], 6}, {NULL}};
const Segment SGI187889[] = {{&S63129[123], 3}, {NULL}};
const Segment SGI187890[] = {{&S63129[88], 6}, {NULL}};
const Segment SGI187891[] = {{&S63129[148], 11}, {NULL}};
const Segment SGI187892[] = {{&S63130[44], 10}, {NULL}};
const Segment SGI187893[] = {{&S63130[44], 22}, {NULL}};
const Segment SGI187894[] = {{&S63130[30], 6}, {NULL}};
const Segment SGI187895[] = {{&S63130[115], 3}, {NULL}};
const Segment SGI187896[] = {{&S63130[80], 6}, {NULL}};
const Segment SGI187897[] = {{&S63130[140], 11}, {NULL}};
const Segment SGI187898[] = {{&S67851[34], 10}, {NULL}};
const Segment SGI187899[] = {{&S67851[27], 6}, {NULL}};
const Segment SGI187900[] = {{&S67851[97], 9}, {NULL}};
const Segment SGI187901[] = {{&S67851[70], 6}, {NULL}};
const Segment SGI187902[] = {{&S74453[34], 7}, {NULL}};
const Segment SGI187903[] = {{&S74453[27], 6}, {NULL}};
const Segment SGI187904[] = {{&S63132[44], 10}, {NULL}};
const Segment SGI187905[] = {{&S63132[44], 22}, {NULL}};
const Segment SGI187906[] = {{&S63132[30], 6}, {NULL}};
const Segment SGI187907[] = {{&S74453[94], 9}, {NULL}};
const Segment SGI187908[] = {{&S74453[67], 6}, {NULL}};
const Segment SGI187909[] = {{&S63132[115], 3}, {NULL}};
const Segment SGI187910[] = {{&S63132[80], 6}, {NULL}};
const Segment SGI187911[] = {{&S63132[141], 12}, {NULL}};
const Segment SGI187912[] = {{&S74456[34], 10}, {NULL}};
const Segment SGI187913[] = {{&S74456[27], 6}, {NULL}};
const Segment SGI187914[] = {{&S74456[97], 9}, {NULL}};
const Segment SGI187915[] = {{&S74456[70], 6}, {NULL}};
const Segment SGI187916[] = {{&S67871[34], 11}, {NULL}};
const Segment SGI187917[] = {{&S67871[27], 6}, {NULL}};
const Segment SGI187918[] = {{&S67871[98], 9}, {NULL}};
const Segment SGI187919[] = {{&S67871[71], 6}, {NULL}};
const Segment SGI187920[] = {{&S74482[34], 10}, {NULL}};
const Segment SGI187921[] = {{&S74482[27], 6}, {NULL}};
const Segment SGI187922[] = {{&S74482[97], 9}, {NULL}};
const Segment SGI187923[] = {{&S74482[70], 6}, {NULL}};
const Segment SGI187924[] = {{&S74490[59], 4}, {NULL}};
const Segment SGI187925[] = {{&S74490[59], 13}, {NULL}};
const Segment SGI187926[] = {{&S74490[45], 6}, {NULL}};
const Segment SGI187927[] = {{&S74490[46], 3}, {NULL}};
const Segment SGI187928[] = {{&S74490[86], 6}, {NULL}};
const Segment SGI187929[] = {{&S74490[0], 14}, {NULL}};
const Segment SGI187930[] = {{&S74491[59], 12}, {NULL}};
const Segment SGI187931[] = {{&S74491[59], 21}, {NULL}};
const Segment SGI187932[] = {{&S74491[45], 6}, {NULL}};
const Segment SGI187933[] = {{&S74491[46], 3}, {NULL}};
const Segment SGI187934[] = {{&S74491[94], 6}, {NULL}};
const Segment SGI187935[] = {{&S74491[0], 14}, {NULL}};
const Segment SGI187936[] = {{&S61866[44], 10}, {NULL}};
const Segment SGI187937[] = {{&S61866[44], 22}, {NULL}};
const Segment SGI187938[] = {{&S61866[30], 6}, {NULL}};
const Segment SGI187939[] = {{&S61866[138], 3}, {NULL}};
const Segment SGI187940[] = {{&S61866[103], 6}, {NULL}};
const Segment SGI187941[] = {{&S67898[34], 10}, {NULL}};
const Segment SGI187942[] = {{&S67898[27], 6}, {NULL}};
const Segment SGI187943[] = {{&S67898[92], 9}, {NULL}};
const Segment SGI187944[] = {{&S67898[65], 6}, {NULL}};
const Segment SGI187945[] = {{&S74500[34], 10}, {NULL}};
const Segment SGI187946[] = {{&S74500[27], 6}, {NULL}};
const Segment SGI187947[] = {{&S74500[97], 9}, {NULL}};
const Segment SGI187948[] = {{&S74500[70], 6}, {NULL}};
const Segment SGI187949[] = {{&S74501[44], 10}, {NULL}};
const Segment SGI187950[] = {{&S74501[44], 22}, {NULL}};
const Segment SGI187951[] = {{&S74501[30], 6}, {NULL}};
const Segment SGI187952[] = {{&S74501[31], 3}, {NULL}};
const Segment SGI187953[] = {{&S74501[80], 6}, {NULL}};
const Segment SGI187954[] = {{&S74506[44], 9}, {NULL}};
const Segment SGI187955[] = {{&S74506[44], 20}, {NULL}};
const Segment SGI187956[] = {{&S74506[30], 6}, {NULL}};
const Segment SGI187957[] = {{&S74506[31], 3}, {NULL}};
const Segment SGI187958[] = {{&S74506[78], 6}, {NULL}};
const Segment SGI187959[] = {{&S74507[44], 9}, {NULL}};
const Segment SGI187960[] = {{&S74507[44], 21}, {NULL}};
const Segment SGI187961[] = {{&S74507[30], 6}, {NULL}};
const Segment SGI187962[] = {{&S74507[31], 3}, {NULL}};
const Segment SGI187963[] = {{&S74507[79], 6}, {NULL}};
const Segment SGI187964[] = {{&S74508[44], 9}, {NULL}};
const Segment SGI187965[] = {{&S74508[44], 21}, {NULL}};
const Segment SGI187966[] = {{&S74508[30], 6}, {NULL}};
const Segment SGI187967[] = {{&S74508[31], 3}, {NULL}};
const Segment SGI187968[] = {{&S74508[84], 6}, {NULL}};
const Segment SGI187969[] = {{&S63209[6], 4}, {NULL}};
const Segment SGI187970[] = {{&S63209[76], 5}, {NULL}};
const Segment SGI187971[] = {{&S74515[34], 11}, {NULL}};
const Segment SGI187972[] = {{&S74515[27], 6}, {NULL}};
const Segment SGI187973[] = {{&S74515[98], 9}, {NULL}};
const Segment SGI187974[] = {{&S74515[71], 6}, {NULL}};
const Segment SGI187975[] = {{&S74517[34], 11}, {NULL}};
const Segment SGI187976[] = {{&S74517[27], 6}, {NULL}};
const Segment SGI187977[] = {{&S74517[98], 9}, {NULL}};
const Segment SGI187978[] = {{&S74517[71], 6}, {NULL}};
const Segment SGI187979[] = {{&S74518[34], 11}, {NULL}};
const Segment SGI187980[] = {{&S74518[27], 6}, {NULL}};
const Segment SGI187981[] = {{&S74518[98], 9}, {NULL}};
const Segment SGI187982[] = {{&S74518[71], 6}, {NULL}};
const Segment SGI187983[] = {{&S74538[44], 11}, {NULL}};
const Segment SGI187984[] = {{&S74538[44], 23}, {NULL}};
const Segment SGI187985[] = {{&S74538[30], 6}, {NULL}};
const Segment SGI187986[] = {{&S74538[31], 3}, {NULL}};
const Segment SGI187987[] = {{&S74538[81], 6}, {NULL}};
const Segment SGI187988[] = {{&S74539[44], 13}, {NULL}};
const Segment SGI187989[] = {{&S74539[44], 25}, {NULL}};
const Segment SGI187990[] = {{&S74539[30], 6}, {NULL}};
const Segment SGI187991[] = {{&S74539[31], 3}, {NULL}};
const Segment SGI187992[] = {{&S74539[83], 6}, {NULL}};
const Segment SGI187993[] = {{&S63245[34], 10}, {NULL}};
const Segment SGI187994[] = {{&S63245[27], 6}, {NULL}};
const Segment SGI187995[] = {{&S63245[97], 9}, {NULL}};
const Segment SGI187996[] = {{&S63245[70], 6}, {NULL}};
const Segment SGI187997[] = {{&S74546[44], 7}, {NULL}};
const Segment SGI187998[] = {{&S74546[44], 23}, {NULL}};
const Segment SGI187999[] = {{&S74546[30], 6}, {NULL}};
const Segment SGI188000[] = {{&S74546[81], 6}, {NULL}};
const Segment SGI188001[] = {{&S74546[108], 13}, {NULL}};
const Segment SGI188002[] = {{&S63297[34], 10}, {NULL}};
const Segment SGI188003[] = {{&S63297[27], 6}, {NULL}};
const Segment SGI188004[] = {{&S63297[97], 9}, {NULL}};
const Segment SGI188005[] = {{&S63297[70], 6}, {NULL}};
const Segment SGI188006[] = {{&S63298[34], 7}, {NULL}};
const Segment SGI188007[] = {{&S63298[27], 6}, {NULL}};
const Segment SGI188008[] = {{&S63298[94], 9}, {NULL}};
const Segment SGI188009[] = {{&S63298[67], 6}, {NULL}};
const Segment SGI188010[] = {{&S61993[34], 14}, {NULL}};
const Segment SGI188011[] = {{&S61993[27], 6}, {NULL}};
const Segment SGI188012[] = {{&S61993[101], 9}, {NULL}};
const Segment SGI188013[] = {{&S61993[74], 6}, {NULL}};
const Segment SGI188014[] = {{&S61993[142], 8}, {NULL}};
const Segment SGI188015[] = {{&S52777[34], 10}, {NULL}};
const Segment SGI188016[] = {{&S52777[27], 6}, {NULL}};
const Segment SGI188017[] = {{&S74611[34], 7}, {NULL}};
const Segment SGI188018[] = {{&S52777[96], 9}, {NULL}};
const Segment SGI188019[] = {{&S74611[27], 6}, {NULL}};
const Segment SGI188020[] = {{&S52777[69], 6}, {NULL}};
const Segment SGI188021[] = {{&S74611[111], 9}, {NULL}};
const Segment SGI188022[] = {{&S74611[84], 6}, {NULL}};
const Segment SGI188023[] = {{&S67971[34], 13}, {NULL}};
const Segment SGI188024[] = {{&S67971[27], 6}, {NULL}};
const Segment SGI188025[] = {{&S67971[100], 9}, {NULL}};
const Segment SGI188026[] = {{&S67971[73], 6}, {NULL}};
const Segment SGI188027[] = {{&S63322[34], 10}, {NULL}};
const Segment SGI188028[] = {{&S63322[27], 6}, {NULL}};
const Segment SGI188029[] = {{&S63322[97], 9}, {NULL}};
const Segment SGI188030[] = {{&S63322[70], 6}, {NULL}};
const Segment SGI188031[] = {{&S52787[34], 12}, {NULL}};
const Segment SGI188032[] = {{&S52787[27], 6}, {NULL}};
const Segment SGI188033[] = {{&S52787[98], 9}, {NULL}};
const Segment SGI188034[] = {{&S52787[71], 6}, {NULL}};
const Segment SGI188035[] = {{&S63328[44], 10}, {NULL}};
const Segment SGI188036[] = {{&S63328[44], 22}, {NULL}};
const Segment SGI188037[] = {{&S63328[30], 6}, {NULL}};
const Segment SGI188038[] = {{&S63328[115], 3}, {NULL}};
const Segment SGI188039[] = {{&S63328[80], 6}, {NULL}};
const Segment SGI188040[] = {{&S63329[38], 10}, {NULL}};
const Segment SGI188041[] = {{&S63329[31], 6}, {NULL}};
const Segment SGI188042[] = {{&S67981[44], 10}, {NULL}};
const Segment SGI188043[] = {{&S67981[44], 22}, {NULL}};
const Segment SGI188044[] = {{&S67981[30], 6}, {NULL}};
const Segment SGI188045[] = {{&S67981[80], 6}, {NULL}};
const Segment SGI188046[] = {{&S67981[107], 13}, {NULL}};
const Segment SGI188047[] = {{&S74629[34], 7}, {NULL}};
const Segment SGI188048[] = {{&S74629[27], 6}, {NULL}};
const Segment SGI188049[] = {{&S74629[94], 9}, {NULL}};
const Segment SGI188050[] = {{&S74629[67], 6}, {NULL}};
const Segment SGI188051[] = {{&S63337[34], 10}, {NULL}};
const Segment SGI188052[] = {{&S63337[27], 6}, {NULL}};
const Segment SGI188053[] = {{&S63337[97], 9}, {NULL}};
const Segment SGI188054[] = {{&S63337[70], 6}, {NULL}};
const Segment SGI188055[] = {{&S74635[50], 13}, {NULL}};
const Segment SGI188056[] = {{&S74635[50], 25}, {NULL}};
const Segment SGI188057[] = {{&S74635[30], 11}, {NULL}};
const Segment SGI188058[] = {{&S74635[31], 3}, {NULL}};
const Segment SGI188059[] = {{&S63338[34], 10}, {NULL}};
const Segment SGI188060[] = {{&S74635[89], 6}, {NULL}};
const Segment SGI188061[] = {{&S63338[27], 6}, {NULL}};
const Segment SGI188062[] = {{&S63338[97], 9}, {NULL}};
const Segment SGI188063[] = {{&S63338[70], 6}, {NULL}};
const Segment SGI188064[] = {{&S63339[34], 9}, {NULL}};
const Segment SGI188065[] = {{&S63339[27], 6}, {NULL}};
const Segment SGI188066[] = {{&S63339[96], 9}, {NULL}};
const Segment SGI188067[] = {{&S63339[69], 6}, {NULL}};
const Segment SGI188068[] = {{&S63340[34], 10}, {NULL}};
const Segment SGI188069[] = {{&S63340[27], 6}, {NULL}};
const Segment SGI188070[] = {{&S74639[34], 7}, {NULL}};
const Segment SGI188071[] = {{&S74639[27], 6}, {NULL}};
const Segment SGI188072[] = {{&S63340[97], 9}, {NULL}};
const Segment SGI188073[] = {{&S63340[70], 6}, {NULL}};
const Segment SGI188074[] = {{&S74639[103], 9}, {NULL}};
const Segment SGI188075[] = {{&S74639[76], 6}, {NULL}};
const Segment SGI188076[] = {{&S74640[34], 10}, {NULL}};
const Segment SGI188077[] = {{&S74640[27], 6}, {NULL}};
const Segment SGI188078[] = {{&S74640[97], 9}, {NULL}};
const Segment SGI188079[] = {{&S74640[70], 6}, {NULL}};
const Segment SGI188080[] = {{&S74645[34], 10}, {NULL}};
const Segment SGI188081[] = {{&S74645[27], 6}, {NULL}};
const Segment SGI188082[] = {{&S74645[97], 9}, {NULL}};
const Segment SGI188083[] = {{&S74645[70], 6}, {NULL}};
const Segment SGI188084[] = {{&S74646[34], 6}, {NULL}};
const Segment SGI188085[] = {{&S74646[27], 6}, {NULL}};
const Segment SGI188086[] = {{&S74646[93], 9}, {NULL}};
const Segment SGI188087[] = {{&S74646[66], 6}, {NULL}};
const Segment SGI188088[] = {{&S63362[44], 10}, {NULL}};
const Segment SGI188089[] = {{&S63362[44], 22}, {NULL}};
const Segment SGI188090[] = {{&S63362[30], 6}, {NULL}};
const Segment SGI188091[] = {{&S63362[131], 3}, {NULL}};
const Segment SGI188092[] = {{&S63362[96], 6}, {NULL}};
const Segment SGI188093[] = {{&S63362[68], 8}, {NULL}};
const Segment SGI188094[] = {{&S63366[34], 10}, {NULL}};
const Segment SGI188095[] = {{&S63366[27], 6}, {NULL}};
const Segment SGI188096[] = {{&S63366[97], 9}, {NULL}};
const Segment SGI188097[] = {{&S63366[70], 6}, {NULL}};
const Segment SGI188098[] = {{&S74665[34], 6}, {NULL}};
const Segment SGI188099[] = {{&S74665[27], 6}, {NULL}};
const Segment SGI188100[] = {{&S74665[93], 9}, {NULL}};
const Segment SGI188101[] = {{&S74665[66], 6}, {NULL}};
const Segment SGI188102[] = {{&S63367[34], 10}, {NULL}};
const Segment SGI188103[] = {{&S63367[27], 6}, {NULL}};
const Segment SGI188104[] = {{&S63367[97], 9}, {NULL}};
const Segment SGI188105[] = {{&S63367[70], 6}, {NULL}};
const Segment SGI188106[] = {{&S63368[34], 10}, {NULL}};
const Segment SGI188107[] = {{&S63368[27], 6}, {NULL}};
const Segment SGI188108[] = {{&S63368[97], 9}, {NULL}};
const Segment SGI188109[] = {{&S63368[70], 6}, {NULL}};
const Segment SGI188110[] = {{&S74668[34], 12}, {NULL}};
const Segment SGI188111[] = {{&S74668[27], 6}, {NULL}};
const Segment SGI188112[] = {{&S74668[99], 9}, {NULL}};
const Segment SGI188113[] = {{&S74668[72], 6}, {NULL}};
const Segment SGI188114[] = {{&S68038[44], 12}, {NULL}};
const Segment SGI188115[] = {{&S68038[44], 24}, {NULL}};
const Segment SGI188116[] = {{&S68038[30], 6}, {NULL}};
const Segment SGI188117[] = {{&S68038[117], 3}, {NULL}};
const Segment SGI188118[] = {{&S68038[82], 6}, {NULL}};
const Segment SGI188119[] = {{&S68039[38], 12}, {NULL}};
const Segment SGI188120[] = {{&S68039[31], 6}, {NULL}};
const Segment SGI188121[] = {{&S63383[44], 11}, {NULL}};
const Segment SGI188122[] = {{&S63383[44], 23}, {NULL}};
const Segment SGI188123[] = {{&S63383[30], 6}, {NULL}};
const Segment SGI188124[] = {{&S63383[116], 3}, {NULL}};
const Segment SGI188125[] = {{&S63383[81], 6}, {NULL}};
const Segment SGI188126[] = {{&S63384[38], 11}, {NULL}};
const Segment SGI188127[] = {{&S63384[31], 6}, {NULL}};
const Segment SGI188128[] = {{&S74685[34], 14}, {NULL}};
const Segment SGI188129[] = {{&S74685[27], 6}, {NULL}};
const Segment SGI188130[] = {{&S74685[101], 9}, {NULL}};
const Segment SGI188131[] = {{&S74685[74], 6}, {NULL}};
const Segment SGI188132[] = {{&S63388[34], 10}, {NULL}};
const Segment SGI188133[] = {{&S63388[27], 6}, {NULL}};
const Segment SGI188134[] = {{&S63388[97], 9}, {NULL}};
const Segment SGI188135[] = {{&S63388[70], 6}, {NULL}};
const Segment SGI188136[] = {{&S63389[34], 11}, {NULL}};
const Segment SGI188137[] = {{&S63389[27], 6}, {NULL}};
const Segment SGI188138[] = {{&S63389[98], 9}, {NULL}};
const Segment SGI188139[] = {{&S63389[71], 6}, {NULL}};
const Segment SGI188140[] = {{&S68057[44], 18}, {NULL}};
const Segment SGI188141[] = {{&S68057[44], 30}, {NULL}};
const Segment SGI188142[] = {{&S68057[30], 6}, {NULL}};
const Segment SGI188143[] = {{&S74691[44], 11}, {NULL}};
const Segment SGI188144[] = {{&S74691[44], 23}, {NULL}};
const Segment SGI188145[] = {{&S74691[30], 6}, {NULL}};
const Segment SGI188146[] = {{&S68057[123], 3}, {NULL}};
const Segment SGI188147[] = {{&S68057[88], 6}, {NULL}};
const Segment SGI188148[] = {{&S74691[31], 3}, {NULL}};
const Segment SGI188149[] = {{&S74691[106], 6}, {NULL}};
const Segment SGI188150[] = {{&S63390[34], 11}, {NULL}};
const Segment SGI188151[] = {{&S74691[69], 8}, {&S74691[69], 8}, {NULL}};
const Segment SGI188152[] = {{&S63390[27], 6}, {NULL}};
const Segment SGI188153[] = {{&S63390[98], 9}, {NULL}};
const Segment SGI188154[] = {{&S63390[71], 6}, {NULL}};
const Segment SGI188155[] = {{&S68058[44], 20}, {NULL}};
const Segment SGI188156[] = {{&S68058[44], 32}, {NULL}};
const Segment SGI188157[] = {{&S68058[30], 6}, {NULL}};
const Segment SGI188158[] = {{&S74692[44], 11}, {NULL}};
const Segment SGI188159[] = {{&S74692[44], 23}, {NULL}};
const Segment SGI188160[] = {{&S74692[30], 6}, {NULL}};
const Segment SGI188161[] = {{&S68058[125], 3}, {NULL}};
const Segment SGI188162[] = {{&S62093[44], 15}, {NULL}};
const Segment SGI188163[] = {{&S68058[90], 6}, {NULL}};
const Segment SGI188164[] = {{&S62093[44], 27}, {NULL}};
const Segment SGI188165[] = {{&S63391[34], 11}, {NULL}};
const Segment SGI188166[] = {{&S74692[31], 3}, {NULL}};
const Segment SGI188167[] = {{&S62093[30], 6}, {NULL}};
const Segment SGI188168[] = {{&S63391[27], 6}, {NULL}};
const Segment SGI188169[] = {{&S74692[96], 6}, {NULL}};
const Segment SGI188170[] = {{&S74692[69], 8}, {NULL}};
const Segment SGI188171[] = {{&S62093[120], 3}, {NULL}};
const Segment SGI188172[] = {{&S63391[98], 9}, {NULL}};
const Segment SGI188173[] = {{&S62093[85], 6}, {NULL}};
const Segment SGI188174[] = {{&S63391[71], 6}, {NULL}};
const Segment SGI188175[] = {{&S68059[44], 24}, {NULL}};
const Segment SGI188176[] = {{&S68059[44], 36}, {NULL}};
const Segment SGI188177[] = {{&S68059[30], 6}, {NULL}};
const Segment SGI188178[] = {{&S74693[44], 10}, {NULL}};
const Segment SGI188179[] = {{&S62094[38], 15}, {NULL}};
const Segment SGI188180[] = {{&S74693[44], 27}, {NULL}};
const Segment SGI188181[] = {{&S62094[31], 6}, {NULL}};
const Segment SGI188182[] = {{&S68059[129], 3}, {NULL}};
const Segment SGI188183[] = {{&S74693[30], 6}, {NULL}};
const Segment SGI188184[] = {{&S68059[94], 6}, {NULL}};
const Segment SGI188185[] = {{&S74693[31], 3}, {NULL}};
const Segment SGI188186[] = {{&S74693[85], 5}, {NULL}};
const Segment SGI188187[] = {{&S68060[44], 10}, {NULL}};
const Segment SGI188188[] = {{&S68060[44], 22}, {NULL}};
const Segment SGI188189[] = {{&S68060[30], 6}, {NULL}};
const Segment SGI188190[] = {{&S68060[115], 3}, {NULL}};
const Segment SGI188191[] = {{&S68060[80], 6}, {NULL}};
const Segment SGI188192[] = {{&S74695[44], 12}, {NULL}};
const Segment SGI188193[] = {{&S74695[44], 24}, {NULL}};
const Segment SGI188194[] = {{&S74695[30], 6}, {NULL}};
const Segment SGI188195[] = {{&S74695[31], 3}, {NULL}};
const Segment SGI188196[] = {{&S74695[82], 6}, {NULL}};
const Segment SGI188197[] = {{&S68061[44], 9}, {NULL}};
const Segment SGI188198[] = {{&S68061[44], 21}, {NULL}};
const Segment SGI188199[] = {{&S68061[30], 6}, {NULL}};
const Segment SGI188200[] = {{&S68061[114], 3}, {NULL}};
const Segment SGI188201[] = {{&S68061[79], 6}, {NULL}};
const Segment SGI188202[] = {{&S68062[38], 9}, {NULL}};
const Segment SGI188203[] = {{&S68062[31], 6}, {NULL}};
const Segment SGI188204[] = {{&S68065[44], 10}, {NULL}};
const Segment SGI188205[] = {{&S68065[44], 22}, {NULL}};
const Segment SGI188206[] = {{&S68065[30], 6}, {NULL}};
const Segment SGI188207[] = {{&S68065[113], 3}, {NULL}};
const Segment SGI188208[] = {{&S68065[80], 5}, {NULL}};
const Segment SGI188209[] = {{&S68065[117], 14}, {NULL}};
const Segment SGI188210[] = {{&S62107[44], 20}, {NULL}};
const Segment SGI188211[] = {{&S62107[44], 32}, {NULL}};
const Segment SGI188212[] = {{&S62107[30], 6}, {NULL}};
const Segment SGI188213[] = {{&S62107[148], 3}, {NULL}};
const Segment SGI188214[] = {{&S62107[113], 6}, {NULL}};
const Segment SGI188215[] = {{&S68067[44], 10}, {NULL}};
const Segment SGI188216[] = {{&S62107[78], 8}, {NULL}};
const Segment SGI188217[] = {{&S68067[44], 22}, {NULL}};
const Segment SGI188218[] = {{&S68067[30], 6}, {NULL}};
const Segment SGI188219[] = {{&S68067[113], 3}, {NULL}};
const Segment SGI188220[] = {{&S68067[80], 5}, {NULL}};
const Segment SGI188221[] = {{&S68067[117], 14}, {NULL}};
const Segment SGI188222[] = {{&S68072[34], 6}, {NULL}};
const Segment SGI188223[] = {{&S68072[27], 6}, {NULL}};
const Segment SGI188224[] = {{&S68072[93], 9}, {NULL}};
const Segment SGI188225[] = {{&S68072[66], 6}, {NULL}};
const Segment SGI188226[] = {{&S66057[32], 20}, {NULL}};
const Segment SGI188227[] = {{&S66057[27], 4}, {NULL}};
const Segment SGI188228[] = {{&S66057[105], 9}, {NULL}};
const Segment SGI188229[] = {{&S66057[78], 6}, {NULL}};
const Segment SGI188230[] = {{&S63416[34], 8}, {NULL}};
const Segment SGI188231[] = {{&S68074[34], 7}, {NULL}};
const Segment SGI188232[] = {{&S63416[27], 6}, {NULL}};
const Segment SGI188233[] = {{&S68074[27], 6}, {NULL}};
const Segment SGI188234[] = {{&S63416[103], 9}, {NULL}};
const Segment SGI188235[] = {{&S63416[76], 6}, {NULL}};
const Segment SGI188236[] = {{&S68074[94], 9}, {NULL}};
const Segment SGI188237[] = {{&S68074[67], 6}, {NULL}};
const Segment SGI188238[] = {{&S68077[34], 16}, {NULL}};
const Segment SGI188239[] = {{&S68077[27], 6}, {NULL}};
const Segment SGI188240[] = {{&S68077[103], 9}, {NULL}};
const Segment SGI188241[] = {{&S68077[76], 6}, {NULL}};
const Segment SGI188242[] = {{&S52913[34], 10}, {NULL}};
const Segment SGI188243[] = {{&S52913[27], 6}, {NULL}};
const Segment SGI188244[] = {{&S52913[96], 9}, {NULL}};
const Segment SGI188245[] = {{&S52913[69], 6}, {NULL}};
const Segment SGI188246[] = {{&S68078[34], 10}, {NULL}};
const Segment SGI188247[] = {{&S68078[27], 6}, {NULL}};
const Segment SGI188248[] = {{&S68078[97], 9}, {NULL}};
const Segment SGI188249[] = {{&S68078[70], 6}, {NULL}};
const Segment SGI188250[] = {{&S68079[34], 13}, {NULL}};
const Segment SGI188251[] = {{&S68079[27], 6}, {NULL}};
const Segment SGI188252[] = {{&S68079[100], 9}, {NULL}};
const Segment SGI188253[] = {{&S68079[73], 6}, {NULL}};
const Segment SGI188254[] = {{&S68080[34], 9}, {NULL}};
const Segment SGI188255[] = {{&S68080[27], 6}, {NULL}};
const Segment SGI188256[] = {{&S68080[96], 9}, {NULL}};
const Segment SGI188257[] = {{&S68080[69], 6}, {NULL}};
const Segment SGI188258[] = {{&S68081[34], 4}, {NULL}};
const Segment SGI188259[] = {{&S68081[27], 6}, {NULL}};
const Segment SGI188260[] = {{&S68081[87], 9}, {NULL}};
const Segment SGI188261[] = {{&S74731[44], 15}, {NULL}};
const Segment SGI188262[] = {{&S68081[60], 6}, {NULL}};
const Segment SGI188263[] = {{&S74731[44], 27}, {NULL}};
const Segment SGI188264[] = {{&S74731[30], 6}, {NULL}};
const Segment SGI188265[] = {{&S74731[31], 3}, {NULL}};
const Segment SGI188266[] = {{&S74731[85], 6}, {NULL}};
const Segment SGI188267[] = {{&S74732[44], 15}, {NULL}};
const Segment SGI188268[] = {{&S74732[44], 27}, {NULL}};
const Segment SGI188269[] = {{&S74732[30], 6}, {NULL}};
const Segment SGI188270[] = {{&S74732[31], 3}, {NULL}};
const Segment SGI188271[] = {{&S74732[85], 6}, {NULL}};
const Segment SGI188272[] = {{&S68084[34], 10}, {NULL}};
const Segment SGI188273[] = {{&S68084[27], 6}, {NULL}};
const Segment SGI188274[] = {{&S68084[97], 9}, {NULL}};
const Segment SGI188275[] = {{&S68084[70], 6}, {NULL}};
const Segment SGI188276[] = {{&S52927[34], 5}, {NULL}};
const Segment SGI188277[] = {{&S52927[27], 6}, {NULL}};
const Segment SGI188278[] = {{&S52927[92], 9}, {NULL}};
const Segment SGI188279[] = {{&S52927[65], 6}, {NULL}};
const Segment SGI188280[] = {{&S52928[34], 9}, {NULL}};
const Segment SGI188281[] = {{&S52928[27], 6}, {NULL}};
const Segment SGI188282[] = {{&S74738[44], 11}, {NULL}};
const Segment SGI188283[] = {{&S74738[44], 23}, {NULL}};
const Segment SGI188284[] = {{&S74738[30], 6}, {NULL}};
const Segment SGI188285[] = {{&S52928[95], 9}, {NULL}};
const Segment SGI188286[] = {{&S52928[68], 6}, {NULL}};
const Segment SGI188287[] = {{&S74738[31], 3}, {NULL}};
const Segment SGI188288[] = {{&S74738[81], 6}, {NULL}};
const Segment SGI188289[] = {{&S52929[34], 14}, {NULL}};
const Segment SGI188290[] = {{&S52929[27], 6}, {NULL}};
const Segment SGI188291[] = {{&S52929[100], 9}, {NULL}};
const Segment SGI188292[] = {{&S52929[73], 6}, {NULL}};
const Segment SGI188293[] = {{&S52931[34], 11}, {NULL}};
const Segment SGI188294[] = {{&S52931[27], 6}, {NULL}};
const Segment SGI188295[] = {{&S52931[97], 9}, {NULL}};
const Segment SGI188296[] = {{&S52931[70], 6}, {NULL}};
const Segment SGI188297[] = {{&S74743[34], 16}, {NULL}};
const Segment SGI188298[] = {{&S74743[27], 6}, {NULL}};
const Segment SGI188299[] = {{&S74743[103], 9}, {NULL}};
const Segment SGI188300[] = {{&S74743[76], 6}, {NULL}};
const Segment SGI188301[] = {{&S52932[44], 10}, {NULL}};
const Segment SGI188302[] = {{&S52932[44], 21}, {NULL}};
const Segment SGI188303[] = {{&S52932[30], 6}, {NULL}};
const Segment SGI188304[] = {{&S52932[114], 3}, {NULL}};
const Segment SGI188305[] = {{&S52932[79], 6}, {NULL}};
const Segment SGI188306[] = {{&S74744[34], 10}, {NULL}};
const Segment SGI188307[] = {{&S74744[27], 6}, {NULL}};
const Segment SGI188308[] = {{&S74744[97], 9}, {NULL}};
const Segment SGI188309[] = {{&S74744[70], 6}, {NULL}};
const Segment SGI188310[] = {{&S52933[38], 10}, {NULL}};
const Segment SGI188311[] = {{&S52933[31], 6}, {NULL}};
const Segment SGI188312[] = {{&S52934[44], 31}, {NULL}};
const Segment SGI188313[] = {{&S52934[44], 42}, {NULL}};
const Segment SGI188314[] = {{&S52934[30], 6}, {NULL}};
const Segment SGI188315[] = {{&S74746[34], 8}, {NULL}};
const Segment SGI188316[] = {{&S74746[27], 6}, {NULL}};
const Segment SGI188317[] = {{&S52934[135], 3}, {NULL}};
const Segment SGI188318[] = {{&S52934[100], 6}, {NULL}};
const Segment SGI188319[] = {{&S74746[107], 9}, {NULL}};
const Segment SGI188320[] = {{&S74746[80], 6}, {NULL}};
const Segment SGI188321[] = {{&S68104[74], 17}, {NULL}};
const Segment SGI188322[] = {{&S68104[44], 22}, {&S68104[74], 29}, {NULL}};
const Segment SGI188323[] = {{&S68104[30], 6}, {NULL}};
const Segment SGI188324[] = {{&S68104[151], 3}, {NULL}};
const Segment SGI188325[] = {{&S68104[117], 6}, {NULL}};
const Segment SGI188326[] = {{&S74755[34], 10}, {NULL}};
const Segment SGI188327[] = {{&S74755[27], 6}, {NULL}};
const Segment SGI188328[] = {{&S74755[97], 9}, {NULL}};
const Segment SGI188329[] = {{&S74755[70], 6}, {NULL}};
const Segment SGI188330[] = {{&S74758[34], 14}, {NULL}};
const Segment SGI188331[] = {{&S74758[27], 6}, {NULL}};
const Segment SGI188332[] = {{&S74758[101], 9}, {NULL}};
const Segment SGI188333[] = {{&S74758[74], 6}, {NULL}};
const Segment SGI188334[] = {{&S68117[44], 7}, {NULL}};
const Segment SGI188335[] = {{&S68117[44], 19}, {NULL}};
const Segment SGI188336[] = {{&S68117[30], 6}, {NULL}};
const Segment SGI188337[] = {{&S68117[112], 3}, {NULL}};
const Segment SGI188338[] = {{&S68117[77], 6}, {NULL}};
const Segment SGI188339[] = {{&S68118[38], 7}, {NULL}};
const Segment SGI188340[] = {{&S68118[31], 6}, {NULL}};
const Segment SGI188341[] = {{&S74779[44], 15}, {NULL}};
const Segment SGI188342[] = {{&S74779[44], 27}, {NULL}};
const Segment SGI188343[] = {{&S74779[30], 6}, {NULL}};
const Segment SGI188344[] = {{&S74779[31], 3}, {NULL}};
const Segment SGI188345[] = {{&S74779[85], 6}, {NULL}};
const Segment SGI188346[] = {{&S74780[44], 15}, {NULL}};
const Segment SGI188347[] = {{&S74780[44], 27}, {NULL}};
const Segment SGI188348[] = {{&S74780[30], 6}, {NULL}};
const Segment SGI188349[] = {{&S74780[31], 3}, {NULL}};
const Segment SGI188350[] = {{&S74780[85], 6}, {NULL}};
const Segment SGI188351[] = {{&S52973[44], 19}, {NULL}};
const Segment SGI188352[] = {{&S52973[44], 30}, {NULL}};
const Segment SGI188353[] = {{&S52973[30], 6}, {NULL}};
const Segment SGI188354[] = {{&S52973[123], 3}, {NULL}};
const Segment SGI188355[] = {{&S52973[88], 6}, {NULL}};
const Segment SGI188356[] = {{&S52974[38], 19}, {NULL}};
const Segment SGI188357[] = {{&S52974[31], 6}, {NULL}};
const Segment SGI188358[] = {{&S52975[44], 13}, {NULL}};
const Segment SGI188359[] = {{&S52975[44], 24}, {NULL}};
const Segment SGI188360[] = {{&S52975[30], 6}, {NULL}};
const Segment SGI188361[] = {{&S52975[117], 3}, {NULL}};
const Segment SGI188362[] = {{&S52975[82], 6}, {NULL}};
const Segment SGI188363[] = {{&S52976[44], 13}, {NULL}};
const Segment SGI188364[] = {{&S52976[44], 25}, {NULL}};
const Segment SGI188365[] = {{&S52976[30], 6}, {NULL}};
const Segment SGI188366[] = {{&S52976[118], 3}, {NULL}};
const Segment SGI188367[] = {{&S52976[83], 6}, {NULL}};
const Segment SGI188368[] = {{&S74787[44], 10}, {NULL}};
const Segment SGI188369[] = {{&S74787[44], 22}, {NULL}};
const Segment SGI188370[] = {{&S74787[30], 6}, {NULL}};
const Segment SGI188371[] = {{&S74787[80], 6}, {NULL}};
const Segment SGI188372[] = {{&S74787[107], 13}, {NULL}};
const Segment SGI188373[] = {{&S74797[44], 12}, {NULL}};
const Segment SGI188374[] = {{&S74797[44], 24}, {NULL}};
const Segment SGI188375[] = {{&S74797[30], 6}, {NULL}};
const Segment SGI188376[] = {{&S74797[31], 3}, {NULL}};
const Segment SGI188377[] = {{&S74797[97], 6}, {NULL}};
const Segment SGI188378[] = {{&S74797[70], 8}, {NULL}};
const Segment SGI188379[] = {{&S74798[44], 12}, {NULL}};
const Segment SGI188380[] = {{&S74798[44], 24}, {NULL}};
const Segment SGI188381[] = {{&S74798[30], 6}, {NULL}};
const Segment SGI188382[] = {{&S74798[31], 3}, {NULL}};
const Segment SGI188383[] = {{&S74798[101], 6}, {NULL}};
const Segment SGI188384[] = {{&S74798[70], 8}, {NULL}};
const Segment SGI188385[] = {{&S74799[44], 12}, {NULL}};
const Segment SGI188386[] = {{&S74799[44], 24}, {NULL}};
const Segment SGI188387[] = {{&S74799[30], 6}, {NULL}};
const Segment SGI188388[] = {{&S74799[31], 3}, {NULL}};
const Segment SGI188389[] = {{&S74799[101], 6}, {NULL}};
const Segment SGI188390[] = {{&S74799[70], 8}, {NULL}};
const Segment SGI188391[] = {{&S52997[44], 10}, {NULL}};
const Segment SGI188392[] = {{&S52997[44], 21}, {NULL}};
const Segment SGI188393[] = {{&S52997[30], 6}, {NULL}};
const Segment SGI188394[] = {{&S52997[114], 3}, {NULL}};
const Segment SGI188395[] = {{&S52997[79], 6}, {NULL}};
const Segment SGI188396[] = {{&S52998[44], 10}, {NULL}};
const Segment SGI188397[] = {{&S52998[44], 22}, {NULL}};
const Segment SGI188398[] = {{&S52998[30], 6}, {NULL}};
const Segment SGI188399[] = {{&S74803[44], 12}, {NULL}};
const Segment SGI188400[] = {{&S74803[44], 24}, {NULL}};
const Segment SGI188401[] = {{&S74803[30], 6}, {NULL}};
const Segment SGI188402[] = {{&S52998[115], 3}, {NULL}};
const Segment SGI188403[] = {{&S52998[80], 6}, {NULL}};
const Segment SGI188404[] = {{&S74803[31], 3}, {NULL}};
const Segment SGI188405[] = {{&S74803[97], 6}, {NULL}};
const Segment SGI188406[] = {{&S74803[70], 8}, {NULL}};
const Segment SGI188407[] = {{&S68171[44], 15}, {NULL}};
const Segment SGI188408[] = {{&S68171[44], 27}, {NULL}};
const Segment SGI188409[] = {{&S52999[38], 10}, {NULL}};
const Segment SGI188410[] = {{&S68171[30], 6}, {NULL}};
const Segment SGI188411[] = {{&S52999[31], 6}, {NULL}};
const Segment SGI188412[] = {{&S68171[120], 3}, {NULL}};
const Segment SGI188413[] = {{&S68171[85], 6}, {NULL}};
const Segment SGI188414[] = {{&S74805[34], 8}, {NULL}};
const Segment SGI188415[] = {{&S74805[27], 6}, {NULL}};
const Segment SGI188416[] = {{&S68172[44], 7}, {NULL}};
const Segment SGI188417[] = {{&S68172[44], 19}, {NULL}};
const Segment SGI188418[] = {{&S68172[30], 6}, {NULL}};
const Segment SGI188419[] = {{&S74805[101], 9}, {NULL}};
const Segment SGI188420[] = {{&S74805[74], 6}, {NULL}};
const Segment SGI188421[] = {{&S68172[112], 3}, {NULL}};
const Segment SGI188422[] = {{&S68172[77], 6}, {NULL}};
const Segment SGI188423[] = {{&S68173[34], 13}, {NULL}};
const Segment SGI188424[] = {{&S68173[27], 6}, {NULL}};
const Segment SGI188425[] = {{&S68173[114], 9}, {NULL}};
const Segment SGI188426[] = {{&S68173[87], 6}, {NULL}};
const Segment SGI188427[] = {{&S68174[34], 12}, {NULL}};
const Segment SGI188428[] = {{&S68174[27], 6}, {NULL}};
const Segment SGI188429[] = {{&S68174[112], 9}, {NULL}};
const Segment SGI188430[] = {{&S68174[85], 6}, {NULL}};
const Segment SGI188431[] = {{&S68178[44], 10}, {NULL}};
const Segment SGI188432[] = {{&S68178[44], 22}, {NULL}};
const Segment SGI188433[] = {{&S68178[30], 6}, {NULL}};
const Segment SGI188434[] = {{&S68178[115], 3}, {NULL}};
const Segment SGI188435[] = {{&S74814[44], 10}, {NULL}};
const Segment SGI188436[] = {{&S68178[80], 6}, {NULL}};
const Segment SGI188437[] = {{&S74814[44], 19}, {NULL}};
const Segment SGI188438[] = {{&S74814[30], 6}, {NULL}};
const Segment SGI188439[] = {{&S74814[31], 3}, {NULL}};
const Segment SGI188440[] = {{&S74814[77], 6}, {NULL}};
const Segment SGI188441[] = {{&S62254[34], 6}, {NULL}};
const Segment SGI188442[] = {{&S62254[27], 6}, {NULL}};
const Segment SGI188443[] = {{&S62254[93], 9}, {NULL}};
const Segment SGI188444[] = {{&S62254[66], 6}, {NULL}};
const Segment SGI188445[] = {{&S63460[34], 9}, {NULL}};
const Segment SGI188446[] = {{&S63460[27], 6}, {NULL}};
const Segment SGI188447[] = {{&S63460[96], 9}, {NULL}};
const Segment SGI188448[] = {{&S63460[69], 6}, {NULL}};
const Segment SGI188449[] = {{&S63461[34], 9}, {NULL}};
const Segment SGI188450[] = {{&S63461[27], 6}, {NULL}};
const Segment SGI188451[] = {{&S63461[96], 9}, {NULL}};
const Segment SGI188452[] = {{&S63461[69], 6}, {NULL}};
const Segment SGI188453[] = {{&S63468[44], 11}, {NULL}};
const Segment SGI188454[] = {{&S63468[44], 23}, {NULL}};
const Segment SGI188455[] = {{&S63468[30], 6}, {NULL}};
const Segment SGI188456[] = {{&S63468[81], 6}, {NULL}};
const Segment SGI188457[] = {{&S63468[108], 13}, {NULL}};
const Segment SGI188458[] = {{&S63469[44], 10}, {NULL}};
const Segment SGI188459[] = {{&S63469[44], 22}, {NULL}};
const Segment SGI188460[] = {{&S63469[30], 6}, {NULL}};
const Segment SGI188461[] = {{&S63469[80], 6}, {NULL}};
const Segment SGI188462[] = {{&S63469[107], 13}, {NULL}};
const Segment SGI188463[] = {{&S63471[34], 10}, {NULL}};
const Segment SGI188464[] = {{&S63471[27], 6}, {NULL}};
const Segment SGI188465[] = {{&S63471[97], 9}, {NULL}};
const Segment SGI188466[] = {{&S63471[70], 6}, {NULL}};
const Segment SGI188467[] = {{&S68234[34], 6}, {NULL}};
const Segment SGI188468[] = {{&S68234[27], 6}, {NULL}};
const Segment SGI188469[] = {{&S68234[93], 9}, {NULL}};
const Segment SGI188470[] = {{&S68234[66], 6}, {NULL}};
const Segment SGI188471[] = {{&S74871[34], 10}, {NULL}};
const Segment SGI188472[] = {{&S63474[34], 10}, {NULL}};
const Segment SGI188473[] = {{&S74871[27], 6}, {NULL}};
const Segment SGI188474[] = {{&S63474[27], 6}, {NULL}};
const Segment SGI188475[] = {{&S74871[97], 9}, {NULL}};
const Segment SGI188476[] = {{&S74871[70], 6}, {NULL}};
const Segment SGI188477[] = {{&S63474[97], 9}, {NULL}};
const Segment SGI188478[] = {{&S63474[70], 6}, {NULL}};
const Segment SGI188479[] = {{&S63474[138], 8}, {NULL}};
const Segment SGI188480[] = {{&S74872[34], 11}, {NULL}};
const Segment SGI188481[] = {{&S74872[27], 6}, {NULL}};
const Segment SGI188482[] = {{&S74872[98], 9}, {NULL}};
const Segment SGI188483[] = {{&S74872[71], 6}, {NULL}};
const Segment SGI188484[] = {{&S68238[34], 11}, {NULL}};
const Segment SGI188485[] = {{&S68238[27], 6}, {NULL}};
const Segment SGI188486[] = {{&S68238[98], 9}, {NULL}};
const Segment SGI188487[] = {{&S68238[71], 6}, {NULL}};
const Segment SGI188488[] = {{&S68239[34], 11}, {NULL}};
const Segment SGI188489[] = {{&S68239[27], 6}, {NULL}};
const Segment SGI188490[] = {{&S68239[98], 9}, {NULL}};
const Segment SGI188491[] = {{&S68239[71], 6}, {NULL}};
const Segment SGI188492[] = {{&S68240[34], 11}, {NULL}};
const Segment SGI188493[] = {{&S68240[27], 6}, {NULL}};
const Segment SGI188494[] = {{&S68240[98], 9}, {NULL}};
const Segment SGI188495[] = {{&S68240[71], 6}, {NULL}};
const Segment SGI188496[] = {{&S74887[44], 7}, {NULL}};
const Segment SGI188497[] = {{&S74887[44], 24}, {NULL}};
const Segment SGI188498[] = {{&S74887[30], 6}, {NULL}};
const Segment SGI188499[] = {{&S74887[82], 6}, {NULL}};
const Segment SGI188500[] = {{&S74887[109], 13}, {NULL}};
const Segment SGI188501[] = {{&S63504[34], 10}, {NULL}};
const Segment SGI188502[] = {{&S63504[27], 6}, {NULL}};
const Segment SGI188503[] = {{&S63504[97], 9}, {NULL}};
const Segment SGI188504[] = {{&S63504[70], 6}, {NULL}};
const Segment SGI188505[] = {{&S63505[34], 11}, {NULL}};
const Segment SGI188506[] = {{&S63505[27], 6}, {NULL}};
const Segment SGI188507[] = {{&S63505[98], 9}, {NULL}};
const Segment SGI188508[] = {{&S63505[71], 6}, {NULL}};
const Segment SGI188509[] = {{&S53118[34], 10}, {NULL}};
const Segment SGI188510[] = {{&S53118[27], 6}, {NULL}};
const Segment SGI188511[] = {{&S53118[96], 9}, {NULL}};
const Segment SGI188512[] = {{&S53118[69], 6}, {NULL}};
const Segment SGI188513[] = {{&S53119[34], 11}, {NULL}};
const Segment SGI188514[] = {{&S53119[27], 6}, {NULL}};
const Segment SGI188515[] = {{&S53119[97], 9}, {NULL}};
const Segment SGI188516[] = {{&S53119[70], 6}, {NULL}};
const Segment SGI188517[] = {{&S53120[34], 10}, {NULL}};
const Segment SGI188518[] = {{&S53120[27], 6}, {NULL}};
const Segment SGI188519[] = {{&S53120[97], 9}, {NULL}};
const Segment SGI188520[] = {{&S53120[70], 6}, {NULL}};
const Segment SGI188521[] = {{&S63516[44], 10}, {NULL}};
const Segment SGI188522[] = {{&S63516[44], 22}, {NULL}};
const Segment SGI188523[] = {{&S63516[30], 6}, {NULL}};
const Segment SGI188524[] = {{&S63516[131], 3}, {NULL}};
const Segment SGI188525[] = {{&S63516[96], 6}, {NULL}};
const Segment SGI188526[] = {{&S63516[68], 8}, {NULL}};
const Segment SGI188527[] = {{&S63517[42], 10}, {NULL}};
const Segment SGI188528[] = {{&S63517[30], 6}, {NULL}};
const Segment SGI188529[] = {{&S63517[129], 3}, {NULL}};
const Segment SGI188530[] = {{&S63517[94], 6}, {NULL}};
const Segment SGI188531[] = {{&S63517[66], 8}, {NULL}};
const Segment SGI188532[] = {{&S68287[44], 9}, {NULL}};
const Segment SGI188533[] = {{&S68287[44], 21}, {NULL}};
const Segment SGI188534[] = {{&S68287[30], 6}, {NULL}};
const Segment SGI188535[] = {{&S68287[114], 3}, {NULL}};
const Segment SGI188536[] = {{&S68287[79], 6}, {NULL}};
const Segment SGI188537[] = {{&S68287[142], 3}, {NULL}};
const Segment SGI188538[] = {{&S68289[44], 9}, {NULL}};
const Segment SGI188539[] = {{&S68289[44], 21}, {NULL}};
const Segment SGI188540[] = {{&S68289[30], 6}, {NULL}};
const Segment SGI188541[] = {{&S68289[114], 3}, {NULL}};
const Segment SGI188542[] = {{&S68289[79], 6}, {NULL}};
const Segment SGI188543[] = {{&S68289[142], 3}, {NULL}};
const Segment SGI188544[] = {{&S68290[44], 9}, {NULL}};
const Segment SGI188545[] = {{&S63529[44], 18}, {NULL}};
const Segment SGI188546[] = {{&S68290[44], 21}, {NULL}};
const Segment SGI188547[] = {{&S63529[44], 30}, {NULL}};
const Segment SGI188548[] = {{&S68290[30], 6}, {NULL}};
const Segment SGI188549[] = {{&S63529[30], 6}, {NULL}};
const Segment SGI188550[] = {{&S63529[123], 3}, {NULL}};
const Segment SGI188551[] = {{&S68290[114], 3}, {NULL}};
const Segment SGI188552[] = {{&S63529[88], 6}, {NULL}};
const Segment SGI188553[] = {{&S68290[79], 6}, {NULL}};
const Segment SGI188554[] = {{&S68290[142], 3}, {NULL}};
const Segment SGI188555[] = {{&S63530[38], 18}, {NULL}};
const Segment SGI188556[] = {{&S63530[31], 6}, {NULL}};
const Segment SGI188557[] = {{&S63532[34], 8}, {NULL}};
const Segment SGI188558[] = {{&S63532[27], 6}, {NULL}};
const Segment SGI188559[] = {{&S68294[44], 7}, {NULL}};
const Segment SGI188560[] = {{&S68294[44], 19}, {NULL}};
const Segment SGI188561[] = {{&S68294[30], 6}, {NULL}};
const Segment SGI188562[] = {{&S63532[100], 9}, {NULL}};
const Segment SGI188563[] = {{&S63532[73], 6}, {NULL}};
const Segment SGI188564[] = {{&S68294[112], 3}, {NULL}};
const Segment SGI188565[] = {{&S68294[77], 6}, {NULL}};
const Segment SGI188566[] = {{&S68295[44], 8}, {NULL}};
const Segment SGI188567[] = {{&S68295[44], 20}, {NULL}};
const Segment SGI188568[] = {{&S68295[30], 6}, {NULL}};
const Segment SGI188569[] = {{&S68295[113], 3}, {NULL}};
const Segment SGI188570[] = {{&S68295[78], 6}, {NULL}};
const Segment SGI188571[] = {{&S68297[34], 6}, {NULL}};
const Segment SGI188572[] = {{&S68297[27], 6}, {NULL}};
const Segment SGI188573[] = {{&S68297[93], 9}, {NULL}};
const Segment SGI188574[] = {{&S68297[66], 6}, {NULL}};
const Segment SGI188575[] = {{&S68298[34], 7}, {NULL}};
const Segment SGI188576[] = {{&S68298[27], 6}, {NULL}};
const Segment SGI188577[] = {{&S68298[94], 9}, {NULL}};
const Segment SGI188578[] = {{&S68298[67], 6}, {NULL}};
const Segment SGI188579[] = {{&S68299[34], 9}, {NULL}};
const Segment SGI188580[] = {{&S68299[27], 6}, {NULL}};
const Segment SGI188581[] = {{&S68299[96], 9}, {NULL}};
const Segment SGI188582[] = {{&S68299[69], 6}, {NULL}};
const Segment SGI188583[] = {{&S68300[34], 9}, {NULL}};
const Segment SGI188584[] = {{&S68300[27], 6}, {NULL}};
const Segment SGI188585[] = {{&S68300[96], 9}, {NULL}};
const Segment SGI188586[] = {{&S68300[69], 6}, {NULL}};
const Segment SGI188587[] = {{&S74955[34], 11}, {NULL}};
const Segment SGI188588[] = {{&S74955[27], 6}, {NULL}};
const Segment SGI188589[] = {{&S74955[98], 9}, {NULL}};
const Segment SGI188590[] = {{&S74955[71], 6}, {NULL}};
const Segment SGI188591[] = {{&S53160[34], 8}, {NULL}};
const Segment SGI188592[] = {{&S53160[27], 6}, {NULL}};
const Segment SGI188593[] = {{&S53160[103], 9}, {NULL}};
const Segment SGI188594[] = {{&S53160[76], 6}, {NULL}};
const Segment SGI188595[] = {{&S74957[34], 11}, {NULL}};
const Segment SGI188596[] = {{&S74957[27], 6}, {NULL}};
const Segment SGI188597[] = {{&S74957[98], 9}, {NULL}};
const Segment SGI188598[] = {{&S74957[71], 6}, {NULL}};
const Segment SGI188599[] = {{&S62374[44], 16}, {NULL}};
const Segment SGI188600[] = {{&S62374[44], 28}, {NULL}};
const Segment SGI188601[] = {{&S62374[30], 6}, {NULL}};
const Segment SGI188602[] = {{&S62374[121], 3}, {NULL}};
const Segment SGI188603[] = {{&S62374[86], 6}, {NULL}};
const Segment SGI188604[] = {{&S74958[34], 11}, {NULL}};
const Segment SGI188605[] = {{&S74958[27], 6}, {NULL}};
const Segment SGI188606[] = {{&S74958[98], 9}, {NULL}};
const Segment SGI188607[] = {{&S74958[71], 6}, {NULL}};
const Segment SGI188608[] = {{&S62375[38], 16}, {NULL}};
const Segment SGI188609[] = {{&S62375[31], 6}, {NULL}};
const Segment SGI188610[] = {{&S53175[44], 18}, {NULL}};
const Segment SGI188611[] = {{&S53175[44], 29}, {NULL}};
const Segment SGI188612[] = {{&S53175[30], 6}, {NULL}};
const Segment SGI188613[] = {{&S53175[140], 3}, {NULL}};
const Segment SGI188614[] = {{&S53175[105], 6}, {NULL}};
const Segment SGI188615[] = {{&S53175[75], 8}, {NULL}};
const Segment SGI188616[] = {{&S53176[44], 18}, {NULL}};
const Segment SGI188617[] = {{&S53176[44], 30}, {NULL}};
const Segment SGI188618[] = {{&S53176[30], 6}, {NULL}};
const Segment SGI188619[] = {{&S53176[143], 3}, {NULL}};
const Segment SGI188620[] = {{&S53176[108], 6}, {NULL}};
const Segment SGI188621[] = {{&S53176[76], 8}, {NULL}};
const Segment SGI188622[] = {{&S68327[34], 10}, {NULL}};
const Segment SGI188623[] = {{&S68327[27], 6}, {NULL}};
const Segment SGI188624[] = {{&S68327[97], 9}, {NULL}};
const Segment SGI188625[] = {{&S68327[70], 6}, {NULL}};
const Segment SGI188626[] = {{&S63573[34], 10}, {NULL}};
const Segment SGI188627[] = {{&S63573[27], 6}, {NULL}};
const Segment SGI188628[] = {{&S63573[97], 9}, {NULL}};
const Segment SGI188629[] = {{&S63573[70], 6}, {NULL}};
const Segment SGI188630[] = {{&S63573[131], 8}, {NULL}};
const Segment SGI188631[] = {{&S63575[34], 10}, {NULL}};
const Segment SGI188632[] = {{&S63575[27], 6}, {NULL}};
const Segment SGI188633[] = {{&S63575[97], 9}, {NULL}};
const Segment SGI188634[] = {{&S63575[70], 6}, {NULL}};
const Segment SGI188635[] = {{&S63575[131], 8}, {NULL}};
const Segment SGI188636[] = {{&S74986[44], 7}, {NULL}};
const Segment SGI188637[] = {{&S74986[44], 15}, {NULL}};
const Segment SGI188638[] = {{&S74986[30], 6}, {NULL}};
const Segment SGI188639[] = {{&S74986[31], 3}, {NULL}};
const Segment SGI188640[] = {{&S74986[73], 6}, {NULL}};
const Segment SGI188641[] = {{&S63577[44], 10}, {NULL}};
const Segment SGI188642[] = {{&S63577[44], 22}, {NULL}};
const Segment SGI188643[] = {{&S63577[30], 6}, {NULL}};
const Segment SGI188644[] = {{&S63577[115], 3}, {NULL}};
const Segment SGI188645[] = {{&S63577[80], 6}, {NULL}};
const Segment SGI188646[] = {{&S63577[140], 7}, {NULL}};
const Segment SGI188647[] = {{&S74988[34], 9}, {NULL}};
const Segment SGI188648[] = {{&S74988[27], 6}, {NULL}};
const Segment SGI188649[] = {{&S74988[96], 9}, {NULL}};
const Segment SGI188650[] = {{&S74988[69], 6}, {NULL}};
const Segment SGI188651[] = {{&S63578[44], 11}, {NULL}};
const Segment SGI188652[] = {{&S63578[44], 23}, {NULL}};
const Segment SGI188653[] = {{&S63578[30], 6}, {NULL}};
const Segment SGI188654[] = {{&S63578[116], 3}, {NULL}};
const Segment SGI188655[] = {{&S63578[81], 6}, {NULL}};
const Segment SGI188656[] = {{&S63578[141], 7}, {NULL}};
const Segment SGI188657[] = {{&S74989[44], 6}, {NULL}};
const Segment SGI188658[] = {{&S74989[44], 18}, {NULL}};
const Segment SGI188659[] = {{&S74989[30], 6}, {NULL}};
const Segment SGI188660[] = {{&S74989[31], 3}, {NULL}};
const Segment SGI188661[] = {{&S74989[76], 6}, {NULL}};
const Segment SGI188662[] = {{&S63579[44], 11}, {NULL}};
const Segment SGI188663[] = {{&S63579[44], 23}, {NULL}};
const Segment SGI188664[] = {{&S63579[30], 6}, {NULL}};
const Segment SGI188665[] = {{&S63579[116], 3}, {NULL}};
const Segment SGI188666[] = {{&S63579[81], 6}, {NULL}};
const Segment SGI188667[] = {{&S63579[141], 7}, {NULL}};
const Segment SGI188668[] = {{&S74990[44], 8}, {NULL}};
const Segment SGI188669[] = {{&S74990[44], 20}, {NULL}};
const Segment SGI188670[] = {{&S74990[30], 6}, {NULL}};
const Segment SGI188671[] = {{&S74990[31], 3}, {NULL}};
const Segment SGI188672[] = {{&S74990[78], 6}, {NULL}};
const Segment SGI188673[] = {{&S74991[45], 7}, {NULL}};
const Segment SGI188674[] = {{&S74991[30], 6}, {NULL}};
const Segment SGI188675[] = {{&S74991[31], 3}, {NULL}};
const Segment SGI188676[] = {{&S74991[82], 5}, {NULL}};
const Segment SGI188677[] = {{&S74991[140], 13}, {NULL}};
const Segment SGI188678[] = {{&S63583[34], 10}, {NULL}};
const Segment SGI188679[] = {{&S63583[27], 6}, {NULL}};
const Segment SGI188680[] = {{&S63583[97], 9}, {NULL}};
const Segment SGI188681[] = {{&S63583[70], 6}, {NULL}};
const Segment SGI188682[] = {{&S63583[138], 8}, {NULL}};
const Segment SGI188683[] = {{&S63584[34], 10}, {NULL}};
const Segment SGI188684[] = {{&S63584[27], 6}, {NULL}};
const Segment SGI188685[] = {{&S63584[97], 9}, {NULL}};
const Segment SGI188686[] = {{&S63584[70], 6}, {NULL}};
const Segment SGI188687[] = {{&S63584[138], 8}, {NULL}};
const Segment SGI188688[] = {{&S63585[34], 11}, {NULL}};
const Segment SGI188689[] = {{&S63585[27], 6}, {NULL}};
const Segment SGI188690[] = {{&S62417[34], 20}, {NULL}};
const Segment SGI188691[] = {{&S63585[98], 9}, {NULL}};
const Segment SGI188692[] = {{&S62417[27], 6}, {NULL}};
const Segment SGI188693[] = {{&S63585[71], 6}, {NULL}};
const Segment SGI188694[] = {{&S63585[139], 8}, {NULL}};
const Segment SGI188695[] = {{&S62417[107], 9}, {NULL}};
const Segment SGI188696[] = {{&S62417[80], 6}, {NULL}};
const Segment SGI188697[] = {{&S63586[34], 11}, {NULL}};
const Segment SGI188698[] = {{&S63586[27], 6}, {NULL}};
const Segment SGI188699[] = {{&S62418[34], 21}, {NULL}};
const Segment SGI188700[] = {{&S62418[27], 6}, {NULL}};
const Segment SGI188701[] = {{&S63586[98], 9}, {NULL}};
const Segment SGI188702[] = {{&S63586[71], 6}, {NULL}};
const Segment SGI188703[] = {{&S62418[108], 9}, {NULL}};
const Segment SGI188704[] = {{&S62418[81], 6}, {NULL}};
const Segment SGI188705[] = {{&S63586[139], 8}, {NULL}};
const Segment SGI188706[] = {{&S63587[34], 18}, {NULL}};
const Segment SGI188707[] = {{&S63587[27], 6}, {NULL}};
const Segment SGI188708[] = {{&S63587[105], 9}, {NULL}};
const Segment SGI188709[] = {{&S63587[78], 6}, {NULL}};
const Segment SGI188710[] = {{&S63587[146], 8}, {NULL}};
const Segment SGI188711[] = {{&S63588[34], 11}, {NULL}};
const Segment SGI188712[] = {{&S63588[27], 6}, {NULL}};
const Segment SGI188713[] = {{&S63588[98], 9}, {NULL}};
const Segment SGI188714[] = {{&S63588[71], 6}, {NULL}};
const Segment SGI188715[] = {{&S63588[139], 8}, {NULL}};
const Segment SGI188716[] = {{&S75006[34], 14}, {NULL}};
const Segment SGI188717[] = {{&S75006[27], 6}, {NULL}};
const Segment SGI188718[] = {{&S75006[114], 9}, {NULL}};
const Segment SGI188719[] = {{&S75006[87], 6}, {NULL}};
const Segment SGI188720[] = {{&S53225[34], 29}, {NULL}};
const Segment SGI188721[] = {{&S53225[27], 6}, {NULL}};
const Segment SGI188722[] = {{&S53225[116], 9}, {NULL}};
const Segment SGI188723[] = {{&S53225[89], 6}, {NULL}};
const Segment SGI188724[] = {{&S75012[34], 13}, {NULL}};
const Segment SGI188725[] = {{&S75012[27], 6}, {NULL}};
const Segment SGI188726[] = {{&S75012[100], 9}, {NULL}};
const Segment SGI188727[] = {{&S75012[73], 6}, {NULL}};
const Segment SGI188728[] = {{&S75013[34], 12}, {NULL}};
const Segment SGI188729[] = {{&S75013[27], 6}, {NULL}};
const Segment SGI188730[] = {{&S75013[99], 9}, {NULL}};
const Segment SGI188731[] = {{&S75013[72], 6}, {NULL}};
const Segment SGI188732[] = {{&S62435[34], 11}, {NULL}};
const Segment SGI188733[] = {{&S62435[27], 6}, {NULL}};
const Segment SGI188734[] = {{&S62435[98], 9}, {NULL}};
const Segment SGI188735[] = {{&S62435[71], 6}, {NULL}};
const Segment SGI188736[] = {{&S68374[44], 12}, {NULL}};
const Segment SGI188737[] = {{&S68374[44], 24}, {NULL}};
const Segment SGI188738[] = {{&S68374[30], 6}, {NULL}};
const Segment SGI188739[] = {{&S68374[117], 3}, {NULL}};
const Segment SGI188740[] = {{&S68374[82], 6}, {NULL}};
const Segment SGI188741[] = {{&S75022[44], 15}, {NULL}};
const Segment SGI188742[] = {{&S75022[44], 27}, {NULL}};
const Segment SGI188743[] = {{&S75022[30], 6}, {NULL}};
const Segment SGI188744[] = {{&S75022[31], 3}, {NULL}};
const Segment SGI188745[] = {{&S75022[85], 6}, {NULL}};
const Segment SGI188746[] = {{&S68375[38], 4}, {NULL}};
const Segment SGI188747[] = {{&S68375[31], 6}, {NULL}};
const Segment SGI188748[] = {{&S63619[34], 9}, {NULL}};
const Segment SGI188749[] = {{&S63619[27], 6}, {NULL}};
const Segment SGI188750[] = {{&S63619[96], 9}, {NULL}};
const Segment SGI188751[] = {{&S63619[69], 6}, {NULL}};
const Segment SGI188752[] = {{&S68383[34], 25}, {NULL}};
const Segment SGI188753[] = {{&S68383[27], 6}, {NULL}};
const Segment SGI188754[] = {{&S68383[112], 9}, {NULL}};
const Segment SGI188755[] = {{&S68383[85], 6}, {NULL}};
const Segment SGI188756[] = {{&S63620[34], 10}, {NULL}};
const Segment SGI188757[] = {{&S63620[27], 6}, {NULL}};
const Segment SGI188758[] = {{&S63620[97], 9}, {NULL}};
const Segment SGI188759[] = {{&S63620[70], 6}, {NULL}};
const Segment SGI188760[] = {{&S63621[34], 10}, {NULL}};
const Segment SGI188761[] = {{&S63621[27], 6}, {NULL}};
const Segment SGI188762[] = {{&S63621[97], 9}, {NULL}};
const Segment SGI188763[] = {{&S63621[70], 6}, {NULL}};
const Segment SGI188764[] = {{&S63622[34], 10}, {NULL}};
const Segment SGI188765[] = {{&S63622[27], 6}, {NULL}};
const Segment SGI188766[] = {{&S63622[97], 9}, {NULL}};
const Segment SGI188767[] = {{&S63622[70], 6}, {NULL}};
const Segment SGI188768[] = {{&S62459[34], 10}, {NULL}};
const Segment SGI188769[] = {{&S62459[27], 6}, {NULL}};
const Segment SGI188770[] = {{&S62459[97], 9}, {NULL}};
const Segment SGI188771[] = {{&S62459[70], 6}, {NULL}};
const Segment SGI188772[] = {{&S68395[44], 12}, {NULL}};
const Segment SGI188773[] = {{&S68395[44], 24}, {NULL}};
const Segment SGI188774[] = {{&S68395[30], 6}, {NULL}};
const Segment SGI188775[] = {{&S68395[117], 3}, {NULL}};
const Segment SGI188776[] = {{&S68395[82], 6}, {NULL}};
const Segment SGI188777[] = {{&S68396[38], 12}, {NULL}};
const Segment SGI188778[] = {{&S68396[31], 6}, {NULL}};
const Segment SGI188779[] = {{&S75054[44], 10}, {NULL}};
const Segment SGI188780[] = {{&S75054[44], 22}, {NULL}};
const Segment SGI188781[] = {{&S75054[30], 6}, {NULL}};
const Segment SGI188782[] = {{&S75054[31], 3}, {NULL}};
const Segment SGI188783[] = {{&S75054[80], 6}, {NULL}};
const Segment SGI188784[] = {{&S75068[34], 15}, {NULL}};
const Segment SGI188785[] = {{&S75068[27], 6}, {NULL}};
const Segment SGI188786[] = {{&S75068[102], 9}, {NULL}};
const Segment SGI188787[] = {{&S75068[75], 6}, {NULL}};
const Segment SGI188788[] = {{&S63653[44], 19}, {NULL}};
const Segment SGI188789[] = {{&S63653[44], 31}, {NULL}};
const Segment SGI188790[] = {{&S63653[30], 6}, {NULL}};
const Segment SGI188791[] = {{&S63653[124], 3}, {NULL}};
const Segment SGI188792[] = {{&S63653[89], 6}, {NULL}};
const Segment SGI188793[] = {{&S63654[38], 13}, {NULL}};
const Segment SGI188794[] = {{&S63654[31], 6}, {NULL}};
const Segment SGI188795[] = {{&S63655[34], 6}, {NULL}};
const Segment SGI188796[] = {{&S63655[27], 6}, {NULL}};
const Segment SGI188797[] = {{&S63655[93], 9}, {NULL}};
const Segment SGI188798[] = {{&S63655[66], 6}, {NULL}};
const Segment SGI188799[] = {{&S63658[34], 10}, {NULL}};
const Segment SGI188800[] = {{&S63658[27], 6}, {NULL}};
const Segment SGI188801[] = {{&S63658[97], 9}, {NULL}};
const Segment SGI188802[] = {{&S63658[70], 6}, {NULL}};
const Segment SGI188803[] = {{&S68426[34], 11}, {NULL}};
const Segment SGI188804[] = {{&S68426[27], 6}, {NULL}};
const Segment SGI188805[] = {{&S68426[98], 9}, {NULL}};
const Segment SGI188806[] = {{&S68426[71], 6}, {NULL}};
const Segment SGI188807[] = {{&S68442[34], 9}, {NULL}};
const Segment SGI188808[] = {{&S68442[27], 6}, {NULL}};
const Segment SGI188809[] = {{&S68442[96], 9}, {NULL}};
const Segment SGI188810[] = {{&S68442[69], 6}, {NULL}};
const Segment SGI188811[] = {{&S68443[34], 9}, {NULL}};
const Segment SGI188812[] = {{&S68443[27], 6}, {NULL}};
const Segment SGI188813[] = {{&S68443[96], 9}, {NULL}};
const Segment SGI188814[] = {{&S68443[69], 6}, {NULL}};
const Segment SGI188815[] = {{&S63687[44], 10}, {NULL}};
const Segment SGI188816[] = {{&S63687[44], 22}, {NULL}};
const Segment SGI188817[] = {{&S63687[30], 6}, {NULL}};
const Segment SGI188818[] = {{&S63687[115], 3}, {NULL}};
const Segment SGI188819[] = {{&S63687[80], 6}, {NULL}};
const Segment SGI188820[] = {{&S63688[44], 10}, {NULL}};
const Segment SGI188821[] = {{&S63688[44], 22}, {NULL}};
const Segment SGI188822[] = {{&S63688[30], 6}, {NULL}};
const Segment SGI188823[] = {{&S63688[115], 3}, {NULL}};
const Segment SGI188824[] = {{&S63688[80], 6}, {NULL}};
const Segment SGI188825[] = {{&S68462[44], 9}, {NULL}};
const Segment SGI188826[] = {{&S68462[44], 21}, {NULL}};
const Segment SGI188827[] = {{&S68462[30], 6}, {NULL}};
const Segment SGI188828[] = {{&S68462[114], 3}, {NULL}};
const Segment SGI188829[] = {{&S68462[79], 6}, {NULL}};
const Segment SGI188830[] = {{&S63701[34], 11}, {NULL}};
const Segment SGI188831[] = {{&S63701[27], 6}, {NULL}};
const Segment SGI188832[] = {{&S63701[98], 9}, {NULL}};
const Segment SGI188833[] = {{&S63701[71], 6}, {NULL}};
const Segment SGI188834[] = {{&S63702[34], 18}, {NULL}};
const Segment SGI188835[] = {{&S63702[27], 6}, {NULL}};
const Segment SGI188836[] = {{&S63702[105], 9}, {NULL}};
const Segment SGI188837[] = {{&S63702[78], 6}, {NULL}};
const Segment SGI188838[] = {{&S53344[44], 10}, {NULL}};
const Segment SGI188839[] = {{&S53344[44], 21}, {NULL}};
const Segment SGI188840[] = {{&S53344[30], 6}, {NULL}};
const Segment SGI188841[] = {{&S53344[114], 3}, {NULL}};
const Segment SGI188842[] = {{&S53344[79], 6}, {NULL}};
const Segment SGI188843[] = {{&S53345[44], 18}, {NULL}};
const Segment SGI188844[] = {{&S53345[44], 29}, {NULL}};
const Segment SGI188845[] = {{&S53345[30], 6}, {NULL}};
const Segment SGI188846[] = {{&S53345[122], 3}, {NULL}};
const Segment SGI188847[] = {{&S53345[87], 6}, {NULL}};
const Segment SGI188848[] = {{&S53346[38], 10}, {NULL}};
const Segment SGI188849[] = {{&S53346[31], 6}, {NULL}};
const Segment SGI188850[] = {{&S53347[44], 18}, {NULL}};
const Segment SGI188851[] = {{&S53347[44], 30}, {NULL}};
const Segment SGI188852[] = {{&S53347[30], 6}, {NULL}};
const Segment SGI188853[] = {{&S63709[34], 10}, {NULL}};
const Segment SGI188854[] = {{&S53347[123], 3}, {NULL}};
const Segment SGI188855[] = {{&S63709[27], 6}, {NULL}};
const Segment SGI188856[] = {{&S53347[88], 6}, {NULL}};
const Segment SGI188857[] = {{&S63709[97], 9}, {NULL}};
const Segment SGI188858[] = {{&S63709[70], 6}, {NULL}};
const Segment SGI188859[] = {{&S63709[138], 8}, {NULL}};
const Segment SGI188860[] = {{&S53348[38], 18}, {NULL}};
const Segment SGI188861[] = {{&S53348[31], 6}, {NULL}};
const Segment SGI188862[] = {{&S63712[34], 15}, {NULL}};
const Segment SGI188863[] = {{&S63712[27], 6}, {NULL}};
const Segment SGI188864[] = {{&S63712[102], 9}, {NULL}};
const Segment SGI188865[] = {{&S63712[75], 6}, {NULL}};
const Segment SGI188866[] = {{&S63713[34], 11}, {NULL}};
const Segment SGI188867[] = {{&S63713[27], 6}, {NULL}};
const Segment SGI188868[] = {{&S63713[98], 9}, {NULL}};
const Segment SGI188869[] = {{&S63713[71], 6}, {NULL}};
const Segment SGI188870[] = {{&S75143[34], 13}, {NULL}};
const Segment SGI188871[] = {{&S75143[27], 6}, {NULL}};
const Segment SGI188872[] = {{&S75143[100], 9}, {NULL}};
const Segment SGI188873[] = {{&S75143[73], 6}, {NULL}};
const Segment SGI188874[] = {{&S75144[34], 5}, {NULL}};
const Segment SGI188875[] = {{&S75144[27], 6}, {NULL}};
const Segment SGI188876[] = {{&S63726[44], 13}, {NULL}};
const Segment SGI188877[] = {{&S63726[44], 33}, {NULL}};
const Segment SGI188878[] = {{&S63726[30], 6}, {NULL}};
const Segment SGI188879[] = {{&S75144[92], 9}, {NULL}};
const Segment SGI188880[] = {{&S75144[65], 6}, {NULL}};
const Segment SGI188881[] = {{&S63726[145], 3}, {NULL}};
const Segment SGI188882[] = {{&S63726[110], 6}, {NULL}};
const Segment SGI188883[] = {{&S63727[44], 13}, {NULL}};
const Segment SGI188884[] = {{&S63727[44], 25}, {NULL}};
const Segment SGI188885[] = {{&S63727[30], 6}, {NULL}};
const Segment SGI188886[] = {{&S63727[86], 6}, {NULL}};
const Segment SGI188887[] = {{&S63728[38], 13}, {NULL}};
const Segment SGI188888[] = {{&S63728[31], 6}, {NULL}};
const Segment SGI188889[] = {{&S63729[44], 15}, {NULL}};
const Segment SGI188890[] = {{&S63729[44], 36}, {NULL}};
const Segment SGI188891[] = {{&S63729[30], 6}, {NULL}};
const Segment SGI188892[] = {{&S63729[148], 3}, {NULL}};
const Segment SGI188893[] = {{&S63729[113], 6}, {NULL}};
const Segment SGI188894[] = {{&S68510[44], 17}, {NULL}};
const Segment SGI188895[] = {{&S68510[44], 29}, {NULL}};
const Segment SGI188896[] = {{&S63730[44], 15}, {NULL}};
const Segment SGI188897[] = {{&S68510[30], 6}, {NULL}};
const Segment SGI188898[] = {{&S63730[44], 27}, {NULL}};
const Segment SGI188899[] = {{&S63730[30], 6}, {NULL}};
const Segment SGI188900[] = {{&S68510[122], 3}, {NULL}};
const Segment SGI188901[] = {{&S68510[87], 6}, {NULL}};
const Segment SGI188902[] = {{&S63730[88], 6}, {NULL}};
const Segment SGI188903[] = {{&S63731[38], 15}, {NULL}};
const Segment SGI188904[] = {{&S63731[31], 6}, {NULL}};
const Segment SGI188905[] = {{&S63732[44], 15}, {NULL}};
const Segment SGI188906[] = {{&S63732[44], 36}, {NULL}};
const Segment SGI188907[] = {{&S63732[30], 6}, {NULL}};
const Segment SGI188908[] = {{&S63732[148], 3}, {NULL}};
const Segment SGI188909[] = {{&S63732[113], 6}, {NULL}};
const Segment SGI188910[] = {{&S53385[44], 9}, {NULL}};
const Segment SGI188911[] = {{&S53385[44], 20}, {NULL}};
const Segment SGI188912[] = {{&S53385[30], 6}, {NULL}};
const Segment SGI188913[] = {{&S53385[113], 3}, {NULL}};
const Segment SGI188914[] = {{&S53385[78], 6}, {NULL}};
const Segment SGI188915[] = {{&S53386[38], 9}, {NULL}};
const Segment SGI188916[] = {{&S53386[31], 6}, {NULL}};
const Segment SGI188917[] = {{&S75180[34], 17}, {NULL}};
const Segment SGI188918[] = {{&S75180[27], 6}, {NULL}};
const Segment SGI188919[] = {{&S75180[111], 9}, {NULL}};
const Segment SGI188920[] = {{&S75180[84], 6}, {NULL}};
const Segment SGI188921[] = {{&S75184[44], 10}, {NULL}};
const Segment SGI188922[] = {{&S75184[44], 22}, {NULL}};
const Segment SGI188923[] = {{&S75184[30], 6}, {NULL}};
const Segment SGI188924[] = {{&S75184[83], 6}, {NULL}};
const Segment SGI188925[] = {{&S75185[44], 22}, {NULL}};
const Segment SGI188926[] = {{&S75185[30], 6}, {NULL}};
const Segment SGI188927[] = {{&S75186[38], 10}, {NULL}};
const Segment SGI188928[] = {{&S75186[31], 6}, {NULL}};
const Segment SGI188929[] = {{&S75187[44], 11}, {NULL}};
const Segment SGI188930[] = {{&S75187[44], 23}, {NULL}};
const Segment SGI188931[] = {{&S75187[30], 6}, {NULL}};
const Segment SGI188932[] = {{&S75187[31], 3}, {NULL}};
const Segment SGI188933[] = {{&S75187[81], 6}, {NULL}};
const Segment SGI188934[] = {{&S75190[44], 11}, {NULL}};
const Segment SGI188935[] = {{&S75190[44], 29}, {NULL}};
const Segment SGI188936[] = {{&S75190[30], 6}, {NULL}};
const Segment SGI188937[] = {{&S75190[31], 3}, {NULL}};
const Segment SGI188938[] = {{&S75190[87], 6}, {NULL}};
const Segment SGI188939[] = {{&S75192[44], 11}, {NULL}};
const Segment SGI188940[] = {{&S75192[44], 20}, {NULL}};
const Segment SGI188941[] = {{&S75192[30], 6}, {NULL}};
const Segment SGI188942[] = {{&S75192[31], 3}, {NULL}};
const Segment SGI188943[] = {{&S75192[78], 6}, {NULL}};
const Segment SGI188944[] = {{&S63784[34], 9}, {NULL}};
const Segment SGI188945[] = {{&S63784[27], 6}, {NULL}};
const Segment SGI188946[] = {{&S63784[96], 9}, {NULL}};
const Segment SGI188947[] = {{&S63784[69], 6}, {NULL}};
const Segment SGI188948[] = {{&S75197[34], 7}, {NULL}};
const Segment SGI188949[] = {{&S75197[27], 6}, {NULL}};
const Segment SGI188950[] = {{&S63786[34], 11}, {NULL}};
const Segment SGI188951[] = {{&S75197[105], 9}, {NULL}};
const Segment SGI188952[] = {{&S63786[27], 6}, {NULL}};
const Segment SGI188953[] = {{&S75197[78], 6}, {NULL}};
const Segment SGI188954[] = {{&S63786[98], 9}, {NULL}};
const Segment SGI188955[] = {{&S63786[71], 6}, {NULL}};
const Segment SGI188956[] = {{&S63787[34], 11}, {NULL}};
const Segment SGI188957[] = {{&S63787[27], 6}, {NULL}};
const Segment SGI188958[] = {{&S63787[98], 9}, {NULL}};
const Segment SGI188959[] = {{&S63787[71], 6}, {NULL}};
const Segment SGI188960[] = {{&S63788[34], 11}, {NULL}};
const Segment SGI188961[] = {{&S63788[27], 6}, {NULL}};
const Segment SGI188962[] = {{&S63788[98], 9}, {NULL}};
const Segment SGI188963[] = {{&S63788[71], 6}, {NULL}};
const Segment SGI188964[] = {{&S75204[34], 9}, {NULL}};
const Segment SGI188965[] = {{&S75204[27], 6}, {NULL}};
const Segment SGI188966[] = {{&S75204[96], 9}, {NULL}};
const Segment SGI188967[] = {{&S75204[69], 6}, {NULL}};
const Segment SGI188968[] = {{&S75205[34], 10}, {NULL}};
const Segment SGI188969[] = {{&S75205[27], 6}, {NULL}};
const Segment SGI188970[] = {{&S75205[97], 9}, {NULL}};
const Segment SGI188971[] = {{&S75205[70], 6}, {NULL}};
const Segment SGI188972[] = {{&S63795[34], 11}, {NULL}};
const Segment SGI188973[] = {{&S63795[27], 6}, {NULL}};
const Segment SGI188974[] = {{&S63795[98], 9}, {NULL}};
const Segment SGI188975[] = {{&S63795[71], 6}, {NULL}};
const Segment SGI188976[] = {{&S63796[34], 18}, {NULL}};
const Segment SGI188977[] = {{&S63796[27], 6}, {NULL}};
const Segment SGI188978[] = {{&S63796[105], 9}, {NULL}};
const Segment SGI188979[] = {{&S63796[78], 6}, {NULL}};
const Segment SGI188980[] = {{&S75223[34], 10}, {NULL}};
const Segment SGI188981[] = {{&S75223[27], 6}, {NULL}};
const Segment SGI188982[] = {{&S75223[97], 9}, {NULL}};
const Segment SGI188983[] = {{&S75223[70], 6}, {NULL}};
const Segment SGI188984[] = {{&S68609[44], 10}, {NULL}};
const Segment SGI188985[] = {{&S68609[44], 22}, {NULL}};
const Segment SGI188986[] = {{&S68609[30], 6}, {NULL}};
const Segment SGI188987[] = {{&S75234[34], 11}, {NULL}};
const Segment SGI188988[] = {{&S75234[27], 6}, {NULL}};
const Segment SGI188989[] = {{&S68609[80], 6}, {NULL}};
const Segment SGI188990[] = {{&S75234[98], 9}, {NULL}};
const Segment SGI188991[] = {{&S75234[71], 6}, {NULL}};
const Segment SGI188992[] = {{&S68609[107], 13}, {NULL}};
const Segment SGI188993[] = {{&S68613[34], 15}, {NULL}};
const Segment SGI188994[] = {{&S75237[34], 10}, {NULL}};
const Segment SGI188995[] = {{&S68613[27], 6}, {NULL}};
const Segment SGI188996[] = {{&S75237[27], 6}, {NULL}};
const Segment SGI188997[] = {{&S75237[97], 9}, {NULL}};
const Segment SGI188998[] = {{&S68613[102], 9}, {NULL}};
const Segment SGI188999[] = {{&S75237[70], 6}, {NULL}};
const Segment SGI189000[] = {{&S68613[75], 6}, {NULL}};
const Segment SGI189001[] = {{&S66413[32], 20}, {NULL}};
const Segment SGI189002[] = {{&S66413[27], 4}, {NULL}};
const Segment SGI189003[] = {{&S66413[105], 9}, {NULL}};
const Segment SGI189004[] = {{&S66413[78], 6}, {NULL}};
const Segment SGI189005[] = {{&S63840[34], 7}, {NULL}};
const Segment SGI189006[] = {{&S63840[27], 6}, {NULL}};
const Segment SGI189007[] = {{&S63840[105], 9}, {NULL}};
const Segment SGI189008[] = {{&S63840[78], 6}, {NULL}};
const Segment SGI189009[] = {{&S53493[41], 27}, {NULL}};
const Segment SGI189010[] = {{&S53493[41], 38}, {NULL}};
const Segment SGI189011[] = {{&S53493[27], 6}, {NULL}};
const Segment SGI189012[] = {{&S53493[128], 3}, {NULL}};
const Segment SGI189013[] = {{&S53493[132], 9}, {NULL}};
const Segment SGI189014[] = {{&S53493[93], 6}, {NULL}};
const Segment SGI189015[] = {{&S75249[44], 11}, {NULL}};
const Segment SGI189016[] = {{&S75249[44], 23}, {NULL}};
const Segment SGI189017[] = {{&S75249[30], 6}, {NULL}};
const Segment SGI189018[] = {{&S75249[31], 3}, {NULL}};
const Segment SGI189019[] = {{&S53494[41], 27}, {NULL}};
const Segment SGI189020[] = {{&S75249[81], 6}, {NULL}};
const Segment SGI189021[] = {{&S53494[41], 38}, {NULL}};
const Segment SGI189022[] = {{&S53494[27], 6}, {NULL}};
const Segment SGI189023[] = {{&S53494[128], 3}, {NULL}};
const Segment SGI189024[] = {{&S53494[132], 9}, {NULL}};
const Segment SGI189025[] = {{&S53494[93], 6}, {NULL}};
const Segment SGI189026[] = {{&S53495[41], 27}, {NULL}};
const Segment SGI189027[] = {{&S75251[44], 11}, {NULL}};
const Segment SGI189028[] = {{&S53495[41], 38}, {NULL}};
const Segment SGI189029[] = {{&S75251[44], 23}, {NULL}};
const Segment SGI189030[] = {{&S53495[27], 6}, {NULL}};
const Segment SGI189031[] = {{&S75251[30], 6}, {NULL}};
const Segment SGI189032[] = {{&S53495[128], 3}, {NULL}};
const Segment SGI189033[] = {{&S75251[84], 6}, {NULL}};
const Segment SGI189034[] = {{&S53495[132], 9}, {NULL}};
const Segment SGI189035[] = {{&S53495[93], 6}, {NULL}};
const Segment SGI189036[] = {{&S75252[44], 7}, {NULL}};
const Segment SGI189037[] = {{&S75252[44], 19}, {NULL}};
const Segment SGI189038[] = {{&S75252[30], 6}, {NULL}};
const Segment SGI189039[] = {{&S53496[41], 27}, {NULL}};
const Segment SGI189040[] = {{&S53496[41], 38}, {NULL}};
const Segment SGI189041[] = {{&S53496[27], 6}, {NULL}};
const Segment SGI189042[] = {{&S75252[31], 3}, {NULL}};
const Segment SGI189043[] = {{&S75252[77], 6}, {NULL}};
const Segment SGI189044[] = {{&S53496[128], 3}, {NULL}};
const Segment SGI189045[] = {{&S53496[132], 9}, {NULL}};
const Segment SGI189046[] = {{&S53496[93], 6}, {NULL}};
const Segment SGI189047[] = {{&S53497[41], 27}, {NULL}};
const Segment SGI189048[] = {{&S53497[41], 38}, {NULL}};
const Segment SGI189049[] = {{&S53497[27], 6}, {NULL}};
const Segment SGI189050[] = {{&S53497[128], 3}, {NULL}};
const Segment SGI189051[] = {{&S53497[132], 9}, {NULL}};
const Segment SGI189052[] = {{&S53497[93], 6}, {NULL}};
const Segment SGI189053[] = {{&S53498[41], 27}, {NULL}};
const Segment SGI189054[] = {{&S53498[41], 38}, {NULL}};
const Segment SGI189055[] = {{&S53498[27], 6}, {NULL}};
const Segment SGI189056[] = {{&S53498[128], 3}, {NULL}};
const Segment SGI189057[] = {{&S53498[132], 9}, {NULL}};
const Segment SGI189058[] = {{&S53498[93], 6}, {NULL}};
const Segment SGI189059[] = {{&S75258[34], 11}, {NULL}};
const Segment SGI189060[] = {{&S75258[27], 6}, {NULL}};
const Segment SGI189061[] = {{&S75258[110], 9}, {NULL}};
const Segment SGI189062[] = {{&S75258[83], 6}, {NULL}};
const Segment SGI189063[] = {{&S75259[34], 11}, {NULL}};
const Segment SGI189064[] = {{&S75259[27], 6}, {NULL}};
const Segment SGI189065[] = {{&S75259[98], 9}, {NULL}};
const Segment SGI189066[] = {{&S75259[71], 6}, {NULL}};
const Segment SGI189067[] = {{&S75261[44], 12}, {NULL}};
const Segment SGI189068[] = {{&S75261[44], 24}, {NULL}};
const Segment SGI189069[] = {{&S75261[30], 6}, {NULL}};
const Segment SGI189070[] = {{&S75261[31], 3}, {NULL}};
const Segment SGI189071[] = {{&S75261[82], 6}, {NULL}};
const Segment SGI189072[] = {{&S63879[34], 6}, {NULL}};
const Segment SGI189073[] = {{&S63879[27], 6}, {NULL}};
const Segment SGI189074[] = {{&S63879[93], 9}, {NULL}};
const Segment SGI189075[] = {{&S63879[66], 6}, {NULL}};
const Segment SGI189076[] = {{&S68647[44], 7}, {NULL}};
const Segment SGI189077[] = {{&S68647[44], 27}, {NULL}};
const Segment SGI189078[] = {{&S68647[30], 6}, {NULL}};
const Segment SGI189079[] = {{&S68647[136], 3}, {NULL}};
const Segment SGI189080[] = {{&S68647[101], 6}, {NULL}};
const Segment SGI189081[] = {{&S68647[73], 8}, {NULL}};
const Segment SGI189082[] = {{&S63889[44], 11}, {NULL}};
const Segment SGI189083[] = {{&S63889[44], 23}, {NULL}};
const Segment SGI189084[] = {{&S63889[30], 6}, {NULL}};
const Segment SGI189085[] = {{&S63889[81], 6}, {NULL}};
const Segment SGI189086[] = {{&S63889[108], 13}, {NULL}};
const Segment SGI189087[] = {{&S63890[44], 10}, {NULL}};
const Segment SGI189088[] = {{&S63890[44], 22}, {NULL}};
const Segment SGI189089[] = {{&S63890[30], 6}, {NULL}};
const Segment SGI189090[] = {{&S63890[80], 6}, {NULL}};
const Segment SGI189091[] = {{&S53533[44], 7}, {NULL}};
const Segment SGI189092[] = {{&S53533[44], 18}, {NULL}};
const Segment SGI189093[] = {{&S63890[107], 13}, {NULL}};
const Segment SGI189094[] = {{&S53533[30], 6}, {NULL}};
const Segment SGI189095[] = {{&S53533[129], 3}, {NULL}};
const Segment SGI189096[] = {{&S53533[94], 6}, {NULL}};
const Segment SGI189097[] = {{&S53533[64], 8}, {NULL}};
const Segment SGI189098[] = {{&S63891[44], 11}, {NULL}};
const Segment SGI189099[] = {{&S63891[44], 23}, {NULL}};
const Segment SGI189100[] = {{&S63891[30], 6}, {NULL}};
const Segment SGI189101[] = {{&S75290[44], 5}, {NULL}};
const Segment SGI189102[] = {{&S75290[44], 17}, {NULL}};
const Segment SGI189103[] = {{&S75290[30], 6}, {NULL}};
const Segment SGI189104[] = {{&S63891[81], 6}, {NULL}};
const Segment SGI189105[] = {{&S75290[31], 3}, {NULL}};
const Segment SGI189106[] = {{&S63891[108], 13}, {NULL}};
const Segment SGI189107[] = {{&S75290[75], 6}, {NULL}};
const Segment SGI189108[] = {{&S53535[44], 11}, {NULL}};
const Segment SGI189109[] = {{&S53535[44], 22}, {NULL}};
const Segment SGI189110[] = {{&S53535[30], 6}, {NULL}};
const Segment SGI189111[] = {{&S53535[133], 3}, {NULL}};
const Segment SGI189112[] = {{&S53535[98], 6}, {NULL}};
const Segment SGI189113[] = {{&S53535[68], 8}, {NULL}};
const Segment SGI189114[] = {{&S68662[44], 11}, {NULL}};
const Segment SGI189115[] = {{&S68662[44], 28}, {NULL}};
const Segment SGI189116[] = {{&S68662[30], 6}, {NULL}};
const Segment SGI189117[] = {{&S68662[127], 3}, {NULL}};
const Segment SGI189118[] = {{&S53536[44], 7}, {NULL}};
const Segment SGI189119[] = {{&S68662[93], 5}, {NULL}};
const Segment SGI189120[] = {{&S53536[44], 19}, {NULL}};
const Segment SGI189121[] = {{&S53536[30], 6}, {NULL}};
const Segment SGI189122[] = {{&S68662[72], 8}, {NULL}};
const Segment SGI189123[] = {{&S53536[130], 3}, {NULL}};
const Segment SGI189124[] = {{&S53536[95], 6}, {NULL}};
const Segment SGI189125[] = {{&S53536[65], 8}, {NULL}};
const Segment SGI189126[] = {{&S53537[44], 11}, {NULL}};
const Segment SGI189127[] = {{&S53537[44], 23}, {NULL}};
const Segment SGI189128[] = {{&S53537[30], 6}, {NULL}};
const Segment SGI189129[] = {{&S53537[116], 3}, {NULL}};
const Segment SGI189130[] = {{&S53537[81], 6}, {NULL}};
const Segment SGI189131[] = {{&S53537[141], 8}, {NULL}};
const Segment SGI189132[] = {{&S68665[44], 10}, {NULL}};
const Segment SGI189133[] = {{&S68665[44], 22}, {NULL}};
const Segment SGI189134[] = {{&S68665[30], 6}, {NULL}};
const Segment SGI189135[] = {{&S68665[115], 3}, {NULL}};
const Segment SGI189136[] = {{&S68665[80], 6}, {NULL}};
const Segment SGI189137[] = {{&S53538[44], 11}, {NULL}};
const Segment SGI189138[] = {{&S53538[44], 21}, {NULL}};
const Segment SGI189139[] = {{&S53538[30], 6}, {NULL}};
const Segment SGI189140[] = {{&S53538[132], 3}, {NULL}};
const Segment SGI189141[] = {{&S53538[97], 6}, {NULL}};
const Segment SGI189142[] = {{&S68666[38], 10}, {NULL}};
const Segment SGI189143[] = {{&S68666[31], 6}, {NULL}};
const Segment SGI189144[] = {{&S53538[67], 8}, {NULL}};
const Segment SGI189145[] = {{&S75305[34], 5}, {NULL}};
const Segment SGI189146[] = {{&S75305[27], 6}, {NULL}};
const Segment SGI189147[] = {{&S75305[92], 9}, {NULL}};
const Segment SGI189148[] = {{&S75305[65], 6}, {NULL}};
const Segment SGI189149[] = {{&S63910[34], 9}, {NULL}};
const Segment SGI189150[] = {{&S63910[27], 6}, {NULL}};
const Segment SGI189151[] = {{&S75306[34], 7}, {NULL}};
const Segment SGI189152[] = {{&S75306[27], 6}, {NULL}};
const Segment SGI189153[] = {{&S63910[96], 9}, {NULL}};
const Segment SGI189154[] = {{&S63910[69], 6}, {NULL}};
const Segment SGI189155[] = {{&S75306[99], 9}, {NULL}};
const Segment SGI189156[] = {{&S75306[72], 6}, {NULL}};
const Segment SGI189157[] = {{&S63911[34], 9}, {NULL}};
const Segment SGI189158[] = {{&S63911[27], 6}, {NULL}};
const Segment SGI189159[] = {{&S63911[96], 9}, {NULL}};
const Segment SGI189160[] = {{&S63911[69], 6}, {NULL}};
const Segment SGI189161[] = {{&S68680[44], 11}, {NULL}};
const Segment SGI189162[] = {{&S68680[44], 23}, {NULL}};
const Segment SGI189163[] = {{&S68680[30], 6}, {NULL}};
const Segment SGI189164[] = {{&S75310[34], 17}, {NULL}};
const Segment SGI189165[] = {{&S75310[27], 6}, {NULL}};
const Segment SGI189166[] = {{&S68680[116], 3}, {NULL}};
const Segment SGI189167[] = {{&S68680[81], 6}, {NULL}};
const Segment SGI189168[] = {{&S75310[104], 9}, {NULL}};
const Segment SGI189169[] = {{&S75310[77], 6}, {NULL}};
const Segment SGI189170[] = {{&S75312[34], 10}, {NULL}};
const Segment SGI189171[] = {{&S53557[34], 29}, {NULL}};
const Segment SGI189172[] = {{&S75312[27], 6}, {NULL}};
const Segment SGI189173[] = {{&S53557[27], 6}, {NULL}};
const Segment SGI189174[] = {{&S75312[97], 9}, {NULL}};
const Segment SGI189175[] = {{&S75312[70], 6}, {NULL}};
const Segment SGI189176[] = {{&S53557[115], 9}, {NULL}};
const Segment SGI189177[] = {{&S53557[88], 6}, {NULL}};
const Segment SGI189178[] = {{&S75313[34], 7}, {NULL}};
const Segment SGI189179[] = {{&S75313[27], 6}, {NULL}};
const Segment SGI189180[] = {{&S75313[105], 9}, {NULL}};
const Segment SGI189181[] = {{&S75313[78], 6}, {NULL}};
const Segment SGI189182[] = {{&S53561[34], 10}, {NULL}};
const Segment SGI189183[] = {{&S53561[27], 6}, {NULL}};
const Segment SGI189184[] = {{&S53561[96], 9}, {NULL}};
const Segment SGI189185[] = {{&S53561[69], 6}, {NULL}};
const Segment SGI189186[] = {{&S53562[34], 11}, {NULL}};
const Segment SGI189187[] = {{&S53562[27], 6}, {NULL}};
const Segment SGI189188[] = {{&S53562[97], 9}, {NULL}};
const Segment SGI189189[] = {{&S53562[70], 6}, {NULL}};
const Segment SGI189190[] = {{&S68695[44], 16}, {NULL}};
const Segment SGI189191[] = {{&S68695[44], 28}, {NULL}};
const Segment SGI189192[] = {{&S68695[30], 6}, {NULL}};
const Segment SGI189193[] = {{&S68695[121], 3}, {NULL}};
const Segment SGI189194[] = {{&S68695[86], 6}, {NULL}};
const Segment SGI189195[] = {{&S63937[34], 7}, {NULL}};
const Segment SGI189196[] = {{&S63937[27], 6}, {NULL}};
const Segment SGI189197[] = {{&S63937[98], 9}, {NULL}};
const Segment SGI189198[] = {{&S63937[71], 6}, {NULL}};
const Segment SGI189199[] = {{&S63951[34], 8}, {NULL}};
const Segment SGI189200[] = {{&S63951[27], 6}, {NULL}};
const Segment SGI189201[] = {{&S63951[101], 9}, {NULL}};
const Segment SGI189202[] = {{&S63951[74], 6}, {NULL}};
const Segment SGI189203[] = {{&S68720[44], 9}, {NULL}};
const Segment SGI189204[] = {{&S68720[44], 21}, {NULL}};
const Segment SGI189205[] = {{&S68720[30], 6}, {NULL}};
const Segment SGI189206[] = {{&S68720[114], 3}, {NULL}};
const Segment SGI189207[] = {{&S68720[79], 6}, {NULL}};
const Segment SGI189208[] = {{&S75350[44], 4}, {NULL}};
const Segment SGI189209[] = {{&S75350[44], 16}, {NULL}};
const Segment SGI189210[] = {{&S75350[30], 6}, {NULL}};
const Segment SGI189211[] = {{&S75350[31], 3}, {NULL}};
const Segment SGI189212[] = {{&S75350[74], 6}, {NULL}};
const Segment SGI189213[] = {{&S75351[44], 8}, {NULL}};
const Segment SGI189214[] = {{&S75351[44], 20}, {NULL}};
const Segment SGI189215[] = {{&S75351[30], 6}, {NULL}};
const Segment SGI189216[] = {{&S75351[31], 3}, {NULL}};
const Segment SGI189217[] = {{&S75351[78], 6}, {NULL}};
const Segment SGI189218[] = {{&S75360[34], 17}, {NULL}};
const Segment SGI189219[] = {{&S75360[27], 6}, {NULL}};
const Segment SGI189220[] = {{&S75360[104], 9}, {NULL}};
const Segment SGI189221[] = {{&S75360[77], 6}, {NULL}};
const Segment SGI189222[] = {{&S68734[34], 6}, {NULL}};
const Segment SGI189223[] = {{&S68734[27], 6}, {NULL}};
const Segment SGI189224[] = {{&S68734[93], 9}, {NULL}};
const Segment SGI189225[] = {{&S68734[66], 6}, {NULL}};
const Segment SGI189226[] = {{&S75370[34], 11}, {NULL}};
const Segment SGI189227[] = {{&S75370[27], 6}, {NULL}};
const Segment SGI189228[] = {{&S53620[41], 18}, {NULL}};
const Segment SGI189229[] = {{&S53620[41], 29}, {NULL}};
const Segment SGI189230[] = {{&S53620[27], 6}, {NULL}};
const Segment SGI189231[] = {{&S75370[98], 9}, {NULL}};
const Segment SGI189232[] = {{&S75370[71], 6}, {NULL}};
const Segment SGI189233[] = {{&S53620[119], 3}, {NULL}};
const Segment SGI189234[] = {{&S53620[123], 9}, {NULL}};
const Segment SGI189235[] = {{&S53620[84], 6}, {NULL}};
const Segment SGI189236[] = {{&S53621[41], 11}, {NULL}};
const Segment SGI189237[] = {{&S53621[41], 22}, {NULL}};
const Segment SGI189238[] = {{&S53621[27], 6}, {NULL}};
const Segment SGI189239[] = {{&S53621[112], 3}, {NULL}};
const Segment SGI189240[] = {{&S53621[116], 9}, {NULL}};
const Segment SGI189241[] = {{&S53621[77], 6}, {NULL}};
const Segment SGI189242[] = {{&S53622[41], 18}, {NULL}};
const Segment SGI189243[] = {{&S53622[41], 29}, {NULL}};
const Segment SGI189244[] = {{&S53622[27], 6}, {NULL}};
const Segment SGI189245[] = {{&S53622[119], 3}, {NULL}};
const Segment SGI189246[] = {{&S53622[123], 9}, {NULL}};
const Segment SGI189247[] = {{&S53622[84], 6}, {NULL}};
const Segment SGI189248[] = {{&S75382[34], 7}, {NULL}};
const Segment SGI189249[] = {{&S75382[27], 6}, {NULL}};
const Segment SGI189250[] = {{&S75382[103], 9}, {NULL}};
const Segment SGI189251[] = {{&S75382[76], 6}, {NULL}};
const Segment SGI189252[] = {{&S75384[44], 7}, {NULL}};
const Segment SGI189253[] = {{&S75384[44], 24}, {NULL}};
const Segment SGI189254[] = {{&S75384[30], 6}, {NULL}};
const Segment SGI189255[] = {{&S75384[82], 6}, {NULL}};
const Segment SGI189256[] = {{&S75384[109], 13}, {NULL}};
const Segment SGI189257[] = {{&S75385[44], 7}, {NULL}};
const Segment SGI189258[] = {{&S75385[44], 24}, {NULL}};
const Segment SGI189259[] = {{&S75385[30], 6}, {NULL}};
const Segment SGI189260[] = {{&S75385[82], 6}, {NULL}};
const Segment SGI189261[] = {{&S75385[109], 13}, {NULL}};
const Segment SGI189262[] = {{&S75386[44], 8}, {NULL}};
const Segment SGI189263[] = {{&S75386[44], 24}, {NULL}};
const Segment SGI189264[] = {{&S75386[30], 6}, {NULL}};
const Segment SGI189265[] = {{&S75386[82], 6}, {NULL}};
const Segment SGI189266[] = {{&S75386[109], 13}, {NULL}};
const Segment SGI189267[] = {{&S75387[44], 7}, {NULL}};
const Segment SGI189268[] = {{&S75387[44], 24}, {NULL}};
const Segment SGI189269[] = {{&S75387[30], 6}, {NULL}};
const Segment SGI189270[] = {{&S75387[82], 6}, {NULL}};
const Segment SGI189271[] = {{&S75387[109], 13}, {NULL}};
const Segment SGI189272[] = {{&S64006[34], 29}, {NULL}};
const Segment SGI189273[] = {{&S64006[27], 6}, {NULL}};
const Segment SGI189274[] = {{&S75388[44], 8}, {NULL}};
const Segment SGI189275[] = {{&S64006[116], 9}, {NULL}};
const Segment SGI189276[] = {{&S75388[44], 24}, {NULL}};
const Segment SGI189277[] = {{&S64006[89], 6}, {NULL}};
const Segment SGI189278[] = {{&S75388[30], 6}, {NULL}};
const Segment SGI189279[] = {{&S75388[82], 6}, {NULL}};
const Segment SGI189280[] = {{&S75388[109], 13}, {NULL}};
const Segment SGI189281[] = {{&S75389[44], 8}, {NULL}};
const Segment SGI189282[] = {{&S75389[44], 24}, {NULL}};
const Segment SGI189283[] = {{&S75389[30], 6}, {NULL}};
const Segment SGI189284[] = {{&S75389[82], 6}, {NULL}};
const Segment SGI189285[] = {{&S75389[109], 13}, {NULL}};
const Segment SGI189286[] = {{&S64013[34], 7}, {NULL}};
const Segment SGI189287[] = {{&S64013[27], 6}, {NULL}};
const Segment SGI189288[] = {{&S64013[94], 9}, {NULL}};
const Segment SGI189289[] = {{&S64013[67], 6}, {NULL}};
const Segment SGI189290[] = {{&S64014[34], 7}, {NULL}};
const Segment SGI189291[] = {{&S64014[27], 6}, {NULL}};
const Segment SGI189292[] = {{&S64014[94], 9}, {NULL}};
const Segment SGI189293[] = {{&S64014[67], 6}, {NULL}};
const Segment SGI189294[] = {{&S53653[34], 18}, {NULL}};
const Segment SGI189295[] = {{&S53653[27], 6}, {NULL}};
const Segment SGI189296[] = {{&S53653[104], 9}, {NULL}};
const Segment SGI189297[] = {{&S53653[77], 6}, {NULL}};
const Segment SGI189298[] = {{&S53654[34], 18}, {NULL}};
const Segment SGI189299[] = {{&S53654[27], 6}, {NULL}};
const Segment SGI189300[] = {{&S53654[104], 9}, {NULL}};
const Segment SGI189301[] = {{&S53654[77], 6}, {NULL}};
const Segment SGI189302[] = {{&S62619[44], 10}, {NULL}};
const Segment SGI189303[] = {{&S62619[44], 22}, {NULL}};
const Segment SGI189304[] = {{&S62619[30], 6}, {NULL}};
const Segment SGI189305[] = {{&S62619[133], 3}, {NULL}};
const Segment SGI189306[] = {{&S62619[98], 6}, {NULL}};
const Segment SGI189307[] = {{&S62619[68], 8}, {NULL}};
const Segment SGI189308[] = {{&S62621[44], 10}, {NULL}};
const Segment SGI189309[] = {{&S62621[44], 22}, {NULL}};
const Segment SGI189310[] = {{&S62621[30], 6}, {NULL}};
const Segment SGI189311[] = {{&S62621[133], 3}, {NULL}};
const Segment SGI189312[] = {{&S62621[98], 6}, {NULL}};
const Segment SGI189313[] = {{&S62621[68], 8}, {NULL}};
const Segment SGI189314[] = {{&S75419[44], 10}, {NULL}};
const Segment SGI189315[] = {{&S75419[44], 22}, {NULL}};
const Segment SGI189316[] = {{&S75419[30], 6}, {NULL}};
const Segment SGI189317[] = {{&S75419[31], 3}, {NULL}};
const Segment SGI189318[] = {{&S75419[80], 6}, {NULL}};
const Segment SGI189319[] = {{&S75420[38], 10}, {NULL}};
const Segment SGI189320[] = {{&S75420[31], 6}, {NULL}};
const Segment SGI189321[] = {{&S75425[44], 9}, {NULL}};
const Segment SGI189322[] = {{&S75425[44], 21}, {NULL}};
const Segment SGI189323[] = {{&S75425[30], 6}, {NULL}};
const Segment SGI189324[] = {{&S75425[31], 3}, {NULL}};
const Segment SGI189325[] = {{&S75425[79], 6}, {NULL}};
const Segment SGI189326[] = {{&S75427[44], 15}, {NULL}};
const Segment SGI189327[] = {{&S75427[44], 27}, {NULL}};
const Segment SGI189328[] = {{&S75427[30], 6}, {NULL}};
const Segment SGI189329[] = {{&S75427[31], 3}, {NULL}};
const Segment SGI189330[] = {{&S75427[85], 6}, {NULL}};
const Segment SGI189331[] = {{&S64056[44], 10}, {NULL}};
const Segment SGI189332[] = {{&S64056[44], 22}, {NULL}};
const Segment SGI189333[] = {{&S64056[30], 6}, {NULL}};
const Segment SGI189334[] = {{&S64056[80], 6}, {NULL}};
const Segment SGI189335[] = {{&S64056[107], 13}, {NULL}};
const Segment SGI189336[] = {{&S53701[34], 11}, {NULL}};
const Segment SGI189337[] = {{&S53701[27], 6}, {NULL}};
const Segment SGI189338[] = {{&S53701[97], 9}, {NULL}};
const Segment SGI189339[] = {{&S53701[70], 6}, {NULL}};
const Segment SGI189340[] = {{&S68753[44], 13}, {NULL}};
const Segment SGI189341[] = {{&S68753[44], 25}, {NULL}};
const Segment SGI189342[] = {{&S68753[30], 6}, {NULL}};
const Segment SGI189343[] = {{&S68753[118], 3}, {NULL}};
const Segment SGI189344[] = {{&S68753[83], 6}, {NULL}};
const Segment SGI189345[] = {{&S75455[34], 11}, {NULL}};
const Segment SGI189346[] = {{&S75455[27], 6}, {NULL}};
const Segment SGI189347[] = {{&S75455[98], 9}, {NULL}};
const Segment SGI189348[] = {{&S75455[71], 6}, {NULL}};
const Segment SGI189349[] = {{&S68754[38], 13}, {NULL}};
const Segment SGI189350[] = {{&S68754[31], 6}, {NULL}};
const Segment SGI189351[] = {{&S68755[44], 13}, {NULL}};
const Segment SGI189352[] = {{&S68755[44], 25}, {NULL}};
const Segment SGI189353[] = {{&S68755[30], 6}, {NULL}};
const Segment SGI189354[] = {{&S68755[118], 3}, {NULL}};
const Segment SGI189355[] = {{&S68755[83], 6}, {NULL}};
const Segment SGI189356[] = {{&S75457[34], 11}, {NULL}};
const Segment SGI189357[] = {{&S75457[27], 6}, {NULL}};
const Segment SGI189358[] = {{&S75457[98], 9}, {NULL}};
const Segment SGI189359[] = {{&S75457[71], 6}, {NULL}};
const Segment SGI189360[] = {{&S53712[44], 10}, {NULL}};
const Segment SGI189361[] = {{&S53712[44], 21}, {NULL}};
const Segment SGI189362[] = {{&S53712[30], 6}, {NULL}};
const Segment SGI189363[] = {{&S53712[131], 3}, {NULL}};
const Segment SGI189364[] = {{&S53712[97], 6}, {NULL}};
const Segment SGI189365[] = {{&S53712[67], 8}, {NULL}};
const Segment SGI189366[] = {{&S53714[44], 10}, {NULL}};
const Segment SGI189367[] = {{&S53714[44], 22}, {NULL}};
const Segment SGI189368[] = {{&S53714[30], 6}, {NULL}};
const Segment SGI189369[] = {{&S53714[115], 3}, {NULL}};
const Segment SGI189370[] = {{&S53714[80], 6}, {NULL}};
const Segment SGI189371[] = {{&S53714[140], 8}, {NULL}};
const Segment SGI189372[] = {{&S75465[34], 14}, {NULL}};
const Segment SGI189373[] = {{&S75465[27], 6}, {NULL}};
const Segment SGI189374[] = {{&S75465[101], 9}, {NULL}};
const Segment SGI189375[] = {{&S75465[74], 6}, {NULL}};
const Segment SGI189376[] = {{&S64094[44], 10}, {NULL}};
const Segment SGI189377[] = {{&S64094[44], 22}, {NULL}};
const Segment SGI189378[] = {{&S53725[44], 10}, {NULL}};
const Segment SGI189379[] = {{&S64094[30], 6}, {NULL}};
const Segment SGI189380[] = {{&S53725[44], 21}, {NULL}};
const Segment SGI189381[] = {{&S53725[30], 6}, {NULL}};
const Segment SGI189382[] = {{&S64094[146], 4}, {NULL}};
const Segment SGI189383[] = {{&S64094[172], 3}, {NULL}};
const Segment SGI189384[] = {{&S53725[132], 3}, {NULL}};
const Segment SGI189385[] = {{&S64094[115], 3}, {&S64094[221], 5}, {NULL}};
const Segment SGI189386[] = {{&S53725[97], 6}, {NULL}};
const Segment SGI189387[] = {{&S53725[67], 8}, {NULL}};
const Segment SGI189388[] = {{&S64094[80], 6}, {NULL}};
const Segment SGI189389[] = {{&S53726[44], 10}, {NULL}};
const Segment SGI189390[] = {{&S53726[44], 22}, {NULL}};
const Segment SGI189391[] = {{&S53726[30], 6}, {NULL}};
const Segment SGI189392[] = {{&S53726[133], 3}, {NULL}};
const Segment SGI189393[] = {{&S53726[98], 6}, {NULL}};
const Segment SGI189394[] = {{&S53726[68], 8}, {NULL}};
const Segment SGI189395[] = {{&S62699[34], 13}, {NULL}};
const Segment SGI189396[] = {{&S62699[27], 6}, {NULL}};
const Segment SGI189397[] = {{&S62699[100], 9}, {NULL}};
const Segment SGI189398[] = {{&S62699[73], 6}, {NULL}};
const Segment SGI189399[] = {{&S75485[34], 11}, {NULL}};
const Segment SGI189400[] = {{&S75485[27], 6}, {NULL}};
const Segment SGI189401[] = {{&S75485[98], 9}, {NULL}};
const Segment SGI189402[] = {{&S64106[34], 9}, {NULL}};
const Segment SGI189403[] = {{&S75485[71], 6}, {NULL}};
const Segment SGI189404[] = {{&S64106[27], 6}, {NULL}};
const Segment SGI189405[] = {{&S64106[96], 9}, {NULL}};
const Segment SGI189406[] = {{&S64106[69], 6}, {NULL}};
const Segment SGI189407[] = {{&S75493[44], 10}, {NULL}};
const Segment SGI189408[] = {{&S75493[44], 22}, {NULL}};
const Segment SGI189409[] = {{&S75493[30], 6}, {NULL}};
const Segment SGI189410[] = {{&S75493[31], 3}, {NULL}};
const Segment SGI189411[] = {{&S75493[80], 6}, {NULL}};
const Segment SGI189412[] = {{&S75495[34], 18}, {NULL}};
const Segment SGI189413[] = {{&S75495[27], 6}, {NULL}};
const Segment SGI189414[] = {{&S75495[105], 9}, {NULL}};
const Segment SGI189415[] = {{&S75495[78], 6}, {NULL}};
const Segment SGI189416[] = {{&S75496[34], 10}, {NULL}};
const Segment SGI189417[] = {{&S75496[27], 6}, {NULL}};
const Segment SGI189418[] = {{&S75496[97], 9}, {NULL}};
const Segment SGI189419[] = {{&S75496[70], 6}, {NULL}};
const Segment SGI189420[] = {{&S68808[34], 12}, {NULL}};
const Segment SGI189421[] = {{&S68808[27], 6}, {NULL}};
const Segment SGI189422[] = {{&S68808[99], 9}, {NULL}};
const Segment SGI189423[] = {{&S64123[44], 13}, {NULL}};
const Segment SGI189424[] = {{&S68808[72], 6}, {NULL}};
const Segment SGI189425[] = {{&S64123[44], 25}, {NULL}};
const Segment SGI189426[] = {{&S64123[30], 6}, {NULL}};
const Segment SGI189427[] = {{&S64123[118], 3}, {NULL}};
const Segment SGI189428[] = {{&S64123[83], 6}, {NULL}};
const Segment SGI189429[] = {{&S75500[34], 18}, {NULL}};
const Segment SGI189430[] = {{&S75500[27], 6}, {NULL}};
const Segment SGI189431[] = {{&S75500[105], 9}, {NULL}};
const Segment SGI189432[] = {{&S75500[78], 6}, {NULL}};
const Segment SGI189433[] = {{&S64124[44], 14}, {NULL}};
const Segment SGI189434[] = {{&S64124[44], 26}, {NULL}};
const Segment SGI189435[] = {{&S64124[30], 6}, {NULL}};
const Segment SGI189436[] = {{&S64124[119], 3}, {NULL}};
const Segment SGI189437[] = {{&S64124[84], 6}, {NULL}};
const Segment SGI189438[] = {{&S75501[44], 11}, {NULL}};
const Segment SGI189439[] = {{&S75501[44], 32}, {NULL}};
const Segment SGI189440[] = {{&S75501[30], 6}, {NULL}};
const Segment SGI189441[] = {{&S75501[31], 3}, {NULL}};
const Segment SGI189442[] = {{&S75501[90], 6}, {NULL}};
const Segment SGI189443[] = {{&S64125[44], 14}, {NULL}};
const Segment SGI189444[] = {{&S64125[44], 26}, {NULL}};
const Segment SGI189445[] = {{&S64125[30], 6}, {NULL}};
const Segment SGI189446[] = {{&S64125[87], 6}, {NULL}};
const Segment SGI189447[] = {{&S64126[38], 13}, {NULL}};
const Segment SGI189448[] = {{&S64126[31], 6}, {NULL}};
const Segment SGI189449[] = {{&S62728[44], 10}, {NULL}};
const Segment SGI189450[] = {{&S64127[44], 24}, {NULL}};
const Segment SGI189451[] = {{&S62728[44], 22}, {NULL}};
const Segment SGI189452[] = {{&S64127[30], 6}, {NULL}};
const Segment SGI189453[] = {{&S62728[30], 6}, {NULL}};
const Segment SGI189454[] = {{&S64127[31], 3}, {NULL}};
const Segment SGI189455[] = {{&S62728[115], 3}, {NULL}};
const Segment SGI189456[] = {{&S62728[80], 6}, {NULL}};
const Segment SGI189457[] = {{&S64127[100], 6}, {NULL}};
const Segment SGI189458[] = {{&S64128[38], 14}, {NULL}};
const Segment SGI189459[] = {{&S64128[31], 6}, {NULL}};
const Segment SGI189460[] = {{&S62730[44], 10}, {NULL}};
const Segment SGI189461[] = {{&S62730[44], 22}, {NULL}};
const Segment SGI189462[] = {{&S62730[30], 6}, {NULL}};
const Segment SGI189463[] = {{&S62730[115], 3}, {NULL}};
const Segment SGI189464[] = {{&S62730[80], 6}, {NULL}};
const Segment SGI189465[] = {{&S64129[43], 13}, {NULL}};
const Segment SGI189466[] = {{&S64129[43], 25}, {NULL}};
const Segment SGI189467[] = {{&S64129[30], 6}, {NULL}};
const Segment SGI189468[] = {{&S64129[117], 3}, {NULL}};
const Segment SGI189469[] = {{&S64129[82], 6}, {NULL}};
const Segment SGI189470[] = {{&S64130[44], 14}, {NULL}};
const Segment SGI189471[] = {{&S64130[44], 26}, {NULL}};
const Segment SGI189472[] = {{&S64130[30], 6}, {NULL}};
const Segment SGI189473[] = {{&S64130[119], 3}, {NULL}};
const Segment SGI189474[] = {{&S64130[84], 6}, {NULL}};
const Segment SGI189475[] = {{&S62733[34], 10}, {NULL}};
const Segment SGI189476[] = {{&S62733[27], 6}, {NULL}};
const Segment SGI189477[] = {{&S62733[97], 9}, {NULL}};
const Segment SGI189478[] = {{&S62733[70], 6}, {NULL}};
const Segment SGI189479[] = {{&S75516[44], 11}, {NULL}};
const Segment SGI189480[] = {{&S75516[44], 23}, {NULL}};
const Segment SGI189481[] = {{&S75516[30], 6}, {NULL}};
const Segment SGI189482[] = {{&S75516[31], 3}, {NULL}};
const Segment SGI189483[] = {{&S75516[81], 6}, {NULL}};
const Segment SGI189484[] = {{&S75517[44], 11}, {NULL}};
const Segment SGI189485[] = {{&S75517[44], 23}, {NULL}};
const Segment SGI189486[] = {{&S75517[30], 6}, {NULL}};
const Segment SGI189487[] = {{&S75517[84], 6}, {NULL}};
const Segment SGI189488[] = {{&S64137[34], 8}, {NULL}};
const Segment SGI189489[] = {{&S64137[27], 6}, {NULL}};
const Segment SGI189490[] = {{&S64137[95], 9}, {NULL}};
const Segment SGI189491[] = {{&S64137[68], 6}, {NULL}};
const Segment SGI189492[] = {{&S75518[44], 11}, {NULL}};
const Segment SGI189493[] = {{&S75518[44], 23}, {NULL}};
const Segment SGI189494[] = {{&S75518[30], 6}, {NULL}};
const Segment SGI189495[] = {{&S75518[84], 4}, {NULL}};
const Segment SGI189496[] = {{&S75518[84], 3}, {NULL}};
const Segment SGI189497[] = {{&S75531[34], 7}, {NULL}};
const Segment SGI189498[] = {{&S75531[27], 6}, {NULL}};
const Segment SGI189499[] = {{&S75531[99], 9}, {NULL}};
const Segment SGI189500[] = {{&S75531[72], 6}, {NULL}};
const Segment SGI189501[] = {{&S75531[140], 8}, {NULL}};
const Segment SGI189502[] = {{&S75532[34], 8}, {NULL}};
const Segment SGI189503[] = {{&S75532[27], 6}, {NULL}};
const Segment SGI189504[] = {{&S75532[99], 9}, {NULL}};
const Segment SGI189505[] = {{&S75532[72], 6}, {NULL}};
const Segment SGI189506[] = {{&S75532[140], 8}, {NULL}};
const Segment SGI189507[] = {{&S75538[34], 10}, {NULL}};
const Segment SGI189508[] = {{&S75538[27], 6}, {NULL}};
const Segment SGI189509[] = {{&S75538[97], 9}, {NULL}};
const Segment SGI189510[] = {{&S75538[70], 6}, {NULL}};
const Segment SGI189511[] = {{&S75541[34], 14}, {NULL}};
const Segment SGI189512[] = {{&S75541[27], 6}, {NULL}};
const Segment SGI189513[] = {{&S75541[101], 9}, {NULL}};
const Segment SGI189514[] = {{&S75541[74], 6}, {NULL}};
const Segment SGI189515[] = {{&S64172[44], 10}, {NULL}};
const Segment SGI189516[] = {{&S64172[44], 22}, {NULL}};
const Segment SGI189517[] = {{&S64172[30], 6}, {NULL}};
const Segment SGI189518[] = {{&S64172[131], 3}, {NULL}};
const Segment SGI189519[] = {{&S64172[96], 6}, {NULL}};
const Segment SGI189520[] = {{&S64172[68], 8}, {NULL}};
const Segment SGI189521[] = {{&S64176[34], 18}, {NULL}};
const Segment SGI189522[] = {{&S64176[27], 6}, {NULL}};
const Segment SGI189523[] = {{&S64176[105], 9}, {NULL}};
const Segment SGI189524[] = {{&S64176[78], 6}, {NULL}};
const Segment SGI189525[] = {{&S64177[34], 19}, {NULL}};
const Segment SGI189526[] = {{&S64177[27], 6}, {NULL}};
const Segment SGI189527[] = {{&S64177[106], 9}, {NULL}};
const Segment SGI189528[] = {{&S64177[79], 6}, {NULL}};
const Segment SGI189529[] = {{&S53816[44], 10}, {NULL}};
const Segment SGI189530[] = {{&S53816[44], 21}, {NULL}};
const Segment SGI189531[] = {{&S53816[30], 6}, {NULL}};
const Segment SGI189532[] = {{&S53816[114], 3}, {NULL}};
const Segment SGI189533[] = {{&S53816[79], 6}, {NULL}};
const Segment SGI189534[] = {{&S75556[44], 7}, {NULL}};
const Segment SGI189535[] = {{&S75556[44], 19}, {NULL}};
const Segment SGI189536[] = {{&S75556[30], 6}, {NULL}};
const Segment SGI189537[] = {{&S75556[31], 3}, {NULL}};
const Segment SGI189538[] = {{&S75556[77], 6}, {NULL}};
const Segment SGI189539[] = {{&S53817[38], 10}, {NULL}};
const Segment SGI189540[] = {{&S53817[31], 6}, {NULL}};
const Segment SGI189541[] = {{&S53818[44], 11}, {NULL}};
const Segment SGI189542[] = {{&S53818[44], 22}, {NULL}};
const Segment SGI189543[] = {{&S53818[30], 6}, {NULL}};
const Segment SGI189544[] = {{&S53818[115], 3}, {NULL}};
const Segment SGI189545[] = {{&S53818[80], 6}, {NULL}};
const Segment SGI189546[] = {{&S53819[44], 11}, {NULL}};
const Segment SGI189547[] = {{&S75560[44], 10}, {NULL}};
const Segment SGI189548[] = {{&S53819[44], 22}, {NULL}};
const Segment SGI189549[] = {{&S75560[44], 22}, {NULL}};
const Segment SGI189550[] = {{&S53819[30], 6}, {NULL}};
const Segment SGI189551[] = {{&S75560[30], 6}, {NULL}};
const Segment SGI189552[] = {{&S53819[115], 3}, {NULL}};
const Segment SGI189553[] = {{&S75560[31], 3}, {NULL}};
const Segment SGI189554[] = {{&S53819[80], 6}, {NULL}};
const Segment SGI189555[] = {{&S75560[95], 6}, {NULL}};
const Segment SGI189556[] = {{&S75560[68], 8}, {NULL}};
const Segment SGI189557[] = {{&S75563[44], 5}, {NULL}};
const Segment SGI189558[] = {{&S75563[44], 17}, {NULL}};
const Segment SGI189559[] = {{&S75563[30], 6}, {NULL}};
const Segment SGI189560[] = {{&S75563[75], 6}, {NULL}};
const Segment SGI189561[] = {{&S75563[102], 13}, {NULL}};
const Segment SGI189562[] = {{&S75564[44], 13}, {NULL}};
const Segment SGI189563[] = {{&S75564[44], 25}, {NULL}};
const Segment SGI189564[] = {{&S75564[30], 6}, {NULL}};
const Segment SGI189565[] = {{&S75564[83], 6}, {NULL}};
const Segment SGI189566[] = {{&S75564[110], 13}, {NULL}};
const Segment SGI189567[] = {{&S75565[34], 7}, {NULL}};
const Segment SGI189568[] = {{&S75565[27], 6}, {NULL}};
const Segment SGI189569[] = {{&S75565[99], 9}, {NULL}};
const Segment SGI189570[] = {{&S75565[72], 6}, {NULL}};
const Segment SGI189571[] = {{&S64204[34], 10}, {NULL}};
const Segment SGI189572[] = {{&S64204[27], 6}, {NULL}};
const Segment SGI189573[] = {{&S64204[97], 9}, {NULL}};
const Segment SGI189574[] = {{&S64204[70], 6}, {NULL}};
const Segment SGI189575[] = {{&S64205[34], 11}, {NULL}};
const Segment SGI189576[] = {{&S64205[27], 6}, {NULL}};
const Segment SGI189577[] = {{&S64205[98], 9}, {NULL}};
const Segment SGI189578[] = {{&S64205[71], 6}, {NULL}};
const Segment SGI189579[] = {{&S64211[34], 11}, {NULL}};
const Segment SGI189580[] = {{&S64211[27], 6}, {NULL}};
const Segment SGI189581[] = {{&S64211[98], 9}, {NULL}};
const Segment SGI189582[] = {{&S64211[71], 6}, {NULL}};
const Segment SGI189583[] = {{&S53850[41], 18}, {NULL}};
const Segment SGI189584[] = {{&S53850[41], 29}, {NULL}};
const Segment SGI189585[] = {{&S53850[27], 6}, {NULL}};
const Segment SGI189586[] = {{&S53850[119], 3}, {NULL}};
const Segment SGI189587[] = {{&S53850[123], 9}, {NULL}};
const Segment SGI189588[] = {{&S53850[84], 6}, {NULL}};
const Segment SGI189589[] = {{&S62837[38], 10}, {NULL}};
const Segment SGI189590[] = {{&S62837[31], 6}, {NULL}};
const Segment SGI189591[] = {{&S75595[44], 9}, {NULL}};
const Segment SGI189592[] = {{&S75595[44], 21}, {NULL}};
const Segment SGI189593[] = {{&S75595[30], 6}, {NULL}};
const Segment SGI189594[] = {{&S75595[31], 3}, {NULL}};
const Segment SGI189595[] = {{&S75595[79], 6}, {NULL}};
const Segment SGI189596[] = {{&S75596[44], 10}, {NULL}};
const Segment SGI189597[] = {{&S75596[44], 22}, {NULL}};
const Segment SGI189598[] = {{&S75596[30], 6}, {NULL}};
const Segment SGI189599[] = {{&S75596[31], 3}, {NULL}};
const Segment SGI189600[] = {{&S75596[80], 6}, {NULL}};
const Segment SGI189601[] = {{&S64227[34], 7}, {NULL}};
const Segment SGI189602[] = {{&S64227[27], 6}, {NULL}};
const Segment SGI189603[] = {{&S64227[99], 9}, {NULL}};
const Segment SGI189604[] = {{&S64227[72], 6}, {NULL}};
const Segment SGI189605[] = {{&S75597[44], 9}, {NULL}};
const Segment SGI189606[] = {{&S75597[44], 21}, {NULL}};
const Segment SGI189607[] = {{&S75597[30], 6}, {NULL}};
const Segment SGI189608[] = {{&S75597[82], 6}, {NULL}};
const Segment SGI189609[] = {{&S62845[34], 10}, {NULL}};
const Segment SGI189610[] = {{&S62845[27], 6}, {NULL}};
const Segment SGI189611[] = {{&S62845[97], 9}, {NULL}};
const Segment SGI189612[] = {{&S62845[70], 6}, {NULL}};
const Segment SGI189613[] = {{&S53872[44], 10}, {NULL}};
const Segment SGI189614[] = {{&S75605[44], 8}, {NULL}};
const Segment SGI189615[] = {{&S75605[44], 20}, {NULL}};
const Segment SGI189616[] = {{&S53872[44], 21}, {NULL}};
const Segment SGI189617[] = {{&S75605[30], 6}, {NULL}};
const Segment SGI189618[] = {{&S53872[30], 6}, {NULL}};
const Segment SGI189619[] = {{&S75605[31], 3}, {NULL}};
const Segment SGI189620[] = {{&S53872[132], 3}, {NULL}};
const Segment SGI189621[] = {{&S75605[78], 6}, {NULL}};
const Segment SGI189622[] = {{&S53872[97], 6}, {NULL}};
const Segment SGI189623[] = {{&S53872[67], 8}, {NULL}};
const Segment SGI189624[] = {{&S64238[34], 12}, {NULL}};
const Segment SGI189625[] = {{&S64238[27], 6}, {NULL}};
const Segment SGI189626[] = {{&S64238[99], 9}, {NULL}};
const Segment SGI189627[] = {{&S64238[72], 6}, {NULL}};
const Segment SGI189628[] = {{&S53873[44], 10}, {NULL}};
const Segment SGI189629[] = {{&S53873[44], 22}, {NULL}};
const Segment SGI189630[] = {{&S53873[30], 6}, {NULL}};
const Segment SGI189631[] = {{&S53873[115], 3}, {NULL}};
const Segment SGI189632[] = {{&S53873[80], 6}, {NULL}};
const Segment SGI189633[] = {{&S53873[140], 8}, {NULL}};
const Segment SGI189634[] = {{&S75609[34], 10}, {NULL}};
const Segment SGI189635[] = {{&S75609[27], 6}, {NULL}};
const Segment SGI189636[] = {{&S75609[97], 9}, {NULL}};
const Segment SGI189637[] = {{&S75609[70], 6}, {NULL}};
const Segment SGI189638[] = {{&S75613[34], 10}, {NULL}};
const Segment SGI189639[] = {{&S75613[27], 6}, {NULL}};
const Segment SGI189640[] = {{&S75613[97], 9}, {NULL}};
const Segment SGI189641[] = {{&S75613[70], 6}, {NULL}};
const Segment SGI189642[] = {{&S75627[34], 18}, {NULL}};
const Segment SGI189643[] = {{&S75627[27], 6}, {NULL}};
const Segment SGI189644[] = {{&S75627[105], 9}, {NULL}};
const Segment SGI189645[] = {{&S75627[78], 6}, {NULL}};
const Segment SGI189646[] = {{&S75628[34], 11}, {NULL}};
const Segment SGI189647[] = {{&S75628[27], 6}, {NULL}};
const Segment SGI189648[] = {{&S75628[98], 9}, {NULL}};
const Segment SGI189649[] = {{&S75628[71], 6}, {NULL}};
const Segment SGI189650[] = {{&S75629[34], 11}, {NULL}};
const Segment SGI189651[] = {{&S75629[27], 6}, {NULL}};
const Segment SGI189652[] = {{&S75629[98], 9}, {NULL}};
const Segment SGI189653[] = {{&S75629[71], 6}, {NULL}};
const Segment SGI189654[] = {{&S75637[44], 18}, {NULL}};
const Segment SGI189655[] = {{&S75637[44], 30}, {NULL}};
const Segment SGI189656[] = {{&S75637[30], 6}, {NULL}};
const Segment SGI189657[] = {{&S75637[88], 6}, {NULL}};
const Segment SGI189658[] = {{&S75637[115], 13}, {NULL}};
const Segment SGI189659[] = {{&S75639[34], 11}, {NULL}};
const Segment SGI189660[] = {{&S75639[27], 6}, {NULL}};
const Segment SGI189661[] = {{&S75639[98], 9}, {NULL}};
const Segment SGI189662[] = {{&S75639[71], 6}, {NULL}};
const Segment SGI189663[] = {{&S75641[34], 7}, {NULL}};
const Segment SGI189664[] = {{&S75641[27], 6}, {NULL}};
const Segment SGI189665[] = {{&S75641[99], 9}, {NULL}};
const Segment SGI189666[] = {{&S75641[72], 6}, {NULL}};
const Segment SGI189667[] = {{&S75641[140], 8}, {NULL}};
const Segment SGI189668[] = {{&S68856[44], 84}, {NULL}};
const Segment SGI189669[] = {{&S68856[44], 96}, {NULL}};
const Segment SGI189670[] = {{&S68856[30], 6}, {NULL}};
const Segment SGI189671[] = {{&S68856[189], 3}, {NULL}};
const Segment SGI189672[] = {{&S68856[121], 6}, {NULL}};
const Segment SGI189673[] = {{&S64289[34], 7}, {NULL}};
const Segment SGI189674[] = {{&S64289[27], 6}, {NULL}};
const Segment SGI189675[] = {{&S68857[44], 10}, {NULL}};
const Segment SGI189676[] = {{&S68857[44], 22}, {NULL}};
const Segment SGI189677[] = {{&S68857[30], 6}, {NULL}};
const Segment SGI189678[] = {{&S64289[98], 9}, {NULL}};
const Segment SGI189679[] = {{&S64289[71], 6}, {NULL}};
const Segment SGI189680[] = {{&S68857[83], 6}, {NULL}};
const Segment SGI189681[] = {{&S64290[34], 6}, {NULL}};
const Segment SGI189682[] = {{&S68858[44], 10}, {NULL}};
const Segment SGI189683[] = {{&S64290[27], 6}, {NULL}};
const Segment SGI189684[] = {{&S68858[44], 22}, {NULL}};
const Segment SGI189685[] = {{&S68858[30], 6}, {NULL}};
const Segment SGI189686[] = {{&S64290[93], 9}, {NULL}};
const Segment SGI189687[] = {{&S68858[115], 3}, {NULL}};
const Segment SGI189688[] = {{&S64290[66], 6}, {NULL}};
const Segment SGI189689[] = {{&S68858[80], 6}, {NULL}};
const Segment SGI189690[] = {{&S64291[34], 9}, {NULL}};
const Segment SGI189691[] = {{&S64291[27], 6}, {NULL}};
const Segment SGI189692[] = {{&S64291[96], 9}, {NULL}};
const Segment SGI189693[] = {{&S64291[69], 6}, {NULL}};
const Segment SGI189694[] = {{&S75650[34], 7}, {NULL}};
const Segment SGI189695[] = {{&S75650[27], 6}, {NULL}};
const Segment SGI189696[] = {{&S75650[99], 9}, {NULL}};
const Segment SGI189697[] = {{&S75650[72], 6}, {NULL}};
const Segment SGI189698[] = {{&S75678[34], 18}, {NULL}};
const Segment SGI189699[] = {{&S75678[27], 6}, {NULL}};
const Segment SGI189700[] = {{&S75678[105], 9}, {NULL}};
const Segment SGI189701[] = {{&S75678[78], 6}, {NULL}};
const Segment SGI189702[] = {{&S75679[34], 19}, {NULL}};
const Segment SGI189703[] = {{&S75679[27], 6}, {NULL}};
const Segment SGI189704[] = {{&S75679[106], 9}, {NULL}};
const Segment SGI189705[] = {{&S75679[79], 6}, {NULL}};
const Segment SGI189706[] = {{&S75680[34], 19}, {NULL}};
const Segment SGI189707[] = {{&S75680[27], 6}, {NULL}};
const Segment SGI189708[] = {{&S75680[106], 9}, {NULL}};
const Segment SGI189709[] = {{&S75680[79], 6}, {NULL}};
const Segment SGI189710[] = {{&S68902[44], 11}, {NULL}};
const Segment SGI189711[] = {{&S68902[44], 23}, {NULL}};
const Segment SGI189712[] = {{&S68902[30], 6}, {NULL}};
const Segment SGI189713[] = {{&S68902[116], 3}, {NULL}};
const Segment SGI189714[] = {{&S68902[81], 6}, {NULL}};
const Segment SGI189715[] = {{&S75681[34], 16}, {NULL}};
const Segment SGI189716[] = {{&S75681[27], 6}, {NULL}};
const Segment SGI189717[] = {{&S75681[103], 9}, {NULL}};
const Segment SGI189718[] = {{&S75681[76], 6}, {NULL}};
const Segment SGI189719[] = {{&S68903[44], 11}, {NULL}};
const Segment SGI189720[] = {{&S68903[44], 23}, {NULL}};
const Segment SGI189721[] = {{&S68903[30], 6}, {NULL}};
const Segment SGI189722[] = {{&S68903[116], 3}, {NULL}};
const Segment SGI189723[] = {{&S75682[34], 16}, {NULL}};
const Segment SGI189724[] = {{&S68903[81], 6}, {NULL}};
const Segment SGI189725[] = {{&S75682[27], 6}, {NULL}};
const Segment SGI189726[] = {{&S75682[103], 9}, {NULL}};
const Segment SGI189727[] = {{&S75682[76], 6}, {NULL}};
const Segment SGI189728[] = {{&S64338[34], 10}, {NULL}};
const Segment SGI189729[] = {{&S64338[27], 6}, {NULL}};
const Segment SGI189730[] = {{&S64338[97], 9}, {NULL}};
const Segment SGI189731[] = {{&S64338[70], 6}, {NULL}};
const Segment SGI189732[] = {{&S64339[34], 9}, {NULL}};
const Segment SGI189733[] = {{&S64339[27], 6}, {NULL}};
const Segment SGI189734[] = {{&S64339[96], 9}, {NULL}};
const Segment SGI189735[] = {{&S64339[69], 6}, {NULL}};
const Segment SGI189736[] = {{&S64340[34], 9}, {NULL}};
const Segment SGI189737[] = {{&S64340[27], 6}, {NULL}};
const Segment SGI189738[] = {{&S64340[96], 9}, {NULL}};
const Segment SGI189739[] = {{&S64340[69], 6}, {NULL}};
const Segment SGI189740[] = {{&S75687[34], 15}, {NULL}};
const Segment SGI189741[] = {{&S75687[27], 6}, {NULL}};
const Segment SGI189742[] = {{&S75687[102], 9}, {NULL}};
const Segment SGI189743[] = {{&S75687[75], 6}, {NULL}};
const Segment SGI189744[] = {{&S64341[34], 10}, {NULL}};
const Segment SGI189745[] = {{&S64341[27], 6}, {NULL}};
const Segment SGI189746[] = {{&S64341[97], 9}, {NULL}};
const Segment SGI189747[] = {{&S64341[70], 6}, {NULL}};
const Segment SGI189748[] = {{&S75688[34], 8}, {NULL}};
const Segment SGI189749[] = {{&S75688[27], 6}, {NULL}};
const Segment SGI189750[] = {{&S75688[100], 9}, {NULL}};
const Segment SGI189751[] = {{&S75688[73], 6}, {NULL}};
const Segment SGI189752[] = {{&S75700[44], 13}, {NULL}};
const Segment SGI189753[] = {{&S75700[44], 29}, {NULL}};
const Segment SGI189754[] = {{&S75700[30], 6}, {NULL}};
const Segment SGI189755[] = {{&S75700[31], 3}, {NULL}};
const Segment SGI189756[] = {{&S75700[87], 6}, {NULL}};
const Segment SGI189757[] = {{&S75706[34], 5}, {NULL}};
const Segment SGI189758[] = {{&S75706[27], 6}, {NULL}};
const Segment SGI189759[] = {{&S75706[101], 9}, {NULL}};
const Segment SGI189760[] = {{&S75706[74], 6}, {NULL}};
const Segment SGI189761[] = {{&S75707[34], 7}, {NULL}};
const Segment SGI189762[] = {{&S75707[27], 6}, {NULL}};
const Segment SGI189763[] = {{&S75707[99], 9}, {NULL}};
const Segment SGI189764[] = {{&S75707[72], 6}, {NULL}};
const Segment SGI189765[] = {{&S64370[44], 11}, {NULL}};
const Segment SGI189766[] = {{&S64370[44], 28}, {NULL}};
const Segment SGI189767[] = {{&S64370[30], 6}, {NULL}};
const Segment SGI189768[] = {{&S64370[121], 3}, {NULL}};
const Segment SGI189769[] = {{&S64370[86], 6}, {NULL}};
const Segment SGI189770[] = {{&S75710[34], 7}, {NULL}};
const Segment SGI189771[] = {{&S75710[27], 6}, {NULL}};
const Segment SGI189772[] = {{&S75710[103], 9}, {NULL}};
const Segment SGI189773[] = {{&S75710[76], 6}, {NULL}};
const Segment SGI189774[] = {{&S64371[44], 11}, {NULL}};
const Segment SGI189775[] = {{&S64371[44], 27}, {NULL}};
const Segment SGI189776[] = {{&S64371[30], 6}, {NULL}};
const Segment SGI189777[] = {{&S64371[120], 3}, {NULL}};
const Segment SGI189778[] = {{&S64371[85], 6}, {NULL}};
const Segment SGI189779[] = {{&S64372[44], 11}, {NULL}};
const Segment SGI189780[] = {{&S64372[44], 28}, {NULL}};
const Segment SGI189781[] = {{&S64372[30], 6}, {NULL}};
const Segment SGI189782[] = {{&S64372[121], 3}, {NULL}};
const Segment SGI189783[] = {{&S64372[86], 6}, {NULL}};
const Segment SGI189784[] = {{&S64373[44], 7}, {NULL}};
const Segment SGI189785[] = {{&S64373[44], 24}, {NULL}};
const Segment SGI189786[] = {{&S64373[30], 6}, {NULL}};
const Segment SGI189787[] = {{&S64373[117], 3}, {NULL}};
const Segment SGI189788[] = {{&S64373[82], 6}, {NULL}};
const Segment SGI189789[] = {{&S64374[38], 7}, {NULL}};
const Segment SGI189790[] = {{&S64374[31], 6}, {NULL}};
const Segment SGI189791[] = {{&S64375[44], 11}, {NULL}};
const Segment SGI189792[] = {{&S64375[44], 28}, {NULL}};
const Segment SGI189793[] = {{&S64375[30], 6}, {NULL}};
const Segment SGI189794[] = {{&S64375[121], 3}, {NULL}};
const Segment SGI189795[] = {{&S64375[86], 6}, {NULL}};
const Segment SGI189796[] = {{&S75719[34], 14}, {NULL}};
const Segment SGI189797[] = {{&S75719[27], 6}, {NULL}};
const Segment SGI189798[] = {{&S75719[101], 9}, {NULL}};
const Segment SGI189799[] = {{&S75719[74], 6}, {NULL}};
const Segment SGI189800[] = {{&S64385[34], 7}, {NULL}};
const Segment SGI189801[] = {{&S64385[27], 6}, {NULL}};
const Segment SGI189802[] = {{&S64385[94], 9}, {NULL}};
const Segment SGI189803[] = {{&S64385[67], 6}, {NULL}};
const Segment SGI189804[] = {{&S64390[34], 11}, {NULL}};
const Segment SGI189805[] = {{&S64390[27], 6}, {NULL}};
const Segment SGI189806[] = {{&S64390[98], 9}, {NULL}};
const Segment SGI189807[] = {{&S64390[71], 6}, {NULL}};
const Segment SGI189808[] = {{&S63019[44], 10}, {NULL}};
const Segment SGI189809[] = {{&S63019[44], 22}, {NULL}};
const Segment SGI189810[] = {{&S63019[30], 6}, {NULL}};
const Segment SGI189811[] = {{&S63019[133], 3}, {NULL}};
const Segment SGI189812[] = {{&S63019[98], 6}, {NULL}};
const Segment SGI189813[] = {{&S63019[68], 8}, {NULL}};
const Segment SGI189814[] = {{&S63021[44], 10}, {NULL}};
const Segment SGI189815[] = {{&S63021[44], 22}, {NULL}};
const Segment SGI189816[] = {{&S63021[30], 6}, {NULL}};
const Segment SGI189817[] = {{&S63021[133], 3}, {NULL}};
const Segment SGI189818[] = {{&S63021[98], 6}, {NULL}};
const Segment SGI189819[] = {{&S63021[68], 8}, {NULL}};
const Segment SGI189820[] = {{&S75749[44], 13}, {NULL}};
const Segment SGI189821[] = {{&S75749[44], 25}, {NULL}};
const Segment SGI189822[] = {{&S75749[30], 6}, {NULL}};
const Segment SGI189823[] = {{&S75749[31], 3}, {NULL}};
const Segment SGI189824[] = {{&S75749[83], 6}, {NULL}};
const Segment SGI189825[] = {{&S68986[34], 6}, {NULL}};
const Segment SGI189826[] = {{&S68986[27], 6}, {NULL}};
const Segment SGI189827[] = {{&S68986[93], 9}, {NULL}};
const Segment SGI189828[] = {{&S68986[66], 6}, {NULL}};
const Segment SGI189829[] = {{&S68993[34], 12}, {NULL}};
const Segment SGI189830[] = {{&S68993[27], 6}, {NULL}};
const Segment SGI189831[] = {{&S68993[99], 9}, {NULL}};
const Segment SGI189832[] = {{&S68993[72], 6}, {NULL}};
const Segment SGI189833[] = {{&S68994[34], 18}, {NULL}};
const Segment SGI189834[] = {{&S68994[27], 6}, {NULL}};
const Segment SGI189835[] = {{&S64424[34], 7}, {NULL}};
const Segment SGI189836[] = {{&S64424[27], 6}, {NULL}};
const Segment SGI189837[] = {{&S68994[114], 9}, {NULL}};
const Segment SGI189838[] = {{&S68994[87], 6}, {NULL}};
const Segment SGI189839[] = {{&S64424[102], 9}, {NULL}};
const Segment SGI189840[] = {{&S64424[75], 6}, {NULL}};
const Segment SGI189841[] = {{&S64424[143], 8}, {NULL}};
const Segment SGI189842[] = {{&S64425[34], 10}, {NULL}};
const Segment SGI189843[] = {{&S64425[27], 6}, {NULL}};
const Segment SGI189844[] = {{&S64425[97], 9}, {NULL}};
const Segment SGI189845[] = {{&S64425[70], 6}, {NULL}};
const Segment SGI189846[] = {{&S64426[34], 10}, {NULL}};
const Segment SGI189847[] = {{&S64426[27], 6}, {NULL}};
const Segment SGI189848[] = {{&S64426[97], 9}, {NULL}};
const Segment SGI189849[] = {{&S64426[70], 6}, {NULL}};
const Segment SGI189850[] = {{&S54093[44], 10}, {NULL}};
const Segment SGI189851[] = {{&S54093[44], 21}, {NULL}};
const Segment SGI189852[] = {{&S54093[30], 6}, {NULL}};
const Segment SGI189853[] = {{&S54093[132], 3}, {NULL}};
const Segment SGI189854[] = {{&S54093[97], 6}, {NULL}};
const Segment SGI189855[] = {{&S54093[67], 8}, {NULL}};
const Segment SGI189856[] = {{&S64438[34], 10}, {NULL}};
const Segment SGI189857[] = {{&S64438[27], 6}, {NULL}};
const Segment SGI189858[] = {{&S54094[44], 10}, {NULL}};
const Segment SGI189859[] = {{&S54094[44], 22}, {NULL}};
const Segment SGI189860[] = {{&S54094[30], 6}, {NULL}};
const Segment SGI189861[] = {{&S64438[97], 9}, {NULL}};
const Segment SGI189862[] = {{&S64438[70], 6}, {NULL}};
const Segment SGI189863[] = {{&S54094[133], 3}, {NULL}};
const Segment SGI189864[] = {{&S54094[98], 6}, {NULL}};
const Segment SGI189865[] = {{&S54094[68], 8}, {NULL}};
const Segment SGI189866[] = {{&S64441[34], 16}, {NULL}};
const Segment SGI189867[] = {{&S64441[27], 6}, {NULL}};
const Segment SGI189868[] = {{&S64441[103], 9}, {NULL}};
const Segment SGI189869[] = {{&S64441[76], 6}, {NULL}};
const Segment SGI189870[] = {{&S64442[34], 16}, {NULL}};
const Segment SGI189871[] = {{&S64442[27], 6}, {NULL}};
const Segment SGI189872[] = {{&S75787[34], 7}, {NULL}};
const Segment SGI189873[] = {{&S64442[103], 9}, {NULL}};
const Segment SGI189874[] = {{&S64442[76], 6}, {NULL}};
const Segment SGI189875[] = {{&S75787[27], 6}, {NULL}};
const Segment SGI189876[] = {{&S75787[95], 9}, {NULL}};
const Segment SGI189877[] = {{&S75787[68], 6}, {NULL}};
const Segment SGI189878[] = {{&S75789[45], 10}, {NULL}};
const Segment SGI189879[] = {{&S75789[30], 6}, {NULL}};
const Segment SGI189880[] = {{&S75789[31], 3}, {NULL}};
const Segment SGI189881[] = {{&S75789[81], 5}, {NULL}};
const Segment SGI189882[] = {{&S75789[139], 13}, {NULL}};
const Segment SGI189883[] = {{&S64449[34], 9}, {NULL}};
const Segment SGI189884[] = {{&S64449[27], 6}, {NULL}};
const Segment SGI189885[] = {{&S64449[96], 9}, {NULL}};
const Segment SGI189886[] = {{&S64449[69], 6}, {NULL}};
const Segment SGI189887[] = {{&S54108[44], 9}, {NULL}};
const Segment SGI189888[] = {{&S54108[44], 20}, {NULL}};
const Segment SGI189889[] = {{&S54108[30], 6}, {NULL}};
const Segment SGI189890[] = {{&S54108[113], 3}, {NULL}};
const Segment SGI189891[] = {{&S54108[78], 6}, {NULL}};
const Segment SGI189892[] = {{&S64450[34], 18}, {NULL}};
const Segment SGI189893[] = {{&S54108[141], 3}, {NULL}};
const Segment SGI189894[] = {{&S64450[27], 6}, {NULL}};
const Segment SGI189895[] = {{&S64450[105], 9}, {NULL}};
const Segment SGI189896[] = {{&S64450[78], 6}, {NULL}};
const Segment SGI189897[] = {{&S75794[34], 23}, {NULL}};
const Segment SGI189898[] = {{&S75794[27], 6}, {NULL}};
const Segment SGI189899[] = {{&S75794[120], 9}, {NULL}};
const Segment SGI189900[] = {{&S75794[93], 6}, {NULL}};
const Segment SGI189901[] = {{&S64451[34], 18}, {NULL}};
const Segment SGI189902[] = {{&S64451[27], 6}, {NULL}};
const Segment SGI189903[] = {{&S64451[105], 9}, {NULL}};
const Segment SGI189904[] = {{&S64451[78], 6}, {NULL}};
const Segment SGI189905[] = {{&S75795[44], 11}, {NULL}};
const Segment SGI189906[] = {{&S75795[44], 23}, {NULL}};
const Segment SGI189907[] = {{&S75795[30], 6}, {NULL}};
const Segment SGI189908[] = {{&S75795[31], 3}, {NULL}};
const Segment SGI189909[] = {{&S75795[81], 6}, {NULL}};
const Segment SGI189910[] = {{&S64452[44], 10}, {NULL}};
const Segment SGI189911[] = {{&S64452[44], 22}, {NULL}};
const Segment SGI189912[] = {{&S64452[30], 6}, {NULL}};
const Segment SGI189913[] = {{&S64452[80], 6}, {NULL}};
const Segment SGI189914[] = {{&S64452[107], 13}, {NULL}};
const Segment SGI189915[] = {{&S75796[44], 11}, {NULL}};
const Segment SGI189916[] = {{&S75796[44], 23}, {NULL}};
const Segment SGI189917[] = {{&S75796[30], 6}, {NULL}};
const Segment SGI189918[] = {{&S75796[31], 3}, {NULL}};
const Segment SGI189919[] = {{&S75796[81], 6}, {NULL}};
const Segment SGI189920[] = {{&S64454[44], 10}, {NULL}};
const Segment SGI189921[] = {{&S64454[44], 22}, {NULL}};
const Segment SGI189922[] = {{&S64454[30], 6}, {NULL}};
const Segment SGI189923[] = {{&S64454[80], 6}, {NULL}};
const Segment SGI189924[] = {{&S64454[107], 13}, {NULL}};
const Segment SGI189925[] = {{&S64455[44], 10}, {NULL}};
const Segment SGI189926[] = {{&S64455[44], 22}, {NULL}};
const Segment SGI189927[] = {{&S64455[30], 6}, {NULL}};
const Segment SGI189928[] = {{&S64455[80], 6}, {NULL}};
const Segment SGI189929[] = {{&S64455[107], 13}, {NULL}};
const Segment SGI189930[] = {{&S64463[34], 14}, {NULL}};
const Segment SGI189931[] = {{&S64463[27], 6}, {NULL}};
const Segment SGI189932[] = {{&S64463[101], 9}, {NULL}};
const Segment SGI189933[] = {{&S64463[74], 6}, {NULL}};
const Segment SGI189934[] = {{&S75809[34], 15}, {NULL}};
const Segment SGI189935[] = {{&S75809[27], 6}, {NULL}};
const Segment SGI189936[] = {{&S75809[102], 9}, {NULL}};
const Segment SGI189937[] = {{&S75809[75], 6}, {NULL}};
const Segment SGI189938[] = {{&S75810[34], 11}, {NULL}};
const Segment SGI189939[] = {{&S75810[27], 6}, {NULL}};
const Segment SGI189940[] = {{&S75810[98], 9}, {NULL}};
const Segment SGI189941[] = {{&S75810[71], 6}, {NULL}};
const Segment SGI189942[] = {{&S64485[44], 10}, {NULL}};
const Segment SGI189943[] = {{&S64485[44], 22}, {NULL}};
const Segment SGI189944[] = {{&S64485[30], 6}, {NULL}};
const Segment SGI189945[] = {{&S64485[80], 6}, {NULL}};
const Segment SGI189946[] = {{&S64485[107], 13}, {NULL}};
const Segment SGI189947[] = {{&S64487[44], 10}, {NULL}};
const Segment SGI189948[] = {{&S64487[44], 22}, {NULL}};
const Segment SGI189949[] = {{&S64487[30], 6}, {NULL}};
const Segment SGI189950[] = {{&S64487[80], 6}, {NULL}};
const Segment SGI189951[] = {{&S64487[107], 13}, {NULL}};
const Segment SGI189952[] = {{&S64488[44], 10}, {NULL}};
const Segment SGI189953[] = {{&S64488[44], 22}, {NULL}};
const Segment SGI189954[] = {{&S64488[30], 6}, {NULL}};
const Segment SGI189955[] = {{&S64488[80], 6}, {NULL}};
const Segment SGI189956[] = {{&S64488[107], 13}, {NULL}};
const Segment SGI189957[] = {{&S64489[44], 11}, {NULL}};
const Segment SGI189958[] = {{&S64489[44], 23}, {NULL}};
const Segment SGI189959[] = {{&S64489[30], 6}, {NULL}};
const Segment SGI189960[] = {{&S64489[81], 6}, {NULL}};
const Segment SGI189961[] = {{&S64489[108], 13}, {NULL}};
const Segment SGI189962[] = {{&S75844[34], 7}, {NULL}};
const Segment SGI189963[] = {{&S75844[27], 6}, {NULL}};
const Segment SGI189964[] = {{&S75844[93], 9}, {NULL}};
const Segment SGI189965[] = {{&S75844[66], 6}, {NULL}};
const Segment SGI189966[] = {{&S63154[44], 14}, {NULL}};
const Segment SGI189967[] = {{&S63154[44], 26}, {NULL}};
const Segment SGI189968[] = {{&S63154[30], 6}, {NULL}};
const Segment SGI189969[] = {{&S63154[142], 3}, {NULL}};
const Segment SGI189970[] = {{&S63154[107], 6}, {NULL}};
const Segment SGI189971[] = {{&S75851[44], 9}, {NULL}};
const Segment SGI189972[] = {{&S75851[44], 21}, {NULL}};
const Segment SGI189973[] = {{&S75851[30], 6}, {NULL}};
const Segment SGI189974[] = {{&S75851[31], 3}, {NULL}};
const Segment SGI189975[] = {{&S75851[79], 6}, {NULL}};
const Segment SGI189976[] = {{&S64511[34], 12}, {NULL}};
const Segment SGI189977[] = {{&S64511[27], 6}, {NULL}};
const Segment SGI189978[] = {{&S64511[99], 9}, {NULL}};
const Segment SGI189979[] = {{&S64511[72], 6}, {NULL}};
const Segment SGI189980[] = {{&S64512[34], 12}, {NULL}};
const Segment SGI189981[] = {{&S64512[27], 6}, {NULL}};
const Segment SGI189982[] = {{&S64512[99], 9}, {NULL}};
const Segment SGI189983[] = {{&S64512[72], 6}, {NULL}};
const Segment SGI189984[] = {{&S64513[34], 7}, {NULL}};
const Segment SGI189985[] = {{&S64513[27], 6}, {NULL}};
const Segment SGI189986[] = {{&S64513[99], 9}, {NULL}};
const Segment SGI189987[] = {{&S64513[72], 6}, {NULL}};
const Segment SGI189988[] = {{&S64514[34], 7}, {NULL}};
const Segment SGI189989[] = {{&S64514[27], 6}, {NULL}};
const Segment SGI189990[] = {{&S64514[99], 9}, {NULL}};
const Segment SGI189991[] = {{&S64514[72], 6}, {NULL}};
const Segment SGI189992[] = {{&S64519[34], 10}, {NULL}};
const Segment SGI189993[] = {{&S64519[27], 6}, {NULL}};
const Segment SGI189994[] = {{&S64519[97], 9}, {NULL}};
const Segment SGI189995[] = {{&S64519[70], 6}, {NULL}};
const Segment SGI189996[] = {{&S75868[44], 7}, {NULL}};
const Segment SGI189997[] = {{&S75868[44], 19}, {NULL}};
const Segment SGI189998[] = {{&S75868[30], 6}, {NULL}};
const Segment SGI189999[] = {{&S75868[31], 3}, {NULL}};
const Segment SGI190000[] = {{&S75868[96], 6}, {NULL}};
const Segment SGI190001[] = {{&S64520[34], 7}, {NULL}};
const Segment SGI190002[] = {{&S75868[65], 8}, {NULL}};
const Segment SGI190003[] = {{&S64520[27], 6}, {NULL}};
const Segment SGI190004[] = {{&S64520[105], 9}, {NULL}};
const Segment SGI190005[] = {{&S64520[78], 6}, {NULL}};
const Segment SGI190006[] = {{&S75869[44], 10}, {NULL}};
const Segment SGI190007[] = {{&S75869[44], 22}, {NULL}};
const Segment SGI190008[] = {{&S75869[30], 6}, {NULL}};
const Segment SGI190009[] = {{&S75869[31], 3}, {NULL}};
const Segment SGI190010[] = {{&S75869[99], 6}, {NULL}};
const Segment SGI190011[] = {{&S75869[68], 8}, {NULL}};
const Segment SGI190012[] = {{&S75870[44], 10}, {NULL}};
const Segment SGI190013[] = {{&S75870[44], 22}, {NULL}};
const Segment SGI190014[] = {{&S75870[30], 6}, {NULL}};
const Segment SGI190015[] = {{&S64524[34], 7}, {NULL}};
const Segment SGI190016[] = {{&S75870[31], 3}, {NULL}};
const Segment SGI190017[] = {{&S64524[27], 6}, {NULL}};
const Segment SGI190018[] = {{&S75870[97], 6}, {NULL}};
const Segment SGI190019[] = {{&S75870[68], 8}, {NULL}};
const Segment SGI190020[] = {{&S64524[99], 9}, {NULL}};
const Segment SGI190021[] = {{&S64524[72], 6}, {NULL}};
const Segment SGI190022[] = {{&S64524[140], 8}, {NULL}};
const Segment SGI190023[] = {{&S75876[44], 15}, {NULL}};
const Segment SGI190024[] = {{&S75876[44], 27}, {NULL}};
const Segment SGI190025[] = {{&S75876[30], 6}, {NULL}};
const Segment SGI190026[] = {{&S75876[31], 3}, {NULL}};
const Segment SGI190027[] = {{&S75876[85], 6}, {NULL}};
const Segment SGI190028[] = {{&S75877[38], 15}, {NULL}};
const Segment SGI190029[] = {{&S75877[31], 6}, {NULL}};
const Segment SGI190030[] = {{&S64533[34], 10}, {NULL}};
const Segment SGI190031[] = {{&S64533[27], 6}, {NULL}};
const Segment SGI190032[] = {{&S64533[97], 9}, {NULL}};
const Segment SGI190033[] = {{&S64533[70], 6}, {NULL}};
const Segment SGI190034[] = {{&S75880[44], 11}, {NULL}};
const Segment SGI190035[] = {{&S75880[44], 23}, {NULL}};
const Segment SGI190036[] = {{&S75880[30], 6}, {NULL}};
const Segment SGI190037[] = {{&S75880[31], 3}, {NULL}};
const Segment SGI190038[] = {{&S75880[81], 6}, {NULL}};
const Segment SGI190039[] = {{&S75881[44], 11}, {NULL}};
const Segment SGI190040[] = {{&S75881[44], 23}, {NULL}};
const Segment SGI190041[] = {{&S75881[30], 6}, {NULL}};
const Segment SGI190042[] = {{&S75881[84], 6}, {NULL}};
const Segment SGI190043[] = {{&S64536[44], 11}, {NULL}};
const Segment SGI190044[] = {{&S64536[44], 23}, {NULL}};
const Segment SGI190045[] = {{&S64536[30], 6}, {NULL}};
const Segment SGI190046[] = {{&S64536[116], 3}, {NULL}};
const Segment SGI190047[] = {{&S64536[81], 6}, {NULL}};
const Segment SGI190048[] = {{&S75882[38], 11}, {NULL}};
const Segment SGI190049[] = {{&S75882[31], 6}, {NULL}};
const Segment SGI190050[] = {{&S64537[38], 11}, {NULL}};
const Segment SGI190051[] = {{&S64537[31], 6}, {NULL}};
const Segment SGI190052[] = {{&S75886[34], 12}, {NULL}};
const Segment SGI190053[] = {{&S75886[27], 6}, {NULL}};
const Segment SGI190054[] = {{&S75886[99], 9}, {NULL}};
const Segment SGI190055[] = {{&S75886[72], 6}, {NULL}};
const Segment SGI190056[] = {{&S75889[34], 7}, {NULL}};
const Segment SGI190057[] = {{&S75889[27], 6}, {NULL}};
const Segment SGI190058[] = {{&S75889[103], 9}, {NULL}};
const Segment SGI190059[] = {{&S75889[76], 6}, {NULL}};
const Segment SGI190060[] = {{&S75891[44], 14}, {NULL}};
const Segment SGI190061[] = {{&S75891[44], 26}, {NULL}};
const Segment SGI190062[] = {{&S75891[30], 6}, {NULL}};
const Segment SGI190063[] = {{&S75891[31], 3}, {NULL}};
const Segment SGI190064[] = {{&S75891[84], 6}, {NULL}};
const Segment SGI190065[] = {{&S75892[44], 7}, {NULL}};
const Segment SGI190066[] = {{&S75892[44], 19}, {NULL}};
const Segment SGI190067[] = {{&S75892[30], 6}, {NULL}};
const Segment SGI190068[] = {{&S75892[31], 3}, {NULL}};
const Segment SGI190069[] = {{&S75892[77], 6}, {NULL}};
const Segment SGI190070[] = {{&S75893[34], 11}, {NULL}};
const Segment SGI190071[] = {{&S75893[27], 6}, {NULL}};
const Segment SGI190072[] = {{&S75893[98], 9}, {NULL}};
const Segment SGI190073[] = {{&S75893[71], 6}, {NULL}};
const Segment SGI190074[] = {{&S75895[34], 7}, {NULL}};
const Segment SGI190075[] = {{&S75895[27], 6}, {NULL}};
const Segment SGI190076[] = {{&S75895[99], 9}, {NULL}};
const Segment SGI190077[] = {{&S75895[72], 6}, {NULL}};
const Segment SGI190078[] = {{&S75898[34], 11}, {NULL}};
const Segment SGI190079[] = {{&S75898[27], 6}, {NULL}};
const Segment SGI190080[] = {{&S75898[98], 9}, {NULL}};
const Segment SGI190081[] = {{&S75898[71], 6}, {NULL}};
const Segment SGI190082[] = {{&S64576[44], 10}, {NULL}};
const Segment SGI190083[] = {{&S64576[44], 22}, {NULL}};
const Segment SGI190084[] = {{&S64576[30], 6}, {NULL}};
const Segment SGI190085[] = {{&S64576[115], 3}, {NULL}};
const Segment SGI190086[] = {{&S64576[80], 6}, {NULL}};
const Segment SGI190087[] = {{&S64577[38], 10}, {NULL}};
const Segment SGI190088[] = {{&S64577[31], 6}, {NULL}};
const Segment SGI190089[] = {{&S69053[45], 10}, {NULL}};
const Segment SGI190090[] = {{&S69053[30], 6}, {NULL}};
const Segment SGI190091[] = {{&S64578[44], 11}, {NULL}};
const Segment SGI190092[] = {{&S64578[44], 23}, {NULL}};
const Segment SGI190093[] = {{&S64578[30], 6}, {NULL}};
const Segment SGI190094[] = {{&S69053[115], 3}, {NULL}};
const Segment SGI190095[] = {{&S64578[116], 3}, {NULL}};
const Segment SGI190096[] = {{&S69053[81], 5}, {NULL}};
const Segment SGI190097[] = {{&S64578[81], 6}, {NULL}};
const Segment SGI190098[] = {{&S69053[139], 13}, {NULL}};
const Segment SGI190099[] = {{&S64579[44], 11}, {NULL}};
const Segment SGI190100[] = {{&S64579[44], 23}, {NULL}};
const Segment SGI190101[] = {{&S64579[30], 6}, {NULL}};
const Segment SGI190102[] = {{&S64579[116], 3}, {NULL}};
const Segment SGI190103[] = {{&S64579[81], 6}, {NULL}};
const Segment SGI190104[] = {{&S69055[45], 10}, {NULL}};
const Segment SGI190105[] = {{&S69055[30], 6}, {NULL}};
const Segment SGI190106[] = {{&S69055[115], 3}, {NULL}};
const Segment SGI190107[] = {{&S64580[34], 9}, {NULL}};
const Segment SGI190108[] = {{&S69055[81], 5}, {NULL}};
const Segment SGI190109[] = {{&S64580[27], 6}, {NULL}};
const Segment SGI190110[] = {{&S69055[139], 13}, {NULL}};
const Segment SGI190111[] = {{&S64580[96], 9}, {NULL}};
const Segment SGI190112[] = {{&S64580[69], 6}, {NULL}};
const Segment SGI190113[] = {{&S64587[34], 7}, {NULL}};
const Segment SGI190114[] = {{&S64587[27], 6}, {NULL}};
const Segment SGI190115[] = {{&S64587[98], 9}, {NULL}};
const Segment SGI190116[] = {{&S64587[71], 6}, {NULL}};
const Segment SGI190117[] = {{&S69070[34], 10}, {NULL}};
const Segment SGI190118[] = {{&S69070[27], 6}, {NULL}};
const Segment SGI190119[] = {{&S69070[97], 9}, {NULL}};
const Segment SGI190120[] = {{&S69070[70], 6}, {NULL}};
const Segment SGI190121[] = {{&S75937[44], 9}, {NULL}};
const Segment SGI190122[] = {{&S75937[44], 21}, {NULL}};
const Segment SGI190123[] = {{&S75937[30], 6}, {NULL}};
const Segment SGI190124[] = {{&S75937[31], 3}, {NULL}};
const Segment SGI190125[] = {{&S75937[79], 6}, {NULL}};
const Segment SGI190126[] = {{&S69072[44], 13}, {NULL}};
const Segment SGI190127[] = {{&S69072[44], 25}, {NULL}};
const Segment SGI190128[] = {{&S69072[30], 6}, {NULL}};
const Segment SGI190129[] = {{&S69072[118], 3}, {NULL}};
const Segment SGI190130[] = {{&S69072[83], 6}, {NULL}};
const Segment SGI190131[] = {{&S69078[34], 25}, {NULL}};
const Segment SGI190132[] = {{&S69078[27], 6}, {NULL}};
const Segment SGI190133[] = {{&S69078[112], 9}, {NULL}};
const Segment SGI190134[] = {{&S69078[85], 6}, {NULL}};
const Segment SGI190135[] = {{&S63268[34], 8}, {NULL}};
const Segment SGI190136[] = {{&S63268[27], 6}, {NULL}};
const Segment SGI190137[] = {{&S63268[95], 9}, {NULL}};
const Segment SGI190138[] = {{&S63268[68], 6}, {NULL}};
const Segment SGI190139[] = {{&S75949[45], 10}, {NULL}};
const Segment SGI190140[] = {{&S75949[30], 6}, {NULL}};
const Segment SGI190141[] = {{&S75949[31], 3}, {NULL}};
const Segment SGI190142[] = {{&S75949[81], 5}, {NULL}};
const Segment SGI190143[] = {{&S75949[139], 13}, {NULL}};
const Segment SGI190144[] = {{&S75952[44], 14}, {NULL}};
const Segment SGI190145[] = {{&S75952[44], 26}, {NULL}};
const Segment SGI190146[] = {{&S75952[30], 6}, {NULL}};
const Segment SGI190147[] = {{&S75952[31], 3}, {NULL}};
const Segment SGI190148[] = {{&S75952[84], 6}, {NULL}};
const Segment SGI190149[] = {{&S64624[34], 9}, {NULL}};
const Segment SGI190150[] = {{&S64624[27], 6}, {NULL}};
const Segment SGI190151[] = {{&S64624[96], 9}, {NULL}};
const Segment SGI190152[] = {{&S64624[69], 6}, {NULL}};
const Segment SGI190153[] = {{&S64624[137], 8}, {NULL}};
const Segment SGI190154[] = {{&S64625[34], 11}, {NULL}};
const Segment SGI190155[] = {{&S64625[27], 6}, {NULL}};
const Segment SGI190156[] = {{&S64625[98], 9}, {NULL}};
const Segment SGI190157[] = {{&S64625[71], 6}, {NULL}};
const Segment SGI190158[] = {{&S64625[139], 8}, {NULL}};
const Segment SGI190159[] = {{&S64626[34], 8}, {NULL}};
const Segment SGI190160[] = {{&S64626[27], 6}, {NULL}};
const Segment SGI190161[] = {{&S64626[95], 9}, {NULL}};
const Segment SGI190162[] = {{&S64626[68], 6}, {NULL}};
const Segment SGI190163[] = {{&S64626[136], 8}, {NULL}};
const Segment SGI190164[] = {{&S64627[34], 7}, {NULL}};
const Segment SGI190165[] = {{&S64627[27], 6}, {NULL}};
const Segment SGI190166[] = {{&S64627[99], 9}, {NULL}};
const Segment SGI190167[] = {{&S64627[72], 6}, {NULL}};
const Segment SGI190168[] = {{&S75985[44], 7}, {NULL}};
const Segment SGI190169[] = {{&S75985[44], 37}, {NULL}};
const Segment SGI190170[] = {{&S75985[30], 6}, {NULL}};
const Segment SGI190171[] = {{&S75985[31], 3}, {NULL}};
const Segment SGI190172[] = {{&S75985[95], 6}, {NULL}};
const Segment SGI190173[] = {{&S75990[44], 11}, {NULL}};
const Segment SGI190174[] = {{&S75990[44], 29}, {NULL}};
const Segment SGI190175[] = {{&S75990[30], 6}, {NULL}};
const Segment SGI190176[] = {{&S75990[31], 3}, {NULL}};
const Segment SGI190177[] = {{&S75990[87], 6}, {NULL}};
const Segment SGI190178[] = {{&S63319[34], 6}, {NULL}};
const Segment SGI190179[] = {{&S63319[27], 6}, {NULL}};
const Segment SGI190180[] = {{&S63319[93], 9}, {NULL}};
const Segment SGI190181[] = {{&S63319[66], 6}, {NULL}};
const Segment SGI190182[] = {{&S75995[34], 10}, {NULL}};
const Segment SGI190183[] = {{&S75995[27], 6}, {NULL}};
const Segment SGI190184[] = {{&S75995[97], 9}, {NULL}};
const Segment SGI190185[] = {{&S75995[70], 6}, {NULL}};
const Segment SGI190186[] = {{&S75996[34], 11}, {NULL}};
const Segment SGI190187[] = {{&S75996[27], 6}, {NULL}};
const Segment SGI190188[] = {{&S75996[98], 9}, {NULL}};
const Segment SGI190189[] = {{&S75996[71], 6}, {NULL}};
const Segment SGI190190[] = {{&S76013[44], 37}, {NULL}};
const Segment SGI190191[] = {{&S76013[30], 6}, {NULL}};
const Segment SGI190192[] = {{&S76013[31], 3}, {NULL}};
const Segment SGI190193[] = {{&S76013[113], 6}, {NULL}};
const Segment SGI190194[] = {{&S76014[44], 37}, {NULL}};
const Segment SGI190195[] = {{&S76014[30], 6}, {NULL}};
const Segment SGI190196[] = {{&S69153[44], 11}, {NULL}};
const Segment SGI190197[] = {{&S69153[44], 23}, {NULL}};
const Segment SGI190198[] = {{&S76014[31], 3}, {NULL}};
const Segment SGI190199[] = {{&S69153[30], 6}, {NULL}};
const Segment SGI190200[] = {{&S76014[113], 6}, {NULL}};
const Segment SGI190201[] = {{&S69153[116], 3}, {NULL}};
const Segment SGI190202[] = {{&S69153[81], 6}, {NULL}};
const Segment SGI190203[] = {{&S76015[44], 37}, {NULL}};
const Segment SGI190204[] = {{&S76015[30], 6}, {NULL}};
const Segment SGI190205[] = {{&S69154[44], 13}, {NULL}};
const Segment SGI190206[] = {{&S69154[44], 39}, {NULL}};
const Segment SGI190207[] = {{&S69154[30], 6}, {NULL}};
const Segment SGI190208[] = {{&S76015[31], 3}, {NULL}};
const Segment SGI190209[] = {{&S76015[113], 6}, {NULL}};
const Segment SGI190210[] = {{&S69154[132], 3}, {NULL}};
const Segment SGI190211[] = {{&S69154[97], 6}, {NULL}};
const Segment SGI190212[] = {{&S76016[44], 37}, {NULL}};
const Segment SGI190213[] = {{&S69155[38], 13}, {NULL}};
const Segment SGI190214[] = {{&S76016[30], 6}, {NULL}};
const Segment SGI190215[] = {{&S69155[31], 6}, {NULL}};
const Segment SGI190216[] = {{&S76016[31], 3}, {NULL}};
const Segment SGI190217[] = {{&S76016[113], 6}, {NULL}};
const Segment SGI190218[] = {{&S76017[44], 37}, {NULL}};
const Segment SGI190219[] = {{&S76017[30], 6}, {NULL}};
const Segment SGI190220[] = {{&S76017[31], 3}, {NULL}};
const Segment SGI190221[] = {{&S76017[113], 6}, {NULL}};
const Segment SGI190222[] = {{&S76023[44], 14}, {NULL}};
const Segment SGI190223[] = {{&S76023[44], 26}, {NULL}};
const Segment SGI190224[] = {{&S76023[30], 6}, {NULL}};
const Segment SGI190225[] = {{&S76023[31], 3}, {NULL}};
const Segment SGI190226[] = {{&S76023[84], 6}, {NULL}};
const Segment SGI190227[] = {{&S69180[44], 10}, {NULL}};
const Segment SGI190228[] = {{&S69180[44], 22}, {NULL}};
const Segment SGI190229[] = {{&S69180[30], 6}, {NULL}};
const Segment SGI190230[] = {{&S69180[115], 3}, {NULL}};
const Segment SGI190231[] = {{&S69180[80], 6}, {NULL}};
const Segment SGI190232[] = {{&S69180[143], 3}, {NULL}};
const Segment SGI190233[] = {{&S76044[44], 9}, {NULL}};
const Segment SGI190234[] = {{&S76044[44], 21}, {NULL}};
const Segment SGI190235[] = {{&S76044[30], 6}, {NULL}};
const Segment SGI190236[] = {{&S76044[31], 3}, {NULL}};
const Segment SGI190237[] = {{&S76044[79], 6}, {NULL}};
const Segment SGI190238[] = {{&S76045[44], 12}, {NULL}};
const Segment SGI190239[] = {{&S76045[44], 24}, {NULL}};
const Segment SGI190240[] = {{&S76045[30], 6}, {NULL}};
const Segment SGI190241[] = {{&S76045[31], 3}, {NULL}};
const Segment SGI190242[] = {{&S76045[82], 6}, {NULL}};
const Segment SGI190243[] = {{&S76046[44], 11}, {NULL}};
const Segment SGI190244[] = {{&S76046[44], 23}, {NULL}};
const Segment SGI190245[] = {{&S76046[30], 6}, {NULL}};
const Segment SGI190246[] = {{&S76046[31], 3}, {NULL}};
const Segment SGI190247[] = {{&S76046[81], 6}, {NULL}};
const Segment SGI190248[] = {{&S76047[44], 14}, {NULL}};
const Segment SGI190249[] = {{&S76047[44], 26}, {NULL}};
const Segment SGI190250[] = {{&S76047[30], 6}, {NULL}};
const Segment SGI190251[] = {{&S76047[31], 3}, {NULL}};
const Segment SGI190252[] = {{&S76047[84], 6}, {NULL}};
const Segment SGI190253[] = {{&S76049[34], 15}, {NULL}};
const Segment SGI190254[] = {{&S76049[27], 6}, {NULL}};
const Segment SGI190255[] = {{&S76049[102], 9}, {NULL}};
const Segment SGI190256[] = {{&S76049[75], 6}, {NULL}};
const Segment SGI190257[] = {{&S76051[44], 15}, {NULL}};
const Segment SGI190258[] = {{&S76051[44], 27}, {NULL}};
const Segment SGI190259[] = {{&S76051[30], 6}, {NULL}};
const Segment SGI190260[] = {{&S76051[31], 3}, {NULL}};
const Segment SGI190261[] = {{&S76051[85], 6}, {NULL}};
const Segment SGI190262[] = {{&S76055[34], 14}, {NULL}};
const Segment SGI190263[] = {{&S76055[27], 6}, {NULL}};
const Segment SGI190264[] = {{&S76055[106], 9}, {NULL}};
const Segment SGI190265[] = {{&S76055[79], 6}, {NULL}};
const Segment SGI190266[] = {{&S69201[44], 7}, {NULL}};
const Segment SGI190267[] = {{&S69201[44], 19}, {NULL}};
const Segment SGI190268[] = {{&S69201[30], 6}, {NULL}};
const Segment SGI190269[] = {{&S69201[112], 3}, {NULL}};
const Segment SGI190270[] = {{&S69201[77], 6}, {NULL}};
const Segment SGI190271[] = {{&S76056[34], 10}, {NULL}};
const Segment SGI190272[] = {{&S76056[27], 6}, {NULL}};
const Segment SGI190273[] = {{&S69202[38], 7}, {NULL}};
const Segment SGI190274[] = {{&S69202[31], 6}, {NULL}};
const Segment SGI190275[] = {{&S76056[97], 9}, {NULL}};
const Segment SGI190276[] = {{&S76056[70], 6}, {NULL}};
const Segment SGI190277[] = {{&S76060[34], 9}, {NULL}};
const Segment SGI190278[] = {{&S76060[27], 6}, {NULL}};
const Segment SGI190279[] = {{&S76060[96], 9}, {NULL}};
const Segment SGI190280[] = {{&S76060[69], 6}, {NULL}};
const Segment SGI190281[] = {{&S76061[34], 9}, {NULL}};
const Segment SGI190282[] = {{&S76061[27], 6}, {NULL}};
const Segment SGI190283[] = {{&S69209[44], 9}, {NULL}};
const Segment SGI190284[] = {{&S69209[44], 30}, {NULL}};
const Segment SGI190285[] = {{&S76061[96], 9}, {NULL}};
const Segment SGI190286[] = {{&S69209[30], 6}, {NULL}};
const Segment SGI190287[] = {{&S76061[69], 6}, {NULL}};
const Segment SGI190288[] = {{&S69209[123], 3}, {NULL}};
const Segment SGI190289[] = {{&S69209[88], 6}, {NULL}};
const Segment SGI190290[] = {{&S76062[34], 9}, {NULL}};
const Segment SGI190291[] = {{&S76062[27], 6}, {NULL}};
const Segment SGI190292[] = {{&S76062[96], 9}, {NULL}};
const Segment SGI190293[] = {{&S76062[69], 6}, {NULL}};
const Segment SGI190294[] = {{&S76071[44], 11}, {NULL}};
const Segment SGI190295[] = {{&S76071[44], 23}, {NULL}};
const Segment SGI190296[] = {{&S76071[30], 6}, {NULL}};
const Segment SGI190297[] = {{&S76071[31], 3}, {NULL}};
const Segment SGI190298[] = {{&S76071[81], 6}, {NULL}};
const Segment SGI190299[] = {{&S76092[34], 15}, {NULL}};
const Segment SGI190300[] = {{&S76092[27], 6}, {NULL}};
const Segment SGI190301[] = {{&S76092[102], 9}, {NULL}};
const Segment SGI190302[] = {{&S76092[75], 6}, {NULL}};
const Segment SGI190303[] = {{&S76093[34], 15}, {NULL}};
const Segment SGI190304[] = {{&S76093[27], 6}, {NULL}};
const Segment SGI190305[] = {{&S76093[102], 9}, {NULL}};
const Segment SGI190306[] = {{&S76093[75], 6}, {NULL}};
const Segment SGI190307[] = {{&S76094[34], 11}, {NULL}};
const Segment SGI190308[] = {{&S76094[27], 6}, {NULL}};
const Segment SGI190309[] = {{&S76094[98], 9}, {NULL}};
const Segment SGI190310[] = {{&S76094[71], 6}, {NULL}};
const Segment SGI190311[] = {{&S69248[34], 15}, {NULL}};
const Segment SGI190312[] = {{&S76095[34], 14}, {NULL}};
const Segment SGI190313[] = {{&S69248[27], 6}, {NULL}};
const Segment SGI190314[] = {{&S76095[27], 6}, {NULL}};
const Segment SGI190315[] = {{&S69248[102], 9}, {NULL}};
const Segment SGI190316[] = {{&S76095[101], 9}, {NULL}};
const Segment SGI190317[] = {{&S69248[75], 6}, {NULL}};
const Segment SGI190318[] = {{&S76095[74], 6}, {NULL}};
const Segment SGI190319[] = {{&S69249[34], 15}, {NULL}};
const Segment SGI190320[] = {{&S76096[44], 10}, {NULL}};
const Segment SGI190321[] = {{&S69249[27], 6}, {NULL}};
const Segment SGI190322[] = {{&S76096[44], 22}, {NULL}};
const Segment SGI190323[] = {{&S76096[30], 6}, {NULL}};
const Segment SGI190324[] = {{&S69249[102], 9}, {NULL}};
const Segment SGI190325[] = {{&S69249[75], 6}, {NULL}};
const Segment SGI190326[] = {{&S76096[80], 6}, {NULL}};
const Segment SGI190327[] = {{&S76096[107], 13}, {NULL}};
const Segment SGI190328[] = {{&S76098[44], 11}, {NULL}};
const Segment SGI190329[] = {{&S76098[44], 23}, {NULL}};
const Segment SGI190330[] = {{&S76098[30], 6}, {NULL}};
const Segment SGI190331[] = {{&S76098[81], 6}, {NULL}};
const Segment SGI190332[] = {{&S76098[108], 13}, {NULL}};
const Segment SGI190333[] = {{&S69255[44], 4}, {NULL}};
const Segment SGI190334[] = {{&S69255[44], 16}, {NULL}};
const Segment SGI190335[] = {{&S69255[30], 6}, {NULL}};
const Segment SGI190336[] = {{&S69255[125], 3}, {NULL}};
const Segment SGI190337[] = {{&S69255[90], 6}, {NULL}};
const Segment SGI190338[] = {{&S69255[62], 8}, {NULL}};
const Segment SGI190339[] = {{&S64808[44], 10}, {NULL}};
const Segment SGI190340[] = {{&S64808[44], 22}, {NULL}};
const Segment SGI190341[] = {{&S64808[30], 6}, {NULL}};
const Segment SGI190342[] = {{&S64808[115], 3}, {NULL}};
const Segment SGI190343[] = {{&S64808[80], 6}, {NULL}};
const Segment SGI190344[] = {{&S64808[150], 3}, {NULL}};
const Segment SGI190345[] = {{&S69257[44], 4}, {NULL}};
const Segment SGI190346[] = {{&S69257[44], 16}, {NULL}};
const Segment SGI190347[] = {{&S69257[30], 6}, {NULL}};
const Segment SGI190348[] = {{&S64809[44], 10}, {NULL}};
const Segment SGI190349[] = {{&S69257[125], 3}, {NULL}};
const Segment SGI190350[] = {{&S64809[44], 22}, {NULL}};
const Segment SGI190351[] = {{&S64809[30], 6}, {NULL}};
const Segment SGI190352[] = {{&S69257[90], 6}, {NULL}};
const Segment SGI190353[] = {{&S69257[62], 8}, {NULL}};
const Segment SGI190354[] = {{&S64809[115], 3}, {NULL}};
const Segment SGI190355[] = {{&S64809[80], 6}, {NULL}};
const Segment SGI190356[] = {{&S64809[150], 3}, {NULL}};
const Segment SGI190357[] = {{&S69258[44], 13}, {NULL}};
const Segment SGI190358[] = {{&S69258[44], 25}, {NULL}};
const Segment SGI190359[] = {{&S69258[30], 6}, {NULL}};
const Segment SGI190360[] = {{&S69258[118], 3}, {NULL}};
const Segment SGI190361[] = {{&S69258[83], 6}, {NULL}};
const Segment SGI190362[] = {{&S64810[44], 11}, {NULL}};
const Segment SGI190363[] = {{&S64810[44], 23}, {NULL}};
const Segment SGI190364[] = {{&S64810[30], 6}, {NULL}};
const Segment SGI190365[] = {{&S64810[116], 3}, {NULL}};
const Segment SGI190366[] = {{&S64810[81], 6}, {NULL}};
const Segment SGI190367[] = {{&S64810[151], 3}, {NULL}};
const Segment SGI190368[] = {{&S64811[44], 18}, {NULL}};
const Segment SGI190369[] = {{&S64811[44], 30}, {NULL}};
const Segment SGI190370[] = {{&S64811[30], 6}, {NULL}};
const Segment SGI190371[] = {{&S64811[123], 3}, {NULL}};
const Segment SGI190372[] = {{&S64811[88], 6}, {NULL}};
const Segment SGI190373[] = {{&S64811[158], 3}, {NULL}};
const Segment SGI190374[] = {{&S64812[44], 10}, {NULL}};
const Segment SGI190375[] = {{&S64812[44], 22}, {NULL}};
const Segment SGI190376[] = {{&S64812[30], 6}, {NULL}};
const Segment SGI190377[] = {{&S64812[115], 3}, {NULL}};
const Segment SGI190378[] = {{&S64812[80], 6}, {NULL}};
const Segment SGI190379[] = {{&S64812[150], 3}, {NULL}};
const Segment SGI190380[] = {{&S64813[44], 11}, {NULL}};
const Segment SGI190381[] = {{&S64813[44], 23}, {NULL}};
const Segment SGI190382[] = {{&S64813[30], 6}, {NULL}};
const Segment SGI190383[] = {{&S64813[116], 3}, {NULL}};
const Segment SGI190384[] = {{&S64813[81], 6}, {NULL}};
const Segment SGI190385[] = {{&S64814[38], 11}, {NULL}};
const Segment SGI190386[] = {{&S64814[31], 6}, {NULL}};
const Segment SGI190387[] = {{&S64815[44], 10}, {NULL}};
const Segment SGI190388[] = {{&S64815[44], 22}, {NULL}};
const Segment SGI190389[] = {{&S64815[30], 6}, {NULL}};
const Segment SGI190390[] = {{&S64815[80], 6}, {NULL}};
const Segment SGI190391[] = {{&S64815[107], 13}, {NULL}};
const Segment SGI190392[] = {{&S64816[34], 10}, {NULL}};
const Segment SGI190393[] = {{&S64816[27], 6}, {NULL}};
const Segment SGI190394[] = {{&S64816[97], 9}, {NULL}};
const Segment SGI190395[] = {{&S64816[70], 6}, {NULL}};
const Segment SGI190396[] = {{&S67479[32], 20}, {NULL}};
const Segment SGI190397[] = {{&S67479[27], 4}, {NULL}};
const Segment SGI190398[] = {{&S67479[105], 9}, {NULL}};
const Segment SGI190399[] = {{&S67479[78], 6}, {NULL}};
const Segment SGI190400[] = {{&S76143[44], 11}, {NULL}};
const Segment SGI190401[] = {{&S76143[44], 23}, {NULL}};
const Segment SGI190402[] = {{&S76143[30], 6}, {NULL}};
const Segment SGI190403[] = {{&S76143[31], 3}, {NULL}};
const Segment SGI190404[] = {{&S76143[81], 6}, {NULL}};
const Segment SGI190405[] = {{&S64833[44], 10}, {NULL}};
const Segment SGI190406[] = {{&S64833[44], 22}, {NULL}};
const Segment SGI190407[] = {{&S64833[30], 6}, {NULL}};
const Segment SGI190408[] = {{&S64833[136], 3}, {NULL}};
const Segment SGI190409[] = {{&S64833[101], 6}, {NULL}};
const Segment SGI190410[] = {{&S64834[44], 10}, {NULL}};
const Segment SGI190411[] = {{&S76146[34], 18}, {NULL}};
const Segment SGI190412[] = {{&S64834[44], 22}, {NULL}};
const Segment SGI190413[] = {{&S76146[27], 6}, {NULL}};
const Segment SGI190414[] = {{&S64834[30], 6}, {NULL}};
const Segment SGI190415[] = {{&S64834[136], 3}, {NULL}};
const Segment SGI190416[] = {{&S76146[105], 9}, {NULL}};
const Segment SGI190417[] = {{&S76146[78], 6}, {NULL}};
const Segment SGI190418[] = {{&S64834[101], 6}, {NULL}};
const Segment SGI190419[] = {{&S76147[34], 19}, {NULL}};
const Segment SGI190420[] = {{&S76147[27], 6}, {NULL}};
const Segment SGI190421[] = {{&S76147[106], 9}, {NULL}};
const Segment SGI190422[] = {{&S76147[79], 6}, {NULL}};
const Segment SGI190423[] = {{&S64836[44], 10}, {NULL}};
const Segment SGI190424[] = {{&S64836[44], 22}, {NULL}};
const Segment SGI190425[] = {{&S64836[30], 6}, {NULL}};
const Segment SGI190426[] = {{&S64836[136], 3}, {NULL}};
const Segment SGI190427[] = {{&S64836[101], 6}, {NULL}};
const Segment SGI190428[] = {{&S76148[34], 16}, {NULL}};
const Segment SGI190429[] = {{&S76148[27], 6}, {NULL}};
const Segment SGI190430[] = {{&S76148[103], 9}, {NULL}};
const Segment SGI190431[] = {{&S76148[76], 6}, {NULL}};
const Segment SGI190432[] = {{&S76149[34], 16}, {NULL}};
const Segment SGI190433[] = {{&S76149[27], 6}, {NULL}};
const Segment SGI190434[] = {{&S76149[103], 9}, {NULL}};
const Segment SGI190435[] = {{&S76149[76], 6}, {NULL}};
const Segment SGI190436[] = {{&S76150[34], 19}, {NULL}};
const Segment SGI190437[] = {{&S76150[27], 6}, {NULL}};
const Segment SGI190438[] = {{&S76150[106], 9}, {NULL}};
const Segment SGI190439[] = {{&S76150[79], 6}, {NULL}};
const Segment SGI190440[] = {{&S76151[34], 10}, {NULL}};
const Segment SGI190441[] = {{&S76151[27], 6}, {NULL}};
const Segment SGI190442[] = {{&S76151[97], 9}, {NULL}};
const Segment SGI190443[] = {{&S76151[70], 6}, {NULL}};
const Segment SGI190444[] = {{&S76152[34], 10}, {NULL}};
const Segment SGI190445[] = {{&S76152[27], 6}, {NULL}};
const Segment SGI190446[] = {{&S76152[97], 9}, {NULL}};
const Segment SGI190447[] = {{&S76152[70], 6}, {NULL}};
const Segment SGI190448[] = {{&S76153[34], 11}, {NULL}};
const Segment SGI190449[] = {{&S76153[27], 6}, {NULL}};
const Segment SGI190450[] = {{&S76153[98], 9}, {NULL}};
const Segment SGI190451[] = {{&S76153[71], 6}, {NULL}};
const Segment SGI190452[] = {{&S76154[34], 11}, {NULL}};
const Segment SGI190453[] = {{&S76154[27], 6}, {NULL}};
const Segment SGI190454[] = {{&S76154[98], 9}, {NULL}};
const Segment SGI190455[] = {{&S76154[71], 6}, {NULL}};
const Segment SGI190456[] = {{&S76157[44], 8}, {NULL}};
const Segment SGI190457[] = {{&S76157[44], 20}, {NULL}};
const Segment SGI190458[] = {{&S76157[30], 6}, {NULL}};
const Segment SGI190459[] = {{&S76157[31], 3}, {NULL}};
const Segment SGI190460[] = {{&S76157[78], 6}, {NULL}};
const Segment SGI190461[] = {{&S76158[38], 8}, {NULL}};
const Segment SGI190462[] = {{&S76158[31], 6}, {NULL}};
const Segment SGI190463[] = {{&S76160[34], 5}, {NULL}};
const Segment SGI190464[] = {{&S76160[27], 6}, {NULL}};
const Segment SGI190465[] = {{&S76160[92], 9}, {NULL}};
const Segment SGI190466[] = {{&S76160[65], 6}, {NULL}};
const Segment SGI190467[] = {{&S76161[34], 5}, {NULL}};
const Segment SGI190468[] = {{&S76161[27], 6}, {NULL}};
const Segment SGI190469[] = {{&S76161[92], 9}, {NULL}};
const Segment SGI190470[] = {{&S76161[65], 6}, {NULL}};
const Segment SGI190471[] = {{&S69335[38], 10}, {NULL}};
const Segment SGI190472[] = {{&S69335[31], 6}, {NULL}};
const Segment SGI190473[] = {{&S54337[34], 10}, {NULL}};
const Segment SGI190474[] = {{&S54337[27], 6}, {NULL}};
const Segment SGI190475[] = {{&S54337[96], 9}, {NULL}};
const Segment SGI190476[] = {{&S54337[69], 6}, {NULL}};
const Segment SGI190477[] = {{&S69336[44], 10}, {NULL}};
const Segment SGI190478[] = {{&S69336[44], 22}, {NULL}};
const Segment SGI190479[] = {{&S69336[30], 6}, {NULL}};
const Segment SGI190480[] = {{&S69336[115], 3}, {NULL}};
const Segment SGI190481[] = {{&S69336[80], 6}, {NULL}};
const Segment SGI190482[] = {{&S69339[44], 12}, {NULL}};
const Segment SGI190483[] = {{&S69339[44], 24}, {NULL}};
const Segment SGI190484[] = {{&S69339[30], 6}, {NULL}};
const Segment SGI190485[] = {{&S69339[117], 3}, {NULL}};
const Segment SGI190486[] = {{&S69339[82], 6}, {NULL}};
const Segment SGI190487[] = {{&S76198[44], 16}, {NULL}};
const Segment SGI190488[] = {{&S76198[44], 28}, {NULL}};
const Segment SGI190489[] = {{&S76198[30], 6}, {NULL}};
const Segment SGI190490[] = {{&S76198[31], 3}, {NULL}};
const Segment SGI190491[] = {{&S76198[86], 6}, {NULL}};
const Segment SGI190492[] = {{&S64899[34], 9}, {NULL}};
const Segment SGI190493[] = {{&S64899[27], 6}, {NULL}};
const Segment SGI190494[] = {{&S64899[96], 9}, {NULL}};
const Segment SGI190495[] = {{&S64899[69], 6}, {NULL}};
const Segment SGI190496[] = {{&S64900[34], 10}, {NULL}};
const Segment SGI190497[] = {{&S64900[27], 6}, {NULL}};
const Segment SGI190498[] = {{&S64900[97], 9}, {NULL}};
const Segment SGI190499[] = {{&S64900[70], 6}, {NULL}};
const Segment SGI190500[] = {{&S64901[34], 10}, {NULL}};
const Segment SGI190501[] = {{&S64901[27], 6}, {NULL}};
const Segment SGI190502[] = {{&S64901[97], 9}, {NULL}};
const Segment SGI190503[] = {{&S64901[70], 6}, {NULL}};
const Segment SGI190504[] = {{&S76205[27], 6}, {NULL}};
const Segment SGI190505[] = {{&S76205[99], 9}, {NULL}};
const Segment SGI190506[] = {{&S76205[72], 6}, {NULL}};
const Segment SGI190507[] = {{&S64902[34], 10}, {NULL}};
const Segment SGI190508[] = {{&S64902[27], 6}, {NULL}};
const Segment SGI190509[] = {{&S64902[97], 9}, {NULL}};
const Segment SGI190510[] = {{&S64902[70], 6}, {NULL}};
const Segment SGI190511[] = {{&S76207[34], 11}, {NULL}};
const Segment SGI190512[] = {{&S76207[27], 6}, {NULL}};
const Segment SGI190513[] = {{&S76207[98], 9}, {NULL}};
const Segment SGI190514[] = {{&S76207[71], 6}, {NULL}};
const Segment SGI190515[] = {{&S76208[34], 10}, {NULL}};
const Segment SGI190516[] = {{&S76208[27], 6}, {NULL}};
const Segment SGI190517[] = {{&S76208[97], 9}, {NULL}};
const Segment SGI190518[] = {{&S76208[70], 6}, {NULL}};
const Segment SGI190519[] = {{&S76213[34], 11}, {NULL}};
const Segment SGI190520[] = {{&S76213[27], 6}, {NULL}};
const Segment SGI190521[] = {{&S69374[44], 7}, {NULL}};
const Segment SGI190522[] = {{&S69374[44], 19}, {NULL}};
const Segment SGI190523[] = {{&S76213[98], 9}, {NULL}};
const Segment SGI190524[] = {{&S69374[30], 6}, {NULL}};
const Segment SGI190525[] = {{&S76213[71], 6}, {NULL}};
const Segment SGI190526[] = {{&S69374[128], 3}, {NULL}};
const Segment SGI190527[] = {{&S69374[93], 6}, {NULL}};
const Segment SGI190528[] = {{&S69374[65], 8}, {NULL}};
const Segment SGI190529[] = {{&S69375[44], 7}, {NULL}};
const Segment SGI190530[] = {{&S69375[44], 19}, {NULL}};
const Segment SGI190531[] = {{&S69375[30], 6}, {NULL}};
const Segment SGI190532[] = {{&S76215[34], 11}, {NULL}};
const Segment SGI190533[] = {{&S76215[27], 6}, {NULL}};
const Segment SGI190534[] = {{&S69375[128], 3}, {NULL}};
const Segment SGI190535[] = {{&S69375[93], 6}, {NULL}};
const Segment SGI190536[] = {{&S76215[98], 9}, {NULL}};
const Segment SGI190537[] = {{&S69375[65], 8}, {NULL}};
const Segment SGI190538[] = {{&S76215[71], 6}, {NULL}};
const Segment SGI190539[] = {{&S76216[34], 11}, {NULL}};
const Segment SGI190540[] = {{&S76216[27], 6}, {NULL}};
const Segment SGI190541[] = {{&S76216[98], 9}, {NULL}};
const Segment SGI190542[] = {{&S76216[71], 6}, {NULL}};
const Segment SGI190543[] = {{&S69377[44], 7}, {NULL}};
const Segment SGI190544[] = {{&S69377[44], 19}, {NULL}};
const Segment SGI190545[] = {{&S69377[30], 6}, {NULL}};
const Segment SGI190546[] = {{&S67589[32], 9}, {NULL}};
const Segment SGI190547[] = {{&S67589[27], 4}, {NULL}};
const Segment SGI190548[] = {{&S69377[128], 3}, {NULL}};
const Segment SGI190549[] = {{&S69377[93], 6}, {NULL}};
const Segment SGI190550[] = {{&S69377[65], 8}, {NULL}};
const Segment SGI190551[] = {{&S67589[94], 9}, {NULL}};
const Segment SGI190552[] = {{&S67589[67], 6}, {NULL}};
const Segment SGI190553[] = {{&S76243[34], 22}, {NULL}};
const Segment SGI190554[] = {{&S76243[27], 6}, {NULL}};
const Segment SGI190555[] = {{&S69402[44], 13}, {NULL}};
const Segment SGI190556[] = {{&S69402[44], 25}, {NULL}};
const Segment SGI190557[] = {{&S69402[30], 6}, {NULL}};
const Segment SGI190558[] = {{&S76243[109], 9}, {NULL}};
const Segment SGI190559[] = {{&S76243[82], 6}, {NULL}};
const Segment SGI190560[] = {{&S69402[118], 3}, {NULL}};
const Segment SGI190561[] = {{&S69402[83], 6}, {NULL}};
const Segment SGI190562[] = {{&S76244[34], 25}, {NULL}};
const Segment SGI190563[] = {{&S76244[27], 6}, {NULL}};
const Segment SGI190564[] = {{&S76244[112], 9}, {NULL}};
const Segment SGI190565[] = {{&S76244[85], 6}, {NULL}};
const Segment SGI190566[] = {{&S76246[34], 20}, {NULL}};
const Segment SGI190567[] = {{&S76246[27], 6}, {NULL}};
const Segment SGI190568[] = {{&S76246[107], 9}, {NULL}};
const Segment SGI190569[] = {{&S76246[80], 6}, {NULL}};
const Segment SGI190570[] = {{&S64950[34], 10}, {NULL}};
const Segment SGI190571[] = {{&S64950[27], 6}, {NULL}};
const Segment SGI190572[] = {{&S64950[97], 9}, {NULL}};
const Segment SGI190573[] = {{&S64950[70], 6}, {NULL}};
const Segment SGI190574[] = {{&S64951[34], 11}, {NULL}};
const Segment SGI190575[] = {{&S64951[27], 6}, {NULL}};
const Segment SGI190576[] = {{&S64951[98], 9}, {NULL}};
const Segment SGI190577[] = {{&S64951[71], 6}, {NULL}};
const Segment SGI190578[] = {{&S64952[34], 10}, {NULL}};
const Segment SGI190579[] = {{&S64952[27], 6}, {NULL}};
const Segment SGI190580[] = {{&S76251[34], 11}, {NULL}};
const Segment SGI190581[] = {{&S64952[97], 9}, {NULL}};
const Segment SGI190582[] = {{&S64952[70], 6}, {NULL}};
const Segment SGI190583[] = {{&S76251[27], 6}, {NULL}};
const Segment SGI190584[] = {{&S76251[98], 9}, {NULL}};
const Segment SGI190585[] = {{&S76251[71], 6}, {NULL}};
const Segment SGI190586[] = {{&S64953[34], 10}, {NULL}};
const Segment SGI190587[] = {{&S64953[27], 6}, {NULL}};
const Segment SGI190588[] = {{&S64953[97], 9}, {NULL}};
const Segment SGI190589[] = {{&S64953[70], 6}, {NULL}};
const Segment SGI190590[] = {{&S64954[34], 9}, {NULL}};
const Segment SGI190591[] = {{&S64954[27], 6}, {NULL}};
const Segment SGI190592[] = {{&S64954[96], 9}, {NULL}};
const Segment SGI190593[] = {{&S64954[69], 6}, {NULL}};
const Segment SGI190594[] = {{&S76254[34], 17}, {NULL}};
const Segment SGI190595[] = {{&S76254[27], 6}, {NULL}};
const Segment SGI190596[] = {{&S76254[104], 9}, {NULL}};
const Segment SGI190597[] = {{&S76254[77], 6}, {NULL}};
const Segment SGI190598[] = {{&S64975[34], 10}, {NULL}};
const Segment SGI190599[] = {{&S64975[27], 6}, {NULL}};
const Segment SGI190600[] = {{&S64975[97], 9}, {NULL}};
const Segment SGI190601[] = {{&S64975[70], 6}, {NULL}};
const Segment SGI190602[] = {{&S76274[34], 14}, {NULL}};
const Segment SGI190603[] = {{&S76274[27], 6}, {NULL}};
const Segment SGI190604[] = {{&S76274[101], 9}, {NULL}};
const Segment SGI190605[] = {{&S76274[74], 6}, {NULL}};
const Segment SGI190606[] = {{&S76275[34], 18}, {NULL}};
const Segment SGI190607[] = {{&S76275[27], 6}, {NULL}};
const Segment SGI190608[] = {{&S76275[105], 9}, {NULL}};
const Segment SGI190609[] = {{&S54448[34], 10}, {NULL}};
const Segment SGI190610[] = {{&S76275[78], 6}, {NULL}};
const Segment SGI190611[] = {{&S54448[27], 6}, {NULL}};
const Segment SGI190612[] = {{&S54448[96], 9}, {NULL}};
const Segment SGI190613[] = {{&S54448[69], 6}, {NULL}};
const Segment SGI190614[] = {{&S76287[34], 16}, {NULL}};
const Segment SGI190615[] = {{&S76287[27], 6}, {NULL}};
const Segment SGI190616[] = {{&S76287[110], 9}, {NULL}};
const Segment SGI190617[] = {{&S76287[83], 6}, {NULL}};
const Segment SGI190618[] = {{&S76288[34], 15}, {NULL}};
const Segment SGI190619[] = {{&S76288[27], 6}, {NULL}};
const Segment SGI190620[] = {{&S76288[108], 9}, {NULL}};
const Segment SGI190621[] = {{&S76288[81], 6}, {NULL}};
const Segment SGI190622[] = {{&S76292[45], 5}, {NULL}};
const Segment SGI190623[] = {{&S76292[30], 6}, {NULL}};
const Segment SGI190624[] = {{&S76292[31], 3}, {NULL}};
const Segment SGI190625[] = {{&S76292[76], 5}, {NULL}};
const Segment SGI190626[] = {{&S63581[34], 9}, {NULL}};
const Segment SGI190627[] = {{&S76292[134], 13}, {NULL}};
const Segment SGI190628[] = {{&S63581[27], 6}, {NULL}};
const Segment SGI190629[] = {{&S63581[96], 9}, {NULL}};
const Segment SGI190630[] = {{&S63581[69], 6}, {NULL}};
const Segment SGI190631[] = {{&S63581[130], 8}, {NULL}};
const Segment SGI190632[] = {{&S76293[44], 5}, {NULL}};
const Segment SGI190633[] = {{&S76293[44], 17}, {NULL}};
const Segment SGI190634[] = {{&S76293[30], 6}, {NULL}};
const Segment SGI190635[] = {{&S76293[110], 3}, {NULL}};
const Segment SGI190636[] = {{&S76293[75], 6}, {NULL}};
const Segment SGI190637[] = {{&S76293[130], 8}, {NULL}};
const Segment SGI190638[] = {{&S76293[139], 13}, {NULL}};
const Segment SGI190639[] = {{&S76294[34], 11}, {NULL}};
const Segment SGI190640[] = {{&S76294[27], 6}, {NULL}};
const Segment SGI190641[] = {{&S76294[98], 9}, {NULL}};
const Segment SGI190642[] = {{&S76294[71], 6}, {NULL}};
const Segment SGI190643[] = {{&S76296[34], 11}, {NULL}};
const Segment SGI190644[] = {{&S76296[27], 6}, {NULL}};
const Segment SGI190645[] = {{&S76296[98], 9}, {NULL}};
const Segment SGI190646[] = {{&S76296[71], 6}, {NULL}};
const Segment SGI190647[] = {{&S76297[34], 11}, {NULL}};
const Segment SGI190648[] = {{&S76297[27], 6}, {NULL}};
const Segment SGI190649[] = {{&S76297[98], 9}, {NULL}};
const Segment SGI190650[] = {{&S76297[71], 6}, {NULL}};
const Segment SGI190651[] = {{&S67691[32], 20}, {NULL}};
const Segment SGI190652[] = {{&S67691[27], 4}, {NULL}};
const Segment SGI190653[] = {{&S67691[105], 9}, {NULL}};
const Segment SGI190654[] = {{&S67691[78], 6}, {NULL}};
const Segment SGI190655[] = {{&S76310[34], 8}, {NULL}};
const Segment SGI190656[] = {{&S76310[27], 6}, {NULL}};
const Segment SGI190657[] = {{&S76310[106], 9}, {NULL}};
const Segment SGI190658[] = {{&S76310[79], 6}, {NULL}};
const Segment SGI190659[] = {{&S76311[34], 8}, {NULL}};
const Segment SGI190660[] = {{&S76311[27], 6}, {NULL}};
const Segment SGI190661[] = {{&S76311[106], 9}, {NULL}};
const Segment SGI190662[] = {{&S76311[79], 6}, {NULL}};
const Segment SGI190663[] = {{&S65017[34], 9}, {NULL}};
const Segment SGI190664[] = {{&S65017[27], 6}, {NULL}};
const Segment SGI190665[] = {{&S65017[96], 9}, {NULL}};
const Segment SGI190666[] = {{&S65017[69], 6}, {NULL}};
const Segment SGI190667[] = {{&S65018[34], 9}, {NULL}};
const Segment SGI190668[] = {{&S65018[27], 6}, {NULL}};
const Segment SGI190669[] = {{&S65018[96], 9}, {NULL}};
const Segment SGI190670[] = {{&S65018[69], 6}, {NULL}};
const Segment SGI190671[] = {{&S76342[34], 7}, {NULL}};
const Segment SGI190672[] = {{&S76342[27], 6}, {NULL}};
const Segment SGI190673[] = {{&S76342[94], 9}, {NULL}};
const Segment SGI190674[] = {{&S76342[67], 6}, {NULL}};
const Segment SGI190675[] = {{&S76343[34], 8}, {NULL}};
const Segment SGI190676[] = {{&S76343[27], 6}, {NULL}};
const Segment SGI190677[] = {{&S76343[95], 9}, {NULL}};
const Segment SGI190678[] = {{&S76343[68], 6}, {NULL}};
const Segment SGI190679[] = {{&S65047[44], 11}, {NULL}};
const Segment SGI190680[] = {{&S65047[44], 23}, {NULL}};
const Segment SGI190681[] = {{&S65047[30], 6}, {NULL}};
const Segment SGI190682[] = {{&S65047[116], 3}, {NULL}};
const Segment SGI190683[] = {{&S65047[81], 6}, {NULL}};
const Segment SGI190684[] = {{&S76344[44], 13}, {NULL}};
const Segment SGI190685[] = {{&S76344[44], 25}, {NULL}};
const Segment SGI190686[] = {{&S76344[30], 6}, {NULL}};
const Segment SGI190687[] = {{&S76344[31], 3}, {NULL}};
const Segment SGI190688[] = {{&S76344[83], 6}, {NULL}};
const Segment SGI190689[] = {{&S65048[38], 11}, {NULL}};
const Segment SGI190690[] = {{&S65048[31], 6}, {NULL}};
const Segment SGI190691[] = {{&S76346[34], 9}, {NULL}};
const Segment SGI190692[] = {{&S76346[27], 6}, {NULL}};
const Segment SGI190693[] = {{&S76346[96], 9}, {NULL}};
const Segment SGI190694[] = {{&S76346[69], 6}, {NULL}};
const Segment SGI190695[] = {{&S76349[59], 7}, {NULL}};
const Segment SGI190696[] = {{&S76349[59], 19}, {NULL}};
const Segment SGI190697[] = {{&S76349[45], 6}, {NULL}};
const Segment SGI190698[] = {{&S76349[46], 3}, {NULL}};
const Segment SGI190699[] = {{&S76349[92], 6}, {NULL}};
const Segment SGI190700[] = {{&S76349[0], 14}, {NULL}};
const Segment SGI190701[] = {{&S76352[44], 11}, {NULL}};
const Segment SGI190702[] = {{&S76352[44], 23}, {NULL}};
const Segment SGI190703[] = {{&S76352[30], 6}, {NULL}};
const Segment SGI190704[] = {{&S76352[31], 3}, {NULL}};
const Segment SGI190705[] = {{&S76352[81], 6}, {NULL}};
const Segment SGI190706[] = {{&S76352[144], 3}, {NULL}};
const Segment SGI190707[] = {{&S76353[44], 10}, {NULL}};
const Segment SGI190708[] = {{&S76353[44], 22}, {NULL}};
const Segment SGI190709[] = {{&S76353[30], 6}, {NULL}};
const Segment SGI190710[] = {{&S76353[31], 3}, {NULL}};
const Segment SGI190711[] = {{&S76353[80], 6}, {NULL}};
const Segment SGI190712[] = {{&S76353[143], 3}, {NULL}};
const Segment SGI190713[] = {{&S54545[44], 10}, {NULL}};
const Segment SGI190714[] = {{&S54545[44], 21}, {NULL}};
const Segment SGI190715[] = {{&S54545[30], 6}, {NULL}};
const Segment SGI190716[] = {{&S76354[44], 11}, {NULL}};
const Segment SGI190717[] = {{&S76354[44], 23}, {NULL}};
const Segment SGI190718[] = {{&S76354[30], 6}, {NULL}};
const Segment SGI190719[] = {{&S54545[132], 3}, {NULL}};
const Segment SGI190720[] = {{&S54545[97], 6}, {NULL}};
const Segment SGI190721[] = {{&S54545[67], 8}, {NULL}};
const Segment SGI190722[] = {{&S76354[31], 3}, {NULL}};
const Segment SGI190723[] = {{&S76354[81], 6}, {NULL}};
const Segment SGI190724[] = {{&S76354[144], 3}, {NULL}};
const Segment SGI190725[] = {{&S76355[44], 11}, {NULL}};
const Segment SGI190726[] = {{&S76355[44], 23}, {NULL}};
const Segment SGI190727[] = {{&S76355[30], 6}, {NULL}};
const Segment SGI190728[] = {{&S76355[31], 3}, {NULL}};
const Segment SGI190729[] = {{&S76355[81], 6}, {NULL}};
const Segment SGI190730[] = {{&S76355[144], 3}, {NULL}};
const Segment SGI190731[] = {{&S54549[44], 10}, {NULL}};
const Segment SGI190732[] = {{&S54549[44], 22}, {NULL}};
const Segment SGI190733[] = {{&S54549[30], 6}, {NULL}};
const Segment SGI190734[] = {{&S54549[115], 3}, {NULL}};
const Segment SGI190735[] = {{&S54549[80], 6}, {NULL}};
const Segment SGI190736[] = {{&S54549[133], 8}, {NULL}};
const Segment SGI190737[] = {{&S76358[44], 7}, {NULL}};
const Segment SGI190738[] = {{&S76358[44], 19}, {NULL}};
const Segment SGI190739[] = {{&S76358[30], 6}, {NULL}};
const Segment SGI190740[] = {{&S76358[31], 3}, {NULL}};
const Segment SGI190741[] = {{&S76358[77], 6}, {NULL}};
const Segment SGI190742[] = {{&S65077[34], 10}, {NULL}};
const Segment SGI190743[] = {{&S65077[27], 6}, {NULL}};
const Segment SGI190744[] = {{&S65077[97], 9}, {NULL}};
const Segment SGI190745[] = {{&S65077[70], 6}, {NULL}};
const Segment SGI190746[] = {{&S65077[138], 8}, {NULL}};
const Segment SGI190747[] = {{&S65078[34], 11}, {NULL}};
const Segment SGI190748[] = {{&S65078[27], 6}, {NULL}};
const Segment SGI190749[] = {{&S65078[98], 9}, {NULL}};
const Segment SGI190750[] = {{&S65078[71], 6}, {NULL}};
const Segment SGI190751[] = {{&S65078[139], 8}, {NULL}};
const Segment SGI190752[] = {{&S65079[34], 11}, {NULL}};
const Segment SGI190753[] = {{&S65079[27], 6}, {NULL}};
const Segment SGI190754[] = {{&S65079[98], 9}, {NULL}};
const Segment SGI190755[] = {{&S65079[71], 6}, {NULL}};
const Segment SGI190756[] = {{&S65079[139], 8}, {NULL}};
const Segment SGI190757[] = {{&S76391[34], 12}, {NULL}};
const Segment SGI190758[] = {{&S76391[27], 6}, {NULL}};
const Segment SGI190759[] = {{&S76391[99], 9}, {NULL}};
const Segment SGI190760[] = {{&S76391[72], 6}, {NULL}};
const Segment SGI190761[] = {{&S76393[44], 11}, {NULL}};
const Segment SGI190762[] = {{&S76393[44], 23}, {NULL}};
const Segment SGI190763[] = {{&S76393[30], 6}, {NULL}};
const Segment SGI190764[] = {{&S76393[31], 3}, {NULL}};
const Segment SGI190765[] = {{&S76393[81], 6}, {NULL}};
const Segment SGI190766[] = {{&S76397[45], 7}, {NULL}};
const Segment SGI190767[] = {{&S76397[30], 6}, {NULL}};
const Segment SGI190768[] = {{&S76397[31], 3}, {NULL}};
const Segment SGI190769[] = {{&S76397[82], 5}, {NULL}};
const Segment SGI190770[] = {{&S76397[140], 13}, {NULL}};
const Segment SGI190771[] = {{&S76398[44], 10}, {NULL}};
const Segment SGI190772[] = {{&S76398[44], 22}, {NULL}};
const Segment SGI190773[] = {{&S76398[30], 6}, {NULL}};
const Segment SGI190774[] = {{&S76398[31], 3}, {NULL}};
const Segment SGI190775[] = {{&S76398[80], 6}, {NULL}};
const Segment SGI190776[] = {{&S76399[45], 7}, {NULL}};
const Segment SGI190777[] = {{&S76399[30], 6}, {NULL}};
const Segment SGI190778[] = {{&S76399[31], 3}, {NULL}};
const Segment SGI190779[] = {{&S76399[83], 5}, {NULL}};
const Segment SGI190780[] = {{&S76399[141], 13}, {NULL}};
const Segment SGI190781[] = {{&S76400[44], 7}, {NULL}};
const Segment SGI190782[] = {{&S76400[44], 24}, {NULL}};
const Segment SGI190783[] = {{&S76400[30], 6}, {NULL}};
const Segment SGI190784[] = {{&S76400[117], 3}, {NULL}};
const Segment SGI190785[] = {{&S76400[82], 6}, {NULL}};
const Segment SGI190786[] = {{&S76400[137], 8}, {NULL}};
const Segment SGI190787[] = {{&S76400[146], 13}, {NULL}};
const Segment SGI190788[] = {{&S76406[34], 15}, {NULL}};
const Segment SGI190789[] = {{&S76406[27], 6}, {NULL}};
const Segment SGI190790[] = {{&S76406[102], 9}, {NULL}};
const Segment SGI190791[] = {{&S76406[75], 6}, {NULL}};
const Segment SGI190792[] = {{&S76407[34], 11}, {NULL}};
const Segment SGI190793[] = {{&S76407[27], 6}, {NULL}};
const Segment SGI190794[] = {{&S76407[98], 9}, {NULL}};
const Segment SGI190795[] = {{&S76407[71], 6}, {NULL}};
const Segment SGI190796[] = {{&S76411[44], 15}, {NULL}};
const Segment SGI190797[] = {{&S76411[44], 27}, {NULL}};
const Segment SGI190798[] = {{&S76411[30], 6}, {NULL}};
const Segment SGI190799[] = {{&S76411[85], 6}, {NULL}};
const Segment SGI190800[] = {{&S76411[112], 13}, {NULL}};
const Segment SGI190801[] = {{&S76418[44], 7}, {NULL}};
const Segment SGI190802[] = {{&S76418[44], 19}, {NULL}};
const Segment SGI190803[] = {{&S76418[30], 6}, {NULL}};
const Segment SGI190804[] = {{&S76418[77], 6}, {NULL}};
const Segment SGI190805[] = {{&S76418[104], 13}, {NULL}};
const Segment SGI190806[] = {{&S76419[44], 8}, {NULL}};
const Segment SGI190807[] = {{&S76419[44], 20}, {NULL}};
const Segment SGI190808[] = {{&S76419[30], 6}, {NULL}};
const Segment SGI190809[] = {{&S76419[78], 6}, {NULL}};
const Segment SGI190810[] = {{&S76419[105], 13}, {NULL}};
const Segment SGI190811[] = {{&S69471[44], 10}, {NULL}};
const Segment SGI190812[] = {{&S76421[34], 10}, {NULL}};
const Segment SGI190813[] = {{&S69471[44], 22}, {NULL}};
const Segment SGI190814[] = {{&S69471[30], 6}, {NULL}};
const Segment SGI190815[] = {{&S76421[27], 6}, {NULL}};
const Segment SGI190816[] = {{&S69471[115], 3}, {NULL}};
const Segment SGI190817[] = {{&S76421[97], 9}, {NULL}};
const Segment SGI190818[] = {{&S76421[70], 6}, {NULL}};
const Segment SGI190819[] = {{&S69471[80], 6}, {NULL}};
const Segment SGI190820[] = {{&S69472[44], 11}, {NULL}};
const Segment SGI190821[] = {{&S69472[44], 23}, {NULL}};
const Segment SGI190822[] = {{&S69472[30], 6}, {NULL}};
const Segment SGI190823[] = {{&S69472[116], 3}, {NULL}};
const Segment SGI190824[] = {{&S69472[81], 6}, {NULL}};
const Segment SGI190825[] = {{&S54635[34], 8}, {NULL}};
const Segment SGI190826[] = {{&S54635[27], 6}, {NULL}};
const Segment SGI190827[] = {{&S54635[94], 9}, {NULL}};
const Segment SGI190828[] = {{&S54635[67], 6}, {NULL}};
const Segment SGI190829[] = {{&S69477[34], 16}, {NULL}};
const Segment SGI190830[] = {{&S69477[27], 6}, {NULL}};
const Segment SGI190831[] = {{&S69477[103], 9}, {NULL}};
const Segment SGI190832[] = {{&S69477[76], 6}, {NULL}};
const Segment SGI190833[] = {{&S69481[34], 10}, {NULL}};
const Segment SGI190834[] = {{&S69481[27], 6}, {NULL}};
const Segment SGI190835[] = {{&S69481[97], 9}, {NULL}};
const Segment SGI190836[] = {{&S69481[70], 6}, {NULL}};
const Segment SGI190837[] = {{&S69481[131], 8}, {NULL}};
const Segment SGI190838[] = {{&S76457[44], 24}, {NULL}};
const Segment SGI190839[] = {{&S76457[44], 36}, {NULL}};
const Segment SGI190840[] = {{&S76457[30], 6}, {NULL}};
const Segment SGI190841[] = {{&S76457[31], 3}, {NULL}};
const Segment SGI190842[] = {{&S76457[94], 6}, {NULL}};
const Segment SGI190843[] = {{&S76458[44], 24}, {NULL}};
const Segment SGI190844[] = {{&S76458[44], 36}, {NULL}};
const Segment SGI190845[] = {{&S76458[30], 6}, {NULL}};
const Segment SGI190846[] = {{&S76458[31], 3}, {NULL}};
const Segment SGI190847[] = {{&S76458[94], 6}, {NULL}};
const Segment SGI190848[] = {{&S76459[38], 24}, {NULL}};
const Segment SGI190849[] = {{&S76459[31], 6}, {NULL}};
const Segment SGI190850[] = {{&S76460[38], 24}, {NULL}};
const Segment SGI190851[] = {{&S76460[31], 6}, {NULL}};
const Segment SGI190852[] = {{&S69512[44], 17}, {NULL}};
const Segment SGI190853[] = {{&S69512[44], 29}, {NULL}};
const Segment SGI190854[] = {{&S69512[30], 6}, {NULL}};
const Segment SGI190855[] = {{&S69512[143], 3}, {NULL}};
const Segment SGI190856[] = {{&S69512[108], 6}, {NULL}};
const Segment SGI190857[] = {{&S69512[75], 8}, {NULL}};
const Segment SGI190858[] = {{&S76466[34], 10}, {NULL}};
const Segment SGI190859[] = {{&S76466[27], 6}, {NULL}};
const Segment SGI190860[] = {{&S76466[97], 9}, {NULL}};
const Segment SGI190861[] = {{&S76466[70], 6}, {NULL}};
const Segment SGI190862[] = {{&S69517[44], 9}, {NULL}};
const Segment SGI190863[] = {{&S69517[44], 21}, {NULL}};
const Segment SGI190864[] = {{&S69517[30], 6}, {NULL}};
const Segment SGI190865[] = {{&S69517[130], 3}, {NULL}};
const Segment SGI190866[] = {{&S69517[95], 6}, {NULL}};
const Segment SGI190867[] = {{&S69517[67], 8}, {NULL}};
const Segment SGI190868[] = {{&S76474[34], 15}, {NULL}};
const Segment SGI190869[] = {{&S76474[27], 6}, {NULL}};
const Segment SGI190870[] = {{&S76474[102], 9}, {NULL}};
const Segment SGI190871[] = {{&S76474[75], 6}, {NULL}};
const Segment SGI190872[] = {{&S63813[34], 13}, {NULL}};
const Segment SGI190873[] = {{&S63813[27], 6}, {NULL}};
const Segment SGI190874[] = {{&S63813[100], 9}, {NULL}};
const Segment SGI190875[] = {{&S63813[73], 6}, {NULL}};
const Segment SGI190876[] = {{&S76501[44], 8}, {NULL}};
const Segment SGI190877[] = {{&S76501[44], 20}, {NULL}};
const Segment SGI190878[] = {{&S76501[30], 6}, {NULL}};
const Segment SGI190879[] = {{&S76501[31], 3}, {NULL}};
const Segment SGI190880[] = {{&S76501[78], 6}, {NULL}};
const Segment SGI190881[] = {{&S76506[34], 13}, {NULL}};
const Segment SGI190882[] = {{&S76506[27], 6}, {NULL}};
const Segment SGI190883[] = {{&S76506[100], 9}, {NULL}};
const Segment SGI190884[] = {{&S76506[73], 6}, {NULL}};
const Segment SGI190885[] = {{&S76507[44], 8}, {NULL}};
const Segment SGI190886[] = {{&S76507[44], 20}, {NULL}};
const Segment SGI190887[] = {{&S76507[30], 6}, {NULL}};
const Segment SGI190888[] = {{&S76507[31], 3}, {NULL}};
const Segment SGI190889[] = {{&S76507[78], 6}, {NULL}};
const Segment SGI190890[] = {{&S76517[44], 20}, {NULL}};
const Segment SGI190891[] = {{&S76517[44], 32}, {NULL}};
const Segment SGI190892[] = {{&S76517[30], 6}, {NULL}};
const Segment SGI190893[] = {{&S76517[31], 3}, {NULL}};
const Segment SGI190894[] = {{&S76517[90], 6}, {NULL}};
const Segment SGI190895[] = {{&S76518[44], 20}, {NULL}};
const Segment SGI190896[] = {{&S76518[44], 32}, {NULL}};
const Segment SGI190897[] = {{&S76518[30], 6}, {NULL}};
const Segment SGI190898[] = {{&S76518[31], 3}, {NULL}};
const Segment SGI190899[] = {{&S76518[90], 6}, {NULL}};
const Segment SGI190900[] = {{&S65132[34], 10}, {NULL}};
const Segment SGI190901[] = {{&S65132[27], 6}, {NULL}};
const Segment SGI190902[] = {{&S65132[97], 9}, {NULL}};
const Segment SGI190903[] = {{&S65132[70], 6}, {NULL}};
const Segment SGI190904[] = {{&S65132[138], 8}, {NULL}};
const Segment SGI190905[] = {{&S65133[34], 10}, {NULL}};
const Segment SGI190906[] = {{&S76524[34], 9}, {NULL}};
const Segment SGI190907[] = {{&S65133[27], 6}, {NULL}};
const Segment SGI190908[] = {{&S76524[27], 6}, {NULL}};
const Segment SGI190909[] = {{&S65133[97], 9}, {NULL}};
const Segment SGI190910[] = {{&S76524[96], 9}, {NULL}};
const Segment SGI190911[] = {{&S65133[70], 6}, {NULL}};
const Segment SGI190912[] = {{&S76524[69], 6}, {NULL}};
const Segment SGI190913[] = {{&S65133[139], 8}, {NULL}};
const Segment SGI190914[] = {{&S54740[34], 10}, {NULL}};
const Segment SGI190915[] = {{&S54740[27], 6}, {NULL}};
const Segment SGI190916[] = {{&S54740[96], 9}, {NULL}};
const Segment SGI190917[] = {{&S54740[69], 6}, {NULL}};
const Segment SGI190918[] = {{&S63868[44], 7}, {NULL}};
const Segment SGI190919[] = {{&S63868[44], 19}, {NULL}};
const Segment SGI190920[] = {{&S63868[30], 6}, {NULL}};
const Segment SGI190921[] = {{&S63868[130], 3}, {NULL}};
const Segment SGI190922[] = {{&S63868[95], 6}, {NULL}};
const Segment SGI190923[] = {{&S63868[65], 8}, {NULL}};
const Segment SGI190924[] = {{&S76529[34], 7}, {NULL}};
const Segment SGI190925[] = {{&S76529[27], 6}, {NULL}};
const Segment SGI190926[] = {{&S76529[94], 9}, {NULL}};
const Segment SGI190927[] = {{&S76529[67], 6}, {NULL}};
const Segment SGI190928[] = {{&S65168[44], 10}, {NULL}};
const Segment SGI190929[] = {{&S65168[44], 22}, {NULL}};
const Segment SGI190930[] = {{&S65168[30], 6}, {NULL}};
const Segment SGI190931[] = {{&S65168[131], 3}, {NULL}};
const Segment SGI190932[] = {{&S65168[96], 6}, {NULL}};
const Segment SGI190933[] = {{&S65168[68], 8}, {NULL}};
const Segment SGI190934[] = {{&S76561[44], 7}, {NULL}};
const Segment SGI190935[] = {{&S76561[44], 18}, {NULL}};
const Segment SGI190936[] = {{&S76561[30], 6}, {NULL}};
const Segment SGI190937[] = {{&S76561[31], 3}, {NULL}};
const Segment SGI190938[] = {{&S76561[76], 6}, {NULL}};
const Segment SGI190939[] = {{&S76562[44], 7}, {NULL}};
const Segment SGI190940[] = {{&S76562[44], 18}, {NULL}};
const Segment SGI190941[] = {{&S76562[30], 6}, {NULL}};
const Segment SGI190942[] = {{&S76562[31], 3}, {NULL}};
const Segment SGI190943[] = {{&S76562[76], 6}, {NULL}};
const Segment SGI190944[] = {{&S76563[44], 7}, {NULL}};
const Segment SGI190945[] = {{&S76563[44], 18}, {NULL}};
const Segment SGI190946[] = {{&S76563[30], 6}, {NULL}};
const Segment SGI190947[] = {{&S76563[31], 3}, {NULL}};
const Segment SGI190948[] = {{&S76563[76], 6}, {NULL}};
const Segment SGI190949[] = {{&S76565[34], 7}, {NULL}};
const Segment SGI190950[] = {{&S76565[27], 6}, {NULL}};
const Segment SGI190951[] = {{&S76565[105], 9}, {NULL}};
const Segment SGI190952[] = {{&S76565[78], 6}, {NULL}};
const Segment SGI190953[] = {{&S76566[44], 13}, {NULL}};
const Segment SGI190954[] = {{&S76566[44], 25}, {NULL}};
const Segment SGI190955[] = {{&S76566[30], 6}, {NULL}};
const Segment SGI190956[] = {{&S76566[31], 3}, {NULL}};
const Segment SGI190957[] = {{&S76566[83], 6}, {NULL}};
const Segment SGI190958[] = {{&S76577[44], 11}, {NULL}};
const Segment SGI190959[] = {{&S76577[44], 23}, {NULL}};
const Segment SGI190960[] = {{&S76577[30], 6}, {NULL}};
const Segment SGI190961[] = {{&S76577[84], 6}, {NULL}};
const Segment SGI190962[] = {{&S76578[44], 11}, {NULL}};
const Segment SGI190963[] = {{&S76578[44], 23}, {NULL}};
const Segment SGI190964[] = {{&S76578[30], 6}, {NULL}};
const Segment SGI190965[] = {{&S76578[31], 3}, {NULL}};
const Segment SGI190966[] = {{&S76578[81], 6}, {NULL}};
const Segment SGI190967[] = {{&S76579[44], 13}, {NULL}};
const Segment SGI190968[] = {{&S76579[44], 25}, {NULL}};
const Segment SGI190969[] = {{&S76579[30], 6}, {NULL}};
const Segment SGI190970[] = {{&S65200[34], 10}, {NULL}};
const Segment SGI190971[] = {{&S65200[27], 6}, {NULL}};
const Segment SGI190972[] = {{&S76579[31], 3}, {NULL}};
const Segment SGI190973[] = {{&S76579[83], 6}, {NULL}};
const Segment SGI190974[] = {{&S65200[97], 9}, {NULL}};
const Segment SGI190975[] = {{&S65200[70], 6}, {NULL}};
const Segment SGI190976[] = {{&S76580[44], 11}, {NULL}};
const Segment SGI190977[] = {{&S76580[44], 37}, {NULL}};
const Segment SGI190978[] = {{&S76580[30], 6}, {NULL}};
const Segment SGI190979[] = {{&S76580[31], 3}, {NULL}};
const Segment SGI190980[] = {{&S76580[95], 6}, {NULL}};
const Segment SGI190981[] = {{&S76581[44], 11}, {NULL}};
const Segment SGI190982[] = {{&S76581[44], 37}, {NULL}};
const Segment SGI190983[] = {{&S76581[30], 6}, {NULL}};
const Segment SGI190984[] = {{&S76581[98], 6}, {NULL}};
const Segment SGI190985[] = {{&S76582[44], 11}, {NULL}};
const Segment SGI190986[] = {{&S76582[44], 37}, {NULL}};
const Segment SGI190987[] = {{&S76582[30], 6}, {NULL}};
const Segment SGI190988[] = {{&S76582[31], 3}, {NULL}};
const Segment SGI190989[] = {{&S76582[95], 6}, {NULL}};
const Segment SGI190990[] = {{&S69647[44], 13}, {NULL}};
const Segment SGI190991[] = {{&S69647[44], 25}, {NULL}};
const Segment SGI190992[] = {{&S69647[30], 6}, {NULL}};
const Segment SGI190993[] = {{&S69647[118], 3}, {NULL}};
const Segment SGI190994[] = {{&S69647[83], 6}, {NULL}};
const Segment SGI190995[] = {{&S76584[44], 6}, {NULL}};
const Segment SGI190996[] = {{&S76584[44], 18}, {NULL}};
const Segment SGI190997[] = {{&S76584[30], 6}, {NULL}};
const Segment SGI190998[] = {{&S76584[31], 3}, {NULL}};
const Segment SGI190999[] = {{&S76584[76], 6}, {NULL}};
const Segment SGI191000[] = {{&S69648[44], 14}, {NULL}};
const Segment SGI191001[] = {{&S69648[44], 26}, {NULL}};
const Segment SGI191002[] = {{&S69648[30], 6}, {NULL}};
const Segment SGI191003[] = {{&S69648[119], 3}, {NULL}};
const Segment SGI191004[] = {{&S69648[84], 6}, {NULL}};
const Segment SGI191005[] = {{&S65215[34], 28}, {NULL}};
const Segment SGI191006[] = {{&S65215[27], 6}, {NULL}};
const Segment SGI191007[] = {{&S65215[115], 9}, {NULL}};
const Segment SGI191008[] = {{&S65215[88], 6}, {NULL}};
const Segment SGI191009[] = {{&S69653[34], 11}, {NULL}};
const Segment SGI191010[] = {{&S69653[27], 6}, {NULL}};
const Segment SGI191011[] = {{&S69653[98], 9}, {NULL}};
const Segment SGI191012[] = {{&S69653[71], 6}, {NULL}};
const Segment SGI191013[] = {{&S65216[34], 12}, {NULL}};
const Segment SGI191014[] = {{&S65216[27], 6}, {NULL}};
const Segment SGI191015[] = {{&S65216[99], 9}, {NULL}};
const Segment SGI191016[] = {{&S65216[72], 6}, {NULL}};
const Segment SGI191017[] = {{&S76618[44], 11}, {NULL}};
const Segment SGI191018[] = {{&S76618[44], 23}, {NULL}};
const Segment SGI191019[] = {{&S76618[30], 6}, {NULL}};
const Segment SGI191020[] = {{&S76618[31], 3}, {NULL}};
const Segment SGI191021[] = {{&S76618[81], 6}, {NULL}};
const Segment SGI191022[] = {{&S76619[38], 11}, {NULL}};
const Segment SGI191023[] = {{&S76619[31], 6}, {NULL}};
const Segment SGI191024[] = {{&S76620[44], 14}, {NULL}};
const Segment SGI191025[] = {{&S76620[44], 26}, {NULL}};
const Segment SGI191026[] = {{&S76620[30], 6}, {NULL}};
const Segment SGI191027[] = {{&S76620[31], 3}, {NULL}};
const Segment SGI191028[] = {{&S76620[84], 6}, {NULL}};
const Segment SGI191029[] = {{&S69687[44], 14}, {NULL}};
const Segment SGI191030[] = {{&S69687[44], 26}, {NULL}};
const Segment SGI191031[] = {{&S69687[30], 6}, {NULL}};
const Segment SGI191032[] = {{&S69687[119], 3}, {NULL}};
const Segment SGI191033[] = {{&S69687[84], 6}, {NULL}};
const Segment SGI191034[] = {{&S76625[44], 5}, {NULL}};
const Segment SGI191035[] = {{&S76625[44], 17}, {NULL}};
const Segment SGI191036[] = {{&S76625[30], 6}, {NULL}};
const Segment SGI191037[] = {{&S76625[31], 3}, {NULL}};
const Segment SGI191038[] = {{&S76625[75], 6}, {NULL}};
const Segment SGI191039[] = {{&S76627[34], 6}, {NULL}};
const Segment SGI191040[] = {{&S76627[27], 6}, {NULL}};
const Segment SGI191041[] = {{&S76627[93], 9}, {NULL}};
const Segment SGI191042[] = {{&S76627[66], 6}, {NULL}};
const Segment SGI191043[] = {{&S73551[44], 14}, {NULL}};
const Segment SGI191044[] = {{&S73551[44], 30}, {NULL}};
const Segment SGI191045[] = {{&S73551[30], 6}, {NULL}};
const Segment SGI191046[] = {{&S73551[123], 3}, {NULL}};
const Segment SGI191047[] = {{&S73551[88], 6}, {NULL}};
const Segment SGI191048[] = {{&S68097[42], 15}, {NULL}};
const Segment SGI191049[] = {{&S68097[42], 27}, {NULL}};
const Segment SGI191050[] = {{&S68097[30], 4}, {NULL}};
const Segment SGI191051[] = {{&S69710[34], 25}, {NULL}};
const Segment SGI191052[] = {{&S69710[27], 6}, {NULL}};
const Segment SGI191053[] = {{&S68097[118], 3}, {NULL}};
const Segment SGI191054[] = {{&S68097[83], 6}, {NULL}};
const Segment SGI191055[] = {{&S69710[112], 9}, {NULL}};
const Segment SGI191056[] = {{&S69710[85], 6}, {NULL}};
const Segment SGI191057[] = {{&S68098[42], 12}, {NULL}};
const Segment SGI191058[] = {{&S68098[42], 24}, {NULL}};
const Segment SGI191059[] = {{&S68098[30], 4}, {NULL}};
const Segment SGI191060[] = {{&S68098[115], 3}, {NULL}};
const Segment SGI191061[] = {{&S68098[80], 6}, {NULL}};
const Segment SGI191062[] = {{&S76661[44], 15}, {NULL}};
const Segment SGI191063[] = {{&S76661[44], 26}, {NULL}};
const Segment SGI191064[] = {{&S76661[30], 6}, {NULL}};
const Segment SGI191065[] = {{&S76661[31], 3}, {NULL}};
const Segment SGI191066[] = {{&S76661[84], 6}, {NULL}};
const Segment SGI191067[] = {{&S69731[54], 10}, {NULL}};
const Segment SGI191068[] = {{&S69731[47], 6}, {NULL}};
const Segment SGI191069[] = {{&S69732[44], 10}, {NULL}};
const Segment SGI191070[] = {{&S69732[44], 22}, {NULL}};
const Segment SGI191071[] = {{&S69732[30], 6}, {NULL}};
const Segment SGI191072[] = {{&S69732[115], 3}, {NULL}};
const Segment SGI191073[] = {{&S69732[80], 6}, {NULL}};
const Segment SGI191074[] = {{&S69735[44], 10}, {NULL}};
const Segment SGI191075[] = {{&S69735[44], 22}, {NULL}};
const Segment SGI191076[] = {{&S69735[30], 6}, {NULL}};
const Segment SGI191077[] = {{&S69735[115], 3}, {NULL}};
const Segment SGI191078[] = {{&S69735[80], 6}, {NULL}};
const Segment SGI191079[] = {{&S54787[34], 16}, {NULL}};
const Segment SGI191080[] = {{&S54787[27], 6}, {NULL}};
const Segment SGI191081[] = {{&S54787[102], 9}, {NULL}};
const Segment SGI191082[] = {{&S54787[75], 6}, {NULL}};
const Segment SGI191083[] = {{&S54788[34], 10}, {NULL}};
const Segment SGI191084[] = {{&S54788[27], 6}, {NULL}};
const Segment SGI191085[] = {{&S54788[97], 9}, {NULL}};
const Segment SGI191086[] = {{&S54788[70], 6}, {NULL}};
const Segment SGI191087[] = {{&S54789[34], 10}, {NULL}};
const Segment SGI191088[] = {{&S54789[27], 6}, {NULL}};
const Segment SGI191089[] = {{&S54789[97], 9}, {NULL}};
const Segment SGI191090[] = {{&S54789[70], 6}, {NULL}};
const Segment SGI191091[] = {{&S54790[34], 18}, {NULL}};
const Segment SGI191092[] = {{&S54790[27], 6}, {NULL}};
const Segment SGI191093[] = {{&S54790[105], 9}, {NULL}};
const Segment SGI191094[] = {{&S54790[78], 6}, {NULL}};
const Segment SGI191095[] = {{&S54791[34], 10}, {NULL}};
const Segment SGI191096[] = {{&S54791[27], 6}, {NULL}};
const Segment SGI191097[] = {{&S54791[97], 9}, {NULL}};
const Segment SGI191098[] = {{&S54791[70], 6}, {NULL}};
const Segment SGI191099[] = {{&S54792[34], 10}, {NULL}};
const Segment SGI191100[] = {{&S54792[27], 6}, {NULL}};
const Segment SGI191101[] = {{&S54792[97], 9}, {NULL}};
const Segment SGI191102[] = {{&S54792[70], 6}, {NULL}};
const Segment SGI191103[] = {{&S69751[55], 10}, {NULL}};
const Segment SGI191104[] = {{&S69751[55], 22}, {NULL}};
const Segment SGI191105[] = {{&S69751[30], 17}, {NULL}};
const Segment SGI191106[] = {{&S69751[126], 3}, {NULL}};
const Segment SGI191107[] = {{&S69751[91], 6}, {NULL}};
const Segment SGI191108[] = {{&S54793[34], 10}, {NULL}};
const Segment SGI191109[] = {{&S54793[27], 6}, {NULL}};
const Segment SGI191110[] = {{&S54793[97], 9}, {NULL}};
const Segment SGI191111[] = {{&S54793[70], 6}, {NULL}};
const Segment SGI191112[] = {{&S76694[44], 8}, {NULL}};
const Segment SGI191113[] = {{&S76694[44], 18}, {NULL}};
const Segment SGI191114[] = {{&S76694[30], 6}, {NULL}};
const Segment SGI191115[] = {{&S76694[31], 3}, {NULL}};
const Segment SGI191116[] = {{&S76694[76], 6}, {NULL}};
const Segment SGI191117[] = {{&S65327[34], 9}, {NULL}};
const Segment SGI191118[] = {{&S65327[27], 6}, {NULL}};
const Segment SGI191119[] = {{&S65327[96], 9}, {NULL}};
const Segment SGI191120[] = {{&S65327[69], 6}, {NULL}};
const Segment SGI191121[] = {{&S76695[44], 8}, {NULL}};
const Segment SGI191122[] = {{&S76695[44], 20}, {NULL}};
const Segment SGI191123[] = {{&S76695[30], 6}, {NULL}};
const Segment SGI191124[] = {{&S76695[31], 3}, {NULL}};
const Segment SGI191125[] = {{&S76695[78], 6}, {NULL}};
const Segment SGI191126[] = {{&S65328[34], 10}, {NULL}};
const Segment SGI191127[] = {{&S65328[27], 6}, {NULL}};
const Segment SGI191128[] = {{&S65328[97], 9}, {NULL}};
const Segment SGI191129[] = {{&S65328[70], 6}, {NULL}};
const Segment SGI191130[] = {{&S76696[44], 8}, {NULL}};
const Segment SGI191131[] = {{&S76696[44], 20}, {NULL}};
const Segment SGI191132[] = {{&S76696[30], 6}, {NULL}};
const Segment SGI191133[] = {{&S76696[31], 3}, {NULL}};
const Segment SGI191134[] = {{&S76696[78], 6}, {NULL}};
const Segment SGI191135[] = {{&S65329[34], 10}, {NULL}};
const Segment SGI191136[] = {{&S65329[27], 6}, {NULL}};
const Segment SGI191137[] = {{&S65329[97], 9}, {NULL}};
const Segment SGI191138[] = {{&S65329[70], 6}, {NULL}};
const Segment SGI191139[] = {{&S65330[34], 11}, {NULL}};
const Segment SGI191140[] = {{&S65330[27], 6}, {NULL}};
const Segment SGI191141[] = {{&S65330[98], 9}, {NULL}};
const Segment SGI191142[] = {{&S65330[71], 6}, {NULL}};
const Segment SGI191143[] = {{&S65331[34], 11}, {NULL}};
const Segment SGI191144[] = {{&S65331[27], 6}, {NULL}};
const Segment SGI191145[] = {{&S65331[98], 9}, {NULL}};
const Segment SGI191146[] = {{&S65331[71], 6}, {NULL}};
const Segment SGI191147[] = {{&S65332[34], 11}, {NULL}};
const Segment SGI191148[] = {{&S65332[27], 6}, {NULL}};
const Segment SGI191149[] = {{&S65332[98], 9}, {NULL}};
const Segment SGI191150[] = {{&S65332[71], 6}, {NULL}};
const Segment SGI191151[] = {{&S69772[44], 7}, {NULL}};
const Segment SGI191152[] = {{&S69772[44], 19}, {NULL}};
const Segment SGI191153[] = {{&S69772[30], 6}, {NULL}};
const Segment SGI191154[] = {{&S69772[112], 3}, {NULL}};
const Segment SGI191155[] = {{&S69772[77], 6}, {NULL}};
const Segment SGI191156[] = {{&S69773[44], 7}, {NULL}};
const Segment SGI191157[] = {{&S69773[44], 19}, {NULL}};
const Segment SGI191158[] = {{&S69773[30], 6}, {NULL}};
const Segment SGI191159[] = {{&S69773[112], 3}, {NULL}};
const Segment SGI191160[] = {{&S69773[77], 6}, {NULL}};
const Segment SGI191161[] = {{&S69774[44], 8}, {NULL}};
const Segment SGI191162[] = {{&S69774[44], 20}, {NULL}};
const Segment SGI191163[] = {{&S69774[30], 6}, {NULL}};
const Segment SGI191164[] = {{&S69774[113], 3}, {NULL}};
const Segment SGI191165[] = {{&S69774[78], 6}, {NULL}};
const Segment SGI191166[] = {{&S69775[44], 12}, {NULL}};
const Segment SGI191167[] = {{&S69775[44], 24}, {NULL}};
const Segment SGI191168[] = {{&S69775[30], 6}, {NULL}};
const Segment SGI191169[] = {{&S69775[117], 3}, {NULL}};
const Segment SGI191170[] = {{&S69775[82], 6}, {NULL}};
const Segment SGI191171[] = {{&S69776[44], 15}, {NULL}};
const Segment SGI191172[] = {{&S69776[44], 27}, {NULL}};
const Segment SGI191173[] = {{&S76714[44], 10}, {NULL}};
const Segment SGI191174[] = {{&S69776[30], 6}, {NULL}};
const Segment SGI191175[] = {{&S76714[44], 22}, {NULL}};
const Segment SGI191176[] = {{&S76714[30], 6}, {NULL}};
const Segment SGI191177[] = {{&S69776[120], 3}, {NULL}};
const Segment SGI191178[] = {{&S76714[31], 3}, {NULL}};
const Segment SGI191179[] = {{&S69776[85], 6}, {NULL}};
const Segment SGI191180[] = {{&S76714[80], 6}, {NULL}};
const Segment SGI191181[] = {{&S76715[44], 10}, {NULL}};
const Segment SGI191182[] = {{&S76715[44], 22}, {NULL}};
const Segment SGI191183[] = {{&S76715[30], 6}, {NULL}};
const Segment SGI191184[] = {{&S76715[83], 6}, {NULL}};
const Segment SGI191185[] = {{&S76716[38], 10}, {NULL}};
const Segment SGI191186[] = {{&S76716[31], 6}, {NULL}};
const Segment SGI191187[] = {{&S65349[34], 18}, {NULL}};
const Segment SGI191188[] = {{&S65349[27], 6}, {NULL}};
const Segment SGI191189[] = {{&S65349[105], 9}, {NULL}};
const Segment SGI191190[] = {{&S69780[34], 25}, {NULL}};
const Segment SGI191191[] = {{&S65349[78], 6}, {NULL}};
const Segment SGI191192[] = {{&S69780[27], 6}, {NULL}};
const Segment SGI191193[] = {{&S69780[112], 9}, {NULL}};
const Segment SGI191194[] = {{&S69780[85], 6}, {NULL}};
const Segment SGI191195[] = {{&S65350[34], 18}, {NULL}};
const Segment SGI191196[] = {{&S65350[27], 6}, {NULL}};
const Segment SGI191197[] = {{&S76719[34], 18}, {NULL}};
const Segment SGI191198[] = {{&S65350[105], 9}, {NULL}};
const Segment SGI191199[] = {{&S65350[78], 6}, {NULL}};
const Segment SGI191200[] = {{&S76719[27], 6}, {NULL}};
const Segment SGI191201[] = {{&S69781[34], 32}, {NULL}};
const Segment SGI191202[] = {{&S69781[27], 6}, {NULL}};
const Segment SGI191203[] = {{&S76719[105], 9}, {NULL}};
const Segment SGI191204[] = {{&S76719[78], 6}, {NULL}};
const Segment SGI191205[] = {{&S69781[119], 9}, {NULL}};
const Segment SGI191206[] = {{&S69781[92], 6}, {NULL}};
const Segment SGI191207[] = {{&S65351[34], 10}, {NULL}};
const Segment SGI191208[] = {{&S65351[27], 6}, {NULL}};
const Segment SGI191209[] = {{&S76720[34], 11}, {NULL}};
const Segment SGI191210[] = {{&S65351[97], 9}, {NULL}};
const Segment SGI191211[] = {{&S65351[70], 6}, {NULL}};
const Segment SGI191212[] = {{&S76720[27], 6}, {NULL}};
const Segment SGI191213[] = {{&S76720[98], 9}, {NULL}};
const Segment SGI191214[] = {{&S76720[71], 6}, {NULL}};
const Segment SGI191215[] = {{&S65352[34], 10}, {NULL}};
const Segment SGI191216[] = {{&S65352[27], 6}, {NULL}};
const Segment SGI191217[] = {{&S65352[97], 9}, {NULL}};
const Segment SGI191218[] = {{&S76721[34], 11}, {NULL}};
const Segment SGI191219[] = {{&S65352[70], 6}, {NULL}};
const Segment SGI191220[] = {{&S76721[27], 6}, {NULL}};
const Segment SGI191221[] = {{&S76721[98], 9}, {NULL}};
const Segment SGI191222[] = {{&S76721[71], 6}, {NULL}};
const Segment SGI191223[] = {{&S65353[34], 12}, {NULL}};
const Segment SGI191224[] = {{&S65353[27], 6}, {NULL}};
const Segment SGI191225[] = {{&S65353[99], 9}, {NULL}};
const Segment SGI191226[] = {{&S65353[72], 6}, {NULL}};
const Segment SGI191227[] = {{&S76722[34], 10}, {NULL}};
const Segment SGI191228[] = {{&S76722[27], 6}, {NULL}};
const Segment SGI191229[] = {{&S76722[97], 9}, {NULL}};
const Segment SGI191230[] = {{&S76722[70], 6}, {NULL}};
const Segment SGI191231[] = {{&S76726[44], 10}, {NULL}};
const Segment SGI191232[] = {{&S76726[44], 22}, {NULL}};
const Segment SGI191233[] = {{&S76726[30], 6}, {NULL}};
const Segment SGI191234[] = {{&S76726[31], 3}, {NULL}};
const Segment SGI191235[] = {{&S76726[80], 6}, {NULL}};
const Segment SGI191236[] = {{&S76726[140], 7}, {NULL}};
const Segment SGI191237[] = {{&S76727[44], 14}, {NULL}};
const Segment SGI191238[] = {{&S76727[44], 26}, {NULL}};
const Segment SGI191239[] = {{&S76727[30], 6}, {NULL}};
const Segment SGI191240[] = {{&S76727[31], 3}, {NULL}};
const Segment SGI191241[] = {{&S76727[84], 6}, {NULL}};
const Segment SGI191242[] = {{&S76727[144], 7}, {NULL}};
const Segment SGI191243[] = {{&S76728[44], 13}, {NULL}};
const Segment SGI191244[] = {{&S76728[44], 25}, {NULL}};
const Segment SGI191245[] = {{&S76728[30], 6}, {NULL}};
const Segment SGI191246[] = {{&S76728[31], 3}, {NULL}};
const Segment SGI191247[] = {{&S76728[83], 6}, {NULL}};
const Segment SGI191248[] = {{&S76728[143], 7}, {NULL}};
const Segment SGI191249[] = {{&S76730[34], 17}, {NULL}};
const Segment SGI191250[] = {{&S76730[27], 6}, {NULL}};
const Segment SGI191251[] = {{&S76730[104], 9}, {NULL}};
const Segment SGI191252[] = {{&S76730[77], 6}, {NULL}};
const Segment SGI191253[] = {{&S65370[34], 10}, {NULL}};
const Segment SGI191254[] = {{&S65370[27], 6}, {NULL}};
const Segment SGI191255[] = {{&S65370[97], 9}, {NULL}};
const Segment SGI191256[] = {{&S65370[70], 6}, {NULL}};
const Segment SGI191257[] = {{&S65371[34], 10}, {NULL}};
const Segment SGI191258[] = {{&S65371[27], 6}, {NULL}};
const Segment SGI191259[] = {{&S65371[97], 9}, {NULL}};
const Segment SGI191260[] = {{&S65371[70], 6}, {NULL}};
const Segment SGI191261[] = {{&S65372[34], 10}, {NULL}};
const Segment SGI191262[] = {{&S65372[27], 6}, {NULL}};
const Segment SGI191263[] = {{&S65372[97], 9}, {NULL}};
const Segment SGI191264[] = {{&S65372[70], 6}, {NULL}};
const Segment SGI191265[] = {{&S65373[34], 14}, {NULL}};
const Segment SGI191266[] = {{&S76740[44], 10}, {NULL}};
const Segment SGI191267[] = {{&S65373[27], 6}, {NULL}};
const Segment SGI191268[] = {{&S76740[44], 22}, {NULL}};
const Segment SGI191269[] = {{&S76740[30], 6}, {NULL}};
const Segment SGI191270[] = {{&S65373[101], 9}, {NULL}};
const Segment SGI191271[] = {{&S76740[31], 3}, {NULL}};
const Segment SGI191272[] = {{&S65373[74], 6}, {NULL}};
const Segment SGI191273[] = {{&S76740[80], 6}, {NULL}};
const Segment SGI191274[] = {{&S65374[34], 6}, {NULL}};
const Segment SGI191275[] = {{&S65374[27], 6}, {NULL}};
const Segment SGI191276[] = {{&S65374[93], 9}, {NULL}};
const Segment SGI191277[] = {{&S65374[66], 6}, {NULL}};
const Segment SGI191278[] = {{&S76743[44], 14}, {NULL}};
const Segment SGI191279[] = {{&S76743[44], 26}, {NULL}};
const Segment SGI191280[] = {{&S76743[30], 6}, {NULL}};
const Segment SGI191281[] = {{&S76743[31], 3}, {NULL}};
const Segment SGI191282[] = {{&S76743[84], 6}, {NULL}};
const Segment SGI191283[] = {{&S65379[44], 9}, {NULL}};
const Segment SGI191284[] = {{&S65379[44], 21}, {NULL}};
const Segment SGI191285[] = {{&S76745[44], 7}, {NULL}};
const Segment SGI191286[] = {{&S65379[30], 6}, {NULL}};
const Segment SGI191287[] = {{&S76745[44], 19}, {NULL}};
const Segment SGI191288[] = {{&S76745[30], 6}, {NULL}};
const Segment SGI191289[] = {{&S65379[114], 3}, {NULL}};
const Segment SGI191290[] = {{&S76745[31], 3}, {NULL}};
const Segment SGI191291[] = {{&S65379[79], 6}, {NULL}};
const Segment SGI191292[] = {{&S76745[77], 6}, {NULL}};
const Segment SGI191293[] = {{&S65380[38], 9}, {NULL}};
const Segment SGI191294[] = {{&S65380[31], 6}, {NULL}};
const Segment SGI191295[] = {{&S76746[44], 11}, {NULL}};
const Segment SGI191296[] = {{&S76746[44], 23}, {NULL}};
const Segment SGI191297[] = {{&S76746[30], 6}, {NULL}};
const Segment SGI191298[] = {{&S69818[44], 7}, {NULL}};
const Segment SGI191299[] = {{&S76746[31], 3}, {NULL}};
const Segment SGI191300[] = {{&S69818[44], 19}, {NULL}};
const Segment SGI191301[] = {{&S69818[30], 6}, {NULL}};
const Segment SGI191302[] = {{&S76746[81], 6}, {NULL}};
const Segment SGI191303[] = {{&S69818[112], 3}, {NULL}};
const Segment SGI191304[] = {{&S69818[77], 6}, {NULL}};
const Segment SGI191305[] = {{&S76747[44], 14}, {NULL}};
const Segment SGI191306[] = {{&S76747[44], 26}, {NULL}};
const Segment SGI191307[] = {{&S76747[30], 6}, {NULL}};
const Segment SGI191308[] = {{&S76747[31], 3}, {NULL}};
const Segment SGI191309[] = {{&S76747[84], 6}, {NULL}};
const Segment SGI191310[] = {{&S76748[34], 14}, {NULL}};
const Segment SGI191311[] = {{&S76748[27], 6}, {NULL}};
const Segment SGI191312[] = {{&S76748[114], 9}, {NULL}};
const Segment SGI191313[] = {{&S76748[87], 6}, {NULL}};
const Segment SGI191314[] = {{&S64141[34], 12}, {NULL}};
const Segment SGI191315[] = {{&S64141[27], 6}, {NULL}};
const Segment SGI191316[] = {{&S64141[99], 9}, {NULL}};
const Segment SGI191317[] = {{&S64141[72], 6}, {NULL}};
const Segment SGI191318[] = {{&S76750[44], 22}, {NULL}};
const Segment SGI191319[] = {{&S76750[44], 34}, {NULL}};
const Segment SGI191320[] = {{&S76750[30], 6}, {NULL}};
const Segment SGI191321[] = {{&S76750[31], 3}, {NULL}};
const Segment SGI191322[] = {{&S76750[92], 6}, {NULL}};
const Segment SGI191323[] = {{&S64142[34], 10}, {NULL}};
const Segment SGI191324[] = {{&S64142[27], 6}, {NULL}};
const Segment SGI191325[] = {{&S64142[97], 9}, {NULL}};
const Segment SGI191326[] = {{&S64142[70], 6}, {NULL}};
const Segment SGI191327[] = {{&S76753[44], 10}, {NULL}};
const Segment SGI191328[] = {{&S76753[44], 22}, {NULL}};
const Segment SGI191329[] = {{&S76753[30], 6}, {NULL}};
const Segment SGI191330[] = {{&S76753[83], 6}, {NULL}};
const Segment SGI191331[] = {{&S64145[34], 17}, {NULL}};
const Segment SGI191332[] = {{&S64145[27], 6}, {NULL}};
const Segment SGI191333[] = {{&S64145[104], 9}, {NULL}};
const Segment SGI191334[] = {{&S64145[77], 6}, {NULL}};
const Segment SGI191335[] = {{&S76754[44], 10}, {NULL}};
const Segment SGI191336[] = {{&S76754[44], 22}, {NULL}};
const Segment SGI191337[] = {{&S76754[30], 6}, {NULL}};
const Segment SGI191338[] = {{&S76754[31], 3}, {NULL}};
const Segment SGI191339[] = {{&S64146[34], 10}, {NULL}};
const Segment SGI191340[] = {{&S76754[80], 6}, {NULL}};
const Segment SGI191341[] = {{&S64146[27], 6}, {NULL}};
const Segment SGI191342[] = {{&S64146[103], 9}, {NULL}};
const Segment SGI191343[] = {{&S64146[76], 6}, {NULL}};
const Segment SGI191344[] = {{&S76755[44], 22}, {NULL}};
const Segment SGI191345[] = {{&S76755[30], 6}, {NULL}};
const Segment SGI191346[] = {{&S64148[34], 15}, {NULL}};
const Segment SGI191347[] = {{&S76756[38], 10}, {NULL}};
const Segment SGI191348[] = {{&S64148[27], 6}, {NULL}};
const Segment SGI191349[] = {{&S76756[31], 6}, {NULL}};
const Segment SGI191350[] = {{&S69835[44], 10}, {NULL}};
const Segment SGI191351[] = {{&S69835[44], 22}, {NULL}};
const Segment SGI191352[] = {{&S64148[102], 9}, {NULL}};
const Segment SGI191353[] = {{&S69835[30], 6}, {NULL}};
const Segment SGI191354[] = {{&S64148[75], 6}, {NULL}};
const Segment SGI191355[] = {{&S69835[131], 3}, {NULL}};
const Segment SGI191356[] = {{&S69835[96], 6}, {NULL}};
const Segment SGI191357[] = {{&S69835[68], 8}, {NULL}};
const Segment SGI191358[] = {{&S76758[44], 5}, {NULL}};
const Segment SGI191359[] = {{&S69836[44], 10}, {NULL}};
const Segment SGI191360[] = {{&S76758[44], 17}, {NULL}};
const Segment SGI191361[] = {{&S69836[44], 22}, {NULL}};
const Segment SGI191362[] = {{&S76758[30], 6}, {NULL}};
const Segment SGI191363[] = {{&S69836[30], 6}, {NULL}};
const Segment SGI191364[] = {{&S64150[34], 6}, {NULL}};
const Segment SGI191365[] = {{&S64150[27], 6}, {NULL}};
const Segment SGI191366[] = {{&S69836[131], 3}, {NULL}};
const Segment SGI191367[] = {{&S76758[75], 6}, {NULL}};
const Segment SGI191368[] = {{&S69836[96], 6}, {NULL}};
const Segment SGI191369[] = {{&S76758[102], 13}, {NULL}};
const Segment SGI191370[] = {{&S64150[93], 9}, {NULL}};
const Segment SGI191371[] = {{&S69836[68], 8}, {NULL}};
const Segment SGI191372[] = {{&S64150[66], 6}, {NULL}};
const Segment SGI191373[] = {{&S76759[44], 13}, {NULL}};
const Segment SGI191374[] = {{&S69837[44], 10}, {NULL}};
const Segment SGI191375[] = {{&S76759[44], 25}, {NULL}};
const Segment SGI191376[] = {{&S64151[34], 10}, {NULL}};
const Segment SGI191377[] = {{&S69837[44], 22}, {NULL}};
const Segment SGI191378[] = {{&S76759[30], 6}, {NULL}};
const Segment SGI191379[] = {{&S69837[30], 6}, {NULL}};
const Segment SGI191380[] = {{&S64151[27], 6}, {NULL}};
const Segment SGI191381[] = {{&S69837[131], 3}, {NULL}};
const Segment SGI191382[] = {{&S64151[97], 9}, {NULL}};
const Segment SGI191383[] = {{&S76759[83], 6}, {NULL}};
const Segment SGI191384[] = {{&S64151[70], 6}, {NULL}};
const Segment SGI191385[] = {{&S69837[96], 6}, {NULL}};
const Segment SGI191386[] = {{&S76759[110], 13}, {NULL}};
const Segment SGI191387[] = {{&S69837[68], 8}, {NULL}};
const Segment SGI191388[] = {{&S69838[44], 10}, {NULL}};
const Segment SGI191389[] = {{&S69838[44], 22}, {NULL}};
const Segment SGI191390[] = {{&S69838[30], 6}, {NULL}};
const Segment SGI191391[] = {{&S69838[131], 3}, {NULL}};
const Segment SGI191392[] = {{&S69838[96], 6}, {NULL}};
const Segment SGI191393[] = {{&S69838[68], 8}, {NULL}};
const Segment SGI191394[] = {{&S69839[44], 10}, {NULL}};
const Segment SGI191395[] = {{&S69839[44], 22}, {NULL}};
const Segment SGI191396[] = {{&S64155[34], 9}, {NULL}};
const Segment SGI191397[] = {{&S69839[30], 6}, {NULL}};
const Segment SGI191398[] = {{&S64155[27], 6}, {NULL}};
const Segment SGI191399[] = {{&S69839[133], 3}, {NULL}};
const Segment SGI191400[] = {{&S69839[98], 6}, {NULL}};
const Segment SGI191401[] = {{&S64155[96], 9}, {NULL}};
const Segment SGI191402[] = {{&S64155[69], 6}, {NULL}};
const Segment SGI191403[] = {{&S69839[68], 8}, {NULL}};
const Segment SGI191404[] = {{&S69840[42], 10}, {NULL}};
const Segment SGI191405[] = {{&S69840[30], 6}, {NULL}};
const Segment SGI191406[] = {{&S69840[129], 3}, {NULL}};
const Segment SGI191407[] = {{&S69840[94], 6}, {NULL}};
const Segment SGI191408[] = {{&S69840[66], 8}, {NULL}};
const Segment SGI191409[] = {{&S69844[34], 8}, {NULL}};
const Segment SGI191410[] = {{&S69844[27], 6}, {NULL}};
const Segment SGI191411[] = {{&S69844[95], 9}, {NULL}};
const Segment SGI191412[] = {{&S69844[68], 6}, {NULL}};
const Segment SGI191413[] = {{&S76774[44], 12}, {NULL}};
const Segment SGI191414[] = {{&S76774[44], 24}, {NULL}};
const Segment SGI191415[] = {{&S76774[30], 6}, {NULL}};
const Segment SGI191416[] = {{&S76774[31], 3}, {NULL}};
const Segment SGI191417[] = {{&S76774[82], 6}, {NULL}};
const Segment SGI191418[] = {{&S76777[44], 12}, {NULL}};
const Segment SGI191419[] = {{&S76777[44], 24}, {NULL}};
const Segment SGI191420[] = {{&S76777[30], 6}, {NULL}};
const Segment SGI191421[] = {{&S76777[31], 3}, {NULL}};
const Segment SGI191422[] = {{&S76777[82], 6}, {NULL}};
const Segment SGI191423[] = {{&S76778[38], 12}, {NULL}};
const Segment SGI191424[] = {{&S76778[31], 6}, {NULL}};
const Segment SGI191425[] = {{&S76785[44], 10}, {NULL}};
const Segment SGI191426[] = {{&S76785[44], 22}, {NULL}};
const Segment SGI191427[] = {{&S76785[30], 6}, {NULL}};
const Segment SGI191428[] = {{&S76785[31], 3}, {NULL}};
const Segment SGI191429[] = {{&S76785[80], 6}, {NULL}};
const Segment SGI191430[] = {{&S76785[139], 10}, {NULL}};
const Segment SGI191431[] = {{&S69862[34], 25}, {NULL}};
const Segment SGI191432[] = {{&S69862[27], 6}, {NULL}};
const Segment SGI191433[] = {{&S69862[112], 9}, {NULL}};
const Segment SGI191434[] = {{&S69862[85], 6}, {NULL}};
const Segment SGI191435[] = {{&S69862[146], 8}, {NULL}};
const Segment SGI191436[] = {{&S76786[34], 7}, {NULL}};
const Segment SGI191437[] = {{&S76786[27], 6}, {NULL}};
const Segment SGI191438[] = {{&S76786[94], 9}, {NULL}};
const Segment SGI191439[] = {{&S76786[67], 6}, {NULL}};
const Segment SGI191440[] = {{&S76788[34], 7}, {NULL}};
const Segment SGI191441[] = {{&S76788[27], 6}, {NULL}};
const Segment SGI191442[] = {{&S76788[94], 9}, {NULL}};
const Segment SGI191443[] = {{&S76788[67], 6}, {NULL}};
const Segment SGI191444[] = {{&S76794[34], 5}, {NULL}};
const Segment SGI191445[] = {{&S76794[27], 6}, {NULL}};
const Segment SGI191446[] = {{&S76794[92], 9}, {NULL}};
const Segment SGI191447[] = {{&S76794[65], 6}, {NULL}};
const Segment SGI191448[] = {{&S65476[44], 10}, {NULL}};
const Segment SGI191449[] = {{&S76821[44], 9}, {NULL}};
const Segment SGI191450[] = {{&S65476[44], 22}, {NULL}};
const Segment SGI191451[] = {{&S76821[44], 21}, {NULL}};
const Segment SGI191452[] = {{&S65476[30], 6}, {NULL}};
const Segment SGI191453[] = {{&S76821[30], 6}, {NULL}};
const Segment SGI191454[] = {{&S65476[135], 3}, {NULL}};
const Segment SGI191455[] = {{&S76821[31], 3}, {NULL}};
const Segment SGI191456[] = {{&S76821[79], 6}, {NULL}};
const Segment SGI191457[] = {{&S65476[100], 6}, {NULL}};
const Segment SGI191458[] = {{&S65476[68], 8}, {NULL}};
const Segment SGI191459[] = {{&S76822[38], 9}, {NULL}};
const Segment SGI191460[] = {{&S76822[31], 6}, {NULL}};
const Segment SGI191461[] = {{&S76823[44], 9}, {NULL}};
const Segment SGI191462[] = {{&S76823[44], 21}, {NULL}};
const Segment SGI191463[] = {{&S76823[30], 6}, {NULL}};
const Segment SGI191464[] = {{&S76823[82], 6}, {NULL}};
const Segment SGI191465[] = {{&S76827[34], 14}, {NULL}};
const Segment SGI191466[] = {{&S76827[27], 6}, {NULL}};
const Segment SGI191467[] = {{&S76827[101], 9}, {NULL}};
const Segment SGI191468[] = {{&S76827[74], 6}, {NULL}};
const Segment SGI191469[] = {{&S76830[34], 7}, {NULL}};
const Segment SGI191470[] = {{&S76830[27], 6}, {NULL}};
const Segment SGI191471[] = {{&S76830[93], 9}, {NULL}};
const Segment SGI191472[] = {{&S76830[66], 6}, {NULL}};
const Segment SGI191473[] = {{&S65498[34], 9}, {NULL}};
const Segment SGI191474[] = {{&S65498[27], 6}, {NULL}};
const Segment SGI191475[] = {{&S65498[96], 9}, {NULL}};
const Segment SGI191476[] = {{&S65498[69], 6}, {NULL}};
const Segment SGI191477[] = {{&S76838[34], 11}, {NULL}};
const Segment SGI191478[] = {{&S76838[27], 6}, {NULL}};
const Segment SGI191479[] = {{&S76838[98], 9}, {NULL}};
const Segment SGI191480[] = {{&S76838[71], 6}, {NULL}};
const Segment SGI191481[] = {{&S65499[34], 9}, {NULL}};
const Segment SGI191482[] = {{&S65499[27], 6}, {NULL}};
const Segment SGI191483[] = {{&S65499[96], 9}, {NULL}};
const Segment SGI191484[] = {{&S65499[69], 6}, {NULL}};
const Segment SGI191485[] = {{&S76839[34], 15}, {NULL}};
const Segment SGI191486[] = {{&S76839[27], 6}, {NULL}};
const Segment SGI191487[] = {{&S76839[102], 9}, {NULL}};
const Segment SGI191488[] = {{&S76839[75], 6}, {NULL}};
const Segment SGI191489[] = {{&S69933[44], 10}, {NULL}};
const Segment SGI191490[] = {{&S65500[34], 9}, {NULL}};
const Segment SGI191491[] = {{&S69933[44], 22}, {NULL}};
const Segment SGI191492[] = {{&S69933[30], 6}, {NULL}};
const Segment SGI191493[] = {{&S65500[27], 6}, {NULL}};
const Segment SGI191494[] = {{&S69933[115], 3}, {NULL}};
const Segment SGI191495[] = {{&S65500[96], 9}, {NULL}};
const Segment SGI191496[] = {{&S69933[80], 6}, {NULL}};
const Segment SGI191497[] = {{&S65500[69], 6}, {NULL}};
const Segment SGI191498[] = {{&S65501[34], 9}, {NULL}};
const Segment SGI191499[] = {{&S65501[27], 6}, {NULL}};
const Segment SGI191500[] = {{&S65501[96], 9}, {NULL}};
const Segment SGI191501[] = {{&S65501[69], 6}, {NULL}};
const Segment SGI191502[] = {{&S69935[44], 10}, {NULL}};
const Segment SGI191503[] = {{&S69935[44], 22}, {NULL}};
const Segment SGI191504[] = {{&S69935[30], 6}, {NULL}};
const Segment SGI191505[] = {{&S69935[115], 3}, {NULL}};
const Segment SGI191506[] = {{&S69935[80], 6}, {NULL}};
const Segment SGI191507[] = {{&S76844[34], 18}, {NULL}};
const Segment SGI191508[] = {{&S76844[27], 6}, {NULL}};
const Segment SGI191509[] = {{&S69936[44], 10}, {NULL}};
const Segment SGI191510[] = {{&S69936[44], 71}, {NULL}};
const Segment SGI191511[] = {{&S69936[30], 6}, {NULL}};
const Segment SGI191512[] = {{&S76844[105], 9}, {NULL}};
const Segment SGI191513[] = {{&S76844[78], 6}, {NULL}};
const Segment SGI191514[] = {{&S76845[34], 11}, {NULL}};
const Segment SGI191515[] = {{&S76845[27], 6}, {NULL}};
const Segment SGI191516[] = {{&S69937[38], 10}, {NULL}};
const Segment SGI191517[] = {{&S69937[31], 6}, {NULL}};
const Segment SGI191518[] = {{&S76845[98], 9}, {NULL}};
const Segment SGI191519[] = {{&S76845[71], 6}, {NULL}};
const Segment SGI191520[] = {{&S69938[44], 10}, {NULL}};
const Segment SGI191521[] = {{&S69938[44], 22}, {NULL}};
const Segment SGI191522[] = {{&S69938[30], 6}, {NULL}};
const Segment SGI191523[] = {{&S69938[80], 6}, {NULL}};
const Segment SGI191524[] = {{&S69941[44], 10}, {NULL}};
const Segment SGI191525[] = {{&S69941[44], 71}, {NULL}};
const Segment SGI191526[] = {{&S69941[30], 6}, {NULL}};
const Segment SGI191527[] = {{&S69949[34], 10}, {NULL}};
const Segment SGI191528[] = {{&S69949[27], 6}, {NULL}};
const Segment SGI191529[] = {{&S69949[97], 9}, {NULL}};
const Segment SGI191530[] = {{&S69949[70], 6}, {NULL}};
const Segment SGI191531[] = {{&S69950[34], 10}, {NULL}};
const Segment SGI191532[] = {{&S69950[27], 6}, {NULL}};
const Segment SGI191533[] = {{&S69950[97], 9}, {NULL}};
const Segment SGI191534[] = {{&S69950[70], 6}, {NULL}};
const Segment SGI191535[] = {{&S69953[44], 4}, {NULL}};
const Segment SGI191536[] = {{&S69953[44], 16}, {NULL}};
const Segment SGI191537[] = {{&S69953[30], 6}, {NULL}};
const Segment SGI191538[] = {{&S69953[109], 3}, {NULL}};
const Segment SGI191539[] = {{&S69953[74], 6}, {NULL}};
const Segment SGI191540[] = {{&S69959[34], 8}, {NULL}};
const Segment SGI191541[] = {{&S69959[27], 6}, {NULL}};
const Segment SGI191542[] = {{&S69959[95], 9}, {NULL}};
const Segment SGI191543[] = {{&S69959[68], 6}, {NULL}};
const Segment SGI191544[] = {{&S64299[6], 4}, {NULL}};
const Segment SGI191545[] = {{&S64299[76], 5}, {NULL}};
const Segment SGI191546[] = {{&S65571[44], 14}, {NULL}};
const Segment SGI191547[] = {{&S65571[44], 31}, {NULL}};
const Segment SGI191548[] = {{&S65571[30], 6}, {NULL}};
const Segment SGI191549[] = {{&S65571[124], 3}, {NULL}};
const Segment SGI191550[] = {{&S65571[89], 6}, {NULL}};
const Segment SGI191551[] = {{&S65572[44], 14}, {NULL}};
const Segment SGI191552[] = {{&S65572[44], 31}, {NULL}};
const Segment SGI191553[] = {{&S65572[30], 6}, {NULL}};
const Segment SGI191554[] = {{&S65572[92], 6}, {NULL}};
const Segment SGI191555[] = {{&S65573[38], 14}, {NULL}};
const Segment SGI191556[] = {{&S65573[31], 6}, {NULL}};
const Segment SGI191557[] = {{&S65576[44], 11}, {NULL}};
const Segment SGI191558[] = {{&S65576[44], 23}, {NULL}};
const Segment SGI191559[] = {{&S65576[30], 6}, {NULL}};
const Segment SGI191560[] = {{&S65576[116], 3}, {NULL}};
const Segment SGI191561[] = {{&S65576[81], 6}, {NULL}};
const Segment SGI191562[] = {{&S65577[38], 11}, {NULL}};
const Segment SGI191563[] = {{&S65577[31], 6}, {NULL}};
const Segment SGI191564[] = {{&S65582[34], 10}, {NULL}};
const Segment SGI191565[] = {{&S65582[27], 6}, {NULL}};
const Segment SGI191566[] = {{&S65582[97], 9}, {NULL}};
const Segment SGI191567[] = {{&S65582[70], 6}, {NULL}};
const Segment SGI191568[] = {{&S70012[44], 11}, {NULL}};
const Segment SGI191569[] = {{&S70012[44], 27}, {NULL}};
const Segment SGI191570[] = {{&S70012[30], 6}, {NULL}};
const Segment SGI191571[] = {{&S70012[120], 3}, {NULL}};
const Segment SGI191572[] = {{&S70012[85], 6}, {NULL}};
const Segment SGI191573[] = {{&S70013[44], 7}, {NULL}};
const Segment SGI191574[] = {{&S70013[44], 23}, {NULL}};
const Segment SGI191575[] = {{&S70013[30], 6}, {NULL}};
const Segment SGI191576[] = {{&S70013[116], 3}, {NULL}};
const Segment SGI191577[] = {{&S70013[81], 6}, {NULL}};
const Segment SGI191578[] = {{&S70014[38], 7}, {NULL}};
const Segment SGI191579[] = {{&S70014[31], 6}, {NULL}};
const Segment SGI191580[] = {{&S70025[45], 10}, {NULL}};
const Segment SGI191581[] = {{&S70025[30], 6}, {NULL}};
const Segment SGI191582[] = {{&S70025[115], 3}, {NULL}};
const Segment SGI191583[] = {{&S70025[81], 5}, {NULL}};
const Segment SGI191584[] = {{&S70025[139], 13}, {NULL}};
const Segment SGI191585[] = {{&S65606[34], 14}, {NULL}};
const Segment SGI191586[] = {{&S65606[27], 6}, {NULL}};
const Segment SGI191587[] = {{&S65606[101], 9}, {NULL}};
const Segment SGI191588[] = {{&S65606[74], 6}, {NULL}};
const Segment SGI191589[] = {{&S65607[34], 14}, {NULL}};
const Segment SGI191590[] = {{&S65607[27], 6}, {NULL}};
const Segment SGI191591[] = {{&S65607[101], 9}, {NULL}};
const Segment SGI191592[] = {{&S65607[74], 6}, {NULL}};
const Segment SGI191593[] = {{&S70028[44], 9}, {NULL}};
const Segment SGI191594[] = {{&S70028[44], 21}, {NULL}};
const Segment SGI191595[] = {{&S70028[30], 6}, {NULL}};
const Segment SGI191596[] = {{&S70028[114], 3}, {NULL}};
const Segment SGI191597[] = {{&S70028[79], 6}, {NULL}};
const Segment SGI191598[] = {{&S70032[44], 22}, {NULL}};
const Segment SGI191599[] = {{&S70032[44], 34}, {NULL}};
const Segment SGI191600[] = {{&S70032[30], 6}, {NULL}};
const Segment SGI191601[] = {{&S70032[127], 3}, {NULL}};
const Segment SGI191602[] = {{&S70032[92], 6}, {NULL}};
const Segment SGI191603[] = {{&S70033[38], 22}, {NULL}};
const Segment SGI191604[] = {{&S70033[31], 6}, {NULL}};
const Segment SGI191605[] = {{&S65632[34], 12}, {NULL}};
const Segment SGI191606[] = {{&S65632[27], 6}, {NULL}};
const Segment SGI191607[] = {{&S65632[109], 9}, {NULL}};
const Segment SGI191608[] = {{&S65632[82], 6}, {NULL}};
const Segment SGI191609[] = {{&S65632[150], 8}, {NULL}};
const Segment SGI191610[] = {{&S65641[44], 10}, {NULL}};
const Segment SGI191611[] = {{&S65641[44], 22}, {NULL}};
const Segment SGI191612[] = {{&S65641[30], 6}, {NULL}};
const Segment SGI191613[] = {{&S65641[115], 3}, {NULL}};
const Segment SGI191614[] = {{&S65641[80], 6}, {NULL}};
const Segment SGI191615[] = {{&S65642[38], 10}, {NULL}};
const Segment SGI191616[] = {{&S65642[31], 6}, {NULL}};
const Segment SGI191617[] = {{&S65681[34], 7}, {NULL}};
const Segment SGI191618[] = {{&S65681[27], 6}, {NULL}};
const Segment SGI191619[] = {{&S65681[98], 9}, {NULL}};
const Segment SGI191620[] = {{&S65681[71], 6}, {NULL}};
const Segment SGI191621[] = {{&S70101[44], 12}, {NULL}};
const Segment SGI191622[] = {{&S70101[44], 24}, {NULL}};
const Segment SGI191623[] = {{&S70101[30], 6}, {NULL}};
const Segment SGI191624[] = {{&S70101[117], 3}, {NULL}};
const Segment SGI191625[] = {{&S70101[82], 6}, {NULL}};
const Segment SGI191626[] = {{&S70102[44], 18}, {NULL}};
const Segment SGI191627[] = {{&S70102[44], 30}, {NULL}};
const Segment SGI191628[] = {{&S70102[30], 6}, {NULL}};
const Segment SGI191629[] = {{&S70102[144], 3}, {NULL}};
const Segment SGI191630[] = {{&S70102[109], 6}, {NULL}};
const Segment SGI191631[] = {{&S70105[44], 18}, {NULL}};
const Segment SGI191632[] = {{&S70105[44], 30}, {NULL}};
const Segment SGI191633[] = {{&S70105[30], 6}, {NULL}};
const Segment SGI191634[] = {{&S70105[144], 3}, {NULL}};
const Segment SGI191635[] = {{&S70105[109], 6}, {NULL}};
const Segment SGI191636[] = {{&S70111[44], 9}, {NULL}};
const Segment SGI191637[] = {{&S70111[44], 21}, {NULL}};
const Segment SGI191638[] = {{&S70111[30], 6}, {NULL}};
const Segment SGI191639[] = {{&S70111[114], 3}, {NULL}};
const Segment SGI191640[] = {{&S70111[79], 6}, {NULL}};
const Segment SGI191641[] = {{&S76877[44], 14}, {NULL}};
const Segment SGI191642[] = {{&S76877[44], 26}, {NULL}};
const Segment SGI191643[] = {{&S76877[30], 6}, {NULL}};
const Segment SGI191644[] = {{&S76877[31], 3}, {NULL}};
const Segment SGI191645[] = {{&S76877[84], 6}, {NULL}};
const Segment SGI191646[] = {{&S70125[44], 10}, {NULL}};
const Segment SGI191647[] = {{&S70125[44], 22}, {NULL}};
const Segment SGI191648[] = {{&S70125[30], 6}, {NULL}};
const Segment SGI191649[] = {{&S70125[131], 3}, {NULL}};
const Segment SGI191650[] = {{&S70125[96], 6}, {NULL}};
const Segment SGI191651[] = {{&S70125[68], 8}, {NULL}};
const Segment SGI191652[] = {{&S76879[34], 7}, {NULL}};
const Segment SGI191653[] = {{&S65717[44], 9}, {NULL}};
const Segment SGI191654[] = {{&S76879[27], 6}, {NULL}};
const Segment SGI191655[] = {{&S65717[44], 21}, {NULL}};
const Segment SGI191656[] = {{&S65717[30], 6}, {NULL}};
const Segment SGI191657[] = {{&S76879[94], 9}, {NULL}};
const Segment SGI191658[] = {{&S65717[114], 3}, {NULL}};
const Segment SGI191659[] = {{&S76879[67], 6}, {NULL}};
const Segment SGI191660[] = {{&S65717[79], 6}, {NULL}};
const Segment SGI191661[] = {{&S70127[44], 10}, {NULL}};
const Segment SGI191662[] = {{&S65718[38], 9}, {NULL}};
const Segment SGI191663[] = {{&S65718[31], 6}, {NULL}};
const Segment SGI191664[] = {{&S70127[44], 22}, {NULL}};
const Segment SGI191665[] = {{&S70127[30], 6}, {NULL}};
const Segment SGI191666[] = {{&S70127[131], 3}, {NULL}};
const Segment SGI191667[] = {{&S70127[96], 6}, {NULL}};
const Segment SGI191668[] = {{&S70127[68], 8}, {NULL}};
const Segment SGI191669[] = {{&S70128[44], 10}, {NULL}};
const Segment SGI191670[] = {{&S70128[44], 22}, {NULL}};
const Segment SGI191671[] = {{&S70128[30], 6}, {NULL}};
const Segment SGI191672[] = {{&S70128[131], 3}, {NULL}};
const Segment SGI191673[] = {{&S70128[96], 6}, {NULL}};
const Segment SGI191674[] = {{&S70128[68], 8}, {NULL}};
const Segment SGI191675[] = {{&S70129[44], 10}, {NULL}};
const Segment SGI191676[] = {{&S70129[44], 22}, {NULL}};
const Segment SGI191677[] = {{&S70129[30], 6}, {NULL}};
const Segment SGI191678[] = {{&S70129[131], 3}, {NULL}};
const Segment SGI191679[] = {{&S70129[96], 6}, {NULL}};
const Segment SGI191680[] = {{&S70129[68], 8}, {NULL}};
const Segment SGI191681[] = {{&S76885[44], 11}, {NULL}};
const Segment SGI191682[] = {{&S76885[44], 23}, {NULL}};
const Segment SGI191683[] = {{&S76885[30], 6}, {NULL}};
const Segment SGI191684[] = {{&S76885[31], 3}, {NULL}};
const Segment SGI191685[] = {{&S76885[81], 6}, {NULL}};
const Segment SGI191686[] = {{&S76889[44], 14}, {NULL}};
const Segment SGI191687[] = {{&S76889[44], 26}, {NULL}};
const Segment SGI191688[] = {{&S76889[30], 6}, {NULL}};
const Segment SGI191689[] = {{&S76889[31], 3}, {NULL}};
const Segment SGI191690[] = {{&S76889[84], 6}, {NULL}};
const Segment SGI191691[] = {{&S64479[44], 13}, {NULL}};
const Segment SGI191692[] = {{&S64479[44], 25}, {NULL}};
const Segment SGI191693[] = {{&S64479[30], 6}, {NULL}};
const Segment SGI191694[] = {{&S64479[136], 3}, {NULL}};
const Segment SGI191695[] = {{&S64479[101], 6}, {NULL}};
const Segment SGI191696[] = {{&S64479[71], 8}, {NULL}};
const Segment SGI191697[] = {{&S65733[34], 9}, {NULL}};
const Segment SGI191698[] = {{&S65733[27], 6}, {NULL}};
const Segment SGI191699[] = {{&S65733[96], 9}, {NULL}};
const Segment SGI191700[] = {{&S65733[69], 6}, {NULL}};
const Segment SGI191701[] = {{&S64482[44], 13}, {NULL}};
const Segment SGI191702[] = {{&S64482[44], 25}, {NULL}};
const Segment SGI191703[] = {{&S65734[34], 10}, {NULL}};
const Segment SGI191704[] = {{&S64482[30], 6}, {NULL}};
const Segment SGI191705[] = {{&S65734[27], 6}, {NULL}};
const Segment SGI191706[] = {{&S70140[34], 9}, {NULL}};
const Segment SGI191707[] = {{&S70140[27], 6}, {NULL}};
const Segment SGI191708[] = {{&S64482[136], 3}, {NULL}};
const Segment SGI191709[] = {{&S64482[101], 6}, {NULL}};
const Segment SGI191710[] = {{&S65734[97], 9}, {NULL}};
const Segment SGI191711[] = {{&S65734[70], 6}, {NULL}};
const Segment SGI191712[] = {{&S64482[71], 8}, {NULL}};
const Segment SGI191713[] = {{&S70140[96], 9}, {NULL}};
const Segment SGI191714[] = {{&S70140[69], 6}, {NULL}};
const Segment SGI191715[] = {{&S76896[44], 9}, {NULL}};
const Segment SGI191716[] = {{&S65735[34], 10}, {NULL}};
const Segment SGI191717[] = {{&S76896[44], 21}, {NULL}};
const Segment SGI191718[] = {{&S76896[30], 6}, {NULL}};
const Segment SGI191719[] = {{&S65735[27], 6}, {NULL}};
const Segment SGI191720[] = {{&S76896[31], 3}, {NULL}};
const Segment SGI191721[] = {{&S65735[97], 9}, {NULL}};
const Segment SGI191722[] = {{&S76896[79], 6}, {NULL}};
const Segment SGI191723[] = {{&S65735[70], 6}, {NULL}};
const Segment SGI191724[] = {{&S65736[34], 10}, {NULL}};
const Segment SGI191725[] = {{&S65736[27], 6}, {NULL}};
const Segment SGI191726[] = {{&S65736[97], 9}, {NULL}};
const Segment SGI191727[] = {{&S65736[70], 6}, {NULL}};
const Segment SGI191728[] = {{&S65737[34], 9}, {NULL}};
const Segment SGI191729[] = {{&S65737[27], 6}, {NULL}};
const Segment SGI191730[] = {{&S65737[96], 9}, {NULL}};
const Segment SGI191731[] = {{&S65737[69], 6}, {NULL}};
const Segment SGI191732[] = {{&S65738[34], 7}, {NULL}};
const Segment SGI191733[] = {{&S65738[27], 6}, {NULL}};
const Segment SGI191734[] = {{&S65738[100], 9}, {NULL}};
const Segment SGI191735[] = {{&S65738[73], 6}, {NULL}};
const Segment SGI191736[] = {{&S65750[34], 13}, {NULL}};
const Segment SGI191737[] = {{&S65750[27], 6}, {NULL}};
const Segment SGI191738[] = {{&S65750[100], 9}, {NULL}};
const Segment SGI191739[] = {{&S65750[73], 6}, {NULL}};
const Segment SGI191740[] = {{&S65751[34], 12}, {NULL}};
const Segment SGI191741[] = {{&S65751[27], 6}, {NULL}};
const Segment SGI191742[] = {{&S65751[109], 9}, {NULL}};
const Segment SGI191743[] = {{&S65751[82], 6}, {NULL}};
const Segment SGI191744[] = {{&S76918[34], 4}, {NULL}};
const Segment SGI191745[] = {{&S76918[27], 6}, {NULL}};
const Segment SGI191746[] = {{&S76918[116], 9}, {NULL}};
const Segment SGI191747[] = {{&S76918[89], 6}, {NULL}};
const Segment SGI191748[] = {{&S76921[44], 21}, {NULL}};
const Segment SGI191749[] = {{&S76921[44], 33}, {NULL}};
const Segment SGI191750[] = {{&S76921[30], 6}, {NULL}};
const Segment SGI191751[] = {{&S76921[31], 3}, {NULL}};
const Segment SGI191752[] = {{&S76921[91], 6}, {NULL}};
const Segment SGI191753[] = {{&S76922[44], 8}, {NULL}};
const Segment SGI191754[] = {{&S76922[44], 20}, {NULL}};
const Segment SGI191755[] = {{&S76922[30], 6}, {NULL}};
const Segment SGI191756[] = {{&S76922[31], 3}, {NULL}};
const Segment SGI191757[] = {{&S76922[78], 6}, {NULL}};
const Segment SGI191758[] = {{&S64521[44], 11}, {NULL}};
const Segment SGI191759[] = {{&S64521[44], 23}, {NULL}};
const Segment SGI191760[] = {{&S64521[30], 6}, {NULL}};
const Segment SGI191761[] = {{&S64521[116], 3}, {NULL}};
const Segment SGI191762[] = {{&S64521[81], 6}, {NULL}};
const Segment SGI191763[] = {{&S64522[38], 11}, {NULL}};
const Segment SGI191764[] = {{&S64522[31], 6}, {NULL}};
const Segment SGI191765[] = {{&S76930[34], 17}, {NULL}};
const Segment SGI191766[] = {{&S76930[27], 6}, {NULL}};
const Segment SGI191767[] = {{&S76930[104], 9}, {NULL}};
const Segment SGI191768[] = {{&S76930[77], 6}, {NULL}};
const Segment SGI191769[] = {{&S76936[34], 11}, {NULL}};
const Segment SGI191770[] = {{&S76936[27], 6}, {NULL}};
const Segment SGI191771[] = {{&S76936[98], 9}, {NULL}};
const Segment SGI191772[] = {{&S76936[71], 6}, {NULL}};
const Segment SGI191773[] = {{&S76937[34], 10}, {NULL}};
const Segment SGI191774[] = {{&S76937[27], 6}, {NULL}};
const Segment SGI191775[] = {{&S64534[44], 11}, {NULL}};
const Segment SGI191776[] = {{&S64534[44], 23}, {NULL}};
const Segment SGI191777[] = {{&S64534[30], 6}, {NULL}};
const Segment SGI191778[] = {{&S76937[97], 9}, {NULL}};
const Segment SGI191779[] = {{&S64534[116], 3}, {NULL}};
const Segment SGI191780[] = {{&S76937[70], 6}, {NULL}};
const Segment SGI191781[] = {{&S64534[81], 6}, {NULL}};
const Segment SGI191782[] = {{&S64535[38], 11}, {NULL}};
const Segment SGI191783[] = {{&S64535[31], 6}, {NULL}};
const Segment SGI191784[] = {{&S76941[34], 25}, {NULL}};
const Segment SGI191785[] = {{&S76941[27], 6}, {NULL}};
const Segment SGI191786[] = {{&S76941[112], 9}, {NULL}};
const Segment SGI191787[] = {{&S76941[85], 6}, {NULL}};
const Segment SGI191788[] = {{&S76942[34], 20}, {NULL}};
const Segment SGI191789[] = {{&S76942[27], 6}, {NULL}};
const Segment SGI191790[] = {{&S76942[107], 9}, {NULL}};
const Segment SGI191791[] = {{&S76942[80], 6}, {NULL}};
const Segment SGI191792[] = {{&S65794[34], 10}, {NULL}};
const Segment SGI191793[] = {{&S65794[27], 6}, {NULL}};
const Segment SGI191794[] = {{&S65794[97], 9}, {NULL}};
const Segment SGI191795[] = {{&S65794[70], 6}, {NULL}};
const Segment SGI191796[] = {{&S65795[34], 10}, {NULL}};
const Segment SGI191797[] = {{&S65795[27], 6}, {NULL}};
const Segment SGI191798[] = {{&S65795[97], 9}, {NULL}};
const Segment SGI191799[] = {{&S65795[70], 6}, {NULL}};
const Segment SGI191800[] = {{&S64547[44], 7}, {NULL}};
const Segment SGI191801[] = {{&S65796[34], 11}, {NULL}};
const Segment SGI191802[] = {{&S64547[44], 19}, {NULL}};
const Segment SGI191803[] = {{&S65796[27], 6}, {NULL}};
const Segment SGI191804[] = {{&S64547[30], 6}, {NULL}};
const Segment SGI191805[] = {{&S64547[112], 3}, {NULL}};
const Segment SGI191806[] = {{&S65796[98], 9}, {NULL}};
const Segment SGI191807[] = {{&S64547[77], 6}, {NULL}};
const Segment SGI191808[] = {{&S65796[71], 6}, {NULL}};
const Segment SGI191809[] = {{&S64548[38], 7}, {NULL}};
const Segment SGI191810[] = {{&S64548[31], 6}, {NULL}};
const Segment SGI191811[] = {{&S65797[34], 10}, {NULL}};
const Segment SGI191812[] = {{&S65797[27], 6}, {NULL}};
const Segment SGI191813[] = {{&S65797[97], 9}, {NULL}};
const Segment SGI191814[] = {{&S65797[70], 6}, {NULL}};
const Segment SGI191815[] = {{&S65798[34], 9}, {NULL}};
const Segment SGI191816[] = {{&S65798[27], 6}, {NULL}};
const Segment SGI191817[] = {{&S65798[96], 9}, {NULL}};
const Segment SGI191818[] = {{&S65798[69], 6}, {NULL}};
const Segment SGI191819[] = {{&S65799[34], 12}, {NULL}};
const Segment SGI191820[] = {{&S65799[27], 6}, {NULL}};
const Segment SGI191821[] = {{&S64552[34], 10}, {NULL}};
const Segment SGI191822[] = {{&S65799[99], 9}, {NULL}};
const Segment SGI191823[] = {{&S64552[27], 6}, {NULL}};
const Segment SGI191824[] = {{&S65799[72], 6}, {NULL}};
const Segment SGI191825[] = {{&S64552[97], 9}, {NULL}};
const Segment SGI191826[] = {{&S64552[70], 6}, {NULL}};
const Segment SGI191827[] = {{&S76970[34], 6}, {NULL}};
const Segment SGI191828[] = {{&S76970[27], 6}, {NULL}};
const Segment SGI191829[] = {{&S76970[93], 9}, {NULL}};
const Segment SGI191830[] = {{&S76970[66], 6}, {NULL}};
const Segment SGI191831[] = {{&S76971[44], 10}, {NULL}};
const Segment SGI191832[] = {{&S76971[44], 22}, {NULL}};
const Segment SGI191833[] = {{&S76971[30], 6}, {NULL}};
const Segment SGI191834[] = {{&S76971[31], 3}, {NULL}};
const Segment SGI191835[] = {{&S76971[95], 6}, {NULL}};
const Segment SGI191836[] = {{&S76971[68], 8}, {NULL}};
const Segment SGI191837[] = {{&S76972[44], 10}, {NULL}};
const Segment SGI191838[] = {{&S76972[44], 22}, {NULL}};
const Segment SGI191839[] = {{&S76972[30], 6}, {NULL}};
const Segment SGI191840[] = {{&S76972[31], 3}, {NULL}};
const Segment SGI191841[] = {{&S76972[127], 6}, {NULL}};
const Segment SGI191842[] = {{&S76972[68], 8}, {&S76972[68], 8}, {NULL}};
const Segment SGI191843[] = {{&S65821[34], 10}, {NULL}};
const Segment SGI191844[] = {{&S65821[27], 6}, {NULL}};
const Segment SGI191845[] = {{&S76973[44], 9}, {NULL}};
const Segment SGI191846[] = {{&S76973[44], 21}, {NULL}};
const Segment SGI191847[] = {{&S76973[30], 6}, {NULL}};
const Segment SGI191848[] = {{&S65821[97], 9}, {NULL}};
const Segment SGI191849[] = {{&S65821[70], 6}, {NULL}};
const Segment SGI191850[] = {{&S76973[31], 3}, {NULL}};
const Segment SGI191851[] = {{&S76973[79], 6}, {NULL}};
const Segment SGI191852[] = {{&S70233[34], 5}, {NULL}};
const Segment SGI191853[] = {{&S70233[27], 6}, {NULL}};
const Segment SGI191854[] = {{&S70233[92], 9}, {NULL}};
const Segment SGI191855[] = {{&S70233[65], 6}, {NULL}};
const Segment SGI191856[] = {{&S65822[34], 10}, {NULL}};
const Segment SGI191857[] = {{&S65822[27], 6}, {NULL}};
const Segment SGI191858[] = {{&S65822[97], 9}, {NULL}};
const Segment SGI191859[] = {{&S65822[70], 6}, {NULL}};
const Segment SGI191860[] = {{&S65823[34], 10}, {NULL}};
const Segment SGI191861[] = {{&S65823[27], 6}, {NULL}};
const Segment SGI191862[] = {{&S65823[97], 9}, {NULL}};
const Segment SGI191863[] = {{&S65823[70], 6}, {NULL}};
const Segment SGI191864[] = {{&S70236[34], 18}, {NULL}};
const Segment SGI191865[] = {{&S70236[27], 6}, {NULL}};
const Segment SGI191866[] = {{&S70236[114], 9}, {NULL}};
const Segment SGI191867[] = {{&S70236[87], 6}, {NULL}};
const Segment SGI191868[] = {{&S65825[34], 11}, {NULL}};
const Segment SGI191869[] = {{&S65825[27], 6}, {NULL}};
const Segment SGI191870[] = {{&S70237[34], 18}, {NULL}};
const Segment SGI191871[] = {{&S65825[98], 9}, {NULL}};
const Segment SGI191872[] = {{&S70237[27], 6}, {NULL}};
const Segment SGI191873[] = {{&S65825[71], 6}, {NULL}};
const Segment SGI191874[] = {{&S70237[114], 9}, {NULL}};
const Segment SGI191875[] = {{&S70237[87], 6}, {NULL}};
const Segment SGI191876[] = {{&S65826[34], 18}, {NULL}};
const Segment SGI191877[] = {{&S65826[27], 6}, {NULL}};
const Segment SGI191878[] = {{&S65826[105], 9}, {NULL}};
const Segment SGI191879[] = {{&S65826[78], 6}, {NULL}};
const Segment SGI191880[] = {{&S76986[44], 7}, {NULL}};
const Segment SGI191881[] = {{&S76986[44], 19}, {NULL}};
const Segment SGI191882[] = {{&S76986[30], 6}, {NULL}};
const Segment SGI191883[] = {{&S76986[77], 6}, {NULL}};
const Segment SGI191884[] = {{&S76986[104], 13}, {NULL}};
const Segment SGI191885[] = {{&S76987[44], 8}, {NULL}};
const Segment SGI191886[] = {{&S76987[44], 20}, {NULL}};
const Segment SGI191887[] = {{&S76987[30], 6}, {NULL}};
const Segment SGI191888[] = {{&S76987[78], 6}, {NULL}};
const Segment SGI191889[] = {{&S76987[105], 13}, {NULL}};
const Segment SGI191890[] = {{&S65834[44], 12}, {NULL}};
const Segment SGI191891[] = {{&S65834[44], 24}, {NULL}};
const Segment SGI191892[] = {{&S65834[30], 6}, {NULL}};
const Segment SGI191893[] = {{&S65834[137], 3}, {NULL}};
const Segment SGI191894[] = {{&S65834[102], 6}, {NULL}};
const Segment SGI191895[] = {{&S65834[70], 8}, {NULL}};
const Segment SGI191896[] = {{&S76988[45], 17}, {NULL}};
const Segment SGI191897[] = {{&S76988[30], 6}, {NULL}};
const Segment SGI191898[] = {{&S76988[31], 3}, {NULL}};
const Segment SGI191899[] = {{&S76988[88], 5}, {NULL}};
const Segment SGI191900[] = {{&S76988[146], 13}, {NULL}};
const Segment SGI191901[] = {{&S65835[44], 10}, {NULL}};
const Segment SGI191902[] = {{&S65835[44], 22}, {NULL}};
const Segment SGI191903[] = {{&S65835[30], 6}, {NULL}};
const Segment SGI191904[] = {{&S65835[131], 3}, {NULL}};
const Segment SGI191905[] = {{&S65835[96], 6}, {NULL}};
const Segment SGI191906[] = {{&S65835[68], 8}, {NULL}};
const Segment SGI191907[] = {{&S55241[34], 7}, {NULL}};
const Segment SGI191908[] = {{&S55241[27], 6}, {NULL}};
const Segment SGI191909[] = {{&S55241[94], 9}, {NULL}};
const Segment SGI191910[] = {{&S55241[67], 6}, {NULL}};
const Segment SGI191911[] = {{&S55243[34], 11}, {NULL}};
const Segment SGI191912[] = {{&S55243[27], 6}, {NULL}};
const Segment SGI191913[] = {{&S55243[97], 9}, {NULL}};
const Segment SGI191914[] = {{&S55243[70], 6}, {NULL}};
const Segment SGI191915[] = {{&S65854[34], 7}, {NULL}};
const Segment SGI191916[] = {{&S65854[27], 6}, {NULL}};
const Segment SGI191917[] = {{&S65854[94], 9}, {NULL}};
const Segment SGI191918[] = {{&S65854[67], 6}, {NULL}};
const Segment SGI191919[] = {{&S65855[34], 7}, {NULL}};
const Segment SGI191920[] = {{&S65855[27], 6}, {NULL}};
const Segment SGI191921[] = {{&S65855[94], 9}, {NULL}};
const Segment SGI191922[] = {{&S65855[67], 6}, {NULL}};
const Segment SGI191923[] = {{&S70281[34], 8}, {NULL}};
const Segment SGI191924[] = {{&S70281[27], 6}, {NULL}};
const Segment SGI191925[] = {{&S70281[95], 9}, {NULL}};
const Segment SGI191926[] = {{&S70281[68], 6}, {NULL}};
const Segment SGI191927[] = {{&S77025[44], 22}, {NULL}};
const Segment SGI191928[] = {{&S77025[44], 34}, {NULL}};
const Segment SGI191929[] = {{&S77025[30], 6}, {NULL}};
const Segment SGI191930[] = {{&S77025[31], 3}, {NULL}};
const Segment SGI191931[] = {{&S77025[92], 6}, {NULL}};
const Segment SGI191932[] = {{&S77026[44], 21}, {NULL}};
const Segment SGI191933[] = {{&S77026[44], 33}, {NULL}};
const Segment SGI191934[] = {{&S77026[30], 6}, {NULL}};
const Segment SGI191935[] = {{&S77026[31], 3}, {NULL}};
const Segment SGI191936[] = {{&S77026[91], 6}, {NULL}};
const Segment SGI191937[] = {{&S65877[44], 5}, {NULL}};
const Segment SGI191938[] = {{&S65877[44], 17}, {NULL}};
const Segment SGI191939[] = {{&S65877[30], 6}, {NULL}};
const Segment SGI191940[] = {{&S65877[110], 3}, {NULL}};
const Segment SGI191941[] = {{&S65877[75], 6}, {NULL}};
const Segment SGI191942[] = {{&S77033[34], 11}, {NULL}};
const Segment SGI191943[] = {{&S77033[27], 6}, {NULL}};
const Segment SGI191944[] = {{&S77033[98], 9}, {NULL}};
const Segment SGI191945[] = {{&S77033[71], 6}, {NULL}};
const Segment SGI191946[] = {{&S55287[34], 10}, {NULL}};
const Segment SGI191947[] = {{&S55287[27], 6}, {NULL}};
const Segment SGI191948[] = {{&S55287[96], 9}, {NULL}};
const Segment SGI191949[] = {{&S55287[69], 6}, {NULL}};
const Segment SGI191950[] = {{&S55288[34], 10}, {NULL}};
const Segment SGI191951[] = {{&S55288[27], 6}, {NULL}};
const Segment SGI191952[] = {{&S55288[96], 9}, {NULL}};
const Segment SGI191953[] = {{&S55288[69], 6}, {NULL}};
const Segment SGI191954[] = {{&S77043[34], 5}, {NULL}};
const Segment SGI191955[] = {{&S77043[27], 6}, {NULL}};
const Segment SGI191956[] = {{&S77043[92], 9}, {NULL}};
const Segment SGI191957[] = {{&S77043[65], 6}, {NULL}};
const Segment SGI191958[] = {{&S55289[34], 10}, {NULL}};
const Segment SGI191959[] = {{&S55289[27], 6}, {NULL}};
const Segment SGI191960[] = {{&S77044[34], 5}, {NULL}};
const Segment SGI191961[] = {{&S55289[96], 9}, {NULL}};
const Segment SGI191962[] = {{&S55289[69], 6}, {NULL}};
const Segment SGI191963[] = {{&S77044[27], 6}, {NULL}};
const Segment SGI191964[] = {{&S77044[92], 9}, {NULL}};
const Segment SGI191965[] = {{&S77044[65], 6}, {NULL}};
const Segment SGI191966[] = {{&S55290[34], 16}, {NULL}};
const Segment SGI191967[] = {{&S55290[27], 6}, {NULL}};
const Segment SGI191968[] = {{&S55290[102], 9}, {NULL}};
const Segment SGI191969[] = {{&S55290[75], 6}, {NULL}};
const Segment SGI191970[] = {{&S55291[34], 10}, {NULL}};
const Segment SGI191971[] = {{&S55291[27], 6}, {NULL}};
const Segment SGI191972[] = {{&S55291[96], 9}, {NULL}};
const Segment SGI191973[] = {{&S55291[69], 6}, {NULL}};
const Segment SGI191974[] = {{&S77048[34], 9}, {NULL}};
const Segment SGI191975[] = {{&S77048[27], 6}, {NULL}};
const Segment SGI191976[] = {{&S77048[96], 9}, {NULL}};
const Segment SGI191977[] = {{&S77048[69], 6}, {NULL}};
const Segment SGI191978[] = {{&S55292[34], 18}, {NULL}};
const Segment SGI191979[] = {{&S55292[27], 6}, {NULL}};
const Segment SGI191980[] = {{&S55292[105], 9}, {NULL}};
const Segment SGI191981[] = {{&S55292[78], 6}, {NULL}};
const Segment SGI191982[] = {{&S77049[34], 9}, {NULL}};
const Segment SGI191983[] = {{&S77049[27], 6}, {NULL}};
const Segment SGI191984[] = {{&S77049[96], 9}, {NULL}};
const Segment SGI191985[] = {{&S55293[34], 10}, {NULL}};
const Segment SGI191986[] = {{&S77049[69], 6}, {NULL}};
const Segment SGI191987[] = {{&S55293[27], 6}, {NULL}};
const Segment SGI191988[] = {{&S55293[96], 9}, {NULL}};
const Segment SGI191989[] = {{&S55293[69], 6}, {NULL}};
const Segment SGI191990[] = {{&S77050[34], 9}, {NULL}};
const Segment SGI191991[] = {{&S77050[27], 6}, {NULL}};
const Segment SGI191992[] = {{&S55294[34], 10}, {NULL}};
const Segment SGI191993[] = {{&S55294[27], 6}, {NULL}};
const Segment SGI191994[] = {{&S77050[96], 9}, {NULL}};
const Segment SGI191995[] = {{&S77050[69], 6}, {NULL}};
const Segment SGI191996[] = {{&S55294[97], 9}, {NULL}};
const Segment SGI191997[] = {{&S55294[70], 6}, {NULL}};
const Segment SGI191998[] = {{&S70320[44], 12}, {NULL}};
const Segment SGI191999[] = {{&S70320[44], 24}, {NULL}};
const Segment SGI192000[] = {{&S70320[30], 6}, {NULL}};
const Segment SGI192001[] = {{&S70320[117], 3}, {NULL}};
const Segment SGI192002[] = {{&S70320[82], 6}, {NULL}};
const Segment SGI192003[] = {{&S70321[44], 16}, {NULL}};
const Segment SGI192004[] = {{&S70321[44], 28}, {NULL}};
const Segment SGI192005[] = {{&S70321[30], 6}, {NULL}};
const Segment SGI192006[] = {{&S70321[121], 3}, {NULL}};
const Segment SGI192007[] = {{&S70321[86], 6}, {NULL}};
const Segment SGI192008[] = {{&S65919[34], 9}, {NULL}};
const Segment SGI192009[] = {{&S65919[27], 6}, {NULL}};
const Segment SGI192010[] = {{&S65919[96], 9}, {NULL}};
const Segment SGI192011[] = {{&S65919[69], 6}, {NULL}};
const Segment SGI192012[] = {{&S65919[137], 8}, {NULL}};
const Segment SGI192013[] = {{&S65920[34], 8}, {NULL}};
const Segment SGI192014[] = {{&S65920[27], 6}, {NULL}};
const Segment SGI192015[] = {{&S65920[95], 9}, {NULL}};
const Segment SGI192016[] = {{&S65920[68], 6}, {NULL}};
const Segment SGI192017[] = {{&S65920[136], 8}, {NULL}};
const Segment SGI192018[] = {{&S65921[34], 11}, {NULL}};
const Segment SGI192019[] = {{&S65921[27], 6}, {NULL}};
const Segment SGI192020[] = {{&S65921[98], 9}, {NULL}};
const Segment SGI192021[] = {{&S65921[71], 6}, {NULL}};
const Segment SGI192022[] = {{&S65921[139], 8}, {NULL}};
const Segment SGI192023[] = {{&S65926[34], 7}, {NULL}};
const Segment SGI192024[] = {{&S65926[27], 6}, {NULL}};
const Segment SGI192025[] = {{&S65926[99], 9}, {NULL}};
const Segment SGI192026[] = {{&S65926[72], 6}, {NULL}};
const Segment SGI192027[] = {{&S77072[44], 12}, {NULL}};
const Segment SGI192028[] = {{&S77072[44], 35}, {NULL}};
const Segment SGI192029[] = {{&S77072[30], 6}, {NULL}};
const Segment SGI192030[] = {{&S77072[31], 3}, {NULL}};
const Segment SGI192031[] = {{&S77072[93], 6}, {NULL}};
const Segment SGI192032[] = {{&S65927[34], 7}, {NULL}};
const Segment SGI192033[] = {{&S65927[27], 6}, {NULL}};
const Segment SGI192034[] = {{&S65927[99], 9}, {NULL}};
const Segment SGI192035[] = {{&S65927[72], 6}, {NULL}};
const Segment SGI192036[] = {{&S65928[44], 10}, {NULL}};
const Segment SGI192037[] = {{&S65928[44], 22}, {NULL}};
const Segment SGI192038[] = {{&S65928[30], 6}, {NULL}};
const Segment SGI192039[] = {{&S65928[115], 3}, {NULL}};
const Segment SGI192040[] = {{&S65928[80], 6}, {NULL}};
const Segment SGI192041[] = {{&S64569[44], 10}, {NULL}};
const Segment SGI192042[] = {{&S65929[38], 10}, {NULL}};
const Segment SGI192043[] = {{&S64569[44], 27}, {NULL}};
const Segment SGI192044[] = {{&S65929[31], 6}, {NULL}};
const Segment SGI192045[] = {{&S64569[30], 6}, {NULL}};
const Segment SGI192046[] = {{&S64569[119], 3}, {NULL}};
const Segment SGI192047[] = {{&S64569[85], 5}, {NULL}};
const Segment SGI192048[] = {{&S55325[34], 9}, {NULL}};
const Segment SGI192049[] = {{&S55325[27], 6}, {NULL}};
const Segment SGI192050[] = {{&S55325[95], 9}, {NULL}};
const Segment SGI192051[] = {{&S55325[68], 6}, {NULL}};
const Segment SGI192052[] = {{&S55326[34], 14}, {NULL}};
const Segment SGI192053[] = {{&S55326[27], 6}, {NULL}};
const Segment SGI192054[] = {{&S55326[100], 9}, {NULL}};
const Segment SGI192055[] = {{&S70347[44], 10}, {NULL}};
const Segment SGI192056[] = {{&S55326[73], 6}, {NULL}};
const Segment SGI192057[] = {{&S70347[44], 22}, {NULL}};
const Segment SGI192058[] = {{&S70347[30], 6}, {NULL}};
const Segment SGI192059[] = {{&S70347[131], 3}, {NULL}};
const Segment SGI192060[] = {{&S70347[96], 6}, {NULL}};
const Segment SGI192061[] = {{&S70347[68], 8}, {NULL}};
const Segment SGI192062[] = {{&S55327[34], 11}, {NULL}};
const Segment SGI192063[] = {{&S55327[27], 6}, {NULL}};
const Segment SGI192064[] = {{&S55327[97], 9}, {NULL}};
const Segment SGI192065[] = {{&S55327[70], 6}, {NULL}};
const Segment SGI192066[] = {{&S55328[34], 9}, {NULL}};
const Segment SGI192067[] = {{&S55328[27], 6}, {NULL}};
const Segment SGI192068[] = {{&S55328[95], 9}, {NULL}};
const Segment SGI192069[] = {{&S55328[68], 6}, {NULL}};
const Segment SGI192070[] = {{&S55329[34], 14}, {NULL}};
const Segment SGI192071[] = {{&S55329[27], 6}, {NULL}};
const Segment SGI192072[] = {{&S65940[59], 10}, {NULL}};
const Segment SGI192073[] = {{&S65940[59], 22}, {NULL}};
const Segment SGI192074[] = {{&S55329[100], 9}, {NULL}};
const Segment SGI192075[] = {{&S65940[45], 6}, {NULL}};
const Segment SGI192076[] = {{&S55329[73], 6}, {NULL}};
const Segment SGI192077[] = {{&S70352[34], 18}, {NULL}};
const Segment SGI192078[] = {{&S65940[130], 3}, {NULL}};
const Segment SGI192079[] = {{&S70352[27], 6}, {NULL}};
const Segment SGI192080[] = {{&S65940[95], 6}, {NULL}};
const Segment SGI192081[] = {{&S65940[0], 14}, {NULL}};
const Segment SGI192082[] = {{&S70352[105], 9}, {NULL}};
const Segment SGI192083[] = {{&S70352[78], 6}, {NULL}};
const Segment SGI192084[] = {{&S55330[34], 11}, {NULL}};
const Segment SGI192085[] = {{&S55330[27], 6}, {NULL}};
const Segment SGI192086[] = {{&S55330[97], 9}, {NULL}};
const Segment SGI192087[] = {{&S55330[70], 6}, {NULL}};
const Segment SGI192088[] = {{&S55331[34], 18}, {NULL}};
const Segment SGI192089[] = {{&S70354[34], 18}, {NULL}};
const Segment SGI192090[] = {{&S55331[27], 6}, {NULL}};
const Segment SGI192091[] = {{&S70354[27], 6}, {NULL}};
const Segment SGI192092[] = {{&S55331[104], 9}, {NULL}};
const Segment SGI192093[] = {{&S55331[77], 6}, {NULL}};
const Segment SGI192094[] = {{&S70354[105], 9}, {NULL}};
const Segment SGI192095[] = {{&S70354[78], 6}, {NULL}};
const Segment SGI192096[] = {{&S64586[34], 10}, {NULL}};
const Segment SGI192097[] = {{&S64586[27], 6}, {NULL}};
const Segment SGI192098[] = {{&S55334[44], 6}, {NULL}};
const Segment SGI192099[] = {{&S64586[97], 9}, {NULL}};
const Segment SGI192100[] = {{&S55334[44], 17}, {NULL}};
const Segment SGI192101[] = {{&S64586[70], 6}, {NULL}};
const Segment SGI192102[] = {{&S55334[30], 6}, {NULL}};
const Segment SGI192103[] = {{&S55334[110], 3}, {NULL}};
const Segment SGI192104[] = {{&S55334[75], 6}, {NULL}};
const Segment SGI192105[] = {{&S55335[38], 6}, {NULL}};
const Segment SGI192106[] = {{&S55335[31], 6}, {NULL}};
const Segment SGI192107[] = {{&S55361[34], 10}, {NULL}};
const Segment SGI192108[] = {{&S55361[27], 6}, {NULL}};
const Segment SGI192109[] = {{&S55361[96], 9}, {NULL}};
const Segment SGI192110[] = {{&S55361[69], 6}, {NULL}};
const Segment SGI192111[] = {{&S65977[44], 10}, {NULL}};
const Segment SGI192112[] = {{&S65977[44], 22}, {NULL}};
const Segment SGI192113[] = {{&S65977[30], 6}, {NULL}};
const Segment SGI192114[] = {{&S65977[115], 3}, {NULL}};
const Segment SGI192115[] = {{&S65977[80], 6}, {NULL}};
const Segment SGI192116[] = {{&S65978[44], 10}, {NULL}};
const Segment SGI192117[] = {{&S65978[44], 22}, {NULL}};
const Segment SGI192118[] = {{&S65978[30], 6}, {NULL}};
const Segment SGI192119[] = {{&S65978[115], 3}, {NULL}};
const Segment SGI192120[] = {{&S65978[80], 6}, {NULL}};
const Segment SGI192121[] = {{&S65979[38], 10}, {NULL}};
const Segment SGI192122[] = {{&S65979[31], 6}, {NULL}};
const Segment SGI192123[] = {{&S65980[44], 18}, {NULL}};
const Segment SGI192124[] = {{&S65980[44], 30}, {NULL}};
const Segment SGI192125[] = {{&S65980[30], 6}, {NULL}};
const Segment SGI192126[] = {{&S65980[123], 3}, {NULL}};
const Segment SGI192127[] = {{&S65980[88], 6}, {NULL}};
const Segment SGI192128[] = {{&S70401[34], 12}, {NULL}};
const Segment SGI192129[] = {{&S70401[27], 6}, {NULL}};
const Segment SGI192130[] = {{&S70401[99], 9}, {NULL}};
const Segment SGI192131[] = {{&S70401[72], 6}, {NULL}};
const Segment SGI192132[] = {{&S70403[34], 4}, {NULL}};
const Segment SGI192133[] = {{&S70403[27], 6}, {NULL}};
const Segment SGI192134[] = {{&S70403[91], 9}, {NULL}};
const Segment SGI192135[] = {{&S70403[64], 6}, {NULL}};
const Segment SGI192136[] = {{&S70417[44], 12}, {NULL}};
const Segment SGI192137[] = {{&S70417[44], 37}, {NULL}};
const Segment SGI192138[] = {{&S70417[30], 6}, {NULL}};
const Segment SGI192139[] = {{&S70417[130], 3}, {NULL}};
const Segment SGI192140[] = {{&S70417[95], 6}, {NULL}};
const Segment SGI192141[] = {{&S64685[44], 12}, {NULL}};
const Segment SGI192142[] = {{&S64685[44], 24}, {NULL}};
const Segment SGI192143[] = {{&S64685[30], 6}, {NULL}};
const Segment SGI192144[] = {{&S64685[135], 3}, {NULL}};
const Segment SGI192145[] = {{&S64685[100], 6}, {NULL}};
const Segment SGI192146[] = {{&S64685[70], 8}, {NULL}};
const Segment SGI192147[] = {{&S64686[44], 12}, {NULL}};
const Segment SGI192148[] = {{&S64686[44], 24}, {NULL}};
const Segment SGI192149[] = {{&S64686[30], 6}, {NULL}};
const Segment SGI192150[] = {{&S64686[135], 3}, {NULL}};
const Segment SGI192151[] = {{&S64686[100], 6}, {NULL}};
const Segment SGI192152[] = {{&S64686[70], 8}, {NULL}};
const Segment SGI192153[] = {{&S70457[34], 6}, {NULL}};
const Segment SGI192154[] = {{&S70457[27], 6}, {NULL}};
const Segment SGI192155[] = {{&S70457[93], 9}, {NULL}};
const Segment SGI192156[] = {{&S70457[66], 6}, {NULL}};
const Segment SGI192157[] = {{&S70471[34], 13}, {NULL}};
const Segment SGI192158[] = {{&S77078[34], 10}, {NULL}};
const Segment SGI192159[] = {{&S70471[27], 6}, {NULL}};
const Segment SGI192160[] = {{&S77078[27], 6}, {NULL}};
const Segment SGI192161[] = {{&S70471[100], 9}, {NULL}};
const Segment SGI192162[] = {{&S70471[73], 6}, {NULL}};
const Segment SGI192163[] = {{&S77078[97], 9}, {NULL}};
const Segment SGI192164[] = {{&S77078[70], 6}, {NULL}};
const Segment SGI192165[] = {{&S77079[34], 23}, {NULL}};
const Segment SGI192166[] = {{&S77079[27], 6}, {NULL}};
const Segment SGI192167[] = {{&S77079[120], 9}, {NULL}};
const Segment SGI192168[] = {{&S77079[93], 6}, {NULL}};
const Segment SGI192169[] = {{&S77080[34], 24}, {NULL}};
const Segment SGI192170[] = {{&S77080[27], 6}, {NULL}};
const Segment SGI192171[] = {{&S77080[121], 9}, {NULL}};
const Segment SGI192172[] = {{&S77080[94], 6}, {NULL}};
const Segment SGI192173[] = {{&S77082[34], 11}, {NULL}};
const Segment SGI192174[] = {{&S77082[27], 6}, {NULL}};
const Segment SGI192175[] = {{&S77082[98], 9}, {NULL}};
const Segment SGI192176[] = {{&S77082[71], 6}, {NULL}};
const Segment SGI192177[] = {{&S77082[139], 8}, {NULL}};
const Segment SGI192178[] = {{&S70491[34], 18}, {NULL}};
const Segment SGI192179[] = {{&S70491[27], 6}, {NULL}};
const Segment SGI192180[] = {{&S70491[114], 9}, {NULL}};
const Segment SGI192181[] = {{&S70491[87], 6}, {NULL}};
const Segment SGI192182[] = {{&S70492[44], 9}, {NULL}};
const Segment SGI192183[] = {{&S70492[44], 21}, {NULL}};
const Segment SGI192184[] = {{&S70492[30], 6}, {NULL}};
const Segment SGI192185[] = {{&S70492[114], 3}, {NULL}};
const Segment SGI192186[] = {{&S70492[79], 6}, {NULL}};
const Segment SGI192187[] = {{&S70493[38], 9}, {NULL}};
const Segment SGI192188[] = {{&S70493[31], 6}, {NULL}};
const Segment SGI192189[] = {{&S77101[34], 6}, {NULL}};
const Segment SGI192190[] = {{&S77101[27], 6}, {NULL}};
const Segment SGI192191[] = {{&S77101[93], 9}, {NULL}};
const Segment SGI192192[] = {{&S77101[66], 6}, {NULL}};
const Segment SGI192193[] = {{&S77103[44], 25}, {NULL}};
const Segment SGI192194[] = {{&S77103[44], 37}, {NULL}};
const Segment SGI192195[] = {{&S77103[30], 6}, {NULL}};
const Segment SGI192196[] = {{&S77103[31], 3}, {NULL}};
const Segment SGI192197[] = {{&S77103[95], 6}, {NULL}};
const Segment SGI192198[] = {{&S64748[34], 11}, {NULL}};
const Segment SGI192199[] = {{&S64748[27], 6}, {NULL}};
const Segment SGI192200[] = {{&S77105[44], 9}, {NULL}};
const Segment SGI192201[] = {{&S77105[44], 21}, {NULL}};
const Segment SGI192202[] = {{&S77105[30], 6}, {NULL}};
const Segment SGI192203[] = {{&S64748[98], 9}, {NULL}};
const Segment SGI192204[] = {{&S77105[31], 3}, {NULL}};
const Segment SGI192205[] = {{&S64748[71], 6}, {NULL}};
const Segment SGI192206[] = {{&S77105[79], 6}, {NULL}};
const Segment SGI192207[] = {{&S64749[34], 7}, {NULL}};
const Segment SGI192208[] = {{&S64749[27], 6}, {NULL}};
const Segment SGI192209[] = {{&S64749[94], 9}, {NULL}};
const Segment SGI192210[] = {{&S64749[67], 6}, {NULL}};
const Segment SGI192211[] = {{&S77108[34], 11}, {NULL}};
const Segment SGI192212[] = {{&S77108[27], 6}, {NULL}};
const Segment SGI192213[] = {{&S77108[98], 9}, {NULL}};
const Segment SGI192214[] = {{&S77108[71], 6}, {NULL}};
const Segment SGI192215[] = {{&S70512[44], 10}, {NULL}};
const Segment SGI192216[] = {{&S77115[34], 16}, {NULL}};
const Segment SGI192217[] = {{&S70512[44], 22}, {NULL}};
const Segment SGI192218[] = {{&S70512[30], 6}, {NULL}};
const Segment SGI192219[] = {{&S77115[27], 6}, {NULL}};
const Segment SGI192220[] = {{&S70512[115], 3}, {NULL}};
const Segment SGI192221[] = {{&S77115[103], 9}, {NULL}};
const Segment SGI192222[] = {{&S70512[80], 6}, {NULL}};
const Segment SGI192223[] = {{&S77115[76], 6}, {NULL}};
const Segment SGI192224[] = {{&S70513[38], 10}, {NULL}};
const Segment SGI192225[] = {{&S70513[31], 6}, {NULL}};
const Segment SGI192226[] = {{&S77116[34], 11}, {NULL}};
const Segment SGI192227[] = {{&S77116[27], 6}, {NULL}};
const Segment SGI192228[] = {{&S77116[110], 9}, {NULL}};
const Segment SGI192229[] = {{&S77116[83], 6}, {NULL}};
const Segment SGI192230[] = {{&S70514[44], 10}, {NULL}};
const Segment SGI192231[] = {{&S70514[44], 22}, {NULL}};
const Segment SGI192232[] = {{&S70514[30], 6}, {NULL}};
const Segment SGI192233[] = {{&S70514[115], 3}, {NULL}};
const Segment SGI192234[] = {{&S70514[80], 6}, {NULL}};
const Segment SGI192235[] = {{&S70516[44], 10}, {NULL}};
const Segment SGI192236[] = {{&S70516[44], 22}, {NULL}};
const Segment SGI192237[] = {{&S70516[30], 6}, {NULL}};
const Segment SGI192238[] = {{&S70516[115], 3}, {NULL}};
const Segment SGI192239[] = {{&S70516[80], 6}, {NULL}};
const Segment SGI192240[] = {{&S70522[44], 10}, {NULL}};
const Segment SGI192241[] = {{&S70522[44], 22}, {NULL}};
const Segment SGI192242[] = {{&S70522[30], 6}, {NULL}};
const Segment SGI192243[] = {{&S70522[115], 3}, {NULL}};
const Segment SGI192244[] = {{&S70522[80], 6}, {NULL}};
const Segment SGI192245[] = {{&S70526[44], 10}, {NULL}};
const Segment SGI192246[] = {{&S70526[44], 47}, {NULL}};
const Segment SGI192247[] = {{&S70526[30], 6}, {NULL}};
const Segment SGI192248[] = {{&S70526[140], 3}, {NULL}};
const Segment SGI192249[] = {{&S70526[105], 6}, {NULL}};
const Segment SGI192250[] = {{&S77138[34], 7}, {NULL}};
const Segment SGI192251[] = {{&S77138[27], 6}, {NULL}};
const Segment SGI192252[] = {{&S77138[94], 9}, {NULL}};
const Segment SGI192253[] = {{&S77138[67], 6}, {NULL}};
const Segment SGI192254[] = {{&S77141[44], 8}, {NULL}};
const Segment SGI192255[] = {{&S77141[44], 20}, {NULL}};
const Segment SGI192256[] = {{&S77141[30], 6}, {NULL}};
const Segment SGI192257[] = {{&S77141[31], 3}, {NULL}};
const Segment SGI192258[] = {{&S77141[78], 6}, {NULL}};
const Segment SGI192259[] = {{&S77147[44], 7}, {NULL}};
const Segment SGI192260[] = {{&S77147[44], 24}, {NULL}};
const Segment SGI192261[] = {{&S77147[30], 6}, {NULL}};
const Segment SGI192262[] = {{&S77147[82], 6}, {NULL}};
const Segment SGI192263[] = {{&S77147[109], 13}, {NULL}};
const Segment SGI192264[] = {{&S77152[44], 19}, {NULL}};
const Segment SGI192265[] = {{&S77152[44], 31}, {NULL}};
const Segment SGI192266[] = {{&S77152[30], 6}, {NULL}};
const Segment SGI192267[] = {{&S77152[31], 3}, {NULL}};
const Segment SGI192268[] = {{&S77152[89], 6}, {NULL}};
const Segment SGI192269[] = {{&S77153[34], 11}, {NULL}};
const Segment SGI192270[] = {{&S77153[27], 6}, {NULL}};
const Segment SGI192271[] = {{&S77153[98], 9}, {NULL}};
const Segment SGI192272[] = {{&S77153[71], 6}, {NULL}};
const Segment SGI192273[] = {{&S70566[34], 11}, {NULL}};
const Segment SGI192274[] = {{&S70566[27], 6}, {NULL}};
const Segment SGI192275[] = {{&S70566[98], 9}, {NULL}};
const Segment SGI192276[] = {{&S70566[71], 6}, {NULL}};
const Segment SGI192277[] = {{&S70567[34], 10}, {NULL}};
const Segment SGI192278[] = {{&S70567[27], 6}, {NULL}};
const Segment SGI192279[] = {{&S70567[97], 9}, {NULL}};
const Segment SGI192280[] = {{&S70567[70], 6}, {NULL}};
const Segment SGI192281[] = {{&S70568[34], 10}, {NULL}};
const Segment SGI192282[] = {{&S70568[27], 6}, {NULL}};
const Segment SGI192283[] = {{&S70568[97], 9}, {NULL}};
const Segment SGI192284[] = {{&S70568[70], 6}, {NULL}};
const Segment SGI192285[] = {{&S70569[34], 11}, {NULL}};
const Segment SGI192286[] = {{&S70569[27], 6}, {NULL}};
const Segment SGI192287[] = {{&S70569[98], 9}, {NULL}};
const Segment SGI192288[] = {{&S70569[71], 6}, {NULL}};
const Segment SGI192289[] = {{&S70572[44], 9}, {NULL}};
const Segment SGI192290[] = {{&S70572[44], 21}, {NULL}};
const Segment SGI192291[] = {{&S70572[30], 6}, {NULL}};
const Segment SGI192292[] = {{&S70572[114], 3}, {NULL}};
const Segment SGI192293[] = {{&S70572[79], 6}, {NULL}};
const Segment SGI192294[] = {{&S77201[34], 8}, {NULL}};
const Segment SGI192295[] = {{&S77201[27], 6}, {NULL}};
const Segment SGI192296[] = {{&S77201[100], 9}, {NULL}};
const Segment SGI192297[] = {{&S77201[73], 6}, {NULL}};
const Segment SGI192298[] = {{&S77204[44], 16}, {NULL}};
const Segment SGI192299[] = {{&S77204[44], 28}, {NULL}};
const Segment SGI192300[] = {{&S77204[30], 6}, {NULL}};
const Segment SGI192301[] = {{&S77204[31], 3}, {NULL}};
const Segment SGI192302[] = {{&S77204[86], 6}, {NULL}};
const Segment SGI192303[] = {{&S64865[34], 13}, {NULL}};
const Segment SGI192304[] = {{&S64865[27], 6}, {NULL}};
const Segment SGI192305[] = {{&S77205[44], 23}, {NULL}};
const Segment SGI192306[] = {{&S77205[44], 35}, {NULL}};
const Segment SGI192307[] = {{&S77205[30], 6}, {NULL}};
const Segment SGI192308[] = {{&S64865[100], 9}, {NULL}};
const Segment SGI192309[] = {{&S77205[31], 3}, {NULL}};
const Segment SGI192310[] = {{&S64865[73], 6}, {NULL}};
const Segment SGI192311[] = {{&S77205[93], 6}, {NULL}};
const Segment SGI192312[] = {{&S64866[34], 19}, {NULL}};
const Segment SGI192313[] = {{&S64866[27], 6}, {NULL}};
const Segment SGI192314[] = {{&S64866[106], 9}, {NULL}};
const Segment SGI192315[] = {{&S64866[79], 6}, {NULL}};
const Segment SGI192316[] = {{&S66110[34], 10}, {NULL}};
const Segment SGI192317[] = {{&S66110[27], 6}, {NULL}};
const Segment SGI192318[] = {{&S66110[97], 9}, {NULL}};
const Segment SGI192319[] = {{&S66110[70], 6}, {NULL}};
const Segment SGI192320[] = {{&S55625[41], 19}, {NULL}};
const Segment SGI192321[] = {{&S55625[41], 30}, {NULL}};
const Segment SGI192322[] = {{&S55625[27], 6}, {NULL}};
const Segment SGI192323[] = {{&S55625[120], 3}, {NULL}};
const Segment SGI192324[] = {{&S55625[124], 9}, {NULL}};
const Segment SGI192325[] = {{&S55625[85], 6}, {NULL}};
const Segment SGI192326[] = {{&S66112[34], 7}, {NULL}};
const Segment SGI192327[] = {{&S66112[27], 6}, {NULL}};
const Segment SGI192328[] = {{&S66112[99], 9}, {NULL}};
const Segment SGI192329[] = {{&S66112[72], 6}, {NULL}};
const Segment SGI192330[] = {{&S66113[34], 15}, {NULL}};
const Segment SGI192331[] = {{&S66113[27], 6}, {NULL}};
const Segment SGI192332[] = {{&S66113[102], 9}, {NULL}};
const Segment SGI192333[] = {{&S66113[75], 6}, {NULL}};
const Segment SGI192334[] = {{&S77226[57], 5}, {NULL}};
const Segment SGI192335[] = {{&S66114[34], 10}, {NULL}};
const Segment SGI192336[] = {{&S77226[57], 21}, {NULL}};
const Segment SGI192337[] = {{&S66114[27], 6}, {NULL}};
const Segment SGI192338[] = {{&S77226[43], 6}, {NULL}};
const Segment SGI192339[] = {{&S77226[44], 3}, {NULL}};
const Segment SGI192340[] = {{&S66114[97], 9}, {NULL}};
const Segment SGI192341[] = {{&S66114[70], 6}, {NULL}};
const Segment SGI192342[] = {{&S77226[92], 6}, {NULL}};
const Segment SGI192343[] = {{&S77226[0], 12}, {NULL}};
const Segment SGI192344[] = {{&S66115[34], 10}, {NULL}};
const Segment SGI192345[] = {{&S66115[27], 6}, {NULL}};
const Segment SGI192346[] = {{&S66115[97], 9}, {NULL}};
const Segment SGI192347[] = {{&S66115[70], 6}, {NULL}};
const Segment SGI192348[] = {{&S64889[44], 10}, {NULL}};
const Segment SGI192349[] = {{&S64889[44], 22}, {NULL}};
const Segment SGI192350[] = {{&S64889[30], 6}, {NULL}};
const Segment SGI192351[] = {{&S64889[136], 3}, {NULL}};
const Segment SGI192352[] = {{&S64889[101], 6}, {NULL}};
const Segment SGI192353[] = {{&S66116[34], 11}, {NULL}};
const Segment SGI192354[] = {{&S64889[68], 8}, {NULL}};
const Segment SGI192355[] = {{&S66116[27], 6}, {NULL}};
const Segment SGI192356[] = {{&S66116[98], 9}, {NULL}};
const Segment SGI192357[] = {{&S66116[71], 6}, {NULL}};
const Segment SGI192358[] = {{&S66117[34], 11}, {NULL}};
const Segment SGI192359[] = {{&S66117[27], 6}, {NULL}};
const Segment SGI192360[] = {{&S64891[44], 10}, {NULL}};
const Segment SGI192361[] = {{&S64891[44], 22}, {NULL}};
const Segment SGI192362[] = {{&S77231[34], 7}, {NULL}};
const Segment SGI192363[] = {{&S64891[30], 6}, {NULL}};
const Segment SGI192364[] = {{&S77231[27], 6}, {NULL}};
const Segment SGI192365[] = {{&S66117[98], 9}, {NULL}};
const Segment SGI192366[] = {{&S66117[71], 6}, {NULL}};
const Segment SGI192367[] = {{&S64891[133], 3}, {NULL}};
const Segment SGI192368[] = {{&S64891[98], 6}, {NULL}};
const Segment SGI192369[] = {{&S77231[99], 9}, {NULL}};
const Segment SGI192370[] = {{&S77231[72], 6}, {NULL}};
const Segment SGI192371[] = {{&S64891[68], 8}, {NULL}};
const Segment SGI192372[] = {{&S66118[34], 11}, {NULL}};
const Segment SGI192373[] = {{&S66118[27], 6}, {NULL}};
const Segment SGI192374[] = {{&S70629[34], 5}, {NULL}};
const Segment SGI192375[] = {{&S70629[27], 6}, {NULL}};
const Segment SGI192376[] = {{&S66118[98], 9}, {NULL}};
const Segment SGI192377[] = {{&S66118[71], 6}, {NULL}};
const Segment SGI192378[] = {{&S70629[92], 9}, {NULL}};
const Segment SGI192379[] = {{&S70629[65], 6}, {NULL}};
const Segment SGI192380[] = {{&S66119[34], 11}, {NULL}};
const Segment SGI192381[] = {{&S66119[27], 6}, {NULL}};
const Segment SGI192382[] = {{&S66119[98], 9}, {NULL}};
const Segment SGI192383[] = {{&S66119[71], 6}, {NULL}};
const Segment SGI192384[] = {{&S70645[44], 6}, {NULL}};
const Segment SGI192385[] = {{&S70645[44], 18}, {NULL}};
const Segment SGI192386[] = {{&S70645[30], 6}, {NULL}};
const Segment SGI192387[] = {{&S70645[127], 3}, {NULL}};
const Segment SGI192388[] = {{&S70645[92], 6}, {NULL}};
const Segment SGI192389[] = {{&S77250[44], 10}, {NULL}};
const Segment SGI192390[] = {{&S77250[44], 22}, {NULL}};
const Segment SGI192391[] = {{&S70645[64], 8}, {NULL}};
const Segment SGI192392[] = {{&S77250[30], 6}, {NULL}};
const Segment SGI192393[] = {{&S77250[80], 6}, {NULL}};
const Segment SGI192394[] = {{&S77250[107], 13}, {NULL}};
const Segment SGI192395[] = {{&S70650[34], 8}, {NULL}};
const Segment SGI192396[] = {{&S70650[27], 6}, {NULL}};
const Segment SGI192397[] = {{&S70650[95], 9}, {NULL}};
const Segment SGI192398[] = {{&S70650[68], 6}, {NULL}};
const Segment SGI192399[] = {{&S66144[44], 10}, {NULL}};
const Segment SGI192400[] = {{&S66144[44], 22}, {NULL}};
const Segment SGI192401[] = {{&S66144[30], 6}, {NULL}};
const Segment SGI192402[] = {{&S66144[80], 6}, {NULL}};
const Segment SGI192403[] = {{&S66144[107], 13}, {NULL}};
const Segment SGI192404[] = {{&S66146[44], 10}, {NULL}};
const Segment SGI192405[] = {{&S66146[44], 22}, {NULL}};
const Segment SGI192406[] = {{&S66146[30], 6}, {NULL}};
const Segment SGI192407[] = {{&S66146[80], 6}, {NULL}};
const Segment SGI192408[] = {{&S66146[107], 13}, {NULL}};
const Segment SGI192409[] = {{&S66148[44], 10}, {NULL}};
const Segment SGI192410[] = {{&S66148[44], 22}, {NULL}};
const Segment SGI192411[] = {{&S66148[30], 6}, {NULL}};
const Segment SGI192412[] = {{&S66148[80], 6}, {NULL}};
const Segment SGI192413[] = {{&S66148[107], 13}, {NULL}};
const Segment SGI192414[] = {{&S66149[34], 8}, {NULL}};
const Segment SGI192415[] = {{&S66149[27], 6}, {NULL}};
const Segment SGI192416[] = {{&S66149[95], 9}, {NULL}};
const Segment SGI192417[] = {{&S66149[68], 6}, {NULL}};
const Segment SGI192418[] = {{&S77273[34], 13}, {NULL}};
const Segment SGI192419[] = {{&S77273[27], 6}, {NULL}};
const Segment SGI192420[] = {{&S77273[100], 9}, {NULL}};
const Segment SGI192421[] = {{&S77273[73], 6}, {NULL}};
const Segment SGI192422[] = {{&S77274[34], 14}, {NULL}};
const Segment SGI192423[] = {{&S77274[27], 6}, {NULL}};
const Segment SGI192424[] = {{&S77274[101], 9}, {NULL}};
const Segment SGI192425[] = {{&S77274[74], 6}, {NULL}};
const Segment SGI192426[] = {{&S64936[44], 10}, {NULL}};
const Segment SGI192427[] = {{&S64936[44], 22}, {NULL}};
const Segment SGI192428[] = {{&S64936[30], 6}, {NULL}};
const Segment SGI192429[] = {{&S70669[34], 6}, {NULL}};
const Segment SGI192430[] = {{&S70669[27], 6}, {NULL}};
const Segment SGI192431[] = {{&S64936[144], 3}, {NULL}};
const Segment SGI192432[] = {{&S64936[109], 6}, {NULL}};
const Segment SGI192433[] = {{&S70669[93], 9}, {NULL}};
const Segment SGI192434[] = {{&S70669[66], 6}, {NULL}};
const Segment SGI192435[] = {{&S77275[34], 8}, {NULL}};
const Segment SGI192436[] = {{&S77275[27], 6}, {NULL}};
const Segment SGI192437[] = {{&S77275[95], 9}, {NULL}};
const Segment SGI192438[] = {{&S77275[68], 6}, {NULL}};
const Segment SGI192439[] = {{&S70670[34], 7}, {NULL}};
const Segment SGI192440[] = {{&S70670[27], 6}, {NULL}};
const Segment SGI192441[] = {{&S66157[44], 8}, {NULL}};
const Segment SGI192442[] = {{&S66157[44], 20}, {NULL}};
const Segment SGI192443[] = {{&S66157[30], 6}, {NULL}};
const Segment SGI192444[] = {{&S70670[94], 9}, {NULL}};
const Segment SGI192445[] = {{&S70670[67], 6}, {NULL}};
const Segment SGI192446[] = {{&S66157[113], 3}, {NULL}};
const Segment SGI192447[] = {{&S66157[78], 6}, {NULL}};
const Segment SGI192448[] = {{&S70671[34], 8}, {NULL}};
const Segment SGI192449[] = {{&S70671[27], 6}, {NULL}};
const Segment SGI192450[] = {{&S66158[44], 7}, {NULL}};
const Segment SGI192451[] = {{&S66158[44], 19}, {NULL}};
const Segment SGI192452[] = {{&S66158[30], 6}, {NULL}};
const Segment SGI192453[] = {{&S70671[95], 9}, {NULL}};
const Segment SGI192454[] = {{&S70671[68], 6}, {NULL}};
const Segment SGI192455[] = {{&S66158[112], 3}, {NULL}};
const Segment SGI192456[] = {{&S66158[77], 6}, {NULL}};
const Segment SGI192457[] = {{&S77281[34], 10}, {NULL}};
const Segment SGI192458[] = {{&S77281[27], 6}, {NULL}};
const Segment SGI192459[] = {{&S77281[97], 9}, {NULL}};
const Segment SGI192460[] = {{&S70674[44], 7}, {NULL}};
const Segment SGI192461[] = {{&S77281[70], 6}, {NULL}};
const Segment SGI192462[] = {{&S70674[44], 19}, {NULL}};
const Segment SGI192463[] = {{&S70674[30], 6}, {NULL}};
const Segment SGI192464[] = {{&S70674[112], 3}, {NULL}};
const Segment SGI192465[] = {{&S70674[77], 6}, {NULL}};
const Segment SGI192466[] = {{&S66162[34], 10}, {NULL}};
const Segment SGI192467[] = {{&S66162[27], 6}, {NULL}};
const Segment SGI192468[] = {{&S66162[97], 9}, {NULL}};
const Segment SGI192469[] = {{&S66162[70], 6}, {NULL}};
const Segment SGI192470[] = {{&S70675[38], 7}, {NULL}};
const Segment SGI192471[] = {{&S70675[31], 6}, {NULL}};
const Segment SGI192472[] = {{&S66168[44], 14}, {NULL}};
const Segment SGI192473[] = {{&S66168[44], 26}, {NULL}};
const Segment SGI192474[] = {{&S66168[30], 6}, {NULL}};
const Segment SGI192475[] = {{&S66168[140], 3}, {NULL}};
const Segment SGI192476[] = {{&S66168[105], 6}, {NULL}};
const Segment SGI192477[] = {{&S66177[34], 11}, {NULL}};
const Segment SGI192478[] = {{&S66177[27], 6}, {NULL}};
const Segment SGI192479[] = {{&S66177[98], 9}, {NULL}};
const Segment SGI192480[] = {{&S66177[71], 6}, {NULL}};
const Segment SGI192481[] = {{&S77304[34], 15}, {NULL}};
const Segment SGI192482[] = {{&S77304[27], 6}, {NULL}};
const Segment SGI192483[] = {{&S77304[102], 9}, {NULL}};
const Segment SGI192484[] = {{&S77304[75], 6}, {NULL}};
const Segment SGI192485[] = {{&S77305[34], 11}, {NULL}};
const Segment SGI192486[] = {{&S77305[27], 6}, {NULL}};
const Segment SGI192487[] = {{&S77305[98], 9}, {NULL}};
const Segment SGI192488[] = {{&S77305[71], 6}, {NULL}};
const Segment SGI192489[] = {{&S77306[34], 14}, {NULL}};
const Segment SGI192490[] = {{&S77306[27], 6}, {NULL}};
const Segment SGI192491[] = {{&S77306[101], 9}, {NULL}};
const Segment SGI192492[] = {{&S77306[74], 6}, {NULL}};
const Segment SGI192493[] = {{&S77307[34], 9}, {NULL}};
const Segment SGI192494[] = {{&S77307[27], 6}, {NULL}};
const Segment SGI192495[] = {{&S77307[96], 9}, {NULL}};
const Segment SGI192496[] = {{&S77307[69], 6}, {NULL}};
const Segment SGI192497[] = {{&S55739[38], 7}, {NULL}};
const Segment SGI192498[] = {{&S64990[34], 10}, {NULL}};
const Segment SGI192499[] = {{&S55739[31], 6}, {NULL}};
const Segment SGI192500[] = {{&S64990[27], 6}, {NULL}};
const Segment SGI192501[] = {{&S64990[97], 9}, {NULL}};
const Segment SGI192502[] = {{&S64990[70], 6}, {NULL}};
const Segment SGI192503[] = {{&S55740[44], 11}, {NULL}};
const Segment SGI192504[] = {{&S55740[44], 29}, {NULL}};
const Segment SGI192505[] = {{&S55740[30], 6}, {NULL}};
const Segment SGI192506[] = {{&S55740[122], 3}, {NULL}};
const Segment SGI192507[] = {{&S55740[87], 6}, {NULL}};
const Segment SGI192508[] = {{&S55741[44], 7}, {NULL}};
const Segment SGI192509[] = {{&S55741[44], 25}, {NULL}};
const Segment SGI192510[] = {{&S55741[30], 6}, {NULL}};
const Segment SGI192511[] = {{&S55741[118], 3}, {NULL}};
const Segment SGI192512[] = {{&S55741[83], 6}, {NULL}};
const Segment SGI192513[] = {{&S55742[44], 7}, {NULL}};
const Segment SGI192514[] = {{&S55742[44], 27}, {NULL}};
const Segment SGI192515[] = {{&S55742[30], 6}, {NULL}};
const Segment SGI192516[] = {{&S55742[120], 3}, {NULL}};
const Segment SGI192517[] = {{&S55742[85], 6}, {NULL}};
const Segment SGI192518[] = {{&S55743[44], 7}, {NULL}};
const Segment SGI192519[] = {{&S55743[44], 26}, {NULL}};
const Segment SGI192520[] = {{&S55743[30], 6}, {NULL}};
const Segment SGI192521[] = {{&S55743[119], 3}, {NULL}};
const Segment SGI192522[] = {{&S55743[84], 6}, {NULL}};
const Segment SGI192523[] = {{&S65003[43], 19}, {NULL}};
const Segment SGI192524[] = {{&S65003[30], 6}, {NULL}};
const Segment SGI192525[] = {{&S65003[31], 3}, {NULL}};
const Segment SGI192526[] = {{&S65003[76], 6}, {NULL}};
const Segment SGI192527[] = {{&S65004[43], 23}, {NULL}};
const Segment SGI192528[] = {{&S65004[30], 6}, {NULL}};
const Segment SGI192529[] = {{&S65004[31], 3}, {NULL}};
const Segment SGI192530[] = {{&S65004[80], 6}, {NULL}};
const Segment SGI192531[] = {{&S66217[34], 7}, {NULL}};
const Segment SGI192532[] = {{&S66217[27], 6}, {NULL}};
const Segment SGI192533[] = {{&S65005[44], 13}, {NULL}};
const Segment SGI192534[] = {{&S65005[44], 25}, {NULL}};
const Segment SGI192535[] = {{&S66217[98], 9}, {NULL}};
const Segment SGI192536[] = {{&S65005[30], 6}, {NULL}};
const Segment SGI192537[] = {{&S66217[71], 6}, {NULL}};
const Segment SGI192538[] = {{&S65005[118], 3}, {NULL}};
const Segment SGI192539[] = {{&S65005[83], 6}, {NULL}};
const Segment SGI192540[] = {{&S65006[44], 17}, {NULL}};
const Segment SGI192541[] = {{&S65006[44], 29}, {NULL}};
const Segment SGI192542[] = {{&S65006[30], 6}, {NULL}};
const Segment SGI192543[] = {{&S65006[122], 3}, {NULL}};
const Segment SGI192544[] = {{&S65006[87], 6}, {NULL}};
const Segment SGI192545[] = {{&S65007[38], 17}, {NULL}};
const Segment SGI192546[] = {{&S65007[31], 6}, {NULL}};
const Segment SGI192547[] = {{&S70749[44], 7}, {NULL}};
const Segment SGI192548[] = {{&S70749[44], 19}, {NULL}};
const Segment SGI192549[] = {{&S70749[30], 6}, {NULL}};
const Segment SGI192550[] = {{&S70749[112], 3}, {NULL}};
const Segment SGI192551[] = {{&S70749[77], 6}, {NULL}};
const Segment SGI192552[] = {{&S70750[44], 7}, {NULL}};
const Segment SGI192553[] = {{&S70750[44], 19}, {NULL}};
const Segment SGI192554[] = {{&S70750[30], 6}, {NULL}};
const Segment SGI192555[] = {{&S70750[112], 3}, {NULL}};
const Segment SGI192556[] = {{&S70750[77], 6}, {NULL}};
const Segment SGI192557[] = {{&S70751[44], 7}, {NULL}};
const Segment SGI192558[] = {{&S70751[44], 19}, {NULL}};
const Segment SGI192559[] = {{&S70751[30], 6}, {NULL}};
const Segment SGI192560[] = {{&S70751[112], 3}, {NULL}};
const Segment SGI192561[] = {{&S70751[77], 6}, {NULL}};
const Segment SGI192562[] = {{&S70759[34], 14}, {NULL}};
const Segment SGI192563[] = {{&S70759[27], 6}, {NULL}};
const Segment SGI192564[] = {{&S70759[101], 9}, {NULL}};
const Segment SGI192565[] = {{&S70759[74], 6}, {NULL}};
const Segment SGI192566[] = {{&S70760[34], 14}, {NULL}};
const Segment SGI192567[] = {{&S70760[27], 6}, {NULL}};
const Segment SGI192568[] = {{&S70760[101], 9}, {NULL}};
const Segment SGI192569[] = {{&S70760[74], 6}, {NULL}};
const Segment SGI192570[] = {{&S70765[52], 10}, {NULL}};
const Segment SGI192571[] = {{&S70765[52], 22}, {NULL}};
const Segment SGI192572[] = {{&S70765[38], 6}, {NULL}};
const Segment SGI192573[] = {{&S70765[88], 5}, {NULL}};
const Segment SGI192574[] = {{&S70765[0], 14}, {NULL}};
const Segment SGI192575[] = {{&S70767[34], 7}, {NULL}};
const Segment SGI192576[] = {{&S70767[27], 6}, {NULL}};
const Segment SGI192577[] = {{&S70767[94], 9}, {NULL}};
const Segment SGI192578[] = {{&S70767[67], 6}, {NULL}};
const Segment SGI192579[] = {{&S70772[34], 25}, {NULL}};
const Segment SGI192580[] = {{&S70772[27], 6}, {NULL}};
const Segment SGI192581[] = {{&S70772[112], 9}, {NULL}};
const Segment SGI192582[] = {{&S70772[85], 6}, {NULL}};
const Segment SGI192583[] = {{&S66257[34], 10}, {NULL}};
const Segment SGI192584[] = {{&S66257[27], 6}, {NULL}};
const Segment SGI192585[] = {{&S66257[97], 9}, {NULL}};
const Segment SGI192586[] = {{&S66257[70], 6}, {NULL}};
const Segment SGI192587[] = {{&S66258[34], 10}, {NULL}};
const Segment SGI192588[] = {{&S66258[27], 6}, {NULL}};
const Segment SGI192589[] = {{&S66258[97], 9}, {NULL}};
const Segment SGI192590[] = {{&S66258[70], 6}, {NULL}};
const Segment SGI192591[] = {{&S55796[34], 10}, {NULL}};
const Segment SGI192592[] = {{&S55796[27], 6}, {NULL}};
const Segment SGI192593[] = {{&S55796[96], 9}, {NULL}};
const Segment SGI192594[] = {{&S55796[69], 6}, {NULL}};
const Segment SGI192595[] = {{&S66259[34], 9}, {NULL}};
const Segment SGI192596[] = {{&S66259[27], 6}, {NULL}};
const Segment SGI192597[] = {{&S66259[96], 9}, {NULL}};
const Segment SGI192598[] = {{&S66259[69], 6}, {NULL}};
const Segment SGI192599[] = {{&S70805[44], 8}, {NULL}};
const Segment SGI192600[] = {{&S70805[44], 20}, {NULL}};
const Segment SGI192601[] = {{&S70805[30], 6}, {NULL}};
const Segment SGI192602[] = {{&S70805[113], 3}, {NULL}};
const Segment SGI192603[] = {{&S70805[78], 6}, {NULL}};
const Segment SGI192604[] = {{&S66287[45], 10}, {NULL}};
const Segment SGI192605[] = {{&S66287[30], 6}, {NULL}};
const Segment SGI192606[] = {{&S66287[115], 3}, {NULL}};
const Segment SGI192607[] = {{&S66287[81], 5}, {NULL}};
const Segment SGI192608[] = {{&S66287[139], 13}, {NULL}};
const Segment SGI192609[] = {{&S66288[44], 10}, {NULL}};
const Segment SGI192610[] = {{&S66288[44], 22}, {NULL}};
const Segment SGI192611[] = {{&S66288[30], 6}, {NULL}};
const Segment SGI192612[] = {{&S66288[115], 3}, {NULL}};
const Segment SGI192613[] = {{&S66288[80], 6}, {NULL}};
const Segment SGI192614[] = {{&S66288[135], 8}, {NULL}};
const Segment SGI192615[] = {{&S66288[144], 13}, {NULL}};
const Segment SGI192616[] = {{&S74550[42], 10}, {NULL}};
const Segment SGI192617[] = {{&S74550[42], 22}, {NULL}};
const Segment SGI192618[] = {{&S74550[30], 5}, {NULL}};
const Segment SGI192619[] = {{&S74550[113], 3}, {NULL}};
const Segment SGI192620[] = {{&S74550[78], 6}, {NULL}};
const Segment SGI192621[] = {{&S66319[34], 10}, {NULL}};
const Segment SGI192622[] = {{&S66319[27], 6}, {NULL}};
const Segment SGI192623[] = {{&S66319[97], 9}, {NULL}};
const Segment SGI192624[] = {{&S66319[70], 6}, {NULL}};
const Segment SGI192625[] = {{&S66319[132], 8}, {NULL}};
const Segment SGI192626[] = {{&S66321[34], 10}, {NULL}};
const Segment SGI192627[] = {{&S66321[27], 6}, {NULL}};
const Segment SGI192628[] = {{&S66321[97], 9}, {NULL}};
const Segment SGI192629[] = {{&S66321[70], 6}, {NULL}};
const Segment SGI192630[] = {{&S66321[131], 8}, {NULL}};
const Segment SGI192631[] = {{&S69353[42], 10}, {NULL}};
const Segment SGI192632[] = {{&S69353[42], 26}, {NULL}};
const Segment SGI192633[] = {{&S69353[30], 4}, {NULL}};
const Segment SGI192634[] = {{&S69353[116], 3}, {NULL}};
const Segment SGI192635[] = {{&S69353[82], 5}, {NULL}};
const Segment SGI192636[] = {{&S77332[34], 10}, {NULL}};
const Segment SGI192637[] = {{&S77332[27], 6}, {NULL}};
const Segment SGI192638[] = {{&S77332[97], 9}, {NULL}};
const Segment SGI192639[] = {{&S77332[70], 6}, {NULL}};
const Segment SGI192640[] = {{&S77333[44], 14}, {NULL}};
const Segment SGI192641[] = {{&S77333[44], 26}, {NULL}};
const Segment SGI192642[] = {{&S77333[30], 6}, {NULL}};
const Segment SGI192643[] = {{&S77333[31], 3}, {NULL}};
const Segment SGI192644[] = {{&S77333[84], 6}, {NULL}};
const Segment SGI192645[] = {{&S77335[34], 13}, {NULL}};
const Segment SGI192646[] = {{&S77335[27], 6}, {NULL}};
const Segment SGI192647[] = {{&S66330[34], 10}, {NULL}};
const Segment SGI192648[] = {{&S77335[100], 9}, {NULL}};
const Segment SGI192649[] = {{&S77335[73], 6}, {NULL}};
const Segment SGI192650[] = {{&S66330[27], 6}, {NULL}};
const Segment SGI192651[] = {{&S66330[97], 9}, {NULL}};
const Segment SGI192652[] = {{&S66330[70], 6}, {NULL}};
const Segment SGI192653[] = {{&S66337[34], 9}, {NULL}};
const Segment SGI192654[] = {{&S66337[27], 6}, {NULL}};
const Segment SGI192655[] = {{&S66337[96], 9}, {NULL}};
const Segment SGI192656[] = {{&S66337[69], 6}, {NULL}};
const Segment SGI192657[] = {{&S55908[34], 4}, {NULL}};
const Segment SGI192658[] = {{&S55908[27], 6}, {NULL}};
const Segment SGI192659[] = {{&S55908[90], 9}, {NULL}};
const Segment SGI192660[] = {{&S55908[63], 6}, {NULL}};
const Segment SGI192661[] = {{&S55909[34], 5}, {NULL}};
const Segment SGI192662[] = {{&S55909[27], 6}, {NULL}};
const Segment SGI192663[] = {{&S55909[91], 9}, {NULL}};
const Segment SGI192664[] = {{&S55909[64], 6}, {NULL}};
const Segment SGI192665[] = {{&S66359[44], 10}, {NULL}};
const Segment SGI192666[] = {{&S66359[44], 22}, {NULL}};
const Segment SGI192667[] = {{&S66359[30], 6}, {NULL}};
const Segment SGI192668[] = {{&S66359[131], 3}, {NULL}};
const Segment SGI192669[] = {{&S65154[34], 10}, {NULL}};
const Segment SGI192670[] = {{&S66359[96], 6}, {NULL}};
const Segment SGI192671[] = {{&S65154[27], 6}, {NULL}};
const Segment SGI192672[] = {{&S66359[68], 8}, {NULL}};
const Segment SGI192673[] = {{&S65154[97], 9}, {NULL}};
const Segment SGI192674[] = {{&S65154[70], 6}, {NULL}};
const Segment SGI192675[] = {{&S55913[34], 13}, {NULL}};
const Segment SGI192676[] = {{&S55913[27], 6}, {NULL}};
const Segment SGI192677[] = {{&S55913[99], 9}, {NULL}};
const Segment SGI192678[] = {{&S55913[72], 6}, {NULL}};
const Segment SGI192679[] = {{&S70870[44], 8}, {NULL}};
const Segment SGI192680[] = {{&S70870[44], 20}, {NULL}};
const Segment SGI192681[] = {{&S70870[30], 6}, {NULL}};
const Segment SGI192682[] = {{&S70870[113], 3}, {NULL}};
const Segment SGI192683[] = {{&S70870[78], 6}, {NULL}};
const Segment SGI192684[] = {{&S55922[34], 19}, {NULL}};
const Segment SGI192685[] = {{&S55922[27], 6}, {NULL}};
const Segment SGI192686[] = {{&S55922[105], 9}, {NULL}};
const Segment SGI192687[] = {{&S55922[78], 6}, {NULL}};
const Segment SGI192688[] = {{&S55923[34], 10}, {NULL}};
const Segment SGI192689[] = {{&S55923[27], 6}, {NULL}};
const Segment SGI192690[] = {{&S55923[96], 9}, {NULL}};
const Segment SGI192691[] = {{&S55923[69], 6}, {NULL}};
const Segment SGI192692[] = {{&S66375[44], 24}, {NULL}};
const Segment SGI192693[] = {{&S66375[44], 36}, {NULL}};
const Segment SGI192694[] = {{&S66375[30], 6}, {NULL}};
const Segment SGI192695[] = {{&S66375[145], 3}, {NULL}};
const Segment SGI192696[] = {{&S66375[110], 6}, {NULL}};
const Segment SGI192697[] = {{&S66375[82], 8}, {NULL}};
const Segment SGI192698[] = {{&S55924[34], 12}, {NULL}};
const Segment SGI192699[] = {{&S55924[27], 6}, {NULL}};
const Segment SGI192700[] = {{&S55924[98], 9}, {NULL}};
const Segment SGI192701[] = {{&S55924[71], 6}, {NULL}};
const Segment SGI192702[] = {{&S65172[34], 6}, {NULL}};
const Segment SGI192703[] = {{&S65172[27], 6}, {NULL}};
const Segment SGI192704[] = {{&S65172[93], 9}, {NULL}};
const Segment SGI192705[] = {{&S65172[66], 6}, {NULL}};
const Segment SGI192706[] = {{&S66379[34], 18}, {NULL}};
const Segment SGI192707[] = {{&S66379[27], 6}, {NULL}};
const Segment SGI192708[] = {{&S66379[105], 9}, {NULL}};
const Segment SGI192709[] = {{&S66379[78], 6}, {NULL}};
const Segment SGI192710[] = {{&S55928[34], 6}, {NULL}};
const Segment SGI192711[] = {{&S55928[27], 6}, {NULL}};
const Segment SGI192712[] = {{&S55928[92], 9}, {NULL}};
const Segment SGI192713[] = {{&S55928[65], 6}, {NULL}};
const Segment SGI192714[] = {{&S66380[34], 10}, {NULL}};
const Segment SGI192715[] = {{&S66380[27], 6}, {NULL}};
const Segment SGI192716[] = {{&S66380[97], 9}, {NULL}};
const Segment SGI192717[] = {{&S66380[70], 6}, {NULL}};
const Segment SGI192718[] = {{&S66381[34], 18}, {NULL}};
const Segment SGI192719[] = {{&S66381[27], 6}, {NULL}};
const Segment SGI192720[] = {{&S66381[105], 9}, {NULL}};
const Segment SGI192721[] = {{&S66381[78], 6}, {NULL}};
const Segment SGI192722[] = {{&S55932[34], 6}, {NULL}};
const Segment SGI192723[] = {{&S66382[34], 10}, {NULL}};
const Segment SGI192724[] = {{&S55932[27], 6}, {NULL}};
const Segment SGI192725[] = {{&S66382[27], 6}, {NULL}};
const Segment SGI192726[] = {{&S55932[93], 9}, {NULL}};
const Segment SGI192727[] = {{&S55932[66], 6}, {NULL}};
const Segment SGI192728[] = {{&S66382[97], 9}, {NULL}};
const Segment SGI192729[] = {{&S66382[70], 6}, {NULL}};
const Segment SGI192730[] = {{&S55933[34], 10}, {NULL}};
const Segment SGI192731[] = {{&S55933[27], 6}, {NULL}};
const Segment SGI192732[] = {{&S66383[34], 9}, {NULL}};
const Segment SGI192733[] = {{&S66383[27], 6}, {NULL}};
const Segment SGI192734[] = {{&S55933[96], 9}, {NULL}};
const Segment SGI192735[] = {{&S55933[69], 6}, {NULL}};
const Segment SGI192736[] = {{&S66383[96], 9}, {NULL}};
const Segment SGI192737[] = {{&S66383[69], 6}, {NULL}};
const Segment SGI192738[] = {{&S66384[34], 7}, {NULL}};
const Segment SGI192739[] = {{&S66384[27], 6}, {NULL}};
const Segment SGI192740[] = {{&S66384[100], 9}, {NULL}};
const Segment SGI192741[] = {{&S66384[73], 6}, {NULL}};
const Segment SGI192742[] = {{&S70894[34], 14}, {NULL}};
const Segment SGI192743[] = {{&S70894[27], 6}, {NULL}};
const Segment SGI192744[] = {{&S70894[101], 9}, {NULL}};
const Segment SGI192745[] = {{&S70894[74], 6}, {NULL}};
const Segment SGI192746[] = {{&S55939[34], 12}, {NULL}};
const Segment SGI192747[] = {{&S55939[27], 6}, {NULL}};
const Segment SGI192748[] = {{&S55939[98], 9}, {NULL}};
const Segment SGI192749[] = {{&S55939[71], 6}, {NULL}};
const Segment SGI192750[] = {{&S55942[34], 10}, {NULL}};
const Segment SGI192751[] = {{&S55942[27], 6}, {NULL}};
const Segment SGI192752[] = {{&S55942[96], 9}, {NULL}};
const Segment SGI192753[] = {{&S55942[69], 6}, {NULL}};
const Segment SGI192754[] = {{&S55943[34], 6}, {NULL}};
const Segment SGI192755[] = {{&S55943[27], 6}, {NULL}};
const Segment SGI192756[] = {{&S55943[92], 9}, {NULL}};
const Segment SGI192757[] = {{&S55943[65], 6}, {NULL}};
const Segment SGI192758[] = {{&S55944[34], 4}, {NULL}};
const Segment SGI192759[] = {{&S55944[27], 6}, {NULL}};
const Segment SGI192760[] = {{&S55944[91], 9}, {NULL}};
const Segment SGI192761[] = {{&S55944[64], 6}, {NULL}};
const Segment SGI192762[] = {{&S70904[34], 15}, {NULL}};
const Segment SGI192763[] = {{&S70904[27], 6}, {NULL}};
const Segment SGI192764[] = {{&S55946[34], 6}, {NULL}};
const Segment SGI192765[] = {{&S70904[102], 9}, {NULL}};
const Segment SGI192766[] = {{&S70904[75], 6}, {NULL}};
const Segment SGI192767[] = {{&S55946[27], 6}, {NULL}};
const Segment SGI192768[] = {{&S55946[92], 9}, {NULL}};
const Segment SGI192769[] = {{&S55946[65], 6}, {NULL}};
const Segment SGI192770[] = {{&S70905[34], 15}, {NULL}};
const Segment SGI192771[] = {{&S70905[27], 6}, {NULL}};
const Segment SGI192772[] = {{&S55947[34], 4}, {NULL}};
const Segment SGI192773[] = {{&S70905[102], 9}, {NULL}};
const Segment SGI192774[] = {{&S55947[27], 6}, {NULL}};
const Segment SGI192775[] = {{&S70905[75], 6}, {NULL}};
const Segment SGI192776[] = {{&S55947[90], 9}, {NULL}};
const Segment SGI192777[] = {{&S55947[63], 6}, {NULL}};
const Segment SGI192778[] = {{&S55948[34], 12}, {NULL}};
const Segment SGI192779[] = {{&S55948[27], 6}, {NULL}};
const Segment SGI192780[] = {{&S55948[98], 9}, {NULL}};
const Segment SGI192781[] = {{&S55948[71], 6}, {NULL}};
const Segment SGI192782[] = {{&S70908[44], 4}, {NULL}};
const Segment SGI192783[] = {{&S70908[44], 16}, {NULL}};
const Segment SGI192784[] = {{&S70908[30], 6}, {NULL}};
const Segment SGI192785[] = {{&S70908[109], 3}, {NULL}};
const Segment SGI192786[] = {{&S70908[74], 6}, {NULL}};
const Segment SGI192787[] = {{&S70909[38], 4}, {NULL}};
const Segment SGI192788[] = {{&S70909[31], 6}, {NULL}};
const Segment SGI192789[] = {{&S70931[44], 9}, {NULL}};
const Segment SGI192790[] = {{&S70931[44], 21}, {NULL}};
const Segment SGI192791[] = {{&S70931[30], 6}, {NULL}};
const Segment SGI192792[] = {{&S70931[114], 3}, {NULL}};
const Segment SGI192793[] = {{&S70931[79], 6}, {NULL}};
const Segment SGI192794[] = {{&S70932[44], 9}, {NULL}};
const Segment SGI192795[] = {{&S70932[44], 21}, {NULL}};
const Segment SGI192796[] = {{&S70932[30], 6}, {NULL}};
const Segment SGI192797[] = {{&S70932[114], 3}, {NULL}};
const Segment SGI192798[] = {{&S70932[79], 6}, {NULL}};
const Segment SGI192799[] = {{&S70935[44], 5}, {NULL}};
const Segment SGI192800[] = {{&S70935[44], 17}, {NULL}};
const Segment SGI192801[] = {{&S70935[30], 6}, {NULL}};
const Segment SGI192802[] = {{&S65240[34], 10}, {NULL}};
const Segment SGI192803[] = {{&S70935[110], 3}, {NULL}};
const Segment SGI192804[] = {{&S65240[27], 6}, {NULL}};
const Segment SGI192805[] = {{&S70935[75], 6}, {NULL}};
const Segment SGI192806[] = {{&S65240[97], 9}, {NULL}};
const Segment SGI192807[] = {{&S65240[70], 6}, {NULL}};
const Segment SGI192808[] = {{&S70936[44], 14}, {NULL}};
const Segment SGI192809[] = {{&S70936[44], 26}, {NULL}};
const Segment SGI192810[] = {{&S70936[30], 6}, {NULL}};
const Segment SGI192811[] = {{&S70936[119], 3}, {NULL}};
const Segment SGI192812[] = {{&S70936[84], 6}, {NULL}};
const Segment SGI192813[] = {{&S70937[44], 10}, {NULL}};
const Segment SGI192814[] = {{&S70937[44], 22}, {NULL}};
const Segment SGI192815[] = {{&S70937[30], 6}, {NULL}};
const Segment SGI192816[] = {{&S70937[115], 3}, {NULL}};
const Segment SGI192817[] = {{&S70937[80], 6}, {NULL}};
const Segment SGI192818[] = {{&S70948[34], 25}, {NULL}};
const Segment SGI192819[] = {{&S55990[44], 10}, {NULL}};
const Segment SGI192820[] = {{&S55990[44], 21}, {NULL}};
const Segment SGI192821[] = {{&S70948[27], 6}, {NULL}};
const Segment SGI192822[] = {{&S55990[30], 6}, {NULL}};
const Segment SGI192823[] = {{&S55990[132], 3}, {NULL}};
const Segment SGI192824[] = {{&S70948[112], 9}, {NULL}};
const Segment SGI192825[] = {{&S70948[85], 6}, {NULL}};
const Segment SGI192826[] = {{&S55990[97], 6}, {NULL}};
const Segment SGI192827[] = {{&S55990[67], 8}, {NULL}};
const Segment SGI192828[] = {{&S55992[44], 10}, {NULL}};
const Segment SGI192829[] = {{&S55992[44], 22}, {NULL}};
const Segment SGI192830[] = {{&S55992[30], 6}, {NULL}};
const Segment SGI192831[] = {{&S66462[34], 14}, {NULL}};
const Segment SGI192832[] = {{&S66462[27], 6}, {NULL}};
const Segment SGI192833[] = {{&S55992[133], 3}, {NULL}};
const Segment SGI192834[] = {{&S55992[98], 6}, {NULL}};
const Segment SGI192835[] = {{&S66462[101], 9}, {NULL}};
const Segment SGI192836[] = {{&S55992[68], 8}, {NULL}};
const Segment SGI192837[] = {{&S66462[74], 6}, {NULL}};
const Segment SGI192838[] = {{&S55995[44], 6}, {NULL}};
const Segment SGI192839[] = {{&S55995[44], 17}, {NULL}};
const Segment SGI192840[] = {{&S55995[30], 6}, {NULL}};
const Segment SGI192841[] = {{&S55995[110], 3}, {NULL}};
const Segment SGI192842[] = {{&S55995[75], 6}, {NULL}};
const Segment SGI192843[] = {{&S66466[44], 8}, {NULL}};
const Segment SGI192844[] = {{&S66466[44], 20}, {NULL}};
const Segment SGI192845[] = {{&S66466[30], 6}, {NULL}};
const Segment SGI192846[] = {{&S66466[113], 3}, {NULL}};
const Segment SGI192847[] = {{&S66466[78], 6}, {NULL}};
const Segment SGI192848[] = {{&S66467[38], 8}, {NULL}};
const Segment SGI192849[] = {{&S66467[31], 6}, {NULL}};
const Segment SGI192850[] = {{&S55997[38], 6}, {NULL}};
const Segment SGI192851[] = {{&S55997[31], 6}, {NULL}};
const Segment SGI192852[] = {{&S65276[44], 12}, {NULL}};
const Segment SGI192853[] = {{&S65276[44], 24}, {NULL}};
const Segment SGI192854[] = {{&S65276[30], 6}, {NULL}};
const Segment SGI192855[] = {{&S65276[117], 3}, {NULL}};
const Segment SGI192856[] = {{&S65276[82], 6}, {NULL}};
const Segment SGI192857[] = {{&S66472[34], 7}, {NULL}};
const Segment SGI192858[] = {{&S66472[27], 6}, {NULL}};
const Segment SGI192859[] = {{&S66472[99], 9}, {NULL}};
const Segment SGI192860[] = {{&S66472[72], 6}, {NULL}};
const Segment SGI192861[] = {{&S66473[34], 7}, {NULL}};
const Segment SGI192862[] = {{&S77348[34], 10}, {NULL}};
const Segment SGI192863[] = {{&S66473[27], 6}, {NULL}};
const Segment SGI192864[] = {{&S77348[27], 6}, {NULL}};
const Segment SGI192865[] = {{&S66473[99], 9}, {NULL}};
const Segment SGI192866[] = {{&S66473[72], 6}, {NULL}};
const Segment SGI192867[] = {{&S77348[97], 9}, {NULL}};
const Segment SGI192868[] = {{&S77348[70], 6}, {NULL}};
const Segment SGI192869[] = {{&S77352[34], 11}, {NULL}};
const Segment SGI192870[] = {{&S77352[27], 6}, {NULL}};
const Segment SGI192871[] = {{&S77352[98], 9}, {NULL}};
const Segment SGI192872[] = {{&S77352[71], 6}, {NULL}};
const Segment SGI192873[] = {{&S77353[34], 11}, {NULL}};
const Segment SGI192874[] = {{&S77353[27], 6}, {NULL}};
const Segment SGI192875[] = {{&S77353[98], 9}, {NULL}};
const Segment SGI192876[] = {{&S77353[71], 6}, {NULL}};
const Segment SGI192877[] = {{&S77354[34], 15}, {NULL}};
const Segment SGI192878[] = {{&S77354[27], 6}, {NULL}};
const Segment SGI192879[] = {{&S77354[102], 9}, {NULL}};
const Segment SGI192880[] = {{&S77354[75], 6}, {NULL}};
const Segment SGI192881[] = {{&S77355[34], 15}, {NULL}};
const Segment SGI192882[] = {{&S77355[27], 6}, {NULL}};
const Segment SGI192883[] = {{&S77355[102], 9}, {NULL}};
const Segment SGI192884[] = {{&S77355[75], 6}, {NULL}};
const Segment SGI192885[] = {{&S77356[34], 11}, {NULL}};
const Segment SGI192886[] = {{&S77356[27], 6}, {NULL}};
const Segment SGI192887[] = {{&S77356[98], 9}, {NULL}};
const Segment SGI192888[] = {{&S77356[71], 6}, {NULL}};
const Segment SGI192889[] = {{&S66496[34], 7}, {NULL}};
const Segment SGI192890[] = {{&S66496[27], 6}, {NULL}};
const Segment SGI192891[] = {{&S66496[94], 9}, {NULL}};
const Segment SGI192892[] = {{&S66496[67], 6}, {NULL}};
const Segment SGI192893[] = {{&S65302[34], 10}, {NULL}};
const Segment SGI192894[] = {{&S65302[27], 6}, {NULL}};
const Segment SGI192895[] = {{&S65302[97], 9}, {NULL}};
const Segment SGI192896[] = {{&S65302[70], 6}, {NULL}};
const Segment SGI192897[] = {{&S66511[44], 11}, {NULL}};
const Segment SGI192898[] = {{&S66511[44], 23}, {NULL}};
const Segment SGI192899[] = {{&S66511[30], 6}, {NULL}};
const Segment SGI192900[] = {{&S66511[116], 3}, {NULL}};
const Segment SGI192901[] = {{&S66511[81], 6}, {NULL}};
const Segment SGI192902[] = {{&S66512[34], 7}, {NULL}};
const Segment SGI192903[] = {{&S66512[27], 6}, {NULL}};
const Segment SGI192904[] = {{&S66512[102], 9}, {NULL}};
const Segment SGI192905[] = {{&S66512[75], 6}, {NULL}};
const Segment SGI192906[] = {{&S77392[44], 10}, {NULL}};
const Segment SGI192907[] = {{&S77392[44], 22}, {NULL}};
const Segment SGI192908[] = {{&S77392[30], 6}, {NULL}};
const Segment SGI192909[] = {{&S77392[31], 3}, {NULL}};
const Segment SGI192910[] = {{&S77392[80], 6}, {NULL}};
const Segment SGI192911[] = {{&S77393[44], 10}, {NULL}};
const Segment SGI192912[] = {{&S77393[44], 22}, {NULL}};
const Segment SGI192913[] = {{&S77393[30], 6}, {NULL}};
const Segment SGI192914[] = {{&S77393[83], 6}, {NULL}};
const Segment SGI192915[] = {{&S66527[44], 7}, {NULL}};
const Segment SGI192916[] = {{&S66527[44], 19}, {NULL}};
const Segment SGI192917[] = {{&S66527[30], 6}, {NULL}};
const Segment SGI192918[] = {{&S66527[112], 3}, {NULL}};
const Segment SGI192919[] = {{&S66527[77], 6}, {NULL}};
const Segment SGI192920[] = {{&S56058[34], 7}, {NULL}};
const Segment SGI192921[] = {{&S56058[27], 6}, {NULL}};
const Segment SGI192922[] = {{&S77394[44], 10}, {NULL}};
const Segment SGI192923[] = {{&S77394[44], 22}, {NULL}};
const Segment SGI192924[] = {{&S77394[30], 6}, {NULL}};
const Segment SGI192925[] = {{&S56058[100], 9}, {NULL}};
const Segment SGI192926[] = {{&S56058[73], 6}, {NULL}};
const Segment SGI192927[] = {{&S77394[83], 6}, {NULL}};
const Segment SGI192928[] = {{&S66528[38], 7}, {NULL}};
const Segment SGI192929[] = {{&S66528[31], 6}, {NULL}};
const Segment SGI192930[] = {{&S77395[44], 10}, {NULL}};
const Segment SGI192931[] = {{&S77395[44], 22}, {NULL}};
const Segment SGI192932[] = {{&S77395[30], 6}, {NULL}};
const Segment SGI192933[] = {{&S66529[44], 8}, {NULL}};
const Segment SGI192934[] = {{&S66529[44], 20}, {NULL}};
const Segment SGI192935[] = {{&S66529[30], 6}, {NULL}};
const Segment SGI192936[] = {{&S77395[31], 3}, {NULL}};
const Segment SGI192937[] = {{&S65338[44], 10}, {NULL}};
const Segment SGI192938[] = {{&S66529[113], 3}, {NULL}};
const Segment SGI192939[] = {{&S77395[80], 6}, {NULL}};
const Segment SGI192940[] = {{&S65338[44], 22}, {NULL}};
const Segment SGI192941[] = {{&S65338[30], 6}, {NULL}};
const Segment SGI192942[] = {{&S66529[78], 6}, {NULL}};
const Segment SGI192943[] = {{&S65338[115], 3}, {NULL}};
const Segment SGI192944[] = {{&S65338[80], 6}, {NULL}};
const Segment SGI192945[] = {{&S77396[38], 10}, {NULL}};
const Segment SGI192946[] = {{&S77396[31], 6}, {NULL}};
const Segment SGI192947[] = {{&S65340[44], 10}, {NULL}};
const Segment SGI192948[] = {{&S65340[44], 22}, {NULL}};
const Segment SGI192949[] = {{&S65340[30], 6}, {NULL}};
const Segment SGI192950[] = {{&S65340[115], 3}, {NULL}};
const Segment SGI192951[] = {{&S65340[80], 6}, {NULL}};
const Segment SGI192952[] = {{&S77331[44], 7}, {NULL}};
const Segment SGI192953[] = {{&S77331[44], 19}, {NULL}};
const Segment SGI192954[] = {{&S77331[30], 6}, {NULL}};
const Segment SGI192955[] = {{&S77331[31], 3}, {NULL}};
const Segment SGI192956[] = {{&S77331[77], 6}, {NULL}};
const Segment SGI192957[] = {{&S77422[34], 17}, {NULL}};
const Segment SGI192958[] = {{&S77422[27], 6}, {NULL}};
const Segment SGI192959[] = {{&S77422[104], 9}, {NULL}};
const Segment SGI192960[] = {{&S77422[77], 6}, {NULL}};
const Segment SGI192961[] = {{&S77425[44], 8}, {NULL}};
const Segment SGI192962[] = {{&S77425[44], 20}, {NULL}};
const Segment SGI192963[] = {{&S77425[30], 6}, {NULL}};
const Segment SGI192964[] = {{&S77425[31], 3}, {NULL}};
const Segment SGI192965[] = {{&S77425[78], 6}, {NULL}};
const Segment SGI192966[] = {{&S70952[44], 11}, {NULL}};
const Segment SGI192967[] = {{&S70952[44], 23}, {NULL}};
const Segment SGI192968[] = {{&S70952[30], 6}, {NULL}};
const Segment SGI192969[] = {{&S70952[116], 3}, {NULL}};
const Segment SGI192970[] = {{&S70952[81], 6}, {NULL}};
const Segment SGI192971[] = {{&S65383[34], 13}, {NULL}};
const Segment SGI192972[] = {{&S65383[27], 6}, {NULL}};
const Segment SGI192973[] = {{&S65383[100], 9}, {NULL}};
const Segment SGI192974[] = {{&S65383[73], 6}, {NULL}};
const Segment SGI192975[] = {{&S77448[34], 11}, {NULL}};
const Segment SGI192976[] = {{&S77448[27], 6}, {NULL}};
const Segment SGI192977[] = {{&S77448[98], 9}, {NULL}};
const Segment SGI192978[] = {{&S77448[71], 6}, {NULL}};
const Segment SGI192979[] = {{&S77448[139], 8}, {NULL}};
const Segment SGI192980[] = {{&S66599[34], 10}, {NULL}};
const Segment SGI192981[] = {{&S66599[27], 6}, {NULL}};
const Segment SGI192982[] = {{&S66599[97], 9}, {NULL}};
const Segment SGI192983[] = {{&S66599[70], 6}, {NULL}};
const Segment SGI192984[] = {{&S66600[34], 10}, {NULL}};
const Segment SGI192985[] = {{&S66600[27], 6}, {NULL}};
const Segment SGI192986[] = {{&S66600[97], 9}, {NULL}};
const Segment SGI192987[] = {{&S66600[70], 6}, {NULL}};
const Segment SGI192988[] = {{&S70991[34], 6}, {NULL}};
const Segment SGI192989[] = {{&S70991[27], 6}, {NULL}};
const Segment SGI192990[] = {{&S70991[93], 9}, {NULL}};
const Segment SGI192991[] = {{&S70991[66], 6}, {NULL}};
const Segment SGI192992[] = {{&S70992[34], 6}, {NULL}};
const Segment SGI192993[] = {{&S70992[27], 6}, {NULL}};
const Segment SGI192994[] = {{&S70992[93], 9}, {NULL}};
const Segment SGI192995[] = {{&S70992[66], 6}, {NULL}};
const Segment SGI192996[] = {{&S77471[44], 18}, {NULL}};
const Segment SGI192997[] = {{&S77471[44], 30}, {NULL}};
const Segment SGI192998[] = {{&S77471[30], 6}, {NULL}};
const Segment SGI192999[] = {{&S77471[31], 3}, {NULL}};
const Segment SGI193000[] = {{&S77471[88], 6}, {NULL}};
const Segment SGI193001[] = {{&S77472[44], 18}, {NULL}};
const Segment SGI193002[] = {{&S77472[44], 30}, {NULL}};
const Segment SGI193003[] = {{&S77472[30], 6}, {NULL}};
const Segment SGI193004[] = {{&S70996[44], 18}, {NULL}};
const Segment SGI193005[] = {{&S70996[44], 30}, {NULL}};
const Segment SGI193006[] = {{&S70996[30], 6}, {NULL}};
const Segment SGI193007[] = {{&S77472[31], 3}, {NULL}};
const Segment SGI193008[] = {{&S77472[88], 6}, {NULL}};
const Segment SGI193009[] = {{&S70996[123], 3}, {NULL}};
const Segment SGI193010[] = {{&S70996[88], 6}, {NULL}};
const Segment SGI193011[] = {{&S70997[44], 18}, {NULL}};
const Segment SGI193012[] = {{&S70997[44], 30}, {NULL}};
const Segment SGI193013[] = {{&S70997[30], 6}, {NULL}};
const Segment SGI193014[] = {{&S70997[123], 3}, {NULL}};
const Segment SGI193015[] = {{&S70997[88], 6}, {NULL}};
const Segment SGI193016[] = {{&S66623[34], 7}, {NULL}};
const Segment SGI193017[] = {{&S77477[44], 10}, {NULL}};
const Segment SGI193018[] = {{&S66623[27], 6}, {NULL}};
const Segment SGI193019[] = {{&S77477[44], 22}, {NULL}};
const Segment SGI193020[] = {{&S77477[30], 6}, {NULL}};
const Segment SGI193021[] = {{&S70999[44], 5}, {NULL}};
const Segment SGI193022[] = {{&S70999[44], 17}, {NULL}};
const Segment SGI193023[] = {{&S70999[30], 6}, {NULL}};
const Segment SGI193024[] = {{&S77477[31], 3}, {NULL}};
const Segment SGI193025[] = {{&S66623[100], 9}, {NULL}};
const Segment SGI193026[] = {{&S66623[73], 6}, {NULL}};
const Segment SGI193027[] = {{&S77477[80], 6}, {NULL}};
const Segment SGI193028[] = {{&S70999[110], 3}, {NULL}};
const Segment SGI193029[] = {{&S70999[75], 6}, {NULL}};
const Segment SGI193030[] = {{&S77477[140], 7}, {NULL}};
const Segment SGI193031[] = {{&S77481[44], 10}, {NULL}};
const Segment SGI193032[] = {{&S77481[44], 20}, {NULL}};
const Segment SGI193033[] = {{&S77481[30], 6}, {NULL}};
const Segment SGI193034[] = {{&S77481[31], 3}, {NULL}};
const Segment SGI193035[] = {{&S77481[78], 6}, {NULL}};
const Segment SGI193036[] = {{&S77482[38], 10}, {NULL}};
const Segment SGI193037[] = {{&S77482[31], 6}, {NULL}};
const Segment SGI193038[] = {{&S71006[44], 10}, {NULL}};
const Segment SGI193039[] = {{&S77483[42], 10}, {NULL}};
const Segment SGI193040[] = {{&S71006[44], 22}, {NULL}};
const Segment SGI193041[] = {{&S71006[30], 6}, {NULL}};
const Segment SGI193042[] = {{&S77483[30], 6}, {NULL}};
const Segment SGI193043[] = {{&S77483[31], 3}, {NULL}};
const Segment SGI193044[] = {{&S71006[113], 3}, {NULL}};
const Segment SGI193045[] = {{&S77483[76], 6}, {NULL}};
const Segment SGI193046[] = {{&S71006[80], 5}, {NULL}};
const Segment SGI193047[] = {{&S71006[117], 14}, {NULL}};
const Segment SGI193048[] = {{&S77484[44], 10}, {NULL}};
const Segment SGI193049[] = {{&S77484[44], 20}, {NULL}};
const Segment SGI193050[] = {{&S77484[30], 6}, {NULL}};
const Segment SGI193051[] = {{&S77484[31], 3}, {NULL}};
const Segment SGI193052[] = {{&S77484[78], 6}, {NULL}};
const Segment SGI193053[] = {{&S77485[45], 10}, {NULL}};
const Segment SGI193054[] = {{&S77485[45], 20}, {NULL}};
const Segment SGI193055[] = {{&S77485[31], 6}, {NULL}};
const Segment SGI193056[] = {{&S77487[44], 16}, {NULL}};
const Segment SGI193057[] = {{&S77487[44], 28}, {NULL}};
const Segment SGI193058[] = {{&S77487[30], 6}, {NULL}};
const Segment SGI193059[] = {{&S77487[31], 3}, {NULL}};
const Segment SGI193060[] = {{&S77487[86], 6}, {NULL}};
const Segment SGI193061[] = {{&S77489[44], 15}, {NULL}};
const Segment SGI193062[] = {{&S77489[44], 27}, {NULL}};
const Segment SGI193063[] = {{&S77489[30], 6}, {NULL}};
const Segment SGI193064[] = {{&S77489[88], 6}, {NULL}};
const Segment SGI193065[] = {{&S77490[38], 15}, {NULL}};
const Segment SGI193066[] = {{&S77490[31], 6}, {NULL}};
const Segment SGI193067[] = {{&S71018[34], 16}, {NULL}};
const Segment SGI193068[] = {{&S77491[44], 15}, {NULL}};
const Segment SGI193069[] = {{&S71018[27], 6}, {NULL}};
const Segment SGI193070[] = {{&S77491[44], 27}, {NULL}};
const Segment SGI193071[] = {{&S77491[30], 6}, {NULL}};
const Segment SGI193072[] = {{&S77491[31], 3}, {NULL}};
const Segment SGI193073[] = {{&S71018[112], 9}, {NULL}};
const Segment SGI193074[] = {{&S71018[85], 6}, {NULL}};
const Segment SGI193075[] = {{&S77491[85], 6}, {NULL}};
const Segment SGI193076[] = {{&S71019[34], 18}, {NULL}};
const Segment SGI193077[] = {{&S71019[27], 6}, {NULL}};
const Segment SGI193078[] = {{&S71019[114], 9}, {NULL}};
const Segment SGI193079[] = {{&S71019[87], 6}, {NULL}};
const Segment SGI193080[] = {{&S71020[34], 18}, {NULL}};
const Segment SGI193081[] = {{&S71020[27], 6}, {NULL}};
const Segment SGI193082[] = {{&S71020[114], 9}, {NULL}};
const Segment SGI193083[] = {{&S71020[87], 6}, {NULL}};
const Segment SGI193084[] = {{&S71021[34], 18}, {NULL}};
const Segment SGI193085[] = {{&S71021[27], 6}, {NULL}};
const Segment SGI193086[] = {{&S71021[114], 9}, {NULL}};
const Segment SGI193087[] = {{&S71021[87], 6}, {NULL}};
const Segment SGI193088[] = {{&S77498[44], 10}, {NULL}};
const Segment SGI193089[] = {{&S77498[44], 22}, {NULL}};
const Segment SGI193090[] = {{&S77498[30], 6}, {NULL}};
const Segment SGI193091[] = {{&S77498[80], 6}, {NULL}};
const Segment SGI193092[] = {{&S77498[107], 13}, {NULL}};
const Segment SGI193093[] = {{&S77499[34], 7}, {NULL}};
const Segment SGI193094[] = {{&S77499[27], 6}, {NULL}};
const Segment SGI193095[] = {{&S77499[99], 9}, {NULL}};
const Segment SGI193096[] = {{&S77499[72], 6}, {NULL}};
const Segment SGI193097[] = {{&S71024[34], 25}, {NULL}};
const Segment SGI193098[] = {{&S71024[27], 6}, {NULL}};
const Segment SGI193099[] = {{&S71024[112], 9}, {NULL}};
const Segment SGI193100[] = {{&S71024[85], 6}, {NULL}};
const Segment SGI193101[] = {{&S71025[34], 25}, {NULL}};
const Segment SGI193102[] = {{&S71025[27], 6}, {NULL}};
const Segment SGI193103[] = {{&S71025[112], 9}, {NULL}};
const Segment SGI193104[] = {{&S71025[85], 6}, {NULL}};
const Segment SGI193105[] = {{&S77502[44], 15}, {NULL}};
const Segment SGI193106[] = {{&S77502[44], 27}, {NULL}};
const Segment SGI193107[] = {{&S77502[30], 6}, {NULL}};
const Segment SGI193108[] = {{&S77502[31], 3}, {NULL}};
const Segment SGI193109[] = {{&S77502[85], 6}, {NULL}};
const Segment SGI193110[] = {{&S77515[34], 8}, {NULL}};
const Segment SGI193111[] = {{&S77515[27], 6}, {NULL}};
const Segment SGI193112[] = {{&S77515[95], 9}, {NULL}};
const Segment SGI193113[] = {{&S77515[68], 6}, {NULL}};
const Segment SGI193114[] = {{&S71040[44], 10}, {NULL}};
const Segment SGI193115[] = {{&S71040[44], 22}, {NULL}};
const Segment SGI193116[] = {{&S71040[30], 6}, {NULL}};
const Segment SGI193117[] = {{&S71040[131], 3}, {NULL}};
const Segment SGI193118[] = {{&S71040[96], 6}, {NULL}};
const Segment SGI193119[] = {{&S71040[68], 8}, {NULL}};
const Segment SGI193120[] = {{&S66684[44], 17}, {NULL}};
const Segment SGI193121[] = {{&S66684[44], 29}, {NULL}};
const Segment SGI193122[] = {{&S66684[30], 6}, {NULL}};
const Segment SGI193123[] = {{&S71041[44], 10}, {NULL}};
const Segment SGI193124[] = {{&S71041[44], 22}, {NULL}};
const Segment SGI193125[] = {{&S71041[30], 6}, {NULL}};
const Segment SGI193126[] = {{&S66684[122], 3}, {NULL}};
const Segment SGI193127[] = {{&S66684[87], 6}, {NULL}};
const Segment SGI193128[] = {{&S71041[131], 3}, {NULL}};
const Segment SGI193129[] = {{&S71041[96], 6}, {NULL}};
const Segment SGI193130[] = {{&S71041[68], 8}, {NULL}};
const Segment SGI193131[] = {{&S66685[44], 18}, {NULL}};
const Segment SGI193132[] = {{&S66685[44], 30}, {NULL}};
const Segment SGI193133[] = {{&S66685[30], 6}, {NULL}};
const Segment SGI193134[] = {{&S66685[123], 3}, {NULL}};
const Segment SGI193135[] = {{&S66685[88], 6}, {NULL}};
const Segment SGI193136[] = {{&S56215[34], 10}, {NULL}};
const Segment SGI193137[] = {{&S66686[38], 18}, {NULL}};
const Segment SGI193138[] = {{&S71043[44], 10}, {NULL}};
const Segment SGI193139[] = {{&S56215[27], 6}, {NULL}};
const Segment SGI193140[] = {{&S66686[31], 6}, {NULL}};
const Segment SGI193141[] = {{&S71043[44], 22}, {NULL}};
const Segment SGI193142[] = {{&S71043[30], 6}, {NULL}};
const Segment SGI193143[] = {{&S71043[131], 3}, {NULL}};
const Segment SGI193144[] = {{&S56215[96], 9}, {NULL}};
const Segment SGI193145[] = {{&S56215[69], 6}, {NULL}};
const Segment SGI193146[] = {{&S71043[96], 6}, {NULL}};
const Segment SGI193147[] = {{&S65493[44], 10}, {NULL}};
const Segment SGI193148[] = {{&S65493[44], 22}, {NULL}};
const Segment SGI193149[] = {{&S71043[68], 8}, {NULL}};
const Segment SGI193150[] = {{&S65493[30], 6}, {NULL}};
const Segment SGI193151[] = {{&S65493[138], 3}, {NULL}};
const Segment SGI193152[] = {{&S65493[103], 6}, {NULL}};
const Segment SGI193153[] = {{&S71044[44], 10}, {NULL}};
const Segment SGI193154[] = {{&S71044[44], 22}, {NULL}};
const Segment SGI193155[] = {{&S71044[30], 6}, {NULL}};
const Segment SGI193156[] = {{&S66688[30], 6}, {NULL}};
const Segment SGI193157[] = {{&S71044[131], 3}, {NULL}};
const Segment SGI193158[] = {{&S71044[96], 6}, {NULL}};
const Segment SGI193159[] = {{&S71044[68], 8}, {NULL}};
const Segment SGI193160[] = {{&S66688[85], 9}, {NULL}};
const Segment SGI193161[] = {{&S66688[57], 6}, {NULL}};
const Segment SGI193162[] = {{&S65496[34], 10}, {NULL}};
const Segment SGI193163[] = {{&S65496[27], 6}, {NULL}};
const Segment SGI193164[] = {{&S71046[44], 12}, {NULL}};
const Segment SGI193165[] = {{&S71046[44], 24}, {NULL}};
const Segment SGI193166[] = {{&S65496[97], 9}, {NULL}};
const Segment SGI193167[] = {{&S71046[30], 6}, {NULL}};
const Segment SGI193168[] = {{&S65496[70], 6}, {NULL}};
const Segment SGI193169[] = {{&S71046[117], 3}, {NULL}};
const Segment SGI193170[] = {{&S71046[82], 6}, {NULL}};
const Segment SGI193171[] = {{&S77541[44], 9}, {NULL}};
const Segment SGI193172[] = {{&S77541[44], 22}, {NULL}};
const Segment SGI193173[] = {{&S77541[30], 6}, {NULL}};
const Segment SGI193174[] = {{&S77541[31], 3}, {NULL}};
const Segment SGI193175[] = {{&S77541[80], 6}, {NULL}};
const Segment SGI193176[] = {{&S77542[44], 16}, {NULL}};
const Segment SGI193177[] = {{&S77542[44], 38}, {NULL}};
const Segment SGI193178[] = {{&S77542[30], 6}, {NULL}};
const Segment SGI193179[] = {{&S77542[31], 3}, {NULL}};
const Segment SGI193180[] = {{&S77542[96], 6}, {NULL}};
const Segment SGI193181[] = {{&S77545[44], 24}, {NULL}};
const Segment SGI193182[] = {{&S77545[30], 6}, {NULL}};
const Segment SGI193183[] = {{&S77545[31], 3}, {NULL}};
const Segment SGI193184[] = {{&S77545[100], 6}, {NULL}};
const Segment SGI193185[] = {{&S66710[44], 10}, {NULL}};
const Segment SGI193186[] = {{&S66710[44], 22}, {NULL}};
const Segment SGI193187[] = {{&S66710[30], 6}, {NULL}};
const Segment SGI193188[] = {{&S66710[131], 3}, {NULL}};
const Segment SGI193189[] = {{&S66710[96], 6}, {NULL}};
const Segment SGI193190[] = {{&S66710[68], 8}, {NULL}};
const Segment SGI193191[] = {{&S77546[44], 14}, {NULL}};
const Segment SGI193192[] = {{&S77546[44], 26}, {NULL}};
const Segment SGI193193[] = {{&S77546[30], 6}, {NULL}};
const Segment SGI193194[] = {{&S77546[31], 3}, {NULL}};
const Segment SGI193195[] = {{&S77546[84], 6}, {NULL}};
const Segment SGI193196[] = {{&S77547[38], 14}, {NULL}};
const Segment SGI193197[] = {{&S77547[31], 6}, {NULL}};
const Segment SGI193198[] = {{&S66712[44], 7}, {NULL}};
const Segment SGI193199[] = {{&S66712[44], 19}, {NULL}};
const Segment SGI193200[] = {{&S66712[30], 6}, {NULL}};
const Segment SGI193201[] = {{&S66712[131], 3}, {NULL}};
const Segment SGI193202[] = {{&S66712[96], 6}, {NULL}};
const Segment SGI193203[] = {{&S66712[65], 8}, {NULL}};
const Segment SGI193204[] = {{&S66713[44], 16}, {NULL}};
const Segment SGI193205[] = {{&S66713[44], 28}, {NULL}};
const Segment SGI193206[] = {{&S66713[30], 6}, {NULL}};
const Segment SGI193207[] = {{&S66713[137], 3}, {NULL}};
const Segment SGI193208[] = {{&S66713[102], 6}, {NULL}};
const Segment SGI193209[] = {{&S66713[74], 8}, {NULL}};
const Segment SGI193210[] = {{&S66714[44], 10}, {NULL}};
const Segment SGI193211[] = {{&S66714[44], 22}, {NULL}};
const Segment SGI193212[] = {{&S66714[30], 6}, {NULL}};
const Segment SGI193213[] = {{&S66714[131], 3}, {NULL}};
const Segment SGI193214[] = {{&S66714[96], 6}, {NULL}};
const Segment SGI193215[] = {{&S66714[68], 8}, {NULL}};
const Segment SGI193216[] = {{&S66715[44], 26}, {NULL}};
const Segment SGI193217[] = {{&S66715[44], 38}, {NULL}};
const Segment SGI193218[] = {{&S66715[30], 6}, {NULL}};
const Segment SGI193219[] = {{&S71075[34], 10}, {NULL}};
const Segment SGI193220[] = {{&S66715[147], 3}, {NULL}};
const Segment SGI193221[] = {{&S71075[27], 6}, {NULL}};
const Segment SGI193222[] = {{&S77554[44], 10}, {NULL}};
const Segment SGI193223[] = {{&S66715[112], 6}, {NULL}};
const Segment SGI193224[] = {{&S77554[44], 22}, {NULL}};
const Segment SGI193225[] = {{&S77554[30], 6}, {NULL}};
const Segment SGI193226[] = {{&S66715[84], 8}, {NULL}};
const Segment SGI193227[] = {{&S71075[97], 9}, {NULL}};
const Segment SGI193228[] = {{&S71075[70], 6}, {NULL}};
const Segment SGI193229[] = {{&S77554[80], 6}, {NULL}};
const Segment SGI193230[] = {{&S77554[107], 13}, {NULL}};
const Segment SGI193231[] = {{&S66716[44], 17}, {NULL}};
const Segment SGI193232[] = {{&S66716[44], 29}, {NULL}};
const Segment SGI193233[] = {{&S66716[30], 6}, {NULL}};
const Segment SGI193234[] = {{&S66716[138], 3}, {NULL}};
const Segment SGI193235[] = {{&S66716[103], 6}, {NULL}};
const Segment SGI193236[] = {{&S66716[75], 8}, {NULL}};
const Segment SGI193237[] = {{&S77555[44], 10}, {NULL}};
const Segment SGI193238[] = {{&S77555[44], 22}, {NULL}};
const Segment SGI193239[] = {{&S77555[30], 6}, {NULL}};
const Segment SGI193240[] = {{&S77555[31], 3}, {NULL}};
const Segment SGI193241[] = {{&S77555[80], 5}, {NULL}};
const Segment SGI193242[] = {{&S77555[117], 14}, {NULL}};
const Segment SGI193243[] = {{&S66717[44], 10}, {NULL}};
const Segment SGI193244[] = {{&S66717[44], 22}, {NULL}};
const Segment SGI193245[] = {{&S66717[30], 6}, {NULL}};
const Segment SGI193246[] = {{&S66717[146], 3}, {NULL}};
const Segment SGI193247[] = {{&S66717[111], 6}, {NULL}};
const Segment SGI193248[] = {{&S66717[68], 8}, {NULL}};
const Segment SGI193249[] = {{&S66718[44], 7}, {NULL}};
const Segment SGI193250[] = {{&S66718[44], 19}, {NULL}};
const Segment SGI193251[] = {{&S66718[30], 6}, {NULL}};
const Segment SGI193252[] = {{&S77558[34], 7}, {NULL}};
const Segment SGI193253[] = {{&S77558[27], 6}, {NULL}};
const Segment SGI193254[] = {{&S66718[128], 3}, {NULL}};
const Segment SGI193255[] = {{&S66718[93], 6}, {NULL}};
const Segment SGI193256[] = {{&S77558[99], 9}, {NULL}};
const Segment SGI193257[] = {{&S66718[65], 8}, {NULL}};
const Segment SGI193258[] = {{&S77558[72], 6}, {NULL}};
const Segment SGI193259[] = {{&S66731[44], 6}, {NULL}};
const Segment SGI193260[] = {{&S66731[44], 18}, {NULL}};
const Segment SGI193261[] = {{&S66731[30], 6}, {NULL}};
const Segment SGI193262[] = {{&S66731[111], 3}, {NULL}};
const Segment SGI193263[] = {{&S66731[76], 6}, {NULL}};
const Segment SGI193264[] = {{&S66732[38], 6}, {NULL}};
const Segment SGI193265[] = {{&S66732[31], 6}, {NULL}};
const Segment SGI193266[] = {{&S56294[38], 10}, {NULL}};
const Segment SGI193267[] = {{&S56294[31], 6}, {NULL}};
const Segment SGI193268[] = {{&S56295[44], 10}, {NULL}};
const Segment SGI193269[] = {{&S56295[44], 21}, {NULL}};
const Segment SGI193270[] = {{&S56295[30], 6}, {NULL}};
const Segment SGI193271[] = {{&S56295[114], 3}, {NULL}};
const Segment SGI193272[] = {{&S56295[79], 6}, {NULL}};
const Segment SGI193273[] = {{&S77605[44], 13}, {NULL}};
const Segment SGI193274[] = {{&S77605[44], 25}, {NULL}};
const Segment SGI193275[] = {{&S77605[30], 6}, {NULL}};
const Segment SGI193276[] = {{&S77605[31], 3}, {NULL}};
const Segment SGI193277[] = {{&S77605[83], 6}, {NULL}};
const Segment SGI193278[] = {{&S77607[44], 8}, {NULL}};
const Segment SGI193279[] = {{&S77607[44], 20}, {NULL}};
const Segment SGI193280[] = {{&S77607[30], 6}, {NULL}};
const Segment SGI193281[] = {{&S77607[31], 3}, {NULL}};
const Segment SGI193282[] = {{&S77607[78], 6}, {NULL}};
const Segment SGI193283[] = {{&S77608[34], 18}, {NULL}};
const Segment SGI193284[] = {{&S77608[27], 6}, {NULL}};
const Segment SGI193285[] = {{&S77608[105], 9}, {NULL}};
const Segment SGI193286[] = {{&S77608[78], 6}, {NULL}};
const Segment SGI193287[] = {{&S66784[34], 10}, {NULL}};
const Segment SGI193288[] = {{&S66784[27], 6}, {NULL}};
const Segment SGI193289[] = {{&S66784[97], 9}, {NULL}};
const Segment SGI193290[] = {{&S66784[70], 6}, {NULL}};
const Segment SGI193291[] = {{&S66784[132], 8}, {NULL}};
const Segment SGI193292[] = {{&S66787[34], 10}, {NULL}};
const Segment SGI193293[] = {{&S66787[27], 6}, {NULL}};
const Segment SGI193294[] = {{&S66787[97], 9}, {NULL}};
const Segment SGI193295[] = {{&S66787[70], 6}, {NULL}};
const Segment SGI193296[] = {{&S66787[131], 8}, {NULL}};
const Segment SGI193297[] = {{&S77631[44], 10}, {NULL}};
const Segment SGI193298[] = {{&S77631[44], 22}, {NULL}};
const Segment SGI193299[] = {{&S71153[34], 25}, {NULL}};
const Segment SGI193300[] = {{&S77631[30], 6}, {NULL}};
const Segment SGI193301[] = {{&S71153[27], 6}, {NULL}};
const Segment SGI193302[] = {{&S77631[31], 3}, {NULL}};
const Segment SGI193303[] = {{&S77631[109], 6}, {NULL}};
const Segment SGI193304[] = {{&S71153[112], 9}, {NULL}};
const Segment SGI193305[] = {{&S71153[85], 6}, {NULL}};
const Segment SGI193306[] = {{&S77631[68], 8}, {NULL}};
const Segment SGI193307[] = {{&S77631[80], 15}, {NULL}};
const Segment SGI193308[] = {{&S77632[44], 10}, {NULL}};
const Segment SGI193309[] = {{&S71154[34], 25}, {NULL}};
const Segment SGI193310[] = {{&S77632[44], 22}, {NULL}};
const Segment SGI193311[] = {{&S77632[30], 6}, {NULL}};
const Segment SGI193312[] = {{&S71154[27], 6}, {NULL}};
const Segment SGI193313[] = {{&S77632[31], 3}, {NULL}};
const Segment SGI193314[] = {{&S71154[112], 9}, {NULL}};
const Segment SGI193315[] = {{&S77632[109], 6}, {NULL}};
const Segment SGI193316[] = {{&S71154[85], 6}, {NULL}};
const Segment SGI193317[] = {{&S77632[68], 8}, {NULL}};
const Segment SGI193318[] = {{&S77632[80], 15}, {NULL}};
const Segment SGI193319[] = {{&S77637[44], 8}, {NULL}};
const Segment SGI193320[] = {{&S77637[44], 20}, {NULL}};
const Segment SGI193321[] = {{&S77637[30], 6}, {NULL}};
const Segment SGI193322[] = {{&S77637[31], 3}, {NULL}};
const Segment SGI193323[] = {{&S77637[78], 6}, {NULL}};
const Segment SGI193324[] = {{&S71161[34], 12}, {NULL}};
const Segment SGI193325[] = {{&S71161[27], 6}, {NULL}};
const Segment SGI193326[] = {{&S71161[99], 9}, {NULL}};
const Segment SGI193327[] = {{&S71161[72], 6}, {NULL}};
const Segment SGI193328[] = {{&S71162[34], 9}, {NULL}};
const Segment SGI193329[] = {{&S71162[27], 6}, {NULL}};
const Segment SGI193330[] = {{&S71162[96], 9}, {NULL}};
const Segment SGI193331[] = {{&S71162[69], 6}, {NULL}};
const Segment SGI193332[] = {{&S66809[34], 9}, {NULL}};
const Segment SGI193333[] = {{&S66809[27], 6}, {NULL}};
const Segment SGI193334[] = {{&S66809[96], 9}, {NULL}};
const Segment SGI193335[] = {{&S66809[69], 6}, {NULL}};
const Segment SGI193336[] = {{&S77649[34], 10}, {NULL}};
const Segment SGI193337[] = {{&S77649[27], 6}, {NULL}};
const Segment SGI193338[] = {{&S77649[97], 9}, {NULL}};
const Segment SGI193339[] = {{&S77649[70], 6}, {NULL}};
const Segment SGI193340[] = {{&S66810[44], 7}, {NULL}};
const Segment SGI193341[] = {{&S66810[44], 24}, {NULL}};
const Segment SGI193342[] = {{&S66810[30], 6}, {NULL}};
const Segment SGI193343[] = {{&S66810[117], 3}, {NULL}};
const Segment SGI193344[] = {{&S66810[82], 6}, {NULL}};
const Segment SGI193345[] = {{&S66811[38], 7}, {NULL}};
const Segment SGI193346[] = {{&S66811[31], 6}, {NULL}};
const Segment SGI193347[] = {{&S66812[44], 11}, {NULL}};
const Segment SGI193348[] = {{&S66812[44], 28}, {NULL}};
const Segment SGI193349[] = {{&S66812[30], 6}, {NULL}};
const Segment SGI193350[] = {{&S66812[121], 3}, {NULL}};
const Segment SGI193351[] = {{&S66812[86], 6}, {NULL}};
const Segment SGI193352[] = {{&S66813[44], 11}, {NULL}};
const Segment SGI193353[] = {{&S66813[44], 28}, {NULL}};
const Segment SGI193354[] = {{&S66813[30], 6}, {NULL}};
const Segment SGI193355[] = {{&S66813[121], 3}, {NULL}};
const Segment SGI193356[] = {{&S66813[86], 6}, {NULL}};
const Segment SGI193357[] = {{&S66814[44], 7}, {NULL}};
const Segment SGI193358[] = {{&S66814[44], 24}, {NULL}};
const Segment SGI193359[] = {{&S66814[30], 6}, {NULL}};
const Segment SGI193360[] = {{&S66814[117], 3}, {NULL}};
const Segment SGI193361[] = {{&S66814[82], 6}, {NULL}};
const Segment SGI193362[] = {{&S77659[34], 14}, {NULL}};
const Segment SGI193363[] = {{&S77659[27], 6}, {NULL}};
const Segment SGI193364[] = {{&S77659[101], 9}, {NULL}};
const Segment SGI193365[] = {{&S77659[74], 6}, {NULL}};
const Segment SGI193366[] = {{&S66815[34], 15}, {NULL}};
const Segment SGI193367[] = {{&S66815[27], 6}, {NULL}};
const Segment SGI193368[] = {{&S66815[102], 9}, {NULL}};
const Segment SGI193369[] = {{&S66815[75], 6}, {NULL}};
const Segment SGI193370[] = {{&S77666[34], 16}, {NULL}};
const Segment SGI193371[] = {{&S77666[27], 6}, {NULL}};
const Segment SGI193372[] = {{&S77666[110], 9}, {NULL}};
const Segment SGI193373[] = {{&S77666[83], 6}, {NULL}};
const Segment SGI193374[] = {{&S77667[34], 15}, {NULL}};
const Segment SGI193375[] = {{&S77667[27], 6}, {NULL}};
const Segment SGI193376[] = {{&S77667[108], 9}, {NULL}};
const Segment SGI193377[] = {{&S77667[81], 6}, {NULL}};
const Segment SGI193378[] = {{&S71191[34], 15}, {NULL}};
const Segment SGI193379[] = {{&S71191[27], 6}, {NULL}};
const Segment SGI193380[] = {{&S71191[102], 9}, {NULL}};
const Segment SGI193381[] = {{&S71191[75], 6}, {NULL}};
const Segment SGI193382[] = {{&S66828[34], 11}, {NULL}};
const Segment SGI193383[] = {{&S66828[27], 6}, {NULL}};
const Segment SGI193384[] = {{&S77672[44], 15}, {NULL}};
const Segment SGI193385[] = {{&S66828[98], 9}, {NULL}};
const Segment SGI193386[] = {{&S77672[44], 27}, {NULL}};
const Segment SGI193387[] = {{&S66828[71], 6}, {NULL}};
const Segment SGI193388[] = {{&S77672[30], 6}, {NULL}};
const Segment SGI193389[] = {{&S77672[31], 3}, {NULL}};
const Segment SGI193390[] = {{&S77672[85], 6}, {NULL}};
const Segment SGI193391[] = {{&S66829[34], 10}, {NULL}};
const Segment SGI193392[] = {{&S66829[27], 6}, {NULL}};
const Segment SGI193393[] = {{&S66829[97], 9}, {NULL}};
const Segment SGI193394[] = {{&S66829[70], 6}, {NULL}};
const Segment SGI193395[] = {{&S66829[131], 8}, {NULL}};
const Segment SGI193396[] = {{&S77687[34], 7}, {NULL}};
const Segment SGI193397[] = {{&S77687[27], 6}, {NULL}};
const Segment SGI193398[] = {{&S66842[44], 9}, {NULL}};
const Segment SGI193399[] = {{&S66842[44], 20}, {NULL}};
const Segment SGI193400[] = {{&S66842[30], 6}, {NULL}};
const Segment SGI193401[] = {{&S77687[98], 9}, {NULL}};
const Segment SGI193402[] = {{&S77687[71], 6}, {NULL}};
const Segment SGI193403[] = {{&S66842[132], 3}, {NULL}};
const Segment SGI193404[] = {{&S66842[97], 6}, {NULL}};
const Segment SGI193405[] = {{&S66842[66], 8}, {NULL}};
const Segment SGI193406[] = {{&S66847[34], 10}, {NULL}};
const Segment SGI193407[] = {{&S66847[27], 6}, {NULL}};
const Segment SGI193408[] = {{&S66847[97], 9}, {NULL}};
const Segment SGI193409[] = {{&S66847[70], 6}, {NULL}};
const Segment SGI193410[] = {{&S66847[138], 8}, {NULL}};
const Segment SGI193411[] = {{&S66848[34], 11}, {NULL}};
const Segment SGI193412[] = {{&S66848[27], 6}, {NULL}};
const Segment SGI193413[] = {{&S66848[98], 9}, {NULL}};
const Segment SGI193414[] = {{&S66848[71], 6}, {NULL}};
const Segment SGI193415[] = {{&S66848[139], 8}, {NULL}};
const Segment SGI193416[] = {{&S77698[44], 20}, {NULL}};
const Segment SGI193417[] = {{&S77698[44], 32}, {NULL}};
const Segment SGI193418[] = {{&S77698[30], 6}, {NULL}};
const Segment SGI193419[] = {{&S77698[31], 3}, {NULL}};
const Segment SGI193420[] = {{&S77698[90], 6}, {NULL}};
const Segment SGI193421[] = {{&S77705[34], 11}, {NULL}};
const Segment SGI193422[] = {{&S77705[27], 6}, {NULL}};
const Segment SGI193423[] = {{&S77705[105], 9}, {NULL}};
const Segment SGI193424[] = {{&S77705[78], 6}, {NULL}};
const Segment SGI193425[] = {{&S77707[34], 11}, {NULL}};
const Segment SGI193426[] = {{&S77707[27], 6}, {NULL}};
const Segment SGI193427[] = {{&S77707[98], 9}, {NULL}};
const Segment SGI193428[] = {{&S77707[71], 6}, {NULL}};
const Segment SGI193429[] = {{&S77708[34], 18}, {NULL}};
const Segment SGI193430[] = {{&S77708[27], 6}, {NULL}};
const Segment SGI193431[] = {{&S77708[105], 9}, {NULL}};
const Segment SGI193432[] = {{&S77708[78], 6}, {NULL}};
const Segment SGI193433[] = {{&S77709[44], 9}, {NULL}};
const Segment SGI193434[] = {{&S77709[44], 21}, {NULL}};
const Segment SGI193435[] = {{&S77709[30], 6}, {NULL}};
const Segment SGI193436[] = {{&S77709[31], 3}, {NULL}};
const Segment SGI193437[] = {{&S77709[79], 5}, {NULL}};
const Segment SGI193438[] = {{&S77723[34], 10}, {NULL}};
const Segment SGI193439[] = {{&S77723[27], 6}, {NULL}};
const Segment SGI193440[] = {{&S77723[97], 9}, {NULL}};
const Segment SGI193441[] = {{&S77723[70], 6}, {NULL}};
const Segment SGI193442[] = {{&S77724[34], 7}, {NULL}};
const Segment SGI193443[] = {{&S77724[27], 6}, {NULL}};
const Segment SGI193444[] = {{&S77724[103], 9}, {NULL}};
const Segment SGI193445[] = {{&S77724[76], 6}, {NULL}};
const Segment SGI193446[] = {{&S77738[34], 9}, {NULL}};
const Segment SGI193447[] = {{&S77738[27], 6}, {NULL}};
const Segment SGI193448[] = {{&S77738[96], 9}, {NULL}};
const Segment SGI193449[] = {{&S77738[69], 6}, {NULL}};
const Segment SGI193450[] = {{&S77739[34], 22}, {NULL}};
const Segment SGI193451[] = {{&S77739[27], 6}, {NULL}};
const Segment SGI193452[] = {{&S77739[109], 9}, {NULL}};
const Segment SGI193453[] = {{&S77739[82], 6}, {NULL}};
const Segment SGI193454[] = {{&S77740[44], 6}, {NULL}};
const Segment SGI193455[] = {{&S77740[44], 18}, {NULL}};
const Segment SGI193456[] = {{&S77740[30], 6}, {NULL}};
const Segment SGI193457[] = {{&S77740[31], 3}, {NULL}};
const Segment SGI193458[] = {{&S77740[76], 6}, {NULL}};
const Segment SGI193459[] = {{&S77743[44], 26}, {NULL}};
const Segment SGI193460[] = {{&S77743[44], 38}, {NULL}};
const Segment SGI193461[] = {{&S77743[30], 6}, {NULL}};
const Segment SGI193462[] = {{&S77743[31], 3}, {NULL}};
const Segment SGI193463[] = {{&S77743[96], 6}, {NULL}};
const Segment SGI193464[] = {{&S71283[34], 6}, {NULL}};
const Segment SGI193465[] = {{&S71283[27], 6}, {NULL}};
const Segment SGI193466[] = {{&S71283[93], 9}, {NULL}};
const Segment SGI193467[] = {{&S71283[66], 6}, {NULL}};
const Segment SGI193468[] = {{&S66906[34], 10}, {NULL}};
const Segment SGI193469[] = {{&S66906[27], 6}, {NULL}};
const Segment SGI193470[] = {{&S77747[34], 10}, {NULL}};
const Segment SGI193471[] = {{&S77747[27], 6}, {NULL}};
const Segment SGI193472[] = {{&S66906[97], 9}, {NULL}};
const Segment SGI193473[] = {{&S66906[70], 6}, {NULL}};
const Segment SGI193474[] = {{&S77747[97], 9}, {NULL}};
const Segment SGI193475[] = {{&S77747[70], 6}, {NULL}};
const Segment SGI193476[] = {{&S66907[34], 10}, {NULL}};
const Segment SGI193477[] = {{&S66907[27], 6}, {NULL}};
const Segment SGI193478[] = {{&S66907[97], 9}, {NULL}};
const Segment SGI193479[] = {{&S66907[70], 6}, {NULL}};
const Segment SGI193480[] = {{&S77752[44], 15}, {NULL}};
const Segment SGI193481[] = {{&S77752[44], 33}, {NULL}};
const Segment SGI193482[] = {{&S77752[30], 6}, {NULL}};
const Segment SGI193483[] = {{&S77752[91], 6}, {NULL}};
const Segment SGI193484[] = {{&S77752[118], 13}, {NULL}};
const Segment SGI193485[] = {{&S66926[44], 7}, {NULL}};
const Segment SGI193486[] = {{&S66926[44], 25}, {NULL}};
const Segment SGI193487[] = {{&S66926[30], 6}, {NULL}};
const Segment SGI193488[] = {{&S66926[118], 3}, {NULL}};
const Segment SGI193489[] = {{&S66926[83], 6}, {NULL}};
const Segment SGI193490[] = {{&S66927[38], 7}, {NULL}};
const Segment SGI193491[] = {{&S66927[31], 6}, {NULL}};
const Segment SGI193492[] = {{&S66928[44], 7}, {NULL}};
const Segment SGI193493[] = {{&S66928[44], 27}, {NULL}};
const Segment SGI193494[] = {{&S66928[30], 6}, {NULL}};
const Segment SGI193495[] = {{&S66928[120], 3}, {NULL}};
const Segment SGI193496[] = {{&S66928[85], 6}, {NULL}};
const Segment SGI193497[] = {{&S66929[44], 7}, {NULL}};
const Segment SGI193498[] = {{&S66929[44], 25}, {NULL}};
const Segment SGI193499[] = {{&S66929[30], 6}, {NULL}};
const Segment SGI193500[] = {{&S66929[118], 3}, {NULL}};
const Segment SGI193501[] = {{&S66929[83], 6}, {NULL}};
const Segment SGI193502[] = {{&S77771[44], 13}, {NULL}};
const Segment SGI193503[] = {{&S77771[44], 25}, {NULL}};
const Segment SGI193504[] = {{&S77771[30], 6}, {NULL}};
const Segment SGI193505[] = {{&S77771[31], 3}, {NULL}};
const Segment SGI193506[] = {{&S77771[83], 6}, {NULL}};
const Segment SGI193507[] = {{&S66930[44], 11}, {NULL}};
const Segment SGI193508[] = {{&S66930[44], 29}, {NULL}};
const Segment SGI193509[] = {{&S66930[30], 6}, {NULL}};
const Segment SGI193510[] = {{&S66930[122], 3}, {NULL}};
const Segment SGI193511[] = {{&S66930[87], 6}, {NULL}};
const Segment SGI193512[] = {{&S71312[34], 7}, {NULL}};
const Segment SGI193513[] = {{&S71312[27], 6}, {NULL}};
const Segment SGI193514[] = {{&S71312[94], 9}, {NULL}};
const Segment SGI193515[] = {{&S71312[67], 6}, {NULL}};
const Segment SGI193516[] = {{&S66931[34], 7}, {NULL}};
const Segment SGI193517[] = {{&S66931[27], 6}, {NULL}};
const Segment SGI193518[] = {{&S66931[94], 9}, {NULL}};
const Segment SGI193519[] = {{&S66931[67], 6}, {NULL}};
const Segment SGI193520[] = {{&S71315[44], 9}, {NULL}};
const Segment SGI193521[] = {{&S71315[44], 21}, {NULL}};
const Segment SGI193522[] = {{&S77777[34], 5}, {NULL}};
const Segment SGI193523[] = {{&S71315[30], 6}, {NULL}};
const Segment SGI193524[] = {{&S77777[27], 6}, {NULL}};
const Segment SGI193525[] = {{&S71315[114], 3}, {NULL}};
const Segment SGI193526[] = {{&S71315[79], 6}, {NULL}};
const Segment SGI193527[] = {{&S77777[92], 9}, {NULL}};
const Segment SGI193528[] = {{&S77777[65], 6}, {NULL}};
const Segment SGI193529[] = {{&S77778[34], 5}, {NULL}};
const Segment SGI193530[] = {{&S77778[27], 6}, {NULL}};
const Segment SGI193531[] = {{&S77778[92], 9}, {NULL}};
const Segment SGI193532[] = {{&S77778[65], 6}, {NULL}};
const Segment SGI193533[] = {{&S77792[34], 7}, {NULL}};
const Segment SGI193534[] = {{&S77792[27], 6}, {NULL}};
const Segment SGI193535[] = {{&S77792[100], 9}, {NULL}};
const Segment SGI193536[] = {{&S77792[73], 6}, {NULL}};
const Segment SGI193537[] = {{&S77792[141], 8}, {NULL}};
const Segment SGI193538[] = {{&S77793[34], 7}, {NULL}};
const Segment SGI193539[] = {{&S77793[27], 6}, {NULL}};
const Segment SGI193540[] = {{&S77793[94], 9}, {NULL}};
const Segment SGI193541[] = {{&S77793[67], 6}, {NULL}};
const Segment SGI193542[] = {{&S71349[44], 21}, {NULL}};
const Segment SGI193543[] = {{&S71349[44], 33}, {NULL}};
const Segment SGI193544[] = {{&S71349[30], 6}, {NULL}};
const Segment SGI193545[] = {{&S71349[126], 3}, {NULL}};
const Segment SGI193546[] = {{&S71349[91], 6}, {NULL}};
const Segment SGI193547[] = {{&S71350[44], 22}, {NULL}};
const Segment SGI193548[] = {{&S71350[44], 34}, {NULL}};
const Segment SGI193549[] = {{&S71350[30], 6}, {NULL}};
const Segment SGI193550[] = {{&S71350[127], 3}, {NULL}};
const Segment SGI193551[] = {{&S71350[92], 6}, {NULL}};
const Segment SGI193552[] = {{&S71352[44], 10}, {NULL}};
const Segment SGI193553[] = {{&S71352[44], 22}, {NULL}};
const Segment SGI193554[] = {{&S71352[30], 6}, {NULL}};
const Segment SGI193555[] = {{&S71352[115], 3}, {NULL}};
const Segment SGI193556[] = {{&S71352[80], 6}, {NULL}};
const Segment SGI193557[] = {{&S71353[38], 10}, {NULL}};
const Segment SGI193558[] = {{&S71353[31], 6}, {NULL}};
const Segment SGI193559[] = {{&S77833[34], 15}, {NULL}};
const Segment SGI193560[] = {{&S77833[27], 6}, {NULL}};
const Segment SGI193561[] = {{&S77833[102], 9}, {NULL}};
const Segment SGI193562[] = {{&S77833[75], 6}, {NULL}};
const Segment SGI193563[] = {{&S77834[34], 11}, {NULL}};
const Segment SGI193564[] = {{&S77834[27], 6}, {NULL}};
const Segment SGI193565[] = {{&S77834[98], 9}, {NULL}};
const Segment SGI193566[] = {{&S77834[71], 6}, {NULL}};
const Segment SGI193567[] = {{&S77850[34], 11}, {NULL}};
const Segment SGI193568[] = {{&S77850[27], 6}, {NULL}};
const Segment SGI193569[] = {{&S77850[110], 9}, {NULL}};
const Segment SGI193570[] = {{&S77850[83], 6}, {NULL}};
const Segment SGI193571[] = {{&S71392[34], 7}, {NULL}};
const Segment SGI193572[] = {{&S71392[27], 6}, {NULL}};
const Segment SGI193573[] = {{&S71392[94], 9}, {NULL}};
const Segment SGI193574[] = {{&S71392[67], 6}, {NULL}};
const Segment SGI193575[] = {{&S71393[44], 10}, {NULL}};
const Segment SGI193576[] = {{&S71393[44], 22}, {NULL}};
const Segment SGI193577[] = {{&S71393[30], 6}, {NULL}};
const Segment SGI193578[] = {{&S71393[115], 3}, {NULL}};
const Segment SGI193579[] = {{&S71393[80], 6}, {NULL}};
const Segment SGI193580[] = {{&S71393[143], 3}, {NULL}};
const Segment SGI193581[] = {{&S71394[44], 10}, {NULL}};
const Segment SGI193582[] = {{&S71394[44], 22}, {NULL}};
const Segment SGI193583[] = {{&S71394[30], 6}, {NULL}};
const Segment SGI193584[] = {{&S71394[115], 3}, {NULL}};
const Segment SGI193585[] = {{&S71394[80], 6}, {NULL}};
const Segment SGI193586[] = {{&S71394[150], 3}, {NULL}};
const Segment SGI193587[] = {{&S77861[34], 7}, {NULL}};
const Segment SGI193588[] = {{&S77861[27], 6}, {NULL}};
const Segment SGI193589[] = {{&S77861[104], 9}, {NULL}};
const Segment SGI193590[] = {{&S77861[77], 6}, {NULL}};
const Segment SGI193591[] = {{&S67032[34], 15}, {NULL}};
const Segment SGI193592[] = {{&S67032[27], 6}, {NULL}};
const Segment SGI193593[] = {{&S67032[102], 9}, {NULL}};
const Segment SGI193594[] = {{&S67032[75], 6}, {NULL}};
const Segment SGI193595[] = {{&S71402[44], 25}, {NULL}};
const Segment SGI193596[] = {{&S71402[44], 37}, {NULL}};
const Segment SGI193597[] = {{&S71402[30], 6}, {NULL}};
const Segment SGI193598[] = {{&S71402[31], 3}, {NULL}};
const Segment SGI193599[] = {{&S71402[95], 6}, {NULL}};
const Segment SGI193600[] = {{&S71403[38], 25}, {NULL}};
const Segment SGI193601[] = {{&S71403[31], 6}, {NULL}};
const Segment SGI193602[] = {{&S77882[34], 7}, {NULL}};
const Segment SGI193603[] = {{&S77882[27], 6}, {NULL}};
const Segment SGI193604[] = {{&S77882[94], 9}, {NULL}};
const Segment SGI193605[] = {{&S77882[67], 6}, {NULL}};
const Segment SGI193606[] = {{&S77895[44], 8}, {NULL}};
const Segment SGI193607[] = {{&S77895[44], 20}, {NULL}};
const Segment SGI193608[] = {{&S77895[30], 6}, {NULL}};
const Segment SGI193609[] = {{&S77895[31], 3}, {NULL}};
const Segment SGI193610[] = {{&S77895[78], 6}, {NULL}};
const Segment SGI193611[] = {{&S77896[44], 14}, {NULL}};
const Segment SGI193612[] = {{&S77896[44], 26}, {NULL}};
const Segment SGI193613[] = {{&S77896[30], 6}, {NULL}};
const Segment SGI193614[] = {{&S77896[31], 3}, {NULL}};
const Segment SGI193615[] = {{&S77896[84], 6}, {NULL}};
const Segment SGI193616[] = {{&S77901[34], 10}, {NULL}};
const Segment SGI193617[] = {{&S77901[27], 6}, {NULL}};
const Segment SGI193618[] = {{&S77901[97], 9}, {NULL}};
const Segment SGI193619[] = {{&S77901[70], 6}, {NULL}};
const Segment SGI193620[] = {{&S67073[44], 12}, {NULL}};
const Segment SGI193621[] = {{&S67073[44], 29}, {NULL}};
const Segment SGI193622[] = {{&S67073[30], 6}, {NULL}};
const Segment SGI193623[] = {{&S77903[44], 10}, {NULL}};
const Segment SGI193624[] = {{&S77903[44], 22}, {NULL}};
const Segment SGI193625[] = {{&S67073[122], 3}, {NULL}};
const Segment SGI193626[] = {{&S77903[30], 6}, {NULL}};
const Segment SGI193627[] = {{&S67073[87], 6}, {NULL}};
const Segment SGI193628[] = {{&S77903[31], 3}, {NULL}};
const Segment SGI193629[] = {{&S77903[80], 6}, {NULL}};
const Segment SGI193630[] = {{&S77903[143], 3}, {NULL}};
const Segment SGI193631[] = {{&S67074[44], 8}, {NULL}};
const Segment SGI193632[] = {{&S67074[44], 25}, {NULL}};
const Segment SGI193633[] = {{&S67074[30], 6}, {NULL}};
const Segment SGI193634[] = {{&S67074[118], 3}, {NULL}};
const Segment SGI193635[] = {{&S67074[83], 6}, {NULL}};
const Segment SGI193636[] = {{&S77904[34], 7}, {NULL}};
const Segment SGI193637[] = {{&S77904[27], 6}, {NULL}};
const Segment SGI193638[] = {{&S77904[99], 9}, {NULL}};
const Segment SGI193639[] = {{&S77904[72], 6}, {NULL}};
const Segment SGI193640[] = {{&S67075[38], 8}, {NULL}};
const Segment SGI193641[] = {{&S67075[31], 6}, {NULL}};
const Segment SGI193642[] = {{&S67076[44], 12}, {NULL}};
const Segment SGI193643[] = {{&S67076[44], 29}, {NULL}};
const Segment SGI193644[] = {{&S77906[34], 7}, {NULL}};
const Segment SGI193645[] = {{&S67076[30], 6}, {NULL}};
const Segment SGI193646[] = {{&S77906[27], 6}, {NULL}};
const Segment SGI193647[] = {{&S67076[122], 3}, {NULL}};
const Segment SGI193648[] = {{&S67076[87], 6}, {NULL}};
const Segment SGI193649[] = {{&S77906[99], 9}, {NULL}};
const Segment SGI193650[] = {{&S77906[72], 6}, {NULL}};
const Segment SGI193651[] = {{&S67077[44], 8}, {NULL}};
const Segment SGI193652[] = {{&S67077[44], 25}, {NULL}};
const Segment SGI193653[] = {{&S67077[30], 6}, {NULL}};
const Segment SGI193654[] = {{&S77907[34], 14}, {NULL}};
const Segment SGI193655[] = {{&S77907[27], 6}, {NULL}};
const Segment SGI193656[] = {{&S67077[118], 3}, {NULL}};
const Segment SGI193657[] = {{&S67077[83], 6}, {NULL}};
const Segment SGI193658[] = {{&S77907[101], 9}, {NULL}};
const Segment SGI193659[] = {{&S77907[74], 6}, {NULL}};
const Segment SGI193660[] = {{&S71455[44], 6}, {NULL}};
const Segment SGI193661[] = {{&S71455[44], 18}, {NULL}};
const Segment SGI193662[] = {{&S71455[30], 6}, {NULL}};
const Segment SGI193663[] = {{&S71455[111], 3}, {NULL}};
const Segment SGI193664[] = {{&S77910[34], 11}, {NULL}};
const Segment SGI193665[] = {{&S71455[76], 6}, {NULL}};
const Segment SGI193666[] = {{&S77910[27], 6}, {NULL}};
const Segment SGI193667[] = {{&S77910[98], 9}, {NULL}};
const Segment SGI193668[] = {{&S77910[71], 6}, {NULL}};
const Segment SGI193669[] = {{&S77916[34], 8}, {NULL}};
const Segment SGI193670[] = {{&S77916[27], 6}, {NULL}};
const Segment SGI193671[] = {{&S77916[106], 9}, {NULL}};
const Segment SGI193672[] = {{&S77916[79], 6}, {NULL}};
const Segment SGI193673[] = {{&S77917[34], 8}, {NULL}};
const Segment SGI193674[] = {{&S77917[27], 6}, {NULL}};
const Segment SGI193675[] = {{&S77917[106], 9}, {NULL}};
const Segment SGI193676[] = {{&S77917[79], 6}, {NULL}};
const Segment SGI193677[] = {{&S77925[34], 11}, {NULL}};
const Segment SGI193678[] = {{&S77925[27], 6}, {NULL}};
const Segment SGI193679[] = {{&S77925[110], 9}, {NULL}};
const Segment SGI193680[] = {{&S77925[83], 6}, {NULL}};
const Segment SGI193681[] = {{&S77959[44], 7}, {NULL}};
const Segment SGI193682[] = {{&S77959[44], 19}, {NULL}};
const Segment SGI193683[] = {{&S77959[30], 6}, {NULL}};
const Segment SGI193684[] = {{&S77959[31], 3}, {NULL}};
const Segment SGI193685[] = {{&S77959[92], 6}, {NULL}};
const Segment SGI193686[] = {{&S77959[65], 8}, {NULL}};
const Segment SGI193687[] = {{&S77960[44], 7}, {NULL}};
const Segment SGI193688[] = {{&S77960[44], 19}, {NULL}};
const Segment SGI193689[] = {{&S77960[30], 6}, {NULL}};
const Segment SGI193690[] = {{&S71514[44], 18}, {NULL}};
const Segment SGI193691[] = {{&S65699[44], 10}, {NULL}};
const Segment SGI193692[] = {{&S77960[31], 3}, {NULL}};
const Segment SGI193693[] = {{&S65699[44], 22}, {NULL}};
const Segment SGI193694[] = {{&S71514[44], 30}, {NULL}};
const Segment SGI193695[] = {{&S77960[92], 6}, {NULL}};
const Segment SGI193696[] = {{&S65699[30], 6}, {NULL}};
const Segment SGI193697[] = {{&S71514[30], 6}, {NULL}};
const Segment SGI193698[] = {{&S77960[65], 8}, {NULL}};
const Segment SGI193699[] = {{&S65699[115], 3}, {NULL}};
const Segment SGI193700[] = {{&S71514[144], 3}, {NULL}};
const Segment SGI193701[] = {{&S65699[80], 6}, {NULL}};
const Segment SGI193702[] = {{&S71514[109], 6}, {NULL}};
const Segment SGI193703[] = {{&S65700[44], 10}, {NULL}};
const Segment SGI193704[] = {{&S65700[44], 22}, {NULL}};
const Segment SGI193705[] = {{&S65700[30], 6}, {NULL}};
const Segment SGI193706[] = {{&S65700[115], 3}, {NULL}};
const Segment SGI193707[] = {{&S65700[80], 6}, {NULL}};
const Segment SGI193708[] = {{&S77966[34], 8}, {NULL}};
const Segment SGI193709[] = {{&S77966[27], 6}, {NULL}};
const Segment SGI193710[] = {{&S77966[95], 9}, {NULL}};
const Segment SGI193711[] = {{&S77966[68], 6}, {NULL}};
const Segment SGI193712[] = {{&S77972[44], 25}, {NULL}};
const Segment SGI193713[] = {{&S77972[44], 37}, {NULL}};
const Segment SGI193714[] = {{&S77972[30], 6}, {NULL}};
const Segment SGI193715[] = {{&S77972[31], 3}, {NULL}};
const Segment SGI193716[] = {{&S77972[95], 6}, {NULL}};
const Segment SGI193717[] = {{&S71526[44], 17}, {NULL}};
const Segment SGI193718[] = {{&S71526[44], 38}, {NULL}};
const Segment SGI193719[] = {{&S71526[30], 6}, {NULL}};
const Segment SGI193720[] = {{&S71526[131], 3}, {NULL}};
const Segment SGI193721[] = {{&S71526[96], 6}, {NULL}};
const Segment SGI193722[] = {{&S77974[44], 15}, {NULL}};
const Segment SGI193723[] = {{&S77974[44], 27}, {NULL}};
const Segment SGI193724[] = {{&S77974[30], 6}, {NULL}};
const Segment SGI193725[] = {{&S71527[38], 17}, {NULL}};
const Segment SGI193726[] = {{&S77974[31], 3}, {NULL}};
const Segment SGI193727[] = {{&S71527[31], 6}, {NULL}};
const Segment SGI193728[] = {{&S77974[85], 6}, {NULL}};
const Segment SGI193729[] = {{&S77979[34], 11}, {NULL}};
const Segment SGI193730[] = {{&S77979[27], 6}, {NULL}};
const Segment SGI193731[] = {{&S77979[98], 9}, {NULL}};
const Segment SGI193732[] = {{&S77979[71], 6}, {NULL}};
const Segment SGI193733[] = {{&S65744[44], 4}, {NULL}};
const Segment SGI193734[] = {{&S65744[44], 16}, {NULL}};
const Segment SGI193735[] = {{&S65744[30], 6}, {NULL}};
const Segment SGI193736[] = {{&S65744[109], 3}, {NULL}};
const Segment SGI193737[] = {{&S65744[74], 6}, {NULL}};
const Segment SGI193738[] = {{&S78002[34], 11}, {NULL}};
const Segment SGI193739[] = {{&S78002[27], 6}, {NULL}};
const Segment SGI193740[] = {{&S78002[98], 9}, {NULL}};
const Segment SGI193741[] = {{&S78002[71], 6}, {NULL}};
const Segment SGI193742[] = {{&S65745[38], 4}, {NULL}};
const Segment SGI193743[] = {{&S65745[31], 6}, {NULL}};
const Segment SGI193744[] = {{&S78003[34], 15}, {NULL}};
const Segment SGI193745[] = {{&S78003[27], 6}, {NULL}};
const Segment SGI193746[] = {{&S78003[104], 9}, {NULL}};
const Segment SGI193747[] = {{&S78003[77], 6}, {NULL}};
const Segment SGI193748[] = {{&S78004[34], 8}, {NULL}};
const Segment SGI193749[] = {{&S78004[27], 6}, {NULL}};
const Segment SGI193750[] = {{&S78004[101], 9}, {NULL}};
const Segment SGI193751[] = {{&S78004[74], 6}, {NULL}};
const Segment SGI193752[] = {{&S78023[34], 7}, {NULL}};
const Segment SGI193753[] = {{&S78023[27], 6}, {NULL}};
const Segment SGI193754[] = {{&S78023[94], 9}, {NULL}};
const Segment SGI193755[] = {{&S78023[67], 6}, {NULL}};
const Segment SGI193756[] = {{&S78025[34], 9}, {NULL}};
const Segment SGI193757[] = {{&S78025[27], 6}, {NULL}};
const Segment SGI193758[] = {{&S78025[96], 9}, {NULL}};
const Segment SGI193759[] = {{&S78025[69], 6}, {NULL}};
const Segment SGI193760[] = {{&S56717[44], 11}, {NULL}};
const Segment SGI193761[] = {{&S56717[44], 22}, {NULL}};
const Segment SGI193762[] = {{&S56717[30], 6}, {NULL}};
const Segment SGI193763[] = {{&S56717[115], 3}, {NULL}};
const Segment SGI193764[] = {{&S56717[80], 6}, {NULL}};
const Segment SGI193765[] = {{&S78033[34], 9}, {NULL}};
const Segment SGI193766[] = {{&S56718[38], 11}, {NULL}};
const Segment SGI193767[] = {{&S56718[31], 6}, {NULL}};
const Segment SGI193768[] = {{&S78033[27], 6}, {NULL}};
const Segment SGI193769[] = {{&S71589[30], 6}, {NULL}};
const Segment SGI193770[] = {{&S78033[96], 9}, {NULL}};
const Segment SGI193771[] = {{&S78033[69], 6}, {NULL}};
const Segment SGI193772[] = {{&S71589[85], 9}, {NULL}};
const Segment SGI193773[] = {{&S71589[57], 6}, {NULL}};
const Segment SGI193774[] = {{&S56719[44], 11}, {NULL}};
const Segment SGI193775[] = {{&S56719[44], 23}, {NULL}};
const Segment SGI193776[] = {{&S56719[30], 6}, {NULL}};
const Segment SGI193777[] = {{&S56719[116], 3}, {NULL}};
const Segment SGI193778[] = {{&S56719[81], 6}, {NULL}};
const Segment SGI193779[] = {{&S71590[44], 11}, {NULL}};
const Segment SGI193780[] = {{&S71590[44], 23}, {NULL}};
const Segment SGI193781[] = {{&S71590[30], 6}, {NULL}};
const Segment SGI193782[] = {{&S71590[116], 3}, {NULL}};
const Segment SGI193783[] = {{&S71590[81], 6}, {NULL}};
const Segment SGI193784[] = {{&S67104[44], 9}, {NULL}};
const Segment SGI193785[] = {{&S67104[44], 21}, {NULL}};
const Segment SGI193786[] = {{&S67104[30], 6}, {NULL}};
const Segment SGI193787[] = {{&S67104[114], 3}, {NULL}};
const Segment SGI193788[] = {{&S67104[79], 6}, {NULL}};
const Segment SGI193789[] = {{&S71601[34], 5}, {NULL}};
const Segment SGI193790[] = {{&S71601[27], 6}, {NULL}};
const Segment SGI193791[] = {{&S67105[38], 9}, {NULL}};
const Segment SGI193792[] = {{&S67105[31], 6}, {NULL}};
const Segment SGI193793[] = {{&S71601[92], 9}, {NULL}};
const Segment SGI193794[] = {{&S71601[65], 6}, {NULL}};
const Segment SGI193795[] = {{&S67106[34], 12}, {NULL}};
const Segment SGI193796[] = {{&S67106[27], 6}, {NULL}};
const Segment SGI193797[] = {{&S67106[99], 9}, {NULL}};
const Segment SGI193798[] = {{&S67106[72], 6}, {NULL}};
const Segment SGI193799[] = {{&S67115[34], 6}, {NULL}};
const Segment SGI193800[] = {{&S67115[27], 6}, {NULL}};
const Segment SGI193801[] = {{&S71612[34], 10}, {NULL}};
const Segment SGI193802[] = {{&S71612[27], 6}, {NULL}};
const Segment SGI193803[] = {{&S67115[93], 9}, {NULL}};
const Segment SGI193804[] = {{&S67115[66], 6}, {NULL}};
const Segment SGI193805[] = {{&S71612[97], 9}, {NULL}};
const Segment SGI193806[] = {{&S71612[70], 6}, {NULL}};
const Segment SGI193807[] = {{&S67117[44], 11}, {NULL}};
const Segment SGI193808[] = {{&S67117[44], 23}, {NULL}};
const Segment SGI193809[] = {{&S67117[30], 6}, {NULL}};
const Segment SGI193810[] = {{&S67117[116], 3}, {NULL}};
const Segment SGI193811[] = {{&S67117[81], 6}, {NULL}};
const Segment SGI193812[] = {{&S67118[44], 18}, {NULL}};
const Segment SGI193813[] = {{&S67118[44], 30}, {NULL}};
const Segment SGI193814[] = {{&S67118[30], 6}, {NULL}};
const Segment SGI193815[] = {{&S67118[123], 3}, {NULL}};
const Segment SGI193816[] = {{&S65816[44], 7}, {NULL}};
const Segment SGI193817[] = {{&S65816[44], 19}, {NULL}};
const Segment SGI193818[] = {{&S67118[88], 6}, {NULL}};
const Segment SGI193819[] = {{&S65816[30], 6}, {NULL}};
const Segment SGI193820[] = {{&S65816[112], 3}, {NULL}};
const Segment SGI193821[] = {{&S65816[77], 6}, {NULL}};
const Segment SGI193822[] = {{&S67119[44], 11}, {NULL}};
const Segment SGI193823[] = {{&S67119[44], 23}, {NULL}};
const Segment SGI193824[] = {{&S67119[30], 6}, {NULL}};
const Segment SGI193825[] = {{&S78047[52], 7}, {NULL}};
const Segment SGI193826[] = {{&S65817[38], 7}, {NULL}};
const Segment SGI193827[] = {{&S78047[52], 24}, {NULL}};
const Segment SGI193828[] = {{&S65817[31], 6}, {NULL}};
const Segment SGI193829[] = {{&S78047[38], 6}, {NULL}};
const Segment SGI193830[] = {{&S67119[116], 3}, {NULL}};
const Segment SGI193831[] = {{&S67119[81], 6}, {NULL}};
const Segment SGI193832[] = {{&S78047[90], 5}, {NULL}};
const Segment SGI193833[] = {{&S78047[0], 14}, {NULL}};
const Segment SGI193834[] = {{&S67120[44], 11}, {NULL}};
const Segment SGI193835[] = {{&S67120[44], 23}, {NULL}};
const Segment SGI193836[] = {{&S67120[30], 6}, {NULL}};
const Segment SGI193837[] = {{&S67120[116], 3}, {NULL}};
const Segment SGI193838[] = {{&S78048[52], 8}, {NULL}};
const Segment SGI193839[] = {{&S78048[52], 24}, {NULL}};
const Segment SGI193840[] = {{&S67120[81], 6}, {NULL}};
const Segment SGI193841[] = {{&S78048[38], 6}, {NULL}};
const Segment SGI193842[] = {{&S78048[90], 5}, {NULL}};
const Segment SGI193843[] = {{&S78048[0], 14}, {NULL}};
const Segment SGI193844[] = {{&S67121[34], 10}, {NULL}};
const Segment SGI193845[] = {{&S67121[27], 6}, {NULL}};
const Segment SGI193846[] = {{&S78049[52], 8}, {NULL}};
const Segment SGI193847[] = {{&S67121[97], 9}, {NULL}};
const Segment SGI193848[] = {{&S78049[52], 24}, {NULL}};
const Segment SGI193849[] = {{&S67121[70], 6}, {NULL}};
const Segment SGI193850[] = {{&S78049[38], 6}, {NULL}};
const Segment SGI193851[] = {{&S78049[90], 5}, {NULL}};
const Segment SGI193852[] = {{&S78049[0], 14}, {NULL}};
const Segment SGI193853[] = {{&S67122[34], 10}, {NULL}};
const Segment SGI193854[] = {{&S67122[27], 6}, {NULL}};
const Segment SGI193855[] = {{&S78050[52], 7}, {NULL}};
const Segment SGI193856[] = {{&S67122[97], 9}, {NULL}};
const Segment SGI193857[] = {{&S78050[52], 24}, {NULL}};
const Segment SGI193858[] = {{&S67122[70], 6}, {NULL}};
const Segment SGI193859[] = {{&S78050[38], 6}, {NULL}};
const Segment SGI193860[] = {{&S56760[44], 5}, {NULL}};
const Segment SGI193861[] = {{&S56760[44], 19}, {NULL}};
const Segment SGI193862[] = {{&S56760[30], 6}, {NULL}};
const Segment SGI193863[] = {{&S78050[90], 5}, {NULL}};
const Segment SGI193864[] = {{&S56760[112], 3}, {NULL}};
const Segment SGI193865[] = {{&S56760[77], 6}, {NULL}};
const Segment SGI193866[] = {{&S78050[0], 14}, {NULL}};
const Segment SGI193867[] = {{&S78051[52], 7}, {NULL}};
const Segment SGI193868[] = {{&S78051[52], 24}, {NULL}};
const Segment SGI193869[] = {{&S78051[38], 6}, {NULL}};
const Segment SGI193870[] = {{&S56761[44], 5}, {NULL}};
const Segment SGI193871[] = {{&S56761[44], 19}, {NULL}};
const Segment SGI193872[] = {{&S56761[30], 6}, {NULL}};
const Segment SGI193873[] = {{&S78051[90], 5}, {NULL}};
const Segment SGI193874[] = {{&S56761[112], 3}, {NULL}};
const Segment SGI193875[] = {{&S56761[77], 6}, {NULL}};
const Segment SGI193876[] = {{&S78051[0], 14}, {NULL}};
const Segment SGI193877[] = {{&S56762[44], 12}, {NULL}};
const Segment SGI193878[] = {{&S56762[44], 26}, {NULL}};
const Segment SGI193879[] = {{&S56762[30], 6}, {NULL}};
const Segment SGI193880[] = {{&S56762[119], 3}, {NULL}};
const Segment SGI193881[] = {{&S56762[84], 6}, {NULL}};
const Segment SGI193882[] = {{&S56763[38], 5}, {NULL}};
const Segment SGI193883[] = {{&S56763[31], 6}, {NULL}};
const Segment SGI193884[] = {{&S56764[44], 9}, {NULL}};
const Segment SGI193885[] = {{&S56764[44], 20}, {NULL}};
const Segment SGI193886[] = {{&S56764[30], 6}, {NULL}};
const Segment SGI193887[] = {{&S56764[113], 3}, {NULL}};
const Segment SGI193888[] = {{&S56764[78], 6}, {NULL}};
const Segment SGI193889[] = {{&S56765[44], 5}, {NULL}};
const Segment SGI193890[] = {{&S56765[44], 16}, {NULL}};
const Segment SGI193891[] = {{&S56765[30], 6}, {NULL}};
const Segment SGI193892[] = {{&S56765[108], 3}, {NULL}};
const Segment SGI193893[] = {{&S56765[74], 5}, {NULL}};
const Segment SGI193894[] = {{&S56766[44], 9}, {NULL}};
const Segment SGI193895[] = {{&S56766[44], 20}, {NULL}};
const Segment SGI193896[] = {{&S56766[30], 6}, {NULL}};
const Segment SGI193897[] = {{&S56766[113], 3}, {NULL}};
const Segment SGI193898[] = {{&S56766[78], 6}, {NULL}};
const Segment SGI193899[] = {{&S56776[41], 19}, {NULL}};
const Segment SGI193900[] = {{&S56776[41], 30}, {NULL}};
const Segment SGI193901[] = {{&S56776[27], 6}, {NULL}};
const Segment SGI193902[] = {{&S56776[120], 3}, {NULL}};
const Segment SGI193903[] = {{&S56776[124], 9}, {NULL}};
const Segment SGI193904[] = {{&S56776[85], 6}, {NULL}};
const Segment SGI193905[] = {{&S78078[44], 5}, {NULL}};
const Segment SGI193906[] = {{&S78078[44], 17}, {NULL}};
const Segment SGI193907[] = {{&S78078[30], 6}, {NULL}};
const Segment SGI193908[] = {{&S78078[31], 3}, {NULL}};
const Segment SGI193909[] = {{&S78078[75], 6}, {NULL}};
const Segment SGI193910[] = {{&S78080[6], 4}, {NULL}};
const Segment SGI193911[] = {{&S78080[81], 5}, {NULL}};
const Segment SGI193912[] = {{&S78084[44], 12}, {NULL}};
const Segment SGI193913[] = {{&S78084[44], 24}, {NULL}};
const Segment SGI193914[] = {{&S78084[30], 6}, {NULL}};
const Segment SGI193915[] = {{&S78084[31], 3}, {NULL}};
const Segment SGI193916[] = {{&S78084[82], 6}, {NULL}};
const Segment SGI193917[] = {{&S78085[34], 7}, {NULL}};
const Segment SGI193918[] = {{&S78085[27], 6}, {NULL}};
const Segment SGI193919[] = {{&S78085[95], 9}, {NULL}};
const Segment SGI193920[] = {{&S78085[68], 6}, {NULL}};
const Segment SGI193921[] = {{&S78087[44], 13}, {NULL}};
const Segment SGI193922[] = {{&S78087[44], 25}, {NULL}};
const Segment SGI193923[] = {{&S78087[30], 6}, {NULL}};
const Segment SGI193924[] = {{&S78087[31], 3}, {NULL}};
const Segment SGI193925[] = {{&S78087[83], 6}, {NULL}};
const Segment SGI193926[] = {{&S78088[44], 13}, {NULL}};
const Segment SGI193927[] = {{&S78088[44], 25}, {NULL}};
const Segment SGI193928[] = {{&S78088[30], 6}, {NULL}};
const Segment SGI193929[] = {{&S78088[86], 6}, {NULL}};
const Segment SGI193930[] = {{&S78089[38], 13}, {NULL}};
const Segment SGI193931[] = {{&S78089[31], 6}, {NULL}};
const Segment SGI193932[] = {{&S78090[34], 7}, {NULL}};
const Segment SGI193933[] = {{&S78090[27], 6}, {NULL}};
const Segment SGI193934[] = {{&S78090[94], 9}, {NULL}};
const Segment SGI193935[] = {{&S78090[67], 6}, {NULL}};
const Segment SGI193936[] = {{&S78091[34], 10}, {NULL}};
const Segment SGI193937[] = {{&S78091[27], 6}, {NULL}};
const Segment SGI193938[] = {{&S78091[97], 9}, {NULL}};
const Segment SGI193939[] = {{&S78091[70], 6}, {NULL}};
const Segment SGI193940[] = {{&S78096[34], 15}, {NULL}};
const Segment SGI193941[] = {{&S78096[27], 6}, {NULL}};
const Segment SGI193942[] = {{&S78096[108], 9}, {NULL}};
const Segment SGI193943[] = {{&S78096[81], 6}, {NULL}};
const Segment SGI193944[] = {{&S78097[34], 9}, {NULL}};
const Segment SGI193945[] = {{&S78097[27], 6}, {NULL}};
const Segment SGI193946[] = {{&S78097[96], 9}, {NULL}};
const Segment SGI193947[] = {{&S78097[69], 6}, {NULL}};
const Segment SGI193948[] = {{&S78101[34], 7}, {NULL}};
const Segment SGI193949[] = {{&S78101[27], 6}, {NULL}};
const Segment SGI193950[] = {{&S78101[99], 9}, {NULL}};
const Segment SGI193951[] = {{&S78101[72], 6}, {NULL}};
const Segment SGI193952[] = {{&S67190[44], 18}, {NULL}};
const Segment SGI193953[] = {{&S67190[44], 30}, {NULL}};
const Segment SGI193954[] = {{&S67190[30], 6}, {NULL}};
const Segment SGI193955[] = {{&S67190[144], 3}, {NULL}};
const Segment SGI193956[] = {{&S67190[109], 6}, {NULL}};
const Segment SGI193957[] = {{&S78104[34], 10}, {NULL}};
const Segment SGI193958[] = {{&S78104[27], 6}, {NULL}};
const Segment SGI193959[] = {{&S78104[97], 9}, {NULL}};
const Segment SGI193960[] = {{&S78104[70], 6}, {NULL}};
const Segment SGI193961[] = {{&S67192[44], 10}, {NULL}};
const Segment SGI193962[] = {{&S78105[34], 11}, {NULL}};
const Segment SGI193963[] = {{&S67192[44], 22}, {NULL}};
const Segment SGI193964[] = {{&S67192[30], 6}, {NULL}};
const Segment SGI193965[] = {{&S78105[27], 6}, {NULL}};
const Segment SGI193966[] = {{&S67192[136], 3}, {NULL}};
const Segment SGI193967[] = {{&S78105[98], 9}, {NULL}};
const Segment SGI193968[] = {{&S67192[101], 6}, {NULL}};
const Segment SGI193969[] = {{&S78105[71], 6}, {NULL}};
const Segment SGI193970[] = {{&S78105[139], 8}, {NULL}};
const Segment SGI193971[] = {{&S78107[34], 11}, {NULL}};
const Segment SGI193972[] = {{&S78107[27], 6}, {NULL}};
const Segment SGI193973[] = {{&S78107[98], 9}, {NULL}};
const Segment SGI193974[] = {{&S78107[71], 6}, {NULL}};
const Segment SGI193975[] = {{&S78107[139], 8}, {NULL}};
const Segment SGI193976[] = {{&S65922[34], 8}, {NULL}};
const Segment SGI193977[] = {{&S65922[27], 6}, {NULL}};
const Segment SGI193978[] = {{&S65922[95], 9}, {NULL}};
const Segment SGI193979[] = {{&S65922[68], 6}, {NULL}};
const Segment SGI193980[] = {{&S65924[34], 16}, {NULL}};
const Segment SGI193981[] = {{&S65924[27], 6}, {NULL}};
const Segment SGI193982[] = {{&S65924[103], 9}, {NULL}};
const Segment SGI193983[] = {{&S65924[76], 6}, {NULL}};
const Segment SGI193984[] = {{&S65925[34], 16}, {NULL}};
const Segment SGI193985[] = {{&S65925[27], 6}, {NULL}};
const Segment SGI193986[] = {{&S65925[103], 9}, {NULL}};
const Segment SGI193987[] = {{&S65925[76], 6}, {NULL}};
const Segment SGI193988[] = {{&S78136[44], 9}, {NULL}};
const Segment SGI193989[] = {{&S78136[44], 21}, {NULL}};
const Segment SGI193990[] = {{&S78136[30], 6}, {NULL}};
const Segment SGI193991[] = {{&S78136[31], 3}, {NULL}};
const Segment SGI193992[] = {{&S78136[79], 6}, {NULL}};
const Segment SGI193993[] = {{&S56862[34], 9}, {NULL}};
const Segment SGI193994[] = {{&S56862[27], 6}, {NULL}};
const Segment SGI193995[] = {{&S56862[95], 9}, {NULL}};
const Segment SGI193996[] = {{&S56862[68], 6}, {NULL}};
const Segment SGI193997[] = {{&S56863[34], 11}, {NULL}};
const Segment SGI193998[] = {{&S56863[27], 6}, {NULL}};
const Segment SGI193999[] = {{&S78139[34], 7}, {NULL}};
const Segment SGI194000[] = {{&S78139[27], 6}, {NULL}};
const Segment SGI194001[] = {{&S56863[97], 9}, {NULL}};
const Segment SGI194002[] = {{&S56863[70], 6}, {NULL}};
const Segment SGI194003[] = {{&S78139[111], 9}, {NULL}};
const Segment SGI194004[] = {{&S78139[84], 6}, {NULL}};
const Segment SGI194005[] = {{&S71661[44], 7}, {NULL}};
const Segment SGI194006[] = {{&S71661[44], 19}, {NULL}};
const Segment SGI194007[] = {{&S71661[30], 6}, {NULL}};
const Segment SGI194008[] = {{&S71661[112], 3}, {NULL}};
const Segment SGI194009[] = {{&S71661[77], 6}, {NULL}};
const Segment SGI194010[] = {{&S78151[34], 7}, {NULL}};
const Segment SGI194011[] = {{&S78151[27], 6}, {NULL}};
const Segment SGI194012[] = {{&S78151[94], 9}, {NULL}};
const Segment SGI194013[] = {{&S78151[67], 6}, {NULL}};
const Segment SGI194014[] = {{&S71662[44], 10}, {NULL}};
const Segment SGI194015[] = {{&S71662[44], 22}, {NULL}};
const Segment SGI194016[] = {{&S71662[30], 6}, {NULL}};
const Segment SGI194017[] = {{&S71662[115], 3}, {NULL}};
const Segment SGI194018[] = {{&S71662[80], 6}, {NULL}};
const Segment SGI194019[] = {{&S78152[44], 5}, {NULL}};
const Segment SGI194020[] = {{&S78152[44], 17}, {NULL}};
const Segment SGI194021[] = {{&S78152[30], 6}, {NULL}};
const Segment SGI194022[] = {{&S71663[44], 8}, {NULL}};
const Segment SGI194023[] = {{&S78152[75], 6}, {NULL}};
const Segment SGI194024[] = {{&S71663[44], 20}, {NULL}};
const Segment SGI194025[] = {{&S71663[30], 6}, {NULL}};
const Segment SGI194026[] = {{&S78152[102], 13}, {NULL}};
const Segment SGI194027[] = {{&S71663[113], 3}, {NULL}};
const Segment SGI194028[] = {{&S71663[78], 6}, {NULL}};
const Segment SGI194029[] = {{&S71675[44], 7}, {NULL}};
const Segment SGI194030[] = {{&S71675[44], 19}, {NULL}};
const Segment SGI194031[] = {{&S71675[30], 6}, {NULL}};
const Segment SGI194032[] = {{&S71675[128], 3}, {NULL}};
const Segment SGI194033[] = {{&S71675[93], 6}, {NULL}};
const Segment SGI194034[] = {{&S71675[65], 8}, {NULL}};
const Segment SGI194035[] = {{&S67210[44], 10}, {NULL}};
const Segment SGI194036[] = {{&S67210[44], 22}, {NULL}};
const Segment SGI194037[] = {{&S67210[30], 6}, {NULL}};
const Segment SGI194038[] = {{&S67210[131], 3}, {NULL}};
const Segment SGI194039[] = {{&S67210[96], 6}, {NULL}};
const Segment SGI194040[] = {{&S67210[68], 8}, {NULL}};
const Segment SGI194041[] = {{&S67215[42], 10}, {NULL}};
const Segment SGI194042[] = {{&S67215[30], 6}, {NULL}};
const Segment SGI194043[] = {{&S67215[129], 3}, {NULL}};
const Segment SGI194044[] = {{&S67215[94], 6}, {NULL}};
const Segment SGI194045[] = {{&S67215[66], 8}, {NULL}};
const Segment SGI194046[] = {{&S78180[44], 18}, {NULL}};
const Segment SGI194047[] = {{&S78180[44], 30}, {NULL}};
const Segment SGI194048[] = {{&S78180[30], 6}, {NULL}};
const Segment SGI194049[] = {{&S78180[31], 3}, {NULL}};
const Segment SGI194050[] = {{&S78180[88], 6}, {NULL}};
const Segment SGI194051[] = {{&S67218[34], 10}, {NULL}};
const Segment SGI194052[] = {{&S67218[27], 6}, {NULL}};
const Segment SGI194053[] = {{&S67218[97], 9}, {NULL}};
const Segment SGI194054[] = {{&S67218[70], 6}, {NULL}};
const Segment SGI194055[] = {{&S78181[44], 12}, {NULL}};
const Segment SGI194056[] = {{&S78181[44], 24}, {NULL}};
const Segment SGI194057[] = {{&S78181[30], 6}, {NULL}};
const Segment SGI194058[] = {{&S78181[31], 3}, {NULL}};
const Segment SGI194059[] = {{&S78181[82], 6}, {NULL}};
const Segment SGI194060[] = {{&S71691[34], 25}, {NULL}};
const Segment SGI194061[] = {{&S71691[27], 6}, {NULL}};
const Segment SGI194062[] = {{&S71691[112], 9}, {NULL}};
const Segment SGI194063[] = {{&S71691[85], 6}, {NULL}};
const Segment SGI194064[] = {{&S78184[44], 18}, {NULL}};
const Segment SGI194065[] = {{&S78184[44], 30}, {NULL}};
const Segment SGI194066[] = {{&S78184[30], 6}, {NULL}};
const Segment SGI194067[] = {{&S78184[31], 3}, {NULL}};
const Segment SGI194068[] = {{&S78184[88], 6}, {NULL}};
const Segment SGI194069[] = {{&S78186[44], 18}, {NULL}};
const Segment SGI194070[] = {{&S78186[44], 30}, {NULL}};
const Segment SGI194071[] = {{&S78186[30], 6}, {NULL}};
const Segment SGI194072[] = {{&S78186[31], 3}, {NULL}};
const Segment SGI194073[] = {{&S78186[88], 6}, {NULL}};
const Segment SGI194074[] = {{&S71695[34], 7}, {NULL}};
const Segment SGI194075[] = {{&S71695[27], 6}, {NULL}};
const Segment SGI194076[] = {{&S71695[98], 9}, {NULL}};
const Segment SGI194077[] = {{&S71695[71], 6}, {NULL}};
const Segment SGI194078[] = {{&S78192[44], 7}, {NULL}};
const Segment SGI194079[] = {{&S78192[44], 15}, {NULL}};
const Segment SGI194080[] = {{&S78192[30], 6}, {NULL}};
const Segment SGI194081[] = {{&S78192[31], 3}, {NULL}};
const Segment SGI194082[] = {{&S78192[73], 6}, {NULL}};
const Segment SGI194083[] = {{&S78193[38], 7}, {NULL}};
const Segment SGI194084[] = {{&S78193[31], 6}, {NULL}};
const Segment SGI194085[] = {{&S67237[34], 10}, {NULL}};
const Segment SGI194086[] = {{&S67237[27], 6}, {NULL}};
const Segment SGI194087[] = {{&S67237[97], 9}, {NULL}};
const Segment SGI194088[] = {{&S67237[70], 6}, {NULL}};
const Segment SGI194089[] = {{&S78201[44], 9}, {NULL}};
const Segment SGI194090[] = {{&S67244[34], 10}, {NULL}};
const Segment SGI194091[] = {{&S78201[44], 21}, {NULL}};
const Segment SGI194092[] = {{&S78201[30], 6}, {NULL}};
const Segment SGI194093[] = {{&S67244[27], 6}, {NULL}};
const Segment SGI194094[] = {{&S78201[31], 3}, {NULL}};
const Segment SGI194095[] = {{&S78201[79], 6}, {NULL}};
const Segment SGI194096[] = {{&S78202[44], 9}, {NULL}};
const Segment SGI194097[] = {{&S78202[44], 21}, {NULL}};
const Segment SGI194098[] = {{&S78202[30], 6}, {NULL}};
const Segment SGI194099[] = {{&S78202[82], 6}, {NULL}};
const Segment SGI194100[] = {{&S78203[38], 9}, {NULL}};
const Segment SGI194101[] = {{&S78203[31], 6}, {NULL}};
const Segment SGI194102[] = {{&S78204[44], 21}, {NULL}};
const Segment SGI194103[] = {{&S78204[30], 6}, {NULL}};
const Segment SGI194104[] = {{&S78205[34], 9}, {NULL}};
const Segment SGI194105[] = {{&S78205[27], 6}, {NULL}};
const Segment SGI194106[] = {{&S78205[96], 9}, {NULL}};
const Segment SGI194107[] = {{&S78205[69], 6}, {NULL}};
const Segment SGI194108[] = {{&S78210[34], 7}, {NULL}};
const Segment SGI194109[] = {{&S78210[27], 6}, {NULL}};
const Segment SGI194110[] = {{&S78210[105], 9}, {NULL}};
const Segment SGI194111[] = {{&S78210[78], 6}, {NULL}};
const Segment SGI194112[] = {{&S56975[34], 11}, {NULL}};
const Segment SGI194113[] = {{&S56975[27], 6}, {NULL}};
const Segment SGI194114[] = {{&S56975[97], 9}, {NULL}};
const Segment SGI194115[] = {{&S56975[70], 6}, {NULL}};
const Segment SGI194116[] = {{&S56976[34], 11}, {NULL}};
const Segment SGI194117[] = {{&S56976[27], 6}, {NULL}};
const Segment SGI194118[] = {{&S56976[97], 9}, {NULL}};
const Segment SGI194119[] = {{&S56976[70], 6}, {NULL}};
const Segment SGI194120[] = {{&S56978[34], 14}, {NULL}};
const Segment SGI194121[] = {{&S56978[27], 6}, {NULL}};
const Segment SGI194122[] = {{&S56978[100], 9}, {NULL}};
const Segment SGI194123[] = {{&S56978[73], 6}, {NULL}};
const Segment SGI194124[] = {{&S71748[44], 9}, {NULL}};
const Segment SGI194125[] = {{&S71748[44], 21}, {NULL}};
const Segment SGI194126[] = {{&S71748[30], 6}, {NULL}};
const Segment SGI194127[] = {{&S71748[114], 3}, {NULL}};
const Segment SGI194128[] = {{&S71748[79], 6}, {NULL}};
const Segment SGI194129[] = {{&S71753[44], 10}, {NULL}};
const Segment SGI194130[] = {{&S71753[44], 22}, {NULL}};
const Segment SGI194131[] = {{&S71753[30], 6}, {NULL}};
const Segment SGI194132[] = {{&S71753[115], 3}, {NULL}};
const Segment SGI194133[] = {{&S78239[34], 7}, {NULL}};
const Segment SGI194134[] = {{&S71753[80], 6}, {NULL}};
const Segment SGI194135[] = {{&S78239[27], 6}, {NULL}};
const Segment SGI194136[] = {{&S78239[94], 9}, {NULL}};
const Segment SGI194137[] = {{&S78239[67], 6}, {NULL}};
const Segment SGI194138[] = {{&S78246[34], 14}, {NULL}};
const Segment SGI194139[] = {{&S78246[27], 6}, {NULL}};
const Segment SGI194140[] = {{&S66062[34], 10}, {NULL}};
const Segment SGI194141[] = {{&S66062[27], 6}, {NULL}};
const Segment SGI194142[] = {{&S78246[101], 9}, {NULL}};
const Segment SGI194143[] = {{&S78246[74], 6}, {NULL}};
const Segment SGI194144[] = {{&S66062[97], 9}, {NULL}};
const Segment SGI194145[] = {{&S66062[70], 6}, {NULL}};
const Segment SGI194146[] = {{&S78257[34], 9}, {NULL}};
const Segment SGI194147[] = {{&S78257[27], 6}, {NULL}};
const Segment SGI194148[] = {{&S78257[96], 9}, {NULL}};
const Segment SGI194149[] = {{&S78257[69], 6}, {NULL}};
const Segment SGI194150[] = {{&S78258[34], 10}, {NULL}};
const Segment SGI194151[] = {{&S78258[27], 6}, {NULL}};
const Segment SGI194152[] = {{&S78258[97], 9}, {NULL}};
const Segment SGI194153[] = {{&S78258[70], 6}, {NULL}};
const Segment SGI194154[] = {{&S57017[44], 10}, {NULL}};
const Segment SGI194155[] = {{&S57017[44], 21}, {NULL}};
const Segment SGI194156[] = {{&S78262[44], 10}, {NULL}};
const Segment SGI194157[] = {{&S57017[30], 6}, {NULL}};
const Segment SGI194158[] = {{&S78262[44], 22}, {NULL}};
const Segment SGI194159[] = {{&S78262[30], 6}, {NULL}};
const Segment SGI194160[] = {{&S57017[132], 3}, {NULL}};
const Segment SGI194161[] = {{&S57017[97], 6}, {NULL}};
const Segment SGI194162[] = {{&S78262[80], 6}, {NULL}};
const Segment SGI194163[] = {{&S57017[67], 8}, {NULL}};
const Segment SGI194164[] = {{&S78262[107], 13}, {NULL}};
const Segment SGI194165[] = {{&S57018[44], 10}, {NULL}};
const Segment SGI194166[] = {{&S57018[44], 22}, {NULL}};
const Segment SGI194167[] = {{&S57018[30], 6}, {NULL}};
const Segment SGI194168[] = {{&S57018[133], 3}, {NULL}};
const Segment SGI194169[] = {{&S57018[98], 6}, {NULL}};
const Segment SGI194170[] = {{&S57018[68], 8}, {NULL}};
const Segment SGI194171[] = {{&S78267[44], 10}, {NULL}};
const Segment SGI194172[] = {{&S78267[44], 22}, {NULL}};
const Segment SGI194173[] = {{&S78267[30], 6}, {NULL}};
const Segment SGI194174[] = {{&S75791[6], 5}, {NULL}};
const Segment SGI194175[] = {{&S78267[80], 6}, {NULL}};
const Segment SGI194176[] = {{&S78267[107], 13}, {NULL}};
const Segment SGI194177[] = {{&S71797[44], 10}, {NULL}};
const Segment SGI194178[] = {{&S71797[44], 22}, {NULL}};
const Segment SGI194179[] = {{&S71797[30], 6}, {NULL}};
const Segment SGI194180[] = {{&S71797[115], 3}, {NULL}};
const Segment SGI194181[] = {{&S71797[80], 6}, {NULL}};
const Segment SGI194182[] = {{&S71798[44], 10}, {NULL}};
const Segment SGI194183[] = {{&S71798[44], 22}, {NULL}};
const Segment SGI194184[] = {{&S71798[30], 6}, {NULL}};
const Segment SGI194185[] = {{&S71798[115], 3}, {NULL}};
const Segment SGI194186[] = {{&S71798[80], 6}, {NULL}};
const Segment SGI194187[] = {{&S71799[44], 10}, {NULL}};
const Segment SGI194188[] = {{&S71799[44], 22}, {NULL}};
const Segment SGI194189[] = {{&S71799[30], 6}, {NULL}};
const Segment SGI194190[] = {{&S71799[115], 3}, {NULL}};
const Segment SGI194191[] = {{&S71799[80], 6}, {NULL}};
const Segment SGI194192[] = {{&S71800[44], 10}, {NULL}};
const Segment SGI194193[] = {{&S71800[44], 24}, {NULL}};
const Segment SGI194194[] = {{&S71800[30], 6}, {NULL}};
const Segment SGI194195[] = {{&S71800[117], 3}, {NULL}};
const Segment SGI194196[] = {{&S71800[82], 6}, {NULL}};
const Segment SGI194197[] = {{&S78296[44], 6}, {NULL}};
const Segment SGI194198[] = {{&S78296[44], 18}, {NULL}};
const Segment SGI194199[] = {{&S78296[30], 6}, {NULL}};
const Segment SGI194200[] = {{&S78296[31], 3}, {NULL}};
const Segment SGI194201[] = {{&S78296[76], 6}, {NULL}};
const Segment SGI194202[] = {{&S78297[44], 7}, {NULL}};
const Segment SGI194203[] = {{&S78297[44], 19}, {NULL}};
const Segment SGI194204[] = {{&S78297[30], 6}, {NULL}};
const Segment SGI194205[] = {{&S78297[31], 3}, {NULL}};
const Segment SGI194206[] = {{&S78297[77], 6}, {NULL}};
const Segment SGI194207[] = {{&S78298[44], 9}, {NULL}};
const Segment SGI194208[] = {{&S78298[44], 21}, {NULL}};
const Segment SGI194209[] = {{&S78298[30], 6}, {NULL}};
const Segment SGI194210[] = {{&S78298[31], 3}, {NULL}};
const Segment SGI194211[] = {{&S78298[94], 6}, {NULL}};
const Segment SGI194212[] = {{&S78298[67], 8}, {NULL}};
const Segment SGI194213[] = {{&S78300[44], 13}, {NULL}};
const Segment SGI194214[] = {{&S78300[44], 25}, {NULL}};
const Segment SGI194215[] = {{&S78300[30], 6}, {NULL}};
const Segment SGI194216[] = {{&S78300[31], 3}, {NULL}};
const Segment SGI194217[] = {{&S78300[83], 6}, {NULL}};
const Segment SGI194218[] = {{&S78301[44], 8}, {NULL}};
const Segment SGI194219[] = {{&S78301[44], 20}, {NULL}};
const Segment SGI194220[] = {{&S78301[30], 6}, {NULL}};
const Segment SGI194221[] = {{&S78301[31], 3}, {NULL}};
const Segment SGI194222[] = {{&S78301[78], 6}, {NULL}};
const Segment SGI194223[] = {{&S78306[44], 19}, {NULL}};
const Segment SGI194224[] = {{&S78306[44], 31}, {NULL}};
const Segment SGI194225[] = {{&S78306[30], 6}, {NULL}};
const Segment SGI194226[] = {{&S78306[31], 3}, {NULL}};
const Segment SGI194227[] = {{&S78306[89], 6}, {NULL}};
const Segment SGI194228[] = {{&S71835[34], 25}, {NULL}};
const Segment SGI194229[] = {{&S71835[27], 6}, {NULL}};
const Segment SGI194230[] = {{&S71835[112], 9}, {NULL}};
const Segment SGI194231[] = {{&S71835[85], 6}, {NULL}};
const Segment SGI194232[] = {{&S71836[34], 25}, {NULL}};
const Segment SGI194233[] = {{&S71836[27], 6}, {NULL}};
const Segment SGI194234[] = {{&S71836[112], 9}, {NULL}};
const Segment SGI194235[] = {{&S71836[85], 6}, {NULL}};
const Segment SGI194236[] = {{&S71837[34], 25}, {NULL}};
const Segment SGI194237[] = {{&S71837[27], 6}, {NULL}};
const Segment SGI194238[] = {{&S71837[112], 9}, {NULL}};
const Segment SGI194239[] = {{&S71837[85], 6}, {NULL}};
const Segment SGI194240[] = {{&S71862[44], 16}, {NULL}};
const Segment SGI194241[] = {{&S71862[44], 28}, {NULL}};
const Segment SGI194242[] = {{&S71862[30], 6}, {NULL}};
const Segment SGI194243[] = {{&S71862[121], 3}, {NULL}};
const Segment SGI194244[] = {{&S71862[86], 6}, {NULL}};
const Segment SGI194245[] = {{&S71867[44], 15}, {NULL}};
const Segment SGI194246[] = {{&S71867[44], 27}, {NULL}};
const Segment SGI194247[] = {{&S71867[30], 6}, {NULL}};
const Segment SGI194248[] = {{&S71867[120], 3}, {NULL}};
const Segment SGI194249[] = {{&S71867[85], 6}, {NULL}};
const Segment SGI194250[] = {{&S78342[44], 6}, {NULL}};
const Segment SGI194251[] = {{&S78342[44], 18}, {NULL}};
const Segment SGI194252[] = {{&S78342[30], 6}, {NULL}};
const Segment SGI194253[] = {{&S78342[31], 3}, {NULL}};
const Segment SGI194254[] = {{&S71868[44], 17}, {NULL}};
const Segment SGI194255[] = {{&S71868[44], 29}, {NULL}};
const Segment SGI194256[] = {{&S78342[76], 6}, {NULL}};
const Segment SGI194257[] = {{&S71868[30], 6}, {NULL}};
const Segment SGI194258[] = {{&S71868[122], 3}, {NULL}};
const Segment SGI194259[] = {{&S71868[87], 6}, {NULL}};
const Segment SGI194260[] = {{&S71869[44], 23}, {NULL}};
const Segment SGI194261[] = {{&S71869[44], 35}, {NULL}};
const Segment SGI194262[] = {{&S71869[30], 6}, {NULL}};
const Segment SGI194263[] = {{&S71869[128], 3}, {NULL}};
const Segment SGI194264[] = {{&S71869[93], 6}, {NULL}};
const Segment SGI194265[] = {{&S71870[44], 13}, {NULL}};
const Segment SGI194266[] = {{&S71870[44], 25}, {NULL}};
const Segment SGI194267[] = {{&S71870[30], 6}, {NULL}};
const Segment SGI194268[] = {{&S71870[118], 3}, {NULL}};
const Segment SGI194269[] = {{&S71870[83], 6}, {NULL}};
const Segment SGI194270[] = {{&S71872[38], 17}, {NULL}};
const Segment SGI194271[] = {{&S71872[31], 6}, {NULL}};
const Segment SGI194272[] = {{&S67244[97], 9}, {NULL}};
const Segment SGI194273[] = {{&S67244[70], 6}, {NULL}};
const Segment SGI194274[] = {{&S57111[41], 17}, {NULL}};
const Segment SGI194275[] = {{&S57111[41], 28}, {NULL}};
const Segment SGI194276[] = {{&S57111[27], 6}, {NULL}};
const Segment SGI194277[] = {{&S57111[118], 3}, {NULL}};
const Segment SGI194278[] = {{&S57111[122], 9}, {NULL}};
const Segment SGI194279[] = {{&S57111[83], 6}, {NULL}};
const Segment SGI194280[] = {{&S78350[44], 9}, {NULL}};
const Segment SGI194281[] = {{&S78350[44], 21}, {NULL}};
const Segment SGI194282[] = {{&S78350[30], 6}, {NULL}};
const Segment SGI194283[] = {{&S78350[31], 3}, {NULL}};
const Segment SGI194284[] = {{&S78350[79], 6}, {NULL}};
const Segment SGI194285[] = {{&S67247[34], 10}, {NULL}};
const Segment SGI194286[] = {{&S67247[27], 6}, {NULL}};
const Segment SGI194287[] = {{&S78351[45], 5}, {NULL}};
const Segment SGI194288[] = {{&S78351[30], 6}, {NULL}};
const Segment SGI194289[] = {{&S67247[97], 9}, {NULL}};
const Segment SGI194290[] = {{&S67247[70], 6}, {NULL}};
const Segment SGI194291[] = {{&S78351[31], 3}, {NULL}};
const Segment SGI194292[] = {{&S78351[76], 5}, {NULL}};
const Segment SGI194293[] = {{&S78351[134], 13}, {NULL}};
const Segment SGI194294[] = {{&S78352[45], 13}, {NULL}};
const Segment SGI194295[] = {{&S78352[30], 6}, {NULL}};
const Segment SGI194296[] = {{&S67249[34], 11}, {NULL}};
const Segment SGI194297[] = {{&S67249[27], 6}, {NULL}};
const Segment SGI194298[] = {{&S78352[31], 3}, {NULL}};
const Segment SGI194299[] = {{&S78352[84], 5}, {NULL}};
const Segment SGI194300[] = {{&S67249[98], 9}, {NULL}};
const Segment SGI194301[] = {{&S78352[142], 13}, {NULL}};
const Segment SGI194302[] = {{&S67249[71], 6}, {NULL}};
const Segment SGI194303[] = {{&S78353[44], 5}, {NULL}};
const Segment SGI194304[] = {{&S78353[44], 17}, {NULL}};
const Segment SGI194305[] = {{&S67250[34], 11}, {NULL}};
const Segment SGI194306[] = {{&S78353[30], 6}, {NULL}};
const Segment SGI194307[] = {{&S67250[27], 6}, {NULL}};
const Segment SGI194308[] = {{&S78353[110], 3}, {NULL}};
const Segment SGI194309[] = {{&S67250[98], 9}, {NULL}};
const Segment SGI194310[] = {{&S78353[75], 6}, {NULL}};
const Segment SGI194311[] = {{&S67250[71], 6}, {NULL}};
const Segment SGI194312[] = {{&S78353[130], 8}, {NULL}};
const Segment SGI194313[] = {{&S78353[139], 13}, {NULL}};
const Segment SGI194314[] = {{&S67251[34], 11}, {NULL}};
const Segment SGI194315[] = {{&S78354[43], 9}, {NULL}};
const Segment SGI194316[] = {{&S67251[27], 6}, {NULL}};
const Segment SGI194317[] = {{&S78354[43], 21}, {NULL}};
const Segment SGI194318[] = {{&S78354[29], 6}, {NULL}};
const Segment SGI194319[] = {{&S67251[98], 9}, {NULL}};
const Segment SGI194320[] = {{&S78354[30], 3}, {NULL}};
const Segment SGI194321[] = {{&S67251[71], 6}, {NULL}};
const Segment SGI194322[] = {{&S78354[78], 6}, {NULL}};
const Segment SGI194323[] = {{&S78355[44], 10}, {NULL}};
const Segment SGI194324[] = {{&S78355[44], 22}, {NULL}};
const Segment SGI194325[] = {{&S78355[30], 6}, {NULL}};
const Segment SGI194326[] = {{&S78355[31], 3}, {NULL}};
const Segment SGI194327[] = {{&S78355[80], 6}, {NULL}};
const Segment SGI194328[] = {{&S78356[38], 10}, {NULL}};
const Segment SGI194329[] = {{&S78356[31], 6}, {NULL}};
const Segment SGI194330[] = {{&S67257[34], 10}, {NULL}};
const Segment SGI194331[] = {{&S67257[27], 6}, {NULL}};
const Segment SGI194332[] = {{&S67257[97], 9}, {NULL}};
const Segment SGI194333[] = {{&S67257[70], 6}, {NULL}};
const Segment SGI194334[] = {{&S57131[44], 14}, {NULL}};
const Segment SGI194335[] = {{&S57131[44], 25}, {NULL}};
const Segment SGI194336[] = {{&S57131[30], 6}, {NULL}};
const Segment SGI194337[] = {{&S57131[136], 3}, {NULL}};
const Segment SGI194338[] = {{&S57131[101], 6}, {NULL}};
const Segment SGI194339[] = {{&S57131[71], 8}, {NULL}};
const Segment SGI194340[] = {{&S78362[34], 10}, {NULL}};
const Segment SGI194341[] = {{&S78362[27], 6}, {NULL}};
const Segment SGI194342[] = {{&S57132[44], 14}, {NULL}};
const Segment SGI194343[] = {{&S57132[44], 26}, {NULL}};
const Segment SGI194344[] = {{&S57132[30], 6}, {NULL}};
const Segment SGI194345[] = {{&S78362[97], 9}, {NULL}};
const Segment SGI194346[] = {{&S78362[70], 6}, {NULL}};
const Segment SGI194347[] = {{&S57132[137], 3}, {NULL}};
const Segment SGI194348[] = {{&S57132[102], 6}, {NULL}};
const Segment SGI194349[] = {{&S57132[72], 8}, {NULL}};
const Segment SGI194350[] = {{&S57133[44], 20}, {NULL}};
const Segment SGI194351[] = {{&S57133[44], 32}, {NULL}};
const Segment SGI194352[] = {{&S57133[30], 6}, {NULL}};
const Segment SGI194353[] = {{&S57133[125], 3}, {NULL}};
const Segment SGI194354[] = {{&S57133[90], 6}, {NULL}};
const Segment SGI194355[] = {{&S57133[150], 8}, {NULL}};
const Segment SGI194356[] = {{&S78368[34], 18}, {NULL}};
const Segment SGI194357[] = {{&S71903[34], 7}, {NULL}};
const Segment SGI194358[] = {{&S71903[27], 6}, {NULL}};
const Segment SGI194359[] = {{&S78368[27], 6}, {NULL}};
const Segment SGI194360[] = {{&S71903[98], 9}, {NULL}};
const Segment SGI194361[] = {{&S78368[105], 9}, {NULL}};
const Segment SGI194362[] = {{&S78368[78], 6}, {NULL}};
const Segment SGI194363[] = {{&S71903[71], 6}, {NULL}};
const Segment SGI194364[] = {{&S78368[146], 8}, {NULL}};
const Segment SGI194365[] = {{&S78369[34], 11}, {NULL}};
const Segment SGI194366[] = {{&S71904[44], 11}, {NULL}};
const Segment SGI194367[] = {{&S71904[44], 23}, {NULL}};
const Segment SGI194368[] = {{&S78369[27], 6}, {NULL}};
const Segment SGI194369[] = {{&S71904[30], 6}, {NULL}};
const Segment SGI194370[] = {{&S71904[116], 3}, {NULL}};
const Segment SGI194371[] = {{&S78369[98], 9}, {NULL}};
const Segment SGI194372[] = {{&S71904[81], 6}, {NULL}};
const Segment SGI194373[] = {{&S78369[71], 6}, {NULL}};
const Segment SGI194374[] = {{&S71905[34], 13}, {NULL}};
const Segment SGI194375[] = {{&S71905[27], 6}, {NULL}};
const Segment SGI194376[] = {{&S71905[100], 9}, {NULL}};
const Segment SGI194377[] = {{&S71905[73], 6}, {NULL}};
const Segment SGI194378[] = {{&S67278[34], 13}, {NULL}};
const Segment SGI194379[] = {{&S78376[34], 24}, {NULL}};
const Segment SGI194380[] = {{&S67278[27], 6}, {NULL}};
const Segment SGI194381[] = {{&S78376[27], 6}, {NULL}};
const Segment SGI194382[] = {{&S67278[100], 9}, {NULL}};
const Segment SGI194383[] = {{&S67278[73], 6}, {NULL}};
const Segment SGI194384[] = {{&S78376[111], 9}, {NULL}};
const Segment SGI194385[] = {{&S78376[84], 6}, {NULL}};
const Segment SGI194386[] = {{&S78380[34], 13}, {NULL}};
const Segment SGI194387[] = {{&S78380[27], 6}, {NULL}};
const Segment SGI194388[] = {{&S78380[100], 9}, {NULL}};
const Segment SGI194389[] = {{&S78380[73], 6}, {NULL}};
const Segment SGI194390[] = {{&S78381[34], 12}, {NULL}};
const Segment SGI194391[] = {{&S78381[27], 6}, {NULL}};
const Segment SGI194392[] = {{&S78381[99], 9}, {NULL}};
const Segment SGI194393[] = {{&S78381[72], 6}, {NULL}};
const Segment SGI194394[] = {{&S78388[34], 24}, {NULL}};
const Segment SGI194395[] = {{&S78388[27], 6}, {NULL}};
const Segment SGI194396[] = {{&S78388[111], 9}, {NULL}};
const Segment SGI194397[] = {{&S78388[84], 6}, {NULL}};
const Segment SGI194398[] = {{&S78389[34], 29}, {NULL}};
const Segment SGI194399[] = {{&S78389[27], 6}, {NULL}};
const Segment SGI194400[] = {{&S78389[116], 9}, {NULL}};
const Segment SGI194401[] = {{&S78389[89], 6}, {NULL}};
const Segment SGI194402[] = {{&S57173[34], 10}, {NULL}};
const Segment SGI194403[] = {{&S57173[27], 6}, {NULL}};
const Segment SGI194404[] = {{&S57173[96], 9}, {NULL}};
const Segment SGI194405[] = {{&S57173[69], 6}, {NULL}};
const Segment SGI194406[] = {{&S57174[34], 10}, {NULL}};
const Segment SGI194407[] = {{&S57174[27], 6}, {NULL}};
const Segment SGI194408[] = {{&S57174[96], 9}, {NULL}};
const Segment SGI194409[] = {{&S57174[69], 6}, {NULL}};
const Segment SGI194410[] = {{&S67304[34], 11}, {NULL}};
const Segment SGI194411[] = {{&S67304[27], 6}, {NULL}};
const Segment SGI194412[] = {{&S57175[34], 11}, {NULL}};
const Segment SGI194413[] = {{&S57175[27], 6}, {NULL}};
const Segment SGI194414[] = {{&S67304[98], 9}, {NULL}};
const Segment SGI194415[] = {{&S67304[71], 6}, {NULL}};
const Segment SGI194416[] = {{&S57175[97], 9}, {NULL}};
const Segment SGI194417[] = {{&S57175[70], 6}, {NULL}};
const Segment SGI194418[] = {{&S67305[34], 10}, {NULL}};
const Segment SGI194419[] = {{&S67305[27], 6}, {NULL}};
const Segment SGI194420[] = {{&S57176[34], 11}, {NULL}};
const Segment SGI194421[] = {{&S57176[27], 6}, {NULL}};
const Segment SGI194422[] = {{&S67305[97], 9}, {NULL}};
const Segment SGI194423[] = {{&S67305[70], 6}, {NULL}};
const Segment SGI194424[] = {{&S57176[97], 9}, {NULL}};
const Segment SGI194425[] = {{&S57176[70], 6}, {NULL}};
const Segment SGI194426[] = {{&S67306[34], 11}, {NULL}};
const Segment SGI194427[] = {{&S78399[44], 10}, {NULL}};
const Segment SGI194428[] = {{&S78399[44], 22}, {NULL}};
const Segment SGI194429[] = {{&S67306[27], 6}, {NULL}};
const Segment SGI194430[] = {{&S78399[30], 6}, {NULL}};
const Segment SGI194431[] = {{&S78399[31], 3}, {NULL}};
const Segment SGI194432[] = {{&S67306[98], 9}, {NULL}};
const Segment SGI194433[] = {{&S67306[71], 6}, {NULL}};
const Segment SGI194434[] = {{&S78399[99], 6}, {NULL}};
const Segment SGI194435[] = {{&S78399[68], 8}, {NULL}};
const Segment SGI194436[] = {{&S67307[34], 7}, {NULL}};
const Segment SGI194437[] = {{&S67307[27], 6}, {NULL}};
const Segment SGI194438[] = {{&S67307[100], 9}, {NULL}};
const Segment SGI194439[] = {{&S67307[73], 6}, {NULL}};
const Segment SGI194440[] = {{&S67308[34], 7}, {NULL}};
const Segment SGI194441[] = {{&S67308[27], 6}, {NULL}};
const Segment SGI194442[] = {{&S67308[100], 9}, {NULL}};
const Segment SGI194443[] = {{&S67308[73], 6}, {NULL}};
const Segment SGI194444[] = {{&S78403[44], 8}, {NULL}};
const Segment SGI194445[] = {{&S78403[44], 24}, {NULL}};
const Segment SGI194446[] = {{&S78403[30], 6}, {NULL}};
const Segment SGI194447[] = {{&S78403[82], 6}, {NULL}};
const Segment SGI194448[] = {{&S78403[109], 13}, {NULL}};
const Segment SGI194449[] = {{&S78404[44], 8}, {NULL}};
const Segment SGI194450[] = {{&S78404[44], 24}, {NULL}};
const Segment SGI194451[] = {{&S78404[30], 6}, {NULL}};
const Segment SGI194452[] = {{&S78404[82], 6}, {NULL}};
const Segment SGI194453[] = {{&S78404[109], 13}, {NULL}};
const Segment SGI194454[] = {{&S71950[34], 10}, {NULL}};
const Segment SGI194455[] = {{&S71950[27], 6}, {NULL}};
const Segment SGI194456[] = {{&S71950[97], 9}, {NULL}};
const Segment SGI194457[] = {{&S71950[70], 6}, {NULL}};
const Segment SGI194458[] = {{&S67315[44], 9}, {NULL}};
const Segment SGI194459[] = {{&S71951[34], 7}, {NULL}};
const Segment SGI194460[] = {{&S67315[44], 21}, {NULL}};
const Segment SGI194461[] = {{&S67315[30], 6}, {NULL}};
const Segment SGI194462[] = {{&S71951[27], 6}, {NULL}};
const Segment SGI194463[] = {{&S67315[134], 3}, {NULL}};
const Segment SGI194464[] = {{&S78407[44], 13}, {NULL}};
const Segment SGI194465[] = {{&S71951[94], 9}, {NULL}};
const Segment SGI194466[] = {{&S78407[44], 25}, {NULL}};
const Segment SGI194467[] = {{&S67315[99], 6}, {NULL}};
const Segment SGI194468[] = {{&S71951[67], 6}, {NULL}};
const Segment SGI194469[] = {{&S78407[30], 6}, {NULL}};
const Segment SGI194470[] = {{&S67315[67], 8}, {NULL}};
const Segment SGI194471[] = {{&S78407[31], 3}, {NULL}};
const Segment SGI194472[] = {{&S78407[83], 6}, {NULL}};
const Segment SGI194473[] = {{&S71952[34], 8}, {NULL}};
const Segment SGI194474[] = {{&S71952[27], 6}, {NULL}};
const Segment SGI194475[] = {{&S71952[95], 9}, {NULL}};
const Segment SGI194476[] = {{&S71952[68], 6}, {NULL}};
const Segment SGI194477[] = {{&S71960[44], 13}, {NULL}};
const Segment SGI194478[] = {{&S71960[44], 25}, {NULL}};
const Segment SGI194479[] = {{&S71960[30], 6}, {NULL}};
const Segment SGI194480[] = {{&S71960[118], 3}, {NULL}};
const Segment SGI194481[] = {{&S71960[83], 6}, {NULL}};
const Segment SGI194482[] = {{&S71966[44], 7}, {NULL}};
const Segment SGI194483[] = {{&S71966[44], 19}, {NULL}};
const Segment SGI194484[] = {{&S71966[30], 6}, {NULL}};
const Segment SGI194485[] = {{&S71966[112], 3}, {NULL}};
const Segment SGI194486[] = {{&S71966[77], 6}, {NULL}};
const Segment SGI194487[] = {{&S57211[11], 1}, {NULL}};
const Segment SGI194488[] = {{&S57211[27], 6}, {NULL}};
const Segment SGI194489[] = {{&S57211[88], 9}, {NULL}};
const Segment SGI194490[] = {{&S57211[61], 6}, {NULL}};
const Segment SGI194491[] = {{&S57212[34], 16}, {NULL}};
const Segment SGI194492[] = {{&S57212[27], 6}, {NULL}};
const Segment SGI194493[] = {{&S71969[34], 10}, {NULL}};
const Segment SGI194494[] = {{&S57212[103], 9}, {NULL}};
const Segment SGI194495[] = {{&S57212[76], 6}, {NULL}};
const Segment SGI194496[] = {{&S71969[27], 6}, {NULL}};
const Segment SGI194497[] = {{&S71969[97], 9}, {NULL}};
const Segment SGI194498[] = {{&S71969[70], 6}, {NULL}};
const Segment SGI194499[] = {{&S57214[11], 1}, {NULL}};
const Segment SGI194500[] = {{&S57214[27], 6}, {NULL}};
const Segment SGI194501[] = {{&S57214[88], 9}, {NULL}};
const Segment SGI194502[] = {{&S57214[61], 6}, {NULL}};
const Segment SGI194503[] = {{&S67341[45], 10}, {NULL}};
const Segment SGI194504[] = {{&S57215[34], 15}, {NULL}};
const Segment SGI194505[] = {{&S67341[30], 6}, {NULL}};
const Segment SGI194506[] = {{&S57215[27], 6}, {NULL}};
const Segment SGI194507[] = {{&S67341[115], 3}, {NULL}};
const Segment SGI194508[] = {{&S57215[101], 9}, {NULL}};
const Segment SGI194509[] = {{&S57215[74], 6}, {NULL}};
const Segment SGI194510[] = {{&S67341[81], 5}, {NULL}};
const Segment SGI194511[] = {{&S67341[139], 13}, {NULL}};
const Segment SGI194512[] = {{&S67343[44], 10}, {NULL}};
const Segment SGI194513[] = {{&S67343[44], 22}, {NULL}};
const Segment SGI194514[] = {{&S67343[30], 6}, {NULL}};
const Segment SGI194515[] = {{&S67343[115], 3}, {NULL}};
const Segment SGI194516[] = {{&S67343[80], 6}, {NULL}};
const Segment SGI194517[] = {{&S67343[135], 8}, {NULL}};
const Segment SGI194518[] = {{&S67343[144], 13}, {NULL}};
const Segment SGI194519[] = {{&S67344[45], 10}, {NULL}};
const Segment SGI194520[] = {{&S67344[30], 6}, {NULL}};
const Segment SGI194521[] = {{&S67344[115], 3}, {NULL}};
const Segment SGI194522[] = {{&S67344[81], 5}, {NULL}};
const Segment SGI194523[] = {{&S67344[139], 13}, {NULL}};
const Segment SGI194524[] = {{&S67348[34], 7}, {NULL}};
const Segment SGI194525[] = {{&S67348[27], 6}, {NULL}};
const Segment SGI194526[] = {{&S67348[100], 9}, {NULL}};
const Segment SGI194527[] = {{&S67348[73], 6}, {NULL}};
const Segment SGI194528[] = {{&S72016[34], 25}, {NULL}};
const Segment SGI194529[] = {{&S72016[27], 6}, {NULL}};
const Segment SGI194530[] = {{&S72016[112], 9}, {NULL}};
const Segment SGI194531[] = {{&S72016[85], 6}, {NULL}};
const Segment SGI194532[] = {{&S72017[34], 32}, {NULL}};
const Segment SGI194533[] = {{&S72017[27], 6}, {NULL}};
const Segment SGI194534[] = {{&S72017[119], 9}, {NULL}};
const Segment SGI194535[] = {{&S72017[92], 6}, {NULL}};
const Segment SGI194536[] = {{&S72020[34], 10}, {NULL}};
const Segment SGI194537[] = {{&S72020[27], 6}, {NULL}};
const Segment SGI194538[] = {{&S72020[102], 9}, {NULL}};
const Segment SGI194539[] = {{&S72020[75], 6}, {NULL}};
const Segment SGI194540[] = {{&S67387[34], 9}, {NULL}};
const Segment SGI194541[] = {{&S57267[34], 11}, {NULL}};
const Segment SGI194542[] = {{&S67387[27], 6}, {NULL}};
const Segment SGI194543[] = {{&S57267[27], 6}, {NULL}};
const Segment SGI194544[] = {{&S67387[96], 9}, {NULL}};
const Segment SGI194545[] = {{&S67387[69], 6}, {NULL}};
const Segment SGI194546[] = {{&S57267[97], 9}, {NULL}};
const Segment SGI194547[] = {{&S57267[70], 6}, {NULL}};
const Segment SGI194548[] = {{&S57269[34], 7}, {NULL}};
const Segment SGI194549[] = {{&S57269[27], 6}, {NULL}};
const Segment SGI194550[] = {{&S57269[93], 9}, {NULL}};
const Segment SGI194551[] = {{&S57269[66], 6}, {NULL}};
const Segment SGI194552[] = {{&S57270[34], 24}, {NULL}};
const Segment SGI194553[] = {{&S57270[27], 6}, {NULL}};
const Segment SGI194554[] = {{&S57270[110], 9}, {NULL}};
const Segment SGI194555[] = {{&S57270[83], 6}, {NULL}};
const Segment SGI194556[] = {{&S57271[34], 11}, {NULL}};
const Segment SGI194557[] = {{&S57271[27], 6}, {NULL}};
const Segment SGI194558[] = {{&S57271[97], 9}, {NULL}};
const Segment SGI194559[] = {{&S57271[70], 6}, {NULL}};
const Segment SGI194560[] = {{&S67397[44], 11}, {NULL}};
const Segment SGI194561[] = {{&S67397[44], 23}, {NULL}};
const Segment SGI194562[] = {{&S67397[30], 6}, {NULL}};
const Segment SGI194563[] = {{&S67397[116], 3}, {NULL}};
const Segment SGI194564[] = {{&S67397[81], 6}, {NULL}};
const Segment SGI194565[] = {{&S67398[38], 11}, {NULL}};
const Segment SGI194566[] = {{&S67398[31], 6}, {NULL}};
const Segment SGI194567[] = {{&S72034[44], 18}, {NULL}};
const Segment SGI194568[] = {{&S72034[44], 30}, {NULL}};
const Segment SGI194569[] = {{&S72034[30], 6}, {NULL}};
const Segment SGI194570[] = {{&S72034[123], 3}, {NULL}};
const Segment SGI194571[] = {{&S72034[88], 6}, {NULL}};
const Segment SGI194572[] = {{&S72035[44], 18}, {NULL}};
const Segment SGI194573[] = {{&S72035[44], 30}, {NULL}};
const Segment SGI194574[] = {{&S72035[30], 6}, {NULL}};
const Segment SGI194575[] = {{&S72035[123], 3}, {NULL}};
const Segment SGI194576[] = {{&S72035[88], 6}, {NULL}};
const Segment SGI194577[] = {{&S67401[34], 12}, {NULL}};
const Segment SGI194578[] = {{&S67401[27], 6}, {NULL}};
const Segment SGI194579[] = {{&S67401[109], 9}, {NULL}};
const Segment SGI194580[] = {{&S67401[82], 6}, {NULL}};
const Segment SGI194581[] = {{&S67401[150], 8}, {NULL}};
const Segment SGI194582[] = {{&S72036[44], 18}, {NULL}};
const Segment SGI194583[] = {{&S72036[44], 30}, {NULL}};
const Segment SGI194584[] = {{&S72036[30], 6}, {NULL}};
const Segment SGI194585[] = {{&S72036[123], 3}, {NULL}};
const Segment SGI194586[] = {{&S72036[88], 6}, {NULL}};
const Segment SGI194587[] = {{&S72037[44], 18}, {NULL}};
const Segment SGI194588[] = {{&S72037[44], 30}, {NULL}};
const Segment SGI194589[] = {{&S72037[30], 6}, {NULL}};
const Segment SGI194590[] = {{&S72037[123], 3}, {NULL}};
const Segment SGI194591[] = {{&S72037[88], 6}, {NULL}};
const Segment SGI194592[] = {{&S72042[34], 15}, {NULL}};
const Segment SGI194593[] = {{&S72042[27], 6}, {NULL}};
const Segment SGI194594[] = {{&S72042[102], 9}, {NULL}};
const Segment SGI194595[] = {{&S72042[75], 6}, {NULL}};
const Segment SGI194596[] = {{&S57295[34], 7}, {NULL}};
const Segment SGI194597[] = {{&S57295[27], 6}, {NULL}};
const Segment SGI194598[] = {{&S57295[100], 9}, {NULL}};
const Segment SGI194599[] = {{&S57295[73], 6}, {NULL}};
const Segment SGI194600[] = {{&S72073[44], 8}, {NULL}};
const Segment SGI194601[] = {{&S72073[44], 20}, {NULL}};
const Segment SGI194602[] = {{&S72073[30], 6}, {NULL}};
const Segment SGI194603[] = {{&S72073[113], 3}, {NULL}};
const Segment SGI194604[] = {{&S72073[78], 6}, {NULL}};
const Segment SGI194605[] = {{&S72074[38], 8}, {NULL}};
const Segment SGI194606[] = {{&S72074[31], 6}, {NULL}};
const Segment SGI194607[] = {{&S66414[34], 20}, {NULL}};
const Segment SGI194608[] = {{&S66414[27], 6}, {NULL}};
const Segment SGI194609[] = {{&S66414[107], 9}, {NULL}};
const Segment SGI194610[] = {{&S66414[80], 6}, {NULL}};
const Segment SGI194611[] = {{&S57333[44], 9}, {NULL}};
const Segment SGI194612[] = {{&S57333[44], 20}, {NULL}};
const Segment SGI194613[] = {{&S57333[30], 6}, {NULL}};
const Segment SGI194614[] = {{&S57333[113], 3}, {NULL}};
const Segment SGI194615[] = {{&S57333[78], 6}, {NULL}};
const Segment SGI194616[] = {{&S57335[44], 9}, {NULL}};
const Segment SGI194617[] = {{&S57335[44], 21}, {NULL}};
const Segment SGI194618[] = {{&S57335[30], 6}, {NULL}};
const Segment SGI194619[] = {{&S57335[114], 3}, {NULL}};
const Segment SGI194620[] = {{&S57335[79], 6}, {NULL}};
const Segment SGI194621[] = {{&S57336[38], 9}, {NULL}};
const Segment SGI194622[] = {{&S57336[31], 6}, {NULL}};
const Segment SGI194623[] = {{&S66426[34], 12}, {NULL}};
const Segment SGI194624[] = {{&S66426[27], 6}, {NULL}};
const Segment SGI194625[] = {{&S66426[99], 9}, {NULL}};
const Segment SGI194626[] = {{&S66426[72], 6}, {NULL}};
const Segment SGI194627[] = {{&S66428[34], 13}, {NULL}};
const Segment SGI194628[] = {{&S66428[27], 6}, {NULL}};
const Segment SGI194629[] = {{&S66428[100], 9}, {NULL}};
const Segment SGI194630[] = {{&S66428[73], 6}, {NULL}};
const Segment SGI194631[] = {{&S78458[44], 14}, {NULL}};
const Segment SGI194632[] = {{&S78458[44], 26}, {NULL}};
const Segment SGI194633[] = {{&S78458[30], 6}, {NULL}};
const Segment SGI194634[] = {{&S78458[31], 3}, {NULL}};
const Segment SGI194635[] = {{&S78458[84], 6}, {NULL}};
const Segment SGI194636[] = {{&S72097[34], 10}, {NULL}};
const Segment SGI194637[] = {{&S72097[27], 6}, {NULL}};
const Segment SGI194638[] = {{&S72097[97], 9}, {NULL}};
const Segment SGI194639[] = {{&S72097[70], 6}, {NULL}};
const Segment SGI194640[] = {{&S66431[34], 6}, {NULL}};
const Segment SGI194641[] = {{&S66431[27], 6}, {NULL}};
const Segment SGI194642[] = {{&S66431[93], 9}, {NULL}};
const Segment SGI194643[] = {{&S66431[66], 6}, {NULL}};
const Segment SGI194644[] = {{&S66432[34], 6}, {NULL}};
const Segment SGI194645[] = {{&S66432[27], 6}, {NULL}};
const Segment SGI194646[] = {{&S78462[34], 10}, {NULL}};
const Segment SGI194647[] = {{&S78462[27], 6}, {NULL}};
const Segment SGI194648[] = {{&S66432[93], 9}, {NULL}};
const Segment SGI194649[] = {{&S66432[66], 6}, {NULL}};
const Segment SGI194650[] = {{&S78462[97], 9}, {NULL}};
const Segment SGI194651[] = {{&S78462[70], 6}, {NULL}};
const Segment SGI194652[] = {{&S78463[44], 4}, {NULL}};
const Segment SGI194653[] = {{&S78463[44], 16}, {NULL}};
const Segment SGI194654[] = {{&S66434[34], 13}, {NULL}};
const Segment SGI194655[] = {{&S78463[30], 6}, {NULL}};
const Segment SGI194656[] = {{&S66434[27], 6}, {NULL}};
const Segment SGI194657[] = {{&S72102[34], 25}, {NULL}};
const Segment SGI194658[] = {{&S78463[31], 3}, {NULL}};
const Segment SGI194659[] = {{&S72102[27], 6}, {NULL}};
const Segment SGI194660[] = {{&S78463[74], 6}, {NULL}};
const Segment SGI194661[] = {{&S66434[100], 9}, {NULL}};
const Segment SGI194662[] = {{&S66434[73], 6}, {NULL}};
const Segment SGI194663[] = {{&S72102[112], 9}, {NULL}};
const Segment SGI194664[] = {{&S72102[85], 6}, {NULL}};
const Segment SGI194665[] = {{&S78464[44], 4}, {NULL}};
const Segment SGI194666[] = {{&S78464[44], 16}, {NULL}};
const Segment SGI194667[] = {{&S78464[30], 6}, {NULL}};
const Segment SGI194668[] = {{&S78464[31], 3}, {NULL}};
const Segment SGI194669[] = {{&S66436[34], 6}, {NULL}};
const Segment SGI194670[] = {{&S78464[74], 6}, {NULL}};
const Segment SGI194671[] = {{&S66436[27], 6}, {NULL}};
const Segment SGI194672[] = {{&S66436[93], 9}, {NULL}};
const Segment SGI194673[] = {{&S66436[66], 6}, {NULL}};
const Segment SGI194674[] = {{&S66437[34], 7}, {NULL}};
const Segment SGI194675[] = {{&S66437[27], 6}, {NULL}};
const Segment SGI194676[] = {{&S66437[94], 9}, {NULL}};
const Segment SGI194677[] = {{&S66437[67], 6}, {NULL}};
const Segment SGI194678[] = {{&S72107[44], 14}, {NULL}};
const Segment SGI194679[] = {{&S72107[44], 26}, {NULL}};
const Segment SGI194680[] = {{&S66438[34], 6}, {NULL}};
const Segment SGI194681[] = {{&S72107[30], 6}, {NULL}};
const Segment SGI194682[] = {{&S66438[27], 6}, {NULL}};
const Segment SGI194683[] = {{&S72107[119], 3}, {NULL}};
const Segment SGI194684[] = {{&S72107[84], 6}, {NULL}};
const Segment SGI194685[] = {{&S66438[93], 9}, {NULL}};
const Segment SGI194686[] = {{&S66438[66], 6}, {NULL}};
const Segment SGI194687[] = {{&S66439[34], 6}, {NULL}};
const Segment SGI194688[] = {{&S66439[27], 6}, {NULL}};
const Segment SGI194689[] = {{&S78469[34], 24}, {NULL}};
const Segment SGI194690[] = {{&S67487[34], 6}, {NULL}};
const Segment SGI194691[] = {{&S78469[27], 6}, {NULL}};
const Segment SGI194692[] = {{&S66439[93], 9}, {NULL}};
const Segment SGI194693[] = {{&S66439[66], 6}, {NULL}};
const Segment SGI194694[] = {{&S67487[27], 6}, {NULL}};
const Segment SGI194695[] = {{&S78469[113], 9}, {NULL}};
const Segment SGI194696[] = {{&S78469[86], 6}, {NULL}};
const Segment SGI194697[] = {{&S67487[93], 9}, {NULL}};
const Segment SGI194698[] = {{&S67487[66], 6}, {NULL}};
const Segment SGI194699[] = {{&S66440[34], 14}, {NULL}};
const Segment SGI194700[] = {{&S66440[27], 6}, {NULL}};
const Segment SGI194701[] = {{&S66440[101], 9}, {NULL}};
const Segment SGI194702[] = {{&S66440[74], 6}, {NULL}};
const Segment SGI194703[] = {{&S78470[59], 4}, {NULL}};
const Segment SGI194704[] = {{&S78470[59], 26}, {NULL}};
const Segment SGI194705[] = {{&S78470[45], 6}, {NULL}};
const Segment SGI194706[] = {{&S78470[46], 3}, {NULL}};
const Segment SGI194707[] = {{&S78470[99], 6}, {NULL}};
const Segment SGI194708[] = {{&S78470[0], 14}, {NULL}};
const Segment SGI194709[] = {{&S67489[44], 7}, {NULL}};
const Segment SGI194710[] = {{&S66441[34], 5}, {NULL}};
const Segment SGI194711[] = {{&S67489[44], 19}, {NULL}};
const Segment SGI194712[] = {{&S67489[30], 6}, {NULL}};
const Segment SGI194713[] = {{&S66441[27], 6}, {NULL}};
const Segment SGI194714[] = {{&S67489[128], 3}, {NULL}};
const Segment SGI194715[] = {{&S66441[92], 9}, {NULL}};
const Segment SGI194716[] = {{&S67489[93], 6}, {NULL}};
const Segment SGI194717[] = {{&S66441[65], 6}, {NULL}};
const Segment SGI194718[] = {{&S67489[65], 8}, {NULL}};
const Segment SGI194719[] = {{&S78471[59], 12}, {NULL}};
const Segment SGI194720[] = {{&S78471[59], 42}, {NULL}};
const Segment SGI194721[] = {{&S78471[45], 6}, {NULL}};
const Segment SGI194722[] = {{&S78471[46], 3}, {NULL}};
const Segment SGI194723[] = {{&S78471[115], 6}, {NULL}};
const Segment SGI194724[] = {{&S66442[34], 6}, {NULL}};
const Segment SGI194725[] = {{&S66442[27], 6}, {NULL}};
const Segment SGI194726[] = {{&S67490[44], 11}, {NULL}};
const Segment SGI194727[] = {{&S78471[0], 14}, {NULL}};
const Segment SGI194728[] = {{&S67490[44], 23}, {NULL}};
const Segment SGI194729[] = {{&S67490[30], 6}, {NULL}};
const Segment SGI194730[] = {{&S66442[93], 9}, {NULL}};
const Segment SGI194731[] = {{&S66442[66], 6}, {NULL}};
const Segment SGI194732[] = {{&S67490[132], 3}, {NULL}};
const Segment SGI194733[] = {{&S67490[97], 6}, {NULL}};
const Segment SGI194734[] = {{&S67490[69], 8}, {NULL}};
const Segment SGI194735[] = {{&S66443[34], 15}, {NULL}};
const Segment SGI194736[] = {{&S66443[27], 6}, {NULL}};
const Segment SGI194737[] = {{&S66443[102], 9}, {NULL}};
const Segment SGI194738[] = {{&S66443[75], 6}, {NULL}};
const Segment SGI194739[] = {{&S78474[34], 9}, {NULL}};
const Segment SGI194740[] = {{&S78474[27], 6}, {NULL}};
const Segment SGI194741[] = {{&S78474[103], 9}, {NULL}};
const Segment SGI194742[] = {{&S78474[76], 6}, {NULL}};
const Segment SGI194743[] = {{&S67497[34], 18}, {NULL}};
const Segment SGI194744[] = {{&S67497[27], 6}, {NULL}};
const Segment SGI194745[] = {{&S67497[105], 9}, {NULL}};
const Segment SGI194746[] = {{&S67497[78], 6}, {NULL}};
const Segment SGI194747[] = {{&S72125[34], 10}, {NULL}};
const Segment SGI194748[] = {{&S72125[27], 6}, {NULL}};
const Segment SGI194749[] = {{&S67499[34], 15}, {NULL}};
const Segment SGI194750[] = {{&S67499[27], 6}, {NULL}};
const Segment SGI194751[] = {{&S72125[97], 9}, {NULL}};
const Segment SGI194752[] = {{&S72125[70], 6}, {NULL}};
const Segment SGI194753[] = {{&S67499[102], 9}, {NULL}};
const Segment SGI194754[] = {{&S67499[75], 6}, {NULL}};
const Segment SGI194755[] = {{&S66458[34], 10}, {NULL}};
const Segment SGI194756[] = {{&S66458[27], 6}, {NULL}};
const Segment SGI194757[] = {{&S66458[97], 9}, {NULL}};
const Segment SGI194758[] = {{&S72131[44], 21}, {NULL}};
const Segment SGI194759[] = {{&S66458[70], 6}, {NULL}};
const Segment SGI194760[] = {{&S72131[44], 33}, {NULL}};
const Segment SGI194761[] = {{&S72131[30], 6}, {NULL}};
const Segment SGI194762[] = {{&S72131[126], 3}, {NULL}};
const Segment SGI194763[] = {{&S72131[91], 6}, {NULL}};
const Segment SGI194764[] = {{&S66460[34], 10}, {NULL}};
const Segment SGI194765[] = {{&S66460[27], 6}, {NULL}};
const Segment SGI194766[] = {{&S66460[97], 9}, {NULL}};
const Segment SGI194767[] = {{&S66460[70], 6}, {NULL}};
const Segment SGI194768[] = {{&S72133[44], 8}, {NULL}};
const Segment SGI194769[] = {{&S72133[44], 20}, {NULL}};
const Segment SGI194770[] = {{&S72133[30], 6}, {NULL}};
const Segment SGI194771[] = {{&S72133[113], 3}, {NULL}};
const Segment SGI194772[] = {{&S72133[78], 6}, {NULL}};
const Segment SGI194773[] = {{&S67511[44], 5}, {NULL}};
const Segment SGI194774[] = {{&S67511[44], 17}, {NULL}};
const Segment SGI194775[] = {{&S67511[30], 6}, {NULL}};
const Segment SGI194776[] = {{&S72134[44], 9}, {NULL}};
const Segment SGI194777[] = {{&S72134[44], 21}, {NULL}};
const Segment SGI194778[] = {{&S72134[30], 6}, {NULL}};
const Segment SGI194779[] = {{&S67511[110], 3}, {NULL}};
const Segment SGI194780[] = {{&S67511[75], 6}, {NULL}};
const Segment SGI194781[] = {{&S72134[114], 3}, {NULL}};
const Segment SGI194782[] = {{&S72134[79], 6}, {NULL}};
const Segment SGI194783[] = {{&S78499[44], 13}, {NULL}};
const Segment SGI194784[] = {{&S78499[44], 25}, {NULL}};
const Segment SGI194785[] = {{&S78499[30], 6}, {NULL}};
const Segment SGI194786[] = {{&S78499[31], 3}, {NULL}};
const Segment SGI194787[] = {{&S78499[83], 6}, {NULL}};
const Segment SGI194788[] = {{&S78503[44], 15}, {NULL}};
const Segment SGI194789[] = {{&S78503[44], 27}, {NULL}};
const Segment SGI194790[] = {{&S78503[30], 6}, {NULL}};
const Segment SGI194791[] = {{&S78503[31], 3}, {NULL}};
const Segment SGI194792[] = {{&S78503[85], 6}, {NULL}};
const Segment SGI194793[] = {{&S78504[34], 9}, {NULL}};
const Segment SGI194794[] = {{&S78504[27], 6}, {NULL}};
const Segment SGI194795[] = {{&S78504[96], 9}, {NULL}};
const Segment SGI194796[] = {{&S78504[69], 6}, {NULL}};
const Segment SGI194797[] = {{&S78505[34], 10}, {NULL}};
const Segment SGI194798[] = {{&S78505[27], 6}, {NULL}};
const Segment SGI194799[] = {{&S78505[97], 9}, {NULL}};
const Segment SGI194800[] = {{&S78505[70], 6}, {NULL}};
const Segment SGI194801[] = {{&S67541[34], 9}, {NULL}};
const Segment SGI194802[] = {{&S67541[27], 6}, {NULL}};
const Segment SGI194803[] = {{&S67541[96], 9}, {NULL}};
const Segment SGI194804[] = {{&S67541[69], 6}, {NULL}};
const Segment SGI194805[] = {{&S67542[34], 10}, {NULL}};
const Segment SGI194806[] = {{&S67542[27], 6}, {NULL}};
const Segment SGI194807[] = {{&S67542[97], 9}, {NULL}};
const Segment SGI194808[] = {{&S67542[70], 6}, {NULL}};
const Segment SGI194809[] = {{&S67543[34], 10}, {NULL}};
const Segment SGI194810[] = {{&S67543[27], 6}, {NULL}};
const Segment SGI194811[] = {{&S67543[97], 9}, {NULL}};
const Segment SGI194812[] = {{&S67543[70], 6}, {NULL}};
const Segment SGI194813[] = {{&S67544[34], 9}, {NULL}};
const Segment SGI194814[] = {{&S67544[27], 6}, {NULL}};
const Segment SGI194815[] = {{&S67544[96], 9}, {NULL}};
const Segment SGI194816[] = {{&S67544[69], 6}, {NULL}};
const Segment SGI194817[] = {{&S67545[34], 10}, {NULL}};
const Segment SGI194818[] = {{&S67545[27], 6}, {NULL}};
const Segment SGI194819[] = {{&S67545[97], 9}, {NULL}};
const Segment SGI194820[] = {{&S67545[70], 6}, {NULL}};
const Segment SGI194821[] = {{&S67549[44], 16}, {NULL}};
const Segment SGI194822[] = {{&S67549[44], 28}, {NULL}};
const Segment SGI194823[] = {{&S67549[30], 6}, {NULL}};
const Segment SGI194824[] = {{&S67549[121], 3}, {NULL}};
const Segment SGI194825[] = {{&S67549[86], 6}, {NULL}};
const Segment SGI194826[] = {{&S67550[38], 16}, {NULL}};
const Segment SGI194827[] = {{&S67550[31], 6}, {NULL}};
const Segment SGI194828[] = {{&S72181[34], 10}, {NULL}};
const Segment SGI194829[] = {{&S72181[27], 6}, {NULL}};
const Segment SGI194830[] = {{&S72181[97], 9}, {NULL}};
const Segment SGI194831[] = {{&S72181[70], 6}, {NULL}};
const Segment SGI194832[] = {{&S78541[44], 9}, {NULL}};
const Segment SGI194833[] = {{&S78541[44], 21}, {NULL}};
const Segment SGI194834[] = {{&S78541[30], 6}, {NULL}};
const Segment SGI194835[] = {{&S78541[31], 3}, {NULL}};
const Segment SGI194836[] = {{&S78541[79], 6}, {NULL}};
const Segment SGI194837[] = {{&S78542[38], 9}, {NULL}};
const Segment SGI194838[] = {{&S78542[31], 6}, {NULL}};
const Segment SGI194839[] = {{&S72187[44], 6}, {NULL}};
const Segment SGI194840[] = {{&S72187[44], 18}, {NULL}};
const Segment SGI194841[] = {{&S72187[30], 6}, {NULL}};
const Segment SGI194842[] = {{&S72187[111], 3}, {NULL}};
const Segment SGI194843[] = {{&S72187[76], 6}, {NULL}};
const Segment SGI194844[] = {{&S78543[44], 9}, {NULL}};
const Segment SGI194845[] = {{&S78543[44], 20}, {NULL}};
const Segment SGI194846[] = {{&S78543[30], 6}, {NULL}};
const Segment SGI194847[] = {{&S72188[38], 6}, {NULL}};
const Segment SGI194848[] = {{&S72188[31], 6}, {NULL}};
const Segment SGI194849[] = {{&S78543[31], 3}, {NULL}};
const Segment SGI194850[] = {{&S78543[78], 6}, {NULL}};
const Segment SGI194851[] = {{&S72189[44], 10}, {NULL}};
const Segment SGI194852[] = {{&S72189[44], 22}, {NULL}};
const Segment SGI194853[] = {{&S78544[44], 9}, {NULL}};
const Segment SGI194854[] = {{&S72189[30], 6}, {NULL}};
const Segment SGI194855[] = {{&S78544[44], 21}, {NULL}};
const Segment SGI194856[] = {{&S78544[30], 6}, {NULL}};
const Segment SGI194857[] = {{&S72189[115], 3}, {NULL}};
const Segment SGI194858[] = {{&S78544[31], 3}, {NULL}};
const Segment SGI194859[] = {{&S72189[80], 6}, {NULL}};
const Segment SGI194860[] = {{&S78544[79], 6}, {NULL}};
const Segment SGI194861[] = {{&S78546[44], 7}, {NULL}};
const Segment SGI194862[] = {{&S78546[44], 19}, {NULL}};
const Segment SGI194863[] = {{&S67571[44], 10}, {NULL}};
const Segment SGI194864[] = {{&S78546[30], 6}, {NULL}};
const Segment SGI194865[] = {{&S67571[44], 22}, {NULL}};
const Segment SGI194866[] = {{&S67571[30], 6}, {NULL}};
const Segment SGI194867[] = {{&S78546[31], 3}, {NULL}};
const Segment SGI194868[] = {{&S67571[131], 3}, {NULL}};
const Segment SGI194869[] = {{&S78546[77], 6}, {NULL}};
const Segment SGI194870[] = {{&S67571[96], 6}, {NULL}};
const Segment SGI194871[] = {{&S67571[68], 8}, {NULL}};
const Segment SGI194872[] = {{&S72193[34], 18}, {NULL}};
const Segment SGI194873[] = {{&S66533[34], 10}, {NULL}};
const Segment SGI194874[] = {{&S72193[27], 6}, {NULL}};
const Segment SGI194875[] = {{&S66533[27], 6}, {NULL}};
const Segment SGI194876[] = {{&S78547[44], 4}, {NULL}};
const Segment SGI194877[] = {{&S78547[44], 16}, {NULL}};
const Segment SGI194878[] = {{&S78547[30], 6}, {NULL}};
const Segment SGI194879[] = {{&S66533[97], 9}, {NULL}};
const Segment SGI194880[] = {{&S72193[114], 9}, {NULL}};
const Segment SGI194881[] = {{&S66533[70], 6}, {NULL}};
const Segment SGI194882[] = {{&S78547[31], 3}, {NULL}};
const Segment SGI194883[] = {{&S72193[87], 6}, {NULL}};
const Segment SGI194884[] = {{&S78547[74], 6}, {NULL}};
const Segment SGI194885[] = {{&S67573[34], 6}, {NULL}};
const Segment SGI194886[] = {{&S67573[27], 6}, {NULL}};
const Segment SGI194887[] = {{&S67573[93], 9}, {NULL}};
const Segment SGI194888[] = {{&S67573[66], 6}, {NULL}};
const Segment SGI194889[] = {{&S72194[34], 18}, {NULL}};
const Segment SGI194890[] = {{&S78548[34], 10}, {NULL}};
const Segment SGI194891[] = {{&S72194[27], 6}, {NULL}};
const Segment SGI194892[] = {{&S78548[27], 6}, {NULL}};
const Segment SGI194893[] = {{&S72194[114], 9}, {NULL}};
const Segment SGI194894[] = {{&S78548[97], 9}, {NULL}};
const Segment SGI194895[] = {{&S78548[70], 6}, {NULL}};
const Segment SGI194896[] = {{&S72194[87], 6}, {NULL}};
const Segment SGI194897[] = {{&S72195[34], 25}, {NULL}};
const Segment SGI194898[] = {{&S72195[27], 6}, {NULL}};
const Segment SGI194899[] = {{&S72195[112], 9}, {NULL}};
const Segment SGI194900[] = {{&S72195[85], 6}, {NULL}};
const Segment SGI194901[] = {{&S72195[146], 8}, {NULL}};
const Segment SGI194902[] = {{&S78552[44], 11}, {NULL}};
const Segment SGI194903[] = {{&S78552[44], 23}, {NULL}};
const Segment SGI194904[] = {{&S78552[30], 6}, {NULL}};
const Segment SGI194905[] = {{&S78552[31], 3}, {NULL}};
const Segment SGI194906[] = {{&S78552[81], 6}, {NULL}};
const Segment SGI194907[] = {{&S72200[34], 25}, {NULL}};
const Segment SGI194908[] = {{&S72200[27], 6}, {NULL}};
const Segment SGI194909[] = {{&S72200[112], 9}, {NULL}};
const Segment SGI194910[] = {{&S72200[85], 6}, {NULL}};
const Segment SGI194911[] = {{&S78557[34], 18}, {NULL}};
const Segment SGI194912[] = {{&S78557[27], 6}, {NULL}};
const Segment SGI194913[] = {{&S67585[34], 10}, {NULL}};
const Segment SGI194914[] = {{&S78557[105], 9}, {NULL}};
const Segment SGI194915[] = {{&S67585[27], 6}, {NULL}};
const Segment SGI194916[] = {{&S78557[78], 6}, {NULL}};
const Segment SGI194917[] = {{&S78557[146], 8}, {NULL}};
const Segment SGI194918[] = {{&S67585[97], 9}, {NULL}};
const Segment SGI194919[] = {{&S67585[70], 6}, {NULL}};
const Segment SGI194920[] = {{&S72203[34], 15}, {NULL}};
const Segment SGI194921[] = {{&S72203[27], 6}, {NULL}};
const Segment SGI194922[] = {{&S67586[34], 10}, {NULL}};
const Segment SGI194923[] = {{&S72203[102], 9}, {NULL}};
const Segment SGI194924[] = {{&S72203[75], 6}, {NULL}};
const Segment SGI194925[] = {{&S67586[27], 6}, {NULL}};
const Segment SGI194926[] = {{&S67586[97], 9}, {NULL}};
const Segment SGI194927[] = {{&S78559[34], 16}, {NULL}};
const Segment SGI194928[] = {{&S67586[70], 6}, {NULL}};
const Segment SGI194929[] = {{&S78559[27], 6}, {NULL}};
const Segment SGI194930[] = {{&S78559[110], 9}, {NULL}};
const Segment SGI194931[] = {{&S78559[83], 6}, {NULL}};
const Segment SGI194932[] = {{&S72204[34], 12}, {NULL}};
const Segment SGI194933[] = {{&S72204[27], 6}, {NULL}};
const Segment SGI194934[] = {{&S72204[99], 9}, {NULL}};
const Segment SGI194935[] = {{&S72204[72], 6}, {NULL}};
const Segment SGI194936[] = {{&S78560[34], 15}, {NULL}};
const Segment SGI194937[] = {{&S78560[27], 6}, {NULL}};
const Segment SGI194938[] = {{&S78560[108], 9}, {NULL}};
const Segment SGI194939[] = {{&S72205[34], 12}, {NULL}};
const Segment SGI194940[] = {{&S78560[81], 6}, {NULL}};
const Segment SGI194941[] = {{&S72205[27], 6}, {NULL}};
const Segment SGI194942[] = {{&S72205[99], 9}, {NULL}};
const Segment SGI194943[] = {{&S72205[72], 6}, {NULL}};
const Segment SGI194944[] = {{&S78579[34], 10}, {NULL}};
const Segment SGI194945[] = {{&S78579[27], 6}, {NULL}};
const Segment SGI194946[] = {{&S78579[97], 9}, {NULL}};
const Segment SGI194947[] = {{&S78579[70], 6}, {NULL}};
const Segment SGI194948[] = {{&S67607[44], 11}, {NULL}};
const Segment SGI194949[] = {{&S67607[44], 23}, {NULL}};
const Segment SGI194950[] = {{&S67607[30], 6}, {NULL}};
const Segment SGI194951[] = {{&S67607[116], 3}, {NULL}};
const Segment SGI194952[] = {{&S67607[81], 6}, {NULL}};
const Segment SGI194953[] = {{&S67613[44], 20}, {NULL}};
const Segment SGI194954[] = {{&S67613[44], 32}, {NULL}};
const Segment SGI194955[] = {{&S67613[30], 6}, {NULL}};
const Segment SGI194956[] = {{&S67613[125], 3}, {NULL}};
const Segment SGI194957[] = {{&S67613[90], 6}, {NULL}};
const Segment SGI194958[] = {{&S67614[38], 20}, {NULL}};
const Segment SGI194959[] = {{&S67614[31], 6}, {NULL}};
const Segment SGI194960[] = {{&S78591[34], 16}, {NULL}};
const Segment SGI194961[] = {{&S78591[27], 6}, {NULL}};
const Segment SGI194962[] = {{&S78591[113], 9}, {NULL}};
const Segment SGI194963[] = {{&S78591[86], 6}, {NULL}};
const Segment SGI194964[] = {{&S72237[44], 10}, {NULL}};
const Segment SGI194965[] = {{&S72237[44], 22}, {NULL}};
const Segment SGI194966[] = {{&S72237[30], 6}, {NULL}};
const Segment SGI194967[] = {{&S72237[115], 3}, {NULL}};
const Segment SGI194968[] = {{&S72237[80], 6}, {NULL}};
const Segment SGI194969[] = {{&S72237[147], 15}, {NULL}};
const Segment SGI194970[] = {{&S67622[34], 6}, {NULL}};
const Segment SGI194971[] = {{&S67622[27], 6}, {NULL}};
const Segment SGI194972[] = {{&S67622[93], 9}, {NULL}};
const Segment SGI194973[] = {{&S67622[66], 6}, {NULL}};
const Segment SGI194974[] = {{&S66592[44], 9}, {NULL}};
const Segment SGI194975[] = {{&S66592[44], 21}, {NULL}};
const Segment SGI194976[] = {{&S66592[30], 6}, {NULL}};
const Segment SGI194977[] = {{&S66592[114], 3}, {NULL}};
const Segment SGI194978[] = {{&S66592[79], 6}, {NULL}};
const Segment SGI194979[] = {{&S66594[44], 5}, {NULL}};
const Segment SGI194980[] = {{&S66594[44], 17}, {NULL}};
const Segment SGI194981[] = {{&S66594[30], 6}, {NULL}};
const Segment SGI194982[] = {{&S66594[110], 3}, {NULL}};
const Segment SGI194983[] = {{&S66594[75], 6}, {NULL}};
const Segment SGI194984[] = {{&S78616[34], 7}, {NULL}};
const Segment SGI194985[] = {{&S78616[27], 6}, {NULL}};
const Segment SGI194986[] = {{&S78616[94], 9}, {NULL}};
const Segment SGI194987[] = {{&S78616[67], 6}, {NULL}};
const Segment SGI194988[] = {{&S67657[34], 10}, {NULL}};
const Segment SGI194989[] = {{&S67657[27], 6}, {NULL}};
const Segment SGI194990[] = {{&S67657[97], 9}, {NULL}};
const Segment SGI194991[] = {{&S67657[70], 6}, {NULL}};
const Segment SGI194992[] = {{&S67658[34], 11}, {NULL}};
const Segment SGI194993[] = {{&S67658[27], 6}, {NULL}};
const Segment SGI194994[] = {{&S67658[98], 9}, {NULL}};
const Segment SGI194995[] = {{&S67658[71], 6}, {NULL}};
const Segment SGI194996[] = {{&S72278[44], 10}, {NULL}};
const Segment SGI194997[] = {{&S72278[44], 22}, {NULL}};
const Segment SGI194998[] = {{&S72278[30], 6}, {NULL}};
const Segment SGI194999[] = {{&S72278[131], 3}, {NULL}};
const Segment SGI195000[] = {{&S72278[96], 6}, {NULL}};
const Segment SGI195001[] = {{&S72278[68], 8}, {NULL}};
const Segment SGI195002[] = {{&S67659[34], 11}, {NULL}};
const Segment SGI195003[] = {{&S78639[44], 8}, {NULL}};
const Segment SGI195004[] = {{&S67659[27], 6}, {NULL}};
const Segment SGI195005[] = {{&S78639[44], 20}, {NULL}};
const Segment SGI195006[] = {{&S78639[30], 6}, {NULL}};
const Segment SGI195007[] = {{&S78639[31], 3}, {NULL}};
const Segment SGI195008[] = {{&S67659[98], 9}, {NULL}};
const Segment SGI195009[] = {{&S67659[71], 6}, {NULL}};
const Segment SGI195010[] = {{&S78639[78], 6}, {NULL}};
const Segment SGI195011[] = {{&S67660[34], 7}, {NULL}};
const Segment SGI195012[] = {{&S67660[27], 6}, {NULL}};
const Segment SGI195013[] = {{&S67660[98], 9}, {NULL}};
const Segment SGI195014[] = {{&S67660[71], 6}, {NULL}};
const Segment SGI195015[] = {{&S78641[34], 10}, {NULL}};
const Segment SGI195016[] = {{&S78641[27], 6}, {NULL}};
const Segment SGI195017[] = {{&S78641[97], 9}, {NULL}};
const Segment SGI195018[] = {{&S78641[70], 6}, {NULL}};
const Segment SGI195019[] = {{&S67661[34], 11}, {NULL}};
const Segment SGI195020[] = {{&S67661[27], 6}, {NULL}};
const Segment SGI195021[] = {{&S67661[98], 9}, {NULL}};
const Segment SGI195022[] = {{&S67661[71], 6}, {NULL}};
const Segment SGI195023[] = {{&S78643[34], 5}, {NULL}};
const Segment SGI195024[] = {{&S78643[27], 6}, {NULL}};
const Segment SGI195025[] = {{&S78643[92], 9}, {NULL}};
const Segment SGI195026[] = {{&S78643[65], 6}, {NULL}};
const Segment SGI195027[] = {{&S78644[34], 8}, {NULL}};
const Segment SGI195028[] = {{&S78644[27], 6}, {NULL}};
const Segment SGI195029[] = {{&S78644[95], 9}, {NULL}};
const Segment SGI195030[] = {{&S78644[68], 6}, {NULL}};
const Segment SGI195031[] = {{&S72289[44], 10}, {NULL}};
const Segment SGI195032[] = {{&S72289[44], 22}, {NULL}};
const Segment SGI195033[] = {{&S72289[30], 6}, {NULL}};
const Segment SGI195034[] = {{&S72289[115], 3}, {NULL}};
const Segment SGI195035[] = {{&S72289[80], 6}, {NULL}};
const Segment SGI195036[] = {{&S66645[34], 10}, {NULL}};
const Segment SGI195037[] = {{&S66645[27], 6}, {NULL}};
const Segment SGI195038[] = {{&S66645[97], 9}, {NULL}};
const Segment SGI195039[] = {{&S66645[70], 6}, {NULL}};
const Segment SGI195040[] = {{&S78655[34], 7}, {NULL}};
const Segment SGI195041[] = {{&S78655[27], 6}, {NULL}};
const Segment SGI195042[] = {{&S78655[99], 9}, {NULL}};
const Segment SGI195043[] = {{&S78655[72], 6}, {NULL}};
const Segment SGI195044[] = {{&S67680[44], 12}, {NULL}};
const Segment SGI195045[] = {{&S67680[44], 24}, {NULL}};
const Segment SGI195046[] = {{&S67680[30], 6}, {NULL}};
const Segment SGI195047[] = {{&S67680[144], 3}, {NULL}};
const Segment SGI195048[] = {{&S67680[109], 6}, {NULL}};
const Segment SGI195049[] = {{&S67680[70], 8}, {NULL}};
const Segment SGI195050[] = {{&S78673[34], 12}, {NULL}};
const Segment SGI195051[] = {{&S78673[27], 6}, {NULL}};
const Segment SGI195052[] = {{&S78673[103], 9}, {NULL}};
const Segment SGI195053[] = {{&S78673[76], 6}, {NULL}};
const Segment SGI195054[] = {{&S78674[34], 12}, {NULL}};
const Segment SGI195055[] = {{&S78674[27], 6}, {NULL}};
const Segment SGI195056[] = {{&S78674[99], 9}, {NULL}};
const Segment SGI195057[] = {{&S78674[72], 6}, {NULL}};
const Segment SGI195058[] = {{&S78675[34], 16}, {NULL}};
const Segment SGI195059[] = {{&S78675[27], 6}, {NULL}};
const Segment SGI195060[] = {{&S78675[103], 9}, {NULL}};
const Segment SGI195061[] = {{&S78675[76], 6}, {NULL}};
const Segment SGI195062[] = {{&S78676[34], 21}, {NULL}};
const Segment SGI195063[] = {{&S78676[27], 6}, {NULL}};
const Segment SGI195064[] = {{&S57606[34], 9}, {NULL}};
const Segment SGI195065[] = {{&S57606[27], 6}, {NULL}};
const Segment SGI195066[] = {{&S78676[108], 9}, {NULL}};
const Segment SGI195067[] = {{&S78676[81], 6}, {NULL}};
const Segment SGI195068[] = {{&S57606[95], 9}, {NULL}};
const Segment SGI195069[] = {{&S57606[68], 6}, {NULL}};
const Segment SGI195070[] = {{&S57606[130], 8}, {NULL}};
const Segment SGI195071[] = {{&S57607[34], 9}, {NULL}};
const Segment SGI195072[] = {{&S57607[27], 6}, {NULL}};
const Segment SGI195073[] = {{&S57607[95], 9}, {NULL}};
const Segment SGI195074[] = {{&S57607[68], 6}, {NULL}};
const Segment SGI195075[] = {{&S57607[129], 8}, {NULL}};
const Segment SGI195076[] = {{&S72328[34], 8}, {NULL}};
const Segment SGI195077[] = {{&S72328[27], 6}, {NULL}};
const Segment SGI195078[] = {{&S72328[95], 9}, {NULL}};
const Segment SGI195079[] = {{&S72328[68], 6}, {NULL}};
const Segment SGI195080[] = {{&S78682[59], 10}, {NULL}};
const Segment SGI195081[] = {{&S78682[59], 22}, {NULL}};
const Segment SGI195082[] = {{&S78682[45], 6}, {NULL}};
const Segment SGI195083[] = {{&S78682[46], 3}, {NULL}};
const Segment SGI195084[] = {{&S78682[95], 6}, {NULL}};
const Segment SGI195085[] = {{&S72330[44], 15}, {NULL}};
const Segment SGI195086[] = {{&S78682[0], 14}, {NULL}};
const Segment SGI195087[] = {{&S72330[44], 27}, {NULL}};
const Segment SGI195088[] = {{&S72330[30], 6}, {NULL}};
const Segment SGI195089[] = {{&S72330[120], 3}, {NULL}};
const Segment SGI195090[] = {{&S72330[85], 6}, {NULL}};
const Segment SGI195091[] = {{&S72331[38], 15}, {NULL}};
const Segment SGI195092[] = {{&S72331[31], 6}, {NULL}};
const Segment SGI195093[] = {{&S78684[57], 10}, {NULL}};
const Segment SGI195094[] = {{&S78684[45], 6}, {NULL}};
const Segment SGI195095[] = {{&S78684[46], 3}, {NULL}};
const Segment SGI195096[] = {{&S78684[93], 6}, {NULL}};
const Segment SGI195097[] = {{&S78684[0], 14}, {NULL}};
const Segment SGI195098[] = {{&S72333[34], 8}, {NULL}};
const Segment SGI195099[] = {{&S72333[27], 6}, {NULL}};
const Segment SGI195100[] = {{&S72333[95], 9}, {NULL}};
const Segment SGI195101[] = {{&S72333[68], 6}, {NULL}};
const Segment SGI195102[] = {{&S78686[44], 13}, {NULL}};
const Segment SGI195103[] = {{&S78686[44], 25}, {NULL}};
const Segment SGI195104[] = {{&S78686[30], 6}, {NULL}};
const Segment SGI195105[] = {{&S78686[31], 3}, {NULL}};
const Segment SGI195106[] = {{&S78686[83], 6}, {NULL}};
const Segment SGI195107[] = {{&S57620[44], 17}, {NULL}};
const Segment SGI195108[] = {{&S57620[44], 28}, {NULL}};
const Segment SGI195109[] = {{&S57620[30], 6}, {NULL}};
const Segment SGI195110[] = {{&S57620[121], 3}, {NULL}};
const Segment SGI195111[] = {{&S57620[86], 6}, {NULL}};
const Segment SGI195112[] = {{&S57621[44], 17}, {NULL}};
const Segment SGI195113[] = {{&S57621[44], 28}, {NULL}};
const Segment SGI195114[] = {{&S57621[30], 6}, {NULL}};
const Segment SGI195115[] = {{&S57621[121], 3}, {NULL}};
const Segment SGI195116[] = {{&S57621[86], 6}, {NULL}};
const Segment SGI195117[] = {{&S78690[34], 8}, {NULL}};
const Segment SGI195118[] = {{&S78690[27], 6}, {NULL}};
const Segment SGI195119[] = {{&S78690[108], 9}, {NULL}};
const Segment SGI195120[] = {{&S78690[81], 6}, {NULL}};
const Segment SGI195121[] = {{&S78692[44], 4}, {NULL}};
const Segment SGI195122[] = {{&S78692[44], 16}, {NULL}};
const Segment SGI195123[] = {{&S78692[30], 6}, {NULL}};
const Segment SGI195124[] = {{&S78692[31], 3}, {NULL}};
const Segment SGI195125[] = {{&S78692[74], 6}, {NULL}};
const Segment SGI195126[] = {{&S78702[34], 10}, {NULL}};
const Segment SGI195127[] = {{&S78702[27], 6}, {NULL}};
const Segment SGI195128[] = {{&S78702[97], 9}, {NULL}};
const Segment SGI195129[] = {{&S78702[70], 6}, {NULL}};
const Segment SGI195130[] = {{&S78703[34], 10}, {NULL}};
const Segment SGI195131[] = {{&S78703[27], 6}, {NULL}};
const Segment SGI195132[] = {{&S78703[102], 9}, {NULL}};
const Segment SGI195133[] = {{&S78703[75], 6}, {NULL}};
const Segment SGI195134[] = {{&S78704[34], 11}, {NULL}};
const Segment SGI195135[] = {{&S78704[27], 6}, {NULL}};
const Segment SGI195136[] = {{&S78704[98], 9}, {NULL}};
const Segment SGI195137[] = {{&S78704[71], 6}, {NULL}};
const Segment SGI195138[] = {{&S72358[44], 15}, {NULL}};
const Segment SGI195139[] = {{&S72358[44], 27}, {NULL}};
const Segment SGI195140[] = {{&S72358[30], 6}, {NULL}};
const Segment SGI195141[] = {{&S72358[85], 6}, {NULL}};
const Segment SGI195142[] = {{&S72358[112], 13}, {NULL}};
const Segment SGI195143[] = {{&S66724[44], 4}, {NULL}};
const Segment SGI195144[] = {{&S66724[44], 16}, {NULL}};
const Segment SGI195145[] = {{&S66724[30], 6}, {NULL}};
const Segment SGI195146[] = {{&S66724[109], 3}, {NULL}};
const Segment SGI195147[] = {{&S72360[44], 20}, {NULL}};
const Segment SGI195148[] = {{&S66724[74], 6}, {NULL}};
const Segment SGI195149[] = {{&S72360[44], 32}, {NULL}};
const Segment SGI195150[] = {{&S72360[30], 6}, {NULL}};
const Segment SGI195151[] = {{&S72360[125], 3}, {NULL}};
const Segment SGI195152[] = {{&S72360[90], 6}, {NULL}};
const Segment SGI195153[] = {{&S66725[38], 4}, {NULL}};
const Segment SGI195154[] = {{&S66725[31], 6}, {NULL}};
const Segment SGI195155[] = {{&S72361[44], 25}, {NULL}};
const Segment SGI195156[] = {{&S72361[44], 37}, {NULL}};
const Segment SGI195157[] = {{&S72361[30], 6}, {NULL}};
const Segment SGI195158[] = {{&S72361[31], 3}, {NULL}};
const Segment SGI195159[] = {{&S72361[95], 6}, {NULL}};
const Segment SGI195160[] = {{&S78711[34], 15}, {NULL}};
const Segment SGI195161[] = {{&S78711[27], 6}, {NULL}};
const Segment SGI195162[] = {{&S78711[102], 9}, {NULL}};
const Segment SGI195163[] = {{&S78711[75], 6}, {NULL}};
const Segment SGI195164[] = {{&S78711[143], 8}, {NULL}};
const Segment SGI195165[] = {{&S72363[44], 10}, {NULL}};
const Segment SGI195166[] = {{&S72363[44], 22}, {NULL}};
const Segment SGI195167[] = {{&S72363[30], 6}, {NULL}};
const Segment SGI195168[] = {{&S72363[114], 3}, {NULL}};
const Segment SGI195169[] = {{&S72363[80], 5}, {NULL}};
const Segment SGI195170[] = {{&S72364[44], 10}, {NULL}};
const Segment SGI195171[] = {{&S72364[44], 22}, {NULL}};
const Segment SGI195172[] = {{&S72364[30], 6}, {NULL}};
const Segment SGI195173[] = {{&S72364[115], 3}, {NULL}};
const Segment SGI195174[] = {{&S72364[80], 6}, {NULL}};
const Segment SGI195175[] = {{&S78716[34], 5}, {NULL}};
const Segment SGI195176[] = {{&S78716[27], 6}, {NULL}};
const Segment SGI195177[] = {{&S78716[92], 9}, {NULL}};
const Segment SGI195178[] = {{&S78716[65], 6}, {NULL}};
const Segment SGI195179[] = {{&S72380[34], 10}, {NULL}};
const Segment SGI195180[] = {{&S72380[27], 6}, {NULL}};
const Segment SGI195181[] = {{&S72380[97], 9}, {NULL}};
const Segment SGI195182[] = {{&S72380[70], 6}, {NULL}};
const Segment SGI195183[] = {{&S78734[34], 6}, {NULL}};
const Segment SGI195184[] = {{&S78734[27], 6}, {NULL}};
const Segment SGI195185[] = {{&S78734[93], 9}, {NULL}};
const Segment SGI195186[] = {{&S78734[66], 6}, {NULL}};
const Segment SGI195187[] = {{&S72383[44], 11}, {NULL}};
const Segment SGI195188[] = {{&S72383[44], 23}, {NULL}};
const Segment SGI195189[] = {{&S72383[30], 6}, {NULL}};
const Segment SGI195190[] = {{&S72383[116], 3}, {NULL}};
const Segment SGI195191[] = {{&S72383[81], 6}, {NULL}};
const Segment SGI195192[] = {{&S78736[44], 11}, {NULL}};
const Segment SGI195193[] = {{&S78736[44], 23}, {NULL}};
const Segment SGI195194[] = {{&S78736[30], 6}, {NULL}};
const Segment SGI195195[] = {{&S78736[31], 3}, {NULL}};
const Segment SGI195196[] = {{&S78736[96], 6}, {NULL}};
const Segment SGI195197[] = {{&S78736[69], 8}, {NULL}};
const Segment SGI195198[] = {{&S72385[34], 10}, {NULL}};
const Segment SGI195199[] = {{&S72385[27], 6}, {NULL}};
const Segment SGI195200[] = {{&S72385[97], 9}, {NULL}};
const Segment SGI195201[] = {{&S72385[70], 6}, {NULL}};
const Segment SGI195202[] = {{&S78738[34], 8}, {NULL}};
const Segment SGI195203[] = {{&S78738[27], 6}, {NULL}};
const Segment SGI195204[] = {{&S78738[95], 9}, {NULL}};
const Segment SGI195205[] = {{&S78738[68], 6}, {NULL}};
const Segment SGI195206[] = {{&S78742[34], 7}, {NULL}};
const Segment SGI195207[] = {{&S78742[27], 6}, {NULL}};
const Segment SGI195208[] = {{&S78742[99], 9}, {NULL}};
const Segment SGI195209[] = {{&S78742[72], 6}, {NULL}};
const Segment SGI195210[] = {{&S78743[44], 10}, {NULL}};
const Segment SGI195211[] = {{&S78743[44], 22}, {NULL}};
const Segment SGI195212[] = {{&S78743[30], 6}, {NULL}};
const Segment SGI195213[] = {{&S78743[80], 6}, {NULL}};
const Segment SGI195214[] = {{&S78743[107], 13}, {NULL}};
const Segment SGI195215[] = {{&S72394[34], 6}, {NULL}};
const Segment SGI195216[] = {{&S72394[27], 6}, {NULL}};
const Segment SGI195217[] = {{&S72394[93], 9}, {NULL}};
const Segment SGI195218[] = {{&S72394[66], 6}, {NULL}};
const Segment SGI195219[] = {{&S72395[44], 6}, {NULL}};
const Segment SGI195220[] = {{&S72395[44], 18}, {NULL}};
const Segment SGI195221[] = {{&S72395[30], 6}, {NULL}};
const Segment SGI195222[] = {{&S72395[76], 6}, {NULL}};
const Segment SGI195223[] = {{&S72395[103], 13}, {NULL}};
const Segment SGI195224[] = {{&S78746[44], 7}, {NULL}};
const Segment SGI195225[] = {{&S78746[44], 25}, {NULL}};
const Segment SGI195226[] = {{&S78746[30], 6}, {NULL}};
const Segment SGI195227[] = {{&S78746[83], 6}, {NULL}};
const Segment SGI195228[] = {{&S78746[110], 13}, {NULL}};
const Segment SGI195229[] = {{&S72396[44], 7}, {NULL}};
const Segment SGI195230[] = {{&S72396[44], 19}, {NULL}};
const Segment SGI195231[] = {{&S72396[30], 6}, {NULL}};
const Segment SGI195232[] = {{&S72396[77], 6}, {NULL}};
const Segment SGI195233[] = {{&S72396[104], 13}, {NULL}};
const Segment SGI195234[] = {{&S72397[44], 7}, {NULL}};
const Segment SGI195235[] = {{&S72397[44], 19}, {NULL}};
const Segment SGI195236[] = {{&S72397[30], 6}, {NULL}};
const Segment SGI195237[] = {{&S72397[77], 6}, {NULL}};
const Segment SGI195238[] = {{&S72397[104], 13}, {NULL}};
const Segment SGI195239[] = {{&S72398[44], 8}, {NULL}};
const Segment SGI195240[] = {{&S72398[44], 20}, {NULL}};
const Segment SGI195241[] = {{&S72398[30], 6}, {NULL}};
const Segment SGI195242[] = {{&S72398[78], 6}, {NULL}};
const Segment SGI195243[] = {{&S72398[105], 13}, {NULL}};
const Segment SGI195244[] = {{&S72399[44], 7}, {NULL}};
const Segment SGI195245[] = {{&S72399[44], 19}, {NULL}};
const Segment SGI195246[] = {{&S72399[30], 6}, {NULL}};
const Segment SGI195247[] = {{&S72399[77], 6}, {NULL}};
const Segment SGI195248[] = {{&S72399[104], 13}, {NULL}};
const Segment SGI195249[] = {{&S78758[34], 9}, {NULL}};
const Segment SGI195250[] = {{&S78758[27], 6}, {NULL}};
const Segment SGI195251[] = {{&S78758[96], 9}, {NULL}};
const Segment SGI195252[] = {{&S78758[69], 6}, {NULL}};
const Segment SGI195253[] = {{&S72414[44], 21}, {NULL}};
const Segment SGI195254[] = {{&S72414[44], 33}, {NULL}};
const Segment SGI195255[] = {{&S72414[30], 6}, {NULL}};
const Segment SGI195256[] = {{&S72414[126], 3}, {NULL}};
const Segment SGI195257[] = {{&S72414[91], 6}, {NULL}};
const Segment SGI195258[] = {{&S72416[34], 15}, {NULL}};
const Segment SGI195259[] = {{&S72416[27], 6}, {NULL}};
const Segment SGI195260[] = {{&S72416[102], 9}, {NULL}};
const Segment SGI195261[] = {{&S72416[75], 6}, {NULL}};
const Segment SGI195262[] = {{&S72417[34], 12}, {NULL}};
const Segment SGI195263[] = {{&S72417[27], 6}, {NULL}};
const Segment SGI195264[] = {{&S72417[99], 9}, {NULL}};
const Segment SGI195265[] = {{&S72417[72], 6}, {NULL}};
const Segment SGI195266[] = {{&S72418[34], 12}, {NULL}};
const Segment SGI195267[] = {{&S72418[27], 6}, {NULL}};
const Segment SGI195268[] = {{&S72418[99], 9}, {NULL}};
const Segment SGI195269[] = {{&S72418[72], 6}, {NULL}};
const Segment SGI195270[] = {{&S72423[44], 13}, {NULL}};
const Segment SGI195271[] = {{&S72423[44], 25}, {NULL}};
const Segment SGI195272[] = {{&S72423[30], 6}, {NULL}};
const Segment SGI195273[] = {{&S72423[118], 3}, {NULL}};
const Segment SGI195274[] = {{&S72423[83], 6}, {NULL}};
const Segment SGI195275[] = {{&S78782[44], 10}, {NULL}};
const Segment SGI195276[] = {{&S78782[44], 19}, {NULL}};
const Segment SGI195277[] = {{&S78782[30], 6}, {NULL}};
const Segment SGI195278[] = {{&S78782[31], 3}, {NULL}};
const Segment SGI195279[] = {{&S78782[77], 6}, {NULL}};
const Segment SGI195280[] = {{&S72424[44], 18}, {NULL}};
const Segment SGI195281[] = {{&S72424[44], 30}, {NULL}};
const Segment SGI195282[] = {{&S72424[30], 6}, {NULL}};
const Segment SGI195283[] = {{&S72424[123], 3}, {NULL}};
const Segment SGI195284[] = {{&S72424[88], 6}, {NULL}};
const Segment SGI195285[] = {{&S78783[44], 8}, {NULL}};
const Segment SGI195286[] = {{&S78783[44], 20}, {NULL}};
const Segment SGI195287[] = {{&S78783[30], 6}, {NULL}};
const Segment SGI195288[] = {{&S78783[31], 3}, {NULL}};
const Segment SGI195289[] = {{&S78783[78], 6}, {NULL}};
const Segment SGI195290[] = {{&S57744[34], 18}, {NULL}};
const Segment SGI195291[] = {{&S57744[27], 6}, {NULL}};
const Segment SGI195292[] = {{&S57744[104], 9}, {NULL}};
const Segment SGI195293[] = {{&S57744[77], 6}, {NULL}};
const Segment SGI195294[] = {{&S57745[34], 11}, {NULL}};
const Segment SGI195295[] = {{&S57745[27], 6}, {NULL}};
const Segment SGI195296[] = {{&S57745[97], 9}, {NULL}};
const Segment SGI195297[] = {{&S57745[70], 6}, {NULL}};
const Segment SGI195298[] = {{&S78798[44], 10}, {NULL}};
const Segment SGI195299[] = {{&S78798[44], 22}, {NULL}};
const Segment SGI195300[] = {{&S78798[30], 6}, {NULL}};
const Segment SGI195301[] = {{&S78798[80], 6}, {NULL}};
const Segment SGI195302[] = {{&S78798[107], 13}, {NULL}};
const Segment SGI195303[] = {{&S67727[44], 10}, {NULL}};
const Segment SGI195304[] = {{&S67727[44], 22}, {NULL}};
const Segment SGI195305[] = {{&S67727[30], 6}, {NULL}};
const Segment SGI195306[] = {{&S67727[115], 3}, {NULL}};
const Segment SGI195307[] = {{&S67727[80], 6}, {NULL}};
const Segment SGI195308[] = {{&S57757[34], 10}, {NULL}};
const Segment SGI195309[] = {{&S57757[27], 6}, {NULL}};
const Segment SGI195310[] = {{&S57757[96], 9}, {NULL}};
const Segment SGI195311[] = {{&S57757[69], 6}, {NULL}};
const Segment SGI195312[] = {{&S57757[137], 8}, {NULL}};
const Segment SGI195313[] = {{&S67729[38], 10}, {NULL}};
const Segment SGI195314[] = {{&S67729[31], 6}, {NULL}};
const Segment SGI195315[] = {{&S78812[34], 8}, {NULL}};
const Segment SGI195316[] = {{&S78812[27], 6}, {NULL}};
const Segment SGI195317[] = {{&S78812[95], 9}, {NULL}};
const Segment SGI195318[] = {{&S78812[68], 6}, {NULL}};
const Segment SGI195319[] = {{&S66850[34], 16}, {NULL}};
const Segment SGI195320[] = {{&S66850[27], 6}, {NULL}};
const Segment SGI195321[] = {{&S66850[103], 9}, {NULL}};
const Segment SGI195322[] = {{&S66850[76], 6}, {NULL}};
const Segment SGI195323[] = {{&S78813[34], 7}, {NULL}};
const Segment SGI195324[] = {{&S78813[27], 6}, {NULL}};
const Segment SGI195325[] = {{&S78813[94], 9}, {NULL}};
const Segment SGI195326[] = {{&S78813[67], 6}, {NULL}};
const Segment SGI195327[] = {{&S67736[44], 10}, {NULL}};
const Segment SGI195328[] = {{&S67736[44], 22}, {NULL}};
const Segment SGI195329[] = {{&S67736[30], 6}, {NULL}};
const Segment SGI195330[] = {{&S67736[115], 3}, {NULL}};
const Segment SGI195331[] = {{&S67736[80], 6}, {NULL}};
const Segment SGI195332[] = {{&S67737[38], 10}, {NULL}};
const Segment SGI195333[] = {{&S67737[31], 6}, {NULL}};
const Segment SGI195334[] = {{&S67738[44], 10}, {NULL}};
const Segment SGI195335[] = {{&S67738[44], 22}, {NULL}};
const Segment SGI195336[] = {{&S67738[30], 6}, {NULL}};
const Segment SGI195337[] = {{&S67738[83], 6}, {NULL}};
const Segment SGI195338[] = {{&S72469[34], 13}, {NULL}};
const Segment SGI195339[] = {{&S72469[27], 6}, {NULL}};
const Segment SGI195340[] = {{&S72469[100], 9}, {NULL}};
const Segment SGI195341[] = {{&S72469[73], 6}, {NULL}};
const Segment SGI195342[] = {{&S72472[34], 6}, {NULL}};
const Segment SGI195343[] = {{&S72472[27], 6}, {NULL}};
const Segment SGI195344[] = {{&S72472[93], 9}, {NULL}};
const Segment SGI195345[] = {{&S72472[66], 6}, {NULL}};
const Segment SGI195346[] = {{&S78831[34], 9}, {NULL}};
const Segment SGI195347[] = {{&S78831[27], 6}, {NULL}};
const Segment SGI195348[] = {{&S78831[96], 9}, {NULL}};
const Segment SGI195349[] = {{&S78831[69], 6}, {NULL}};
const Segment SGI195350[] = {{&S78832[34], 9}, {NULL}};
const Segment SGI195351[] = {{&S78832[27], 6}, {NULL}};
const Segment SGI195352[] = {{&S78832[96], 9}, {NULL}};
const Segment SGI195353[] = {{&S78832[69], 6}, {NULL}};
const Segment SGI195354[] = {{&S78833[34], 9}, {NULL}};
const Segment SGI195355[] = {{&S78833[27], 6}, {NULL}};
const Segment SGI195356[] = {{&S78833[96], 9}, {NULL}};
const Segment SGI195357[] = {{&S78833[69], 6}, {NULL}};
const Segment SGI195358[] = {{&S67760[34], 10}, {NULL}};
const Segment SGI195359[] = {{&S67760[27], 6}, {NULL}};
const Segment SGI195360[] = {{&S67760[97], 9}, {NULL}};
const Segment SGI195361[] = {{&S67760[70], 6}, {NULL}};
const Segment SGI195362[] = {{&S78836[44], 11}, {NULL}};
const Segment SGI195363[] = {{&S78836[44], 23}, {NULL}};
const Segment SGI195364[] = {{&S78836[30], 6}, {NULL}};
const Segment SGI195365[] = {{&S78836[31], 3}, {NULL}};
const Segment SGI195366[] = {{&S67761[34], 11}, {NULL}};
const Segment SGI195367[] = {{&S78836[81], 6}, {NULL}};
const Segment SGI195368[] = {{&S67761[27], 6}, {NULL}};
const Segment SGI195369[] = {{&S67761[98], 9}, {NULL}};
const Segment SGI195370[] = {{&S67761[71], 6}, {NULL}};
const Segment SGI195371[] = {{&S78841[34], 7}, {NULL}};
const Segment SGI195372[] = {{&S78841[27], 6}, {NULL}};
const Segment SGI195373[] = {{&S78841[94], 9}, {NULL}};
const Segment SGI195374[] = {{&S78841[67], 6}, {NULL}};
const Segment SGI195375[] = {{&S78842[52], 15}, {NULL}};
const Segment SGI195376[] = {{&S78842[52], 27}, {NULL}};
const Segment SGI195377[] = {{&S78842[38], 6}, {NULL}};
const Segment SGI195378[] = {{&S78842[93], 5}, {NULL}};
const Segment SGI195379[] = {{&S78842[0], 14}, {NULL}};
const Segment SGI195380[] = {{&S66893[44], 12}, {NULL}};
const Segment SGI195381[] = {{&S66893[44], 24}, {NULL}};
const Segment SGI195382[] = {{&S66893[30], 6}, {NULL}};
const Segment SGI195383[] = {{&S66893[135], 3}, {NULL}};
const Segment SGI195384[] = {{&S66893[100], 6}, {NULL}};
const Segment SGI195385[] = {{&S66893[70], 8}, {NULL}};
const Segment SGI195386[] = {{&S78845[44], 18}, {NULL}};
const Segment SGI195387[] = {{&S78845[44], 30}, {NULL}};
const Segment SGI195388[] = {{&S78845[30], 6}, {NULL}};
const Segment SGI195389[] = {{&S78845[31], 3}, {NULL}};
const Segment SGI195390[] = {{&S66895[44], 9}, {NULL}};
const Segment SGI195391[] = {{&S78845[88], 6}, {NULL}};
const Segment SGI195392[] = {{&S66895[44], 21}, {NULL}};
const Segment SGI195393[] = {{&S66895[30], 6}, {NULL}};
const Segment SGI195394[] = {{&S66895[114], 3}, {NULL}};
const Segment SGI195395[] = {{&S66895[79], 6}, {NULL}};
const Segment SGI195396[] = {{&S66895[132], 11}, {NULL}};
const Segment SGI195397[] = {{&S78846[34], 11}, {NULL}};
const Segment SGI195398[] = {{&S78846[27], 6}, {NULL}};
const Segment SGI195399[] = {{&S78846[98], 9}, {NULL}};
const Segment SGI195400[] = {{&S78846[71], 6}, {NULL}};
const Segment SGI195401[] = {{&S78848[34], 11}, {NULL}};
const Segment SGI195402[] = {{&S78848[27], 6}, {NULL}};
const Segment SGI195403[] = {{&S78848[98], 9}, {NULL}};
const Segment SGI195404[] = {{&S78848[71], 6}, {NULL}};
const Segment SGI195405[] = {{&S78854[34], 12}, {NULL}};
const Segment SGI195406[] = {{&S78854[27], 6}, {NULL}};
const Segment SGI195407[] = {{&S78854[99], 9}, {NULL}};
const Segment SGI195408[] = {{&S78854[72], 6}, {NULL}};
const Segment SGI195409[] = {{&S78857[34], 16}, {NULL}};
const Segment SGI195410[] = {{&S78857[27], 6}, {NULL}};
const Segment SGI195411[] = {{&S78857[110], 9}, {NULL}};
const Segment SGI195412[] = {{&S78857[83], 6}, {NULL}};
const Segment SGI195413[] = {{&S78858[34], 15}, {NULL}};
const Segment SGI195414[] = {{&S78858[27], 6}, {NULL}};
const Segment SGI195415[] = {{&S78858[108], 9}, {NULL}};
const Segment SGI195416[] = {{&S78858[81], 6}, {NULL}};
const Segment SGI195417[] = {{&S72519[34], 13}, {NULL}};
const Segment SGI195418[] = {{&S72519[27], 6}, {NULL}};
const Segment SGI195419[] = {{&S72519[114], 9}, {NULL}};
const Segment SGI195420[] = {{&S72519[87], 6}, {NULL}};
const Segment SGI195421[] = {{&S67798[44], 7}, {NULL}};
const Segment SGI195422[] = {{&S67798[44], 19}, {NULL}};
const Segment SGI195423[] = {{&S67798[30], 6}, {NULL}};
const Segment SGI195424[] = {{&S67798[112], 3}, {NULL}};
const Segment SGI195425[] = {{&S67798[77], 6}, {NULL}};
const Segment SGI195426[] = {{&S67799[38], 7}, {NULL}};
const Segment SGI195427[] = {{&S67799[31], 6}, {NULL}};
const Segment SGI195428[] = {{&S72524[44], 6}, {NULL}};
const Segment SGI195429[] = {{&S72524[44], 27}, {NULL}};
const Segment SGI195430[] = {{&S72524[30], 6}, {NULL}};
const Segment SGI195431[] = {{&S72524[119], 3}, {NULL}};
const Segment SGI195432[] = {{&S72524[85], 5}, {NULL}};
const Segment SGI195433[] = {{&S78877[34], 6}, {NULL}};
const Segment SGI195434[] = {{&S78877[27], 6}, {NULL}};
const Segment SGI195435[] = {{&S78877[93], 9}, {NULL}};
const Segment SGI195436[] = {{&S78877[66], 6}, {NULL}};
const Segment SGI195437[] = {{&S57859[34], 9}, {NULL}};
const Segment SGI195438[] = {{&S66944[34], 10}, {NULL}};
const Segment SGI195439[] = {{&S57859[27], 6}, {NULL}};
const Segment SGI195440[] = {{&S66944[27], 6}, {NULL}};
const Segment SGI195441[] = {{&S57859[95], 9}, {NULL}};
const Segment SGI195442[] = {{&S66944[97], 9}, {NULL}};
const Segment SGI195443[] = {{&S57859[68], 6}, {NULL}};
const Segment SGI195444[] = {{&S66944[70], 6}, {NULL}};
const Segment SGI195445[] = {{&S72543[44], 29}, {NULL}};
const Segment SGI195446[] = {{&S72543[44], 41}, {NULL}};
const Segment SGI195447[] = {{&S72543[30], 6}, {NULL}};
const Segment SGI195448[] = {{&S67822[34], 9}, {NULL}};
const Segment SGI195449[] = {{&S72543[134], 3}, {NULL}};
const Segment SGI195450[] = {{&S67822[27], 6}, {NULL}};
const Segment SGI195451[] = {{&S72543[99], 6}, {NULL}};
const Segment SGI195452[] = {{&S57860[27], 6}, {NULL}};
const Segment SGI195453[] = {{&S67822[96], 9}, {NULL}};
const Segment SGI195454[] = {{&S78886[59], 10}, {NULL}};
const Segment SGI195455[] = {{&S67822[69], 6}, {NULL}};
const Segment SGI195456[] = {{&S78886[59], 22}, {NULL}};
const Segment SGI195457[] = {{&S78886[45], 6}, {NULL}};
const Segment SGI195458[] = {{&S57860[116], 9}, {NULL}};
const Segment SGI195459[] = {{&S57860[89], 6}, {NULL}};
const Segment SGI195460[] = {{&S78886[46], 3}, {NULL}};
const Segment SGI195461[] = {{&S78886[95], 6}, {NULL}};
const Segment SGI195462[] = {{&S78886[0], 14}, {NULL}};
const Segment SGI195463[] = {{&S72544[44], 23}, {NULL}};
const Segment SGI195464[] = {{&S72544[44], 35}, {NULL}};
const Segment SGI195465[] = {{&S72544[30], 6}, {NULL}};
const Segment SGI195466[] = {{&S67823[34], 9}, {NULL}};
const Segment SGI195467[] = {{&S67823[27], 6}, {NULL}};
const Segment SGI195468[] = {{&S72544[128], 3}, {NULL}};
const Segment SGI195469[] = {{&S72544[93], 6}, {NULL}};
const Segment SGI195470[] = {{&S57861[34], 9}, {NULL}};
const Segment SGI195471[] = {{&S57861[27], 6}, {NULL}};
const Segment SGI195472[] = {{&S67823[96], 9}, {NULL}};
const Segment SGI195473[] = {{&S67823[69], 6}, {NULL}};
const Segment SGI195474[] = {{&S78887[59], 11}, {NULL}};
const Segment SGI195475[] = {{&S78887[59], 23}, {NULL}};
const Segment SGI195476[] = {{&S78887[45], 6}, {NULL}};
const Segment SGI195477[] = {{&S57861[95], 9}, {NULL}};
const Segment SGI195478[] = {{&S57861[68], 6}, {NULL}};
const Segment SGI195479[] = {{&S78887[46], 3}, {NULL}};
const Segment SGI195480[] = {{&S78887[96], 6}, {NULL}};
const Segment SGI195481[] = {{&S72545[38], 29}, {NULL}};
const Segment SGI195482[] = {{&S78887[0], 14}, {NULL}};
const Segment SGI195483[] = {{&S72545[31], 6}, {NULL}};
const Segment SGI195484[] = {{&S67824[34], 14}, {NULL}};
const Segment SGI195485[] = {{&S67824[27], 6}, {NULL}};
const Segment SGI195486[] = {{&S57862[41], 18}, {NULL}};
const Segment SGI195487[] = {{&S67824[101], 9}, {NULL}};
const Segment SGI195488[] = {{&S57862[41], 29}, {NULL}};
const Segment SGI195489[] = {{&S67824[74], 6}, {NULL}};
const Segment SGI195490[] = {{&S57862[27], 6}, {NULL}};
const Segment SGI195491[] = {{&S78888[44], 12}, {NULL}};
const Segment SGI195492[] = {{&S78888[44], 24}, {NULL}};
const Segment SGI195493[] = {{&S57862[119], 3}, {NULL}};
const Segment SGI195494[] = {{&S78888[30], 6}, {NULL}};
const Segment SGI195495[] = {{&S57862[123], 9}, {NULL}};
const Segment SGI195496[] = {{&S57862[84], 6}, {NULL}};
const Segment SGI195497[] = {{&S78888[31], 3}, {NULL}};
const Segment SGI195498[] = {{&S78888[82], 6}, {NULL}};
const Segment SGI195499[] = {{&S57863[41], 18}, {NULL}};
const Segment SGI195500[] = {{&S57863[41], 29}, {NULL}};
const Segment SGI195501[] = {{&S57863[27], 6}, {NULL}};
const Segment SGI195502[] = {{&S78889[44], 7}, {NULL}};
const Segment SGI195503[] = {{&S78889[44], 19}, {NULL}};
const Segment SGI195504[] = {{&S57863[119], 3}, {NULL}};
const Segment SGI195505[] = {{&S78889[30], 6}, {NULL}};
const Segment SGI195506[] = {{&S57863[123], 9}, {NULL}};
const Segment SGI195507[] = {{&S57863[84], 6}, {NULL}};
const Segment SGI195508[] = {{&S78889[31], 3}, {NULL}};
const Segment SGI195509[] = {{&S78889[77], 6}, {NULL}};
const Segment SGI195510[] = {{&S57865[41], 18}, {NULL}};
const Segment SGI195511[] = {{&S57865[41], 29}, {NULL}};
const Segment SGI195512[] = {{&S57865[27], 6}, {NULL}};
const Segment SGI195513[] = {{&S72551[34], 11}, {NULL}};
const Segment SGI195514[] = {{&S72551[27], 6}, {NULL}};
const Segment SGI195515[] = {{&S57865[119], 3}, {NULL}};
const Segment SGI195516[] = {{&S57865[123], 9}, {NULL}};
const Segment SGI195517[] = {{&S57865[84], 6}, {NULL}};
const Segment SGI195518[] = {{&S72551[98], 9}, {NULL}};
const Segment SGI195519[] = {{&S72551[71], 6}, {NULL}};
const Segment SGI195520[] = {{&S57866[41], 19}, {NULL}};
const Segment SGI195521[] = {{&S72552[34], 8}, {NULL}};
const Segment SGI195522[] = {{&S57866[41], 30}, {NULL}};
const Segment SGI195523[] = {{&S57866[27], 6}, {NULL}};
const Segment SGI195524[] = {{&S72552[27], 6}, {NULL}};
const Segment SGI195525[] = {{&S57866[120], 3}, {NULL}};
const Segment SGI195526[] = {{&S78894[34], 11}, {NULL}};
const Segment SGI195527[] = {{&S72552[95], 9}, {NULL}};
const Segment SGI195528[] = {{&S57866[124], 9}, {NULL}};
const Segment SGI195529[] = {{&S78894[27], 6}, {NULL}};
const Segment SGI195530[] = {{&S72552[68], 6}, {NULL}};
const Segment SGI195531[] = {{&S57866[85], 6}, {NULL}};
const Segment SGI195532[] = {{&S78894[98], 9}, {NULL}};
const Segment SGI195533[] = {{&S78894[71], 6}, {NULL}};
const Segment SGI195534[] = {{&S57867[41], 18}, {NULL}};
const Segment SGI195535[] = {{&S57867[41], 29}, {NULL}};
const Segment SGI195536[] = {{&S57867[27], 6}, {NULL}};
const Segment SGI195537[] = {{&S57867[119], 3}, {NULL}};
const Segment SGI195538[] = {{&S57867[123], 9}, {NULL}};
const Segment SGI195539[] = {{&S57867[84], 6}, {NULL}};
const Segment SGI195540[] = {{&S57868[34], 9}, {NULL}};
const Segment SGI195541[] = {{&S57868[27], 6}, {NULL}};
const Segment SGI195542[] = {{&S57868[95], 9}, {NULL}};
const Segment SGI195543[] = {{&S57868[68], 6}, {NULL}};
const Segment SGI195544[] = {{&S57870[41], 18}, {NULL}};
const Segment SGI195545[] = {{&S57870[41], 29}, {NULL}};
const Segment SGI195546[] = {{&S57870[27], 6}, {NULL}};
const Segment SGI195547[] = {{&S57870[119], 3}, {NULL}};
const Segment SGI195548[] = {{&S57870[123], 9}, {NULL}};
const Segment SGI195549[] = {{&S57870[84], 6}, {NULL}};
const Segment SGI195550[] = {{&S67843[44], 17}, {NULL}};
const Segment SGI195551[] = {{&S67843[44], 37}, {NULL}};
const Segment SGI195552[] = {{&S57871[41], 18}, {NULL}};
const Segment SGI195553[] = {{&S67843[30], 6}, {NULL}};
const Segment SGI195554[] = {{&S57871[41], 29}, {NULL}};
const Segment SGI195555[] = {{&S57871[27], 6}, {NULL}};
const Segment SGI195556[] = {{&S67843[130], 3}, {NULL}};
const Segment SGI195557[] = {{&S67843[95], 6}, {NULL}};
const Segment SGI195558[] = {{&S57871[119], 3}, {NULL}};
const Segment SGI195559[] = {{&S57871[123], 9}, {NULL}};
const Segment SGI195560[] = {{&S57871[84], 6}, {NULL}};
const Segment SGI195561[] = {{&S78904[44], 7}, {NULL}};
const Segment SGI195562[] = {{&S78904[44], 19}, {NULL}};
const Segment SGI195563[] = {{&S78904[30], 6}, {NULL}};
const Segment SGI195564[] = {{&S57872[41], 19}, {NULL}};
const Segment SGI195565[] = {{&S57872[41], 30}, {NULL}};
const Segment SGI195566[] = {{&S57872[27], 6}, {NULL}};
const Segment SGI195567[] = {{&S78904[31], 3}, {NULL}};
const Segment SGI195568[] = {{&S78904[77], 6}, {NULL}};
const Segment SGI195569[] = {{&S57872[120], 3}, {NULL}};
const Segment SGI195570[] = {{&S57872[124], 9}, {NULL}};
const Segment SGI195571[] = {{&S57872[85], 6}, {NULL}};
const Segment SGI195572[] = {{&S57873[34], 11}, {NULL}};
const Segment SGI195573[] = {{&S57873[27], 6}, {NULL}};
const Segment SGI195574[] = {{&S57873[97], 9}, {NULL}};
const Segment SGI195575[] = {{&S57873[70], 6}, {NULL}};
const Segment SGI195576[] = {{&S78906[44], 10}, {NULL}};
const Segment SGI195577[] = {{&S78906[44], 22}, {NULL}};
const Segment SGI195578[] = {{&S78906[30], 6}, {NULL}};
const Segment SGI195579[] = {{&S78906[111], 2}, {NULL}};
const Segment SGI195580[] = {{&S78906[80], 5}, {NULL}};
const Segment SGI195581[] = {{&S67850[34], 10}, {NULL}};
const Segment SGI195582[] = {{&S67850[27], 6}, {NULL}};
const Segment SGI195583[] = {{&S57875[41], 23}, {NULL}};
const Segment SGI195584[] = {{&S57875[41], 34}, {NULL}};
const Segment SGI195585[] = {{&S57875[27], 6}, {NULL}};
const Segment SGI195586[] = {{&S67850[97], 9}, {NULL}};
const Segment SGI195587[] = {{&S67850[70], 6}, {NULL}};
const Segment SGI195588[] = {{&S57875[124], 3}, {NULL}};
const Segment SGI195589[] = {{&S57875[128], 9}, {NULL}};
const Segment SGI195590[] = {{&S57875[89], 6}, {NULL}};
const Segment SGI195591[] = {{&S57878[41], 18}, {NULL}};
const Segment SGI195592[] = {{&S57878[41], 29}, {NULL}};
const Segment SGI195593[] = {{&S57878[27], 6}, {NULL}};
const Segment SGI195594[] = {{&S57878[119], 3}, {NULL}};
const Segment SGI195595[] = {{&S57878[123], 9}, {NULL}};
const Segment SGI195596[] = {{&S57878[84], 6}, {NULL}};
const Segment SGI195597[] = {{&S57879[34], 5}, {NULL}};
const Segment SGI195598[] = {{&S57879[27], 6}, {NULL}};
const Segment SGI195599[] = {{&S57879[91], 9}, {NULL}};
const Segment SGI195600[] = {{&S57879[64], 6}, {NULL}};
const Segment SGI195601[] = {{&S57880[41], 27}, {NULL}};
const Segment SGI195602[] = {{&S57880[41], 38}, {NULL}};
const Segment SGI195603[] = {{&S57880[27], 6}, {NULL}};
const Segment SGI195604[] = {{&S57880[128], 3}, {NULL}};
const Segment SGI195605[] = {{&S57880[132], 9}, {NULL}};
const Segment SGI195606[] = {{&S57880[93], 6}, {NULL}};
const Segment SGI195607[] = {{&S57881[41], 19}, {NULL}};
const Segment SGI195608[] = {{&S57881[41], 30}, {NULL}};
const Segment SGI195609[] = {{&S57881[27], 6}, {NULL}};
const Segment SGI195610[] = {{&S57881[120], 3}, {NULL}};
const Segment SGI195611[] = {{&S57881[124], 9}, {NULL}};
const Segment SGI195612[] = {{&S57881[85], 6}, {NULL}};
const Segment SGI195613[] = {{&S57882[41], 18}, {NULL}};
const Segment SGI195614[] = {{&S57882[41], 29}, {NULL}};
const Segment SGI195615[] = {{&S57882[27], 6}, {NULL}};
const Segment SGI195616[] = {{&S57882[119], 3}, {NULL}};
const Segment SGI195617[] = {{&S57882[123], 9}, {NULL}};
const Segment SGI195618[] = {{&S57882[84], 6}, {NULL}};
const Segment SGI195619[] = {{&S67872[34], 11}, {NULL}};
const Segment SGI195620[] = {{&S67872[27], 6}, {NULL}};
const Segment SGI195621[] = {{&S67872[98], 9}, {NULL}};
const Segment SGI195622[] = {{&S67872[71], 6}, {NULL}};
const Segment SGI195623[] = {{&S57894[34], 6}, {NULL}};
const Segment SGI195624[] = {{&S57894[27], 6}, {NULL}};
const Segment SGI195625[] = {{&S57894[92], 9}, {NULL}};
const Segment SGI195626[] = {{&S57894[65], 6}, {NULL}};
const Segment SGI195627[] = {{&S57897[34], 15}, {NULL}};
const Segment SGI195628[] = {{&S57897[27], 6}, {NULL}};
const Segment SGI195629[] = {{&S57897[108], 9}, {NULL}};
const Segment SGI195630[] = {{&S57897[81], 6}, {NULL}};
const Segment SGI195631[] = {{&S57898[34], 22}, {NULL}};
const Segment SGI195632[] = {{&S57898[27], 6}, {NULL}};
const Segment SGI195633[] = {{&S57898[115], 9}, {NULL}};
const Segment SGI195634[] = {{&S57898[88], 6}, {NULL}};
const Segment SGI195635[] = {{&S72608[44], 13}, {NULL}};
const Segment SGI195636[] = {{&S72608[44], 25}, {NULL}};
const Segment SGI195637[] = {{&S72608[30], 6}, {NULL}};
const Segment SGI195638[] = {{&S72608[118], 3}, {NULL}};
const Segment SGI195639[] = {{&S72608[83], 6}, {NULL}};
const Segment SGI195640[] = {{&S78946[34], 15}, {NULL}};
const Segment SGI195641[] = {{&S72609[34], 25}, {NULL}};
const Segment SGI195642[] = {{&S78946[27], 6}, {NULL}};
const Segment SGI195643[] = {{&S72609[27], 6}, {NULL}};
const Segment SGI195644[] = {{&S78946[117], 9}, {NULL}};
const Segment SGI195645[] = {{&S72609[112], 9}, {NULL}};
const Segment SGI195646[] = {{&S78946[90], 6}, {NULL}};
const Segment SGI195647[] = {{&S72609[85], 6}, {NULL}};
const Segment SGI195648[] = {{&S78947[44], 9}, {NULL}};
const Segment SGI195649[] = {{&S78947[44], 21}, {NULL}};
const Segment SGI195650[] = {{&S78947[30], 6}, {NULL}};
const Segment SGI195651[] = {{&S78947[31], 3}, {NULL}};
const Segment SGI195652[] = {{&S78947[79], 6}, {NULL}};
const Segment SGI195653[] = {{&S67026[34], 10}, {NULL}};
const Segment SGI195654[] = {{&S67026[27], 6}, {NULL}};
const Segment SGI195655[] = {{&S67026[97], 9}, {NULL}};
const Segment SGI195656[] = {{&S67026[70], 6}, {NULL}};
const Segment SGI195657[] = {{&S78951[44], 9}, {NULL}};
const Segment SGI195658[] = {{&S78951[44], 21}, {NULL}};
const Segment SGI195659[] = {{&S78951[30], 6}, {NULL}};
const Segment SGI195660[] = {{&S78951[31], 3}, {NULL}};
const Segment SGI195661[] = {{&S78951[98], 6}, {NULL}};
const Segment SGI195662[] = {{&S78951[67], 8}, {NULL}};
const Segment SGI195663[] = {{&S72623[59], 10}, {NULL}};
const Segment SGI195664[] = {{&S72623[59], 22}, {NULL}};
const Segment SGI195665[] = {{&S72623[45], 6}, {NULL}};
const Segment SGI195666[] = {{&S72623[130], 3}, {NULL}};
const Segment SGI195667[] = {{&S72623[95], 6}, {NULL}};
const Segment SGI195668[] = {{&S72623[0], 14}, {NULL}};
const Segment SGI195669[] = {{&S67046[44], 7}, {NULL}};
const Segment SGI195670[] = {{&S67046[44], 19}, {NULL}};
const Segment SGI195671[] = {{&S67046[30], 6}, {NULL}};
const Segment SGI195672[] = {{&S67046[112], 3}, {NULL}};
const Segment SGI195673[] = {{&S67046[77], 6}, {NULL}};
const Segment SGI195674[] = {{&S57949[44], 12}, {NULL}};
const Segment SGI195675[] = {{&S57949[44], 27}, {NULL}};
const Segment SGI195676[] = {{&S57949[30], 6}, {NULL}};
const Segment SGI195677[] = {{&S57949[138], 3}, {NULL}};
const Segment SGI195678[] = {{&S57949[103], 6}, {NULL}};
const Segment SGI195679[] = {{&S57949[73], 8}, {NULL}};
const Segment SGI195680[] = {{&S57950[44], 12}, {NULL}};
const Segment SGI195681[] = {{&S57950[44], 24}, {NULL}};
const Segment SGI195682[] = {{&S57950[30], 6}, {NULL}};
const Segment SGI195683[] = {{&S57950[135], 3}, {NULL}};
const Segment SGI195684[] = {{&S57950[100], 6}, {NULL}};
const Segment SGI195685[] = {{&S57950[70], 8}, {NULL}};
const Segment SGI195686[] = {{&S67070[34], 10}, {NULL}};
const Segment SGI195687[] = {{&S67070[27], 6}, {NULL}};
const Segment SGI195688[] = {{&S67070[97], 9}, {NULL}};
const Segment SGI195689[] = {{&S67070[70], 6}, {NULL}};
const Segment SGI195690[] = {{&S79111[44], 14}, {NULL}};
const Segment SGI195691[] = {{&S79111[44], 26}, {NULL}};
const Segment SGI195692[] = {{&S79111[30], 6}, {NULL}};
const Segment SGI195693[] = {{&S79111[31], 3}, {NULL}};
const Segment SGI195694[] = {{&S79111[84], 6}, {NULL}};
const Segment SGI195695[] = {{&S72693[44], 7}, {NULL}};
const Segment SGI195696[] = {{&S72693[44], 19}, {NULL}};
const Segment SGI195697[] = {{&S72693[30], 6}, {NULL}};
const Segment SGI195698[] = {{&S72693[112], 3}, {NULL}};
const Segment SGI195699[] = {{&S72693[77], 6}, {NULL}};
const Segment SGI195700[] = {{&S72694[44], 7}, {NULL}};
const Segment SGI195701[] = {{&S72694[44], 19}, {NULL}};
const Segment SGI195702[] = {{&S72694[30], 6}, {NULL}};
const Segment SGI195703[] = {{&S72694[112], 3}, {NULL}};
const Segment SGI195704[] = {{&S72694[77], 6}, {NULL}};
const Segment SGI195705[] = {{&S72695[44], 7}, {NULL}};
const Segment SGI195706[] = {{&S72695[44], 19}, {NULL}};
const Segment SGI195707[] = {{&S72695[30], 6}, {NULL}};
const Segment SGI195708[] = {{&S72695[112], 3}, {NULL}};
const Segment SGI195709[] = {{&S72695[77], 6}, {NULL}};
const Segment SGI195710[] = {{&S67980[44], 10}, {NULL}};
const Segment SGI195711[] = {{&S67980[44], 22}, {NULL}};
const Segment SGI195712[] = {{&S67980[30], 6}, {NULL}};
const Segment SGI195713[] = {{&S67980[80], 6}, {NULL}};
const Segment SGI195714[] = {{&S67980[107], 13}, {NULL}};
const Segment SGI195715[] = {{&S72696[44], 7}, {NULL}};
const Segment SGI195716[] = {{&S72696[44], 19}, {NULL}};
const Segment SGI195717[] = {{&S72696[30], 6}, {NULL}};
const Segment SGI195718[] = {{&S72696[112], 3}, {NULL}};
const Segment SGI195719[] = {{&S72696[77], 6}, {NULL}};
const Segment SGI195720[] = {{&S67982[44], 10}, {NULL}};
const Segment SGI195721[] = {{&S67982[44], 22}, {NULL}};
const Segment SGI195722[] = {{&S67982[30], 6}, {NULL}};
const Segment SGI195723[] = {{&S67982[80], 6}, {NULL}};
const Segment SGI195724[] = {{&S67982[107], 13}, {NULL}};
const Segment SGI195725[] = {{&S72705[59], 10}, {NULL}};
const Segment SGI195726[] = {{&S72705[59], 22}, {NULL}};
const Segment SGI195727[] = {{&S72705[45], 6}, {NULL}};
const Segment SGI195728[] = {{&S72705[130], 3}, {NULL}};
const Segment SGI195729[] = {{&S72705[95], 6}, {NULL}};
const Segment SGI195730[] = {{&S72705[0], 14}, {NULL}};
const Segment SGI195731[] = {{&S72710[44], 11}, {NULL}};
const Segment SGI195732[] = {{&S72710[44], 23}, {NULL}};
const Segment SGI195733[] = {{&S72710[30], 6}, {NULL}};
const Segment SGI195734[] = {{&S72710[116], 3}, {NULL}};
const Segment SGI195735[] = {{&S72710[81], 6}, {NULL}};
const Segment SGI195736[] = {{&S72720[44], 18}, {NULL}};
const Segment SGI195737[] = {{&S72720[44], 30}, {NULL}};
const Segment SGI195738[] = {{&S72720[30], 6}, {NULL}};
const Segment SGI195739[] = {{&S72720[123], 3}, {NULL}};
const Segment SGI195740[] = {{&S72720[88], 6}, {NULL}};
const Segment SGI195741[] = {{&S68012[34], 10}, {NULL}};
const Segment SGI195742[] = {{&S68012[27], 6}, {NULL}};
const Segment SGI195743[] = {{&S68012[97], 9}, {NULL}};
const Segment SGI195744[] = {{&S68012[70], 6}, {NULL}};
const Segment SGI195745[] = {{&S68013[34], 11}, {NULL}};
const Segment SGI195746[] = {{&S68013[27], 6}, {NULL}};
const Segment SGI195747[] = {{&S68013[98], 9}, {NULL}};
const Segment SGI195748[] = {{&S68013[71], 6}, {NULL}};
const Segment SGI195749[] = {{&S68040[34], 7}, {NULL}};
const Segment SGI195750[] = {{&S68040[27], 6}, {NULL}};
const Segment SGI195751[] = {{&S68040[99], 9}, {NULL}};
const Segment SGI195752[] = {{&S68040[72], 6}, {NULL}};
const Segment SGI195753[] = {{&S72748[44], 12}, {NULL}};
const Segment SGI195754[] = {{&S72748[44], 24}, {NULL}};
const Segment SGI195755[] = {{&S72748[30], 6}, {NULL}};
const Segment SGI195756[] = {{&S72748[117], 3}, {NULL}};
const Segment SGI195757[] = {{&S72748[82], 6}, {NULL}};
const Segment SGI195758[] = {{&S68041[34], 7}, {NULL}};
const Segment SGI195759[] = {{&S68041[27], 6}, {NULL}};
const Segment SGI195760[] = {{&S68041[99], 9}, {NULL}};
const Segment SGI195761[] = {{&S68041[72], 6}, {NULL}};
const Segment SGI195762[] = {{&S68042[34], 10}, {NULL}};
const Segment SGI195763[] = {{&S68042[27], 6}, {NULL}};
const Segment SGI195764[] = {{&S68042[97], 9}, {NULL}};
const Segment SGI195765[] = {{&S68042[70], 6}, {NULL}};
const Segment SGI195766[] = {{&S72758[44], 25}, {NULL}};
const Segment SGI195767[] = {{&S72758[44], 37}, {NULL}};
const Segment SGI195768[] = {{&S72758[30], 6}, {NULL}};
const Segment SGI195769[] = {{&S72758[31], 3}, {NULL}};
const Segment SGI195770[] = {{&S72758[95], 6}, {NULL}};
const Segment SGI195771[] = {{&S68064[44], 10}, {NULL}};
const Segment SGI195772[] = {{&S68064[44], 22}, {NULL}};
const Segment SGI195773[] = {{&S68064[30], 6}, {NULL}};
const Segment SGI195774[] = {{&S68064[113], 3}, {NULL}};
const Segment SGI195775[] = {{&S68064[80], 5}, {NULL}};
const Segment SGI195776[] = {{&S68064[117], 14}, {NULL}};
const Segment SGI195777[] = {{&S68066[44], 10}, {NULL}};
const Segment SGI195778[] = {{&S68066[44], 22}, {NULL}};
const Segment SGI195779[] = {{&S68066[30], 6}, {NULL}};
const Segment SGI195780[] = {{&S68066[113], 3}, {NULL}};
const Segment SGI195781[] = {{&S68066[80], 5}, {NULL}};
const Segment SGI195782[] = {{&S68066[117], 14}, {NULL}};
const Segment SGI195783[] = {{&S68068[44], 10}, {NULL}};
const Segment SGI195784[] = {{&S68068[44], 22}, {NULL}};
const Segment SGI195785[] = {{&S68068[30], 6}, {NULL}};
const Segment SGI195786[] = {{&S68068[113], 3}, {NULL}};
const Segment SGI195787[] = {{&S68068[80], 5}, {NULL}};
const Segment SGI195788[] = {{&S68068[117], 14}, {NULL}};
const Segment SGI195789[] = {{&S68069[34], 9}, {NULL}};
const Segment SGI195790[] = {{&S68069[27], 6}, {NULL}};
const Segment SGI195791[] = {{&S68069[96], 9}, {NULL}};
const Segment SGI195792[] = {{&S68069[69], 6}, {NULL}};
const Segment SGI195793[] = {{&S68070[34], 10}, {NULL}};
const Segment SGI195794[] = {{&S68070[27], 6}, {NULL}};
const Segment SGI195795[] = {{&S68070[97], 9}, {NULL}};
const Segment SGI195796[] = {{&S68070[70], 6}, {NULL}};
const Segment SGI195797[] = {{&S68073[34], 7}, {NULL}};
const Segment SGI195798[] = {{&S68073[27], 6}, {NULL}};
const Segment SGI195799[] = {{&S72787[44], 10}, {NULL}};
const Segment SGI195800[] = {{&S68073[94], 9}, {NULL}};
const Segment SGI195801[] = {{&S72787[44], 22}, {NULL}};
const Segment SGI195802[] = {{&S68073[67], 6}, {NULL}};
const Segment SGI195803[] = {{&S72787[30], 6}, {NULL}};
const Segment SGI195804[] = {{&S72787[131], 3}, {NULL}};
const Segment SGI195805[] = {{&S72787[96], 6}, {NULL}};
const Segment SGI195806[] = {{&S72787[68], 8}, {NULL}};
const Segment SGI195807[] = {{&S68075[34], 16}, {NULL}};
const Segment SGI195808[] = {{&S68075[27], 6}, {NULL}};
const Segment SGI195809[] = {{&S68075[103], 9}, {NULL}};
const Segment SGI195810[] = {{&S68075[76], 6}, {NULL}};
const Segment SGI195811[] = {{&S72789[44], 10}, {NULL}};
const Segment SGI195812[] = {{&S72789[44], 22}, {NULL}};
const Segment SGI195813[] = {{&S72789[30], 6}, {NULL}};
const Segment SGI195814[] = {{&S72789[131], 3}, {NULL}};
const Segment SGI195815[] = {{&S72789[96], 6}, {NULL}};
const Segment SGI195816[] = {{&S72789[68], 8}, {NULL}};
const Segment SGI195817[] = {{&S68082[34], 12}, {NULL}};
const Segment SGI195818[] = {{&S68082[27], 6}, {NULL}};
const Segment SGI195819[] = {{&S68082[99], 9}, {NULL}};
const Segment SGI195820[] = {{&S68082[72], 6}, {NULL}};
const Segment SGI195821[] = {{&S68083[34], 18}, {NULL}};
const Segment SGI195822[] = {{&S72796[44], 9}, {NULL}};
const Segment SGI195823[] = {{&S72796[44], 21}, {NULL}};
const Segment SGI195824[] = {{&S68083[27], 6}, {NULL}};
const Segment SGI195825[] = {{&S72796[30], 6}, {NULL}};
const Segment SGI195826[] = {{&S72796[114], 3}, {NULL}};
const Segment SGI195827[] = {{&S68083[105], 9}, {NULL}};
const Segment SGI195828[] = {{&S72796[79], 6}, {NULL}};
const Segment SGI195829[] = {{&S68083[78], 6}, {NULL}};
const Segment SGI195830[] = {{&S67231[34], 10}, {NULL}};
const Segment SGI195831[] = {{&S67231[27], 6}, {NULL}};
const Segment SGI195832[] = {{&S67231[97], 9}, {NULL}};
const Segment SGI195833[] = {{&S67231[70], 6}, {NULL}};
const Segment SGI195834[] = {{&S68085[34], 17}, {NULL}};
const Segment SGI195835[] = {{&S68085[27], 6}, {NULL}};
const Segment SGI195836[] = {{&S68085[104], 9}, {NULL}};
const Segment SGI195837[] = {{&S68085[77], 6}, {NULL}};
const Segment SGI195838[] = {{&S67235[34], 10}, {NULL}};
const Segment SGI195839[] = {{&S67235[27], 6}, {NULL}};
const Segment SGI195840[] = {{&S67235[97], 9}, {NULL}};
const Segment SGI195841[] = {{&S67235[70], 6}, {NULL}};
const Segment SGI195842[] = {{&S72803[34], 6}, {NULL}};
const Segment SGI195843[] = {{&S72803[27], 6}, {NULL}};
const Segment SGI195844[] = {{&S72803[93], 9}, {NULL}};
const Segment SGI195845[] = {{&S72803[66], 6}, {NULL}};
const Segment SGI195846[] = {{&S72806[44], 10}, {NULL}};
const Segment SGI195847[] = {{&S72806[44], 22}, {NULL}};
const Segment SGI195848[] = {{&S72806[30], 6}, {NULL}};
const Segment SGI195849[] = {{&S72806[113], 3}, {NULL}};
const Segment SGI195850[] = {{&S72806[80], 5}, {NULL}};
const Segment SGI195851[] = {{&S72806[117], 14}, {NULL}};
const Segment SGI195852[] = {{&S78999[34], 11}, {NULL}};
const Segment SGI195853[] = {{&S78999[27], 6}, {NULL}};
const Segment SGI195854[] = {{&S78999[98], 9}, {NULL}};
const Segment SGI195855[] = {{&S78999[71], 6}, {NULL}};
const Segment SGI195856[] = {{&S68105[34], 10}, {NULL}};
const Segment SGI195857[] = {{&S68105[27], 6}, {NULL}};
const Segment SGI195858[] = {{&S68105[97], 9}, {NULL}};
const Segment SGI195859[] = {{&S68105[70], 6}, {NULL}};
const Segment SGI195860[] = {{&S79009[34], 24}, {NULL}};
const Segment SGI195861[] = {{&S79009[27], 6}, {NULL}};
const Segment SGI195862[] = {{&S79009[111], 9}, {NULL}};
const Segment SGI195863[] = {{&S79009[84], 6}, {NULL}};
const Segment SGI195864[] = {{&S79015[34], 9}, {NULL}};
const Segment SGI195865[] = {{&S79015[27], 6}, {NULL}};
const Segment SGI195866[] = {{&S79015[96], 9}, {NULL}};
const Segment SGI195867[] = {{&S79015[69], 6}, {NULL}};
const Segment SGI195868[] = {{&S79016[34], 9}, {NULL}};
const Segment SGI195869[] = {{&S79016[27], 6}, {NULL}};
const Segment SGI195870[] = {{&S72832[34], 10}, {NULL}};
const Segment SGI195871[] = {{&S79016[96], 9}, {NULL}};
const Segment SGI195872[] = {{&S79016[69], 6}, {NULL}};
const Segment SGI195873[] = {{&S72832[27], 6}, {NULL}};
const Segment SGI195874[] = {{&S72832[97], 9}, {NULL}};
const Segment SGI195875[] = {{&S72832[70], 6}, {NULL}};
const Segment SGI195876[] = {{&S72832[132], 8}, {NULL}};
const Segment SGI195877[] = {{&S72833[34], 10}, {NULL}};
const Segment SGI195878[] = {{&S72833[27], 6}, {NULL}};
const Segment SGI195879[] = {{&S72833[97], 9}, {NULL}};
const Segment SGI195880[] = {{&S72833[70], 6}, {NULL}};
const Segment SGI195881[] = {{&S72833[131], 8}, {NULL}};
const Segment SGI195882[] = {{&S79026[44], 13}, {NULL}};
const Segment SGI195883[] = {{&S79026[44], 25}, {NULL}};
const Segment SGI195884[] = {{&S79026[30], 6}, {NULL}};
const Segment SGI195885[] = {{&S79026[31], 3}, {NULL}};
const Segment SGI195886[] = {{&S79026[83], 6}, {NULL}};
const Segment SGI195887[] = {{&S72846[44], 10}, {NULL}};
const Segment SGI195888[] = {{&S72846[44], 22}, {NULL}};
const Segment SGI195889[] = {{&S72846[30], 6}, {NULL}};
const Segment SGI195890[] = {{&S72846[113], 3}, {NULL}};
const Segment SGI195891[] = {{&S72846[80], 5}, {NULL}};
const Segment SGI195892[] = {{&S72846[117], 14}, {NULL}};
const Segment SGI195893[] = {{&S79035[44], 10}, {NULL}};
const Segment SGI195894[] = {{&S79035[44], 22}, {NULL}};
const Segment SGI195895[] = {{&S79035[30], 6}, {NULL}};
const Segment SGI195896[] = {{&S79035[31], 3}, {NULL}};
const Segment SGI195897[] = {{&S79035[80], 6}, {NULL}};
const Segment SGI195898[] = {{&S58165[41], 8}, {NULL}};
const Segment SGI195899[] = {{&S79035[140], 11}, {NULL}};
const Segment SGI195900[] = {{&S58165[41], 19}, {NULL}};
const Segment SGI195901[] = {{&S58165[27], 6}, {NULL}};
const Segment SGI195902[] = {{&S58165[109], 3}, {NULL}};
const Segment SGI195903[] = {{&S58165[113], 9}, {NULL}};
const Segment SGI195904[] = {{&S58165[74], 6}, {NULL}};
const Segment SGI195905[] = {{&S68142[34], 10}, {NULL}};
const Segment SGI195906[] = {{&S68142[27], 6}, {NULL}};
const Segment SGI195907[] = {{&S68142[97], 9}, {NULL}};
const Segment SGI195908[] = {{&S68142[70], 6}, {NULL}};
const Segment SGI195909[] = {{&S79041[34], 14}, {NULL}};
const Segment SGI195910[] = {{&S79041[27], 6}, {NULL}};
const Segment SGI195911[] = {{&S79041[101], 9}, {NULL}};
const Segment SGI195912[] = {{&S79041[74], 6}, {NULL}};
const Segment SGI195913[] = {{&S79044[44], 5}, {NULL}};
const Segment SGI195914[] = {{&S79044[44], 17}, {NULL}};
const Segment SGI195915[] = {{&S79044[30], 6}, {NULL}};
const Segment SGI195916[] = {{&S79044[75], 6}, {NULL}};
const Segment SGI195917[] = {{&S79044[102], 13}, {NULL}};
const Segment SGI195918[] = {{&S68150[34], 10}, {NULL}};
const Segment SGI195919[] = {{&S68150[27], 6}, {NULL}};
const Segment SGI195920[] = {{&S68150[97], 9}, {NULL}};
const Segment SGI195921[] = {{&S68150[70], 6}, {NULL}};
const Segment SGI195922[] = {{&S68151[34], 10}, {NULL}};
const Segment SGI195923[] = {{&S68151[27], 6}, {NULL}};
const Segment SGI195924[] = {{&S68151[97], 9}, {NULL}};
const Segment SGI195925[] = {{&S68151[70], 6}, {NULL}};
const Segment SGI195926[] = {{&S68152[34], 10}, {NULL}};
const Segment SGI195927[] = {{&S68152[27], 6}, {NULL}};
const Segment SGI195928[] = {{&S68152[97], 9}, {NULL}};
const Segment SGI195929[] = {{&S68152[70], 6}, {NULL}};
const Segment SGI195930[] = {{&S68153[34], 11}, {NULL}};
const Segment SGI195931[] = {{&S68153[27], 6}, {NULL}};
const Segment SGI195932[] = {{&S68153[98], 9}, {NULL}};
const Segment SGI195933[] = {{&S68153[71], 6}, {NULL}};
const Segment SGI195934[] = {{&S67319[34], 15}, {NULL}};
const Segment SGI195935[] = {{&S67319[27], 6}, {NULL}};
const Segment SGI195936[] = {{&S67319[102], 9}, {NULL}};
const Segment SGI195937[] = {{&S67319[75], 6}, {NULL}};
const Segment SGI195938[] = {{&S79071[34], 11}, {NULL}};
const Segment SGI195939[] = {{&S79071[27], 6}, {NULL}};
const Segment SGI195940[] = {{&S79071[98], 9}, {NULL}};
const Segment SGI195941[] = {{&S79071[71], 6}, {NULL}};
const Segment SGI195942[] = {{&S68179[34], 10}, {NULL}};
const Segment SGI195943[] = {{&S68179[27], 6}, {NULL}};
const Segment SGI195944[] = {{&S68179[97], 9}, {NULL}};
const Segment SGI195945[] = {{&S68179[70], 6}, {NULL}};
const Segment SGI195946[] = {{&S68180[34], 10}, {NULL}};
const Segment SGI195947[] = {{&S68180[27], 6}, {NULL}};
const Segment SGI195948[] = {{&S68180[97], 9}, {NULL}};
const Segment SGI195949[] = {{&S68180[70], 6}, {NULL}};
const Segment SGI195950[] = {{&S68182[34], 16}, {NULL}};
const Segment SGI195951[] = {{&S68182[27], 6}, {NULL}};
const Segment SGI195952[] = {{&S68182[110], 9}, {NULL}};
const Segment SGI195953[] = {{&S68182[83], 6}, {NULL}};
const Segment SGI195954[] = {{&S58230[34], 7}, {NULL}};
const Segment SGI195955[] = {{&S58230[27], 6}, {NULL}};
const Segment SGI195956[] = {{&S58230[93], 9}, {NULL}};
const Segment SGI195957[] = {{&S58230[66], 6}, {NULL}};
const Segment SGI195958[] = {{&S58231[34], 11}, {NULL}};
const Segment SGI195959[] = {{&S58231[27], 6}, {NULL}};
const Segment SGI195960[] = {{&S58231[98], 9}, {NULL}};
const Segment SGI195961[] = {{&S58231[71], 6}, {NULL}};
const Segment SGI195962[] = {{&S67355[44], 12}, {NULL}};
const Segment SGI195963[] = {{&S67355[44], 24}, {NULL}};
const Segment SGI195964[] = {{&S67355[30], 6}, {NULL}};
const Segment SGI195965[] = {{&S67355[135], 3}, {NULL}};
const Segment SGI195966[] = {{&S67355[100], 6}, {NULL}};
const Segment SGI195967[] = {{&S67355[70], 8}, {NULL}};
const Segment SGI195968[] = {{&S68200[44], 10}, {NULL}};
const Segment SGI195969[] = {{&S68200[44], 22}, {NULL}};
const Segment SGI195970[] = {{&S68200[30], 6}, {NULL}};
const Segment SGI195971[] = {{&S79105[44], 9}, {NULL}};
const Segment SGI195972[] = {{&S79105[44], 21}, {NULL}};
const Segment SGI195973[] = {{&S79105[30], 6}, {NULL}};
const Segment SGI195974[] = {{&S67356[44], 12}, {NULL}};
const Segment SGI195975[] = {{&S67356[44], 24}, {NULL}};
const Segment SGI195976[] = {{&S68200[115], 3}, {NULL}};
const Segment SGI195977[] = {{&S67356[30], 6}, {NULL}};
const Segment SGI195978[] = {{&S79105[31], 3}, {NULL}};
const Segment SGI195979[] = {{&S68200[80], 6}, {NULL}};
const Segment SGI195980[] = {{&S79105[79], 6}, {NULL}};
const Segment SGI195981[] = {{&S68200[140], 11}, {NULL}};
const Segment SGI195982[] = {{&S67356[135], 3}, {NULL}};
const Segment SGI195983[] = {{&S67356[100], 6}, {NULL}};
const Segment SGI195984[] = {{&S67356[70], 8}, {NULL}};
const Segment SGI195985[] = {{&S68213[44], 14}, {NULL}};
const Segment SGI195986[] = {{&S68213[44], 26}, {NULL}};
const Segment SGI195987[] = {{&S68213[30], 6}, {NULL}};
const Segment SGI195988[] = {{&S68213[119], 3}, {NULL}};
const Segment SGI195989[] = {{&S68213[84], 6}, {NULL}};
const Segment SGI195990[] = {{&S79120[34], 12}, {NULL}};
const Segment SGI195991[] = {{&S79120[27], 6}, {NULL}};
const Segment SGI195992[] = {{&S79120[99], 9}, {NULL}};
const Segment SGI195993[] = {{&S79120[72], 6}, {NULL}};
const Segment SGI195994[] = {{&S68214[44], 10}, {NULL}};
const Segment SGI195995[] = {{&S68214[44], 22}, {NULL}};
const Segment SGI195996[] = {{&S68214[30], 6}, {NULL}};
const Segment SGI195997[] = {{&S68214[115], 3}, {NULL}};
const Segment SGI195998[] = {{&S68214[80], 6}, {NULL}};
const Segment SGI195999[] = {{&S68215[44], 14}, {NULL}};
const Segment SGI196000[] = {{&S68215[44], 26}, {NULL}};
const Segment SGI196001[] = {{&S68215[30], 6}, {NULL}};
const Segment SGI196002[] = {{&S68215[119], 3}, {NULL}};
const Segment SGI196003[] = {{&S68215[84], 6}, {NULL}};
const Segment SGI196004[] = {{&S68216[38], 10}, {NULL}};
const Segment SGI196005[] = {{&S68216[31], 6}, {NULL}};
const Segment SGI196006[] = {{&S68217[34], 10}, {NULL}};
const Segment SGI196007[] = {{&S68217[27], 6}, {NULL}};
const Segment SGI196008[] = {{&S68217[97], 9}, {NULL}};
const Segment SGI196009[] = {{&S68217[70], 6}, {NULL}};
const Segment SGI196010[] = {{&S58279[34], 7}, {NULL}};
const Segment SGI196011[] = {{&S58279[27], 6}, {NULL}};
const Segment SGI196012[] = {{&S68232[34], 7}, {NULL}};
const Segment SGI196013[] = {{&S58279[102], 9}, {NULL}};
const Segment SGI196014[] = {{&S68232[27], 6}, {NULL}};
const Segment SGI196015[] = {{&S58279[75], 6}, {NULL}};
const Segment SGI196016[] = {{&S68232[105], 9}, {NULL}};
const Segment SGI196017[] = {{&S68232[78], 6}, {NULL}};
const Segment SGI196018[] = {{&S58280[34], 7}, {NULL}};
const Segment SGI196019[] = {{&S58280[27], 6}, {NULL}};
const Segment SGI196020[] = {{&S68233[34], 7}, {NULL}};
const Segment SGI196021[] = {{&S58280[102], 9}, {NULL}};
const Segment SGI196022[] = {{&S58280[75], 6}, {NULL}};
const Segment SGI196023[] = {{&S68233[27], 6}, {NULL}};
const Segment SGI196024[] = {{&S68233[105], 9}, {NULL}};
const Segment SGI196025[] = {{&S68233[78], 6}, {NULL}};
const Segment SGI196026[] = {{&S79144[44], 8}, {NULL}};
const Segment SGI196027[] = {{&S79144[44], 20}, {NULL}};
const Segment SGI196028[] = {{&S79144[30], 6}, {NULL}};
const Segment SGI196029[] = {{&S79144[31], 3}, {NULL}};
const Segment SGI196030[] = {{&S79144[78], 6}, {NULL}};
const Segment SGI196031[] = {{&S68236[34], 9}, {NULL}};
const Segment SGI196032[] = {{&S68236[27], 6}, {NULL}};
const Segment SGI196033[] = {{&S68236[96], 9}, {NULL}};
const Segment SGI196034[] = {{&S68236[69], 6}, {NULL}};
const Segment SGI196035[] = {{&S79151[34], 6}, {NULL}};
const Segment SGI196036[] = {{&S79151[27], 6}, {NULL}};
const Segment SGI196037[] = {{&S79151[93], 9}, {NULL}};
const Segment SGI196038[] = {{&S79151[66], 6}, {NULL}};
const Segment SGI196039[] = {{&S79152[44], 11}, {NULL}};
const Segment SGI196040[] = {{&S79152[44], 23}, {NULL}};
const Segment SGI196041[] = {{&S79152[30], 6}, {NULL}};
const Segment SGI196042[] = {{&S79152[31], 3}, {NULL}};
const Segment SGI196043[] = {{&S79152[81], 6}, {NULL}};
const Segment SGI196044[] = {{&S58294[44], 25}, {NULL}};
const Segment SGI196045[] = {{&S58294[44], 37}, {NULL}};
const Segment SGI196046[] = {{&S58294[30], 6}, {NULL}};
const Segment SGI196047[] = {{&S58294[148], 3}, {NULL}};
const Segment SGI196048[] = {{&S58294[113], 6}, {NULL}};
const Segment SGI196049[] = {{&S58294[83], 8}, {NULL}};
const Segment SGI196050[] = {{&S79153[34], 8}, {NULL}};
const Segment SGI196051[] = {{&S79153[27], 6}, {NULL}};
const Segment SGI196052[] = {{&S79153[94], 9}, {NULL}};
const Segment SGI196053[] = {{&S79153[67], 6}, {NULL}};
const Segment SGI196054[] = {{&S58295[44], 25}, {NULL}};
const Segment SGI196055[] = {{&S58295[44], 37}, {NULL}};
const Segment SGI196056[] = {{&S58295[30], 6}, {NULL}};
const Segment SGI196057[] = {{&S58295[148], 3}, {NULL}};
const Segment SGI196058[] = {{&S58295[113], 6}, {NULL}};
const Segment SGI196059[] = {{&S58295[83], 8}, {NULL}};
const Segment SGI196060[] = {{&S79154[44], 12}, {NULL}};
const Segment SGI196061[] = {{&S79154[44], 24}, {NULL}};
const Segment SGI196062[] = {{&S79154[30], 6}, {NULL}};
const Segment SGI196063[] = {{&S79154[31], 3}, {NULL}};
const Segment SGI196064[] = {{&S79154[82], 6}, {NULL}};
const Segment SGI196065[] = {{&S58296[44], 12}, {NULL}};
const Segment SGI196066[] = {{&S58296[44], 24}, {NULL}};
const Segment SGI196067[] = {{&S58296[30], 6}, {NULL}};
const Segment SGI196068[] = {{&S58296[117], 3}, {NULL}};
const Segment SGI196069[] = {{&S58296[82], 6}, {NULL}};
const Segment SGI196070[] = {{&S58296[135], 8}, {NULL}};
const Segment SGI196071[] = {{&S79156[34], 9}, {NULL}};
const Segment SGI196072[] = {{&S79156[27], 6}, {NULL}};
const Segment SGI196073[] = {{&S79156[96], 9}, {NULL}};
const Segment SGI196074[] = {{&S79156[69], 6}, {NULL}};
const Segment SGI196075[] = {{&S79162[34], 10}, {NULL}};
const Segment SGI196076[] = {{&S79162[27], 6}, {NULL}};
const Segment SGI196077[] = {{&S79162[97], 9}, {NULL}};
const Segment SGI196078[] = {{&S79162[70], 6}, {NULL}};
const Segment SGI196079[] = {{&S79164[34], 7}, {NULL}};
const Segment SGI196080[] = {{&S79164[27], 6}, {NULL}};
const Segment SGI196081[] = {{&S79164[98], 9}, {NULL}};
const Segment SGI196082[] = {{&S79164[71], 6}, {NULL}};
const Segment SGI196083[] = {{&S79164[139], 8}, {NULL}};
const Segment SGI196084[] = {{&S79165[34], 9}, {NULL}};
const Segment SGI196085[] = {{&S79165[27], 6}, {NULL}};
const Segment SGI196086[] = {{&S79165[96], 9}, {NULL}};
const Segment SGI196087[] = {{&S79165[69], 6}, {NULL}};
const Segment SGI196088[] = {{&S79166[34], 11}, {NULL}};
const Segment SGI196089[] = {{&S79166[27], 6}, {NULL}};
const Segment SGI196090[] = {{&S79166[98], 9}, {NULL}};
const Segment SGI196091[] = {{&S79166[71], 6}, {NULL}};
const Segment SGI196092[] = {{&S79167[34], 10}, {NULL}};
const Segment SGI196093[] = {{&S79167[27], 6}, {NULL}};
const Segment SGI196094[] = {{&S79167[97], 9}, {NULL}};
const Segment SGI196095[] = {{&S79167[70], 6}, {NULL}};
const Segment SGI196096[] = {{&S79168[34], 15}, {NULL}};
const Segment SGI196097[] = {{&S79168[27], 6}, {NULL}};
const Segment SGI196098[] = {{&S79168[102], 9}, {NULL}};
const Segment SGI196099[] = {{&S79168[75], 6}, {NULL}};
const Segment SGI196100[] = {{&S79168[143], 8}, {NULL}};
const Segment SGI196101[] = {{&S79171[34], 16}, {NULL}};
const Segment SGI196102[] = {{&S79171[27], 6}, {NULL}};
const Segment SGI196103[] = {{&S79171[103], 9}, {NULL}};
const Segment SGI196104[] = {{&S79171[76], 6}, {NULL}};
const Segment SGI196105[] = {{&S79172[34], 18}, {NULL}};
const Segment SGI196106[] = {{&S79172[27], 6}, {NULL}};
const Segment SGI196107[] = {{&S79172[105], 9}, {NULL}};
const Segment SGI196108[] = {{&S79172[78], 6}, {NULL}};
const Segment SGI196109[] = {{&S79173[34], 16}, {NULL}};
const Segment SGI196110[] = {{&S79173[27], 6}, {NULL}};
const Segment SGI196111[] = {{&S79173[103], 9}, {NULL}};
const Segment SGI196112[] = {{&S79173[76], 6}, {NULL}};
const Segment SGI196113[] = {{&S79174[34], 19}, {NULL}};
const Segment SGI196114[] = {{&S79174[27], 6}, {NULL}};
const Segment SGI196115[] = {{&S79174[106], 9}, {NULL}};
const Segment SGI196116[] = {{&S79174[79], 6}, {NULL}};
const Segment SGI196117[] = {{&S68288[44], 9}, {NULL}};
const Segment SGI196118[] = {{&S68288[44], 21}, {NULL}};
const Segment SGI196119[] = {{&S68288[30], 6}, {NULL}};
const Segment SGI196120[] = {{&S68288[114], 3}, {NULL}};
const Segment SGI196121[] = {{&S68288[79], 6}, {NULL}};
const Segment SGI196122[] = {{&S68288[142], 3}, {NULL}};
const Segment SGI196123[] = {{&S73041[44], 7}, {NULL}};
const Segment SGI196124[] = {{&S73041[44], 24}, {NULL}};
const Segment SGI196125[] = {{&S73041[30], 6}, {NULL}};
const Segment SGI196126[] = {{&S73041[117], 3}, {NULL}};
const Segment SGI196127[] = {{&S73041[82], 6}, {NULL}};
const Segment SGI196128[] = {{&S79207[44], 17}, {NULL}};
const Segment SGI196129[] = {{&S79207[44], 29}, {NULL}};
const Segment SGI196130[] = {{&S79207[30], 6}, {NULL}};
const Segment SGI196131[] = {{&S79207[31], 3}, {NULL}};
const Segment SGI196132[] = {{&S79207[87], 6}, {NULL}};
const Segment SGI196133[] = {{&S79208[59], 5}, {NULL}};
const Segment SGI196134[] = {{&S79208[59], 17}, {NULL}};
const Segment SGI196135[] = {{&S79208[45], 6}, {NULL}};
const Segment SGI196136[] = {{&S79208[46], 3}, {NULL}};
const Segment SGI196137[] = {{&S79208[90], 6}, {NULL}};
const Segment SGI196138[] = {{&S79208[0], 14}, {NULL}};
const Segment SGI196139[] = {{&S79211[34], 11}, {NULL}};
const Segment SGI196140[] = {{&S73063[34], 7}, {NULL}};
const Segment SGI196141[] = {{&S79211[27], 6}, {NULL}};
const Segment SGI196142[] = {{&S73063[27], 6}, {NULL}};
const Segment SGI196143[] = {{&S79211[98], 9}, {NULL}};
const Segment SGI196144[] = {{&S79211[71], 6}, {NULL}};
const Segment SGI196145[] = {{&S73063[94], 9}, {NULL}};
const Segment SGI196146[] = {{&S73063[67], 6}, {NULL}};
const Segment SGI196147[] = {{&S68328[34], 10}, {NULL}};
const Segment SGI196148[] = {{&S68328[27], 6}, {NULL}};
const Segment SGI196149[] = {{&S68328[97], 9}, {NULL}};
const Segment SGI196150[] = {{&S68328[70], 6}, {NULL}};
const Segment SGI196151[] = {{&S79212[34], 10}, {NULL}};
const Segment SGI196152[] = {{&S79212[27], 6}, {NULL}};
const Segment SGI196153[] = {{&S79212[97], 9}, {NULL}};
const Segment SGI196154[] = {{&S79212[70], 6}, {NULL}};
const Segment SGI196155[] = {{&S73065[44], 12}, {NULL}};
const Segment SGI196156[] = {{&S73065[44], 24}, {NULL}};
const Segment SGI196157[] = {{&S73065[30], 6}, {NULL}};
const Segment SGI196158[] = {{&S73065[117], 3}, {NULL}};
const Segment SGI196159[] = {{&S73065[82], 6}, {NULL}};
const Segment SGI196160[] = {{&S79213[34], 7}, {NULL}};
const Segment SGI196161[] = {{&S79213[27], 6}, {NULL}};
const Segment SGI196162[] = {{&S79213[104], 9}, {NULL}};
const Segment SGI196163[] = {{&S79213[77], 6}, {NULL}};
const Segment SGI196164[] = {{&S79214[34], 9}, {NULL}};
const Segment SGI196165[] = {{&S79214[27], 6}, {NULL}};
const Segment SGI196166[] = {{&S79214[96], 9}, {NULL}};
const Segment SGI196167[] = {{&S79214[69], 6}, {NULL}};
const Segment SGI196168[] = {{&S79216[44], 9}, {NULL}};
const Segment SGI196169[] = {{&S79216[44], 21}, {NULL}};
const Segment SGI196170[] = {{&S79216[30], 6}, {NULL}};
const Segment SGI196171[] = {{&S79216[31], 3}, {NULL}};
const Segment SGI196172[] = {{&S79216[79], 6}, {NULL}};
const Segment SGI196173[] = {{&S79217[34], 7}, {NULL}};
const Segment SGI196174[] = {{&S79217[27], 6}, {NULL}};
const Segment SGI196175[] = {{&S79217[99], 9}, {NULL}};
const Segment SGI196176[] = {{&S79217[72], 6}, {NULL}};
const Segment SGI196177[] = {{&S67403[44], 15}, {NULL}};
const Segment SGI196178[] = {{&S67403[44], 27}, {NULL}};
const Segment SGI196179[] = {{&S67403[30], 6}, {NULL}};
const Segment SGI196180[] = {{&S67403[120], 3}, {NULL}};
const Segment SGI196181[] = {{&S67403[85], 6}, {NULL}};
const Segment SGI196182[] = {{&S67404[38], 6}, {NULL}};
const Segment SGI196183[] = {{&S67404[31], 6}, {NULL}};
const Segment SGI196184[] = {{&S73088[44], 19}, {NULL}};
const Segment SGI196185[] = {{&S73088[44], 32}, {NULL}};
const Segment SGI196186[] = {{&S73088[30], 6}, {NULL}};
const Segment SGI196187[] = {{&S73088[125], 3}, {NULL}};
const Segment SGI196188[] = {{&S73088[90], 6}, {NULL}};
const Segment SGI196189[] = {{&S58412[44], 15}, {NULL}};
const Segment SGI196190[] = {{&S58412[44], 26}, {NULL}};
const Segment SGI196191[] = {{&S58412[30], 6}, {NULL}};
const Segment SGI196192[] = {{&S58412[119], 3}, {NULL}};
const Segment SGI196193[] = {{&S58412[84], 6}, {NULL}};
const Segment SGI196194[] = {{&S58413[38], 15}, {NULL}};
const Segment SGI196195[] = {{&S58413[31], 6}, {NULL}};
const Segment SGI196196[] = {{&S58414[44], 15}, {NULL}};
const Segment SGI196197[] = {{&S58414[44], 26}, {NULL}};
const Segment SGI196198[] = {{&S58414[30], 6}, {NULL}};
const Segment SGI196199[] = {{&S58414[119], 3}, {NULL}};
const Segment SGI196200[] = {{&S58414[84], 6}, {NULL}};
const Segment SGI196201[] = {{&S79245[44], 25}, {NULL}};
const Segment SGI196202[] = {{&S79245[44], 37}, {NULL}};
const Segment SGI196203[] = {{&S79245[30], 6}, {NULL}};
const Segment SGI196204[] = {{&S79245[31], 3}, {NULL}};
const Segment SGI196205[] = {{&S58415[44], 11}, {NULL}};
const Segment SGI196206[] = {{&S58415[44], 22}, {NULL}};
const Segment SGI196207[] = {{&S79245[95], 6}, {NULL}};
const Segment SGI196208[] = {{&S58415[30], 6}, {NULL}};
const Segment SGI196209[] = {{&S58415[115], 3}, {NULL}};
const Segment SGI196210[] = {{&S58415[80], 6}, {NULL}};
const Segment SGI196211[] = {{&S79246[42], 25}, {NULL}};
const Segment SGI196212[] = {{&S79246[30], 6}, {NULL}};
const Segment SGI196213[] = {{&S79246[31], 3}, {NULL}};
const Segment SGI196214[] = {{&S58416[44], 11}, {NULL}};
const Segment SGI196215[] = {{&S58416[44], 22}, {NULL}};
const Segment SGI196216[] = {{&S79246[93], 6}, {NULL}};
const Segment SGI196217[] = {{&S58416[30], 6}, {NULL}};
const Segment SGI196218[] = {{&S58416[115], 3}, {NULL}};
const Segment SGI196219[] = {{&S58416[80], 6}, {NULL}};
const Segment SGI196220[] = {{&S68370[44], 11}, {NULL}};
const Segment SGI196221[] = {{&S68370[44], 23}, {NULL}};
const Segment SGI196222[] = {{&S68370[30], 6}, {NULL}};
const Segment SGI196223[] = {{&S68370[116], 3}, {NULL}};
const Segment SGI196224[] = {{&S68370[81], 6}, {NULL}};
const Segment SGI196225[] = {{&S68371[38], 11}, {NULL}};
const Segment SGI196226[] = {{&S68371[31], 6}, {NULL}};
const Segment SGI196227[] = {{&S73111[44], 14}, {NULL}};
const Segment SGI196228[] = {{&S73111[44], 26}, {NULL}};
const Segment SGI196229[] = {{&S73111[30], 6}, {NULL}};
const Segment SGI196230[] = {{&S73111[135], 3}, {NULL}};
const Segment SGI196231[] = {{&S68376[34], 10}, {NULL}};
const Segment SGI196232[] = {{&S73111[100], 6}, {NULL}};
const Segment SGI196233[] = {{&S68376[27], 6}, {NULL}};
const Segment SGI196234[] = {{&S73111[72], 8}, {NULL}};
const Segment SGI196235[] = {{&S68376[97], 9}, {NULL}};
const Segment SGI196236[] = {{&S68376[70], 6}, {NULL}};
const Segment SGI196237[] = {{&S73112[44], 14}, {NULL}};
const Segment SGI196238[] = {{&S73112[44], 26}, {NULL}};
const Segment SGI196239[] = {{&S73112[30], 6}, {NULL}};
const Segment SGI196240[] = {{&S68377[34], 10}, {NULL}};
const Segment SGI196241[] = {{&S68377[27], 6}, {NULL}};
const Segment SGI196242[] = {{&S73112[135], 3}, {NULL}};
const Segment SGI196243[] = {{&S73112[100], 6}, {NULL}};
const Segment SGI196244[] = {{&S73112[72], 8}, {NULL}};
const Segment SGI196245[] = {{&S68377[97], 9}, {NULL}};
const Segment SGI196246[] = {{&S68377[70], 6}, {NULL}};
const Segment SGI196247[] = {{&S73114[44], 14}, {NULL}};
const Segment SGI196248[] = {{&S73114[44], 26}, {NULL}};
const Segment SGI196249[] = {{&S73114[30], 6}, {NULL}};
const Segment SGI196250[] = {{&S73114[135], 3}, {NULL}};
const Segment SGI196251[] = {{&S73114[100], 6}, {NULL}};
const Segment SGI196252[] = {{&S73114[72], 8}, {NULL}};
const Segment SGI196253[] = {{&S79263[44], 7}, {NULL}};
const Segment SGI196254[] = {{&S79263[44], 24}, {NULL}};
const Segment SGI196255[] = {{&S79263[30], 6}, {NULL}};
const Segment SGI196256[] = {{&S79263[82], 6}, {NULL}};
const Segment SGI196257[] = {{&S79263[109], 13}, {NULL}};
const Segment SGI196258[] = {{&S73117[44], 7}, {NULL}};
const Segment SGI196259[] = {{&S73117[44], 19}, {NULL}};
const Segment SGI196260[] = {{&S73117[30], 6}, {NULL}};
const Segment SGI196261[] = {{&S79264[34], 11}, {NULL}};
const Segment SGI196262[] = {{&S79264[27], 6}, {NULL}};
const Segment SGI196263[] = {{&S73117[112], 3}, {NULL}};
const Segment SGI196264[] = {{&S73117[77], 6}, {NULL}};
const Segment SGI196265[] = {{&S79264[98], 9}, {NULL}};
const Segment SGI196266[] = {{&S79264[71], 6}, {NULL}};
const Segment SGI196267[] = {{&S79265[34], 10}, {NULL}};
const Segment SGI196268[] = {{&S79265[27], 6}, {NULL}};
const Segment SGI196269[] = {{&S79265[97], 9}, {NULL}};
const Segment SGI196270[] = {{&S79265[70], 6}, {NULL}};
const Segment SGI196271[] = {{&S79266[34], 11}, {NULL}};
const Segment SGI196272[] = {{&S79266[27], 6}, {NULL}};
const Segment SGI196273[] = {{&S58441[34], 14}, {NULL}};
const Segment SGI196274[] = {{&S58441[27], 6}, {NULL}};
const Segment SGI196275[] = {{&S79266[98], 9}, {NULL}};
const Segment SGI196276[] = {{&S79266[71], 6}, {NULL}};
const Segment SGI196277[] = {{&S58441[100], 9}, {NULL}};
const Segment SGI196278[] = {{&S58441[73], 6}, {NULL}};
const Segment SGI196279[] = {{&S58441[142], 8}, {NULL}};
const Segment SGI196280[] = {{&S73122[34], 9}, {NULL}};
const Segment SGI196281[] = {{&S73122[27], 6}, {NULL}};
const Segment SGI196282[] = {{&S73122[96], 9}, {NULL}};
const Segment SGI196283[] = {{&S73122[69], 6}, {NULL}};
const Segment SGI196284[] = {{&S58442[34], 14}, {NULL}};
const Segment SGI196285[] = {{&S58442[27], 6}, {NULL}};
const Segment SGI196286[] = {{&S58442[100], 9}, {NULL}};
const Segment SGI196287[] = {{&S58442[73], 6}, {NULL}};
const Segment SGI196288[] = {{&S58442[141], 8}, {NULL}};
const Segment SGI196289[] = {{&S73123[34], 9}, {NULL}};
const Segment SGI196290[] = {{&S73123[27], 6}, {NULL}};
const Segment SGI196291[] = {{&S73123[96], 9}, {NULL}};
const Segment SGI196292[] = {{&S73123[69], 6}, {NULL}};
const Segment SGI196293[] = {{&S58446[34], 10}, {NULL}};
const Segment SGI196294[] = {{&S58446[27], 6}, {NULL}};
const Segment SGI196295[] = {{&S58446[96], 9}, {NULL}};
const Segment SGI196296[] = {{&S58446[69], 6}, {NULL}};
const Segment SGI196297[] = {{&S68397[34], 7}, {NULL}};
const Segment SGI196298[] = {{&S58447[34], 10}, {NULL}};
const Segment SGI196299[] = {{&S68397[27], 6}, {NULL}};
const Segment SGI196300[] = {{&S58447[27], 6}, {NULL}};
const Segment SGI196301[] = {{&S58447[96], 9}, {NULL}};
const Segment SGI196302[] = {{&S68397[99], 9}, {NULL}};
const Segment SGI196303[] = {{&S58447[69], 6}, {NULL}};
const Segment SGI196304[] = {{&S68397[72], 6}, {NULL}};
const Segment SGI196305[] = {{&S73128[44], 11}, {NULL}};
const Segment SGI196306[] = {{&S73128[44], 23}, {NULL}};
const Segment SGI196307[] = {{&S73128[30], 6}, {NULL}};
const Segment SGI196308[] = {{&S73128[116], 3}, {NULL}};
const Segment SGI196309[] = {{&S73128[81], 6}, {NULL}};
const Segment SGI196310[] = {{&S68398[44], 10}, {NULL}};
const Segment SGI196311[] = {{&S68398[44], 22}, {NULL}};
const Segment SGI196312[] = {{&S68398[30], 6}, {NULL}};
const Segment SGI196313[] = {{&S68398[80], 6}, {NULL}};
const Segment SGI196314[] = {{&S68398[107], 13}, {NULL}};
const Segment SGI196315[] = {{&S68399[44], 11}, {NULL}};
const Segment SGI196316[] = {{&S68399[44], 23}, {NULL}};
const Segment SGI196317[] = {{&S68399[30], 6}, {NULL}};
const Segment SGI196318[] = {{&S68399[81], 6}, {NULL}};
const Segment SGI196319[] = {{&S68399[108], 13}, {NULL}};
const Segment SGI196320[] = {{&S68402[34], 10}, {NULL}};
const Segment SGI196321[] = {{&S68402[27], 6}, {NULL}};
const Segment SGI196322[] = {{&S68402[97], 9}, {NULL}};
const Segment SGI196323[] = {{&S68402[70], 6}, {NULL}};
const Segment SGI196324[] = {{&S68403[34], 10}, {NULL}};
const Segment SGI196325[] = {{&S68403[27], 6}, {NULL}};
const Segment SGI196326[] = {{&S79287[34], 8}, {NULL}};
const Segment SGI196327[] = {{&S79287[27], 6}, {NULL}};
const Segment SGI196328[] = {{&S68403[97], 9}, {NULL}};
const Segment SGI196329[] = {{&S68403[70], 6}, {NULL}};
const Segment SGI196330[] = {{&S79287[95], 9}, {NULL}};
const Segment SGI196331[] = {{&S79287[68], 6}, {NULL}};
const Segment SGI196332[] = {{&S68404[34], 10}, {NULL}};
const Segment SGI196333[] = {{&S68404[27], 6}, {NULL}};
const Segment SGI196334[] = {{&S68404[97], 9}, {NULL}};
const Segment SGI196335[] = {{&S68404[70], 6}, {NULL}};
const Segment SGI196336[] = {{&S68405[34], 14}, {NULL}};
const Segment SGI196337[] = {{&S68405[27], 6}, {NULL}};
const Segment SGI196338[] = {{&S68405[101], 9}, {NULL}};
const Segment SGI196339[] = {{&S68405[74], 6}, {NULL}};
const Segment SGI196340[] = {{&S79291[34], 10}, {NULL}};
const Segment SGI196341[] = {{&S79291[27], 6}, {NULL}};
const Segment SGI196342[] = {{&S79291[97], 9}, {NULL}};
const Segment SGI196343[] = {{&S79291[70], 6}, {NULL}};
const Segment SGI196344[] = {{&S79295[34], 7}, {NULL}};
const Segment SGI196345[] = {{&S79295[27], 6}, {NULL}};
const Segment SGI196346[] = {{&S79295[99], 9}, {NULL}};
const Segment SGI196347[] = {{&S79295[72], 6}, {NULL}};
const Segment SGI196348[] = {{&S79307[44], 9}, {NULL}};
const Segment SGI196349[] = {{&S79307[44], 21}, {NULL}};
const Segment SGI196350[] = {{&S79307[30], 6}, {NULL}};
const Segment SGI196351[] = {{&S79307[31], 3}, {NULL}};
const Segment SGI196352[] = {{&S79307[94], 6}, {NULL}};
const Segment SGI196353[] = {{&S79307[67], 8}, {NULL}};
const Segment SGI196354[] = {{&S79308[44], 9}, {NULL}};
const Segment SGI196355[] = {{&S79308[44], 21}, {NULL}};
const Segment SGI196356[] = {{&S79308[30], 6}, {NULL}};
const Segment SGI196357[] = {{&S79308[31], 3}, {NULL}};
const Segment SGI196358[] = {{&S79308[94], 6}, {NULL}};
const Segment SGI196359[] = {{&S79308[67], 8}, {NULL}};
const Segment SGI196360[] = {{&S68428[34], 6}, {NULL}};
const Segment SGI196361[] = {{&S68428[27], 6}, {NULL}};
const Segment SGI196362[] = {{&S68428[93], 9}, {NULL}};
const Segment SGI196363[] = {{&S79309[44], 9}, {NULL}};
const Segment SGI196364[] = {{&S68428[66], 6}, {NULL}};
const Segment SGI196365[] = {{&S79309[44], 21}, {NULL}};
const Segment SGI196366[] = {{&S79309[30], 6}, {NULL}};
const Segment SGI196367[] = {{&S58489[44], 14}, {NULL}};
const Segment SGI196368[] = {{&S58489[44], 25}, {NULL}};
const Segment SGI196369[] = {{&S79309[31], 3}, {NULL}};
const Segment SGI196370[] = {{&S58489[30], 6}, {NULL}};
const Segment SGI196371[] = {{&S79309[94], 6}, {NULL}};
const Segment SGI196372[] = {{&S79309[67], 8}, {NULL}};
const Segment SGI196373[] = {{&S58489[147], 3}, {NULL}};
const Segment SGI196374[] = {{&S58489[112], 6}, {NULL}};
const Segment SGI196375[] = {{&S58490[44], 14}, {NULL}};
const Segment SGI196376[] = {{&S58490[44], 26}, {NULL}};
const Segment SGI196377[] = {{&S58490[30], 6}, {NULL}};
const Segment SGI196378[] = {{&S58490[119], 3}, {NULL}};
const Segment SGI196379[] = {{&S58490[84], 6}, {NULL}};
const Segment SGI196380[] = {{&S73172[34], 25}, {NULL}};
const Segment SGI196381[] = {{&S73172[27], 6}, {NULL}};
const Segment SGI196382[] = {{&S73172[112], 9}, {NULL}};
const Segment SGI196383[] = {{&S73172[85], 6}, {NULL}};
const Segment SGI196384[] = {{&S79320[34], 8}, {NULL}};
const Segment SGI196385[] = {{&S79320[27], 6}, {NULL}};
const Segment SGI196386[] = {{&S79320[100], 9}, {NULL}};
const Segment SGI196387[] = {{&S79320[73], 6}, {NULL}};
const Segment SGI196388[] = {{&S79321[34], 15}, {NULL}};
const Segment SGI196389[] = {{&S79321[27], 6}, {NULL}};
const Segment SGI196390[] = {{&S79321[102], 9}, {NULL}};
const Segment SGI196391[] = {{&S79321[75], 6}, {NULL}};
const Segment SGI196392[] = {{&S68444[34], 8}, {NULL}};
const Segment SGI196393[] = {{&S68444[27], 6}, {NULL}};
const Segment SGI196394[] = {{&S68444[100], 9}, {NULL}};
const Segment SGI196395[] = {{&S68444[73], 6}, {NULL}};
const Segment SGI196396[] = {{&S79322[34], 8}, {NULL}};
const Segment SGI196397[] = {{&S79322[27], 6}, {NULL}};
const Segment SGI196398[] = {{&S79322[95], 9}, {NULL}};
const Segment SGI196399[] = {{&S79322[68], 6}, {NULL}};
const Segment SGI196400[] = {{&S68445[34], 14}, {NULL}};
const Segment SGI196401[] = {{&S68445[27], 6}, {NULL}};
const Segment SGI196402[] = {{&S68445[101], 9}, {NULL}};
const Segment SGI196403[] = {{&S68445[74], 6}, {NULL}};
const Segment SGI196404[] = {{&S68446[34], 11}, {NULL}};
const Segment SGI196405[] = {{&S68446[27], 6}, {NULL}};
const Segment SGI196406[] = {{&S68446[98], 9}, {NULL}};
const Segment SGI196407[] = {{&S68446[71], 6}, {NULL}};
const Segment SGI196408[] = {{&S79328[44], 9}, {NULL}};
const Segment SGI196409[] = {{&S79328[44], 21}, {NULL}};
const Segment SGI196410[] = {{&S79328[30], 6}, {NULL}};
const Segment SGI196411[] = {{&S79328[31], 3}, {NULL}};
const Segment SGI196412[] = {{&S79328[79], 6}, {NULL}};
const Segment SGI196413[] = {{&S79330[34], 11}, {NULL}};
const Segment SGI196414[] = {{&S79330[27], 6}, {NULL}};
const Segment SGI196415[] = {{&S79330[98], 9}, {NULL}};
const Segment SGI196416[] = {{&S79330[71], 6}, {NULL}};
const Segment SGI196417[] = {{&S79331[34], 11}, {NULL}};
const Segment SGI196418[] = {{&S79331[27], 6}, {NULL}};
const Segment SGI196419[] = {{&S79331[98], 9}, {NULL}};
const Segment SGI196420[] = {{&S79331[71], 6}, {NULL}};
const Segment SGI196421[] = {{&S79332[34], 11}, {NULL}};
const Segment SGI196422[] = {{&S79332[27], 6}, {NULL}};
const Segment SGI196423[] = {{&S79332[98], 9}, {NULL}};
const Segment SGI196424[] = {{&S79332[71], 6}, {NULL}};
const Segment SGI196425[] = {{&S79333[44], 15}, {NULL}};
const Segment SGI196426[] = {{&S79333[44], 27}, {NULL}};
const Segment SGI196427[] = {{&S79333[30], 6}, {NULL}};
const Segment SGI196428[] = {{&S58520[44], 10}, {NULL}};
const Segment SGI196429[] = {{&S58520[44], 21}, {NULL}};
const Segment SGI196430[] = {{&S58520[30], 6}, {NULL}};
const Segment SGI196431[] = {{&S79333[31], 3}, {NULL}};
const Segment SGI196432[] = {{&S79333[85], 6}, {NULL}};
const Segment SGI196433[] = {{&S58520[132], 3}, {NULL}};
const Segment SGI196434[] = {{&S79333[145], 13}, {NULL}};
const Segment SGI196435[] = {{&S58520[97], 6}, {NULL}};
const Segment SGI196436[] = {{&S58520[67], 8}, {NULL}};
const Segment SGI196437[] = {{&S58522[44], 10}, {NULL}};
const Segment SGI196438[] = {{&S58522[44], 22}, {NULL}};
const Segment SGI196439[] = {{&S58522[30], 6}, {NULL}};
const Segment SGI196440[] = {{&S58522[115], 3}, {NULL}};
const Segment SGI196441[] = {{&S58522[80], 6}, {NULL}};
const Segment SGI196442[] = {{&S58522[133], 8}, {NULL}};
const Segment SGI196443[] = {{&S79351[44], 18}, {NULL}};
const Segment SGI196444[] = {{&S79351[44], 30}, {NULL}};
const Segment SGI196445[] = {{&S79351[30], 6}, {NULL}};
const Segment SGI196446[] = {{&S67552[34], 18}, {NULL}};
const Segment SGI196447[] = {{&S67552[27], 6}, {NULL}};
const Segment SGI196448[] = {{&S79351[88], 6}, {NULL}};
const Segment SGI196449[] = {{&S79351[115], 13}, {NULL}};
const Segment SGI196450[] = {{&S67552[105], 9}, {NULL}};
const Segment SGI196451[] = {{&S67552[78], 6}, {NULL}};
const Segment SGI196452[] = {{&S67555[34], 10}, {NULL}};
const Segment SGI196453[] = {{&S67555[27], 6}, {NULL}};
const Segment SGI196454[] = {{&S67555[97], 9}, {NULL}};
const Segment SGI196455[] = {{&S67555[70], 6}, {NULL}};
const Segment SGI196456[] = {{&S73238[44], 8}, {NULL}};
const Segment SGI196457[] = {{&S73238[44], 20}, {NULL}};
const Segment SGI196458[] = {{&S73238[30], 6}, {NULL}};
const Segment SGI196459[] = {{&S73238[113], 3}, {NULL}};
const Segment SGI196460[] = {{&S73238[78], 6}, {NULL}};
const Segment SGI196461[] = {{&S79373[44], 4}, {NULL}};
const Segment SGI196462[] = {{&S79373[44], 16}, {NULL}};
const Segment SGI196463[] = {{&S79373[30], 6}, {NULL}};
const Segment SGI196464[] = {{&S79373[31], 3}, {NULL}};
const Segment SGI196465[] = {{&S79373[74], 6}, {NULL}};
const Segment SGI196466[] = {{&S79374[44], 9}, {NULL}};
const Segment SGI196467[] = {{&S79374[44], 21}, {NULL}};
const Segment SGI196468[] = {{&S79374[30], 6}, {NULL}};
const Segment SGI196469[] = {{&S79374[31], 3}, {NULL}};
const Segment SGI196470[] = {{&S79374[79], 6}, {NULL}};
const Segment SGI196471[] = {{&S79375[34], 8}, {NULL}};
const Segment SGI196472[] = {{&S79375[27], 6}, {NULL}};
const Segment SGI196473[] = {{&S79375[95], 9}, {NULL}};
const Segment SGI196474[] = {{&S79375[68], 6}, {NULL}};
const Segment SGI196475[] = {{&S79377[44], 13}, {NULL}};
const Segment SGI196476[] = {{&S79377[44], 25}, {NULL}};
const Segment SGI196477[] = {{&S79377[30], 6}, {NULL}};
const Segment SGI196478[] = {{&S79377[31], 3}, {NULL}};
const Segment SGI196479[] = {{&S79377[83], 6}, {NULL}};
const Segment SGI196480[] = {{&S68517[52], 10}, {NULL}};
const Segment SGI196481[] = {{&S68517[52], 22}, {NULL}};
const Segment SGI196482[] = {{&S68517[38], 6}, {NULL}};
const Segment SGI196483[] = {{&S68517[88], 5}, {NULL}};
const Segment SGI196484[] = {{&S68517[11], 3}, {NULL}};
const Segment SGI196485[] = {{&S68518[52], 10}, {NULL}};
const Segment SGI196486[] = {{&S79381[44], 14}, {NULL}};
const Segment SGI196487[] = {{&S68518[52], 22}, {NULL}};
const Segment SGI196488[] = {{&S79381[44], 26}, {NULL}};
const Segment SGI196489[] = {{&S68518[38], 6}, {NULL}};
const Segment SGI196490[] = {{&S79381[30], 6}, {NULL}};
const Segment SGI196491[] = {{&S79381[31], 3}, {NULL}};
const Segment SGI196492[] = {{&S68518[88], 5}, {NULL}};
const Segment SGI196493[] = {{&S79381[84], 6}, {NULL}};
const Segment SGI196494[] = {{&S68518[11], 3}, {NULL}};
const Segment SGI196495[] = {{&S68519[52], 10}, {NULL}};
const Segment SGI196496[] = {{&S68519[52], 22}, {NULL}};
const Segment SGI196497[] = {{&S79382[44], 14}, {NULL}};
const Segment SGI196498[] = {{&S68519[38], 6}, {NULL}};
const Segment SGI196499[] = {{&S79382[44], 26}, {NULL}};
const Segment SGI196500[] = {{&S79382[30], 6}, {NULL}};
const Segment SGI196501[] = {{&S68519[88], 5}, {NULL}};
const Segment SGI196502[] = {{&S79382[87], 6}, {NULL}};
const Segment SGI196503[] = {{&S68519[11], 3}, {NULL}};
const Segment SGI196504[] = {{&S79383[38], 14}, {NULL}};
const Segment SGI196505[] = {{&S79383[31], 6}, {NULL}};
const Segment SGI196506[] = {{&S79385[34], 6}, {NULL}};
const Segment SGI196507[] = {{&S79385[27], 6}, {NULL}};
const Segment SGI196508[] = {{&S67591[34], 9}, {NULL}};
const Segment SGI196509[] = {{&S79385[93], 9}, {NULL}};
const Segment SGI196510[] = {{&S79385[66], 6}, {NULL}};
const Segment SGI196511[] = {{&S67591[27], 6}, {NULL}};
const Segment SGI196512[] = {{&S67591[96], 9}, {NULL}};
const Segment SGI196513[] = {{&S67591[69], 6}, {NULL}};
const Segment SGI196514[] = {{&S79386[34], 16}, {NULL}};
const Segment SGI196515[] = {{&S79386[27], 6}, {NULL}};
const Segment SGI196516[] = {{&S79386[103], 9}, {NULL}};
const Segment SGI196517[] = {{&S79386[76], 6}, {NULL}};
const Segment SGI196518[] = {{&S73267[34], 25}, {NULL}};
const Segment SGI196519[] = {{&S73267[27], 6}, {NULL}};
const Segment SGI196520[] = {{&S73267[112], 9}, {NULL}};
const Segment SGI196521[] = {{&S73267[85], 6}, {NULL}};
const Segment SGI196522[] = {{&S67609[44], 25}, {NULL}};
const Segment SGI196523[] = {{&S67609[44], 37}, {NULL}};
const Segment SGI196524[] = {{&S67609[30], 6}, {NULL}};
const Segment SGI196525[] = {{&S67609[31], 3}, {NULL}};
const Segment SGI196526[] = {{&S67609[95], 6}, {NULL}};
const Segment SGI196527[] = {{&S73286[44], 5}, {NULL}};
const Segment SGI196528[] = {{&S73286[44], 17}, {NULL}};
const Segment SGI196529[] = {{&S73286[30], 6}, {NULL}};
const Segment SGI196530[] = {{&S73286[110], 3}, {NULL}};
const Segment SGI196531[] = {{&S73286[75], 6}, {NULL}};
const Segment SGI196532[] = {{&S79419[34], 8}, {NULL}};
const Segment SGI196533[] = {{&S79419[27], 6}, {NULL}};
const Segment SGI196534[] = {{&S79419[95], 9}, {NULL}};
const Segment SGI196535[] = {{&S79419[68], 6}, {NULL}};
const Segment SGI196536[] = {{&S79429[44], 7}, {NULL}};
const Segment SGI196537[] = {{&S73308[34], 7}, {NULL}};
const Segment SGI196538[] = {{&S79429[44], 19}, {NULL}};
const Segment SGI196539[] = {{&S79429[30], 6}, {NULL}};
const Segment SGI196540[] = {{&S73308[27], 6}, {NULL}};
const Segment SGI196541[] = {{&S79429[31], 3}, {NULL}};
const Segment SGI196542[] = {{&S79429[77], 6}, {NULL}};
const Segment SGI196543[] = {{&S58641[34], 10}, {NULL}};
const Segment SGI196544[] = {{&S58641[27], 6}, {NULL}};
const Segment SGI196545[] = {{&S58641[96], 9}, {NULL}};
const Segment SGI196546[] = {{&S58641[69], 6}, {NULL}};
const Segment SGI196547[] = {{&S79438[44], 10}, {NULL}};
const Segment SGI196548[] = {{&S79438[44], 19}, {NULL}};
const Segment SGI196549[] = {{&S79438[30], 6}, {NULL}};
const Segment SGI196550[] = {{&S79438[31], 3}, {NULL}};
const Segment SGI196551[] = {{&S79438[77], 6}, {NULL}};
const Segment SGI196552[] = {{&S79442[44], 15}, {NULL}};
const Segment SGI196553[] = {{&S79442[44], 27}, {NULL}};
const Segment SGI196554[] = {{&S79442[30], 6}, {NULL}};
const Segment SGI196555[] = {{&S79442[31], 3}, {NULL}};
const Segment SGI196556[] = {{&S79442[85], 6}, {NULL}};
const Segment SGI196557[] = {{&S79444[44], 7}, {NULL}};
const Segment SGI196558[] = {{&S79444[44], 19}, {NULL}};
const Segment SGI196559[] = {{&S79444[30], 6}, {NULL}};
const Segment SGI196560[] = {{&S79444[31], 3}, {NULL}};
const Segment SGI196561[] = {{&S79444[77], 6}, {NULL}};
const Segment SGI196562[] = {{&S79448[34], 25}, {NULL}};
const Segment SGI196563[] = {{&S79448[27], 6}, {NULL}};
const Segment SGI196564[] = {{&S79448[112], 9}, {NULL}};
const Segment SGI196565[] = {{&S79448[85], 6}, {NULL}};
const Segment SGI196566[] = {{&S79449[34], 25}, {NULL}};
const Segment SGI196567[] = {{&S79449[27], 6}, {NULL}};
const Segment SGI196568[] = {{&S79449[112], 9}, {NULL}};
const Segment SGI196569[] = {{&S79449[85], 6}, {NULL}};
const Segment SGI196570[] = {{&S79450[34], 25}, {NULL}};
const Segment SGI196571[] = {{&S79450[27], 6}, {NULL}};
const Segment SGI196572[] = {{&S79450[112], 9}, {NULL}};
const Segment SGI196573[] = {{&S79450[85], 6}, {NULL}};
const Segment SGI196574[] = {{&S68606[44], 7}, {NULL}};
const Segment SGI196575[] = {{&S68606[44], 29}, {NULL}};
const Segment SGI196576[] = {{&S68606[30], 6}, {NULL}};
const Segment SGI196577[] = {{&S68606[122], 3}, {NULL}};
const Segment SGI196578[] = {{&S68606[87], 6}, {NULL}};
const Segment SGI196579[] = {{&S79457[34], 9}, {NULL}};
const Segment SGI196580[] = {{&S79457[27], 6}, {NULL}};
const Segment SGI196581[] = {{&S68607[44], 7}, {NULL}};
const Segment SGI196582[] = {{&S68607[44], 25}, {NULL}};
const Segment SGI196583[] = {{&S68607[30], 6}, {NULL}};
const Segment SGI196584[] = {{&S79457[96], 9}, {NULL}};
const Segment SGI196585[] = {{&S79457[69], 6}, {NULL}};
const Segment SGI196586[] = {{&S68607[118], 3}, {NULL}};
const Segment SGI196587[] = {{&S68607[83], 6}, {NULL}};
const Segment SGI196588[] = {{&S68608[38], 7}, {NULL}};
const Segment SGI196589[] = {{&S68608[31], 6}, {NULL}};
const Segment SGI196590[] = {{&S79459[34], 11}, {NULL}};
const Segment SGI196591[] = {{&S79459[27], 6}, {NULL}};
const Segment SGI196592[] = {{&S79459[98], 9}, {NULL}};
const Segment SGI196593[] = {{&S79459[71], 6}, {NULL}};
const Segment SGI196594[] = {{&S68610[44], 10}, {NULL}};
const Segment SGI196595[] = {{&S68610[44], 22}, {NULL}};
const Segment SGI196596[] = {{&S68610[30], 6}, {NULL}};
const Segment SGI196597[] = {{&S68610[80], 6}, {NULL}};
const Segment SGI196598[] = {{&S68610[107], 13}, {NULL}};
const Segment SGI196599[] = {{&S68626[34], 9}, {NULL}};
const Segment SGI196600[] = {{&S68626[27], 6}, {NULL}};
const Segment SGI196601[] = {{&S68626[96], 9}, {NULL}};
const Segment SGI196602[] = {{&S68626[69], 6}, {NULL}};
const Segment SGI196603[] = {{&S67692[34], 20}, {NULL}};
const Segment SGI196604[] = {{&S67692[27], 6}, {NULL}};
const Segment SGI196605[] = {{&S68627[34], 9}, {NULL}};
const Segment SGI196606[] = {{&S68627[27], 6}, {NULL}};
const Segment SGI196607[] = {{&S67692[107], 9}, {NULL}};
const Segment SGI196608[] = {{&S67692[80], 6}, {NULL}};
const Segment SGI196609[] = {{&S68627[96], 9}, {NULL}};
const Segment SGI196610[] = {{&S68627[69], 6}, {NULL}};
const Segment SGI196611[] = {{&S79483[59], 10}, {NULL}};
const Segment SGI196612[] = {{&S79483[59], 22}, {NULL}};
const Segment SGI196613[] = {{&S79483[45], 6}, {NULL}};
const Segment SGI196614[] = {{&S79483[46], 3}, {NULL}};
const Segment SGI196615[] = {{&S79483[95], 6}, {NULL}};
const Segment SGI196616[] = {{&S79483[0], 14}, {NULL}};
const Segment SGI196617[] = {{&S79491[44], 11}, {NULL}};
const Segment SGI196618[] = {{&S79491[44], 23}, {NULL}};
const Segment SGI196619[] = {{&S79491[30], 6}, {NULL}};
const Segment SGI196620[] = {{&S79491[84], 6}, {NULL}};
const Segment SGI196621[] = {{&S79492[44], 11}, {NULL}};
const Segment SGI196622[] = {{&S79492[44], 23}, {NULL}};
const Segment SGI196623[] = {{&S79492[30], 6}, {NULL}};
const Segment SGI196624[] = {{&S79492[31], 3}, {NULL}};
const Segment SGI196625[] = {{&S79492[81], 6}, {NULL}};
const Segment SGI196626[] = {{&S79493[38], 11}, {NULL}};
const Segment SGI196627[] = {{&S79493[31], 6}, {NULL}};
const Segment SGI196628[] = {{&S79495[44], 11}, {NULL}};
const Segment SGI196629[] = {{&S79495[44], 23}, {NULL}};
const Segment SGI196630[] = {{&S79495[30], 6}, {NULL}};
const Segment SGI196631[] = {{&S79495[31], 3}, {NULL}};
const Segment SGI196632[] = {{&S79495[81], 6}, {NULL}};
const Segment SGI196633[] = {{&S58723[38], 10}, {NULL}};
const Segment SGI196634[] = {{&S58723[31], 6}, {NULL}};
const Segment SGI196635[] = {{&S68648[44], 7}, {NULL}};
const Segment SGI196636[] = {{&S79496[44], 11}, {NULL}};
const Segment SGI196637[] = {{&S68648[44], 27}, {NULL}};
const Segment SGI196638[] = {{&S79496[44], 23}, {NULL}};
const Segment SGI196639[] = {{&S68648[30], 6}, {NULL}};
const Segment SGI196640[] = {{&S79496[30], 6}, {NULL}};
const Segment SGI196641[] = {{&S79496[31], 3}, {NULL}};
const Segment SGI196642[] = {{&S68648[136], 3}, {NULL}};
const Segment SGI196643[] = {{&S79496[81], 6}, {NULL}};
const Segment SGI196644[] = {{&S68648[101], 6}, {NULL}};
const Segment SGI196645[] = {{&S68648[73], 8}, {NULL}};
const Segment SGI196646[] = {{&S58725[44], 10}, {NULL}};
const Segment SGI196647[] = {{&S58725[44], 21}, {NULL}};
const Segment SGI196648[] = {{&S58725[30], 6}, {NULL}};
const Segment SGI196649[] = {{&S58725[114], 3}, {NULL}};
const Segment SGI196650[] = {{&S79497[44], 10}, {NULL}};
const Segment SGI196651[] = {{&S58725[79], 6}, {NULL}};
const Segment SGI196652[] = {{&S79497[44], 22}, {NULL}};
const Segment SGI196653[] = {{&S79497[30], 6}, {NULL}};
const Segment SGI196654[] = {{&S79497[31], 3}, {NULL}};
const Segment SGI196655[] = {{&S79497[80], 6}, {NULL}};
const Segment SGI196656[] = {{&S79498[34], 14}, {NULL}};
const Segment SGI196657[] = {{&S79498[27], 6}, {NULL}};
const Segment SGI196658[] = {{&S79498[101], 9}, {NULL}};
const Segment SGI196659[] = {{&S79498[74], 6}, {NULL}};
const Segment SGI196660[] = {{&S79500[44], 13}, {NULL}};
const Segment SGI196661[] = {{&S79500[44], 25}, {NULL}};
const Segment SGI196662[] = {{&S79500[30], 6}, {NULL}};
const Segment SGI196663[] = {{&S79500[31], 3}, {NULL}};
const Segment SGI196664[] = {{&S79500[83], 6}, {NULL}};
const Segment SGI196665[] = {{&S79501[34], 11}, {NULL}};
const Segment SGI196666[] = {{&S79501[27], 6}, {NULL}};
const Segment SGI196667[] = {{&S79501[98], 9}, {NULL}};
const Segment SGI196668[] = {{&S79501[71], 6}, {NULL}};
const Segment SGI196669[] = {{&S79502[44], 11}, {NULL}};
const Segment SGI196670[] = {{&S79502[44], 23}, {NULL}};
const Segment SGI196671[] = {{&S79502[30], 6}, {NULL}};
const Segment SGI196672[] = {{&S79502[31], 3}, {NULL}};
const Segment SGI196673[] = {{&S79502[81], 6}, {NULL}};
const Segment SGI196674[] = {{&S68667[34], 16}, {NULL}};
const Segment SGI196675[] = {{&S68667[27], 6}, {NULL}};
const Segment SGI196676[] = {{&S68667[110], 9}, {NULL}};
const Segment SGI196677[] = {{&S68667[83], 6}, {NULL}};
const Segment SGI196678[] = {{&S79514[34], 7}, {NULL}};
const Segment SGI196679[] = {{&S79514[27], 6}, {NULL}};
const Segment SGI196680[] = {{&S79514[112], 9}, {NULL}};
const Segment SGI196681[] = {{&S79514[85], 6}, {NULL}};
const Segment SGI196682[] = {{&S79515[34], 11}, {NULL}};
const Segment SGI196683[] = {{&S79515[27], 6}, {NULL}};
const Segment SGI196684[] = {{&S79515[110], 9}, {NULL}};
const Segment SGI196685[] = {{&S79515[83], 6}, {NULL}};
const Segment SGI196686[] = {{&S68676[44], 11}, {NULL}};
const Segment SGI196687[] = {{&S68676[44], 23}, {NULL}};
const Segment SGI196688[] = {{&S68676[30], 6}, {NULL}};
const Segment SGI196689[] = {{&S68676[116], 3}, {NULL}};
const Segment SGI196690[] = {{&S68676[81], 6}, {NULL}};
const Segment SGI196691[] = {{&S68677[44], 11}, {NULL}};
const Segment SGI196692[] = {{&S68677[44], 23}, {NULL}};
const Segment SGI196693[] = {{&S68677[30], 6}, {NULL}};
const Segment SGI196694[] = {{&S68677[116], 3}, {NULL}};
const Segment SGI196695[] = {{&S68677[81], 6}, {NULL}};
const Segment SGI196696[] = {{&S68678[38], 11}, {NULL}};
const Segment SGI196697[] = {{&S68678[31], 6}, {NULL}};
const Segment SGI196698[] = {{&S68679[44], 11}, {NULL}};
const Segment SGI196699[] = {{&S68679[44], 23}, {NULL}};
const Segment SGI196700[] = {{&S68679[30], 6}, {NULL}};
const Segment SGI196701[] = {{&S68679[84], 6}, {NULL}};
const Segment SGI196702[] = {{&S68686[34], 9}, {NULL}};
const Segment SGI196703[] = {{&S68686[27], 6}, {NULL}};
const Segment SGI196704[] = {{&S68686[96], 9}, {NULL}};
const Segment SGI196705[] = {{&S68686[69], 6}, {NULL}};
const Segment SGI196706[] = {{&S79541[44], 10}, {NULL}};
const Segment SGI196707[] = {{&S79541[44], 22}, {NULL}};
const Segment SGI196708[] = {{&S79541[30], 6}, {NULL}};
const Segment SGI196709[] = {{&S79541[31], 3}, {NULL}};
const Segment SGI196710[] = {{&S79541[80], 6}, {NULL}};
const Segment SGI196711[] = {{&S79542[44], 10}, {NULL}};
const Segment SGI196712[] = {{&S79542[44], 22}, {NULL}};
const Segment SGI196713[] = {{&S79542[30], 6}, {NULL}};
const Segment SGI196714[] = {{&S79542[31], 3}, {NULL}};
const Segment SGI196715[] = {{&S79542[80], 6}, {NULL}};
const Segment SGI196716[] = {{&S73308[94], 9}, {NULL}};
const Segment SGI196717[] = {{&S73308[67], 6}, {NULL}};
const Segment SGI196718[] = {{&S73311[34], 9}, {NULL}};
const Segment SGI196719[] = {{&S73311[27], 6}, {NULL}};
const Segment SGI196720[] = {{&S73311[96], 9}, {NULL}};
const Segment SGI196721[] = {{&S73311[69], 6}, {NULL}};
const Segment SGI196722[] = {{&S67794[34], 13}, {NULL}};
const Segment SGI196723[] = {{&S67794[27], 6}, {NULL}};
const Segment SGI196724[] = {{&S67794[100], 9}, {NULL}};
const Segment SGI196725[] = {{&S67794[73], 6}, {NULL}};
const Segment SGI196726[] = {{&S79558[44], 10}, {NULL}};
const Segment SGI196727[] = {{&S79558[44], 22}, {NULL}};
const Segment SGI196728[] = {{&S79558[30], 6}, {NULL}};
const Segment SGI196729[] = {{&S79558[31], 3}, {NULL}};
const Segment SGI196730[] = {{&S79558[80], 6}, {NULL}};
const Segment SGI196731[] = {{&S79558[139], 10}, {NULL}};
const Segment SGI196732[] = {{&S67796[34], 19}, {NULL}};
const Segment SGI196733[] = {{&S67796[27], 6}, {NULL}};
const Segment SGI196734[] = {{&S67796[108], 9}, {NULL}};
const Segment SGI196735[] = {{&S67796[81], 6}, {NULL}};
const Segment SGI196736[] = {{&S73323[34], 17}, {NULL}};
const Segment SGI196737[] = {{&S68723[34], 9}, {NULL}};
const Segment SGI196738[] = {{&S73323[27], 6}, {NULL}};
const Segment SGI196739[] = {{&S68723[27], 6}, {NULL}};
const Segment SGI196740[] = {{&S73323[104], 9}, {NULL}};
const Segment SGI196741[] = {{&S68723[96], 9}, {NULL}};
const Segment SGI196742[] = {{&S73323[77], 6}, {NULL}};
const Segment SGI196743[] = {{&S68723[69], 6}, {NULL}};
const Segment SGI196744[] = {{&S68724[34], 9}, {NULL}};
const Segment SGI196745[] = {{&S68724[27], 6}, {NULL}};
const Segment SGI196746[] = {{&S68724[96], 9}, {NULL}};
const Segment SGI196747[] = {{&S68724[69], 6}, {NULL}};
const Segment SGI196748[] = {{&S73325[44], 10}, {NULL}};
const Segment SGI196749[] = {{&S73325[44], 22}, {NULL}};
const Segment SGI196750[] = {{&S73325[30], 6}, {NULL}};
const Segment SGI196751[] = {{&S73325[115], 3}, {NULL}};
const Segment SGI196752[] = {{&S73325[80], 6}, {NULL}};
const Segment SGI196753[] = {{&S68725[34], 10}, {NULL}};
const Segment SGI196754[] = {{&S68725[27], 6}, {NULL}};
const Segment SGI196755[] = {{&S68725[97], 9}, {NULL}};
const Segment SGI196756[] = {{&S68725[70], 6}, {NULL}};
const Segment SGI196757[] = {{&S79569[44], 13}, {NULL}};
const Segment SGI196758[] = {{&S79569[44], 29}, {NULL}};
const Segment SGI196759[] = {{&S79569[30], 6}, {NULL}};
const Segment SGI196760[] = {{&S79569[31], 3}, {NULL}};
const Segment SGI196761[] = {{&S79569[87], 6}, {NULL}};
const Segment SGI196762[] = {{&S67807[44], 10}, {NULL}};
const Segment SGI196763[] = {{&S67807[44], 21}, {NULL}};
const Segment SGI196764[] = {{&S67807[30], 6}, {NULL}};
const Segment SGI196765[] = {{&S67807[114], 3}, {NULL}};
const Segment SGI196766[] = {{&S67807[79], 6}, {NULL}};
const Segment SGI196767[] = {{&S67808[38], 10}, {NULL}};
const Segment SGI196768[] = {{&S79571[44], 5}, {NULL}};
const Segment SGI196769[] = {{&S67808[31], 6}, {NULL}};
const Segment SGI196770[] = {{&S79571[44], 17}, {NULL}};
const Segment SGI196771[] = {{&S79571[30], 6}, {NULL}};
const Segment SGI196772[] = {{&S79571[75], 6}, {NULL}};
const Segment SGI196773[] = {{&S79571[102], 13}, {NULL}};
const Segment SGI196774[] = {{&S79574[34], 9}, {NULL}};
const Segment SGI196775[] = {{&S79574[27], 6}, {NULL}};
const Segment SGI196776[] = {{&S79574[96], 9}, {NULL}};
const Segment SGI196777[] = {{&S79574[69], 6}, {NULL}};
const Segment SGI196778[] = {{&S79575[34], 11}, {NULL}};
const Segment SGI196779[] = {{&S79575[27], 6}, {NULL}};
const Segment SGI196780[] = {{&S79575[98], 9}, {NULL}};
const Segment SGI196781[] = {{&S79575[71], 6}, {NULL}};
const Segment SGI196782[] = {{&S79576[34], 10}, {NULL}};
const Segment SGI196783[] = {{&S79576[27], 6}, {NULL}};
const Segment SGI196784[] = {{&S79576[97], 9}, {NULL}};
const Segment SGI196785[] = {{&S79576[70], 6}, {NULL}};
const Segment SGI196786[] = {{&S67827[34], 13}, {NULL}};
const Segment SGI196787[] = {{&S67827[27], 6}, {NULL}};
const Segment SGI196788[] = {{&S67827[100], 9}, {NULL}};
const Segment SGI196789[] = {{&S67827[73], 6}, {NULL}};
const Segment SGI196790[] = {{&S68751[34], 10}, {NULL}};
const Segment SGI196791[] = {{&S68751[27], 6}, {NULL}};
const Segment SGI196792[] = {{&S68751[97], 9}, {NULL}};
const Segment SGI196793[] = {{&S68751[70], 6}, {NULL}};
const Segment SGI196794[] = {{&S68752[34], 10}, {NULL}};
const Segment SGI196795[] = {{&S68752[27], 6}, {NULL}};
const Segment SGI196796[] = {{&S68752[97], 9}, {NULL}};
const Segment SGI196797[] = {{&S68752[70], 6}, {NULL}};
const Segment SGI196798[] = {{&S79595[34], 10}, {NULL}};
const Segment SGI196799[] = {{&S79595[27], 6}, {NULL}};
const Segment SGI196800[] = {{&S79595[97], 9}, {NULL}};
const Segment SGI196801[] = {{&S79595[70], 6}, {NULL}};
const Segment SGI196802[] = {{&S73363[34], 12}, {NULL}};
const Segment SGI196803[] = {{&S73363[27], 6}, {NULL}};
const Segment SGI196804[] = {{&S73363[99], 9}, {NULL}};
const Segment SGI196805[] = {{&S73363[72], 6}, {NULL}};
const Segment SGI196806[] = {{&S79602[34], 16}, {NULL}};
const Segment SGI196807[] = {{&S79602[27], 6}, {NULL}};
const Segment SGI196808[] = {{&S79602[110], 9}, {NULL}};
const Segment SGI196809[] = {{&S79602[83], 6}, {NULL}};
const Segment SGI196810[] = {{&S79603[34], 15}, {NULL}};
const Segment SGI196811[] = {{&S79603[27], 6}, {NULL}};
const Segment SGI196812[] = {{&S79603[108], 9}, {NULL}};
const Segment SGI196813[] = {{&S79603[81], 6}, {NULL}};
const Segment SGI196814[] = {{&S79605[44], 7}, {NULL}};
const Segment SGI196815[] = {{&S79605[44], 19}, {NULL}};
const Segment SGI196816[] = {{&S79605[30], 6}, {NULL}};
const Segment SGI196817[] = {{&S79605[31], 3}, {NULL}};
const Segment SGI196818[] = {{&S79605[77], 6}, {NULL}};
const Segment SGI196819[] = {{&S79606[38], 7}, {NULL}};
const Segment SGI196820[] = {{&S79606[31], 6}, {NULL}};
const Segment SGI196821[] = {{&S79607[44], 16}, {NULL}};
const Segment SGI196822[] = {{&S79607[44], 28}, {NULL}};
const Segment SGI196823[] = {{&S79607[30], 6}, {NULL}};
const Segment SGI196824[] = {{&S79607[31], 3}, {NULL}};
const Segment SGI196825[] = {{&S79607[86], 6}, {NULL}};
const Segment SGI196826[] = {{&S73391[44], 7}, {NULL}};
const Segment SGI196827[] = {{&S73391[44], 19}, {NULL}};
const Segment SGI196828[] = {{&S73391[30], 6}, {NULL}};
const Segment SGI196829[] = {{&S73391[112], 3}, {NULL}};
const Segment SGI196830[] = {{&S73391[77], 6}, {NULL}};
const Segment SGI196831[] = {{&S67873[44], 12}, {NULL}};
const Segment SGI196832[] = {{&S67873[44], 24}, {NULL}};
const Segment SGI196833[] = {{&S67873[30], 6}, {NULL}};
const Segment SGI196834[] = {{&S67873[117], 3}, {NULL}};
const Segment SGI196835[] = {{&S67873[82], 6}, {NULL}};
const Segment SGI196836[] = {{&S67875[44], 12}, {NULL}};
const Segment SGI196837[] = {{&S67875[44], 24}, {NULL}};
const Segment SGI196838[] = {{&S67875[30], 6}, {NULL}};
const Segment SGI196839[] = {{&S67875[117], 3}, {NULL}};
const Segment SGI196840[] = {{&S67875[82], 6}, {NULL}};
const Segment SGI196841[] = {{&S68805[34], 10}, {NULL}};
const Segment SGI196842[] = {{&S68805[27], 6}, {NULL}};
const Segment SGI196843[] = {{&S68805[97], 9}, {NULL}};
const Segment SGI196844[] = {{&S68805[70], 6}, {NULL}};
const Segment SGI196845[] = {{&S68809[34], 12}, {NULL}};
const Segment SGI196846[] = {{&S68809[27], 6}, {NULL}};
const Segment SGI196847[] = {{&S68809[99], 9}, {NULL}};
const Segment SGI196848[] = {{&S68809[72], 6}, {NULL}};
const Segment SGI196849[] = {{&S73409[44], 9}, {NULL}};
const Segment SGI196850[] = {{&S73409[44], 21}, {NULL}};
const Segment SGI196851[] = {{&S73409[30], 6}, {NULL}};
const Segment SGI196852[] = {{&S73409[114], 3}, {NULL}};
const Segment SGI196853[] = {{&S73409[79], 6}, {NULL}};
const Segment SGI196854[] = {{&S68815[34], 8}, {NULL}};
const Segment SGI196855[] = {{&S68815[27], 6}, {NULL}};
const Segment SGI196856[] = {{&S68815[95], 9}, {NULL}};
const Segment SGI196857[] = {{&S68815[68], 6}, {NULL}};
const Segment SGI196858[] = {{&S79643[44], 10}, {NULL}};
const Segment SGI196859[] = {{&S79643[44], 22}, {NULL}};
const Segment SGI196860[] = {{&S79643[30], 6}, {NULL}};
const Segment SGI196861[] = {{&S79643[31], 3}, {NULL}};
const Segment SGI196862[] = {{&S79643[80], 6}, {NULL}};
const Segment SGI196863[] = {{&S79644[38], 10}, {NULL}};
const Segment SGI196864[] = {{&S79644[31], 6}, {NULL}};
const Segment SGI196865[] = {{&S79645[34], 4}, {NULL}};
const Segment SGI196866[] = {{&S79645[27], 6}, {NULL}};
const Segment SGI196867[] = {{&S79645[91], 9}, {NULL}};
const Segment SGI196868[] = {{&S79645[64], 6}, {NULL}};
const Segment SGI196869[] = {{&S73419[34], 12}, {NULL}};
const Segment SGI196870[] = {{&S73419[27], 6}, {NULL}};
const Segment SGI196871[] = {{&S73419[99], 9}, {NULL}};
const Segment SGI196872[] = {{&S73419[72], 6}, {NULL}};
const Segment SGI196873[] = {{&S73419[140], 8}, {NULL}};
const Segment SGI196874[] = {{&S67903[34], 6}, {NULL}};
const Segment SGI196875[] = {{&S67903[27], 6}, {NULL}};
const Segment SGI196876[] = {{&S67903[93], 9}, {NULL}};
const Segment SGI196877[] = {{&S67903[66], 6}, {NULL}};
const Segment SGI196878[] = {{&S79658[44], 7}, {NULL}};
const Segment SGI196879[] = {{&S79658[44], 23}, {NULL}};
const Segment SGI196880[] = {{&S79658[30], 6}, {NULL}};
const Segment SGI196881[] = {{&S79658[81], 6}, {NULL}};
const Segment SGI196882[] = {{&S79658[108], 13}, {NULL}};
const Segment SGI196883[] = {{&S79659[44], 7}, {NULL}};
const Segment SGI196884[] = {{&S79659[44], 22}, {NULL}};
const Segment SGI196885[] = {{&S79659[30], 6}, {NULL}};
const Segment SGI196886[] = {{&S79659[80], 6}, {NULL}};
const Segment SGI196887[] = {{&S79659[107], 13}, {NULL}};
const Segment SGI196888[] = {{&S79677[34], 11}, {NULL}};
const Segment SGI196889[] = {{&S79677[27], 6}, {NULL}};
const Segment SGI196890[] = {{&S79677[98], 9}, {NULL}};
const Segment SGI196891[] = {{&S79677[71], 6}, {NULL}};
const Segment SGI196892[] = {{&S68859[44], 10}, {NULL}};
const Segment SGI196893[] = {{&S68859[44], 22}, {NULL}};
const Segment SGI196894[] = {{&S68859[30], 6}, {NULL}};
const Segment SGI196895[] = {{&S68859[115], 3}, {NULL}};
const Segment SGI196896[] = {{&S68859[80], 6}, {NULL}};
const Segment SGI196897[] = {{&S68860[38], 10}, {NULL}};
const Segment SGI196898[] = {{&S68860[31], 6}, {NULL}};
const Segment SGI196899[] = {{&S68861[44], 10}, {NULL}};
const Segment SGI196900[] = {{&S68861[44], 22}, {NULL}};
const Segment SGI196901[] = {{&S68861[30], 6}, {NULL}};
const Segment SGI196902[] = {{&S68861[83], 6}, {NULL}};
const Segment SGI196903[] = {{&S67936[44], 9}, {NULL}};
const Segment SGI196904[] = {{&S67936[44], 21}, {NULL}};
const Segment SGI196905[] = {{&S67936[30], 6}, {NULL}};
const Segment SGI196906[] = {{&S67936[143], 3}, {NULL}};
const Segment SGI196907[] = {{&S67936[108], 6}, {NULL}};
const Segment SGI196908[] = {{&S73462[44], 10}, {NULL}};
const Segment SGI196909[] = {{&S73462[44], 22}, {NULL}};
const Segment SGI196910[] = {{&S73462[30], 6}, {NULL}};
const Segment SGI196911[] = {{&S73462[80], 6}, {NULL}};
const Segment SGI196912[] = {{&S73462[107], 13}, {NULL}};
const Segment SGI196913[] = {{&S68871[34], 10}, {NULL}};
const Segment SGI196914[] = {{&S68871[27], 6}, {NULL}};
const Segment SGI196915[] = {{&S73463[44], 20}, {NULL}};
const Segment SGI196916[] = {{&S73463[44], 32}, {NULL}};
const Segment SGI196917[] = {{&S73463[30], 6}, {NULL}};
const Segment SGI196918[] = {{&S68871[97], 9}, {NULL}};
const Segment SGI196919[] = {{&S68871[70], 6}, {NULL}};
const Segment SGI196920[] = {{&S73463[125], 3}, {NULL}};
const Segment SGI196921[] = {{&S73463[90], 6}, {NULL}};
const Segment SGI196922[] = {{&S68872[34], 10}, {NULL}};
const Segment SGI196923[] = {{&S68872[27], 6}, {NULL}};
const Segment SGI196924[] = {{&S68872[97], 9}, {NULL}};
const Segment SGI196925[] = {{&S68872[70], 6}, {NULL}};
const Segment SGI196926[] = {{&S67947[44], 8}, {NULL}};
const Segment SGI196927[] = {{&S67947[44], 20}, {NULL}};
const Segment SGI196928[] = {{&S67947[30], 6}, {NULL}};
const Segment SGI196929[] = {{&S67947[113], 3}, {NULL}};
const Segment SGI196930[] = {{&S67947[78], 6}, {NULL}};
const Segment SGI196931[] = {{&S68873[34], 10}, {NULL}};
const Segment SGI196932[] = {{&S68873[27], 6}, {NULL}};
const Segment SGI196933[] = {{&S68873[97], 9}, {NULL}};
const Segment SGI196934[] = {{&S68873[70], 6}, {NULL}};
const Segment SGI196935[] = {{&S67948[38], 8}, {NULL}};
const Segment SGI196936[] = {{&S67948[31], 6}, {NULL}};
const Segment SGI196937[] = {{&S68874[34], 11}, {NULL}};
const Segment SGI196938[] = {{&S68874[27], 6}, {NULL}};
const Segment SGI196939[] = {{&S68874[98], 9}, {NULL}};
const Segment SGI196940[] = {{&S68874[71], 6}, {NULL}};
const Segment SGI196941[] = {{&S67953[34], 10}, {NULL}};
const Segment SGI196942[] = {{&S67953[27], 6}, {NULL}};
const Segment SGI196943[] = {{&S67953[97], 9}, {NULL}};
const Segment SGI196944[] = {{&S67953[70], 6}, {NULL}};
const Segment SGI196945[] = {{&S68879[34], 9}, {NULL}};
const Segment SGI196946[] = {{&S68879[27], 6}, {NULL}};
const Segment SGI196947[] = {{&S68879[96], 9}, {NULL}};
const Segment SGI196948[] = {{&S68879[69], 6}, {NULL}};
const Segment SGI196949[] = {{&S79709[34], 11}, {NULL}};
const Segment SGI196950[] = {{&S79709[27], 6}, {NULL}};
const Segment SGI196951[] = {{&S79709[98], 9}, {NULL}};
const Segment SGI196952[] = {{&S79709[71], 6}, {NULL}};
const Segment SGI196953[] = {{&S79722[34], 7}, {NULL}};
const Segment SGI196954[] = {{&S68894[34], 15}, {NULL}};
const Segment SGI196955[] = {{&S79722[27], 6}, {NULL}};
const Segment SGI196956[] = {{&S68894[27], 6}, {NULL}};
const Segment SGI196957[] = {{&S79722[99], 9}, {NULL}};
const Segment SGI196958[] = {{&S79722[72], 6}, {NULL}};
const Segment SGI196959[] = {{&S68894[102], 9}, {NULL}};
const Segment SGI196960[] = {{&S68894[75], 6}, {NULL}};
const Segment SGI196961[] = {{&S79725[34], 20}, {NULL}};
const Segment SGI196962[] = {{&S79725[27], 6}, {NULL}};
const Segment SGI196963[] = {{&S79725[107], 9}, {NULL}};
const Segment SGI196964[] = {{&S79725[80], 6}, {NULL}};
const Segment SGI196965[] = {{&S79726[34], 25}, {NULL}};
const Segment SGI196966[] = {{&S79726[27], 6}, {NULL}};
const Segment SGI196967[] = {{&S79726[112], 9}, {NULL}};
const Segment SGI196968[] = {{&S79726[85], 6}, {NULL}};
const Segment SGI196969[] = {{&S79729[44], 9}, {NULL}};
const Segment SGI196970[] = {{&S79729[44], 21}, {NULL}};
const Segment SGI196971[] = {{&S79729[30], 6}, {NULL}};
const Segment SGI196972[] = {{&S79729[31], 3}, {NULL}};
const Segment SGI196973[] = {{&S79729[79], 6}, {NULL}};
const Segment SGI196974[] = {{&S68904[44], 11}, {NULL}};
const Segment SGI196975[] = {{&S68904[44], 23}, {NULL}};
const Segment SGI196976[] = {{&S68904[30], 6}, {NULL}};
const Segment SGI196977[] = {{&S68904[116], 3}, {NULL}};
const Segment SGI196978[] = {{&S68904[81], 6}, {NULL}};
const Segment SGI196979[] = {{&S68905[38], 11}, {NULL}};
const Segment SGI196980[] = {{&S68905[31], 6}, {NULL}};
const Segment SGI196981[] = {{&S68906[44], 11}, {NULL}};
const Segment SGI196982[] = {{&S68906[44], 26}, {NULL}};
const Segment SGI196983[] = {{&S68906[30], 6}, {NULL}};
const Segment SGI196984[] = {{&S68906[119], 3}, {NULL}};
const Segment SGI196985[] = {{&S68906[84], 6}, {NULL}};
const Segment SGI196986[] = {{&S79735[34], 11}, {NULL}};
const Segment SGI196987[] = {{&S79735[27], 6}, {NULL}};
const Segment SGI196988[] = {{&S79735[98], 9}, {NULL}};
const Segment SGI196989[] = {{&S79735[71], 6}, {NULL}};
const Segment SGI196990[] = {{&S79735[139], 8}, {NULL}};
const Segment SGI196991[] = {{&S59009[34], 18}, {NULL}};
const Segment SGI196992[] = {{&S59009[27], 6}, {NULL}};
const Segment SGI196993[] = {{&S59009[104], 9}, {NULL}};
const Segment SGI196994[] = {{&S59009[77], 6}, {NULL}};
const Segment SGI196995[] = {{&S68911[34], 18}, {NULL}};
const Segment SGI196996[] = {{&S68911[27], 6}, {NULL}};
const Segment SGI196997[] = {{&S68911[113], 9}, {NULL}};
const Segment SGI196998[] = {{&S68911[86], 6}, {NULL}};
const Segment SGI196999[] = {{&S68912[34], 8}, {NULL}};
const Segment SGI197000[] = {{&S68912[27], 6}, {NULL}};
const Segment SGI197001[] = {{&S68912[103], 9}, {NULL}};
const Segment SGI197002[] = {{&S68912[76], 6}, {NULL}};
const Segment SGI197003[] = {{&S68913[34], 18}, {NULL}};
const Segment SGI197004[] = {{&S68913[27], 6}, {NULL}};
const Segment SGI197005[] = {{&S68913[113], 9}, {NULL}};
const Segment SGI197006[] = {{&S68913[86], 6}, {NULL}};
const Segment SGI197007[] = {{&S68914[34], 15}, {NULL}};
const Segment SGI197008[] = {{&S79744[34], 10}, {NULL}};
const Segment SGI197009[] = {{&S68914[27], 6}, {NULL}};
const Segment SGI197010[] = {{&S79744[27], 6}, {NULL}};
const Segment SGI197011[] = {{&S68914[110], 9}, {NULL}};
const Segment SGI197012[] = {{&S68914[83], 6}, {NULL}};
const Segment SGI197013[] = {{&S79744[97], 9}, {NULL}};
const Segment SGI197014[] = {{&S79744[70], 6}, {NULL}};
const Segment SGI197015[] = {{&S68915[34], 18}, {NULL}};
const Segment SGI197016[] = {{&S68915[27], 6}, {NULL}};
const Segment SGI197017[] = {{&S79745[34], 14}, {NULL}};
const Segment SGI197018[] = {{&S79745[27], 6}, {NULL}};
const Segment SGI197019[] = {{&S68915[113], 9}, {NULL}};
const Segment SGI197020[] = {{&S68915[86], 6}, {NULL}};
const Segment SGI197021[] = {{&S79745[101], 9}, {NULL}};
const Segment SGI197022[] = {{&S79745[74], 6}, {NULL}};
const Segment SGI197023[] = {{&S79746[34], 13}, {NULL}};
const Segment SGI197024[] = {{&S79746[27], 6}, {NULL}};
const Segment SGI197025[] = {{&S79746[100], 9}, {NULL}};
const Segment SGI197026[] = {{&S79746[73], 6}, {NULL}};
const Segment SGI197027[] = {{&S68931[44], 10}, {NULL}};
const Segment SGI197028[] = {{&S68931[44], 22}, {NULL}};
const Segment SGI197029[] = {{&S68931[30], 6}, {NULL}};
const Segment SGI197030[] = {{&S68931[115], 3}, {NULL}};
const Segment SGI197031[] = {{&S68931[80], 6}, {NULL}};
const Segment SGI197032[] = {{&S68932[44], 10}, {NULL}};
const Segment SGI197033[] = {{&S68932[44], 22}, {NULL}};
const Segment SGI197034[] = {{&S68932[30], 6}, {NULL}};
const Segment SGI197035[] = {{&S68932[83], 6}, {NULL}};
const Segment SGI197036[] = {{&S68933[38], 10}, {NULL}};
const Segment SGI197037[] = {{&S68933[31], 6}, {NULL}};
const Segment SGI197038[] = {{&S68934[44], 22}, {NULL}};
const Segment SGI197039[] = {{&S68934[30], 6}, {NULL}};
const Segment SGI197040[] = {{&S68935[44], 11}, {NULL}};
const Segment SGI197041[] = {{&S68935[44], 23}, {NULL}};
const Segment SGI197042[] = {{&S68935[30], 6}, {NULL}};
const Segment SGI197043[] = {{&S68935[116], 3}, {NULL}};
const Segment SGI197044[] = {{&S68935[81], 6}, {NULL}};
const Segment SGI197045[] = {{&S68936[38], 11}, {NULL}};
const Segment SGI197046[] = {{&S68936[31], 6}, {NULL}};
const Segment SGI197047[] = {{&S68939[34], 12}, {NULL}};
const Segment SGI197048[] = {{&S68939[27], 6}, {NULL}};
const Segment SGI197049[] = {{&S68939[99], 9}, {NULL}};
const Segment SGI197050[] = {{&S68939[72], 6}, {NULL}};
const Segment SGI197051[] = {{&S73555[34], 21}, {NULL}};
const Segment SGI197052[] = {{&S73555[27], 6}, {NULL}};
const Segment SGI197053[] = {{&S73555[108], 9}, {NULL}};
const Segment SGI197054[] = {{&S73555[81], 6}, {NULL}};
const Segment SGI197055[] = {{&S68952[34], 7}, {NULL}};
const Segment SGI197056[] = {{&S68952[27], 6}, {NULL}};
const Segment SGI197057[] = {{&S68952[105], 9}, {NULL}};
const Segment SGI197058[] = {{&S68952[78], 6}, {NULL}};
const Segment SGI197059[] = {{&S68953[34], 7}, {NULL}};
const Segment SGI197060[] = {{&S68953[27], 6}, {NULL}};
const Segment SGI197061[] = {{&S68953[105], 9}, {NULL}};
const Segment SGI197062[] = {{&S68953[78], 6}, {NULL}};
const Segment SGI197063[] = {{&S68984[34], 7}, {NULL}};
const Segment SGI197064[] = {{&S68984[27], 6}, {NULL}};
const Segment SGI197065[] = {{&S68984[107], 9}, {NULL}};
const Segment SGI197066[] = {{&S68984[80], 6}, {NULL}};
const Segment SGI197067[] = {{&S68985[34], 7}, {NULL}};
const Segment SGI197068[] = {{&S68985[27], 6}, {NULL}};
const Segment SGI197069[] = {{&S68985[107], 9}, {NULL}};
const Segment SGI197070[] = {{&S68985[80], 6}, {NULL}};
const Segment SGI197071[] = {{&S73597[44], 13}, {NULL}};
const Segment SGI197072[] = {{&S73597[44], 25}, {NULL}};
const Segment SGI197073[] = {{&S73597[30], 6}, {NULL}};
const Segment SGI197074[] = {{&S73597[118], 3}, {NULL}};
const Segment SGI197075[] = {{&S73597[83], 6}, {NULL}};
const Segment SGI197076[] = {{&S68987[34], 6}, {NULL}};
const Segment SGI197077[] = {{&S68987[27], 6}, {NULL}};
const Segment SGI197078[] = {{&S68987[93], 9}, {NULL}};
const Segment SGI197079[] = {{&S68987[66], 6}, {NULL}};
const Segment SGI197080[] = {{&S68988[44], 10}, {NULL}};
const Segment SGI197081[] = {{&S68988[44], 22}, {NULL}};
const Segment SGI197082[] = {{&S68988[30], 6}, {NULL}};
const Segment SGI197083[] = {{&S68988[131], 3}, {NULL}};
const Segment SGI197084[] = {{&S68988[96], 6}, {NULL}};
const Segment SGI197085[] = {{&S73600[44], 12}, {NULL}};
const Segment SGI197086[] = {{&S73600[44], 37}, {NULL}};
const Segment SGI197087[] = {{&S68988[68], 8}, {NULL}};
const Segment SGI197088[] = {{&S73600[30], 6}, {NULL}};
const Segment SGI197089[] = {{&S73600[130], 3}, {NULL}};
const Segment SGI197090[] = {{&S73600[95], 6}, {NULL}};
const Segment SGI197091[] = {{&S68989[44], 10}, {NULL}};
const Segment SGI197092[] = {{&S68989[44], 22}, {NULL}};
const Segment SGI197093[] = {{&S68989[30], 6}, {NULL}};
const Segment SGI197094[] = {{&S68989[131], 3}, {NULL}};
const Segment SGI197095[] = {{&S68989[96], 6}, {NULL}};
const Segment SGI197096[] = {{&S68989[68], 8}, {NULL}};
const Segment SGI197097[] = {{&S68990[44], 10}, {NULL}};
const Segment SGI197098[] = {{&S68990[44], 22}, {NULL}};
const Segment SGI197099[] = {{&S68990[30], 6}, {NULL}};
const Segment SGI197100[] = {{&S68990[144], 3}, {NULL}};
const Segment SGI197101[] = {{&S68990[109], 6}, {NULL}};
const Segment SGI197102[] = {{&S68990[68], 8}, {&S68990[68], 8}, {NULL}};
const Segment SGI197103[] = {{&S68991[44], 10}, {NULL}};
const Segment SGI197104[] = {{&S68991[44], 28}, {NULL}};
const Segment SGI197105[] = {{&S68991[30], 6}, {NULL}};
const Segment SGI197106[] = {{&S68991[150], 3}, {NULL}};
const Segment SGI197107[] = {{&S68991[115], 6}, {NULL}};
const Segment SGI197108[] = {{&S68991[74], 8}, {NULL}};
const Segment SGI197109[] = {{&S68992[44], 10}, {NULL}};
const Segment SGI197110[] = {{&S68992[44], 45}, {NULL}};
const Segment SGI197111[] = {{&S68992[30], 6}, {NULL}};
const Segment SGI197112[] = {{&S68992[164], 3}, {NULL}};
const Segment SGI197113[] = {{&S68992[129], 6}, {NULL}};
const Segment SGI197114[] = {{&S68992[91], 8}, {NULL}};
const Segment SGI197115[] = {{&S79992[44], 13}, {NULL}};
const Segment SGI197116[] = {{&S79992[44], 25}, {NULL}};
const Segment SGI197117[] = {{&S79992[30], 6}, {NULL}};
const Segment SGI197118[] = {{&S79992[31], 3}, {NULL}};
const Segment SGI197119[] = {{&S79992[83], 6}, {NULL}};
const Segment SGI197120[] = {{&S69013[34], 18}, {NULL}};
const Segment SGI197121[] = {{&S69013[27], 6}, {NULL}};
const Segment SGI197122[] = {{&S69013[105], 9}, {NULL}};
const Segment SGI197123[] = {{&S69013[78], 6}, {NULL}};
const Segment SGI197124[] = {{&S69014[34], 18}, {NULL}};
const Segment SGI197125[] = {{&S69014[27], 6}, {NULL}};
const Segment SGI197126[] = {{&S69014[105], 9}, {NULL}};
const Segment SGI197127[] = {{&S69014[78], 6}, {NULL}};
const Segment SGI197128[] = {{&S69015[34], 7}, {NULL}};
const Segment SGI197129[] = {{&S69015[27], 6}, {NULL}};
const Segment SGI197130[] = {{&S69015[102], 9}, {NULL}};
const Segment SGI197131[] = {{&S69015[75], 6}, {NULL}};
const Segment SGI197132[] = {{&S69017[34], 11}, {NULL}};
const Segment SGI197133[] = {{&S69017[27], 6}, {NULL}};
const Segment SGI197134[] = {{&S69017[98], 9}, {NULL}};
const Segment SGI197135[] = {{&S69017[71], 6}, {NULL}};
const Segment SGI197136[] = {{&S69030[43], 23}, {NULL}};
const Segment SGI197137[] = {{&S69030[30], 6}, {NULL}};
const Segment SGI197138[] = {{&S69030[31], 3}, {NULL}};
const Segment SGI197139[] = {{&S69030[80], 6}, {NULL}};
const Segment SGI197140[] = {{&S69031[44], 17}, {NULL}};
const Segment SGI197141[] = {{&S69031[44], 29}, {NULL}};
const Segment SGI197142[] = {{&S69031[30], 6}, {NULL}};
const Segment SGI197143[] = {{&S69031[122], 3}, {NULL}};
const Segment SGI197144[] = {{&S69031[87], 6}, {NULL}};
const Segment SGI197145[] = {{&S69032[38], 17}, {NULL}};
const Segment SGI197146[] = {{&S69032[31], 6}, {NULL}};
const Segment SGI197147[] = {{&S79764[44], 22}, {NULL}};
const Segment SGI197148[] = {{&S79764[44], 32}, {NULL}};
const Segment SGI197149[] = {{&S79764[30], 6}, {NULL}};
const Segment SGI197150[] = {{&S79764[31], 3}, {NULL}};
const Segment SGI197151[] = {{&S79764[90], 6}, {NULL}};
const Segment SGI197152[] = {{&S79766[34], 13}, {NULL}};
const Segment SGI197153[] = {{&S79766[27], 6}, {NULL}};
const Segment SGI197154[] = {{&S79766[100], 9}, {NULL}};
const Segment SGI197155[] = {{&S79766[73], 6}, {NULL}};
const Segment SGI197156[] = {{&S73663[44], 6}, {NULL}};
const Segment SGI197157[] = {{&S73663[44], 18}, {NULL}};
const Segment SGI197158[] = {{&S73663[30], 6}, {NULL}};
const Segment SGI197159[] = {{&S73663[111], 3}, {NULL}};
const Segment SGI197160[] = {{&S73663[76], 6}, {NULL}};
const Segment SGI197161[] = {{&S79767[34], 5}, {NULL}};
const Segment SGI197162[] = {{&S79767[27], 6}, {NULL}};
const Segment SGI197163[] = {{&S79767[92], 9}, {NULL}};
const Segment SGI197164[] = {{&S79767[65], 6}, {NULL}};
const Segment SGI197165[] = {{&S79768[27], 6}, {NULL}};
const Segment SGI197166[] = {{&S73665[34], 10}, {NULL}};
const Segment SGI197167[] = {{&S59168[38], 11}, {NULL}};
const Segment SGI197168[] = {{&S73665[27], 6}, {NULL}};
const Segment SGI197169[] = {{&S59168[31], 6}, {NULL}};
const Segment SGI197170[] = {{&S79768[104], 9}, {NULL}};
const Segment SGI197171[] = {{&S79768[77], 6}, {NULL}};
const Segment SGI197172[] = {{&S73665[97], 9}, {NULL}};
const Segment SGI197173[] = {{&S73665[70], 6}, {NULL}};
const Segment SGI197174[] = {{&S59169[44], 11}, {NULL}};
const Segment SGI197175[] = {{&S59169[44], 22}, {NULL}};
const Segment SGI197176[] = {{&S59169[30], 6}, {NULL}};
const Segment SGI197177[] = {{&S59169[115], 3}, {NULL}};
const Segment SGI197178[] = {{&S59169[80], 6}, {NULL}};
const Segment SGI197179[] = {{&S79776[34], 15}, {NULL}};
const Segment SGI197180[] = {{&S79776[27], 6}, {NULL}};
const Segment SGI197181[] = {{&S79776[102], 9}, {NULL}};
const Segment SGI197182[] = {{&S79776[75], 6}, {NULL}};
const Segment SGI197183[] = {{&S69054[45], 10}, {NULL}};
const Segment SGI197184[] = {{&S69054[30], 6}, {NULL}};
const Segment SGI197185[] = {{&S79777[34], 15}, {NULL}};
const Segment SGI197186[] = {{&S79777[27], 6}, {NULL}};
const Segment SGI197187[] = {{&S69054[115], 3}, {NULL}};
const Segment SGI197188[] = {{&S69054[81], 5}, {NULL}};
const Segment SGI197189[] = {{&S79777[102], 9}, {NULL}};
const Segment SGI197190[] = {{&S69054[139], 13}, {NULL}};
const Segment SGI197191[] = {{&S79777[75], 6}, {NULL}};
const Segment SGI197192[] = {{&S79778[34], 11}, {NULL}};
const Segment SGI197193[] = {{&S79778[27], 6}, {NULL}};
const Segment SGI197194[] = {{&S79778[98], 9}, {NULL}};
const Segment SGI197195[] = {{&S79778[71], 6}, {NULL}};
const Segment SGI197196[] = {{&S79779[34], 5}, {NULL}};
const Segment SGI197197[] = {{&S79779[27], 6}, {NULL}};
const Segment SGI197198[] = {{&S79779[92], 9}, {NULL}};
const Segment SGI197199[] = {{&S79779[65], 6}, {NULL}};
const Segment SGI197200[] = {{&S79780[34], 13}, {NULL}};
const Segment SGI197201[] = {{&S79780[27], 6}, {NULL}};
const Segment SGI197202[] = {{&S79780[100], 9}, {NULL}};
const Segment SGI197203[] = {{&S79780[73], 6}, {NULL}};
const Segment SGI197204[] = {{&S73684[44], 24}, {NULL}};
const Segment SGI197205[] = {{&S73684[44], 36}, {NULL}};
const Segment SGI197206[] = {{&S73684[30], 6}, {NULL}};
const Segment SGI197207[] = {{&S73684[145], 3}, {NULL}};
const Segment SGI197208[] = {{&S73684[110], 6}, {NULL}};
const Segment SGI197209[] = {{&S73684[82], 8}, {NULL}};
const Segment SGI197210[] = {{&S73685[44], 24}, {NULL}};
const Segment SGI197211[] = {{&S73685[44], 36}, {NULL}};
const Segment SGI197212[] = {{&S73685[30], 6}, {NULL}};
const Segment SGI197213[] = {{&S73685[146], 3}, {NULL}};
const Segment SGI197214[] = {{&S73685[111], 6}, {NULL}};
const Segment SGI197215[] = {{&S73685[82], 8}, {NULL}};
const Segment SGI197216[] = {{&S69071[34], 10}, {NULL}};
const Segment SGI197217[] = {{&S69071[27], 6}, {NULL}};
const Segment SGI197218[] = {{&S79789[44], 9}, {NULL}};
const Segment SGI197219[] = {{&S69071[97], 9}, {NULL}};
const Segment SGI197220[] = {{&S79789[44], 20}, {NULL}};
const Segment SGI197221[] = {{&S69071[70], 6}, {NULL}};
const Segment SGI197222[] = {{&S79789[30], 6}, {NULL}};
const Segment SGI197223[] = {{&S79789[31], 3}, {NULL}};
const Segment SGI197224[] = {{&S79789[78], 6}, {NULL}};
const Segment SGI197225[] = {{&S79790[44], 9}, {NULL}};
const Segment SGI197226[] = {{&S69073[44], 23}, {NULL}};
const Segment SGI197227[] = {{&S79790[44], 20}, {NULL}};
const Segment SGI197228[] = {{&S79790[30], 6}, {NULL}};
const Segment SGI197229[] = {{&S69073[44], 35}, {NULL}};
const Segment SGI197230[] = {{&S69073[30], 6}, {NULL}};
const Segment SGI197231[] = {{&S79790[31], 3}, {NULL}};
const Segment SGI197232[] = {{&S69073[128], 3}, {NULL}};
const Segment SGI197233[] = {{&S79790[78], 6}, {NULL}};
const Segment SGI197234[] = {{&S69073[93], 6}, {NULL}};
const Segment SGI197235[] = {{&S79791[38], 9}, {NULL}};
const Segment SGI197236[] = {{&S79791[31], 6}, {NULL}};
const Segment SGI197237[] = {{&S68212[34], 9}, {NULL}};
const Segment SGI197238[] = {{&S68212[27], 6}, {NULL}};
const Segment SGI197239[] = {{&S68212[96], 9}, {NULL}};
const Segment SGI197240[] = {{&S68212[69], 6}, {NULL}};
const Segment SGI197241[] = {{&S79805[34], 9}, {NULL}};
const Segment SGI197242[] = {{&S79805[27], 6}, {NULL}};
const Segment SGI197243[] = {{&S79805[103], 9}, {NULL}};
const Segment SGI197244[] = {{&S79805[76], 6}, {NULL}};
const Segment SGI197245[] = {{&S59228[44], 8}, {NULL}};
const Segment SGI197246[] = {{&S59228[44], 20}, {NULL}};
const Segment SGI197247[] = {{&S59228[30], 6}, {NULL}};
const Segment SGI197248[] = {{&S59228[113], 3}, {NULL}};
const Segment SGI197249[] = {{&S59228[78], 6}, {NULL}};
const Segment SGI197250[] = {{&S69117[34], 6}, {NULL}};
const Segment SGI197251[] = {{&S69117[27], 6}, {NULL}};
const Segment SGI197252[] = {{&S69117[93], 9}, {NULL}};
const Segment SGI197253[] = {{&S59243[34], 8}, {NULL}};
const Segment SGI197254[] = {{&S69117[66], 6}, {NULL}};
const Segment SGI197255[] = {{&S59243[27], 6}, {NULL}};
const Segment SGI197256[] = {{&S59243[103], 9}, {NULL}};
const Segment SGI197257[] = {{&S59243[76], 6}, {NULL}};
const Segment SGI197258[] = {{&S79834[34], 6}, {NULL}};
const Segment SGI197259[] = {{&S79834[27], 6}, {NULL}};
const Segment SGI197260[] = {{&S79834[93], 9}, {NULL}};
const Segment SGI197261[] = {{&S79834[66], 6}, {NULL}};
const Segment SGI197262[] = {{&S79835[34], 6}, {NULL}};
const Segment SGI197263[] = {{&S79835[27], 6}, {NULL}};
const Segment SGI197264[] = {{&S79835[93], 9}, {NULL}};
const Segment SGI197265[] = {{&S79835[66], 6}, {NULL}};
const Segment SGI197266[] = {{&S79836[44], 10}, {NULL}};
const Segment SGI197267[] = {{&S79836[44], 22}, {NULL}};
const Segment SGI197268[] = {{&S79836[30], 6}, {NULL}};
const Segment SGI197269[] = {{&S79836[31], 3}, {NULL}};
const Segment SGI197270[] = {{&S79836[80], 6}, {NULL}};
const Segment SGI197271[] = {{&S79837[44], 10}, {NULL}};
const Segment SGI197272[] = {{&S79837[44], 22}, {NULL}};
const Segment SGI197273[] = {{&S79837[30], 6}, {NULL}};
const Segment SGI197274[] = {{&S79837[83], 6}, {NULL}};
const Segment SGI197275[] = {{&S79838[38], 10}, {NULL}};
const Segment SGI197276[] = {{&S79838[31], 6}, {NULL}};
const Segment SGI197277[] = {{&S79855[44], 15}, {NULL}};
const Segment SGI197278[] = {{&S79855[44], 27}, {NULL}};
const Segment SGI197279[] = {{&S79855[30], 6}, {NULL}};
const Segment SGI197280[] = {{&S79855[31], 3}, {NULL}};
const Segment SGI197281[] = {{&S79855[85], 6}, {NULL}};
const Segment SGI197282[] = {{&S79856[34], 7}, {NULL}};
const Segment SGI197283[] = {{&S79856[27], 6}, {NULL}};
const Segment SGI197284[] = {{&S79856[100], 9}, {NULL}};
const Segment SGI197285[] = {{&S79856[73], 6}, {NULL}};
const Segment SGI197286[] = {{&S79856[141], 8}, {NULL}};
const Segment SGI197287[] = {{&S73769[34], 10}, {NULL}};
const Segment SGI197288[] = {{&S73769[27], 6}, {NULL}};
const Segment SGI197289[] = {{&S73769[97], 9}, {NULL}};
const Segment SGI197290[] = {{&S73769[70], 6}, {NULL}};
const Segment SGI197291[] = {{&S73773[34], 10}, {NULL}};
const Segment SGI197292[] = {{&S73773[27], 6}, {NULL}};
const Segment SGI197293[] = {{&S73773[97], 9}, {NULL}};
const Segment SGI197294[] = {{&S73773[70], 6}, {NULL}};
const Segment SGI197295[] = {{&S69161[34], 9}, {NULL}};
const Segment SGI197296[] = {{&S69161[27], 6}, {NULL}};
const Segment SGI197297[] = {{&S69161[96], 9}, {NULL}};
const Segment SGI197298[] = {{&S69161[69], 6}, {NULL}};
const Segment SGI197299[] = {{&S69162[34], 11}, {NULL}};
const Segment SGI197300[] = {{&S69162[27], 6}, {NULL}};
const Segment SGI197301[] = {{&S69162[98], 9}, {NULL}};
const Segment SGI197302[] = {{&S69162[71], 6}, {NULL}};
const Segment SGI197303[] = {{&S69171[34], 9}, {NULL}};
const Segment SGI197304[] = {{&S69171[27], 6}, {NULL}};
const Segment SGI197305[] = {{&S69171[96], 9}, {NULL}};
const Segment SGI197306[] = {{&S69171[69], 6}, {NULL}};
const Segment SGI197307[] = {{&S69172[34], 11}, {NULL}};
const Segment SGI197308[] = {{&S69172[27], 6}, {NULL}};
const Segment SGI197309[] = {{&S69172[98], 9}, {NULL}};
const Segment SGI197310[] = {{&S69172[71], 6}, {NULL}};
const Segment SGI197311[] = {{&S59309[34], 9}, {NULL}};
const Segment SGI197312[] = {{&S59309[27], 6}, {NULL}};
const Segment SGI197313[] = {{&S59309[95], 9}, {NULL}};
const Segment SGI197314[] = {{&S59309[68], 6}, {NULL}};
const Segment SGI197315[] = {{&S59309[137], 8}, {NULL}};
const Segment SGI197316[] = {{&S59310[34], 9}, {NULL}};
const Segment SGI197317[] = {{&S59310[27], 6}, {NULL}};
const Segment SGI197318[] = {{&S59310[95], 9}, {NULL}};
const Segment SGI197319[] = {{&S59310[68], 6}, {NULL}};
const Segment SGI197320[] = {{&S59310[136], 8}, {NULL}};
const Segment SGI197321[] = {{&S69181[44], 10}, {NULL}};
const Segment SGI197322[] = {{&S69181[44], 22}, {NULL}};
const Segment SGI197323[] = {{&S69181[30], 6}, {NULL}};
const Segment SGI197324[] = {{&S69181[115], 3}, {NULL}};
const Segment SGI197325[] = {{&S69181[80], 6}, {NULL}};
const Segment SGI197326[] = {{&S69181[143], 3}, {NULL}};
const Segment SGI197327[] = {{&S79895[34], 21}, {NULL}};
const Segment SGI197328[] = {{&S79895[27], 6}, {NULL}};
const Segment SGI197329[] = {{&S79895[108], 9}, {NULL}};
const Segment SGI197330[] = {{&S79895[81], 6}, {NULL}};
const Segment SGI197331[] = {{&S79896[34], 25}, {NULL}};
const Segment SGI197332[] = {{&S79896[27], 6}, {NULL}};
const Segment SGI197333[] = {{&S79896[112], 9}, {NULL}};
const Segment SGI197334[] = {{&S79896[85], 6}, {NULL}};
const Segment SGI197335[] = {{&S59332[34], 10}, {NULL}};
const Segment SGI197336[] = {{&S59332[27], 6}, {NULL}};
const Segment SGI197337[] = {{&S59332[96], 9}, {NULL}};
const Segment SGI197338[] = {{&S59332[69], 6}, {NULL}};
const Segment SGI197339[] = {{&S79915[44], 15}, {NULL}};
const Segment SGI197340[] = {{&S79915[44], 27}, {NULL}};
const Segment SGI197341[] = {{&S79915[30], 6}, {NULL}};
const Segment SGI197342[] = {{&S79915[31], 3}, {NULL}};
const Segment SGI197343[] = {{&S79915[85], 5}, {NULL}};
const Segment SGI197344[] = {{&S79915[122], 14}, {NULL}};
const Segment SGI197345[] = {{&S79917[34], 6}, {NULL}};
const Segment SGI197346[] = {{&S79917[27], 6}, {NULL}};
const Segment SGI197347[] = {{&S79917[93], 9}, {NULL}};
const Segment SGI197348[] = {{&S79917[66], 6}, {NULL}};
const Segment SGI197349[] = {{&S69216[44], 13}, {NULL}};
const Segment SGI197350[] = {{&S69216[44], 25}, {NULL}};
const Segment SGI197351[] = {{&S69216[30], 6}, {NULL}};
const Segment SGI197352[] = {{&S69216[118], 3}, {NULL}};
const Segment SGI197353[] = {{&S69216[83], 6}, {NULL}};
const Segment SGI197354[] = {{&S69217[44], 14}, {NULL}};
const Segment SGI197355[] = {{&S69217[44], 26}, {NULL}};
const Segment SGI197356[] = {{&S69217[30], 6}, {NULL}};
const Segment SGI197357[] = {{&S69217[119], 3}, {NULL}};
const Segment SGI197358[] = {{&S69217[84], 6}, {NULL}};
const Segment SGI197359[] = {{&S69218[44], 14}, {NULL}};
const Segment SGI197360[] = {{&S69218[44], 26}, {NULL}};
const Segment SGI197361[] = {{&S69218[30], 6}, {NULL}};
const Segment SGI197362[] = {{&S69218[87], 6}, {NULL}};
const Segment SGI197363[] = {{&S69219[44], 13}, {NULL}};
const Segment SGI197364[] = {{&S69219[44], 25}, {NULL}};
const Segment SGI197365[] = {{&S69219[30], 6}, {NULL}};
const Segment SGI197366[] = {{&S69219[86], 6}, {NULL}};
const Segment SGI197367[] = {{&S79934[34], 18}, {NULL}};
const Segment SGI197368[] = {{&S79934[27], 6}, {NULL}};
const Segment SGI197369[] = {{&S79934[105], 9}, {NULL}};
const Segment SGI197370[] = {{&S79934[78], 6}, {NULL}};
const Segment SGI197371[] = {{&S59362[34], 18}, {NULL}};
const Segment SGI197372[] = {{&S69226[34], 9}, {NULL}};
const Segment SGI197373[] = {{&S59362[27], 6}, {NULL}};
const Segment SGI197374[] = {{&S69226[27], 6}, {NULL}};
const Segment SGI197375[] = {{&S59362[104], 9}, {NULL}};
const Segment SGI197376[] = {{&S59362[77], 6}, {NULL}};
const Segment SGI197377[] = {{&S69226[96], 9}, {NULL}};
const Segment SGI197378[] = {{&S69226[69], 6}, {NULL}};
const Segment SGI197379[] = {{&S69227[34], 10}, {NULL}};
const Segment SGI197380[] = {{&S69227[27], 6}, {NULL}};
const Segment SGI197381[] = {{&S69227[97], 9}, {NULL}};
const Segment SGI197382[] = {{&S69227[70], 6}, {NULL}};
const Segment SGI197383[] = {{&S69228[34], 9}, {NULL}};
const Segment SGI197384[] = {{&S69228[27], 6}, {NULL}};
const Segment SGI197385[] = {{&S69228[96], 9}, {NULL}};
const Segment SGI197386[] = {{&S69228[69], 6}, {NULL}};
const Segment SGI197387[] = {{&S68366[44], 7}, {NULL}};
const Segment SGI197388[] = {{&S68366[44], 19}, {NULL}};
const Segment SGI197389[] = {{&S68366[30], 6}, {NULL}};
const Segment SGI197390[] = {{&S68366[130], 3}, {NULL}};
const Segment SGI197391[] = {{&S68366[95], 6}, {NULL}};
const Segment SGI197392[] = {{&S68366[65], 8}, {NULL}};
const Segment SGI197393[] = {{&S68367[44], 7}, {NULL}};
const Segment SGI197394[] = {{&S68367[44], 19}, {NULL}};
const Segment SGI197395[] = {{&S68367[30], 6}, {NULL}};
const Segment SGI197396[] = {{&S68367[130], 3}, {NULL}};
const Segment SGI197397[] = {{&S68367[95], 6}, {NULL}};
const Segment SGI197398[] = {{&S68367[65], 8}, {NULL}};
const Segment SGI197399[] = {{&S79949[45], 10}, {NULL}};
const Segment SGI197400[] = {{&S79949[30], 6}, {NULL}};
const Segment SGI197401[] = {{&S79949[31], 3}, {NULL}};
const Segment SGI197402[] = {{&S79949[81], 5}, {NULL}};
const Segment SGI197403[] = {{&S79949[139], 13}, {NULL}};
const Segment SGI197404[] = {{&S68373[34], 10}, {NULL}};
const Segment SGI197405[] = {{&S68373[27], 6}, {NULL}};
const Segment SGI197406[] = {{&S79952[45], 5}, {NULL}};
const Segment SGI197407[] = {{&S79952[30], 6}, {NULL}};
const Segment SGI197408[] = {{&S68373[97], 9}, {NULL}};
const Segment SGI197409[] = {{&S68373[70], 6}, {NULL}};
const Segment SGI197410[] = {{&S79952[31], 3}, {NULL}};
const Segment SGI197411[] = {{&S79952[76], 5}, {NULL}};
const Segment SGI197412[] = {{&S79952[134], 13}, {NULL}};
const Segment SGI197413[] = {{&S79953[45], 13}, {NULL}};
const Segment SGI197414[] = {{&S79953[30], 6}, {NULL}};
const Segment SGI197415[] = {{&S79953[31], 3}, {NULL}};
const Segment SGI197416[] = {{&S79953[84], 5}, {NULL}};
const Segment SGI197417[] = {{&S79953[142], 13}, {NULL}};
const Segment SGI197418[] = {{&S73876[34], 12}, {NULL}};
const Segment SGI197419[] = {{&S73876[27], 6}, {NULL}};
const Segment SGI197420[] = {{&S79960[34], 15}, {NULL}};
const Segment SGI197421[] = {{&S79960[27], 6}, {NULL}};
const Segment SGI197422[] = {{&S73876[99], 9}, {NULL}};
const Segment SGI197423[] = {{&S73876[72], 6}, {NULL}};
const Segment SGI197424[] = {{&S79960[102], 9}, {NULL}};
const Segment SGI197425[] = {{&S79960[75], 6}, {NULL}};
const Segment SGI197426[] = {{&S79961[44], 10}, {NULL}};
const Segment SGI197427[] = {{&S79961[44], 22}, {NULL}};
const Segment SGI197428[] = {{&S79961[30], 6}, {NULL}};
const Segment SGI197429[] = {{&S79961[31], 3}, {NULL}};
const Segment SGI197430[] = {{&S79961[80], 6}, {NULL}};
const Segment SGI197431[] = {{&S69256[44], 4}, {NULL}};
const Segment SGI197432[] = {{&S69256[44], 16}, {NULL}};
const Segment SGI197433[] = {{&S69256[30], 6}, {NULL}};
const Segment SGI197434[] = {{&S69256[125], 3}, {NULL}};
const Segment SGI197435[] = {{&S69256[90], 6}, {NULL}};
const Segment SGI197436[] = {{&S69256[62], 8}, {NULL}};
const Segment SGI197437[] = {{&S69264[34], 10}, {NULL}};
const Segment SGI197438[] = {{&S69264[27], 6}, {NULL}};
const Segment SGI197439[] = {{&S69264[97], 9}, {NULL}};
const Segment SGI197440[] = {{&S69264[70], 6}, {NULL}};
const Segment SGI197441[] = {{&S79974[34], 6}, {NULL}};
const Segment SGI197442[] = {{&S79974[27], 6}, {NULL}};
const Segment SGI197443[] = {{&S79974[93], 9}, {NULL}};
const Segment SGI197444[] = {{&S79974[66], 6}, {NULL}};
const Segment SGI197445[] = {{&S73893[34], 14}, {NULL}};
const Segment SGI197446[] = {{&S73893[27], 6}, {NULL}};
const Segment SGI197447[] = {{&S73893[101], 9}, {NULL}};
const Segment SGI197448[] = {{&S73893[74], 6}, {NULL}};
const Segment SGI197449[] = {{&S73893[142], 8}, {NULL}};
const Segment SGI197450[] = {{&S69270[34], 9}, {NULL}};
const Segment SGI197451[] = {{&S69270[27], 6}, {NULL}};
const Segment SGI197452[] = {{&S69270[96], 9}, {NULL}};
const Segment SGI197453[] = {{&S69270[69], 6}, {NULL}};
const Segment SGI197454[] = {{&S59410[41], 19}, {NULL}};
const Segment SGI197455[] = {{&S59410[41], 30}, {NULL}};
const Segment SGI197456[] = {{&S59410[27], 6}, {NULL}};
const Segment SGI197457[] = {{&S59410[120], 3}, {NULL}};
const Segment SGI197458[] = {{&S79980[34], 20}, {NULL}};
const Segment SGI197459[] = {{&S59410[124], 9}, {NULL}};
const Segment SGI197460[] = {{&S59410[85], 6}, {NULL}};
const Segment SGI197461[] = {{&S79980[27], 6}, {NULL}};
const Segment SGI197462[] = {{&S79980[117], 9}, {NULL}};
const Segment SGI197463[] = {{&S79980[90], 6}, {NULL}};
const Segment SGI197464[] = {{&S69272[44], 10}, {NULL}};
const Segment SGI197465[] = {{&S69272[44], 22}, {NULL}};
const Segment SGI197466[] = {{&S69272[30], 6}, {NULL}};
const Segment SGI197467[] = {{&S69272[136], 3}, {NULL}};
const Segment SGI197468[] = {{&S69272[101], 6}, {NULL}};
const Segment SGI197469[] = {{&S79981[34], 41}, {NULL}};
const Segment SGI197470[] = {{&S79981[27], 6}, {NULL}};
const Segment SGI197471[] = {{&S79981[138], 9}, {NULL}};
const Segment SGI197472[] = {{&S79981[111], 6}, {NULL}};
const Segment SGI197473[] = {{&S79984[34], 10}, {NULL}};
const Segment SGI197474[] = {{&S79984[27], 6}, {NULL}};
const Segment SGI197475[] = {{&S79984[97], 9}, {NULL}};
const Segment SGI197476[] = {{&S79984[70], 6}, {NULL}};
const Segment SGI197477[] = {{&S79985[44], 15}, {NULL}};
const Segment SGI197478[] = {{&S79985[44], 27}, {NULL}};
const Segment SGI197479[] = {{&S79985[30], 6}, {NULL}};
const Segment SGI197480[] = {{&S79985[85], 6}, {NULL}};
const Segment SGI197481[] = {{&S79985[112], 13}, {NULL}};
const Segment SGI197482[] = {{&S69282[34], 10}, {NULL}};
const Segment SGI197483[] = {{&S69282[27], 6}, {NULL}};
const Segment SGI197484[] = {{&S69282[97], 9}, {NULL}};
const Segment SGI197485[] = {{&S69282[70], 6}, {NULL}};
const Segment SGI197486[] = {{&S69283[34], 9}, {NULL}};
const Segment SGI197487[] = {{&S69283[27], 6}, {NULL}};
const Segment SGI197488[] = {{&S69283[96], 9}, {NULL}};
const Segment SGI197489[] = {{&S69283[69], 6}, {NULL}};
const Segment SGI197490[] = {{&S69284[34], 10}, {NULL}};
const Segment SGI197491[] = {{&S69284[27], 6}, {NULL}};
const Segment SGI197492[] = {{&S69284[97], 9}, {NULL}};
const Segment SGI197493[] = {{&S69284[70], 6}, {NULL}};
const Segment SGI197494[] = {{&S69285[34], 10}, {NULL}};
const Segment SGI197495[] = {{&S69285[27], 6}, {NULL}};
const Segment SGI197496[] = {{&S69285[97], 9}, {NULL}};
const Segment SGI197497[] = {{&S69285[70], 6}, {NULL}};
const Segment SGI197498[] = {{&S79994[44], 13}, {NULL}};
const Segment SGI197499[] = {{&S79994[44], 25}, {NULL}};
const Segment SGI197500[] = {{&S79994[30], 6}, {NULL}};
const Segment SGI197501[] = {{&S79994[31], 3}, {NULL}};
const Segment SGI197502[] = {{&S79994[83], 6}, {NULL}};
const Segment SGI197503[] = {{&S79998[34], 7}, {NULL}};
const Segment SGI197504[] = {{&S79998[27], 6}, {NULL}};
const Segment SGI197505[] = {{&S79998[94], 9}, {NULL}};
const Segment SGI197506[] = {{&S79998[67], 6}, {NULL}};
const Segment SGI197507[] = {{&S73923[34], 10}, {NULL}};
const Segment SGI197508[] = {{&S73923[27], 6}, {NULL}};
const Segment SGI197509[] = {{&S73923[104], 9}, {NULL}};
const Segment SGI197510[] = {{&S73923[77], 6}, {NULL}};
const Segment SGI197511[] = {{&S73925[44], 6}, {NULL}};
const Segment SGI197512[] = {{&S73925[44], 18}, {NULL}};
const Segment SGI197513[] = {{&S73925[30], 6}, {NULL}};
const Segment SGI197514[] = {{&S73925[127], 3}, {NULL}};
const Segment SGI197515[] = {{&S73925[92], 6}, {NULL}};
const Segment SGI197516[] = {{&S73925[64], 8}, {NULL}};
const Segment SGI197517[] = {{&S73928[34], 19}, {NULL}};
const Segment SGI197518[] = {{&S73928[27], 6}, {NULL}};
const Segment SGI197519[] = {{&S80007[59], 15}, {NULL}};
const Segment SGI197520[] = {{&S80007[59], 33}, {NULL}};
const Segment SGI197521[] = {{&S80007[45], 6}, {NULL}};
const Segment SGI197522[] = {{&S73928[106], 9}, {NULL}};
const Segment SGI197523[] = {{&S80007[46], 3}, {NULL}};
const Segment SGI197524[] = {{&S73928[79], 6}, {NULL}};
const Segment SGI197525[] = {{&S80007[106], 6}, {NULL}};
const Segment SGI197526[] = {{&S69303[44], 7}, {NULL}};
const Segment SGI197527[] = {{&S69303[44], 19}, {NULL}};
const Segment SGI197528[] = {{&S69303[30], 6}, {NULL}};
const Segment SGI197529[] = {{&S80007[0], 14}, {NULL}};
const Segment SGI197530[] = {{&S69303[130], 3}, {NULL}};
const Segment SGI197531[] = {{&S69303[95], 6}, {NULL}};
const Segment SGI197532[] = {{&S69303[65], 8}, {NULL}};
const Segment SGI197533[] = {{&S80008[44], 7}, {NULL}};
const Segment SGI197534[] = {{&S80008[44], 19}, {NULL}};
const Segment SGI197535[] = {{&S80008[30], 6}, {NULL}};
const Segment SGI197536[] = {{&S80008[31], 3}, {NULL}};
const Segment SGI197537[] = {{&S80008[77], 6}, {NULL}};
const Segment SGI197538[] = {{&S80014[44], 8}, {NULL}};
const Segment SGI197539[] = {{&S80014[44], 20}, {NULL}};
const Segment SGI197540[] = {{&S80014[30], 6}, {NULL}};
const Segment SGI197541[] = {{&S80014[31], 3}, {NULL}};
const Segment SGI197542[] = {{&S80014[78], 6}, {NULL}};
const Segment SGI197543[] = {{&S80015[44], 9}, {NULL}};
const Segment SGI197544[] = {{&S80015[44], 21}, {NULL}};
const Segment SGI197545[] = {{&S80015[30], 6}, {NULL}};
const Segment SGI197546[] = {{&S80015[31], 3}, {NULL}};
const Segment SGI197547[] = {{&S80015[79], 6}, {NULL}};
const Segment SGI197548[] = {{&S73943[34], 10}, {NULL}};
const Segment SGI197549[] = {{&S73943[27], 6}, {NULL}};
const Segment SGI197550[] = {{&S73943[97], 9}, {NULL}};
const Segment SGI197551[] = {{&S73943[70], 6}, {NULL}};
const Segment SGI197552[] = {{&S80024[34], 7}, {NULL}};
const Segment SGI197553[] = {{&S80024[27], 6}, {NULL}};
const Segment SGI197554[] = {{&S80024[99], 9}, {NULL}};
const Segment SGI197555[] = {{&S80024[72], 6}, {NULL}};
const Segment SGI197556[] = {{&S69334[44], 10}, {NULL}};
const Segment SGI197557[] = {{&S69334[44], 22}, {NULL}};
const Segment SGI197558[] = {{&S69334[30], 6}, {NULL}};
const Segment SGI197559[] = {{&S69334[115], 3}, {NULL}};
const Segment SGI197560[] = {{&S69334[80], 6}, {NULL}};
const Segment SGI197561[] = {{&S73961[44], 20}, {NULL}};
const Segment SGI197562[] = {{&S73961[44], 32}, {NULL}};
const Segment SGI197563[] = {{&S73961[30], 6}, {NULL}};
const Segment SGI197564[] = {{&S73961[125], 3}, {NULL}};
const Segment SGI197565[] = {{&S73961[90], 6}, {NULL}};
const Segment SGI197566[] = {{&S69340[34], 18}, {NULL}};
const Segment SGI197567[] = {{&S69340[27], 6}, {NULL}};
const Segment SGI197568[] = {{&S69340[105], 9}, {NULL}};
const Segment SGI197569[] = {{&S69340[78], 6}, {NULL}};
const Segment SGI197570[] = {{&S69341[44], 10}, {NULL}};
const Segment SGI197571[] = {{&S69341[44], 22}, {NULL}};
const Segment SGI197572[] = {{&S69341[30], 6}, {NULL}};
const Segment SGI197573[] = {{&S69341[80], 6}, {NULL}};
const Segment SGI197574[] = {{&S69341[107], 13}, {NULL}};
const Segment SGI197575[] = {{&S80046[44], 10}, {NULL}};
const Segment SGI197576[] = {{&S80046[44], 20}, {NULL}};
const Segment SGI197577[] = {{&S80046[30], 6}, {NULL}};
const Segment SGI197578[] = {{&S80046[31], 3}, {NULL}};
const Segment SGI197579[] = {{&S80046[78], 6}, {NULL}};
const Segment SGI197580[] = {{&S69354[44], 11}, {NULL}};
const Segment SGI197581[] = {{&S69354[44], 23}, {NULL}};
const Segment SGI197582[] = {{&S69354[30], 6}, {NULL}};
const Segment SGI197583[] = {{&S69354[116], 3}, {NULL}};
const Segment SGI197584[] = {{&S69354[81], 6}, {NULL}};
const Segment SGI197585[] = {{&S69355[38], 11}, {NULL}};
const Segment SGI197586[] = {{&S69355[31], 6}, {NULL}};
const Segment SGI197587[] = {{&S80062[44], 8}, {NULL}};
const Segment SGI197588[] = {{&S80062[44], 19}, {NULL}};
const Segment SGI197589[] = {{&S80062[30], 6}, {NULL}};
const Segment SGI197590[] = {{&S80062[31], 3}, {NULL}};
const Segment SGI197591[] = {{&S80062[98], 6}, {NULL}};
const Segment SGI197592[] = {{&S80062[65], 8}, {NULL}};
const Segment SGI197593[] = {{&S68509[44], 10}, {NULL}};
const Segment SGI197594[] = {{&S68509[44], 22}, {NULL}};
const Segment SGI197595[] = {{&S68509[30], 6}, {NULL}};
const Segment SGI197596[] = {{&S68509[133], 3}, {NULL}};
const Segment SGI197597[] = {{&S68509[98], 6}, {NULL}};
const Segment SGI197598[] = {{&S68509[68], 8}, {NULL}};
const Segment SGI197599[] = {{&S68513[34], 6}, {NULL}};
const Segment SGI197600[] = {{&S68513[27], 6}, {NULL}};
const Segment SGI197601[] = {{&S68513[93], 9}, {NULL}};
const Segment SGI197602[] = {{&S68513[66], 6}, {NULL}};
const Segment SGI197603[] = {{&S80074[34], 4}, {NULL}};
const Segment SGI197604[] = {{&S80074[27], 6}, {NULL}};
const Segment SGI197605[] = {{&S80074[91], 9}, {NULL}};
const Segment SGI197606[] = {{&S69376[44], 7}, {NULL}};
const Segment SGI197607[] = {{&S80074[64], 6}, {NULL}};
const Segment SGI197608[] = {{&S69376[44], 19}, {NULL}};
const Segment SGI197609[] = {{&S69376[30], 6}, {NULL}};
const Segment SGI197610[] = {{&S69376[128], 3}, {NULL}};
const Segment SGI197611[] = {{&S69376[93], 6}, {NULL}};
const Segment SGI197612[] = {{&S69376[65], 8}, {NULL}};
const Segment SGI197613[] = {{&S80076[34], 21}, {NULL}};
const Segment SGI197614[] = {{&S80076[27], 6}, {NULL}};
const Segment SGI197615[] = {{&S80076[108], 9}, {NULL}};
const Segment SGI197616[] = {{&S80076[81], 6}, {NULL}};
const Segment SGI197617[] = {{&S80077[34], 19}, {NULL}};
const Segment SGI197618[] = {{&S80077[27], 6}, {NULL}};
const Segment SGI197619[] = {{&S80077[106], 9}, {NULL}};
const Segment SGI197620[] = {{&S80077[79], 6}, {NULL}};
const Segment SGI197621[] = {{&S80079[44], 7}, {NULL}};
const Segment SGI197622[] = {{&S80079[44], 15}, {NULL}};
const Segment SGI197623[] = {{&S80079[30], 6}, {NULL}};
const Segment SGI197624[] = {{&S80079[31], 3}, {NULL}};
const Segment SGI197625[] = {{&S80079[73], 6}, {NULL}};
const Segment SGI197626[] = {{&S80082[45], 7}, {NULL}};
const Segment SGI197627[] = {{&S69390[44], 10}, {NULL}};
const Segment SGI197628[] = {{&S69390[44], 22}, {NULL}};
const Segment SGI197629[] = {{&S80082[30], 6}, {NULL}};
const Segment SGI197630[] = {{&S69390[30], 6}, {NULL}};
const Segment SGI197631[] = {{&S69390[115], 3}, {NULL}};
const Segment SGI197632[] = {{&S80082[31], 3}, {NULL}};
const Segment SGI197633[] = {{&S69390[80], 6}, {NULL}};
const Segment SGI197634[] = {{&S80082[84], 5}, {NULL}};
const Segment SGI197635[] = {{&S80082[142], 13}, {NULL}};
const Segment SGI197636[] = {{&S80414[34], 7}, {NULL}};
const Segment SGI197637[] = {{&S80414[27], 6}, {NULL}};
const Segment SGI197638[] = {{&S80414[67], 6}, {NULL}};
const Segment SGI197639[] = {{&S69391[38], 10}, {NULL}};
const Segment SGI197640[] = {{&S69391[31], 6}, {NULL}};
const Segment SGI197641[] = {{&S80084[34], 7}, {NULL}};
const Segment SGI197642[] = {{&S80084[27], 6}, {NULL}};
const Segment SGI197643[] = {{&S80084[112], 9}, {NULL}};
const Segment SGI197644[] = {{&S80084[85], 6}, {NULL}};
const Segment SGI197645[] = {{&S80085[34], 11}, {NULL}};
const Segment SGI197646[] = {{&S80085[27], 6}, {NULL}};
const Segment SGI197647[] = {{&S80085[110], 9}, {NULL}};
const Segment SGI197648[] = {{&S80085[83], 6}, {NULL}};
const Segment SGI197649[] = {{&S80086[44], 10}, {NULL}};
const Segment SGI197650[] = {{&S80086[44], 22}, {NULL}};
const Segment SGI197651[] = {{&S80086[30], 6}, {NULL}};
const Segment SGI197652[] = {{&S80086[31], 3}, {NULL}};
const Segment SGI197653[] = {{&S80086[80], 6}, {NULL}};
const Segment SGI197654[] = {{&S80086[143], 3}, {NULL}};
const Segment SGI197655[] = {{&S74032[34], 14}, {NULL}};
const Segment SGI197656[] = {{&S74032[27], 6}, {NULL}};
const Segment SGI197657[] = {{&S80089[34], 5}, {NULL}};
const Segment SGI197658[] = {{&S80089[27], 6}, {NULL}};
const Segment SGI197659[] = {{&S74032[101], 9}, {NULL}};
const Segment SGI197660[] = {{&S74032[74], 6}, {NULL}};
const Segment SGI197661[] = {{&S80089[92], 9}, {NULL}};
const Segment SGI197662[] = {{&S80089[65], 6}, {NULL}};
const Segment SGI197663[] = {{&S80090[34], 5}, {NULL}};
const Segment SGI197664[] = {{&S80090[27], 6}, {NULL}};
const Segment SGI197665[] = {{&S80090[92], 9}, {NULL}};
const Segment SGI197666[] = {{&S80090[65], 6}, {NULL}};
const Segment SGI197667[] = {{&S69406[34], 10}, {NULL}};
const Segment SGI197668[] = {{&S69406[27], 6}, {NULL}};
const Segment SGI197669[] = {{&S69406[97], 9}, {NULL}};
const Segment SGI197670[] = {{&S69406[70], 6}, {NULL}};
const Segment SGI197671[] = {{&S69407[34], 11}, {NULL}};
const Segment SGI197672[] = {{&S69407[27], 6}, {NULL}};
const Segment SGI197673[] = {{&S69407[98], 9}, {NULL}};
const Segment SGI197674[] = {{&S69407[71], 6}, {NULL}};
const Segment SGI197675[] = {{&S74037[44], 11}, {NULL}};
const Segment SGI197676[] = {{&S74037[44], 23}, {NULL}};
const Segment SGI197677[] = {{&S74037[30], 6}, {NULL}};
const Segment SGI197678[] = {{&S74037[116], 3}, {NULL}};
const Segment SGI197679[] = {{&S74037[81], 6}, {NULL}};
const Segment SGI197680[] = {{&S69408[34], 10}, {NULL}};
const Segment SGI197681[] = {{&S69408[27], 6}, {NULL}};
const Segment SGI197682[] = {{&S69408[97], 9}, {NULL}};
const Segment SGI197683[] = {{&S69408[70], 6}, {NULL}};
const Segment SGI197684[] = {{&S74038[44], 15}, {NULL}};
const Segment SGI197685[] = {{&S74038[44], 27}, {NULL}};
const Segment SGI197686[] = {{&S74038[30], 6}, {NULL}};
const Segment SGI197687[] = {{&S74038[120], 3}, {NULL}};
const Segment SGI197688[] = {{&S74038[85], 6}, {NULL}};
const Segment SGI197689[] = {{&S59570[44], 10}, {NULL}};
const Segment SGI197690[] = {{&S59570[44], 21}, {NULL}};
const Segment SGI197691[] = {{&S59570[30], 6}, {NULL}};
const Segment SGI197692[] = {{&S59570[132], 3}, {NULL}};
const Segment SGI197693[] = {{&S59570[97], 6}, {NULL}};
const Segment SGI197694[] = {{&S59570[67], 8}, {NULL}};
const Segment SGI197695[] = {{&S59571[44], 10}, {NULL}};
const Segment SGI197696[] = {{&S59571[44], 21}, {NULL}};
const Segment SGI197697[] = {{&S59571[30], 6}, {NULL}};
const Segment SGI197698[] = {{&S59571[132], 3}, {NULL}};
const Segment SGI197699[] = {{&S59571[97], 6}, {NULL}};
const Segment SGI197700[] = {{&S59571[67], 8}, {NULL}};
const Segment SGI197701[] = {{&S59572[44], 10}, {NULL}};
const Segment SGI197702[] = {{&S59572[44], 22}, {NULL}};
const Segment SGI197703[] = {{&S59572[30], 6}, {NULL}};
const Segment SGI197704[] = {{&S80104[34], 7}, {NULL}};
const Segment SGI197705[] = {{&S80104[27], 6}, {NULL}};
const Segment SGI197706[] = {{&S59572[133], 3}, {NULL}};
const Segment SGI197707[] = {{&S59572[98], 6}, {NULL}};
const Segment SGI197708[] = {{&S59572[68], 8}, {NULL}};
const Segment SGI197709[] = {{&S80104[99], 9}, {NULL}};
const Segment SGI197710[] = {{&S80104[72], 6}, {NULL}};
const Segment SGI197711[] = {{&S80104[140], 8}, {NULL}};
const Segment SGI197712[] = {{&S69419[44], 12}, {NULL}};
const Segment SGI197713[] = {{&S69419[44], 24}, {NULL}};
const Segment SGI197714[] = {{&S69419[30], 6}, {NULL}};
const Segment SGI197715[] = {{&S69419[133], 3}, {NULL}};
const Segment SGI197716[] = {{&S69419[98], 6}, {NULL}};
const Segment SGI197717[] = {{&S69419[70], 8}, {NULL}};
const Segment SGI197718[] = {{&S69427[34], 10}, {NULL}};
const Segment SGI197719[] = {{&S69427[27], 6}, {NULL}};
const Segment SGI197720[] = {{&S69427[97], 9}, {NULL}};
const Segment SGI197721[] = {{&S69427[70], 6}, {NULL}};
const Segment SGI197722[] = {{&S69427[138], 8}, {NULL}};
const Segment SGI197723[] = {{&S74076[44], 7}, {NULL}};
const Segment SGI197724[] = {{&S74076[44], 19}, {NULL}};
const Segment SGI197725[] = {{&S74076[30], 6}, {NULL}};
const Segment SGI197726[] = {{&S74076[128], 3}, {NULL}};
const Segment SGI197727[] = {{&S74076[93], 6}, {NULL}};
const Segment SGI197728[] = {{&S74076[65], 8}, {NULL}};
const Segment SGI197729[] = {{&S80145[44], 9}, {NULL}};
const Segment SGI197730[] = {{&S80145[44], 21}, {NULL}};
const Segment SGI197731[] = {{&S80145[30], 6}, {NULL}};
const Segment SGI197732[] = {{&S80145[31], 3}, {NULL}};
const Segment SGI197733[] = {{&S80145[79], 6}, {NULL}};
const Segment SGI197734[] = {{&S80146[44], 9}, {NULL}};
const Segment SGI197735[] = {{&S80146[44], 21}, {NULL}};
const Segment SGI197736[] = {{&S80146[30], 6}, {NULL}};
const Segment SGI197737[] = {{&S69457[34], 10}, {NULL}};
const Segment SGI197738[] = {{&S69457[27], 6}, {NULL}};
const Segment SGI197739[] = {{&S80146[82], 6}, {NULL}};
const Segment SGI197740[] = {{&S69457[97], 9}, {NULL}};
const Segment SGI197741[] = {{&S69457[70], 6}, {NULL}};
const Segment SGI197742[] = {{&S74081[44], 9}, {NULL}};
const Segment SGI197743[] = {{&S74081[44], 21}, {NULL}};
const Segment SGI197744[] = {{&S74081[30], 6}, {NULL}};
const Segment SGI197745[] = {{&S74081[114], 3}, {NULL}};
const Segment SGI197746[] = {{&S74081[79], 6}, {NULL}};
const Segment SGI197747[] = {{&S69458[34], 9}, {NULL}};
const Segment SGI197748[] = {{&S69458[27], 6}, {NULL}};
const Segment SGI197749[] = {{&S69458[96], 9}, {NULL}};
const Segment SGI197750[] = {{&S69458[69], 6}, {NULL}};
const Segment SGI197751[] = {{&S80153[34], 18}, {NULL}};
const Segment SGI197752[] = {{&S80153[27], 6}, {NULL}};
const Segment SGI197753[] = {{&S80153[105], 9}, {NULL}};
const Segment SGI197754[] = {{&S80153[78], 6}, {NULL}};
const Segment SGI197755[] = {{&S80158[44], 11}, {NULL}};
const Segment SGI197756[] = {{&S80158[44], 23}, {NULL}};
const Segment SGI197757[] = {{&S80158[30], 6}, {NULL}};
const Segment SGI197758[] = {{&S80158[31], 3}, {NULL}};
const Segment SGI197759[] = {{&S80158[81], 6}, {NULL}};
const Segment SGI197760[] = {{&S74095[44], 10}, {NULL}};
const Segment SGI197761[] = {{&S74095[44], 22}, {NULL}};
const Segment SGI197762[] = {{&S74095[30], 6}, {NULL}};
const Segment SGI197763[] = {{&S74095[115], 3}, {NULL}};
const Segment SGI197764[] = {{&S74095[80], 6}, {NULL}};
const Segment SGI197765[] = {{&S68618[44], 6}, {NULL}};
const Segment SGI197766[] = {{&S68618[44], 18}, {NULL}};
const Segment SGI197767[] = {{&S68618[30], 6}, {NULL}};
const Segment SGI197768[] = {{&S80161[44], 11}, {NULL}};
const Segment SGI197769[] = {{&S80161[44], 23}, {NULL}};
const Segment SGI197770[] = {{&S80161[30], 6}, {NULL}};
const Segment SGI197771[] = {{&S68618[129], 3}, {NULL}};
const Segment SGI197772[] = {{&S68618[94], 6}, {NULL}};
const Segment SGI197773[] = {{&S80161[31], 3}, {NULL}};
const Segment SGI197774[] = {{&S68618[64], 8}, {NULL}};
const Segment SGI197775[] = {{&S80161[81], 6}, {NULL}};
const Segment SGI197776[] = {{&S68620[44], 6}, {NULL}};
const Segment SGI197777[] = {{&S68620[44], 18}, {NULL}};
const Segment SGI197778[] = {{&S68620[30], 6}, {NULL}};
const Segment SGI197779[] = {{&S68620[129], 3}, {NULL}};
const Segment SGI197780[] = {{&S68620[94], 6}, {NULL}};
const Segment SGI197781[] = {{&S68620[64], 8}, {NULL}};
const Segment SGI197782[] = {{&S69478[34], 11}, {NULL}};
const Segment SGI197783[] = {{&S69478[27], 6}, {NULL}};
const Segment SGI197784[] = {{&S69478[98], 9}, {NULL}};
const Segment SGI197785[] = {{&S69478[71], 6}, {NULL}};
const Segment SGI197786[] = {{&S69479[34], 11}, {NULL}};
const Segment SGI197787[] = {{&S69479[27], 6}, {NULL}};
const Segment SGI197788[] = {{&S69479[98], 9}, {NULL}};
const Segment SGI197789[] = {{&S69479[71], 6}, {NULL}};
const Segment SGI197790[] = {{&S80167[34], 11}, {NULL}};
const Segment SGI197791[] = {{&S80167[27], 6}, {NULL}};
const Segment SGI197792[] = {{&S80167[98], 9}, {NULL}};
const Segment SGI197793[] = {{&S80167[71], 6}, {NULL}};
const Segment SGI197794[] = {{&S69480[34], 10}, {NULL}};
const Segment SGI197795[] = {{&S69480[27], 6}, {NULL}};
const Segment SGI197796[] = {{&S69480[97], 9}, {NULL}};
const Segment SGI197797[] = {{&S69480[70], 6}, {NULL}};
const Segment SGI197798[] = {{&S69480[131], 8}, {NULL}};
const Segment SGI197799[] = {{&S80169[34], 11}, {NULL}};
const Segment SGI197800[] = {{&S80169[27], 6}, {NULL}};
const Segment SGI197801[] = {{&S80169[98], 9}, {NULL}};
const Segment SGI197802[] = {{&S80169[71], 6}, {NULL}};
const Segment SGI197803[] = {{&S69482[34], 10}, {NULL}};
const Segment SGI197804[] = {{&S69482[27], 6}, {NULL}};
const Segment SGI197805[] = {{&S69482[97], 9}, {NULL}};
const Segment SGI197806[] = {{&S69482[70], 6}, {NULL}};
const Segment SGI197807[] = {{&S69483[34], 7}, {NULL}};
const Segment SGI197808[] = {{&S69483[27], 6}, {NULL}};
const Segment SGI197809[] = {{&S69483[98], 9}, {NULL}};
const Segment SGI197810[] = {{&S69483[71], 6}, {NULL}};
const Segment SGI197811[] = {{&S80173[44], 9}, {NULL}};
const Segment SGI197812[] = {{&S80173[44], 21}, {NULL}};
const Segment SGI197813[] = {{&S80173[30], 6}, {NULL}};
const Segment SGI197814[] = {{&S80173[31], 3}, {NULL}};
const Segment SGI197815[] = {{&S80173[79], 6}, {NULL}};
const Segment SGI197816[] = {{&S74117[44], 9}, {NULL}};
const Segment SGI197817[] = {{&S74117[44], 21}, {NULL}};
const Segment SGI197818[] = {{&S74117[30], 6}, {NULL}};
const Segment SGI197819[] = {{&S74117[114], 3}, {NULL}};
const Segment SGI197820[] = {{&S74117[79], 6}, {NULL}};
const Segment SGI197821[] = {{&S80177[34], 9}, {NULL}};
const Segment SGI197822[] = {{&S74118[44], 9}, {NULL}};
const Segment SGI197823[] = {{&S74118[44], 21}, {NULL}};
const Segment SGI197824[] = {{&S80177[27], 6}, {NULL}};
const Segment SGI197825[] = {{&S74118[30], 6}, {NULL}};
const Segment SGI197826[] = {{&S74118[114], 3}, {NULL}};
const Segment SGI197827[] = {{&S80177[96], 9}, {NULL}};
const Segment SGI197828[] = {{&S80177[69], 6}, {NULL}};
const Segment SGI197829[] = {{&S74118[79], 6}, {NULL}};
const Segment SGI197830[] = {{&S74119[44], 19}, {NULL}};
const Segment SGI197831[] = {{&S80178[34], 11}, {NULL}};
const Segment SGI197832[] = {{&S74119[44], 31}, {NULL}};
const Segment SGI197833[] = {{&S74119[30], 6}, {NULL}};
const Segment SGI197834[] = {{&S80178[27], 6}, {NULL}};
const Segment SGI197835[] = {{&S74119[124], 3}, {NULL}};
const Segment SGI197836[] = {{&S80178[98], 9}, {NULL}};
const Segment SGI197837[] = {{&S74119[89], 6}, {NULL}};
const Segment SGI197838[] = {{&S80178[71], 6}, {NULL}};
const Segment SGI197839[] = {{&S80179[34], 10}, {NULL}};
const Segment SGI197840[] = {{&S80179[27], 6}, {NULL}};
const Segment SGI197841[] = {{&S80179[97], 9}, {NULL}};
const Segment SGI197842[] = {{&S80179[70], 6}, {NULL}};
const Segment SGI197843[] = {{&S80188[34], 15}, {NULL}};
const Segment SGI197844[] = {{&S80188[27], 6}, {NULL}};
const Segment SGI197845[] = {{&S80188[102], 9}, {NULL}};
const Segment SGI197846[] = {{&S80188[75], 6}, {NULL}};
const Segment SGI197847[] = {{&S69513[44], 17}, {NULL}};
const Segment SGI197848[] = {{&S69513[44], 29}, {NULL}};
const Segment SGI197849[] = {{&S69513[30], 6}, {NULL}};
const Segment SGI197850[] = {{&S69513[138], 3}, {NULL}};
const Segment SGI197851[] = {{&S69513[103], 6}, {NULL}};
const Segment SGI197852[] = {{&S69513[75], 8}, {NULL}};
const Segment SGI197853[] = {{&S69518[44], 13}, {NULL}};
const Segment SGI197854[] = {{&S69518[44], 25}, {NULL}};
const Segment SGI197855[] = {{&S69518[30], 6}, {NULL}};
const Segment SGI197856[] = {{&S69518[134], 3}, {NULL}};
const Segment SGI197857[] = {{&S69518[99], 6}, {NULL}};
const Segment SGI197858[] = {{&S69518[71], 8}, {NULL}};
const Segment SGI197859[] = {{&S69519[44], 12}, {NULL}};
const Segment SGI197860[] = {{&S69519[44], 24}, {NULL}};
const Segment SGI197861[] = {{&S69519[30], 6}, {NULL}};
const Segment SGI197862[] = {{&S69519[133], 3}, {NULL}};
const Segment SGI197863[] = {{&S69519[98], 6}, {NULL}};
const Segment SGI197864[] = {{&S69519[70], 8}, {NULL}};
const Segment SGI197865[] = {{&S74152[44], 8}, {NULL}};
const Segment SGI197866[] = {{&S74152[44], 20}, {NULL}};
const Segment SGI197867[] = {{&S74152[30], 6}, {NULL}};
const Segment SGI197868[] = {{&S74152[113], 3}, {NULL}};
const Segment SGI197869[] = {{&S74152[78], 6}, {NULL}};
const Segment SGI197870[] = {{&S74153[38], 8}, {NULL}};
const Segment SGI197871[] = {{&S74153[31], 6}, {NULL}};
const Segment SGI197872[] = {{&S68683[34], 10}, {NULL}};
const Segment SGI197873[] = {{&S68683[27], 6}, {NULL}};
const Segment SGI197874[] = {{&S68683[97], 9}, {NULL}};
const Segment SGI197875[] = {{&S68683[70], 6}, {NULL}};
const Segment SGI197876[] = {{&S74155[44], 13}, {NULL}};
const Segment SGI197877[] = {{&S74155[44], 25}, {NULL}};
const Segment SGI197878[] = {{&S74155[30], 6}, {NULL}};
const Segment SGI197879[] = {{&S74155[118], 3}, {NULL}};
const Segment SGI197880[] = {{&S74155[83], 6}, {NULL}};
const Segment SGI197881[] = {{&S74156[38], 13}, {NULL}};
const Segment SGI197882[] = {{&S74156[31], 6}, {NULL}};
const Segment SGI197883[] = {{&S69534[34], 10}, {NULL}};
const Segment SGI197884[] = {{&S69534[27], 6}, {NULL}};
const Segment SGI197885[] = {{&S69534[97], 9}, {NULL}};
const Segment SGI197886[] = {{&S69534[70], 6}, {NULL}};
const Segment SGI197887[] = {{&S69535[34], 11}, {NULL}};
const Segment SGI197888[] = {{&S69535[27], 6}, {NULL}};
const Segment SGI197889[] = {{&S69535[98], 9}, {NULL}};
const Segment SGI197890[] = {{&S69535[71], 6}, {NULL}};
const Segment SGI197891[] = {{&S80227[44], 10}, {NULL}};
const Segment SGI197892[] = {{&S80227[44], 22}, {NULL}};
const Segment SGI197893[] = {{&S80227[30], 6}, {NULL}};
const Segment SGI197894[] = {{&S80227[31], 3}, {NULL}};
const Segment SGI197895[] = {{&S80227[80], 6}, {NULL}};
const Segment SGI197896[] = {{&S74164[34], 9}, {NULL}};
const Segment SGI197897[] = {{&S74164[27], 6}, {NULL}};
const Segment SGI197898[] = {{&S74164[96], 9}, {NULL}};
const Segment SGI197899[] = {{&S74164[69], 6}, {NULL}};
const Segment SGI197900[] = {{&S80228[44], 10}, {NULL}};
const Segment SGI197901[] = {{&S80228[44], 22}, {NULL}};
const Segment SGI197902[] = {{&S80228[30], 6}, {NULL}};
const Segment SGI197903[] = {{&S80228[83], 6}, {NULL}};
const Segment SGI197904[] = {{&S68699[44], 6}, {NULL}};
const Segment SGI197905[] = {{&S68699[44], 20}, {NULL}};
const Segment SGI197906[] = {{&S68699[30], 6}, {NULL}};
const Segment SGI197907[] = {{&S68699[113], 3}, {NULL}};
const Segment SGI197908[] = {{&S68699[78], 6}, {NULL}};
const Segment SGI197909[] = {{&S68700[38], 6}, {NULL}};
const Segment SGI197910[] = {{&S68700[31], 6}, {NULL}};
const Segment SGI197911[] = {{&S80233[44], 8}, {NULL}};
const Segment SGI197912[] = {{&S80233[44], 26}, {NULL}};
const Segment SGI197913[] = {{&S80233[30], 6}, {NULL}};
const Segment SGI197914[] = {{&S80233[31], 3}, {NULL}};
const Segment SGI197915[] = {{&S80233[84], 6}, {NULL}};
const Segment SGI197916[] = {{&S80234[44], 6}, {NULL}};
const Segment SGI197917[] = {{&S80234[44], 18}, {NULL}};
const Segment SGI197918[] = {{&S80234[30], 6}, {NULL}};
const Segment SGI197919[] = {{&S80234[31], 3}, {NULL}};
const Segment SGI197920[] = {{&S80234[76], 6}, {NULL}};
const Segment SGI197921[] = {{&S69552[34], 10}, {NULL}};
const Segment SGI197922[] = {{&S69552[27], 6}, {NULL}};
const Segment SGI197923[] = {{&S69552[97], 9}, {NULL}};
const Segment SGI197924[] = {{&S69552[70], 6}, {NULL}};
const Segment SGI197925[] = {{&S74184[44], 15}, {NULL}};
const Segment SGI197926[] = {{&S74184[44], 27}, {NULL}};
const Segment SGI197927[] = {{&S74184[30], 6}, {NULL}};
const Segment SGI197928[] = {{&S74184[85], 6}, {NULL}};
const Segment SGI197929[] = {{&S74184[112], 13}, {NULL}};
const Segment SGI197930[] = {{&S69565[34], 10}, {NULL}};
const Segment SGI197931[] = {{&S69565[27], 6}, {NULL}};
const Segment SGI197932[] = {{&S69565[97], 9}, {NULL}};
const Segment SGI197933[] = {{&S69565[70], 6}, {NULL}};
const Segment SGI197934[] = {{&S69566[34], 11}, {NULL}};
const Segment SGI197935[] = {{&S69566[27], 6}, {NULL}};
const Segment SGI197936[] = {{&S69566[98], 9}, {NULL}};
const Segment SGI197937[] = {{&S69566[71], 6}, {NULL}};
const Segment SGI197938[] = {{&S69567[34], 11}, {NULL}};
const Segment SGI197939[] = {{&S69567[27], 6}, {NULL}};
const Segment SGI197940[] = {{&S69567[98], 9}, {NULL}};
const Segment SGI197941[] = {{&S69567[71], 6}, {NULL}};
const Segment SGI197942[] = {{&S80256[34], 9}, {NULL}};
const Segment SGI197943[] = {{&S80256[27], 6}, {NULL}};
const Segment SGI197944[] = {{&S80256[96], 9}, {NULL}};
const Segment SGI197945[] = {{&S80256[69], 6}, {NULL}};
const Segment SGI197946[] = {{&S69570[34], 25}, {NULL}};
const Segment SGI197947[] = {{&S69570[27], 6}, {NULL}};
const Segment SGI197948[] = {{&S69570[112], 9}, {NULL}};
const Segment SGI197949[] = {{&S69570[85], 6}, {NULL}};
const Segment SGI197950[] = {{&S80257[44], 10}, {NULL}};
const Segment SGI197951[] = {{&S80257[44], 22}, {NULL}};
const Segment SGI197952[] = {{&S80257[30], 6}, {NULL}};
const Segment SGI197953[] = {{&S80257[80], 6}, {NULL}};
const Segment SGI197954[] = {{&S80257[107], 13}, {NULL}};
const Segment SGI197955[] = {{&S69571[34], 16}, {NULL}};
const Segment SGI197956[] = {{&S69571[27], 6}, {NULL}};
const Segment SGI197957[] = {{&S69571[103], 9}, {NULL}};
const Segment SGI197958[] = {{&S69571[76], 6}, {NULL}};
const Segment SGI197959[] = {{&S74198[44], 17}, {NULL}};
const Segment SGI197960[] = {{&S74198[44], 29}, {NULL}};
const Segment SGI197961[] = {{&S74198[30], 6}, {NULL}};
const Segment SGI197962[] = {{&S74198[122], 3}, {NULL}};
const Segment SGI197963[] = {{&S74198[87], 6}, {NULL}};
const Segment SGI197964[] = {{&S80261[34], 9}, {NULL}};
const Segment SGI197965[] = {{&S80261[27], 6}, {NULL}};
const Segment SGI197966[] = {{&S80261[96], 9}, {NULL}};
const Segment SGI197967[] = {{&S80261[69], 6}, {NULL}};
const Segment SGI197968[] = {{&S74201[34], 6}, {NULL}};
const Segment SGI197969[] = {{&S74201[27], 6}, {NULL}};
const Segment SGI197970[] = {{&S74201[93], 9}, {NULL}};
const Segment SGI197971[] = {{&S74201[66], 6}, {NULL}};
const Segment SGI197972[] = {{&S74210[34], 25}, {NULL}};
const Segment SGI197973[] = {{&S74210[27], 6}, {NULL}};
const Segment SGI197974[] = {{&S74210[112], 9}, {NULL}};
const Segment SGI197975[] = {{&S74210[85], 6}, {NULL}};
const Segment SGI197976[] = {{&S69592[44], 10}, {NULL}};
const Segment SGI197977[] = {{&S69592[44], 22}, {NULL}};
const Segment SGI197978[] = {{&S69592[30], 6}, {NULL}};
const Segment SGI197979[] = {{&S69592[115], 3}, {NULL}};
const Segment SGI197980[] = {{&S69592[80], 6}, {NULL}};
const Segment SGI197981[] = {{&S69592[139], 10}, {NULL}};
const Segment SGI197982[] = {{&S69593[44], 18}, {NULL}};
const Segment SGI197983[] = {{&S69593[44], 30}, {NULL}};
const Segment SGI197984[] = {{&S69593[30], 6}, {NULL}};
const Segment SGI197985[] = {{&S74215[34], 7}, {NULL}};
const Segment SGI197986[] = {{&S74215[27], 6}, {NULL}};
const Segment SGI197987[] = {{&S69593[123], 3}, {NULL}};
const Segment SGI197988[] = {{&S69593[88], 6}, {NULL}};
const Segment SGI197989[] = {{&S74215[94], 9}, {NULL}};
const Segment SGI197990[] = {{&S74215[67], 6}, {NULL}};
const Segment SGI197991[] = {{&S69593[147], 10}, {NULL}};
const Segment SGI197992[] = {{&S69594[44], 10}, {NULL}};
const Segment SGI197993[] = {{&S69594[44], 22}, {NULL}};
const Segment SGI197994[] = {{&S69594[30], 6}, {NULL}};
const Segment SGI197995[] = {{&S69594[115], 3}, {NULL}};
const Segment SGI197996[] = {{&S69594[80], 6}, {NULL}};
const Segment SGI197997[] = {{&S69594[139], 10}, {NULL}};
const Segment SGI197998[] = {{&S69595[44], 11}, {NULL}};
const Segment SGI197999[] = {{&S69595[44], 23}, {NULL}};
const Segment SGI198000[] = {{&S69595[30], 6}, {NULL}};
const Segment SGI198001[] = {{&S69595[116], 3}, {NULL}};
const Segment SGI198002[] = {{&S69595[81], 6}, {NULL}};
const Segment SGI198003[] = {{&S69595[140], 10}, {NULL}};
const Segment SGI198004[] = {{&S69599[44], 9}, {NULL}};
const Segment SGI198005[] = {{&S69599[44], 21}, {NULL}};
const Segment SGI198006[] = {{&S69599[30], 6}, {NULL}};
const Segment SGI198007[] = {{&S69599[114], 3}, {NULL}};
const Segment SGI198008[] = {{&S69599[79], 6}, {NULL}};
const Segment SGI198009[] = {{&S69600[38], 9}, {NULL}};
const Segment SGI198010[] = {{&S69600[31], 6}, {NULL}};
const Segment SGI198011[] = {{&S80296[44], 16}, {NULL}};
const Segment SGI198012[] = {{&S80296[44], 26}, {NULL}};
const Segment SGI198013[] = {{&S80296[30], 6}, {NULL}};
const Segment SGI198014[] = {{&S80296[31], 3}, {NULL}};
const Segment SGI198015[] = {{&S80296[84], 6}, {NULL}};
const Segment SGI198016[] = {{&S80297[44], 16}, {NULL}};
const Segment SGI198017[] = {{&S80297[44], 26}, {NULL}};
const Segment SGI198018[] = {{&S80297[30], 6}, {NULL}};
const Segment SGI198019[] = {{&S80297[31], 3}, {NULL}};
const Segment SGI198020[] = {{&S80297[84], 6}, {NULL}};
const Segment SGI198021[] = {{&S80308[34], 13}, {NULL}};
const Segment SGI198022[] = {{&S80308[27], 6}, {NULL}};
const Segment SGI198023[] = {{&S80308[104], 9}, {NULL}};
const Segment SGI198024[] = {{&S80308[77], 6}, {NULL}};
const Segment SGI198025[] = {{&S80310[44], 15}, {NULL}};
const Segment SGI198026[] = {{&S80310[44], 33}, {NULL}};
const Segment SGI198027[] = {{&S80310[30], 6}, {NULL}};
const Segment SGI198028[] = {{&S80310[91], 6}, {NULL}};
const Segment SGI198029[] = {{&S80310[118], 13}, {NULL}};
const Segment SGI198030[] = {{&S59806[44], 10}, {NULL}};
const Segment SGI198031[] = {{&S59806[44], 22}, {NULL}};
const Segment SGI198032[] = {{&S59806[30], 6}, {NULL}};
const Segment SGI198033[] = {{&S59806[115], 3}, {NULL}};
const Segment SGI198034[] = {{&S59806[80], 6}, {NULL}};
const Segment SGI198035[] = {{&S74254[44], 31}, {NULL}};
const Segment SGI198036[] = {{&S59806[133], 8}, {NULL}};
const Segment SGI198037[] = {{&S74254[44], 43}, {NULL}};
const Segment SGI198038[] = {{&S74254[30], 6}, {NULL}};
const Segment SGI198039[] = {{&S80312[44], 10}, {NULL}};
const Segment SGI198040[] = {{&S80312[44], 22}, {NULL}};
const Segment SGI198041[] = {{&S80312[30], 6}, {NULL}};
const Segment SGI198042[] = {{&S74254[136], 3}, {NULL}};
const Segment SGI198043[] = {{&S74254[101], 6}, {NULL}};
const Segment SGI198044[] = {{&S80312[111], 2}, {NULL}};
const Segment SGI198045[] = {{&S80312[80], 5}, {NULL}};
const Segment SGI198046[] = {{&S74261[44], 6}, {NULL}};
const Segment SGI198047[] = {{&S74261[44], 18}, {NULL}};
const Segment SGI198048[] = {{&S74261[30], 6}, {NULL}};
const Segment SGI198049[] = {{&S74261[109], 3}, {NULL}};
const Segment SGI198050[] = {{&S74261[76], 5}, {NULL}};
const Segment SGI198051[] = {{&S74261[113], 14}, {NULL}};
const Segment SGI198052[] = {{&S74262[44], 7}, {NULL}};
const Segment SGI198053[] = {{&S74262[44], 19}, {NULL}};
const Segment SGI198054[] = {{&S74262[30], 6}, {NULL}};
const Segment SGI198055[] = {{&S74262[110], 3}, {NULL}};
const Segment SGI198056[] = {{&S74262[77], 5}, {NULL}};
const Segment SGI198057[] = {{&S74262[114], 14}, {NULL}};
const Segment SGI198058[] = {{&S74263[44], 7}, {NULL}};
const Segment SGI198059[] = {{&S74263[44], 19}, {NULL}};
const Segment SGI198060[] = {{&S74263[30], 6}, {NULL}};
const Segment SGI198061[] = {{&S74263[110], 3}, {NULL}};
const Segment SGI198062[] = {{&S74263[77], 5}, {NULL}};
const Segment SGI198063[] = {{&S74263[114], 14}, {NULL}};
const Segment SGI198064[] = {{&S74264[44], 7}, {NULL}};
const Segment SGI198065[] = {{&S74264[44], 19}, {NULL}};
const Segment SGI198066[] = {{&S74264[30], 6}, {NULL}};
const Segment SGI198067[] = {{&S74264[110], 3}, {NULL}};
const Segment SGI198068[] = {{&S74264[77], 5}, {NULL}};
const Segment SGI198069[] = {{&S74264[114], 14}, {NULL}};
const Segment SGI198070[] = {{&S74265[44], 8}, {NULL}};
const Segment SGI198071[] = {{&S74265[44], 20}, {NULL}};
const Segment SGI198072[] = {{&S74265[30], 6}, {NULL}};
const Segment SGI198073[] = {{&S74265[111], 3}, {NULL}};
const Segment SGI198074[] = {{&S74265[78], 5}, {NULL}};
const Segment SGI198075[] = {{&S74265[115], 14}, {NULL}};
const Segment SGI198076[] = {{&S80329[34], 13}, {NULL}};
const Segment SGI198077[] = {{&S80329[27], 6}, {NULL}};
const Segment SGI198078[] = {{&S80329[100], 9}, {NULL}};
const Segment SGI198079[] = {{&S80329[73], 6}, {NULL}};
const Segment SGI198080[] = {{&S80331[34], 14}, {NULL}};
const Segment SGI198081[] = {{&S80331[27], 6}, {NULL}};
const Segment SGI198082[] = {{&S80331[101], 9}, {NULL}};
const Segment SGI198083[] = {{&S80331[74], 6}, {NULL}};
const Segment SGI198084[] = {{&S59832[44], 5}, {NULL}};
const Segment SGI198085[] = {{&S59832[44], 17}, {NULL}};
const Segment SGI198086[] = {{&S59832[30], 6}, {NULL}};
const Segment SGI198087[] = {{&S59832[128], 3}, {NULL}};
const Segment SGI198088[] = {{&S59832[93], 6}, {NULL}};
const Segment SGI198089[] = {{&S59832[63], 8}, {NULL}};
const Segment SGI198090[] = {{&S59833[44], 9}, {NULL}};
const Segment SGI198091[] = {{&S59833[44], 21}, {NULL}};
const Segment SGI198092[] = {{&S59833[30], 6}, {NULL}};
const Segment SGI198093[] = {{&S59833[132], 3}, {NULL}};
const Segment SGI198094[] = {{&S68828[34], 10}, {NULL}};
const Segment SGI198095[] = {{&S59833[97], 6}, {NULL}};
const Segment SGI198096[] = {{&S68828[27], 6}, {NULL}};
const Segment SGI198097[] = {{&S59833[67], 8}, {NULL}};
const Segment SGI198098[] = {{&S68828[97], 9}, {NULL}};
const Segment SGI198099[] = {{&S68828[70], 6}, {NULL}};
const Segment SGI198100[] = {{&S69656[34], 10}, {NULL}};
const Segment SGI198101[] = {{&S69656[27], 6}, {NULL}};
const Segment SGI198102[] = {{&S69656[97], 9}, {NULL}};
const Segment SGI198103[] = {{&S69656[70], 6}, {NULL}};
const Segment SGI198104[] = {{&S69657[34], 11}, {NULL}};
const Segment SGI198105[] = {{&S69657[27], 6}, {NULL}};
const Segment SGI198106[] = {{&S69657[98], 9}, {NULL}};
const Segment SGI198107[] = {{&S69657[71], 6}, {NULL}};
const Segment SGI198108[] = {{&S69658[34], 9}, {NULL}};
const Segment SGI198109[] = {{&S69658[27], 6}, {NULL}};
const Segment SGI198110[] = {{&S69658[96], 9}, {NULL}};
const Segment SGI198111[] = {{&S80345[44], 7}, {NULL}};
const Segment SGI198112[] = {{&S69658[69], 6}, {NULL}};
const Segment SGI198113[] = {{&S80345[44], 25}, {NULL}};
const Segment SGI198114[] = {{&S80345[30], 6}, {NULL}};
const Segment SGI198115[] = {{&S80345[31], 3}, {NULL}};
const Segment SGI198116[] = {{&S80345[83], 6}, {NULL}};
const Segment SGI198117[] = {{&S80352[34], 10}, {NULL}};
const Segment SGI198118[] = {{&S80352[27], 6}, {NULL}};
const Segment SGI198119[] = {{&S80352[97], 9}, {NULL}};
const Segment SGI198120[] = {{&S80352[70], 6}, {NULL}};
const Segment SGI198121[] = {{&S68851[34], 10}, {NULL}};
const Segment SGI198122[] = {{&S68851[27], 6}, {NULL}};
const Segment SGI198123[] = {{&S68851[97], 9}, {NULL}};
const Segment SGI198124[] = {{&S68851[70], 6}, {NULL}};
const Segment SGI198125[] = {{&S80359[31], 8}, {NULL}};
const Segment SGI198126[] = {{&S80359[54], 3}, {NULL}};
const Segment SGI198127[] = {{&S68852[34], 10}, {NULL}};
const Segment SGI198128[] = {{&S68852[27], 6}, {NULL}};
const Segment SGI198129[] = {{&S68852[97], 9}, {NULL}};
const Segment SGI198130[] = {{&S68852[70], 6}, {NULL}};
const Segment SGI198131[] = {{&S80361[39], 8}, {NULL}};
const Segment SGI198132[] = {{&S80361[103], 3}, {NULL}};
const Segment SGI198133[] = {{&S80361[92], 3}, {NULL}};
const Segment SGI198134[] = {{&S80361[69], 6}, {NULL}};
const Segment SGI198135[] = {{&S80366[44], 6}, {NULL}};
const Segment SGI198136[] = {{&S80366[44], 18}, {NULL}};
const Segment SGI198137[] = {{&S80366[30], 6}, {NULL}};
const Segment SGI198138[] = {{&S80366[31], 3}, {NULL}};
const Segment SGI198139[] = {{&S80366[76], 6}, {NULL}};
const Segment SGI198140[] = {{&S74311[44], 7}, {NULL}};
const Segment SGI198141[] = {{&S74311[44], 19}, {NULL}};
const Segment SGI198142[] = {{&S74311[30], 6}, {NULL}};
const Segment SGI198143[] = {{&S74311[112], 3}, {NULL}};
const Segment SGI198144[] = {{&S74311[77], 6}, {NULL}};
const Segment SGI198145[] = {{&S80370[34], 9}, {NULL}};
const Segment SGI198146[] = {{&S80370[27], 6}, {NULL}};
const Segment SGI198147[] = {{&S80370[96], 9}, {NULL}};
const Segment SGI198148[] = {{&S80370[69], 6}, {NULL}};
const Segment SGI198149[] = {{&S80375[34], 10}, {NULL}};
const Segment SGI198150[] = {{&S80375[27], 6}, {NULL}};
const Segment SGI198151[] = {{&S80375[97], 9}, {NULL}};
const Segment SGI198152[] = {{&S80375[70], 6}, {NULL}};
const Segment SGI198153[] = {{&S80380[34], 12}, {NULL}};
const Segment SGI198154[] = {{&S80380[27], 6}, {NULL}};
const Segment SGI198155[] = {{&S80380[99], 9}, {NULL}};
const Segment SGI198156[] = {{&S80380[72], 6}, {NULL}};
const Segment SGI198157[] = {{&S80382[34], 7}, {NULL}};
const Segment SGI198158[] = {{&S80382[27], 6}, {NULL}};
const Segment SGI198159[] = {{&S80382[99], 9}, {NULL}};
const Segment SGI198160[] = {{&S80382[72], 6}, {NULL}};
const Segment SGI198161[] = {{&S80382[140], 8}, {NULL}};
const Segment SGI198162[] = {{&S74333[34], 12}, {NULL}};
const Segment SGI198163[] = {{&S74333[27], 6}, {NULL}};
const Segment SGI198164[] = {{&S74333[99], 9}, {NULL}};
const Segment SGI198165[] = {{&S74333[72], 6}, {NULL}};
const Segment SGI198166[] = {{&S74333[140], 8}, {NULL}};
const Segment SGI198167[] = {{&S69709[34], 7}, {NULL}};
const Segment SGI198168[] = {{&S69709[27], 6}, {NULL}};
const Segment SGI198169[] = {{&S69709[99], 9}, {NULL}};
const Segment SGI198170[] = {{&S69709[72], 6}, {NULL}};
const Segment SGI198171[] = {{&S69709[140], 8}, {NULL}};
const Segment SGI198172[] = {{&S74340[34], 16}, {NULL}};
const Segment SGI198173[] = {{&S74340[27], 6}, {NULL}};
const Segment SGI198174[] = {{&S74340[103], 9}, {NULL}};
const Segment SGI198175[] = {{&S74340[76], 6}, {NULL}};
const Segment SGI198176[] = {{&S74350[34], 10}, {NULL}};
const Segment SGI198177[] = {{&S74350[27], 6}, {NULL}};
const Segment SGI198178[] = {{&S74350[97], 9}, {NULL}};
const Segment SGI198179[] = {{&S74350[70], 6}, {NULL}};
const Segment SGI198180[] = {{&S80411[44], 12}, {NULL}};
const Segment SGI198181[] = {{&S80411[44], 24}, {NULL}};
const Segment SGI198182[] = {{&S80411[30], 6}, {NULL}};
const Segment SGI198183[] = {{&S80411[31], 3}, {NULL}};
const Segment SGI198184[] = {{&S80411[101], 6}, {NULL}};
const Segment SGI198185[] = {{&S80411[70], 8}, {NULL}};
const Segment SGI198186[] = {{&S80412[44], 12}, {NULL}};
const Segment SGI198187[] = {{&S80412[44], 24}, {NULL}};
const Segment SGI198188[] = {{&S80412[30], 6}, {NULL}};
const Segment SGI198189[] = {{&S80412[31], 3}, {NULL}};
const Segment SGI198190[] = {{&S80412[97], 6}, {NULL}};
const Segment SGI198191[] = {{&S80412[70], 8}, {NULL}};
const Segment SGI198192[] = {{&S69733[44], 10}, {NULL}};
const Segment SGI198193[] = {{&S69733[44], 22}, {NULL}};
const Segment SGI198194[] = {{&S69733[30], 6}, {NULL}};
const Segment SGI198195[] = {{&S69733[115], 3}, {NULL}};
const Segment SGI198196[] = {{&S69733[80], 6}, {NULL}};
const Segment SGI198197[] = {{&S69734[38], 10}, {NULL}};
const Segment SGI198198[] = {{&S69734[31], 6}, {NULL}};
const Segment SGI198199[] = {{&S74356[44], 9}, {NULL}};
const Segment SGI198200[] = {{&S74356[44], 21}, {NULL}};
const Segment SGI198201[] = {{&S74356[30], 6}, {NULL}};
const Segment SGI198202[] = {{&S74356[114], 3}, {NULL}};
const Segment SGI198203[] = {{&S74356[79], 6}, {NULL}};
const Segment SGI198204[] = {{&S69736[44], 10}, {NULL}};
const Segment SGI198205[] = {{&S69736[44], 22}, {NULL}};
const Segment SGI198206[] = {{&S69736[30], 6}, {NULL}};
const Segment SGI198207[] = {{&S69736[115], 3}, {NULL}};
const Segment SGI198208[] = {{&S74358[44], 13}, {NULL}};
const Segment SGI198209[] = {{&S74358[44], 25}, {NULL}};
const Segment SGI198210[] = {{&S69736[80], 6}, {NULL}};
const Segment SGI198211[] = {{&S74358[30], 6}, {NULL}};
const Segment SGI198212[] = {{&S74358[118], 3}, {NULL}};
const Segment SGI198213[] = {{&S74358[83], 6}, {NULL}};
const Segment SGI198214[] = {{&S80421[34], 11}, {NULL}};
const Segment SGI198215[] = {{&S80421[27], 6}, {NULL}};
const Segment SGI198216[] = {{&S80421[98], 9}, {NULL}};
const Segment SGI198217[] = {{&S80421[71], 6}, {NULL}};
const Segment SGI198218[] = {{&S69741[34], 10}, {NULL}};
const Segment SGI198219[] = {{&S80422[34], 11}, {NULL}};
const Segment SGI198220[] = {{&S69741[27], 6}, {NULL}};
const Segment SGI198221[] = {{&S80422[27], 6}, {NULL}};
const Segment SGI198222[] = {{&S69741[97], 9}, {NULL}};
const Segment SGI198223[] = {{&S69741[70], 6}, {NULL}};
const Segment SGI198224[] = {{&S80422[98], 9}, {NULL}};
const Segment SGI198225[] = {{&S80422[71], 6}, {NULL}};
const Segment SGI198226[] = {{&S80424[44], 11}, {NULL}};
const Segment SGI198227[] = {{&S80424[44], 23}, {NULL}};
const Segment SGI198228[] = {{&S80424[30], 6}, {NULL}};
const Segment SGI198229[] = {{&S80424[31], 3}, {NULL}};
const Segment SGI198230[] = {{&S80424[81], 6}, {NULL}};
const Segment SGI198231[] = {{&S68930[34], 10}, {NULL}};
const Segment SGI198232[] = {{&S68930[27], 6}, {NULL}};
const Segment SGI198233[] = {{&S68930[97], 9}, {NULL}};
const Segment SGI198234[] = {{&S68930[70], 6}, {NULL}};
const Segment SGI198235[] = {{&S80426[57], 13}, {NULL}};
const Segment SGI198236[] = {{&S80426[57], 25}, {NULL}};
const Segment SGI198237[] = {{&S80426[43], 6}, {NULL}};
const Segment SGI198238[] = {{&S80426[44], 3}, {NULL}};
const Segment SGI198239[] = {{&S80426[96], 6}, {NULL}};
const Segment SGI198240[] = {{&S80426[0], 12}, {NULL}};
const Segment SGI198241[] = {{&S69753[44], 10}, {NULL}};
const Segment SGI198242[] = {{&S69753[44], 22}, {NULL}};
const Segment SGI198243[] = {{&S69753[30], 6}, {NULL}};
const Segment SGI198244[] = {{&S69753[115], 3}, {NULL}};
const Segment SGI198245[] = {{&S69753[80], 6}, {NULL}};
const Segment SGI198246[] = {{&S80433[34], 23}, {NULL}};
const Segment SGI198247[] = {{&S80433[27], 6}, {NULL}};
const Segment SGI198248[] = {{&S80433[120], 9}, {NULL}};
const Segment SGI198249[] = {{&S80433[93], 6}, {NULL}};
const Segment SGI198250[] = {{&S68948[34], 10}, {NULL}};
const Segment SGI198251[] = {{&S68948[27], 6}, {NULL}};
const Segment SGI198252[] = {{&S68948[97], 9}, {NULL}};
const Segment SGI198253[] = {{&S68948[70], 6}, {NULL}};
const Segment SGI198254[] = {{&S80443[34], 7}, {NULL}};
const Segment SGI198255[] = {{&S80443[27], 6}, {NULL}};
const Segment SGI198256[] = {{&S80443[104], 9}, {NULL}};
const Segment SGI198257[] = {{&S80443[77], 6}, {NULL}};
const Segment SGI198258[] = {{&S74397[44], 10}, {NULL}};
const Segment SGI198259[] = {{&S74397[44], 22}, {NULL}};
const Segment SGI198260[] = {{&S74397[30], 6}, {NULL}};
const Segment SGI198261[] = {{&S74397[113], 3}, {NULL}};
const Segment SGI198262[] = {{&S74397[80], 5}, {NULL}};
const Segment SGI198263[] = {{&S74397[117], 14}, {NULL}};
const Segment SGI198264[] = {{&S68973[44], 10}, {NULL}};
const Segment SGI198265[] = {{&S68973[44], 22}, {NULL}};
const Segment SGI198266[] = {{&S68973[30], 6}, {NULL}};
const Segment SGI198267[] = {{&S68973[115], 3}, {NULL}};
const Segment SGI198268[] = {{&S68973[80], 6}, {NULL}};
const Segment SGI198269[] = {{&S68976[38], 10}, {NULL}};
const Segment SGI198270[] = {{&S68976[31], 6}, {NULL}};
const Segment SGI198271[] = {{&S74413[44], 9}, {NULL}};
const Segment SGI198272[] = {{&S74413[44], 21}, {NULL}};
const Segment SGI198273[] = {{&S74413[30], 6}, {NULL}};
const Segment SGI198274[] = {{&S74413[130], 3}, {NULL}};
const Segment SGI198275[] = {{&S74413[95], 6}, {NULL}};
const Segment SGI198276[] = {{&S74413[67], 8}, {NULL}};
const Segment SGI198277[] = {{&S68978[34], 10}, {NULL}};
const Segment SGI198278[] = {{&S68978[27], 6}, {NULL}};
const Segment SGI198279[] = {{&S68978[97], 9}, {NULL}};
const Segment SGI198280[] = {{&S68978[70], 6}, {NULL}};
const Segment SGI198281[] = {{&S68979[34], 10}, {NULL}};
const Segment SGI198282[] = {{&S68979[27], 6}, {NULL}};
const Segment SGI198283[] = {{&S68979[97], 9}, {NULL}};
const Segment SGI198284[] = {{&S68979[70], 6}, {NULL}};
const Segment SGI198285[] = {{&S80483[45], 7}, {NULL}};
const Segment SGI198286[] = {{&S80483[30], 6}, {NULL}};
const Segment SGI198287[] = {{&S80483[31], 3}, {NULL}};
const Segment SGI198288[] = {{&S80483[83], 5}, {NULL}};
const Segment SGI198289[] = {{&S80483[141], 13}, {NULL}};
const Segment SGI198290[] = {{&S74428[44], 7}, {NULL}};
const Segment SGI198291[] = {{&S74428[44], 19}, {NULL}};
const Segment SGI198292[] = {{&S74428[30], 6}, {NULL}};
const Segment SGI198293[] = {{&S74428[112], 3}, {NULL}};
const Segment SGI198294[] = {{&S74428[77], 6}, {NULL}};
const Segment SGI198295[] = {{&S74439[44], 11}, {NULL}};
const Segment SGI198296[] = {{&S74439[44], 23}, {NULL}};
const Segment SGI198297[] = {{&S74439[30], 6}, {NULL}};
const Segment SGI198298[] = {{&S74439[31], 3}, {NULL}};
const Segment SGI198299[] = {{&S74439[81], 6}, {NULL}};
const Segment SGI198300[] = {{&S80499[34], 24}, {NULL}};
const Segment SGI198301[] = {{&S80499[27], 6}, {NULL}};
const Segment SGI198302[] = {{&S80499[113], 9}, {NULL}};
const Segment SGI198303[] = {{&S80499[86], 6}, {NULL}};
const Segment SGI198304[] = {{&S80500[34], 23}, {NULL}};
const Segment SGI198305[] = {{&S80500[27], 6}, {NULL}};
const Segment SGI198306[] = {{&S80500[112], 9}, {NULL}};
const Segment SGI198307[] = {{&S80500[85], 6}, {NULL}};
const Segment SGI198308[] = {{&S69821[34], 10}, {NULL}};
const Segment SGI198309[] = {{&S69821[27], 6}, {NULL}};
const Segment SGI198310[] = {{&S69821[97], 9}, {NULL}};
const Segment SGI198311[] = {{&S69821[70], 6}, {NULL}};
const Segment SGI198312[] = {{&S69822[34], 5}, {NULL}};
const Segment SGI198313[] = {{&S69822[27], 6}, {NULL}};
const Segment SGI198314[] = {{&S69822[92], 9}, {NULL}};
const Segment SGI198315[] = {{&S69822[65], 6}, {NULL}};
const Segment SGI198316[] = {{&S74451[44], 9}, {NULL}};
const Segment SGI198317[] = {{&S74451[44], 21}, {NULL}};
const Segment SGI198318[] = {{&S74451[30], 6}, {NULL}};
const Segment SGI198319[] = {{&S74451[114], 3}, {NULL}};
const Segment SGI198320[] = {{&S74451[79], 6}, {NULL}};
const Segment SGI198321[] = {{&S69834[44], 10}, {NULL}};
const Segment SGI198322[] = {{&S69834[44], 22}, {NULL}};
const Segment SGI198323[] = {{&S69834[30], 6}, {NULL}};
const Segment SGI198324[] = {{&S69834[131], 3}, {NULL}};
const Segment SGI198325[] = {{&S69834[96], 6}, {NULL}};
const Segment SGI198326[] = {{&S69834[68], 8}, {NULL}};
const Segment SGI198327[] = {{&S69841[44], 14}, {NULL}};
const Segment SGI198328[] = {{&S69841[44], 26}, {NULL}};
const Segment SGI198329[] = {{&S69841[30], 6}, {NULL}};
const Segment SGI198330[] = {{&S69841[119], 3}, {NULL}};
const Segment SGI198331[] = {{&S69841[84], 6}, {NULL}};
const Segment SGI198332[] = {{&S69842[44], 8}, {NULL}};
const Segment SGI198333[] = {{&S69842[44], 20}, {NULL}};
const Segment SGI198334[] = {{&S69842[30], 6}, {NULL}};
const Segment SGI198335[] = {{&S69842[113], 3}, {NULL}};
const Segment SGI198336[] = {{&S69842[78], 6}, {NULL}};
const Segment SGI198337[] = {{&S69843[38], 8}, {NULL}};
const Segment SGI198338[] = {{&S69843[31], 6}, {NULL}};
const Segment SGI198339[] = {{&S80545[44], 10}, {NULL}};
const Segment SGI198340[] = {{&S80545[44], 22}, {NULL}};
const Segment SGI198341[] = {{&S80545[30], 6}, {NULL}};
const Segment SGI198342[] = {{&S80545[31], 3}, {NULL}};
const Segment SGI198343[] = {{&S80545[80], 6}, {NULL}};
const Segment SGI198344[] = {{&S80545[140], 11}, {NULL}};
const Segment SGI198345[] = {{&S69861[34], 7}, {NULL}};
const Segment SGI198346[] = {{&S69861[27], 6}, {NULL}};
const Segment SGI198347[] = {{&S69861[100], 9}, {NULL}};
const Segment SGI198348[] = {{&S69861[73], 6}, {NULL}};
const Segment SGI198349[] = {{&S80547[44], 17}, {NULL}};
const Segment SGI198350[] = {{&S80547[44], 29}, {NULL}};
const Segment SGI198351[] = {{&S80547[30], 6}, {NULL}};
const Segment SGI198352[] = {{&S80547[31], 3}, {NULL}};
const Segment SGI198353[] = {{&S80547[87], 6}, {NULL}};
const Segment SGI198354[] = {{&S80550[44], 17}, {NULL}};
const Segment SGI198355[] = {{&S80550[44], 29}, {NULL}};
const Segment SGI198356[] = {{&S80550[30], 6}, {NULL}};
const Segment SGI198357[] = {{&S80550[31], 3}, {NULL}};
const Segment SGI198358[] = {{&S80550[87], 6}, {NULL}};
const Segment SGI198359[] = {{&S80551[44], 10}, {NULL}};
const Segment SGI198360[] = {{&S80551[44], 22}, {NULL}};
const Segment SGI198361[] = {{&S80551[30], 6}, {NULL}};
const Segment SGI198362[] = {{&S80551[31], 3}, {NULL}};
const Segment SGI198363[] = {{&S80551[80], 6}, {NULL}};
const Segment SGI198364[] = {{&S80559[34], 11}, {NULL}};
const Segment SGI198365[] = {{&S80559[27], 6}, {NULL}};
const Segment SGI198366[] = {{&S80559[98], 9}, {NULL}};
const Segment SGI198367[] = {{&S80559[71], 6}, {NULL}};
const Segment SGI198368[] = {{&S74510[44], 10}, {NULL}};
const Segment SGI198369[] = {{&S74510[44], 22}, {NULL}};
const Segment SGI198370[] = {{&S74510[30], 6}, {NULL}};
const Segment SGI198371[] = {{&S74510[80], 6}, {NULL}};
const Segment SGI198372[] = {{&S74510[107], 13}, {NULL}};
const Segment SGI198373[] = {{&S80564[44], 7}, {NULL}};
const Segment SGI198374[] = {{&S80564[44], 23}, {NULL}};
const Segment SGI198375[] = {{&S80564[30], 6}, {NULL}};
const Segment SGI198376[] = {{&S80564[81], 6}, {NULL}};
const Segment SGI198377[] = {{&S80564[108], 13}, {NULL}};
const Segment SGI198378[] = {{&S80567[34], 10}, {NULL}};
const Segment SGI198379[] = {{&S80567[27], 6}, {NULL}};
const Segment SGI198380[] = {{&S80567[97], 9}, {NULL}};
const Segment SGI198381[] = {{&S80567[70], 6}, {NULL}};
const Segment SGI198382[] = {{&S80567[131], 8}, {NULL}};
const Segment SGI198383[] = {{&S69092[44], 7}, {NULL}};
const Segment SGI198384[] = {{&S69092[44], 19}, {NULL}};
const Segment SGI198385[] = {{&S69092[30], 6}, {NULL}};
const Segment SGI198386[] = {{&S69092[130], 3}, {NULL}};
const Segment SGI198387[] = {{&S69092[95], 6}, {NULL}};
const Segment SGI198388[] = {{&S69092[65], 8}, {NULL}};
const Segment SGI198389[] = {{&S80571[44], 10}, {NULL}};
const Segment SGI198390[] = {{&S80571[44], 22}, {NULL}};
const Segment SGI198391[] = {{&S80571[30], 6}, {NULL}};
const Segment SGI198392[] = {{&S80571[80], 6}, {NULL}};
const Segment SGI198393[] = {{&S80571[107], 13}, {NULL}};
const Segment SGI198394[] = {{&S80572[34], 7}, {NULL}};
const Segment SGI198395[] = {{&S80572[27], 6}, {NULL}};
const Segment SGI198396[] = {{&S80572[112], 9}, {NULL}};
const Segment SGI198397[] = {{&S80572[85], 6}, {NULL}};
const Segment SGI198398[] = {{&S80573[34], 11}, {NULL}};
const Segment SGI198399[] = {{&S80573[27], 6}, {NULL}};
const Segment SGI198400[] = {{&S80573[110], 9}, {NULL}};
const Segment SGI198401[] = {{&S80573[83], 6}, {NULL}};
const Segment SGI198402[] = {{&S69894[34], 9}, {NULL}};
const Segment SGI198403[] = {{&S69894[27], 6}, {NULL}};
const Segment SGI198404[] = {{&S69894[96], 9}, {NULL}};
const Segment SGI198405[] = {{&S60121[44], 23}, {NULL}};
const Segment SGI198406[] = {{&S69894[69], 6}, {NULL}};
const Segment SGI198407[] = {{&S60121[44], 34}, {NULL}};
const Segment SGI198408[] = {{&S60121[30], 6}, {NULL}};
const Segment SGI198409[] = {{&S60121[127], 3}, {NULL}};
const Segment SGI198410[] = {{&S60121[92], 6}, {NULL}};
const Segment SGI198411[] = {{&S60122[44], 23}, {NULL}};
const Segment SGI198412[] = {{&S60122[44], 34}, {NULL}};
const Segment SGI198413[] = {{&S60122[30], 6}, {NULL}};
const Segment SGI198414[] = {{&S60122[127], 3}, {NULL}};
const Segment SGI198415[] = {{&S60122[92], 6}, {NULL}};
const Segment SGI198416[] = {{&S60123[44], 23}, {NULL}};
const Segment SGI198417[] = {{&S60123[44], 34}, {NULL}};
const Segment SGI198418[] = {{&S60123[30], 6}, {NULL}};
const Segment SGI198419[] = {{&S60123[127], 3}, {NULL}};
const Segment SGI198420[] = {{&S60123[92], 6}, {NULL}};
const Segment SGI198421[] = {{&S74541[34], 6}, {NULL}};
const Segment SGI198422[] = {{&S74541[27], 6}, {NULL}};
const Segment SGI198423[] = {{&S74541[93], 9}, {NULL}};
const Segment SGI198424[] = {{&S74541[66], 6}, {NULL}};
const Segment SGI198425[] = {{&S60124[38], 12}, {NULL}};
const Segment SGI198426[] = {{&S60124[31], 6}, {NULL}};
const Segment SGI198427[] = {{&S60125[44], 12}, {NULL}};
const Segment SGI198428[] = {{&S60125[44], 23}, {NULL}};
const Segment SGI198429[] = {{&S60125[30], 6}, {NULL}};
const Segment SGI198430[] = {{&S60125[116], 3}, {NULL}};
const Segment SGI198431[] = {{&S60125[81], 6}, {NULL}};
const Segment SGI198432[] = {{&S60126[44], 12}, {NULL}};
const Segment SGI198433[] = {{&S60126[44], 23}, {NULL}};
const Segment SGI198434[] = {{&S74545[44], 15}, {NULL}};
const Segment SGI198435[] = {{&S60126[30], 6}, {NULL}};
const Segment SGI198436[] = {{&S74545[44], 27}, {NULL}};
const Segment SGI198437[] = {{&S74545[30], 6}, {NULL}};
const Segment SGI198438[] = {{&S60126[116], 3}, {NULL}};
const Segment SGI198439[] = {{&S60126[81], 6}, {NULL}};
const Segment SGI198440[] = {{&S74545[120], 3}, {NULL}};
const Segment SGI198441[] = {{&S74545[85], 6}, {NULL}};
const Segment SGI198442[] = {{&S60127[44], 23}, {NULL}};
const Segment SGI198443[] = {{&S60127[44], 34}, {NULL}};
const Segment SGI198444[] = {{&S60127[30], 6}, {NULL}};
const Segment SGI198445[] = {{&S60127[127], 3}, {NULL}};
const Segment SGI198446[] = {{&S60127[92], 6}, {NULL}};
const Segment SGI198447[] = {{&S72439[32], 12}, {NULL}};
const Segment SGI198448[] = {{&S72439[27], 4}, {NULL}};
const Segment SGI198449[] = {{&S72439[97], 9}, {NULL}};
const Segment SGI198450[] = {{&S72439[70], 6}, {NULL}};
const Segment SGI198451[] = {{&S80608[44], 29}, {NULL}};
const Segment SGI198452[] = {{&S80608[44], 41}, {NULL}};
const Segment SGI198453[] = {{&S80608[30], 6}, {NULL}};
const Segment SGI198454[] = {{&S80608[31], 3}, {NULL}};
const Segment SGI198455[] = {{&S80608[99], 6}, {NULL}};
const Segment SGI198456[] = {{&S80609[34], 21}, {NULL}};
const Segment SGI198457[] = {{&S80609[27], 6}, {NULL}};
const Segment SGI198458[] = {{&S80609[118], 9}, {NULL}};
const Segment SGI198459[] = {{&S80609[91], 6}, {NULL}};
const Segment SGI198460[] = {{&S74551[44], 10}, {NULL}};
const Segment SGI198461[] = {{&S74551[44], 22}, {NULL}};
const Segment SGI198462[] = {{&S74551[30], 6}, {NULL}};
const Segment SGI198463[] = {{&S74551[115], 3}, {NULL}};
const Segment SGI198464[] = {{&S74551[80], 6}, {NULL}};
const Segment SGI198465[] = {{&S80617[34], 13}, {NULL}};
const Segment SGI198466[] = {{&S80617[27], 6}, {NULL}};
const Segment SGI198467[] = {{&S80617[100], 9}, {NULL}};
const Segment SGI198468[] = {{&S80617[73], 6}, {NULL}};
const Segment SGI198469[] = {{&S80618[34], 10}, {NULL}};
const Segment SGI198470[] = {{&S80618[27], 6}, {NULL}};
const Segment SGI198471[] = {{&S80618[97], 9}, {NULL}};
const Segment SGI198472[] = {{&S80618[70], 6}, {NULL}};
const Segment SGI198473[] = {{&S80620[34], 7}, {NULL}};
const Segment SGI198474[] = {{&S80620[27], 6}, {NULL}};
const Segment SGI198475[] = {{&S80620[99], 9}, {NULL}};
const Segment SGI198476[] = {{&S80620[72], 6}, {NULL}};
const Segment SGI198477[] = {{&S80621[34], 7}, {NULL}};
const Segment SGI198478[] = {{&S80621[27], 6}, {NULL}};
const Segment SGI198479[] = {{&S80621[99], 9}, {NULL}};
const Segment SGI198480[] = {{&S80621[72], 6}, {NULL}};
const Segment SGI198481[] = {{&S80628[34], 9}, {NULL}};
const Segment SGI198482[] = {{&S80628[27], 6}, {NULL}};
const Segment SGI198483[] = {{&S80628[96], 9}, {NULL}};
const Segment SGI198484[] = {{&S80628[69], 6}, {NULL}};
const Segment SGI198485[] = {{&S80629[34], 7}, {NULL}};
const Segment SGI198486[] = {{&S80629[27], 6}, {NULL}};
const Segment SGI198487[] = {{&S80629[112], 9}, {NULL}};
const Segment SGI198488[] = {{&S80629[85], 6}, {NULL}};
const Segment SGI198489[] = {{&S80630[34], 11}, {NULL}};
const Segment SGI198490[] = {{&S80630[27], 6}, {NULL}};
const Segment SGI198491[] = {{&S80630[110], 9}, {NULL}};
const Segment SGI198492[] = {{&S80630[83], 6}, {NULL}};
const Segment SGI198493[] = {{&S80638[34], 7}, {NULL}};
const Segment SGI198494[] = {{&S80638[27], 6}, {NULL}};
const Segment SGI198495[] = {{&S80638[94], 9}, {NULL}};
const Segment SGI198496[] = {{&S80638[67], 6}, {NULL}};
const Segment SGI198497[] = {{&S80638[128], 8}, {NULL}};
const Segment SGI198498[] = {{&S69178[34], 9}, {NULL}};
const Segment SGI198499[] = {{&S69178[27], 6}, {NULL}};
const Segment SGI198500[] = {{&S69178[96], 9}, {NULL}};
const Segment SGI198501[] = {{&S69178[69], 6}, {NULL}};
const Segment SGI198502[] = {{&S80640[34], 7}, {NULL}};
const Segment SGI198503[] = {{&S80640[27], 6}, {NULL}};
const Segment SGI198504[] = {{&S80640[94], 9}, {NULL}};
const Segment SGI198505[] = {{&S80640[67], 6}, {NULL}};
const Segment SGI198506[] = {{&S60195[34], 18}, {NULL}};
const Segment SGI198507[] = {{&S60195[27], 6}, {NULL}};
const Segment SGI198508[] = {{&S60195[105], 9}, {NULL}};
const Segment SGI198509[] = {{&S60195[78], 6}, {NULL}};
const Segment SGI198510[] = {{&S60196[34], 10}, {NULL}};
const Segment SGI198511[] = {{&S60196[27], 6}, {NULL}};
const Segment SGI198512[] = {{&S60196[96], 9}, {NULL}};
const Segment SGI198513[] = {{&S60196[69], 6}, {NULL}};
const Segment SGI198514[] = {{&S74605[44], 7}, {NULL}};
const Segment SGI198515[] = {{&S74605[44], 19}, {NULL}};
const Segment SGI198516[] = {{&S74605[30], 6}, {NULL}};
const Segment SGI198517[] = {{&S74605[112], 3}, {NULL}};
const Segment SGI198518[] = {{&S74605[77], 6}, {NULL}};
const Segment SGI198519[] = {{&S74606[34], 20}, {NULL}};
const Segment SGI198520[] = {{&S74606[27], 6}, {NULL}};
const Segment SGI198521[] = {{&S74606[107], 9}, {NULL}};
const Segment SGI198522[] = {{&S74606[80], 6}, {NULL}};
const Segment SGI198523[] = {{&S69199[34], 10}, {NULL}};
const Segment SGI198524[] = {{&S69199[27], 6}, {NULL}};
const Segment SGI198525[] = {{&S69199[97], 9}, {NULL}};
const Segment SGI198526[] = {{&S69199[70], 6}, {NULL}};
const Segment SGI198527[] = {{&S60211[34], 7}, {NULL}};
const Segment SGI198528[] = {{&S60211[27], 6}, {NULL}};
const Segment SGI198529[] = {{&S60211[102], 9}, {NULL}};
const Segment SGI198530[] = {{&S60211[75], 6}, {NULL}};
const Segment SGI198531[] = {{&S80668[34], 11}, {NULL}};
const Segment SGI198532[] = {{&S80668[27], 6}, {NULL}};
const Segment SGI198533[] = {{&S80668[98], 9}, {NULL}};
const Segment SGI198534[] = {{&S80668[71], 6}, {NULL}};
const Segment SGI198535[] = {{&S60218[34], 10}, {NULL}};
const Segment SGI198536[] = {{&S60218[27], 6}, {NULL}};
const Segment SGI198537[] = {{&S60218[96], 9}, {NULL}};
const Segment SGI198538[] = {{&S60218[69], 6}, {NULL}};
const Segment SGI198539[] = {{&S80671[44], 12}, {NULL}};
const Segment SGI198540[] = {{&S80671[44], 24}, {NULL}};
const Segment SGI198541[] = {{&S80671[30], 6}, {NULL}};
const Segment SGI198542[] = {{&S80671[31], 3}, {NULL}};
const Segment SGI198543[] = {{&S80671[82], 6}, {NULL}};
const Segment SGI198544[] = {{&S80672[34], 5}, {NULL}};
const Segment SGI198545[] = {{&S80672[27], 6}, {NULL}};
const Segment SGI198546[] = {{&S80672[92], 9}, {NULL}};
const Segment SGI198547[] = {{&S80672[65], 6}, {NULL}};
const Segment SGI198548[] = {{&S80674[34], 10}, {NULL}};
const Segment SGI198549[] = {{&S80674[27], 6}, {NULL}};
const Segment SGI198550[] = {{&S80674[97], 9}, {NULL}};
const Segment SGI198551[] = {{&S80674[70], 6}, {NULL}};
const Segment SGI198552[] = {{&S80681[45], 10}, {NULL}};
const Segment SGI198553[] = {{&S80681[30], 6}, {NULL}};
const Segment SGI198554[] = {{&S80681[31], 3}, {NULL}};
const Segment SGI198555[] = {{&S80681[88], 5}, {NULL}};
const Segment SGI198556[] = {{&S80681[146], 13}, {NULL}};
const Segment SGI198557[] = {{&S80682[34], 7}, {NULL}};
const Segment SGI198558[] = {{&S80682[27], 6}, {NULL}};
const Segment SGI198559[] = {{&S80682[99], 9}, {NULL}};
const Segment SGI198560[] = {{&S80682[72], 6}, {NULL}};
const Segment SGI198561[] = {{&S80684[34], 9}, {NULL}};
const Segment SGI198562[] = {{&S80684[27], 6}, {NULL}};
const Segment SGI198563[] = {{&S80684[96], 9}, {NULL}};
const Segment SGI198564[] = {{&S80684[69], 6}, {NULL}};
const Segment SGI198565[] = {{&S80685[34], 10}, {NULL}};
const Segment SGI198566[] = {{&S80685[27], 6}, {NULL}};
const Segment SGI198567[] = {{&S80685[97], 9}, {NULL}};
const Segment SGI198568[] = {{&S80685[70], 6}, {NULL}};
const Segment SGI198569[] = {{&S74651[34], 10}, {NULL}};
const Segment SGI198570[] = {{&S74651[27], 6}, {NULL}};
const Segment SGI198571[] = {{&S74651[97], 9}, {NULL}};
const Segment SGI198572[] = {{&S74651[70], 6}, {NULL}};
const Segment SGI198573[] = {{&S80719[44], 18}, {NULL}};
const Segment SGI198574[] = {{&S80719[44], 30}, {NULL}};
const Segment SGI198575[] = {{&S80719[30], 6}, {NULL}};
const Segment SGI198576[] = {{&S80719[31], 3}, {NULL}};
const Segment SGI198577[] = {{&S80719[88], 6}, {NULL}};
const Segment SGI198578[] = {{&S80721[34], 18}, {NULL}};
const Segment SGI198579[] = {{&S80721[27], 6}, {NULL}};
const Segment SGI198580[] = {{&S80721[105], 9}, {NULL}};
const Segment SGI198581[] = {{&S80721[78], 6}, {NULL}};
const Segment SGI198582[] = {{&S80722[34], 19}, {NULL}};
const Segment SGI198583[] = {{&S80722[27], 6}, {NULL}};
const Segment SGI198584[] = {{&S80722[106], 9}, {NULL}};
const Segment SGI198585[] = {{&S80722[79], 6}, {NULL}};
const Segment SGI198586[] = {{&S80723[34], 16}, {NULL}};
const Segment SGI198587[] = {{&S80723[27], 6}, {NULL}};
const Segment SGI198588[] = {{&S80723[103], 9}, {NULL}};
const Segment SGI198589[] = {{&S80723[76], 6}, {NULL}};
const Segment SGI198590[] = {{&S80724[34], 19}, {NULL}};
const Segment SGI198591[] = {{&S80724[27], 6}, {NULL}};
const Segment SGI198592[] = {{&S80724[106], 9}, {NULL}};
const Segment SGI198593[] = {{&S80724[79], 6}, {NULL}};
const Segment SGI198594[] = {{&S69275[34], 10}, {NULL}};
const Segment SGI198595[] = {{&S69275[27], 6}, {NULL}};
const Segment SGI198596[] = {{&S69275[97], 9}, {NULL}};
const Segment SGI198597[] = {{&S69275[70], 6}, {NULL}};
const Segment SGI198598[] = {{&S69934[44], 10}, {NULL}};
const Segment SGI198599[] = {{&S69934[44], 22}, {NULL}};
const Segment SGI198600[] = {{&S69934[30], 6}, {NULL}};
const Segment SGI198601[] = {{&S69934[115], 3}, {NULL}};
const Segment SGI198602[] = {{&S69934[80], 6}, {NULL}};
const Segment SGI198603[] = {{&S69940[38], 10}, {NULL}};
const Segment SGI198604[] = {{&S69940[31], 6}, {NULL}};
const Segment SGI198605[] = {{&S74699[44], 13}, {NULL}};
const Segment SGI198606[] = {{&S74699[44], 25}, {NULL}};
const Segment SGI198607[] = {{&S74699[30], 6}, {NULL}};
const Segment SGI198608[] = {{&S69942[44], 10}, {NULL}};
const Segment SGI198609[] = {{&S69942[44], 22}, {NULL}};
const Segment SGI198610[] = {{&S69942[30], 6}, {NULL}};
const Segment SGI198611[] = {{&S74699[118], 3}, {NULL}};
const Segment SGI198612[] = {{&S74699[83], 6}, {NULL}};
const Segment SGI198613[] = {{&S69942[80], 6}, {NULL}};
const Segment SGI198614[] = {{&S69943[44], 10}, {NULL}};
const Segment SGI198615[] = {{&S74700[44], 13}, {NULL}};
const Segment SGI198616[] = {{&S69943[44], 22}, {NULL}};
const Segment SGI198617[] = {{&S69943[30], 6}, {NULL}};
const Segment SGI198618[] = {{&S74700[44], 25}, {NULL}};
const Segment SGI198619[] = {{&S74700[30], 6}, {NULL}};
const Segment SGI198620[] = {{&S69943[115], 3}, {NULL}};
const Segment SGI198621[] = {{&S74700[118], 3}, {NULL}};
const Segment SGI198622[] = {{&S69943[80], 6}, {NULL}};
const Segment SGI198623[] = {{&S74700[83], 6}, {NULL}};
const Segment SGI198624[] = {{&S69944[44], 10}, {NULL}};
const Segment SGI198625[] = {{&S69944[44], 22}, {NULL}};
const Segment SGI198626[] = {{&S69944[30], 6}, {NULL}};
const Segment SGI198627[] = {{&S69944[115], 3}, {NULL}};
const Segment SGI198628[] = {{&S69944[80], 6}, {NULL}};
const Segment SGI198629[] = {{&S69945[44], 10}, {NULL}};
const Segment SGI198630[] = {{&S69945[44], 22}, {NULL}};
const Segment SGI198631[] = {{&S69945[30], 6}, {NULL}};
const Segment SGI198632[] = {{&S69945[80], 6}, {NULL}};
const Segment SGI198633[] = {{&S69946[38], 11}, {NULL}};
const Segment SGI198634[] = {{&S69946[31], 6}, {NULL}};
const Segment SGI198635[] = {{&S69947[93], 29}, {NULL}};
const Segment SGI198636[] = {{&S69948[34], 10}, {NULL}};
const Segment SGI198637[] = {{&S69948[27], 6}, {NULL}};
const Segment SGI198638[] = {{&S74709[44], 4}, {NULL}};
const Segment SGI198639[] = {{&S74709[44], 17}, {NULL}};
const Segment SGI198640[] = {{&S74709[30], 6}, {NULL}};
const Segment SGI198641[] = {{&S69948[97], 9}, {NULL}};
const Segment SGI198642[] = {{&S69948[70], 6}, {NULL}};
const Segment SGI198643[] = {{&S74709[110], 3}, {NULL}};
const Segment SGI198644[] = {{&S74709[75], 6}, {NULL}};
const Segment SGI198645[] = {{&S74710[38], 4}, {NULL}};
const Segment SGI198646[] = {{&S74710[31], 6}, {NULL}};
const Segment SGI198647[] = {{&S69951[34], 10}, {NULL}};
const Segment SGI198648[] = {{&S69951[27], 6}, {NULL}};
const Segment SGI198649[] = {{&S69951[97], 9}, {NULL}};
const Segment SGI198650[] = {{&S69951[70], 6}, {NULL}};
const Segment SGI198651[] = {{&S69956[44], 15}, {NULL}};
const Segment SGI198652[] = {{&S69956[44], 27}, {NULL}};
const Segment SGI198653[] = {{&S69956[30], 6}, {NULL}};
const Segment SGI198654[] = {{&S69956[120], 3}, {NULL}};
const Segment SGI198655[] = {{&S69956[85], 6}, {NULL}};
const Segment SGI198656[] = {{&S60330[44], 10}, {NULL}};
const Segment SGI198657[] = {{&S60330[44], 21}, {NULL}};
const Segment SGI198658[] = {{&S60330[30], 6}, {NULL}};
const Segment SGI198659[] = {{&S60330[132], 3}, {NULL}};
const Segment SGI198660[] = {{&S60330[97], 6}, {NULL}};
const Segment SGI198661[] = {{&S60330[67], 8}, {NULL}};
const Segment SGI198662[] = {{&S69957[38], 15}, {NULL}};
const Segment SGI198663[] = {{&S69957[31], 6}, {NULL}};
const Segment SGI198664[] = {{&S60331[44], 10}, {NULL}};
const Segment SGI198665[] = {{&S60331[44], 22}, {NULL}};
const Segment SGI198666[] = {{&S60331[30], 6}, {NULL}};
const Segment SGI198667[] = {{&S60331[115], 3}, {NULL}};
const Segment SGI198668[] = {{&S60331[80], 6}, {NULL}};
const Segment SGI198669[] = {{&S60331[140], 8}, {NULL}};
const Segment SGI198670[] = {{&S60341[34], 10}, {NULL}};
const Segment SGI198671[] = {{&S60341[27], 6}, {NULL}};
const Segment SGI198672[] = {{&S60341[97], 9}, {NULL}};
const Segment SGI198673[] = {{&S60341[70], 6}, {NULL}};
const Segment SGI198674[] = {{&S60342[34], 10}, {NULL}};
const Segment SGI198675[] = {{&S60342[27], 6}, {NULL}};
const Segment SGI198676[] = {{&S60342[96], 9}, {NULL}};
const Segment SGI198677[] = {{&S60342[69], 6}, {NULL}};
const Segment SGI198678[] = {{&S74745[44], 10}, {NULL}};
const Segment SGI198679[] = {{&S74745[44], 22}, {NULL}};
const Segment SGI198680[] = {{&S74745[30], 6}, {NULL}};
const Segment SGI198681[] = {{&S74745[115], 3}, {NULL}};
const Segment SGI198682[] = {{&S74745[80], 6}, {NULL}};
const Segment SGI198683[] = {{&S74750[34], 7}, {NULL}};
const Segment SGI198684[] = {{&S74750[27], 6}, {NULL}};
const Segment SGI198685[] = {{&S69351[44], 10}, {NULL}};
const Segment SGI198686[] = {{&S69351[44], 27}, {NULL}};
const Segment SGI198687[] = {{&S69351[30], 6}, {NULL}};
const Segment SGI198688[] = {{&S74750[94], 9}, {NULL}};
const Segment SGI198689[] = {{&S74750[67], 6}, {NULL}};
const Segment SGI198690[] = {{&S69351[119], 3}, {NULL}};
const Segment SGI198691[] = {{&S69351[85], 5}, {NULL}};
const Segment SGI198692[] = {{&S74754[44], 10}, {NULL}};
const Segment SGI198693[] = {{&S74754[44], 22}, {NULL}};
const Segment SGI198694[] = {{&S74754[30], 6}, {NULL}};
const Segment SGI198695[] = {{&S74754[80], 6}, {NULL}};
const Segment SGI198696[] = {{&S74754[107], 13}, {NULL}};
const Segment SGI198697[] = {{&S60371[44], 6}, {NULL}};
const Segment SGI198698[] = {{&S60371[44], 17}, {NULL}};
const Segment SGI198699[] = {{&S60371[30], 6}, {NULL}};
const Segment SGI198700[] = {{&S60371[128], 3}, {NULL}};
const Segment SGI198701[] = {{&S60371[93], 6}, {NULL}};
const Segment SGI198702[] = {{&S60371[63], 8}, {NULL}};
const Segment SGI198703[] = {{&S60373[44], 6}, {NULL}};
const Segment SGI198704[] = {{&S60373[44], 17}, {NULL}};
const Segment SGI198705[] = {{&S60373[30], 6}, {NULL}};
const Segment SGI198706[] = {{&S60373[128], 3}, {NULL}};
const Segment SGI198707[] = {{&S60373[93], 6}, {NULL}};
const Segment SGI198708[] = {{&S60373[63], 8}, {NULL}};
const Segment SGI198709[] = {{&S60374[44], 6}, {NULL}};
const Segment SGI198710[] = {{&S60374[44], 18}, {NULL}};
const Segment SGI198711[] = {{&S60374[30], 6}, {NULL}};
const Segment SGI198712[] = {{&S60374[129], 3}, {NULL}};
const Segment SGI198713[] = {{&S60374[94], 6}, {NULL}};
const Segment SGI198714[] = {{&S60374[64], 8}, {NULL}};
const Segment SGI198715[] = {{&S74772[44], 18}, {NULL}};
const Segment SGI198716[] = {{&S74772[44], 30}, {NULL}};
const Segment SGI198717[] = {{&S74772[30], 6}, {NULL}};
const Segment SGI198718[] = {{&S74772[123], 3}, {NULL}};
const Segment SGI198719[] = {{&S74772[88], 6}, {NULL}};
const Segment SGI198720[] = {{&S70026[45], 10}, {NULL}};
const Segment SGI198721[] = {{&S70026[30], 6}, {NULL}};
const Segment SGI198722[] = {{&S70026[115], 3}, {NULL}};
const Segment SGI198723[] = {{&S70026[81], 5}, {NULL}};
const Segment SGI198724[] = {{&S70026[139], 13}, {NULL}};
const Segment SGI198725[] = {{&S70027[44], 10}, {NULL}};
const Segment SGI198726[] = {{&S70027[44], 22}, {NULL}};
const Segment SGI198727[] = {{&S70027[30], 6}, {NULL}};
const Segment SGI198728[] = {{&S70027[115], 3}, {NULL}};
const Segment SGI198729[] = {{&S70027[80], 6}, {NULL}};
const Segment SGI198730[] = {{&S70027[135], 8}, {NULL}};
const Segment SGI198731[] = {{&S70027[144], 13}, {NULL}};
const Segment SGI198732[] = {{&S74788[34], 17}, {NULL}};
const Segment SGI198733[] = {{&S74788[27], 6}, {NULL}};
const Segment SGI198734[] = {{&S74788[104], 9}, {NULL}};
const Segment SGI198735[] = {{&S74788[77], 6}, {NULL}};
const Segment SGI198736[] = {{&S70034[44], 17}, {NULL}};
const Segment SGI198737[] = {{&S70034[44], 37}, {NULL}};
const Segment SGI198738[] = {{&S70034[30], 6}, {NULL}};
const Segment SGI198739[] = {{&S70034[130], 3}, {NULL}};
const Segment SGI198740[] = {{&S70034[95], 6}, {NULL}};
const Segment SGI198741[] = {{&S69404[34], 10}, {NULL}};
const Segment SGI198742[] = {{&S69404[27], 6}, {NULL}};
const Segment SGI198743[] = {{&S70035[38], 17}, {NULL}};
const Segment SGI198744[] = {{&S70035[31], 6}, {NULL}};
const Segment SGI198745[] = {{&S69404[97], 9}, {NULL}};
const Segment SGI198746[] = {{&S69404[70], 6}, {NULL}};
const Segment SGI198747[] = {{&S74800[44], 25}, {NULL}};
const Segment SGI198748[] = {{&S74800[44], 37}, {NULL}};
const Segment SGI198749[] = {{&S74800[30], 6}, {NULL}};
const Segment SGI198750[] = {{&S74800[146], 3}, {NULL}};
const Segment SGI198751[] = {{&S74800[111], 6}, {NULL}};
const Segment SGI198752[] = {{&S74800[83], 8}, {NULL}};
const Segment SGI198753[] = {{&S74801[44], 25}, {NULL}};
const Segment SGI198754[] = {{&S74801[44], 37}, {NULL}};
const Segment SGI198755[] = {{&S74801[30], 6}, {NULL}};
const Segment SGI198756[] = {{&S74801[146], 3}, {NULL}};
const Segment SGI198757[] = {{&S74801[111], 6}, {NULL}};
const Segment SGI198758[] = {{&S74801[83], 8}, {NULL}};
const Segment SGI198759[] = {{&S74815[34], 25}, {NULL}};
const Segment SGI198760[] = {{&S74815[27], 6}, {NULL}};
const Segment SGI198761[] = {{&S74815[112], 9}, {NULL}};
const Segment SGI198762[] = {{&S74815[85], 6}, {NULL}};
const Segment SGI198763[] = {{&S74818[44], 18}, {NULL}};
const Segment SGI198764[] = {{&S74818[44], 30}, {NULL}};
const Segment SGI198765[] = {{&S74818[30], 6}, {NULL}};
const Segment SGI198766[] = {{&S74818[88], 6}, {NULL}};
const Segment SGI198767[] = {{&S74818[115], 13}, {NULL}};
const Segment SGI198768[] = {{&S70093[44], 6}, {NULL}};
const Segment SGI198769[] = {{&S70093[44], 18}, {NULL}};
const Segment SGI198770[] = {{&S70093[30], 6}, {NULL}};
const Segment SGI198771[] = {{&S70093[111], 3}, {NULL}};
const Segment SGI198772[] = {{&S70093[76], 6}, {NULL}};
const Segment SGI198773[] = {{&S70094[38], 6}, {NULL}};
const Segment SGI198774[] = {{&S70094[31], 6}, {NULL}};
const Segment SGI198775[] = {{&S80759[44], 5}, {NULL}};
const Segment SGI198776[] = {{&S80759[44], 17}, {NULL}};
const Segment SGI198777[] = {{&S80759[30], 6}, {NULL}};
const Segment SGI198778[] = {{&S80759[31], 3}, {NULL}};
const Segment SGI198779[] = {{&S80759[75], 6}, {NULL}};
const Segment SGI198780[] = {{&S80760[44], 20}, {NULL}};
const Segment SGI198781[] = {{&S80760[44], 32}, {NULL}};
const Segment SGI198782[] = {{&S80760[30], 6}, {NULL}};
const Segment SGI198783[] = {{&S80760[31], 3}, {NULL}};
const Segment SGI198784[] = {{&S80760[90], 6}, {NULL}};
const Segment SGI198785[] = {{&S80762[38], 20}, {NULL}};
const Segment SGI198786[] = {{&S80762[31], 6}, {NULL}};
const Segment SGI198787[] = {{&S70103[44], 10}, {NULL}};
const Segment SGI198788[] = {{&S70103[44], 22}, {NULL}};
const Segment SGI198789[] = {{&S70103[30], 6}, {NULL}};
const Segment SGI198790[] = {{&S70103[136], 3}, {NULL}};
const Segment SGI198791[] = {{&S70103[101], 6}, {NULL}};
const Segment SGI198792[] = {{&S80765[59], 15}, {NULL}};
const Segment SGI198793[] = {{&S80765[59], 27}, {NULL}};
const Segment SGI198794[] = {{&S80765[45], 6}, {NULL}};
const Segment SGI198795[] = {{&S70104[44], 18}, {NULL}};
const Segment SGI198796[] = {{&S70104[44], 30}, {NULL}};
const Segment SGI198797[] = {{&S70104[30], 6}, {NULL}};
const Segment SGI198798[] = {{&S80765[46], 3}, {NULL}};
const Segment SGI198799[] = {{&S80765[100], 6}, {NULL}};
const Segment SGI198800[] = {{&S70104[144], 3}, {NULL}};
const Segment SGI198801[] = {{&S80765[0], 14}, {NULL}};
const Segment SGI198802[] = {{&S70104[109], 6}, {NULL}};
const Segment SGI198803[] = {{&S80775[34], 6}, {NULL}};
const Segment SGI198804[] = {{&S80775[27], 6}, {NULL}};
const Segment SGI198805[] = {{&S80775[93], 9}, {NULL}};
const Segment SGI198806[] = {{&S80775[66], 6}, {NULL}};
const Segment SGI198807[] = {{&S80780[34], 11}, {NULL}};
const Segment SGI198808[] = {{&S80780[27], 6}, {NULL}};
const Segment SGI198809[] = {{&S80780[98], 9}, {NULL}};
const Segment SGI198810[] = {{&S80780[71], 6}, {NULL}};
const Segment SGI198811[] = {{&S80780[139], 8}, {NULL}};
const Segment SGI198812[] = {{&S80781[34], 11}, {NULL}};
const Segment SGI198813[] = {{&S80781[27], 6}, {NULL}};
const Segment SGI198814[] = {{&S80781[98], 9}, {NULL}};
const Segment SGI198815[] = {{&S80781[71], 6}, {NULL}};
const Segment SGI198816[] = {{&S80781[139], 8}, {NULL}};
const Segment SGI198817[] = {{&S70126[44], 10}, {NULL}};
const Segment SGI198818[] = {{&S70126[44], 22}, {NULL}};
const Segment SGI198819[] = {{&S70126[30], 6}, {NULL}};
const Segment SGI198820[] = {{&S70126[131], 3}, {NULL}};
const Segment SGI198821[] = {{&S70126[96], 6}, {NULL}};
const Segment SGI198822[] = {{&S70126[68], 8}, {NULL}};
const Segment SGI198823[] = {{&S74888[34], 25}, {NULL}};
const Segment SGI198824[] = {{&S74888[27], 6}, {NULL}};
const Segment SGI198825[] = {{&S74888[112], 9}, {NULL}};
const Segment SGI198826[] = {{&S74888[85], 6}, {NULL}};
const Segment SGI198827[] = {{&S74889[34], 25}, {NULL}};
const Segment SGI198828[] = {{&S74889[27], 6}, {NULL}};
const Segment SGI198829[] = {{&S74889[112], 9}, {NULL}};
const Segment SGI198830[] = {{&S74889[85], 6}, {NULL}};
const Segment SGI198831[] = {{&S74890[34], 25}, {NULL}};
const Segment SGI198832[] = {{&S74890[27], 6}, {NULL}};
const Segment SGI198833[] = {{&S74890[112], 9}, {NULL}};
const Segment SGI198834[] = {{&S74890[85], 6}, {NULL}};
const Segment SGI198835[] = {{&S74891[34], 25}, {NULL}};
const Segment SGI198836[] = {{&S74891[27], 6}, {NULL}};
const Segment SGI198837[] = {{&S74891[112], 9}, {NULL}};
const Segment SGI198838[] = {{&S74891[85], 6}, {NULL}};
const Segment SGI198839[] = {{&S70141[34], 11}, {NULL}};
const Segment SGI198840[] = {{&S70141[27], 6}, {NULL}};
const Segment SGI198841[] = {{&S70141[98], 9}, {NULL}};
const Segment SGI198842[] = {{&S70141[71], 6}, {NULL}};
const Segment SGI198843[] = {{&S78709[42], 20}, {NULL}};
const Segment SGI198844[] = {{&S78709[30], 4}, {NULL}};
const Segment SGI198845[] = {{&S78709[31], 3}, {NULL}};
const Segment SGI198846[] = {{&S78709[76], 6}, {NULL}};
const Segment SGI198847[] = {{&S80811[44], 15}, {NULL}};
const Segment SGI198848[] = {{&S80811[44], 27}, {NULL}};
const Segment SGI198849[] = {{&S80811[30], 6}, {NULL}};
const Segment SGI198850[] = {{&S80811[116], 2}, {NULL}};
const Segment SGI198851[] = {{&S80811[85], 5}, {NULL}};
const Segment SGI198852[] = {{&S70153[34], 9}, {NULL}};
const Segment SGI198853[] = {{&S70153[27], 6}, {NULL}};
const Segment SGI198854[] = {{&S70153[96], 9}, {NULL}};
const Segment SGI198855[] = {{&S70153[69], 6}, {NULL}};
const Segment SGI198856[] = {{&S70154[44], 8}, {NULL}};
const Segment SGI198857[] = {{&S70154[44], 20}, {NULL}};
const Segment SGI198858[] = {{&S70154[30], 6}, {NULL}};
const Segment SGI198859[] = {{&S80813[44], 15}, {NULL}};
const Segment SGI198860[] = {{&S80813[44], 27}, {NULL}};
const Segment SGI198861[] = {{&S80813[30], 6}, {NULL}};
const Segment SGI198862[] = {{&S70154[113], 3}, {NULL}};
const Segment SGI198863[] = {{&S70154[78], 6}, {NULL}};
const Segment SGI198864[] = {{&S80813[116], 2}, {NULL}};
const Segment SGI198865[] = {{&S80813[85], 5}, {NULL}};
const Segment SGI198866[] = {{&S70155[38], 8}, {NULL}};
const Segment SGI198867[] = {{&S70155[31], 6}, {NULL}};
const Segment SGI198868[] = {{&S80814[44], 15}, {NULL}};
const Segment SGI198869[] = {{&S80814[44], 27}, {NULL}};
const Segment SGI198870[] = {{&S80814[30], 6}, {NULL}};
const Segment SGI198871[] = {{&S80814[116], 2}, {NULL}};
const Segment SGI198872[] = {{&S80814[85], 5}, {NULL}};
const Segment SGI198873[] = {{&S70156[44], 7}, {NULL}};
const Segment SGI198874[] = {{&S70156[44], 19}, {NULL}};
const Segment SGI198875[] = {{&S70156[30], 6}, {NULL}};
const Segment SGI198876[] = {{&S70156[112], 3}, {NULL}};
const Segment SGI198877[] = {{&S70156[77], 6}, {NULL}};
const Segment SGI198878[] = {{&S80815[44], 15}, {NULL}};
const Segment SGI198879[] = {{&S80815[44], 27}, {NULL}};
const Segment SGI198880[] = {{&S80815[30], 6}, {NULL}};
const Segment SGI198881[] = {{&S80815[116], 2}, {NULL}};
const Segment SGI198882[] = {{&S80815[85], 5}, {NULL}};
const Segment SGI198883[] = {{&S70157[34], 15}, {NULL}};
const Segment SGI198884[] = {{&S70157[27], 6}, {NULL}};
const Segment SGI198885[] = {{&S70157[105], 9}, {NULL}};
const Segment SGI198886[] = {{&S70157[78], 6}, {NULL}};
const Segment SGI198887[] = {{&S70157[146], 8}, {NULL}};
const Segment SGI198888[] = {{&S60542[44], 9}, {NULL}};
const Segment SGI198889[] = {{&S70158[34], 9}, {NULL}};
const Segment SGI198890[] = {{&S60542[44], 21}, {NULL}};
const Segment SGI198891[] = {{&S60542[30], 6}, {NULL}};
const Segment SGI198892[] = {{&S70158[27], 6}, {NULL}};
const Segment SGI198893[] = {{&S60542[132], 3}, {NULL}};
const Segment SGI198894[] = {{&S70158[96], 9}, {NULL}};
const Segment SGI198895[] = {{&S70158[69], 6}, {NULL}};
const Segment SGI198896[] = {{&S60542[97], 6}, {NULL}};
const Segment SGI198897[] = {{&S60542[67], 8}, {NULL}};
const Segment SGI198898[] = {{&S70159[34], 9}, {NULL}};
const Segment SGI198899[] = {{&S70159[27], 6}, {NULL}};
const Segment SGI198900[] = {{&S70159[96], 9}, {NULL}};
const Segment SGI198901[] = {{&S70159[69], 6}, {NULL}};
const Segment SGI198902[] = {{&S70160[34], 9}, {NULL}};
const Segment SGI198903[] = {{&S70160[27], 6}, {NULL}};
const Segment SGI198904[] = {{&S70160[96], 9}, {NULL}};
const Segment SGI198905[] = {{&S70160[69], 6}, {NULL}};
const Segment SGI198906[] = {{&S70161[34], 9}, {NULL}};
const Segment SGI198907[] = {{&S70161[27], 6}, {NULL}};
const Segment SGI198908[] = {{&S70161[96], 9}, {NULL}};
const Segment SGI198909[] = {{&S70161[69], 6}, {NULL}};
const Segment SGI198910[] = {{&S80839[44], 10}, {NULL}};
const Segment SGI198911[] = {{&S80839[44], 27}, {NULL}};
const Segment SGI198912[] = {{&S80839[30], 6}, {NULL}};
const Segment SGI198913[] = {{&S80839[31], 3}, {NULL}};
const Segment SGI198914[] = {{&S80839[85], 6}, {NULL}};
const Segment SGI198915[] = {{&S74946[44], 17}, {NULL}};
const Segment SGI198916[] = {{&S74946[44], 29}, {NULL}};
const Segment SGI198917[] = {{&S74946[30], 6}, {NULL}};
const Segment SGI198918[] = {{&S74946[122], 3}, {NULL}};
const Segment SGI198919[] = {{&S74946[87], 6}, {NULL}};
const Segment SGI198920[] = {{&S74947[44], 20}, {NULL}};
const Segment SGI198921[] = {{&S74947[30], 6}, {NULL}};
const Segment SGI198922[] = {{&S74947[127], 3}, {NULL}};
const Segment SGI198923[] = {{&S74947[92], 6}, {NULL}};
const Segment SGI198924[] = {{&S74949[34], 13}, {NULL}};
const Segment SGI198925[] = {{&S74949[27], 6}, {NULL}};
const Segment SGI198926[] = {{&S74949[100], 9}, {NULL}};
const Segment SGI198927[] = {{&S74949[73], 6}, {NULL}};
const Segment SGI198928[] = {{&S80848[34], 15}, {NULL}};
const Segment SGI198929[] = {{&S80848[27], 6}, {NULL}};
const Segment SGI198930[] = {{&S80848[102], 9}, {NULL}};
const Segment SGI198931[] = {{&S80848[75], 6}, {NULL}};
const Segment SGI198932[] = {{&S80852[44], 6}, {NULL}};
const Segment SGI198933[] = {{&S80852[44], 18}, {NULL}};
const Segment SGI198934[] = {{&S80852[30], 6}, {NULL}};
const Segment SGI198935[] = {{&S80852[31], 3}, {NULL}};
const Segment SGI198936[] = {{&S80852[76], 6}, {NULL}};
const Segment SGI198937[] = {{&S80853[44], 12}, {NULL}};
const Segment SGI198938[] = {{&S80853[44], 24}, {NULL}};
const Segment SGI198939[] = {{&S80853[30], 6}, {NULL}};
const Segment SGI198940[] = {{&S80853[31], 3}, {NULL}};
const Segment SGI198941[] = {{&S80853[82], 6}, {NULL}};
const Segment SGI198942[] = {{&S80854[44], 6}, {NULL}};
const Segment SGI198943[] = {{&S80854[44], 18}, {NULL}};
const Segment SGI198944[] = {{&S80854[30], 6}, {NULL}};
const Segment SGI198945[] = {{&S80854[79], 6}, {NULL}};
const Segment SGI198946[] = {{&S80856[44], 12}, {NULL}};
const Segment SGI198947[] = {{&S80856[44], 24}, {NULL}};
const Segment SGI198948[] = {{&S80856[30], 6}, {NULL}};
const Segment SGI198949[] = {{&S80856[31], 3}, {NULL}};
const Segment SGI198950[] = {{&S80856[82], 6}, {NULL}};
const Segment SGI198951[] = {{&S80857[34], 7}, {NULL}};
const Segment SGI198952[] = {{&S80857[27], 6}, {NULL}};
const Segment SGI198953[] = {{&S80857[99], 9}, {NULL}};
const Segment SGI198954[] = {{&S80857[72], 6}, {NULL}};
const Segment SGI198955[] = {{&S70203[34], 9}, {NULL}};
const Segment SGI198956[] = {{&S70203[27], 6}, {NULL}};
const Segment SGI198957[] = {{&S70203[96], 9}, {NULL}};
const Segment SGI198958[] = {{&S70203[69], 6}, {NULL}};
const Segment SGI198959[] = {{&S74968[34], 6}, {NULL}};
const Segment SGI198960[] = {{&S74968[27], 6}, {NULL}};
const Segment SGI198961[] = {{&S74968[93], 9}, {NULL}};
const Segment SGI198962[] = {{&S74968[66], 6}, {NULL}};
const Segment SGI198963[] = {{&S70213[34], 10}, {NULL}};
const Segment SGI198964[] = {{&S70213[27], 6}, {NULL}};
const Segment SGI198965[] = {{&S70213[97], 9}, {NULL}};
const Segment SGI198966[] = {{&S70213[70], 6}, {NULL}};
const Segment SGI198967[] = {{&S70214[34], 11}, {NULL}};
const Segment SGI198968[] = {{&S70214[27], 6}, {NULL}};
const Segment SGI198969[] = {{&S70214[98], 9}, {NULL}};
const Segment SGI198970[] = {{&S70214[71], 6}, {NULL}};
const Segment SGI198971[] = {{&S60586[44], 17}, {NULL}};
const Segment SGI198972[] = {{&S60586[44], 28}, {NULL}};
const Segment SGI198973[] = {{&S60586[30], 6}, {NULL}};
const Segment SGI198974[] = {{&S60586[121], 3}, {NULL}};
const Segment SGI198975[] = {{&S60586[86], 6}, {NULL}};
const Segment SGI198976[] = {{&S60587[38], 17}, {NULL}};
const Segment SGI198977[] = {{&S60587[31], 6}, {NULL}};
const Segment SGI198978[] = {{&S60588[44], 17}, {NULL}};
const Segment SGI198979[] = {{&S60588[44], 28}, {NULL}};
const Segment SGI198980[] = {{&S60588[30], 6}, {NULL}};
const Segment SGI198981[] = {{&S60588[121], 3}, {NULL}};
const Segment SGI198982[] = {{&S60588[86], 6}, {NULL}};
const Segment SGI198983[] = {{&S60589[44], 17}, {NULL}};
const Segment SGI198984[] = {{&S60589[44], 28}, {NULL}};
const Segment SGI198985[] = {{&S60589[30], 6}, {NULL}};
const Segment SGI198986[] = {{&S60589[121], 3}, {NULL}};
const Segment SGI198987[] = {{&S60589[86], 6}, {NULL}};
const Segment SGI198988[] = {{&S74992[44], 7}, {NULL}};
const Segment SGI198989[] = {{&S74992[44], 19}, {NULL}};
const Segment SGI198990[] = {{&S74992[30], 6}, {NULL}};
const Segment SGI198991[] = {{&S74992[112], 3}, {NULL}};
const Segment SGI198992[] = {{&S74992[77], 6}, {NULL}};
const Segment SGI198993[] = {{&S80886[34], 13}, {NULL}};
const Segment SGI198994[] = {{&S80886[27], 6}, {NULL}};
const Segment SGI198995[] = {{&S80886[100], 9}, {NULL}};
const Segment SGI198996[] = {{&S80886[73], 6}, {NULL}};
const Segment SGI198997[] = {{&S80887[34], 13}, {NULL}};
const Segment SGI198998[] = {{&S80887[27], 6}, {NULL}};
const Segment SGI198999[] = {{&S80887[100], 9}, {NULL}};
const Segment SGI199000[] = {{&S80887[73], 6}, {NULL}};
const Segment SGI199001[] = {{&S74995[44], 10}, {NULL}};
const Segment SGI199002[] = {{&S74995[44], 22}, {NULL}};
const Segment SGI199003[] = {{&S74995[30], 6}, {NULL}};
const Segment SGI199004[] = {{&S80888[44], 11}, {NULL}};
const Segment SGI199005[] = {{&S70232[44], 13}, {NULL}};
const Segment SGI199006[] = {{&S80888[44], 23}, {NULL}};
const Segment SGI199007[] = {{&S70232[44], 25}, {NULL}};
const Segment SGI199008[] = {{&S80888[30], 6}, {NULL}};
const Segment SGI199009[] = {{&S70232[30], 6}, {NULL}};
const Segment SGI199010[] = {{&S74995[115], 3}, {NULL}};
const Segment SGI199011[] = {{&S80888[31], 3}, {NULL}};
const Segment SGI199012[] = {{&S74995[80], 6}, {NULL}};
const Segment SGI199013[] = {{&S70232[118], 3}, {NULL}};
const Segment SGI199014[] = {{&S80888[81], 6}, {NULL}};
const Segment SGI199015[] = {{&S74995[143], 3}, {NULL}};
const Segment SGI199016[] = {{&S70232[83], 6}, {NULL}};
const Segment SGI199017[] = {{&S70234[34], 10}, {NULL}};
const Segment SGI199018[] = {{&S70234[27], 6}, {NULL}};
const Segment SGI199019[] = {{&S70234[97], 9}, {NULL}};
const Segment SGI199020[] = {{&S70234[70], 6}, {NULL}};
const Segment SGI199021[] = {{&S74998[44], 10}, {NULL}};
const Segment SGI199022[] = {{&S74998[44], 22}, {NULL}};
const Segment SGI199023[] = {{&S74998[30], 6}, {NULL}};
const Segment SGI199024[] = {{&S74998[80], 6}, {NULL}};
const Segment SGI199025[] = {{&S74998[107], 13}, {NULL}};
const Segment SGI199026[] = {{&S70235[34], 11}, {NULL}};
const Segment SGI199027[] = {{&S70235[27], 6}, {NULL}};
const Segment SGI199028[] = {{&S70235[98], 9}, {NULL}};
const Segment SGI199029[] = {{&S70235[71], 6}, {NULL}};
const Segment SGI199030[] = {{&S75000[34], 21}, {NULL}};
const Segment SGI199031[] = {{&S75000[27], 6}, {NULL}};
const Segment SGI199032[] = {{&S75000[108], 9}, {NULL}};
const Segment SGI199033[] = {{&S75000[81], 6}, {NULL}};
const Segment SGI199034[] = {{&S80897[34], 18}, {NULL}};
const Segment SGI199035[] = {{&S80897[27], 6}, {NULL}};
const Segment SGI199036[] = {{&S80897[105], 9}, {NULL}};
const Segment SGI199037[] = {{&S80897[78], 6}, {NULL}};
const Segment SGI199038[] = {{&S80898[34], 19}, {NULL}};
const Segment SGI199039[] = {{&S80898[27], 6}, {NULL}};
const Segment SGI199040[] = {{&S80898[106], 9}, {NULL}};
const Segment SGI199041[] = {{&S80898[79], 6}, {NULL}};
const Segment SGI199042[] = {{&S80905[34], 7}, {NULL}};
const Segment SGI199043[] = {{&S80905[27], 6}, {NULL}};
const Segment SGI199044[] = {{&S80905[99], 9}, {NULL}};
const Segment SGI199045[] = {{&S80905[72], 6}, {NULL}};
const Segment SGI199046[] = {{&S80906[44], 12}, {NULL}};
const Segment SGI199047[] = {{&S80906[44], 24}, {NULL}};
const Segment SGI199048[] = {{&S80906[30], 6}, {NULL}};
const Segment SGI199049[] = {{&S80906[31], 3}, {NULL}};
const Segment SGI199050[] = {{&S80906[82], 6}, {NULL}};
const Segment SGI199051[] = {{&S80916[44], 10}, {NULL}};
const Segment SGI199052[] = {{&S80916[44], 22}, {NULL}};
const Segment SGI199053[] = {{&S80916[30], 6}, {NULL}};
const Segment SGI199054[] = {{&S80916[80], 6}, {NULL}};
const Segment SGI199055[] = {{&S80916[107], 13}, {NULL}};
const Segment SGI199056[] = {{&S80918[34], 10}, {NULL}};
const Segment SGI199057[] = {{&S80918[27], 6}, {NULL}};
const Segment SGI199058[] = {{&S80918[97], 9}, {NULL}};
const Segment SGI199059[] = {{&S80918[70], 6}, {NULL}};
const Segment SGI199060[] = {{&S75033[44], 10}, {NULL}};
const Segment SGI199061[] = {{&S75033[44], 22}, {NULL}};
const Segment SGI199062[] = {{&S75033[30], 6}, {NULL}};
const Segment SGI199063[] = {{&S75033[80], 6}, {NULL}};
const Segment SGI199064[] = {{&S75033[107], 13}, {NULL}};
const Segment SGI199065[] = {{&S75038[34], 25}, {NULL}};
const Segment SGI199066[] = {{&S75038[27], 6}, {NULL}};
const Segment SGI199067[] = {{&S75038[112], 9}, {NULL}};
const Segment SGI199068[] = {{&S75038[85], 6}, {NULL}};
const Segment SGI199069[] = {{&S75041[34], 8}, {NULL}};
const Segment SGI199070[] = {{&S75041[27], 6}, {NULL}};
const Segment SGI199071[] = {{&S75041[95], 9}, {NULL}};
const Segment SGI199072[] = {{&S75041[68], 6}, {NULL}};
const Segment SGI199073[] = {{&S70284[34], 9}, {NULL}};
const Segment SGI199074[] = {{&S70284[27], 6}, {NULL}};
const Segment SGI199075[] = {{&S70284[96], 9}, {NULL}};
const Segment SGI199076[] = {{&S70284[69], 6}, {NULL}};
const Segment SGI199077[] = {{&S70285[34], 9}, {NULL}};
const Segment SGI199078[] = {{&S70285[27], 6}, {NULL}};
const Segment SGI199079[] = {{&S70285[96], 9}, {NULL}};
const Segment SGI199080[] = {{&S70285[69], 6}, {NULL}};
const Segment SGI199081[] = {{&S70287[34], 10}, {NULL}};
const Segment SGI199082[] = {{&S70287[27], 6}, {NULL}};
const Segment SGI199083[] = {{&S70287[97], 9}, {NULL}};
const Segment SGI199084[] = {{&S70287[70], 6}, {NULL}};
const Segment SGI199085[] = {{&S70288[34], 18}, {NULL}};
const Segment SGI199086[] = {{&S70288[27], 6}, {NULL}};
const Segment SGI199087[] = {{&S70288[105], 9}, {NULL}};
const Segment SGI199088[] = {{&S70288[78], 6}, {NULL}};
const Segment SGI199089[] = {{&S70289[34], 10}, {NULL}};
const Segment SGI199090[] = {{&S70289[27], 6}, {NULL}};
const Segment SGI199091[] = {{&S70289[97], 9}, {NULL}};
const Segment SGI199092[] = {{&S70289[70], 6}, {NULL}};
const Segment SGI199093[] = {{&S70290[34], 11}, {NULL}};
const Segment SGI199094[] = {{&S70290[27], 6}, {NULL}};
const Segment SGI199095[] = {{&S70290[98], 9}, {NULL}};
const Segment SGI199096[] = {{&S70290[71], 6}, {NULL}};
const Segment SGI199097[] = {{&S80950[34], 10}, {NULL}};
const Segment SGI199098[] = {{&S80950[27], 6}, {NULL}};
const Segment SGI199099[] = {{&S70291[34], 11}, {NULL}};
const Segment SGI199100[] = {{&S70291[27], 6}, {NULL}};
const Segment SGI199101[] = {{&S80950[97], 9}, {NULL}};
const Segment SGI199102[] = {{&S80950[70], 6}, {NULL}};
const Segment SGI199103[] = {{&S70291[98], 9}, {NULL}};
const Segment SGI199104[] = {{&S70291[71], 6}, {NULL}};
const Segment SGI199105[] = {{&S70292[34], 10}, {NULL}};
const Segment SGI199106[] = {{&S70292[27], 6}, {NULL}};
const Segment SGI199107[] = {{&S70292[97], 9}, {NULL}};
const Segment SGI199108[] = {{&S70292[70], 6}, {NULL}};
const Segment SGI199109[] = {{&S80959[34], 4}, {NULL}};
const Segment SGI199110[] = {{&S80959[27], 6}, {NULL}};
const Segment SGI199111[] = {{&S80959[91], 9}, {NULL}};
const Segment SGI199112[] = {{&S80959[64], 6}, {NULL}};
const Segment SGI199113[] = {{&S75070[34], 8}, {NULL}};
const Segment SGI199114[] = {{&S75070[27], 6}, {NULL}};
const Segment SGI199115[] = {{&S75070[95], 9}, {NULL}};
const Segment SGI199116[] = {{&S75070[68], 6}, {NULL}};
const Segment SGI199117[] = {{&S80962[34], 11}, {NULL}};
const Segment SGI199118[] = {{&S80962[27], 6}, {NULL}};
const Segment SGI199119[] = {{&S80962[110], 9}, {NULL}};
const Segment SGI199120[] = {{&S80962[83], 6}, {NULL}};
const Segment SGI199121[] = {{&S75071[44], 11}, {NULL}};
const Segment SGI199122[] = {{&S75071[44], 23}, {NULL}};
const Segment SGI199123[] = {{&S75071[30], 6}, {NULL}};
const Segment SGI199124[] = {{&S75071[116], 3}, {NULL}};
const Segment SGI199125[] = {{&S75071[81], 6}, {NULL}};
const Segment SGI199126[] = {{&S80964[44], 17}, {NULL}};
const Segment SGI199127[] = {{&S80964[44], 29}, {NULL}};
const Segment SGI199128[] = {{&S80964[30], 6}, {NULL}};
const Segment SGI199129[] = {{&S60687[34], 11}, {NULL}};
const Segment SGI199130[] = {{&S60687[27], 6}, {NULL}};
const Segment SGI199131[] = {{&S80964[31], 3}, {NULL}};
const Segment SGI199132[] = {{&S80964[87], 6}, {NULL}};
const Segment SGI199133[] = {{&S75073[44], 11}, {NULL}};
const Segment SGI199134[] = {{&S75073[44], 23}, {NULL}};
const Segment SGI199135[] = {{&S75073[30], 6}, {NULL}};
const Segment SGI199136[] = {{&S75073[116], 3}, {NULL}};
const Segment SGI199137[] = {{&S75073[81], 6}, {NULL}};
const Segment SGI199138[] = {{&S80981[34], 10}, {NULL}};
const Segment SGI199139[] = {{&S80981[27], 6}, {NULL}};
const Segment SGI199140[] = {{&S80981[97], 9}, {NULL}};
const Segment SGI199141[] = {{&S80981[70], 6}, {NULL}};
const Segment SGI199142[] = {{&S80986[34], 10}, {NULL}};
const Segment SGI199143[] = {{&S80986[27], 6}, {NULL}};
const Segment SGI199144[] = {{&S80986[97], 9}, {NULL}};
const Segment SGI199145[] = {{&S80986[70], 6}, {NULL}};
const Segment SGI199146[] = {{&S69616[34], 10}, {NULL}};
const Segment SGI199147[] = {{&S69616[27], 6}, {NULL}};
const Segment SGI199148[] = {{&S69616[97], 9}, {NULL}};
const Segment SGI199149[] = {{&S69616[70], 6}, {NULL}};
const Segment SGI199150[] = {{&S70348[44], 10}, {NULL}};
const Segment SGI199151[] = {{&S70348[44], 22}, {NULL}};
const Segment SGI199152[] = {{&S70348[30], 6}, {NULL}};
const Segment SGI199153[] = {{&S70348[131], 3}, {NULL}};
const Segment SGI199154[] = {{&S70348[96], 6}, {NULL}};
const Segment SGI199155[] = {{&S70348[68], 8}, {NULL}};
const Segment SGI199156[] = {{&S81004[34], 7}, {NULL}};
const Segment SGI199157[] = {{&S81004[27], 6}, {NULL}};
const Segment SGI199158[] = {{&S81004[99], 9}, {NULL}};
const Segment SGI199159[] = {{&S81004[72], 6}, {NULL}};
const Segment SGI199160[] = {{&S70353[34], 11}, {NULL}};
const Segment SGI199161[] = {{&S70353[27], 6}, {NULL}};
const Segment SGI199162[] = {{&S70353[98], 9}, {NULL}};
const Segment SGI199163[] = {{&S70353[71], 6}, {NULL}};
const Segment SGI199164[] = {{&S81006[44], 12}, {NULL}};
const Segment SGI199165[] = {{&S81006[44], 40}, {NULL}};
const Segment SGI199166[] = {{&S81006[30], 6}, {NULL}};
const Segment SGI199167[] = {{&S81006[31], 3}, {NULL}};
const Segment SGI199168[] = {{&S81006[98], 6}, {NULL}};
const Segment SGI199169[] = {{&S81006[157], 10}, {NULL}};
const Segment SGI199170[] = {{&S70356[34], 8}, {NULL}};
const Segment SGI199171[] = {{&S70356[27], 6}, {NULL}};
const Segment SGI199172[] = {{&S70356[103], 9}, {NULL}};
const Segment SGI199173[] = {{&S70356[76], 6}, {NULL}};
const Segment SGI199174[] = {{&S70357[34], 18}, {NULL}};
const Segment SGI199175[] = {{&S70357[27], 6}, {NULL}};
const Segment SGI199176[] = {{&S70357[113], 9}, {NULL}};
const Segment SGI199177[] = {{&S70357[86], 6}, {NULL}};
const Segment SGI199178[] = {{&S70358[34], 18}, {NULL}};
const Segment SGI199179[] = {{&S70358[27], 6}, {NULL}};
const Segment SGI199180[] = {{&S81011[34], 13}, {NULL}};
const Segment SGI199181[] = {{&S70358[113], 9}, {NULL}};
const Segment SGI199182[] = {{&S70358[86], 6}, {NULL}};
const Segment SGI199183[] = {{&S81011[27], 6}, {NULL}};
const Segment SGI199184[] = {{&S81011[100], 9}, {NULL}};
const Segment SGI199185[] = {{&S81011[73], 6}, {NULL}};
const Segment SGI199186[] = {{&S70359[34], 18}, {NULL}};
const Segment SGI199187[] = {{&S70359[27], 6}, {NULL}};
const Segment SGI199188[] = {{&S70359[113], 9}, {NULL}};
const Segment SGI199189[] = {{&S81012[34], 13}, {NULL}};
const Segment SGI199190[] = {{&S70359[86], 6}, {NULL}};
const Segment SGI199191[] = {{&S81012[27], 6}, {NULL}};
const Segment SGI199192[] = {{&S81012[113], 9}, {NULL}};
const Segment SGI199193[] = {{&S81012[86], 6}, {NULL}};
const Segment SGI199194[] = {{&S70360[34], 20}, {NULL}};
const Segment SGI199195[] = {{&S70360[27], 6}, {NULL}};
const Segment SGI199196[] = {{&S70360[115], 9}, {NULL}};
const Segment SGI199197[] = {{&S70360[88], 6}, {NULL}};
const Segment SGI199198[] = {{&S70361[34], 15}, {NULL}};
const Segment SGI199199[] = {{&S81015[44], 8}, {NULL}};
const Segment SGI199200[] = {{&S70361[27], 6}, {NULL}};
const Segment SGI199201[] = {{&S81015[44], 20}, {NULL}};
const Segment SGI199202[] = {{&S81015[30], 6}, {NULL}};
const Segment SGI199203[] = {{&S70361[110], 9}, {NULL}};
const Segment SGI199204[] = {{&S81015[31], 3}, {NULL}};
const Segment SGI199205[] = {{&S70361[83], 6}, {NULL}};
const Segment SGI199206[] = {{&S81015[78], 6}, {NULL}};
const Segment SGI199207[] = {{&S81018[44], 11}, {NULL}};
const Segment SGI199208[] = {{&S81018[44], 23}, {NULL}};
const Segment SGI199209[] = {{&S81018[30], 6}, {NULL}};
const Segment SGI199210[] = {{&S81018[84], 6}, {NULL}};
const Segment SGI199211[] = {{&S81019[44], 11}, {NULL}};
const Segment SGI199212[] = {{&S81019[44], 23}, {NULL}};
const Segment SGI199213[] = {{&S81019[30], 6}, {NULL}};
const Segment SGI199214[] = {{&S81019[31], 3}, {NULL}};
const Segment SGI199215[] = {{&S81019[81], 6}, {NULL}};
const Segment SGI199216[] = {{&S81020[38], 11}, {NULL}};
const Segment SGI199217[] = {{&S81020[31], 6}, {NULL}};
const Segment SGI199218[] = {{&S81024[34], 8}, {NULL}};
const Segment SGI199219[] = {{&S81024[27], 6}, {NULL}};
const Segment SGI199220[] = {{&S81024[94], 9}, {NULL}};
const Segment SGI199221[] = {{&S81024[67], 6}, {NULL}};
const Segment SGI199222[] = {{&S75152[34], 25}, {NULL}};
const Segment SGI199223[] = {{&S75152[27], 6}, {NULL}};
const Segment SGI199224[] = {{&S75152[112], 9}, {NULL}};
const Segment SGI199225[] = {{&S75152[85], 6}, {NULL}};
const Segment SGI199226[] = {{&S70391[34], 9}, {NULL}};
const Segment SGI199227[] = {{&S70391[27], 6}, {NULL}};
const Segment SGI199228[] = {{&S70391[96], 9}, {NULL}};
const Segment SGI199229[] = {{&S70391[69], 6}, {NULL}};
const Segment SGI199230[] = {{&S75157[44], 10}, {NULL}};
const Segment SGI199231[] = {{&S75157[44], 22}, {NULL}};
const Segment SGI199232[] = {{&S75157[30], 6}, {NULL}};
const Segment SGI199233[] = {{&S75157[80], 6}, {NULL}};
const Segment SGI199234[] = {{&S75157[107], 13}, {NULL}};
const Segment SGI199235[] = {{&S70402[34], 4}, {NULL}};
const Segment SGI199236[] = {{&S70402[27], 6}, {NULL}};
const Segment SGI199237[] = {{&S70402[91], 9}, {NULL}};
const Segment SGI199238[] = {{&S70402[64], 6}, {NULL}};
const Segment SGI199239[] = {{&S81067[34], 18}, {NULL}};
const Segment SGI199240[] = {{&S81067[27], 6}, {NULL}};
const Segment SGI199241[] = {{&S81067[105], 9}, {NULL}};
const Segment SGI199242[] = {{&S81067[78], 6}, {NULL}};
const Segment SGI199243[] = {{&S75189[34], 5}, {NULL}};
const Segment SGI199244[] = {{&S75189[27], 6}, {NULL}};
const Segment SGI199245[] = {{&S75189[92], 9}, {NULL}};
const Segment SGI199246[] = {{&S75189[65], 6}, {NULL}};
const Segment SGI199247[] = {{&S81078[34], 7}, {NULL}};
const Segment SGI199248[] = {{&S81078[27], 6}, {NULL}};
const Segment SGI199249[] = {{&S81078[94], 9}, {NULL}};
const Segment SGI199250[] = {{&S81078[67], 6}, {NULL}};
const Segment SGI199251[] = {{&S60687[97], 9}, {NULL}};
const Segment SGI199252[] = {{&S60687[70], 6}, {NULL}};
const Segment SGI199253[] = {{&S81079[34], 11}, {NULL}};
const Segment SGI199254[] = {{&S81079[27], 6}, {NULL}};
const Segment SGI199255[] = {{&S81079[98], 9}, {NULL}};
const Segment SGI199256[] = {{&S81079[71], 6}, {NULL}};
const Segment SGI199257[] = {{&S81086[34], 8}, {NULL}};
const Segment SGI199258[] = {{&S81086[27], 6}, {NULL}};
const Segment SGI199259[] = {{&S81086[94], 9}, {NULL}};
const Segment SGI199260[] = {{&S81086[67], 6}, {NULL}};
const Segment SGI199261[] = {{&S81100[34], 7}, {NULL}};
const Segment SGI199262[] = {{&S81100[27], 6}, {NULL}};
const Segment SGI199263[] = {{&S81100[94], 9}, {NULL}};
const Segment SGI199264[] = {{&S81100[67], 6}, {NULL}};
const Segment SGI199265[] = {{&S81100[128], 8}, {NULL}};
const Segment SGI199266[] = {{&S81101[44], 10}, {NULL}};
const Segment SGI199267[] = {{&S81101[44], 41}, {NULL}};
const Segment SGI199268[] = {{&S81101[30], 6}, {NULL}};
const Segment SGI199269[] = {{&S81101[31], 3}, {NULL}};
const Segment SGI199270[] = {{&S81101[125], 6}, {NULL}};
const Segment SGI199271[] = {{&S81101[87], 8}, {NULL}};
const Segment SGI199272[] = {{&S81102[44], 10}, {NULL}};
const Segment SGI199273[] = {{&S81102[44], 22}, {NULL}};
const Segment SGI199274[] = {{&S81102[30], 6}, {NULL}};
const Segment SGI199275[] = {{&S81102[31], 3}, {NULL}};
const Segment SGI199276[] = {{&S81102[106], 6}, {NULL}};
const Segment SGI199277[] = {{&S81102[68], 8}, {NULL}};
const Segment SGI199278[] = {{&S81103[44], 10}, {NULL}};
const Segment SGI199279[] = {{&S81103[44], 22}, {NULL}};
const Segment SGI199280[] = {{&S81103[30], 6}, {NULL}};
const Segment SGI199281[] = {{&S81103[31], 3}, {NULL}};
const Segment SGI199282[] = {{&S81103[105], 6}, {NULL}};
const Segment SGI199283[] = {{&S81103[68], 8}, {NULL}};
const Segment SGI199284[] = {{&S81104[44], 10}, {NULL}};
const Segment SGI199285[] = {{&S81104[44], 28}, {NULL}};
const Segment SGI199286[] = {{&S81104[30], 6}, {NULL}};
const Segment SGI199287[] = {{&S81104[31], 3}, {NULL}};
const Segment SGI199288[] = {{&S81104[109], 6}, {NULL}};
const Segment SGI199289[] = {{&S81104[74], 8}, {NULL}};
const Segment SGI199290[] = {{&S81105[34], 18}, {NULL}};
const Segment SGI199291[] = {{&S81105[27], 6}, {NULL}};
const Segment SGI199292[] = {{&S81105[105], 9}, {NULL}};
const Segment SGI199293[] = {{&S81105[78], 6}, {NULL}};
const Segment SGI199294[] = {{&S81107[44], 5}, {NULL}};
const Segment SGI199295[] = {{&S81107[44], 17}, {NULL}};
const Segment SGI199296[] = {{&S81107[30], 6}, {NULL}};
const Segment SGI199297[] = {{&S81107[75], 6}, {NULL}};
const Segment SGI199298[] = {{&S81107[102], 13}, {NULL}};
const Segment SGI199299[] = {{&S70439[34], 10}, {NULL}};
const Segment SGI199300[] = {{&S70439[27], 6}, {NULL}};
const Segment SGI199301[] = {{&S70439[97], 9}, {NULL}};
const Segment SGI199302[] = {{&S70439[70], 6}, {NULL}};
const Segment SGI199303[] = {{&S70440[34], 11}, {NULL}};
const Segment SGI199304[] = {{&S70440[27], 6}, {NULL}};
const Segment SGI199305[] = {{&S70440[98], 9}, {NULL}};
const Segment SGI199306[] = {{&S70440[71], 6}, {NULL}};
const Segment SGI199307[] = {{&S81138[44], 12}, {NULL}};
const Segment SGI199308[] = {{&S81138[44], 24}, {NULL}};
const Segment SGI199309[] = {{&S81138[30], 6}, {NULL}};
const Segment SGI199310[] = {{&S81138[31], 3}, {NULL}};
const Segment SGI199311[] = {{&S81138[82], 6}, {NULL}};
const Segment SGI199312[] = {{&S81139[44], 9}, {NULL}};
const Segment SGI199313[] = {{&S81139[44], 21}, {NULL}};
const Segment SGI199314[] = {{&S81139[30], 6}, {NULL}};
const Segment SGI199315[] = {{&S81139[31], 3}, {NULL}};
const Segment SGI199316[] = {{&S81139[79], 6}, {NULL}};
const Segment SGI199317[] = {{&S70469[34], 8}, {NULL}};
const Segment SGI199318[] = {{&S70469[27], 6}, {NULL}};
const Segment SGI199319[] = {{&S70469[95], 9}, {NULL}};
const Segment SGI199320[] = {{&S70469[68], 6}, {NULL}};
const Segment SGI199321[] = {{&S70472[34], 13}, {NULL}};
const Segment SGI199322[] = {{&S70472[27], 6}, {NULL}};
const Segment SGI199323[] = {{&S70472[100], 9}, {NULL}};
const Segment SGI199324[] = {{&S70472[73], 6}, {NULL}};
const Segment SGI199325[] = {{&S81144[34], 7}, {NULL}};
const Segment SGI199326[] = {{&S81144[27], 6}, {NULL}};
const Segment SGI199327[] = {{&S81144[94], 9}, {NULL}};
const Segment SGI199328[] = {{&S81144[67], 6}, {NULL}};
const Segment SGI199329[] = {{&S81154[34], 7}, {NULL}};
const Segment SGI199330[] = {{&S81154[27], 6}, {NULL}};
const Segment SGI199331[] = {{&S81154[99], 9}, {NULL}};
const Segment SGI199332[] = {{&S81154[72], 6}, {NULL}};
const Segment SGI199333[] = {{&S70510[44], 10}, {NULL}};
const Segment SGI199334[] = {{&S70510[44], 22}, {NULL}};
const Segment SGI199335[] = {{&S70510[30], 6}, {NULL}};
const Segment SGI199336[] = {{&S70510[115], 3}, {NULL}};
const Segment SGI199337[] = {{&S70510[80], 6}, {NULL}};
const Segment SGI199338[] = {{&S70511[38], 10}, {NULL}};
const Segment SGI199339[] = {{&S70511[31], 6}, {NULL}};
const Segment SGI199340[] = {{&S70515[44], 10}, {NULL}};
const Segment SGI199341[] = {{&S70515[44], 22}, {NULL}};
const Segment SGI199342[] = {{&S70515[30], 6}, {NULL}};
const Segment SGI199343[] = {{&S70515[83], 6}, {NULL}};
const Segment SGI199344[] = {{&S81186[34], 9}, {NULL}};
const Segment SGI199345[] = {{&S81186[27], 6}, {NULL}};
const Segment SGI199346[] = {{&S81186[96], 9}, {NULL}};
const Segment SGI199347[] = {{&S81186[69], 6}, {NULL}};
const Segment SGI199348[] = {{&S70517[52], 10}, {NULL}};
const Segment SGI199349[] = {{&S70517[52], 22}, {NULL}};
const Segment SGI199350[] = {{&S70517[38], 6}, {NULL}};
const Segment SGI199351[] = {{&S70518[44], 10}, {NULL}};
const Segment SGI199352[] = {{&S70518[44], 22}, {NULL}};
const Segment SGI199353[] = {{&S70518[30], 6}, {NULL}};
const Segment SGI199354[] = {{&S70518[80], 6}, {NULL}};
const Segment SGI199355[] = {{&S70519[44], 10}, {NULL}};
const Segment SGI199356[] = {{&S70519[44], 22}, {NULL}};
const Segment SGI199357[] = {{&S70519[30], 6}, {NULL}};
const Segment SGI199358[] = {{&S70519[115], 3}, {NULL}};
const Segment SGI199359[] = {{&S70519[80], 6}, {NULL}};
const Segment SGI199360[] = {{&S70520[38], 11}, {NULL}};
const Segment SGI199361[] = {{&S70520[31], 6}, {NULL}};
const Segment SGI199362[] = {{&S70521[46], 10}, {NULL}};
const Segment SGI199363[] = {{&S70521[46], 63}, {NULL}};
const Segment SGI199364[] = {{&S70521[32], 6}, {NULL}};
const Segment SGI199365[] = {{&S70523[44], 9}, {NULL}};
const Segment SGI199366[] = {{&S70523[30], 6}, {NULL}};
const Segment SGI199367[] = {{&S70525[99], 28}, {NULL}};
const Segment SGI199368[] = {{&S81200[44], 10}, {NULL}};
const Segment SGI199369[] = {{&S81200[44], 22}, {NULL}};
const Segment SGI199370[] = {{&S81200[30], 6}, {NULL}};
const Segment SGI199371[] = {{&S81200[80], 6}, {NULL}};
const Segment SGI199372[] = {{&S81200[107], 13}, {NULL}};
const Segment SGI199373[] = {{&S70527[44], 11}, {NULL}};
const Segment SGI199374[] = {{&S70527[44], 23}, {NULL}};
const Segment SGI199375[] = {{&S70527[30], 6}, {NULL}};
const Segment SGI199376[] = {{&S81201[44], 11}, {NULL}};
const Segment SGI199377[] = {{&S70527[116], 3}, {NULL}};
const Segment SGI199378[] = {{&S81201[44], 23}, {NULL}};
const Segment SGI199379[] = {{&S81201[30], 6}, {NULL}};
const Segment SGI199380[] = {{&S70527[81], 6}, {NULL}};
const Segment SGI199381[] = {{&S81201[81], 6}, {NULL}};
const Segment SGI199382[] = {{&S81201[108], 13}, {NULL}};
const Segment SGI199383[] = {{&S70530[34], 12}, {NULL}};
const Segment SGI199384[] = {{&S70530[27], 6}, {NULL}};
const Segment SGI199385[] = {{&S70530[99], 9}, {NULL}};
const Segment SGI199386[] = {{&S70530[72], 6}, {NULL}};
const Segment SGI199387[] = {{&S70531[34], 12}, {NULL}};
const Segment SGI199388[] = {{&S70531[27], 6}, {NULL}};
const Segment SGI199389[] = {{&S70531[99], 9}, {NULL}};
const Segment SGI199390[] = {{&S70531[72], 6}, {NULL}};
const Segment SGI199391[] = {{&S81209[34], 11}, {NULL}};
const Segment SGI199392[] = {{&S81209[27], 6}, {NULL}};
const Segment SGI199393[] = {{&S81209[98], 9}, {NULL}};
const Segment SGI199394[] = {{&S81209[71], 6}, {NULL}};
const Segment SGI199395[] = {{&S81211[34], 10}, {NULL}};
const Segment SGI199396[] = {{&S81211[27], 6}, {NULL}};
const Segment SGI199397[] = {{&S81211[97], 9}, {NULL}};
const Segment SGI199398[] = {{&S81211[70], 6}, {NULL}};
const Segment SGI199399[] = {{&S70542[44], 10}, {NULL}};
const Segment SGI199400[] = {{&S70542[44], 21}, {NULL}};
const Segment SGI199401[] = {{&S70542[30], 6}, {NULL}};
const Segment SGI199402[] = {{&S70542[114], 3}, {NULL}};
const Segment SGI199403[] = {{&S70542[79], 6}, {NULL}};
const Segment SGI199404[] = {{&S70543[38], 10}, {NULL}};
const Segment SGI199405[] = {{&S70543[31], 6}, {NULL}};
const Segment SGI199406[] = {{&S70544[44], 10}, {NULL}};
const Segment SGI199407[] = {{&S70544[44], 21}, {NULL}};
const Segment SGI199408[] = {{&S70544[30], 6}, {NULL}};
const Segment SGI199409[] = {{&S70544[114], 3}, {NULL}};
const Segment SGI199410[] = {{&S70544[79], 6}, {NULL}};
const Segment SGI199411[] = {{&S81217[44], 10}, {NULL}};
const Segment SGI199412[] = {{&S81217[44], 22}, {NULL}};
const Segment SGI199413[] = {{&S81217[30], 6}, {NULL}};
const Segment SGI199414[] = {{&S70545[44], 15}, {NULL}};
const Segment SGI199415[] = {{&S81217[80], 6}, {NULL}};
const Segment SGI199416[] = {{&S70545[44], 27}, {NULL}};
const Segment SGI199417[] = {{&S70545[30], 6}, {NULL}};
const Segment SGI199418[] = {{&S81217[107], 13}, {NULL}};
const Segment SGI199419[] = {{&S70545[85], 6}, {NULL}};
const Segment SGI199420[] = {{&S70545[112], 13}, {NULL}};
const Segment SGI199421[] = {{&S70546[34], 7}, {NULL}};
const Segment SGI199422[] = {{&S70546[27], 6}, {NULL}};
const Segment SGI199423[] = {{&S70546[100], 9}, {NULL}};
const Segment SGI199424[] = {{&S75198[44], 9}, {NULL}};
const Segment SGI199425[] = {{&S70546[73], 6}, {NULL}};
const Segment SGI199426[] = {{&S75198[44], 21}, {NULL}};
const Segment SGI199427[] = {{&S75198[30], 6}, {NULL}};
const Segment SGI199428[] = {{&S75198[114], 3}, {NULL}};
const Segment SGI199429[] = {{&S75198[79], 6}, {NULL}};
const Segment SGI199430[] = {{&S75201[34], 25}, {NULL}};
const Segment SGI199431[] = {{&S75201[27], 6}, {NULL}};
const Segment SGI199432[] = {{&S75201[112], 9}, {NULL}};
const Segment SGI199433[] = {{&S75201[85], 6}, {NULL}};
const Segment SGI199434[] = {{&S75202[34], 25}, {NULL}};
const Segment SGI199435[] = {{&S75202[27], 6}, {NULL}};
const Segment SGI199436[] = {{&S70552[34], 10}, {NULL}};
const Segment SGI199437[] = {{&S70552[27], 6}, {NULL}};
const Segment SGI199438[] = {{&S75202[112], 9}, {NULL}};
const Segment SGI199439[] = {{&S75202[85], 6}, {NULL}};
const Segment SGI199440[] = {{&S70552[97], 9}, {NULL}};
const Segment SGI199441[] = {{&S70552[70], 6}, {NULL}};
const Segment SGI199442[] = {{&S70552[138], 8}, {NULL}};
const Segment SGI199443[] = {{&S75203[34], 25}, {NULL}};
const Segment SGI199444[] = {{&S81228[34], 11}, {NULL}};
const Segment SGI199445[] = {{&S75203[27], 6}, {NULL}};
const Segment SGI199446[] = {{&S81228[27], 6}, {NULL}};
const Segment SGI199447[] = {{&S75203[112], 9}, {NULL}};
const Segment SGI199448[] = {{&S81228[98], 9}, {NULL}};
const Segment SGI199449[] = {{&S75203[85], 6}, {NULL}};
const Segment SGI199450[] = {{&S81228[71], 6}, {NULL}};
const Segment SGI199451[] = {{&S81229[34], 11}, {NULL}};
const Segment SGI199452[] = {{&S81229[27], 6}, {NULL}};
const Segment SGI199453[] = {{&S81229[98], 9}, {NULL}};
const Segment SGI199454[] = {{&S81229[71], 6}, {NULL}};
const Segment SGI199455[] = {{&S75209[34], 12}, {NULL}};
const Segment SGI199456[] = {{&S75209[27], 6}, {NULL}};
const Segment SGI199457[] = {{&S75209[99], 9}, {NULL}};
const Segment SGI199458[] = {{&S75209[72], 6}, {NULL}};
const Segment SGI199459[] = {{&S70577[34], 12}, {NULL}};
const Segment SGI199460[] = {{&S70577[27], 6}, {NULL}};
const Segment SGI199461[] = {{&S70577[99], 9}, {NULL}};
const Segment SGI199462[] = {{&S70577[72], 6}, {NULL}};
const Segment SGI199463[] = {{&S81257[34], 10}, {NULL}};
const Segment SGI199464[] = {{&S81257[27], 6}, {NULL}};
const Segment SGI199465[] = {{&S81257[97], 9}, {NULL}};
const Segment SGI199466[] = {{&S81257[70], 6}, {NULL}};
const Segment SGI199467[] = {{&S81258[45], 7}, {NULL}};
const Segment SGI199468[] = {{&S81258[30], 6}, {NULL}};
const Segment SGI199469[] = {{&S81258[31], 3}, {NULL}};
const Segment SGI199470[] = {{&S81258[83], 5}, {NULL}};
const Segment SGI199471[] = {{&S81258[141], 13}, {NULL}};
const Segment SGI199472[] = {{&S70590[38], 6}, {NULL}};
const Segment SGI199473[] = {{&S70590[31], 6}, {NULL}};
const Segment SGI199474[] = {{&S70591[44], 6}, {NULL}};
const Segment SGI199475[] = {{&S70591[44], 29}, {NULL}};
const Segment SGI199476[] = {{&S70591[30], 6}, {NULL}};
const Segment SGI199477[] = {{&S70591[122], 3}, {NULL}};
const Segment SGI199478[] = {{&S70591[87], 6}, {NULL}};
const Segment SGI199479[] = {{&S70596[34], 10}, {NULL}};
const Segment SGI199480[] = {{&S70596[27], 6}, {NULL}};
const Segment SGI199481[] = {{&S70596[97], 9}, {NULL}};
const Segment SGI199482[] = {{&S70596[70], 6}, {NULL}};
const Segment SGI199483[] = {{&S70597[34], 10}, {NULL}};
const Segment SGI199484[] = {{&S70597[27], 6}, {NULL}};
const Segment SGI199485[] = {{&S70597[97], 9}, {NULL}};
const Segment SGI199486[] = {{&S70597[70], 6}, {NULL}};
const Segment SGI199487[] = {{&S70598[34], 10}, {NULL}};
const Segment SGI199488[] = {{&S70598[27], 6}, {NULL}};
const Segment SGI199489[] = {{&S81274[44], 8}, {NULL}};
const Segment SGI199490[] = {{&S81274[44], 26}, {NULL}};
const Segment SGI199491[] = {{&S81274[30], 6}, {NULL}};
const Segment SGI199492[] = {{&S70598[97], 9}, {NULL}};
const Segment SGI199493[] = {{&S70598[70], 6}, {NULL}};
const Segment SGI199494[] = {{&S81274[31], 3}, {NULL}};
const Segment SGI199495[] = {{&S81274[84], 6}, {NULL}};
const Segment SGI199496[] = {{&S70599[34], 11}, {NULL}};
const Segment SGI199497[] = {{&S70599[27], 6}, {NULL}};
const Segment SGI199498[] = {{&S81275[44], 8}, {NULL}};
const Segment SGI199499[] = {{&S81275[44], 15}, {NULL}};
const Segment SGI199500[] = {{&S81275[30], 6}, {NULL}};
const Segment SGI199501[] = {{&S70599[98], 9}, {NULL}};
const Segment SGI199502[] = {{&S70599[71], 6}, {NULL}};
const Segment SGI199503[] = {{&S81275[31], 3}, {NULL}};
const Segment SGI199504[] = {{&S81275[73], 6}, {NULL}};
const Segment SGI199505[] = {{&S81296[44], 7}, {NULL}};
const Segment SGI199506[] = {{&S81296[44], 23}, {NULL}};
const Segment SGI199507[] = {{&S81296[30], 6}, {NULL}};
const Segment SGI199508[] = {{&S81296[81], 6}, {NULL}};
const Segment SGI199509[] = {{&S81296[108], 13}, {NULL}};
const Segment SGI199510[] = {{&S81297[44], 7}, {NULL}};
const Segment SGI199511[] = {{&S81297[44], 23}, {NULL}};
const Segment SGI199512[] = {{&S81297[30], 6}, {NULL}};
const Segment SGI199513[] = {{&S81297[81], 6}, {NULL}};
const Segment SGI199514[] = {{&S81297[108], 13}, {NULL}};
const Segment SGI199515[] = {{&S81298[44], 7}, {NULL}};
const Segment SGI199516[] = {{&S81298[44], 24}, {NULL}};
const Segment SGI199517[] = {{&S81298[30], 6}, {NULL}};
const Segment SGI199518[] = {{&S81298[82], 6}, {NULL}};
const Segment SGI199519[] = {{&S81298[109], 13}, {NULL}};
const Segment SGI199520[] = {{&S81302[34], 18}, {NULL}};
const Segment SGI199521[] = {{&S81302[27], 6}, {NULL}};
const Segment SGI199522[] = {{&S81302[105], 9}, {NULL}};
const Segment SGI199523[] = {{&S81302[78], 6}, {NULL}};
const Segment SGI199524[] = {{&S81302[146], 8}, {NULL}};
const Segment SGI199525[] = {{&S70631[34], 9}, {NULL}};
const Segment SGI199526[] = {{&S70631[27], 6}, {NULL}};
const Segment SGI199527[] = {{&S70631[96], 9}, {NULL}};
const Segment SGI199528[] = {{&S70631[69], 6}, {NULL}};
const Segment SGI199529[] = {{&S70632[34], 10}, {NULL}};
const Segment SGI199530[] = {{&S70632[27], 6}, {NULL}};
const Segment SGI199531[] = {{&S70632[97], 9}, {NULL}};
const Segment SGI199532[] = {{&S70632[70], 6}, {NULL}};
const Segment SGI199533[] = {{&S70633[34], 9}, {NULL}};
const Segment SGI199534[] = {{&S70633[27], 6}, {NULL}};
const Segment SGI199535[] = {{&S70633[96], 9}, {NULL}};
const Segment SGI199536[] = {{&S70633[69], 6}, {NULL}};
const Segment SGI199537[] = {{&S70634[34], 10}, {NULL}};
const Segment SGI199538[] = {{&S70634[27], 6}, {NULL}};
const Segment SGI199539[] = {{&S70634[97], 9}, {NULL}};
const Segment SGI199540[] = {{&S70634[70], 6}, {NULL}};
const Segment SGI199541[] = {{&S69960[44], 6}, {NULL}};
const Segment SGI199542[] = {{&S69960[44], 18}, {NULL}};
const Segment SGI199543[] = {{&S69960[30], 6}, {NULL}};
const Segment SGI199544[] = {{&S69960[111], 3}, {NULL}};
const Segment SGI199545[] = {{&S69960[76], 6}, {NULL}};
const Segment SGI199546[] = {{&S69961[38], 6}, {NULL}};
const Segment SGI199547[] = {{&S69961[31], 6}, {NULL}};
const Segment SGI199548[] = {{&S70646[44], 6}, {NULL}};
const Segment SGI199549[] = {{&S70646[44], 18}, {NULL}};
const Segment SGI199550[] = {{&S70646[30], 6}, {NULL}};
const Segment SGI199551[] = {{&S70646[127], 3}, {NULL}};
const Segment SGI199552[] = {{&S70646[92], 6}, {NULL}};
const Segment SGI199553[] = {{&S70646[64], 8}, {NULL}};
const Segment SGI199554[] = {{&S70649[34], 10}, {NULL}};
const Segment SGI199555[] = {{&S70649[27], 6}, {NULL}};
const Segment SGI199556[] = {{&S70649[97], 9}, {NULL}};
const Segment SGI199557[] = {{&S70649[70], 6}, {NULL}};
const Segment SGI199558[] = {{&S81327[44], 8}, {NULL}};
const Segment SGI199559[] = {{&S81327[44], 20}, {NULL}};
const Segment SGI199560[] = {{&S81327[30], 6}, {NULL}};
const Segment SGI199561[] = {{&S81327[31], 3}, {NULL}};
const Segment SGI199562[] = {{&S81327[78], 6}, {NULL}};
const Segment SGI199563[] = {{&S81330[31], 8}, {NULL}};
const Segment SGI199564[] = {{&S81330[55], 3}, {NULL}};
const Segment SGI199565[] = {{&S81331[44], 13}, {NULL}};
const Segment SGI199566[] = {{&S81331[44], 25}, {NULL}};
const Segment SGI199567[] = {{&S81331[30], 6}, {NULL}};
const Segment SGI199568[] = {{&S81331[31], 3}, {NULL}};
const Segment SGI199569[] = {{&S81331[83], 6}, {NULL}};
const Segment SGI199570[] = {{&S81333[44], 13}, {NULL}};
const Segment SGI199571[] = {{&S81333[44], 25}, {NULL}};
const Segment SGI199572[] = {{&S81333[30], 6}, {NULL}};
const Segment SGI199573[] = {{&S81333[31], 3}, {NULL}};
const Segment SGI199574[] = {{&S81333[83], 6}, {NULL}};
const Segment SGI199575[] = {{&S81334[34], 10}, {NULL}};
const Segment SGI199576[] = {{&S81334[27], 6}, {NULL}};
const Segment SGI199577[] = {{&S81334[97], 9}, {NULL}};
const Segment SGI199578[] = {{&S81334[70], 6}, {NULL}};
const Segment SGI199579[] = {{&S81340[34], 8}, {NULL}};
const Segment SGI199580[] = {{&S81340[27], 6}, {NULL}};
const Segment SGI199581[] = {{&S81340[95], 9}, {NULL}};
const Segment SGI199582[] = {{&S81340[68], 6}, {NULL}};
const Segment SGI199583[] = {{&S81341[34], 10}, {NULL}};
const Segment SGI199584[] = {{&S81341[27], 6}, {NULL}};
const Segment SGI199585[] = {{&S81341[97], 9}, {NULL}};
const Segment SGI199586[] = {{&S81341[70], 6}, {NULL}};
const Segment SGI199587[] = {{&S81347[34], 7}, {NULL}};
const Segment SGI199588[] = {{&S81347[27], 6}, {NULL}};
const Segment SGI199589[] = {{&S81347[104], 9}, {NULL}};
const Segment SGI199590[] = {{&S81347[77], 6}, {NULL}};
const Segment SGI199591[] = {{&S60775[44], 7}, {NULL}};
const Segment SGI199592[] = {{&S60775[44], 18}, {NULL}};
const Segment SGI199593[] = {{&S60775[30], 6}, {NULL}};
const Segment SGI199594[] = {{&S70685[44], 6}, {NULL}};
const Segment SGI199595[] = {{&S60775[129], 3}, {NULL}};
const Segment SGI199596[] = {{&S70685[44], 19}, {NULL}};
const Segment SGI199597[] = {{&S70685[30], 6}, {NULL}};
const Segment SGI199598[] = {{&S60775[94], 6}, {NULL}};
const Segment SGI199599[] = {{&S60775[64], 8}, {NULL}};
const Segment SGI199600[] = {{&S70685[112], 3}, {NULL}};
const Segment SGI199601[] = {{&S70685[77], 6}, {NULL}};
const Segment SGI199602[] = {{&S81355[34], 12}, {NULL}};
const Segment SGI199603[] = {{&S81355[27], 6}, {NULL}};
const Segment SGI199604[] = {{&S60776[44], 7}, {NULL}};
const Segment SGI199605[] = {{&S60776[44], 19}, {NULL}};
const Segment SGI199606[] = {{&S81355[99], 9}, {NULL}};
const Segment SGI199607[] = {{&S60776[30], 6}, {NULL}};
const Segment SGI199608[] = {{&S81355[72], 6}, {NULL}};
const Segment SGI199609[] = {{&S70686[44], 6}, {NULL}};
const Segment SGI199610[] = {{&S70686[44], 19}, {NULL}};
const Segment SGI199611[] = {{&S70686[30], 6}, {NULL}};
const Segment SGI199612[] = {{&S60776[132], 3}, {NULL}};
const Segment SGI199613[] = {{&S60776[97], 6}, {NULL}};
const Segment SGI199614[] = {{&S70686[112], 3}, {NULL}};
const Segment SGI199615[] = {{&S60776[65], 8}, {NULL}};
const Segment SGI199616[] = {{&S70686[77], 6}, {NULL}};
const Segment SGI199617[] = {{&S70687[38], 6}, {NULL}};
const Segment SGI199618[] = {{&S70687[31], 6}, {NULL}};
const Segment SGI199619[] = {{&S70688[44], 6}, {NULL}};
const Segment SGI199620[] = {{&S70688[44], 19}, {NULL}};
const Segment SGI199621[] = {{&S70688[30], 6}, {NULL}};
const Segment SGI199622[] = {{&S70688[112], 3}, {NULL}};
const Segment SGI199623[] = {{&S70688[77], 6}, {NULL}};
const Segment SGI199624[] = {{&S81361[34], 17}, {NULL}};
const Segment SGI199625[] = {{&S81361[27], 6}, {NULL}};
const Segment SGI199626[] = {{&S81361[112], 9}, {NULL}};
const Segment SGI199627[] = {{&S81361[85], 6}, {NULL}};
const Segment SGI199628[] = {{&S81362[34], 8}, {NULL}};
const Segment SGI199629[] = {{&S81362[27], 6}, {NULL}};
const Segment SGI199630[] = {{&S81362[111], 9}, {NULL}};
const Segment SGI199631[] = {{&S81362[84], 6}, {NULL}};
const Segment SGI199632[] = {{&S75247[44], 11}, {NULL}};
const Segment SGI199633[] = {{&S75247[44], 26}, {NULL}};
const Segment SGI199634[] = {{&S75247[30], 6}, {NULL}};
const Segment SGI199635[] = {{&S75247[118], 3}, {NULL}};
const Segment SGI199636[] = {{&S75247[84], 5}, {NULL}};
const Segment SGI199637[] = {{&S81371[34], 12}, {NULL}};
const Segment SGI199638[] = {{&S81371[27], 6}, {NULL}};
const Segment SGI199639[] = {{&S75253[34], 16}, {NULL}};
const Segment SGI199640[] = {{&S75253[27], 6}, {NULL}};
const Segment SGI199641[] = {{&S81371[99], 9}, {NULL}};
const Segment SGI199642[] = {{&S81371[72], 6}, {NULL}};
const Segment SGI199643[] = {{&S75253[103], 9}, {NULL}};
const Segment SGI199644[] = {{&S75253[76], 6}, {NULL}};
const Segment SGI199645[] = {{&S70031[34], 6}, {NULL}};
const Segment SGI199646[] = {{&S70031[27], 6}, {NULL}};
const Segment SGI199647[] = {{&S70031[93], 9}, {NULL}};
const Segment SGI199648[] = {{&S70031[66], 6}, {NULL}};
const Segment SGI199649[] = {{&S75254[44], 5}, {NULL}};
const Segment SGI199650[] = {{&S75254[44], 17}, {NULL}};
const Segment SGI199651[] = {{&S75254[30], 6}, {NULL}};
const Segment SGI199652[] = {{&S75254[110], 3}, {NULL}};
const Segment SGI199653[] = {{&S75254[75], 6}, {NULL}};
const Segment SGI199654[] = {{&S70707[34], 10}, {NULL}};
const Segment SGI199655[] = {{&S70707[27], 6}, {NULL}};
const Segment SGI199656[] = {{&S75255[38], 5}, {NULL}};
const Segment SGI199657[] = {{&S75255[31], 6}, {NULL}};
const Segment SGI199658[] = {{&S70707[97], 9}, {NULL}};
const Segment SGI199659[] = {{&S70707[70], 6}, {NULL}};
const Segment SGI199660[] = {{&S81375[44], 7}, {NULL}};
const Segment SGI199661[] = {{&S81375[44], 24}, {NULL}};
const Segment SGI199662[] = {{&S81375[30], 6}, {NULL}};
const Segment SGI199663[] = {{&S70708[34], 9}, {NULL}};
const Segment SGI199664[] = {{&S81375[82], 6}, {NULL}};
const Segment SGI199665[] = {{&S70708[27], 6}, {NULL}};
const Segment SGI199666[] = {{&S81375[109], 13}, {NULL}};
const Segment SGI199667[] = {{&S70708[96], 9}, {NULL}};
const Segment SGI199668[] = {{&S70708[69], 6}, {NULL}};
const Segment SGI199669[] = {{&S75260[44], 13}, {NULL}};
const Segment SGI199670[] = {{&S75260[44], 25}, {NULL}};
const Segment SGI199671[] = {{&S75260[30], 6}, {NULL}};
const Segment SGI199672[] = {{&S75260[118], 3}, {NULL}};
const Segment SGI199673[] = {{&S75260[83], 6}, {NULL}};
const Segment SGI199674[] = {{&S75262[44], 11}, {NULL}};
const Segment SGI199675[] = {{&S75262[44], 23}, {NULL}};
const Segment SGI199676[] = {{&S75262[30], 6}, {NULL}};
const Segment SGI199677[] = {{&S75262[81], 6}, {NULL}};
const Segment SGI199678[] = {{&S75262[108], 13}, {NULL}};
const Segment SGI199679[] = {{&S81396[34], 7}, {NULL}};
const Segment SGI199680[] = {{&S81396[27], 6}, {NULL}};
const Segment SGI199681[] = {{&S81396[99], 9}, {NULL}};
const Segment SGI199682[] = {{&S81396[72], 6}, {NULL}};
const Segment SGI199683[] = {{&S81397[34], 17}, {NULL}};
const Segment SGI199684[] = {{&S81397[27], 6}, {NULL}};
const Segment SGI199685[] = {{&S81397[104], 9}, {NULL}};
const Segment SGI199686[] = {{&S81397[77], 6}, {NULL}};
const Segment SGI199687[] = {{&S70061[34], 17}, {NULL}};
const Segment SGI199688[] = {{&S70061[27], 6}, {NULL}};
const Segment SGI199689[] = {{&S70061[104], 9}, {NULL}};
const Segment SGI199690[] = {{&S70061[77], 6}, {NULL}};
const Segment SGI199691[] = {{&S81402[44], 10}, {NULL}};
const Segment SGI199692[] = {{&S81402[44], 22}, {NULL}};
const Segment SGI199693[] = {{&S81402[30], 6}, {NULL}};
const Segment SGI199694[] = {{&S81402[31], 3}, {NULL}};
const Segment SGI199695[] = {{&S81402[99], 6}, {NULL}};
const Segment SGI199696[] = {{&S81402[68], 8}, {NULL}};
const Segment SGI199697[] = {{&S81403[44], 10}, {NULL}};
const Segment SGI199698[] = {{&S81403[44], 22}, {NULL}};
const Segment SGI199699[] = {{&S81403[30], 6}, {NULL}};
const Segment SGI199700[] = {{&S81403[31], 3}, {NULL}};
const Segment SGI199701[] = {{&S81403[95], 6}, {NULL}};
const Segment SGI199702[] = {{&S81403[68], 8}, {NULL}};
const Segment SGI199703[] = {{&S70752[44], 7}, {NULL}};
const Segment SGI199704[] = {{&S70752[44], 19}, {NULL}};
const Segment SGI199705[] = {{&S70752[30], 6}, {NULL}};
const Segment SGI199706[] = {{&S70752[112], 3}, {NULL}};
const Segment SGI199707[] = {{&S70752[77], 6}, {NULL}};
const Segment SGI199708[] = {{&S70757[44], 10}, {NULL}};
const Segment SGI199709[] = {{&S70757[44], 22}, {NULL}};
const Segment SGI199710[] = {{&S70757[30], 6}, {NULL}};
const Segment SGI199711[] = {{&S70757[115], 3}, {NULL}};
const Segment SGI199712[] = {{&S70757[80], 6}, {NULL}};
const Segment SGI199713[] = {{&S70758[38], 10}, {NULL}};
const Segment SGI199714[] = {{&S70758[31], 6}, {NULL}};
const Segment SGI199715[] = {{&S70761[52], 10}, {NULL}};
const Segment SGI199716[] = {{&S70761[52], 22}, {NULL}};
const Segment SGI199717[] = {{&S70761[38], 6}, {NULL}};
const Segment SGI199718[] = {{&S70761[88], 5}, {NULL}};
const Segment SGI199719[] = {{&S70761[11], 3}, {NULL}};
const Segment SGI199720[] = {{&S70762[52], 10}, {NULL}};
const Segment SGI199721[] = {{&S70762[52], 22}, {NULL}};
const Segment SGI199722[] = {{&S70762[38], 6}, {NULL}};
const Segment SGI199723[] = {{&S70762[88], 5}, {NULL}};
const Segment SGI199724[] = {{&S70762[11], 3}, {NULL}};
const Segment SGI199725[] = {{&S70763[52], 18}, {NULL}};
const Segment SGI199726[] = {{&S70763[52], 30}, {NULL}};
const Segment SGI199727[] = {{&S70763[38], 6}, {NULL}};
const Segment SGI199728[] = {{&S70763[96], 5}, {NULL}};
const Segment SGI199729[] = {{&S70763[11], 3}, {NULL}};
const Segment SGI199730[] = {{&S70764[52], 10}, {NULL}};
const Segment SGI199731[] = {{&S70764[52], 22}, {NULL}};
const Segment SGI199732[] = {{&S70764[38], 6}, {NULL}};
const Segment SGI199733[] = {{&S70764[88], 5}, {NULL}};
const Segment SGI199734[] = {{&S70764[11], 3}, {NULL}};
const Segment SGI199735[] = {{&S70766[34], 11}, {NULL}};
const Segment SGI199736[] = {{&S70766[27], 6}, {NULL}};
const Segment SGI199737[] = {{&S70766[98], 9}, {NULL}};
const Segment SGI199738[] = {{&S70766[71], 6}, {NULL}};
const Segment SGI199739[] = {{&S70768[44], 9}, {NULL}};
const Segment SGI199740[] = {{&S81428[34], 10}, {NULL}};
const Segment SGI199741[] = {{&S70768[44], 21}, {NULL}};
const Segment SGI199742[] = {{&S81428[27], 6}, {NULL}};
const Segment SGI199743[] = {{&S70768[30], 6}, {NULL}};
const Segment SGI199744[] = {{&S70768[114], 3}, {NULL}};
const Segment SGI199745[] = {{&S81428[97], 9}, {NULL}};
const Segment SGI199746[] = {{&S70768[79], 6}, {NULL}};
const Segment SGI199747[] = {{&S81428[70], 6}, {NULL}};
const Segment SGI199748[] = {{&S70769[38], 9}, {NULL}};
const Segment SGI199749[] = {{&S70769[31], 6}, {NULL}};
const Segment SGI199750[] = {{&S70770[44], 24}, {NULL}};
const Segment SGI199751[] = {{&S70770[44], 36}, {NULL}};
const Segment SGI199752[] = {{&S70770[30], 6}, {NULL}};
const Segment SGI199753[] = {{&S70770[129], 3}, {NULL}};
const Segment SGI199754[] = {{&S81431[34], 11}, {NULL}};
const Segment SGI199755[] = {{&S70770[94], 6}, {NULL}};
const Segment SGI199756[] = {{&S81431[27], 6}, {NULL}};
const Segment SGI199757[] = {{&S81431[111], 9}, {NULL}};
const Segment SGI199758[] = {{&S81431[84], 6}, {NULL}};
const Segment SGI199759[] = {{&S81457[44], 14}, {NULL}};
const Segment SGI199760[] = {{&S81457[44], 26}, {NULL}};
const Segment SGI199761[] = {{&S81457[30], 6}, {NULL}};
const Segment SGI199762[] = {{&S81457[31], 3}, {NULL}};
const Segment SGI199763[] = {{&S81457[84], 6}, {NULL}};
const Segment SGI199764[] = {{&S70804[34], 9}, {NULL}};
const Segment SGI199765[] = {{&S70804[27], 6}, {NULL}};
const Segment SGI199766[] = {{&S70804[96], 9}, {NULL}};
const Segment SGI199767[] = {{&S70804[69], 6}, {NULL}};
const Segment SGI199768[] = {{&S81466[44], 12}, {NULL}};
const Segment SGI199769[] = {{&S81466[44], 24}, {NULL}};
const Segment SGI199770[] = {{&S81466[30], 6}, {NULL}};
const Segment SGI199771[] = {{&S81466[31], 3}, {NULL}};
const Segment SGI199772[] = {{&S81466[82], 6}, {NULL}};
const Segment SGI199773[] = {{&S70143[34], 9}, {NULL}};
const Segment SGI199774[] = {{&S70143[27], 6}, {NULL}};
const Segment SGI199775[] = {{&S70143[96], 9}, {NULL}};
const Segment SGI199776[] = {{&S70143[69], 6}, {NULL}};
const Segment SGI199777[] = {{&S81479[34], 10}, {NULL}};
const Segment SGI199778[] = {{&S81479[27], 6}, {NULL}};
const Segment SGI199779[] = {{&S81479[97], 9}, {NULL}};
const Segment SGI199780[] = {{&S81479[70], 6}, {NULL}};
const Segment SGI199781[] = {{&S81485[44], 19}, {NULL}};
const Segment SGI199782[] = {{&S81485[30], 6}, {NULL}};
const Segment SGI199783[] = {{&S81485[31], 3}, {NULL}};
const Segment SGI199784[] = {{&S81485[91], 6}, {NULL}};
const Segment SGI199785[] = {{&S70828[44], 8}, {NULL}};
const Segment SGI199786[] = {{&S70828[44], 26}, {NULL}};
const Segment SGI199787[] = {{&S70828[30], 6}, {NULL}};
const Segment SGI199788[] = {{&S70828[119], 3}, {NULL}};
const Segment SGI199789[] = {{&S70828[84], 6}, {NULL}};
const Segment SGI199790[] = {{&S81486[44], 9}, {NULL}};
const Segment SGI199791[] = {{&S81486[30], 6}, {NULL}};
const Segment SGI199792[] = {{&S81486[126], 3}, {NULL}};
const Segment SGI199793[] = {{&S81486[91], 6}, {NULL}};
const Segment SGI199794[] = {{&S81486[146], 8}, {NULL}};
const Segment SGI199795[] = {{&S70829[38], 8}, {NULL}};
const Segment SGI199796[] = {{&S70829[31], 6}, {NULL}};
const Segment SGI199797[] = {{&S81487[31], 6}, {NULL}};
const Segment SGI199798[] = {{&S70830[44], 8}, {NULL}};
const Segment SGI199799[] = {{&S70830[44], 26}, {NULL}};
const Segment SGI199800[] = {{&S70830[30], 6}, {NULL}};
const Segment SGI199801[] = {{&S75345[45], 10}, {NULL}};
const Segment SGI199802[] = {{&S70830[119], 3}, {NULL}};
const Segment SGI199803[] = {{&S75345[30], 6}, {NULL}};
const Segment SGI199804[] = {{&S70830[84], 6}, {NULL}};
const Segment SGI199805[] = {{&S75345[115], 3}, {NULL}};
const Segment SGI199806[] = {{&S75345[81], 5}, {NULL}};
const Segment SGI199807[] = {{&S75345[139], 13}, {NULL}};
const Segment SGI199808[] = {{&S70834[34], 7}, {NULL}};
const Segment SGI199809[] = {{&S70834[27], 6}, {NULL}};
const Segment SGI199810[] = {{&S70834[100], 9}, {NULL}};
const Segment SGI199811[] = {{&S70834[73], 6}, {NULL}};
const Segment SGI199812[] = {{&S75352[34], 9}, {NULL}};
const Segment SGI199813[] = {{&S75352[27], 6}, {NULL}};
const Segment SGI199814[] = {{&S75352[96], 9}, {NULL}};
const Segment SGI199815[] = {{&S75352[69], 6}, {NULL}};
const Segment SGI199816[] = {{&S81516[34], 10}, {NULL}};
const Segment SGI199817[] = {{&S81516[27], 6}, {NULL}};
const Segment SGI199818[] = {{&S81516[97], 9}, {NULL}};
const Segment SGI199819[] = {{&S81516[70], 6}, {NULL}};
const Segment SGI199820[] = {{&S70865[34], 11}, {NULL}};
const Segment SGI199821[] = {{&S70865[27], 6}, {NULL}};
const Segment SGI199822[] = {{&S70865[98], 9}, {NULL}};
const Segment SGI199823[] = {{&S70865[71], 6}, {NULL}};
const Segment SGI199824[] = {{&S75383[34], 6}, {NULL}};
const Segment SGI199825[] = {{&S75383[27], 6}, {NULL}};
const Segment SGI199826[] = {{&S75383[93], 9}, {NULL}};
const Segment SGI199827[] = {{&S75383[66], 6}, {NULL}};
const Segment SGI199828[] = {{&S70210[44], 12}, {NULL}};
const Segment SGI199829[] = {{&S70210[44], 24}, {NULL}};
const Segment SGI199830[] = {{&S70210[30], 6}, {NULL}};
const Segment SGI199831[] = {{&S70210[138], 3}, {NULL}};
const Segment SGI199832[] = {{&S70210[103], 6}, {NULL}};
const Segment SGI199833[] = {{&S70210[70], 8}, {NULL}};
const Segment SGI199834[] = {{&S81536[34], 13}, {NULL}};
const Segment SGI199835[] = {{&S81536[27], 6}, {NULL}};
const Segment SGI199836[] = {{&S81536[100], 9}, {NULL}};
const Segment SGI199837[] = {{&S81536[73], 6}, {NULL}};
const Segment SGI199838[] = {{&S70220[34], 10}, {NULL}};
const Segment SGI199839[] = {{&S70220[27], 6}, {NULL}};
const Segment SGI199840[] = {{&S70220[97], 9}, {NULL}};
const Segment SGI199841[] = {{&S70220[70], 6}, {NULL}};
const Segment SGI199842[] = {{&S70886[44], 7}, {NULL}};
const Segment SGI199843[] = {{&S70886[44], 19}, {NULL}};
const Segment SGI199844[] = {{&S70886[30], 6}, {NULL}};
const Segment SGI199845[] = {{&S70886[112], 3}, {NULL}};
const Segment SGI199846[] = {{&S70886[77], 6}, {NULL}};
const Segment SGI199847[] = {{&S70887[38], 7}, {NULL}};
const Segment SGI199848[] = {{&S70887[31], 6}, {NULL}};
const Segment SGI199849[] = {{&S70888[34], 10}, {NULL}};
const Segment SGI199850[] = {{&S70888[27], 6}, {NULL}};
const Segment SGI199851[] = {{&S70888[97], 9}, {NULL}};
const Segment SGI199852[] = {{&S70888[70], 6}, {NULL}};
const Segment SGI199853[] = {{&S70889[34], 10}, {NULL}};
const Segment SGI199854[] = {{&S70889[27], 6}, {NULL}};
const Segment SGI199855[] = {{&S70889[97], 9}, {NULL}};
const Segment SGI199856[] = {{&S70889[70], 6}, {NULL}};
const Segment SGI199857[] = {{&S81550[34], 15}, {NULL}};
const Segment SGI199858[] = {{&S81550[27], 6}, {NULL}};
const Segment SGI199859[] = {{&S81550[102], 9}, {NULL}};
const Segment SGI199860[] = {{&S81550[75], 6}, {NULL}};
const Segment SGI199861[] = {{&S70890[34], 10}, {NULL}};
const Segment SGI199862[] = {{&S70890[27], 6}, {NULL}};
const Segment SGI199863[] = {{&S70890[97], 9}, {NULL}};
const Segment SGI199864[] = {{&S70890[70], 6}, {NULL}};
const Segment SGI199865[] = {{&S81551[34], 11}, {NULL}};
const Segment SGI199866[] = {{&S81551[27], 6}, {NULL}};
const Segment SGI199867[] = {{&S81551[98], 9}, {NULL}};
const Segment SGI199868[] = {{&S70891[34], 9}, {NULL}};
const Segment SGI199869[] = {{&S81551[71], 6}, {NULL}};
const Segment SGI199870[] = {{&S70891[27], 6}, {NULL}};
const Segment SGI199871[] = {{&S70891[96], 9}, {NULL}};
const Segment SGI199872[] = {{&S70891[69], 6}, {NULL}};
const Segment SGI199873[] = {{&S75413[44], 6}, {NULL}};
const Segment SGI199874[] = {{&S75413[44], 18}, {NULL}};
const Segment SGI199875[] = {{&S81552[34], 7}, {NULL}};
const Segment SGI199876[] = {{&S75413[30], 6}, {NULL}};
const Segment SGI199877[] = {{&S81552[27], 6}, {NULL}};
const Segment SGI199878[] = {{&S75413[127], 3}, {NULL}};
const Segment SGI199879[] = {{&S60915[34], 10}, {NULL}};
const Segment SGI199880[] = {{&S75413[92], 6}, {NULL}};
const Segment SGI199881[] = {{&S81552[99], 9}, {NULL}};
const Segment SGI199882[] = {{&S81552[72], 6}, {NULL}};
const Segment SGI199883[] = {{&S60915[27], 6}, {NULL}};
const Segment SGI199884[] = {{&S75413[64], 8}, {NULL}};
const Segment SGI199885[] = {{&S60915[96], 9}, {NULL}};
const Segment SGI199886[] = {{&S60915[69], 6}, {NULL}};
const Segment SGI199887[] = {{&S75415[44], 6}, {NULL}};
const Segment SGI199888[] = {{&S75415[44], 18}, {NULL}};
const Segment SGI199889[] = {{&S75415[30], 6}, {NULL}};
const Segment SGI199890[] = {{&S75415[127], 3}, {NULL}};
const Segment SGI199891[] = {{&S75415[92], 6}, {NULL}};
const Segment SGI199892[] = {{&S75415[64], 8}, {NULL}};
const Segment SGI199893[] = {{&S81562[34], 7}, {NULL}};
const Segment SGI199894[] = {{&S81562[27], 6}, {NULL}};
const Segment SGI199895[] = {{&S81562[99], 9}, {NULL}};
const Segment SGI199896[] = {{&S81562[72], 6}, {NULL}};
const Segment SGI199897[] = {{&S81562[140], 8}, {NULL}};
const Segment SGI199898[] = {{&S70906[34], 15}, {NULL}};
const Segment SGI199899[] = {{&S70906[27], 6}, {NULL}};
const Segment SGI199900[] = {{&S70906[102], 9}, {NULL}};
const Segment SGI199901[] = {{&S70906[75], 6}, {NULL}};
const Segment SGI199902[] = {{&S75426[34], 5}, {NULL}};
const Segment SGI199903[] = {{&S75426[27], 6}, {NULL}};
const Segment SGI199904[] = {{&S75426[92], 9}, {NULL}};
const Segment SGI199905[] = {{&S75426[65], 6}, {NULL}};
const Segment SGI199906[] = {{&S70251[43], 23}, {NULL}};
const Segment SGI199907[] = {{&S70251[30], 6}, {NULL}};
const Segment SGI199908[] = {{&S70251[31], 3}, {NULL}};
const Segment SGI199909[] = {{&S70251[80], 6}, {NULL}};
const Segment SGI199910[] = {{&S70252[43], 19}, {NULL}};
const Segment SGI199911[] = {{&S70252[30], 6}, {NULL}};
const Segment SGI199912[] = {{&S70252[31], 3}, {NULL}};
const Segment SGI199913[] = {{&S70252[76], 6}, {NULL}};
const Segment SGI199914[] = {{&S70253[44], 17}, {NULL}};
const Segment SGI199915[] = {{&S70253[44], 29}, {NULL}};
const Segment SGI199916[] = {{&S70253[30], 6}, {NULL}};
const Segment SGI199917[] = {{&S70253[122], 3}, {NULL}};
const Segment SGI199918[] = {{&S70253[87], 6}, {NULL}};
const Segment SGI199919[] = {{&S75431[44], 21}, {NULL}};
const Segment SGI199920[] = {{&S75431[44], 33}, {NULL}};
const Segment SGI199921[] = {{&S75431[30], 6}, {NULL}};
const Segment SGI199922[] = {{&S75431[126], 3}, {NULL}};
const Segment SGI199923[] = {{&S75431[91], 6}, {NULL}};
const Segment SGI199924[] = {{&S70254[38], 17}, {NULL}};
const Segment SGI199925[] = {{&S70254[31], 6}, {NULL}};
const Segment SGI199926[] = {{&S75432[44], 18}, {NULL}};
const Segment SGI199927[] = {{&S75432[44], 30}, {NULL}};
const Segment SGI199928[] = {{&S75432[30], 6}, {NULL}};
const Segment SGI199929[] = {{&S75432[123], 3}, {NULL}};
const Segment SGI199930[] = {{&S75432[88], 6}, {NULL}};
const Segment SGI199931[] = {{&S75433[44], 21}, {NULL}};
const Segment SGI199932[] = {{&S75433[44], 33}, {NULL}};
const Segment SGI199933[] = {{&S75433[30], 6}, {NULL}};
const Segment SGI199934[] = {{&S75433[126], 3}, {NULL}};
const Segment SGI199935[] = {{&S75433[91], 6}, {NULL}};
const Segment SGI199936[] = {{&S81577[34], 14}, {NULL}};
const Segment SGI199937[] = {{&S81577[27], 6}, {NULL}};
const Segment SGI199938[] = {{&S81577[106], 9}, {NULL}};
const Segment SGI199939[] = {{&S81577[79], 6}, {NULL}};
const Segment SGI199940[] = {{&S75434[44], 18}, {NULL}};
const Segment SGI199941[] = {{&S75434[44], 30}, {NULL}};
const Segment SGI199942[] = {{&S75434[30], 6}, {NULL}};
const Segment SGI199943[] = {{&S75434[123], 3}, {NULL}};
const Segment SGI199944[] = {{&S75434[88], 6}, {NULL}};
const Segment SGI199945[] = {{&S75437[34], 25}, {NULL}};
const Segment SGI199946[] = {{&S75437[27], 6}, {NULL}};
const Segment SGI199947[] = {{&S75437[112], 9}, {NULL}};
const Segment SGI199948[] = {{&S75437[85], 6}, {NULL}};
const Segment SGI199949[] = {{&S75438[34], 25}, {NULL}};
const Segment SGI199950[] = {{&S75438[27], 6}, {NULL}};
const Segment SGI199951[] = {{&S75438[112], 9}, {NULL}};
const Segment SGI199952[] = {{&S75438[85], 6}, {NULL}};
const Segment SGI199953[] = {{&S75439[34], 25}, {NULL}};
const Segment SGI199954[] = {{&S75439[27], 6}, {NULL}};
const Segment SGI199955[] = {{&S75439[112], 9}, {NULL}};
const Segment SGI199956[] = {{&S75439[85], 6}, {NULL}};
const Segment SGI199957[] = {{&S70933[38], 10}, {NULL}};
const Segment SGI199958[] = {{&S70933[31], 6}, {NULL}};
const Segment SGI199959[] = {{&S70934[44], 9}, {NULL}};
const Segment SGI199960[] = {{&S70934[44], 21}, {NULL}};
const Segment SGI199961[] = {{&S70934[30], 6}, {NULL}};
const Segment SGI199962[] = {{&S70934[114], 3}, {NULL}};
const Segment SGI199963[] = {{&S70934[79], 6}, {NULL}};
const Segment SGI199964[] = {{&S70938[44], 15}, {NULL}};
const Segment SGI199965[] = {{&S70938[44], 27}, {NULL}};
const Segment SGI199966[] = {{&S70938[30], 6}, {NULL}};
const Segment SGI199967[] = {{&S70938[120], 3}, {NULL}};
const Segment SGI199968[] = {{&S70938[85], 6}, {NULL}};
const Segment SGI199969[] = {{&S70939[44], 5}, {NULL}};
const Segment SGI199970[] = {{&S70939[44], 17}, {NULL}};
const Segment SGI199971[] = {{&S70939[30], 6}, {NULL}};
const Segment SGI199972[] = {{&S70939[110], 3}, {NULL}};
const Segment SGI199973[] = {{&S70939[75], 6}, {NULL}};
const Segment SGI199974[] = {{&S70940[44], 9}, {NULL}};
const Segment SGI199975[] = {{&S70940[44], 17}, {NULL}};
const Segment SGI199976[] = {{&S70940[30], 6}, {NULL}};
const Segment SGI199977[] = {{&S70940[110], 3}, {NULL}};
const Segment SGI199978[] = {{&S70940[75], 6}, {NULL}};
const Segment SGI199979[] = {{&S75451[34], 7}, {NULL}};
const Segment SGI199980[] = {{&S75451[27], 6}, {NULL}};
const Segment SGI199981[] = {{&S75451[94], 9}, {NULL}};
const Segment SGI199982[] = {{&S75451[67], 6}, {NULL}};
const Segment SGI199983[] = {{&S75456[44], 6}, {NULL}};
const Segment SGI199984[] = {{&S75456[44], 18}, {NULL}};
const Segment SGI199985[] = {{&S75456[30], 6}, {NULL}};
const Segment SGI199986[] = {{&S75456[76], 6}, {NULL}};
const Segment SGI199987[] = {{&S75456[103], 13}, {NULL}};
const Segment SGI199988[] = {{&S75498[44], 12}, {NULL}};
const Segment SGI199989[] = {{&S75498[44], 24}, {NULL}};
const Segment SGI199990[] = {{&S75498[30], 6}, {NULL}};
const Segment SGI199991[] = {{&S75498[117], 3}, {NULL}};
const Segment SGI199992[] = {{&S75498[82], 6}, {NULL}};
const Segment SGI199993[] = {{&S75499[38], 12}, {NULL}};
const Segment SGI199994[] = {{&S75499[31], 6}, {NULL}};
const Segment SGI199995[] = {{&S61027[44], 10}, {NULL}};
const Segment SGI199996[] = {{&S61027[44], 21}, {NULL}};
const Segment SGI199997[] = {{&S61027[30], 6}, {NULL}};
const Segment SGI199998[] = {{&S61027[132], 3}, {NULL}};
const Segment SGI199999[] = {{&S61027[97], 6}, {NULL}};
const Segment SGI200000[] = {{&S61027[67], 8}, {NULL}};
const Segment SGI200001[] = {{&S61028[44], 10}, {NULL}};
const Segment SGI200002[] = {{&S61028[44], 22}, {NULL}};
const Segment SGI200003[] = {{&S61028[30], 6}, {NULL}};
const Segment SGI200004[] = {{&S61028[115], 3}, {NULL}};
const Segment SGI200005[] = {{&S61028[80], 6}, {NULL}};
const Segment SGI200006[] = {{&S61028[140], 8}, {NULL}};
const Segment SGI200007[] = {{&S75503[44], 17}, {NULL}};
const Segment SGI200008[] = {{&S75503[44], 29}, {NULL}};
const Segment SGI200009[] = {{&S75503[30], 6}, {NULL}};
const Segment SGI200010[] = {{&S75503[122], 3}, {NULL}};
const Segment SGI200011[] = {{&S61029[44], 10}, {NULL}};
const Segment SGI200012[] = {{&S75503[87], 6}, {NULL}};
const Segment SGI200013[] = {{&S61029[44], 22}, {NULL}};
const Segment SGI200014[] = {{&S61029[30], 6}, {NULL}};
const Segment SGI200015[] = {{&S61029[115], 3}, {NULL}};
const Segment SGI200016[] = {{&S61029[80], 6}, {NULL}};
const Segment SGI200017[] = {{&S61029[140], 8}, {NULL}};
const Segment SGI200018[] = {{&S75504[44], 17}, {NULL}};
const Segment SGI200019[] = {{&S75504[44], 30}, {NULL}};
const Segment SGI200020[] = {{&S75504[30], 6}, {NULL}};
const Segment SGI200021[] = {{&S75504[123], 3}, {NULL}};
const Segment SGI200022[] = {{&S75504[88], 6}, {NULL}};
const Segment SGI200023[] = {{&S75522[44], 9}, {NULL}};
const Segment SGI200024[] = {{&S75522[44], 30}, {NULL}};
const Segment SGI200025[] = {{&S75522[30], 6}, {NULL}};
const Segment SGI200026[] = {{&S75522[123], 3}, {NULL}};
const Segment SGI200027[] = {{&S75522[88], 6}, {NULL}};
const Segment SGI200028[] = {{&S75524[44], 10}, {NULL}};
const Segment SGI200029[] = {{&S75524[44], 22}, {NULL}};
const Segment SGI200030[] = {{&S75524[30], 6}, {NULL}};
const Segment SGI200031[] = {{&S75524[146], 4}, {NULL}};
const Segment SGI200032[] = {{&S75524[172], 3}, {NULL}};
const Segment SGI200033[] = {{&S75524[115], 3}, {&S75524[221], 5}, {NULL}};
const Segment SGI200034[] = {{&S75524[80], 6}, {NULL}};
const Segment SGI200035[] = {{&S70383[44], 24}, {NULL}};
const Segment SGI200036[] = {{&S70383[44], 36}, {NULL}};
const Segment SGI200037[] = {{&S70383[30], 6}, {NULL}};
const Segment SGI200038[] = {{&S70383[147], 3}, {NULL}};
const Segment SGI200039[] = {{&S70383[112], 6}, {NULL}};
const Segment SGI200040[] = {{&S70383[82], 8}, {NULL}};
const Segment SGI200041[] = {{&S75533[34], 15}, {NULL}};
const Segment SGI200042[] = {{&S75533[27], 6}, {NULL}};
const Segment SGI200043[] = {{&S75533[102], 9}, {NULL}};
const Segment SGI200044[] = {{&S75533[75], 6}, {NULL}};
const Segment SGI200045[] = {{&S61068[44], 9}, {NULL}};
const Segment SGI200046[] = {{&S61068[44], 20}, {NULL}};
const Segment SGI200047[] = {{&S61068[30], 6}, {NULL}};
const Segment SGI200048[] = {{&S61068[113], 3}, {NULL}};
const Segment SGI200049[] = {{&S61068[78], 6}, {NULL}};
const Segment SGI200050[] = {{&S61068[141], 3}, {NULL}};
const Segment SGI200051[] = {{&S75557[44], 9}, {NULL}};
const Segment SGI200052[] = {{&S75557[44], 21}, {NULL}};
const Segment SGI200053[] = {{&S75557[30], 6}, {NULL}};
const Segment SGI200054[] = {{&S75557[114], 3}, {NULL}};
const Segment SGI200055[] = {{&S75557[79], 6}, {NULL}};
const Segment SGI200056[] = {{&S61093[34], 11}, {NULL}};
const Segment SGI200057[] = {{&S61093[27], 6}, {NULL}};
const Segment SGI200058[] = {{&S61093[97], 9}, {NULL}};
const Segment SGI200059[] = {{&S61093[70], 6}, {NULL}};
const Segment SGI200060[] = {{&S70955[34], 10}, {NULL}};
const Segment SGI200061[] = {{&S70955[27], 6}, {NULL}};
const Segment SGI200062[] = {{&S70955[97], 9}, {NULL}};
const Segment SGI200063[] = {{&S70955[70], 6}, {NULL}};
const Segment SGI200064[] = {{&S70436[44], 13}, {NULL}};
const Segment SGI200065[] = {{&S70436[44], 25}, {NULL}};
const Segment SGI200066[] = {{&S70436[30], 6}, {NULL}};
const Segment SGI200067[] = {{&S70436[136], 3}, {NULL}};
const Segment SGI200068[] = {{&S70956[34], 10}, {NULL}};
const Segment SGI200069[] = {{&S70956[27], 6}, {NULL}};
const Segment SGI200070[] = {{&S70436[101], 6}, {NULL}};
const Segment SGI200071[] = {{&S70436[71], 8}, {NULL}};
const Segment SGI200072[] = {{&S70956[97], 9}, {NULL}};
const Segment SGI200073[] = {{&S70956[70], 6}, {NULL}};
const Segment SGI200074[] = {{&S70957[34], 10}, {NULL}};
const Segment SGI200075[] = {{&S70957[27], 6}, {NULL}};
const Segment SGI200076[] = {{&S70957[97], 9}, {NULL}};
const Segment SGI200077[] = {{&S70957[70], 6}, {NULL}};
const Segment SGI200078[] = {{&S70958[34], 11}, {NULL}};
const Segment SGI200079[] = {{&S70958[27], 6}, {NULL}};
const Segment SGI200080[] = {{&S70958[98], 9}, {NULL}};
const Segment SGI200081[] = {{&S70958[71], 6}, {NULL}};
const Segment SGI200082[] = {{&S70959[34], 12}, {NULL}};
const Segment SGI200083[] = {{&S70959[27], 6}, {NULL}};
const Segment SGI200084[] = {{&S70959[99], 9}, {NULL}};
const Segment SGI200085[] = {{&S70959[72], 6}, {NULL}};
const Segment SGI200086[] = {{&S81611[44], 14}, {NULL}};
const Segment SGI200087[] = {{&S81611[44], 26}, {NULL}};
const Segment SGI200088[] = {{&S81611[30], 6}, {NULL}};
const Segment SGI200089[] = {{&S81611[31], 3}, {NULL}};
const Segment SGI200090[] = {{&S81611[84], 6}, {NULL}};
const Segment SGI200091[] = {{&S81613[34], 13}, {NULL}};
const Segment SGI200092[] = {{&S81613[27], 6}, {NULL}};
const Segment SGI200093[] = {{&S81613[100], 9}, {NULL}};
const Segment SGI200094[] = {{&S81613[73], 6}, {NULL}};
const Segment SGI200095[] = {{&S75601[44], 10}, {NULL}};
const Segment SGI200096[] = {{&S75601[44], 22}, {NULL}};
const Segment SGI200097[] = {{&S75601[30], 6}, {NULL}};
const Segment SGI200098[] = {{&S75601[113], 3}, {NULL}};
const Segment SGI200099[] = {{&S75601[80], 5}, {NULL}};
const Segment SGI200100[] = {{&S75601[117], 14}, {NULL}};
const Segment SGI200101[] = {{&S81616[44], 11}, {NULL}};
const Segment SGI200102[] = {{&S81616[44], 23}, {NULL}};
const Segment SGI200103[] = {{&S81616[30], 6}, {NULL}};
const Segment SGI200104[] = {{&S81616[31], 3}, {NULL}};
const Segment SGI200105[] = {{&S81616[81], 6}, {NULL}};
const Segment SGI200106[] = {{&S75603[44], 10}, {NULL}};
const Segment SGI200107[] = {{&S75603[44], 22}, {NULL}};
const Segment SGI200108[] = {{&S75603[30], 6}, {NULL}};
const Segment SGI200109[] = {{&S75603[113], 3}, {NULL}};
const Segment SGI200110[] = {{&S75603[80], 5}, {NULL}};
const Segment SGI200111[] = {{&S75603[117], 14}, {NULL}};
const Segment SGI200112[] = {{&S81618[45], 7}, {NULL}};
const Segment SGI200113[] = {{&S81618[30], 6}, {NULL}};
const Segment SGI200114[] = {{&S81618[31], 3}, {NULL}};
const Segment SGI200115[] = {{&S81618[82], 5}, {NULL}};
const Segment SGI200116[] = {{&S81618[140], 13}, {NULL}};
const Segment SGI200117[] = {{&S81619[34], 14}, {NULL}};
const Segment SGI200118[] = {{&S81619[27], 6}, {NULL}};
const Segment SGI200119[] = {{&S75606[44], 11}, {NULL}};
const Segment SGI200120[] = {{&S75606[44], 23}, {NULL}};
const Segment SGI200121[] = {{&S75606[30], 6}, {NULL}};
const Segment SGI200122[] = {{&S81619[101], 9}, {NULL}};
const Segment SGI200123[] = {{&S81619[74], 6}, {NULL}};
const Segment SGI200124[] = {{&S75606[116], 3}, {NULL}};
const Segment SGI200125[] = {{&S75606[81], 6}, {NULL}};
const Segment SGI200126[] = {{&S75607[38], 11}, {NULL}};
const Segment SGI200127[] = {{&S75607[31], 6}, {NULL}};
const Segment SGI200128[] = {{&S70981[44], 5}, {NULL}};
const Segment SGI200129[] = {{&S70981[44], 17}, {NULL}};
const Segment SGI200130[] = {{&S70981[30], 6}, {NULL}};
const Segment SGI200131[] = {{&S70981[110], 3}, {NULL}};
const Segment SGI200132[] = {{&S70981[75], 6}, {NULL}};
const Segment SGI200133[] = {{&S70982[44], 11}, {NULL}};
const Segment SGI200134[] = {{&S70982[44], 23}, {NULL}};
const Segment SGI200135[] = {{&S70982[30], 6}, {NULL}};
const Segment SGI200136[] = {{&S70982[116], 3}, {NULL}};
const Segment SGI200137[] = {{&S70982[81], 6}, {NULL}};
const Segment SGI200138[] = {{&S75610[34], 25}, {NULL}};
const Segment SGI200139[] = {{&S75610[27], 6}, {NULL}};
const Segment SGI200140[] = {{&S75610[112], 9}, {NULL}};
const Segment SGI200141[] = {{&S75610[85], 6}, {NULL}};
const Segment SGI200142[] = {{&S75611[34], 25}, {NULL}};
const Segment SGI200143[] = {{&S75611[27], 6}, {NULL}};
const Segment SGI200144[] = {{&S75611[112], 9}, {NULL}};
const Segment SGI200145[] = {{&S75611[85], 6}, {NULL}};
const Segment SGI200146[] = {{&S81629[45], 5}, {NULL}};
const Segment SGI200147[] = {{&S81629[30], 6}, {NULL}};
const Segment SGI200148[] = {{&S81629[31], 3}, {NULL}};
const Segment SGI200149[] = {{&S81629[76], 5}, {NULL}};
const Segment SGI200150[] = {{&S81629[134], 13}, {NULL}};
const Segment SGI200151[] = {{&S81630[45], 10}, {NULL}};
const Segment SGI200152[] = {{&S81630[30], 6}, {NULL}};
const Segment SGI200153[] = {{&S81630[31], 3}, {NULL}};
const Segment SGI200154[] = {{&S81630[81], 5}, {NULL}};
const Segment SGI200155[] = {{&S81630[139], 13}, {NULL}};
const Segment SGI200156[] = {{&S81631[44], 10}, {NULL}};
const Segment SGI200157[] = {{&S81631[44], 22}, {NULL}};
const Segment SGI200158[] = {{&S81631[30], 6}, {NULL}};
const Segment SGI200159[] = {{&S81631[115], 3}, {NULL}};
const Segment SGI200160[] = {{&S81631[80], 6}, {NULL}};
const Segment SGI200161[] = {{&S81631[135], 8}, {NULL}};
const Segment SGI200162[] = {{&S81631[144], 13}, {NULL}};
const Segment SGI200163[] = {{&S81633[34], 10}, {NULL}};
const Segment SGI200164[] = {{&S81633[27], 6}, {NULL}};
const Segment SGI200165[] = {{&S81633[97], 9}, {NULL}};
const Segment SGI200166[] = {{&S81633[70], 6}, {NULL}};
const Segment SGI200167[] = {{&S81634[44], 17}, {NULL}};
const Segment SGI200168[] = {{&S81634[44], 37}, {NULL}};
const Segment SGI200169[] = {{&S81634[30], 6}, {NULL}};
const Segment SGI200170[] = {{&S70998[44], 13}, {NULL}};
const Segment SGI200171[] = {{&S81634[31], 3}, {NULL}};
const Segment SGI200172[] = {{&S70998[44], 25}, {NULL}};
const Segment SGI200173[] = {{&S70998[30], 6}, {NULL}};
const Segment SGI200174[] = {{&S81634[95], 6}, {NULL}};
const Segment SGI200175[] = {{&S70998[118], 3}, {NULL}};
const Segment SGI200176[] = {{&S70998[83], 6}, {NULL}};
const Segment SGI200177[] = {{&S61154[34], 10}, {NULL}};
const Segment SGI200178[] = {{&S61154[27], 6}, {NULL}};
const Segment SGI200179[] = {{&S61154[96], 9}, {NULL}};
const Segment SGI200180[] = {{&S61154[69], 6}, {NULL}};
const Segment SGI200181[] = {{&S71000[44], 12}, {NULL}};
const Segment SGI200182[] = {{&S81636[44], 12}, {NULL}};
const Segment SGI200183[] = {{&S71000[44], 29}, {NULL}};
const Segment SGI200184[] = {{&S81636[44], 24}, {NULL}};
const Segment SGI200185[] = {{&S71000[30], 6}, {NULL}};
const Segment SGI200186[] = {{&S81636[30], 6}, {NULL}};
const Segment SGI200187[] = {{&S81636[31], 3}, {NULL}};
const Segment SGI200188[] = {{&S71000[122], 3}, {NULL}};
const Segment SGI200189[] = {{&S81636[82], 6}, {NULL}};
const Segment SGI200190[] = {{&S71000[87], 6}, {NULL}};
const Segment SGI200191[] = {{&S61155[34], 18}, {NULL}};
const Segment SGI200192[] = {{&S61155[27], 6}, {NULL}};
const Segment SGI200193[] = {{&S61155[105], 9}, {NULL}};
const Segment SGI200194[] = {{&S61155[78], 6}, {NULL}};
const Segment SGI200195[] = {{&S71001[44], 8}, {NULL}};
const Segment SGI200196[] = {{&S81637[34], 7}, {NULL}};
const Segment SGI200197[] = {{&S71001[44], 25}, {NULL}};
const Segment SGI200198[] = {{&S81637[27], 6}, {NULL}};
const Segment SGI200199[] = {{&S71001[30], 6}, {NULL}};
const Segment SGI200200[] = {{&S71001[118], 3}, {NULL}};
const Segment SGI200201[] = {{&S81637[94], 9}, {NULL}};
const Segment SGI200202[] = {{&S81637[67], 6}, {NULL}};
const Segment SGI200203[] = {{&S71001[83], 6}, {NULL}};
const Segment SGI200204[] = {{&S71002[38], 8}, {NULL}};
const Segment SGI200205[] = {{&S71002[31], 6}, {NULL}};
const Segment SGI200206[] = {{&S81638[44], 12}, {NULL}};
const Segment SGI200207[] = {{&S81638[44], 33}, {NULL}};
const Segment SGI200208[] = {{&S81638[30], 6}, {NULL}};
const Segment SGI200209[] = {{&S81638[31], 3}, {NULL}};
const Segment SGI200210[] = {{&S81638[91], 6}, {NULL}};
const Segment SGI200211[] = {{&S71003[44], 12}, {NULL}};
const Segment SGI200212[] = {{&S71003[44], 29}, {NULL}};
const Segment SGI200213[] = {{&S71003[30], 6}, {NULL}};
const Segment SGI200214[] = {{&S71003[122], 3}, {NULL}};
const Segment SGI200215[] = {{&S71003[87], 6}, {NULL}};
const Segment SGI200216[] = {{&S71004[44], 8}, {NULL}};
const Segment SGI200217[] = {{&S71004[44], 25}, {NULL}};
const Segment SGI200218[] = {{&S71004[30], 6}, {NULL}};
const Segment SGI200219[] = {{&S71004[118], 3}, {NULL}};
const Segment SGI200220[] = {{&S70497[44], 8}, {NULL}};
const Segment SGI200221[] = {{&S70497[44], 20}, {NULL}};
const Segment SGI200222[] = {{&S71004[83], 6}, {NULL}};
const Segment SGI200223[] = {{&S70497[30], 6}, {NULL}};
const Segment SGI200224[] = {{&S70497[113], 3}, {NULL}};
const Segment SGI200225[] = {{&S70497[78], 6}, {NULL}};
const Segment SGI200226[] = {{&S71005[44], 10}, {NULL}};
const Segment SGI200227[] = {{&S71005[44], 22}, {NULL}};
const Segment SGI200228[] = {{&S71005[30], 6}, {NULL}};
const Segment SGI200229[] = {{&S70498[38], 8}, {NULL}};
const Segment SGI200230[] = {{&S70498[31], 6}, {NULL}};
const Segment SGI200231[] = {{&S71005[113], 3}, {NULL}};
const Segment SGI200232[] = {{&S71005[80], 5}, {NULL}};
const Segment SGI200233[] = {{&S71005[117], 14}, {NULL}};
const Segment SGI200234[] = {{&S71007[44], 10}, {NULL}};
const Segment SGI200235[] = {{&S71007[44], 22}, {NULL}};
const Segment SGI200236[] = {{&S71007[30], 6}, {NULL}};
const Segment SGI200237[] = {{&S71007[113], 3}, {NULL}};
const Segment SGI200238[] = {{&S71007[80], 5}, {NULL}};
const Segment SGI200239[] = {{&S71007[117], 14}, {NULL}};
const Segment SGI200240[] = {{&S71008[44], 10}, {NULL}};
const Segment SGI200241[] = {{&S71008[44], 22}, {NULL}};
const Segment SGI200242[] = {{&S71008[30], 6}, {NULL}};
const Segment SGI200243[] = {{&S71008[113], 3}, {NULL}};
const Segment SGI200244[] = {{&S71008[80], 5}, {NULL}};
const Segment SGI200245[] = {{&S71008[117], 14}, {NULL}};
const Segment SGI200246[] = {{&S71010[34], 9}, {NULL}};
const Segment SGI200247[] = {{&S71010[27], 6}, {NULL}};
const Segment SGI200248[] = {{&S71010[96], 9}, {NULL}};
const Segment SGI200249[] = {{&S71010[69], 6}, {NULL}};
const Segment SGI200250[] = {{&S75644[52], 18}, {NULL}};
const Segment SGI200251[] = {{&S75644[52], 30}, {NULL}};
const Segment SGI200252[] = {{&S75644[38], 6}, {NULL}};
const Segment SGI200253[] = {{&S75644[96], 5}, {NULL}};
const Segment SGI200254[] = {{&S75644[0], 14}, {NULL}};
const Segment SGI200255[] = {{&S71022[34], 7}, {NULL}};
const Segment SGI200256[] = {{&S71022[27], 6}, {NULL}};
const Segment SGI200257[] = {{&S71022[102], 9}, {NULL}};
const Segment SGI200258[] = {{&S71022[75], 6}, {NULL}};
const Segment SGI200259[] = {{&S61194[34], 10}, {NULL}};
const Segment SGI200260[] = {{&S61194[27], 6}, {NULL}};
const Segment SGI200261[] = {{&S81676[44], 7}, {NULL}};
const Segment SGI200262[] = {{&S70529[44], 25}, {NULL}};
const Segment SGI200263[] = {{&S81676[44], 27}, {NULL}};
const Segment SGI200264[] = {{&S70529[44], 37}, {NULL}};
const Segment SGI200265[] = {{&S81676[30], 6}, {NULL}};
const Segment SGI200266[] = {{&S70529[30], 6}, {NULL}};
const Segment SGI200267[] = {{&S61194[96], 9}, {NULL}};
const Segment SGI200268[] = {{&S61194[69], 6}, {NULL}};
const Segment SGI200269[] = {{&S81676[31], 3}, {NULL}};
const Segment SGI200270[] = {{&S70529[130], 3}, {NULL}};
const Segment SGI200271[] = {{&S81676[100], 6}, {NULL}};
const Segment SGI200272[] = {{&S70529[95], 6}, {NULL}};
const Segment SGI200273[] = {{&S81676[73], 8}, {NULL}};
const Segment SGI200274[] = {{&S81677[44], 7}, {NULL}};
const Segment SGI200275[] = {{&S81677[44], 28}, {NULL}};
const Segment SGI200276[] = {{&S81677[30], 6}, {NULL}};
const Segment SGI200277[] = {{&S81677[31], 3}, {NULL}};
const Segment SGI200278[] = {{&S81677[101], 6}, {NULL}};
const Segment SGI200279[] = {{&S81677[74], 8}, {NULL}};
const Segment SGI200280[] = {{&S71042[44], 10}, {NULL}};
const Segment SGI200281[] = {{&S71042[44], 22}, {NULL}};
const Segment SGI200282[] = {{&S71042[30], 6}, {NULL}};
const Segment SGI200283[] = {{&S71042[131], 3}, {NULL}};
const Segment SGI200284[] = {{&S71042[96], 6}, {NULL}};
const Segment SGI200285[] = {{&S81685[44], 7}, {NULL}};
const Segment SGI200286[] = {{&S81685[44], 15}, {NULL}};
const Segment SGI200287[] = {{&S71042[68], 8}, {NULL}};
const Segment SGI200288[] = {{&S81685[30], 6}, {NULL}};
const Segment SGI200289[] = {{&S81685[31], 3}, {NULL}};
const Segment SGI200290[] = {{&S81685[73], 6}, {NULL}};
const Segment SGI200291[] = {{&S81687[34], 10}, {NULL}};
const Segment SGI200292[] = {{&S81687[27], 6}, {NULL}};
const Segment SGI200293[] = {{&S81687[97], 9}, {NULL}};
const Segment SGI200294[] = {{&S81687[70], 6}, {NULL}};
const Segment SGI200295[] = {{&S71050[34], 16}, {NULL}};
const Segment SGI200296[] = {{&S71050[27], 6}, {NULL}};
const Segment SGI200297[] = {{&S71050[110], 9}, {NULL}};
const Segment SGI200298[] = {{&S71050[83], 6}, {NULL}};
const Segment SGI200299[] = {{&S81705[44], 17}, {NULL}};
const Segment SGI200300[] = {{&S81705[44], 29}, {NULL}};
const Segment SGI200301[] = {{&S81705[30], 6}, {NULL}};
const Segment SGI200302[] = {{&S81705[31], 3}, {NULL}};
const Segment SGI200303[] = {{&S81705[87], 6}, {NULL}};
const Segment SGI200304[] = {{&S81706[44], 17}, {NULL}};
const Segment SGI200305[] = {{&S81706[44], 29}, {NULL}};
const Segment SGI200306[] = {{&S81706[30], 6}, {NULL}};
const Segment SGI200307[] = {{&S81706[31], 3}, {NULL}};
const Segment SGI200308[] = {{&S81706[87], 6}, {NULL}};
const Segment SGI200309[] = {{&S75702[44], 15}, {NULL}};
const Segment SGI200310[] = {{&S75702[44], 27}, {NULL}};
const Segment SGI200311[] = {{&S75702[30], 6}, {NULL}};
const Segment SGI200312[] = {{&S81707[34], 10}, {NULL}};
const Segment SGI200313[] = {{&S81707[27], 6}, {NULL}};
const Segment SGI200314[] = {{&S75702[120], 3}, {NULL}};
const Segment SGI200315[] = {{&S75702[85], 6}, {NULL}};
const Segment SGI200316[] = {{&S81707[97], 9}, {NULL}};
const Segment SGI200317[] = {{&S81707[70], 6}, {NULL}};
const Segment SGI200318[] = {{&S81707[132], 8}, {NULL}};
const Segment SGI200319[] = {{&S75703[44], 15}, {NULL}};
const Segment SGI200320[] = {{&S75703[44], 27}, {NULL}};
const Segment SGI200321[] = {{&S75703[30], 6}, {NULL}};
const Segment SGI200322[] = {{&S75703[88], 4}, {NULL}};
const Segment SGI200323[] = {{&S75703[88], 3}, {NULL}};
const Segment SGI200324[] = {{&S71073[34], 10}, {NULL}};
const Segment SGI200325[] = {{&S71073[27], 6}, {NULL}};
const Segment SGI200326[] = {{&S71073[97], 9}, {NULL}};
const Segment SGI200327[] = {{&S71073[70], 6}, {NULL}};
const Segment SGI200328[] = {{&S81710[34], 10}, {NULL}};
const Segment SGI200329[] = {{&S81710[27], 6}, {NULL}};
const Segment SGI200330[] = {{&S81710[97], 9}, {NULL}};
const Segment SGI200331[] = {{&S81710[70], 6}, {NULL}};
const Segment SGI200332[] = {{&S71074[34], 10}, {NULL}};
const Segment SGI200333[] = {{&S81710[131], 8}, {NULL}};
const Segment SGI200334[] = {{&S71074[27], 6}, {NULL}};
const Segment SGI200335[] = {{&S71074[97], 9}, {NULL}};
const Segment SGI200336[] = {{&S71074[70], 6}, {NULL}};
const Segment SGI200337[] = {{&S81711[44], 8}, {NULL}};
const Segment SGI200338[] = {{&S81711[44], 20}, {NULL}};
const Segment SGI200339[] = {{&S81711[30], 6}, {NULL}};
const Segment SGI200340[] = {{&S81711[31], 3}, {NULL}};
const Segment SGI200341[] = {{&S81711[78], 6}, {NULL}};
const Segment SGI200342[] = {{&S75709[44], 10}, {NULL}};
const Segment SGI200343[] = {{&S75709[44], 27}, {NULL}};
const Segment SGI200344[] = {{&S75709[30], 6}, {NULL}};
const Segment SGI200345[] = {{&S75709[120], 3}, {NULL}};
const Segment SGI200346[] = {{&S75709[85], 6}, {NULL}};
const Segment SGI200347[] = {{&S71084[34], 7}, {NULL}};
const Segment SGI200348[] = {{&S71084[27], 6}, {NULL}};
const Segment SGI200349[] = {{&S71084[97], 9}, {NULL}};
const Segment SGI200350[] = {{&S71084[70], 6}, {NULL}};
const Segment SGI200351[] = {{&S71084[138], 8}, {NULL}};
const Segment SGI200352[] = {{&S75717[44], 8}, {NULL}};
const Segment SGI200353[] = {{&S75717[44], 20}, {NULL}};
const Segment SGI200354[] = {{&S75717[30], 6}, {NULL}};
const Segment SGI200355[] = {{&S75717[113], 3}, {NULL}};
const Segment SGI200356[] = {{&S75717[78], 6}, {NULL}};
const Segment SGI200357[] = {{&S75723[34], 13}, {NULL}};
const Segment SGI200358[] = {{&S75723[27], 6}, {NULL}};
const Segment SGI200359[] = {{&S75723[100], 9}, {NULL}};
const Segment SGI200360[] = {{&S75723[73], 6}, {NULL}};
const Segment SGI200361[] = {{&S81744[34], 7}, {NULL}};
const Segment SGI200362[] = {{&S81744[27], 6}, {NULL}};
const Segment SGI200363[] = {{&S81744[94], 9}, {NULL}};
const Segment SGI200364[] = {{&S81744[67], 6}, {NULL}};
const Segment SGI200365[] = {{&S81745[34], 8}, {NULL}};
const Segment SGI200366[] = {{&S81745[27], 6}, {NULL}};
const Segment SGI200367[] = {{&S81745[95], 9}, {NULL}};
const Segment SGI200368[] = {{&S81745[68], 6}, {NULL}};
const Segment SGI200369[] = {{&S81753[44], 7}, {NULL}};
const Segment SGI200370[] = {{&S81753[44], 25}, {NULL}};
const Segment SGI200371[] = {{&S81753[30], 6}, {NULL}};
const Segment SGI200372[] = {{&S81753[31], 3}, {NULL}};
const Segment SGI200373[] = {{&S81753[83], 6}, {NULL}};
const Segment SGI200374[] = {{&S71117[34], 10}, {NULL}};
const Segment SGI200375[] = {{&S71117[27], 6}, {NULL}};
const Segment SGI200376[] = {{&S71117[97], 9}, {NULL}};
const Segment SGI200377[] = {{&S71117[70], 6}, {NULL}};
const Segment SGI200378[] = {{&S75752[44], 9}, {NULL}};
const Segment SGI200379[] = {{&S75752[44], 21}, {NULL}};
const Segment SGI200380[] = {{&S75752[30], 6}, {NULL}};
const Segment SGI200381[] = {{&S75752[114], 3}, {NULL}};
const Segment SGI200382[] = {{&S75752[79], 6}, {NULL}};
const Segment SGI200383[] = {{&S75753[38], 9}, {NULL}};
const Segment SGI200384[] = {{&S75753[31], 6}, {NULL}};
const Segment SGI200385[] = {{&S71122[34], 9}, {NULL}};
const Segment SGI200386[] = {{&S71122[27], 6}, {NULL}};
const Segment SGI200387[] = {{&S71122[96], 9}, {NULL}};
const Segment SGI200388[] = {{&S71122[69], 6}, {NULL}};
const Segment SGI200389[] = {{&S81759[34], 15}, {NULL}};
const Segment SGI200390[] = {{&S81759[27], 6}, {NULL}};
const Segment SGI200391[] = {{&S81759[102], 9}, {NULL}};
const Segment SGI200392[] = {{&S81759[75], 6}, {NULL}};
const Segment SGI200393[] = {{&S75760[34], 25}, {NULL}};
const Segment SGI200394[] = {{&S75760[27], 6}, {NULL}};
const Segment SGI200395[] = {{&S75760[112], 9}, {NULL}};
const Segment SGI200396[] = {{&S75760[85], 6}, {NULL}};
const Segment SGI200397[] = {{&S71130[34], 7}, {NULL}};
const Segment SGI200398[] = {{&S71130[27], 6}, {NULL}};
const Segment SGI200399[] = {{&S71130[99], 9}, {NULL}};
const Segment SGI200400[] = {{&S75762[34], 25}, {NULL}};
const Segment SGI200401[] = {{&S71130[72], 6}, {NULL}};
const Segment SGI200402[] = {{&S75762[27], 6}, {NULL}};
const Segment SGI200403[] = {{&S75762[112], 9}, {NULL}};
const Segment SGI200404[] = {{&S75762[85], 6}, {NULL}};
const Segment SGI200405[] = {{&S71131[34], 7}, {NULL}};
const Segment SGI200406[] = {{&S71131[27], 6}, {NULL}};
const Segment SGI200407[] = {{&S71131[99], 9}, {NULL}};
const Segment SGI200408[] = {{&S71131[72], 6}, {NULL}};
const Segment SGI200409[] = {{&S75763[34], 32}, {NULL}};
const Segment SGI200410[] = {{&S75763[27], 6}, {NULL}};
const Segment SGI200411[] = {{&S75763[119], 9}, {NULL}};
const Segment SGI200412[] = {{&S75763[92], 6}, {NULL}};
const Segment SGI200413[] = {{&S81773[34], 25}, {NULL}};
const Segment SGI200414[] = {{&S81773[27], 6}, {NULL}};
const Segment SGI200415[] = {{&S81773[112], 9}, {NULL}};
const Segment SGI200416[] = {{&S81773[85], 6}, {NULL}};
const Segment SGI200417[] = {{&S81774[34], 25}, {NULL}};
const Segment SGI200418[] = {{&S81774[27], 6}, {NULL}};
const Segment SGI200419[] = {{&S71137[44], 10}, {NULL}};
const Segment SGI200420[] = {{&S71137[44], 22}, {NULL}};
const Segment SGI200421[] = {{&S81774[112], 9}, {NULL}};
const Segment SGI200422[] = {{&S71137[30], 6}, {NULL}};
const Segment SGI200423[] = {{&S81774[85], 6}, {NULL}};
const Segment SGI200424[] = {{&S71137[80], 6}, {NULL}};
const Segment SGI200425[] = {{&S71137[107], 13}, {NULL}};
const Segment SGI200426[] = {{&S81775[34], 25}, {NULL}};
const Segment SGI200427[] = {{&S81775[27], 6}, {NULL}};
const Segment SGI200428[] = {{&S81775[112], 9}, {NULL}};
const Segment SGI200429[] = {{&S81775[85], 6}, {NULL}};
const Segment SGI200430[] = {{&S71151[34], 10}, {NULL}};
const Segment SGI200431[] = {{&S71151[27], 6}, {NULL}};
const Segment SGI200432[] = {{&S71151[97], 9}, {NULL}};
const Segment SGI200433[] = {{&S71151[70], 6}, {NULL}};
const Segment SGI200434[] = {{&S75792[44], 19}, {NULL}};
const Segment SGI200435[] = {{&S75792[44], 31}, {NULL}};
const Segment SGI200436[] = {{&S75792[30], 6}, {NULL}};
const Segment SGI200437[] = {{&S75792[124], 3}, {NULL}};
const Segment SGI200438[] = {{&S75792[89], 6}, {NULL}};
const Segment SGI200439[] = {{&S75793[44], 19}, {NULL}};
const Segment SGI200440[] = {{&S75793[44], 31}, {NULL}};
const Segment SGI200441[] = {{&S75793[30], 6}, {NULL}};
const Segment SGI200442[] = {{&S75793[124], 3}, {NULL}};
const Segment SGI200443[] = {{&S75793[89], 6}, {NULL}};
const Segment SGI200444[] = {{&S71160[34], 11}, {NULL}};
const Segment SGI200445[] = {{&S71160[27], 6}, {NULL}};
const Segment SGI200446[] = {{&S71160[98], 9}, {NULL}};
const Segment SGI200447[] = {{&S71160[71], 6}, {NULL}};
const Segment SGI200448[] = {{&S71160[139], 8}, {NULL}};
const Segment SGI200449[] = {{&S75798[44], 6}, {NULL}};
const Segment SGI200450[] = {{&S75798[44], 18}, {NULL}};
const Segment SGI200451[] = {{&S75798[30], 6}, {NULL}};
const Segment SGI200452[] = {{&S75798[111], 3}, {NULL}};
const Segment SGI200453[] = {{&S75798[76], 6}, {NULL}};
const Segment SGI200454[] = {{&S81807[34], 11}, {NULL}};
const Segment SGI200455[] = {{&S81807[27], 6}, {NULL}};
const Segment SGI200456[] = {{&S81807[98], 9}, {NULL}};
const Segment SGI200457[] = {{&S81807[71], 6}, {NULL}};
const Segment SGI200458[] = {{&S81808[34], 11}, {NULL}};
const Segment SGI200459[] = {{&S81808[27], 6}, {NULL}};
const Segment SGI200460[] = {{&S81808[98], 9}, {NULL}};
const Segment SGI200461[] = {{&S81808[71], 6}, {NULL}};
const Segment SGI200462[] = {{&S81810[45], 7}, {NULL}};
const Segment SGI200463[] = {{&S81810[30], 6}, {NULL}};
const Segment SGI200464[] = {{&S81810[31], 3}, {NULL}};
const Segment SGI200465[] = {{&S81810[83], 5}, {NULL}};
const Segment SGI200466[] = {{&S81810[141], 13}, {NULL}};
const Segment SGI200467[] = {{&S75806[34], 10}, {NULL}};
const Segment SGI200468[] = {{&S75806[27], 6}, {NULL}};
const Segment SGI200469[] = {{&S75806[97], 9}, {NULL}};
const Segment SGI200470[] = {{&S75806[70], 6}, {NULL}};
const Segment SGI200471[] = {{&S81816[34], 7}, {NULL}};
const Segment SGI200472[] = {{&S81816[27], 6}, {NULL}};
const Segment SGI200473[] = {{&S81816[94], 9}, {NULL}};
const Segment SGI200474[] = {{&S81816[67], 6}, {NULL}};
const Segment SGI200475[] = {{&S75819[34], 6}, {NULL}};
const Segment SGI200476[] = {{&S75819[27], 6}, {NULL}};
const Segment SGI200477[] = {{&S75819[93], 9}, {NULL}};
const Segment SGI200478[] = {{&S75819[66], 6}, {NULL}};
const Segment SGI200479[] = {{&S81825[44], 7}, {NULL}};
const Segment SGI200480[] = {{&S81825[44], 24}, {NULL}};
const Segment SGI200481[] = {{&S81825[30], 6}, {NULL}};
const Segment SGI200482[] = {{&S81825[82], 6}, {NULL}};
const Segment SGI200483[] = {{&S81825[109], 13}, {NULL}};
const Segment SGI200484[] = {{&S61394[34], 15}, {NULL}};
const Segment SGI200485[] = {{&S61394[27], 6}, {NULL}};
const Segment SGI200486[] = {{&S61394[101], 9}, {NULL}};
const Segment SGI200487[] = {{&S61394[74], 6}, {NULL}};
const Segment SGI200488[] = {{&S81831[34], 7}, {NULL}};
const Segment SGI200489[] = {{&S81831[27], 6}, {NULL}};
const Segment SGI200490[] = {{&S81831[99], 9}, {NULL}};
const Segment SGI200491[] = {{&S81831[72], 6}, {NULL}};
const Segment SGI200492[] = {{&S61395[11], 1}, {NULL}};
const Segment SGI200493[] = {{&S61395[27], 6}, {NULL}};
const Segment SGI200494[] = {{&S61395[88], 9}, {NULL}};
const Segment SGI200495[] = {{&S61395[61], 6}, {NULL}};
const Segment SGI200496[] = {{&S81832[44], 10}, {NULL}};
const Segment SGI200497[] = {{&S81832[44], 22}, {NULL}};
const Segment SGI200498[] = {{&S81832[30], 6}, {NULL}};
const Segment SGI200499[] = {{&S81832[31], 3}, {NULL}};
const Segment SGI200500[] = {{&S81832[80], 6}, {NULL}};
const Segment SGI200501[] = {{&S81832[140], 7}, {NULL}};
const Segment SGI200502[] = {{&S61399[34], 16}, {NULL}};
const Segment SGI200503[] = {{&S61399[27], 6}, {NULL}};
const Segment SGI200504[] = {{&S61399[103], 9}, {NULL}};
const Segment SGI200505[] = {{&S61399[76], 6}, {NULL}};
const Segment SGI200506[] = {{&S71201[34], 18}, {NULL}};
const Segment SGI200507[] = {{&S61400[11], 1}, {NULL}};
const Segment SGI200508[] = {{&S71201[27], 6}, {NULL}};
const Segment SGI200509[] = {{&S61400[27], 6}, {NULL}};
const Segment SGI200510[] = {{&S61400[88], 9}, {NULL}};
const Segment SGI200511[] = {{&S71201[105], 9}, {NULL}};
const Segment SGI200512[] = {{&S61400[61], 6}, {NULL}};
const Segment SGI200513[] = {{&S71201[78], 6}, {NULL}};
const Segment SGI200514[] = {{&S71202[34], 10}, {NULL}};
const Segment SGI200515[] = {{&S71202[27], 6}, {NULL}};
const Segment SGI200516[] = {{&S71202[97], 9}, {NULL}};
const Segment SGI200517[] = {{&S71202[70], 6}, {NULL}};
const Segment SGI200518[] = {{&S81841[34], 13}, {NULL}};
const Segment SGI200519[] = {{&S81841[27], 6}, {NULL}};
const Segment SGI200520[] = {{&S81841[100], 9}, {NULL}};
const Segment SGI200521[] = {{&S81841[73], 6}, {NULL}};
const Segment SGI200522[] = {{&S71206[44], 10}, {NULL}};
const Segment SGI200523[] = {{&S71206[44], 22}, {NULL}};
const Segment SGI200524[] = {{&S71206[30], 6}, {NULL}};
const Segment SGI200525[] = {{&S71206[115], 3}, {NULL}};
const Segment SGI200526[] = {{&S71206[80], 6}, {NULL}};
const Segment SGI200527[] = {{&S71242[34], 10}, {NULL}};
const Segment SGI200528[] = {{&S71242[27], 6}, {NULL}};
const Segment SGI200529[] = {{&S71242[97], 9}, {NULL}};
const Segment SGI200530[] = {{&S71242[70], 6}, {NULL}};
const Segment SGI200531[] = {{&S71243[34], 11}, {NULL}};
const Segment SGI200532[] = {{&S71243[27], 6}, {NULL}};
const Segment SGI200533[] = {{&S71243[98], 9}, {NULL}};
const Segment SGI200534[] = {{&S71243[71], 6}, {NULL}};
const Segment SGI200535[] = {{&S71244[34], 11}, {NULL}};
const Segment SGI200536[] = {{&S71244[27], 6}, {NULL}};
const Segment SGI200537[] = {{&S71244[98], 9}, {NULL}};
const Segment SGI200538[] = {{&S71244[71], 6}, {NULL}};
const Segment SGI200539[] = {{&S71247[34], 11}, {NULL}};
const Segment SGI200540[] = {{&S71247[27], 6}, {NULL}};
const Segment SGI200541[] = {{&S71247[98], 9}, {NULL}};
const Segment SGI200542[] = {{&S71247[71], 6}, {NULL}};
const Segment SGI200543[] = {{&S81893[34], 7}, {NULL}};
const Segment SGI200544[] = {{&S81893[27], 6}, {NULL}};
const Segment SGI200545[] = {{&S81893[94], 9}, {NULL}};
const Segment SGI200546[] = {{&S81893[67], 6}, {NULL}};
const Segment SGI200547[] = {{&S81896[44], 18}, {NULL}};
const Segment SGI200548[] = {{&S81896[44], 37}, {NULL}};
const Segment SGI200549[] = {{&S81896[30], 6}, {NULL}};
const Segment SGI200550[] = {{&S81896[31], 3}, {NULL}};
const Segment SGI200551[] = {{&S81896[95], 6}, {NULL}};
const Segment SGI200552[] = {{&S81900[34], 10}, {NULL}};
const Segment SGI200553[] = {{&S81900[27], 6}, {NULL}};
const Segment SGI200554[] = {{&S81900[97], 9}, {NULL}};
const Segment SGI200555[] = {{&S81900[70], 6}, {NULL}};
const Segment SGI200556[] = {{&S81904[44], 13}, {NULL}};
const Segment SGI200557[] = {{&S81904[44], 25}, {NULL}};
const Segment SGI200558[] = {{&S81904[30], 6}, {NULL}};
const Segment SGI200559[] = {{&S81904[31], 3}, {NULL}};
const Segment SGI200560[] = {{&S81904[83], 6}, {NULL}};
const Segment SGI200561[] = {{&S81905[44], 11}, {NULL}};
const Segment SGI200562[] = {{&S81905[44], 23}, {NULL}};
const Segment SGI200563[] = {{&S81905[30], 6}, {NULL}};
const Segment SGI200564[] = {{&S81905[31], 3}, {NULL}};
const Segment SGI200565[] = {{&S81905[81], 6}, {NULL}};
const Segment SGI200566[] = {{&S81916[34], 5}, {NULL}};
const Segment SGI200567[] = {{&S81916[27], 6}, {NULL}};
const Segment SGI200568[] = {{&S81916[92], 9}, {NULL}};
const Segment SGI200569[] = {{&S81916[65], 6}, {NULL}};
const Segment SGI200570[] = {{&S81917[34], 17}, {NULL}};
const Segment SGI200571[] = {{&S81917[27], 6}, {NULL}};
const Segment SGI200572[] = {{&S81917[104], 9}, {NULL}};
const Segment SGI200573[] = {{&S81917[77], 6}, {NULL}};
const Segment SGI200574[] = {{&S71293[34], 7}, {NULL}};
const Segment SGI200575[] = {{&S71293[27], 6}, {NULL}};
const Segment SGI200576[] = {{&S71293[99], 9}, {NULL}};
const Segment SGI200577[] = {{&S71293[72], 6}, {NULL}};
const Segment SGI200578[] = {{&S71296[44], 10}, {NULL}};
const Segment SGI200579[] = {{&S71296[44], 22}, {NULL}};
const Segment SGI200580[] = {{&S71296[30], 6}, {NULL}};
const Segment SGI200581[] = {{&S71296[80], 6}, {NULL}};
const Segment SGI200582[] = {{&S71296[107], 13}, {NULL}};
const Segment SGI200583[] = {{&S75950[44], 4}, {NULL}};
const Segment SGI200584[] = {{&S75950[44], 16}, {NULL}};
const Segment SGI200585[] = {{&S75950[30], 6}, {NULL}};
const Segment SGI200586[] = {{&S75950[109], 3}, {NULL}};
const Segment SGI200587[] = {{&S75950[74], 6}, {NULL}};
const Segment SGI200588[] = {{&S71322[34], 13}, {NULL}};
const Segment SGI200589[] = {{&S71322[27], 6}, {NULL}};
const Segment SGI200590[] = {{&S71322[100], 9}, {NULL}};
const Segment SGI200591[] = {{&S71322[73], 6}, {NULL}};
const Segment SGI200592[] = {{&S71335[44], 10}, {NULL}};
const Segment SGI200593[] = {{&S71335[44], 22}, {NULL}};
const Segment SGI200594[] = {{&S71335[30], 6}, {NULL}};
const Segment SGI200595[] = {{&S71335[80], 6}, {NULL}};
const Segment SGI200596[] = {{&S71335[107], 13}, {NULL}};
const Segment SGI200597[] = {{&S81969[44], 7}, {NULL}};
const Segment SGI200598[] = {{&S81969[44], 19}, {NULL}};
const Segment SGI200599[] = {{&S81969[30], 6}, {NULL}};
const Segment SGI200600[] = {{&S81969[31], 3}, {NULL}};
const Segment SGI200601[] = {{&S81969[77], 6}, {NULL}};
const Segment SGI200602[] = {{&S81972[34], 10}, {NULL}};
const Segment SGI200603[] = {{&S81972[27], 6}, {NULL}};
const Segment SGI200604[] = {{&S81972[97], 9}, {NULL}};
const Segment SGI200605[] = {{&S81972[70], 6}, {NULL}};
const Segment SGI200606[] = {{&S81973[34], 10}, {NULL}};
const Segment SGI200607[] = {{&S81973[27], 6}, {NULL}};
const Segment SGI200608[] = {{&S81973[97], 9}, {NULL}};
const Segment SGI200609[] = {{&S81973[70], 6}, {NULL}};
const Segment SGI200610[] = {{&S81974[44], 23}, {NULL}};
const Segment SGI200611[] = {{&S81974[44], 35}, {NULL}};
const Segment SGI200612[] = {{&S81974[30], 6}, {NULL}};
const Segment SGI200613[] = {{&S81974[31], 3}, {NULL}};
const Segment SGI200614[] = {{&S81974[93], 6}, {NULL}};
const Segment SGI200615[] = {{&S81983[44], 14}, {NULL}};
const Segment SGI200616[] = {{&S81983[44], 26}, {NULL}};
const Segment SGI200617[] = {{&S81983[30], 6}, {NULL}};
const Segment SGI200618[] = {{&S81983[87], 6}, {NULL}};
const Segment SGI200619[] = {{&S81985[44], 14}, {NULL}};
const Segment SGI200620[] = {{&S81985[44], 26}, {NULL}};
const Segment SGI200621[] = {{&S81985[30], 6}, {NULL}};
const Segment SGI200622[] = {{&S81985[31], 3}, {NULL}};
const Segment SGI200623[] = {{&S71364[44], 10}, {NULL}};
const Segment SGI200624[] = {{&S81985[84], 5}, {NULL}};
const Segment SGI200625[] = {{&S71364[44], 22}, {NULL}};
const Segment SGI200626[] = {{&S71364[30], 6}, {NULL}};
const Segment SGI200627[] = {{&S71364[80], 6}, {NULL}};
const Segment SGI200628[] = {{&S71364[107], 13}, {NULL}};
const Segment SGI200629[] = {{&S81986[34], 9}, {NULL}};
const Segment SGI200630[] = {{&S81986[27], 6}, {NULL}};
const Segment SGI200631[] = {{&S81986[96], 9}, {NULL}};
const Segment SGI200632[] = {{&S81986[69], 6}, {NULL}};
const Segment SGI200633[] = {{&S71365[44], 11}, {NULL}};
const Segment SGI200634[] = {{&S71365[44], 23}, {NULL}};
const Segment SGI200635[] = {{&S71365[30], 6}, {NULL}};
const Segment SGI200636[] = {{&S71365[81], 6}, {NULL}};
const Segment SGI200637[] = {{&S71365[108], 13}, {NULL}};
const Segment SGI200638[] = {{&S81987[34], 9}, {NULL}};
const Segment SGI200639[] = {{&S81987[27], 6}, {NULL}};
const Segment SGI200640[] = {{&S81987[96], 9}, {NULL}};
const Segment SGI200641[] = {{&S81987[69], 6}, {NULL}};
const Segment SGI200642[] = {{&S81988[34], 11}, {NULL}};
const Segment SGI200643[] = {{&S81988[27], 6}, {NULL}};
const Segment SGI200644[] = {{&S81988[105], 9}, {NULL}};
const Segment SGI200645[] = {{&S81988[78], 6}, {NULL}};
const Segment SGI200646[] = {{&S81994[44], 6}, {NULL}};
const Segment SGI200647[] = {{&S81994[44], 18}, {NULL}};
const Segment SGI200648[] = {{&S71376[34], 18}, {NULL}};
const Segment SGI200649[] = {{&S81994[30], 6}, {NULL}};
const Segment SGI200650[] = {{&S71376[27], 6}, {NULL}};
const Segment SGI200651[] = {{&S81994[76], 6}, {NULL}};
const Segment SGI200652[] = {{&S71376[105], 9}, {NULL}};
const Segment SGI200653[] = {{&S71376[78], 6}, {NULL}};
const Segment SGI200654[] = {{&S81994[103], 13}, {NULL}};
const Segment SGI200655[] = {{&S82002[44], 10}, {NULL}};
const Segment SGI200656[] = {{&S82002[44], 22}, {NULL}};
const Segment SGI200657[] = {{&S82002[30], 6}, {NULL}};
const Segment SGI200658[] = {{&S82002[31], 3}, {NULL}};
const Segment SGI200659[] = {{&S82002[80], 5}, {NULL}};
const Segment SGI200660[] = {{&S82002[117], 14}, {NULL}};
const Segment SGI200661[] = {{&S82003[44], 11}, {NULL}};
const Segment SGI200662[] = {{&S82003[44], 23}, {NULL}};
const Segment SGI200663[] = {{&S82003[30], 6}, {NULL}};
const Segment SGI200664[] = {{&S82003[31], 3}, {NULL}};
const Segment SGI200665[] = {{&S82003[81], 5}, {NULL}};
const Segment SGI200666[] = {{&S82003[118], 14}, {NULL}};
const Segment SGI200667[] = {{&S71389[44], 10}, {NULL}};
const Segment SGI200668[] = {{&S71389[44], 22}, {NULL}};
const Segment SGI200669[] = {{&S71389[30], 6}, {NULL}};
const Segment SGI200670[] = {{&S71389[131], 3}, {NULL}};
const Segment SGI200671[] = {{&S71389[96], 6}, {NULL}};
const Segment SGI200672[] = {{&S71389[68], 8}, {NULL}};
const Segment SGI200673[] = {{&S71390[44], 10}, {NULL}};
const Segment SGI200674[] = {{&S71390[44], 22}, {NULL}};
const Segment SGI200675[] = {{&S71390[30], 6}, {NULL}};
const Segment SGI200676[] = {{&S71390[131], 3}, {NULL}};
const Segment SGI200677[] = {{&S71390[96], 6}, {NULL}};
const Segment SGI200678[] = {{&S82006[34], 7}, {NULL}};
const Segment SGI200679[] = {{&S71390[68], 8}, {NULL}};
const Segment SGI200680[] = {{&S82006[27], 6}, {NULL}};
const Segment SGI200681[] = {{&S82006[100], 9}, {NULL}};
const Segment SGI200682[] = {{&S82006[73], 6}, {NULL}};
const Segment SGI200683[] = {{&S82006[141], 8}, {NULL}};
const Segment SGI200684[] = {{&S71395[44], 10}, {NULL}};
const Segment SGI200685[] = {{&S71395[44], 22}, {NULL}};
const Segment SGI200686[] = {{&S71395[30], 6}, {NULL}};
const Segment SGI200687[] = {{&S71395[115], 3}, {NULL}};
const Segment SGI200688[] = {{&S71395[80], 6}, {NULL}};
const Segment SGI200689[] = {{&S71395[150], 3}, {NULL}};
const Segment SGI200690[] = {{&S71396[44], 10}, {NULL}};
const Segment SGI200691[] = {{&S71396[44], 22}, {NULL}};
const Segment SGI200692[] = {{&S71396[30], 6}, {NULL}};
const Segment SGI200693[] = {{&S71396[115], 3}, {NULL}};
const Segment SGI200694[] = {{&S71396[80], 6}, {NULL}};
const Segment SGI200695[] = {{&S71396[143], 3}, {NULL}};
const Segment SGI200696[] = {{&S71397[34], 11}, {NULL}};
const Segment SGI200697[] = {{&S71397[27], 6}, {NULL}};
const Segment SGI200698[] = {{&S71397[98], 9}, {NULL}};
const Segment SGI200699[] = {{&S71397[71], 6}, {NULL}};
const Segment SGI200700[] = {{&S71398[34], 11}, {NULL}};
const Segment SGI200701[] = {{&S71398[27], 6}, {NULL}};
const Segment SGI200702[] = {{&S71398[98], 9}, {NULL}};
const Segment SGI200703[] = {{&S71398[71], 6}, {NULL}};
const Segment SGI200704[] = {{&S71401[34], 9}, {NULL}};
const Segment SGI200705[] = {{&S71401[27], 6}, {NULL}};
const Segment SGI200706[] = {{&S71401[96], 9}, {NULL}};
const Segment SGI200707[] = {{&S71401[69], 6}, {NULL}};
const Segment SGI200708[] = {{&S76057[34], 6}, {NULL}};
const Segment SGI200709[] = {{&S76057[27], 6}, {NULL}};
const Segment SGI200710[] = {{&S76057[93], 9}, {NULL}};
const Segment SGI200711[] = {{&S76057[66], 6}, {NULL}};
const Segment SGI200712[] = {{&S61597[41], 19}, {NULL}};
const Segment SGI200713[] = {{&S61597[41], 30}, {NULL}};
const Segment SGI200714[] = {{&S61597[27], 6}, {NULL}};
const Segment SGI200715[] = {{&S61597[120], 3}, {NULL}};
const Segment SGI200716[] = {{&S61597[124], 9}, {NULL}};
const Segment SGI200717[] = {{&S61597[85], 6}, {NULL}};
const Segment SGI200718[] = {{&S82036[44], 10}, {NULL}};
const Segment SGI200719[] = {{&S82036[44], 22}, {NULL}};
const Segment SGI200720[] = {{&S82036[30], 6}, {NULL}};
const Segment SGI200721[] = {{&S82036[111], 2}, {NULL}};
const Segment SGI200722[] = {{&S82036[80], 5}, {NULL}};
const Segment SGI200723[] = {{&S71422[44], 10}, {NULL}};
const Segment SGI200724[] = {{&S71422[44], 22}, {NULL}};
const Segment SGI200725[] = {{&S71422[30], 6}, {NULL}};
const Segment SGI200726[] = {{&S71422[115], 3}, {NULL}};
const Segment SGI200727[] = {{&S71422[80], 6}, {NULL}};
const Segment SGI200728[] = {{&S76075[34], 15}, {NULL}};
const Segment SGI200729[] = {{&S76075[27], 6}, {NULL}};
const Segment SGI200730[] = {{&S76075[102], 9}, {NULL}};
const Segment SGI200731[] = {{&S76075[75], 6}, {NULL}};
const Segment SGI200732[] = {{&S82053[34], 21}, {NULL}};
const Segment SGI200733[] = {{&S82053[27], 6}, {NULL}};
const Segment SGI200734[] = {{&S82053[108], 9}, {NULL}};
const Segment SGI200735[] = {{&S82053[81], 6}, {NULL}};
const Segment SGI200736[] = {{&S82054[34], 22}, {NULL}};
const Segment SGI200737[] = {{&S82054[27], 6}, {NULL}};
const Segment SGI200738[] = {{&S82054[109], 9}, {NULL}};
const Segment SGI200739[] = {{&S82054[82], 6}, {NULL}};
const Segment SGI200740[] = {{&S82055[34], 11}, {NULL}};
const Segment SGI200741[] = {{&S82055[27], 6}, {NULL}};
const Segment SGI200742[] = {{&S82055[98], 9}, {NULL}};
const Segment SGI200743[] = {{&S82055[71], 6}, {NULL}};
const Segment SGI200744[] = {{&S82058[44], 15}, {NULL}};
const Segment SGI200745[] = {{&S82058[44], 27}, {NULL}};
const Segment SGI200746[] = {{&S82058[30], 6}, {NULL}};
const Segment SGI200747[] = {{&S82058[31], 3}, {NULL}};
const Segment SGI200748[] = {{&S82058[85], 6}, {NULL}};
const Segment SGI200749[] = {{&S82070[44], 16}, {NULL}};
const Segment SGI200750[] = {{&S82070[44], 28}, {NULL}};
const Segment SGI200751[] = {{&S82070[30], 6}, {NULL}};
const Segment SGI200752[] = {{&S82070[31], 3}, {NULL}};
const Segment SGI200753[] = {{&S82070[86], 6}, {NULL}};
const Segment SGI200754[] = {{&S71466[34], 9}, {NULL}};
const Segment SGI200755[] = {{&S71466[27], 6}, {NULL}};
const Segment SGI200756[] = {{&S71466[96], 9}, {NULL}};
const Segment SGI200757[] = {{&S71466[69], 6}, {NULL}};
const Segment SGI200758[] = {{&S61652[34], 9}, {NULL}};
const Segment SGI200759[] = {{&S61652[27], 6}, {NULL}};
const Segment SGI200760[] = {{&S61652[95], 9}, {NULL}};
const Segment SGI200761[] = {{&S61652[68], 6}, {NULL}};
const Segment SGI200762[] = {{&S82085[44], 10}, {NULL}};
const Segment SGI200763[] = {{&S82085[44], 22}, {NULL}};
const Segment SGI200764[] = {{&S82085[30], 6}, {NULL}};
const Segment SGI200765[] = {{&S82085[80], 6}, {NULL}};
const Segment SGI200766[] = {{&S82085[107], 13}, {NULL}};
const Segment SGI200767[] = {{&S61665[44], 10}, {NULL}};
const Segment SGI200768[] = {{&S61665[44], 21}, {NULL}};
const Segment SGI200769[] = {{&S61665[30], 6}, {NULL}};
const Segment SGI200770[] = {{&S61665[114], 3}, {NULL}};
const Segment SGI200771[] = {{&S61665[79], 6}, {NULL}};
const Segment SGI200772[] = {{&S61666[44], 18}, {NULL}};
const Segment SGI200773[] = {{&S61666[44], 29}, {NULL}};
const Segment SGI200774[] = {{&S61666[30], 6}, {NULL}};
const Segment SGI200775[] = {{&S61666[122], 3}, {NULL}};
const Segment SGI200776[] = {{&S61666[87], 6}, {NULL}};
const Segment SGI200777[] = {{&S82090[44], 7}, {NULL}};
const Segment SGI200778[] = {{&S82090[44], 24}, {NULL}};
const Segment SGI200779[] = {{&S82090[30], 6}, {NULL}};
const Segment SGI200780[] = {{&S61667[44], 18}, {NULL}};
const Segment SGI200781[] = {{&S61667[44], 30}, {NULL}};
const Segment SGI200782[] = {{&S61667[30], 6}, {NULL}};
const Segment SGI200783[] = {{&S82090[82], 6}, {NULL}};
const Segment SGI200784[] = {{&S61667[123], 3}, {NULL}};
const Segment SGI200785[] = {{&S82090[109], 13}, {NULL}};
const Segment SGI200786[] = {{&S61667[88], 6}, {NULL}};
const Segment SGI200787[] = {{&S61668[38], 18}, {NULL}};
const Segment SGI200788[] = {{&S82091[44], 5}, {NULL}};
const Segment SGI200789[] = {{&S61668[31], 6}, {NULL}};
const Segment SGI200790[] = {{&S82091[44], 17}, {NULL}};
const Segment SGI200791[] = {{&S82091[30], 6}, {NULL}};
const Segment SGI200792[] = {{&S82091[75], 6}, {NULL}};
const Segment SGI200793[] = {{&S82091[102], 13}, {NULL}};
const Segment SGI200794[] = {{&S82092[44], 13}, {NULL}};
const Segment SGI200795[] = {{&S82092[44], 25}, {NULL}};
const Segment SGI200796[] = {{&S82092[30], 6}, {NULL}};
const Segment SGI200797[] = {{&S82092[83], 6}, {NULL}};
const Segment SGI200798[] = {{&S82092[110], 13}, {NULL}};
const Segment SGI200799[] = {{&S71498[44], 11}, {NULL}};
const Segment SGI200800[] = {{&S71498[44], 23}, {NULL}};
const Segment SGI200801[] = {{&S71498[30], 6}, {NULL}};
const Segment SGI200802[] = {{&S71498[116], 3}, {NULL}};
const Segment SGI200803[] = {{&S71498[81], 6}, {NULL}};
const Segment SGI200804[] = {{&S82096[34], 11}, {NULL}};
const Segment SGI200805[] = {{&S82096[27], 6}, {NULL}};
const Segment SGI200806[] = {{&S82096[105], 9}, {NULL}};
const Segment SGI200807[] = {{&S82096[78], 6}, {NULL}};
const Segment SGI200808[] = {{&S71502[34], 6}, {NULL}};
const Segment SGI200809[] = {{&S71502[27], 6}, {NULL}};
const Segment SGI200810[] = {{&S71502[93], 9}, {NULL}};
const Segment SGI200811[] = {{&S71502[66], 6}, {NULL}};
const Segment SGI200812[] = {{&S71513[44], 10}, {NULL}};
const Segment SGI200813[] = {{&S71513[44], 22}, {NULL}};
const Segment SGI200814[] = {{&S71513[30], 6}, {NULL}};
const Segment SGI200815[] = {{&S71513[136], 3}, {NULL}};
const Segment SGI200816[] = {{&S71513[101], 6}, {NULL}};
const Segment SGI200817[] = {{&S71515[44], 10}, {NULL}};
const Segment SGI200818[] = {{&S71515[44], 22}, {NULL}};
const Segment SGI200819[] = {{&S71515[30], 6}, {NULL}};
const Segment SGI200820[] = {{&S61692[34], 17}, {NULL}};
const Segment SGI200821[] = {{&S71515[136], 3}, {NULL}};
const Segment SGI200822[] = {{&S61692[27], 6}, {NULL}};
const Segment SGI200823[] = {{&S71515[101], 6}, {NULL}};
const Segment SGI200824[] = {{&S61692[103], 9}, {NULL}};
const Segment SGI200825[] = {{&S61692[76], 6}, {NULL}};
const Segment SGI200826[] = {{&S61693[34], 17}, {NULL}};
const Segment SGI200827[] = {{&S61693[27], 6}, {NULL}};
const Segment SGI200828[] = {{&S61693[103], 9}, {NULL}};
const Segment SGI200829[] = {{&S61693[76], 6}, {NULL}};
const Segment SGI200830[] = {{&S82127[44], 14}, {NULL}};
const Segment SGI200831[] = {{&S82127[44], 26}, {NULL}};
const Segment SGI200832[] = {{&S82127[30], 6}, {NULL}};
const Segment SGI200833[] = {{&S82127[31], 3}, {NULL}};
const Segment SGI200834[] = {{&S82127[84], 6}, {NULL}};
const Segment SGI200835[] = {{&S82128[34], 10}, {NULL}};
const Segment SGI200836[] = {{&S82128[27], 6}, {NULL}};
const Segment SGI200837[] = {{&S82128[96], 9}, {NULL}};
const Segment SGI200838[] = {{&S82128[69], 6}, {NULL}};
const Segment SGI200839[] = {{&S82130[44], 14}, {NULL}};
const Segment SGI200840[] = {{&S82130[44], 26}, {NULL}};
const Segment SGI200841[] = {{&S82130[30], 6}, {NULL}};
const Segment SGI200842[] = {{&S82130[31], 3}, {NULL}};
const Segment SGI200843[] = {{&S82130[84], 6}, {NULL}};
const Segment SGI200844[] = {{&S82131[44], 8}, {NULL}};
const Segment SGI200845[] = {{&S82131[44], 20}, {NULL}};
const Segment SGI200846[] = {{&S82131[30], 6}, {NULL}};
const Segment SGI200847[] = {{&S76195[44], 9}, {NULL}};
const Segment SGI200848[] = {{&S76195[44], 21}, {NULL}};
const Segment SGI200849[] = {{&S76195[30], 6}, {NULL}};
const Segment SGI200850[] = {{&S82131[31], 3}, {NULL}};
const Segment SGI200851[] = {{&S82131[78], 6}, {NULL}};
const Segment SGI200852[] = {{&S76195[114], 3}, {NULL}};
const Segment SGI200853[] = {{&S76195[79], 6}, {NULL}};
const Segment SGI200854[] = {{&S76196[44], 9}, {NULL}};
const Segment SGI200855[] = {{&S76196[44], 21}, {NULL}};
const Segment SGI200856[] = {{&S76196[30], 6}, {NULL}};
const Segment SGI200857[] = {{&S76196[114], 3}, {NULL}};
const Segment SGI200858[] = {{&S76196[79], 6}, {NULL}};
const Segment SGI200859[] = {{&S82134[34], 7}, {NULL}};
const Segment SGI200860[] = {{&S82134[27], 6}, {NULL}};
const Segment SGI200861[] = {{&S82134[104], 9}, {NULL}};
const Segment SGI200862[] = {{&S82134[77], 6}, {NULL}};
const Segment SGI200863[] = {{&S82135[34], 10}, {NULL}};
const Segment SGI200864[] = {{&S82135[27], 6}, {NULL}};
const Segment SGI200865[] = {{&S82135[97], 9}, {NULL}};
const Segment SGI200866[] = {{&S82135[70], 6}, {NULL}};
const Segment SGI200867[] = {{&S82136[11], 1}, {NULL}};
const Segment SGI200868[] = {{&S82136[44], 10}, {NULL}};
const Segment SGI200869[] = {{&S82136[30], 6}, {NULL}};
const Segment SGI200870[] = {{&S82136[31], 3}, {NULL}};
const Segment SGI200871[] = {{&S82136[82], 6}, {NULL}};
const Segment SGI200872[] = {{&S82137[44], 11}, {NULL}};
const Segment SGI200873[] = {{&S82137[44], 23}, {NULL}};
const Segment SGI200874[] = {{&S82137[30], 6}, {NULL}};
const Segment SGI200875[] = {{&S82137[31], 3}, {NULL}};
const Segment SGI200876[] = {{&S82137[81], 6}, {NULL}};
const Segment SGI200877[] = {{&S82139[44], 9}, {NULL}};
const Segment SGI200878[] = {{&S82139[44], 21}, {NULL}};
const Segment SGI200879[] = {{&S82139[30], 6}, {NULL}};
const Segment SGI200880[] = {{&S82139[31], 3}, {NULL}};
const Segment SGI200881[] = {{&S82139[79], 6}, {NULL}};
const Segment SGI200882[] = {{&S82147[34], 24}, {NULL}};
const Segment SGI200883[] = {{&S82147[27], 6}, {NULL}};
const Segment SGI200884[] = {{&S76219[44], 11}, {NULL}};
const Segment SGI200885[] = {{&S76219[44], 23}, {NULL}};
const Segment SGI200886[] = {{&S76219[30], 6}, {NULL}};
const Segment SGI200887[] = {{&S82147[111], 9}, {NULL}};
const Segment SGI200888[] = {{&S82147[84], 6}, {NULL}};
const Segment SGI200889[] = {{&S76219[116], 3}, {NULL}};
const Segment SGI200890[] = {{&S76219[81], 6}, {NULL}};
const Segment SGI200891[] = {{&S70866[34], 6}, {NULL}};
const Segment SGI200892[] = {{&S70866[27], 6}, {NULL}};
const Segment SGI200893[] = {{&S70866[93], 9}, {NULL}};
const Segment SGI200894[] = {{&S70866[66], 6}, {NULL}};
const Segment SGI200895[] = {{&S70868[34], 6}, {NULL}};
const Segment SGI200896[] = {{&S70868[27], 6}, {NULL}};
const Segment SGI200897[] = {{&S70868[93], 9}, {NULL}};
const Segment SGI200898[] = {{&S70868[66], 6}, {NULL}};
const Segment SGI200899[] = {{&S71522[34], 10}, {NULL}};
const Segment SGI200900[] = {{&S71522[27], 6}, {NULL}};
const Segment SGI200901[] = {{&S71522[97], 9}, {NULL}};
const Segment SGI200902[] = {{&S71522[70], 6}, {NULL}};
const Segment SGI200903[] = {{&S82152[34], 9}, {NULL}};
const Segment SGI200904[] = {{&S82152[27], 6}, {NULL}};
const Segment SGI200905[] = {{&S82152[96], 9}, {NULL}};
const Segment SGI200906[] = {{&S82152[69], 6}, {NULL}};
const Segment SGI200907[] = {{&S71523[34], 18}, {NULL}};
const Segment SGI200908[] = {{&S71523[27], 6}, {NULL}};
const Segment SGI200909[] = {{&S71523[105], 9}, {NULL}};
const Segment SGI200910[] = {{&S71523[78], 6}, {NULL}};
const Segment SGI200911[] = {{&S82153[34], 9}, {NULL}};
const Segment SGI200912[] = {{&S82153[27], 6}, {NULL}};
const Segment SGI200913[] = {{&S82153[96], 9}, {NULL}};
const Segment SGI200914[] = {{&S82153[69], 6}, {NULL}};
const Segment SGI200915[] = {{&S71524[34], 10}, {NULL}};
const Segment SGI200916[] = {{&S71524[27], 6}, {NULL}};
const Segment SGI200917[] = {{&S71524[97], 9}, {NULL}};
const Segment SGI200918[] = {{&S71524[70], 6}, {NULL}};
const Segment SGI200919[] = {{&S71544[34], 11}, {NULL}};
const Segment SGI200920[] = {{&S71544[27], 6}, {NULL}};
const Segment SGI200921[] = {{&S71544[98], 9}, {NULL}};
const Segment SGI200922[] = {{&S71544[71], 6}, {NULL}};
const Segment SGI200923[] = {{&S71545[34], 11}, {NULL}};
const Segment SGI200924[] = {{&S71545[27], 6}, {NULL}};
const Segment SGI200925[] = {{&S71545[98], 9}, {NULL}};
const Segment SGI200926[] = {{&S71545[71], 6}, {NULL}};
const Segment SGI200927[] = {{&S71546[34], 10}, {NULL}};
const Segment SGI200928[] = {{&S71546[27], 6}, {NULL}};
const Segment SGI200929[] = {{&S71546[97], 9}, {NULL}};
const Segment SGI200930[] = {{&S71546[70], 6}, {NULL}};
const Segment SGI200931[] = {{&S71547[34], 7}, {NULL}};
const Segment SGI200932[] = {{&S71547[27], 6}, {NULL}};
const Segment SGI200933[] = {{&S71547[94], 9}, {NULL}};
const Segment SGI200934[] = {{&S71547[67], 6}, {NULL}};
const Segment SGI200935[] = {{&S71548[44], 15}, {NULL}};
const Segment SGI200936[] = {{&S71548[44], 27}, {NULL}};
const Segment SGI200937[] = {{&S71548[30], 6}, {NULL}};
const Segment SGI200938[] = {{&S71548[85], 6}, {NULL}};
const Segment SGI200939[] = {{&S71548[112], 13}, {NULL}};
const Segment SGI200940[] = {{&S61748[34], 10}, {NULL}};
const Segment SGI200941[] = {{&S61748[27], 6}, {NULL}};
const Segment SGI200942[] = {{&S61748[96], 9}, {NULL}};
const Segment SGI200943[] = {{&S61748[69], 6}, {NULL}};
const Segment SGI200944[] = {{&S82202[44], 13}, {NULL}};
const Segment SGI200945[] = {{&S82202[44], 25}, {NULL}};
const Segment SGI200946[] = {{&S82202[30], 6}, {NULL}};
const Segment SGI200947[] = {{&S82202[31], 3}, {NULL}};
const Segment SGI200948[] = {{&S82202[83], 6}, {NULL}};
const Segment SGI200949[] = {{&S61767[38], 11}, {NULL}};
const Segment SGI200950[] = {{&S61767[31], 6}, {NULL}};
const Segment SGI200951[] = {{&S70942[44], 10}, {NULL}};
const Segment SGI200952[] = {{&S70942[44], 22}, {NULL}};
const Segment SGI200953[] = {{&S70942[30], 6}, {NULL}};
const Segment SGI200954[] = {{&S70942[115], 3}, {NULL}};
const Segment SGI200955[] = {{&S70942[80], 6}, {NULL}};
const Segment SGI200956[] = {{&S61768[44], 11}, {NULL}};
const Segment SGI200957[] = {{&S61768[44], 22}, {NULL}};
const Segment SGI200958[] = {{&S61768[30], 6}, {NULL}};
const Segment SGI200959[] = {{&S61768[115], 3}, {NULL}};
const Segment SGI200960[] = {{&S61768[80], 6}, {NULL}};
const Segment SGI200961[] = {{&S82221[45], 7}, {NULL}};
const Segment SGI200962[] = {{&S82221[30], 6}, {NULL}};
const Segment SGI200963[] = {{&S70945[44], 10}, {NULL}};
const Segment SGI200964[] = {{&S70945[44], 22}, {NULL}};
const Segment SGI200965[] = {{&S82221[31], 3}, {NULL}};
const Segment SGI200966[] = {{&S70945[30], 6}, {NULL}};
const Segment SGI200967[] = {{&S82221[82], 5}, {NULL}};
const Segment SGI200968[] = {{&S70945[115], 3}, {NULL}};
const Segment SGI200969[] = {{&S82221[140], 13}, {NULL}};
const Segment SGI200970[] = {{&S70945[80], 6}, {NULL}};
const Segment SGI200971[] = {{&S71588[30], 6}, {NULL}};
const Segment SGI200972[] = {{&S82222[34], 9}, {NULL}};
const Segment SGI200973[] = {{&S82222[27], 6}, {NULL}};
const Segment SGI200974[] = {{&S71588[85], 9}, {NULL}};
const Segment SGI200975[] = {{&S71588[57], 6}, {NULL}};
const Segment SGI200976[] = {{&S82222[96], 9}, {NULL}};
const Segment SGI200977[] = {{&S82222[69], 6}, {NULL}};
const Segment SGI200978[] = {{&S82223[34], 13}, {NULL}};
const Segment SGI200979[] = {{&S82223[27], 6}, {NULL}};
const Segment SGI200980[] = {{&S82223[100], 9}, {NULL}};
const Segment SGI200981[] = {{&S82223[73], 6}, {NULL}};
const Segment SGI200982[] = {{&S82229[34], 10}, {NULL}};
const Segment SGI200983[] = {{&S82229[27], 6}, {NULL}};
const Segment SGI200984[] = {{&S82229[97], 9}, {NULL}};
const Segment SGI200985[] = {{&S82229[70], 6}, {NULL}};
const Segment SGI200986[] = {{&S71598[44], 10}, {NULL}};
const Segment SGI200987[] = {{&S71598[44], 22}, {NULL}};
const Segment SGI200988[] = {{&S71598[30], 6}, {NULL}};
const Segment SGI200989[] = {{&S71598[115], 3}, {NULL}};
const Segment SGI200990[] = {{&S71598[80], 6}, {NULL}};
const Segment SGI200991[] = {{&S82231[34], 12}, {NULL}};
const Segment SGI200992[] = {{&S71599[38], 10}, {NULL}};
const Segment SGI200993[] = {{&S82231[27], 6}, {NULL}};
const Segment SGI200994[] = {{&S71599[31], 6}, {NULL}};
const Segment SGI200995[] = {{&S82231[99], 9}, {NULL}};
const Segment SGI200996[] = {{&S82231[72], 6}, {NULL}};
const Segment SGI200997[] = {{&S82232[34], 15}, {NULL}};
const Segment SGI200998[] = {{&S82232[27], 6}, {NULL}};
const Segment SGI200999[] = {{&S82232[102], 9}, {NULL}};
const Segment SGI201000[] = {{&S82232[75], 6}, {NULL}};
const Segment SGI201001[] = {{&S71603[34], 10}, {NULL}};
const Segment SGI201002[] = {{&S71603[27], 6}, {NULL}};
const Segment SGI201003[] = {{&S71603[97], 9}, {NULL}};
const Segment SGI201004[] = {{&S71603[70], 6}, {NULL}};
const Segment SGI201005[] = {{&S82237[34], 7}, {NULL}};
const Segment SGI201006[] = {{&S82237[27], 6}, {NULL}};
const Segment SGI201007[] = {{&S82237[99], 9}, {NULL}};
const Segment SGI201008[] = {{&S82237[72], 6}, {NULL}};
const Segment SGI201009[] = {{&S82237[140], 8}, {NULL}};
const Segment SGI201010[] = {{&S71611[34], 10}, {NULL}};
const Segment SGI201011[] = {{&S71611[27], 6}, {NULL}};
const Segment SGI201012[] = {{&S71611[97], 9}, {NULL}};
const Segment SGI201013[] = {{&S71611[70], 6}, {NULL}};
const Segment SGI201014[] = {{&S82245[34], 15}, {NULL}};
const Segment SGI201015[] = {{&S82245[27], 6}, {NULL}};
const Segment SGI201016[] = {{&S82245[102], 9}, {NULL}};
const Segment SGI201017[] = {{&S76220[44], 16}, {NULL}};
const Segment SGI201018[] = {{&S82245[75], 6}, {NULL}};
const Segment SGI201019[] = {{&S76220[44], 38}, {NULL}};
const Segment SGI201020[] = {{&S76220[30], 6}, {NULL}};
const Segment SGI201021[] = {{&S82245[143], 8}, {NULL}};
const Segment SGI201022[] = {{&S76220[131], 3}, {NULL}};
const Segment SGI201023[] = {{&S76220[96], 6}, {NULL}};
const Segment SGI201024[] = {{&S82246[34], 14}, {NULL}};
const Segment SGI201025[] = {{&S82246[27], 6}, {NULL}};
const Segment SGI201026[] = {{&S82246[101], 9}, {NULL}};
const Segment SGI201027[] = {{&S82246[74], 6}, {NULL}};
const Segment SGI201028[] = {{&S82246[142], 8}, {NULL}};
const Segment SGI201029[] = {{&S76237[44], 12}, {NULL}};
const Segment SGI201030[] = {{&S76237[44], 24}, {NULL}};
const Segment SGI201031[] = {{&S76237[30], 6}, {NULL}};
const Segment SGI201032[] = {{&S76237[133], 3}, {NULL}};
const Segment SGI201033[] = {{&S76237[98], 6}, {NULL}};
const Segment SGI201034[] = {{&S76237[70], 8}, {NULL}};
const Segment SGI201035[] = {{&S76247[44], 11}, {NULL}};
const Segment SGI201036[] = {{&S76247[44], 23}, {NULL}};
const Segment SGI201037[] = {{&S76247[30], 6}, {NULL}};
const Segment SGI201038[] = {{&S76247[116], 3}, {NULL}};
const Segment SGI201039[] = {{&S76247[81], 6}, {NULL}};
const Segment SGI201040[] = {{&S76248[38], 11}, {NULL}};
const Segment SGI201041[] = {{&S76248[31], 6}, {NULL}};
const Segment SGI201042[] = {{&S76253[34], 25}, {NULL}};
const Segment SGI201043[] = {{&S76253[27], 6}, {NULL}};
const Segment SGI201044[] = {{&S76253[112], 9}, {NULL}};
const Segment SGI201045[] = {{&S76253[85], 6}, {NULL}};
const Segment SGI201046[] = {{&S82279[44], 7}, {NULL}};
const Segment SGI201047[] = {{&S82279[44], 19}, {NULL}};
const Segment SGI201048[] = {{&S82279[30], 6}, {NULL}};
const Segment SGI201049[] = {{&S82279[31], 3}, {NULL}};
const Segment SGI201050[] = {{&S82279[96], 6}, {NULL}};
const Segment SGI201051[] = {{&S82279[65], 8}, {NULL}};
const Segment SGI201052[] = {{&S82280[44], 7}, {NULL}};
const Segment SGI201053[] = {{&S82280[44], 19}, {NULL}};
const Segment SGI201054[] = {{&S82280[30], 6}, {NULL}};
const Segment SGI201055[] = {{&S82280[31], 3}, {NULL}};
const Segment SGI201056[] = {{&S82280[96], 6}, {NULL}};
const Segment SGI201057[] = {{&S82280[65], 8}, {NULL}};
const Segment SGI201058[] = {{&S82281[44], 7}, {NULL}};
const Segment SGI201059[] = {{&S82281[44], 19}, {NULL}};
const Segment SGI201060[] = {{&S82281[30], 6}, {NULL}};
const Segment SGI201061[] = {{&S82281[31], 3}, {NULL}};
const Segment SGI201062[] = {{&S82281[92], 6}, {NULL}};
const Segment SGI201063[] = {{&S82281[65], 8}, {NULL}};
const Segment SGI201064[] = {{&S82284[34], 10}, {NULL}};
const Segment SGI201065[] = {{&S82284[27], 6}, {NULL}};
const Segment SGI201066[] = {{&S82284[97], 9}, {NULL}};
const Segment SGI201067[] = {{&S82284[70], 6}, {NULL}};
const Segment SGI201068[] = {{&S82288[44], 7}, {NULL}};
const Segment SGI201069[] = {{&S82288[44], 19}, {NULL}};
const Segment SGI201070[] = {{&S82288[30], 6}, {NULL}};
const Segment SGI201071[] = {{&S82288[31], 3}, {NULL}};
const Segment SGI201072[] = {{&S82288[92], 6}, {NULL}};
const Segment SGI201073[] = {{&S82288[65], 8}, {NULL}};
const Segment SGI201074[] = {{&S82289[34], 8}, {NULL}};
const Segment SGI201075[] = {{&S82289[27], 6}, {NULL}};
const Segment SGI201076[] = {{&S82289[95], 9}, {NULL}};
const Segment SGI201077[] = {{&S82289[68], 6}, {NULL}};
const Segment SGI201078[] = {{&S71676[44], 7}, {NULL}};
const Segment SGI201079[] = {{&S71676[44], 19}, {NULL}};
const Segment SGI201080[] = {{&S71676[30], 6}, {NULL}};
const Segment SGI201081[] = {{&S71676[128], 3}, {NULL}};
const Segment SGI201082[] = {{&S71676[93], 6}, {NULL}};
const Segment SGI201083[] = {{&S71676[65], 8}, {NULL}};
const Segment SGI201084[] = {{&S82291[34], 5}, {NULL}};
const Segment SGI201085[] = {{&S82291[27], 6}, {NULL}};
const Segment SGI201086[] = {{&S82291[92], 9}, {NULL}};
const Segment SGI201087[] = {{&S82291[65], 6}, {NULL}};
const Segment SGI201088[] = {{&S82292[34], 8}, {NULL}};
const Segment SGI201089[] = {{&S82292[27], 6}, {NULL}};
const Segment SGI201090[] = {{&S82292[100], 9}, {NULL}};
const Segment SGI201091[] = {{&S82292[73], 6}, {NULL}};
const Segment SGI201092[] = {{&S61867[44], 10}, {NULL}};
const Segment SGI201093[] = {{&S61867[44], 21}, {NULL}};
const Segment SGI201094[] = {{&S61867[30], 6}, {NULL}};
const Segment SGI201095[] = {{&S61867[137], 3}, {NULL}};
const Segment SGI201096[] = {{&S61867[102], 6}, {NULL}};
const Segment SGI201097[] = {{&S61868[44], 10}, {NULL}};
const Segment SGI201098[] = {{&S61868[44], 21}, {NULL}};
const Segment SGI201099[] = {{&S61868[30], 6}, {NULL}};
const Segment SGI201100[] = {{&S61868[139], 3}, {NULL}};
const Segment SGI201101[] = {{&S61868[104], 6}, {NULL}};
const Segment SGI201102[] = {{&S82295[34], 10}, {NULL}};
const Segment SGI201103[] = {{&S82295[27], 6}, {NULL}};
const Segment SGI201104[] = {{&S61868[74], 9}, {NULL}};
const Segment SGI201105[] = {{&S82295[97], 9}, {NULL}};
const Segment SGI201106[] = {{&S82295[70], 6}, {NULL}};
const Segment SGI201107[] = {{&S61869[44], 10}, {NULL}};
const Segment SGI201108[] = {{&S61869[44], 21}, {NULL}};
const Segment SGI201109[] = {{&S61869[30], 6}, {NULL}};
const Segment SGI201110[] = {{&S61869[137], 3}, {NULL}};
const Segment SGI201111[] = {{&S61869[102], 6}, {NULL}};
const Segment SGI201112[] = {{&S61869[74], 9}, {NULL}};
const Segment SGI201113[] = {{&S61870[44], 10}, {NULL}};
const Segment SGI201114[] = {{&S61870[44], 21}, {NULL}};
const Segment SGI201115[] = {{&S61870[30], 6}, {NULL}};
const Segment SGI201116[] = {{&S61870[139], 3}, {NULL}};
const Segment SGI201117[] = {{&S61870[104], 6}, {NULL}};
const Segment SGI201118[] = {{&S61870[74], 9}, {NULL}};
const Segment SGI201119[] = {{&S61871[44], 10}, {NULL}};
const Segment SGI201120[] = {{&S61871[44], 22}, {NULL}};
const Segment SGI201121[] = {{&S61871[30], 6}, {NULL}};
const Segment SGI201122[] = {{&S61871[138], 3}, {NULL}};
const Segment SGI201123[] = {{&S61871[103], 6}, {NULL}};
const Segment SGI201124[] = {{&S61871[75], 9}, {NULL}};
const Segment SGI201125[] = {{&S61872[44], 10}, {NULL}};
const Segment SGI201126[] = {{&S61872[44], 22}, {NULL}};
const Segment SGI201127[] = {{&S61872[30], 6}, {NULL}};
const Segment SGI201128[] = {{&S61872[138], 3}, {NULL}};
const Segment SGI201129[] = {{&S61872[103], 6}, {NULL}};
const Segment SGI201130[] = {{&S61872[75], 9}, {NULL}};
const Segment SGI201131[] = {{&S76290[34], 20}, {NULL}};
const Segment SGI201132[] = {{&S76290[27], 6}, {NULL}};
const Segment SGI201133[] = {{&S76290[107], 9}, {NULL}};
const Segment SGI201134[] = {{&S76290[80], 6}, {NULL}};
const Segment SGI201135[] = {{&S61873[44], 10}, {NULL}};
const Segment SGI201136[] = {{&S61873[44], 22}, {NULL}};
const Segment SGI201137[] = {{&S61873[30], 6}, {NULL}};
const Segment SGI201138[] = {{&S61873[138], 3}, {NULL}};
const Segment SGI201139[] = {{&S61873[103], 6}, {NULL}};
const Segment SGI201140[] = {{&S61873[75], 9}, {NULL}};
const Segment SGI201141[] = {{&S71698[34], 10}, {NULL}};
const Segment SGI201142[] = {{&S71698[27], 6}, {NULL}};
const Segment SGI201143[] = {{&S71698[97], 9}, {NULL}};
const Segment SGI201144[] = {{&S71698[70], 6}, {NULL}};
const Segment SGI201145[] = {{&S71699[34], 10}, {NULL}};
const Segment SGI201146[] = {{&S71699[27], 6}, {NULL}};
const Segment SGI201147[] = {{&S71699[97], 9}, {NULL}};
const Segment SGI201148[] = {{&S71699[70], 6}, {NULL}};
const Segment SGI201149[] = {{&S82313[45], 10}, {NULL}};
const Segment SGI201150[] = {{&S82313[30], 6}, {NULL}};
const Segment SGI201151[] = {{&S71700[34], 9}, {NULL}};
const Segment SGI201152[] = {{&S71700[27], 6}, {NULL}};
const Segment SGI201153[] = {{&S82313[31], 3}, {NULL}};
const Segment SGI201154[] = {{&S82313[81], 5}, {NULL}};
const Segment SGI201155[] = {{&S71700[96], 9}, {NULL}};
const Segment SGI201156[] = {{&S82313[139], 13}, {NULL}};
const Segment SGI201157[] = {{&S71700[69], 6}, {NULL}};
const Segment SGI201158[] = {{&S82321[34], 6}, {NULL}};
const Segment SGI201159[] = {{&S82321[27], 6}, {NULL}};
const Segment SGI201160[] = {{&S82321[93], 9}, {NULL}};
const Segment SGI201161[] = {{&S82321[66], 6}, {NULL}};
const Segment SGI201162[] = {{&S82327[34], 8}, {NULL}};
const Segment SGI201163[] = {{&S82327[27], 6}, {NULL}};
const Segment SGI201164[] = {{&S82327[100], 9}, {NULL}};
const Segment SGI201165[] = {{&S82327[73], 6}, {NULL}};
const Segment SGI201166[] = {{&S82330[34], 9}, {NULL}};
const Segment SGI201167[] = {{&S82330[27], 6}, {NULL}};
const Segment SGI201168[] = {{&S82330[96], 9}, {NULL}};
const Segment SGI201169[] = {{&S82330[69], 6}, {NULL}};
const Segment SGI201170[] = {{&S61902[34], 11}, {NULL}};
const Segment SGI201171[] = {{&S61902[27], 6}, {NULL}};
const Segment SGI201172[] = {{&S82331[34], 9}, {NULL}};
const Segment SGI201173[] = {{&S82331[27], 6}, {NULL}};
const Segment SGI201174[] = {{&S61902[97], 9}, {NULL}};
const Segment SGI201175[] = {{&S61902[70], 6}, {NULL}};
const Segment SGI201176[] = {{&S82331[96], 9}, {NULL}};
const Segment SGI201177[] = {{&S82331[69], 6}, {NULL}};
const Segment SGI201178[] = {{&S82342[34], 11}, {NULL}};
const Segment SGI201179[] = {{&S82342[27], 6}, {NULL}};
const Segment SGI201180[] = {{&S82342[105], 9}, {NULL}};
const Segment SGI201181[] = {{&S82342[78], 6}, {NULL}};
const Segment SGI201182[] = {{&S82350[34], 7}, {NULL}};
const Segment SGI201183[] = {{&S82350[27], 6}, {NULL}};
const Segment SGI201184[] = {{&S82350[99], 9}, {NULL}};
const Segment SGI201185[] = {{&S82350[72], 6}, {NULL}};
const Segment SGI201186[] = {{&S82351[44], 15}, {NULL}};
const Segment SGI201187[] = {{&S82351[44], 27}, {NULL}};
const Segment SGI201188[] = {{&S82351[30], 6}, {NULL}};
const Segment SGI201189[] = {{&S82351[31], 3}, {NULL}};
const Segment SGI201190[] = {{&S82351[85], 6}, {NULL}};
const Segment SGI201191[] = {{&S82352[44], 5}, {NULL}};
const Segment SGI201192[] = {{&S82352[44], 17}, {NULL}};
const Segment SGI201193[] = {{&S82352[30], 6}, {NULL}};
const Segment SGI201194[] = {{&S82352[75], 6}, {NULL}};
const Segment SGI201195[] = {{&S82352[102], 13}, {NULL}};
const Segment SGI201196[] = {{&S76351[34], 15}, {NULL}};
const Segment SGI201197[] = {{&S76351[27], 6}, {NULL}};
const Segment SGI201198[] = {{&S76351[102], 9}, {NULL}};
const Segment SGI201199[] = {{&S76351[75], 6}, {NULL}};
const Segment SGI201200[] = {{&S76360[34], 25}, {NULL}};
const Segment SGI201201[] = {{&S76360[27], 6}, {NULL}};
const Segment SGI201202[] = {{&S76360[112], 9}, {NULL}};
const Segment SGI201203[] = {{&S76360[85], 6}, {NULL}};
const Segment SGI201204[] = {{&S82381[34], 8}, {NULL}};
const Segment SGI201205[] = {{&S82381[27], 6}, {NULL}};
const Segment SGI201206[] = {{&S82381[94], 9}, {NULL}};
const Segment SGI201207[] = {{&S82381[67], 6}, {NULL}};
const Segment SGI201208[] = {{&S82386[44], 11}, {NULL}};
const Segment SGI201209[] = {{&S82386[44], 23}, {NULL}};
const Segment SGI201210[] = {{&S82386[30], 6}, {NULL}};
const Segment SGI201211[] = {{&S82386[31], 3}, {NULL}};
const Segment SGI201212[] = {{&S82386[81], 6}, {NULL}};
const Segment SGI201213[] = {{&S82390[34], 10}, {NULL}};
const Segment SGI201214[] = {{&S82390[27], 6}, {NULL}};
const Segment SGI201215[] = {{&S82390[97], 9}, {NULL}};
const Segment SGI201216[] = {{&S82390[70], 6}, {NULL}};
const Segment SGI201217[] = {{&S82393[44], 7}, {NULL}};
const Segment SGI201218[] = {{&S82393[44], 19}, {NULL}};
const Segment SGI201219[] = {{&S82393[30], 6}, {NULL}};
const Segment SGI201220[] = {{&S82393[80], 6}, {NULL}};
const Segment SGI201221[] = {{&S82394[44], 7}, {NULL}};
const Segment SGI201222[] = {{&S82394[44], 19}, {NULL}};
const Segment SGI201223[] = {{&S82394[30], 6}, {NULL}};
const Segment SGI201224[] = {{&S82394[31], 3}, {NULL}};
const Segment SGI201225[] = {{&S82394[77], 6}, {NULL}};
const Segment SGI201226[] = {{&S82395[44], 19}, {NULL}};
const Segment SGI201227[] = {{&S82395[44], 31}, {NULL}};
const Segment SGI201228[] = {{&S82395[30], 6}, {NULL}};
const Segment SGI201229[] = {{&S82395[31], 3}, {NULL}};
const Segment SGI201230[] = {{&S82395[88], 6}, {NULL}};
const Segment SGI201231[] = {{&S71801[44], 10}, {NULL}};
const Segment SGI201232[] = {{&S71801[44], 22}, {NULL}};
const Segment SGI201233[] = {{&S71801[30], 6}, {NULL}};
const Segment SGI201234[] = {{&S82396[44], 7}, {NULL}};
const Segment SGI201235[] = {{&S82396[44], 19}, {NULL}};
const Segment SGI201236[] = {{&S71801[115], 3}, {NULL}};
const Segment SGI201237[] = {{&S82396[30], 6}, {NULL}};
const Segment SGI201238[] = {{&S71801[80], 6}, {NULL}};
const Segment SGI201239[] = {{&S82396[80], 4}, {NULL}};
const Segment SGI201240[] = {{&S82396[80], 3}, {NULL}};
const Segment SGI201241[] = {{&S71802[38], 10}, {NULL}};
const Segment SGI201242[] = {{&S71802[31], 6}, {NULL}};
const Segment SGI201243[] = {{&S76396[44], 10}, {NULL}};
const Segment SGI201244[] = {{&S76396[44], 22}, {NULL}};
const Segment SGI201245[] = {{&S76396[30], 6}, {NULL}};
const Segment SGI201246[] = {{&S76396[131], 3}, {NULL}};
const Segment SGI201247[] = {{&S71803[44], 10}, {NULL}};
const Segment SGI201248[] = {{&S76396[96], 6}, {NULL}};
const Segment SGI201249[] = {{&S71803[44], 24}, {NULL}};
const Segment SGI201250[] = {{&S71803[30], 6}, {NULL}};
const Segment SGI201251[] = {{&S76396[68], 8}, {NULL}};
const Segment SGI201252[] = {{&S71803[117], 3}, {NULL}};
const Segment SGI201253[] = {{&S71803[82], 6}, {NULL}};
const Segment SGI201254[] = {{&S82400[34], 7}, {NULL}};
const Segment SGI201255[] = {{&S82400[27], 6}, {NULL}};
const Segment SGI201256[] = {{&S82400[94], 9}, {NULL}};
const Segment SGI201257[] = {{&S82400[67], 6}, {NULL}};
const Segment SGI201258[] = {{&S82401[34], 8}, {NULL}};
const Segment SGI201259[] = {{&S82401[27], 6}, {NULL}};
const Segment SGI201260[] = {{&S82401[95], 9}, {NULL}};
const Segment SGI201261[] = {{&S82401[68], 6}, {NULL}};
const Segment SGI201262[] = {{&S61991[44], 10}, {NULL}};
const Segment SGI201263[] = {{&S61991[44], 21}, {NULL}};
const Segment SGI201264[] = {{&S61991[30], 6}, {NULL}};
const Segment SGI201265[] = {{&S71816[34], 7}, {NULL}};
const Segment SGI201266[] = {{&S71816[27], 6}, {NULL}};
const Segment SGI201267[] = {{&S61991[114], 3}, {NULL}};
const Segment SGI201268[] = {{&S61991[79], 6}, {NULL}};
const Segment SGI201269[] = {{&S71816[99], 9}, {NULL}};
const Segment SGI201270[] = {{&S71816[72], 6}, {NULL}};
const Segment SGI201271[] = {{&S82409[34], 15}, {NULL}};
const Segment SGI201272[] = {{&S82409[27], 6}, {NULL}};
const Segment SGI201273[] = {{&S82409[102], 9}, {NULL}};
const Segment SGI201274[] = {{&S61992[38], 10}, {NULL}};
const Segment SGI201275[] = {{&S82409[75], 6}, {NULL}};
const Segment SGI201276[] = {{&S61992[31], 6}, {NULL}};
const Segment SGI201277[] = {{&S71817[34], 7}, {NULL}};
const Segment SGI201278[] = {{&S71817[27], 6}, {NULL}};
const Segment SGI201279[] = {{&S71817[99], 9}, {NULL}};
const Segment SGI201280[] = {{&S71817[72], 6}, {NULL}};
const Segment SGI201281[] = {{&S82410[34], 10}, {NULL}};
const Segment SGI201282[] = {{&S82410[27], 6}, {NULL}};
const Segment SGI201283[] = {{&S82410[97], 9}, {NULL}};
const Segment SGI201284[] = {{&S82410[70], 6}, {NULL}};
const Segment SGI201285[] = {{&S61994[34], 14}, {NULL}};
const Segment SGI201286[] = {{&S61994[27], 6}, {NULL}};
const Segment SGI201287[] = {{&S61994[100], 9}, {NULL}};
const Segment SGI201288[] = {{&S61994[73], 6}, {NULL}};
const Segment SGI201289[] = {{&S82411[34], 11}, {NULL}};
const Segment SGI201290[] = {{&S61994[141], 8}, {NULL}};
const Segment SGI201291[] = {{&S82411[27], 6}, {NULL}};
const Segment SGI201292[] = {{&S82411[98], 9}, {NULL}};
const Segment SGI201293[] = {{&S82411[71], 6}, {NULL}};
const Segment SGI201294[] = {{&S82412[34], 11}, {NULL}};
const Segment SGI201295[] = {{&S82412[27], 6}, {NULL}};
const Segment SGI201296[] = {{&S71822[34], 11}, {NULL}};
const Segment SGI201297[] = {{&S82412[98], 9}, {NULL}};
const Segment SGI201298[] = {{&S71822[27], 6}, {NULL}};
const Segment SGI201299[] = {{&S82412[71], 6}, {NULL}};
const Segment SGI201300[] = {{&S71822[98], 9}, {NULL}};
const Segment SGI201301[] = {{&S71822[71], 6}, {NULL}};
const Segment SGI201302[] = {{&S82415[44], 7}, {NULL}};
const Segment SGI201303[] = {{&S82415[44], 24}, {NULL}};
const Segment SGI201304[] = {{&S82415[30], 6}, {NULL}};
const Segment SGI201305[] = {{&S82415[82], 6}, {NULL}};
const Segment SGI201306[] = {{&S82415[109], 13}, {NULL}};
const Segment SGI201307[] = {{&S62027[34], 18}, {NULL}};
const Segment SGI201308[] = {{&S62027[27], 6}, {NULL}};
const Segment SGI201309[] = {{&S62027[105], 9}, {NULL}};
const Segment SGI201310[] = {{&S62027[78], 6}, {NULL}};
const Segment SGI201311[] = {{&S62028[34], 10}, {NULL}};
const Segment SGI201312[] = {{&S62028[27], 6}, {NULL}};
const Segment SGI201313[] = {{&S62028[96], 9}, {NULL}};
const Segment SGI201314[] = {{&S62028[69], 6}, {NULL}};
const Segment SGI201315[] = {{&S71866[34], 10}, {NULL}};
const Segment SGI201316[] = {{&S71866[27], 6}, {NULL}};
const Segment SGI201317[] = {{&S71866[97], 9}, {NULL}};
const Segment SGI201318[] = {{&S71866[70], 6}, {NULL}};
const Segment SGI201319[] = {{&S71871[44], 10}, {NULL}};
const Segment SGI201320[] = {{&S71871[44], 21}, {NULL}};
const Segment SGI201321[] = {{&S71871[30], 6}, {NULL}};
const Segment SGI201322[] = {{&S71871[114], 3}, {NULL}};
const Segment SGI201323[] = {{&S71871[79], 6}, {NULL}};
const Segment SGI201324[] = {{&S82460[34], 10}, {NULL}};
const Segment SGI201325[] = {{&S82460[27], 6}, {NULL}};
const Segment SGI201326[] = {{&S82460[114], 9}, {NULL}};
const Segment SGI201327[] = {{&S82460[87], 6}, {NULL}};
const Segment SGI201328[] = {{&S71876[44], 13}, {NULL}};
const Segment SGI201329[] = {{&S71876[44], 25}, {NULL}};
const Segment SGI201330[] = {{&S71876[30], 6}, {NULL}};
const Segment SGI201331[] = {{&S71876[118], 3}, {NULL}};
const Segment SGI201332[] = {{&S71876[83], 6}, {NULL}};
const Segment SGI201333[] = {{&S82463[31], 8}, {NULL}};
const Segment SGI201334[] = {{&S82463[54], 3}, {NULL}};
const Segment SGI201335[] = {{&S71877[44], 16}, {NULL}};
const Segment SGI201336[] = {{&S71877[44], 28}, {NULL}};
const Segment SGI201337[] = {{&S71877[30], 6}, {NULL}};
const Segment SGI201338[] = {{&S71877[121], 3}, {NULL}};
const Segment SGI201339[] = {{&S71877[86], 6}, {NULL}};
const Segment SGI201340[] = {{&S82469[34], 9}, {NULL}};
const Segment SGI201341[] = {{&S82469[27], 6}, {NULL}};
const Segment SGI201342[] = {{&S82469[96], 9}, {NULL}};
const Segment SGI201343[] = {{&S82469[69], 6}, {NULL}};
const Segment SGI201344[] = {{&S82470[34], 7}, {NULL}};
const Segment SGI201345[] = {{&S82470[27], 6}, {NULL}};
const Segment SGI201346[] = {{&S82470[95], 9}, {NULL}};
const Segment SGI201347[] = {{&S82470[68], 6}, {NULL}};
const Segment SGI201348[] = {{&S62060[34], 18}, {NULL}};
const Segment SGI201349[] = {{&S62060[27], 6}, {NULL}};
const Segment SGI201350[] = {{&S82473[45], 15}, {NULL}};
const Segment SGI201351[] = {{&S62060[105], 9}, {NULL}};
const Segment SGI201352[] = {{&S62060[78], 6}, {NULL}};
const Segment SGI201353[] = {{&S82473[30], 6}, {NULL}};
const Segment SGI201354[] = {{&S82473[31], 3}, {NULL}};
const Segment SGI201355[] = {{&S82473[86], 5}, {NULL}};
const Segment SGI201356[] = {{&S82473[144], 13}, {NULL}};
const Segment SGI201357[] = {{&S62061[34], 10}, {NULL}};
const Segment SGI201358[] = {{&S62061[27], 6}, {NULL}};
const Segment SGI201359[] = {{&S62061[96], 9}, {NULL}};
const Segment SGI201360[] = {{&S62061[69], 6}, {NULL}};
const Segment SGI201361[] = {{&S71120[44], 8}, {NULL}};
const Segment SGI201362[] = {{&S71120[44], 20}, {NULL}};
const Segment SGI201363[] = {{&S71120[30], 6}, {NULL}};
const Segment SGI201364[] = {{&S71120[113], 3}, {NULL}};
const Segment SGI201365[] = {{&S71120[78], 6}, {NULL}};
const Segment SGI201366[] = {{&S71121[38], 8}, {NULL}};
const Segment SGI201367[] = {{&S71121[31], 6}, {NULL}};
const Segment SGI201368[] = {{&S82484[44], 15}, {NULL}};
const Segment SGI201369[] = {{&S82484[44], 27}, {NULL}};
const Segment SGI201370[] = {{&S82484[30], 6}, {NULL}};
const Segment SGI201371[] = {{&S82484[31], 3}, {NULL}};
const Segment SGI201372[] = {{&S82484[85], 6}, {NULL}};
const Segment SGI201373[] = {{&S82488[34], 7}, {NULL}};
const Segment SGI201374[] = {{&S82488[27], 6}, {NULL}};
const Segment SGI201375[] = {{&S76502[34], 11}, {NULL}};
const Segment SGI201376[] = {{&S76502[27], 6}, {NULL}};
const Segment SGI201377[] = {{&S82488[99], 9}, {NULL}};
const Segment SGI201378[] = {{&S82488[72], 6}, {NULL}};
const Segment SGI201379[] = {{&S76502[98], 9}, {NULL}};
const Segment SGI201380[] = {{&S76502[71], 6}, {NULL}};
const Segment SGI201381[] = {{&S82490[44], 10}, {NULL}};
const Segment SGI201382[] = {{&S82490[44], 22}, {NULL}};
const Segment SGI201383[] = {{&S82490[30], 6}, {NULL}};
const Segment SGI201384[] = {{&S82490[31], 3}, {NULL}};
const Segment SGI201385[] = {{&S82490[80], 5}, {NULL}};
const Segment SGI201386[] = {{&S82490[117], 14}, {NULL}};
const Segment SGI201387[] = {{&S82491[44], 11}, {NULL}};
const Segment SGI201388[] = {{&S82491[44], 23}, {NULL}};
const Segment SGI201389[] = {{&S82491[30], 6}, {NULL}};
const Segment SGI201390[] = {{&S82491[31], 3}, {NULL}};
const Segment SGI201391[] = {{&S82491[81], 5}, {NULL}};
const Segment SGI201392[] = {{&S82491[118], 14}, {NULL}};
const Segment SGI201393[] = {{&S76508[45], 10}, {NULL}};
const Segment SGI201394[] = {{&S76508[30], 6}, {NULL}};
const Segment SGI201395[] = {{&S76508[115], 3}, {NULL}};
const Segment SGI201396[] = {{&S76508[81], 5}, {NULL}};
const Segment SGI201397[] = {{&S82492[44], 10}, {NULL}};
const Segment SGI201398[] = {{&S76508[139], 13}, {NULL}};
const Segment SGI201399[] = {{&S82492[44], 22}, {NULL}};
const Segment SGI201400[] = {{&S82492[30], 6}, {NULL}};
const Segment SGI201401[] = {{&S82492[80], 6}, {NULL}};
const Segment SGI201402[] = {{&S82492[107], 13}, {NULL}};
const Segment SGI201403[] = {{&S76509[45], 18}, {NULL}};
const Segment SGI201404[] = {{&S76509[30], 6}, {NULL}};
const Segment SGI201405[] = {{&S76509[123], 3}, {NULL}};
const Segment SGI201406[] = {{&S76509[89], 5}, {NULL}};
const Segment SGI201407[] = {{&S76509[147], 13}, {NULL}};
const Segment SGI201408[] = {{&S76511[45], 11}, {NULL}};
const Segment SGI201409[] = {{&S76511[30], 6}, {NULL}};
const Segment SGI201410[] = {{&S76511[116], 3}, {NULL}};
const Segment SGI201411[] = {{&S76511[82], 5}, {NULL}};
const Segment SGI201412[] = {{&S82496[44], 22}, {NULL}};
const Segment SGI201413[] = {{&S82496[30], 6}, {NULL}};
const Segment SGI201414[] = {{&S76511[140], 13}, {NULL}};
const Segment SGI201415[] = {{&S82496[31], 3}, {NULL}};
const Segment SGI201416[] = {{&S82496[82], 6}, {NULL}};
const Segment SGI201417[] = {{&S82497[44], 10}, {NULL}};
const Segment SGI201418[] = {{&S82497[30], 6}, {NULL}};
const Segment SGI201419[] = {{&S82497[31], 3}, {NULL}};
const Segment SGI201420[] = {{&S82497[68], 6}, {NULL}};
const Segment SGI201421[] = {{&S62095[44], 15}, {NULL}};
const Segment SGI201422[] = {{&S62095[44], 27}, {NULL}};
const Segment SGI201423[] = {{&S62095[30], 6}, {NULL}};
const Segment SGI201424[] = {{&S62095[120], 3}, {NULL}};
const Segment SGI201425[] = {{&S62095[85], 6}, {NULL}};
const Segment SGI201426[] = {{&S82503[34], 10}, {NULL}};
const Segment SGI201427[] = {{&S82503[27], 6}, {NULL}};
const Segment SGI201428[] = {{&S82503[97], 9}, {NULL}};
const Segment SGI201429[] = {{&S82503[70], 6}, {NULL}};
const Segment SGI201430[] = {{&S82504[34], 11}, {NULL}};
const Segment SGI201431[] = {{&S82504[27], 6}, {NULL}};
const Segment SGI201432[] = {{&S82504[98], 9}, {NULL}};
const Segment SGI201433[] = {{&S82504[71], 6}, {NULL}};
const Segment SGI201434[] = {{&S62108[44], 21}, {NULL}};
const Segment SGI201435[] = {{&S62108[44], 33}, {NULL}};
const Segment SGI201436[] = {{&S62108[30], 6}, {NULL}};
const Segment SGI201437[] = {{&S62108[177], 3}, {NULL}};
const Segment SGI201438[] = {{&S62108[142], 6}, {NULL}};
const Segment SGI201439[] = {{&S62108[79], 8}, {NULL}};
const Segment SGI201440[] = {{&S62109[44], 22}, {NULL}};
const Segment SGI201441[] = {{&S62109[44], 34}, {NULL}};
const Segment SGI201442[] = {{&S62109[30], 6}, {NULL}};
const Segment SGI201443[] = {{&S62109[154], 3}, {NULL}};
const Segment SGI201444[] = {{&S62109[119], 6}, {NULL}};
const Segment SGI201445[] = {{&S62109[80], 8}, {NULL}};
const Segment SGI201446[] = {{&S62110[44], 21}, {NULL}};
const Segment SGI201447[] = {{&S62110[44], 33}, {NULL}};
const Segment SGI201448[] = {{&S62110[30], 6}, {NULL}};
const Segment SGI201449[] = {{&S71900[34], 10}, {NULL}};
const Segment SGI201450[] = {{&S71900[27], 6}, {NULL}};
const Segment SGI201451[] = {{&S62110[163], 3}, {NULL}};
const Segment SGI201452[] = {{&S62110[128], 6}, {NULL}};
const Segment SGI201453[] = {{&S62110[79], 8}, {NULL}};
const Segment SGI201454[] = {{&S71900[97], 9}, {NULL}};
const Segment SGI201455[] = {{&S71900[70], 6}, {NULL}};
const Segment SGI201456[] = {{&S62111[55], 17}, {NULL}};
const Segment SGI201457[] = {{&S62111[55], 29}, {NULL}};
const Segment SGI201458[] = {{&S71901[34], 11}, {NULL}};
const Segment SGI201459[] = {{&S62111[30], 17}, {NULL}};
const Segment SGI201460[] = {{&S71901[27], 6}, {NULL}};
const Segment SGI201461[] = {{&S62111[167], 3}, {NULL}};
const Segment SGI201462[] = {{&S71901[98], 9}, {NULL}};
const Segment SGI201463[] = {{&S62111[132], 6}, {NULL}};
const Segment SGI201464[] = {{&S71901[71], 6}, {NULL}};
const Segment SGI201465[] = {{&S62111[86], 8}, {NULL}};
const Segment SGI201466[] = {{&S62112[44], 18}, {NULL}};
const Segment SGI201467[] = {{&S62112[44], 30}, {NULL}};
const Segment SGI201468[] = {{&S62112[30], 6}, {NULL}};
const Segment SGI201469[] = {{&S62112[123], 3}, {NULL}};
const Segment SGI201470[] = {{&S62112[88], 6}, {NULL}};
const Segment SGI201471[] = {{&S62112[148], 8}, {NULL}};
const Segment SGI201472[] = {{&S71906[34], 13}, {NULL}};
const Segment SGI201473[] = {{&S71906[27], 6}, {NULL}};
const Segment SGI201474[] = {{&S71906[100], 9}, {NULL}};
const Segment SGI201475[] = {{&S71906[73], 6}, {NULL}};
const Segment SGI201476[] = {{&S71907[34], 11}, {NULL}};
const Segment SGI201477[] = {{&S71907[27], 6}, {NULL}};
const Segment SGI201478[] = {{&S71907[98], 9}, {NULL}};
const Segment SGI201479[] = {{&S71907[71], 6}, {NULL}};
const Segment SGI201480[] = {{&S71908[34], 7}, {NULL}};
const Segment SGI201481[] = {{&S71908[27], 6}, {NULL}};
const Segment SGI201482[] = {{&S71908[98], 9}, {NULL}};
const Segment SGI201483[] = {{&S71908[71], 6}, {NULL}};
const Segment SGI201484[] = {{&S82539[44], 15}, {NULL}};
const Segment SGI201485[] = {{&S82539[44], 27}, {NULL}};
const Segment SGI201486[] = {{&S82539[30], 6}, {NULL}};
const Segment SGI201487[] = {{&S82539[31], 3}, {NULL}};
const Segment SGI201488[] = {{&S82539[100], 6}, {NULL}};
const Segment SGI201489[] = {{&S82539[73], 8}, {NULL}};
const Segment SGI201490[] = {{&S82540[44], 15}, {NULL}};
const Segment SGI201491[] = {{&S82540[44], 27}, {NULL}};
const Segment SGI201492[] = {{&S82540[30], 6}, {NULL}};
const Segment SGI201493[] = {{&S82540[31], 3}, {NULL}};
const Segment SGI201494[] = {{&S82540[104], 6}, {NULL}};
const Segment SGI201495[] = {{&S82540[73], 8}, {NULL}};
const Segment SGI201496[] = {{&S76564[34], 10}, {NULL}};
const Segment SGI201497[] = {{&S76564[27], 6}, {NULL}};
const Segment SGI201498[] = {{&S76564[97], 9}, {NULL}};
const Segment SGI201499[] = {{&S76564[70], 6}, {NULL}};
const Segment SGI201500[] = {{&S82545[44], 13}, {NULL}};
const Segment SGI201501[] = {{&S82545[44], 25}, {NULL}};
const Segment SGI201502[] = {{&S82545[30], 6}, {NULL}};
const Segment SGI201503[] = {{&S82545[31], 3}, {NULL}};
const Segment SGI201504[] = {{&S82545[83], 6}, {NULL}};
const Segment SGI201505[] = {{&S82564[44], 7}, {NULL}};
const Segment SGI201506[] = {{&S82564[44], 15}, {NULL}};
const Segment SGI201507[] = {{&S82564[30], 6}, {NULL}};
const Segment SGI201508[] = {{&S82564[31], 3}, {NULL}};
const Segment SGI201509[] = {{&S82564[73], 6}, {NULL}};
const Segment SGI201510[] = {{&S82565[44], 11}, {NULL}};
const Segment SGI201511[] = {{&S82565[44], 23}, {NULL}};
const Segment SGI201512[] = {{&S82565[30], 6}, {NULL}};
const Segment SGI201513[] = {{&S82565[31], 3}, {NULL}};
const Segment SGI201514[] = {{&S82565[81], 6}, {NULL}};
const Segment SGI201515[] = {{&S82570[34], 10}, {NULL}};
const Segment SGI201516[] = {{&S82570[27], 6}, {NULL}};
const Segment SGI201517[] = {{&S82570[97], 9}, {NULL}};
const Segment SGI201518[] = {{&S82570[70], 6}, {NULL}};
const Segment SGI201519[] = {{&S82572[44], 25}, {NULL}};
const Segment SGI201520[] = {{&S82572[44], 37}, {NULL}};
const Segment SGI201521[] = {{&S82572[30], 6}, {NULL}};
const Segment SGI201522[] = {{&S82572[31], 3}, {NULL}};
const Segment SGI201523[] = {{&S82572[95], 6}, {NULL}};
const Segment SGI201524[] = {{&S82573[34], 11}, {NULL}};
const Segment SGI201525[] = {{&S82573[27], 6}, {NULL}};
const Segment SGI201526[] = {{&S82573[98], 9}, {NULL}};
const Segment SGI201527[] = {{&S82573[71], 6}, {NULL}};
const Segment SGI201528[] = {{&S82574[34], 10}, {NULL}};
const Segment SGI201529[] = {{&S82574[27], 6}, {NULL}};
const Segment SGI201530[] = {{&S82574[97], 9}, {NULL}};
const Segment SGI201531[] = {{&S82574[70], 6}, {NULL}};
const Segment SGI201532[] = {{&S82576[34], 8}, {NULL}};
const Segment SGI201533[] = {{&S82576[27], 6}, {NULL}};
const Segment SGI201534[] = {{&S82576[95], 9}, {NULL}};
const Segment SGI201535[] = {{&S82576[68], 6}, {NULL}};
const Segment SGI201536[] = {{&S82578[34], 15}, {NULL}};
const Segment SGI201537[] = {{&S82578[27], 6}, {NULL}};
const Segment SGI201538[] = {{&S82578[117], 9}, {NULL}};
const Segment SGI201539[] = {{&S82578[90], 6}, {NULL}};
const Segment SGI201540[] = {{&S82581[44], 11}, {NULL}};
const Segment SGI201541[] = {{&S82581[44], 36}, {NULL}};
const Segment SGI201542[] = {{&S82581[30], 6}, {NULL}};
const Segment SGI201543[] = {{&S82581[31], 3}, {NULL}};
const Segment SGI201544[] = {{&S82581[94], 6}, {NULL}};
const Segment SGI201545[] = {{&S71953[34], 10}, {NULL}};
const Segment SGI201546[] = {{&S71953[27], 6}, {NULL}};
const Segment SGI201547[] = {{&S71953[97], 9}, {NULL}};
const Segment SGI201548[] = {{&S71953[70], 6}, {NULL}};
const Segment SGI201549[] = {{&S82586[34], 9}, {NULL}};
const Segment SGI201550[] = {{&S82586[27], 6}, {NULL}};
const Segment SGI201551[] = {{&S82586[96], 9}, {NULL}};
const Segment SGI201552[] = {{&S82586[69], 6}, {NULL}};
const Segment SGI201553[] = {{&S76630[34], 12}, {NULL}};
const Segment SGI201554[] = {{&S76630[27], 6}, {NULL}};
const Segment SGI201555[] = {{&S82594[34], 24}, {NULL}};
const Segment SGI201556[] = {{&S82594[27], 6}, {NULL}};
const Segment SGI201557[] = {{&S76630[99], 9}, {NULL}};
const Segment SGI201558[] = {{&S76630[72], 6}, {NULL}};
const Segment SGI201559[] = {{&S82594[113], 9}, {NULL}};
const Segment SGI201560[] = {{&S82594[86], 6}, {NULL}};
const Segment SGI201561[] = {{&S71967[34], 9}, {NULL}};
const Segment SGI201562[] = {{&S71967[27], 6}, {NULL}};
const Segment SGI201563[] = {{&S71967[96], 9}, {NULL}};
const Segment SGI201564[] = {{&S71967[69], 6}, {NULL}};
const Segment SGI201565[] = {{&S76631[34], 12}, {NULL}};
const Segment SGI201566[] = {{&S76631[27], 6}, {NULL}};
const Segment SGI201567[] = {{&S82595[34], 23}, {NULL}};
const Segment SGI201568[] = {{&S82595[27], 6}, {NULL}};
const Segment SGI201569[] = {{&S76631[99], 9}, {NULL}};
const Segment SGI201570[] = {{&S76631[72], 6}, {NULL}};
const Segment SGI201571[] = {{&S82595[112], 9}, {NULL}};
const Segment SGI201572[] = {{&S82595[85], 6}, {NULL}};
const Segment SGI201573[] = {{&S71968[34], 10}, {NULL}};
const Segment SGI201574[] = {{&S71968[27], 6}, {NULL}};
const Segment SGI201575[] = {{&S71968[97], 9}, {NULL}};
const Segment SGI201576[] = {{&S71968[70], 6}, {NULL}};
const Segment SGI201577[] = {{&S71284[44], 12}, {NULL}};
const Segment SGI201578[] = {{&S71284[44], 24}, {NULL}};
const Segment SGI201579[] = {{&S71284[30], 6}, {NULL}};
const Segment SGI201580[] = {{&S76632[34], 15}, {NULL}};
const Segment SGI201581[] = {{&S76632[27], 6}, {NULL}};
const Segment SGI201582[] = {{&S71284[135], 3}, {NULL}};
const Segment SGI201583[] = {{&S71284[100], 6}, {NULL}};
const Segment SGI201584[] = {{&S71284[70], 8}, {NULL}};
const Segment SGI201585[] = {{&S76632[102], 9}, {NULL}};
const Segment SGI201586[] = {{&S76632[75], 6}, {NULL}};
const Segment SGI201587[] = {{&S71970[34], 17}, {NULL}};
const Segment SGI201588[] = {{&S71970[27], 6}, {NULL}};
const Segment SGI201589[] = {{&S71970[104], 9}, {NULL}};
const Segment SGI201590[] = {{&S71970[77], 6}, {NULL}};
const Segment SGI201591[] = {{&S71290[34], 10}, {NULL}};
const Segment SGI201592[] = {{&S71290[27], 6}, {NULL}};
const Segment SGI201593[] = {{&S71290[97], 9}, {NULL}};
const Segment SGI201594[] = {{&S71290[70], 6}, {NULL}};
const Segment SGI201595[] = {{&S71982[44], 10}, {NULL}};
const Segment SGI201596[] = {{&S71982[44], 22}, {NULL}};
const Segment SGI201597[] = {{&S71982[30], 6}, {NULL}};
const Segment SGI201598[] = {{&S71982[115], 3}, {NULL}};
const Segment SGI201599[] = {{&S71982[80], 6}, {NULL}};
const Segment SGI201600[] = {{&S82614[44], 11}, {NULL}};
const Segment SGI201601[] = {{&S82614[44], 23}, {NULL}};
const Segment SGI201602[] = {{&S82614[30], 6}, {NULL}};
const Segment SGI201603[] = {{&S82614[31], 3}, {NULL}};
const Segment SGI201604[] = {{&S82614[81], 6}, {NULL}};
const Segment SGI201605[] = {{&S71983[44], 10}, {NULL}};
const Segment SGI201606[] = {{&S71983[44], 33}, {NULL}};
const Segment SGI201607[] = {{&S71983[30], 6}, {NULL}};
const Segment SGI201608[] = {{&S71983[126], 3}, {NULL}};
const Segment SGI201609[] = {{&S71983[91], 6}, {NULL}};
const Segment SGI201610[] = {{&S71984[44], 18}, {NULL}};
const Segment SGI201611[] = {{&S71984[44], 30}, {NULL}};
const Segment SGI201612[] = {{&S71984[30], 6}, {NULL}};
const Segment SGI201613[] = {{&S71984[123], 3}, {NULL}};
const Segment SGI201614[] = {{&S71984[88], 6}, {NULL}};
const Segment SGI201615[] = {{&S71985[38], 18}, {NULL}};
const Segment SGI201616[] = {{&S71985[31], 6}, {NULL}};
const Segment SGI201617[] = {{&S82619[59], 10}, {NULL}};
const Segment SGI201618[] = {{&S82619[59], 22}, {NULL}};
const Segment SGI201619[] = {{&S82619[45], 6}, {NULL}};
const Segment SGI201620[] = {{&S82619[46], 3}, {NULL}};
const Segment SGI201621[] = {{&S82619[95], 6}, {NULL}};
const Segment SGI201622[] = {{&S82619[0], 14}, {NULL}};
const Segment SGI201623[] = {{&S82621[44], 11}, {NULL}};
const Segment SGI201624[] = {{&S82621[44], 23}, {NULL}};
const Segment SGI201625[] = {{&S82621[30], 6}, {NULL}};
const Segment SGI201626[] = {{&S82621[31], 3}, {NULL}};
const Segment SGI201627[] = {{&S82621[81], 6}, {NULL}};
const Segment SGI201628[] = {{&S82622[44], 16}, {NULL}};
const Segment SGI201629[] = {{&S82622[44], 28}, {NULL}};
const Segment SGI201630[] = {{&S82622[30], 6}, {NULL}};
const Segment SGI201631[] = {{&S82622[31], 3}, {NULL}};
const Segment SGI201632[] = {{&S82622[86], 6}, {NULL}};
const Segment SGI201633[] = {{&S82625[44], 12}, {NULL}};
const Segment SGI201634[] = {{&S82625[44], 40}, {NULL}};
const Segment SGI201635[] = {{&S82625[30], 6}, {NULL}};
const Segment SGI201636[] = {{&S82625[31], 3}, {NULL}};
const Segment SGI201637[] = {{&S82625[98], 6}, {NULL}};
const Segment SGI201638[] = {{&S82625[158], 11}, {NULL}};
const Segment SGI201639[] = {{&S76671[34], 7}, {NULL}};
const Segment SGI201640[] = {{&S76671[27], 6}, {NULL}};
const Segment SGI201641[] = {{&S62255[34], 6}, {NULL}};
const Segment SGI201642[] = {{&S62255[27], 6}, {NULL}};
const Segment SGI201643[] = {{&S76671[99], 9}, {NULL}};
const Segment SGI201644[] = {{&S76671[72], 6}, {NULL}};
const Segment SGI201645[] = {{&S62255[92], 9}, {NULL}};
const Segment SGI201646[] = {{&S62255[65], 6}, {NULL}};
const Segment SGI201647[] = {{&S82634[34], 10}, {NULL}};
const Segment SGI201648[] = {{&S82634[27], 6}, {NULL}};
const Segment SGI201649[] = {{&S82634[97], 9}, {NULL}};
const Segment SGI201650[] = {{&S82634[70], 6}, {NULL}};
const Segment SGI201651[] = {{&S72007[45], 10}, {NULL}};
const Segment SGI201652[] = {{&S72007[30], 6}, {NULL}};
const Segment SGI201653[] = {{&S72007[115], 3}, {NULL}};
const Segment SGI201654[] = {{&S72007[81], 5}, {NULL}};
const Segment SGI201655[] = {{&S72007[139], 13}, {NULL}};
const Segment SGI201656[] = {{&S82635[44], 16}, {NULL}};
const Segment SGI201657[] = {{&S82635[44], 28}, {NULL}};
const Segment SGI201658[] = {{&S82635[30], 6}, {NULL}};
const Segment SGI201659[] = {{&S71328[34], 10}, {NULL}};
const Segment SGI201660[] = {{&S71328[27], 6}, {NULL}};
const Segment SGI201661[] = {{&S82635[31], 3}, {NULL}};
const Segment SGI201662[] = {{&S82635[86], 6}, {NULL}};
const Segment SGI201663[] = {{&S71328[97], 9}, {NULL}};
const Segment SGI201664[] = {{&S71328[70], 6}, {NULL}};
const Segment SGI201665[] = {{&S72008[45], 11}, {NULL}};
const Segment SGI201666[] = {{&S72008[30], 6}, {NULL}};
const Segment SGI201667[] = {{&S72008[116], 3}, {NULL}};
const Segment SGI201668[] = {{&S72008[82], 5}, {NULL}};
const Segment SGI201669[] = {{&S82636[44], 5}, {NULL}};
const Segment SGI201670[] = {{&S82636[44], 17}, {NULL}};
const Segment SGI201671[] = {{&S72008[140], 13}, {NULL}};
const Segment SGI201672[] = {{&S82636[30], 6}, {NULL}};
const Segment SGI201673[] = {{&S82636[31], 3}, {NULL}};
const Segment SGI201674[] = {{&S82636[75], 6}, {NULL}};
const Segment SGI201675[] = {{&S72009[44], 10}, {NULL}};
const Segment SGI201676[] = {{&S72009[44], 22}, {NULL}};
const Segment SGI201677[] = {{&S72009[30], 6}, {NULL}};
const Segment SGI201678[] = {{&S72009[115], 3}, {NULL}};
const Segment SGI201679[] = {{&S72009[80], 6}, {NULL}};
const Segment SGI201680[] = {{&S72009[135], 8}, {NULL}};
const Segment SGI201681[] = {{&S72009[144], 13}, {NULL}};
const Segment SGI201682[] = {{&S62263[34], 10}, {NULL}};
const Segment SGI201683[] = {{&S62263[27], 6}, {NULL}};
const Segment SGI201684[] = {{&S62263[96], 9}, {NULL}};
const Segment SGI201685[] = {{&S62263[69], 6}, {NULL}};
const Segment SGI201686[] = {{&S82642[34], 11}, {NULL}};
const Segment SGI201687[] = {{&S82642[27], 6}, {NULL}};
const Segment SGI201688[] = {{&S82642[108], 9}, {NULL}};
const Segment SGI201689[] = {{&S82642[81], 6}, {NULL}};
const Segment SGI201690[] = {{&S82649[44], 15}, {NULL}};
const Segment SGI201691[] = {{&S82649[44], 33}, {NULL}};
const Segment SGI201692[] = {{&S82649[30], 6}, {NULL}};
const Segment SGI201693[] = {{&S82649[91], 6}, {NULL}};
const Segment SGI201694[] = {{&S82649[118], 13}, {NULL}};
const Segment SGI201695[] = {{&S82654[34], 4}, {NULL}};
const Segment SGI201696[] = {{&S82654[27], 6}, {NULL}};
const Segment SGI201697[] = {{&S82654[91], 9}, {NULL}};
const Segment SGI201698[] = {{&S82654[64], 6}, {NULL}};
const Segment SGI201699[] = {{&S62284[44], 7}, {NULL}};
const Segment SGI201700[] = {{&S62284[44], 19}, {NULL}};
const Segment SGI201701[] = {{&S62284[30], 6}, {NULL}};
const Segment SGI201702[] = {{&S62284[112], 3}, {NULL}};
const Segment SGI201703[] = {{&S76705[44], 10}, {NULL}};
const Segment SGI201704[] = {{&S76705[44], 22}, {NULL}};
const Segment SGI201705[] = {{&S62284[77], 6}, {NULL}};
const Segment SGI201706[] = {{&S82656[34], 9}, {NULL}};
const Segment SGI201707[] = {{&S76705[30], 6}, {NULL}};
const Segment SGI201708[] = {{&S82656[27], 6}, {NULL}};
const Segment SGI201709[] = {{&S62284[137], 8}, {NULL}};
const Segment SGI201710[] = {{&S76705[131], 3}, {NULL}};
const Segment SGI201711[] = {{&S76705[96], 6}, {NULL}};
const Segment SGI201712[] = {{&S82656[96], 9}, {NULL}};
const Segment SGI201713[] = {{&S82656[69], 6}, {NULL}};
const Segment SGI201714[] = {{&S76705[68], 8}, {NULL}};
const Segment SGI201715[] = {{&S76707[44], 10}, {NULL}};
const Segment SGI201716[] = {{&S76707[44], 22}, {NULL}};
const Segment SGI201717[] = {{&S76707[30], 6}, {NULL}};
const Segment SGI201718[] = {{&S76707[131], 3}, {NULL}};
const Segment SGI201719[] = {{&S76707[96], 6}, {NULL}};
const Segment SGI201720[] = {{&S76707[68], 8}, {NULL}};
const Segment SGI201721[] = {{&S76708[44], 10}, {NULL}};
const Segment SGI201722[] = {{&S76708[44], 22}, {NULL}};
const Segment SGI201723[] = {{&S76708[30], 6}, {NULL}};
const Segment SGI201724[] = {{&S76708[131], 3}, {NULL}};
const Segment SGI201725[] = {{&S76708[96], 6}, {NULL}};
const Segment SGI201726[] = {{&S76708[68], 8}, {NULL}};
const Segment SGI201727[] = {{&S76709[44], 10}, {NULL}};
const Segment SGI201728[] = {{&S76709[44], 22}, {NULL}};
const Segment SGI201729[] = {{&S76709[30], 6}, {NULL}};
const Segment SGI201730[] = {{&S76709[131], 3}, {NULL}};
const Segment SGI201731[] = {{&S76709[96], 6}, {NULL}};
const Segment SGI201732[] = {{&S76709[68], 8}, {NULL}};
const Segment SGI201733[] = {{&S72043[44], 10}, {NULL}};
const Segment SGI201734[] = {{&S72043[44], 22}, {NULL}};
const Segment SGI201735[] = {{&S72043[30], 6}, {NULL}};
const Segment SGI201736[] = {{&S72043[80], 6}, {NULL}};
const Segment SGI201737[] = {{&S72043[107], 13}, {NULL}};
const Segment SGI201738[] = {{&S71375[34], 10}, {NULL}};
const Segment SGI201739[] = {{&S71375[27], 6}, {NULL}};
const Segment SGI201740[] = {{&S71375[97], 9}, {NULL}};
const Segment SGI201741[] = {{&S71375[70], 6}, {NULL}};
const Segment SGI201742[] = {{&S71378[34], 13}, {NULL}};
const Segment SGI201743[] = {{&S71378[27], 6}, {NULL}};
const Segment SGI201744[] = {{&S71378[100], 9}, {NULL}};
const Segment SGI201745[] = {{&S71378[73], 6}, {NULL}};
const Segment SGI201746[] = {{&S71379[34], 17}, {NULL}};
const Segment SGI201747[] = {{&S71379[27], 6}, {NULL}};
const Segment SGI201748[] = {{&S71379[104], 9}, {NULL}};
const Segment SGI201749[] = {{&S71379[77], 6}, {NULL}};
const Segment SGI201750[] = {{&S82693[34], 6}, {NULL}};
const Segment SGI201751[] = {{&S82693[27], 6}, {NULL}};
const Segment SGI201752[] = {{&S82693[93], 9}, {NULL}};
const Segment SGI201753[] = {{&S82693[66], 6}, {NULL}};
const Segment SGI201754[] = {{&S76741[45], 10}, {NULL}};
const Segment SGI201755[] = {{&S82694[44], 10}, {NULL}};
const Segment SGI201756[] = {{&S76741[30], 6}, {NULL}};
const Segment SGI201757[] = {{&S82694[44], 22}, {NULL}};
const Segment SGI201758[] = {{&S82694[30], 6}, {NULL}};
const Segment SGI201759[] = {{&S72075[34], 10}, {NULL}};
const Segment SGI201760[] = {{&S72075[27], 6}, {NULL}};
const Segment SGI201761[] = {{&S82694[31], 3}, {NULL}};
const Segment SGI201762[] = {{&S76741[115], 3}, {NULL}};
const Segment SGI201763[] = {{&S82694[80], 6}, {NULL}};
const Segment SGI201764[] = {{&S76741[81], 5}, {NULL}};
const Segment SGI201765[] = {{&S72075[97], 9}, {NULL}};
const Segment SGI201766[] = {{&S76741[139], 13}, {NULL}};
const Segment SGI201767[] = {{&S72075[70], 6}, {NULL}};
const Segment SGI201768[] = {{&S82695[44], 10}, {NULL}};
const Segment SGI201769[] = {{&S72076[34], 10}, {NULL}};
const Segment SGI201770[] = {{&S82695[44], 22}, {NULL}};
const Segment SGI201771[] = {{&S82695[30], 6}, {NULL}};
const Segment SGI201772[] = {{&S72076[27], 6}, {NULL}};
const Segment SGI201773[] = {{&S82695[31], 3}, {NULL}};
const Segment SGI201774[] = {{&S72076[97], 9}, {NULL}};
const Segment SGI201775[] = {{&S82695[80], 6}, {NULL}};
const Segment SGI201776[] = {{&S72076[70], 6}, {NULL}};
const Segment SGI201777[] = {{&S72077[34], 10}, {NULL}};
const Segment SGI201778[] = {{&S72077[27], 6}, {NULL}};
const Segment SGI201779[] = {{&S72077[97], 9}, {NULL}};
const Segment SGI201780[] = {{&S72077[70], 6}, {NULL}};
const Segment SGI201781[] = {{&S82697[44], 18}, {NULL}};
const Segment SGI201782[] = {{&S82697[44], 30}, {NULL}};
const Segment SGI201783[] = {{&S82697[30], 6}, {NULL}};
const Segment SGI201784[] = {{&S82697[31], 3}, {NULL}};
const Segment SGI201785[] = {{&S82697[88], 6}, {NULL}};
const Segment SGI201786[] = {{&S72080[34], 11}, {NULL}};
const Segment SGI201787[] = {{&S72080[27], 6}, {NULL}};
const Segment SGI201788[] = {{&S72080[98], 9}, {NULL}};
const Segment SGI201789[] = {{&S72080[71], 6}, {NULL}};
const Segment SGI201790[] = {{&S82700[45], 7}, {NULL}};
const Segment SGI201791[] = {{&S82700[30], 6}, {NULL}};
const Segment SGI201792[] = {{&S82700[31], 3}, {NULL}};
const Segment SGI201793[] = {{&S82700[83], 5}, {NULL}};
const Segment SGI201794[] = {{&S82700[141], 13}, {NULL}};
const Segment SGI201795[] = {{&S82701[44], 7}, {NULL}};
const Segment SGI201796[] = {{&S82701[44], 24}, {NULL}};
const Segment SGI201797[] = {{&S82701[30], 6}, {NULL}};
const Segment SGI201798[] = {{&S82701[117], 3}, {NULL}};
const Segment SGI201799[] = {{&S82701[82], 6}, {NULL}};
const Segment SGI201800[] = {{&S82701[137], 8}, {NULL}};
const Segment SGI201801[] = {{&S82701[146], 13}, {NULL}};
const Segment SGI201802[] = {{&S82703[44], 5}, {NULL}};
const Segment SGI201803[] = {{&S82703[44], 17}, {NULL}};
const Segment SGI201804[] = {{&S82703[30], 6}, {NULL}};
const Segment SGI201805[] = {{&S82703[78], 6}, {NULL}};
const Segment SGI201806[] = {{&S82705[59], 14}, {NULL}};
const Segment SGI201807[] = {{&S82705[59], 26}, {NULL}};
const Segment SGI201808[] = {{&S82705[45], 6}, {NULL}};
const Segment SGI201809[] = {{&S82705[46], 3}, {NULL}};
const Segment SGI201810[] = {{&S82705[99], 6}, {NULL}};
const Segment SGI201811[] = {{&S82705[0], 14}, {NULL}};
const Segment SGI201812[] = {{&S76761[34], 6}, {NULL}};
const Segment SGI201813[] = {{&S76761[27], 6}, {NULL}};
const Segment SGI201814[] = {{&S76761[93], 9}, {NULL}};
const Segment SGI201815[] = {{&S76761[66], 6}, {NULL}};
const Segment SGI201816[] = {{&S82708[44], 7}, {NULL}};
const Segment SGI201817[] = {{&S82708[44], 19}, {NULL}};
const Segment SGI201818[] = {{&S82708[30], 6}, {NULL}};
const Segment SGI201819[] = {{&S82708[31], 3}, {NULL}};
const Segment SGI201820[] = {{&S82708[77], 6}, {NULL}};
const Segment SGI201821[] = {{&S72098[34], 10}, {NULL}};
const Segment SGI201822[] = {{&S72098[27], 6}, {NULL}};
const Segment SGI201823[] = {{&S72098[97], 9}, {NULL}};
const Segment SGI201824[] = {{&S72098[70], 6}, {NULL}};
const Segment SGI201825[] = {{&S82715[34], 7}, {NULL}};
const Segment SGI201826[] = {{&S82715[27], 6}, {NULL}};
const Segment SGI201827[] = {{&S72103[44], 10}, {NULL}};
const Segment SGI201828[] = {{&S72103[44], 22}, {NULL}};
const Segment SGI201829[] = {{&S72103[30], 6}, {NULL}};
const Segment SGI201830[] = {{&S82715[98], 9}, {NULL}};
const Segment SGI201831[] = {{&S72103[131], 3}, {NULL}};
const Segment SGI201832[] = {{&S82715[71], 6}, {NULL}};
const Segment SGI201833[] = {{&S72103[96], 6}, {NULL}};
const Segment SGI201834[] = {{&S72103[68], 8}, {NULL}};
const Segment SGI201835[] = {{&S82715[139], 8}, {NULL}};
const Segment SGI201836[] = {{&S82721[44], 18}, {NULL}};
const Segment SGI201837[] = {{&S82721[44], 30}, {NULL}};
const Segment SGI201838[] = {{&S82721[30], 6}, {NULL}};
const Segment SGI201839[] = {{&S82721[88], 6}, {NULL}};
const Segment SGI201840[] = {{&S82721[115], 13}, {NULL}};
const Segment SGI201841[] = {{&S72126[34], 10}, {NULL}};
const Segment SGI201842[] = {{&S72126[27], 6}, {NULL}};
const Segment SGI201843[] = {{&S72126[97], 9}, {NULL}};
const Segment SGI201844[] = {{&S72126[70], 6}, {NULL}};
const Segment SGI201845[] = {{&S76795[34], 13}, {NULL}};
const Segment SGI201846[] = {{&S76795[27], 6}, {NULL}};
const Segment SGI201847[] = {{&S76795[100], 9}, {NULL}};
const Segment SGI201848[] = {{&S76795[73], 6}, {NULL}};
const Segment SGI201849[] = {{&S71459[34], 20}, {NULL}};
const Segment SGI201850[] = {{&S71459[27], 6}, {NULL}};
const Segment SGI201851[] = {{&S71459[107], 9}, {NULL}};
const Segment SGI201852[] = {{&S71459[80], 6}, {NULL}};
const Segment SGI201853[] = {{&S71461[34], 19}, {NULL}};
const Segment SGI201854[] = {{&S71461[27], 6}, {NULL}};
const Segment SGI201855[] = {{&S71461[106], 9}, {NULL}};
const Segment SGI201856[] = {{&S71461[79], 6}, {NULL}};
const Segment SGI201857[] = {{&S71462[34], 14}, {NULL}};
const Segment SGI201858[] = {{&S71462[27], 6}, {NULL}};
const Segment SGI201859[] = {{&S71462[101], 9}, {NULL}};
const Segment SGI201860[] = {{&S71462[74], 6}, {NULL}};
const Segment SGI201861[] = {{&S72135[34], 10}, {NULL}};
const Segment SGI201862[] = {{&S72135[27], 6}, {NULL}};
const Segment SGI201863[] = {{&S72135[97], 9}, {NULL}};
const Segment SGI201864[] = {{&S72135[70], 6}, {NULL}};
const Segment SGI201865[] = {{&S82747[44], 9}, {NULL}};
const Segment SGI201866[] = {{&S82747[44], 19}, {NULL}};
const Segment SGI201867[] = {{&S82747[30], 6}, {NULL}};
const Segment SGI201868[] = {{&S82747[31], 3}, {NULL}};
const Segment SGI201869[] = {{&S82747[77], 6}, {NULL}};
const Segment SGI201870[] = {{&S82748[44], 9}, {NULL}};
const Segment SGI201871[] = {{&S82748[44], 26}, {NULL}};
const Segment SGI201872[] = {{&S82748[30], 6}, {NULL}};
const Segment SGI201873[] = {{&S72138[34], 7}, {NULL}};
const Segment SGI201874[] = {{&S82748[31], 3}, {NULL}};
const Segment SGI201875[] = {{&S72138[27], 6}, {NULL}};
const Segment SGI201876[] = {{&S82748[84], 6}, {NULL}};
const Segment SGI201877[] = {{&S72138[100], 9}, {NULL}};
const Segment SGI201878[] = {{&S72138[73], 6}, {NULL}};
const Segment SGI201879[] = {{&S72139[34], 7}, {NULL}};
const Segment SGI201880[] = {{&S72139[27], 6}, {NULL}};
const Segment SGI201881[] = {{&S72139[100], 9}, {NULL}};
const Segment SGI201882[] = {{&S72139[73], 6}, {NULL}};
const Segment SGI201883[] = {{&S82750[44], 12}, {NULL}};
const Segment SGI201884[] = {{&S82750[44], 22}, {NULL}};
const Segment SGI201885[] = {{&S82750[30], 6}, {NULL}};
const Segment SGI201886[] = {{&S82750[31], 3}, {NULL}};
const Segment SGI201887[] = {{&S82750[80], 6}, {NULL}};
const Segment SGI201888[] = {{&S82751[44], 5}, {NULL}};
const Segment SGI201889[] = {{&S82751[44], 15}, {NULL}};
const Segment SGI201890[] = {{&S82751[30], 6}, {NULL}};
const Segment SGI201891[] = {{&S82751[31], 3}, {NULL}};
const Segment SGI201892[] = {{&S82751[73], 6}, {NULL}};
const Segment SGI201893[] = {{&S82755[44], 25}, {NULL}};
const Segment SGI201894[] = {{&S82755[44], 36}, {NULL}};
const Segment SGI201895[] = {{&S82755[30], 6}, {NULL}};
const Segment SGI201896[] = {{&S82755[31], 3}, {NULL}};
const Segment SGI201897[] = {{&S82755[115], 6}, {NULL}};
const Segment SGI201898[] = {{&S82755[82], 8}, {NULL}};
const Segment SGI201899[] = {{&S82761[44], 10}, {NULL}};
const Segment SGI201900[] = {{&S82761[44], 22}, {NULL}};
const Segment SGI201901[] = {{&S82761[30], 6}, {NULL}};
const Segment SGI201902[] = {{&S82761[31], 3}, {NULL}};
const Segment SGI201903[] = {{&S82761[80], 6}, {NULL}};
const Segment SGI201904[] = {{&S82761[139], 10}, {NULL}};
const Segment SGI201905[] = {{&S76824[44], 7}, {NULL}};
const Segment SGI201906[] = {{&S76824[44], 19}, {NULL}};
const Segment SGI201907[] = {{&S76824[30], 6}, {NULL}};
const Segment SGI201908[] = {{&S76824[112], 3}, {NULL}};
const Segment SGI201909[] = {{&S76824[77], 6}, {NULL}};
const Segment SGI201910[] = {{&S82762[44], 18}, {NULL}};
const Segment SGI201911[] = {{&S82762[44], 30}, {NULL}};
const Segment SGI201912[] = {{&S82762[30], 6}, {NULL}};
const Segment SGI201913[] = {{&S82762[31], 3}, {NULL}};
const Segment SGI201914[] = {{&S82762[88], 6}, {NULL}};
const Segment SGI201915[] = {{&S82762[147], 10}, {NULL}};
const Segment SGI201916[] = {{&S62419[34], 20}, {NULL}};
const Segment SGI201917[] = {{&S76825[38], 7}, {NULL}};
const Segment SGI201918[] = {{&S62419[27], 6}, {NULL}};
const Segment SGI201919[] = {{&S76825[31], 6}, {NULL}};
const Segment SGI201920[] = {{&S62419[106], 9}, {NULL}};
const Segment SGI201921[] = {{&S62419[79], 6}, {NULL}};
const Segment SGI201922[] = {{&S82763[34], 7}, {NULL}};
const Segment SGI201923[] = {{&S82763[27], 6}, {NULL}};
const Segment SGI201924[] = {{&S82763[94], 9}, {NULL}};
const Segment SGI201925[] = {{&S82763[67], 6}, {NULL}};
const Segment SGI201926[] = {{&S62420[44], 18}, {NULL}};
const Segment SGI201927[] = {{&S62420[44], 29}, {NULL}};
const Segment SGI201928[] = {{&S62420[30], 6}, {NULL}};
const Segment SGI201929[] = {{&S62420[122], 3}, {NULL}};
const Segment SGI201930[] = {{&S62420[87], 6}, {NULL}};
const Segment SGI201931[] = {{&S76828[44], 14}, {NULL}};
const Segment SGI201932[] = {{&S72162[44], 7}, {NULL}};
const Segment SGI201933[] = {{&S76828[44], 26}, {NULL}};
const Segment SGI201934[] = {{&S76828[30], 6}, {NULL}};
const Segment SGI201935[] = {{&S72162[44], 19}, {NULL}};
const Segment SGI201936[] = {{&S72162[30], 6}, {NULL}};
const Segment SGI201937[] = {{&S76828[119], 3}, {NULL}};
const Segment SGI201938[] = {{&S72162[112], 3}, {NULL}};
const Segment SGI201939[] = {{&S62421[44], 10}, {NULL}};
const Segment SGI201940[] = {{&S76828[84], 6}, {NULL}};
const Segment SGI201941[] = {{&S62421[44], 21}, {NULL}};
const Segment SGI201942[] = {{&S72162[77], 6}, {NULL}};
const Segment SGI201943[] = {{&S62421[30], 6}, {NULL}};
const Segment SGI201944[] = {{&S62421[114], 3}, {NULL}};
const Segment SGI201945[] = {{&S62421[79], 6}, {NULL}};
const Segment SGI201946[] = {{&S72163[38], 7}, {NULL}};
const Segment SGI201947[] = {{&S72163[31], 6}, {NULL}};
const Segment SGI201948[] = {{&S76829[44], 9}, {NULL}};
const Segment SGI201949[] = {{&S76829[44], 21}, {NULL}};
const Segment SGI201950[] = {{&S76829[30], 6}, {NULL}};
const Segment SGI201951[] = {{&S71500[44], 17}, {NULL}};
const Segment SGI201952[] = {{&S71500[44], 29}, {NULL}};
const Segment SGI201953[] = {{&S71500[30], 6}, {NULL}};
const Segment SGI201954[] = {{&S62422[44], 18}, {NULL}};
const Segment SGI201955[] = {{&S62422[44], 30}, {NULL}};
const Segment SGI201956[] = {{&S76829[114], 3}, {NULL}};
const Segment SGI201957[] = {{&S62422[30], 6}, {NULL}};
const Segment SGI201958[] = {{&S76829[79], 6}, {NULL}};
const Segment SGI201959[] = {{&S71500[122], 3}, {NULL}};
const Segment SGI201960[] = {{&S71500[87], 6}, {NULL}};
const Segment SGI201961[] = {{&S62422[123], 3}, {NULL}};
const Segment SGI201962[] = {{&S62422[88], 6}, {NULL}};
const Segment SGI201963[] = {{&S71501[38], 17}, {NULL}};
const Segment SGI201964[] = {{&S71501[31], 6}, {NULL}};
const Segment SGI201965[] = {{&S62423[38], 18}, {NULL}};
const Segment SGI201966[] = {{&S62423[31], 6}, {NULL}};
const Segment SGI201967[] = {{&S62433[34], 7}, {NULL}};
const Segment SGI201968[] = {{&S62433[27], 6}, {NULL}};
const Segment SGI201969[] = {{&S62433[93], 9}, {NULL}};
const Segment SGI201970[] = {{&S62433[66], 6}, {NULL}};
const Segment SGI201971[] = {{&S62434[34], 11}, {NULL}};
const Segment SGI201972[] = {{&S62434[27], 6}, {NULL}};
const Segment SGI201973[] = {{&S62434[97], 9}, {NULL}};
const Segment SGI201974[] = {{&S62434[70], 6}, {NULL}};
const Segment SGI201975[] = {{&S72182[34], 10}, {NULL}};
const Segment SGI201976[] = {{&S72182[27], 6}, {NULL}};
const Segment SGI201977[] = {{&S72182[97], 9}, {NULL}};
const Segment SGI201978[] = {{&S72182[70], 6}, {NULL}};
const Segment SGI201979[] = {{&S62455[34], 10}, {NULL}};
const Segment SGI201980[] = {{&S62455[27], 6}, {NULL}};
const Segment SGI201981[] = {{&S62455[96], 9}, {NULL}};
const Segment SGI201982[] = {{&S62455[69], 6}, {NULL}};
const Segment SGI201983[] = {{&S62456[34], 10}, {NULL}};
const Segment SGI201984[] = {{&S62456[27], 6}, {NULL}};
const Segment SGI201985[] = {{&S62456[96], 9}, {NULL}};
const Segment SGI201986[] = {{&S62456[69], 6}, {NULL}};
const Segment SGI201987[] = {{&S62457[34], 10}, {NULL}};
const Segment SGI201988[] = {{&S62457[27], 6}, {NULL}};
const Segment SGI201989[] = {{&S62457[96], 9}, {NULL}};
const Segment SGI201990[] = {{&S62457[69], 6}, {NULL}};
const Segment SGI201991[] = {{&S62458[34], 10}, {NULL}};
const Segment SGI201992[] = {{&S62458[27], 6}, {NULL}};
const Segment SGI201993[] = {{&S62458[96], 9}, {NULL}};
const Segment SGI201994[] = {{&S62458[69], 6}, {NULL}};
const Segment SGI201995[] = {{&S62460[34], 16}, {NULL}};
const Segment SGI201996[] = {{&S62460[27], 6}, {NULL}};
const Segment SGI201997[] = {{&S62460[102], 9}, {NULL}};
const Segment SGI201998[] = {{&S62460[75], 6}, {NULL}};
const Segment SGI201999[] = {{&S62461[34], 11}, {NULL}};
const Segment SGI202000[] = {{&S62461[27], 6}, {NULL}};
const Segment SGI202001[] = {{&S62461[97], 9}, {NULL}};
const Segment SGI202002[] = {{&S62461[70], 6}, {NULL}};
const Segment SGI202003[] = {{&S62462[34], 11}, {NULL}};
const Segment SGI202004[] = {{&S62462[27], 6}, {NULL}};
const Segment SGI202005[] = {{&S62462[97], 9}, {NULL}};
const Segment SGI202006[] = {{&S62462[70], 6}, {NULL}};
const Segment SGI202007[] = {{&S62464[34], 9}, {NULL}};
const Segment SGI202008[] = {{&S62464[27], 6}, {NULL}};
const Segment SGI202009[] = {{&S62464[96], 9}, {NULL}};
const Segment SGI202010[] = {{&S62464[69], 6}, {NULL}};
const Segment SGI202011[] = {{&S76893[44], 11}, {NULL}};
const Segment SGI202012[] = {{&S72233[34], 13}, {NULL}};
const Segment SGI202013[] = {{&S76893[44], 23}, {NULL}};
const Segment SGI202014[] = {{&S72233[27], 6}, {NULL}};
const Segment SGI202015[] = {{&S76893[30], 6}, {NULL}};
const Segment SGI202016[] = {{&S76893[116], 3}, {NULL}};
const Segment SGI202017[] = {{&S72233[100], 9}, {NULL}};
const Segment SGI202018[] = {{&S72233[73], 6}, {NULL}};
const Segment SGI202019[] = {{&S76893[81], 6}, {NULL}};
const Segment SGI202020[] = {{&S72234[34], 19}, {NULL}};
const Segment SGI202021[] = {{&S76894[44], 11}, {NULL}};
const Segment SGI202022[] = {{&S72234[27], 6}, {NULL}};
const Segment SGI202023[] = {{&S76894[44], 23}, {NULL}};
const Segment SGI202024[] = {{&S76894[30], 6}, {NULL}};
const Segment SGI202025[] = {{&S72234[106], 9}, {NULL}};
const Segment SGI202026[] = {{&S76894[116], 3}, {NULL}};
const Segment SGI202027[] = {{&S72234[79], 6}, {NULL}};
const Segment SGI202028[] = {{&S76894[81], 6}, {NULL}};
const Segment SGI202029[] = {{&S76895[38], 11}, {NULL}};
const Segment SGI202030[] = {{&S76895[31], 6}, {NULL}};
const Segment SGI202031[] = {{&S62532[34], 12}, {NULL}};
const Segment SGI202032[] = {{&S62532[27], 6}, {NULL}};
const Segment SGI202033[] = {{&S62532[98], 9}, {NULL}};
const Segment SGI202034[] = {{&S62532[71], 6}, {NULL}};
const Segment SGI202035[] = {{&S72279[44], 10}, {NULL}};
const Segment SGI202036[] = {{&S72279[44], 22}, {NULL}};
const Segment SGI202037[] = {{&S72279[30], 6}, {NULL}};
const Segment SGI202038[] = {{&S72279[131], 3}, {NULL}};
const Segment SGI202039[] = {{&S72279[96], 6}, {NULL}};
const Segment SGI202040[] = {{&S72279[68], 8}, {NULL}};
const Segment SGI202041[] = {{&S76944[34], 17}, {NULL}};
const Segment SGI202042[] = {{&S76944[27], 6}, {NULL}};
const Segment SGI202043[] = {{&S76944[104], 9}, {NULL}};
const Segment SGI202044[] = {{&S76944[77], 6}, {NULL}};
const Segment SGI202045[] = {{&S76945[34], 17}, {NULL}};
const Segment SGI202046[] = {{&S76945[27], 6}, {NULL}};
const Segment SGI202047[] = {{&S76945[104], 9}, {NULL}};
const Segment SGI202048[] = {{&S76945[77], 6}, {NULL}};
const Segment SGI202049[] = {{&S76948[44], 6}, {NULL}};
const Segment SGI202050[] = {{&S76948[44], 18}, {NULL}};
const Segment SGI202051[] = {{&S76948[30], 6}, {NULL}};
const Segment SGI202052[] = {{&S72294[34], 9}, {NULL}};
const Segment SGI202053[] = {{&S76948[109], 3}, {NULL}};
const Segment SGI202054[] = {{&S72294[27], 6}, {NULL}};
const Segment SGI202055[] = {{&S76948[76], 5}, {NULL}};
const Segment SGI202056[] = {{&S76948[113], 14}, {NULL}};
const Segment SGI202057[] = {{&S72294[96], 9}, {NULL}};
const Segment SGI202058[] = {{&S72294[69], 6}, {NULL}};
const Segment SGI202059[] = {{&S76949[44], 7}, {NULL}};
const Segment SGI202060[] = {{&S76949[44], 19}, {NULL}};
const Segment SGI202061[] = {{&S76949[30], 6}, {NULL}};
const Segment SGI202062[] = {{&S76949[110], 3}, {NULL}};
const Segment SGI202063[] = {{&S76949[77], 5}, {NULL}};
const Segment SGI202064[] = {{&S76949[114], 14}, {NULL}};
const Segment SGI202065[] = {{&S76950[44], 8}, {NULL}};
const Segment SGI202066[] = {{&S76950[44], 20}, {NULL}};
const Segment SGI202067[] = {{&S76950[30], 6}, {NULL}};
const Segment SGI202068[] = {{&S76950[111], 3}, {NULL}};
const Segment SGI202069[] = {{&S76950[78], 5}, {NULL}};
const Segment SGI202070[] = {{&S76950[115], 14}, {NULL}};
const Segment SGI202071[] = {{&S76951[44], 7}, {NULL}};
const Segment SGI202072[] = {{&S76951[44], 19}, {NULL}};
const Segment SGI202073[] = {{&S76951[30], 6}, {NULL}};
const Segment SGI202074[] = {{&S76951[110], 3}, {NULL}};
const Segment SGI202075[] = {{&S76951[77], 5}, {NULL}};
const Segment SGI202076[] = {{&S76951[114], 14}, {NULL}};
const Segment SGI202077[] = {{&S76952[44], 7}, {NULL}};
const Segment SGI202078[] = {{&S76952[44], 19}, {NULL}};
const Segment SGI202079[] = {{&S76952[30], 6}, {NULL}};
const Segment SGI202080[] = {{&S76952[110], 3}, {NULL}};
const Segment SGI202081[] = {{&S76952[77], 5}, {NULL}};
const Segment SGI202082[] = {{&S76952[114], 14}, {NULL}};
const Segment SGI202083[] = {{&S71667[34], 10}, {NULL}};
const Segment SGI202084[] = {{&S71667[27], 6}, {NULL}};
const Segment SGI202085[] = {{&S71667[97], 9}, {NULL}};
const Segment SGI202086[] = {{&S71667[70], 6}, {NULL}};
const Segment SGI202087[] = {{&S72334[34], 7}, {NULL}};
const Segment SGI202088[] = {{&S72334[27], 6}, {NULL}};
const Segment SGI202089[] = {{&S72334[99], 9}, {NULL}};
const Segment SGI202090[] = {{&S72334[72], 6}, {NULL}};
const Segment SGI202091[] = {{&S72334[140], 8}, {NULL}};
const Segment SGI202092[] = {{&S82777[34], 8}, {NULL}};
const Segment SGI202093[] = {{&S82777[27], 6}, {NULL}};
const Segment SGI202094[] = {{&S82777[100], 9}, {NULL}};
const Segment SGI202095[] = {{&S82777[73], 6}, {NULL}};
const Segment SGI202096[] = {{&S82778[34], 7}, {NULL}};
const Segment SGI202097[] = {{&S82778[27], 6}, {NULL}};
const Segment SGI202098[] = {{&S82778[94], 9}, {NULL}};
const Segment SGI202099[] = {{&S82778[67], 6}, {NULL}};
const Segment SGI202100[] = {{&S82786[34], 9}, {NULL}};
const Segment SGI202101[] = {{&S82786[27], 6}, {NULL}};
const Segment SGI202102[] = {{&S82786[96], 9}, {NULL}};
const Segment SGI202103[] = {{&S82786[69], 6}, {NULL}};
const Segment SGI202104[] = {{&S72365[44], 10}, {NULL}};
const Segment SGI202105[] = {{&S72365[44], 22}, {NULL}};
const Segment SGI202106[] = {{&S72365[30], 6}, {NULL}};
const Segment SGI202107[] = {{&S72365[115], 3}, {NULL}};
const Segment SGI202108[] = {{&S72365[80], 6}, {NULL}};
const Segment SGI202109[] = {{&S62620[44], 10}, {NULL}};
const Segment SGI202110[] = {{&S62620[44], 21}, {NULL}};
const Segment SGI202111[] = {{&S62620[30], 6}, {NULL}};
const Segment SGI202112[] = {{&S62620[132], 3}, {NULL}};
const Segment SGI202113[] = {{&S62620[97], 6}, {NULL}};
const Segment SGI202114[] = {{&S62620[67], 8}, {NULL}};
const Segment SGI202115[] = {{&S62622[44], 10}, {NULL}};
const Segment SGI202116[] = {{&S62622[44], 22}, {NULL}};
const Segment SGI202117[] = {{&S62622[30], 6}, {NULL}};
const Segment SGI202118[] = {{&S62622[133], 3}, {NULL}};
const Segment SGI202119[] = {{&S62622[98], 6}, {NULL}};
const Segment SGI202120[] = {{&S62622[68], 8}, {NULL}};
const Segment SGI202121[] = {{&S82804[34], 11}, {NULL}};
const Segment SGI202122[] = {{&S82804[27], 6}, {NULL}};
const Segment SGI202123[] = {{&S82804[108], 9}, {NULL}};
const Segment SGI202124[] = {{&S82804[81], 6}, {NULL}};
const Segment SGI202125[] = {{&S82806[34], 11}, {NULL}};
const Segment SGI202126[] = {{&S82806[27], 6}, {NULL}};
const Segment SGI202127[] = {{&S82806[108], 9}, {NULL}};
const Segment SGI202128[] = {{&S82806[81], 6}, {NULL}};
const Segment SGI202129[] = {{&S72381[34], 10}, {NULL}};
const Segment SGI202130[] = {{&S71721[34], 10}, {NULL}};
const Segment SGI202131[] = {{&S72381[27], 6}, {NULL}};
const Segment SGI202132[] = {{&S71721[27], 6}, {NULL}};
const Segment SGI202133[] = {{&S72381[97], 9}, {NULL}};
const Segment SGI202134[] = {{&S72381[70], 6}, {NULL}};
const Segment SGI202135[] = {{&S71721[97], 9}, {NULL}};
const Segment SGI202136[] = {{&S71721[70], 6}, {NULL}};
const Segment SGI202137[] = {{&S71721[138], 8}, {NULL}};
const Segment SGI202138[] = {{&S72384[34], 10}, {NULL}};
const Segment SGI202139[] = {{&S72384[27], 6}, {NULL}};
const Segment SGI202140[] = {{&S72384[97], 9}, {NULL}};
const Segment SGI202141[] = {{&S72384[70], 6}, {NULL}};
const Segment SGI202142[] = {{&S62640[44], 18}, {NULL}};
const Segment SGI202143[] = {{&S62640[44], 29}, {NULL}};
const Segment SGI202144[] = {{&S62640[30], 6}, {NULL}};
const Segment SGI202145[] = {{&S62640[122], 3}, {NULL}};
const Segment SGI202146[] = {{&S62640[87], 6}, {NULL}};
const Segment SGI202147[] = {{&S62640[157], 3}, {NULL}};
const Segment SGI202148[] = {{&S82842[34], 6}, {NULL}};
const Segment SGI202149[] = {{&S82842[27], 6}, {NULL}};
const Segment SGI202150[] = {{&S82842[93], 9}, {NULL}};
const Segment SGI202151[] = {{&S82842[66], 6}, {NULL}};
const Segment SGI202152[] = {{&S72411[34], 15}, {NULL}};
const Segment SGI202153[] = {{&S72411[27], 6}, {NULL}};
const Segment SGI202154[] = {{&S72411[102], 9}, {NULL}};
const Segment SGI202155[] = {{&S72411[75], 6}, {NULL}};
const Segment SGI202156[] = {{&S72412[34], 15}, {NULL}};
const Segment SGI202157[] = {{&S72412[27], 6}, {NULL}};
const Segment SGI202158[] = {{&S72412[102], 9}, {NULL}};
const Segment SGI202159[] = {{&S72412[75], 6}, {NULL}};
const Segment SGI202160[] = {{&S82849[44], 7}, {NULL}};
const Segment SGI202161[] = {{&S82849[44], 19}, {NULL}};
const Segment SGI202162[] = {{&S82849[30], 6}, {NULL}};
const Segment SGI202163[] = {{&S82849[31], 3}, {NULL}};
const Segment SGI202164[] = {{&S82849[77], 6}, {NULL}};
const Segment SGI202165[] = {{&S82849[140], 3}, {NULL}};
const Segment SGI202166[] = {{&S82850[44], 13}, {NULL}};
const Segment SGI202167[] = {{&S82850[44], 25}, {NULL}};
const Segment SGI202168[] = {{&S82850[30], 6}, {NULL}};
const Segment SGI202169[] = {{&S82850[31], 3}, {NULL}};
const Segment SGI202170[] = {{&S82850[83], 6}, {NULL}};
const Segment SGI202171[] = {{&S71766[44], 10}, {NULL}};
const Segment SGI202172[] = {{&S71766[44], 22}, {NULL}};
const Segment SGI202173[] = {{&S71766[30], 6}, {NULL}};
const Segment SGI202174[] = {{&S82851[34], 8}, {NULL}};
const Segment SGI202175[] = {{&S71766[133], 3}, {NULL}};
const Segment SGI202176[] = {{&S82851[27], 6}, {NULL}};
const Segment SGI202177[] = {{&S71766[98], 6}, {NULL}};
const Segment SGI202178[] = {{&S71766[68], 8}, {NULL}};
const Segment SGI202179[] = {{&S82851[95], 9}, {NULL}};
const Segment SGI202180[] = {{&S82851[68], 6}, {NULL}};
const Segment SGI202181[] = {{&S82853[34], 5}, {NULL}};
const Segment SGI202182[] = {{&S82853[27], 6}, {NULL}};
const Segment SGI202183[] = {{&S82853[92], 9}, {NULL}};
const Segment SGI202184[] = {{&S82853[65], 6}, {NULL}};
const Segment SGI202185[] = {{&S72425[44], 9}, {NULL}};
const Segment SGI202186[] = {{&S72425[44], 21}, {NULL}};
const Segment SGI202187[] = {{&S72425[30], 6}, {NULL}};
const Segment SGI202188[] = {{&S72425[114], 3}, {NULL}};
const Segment SGI202189[] = {{&S72425[79], 6}, {NULL}};
const Segment SGI202190[] = {{&S82858[44], 10}, {NULL}};
const Segment SGI202191[] = {{&S82858[44], 22}, {NULL}};
const Segment SGI202192[] = {{&S82858[30], 6}, {NULL}};
const Segment SGI202193[] = {{&S72429[44], 13}, {NULL}};
const Segment SGI202194[] = {{&S72429[44], 25}, {NULL}};
const Segment SGI202195[] = {{&S72429[30], 6}, {NULL}};
const Segment SGI202196[] = {{&S82858[80], 6}, {NULL}};
const Segment SGI202197[] = {{&S72429[118], 3}, {NULL}};
const Segment SGI202198[] = {{&S82858[107], 13}, {NULL}};
const Segment SGI202199[] = {{&S72429[83], 6}, {NULL}};
const Segment SGI202200[] = {{&S72429[150], 15}, {NULL}};
const Segment SGI202201[] = {{&S82859[44], 6}, {NULL}};
const Segment SGI202202[] = {{&S82859[44], 18}, {NULL}};
const Segment SGI202203[] = {{&S72430[44], 8}, {NULL}};
const Segment SGI202204[] = {{&S82859[30], 6}, {NULL}};
const Segment SGI202205[] = {{&S72430[44], 20}, {NULL}};
const Segment SGI202206[] = {{&S72430[30], 6}, {NULL}};
const Segment SGI202207[] = {{&S72430[113], 3}, {NULL}};
const Segment SGI202208[] = {{&S82859[76], 6}, {NULL}};
const Segment SGI202209[] = {{&S72430[78], 6}, {NULL}};
const Segment SGI202210[] = {{&S82859[103], 13}, {NULL}};
const Segment SGI202211[] = {{&S72430[145], 15}, {NULL}};
const Segment SGI202212[] = {{&S82861[34], 5}, {NULL}};
const Segment SGI202213[] = {{&S82861[27], 6}, {NULL}};
const Segment SGI202214[] = {{&S82861[92], 9}, {NULL}};
const Segment SGI202215[] = {{&S82861[65], 6}, {NULL}};
const Segment SGI202216[] = {{&S62697[34], 13}, {NULL}};
const Segment SGI202217[] = {{&S72433[34], 10}, {NULL}};
const Segment SGI202218[] = {{&S62697[27], 6}, {NULL}};
const Segment SGI202219[] = {{&S72433[27], 6}, {NULL}};
const Segment SGI202220[] = {{&S62697[100], 9}, {NULL}};
const Segment SGI202221[] = {{&S62697[73], 6}, {NULL}};
const Segment SGI202222[] = {{&S72433[97], 9}, {NULL}};
const Segment SGI202223[] = {{&S72433[70], 6}, {NULL}};
const Segment SGI202224[] = {{&S62698[34], 19}, {NULL}};
const Segment SGI202225[] = {{&S62698[27], 6}, {NULL}};
const Segment SGI202226[] = {{&S62698[105], 9}, {NULL}};
const Segment SGI202227[] = {{&S62698[78], 6}, {NULL}};
const Segment SGI202228[] = {{&S62729[44], 10}, {NULL}};
const Segment SGI202229[] = {{&S62729[44], 21}, {NULL}};
const Segment SGI202230[] = {{&S62729[30], 6}, {NULL}};
const Segment SGI202231[] = {{&S62729[114], 3}, {NULL}};
const Segment SGI202232[] = {{&S62729[79], 6}, {NULL}};
const Segment SGI202233[] = {{&S62731[44], 10}, {NULL}};
const Segment SGI202234[] = {{&S62731[44], 22}, {NULL}};
const Segment SGI202235[] = {{&S62731[30], 6}, {NULL}};
const Segment SGI202236[] = {{&S62731[115], 3}, {NULL}};
const Segment SGI202237[] = {{&S62731[80], 6}, {NULL}};
const Segment SGI202238[] = {{&S62732[38], 10}, {NULL}};
const Segment SGI202239[] = {{&S62732[31], 6}, {NULL}};
const Segment SGI202240[] = {{&S62734[34], 10}, {NULL}};
const Segment SGI202241[] = {{&S62734[27], 6}, {NULL}};
const Segment SGI202242[] = {{&S82902[44], 4}, {NULL}};
const Segment SGI202243[] = {{&S82902[44], 16}, {NULL}};
const Segment SGI202244[] = {{&S62734[96], 9}, {NULL}};
const Segment SGI202245[] = {{&S82902[30], 6}, {NULL}};
const Segment SGI202246[] = {{&S62734[69], 6}, {NULL}};
const Segment SGI202247[] = {{&S82902[31], 3}, {NULL}};
const Segment SGI202248[] = {{&S82902[74], 6}, {NULL}};
const Segment SGI202249[] = {{&S82911[45], 7}, {NULL}};
const Segment SGI202250[] = {{&S82911[30], 6}, {NULL}};
const Segment SGI202251[] = {{&S82911[31], 3}, {NULL}};
const Segment SGI202252[] = {{&S82911[83], 5}, {NULL}};
const Segment SGI202253[] = {{&S82911[141], 13}, {NULL}};
const Segment SGI202254[] = {{&S82912[44], 13}, {NULL}};
const Segment SGI202255[] = {{&S82912[44], 25}, {NULL}};
const Segment SGI202256[] = {{&S82912[30], 6}, {NULL}};
const Segment SGI202257[] = {{&S82912[31], 3}, {NULL}};
const Segment SGI202258[] = {{&S82912[83], 6}, {NULL}};
const Segment SGI202259[] = {{&S82913[38], 13}, {NULL}};
const Segment SGI202260[] = {{&S82913[31], 6}, {NULL}};
const Segment SGI202261[] = {{&S77038[44], 10}, {NULL}};
const Segment SGI202262[] = {{&S77038[44], 22}, {NULL}};
const Segment SGI202263[] = {{&S77038[30], 6}, {NULL}};
const Segment SGI202264[] = {{&S82914[44], 13}, {NULL}};
const Segment SGI202265[] = {{&S77038[113], 3}, {NULL}};
const Segment SGI202266[] = {{&S82914[44], 25}, {NULL}};
const Segment SGI202267[] = {{&S82914[30], 6}, {NULL}};
const Segment SGI202268[] = {{&S77038[80], 5}, {NULL}};
const Segment SGI202269[] = {{&S77038[117], 14}, {NULL}};
const Segment SGI202270[] = {{&S82914[86], 6}, {NULL}};
const Segment SGI202271[] = {{&S77045[34], 8}, {NULL}};
const Segment SGI202272[] = {{&S77045[27], 6}, {NULL}};
const Segment SGI202273[] = {{&S77045[95], 9}, {NULL}};
const Segment SGI202274[] = {{&S77045[68], 6}, {NULL}};
const Segment SGI202275[] = {{&S82922[44], 16}, {NULL}};
const Segment SGI202276[] = {{&S82922[44], 41}, {NULL}};
const Segment SGI202277[] = {{&S82922[30], 6}, {NULL}};
const Segment SGI202278[] = {{&S82922[31], 3}, {NULL}};
const Segment SGI202279[] = {{&S82922[99], 6}, {NULL}};
const Segment SGI202280[] = {{&S82924[34], 10}, {NULL}};
const Segment SGI202281[] = {{&S82924[27], 6}, {NULL}};
const Segment SGI202282[] = {{&S82924[97], 9}, {NULL}};
const Segment SGI202283[] = {{&S82924[70], 6}, {NULL}};
const Segment SGI202284[] = {{&S77052[44], 10}, {NULL}};
const Segment SGI202285[] = {{&S77052[44], 22}, {NULL}};
const Segment SGI202286[] = {{&S77052[30], 6}, {NULL}};
const Segment SGI202287[] = {{&S77052[113], 3}, {NULL}};
const Segment SGI202288[] = {{&S77052[80], 5}, {NULL}};
const Segment SGI202289[] = {{&S77052[117], 14}, {NULL}};
const Segment SGI202290[] = {{&S82927[44], 14}, {NULL}};
const Segment SGI202291[] = {{&S82927[44], 26}, {NULL}};
const Segment SGI202292[] = {{&S82927[30], 6}, {NULL}};
const Segment SGI202293[] = {{&S82927[84], 6}, {NULL}};
const Segment SGI202294[] = {{&S82927[111], 13}, {NULL}};
const Segment SGI202295[] = {{&S62788[44], 15}, {NULL}};
const Segment SGI202296[] = {{&S62788[44], 26}, {NULL}};
const Segment SGI202297[] = {{&S62788[30], 6}, {NULL}};
const Segment SGI202298[] = {{&S82938[34], 9}, {NULL}};
const Segment SGI202299[] = {{&S71879[44], 10}, {NULL}};
const Segment SGI202300[] = {{&S71879[44], 22}, {NULL}};
const Segment SGI202301[] = {{&S82938[27], 6}, {NULL}};
const Segment SGI202302[] = {{&S62788[137], 3}, {NULL}};
const Segment SGI202303[] = {{&S71879[30], 6}, {NULL}};
const Segment SGI202304[] = {{&S62788[102], 6}, {NULL}};
const Segment SGI202305[] = {{&S62788[72], 8}, {NULL}};
const Segment SGI202306[] = {{&S71879[115], 3}, {NULL}};
const Segment SGI202307[] = {{&S82938[103], 9}, {NULL}};
const Segment SGI202308[] = {{&S82938[76], 6}, {NULL}};
const Segment SGI202309[] = {{&S71879[80], 6}, {NULL}};
const Segment SGI202310[] = {{&S62789[44], 16}, {NULL}};
const Segment SGI202311[] = {{&S62789[44], 27}, {NULL}};
const Segment SGI202312[] = {{&S71880[44], 10}, {NULL}};
const Segment SGI202313[] = {{&S62789[30], 6}, {NULL}};
const Segment SGI202314[] = {{&S71880[44], 22}, {NULL}};
const Segment SGI202315[] = {{&S71880[30], 6}, {NULL}};
const Segment SGI202316[] = {{&S62789[138], 3}, {NULL}};
const Segment SGI202317[] = {{&S71880[115], 3}, {NULL}};
const Segment SGI202318[] = {{&S62789[103], 6}, {NULL}};
const Segment SGI202319[] = {{&S71880[80], 6}, {NULL}};
const Segment SGI202320[] = {{&S62789[73], 8}, {NULL}};
const Segment SGI202321[] = {{&S62790[44], 16}, {NULL}};
const Segment SGI202322[] = {{&S71881[54], 10}, {NULL}};
const Segment SGI202323[] = {{&S62790[44], 28}, {NULL}};
const Segment SGI202324[] = {{&S71881[54], 22}, {NULL}};
const Segment SGI202325[] = {{&S62790[30], 6}, {NULL}};
const Segment SGI202326[] = {{&S71881[30], 16}, {NULL}};
const Segment SGI202327[] = {{&S62790[139], 3}, {NULL}};
const Segment SGI202328[] = {{&S71881[125], 3}, {NULL}};
const Segment SGI202329[] = {{&S62790[104], 6}, {NULL}};
const Segment SGI202330[] = {{&S71881[90], 6}, {NULL}};
const Segment SGI202331[] = {{&S62790[74], 8}, {NULL}};
const Segment SGI202332[] = {{&S82942[52], 10}, {NULL}};
const Segment SGI202333[] = {{&S82942[52], 22}, {NULL}};
const Segment SGI202334[] = {{&S82942[38], 6}, {NULL}};
const Segment SGI202335[] = {{&S82942[88], 5}, {NULL}};
const Segment SGI202336[] = {{&S82942[0], 14}, {NULL}};
const Segment SGI202337[] = {{&S62791[44], 16}, {NULL}};
const Segment SGI202338[] = {{&S62791[44], 28}, {NULL}};
const Segment SGI202339[] = {{&S62791[30], 6}, {NULL}};
const Segment SGI202340[] = {{&S62791[139], 3}, {NULL}};
const Segment SGI202341[] = {{&S62791[104], 6}, {NULL}};
const Segment SGI202342[] = {{&S62791[74], 8}, {NULL}};
const Segment SGI202343[] = {{&S82943[34], 9}, {NULL}};
const Segment SGI202344[] = {{&S82943[27], 6}, {NULL}};
const Segment SGI202345[] = {{&S82943[93], 9}, {NULL}};
const Segment SGI202346[] = {{&S82943[66], 6}, {NULL}};
const Segment SGI202347[] = {{&S82946[34], 13}, {NULL}};
const Segment SGI202348[] = {{&S82946[27], 6}, {NULL}};
const Segment SGI202349[] = {{&S82946[100], 9}, {NULL}};
const Segment SGI202350[] = {{&S82946[73], 6}, {NULL}};
const Segment SGI202351[] = {{&S77085[34], 5}, {NULL}};
const Segment SGI202352[] = {{&S77085[27], 6}, {NULL}};
const Segment SGI202353[] = {{&S77085[92], 9}, {NULL}};
const Segment SGI202354[] = {{&S77085[65], 6}, {NULL}};
const Segment SGI202355[] = {{&S82984[31], 5}, {NULL}};
const Segment SGI202356[] = {{&S82984[51], 3}, {NULL}};
const Segment SGI202357[] = {{&S62838[38], 10}, {NULL}};
const Segment SGI202358[] = {{&S62838[31], 6}, {NULL}};
const Segment SGI202359[] = {{&S62839[44], 10}, {NULL}};
const Segment SGI202360[] = {{&S62839[44], 21}, {NULL}};
const Segment SGI202361[] = {{&S62839[30], 6}, {NULL}};
const Segment SGI202362[] = {{&S62839[114], 3}, {NULL}};
const Segment SGI202363[] = {{&S62839[79], 6}, {NULL}};
const Segment SGI202364[] = {{&S62839[134], 8}, {NULL}};
const Segment SGI202365[] = {{&S62840[44], 10}, {NULL}};
const Segment SGI202366[] = {{&S62840[44], 22}, {NULL}};
const Segment SGI202367[] = {{&S62840[30], 6}, {NULL}};
const Segment SGI202368[] = {{&S62840[115], 3}, {NULL}};
const Segment SGI202369[] = {{&S62840[80], 6}, {NULL}};
const Segment SGI202370[] = {{&S62841[44], 10}, {NULL}};
const Segment SGI202371[] = {{&S62841[44], 21}, {NULL}};
const Segment SGI202372[] = {{&S62841[30], 6}, {NULL}};
const Segment SGI202373[] = {{&S62841[114], 3}, {NULL}};
const Segment SGI202374[] = {{&S62841[79], 6}, {NULL}};
const Segment SGI202375[] = {{&S82990[34], 7}, {NULL}};
const Segment SGI202376[] = {{&S82990[27], 6}, {NULL}};
const Segment SGI202377[] = {{&S82990[94], 9}, {NULL}};
const Segment SGI202378[] = {{&S82990[67], 6}, {NULL}};
const Segment SGI202379[] = {{&S82992[34], 10}, {NULL}};
const Segment SGI202380[] = {{&S82992[27], 6}, {NULL}};
const Segment SGI202381[] = {{&S82992[97], 9}, {NULL}};
const Segment SGI202382[] = {{&S82992[70], 6}, {NULL}};
const Segment SGI202383[] = {{&S62847[34], 10}, {NULL}};
const Segment SGI202384[] = {{&S62847[27], 6}, {NULL}};
const Segment SGI202385[] = {{&S62847[96], 9}, {NULL}};
const Segment SGI202386[] = {{&S62847[69], 6}, {NULL}};
const Segment SGI202387[] = {{&S72440[34], 9}, {NULL}};
const Segment SGI202388[] = {{&S82995[44], 7}, {NULL}};
const Segment SGI202389[] = {{&S72440[27], 6}, {NULL}};
const Segment SGI202390[] = {{&S82995[44], 19}, {NULL}};
const Segment SGI202391[] = {{&S82995[30], 6}, {NULL}};
const Segment SGI202392[] = {{&S82995[31], 3}, {NULL}};
const Segment SGI202393[] = {{&S72440[96], 9}, {NULL}};
const Segment SGI202394[] = {{&S82995[77], 5}, {NULL}};
const Segment SGI202395[] = {{&S72440[69], 6}, {NULL}};
const Segment SGI202396[] = {{&S82995[114], 14}, {NULL}};
const Segment SGI202397[] = {{&S72440[137], 8}, {NULL}};
const Segment SGI202398[] = {{&S72441[34], 9}, {NULL}};
const Segment SGI202399[] = {{&S72441[27], 6}, {NULL}};
const Segment SGI202400[] = {{&S82996[44], 8}, {NULL}};
const Segment SGI202401[] = {{&S82996[44], 20}, {NULL}};
const Segment SGI202402[] = {{&S82996[30], 6}, {NULL}};
const Segment SGI202403[] = {{&S72441[96], 9}, {NULL}};
const Segment SGI202404[] = {{&S82996[31], 3}, {NULL}};
const Segment SGI202405[] = {{&S72441[69], 6}, {NULL}};
const Segment SGI202406[] = {{&S82996[78], 5}, {NULL}};
const Segment SGI202407[] = {{&S72441[137], 8}, {NULL}};
const Segment SGI202408[] = {{&S82996[115], 14}, {NULL}};
const Segment SGI202409[] = {{&S72442[34], 9}, {NULL}};
const Segment SGI202410[] = {{&S82997[44], 17}, {NULL}};
const Segment SGI202411[] = {{&S72442[27], 6}, {NULL}};
const Segment SGI202412[] = {{&S82997[44], 27}, {NULL}};
const Segment SGI202413[] = {{&S82997[30], 6}, {NULL}};
const Segment SGI202414[] = {{&S82997[31], 3}, {NULL}};
const Segment SGI202415[] = {{&S72442[96], 9}, {NULL}};
const Segment SGI202416[] = {{&S72442[69], 6}, {NULL}};
const Segment SGI202417[] = {{&S82997[85], 6}, {NULL}};
const Segment SGI202418[] = {{&S72442[137], 8}, {NULL}};
const Segment SGI202419[] = {{&S82998[44], 17}, {NULL}};
const Segment SGI202420[] = {{&S72443[34], 10}, {NULL}};
const Segment SGI202421[] = {{&S82998[44], 27}, {NULL}};
const Segment SGI202422[] = {{&S72443[27], 6}, {NULL}};
const Segment SGI202423[] = {{&S82998[30], 6}, {NULL}};
const Segment SGI202424[] = {{&S82998[31], 3}, {NULL}};
const Segment SGI202425[] = {{&S72443[97], 9}, {NULL}};
const Segment SGI202426[] = {{&S82998[85], 6}, {NULL}};
const Segment SGI202427[] = {{&S72443[70], 6}, {NULL}};
const Segment SGI202428[] = {{&S83000[34], 6}, {NULL}};
const Segment SGI202429[] = {{&S83000[27], 6}, {NULL}};
const Segment SGI202430[] = {{&S83000[93], 9}, {NULL}};
const Segment SGI202431[] = {{&S83000[66], 6}, {NULL}};
const Segment SGI202432[] = {{&S77139[44], 9}, {NULL}};
const Segment SGI202433[] = {{&S77139[44], 30}, {NULL}};
const Segment SGI202434[] = {{&S77139[30], 6}, {NULL}};
const Segment SGI202435[] = {{&S77139[123], 3}, {NULL}};
const Segment SGI202436[] = {{&S77139[88], 6}, {NULL}};
const Segment SGI202437[] = {{&S83006[34], 9}, {NULL}};
const Segment SGI202438[] = {{&S83006[27], 6}, {NULL}};
const Segment SGI202439[] = {{&S83006[96], 9}, {NULL}};
const Segment SGI202440[] = {{&S83006[69], 6}, {NULL}};
const Segment SGI202441[] = {{&S83009[34], 11}, {NULL}};
const Segment SGI202442[] = {{&S83009[27], 6}, {NULL}};
const Segment SGI202443[] = {{&S83009[98], 9}, {NULL}};
const Segment SGI202444[] = {{&S83009[71], 6}, {NULL}};
const Segment SGI202445[] = {{&S83010[34], 13}, {NULL}};
const Segment SGI202446[] = {{&S83010[27], 6}, {NULL}};
const Segment SGI202447[] = {{&S83010[100], 9}, {NULL}};
const Segment SGI202448[] = {{&S83010[73], 6}, {NULL}};
const Segment SGI202449[] = {{&S77150[34], 11}, {NULL}};
const Segment SGI202450[] = {{&S77150[27], 6}, {NULL}};
const Segment SGI202451[] = {{&S77150[98], 9}, {NULL}};
const Segment SGI202452[] = {{&S77150[71], 6}, {NULL}};
const Segment SGI202453[] = {{&S62876[34], 10}, {NULL}};
const Segment SGI202454[] = {{&S62876[27], 6}, {NULL}};
const Segment SGI202455[] = {{&S62876[96], 9}, {NULL}};
const Segment SGI202456[] = {{&S62876[69], 6}, {NULL}};
const Segment SGI202457[] = {{&S83016[44], 7}, {NULL}};
const Segment SGI202458[] = {{&S83016[44], 23}, {NULL}};
const Segment SGI202459[] = {{&S83016[30], 6}, {NULL}};
const Segment SGI202460[] = {{&S83016[81], 6}, {NULL}};
const Segment SGI202461[] = {{&S83016[108], 13}, {NULL}};
const Segment SGI202462[] = {{&S83017[44], 7}, {NULL}};
const Segment SGI202463[] = {{&S83017[44], 24}, {NULL}};
const Segment SGI202464[] = {{&S83017[30], 6}, {NULL}};
const Segment SGI202465[] = {{&S77155[34], 13}, {NULL}};
const Segment SGI202466[] = {{&S77155[27], 6}, {NULL}};
const Segment SGI202467[] = {{&S83017[82], 6}, {NULL}};
const Segment SGI202468[] = {{&S83017[109], 13}, {NULL}};
const Segment SGI202469[] = {{&S77155[100], 9}, {NULL}};
const Segment SGI202470[] = {{&S77155[73], 6}, {NULL}};
const Segment SGI202471[] = {{&S77158[34], 7}, {NULL}};
const Segment SGI202472[] = {{&S77158[27], 6}, {NULL}};
const Segment SGI202473[] = {{&S77158[98], 9}, {NULL}};
const Segment SGI202474[] = {{&S77158[71], 6}, {NULL}};
const Segment SGI202475[] = {{&S83022[34], 7}, {NULL}};
const Segment SGI202476[] = {{&S83022[27], 6}, {NULL}};
const Segment SGI202477[] = {{&S83022[99], 9}, {NULL}};
const Segment SGI202478[] = {{&S83022[72], 6}, {NULL}};
const Segment SGI202479[] = {{&S83025[34], 10}, {NULL}};
const Segment SGI202480[] = {{&S83025[27], 6}, {NULL}};
const Segment SGI202481[] = {{&S83025[97], 9}, {NULL}};
const Segment SGI202482[] = {{&S83025[70], 6}, {NULL}};
const Segment SGI202483[] = {{&S83031[34], 9}, {NULL}};
const Segment SGI202484[] = {{&S83031[27], 6}, {NULL}};
const Segment SGI202485[] = {{&S83031[96], 9}, {NULL}};
const Segment SGI202486[] = {{&S83031[69], 6}, {NULL}};
const Segment SGI202487[] = {{&S83760[44], 11}, {NULL}};
const Segment SGI202488[] = {{&S83760[44], 23}, {NULL}};
const Segment SGI202489[] = {{&S83760[30], 6}, {NULL}};
const Segment SGI202490[] = {{&S83760[31], 3}, {NULL}};
const Segment SGI202491[] = {{&S83760[81], 6}, {NULL}};
const Segment SGI202492[] = {{&S72494[44], 12}, {NULL}};
const Segment SGI202493[] = {{&S72494[44], 34}, {NULL}};
const Segment SGI202494[] = {{&S72494[30], 6}, {NULL}};
const Segment SGI202495[] = {{&S72494[127], 3}, {NULL}};
const Segment SGI202496[] = {{&S72494[92], 6}, {NULL}};
const Segment SGI202497[] = {{&S72494[152], 7}, {NULL}};
const Segment SGI202498[] = {{&S83048[34], 10}, {NULL}};
const Segment SGI202499[] = {{&S83048[27], 6}, {NULL}};
const Segment SGI202500[] = {{&S83048[97], 9}, {NULL}};
const Segment SGI202501[] = {{&S83048[70], 6}, {NULL}};
const Segment SGI202502[] = {{&S83054[34], 9}, {NULL}};
const Segment SGI202503[] = {{&S83054[27], 6}, {NULL}};
const Segment SGI202504[] = {{&S83054[96], 9}, {NULL}};
const Segment SGI202505[] = {{&S83054[69], 6}, {NULL}};
const Segment SGI202506[] = {{&S72515[34], 10}, {NULL}};
const Segment SGI202507[] = {{&S72515[27], 6}, {NULL}};
const Segment SGI202508[] = {{&S72515[97], 9}, {NULL}};
const Segment SGI202509[] = {{&S72515[70], 6}, {NULL}};
const Segment SGI202510[] = {{&S72516[34], 10}, {NULL}};
const Segment SGI202511[] = {{&S72516[27], 6}, {NULL}};
const Segment SGI202512[] = {{&S72516[97], 9}, {NULL}};
const Segment SGI202513[] = {{&S72516[70], 6}, {NULL}};
const Segment SGI202514[] = {{&S72517[34], 10}, {NULL}};
const Segment SGI202515[] = {{&S77207[34], 10}, {NULL}};
const Segment SGI202516[] = {{&S72517[27], 6}, {NULL}};
const Segment SGI202517[] = {{&S77207[27], 6}, {NULL}};
const Segment SGI202518[] = {{&S72517[97], 9}, {NULL}};
const Segment SGI202519[] = {{&S83064[59], 10}, {NULL}};
const Segment SGI202520[] = {{&S72517[70], 6}, {NULL}};
const Segment SGI202521[] = {{&S83064[59], 22}, {NULL}};
const Segment SGI202522[] = {{&S77207[97], 9}, {NULL}};
const Segment SGI202523[] = {{&S83064[45], 6}, {NULL}};
const Segment SGI202524[] = {{&S77207[70], 6}, {NULL}};
const Segment SGI202525[] = {{&S83064[46], 3}, {NULL}};
const Segment SGI202526[] = {{&S83064[95], 6}, {NULL}};
const Segment SGI202527[] = {{&S83064[0], 14}, {NULL}};
const Segment SGI202528[] = {{&S72518[34], 11}, {NULL}};
const Segment SGI202529[] = {{&S72518[27], 6}, {NULL}};
const Segment SGI202530[] = {{&S72518[98], 9}, {NULL}};
const Segment SGI202531[] = {{&S72518[71], 6}, {NULL}};
const Segment SGI202532[] = {{&S83065[34], 10}, {NULL}};
const Segment SGI202533[] = {{&S83065[27], 6}, {NULL}};
const Segment SGI202534[] = {{&S83065[97], 9}, {NULL}};
const Segment SGI202535[] = {{&S83065[70], 6}, {NULL}};
const Segment SGI202536[] = {{&S72521[34], 9}, {NULL}};
const Segment SGI202537[] = {{&S72521[27], 6}, {NULL}};
const Segment SGI202538[] = {{&S72521[101], 9}, {NULL}};
const Segment SGI202539[] = {{&S72521[74], 6}, {NULL}};
const Segment SGI202540[] = {{&S83073[44], 5}, {NULL}};
const Segment SGI202541[] = {{&S83073[44], 17}, {NULL}};
const Segment SGI202542[] = {{&S83073[30], 6}, {NULL}};
const Segment SGI202543[] = {{&S83073[31], 3}, {NULL}};
const Segment SGI202544[] = {{&S83073[75], 6}, {NULL}};
const Segment SGI202545[] = {{&S83074[34], 7}, {NULL}};
const Segment SGI202546[] = {{&S83074[27], 6}, {NULL}};
const Segment SGI202547[] = {{&S83074[99], 9}, {NULL}};
const Segment SGI202548[] = {{&S83074[72], 6}, {NULL}};
const Segment SGI202549[] = {{&S83075[34], 7}, {NULL}};
const Segment SGI202550[] = {{&S83075[27], 6}, {NULL}};
const Segment SGI202551[] = {{&S77222[6], 4}, {NULL}};
const Segment SGI202552[] = {{&S83075[94], 9}, {NULL}};
const Segment SGI202553[] = {{&S77222[82], 5}, {NULL}};
const Segment SGI202554[] = {{&S83075[67], 6}, {NULL}};
const Segment SGI202555[] = {{&S83076[34], 7}, {NULL}};
const Segment SGI202556[] = {{&S83076[27], 6}, {NULL}};
const Segment SGI202557[] = {{&S83076[94], 9}, {NULL}};
const Segment SGI202558[] = {{&S83076[67], 6}, {NULL}};
const Segment SGI202559[] = {{&S83077[34], 7}, {NULL}};
const Segment SGI202560[] = {{&S83077[27], 6}, {NULL}};
const Segment SGI202561[] = {{&S83077[94], 9}, {NULL}};
const Segment SGI202562[] = {{&S83077[67], 6}, {NULL}};
const Segment SGI202563[] = {{&S77230[44], 8}, {NULL}};
const Segment SGI202564[] = {{&S77230[44], 20}, {NULL}};
const Segment SGI202565[] = {{&S77230[30], 6}, {NULL}};
const Segment SGI202566[] = {{&S77230[113], 3}, {NULL}};
const Segment SGI202567[] = {{&S77230[78], 6}, {NULL}};
const Segment SGI202568[] = {{&S83099[44], 7}, {NULL}};
const Segment SGI202569[] = {{&S83099[44], 19}, {NULL}};
const Segment SGI202570[] = {{&S83099[30], 6}, {NULL}};
const Segment SGI202571[] = {{&S83099[31], 3}, {NULL}};
const Segment SGI202572[] = {{&S83099[106], 6}, {NULL}};
const Segment SGI202573[] = {{&S83099[65], 8}, {NULL}};
const Segment SGI202574[] = {{&S83099[77], 15}, {NULL}};
const Segment SGI202575[] = {{&S83100[44], 7}, {NULL}};
const Segment SGI202576[] = {{&S83100[44], 19}, {NULL}};
const Segment SGI202577[] = {{&S83100[30], 6}, {NULL}};
const Segment SGI202578[] = {{&S83100[31], 3}, {NULL}};
const Segment SGI202579[] = {{&S83100[106], 6}, {NULL}};
const Segment SGI202580[] = {{&S83100[65], 8}, {NULL}};
const Segment SGI202581[] = {{&S83100[77], 15}, {NULL}};
const Segment SGI202582[] = {{&S83102[34], 7}, {NULL}};
const Segment SGI202583[] = {{&S83102[27], 6}, {NULL}};
const Segment SGI202584[] = {{&S83102[94], 9}, {NULL}};
const Segment SGI202585[] = {{&S83102[67], 6}, {NULL}};
const Segment SGI202586[] = {{&S83103[34], 6}, {NULL}};
const Segment SGI202587[] = {{&S83103[27], 6}, {NULL}};
const Segment SGI202588[] = {{&S83103[93], 9}, {NULL}};
const Segment SGI202589[] = {{&S83103[66], 6}, {NULL}};
const Segment SGI202590[] = {{&S71957[34], 18}, {NULL}};
const Segment SGI202591[] = {{&S71957[27], 6}, {NULL}};
const Segment SGI202592[] = {{&S71957[105], 9}, {NULL}};
const Segment SGI202593[] = {{&S71957[78], 6}, {NULL}};
const Segment SGI202594[] = {{&S83111[34], 7}, {NULL}};
const Segment SGI202595[] = {{&S83111[27], 6}, {NULL}};
const Segment SGI202596[] = {{&S83111[99], 9}, {NULL}};
const Segment SGI202597[] = {{&S83111[72], 6}, {NULL}};
const Segment SGI202598[] = {{&S83111[140], 8}, {NULL}};
const Segment SGI202599[] = {{&S83113[34], 15}, {NULL}};
const Segment SGI202600[] = {{&S83113[27], 6}, {NULL}};
const Segment SGI202601[] = {{&S83113[102], 9}, {NULL}};
const Segment SGI202602[] = {{&S83113[75], 6}, {NULL}};
const Segment SGI202603[] = {{&S83118[34], 12}, {NULL}};
const Segment SGI202604[] = {{&S83118[27], 6}, {NULL}};
const Segment SGI202605[] = {{&S83118[96], 9}, {NULL}};
const Segment SGI202606[] = {{&S83118[69], 6}, {NULL}};
const Segment SGI202607[] = {{&S83119[34], 4}, {NULL}};
const Segment SGI202608[] = {{&S83119[27], 6}, {NULL}};
const Segment SGI202609[] = {{&S83119[88], 9}, {NULL}};
const Segment SGI202610[] = {{&S83119[61], 6}, {NULL}};
const Segment SGI202611[] = {{&S83122[34], 9}, {NULL}};
const Segment SGI202612[] = {{&S83122[27], 6}, {NULL}};
const Segment SGI202613[] = {{&S83122[96], 9}, {NULL}};
const Segment SGI202614[] = {{&S83122[69], 6}, {NULL}};
const Segment SGI202615[] = {{&S72602[44], 31}, {NULL}};
const Segment SGI202616[] = {{&S72602[30], 6}, {NULL}};
const Segment SGI202617[] = {{&S72603[44], 14}, {NULL}};
const Segment SGI202618[] = {{&S72603[44], 26}, {NULL}};
const Segment SGI202619[] = {{&S72603[30], 6}, {NULL}};
const Segment SGI202620[] = {{&S72603[119], 3}, {NULL}};
const Segment SGI202621[] = {{&S72603[84], 6}, {NULL}};
const Segment SGI202622[] = {{&S72604[44], 14}, {NULL}};
const Segment SGI202623[] = {{&S72604[44], 26}, {NULL}};
const Segment SGI202624[] = {{&S72604[30], 6}, {NULL}};
const Segment SGI202625[] = {{&S72604[87], 6}, {NULL}};
const Segment SGI202626[] = {{&S72605[38], 14}, {NULL}};
const Segment SGI202627[] = {{&S72605[31], 6}, {NULL}};
const Segment SGI202628[] = {{&S72606[44], 12}, {NULL}};
const Segment SGI202629[] = {{&S72606[44], 24}, {NULL}};
const Segment SGI202630[] = {{&S72606[30], 6}, {NULL}};
const Segment SGI202631[] = {{&S77297[34], 22}, {NULL}};
const Segment SGI202632[] = {{&S72606[117], 3}, {NULL}};
const Segment SGI202633[] = {{&S77297[27], 6}, {NULL}};
const Segment SGI202634[] = {{&S72606[82], 6}, {NULL}};
const Segment SGI202635[] = {{&S77297[109], 9}, {NULL}};
const Segment SGI202636[] = {{&S77297[82], 6}, {NULL}};
const Segment SGI202637[] = {{&S72607[38], 12}, {NULL}};
const Segment SGI202638[] = {{&S72607[31], 6}, {NULL}};
const Segment SGI202639[] = {{&S83146[34], 10}, {NULL}};
const Segment SGI202640[] = {{&S83146[27], 6}, {NULL}};
const Segment SGI202641[] = {{&S83146[97], 9}, {NULL}};
const Segment SGI202642[] = {{&S83146[70], 6}, {NULL}};
const Segment SGI202643[] = {{&S77309[44], 13}, {NULL}};
const Segment SGI202644[] = {{&S77309[44], 25}, {NULL}};
const Segment SGI202645[] = {{&S77309[30], 6}, {NULL}};
const Segment SGI202646[] = {{&S77309[118], 3}, {NULL}};
const Segment SGI202647[] = {{&S77309[83], 6}, {NULL}};
const Segment SGI202648[] = {{&S83152[44], 8}, {NULL}};
const Segment SGI202649[] = {{&S83152[44], 20}, {NULL}};
const Segment SGI202650[] = {{&S77312[34], 16}, {NULL}};
const Segment SGI202651[] = {{&S83152[30], 6}, {NULL}};
const Segment SGI202652[] = {{&S77312[27], 6}, {NULL}};
const Segment SGI202653[] = {{&S83152[31], 3}, {NULL}};
const Segment SGI202654[] = {{&S83152[78], 6}, {NULL}};
const Segment SGI202655[] = {{&S77312[103], 9}, {NULL}};
const Segment SGI202656[] = {{&S77312[76], 6}, {NULL}};
const Segment SGI202657[] = {{&S72019[34], 10}, {NULL}};
const Segment SGI202658[] = {{&S72019[27], 6}, {NULL}};
const Segment SGI202659[] = {{&S72019[97], 9}, {NULL}};
const Segment SGI202660[] = {{&S72019[70], 6}, {NULL}};
const Segment SGI202661[] = {{&S72019[138], 8}, {NULL}};
const Segment SGI202662[] = {{&S83153[34], 8}, {NULL}};
const Segment SGI202663[] = {{&S83153[27], 6}, {NULL}};
const Segment SGI202664[] = {{&S83153[95], 9}, {NULL}};
const Segment SGI202665[] = {{&S72626[59], 10}, {NULL}};
const Segment SGI202666[] = {{&S83153[68], 6}, {NULL}};
const Segment SGI202667[] = {{&S72626[59], 22}, {NULL}};
const Segment SGI202668[] = {{&S72626[45], 6}, {NULL}};
const Segment SGI202669[] = {{&S72626[130], 3}, {NULL}};
const Segment SGI202670[] = {{&S72626[95], 6}, {NULL}};
const Segment SGI202671[] = {{&S72626[0], 14}, {NULL}};
const Segment SGI202672[] = {{&S83158[44], 6}, {NULL}};
const Segment SGI202673[] = {{&S83158[44], 18}, {NULL}};
const Segment SGI202674[] = {{&S83158[30], 6}, {NULL}};
const Segment SGI202675[] = {{&S83158[31], 3}, {NULL}};
const Segment SGI202676[] = {{&S83158[76], 6}, {NULL}};
const Segment SGI202677[] = {{&S83161[44], 14}, {NULL}};
const Segment SGI202678[] = {{&S83161[44], 26}, {NULL}};
const Segment SGI202679[] = {{&S83161[30], 6}, {NULL}};
const Segment SGI202680[] = {{&S83161[31], 3}, {NULL}};
const Segment SGI202681[] = {{&S83161[84], 6}, {NULL}};
const Segment SGI202682[] = {{&S83162[44], 9}, {NULL}};
const Segment SGI202683[] = {{&S83162[44], 21}, {NULL}};
const Segment SGI202684[] = {{&S83162[30], 6}, {NULL}};
const Segment SGI202685[] = {{&S83162[31], 3}, {NULL}};
const Segment SGI202686[] = {{&S83162[79], 6}, {NULL}};
const Segment SGI202687[] = {{&S83166[34], 11}, {NULL}};
const Segment SGI202688[] = {{&S83166[27], 6}, {NULL}};
const Segment SGI202689[] = {{&S83166[98], 9}, {NULL}};
const Segment SGI202690[] = {{&S83166[71], 6}, {NULL}};
const Segment SGI202691[] = {{&S83167[34], 6}, {NULL}};
const Segment SGI202692[] = {{&S83167[27], 6}, {NULL}};
const Segment SGI202693[] = {{&S83167[93], 9}, {NULL}};
const Segment SGI202694[] = {{&S83167[66], 6}, {NULL}};
const Segment SGI202695[] = {{&S77337[34], 10}, {NULL}};
const Segment SGI202696[] = {{&S77337[27], 6}, {NULL}};
const Segment SGI202697[] = {{&S77337[97], 9}, {NULL}};
const Segment SGI202698[] = {{&S77337[70], 6}, {NULL}};
const Segment SGI202699[] = {{&S83169[34], 9}, {NULL}};
const Segment SGI202700[] = {{&S83169[27], 6}, {NULL}};
const Segment SGI202701[] = {{&S83169[93], 9}, {NULL}};
const Segment SGI202702[] = {{&S83169[66], 6}, {NULL}};
const Segment SGI202703[] = {{&S83175[34], 11}, {NULL}};
const Segment SGI202704[] = {{&S83175[27], 6}, {NULL}};
const Segment SGI202705[] = {{&S83175[98], 9}, {NULL}};
const Segment SGI202706[] = {{&S83175[71], 6}, {NULL}};
const Segment SGI202707[] = {{&S83176[34], 11}, {NULL}};
const Segment SGI202708[] = {{&S77346[44], 19}, {NULL}};
const Segment SGI202709[] = {{&S77346[44], 31}, {NULL}};
const Segment SGI202710[] = {{&S83176[27], 6}, {NULL}};
const Segment SGI202711[] = {{&S77346[30], 6}, {NULL}};
const Segment SGI202712[] = {{&S77346[124], 3}, {NULL}};
const Segment SGI202713[] = {{&S83176[98], 9}, {NULL}};
const Segment SGI202714[] = {{&S83176[71], 6}, {NULL}};
const Segment SGI202715[] = {{&S77346[89], 6}, {NULL}};
const Segment SGI202716[] = {{&S83177[34], 11}, {NULL}};
const Segment SGI202717[] = {{&S77347[38], 19}, {NULL}};
const Segment SGI202718[] = {{&S83177[27], 6}, {NULL}};
const Segment SGI202719[] = {{&S77347[31], 6}, {NULL}};
const Segment SGI202720[] = {{&S83177[98], 9}, {NULL}};
const Segment SGI202721[] = {{&S83177[71], 6}, {NULL}};
const Segment SGI202722[] = {{&S72673[34], 11}, {NULL}};
const Segment SGI202723[] = {{&S72673[27], 6}, {NULL}};
const Segment SGI202724[] = {{&S72673[98], 9}, {NULL}};
const Segment SGI202725[] = {{&S72673[71], 6}, {NULL}};
const Segment SGI202726[] = {{&S72674[34], 10}, {NULL}};
const Segment SGI202727[] = {{&S72674[27], 6}, {NULL}};
const Segment SGI202728[] = {{&S72674[97], 9}, {NULL}};
const Segment SGI202729[] = {{&S77357[44], 10}, {NULL}};
const Segment SGI202730[] = {{&S72674[70], 6}, {NULL}};
const Segment SGI202731[] = {{&S77357[44], 22}, {NULL}};
const Segment SGI202732[] = {{&S77357[30], 6}, {NULL}};
const Segment SGI202733[] = {{&S77357[80], 6}, {NULL}};
const Segment SGI202734[] = {{&S77357[107], 13}, {NULL}};
const Segment SGI202735[] = {{&S72678[34], 25}, {NULL}};
const Segment SGI202736[] = {{&S72678[27], 6}, {NULL}};
const Segment SGI202737[] = {{&S72678[112], 9}, {NULL}};
const Segment SGI202738[] = {{&S72678[85], 6}, {NULL}};
const Segment SGI202739[] = {{&S72680[34], 5}, {NULL}};
const Segment SGI202740[] = {{&S72680[27], 6}, {NULL}};
const Segment SGI202741[] = {{&S72680[92], 9}, {NULL}};
const Segment SGI202742[] = {{&S72680[65], 6}, {NULL}};
const Segment SGI202743[] = {{&S83197[34], 7}, {NULL}};
const Segment SGI202744[] = {{&S83197[27], 6}, {NULL}};
const Segment SGI202745[] = {{&S83197[98], 9}, {NULL}};
const Segment SGI202746[] = {{&S83197[71], 6}, {NULL}};
const Segment SGI202747[] = {{&S83213[34], 10}, {NULL}};
const Segment SGI202748[] = {{&S83213[27], 6}, {NULL}};
const Segment SGI202749[] = {{&S72697[44], 7}, {NULL}};
const Segment SGI202750[] = {{&S72697[44], 19}, {NULL}};
const Segment SGI202751[] = {{&S72697[30], 6}, {NULL}};
const Segment SGI202752[] = {{&S83213[97], 9}, {NULL}};
const Segment SGI202753[] = {{&S83213[70], 6}, {NULL}};
const Segment SGI202754[] = {{&S72697[112], 3}, {NULL}};
const Segment SGI202755[] = {{&S72697[77], 6}, {NULL}};
const Segment SGI202756[] = {{&S72698[38], 7}, {NULL}};
const Segment SGI202757[] = {{&S72698[31], 6}, {NULL}};
const Segment SGI202758[] = {{&S72100[34], 10}, {NULL}};
const Segment SGI202759[] = {{&S83217[34], 8}, {NULL}};
const Segment SGI202760[] = {{&S72100[27], 6}, {NULL}};
const Segment SGI202761[] = {{&S83217[27], 6}, {NULL}};
const Segment SGI202762[] = {{&S72100[97], 9}, {NULL}};
const Segment SGI202763[] = {{&S83217[95], 9}, {NULL}};
const Segment SGI202764[] = {{&S72100[70], 6}, {NULL}};
const Segment SGI202765[] = {{&S83217[68], 6}, {NULL}};
const Segment SGI202766[] = {{&S83218[44], 11}, {NULL}};
const Segment SGI202767[] = {{&S83218[44], 23}, {NULL}};
const Segment SGI202768[] = {{&S83218[30], 6}, {NULL}};
const Segment SGI202769[] = {{&S83218[31], 3}, {NULL}};
const Segment SGI202770[] = {{&S83218[81], 6}, {NULL}};
const Segment SGI202771[] = {{&S83220[44], 10}, {NULL}};
const Segment SGI202772[] = {{&S83220[44], 27}, {NULL}};
const Segment SGI202773[] = {{&S83220[30], 6}, {NULL}};
const Segment SGI202774[] = {{&S83220[31], 3}, {NULL}};
const Segment SGI202775[] = {{&S72706[59], 10}, {NULL}};
const Segment SGI202776[] = {{&S83220[85], 6}, {NULL}};
const Segment SGI202777[] = {{&S72706[59], 22}, {NULL}};
const Segment SGI202778[] = {{&S72706[45], 6}, {NULL}};
const Segment SGI202779[] = {{&S72706[130], 3}, {NULL}};
const Segment SGI202780[] = {{&S72706[95], 6}, {NULL}};
const Segment SGI202781[] = {{&S72706[0], 14}, {NULL}};
const Segment SGI202782[] = {{&S83221[38], 10}, {NULL}};
const Segment SGI202783[] = {{&S83221[31], 6}, {NULL}};
const Segment SGI202784[] = {{&S83223[44], 10}, {NULL}};
const Segment SGI202785[] = {{&S83223[44], 22}, {NULL}};
const Segment SGI202786[] = {{&S83223[30], 6}, {NULL}};
const Segment SGI202787[] = {{&S83223[31], 3}, {NULL}};
const Segment SGI202788[] = {{&S83223[80], 6}, {NULL}};
const Segment SGI202789[] = {{&S83224[38], 10}, {NULL}};
const Segment SGI202790[] = {{&S83224[31], 6}, {NULL}};
const Segment SGI202791[] = {{&S83226[34], 7}, {NULL}};
const Segment SGI202792[] = {{&S83226[27], 6}, {NULL}};
const Segment SGI202793[] = {{&S83226[99], 9}, {NULL}};
const Segment SGI202794[] = {{&S83226[72], 6}, {NULL}};
const Segment SGI202795[] = {{&S83227[34], 7}, {NULL}};
const Segment SGI202796[] = {{&S83227[27], 6}, {NULL}};
const Segment SGI202797[] = {{&S83227[94], 9}, {NULL}};
const Segment SGI202798[] = {{&S83227[67], 6}, {NULL}};
const Segment SGI202799[] = {{&S77407[34], 9}, {NULL}};
const Segment SGI202800[] = {{&S77407[27], 6}, {NULL}};
const Segment SGI202801[] = {{&S77407[96], 9}, {NULL}};
const Segment SGI202802[] = {{&S77407[69], 6}, {NULL}};
const Segment SGI202803[] = {{&S77409[34], 5}, {NULL}};
const Segment SGI202804[] = {{&S77409[27], 6}, {NULL}};
const Segment SGI202805[] = {{&S83232[34], 15}, {NULL}};
const Segment SGI202806[] = {{&S83232[27], 6}, {NULL}};
const Segment SGI202807[] = {{&S72722[34], 15}, {NULL}};
const Segment SGI202808[] = {{&S77409[92], 9}, {NULL}};
const Segment SGI202809[] = {{&S77409[65], 6}, {NULL}};
const Segment SGI202810[] = {{&S72722[27], 6}, {NULL}};
const Segment SGI202811[] = {{&S83232[102], 9}, {NULL}};
const Segment SGI202812[] = {{&S77409[133], 8}, {NULL}};
const Segment SGI202813[] = {{&S83232[75], 6}, {NULL}};
const Segment SGI202814[] = {{&S72722[102], 9}, {NULL}};
const Segment SGI202815[] = {{&S72722[75], 6}, {NULL}};
const Segment SGI202816[] = {{&S72724[34], 10}, {NULL}};
const Segment SGI202817[] = {{&S72724[27], 6}, {NULL}};
const Segment SGI202818[] = {{&S72724[97], 9}, {NULL}};
const Segment SGI202819[] = {{&S72724[70], 6}, {NULL}};
const Segment SGI202820[] = {{&S72725[34], 10}, {NULL}};
const Segment SGI202821[] = {{&S72725[27], 6}, {NULL}};
const Segment SGI202822[] = {{&S72725[97], 9}, {NULL}};
const Segment SGI202823[] = {{&S72725[70], 6}, {NULL}};
const Segment SGI202824[] = {{&S72726[34], 10}, {NULL}};
const Segment SGI202825[] = {{&S72726[27], 6}, {NULL}};
const Segment SGI202826[] = {{&S72726[97], 9}, {NULL}};
const Segment SGI202827[] = {{&S72726[70], 6}, {NULL}};
const Segment SGI202828[] = {{&S83240[44], 11}, {NULL}};
const Segment SGI202829[] = {{&S83240[44], 23}, {NULL}};
const Segment SGI202830[] = {{&S83240[30], 6}, {NULL}};
const Segment SGI202831[] = {{&S83240[31], 3}, {NULL}};
const Segment SGI202832[] = {{&S83240[81], 5}, {NULL}};
const Segment SGI202833[] = {{&S83240[118], 14}, {NULL}};
const Segment SGI202834[] = {{&S83242[44], 10}, {NULL}};
const Segment SGI202835[] = {{&S83242[44], 22}, {NULL}};
const Segment SGI202836[] = {{&S83242[30], 6}, {NULL}};
const Segment SGI202837[] = {{&S83242[31], 3}, {NULL}};
const Segment SGI202838[] = {{&S83242[80], 5}, {NULL}};
const Segment SGI202839[] = {{&S83242[117], 14}, {NULL}};
const Segment SGI202840[] = {{&S63020[44], 10}, {NULL}};
const Segment SGI202841[] = {{&S63020[44], 21}, {NULL}};
const Segment SGI202842[] = {{&S63020[30], 6}, {NULL}};
const Segment SGI202843[] = {{&S83245[34], 10}, {NULL}};
const Segment SGI202844[] = {{&S63020[132], 3}, {NULL}};
const Segment SGI202845[] = {{&S83245[27], 6}, {NULL}};
const Segment SGI202846[] = {{&S63020[97], 6}, {NULL}};
const Segment SGI202847[] = {{&S63020[67], 8}, {NULL}};
const Segment SGI202848[] = {{&S83245[97], 9}, {NULL}};
const Segment SGI202849[] = {{&S83245[70], 6}, {NULL}};
const Segment SGI202850[] = {{&S63022[44], 10}, {NULL}};
const Segment SGI202851[] = {{&S63022[44], 22}, {NULL}};
const Segment SGI202852[] = {{&S63022[30], 6}, {NULL}};
const Segment SGI202853[] = {{&S63022[115], 3}, {NULL}};
const Segment SGI202854[] = {{&S63022[80], 6}, {NULL}};
const Segment SGI202855[] = {{&S63022[133], 8}, {NULL}};
const Segment SGI202856[] = {{&S83248[34], 19}, {NULL}};
const Segment SGI202857[] = {{&S83248[27], 6}, {NULL}};
const Segment SGI202858[] = {{&S83248[106], 9}, {NULL}};
const Segment SGI202859[] = {{&S83248[79], 6}, {NULL}};
const Segment SGI202860[] = {{&S77431[34], 10}, {NULL}};
const Segment SGI202861[] = {{&S77431[27], 6}, {NULL}};
const Segment SGI202862[] = {{&S83249[34], 18}, {NULL}};
const Segment SGI202863[] = {{&S83249[27], 6}, {NULL}};
const Segment SGI202864[] = {{&S77431[97], 9}, {NULL}};
const Segment SGI202865[] = {{&S77431[70], 6}, {NULL}};
const Segment SGI202866[] = {{&S83249[105], 9}, {NULL}};
const Segment SGI202867[] = {{&S83249[78], 6}, {NULL}};
const Segment SGI202868[] = {{&S83250[34], 5}, {NULL}};
const Segment SGI202869[] = {{&S83250[27], 6}, {NULL}};
const Segment SGI202870[] = {{&S83250[92], 9}, {NULL}};
const Segment SGI202871[] = {{&S83250[65], 6}, {NULL}};
const Segment SGI202872[] = {{&S83251[34], 10}, {NULL}};
const Segment SGI202873[] = {{&S83251[27], 6}, {NULL}};
const Segment SGI202874[] = {{&S83251[97], 9}, {NULL}};
const Segment SGI202875[] = {{&S83251[70], 6}, {NULL}};
const Segment SGI202876[] = {{&S72754[34], 7}, {NULL}};
const Segment SGI202877[] = {{&S72754[27], 6}, {NULL}};
const Segment SGI202878[] = {{&S72754[99], 9}, {NULL}};
const Segment SGI202879[] = {{&S72754[72], 6}, {NULL}};
const Segment SGI202880[] = {{&S72755[34], 7}, {NULL}};
const Segment SGI202881[] = {{&S72755[27], 6}, {NULL}};
const Segment SGI202882[] = {{&S72755[99], 9}, {NULL}};
const Segment SGI202883[] = {{&S72755[72], 6}, {NULL}};
const Segment SGI202884[] = {{&S77449[34], 15}, {NULL}};
const Segment SGI202885[] = {{&S77449[27], 6}, {NULL}};
const Segment SGI202886[] = {{&S77449[102], 9}, {NULL}};
const Segment SGI202887[] = {{&S77449[75], 6}, {NULL}};
const Segment SGI202888[] = {{&S77451[34], 12}, {NULL}};
const Segment SGI202889[] = {{&S77451[27], 6}, {NULL}};
const Segment SGI202890[] = {{&S77451[99], 9}, {NULL}};
const Segment SGI202891[] = {{&S77451[72], 6}, {NULL}};
const Segment SGI202892[] = {{&S77452[34], 4}, {NULL}};
const Segment SGI202893[] = {{&S77452[27], 6}, {NULL}};
const Segment SGI202894[] = {{&S77452[91], 9}, {NULL}};
const Segment SGI202895[] = {{&S77452[64], 6}, {NULL}};
const Segment SGI202896[] = {{&S72171[34], 7}, {NULL}};
const Segment SGI202897[] = {{&S72171[27], 6}, {NULL}};
const Segment SGI202898[] = {{&S72171[94], 9}, {NULL}};
const Segment SGI202899[] = {{&S72171[67], 6}, {NULL}};
const Segment SGI202900[] = {{&S77457[34], 6}, {NULL}};
const Segment SGI202901[] = {{&S77457[27], 6}, {NULL}};
const Segment SGI202902[] = {{&S77457[93], 9}, {NULL}};
const Segment SGI202903[] = {{&S77457[66], 6}, {NULL}};
const Segment SGI202904[] = {{&S63075[34], 10}, {NULL}};
const Segment SGI202905[] = {{&S63075[27], 6}, {NULL}};
const Segment SGI202906[] = {{&S63075[96], 9}, {NULL}};
const Segment SGI202907[] = {{&S63075[69], 6}, {NULL}};
const Segment SGI202908[] = {{&S63075[137], 8}, {NULL}};
const Segment SGI202909[] = {{&S72788[44], 10}, {NULL}};
const Segment SGI202910[] = {{&S72788[44], 22}, {NULL}};
const Segment SGI202911[] = {{&S72788[30], 6}, {NULL}};
const Segment SGI202912[] = {{&S72788[131], 3}, {NULL}};
const Segment SGI202913[] = {{&S72788[96], 6}, {NULL}};
const Segment SGI202914[] = {{&S72788[68], 8}, {NULL}};
const Segment SGI202915[] = {{&S72790[44], 10}, {NULL}};
const Segment SGI202916[] = {{&S72790[44], 22}, {NULL}};
const Segment SGI202917[] = {{&S72790[30], 6}, {NULL}};
const Segment SGI202918[] = {{&S72790[135], 3}, {NULL}};
const Segment SGI202919[] = {{&S72790[100], 6}, {NULL}};
const Segment SGI202920[] = {{&S72790[68], 8}, {NULL}};
const Segment SGI202921[] = {{&S72201[44], 11}, {NULL}};
const Segment SGI202922[] = {{&S72201[44], 23}, {NULL}};
const Segment SGI202923[] = {{&S72201[30], 6}, {NULL}};
const Segment SGI202924[] = {{&S72201[116], 3}, {NULL}};
const Segment SGI202925[] = {{&S72201[81], 6}, {NULL}};
const Segment SGI202926[] = {{&S63093[44], 7}, {NULL}};
const Segment SGI202927[] = {{&S63093[44], 18}, {NULL}};
const Segment SGI202928[] = {{&S63093[30], 6}, {NULL}};
const Segment SGI202929[] = {{&S63093[129], 3}, {NULL}};
const Segment SGI202930[] = {{&S63093[94], 6}, {NULL}};
const Segment SGI202931[] = {{&S63093[64], 8}, {NULL}};
const Segment SGI202932[] = {{&S63094[44], 7}, {NULL}};
const Segment SGI202933[] = {{&S63094[44], 19}, {NULL}};
const Segment SGI202934[] = {{&S63094[30], 6}, {NULL}};
const Segment SGI202935[] = {{&S63094[132], 3}, {NULL}};
const Segment SGI202936[] = {{&S63094[97], 6}, {NULL}};
const Segment SGI202937[] = {{&S63094[65], 8}, {NULL}};
const Segment SGI202938[] = {{&S72807[44], 10}, {NULL}};
const Segment SGI202939[] = {{&S72807[44], 22}, {NULL}};
const Segment SGI202940[] = {{&S72807[30], 6}, {NULL}};
const Segment SGI202941[] = {{&S72807[113], 3}, {NULL}};
const Segment SGI202942[] = {{&S72807[80], 5}, {NULL}};
const Segment SGI202943[] = {{&S72807[117], 14}, {NULL}};
const Segment SGI202944[] = {{&S72808[44], 10}, {NULL}};
const Segment SGI202945[] = {{&S72808[44], 22}, {NULL}};
const Segment SGI202946[] = {{&S72808[30], 6}, {NULL}};
const Segment SGI202947[] = {{&S83320[44], 8}, {NULL}};
const Segment SGI202948[] = {{&S83320[44], 20}, {NULL}};
const Segment SGI202949[] = {{&S83320[30], 6}, {NULL}};
const Segment SGI202950[] = {{&S72808[113], 3}, {NULL}};
const Segment SGI202951[] = {{&S72808[80], 5}, {NULL}};
const Segment SGI202952[] = {{&S83320[31], 3}, {NULL}};
const Segment SGI202953[] = {{&S72808[117], 14}, {NULL}};
const Segment SGI202954[] = {{&S83320[78], 6}, {NULL}};
const Segment SGI202955[] = {{&S72809[44], 11}, {NULL}};
const Segment SGI202956[] = {{&S72809[44], 23}, {NULL}};
const Segment SGI202957[] = {{&S72809[30], 6}, {NULL}};
const Segment SGI202958[] = {{&S72809[114], 3}, {NULL}};
const Segment SGI202959[] = {{&S72809[81], 5}, {NULL}};
const Segment SGI202960[] = {{&S72809[118], 14}, {NULL}};
const Segment SGI202961[] = {{&S83322[34], 7}, {NULL}};
const Segment SGI202962[] = {{&S83322[27], 6}, {NULL}};
const Segment SGI202963[] = {{&S83322[94], 9}, {NULL}};
const Segment SGI202964[] = {{&S83322[67], 6}, {NULL}};
const Segment SGI202965[] = {{&S72810[34], 7}, {NULL}};
const Segment SGI202966[] = {{&S72810[27], 6}, {NULL}};
const Segment SGI202967[] = {{&S72810[99], 9}, {NULL}};
const Segment SGI202968[] = {{&S72810[72], 6}, {NULL}};
const Segment SGI202969[] = {{&S83336[44], 12}, {NULL}};
const Segment SGI202970[] = {{&S83336[44], 24}, {NULL}};
const Segment SGI202971[] = {{&S83336[30], 6}, {NULL}};
const Segment SGI202972[] = {{&S83336[31], 3}, {NULL}};
const Segment SGI202973[] = {{&S83336[82], 6}, {NULL}};
const Segment SGI202974[] = {{&S83343[44], 14}, {NULL}};
const Segment SGI202975[] = {{&S83343[44], 26}, {NULL}};
const Segment SGI202976[] = {{&S83343[30], 6}, {NULL}};
const Segment SGI202977[] = {{&S83343[31], 3}, {NULL}};
const Segment SGI202978[] = {{&S83343[84], 6}, {NULL}};
const Segment SGI202979[] = {{&S83344[44], 6}, {NULL}};
const Segment SGI202980[] = {{&S83344[44], 18}, {NULL}};
const Segment SGI202981[] = {{&S83344[30], 6}, {NULL}};
const Segment SGI202982[] = {{&S83344[31], 3}, {NULL}};
const Segment SGI202983[] = {{&S83344[76], 6}, {NULL}};
const Segment SGI202984[] = {{&S83346[34], 12}, {NULL}};
const Segment SGI202985[] = {{&S83346[27], 6}, {NULL}};
const Segment SGI202986[] = {{&S83346[96], 9}, {NULL}};
const Segment SGI202987[] = {{&S83346[69], 6}, {NULL}};
const Segment SGI202988[] = {{&S83347[34], 4}, {NULL}};
const Segment SGI202989[] = {{&S83347[27], 6}, {NULL}};
const Segment SGI202990[] = {{&S77544[44], 10}, {NULL}};
const Segment SGI202991[] = {{&S77544[44], 22}, {NULL}};
const Segment SGI202992[] = {{&S77544[30], 6}, {NULL}};
const Segment SGI202993[] = {{&S83347[88], 9}, {NULL}};
const Segment SGI202994[] = {{&S83347[61], 6}, {NULL}};
const Segment SGI202995[] = {{&S77544[113], 3}, {NULL}};
const Segment SGI202996[] = {{&S77544[80], 5}, {NULL}};
const Segment SGI202997[] = {{&S77544[117], 14}, {NULL}};
const Segment SGI202998[] = {{&S83348[44], 19}, {NULL}};
const Segment SGI202999[] = {{&S83348[44], 31}, {NULL}};
const Segment SGI203000[] = {{&S83348[30], 6}, {NULL}};
const Segment SGI203001[] = {{&S83348[31], 3}, {NULL}};
const Segment SGI203002[] = {{&S83348[89], 6}, {NULL}};
const Segment SGI203003[] = {{&S72845[44], 10}, {NULL}};
const Segment SGI203004[] = {{&S72845[44], 22}, {NULL}};
const Segment SGI203005[] = {{&S77548[44], 11}, {NULL}};
const Segment SGI203006[] = {{&S72845[30], 6}, {NULL}};
const Segment SGI203007[] = {{&S77548[44], 23}, {NULL}};
const Segment SGI203008[] = {{&S83350[34], 9}, {NULL}};
const Segment SGI203009[] = {{&S77548[30], 6}, {NULL}};
const Segment SGI203010[] = {{&S83350[27], 6}, {NULL}};
const Segment SGI203011[] = {{&S72845[113], 3}, {NULL}};
const Segment SGI203012[] = {{&S77548[116], 3}, {NULL}};
const Segment SGI203013[] = {{&S72845[80], 5}, {NULL}};
const Segment SGI203014[] = {{&S77548[81], 6}, {NULL}};
const Segment SGI203015[] = {{&S83350[96], 9}, {NULL}};
const Segment SGI203016[] = {{&S83350[69], 6}, {NULL}};
const Segment SGI203017[] = {{&S72845[117], 14}, {NULL}};
const Segment SGI203018[] = {{&S77549[44], 11}, {NULL}};
const Segment SGI203019[] = {{&S77549[44], 23}, {NULL}};
const Segment SGI203020[] = {{&S77549[30], 6}, {NULL}};
const Segment SGI203021[] = {{&S77549[116], 3}, {NULL}};
const Segment SGI203022[] = {{&S77549[81], 6}, {NULL}};
const Segment SGI203023[] = {{&S72847[44], 11}, {NULL}};
const Segment SGI203024[] = {{&S72847[44], 23}, {NULL}};
const Segment SGI203025[] = {{&S72847[30], 6}, {NULL}};
const Segment SGI203026[] = {{&S72847[114], 3}, {NULL}};
const Segment SGI203027[] = {{&S72847[81], 5}, {NULL}};
const Segment SGI203028[] = {{&S72847[118], 14}, {NULL}};
const Segment SGI203029[] = {{&S77550[44], 11}, {NULL}};
const Segment SGI203030[] = {{&S77550[44], 23}, {NULL}};
const Segment SGI203031[] = {{&S77550[30], 6}, {NULL}};
const Segment SGI203032[] = {{&S77550[84], 6}, {NULL}};
const Segment SGI203033[] = {{&S77551[34], 15}, {NULL}};
const Segment SGI203034[] = {{&S83355[34], 13}, {NULL}};
const Segment SGI203035[] = {{&S77551[27], 6}, {NULL}};
const Segment SGI203036[] = {{&S83355[27], 6}, {NULL}};
const Segment SGI203037[] = {{&S77551[102], 9}, {NULL}};
const Segment SGI203038[] = {{&S77551[75], 6}, {NULL}};
const Segment SGI203039[] = {{&S83355[100], 9}, {NULL}};
const Segment SGI203040[] = {{&S83355[73], 6}, {NULL}};
const Segment SGI203041[] = {{&S63151[34], 10}, {NULL}};
const Segment SGI203042[] = {{&S63151[27], 6}, {NULL}};
const Segment SGI203043[] = {{&S63151[97], 9}, {NULL}};
const Segment SGI203044[] = {{&S63151[70], 6}, {NULL}};
const Segment SGI203045[] = {{&S83359[34], 10}, {NULL}};
const Segment SGI203046[] = {{&S83359[27], 6}, {NULL}};
const Segment SGI203047[] = {{&S83359[97], 9}, {NULL}};
const Segment SGI203048[] = {{&S83359[70], 6}, {NULL}};
const Segment SGI203049[] = {{&S63155[44], 14}, {NULL}};
const Segment SGI203050[] = {{&S63155[44], 25}, {NULL}};
const Segment SGI203051[] = {{&S63155[30], 6}, {NULL}};
const Segment SGI203052[] = {{&S63155[141], 3}, {NULL}};
const Segment SGI203053[] = {{&S63155[106], 6}, {NULL}};
const Segment SGI203054[] = {{&S63156[44], 14}, {NULL}};
const Segment SGI203055[] = {{&S63156[44], 25}, {NULL}};
const Segment SGI203056[] = {{&S63156[30], 6}, {NULL}};
const Segment SGI203057[] = {{&S63156[141], 3}, {NULL}};
const Segment SGI203058[] = {{&S63156[106], 6}, {NULL}};
const Segment SGI203059[] = {{&S63156[78], 9}, {NULL}};
const Segment SGI203060[] = {{&S63157[44], 14}, {NULL}};
const Segment SGI203061[] = {{&S63157[44], 26}, {NULL}};
const Segment SGI203062[] = {{&S63157[30], 6}, {NULL}};
const Segment SGI203063[] = {{&S63157[142], 3}, {NULL}};
const Segment SGI203064[] = {{&S63157[107], 6}, {NULL}};
const Segment SGI203065[] = {{&S63157[79], 9}, {NULL}};
const Segment SGI203066[] = {{&S83369[34], 15}, {NULL}};
const Segment SGI203067[] = {{&S83369[27], 6}, {NULL}};
const Segment SGI203068[] = {{&S83369[100], 9}, {NULL}};
const Segment SGI203069[] = {{&S83369[73], 6}, {NULL}};
const Segment SGI203070[] = {{&S77570[34], 7}, {NULL}};
const Segment SGI203071[] = {{&S77570[27], 6}, {NULL}};
const Segment SGI203072[] = {{&S77570[98], 9}, {NULL}};
const Segment SGI203073[] = {{&S77570[71], 6}, {NULL}};
const Segment SGI203074[] = {{&S83374[34], 10}, {NULL}};
const Segment SGI203075[] = {{&S83374[27], 6}, {NULL}};
const Segment SGI203076[] = {{&S83374[97], 9}, {NULL}};
const Segment SGI203077[] = {{&S83374[70], 6}, {NULL}};
const Segment SGI203078[] = {{&S83374[138], 8}, {NULL}};
const Segment SGI203079[] = {{&S83387[38], 10}, {NULL}};
const Segment SGI203080[] = {{&S83387[31], 6}, {NULL}};
const Segment SGI203081[] = {{&S83388[44], 10}, {NULL}};
const Segment SGI203082[] = {{&S83388[44], 22}, {NULL}};
const Segment SGI203083[] = {{&S83388[30], 6}, {NULL}};
const Segment SGI203084[] = {{&S83388[31], 3}, {NULL}};
const Segment SGI203085[] = {{&S83388[80], 6}, {NULL}};
const Segment SGI203086[] = {{&S83389[44], 10}, {NULL}};
const Segment SGI203087[] = {{&S83389[44], 22}, {NULL}};
const Segment SGI203088[] = {{&S83389[30], 6}, {NULL}};
const Segment SGI203089[] = {{&S83389[31], 3}, {NULL}};
const Segment SGI203090[] = {{&S83389[80], 6}, {NULL}};
const Segment SGI203091[] = {{&S77600[44], 21}, {NULL}};
const Segment SGI203092[] = {{&S77600[44], 33}, {NULL}};
const Segment SGI203093[] = {{&S77600[30], 6}, {NULL}};
const Segment SGI203094[] = {{&S77600[142], 3}, {NULL}};
const Segment SGI203095[] = {{&S77600[107], 6}, {NULL}};
const Segment SGI203096[] = {{&S77600[79], 8}, {NULL}};
const Segment SGI203097[] = {{&S77601[44], 21}, {NULL}};
const Segment SGI203098[] = {{&S77601[44], 33}, {NULL}};
const Segment SGI203099[] = {{&S77601[30], 6}, {NULL}};
const Segment SGI203100[] = {{&S77601[142], 3}, {NULL}};
const Segment SGI203101[] = {{&S77601[107], 6}, {NULL}};
const Segment SGI203102[] = {{&S77601[79], 8}, {NULL}};
const Segment SGI203103[] = {{&S83407[44], 18}, {NULL}};
const Segment SGI203104[] = {{&S83407[44], 30}, {NULL}};
const Segment SGI203105[] = {{&S83407[30], 6}, {NULL}};
const Segment SGI203106[] = {{&S83407[31], 3}, {NULL}};
const Segment SGI203107[] = {{&S83407[117], 6}, {NULL}};
const Segment SGI203108[] = {{&S83407[76], 8}, {NULL}};
const Segment SGI203109[] = {{&S83407[88], 15}, {NULL}};
const Segment SGI203110[] = {{&S83408[44], 18}, {NULL}};
const Segment SGI203111[] = {{&S83408[44], 30}, {NULL}};
const Segment SGI203112[] = {{&S83408[30], 6}, {NULL}};
const Segment SGI203113[] = {{&S83408[31], 3}, {NULL}};
const Segment SGI203114[] = {{&S83408[117], 6}, {NULL}};
const Segment SGI203115[] = {{&S83408[76], 8}, {NULL}};
const Segment SGI203116[] = {{&S83408[88], 15}, {NULL}};
const Segment SGI203117[] = {{&S77610[34], 14}, {NULL}};
const Segment SGI203118[] = {{&S77610[27], 6}, {NULL}};
const Segment SGI203119[] = {{&S77610[101], 9}, {NULL}};
const Segment SGI203120[] = {{&S77610[74], 6}, {NULL}};
const Segment SGI203121[] = {{&S77610[142], 8}, {NULL}};
const Segment SGI203122[] = {{&S83410[34], 8}, {NULL}};
const Segment SGI203123[] = {{&S83410[27], 6}, {NULL}};
const Segment SGI203124[] = {{&S83410[95], 9}, {NULL}};
const Segment SGI203125[] = {{&S83410[68], 6}, {NULL}};
const Segment SGI203126[] = {{&S83427[45], 7}, {NULL}};
const Segment SGI203127[] = {{&S83427[30], 6}, {NULL}};
const Segment SGI203128[] = {{&S83427[31], 3}, {NULL}};
const Segment SGI203129[] = {{&S83427[82], 5}, {NULL}};
const Segment SGI203130[] = {{&S83427[140], 13}, {NULL}};
const Segment SGI203131[] = {{&S77643[34], 12}, {NULL}};
const Segment SGI203132[] = {{&S77643[27], 6}, {NULL}};
const Segment SGI203133[] = {{&S77643[99], 9}, {NULL}};
const Segment SGI203134[] = {{&S77643[72], 6}, {NULL}};
const Segment SGI203135[] = {{&S77645[34], 17}, {NULL}};
const Segment SGI203136[] = {{&S77645[27], 6}, {NULL}};
const Segment SGI203137[] = {{&S83442[44], 12}, {NULL}};
const Segment SGI203138[] = {{&S77645[104], 9}, {NULL}};
const Segment SGI203139[] = {{&S83442[44], 24}, {NULL}};
const Segment SGI203140[] = {{&S77645[77], 6}, {NULL}};
const Segment SGI203141[] = {{&S83442[30], 6}, {NULL}};
const Segment SGI203142[] = {{&S83442[31], 3}, {NULL}};
const Segment SGI203143[] = {{&S83442[82], 6}, {NULL}};
const Segment SGI203144[] = {{&S83450[34], 7}, {NULL}};
const Segment SGI203145[] = {{&S83450[27], 6}, {NULL}};
const Segment SGI203146[] = {{&S83450[106], 9}, {NULL}};
const Segment SGI203147[] = {{&S83450[79], 6}, {NULL}};
const Segment SGI203148[] = {{&S83450[147], 8}, {NULL}};
const Segment SGI203149[] = {{&S83452[34], 13}, {NULL}};
const Segment SGI203150[] = {{&S83452[27], 6}, {NULL}};
const Segment SGI203151[] = {{&S83452[100], 9}, {NULL}};
const Segment SGI203152[] = {{&S83452[73], 6}, {NULL}};
const Segment SGI203153[] = {{&S83453[34], 9}, {NULL}};
const Segment SGI203154[] = {{&S83453[27], 6}, {NULL}};
const Segment SGI203155[] = {{&S83453[96], 9}, {NULL}};
const Segment SGI203156[] = {{&S83453[69], 6}, {NULL}};
const Segment SGI203157[] = {{&S72972[44], 7}, {NULL}};
const Segment SGI203158[] = {{&S72972[44], 25}, {NULL}};
const Segment SGI203159[] = {{&S72972[30], 6}, {NULL}};
const Segment SGI203160[] = {{&S72972[118], 3}, {NULL}};
const Segment SGI203161[] = {{&S72972[83], 6}, {NULL}};
const Segment SGI203162[] = {{&S72973[38], 7}, {NULL}};
const Segment SGI203163[] = {{&S72973[31], 6}, {NULL}};
const Segment SGI203164[] = {{&S83461[34], 5}, {NULL}};
const Segment SGI203165[] = {{&S83461[27], 6}, {NULL}};
const Segment SGI203166[] = {{&S83461[92], 9}, {NULL}};
const Segment SGI203167[] = {{&S83461[65], 6}, {NULL}};
const Segment SGI203168[] = {{&S83461[126], 8}, {NULL}};
const Segment SGI203169[] = {{&S83462[34], 5}, {NULL}};
const Segment SGI203170[] = {{&S83462[27], 6}, {NULL}};
const Segment SGI203171[] = {{&S83462[92], 9}, {NULL}};
const Segment SGI203172[] = {{&S83462[65], 6}, {NULL}};
const Segment SGI203173[] = {{&S83462[133], 8}, {NULL}};
const Segment SGI203174[] = {{&S83467[34], 18}, {NULL}};
const Segment SGI203175[] = {{&S83467[27], 6}, {NULL}};
const Segment SGI203176[] = {{&S83467[105], 9}, {NULL}};
const Segment SGI203177[] = {{&S83467[78], 6}, {NULL}};
const Segment SGI203178[] = {{&S83474[34], 6}, {NULL}};
const Segment SGI203179[] = {{&S83474[27], 6}, {NULL}};
const Segment SGI203180[] = {{&S83474[93], 9}, {NULL}};
const Segment SGI203181[] = {{&S83474[66], 6}, {NULL}};
const Segment SGI203182[] = {{&S83475[34], 18}, {NULL}};
const Segment SGI203183[] = {{&S83475[27], 6}, {NULL}};
const Segment SGI203184[] = {{&S83475[105], 9}, {NULL}};
const Segment SGI203185[] = {{&S83475[78], 6}, {NULL}};
const Segment SGI203186[] = {{&S77694[59], 10}, {NULL}};
const Segment SGI203187[] = {{&S77694[59], 22}, {NULL}};
const Segment SGI203188[] = {{&S77694[45], 6}, {NULL}};
const Segment SGI203189[] = {{&S77694[130], 3}, {NULL}};
const Segment SGI203190[] = {{&S77694[95], 6}, {NULL}};
const Segment SGI203191[] = {{&S77694[0], 14}, {NULL}};
const Segment SGI203192[] = {{&S77695[59], 10}, {NULL}};
const Segment SGI203193[] = {{&S77695[59], 22}, {NULL}};
const Segment SGI203194[] = {{&S77695[45], 6}, {NULL}};
const Segment SGI203195[] = {{&S77695[130], 3}, {NULL}};
const Segment SGI203196[] = {{&S77695[95], 6}, {NULL}};
const Segment SGI203197[] = {{&S77695[0], 14}, {NULL}};
const Segment SGI203198[] = {{&S77706[34], 6}, {NULL}};
const Segment SGI203199[] = {{&S77706[27], 6}, {NULL}};
const Segment SGI203200[] = {{&S77706[93], 9}, {NULL}};
const Segment SGI203201[] = {{&S77706[66], 6}, {NULL}};
const Segment SGI203202[] = {{&S73016[34], 18}, {NULL}};
const Segment SGI203203[] = {{&S73016[27], 6}, {NULL}};
const Segment SGI203204[] = {{&S73016[105], 9}, {NULL}};
const Segment SGI203205[] = {{&S73016[78], 6}, {NULL}};
const Segment SGI203206[] = {{&S73017[34], 18}, {NULL}};
const Segment SGI203207[] = {{&S73017[27], 6}, {NULL}};
const Segment SGI203208[] = {{&S73017[105], 9}, {NULL}};
const Segment SGI203209[] = {{&S73017[78], 6}, {NULL}};
const Segment SGI203210[] = {{&S77712[34], 10}, {NULL}};
const Segment SGI203211[] = {{&S77712[27], 6}, {NULL}};
const Segment SGI203212[] = {{&S77712[97], 9}, {NULL}};
const Segment SGI203213[] = {{&S77712[70], 6}, {NULL}};
const Segment SGI203214[] = {{&S77714[34], 14}, {NULL}};
const Segment SGI203215[] = {{&S77714[27], 6}, {NULL}};
const Segment SGI203216[] = {{&S77714[101], 9}, {NULL}};
const Segment SGI203217[] = {{&S77714[74], 6}, {NULL}};
const Segment SGI203218[] = {{&S63320[34], 6}, {NULL}};
const Segment SGI203219[] = {{&S63320[27], 6}, {NULL}};
const Segment SGI203220[] = {{&S63320[92], 9}, {NULL}};
const Segment SGI203221[] = {{&S63320[65], 6}, {NULL}};
const Segment SGI203222[] = {{&S77720[44], 13}, {NULL}};
const Segment SGI203223[] = {{&S77720[44], 25}, {NULL}};
const Segment SGI203224[] = {{&S77720[30], 6}, {NULL}};
const Segment SGI203225[] = {{&S77720[117], 3}, {NULL}};
const Segment SGI203226[] = {{&S77720[83], 5}, {NULL}};
const Segment SGI203227[] = {{&S77721[44], 13}, {NULL}};
const Segment SGI203228[] = {{&S77721[44], 25}, {NULL}};
const Segment SGI203229[] = {{&S77721[30], 6}, {NULL}};
const Segment SGI203230[] = {{&S77721[118], 3}, {NULL}};
const Segment SGI203231[] = {{&S77721[83], 6}, {NULL}};
const Segment SGI203232[] = {{&S83509[34], 9}, {NULL}};
const Segment SGI203233[] = {{&S83509[27], 6}, {NULL}};
const Segment SGI203234[] = {{&S83509[96], 9}, {NULL}};
const Segment SGI203235[] = {{&S83509[69], 6}, {NULL}};
const Segment SGI203236[] = {{&S63335[38], 11}, {NULL}};
const Segment SGI203237[] = {{&S63335[31], 6}, {NULL}};
const Segment SGI203238[] = {{&S73042[44], 9}, {NULL}};
const Segment SGI203239[] = {{&S73042[44], 21}, {NULL}};
const Segment SGI203240[] = {{&S73042[30], 6}, {NULL}};
const Segment SGI203241[] = {{&S73042[114], 3}, {NULL}};
const Segment SGI203242[] = {{&S73042[79], 6}, {NULL}};
const Segment SGI203243[] = {{&S73042[139], 7}, {NULL}};
const Segment SGI203244[] = {{&S63336[44], 11}, {NULL}};
const Segment SGI203245[] = {{&S63336[44], 22}, {NULL}};
const Segment SGI203246[] = {{&S63336[30], 6}, {NULL}};
const Segment SGI203247[] = {{&S63336[115], 3}, {NULL}};
const Segment SGI203248[] = {{&S63336[80], 6}, {NULL}};
const Segment SGI203249[] = {{&S73043[44], 8}, {NULL}};
const Segment SGI203250[] = {{&S73043[44], 20}, {NULL}};
const Segment SGI203251[] = {{&S73043[30], 6}, {NULL}};
const Segment SGI203252[] = {{&S73043[113], 3}, {NULL}};
const Segment SGI203253[] = {{&S73043[78], 6}, {NULL}};
const Segment SGI203254[] = {{&S73043[138], 7}, {NULL}};
const Segment SGI203255[] = {{&S83519[44], 9}, {NULL}};
const Segment SGI203256[] = {{&S83519[44], 21}, {NULL}};
const Segment SGI203257[] = {{&S83519[30], 6}, {NULL}};
const Segment SGI203258[] = {{&S83519[31], 3}, {NULL}};
const Segment SGI203259[] = {{&S83519[122], 6}, {NULL}};
const Segment SGI203260[] = {{&S83519[67], 8}, {NULL}};
const Segment SGI203261[] = {{&S73047[34], 10}, {NULL}};
const Segment SGI203262[] = {{&S73047[27], 6}, {NULL}};
const Segment SGI203263[] = {{&S73047[97], 9}, {NULL}};
const Segment SGI203264[] = {{&S73047[70], 6}, {NULL}};
const Segment SGI203265[] = {{&S83522[44], 15}, {NULL}};
const Segment SGI203266[] = {{&S83522[44], 25}, {NULL}};
const Segment SGI203267[] = {{&S83522[30], 6}, {NULL}};
const Segment SGI203268[] = {{&S73048[34], 10}, {NULL}};
const Segment SGI203269[] = {{&S73048[27], 6}, {NULL}};
const Segment SGI203270[] = {{&S83522[31], 3}, {NULL}};
const Segment SGI203271[] = {{&S83522[83], 6}, {NULL}};
const Segment SGI203272[] = {{&S73048[97], 9}, {NULL}};
const Segment SGI203273[] = {{&S73048[70], 6}, {NULL}};
const Segment SGI203274[] = {{&S73049[34], 10}, {NULL}};
const Segment SGI203275[] = {{&S73049[27], 6}, {NULL}};
const Segment SGI203276[] = {{&S73049[97], 9}, {NULL}};
const Segment SGI203277[] = {{&S73049[70], 6}, {NULL}};
const Segment SGI203278[] = {{&S83539[44], 10}, {NULL}};
const Segment SGI203279[] = {{&S83539[44], 30}, {NULL}};
const Segment SGI203280[] = {{&S73064[34], 7}, {NULL}};
const Segment SGI203281[] = {{&S83539[30], 6}, {NULL}};
const Segment SGI203282[] = {{&S73064[27], 6}, {NULL}};
const Segment SGI203283[] = {{&S83539[31], 3}, {NULL}};
const Segment SGI203284[] = {{&S83539[88], 6}, {NULL}};
const Segment SGI203285[] = {{&S73064[94], 9}, {NULL}};
const Segment SGI203286[] = {{&S73064[67], 6}, {NULL}};
const Segment SGI203287[] = {{&S83543[34], 7}, {NULL}};
const Segment SGI203288[] = {{&S83543[27], 6}, {NULL}};
const Segment SGI203289[] = {{&S83543[112], 9}, {NULL}};
const Segment SGI203290[] = {{&S83543[85], 6}, {NULL}};
const Segment SGI203291[] = {{&S83543[153], 8}, {NULL}};
const Segment SGI203292[] = {{&S83544[34], 7}, {NULL}};
const Segment SGI203293[] = {{&S83544[27], 6}, {NULL}};
const Segment SGI203294[] = {{&S83544[99], 9}, {NULL}};
const Segment SGI203295[] = {{&S83544[72], 6}, {NULL}};
const Segment SGI203296[] = {{&S83550[34], 7}, {NULL}};
const Segment SGI203297[] = {{&S83550[27], 6}, {NULL}};
const Segment SGI203298[] = {{&S83550[94], 9}, {NULL}};
const Segment SGI203299[] = {{&S83550[67], 6}, {NULL}};
const Segment SGI203300[] = {{&S77779[44], 8}, {NULL}};
const Segment SGI203301[] = {{&S77779[44], 20}, {NULL}};
const Segment SGI203302[] = {{&S77779[30], 6}, {NULL}};
const Segment SGI203303[] = {{&S77779[113], 3}, {NULL}};
const Segment SGI203304[] = {{&S77779[78], 6}, {NULL}};
const Segment SGI203305[] = {{&S73095[34], 11}, {NULL}};
const Segment SGI203306[] = {{&S73095[27], 6}, {NULL}};
const Segment SGI203307[] = {{&S73095[98], 9}, {NULL}};
const Segment SGI203308[] = {{&S73095[71], 6}, {NULL}};
const Segment SGI203309[] = {{&S77780[44], 10}, {NULL}};
const Segment SGI203310[] = {{&S77780[44], 22}, {NULL}};
const Segment SGI203311[] = {{&S77780[30], 6}, {NULL}};
const Segment SGI203312[] = {{&S81507[42], 12}, {NULL}};
const Segment SGI203313[] = {{&S81507[42], 24}, {NULL}};
const Segment SGI203314[] = {{&S77780[115], 3}, {NULL}};
const Segment SGI203315[] = {{&S81507[30], 4}, {NULL}};
const Segment SGI203316[] = {{&S77780[80], 6}, {NULL}};
const Segment SGI203317[] = {{&S77780[133], 11}, {NULL}};
const Segment SGI203318[] = {{&S81507[115], 3}, {NULL}};
const Segment SGI203319[] = {{&S81507[80], 6}, {NULL}};
const Segment SGI203320[] = {{&S77781[44], 10}, {NULL}};
const Segment SGI203321[] = {{&S81508[42], 8}, {NULL}};
const Segment SGI203322[] = {{&S77781[44], 22}, {NULL}};
const Segment SGI203323[] = {{&S81508[42], 23}, {NULL}};
const Segment SGI203324[] = {{&S77781[30], 6}, {NULL}};
const Segment SGI203325[] = {{&S81508[30], 4}, {NULL}};
const Segment SGI203326[] = {{&S81508[114], 3}, {NULL}};
const Segment SGI203327[] = {{&S77781[115], 3}, {NULL}};
const Segment SGI203328[] = {{&S81508[79], 6}, {NULL}};
const Segment SGI203329[] = {{&S77781[80], 6}, {NULL}};
const Segment SGI203330[] = {{&S77781[134], 12}, {NULL}};
const Segment SGI203331[] = {{&S63407[34], 10}, {NULL}};
const Segment SGI203332[] = {{&S63407[27], 6}, {NULL}};
const Segment SGI203333[] = {{&S63407[96], 9}, {NULL}};
const Segment SGI203334[] = {{&S63407[69], 6}, {NULL}};
const Segment SGI203335[] = {{&S77794[34], 11}, {NULL}};
const Segment SGI203336[] = {{&S77794[27], 6}, {NULL}};
const Segment SGI203337[] = {{&S83582[44], 7}, {NULL}};
const Segment SGI203338[] = {{&S73113[44], 14}, {NULL}};
const Segment SGI203339[] = {{&S83582[44], 19}, {NULL}};
const Segment SGI203340[] = {{&S73113[44], 26}, {NULL}};
const Segment SGI203341[] = {{&S83582[30], 6}, {NULL}};
const Segment SGI203342[] = {{&S77794[98], 9}, {NULL}};
const Segment SGI203343[] = {{&S73113[30], 6}, {NULL}};
const Segment SGI203344[] = {{&S77794[71], 6}, {NULL}};
const Segment SGI203345[] = {{&S83582[31], 3}, {NULL}};
const Segment SGI203346[] = {{&S73113[135], 3}, {NULL}};
const Segment SGI203347[] = {{&S83582[92], 6}, {NULL}};
const Segment SGI203348[] = {{&S73113[100], 6}, {NULL}};
const Segment SGI203349[] = {{&S83582[65], 8}, {NULL}};
const Segment SGI203350[] = {{&S73113[72], 8}, {NULL}};
const Segment SGI203351[] = {{&S83589[44], 11}, {NULL}};
const Segment SGI203352[] = {{&S83589[44], 23}, {NULL}};
const Segment SGI203353[] = {{&S83589[30], 6}, {NULL}};
const Segment SGI203354[] = {{&S83589[31], 3}, {NULL}};
const Segment SGI203355[] = {{&S83589[81], 6}, {NULL}};
const Segment SGI203356[] = {{&S73129[34], 7}, {NULL}};
const Segment SGI203357[] = {{&S73129[27], 6}, {NULL}};
const Segment SGI203358[] = {{&S73129[98], 9}, {NULL}};
const Segment SGI203359[] = {{&S73129[71], 6}, {NULL}};
const Segment SGI203360[] = {{&S83599[44], 10}, {NULL}};
const Segment SGI203361[] = {{&S83599[44], 22}, {NULL}};
const Segment SGI203362[] = {{&S83599[30], 6}, {NULL}};
const Segment SGI203363[] = {{&S83599[31], 3}, {NULL}};
const Segment SGI203364[] = {{&S83599[80], 6}, {NULL}};
const Segment SGI203365[] = {{&S73133[34], 13}, {NULL}};
const Segment SGI203366[] = {{&S73133[27], 6}, {NULL}};
const Segment SGI203367[] = {{&S73133[100], 9}, {NULL}};
const Segment SGI203368[] = {{&S73133[73], 6}, {NULL}};
const Segment SGI203369[] = {{&S73133[141], 8}, {NULL}};
const Segment SGI203370[] = {{&S73134[34], 14}, {NULL}};
const Segment SGI203371[] = {{&S73134[27], 6}, {NULL}};
const Segment SGI203372[] = {{&S73134[101], 9}, {NULL}};
const Segment SGI203373[] = {{&S73134[74], 6}, {NULL}};
const Segment SGI203374[] = {{&S73134[142], 8}, {NULL}};
const Segment SGI203375[] = {{&S73135[34], 10}, {NULL}};
const Segment SGI203376[] = {{&S73135[27], 6}, {NULL}};
const Segment SGI203377[] = {{&S73135[97], 9}, {NULL}};
const Segment SGI203378[] = {{&S73135[70], 6}, {NULL}};
const Segment SGI203379[] = {{&S83608[34], 7}, {NULL}};
const Segment SGI203380[] = {{&S83608[27], 6}, {NULL}};
const Segment SGI203381[] = {{&S73135[138], 8}, {NULL}};
const Segment SGI203382[] = {{&S83608[94], 9}, {NULL}};
const Segment SGI203383[] = {{&S83608[67], 6}, {NULL}};
const Segment SGI203384[] = {{&S83613[45], 13}, {NULL}};
const Segment SGI203385[] = {{&S83613[30], 6}, {NULL}};
const Segment SGI203386[] = {{&S83613[31], 3}, {NULL}};
const Segment SGI203387[] = {{&S83613[84], 5}, {NULL}};
const Segment SGI203388[] = {{&S83613[142], 13}, {NULL}};
const Segment SGI203389[] = {{&S83615[44], 11}, {NULL}};
const Segment SGI203390[] = {{&S83615[44], 23}, {NULL}};
const Segment SGI203391[] = {{&S83615[30], 6}, {NULL}};
const Segment SGI203392[] = {{&S83615[31], 3}, {NULL}};
const Segment SGI203393[] = {{&S83615[81], 6}, {NULL}};
const Segment SGI203394[] = {{&S77836[44], 29}, {NULL}};
const Segment SGI203395[] = {{&S77836[44], 41}, {NULL}};
const Segment SGI203396[] = {{&S77836[30], 6}, {NULL}};
const Segment SGI203397[] = {{&S77836[134], 3}, {NULL}};
const Segment SGI203398[] = {{&S77836[99], 6}, {NULL}};
const Segment SGI203399[] = {{&S83616[44], 11}, {NULL}};
const Segment SGI203400[] = {{&S83616[44], 23}, {NULL}};
const Segment SGI203401[] = {{&S83616[30], 6}, {NULL}};
const Segment SGI203402[] = {{&S83616[31], 3}, {NULL}};
const Segment SGI203403[] = {{&S83616[81], 6}, {NULL}};
const Segment SGI203404[] = {{&S77837[44], 41}, {NULL}};
const Segment SGI203405[] = {{&S77837[44], 53}, {NULL}};
const Segment SGI203406[] = {{&S77837[30], 6}, {NULL}};
const Segment SGI203407[] = {{&S77837[146], 3}, {NULL}};
const Segment SGI203408[] = {{&S77837[111], 6}, {NULL}};
const Segment SGI203409[] = {{&S77840[44], 10}, {NULL}};
const Segment SGI203410[] = {{&S77840[44], 22}, {NULL}};
const Segment SGI203411[] = {{&S77840[30], 6}, {NULL}};
const Segment SGI203412[] = {{&S77840[115], 3}, {NULL}};
const Segment SGI203413[] = {{&S77840[80], 6}, {NULL}};
const Segment SGI203414[] = {{&S77840[140], 11}, {NULL}};
const Segment SGI203415[] = {{&S77841[44], 10}, {NULL}};
const Segment SGI203416[] = {{&S77841[44], 22}, {NULL}};
const Segment SGI203417[] = {{&S77841[30], 6}, {NULL}};
const Segment SGI203418[] = {{&S77841[115], 3}, {NULL}};
const Segment SGI203419[] = {{&S77841[80], 6}, {NULL}};
const Segment SGI203420[] = {{&S77841[133], 11}, {NULL}};
const Segment SGI203421[] = {{&S77842[44], 10}, {NULL}};
const Segment SGI203422[] = {{&S77842[44], 22}, {NULL}};
const Segment SGI203423[] = {{&S77842[30], 6}, {NULL}};
const Segment SGI203424[] = {{&S77842[115], 3}, {NULL}};
const Segment SGI203425[] = {{&S77842[80], 6}, {NULL}};
const Segment SGI203426[] = {{&S77842[134], 12}, {NULL}};
const Segment SGI203427[] = {{&S77843[34], 6}, {NULL}};
const Segment SGI203428[] = {{&S77843[27], 6}, {NULL}};
const Segment SGI203429[] = {{&S77843[93], 9}, {NULL}};
const Segment SGI203430[] = {{&S77843[66], 6}, {NULL}};
const Segment SGI203431[] = {{&S83629[34], 18}, {NULL}};
const Segment SGI203432[] = {{&S83629[27], 6}, {NULL}};
const Segment SGI203433[] = {{&S83629[105], 9}, {NULL}};
const Segment SGI203434[] = {{&S83629[78], 6}, {NULL}};
const Segment SGI203435[] = {{&S83629[146], 8}, {NULL}};
const Segment SGI203436[] = {{&S73162[34], 10}, {NULL}};
const Segment SGI203437[] = {{&S73162[27], 6}, {NULL}};
const Segment SGI203438[] = {{&S73162[97], 9}, {NULL}};
const Segment SGI203439[] = {{&S73162[70], 6}, {NULL}};
const Segment SGI203440[] = {{&S73163[34], 11}, {NULL}};
const Segment SGI203441[] = {{&S73163[27], 6}, {NULL}};
const Segment SGI203442[] = {{&S73163[98], 9}, {NULL}};
const Segment SGI203443[] = {{&S73163[71], 6}, {NULL}};
const Segment SGI203444[] = {{&S83632[44], 10}, {NULL}};
const Segment SGI203445[] = {{&S83632[44], 22}, {NULL}};
const Segment SGI203446[] = {{&S83632[30], 6}, {NULL}};
const Segment SGI203447[] = {{&S83632[31], 3}, {NULL}};
const Segment SGI203448[] = {{&S83632[80], 6}, {NULL}};
const Segment SGI203449[] = {{&S83632[139], 10}, {NULL}};
const Segment SGI203450[] = {{&S73164[34], 7}, {NULL}};
const Segment SGI203451[] = {{&S73164[27], 6}, {NULL}};
const Segment SGI203452[] = {{&S73164[99], 9}, {NULL}};
const Segment SGI203453[] = {{&S73164[72], 6}, {NULL}};
const Segment SGI203454[] = {{&S72236[34], 6}, {NULL}};
const Segment SGI203455[] = {{&S83643[34], 11}, {NULL}};
const Segment SGI203456[] = {{&S72236[27], 6}, {NULL}};
const Segment SGI203457[] = {{&S83643[27], 6}, {NULL}};
const Segment SGI203458[] = {{&S72236[93], 9}, {NULL}};
const Segment SGI203459[] = {{&S83643[98], 9}, {NULL}};
const Segment SGI203460[] = {{&S72236[66], 6}, {NULL}};
const Segment SGI203461[] = {{&S83643[71], 6}, {NULL}};
const Segment SGI203462[] = {{&S83653[34], 10}, {NULL}};
const Segment SGI203463[] = {{&S83653[27], 6}, {NULL}};
const Segment SGI203464[] = {{&S83653[97], 9}, {NULL}};
const Segment SGI203465[] = {{&S83653[70], 6}, {NULL}};
const Segment SGI203466[] = {{&S83662[44], 10}, {NULL}};
const Segment SGI203467[] = {{&S83662[44], 22}, {NULL}};
const Segment SGI203468[] = {{&S83662[30], 6}, {NULL}};
const Segment SGI203469[] = {{&S83662[115], 3}, {NULL}};
const Segment SGI203470[] = {{&S83662[80], 6}, {NULL}};
const Segment SGI203471[] = {{&S83662[135], 8}, {NULL}};
const Segment SGI203472[] = {{&S83662[144], 13}, {NULL}};
const Segment SGI203473[] = {{&S83664[34], 7}, {NULL}};
const Segment SGI203474[] = {{&S83664[27], 6}, {NULL}};
const Segment SGI203475[] = {{&S77889[44], 10}, {NULL}};
const Segment SGI203476[] = {{&S77889[44], 22}, {NULL}};
const Segment SGI203477[] = {{&S77889[30], 6}, {NULL}};
const Segment SGI203478[] = {{&S83664[94], 9}, {NULL}};
const Segment SGI203479[] = {{&S83664[67], 6}, {NULL}};
const Segment SGI203480[] = {{&S77889[131], 3}, {NULL}};
const Segment SGI203481[] = {{&S77889[96], 6}, {NULL}};
const Segment SGI203482[] = {{&S77889[68], 8}, {NULL}};
const Segment SGI203483[] = {{&S77890[44], 10}, {NULL}};
const Segment SGI203484[] = {{&S77890[44], 22}, {NULL}};
const Segment SGI203485[] = {{&S77890[30], 6}, {NULL}};
const Segment SGI203486[] = {{&S73203[34], 10}, {NULL}};
const Segment SGI203487[] = {{&S77890[131], 3}, {NULL}};
const Segment SGI203488[] = {{&S73203[27], 6}, {NULL}};
const Segment SGI203489[] = {{&S77890[96], 6}, {NULL}};
const Segment SGI203490[] = {{&S77890[68], 8}, {NULL}};
const Segment SGI203491[] = {{&S73203[97], 9}, {NULL}};
const Segment SGI203492[] = {{&S73203[70], 6}, {NULL}};
const Segment SGI203493[] = {{&S77891[44], 10}, {NULL}};
const Segment SGI203494[] = {{&S77891[44], 22}, {NULL}};
const Segment SGI203495[] = {{&S77891[30], 6}, {NULL}};
const Segment SGI203496[] = {{&S73204[34], 10}, {NULL}};
const Segment SGI203497[] = {{&S73204[27], 6}, {NULL}};
const Segment SGI203498[] = {{&S77891[131], 3}, {NULL}};
const Segment SGI203499[] = {{&S77891[96], 6}, {NULL}};
const Segment SGI203500[] = {{&S73204[97], 9}, {NULL}};
const Segment SGI203501[] = {{&S77891[68], 8}, {NULL}};
const Segment SGI203502[] = {{&S73204[70], 6}, {NULL}};
const Segment SGI203503[] = {{&S72266[44], 11}, {NULL}};
const Segment SGI203504[] = {{&S72266[44], 23}, {NULL}};
const Segment SGI203505[] = {{&S72266[30], 6}, {NULL}};
const Segment SGI203506[] = {{&S77892[44], 10}, {NULL}};
const Segment SGI203507[] = {{&S77892[44], 22}, {NULL}};
const Segment SGI203508[] = {{&S77892[30], 6}, {NULL}};
const Segment SGI203509[] = {{&S73205[34], 10}, {NULL}};
const Segment SGI203510[] = {{&S72266[116], 3}, {NULL}};
const Segment SGI203511[] = {{&S73205[27], 6}, {NULL}};
const Segment SGI203512[] = {{&S72266[81], 6}, {NULL}};
const Segment SGI203513[] = {{&S77892[131], 3}, {NULL}};
const Segment SGI203514[] = {{&S77892[96], 6}, {NULL}};
const Segment SGI203515[] = {{&S73205[97], 9}, {NULL}};
const Segment SGI203516[] = {{&S73205[70], 6}, {NULL}};
const Segment SGI203517[] = {{&S77892[68], 8}, {NULL}};
const Segment SGI203518[] = {{&S72267[38], 11}, {NULL}};
const Segment SGI203519[] = {{&S72267[31], 6}, {NULL}};
const Segment SGI203520[] = {{&S73206[34], 11}, {NULL}};
const Segment SGI203521[] = {{&S77893[44], 10}, {NULL}};
const Segment SGI203522[] = {{&S73206[27], 6}, {NULL}};
const Segment SGI203523[] = {{&S77893[44], 22}, {NULL}};
const Segment SGI203524[] = {{&S77893[30], 6}, {NULL}};
const Segment SGI203525[] = {{&S73206[98], 9}, {NULL}};
const Segment SGI203526[] = {{&S73206[71], 6}, {NULL}};
const Segment SGI203527[] = {{&S77893[131], 3}, {NULL}};
const Segment SGI203528[] = {{&S77893[96], 6}, {NULL}};
const Segment SGI203529[] = {{&S77893[68], 8}, {NULL}};
const Segment SGI203530[] = {{&S73207[34], 13}, {NULL}};
const Segment SGI203531[] = {{&S73207[27], 6}, {NULL}};
const Segment SGI203532[] = {{&S77894[44], 9}, {NULL}};
const Segment SGI203533[] = {{&S77894[44], 21}, {NULL}};
const Segment SGI203534[] = {{&S77894[30], 6}, {NULL}};
const Segment SGI203535[] = {{&S73207[100], 9}, {NULL}};
const Segment SGI203536[] = {{&S73207[73], 6}, {NULL}};
const Segment SGI203537[] = {{&S77894[114], 3}, {NULL}};
const Segment SGI203538[] = {{&S77894[79], 6}, {NULL}};
const Segment SGI203539[] = {{&S83677[44], 8}, {NULL}};
const Segment SGI203540[] = {{&S83677[44], 19}, {NULL}};
const Segment SGI203541[] = {{&S83677[30], 6}, {NULL}};
const Segment SGI203542[] = {{&S83677[31], 3}, {NULL}};
const Segment SGI203543[] = {{&S83677[77], 6}, {NULL}};
const Segment SGI203544[] = {{&S77897[44], 15}, {NULL}};
const Segment SGI203545[] = {{&S77897[44], 27}, {NULL}};
const Segment SGI203546[] = {{&S77897[30], 6}, {NULL}};
const Segment SGI203547[] = {{&S77897[120], 3}, {NULL}};
const Segment SGI203548[] = {{&S77897[85], 6}, {NULL}};
const Segment SGI203549[] = {{&S83682[34], 9}, {NULL}};
const Segment SGI203550[] = {{&S83682[27], 6}, {NULL}};
const Segment SGI203551[] = {{&S83682[96], 9}, {NULL}};
const Segment SGI203552[] = {{&S83682[69], 6}, {NULL}};
const Segment SGI203553[] = {{&S77900[34], 10}, {NULL}};
const Segment SGI203554[] = {{&S77900[27], 6}, {NULL}};
const Segment SGI203555[] = {{&S77900[97], 9}, {NULL}};
const Segment SGI203556[] = {{&S77900[70], 6}, {NULL}};
const Segment SGI203557[] = {{&S83686[59], 8}, {NULL}};
const Segment SGI203558[] = {{&S83686[59], 24}, {NULL}};
const Segment SGI203559[] = {{&S83686[45], 6}, {NULL}};
const Segment SGI203560[] = {{&S83686[46], 3}, {NULL}};
const Segment SGI203561[] = {{&S83686[97], 6}, {NULL}};
const Segment SGI203562[] = {{&S83686[0], 14}, {NULL}};
const Segment SGI203563[] = {{&S83687[59], 7}, {NULL}};
const Segment SGI203564[] = {{&S83687[59], 24}, {NULL}};
const Segment SGI203565[] = {{&S83687[45], 6}, {NULL}};
const Segment SGI203566[] = {{&S83687[46], 3}, {NULL}};
const Segment SGI203567[] = {{&S83687[97], 6}, {NULL}};
const Segment SGI203568[] = {{&S83687[0], 14}, {NULL}};
const Segment SGI203569[] = {{&S83692[45], 5}, {NULL}};
const Segment SGI203570[] = {{&S83692[30], 6}, {NULL}};
const Segment SGI203571[] = {{&S77911[34], 5}, {NULL}};
const Segment SGI203572[] = {{&S83692[31], 3}, {NULL}};
const Segment SGI203573[] = {{&S77911[27], 6}, {NULL}};
const Segment SGI203574[] = {{&S83692[76], 5}, {NULL}};
const Segment SGI203575[] = {{&S83692[134], 13}, {NULL}};
const Segment SGI203576[] = {{&S77911[92], 9}, {NULL}};
const Segment SGI203577[] = {{&S77911[65], 6}, {NULL}};
const Segment SGI203578[] = {{&S77911[133], 8}, {NULL}};
const Segment SGI203579[] = {{&S83693[45], 13}, {NULL}};
const Segment SGI203580[] = {{&S83693[30], 6}, {NULL}};
const Segment SGI203581[] = {{&S83693[31], 3}, {NULL}};
const Segment SGI203582[] = {{&S83693[84], 5}, {NULL}};
const Segment SGI203583[] = {{&S83693[142], 13}, {NULL}};
const Segment SGI203584[] = {{&S77914[34], 25}, {NULL}};
const Segment SGI203585[] = {{&S77914[27], 6}, {NULL}};
const Segment SGI203586[] = {{&S77914[112], 9}, {NULL}};
const Segment SGI203587[] = {{&S77914[85], 6}, {NULL}};
const Segment SGI203588[] = {{&S77915[34], 25}, {NULL}};
const Segment SGI203589[] = {{&S77915[27], 6}, {NULL}};
const Segment SGI203590[] = {{&S77915[112], 9}, {NULL}};
const Segment SGI203591[] = {{&S77915[85], 6}, {NULL}};
const Segment SGI203592[] = {{&S77918[34], 21}, {NULL}};
const Segment SGI203593[] = {{&S77918[27], 6}, {NULL}};
const Segment SGI203594[] = {{&S73239[44], 11}, {NULL}};
const Segment SGI203595[] = {{&S73239[44], 23}, {NULL}};
const Segment SGI203596[] = {{&S73239[30], 6}, {NULL}};
const Segment SGI203597[] = {{&S77918[108], 9}, {NULL}};
const Segment SGI203598[] = {{&S77918[81], 6}, {NULL}};
const Segment SGI203599[] = {{&S73239[116], 3}, {NULL}};
const Segment SGI203600[] = {{&S73239[81], 6}, {NULL}};
const Segment SGI203601[] = {{&S73240[44], 84}, {NULL}};
const Segment SGI203602[] = {{&S73240[44], 96}, {NULL}};
const Segment SGI203603[] = {{&S73240[30], 6}, {NULL}};
const Segment SGI203604[] = {{&S73240[189], 3}, {NULL}};
const Segment SGI203605[] = {{&S73240[121], 6}, {NULL}};
const Segment SGI203606[] = {{&S83705[44], 16}, {NULL}};
const Segment SGI203607[] = {{&S83705[44], 28}, {NULL}};
const Segment SGI203608[] = {{&S83705[30], 6}, {NULL}};
const Segment SGI203609[] = {{&S83705[31], 3}, {NULL}};
const Segment SGI203610[] = {{&S83705[86], 6}, {NULL}};
const Segment SGI203611[] = {{&S73241[38], 11}, {NULL}};
const Segment SGI203612[] = {{&S73241[31], 6}, {NULL}};
const Segment SGI203613[] = {{&S83706[45], 13}, {NULL}};
const Segment SGI203614[] = {{&S83706[30], 6}, {NULL}};
const Segment SGI203615[] = {{&S73242[34], 7}, {NULL}};
const Segment SGI203616[] = {{&S73242[27], 6}, {NULL}};
const Segment SGI203617[] = {{&S83706[31], 3}, {NULL}};
const Segment SGI203618[] = {{&S83706[84], 5}, {NULL}};
const Segment SGI203619[] = {{&S73242[99], 9}, {NULL}};
const Segment SGI203620[] = {{&S83706[142], 13}, {NULL}};
const Segment SGI203621[] = {{&S73242[72], 6}, {NULL}};
const Segment SGI203622[] = {{&S73242[140], 8}, {NULL}};
const Segment SGI203623[] = {{&S73243[34], 11}, {NULL}};
const Segment SGI203624[] = {{&S73243[27], 6}, {NULL}};
const Segment SGI203625[] = {{&S73243[98], 9}, {NULL}};
const Segment SGI203626[] = {{&S73243[71], 6}, {NULL}};
const Segment SGI203627[] = {{&S73244[34], 11}, {NULL}};
const Segment SGI203628[] = {{&S73244[27], 6}, {NULL}};
const Segment SGI203629[] = {{&S73244[98], 9}, {NULL}};
const Segment SGI203630[] = {{&S72311[34], 10}, {NULL}};
const Segment SGI203631[] = {{&S73244[71], 6}, {NULL}};
const Segment SGI203632[] = {{&S72311[27], 6}, {NULL}};
const Segment SGI203633[] = {{&S83711[34], 7}, {NULL}};
const Segment SGI203634[] = {{&S72311[97], 9}, {NULL}};
const Segment SGI203635[] = {{&S72311[70], 6}, {NULL}};
const Segment SGI203636[] = {{&S83711[27], 6}, {NULL}};
const Segment SGI203637[] = {{&S83711[94], 9}, {NULL}};
const Segment SGI203638[] = {{&S83711[67], 6}, {NULL}};
const Segment SGI203639[] = {{&S83716[34], 9}, {NULL}};
const Segment SGI203640[] = {{&S83716[27], 6}, {NULL}};
const Segment SGI203641[] = {{&S83716[96], 9}, {NULL}};
const Segment SGI203642[] = {{&S83716[69], 6}, {NULL}};
const Segment SGI203643[] = {{&S83719[34], 7}, {NULL}};
const Segment SGI203644[] = {{&S83719[27], 6}, {NULL}};
const Segment SGI203645[] = {{&S83719[99], 9}, {NULL}};
const Segment SGI203646[] = {{&S83719[72], 6}, {NULL}};
const Segment SGI203647[] = {{&S83719[140], 8}, {NULL}};
const Segment SGI203648[] = {{&S73256[34], 11}, {NULL}};
const Segment SGI203649[] = {{&S73256[27], 6}, {NULL}};
const Segment SGI203650[] = {{&S73256[98], 9}, {NULL}};
const Segment SGI203651[] = {{&S73256[71], 6}, {NULL}};
const Segment SGI203652[] = {{&S73260[44], 11}, {NULL}};
const Segment SGI203653[] = {{&S73260[44], 23}, {NULL}};
const Segment SGI203654[] = {{&S73260[30], 6}, {NULL}};
const Segment SGI203655[] = {{&S73260[116], 3}, {NULL}};
const Segment SGI203656[] = {{&S73260[81], 6}, {NULL}};
const Segment SGI203657[] = {{&S73261[44], 7}, {NULL}};
const Segment SGI203658[] = {{&S73261[44], 19}, {NULL}};
const Segment SGI203659[] = {{&S73261[30], 6}, {NULL}};
const Segment SGI203660[] = {{&S73261[112], 3}, {NULL}};
const Segment SGI203661[] = {{&S73261[77], 6}, {NULL}};
const Segment SGI203662[] = {{&S73262[38], 7}, {NULL}};
const Segment SGI203663[] = {{&S73262[31], 6}, {NULL}};
const Segment SGI203664[] = {{&S83730[34], 7}, {NULL}};
const Segment SGI203665[] = {{&S83730[27], 6}, {NULL}};
const Segment SGI203666[] = {{&S83730[99], 9}, {NULL}};
const Segment SGI203667[] = {{&S83730[72], 6}, {NULL}};
const Segment SGI203668[] = {{&S83731[44], 7}, {NULL}};
const Segment SGI203669[] = {{&S83731[44], 23}, {NULL}};
const Segment SGI203670[] = {{&S83731[30], 6}, {NULL}};
const Segment SGI203671[] = {{&S83731[81], 6}, {NULL}};
const Segment SGI203672[] = {{&S83731[108], 13}, {NULL}};
const Segment SGI203673[] = {{&S83732[44], 7}, {NULL}};
const Segment SGI203674[] = {{&S83732[44], 23}, {NULL}};
const Segment SGI203675[] = {{&S83732[30], 6}, {NULL}};
const Segment SGI203676[] = {{&S83732[81], 6}, {NULL}};
const Segment SGI203677[] = {{&S72338[34], 15}, {NULL}};
const Segment SGI203678[] = {{&S83732[108], 13}, {NULL}};
const Segment SGI203679[] = {{&S72338[27], 6}, {NULL}};
const Segment SGI203680[] = {{&S72338[102], 9}, {NULL}};
const Segment SGI203681[] = {{&S72338[75], 6}, {NULL}};
const Segment SGI203682[] = {{&S72339[34], 6}, {NULL}};
const Segment SGI203683[] = {{&S72339[27], 6}, {NULL}};
const Segment SGI203684[] = {{&S72339[93], 9}, {NULL}};
const Segment SGI203685[] = {{&S72339[66], 6}, {NULL}};
const Segment SGI203686[] = {{&S83736[34], 13}, {NULL}};
const Segment SGI203687[] = {{&S83736[27], 6}, {NULL}};
const Segment SGI203688[] = {{&S83736[104], 9}, {NULL}};
const Segment SGI203689[] = {{&S83736[77], 6}, {NULL}};
const Segment SGI203690[] = {{&S73279[44], 10}, {NULL}};
const Segment SGI203691[] = {{&S73279[44], 22}, {NULL}};
const Segment SGI203692[] = {{&S73279[30], 6}, {NULL}};
const Segment SGI203693[] = {{&S73279[115], 3}, {NULL}};
const Segment SGI203694[] = {{&S73279[80], 6}, {NULL}};
const Segment SGI203695[] = {{&S73280[44], 10}, {NULL}};
const Segment SGI203696[] = {{&S73280[44], 22}, {NULL}};
const Segment SGI203697[] = {{&S73280[30], 6}, {NULL}};
const Segment SGI203698[] = {{&S73280[115], 3}, {NULL}};
const Segment SGI203699[] = {{&S73280[80], 6}, {NULL}};
const Segment SGI203700[] = {{&S73281[38], 10}, {NULL}};
const Segment SGI203701[] = {{&S73281[31], 6}, {NULL}};
const Segment SGI203702[] = {{&S73287[44], 7}, {NULL}};
const Segment SGI203703[] = {{&S73287[44], 26}, {NULL}};
const Segment SGI203704[] = {{&S73287[30], 6}, {NULL}};
const Segment SGI203705[] = {{&S73287[119], 3}, {NULL}};
const Segment SGI203706[] = {{&S73287[84], 6}, {NULL}};
const Segment SGI203707[] = {{&S73288[44], 7}, {NULL}};
const Segment SGI203708[] = {{&S73288[44], 27}, {NULL}};
const Segment SGI203709[] = {{&S73288[30], 6}, {NULL}};
const Segment SGI203710[] = {{&S77981[44], 9}, {NULL}};
const Segment SGI203711[] = {{&S77981[44], 21}, {NULL}};
const Segment SGI203712[] = {{&S73288[120], 3}, {NULL}};
const Segment SGI203713[] = {{&S77981[30], 6}, {NULL}};
const Segment SGI203714[] = {{&S73288[85], 6}, {NULL}};
const Segment SGI203715[] = {{&S77981[114], 3}, {NULL}};
const Segment SGI203716[] = {{&S77981[79], 6}, {NULL}};
const Segment SGI203717[] = {{&S73289[44], 7}, {NULL}};
const Segment SGI203718[] = {{&S73289[44], 27}, {NULL}};
const Segment SGI203719[] = {{&S73289[30], 6}, {NULL}};
const Segment SGI203720[] = {{&S73289[120], 3}, {NULL}};
const Segment SGI203721[] = {{&S73289[85], 6}, {NULL}};
const Segment SGI203722[] = {{&S77983[38], 9}, {NULL}};
const Segment SGI203723[] = {{&S77983[31], 6}, {NULL}};
const Segment SGI203724[] = {{&S83759[44], 11}, {NULL}};
const Segment SGI203725[] = {{&S83759[44], 23}, {NULL}};
const Segment SGI203726[] = {{&S73290[44], 7}, {NULL}};
const Segment SGI203727[] = {{&S83759[30], 6}, {NULL}};
const Segment SGI203728[] = {{&S73290[44], 27}, {NULL}};
const Segment SGI203729[] = {{&S73290[30], 6}, {NULL}};
const Segment SGI203730[] = {{&S83759[31], 3}, {NULL}};
const Segment SGI203731[] = {{&S73290[120], 3}, {NULL}};
const Segment SGI203732[] = {{&S83759[81], 6}, {NULL}};
const Segment SGI203733[] = {{&S73290[85], 6}, {NULL}};
const Segment SGI203734[] = {{&S73291[44], 7}, {NULL}};
const Segment SGI203735[] = {{&S73291[44], 27}, {NULL}};
const Segment SGI203736[] = {{&S73291[30], 6}, {NULL}};
const Segment SGI203737[] = {{&S73291[88], 6}, {NULL}};
const Segment SGI203738[] = {{&S83761[34], 10}, {NULL}};
const Segment SGI203739[] = {{&S83761[27], 6}, {NULL}};
const Segment SGI203740[] = {{&S83761[97], 9}, {NULL}};
const Segment SGI203741[] = {{&S83761[70], 6}, {NULL}};
const Segment SGI203742[] = {{&S73292[44], 7}, {NULL}};
const Segment SGI203743[] = {{&S73292[44], 27}, {NULL}};
const Segment SGI203744[] = {{&S73292[30], 6}, {NULL}};
const Segment SGI203745[] = {{&S77986[44], 6}, {NULL}};
const Segment SGI203746[] = {{&S77986[44], 18}, {NULL}};
const Segment SGI203747[] = {{&S77986[30], 6}, {NULL}};
const Segment SGI203748[] = {{&S73292[120], 3}, {NULL}};
const Segment SGI203749[] = {{&S73292[85], 6}, {NULL}};
const Segment SGI203750[] = {{&S83762[34], 22}, {NULL}};
const Segment SGI203751[] = {{&S77986[76], 6}, {NULL}};
const Segment SGI203752[] = {{&S83762[27], 6}, {NULL}};
const Segment SGI203753[] = {{&S77986[103], 13}, {NULL}};
const Segment SGI203754[] = {{&S83762[109], 9}, {NULL}};
const Segment SGI203755[] = {{&S83762[82], 6}, {NULL}};
const Segment SGI203756[] = {{&S73293[38], 7}, {NULL}};
const Segment SGI203757[] = {{&S73293[31], 6}, {NULL}};
const Segment SGI203758[] = {{&S73294[44], 7}, {NULL}};
const Segment SGI203759[] = {{&S73294[44], 27}, {NULL}};
const Segment SGI203760[] = {{&S73294[30], 6}, {NULL}};
const Segment SGI203761[] = {{&S73294[120], 3}, {NULL}};
const Segment SGI203762[] = {{&S73294[85], 6}, {NULL}};
const Segment SGI203763[] = {{&S77990[34], 15}, {NULL}};
const Segment SGI203764[] = {{&S77990[27], 6}, {NULL}};
const Segment SGI203765[] = {{&S83766[34], 11}, {NULL}};
const Segment SGI203766[] = {{&S77990[102], 9}, {NULL}};
const Segment SGI203767[] = {{&S77990[75], 6}, {NULL}};
const Segment SGI203768[] = {{&S83766[27], 6}, {NULL}};
const Segment SGI203769[] = {{&S83766[98], 9}, {NULL}};
const Segment SGI203770[] = {{&S83766[71], 6}, {NULL}};
const Segment SGI203771[] = {{&S77991[34], 25}, {NULL}};
const Segment SGI203772[] = {{&S77991[27], 6}, {NULL}};
const Segment SGI203773[] = {{&S77991[112], 9}, {NULL}};
const Segment SGI203774[] = {{&S77991[85], 6}, {NULL}};
const Segment SGI203775[] = {{&S83768[31], 8}, {NULL}};
const Segment SGI203776[] = {{&S83768[55], 3}, {NULL}};
const Segment SGI203777[] = {{&S77992[34], 19}, {NULL}};
const Segment SGI203778[] = {{&S77992[27], 6}, {NULL}};
const Segment SGI203779[] = {{&S77992[106], 9}, {NULL}};
const Segment SGI203780[] = {{&S83769[34], 10}, {NULL}};
const Segment SGI203781[] = {{&S77992[79], 6}, {NULL}};
const Segment SGI203782[] = {{&S83769[27], 6}, {NULL}};
const Segment SGI203783[] = {{&S83769[97], 9}, {NULL}};
const Segment SGI203784[] = {{&S83769[70], 6}, {NULL}};
const Segment SGI203785[] = {{&S77995[44], 19}, {NULL}};
const Segment SGI203786[] = {{&S77995[44], 31}, {NULL}};
const Segment SGI203787[] = {{&S77995[30], 6}, {NULL}};
const Segment SGI203788[] = {{&S77995[124], 3}, {NULL}};
const Segment SGI203789[] = {{&S77995[89], 6}, {NULL}};
const Segment SGI203790[] = {{&S83775[34], 7}, {NULL}};
const Segment SGI203791[] = {{&S83775[27], 6}, {NULL}};
const Segment SGI203792[] = {{&S83775[93], 9}, {NULL}};
const Segment SGI203793[] = {{&S83775[66], 6}, {NULL}};
const Segment SGI203794[] = {{&S77998[44], 12}, {NULL}};
const Segment SGI203795[] = {{&S77998[44], 24}, {NULL}};
const Segment SGI203796[] = {{&S77998[30], 6}, {NULL}};
const Segment SGI203797[] = {{&S77998[117], 3}, {NULL}};
const Segment SGI203798[] = {{&S77998[82], 6}, {NULL}};
const Segment SGI203799[] = {{&S72391[44], 16}, {NULL}};
const Segment SGI203800[] = {{&S72391[44], 28}, {NULL}};
const Segment SGI203801[] = {{&S72391[30], 6}, {NULL}};
const Segment SGI203802[] = {{&S72391[121], 3}, {NULL}};
const Segment SGI203803[] = {{&S72391[86], 6}, {NULL}};
const Segment SGI203804[] = {{&S83779[34], 9}, {NULL}};
const Segment SGI203805[] = {{&S83779[27], 6}, {NULL}};
const Segment SGI203806[] = {{&S83779[96], 9}, {NULL}};
const Segment SGI203807[] = {{&S83779[69], 6}, {NULL}};
const Segment SGI203808[] = {{&S83780[34], 11}, {NULL}};
const Segment SGI203809[] = {{&S83780[27], 6}, {NULL}};
const Segment SGI203810[] = {{&S83780[98], 9}, {NULL}};
const Segment SGI203811[] = {{&S83780[71], 6}, {NULL}};
const Segment SGI203812[] = {{&S83781[34], 8}, {NULL}};
const Segment SGI203813[] = {{&S83781[27], 6}, {NULL}};
const Segment SGI203814[] = {{&S83781[111], 9}, {NULL}};
const Segment SGI203815[] = {{&S83781[84], 6}, {NULL}};
const Segment SGI203816[] = {{&S83782[34], 10}, {NULL}};
const Segment SGI203817[] = {{&S83782[27], 6}, {NULL}};
const Segment SGI203818[] = {{&S83782[97], 9}, {NULL}};
const Segment SGI203819[] = {{&S83782[70], 6}, {NULL}};
const Segment SGI203820[] = {{&S83783[44], 19}, {NULL}};
const Segment SGI203821[] = {{&S83783[44], 34}, {NULL}};
const Segment SGI203822[] = {{&S83783[30], 6}, {NULL}};
const Segment SGI203823[] = {{&S83783[31], 3}, {NULL}};
const Segment SGI203824[] = {{&S83783[92], 6}, {NULL}};
const Segment SGI203825[] = {{&S83788[34], 4}, {NULL}};
const Segment SGI203826[] = {{&S83788[27], 6}, {NULL}};
const Segment SGI203827[] = {{&S83788[91], 9}, {NULL}};
const Segment SGI203828[] = {{&S83788[64], 6}, {NULL}};
const Segment SGI203829[] = {{&S83789[44], 11}, {NULL}};
const Segment SGI203830[] = {{&S83789[44], 23}, {NULL}};
const Segment SGI203831[] = {{&S83789[30], 6}, {NULL}};
const Segment SGI203832[] = {{&S83789[31], 3}, {NULL}};
const Segment SGI203833[] = {{&S83789[81], 6}, {NULL}};
const Segment SGI203834[] = {{&S83790[44], 11}, {NULL}};
const Segment SGI203835[] = {{&S83790[44], 23}, {NULL}};
const Segment SGI203836[] = {{&S83790[30], 6}, {NULL}};
const Segment SGI203837[] = {{&S83790[31], 3}, {NULL}};
const Segment SGI203838[] = {{&S83790[81], 6}, {NULL}};
const Segment SGI203839[] = {{&S83791[34], 13}, {NULL}};
const Segment SGI203840[] = {{&S83791[27], 6}, {NULL}};
const Segment SGI203841[] = {{&S83791[100], 9}, {NULL}};
const Segment SGI203842[] = {{&S83791[73], 6}, {NULL}};
const Segment SGI203843[] = {{&S83794[44], 5}, {NULL}};
const Segment SGI203844[] = {{&S83794[44], 17}, {NULL}};
const Segment SGI203845[] = {{&S83794[30], 6}, {NULL}};
const Segment SGI203846[] = {{&S83794[75], 6}, {NULL}};
const Segment SGI203847[] = {{&S83794[102], 13}, {NULL}};
const Segment SGI203848[] = {{&S83795[44], 13}, {NULL}};
const Segment SGI203849[] = {{&S83795[44], 25}, {NULL}};
const Segment SGI203850[] = {{&S83795[30], 6}, {NULL}};
const Segment SGI203851[] = {{&S78028[34], 11}, {NULL}};
const Segment SGI203852[] = {{&S78028[27], 6}, {NULL}};
const Segment SGI203853[] = {{&S83795[83], 6}, {NULL}};
const Segment SGI203854[] = {{&S83795[110], 13}, {NULL}};
const Segment SGI203855[] = {{&S78028[98], 9}, {NULL}};
const Segment SGI203856[] = {{&S78028[71], 6}, {NULL}};
const Segment SGI203857[] = {{&S83798[34], 10}, {NULL}};
const Segment SGI203858[] = {{&S83798[27], 6}, {NULL}};
const Segment SGI203859[] = {{&S83798[97], 9}, {NULL}};
const Segment SGI203860[] = {{&S83798[70], 6}, {NULL}};
const Segment SGI203861[] = {{&S78031[34], 11}, {NULL}};
const Segment SGI203862[] = {{&S78031[27], 6}, {NULL}};
const Segment SGI203863[] = {{&S78031[98], 9}, {NULL}};
const Segment SGI203864[] = {{&S78031[71], 6}, {NULL}};
const Segment SGI203865[] = {{&S78035[34], 15}, {NULL}};
const Segment SGI203866[] = {{&S78035[27], 6}, {NULL}};
const Segment SGI203867[] = {{&S78035[102], 9}, {NULL}};
const Segment SGI203868[] = {{&S78035[75], 6}, {NULL}};
const Segment SGI203869[] = {{&S78041[34], 6}, {NULL}};
const Segment SGI203870[] = {{&S78041[27], 6}, {NULL}};
const Segment SGI203871[] = {{&S73365[34], 9}, {NULL}};
const Segment SGI203872[] = {{&S73365[27], 6}, {NULL}};
const Segment SGI203873[] = {{&S78041[93], 9}, {NULL}};
const Segment SGI203874[] = {{&S78041[66], 6}, {NULL}};
const Segment SGI203875[] = {{&S73365[96], 9}, {NULL}};
const Segment SGI203876[] = {{&S73365[69], 6}, {NULL}};
const Segment SGI203877[] = {{&S73366[34], 10}, {NULL}};
const Segment SGI203878[] = {{&S73366[27], 6}, {NULL}};
const Segment SGI203879[] = {{&S73366[97], 9}, {NULL}};
const Segment SGI203880[] = {{&S73366[70], 6}, {NULL}};
const Segment SGI203881[] = {{&S73367[34], 10}, {NULL}};
const Segment SGI203882[] = {{&S73367[27], 6}, {NULL}};
const Segment SGI203883[] = {{&S73367[97], 9}, {NULL}};
const Segment SGI203884[] = {{&S73367[70], 6}, {NULL}};
const Segment SGI203885[] = {{&S78045[34], 9}, {NULL}};
const Segment SGI203886[] = {{&S78045[27], 6}, {NULL}};
const Segment SGI203887[] = {{&S78045[96], 9}, {NULL}};
const Segment SGI203888[] = {{&S78045[69], 6}, {NULL}};
const Segment SGI203889[] = {{&S73368[34], 7}, {NULL}};
const Segment SGI203890[] = {{&S73368[27], 6}, {NULL}};
const Segment SGI203891[] = {{&S73368[98], 9}, {NULL}};
const Segment SGI203892[] = {{&S73368[71], 6}, {NULL}};
const Segment SGI203893[] = {{&S73369[34], 10}, {NULL}};
const Segment SGI203894[] = {{&S73369[27], 6}, {NULL}};
const Segment SGI203895[] = {{&S73369[97], 9}, {NULL}};
const Segment SGI203896[] = {{&S73369[70], 6}, {NULL}};
const Segment SGI203897[] = {{&S73370[34], 9}, {NULL}};
const Segment SGI203898[] = {{&S73370[27], 6}, {NULL}};
const Segment SGI203899[] = {{&S73370[96], 9}, {NULL}};
const Segment SGI203900[] = {{&S73370[69], 6}, {NULL}};
const Segment SGI203901[] = {{&S78053[44], 14}, {NULL}};
const Segment SGI203902[] = {{&S78053[44], 31}, {NULL}};
const Segment SGI203903[] = {{&S78053[30], 6}, {NULL}};
const Segment SGI203904[] = {{&S78053[89], 6}, {NULL}};
const Segment SGI203905[] = {{&S78053[116], 13}, {NULL}};
const Segment SGI203906[] = {{&S81826[42], 11}, {NULL}};
const Segment SGI203907[] = {{&S81826[42], 23}, {NULL}};
const Segment SGI203908[] = {{&S81826[30], 5}, {NULL}};
const Segment SGI203909[] = {{&S81826[114], 3}, {NULL}};
const Segment SGI203910[] = {{&S81826[79], 6}, {NULL}};
const Segment SGI203911[] = {{&S72468[44], 7}, {NULL}};
const Segment SGI203912[] = {{&S72468[44], 19}, {NULL}};
const Segment SGI203913[] = {{&S72468[30], 6}, {NULL}};
const Segment SGI203914[] = {{&S72468[130], 3}, {NULL}};
const Segment SGI203915[] = {{&S72468[95], 6}, {NULL}};
const Segment SGI203916[] = {{&S72468[65], 8}, {NULL}};
const Segment SGI203917[] = {{&S72471[34], 10}, {NULL}};
const Segment SGI203918[] = {{&S72471[27], 6}, {NULL}};
const Segment SGI203919[] = {{&S72471[97], 9}, {NULL}};
const Segment SGI203920[] = {{&S72471[70], 6}, {NULL}};
const Segment SGI203921[] = {{&S73393[34], 11}, {NULL}};
const Segment SGI203922[] = {{&S73393[27], 6}, {NULL}};
const Segment SGI203923[] = {{&S73393[98], 9}, {NULL}};
const Segment SGI203924[] = {{&S73393[71], 6}, {NULL}};
const Segment SGI203925[] = {{&S73424[34], 7}, {NULL}};
const Segment SGI203926[] = {{&S73424[27], 6}, {NULL}};
const Segment SGI203927[] = {{&S73424[98], 9}, {NULL}};
const Segment SGI203928[] = {{&S78102[34], 15}, {NULL}};
const Segment SGI203929[] = {{&S73424[71], 6}, {NULL}};
const Segment SGI203930[] = {{&S78102[27], 6}, {NULL}};
const Segment SGI203931[] = {{&S78102[102], 9}, {NULL}};
const Segment SGI203932[] = {{&S78102[75], 6}, {NULL}};
const Segment SGI203933[] = {{&S73426[34], 6}, {NULL}};
const Segment SGI203934[] = {{&S73426[27], 6}, {NULL}};
const Segment SGI203935[] = {{&S73426[93], 9}, {NULL}};
const Segment SGI203936[] = {{&S73426[66], 6}, {NULL}};
const Segment SGI203937[] = {{&S78109[34], 17}, {NULL}};
const Segment SGI203938[] = {{&S78109[27], 6}, {NULL}};
const Segment SGI203939[] = {{&S78109[104], 9}, {NULL}};
const Segment SGI203940[] = {{&S78109[77], 6}, {NULL}};
const Segment SGI203941[] = {{&S76309[32], 12}, {NULL}};
const Segment SGI203942[] = {{&S76309[27], 4}, {NULL}};
const Segment SGI203943[] = {{&S78110[44], 12}, {NULL}};
const Segment SGI203944[] = {{&S76309[97], 9}, {NULL}};
const Segment SGI203945[] = {{&S78110[44], 24}, {NULL}};
const Segment SGI203946[] = {{&S76309[70], 6}, {NULL}};
const Segment SGI203947[] = {{&S78110[30], 6}, {NULL}};
const Segment SGI203948[] = {{&S78110[117], 3}, {NULL}};
const Segment SGI203949[] = {{&S78110[82], 6}, {NULL}};
const Segment SGI203950[] = {{&S63542[34], 10}, {NULL}};
const Segment SGI203951[] = {{&S63542[27], 6}, {NULL}};
const Segment SGI203952[] = {{&S63542[96], 9}, {NULL}};
const Segment SGI203953[] = {{&S63542[69], 6}, {NULL}};
const Segment SGI203954[] = {{&S63543[34], 18}, {NULL}};
const Segment SGI203955[] = {{&S63543[27], 6}, {NULL}};
const Segment SGI203956[] = {{&S63543[104], 9}, {NULL}};
const Segment SGI203957[] = {{&S63543[77], 6}, {NULL}};
const Segment SGI203958[] = {{&S63544[34], 18}, {NULL}};
const Segment SGI203959[] = {{&S63544[27], 6}, {NULL}};
const Segment SGI203960[] = {{&S63544[104], 9}, {NULL}};
const Segment SGI203961[] = {{&S63544[77], 6}, {NULL}};
const Segment SGI203962[] = {{&S72525[34], 16}, {NULL}};
const Segment SGI203963[] = {{&S72525[27], 6}, {NULL}};
const Segment SGI203964[] = {{&S72525[103], 9}, {NULL}};
const Segment SGI203965[] = {{&S72525[76], 6}, {NULL}};
const Segment SGI203966[] = {{&S73460[44], 10}, {NULL}};
const Segment SGI203967[] = {{&S73460[44], 22}, {NULL}};
const Segment SGI203968[] = {{&S73460[30], 6}, {NULL}};
const Segment SGI203969[] = {{&S63561[44], 8}, {NULL}};
const Segment SGI203970[] = {{&S63561[44], 19}, {NULL}};
const Segment SGI203971[] = {{&S63561[30], 6}, {NULL}};
const Segment SGI203972[] = {{&S73460[80], 6}, {NULL}};
const Segment SGI203973[] = {{&S73460[107], 13}, {NULL}};
const Segment SGI203974[] = {{&S63561[112], 3}, {NULL}};
const Segment SGI203975[] = {{&S63561[77], 6}, {NULL}};
const Segment SGI203976[] = {{&S73461[44], 10}, {NULL}};
const Segment SGI203977[] = {{&S73461[44], 22}, {NULL}};
const Segment SGI203978[] = {{&S63562[38], 8}, {NULL}};
const Segment SGI203979[] = {{&S73461[30], 6}, {NULL}};
const Segment SGI203980[] = {{&S63562[31], 6}, {NULL}};
const Segment SGI203981[] = {{&S73461[80], 6}, {NULL}};
const Segment SGI203982[] = {{&S73461[107], 13}, {NULL}};
const Segment SGI203983[] = {{&S78140[34], 11}, {NULL}};
const Segment SGI203984[] = {{&S78140[27], 6}, {NULL}};
const Segment SGI203985[] = {{&S78140[98], 9}, {NULL}};
const Segment SGI203986[] = {{&S78140[71], 6}, {NULL}};
const Segment SGI203987[] = {{&S78142[44], 8}, {NULL}};
const Segment SGI203988[] = {{&S78142[44], 20}, {NULL}};
const Segment SGI203989[] = {{&S78142[30], 6}, {NULL}};
const Segment SGI203990[] = {{&S78142[113], 3}, {NULL}};
const Segment SGI203991[] = {{&S78142[78], 6}, {NULL}};
const Segment SGI203992[] = {{&S72550[44], 7}, {NULL}};
const Segment SGI203993[] = {{&S72550[44], 19}, {NULL}};
const Segment SGI203994[] = {{&S72550[30], 6}, {NULL}};
const Segment SGI203995[] = {{&S72550[112], 3}, {NULL}};
const Segment SGI203996[] = {{&S72550[77], 6}, {NULL}};
const Segment SGI203997[] = {{&S78149[44], 10}, {NULL}};
const Segment SGI203998[] = {{&S78149[44], 22}, {NULL}};
const Segment SGI203999[] = {{&S78149[30], 6}, {NULL}};
const Segment SGI204000[] = {{&S78149[80], 6}, {NULL}};
const Segment SGI204001[] = {{&S78149[107], 13}, {NULL}};
const Segment SGI204002[] = {{&S63580[34], 9}, {NULL}};
const Segment SGI204003[] = {{&S63580[27], 6}, {NULL}};
const Segment SGI204004[] = {{&S63580[95], 9}, {NULL}};
const Segment SGI204005[] = {{&S63580[68], 6}, {NULL}};
const Segment SGI204006[] = {{&S63580[129], 8}, {NULL}};
const Segment SGI204007[] = {{&S78154[44], 11}, {NULL}};
const Segment SGI204008[] = {{&S78154[44], 23}, {NULL}};
const Segment SGI204009[] = {{&S78154[30], 6}, {NULL}};
const Segment SGI204010[] = {{&S78154[81], 6}, {NULL}};
const Segment SGI204011[] = {{&S78154[108], 13}, {NULL}};
const Segment SGI204012[] = {{&S83848[34], 8}, {NULL}};
const Segment SGI204013[] = {{&S83848[27], 6}, {NULL}};
const Segment SGI204014[] = {{&S83848[95], 9}, {NULL}};
const Segment SGI204015[] = {{&S83848[68], 6}, {NULL}};
const Segment SGI204016[] = {{&S73523[34], 10}, {NULL}};
const Segment SGI204017[] = {{&S73523[27], 6}, {NULL}};
const Segment SGI204018[] = {{&S73523[97], 9}, {NULL}};
const Segment SGI204019[] = {{&S73523[70], 6}, {NULL}};
const Segment SGI204020[] = {{&S83853[59], 15}, {NULL}};
const Segment SGI204021[] = {{&S83853[59], 33}, {NULL}};
const Segment SGI204022[] = {{&S73523[138], 8}, {NULL}};
const Segment SGI204023[] = {{&S83853[45], 6}, {NULL}};
const Segment SGI204024[] = {{&S83853[46], 3}, {NULL}};
const Segment SGI204025[] = {{&S83853[106], 6}, {NULL}};
const Segment SGI204026[] = {{&S83853[0], 14}, {NULL}};
const Segment SGI204027[] = {{&S83858[34], 12}, {NULL}};
const Segment SGI204028[] = {{&S83858[27], 6}, {NULL}};
const Segment SGI204029[] = {{&S83858[99], 9}, {NULL}};
const Segment SGI204030[] = {{&S83858[72], 6}, {NULL}};
const Segment SGI204031[] = {{&S83864[34], 7}, {NULL}};
const Segment SGI204032[] = {{&S83864[27], 6}, {NULL}};
const Segment SGI204033[] = {{&S83864[94], 9}, {NULL}};
const Segment SGI204034[] = {{&S83864[67], 6}, {NULL}};
const Segment SGI204035[] = {{&S83865[34], 10}, {NULL}};
const Segment SGI204036[] = {{&S83865[27], 6}, {NULL}};
const Segment SGI204037[] = {{&S78211[44], 18}, {NULL}};
const Segment SGI204038[] = {{&S78211[44], 30}, {NULL}};
const Segment SGI204039[] = {{&S83865[97], 9}, {NULL}};
const Segment SGI204040[] = {{&S78211[30], 6}, {NULL}};
const Segment SGI204041[] = {{&S83865[70], 6}, {NULL}};
const Segment SGI204042[] = {{&S78211[123], 3}, {NULL}};
const Segment SGI204043[] = {{&S78211[88], 6}, {NULL}};
const Segment SGI204044[] = {{&S83866[34], 5}, {NULL}};
const Segment SGI204045[] = {{&S83866[27], 6}, {NULL}};
const Segment SGI204046[] = {{&S83866[92], 9}, {NULL}};
const Segment SGI204047[] = {{&S83866[65], 6}, {NULL}};
const Segment SGI204048[] = {{&S83867[34], 13}, {NULL}};
const Segment SGI204049[] = {{&S78214[34], 7}, {NULL}};
const Segment SGI204050[] = {{&S83867[27], 6}, {NULL}};
const Segment SGI204051[] = {{&S78214[27], 6}, {NULL}};
const Segment SGI204052[] = {{&S83867[100], 9}, {NULL}};
const Segment SGI204053[] = {{&S83867[73], 6}, {NULL}};
const Segment SGI204054[] = {{&S78214[94], 9}, {NULL}};
const Segment SGI204055[] = {{&S78214[67], 6}, {NULL}};
const Segment SGI204056[] = {{&S83870[59], 6}, {NULL}};
const Segment SGI204057[] = {{&S83870[59], 18}, {NULL}};
const Segment SGI204058[] = {{&S83870[45], 6}, {NULL}};
const Segment SGI204059[] = {{&S83870[46], 3}, {NULL}};
const Segment SGI204060[] = {{&S83870[91], 6}, {NULL}};
const Segment SGI204061[] = {{&S83870[0], 14}, {NULL}};
const Segment SGI204062[] = {{&S83871[59], 6}, {NULL}};
const Segment SGI204063[] = {{&S83871[59], 18}, {NULL}};
const Segment SGI204064[] = {{&S83871[45], 6}, {NULL}};
const Segment SGI204065[] = {{&S83871[46], 3}, {NULL}};
const Segment SGI204066[] = {{&S83871[91], 6}, {NULL}};
const Segment SGI204067[] = {{&S83871[0], 14}, {NULL}};
const Segment SGI204068[] = {{&S78242[44], 9}, {NULL}};
const Segment SGI204069[] = {{&S78242[44], 21}, {NULL}};
const Segment SGI204070[] = {{&S78242[30], 6}, {NULL}};
const Segment SGI204071[] = {{&S78242[130], 3}, {NULL}};
const Segment SGI204072[] = {{&S78242[95], 6}, {NULL}};
const Segment SGI204073[] = {{&S78242[67], 8}, {NULL}};
const Segment SGI204074[] = {{&S78248[45], 10}, {NULL}};
const Segment SGI204075[] = {{&S78248[30], 6}, {NULL}};
const Segment SGI204076[] = {{&S78248[115], 3}, {NULL}};
const Segment SGI204077[] = {{&S78248[81], 5}, {NULL}};
const Segment SGI204078[] = {{&S78248[139], 13}, {NULL}};
const Segment SGI204079[] = {{&S73585[34], 9}, {NULL}};
const Segment SGI204080[] = {{&S73585[27], 6}, {NULL}};
const Segment SGI204081[] = {{&S73585[96], 9}, {NULL}};
const Segment SGI204082[] = {{&S73585[69], 6}, {NULL}};
const Segment SGI204083[] = {{&S73586[34], 10}, {NULL}};
const Segment SGI204084[] = {{&S73586[27], 6}, {NULL}};
const Segment SGI204085[] = {{&S73586[97], 9}, {NULL}};
const Segment SGI204086[] = {{&S73586[70], 6}, {NULL}};
const Segment SGI204087[] = {{&S72665[44], 9}, {NULL}};
const Segment SGI204088[] = {{&S72665[44], 21}, {NULL}};
const Segment SGI204089[] = {{&S72665[30], 6}, {NULL}};
const Segment SGI204090[] = {{&S72665[132], 3}, {NULL}};
const Segment SGI204091[] = {{&S72665[97], 6}, {NULL}};
const Segment SGI204092[] = {{&S73587[34], 10}, {NULL}};
const Segment SGI204093[] = {{&S73587[27], 6}, {NULL}};
const Segment SGI204094[] = {{&S72665[67], 8}, {NULL}};
const Segment SGI204095[] = {{&S73587[97], 9}, {NULL}};
const Segment SGI204096[] = {{&S73587[70], 6}, {NULL}};
const Segment SGI204097[] = {{&S73588[34], 9}, {NULL}};
const Segment SGI204098[] = {{&S73588[27], 6}, {NULL}};
const Segment SGI204099[] = {{&S73588[96], 9}, {NULL}};
const Segment SGI204100[] = {{&S73588[69], 6}, {NULL}};
const Segment SGI204101[] = {{&S78259[44], 8}, {NULL}};
const Segment SGI204102[] = {{&S78259[44], 20}, {NULL}};
const Segment SGI204103[] = {{&S78259[30], 6}, {NULL}};
const Segment SGI204104[] = {{&S78259[113], 3}, {NULL}};
const Segment SGI204105[] = {{&S78259[78], 6}, {NULL}};
const Segment SGI204106[] = {{&S78260[44], 15}, {NULL}};
const Segment SGI204107[] = {{&S78260[44], 27}, {NULL}};
const Segment SGI204108[] = {{&S78260[30], 6}, {NULL}};
const Segment SGI204109[] = {{&S78260[85], 6}, {NULL}};
const Segment SGI204110[] = {{&S78260[112], 13}, {NULL}};
const Segment SGI204111[] = {{&S73598[34], 7}, {NULL}};
const Segment SGI204112[] = {{&S73598[27], 6}, {NULL}};
const Segment SGI204113[] = {{&S73598[98], 9}, {NULL}};
const Segment SGI204114[] = {{&S73598[71], 6}, {NULL}};
const Segment SGI204115[] = {{&S73599[34], 9}, {NULL}};
const Segment SGI204116[] = {{&S73599[27], 6}, {NULL}};
const Segment SGI204117[] = {{&S73599[96], 9}, {NULL}};
const Segment SGI204118[] = {{&S73599[69], 6}, {NULL}};
const Segment SGI204119[] = {{&S73619[34], 10}, {NULL}};
const Segment SGI204120[] = {{&S73619[27], 6}, {NULL}};
const Segment SGI204121[] = {{&S73619[97], 9}, {NULL}};
const Segment SGI204122[] = {{&S73619[70], 6}, {NULL}};
const Segment SGI204123[] = {{&S83943[44], 13}, {NULL}};
const Segment SGI204124[] = {{&S83943[44], 36}, {NULL}};
const Segment SGI204125[] = {{&S83943[30], 6}, {NULL}};
const Segment SGI204126[] = {{&S83943[31], 3}, {NULL}};
const Segment SGI204127[] = {{&S83943[94], 6}, {NULL}};
const Segment SGI204128[] = {{&S73624[44], 9}, {NULL}};
const Segment SGI204129[] = {{&S73624[44], 21}, {NULL}};
const Segment SGI204130[] = {{&S73624[30], 6}, {NULL}};
const Segment SGI204131[] = {{&S73624[114], 3}, {NULL}};
const Segment SGI204132[] = {{&S73624[79], 6}, {NULL}};
const Segment SGI204133[] = {{&S73625[38], 9}, {NULL}};
const Segment SGI204134[] = {{&S73625[31], 6}, {NULL}};
const Segment SGI204135[] = {{&S83949[34], 18}, {NULL}};
const Segment SGI204136[] = {{&S83949[27], 6}, {NULL}};
const Segment SGI204137[] = {{&S83949[115], 9}, {NULL}};
const Segment SGI204138[] = {{&S83949[88], 6}, {NULL}};
const Segment SGI204139[] = {{&S83952[34], 9}, {NULL}};
const Segment SGI204140[] = {{&S83952[27], 6}, {NULL}};
const Segment SGI204141[] = {{&S83952[96], 9}, {NULL}};
const Segment SGI204142[] = {{&S83952[69], 6}, {NULL}};
const Segment SGI204143[] = {{&S83953[34], 11}, {NULL}};
const Segment SGI204144[] = {{&S83953[27], 6}, {NULL}};
const Segment SGI204145[] = {{&S83953[98], 9}, {NULL}};
const Segment SGI204146[] = {{&S83953[71], 6}, {NULL}};
const Segment SGI204147[] = {{&S78307[44], 10}, {NULL}};
const Segment SGI204148[] = {{&S78307[44], 22}, {NULL}};
const Segment SGI204149[] = {{&S78307[30], 6}, {NULL}};
const Segment SGI204150[] = {{&S83954[34], 7}, {NULL}};
const Segment SGI204151[] = {{&S83954[27], 6}, {NULL}};
const Segment SGI204152[] = {{&S78307[115], 3}, {NULL}};
const Segment SGI204153[] = {{&S78307[80], 6}, {NULL}};
const Segment SGI204154[] = {{&S83954[99], 9}, {NULL}};
const Segment SGI204155[] = {{&S83954[72], 6}, {NULL}};
const Segment SGI204156[] = {{&S83958[34], 14}, {NULL}};
const Segment SGI204157[] = {{&S83958[27], 6}, {NULL}};
const Segment SGI204158[] = {{&S83958[100], 9}, {NULL}};
const Segment SGI204159[] = {{&S83958[74], 5}, {NULL}};
const Segment SGI204160[] = {{&S78315[44], 18}, {NULL}};
const Segment SGI204161[] = {{&S83960[34], 7}, {NULL}};
const Segment SGI204162[] = {{&S78315[44], 30}, {NULL}};
const Segment SGI204163[] = {{&S78315[30], 6}, {NULL}};
const Segment SGI204164[] = {{&S83960[27], 6}, {NULL}};
const Segment SGI204165[] = {{&S83960[112], 9}, {NULL}};
const Segment SGI204166[] = {{&S78315[88], 6}, {NULL}};
const Segment SGI204167[] = {{&S83960[85], 6}, {NULL}};
const Segment SGI204168[] = {{&S78315[115], 13}, {NULL}};
const Segment SGI204169[] = {{&S83961[34], 11}, {NULL}};
const Segment SGI204170[] = {{&S83961[27], 6}, {NULL}};
const Segment SGI204171[] = {{&S83961[110], 9}, {NULL}};
const Segment SGI204172[] = {{&S83961[83], 6}, {NULL}};
const Segment SGI204173[] = {{&S78318[44], 11}, {NULL}};
const Segment SGI204174[] = {{&S78318[44], 23}, {NULL}};
const Segment SGI204175[] = {{&S78318[30], 6}, {NULL}};
const Segment SGI204176[] = {{&S83962[44], 8}, {NULL}};
const Segment SGI204177[] = {{&S83962[44], 20}, {NULL}};
const Segment SGI204178[] = {{&S83962[30], 6}, {NULL}};
const Segment SGI204179[] = {{&S78318[116], 3}, {NULL}};
const Segment SGI204180[] = {{&S78318[81], 6}, {NULL}};
const Segment SGI204181[] = {{&S83962[31], 3}, {NULL}};
const Segment SGI204182[] = {{&S78318[141], 7}, {NULL}};
const Segment SGI204183[] = {{&S83962[78], 6}, {NULL}};
const Segment SGI204184[] = {{&S83970[59], 11}, {NULL}};
const Segment SGI204185[] = {{&S83970[59], 23}, {NULL}};
const Segment SGI204186[] = {{&S83970[45], 6}, {NULL}};
const Segment SGI204187[] = {{&S83970[46], 3}, {NULL}};
const Segment SGI204188[] = {{&S83970[96], 6}, {NULL}};
const Segment SGI204189[] = {{&S83970[0], 14}, {NULL}};
const Segment SGI204190[] = {{&S73659[34], 6}, {NULL}};
const Segment SGI204191[] = {{&S73659[27], 6}, {NULL}};
const Segment SGI204192[] = {{&S73659[93], 9}, {NULL}};
const Segment SGI204193[] = {{&S73659[66], 6}, {NULL}};
const Segment SGI204194[] = {{&S73661[34], 9}, {NULL}};
const Segment SGI204195[] = {{&S73661[27], 6}, {NULL}};
const Segment SGI204196[] = {{&S73661[96], 9}, {NULL}};
const Segment SGI204197[] = {{&S73661[69], 6}, {NULL}};
const Segment SGI204198[] = {{&S83974[34], 13}, {NULL}};
const Segment SGI204199[] = {{&S83974[27], 6}, {NULL}};
const Segment SGI204200[] = {{&S83974[100], 9}, {NULL}};
const Segment SGI204201[] = {{&S83974[73], 6}, {NULL}};
const Segment SGI204202[] = {{&S73662[44], 10}, {NULL}};
const Segment SGI204203[] = {{&S73662[44], 22}, {NULL}};
const Segment SGI204204[] = {{&S73662[30], 6}, {NULL}};
const Segment SGI204205[] = {{&S73662[115], 3}, {NULL}};
const Segment SGI204206[] = {{&S73662[80], 6}, {NULL}};
const Segment SGI204207[] = {{&S83977[34], 5}, {NULL}};
const Segment SGI204208[] = {{&S83977[27], 6}, {NULL}};
const Segment SGI204209[] = {{&S73664[44], 6}, {NULL}};
const Segment SGI204210[] = {{&S73664[44], 18}, {NULL}};
const Segment SGI204211[] = {{&S73664[30], 6}, {NULL}};
const Segment SGI204212[] = {{&S83977[92], 9}, {NULL}};
const Segment SGI204213[] = {{&S83977[65], 6}, {NULL}};
const Segment SGI204214[] = {{&S73664[111], 3}, {NULL}};
const Segment SGI204215[] = {{&S73664[76], 6}, {NULL}};
const Segment SGI204216[] = {{&S73666[34], 10}, {NULL}};
const Segment SGI204217[] = {{&S73666[27], 6}, {NULL}};
const Segment SGI204218[] = {{&S73666[97], 9}, {NULL}};
const Segment SGI204219[] = {{&S73666[70], 6}, {NULL}};
const Segment SGI204220[] = {{&S73668[34], 7}, {NULL}};
const Segment SGI204221[] = {{&S73668[27], 6}, {NULL}};
const Segment SGI204222[] = {{&S73668[98], 9}, {NULL}};
const Segment SGI204223[] = {{&S73668[71], 6}, {NULL}};
const Segment SGI204224[] = {{&S83986[34], 10}, {NULL}};
const Segment SGI204225[] = {{&S83986[27], 6}, {NULL}};
const Segment SGI204226[] = {{&S83986[97], 9}, {NULL}};
const Segment SGI204227[] = {{&S83986[70], 6}, {NULL}};
const Segment SGI204228[] = {{&S78349[44], 14}, {NULL}};
const Segment SGI204229[] = {{&S78349[44], 26}, {NULL}};
const Segment SGI204230[] = {{&S78349[30], 6}, {NULL}};
const Segment SGI204231[] = {{&S78349[119], 3}, {NULL}};
const Segment SGI204232[] = {{&S78349[84], 6}, {NULL}};
const Segment SGI204233[] = {{&S73677[34], 13}, {NULL}};
const Segment SGI204234[] = {{&S73677[27], 6}, {NULL}};
const Segment SGI204235[] = {{&S73677[100], 9}, {NULL}};
const Segment SGI204236[] = {{&S73677[73], 6}, {NULL}};
const Segment SGI204237[] = {{&S73678[34], 11}, {NULL}};
const Segment SGI204238[] = {{&S73678[27], 6}, {NULL}};
const Segment SGI204239[] = {{&S73678[98], 9}, {NULL}};
const Segment SGI204240[] = {{&S73678[71], 6}, {NULL}};
const Segment SGI204241[] = {{&S78357[34], 10}, {NULL}};
const Segment SGI204242[] = {{&S78357[27], 6}, {NULL}};
const Segment SGI204243[] = {{&S78357[97], 9}, {NULL}};
const Segment SGI204244[] = {{&S78357[70], 6}, {NULL}};
const Segment SGI204245[] = {{&S63636[34], 10}, {NULL}};
const Segment SGI204246[] = {{&S63636[27], 6}, {NULL}};
const Segment SGI204247[] = {{&S63636[96], 9}, {NULL}};
const Segment SGI204248[] = {{&S63636[69], 6}, {NULL}};
const Segment SGI204249[] = {{&S84003[27], 6}, {NULL}};
const Segment SGI204250[] = {{&S78363[44], 9}, {NULL}};
const Segment SGI204251[] = {{&S78363[44], 21}, {NULL}};
const Segment SGI204252[] = {{&S78363[30], 6}, {NULL}};
const Segment SGI204253[] = {{&S84003[111], 9}, {NULL}};
const Segment SGI204254[] = {{&S84003[84], 6}, {NULL}};
const Segment SGI204255[] = {{&S73686[44], 24}, {NULL}};
const Segment SGI204256[] = {{&S78363[114], 3}, {NULL}};
const Segment SGI204257[] = {{&S73686[44], 36}, {NULL}};
const Segment SGI204258[] = {{&S73686[30], 6}, {NULL}};
const Segment SGI204259[] = {{&S78363[79], 6}, {NULL}};
const Segment SGI204260[] = {{&S73686[145], 3}, {NULL}};
const Segment SGI204261[] = {{&S73686[110], 6}, {NULL}};
const Segment SGI204262[] = {{&S73686[82], 8}, {NULL}};
const Segment SGI204263[] = {{&S78364[44], 11}, {NULL}};
const Segment SGI204264[] = {{&S78364[44], 23}, {NULL}};
const Segment SGI204265[] = {{&S78364[30], 6}, {NULL}};
const Segment SGI204266[] = {{&S78364[116], 3}, {NULL}};
const Segment SGI204267[] = {{&S78364[81], 6}, {NULL}};
const Segment SGI204268[] = {{&S78365[44], 11}, {NULL}};
const Segment SGI204269[] = {{&S78365[44], 23}, {NULL}};
const Segment SGI204270[] = {{&S78365[30], 6}, {NULL}};
const Segment SGI204271[] = {{&S78365[116], 3}, {NULL}};
const Segment SGI204272[] = {{&S78365[81], 6}, {NULL}};
const Segment SGI204273[] = {{&S84009[34], 8}, {NULL}};
const Segment SGI204274[] = {{&S84009[27], 6}, {NULL}};
const Segment SGI204275[] = {{&S84009[95], 9}, {NULL}};
const Segment SGI204276[] = {{&S84009[68], 6}, {NULL}};
const Segment SGI204277[] = {{&S84010[44], 11}, {NULL}};
const Segment SGI204278[] = {{&S84010[44], 23}, {NULL}};
const Segment SGI204279[] = {{&S84010[30], 6}, {NULL}};
const Segment SGI204280[] = {{&S84010[31], 3}, {NULL}};
const Segment SGI204281[] = {{&S84010[81], 6}, {NULL}};
const Segment SGI204282[] = {{&S84013[44], 14}, {NULL}};
const Segment SGI204283[] = {{&S84013[44], 26}, {NULL}};
const Segment SGI204284[] = {{&S78372[44], 20}, {NULL}};
const Segment SGI204285[] = {{&S84013[30], 6}, {NULL}};
const Segment SGI204286[] = {{&S73698[44], 10}, {NULL}};
const Segment SGI204287[] = {{&S78372[44], 32}, {NULL}};
const Segment SGI204288[] = {{&S78372[30], 6}, {NULL}};
const Segment SGI204289[] = {{&S73698[44], 22}, {NULL}};
const Segment SGI204290[] = {{&S73698[30], 6}, {NULL}};
const Segment SGI204291[] = {{&S84013[31], 3}, {NULL}};
const Segment SGI204292[] = {{&S78372[125], 3}, {NULL}};
const Segment SGI204293[] = {{&S84013[84], 6}, {NULL}};
const Segment SGI204294[] = {{&S73698[131], 3}, {NULL}};
const Segment SGI204295[] = {{&S78372[90], 6}, {NULL}};
const Segment SGI204296[] = {{&S73698[96], 6}, {NULL}};
const Segment SGI204297[] = {{&S73698[68], 8}, {NULL}};
const Segment SGI204298[] = {{&S78373[38], 20}, {NULL}};
const Segment SGI204299[] = {{&S78373[31], 6}, {NULL}};
const Segment SGI204300[] = {{&S73699[44], 10}, {NULL}};
const Segment SGI204301[] = {{&S73699[44], 22}, {NULL}};
const Segment SGI204302[] = {{&S73699[30], 6}, {NULL}};
const Segment SGI204303[] = {{&S73699[131], 3}, {NULL}};
const Segment SGI204304[] = {{&S73699[96], 6}, {NULL}};
const Segment SGI204305[] = {{&S73699[68], 8}, {NULL}};
const Segment SGI204306[] = {{&S73700[44], 10}, {NULL}};
const Segment SGI204307[] = {{&S73700[44], 22}, {NULL}};
const Segment SGI204308[] = {{&S73700[30], 6}, {NULL}};
const Segment SGI204309[] = {{&S73700[141], 3}, {NULL}};
const Segment SGI204310[] = {{&S73700[106], 6}, {NULL}};
const Segment SGI204311[] = {{&S73700[68], 8}, {NULL}};
const Segment SGI204312[] = {{&S73701[44], 10}, {NULL}};
const Segment SGI204313[] = {{&S73701[44], 22}, {NULL}};
const Segment SGI204314[] = {{&S73701[30], 6}, {NULL}};
const Segment SGI204315[] = {{&S84019[38], 14}, {NULL}};
const Segment SGI204316[] = {{&S73701[151], 3}, {NULL}};
const Segment SGI204317[] = {{&S84019[31], 6}, {NULL}};
const Segment SGI204318[] = {{&S73701[116], 6}, {NULL}};
const Segment SGI204319[] = {{&S73701[68], 8}, {&S73701[68], 8}, {NULL}};
const Segment SGI204320[] = {{&S73702[44], 9}, {NULL}};
const Segment SGI204321[] = {{&S73702[44], 46}, {NULL}};
const Segment SGI204322[] = {{&S73702[30], 6}, {NULL}};
const Segment SGI204323[] = {{&S73702[170], 3}, {NULL}};
const Segment SGI204324[] = {{&S73702[135], 6}, {NULL}};
const Segment SGI204325[] = {{&S73702[92], 8}, {NULL}};
const Segment SGI204326[] = {{&S73703[44], 10}, {NULL}};
const Segment SGI204327[] = {{&S73703[44], 22}, {NULL}};
const Segment SGI204328[] = {{&S73703[30], 6}, {NULL}};
const Segment SGI204329[] = {{&S73703[151], 3}, {NULL}};
const Segment SGI204330[] = {{&S73703[116], 6}, {NULL}};
const Segment SGI204331[] = {{&S73703[68], 8}, {&S73703[68], 8}, {NULL}};
const Segment SGI204332[] = {{&S84032[44], 15}, {NULL}};
const Segment SGI204333[] = {{&S84032[44], 27}, {NULL}};
const Segment SGI204334[] = {{&S84032[30], 6}, {NULL}};
const Segment SGI204335[] = {{&S84032[31], 3}, {NULL}};
const Segment SGI204336[] = {{&S84032[85], 6}, {NULL}};
const Segment SGI204337[] = {{&S84041[34], 7}, {NULL}};
const Segment SGI204338[] = {{&S84041[27], 6}, {NULL}};
const Segment SGI204339[] = {{&S84041[106], 9}, {NULL}};
const Segment SGI204340[] = {{&S84041[79], 6}, {NULL}};
const Segment SGI204341[] = {{&S84041[147], 8}, {NULL}};
const Segment SGI204342[] = {{&S73730[34], 10}, {NULL}};
const Segment SGI204343[] = {{&S73730[27], 6}, {NULL}};
const Segment SGI204344[] = {{&S73730[97], 9}, {NULL}};
const Segment SGI204345[] = {{&S73730[70], 6}, {NULL}};
const Segment SGI204346[] = {{&S78408[34], 25}, {NULL}};
const Segment SGI204347[] = {{&S78408[27], 6}, {NULL}};
const Segment SGI204348[] = {{&S78408[112], 9}, {NULL}};
const Segment SGI204349[] = {{&S78408[85], 6}, {NULL}};
const Segment SGI204350[] = {{&S84050[34], 7}, {NULL}};
const Segment SGI204351[] = {{&S84050[27], 6}, {NULL}};
const Segment SGI204352[] = {{&S84050[94], 9}, {NULL}};
const Segment SGI204353[] = {{&S84050[67], 6}, {NULL}};
const Segment SGI204354[] = {{&S78409[34], 25}, {NULL}};
const Segment SGI204355[] = {{&S78409[27], 6}, {NULL}};
const Segment SGI204356[] = {{&S73733[44], 7}, {NULL}};
const Segment SGI204357[] = {{&S78409[112], 9}, {NULL}};
const Segment SGI204358[] = {{&S73733[44], 19}, {NULL}};
const Segment SGI204359[] = {{&S78409[85], 6}, {NULL}};
const Segment SGI204360[] = {{&S73733[30], 6}, {NULL}};
const Segment SGI204361[] = {{&S73733[112], 3}, {NULL}};
const Segment SGI204362[] = {{&S84051[34], 4}, {NULL}};
const Segment SGI204363[] = {{&S73733[77], 6}, {NULL}};
const Segment SGI204364[] = {{&S84051[27], 6}, {NULL}};
const Segment SGI204365[] = {{&S84051[91], 9}, {NULL}};
const Segment SGI204366[] = {{&S84051[64], 6}, {NULL}};
const Segment SGI204367[] = {{&S73734[38], 7}, {NULL}};
const Segment SGI204368[] = {{&S73734[31], 6}, {NULL}};
const Segment SGI204369[] = {{&S63714[41], 84}, {NULL}};
const Segment SGI204370[] = {{&S63714[41], 95}, {NULL}};
const Segment SGI204371[] = {{&S63714[27], 6}, {NULL}};
const Segment SGI204372[] = {{&S63714[62], 3}, {NULL}};
const Segment SGI204373[] = {{&S63714[190], 9}, {NULL}};
const Segment SGI204374[] = {{&S63714[118], 6}, {NULL}};
const Segment SGI204375[] = {{&S63715[41], 18}, {NULL}};
const Segment SGI204376[] = {{&S63715[41], 29}, {NULL}};
const Segment SGI204377[] = {{&S63715[27], 6}, {NULL}};
const Segment SGI204378[] = {{&S63715[119], 3}, {NULL}};
const Segment SGI204379[] = {{&S63715[123], 9}, {NULL}};
const Segment SGI204380[] = {{&S63715[84], 6}, {NULL}};
const Segment SGI204381[] = {{&S73768[34], 9}, {NULL}};
const Segment SGI204382[] = {{&S73768[27], 6}, {NULL}};
const Segment SGI204383[] = {{&S73768[96], 9}, {NULL}};
const Segment SGI204384[] = {{&S73768[69], 6}, {NULL}};
const Segment SGI204385[] = {{&S73770[34], 10}, {NULL}};
const Segment SGI204386[] = {{&S73770[27], 6}, {NULL}};
const Segment SGI204387[] = {{&S73770[97], 9}, {NULL}};
const Segment SGI204388[] = {{&S73770[70], 6}, {NULL}};
const Segment SGI204389[] = {{&S73772[34], 10}, {NULL}};
const Segment SGI204390[] = {{&S73772[27], 6}, {NULL}};
const Segment SGI204391[] = {{&S73772[97], 9}, {NULL}};
const Segment SGI204392[] = {{&S73772[70], 6}, {NULL}};
const Segment SGI204393[] = {{&S84112[34], 7}, {NULL}};
const Segment SGI204394[] = {{&S84112[27], 6}, {NULL}};
const Segment SGI204395[] = {{&S84112[94], 9}, {NULL}};
const Segment SGI204396[] = {{&S84112[67], 6}, {NULL}};
const Segment SGI204397[] = {{&S63760[44], 10}, {NULL}};
const Segment SGI204398[] = {{&S63760[44], 21}, {NULL}};
const Segment SGI204399[] = {{&S63760[30], 6}, {NULL}};
const Segment SGI204400[] = {{&S84113[34], 11}, {NULL}};
const Segment SGI204401[] = {{&S84113[27], 6}, {NULL}};
const Segment SGI204402[] = {{&S63760[79], 6}, {NULL}};
const Segment SGI204403[] = {{&S84113[98], 9}, {NULL}};
const Segment SGI204404[] = {{&S84113[71], 6}, {NULL}};
const Segment SGI204405[] = {{&S63760[106], 13}, {NULL}};
const Segment SGI204406[] = {{&S78475[44], 10}, {NULL}};
const Segment SGI204407[] = {{&S78475[44], 17}, {NULL}};
const Segment SGI204408[] = {{&S78475[30], 6}, {NULL}};
const Segment SGI204409[] = {{&S78475[110], 3}, {NULL}};
const Segment SGI204410[] = {{&S78475[75], 6}, {NULL}};
const Segment SGI204411[] = {{&S84114[34], 10}, {NULL}};
const Segment SGI204412[] = {{&S84114[27], 6}, {NULL}};
const Segment SGI204413[] = {{&S84114[94], 9}, {NULL}};
const Segment SGI204414[] = {{&S84114[67], 6}, {NULL}};
const Segment SGI204415[] = {{&S78476[34], 11}, {NULL}};
const Segment SGI204416[] = {{&S78476[27], 6}, {NULL}};
const Segment SGI204417[] = {{&S78476[98], 9}, {NULL}};
const Segment SGI204418[] = {{&S78476[71], 6}, {NULL}};
const Segment SGI204419[] = {{&S78476[139], 8}, {NULL}};
const Segment SGI204420[] = {{&S84120[34], 8}, {NULL}};
const Segment SGI204421[] = {{&S84120[27], 6}, {NULL}};
const Segment SGI204422[] = {{&S84120[95], 9}, {NULL}};
const Segment SGI204423[] = {{&S84120[68], 6}, {NULL}};
const Segment SGI204424[] = {{&S72766[34], 6}, {NULL}};
const Segment SGI204425[] = {{&S72766[27], 6}, {NULL}};
const Segment SGI204426[] = {{&S72766[93], 9}, {NULL}};
const Segment SGI204427[] = {{&S72766[66], 6}, {NULL}};
const Segment SGI204428[] = {{&S84124[44], 26}, {NULL}};
const Segment SGI204429[] = {{&S84124[44], 38}, {NULL}};
const Segment SGI204430[] = {{&S84124[30], 6}, {NULL}};
const Segment SGI204431[] = {{&S84124[31], 3}, {NULL}};
const Segment SGI204432[] = {{&S84124[96], 6}, {NULL}};
const Segment SGI204433[] = {{&S73808[44], 13}, {NULL}};
const Segment SGI204434[] = {{&S73808[44], 25}, {NULL}};
const Segment SGI204435[] = {{&S73808[30], 6}, {NULL}};
const Segment SGI204436[] = {{&S73808[118], 3}, {NULL}};
const Segment SGI204437[] = {{&S73808[83], 6}, {NULL}};
const Segment SGI204438[] = {{&S84128[34], 11}, {NULL}};
const Segment SGI204439[] = {{&S84128[27], 6}, {NULL}};
const Segment SGI204440[] = {{&S63781[44], 9}, {NULL}};
const Segment SGI204441[] = {{&S63781[44], 20}, {NULL}};
const Segment SGI204442[] = {{&S63781[30], 6}, {NULL}};
const Segment SGI204443[] = {{&S84128[98], 9}, {NULL}};
const Segment SGI204444[] = {{&S84128[71], 6}, {NULL}};
const Segment SGI204445[] = {{&S63781[113], 3}, {NULL}};
const Segment SGI204446[] = {{&S63781[78], 6}, {NULL}};
const Segment SGI204447[] = {{&S63782[44], 29}, {NULL}};
const Segment SGI204448[] = {{&S63782[44], 40}, {NULL}};
const Segment SGI204449[] = {{&S63782[30], 6}, {NULL}};
const Segment SGI204450[] = {{&S63782[133], 3}, {NULL}};
const Segment SGI204451[] = {{&S63782[98], 6}, {NULL}};
const Segment SGI204452[] = {{&S63783[38], 9}, {NULL}};
const Segment SGI204453[] = {{&S63783[31], 6}, {NULL}};
const Segment SGI204454[] = {{&S84131[44], 10}, {NULL}};
const Segment SGI204455[] = {{&S84131[44], 22}, {NULL}};
const Segment SGI204456[] = {{&S84131[30], 6}, {NULL}};
const Segment SGI204457[] = {{&S84131[31], 3}, {NULL}};
const Segment SGI204458[] = {{&S84131[80], 6}, {NULL}};
const Segment SGI204459[] = {{&S84131[133], 11}, {NULL}};
const Segment SGI204460[] = {{&S63790[34], 25}, {NULL}};
const Segment SGI204461[] = {{&S63790[27], 6}, {NULL}};
const Segment SGI204462[] = {{&S63790[112], 9}, {NULL}};
const Segment SGI204463[] = {{&S63790[85], 6}, {NULL}};
const Segment SGI204464[] = {{&S84138[44], 10}, {NULL}};
const Segment SGI204465[] = {{&S84138[44], 30}, {NULL}};
const Segment SGI204466[] = {{&S84138[30], 6}, {NULL}};
const Segment SGI204467[] = {{&S63791[34], 12}, {NULL}};
const Segment SGI204468[] = {{&S63791[27], 6}, {NULL}};
const Segment SGI204469[] = {{&S84138[31], 3}, {NULL}};
const Segment SGI204470[] = {{&S84138[88], 6}, {NULL}};
const Segment SGI204471[] = {{&S63791[99], 9}, {NULL}};
const Segment SGI204472[] = {{&S63791[72], 6}, {NULL}};
const Segment SGI204473[] = {{&S78506[44], 10}, {NULL}};
const Segment SGI204474[] = {{&S78506[44], 22}, {NULL}};
const Segment SGI204475[] = {{&S78506[30], 6}, {NULL}};
const Segment SGI204476[] = {{&S78506[115], 3}, {NULL}};
const Segment SGI204477[] = {{&S78506[80], 6}, {NULL}};
const Segment SGI204478[] = {{&S84139[44], 6}, {NULL}};
const Segment SGI204479[] = {{&S84139[44], 18}, {NULL}};
const Segment SGI204480[] = {{&S84139[30], 6}, {NULL}};
const Segment SGI204481[] = {{&S63792[34], 10}, {NULL}};
const Segment SGI204482[] = {{&S63792[27], 6}, {NULL}};
const Segment SGI204483[] = {{&S84139[76], 6}, {NULL}};
const Segment SGI204484[] = {{&S84139[103], 13}, {NULL}};
const Segment SGI204485[] = {{&S63792[97], 9}, {NULL}};
const Segment SGI204486[] = {{&S63792[70], 6}, {NULL}};
const Segment SGI204487[] = {{&S78507[44], 9}, {NULL}};
const Segment SGI204488[] = {{&S78507[44], 21}, {NULL}};
const Segment SGI204489[] = {{&S78507[30], 6}, {NULL}};
const Segment SGI204490[] = {{&S78507[114], 3}, {NULL}};
const Segment SGI204491[] = {{&S78507[79], 6}, {NULL}};
const Segment SGI204492[] = {{&S84140[44], 10}, {NULL}};
const Segment SGI204493[] = {{&S84140[44], 22}, {NULL}};
const Segment SGI204494[] = {{&S84140[30], 6}, {NULL}};
const Segment SGI204495[] = {{&S84140[80], 6}, {NULL}};
const Segment SGI204496[] = {{&S78508[38], 9}, {NULL}};
const Segment SGI204497[] = {{&S78508[31], 6}, {NULL}};
const Segment SGI204498[] = {{&S84140[107], 13}, {NULL}};
const Segment SGI204499[] = {{&S73830[34], 10}, {NULL}};
const Segment SGI204500[] = {{&S84141[44], 14}, {NULL}};
const Segment SGI204501[] = {{&S73830[27], 6}, {NULL}};
const Segment SGI204502[] = {{&S84141[44], 26}, {NULL}};
const Segment SGI204503[] = {{&S84141[30], 6}, {NULL}};
const Segment SGI204504[] = {{&S78509[34], 10}, {NULL}};
const Segment SGI204505[] = {{&S78509[27], 6}, {NULL}};
const Segment SGI204506[] = {{&S73830[97], 9}, {NULL}};
const Segment SGI204507[] = {{&S73830[70], 6}, {NULL}};
const Segment SGI204508[] = {{&S84141[84], 6}, {NULL}};
const Segment SGI204509[] = {{&S78509[102], 9}, {NULL}};
const Segment SGI204510[] = {{&S84141[111], 13}, {NULL}};
const Segment SGI204511[] = {{&S78509[75], 6}, {NULL}};
const Segment SGI204512[] = {{&S84142[44], 15}, {NULL}};
const Segment SGI204513[] = {{&S84142[44], 27}, {NULL}};
const Segment SGI204514[] = {{&S84142[30], 6}, {NULL}};
const Segment SGI204515[] = {{&S63799[44], 10}, {NULL}};
const Segment SGI204516[] = {{&S63799[44], 21}, {NULL}};
const Segment SGI204517[] = {{&S84142[31], 3}, {NULL}};
const Segment SGI204518[] = {{&S63799[30], 6}, {NULL}};
const Segment SGI204519[] = {{&S84142[85], 6}, {NULL}};
const Segment SGI204520[] = {{&S63799[114], 3}, {NULL}};
const Segment SGI204521[] = {{&S78511[44], 19}, {NULL}};
const Segment SGI204522[] = {{&S63799[79], 6}, {NULL}};
const Segment SGI204523[] = {{&S78511[44], 31}, {NULL}};
const Segment SGI204524[] = {{&S78511[30], 6}, {NULL}};
const Segment SGI204525[] = {{&S78511[124], 3}, {NULL}};
const Segment SGI204526[] = {{&S73833[34], 9}, {NULL}};
const Segment SGI204527[] = {{&S78511[89], 6}, {NULL}};
const Segment SGI204528[] = {{&S73833[27], 6}, {NULL}};
const Segment SGI204529[] = {{&S73833[96], 9}, {NULL}};
const Segment SGI204530[] = {{&S73833[69], 6}, {NULL}};
const Segment SGI204531[] = {{&S63800[38], 10}, {NULL}};
const Segment SGI204532[] = {{&S63800[31], 6}, {NULL}};
const Segment SGI204533[] = {{&S78512[44], 19}, {NULL}};
const Segment SGI204534[] = {{&S78512[44], 31}, {NULL}};
const Segment SGI204535[] = {{&S78512[30], 6}, {NULL}};
const Segment SGI204536[] = {{&S73834[34], 9}, {NULL}};
const Segment SGI204537[] = {{&S73834[27], 6}, {NULL}};
const Segment SGI204538[] = {{&S78512[124], 3}, {NULL}};
const Segment SGI204539[] = {{&S78512[89], 6}, {NULL}};
const Segment SGI204540[] = {{&S73834[96], 9}, {NULL}};
const Segment SGI204541[] = {{&S73834[69], 6}, {NULL}};
const Segment SGI204542[] = {{&S78513[44], 19}, {NULL}};
const Segment SGI204543[] = {{&S78513[44], 31}, {NULL}};
const Segment SGI204544[] = {{&S78513[30], 6}, {NULL}};
const Segment SGI204545[] = {{&S73835[34], 15}, {NULL}};
const Segment SGI204546[] = {{&S78513[124], 3}, {NULL}};
const Segment SGI204547[] = {{&S73835[27], 6}, {NULL}};
const Segment SGI204548[] = {{&S78513[89], 6}, {NULL}};
const Segment SGI204549[] = {{&S73835[114], 9}, {NULL}};
const Segment SGI204550[] = {{&S73835[87], 6}, {NULL}};
const Segment SGI204551[] = {{&S73835[155], 8}, {NULL}};
const Segment SGI204552[] = {{&S84152[44], 6}, {NULL}};
const Segment SGI204553[] = {{&S84152[44], 18}, {NULL}};
const Segment SGI204554[] = {{&S84152[30], 6}, {NULL}};
const Segment SGI204555[] = {{&S84152[76], 6}, {NULL}};
const Segment SGI204556[] = {{&S84152[103], 13}, {NULL}};
const Segment SGI204557[] = {{&S73840[44], 7}, {NULL}};
const Segment SGI204558[] = {{&S73840[44], 23}, {NULL}};
const Segment SGI204559[] = {{&S73840[30], 6}, {NULL}};
const Segment SGI204560[] = {{&S73840[116], 3}, {NULL}};
const Segment SGI204561[] = {{&S73840[81], 6}, {NULL}};
const Segment SGI204562[] = {{&S73841[38], 7}, {NULL}};
const Segment SGI204563[] = {{&S73841[31], 6}, {NULL}};
const Segment SGI204564[] = {{&S63812[34], 19}, {NULL}};
const Segment SGI204565[] = {{&S63812[27], 6}, {NULL}};
const Segment SGI204566[] = {{&S63812[105], 9}, {NULL}};
const Segment SGI204567[] = {{&S63812[78], 6}, {NULL}};
const Segment SGI204568[] = {{&S73842[44], 7}, {NULL}};
const Segment SGI204569[] = {{&S73842[44], 23}, {NULL}};
const Segment SGI204570[] = {{&S73842[30], 6}, {NULL}};
const Segment SGI204571[] = {{&S73842[116], 3}, {NULL}};
const Segment SGI204572[] = {{&S73842[81], 6}, {NULL}};
const Segment SGI204573[] = {{&S63814[34], 13}, {NULL}};
const Segment SGI204574[] = {{&S63814[27], 6}, {NULL}};
const Segment SGI204575[] = {{&S63814[100], 9}, {NULL}};
const Segment SGI204576[] = {{&S63814[73], 6}, {NULL}};
const Segment SGI204577[] = {{&S73843[44], 7}, {NULL}};
const Segment SGI204578[] = {{&S73843[44], 23}, {NULL}};
const Segment SGI204579[] = {{&S73843[30], 6}, {NULL}};
const Segment SGI204580[] = {{&S73843[116], 3}, {NULL}};
const Segment SGI204581[] = {{&S73843[81], 6}, {NULL}};
const Segment SGI204582[] = {{&S73844[44], 7}, {NULL}};
const Segment SGI204583[] = {{&S73844[44], 23}, {NULL}};
const Segment SGI204584[] = {{&S73844[30], 6}, {NULL}};
const Segment SGI204585[] = {{&S73844[116], 3}, {NULL}};
const Segment SGI204586[] = {{&S73844[81], 6}, {NULL}};
const Segment SGI204587[] = {{&S73846[34], 16}, {NULL}};
const Segment SGI204588[] = {{&S73846[27], 6}, {NULL}};
const Segment SGI204589[] = {{&S73846[110], 9}, {NULL}};
const Segment SGI204590[] = {{&S73846[83], 6}, {NULL}};
const Segment SGI204591[] = {{&S78549[34], 6}, {NULL}};
const Segment SGI204592[] = {{&S78549[27], 6}, {NULL}};
const Segment SGI204593[] = {{&S78549[93], 9}, {NULL}};
const Segment SGI204594[] = {{&S78549[66], 6}, {NULL}};
const Segment SGI204595[] = {{&S73869[34], 15}, {NULL}};
const Segment SGI204596[] = {{&S73869[27], 6}, {NULL}};
const Segment SGI204597[] = {{&S78558[34], 13}, {NULL}};
const Segment SGI204598[] = {{&S78558[27], 6}, {NULL}};
const Segment SGI204599[] = {{&S73869[102], 9}, {NULL}};
const Segment SGI204600[] = {{&S73869[75], 6}, {NULL}};
const Segment SGI204601[] = {{&S78558[100], 9}, {NULL}};
const Segment SGI204602[] = {{&S78558[73], 6}, {NULL}};
const Segment SGI204603[] = {{&S73870[34], 7}, {NULL}};
const Segment SGI204604[] = {{&S73870[27], 6}, {NULL}};
const Segment SGI204605[] = {{&S73870[98], 9}, {NULL}};
const Segment SGI204606[] = {{&S73870[71], 6}, {NULL}};
const Segment SGI204607[] = {{&S73871[34], 7}, {NULL}};
const Segment SGI204608[] = {{&S73871[27], 6}, {NULL}};
const Segment SGI204609[] = {{&S73871[98], 9}, {NULL}};
const Segment SGI204610[] = {{&S73871[71], 6}, {NULL}};
const Segment SGI204611[] = {{&S78563[34], 7}, {NULL}};
const Segment SGI204612[] = {{&S78563[27], 6}, {NULL}};
const Segment SGI204613[] = {{&S78563[110], 9}, {NULL}};
const Segment SGI204614[] = {{&S78563[83], 6}, {NULL}};
const Segment SGI204615[] = {{&S84202[44], 8}, {NULL}};
const Segment SGI204616[] = {{&S84202[44], 20}, {NULL}};
const Segment SGI204617[] = {{&S84202[30], 6}, {NULL}};
const Segment SGI204618[] = {{&S73875[34], 16}, {NULL}};
const Segment SGI204619[] = {{&S73875[27], 6}, {NULL}};
const Segment SGI204620[] = {{&S84202[31], 3}, {NULL}};
const Segment SGI204621[] = {{&S84202[78], 6}, {NULL}};
const Segment SGI204622[] = {{&S73875[103], 9}, {NULL}};
const Segment SGI204623[] = {{&S73875[76], 6}, {NULL}};
const Segment SGI204624[] = {{&S73877[34], 16}, {NULL}};
const Segment SGI204625[] = {{&S73877[27], 6}, {NULL}};
const Segment SGI204626[] = {{&S73877[103], 9}, {NULL}};
const Segment SGI204627[] = {{&S73877[76], 6}, {NULL}};
const Segment SGI204628[] = {{&S63867[44], 7}, {NULL}};
const Segment SGI204629[] = {{&S63867[44], 18}, {NULL}};
const Segment SGI204630[] = {{&S63867[30], 6}, {NULL}};
const Segment SGI204631[] = {{&S63867[129], 3}, {NULL}};
const Segment SGI204632[] = {{&S63867[94], 6}, {NULL}};
const Segment SGI204633[] = {{&S63867[64], 8}, {NULL}};
const Segment SGI204634[] = {{&S84211[44], 12}, {NULL}};
const Segment SGI204635[] = {{&S84211[44], 24}, {NULL}};
const Segment SGI204636[] = {{&S84211[30], 6}, {NULL}};
const Segment SGI204637[] = {{&S84211[31], 3}, {NULL}};
const Segment SGI204638[] = {{&S84211[82], 6}, {NULL}};
const Segment SGI204639[] = {{&S63869[44], 7}, {NULL}};
const Segment SGI204640[] = {{&S84211[135], 11}, {NULL}};
const Segment SGI204641[] = {{&S63869[44], 19}, {NULL}};
const Segment SGI204642[] = {{&S63869[30], 6}, {NULL}};
const Segment SGI204643[] = {{&S63869[112], 3}, {NULL}};
const Segment SGI204644[] = {{&S63869[77], 6}, {NULL}};
const Segment SGI204645[] = {{&S63869[137], 8}, {NULL}};
const Segment SGI204646[] = {{&S84212[34], 7}, {NULL}};
const Segment SGI204647[] = {{&S84212[27], 6}, {NULL}};
const Segment SGI204648[] = {{&S84212[99], 9}, {NULL}};
const Segment SGI204649[] = {{&S84212[72], 6}, {NULL}};
const Segment SGI204650[] = {{&S84216[34], 10}, {NULL}};
const Segment SGI204651[] = {{&S84216[27], 6}, {NULL}};
const Segment SGI204652[] = {{&S84216[97], 9}, {NULL}};
const Segment SGI204653[] = {{&S84216[70], 6}, {NULL}};
const Segment SGI204654[] = {{&S84217[34], 10}, {NULL}};
const Segment SGI204655[] = {{&S84217[27], 6}, {NULL}};
const Segment SGI204656[] = {{&S78588[34], 6}, {NULL}};
const Segment SGI204657[] = {{&S78588[27], 6}, {NULL}};
const Segment SGI204658[] = {{&S84217[97], 9}, {NULL}};
const Segment SGI204659[] = {{&S84217[70], 6}, {NULL}};
const Segment SGI204660[] = {{&S78588[93], 9}, {NULL}};
const Segment SGI204661[] = {{&S78588[66], 6}, {NULL}};
const Segment SGI204662[] = {{&S72877[44], 10}, {NULL}};
const Segment SGI204663[] = {{&S72877[44], 22}, {NULL}};
const Segment SGI204664[] = {{&S72877[30], 6}, {NULL}};
const Segment SGI204665[] = {{&S72877[115], 3}, {NULL}};
const Segment SGI204666[] = {{&S72877[80], 6}, {NULL}};
const Segment SGI204667[] = {{&S84221[34], 5}, {NULL}};
const Segment SGI204668[] = {{&S84221[27], 6}, {NULL}};
const Segment SGI204669[] = {{&S84221[92], 9}, {NULL}};
const Segment SGI204670[] = {{&S84221[65], 6}, {NULL}};
const Segment SGI204671[] = {{&S84231[44], 7}, {NULL}};
const Segment SGI204672[] = {{&S84231[44], 25}, {NULL}};
const Segment SGI204673[] = {{&S84231[30], 6}, {NULL}};
const Segment SGI204674[] = {{&S84231[83], 6}, {NULL}};
const Segment SGI204675[] = {{&S84231[110], 13}, {NULL}};
const Segment SGI204676[] = {{&S78605[34], 7}, {NULL}};
const Segment SGI204677[] = {{&S78605[27], 6}, {NULL}};
const Segment SGI204678[] = {{&S78605[99], 9}, {NULL}};
const Segment SGI204679[] = {{&S78605[72], 6}, {NULL}};
const Segment SGI204680[] = {{&S84235[34], 7}, {NULL}};
const Segment SGI204681[] = {{&S84235[27], 6}, {NULL}};
const Segment SGI204682[] = {{&S78607[44], 15}, {NULL}};
const Segment SGI204683[] = {{&S78607[44], 27}, {NULL}};
const Segment SGI204684[] = {{&S78607[30], 6}, {NULL}};
const Segment SGI204685[] = {{&S84235[93], 9}, {NULL}};
const Segment SGI204686[] = {{&S84235[66], 6}, {NULL}};
const Segment SGI204687[] = {{&S78607[120], 3}, {NULL}};
const Segment SGI204688[] = {{&S84235[134], 8}, {NULL}};
const Segment SGI204689[] = {{&S78607[85], 6}, {NULL}};
const Segment SGI204690[] = {{&S78608[44], 13}, {NULL}};
const Segment SGI204691[] = {{&S78608[44], 25}, {NULL}};
const Segment SGI204692[] = {{&S78608[30], 6}, {NULL}};
const Segment SGI204693[] = {{&S78608[118], 3}, {NULL}};
const Segment SGI204694[] = {{&S78608[83], 6}, {NULL}};
const Segment SGI204695[] = {{&S78609[44], 11}, {NULL}};
const Segment SGI204696[] = {{&S78609[44], 23}, {NULL}};
const Segment SGI204697[] = {{&S78609[30], 6}, {NULL}};
const Segment SGI204698[] = {{&S84239[34], 12}, {NULL}};
const Segment SGI204699[] = {{&S84239[27], 6}, {NULL}};
const Segment SGI204700[] = {{&S78609[116], 3}, {NULL}};
const Segment SGI204701[] = {{&S78609[81], 6}, {NULL}};
const Segment SGI204702[] = {{&S84239[99], 9}, {NULL}};
const Segment SGI204703[] = {{&S84239[72], 6}, {NULL}};
const Segment SGI204704[] = {{&S73924[44], 6}, {NULL}};
const Segment SGI204705[] = {{&S73924[44], 18}, {NULL}};
const Segment SGI204706[] = {{&S73924[30], 6}, {NULL}};
const Segment SGI204707[] = {{&S73924[127], 3}, {NULL}};
const Segment SGI204708[] = {{&S73924[92], 6}, {NULL}};
const Segment SGI204709[] = {{&S73924[64], 8}, {NULL}};
const Segment SGI204710[] = {{&S72904[44], 13}, {NULL}};
const Segment SGI204711[] = {{&S72904[44], 25}, {NULL}};
const Segment SGI204712[] = {{&S72904[30], 6}, {NULL}};
const Segment SGI204713[] = {{&S84242[34], 24}, {NULL}};
const Segment SGI204714[] = {{&S73926[34], 13}, {NULL}};
const Segment SGI204715[] = {{&S73926[27], 6}, {NULL}};
const Segment SGI204716[] = {{&S84242[27], 6}, {NULL}};
const Segment SGI204717[] = {{&S72904[136], 3}, {NULL}};
const Segment SGI204718[] = {{&S72904[101], 6}, {NULL}};
const Segment SGI204719[] = {{&S72904[71], 8}, {NULL}};
const Segment SGI204720[] = {{&S73926[100], 9}, {NULL}};
const Segment SGI204721[] = {{&S84242[113], 9}, {NULL}};
const Segment SGI204722[] = {{&S73926[73], 6}, {NULL}};
const Segment SGI204723[] = {{&S84242[86], 6}, {NULL}};
const Segment SGI204724[] = {{&S73927[34], 19}, {NULL}};
const Segment SGI204725[] = {{&S84243[34], 23}, {NULL}};
const Segment SGI204726[] = {{&S73927[27], 6}, {NULL}};
const Segment SGI204727[] = {{&S78615[34], 6}, {NULL}};
const Segment SGI204728[] = {{&S84243[27], 6}, {NULL}};
const Segment SGI204729[] = {{&S78615[27], 6}, {NULL}};
const Segment SGI204730[] = {{&S73927[106], 9}, {NULL}};
const Segment SGI204731[] = {{&S73927[79], 6}, {NULL}};
const Segment SGI204732[] = {{&S84243[112], 9}, {NULL}};
const Segment SGI204733[] = {{&S84243[85], 6}, {NULL}};
const Segment SGI204734[] = {{&S78615[93], 9}, {NULL}};
const Segment SGI204735[] = {{&S78615[66], 6}, {NULL}};
const Segment SGI204736[] = {{&S84244[34], 19}, {NULL}};
const Segment SGI204737[] = {{&S84244[27], 6}, {NULL}};
const Segment SGI204738[] = {{&S84244[106], 9}, {NULL}};
const Segment SGI204739[] = {{&S84244[79], 6}, {NULL}};
const Segment SGI204740[] = {{&S84245[34], 18}, {NULL}};
const Segment SGI204741[] = {{&S84245[27], 6}, {NULL}};
const Segment SGI204742[] = {{&S84245[105], 9}, {NULL}};
const Segment SGI204743[] = {{&S84245[78], 6}, {NULL}};
const Segment SGI204744[] = {{&S84246[44], 15}, {NULL}};
const Segment SGI204745[] = {{&S84246[44], 27}, {NULL}};
const Segment SGI204746[] = {{&S84246[30], 6}, {NULL}};
const Segment SGI204747[] = {{&S84246[31], 3}, {NULL}};
const Segment SGI204748[] = {{&S84246[85], 6}, {NULL}};
const Segment SGI204749[] = {{&S84254[34], 13}, {NULL}};
const Segment SGI204750[] = {{&S84254[27], 6}, {NULL}};
const Segment SGI204751[] = {{&S84254[100], 9}, {NULL}};
const Segment SGI204752[] = {{&S84254[73], 6}, {NULL}};
const Segment SGI204753[] = {{&S73944[34], 10}, {NULL}};
const Segment SGI204754[] = {{&S73944[27], 6}, {NULL}};
const Segment SGI204755[] = {{&S73944[97], 9}, {NULL}};
const Segment SGI204756[] = {{&S73944[70], 6}, {NULL}};
const Segment SGI204757[] = {{&S84256[34], 18}, {NULL}};
const Segment SGI204758[] = {{&S84256[27], 6}, {NULL}};
const Segment SGI204759[] = {{&S84256[105], 9}, {NULL}};
const Segment SGI204760[] = {{&S84256[78], 6}, {NULL}};
const Segment SGI204761[] = {{&S63940[34], 10}, {NULL}};
const Segment SGI204762[] = {{&S63940[27], 6}, {NULL}};
const Segment SGI204763[] = {{&S63940[97], 9}, {NULL}};
const Segment SGI204764[] = {{&S63940[70], 6}, {NULL}};
const Segment SGI204765[] = {{&S63941[34], 10}, {NULL}};
const Segment SGI204766[] = {{&S63941[27], 6}, {NULL}};
const Segment SGI204767[] = {{&S63941[96], 9}, {NULL}};
const Segment SGI204768[] = {{&S63941[69], 6}, {NULL}};
const Segment SGI204769[] = {{&S78646[44], 13}, {NULL}};
const Segment SGI204770[] = {{&S78646[44], 25}, {NULL}};
const Segment SGI204771[] = {{&S78646[30], 6}, {NULL}};
const Segment SGI204772[] = {{&S78646[118], 3}, {NULL}};
const Segment SGI204773[] = {{&S78646[83], 6}, {NULL}};
const Segment SGI204774[] = {{&S73958[34], 12}, {NULL}};
const Segment SGI204775[] = {{&S73958[27], 6}, {NULL}};
const Segment SGI204776[] = {{&S73958[99], 9}, {NULL}};
const Segment SGI204777[] = {{&S73958[72], 6}, {NULL}};
const Segment SGI204778[] = {{&S73959[34], 12}, {NULL}};
const Segment SGI204779[] = {{&S73959[27], 6}, {NULL}};
const Segment SGI204780[] = {{&S73959[99], 9}, {NULL}};
const Segment SGI204781[] = {{&S73959[72], 6}, {NULL}};
const Segment SGI204782[] = {{&S78656[44], 11}, {NULL}};
const Segment SGI204783[] = {{&S78656[44], 23}, {NULL}};
const Segment SGI204784[] = {{&S78656[30], 6}, {NULL}};
const Segment SGI204785[] = {{&S78656[116], 3}, {NULL}};
const Segment SGI204786[] = {{&S78656[81], 6}, {NULL}};
const Segment SGI204787[] = {{&S73967[34], 16}, {NULL}};
const Segment SGI204788[] = {{&S73967[27], 6}, {NULL}};
const Segment SGI204789[] = {{&S73967[103], 9}, {NULL}};
const Segment SGI204790[] = {{&S73967[76], 6}, {NULL}};
const Segment SGI204791[] = {{&S78657[44], 7}, {NULL}};
const Segment SGI204792[] = {{&S78657[44], 19}, {NULL}};
const Segment SGI204793[] = {{&S78657[30], 6}, {NULL}};
const Segment SGI204794[] = {{&S78657[112], 3}, {NULL}};
const Segment SGI204795[] = {{&S78657[77], 6}, {NULL}};
const Segment SGI204796[] = {{&S73968[34], 25}, {NULL}};
const Segment SGI204797[] = {{&S73968[27], 6}, {NULL}};
const Segment SGI204798[] = {{&S73968[112], 9}, {NULL}};
const Segment SGI204799[] = {{&S73968[85], 6}, {NULL}};
const Segment SGI204800[] = {{&S84286[45], 7}, {NULL}};
const Segment SGI204801[] = {{&S84286[30], 6}, {NULL}};
const Segment SGI204802[] = {{&S84286[31], 3}, {NULL}};
const Segment SGI204803[] = {{&S84286[82], 5}, {NULL}};
const Segment SGI204804[] = {{&S84286[140], 13}, {NULL}};
const Segment SGI204805[] = {{&S73969[34], 16}, {NULL}};
const Segment SGI204806[] = {{&S73969[27], 6}, {NULL}};
const Segment SGI204807[] = {{&S73969[103], 9}, {NULL}};
const Segment SGI204808[] = {{&S73969[76], 6}, {NULL}};
const Segment SGI204809[] = {{&S84290[34], 4}, {NULL}};
const Segment SGI204810[] = {{&S84290[27], 6}, {NULL}};
const Segment SGI204811[] = {{&S84290[91], 9}, {NULL}};
const Segment SGI204812[] = {{&S84290[64], 6}, {NULL}};
const Segment SGI204813[] = {{&S84299[44], 5}, {NULL}};
const Segment SGI204814[] = {{&S84299[44], 17}, {NULL}};
const Segment SGI204815[] = {{&S84299[30], 6}, {NULL}};
const Segment SGI204816[] = {{&S84299[31], 3}, {NULL}};
const Segment SGI204817[] = {{&S84299[75], 6}, {NULL}};
const Segment SGI204818[] = {{&S84300[44], 10}, {NULL}};
const Segment SGI204819[] = {{&S84300[44], 22}, {NULL}};
const Segment SGI204820[] = {{&S84300[30], 6}, {NULL}};
const Segment SGI204821[] = {{&S84300[31], 3}, {NULL}};
const Segment SGI204822[] = {{&S84300[80], 6}, {NULL}};
const Segment SGI204823[] = {{&S84300[140], 11}, {NULL}};
const Segment SGI204824[] = {{&S84301[59], 7}, {NULL}};
const Segment SGI204825[] = {{&S84301[59], 19}, {NULL}};
const Segment SGI204826[] = {{&S84301[45], 6}, {NULL}};
const Segment SGI204827[] = {{&S84301[46], 3}, {NULL}};
const Segment SGI204828[] = {{&S84301[92], 6}, {NULL}};
const Segment SGI204829[] = {{&S84301[0], 14}, {NULL}};
const Segment SGI204830[] = {{&S78697[44], 10}, {NULL}};
const Segment SGI204831[] = {{&S78697[44], 22}, {NULL}};
const Segment SGI204832[] = {{&S78697[30], 6}, {NULL}};
const Segment SGI204833[] = {{&S78697[115], 3}, {NULL}};
const Segment SGI204834[] = {{&S78697[80], 6}, {NULL}};
const Segment SGI204835[] = {{&S78697[140], 11}, {NULL}};
const Segment SGI204836[] = {{&S74011[34], 11}, {NULL}};
const Segment SGI204837[] = {{&S74011[27], 6}, {NULL}};
const Segment SGI204838[] = {{&S74011[98], 9}, {NULL}};
const Segment SGI204839[] = {{&S74011[71], 6}, {NULL}};
const Segment SGI204840[] = {{&S74011[139], 8}, {NULL}};
const Segment SGI204841[] = {{&S74015[44], 10}, {NULL}};
const Segment SGI204842[] = {{&S74015[44], 22}, {NULL}};
const Segment SGI204843[] = {{&S74015[30], 6}, {NULL}};
const Segment SGI204844[] = {{&S74015[80], 6}, {NULL}};
const Segment SGI204845[] = {{&S74015[107], 13}, {NULL}};
const Segment SGI204846[] = {{&S73013[44], 11}, {NULL}};
const Segment SGI204847[] = {{&S73013[44], 23}, {NULL}};
const Segment SGI204848[] = {{&S73013[30], 6}, {NULL}};
const Segment SGI204849[] = {{&S73013[116], 3}, {NULL}};
const Segment SGI204850[] = {{&S73013[81], 6}, {NULL}};
const Segment SGI204851[] = {{&S73014[44], 10}, {NULL}};
const Segment SGI204852[] = {{&S73014[44], 22}, {NULL}};
const Segment SGI204853[] = {{&S73014[30], 6}, {NULL}};
const Segment SGI204854[] = {{&S73014[115], 3}, {NULL}};
const Segment SGI204855[] = {{&S73014[80], 6}, {NULL}};
const Segment SGI204856[] = {{&S78714[34], 12}, {NULL}};
const Segment SGI204857[] = {{&S78714[27], 6}, {NULL}};
const Segment SGI204858[] = {{&S73015[38], 11}, {NULL}};
const Segment SGI204859[] = {{&S73015[31], 6}, {NULL}};
const Segment SGI204860[] = {{&S78714[112], 9}, {NULL}};
const Segment SGI204861[] = {{&S78714[85], 6}, {NULL}};
const Segment SGI204862[] = {{&S74039[34], 7}, {NULL}};
const Segment SGI204863[] = {{&S74039[27], 6}, {NULL}};
const Segment SGI204864[] = {{&S74039[100], 9}, {NULL}};
const Segment SGI204865[] = {{&S74039[73], 6}, {NULL}};
const Segment SGI204866[] = {{&S78739[44], 13}, {NULL}};
const Segment SGI204867[] = {{&S78739[44], 30}, {NULL}};
const Segment SGI204868[] = {{&S78739[30], 6}, {NULL}};
const Segment SGI204869[] = {{&S78739[123], 3}, {NULL}};
const Segment SGI204870[] = {{&S78739[88], 6}, {NULL}};
const Segment SGI204871[] = {{&S74046[44], 10}, {NULL}};
const Segment SGI204872[] = {{&S74046[44], 22}, {NULL}};
const Segment SGI204873[] = {{&S74046[30], 6}, {NULL}};
const Segment SGI204874[] = {{&S74046[115], 3}, {NULL}};
const Segment SGI204875[] = {{&S74046[80], 6}, {NULL}};
const Segment SGI204876[] = {{&S74046[140], 7}, {NULL}};
const Segment SGI204877[] = {{&S78744[6], 4}, {NULL}};
const Segment SGI204878[] = {{&S78744[82], 5}, {NULL}};
const Segment SGI204879[] = {{&S78752[44], 17}, {NULL}};
const Segment SGI204880[] = {{&S78752[44], 38}, {NULL}};
const Segment SGI204881[] = {{&S78752[30], 6}, {NULL}};
const Segment SGI204882[] = {{&S78752[131], 3}, {NULL}};
const Segment SGI204883[] = {{&S78752[96], 6}, {NULL}};
const Segment SGI204884[] = {{&S74077[44], 7}, {NULL}};
const Segment SGI204885[] = {{&S74077[44], 19}, {NULL}};
const Segment SGI204886[] = {{&S74077[30], 6}, {NULL}};
const Segment SGI204887[] = {{&S74077[128], 3}, {NULL}};
const Segment SGI204888[] = {{&S74077[93], 6}, {NULL}};
const Segment SGI204889[] = {{&S74077[65], 8}, {NULL}};
const Segment SGI204890[] = {{&S74080[44], 9}, {NULL}};
const Segment SGI204891[] = {{&S74080[44], 21}, {NULL}};
const Segment SGI204892[] = {{&S74080[30], 6}, {NULL}};
const Segment SGI204893[] = {{&S74080[114], 3}, {NULL}};
const Segment SGI204894[] = {{&S74080[79], 6}, {NULL}};
const Segment SGI204895[] = {{&S74082[44], 13}, {NULL}};
const Segment SGI204896[] = {{&S74082[44], 25}, {NULL}};
const Segment SGI204897[] = {{&S74082[30], 6}, {NULL}};
const Segment SGI204898[] = {{&S74082[118], 3}, {NULL}};
const Segment SGI204899[] = {{&S74082[83], 6}, {NULL}};
const Segment SGI204900[] = {{&S74093[34], 9}, {NULL}};
const Segment SGI204901[] = {{&S74093[27], 6}, {NULL}};
const Segment SGI204902[] = {{&S74093[96], 9}, {NULL}};
const Segment SGI204903[] = {{&S74093[69], 6}, {NULL}};
const Segment SGI204904[] = {{&S78789[34], 15}, {NULL}};
const Segment SGI204905[] = {{&S78789[27], 6}, {NULL}};
const Segment SGI204906[] = {{&S78789[102], 9}, {NULL}};
const Segment SGI204907[] = {{&S78789[75], 6}, {NULL}};
const Segment SGI204908[] = {{&S78794[44], 10}, {NULL}};
const Segment SGI204909[] = {{&S78794[44], 22}, {NULL}};
const Segment SGI204910[] = {{&S78794[30], 6}, {NULL}};
const Segment SGI204911[] = {{&S78794[113], 3}, {NULL}};
const Segment SGI204912[] = {{&S78794[80], 5}, {NULL}};
const Segment SGI204913[] = {{&S64138[27], 6}, {NULL}};
const Segment SGI204914[] = {{&S64138[81], 9}, {NULL}};
const Segment SGI204915[] = {{&S64138[54], 6}, {NULL}};
const Segment SGI204916[] = {{&S64139[34], 19}, {NULL}};
const Segment SGI204917[] = {{&S64139[27], 6}, {NULL}};
const Segment SGI204918[] = {{&S64139[105], 9}, {NULL}};
const Segment SGI204919[] = {{&S64139[78], 6}, {NULL}};
const Segment SGI204920[] = {{&S64140[34], 17}, {NULL}};
const Segment SGI204921[] = {{&S64140[27], 6}, {NULL}};
const Segment SGI204922[] = {{&S64140[103], 9}, {NULL}};
const Segment SGI204923[] = {{&S64140[76], 6}, {NULL}};
const Segment SGI204924[] = {{&S78819[34], 17}, {NULL}};
const Segment SGI204925[] = {{&S78819[27], 6}, {NULL}};
const Segment SGI204926[] = {{&S78819[104], 9}, {NULL}};
const Segment SGI204927[] = {{&S78819[77], 6}, {NULL}};
const Segment SGI204928[] = {{&S64143[34], 4}, {NULL}};
const Segment SGI204929[] = {{&S64143[27], 6}, {NULL}};
const Segment SGI204930[] = {{&S64143[90], 9}, {NULL}};
const Segment SGI204931[] = {{&S64143[63], 6}, {NULL}};
const Segment SGI204932[] = {{&S74126[44], 10}, {NULL}};
const Segment SGI204933[] = {{&S64144[34], 6}, {NULL}};
const Segment SGI204934[] = {{&S74126[44], 22}, {NULL}};
const Segment SGI204935[] = {{&S74126[30], 6}, {NULL}};
const Segment SGI204936[] = {{&S64144[27], 6}, {NULL}};
const Segment SGI204937[] = {{&S74126[115], 3}, {NULL}};
const Segment SGI204938[] = {{&S64144[92], 9}, {NULL}};
const Segment SGI204939[] = {{&S74126[80], 6}, {NULL}};
const Segment SGI204940[] = {{&S64144[65], 6}, {NULL}};
const Segment SGI204941[] = {{&S74127[38], 10}, {NULL}};
const Segment SGI204942[] = {{&S74127[31], 6}, {NULL}};
const Segment SGI204943[] = {{&S78825[44], 9}, {NULL}};
const Segment SGI204944[] = {{&S78825[44], 21}, {NULL}};
const Segment SGI204945[] = {{&S78825[30], 6}, {NULL}};
const Segment SGI204946[] = {{&S78825[114], 3}, {NULL}};
const Segment SGI204947[] = {{&S78825[79], 6}, {NULL}};
const Segment SGI204948[] = {{&S64147[34], 9}, {NULL}};
const Segment SGI204949[] = {{&S64147[27], 6}, {NULL}};
const Segment SGI204950[] = {{&S64147[95], 9}, {NULL}};
const Segment SGI204951[] = {{&S64147[68], 6}, {NULL}};
const Segment SGI204952[] = {{&S78827[34], 10}, {NULL}};
const Segment SGI204953[] = {{&S78827[27], 6}, {NULL}};
const Segment SGI204954[] = {{&S64149[34], 18}, {NULL}};
const Segment SGI204955[] = {{&S78827[97], 9}, {NULL}};
const Segment SGI204956[] = {{&S78827[70], 6}, {NULL}};
const Segment SGI204957[] = {{&S64149[27], 6}, {NULL}};
const Segment SGI204958[] = {{&S64149[104], 9}, {NULL}};
const Segment SGI204959[] = {{&S64149[77], 6}, {NULL}};
const Segment SGI204960[] = {{&S64152[34], 10}, {NULL}};
const Segment SGI204961[] = {{&S64152[27], 6}, {NULL}};
const Segment SGI204962[] = {{&S64152[96], 9}, {NULL}};
const Segment SGI204963[] = {{&S64152[69], 6}, {NULL}};
const Segment SGI204964[] = {{&S64153[34], 6}, {NULL}};
const Segment SGI204965[] = {{&S64153[27], 6}, {NULL}};
const Segment SGI204966[] = {{&S64153[92], 9}, {NULL}};
const Segment SGI204967[] = {{&S64153[65], 6}, {NULL}};
const Segment SGI204968[] = {{&S84306[34], 12}, {NULL}};
const Segment SGI204969[] = {{&S84306[27], 6}, {NULL}};
const Segment SGI204970[] = {{&S84306[99], 9}, {NULL}};
const Segment SGI204971[] = {{&S84306[72], 6}, {NULL}};
const Segment SGI204972[] = {{&S64154[34], 11}, {NULL}};
const Segment SGI204973[] = {{&S64154[27], 6}, {NULL}};
const Segment SGI204974[] = {{&S64154[97], 9}, {NULL}};
const Segment SGI204975[] = {{&S64154[70], 6}, {NULL}};
const Segment SGI204976[] = {{&S64156[34], 8}, {NULL}};
const Segment SGI204977[] = {{&S64156[27], 6}, {NULL}};
const Segment SGI204978[] = {{&S64156[94], 9}, {NULL}};
const Segment SGI204979[] = {{&S64156[67], 6}, {NULL}};
const Segment SGI204980[] = {{&S78839[44], 7}, {NULL}};
const Segment SGI204981[] = {{&S78839[44], 19}, {NULL}};
const Segment SGI204982[] = {{&S78839[30], 6}, {NULL}};
const Segment SGI204983[] = {{&S78839[112], 3}, {NULL}};
const Segment SGI204984[] = {{&S78839[77], 6}, {NULL}};
const Segment SGI204985[] = {{&S64157[34], 6}, {NULL}};
const Segment SGI204986[] = {{&S64157[27], 6}, {NULL}};
const Segment SGI204987[] = {{&S64157[91], 9}, {NULL}};
const Segment SGI204988[] = {{&S64157[64], 6}, {NULL}};
const Segment SGI204989[] = {{&S78840[38], 7}, {NULL}};
const Segment SGI204990[] = {{&S84312[44], 11}, {NULL}};
const Segment SGI204991[] = {{&S78840[31], 6}, {NULL}};
const Segment SGI204992[] = {{&S84312[44], 23}, {NULL}};
const Segment SGI204993[] = {{&S84312[30], 6}, {NULL}};
const Segment SGI204994[] = {{&S84312[31], 3}, {NULL}};
const Segment SGI204995[] = {{&S84312[81], 6}, {NULL}};
const Segment SGI204996[] = {{&S64158[34], 10}, {NULL}};
const Segment SGI204997[] = {{&S64158[27], 6}, {NULL}};
const Segment SGI204998[] = {{&S64158[96], 9}, {NULL}};
const Segment SGI204999[] = {{&S64158[69], 6}, {NULL}};
const Segment SGI205000[] = {{&S84318[34], 10}, {NULL}};
const Segment SGI205001[] = {{&S84318[27], 6}, {NULL}};
const Segment SGI205002[] = {{&S84318[97], 9}, {NULL}};
const Segment SGI205003[] = {{&S84318[70], 6}, {NULL}};
const Segment SGI205004[] = {{&S84320[34], 10}, {NULL}};
const Segment SGI205005[] = {{&S84320[27], 6}, {NULL}};
const Segment SGI205006[] = {{&S84320[97], 9}, {NULL}};
const Segment SGI205007[] = {{&S84320[70], 6}, {NULL}};
const Segment SGI205008[] = {{&S84321[34], 11}, {NULL}};
const Segment SGI205009[] = {{&S84321[27], 6}, {NULL}};
const Segment SGI205010[] = {{&S84321[98], 9}, {NULL}};
const Segment SGI205011[] = {{&S84321[71], 6}, {NULL}};
const Segment SGI205012[] = {{&S74160[34], 13}, {NULL}};
const Segment SGI205013[] = {{&S73173[34], 7}, {NULL}};
const Segment SGI205014[] = {{&S74160[27], 6}, {NULL}};
const Segment SGI205015[] = {{&S73173[27], 6}, {NULL}};
const Segment SGI205016[] = {{&S74160[100], 9}, {NULL}};
const Segment SGI205017[] = {{&S73173[94], 9}, {NULL}};
const Segment SGI205018[] = {{&S74160[73], 6}, {NULL}};
const Segment SGI205019[] = {{&S73173[67], 6}, {NULL}};
const Segment SGI205020[] = {{&S74161[34], 16}, {NULL}};
const Segment SGI205021[] = {{&S74161[27], 6}, {NULL}};
const Segment SGI205022[] = {{&S74161[103], 9}, {NULL}};
const Segment SGI205023[] = {{&S74161[76], 6}, {NULL}};
const Segment SGI205024[] = {{&S84331[34], 10}, {NULL}};
const Segment SGI205025[] = {{&S84331[27], 6}, {NULL}};
const Segment SGI205026[] = {{&S84331[97], 9}, {NULL}};
const Segment SGI205027[] = {{&S84331[70], 6}, {NULL}};
const Segment SGI205028[] = {{&S84332[34], 10}, {NULL}};
const Segment SGI205029[] = {{&S84332[27], 6}, {NULL}};
const Segment SGI205030[] = {{&S84332[97], 9}, {NULL}};
const Segment SGI205031[] = {{&S84332[70], 6}, {NULL}};
const Segment SGI205032[] = {{&S84335[34], 14}, {NULL}};
const Segment SGI205033[] = {{&S84335[27], 6}, {NULL}};
const Segment SGI205034[] = {{&S84335[106], 9}, {NULL}};
const Segment SGI205035[] = {{&S84335[79], 6}, {NULL}};
const Segment SGI205036[] = {{&S64202[44], 11}, {NULL}};
const Segment SGI205037[] = {{&S64202[44], 22}, {NULL}};
const Segment SGI205038[] = {{&S64202[30], 6}, {NULL}};
const Segment SGI205039[] = {{&S64202[133], 3}, {NULL}};
const Segment SGI205040[] = {{&S64202[98], 6}, {NULL}};
const Segment SGI205041[] = {{&S64202[68], 8}, {NULL}};
const Segment SGI205042[] = {{&S64203[34], 7}, {NULL}};
const Segment SGI205043[] = {{&S64203[27], 6}, {NULL}};
const Segment SGI205044[] = {{&S64203[100], 9}, {NULL}};
const Segment SGI205045[] = {{&S64203[73], 6}, {NULL}};
const Segment SGI205046[] = {{&S78893[44], 9}, {NULL}};
const Segment SGI205047[] = {{&S78893[44], 21}, {NULL}};
const Segment SGI205048[] = {{&S78893[30], 6}, {NULL}};
const Segment SGI205049[] = {{&S78893[114], 3}, {NULL}};
const Segment SGI205050[] = {{&S78893[79], 6}, {NULL}};
const Segment SGI205051[] = {{&S64212[34], 18}, {NULL}};
const Segment SGI205052[] = {{&S64212[27], 6}, {NULL}};
const Segment SGI205053[] = {{&S64212[105], 9}, {NULL}};
const Segment SGI205054[] = {{&S64212[78], 6}, {NULL}};
const Segment SGI205055[] = {{&S74200[34], 6}, {NULL}};
const Segment SGI205056[] = {{&S64213[34], 10}, {NULL}};
const Segment SGI205057[] = {{&S74200[27], 6}, {NULL}};
const Segment SGI205058[] = {{&S64213[27], 6}, {NULL}};
const Segment SGI205059[] = {{&S74200[93], 9}, {NULL}};
const Segment SGI205060[] = {{&S64213[96], 9}, {NULL}};
const Segment SGI205061[] = {{&S74200[66], 6}, {NULL}};
const Segment SGI205062[] = {{&S64213[69], 6}, {NULL}};
const Segment SGI205063[] = {{&S64215[44], 10}, {NULL}};
const Segment SGI205064[] = {{&S64215[44], 21}, {NULL}};
const Segment SGI205065[] = {{&S64215[30], 6}, {NULL}};
const Segment SGI205066[] = {{&S64215[114], 3}, {NULL}};
const Segment SGI205067[] = {{&S64215[79], 6}, {NULL}};
const Segment SGI205068[] = {{&S64216[38], 10}, {NULL}};
const Segment SGI205069[] = {{&S64216[31], 6}, {NULL}};
const Segment SGI205070[] = {{&S78905[44], 14}, {NULL}};
const Segment SGI205071[] = {{&S78905[44], 26}, {NULL}};
const Segment SGI205072[] = {{&S78905[30], 6}, {NULL}};
const Segment SGI205073[] = {{&S78905[119], 3}, {NULL}};
const Segment SGI205074[] = {{&S78905[84], 6}, {NULL}};
const Segment SGI205075[] = {{&S74211[34], 6}, {NULL}};
const Segment SGI205076[] = {{&S74211[27], 6}, {NULL}};
const Segment SGI205077[] = {{&S74211[93], 9}, {NULL}};
const Segment SGI205078[] = {{&S74211[66], 6}, {NULL}};
const Segment SGI205079[] = {{&S78909[34], 7}, {NULL}};
const Segment SGI205080[] = {{&S78909[27], 6}, {NULL}};
const Segment SGI205081[] = {{&S78909[94], 9}, {NULL}};
const Segment SGI205082[] = {{&S78909[67], 6}, {NULL}};
const Segment SGI205083[] = {{&S84395[34], 10}, {NULL}};
const Segment SGI205084[] = {{&S84395[27], 6}, {NULL}};
const Segment SGI205085[] = {{&S84395[97], 9}, {NULL}};
const Segment SGI205086[] = {{&S84395[70], 6}, {NULL}};
const Segment SGI205087[] = {{&S84402[44], 10}, {NULL}};
const Segment SGI205088[] = {{&S84402[44], 22}, {NULL}};
const Segment SGI205089[] = {{&S84402[30], 6}, {NULL}};
const Segment SGI205090[] = {{&S84402[31], 3}, {NULL}};
const Segment SGI205091[] = {{&S84402[80], 6}, {NULL}};
const Segment SGI205092[] = {{&S74255[34], 9}, {NULL}};
const Segment SGI205093[] = {{&S74255[27], 6}, {NULL}};
const Segment SGI205094[] = {{&S74255[101], 9}, {NULL}};
const Segment SGI205095[] = {{&S74255[74], 6}, {NULL}};
const Segment SGI205096[] = {{&S84409[44], 7}, {NULL}};
const Segment SGI205097[] = {{&S84409[44], 19}, {NULL}};
const Segment SGI205098[] = {{&S84409[30], 6}, {NULL}};
const Segment SGI205099[] = {{&S84409[31], 3}, {NULL}};
const Segment SGI205100[] = {{&S84409[106], 6}, {NULL}};
const Segment SGI205101[] = {{&S84409[65], 8}, {NULL}};
const Segment SGI205102[] = {{&S84409[77], 15}, {NULL}};
const Segment SGI205103[] = {{&S84415[45], 10}, {NULL}};
const Segment SGI205104[] = {{&S84415[30], 6}, {NULL}};
const Segment SGI205105[] = {{&S84415[31], 3}, {NULL}};
const Segment SGI205106[] = {{&S84415[81], 5}, {NULL}};
const Segment SGI205107[] = {{&S84415[139], 13}, {NULL}};
const Segment SGI205108[] = {{&S84416[34], 11}, {NULL}};
const Segment SGI205109[] = {{&S84416[27], 6}, {NULL}};
const Segment SGI205110[] = {{&S84416[98], 9}, {NULL}};
const Segment SGI205111[] = {{&S84416[71], 6}, {NULL}};
const Segment SGI205112[] = {{&S84422[31], 8}, {NULL}};
const Segment SGI205113[] = {{&S84422[54], 3}, {NULL}};
const Segment SGI205114[] = {{&S84423[44], 21}, {NULL}};
const Segment SGI205115[] = {{&S84423[44], 43}, {NULL}};
const Segment SGI205116[] = {{&S84423[30], 6}, {NULL}};
const Segment SGI205117[] = {{&S84423[31], 3}, {NULL}};
const Segment SGI205118[] = {{&S84423[101], 6}, {NULL}};
const Segment SGI205119[] = {{&S84425[44], 10}, {NULL}};
const Segment SGI205120[] = {{&S84425[44], 22}, {NULL}};
const Segment SGI205121[] = {{&S84425[30], 6}, {NULL}};
const Segment SGI205122[] = {{&S84425[31], 3}, {NULL}};
const Segment SGI205123[] = {{&S84425[80], 6}, {NULL}};
const Segment SGI205124[] = {{&S84426[44], 10}, {NULL}};
const Segment SGI205125[] = {{&S84426[44], 22}, {NULL}};
const Segment SGI205126[] = {{&S84426[30], 6}, {NULL}};
const Segment SGI205127[] = {{&S84426[31], 3}, {NULL}};
const Segment SGI205128[] = {{&S84426[80], 6}, {NULL}};
const Segment SGI205129[] = {{&S84427[34], 18}, {NULL}};
const Segment SGI205130[] = {{&S84427[27], 6}, {NULL}};
const Segment SGI205131[] = {{&S84427[105], 9}, {NULL}};
const Segment SGI205132[] = {{&S84427[78], 6}, {NULL}};
const Segment SGI205133[] = {{&S84432[34], 9}, {NULL}};
const Segment SGI205134[] = {{&S84432[27], 6}, {NULL}};
const Segment SGI205135[] = {{&S84432[96], 9}, {NULL}};
const Segment SGI205136[] = {{&S84432[69], 6}, {NULL}};
const Segment SGI205137[] = {{&S73310[44], 14}, {NULL}};
const Segment SGI205138[] = {{&S73310[44], 26}, {NULL}};
const Segment SGI205139[] = {{&S73310[30], 6}, {NULL}};
const Segment SGI205140[] = {{&S73310[119], 3}, {NULL}};
const Segment SGI205141[] = {{&S73310[84], 6}, {NULL}};
const Segment SGI205142[] = {{&S84439[45], 10}, {NULL}};
const Segment SGI205143[] = {{&S84439[30], 6}, {NULL}};
const Segment SGI205144[] = {{&S84439[31], 3}, {NULL}};
const Segment SGI205145[] = {{&S84439[81], 5}, {NULL}};
const Segment SGI205146[] = {{&S84439[139], 13}, {NULL}};
const Segment SGI205147[] = {{&S84441[34], 10}, {NULL}};
const Segment SGI205148[] = {{&S84441[27], 6}, {NULL}};
const Segment SGI205149[] = {{&S84441[97], 9}, {NULL}};
const Segment SGI205150[] = {{&S84441[70], 6}, {NULL}};
const Segment SGI205151[] = {{&S84441[131], 8}, {NULL}};
const Segment SGI205152[] = {{&S84445[34], 8}, {NULL}};
const Segment SGI205153[] = {{&S74310[34], 7}, {NULL}};
const Segment SGI205154[] = {{&S84445[27], 6}, {NULL}};
const Segment SGI205155[] = {{&S74310[27], 6}, {NULL}};
const Segment SGI205156[] = {{&S84445[95], 9}, {NULL}};
const Segment SGI205157[] = {{&S84445[68], 6}, {NULL}};
const Segment SGI205158[] = {{&S74310[99], 9}, {NULL}};
const Segment SGI205159[] = {{&S74310[72], 6}, {NULL}};
const Segment SGI205160[] = {{&S74310[140], 8}, {NULL}};
const Segment SGI205161[] = {{&S84446[34], 9}, {NULL}};
const Segment SGI205162[] = {{&S84446[27], 6}, {NULL}};
const Segment SGI205163[] = {{&S84446[96], 9}, {NULL}};
const Segment SGI205164[] = {{&S84446[69], 6}, {NULL}};
const Segment SGI205165[] = {{&S84448[44], 10}, {NULL}};
const Segment SGI205166[] = {{&S84448[44], 22}, {NULL}};
const Segment SGI205167[] = {{&S84448[30], 6}, {NULL}};
const Segment SGI205168[] = {{&S64300[6], 4}, {NULL}};
const Segment SGI205169[] = {{&S64300[76], 5}, {NULL}};
const Segment SGI205170[] = {{&S84448[80], 6}, {NULL}};
const Segment SGI205171[] = {{&S84448[107], 13}, {NULL}};
const Segment SGI205172[] = {{&S64301[41], 18}, {NULL}};
const Segment SGI205173[] = {{&S64301[41], 29}, {NULL}};
const Segment SGI205174[] = {{&S64301[27], 6}, {NULL}};
const Segment SGI205175[] = {{&S64301[119], 3}, {NULL}};
const Segment SGI205176[] = {{&S64301[123], 9}, {NULL}};
const Segment SGI205177[] = {{&S64301[84], 6}, {NULL}};
const Segment SGI205178[] = {{&S84453[34], 8}, {NULL}};
const Segment SGI205179[] = {{&S84453[27], 6}, {NULL}};
const Segment SGI205180[] = {{&S84453[95], 9}, {NULL}};
const Segment SGI205181[] = {{&S84453[68], 6}, {NULL}};
const Segment SGI205182[] = {{&S84454[34], 6}, {NULL}};
const Segment SGI205183[] = {{&S84454[27], 6}, {NULL}};
const Segment SGI205184[] = {{&S84454[93], 9}, {NULL}};
const Segment SGI205185[] = {{&S84454[66], 6}, {NULL}};
const Segment SGI205186[] = {{&S74326[34], 28}, {NULL}};
const Segment SGI205187[] = {{&S74326[27], 6}, {NULL}};
const Segment SGI205188[] = {{&S74326[115], 9}, {NULL}};
const Segment SGI205189[] = {{&S74326[88], 6}, {NULL}};
const Segment SGI205190[] = {{&S74327[34], 12}, {NULL}};
const Segment SGI205191[] = {{&S74327[27], 6}, {NULL}};
const Segment SGI205192[] = {{&S74327[99], 9}, {NULL}};
const Segment SGI205193[] = {{&S74327[72], 6}, {NULL}};
const Segment SGI205194[] = {{&S84463[44], 6}, {NULL}};
const Segment SGI205195[] = {{&S74330[34], 10}, {NULL}};
const Segment SGI205196[] = {{&S84463[44], 18}, {NULL}};
const Segment SGI205197[] = {{&S84463[30], 6}, {NULL}};
const Segment SGI205198[] = {{&S74330[27], 6}, {NULL}};
const Segment SGI205199[] = {{&S74330[97], 9}, {NULL}};
const Segment SGI205200[] = {{&S84463[76], 6}, {NULL}};
const Segment SGI205201[] = {{&S74330[70], 6}, {NULL}};
const Segment SGI205202[] = {{&S84463[103], 13}, {NULL}};
const Segment SGI205203[] = {{&S74331[34], 10}, {NULL}};
const Segment SGI205204[] = {{&S74331[27], 6}, {NULL}};
const Segment SGI205205[] = {{&S74331[97], 9}, {NULL}};
const Segment SGI205206[] = {{&S74331[70], 6}, {NULL}};
const Segment SGI205207[] = {{&S74332[34], 10}, {NULL}};
const Segment SGI205208[] = {{&S74332[27], 6}, {NULL}};
const Segment SGI205209[] = {{&S74332[97], 9}, {NULL}};
const Segment SGI205210[] = {{&S74332[70], 6}, {NULL}};
const Segment SGI205211[] = {{&S84467[34], 7}, {NULL}};
const Segment SGI205212[] = {{&S84467[27], 6}, {NULL}};
const Segment SGI205213[] = {{&S84467[99], 9}, {NULL}};
const Segment SGI205214[] = {{&S84467[72], 6}, {NULL}};
const Segment SGI205215[] = {{&S74335[34], 6}, {NULL}};
const Segment SGI205216[] = {{&S74335[27], 6}, {NULL}};
const Segment SGI205217[] = {{&S74335[93], 9}, {NULL}};
const Segment SGI205218[] = {{&S74335[66], 6}, {NULL}};
const Segment SGI205219[] = {{&S74336[34], 11}, {NULL}};
const Segment SGI205220[] = {{&S74336[27], 6}, {NULL}};
const Segment SGI205221[] = {{&S84472[34], 10}, {NULL}};
const Segment SGI205222[] = {{&S74336[98], 9}, {NULL}};
const Segment SGI205223[] = {{&S84472[27], 6}, {NULL}};
const Segment SGI205224[] = {{&S74336[71], 6}, {NULL}};
const Segment SGI205225[] = {{&S84472[97], 9}, {NULL}};
const Segment SGI205226[] = {{&S84472[70], 6}, {NULL}};
const Segment SGI205227[] = {{&S84472[131], 8}, {NULL}};
const Segment SGI205228[] = {{&S74337[34], 11}, {NULL}};
const Segment SGI205229[] = {{&S74337[27], 6}, {NULL}};
const Segment SGI205230[] = {{&S74337[98], 9}, {NULL}};
const Segment SGI205231[] = {{&S74337[71], 6}, {NULL}};
const Segment SGI205232[] = {{&S74338[34], 11}, {NULL}};
const Segment SGI205233[] = {{&S74338[27], 6}, {NULL}};
const Segment SGI205234[] = {{&S74338[98], 9}, {NULL}};
const Segment SGI205235[] = {{&S74338[71], 6}, {NULL}};
const Segment SGI205236[] = {{&S74341[34], 10}, {NULL}};
const Segment SGI205237[] = {{&S74341[27], 6}, {NULL}};
const Segment SGI205238[] = {{&S74341[97], 9}, {NULL}};
const Segment SGI205239[] = {{&S74341[70], 6}, {NULL}};
const Segment SGI205240[] = {{&S84480[44], 11}, {NULL}};
const Segment SGI205241[] = {{&S84480[44], 23}, {NULL}};
const Segment SGI205242[] = {{&S84480[30], 6}, {NULL}};
const Segment SGI205243[] = {{&S84480[31], 3}, {NULL}};
const Segment SGI205244[] = {{&S84480[81], 6}, {NULL}};
const Segment SGI205245[] = {{&S84481[44], 11}, {NULL}};
const Segment SGI205246[] = {{&S84481[44], 23}, {NULL}};
const Segment SGI205247[] = {{&S84481[30], 6}, {NULL}};
const Segment SGI205248[] = {{&S84481[31], 3}, {NULL}};
const Segment SGI205249[] = {{&S84481[81], 6}, {NULL}};
const Segment SGI205250[] = {{&S64342[34], 10}, {NULL}};
const Segment SGI205251[] = {{&S64342[27], 6}, {NULL}};
const Segment SGI205252[] = {{&S64342[96], 9}, {NULL}};
const Segment SGI205253[] = {{&S64342[69], 6}, {NULL}};
const Segment SGI205254[] = {{&S74351[34], 10}, {NULL}};
const Segment SGI205255[] = {{&S74351[27], 6}, {NULL}};
const Segment SGI205256[] = {{&S74351[97], 9}, {NULL}};
const Segment SGI205257[] = {{&S74351[70], 6}, {NULL}};
const Segment SGI205258[] = {{&S84507[45], 7}, {NULL}};
const Segment SGI205259[] = {{&S84507[30], 6}, {NULL}};
const Segment SGI205260[] = {{&S84507[31], 3}, {NULL}};
const Segment SGI205261[] = {{&S84507[83], 5}, {NULL}};
const Segment SGI205262[] = {{&S84507[141], 13}, {NULL}};
const Segment SGI205263[] = {{&S84511[34], 12}, {NULL}};
const Segment SGI205264[] = {{&S84511[27], 6}, {NULL}};
const Segment SGI205265[] = {{&S84511[99], 9}, {NULL}};
const Segment SGI205266[] = {{&S84511[72], 6}, {NULL}};
const Segment SGI205267[] = {{&S84512[59], 11}, {NULL}};
const Segment SGI205268[] = {{&S84512[59], 23}, {NULL}};
const Segment SGI205269[] = {{&S84512[45], 6}, {NULL}};
const Segment SGI205270[] = {{&S84512[46], 3}, {NULL}};
const Segment SGI205271[] = {{&S84512[96], 6}, {NULL}};
const Segment SGI205272[] = {{&S84512[0], 14}, {NULL}};
const Segment SGI205273[] = {{&S84516[44], 22}, {NULL}};
const Segment SGI205274[] = {{&S84516[44], 33}, {NULL}};
const Segment SGI205275[] = {{&S84516[30], 6}, {NULL}};
const Segment SGI205276[] = {{&S84516[31], 3}, {NULL}};
const Segment SGI205277[] = {{&S84516[91], 6}, {NULL}};
const Segment SGI205278[] = {{&S84517[44], 10}, {NULL}};
const Segment SGI205279[] = {{&S84517[44], 22}, {NULL}};
const Segment SGI205280[] = {{&S84517[30], 6}, {NULL}};
const Segment SGI205281[] = {{&S84517[31], 3}, {NULL}};
const Segment SGI205282[] = {{&S84517[80], 6}, {NULL}};
const Segment SGI205283[] = {{&S84523[34], 13}, {NULL}};
const Segment SGI205284[] = {{&S84523[27], 6}, {NULL}};
const Segment SGI205285[] = {{&S84523[100], 9}, {NULL}};
const Segment SGI205286[] = {{&S84523[73], 6}, {NULL}};
const Segment SGI205287[] = {{&S74396[44], 10}, {NULL}};
const Segment SGI205288[] = {{&S74396[44], 22}, {NULL}};
const Segment SGI205289[] = {{&S74396[30], 6}, {NULL}};
const Segment SGI205290[] = {{&S84525[44], 10}, {NULL}};
const Segment SGI205291[] = {{&S84525[44], 22}, {NULL}};
const Segment SGI205292[] = {{&S84525[30], 6}, {NULL}};
const Segment SGI205293[] = {{&S74396[113], 3}, {NULL}};
const Segment SGI205294[] = {{&S74396[80], 5}, {NULL}};
const Segment SGI205295[] = {{&S84525[31], 3}, {NULL}};
const Segment SGI205296[] = {{&S74396[117], 14}, {NULL}};
const Segment SGI205297[] = {{&S84525[80], 6}, {NULL}};
const Segment SGI205298[] = {{&S84525[143], 3}, {NULL}};
const Segment SGI205299[] = {{&S74398[44], 10}, {NULL}};
const Segment SGI205300[] = {{&S74398[44], 22}, {NULL}};
const Segment SGI205301[] = {{&S74398[30], 6}, {NULL}};
const Segment SGI205302[] = {{&S74398[113], 3}, {NULL}};
const Segment SGI205303[] = {{&S74398[80], 5}, {NULL}};
const Segment SGI205304[] = {{&S74398[117], 14}, {NULL}};
const Segment SGI205305[] = {{&S74399[34], 9}, {NULL}};
const Segment SGI205306[] = {{&S74399[27], 6}, {NULL}};
const Segment SGI205307[] = {{&S74399[96], 9}, {NULL}};
const Segment SGI205308[] = {{&S74399[69], 6}, {NULL}};
const Segment SGI205309[] = {{&S74400[34], 11}, {NULL}};
const Segment SGI205310[] = {{&S74400[27], 6}, {NULL}};
const Segment SGI205311[] = {{&S74400[98], 9}, {NULL}};
const Segment SGI205312[] = {{&S74400[71], 6}, {NULL}};
const Segment SGI205313[] = {{&S74401[34], 11}, {NULL}};
const Segment SGI205314[] = {{&S74401[27], 6}, {NULL}};
const Segment SGI205315[] = {{&S74401[98], 9}, {NULL}};
const Segment SGI205316[] = {{&S74401[71], 6}, {NULL}};
const Segment SGI205317[] = {{&S74402[34], 11}, {NULL}};
const Segment SGI205318[] = {{&S74402[27], 6}, {NULL}};
const Segment SGI205319[] = {{&S74402[98], 9}, {NULL}};
const Segment SGI205320[] = {{&S74402[71], 6}, {NULL}};
const Segment SGI205321[] = {{&S74414[44], 9}, {NULL}};
const Segment SGI205322[] = {{&S74414[44], 21}, {NULL}};
const Segment SGI205323[] = {{&S74414[30], 6}, {NULL}};
const Segment SGI205324[] = {{&S74414[130], 3}, {NULL}};
const Segment SGI205325[] = {{&S74414[95], 6}, {NULL}};
const Segment SGI205326[] = {{&S74414[67], 8}, {NULL}};
const Segment SGI205327[] = {{&S74419[34], 17}, {NULL}};
const Segment SGI205328[] = {{&S74419[27], 6}, {NULL}};
const Segment SGI205329[] = {{&S74419[104], 9}, {NULL}};
const Segment SGI205330[] = {{&S74419[77], 6}, {NULL}};
const Segment SGI205331[] = {{&S74420[34], 8}, {NULL}};
const Segment SGI205332[] = {{&S74420[27], 6}, {NULL}};
const Segment SGI205333[] = {{&S74420[103], 9}, {NULL}};
const Segment SGI205334[] = {{&S74420[76], 6}, {NULL}};
const Segment SGI205335[] = {{&S74421[34], 18}, {NULL}};
const Segment SGI205336[] = {{&S74421[27], 6}, {NULL}};
const Segment SGI205337[] = {{&S74421[113], 9}, {NULL}};
const Segment SGI205338[] = {{&S74421[86], 6}, {NULL}};
const Segment SGI205339[] = {{&S78998[44], 9}, {NULL}};
const Segment SGI205340[] = {{&S78998[44], 21}, {NULL}};
const Segment SGI205341[] = {{&S78998[30], 6}, {NULL}};
const Segment SGI205342[] = {{&S78998[114], 3}, {NULL}};
const Segment SGI205343[] = {{&S78998[79], 6}, {NULL}};
const Segment SGI205344[] = {{&S74426[44], 7}, {NULL}};
const Segment SGI205345[] = {{&S74426[44], 24}, {NULL}};
const Segment SGI205346[] = {{&S74426[30], 6}, {NULL}};
const Segment SGI205347[] = {{&S74426[82], 6}, {NULL}};
const Segment SGI205348[] = {{&S74426[109], 13}, {NULL}};
const Segment SGI205349[] = {{&S74427[44], 7}, {NULL}};
const Segment SGI205350[] = {{&S74427[44], 24}, {NULL}};
const Segment SGI205351[] = {{&S74427[30], 6}, {NULL}};
const Segment SGI205352[] = {{&S74427[82], 6}, {NULL}};
const Segment SGI205353[] = {{&S74427[109], 13}, {NULL}};
const Segment SGI205354[] = {{&S79008[34], 18}, {NULL}};
const Segment SGI205355[] = {{&S79008[27], 6}, {NULL}};
const Segment SGI205356[] = {{&S79008[114], 9}, {NULL}};
const Segment SGI205357[] = {{&S79008[87], 6}, {NULL}};
const Segment SGI205358[] = {{&S79010[34], 25}, {NULL}};
const Segment SGI205359[] = {{&S79010[27], 6}, {NULL}};
const Segment SGI205360[] = {{&S79010[112], 9}, {NULL}};
const Segment SGI205361[] = {{&S79010[85], 6}, {NULL}};
const Segment SGI205362[] = {{&S79011[34], 25}, {NULL}};
const Segment SGI205363[] = {{&S79011[27], 6}, {NULL}};
const Segment SGI205364[] = {{&S79011[112], 9}, {NULL}};
const Segment SGI205365[] = {{&S79011[85], 6}, {NULL}};
const Segment SGI205366[] = {{&S79012[34], 25}, {NULL}};
const Segment SGI205367[] = {{&S79012[27], 6}, {NULL}};
const Segment SGI205368[] = {{&S79012[112], 9}, {NULL}};
const Segment SGI205369[] = {{&S79012[85], 6}, {NULL}};
const Segment SGI205370[] = {{&S74454[34], 7}, {NULL}};
const Segment SGI205371[] = {{&S74454[27], 6}, {NULL}};
const Segment SGI205372[] = {{&S74454[97], 9}, {NULL}};
const Segment SGI205373[] = {{&S74454[70], 6}, {NULL}};
const Segment SGI205374[] = {{&S74454[138], 8}, {NULL}};
const Segment SGI205375[] = {{&S64480[44], 13}, {NULL}};
const Segment SGI205376[] = {{&S64480[44], 24}, {NULL}};
const Segment SGI205377[] = {{&S64480[30], 6}, {NULL}};
const Segment SGI205378[] = {{&S64480[135], 3}, {NULL}};
const Segment SGI205379[] = {{&S64480[100], 6}, {NULL}};
const Segment SGI205380[] = {{&S64480[70], 8}, {NULL}};
const Segment SGI205381[] = {{&S64481[44], 12}, {NULL}};
const Segment SGI205382[] = {{&S64481[44], 23}, {NULL}};
const Segment SGI205383[] = {{&S64481[30], 6}, {NULL}};
const Segment SGI205384[] = {{&S64481[134], 3}, {NULL}};
const Segment SGI205385[] = {{&S64481[99], 6}, {NULL}};
const Segment SGI205386[] = {{&S64481[69], 8}, {NULL}};
const Segment SGI205387[] = {{&S64483[44], 13}, {NULL}};
const Segment SGI205388[] = {{&S64483[44], 25}, {NULL}};
const Segment SGI205389[] = {{&S64483[30], 6}, {NULL}};
const Segment SGI205390[] = {{&S64483[136], 3}, {NULL}};
const Segment SGI205391[] = {{&S64483[101], 6}, {NULL}};
const Segment SGI205392[] = {{&S64483[71], 8}, {NULL}};
const Segment SGI205393[] = {{&S64484[44], 12}, {NULL}};
const Segment SGI205394[] = {{&S79043[44], 12}, {NULL}};
const Segment SGI205395[] = {{&S64484[44], 24}, {NULL}};
const Segment SGI205396[] = {{&S79043[44], 24}, {NULL}};
const Segment SGI205397[] = {{&S64484[30], 6}, {NULL}};
const Segment SGI205398[] = {{&S79043[30], 6}, {NULL}};
const Segment SGI205399[] = {{&S64484[135], 3}, {NULL}};
const Segment SGI205400[] = {{&S79043[117], 3}, {NULL}};
const Segment SGI205401[] = {{&S64484[100], 6}, {NULL}};
const Segment SGI205402[] = {{&S79043[82], 6}, {NULL}};
const Segment SGI205403[] = {{&S64484[70], 8}, {NULL}};
const Segment SGI205404[] = {{&S79045[44], 13}, {NULL}};
const Segment SGI205405[] = {{&S79045[44], 25}, {NULL}};
const Segment SGI205406[] = {{&S79045[30], 6}, {NULL}};
const Segment SGI205407[] = {{&S79045[118], 3}, {NULL}};
const Segment SGI205408[] = {{&S79045[83], 6}, {NULL}};
const Segment SGI205409[] = {{&S79046[38], 13}, {NULL}};
const Segment SGI205410[] = {{&S79046[31], 6}, {NULL}};
const Segment SGI205411[] = {{&S74498[44], 7}, {NULL}};
const Segment SGI205412[] = {{&S74498[44], 19}, {NULL}};
const Segment SGI205413[] = {{&S74498[30], 6}, {NULL}};
const Segment SGI205414[] = {{&S64508[34], 10}, {NULL}};
const Segment SGI205415[] = {{&S64508[27], 6}, {NULL}};
const Segment SGI205416[] = {{&S74498[112], 3}, {NULL}};
const Segment SGI205417[] = {{&S74498[77], 6}, {NULL}};
const Segment SGI205418[] = {{&S64508[96], 9}, {NULL}};
const Segment SGI205419[] = {{&S64508[69], 6}, {NULL}};
const Segment SGI205420[] = {{&S74498[144], 15}, {NULL}};
const Segment SGI205421[] = {{&S74499[44], 8}, {NULL}};
const Segment SGI205422[] = {{&S74499[44], 20}, {NULL}};
const Segment SGI205423[] = {{&S74499[30], 6}, {NULL}};
const Segment SGI205424[] = {{&S64509[34], 18}, {NULL}};
const Segment SGI205425[] = {{&S64509[27], 6}, {NULL}};
const Segment SGI205426[] = {{&S74499[113], 3}, {NULL}};
const Segment SGI205427[] = {{&S74499[78], 6}, {NULL}};
const Segment SGI205428[] = {{&S64509[104], 9}, {NULL}};
const Segment SGI205429[] = {{&S64509[77], 6}, {NULL}};
const Segment SGI205430[] = {{&S74499[145], 15}, {NULL}};
const Segment SGI205431[] = {{&S79068[44], 7}, {NULL}};
const Segment SGI205432[] = {{&S79068[44], 24}, {NULL}};
const Segment SGI205433[] = {{&S79068[30], 6}, {NULL}};
const Segment SGI205434[] = {{&S64510[34], 18}, {NULL}};
const Segment SGI205435[] = {{&S64510[27], 6}, {NULL}};
const Segment SGI205436[] = {{&S79068[82], 6}, {NULL}};
const Segment SGI205437[] = {{&S64510[104], 9}, {NULL}};
const Segment SGI205438[] = {{&S64510[77], 6}, {NULL}};
const Segment SGI205439[] = {{&S79068[109], 13}, {NULL}};
const Segment SGI205440[] = {{&S84534[44], 11}, {NULL}};
const Segment SGI205441[] = {{&S84534[44], 23}, {NULL}};
const Segment SGI205442[] = {{&S84534[30], 6}, {NULL}};
const Segment SGI205443[] = {{&S84534[31], 3}, {NULL}};
const Segment SGI205444[] = {{&S84534[81], 6}, {NULL}};
const Segment SGI205445[] = {{&S79069[44], 7}, {NULL}};
const Segment SGI205446[] = {{&S79069[44], 24}, {NULL}};
const Segment SGI205447[] = {{&S79069[30], 6}, {NULL}};
const Segment SGI205448[] = {{&S79069[82], 6}, {NULL}};
const Segment SGI205449[] = {{&S84535[44], 12}, {NULL}};
const Segment SGI205450[] = {{&S79069[109], 13}, {NULL}};
const Segment SGI205451[] = {{&S84535[44], 24}, {NULL}};
const Segment SGI205452[] = {{&S73544[44], 12}, {NULL}};
const Segment SGI205453[] = {{&S84535[30], 6}, {NULL}};
const Segment SGI205454[] = {{&S73544[44], 24}, {NULL}};
const Segment SGI205455[] = {{&S73544[30], 6}, {NULL}};
const Segment SGI205456[] = {{&S84535[31], 3}, {NULL}};
const Segment SGI205457[] = {{&S73544[117], 3}, {NULL}};
const Segment SGI205458[] = {{&S84535[82], 6}, {NULL}};
const Segment SGI205459[] = {{&S73544[82], 6}, {NULL}};
const Segment SGI205460[] = {{&S73549[34], 10}, {NULL}};
const Segment SGI205461[] = {{&S73549[27], 6}, {NULL}};
const Segment SGI205462[] = {{&S73549[97], 9}, {NULL}};
const Segment SGI205463[] = {{&S73549[70], 6}, {NULL}};
const Segment SGI205464[] = {{&S84544[44], 13}, {NULL}};
const Segment SGI205465[] = {{&S84544[44], 25}, {NULL}};
const Segment SGI205466[] = {{&S84544[30], 6}, {NULL}};
const Segment SGI205467[] = {{&S84544[31], 3}, {NULL}};
const Segment SGI205468[] = {{&S84544[83], 6}, {NULL}};
const Segment SGI205469[] = {{&S73554[34], 7}, {NULL}};
const Segment SGI205470[] = {{&S73554[27], 6}, {NULL}};
const Segment SGI205471[] = {{&S73554[94], 9}, {NULL}};
const Segment SGI205472[] = {{&S73554[67], 6}, {NULL}};
const Segment SGI205473[] = {{&S84545[44], 15}, {NULL}};
const Segment SGI205474[] = {{&S84545[44], 27}, {NULL}};
const Segment SGI205475[] = {{&S84545[30], 6}, {NULL}};
const Segment SGI205476[] = {{&S84545[31], 3}, {NULL}};
const Segment SGI205477[] = {{&S84545[85], 6}, {NULL}};
const Segment SGI205478[] = {{&S84549[34], 11}, {NULL}};
const Segment SGI205479[] = {{&S84549[27], 6}, {NULL}};
const Segment SGI205480[] = {{&S84549[107], 9}, {NULL}};
const Segment SGI205481[] = {{&S84549[80], 6}, {NULL}};
const Segment SGI205482[] = {{&S84552[34], 18}, {NULL}};
const Segment SGI205483[] = {{&S84552[27], 6}, {NULL}};
const Segment SGI205484[] = {{&S84552[105], 9}, {NULL}};
const Segment SGI205485[] = {{&S84552[78], 6}, {NULL}};
const Segment SGI205486[] = {{&S84552[146], 8}, {NULL}};
const Segment SGI205487[] = {{&S84555[34], 13}, {NULL}};
const Segment SGI205488[] = {{&S74503[44], 7}, {NULL}};
const Segment SGI205489[] = {{&S84555[27], 6}, {NULL}};
const Segment SGI205490[] = {{&S74503[44], 18}, {NULL}};
const Segment SGI205491[] = {{&S74503[30], 6}, {NULL}};
const Segment SGI205492[] = {{&S74503[111], 3}, {NULL}};
const Segment SGI205493[] = {{&S84555[100], 9}, {NULL}};
const Segment SGI205494[] = {{&S84555[73], 6}, {NULL}};
const Segment SGI205495[] = {{&S74503[76], 6}, {NULL}};
const Segment SGI205496[] = {{&S74504[38], 7}, {NULL}};
const Segment SGI205497[] = {{&S74504[31], 6}, {NULL}};
const Segment SGI205498[] = {{&S74509[44], 10}, {NULL}};
const Segment SGI205499[] = {{&S74509[44], 22}, {NULL}};
const Segment SGI205500[] = {{&S74509[30], 6}, {NULL}};
const Segment SGI205501[] = {{&S74509[80], 6}, {NULL}};
const Segment SGI205502[] = {{&S74509[107], 13}, {NULL}};
const Segment SGI205503[] = {{&S79108[34], 11}, {NULL}};
const Segment SGI205504[] = {{&S79108[27], 6}, {NULL}};
const Segment SGI205505[] = {{&S74514[34], 6}, {NULL}};
const Segment SGI205506[] = {{&S74514[27], 6}, {NULL}};
const Segment SGI205507[] = {{&S79108[98], 9}, {NULL}};
const Segment SGI205508[] = {{&S79108[71], 6}, {NULL}};
const Segment SGI205509[] = {{&S74514[93], 9}, {NULL}};
const Segment SGI205510[] = {{&S74514[66], 6}, {NULL}};
const Segment SGI205511[] = {{&S64553[34], 10}, {NULL}};
const Segment SGI205512[] = {{&S64553[27], 6}, {NULL}};
const Segment SGI205513[] = {{&S64553[96], 9}, {NULL}};
const Segment SGI205514[] = {{&S64553[69], 6}, {NULL}};
const Segment SGI205515[] = {{&S79110[34], 9}, {NULL}};
const Segment SGI205516[] = {{&S79110[27], 6}, {NULL}};
const Segment SGI205517[] = {{&S79110[96], 9}, {NULL}};
const Segment SGI205518[] = {{&S79110[69], 6}, {NULL}};
const Segment SGI205519[] = {{&S79110[137], 8}, {NULL}};
const Segment SGI205520[] = {{&S79117[34], 11}, {NULL}};
const Segment SGI205521[] = {{&S79117[27], 6}, {NULL}};
const Segment SGI205522[] = {{&S79117[98], 9}, {NULL}};
const Segment SGI205523[] = {{&S79117[71], 6}, {NULL}};
const Segment SGI205524[] = {{&S73596[34], 6}, {NULL}};
const Segment SGI205525[] = {{&S73596[27], 6}, {NULL}};
const Segment SGI205526[] = {{&S64568[44], 10}, {NULL}};
const Segment SGI205527[] = {{&S64568[44], 21}, {NULL}};
const Segment SGI205528[] = {{&S64568[30], 6}, {NULL}};
const Segment SGI205529[] = {{&S73596[93], 9}, {NULL}};
const Segment SGI205530[] = {{&S73596[66], 6}, {NULL}};
const Segment SGI205531[] = {{&S64568[113], 3}, {NULL}};
const Segment SGI205532[] = {{&S64568[79], 5}, {NULL}};
const Segment SGI205533[] = {{&S64570[44], 10}, {NULL}};
const Segment SGI205534[] = {{&S64570[44], 21}, {NULL}};
const Segment SGI205535[] = {{&S64570[30], 6}, {NULL}};
const Segment SGI205536[] = {{&S64570[113], 3}, {NULL}};
const Segment SGI205537[] = {{&S64570[79], 5}, {NULL}};
const Segment SGI205538[] = {{&S64571[44], 10}, {NULL}};
const Segment SGI205539[] = {{&S64571[44], 21}, {NULL}};
const Segment SGI205540[] = {{&S64571[30], 6}, {NULL}};
const Segment SGI205541[] = {{&S64571[114], 3}, {NULL}};
const Segment SGI205542[] = {{&S64571[79], 6}, {NULL}};
const Segment SGI205543[] = {{&S64572[44], 11}, {NULL}};
const Segment SGI205544[] = {{&S64572[44], 26}, {NULL}};
const Segment SGI205545[] = {{&S64572[30], 6}, {NULL}};
const Segment SGI205546[] = {{&S64572[118], 3}, {NULL}};
const Segment SGI205547[] = {{&S64572[84], 5}, {NULL}};
const Segment SGI205548[] = {{&S84595[44], 10}, {NULL}};
const Segment SGI205549[] = {{&S84595[44], 22}, {NULL}};
const Segment SGI205550[] = {{&S84595[30], 6}, {NULL}};
const Segment SGI205551[] = {{&S74540[44], 14}, {NULL}};
const Segment SGI205552[] = {{&S74540[44], 26}, {NULL}};
const Segment SGI205553[] = {{&S84595[31], 3}, {NULL}};
const Segment SGI205554[] = {{&S74540[30], 6}, {NULL}};
const Segment SGI205555[] = {{&S84595[80], 6}, {NULL}};
const Segment SGI205556[] = {{&S74540[135], 3}, {NULL}};
const Segment SGI205557[] = {{&S74540[100], 6}, {NULL}};
const Segment SGI205558[] = {{&S74540[72], 8}, {NULL}};
const Segment SGI205559[] = {{&S84601[45], 10}, {NULL}};
const Segment SGI205560[] = {{&S84601[30], 6}, {NULL}};
const Segment SGI205561[] = {{&S84601[31], 3}, {NULL}};
const Segment SGI205562[] = {{&S84601[81], 5}, {NULL}};
const Segment SGI205563[] = {{&S84601[139], 13}, {NULL}};
const Segment SGI205564[] = {{&S74547[34], 10}, {NULL}};
const Segment SGI205565[] = {{&S74547[27], 6}, {NULL}};
const Segment SGI205566[] = {{&S74547[97], 9}, {NULL}};
const Segment SGI205567[] = {{&S74547[70], 6}, {NULL}};
const Segment SGI205568[] = {{&S74548[34], 11}, {NULL}};
const Segment SGI205569[] = {{&S74548[27], 6}, {NULL}};
const Segment SGI205570[] = {{&S74548[98], 9}, {NULL}};
const Segment SGI205571[] = {{&S74548[71], 6}, {NULL}};
const Segment SGI205572[] = {{&S64585[34], 10}, {NULL}};
const Segment SGI205573[] = {{&S64585[27], 6}, {NULL}};
const Segment SGI205574[] = {{&S84604[44], 13}, {NULL}};
const Segment SGI205575[] = {{&S84604[44], 25}, {NULL}};
const Segment SGI205576[] = {{&S84604[30], 6}, {NULL}};
const Segment SGI205577[] = {{&S64585[97], 9}, {NULL}};
const Segment SGI205578[] = {{&S64585[70], 6}, {NULL}};
const Segment SGI205579[] = {{&S84604[31], 3}, {NULL}};
const Segment SGI205580[] = {{&S84604[83], 6}, {NULL}};
const Segment SGI205581[] = {{&S74549[34], 10}, {NULL}};
const Segment SGI205582[] = {{&S74549[27], 6}, {NULL}};
const Segment SGI205583[] = {{&S74549[97], 9}, {NULL}};
const Segment SGI205584[] = {{&S74549[70], 6}, {NULL}};
const Segment SGI205585[] = {{&S84605[44], 10}, {NULL}};
const Segment SGI205586[] = {{&S84605[44], 22}, {NULL}};
const Segment SGI205587[] = {{&S84605[30], 6}, {NULL}};
const Segment SGI205588[] = {{&S84605[31], 3}, {NULL}};
const Segment SGI205589[] = {{&S79142[44], 9}, {NULL}};
const Segment SGI205590[] = {{&S84605[99], 6}, {NULL}};
const Segment SGI205591[] = {{&S79142[44], 21}, {NULL}};
const Segment SGI205592[] = {{&S79142[30], 6}, {NULL}};
const Segment SGI205593[] = {{&S84605[68], 8}, {NULL}};
const Segment SGI205594[] = {{&S79142[143], 3}, {NULL}};
const Segment SGI205595[] = {{&S79142[108], 6}, {NULL}};
const Segment SGI205596[] = {{&S79142[67], 8}, {NULL}};
const Segment SGI205597[] = {{&S79142[79], 15}, {NULL}};
const Segment SGI205598[] = {{&S74552[34], 9}, {NULL}};
const Segment SGI205599[] = {{&S74552[27], 6}, {NULL}};
const Segment SGI205600[] = {{&S74552[96], 9}, {NULL}};
const Segment SGI205601[] = {{&S74552[69], 6}, {NULL}};
const Segment SGI205602[] = {{&S79143[44], 9}, {NULL}};
const Segment SGI205603[] = {{&S79143[44], 21}, {NULL}};
const Segment SGI205604[] = {{&S79143[30], 6}, {NULL}};
const Segment SGI205605[] = {{&S79143[143], 3}, {NULL}};
const Segment SGI205606[] = {{&S79143[108], 6}, {NULL}};
const Segment SGI205607[] = {{&S79143[67], 8}, {NULL}};
const Segment SGI205608[] = {{&S79143[79], 15}, {NULL}};
const Segment SGI205609[] = {{&S74553[34], 9}, {NULL}};
const Segment SGI205610[] = {{&S74553[27], 6}, {NULL}};
const Segment SGI205611[] = {{&S74553[96], 9}, {NULL}};
const Segment SGI205612[] = {{&S74553[69], 6}, {NULL}};
const Segment SGI205613[] = {{&S84610[34], 13}, {NULL}};
const Segment SGI205614[] = {{&S84610[27], 6}, {NULL}};
const Segment SGI205615[] = {{&S74554[34], 10}, {NULL}};
const Segment SGI205616[] = {{&S74554[27], 6}, {NULL}};
const Segment SGI205617[] = {{&S84610[100], 9}, {NULL}};
const Segment SGI205618[] = {{&S84610[73], 6}, {NULL}};
const Segment SGI205619[] = {{&S74554[97], 9}, {NULL}};
const Segment SGI205620[] = {{&S74554[70], 6}, {NULL}};
const Segment SGI205621[] = {{&S74554[131], 8}, {NULL}};
const Segment SGI205622[] = {{&S74555[34], 10}, {NULL}};
const Segment SGI205623[] = {{&S74555[27], 6}, {NULL}};
const Segment SGI205624[] = {{&S84612[59], 6}, {NULL}};
const Segment SGI205625[] = {{&S84612[59], 18}, {NULL}};
const Segment SGI205626[] = {{&S84612[45], 6}, {NULL}};
const Segment SGI205627[] = {{&S74555[97], 9}, {NULL}};
const Segment SGI205628[] = {{&S84612[46], 3}, {NULL}};
const Segment SGI205629[] = {{&S74555[70], 6}, {NULL}};
const Segment SGI205630[] = {{&S84612[91], 6}, {NULL}};
const Segment SGI205631[] = {{&S74555[131], 8}, {NULL}};
const Segment SGI205632[] = {{&S84612[0], 14}, {NULL}};
const Segment SGI205633[] = {{&S84616[44], 15}, {NULL}};
const Segment SGI205634[] = {{&S84616[44], 33}, {NULL}};
const Segment SGI205635[] = {{&S84616[30], 6}, {NULL}};
const Segment SGI205636[] = {{&S84616[31], 3}, {NULL}};
const Segment SGI205637[] = {{&S84616[91], 5}, {NULL}};
const Segment SGI205638[] = {{&S84616[128], 14}, {NULL}};
const Segment SGI205639[] = {{&S84617[34], 17}, {NULL}};
const Segment SGI205640[] = {{&S84617[27], 6}, {NULL}};
const Segment SGI205641[] = {{&S84617[111], 9}, {NULL}};
const Segment SGI205642[] = {{&S84617[84], 6}, {NULL}};
const Segment SGI205643[] = {{&S84628[34], 16}, {NULL}};
const Segment SGI205644[] = {{&S84628[27], 6}, {NULL}};
const Segment SGI205645[] = {{&S84628[116], 9}, {NULL}};
const Segment SGI205646[] = {{&S84628[89], 6}, {NULL}};
const Segment SGI205647[] = {{&S84629[34], 19}, {NULL}};
const Segment SGI205648[] = {{&S84629[27], 6}, {NULL}};
const Segment SGI205649[] = {{&S84629[119], 9}, {NULL}};
const Segment SGI205650[] = {{&S84629[92], 6}, {NULL}};
const Segment SGI205651[] = {{&S84630[34], 10}, {NULL}};
const Segment SGI205652[] = {{&S84630[27], 6}, {NULL}};
const Segment SGI205653[] = {{&S84630[97], 9}, {NULL}};
const Segment SGI205654[] = {{&S84630[70], 6}, {NULL}};
const Segment SGI205655[] = {{&S84633[34], 18}, {NULL}};
const Segment SGI205656[] = {{&S84633[27], 6}, {NULL}};
const Segment SGI205657[] = {{&S84633[105], 9}, {NULL}};
const Segment SGI205658[] = {{&S84633[78], 6}, {NULL}};
const Segment SGI205659[] = {{&S84635[34], 9}, {NULL}};
const Segment SGI205660[] = {{&S84635[27], 6}, {NULL}};
const Segment SGI205661[] = {{&S84635[93], 9}, {NULL}};
const Segment SGI205662[] = {{&S84635[66], 6}, {NULL}};
const Segment SGI205663[] = {{&S84641[34], 9}, {NULL}};
const Segment SGI205664[] = {{&S84641[27], 6}, {NULL}};
const Segment SGI205665[] = {{&S84641[96], 9}, {NULL}};
const Segment SGI205666[] = {{&S84641[69], 6}, {NULL}};
const Segment SGI205667[] = {{&S84642[34], 11}, {NULL}};
const Segment SGI205668[] = {{&S84642[27], 6}, {NULL}};
const Segment SGI205669[] = {{&S84642[98], 9}, {NULL}};
const Segment SGI205670[] = {{&S84642[71], 6}, {NULL}};
const Segment SGI205671[] = {{&S84648[34], 10}, {NULL}};
const Segment SGI205672[] = {{&S84648[27], 6}, {NULL}};
const Segment SGI205673[] = {{&S84648[97], 9}, {NULL}};
const Segment SGI205674[] = {{&S84648[70], 6}, {NULL}};
const Segment SGI205675[] = {{&S84649[34], 10}, {NULL}};
const Segment SGI205676[] = {{&S84649[27], 6}, {NULL}};
const Segment SGI205677[] = {{&S84649[97], 9}, {NULL}};
const Segment SGI205678[] = {{&S84649[70], 6}, {NULL}};
const Segment SGI205679[] = {{&S74614[44], 13}, {NULL}};
const Segment SGI205680[] = {{&S74614[44], 25}, {NULL}};
const Segment SGI205681[] = {{&S74614[30], 6}, {NULL}};
const Segment SGI205682[] = {{&S84656[44], 15}, {NULL}};
const Segment SGI205683[] = {{&S84656[44], 33}, {NULL}};
const Segment SGI205684[] = {{&S84656[30], 6}, {NULL}};
const Segment SGI205685[] = {{&S74614[118], 3}, {NULL}};
const Segment SGI205686[] = {{&S74614[83], 6}, {NULL}};
const Segment SGI205687[] = {{&S84656[91], 6}, {NULL}};
const Segment SGI205688[] = {{&S84656[118], 13}, {NULL}};
const Segment SGI205689[] = {{&S79223[34], 11}, {NULL}};
const Segment SGI205690[] = {{&S79223[27], 6}, {NULL}};
const Segment SGI205691[] = {{&S79223[98], 9}, {NULL}};
const Segment SGI205692[] = {{&S79223[71], 6}, {NULL}};
const Segment SGI205693[] = {{&S74638[34], 18}, {NULL}};
const Segment SGI205694[] = {{&S74638[27], 6}, {NULL}};
const Segment SGI205695[] = {{&S74638[105], 9}, {NULL}};
const Segment SGI205696[] = {{&S74638[78], 6}, {NULL}};
const Segment SGI205697[] = {{&S64687[44], 12}, {NULL}};
const Segment SGI205698[] = {{&S64687[44], 23}, {NULL}};
const Segment SGI205699[] = {{&S64687[30], 6}, {NULL}};
const Segment SGI205700[] = {{&S64687[134], 3}, {NULL}};
const Segment SGI205701[] = {{&S64687[99], 6}, {NULL}};
const Segment SGI205702[] = {{&S64687[69], 8}, {NULL}};
const Segment SGI205703[] = {{&S64688[44], 12}, {NULL}};
const Segment SGI205704[] = {{&S64688[44], 24}, {NULL}};
const Segment SGI205705[] = {{&S79229[44], 6}, {NULL}};
const Segment SGI205706[] = {{&S64688[30], 6}, {NULL}};
const Segment SGI205707[] = {{&S79229[44], 18}, {NULL}};
const Segment SGI205708[] = {{&S79229[30], 6}, {NULL}};
const Segment SGI205709[] = {{&S64688[117], 3}, {NULL}};
const Segment SGI205710[] = {{&S64688[82], 6}, {NULL}};
const Segment SGI205711[] = {{&S79229[76], 6}, {NULL}};
const Segment SGI205712[] = {{&S64688[142], 8}, {NULL}};
const Segment SGI205713[] = {{&S79229[103], 13}, {NULL}};
const Segment SGI205714[] = {{&S74648[34], 13}, {NULL}};
const Segment SGI205715[] = {{&S74648[27], 6}, {NULL}};
const Segment SGI205716[] = {{&S73722[44], 15}, {NULL}};
const Segment SGI205717[] = {{&S73722[44], 27}, {NULL}};
const Segment SGI205718[] = {{&S73722[30], 6}, {NULL}};
const Segment SGI205719[] = {{&S74648[100], 9}, {NULL}};
const Segment SGI205720[] = {{&S74648[73], 6}, {NULL}};
const Segment SGI205721[] = {{&S73722[120], 3}, {NULL}};
const Segment SGI205722[] = {{&S73722[85], 6}, {NULL}};
const Segment SGI205723[] = {{&S74649[34], 19}, {NULL}};
const Segment SGI205724[] = {{&S74649[27], 6}, {NULL}};
const Segment SGI205725[] = {{&S74649[106], 9}, {NULL}};
const Segment SGI205726[] = {{&S74649[79], 6}, {NULL}};
const Segment SGI205727[] = {{&S74650[34], 10}, {NULL}};
const Segment SGI205728[] = {{&S74650[27], 6}, {NULL}};
const Segment SGI205729[] = {{&S74650[97], 9}, {NULL}};
const Segment SGI205730[] = {{&S74650[70], 6}, {NULL}};
const Segment SGI205731[] = {{&S79242[44], 20}, {NULL}};
const Segment SGI205732[] = {{&S79242[44], 32}, {NULL}};
const Segment SGI205733[] = {{&S79242[30], 6}, {NULL}};
const Segment SGI205734[] = {{&S79242[125], 3}, {NULL}};
const Segment SGI205735[] = {{&S79242[90], 6}, {NULL}};
const Segment SGI205736[] = {{&S84700[34], 14}, {NULL}};
const Segment SGI205737[] = {{&S84700[27], 6}, {NULL}};
const Segment SGI205738[] = {{&S84700[101], 9}, {NULL}};
const Segment SGI205739[] = {{&S84700[74], 6}, {NULL}};
const Segment SGI205740[] = {{&S79244[34], 10}, {NULL}};
const Segment SGI205741[] = {{&S79244[27], 6}, {NULL}};
const Segment SGI205742[] = {{&S79244[97], 9}, {NULL}};
const Segment SGI205743[] = {{&S79244[70], 6}, {NULL}};
const Segment SGI205744[] = {{&S79248[59], 18}, {NULL}};
const Segment SGI205745[] = {{&S79248[59], 39}, {NULL}};
const Segment SGI205746[] = {{&S79248[45], 6}, {NULL}};
const Segment SGI205747[] = {{&S79248[147], 3}, {NULL}};
const Segment SGI205748[] = {{&S79248[112], 6}, {NULL}};
const Segment SGI205749[] = {{&S79248[0], 14}, {NULL}};
const Segment SGI205750[] = {{&S79251[34], 17}, {NULL}};
const Segment SGI205751[] = {{&S79251[27], 6}, {NULL}};
const Segment SGI205752[] = {{&S79251[115], 9}, {NULL}};
const Segment SGI205753[] = {{&S79251[88], 6}, {NULL}};
const Segment SGI205754[] = {{&S84712[34], 6}, {NULL}};
const Segment SGI205755[] = {{&S84712[27], 6}, {NULL}};
const Segment SGI205756[] = {{&S84712[93], 9}, {NULL}};
const Segment SGI205757[] = {{&S84712[66], 6}, {NULL}};
const Segment SGI205758[] = {{&S74670[34], 9}, {NULL}};
const Segment SGI205759[] = {{&S74670[27], 6}, {NULL}};
const Segment SGI205760[] = {{&S74670[96], 9}, {NULL}};
const Segment SGI205761[] = {{&S74670[69], 6}, {NULL}};
const Segment SGI205762[] = {{&S74671[34], 9}, {NULL}};
const Segment SGI205763[] = {{&S74671[27], 6}, {NULL}};
const Segment SGI205764[] = {{&S74671[96], 9}, {NULL}};
const Segment SGI205765[] = {{&S74671[69], 6}, {NULL}};
const Segment SGI205766[] = {{&S84715[44], 7}, {NULL}};
const Segment SGI205767[] = {{&S84715[44], 33}, {NULL}};
const Segment SGI205768[] = {{&S84715[30], 6}, {NULL}};
const Segment SGI205769[] = {{&S84715[94], 6}, {NULL}};
const Segment SGI205770[] = {{&S74672[34], 10}, {NULL}};
const Segment SGI205771[] = {{&S74672[27], 6}, {NULL}};
const Segment SGI205772[] = {{&S74672[97], 9}, {NULL}};
const Segment SGI205773[] = {{&S74672[70], 6}, {NULL}};
const Segment SGI205774[] = {{&S84716[44], 7}, {NULL}};
const Segment SGI205775[] = {{&S84716[44], 33}, {NULL}};
const Segment SGI205776[] = {{&S84716[30], 6}, {NULL}};
const Segment SGI205777[] = {{&S84716[31], 3}, {NULL}};
const Segment SGI205778[] = {{&S84716[91], 6}, {NULL}};
const Segment SGI205779[] = {{&S79259[34], 6}, {NULL}};
const Segment SGI205780[] = {{&S79259[27], 6}, {NULL}};
const Segment SGI205781[] = {{&S79259[93], 9}, {NULL}};
const Segment SGI205782[] = {{&S79259[66], 6}, {NULL}};
const Segment SGI205783[] = {{&S84717[44], 7}, {NULL}};
const Segment SGI205784[] = {{&S84717[44], 34}, {NULL}};
const Segment SGI205785[] = {{&S84717[30], 6}, {NULL}};
const Segment SGI205786[] = {{&S84717[31], 3}, {NULL}};
const Segment SGI205787[] = {{&S84717[92], 6}, {NULL}};
const Segment SGI205788[] = {{&S79262[34], 11}, {NULL}};
const Segment SGI205789[] = {{&S79262[27], 6}, {NULL}};
const Segment SGI205790[] = {{&S79262[98], 9}, {NULL}};
const Segment SGI205791[] = {{&S79262[71], 6}, {NULL}};
const Segment SGI205792[] = {{&S84722[34], 13}, {NULL}};
const Segment SGI205793[] = {{&S84722[27], 6}, {NULL}};
const Segment SGI205794[] = {{&S84722[100], 9}, {NULL}};
const Segment SGI205795[] = {{&S84722[73], 6}, {NULL}};
const Segment SGI205796[] = {{&S74682[34], 10}, {NULL}};
const Segment SGI205797[] = {{&S74682[27], 6}, {NULL}};
const Segment SGI205798[] = {{&S74682[97], 9}, {NULL}};
const Segment SGI205799[] = {{&S74682[70], 6}, {NULL}};
const Segment SGI205800[] = {{&S74683[34], 10}, {NULL}};
const Segment SGI205801[] = {{&S73765[44], 7}, {NULL}};
const Segment SGI205802[] = {{&S74683[27], 6}, {NULL}};
const Segment SGI205803[] = {{&S73765[44], 19}, {NULL}};
const Segment SGI205804[] = {{&S73765[30], 6}, {NULL}};
const Segment SGI205805[] = {{&S74683[97], 9}, {NULL}};
const Segment SGI205806[] = {{&S73765[112], 3}, {NULL}};
const Segment SGI205807[] = {{&S74683[70], 6}, {NULL}};
const Segment SGI205808[] = {{&S73765[77], 6}, {NULL}};
const Segment SGI205809[] = {{&S74684[34], 10}, {NULL}};
const Segment SGI205810[] = {{&S74684[27], 6}, {NULL}};
const Segment SGI205811[] = {{&S74684[97], 9}, {NULL}};
const Segment SGI205812[] = {{&S74684[70], 6}, {NULL}};
const Segment SGI205813[] = {{&S64750[34], 11}, {NULL}};
const Segment SGI205814[] = {{&S64750[27], 6}, {NULL}};
const Segment SGI205815[] = {{&S64750[97], 9}, {NULL}};
const Segment SGI205816[] = {{&S64750[70], 6}, {NULL}};
const Segment SGI205817[] = {{&S64751[34], 7}, {NULL}};
const Segment SGI205818[] = {{&S64751[27], 6}, {NULL}};
const Segment SGI205819[] = {{&S64751[93], 9}, {NULL}};
const Segment SGI205820[] = {{&S64751[66], 6}, {NULL}};
const Segment SGI205821[] = {{&S84732[44], 9}, {NULL}};
const Segment SGI205822[] = {{&S84732[44], 21}, {NULL}};
const Segment SGI205823[] = {{&S84732[30], 6}, {NULL}};
const Segment SGI205824[] = {{&S84732[31], 3}, {NULL}};
const Segment SGI205825[] = {{&S84732[79], 6}, {NULL}};
const Segment SGI205826[] = {{&S84734[34], 11}, {NULL}};
const Segment SGI205827[] = {{&S84734[27], 6}, {NULL}};
const Segment SGI205828[] = {{&S84734[98], 9}, {NULL}};
const Segment SGI205829[] = {{&S84734[71], 6}, {NULL}};
const Segment SGI205830[] = {{&S84735[34], 11}, {NULL}};
const Segment SGI205831[] = {{&S84735[27], 6}, {NULL}};
const Segment SGI205832[] = {{&S84735[98], 9}, {NULL}};
const Segment SGI205833[] = {{&S84735[71], 6}, {NULL}};
const Segment SGI205834[] = {{&S84737[34], 11}, {NULL}};
const Segment SGI205835[] = {{&S84737[27], 6}, {NULL}};
const Segment SGI205836[] = {{&S84737[98], 9}, {NULL}};
const Segment SGI205837[] = {{&S84737[71], 6}, {NULL}};
const Segment SGI205838[] = {{&S84738[45], 10}, {NULL}};
const Segment SGI205839[] = {{&S84738[30], 6}, {NULL}};
const Segment SGI205840[] = {{&S84738[31], 3}, {NULL}};
const Segment SGI205841[] = {{&S84738[81], 5}, {NULL}};
const Segment SGI205842[] = {{&S84738[139], 13}, {NULL}};
const Segment SGI205843[] = {{&S74701[44], 7}, {NULL}};
const Segment SGI205844[] = {{&S74701[44], 23}, {NULL}};
const Segment SGI205845[] = {{&S74701[30], 6}, {NULL}};
const Segment SGI205846[] = {{&S74701[116], 3}, {NULL}};
const Segment SGI205847[] = {{&S74701[81], 6}, {NULL}};
const Segment SGI205848[] = {{&S74702[44], 7}, {NULL}};
const Segment SGI205849[] = {{&S74702[44], 23}, {NULL}};
const Segment SGI205850[] = {{&S74702[30], 6}, {NULL}};
const Segment SGI205851[] = {{&S74702[116], 3}, {NULL}};
const Segment SGI205852[] = {{&S74702[81], 6}, {NULL}};
const Segment SGI205853[] = {{&S74703[44], 7}, {NULL}};
const Segment SGI205854[] = {{&S74703[44], 23}, {NULL}};
const Segment SGI205855[] = {{&S74703[30], 6}, {NULL}};
const Segment SGI205856[] = {{&S74703[116], 3}, {NULL}};
const Segment SGI205857[] = {{&S74703[81], 6}, {NULL}};
const Segment SGI205858[] = {{&S79294[34], 10}, {NULL}};
const Segment SGI205859[] = {{&S79294[27], 6}, {NULL}};
const Segment SGI205860[] = {{&S84744[34], 10}, {NULL}};
const Segment SGI205861[] = {{&S79294[97], 9}, {NULL}};
const Segment SGI205862[] = {{&S79294[70], 6}, {NULL}};
const Segment SGI205863[] = {{&S84744[27], 6}, {NULL}};
const Segment SGI205864[] = {{&S74704[44], 7}, {NULL}};
const Segment SGI205865[] = {{&S74704[44], 23}, {NULL}};
const Segment SGI205866[] = {{&S74704[30], 6}, {NULL}};
const Segment SGI205867[] = {{&S84744[97], 9}, {NULL}};
const Segment SGI205868[] = {{&S84744[70], 6}, {NULL}};
const Segment SGI205869[] = {{&S74704[116], 3}, {NULL}};
const Segment SGI205870[] = {{&S74704[81], 6}, {NULL}};
const Segment SGI205871[] = {{&S74705[38], 7}, {NULL}};
const Segment SGI205872[] = {{&S74705[31], 6}, {NULL}};
const Segment SGI205873[] = {{&S74706[44], 7}, {NULL}};
const Segment SGI205874[] = {{&S74706[44], 23}, {NULL}};
const Segment SGI205875[] = {{&S74706[30], 6}, {NULL}};
const Segment SGI205876[] = {{&S84747[34], 7}, {NULL}};
const Segment SGI205877[] = {{&S84747[27], 6}, {NULL}};
const Segment SGI205878[] = {{&S74706[116], 3}, {NULL}};
const Segment SGI205879[] = {{&S74706[81], 6}, {NULL}};
const Segment SGI205880[] = {{&S84747[94], 9}, {NULL}};
const Segment SGI205881[] = {{&S84747[67], 6}, {NULL}};
const Segment SGI205882[] = {{&S74707[44], 7}, {NULL}};
const Segment SGI205883[] = {{&S74707[44], 23}, {NULL}};
const Segment SGI205884[] = {{&S74707[30], 6}, {NULL}};
const Segment SGI205885[] = {{&S84748[34], 8}, {NULL}};
const Segment SGI205886[] = {{&S74707[116], 3}, {NULL}};
const Segment SGI205887[] = {{&S84748[27], 6}, {NULL}};
const Segment SGI205888[] = {{&S74707[81], 6}, {NULL}};
const Segment SGI205889[] = {{&S84748[95], 9}, {NULL}};
const Segment SGI205890[] = {{&S84748[68], 6}, {NULL}};
const Segment SGI205891[] = {{&S84750[34], 10}, {NULL}};
const Segment SGI205892[] = {{&S84750[27], 6}, {NULL}};
const Segment SGI205893[] = {{&S84750[97], 9}, {NULL}};
const Segment SGI205894[] = {{&S84750[70], 6}, {NULL}};
const Segment SGI205895[] = {{&S84751[44], 13}, {NULL}};
const Segment SGI205896[] = {{&S84751[44], 25}, {NULL}};
const Segment SGI205897[] = {{&S84751[30], 6}, {NULL}};
const Segment SGI205898[] = {{&S84751[83], 6}, {NULL}};
const Segment SGI205899[] = {{&S84751[110], 13}, {NULL}};
const Segment SGI205900[] = {{&S84755[34], 11}, {NULL}};
const Segment SGI205901[] = {{&S84755[27], 6}, {NULL}};
const Segment SGI205902[] = {{&S84755[98], 9}, {NULL}};
const Segment SGI205903[] = {{&S84755[71], 6}, {NULL}};
const Segment SGI205904[] = {{&S79313[44], 9}, {NULL}};
const Segment SGI205905[] = {{&S79313[44], 21}, {NULL}};
const Segment SGI205906[] = {{&S79313[30], 6}, {NULL}};
const Segment SGI205907[] = {{&S84756[34], 11}, {NULL}};
const Segment SGI205908[] = {{&S84756[27], 6}, {NULL}};
const Segment SGI205909[] = {{&S79313[130], 3}, {NULL}};
const Segment SGI205910[] = {{&S79313[95], 6}, {NULL}};
const Segment SGI205911[] = {{&S79313[67], 8}, {NULL}};
const Segment SGI205912[] = {{&S84756[98], 9}, {NULL}};
const Segment SGI205913[] = {{&S84756[71], 6}, {NULL}};
const Segment SGI205914[] = {{&S84757[34], 11}, {NULL}};
const Segment SGI205915[] = {{&S84757[27], 6}, {NULL}};
const Segment SGI205916[] = {{&S84757[98], 9}, {NULL}};
const Segment SGI205917[] = {{&S84757[71], 6}, {NULL}};
const Segment SGI205918[] = {{&S79315[44], 9}, {NULL}};
const Segment SGI205919[] = {{&S79315[44], 30}, {NULL}};
const Segment SGI205920[] = {{&S79315[30], 6}, {NULL}};
const Segment SGI205921[] = {{&S79315[123], 3}, {NULL}};
const Segment SGI205922[] = {{&S79315[88], 6}, {NULL}};
const Segment SGI205923[] = {{&S84759[59], 11}, {NULL}};
const Segment SGI205924[] = {{&S84759[59], 23}, {NULL}};
const Segment SGI205925[] = {{&S84759[45], 6}, {NULL}};
const Segment SGI205926[] = {{&S84759[46], 3}, {NULL}};
const Segment SGI205927[] = {{&S84759[96], 6}, {NULL}};
const Segment SGI205928[] = {{&S84759[0], 14}, {NULL}};
const Segment SGI205929[] = {{&S84762[34], 10}, {NULL}};
const Segment SGI205930[] = {{&S84762[27], 6}, {NULL}};
const Segment SGI205931[] = {{&S84762[97], 9}, {NULL}};
const Segment SGI205932[] = {{&S84762[70], 6}, {NULL}};
const Segment SGI205933[] = {{&S74739[34], 10}, {NULL}};
const Segment SGI205934[] = {{&S74739[27], 6}, {NULL}};
const Segment SGI205935[] = {{&S74739[97], 9}, {NULL}};
const Segment SGI205936[] = {{&S74739[70], 6}, {NULL}};
const Segment SGI205937[] = {{&S74742[34], 10}, {NULL}};
const Segment SGI205938[] = {{&S74742[27], 6}, {NULL}};
const Segment SGI205939[] = {{&S74742[97], 9}, {NULL}};
const Segment SGI205940[] = {{&S74742[70], 6}, {NULL}};
const Segment SGI205941[] = {{&S79334[44], 11}, {NULL}};
const Segment SGI205942[] = {{&S79334[44], 23}, {NULL}};
const Segment SGI205943[] = {{&S79334[30], 6}, {NULL}};
const Segment SGI205944[] = {{&S79334[116], 3}, {NULL}};
const Segment SGI205945[] = {{&S79334[81], 6}, {NULL}};
const Segment SGI205946[] = {{&S84779[44], 10}, {NULL}};
const Segment SGI205947[] = {{&S84779[44], 22}, {NULL}};
const Segment SGI205948[] = {{&S84779[30], 6}, {NULL}};
const Segment SGI205949[] = {{&S84779[31], 3}, {NULL}};
const Segment SGI205950[] = {{&S84779[80], 6}, {NULL}};
const Segment SGI205951[] = {{&S74747[44], 10}, {NULL}};
const Segment SGI205952[] = {{&S74747[44], 22}, {NULL}};
const Segment SGI205953[] = {{&S74747[30], 6}, {NULL}};
const Segment SGI205954[] = {{&S74747[113], 3}, {NULL}};
const Segment SGI205955[] = {{&S74747[80], 5}, {NULL}};
const Segment SGI205956[] = {{&S74748[44], 10}, {NULL}};
const Segment SGI205957[] = {{&S74748[44], 22}, {NULL}};
const Segment SGI205958[] = {{&S74748[30], 6}, {NULL}};
const Segment SGI205959[] = {{&S84783[44], 25}, {NULL}};
const Segment SGI205960[] = {{&S74748[113], 3}, {NULL}};
const Segment SGI205961[] = {{&S84783[44], 36}, {NULL}};
const Segment SGI205962[] = {{&S84783[30], 6}, {NULL}};
const Segment SGI205963[] = {{&S74748[80], 5}, {NULL}};
const Segment SGI205964[] = {{&S84783[31], 3}, {NULL}};
const Segment SGI205965[] = {{&S84783[94], 6}, {NULL}};
const Segment SGI205966[] = {{&S74749[44], 10}, {NULL}};
const Segment SGI205967[] = {{&S74749[44], 22}, {NULL}};
const Segment SGI205968[] = {{&S74749[30], 6}, {NULL}};
const Segment SGI205969[] = {{&S74749[113], 3}, {NULL}};
const Segment SGI205970[] = {{&S74749[80], 5}, {NULL}};
const Segment SGI205971[] = {{&S79343[44], 8}, {NULL}};
const Segment SGI205972[] = {{&S79343[44], 20}, {NULL}};
const Segment SGI205973[] = {{&S79343[30], 6}, {NULL}};
const Segment SGI205974[] = {{&S79343[113], 3}, {NULL}};
const Segment SGI205975[] = {{&S79343[78], 6}, {NULL}};
const Segment SGI205976[] = {{&S84786[34], 8}, {NULL}};
const Segment SGI205977[] = {{&S84786[27], 6}, {NULL}};
const Segment SGI205978[] = {{&S84786[95], 9}, {NULL}};
const Segment SGI205979[] = {{&S84786[68], 6}, {NULL}};
const Segment SGI205980[] = {{&S79345[44], 18}, {NULL}};
const Segment SGI205981[] = {{&S79345[44], 30}, {NULL}};
const Segment SGI205982[] = {{&S79345[30], 6}, {NULL}};
const Segment SGI205983[] = {{&S79345[88], 6}, {NULL}};
const Segment SGI205984[] = {{&S79345[115], 13}, {NULL}};
const Segment SGI205985[] = {{&S74753[44], 10}, {NULL}};
const Segment SGI205986[] = {{&S74753[44], 22}, {NULL}};
const Segment SGI205987[] = {{&S74753[30], 6}, {NULL}};
const Segment SGI205988[] = {{&S74753[80], 6}, {NULL}};
const Segment SGI205989[] = {{&S74753[107], 13}, {NULL}};
const Segment SGI205990[] = {{&S84792[45], 7}, {NULL}};
const Segment SGI205991[] = {{&S84792[30], 6}, {NULL}};
const Segment SGI205992[] = {{&S84792[31], 3}, {NULL}};
const Segment SGI205993[] = {{&S84792[83], 5}, {NULL}};
const Segment SGI205994[] = {{&S84792[141], 13}, {NULL}};
const Segment SGI205995[] = {{&S79350[34], 25}, {NULL}};
const Segment SGI205996[] = {{&S79350[27], 6}, {NULL}};
const Segment SGI205997[] = {{&S79350[112], 9}, {NULL}};
const Segment SGI205998[] = {{&S79350[85], 6}, {NULL}};
const Segment SGI205999[] = {{&S84794[34], 9}, {NULL}};
const Segment SGI206000[] = {{&S84794[27], 6}, {NULL}};
const Segment SGI206001[] = {{&S84794[96], 9}, {NULL}};
const Segment SGI206002[] = {{&S84794[69], 6}, {NULL}};
const Segment SGI206003[] = {{&S84795[44], 6}, {NULL}};
const Segment SGI206004[] = {{&S84795[44], 18}, {NULL}};
const Segment SGI206005[] = {{&S84795[30], 6}, {NULL}};
const Segment SGI206006[] = {{&S84795[31], 3}, {NULL}};
const Segment SGI206007[] = {{&S84795[76], 6}, {NULL}};
const Segment SGI206008[] = {{&S79354[44], 15}, {NULL}};
const Segment SGI206009[] = {{&S79354[44], 27}, {NULL}};
const Segment SGI206010[] = {{&S79354[30], 6}, {NULL}};
const Segment SGI206011[] = {{&S79354[85], 6}, {NULL}};
const Segment SGI206012[] = {{&S79354[112], 13}, {NULL}};
const Segment SGI206013[] = {{&S84798[44], 6}, {NULL}};
const Segment SGI206014[] = {{&S84798[44], 18}, {NULL}};
const Segment SGI206015[] = {{&S84798[30], 6}, {NULL}};
const Segment SGI206016[] = {{&S84798[31], 3}, {NULL}};
const Segment SGI206017[] = {{&S84798[76], 6}, {NULL}};
const Segment SGI206018[] = {{&S84808[34], 10}, {NULL}};
const Segment SGI206019[] = {{&S84808[27], 6}, {NULL}};
const Segment SGI206020[] = {{&S84808[97], 9}, {NULL}};
const Segment SGI206021[] = {{&S84808[70], 6}, {NULL}};
const Segment SGI206022[] = {{&S84809[34], 10}, {NULL}};
const Segment SGI206023[] = {{&S79368[44], 18}, {NULL}};
const Segment SGI206024[] = {{&S84809[27], 6}, {NULL}};
const Segment SGI206025[] = {{&S79368[44], 30}, {NULL}};
const Segment SGI206026[] = {{&S79368[30], 6}, {NULL}};
const Segment SGI206027[] = {{&S84809[97], 9}, {NULL}};
const Segment SGI206028[] = {{&S84809[70], 6}, {NULL}};
const Segment SGI206029[] = {{&S79368[152], 3}, {NULL}};
const Segment SGI206030[] = {{&S79368[117], 6}, {NULL}};
const Segment SGI206031[] = {{&S79368[76], 8}, {NULL}};
const Segment SGI206032[] = {{&S79368[88], 15}, {NULL}};
const Segment SGI206033[] = {{&S84810[34], 10}, {NULL}};
const Segment SGI206034[] = {{&S84810[27], 6}, {NULL}};
const Segment SGI206035[] = {{&S79369[44], 18}, {NULL}};
const Segment SGI206036[] = {{&S79369[44], 30}, {NULL}};
const Segment SGI206037[] = {{&S79369[30], 6}, {NULL}};
const Segment SGI206038[] = {{&S84810[97], 9}, {NULL}};
const Segment SGI206039[] = {{&S84810[70], 6}, {NULL}};
const Segment SGI206040[] = {{&S79369[152], 3}, {NULL}};
const Segment SGI206041[] = {{&S74781[44], 17}, {NULL}};
const Segment SGI206042[] = {{&S79369[117], 6}, {NULL}};
const Segment SGI206043[] = {{&S74781[44], 29}, {NULL}};
const Segment SGI206044[] = {{&S74781[30], 6}, {NULL}};
const Segment SGI206045[] = {{&S79369[76], 8}, {NULL}};
const Segment SGI206046[] = {{&S79369[88], 15}, {NULL}};
const Segment SGI206047[] = {{&S74781[122], 3}, {NULL}};
const Segment SGI206048[] = {{&S74781[87], 6}, {NULL}};
const Segment SGI206049[] = {{&S79370[44], 18}, {NULL}};
const Segment SGI206050[] = {{&S79370[44], 30}, {NULL}};
const Segment SGI206051[] = {{&S79370[30], 6}, {NULL}};
const Segment SGI206052[] = {{&S79370[152], 3}, {NULL}};
const Segment SGI206053[] = {{&S79370[117], 6}, {NULL}};
const Segment SGI206054[] = {{&S79370[76], 8}, {NULL}};
const Segment SGI206055[] = {{&S79370[88], 15}, {NULL}};
const Segment SGI206056[] = {{&S74789[44], 10}, {NULL}};
const Segment SGI206057[] = {{&S74789[44], 22}, {NULL}};
const Segment SGI206058[] = {{&S74789[30], 6}, {NULL}};
const Segment SGI206059[] = {{&S74789[80], 6}, {NULL}};
const Segment SGI206060[] = {{&S74789[107], 13}, {NULL}};
const Segment SGI206061[] = {{&S84824[34], 11}, {NULL}};
const Segment SGI206062[] = {{&S84824[27], 6}, {NULL}};
const Segment SGI206063[] = {{&S84824[108], 9}, {NULL}};
const Segment SGI206064[] = {{&S84824[81], 6}, {NULL}};
const Segment SGI206065[] = {{&S84829[44], 10}, {NULL}};
const Segment SGI206066[] = {{&S84829[44], 22}, {NULL}};
const Segment SGI206067[] = {{&S84829[30], 6}, {NULL}};
const Segment SGI206068[] = {{&S84829[31], 3}, {NULL}};
const Segment SGI206069[] = {{&S84829[106], 6}, {NULL}};
const Segment SGI206070[] = {{&S84829[68], 8}, {NULL}};
const Segment SGI206071[] = {{&S84830[34], 13}, {NULL}};
const Segment SGI206072[] = {{&S84830[27], 6}, {NULL}};
const Segment SGI206073[] = {{&S84830[100], 9}, {NULL}};
const Segment SGI206074[] = {{&S84830[73], 6}, {NULL}};
const Segment SGI206075[] = {{&S74802[44], 25}, {NULL}};
const Segment SGI206076[] = {{&S74802[44], 37}, {NULL}};
const Segment SGI206077[] = {{&S74802[30], 6}, {NULL}};
const Segment SGI206078[] = {{&S84831[34], 9}, {NULL}};
const Segment SGI206079[] = {{&S84831[27], 6}, {NULL}};
const Segment SGI206080[] = {{&S74802[146], 3}, {NULL}};
const Segment SGI206081[] = {{&S74802[111], 6}, {NULL}};
const Segment SGI206082[] = {{&S74802[83], 8}, {NULL}};
const Segment SGI206083[] = {{&S84831[96], 9}, {NULL}};
const Segment SGI206084[] = {{&S84831[69], 6}, {NULL}};
const Segment SGI206085[] = {{&S84833[34], 7}, {NULL}};
const Segment SGI206086[] = {{&S84833[27], 6}, {NULL}};
const Segment SGI206087[] = {{&S84833[99], 9}, {NULL}};
const Segment SGI206088[] = {{&S84833[72], 6}, {NULL}};
const Segment SGI206089[] = {{&S74808[34], 15}, {NULL}};
const Segment SGI206090[] = {{&S74808[27], 6}, {NULL}};
const Segment SGI206091[] = {{&S74808[102], 9}, {NULL}};
const Segment SGI206092[] = {{&S74808[75], 6}, {NULL}};
const Segment SGI206093[] = {{&S74809[34], 11}, {NULL}};
const Segment SGI206094[] = {{&S74809[27], 6}, {NULL}};
const Segment SGI206095[] = {{&S74809[98], 9}, {NULL}};
const Segment SGI206096[] = {{&S74809[71], 6}, {NULL}};
const Segment SGI206097[] = {{&S74811[44], 12}, {NULL}};
const Segment SGI206098[] = {{&S74811[44], 34}, {NULL}};
const Segment SGI206099[] = {{&S74811[30], 6}, {NULL}};
const Segment SGI206100[] = {{&S74811[127], 3}, {NULL}};
const Segment SGI206101[] = {{&S74811[92], 6}, {NULL}};
const Segment SGI206102[] = {{&S74811[152], 11}, {NULL}};
const Segment SGI206103[] = {{&S74819[44], 10}, {NULL}};
const Segment SGI206104[] = {{&S74819[44], 22}, {NULL}};
const Segment SGI206105[] = {{&S74819[30], 6}, {NULL}};
const Segment SGI206106[] = {{&S74819[80], 6}, {NULL}};
const Segment SGI206107[] = {{&S74819[107], 13}, {NULL}};
const Segment SGI206108[] = {{&S74820[44], 10}, {NULL}};
const Segment SGI206109[] = {{&S74820[44], 22}, {NULL}};
const Segment SGI206110[] = {{&S74820[30], 6}, {NULL}};
const Segment SGI206111[] = {{&S74820[80], 6}, {NULL}};
const Segment SGI206112[] = {{&S74820[107], 13}, {NULL}};
const Segment SGI206113[] = {{&S74821[44], 18}, {NULL}};
const Segment SGI206114[] = {{&S74821[44], 30}, {NULL}};
const Segment SGI206115[] = {{&S74821[30], 6}, {NULL}};
const Segment SGI206116[] = {{&S74821[88], 6}, {NULL}};
const Segment SGI206117[] = {{&S74821[115], 13}, {NULL}};
const Segment SGI206118[] = {{&S74822[34], 10}, {NULL}};
const Segment SGI206119[] = {{&S74822[27], 6}, {NULL}};
const Segment SGI206120[] = {{&S74822[97], 9}, {NULL}};
const Segment SGI206121[] = {{&S74822[70], 6}, {NULL}};
const Segment SGI206122[] = {{&S74823[34], 11}, {NULL}};
const Segment SGI206123[] = {{&S74823[27], 6}, {NULL}};
const Segment SGI206124[] = {{&S74823[98], 9}, {NULL}};
const Segment SGI206125[] = {{&S74823[71], 6}, {NULL}};
const Segment SGI206126[] = {{&S79423[34], 9}, {NULL}};
const Segment SGI206127[] = {{&S79423[27], 6}, {NULL}};
const Segment SGI206128[] = {{&S79423[96], 9}, {NULL}};
const Segment SGI206129[] = {{&S79423[69], 6}, {NULL}};
const Segment SGI206130[] = {{&S79443[44], 12}, {NULL}};
const Segment SGI206131[] = {{&S79443[44], 24}, {NULL}};
const Segment SGI206132[] = {{&S79443[30], 6}, {NULL}};
const Segment SGI206133[] = {{&S79443[117], 3}, {NULL}};
const Segment SGI206134[] = {{&S79443[82], 6}, {NULL}};
const Segment SGI206135[] = {{&S79445[34], 7}, {NULL}};
const Segment SGI206136[] = {{&S79445[27], 6}, {NULL}};
const Segment SGI206137[] = {{&S79445[94], 9}, {NULL}};
const Segment SGI206138[] = {{&S79445[67], 6}, {NULL}};
const Segment SGI206139[] = {{&S79446[120], 11}, {NULL}};
const Segment SGI206140[] = {{&S79446[120], 36}, {NULL}};
const Segment SGI206141[] = {{&S79446[30], 6}, {NULL}};
const Segment SGI206142[] = {{&S79446[103], 3}, {NULL}};
const Segment SGI206143[] = {{&S79446[68], 6}, {NULL}};
const Segment SGI206144[] = {{&S79447[120], 11}, {NULL}};
const Segment SGI206145[] = {{&S79447[120], 36}, {NULL}};
const Segment SGI206146[] = {{&S79447[30], 6}, {NULL}};
const Segment SGI206147[] = {{&S79447[103], 3}, {NULL}};
const Segment SGI206148[] = {{&S79447[68], 6}, {NULL}};
const Segment SGI206149[] = {{&S84834[44], 7}, {NULL}};
const Segment SGI206150[] = {{&S84834[44], 24}, {NULL}};
const Segment SGI206151[] = {{&S84834[30], 6}, {NULL}};
const Segment SGI206152[] = {{&S84834[82], 6}, {NULL}};
const Segment SGI206153[] = {{&S84834[109], 13}, {NULL}};
const Segment SGI206154[] = {{&S79451[44], 10}, {NULL}};
const Segment SGI206155[] = {{&S79451[44], 22}, {NULL}};
const Segment SGI206156[] = {{&S79451[30], 6}, {NULL}};
const Segment SGI206157[] = {{&S84837[34], 7}, {NULL}};
const Segment SGI206158[] = {{&S84837[27], 6}, {NULL}};
const Segment SGI206159[] = {{&S79451[115], 3}, {NULL}};
const Segment SGI206160[] = {{&S79451[80], 6}, {NULL}};
const Segment SGI206161[] = {{&S84837[94], 9}, {NULL}};
const Segment SGI206162[] = {{&S84837[67], 6}, {NULL}};
const Segment SGI206163[] = {{&S79452[44], 9}, {NULL}};
const Segment SGI206164[] = {{&S79452[44], 21}, {NULL}};
const Segment SGI206165[] = {{&S79452[30], 6}, {NULL}};
const Segment SGI206166[] = {{&S79452[114], 3}, {NULL}};
const Segment SGI206167[] = {{&S79452[79], 6}, {NULL}};
const Segment SGI206168[] = {{&S64867[34], 19}, {NULL}};
const Segment SGI206169[] = {{&S64867[27], 6}, {NULL}};
const Segment SGI206170[] = {{&S64867[105], 9}, {NULL}};
const Segment SGI206171[] = {{&S64867[78], 6}, {NULL}};
const Segment SGI206172[] = {{&S64868[34], 13}, {NULL}};
const Segment SGI206173[] = {{&S64868[27], 6}, {NULL}};
const Segment SGI206174[] = {{&S64868[99], 9}, {NULL}};
const Segment SGI206175[] = {{&S64868[72], 6}, {NULL}};
const Segment SGI206176[] = {{&S84855[44], 11}, {NULL}};
const Segment SGI206177[] = {{&S84855[44], 23}, {NULL}};
const Segment SGI206178[] = {{&S84855[30], 6}, {NULL}};
const Segment SGI206179[] = {{&S84855[31], 3}, {NULL}};
const Segment SGI206180[] = {{&S84855[81], 6}, {NULL}};
const Segment SGI206181[] = {{&S84856[44], 12}, {NULL}};
const Segment SGI206182[] = {{&S84856[44], 24}, {NULL}};
const Segment SGI206183[] = {{&S84856[30], 6}, {NULL}};
const Segment SGI206184[] = {{&S84856[31], 3}, {NULL}};
const Segment SGI206185[] = {{&S84856[82], 6}, {NULL}};
const Segment SGI206186[] = {{&S84857[44], 12}, {NULL}};
const Segment SGI206187[] = {{&S84857[44], 24}, {NULL}};
const Segment SGI206188[] = {{&S84857[30], 6}, {NULL}};
const Segment SGI206189[] = {{&S84857[31], 3}, {NULL}};
const Segment SGI206190[] = {{&S84857[82], 6}, {NULL}};
const Segment SGI206191[] = {{&S74892[34], 7}, {NULL}};
const Segment SGI206192[] = {{&S74892[27], 6}, {NULL}};
const Segment SGI206193[] = {{&S74892[99], 9}, {NULL}};
const Segment SGI206194[] = {{&S74892[72], 6}, {NULL}};
const Segment SGI206195[] = {{&S74892[140], 8}, {NULL}};
const Segment SGI206196[] = {{&S74014[44], 10}, {NULL}};
const Segment SGI206197[] = {{&S74014[44], 24}, {NULL}};
const Segment SGI206198[] = {{&S74014[30], 6}, {NULL}};
const Segment SGI206199[] = {{&S74014[117], 3}, {NULL}};
const Segment SGI206200[] = {{&S74014[82], 6}, {NULL}};
const Segment SGI206201[] = {{&S64890[44], 10}, {NULL}};
const Segment SGI206202[] = {{&S64890[44], 21}, {NULL}};
const Segment SGI206203[] = {{&S64890[30], 6}, {NULL}};
const Segment SGI206204[] = {{&S64890[132], 3}, {NULL}};
const Segment SGI206205[] = {{&S64890[97], 6}, {NULL}};
const Segment SGI206206[] = {{&S64890[67], 8}, {NULL}};
const Segment SGI206207[] = {{&S84874[34], 7}, {NULL}};
const Segment SGI206208[] = {{&S84874[27], 6}, {NULL}};
const Segment SGI206209[] = {{&S84874[94], 9}, {NULL}};
const Segment SGI206210[] = {{&S84874[67], 6}, {NULL}};
const Segment SGI206211[] = {{&S64892[44], 10}, {NULL}};
const Segment SGI206212[] = {{&S64892[44], 22}, {NULL}};
const Segment SGI206213[] = {{&S64892[30], 6}, {NULL}};
const Segment SGI206214[] = {{&S64892[135], 3}, {NULL}};
const Segment SGI206215[] = {{&S64892[100], 6}, {NULL}};
const Segment SGI206216[] = {{&S64892[68], 8}, {NULL}};
const Segment SGI206217[] = {{&S84875[34], 12}, {NULL}};
const Segment SGI206218[] = {{&S84875[27], 6}, {NULL}};
const Segment SGI206219[] = {{&S84875[99], 9}, {NULL}};
const Segment SGI206220[] = {{&S84875[72], 6}, {NULL}};
const Segment SGI206221[] = {{&S64893[34], 10}, {NULL}};
const Segment SGI206222[] = {{&S64893[27], 6}, {NULL}};
const Segment SGI206223[] = {{&S64893[96], 9}, {NULL}};
const Segment SGI206224[] = {{&S64893[69], 6}, {NULL}};
const Segment SGI206225[] = {{&S64894[34], 15}, {NULL}};
const Segment SGI206226[] = {{&S64894[27], 6}, {NULL}};
const Segment SGI206227[] = {{&S64894[101], 9}, {NULL}};
const Segment SGI206228[] = {{&S64894[74], 6}, {NULL}};
const Segment SGI206229[] = {{&S84880[45], 7}, {NULL}};
const Segment SGI206230[] = {{&S84880[30], 6}, {NULL}};
const Segment SGI206231[] = {{&S84880[31], 3}, {NULL}};
const Segment SGI206232[] = {{&S84880[84], 5}, {NULL}};
const Segment SGI206233[] = {{&S79504[44], 12}, {NULL}};
const Segment SGI206234[] = {{&S79504[44], 24}, {NULL}};
const Segment SGI206235[] = {{&S79504[30], 6}, {NULL}};
const Segment SGI206236[] = {{&S79504[117], 3}, {NULL}};
const Segment SGI206237[] = {{&S79504[82], 6}, {NULL}};
const Segment SGI206238[] = {{&S79511[44], 12}, {NULL}};
const Segment SGI206239[] = {{&S79511[44], 31}, {NULL}};
const Segment SGI206240[] = {{&S79511[30], 6}, {NULL}};
const Segment SGI206241[] = {{&S84880[142], 13}, {NULL}};
const Segment SGI206242[] = {{&S64913[34], 9}, {NULL}};
const Segment SGI206243[] = {{&S79511[124], 3}, {NULL}};
const Segment SGI206244[] = {{&S64913[27], 6}, {NULL}};
const Segment SGI206245[] = {{&S79511[89], 6}, {NULL}};
const Segment SGI206246[] = {{&S64913[95], 9}, {NULL}};
const Segment SGI206247[] = {{&S64913[68], 6}, {NULL}};
const Segment SGI206248[] = {{&S84881[44], 13}, {NULL}};
const Segment SGI206249[] = {{&S84881[44], 25}, {NULL}};
const Segment SGI206250[] = {{&S84881[30], 6}, {NULL}};
const Segment SGI206251[] = {{&S79512[44], 12}, {NULL}};
const Segment SGI206252[] = {{&S79512[44], 31}, {NULL}};
const Segment SGI206253[] = {{&S84881[31], 3}, {NULL}};
const Segment SGI206254[] = {{&S79512[30], 6}, {NULL}};
const Segment SGI206255[] = {{&S84881[83], 6}, {NULL}};
const Segment SGI206256[] = {{&S79512[124], 3}, {NULL}};
const Segment SGI206257[] = {{&S79512[89], 6}, {NULL}};
const Segment SGI206258[] = {{&S84887[44], 9}, {NULL}};
const Segment SGI206259[] = {{&S84887[44], 21}, {NULL}};
const Segment SGI206260[] = {{&S84887[30], 6}, {NULL}};
const Segment SGI206261[] = {{&S84887[31], 3}, {NULL}};
const Segment SGI206262[] = {{&S84887[94], 6}, {NULL}};
const Segment SGI206263[] = {{&S84887[67], 8}, {NULL}};
const Segment SGI206264[] = {{&S74950[34], 13}, {NULL}};
const Segment SGI206265[] = {{&S74950[27], 6}, {NULL}};
const Segment SGI206266[] = {{&S74950[100], 9}, {NULL}};
const Segment SGI206267[] = {{&S74950[73], 6}, {NULL}};
const Segment SGI206268[] = {{&S74952[44], 16}, {NULL}};
const Segment SGI206269[] = {{&S74952[44], 28}, {NULL}};
const Segment SGI206270[] = {{&S74952[30], 6}, {NULL}};
const Segment SGI206271[] = {{&S74952[121], 3}, {NULL}};
const Segment SGI206272[] = {{&S74952[86], 6}, {NULL}};
const Segment SGI206273[] = {{&S74953[38], 16}, {NULL}};
const Segment SGI206274[] = {{&S74953[31], 6}, {NULL}};
const Segment SGI206275[] = {{&S84897[44], 10}, {NULL}};
const Segment SGI206276[] = {{&S84897[44], 22}, {NULL}};
const Segment SGI206277[] = {{&S84897[30], 6}, {NULL}};
const Segment SGI206278[] = {{&S84897[31], 3}, {NULL}};
const Segment SGI206279[] = {{&S84897[80], 6}, {NULL}};
const Segment SGI206280[] = {{&S74959[34], 9}, {NULL}};
const Segment SGI206281[] = {{&S74959[27], 6}, {NULL}};
const Segment SGI206282[] = {{&S74959[96], 9}, {NULL}};
const Segment SGI206283[] = {{&S74959[69], 6}, {NULL}};
const Segment SGI206284[] = {{&S64937[44], 10}, {NULL}};
const Segment SGI206285[] = {{&S64937[44], 21}, {NULL}};
const Segment SGI206286[] = {{&S64937[30], 6}, {NULL}};
const Segment SGI206287[] = {{&S64937[143], 3}, {NULL}};
const Segment SGI206288[] = {{&S64937[108], 6}, {NULL}};
const Segment SGI206289[] = {{&S84901[44], 11}, {NULL}};
const Segment SGI206290[] = {{&S84901[44], 23}, {NULL}};
const Segment SGI206291[] = {{&S84901[30], 6}, {NULL}};
const Segment SGI206292[] = {{&S84901[31], 3}, {NULL}};
const Segment SGI206293[] = {{&S84901[81], 5}, {NULL}};
const Segment SGI206294[] = {{&S64938[44], 10}, {NULL}};
const Segment SGI206295[] = {{&S64938[44], 21}, {NULL}};
const Segment SGI206296[] = {{&S64938[30], 6}, {NULL}};
const Segment SGI206297[] = {{&S64938[143], 3}, {NULL}};
const Segment SGI206298[] = {{&S64938[108], 6}, {NULL}};
const Segment SGI206299[] = {{&S64939[44], 10}, {NULL}};
const Segment SGI206300[] = {{&S64939[44], 22}, {NULL}};
const Segment SGI206301[] = {{&S64939[30], 6}, {NULL}};
const Segment SGI206302[] = {{&S64939[115], 3}, {NULL}};
const Segment SGI206303[] = {{&S64939[80], 6}, {NULL}};
const Segment SGI206304[] = {{&S84904[34], 10}, {NULL}};
const Segment SGI206305[] = {{&S84904[27], 6}, {NULL}};
const Segment SGI206306[] = {{&S84904[97], 9}, {NULL}};
const Segment SGI206307[] = {{&S84904[70], 6}, {NULL}};
const Segment SGI206308[] = {{&S64940[44], 10}, {NULL}};
const Segment SGI206309[] = {{&S64940[44], 22}, {NULL}};
const Segment SGI206310[] = {{&S64940[30], 6}, {NULL}};
const Segment SGI206311[] = {{&S79543[44], 9}, {NULL}};
const Segment SGI206312[] = {{&S79543[44], 21}, {NULL}};
const Segment SGI206313[] = {{&S79543[30], 6}, {NULL}};
const Segment SGI206314[] = {{&S64940[115], 3}, {NULL}};
const Segment SGI206315[] = {{&S64940[80], 6}, {NULL}};
const Segment SGI206316[] = {{&S79543[114], 3}, {NULL}};
const Segment SGI206317[] = {{&S79543[79], 6}, {NULL}};
const Segment SGI206318[] = {{&S64941[44], 10}, {NULL}};
const Segment SGI206319[] = {{&S64941[44], 22}, {NULL}};
const Segment SGI206320[] = {{&S64941[30], 6}, {NULL}};
const Segment SGI206321[] = {{&S64941[115], 3}, {NULL}};
const Segment SGI206322[] = {{&S64941[80], 6}, {NULL}};
const Segment SGI206323[] = {{&S84908[34], 10}, {NULL}};
const Segment SGI206324[] = {{&S84908[27], 6}, {NULL}};
const Segment SGI206325[] = {{&S84908[97], 9}, {NULL}};
const Segment SGI206326[] = {{&S84908[70], 6}, {NULL}};
const Segment SGI206327[] = {{&S79548[44], 9}, {NULL}};
const Segment SGI206328[] = {{&S79548[44], 21}, {NULL}};
const Segment SGI206329[] = {{&S79548[30], 6}, {NULL}};
const Segment SGI206330[] = {{&S79548[114], 3}, {NULL}};
const Segment SGI206331[] = {{&S79548[79], 6}, {NULL}};
const Segment SGI206332[] = {{&S84910[34], 15}, {NULL}};
const Segment SGI206333[] = {{&S84910[27], 6}, {NULL}};
const Segment SGI206334[] = {{&S84910[117], 9}, {NULL}};
const Segment SGI206335[] = {{&S84910[90], 6}, {NULL}};
const Segment SGI206336[] = {{&S84916[44], 7}, {NULL}};
const Segment SGI206337[] = {{&S84916[44], 24}, {NULL}};
const Segment SGI206338[] = {{&S84916[30], 6}, {NULL}};
const Segment SGI206339[] = {{&S84916[82], 6}, {NULL}};
const Segment SGI206340[] = {{&S84916[109], 13}, {NULL}};
const Segment SGI206341[] = {{&S74091[34], 10}, {NULL}};
const Segment SGI206342[] = {{&S74091[27], 6}, {NULL}};
const Segment SGI206343[] = {{&S74091[97], 9}, {NULL}};
const Segment SGI206344[] = {{&S74091[70], 6}, {NULL}};
const Segment SGI206345[] = {{&S84918[34], 8}, {NULL}};
const Segment SGI206346[] = {{&S84918[27], 6}, {NULL}};
const Segment SGI206347[] = {{&S84918[94], 9}, {NULL}};
const Segment SGI206348[] = {{&S84918[67], 6}, {NULL}};
const Segment SGI206349[] = {{&S74997[44], 10}, {NULL}};
const Segment SGI206350[] = {{&S74997[44], 22}, {NULL}};
const Segment SGI206351[] = {{&S74997[30], 6}, {NULL}};
const Segment SGI206352[] = {{&S74997[80], 6}, {NULL}};
const Segment SGI206353[] = {{&S74997[107], 13}, {NULL}};
const Segment SGI206354[] = {{&S84928[44], 10}, {NULL}};
const Segment SGI206355[] = {{&S84928[44], 22}, {NULL}};
const Segment SGI206356[] = {{&S84928[30], 6}, {NULL}};
const Segment SGI206357[] = {{&S84928[80], 6}, {NULL}};
const Segment SGI206358[] = {{&S84928[107], 13}, {NULL}};
const Segment SGI206359[] = {{&S84930[34], 7}, {NULL}};
const Segment SGI206360[] = {{&S84930[27], 6}, {NULL}};
const Segment SGI206361[] = {{&S84930[99], 9}, {NULL}};
const Segment SGI206362[] = {{&S84930[72], 6}, {NULL}};
const Segment SGI206363[] = {{&S84930[140], 8}, {NULL}};
const Segment SGI206364[] = {{&S75004[34], 10}, {NULL}};
const Segment SGI206365[] = {{&S75004[27], 6}, {NULL}};
const Segment SGI206366[] = {{&S84931[34], 9}, {NULL}};
const Segment SGI206367[] = {{&S84931[27], 6}, {NULL}};
const Segment SGI206368[] = {{&S75004[97], 9}, {NULL}};
const Segment SGI206369[] = {{&S75004[70], 6}, {NULL}};
const Segment SGI206370[] = {{&S84931[96], 9}, {NULL}};
const Segment SGI206371[] = {{&S84931[69], 6}, {NULL}};
const Segment SGI206372[] = {{&S75004[131], 8}, {NULL}};
const Segment SGI206373[] = {{&S84931[130], 8}, {NULL}};
const Segment SGI206374[] = {{&S75007[34], 6}, {NULL}};
const Segment SGI206375[] = {{&S75007[27], 6}, {NULL}};
const Segment SGI206376[] = {{&S75007[93], 9}, {NULL}};
const Segment SGI206377[] = {{&S75007[66], 6}, {NULL}};
const Segment SGI206378[] = {{&S84936[44], 10}, {NULL}};
const Segment SGI206379[] = {{&S84936[44], 22}, {NULL}};
const Segment SGI206380[] = {{&S84936[30], 6}, {NULL}};
const Segment SGI206381[] = {{&S84936[31], 3}, {NULL}};
const Segment SGI206382[] = {{&S84936[80], 5}, {NULL}};
const Segment SGI206383[] = {{&S84936[117], 14}, {NULL}};
const Segment SGI206384[] = {{&S84937[44], 11}, {NULL}};
const Segment SGI206385[] = {{&S84937[44], 23}, {NULL}};
const Segment SGI206386[] = {{&S84937[30], 6}, {NULL}};
const Segment SGI206387[] = {{&S84937[31], 3}, {NULL}};
const Segment SGI206388[] = {{&S84937[81], 5}, {NULL}};
const Segment SGI206389[] = {{&S84937[118], 14}, {NULL}};
const Segment SGI206390[] = {{&S64989[34], 10}, {NULL}};
const Segment SGI206391[] = {{&S64989[27], 6}, {NULL}};
const Segment SGI206392[] = {{&S64989[96], 9}, {NULL}};
const Segment SGI206393[] = {{&S64989[69], 6}, {NULL}};
const Segment SGI206394[] = {{&S84939[34], 9}, {NULL}};
const Segment SGI206395[] = {{&S84939[27], 6}, {NULL}};
const Segment SGI206396[] = {{&S84939[103], 9}, {NULL}};
const Segment SGI206397[] = {{&S84939[76], 6}, {NULL}};
const Segment SGI206398[] = {{&S79598[44], 10}, {NULL}};
const Segment SGI206399[] = {{&S79598[44], 22}, {NULL}};
const Segment SGI206400[] = {{&S79598[30], 6}, {NULL}};
const Segment SGI206401[] = {{&S79598[115], 3}, {NULL}};
const Segment SGI206402[] = {{&S79598[80], 6}, {NULL}};
const Segment SGI206403[] = {{&S75031[34], 7}, {NULL}};
const Segment SGI206404[] = {{&S75031[27], 6}, {NULL}};
const Segment SGI206405[] = {{&S75031[98], 9}, {NULL}};
const Segment SGI206406[] = {{&S75031[71], 6}, {NULL}};
const Segment SGI206407[] = {{&S75032[34], 7}, {NULL}};
const Segment SGI206408[] = {{&S75032[27], 6}, {NULL}};
const Segment SGI206409[] = {{&S65008[43], 18}, {NULL}};
const Segment SGI206410[] = {{&S65008[30], 6}, {NULL}};
const Segment SGI206411[] = {{&S75032[98], 9}, {NULL}};
const Segment SGI206412[] = {{&S75032[71], 6}, {NULL}};
const Segment SGI206413[] = {{&S65008[31], 3}, {NULL}};
const Segment SGI206414[] = {{&S65008[75], 6}, {NULL}};
const Segment SGI206415[] = {{&S75034[44], 10}, {NULL}};
const Segment SGI206416[] = {{&S75034[44], 22}, {NULL}};
const Segment SGI206417[] = {{&S75034[30], 6}, {NULL}};
const Segment SGI206418[] = {{&S75034[80], 6}, {NULL}};
const Segment SGI206419[] = {{&S75034[107], 13}, {NULL}};
const Segment SGI206420[] = {{&S84979[44], 11}, {NULL}};
const Segment SGI206421[] = {{&S84979[44], 23}, {NULL}};
const Segment SGI206422[] = {{&S84979[30], 6}, {NULL}};
const Segment SGI206423[] = {{&S84979[31], 3}, {NULL}};
const Segment SGI206424[] = {{&S84979[81], 6}, {NULL}};
const Segment SGI206425[] = {{&S84983[34], 11}, {NULL}};
const Segment SGI206426[] = {{&S84983[27], 6}, {NULL}};
const Segment SGI206427[] = {{&S84983[98], 9}, {NULL}};
const Segment SGI206428[] = {{&S84983[71], 6}, {NULL}};
const Segment SGI206429[] = {{&S84984[34], 17}, {NULL}};
const Segment SGI206430[] = {{&S84984[27], 6}, {NULL}};
const Segment SGI206431[] = {{&S84984[102], 9}, {NULL}};
const Segment SGI206432[] = {{&S84984[75], 6}, {NULL}};
const Segment SGI206433[] = {{&S84985[44], 19}, {NULL}};
const Segment SGI206434[] = {{&S84985[44], 31}, {NULL}};
const Segment SGI206435[] = {{&S84985[30], 6}, {NULL}};
const Segment SGI206436[] = {{&S84985[31], 3}, {NULL}};
const Segment SGI206437[] = {{&S84985[89], 6}, {NULL}};
const Segment SGI206438[] = {{&S84988[45], 15}, {NULL}};
const Segment SGI206439[] = {{&S84988[30], 6}, {NULL}};
const Segment SGI206440[] = {{&S84988[31], 3}, {NULL}};
const Segment SGI206441[] = {{&S84988[86], 5}, {NULL}};
const Segment SGI206442[] = {{&S84988[144], 13}, {NULL}};
const Segment SGI206443[] = {{&S79651[34], 10}, {NULL}};
const Segment SGI206444[] = {{&S79651[27], 6}, {NULL}};
const Segment SGI206445[] = {{&S79651[97], 9}, {NULL}};
const Segment SGI206446[] = {{&S79651[70], 6}, {NULL}};
const Segment SGI206447[] = {{&S84994[44], 10}, {NULL}};
const Segment SGI206448[] = {{&S84994[44], 36}, {NULL}};
const Segment SGI206449[] = {{&S84994[30], 6}, {NULL}};
const Segment SGI206450[] = {{&S84994[31], 3}, {NULL}};
const Segment SGI206451[] = {{&S84994[109], 6}, {NULL}};
const Segment SGI206452[] = {{&S79656[44], 4}, {NULL}};
const Segment SGI206453[] = {{&S79656[44], 16}, {NULL}};
const Segment SGI206454[] = {{&S79656[30], 6}, {NULL}};
const Segment SGI206455[] = {{&S84994[82], 8}, {NULL}};
const Segment SGI206456[] = {{&S79656[109], 3}, {NULL}};
const Segment SGI206457[] = {{&S79656[74], 6}, {NULL}};
const Segment SGI206458[] = {{&S84995[44], 10}, {NULL}};
const Segment SGI206459[] = {{&S84995[44], 22}, {NULL}};
const Segment SGI206460[] = {{&S84995[30], 6}, {NULL}};
const Segment SGI206461[] = {{&S84995[31], 3}, {NULL}};
const Segment SGI206462[] = {{&S84995[95], 6}, {NULL}};
const Segment SGI206463[] = {{&S84995[68], 8}, {NULL}};
const Segment SGI206464[] = {{&S84996[44], 10}, {NULL}};
const Segment SGI206465[] = {{&S84996[44], 22}, {NULL}};
const Segment SGI206466[] = {{&S84996[30], 6}, {NULL}};
const Segment SGI206467[] = {{&S84996[31], 3}, {NULL}};
const Segment SGI206468[] = {{&S84996[106], 6}, {NULL}};
const Segment SGI206469[] = {{&S84996[68], 8}, {NULL}};
const Segment SGI206470[] = {{&S84997[44], 7}, {NULL}};
const Segment SGI206471[] = {{&S84997[44], 24}, {NULL}};
const Segment SGI206472[] = {{&S84997[30], 6}, {NULL}};
const Segment SGI206473[] = {{&S79661[44], 8}, {NULL}};
const Segment SGI206474[] = {{&S79661[44], 20}, {NULL}};
const Segment SGI206475[] = {{&S79661[30], 6}, {NULL}};
const Segment SGI206476[] = {{&S84997[82], 6}, {NULL}};
const Segment SGI206477[] = {{&S84997[109], 13}, {NULL}};
const Segment SGI206478[] = {{&S79661[113], 3}, {NULL}};
const Segment SGI206479[] = {{&S79661[78], 6}, {NULL}};
const Segment SGI206480[] = {{&S79662[34], 25}, {NULL}};
const Segment SGI206481[] = {{&S79662[27], 6}, {NULL}};
const Segment SGI206482[] = {{&S79662[112], 9}, {NULL}};
const Segment SGI206483[] = {{&S79662[85], 6}, {NULL}};
const Segment SGI206484[] = {{&S85000[34], 10}, {NULL}};
const Segment SGI206485[] = {{&S85000[27], 6}, {NULL}};
const Segment SGI206486[] = {{&S85000[97], 9}, {NULL}};
const Segment SGI206487[] = {{&S85000[70], 6}, {NULL}};
const Segment SGI206488[] = {{&S85001[34], 14}, {NULL}};
const Segment SGI206489[] = {{&S85001[27], 6}, {NULL}};
const Segment SGI206490[] = {{&S85001[101], 9}, {NULL}};
const Segment SGI206491[] = {{&S85001[74], 6}, {NULL}};
const Segment SGI206492[] = {{&S85007[34], 9}, {NULL}};
const Segment SGI206493[] = {{&S85007[27], 6}, {NULL}};
const Segment SGI206494[] = {{&S85007[96], 9}, {NULL}};
const Segment SGI206495[] = {{&S85007[69], 6}, {NULL}};
const Segment SGI206496[] = {{&S79673[34], 7}, {NULL}};
const Segment SGI206497[] = {{&S79673[27], 6}, {NULL}};
const Segment SGI206498[] = {{&S65076[34], 10}, {NULL}};
const Segment SGI206499[] = {{&S65076[27], 6}, {NULL}};
const Segment SGI206500[] = {{&S79673[94], 9}, {NULL}};
const Segment SGI206501[] = {{&S79673[67], 6}, {NULL}};
const Segment SGI206502[] = {{&S65076[96], 9}, {NULL}};
const Segment SGI206503[] = {{&S65076[69], 6}, {NULL}};
const Segment SGI206504[] = {{&S85009[34], 9}, {NULL}};
const Segment SGI206505[] = {{&S85009[27], 6}, {NULL}};
const Segment SGI206506[] = {{&S85009[96], 9}, {NULL}};
const Segment SGI206507[] = {{&S85009[69], 6}, {NULL}};
const Segment SGI206508[] = {{&S85014[34], 11}, {NULL}};
const Segment SGI206509[] = {{&S85014[27], 6}, {NULL}};
const Segment SGI206510[] = {{&S85014[98], 9}, {NULL}};
const Segment SGI206511[] = {{&S85014[71], 6}, {NULL}};
const Segment SGI206512[] = {{&S74213[34], 6}, {NULL}};
const Segment SGI206513[] = {{&S74213[27], 6}, {NULL}};
const Segment SGI206514[] = {{&S74213[93], 9}, {NULL}};
const Segment SGI206515[] = {{&S74213[66], 6}, {NULL}};
const Segment SGI206516[] = {{&S74214[34], 6}, {NULL}};
const Segment SGI206517[] = {{&S74214[27], 6}, {NULL}};
const Segment SGI206518[] = {{&S74214[93], 9}, {NULL}};
const Segment SGI206519[] = {{&S74214[66], 6}, {NULL}};
const Segment SGI206520[] = {{&S85029[34], 7}, {NULL}};
const Segment SGI206521[] = {{&S85029[27], 6}, {NULL}};
const Segment SGI206522[] = {{&S85029[98], 9}, {NULL}};
const Segment SGI206523[] = {{&S85029[71], 6}, {NULL}};
const Segment SGI206524[] = {{&S85036[44], 10}, {NULL}};
const Segment SGI206525[] = {{&S85036[44], 22}, {NULL}};
const Segment SGI206526[] = {{&S85036[30], 6}, {NULL}};
const Segment SGI206527[] = {{&S85036[80], 6}, {NULL}};
const Segment SGI206528[] = {{&S85036[107], 13}, {NULL}};
const Segment SGI206529[] = {{&S85038[34], 11}, {NULL}};
const Segment SGI206530[] = {{&S85038[27], 6}, {NULL}};
const Segment SGI206531[] = {{&S85038[98], 9}, {NULL}};
const Segment SGI206532[] = {{&S85038[71], 6}, {NULL}};
const Segment SGI206533[] = {{&S85039[34], 10}, {NULL}};
const Segment SGI206534[] = {{&S85039[27], 6}, {NULL}};
const Segment SGI206535[] = {{&S85039[97], 9}, {NULL}};
const Segment SGI206536[] = {{&S85039[70], 6}, {NULL}};
const Segment SGI206537[] = {{&S79710[34], 10}, {NULL}};
const Segment SGI206538[] = {{&S79710[27], 6}, {NULL}};
const Segment SGI206539[] = {{&S79710[104], 9}, {NULL}};
const Segment SGI206540[] = {{&S79710[77], 6}, {NULL}};
const Segment SGI206541[] = {{&S65124[41], 18}, {NULL}};
const Segment SGI206542[] = {{&S65124[41], 29}, {NULL}};
const Segment SGI206543[] = {{&S65124[27], 6}, {NULL}};
const Segment SGI206544[] = {{&S79715[44], 10}, {NULL}};
const Segment SGI206545[] = {{&S79715[44], 22}, {NULL}};
const Segment SGI206546[] = {{&S65124[119], 3}, {NULL}};
const Segment SGI206547[] = {{&S79715[30], 6}, {NULL}};
const Segment SGI206548[] = {{&S65124[123], 9}, {NULL}};
const Segment SGI206549[] = {{&S65124[84], 6}, {NULL}};
const Segment SGI206550[] = {{&S79715[80], 6}, {NULL}};
const Segment SGI206551[] = {{&S79715[107], 13}, {NULL}};
const Segment SGI206552[] = {{&S65125[41], 19}, {NULL}};
const Segment SGI206553[] = {{&S65125[41], 30}, {NULL}};
const Segment SGI206554[] = {{&S65125[27], 6}, {NULL}};
const Segment SGI206555[] = {{&S65125[120], 3}, {NULL}};
const Segment SGI206556[] = {{&S65125[124], 9}, {NULL}};
const Segment SGI206557[] = {{&S65125[85], 6}, {NULL}};
const Segment SGI206558[] = {{&S75147[34], 12}, {NULL}};
const Segment SGI206559[] = {{&S75147[27], 6}, {NULL}};
const Segment SGI206560[] = {{&S75147[99], 9}, {NULL}};
const Segment SGI206561[] = {{&S75147[72], 6}, {NULL}};
const Segment SGI206562[] = {{&S75153[44], 10}, {NULL}};
const Segment SGI206563[] = {{&S75153[44], 22}, {NULL}};
const Segment SGI206564[] = {{&S75153[30], 6}, {NULL}};
const Segment SGI206565[] = {{&S75153[115], 3}, {NULL}};
const Segment SGI206566[] = {{&S75153[80], 6}, {NULL}};
const Segment SGI206567[] = {{&S75153[140], 13}, {NULL}};
const Segment SGI206568[] = {{&S75154[44], 84}, {NULL}};
const Segment SGI206569[] = {{&S75154[44], 96}, {NULL}};
const Segment SGI206570[] = {{&S75154[30], 6}, {NULL}};
const Segment SGI206571[] = {{&S75154[189], 3}, {NULL}};
const Segment SGI206572[] = {{&S75154[121], 6}, {NULL}};
const Segment SGI206573[] = {{&S85065[34], 19}, {NULL}};
const Segment SGI206574[] = {{&S85065[27], 6}, {NULL}};
const Segment SGI206575[] = {{&S85065[106], 9}, {NULL}};
const Segment SGI206576[] = {{&S75155[44], 10}, {NULL}};
const Segment SGI206577[] = {{&S85065[79], 6}, {NULL}};
const Segment SGI206578[] = {{&S75155[44], 22}, {NULL}};
const Segment SGI206579[] = {{&S75155[30], 6}, {NULL}};
const Segment SGI206580[] = {{&S75155[115], 3}, {NULL}};
const Segment SGI206581[] = {{&S75155[80], 6}, {NULL}};
const Segment SGI206582[] = {{&S75156[44], 10}, {NULL}};
const Segment SGI206583[] = {{&S75156[44], 22}, {NULL}};
const Segment SGI206584[] = {{&S75156[30], 6}, {NULL}};
const Segment SGI206585[] = {{&S75156[80], 6}, {NULL}};
const Segment SGI206586[] = {{&S75156[107], 13}, {NULL}};
const Segment SGI206587[] = {{&S85068[59], 21}, {NULL}};
const Segment SGI206588[] = {{&S85068[59], 33}, {NULL}};
const Segment SGI206589[] = {{&S85068[45], 6}, {NULL}};
const Segment SGI206590[] = {{&S85068[46], 3}, {NULL}};
const Segment SGI206591[] = {{&S85068[106], 6}, {NULL}};
const Segment SGI206592[] = {{&S85068[0], 14}, {NULL}};
const Segment SGI206593[] = {{&S85069[59], 25}, {NULL}};
const Segment SGI206594[] = {{&S85069[59], 37}, {NULL}};
const Segment SGI206595[] = {{&S85069[45], 6}, {NULL}};
const Segment SGI206596[] = {{&S85069[46], 3}, {NULL}};
const Segment SGI206597[] = {{&S85069[110], 6}, {NULL}};
const Segment SGI206598[] = {{&S85069[0], 14}, {NULL}};
const Segment SGI206599[] = {{&S75160[34], 10}, {NULL}};
const Segment SGI206600[] = {{&S75160[27], 6}, {NULL}};
const Segment SGI206601[] = {{&S85070[44], 14}, {NULL}};
const Segment SGI206602[] = {{&S85070[44], 26}, {NULL}};
const Segment SGI206603[] = {{&S75160[97], 9}, {NULL}};
const Segment SGI206604[] = {{&S85070[30], 6}, {NULL}};
const Segment SGI206605[] = {{&S75160[70], 6}, {NULL}};
const Segment SGI206606[] = {{&S85070[31], 3}, {NULL}};
const Segment SGI206607[] = {{&S85070[84], 6}, {NULL}};
const Segment SGI206608[] = {{&S79743[44], 10}, {NULL}};
const Segment SGI206609[] = {{&S79743[44], 22}, {NULL}};
const Segment SGI206610[] = {{&S79743[30], 6}, {NULL}};
const Segment SGI206611[] = {{&S79743[113], 3}, {NULL}};
const Segment SGI206612[] = {{&S85071[34], 14}, {NULL}};
const Segment SGI206613[] = {{&S79743[80], 5}, {NULL}};
const Segment SGI206614[] = {{&S85071[27], 6}, {NULL}};
const Segment SGI206615[] = {{&S79743[117], 14}, {NULL}};
const Segment SGI206616[] = {{&S85071[101], 9}, {NULL}};
const Segment SGI206617[] = {{&S85071[74], 6}, {NULL}};
const Segment SGI206618[] = {{&S85072[34], 10}, {NULL}};
const Segment SGI206619[] = {{&S85072[27], 6}, {NULL}};
const Segment SGI206620[] = {{&S85072[97], 9}, {NULL}};
const Segment SGI206621[] = {{&S85072[70], 6}, {NULL}};
const Segment SGI206622[] = {{&S65155[34], 10}, {NULL}};
const Segment SGI206623[] = {{&S65155[27], 6}, {NULL}};
const Segment SGI206624[] = {{&S65155[96], 9}, {NULL}};
const Segment SGI206625[] = {{&S65155[69], 6}, {NULL}};
const Segment SGI206626[] = {{&S85076[59], 10}, {NULL}};
const Segment SGI206627[] = {{&S85076[59], 22}, {NULL}};
const Segment SGI206628[] = {{&S85076[45], 6}, {NULL}};
const Segment SGI206629[] = {{&S85076[46], 3}, {NULL}};
const Segment SGI206630[] = {{&S85076[95], 6}, {NULL}};
const Segment SGI206631[] = {{&S85076[0], 14}, {NULL}};
const Segment SGI206632[] = {{&S85078[44], 10}, {NULL}};
const Segment SGI206633[] = {{&S85078[44], 22}, {NULL}};
const Segment SGI206634[] = {{&S85078[30], 6}, {NULL}};
const Segment SGI206635[] = {{&S85078[146], 4}, {NULL}};
const Segment SGI206636[] = {{&S65163[34], 10}, {NULL}};
const Segment SGI206637[] = {{&S85078[172], 3}, {NULL}};
const Segment SGI206638[] = {{&S65163[27], 6}, {NULL}};
const Segment SGI206639[] = {{&S85078[31], 3}, {&S85078[221], 5}, {NULL}};
const Segment SGI206640[] = {{&S85078[80], 6}, {NULL}};
const Segment SGI206641[] = {{&S65163[97], 9}, {NULL}};
const Segment SGI206642[] = {{&S65163[70], 6}, {NULL}};
const Segment SGI206643[] = {{&S65164[34], 10}, {NULL}};
const Segment SGI206644[] = {{&S65164[27], 6}, {NULL}};
const Segment SGI206645[] = {{&S65164[96], 9}, {NULL}};
const Segment SGI206646[] = {{&S65164[69], 6}, {NULL}};
const Segment SGI206647[] = {{&S65165[34], 10}, {NULL}};
const Segment SGI206648[] = {{&S65165[27], 6}, {NULL}};
const Segment SGI206649[] = {{&S65165[97], 9}, {NULL}};
const Segment SGI206650[] = {{&S65165[70], 6}, {NULL}};
const Segment SGI206651[] = {{&S85082[34], 18}, {NULL}};
const Segment SGI206652[] = {{&S85082[27], 6}, {NULL}};
const Segment SGI206653[] = {{&S85082[105], 9}, {NULL}};
const Segment SGI206654[] = {{&S85082[78], 6}, {NULL}};
const Segment SGI206655[] = {{&S85083[34], 11}, {NULL}};
const Segment SGI206656[] = {{&S85083[27], 6}, {NULL}};
const Segment SGI206657[] = {{&S85083[98], 9}, {NULL}};
const Segment SGI206658[] = {{&S85083[71], 6}, {NULL}};
const Segment SGI206659[] = {{&S65171[34], 6}, {NULL}};
const Segment SGI206660[] = {{&S65171[27], 6}, {NULL}};
const Segment SGI206661[] = {{&S85087[44], 10}, {NULL}};
const Segment SGI206662[] = {{&S79769[44], 9}, {NULL}};
const Segment SGI206663[] = {{&S85087[44], 22}, {NULL}};
const Segment SGI206664[] = {{&S79769[44], 21}, {NULL}};
const Segment SGI206665[] = {{&S85087[30], 6}, {NULL}};
const Segment SGI206666[] = {{&S79769[30], 6}, {NULL}};
const Segment SGI206667[] = {{&S65171[92], 9}, {NULL}};
const Segment SGI206668[] = {{&S65171[65], 6}, {NULL}};
const Segment SGI206669[] = {{&S85087[31], 3}, {NULL}};
const Segment SGI206670[] = {{&S79769[114], 3}, {NULL}};
const Segment SGI206671[] = {{&S85087[80], 6}, {NULL}};
const Segment SGI206672[] = {{&S79769[79], 6}, {NULL}};
const Segment SGI206673[] = {{&S85089[45], 10}, {NULL}};
const Segment SGI206674[] = {{&S85089[30], 6}, {NULL}};
const Segment SGI206675[] = {{&S85089[31], 3}, {NULL}};
const Segment SGI206676[] = {{&S85089[81], 5}, {NULL}};
const Segment SGI206677[] = {{&S85089[139], 13}, {NULL}};
const Segment SGI206678[] = {{&S85090[45], 13}, {NULL}};
const Segment SGI206679[] = {{&S85090[30], 6}, {NULL}};
const Segment SGI206680[] = {{&S85090[31], 3}, {NULL}};
const Segment SGI206681[] = {{&S85090[84], 5}, {NULL}};
const Segment SGI206682[] = {{&S85090[142], 13}, {NULL}};
const Segment SGI206683[] = {{&S85091[34], 8}, {NULL}};
const Segment SGI206684[] = {{&S85091[27], 6}, {NULL}};
const Segment SGI206685[] = {{&S85091[100], 9}, {NULL}};
const Segment SGI206686[] = {{&S85091[73], 6}, {NULL}};
const Segment SGI206687[] = {{&S85096[44], 10}, {NULL}};
const Segment SGI206688[] = {{&S85096[44], 22}, {NULL}};
const Segment SGI206689[] = {{&S85096[30], 6}, {NULL}};
const Segment SGI206690[] = {{&S85096[31], 3}, {NULL}};
const Segment SGI206691[] = {{&S85096[80], 6}, {NULL}};
const Segment SGI206692[] = {{&S79784[44], 10}, {NULL}};
const Segment SGI206693[] = {{&S79784[44], 22}, {NULL}};
const Segment SGI206694[] = {{&S79784[30], 6}, {NULL}};
const Segment SGI206695[] = {{&S79784[115], 3}, {NULL}};
const Segment SGI206696[] = {{&S79784[80], 6}, {NULL}};
const Segment SGI206697[] = {{&S79784[143], 3}, {NULL}};
const Segment SGI206698[] = {{&S79786[44], 11}, {NULL}};
const Segment SGI206699[] = {{&S79786[44], 23}, {NULL}};
const Segment SGI206700[] = {{&S79786[30], 6}, {NULL}};
const Segment SGI206701[] = {{&S75206[34], 11}, {NULL}};
const Segment SGI206702[] = {{&S75206[27], 6}, {NULL}};
const Segment SGI206703[] = {{&S79786[116], 3}, {NULL}};
const Segment SGI206704[] = {{&S79786[81], 6}, {NULL}};
const Segment SGI206705[] = {{&S75206[98], 9}, {NULL}};
const Segment SGI206706[] = {{&S79786[144], 3}, {NULL}};
const Segment SGI206707[] = {{&S75206[71], 6}, {NULL}};
const Segment SGI206708[] = {{&S85103[34], 10}, {NULL}};
const Segment SGI206709[] = {{&S85103[27], 6}, {NULL}};
const Segment SGI206710[] = {{&S75207[34], 11}, {NULL}};
const Segment SGI206711[] = {{&S85103[97], 9}, {NULL}};
const Segment SGI206712[] = {{&S75207[27], 6}, {NULL}};
const Segment SGI206713[] = {{&S85103[70], 6}, {NULL}};
const Segment SGI206714[] = {{&S75207[98], 9}, {NULL}};
const Segment SGI206715[] = {{&S75207[71], 6}, {NULL}};
const Segment SGI206716[] = {{&S75208[34], 11}, {NULL}};
const Segment SGI206717[] = {{&S75208[27], 6}, {NULL}};
const Segment SGI206718[] = {{&S75208[98], 9}, {NULL}};
const Segment SGI206719[] = {{&S75208[71], 6}, {NULL}};
const Segment SGI206720[] = {{&S85111[44], 10}, {NULL}};
const Segment SGI206721[] = {{&S85111[44], 22}, {NULL}};
const Segment SGI206722[] = {{&S85111[30], 6}, {NULL}};
const Segment SGI206723[] = {{&S85111[31], 3}, {NULL}};
const Segment SGI206724[] = {{&S85111[95], 6}, {NULL}};
const Segment SGI206725[] = {{&S85111[68], 8}, {NULL}};
const Segment SGI206726[] = {{&S85112[34], 7}, {NULL}};
const Segment SGI206727[] = {{&S85112[27], 6}, {NULL}};
const Segment SGI206728[] = {{&S85112[94], 9}, {NULL}};
const Segment SGI206729[] = {{&S85112[67], 6}, {NULL}};
const Segment SGI206730[] = {{&S79802[44], 16}, {NULL}};
const Segment SGI206731[] = {{&S79802[44], 28}, {NULL}};
const Segment SGI206732[] = {{&S79802[30], 6}, {NULL}};
const Segment SGI206733[] = {{&S79802[121], 3}, {NULL}};
const Segment SGI206734[] = {{&S79802[86], 6}, {NULL}};
const Segment SGI206735[] = {{&S85118[34], 7}, {NULL}};
const Segment SGI206736[] = {{&S85118[27], 6}, {NULL}};
const Segment SGI206737[] = {{&S85118[98], 9}, {NULL}};
const Segment SGI206738[] = {{&S85118[71], 6}, {NULL}};
const Segment SGI206739[] = {{&S85119[34], 13}, {NULL}};
const Segment SGI206740[] = {{&S85119[27], 6}, {NULL}};
const Segment SGI206741[] = {{&S85119[100], 9}, {NULL}};
const Segment SGI206742[] = {{&S85119[73], 6}, {NULL}};
const Segment SGI206743[] = {{&S85120[34], 11}, {NULL}};
const Segment SGI206744[] = {{&S85120[27], 6}, {NULL}};
const Segment SGI206745[] = {{&S85120[98], 9}, {NULL}};
const Segment SGI206746[] = {{&S85120[71], 6}, {NULL}};
const Segment SGI206747[] = {{&S75228[44], 8}, {NULL}};
const Segment SGI206748[] = {{&S85121[34], 11}, {NULL}};
const Segment SGI206749[] = {{&S75228[44], 20}, {NULL}};
const Segment SGI206750[] = {{&S75228[30], 6}, {NULL}};
const Segment SGI206751[] = {{&S85121[27], 6}, {NULL}};
const Segment SGI206752[] = {{&S75228[113], 3}, {NULL}};
const Segment SGI206753[] = {{&S85121[98], 9}, {NULL}};
const Segment SGI206754[] = {{&S75228[78], 6}, {NULL}};
const Segment SGI206755[] = {{&S85121[71], 6}, {NULL}};
const Segment SGI206756[] = {{&S85122[34], 11}, {NULL}};
const Segment SGI206757[] = {{&S75229[38], 8}, {NULL}};
const Segment SGI206758[] = {{&S75229[31], 6}, {NULL}};
const Segment SGI206759[] = {{&S85122[27], 6}, {NULL}};
const Segment SGI206760[] = {{&S85122[98], 9}, {NULL}};
const Segment SGI206761[] = {{&S85122[71], 6}, {NULL}};
const Segment SGI206762[] = {{&S75230[44], 10}, {NULL}};
const Segment SGI206763[] = {{&S75230[44], 22}, {NULL}};
const Segment SGI206764[] = {{&S75230[30], 6}, {NULL}};
const Segment SGI206765[] = {{&S75230[115], 3}, {NULL}};
const Segment SGI206766[] = {{&S75230[80], 6}, {NULL}};
const Segment SGI206767[] = {{&S75231[44], 10}, {NULL}};
const Segment SGI206768[] = {{&S75231[44], 22}, {NULL}};
const Segment SGI206769[] = {{&S75231[30], 6}, {NULL}};
const Segment SGI206770[] = {{&S75231[83], 6}, {NULL}};
const Segment SGI206771[] = {{&S85127[34], 20}, {NULL}};
const Segment SGI206772[] = {{&S85127[27], 6}, {NULL}};
const Segment SGI206773[] = {{&S85127[107], 9}, {NULL}};
const Segment SGI206774[] = {{&S85127[80], 6}, {NULL}};
const Segment SGI206775[] = {{&S85128[34], 21}, {NULL}};
const Segment SGI206776[] = {{&S85128[27], 6}, {NULL}};
const Segment SGI206777[] = {{&S85128[108], 9}, {NULL}};
const Segment SGI206778[] = {{&S85128[81], 6}, {NULL}};
const Segment SGI206779[] = {{&S75236[34], 7}, {NULL}};
const Segment SGI206780[] = {{&S75236[27], 6}, {NULL}};
const Segment SGI206781[] = {{&S75236[98], 9}, {NULL}};
const Segment SGI206782[] = {{&S75236[71], 6}, {NULL}};
const Segment SGI206783[] = {{&S85132[34], 15}, {NULL}};
const Segment SGI206784[] = {{&S85132[27], 6}, {NULL}};
const Segment SGI206785[] = {{&S85132[102], 9}, {NULL}};
const Segment SGI206786[] = {{&S85132[75], 6}, {NULL}};
const Segment SGI206787[] = {{&S65238[34], 10}, {NULL}};
const Segment SGI206788[] = {{&S65238[27], 6}, {NULL}};
const Segment SGI206789[] = {{&S65238[97], 9}, {NULL}};
const Segment SGI206790[] = {{&S65238[70], 6}, {NULL}};
const Segment SGI206791[] = {{&S65239[34], 10}, {NULL}};
const Segment SGI206792[] = {{&S65239[27], 6}, {NULL}};
const Segment SGI206793[] = {{&S65239[96], 9}, {NULL}};
const Segment SGI206794[] = {{&S65239[69], 6}, {NULL}};
const Segment SGI206795[] = {{&S79839[44], 7}, {NULL}};
const Segment SGI206796[] = {{&S79839[44], 19}, {NULL}};
const Segment SGI206797[] = {{&S79839[30], 6}, {NULL}};
const Segment SGI206798[] = {{&S79839[128], 3}, {NULL}};
const Segment SGI206799[] = {{&S79839[93], 6}, {NULL}};
const Segment SGI206800[] = {{&S79839[65], 8}, {NULL}};
const Segment SGI206801[] = {{&S65253[44], 10}, {NULL}};
const Segment SGI206802[] = {{&S65253[44], 21}, {NULL}};
const Segment SGI206803[] = {{&S65253[30], 6}, {NULL}};
const Segment SGI206804[] = {{&S65253[114], 3}, {NULL}};
const Segment SGI206805[] = {{&S65253[79], 6}, {NULL}};
const Segment SGI206806[] = {{&S65254[38], 10}, {NULL}};
const Segment SGI206807[] = {{&S65254[31], 6}, {NULL}};
const Segment SGI206808[] = {{&S79851[34], 10}, {NULL}};
const Segment SGI206809[] = {{&S79851[27], 6}, {NULL}};
const Segment SGI206810[] = {{&S79851[97], 9}, {NULL}};
const Segment SGI206811[] = {{&S79851[70], 6}, {NULL}};
const Segment SGI206812[] = {{&S65255[34], 12}, {NULL}};
const Segment SGI206813[] = {{&S65255[27], 6}, {NULL}};
const Segment SGI206814[] = {{&S65255[99], 9}, {NULL}};
const Segment SGI206815[] = {{&S65255[72], 6}, {NULL}};
const Segment SGI206816[] = {{&S75264[34], 9}, {NULL}};
const Segment SGI206817[] = {{&S85161[44], 10}, {NULL}};
const Segment SGI206818[] = {{&S75264[27], 6}, {NULL}};
const Segment SGI206819[] = {{&S85161[44], 22}, {NULL}};
const Segment SGI206820[] = {{&S85161[30], 6}, {NULL}};
const Segment SGI206821[] = {{&S75264[96], 9}, {NULL}};
const Segment SGI206822[] = {{&S85161[31], 3}, {NULL}};
const Segment SGI206823[] = {{&S75264[69], 6}, {NULL}};
const Segment SGI206824[] = {{&S65256[34], 25}, {NULL}};
const Segment SGI206825[] = {{&S85161[95], 6}, {NULL}};
const Segment SGI206826[] = {{&S65256[27], 6}, {NULL}};
const Segment SGI206827[] = {{&S85161[68], 8}, {NULL}};
const Segment SGI206828[] = {{&S65256[112], 9}, {NULL}};
const Segment SGI206829[] = {{&S65256[85], 6}, {NULL}};
const Segment SGI206830[] = {{&S75265[34], 11}, {NULL}};
const Segment SGI206831[] = {{&S75265[27], 6}, {NULL}};
const Segment SGI206832[] = {{&S85162[44], 10}, {NULL}};
const Segment SGI206833[] = {{&S85162[44], 22}, {NULL}};
const Segment SGI206834[] = {{&S85162[30], 6}, {NULL}};
const Segment SGI206835[] = {{&S75265[98], 9}, {NULL}};
const Segment SGI206836[] = {{&S75265[71], 6}, {NULL}};
const Segment SGI206837[] = {{&S65257[34], 10}, {NULL}};
const Segment SGI206838[] = {{&S85162[31], 3}, {NULL}};
const Segment SGI206839[] = {{&S65257[27], 6}, {NULL}};
const Segment SGI206840[] = {{&S85162[95], 6}, {NULL}};
const Segment SGI206841[] = {{&S85162[68], 8}, {NULL}};
const Segment SGI206842[] = {{&S65257[97], 9}, {NULL}};
const Segment SGI206843[] = {{&S65257[70], 6}, {NULL}};
const Segment SGI206844[] = {{&S85163[44], 10}, {NULL}};
const Segment SGI206845[] = {{&S85163[44], 22}, {NULL}};
const Segment SGI206846[] = {{&S85163[30], 6}, {NULL}};
const Segment SGI206847[] = {{&S85163[31], 3}, {NULL}};
const Segment SGI206848[] = {{&S85163[95], 6}, {NULL}};
const Segment SGI206849[] = {{&S85163[68], 8}, {NULL}};
const Segment SGI206850[] = {{&S79860[44], 7}, {NULL}};
const Segment SGI206851[] = {{&S79860[44], 19}, {NULL}};
const Segment SGI206852[] = {{&S79860[30], 6}, {NULL}};
const Segment SGI206853[] = {{&S79860[112], 3}, {NULL}};
const Segment SGI206854[] = {{&S79860[77], 6}, {NULL}};
const Segment SGI206855[] = {{&S85174[34], 9}, {NULL}};
const Segment SGI206856[] = {{&S85174[27], 6}, {NULL}};
const Segment SGI206857[] = {{&S85174[96], 9}, {NULL}};
const Segment SGI206858[] = {{&S85174[69], 6}, {NULL}};
const Segment SGI206859[] = {{&S85176[44], 8}, {NULL}};
const Segment SGI206860[] = {{&S85176[44], 20}, {NULL}};
const Segment SGI206861[] = {{&S85176[30], 6}, {NULL}};
const Segment SGI206862[] = {{&S85176[31], 3}, {NULL}};
const Segment SGI206863[] = {{&S85176[78], 6}, {NULL}};
const Segment SGI206864[] = {{&S65277[44], 12}, {NULL}};
const Segment SGI206865[] = {{&S65277[44], 23}, {NULL}};
const Segment SGI206866[] = {{&S65277[30], 6}, {NULL}};
const Segment SGI206867[] = {{&S65277[116], 3}, {NULL}};
const Segment SGI206868[] = {{&S65277[81], 6}, {NULL}};
const Segment SGI206869[] = {{&S65278[44], 12}, {NULL}};
const Segment SGI206870[] = {{&S65278[44], 25}, {NULL}};
const Segment SGI206871[] = {{&S65278[30], 6}, {NULL}};
const Segment SGI206872[] = {{&S65278[118], 3}, {NULL}};
const Segment SGI206873[] = {{&S65278[83], 6}, {NULL}};
const Segment SGI206874[] = {{&S65279[38], 12}, {NULL}};
const Segment SGI206875[] = {{&S65279[31], 6}, {NULL}};
const Segment SGI206876[] = {{&S75303[44], 18}, {NULL}};
const Segment SGI206877[] = {{&S75303[44], 30}, {NULL}};
const Segment SGI206878[] = {{&S75303[30], 6}, {NULL}};
const Segment SGI206879[] = {{&S85192[34], 13}, {NULL}};
const Segment SGI206880[] = {{&S85192[27], 6}, {NULL}};
const Segment SGI206881[] = {{&S75303[139], 3}, {NULL}};
const Segment SGI206882[] = {{&S75303[104], 6}, {NULL}};
const Segment SGI206883[] = {{&S75303[76], 8}, {NULL}};
const Segment SGI206884[] = {{&S85192[113], 9}, {NULL}};
const Segment SGI206885[] = {{&S85192[86], 6}, {NULL}};
const Segment SGI206886[] = {{&S75304[44], 18}, {NULL}};
const Segment SGI206887[] = {{&S75304[44], 30}, {NULL}};
const Segment SGI206888[] = {{&S75304[30], 6}, {NULL}};
const Segment SGI206889[] = {{&S85193[45], 8}, {NULL}};
const Segment SGI206890[] = {{&S85193[30], 6}, {NULL}};
const Segment SGI206891[] = {{&S75304[139], 3}, {NULL}};
const Segment SGI206892[] = {{&S75304[104], 6}, {NULL}};
const Segment SGI206893[] = {{&S85193[31], 3}, {NULL}};
const Segment SGI206894[] = {{&S75304[76], 8}, {NULL}};
const Segment SGI206895[] = {{&S85193[83], 5}, {NULL}};
const Segment SGI206896[] = {{&S85193[141], 13}, {NULL}};
const Segment SGI206897[] = {{&S85194[34], 18}, {NULL}};
const Segment SGI206898[] = {{&S85194[27], 6}, {NULL}};
const Segment SGI206899[] = {{&S85194[105], 9}, {NULL}};
const Segment SGI206900[] = {{&S85194[78], 6}, {NULL}};
const Segment SGI206901[] = {{&S75307[34], 10}, {NULL}};
const Segment SGI206902[] = {{&S75307[27], 6}, {NULL}};
const Segment SGI206903[] = {{&S75307[97], 9}, {NULL}};
const Segment SGI206904[] = {{&S75307[70], 6}, {NULL}};
const Segment SGI206905[] = {{&S85195[34], 7}, {NULL}};
const Segment SGI206906[] = {{&S85195[27], 6}, {NULL}};
const Segment SGI206907[] = {{&S85195[99], 9}, {NULL}};
const Segment SGI206908[] = {{&S85195[72], 6}, {NULL}};
const Segment SGI206909[] = {{&S75308[44], 10}, {NULL}};
const Segment SGI206910[] = {{&S75308[44], 22}, {NULL}};
const Segment SGI206911[] = {{&S75308[30], 6}, {NULL}};
const Segment SGI206912[] = {{&S65300[34], 10}, {NULL}};
const Segment SGI206913[] = {{&S65300[27], 6}, {NULL}};
const Segment SGI206914[] = {{&S75308[80], 6}, {NULL}};
const Segment SGI206915[] = {{&S65300[96], 9}, {NULL}};
const Segment SGI206916[] = {{&S65300[69], 6}, {NULL}};
const Segment SGI206917[] = {{&S75308[107], 13}, {NULL}};
const Segment SGI206918[] = {{&S85197[34], 4}, {NULL}};
const Segment SGI206919[] = {{&S85197[27], 6}, {NULL}};
const Segment SGI206920[] = {{&S65301[34], 10}, {NULL}};
const Segment SGI206921[] = {{&S75309[34], 7}, {NULL}};
const Segment SGI206922[] = {{&S65301[27], 6}, {NULL}};
const Segment SGI206923[] = {{&S75309[27], 6}, {NULL}};
const Segment SGI206924[] = {{&S85197[91], 9}, {NULL}};
const Segment SGI206925[] = {{&S85197[64], 6}, {NULL}};
const Segment SGI206926[] = {{&S65301[96], 9}, {NULL}};
const Segment SGI206927[] = {{&S65301[69], 6}, {NULL}};
const Segment SGI206928[] = {{&S75309[104], 9}, {NULL}};
const Segment SGI206929[] = {{&S75309[77], 6}, {NULL}};
const Segment SGI206930[] = {{&S75309[145], 8}, {NULL}};
const Segment SGI206931[] = {{&S85198[34], 4}, {NULL}};
const Segment SGI206932[] = {{&S85198[27], 6}, {NULL}};
const Segment SGI206933[] = {{&S85198[91], 9}, {NULL}};
const Segment SGI206934[] = {{&S85198[64], 6}, {NULL}};
const Segment SGI206935[] = {{&S85199[34], 24}, {NULL}};
const Segment SGI206936[] = {{&S85199[27], 6}, {NULL}};
const Segment SGI206937[] = {{&S85199[111], 9}, {NULL}};
const Segment SGI206938[] = {{&S85199[84], 6}, {NULL}};
const Segment SGI206939[] = {{&S85200[44], 14}, {NULL}};
const Segment SGI206940[] = {{&S85200[44], 31}, {NULL}};
const Segment SGI206941[] = {{&S85200[30], 6}, {NULL}};
const Segment SGI206942[] = {{&S85200[89], 6}, {NULL}};
const Segment SGI206943[] = {{&S85200[116], 13}, {NULL}};
const Segment SGI206944[] = {{&S79913[34], 15}, {NULL}};
const Segment SGI206945[] = {{&S79913[27], 6}, {NULL}};
const Segment SGI206946[] = {{&S79913[102], 9}, {NULL}};
const Segment SGI206947[] = {{&S79913[75], 6}, {NULL}};
const Segment SGI206948[] = {{&S85206[44], 7}, {NULL}};
const Segment SGI206949[] = {{&S85206[44], 19}, {NULL}};
const Segment SGI206950[] = {{&S85206[30], 6}, {NULL}};
const Segment SGI206951[] = {{&S85206[31], 3}, {NULL}};
const Segment SGI206952[] = {{&S85206[77], 5}, {NULL}};
const Segment SGI206953[] = {{&S79923[34], 25}, {NULL}};
const Segment SGI206954[] = {{&S79923[27], 6}, {NULL}};
const Segment SGI206955[] = {{&S79923[112], 9}, {NULL}};
const Segment SGI206956[] = {{&S79923[85], 6}, {NULL}};
const Segment SGI206957[] = {{&S79924[34], 25}, {NULL}};
const Segment SGI206958[] = {{&S79924[27], 6}, {NULL}};
const Segment SGI206959[] = {{&S79924[112], 9}, {NULL}};
const Segment SGI206960[] = {{&S79924[85], 6}, {NULL}};
const Segment SGI206961[] = {{&S85213[45], 10}, {NULL}};
const Segment SGI206962[] = {{&S85213[30], 6}, {NULL}};
const Segment SGI206963[] = {{&S79925[44], 9}, {NULL}};
const Segment SGI206964[] = {{&S79925[44], 29}, {NULL}};
const Segment SGI206965[] = {{&S79925[30], 6}, {NULL}};
const Segment SGI206966[] = {{&S85213[31], 3}, {NULL}};
const Segment SGI206967[] = {{&S85213[81], 5}, {NULL}};
const Segment SGI206968[] = {{&S79925[122], 3}, {NULL}};
const Segment SGI206969[] = {{&S85213[139], 13}, {NULL}};
const Segment SGI206970[] = {{&S79925[87], 6}, {NULL}};
const Segment SGI206971[] = {{&S85214[34], 9}, {NULL}};
const Segment SGI206972[] = {{&S85214[27], 6}, {NULL}};
const Segment SGI206973[] = {{&S79926[44], 9}, {NULL}};
const Segment SGI206974[] = {{&S79926[44], 29}, {NULL}};
const Segment SGI206975[] = {{&S79926[30], 6}, {NULL}};
const Segment SGI206976[] = {{&S85214[96], 9}, {NULL}};
const Segment SGI206977[] = {{&S85214[69], 6}, {NULL}};
const Segment SGI206978[] = {{&S79926[122], 3}, {NULL}};
const Segment SGI206979[] = {{&S79926[87], 6}, {NULL}};
const Segment SGI206980[] = {{&S79927[44], 9}, {NULL}};
const Segment SGI206981[] = {{&S79927[44], 29}, {NULL}};
const Segment SGI206982[] = {{&S79927[30], 6}, {NULL}};
const Segment SGI206983[] = {{&S79927[122], 3}, {NULL}};
const Segment SGI206984[] = {{&S79927[87], 6}, {NULL}};
const Segment SGI206985[] = {{&S85216[34], 12}, {NULL}};
const Segment SGI206986[] = {{&S85216[27], 6}, {NULL}};
const Segment SGI206987[] = {{&S85216[99], 9}, {NULL}};
const Segment SGI206988[] = {{&S85216[72], 6}, {NULL}};
const Segment SGI206989[] = {{&S79928[44], 9}, {NULL}};
const Segment SGI206990[] = {{&S79928[44], 29}, {NULL}};
const Segment SGI206991[] = {{&S79928[30], 6}, {NULL}};
const Segment SGI206992[] = {{&S79928[122], 3}, {NULL}};
const Segment SGI206993[] = {{&S79928[87], 6}, {NULL}};
const Segment SGI206994[] = {{&S65339[44], 10}, {NULL}};
const Segment SGI206995[] = {{&S65339[44], 21}, {NULL}};
const Segment SGI206996[] = {{&S65339[30], 6}, {NULL}};
const Segment SGI206997[] = {{&S85222[44], 7}, {NULL}};
const Segment SGI206998[] = {{&S85222[44], 24}, {NULL}};
const Segment SGI206999[] = {{&S85222[30], 6}, {NULL}};
const Segment SGI207000[] = {{&S65339[114], 3}, {NULL}};
const Segment SGI207001[] = {{&S65339[79], 6}, {NULL}};
const Segment SGI207002[] = {{&S85222[82], 6}, {NULL}};
const Segment SGI207003[] = {{&S85222[109], 13}, {NULL}};
const Segment SGI207004[] = {{&S75346[45], 10}, {NULL}};
const Segment SGI207005[] = {{&S75346[30], 6}, {NULL}};
const Segment SGI207006[] = {{&S79935[34], 11}, {NULL}};
const Segment SGI207007[] = {{&S79935[27], 6}, {NULL}};
const Segment SGI207008[] = {{&S75346[115], 3}, {NULL}};
const Segment SGI207009[] = {{&S65341[44], 10}, {NULL}};
const Segment SGI207010[] = {{&S65341[44], 22}, {NULL}};
const Segment SGI207011[] = {{&S75346[81], 5}, {NULL}};
const Segment SGI207012[] = {{&S65341[30], 6}, {NULL}};
const Segment SGI207013[] = {{&S79935[98], 9}, {NULL}};
const Segment SGI207014[] = {{&S75346[139], 13}, {NULL}};
const Segment SGI207015[] = {{&S79935[71], 6}, {NULL}};
const Segment SGI207016[] = {{&S65341[115], 3}, {NULL}};
const Segment SGI207017[] = {{&S65341[80], 6}, {NULL}};
const Segment SGI207018[] = {{&S74492[38], 10}, {NULL}};
const Segment SGI207019[] = {{&S74492[31], 6}, {NULL}};
const Segment SGI207020[] = {{&S74493[59], 10}, {NULL}};
const Segment SGI207021[] = {{&S74493[59], 67}, {NULL}};
const Segment SGI207022[] = {{&S74493[45], 6}, {NULL}};
const Segment SGI207023[] = {{&S74494[44], 10}, {NULL}};
const Segment SGI207024[] = {{&S74494[44], 22}, {NULL}};
const Segment SGI207025[] = {{&S74494[30], 6}, {NULL}};
const Segment SGI207026[] = {{&S74494[114], 3}, {NULL}};
const Segment SGI207027[] = {{&S74494[80], 5}, {NULL}};
const Segment SGI207028[] = {{&S79948[44], 7}, {NULL}};
const Segment SGI207029[] = {{&S79948[44], 19}, {NULL}};
const Segment SGI207030[] = {{&S79948[30], 6}, {NULL}};
const Segment SGI207031[] = {{&S79948[112], 3}, {NULL}};
const Segment SGI207032[] = {{&S79948[77], 6}, {NULL}};
const Segment SGI207033[] = {{&S75369[34], 11}, {NULL}};
const Segment SGI207034[] = {{&S75369[27], 6}, {NULL}};
const Segment SGI207035[] = {{&S75369[98], 9}, {NULL}};
const Segment SGI207036[] = {{&S75369[71], 6}, {NULL}};
const Segment SGI207037[] = {{&S79959[34], 6}, {NULL}};
const Segment SGI207038[] = {{&S79959[27], 6}, {NULL}};
const Segment SGI207039[] = {{&S79959[93], 9}, {NULL}};
const Segment SGI207040[] = {{&S79959[66], 6}, {NULL}};
const Segment SGI207041[] = {{&S79965[44], 12}, {NULL}};
const Segment SGI207042[] = {{&S79965[44], 37}, {NULL}};
const Segment SGI207043[] = {{&S79965[30], 6}, {NULL}};
const Segment SGI207044[] = {{&S79965[130], 3}, {NULL}};
const Segment SGI207045[] = {{&S79965[95], 6}, {NULL}};
const Segment SGI207046[] = {{&S79966[44], 14}, {NULL}};
const Segment SGI207047[] = {{&S79966[44], 31}, {NULL}};
const Segment SGI207048[] = {{&S79966[30], 6}, {NULL}};
const Segment SGI207049[] = {{&S85260[44], 11}, {NULL}};
const Segment SGI207050[] = {{&S85260[44], 23}, {NULL}};
const Segment SGI207051[] = {{&S85260[30], 6}, {NULL}};
const Segment SGI207052[] = {{&S79966[89], 6}, {NULL}};
const Segment SGI207053[] = {{&S79966[116], 13}, {NULL}};
const Segment SGI207054[] = {{&S85260[31], 3}, {NULL}};
const Segment SGI207055[] = {{&S85260[81], 6}, {NULL}};
const Segment SGI207056[] = {{&S85260[151], 3}, {NULL}};
const Segment SGI207057[] = {{&S79968[44], 15}, {NULL}};
const Segment SGI207058[] = {{&S79968[44], 33}, {NULL}};
const Segment SGI207059[] = {{&S65381[34], 19}, {NULL}};
const Segment SGI207060[] = {{&S79968[30], 6}, {NULL}};
const Segment SGI207061[] = {{&S65381[27], 6}, {NULL}};
const Segment SGI207062[] = {{&S65381[105], 9}, {NULL}};
const Segment SGI207063[] = {{&S79968[91], 6}, {NULL}};
const Segment SGI207064[] = {{&S65381[78], 6}, {NULL}};
const Segment SGI207065[] = {{&S79968[118], 13}, {NULL}};
const Segment SGI207066[] = {{&S65382[34], 13}, {NULL}};
const Segment SGI207067[] = {{&S65382[27], 6}, {NULL}};
const Segment SGI207068[] = {{&S65382[100], 9}, {NULL}};
const Segment SGI207069[] = {{&S65382[73], 6}, {NULL}};
const Segment SGI207070[] = {{&S79970[34], 6}, {NULL}};
const Segment SGI207071[] = {{&S79970[27], 6}, {NULL}};
const Segment SGI207072[] = {{&S79970[93], 9}, {NULL}};
const Segment SGI207073[] = {{&S79970[66], 6}, {NULL}};
const Segment SGI207074[] = {{&S79971[34], 13}, {NULL}};
const Segment SGI207075[] = {{&S79971[27], 6}, {NULL}};
const Segment SGI207076[] = {{&S79971[114], 9}, {NULL}};
const Segment SGI207077[] = {{&S79971[87], 6}, {NULL}};
const Segment SGI207078[] = {{&S79972[34], 12}, {NULL}};
const Segment SGI207079[] = {{&S79972[27], 6}, {NULL}};
const Segment SGI207080[] = {{&S79972[112], 9}, {NULL}};
const Segment SGI207081[] = {{&S79972[85], 6}, {NULL}};
const Segment SGI207082[] = {{&S79973[34], 12}, {NULL}};
const Segment SGI207083[] = {{&S79973[27], 6}, {NULL}};
const Segment SGI207084[] = {{&S79973[99], 9}, {NULL}};
const Segment SGI207085[] = {{&S79973[72], 6}, {NULL}};
const Segment SGI207086[] = {{&S79973[140], 8}, {NULL}};
const Segment SGI207087[] = {{&S85273[34], 9}, {NULL}};
const Segment SGI207088[] = {{&S85273[27], 6}, {NULL}};
const Segment SGI207089[] = {{&S85273[96], 9}, {NULL}};
const Segment SGI207090[] = {{&S85273[69], 6}, {NULL}};
const Segment SGI207091[] = {{&S79975[34], 6}, {NULL}};
const Segment SGI207092[] = {{&S79975[27], 6}, {NULL}};
const Segment SGI207093[] = {{&S79975[93], 9}, {NULL}};
const Segment SGI207094[] = {{&S79975[66], 6}, {NULL}};
const Segment SGI207095[] = {{&S85274[34], 9}, {NULL}};
const Segment SGI207096[] = {{&S85274[27], 6}, {NULL}};
const Segment SGI207097[] = {{&S85274[96], 9}, {NULL}};
const Segment SGI207098[] = {{&S85274[69], 6}, {NULL}};
const Segment SGI207099[] = {{&S79976[44], 14}, {NULL}};
const Segment SGI207100[] = {{&S79976[44], 26}, {NULL}};
const Segment SGI207101[] = {{&S79976[30], 6}, {NULL}};
const Segment SGI207102[] = {{&S79976[84], 6}, {NULL}};
const Segment SGI207103[] = {{&S79976[111], 13}, {NULL}};
const Segment SGI207104[] = {{&S79977[44], 23}, {NULL}};
const Segment SGI207105[] = {{&S79977[44], 35}, {NULL}};
const Segment SGI207106[] = {{&S79977[30], 6}, {NULL}};
const Segment SGI207107[] = {{&S79977[93], 6}, {NULL}};
const Segment SGI207108[] = {{&S79977[120], 13}, {NULL}};
const Segment SGI207109[] = {{&S85279[34], 15}, {NULL}};
const Segment SGI207110[] = {{&S85279[27], 6}, {NULL}};
const Segment SGI207111[] = {{&S85279[109], 9}, {NULL}};
const Segment SGI207112[] = {{&S85279[82], 6}, {NULL}};
const Segment SGI207113[] = {{&S79982[34], 11}, {NULL}};
const Segment SGI207114[] = {{&S79982[27], 6}, {NULL}};
const Segment SGI207115[] = {{&S79982[98], 9}, {NULL}};
const Segment SGI207116[] = {{&S79982[71], 6}, {NULL}};
const Segment SGI207117[] = {{&S85285[34], 5}, {NULL}};
const Segment SGI207118[] = {{&S85285[27], 6}, {NULL}};
const Segment SGI207119[] = {{&S85285[92], 9}, {NULL}};
const Segment SGI207120[] = {{&S85285[65], 6}, {NULL}};
const Segment SGI207121[] = {{&S75414[44], 6}, {NULL}};
const Segment SGI207122[] = {{&S75414[44], 18}, {NULL}};
const Segment SGI207123[] = {{&S75414[30], 6}, {NULL}};
const Segment SGI207124[] = {{&S75414[127], 3}, {NULL}};
const Segment SGI207125[] = {{&S75414[92], 6}, {NULL}};
const Segment SGI207126[] = {{&S75414[64], 8}, {NULL}};
const Segment SGI207127[] = {{&S85288[34], 7}, {NULL}};
const Segment SGI207128[] = {{&S85288[27], 6}, {NULL}};
const Segment SGI207129[] = {{&S85288[94], 9}, {NULL}};
const Segment SGI207130[] = {{&S85288[67], 6}, {NULL}};
const Segment SGI207131[] = {{&S75416[44], 6}, {NULL}};
const Segment SGI207132[] = {{&S75416[44], 18}, {NULL}};
const Segment SGI207133[] = {{&S75416[30], 6}, {NULL}};
const Segment SGI207134[] = {{&S75416[149], 3}, {NULL}};
const Segment SGI207135[] = {{&S75416[114], 6}, {NULL}};
const Segment SGI207136[] = {{&S75416[64], 8}, {NULL}};
const Segment SGI207137[] = {{&S85296[34], 14}, {NULL}};
const Segment SGI207138[] = {{&S85296[27], 6}, {NULL}};
const Segment SGI207139[] = {{&S85296[101], 9}, {NULL}};
const Segment SGI207140[] = {{&S85296[74], 6}, {NULL}};
const Segment SGI207141[] = {{&S85296[142], 8}, {NULL}};
const Segment SGI207142[] = {{&S75436[34], 10}, {NULL}};
const Segment SGI207143[] = {{&S75436[27], 6}, {NULL}};
const Segment SGI207144[] = {{&S75436[97], 9}, {NULL}};
const Segment SGI207145[] = {{&S75436[70], 6}, {NULL}};
const Segment SGI207146[] = {{&S80016[44], 17}, {NULL}};
const Segment SGI207147[] = {{&S80016[44], 29}, {NULL}};
const Segment SGI207148[] = {{&S80016[30], 6}, {NULL}};
const Segment SGI207149[] = {{&S80016[122], 3}, {NULL}};
const Segment SGI207150[] = {{&S80016[87], 6}, {NULL}};
const Segment SGI207151[] = {{&S85311[34], 7}, {NULL}};
const Segment SGI207152[] = {{&S85311[27], 6}, {NULL}};
const Segment SGI207153[] = {{&S85311[98], 9}, {NULL}};
const Segment SGI207154[] = {{&S85311[71], 6}, {NULL}};
const Segment SGI207155[] = {{&S75440[34], 11}, {NULL}};
const Segment SGI207156[] = {{&S75440[27], 6}, {NULL}};
const Segment SGI207157[] = {{&S75440[98], 9}, {NULL}};
const Segment SGI207158[] = {{&S75440[71], 6}, {NULL}};
const Segment SGI207159[] = {{&S75441[34], 10}, {NULL}};
const Segment SGI207160[] = {{&S75441[27], 6}, {NULL}};
const Segment SGI207161[] = {{&S80021[44], 7}, {NULL}};
const Segment SGI207162[] = {{&S80021[44], 19}, {NULL}};
const Segment SGI207163[] = {{&S80021[30], 6}, {NULL}};
const Segment SGI207164[] = {{&S75441[97], 9}, {NULL}};
const Segment SGI207165[] = {{&S75441[70], 6}, {NULL}};
const Segment SGI207166[] = {{&S80021[112], 3}, {NULL}};
const Segment SGI207167[] = {{&S80021[77], 6}, {NULL}};
const Segment SGI207168[] = {{&S75442[34], 10}, {NULL}};
const Segment SGI207169[] = {{&S75442[27], 6}, {NULL}};
const Segment SGI207170[] = {{&S75442[97], 9}, {NULL}};
const Segment SGI207171[] = {{&S75442[70], 6}, {NULL}};
const Segment SGI207172[] = {{&S75442[138], 8}, {NULL}};
const Segment SGI207173[] = {{&S85320[34], 13}, {NULL}};
const Segment SGI207174[] = {{&S85320[27], 6}, {NULL}};
const Segment SGI207175[] = {{&S85320[100], 9}, {NULL}};
const Segment SGI207176[] = {{&S85320[73], 6}, {NULL}};
const Segment SGI207177[] = {{&S85352[34], 5}, {NULL}};
const Segment SGI207178[] = {{&S85352[27], 6}, {NULL}};
const Segment SGI207179[] = {{&S85352[92], 9}, {NULL}};
const Segment SGI207180[] = {{&S85352[65], 6}, {NULL}};
const Segment SGI207181[] = {{&S65487[44], 10}, {NULL}};
const Segment SGI207182[] = {{&S65487[44], 21}, {NULL}};
const Segment SGI207183[] = {{&S65487[30], 6}, {NULL}};
const Segment SGI207184[] = {{&S65487[137], 3}, {NULL}};
const Segment SGI207185[] = {{&S65487[102], 6}, {NULL}};
const Segment SGI207186[] = {{&S85355[44], 11}, {NULL}};
const Segment SGI207187[] = {{&S85355[44], 23}, {NULL}};
const Segment SGI207188[] = {{&S85355[30], 6}, {NULL}};
const Segment SGI207189[] = {{&S65488[44], 10}, {NULL}};
const Segment SGI207190[] = {{&S65488[44], 21}, {NULL}};
const Segment SGI207191[] = {{&S65488[30], 6}, {NULL}};
const Segment SGI207192[] = {{&S85355[31], 3}, {NULL}};
const Segment SGI207193[] = {{&S85355[81], 6}, {NULL}};
const Segment SGI207194[] = {{&S65488[137], 3}, {NULL}};
const Segment SGI207195[] = {{&S65488[102], 6}, {NULL}};
const Segment SGI207196[] = {{&S85356[34], 7}, {NULL}};
const Segment SGI207197[] = {{&S85356[27], 6}, {NULL}};
const Segment SGI207198[] = {{&S65489[44], 10}, {NULL}};
const Segment SGI207199[] = {{&S65489[44], 21}, {NULL}};
const Segment SGI207200[] = {{&S85356[94], 9}, {NULL}};
const Segment SGI207201[] = {{&S65489[30], 6}, {NULL}};
const Segment SGI207202[] = {{&S85356[67], 6}, {NULL}};
const Segment SGI207203[] = {{&S65489[139], 3}, {NULL}};
const Segment SGI207204[] = {{&S65489[104], 6}, {NULL}};
const Segment SGI207205[] = {{&S65489[74], 9}, {NULL}};
const Segment SGI207206[] = {{&S65490[44], 10}, {NULL}};
const Segment SGI207207[] = {{&S65490[44], 21}, {NULL}};
const Segment SGI207208[] = {{&S65490[30], 6}, {NULL}};
const Segment SGI207209[] = {{&S65490[139], 3}, {NULL}};
const Segment SGI207210[] = {{&S65490[104], 6}, {NULL}};
const Segment SGI207211[] = {{&S65490[74], 9}, {NULL}};
const Segment SGI207212[] = {{&S65491[44], 10}, {NULL}};
const Segment SGI207213[] = {{&S65491[44], 22}, {NULL}};
const Segment SGI207214[] = {{&S65491[30], 6}, {NULL}};
const Segment SGI207215[] = {{&S65491[138], 3}, {NULL}};
const Segment SGI207216[] = {{&S65491[103], 6}, {NULL}};
const Segment SGI207217[] = {{&S65491[75], 9}, {NULL}};
const Segment SGI207218[] = {{&S65492[44], 10}, {NULL}};
const Segment SGI207219[] = {{&S65492[44], 22}, {NULL}};
const Segment SGI207220[] = {{&S65492[30], 6}, {NULL}};
const Segment SGI207221[] = {{&S65492[138], 3}, {NULL}};
const Segment SGI207222[] = {{&S65492[103], 6}, {NULL}};
const Segment SGI207223[] = {{&S65492[75], 9}, {NULL}};
const Segment SGI207224[] = {{&S65494[34], 10}, {NULL}};
const Segment SGI207225[] = {{&S65494[27], 6}, {NULL}};
const Segment SGI207226[] = {{&S65494[96], 9}, {NULL}};
const Segment SGI207227[] = {{&S65494[69], 6}, {NULL}};
const Segment SGI207228[] = {{&S65495[34], 10}, {NULL}};
const Segment SGI207229[] = {{&S65495[27], 6}, {NULL}};
const Segment SGI207230[] = {{&S65495[96], 9}, {NULL}};
const Segment SGI207231[] = {{&S65495[69], 6}, {NULL}};
const Segment SGI207232[] = {{&S85370[34], 8}, {NULL}};
const Segment SGI207233[] = {{&S85370[27], 6}, {NULL}};
const Segment SGI207234[] = {{&S85370[100], 9}, {NULL}};
const Segment SGI207235[] = {{&S85370[73], 6}, {NULL}};
const Segment SGI207236[] = {{&S74647[44], 7}, {NULL}};
const Segment SGI207237[] = {{&S74647[44], 19}, {NULL}};
const Segment SGI207238[] = {{&S74647[30], 6}, {NULL}};
const Segment SGI207239[] = {{&S74647[112], 3}, {NULL}};
const Segment SGI207240[] = {{&S74647[77], 6}, {NULL}};
const Segment SGI207241[] = {{&S85381[34], 7}, {NULL}};
const Segment SGI207242[] = {{&S85381[27], 6}, {NULL}};
const Segment SGI207243[] = {{&S85381[99], 9}, {NULL}};
const Segment SGI207244[] = {{&S85381[72], 6}, {NULL}};
const Segment SGI207245[] = {{&S65515[34], 10}, {NULL}};
const Segment SGI207246[] = {{&S65515[27], 6}, {NULL}};
const Segment SGI207247[] = {{&S65515[96], 9}, {NULL}};
const Segment SGI207248[] = {{&S65515[69], 6}, {NULL}};
const Segment SGI207249[] = {{&S65516[34], 10}, {NULL}};
const Segment SGI207250[] = {{&S65516[27], 6}, {NULL}};
const Segment SGI207251[] = {{&S65516[96], 9}, {NULL}};
const Segment SGI207252[] = {{&S65516[69], 6}, {NULL}};
const Segment SGI207253[] = {{&S85389[34], 14}, {NULL}};
const Segment SGI207254[] = {{&S85389[27], 6}, {NULL}};
const Segment SGI207255[] = {{&S85389[101], 9}, {NULL}};
const Segment SGI207256[] = {{&S85389[74], 6}, {NULL}};
const Segment SGI207257[] = {{&S85389[142], 8}, {NULL}};
const Segment SGI207258[] = {{&S85390[34], 15}, {NULL}};
const Segment SGI207259[] = {{&S85390[27], 6}, {NULL}};
const Segment SGI207260[] = {{&S85390[102], 9}, {NULL}};
const Segment SGI207261[] = {{&S85390[75], 6}, {NULL}};
const Segment SGI207262[] = {{&S85390[143], 8}, {NULL}};
const Segment SGI207263[] = {{&S85393[44], 11}, {NULL}};
const Segment SGI207264[] = {{&S85393[44], 23}, {NULL}};
const Segment SGI207265[] = {{&S85393[30], 6}, {NULL}};
const Segment SGI207266[] = {{&S85393[31], 3}, {NULL}};
const Segment SGI207267[] = {{&S85393[81], 6}, {NULL}};
const Segment SGI207268[] = {{&S85398[44], 10}, {NULL}};
const Segment SGI207269[] = {{&S85398[44], 22}, {NULL}};
const Segment SGI207270[] = {{&S85398[30], 6}, {NULL}};
const Segment SGI207271[] = {{&S85398[80], 6}, {NULL}};
const Segment SGI207272[] = {{&S85398[107], 13}, {NULL}};
const Segment SGI207273[] = {{&S85399[34], 7}, {NULL}};
const Segment SGI207274[] = {{&S85399[27], 6}, {NULL}};
const Segment SGI207275[] = {{&S85399[94], 9}, {NULL}};
const Segment SGI207276[] = {{&S85399[67], 6}, {NULL}};
const Segment SGI207277[] = {{&S85400[45], 10}, {NULL}};
const Segment SGI207278[] = {{&S85400[30], 6}, {NULL}};
const Segment SGI207279[] = {{&S85400[31], 3}, {NULL}};
const Segment SGI207280[] = {{&S85400[81], 5}, {NULL}};
const Segment SGI207281[] = {{&S85400[139], 13}, {NULL}};
const Segment SGI207282[] = {{&S85401[44], 10}, {NULL}};
const Segment SGI207283[] = {{&S85401[44], 22}, {NULL}};
const Segment SGI207284[] = {{&S85401[30], 6}, {NULL}};
const Segment SGI207285[] = {{&S85401[115], 3}, {NULL}};
const Segment SGI207286[] = {{&S85401[80], 6}, {NULL}};
const Segment SGI207287[] = {{&S85401[135], 8}, {NULL}};
const Segment SGI207288[] = {{&S85401[144], 13}, {NULL}};
const Segment SGI207289[] = {{&S85408[34], 19}, {NULL}};
const Segment SGI207290[] = {{&S85408[27], 6}, {NULL}};
const Segment SGI207291[] = {{&S85408[106], 9}, {NULL}};
const Segment SGI207292[] = {{&S85408[79], 6}, {NULL}};
const Segment SGI207293[] = {{&S85409[34], 18}, {NULL}};
const Segment SGI207294[] = {{&S85409[27], 6}, {NULL}};
const Segment SGI207295[] = {{&S85409[105], 9}, {NULL}};
const Segment SGI207296[] = {{&S85409[78], 6}, {NULL}};
const Segment SGI207297[] = {{&S85410[34], 9}, {NULL}};
const Segment SGI207298[] = {{&S85410[27], 6}, {NULL}};
const Segment SGI207299[] = {{&S85410[96], 9}, {NULL}};
const Segment SGI207300[] = {{&S85410[69], 6}, {NULL}};
const Segment SGI207301[] = {{&S85411[34], 11}, {NULL}};
const Segment SGI207302[] = {{&S85411[27], 6}, {NULL}};
const Segment SGI207303[] = {{&S85411[98], 9}, {NULL}};
const Segment SGI207304[] = {{&S85411[71], 6}, {NULL}};
const Segment SGI207305[] = {{&S85414[34], 10}, {NULL}};
const Segment SGI207306[] = {{&S85414[27], 6}, {NULL}};
const Segment SGI207307[] = {{&S85414[97], 9}, {NULL}};
const Segment SGI207308[] = {{&S85414[70], 6}, {NULL}};
const Segment SGI207309[] = {{&S75561[44], 8}, {NULL}};
const Segment SGI207310[] = {{&S75561[44], 20}, {NULL}};
const Segment SGI207311[] = {{&S75561[30], 6}, {NULL}};
const Segment SGI207312[] = {{&S85415[34], 7}, {NULL}};
const Segment SGI207313[] = {{&S75561[113], 3}, {NULL}};
const Segment SGI207314[] = {{&S80149[44], 7}, {NULL}};
const Segment SGI207315[] = {{&S85415[27], 6}, {NULL}};
const Segment SGI207316[] = {{&S80149[44], 19}, {NULL}};
const Segment SGI207317[] = {{&S75561[78], 6}, {NULL}};
const Segment SGI207318[] = {{&S80149[30], 6}, {NULL}};
const Segment SGI207319[] = {{&S85415[94], 9}, {NULL}};
const Segment SGI207320[] = {{&S85415[67], 6}, {NULL}};
const Segment SGI207321[] = {{&S80149[128], 3}, {NULL}};
const Segment SGI207322[] = {{&S80149[93], 6}, {NULL}};
const Segment SGI207323[] = {{&S80149[65], 8}, {NULL}};
const Segment SGI207324[] = {{&S80157[44], 7}, {NULL}};
const Segment SGI207325[] = {{&S80157[44], 19}, {NULL}};
const Segment SGI207326[] = {{&S80157[30], 6}, {NULL}};
const Segment SGI207327[] = {{&S80157[112], 3}, {NULL}};
const Segment SGI207328[] = {{&S80157[77], 6}, {NULL}};
const Segment SGI207329[] = {{&S80165[44], 10}, {NULL}};
const Segment SGI207330[] = {{&S80165[44], 22}, {NULL}};
const Segment SGI207331[] = {{&S80165[30], 6}, {NULL}};
const Segment SGI207332[] = {{&S80165[115], 3}, {NULL}};
const Segment SGI207333[] = {{&S80165[80], 6}, {NULL}};
const Segment SGI207334[] = {{&S80165[143], 3}, {NULL}};
const Segment SGI207335[] = {{&S80171[44], 8}, {NULL}};
const Segment SGI207336[] = {{&S80171[44], 20}, {NULL}};
const Segment SGI207337[] = {{&S80171[30], 6}, {NULL}};
const Segment SGI207338[] = {{&S80171[113], 3}, {NULL}};
const Segment SGI207339[] = {{&S80171[78], 6}, {NULL}};
const Segment SGI207340[] = {{&S75598[34], 18}, {NULL}};
const Segment SGI207341[] = {{&S75598[27], 6}, {NULL}};
const Segment SGI207342[] = {{&S75598[105], 9}, {NULL}};
const Segment SGI207343[] = {{&S75598[78], 6}, {NULL}};
const Segment SGI207344[] = {{&S74751[44], 6}, {NULL}};
const Segment SGI207345[] = {{&S74751[44], 18}, {NULL}};
const Segment SGI207346[] = {{&S74751[30], 6}, {NULL}};
const Segment SGI207347[] = {{&S74751[111], 3}, {NULL}};
const Segment SGI207348[] = {{&S74751[76], 6}, {NULL}};
const Segment SGI207349[] = {{&S80182[44], 10}, {NULL}};
const Segment SGI207350[] = {{&S80182[44], 22}, {NULL}};
const Segment SGI207351[] = {{&S80182[30], 6}, {NULL}};
const Segment SGI207352[] = {{&S75602[44], 10}, {NULL}};
const Segment SGI207353[] = {{&S75602[44], 22}, {NULL}};
const Segment SGI207354[] = {{&S80182[80], 6}, {NULL}};
const Segment SGI207355[] = {{&S75602[30], 6}, {NULL}};
const Segment SGI207356[] = {{&S80182[107], 13}, {NULL}};
const Segment SGI207357[] = {{&S85460[34], 8}, {NULL}};
const Segment SGI207358[] = {{&S74752[38], 6}, {NULL}};
const Segment SGI207359[] = {{&S75602[113], 3}, {NULL}};
const Segment SGI207360[] = {{&S74752[31], 6}, {NULL}};
const Segment SGI207361[] = {{&S85460[27], 6}, {NULL}};
const Segment SGI207362[] = {{&S75602[80], 5}, {NULL}};
const Segment SGI207363[] = {{&S75602[117], 14}, {NULL}};
const Segment SGI207364[] = {{&S85460[95], 9}, {NULL}};
const Segment SGI207365[] = {{&S85460[68], 6}, {NULL}};
const Segment SGI207366[] = {{&S80183[34], 25}, {NULL}};
const Segment SGI207367[] = {{&S80183[27], 6}, {NULL}};
const Segment SGI207368[] = {{&S80183[112], 9}, {NULL}};
const Segment SGI207369[] = {{&S80183[85], 6}, {NULL}};
const Segment SGI207370[] = {{&S75604[44], 10}, {NULL}};
const Segment SGI207371[] = {{&S75604[44], 22}, {NULL}};
const Segment SGI207372[] = {{&S75604[30], 6}, {NULL}};
const Segment SGI207373[] = {{&S75604[113], 3}, {NULL}};
const Segment SGI207374[] = {{&S75604[80], 5}, {NULL}};
const Segment SGI207375[] = {{&S75604[117], 14}, {NULL}};
const Segment SGI207376[] = {{&S65608[34], 10}, {NULL}};
const Segment SGI207377[] = {{&S65608[27], 6}, {NULL}};
const Segment SGI207378[] = {{&S65608[96], 9}, {NULL}};
const Segment SGI207379[] = {{&S65608[69], 6}, {NULL}};
const Segment SGI207380[] = {{&S65609[34], 18}, {NULL}};
const Segment SGI207381[] = {{&S65609[27], 6}, {NULL}};
const Segment SGI207382[] = {{&S65609[105], 9}, {NULL}};
const Segment SGI207383[] = {{&S65609[78], 6}, {NULL}};
const Segment SGI207384[] = {{&S75612[44], 10}, {NULL}};
const Segment SGI207385[] = {{&S75612[44], 22}, {NULL}};
const Segment SGI207386[] = {{&S75612[30], 6}, {NULL}};
const Segment SGI207387[] = {{&S75612[80], 6}, {NULL}};
const Segment SGI207388[] = {{&S75612[107], 13}, {NULL}};
const Segment SGI207389[] = {{&S85475[44], 7}, {NULL}};
const Segment SGI207390[] = {{&S85475[44], 19}, {NULL}};
const Segment SGI207391[] = {{&S85475[30], 6}, {NULL}};
const Segment SGI207392[] = {{&S85475[31], 3}, {NULL}};
const Segment SGI207393[] = {{&S85475[77], 6}, {NULL}};
const Segment SGI207394[] = {{&S85478[31], 8}, {NULL}};
const Segment SGI207395[] = {{&S85478[55], 3}, {NULL}};
const Segment SGI207396[] = {{&S85480[44], 10}, {NULL}};
const Segment SGI207397[] = {{&S85480[44], 22}, {NULL}};
const Segment SGI207398[] = {{&S85480[30], 6}, {NULL}};
const Segment SGI207399[] = {{&S85480[31], 3}, {NULL}};
const Segment SGI207400[] = {{&S85480[80], 6}, {NULL}};
const Segment SGI207401[] = {{&S85480[133], 11}, {NULL}};
const Segment SGI207402[] = {{&S85484[34], 10}, {NULL}};
const Segment SGI207403[] = {{&S85484[27], 6}, {NULL}};
const Segment SGI207404[] = {{&S85484[97], 9}, {NULL}};
const Segment SGI207405[] = {{&S85484[70], 6}, {NULL}};
const Segment SGI207406[] = {{&S85486[34], 9}, {NULL}};
const Segment SGI207407[] = {{&S85486[27], 6}, {NULL}};
const Segment SGI207408[] = {{&S85486[93], 9}, {NULL}};
const Segment SGI207409[] = {{&S85486[66], 6}, {NULL}};
const Segment SGI207410[] = {{&S74785[34], 6}, {NULL}};
const Segment SGI207411[] = {{&S74785[27], 6}, {NULL}};
const Segment SGI207412[] = {{&S74785[93], 9}, {NULL}};
const Segment SGI207413[] = {{&S74785[66], 6}, {NULL}};
const Segment SGI207414[] = {{&S75635[34], 13}, {NULL}};
const Segment SGI207415[] = {{&S75635[27], 6}, {NULL}};
const Segment SGI207416[] = {{&S75635[100], 9}, {NULL}};
const Segment SGI207417[] = {{&S75635[73], 6}, {NULL}};
const Segment SGI207418[] = {{&S85493[34], 9}, {NULL}};
const Segment SGI207419[] = {{&S85493[27], 6}, {NULL}};
const Segment SGI207420[] = {{&S85493[96], 9}, {NULL}};
const Segment SGI207421[] = {{&S85493[69], 6}, {NULL}};
const Segment SGI207422[] = {{&S85495[44], 10}, {NULL}};
const Segment SGI207423[] = {{&S85495[44], 22}, {NULL}};
const Segment SGI207424[] = {{&S85495[30], 6}, {NULL}};
const Segment SGI207425[] = {{&S85495[115], 3}, {NULL}};
const Segment SGI207426[] = {{&S85495[80], 6}, {NULL}};
const Segment SGI207427[] = {{&S85495[135], 8}, {NULL}};
const Segment SGI207428[] = {{&S85495[144], 13}, {NULL}};
const Segment SGI207429[] = {{&S85496[45], 10}, {NULL}};
const Segment SGI207430[] = {{&S85496[30], 6}, {NULL}};
const Segment SGI207431[] = {{&S85496[31], 3}, {NULL}};
const Segment SGI207432[] = {{&S85496[81], 5}, {NULL}};
const Segment SGI207433[] = {{&S85496[139], 13}, {NULL}};
const Segment SGI207434[] = {{&S85501[34], 6}, {NULL}};
const Segment SGI207435[] = {{&S85501[27], 6}, {NULL}};
const Segment SGI207436[] = {{&S85501[93], 9}, {NULL}};
const Segment SGI207437[] = {{&S85501[66], 6}, {NULL}};
const Segment SGI207438[] = {{&S80237[44], 10}, {NULL}};
const Segment SGI207439[] = {{&S80237[44], 22}, {NULL}};
const Segment SGI207440[] = {{&S80237[30], 6}, {NULL}};
const Segment SGI207441[] = {{&S80237[131], 3}, {NULL}};
const Segment SGI207442[] = {{&S80237[96], 6}, {NULL}};
const Segment SGI207443[] = {{&S80237[68], 8}, {NULL}};
const Segment SGI207444[] = {{&S80238[44], 10}, {NULL}};
const Segment SGI207445[] = {{&S80238[44], 22}, {NULL}};
const Segment SGI207446[] = {{&S80238[30], 6}, {NULL}};
const Segment SGI207447[] = {{&S80238[131], 3}, {NULL}};
const Segment SGI207448[] = {{&S80238[96], 6}, {NULL}};
const Segment SGI207449[] = {{&S80238[68], 8}, {NULL}};
const Segment SGI207450[] = {{&S80239[44], 10}, {NULL}};
const Segment SGI207451[] = {{&S80239[44], 22}, {NULL}};
const Segment SGI207452[] = {{&S80239[30], 6}, {NULL}};
const Segment SGI207453[] = {{&S85509[34], 17}, {NULL}};
const Segment SGI207454[] = {{&S85509[27], 6}, {NULL}};
const Segment SGI207455[] = {{&S80239[131], 3}, {NULL}};
const Segment SGI207456[] = {{&S80239[96], 6}, {NULL}};
const Segment SGI207457[] = {{&S85509[104], 9}, {NULL}};
const Segment SGI207458[] = {{&S80239[68], 8}, {NULL}};
const Segment SGI207459[] = {{&S85509[77], 6}, {NULL}};
const Segment SGI207460[] = {{&S85510[44], 10}, {NULL}};
const Segment SGI207461[] = {{&S85510[44], 22}, {NULL}};
const Segment SGI207462[] = {{&S85510[30], 6}, {NULL}};
const Segment SGI207463[] = {{&S85510[31], 3}, {NULL}};
const Segment SGI207464[] = {{&S85510[80], 5}, {NULL}};
const Segment SGI207465[] = {{&S85510[117], 14}, {NULL}};
const Segment SGI207466[] = {{&S80244[44], 16}, {NULL}};
const Segment SGI207467[] = {{&S80244[44], 28}, {NULL}};
const Segment SGI207468[] = {{&S80244[30], 6}, {NULL}};
const Segment SGI207469[] = {{&S80244[121], 3}, {NULL}};
const Segment SGI207470[] = {{&S80244[86], 6}, {NULL}};
const Segment SGI207471[] = {{&S80245[38], 16}, {NULL}};
const Segment SGI207472[] = {{&S80245[31], 6}, {NULL}};
const Segment SGI207473[] = {{&S80248[44], 22}, {NULL}};
const Segment SGI207474[] = {{&S80248[44], 34}, {NULL}};
const Segment SGI207475[] = {{&S80248[30], 6}, {NULL}};
const Segment SGI207476[] = {{&S80248[127], 3}, {NULL}};
const Segment SGI207477[] = {{&S80248[92], 6}, {NULL}};
const Segment SGI207478[] = {{&S85537[34], 13}, {NULL}};
const Segment SGI207479[] = {{&S85537[27], 6}, {NULL}};
const Segment SGI207480[] = {{&S85537[100], 9}, {NULL}};
const Segment SGI207481[] = {{&S85537[73], 6}, {NULL}};
const Segment SGI207482[] = {{&S65701[44], 10}, {NULL}};
const Segment SGI207483[] = {{&S65701[44], 21}, {NULL}};
const Segment SGI207484[] = {{&S74851[34], 17}, {NULL}};
const Segment SGI207485[] = {{&S65701[30], 6}, {NULL}};
const Segment SGI207486[] = {{&S74851[27], 6}, {NULL}};
const Segment SGI207487[] = {{&S65701[114], 3}, {NULL}};
const Segment SGI207488[] = {{&S65701[79], 6}, {NULL}};
const Segment SGI207489[] = {{&S74851[104], 9}, {NULL}};
const Segment SGI207490[] = {{&S74851[77], 6}, {NULL}};
const Segment SGI207491[] = {{&S65702[44], 10}, {NULL}};
const Segment SGI207492[] = {{&S65702[44], 22}, {NULL}};
const Segment SGI207493[] = {{&S65702[30], 6}, {NULL}};
const Segment SGI207494[] = {{&S65702[115], 3}, {NULL}};
const Segment SGI207495[] = {{&S65702[80], 6}, {NULL}};
const Segment SGI207496[] = {{&S65703[38], 10}, {NULL}};
const Segment SGI207497[] = {{&S65703[31], 6}, {NULL}};
const Segment SGI207498[] = {{&S80289[44], 25}, {NULL}};
const Segment SGI207499[] = {{&S80289[44], 37}, {NULL}};
const Segment SGI207500[] = {{&S80289[30], 6}, {NULL}};
const Segment SGI207501[] = {{&S80289[51], 3}, {NULL}};
const Segment SGI207502[] = {{&S80289[95], 6}, {NULL}};
const Segment SGI207503[] = {{&S80290[71], 21}, {NULL}};
const Segment SGI207504[] = {{&S80290[71], 33}, {NULL}};
const Segment SGI207505[] = {{&S80290[30], 33}, {NULL}};
const Segment SGI207506[] = {{&S80290[153], 3}, {NULL}};
const Segment SGI207507[] = {{&S80290[118], 6}, {NULL}};
const Segment SGI207508[] = {{&S74875[34], 7}, {NULL}};
const Segment SGI207509[] = {{&S74875[27], 6}, {NULL}};
const Segment SGI207510[] = {{&S74875[94], 9}, {NULL}};
const Segment SGI207511[] = {{&S74875[67], 6}, {NULL}};
const Segment SGI207512[] = {{&S80291[44], 25}, {NULL}};
const Segment SGI207513[] = {{&S80291[44], 37}, {NULL}};
const Segment SGI207514[] = {{&S80291[30], 6}, {NULL}};
const Segment SGI207515[] = {{&S80291[130], 3}, {NULL}};
const Segment SGI207516[] = {{&S80291[95], 6}, {NULL}};
const Segment SGI207517[] = {{&S74877[34], 11}, {NULL}};
const Segment SGI207518[] = {{&S74877[27], 6}, {NULL}};
const Segment SGI207519[] = {{&S74877[98], 9}, {NULL}};
const Segment SGI207520[] = {{&S74877[71], 6}, {NULL}};
const Segment SGI207521[] = {{&S80292[44], 21}, {NULL}};
const Segment SGI207522[] = {{&S80292[44], 33}, {NULL}};
const Segment SGI207523[] = {{&S80292[30], 6}, {NULL}};
const Segment SGI207524[] = {{&S80292[126], 3}, {NULL}};
const Segment SGI207525[] = {{&S80292[91], 6}, {NULL}};
const Segment SGI207526[] = {{&S74879[34], 5}, {NULL}};
const Segment SGI207527[] = {{&S74879[27], 6}, {NULL}};
const Segment SGI207528[] = {{&S74879[92], 9}, {NULL}};
const Segment SGI207529[] = {{&S74879[65], 6}, {NULL}};
const Segment SGI207530[] = {{&S80302[34], 16}, {NULL}};
const Segment SGI207531[] = {{&S80302[27], 6}, {NULL}};
const Segment SGI207532[] = {{&S80302[112], 9}, {NULL}};
const Segment SGI207533[] = {{&S80302[85], 6}, {NULL}};
const Segment SGI207534[] = {{&S85582[44], 14}, {NULL}};
const Segment SGI207535[] = {{&S85582[44], 39}, {NULL}};
const Segment SGI207536[] = {{&S85582[30], 6}, {NULL}};
const Segment SGI207537[] = {{&S85582[31], 3}, {NULL}};
const Segment SGI207538[] = {{&S85582[97], 6}, {NULL}};
const Segment SGI207539[] = {{&S85582[157], 11}, {NULL}};
const Segment SGI207540[] = {{&S80303[34], 18}, {NULL}};
const Segment SGI207541[] = {{&S80303[27], 6}, {NULL}};
const Segment SGI207542[] = {{&S80303[114], 9}, {NULL}};
const Segment SGI207543[] = {{&S80303[87], 6}, {NULL}};
const Segment SGI207544[] = {{&S80304[34], 7}, {NULL}};
const Segment SGI207545[] = {{&S80304[27], 6}, {NULL}};
const Segment SGI207546[] = {{&S80304[94], 9}, {NULL}};
const Segment SGI207547[] = {{&S80304[67], 6}, {NULL}};
const Segment SGI207548[] = {{&S85587[45], 7}, {NULL}};
const Segment SGI207549[] = {{&S85587[30], 6}, {NULL}};
const Segment SGI207550[] = {{&S85587[31], 3}, {NULL}};
const Segment SGI207551[] = {{&S85587[83], 5}, {NULL}};
const Segment SGI207552[] = {{&S85587[141], 13}, {NULL}};
const Segment SGI207553[] = {{&S85588[34], 9}, {NULL}};
const Segment SGI207554[] = {{&S85588[27], 6}, {NULL}};
const Segment SGI207555[] = {{&S85588[96], 9}, {NULL}};
const Segment SGI207556[] = {{&S85588[69], 6}, {NULL}};
const Segment SGI207557[] = {{&S85589[34], 8}, {NULL}};
const Segment SGI207558[] = {{&S85589[27], 6}, {NULL}};
const Segment SGI207559[] = {{&S85589[95], 9}, {NULL}};
const Segment SGI207560[] = {{&S85589[68], 6}, {NULL}};
const Segment SGI207561[] = {{&S85592[44], 11}, {NULL}};
const Segment SGI207562[] = {{&S85592[44], 23}, {NULL}};
const Segment SGI207563[] = {{&S85592[30], 6}, {NULL}};
const Segment SGI207564[] = {{&S85592[31], 3}, {NULL}};
const Segment SGI207565[] = {{&S85592[81], 5}, {NULL}};
const Segment SGI207566[] = {{&S85592[118], 14}, {NULL}};
const Segment SGI207567[] = {{&S80317[34], 7}, {NULL}};
const Segment SGI207568[] = {{&S80317[27], 6}, {NULL}};
const Segment SGI207569[] = {{&S85594[44], 10}, {NULL}};
const Segment SGI207570[] = {{&S80317[94], 9}, {NULL}};
const Segment SGI207571[] = {{&S80317[67], 6}, {NULL}};
const Segment SGI207572[] = {{&S85594[44], 22}, {NULL}};
const Segment SGI207573[] = {{&S85594[30], 6}, {NULL}};
const Segment SGI207574[] = {{&S85594[31], 3}, {NULL}};
const Segment SGI207575[] = {{&S85594[80], 5}, {NULL}};
const Segment SGI207576[] = {{&S85594[117], 14}, {NULL}};
const Segment SGI207577[] = {{&S85597[34], 13}, {NULL}};
const Segment SGI207578[] = {{&S85597[27], 6}, {NULL}};
const Segment SGI207579[] = {{&S85597[100], 9}, {NULL}};
const Segment SGI207580[] = {{&S85597[73], 6}, {NULL}};
const Segment SGI207581[] = {{&S80322[52], 10}, {NULL}};
const Segment SGI207582[] = {{&S80322[52], 22}, {NULL}};
const Segment SGI207583[] = {{&S80322[38], 6}, {NULL}};
const Segment SGI207584[] = {{&S80322[88], 5}, {NULL}};
const Segment SGI207585[] = {{&S80322[0], 14}, {NULL}};
const Segment SGI207586[] = {{&S85599[34], 5}, {NULL}};
const Segment SGI207587[] = {{&S85599[27], 6}, {NULL}};
const Segment SGI207588[] = {{&S85599[92], 9}, {NULL}};
const Segment SGI207589[] = {{&S85599[65], 6}, {NULL}};
const Segment SGI207590[] = {{&S85600[34], 13}, {NULL}};
const Segment SGI207591[] = {{&S85600[27], 6}, {NULL}};
const Segment SGI207592[] = {{&S85600[100], 9}, {NULL}};
const Segment SGI207593[] = {{&S85600[73], 6}, {NULL}};
const Segment SGI207594[] = {{&S85601[34], 7}, {NULL}};
const Segment SGI207595[] = {{&S85601[27], 6}, {NULL}};
const Segment SGI207596[] = {{&S85601[98], 9}, {NULL}};
const Segment SGI207597[] = {{&S85601[71], 6}, {NULL}};
const Segment SGI207598[] = {{&S85602[59], 4}, {NULL}};
const Segment SGI207599[] = {{&S85602[59], 16}, {NULL}};
const Segment SGI207600[] = {{&S85602[45], 6}, {NULL}};
const Segment SGI207601[] = {{&S85602[46], 3}, {NULL}};
const Segment SGI207602[] = {{&S85602[89], 6}, {NULL}};
const Segment SGI207603[] = {{&S85602[0], 14}, {NULL}};
const Segment SGI207604[] = {{&S85606[34], 4}, {NULL}};
const Segment SGI207605[] = {{&S85606[27], 6}, {NULL}};
const Segment SGI207606[] = {{&S85606[91], 9}, {NULL}};
const Segment SGI207607[] = {{&S85606[64], 6}, {NULL}};
const Segment SGI207608[] = {{&S85612[34], 10}, {NULL}};
const Segment SGI207609[] = {{&S85612[27], 6}, {NULL}};
const Segment SGI207610[] = {{&S85612[97], 9}, {NULL}};
const Segment SGI207611[] = {{&S85612[70], 6}, {NULL}};
const Segment SGI207612[] = {{&S85613[34], 15}, {NULL}};
const Segment SGI207613[] = {{&S85613[27], 6}, {NULL}};
const Segment SGI207614[] = {{&S85613[117], 9}, {NULL}};
const Segment SGI207615[] = {{&S85613[90], 6}, {NULL}};
const Segment SGI207616[] = {{&S85615[44], 5}, {NULL}};
const Segment SGI207617[] = {{&S85615[44], 17}, {NULL}};
const Segment SGI207618[] = {{&S85615[30], 6}, {NULL}};
const Segment SGI207619[] = {{&S85615[75], 6}, {NULL}};
const Segment SGI207620[] = {{&S85615[102], 13}, {NULL}};
const Segment SGI207621[] = {{&S85620[44], 7}, {NULL}};
const Segment SGI207622[] = {{&S85620[44], 19}, {NULL}};
const Segment SGI207623[] = {{&S85620[30], 6}, {NULL}};
const Segment SGI207624[] = {{&S85620[31], 3}, {NULL}};
const Segment SGI207625[] = {{&S85620[77], 5}, {NULL}};
const Segment SGI207626[] = {{&S85620[114], 14}, {NULL}};
const Segment SGI207627[] = {{&S85621[44], 8}, {NULL}};
const Segment SGI207628[] = {{&S85621[44], 20}, {NULL}};
const Segment SGI207629[] = {{&S85621[30], 6}, {NULL}};
const Segment SGI207630[] = {{&S85621[31], 3}, {NULL}};
const Segment SGI207631[] = {{&S85621[78], 5}, {NULL}};
const Segment SGI207632[] = {{&S85621[115], 14}, {NULL}};
const Segment SGI207633[] = {{&S74969[34], 29}, {NULL}};
const Segment SGI207634[] = {{&S74969[27], 6}, {NULL}};
const Segment SGI207635[] = {{&S74969[116], 9}, {NULL}};
const Segment SGI207636[] = {{&S74969[89], 6}, {NULL}};
const Segment SGI207637[] = {{&S85630[34], 13}, {NULL}};
const Segment SGI207638[] = {{&S85630[27], 6}, {NULL}};
const Segment SGI207639[] = {{&S85630[113], 9}, {NULL}};
const Segment SGI207640[] = {{&S85630[86], 6}, {NULL}};
const Segment SGI207641[] = {{&S85633[34], 7}, {NULL}};
const Segment SGI207642[] = {{&S85633[27], 6}, {NULL}};
const Segment SGI207643[] = {{&S85633[99], 9}, {NULL}};
const Segment SGI207644[] = {{&S85633[72], 6}, {NULL}};
const Segment SGI207645[] = {{&S80373[44], 18}, {NULL}};
const Segment SGI207646[] = {{&S80373[44], 30}, {NULL}};
const Segment SGI207647[] = {{&S80373[30], 6}, {NULL}};
const Segment SGI207648[] = {{&S80373[123], 3}, {NULL}};
const Segment SGI207649[] = {{&S80373[88], 6}, {NULL}};
const Segment SGI207650[] = {{&S85635[44], 10}, {NULL}};
const Segment SGI207651[] = {{&S85635[44], 22}, {NULL}};
const Segment SGI207652[] = {{&S85635[30], 6}, {NULL}};
const Segment SGI207653[] = {{&S85635[80], 6}, {NULL}};
const Segment SGI207654[] = {{&S85635[107], 13}, {NULL}};
const Segment SGI207655[] = {{&S80378[44], 10}, {NULL}};
const Segment SGI207656[] = {{&S85639[44], 5}, {NULL}};
const Segment SGI207657[] = {{&S80378[44], 27}, {NULL}};
const Segment SGI207658[] = {{&S80378[30], 6}, {NULL}};
const Segment SGI207659[] = {{&S85639[44], 17}, {NULL}};
const Segment SGI207660[] = {{&S85639[30], 6}, {NULL}};
const Segment SGI207661[] = {{&S80378[120], 3}, {NULL}};
const Segment SGI207662[] = {{&S80378[85], 6}, {NULL}};
const Segment SGI207663[] = {{&S85639[75], 6}, {NULL}};
const Segment SGI207664[] = {{&S85639[102], 13}, {NULL}};
const Segment SGI207665[] = {{&S80383[34], 25}, {NULL}};
const Segment SGI207666[] = {{&S80383[27], 6}, {NULL}};
const Segment SGI207667[] = {{&S80383[112], 9}, {NULL}};
const Segment SGI207668[] = {{&S80383[85], 6}, {NULL}};
const Segment SGI207669[] = {{&S80384[34], 32}, {NULL}};
const Segment SGI207670[] = {{&S80384[27], 6}, {NULL}};
const Segment SGI207671[] = {{&S80384[119], 9}, {NULL}};
const Segment SGI207672[] = {{&S80384[92], 6}, {NULL}};
const Segment SGI207673[] = {{&S80386[34], 11}, {NULL}};
const Segment SGI207674[] = {{&S65847[44], 7}, {NULL}};
const Segment SGI207675[] = {{&S65847[44], 20}, {NULL}};
const Segment SGI207676[] = {{&S80386[27], 6}, {NULL}};
const Segment SGI207677[] = {{&S65847[30], 6}, {NULL}};
const Segment SGI207678[] = {{&S65847[113], 3}, {NULL}};
const Segment SGI207679[] = {{&S80386[98], 9}, {NULL}};
const Segment SGI207680[] = {{&S80386[71], 6}, {NULL}};
const Segment SGI207681[] = {{&S65847[78], 6}, {NULL}};
const Segment SGI207682[] = {{&S65848[44], 7}, {NULL}};
const Segment SGI207683[] = {{&S65848[44], 20}, {NULL}};
const Segment SGI207684[] = {{&S65848[30], 6}, {NULL}};
const Segment SGI207685[] = {{&S65848[113], 3}, {NULL}};
const Segment SGI207686[] = {{&S65848[78], 6}, {NULL}};
const Segment SGI207687[] = {{&S65849[38], 7}, {NULL}};
const Segment SGI207688[] = {{&S65849[31], 6}, {NULL}};
const Segment SGI207689[] = {{&S65850[44], 7}, {NULL}};
const Segment SGI207690[] = {{&S65850[44], 20}, {NULL}};
const Segment SGI207691[] = {{&S65850[30], 6}, {NULL}};
const Segment SGI207692[] = {{&S65850[113], 3}, {NULL}};
const Segment SGI207693[] = {{&S65850[78], 6}, {NULL}};
const Segment SGI207694[] = {{&S85665[44], 15}, {NULL}};
const Segment SGI207695[] = {{&S85665[44], 27}, {NULL}};
const Segment SGI207696[] = {{&S85665[30], 6}, {NULL}};
const Segment SGI207697[] = {{&S85665[31], 3}, {NULL}};
const Segment SGI207698[] = {{&S85665[85], 6}, {NULL}};
const Segment SGI207699[] = {{&S75636[34], 19}, {NULL}};
const Segment SGI207700[] = {{&S75636[27], 6}, {NULL}};
const Segment SGI207701[] = {{&S75636[106], 9}, {NULL}};
const Segment SGI207702[] = {{&S75636[79], 6}, {NULL}};
const Segment SGI207703[] = {{&S75640[44], 10}, {NULL}};
const Segment SGI207704[] = {{&S75640[44], 22}, {NULL}};
const Segment SGI207705[] = {{&S75640[30], 6}, {NULL}};
const Segment SGI207706[] = {{&S75640[80], 6}, {NULL}};
const Segment SGI207707[] = {{&S75640[107], 13}, {NULL}};
const Segment SGI207708[] = {{&S75643[52], 10}, {NULL}};
const Segment SGI207709[] = {{&S75643[52], 22}, {NULL}};
const Segment SGI207710[] = {{&S75643[38], 6}, {NULL}};
const Segment SGI207711[] = {{&S75643[88], 5}, {NULL}};
const Segment SGI207712[] = {{&S75643[11], 3}, {NULL}};
const Segment SGI207713[] = {{&S75039[34], 7}, {NULL}};
const Segment SGI207714[] = {{&S75039[27], 6}, {NULL}};
const Segment SGI207715[] = {{&S75039[94], 9}, {NULL}};
const Segment SGI207716[] = {{&S75039[67], 6}, {NULL}};
const Segment SGI207717[] = {{&S75645[52], 11}, {NULL}};
const Segment SGI207718[] = {{&S75645[52], 23}, {NULL}};
const Segment SGI207719[] = {{&S75645[38], 6}, {NULL}};
const Segment SGI207720[] = {{&S75645[89], 5}, {NULL}};
const Segment SGI207721[] = {{&S75645[11], 3}, {NULL}};
const Segment SGI207722[] = {{&S75040[34], 6}, {NULL}};
const Segment SGI207723[] = {{&S75040[27], 6}, {NULL}};
const Segment SGI207724[] = {{&S75040[93], 9}, {NULL}};
const Segment SGI207725[] = {{&S75040[66], 6}, {NULL}};
const Segment SGI207726[] = {{&S75646[52], 10}, {NULL}};
const Segment SGI207727[] = {{&S75646[52], 22}, {NULL}};
const Segment SGI207728[] = {{&S75646[38], 6}, {NULL}};
const Segment SGI207729[] = {{&S75646[88], 5}, {NULL}};
const Segment SGI207730[] = {{&S75646[11], 3}, {NULL}};
const Segment SGI207731[] = {{&S85689[44], 7}, {NULL}};
const Segment SGI207732[] = {{&S85689[44], 19}, {NULL}};
const Segment SGI207733[] = {{&S85689[30], 6}, {NULL}};
const Segment SGI207734[] = {{&S85689[31], 3}, {NULL}};
const Segment SGI207735[] = {{&S85689[77], 6}, {NULL}};
const Segment SGI207736[] = {{&S75647[52], 18}, {NULL}};
const Segment SGI207737[] = {{&S75647[52], 30}, {NULL}};
const Segment SGI207738[] = {{&S75647[38], 6}, {NULL}};
const Segment SGI207739[] = {{&S75647[96], 5}, {NULL}};
const Segment SGI207740[] = {{&S75647[11], 3}, {NULL}};
const Segment SGI207741[] = {{&S85690[57], 4}, {NULL}};
const Segment SGI207742[] = {{&S85690[57], 16}, {NULL}};
const Segment SGI207743[] = {{&S85690[43], 6}, {NULL}};
const Segment SGI207744[] = {{&S85690[44], 3}, {NULL}};
const Segment SGI207745[] = {{&S85690[87], 6}, {NULL}};
const Segment SGI207746[] = {{&S85690[0], 12}, {NULL}};
const Segment SGI207747[] = {{&S75648[34], 9}, {NULL}};
const Segment SGI207748[] = {{&S75648[27], 6}, {NULL}};
const Segment SGI207749[] = {{&S75648[96], 9}, {NULL}};
const Segment SGI207750[] = {{&S75648[69], 6}, {NULL}};
const Segment SGI207751[] = {{&S75649[34], 9}, {NULL}};
const Segment SGI207752[] = {{&S75649[27], 6}, {NULL}};
const Segment SGI207753[] = {{&S75649[96], 9}, {NULL}};
const Segment SGI207754[] = {{&S75649[69], 6}, {NULL}};
const Segment SGI207755[] = {{&S85699[44], 25}, {NULL}};
const Segment SGI207756[] = {{&S85699[44], 37}, {NULL}};
const Segment SGI207757[] = {{&S85699[30], 6}, {NULL}};
const Segment SGI207758[] = {{&S85699[31], 3}, {NULL}};
const Segment SGI207759[] = {{&S85699[95], 6}, {NULL}};
const Segment SGI207760[] = {{&S85708[44], 9}, {NULL}};
const Segment SGI207761[] = {{&S85708[44], 21}, {NULL}};
const Segment SGI207762[] = {{&S85708[30], 6}, {NULL}};
const Segment SGI207763[] = {{&S85708[31], 3}, {NULL}};
const Segment SGI207764[] = {{&S85708[79], 6}, {NULL}};
const Segment SGI207765[] = {{&S85712[34], 13}, {NULL}};
const Segment SGI207766[] = {{&S85712[27], 6}, {NULL}};
const Segment SGI207767[] = {{&S85712[100], 9}, {NULL}};
const Segment SGI207768[] = {{&S85712[73], 6}, {NULL}};
const Segment SGI207769[] = {{&S85713[34], 7}, {NULL}};
const Segment SGI207770[] = {{&S85713[27], 6}, {NULL}};
const Segment SGI207771[] = {{&S85713[94], 9}, {NULL}};
const Segment SGI207772[] = {{&S85713[67], 6}, {NULL}};
const Segment SGI207773[] = {{&S80417[44], 11}, {NULL}};
const Segment SGI207774[] = {{&S80417[44], 23}, {NULL}};
const Segment SGI207775[] = {{&S80417[30], 6}, {NULL}};
const Segment SGI207776[] = {{&S80417[132], 3}, {NULL}};
const Segment SGI207777[] = {{&S80417[97], 6}, {NULL}};
const Segment SGI207778[] = {{&S80417[69], 8}, {NULL}};
const Segment SGI207779[] = {{&S75074[34], 15}, {NULL}};
const Segment SGI207780[] = {{&S75074[27], 6}, {NULL}};
const Segment SGI207781[] = {{&S75074[102], 9}, {NULL}};
const Segment SGI207782[] = {{&S75074[75], 6}, {NULL}};
const Segment SGI207783[] = {{&S75075[34], 6}, {NULL}};
const Segment SGI207784[] = {{&S75075[27], 6}, {NULL}};
const Segment SGI207785[] = {{&S75075[93], 9}, {NULL}};
const Segment SGI207786[] = {{&S75075[66], 6}, {NULL}};
const Segment SGI207787[] = {{&S85720[44], 11}, {NULL}};
const Segment SGI207788[] = {{&S85720[44], 23}, {NULL}};
const Segment SGI207789[] = {{&S85720[30], 6}, {NULL}};
const Segment SGI207790[] = {{&S85720[31], 3}, {NULL}};
const Segment SGI207791[] = {{&S85720[81], 6}, {NULL}};
const Segment SGI207792[] = {{&S85725[61], 10}, {NULL}};
const Segment SGI207793[] = {{&S85725[61], 22}, {NULL}};
const Segment SGI207794[] = {{&S85725[47], 6}, {NULL}};
const Segment SGI207795[] = {{&S85725[48], 3}, {NULL}};
const Segment SGI207796[] = {{&S85725[97], 6}, {NULL}};
const Segment SGI207797[] = {{&S85725[0], 14}, {NULL}};
const Segment SGI207798[] = {{&S80437[44], 20}, {NULL}};
const Segment SGI207799[] = {{&S80437[44], 32}, {NULL}};
const Segment SGI207800[] = {{&S85730[34], 6}, {NULL}};
const Segment SGI207801[] = {{&S80437[30], 6}, {NULL}};
const Segment SGI207802[] = {{&S85730[27], 6}, {NULL}};
const Segment SGI207803[] = {{&S80437[125], 3}, {NULL}};
const Segment SGI207804[] = {{&S85730[93], 9}, {NULL}};
const Segment SGI207805[] = {{&S80437[90], 6}, {NULL}};
const Segment SGI207806[] = {{&S85730[66], 6}, {NULL}};
const Segment SGI207807[] = {{&S85736[45], 13}, {NULL}};
const Segment SGI207808[] = {{&S85736[30], 6}, {NULL}};
const Segment SGI207809[] = {{&S85736[31], 3}, {NULL}};
const Segment SGI207810[] = {{&S85736[84], 5}, {NULL}};
const Segment SGI207811[] = {{&S85736[142], 13}, {NULL}};
const Segment SGI207812[] = {{&S80444[44], 14}, {NULL}};
const Segment SGI207813[] = {{&S80444[44], 33}, {NULL}};
const Segment SGI207814[] = {{&S80444[30], 6}, {NULL}};
const Segment SGI207815[] = {{&S80444[126], 3}, {NULL}};
const Segment SGI207816[] = {{&S80444[91], 6}, {NULL}};
const Segment SGI207817[] = {{&S80445[44], 16}, {NULL}};
const Segment SGI207818[] = {{&S80445[44], 35}, {NULL}};
const Segment SGI207819[] = {{&S80445[30], 6}, {NULL}};
const Segment SGI207820[] = {{&S80445[128], 3}, {NULL}};
const Segment SGI207821[] = {{&S80445[93], 6}, {NULL}};
const Segment SGI207822[] = {{&S80446[44], 16}, {NULL}};
const Segment SGI207823[] = {{&S80446[44], 35}, {NULL}};
const Segment SGI207824[] = {{&S80446[30], 6}, {NULL}};
const Segment SGI207825[] = {{&S80446[128], 3}, {NULL}};
const Segment SGI207826[] = {{&S80446[93], 6}, {NULL}};
const Segment SGI207827[] = {{&S85742[44], 15}, {NULL}};
const Segment SGI207828[] = {{&S85742[44], 27}, {NULL}};
const Segment SGI207829[] = {{&S85742[30], 6}, {NULL}};
const Segment SGI207830[] = {{&S75708[34], 13}, {NULL}};
const Segment SGI207831[] = {{&S75708[27], 6}, {NULL}};
const Segment SGI207832[] = {{&S85742[85], 6}, {NULL}};
const Segment SGI207833[] = {{&S85742[112], 13}, {NULL}};
const Segment SGI207834[] = {{&S75708[100], 9}, {NULL}};
const Segment SGI207835[] = {{&S75708[73], 6}, {NULL}};
const Segment SGI207836[] = {{&S85743[44], 14}, {NULL}};
const Segment SGI207837[] = {{&S85743[44], 26}, {NULL}};
const Segment SGI207838[] = {{&S85743[30], 6}, {NULL}};
const Segment SGI207839[] = {{&S65957[44], 12}, {NULL}};
const Segment SGI207840[] = {{&S65957[44], 23}, {NULL}};
const Segment SGI207841[] = {{&S85743[84], 6}, {NULL}};
const Segment SGI207842[] = {{&S65957[30], 6}, {NULL}};
const Segment SGI207843[] = {{&S85743[111], 13}, {NULL}};
const Segment SGI207844[] = {{&S65957[116], 3}, {NULL}};
const Segment SGI207845[] = {{&S80450[34], 13}, {NULL}};
const Segment SGI207846[] = {{&S65957[81], 6}, {NULL}};
const Segment SGI207847[] = {{&S80450[27], 6}, {NULL}};
const Segment SGI207848[] = {{&S80450[114], 9}, {NULL}};
const Segment SGI207849[] = {{&S80450[87], 6}, {NULL}};
const Segment SGI207850[] = {{&S75711[34], 9}, {NULL}};
const Segment SGI207851[] = {{&S65958[38], 12}, {NULL}};
const Segment SGI207852[] = {{&S65958[31], 6}, {NULL}};
const Segment SGI207853[] = {{&S75711[27], 6}, {NULL}};
const Segment SGI207854[] = {{&S85745[34], 7}, {NULL}};
const Segment SGI207855[] = {{&S85745[27], 6}, {NULL}};
const Segment SGI207856[] = {{&S75711[96], 9}, {NULL}};
const Segment SGI207857[] = {{&S75711[69], 6}, {NULL}};
const Segment SGI207858[] = {{&S80451[34], 12}, {NULL}};
const Segment SGI207859[] = {{&S75711[137], 8}, {NULL}};
const Segment SGI207860[] = {{&S85745[99], 9}, {NULL}};
const Segment SGI207861[] = {{&S80451[27], 6}, {NULL}};
const Segment SGI207862[] = {{&S85745[72], 6}, {NULL}};
const Segment SGI207863[] = {{&S80451[99], 9}, {NULL}};
const Segment SGI207864[] = {{&S80451[72], 6}, {NULL}};
const Segment SGI207865[] = {{&S75712[34], 9}, {NULL}};
const Segment SGI207866[] = {{&S75712[27], 6}, {NULL}};
const Segment SGI207867[] = {{&S75712[96], 9}, {NULL}};
const Segment SGI207868[] = {{&S75712[69], 6}, {NULL}};
const Segment SGI207869[] = {{&S75712[137], 8}, {NULL}};
const Segment SGI207870[] = {{&S75713[34], 9}, {NULL}};
const Segment SGI207871[] = {{&S75713[27], 6}, {NULL}};
const Segment SGI207872[] = {{&S85749[34], 9}, {NULL}};
const Segment SGI207873[] = {{&S75713[96], 9}, {NULL}};
const Segment SGI207874[] = {{&S75713[69], 6}, {NULL}};
const Segment SGI207875[] = {{&S85749[27], 6}, {NULL}};
const Segment SGI207876[] = {{&S75713[137], 8}, {NULL}};
const Segment SGI207877[] = {{&S85749[96], 9}, {NULL}};
const Segment SGI207878[] = {{&S85749[69], 6}, {NULL}};
const Segment SGI207879[] = {{&S85757[34], 11}, {NULL}};
const Segment SGI207880[] = {{&S85757[27], 6}, {NULL}};
const Segment SGI207881[] = {{&S85757[98], 9}, {NULL}};
const Segment SGI207882[] = {{&S85757[71], 6}, {NULL}};
const Segment SGI207883[] = {{&S85758[34], 10}, {NULL}};
const Segment SGI207884[] = {{&S85758[27], 6}, {NULL}};
const Segment SGI207885[] = {{&S85758[97], 9}, {NULL}};
const Segment SGI207886[] = {{&S85758[70], 6}, {NULL}};
const Segment SGI207887[] = {{&S80486[44], 6}, {NULL}};
const Segment SGI207888[] = {{&S80486[44], 18}, {NULL}};
const Segment SGI207889[] = {{&S80486[30], 6}, {NULL}};
const Segment SGI207890[] = {{&S80486[111], 3}, {NULL}};
const Segment SGI207891[] = {{&S80486[76], 6}, {NULL}};
const Segment SGI207892[] = {{&S80488[34], 11}, {NULL}};
const Segment SGI207893[] = {{&S80488[27], 6}, {NULL}};
const Segment SGI207894[] = {{&S80488[98], 9}, {NULL}};
const Segment SGI207895[] = {{&S80488[71], 6}, {NULL}};
const Segment SGI207896[] = {{&S80492[34], 6}, {NULL}};
const Segment SGI207897[] = {{&S80492[27], 6}, {NULL}};
const Segment SGI207898[] = {{&S80492[93], 9}, {NULL}};
const Segment SGI207899[] = {{&S80492[66], 6}, {NULL}};
const Segment SGI207900[] = {{&S80493[34], 9}, {NULL}};
const Segment SGI207901[] = {{&S80493[27], 6}, {NULL}};
const Segment SGI207902[] = {{&S66002[34], 10}, {NULL}};
const Segment SGI207903[] = {{&S66002[27], 6}, {NULL}};
const Segment SGI207904[] = {{&S66002[96], 9}, {NULL}};
const Segment SGI207905[] = {{&S66002[69], 6}, {NULL}};
const Segment SGI207906[] = {{&S66003[34], 10}, {NULL}};
const Segment SGI207907[] = {{&S66003[27], 6}, {NULL}};
const Segment SGI207908[] = {{&S66003[96], 9}, {NULL}};
const Segment SGI207909[] = {{&S66003[69], 6}, {NULL}};
const Segment SGI207910[] = {{&S75755[34], 10}, {NULL}};
const Segment SGI207911[] = {{&S75755[27], 6}, {NULL}};
const Segment SGI207912[] = {{&S75755[97], 9}, {NULL}};
const Segment SGI207913[] = {{&S75755[70], 6}, {NULL}};
const Segment SGI207914[] = {{&S75756[34], 10}, {NULL}};
const Segment SGI207915[] = {{&S75756[27], 6}, {NULL}};
const Segment SGI207916[] = {{&S75756[97], 9}, {NULL}};
const Segment SGI207917[] = {{&S75756[70], 6}, {NULL}};
const Segment SGI207918[] = {{&S75757[34], 10}, {NULL}};
const Segment SGI207919[] = {{&S75757[27], 6}, {NULL}};
const Segment SGI207920[] = {{&S75757[97], 9}, {NULL}};
const Segment SGI207921[] = {{&S75757[70], 6}, {NULL}};
const Segment SGI207922[] = {{&S75758[44], 10}, {NULL}};
const Segment SGI207923[] = {{&S75758[44], 22}, {NULL}};
const Segment SGI207924[] = {{&S75758[30], 6}, {NULL}};
const Segment SGI207925[] = {{&S75758[115], 3}, {NULL}};
const Segment SGI207926[] = {{&S75758[80], 6}, {NULL}};
const Segment SGI207927[] = {{&S85810[44], 19}, {NULL}};
const Segment SGI207928[] = {{&S85810[44], 31}, {NULL}};
const Segment SGI207929[] = {{&S85810[30], 6}, {NULL}};
const Segment SGI207930[] = {{&S85810[31], 3}, {NULL}};
const Segment SGI207931[] = {{&S75759[44], 11}, {NULL}};
const Segment SGI207932[] = {{&S85810[89], 6}, {NULL}};
const Segment SGI207933[] = {{&S75759[44], 23}, {NULL}};
const Segment SGI207934[] = {{&S75759[30], 6}, {NULL}};
const Segment SGI207935[] = {{&S75759[116], 3}, {NULL}};
const Segment SGI207936[] = {{&S75759[81], 6}, {NULL}};
const Segment SGI207937[] = {{&S85811[44], 13}, {NULL}};
const Segment SGI207938[] = {{&S85811[44], 25}, {NULL}};
const Segment SGI207939[] = {{&S85811[30], 6}, {NULL}};
const Segment SGI207940[] = {{&S85811[31], 3}, {NULL}};
const Segment SGI207941[] = {{&S85811[83], 6}, {NULL}};
const Segment SGI207942[] = {{&S85813[44], 13}, {NULL}};
const Segment SGI207943[] = {{&S85813[44], 25}, {NULL}};
const Segment SGI207944[] = {{&S85813[30], 6}, {NULL}};
const Segment SGI207945[] = {{&S85813[31], 3}, {NULL}};
const Segment SGI207946[] = {{&S85813[83], 6}, {NULL}};
const Segment SGI207947[] = {{&S75764[34], 10}, {NULL}};
const Segment SGI207948[] = {{&S75764[27], 6}, {NULL}};
const Segment SGI207949[] = {{&S75764[97], 9}, {NULL}};
const Segment SGI207950[] = {{&S75764[70], 6}, {NULL}};
const Segment SGI207951[] = {{&S85814[44], 10}, {NULL}};
const Segment SGI207952[] = {{&S85814[44], 22}, {NULL}};
const Segment SGI207953[] = {{&S85814[30], 6}, {NULL}};
const Segment SGI207954[] = {{&S85814[31], 3}, {NULL}};
const Segment SGI207955[] = {{&S85814[80], 6}, {NULL}};
const Segment SGI207956[] = {{&S85815[38], 10}, {NULL}};
const Segment SGI207957[] = {{&S85815[31], 6}, {NULL}};
const Segment SGI207958[] = {{&S85816[44], 10}, {NULL}};
const Segment SGI207959[] = {{&S85816[44], 22}, {NULL}};
const Segment SGI207960[] = {{&S85816[30], 6}, {NULL}};
const Segment SGI207961[] = {{&S85816[31], 3}, {NULL}};
const Segment SGI207962[] = {{&S85816[80], 6}, {NULL}};
const Segment SGI207963[] = {{&S85826[34], 15}, {NULL}};
const Segment SGI207964[] = {{&S66041[34], 8}, {NULL}};
const Segment SGI207965[] = {{&S85826[27], 6}, {NULL}};
const Segment SGI207966[] = {{&S66041[27], 6}, {NULL}};
const Segment SGI207967[] = {{&S85826[102], 9}, {NULL}};
const Segment SGI207968[] = {{&S66041[94], 9}, {NULL}};
const Segment SGI207969[] = {{&S85826[75], 6}, {NULL}};
const Segment SGI207970[] = {{&S66041[67], 6}, {NULL}};
const Segment SGI207971[] = {{&S85827[34], 16}, {NULL}};
const Segment SGI207972[] = {{&S85827[27], 6}, {NULL}};
const Segment SGI207973[] = {{&S85827[103], 9}, {NULL}};
const Segment SGI207974[] = {{&S85827[76], 6}, {NULL}};
const Segment SGI207975[] = {{&S75784[34], 11}, {NULL}};
const Segment SGI207976[] = {{&S75784[27], 6}, {NULL}};
const Segment SGI207977[] = {{&S75784[98], 9}, {NULL}};
const Segment SGI207978[] = {{&S75784[71], 6}, {NULL}};
const Segment SGI207979[] = {{&S85831[44], 10}, {NULL}};
const Segment SGI207980[] = {{&S85831[44], 22}, {NULL}};
const Segment SGI207981[] = {{&S85831[30], 6}, {NULL}};
const Segment SGI207982[] = {{&S85831[31], 3}, {NULL}};
const Segment SGI207983[] = {{&S85831[95], 6}, {NULL}};
const Segment SGI207984[] = {{&S85831[68], 8}, {NULL}};
const Segment SGI207985[] = {{&S85832[44], 12}, {NULL}};
const Segment SGI207986[] = {{&S85832[44], 24}, {NULL}};
const Segment SGI207987[] = {{&S85832[30], 6}, {NULL}};
const Segment SGI207988[] = {{&S85832[31], 3}, {NULL}};
const Segment SGI207989[] = {{&S85832[82], 6}, {NULL}};
const Segment SGI207990[] = {{&S66055[34], 20}, {NULL}};
const Segment SGI207991[] = {{&S66055[27], 6}, {NULL}};
const Segment SGI207992[] = {{&S66055[106], 9}, {NULL}};
const Segment SGI207993[] = {{&S66055[79], 6}, {NULL}};
const Segment SGI207994[] = {{&S66056[34], 21}, {NULL}};
const Segment SGI207995[] = {{&S66056[27], 6}, {NULL}};
const Segment SGI207996[] = {{&S66056[108], 9}, {NULL}};
const Segment SGI207997[] = {{&S66056[81], 6}, {NULL}};
const Segment SGI207998[] = {{&S66061[34], 10}, {NULL}};
const Segment SGI207999[] = {{&S66061[27], 6}, {NULL}};
const Segment SGI208000[] = {{&S66061[97], 9}, {NULL}};
const Segment SGI208001[] = {{&S66061[70], 6}, {NULL}};
const Segment SGI208002[] = {{&S75807[34], 10}, {NULL}};
const Segment SGI208003[] = {{&S75807[27], 6}, {NULL}};
const Segment SGI208004[] = {{&S80493[96], 9}, {NULL}};
const Segment SGI208005[] = {{&S80493[69], 6}, {NULL}};
const Segment SGI208006[] = {{&S75807[97], 9}, {NULL}};
const Segment SGI208007[] = {{&S75807[70], 6}, {NULL}};
const Segment SGI208008[] = {{&S85849[34], 8}, {NULL}};
const Segment SGI208009[] = {{&S85849[27], 6}, {NULL}};
const Segment SGI208010[] = {{&S85849[100], 9}, {NULL}};
const Segment SGI208011[] = {{&S85849[73], 6}, {NULL}};
const Segment SGI208012[] = {{&S80495[34], 9}, {NULL}};
const Segment SGI208013[] = {{&S80495[27], 6}, {NULL}};
const Segment SGI208014[] = {{&S80495[96], 9}, {NULL}};
const Segment SGI208015[] = {{&S80495[69], 6}, {NULL}};
const Segment SGI208016[] = {{&S85851[34], 7}, {NULL}};
const Segment SGI208017[] = {{&S85851[27], 6}, {NULL}};
const Segment SGI208018[] = {{&S85851[99], 9}, {NULL}};
const Segment SGI208019[] = {{&S85851[72], 6}, {NULL}};
const Segment SGI208020[] = {{&S85852[44], 12}, {NULL}};
const Segment SGI208021[] = {{&S85852[44], 24}, {NULL}};
const Segment SGI208022[] = {{&S85852[30], 6}, {NULL}};
const Segment SGI208023[] = {{&S85852[31], 3}, {NULL}};
const Segment SGI208024[] = {{&S85852[82], 6}, {NULL}};
const Segment SGI208025[] = {{&S75816[34], 6}, {NULL}};
const Segment SGI208026[] = {{&S75816[27], 6}, {NULL}};
const Segment SGI208027[] = {{&S75816[93], 9}, {NULL}};
const Segment SGI208028[] = {{&S75816[66], 6}, {NULL}};
const Segment SGI208029[] = {{&S75817[34], 11}, {NULL}};
const Segment SGI208030[] = {{&S75817[27], 6}, {NULL}};
const Segment SGI208031[] = {{&S75817[98], 9}, {NULL}};
const Segment SGI208032[] = {{&S75817[71], 6}, {NULL}};
const Segment SGI208033[] = {{&S85856[34], 7}, {NULL}};
const Segment SGI208034[] = {{&S85856[27], 6}, {NULL}};
const Segment SGI208035[] = {{&S85856[100], 9}, {NULL}};
const Segment SGI208036[] = {{&S85856[73], 6}, {NULL}};
const Segment SGI208037[] = {{&S85856[141], 8}, {NULL}};
const Segment SGI208038[] = {{&S85857[34], 7}, {NULL}};
const Segment SGI208039[] = {{&S85857[27], 6}, {NULL}};
const Segment SGI208040[] = {{&S75820[34], 6}, {NULL}};
const Segment SGI208041[] = {{&S75820[27], 6}, {NULL}};
const Segment SGI208042[] = {{&S85857[94], 9}, {NULL}};
const Segment SGI208043[] = {{&S85857[67], 6}, {NULL}};
const Segment SGI208044[] = {{&S75820[93], 9}, {NULL}};
const Segment SGI208045[] = {{&S75820[66], 6}, {NULL}};
const Segment SGI208046[] = {{&S85867[34], 41}, {NULL}};
const Segment SGI208047[] = {{&S85867[27], 6}, {NULL}};
const Segment SGI208048[] = {{&S85867[138], 9}, {NULL}};
const Segment SGI208049[] = {{&S85867[111], 6}, {NULL}};
const Segment SGI208050[] = {{&S85868[34], 20}, {NULL}};
const Segment SGI208051[] = {{&S85868[27], 6}, {NULL}};
const Segment SGI208052[] = {{&S85868[117], 9}, {NULL}};
const Segment SGI208053[] = {{&S85868[90], 6}, {NULL}};
const Segment SGI208054[] = {{&S80526[45], 10}, {NULL}};
const Segment SGI208055[] = {{&S80526[30], 6}, {NULL}};
const Segment SGI208056[] = {{&S80526[115], 3}, {NULL}};
const Segment SGI208057[] = {{&S80526[81], 5}, {NULL}};
const Segment SGI208058[] = {{&S80526[139], 13}, {NULL}};
const Segment SGI208059[] = {{&S80536[34], 7}, {NULL}};
const Segment SGI208060[] = {{&S80536[27], 6}, {NULL}};
const Segment SGI208061[] = {{&S80536[94], 9}, {NULL}};
const Segment SGI208062[] = {{&S80536[67], 6}, {NULL}};
const Segment SGI208063[] = {{&S80537[59], 10}, {NULL}};
const Segment SGI208064[] = {{&S80537[59], 22}, {NULL}};
const Segment SGI208065[] = {{&S80537[45], 6}, {NULL}};
const Segment SGI208066[] = {{&S80537[130], 3}, {NULL}};
const Segment SGI208067[] = {{&S80537[95], 6}, {NULL}};
const Segment SGI208068[] = {{&S80537[0], 14}, {NULL}};
const Segment SGI208069[] = {{&S80539[57], 10}, {NULL}};
const Segment SGI208070[] = {{&S80539[45], 6}, {NULL}};
const Segment SGI208071[] = {{&S80539[128], 3}, {NULL}};
const Segment SGI208072[] = {{&S80539[93], 6}, {NULL}};
const Segment SGI208073[] = {{&S80539[0], 14}, {NULL}};
const Segment SGI208074[] = {{&S85886[34], 9}, {NULL}};
const Segment SGI208075[] = {{&S85886[27], 6}, {NULL}};
const Segment SGI208076[] = {{&S85886[96], 9}, {NULL}};
const Segment SGI208077[] = {{&S85886[69], 6}, {NULL}};
const Segment SGI208078[] = {{&S80541[59], 11}, {NULL}};
const Segment SGI208079[] = {{&S80541[59], 23}, {NULL}};
const Segment SGI208080[] = {{&S80541[45], 6}, {NULL}};
const Segment SGI208081[] = {{&S80541[131], 3}, {NULL}};
const Segment SGI208082[] = {{&S80541[96], 6}, {NULL}};
const Segment SGI208083[] = {{&S80541[0], 14}, {NULL}};
const Segment SGI208084[] = {{&S85889[34], 13}, {NULL}};
const Segment SGI208085[] = {{&S85889[27], 6}, {NULL}};
const Segment SGI208086[] = {{&S85889[113], 9}, {NULL}};
const Segment SGI208087[] = {{&S85889[86], 6}, {NULL}};
const Segment SGI208088[] = {{&S80546[44], 16}, {NULL}};
const Segment SGI208089[] = {{&S80546[44], 28}, {NULL}};
const Segment SGI208090[] = {{&S80546[30], 6}, {NULL}};
const Segment SGI208091[] = {{&S80546[121], 3}, {NULL}};
const Segment SGI208092[] = {{&S80546[86], 6}, {NULL}};
const Segment SGI208093[] = {{&S80548[44], 10}, {NULL}};
const Segment SGI208094[] = {{&S80548[44], 22}, {NULL}};
const Segment SGI208095[] = {{&S80548[30], 6}, {NULL}};
const Segment SGI208096[] = {{&S80548[80], 6}, {NULL}};
const Segment SGI208097[] = {{&S80548[107], 13}, {NULL}};
const Segment SGI208098[] = {{&S80552[44], 6}, {NULL}};
const Segment SGI208099[] = {{&S80552[44], 18}, {NULL}};
const Segment SGI208100[] = {{&S80552[30], 6}, {NULL}};
const Segment SGI208101[] = {{&S80552[111], 3}, {NULL}};
const Segment SGI208102[] = {{&S80552[76], 6}, {NULL}};
const Segment SGI208103[] = {{&S75883[44], 7}, {NULL}};
const Segment SGI208104[] = {{&S75883[44], 19}, {NULL}};
const Segment SGI208105[] = {{&S75883[30], 6}, {NULL}};
const Segment SGI208106[] = {{&S75883[112], 3}, {NULL}};
const Segment SGI208107[] = {{&S75883[77], 6}, {NULL}};
const Segment SGI208108[] = {{&S75884[38], 7}, {NULL}};
const Segment SGI208109[] = {{&S75884[31], 6}, {NULL}};
const Segment SGI208110[] = {{&S75885[44], 12}, {NULL}};
const Segment SGI208111[] = {{&S75885[44], 24}, {NULL}};
const Segment SGI208112[] = {{&S75885[30], 6}, {NULL}};
const Segment SGI208113[] = {{&S75885[117], 3}, {NULL}};
const Segment SGI208114[] = {{&S75885[82], 6}, {NULL}};
const Segment SGI208115[] = {{&S75890[34], 10}, {NULL}};
const Segment SGI208116[] = {{&S80568[34], 7}, {NULL}};
const Segment SGI208117[] = {{&S75890[27], 6}, {NULL}};
const Segment SGI208118[] = {{&S80568[27], 6}, {NULL}};
const Segment SGI208119[] = {{&S75890[97], 9}, {NULL}};
const Segment SGI208120[] = {{&S75890[70], 6}, {NULL}};
const Segment SGI208121[] = {{&S80568[98], 9}, {NULL}};
const Segment SGI208122[] = {{&S80568[71], 6}, {NULL}};
const Segment SGI208123[] = {{&S85928[34], 7}, {NULL}};
const Segment SGI208124[] = {{&S85928[27], 6}, {NULL}};
const Segment SGI208125[] = {{&S85928[94], 9}, {NULL}};
const Segment SGI208126[] = {{&S85928[67], 6}, {NULL}};
const Segment SGI208127[] = {{&S85931[44], 11}, {NULL}};
const Segment SGI208128[] = {{&S85931[44], 23}, {NULL}};
const Segment SGI208129[] = {{&S85931[30], 6}, {NULL}};
const Segment SGI208130[] = {{&S85931[31], 3}, {NULL}};
const Segment SGI208131[] = {{&S85931[110], 6}, {NULL}};
const Segment SGI208132[] = {{&S85931[69], 8}, {NULL}};
const Segment SGI208133[] = {{&S85931[81], 15}, {NULL}};
const Segment SGI208134[] = {{&S85938[44], 10}, {NULL}};
const Segment SGI208135[] = {{&S85938[44], 22}, {NULL}};
const Segment SGI208136[] = {{&S85938[30], 6}, {NULL}};
const Segment SGI208137[] = {{&S85938[31], 3}, {NULL}};
const Segment SGI208138[] = {{&S85938[95], 6}, {NULL}};
const Segment SGI208139[] = {{&S85938[68], 8}, {NULL}};
const Segment SGI208140[] = {{&S85944[34], 8}, {NULL}};
const Segment SGI208141[] = {{&S85944[27], 6}, {NULL}};
const Segment SGI208142[] = {{&S66179[34], 7}, {NULL}};
const Segment SGI208143[] = {{&S66179[27], 6}, {NULL}};
const Segment SGI208144[] = {{&S85944[95], 9}, {NULL}};
const Segment SGI208145[] = {{&S85944[68], 6}, {NULL}};
const Segment SGI208146[] = {{&S66179[94], 9}, {NULL}};
const Segment SGI208147[] = {{&S66179[67], 6}, {NULL}};
const Segment SGI208148[] = {{&S66180[34], 14}, {NULL}};
const Segment SGI208149[] = {{&S66180[27], 6}, {NULL}};
const Segment SGI208150[] = {{&S66180[100], 9}, {NULL}};
const Segment SGI208151[] = {{&S66180[73], 6}, {NULL}};
const Segment SGI208152[] = {{&S80602[34], 12}, {NULL}};
const Segment SGI208153[] = {{&S80602[27], 6}, {NULL}};
const Segment SGI208154[] = {{&S80602[99], 9}, {NULL}};
const Segment SGI208155[] = {{&S80602[72], 6}, {NULL}};
const Segment SGI208156[] = {{&S85948[34], 15}, {NULL}};
const Segment SGI208157[] = {{&S85948[27], 6}, {NULL}};
const Segment SGI208158[] = {{&S85948[102], 9}, {NULL}};
const Segment SGI208159[] = {{&S85948[75], 6}, {NULL}};
const Segment SGI208160[] = {{&S80611[44], 10}, {NULL}};
const Segment SGI208161[] = {{&S80611[44], 22}, {NULL}};
const Segment SGI208162[] = {{&S80611[30], 6}, {NULL}};
const Segment SGI208163[] = {{&S80611[131], 3}, {NULL}};
const Segment SGI208164[] = {{&S80611[96], 6}, {NULL}};
const Segment SGI208165[] = {{&S80611[68], 8}, {NULL}};
const Segment SGI208166[] = {{&S85956[59], 7}, {NULL}};
const Segment SGI208167[] = {{&S85956[59], 19}, {NULL}};
const Segment SGI208168[] = {{&S85956[45], 6}, {NULL}};
const Segment SGI208169[] = {{&S85956[46], 3}, {NULL}};
const Segment SGI208170[] = {{&S85956[92], 6}, {NULL}};
const Segment SGI208171[] = {{&S85956[0], 14}, {NULL}};
const Segment SGI208172[] = {{&S85957[59], 8}, {NULL}};
const Segment SGI208173[] = {{&S85957[59], 20}, {NULL}};
const Segment SGI208174[] = {{&S85957[45], 6}, {NULL}};
const Segment SGI208175[] = {{&S85957[46], 3}, {NULL}};
const Segment SGI208176[] = {{&S85957[93], 6}, {NULL}};
const Segment SGI208177[] = {{&S85957[0], 14}, {NULL}};
const Segment SGI208178[] = {{&S75347[34], 10}, {NULL}};
const Segment SGI208179[] = {{&S75347[27], 6}, {NULL}};
const Segment SGI208180[] = {{&S75347[97], 9}, {NULL}};
const Segment SGI208181[] = {{&S75347[70], 6}, {NULL}};
const Segment SGI208182[] = {{&S85961[44], 24}, {NULL}};
const Segment SGI208183[] = {{&S85961[44], 36}, {NULL}};
const Segment SGI208184[] = {{&S85961[30], 6}, {NULL}};
const Segment SGI208185[] = {{&S66199[34], 18}, {NULL}};
const Segment SGI208186[] = {{&S66199[27], 6}, {NULL}};
const Segment SGI208187[] = {{&S85961[31], 3}, {NULL}};
const Segment SGI208188[] = {{&S85961[94], 6}, {NULL}};
const Segment SGI208189[] = {{&S66199[104], 9}, {NULL}};
const Segment SGI208190[] = {{&S66199[77], 6}, {NULL}};
const Segment SGI208191[] = {{&S85962[38], 24}, {NULL}};
const Segment SGI208192[] = {{&S85962[31], 6}, {NULL}};
const Segment SGI208193[] = {{&S66200[34], 10}, {NULL}};
const Segment SGI208194[] = {{&S80622[44], 19}, {NULL}};
const Segment SGI208195[] = {{&S66200[27], 6}, {NULL}};
const Segment SGI208196[] = {{&S80622[44], 31}, {NULL}};
const Segment SGI208197[] = {{&S80622[30], 6}, {NULL}};
const Segment SGI208198[] = {{&S80622[124], 3}, {NULL}};
const Segment SGI208199[] = {{&S66200[96], 9}, {NULL}};
const Segment SGI208200[] = {{&S66200[69], 6}, {NULL}};
const Segment SGI208201[] = {{&S80622[89], 6}, {NULL}};
const Segment SGI208202[] = {{&S85964[34], 13}, {NULL}};
const Segment SGI208203[] = {{&S85964[27], 6}, {NULL}};
const Segment SGI208204[] = {{&S80624[44], 14}, {NULL}};
const Segment SGI208205[] = {{&S80624[44], 31}, {NULL}};
const Segment SGI208206[] = {{&S85964[100], 9}, {NULL}};
const Segment SGI208207[] = {{&S80624[30], 6}, {NULL}};
const Segment SGI208208[] = {{&S85964[73], 6}, {NULL}};
const Segment SGI208209[] = {{&S80624[89], 6}, {NULL}};
const Segment SGI208210[] = {{&S80624[116], 13}, {NULL}};
const Segment SGI208211[] = {{&S75951[44], 4}, {NULL}};
const Segment SGI208212[] = {{&S75951[44], 16}, {NULL}};
const Segment SGI208213[] = {{&S75951[30], 6}, {NULL}};
const Segment SGI208214[] = {{&S75951[109], 3}, {NULL}};
const Segment SGI208215[] = {{&S75951[74], 6}, {NULL}};
const Segment SGI208216[] = {{&S85970[34], 19}, {NULL}};
const Segment SGI208217[] = {{&S85970[27], 6}, {NULL}};
const Segment SGI208218[] = {{&S75955[34], 10}, {NULL}};
const Segment SGI208219[] = {{&S75955[27], 6}, {NULL}};
const Segment SGI208220[] = {{&S85970[106], 9}, {NULL}};
const Segment SGI208221[] = {{&S85970[79], 6}, {NULL}};
const Segment SGI208222[] = {{&S75955[97], 9}, {NULL}};
const Segment SGI208223[] = {{&S75955[70], 6}, {NULL}};
const Segment SGI208224[] = {{&S85971[34], 18}, {NULL}};
const Segment SGI208225[] = {{&S85971[27], 6}, {NULL}};
const Segment SGI208226[] = {{&S85971[105], 9}, {NULL}};
const Segment SGI208227[] = {{&S85971[78], 6}, {NULL}};
const Segment SGI208228[] = {{&S75957[34], 7}, {NULL}};
const Segment SGI208229[] = {{&S75957[27], 6}, {NULL}};
const Segment SGI208230[] = {{&S75957[94], 9}, {NULL}};
const Segment SGI208231[] = {{&S75957[67], 6}, {NULL}};
const Segment SGI208232[] = {{&S80636[44], 11}, {NULL}};
const Segment SGI208233[] = {{&S80636[44], 23}, {NULL}};
const Segment SGI208234[] = {{&S80636[30], 6}, {NULL}};
const Segment SGI208235[] = {{&S80636[116], 3}, {NULL}};
const Segment SGI208236[] = {{&S80636[81], 6}, {NULL}};
const Segment SGI208237[] = {{&S66220[44], 17}, {NULL}};
const Segment SGI208238[] = {{&S66220[44], 28}, {NULL}};
const Segment SGI208239[] = {{&S66220[30], 6}, {NULL}};
const Segment SGI208240[] = {{&S66220[121], 3}, {NULL}};
const Segment SGI208241[] = {{&S66220[86], 6}, {NULL}};
const Segment SGI208242[] = {{&S80641[34], 7}, {NULL}};
const Segment SGI208243[] = {{&S80641[27], 6}, {NULL}};
const Segment SGI208244[] = {{&S80641[98], 9}, {NULL}};
const Segment SGI208245[] = {{&S80641[71], 6}, {NULL}};
const Segment SGI208246[] = {{&S66221[44], 17}, {NULL}};
const Segment SGI208247[] = {{&S66221[44], 28}, {NULL}};
const Segment SGI208248[] = {{&S66221[30], 6}, {NULL}};
const Segment SGI208249[] = {{&S66221[121], 3}, {NULL}};
const Segment SGI208250[] = {{&S66221[86], 6}, {NULL}};
const Segment SGI208251[] = {{&S66222[38], 17}, {NULL}};
const Segment SGI208252[] = {{&S66222[31], 6}, {NULL}};
const Segment SGI208253[] = {{&S66223[34], 11}, {NULL}};
const Segment SGI208254[] = {{&S66223[27], 6}, {NULL}};
const Segment SGI208255[] = {{&S66223[97], 9}, {NULL}};
const Segment SGI208256[] = {{&S66223[70], 6}, {NULL}};
const Segment SGI208257[] = {{&S85988[44], 15}, {NULL}};
const Segment SGI208258[] = {{&S85988[44], 33}, {NULL}};
const Segment SGI208259[] = {{&S85988[30], 6}, {NULL}};
const Segment SGI208260[] = {{&S85988[31], 3}, {NULL}};
const Segment SGI208261[] = {{&S85988[91], 5}, {NULL}};
const Segment SGI208262[] = {{&S85988[128], 14}, {NULL}};
const Segment SGI208263[] = {{&S85989[44], 8}, {NULL}};
const Segment SGI208264[] = {{&S85989[44], 24}, {NULL}};
const Segment SGI208265[] = {{&S85989[30], 6}, {NULL}};
const Segment SGI208266[] = {{&S85989[82], 6}, {NULL}};
const Segment SGI208267[] = {{&S85989[109], 13}, {NULL}};
const Segment SGI208268[] = {{&S85990[44], 8}, {NULL}};
const Segment SGI208269[] = {{&S85990[44], 24}, {NULL}};
const Segment SGI208270[] = {{&S85990[30], 6}, {NULL}};
const Segment SGI208271[] = {{&S85990[82], 6}, {NULL}};
const Segment SGI208272[] = {{&S85990[109], 13}, {NULL}};
const Segment SGI208273[] = {{&S85998[34], 13}, {NULL}};
const Segment SGI208274[] = {{&S85998[27], 6}, {NULL}};
const Segment SGI208275[] = {{&S85998[100], 9}, {NULL}};
const Segment SGI208276[] = {{&S85998[73], 6}, {NULL}};
const Segment SGI208277[] = {{&S66245[38], 11}, {NULL}};
const Segment SGI208278[] = {{&S66245[31], 6}, {NULL}};
const Segment SGI208279[] = {{&S66246[44], 11}, {NULL}};
const Segment SGI208280[] = {{&S66246[44], 22}, {NULL}};
const Segment SGI208281[] = {{&S66246[30], 6}, {NULL}};
const Segment SGI208282[] = {{&S66246[115], 3}, {NULL}};
const Segment SGI208283[] = {{&S66246[80], 6}, {NULL}};
const Segment SGI208284[] = {{&S86004[34], 20}, {NULL}};
const Segment SGI208285[] = {{&S86004[27], 6}, {NULL}};
const Segment SGI208286[] = {{&S86004[117], 9}, {NULL}};
const Segment SGI208287[] = {{&S86004[90], 6}, {NULL}};
const Segment SGI208288[] = {{&S86005[34], 41}, {NULL}};
const Segment SGI208289[] = {{&S86005[27], 6}, {NULL}};
const Segment SGI208290[] = {{&S86005[138], 9}, {NULL}};
const Segment SGI208291[] = {{&S86005[111], 6}, {NULL}};
const Segment SGI208292[] = {{&S86011[34], 5}, {NULL}};
const Segment SGI208293[] = {{&S86011[27], 6}, {NULL}};
const Segment SGI208294[] = {{&S86011[92], 9}, {NULL}};
const Segment SGI208295[] = {{&S86011[65], 6}, {NULL}};
const Segment SGI208296[] = {{&S80686[34], 7}, {NULL}};
const Segment SGI208297[] = {{&S80686[27], 6}, {NULL}};
const Segment SGI208298[] = {{&S80686[94], 9}, {NULL}};
const Segment SGI208299[] = {{&S80686[67], 6}, {NULL}};
const Segment SGI208300[] = {{&S80687[34], 7}, {NULL}};
const Segment SGI208301[] = {{&S80687[27], 6}, {NULL}};
const Segment SGI208302[] = {{&S80687[94], 9}, {NULL}};
const Segment SGI208303[] = {{&S80687[67], 6}, {NULL}};
const Segment SGI208304[] = {{&S80690[44], 7}, {NULL}};
const Segment SGI208305[] = {{&S80690[44], 19}, {NULL}};
const Segment SGI208306[] = {{&S80690[30], 6}, {NULL}};
const Segment SGI208307[] = {{&S80690[112], 3}, {NULL}};
const Segment SGI208308[] = {{&S80690[77], 6}, {NULL}};
const Segment SGI208309[] = {{&S80694[44], 7}, {NULL}};
const Segment SGI208310[] = {{&S80694[44], 19}, {NULL}};
const Segment SGI208311[] = {{&S80694[30], 6}, {NULL}};
const Segment SGI208312[] = {{&S80694[112], 3}, {NULL}};
const Segment SGI208313[] = {{&S80694[77], 6}, {NULL}};
const Segment SGI208314[] = {{&S80695[44], 11}, {NULL}};
const Segment SGI208315[] = {{&S80695[44], 23}, {NULL}};
const Segment SGI208316[] = {{&S80695[30], 6}, {NULL}};
const Segment SGI208317[] = {{&S80695[116], 3}, {NULL}};
const Segment SGI208318[] = {{&S80695[81], 6}, {NULL}};
const Segment SGI208319[] = {{&S86055[34], 4}, {NULL}};
const Segment SGI208320[] = {{&S86055[27], 6}, {NULL}};
const Segment SGI208321[] = {{&S86055[91], 9}, {NULL}};
const Segment SGI208322[] = {{&S86055[64], 6}, {NULL}};
const Segment SGI208323[] = {{&S86064[34], 10}, {NULL}};
const Segment SGI208324[] = {{&S86064[27], 6}, {NULL}};
const Segment SGI208325[] = {{&S86064[97], 9}, {NULL}};
const Segment SGI208326[] = {{&S86064[70], 6}, {NULL}};
const Segment SGI208327[] = {{&S86066[34], 16}, {NULL}};
const Segment SGI208328[] = {{&S86066[27], 6}, {NULL}};
const Segment SGI208329[] = {{&S86066[103], 9}, {NULL}};
const Segment SGI208330[] = {{&S86066[76], 6}, {NULL}};
const Segment SGI208331[] = {{&S86067[44], 15}, {NULL}};
const Segment SGI208332[] = {{&S86067[44], 27}, {NULL}};
const Segment SGI208333[] = {{&S86067[30], 6}, {NULL}};
const Segment SGI208334[] = {{&S86067[31], 3}, {NULL}};
const Segment SGI208335[] = {{&S86067[85], 6}, {NULL}};
const Segment SGI208336[] = {{&S86068[34], 13}, {NULL}};
const Segment SGI208337[] = {{&S86068[27], 6}, {NULL}};
const Segment SGI208338[] = {{&S76076[34], 9}, {NULL}};
const Segment SGI208339[] = {{&S76076[27], 6}, {NULL}};
const Segment SGI208340[] = {{&S86068[100], 9}, {NULL}};
const Segment SGI208341[] = {{&S86068[73], 6}, {NULL}};
const Segment SGI208342[] = {{&S76076[96], 9}, {NULL}};
const Segment SGI208343[] = {{&S76076[69], 6}, {NULL}};
const Segment SGI208344[] = {{&S76077[34], 9}, {NULL}};
const Segment SGI208345[] = {{&S76077[27], 6}, {NULL}};
const Segment SGI208346[] = {{&S76077[96], 9}, {NULL}};
const Segment SGI208347[] = {{&S76077[69], 6}, {NULL}};
const Segment SGI208348[] = {{&S86071[34], 11}, {NULL}};
const Segment SGI208349[] = {{&S86071[27], 6}, {NULL}};
const Segment SGI208350[] = {{&S86071[98], 9}, {NULL}};
const Segment SGI208351[] = {{&S86071[71], 6}, {NULL}};
const Segment SGI208352[] = {{&S86073[34], 10}, {NULL}};
const Segment SGI208353[] = {{&S86073[27], 6}, {NULL}};
const Segment SGI208354[] = {{&S86073[97], 9}, {NULL}};
const Segment SGI208355[] = {{&S86073[70], 6}, {NULL}};
const Segment SGI208356[] = {{&S66341[34], 15}, {NULL}};
const Segment SGI208357[] = {{&S66341[27], 6}, {NULL}};
const Segment SGI208358[] = {{&S66341[102], 9}, {NULL}};
const Segment SGI208359[] = {{&S66341[75], 6}, {NULL}};
const Segment SGI208360[] = {{&S86087[44], 10}, {NULL}};
const Segment SGI208361[] = {{&S86087[44], 22}, {NULL}};
const Segment SGI208362[] = {{&S86087[30], 6}, {NULL}};
const Segment SGI208363[] = {{&S86087[31], 3}, {NULL}};
const Segment SGI208364[] = {{&S86087[80], 5}, {NULL}};
const Segment SGI208365[] = {{&S86087[117], 14}, {NULL}};
const Segment SGI208366[] = {{&S86089[44], 4}, {NULL}};
const Segment SGI208367[] = {{&S86089[44], 16}, {NULL}};
const Segment SGI208368[] = {{&S86089[30], 6}, {NULL}};
const Segment SGI208369[] = {{&S86089[31], 3}, {NULL}};
const Segment SGI208370[] = {{&S86089[74], 6}, {NULL}};
const Segment SGI208371[] = {{&S80774[34], 8}, {NULL}};
const Segment SGI208372[] = {{&S80774[27], 6}, {NULL}};
const Segment SGI208373[] = {{&S80774[95], 9}, {NULL}};
const Segment SGI208374[] = {{&S80774[68], 6}, {NULL}};
const Segment SGI208375[] = {{&S86095[34], 7}, {NULL}};
const Segment SGI208376[] = {{&S86095[27], 6}, {NULL}};
const Segment SGI208377[] = {{&S86095[104], 9}, {NULL}};
const Segment SGI208378[] = {{&S86095[77], 6}, {NULL}};
const Segment SGI208379[] = {{&S80783[44], 11}, {NULL}};
const Segment SGI208380[] = {{&S80783[44], 23}, {NULL}};
const Segment SGI208381[] = {{&S80783[30], 6}, {NULL}};
const Segment SGI208382[] = {{&S80783[81], 6}, {NULL}};
const Segment SGI208383[] = {{&S86102[44], 7}, {NULL}};
const Segment SGI208384[] = {{&S80783[108], 13}, {NULL}};
const Segment SGI208385[] = {{&S86102[44], 25}, {NULL}};
const Segment SGI208386[] = {{&S86102[30], 6}, {NULL}};
const Segment SGI208387[] = {{&S86102[83], 6}, {NULL}};
const Segment SGI208388[] = {{&S86102[110], 13}, {NULL}};
const Segment SGI208389[] = {{&S79089[32], 25}, {NULL}};
const Segment SGI208390[] = {{&S79089[27], 4}, {NULL}};
const Segment SGI208391[] = {{&S79089[110], 9}, {NULL}};
const Segment SGI208392[] = {{&S79089[83], 6}, {NULL}};
const Segment SGI208393[] = {{&S79090[11], 1}, {NULL}};
const Segment SGI208394[] = {{&S79090[27], 4}, {NULL}};
const Segment SGI208395[] = {{&S79090[86], 9}, {NULL}};
const Segment SGI208396[] = {{&S79090[59], 6}, {NULL}};
const Segment SGI208397[] = {{&S75458[34], 7}, {NULL}};
const Segment SGI208398[] = {{&S75458[27], 6}, {NULL}};
const Segment SGI208399[] = {{&S75458[94], 9}, {NULL}};
const Segment SGI208400[] = {{&S75458[67], 6}, {NULL}};
const Segment SGI208401[] = {{&S66385[34], 10}, {NULL}};
const Segment SGI208402[] = {{&S66385[27], 6}, {NULL}};
const Segment SGI208403[] = {{&S66385[97], 9}, {NULL}};
const Segment SGI208404[] = {{&S66385[70], 6}, {NULL}};
const Segment SGI208405[] = {{&S86124[44], 11}, {NULL}};
const Segment SGI208406[] = {{&S76085[44], 11}, {NULL}};
const Segment SGI208407[] = {{&S86124[44], 23}, {NULL}};
const Segment SGI208408[] = {{&S76085[44], 23}, {NULL}};
const Segment SGI208409[] = {{&S86124[30], 6}, {NULL}};
const Segment SGI208410[] = {{&S76085[30], 6}, {NULL}};
const Segment SGI208411[] = {{&S86124[31], 3}, {NULL}};
const Segment SGI208412[] = {{&S76085[116], 3}, {NULL}};
const Segment SGI208413[] = {{&S76085[81], 6}, {NULL}};
const Segment SGI208414[] = {{&S86124[100], 6}, {NULL}};
const Segment SGI208415[] = {{&S86124[69], 8}, {NULL}};
const Segment SGI208416[] = {{&S76086[38], 11}, {NULL}};
const Segment SGI208417[] = {{&S76086[31], 6}, {NULL}};
const Segment SGI208418[] = {{&S86125[44], 11}, {NULL}};
const Segment SGI208419[] = {{&S86125[44], 23}, {NULL}};
const Segment SGI208420[] = {{&S86125[30], 6}, {NULL}};
const Segment SGI208421[] = {{&S86125[31], 3}, {NULL}};
const Segment SGI208422[] = {{&S86125[100], 6}, {NULL}};
const Segment SGI208423[] = {{&S86125[69], 8}, {NULL}};
const Segment SGI208424[] = {{&S76087[34], 14}, {NULL}};
const Segment SGI208425[] = {{&S76087[27], 6}, {NULL}};
const Segment SGI208426[] = {{&S76087[111], 9}, {NULL}};
const Segment SGI208427[] = {{&S76087[84], 6}, {NULL}};
const Segment SGI208428[] = {{&S76087[152], 8}, {NULL}};
const Segment SGI208429[] = {{&S86132[34], 10}, {NULL}};
const Segment SGI208430[] = {{&S86132[27], 6}, {NULL}};
const Segment SGI208431[] = {{&S86132[97], 9}, {NULL}};
const Segment SGI208432[] = {{&S86132[70], 6}, {NULL}};
const Segment SGI208433[] = {{&S66411[34], 20}, {NULL}};
const Segment SGI208434[] = {{&S66411[27], 6}, {NULL}};
const Segment SGI208435[] = {{&S66411[106], 9}, {NULL}};
const Segment SGI208436[] = {{&S66411[79], 6}, {NULL}};
const Segment SGI208437[] = {{&S66412[34], 21}, {NULL}};
const Segment SGI208438[] = {{&S66412[27], 6}, {NULL}};
const Segment SGI208439[] = {{&S66412[107], 9}, {NULL}};
const Segment SGI208440[] = {{&S66412[80], 6}, {NULL}};
const Segment SGI208441[] = {{&S86144[44], 7}, {NULL}};
const Segment SGI208442[] = {{&S86144[44], 19}, {NULL}};
const Segment SGI208443[] = {{&S86144[30], 6}, {NULL}};
const Segment SGI208444[] = {{&S86144[31], 3}, {NULL}};
const Segment SGI208445[] = {{&S86144[77], 6}, {NULL}};
const Segment SGI208446[] = {{&S80840[44], 17}, {NULL}};
const Segment SGI208447[] = {{&S80840[44], 29}, {NULL}};
const Segment SGI208448[] = {{&S80840[30], 6}, {NULL}};
const Segment SGI208449[] = {{&S80840[122], 3}, {NULL}};
const Segment SGI208450[] = {{&S80840[87], 6}, {NULL}};
const Segment SGI208451[] = {{&S86156[44], 13}, {NULL}};
const Segment SGI208452[] = {{&S80842[34], 10}, {NULL}};
const Segment SGI208453[] = {{&S86156[44], 25}, {NULL}};
const Segment SGI208454[] = {{&S86156[30], 6}, {NULL}};
const Segment SGI208455[] = {{&S80842[27], 6}, {NULL}};
const Segment SGI208456[] = {{&S80842[97], 9}, {NULL}};
const Segment SGI208457[] = {{&S86156[83], 6}, {NULL}};
const Segment SGI208458[] = {{&S80842[70], 6}, {NULL}};
const Segment SGI208459[] = {{&S86156[110], 13}, {NULL}};
const Segment SGI208460[] = {{&S66427[34], 8}, {NULL}};
const Segment SGI208461[] = {{&S66427[27], 6}, {NULL}};
const Segment SGI208462[] = {{&S66427[94], 9}, {NULL}};
const Segment SGI208463[] = {{&S66427[67], 6}, {NULL}};
const Segment SGI208464[] = {{&S86158[44], 10}, {NULL}};
const Segment SGI208465[] = {{&S86158[44], 22}, {NULL}};
const Segment SGI208466[] = {{&S86158[30], 6}, {NULL}};
const Segment SGI208467[] = {{&S66429[34], 19}, {NULL}};
const Segment SGI208468[] = {{&S86158[31], 3}, {NULL}};
const Segment SGI208469[] = {{&S66429[27], 6}, {NULL}};
const Segment SGI208470[] = {{&S86158[80], 6}, {NULL}};
const Segment SGI208471[] = {{&S86158[143], 3}, {NULL}};
const Segment SGI208472[] = {{&S66429[105], 9}, {NULL}};
const Segment SGI208473[] = {{&S66429[78], 6}, {NULL}};
const Segment SGI208474[] = {{&S80846[34], 10}, {NULL}};
const Segment SGI208475[] = {{&S80846[27], 6}, {NULL}};
const Segment SGI208476[] = {{&S80846[97], 9}, {NULL}};
const Segment SGI208477[] = {{&S80846[70], 6}, {NULL}};
const Segment SGI208478[] = {{&S66430[34], 10}, {NULL}};
const Segment SGI208479[] = {{&S66430[27], 6}, {NULL}};
const Segment SGI208480[] = {{&S86160[34], 11}, {NULL}};
const Segment SGI208481[] = {{&S66430[96], 9}, {NULL}};
const Segment SGI208482[] = {{&S66430[69], 6}, {NULL}};
const Segment SGI208483[] = {{&S86160[27], 6}, {NULL}};
const Segment SGI208484[] = {{&S86160[98], 9}, {NULL}};
const Segment SGI208485[] = {{&S86160[71], 6}, {NULL}};
const Segment SGI208486[] = {{&S86161[34], 13}, {NULL}};
const Segment SGI208487[] = {{&S86161[27], 6}, {NULL}};
const Segment SGI208488[] = {{&S86161[100], 9}, {NULL}};
const Segment SGI208489[] = {{&S86161[73], 6}, {NULL}};
const Segment SGI208490[] = {{&S66433[27], 6}, {NULL}};
const Segment SGI208491[] = {{&S66433[81], 9}, {NULL}};
const Segment SGI208492[] = {{&S66433[54], 6}, {NULL}};
const Segment SGI208493[] = {{&S66435[34], 4}, {NULL}};
const Segment SGI208494[] = {{&S66435[27], 6}, {NULL}};
const Segment SGI208495[] = {{&S66435[91], 9}, {NULL}};
const Segment SGI208496[] = {{&S66435[64], 6}, {NULL}};
const Segment SGI208497[] = {{&S80855[44], 13}, {NULL}};
const Segment SGI208498[] = {{&S80855[44], 25}, {NULL}};
const Segment SGI208499[] = {{&S80855[30], 6}, {NULL}};
const Segment SGI208500[] = {{&S80855[118], 3}, {NULL}};
const Segment SGI208501[] = {{&S80855[83], 6}, {NULL}};
const Segment SGI208502[] = {{&S80861[44], 7}, {NULL}};
const Segment SGI208503[] = {{&S86174[34], 13}, {NULL}};
const Segment SGI208504[] = {{&S80861[44], 19}, {NULL}};
const Segment SGI208505[] = {{&S80861[30], 6}, {NULL}};
const Segment SGI208506[] = {{&S86174[27], 6}, {NULL}};
const Segment SGI208507[] = {{&S66444[34], 10}, {NULL}};
const Segment SGI208508[] = {{&S80861[31], 3}, {NULL}};
const Segment SGI208509[] = {{&S66444[27], 6}, {NULL}};
const Segment SGI208510[] = {{&S86174[100], 9}, {NULL}};
const Segment SGI208511[] = {{&S80861[77], 6}, {NULL}};
const Segment SGI208512[] = {{&S86174[73], 6}, {NULL}};
const Segment SGI208513[] = {{&S66444[97], 9}, {NULL}};
const Segment SGI208514[] = {{&S66444[70], 6}, {NULL}};
const Segment SGI208515[] = {{&S86175[34], 8}, {NULL}};
const Segment SGI208516[] = {{&S86175[27], 6}, {NULL}};
const Segment SGI208517[] = {{&S66445[34], 10}, {NULL}};
const Segment SGI208518[] = {{&S66445[27], 6}, {NULL}};
const Segment SGI208519[] = {{&S86175[100], 9}, {NULL}};
const Segment SGI208520[] = {{&S86175[73], 6}, {NULL}};
const Segment SGI208521[] = {{&S66445[96], 9}, {NULL}};
const Segment SGI208522[] = {{&S66445[69], 6}, {NULL}};
const Segment SGI208523[] = {{&S80865[34], 14}, {NULL}};
const Segment SGI208524[] = {{&S80865[27], 6}, {NULL}};
const Segment SGI208525[] = {{&S80865[101], 9}, {NULL}};
const Segment SGI208526[] = {{&S80865[74], 6}, {NULL}};
const Segment SGI208527[] = {{&S75534[80], 6}, {NULL}};
const Segment SGI208528[] = {{&S86179[34], 13}, {NULL}};
const Segment SGI208529[] = {{&S86179[27], 6}, {NULL}};
const Segment SGI208530[] = {{&S86179[100], 9}, {NULL}};
const Segment SGI208531[] = {{&S86179[73], 6}, {NULL}};
const Segment SGI208532[] = {{&S75535[44], 7}, {NULL}};
const Segment SGI208533[] = {{&S75535[44], 19}, {NULL}};
const Segment SGI208534[] = {{&S75535[30], 6}, {NULL}};
const Segment SGI208535[] = {{&S75535[112], 3}, {NULL}};
const Segment SGI208536[] = {{&S75535[77], 6}, {NULL}};
const Segment SGI208537[] = {{&S75536[38], 7}, {NULL}};
const Segment SGI208538[] = {{&S75536[31], 6}, {NULL}};
const Segment SGI208539[] = {{&S86183[44], 17}, {NULL}};
const Segment SGI208540[] = {{&S86183[44], 29}, {NULL}};
const Segment SGI208541[] = {{&S86183[30], 6}, {NULL}};
const Segment SGI208542[] = {{&S86183[31], 3}, {NULL}};
const Segment SGI208543[] = {{&S86183[87], 6}, {NULL}};
const Segment SGI208544[] = {{&S86184[44], 25}, {NULL}};
const Segment SGI208545[] = {{&S86184[44], 37}, {NULL}};
const Segment SGI208546[] = {{&S86184[30], 6}, {NULL}};
const Segment SGI208547[] = {{&S86184[31], 3}, {NULL}};
const Segment SGI208548[] = {{&S86184[95], 6}, {NULL}};
const Segment SGI208549[] = {{&S66457[34], 10}, {NULL}};
const Segment SGI208550[] = {{&S66457[27], 6}, {NULL}};
const Segment SGI208551[] = {{&S66457[96], 9}, {NULL}};
const Segment SGI208552[] = {{&S66457[69], 6}, {NULL}};
const Segment SGI208553[] = {{&S86185[42], 25}, {NULL}};
const Segment SGI208554[] = {{&S86185[30], 6}, {NULL}};
const Segment SGI208555[] = {{&S86185[31], 3}, {NULL}};
const Segment SGI208556[] = {{&S86185[93], 6}, {NULL}};
const Segment SGI208557[] = {{&S66459[34], 10}, {NULL}};
const Segment SGI208558[] = {{&S66459[27], 6}, {NULL}};
const Segment SGI208559[] = {{&S66459[97], 9}, {NULL}};
const Segment SGI208560[] = {{&S66459[70], 6}, {NULL}};
const Segment SGI208561[] = {{&S66461[34], 10}, {NULL}};
const Segment SGI208562[] = {{&S66461[27], 6}, {NULL}};
const Segment SGI208563[] = {{&S66461[97], 9}, {NULL}};
const Segment SGI208564[] = {{&S66461[70], 6}, {NULL}};
const Segment SGI208565[] = {{&S86200[34], 8}, {NULL}};
const Segment SGI208566[] = {{&S86200[27], 6}, {NULL}};
const Segment SGI208567[] = {{&S75559[34], 9}, {NULL}};
const Segment SGI208568[] = {{&S80895[34], 7}, {NULL}};
const Segment SGI208569[] = {{&S86200[101], 9}, {NULL}};
const Segment SGI208570[] = {{&S75559[27], 6}, {NULL}};
const Segment SGI208571[] = {{&S86200[74], 6}, {NULL}};
const Segment SGI208572[] = {{&S80895[27], 6}, {NULL}};
const Segment SGI208573[] = {{&S75559[96], 9}, {NULL}};
const Segment SGI208574[] = {{&S80895[94], 9}, {NULL}};
const Segment SGI208575[] = {{&S75559[69], 6}, {NULL}};
const Segment SGI208576[] = {{&S80895[67], 6}, {NULL}};
const Segment SGI208577[] = {{&S80896[34], 10}, {NULL}};
const Segment SGI208578[] = {{&S80896[27], 6}, {NULL}};
const Segment SGI208579[] = {{&S80896[97], 9}, {NULL}};
const Segment SGI208580[] = {{&S80896[70], 6}, {NULL}};
const Segment SGI208581[] = {{&S80902[44], 18}, {NULL}};
const Segment SGI208582[] = {{&S80902[44], 30}, {NULL}};
const Segment SGI208583[] = {{&S80902[30], 6}, {NULL}};
const Segment SGI208584[] = {{&S80902[123], 3}, {NULL}};
const Segment SGI208585[] = {{&S80902[88], 6}, {NULL}};
const Segment SGI208586[] = {{&S80903[38], 18}, {NULL}};
const Segment SGI208587[] = {{&S80903[31], 6}, {NULL}};
const Segment SGI208588[] = {{&S80909[34], 12}, {NULL}};
const Segment SGI208589[] = {{&S80909[27], 6}, {NULL}};
const Segment SGI208590[] = {{&S80909[99], 9}, {NULL}};
const Segment SGI208591[] = {{&S80909[72], 6}, {NULL}};
const Segment SGI208592[] = {{&S66492[44], 9}, {NULL}};
const Segment SGI208593[] = {{&S66492[44], 21}, {NULL}};
const Segment SGI208594[] = {{&S66492[30], 6}, {NULL}};
const Segment SGI208595[] = {{&S66492[132], 3}, {NULL}};
const Segment SGI208596[] = {{&S66492[97], 6}, {NULL}};
const Segment SGI208597[] = {{&S80910[34], 12}, {NULL}};
const Segment SGI208598[] = {{&S66492[67], 8}, {NULL}};
const Segment SGI208599[] = {{&S80910[27], 6}, {NULL}};
const Segment SGI208600[] = {{&S80910[99], 9}, {NULL}};
const Segment SGI208601[] = {{&S80910[72], 6}, {NULL}};
const Segment SGI208602[] = {{&S66493[44], 5}, {NULL}};
const Segment SGI208603[] = {{&S66493[44], 17}, {NULL}};
const Segment SGI208604[] = {{&S66493[30], 6}, {NULL}};
const Segment SGI208605[] = {{&S66493[110], 3}, {NULL}};
const Segment SGI208606[] = {{&S80911[44], 7}, {NULL}};
const Segment SGI208607[] = {{&S66493[75], 6}, {NULL}};
const Segment SGI208608[] = {{&S80911[44], 23}, {NULL}};
const Segment SGI208609[] = {{&S80911[30], 6}, {NULL}};
const Segment SGI208610[] = {{&S66493[135], 8}, {NULL}};
const Segment SGI208611[] = {{&S80911[61], 3}, {NULL}};
const Segment SGI208612[] = {{&S80911[81], 6}, {NULL}};
const Segment SGI208613[] = {{&S66494[34], 7}, {NULL}};
const Segment SGI208614[] = {{&S66494[27], 6}, {NULL}};
const Segment SGI208615[] = {{&S80912[44], 7}, {NULL}};
const Segment SGI208616[] = {{&S66494[100], 9}, {NULL}};
const Segment SGI208617[] = {{&S80912[44], 23}, {NULL}};
const Segment SGI208618[] = {{&S66494[73], 6}, {NULL}};
const Segment SGI208619[] = {{&S80912[30], 6}, {NULL}};
const Segment SGI208620[] = {{&S80912[116], 3}, {NULL}};
const Segment SGI208621[] = {{&S80912[81], 6}, {NULL}};
const Segment SGI208622[] = {{&S66495[34], 7}, {NULL}};
const Segment SGI208623[] = {{&S66495[27], 6}, {NULL}};
const Segment SGI208624[] = {{&S80913[44], 7}, {NULL}};
const Segment SGI208625[] = {{&S80913[44], 23}, {NULL}};
const Segment SGI208626[] = {{&S66495[100], 9}, {NULL}};
const Segment SGI208627[] = {{&S80913[30], 6}, {NULL}};
const Segment SGI208628[] = {{&S66495[73], 6}, {NULL}};
const Segment SGI208629[] = {{&S80913[116], 3}, {NULL}};
const Segment SGI208630[] = {{&S80913[81], 6}, {NULL}};
const Segment SGI208631[] = {{&S80914[44], 17}, {NULL}};
const Segment SGI208632[] = {{&S80914[44], 40}, {NULL}};
const Segment SGI208633[] = {{&S80914[30], 6}, {NULL}};
const Segment SGI208634[] = {{&S80914[133], 3}, {NULL}};
const Segment SGI208635[] = {{&S80914[98], 6}, {NULL}};
const Segment SGI208636[] = {{&S80917[44], 15}, {NULL}};
const Segment SGI208637[] = {{&S80917[44], 27}, {NULL}};
const Segment SGI208638[] = {{&S80917[30], 6}, {NULL}};
const Segment SGI208639[] = {{&S80917[120], 3}, {NULL}};
const Segment SGI208640[] = {{&S80917[85], 6}, {NULL}};
const Segment SGI208641[] = {{&S66513[44], 4}, {NULL}};
const Segment SGI208642[] = {{&S66513[44], 15}, {NULL}};
const Segment SGI208643[] = {{&S66513[30], 6}, {NULL}};
const Segment SGI208644[] = {{&S66513[108], 3}, {NULL}};
const Segment SGI208645[] = {{&S66513[73], 6}, {NULL}};
const Segment SGI208646[] = {{&S66514[38], 4}, {NULL}};
const Segment SGI208647[] = {{&S66514[31], 6}, {NULL}};
const Segment SGI208648[] = {{&S76238[44], 12}, {NULL}};
const Segment SGI208649[] = {{&S76238[44], 24}, {NULL}};
const Segment SGI208650[] = {{&S76238[30], 6}, {NULL}};
const Segment SGI208651[] = {{&S76238[133], 3}, {NULL}};
const Segment SGI208652[] = {{&S76238[98], 6}, {NULL}};
const Segment SGI208653[] = {{&S76238[70], 8}, {NULL}};
const Segment SGI208654[] = {{&S76249[34], 7}, {NULL}};
const Segment SGI208655[] = {{&S76249[27], 6}, {NULL}};
const Segment SGI208656[] = {{&S76249[104], 9}, {NULL}};
const Segment SGI208657[] = {{&S76249[77], 6}, {NULL}};
const Segment SGI208658[] = {{&S76252[34], 9}, {NULL}};
const Segment SGI208659[] = {{&S76252[27], 6}, {NULL}};
const Segment SGI208660[] = {{&S76252[96], 9}, {NULL}};
const Segment SGI208661[] = {{&S76252[69], 6}, {NULL}};
const Segment SGI208662[] = {{&S66532[34], 10}, {NULL}};
const Segment SGI208663[] = {{&S66532[27], 6}, {NULL}};
const Segment SGI208664[] = {{&S66532[97], 9}, {NULL}};
const Segment SGI208665[] = {{&S66532[70], 6}, {NULL}};
const Segment SGI208666[] = {{&S80952[34], 9}, {NULL}};
const Segment SGI208667[] = {{&S80952[27], 6}, {NULL}};
const Segment SGI208668[] = {{&S80952[96], 9}, {NULL}};
const Segment SGI208669[] = {{&S80952[69], 6}, {NULL}};
const Segment SGI208670[] = {{&S80956[44], 29}, {NULL}};
const Segment SGI208671[] = {{&S80956[44], 41}, {NULL}};
const Segment SGI208672[] = {{&S80956[30], 6}, {NULL}};
const Segment SGI208673[] = {{&S80956[134], 3}, {NULL}};
const Segment SGI208674[] = {{&S80956[99], 6}, {NULL}};
const Segment SGI208675[] = {{&S80969[44], 10}, {NULL}};
const Segment SGI208676[] = {{&S80969[44], 21}, {NULL}};
const Segment SGI208677[] = {{&S80969[30], 6}, {NULL}};
const Segment SGI208678[] = {{&S80969[114], 3}, {NULL}};
const Segment SGI208679[] = {{&S80969[79], 6}, {NULL}};
const Segment SGI208680[] = {{&S80969[146], 15}, {NULL}};
const Segment SGI208681[] = {{&S80972[34], 12}, {NULL}};
const Segment SGI208682[] = {{&S80972[27], 6}, {NULL}};
const Segment SGI208683[] = {{&S80972[99], 9}, {NULL}};
const Segment SGI208684[] = {{&S80972[72], 6}, {NULL}};
const Segment SGI208685[] = {{&S76289[34], 20}, {NULL}};
const Segment SGI208686[] = {{&S76289[27], 6}, {NULL}};
const Segment SGI208687[] = {{&S76289[107], 9}, {NULL}};
const Segment SGI208688[] = {{&S76289[80], 6}, {NULL}};
const Segment SGI208689[] = {{&S80984[34], 25}, {NULL}};
const Segment SGI208690[] = {{&S80984[27], 6}, {NULL}};
const Segment SGI208691[] = {{&S80984[112], 9}, {NULL}};
const Segment SGI208692[] = {{&S80984[85], 6}, {NULL}};
const Segment SGI208693[] = {{&S66587[34], 10}, {NULL}};
const Segment SGI208694[] = {{&S66587[27], 6}, {NULL}};
const Segment SGI208695[] = {{&S66587[96], 9}, {NULL}};
const Segment SGI208696[] = {{&S66587[69], 6}, {NULL}};
const Segment SGI208697[] = {{&S66587[137], 8}, {NULL}};
const Segment SGI208698[] = {{&S66590[44], 10}, {NULL}};
const Segment SGI208699[] = {{&S66590[44], 21}, {NULL}};
const Segment SGI208700[] = {{&S66590[30], 6}, {NULL}};
const Segment SGI208701[] = {{&S66590[114], 3}, {NULL}};
const Segment SGI208702[] = {{&S66590[79], 6}, {NULL}};
const Segment SGI208703[] = {{&S66591[38], 10}, {NULL}};
const Segment SGI208704[] = {{&S66591[31], 6}, {NULL}};
const Segment SGI208705[] = {{&S66593[44], 9}, {NULL}};
const Segment SGI208706[] = {{&S66593[44], 20}, {NULL}};
const Segment SGI208707[] = {{&S66593[30], 6}, {NULL}};
const Segment SGI208708[] = {{&S66593[113], 3}, {NULL}};
const Segment SGI208709[] = {{&S66593[78], 6}, {NULL}};
const Segment SGI208710[] = {{&S75685[43], 6}, {NULL}};
const Segment SGI208711[] = {{&S75685[43], 18}, {NULL}};
const Segment SGI208712[] = {{&S75685[30], 6}, {NULL}};
const Segment SGI208713[] = {{&S75685[110], 3}, {NULL}};
const Segment SGI208714[] = {{&S75685[75], 6}, {NULL}};
const Segment SGI208715[] = {{&S66595[44], 5}, {NULL}};
const Segment SGI208716[] = {{&S66595[44], 17}, {NULL}};
const Segment SGI208717[] = {{&S66595[30], 6}, {NULL}};
const Segment SGI208718[] = {{&S66595[110], 3}, {NULL}};
const Segment SGI208719[] = {{&S75686[38], 6}, {NULL}};
const Segment SGI208720[] = {{&S75686[31], 6}, {NULL}};
const Segment SGI208721[] = {{&S66595[75], 6}, {NULL}};
const Segment SGI208722[] = {{&S66596[38], 5}, {NULL}};
const Segment SGI208723[] = {{&S66596[31], 6}, {NULL}};
const Segment SGI208724[] = {{&S66597[44], 12}, {NULL}};
const Segment SGI208725[] = {{&S66597[44], 24}, {NULL}};
const Segment SGI208726[] = {{&S66597[30], 6}, {NULL}};
const Segment SGI208727[] = {{&S66597[117], 3}, {NULL}};
const Segment SGI208728[] = {{&S66597[82], 6}, {NULL}};
const Segment SGI208729[] = {{&S66598[44], 9}, {NULL}};
const Segment SGI208730[] = {{&S66598[44], 21}, {NULL}};
const Segment SGI208731[] = {{&S66598[30], 6}, {NULL}};
const Segment SGI208732[] = {{&S66598[114], 3}, {NULL}};
const Segment SGI208733[] = {{&S66598[79], 6}, {NULL}};
const Segment SGI208734[] = {{&S81021[44], 23}, {NULL}};
const Segment SGI208735[] = {{&S81021[44], 35}, {NULL}};
const Segment SGI208736[] = {{&S81021[30], 6}, {NULL}};
const Segment SGI208737[] = {{&S81021[128], 3}, {NULL}};
const Segment SGI208738[] = {{&S81021[93], 6}, {NULL}};
const Segment SGI208739[] = {{&S81023[44], 10}, {NULL}};
const Segment SGI208740[] = {{&S81023[44], 24}, {NULL}};
const Segment SGI208741[] = {{&S81023[30], 6}, {NULL}};
const Segment SGI208742[] = {{&S81023[117], 3}, {NULL}};
const Segment SGI208743[] = {{&S81023[82], 6}, {NULL}};
const Segment SGI208744[] = {{&S75724[34], 6}, {NULL}};
const Segment SGI208745[] = {{&S75724[27], 6}, {NULL}};
const Segment SGI208746[] = {{&S75724[93], 9}, {NULL}};
const Segment SGI208747[] = {{&S75724[66], 6}, {NULL}};
const Segment SGI208748[] = {{&S66644[34], 10}, {NULL}};
const Segment SGI208749[] = {{&S66644[27], 6}, {NULL}};
const Segment SGI208750[] = {{&S66644[96], 9}, {NULL}};
const Segment SGI208751[] = {{&S66644[69], 6}, {NULL}};
const Segment SGI208752[] = {{&S86244[44], 6}, {NULL}};
const Segment SGI208753[] = {{&S86244[44], 18}, {NULL}};
const Segment SGI208754[] = {{&S86244[30], 6}, {NULL}};
const Segment SGI208755[] = {{&S86244[31], 3}, {NULL}};
const Segment SGI208756[] = {{&S86244[97], 6}, {NULL}};
const Segment SGI208757[] = {{&S86244[64], 8}, {NULL}};
const Segment SGI208758[] = {{&S86245[44], 6}, {NULL}};
const Segment SGI208759[] = {{&S86245[44], 18}, {NULL}};
const Segment SGI208760[] = {{&S86245[30], 6}, {NULL}};
const Segment SGI208761[] = {{&S86245[31], 3}, {NULL}};
const Segment SGI208762[] = {{&S86245[97], 6}, {NULL}};
const Segment SGI208763[] = {{&S86245[64], 8}, {NULL}};
const Segment SGI208764[] = {{&S76395[44], 10}, {NULL}};
const Segment SGI208765[] = {{&S76395[44], 22}, {NULL}};
const Segment SGI208766[] = {{&S76395[30], 6}, {NULL}};
const Segment SGI208767[] = {{&S76395[131], 3}, {NULL}};
const Segment SGI208768[] = {{&S76395[96], 6}, {NULL}};
const Segment SGI208769[] = {{&S76395[68], 8}, {NULL}};
const Segment SGI208770[] = {{&S86253[34], 7}, {NULL}};
const Segment SGI208771[] = {{&S86253[27], 6}, {NULL}};
const Segment SGI208772[] = {{&S86253[94], 9}, {NULL}};
const Segment SGI208773[] = {{&S86253[67], 6}, {NULL}};
const Segment SGI208774[] = {{&S81081[44], 5}, {NULL}};
const Segment SGI208775[] = {{&S81081[44], 17}, {NULL}};
const Segment SGI208776[] = {{&S81081[30], 6}, {NULL}};
const Segment SGI208777[] = {{&S81081[110], 3}, {NULL}};
const Segment SGI208778[] = {{&S81081[75], 6}, {NULL}};
const Segment SGI208779[] = {{&S86258[45], 6}, {NULL}};
const Segment SGI208780[] = {{&S86258[30], 6}, {NULL}};
const Segment SGI208781[] = {{&S86258[31], 3}, {NULL}};
const Segment SGI208782[] = {{&S86258[77], 5}, {NULL}};
const Segment SGI208783[] = {{&S86258[135], 13}, {NULL}};
const Segment SGI208784[] = {{&S76410[44], 15}, {NULL}};
const Segment SGI208785[] = {{&S76410[44], 27}, {NULL}};
const Segment SGI208786[] = {{&S76410[30], 6}, {NULL}};
const Segment SGI208787[] = {{&S76410[120], 3}, {NULL}};
const Segment SGI208788[] = {{&S76410[85], 6}, {NULL}};
const Segment SGI208789[] = {{&S86260[44], 11}, {NULL}};
const Segment SGI208790[] = {{&S86260[44], 28}, {NULL}};
const Segment SGI208791[] = {{&S86260[30], 6}, {NULL}};
const Segment SGI208792[] = {{&S86260[31], 3}, {NULL}};
const Segment SGI208793[] = {{&S86260[86], 5}, {NULL}};
const Segment SGI208794[] = {{&S66691[34], 10}, {NULL}};
const Segment SGI208795[] = {{&S66691[27], 6}, {NULL}};
const Segment SGI208796[] = {{&S66691[97], 9}, {NULL}};
const Segment SGI208797[] = {{&S66691[70], 6}, {NULL}};
const Segment SGI208798[] = {{&S76414[34], 11}, {NULL}};
const Segment SGI208799[] = {{&S76414[27], 6}, {NULL}};
const Segment SGI208800[] = {{&S66692[34], 10}, {NULL}};
const Segment SGI208801[] = {{&S66692[27], 6}, {NULL}};
const Segment SGI208802[] = {{&S76414[98], 9}, {NULL}};
const Segment SGI208803[] = {{&S76414[71], 6}, {NULL}};
const Segment SGI208804[] = {{&S66692[96], 9}, {NULL}};
const Segment SGI208805[] = {{&S66692[69], 6}, {NULL}};
const Segment SGI208806[] = {{&S76415[34], 11}, {NULL}};
const Segment SGI208807[] = {{&S76415[27], 6}, {NULL}};
const Segment SGI208808[] = {{&S76415[98], 9}, {NULL}};
const Segment SGI208809[] = {{&S76415[71], 6}, {NULL}};
const Segment SGI208810[] = {{&S76416[34], 11}, {NULL}};
const Segment SGI208811[] = {{&S76416[27], 6}, {NULL}};
const Segment SGI208812[] = {{&S76416[98], 9}, {NULL}};
const Segment SGI208813[] = {{&S76416[71], 6}, {NULL}};
const Segment SGI208814[] = {{&S86271[44], 18}, {NULL}};
const Segment SGI208815[] = {{&S86271[44], 30}, {NULL}};
const Segment SGI208816[] = {{&S86271[30], 6}, {NULL}};
const Segment SGI208817[] = {{&S86271[31], 3}, {NULL}};
const Segment SGI208818[] = {{&S86271[88], 6}, {NULL}};
const Segment SGI208819[] = {{&S86271[148], 11}, {NULL}};
const Segment SGI208820[] = {{&S76420[34], 8}, {NULL}};
const Segment SGI208821[] = {{&S76420[27], 6}, {NULL}};
const Segment SGI208822[] = {{&S76420[102], 9}, {NULL}};
const Segment SGI208823[] = {{&S76420[75], 6}, {NULL}};
const Segment SGI208824[] = {{&S86272[45], 5}, {NULL}};
const Segment SGI208825[] = {{&S86272[30], 6}, {NULL}};
const Segment SGI208826[] = {{&S86272[31], 3}, {NULL}};
const Segment SGI208827[] = {{&S86272[76], 5}, {NULL}};
const Segment SGI208828[] = {{&S86272[134], 13}, {NULL}};
const Segment SGI208829[] = {{&S86273[45], 16}, {NULL}};
const Segment SGI208830[] = {{&S86273[30], 6}, {NULL}};
const Segment SGI208831[] = {{&S86273[31], 3}, {NULL}};
const Segment SGI208832[] = {{&S86273[87], 5}, {NULL}};
const Segment SGI208833[] = {{&S86273[145], 13}, {NULL}};
const Segment SGI208834[] = {{&S81106[44], 22}, {NULL}};
const Segment SGI208835[] = {{&S81106[44], 35}, {NULL}};
const Segment SGI208836[] = {{&S81106[30], 6}, {NULL}};
const Segment SGI208837[] = {{&S81106[128], 3}, {NULL}};
const Segment SGI208838[] = {{&S81106[93], 6}, {NULL}};
const Segment SGI208839[] = {{&S86275[34], 10}, {NULL}};
const Segment SGI208840[] = {{&S86275[27], 6}, {NULL}};
const Segment SGI208841[] = {{&S86275[97], 9}, {NULL}};
const Segment SGI208842[] = {{&S86275[70], 6}, {NULL}};
const Segment SGI208843[] = {{&S86280[34], 8}, {NULL}};
const Segment SGI208844[] = {{&S86280[27], 6}, {NULL}};
const Segment SGI208845[] = {{&S86280[95], 9}, {NULL}};
const Segment SGI208846[] = {{&S86280[68], 6}, {NULL}};
const Segment SGI208847[] = {{&S86281[44], 13}, {NULL}};
const Segment SGI208848[] = {{&S86281[44], 25}, {NULL}};
const Segment SGI208849[] = {{&S86281[30], 6}, {NULL}};
const Segment SGI208850[] = {{&S86281[31], 3}, {NULL}};
const Segment SGI208851[] = {{&S86281[98], 6}, {NULL}};
const Segment SGI208852[] = {{&S86281[71], 8}, {NULL}};
const Segment SGI208853[] = {{&S86298[44], 9}, {NULL}};
const Segment SGI208854[] = {{&S86298[44], 21}, {NULL}};
const Segment SGI208855[] = {{&S86298[30], 6}, {NULL}};
const Segment SGI208856[] = {{&S86298[31], 3}, {NULL}};
const Segment SGI208857[] = {{&S86298[79], 6}, {NULL}};
const Segment SGI208858[] = {{&S81145[34], 10}, {NULL}};
const Segment SGI208859[] = {{&S81145[27], 6}, {NULL}};
const Segment SGI208860[] = {{&S86307[34], 13}, {NULL}};
const Segment SGI208861[] = {{&S81145[97], 9}, {NULL}};
const Segment SGI208862[] = {{&S81145[70], 6}, {NULL}};
const Segment SGI208863[] = {{&S86307[27], 6}, {NULL}};
const Segment SGI208864[] = {{&S76463[34], 9}, {NULL}};
const Segment SGI208865[] = {{&S86307[100], 9}, {NULL}};
const Segment SGI208866[] = {{&S86307[73], 6}, {NULL}};
const Segment SGI208867[] = {{&S76463[27], 6}, {NULL}};
const Segment SGI208868[] = {{&S76463[96], 9}, {NULL}};
const Segment SGI208869[] = {{&S76463[69], 6}, {NULL}};
const Segment SGI208870[] = {{&S81148[45], 15}, {NULL}};
const Segment SGI208871[] = {{&S81148[30], 6}, {NULL}};
const Segment SGI208872[] = {{&S81148[120], 3}, {NULL}};
const Segment SGI208873[] = {{&S81148[86], 5}, {NULL}};
const Segment SGI208874[] = {{&S81148[144], 13}, {NULL}};
const Segment SGI208875[] = {{&S81149[44], 15}, {NULL}};
const Segment SGI208876[] = {{&S81149[44], 27}, {NULL}};
const Segment SGI208877[] = {{&S81149[30], 6}, {NULL}};
const Segment SGI208878[] = {{&S81149[120], 3}, {NULL}};
const Segment SGI208879[] = {{&S81149[85], 6}, {NULL}};
const Segment SGI208880[] = {{&S81149[140], 8}, {NULL}};
const Segment SGI208881[] = {{&S81149[149], 13}, {NULL}};
const Segment SGI208882[] = {{&S76470[44], 15}, {NULL}};
const Segment SGI208883[] = {{&S76470[44], 27}, {NULL}};
const Segment SGI208884[] = {{&S76470[30], 6}, {NULL}};
const Segment SGI208885[] = {{&S76470[120], 3}, {NULL}};
const Segment SGI208886[] = {{&S76470[85], 6}, {NULL}};
const Segment SGI208887[] = {{&S76471[34], 10}, {NULL}};
const Segment SGI208888[] = {{&S76471[27], 6}, {NULL}};
const Segment SGI208889[] = {{&S76471[97], 9}, {NULL}};
const Segment SGI208890[] = {{&S76471[70], 6}, {NULL}};
const Segment SGI208891[] = {{&S81153[44], 13}, {NULL}};
const Segment SGI208892[] = {{&S81153[44], 25}, {NULL}};
const Segment SGI208893[] = {{&S81153[30], 6}, {NULL}};
const Segment SGI208894[] = {{&S81153[118], 3}, {NULL}};
const Segment SGI208895[] = {{&S81153[83], 6}, {NULL}};
const Segment SGI208896[] = {{&S76472[34], 9}, {NULL}};
const Segment SGI208897[] = {{&S76472[27], 6}, {NULL}};
const Segment SGI208898[] = {{&S76472[96], 9}, {NULL}};
const Segment SGI208899[] = {{&S76472[69], 6}, {NULL}};
const Segment SGI208900[] = {{&S76473[52], 10}, {NULL}};
const Segment SGI208901[] = {{&S76473[52], 22}, {NULL}};
const Segment SGI208902[] = {{&S76473[38], 6}, {NULL}};
const Segment SGI208903[] = {{&S76473[88], 5}, {NULL}};
const Segment SGI208904[] = {{&S76473[11], 3}, {NULL}};
const Segment SGI208905[] = {{&S81176[34], 7}, {NULL}};
const Segment SGI208906[] = {{&S81176[27], 6}, {NULL}};
const Segment SGI208907[] = {{&S81176[94], 9}, {NULL}};
const Segment SGI208908[] = {{&S81176[67], 6}, {NULL}};
const Segment SGI208909[] = {{&S86344[34], 13}, {NULL}};
const Segment SGI208910[] = {{&S86344[27], 6}, {NULL}};
const Segment SGI208911[] = {{&S86344[100], 9}, {NULL}};
const Segment SGI208912[] = {{&S86344[73], 6}, {NULL}};
const Segment SGI208913[] = {{&S86351[34], 7}, {NULL}};
const Segment SGI208914[] = {{&S86351[27], 6}, {NULL}};
const Segment SGI208915[] = {{&S86351[112], 9}, {NULL}};
const Segment SGI208916[] = {{&S86351[85], 6}, {NULL}};
const Segment SGI208917[] = {{&S76503[34], 11}, {NULL}};
const Segment SGI208918[] = {{&S76503[27], 6}, {NULL}};
const Segment SGI208919[] = {{&S76503[98], 9}, {NULL}};
const Segment SGI208920[] = {{&S76503[71], 6}, {NULL}};
const Segment SGI208921[] = {{&S86355[34], 8}, {NULL}};
const Segment SGI208922[] = {{&S86355[27], 6}, {NULL}};
const Segment SGI208923[] = {{&S86355[95], 9}, {NULL}};
const Segment SGI208924[] = {{&S86355[68], 6}, {NULL}};
const Segment SGI208925[] = {{&S76510[45], 10}, {NULL}};
const Segment SGI208926[] = {{&S76510[30], 6}, {NULL}};
const Segment SGI208927[] = {{&S76510[115], 3}, {NULL}};
const Segment SGI208928[] = {{&S76510[81], 5}, {NULL}};
const Segment SGI208929[] = {{&S76510[139], 13}, {NULL}};
const Segment SGI208930[] = {{&S81196[44], 11}, {NULL}};
const Segment SGI208931[] = {{&S81196[44], 23}, {NULL}};
const Segment SGI208932[] = {{&S81196[30], 6}, {NULL}};
const Segment SGI208933[] = {{&S81196[116], 3}, {NULL}};
const Segment SGI208934[] = {{&S86362[34], 11}, {NULL}};
const Segment SGI208935[] = {{&S81196[81], 6}, {NULL}};
const Segment SGI208936[] = {{&S86362[27], 6}, {NULL}};
const Segment SGI208937[] = {{&S81196[141], 11}, {NULL}};
const Segment SGI208938[] = {{&S76512[45], 10}, {NULL}};
const Segment SGI208939[] = {{&S86362[98], 9}, {NULL}};
const Segment SGI208940[] = {{&S76512[30], 6}, {NULL}};
const Segment SGI208941[] = {{&S86362[71], 6}, {NULL}};
const Segment SGI208942[] = {{&S76512[115], 3}, {NULL}};
const Segment SGI208943[] = {{&S76512[81], 5}, {NULL}};
const Segment SGI208944[] = {{&S76512[139], 13}, {NULL}};
const Segment SGI208945[] = {{&S86370[34], 24}, {NULL}};
const Segment SGI208946[] = {{&S86370[27], 6}, {NULL}};
const Segment SGI208947[] = {{&S86370[113], 9}, {NULL}};
const Segment SGI208948[] = {{&S86370[86], 6}, {NULL}};
const Segment SGI208949[] = {{&S86371[34], 23}, {NULL}};
const Segment SGI208950[] = {{&S86371[27], 6}, {NULL}};
const Segment SGI208951[] = {{&S86371[112], 9}, {NULL}};
const Segment SGI208952[] = {{&S86371[85], 6}, {NULL}};
const Segment SGI208953[] = {{&S76527[34], 16}, {NULL}};
const Segment SGI208954[] = {{&S76527[27], 6}, {NULL}};
const Segment SGI208955[] = {{&S76527[110], 9}, {NULL}};
const Segment SGI208956[] = {{&S76527[83], 6}, {NULL}};
const Segment SGI208957[] = {{&S86375[44], 14}, {NULL}};
const Segment SGI208958[] = {{&S86375[44], 26}, {NULL}};
const Segment SGI208959[] = {{&S86375[30], 6}, {NULL}};
const Segment SGI208960[] = {{&S86375[84], 6}, {NULL}};
const Segment SGI208961[] = {{&S86375[111], 13}, {NULL}};
const Segment SGI208962[] = {{&S86376[44], 15}, {NULL}};
const Segment SGI208963[] = {{&S86376[44], 27}, {NULL}};
const Segment SGI208964[] = {{&S86376[30], 6}, {NULL}};
const Segment SGI208965[] = {{&S86376[85], 6}, {NULL}};
const Segment SGI208966[] = {{&S86376[112], 13}, {NULL}};
const Segment SGI208967[] = {{&S76531[44], 13}, {NULL}};
const Segment SGI208968[] = {{&S76531[44], 29}, {NULL}};
const Segment SGI208969[] = {{&S76531[30], 6}, {NULL}};
const Segment SGI208970[] = {{&S76531[122], 3}, {NULL}};
const Segment SGI208971[] = {{&S76531[87], 6}, {NULL}};
const Segment SGI208972[] = {{&S86377[44], 6}, {NULL}};
const Segment SGI208973[] = {{&S86377[44], 18}, {NULL}};
const Segment SGI208974[] = {{&S86377[30], 6}, {NULL}};
const Segment SGI208975[] = {{&S86377[76], 6}, {NULL}};
const Segment SGI208976[] = {{&S86377[103], 13}, {NULL}};
const Segment SGI208977[] = {{&S76532[44], 13}, {NULL}};
const Segment SGI208978[] = {{&S76532[44], 29}, {NULL}};
const Segment SGI208979[] = {{&S76532[30], 6}, {NULL}};
const Segment SGI208980[] = {{&S76532[90], 6}, {NULL}};
const Segment SGI208981[] = {{&S81220[34], 11}, {NULL}};
const Segment SGI208982[] = {{&S81220[27], 6}, {NULL}};
const Segment SGI208983[] = {{&S76533[44], 35}, {NULL}};
const Segment SGI208984[] = {{&S76533[30], 6}, {NULL}};
const Segment SGI208985[] = {{&S81220[98], 9}, {NULL}};
const Segment SGI208986[] = {{&S81220[71], 6}, {NULL}};
const Segment SGI208987[] = {{&S76533[112], 6}, {NULL}};
const Segment SGI208988[] = {{&S86381[34], 13}, {NULL}};
const Segment SGI208989[] = {{&S76534[38], 13}, {NULL}};
const Segment SGI208990[] = {{&S86381[27], 6}, {NULL}};
const Segment SGI208991[] = {{&S76534[31], 6}, {NULL}};
const Segment SGI208992[] = {{&S86381[113], 9}, {NULL}};
const Segment SGI208993[] = {{&S86381[86], 6}, {NULL}};
const Segment SGI208994[] = {{&S76535[44], 35}, {NULL}};
const Segment SGI208995[] = {{&S76535[30], 6}, {NULL}};
const Segment SGI208996[] = {{&S86382[34], 7}, {NULL}};
const Segment SGI208997[] = {{&S86382[27], 6}, {NULL}};
const Segment SGI208998[] = {{&S76535[112], 6}, {NULL}};
const Segment SGI208999[] = {{&S86382[99], 9}, {NULL}};
const Segment SGI209000[] = {{&S86382[72], 6}, {NULL}};
const Segment SGI209001[] = {{&S81226[34], 7}, {NULL}};
const Segment SGI209002[] = {{&S81226[27], 6}, {NULL}};
const Segment SGI209003[] = {{&S76536[44], 39}, {NULL}};
const Segment SGI209004[] = {{&S76536[30], 6}, {NULL}};
const Segment SGI209005[] = {{&S81226[94], 9}, {NULL}};
const Segment SGI209006[] = {{&S81226[67], 6}, {NULL}};
const Segment SGI209007[] = {{&S76536[116], 6}, {NULL}};
const Segment SGI209008[] = {{&S86384[34], 8}, {NULL}};
const Segment SGI209009[] = {{&S86384[27], 6}, {NULL}};
const Segment SGI209010[] = {{&S86384[95], 9}, {NULL}};
const Segment SGI209011[] = {{&S86384[68], 6}, {NULL}};
const Segment SGI209012[] = {{&S86387[44], 16}, {NULL}};
const Segment SGI209013[] = {{&S86387[44], 28}, {NULL}};
const Segment SGI209014[] = {{&S86387[30], 6}, {NULL}};
const Segment SGI209015[] = {{&S86387[31], 3}, {NULL}};
const Segment SGI209016[] = {{&S86387[86], 6}, {NULL}};
const Segment SGI209017[] = {{&S76542[34], 7}, {NULL}};
const Segment SGI209018[] = {{&S76542[27], 6}, {NULL}};
const Segment SGI209019[] = {{&S76542[104], 9}, {NULL}};
const Segment SGI209020[] = {{&S76542[77], 6}, {NULL}};
const Segment SGI209021[] = {{&S86390[44], 14}, {NULL}};
const Segment SGI209022[] = {{&S86390[44], 26}, {NULL}};
const Segment SGI209023[] = {{&S86390[30], 6}, {NULL}};
const Segment SGI209024[] = {{&S86390[31], 3}, {NULL}};
const Segment SGI209025[] = {{&S86390[84], 5}, {NULL}};
const Segment SGI209026[] = {{&S87829[34], 7}, {NULL}};
const Segment SGI209027[] = {{&S87829[27], 6}, {NULL}};
const Segment SGI209028[] = {{&S87829[67], 6}, {NULL}};
const Segment SGI209029[] = {{&S86396[52], 12}, {NULL}};
const Segment SGI209030[] = {{&S86396[52], 21}, {NULL}};
const Segment SGI209031[] = {{&S86396[38], 6}, {NULL}};
const Segment SGI209032[] = {{&S86396[87], 5}, {NULL}};
const Segment SGI209033[] = {{&S86396[0], 14}, {NULL}};
const Segment SGI209034[] = {{&S86397[52], 4}, {NULL}};
const Segment SGI209035[] = {{&S86397[52], 13}, {NULL}};
const Segment SGI209036[] = {{&S86397[38], 6}, {NULL}};
const Segment SGI209037[] = {{&S86397[79], 5}, {NULL}};
const Segment SGI209038[] = {{&S86397[0], 14}, {NULL}};
const Segment SGI209039[] = {{&S86399[34], 7}, {NULL}};
const Segment SGI209040[] = {{&S86399[27], 6}, {NULL}};
const Segment SGI209041[] = {{&S86399[112], 9}, {NULL}};
const Segment SGI209042[] = {{&S86399[85], 6}, {NULL}};
const Segment SGI209043[] = {{&S86399[153], 8}, {NULL}};
const Segment SGI209044[] = {{&S86405[52], 10}, {NULL}};
const Segment SGI209045[] = {{&S86405[52], 22}, {NULL}};
const Segment SGI209046[] = {{&S86405[38], 6}, {NULL}};
const Segment SGI209047[] = {{&S86405[88], 5}, {NULL}};
const Segment SGI209048[] = {{&S86405[0], 14}, {NULL}};
const Segment SGI209049[] = {{&S86406[52], 13}, {NULL}};
const Segment SGI209050[] = {{&S86406[52], 25}, {NULL}};
const Segment SGI209051[] = {{&S86406[38], 6}, {NULL}};
const Segment SGI209052[] = {{&S86406[91], 5}, {NULL}};
const Segment SGI209053[] = {{&S86406[0], 14}, {NULL}};
const Segment SGI209054[] = {{&S76567[44], 10}, {NULL}};
const Segment SGI209055[] = {{&S76567[44], 22}, {NULL}};
const Segment SGI209056[] = {{&S76567[30], 6}, {NULL}};
const Segment SGI209057[] = {{&S76567[115], 3}, {NULL}};
const Segment SGI209058[] = {{&S76567[80], 6}, {NULL}};
const Segment SGI209059[] = {{&S76567[140], 13}, {NULL}};
const Segment SGI209060[] = {{&S81265[59], 10}, {NULL}};
const Segment SGI209061[] = {{&S81265[59], 22}, {NULL}};
const Segment SGI209062[] = {{&S81265[45], 6}, {NULL}};
const Segment SGI209063[] = {{&S81265[130], 3}, {NULL}};
const Segment SGI209064[] = {{&S81265[95], 6}, {NULL}};
const Segment SGI209065[] = {{&S81265[0], 14}, {NULL}};
const Segment SGI209066[] = {{&S86413[34], 10}, {NULL}};
const Segment SGI209067[] = {{&S86413[27], 6}, {NULL}};
const Segment SGI209068[] = {{&S86413[97], 9}, {NULL}};
const Segment SGI209069[] = {{&S86413[70], 6}, {NULL}};
const Segment SGI209070[] = {{&S86413[138], 8}, {NULL}};
const Segment SGI209071[] = {{&S81267[59], 10}, {NULL}};
const Segment SGI209072[] = {{&S81267[59], 22}, {NULL}};
const Segment SGI209073[] = {{&S81267[45], 6}, {NULL}};
const Segment SGI209074[] = {{&S81267[130], 3}, {NULL}};
const Segment SGI209075[] = {{&S81267[95], 6}, {NULL}};
const Segment SGI209076[] = {{&S81267[0], 14}, {NULL}};
const Segment SGI209077[] = {{&S66886[34], 7}, {NULL}};
const Segment SGI209078[] = {{&S66886[27], 6}, {NULL}};
const Segment SGI209079[] = {{&S76575[44], 17}, {NULL}};
const Segment SGI209080[] = {{&S76575[44], 29}, {NULL}};
const Segment SGI209081[] = {{&S76575[30], 6}, {NULL}};
const Segment SGI209082[] = {{&S66886[100], 9}, {NULL}};
const Segment SGI209083[] = {{&S66886[73], 6}, {NULL}};
const Segment SGI209084[] = {{&S76575[138], 3}, {NULL}};
const Segment SGI209085[] = {{&S76575[103], 6}, {NULL}};
const Segment SGI209086[] = {{&S76575[75], 8}, {NULL}};
const Segment SGI209087[] = {{&S76576[44], 5}, {NULL}};
const Segment SGI209088[] = {{&S76576[44], 17}, {NULL}};
const Segment SGI209089[] = {{&S76576[30], 6}, {NULL}};
const Segment SGI209090[] = {{&S81271[45], 10}, {NULL}};
const Segment SGI209091[] = {{&S81271[30], 6}, {NULL}};
const Segment SGI209092[] = {{&S76576[152], 3}, {NULL}};
const Segment SGI209093[] = {{&S76576[117], 6}, {NULL}};
const Segment SGI209094[] = {{&S81271[115], 3}, {NULL}};
const Segment SGI209095[] = {{&S76576[63], 8}, {NULL}};
const Segment SGI209096[] = {{&S81271[81], 5}, {NULL}};
const Segment SGI209097[] = {{&S81271[139], 13}, {NULL}};
const Segment SGI209098[] = {{&S66894[44], 12}, {NULL}};
const Segment SGI209099[] = {{&S66894[44], 24}, {NULL}};
const Segment SGI209100[] = {{&S66894[30], 6}, {NULL}};
const Segment SGI209101[] = {{&S66894[135], 3}, {NULL}};
const Segment SGI209102[] = {{&S66894[100], 6}, {NULL}};
const Segment SGI209103[] = {{&S66894[70], 8}, {NULL}};
const Segment SGI209104[] = {{&S86426[44], 7}, {NULL}};
const Segment SGI209105[] = {{&S86426[44], 17}, {NULL}};
const Segment SGI209106[] = {{&S86426[30], 6}, {NULL}};
const Segment SGI209107[] = {{&S86426[31], 3}, {NULL}};
const Segment SGI209108[] = {{&S86426[75], 6}, {NULL}};
const Segment SGI209109[] = {{&S66896[44], 9}, {NULL}};
const Segment SGI209110[] = {{&S66896[44], 20}, {NULL}};
const Segment SGI209111[] = {{&S66896[30], 6}, {NULL}};
const Segment SGI209112[] = {{&S66896[113], 3}, {NULL}};
const Segment SGI209113[] = {{&S66896[78], 6}, {NULL}};
const Segment SGI209114[] = {{&S66896[131], 11}, {NULL}};
const Segment SGI209115[] = {{&S76585[34], 10}, {NULL}};
const Segment SGI209116[] = {{&S76585[27], 6}, {NULL}};
const Segment SGI209117[] = {{&S76585[97], 9}, {NULL}};
const Segment SGI209118[] = {{&S76585[70], 6}, {NULL}};
const Segment SGI209119[] = {{&S66897[44], 9}, {NULL}};
const Segment SGI209120[] = {{&S66897[44], 20}, {NULL}};
const Segment SGI209121[] = {{&S66897[30], 6}, {NULL}};
const Segment SGI209122[] = {{&S81282[34], 12}, {NULL}};
const Segment SGI209123[] = {{&S66897[113], 3}, {NULL}};
const Segment SGI209124[] = {{&S81282[27], 6}, {NULL}};
const Segment SGI209125[] = {{&S66897[78], 6}, {NULL}};
const Segment SGI209126[] = {{&S66897[132], 12}, {NULL}};
const Segment SGI209127[] = {{&S76586[34], 11}, {NULL}};
const Segment SGI209128[] = {{&S76586[27], 6}, {NULL}};
const Segment SGI209129[] = {{&S81282[99], 9}, {NULL}};
const Segment SGI209130[] = {{&S81282[72], 6}, {NULL}};
const Segment SGI209131[] = {{&S76586[98], 9}, {NULL}};
const Segment SGI209132[] = {{&S76586[71], 6}, {NULL}};
const Segment SGI209133[] = {{&S76587[34], 11}, {NULL}};
const Segment SGI209134[] = {{&S76587[27], 6}, {NULL}};
const Segment SGI209135[] = {{&S76587[98], 9}, {NULL}};
const Segment SGI209136[] = {{&S76587[71], 6}, {NULL}};
const Segment SGI209137[] = {{&S66901[44], 10}, {NULL}};
const Segment SGI209138[] = {{&S66901[44], 21}, {NULL}};
const Segment SGI209139[] = {{&S66901[30], 6}, {NULL}};
const Segment SGI209140[] = {{&S66901[114], 3}, {NULL}};
const Segment SGI209141[] = {{&S66901[79], 6}, {NULL}};
const Segment SGI209142[] = {{&S66901[149], 3}, {NULL}};
const Segment SGI209143[] = {{&S76589[34], 5}, {NULL}};
const Segment SGI209144[] = {{&S76589[27], 6}, {NULL}};
const Segment SGI209145[] = {{&S76589[92], 9}, {NULL}};
const Segment SGI209146[] = {{&S76589[65], 6}, {NULL}};
const Segment SGI209147[] = {{&S66902[34], 7}, {NULL}};
const Segment SGI209148[] = {{&S66902[27], 6}, {NULL}};
const Segment SGI209149[] = {{&S81288[44], 9}, {NULL}};
const Segment SGI209150[] = {{&S81288[44], 21}, {NULL}};
const Segment SGI209151[] = {{&S81288[30], 6}, {NULL}};
const Segment SGI209152[] = {{&S66902[93], 9}, {NULL}};
const Segment SGI209153[] = {{&S66902[66], 6}, {NULL}};
const Segment SGI209154[] = {{&S81288[114], 3}, {NULL}};
const Segment SGI209155[] = {{&S81288[79], 6}, {NULL}};
const Segment SGI209156[] = {{&S66903[34], 11}, {NULL}};
const Segment SGI209157[] = {{&S66903[27], 6}, {NULL}};
const Segment SGI209158[] = {{&S66903[97], 9}, {NULL}};
const Segment SGI209159[] = {{&S66903[70], 6}, {NULL}};
const Segment SGI209160[] = {{&S86443[44], 7}, {NULL}};
const Segment SGI209161[] = {{&S86443[44], 19}, {NULL}};
const Segment SGI209162[] = {{&S86443[30], 6}, {NULL}};
const Segment SGI209163[] = {{&S86443[31], 3}, {NULL}};
const Segment SGI209164[] = {{&S86443[128], 6}, {NULL}};
const Segment SGI209165[] = {{&S86443[65], 8}, {NULL}};
const Segment SGI209166[] = {{&S86444[44], 12}, {NULL}};
const Segment SGI209167[] = {{&S86444[44], 24}, {NULL}};
const Segment SGI209168[] = {{&S86444[30], 6}, {NULL}};
const Segment SGI209169[] = {{&S86444[31], 3}, {NULL}};
const Segment SGI209170[] = {{&S86444[131], 6}, {NULL}};
const Segment SGI209171[] = {{&S86444[70], 8}, {NULL}};
const Segment SGI209172[] = {{&S86445[44], 7}, {NULL}};
const Segment SGI209173[] = {{&S86445[44], 19}, {NULL}};
const Segment SGI209174[] = {{&S86445[30], 6}, {NULL}};
const Segment SGI209175[] = {{&S86445[31], 3}, {NULL}};
const Segment SGI209176[] = {{&S86445[92], 6}, {NULL}};
const Segment SGI209177[] = {{&S86445[65], 8}, {NULL}};
const Segment SGI209178[] = {{&S86450[45], 7}, {NULL}};
const Segment SGI209179[] = {{&S86450[30], 6}, {NULL}};
const Segment SGI209180[] = {{&S86450[31], 3}, {NULL}};
const Segment SGI209181[] = {{&S81304[34], 17}, {NULL}};
const Segment SGI209182[] = {{&S86450[83], 5}, {NULL}};
const Segment SGI209183[] = {{&S81304[27], 6}, {NULL}};
const Segment SGI209184[] = {{&S86450[141], 13}, {NULL}};
const Segment SGI209185[] = {{&S81304[104], 9}, {NULL}};
const Segment SGI209186[] = {{&S81304[77], 6}, {NULL}};
const Segment SGI209187[] = {{&S86454[34], 10}, {NULL}};
const Segment SGI209188[] = {{&S86454[27], 6}, {NULL}};
const Segment SGI209189[] = {{&S86454[97], 9}, {NULL}};
const Segment SGI209190[] = {{&S86454[70], 6}, {NULL}};
const Segment SGI209191[] = {{&S86456[59], 10}, {NULL}};
const Segment SGI209192[] = {{&S86456[59], 22}, {NULL}};
const Segment SGI209193[] = {{&S86456[45], 6}, {NULL}};
const Segment SGI209194[] = {{&S86456[46], 3}, {NULL}};
const Segment SGI209195[] = {{&S86456[95], 6}, {NULL}};
const Segment SGI209196[] = {{&S86456[0], 14}, {NULL}};
const Segment SGI209197[] = {{&S66932[34], 6}, {NULL}};
const Segment SGI209198[] = {{&S86458[57], 10}, {NULL}};
const Segment SGI209199[] = {{&S66932[27], 6}, {NULL}};
const Segment SGI209200[] = {{&S86458[45], 6}, {NULL}};
const Segment SGI209201[] = {{&S86458[46], 3}, {NULL}};
const Segment SGI209202[] = {{&S66932[93], 9}, {NULL}};
const Segment SGI209203[] = {{&S66932[66], 6}, {NULL}};
const Segment SGI209204[] = {{&S86458[93], 6}, {NULL}};
const Segment SGI209205[] = {{&S86458[0], 14}, {NULL}};
const Segment SGI209206[] = {{&S86459[59], 11}, {NULL}};
const Segment SGI209207[] = {{&S86459[59], 23}, {NULL}};
const Segment SGI209208[] = {{&S86459[45], 6}, {NULL}};
const Segment SGI209209[] = {{&S86459[46], 3}, {NULL}};
const Segment SGI209210[] = {{&S86459[96], 6}, {NULL}};
const Segment SGI209211[] = {{&S86459[0], 14}, {NULL}};
const Segment SGI209212[] = {{&S86460[34], 9}, {NULL}};
const Segment SGI209213[] = {{&S86460[27], 6}, {NULL}};
const Segment SGI209214[] = {{&S86460[96], 9}, {NULL}};
const Segment SGI209215[] = {{&S86460[69], 6}, {NULL}};
const Segment SGI209216[] = {{&S86461[34], 5}, {NULL}};
const Segment SGI209217[] = {{&S86461[27], 6}, {NULL}};
const Segment SGI209218[] = {{&S86461[96], 9}, {NULL}};
const Segment SGI209219[] = {{&S86461[69], 6}, {NULL}};
const Segment SGI209220[] = {{&S66945[34], 10}, {NULL}};
const Segment SGI209221[] = {{&S66945[27], 6}, {NULL}};
const Segment SGI209222[] = {{&S66945[96], 9}, {NULL}};
const Segment SGI209223[] = {{&S66945[69], 6}, {NULL}};
const Segment SGI209224[] = {{&S86469[44], 21}, {NULL}};
const Segment SGI209225[] = {{&S86469[44], 33}, {NULL}};
const Segment SGI209226[] = {{&S86469[30], 6}, {NULL}};
const Segment SGI209227[] = {{&S86469[31], 3}, {NULL}};
const Segment SGI209228[] = {{&S86469[91], 6}, {NULL}};
const Segment SGI209229[] = {{&S86471[34], 24}, {NULL}};
const Segment SGI209230[] = {{&S86471[27], 6}, {NULL}};
const Segment SGI209231[] = {{&S86471[111], 9}, {NULL}};
const Segment SGI209232[] = {{&S86471[84], 6}, {NULL}};
const Segment SGI209233[] = {{&S81338[44], 13}, {NULL}};
const Segment SGI209234[] = {{&S81338[44], 25}, {NULL}};
const Segment SGI209235[] = {{&S81338[30], 6}, {NULL}};
const Segment SGI209236[] = {{&S86472[27], 6}, {NULL}};
const Segment SGI209237[] = {{&S81338[118], 3}, {NULL}};
const Segment SGI209238[] = {{&S81338[83], 6}, {NULL}};
const Segment SGI209239[] = {{&S86472[103], 9}, {NULL}};
const Segment SGI209240[] = {{&S86472[76], 6}, {NULL}};
const Segment SGI209241[] = {{&S86473[34], 8}, {NULL}};
const Segment SGI209242[] = {{&S86473[27], 6}, {NULL}};
const Segment SGI209243[] = {{&S86473[101], 9}, {NULL}};
const Segment SGI209244[] = {{&S86473[74], 6}, {NULL}};
const Segment SGI209245[] = {{&S81346[44], 7}, {NULL}};
const Segment SGI209246[] = {{&S81346[44], 19}, {NULL}};
const Segment SGI209247[] = {{&S81346[30], 6}, {NULL}};
const Segment SGI209248[] = {{&S81346[112], 3}, {NULL}};
const Segment SGI209249[] = {{&S81346[77], 6}, {NULL}};
const Segment SGI209250[] = {{&S86480[34], 7}, {NULL}};
const Segment SGI209251[] = {{&S86480[27], 6}, {NULL}};
const Segment SGI209252[] = {{&S86480[94], 9}, {NULL}};
const Segment SGI209253[] = {{&S86480[67], 6}, {NULL}};
const Segment SGI209254[] = {{&S81349[44], 10}, {NULL}};
const Segment SGI209255[] = {{&S81349[44], 22}, {NULL}};
const Segment SGI209256[] = {{&S81349[30], 6}, {NULL}};
const Segment SGI209257[] = {{&S81349[115], 3}, {NULL}};
const Segment SGI209258[] = {{&S81349[80], 6}, {NULL}};
const Segment SGI209259[] = {{&S81349[134], 12}, {NULL}};
const Segment SGI209260[] = {{&S86481[34], 5}, {NULL}};
const Segment SGI209261[] = {{&S86481[27], 6}, {NULL}};
const Segment SGI209262[] = {{&S86481[92], 9}, {NULL}};
const Segment SGI209263[] = {{&S86481[65], 6}, {NULL}};
const Segment SGI209264[] = {{&S81350[44], 10}, {NULL}};
const Segment SGI209265[] = {{&S81350[44], 22}, {NULL}};
const Segment SGI209266[] = {{&S81350[30], 6}, {NULL}};
const Segment SGI209267[] = {{&S81350[115], 3}, {NULL}};
const Segment SGI209268[] = {{&S81350[80], 6}, {NULL}};
const Segment SGI209269[] = {{&S86482[34], 13}, {NULL}};
const Segment SGI209270[] = {{&S81350[133], 11}, {NULL}};
const Segment SGI209271[] = {{&S86482[27], 6}, {NULL}};
const Segment SGI209272[] = {{&S86482[100], 9}, {NULL}};
const Segment SGI209273[] = {{&S86482[73], 6}, {NULL}};
const Segment SGI209274[] = {{&S66974[38], 7}, {NULL}};
const Segment SGI209275[] = {{&S66974[31], 6}, {NULL}};
const Segment SGI209276[] = {{&S81353[34], 13}, {NULL}};
const Segment SGI209277[] = {{&S81353[27], 6}, {NULL}};
const Segment SGI209278[] = {{&S76664[44], 17}, {NULL}};
const Segment SGI209279[] = {{&S76664[44], 29}, {NULL}};
const Segment SGI209280[] = {{&S81353[100], 9}, {NULL}};
const Segment SGI209281[] = {{&S76664[30], 6}, {NULL}};
const Segment SGI209282[] = {{&S81353[73], 6}, {NULL}};
const Segment SGI209283[] = {{&S66975[44], 7}, {NULL}};
const Segment SGI209284[] = {{&S76664[122], 3}, {NULL}};
const Segment SGI209285[] = {{&S66975[44], 25}, {NULL}};
const Segment SGI209286[] = {{&S66975[30], 6}, {NULL}};
const Segment SGI209287[] = {{&S76664[87], 6}, {NULL}};
const Segment SGI209288[] = {{&S66975[118], 3}, {NULL}};
const Segment SGI209289[] = {{&S86486[44], 16}, {NULL}};
const Segment SGI209290[] = {{&S66975[83], 6}, {NULL}};
const Segment SGI209291[] = {{&S86486[44], 28}, {NULL}};
const Segment SGI209292[] = {{&S86486[30], 6}, {NULL}};
const Segment SGI209293[] = {{&S86486[31], 3}, {NULL}};
const Segment SGI209294[] = {{&S86486[86], 6}, {NULL}};
const Segment SGI209295[] = {{&S76665[44], 12}, {NULL}};
const Segment SGI209296[] = {{&S76665[44], 34}, {NULL}};
const Segment SGI209297[] = {{&S66976[44], 7}, {NULL}};
const Segment SGI209298[] = {{&S76665[30], 6}, {NULL}};
const Segment SGI209299[] = {{&S66976[44], 27}, {NULL}};
const Segment SGI209300[] = {{&S66976[30], 6}, {NULL}};
const Segment SGI209301[] = {{&S76665[127], 3}, {NULL}};
const Segment SGI209302[] = {{&S66976[120], 3}, {NULL}};
const Segment SGI209303[] = {{&S76665[92], 6}, {NULL}};
const Segment SGI209304[] = {{&S66976[85], 6}, {NULL}};
const Segment SGI209305[] = {{&S76665[151], 10}, {NULL}};
const Segment SGI209306[] = {{&S86488[44], 14}, {NULL}};
const Segment SGI209307[] = {{&S86488[44], 31}, {NULL}};
const Segment SGI209308[] = {{&S76666[44], 7}, {NULL}};
const Segment SGI209309[] = {{&S86488[30], 6}, {NULL}};
const Segment SGI209310[] = {{&S76666[44], 19}, {NULL}};
const Segment SGI209311[] = {{&S76666[30], 6}, {NULL}};
const Segment SGI209312[] = {{&S76666[112], 3}, {NULL}};
const Segment SGI209313[] = {{&S86488[89], 6}, {NULL}};
const Segment SGI209314[] = {{&S76666[77], 6}, {NULL}};
const Segment SGI209315[] = {{&S86488[116], 13}, {NULL}};
const Segment SGI209316[] = {{&S81359[34], 13}, {NULL}};
const Segment SGI209317[] = {{&S81359[27], 6}, {NULL}};
const Segment SGI209318[] = {{&S81359[100], 9}, {NULL}};
const Segment SGI209319[] = {{&S81359[73], 6}, {NULL}};
const Segment SGI209320[] = {{&S76667[34], 9}, {NULL}};
const Segment SGI209321[] = {{&S76667[27], 6}, {NULL}};
const Segment SGI209322[] = {{&S76667[96], 9}, {NULL}};
const Segment SGI209323[] = {{&S76667[69], 6}, {NULL}};
const Segment SGI209324[] = {{&S76668[34], 9}, {NULL}};
const Segment SGI209325[] = {{&S76668[27], 6}, {NULL}};
const Segment SGI209326[] = {{&S76668[96], 9}, {NULL}};
const Segment SGI209327[] = {{&S76668[69], 6}, {NULL}};
const Segment SGI209328[] = {{&S86492[44], 7}, {NULL}};
const Segment SGI209329[] = {{&S86492[44], 19}, {NULL}};
const Segment SGI209330[] = {{&S86492[30], 6}, {NULL}};
const Segment SGI209331[] = {{&S86492[31], 3}, {NULL}};
const Segment SGI209332[] = {{&S86492[77], 6}, {NULL}};
const Segment SGI209333[] = {{&S86492[131], 12}, {NULL}};
const Segment SGI209334[] = {{&S76670[44], 20}, {NULL}};
const Segment SGI209335[] = {{&S76670[44], 32}, {NULL}};
const Segment SGI209336[] = {{&S76670[30], 6}, {NULL}};
const Segment SGI209337[] = {{&S76670[125], 3}, {NULL}};
const Segment SGI209338[] = {{&S76670[90], 6}, {NULL}};
const Segment SGI209339[] = {{&S86493[44], 7}, {NULL}};
const Segment SGI209340[] = {{&S86493[44], 19}, {NULL}};
const Segment SGI209341[] = {{&S86493[30], 6}, {NULL}};
const Segment SGI209342[] = {{&S86493[31], 3}, {NULL}};
const Segment SGI209343[] = {{&S86493[77], 6}, {NULL}};
const Segment SGI209344[] = {{&S86493[130], 11}, {NULL}};
const Segment SGI209345[] = {{&S86496[34], 14}, {NULL}};
const Segment SGI209346[] = {{&S86496[27], 6}, {NULL}};
const Segment SGI209347[] = {{&S86496[106], 9}, {NULL}};
const Segment SGI209348[] = {{&S86496[79], 6}, {NULL}};
const Segment SGI209349[] = {{&S86499[44], 14}, {NULL}};
const Segment SGI209350[] = {{&S86499[44], 26}, {NULL}};
const Segment SGI209351[] = {{&S86499[30], 6}, {NULL}};
const Segment SGI209352[] = {{&S86499[84], 6}, {NULL}};
const Segment SGI209353[] = {{&S86499[111], 13}, {NULL}};
const Segment SGI209354[] = {{&S86501[44], 5}, {NULL}};
const Segment SGI209355[] = {{&S86501[44], 17}, {NULL}};
const Segment SGI209356[] = {{&S86501[30], 6}, {NULL}};
const Segment SGI209357[] = {{&S86501[31], 3}, {NULL}};
const Segment SGI209358[] = {{&S86501[75], 6}, {NULL}};
const Segment SGI209359[] = {{&S86501[135], 13}, {NULL}};
const Segment SGI209360[] = {{&S86510[34], 6}, {NULL}};
const Segment SGI209361[] = {{&S86510[27], 6}, {NULL}};
const Segment SGI209362[] = {{&S86510[93], 9}, {NULL}};
const Segment SGI209363[] = {{&S86510[66], 6}, {NULL}};
const Segment SGI209364[] = {{&S86512[34], 11}, {NULL}};
const Segment SGI209365[] = {{&S86512[27], 6}, {NULL}};
const Segment SGI209366[] = {{&S86512[98], 9}, {NULL}};
const Segment SGI209367[] = {{&S86512[71], 6}, {NULL}};
const Segment SGI209368[] = {{&S86514[44], 5}, {NULL}};
const Segment SGI209369[] = {{&S86514[44], 17}, {NULL}};
const Segment SGI209370[] = {{&S86514[30], 6}, {NULL}};
const Segment SGI209371[] = {{&S86514[75], 6}, {NULL}};
const Segment SGI209372[] = {{&S86514[102], 13}, {NULL}};
const Segment SGI209373[] = {{&S86515[34], 10}, {NULL}};
const Segment SGI209374[] = {{&S86515[27], 6}, {NULL}};
const Segment SGI209375[] = {{&S86515[97], 9}, {NULL}};
const Segment SGI209376[] = {{&S86515[70], 6}, {NULL}};
const Segment SGI209377[] = {{&S76706[44], 10}, {NULL}};
const Segment SGI209378[] = {{&S76706[44], 22}, {NULL}};
const Segment SGI209379[] = {{&S76706[30], 6}, {NULL}};
const Segment SGI209380[] = {{&S67025[34], 10}, {NULL}};
const Segment SGI209381[] = {{&S76706[131], 3}, {NULL}};
const Segment SGI209382[] = {{&S67025[27], 6}, {NULL}};
const Segment SGI209383[] = {{&S76706[96], 6}, {NULL}};
const Segment SGI209384[] = {{&S76706[68], 8}, {NULL}};
const Segment SGI209385[] = {{&S67025[96], 9}, {NULL}};
const Segment SGI209386[] = {{&S67025[69], 6}, {NULL}};
const Segment SGI209387[] = {{&S67027[34], 10}, {NULL}};
const Segment SGI209388[] = {{&S67027[27], 6}, {NULL}};
const Segment SGI209389[] = {{&S67027[96], 9}, {NULL}};
const Segment SGI209390[] = {{&S67027[69], 6}, {NULL}};
const Segment SGI209391[] = {{&S76710[42], 10}, {NULL}};
const Segment SGI209392[] = {{&S76710[30], 6}, {NULL}};
const Segment SGI209393[] = {{&S86520[34], 13}, {NULL}};
const Segment SGI209394[] = {{&S86520[27], 6}, {NULL}};
const Segment SGI209395[] = {{&S76710[129], 3}, {NULL}};
const Segment SGI209396[] = {{&S76710[94], 6}, {NULL}};
const Segment SGI209397[] = {{&S76710[66], 8}, {NULL}};
const Segment SGI209398[] = {{&S86520[100], 9}, {NULL}};
const Segment SGI209399[] = {{&S86520[73], 6}, {NULL}};
const Segment SGI209400[] = {{&S67031[34], 10}, {NULL}};
const Segment SGI209401[] = {{&S67031[27], 6}, {NULL}};
const Segment SGI209402[] = {{&S67031[97], 9}, {NULL}};
const Segment SGI209403[] = {{&S67031[70], 6}, {NULL}};
const Segment SGI209404[] = {{&S86525[34], 10}, {NULL}};
const Segment SGI209405[] = {{&S86525[27], 6}, {NULL}};
const Segment SGI209406[] = {{&S86525[97], 9}, {NULL}};
const Segment SGI209407[] = {{&S86525[70], 6}, {NULL}};
const Segment SGI209408[] = {{&S76718[34], 8}, {NULL}};
const Segment SGI209409[] = {{&S76718[27], 6}, {NULL}};
const Segment SGI209410[] = {{&S81411[44], 23}, {NULL}};
const Segment SGI209411[] = {{&S81411[44], 35}, {NULL}};
const Segment SGI209412[] = {{&S76718[101], 9}, {NULL}};
const Segment SGI209413[] = {{&S81411[30], 6}, {NULL}};
const Segment SGI209414[] = {{&S76718[74], 6}, {NULL}};
const Segment SGI209415[] = {{&S81411[128], 3}, {NULL}};
const Segment SGI209416[] = {{&S81411[93], 6}, {NULL}};
const Segment SGI209417[] = {{&S81412[44], 14}, {NULL}};
const Segment SGI209418[] = {{&S81412[44], 26}, {NULL}};
const Segment SGI209419[] = {{&S81412[30], 6}, {NULL}};
const Segment SGI209420[] = {{&S81412[119], 3}, {NULL}};
const Segment SGI209421[] = {{&S75747[34], 10}, {NULL}};
const Segment SGI209422[] = {{&S81412[84], 6}, {NULL}};
const Segment SGI209423[] = {{&S75747[27], 6}, {NULL}};
const Segment SGI209424[] = {{&S81413[44], 12}, {NULL}};
const Segment SGI209425[] = {{&S81413[44], 24}, {NULL}};
const Segment SGI209426[] = {{&S81413[30], 6}, {NULL}};
const Segment SGI209427[] = {{&S81413[117], 3}, {NULL}};
const Segment SGI209428[] = {{&S81413[82], 6}, {NULL}};
const Segment SGI209429[] = {{&S76723[34], 10}, {NULL}};
const Segment SGI209430[] = {{&S76723[27], 6}, {NULL}};
const Segment SGI209431[] = {{&S76723[97], 9}, {NULL}};
const Segment SGI209432[] = {{&S76723[70], 6}, {NULL}};
const Segment SGI209433[] = {{&S76724[34], 11}, {NULL}};
const Segment SGI209434[] = {{&S76724[27], 6}, {NULL}};
const Segment SGI209435[] = {{&S76724[98], 9}, {NULL}};
const Segment SGI209436[] = {{&S76724[71], 6}, {NULL}};
const Segment SGI209437[] = {{&S76729[34], 18}, {NULL}};
const Segment SGI209438[] = {{&S76729[27], 6}, {NULL}};
const Segment SGI209439[] = {{&S76729[105], 9}, {NULL}};
const Segment SGI209440[] = {{&S76729[78], 6}, {NULL}};
const Segment SGI209441[] = {{&S81422[34], 6}, {NULL}};
const Segment SGI209442[] = {{&S81422[27], 6}, {NULL}};
const Segment SGI209443[] = {{&S81422[93], 9}, {NULL}};
const Segment SGI209444[] = {{&S81422[66], 6}, {NULL}};
const Segment SGI209445[] = {{&S67047[44], 7}, {NULL}};
const Segment SGI209446[] = {{&S67047[44], 18}, {NULL}};
const Segment SGI209447[] = {{&S67047[30], 6}, {NULL}};
const Segment SGI209448[] = {{&S67047[111], 3}, {NULL}};
const Segment SGI209449[] = {{&S67047[76], 6}, {NULL}};
const Segment SGI209450[] = {{&S76734[34], 10}, {NULL}};
const Segment SGI209451[] = {{&S76734[27], 6}, {NULL}};
const Segment SGI209452[] = {{&S76734[97], 9}, {NULL}};
const Segment SGI209453[] = {{&S76734[70], 6}, {NULL}};
const Segment SGI209454[] = {{&S81427[44], 16}, {NULL}};
const Segment SGI209455[] = {{&S81427[44], 28}, {NULL}};
const Segment SGI209456[] = {{&S81427[30], 6}, {NULL}};
const Segment SGI209457[] = {{&S81427[121], 3}, {NULL}};
const Segment SGI209458[] = {{&S81427[86], 6}, {NULL}};
const Segment SGI209459[] = {{&S86554[44], 17}, {NULL}};
const Segment SGI209460[] = {{&S86554[44], 29}, {NULL}};
const Segment SGI209461[] = {{&S86554[30], 6}, {NULL}};
const Segment SGI209462[] = {{&S86554[31], 3}, {NULL}};
const Segment SGI209463[] = {{&S86554[87], 6}, {NULL}};
const Segment SGI209464[] = {{&S86555[44], 17}, {NULL}};
const Segment SGI209465[] = {{&S86555[44], 29}, {NULL}};
const Segment SGI209466[] = {{&S86555[30], 6}, {NULL}};
const Segment SGI209467[] = {{&S86555[31], 3}, {NULL}};
const Segment SGI209468[] = {{&S86555[87], 6}, {NULL}};
const Segment SGI209469[] = {{&S76742[45], 10}, {NULL}};
const Segment SGI209470[] = {{&S76742[30], 6}, {NULL}};
const Segment SGI209471[] = {{&S76742[115], 3}, {NULL}};
const Segment SGI209472[] = {{&S76742[81], 5}, {NULL}};
const Segment SGI209473[] = {{&S76742[139], 13}, {NULL}};
const Segment SGI209474[] = {{&S81436[34], 5}, {NULL}};
const Segment SGI209475[] = {{&S81436[27], 6}, {NULL}};
const Segment SGI209476[] = {{&S81436[92], 9}, {NULL}};
const Segment SGI209477[] = {{&S81436[65], 6}, {NULL}};
const Segment SGI209478[] = {{&S76751[34], 11}, {NULL}};
const Segment SGI209479[] = {{&S76751[27], 6}, {NULL}};
const Segment SGI209480[] = {{&S76751[98], 9}, {NULL}};
const Segment SGI209481[] = {{&S76751[71], 6}, {NULL}};
const Segment SGI209482[] = {{&S67069[34], 10}, {NULL}};
const Segment SGI209483[] = {{&S67069[27], 6}, {NULL}};
const Segment SGI209484[] = {{&S76752[34], 15}, {NULL}};
const Segment SGI209485[] = {{&S76752[27], 6}, {NULL}};
const Segment SGI209486[] = {{&S67069[96], 9}, {NULL}};
const Segment SGI209487[] = {{&S67069[69], 6}, {NULL}};
const Segment SGI209488[] = {{&S76752[102], 9}, {NULL}};
const Segment SGI209489[] = {{&S76752[75], 6}, {NULL}};
const Segment SGI209490[] = {{&S86571[34], 12}, {NULL}};
const Segment SGI209491[] = {{&S86571[27], 6}, {NULL}};
const Segment SGI209492[] = {{&S86571[99], 9}, {NULL}};
const Segment SGI209493[] = {{&S86571[72], 6}, {NULL}};
const Segment SGI209494[] = {{&S67071[34], 10}, {NULL}};
const Segment SGI209495[] = {{&S67071[27], 6}, {NULL}};
const Segment SGI209496[] = {{&S67071[96], 9}, {NULL}};
const Segment SGI209497[] = {{&S67071[69], 6}, {NULL}};
const Segment SGI209498[] = {{&S86573[34], 9}, {NULL}};
const Segment SGI209499[] = {{&S86573[27], 6}, {NULL}};
const Segment SGI209500[] = {{&S86573[95], 9}, {NULL}};
const Segment SGI209501[] = {{&S86573[68], 6}, {NULL}};
const Segment SGI209502[] = {{&S76757[34], 10}, {NULL}};
const Segment SGI209503[] = {{&S76757[27], 6}, {NULL}};
const Segment SGI209504[] = {{&S76757[97], 9}, {NULL}};
const Segment SGI209505[] = {{&S76757[70], 6}, {NULL}};
const Segment SGI209506[] = {{&S86579[34], 16}, {NULL}};
const Segment SGI209507[] = {{&S86579[27], 6}, {NULL}};
const Segment SGI209508[] = {{&S86579[111], 9}, {NULL}};
const Segment SGI209509[] = {{&S86579[84], 6}, {NULL}};
const Segment SGI209510[] = {{&S76762[34], 9}, {NULL}};
const Segment SGI209511[] = {{&S76762[27], 6}, {NULL}};
const Segment SGI209512[] = {{&S76762[96], 9}, {NULL}};
const Segment SGI209513[] = {{&S86580[45], 7}, {NULL}};
const Segment SGI209514[] = {{&S76762[69], 6}, {NULL}};
const Segment SGI209515[] = {{&S86580[30], 6}, {NULL}};
const Segment SGI209516[] = {{&S86580[31], 3}, {NULL}};
const Segment SGI209517[] = {{&S86580[83], 5}, {NULL}};
const Segment SGI209518[] = {{&S86580[141], 13}, {NULL}};
const Segment SGI209519[] = {{&S86585[34], 13}, {NULL}};
const Segment SGI209520[] = {{&S86585[27], 6}, {NULL}};
const Segment SGI209521[] = {{&S86585[100], 9}, {NULL}};
const Segment SGI209522[] = {{&S86585[73], 6}, {NULL}};
const Segment SGI209523[] = {{&S81472[44], 10}, {NULL}};
const Segment SGI209524[] = {{&S81472[44], 22}, {NULL}};
const Segment SGI209525[] = {{&S81472[30], 6}, {NULL}};
const Segment SGI209526[] = {{&S81472[115], 3}, {NULL}};
const Segment SGI209527[] = {{&S81472[80], 6}, {NULL}};
const Segment SGI209528[] = {{&S86593[34], 24}, {NULL}};
const Segment SGI209529[] = {{&S86593[27], 6}, {NULL}};
const Segment SGI209530[] = {{&S86593[113], 9}, {NULL}};
const Segment SGI209531[] = {{&S86593[86], 6}, {NULL}};
const Segment SGI209532[] = {{&S81473[44], 10}, {NULL}};
const Segment SGI209533[] = {{&S81473[44], 22}, {NULL}};
const Segment SGI209534[] = {{&S81473[30], 6}, {NULL}};
const Segment SGI209535[] = {{&S81473[115], 3}, {NULL}};
const Segment SGI209536[] = {{&S81473[80], 6}, {NULL}};
const Segment SGI209537[] = {{&S86594[34], 23}, {NULL}};
const Segment SGI209538[] = {{&S86594[27], 6}, {NULL}};
const Segment SGI209539[] = {{&S81474[54], 10}, {NULL}};
const Segment SGI209540[] = {{&S81474[47], 6}, {NULL}};
const Segment SGI209541[] = {{&S86594[112], 9}, {NULL}};
const Segment SGI209542[] = {{&S86594[85], 6}, {NULL}};
const Segment SGI209543[] = {{&S81475[44], 10}, {NULL}};
const Segment SGI209544[] = {{&S81475[44], 22}, {NULL}};
const Segment SGI209545[] = {{&S81475[30], 6}, {NULL}};
const Segment SGI209546[] = {{&S81475[80], 6}, {NULL}};
const Segment SGI209547[] = {{&S86600[59], 10}, {NULL}};
const Segment SGI209548[] = {{&S86600[59], 22}, {NULL}};
const Segment SGI209549[] = {{&S86600[45], 6}, {NULL}};
const Segment SGI209550[] = {{&S86600[46], 3}, {NULL}};
const Segment SGI209551[] = {{&S86600[95], 6}, {NULL}};
const Segment SGI209552[] = {{&S86600[0], 14}, {NULL}};
const Segment SGI209553[] = {{&S86601[59], 11}, {NULL}};
const Segment SGI209554[] = {{&S86601[59], 23}, {NULL}};
const Segment SGI209555[] = {{&S86601[45], 6}, {NULL}};
const Segment SGI209556[] = {{&S86601[46], 3}, {NULL}};
const Segment SGI209557[] = {{&S86601[96], 6}, {NULL}};
const Segment SGI209558[] = {{&S86601[0], 14}, {NULL}};
const Segment SGI209559[] = {{&S81491[44], 18}, {NULL}};
const Segment SGI209560[] = {{&S81491[44], 30}, {NULL}};
const Segment SGI209561[] = {{&S81491[30], 6}, {NULL}};
const Segment SGI209562[] = {{&S81491[123], 3}, {NULL}};
const Segment SGI209563[] = {{&S81491[88], 6}, {NULL}};
const Segment SGI209564[] = {{&S86613[44], 10}, {NULL}};
const Segment SGI209565[] = {{&S86613[44], 22}, {NULL}};
const Segment SGI209566[] = {{&S86613[30], 6}, {NULL}};
const Segment SGI209567[] = {{&S81495[34], 12}, {NULL}};
const Segment SGI209568[] = {{&S81495[27], 6}, {NULL}};
const Segment SGI209569[] = {{&S86613[31], 3}, {NULL}};
const Segment SGI209570[] = {{&S86613[80], 6}, {NULL}};
const Segment SGI209571[] = {{&S86613[150], 3}, {NULL}};
const Segment SGI209572[] = {{&S81495[99], 9}, {NULL}};
const Segment SGI209573[] = {{&S81495[72], 6}, {NULL}};
const Segment SGI209574[] = {{&S86614[34], 9}, {NULL}};
const Segment SGI209575[] = {{&S86614[27], 6}, {NULL}};
const Segment SGI209576[] = {{&S86614[96], 9}, {NULL}};
const Segment SGI209577[] = {{&S86614[69], 6}, {NULL}};
const Segment SGI209578[] = {{&S86615[34], 8}, {NULL}};
const Segment SGI209579[] = {{&S86615[27], 6}, {NULL}};
const Segment SGI209580[] = {{&S86615[95], 9}, {NULL}};
const Segment SGI209581[] = {{&S86615[68], 6}, {NULL}};
const Segment SGI209582[] = {{&S86616[34], 7}, {NULL}};
const Segment SGI209583[] = {{&S86616[27], 6}, {NULL}};
const Segment SGI209584[] = {{&S86616[94], 9}, {NULL}};
const Segment SGI209585[] = {{&S86616[67], 6}, {NULL}};
const Segment SGI209586[] = {{&S86619[34], 9}, {NULL}};
const Segment SGI209587[] = {{&S86619[27], 6}, {NULL}};
const Segment SGI209588[] = {{&S81505[44], 4}, {NULL}};
const Segment SGI209589[] = {{&S81505[44], 16}, {NULL}};
const Segment SGI209590[] = {{&S81505[30], 6}, {NULL}};
const Segment SGI209591[] = {{&S86619[103], 9}, {NULL}};
const Segment SGI209592[] = {{&S86619[76], 6}, {NULL}};
const Segment SGI209593[] = {{&S81505[109], 3}, {NULL}};
const Segment SGI209594[] = {{&S81505[74], 6}, {NULL}};
const Segment SGI209595[] = {{&S86620[34], 10}, {NULL}};
const Segment SGI209596[] = {{&S86620[27], 6}, {NULL}};
const Segment SGI209597[] = {{&S86620[97], 9}, {NULL}};
const Segment SGI209598[] = {{&S86620[70], 6}, {NULL}};
const Segment SGI209599[] = {{&S86620[138], 8}, {NULL}};
const Segment SGI209600[] = {{&S86630[34], 7}, {NULL}};
const Segment SGI209601[] = {{&S86630[27], 6}, {NULL}};
const Segment SGI209602[] = {{&S86630[100], 9}, {NULL}};
const Segment SGI209603[] = {{&S86630[73], 6}, {NULL}};
const Segment SGI209604[] = {{&S86630[141], 8}, {NULL}};
const Segment SGI209605[] = {{&S86633[34], 10}, {NULL}};
const Segment SGI209606[] = {{&S86633[27], 6}, {NULL}};
const Segment SGI209607[] = {{&S86633[97], 9}, {NULL}};
const Segment SGI209608[] = {{&S86633[70], 6}, {NULL}};
const Segment SGI209609[] = {{&S86634[34], 10}, {NULL}};
const Segment SGI209610[] = {{&S86634[27], 6}, {NULL}};
const Segment SGI209611[] = {{&S86634[97], 9}, {NULL}};
const Segment SGI209612[] = {{&S86634[70], 6}, {NULL}};
const Segment SGI209613[] = {{&S75747[97], 9}, {NULL}};
const Segment SGI209614[] = {{&S75747[70], 6}, {NULL}};
const Segment SGI209615[] = {{&S86642[44], 7}, {NULL}};
const Segment SGI209616[] = {{&S86642[44], 24}, {NULL}};
const Segment SGI209617[] = {{&S86642[30], 6}, {NULL}};
const Segment SGI209618[] = {{&S86642[82], 6}, {NULL}};
const Segment SGI209619[] = {{&S86642[109], 13}, {NULL}};
const Segment SGI209620[] = {{&S76847[44], 9}, {NULL}};
const Segment SGI209621[] = {{&S76847[44], 21}, {NULL}};
const Segment SGI209622[] = {{&S76847[30], 6}, {NULL}};
const Segment SGI209623[] = {{&S76847[114], 3}, {NULL}};
const Segment SGI209624[] = {{&S86645[34], 8}, {NULL}};
const Segment SGI209625[] = {{&S76847[79], 6}, {NULL}};
const Segment SGI209626[] = {{&S86645[27], 6}, {NULL}};
const Segment SGI209627[] = {{&S86645[95], 9}, {NULL}};
const Segment SGI209628[] = {{&S86645[68], 6}, {NULL}};
const Segment SGI209629[] = {{&S81540[34], 7}, {NULL}};
const Segment SGI209630[] = {{&S81540[27], 6}, {NULL}};
const Segment SGI209631[] = {{&S81540[94], 9}, {NULL}};
const Segment SGI209632[] = {{&S81540[67], 6}, {NULL}};
const Segment SGI209633[] = {{&S86647[34], 16}, {NULL}};
const Segment SGI209634[] = {{&S86647[27], 6}, {NULL}};
const Segment SGI209635[] = {{&S86647[113], 9}, {NULL}};
const Segment SGI209636[] = {{&S86647[86], 6}, {NULL}};
const Segment SGI209637[] = {{&S76852[34], 10}, {NULL}};
const Segment SGI209638[] = {{&S76852[27], 6}, {NULL}};
const Segment SGI209639[] = {{&S76852[97], 9}, {NULL}};
const Segment SGI209640[] = {{&S76852[70], 6}, {NULL}};
const Segment SGI209641[] = {{&S81544[44], 15}, {NULL}};
const Segment SGI209642[] = {{&S81544[44], 27}, {NULL}};
const Segment SGI209643[] = {{&S81544[30], 6}, {NULL}};
const Segment SGI209644[] = {{&S81544[120], 3}, {NULL}};
const Segment SGI209645[] = {{&S81544[85], 6}, {NULL}};
const Segment SGI209646[] = {{&S76853[34], 10}, {NULL}};
const Segment SGI209647[] = {{&S76853[27], 6}, {NULL}};
const Segment SGI209648[] = {{&S76853[97], 9}, {NULL}};
const Segment SGI209649[] = {{&S76853[70], 6}, {NULL}};
const Segment SGI209650[] = {{&S81545[44], 6}, {NULL}};
const Segment SGI209651[] = {{&S81545[44], 18}, {NULL}};
const Segment SGI209652[] = {{&S81545[30], 6}, {NULL}};
const Segment SGI209653[] = {{&S81545[76], 6}, {NULL}};
const Segment SGI209654[] = {{&S81545[103], 13}, {NULL}};
const Segment SGI209655[] = {{&S81549[34], 6}, {NULL}};
const Segment SGI209656[] = {{&S81549[27], 6}, {NULL}};
const Segment SGI209657[] = {{&S81549[93], 9}, {NULL}};
const Segment SGI209658[] = {{&S81549[66], 6}, {NULL}};
const Segment SGI209659[] = {{&S81553[44], 13}, {NULL}};
const Segment SGI209660[] = {{&S81553[44], 25}, {NULL}};
const Segment SGI209661[] = {{&S81553[30], 6}, {NULL}};
const Segment SGI209662[] = {{&S81553[31], 3}, {NULL}};
const Segment SGI209663[] = {{&S81553[83], 6}, {NULL}};
const Segment SGI209664[] = {{&S81560[44], 11}, {NULL}};
const Segment SGI209665[] = {{&S81560[44], 23}, {NULL}};
const Segment SGI209666[] = {{&S81560[30], 6}, {NULL}};
const Segment SGI209667[] = {{&S81560[116], 3}, {NULL}};
const Segment SGI209668[] = {{&S81560[81], 6}, {NULL}};
const Segment SGI209669[] = {{&S81563[44], 18}, {NULL}};
const Segment SGI209670[] = {{&S81563[44], 30}, {NULL}};
const Segment SGI209671[] = {{&S81563[30], 6}, {NULL}};
const Segment SGI209672[] = {{&S81563[123], 3}, {NULL}};
const Segment SGI209673[] = {{&S81563[88], 6}, {NULL}};
const Segment SGI209674[] = {{&S86681[44], 18}, {NULL}};
const Segment SGI209675[] = {{&S86681[44], 30}, {NULL}};
const Segment SGI209676[] = {{&S86681[30], 6}, {NULL}};
const Segment SGI209677[] = {{&S86681[31], 3}, {NULL}};
const Segment SGI209678[] = {{&S86681[103], 6}, {NULL}};
const Segment SGI209679[] = {{&S86681[76], 8}, {NULL}};
const Segment SGI209680[] = {{&S86682[44], 10}, {NULL}};
const Segment SGI209681[] = {{&S86682[44], 22}, {NULL}};
const Segment SGI209682[] = {{&S86682[30], 6}, {NULL}};
const Segment SGI209683[] = {{&S86682[31], 3}, {NULL}};
const Segment SGI209684[] = {{&S86682[95], 6}, {NULL}};
const Segment SGI209685[] = {{&S86682[68], 8}, {NULL}};
const Segment SGI209686[] = {{&S81569[44], 11}, {NULL}};
const Segment SGI209687[] = {{&S81569[44], 23}, {NULL}};
const Segment SGI209688[] = {{&S81569[30], 6}, {NULL}};
const Segment SGI209689[] = {{&S81569[116], 3}, {NULL}};
const Segment SGI209690[] = {{&S81569[81], 6}, {NULL}};
const Segment SGI209691[] = {{&S81570[44], 10}, {NULL}};
const Segment SGI209692[] = {{&S81570[44], 22}, {NULL}};
const Segment SGI209693[] = {{&S81570[30], 6}, {NULL}};
const Segment SGI209694[] = {{&S81570[115], 3}, {NULL}};
const Segment SGI209695[] = {{&S81570[80], 6}, {NULL}};
const Segment SGI209696[] = {{&S81574[34], 7}, {NULL}};
const Segment SGI209697[] = {{&S81574[27], 6}, {NULL}};
const Segment SGI209698[] = {{&S81574[94], 9}, {NULL}};
const Segment SGI209699[] = {{&S81574[67], 6}, {NULL}};
const Segment SGI209700[] = {{&S86699[34], 14}, {NULL}};
const Segment SGI209701[] = {{&S86699[27], 6}, {NULL}};
const Segment SGI209702[] = {{&S86699[101], 9}, {NULL}};
const Segment SGI209703[] = {{&S86699[74], 6}, {NULL}};
const Segment SGI209704[] = {{&S67230[34], 10}, {NULL}};
const Segment SGI209705[] = {{&S67230[27], 6}, {NULL}};
const Segment SGI209706[] = {{&S67230[97], 9}, {NULL}};
const Segment SGI209707[] = {{&S67230[70], 6}, {NULL}};
const Segment SGI209708[] = {{&S67232[34], 7}, {NULL}};
const Segment SGI209709[] = {{&S67232[27], 6}, {NULL}};
const Segment SGI209710[] = {{&S67232[94], 9}, {NULL}};
const Segment SGI209711[] = {{&S67232[67], 6}, {NULL}};
const Segment SGI209712[] = {{&S67233[34], 11}, {NULL}};
const Segment SGI209713[] = {{&S67233[27], 6}, {NULL}};
const Segment SGI209714[] = {{&S67233[98], 9}, {NULL}};
const Segment SGI209715[] = {{&S67233[71], 6}, {NULL}};
const Segment SGI209716[] = {{&S67234[34], 10}, {NULL}};
const Segment SGI209717[] = {{&S67234[27], 6}, {NULL}};
const Segment SGI209718[] = {{&S67234[97], 9}, {NULL}};
const Segment SGI209719[] = {{&S67234[70], 6}, {NULL}};
const Segment SGI209720[] = {{&S86710[45], 6}, {NULL}};
const Segment SGI209721[] = {{&S86710[30], 6}, {NULL}};
const Segment SGI209722[] = {{&S86710[31], 3}, {NULL}};
const Segment SGI209723[] = {{&S86710[77], 5}, {NULL}};
const Segment SGI209724[] = {{&S86710[135], 13}, {NULL}};
const Segment SGI209725[] = {{&S86713[34], 10}, {NULL}};
const Segment SGI209726[] = {{&S86713[27], 6}, {NULL}};
const Segment SGI209727[] = {{&S86713[97], 9}, {NULL}};
const Segment SGI209728[] = {{&S86713[70], 6}, {NULL}};
const Segment SGI209729[] = {{&S86715[44], 9}, {NULL}};
const Segment SGI209730[] = {{&S86715[44], 21}, {NULL}};
const Segment SGI209731[] = {{&S86715[30], 6}, {NULL}};
const Segment SGI209732[] = {{&S86715[31], 3}, {NULL}};
const Segment SGI209733[] = {{&S86715[79], 6}, {NULL}};
const Segment SGI209734[] = {{&S86721[44], 12}, {NULL}};
const Segment SGI209735[] = {{&S86721[44], 24}, {NULL}};
const Segment SGI209736[] = {{&S86721[30], 6}, {NULL}};
const Segment SGI209737[] = {{&S86721[31], 3}, {NULL}};
const Segment SGI209738[] = {{&S86721[82], 6}, {NULL}};
const Segment SGI209739[] = {{&S86723[34], 7}, {NULL}};
const Segment SGI209740[] = {{&S86723[27], 6}, {NULL}};
const Segment SGI209741[] = {{&S86723[99], 9}, {NULL}};
const Segment SGI209742[] = {{&S86723[72], 6}, {NULL}};
const Segment SGI209743[] = {{&S76931[34], 6}, {NULL}};
const Segment SGI209744[] = {{&S76931[27], 6}, {NULL}};
const Segment SGI209745[] = {{&S76931[93], 9}, {NULL}};
const Segment SGI209746[] = {{&S76931[66], 6}, {NULL}};
const Segment SGI209747[] = {{&S86727[34], 12}, {NULL}};
const Segment SGI209748[] = {{&S86727[27], 6}, {NULL}};
const Segment SGI209749[] = {{&S86727[99], 9}, {NULL}};
const Segment SGI209750[] = {{&S86727[72], 6}, {NULL}};
const Segment SGI209751[] = {{&S75847[44], 11}, {NULL}};
const Segment SGI209752[] = {{&S75847[44], 23}, {NULL}};
const Segment SGI209753[] = {{&S75847[30], 6}, {NULL}};
const Segment SGI209754[] = {{&S75847[134], 3}, {NULL}};
const Segment SGI209755[] = {{&S75847[99], 6}, {NULL}};
const Segment SGI209756[] = {{&S75847[69], 8}, {NULL}};
const Segment SGI209757[] = {{&S86729[44], 7}, {NULL}};
const Segment SGI209758[] = {{&S86729[44], 19}, {NULL}};
const Segment SGI209759[] = {{&S86729[30], 6}, {NULL}};
const Segment SGI209760[] = {{&S86729[31], 3}, {NULL}};
const Segment SGI209761[] = {{&S86729[77], 5}, {NULL}};
const Segment SGI209762[] = {{&S76938[44], 11}, {NULL}};
const Segment SGI209763[] = {{&S76938[44], 23}, {NULL}};
const Segment SGI209764[] = {{&S76938[30], 6}, {NULL}};
const Segment SGI209765[] = {{&S76938[116], 3}, {NULL}};
const Segment SGI209766[] = {{&S76938[81], 6}, {NULL}};
const Segment SGI209767[] = {{&S76938[151], 3}, {NULL}};
const Segment SGI209768[] = {{&S81620[44], 9}, {NULL}};
const Segment SGI209769[] = {{&S81620[44], 21}, {NULL}};
const Segment SGI209770[] = {{&S81620[30], 6}, {NULL}};
const Segment SGI209771[] = {{&S81620[114], 3}, {NULL}};
const Segment SGI209772[] = {{&S81620[79], 6}, {NULL}};
const Segment SGI209773[] = {{&S86734[34], 17}, {NULL}};
const Segment SGI209774[] = {{&S86734[27], 6}, {NULL}};
const Segment SGI209775[] = {{&S86734[104], 9}, {NULL}};
const Segment SGI209776[] = {{&S86734[77], 6}, {NULL}};
const Segment SGI209777[] = {{&S86736[44], 7}, {NULL}};
const Segment SGI209778[] = {{&S86736[44], 19}, {NULL}};
const Segment SGI209779[] = {{&S86736[30], 6}, {NULL}};
const Segment SGI209780[] = {{&S86736[31], 3}, {NULL}};
const Segment SGI209781[] = {{&S86736[77], 6}, {NULL}};
const Segment SGI209782[] = {{&S86738[34], 10}, {NULL}};
const Segment SGI209783[] = {{&S86738[27], 6}, {NULL}};
const Segment SGI209784[] = {{&S86738[97], 9}, {NULL}};
const Segment SGI209785[] = {{&S86738[70], 6}, {NULL}};
const Segment SGI209786[] = {{&S86739[34], 9}, {NULL}};
const Segment SGI209787[] = {{&S86739[27], 6}, {NULL}};
const Segment SGI209788[] = {{&S67276[34], 9}, {NULL}};
const Segment SGI209789[] = {{&S67276[27], 6}, {NULL}};
const Segment SGI209790[] = {{&S86739[96], 9}, {NULL}};
const Segment SGI209791[] = {{&S86739[69], 6}, {NULL}};
const Segment SGI209792[] = {{&S67276[95], 9}, {NULL}};
const Segment SGI209793[] = {{&S67276[68], 6}, {NULL}};
const Segment SGI209794[] = {{&S67276[136], 8}, {NULL}};
const Segment SGI209795[] = {{&S86741[34], 10}, {NULL}};
const Segment SGI209796[] = {{&S86741[27], 6}, {NULL}};
const Segment SGI209797[] = {{&S86741[97], 9}, {NULL}};
const Segment SGI209798[] = {{&S86741[70], 6}, {NULL}};
const Segment SGI209799[] = {{&S86746[44], 5}, {NULL}};
const Segment SGI209800[] = {{&S86746[44], 17}, {NULL}};
const Segment SGI209801[] = {{&S86746[30], 6}, {NULL}};
const Segment SGI209802[] = {{&S86746[75], 6}, {NULL}};
const Segment SGI209803[] = {{&S86746[102], 13}, {NULL}};
const Segment SGI209804[] = {{&S76975[34], 25}, {NULL}};
const Segment SGI209805[] = {{&S76975[27], 6}, {NULL}};
const Segment SGI209806[] = {{&S76975[112], 9}, {NULL}};
const Segment SGI209807[] = {{&S76975[85], 6}, {NULL}};
const Segment SGI209808[] = {{&S76977[34], 10}, {NULL}};
const Segment SGI209809[] = {{&S76977[27], 6}, {NULL}};
const Segment SGI209810[] = {{&S76977[97], 9}, {NULL}};
const Segment SGI209811[] = {{&S76977[70], 6}, {NULL}};
const Segment SGI209812[] = {{&S86764[34], 9}, {NULL}};
const Segment SGI209813[] = {{&S86764[27], 6}, {NULL}};
const Segment SGI209814[] = {{&S86764[96], 9}, {NULL}};
const Segment SGI209815[] = {{&S86764[69], 6}, {NULL}};
const Segment SGI209816[] = {{&S67309[34], 11}, {NULL}};
const Segment SGI209817[] = {{&S67309[27], 6}, {NULL}};
const Segment SGI209818[] = {{&S67309[97], 9}, {NULL}};
const Segment SGI209819[] = {{&S67309[70], 6}, {NULL}};
const Segment SGI209820[] = {{&S67318[34], 15}, {NULL}};
const Segment SGI209821[] = {{&S67318[27], 6}, {NULL}};
const Segment SGI209822[] = {{&S67318[101], 9}, {NULL}};
const Segment SGI209823[] = {{&S67318[74], 6}, {NULL}};
const Segment SGI209824[] = {{&S86795[44], 10}, {NULL}};
const Segment SGI209825[] = {{&S86795[44], 22}, {NULL}};
const Segment SGI209826[] = {{&S86795[30], 6}, {NULL}};
const Segment SGI209827[] = {{&S86795[31], 3}, {NULL}};
const Segment SGI209828[] = {{&S86795[95], 6}, {NULL}};
const Segment SGI209829[] = {{&S86795[68], 8}, {NULL}};
const Segment SGI209830[] = {{&S86796[44], 10}, {NULL}};
const Segment SGI209831[] = {{&S86796[44], 22}, {NULL}};
const Segment SGI209832[] = {{&S86796[30], 6}, {NULL}};
const Segment SGI209833[] = {{&S86796[31], 3}, {NULL}};
const Segment SGI209834[] = {{&S86796[99], 6}, {NULL}};
const Segment SGI209835[] = {{&S86796[68], 8}, {NULL}};
const Segment SGI209836[] = {{&S86801[34], 13}, {NULL}};
const Segment SGI209837[] = {{&S86801[27], 6}, {NULL}};
const Segment SGI209838[] = {{&S86801[100], 9}, {NULL}};
const Segment SGI209839[] = {{&S86801[73], 6}, {NULL}};
const Segment SGI209840[] = {{&S86804[34], 10}, {NULL}};
const Segment SGI209841[] = {{&S86804[27], 6}, {NULL}};
const Segment SGI209842[] = {{&S86804[97], 9}, {NULL}};
const Segment SGI209843[] = {{&S86804[70], 6}, {NULL}};
const Segment SGI209844[] = {{&S86806[34], 15}, {NULL}};
const Segment SGI209845[] = {{&S86806[27], 6}, {NULL}};
const Segment SGI209846[] = {{&S86806[102], 9}, {NULL}};
const Segment SGI209847[] = {{&S86806[75], 6}, {NULL}};
const Segment SGI209848[] = {{&S86810[34], 5}, {NULL}};
const Segment SGI209849[] = {{&S86810[27], 6}, {NULL}};
const Segment SGI209850[] = {{&S86810[92], 9}, {NULL}};
const Segment SGI209851[] = {{&S86810[65], 6}, {NULL}};
const Segment SGI209852[] = {{&S86811[34], 11}, {NULL}};
const Segment SGI209853[] = {{&S86811[27], 6}, {NULL}};
const Segment SGI209854[] = {{&S86811[98], 9}, {NULL}};
const Segment SGI209855[] = {{&S86811[71], 6}, {NULL}};
const Segment SGI209856[] = {{&S77034[44], 7}, {NULL}};
const Segment SGI209857[] = {{&S77034[44], 19}, {NULL}};
const Segment SGI209858[] = {{&S67357[44], 12}, {NULL}};
const Segment SGI209859[] = {{&S77034[30], 6}, {NULL}};
const Segment SGI209860[] = {{&S67357[44], 23}, {NULL}};
const Segment SGI209861[] = {{&S67357[30], 6}, {NULL}};
const Segment SGI209862[] = {{&S86813[34], 8}, {NULL}};
const Segment SGI209863[] = {{&S77034[112], 3}, {NULL}};
const Segment SGI209864[] = {{&S86813[27], 6}, {NULL}};
const Segment SGI209865[] = {{&S67357[134], 3}, {NULL}};
const Segment SGI209866[] = {{&S77034[77], 6}, {NULL}};
const Segment SGI209867[] = {{&S67357[99], 6}, {NULL}};
const Segment SGI209868[] = {{&S67357[69], 8}, {NULL}};
const Segment SGI209869[] = {{&S86813[95], 9}, {NULL}};
const Segment SGI209870[] = {{&S86813[68], 6}, {NULL}};
const Segment SGI209871[] = {{&S77035[44], 7}, {NULL}};
const Segment SGI209872[] = {{&S77035[44], 19}, {NULL}};
const Segment SGI209873[] = {{&S77035[30], 6}, {NULL}};
const Segment SGI209874[] = {{&S67358[44], 12}, {NULL}};
const Segment SGI209875[] = {{&S67358[44], 24}, {NULL}};
const Segment SGI209876[] = {{&S67358[30], 6}, {NULL}};
const Segment SGI209877[] = {{&S77035[80], 6}, {NULL}};
const Segment SGI209878[] = {{&S67358[135], 3}, {NULL}};
const Segment SGI209879[] = {{&S67358[100], 6}, {NULL}};
const Segment SGI209880[] = {{&S67358[70], 8}, {NULL}};
const Segment SGI209881[] = {{&S86815[59], 14}, {NULL}};
const Segment SGI209882[] = {{&S86815[59], 31}, {NULL}};
const Segment SGI209883[] = {{&S86815[45], 6}, {NULL}};
const Segment SGI209884[] = {{&S77036[44], 8}, {NULL}};
const Segment SGI209885[] = {{&S77036[44], 20}, {NULL}};
const Segment SGI209886[] = {{&S77036[30], 6}, {NULL}};
const Segment SGI209887[] = {{&S86815[46], 3}, {NULL}};
const Segment SGI209888[] = {{&S86815[104], 6}, {NULL}};
const Segment SGI209889[] = {{&S77036[113], 3}, {NULL}};
const Segment SGI209890[] = {{&S77036[78], 6}, {NULL}};
const Segment SGI209891[] = {{&S86815[0], 14}, {NULL}};
const Segment SGI209892[] = {{&S77037[44], 10}, {NULL}};
const Segment SGI209893[] = {{&S77037[44], 22}, {NULL}};
const Segment SGI209894[] = {{&S77037[30], 6}, {NULL}};
const Segment SGI209895[] = {{&S77037[113], 3}, {NULL}};
const Segment SGI209896[] = {{&S77037[80], 5}, {NULL}};
const Segment SGI209897[] = {{&S77037[117], 14}, {NULL}};
const Segment SGI209898[] = {{&S75953[34], 25}, {NULL}};
const Segment SGI209899[] = {{&S75953[27], 6}, {NULL}};
const Segment SGI209900[] = {{&S75953[112], 9}, {NULL}};
const Segment SGI209901[] = {{&S75953[85], 6}, {NULL}};
const Segment SGI209902[] = {{&S86822[34], 10}, {NULL}};
const Segment SGI209903[] = {{&S86822[27], 6}, {NULL}};
const Segment SGI209904[] = {{&S77042[34], 9}, {NULL}};
const Segment SGI209905[] = {{&S86822[97], 9}, {NULL}};
const Segment SGI209906[] = {{&S77042[27], 6}, {NULL}};
const Segment SGI209907[] = {{&S86822[70], 6}, {NULL}};
const Segment SGI209908[] = {{&S77042[96], 9}, {NULL}};
const Segment SGI209909[] = {{&S77042[69], 6}, {NULL}};
const Segment SGI209910[] = {{&S75956[34], 6}, {NULL}};
const Segment SGI209911[] = {{&S75956[27], 6}, {NULL}};
const Segment SGI209912[] = {{&S75956[93], 9}, {NULL}};
const Segment SGI209913[] = {{&S75956[66], 6}, {NULL}};
const Segment SGI209914[] = {{&S86824[44], 4}, {NULL}};
const Segment SGI209915[] = {{&S86824[44], 16}, {NULL}};
const Segment SGI209916[] = {{&S86824[30], 6}, {NULL}};
const Segment SGI209917[] = {{&S86824[31], 3}, {NULL}};
const Segment SGI209918[] = {{&S86824[74], 6}, {NULL}};
const Segment SGI209919[] = {{&S86825[34], 11}, {NULL}};
const Segment SGI209920[] = {{&S86825[27], 6}, {NULL}};
const Segment SGI209921[] = {{&S77046[34], 15}, {NULL}};
const Segment SGI209922[] = {{&S77046[27], 6}, {NULL}};
const Segment SGI209923[] = {{&S86825[98], 9}, {NULL}};
const Segment SGI209924[] = {{&S86825[71], 6}, {NULL}};
const Segment SGI209925[] = {{&S77046[102], 9}, {NULL}};
const Segment SGI209926[] = {{&S77046[75], 6}, {NULL}};
const Segment SGI209927[] = {{&S77047[34], 11}, {NULL}};
const Segment SGI209928[] = {{&S86826[44], 10}, {NULL}};
const Segment SGI209929[] = {{&S77047[27], 6}, {NULL}};
const Segment SGI209930[] = {{&S86826[44], 22}, {NULL}};
const Segment SGI209931[] = {{&S86826[30], 6}, {NULL}};
const Segment SGI209932[] = {{&S77047[98], 9}, {NULL}};
const Segment SGI209933[] = {{&S86826[31], 3}, {NULL}};
const Segment SGI209934[] = {{&S77047[71], 6}, {NULL}};
const Segment SGI209935[] = {{&S86826[95], 6}, {NULL}};
const Segment SGI209936[] = {{&S86826[68], 8}, {NULL}};
const Segment SGI209937[] = {{&S86827[44], 10}, {NULL}};
const Segment SGI209938[] = {{&S86827[44], 22}, {NULL}};
const Segment SGI209939[] = {{&S86827[30], 6}, {NULL}};
const Segment SGI209940[] = {{&S86827[31], 3}, {NULL}};
const Segment SGI209941[] = {{&S86827[95], 6}, {NULL}};
const Segment SGI209942[] = {{&S86827[68], 8}, {NULL}};
const Segment SGI209943[] = {{&S77051[44], 10}, {NULL}};
const Segment SGI209944[] = {{&S77051[44], 22}, {NULL}};
const Segment SGI209945[] = {{&S77051[30], 6}, {NULL}};
const Segment SGI209946[] = {{&S77051[113], 3}, {NULL}};
const Segment SGI209947[] = {{&S77051[80], 5}, {NULL}};
const Segment SGI209948[] = {{&S67380[34], 15}, {NULL}};
const Segment SGI209949[] = {{&S67380[27], 6}, {NULL}};
const Segment SGI209950[] = {{&S77051[117], 14}, {NULL}};
const Segment SGI209951[] = {{&S67380[102], 9}, {NULL}};
const Segment SGI209952[] = {{&S67380[75], 6}, {NULL}};
const Segment SGI209953[] = {{&S77053[44], 11}, {NULL}};
const Segment SGI209954[] = {{&S77053[44], 23}, {NULL}};
const Segment SGI209955[] = {{&S77053[30], 6}, {NULL}};
const Segment SGI209956[] = {{&S77053[116], 3}, {NULL}};
const Segment SGI209957[] = {{&S77053[81], 6}, {NULL}};
const Segment SGI209958[] = {{&S77054[34], 12}, {NULL}};
const Segment SGI209959[] = {{&S77054[27], 6}, {NULL}};
const Segment SGI209960[] = {{&S77054[99], 9}, {NULL}};
const Segment SGI209961[] = {{&S77054[72], 6}, {NULL}};
const Segment SGI209962[] = {{&S86845[44], 12}, {NULL}};
const Segment SGI209963[] = {{&S86845[44], 24}, {NULL}};
const Segment SGI209964[] = {{&S86845[30], 6}, {NULL}};
const Segment SGI209965[] = {{&S86845[31], 3}, {NULL}};
const Segment SGI209966[] = {{&S86845[82], 6}, {NULL}};
const Segment SGI209967[] = {{&S86848[34], 10}, {NULL}};
const Segment SGI209968[] = {{&S86848[27], 6}, {NULL}};
const Segment SGI209969[] = {{&S81640[34], 13}, {NULL}};
const Segment SGI209970[] = {{&S81640[27], 6}, {NULL}};
const Segment SGI209971[] = {{&S86848[97], 9}, {NULL}};
const Segment SGI209972[] = {{&S86848[70], 6}, {NULL}};
const Segment SGI209973[] = {{&S81640[100], 9}, {NULL}};
const Segment SGI209974[] = {{&S81640[73], 6}, {NULL}};
const Segment SGI209975[] = {{&S86849[34], 8}, {NULL}};
const Segment SGI209976[] = {{&S86849[27], 6}, {NULL}};
const Segment SGI209977[] = {{&S86849[100], 9}, {NULL}};
const Segment SGI209978[] = {{&S86849[73], 6}, {NULL}};
const Segment SGI209979[] = {{&S67402[34], 10}, {NULL}};
const Segment SGI209980[] = {{&S67402[27], 6}, {NULL}};
const Segment SGI209981[] = {{&S86850[44], 17}, {NULL}};
const Segment SGI209982[] = {{&S86850[44], 29}, {NULL}};
const Segment SGI209983[] = {{&S86850[30], 6}, {NULL}};
const Segment SGI209984[] = {{&S67402[96], 9}, {NULL}};
const Segment SGI209985[] = {{&S67402[69], 6}, {NULL}};
const Segment SGI209986[] = {{&S86850[31], 3}, {NULL}};
const Segment SGI209987[] = {{&S86850[87], 6}, {NULL}};
const Segment SGI209988[] = {{&S86851[34], 7}, {NULL}};
const Segment SGI209989[] = {{&S86851[27], 6}, {NULL}};
const Segment SGI209990[] = {{&S86851[99], 9}, {NULL}};
const Segment SGI209991[] = {{&S86851[72], 6}, {NULL}};
const Segment SGI209992[] = {{&S86851[140], 8}, {NULL}};
const Segment SGI209993[] = {{&S67405[34], 11}, {NULL}};
const Segment SGI209994[] = {{&S67405[27], 6}, {NULL}};
const Segment SGI209995[] = {{&S67405[97], 9}, {NULL}};
const Segment SGI209996[] = {{&S67405[70], 6}, {NULL}};
const Segment SGI209997[] = {{&S86853[34], 10}, {NULL}};
const Segment SGI209998[] = {{&S86853[27], 6}, {NULL}};
const Segment SGI209999[] = {{&S67406[34], 11}, {NULL}};
const Segment SGI210000[] = {{&S67406[27], 6}, {NULL}};
const Segment SGI210001[] = {{&S86853[97], 9}, {NULL}};
const Segment SGI210002[] = {{&S86853[70], 6}, {NULL}};
const Segment SGI210003[] = {{&S67406[97], 9}, {NULL}};
const Segment SGI210004[] = {{&S67406[70], 6}, {NULL}};
const Segment SGI210005[] = {{&S86853[138], 8}, {NULL}};
const Segment SGI210006[] = {{&S86854[34], 21}, {NULL}};
const Segment SGI210007[] = {{&S86854[27], 6}, {NULL}};
const Segment SGI210008[] = {{&S67407[44], 8}, {NULL}};
const Segment SGI210009[] = {{&S67407[44], 19}, {NULL}};
const Segment SGI210010[] = {{&S67407[30], 6}, {NULL}};
const Segment SGI210011[] = {{&S86854[118], 9}, {NULL}};
const Segment SGI210012[] = {{&S86854[91], 6}, {NULL}};
const Segment SGI210013[] = {{&S67407[112], 3}, {NULL}};
const Segment SGI210014[] = {{&S67407[77], 6}, {NULL}};
const Segment SGI210015[] = {{&S75998[34], 10}, {NULL}};
const Segment SGI210016[] = {{&S75998[27], 6}, {NULL}};
const Segment SGI210017[] = {{&S75998[97], 9}, {NULL}};
const Segment SGI210018[] = {{&S75998[70], 6}, {NULL}};
const Segment SGI210019[] = {{&S67408[38], 8}, {NULL}};
const Segment SGI210020[] = {{&S67408[31], 6}, {NULL}};
const Segment SGI210021[] = {{&S86861[34], 7}, {NULL}};
const Segment SGI210022[] = {{&S86861[27], 6}, {NULL}};
const Segment SGI210023[] = {{&S86861[99], 9}, {NULL}};
const Segment SGI210024[] = {{&S86861[72], 6}, {NULL}};
const Segment SGI210025[] = {{&S86861[140], 8}, {NULL}};
const Segment SGI210026[] = {{&S86863[34], 10}, {NULL}};
const Segment SGI210027[] = {{&S86863[27], 6}, {NULL}};
const Segment SGI210028[] = {{&S86863[97], 9}, {NULL}};
const Segment SGI210029[] = {{&S86863[70], 6}, {NULL}};
const Segment SGI210030[] = {{&S86866[34], 15}, {NULL}};
const Segment SGI210031[] = {{&S86866[27], 6}, {NULL}};
const Segment SGI210032[] = {{&S86866[117], 9}, {NULL}};
const Segment SGI210033[] = {{&S86866[90], 6}, {NULL}};
const Segment SGI210034[] = {{&S86875[34], 14}, {NULL}};
const Segment SGI210035[] = {{&S86875[27], 6}, {NULL}};
const Segment SGI210036[] = {{&S86875[101], 9}, {NULL}};
const Segment SGI210037[] = {{&S86875[74], 6}, {NULL}};
const Segment SGI210038[] = {{&S86877[34], 6}, {NULL}};
const Segment SGI210039[] = {{&S86877[27], 6}, {NULL}};
const Segment SGI210040[] = {{&S86877[93], 9}, {NULL}};
const Segment SGI210041[] = {{&S86877[66], 6}, {NULL}};
const Segment SGI210042[] = {{&S86880[34], 11}, {NULL}};
const Segment SGI210043[] = {{&S86880[27], 6}, {NULL}};
const Segment SGI210044[] = {{&S86880[98], 9}, {NULL}};
const Segment SGI210045[] = {{&S86880[71], 6}, {NULL}};
const Segment SGI210046[] = {{&S86881[34], 11}, {NULL}};
const Segment SGI210047[] = {{&S86881[27], 6}, {NULL}};
const Segment SGI210048[] = {{&S86881[98], 9}, {NULL}};
const Segment SGI210049[] = {{&S86881[71], 6}, {NULL}};
const Segment SGI210050[] = {{&S86882[34], 11}, {NULL}};
const Segment SGI210051[] = {{&S86882[27], 6}, {NULL}};
const Segment SGI210052[] = {{&S86882[98], 9}, {NULL}};
const Segment SGI210053[] = {{&S86882[71], 6}, {NULL}};
const Segment SGI210054[] = {{&S77120[34], 25}, {NULL}};
const Segment SGI210055[] = {{&S77120[27], 6}, {NULL}};
const Segment SGI210056[] = {{&S77120[112], 9}, {NULL}};
const Segment SGI210057[] = {{&S77120[85], 6}, {NULL}};
const Segment SGI210058[] = {{&S77121[34], 16}, {NULL}};
const Segment SGI210059[] = {{&S77121[27], 6}, {NULL}};
const Segment SGI210060[] = {{&S77121[103], 9}, {NULL}};
const Segment SGI210061[] = {{&S77121[76], 6}, {NULL}};
const Segment SGI210062[] = {{&S86903[44], 14}, {NULL}};
const Segment SGI210063[] = {{&S86903[44], 26}, {NULL}};
const Segment SGI210064[] = {{&S86903[30], 6}, {NULL}};
const Segment SGI210065[] = {{&S86903[115], 2}, {NULL}};
const Segment SGI210066[] = {{&S86903[84], 5}, {NULL}};
const Segment SGI210067[] = {{&S86904[44], 14}, {NULL}};
const Segment SGI210068[] = {{&S86904[44], 26}, {NULL}};
const Segment SGI210069[] = {{&S86904[30], 6}, {NULL}};
const Segment SGI210070[] = {{&S86904[115], 2}, {NULL}};
const Segment SGI210071[] = {{&S86904[84], 5}, {NULL}};
const Segment SGI210072[] = {{&S86905[44], 14}, {NULL}};
const Segment SGI210073[] = {{&S86905[44], 26}, {NULL}};
const Segment SGI210074[] = {{&S86905[30], 6}, {NULL}};
const Segment SGI210075[] = {{&S86905[115], 2}, {NULL}};
const Segment SGI210076[] = {{&S86905[84], 5}, {NULL}};
const Segment SGI210077[] = {{&S76058[34], 6}, {NULL}};
const Segment SGI210078[] = {{&S76058[27], 6}, {NULL}};
const Segment SGI210079[] = {{&S86906[44], 14}, {NULL}};
const Segment SGI210080[] = {{&S86906[44], 26}, {NULL}};
const Segment SGI210081[] = {{&S76058[93], 9}, {NULL}};
const Segment SGI210082[] = {{&S86906[30], 6}, {NULL}};
const Segment SGI210083[] = {{&S76058[66], 6}, {NULL}};
const Segment SGI210084[] = {{&S86906[115], 2}, {NULL}};
const Segment SGI210085[] = {{&S86906[84], 5}, {NULL}};
const Segment SGI210086[] = {{&S86907[44], 14}, {NULL}};
const Segment SGI210087[] = {{&S86907[44], 26}, {NULL}};
const Segment SGI210088[] = {{&S86907[30], 6}, {NULL}};
const Segment SGI210089[] = {{&S86907[115], 2}, {NULL}};
const Segment SGI210090[] = {{&S86907[84], 5}, {NULL}};
const Segment SGI210091[] = {{&S67477[34], 20}, {NULL}};
const Segment SGI210092[] = {{&S67477[27], 6}, {NULL}};
const Segment SGI210093[] = {{&S67477[106], 9}, {NULL}};
const Segment SGI210094[] = {{&S67477[79], 6}, {NULL}};
const Segment SGI210095[] = {{&S77142[44], 8}, {NULL}};
const Segment SGI210096[] = {{&S77142[44], 20}, {NULL}};
const Segment SGI210097[] = {{&S77142[30], 6}, {NULL}};
const Segment SGI210098[] = {{&S77142[113], 3}, {NULL}};
const Segment SGI210099[] = {{&S77142[78], 6}, {NULL}};
const Segment SGI210100[] = {{&S67478[34], 21}, {NULL}};
const Segment SGI210101[] = {{&S67478[27], 6}, {NULL}};
const Segment SGI210102[] = {{&S67478[108], 9}, {NULL}};
const Segment SGI210103[] = {{&S67478[81], 6}, {NULL}};
const Segment SGI210104[] = {{&S77144[34], 9}, {NULL}};
const Segment SGI210105[] = {{&S77144[27], 6}, {NULL}};
const Segment SGI210106[] = {{&S77144[96], 9}, {NULL}};
const Segment SGI210107[] = {{&S77144[69], 6}, {NULL}};
const Segment SGI210108[] = {{&S77145[34], 10}, {NULL}};
const Segment SGI210109[] = {{&S77145[27], 6}, {NULL}};
const Segment SGI210110[] = {{&S77145[97], 9}, {NULL}};
const Segment SGI210111[] = {{&S77145[70], 6}, {NULL}};
const Segment SGI210112[] = {{&S77146[34], 10}, {NULL}};
const Segment SGI210113[] = {{&S77146[27], 6}, {NULL}};
const Segment SGI210114[] = {{&S77146[97], 9}, {NULL}};
const Segment SGI210115[] = {{&S77146[70], 6}, {NULL}};
const Segment SGI210116[] = {{&S77151[34], 11}, {NULL}};
const Segment SGI210117[] = {{&S77151[27], 6}, {NULL}};
const Segment SGI210118[] = {{&S77151[98], 9}, {NULL}};
const Segment SGI210119[] = {{&S77151[71], 6}, {NULL}};
const Segment SGI210120[] = {{&S77154[44], 11}, {NULL}};
const Segment SGI210121[] = {{&S77154[44], 23}, {NULL}};
const Segment SGI210122[] = {{&S77154[30], 6}, {NULL}};
const Segment SGI210123[] = {{&S86933[44], 8}, {NULL}};
const Segment SGI210124[] = {{&S77154[116], 3}, {NULL}};
const Segment SGI210125[] = {{&S86933[44], 20}, {NULL}};
const Segment SGI210126[] = {{&S86933[30], 6}, {NULL}};
const Segment SGI210127[] = {{&S77154[81], 6}, {NULL}};
const Segment SGI210128[] = {{&S86933[31], 3}, {NULL}};
const Segment SGI210129[] = {{&S86933[97], 6}, {NULL}};
const Segment SGI210130[] = {{&S86933[66], 8}, {NULL}};
const Segment SGI210131[] = {{&S86934[44], 8}, {NULL}};
const Segment SGI210132[] = {{&S86934[44], 20}, {NULL}};
const Segment SGI210133[] = {{&S86934[30], 6}, {NULL}};
const Segment SGI210134[] = {{&S86934[31], 3}, {NULL}};
const Segment SGI210135[] = {{&S86934[93], 6}, {NULL}};
const Segment SGI210136[] = {{&S86934[66], 8}, {NULL}};
const Segment SGI210137[] = {{&S81656[34], 17}, {NULL}};
const Segment SGI210138[] = {{&S81656[27], 6}, {NULL}};
const Segment SGI210139[] = {{&S81656[104], 9}, {NULL}};
const Segment SGI210140[] = {{&S81656[77], 6}, {NULL}};
const Segment SGI210141[] = {{&S86952[34], 10}, {NULL}};
const Segment SGI210142[] = {{&S86952[27], 6}, {NULL}};
const Segment SGI210143[] = {{&S86952[97], 9}, {NULL}};
const Segment SGI210144[] = {{&S86952[70], 6}, {NULL}};
const Segment SGI210145[] = {{&S86964[44], 28}, {NULL}};
const Segment SGI210146[] = {{&S86964[44], 40}, {NULL}};
const Segment SGI210147[] = {{&S86964[30], 6}, {NULL}};
const Segment SGI210148[] = {{&S86964[31], 3}, {NULL}};
const Segment SGI210149[] = {{&S86964[98], 6}, {NULL}};
const Segment SGI210150[] = {{&S86965[44], 29}, {NULL}};
const Segment SGI210151[] = {{&S86965[44], 41}, {NULL}};
const Segment SGI210152[] = {{&S86965[30], 6}, {NULL}};
const Segment SGI210153[] = {{&S86965[31], 3}, {NULL}};
const Segment SGI210154[] = {{&S86965[99], 6}, {NULL}};
const Segment SGI210155[] = {{&S86968[45], 4}, {NULL}};
const Segment SGI210156[] = {{&S81686[34], 12}, {NULL}};
const Segment SGI210157[] = {{&S86968[30], 6}, {NULL}};
const Segment SGI210158[] = {{&S81686[27], 6}, {NULL}};
const Segment SGI210159[] = {{&S86968[31], 3}, {NULL}};
const Segment SGI210160[] = {{&S81686[99], 9}, {NULL}};
const Segment SGI210161[] = {{&S81686[72], 6}, {NULL}};
const Segment SGI210162[] = {{&S86968[75], 5}, {NULL}};
const Segment SGI210163[] = {{&S86968[133], 13}, {NULL}};
const Segment SGI210164[] = {{&S86970[34], 13}, {NULL}};
const Segment SGI210165[] = {{&S86970[27], 6}, {NULL}};
const Segment SGI210166[] = {{&S86970[100], 9}, {NULL}};
const Segment SGI210167[] = {{&S86970[73], 6}, {NULL}};
const Segment SGI210168[] = {{&S77210[44], 15}, {NULL}};
const Segment SGI210169[] = {{&S77210[44], 27}, {NULL}};
const Segment SGI210170[] = {{&S77210[30], 6}, {NULL}};
const Segment SGI210171[] = {{&S77210[85], 6}, {NULL}};
const Segment SGI210172[] = {{&S77210[112], 13}, {NULL}};
const Segment SGI210173[] = {{&S77211[38], 7}, {NULL}};
const Segment SGI210174[] = {{&S77211[31], 6}, {NULL}};
const Segment SGI210175[] = {{&S67553[34], 10}, {NULL}};
const Segment SGI210176[] = {{&S67553[27], 6}, {NULL}};
const Segment SGI210177[] = {{&S67553[96], 9}, {NULL}};
const Segment SGI210178[] = {{&S67553[69], 6}, {NULL}};
const Segment SGI210179[] = {{&S77212[44], 7}, {NULL}};
const Segment SGI210180[] = {{&S77212[44], 23}, {NULL}};
const Segment SGI210181[] = {{&S81698[34], 13}, {NULL}};
const Segment SGI210182[] = {{&S77212[30], 6}, {NULL}};
const Segment SGI210183[] = {{&S81698[27], 6}, {NULL}};
const Segment SGI210184[] = {{&S77212[116], 3}, {NULL}};
const Segment SGI210185[] = {{&S77212[81], 6}, {NULL}};
const Segment SGI210186[] = {{&S81698[100], 9}, {NULL}};
const Segment SGI210187[] = {{&S81698[73], 6}, {NULL}};
const Segment SGI210188[] = {{&S67554[34], 18}, {NULL}};
const Segment SGI210189[] = {{&S67554[27], 6}, {NULL}};
const Segment SGI210190[] = {{&S67554[104], 9}, {NULL}};
const Segment SGI210191[] = {{&S67554[77], 6}, {NULL}};
const Segment SGI210192[] = {{&S77213[44], 7}, {NULL}};
const Segment SGI210193[] = {{&S77213[44], 23}, {NULL}};
const Segment SGI210194[] = {{&S77213[30], 6}, {NULL}};
const Segment SGI210195[] = {{&S77213[116], 3}, {NULL}};
const Segment SGI210196[] = {{&S77213[81], 6}, {NULL}};
const Segment SGI210197[] = {{&S81700[44], 7}, {NULL}};
const Segment SGI210198[] = {{&S81700[44], 19}, {NULL}};
const Segment SGI210199[] = {{&S81700[30], 6}, {NULL}};
const Segment SGI210200[] = {{&S81700[111], 3}, {NULL}};
const Segment SGI210201[] = {{&S81700[77], 5}, {NULL}};
const Segment SGI210202[] = {{&S77214[44], 7}, {NULL}};
const Segment SGI210203[] = {{&S77214[44], 23}, {NULL}};
const Segment SGI210204[] = {{&S77214[30], 6}, {NULL}};
const Segment SGI210205[] = {{&S77214[116], 3}, {NULL}};
const Segment SGI210206[] = {{&S77214[81], 6}, {NULL}};
const Segment SGI210207[] = {{&S81701[44], 7}, {NULL}};
const Segment SGI210208[] = {{&S81701[44], 19}, {NULL}};
const Segment SGI210209[] = {{&S81701[30], 6}, {NULL}};
const Segment SGI210210[] = {{&S81701[111], 3}, {NULL}};
const Segment SGI210211[] = {{&S81701[77], 5}, {NULL}};
const Segment SGI210212[] = {{&S77215[34], 10}, {NULL}};
const Segment SGI210213[] = {{&S77215[27], 6}, {NULL}};
const Segment SGI210214[] = {{&S77215[97], 9}, {NULL}};
const Segment SGI210215[] = {{&S77215[70], 6}, {NULL}};
const Segment SGI210216[] = {{&S77217[44], 7}, {NULL}};
const Segment SGI210217[] = {{&S77217[44], 24}, {NULL}};
const Segment SGI210218[] = {{&S77217[30], 6}, {NULL}};
const Segment SGI210219[] = {{&S77217[82], 6}, {NULL}};
const Segment SGI210220[] = {{&S77217[109], 13}, {NULL}};
const Segment SGI210221[] = {{&S77218[44], 7}, {NULL}};
const Segment SGI210222[] = {{&S77218[44], 23}, {NULL}};
const Segment SGI210223[] = {{&S77218[30], 6}, {NULL}};
const Segment SGI210224[] = {{&S77218[81], 6}, {NULL}};
const Segment SGI210225[] = {{&S77218[108], 13}, {NULL}};
const Segment SGI210226[] = {{&S77223[6], 4}, {NULL}};
const Segment SGI210227[] = {{&S77223[82], 5}, {NULL}};
const Segment SGI210228[] = {{&S81716[34], 25}, {NULL}};
const Segment SGI210229[] = {{&S81716[27], 6}, {NULL}};
const Segment SGI210230[] = {{&S81716[112], 9}, {NULL}};
const Segment SGI210231[] = {{&S81716[85], 6}, {NULL}};
const Segment SGI210232[] = {{&S77229[34], 17}, {NULL}};
const Segment SGI210233[] = {{&S77229[27], 6}, {NULL}};
const Segment SGI210234[] = {{&S77229[112], 9}, {NULL}};
const Segment SGI210235[] = {{&S77229[85], 6}, {NULL}};
const Segment SGI210236[] = {{&S81720[34], 6}, {NULL}};
const Segment SGI210237[] = {{&S81720[27], 6}, {NULL}};
const Segment SGI210238[] = {{&S81720[93], 9}, {NULL}};
const Segment SGI210239[] = {{&S81720[66], 6}, {NULL}};
const Segment SGI210240[] = {{&S81722[44], 11}, {NULL}};
const Segment SGI210241[] = {{&S81722[44], 23}, {NULL}};
const Segment SGI210242[] = {{&S81722[30], 6}, {NULL}};
const Segment SGI210243[] = {{&S81722[116], 3}, {NULL}};
const Segment SGI210244[] = {{&S81722[81], 6}, {NULL}};
const Segment SGI210245[] = {{&S81723[44], 13}, {NULL}};
const Segment SGI210246[] = {{&S81723[44], 25}, {NULL}};
const Segment SGI210247[] = {{&S81723[30], 6}, {NULL}};
const Segment SGI210248[] = {{&S81723[31], 3}, {NULL}};
const Segment SGI210249[] = {{&S81723[83], 6}, {NULL}};
const Segment SGI210250[] = {{&S67590[34], 9}, {NULL}};
const Segment SGI210251[] = {{&S67590[27], 6}, {NULL}};
const Segment SGI210252[] = {{&S67590[95], 9}, {NULL}};
const Segment SGI210253[] = {{&S67590[68], 6}, {NULL}};
const Segment SGI210254[] = {{&S77248[44], 25}, {NULL}};
const Segment SGI210255[] = {{&S77248[44], 37}, {NULL}};
const Segment SGI210256[] = {{&S77248[30], 6}, {NULL}};
const Segment SGI210257[] = {{&S77248[130], 3}, {NULL}};
const Segment SGI210258[] = {{&S77248[95], 6}, {NULL}};
const Segment SGI210259[] = {{&S86972[34], 10}, {NULL}};
const Segment SGI210260[] = {{&S86972[27], 6}, {NULL}};
const Segment SGI210261[] = {{&S86972[97], 9}, {NULL}};
const Segment SGI210262[] = {{&S86972[70], 6}, {NULL}};
const Segment SGI210263[] = {{&S86973[34], 8}, {NULL}};
const Segment SGI210264[] = {{&S86973[27], 6}, {NULL}};
const Segment SGI210265[] = {{&S86973[95], 9}, {NULL}};
const Segment SGI210266[] = {{&S86973[68], 6}, {NULL}};
const Segment SGI210267[] = {{&S86978[44], 4}, {NULL}};
const Segment SGI210268[] = {{&S86978[44], 16}, {NULL}};
const Segment SGI210269[] = {{&S86978[30], 6}, {NULL}};
const Segment SGI210270[] = {{&S86978[74], 6}, {NULL}};
const Segment SGI210271[] = {{&S86978[101], 13}, {NULL}};
const Segment SGI210272[] = {{&S76221[44], 16}, {NULL}};
const Segment SGI210273[] = {{&S76221[44], 28}, {NULL}};
const Segment SGI210274[] = {{&S76221[30], 6}, {NULL}};
const Segment SGI210275[] = {{&S76221[86], 6}, {NULL}};
const Segment SGI210276[] = {{&S76221[123], 3}, {NULL}};
const Segment SGI210277[] = {{&S86984[34], 9}, {NULL}};
const Segment SGI210278[] = {{&S86984[27], 6}, {NULL}};
const Segment SGI210279[] = {{&S86984[93], 9}, {NULL}};
const Segment SGI210280[] = {{&S86984[66], 6}, {NULL}};
const Segment SGI210281[] = {{&S81761[44], 11}, {NULL}};
const Segment SGI210282[] = {{&S81761[44], 23}, {NULL}};
const Segment SGI210283[] = {{&S81761[30], 6}, {NULL}};
const Segment SGI210284[] = {{&S67629[44], 23}, {NULL}};
const Segment SGI210285[] = {{&S86985[44], 21}, {NULL}};
const Segment SGI210286[] = {{&S81761[115], 3}, {NULL}};
const Segment SGI210287[] = {{&S67629[44], 35}, {NULL}};
const Segment SGI210288[] = {{&S86985[44], 43}, {NULL}};
const Segment SGI210289[] = {{&S67629[30], 6}, {NULL}};
const Segment SGI210290[] = {{&S81761[81], 5}, {NULL}};
const Segment SGI210291[] = {{&S86985[30], 6}, {NULL}};
const Segment SGI210292[] = {{&S67629[146], 3}, {NULL}};
const Segment SGI210293[] = {{&S86985[31], 3}, {NULL}};
const Segment SGI210294[] = {{&S67629[111], 6}, {NULL}};
const Segment SGI210295[] = {{&S86985[101], 6}, {NULL}};
const Segment SGI210296[] = {{&S67629[81], 8}, {NULL}};
const Segment SGI210297[] = {{&S67630[44], 23}, {NULL}};
const Segment SGI210298[] = {{&S67630[44], 34}, {NULL}};
const Segment SGI210299[] = {{&S67630[30], 6}, {NULL}};
const Segment SGI210300[] = {{&S67630[145], 3}, {NULL}};
const Segment SGI210301[] = {{&S67630[110], 6}, {NULL}};
const Segment SGI210302[] = {{&S67630[80], 8}, {NULL}};
const Segment SGI210303[] = {{&S81767[34], 8}, {NULL}};
const Segment SGI210304[] = {{&S81767[27], 6}, {NULL}};
const Segment SGI210305[] = {{&S81767[96], 9}, {NULL}};
const Segment SGI210306[] = {{&S81767[69], 6}, {NULL}};
const Segment SGI210307[] = {{&S81767[130], 8}, {NULL}};
const Segment SGI210308[] = {{&S86992[34], 10}, {NULL}};
const Segment SGI210309[] = {{&S86992[27], 6}, {NULL}};
const Segment SGI210310[] = {{&S86992[97], 9}, {NULL}};
const Segment SGI210311[] = {{&S86992[70], 6}, {NULL}};
const Segment SGI210312[] = {{&S81768[34], 8}, {NULL}};
const Segment SGI210313[] = {{&S81768[27], 6}, {NULL}};
const Segment SGI210314[] = {{&S86992[138], 8}, {NULL}};
const Segment SGI210315[] = {{&S81768[96], 9}, {NULL}};
const Segment SGI210316[] = {{&S81768[69], 6}, {NULL}};
const Segment SGI210317[] = {{&S81768[130], 8}, {NULL}};
const Segment SGI210318[] = {{&S81769[34], 21}, {NULL}};
const Segment SGI210319[] = {{&S81769[27], 6}, {NULL}};
const Segment SGI210320[] = {{&S81769[108], 9}, {NULL}};
const Segment SGI210321[] = {{&S81769[81], 6}, {NULL}};
const Segment SGI210322[] = {{&S67640[38], 8}, {NULL}};
const Segment SGI210323[] = {{&S67640[31], 6}, {NULL}};
const Segment SGI210324[] = {{&S67641[44], 8}, {NULL}};
const Segment SGI210325[] = {{&S67641[44], 28}, {NULL}};
const Segment SGI210326[] = {{&S67641[30], 6}, {NULL}};
const Segment SGI210327[] = {{&S67641[121], 3}, {NULL}};
const Segment SGI210328[] = {{&S67641[86], 6}, {NULL}};
const Segment SGI210329[] = {{&S67642[44], 12}, {NULL}};
const Segment SGI210330[] = {{&S67642[44], 32}, {NULL}};
const Segment SGI210331[] = {{&S67642[30], 6}, {NULL}};
const Segment SGI210332[] = {{&S67642[125], 3}, {NULL}};
const Segment SGI210333[] = {{&S67642[90], 6}, {NULL}};
const Segment SGI210334[] = {{&S87002[34], 9}, {NULL}};
const Segment SGI210335[] = {{&S87002[27], 6}, {NULL}};
const Segment SGI210336[] = {{&S87002[93], 9}, {NULL}};
const Segment SGI210337[] = {{&S87002[66], 6}, {NULL}};
const Segment SGI210338[] = {{&S87012[34], 5}, {NULL}};
const Segment SGI210339[] = {{&S67655[34], 7}, {NULL}};
const Segment SGI210340[] = {{&S87012[27], 6}, {NULL}};
const Segment SGI210341[] = {{&S67655[27], 6}, {NULL}};
const Segment SGI210342[] = {{&S87012[92], 9}, {NULL}};
const Segment SGI210343[] = {{&S87012[65], 6}, {NULL}};
const Segment SGI210344[] = {{&S67655[94], 9}, {NULL}};
const Segment SGI210345[] = {{&S67655[67], 6}, {NULL}};
const Segment SGI210346[] = {{&S87015[34], 6}, {NULL}};
const Segment SGI210347[] = {{&S87015[27], 6}, {NULL}};
const Segment SGI210348[] = {{&S87015[93], 9}, {NULL}};
const Segment SGI210349[] = {{&S87015[66], 6}, {NULL}};
const Segment SGI210350[] = {{&S67689[34], 21}, {NULL}};
const Segment SGI210351[] = {{&S67689[27], 6}, {NULL}};
const Segment SGI210352[] = {{&S67689[107], 9}, {NULL}};
const Segment SGI210353[] = {{&S67689[80], 6}, {NULL}};
const Segment SGI210354[] = {{&S81822[44], 7}, {NULL}};
const Segment SGI210355[] = {{&S81822[44], 24}, {NULL}};
const Segment SGI210356[] = {{&S81822[30], 6}, {NULL}};
const Segment SGI210357[] = {{&S81822[117], 3}, {NULL}};
const Segment SGI210358[] = {{&S67690[34], 20}, {NULL}};
const Segment SGI210359[] = {{&S81822[82], 6}, {NULL}};
const Segment SGI210360[] = {{&S67690[27], 6}, {NULL}};
const Segment SGI210361[] = {{&S81822[141], 10}, {NULL}};
const Segment SGI210362[] = {{&S67690[106], 9}, {NULL}};
const Segment SGI210363[] = {{&S67690[79], 6}, {NULL}};
const Segment SGI210364[] = {{&S81827[44], 11}, {NULL}};
const Segment SGI210365[] = {{&S87058[44], 8}, {NULL}};
const Segment SGI210366[] = {{&S81827[44], 23}, {NULL}};
const Segment SGI210367[] = {{&S81827[30], 6}, {NULL}};
const Segment SGI210368[] = {{&S87058[44], 20}, {NULL}};
const Segment SGI210369[] = {{&S87058[30], 6}, {NULL}};
const Segment SGI210370[] = {{&S81827[116], 3}, {NULL}};
const Segment SGI210371[] = {{&S87058[31], 3}, {NULL}};
const Segment SGI210372[] = {{&S81827[81], 6}, {NULL}};
const Segment SGI210373[] = {{&S87058[78], 6}, {NULL}};
const Segment SGI210374[] = {{&S76299[34], 9}, {NULL}};
const Segment SGI210375[] = {{&S76299[27], 6}, {NULL}};
const Segment SGI210376[] = {{&S76299[96], 9}, {NULL}};
const Segment SGI210377[] = {{&S76299[69], 6}, {NULL}};
const Segment SGI210378[] = {{&S87059[44], 8}, {NULL}};
const Segment SGI210379[] = {{&S87059[44], 20}, {NULL}};
const Segment SGI210380[] = {{&S87059[30], 6}, {NULL}};
const Segment SGI210381[] = {{&S87059[31], 3}, {NULL}};
const Segment SGI210382[] = {{&S87059[78], 6}, {NULL}};
const Segment SGI210383[] = {{&S87060[44], 8}, {NULL}};
const Segment SGI210384[] = {{&S87060[44], 20}, {NULL}};
const Segment SGI210385[] = {{&S87060[30], 6}, {NULL}};
const Segment SGI210386[] = {{&S87060[31], 3}, {NULL}};
const Segment SGI210387[] = {{&S87060[78], 6}, {NULL}};
const Segment SGI210388[] = {{&S87066[44], 10}, {NULL}};
const Segment SGI210389[] = {{&S87066[44], 22}, {NULL}};
const Segment SGI210390[] = {{&S87066[30], 6}, {NULL}};
const Segment SGI210391[] = {{&S87066[31], 3}, {NULL}};
const Segment SGI210392[] = {{&S87066[109], 6}, {NULL}};
const Segment SGI210393[] = {{&S87066[68], 8}, {NULL}};
const Segment SGI210394[] = {{&S87066[80], 15}, {NULL}};
const Segment SGI210395[] = {{&S67709[34], 9}, {NULL}};
const Segment SGI210396[] = {{&S67709[27], 6}, {NULL}};
const Segment SGI210397[] = {{&S67709[95], 9}, {NULL}};
const Segment SGI210398[] = {{&S67709[68], 6}, {NULL}};
const Segment SGI210399[] = {{&S87074[59], 10}, {NULL}};
const Segment SGI210400[] = {{&S87074[59], 22}, {NULL}};
const Segment SGI210401[] = {{&S87074[45], 6}, {NULL}};
const Segment SGI210402[] = {{&S87074[46], 3}, {NULL}};
const Segment SGI210403[] = {{&S87074[95], 6}, {NULL}};
const Segment SGI210404[] = {{&S87074[0], 14}, {NULL}};
const Segment SGI210405[] = {{&S87078[34], 10}, {NULL}};
const Segment SGI210406[] = {{&S87078[27], 6}, {NULL}};
const Segment SGI210407[] = {{&S87078[96], 9}, {NULL}};
const Segment SGI210408[] = {{&S87078[69], 6}, {NULL}};
const Segment SGI210409[] = {{&S77291[44], 10}, {NULL}};
const Segment SGI210410[] = {{&S77291[44], 22}, {NULL}};
const Segment SGI210411[] = {{&S77291[30], 6}, {NULL}};
const Segment SGI210412[] = {{&S77291[135], 3}, {NULL}};
const Segment SGI210413[] = {{&S77291[100], 6}, {NULL}};
const Segment SGI210414[] = {{&S77291[68], 8}, {NULL}};
const Segment SGI210415[] = {{&S77292[44], 10}, {NULL}};
const Segment SGI210416[] = {{&S77292[44], 22}, {NULL}};
const Segment SGI210417[] = {{&S77292[30], 6}, {NULL}};
const Segment SGI210418[] = {{&S77292[135], 3}, {NULL}};
const Segment SGI210419[] = {{&S77292[100], 6}, {NULL}};
const Segment SGI210420[] = {{&S77292[68], 8}, {NULL}};
const Segment SGI210421[] = {{&S77293[44], 10}, {NULL}};
const Segment SGI210422[] = {{&S77293[44], 22}, {NULL}};
const Segment SGI210423[] = {{&S77293[30], 6}, {NULL}};
const Segment SGI210424[] = {{&S77293[135], 3}, {NULL}};
const Segment SGI210425[] = {{&S77293[100], 6}, {NULL}};
const Segment SGI210426[] = {{&S77293[68], 8}, {NULL}};
const Segment SGI210427[] = {{&S87091[45], 10}, {NULL}};
const Segment SGI210428[] = {{&S87091[30], 6}, {NULL}};
const Segment SGI210429[] = {{&S87091[31], 3}, {NULL}};
const Segment SGI210430[] = {{&S87091[81], 5}, {NULL}};
const Segment SGI210431[] = {{&S87091[139], 13}, {NULL}};
const Segment SGI210432[] = {{&S77299[44], 20}, {NULL}};
const Segment SGI210433[] = {{&S77299[44], 32}, {NULL}};
const Segment SGI210434[] = {{&S77299[30], 6}, {NULL}};
const Segment SGI210435[] = {{&S77299[141], 3}, {NULL}};
const Segment SGI210436[] = {{&S77299[106], 6}, {NULL}};
const Segment SGI210437[] = {{&S77299[78], 8}, {NULL}};
const Segment SGI210438[] = {{&S87093[34], 9}, {NULL}};
const Segment SGI210439[] = {{&S87093[27], 6}, {NULL}};
const Segment SGI210440[] = {{&S87093[96], 9}, {NULL}};
const Segment SGI210441[] = {{&S87093[69], 6}, {NULL}};
const Segment SGI210442[] = {{&S77302[34], 10}, {NULL}};
const Segment SGI210443[] = {{&S77302[27], 6}, {NULL}};
const Segment SGI210444[] = {{&S77302[97], 9}, {NULL}};
const Segment SGI210445[] = {{&S77302[70], 6}, {NULL}};
const Segment SGI210446[] = {{&S77303[34], 10}, {NULL}};
const Segment SGI210447[] = {{&S77303[27], 6}, {NULL}};
const Segment SGI210448[] = {{&S77303[97], 9}, {NULL}};
const Segment SGI210449[] = {{&S77303[70], 6}, {NULL}};
const Segment SGI210450[] = {{&S87101[34], 10}, {NULL}};
const Segment SGI210451[] = {{&S87101[27], 6}, {NULL}};
const Segment SGI210452[] = {{&S87101[97], 9}, {NULL}};
const Segment SGI210453[] = {{&S87101[70], 6}, {NULL}};
const Segment SGI210454[] = {{&S87102[27], 6}, {NULL}};
const Segment SGI210455[] = {{&S87102[99], 9}, {NULL}};
const Segment SGI210456[] = {{&S87102[72], 6}, {NULL}};
const Segment SGI210457[] = {{&S87103[34], 11}, {NULL}};
const Segment SGI210458[] = {{&S87103[27], 6}, {NULL}};
const Segment SGI210459[] = {{&S87103[98], 9}, {NULL}};
const Segment SGI210460[] = {{&S87103[71], 6}, {NULL}};
const Segment SGI210461[] = {{&S87105[44], 8}, {NULL}};
const Segment SGI210462[] = {{&S87105[44], 20}, {NULL}};
const Segment SGI210463[] = {{&S87105[30], 6}, {NULL}};
const Segment SGI210464[] = {{&S87105[31], 3}, {NULL}};
const Segment SGI210465[] = {{&S87105[78], 6}, {NULL}};
const Segment SGI210466[] = {{&S87108[44], 8}, {NULL}};
const Segment SGI210467[] = {{&S87108[44], 20}, {NULL}};
const Segment SGI210468[] = {{&S87108[30], 6}, {NULL}};
const Segment SGI210469[] = {{&S87108[31], 3}, {NULL}};
const Segment SGI210470[] = {{&S87108[78], 6}, {NULL}};
const Segment SGI210471[] = {{&S81892[45], 10}, {NULL}};
const Segment SGI210472[] = {{&S81892[27], 17}, {NULL}};
const Segment SGI210473[] = {{&S81892[108], 9}, {NULL}};
const Segment SGI210474[] = {{&S81892[81], 6}, {NULL}};
const Segment SGI210475[] = {{&S76359[34], 10}, {NULL}};
const Segment SGI210476[] = {{&S76359[27], 6}, {NULL}};
const Segment SGI210477[] = {{&S76359[110], 9}, {NULL}};
const Segment SGI210478[] = {{&S76359[83], 6}, {NULL}};
const Segment SGI210479[] = {{&S87116[34], 5}, {NULL}};
const Segment SGI210480[] = {{&S81898[38], 7}, {NULL}};
const Segment SGI210481[] = {{&S87116[27], 6}, {NULL}};
const Segment SGI210482[] = {{&S81898[31], 6}, {NULL}};
const Segment SGI210483[] = {{&S87116[92], 9}, {NULL}};
const Segment SGI210484[] = {{&S87116[65], 6}, {NULL}};
const Segment SGI210485[] = {{&S76361[34], 6}, {NULL}};
const Segment SGI210486[] = {{&S76361[27], 6}, {NULL}};
const Segment SGI210487[] = {{&S76361[93], 9}, {NULL}};
const Segment SGI210488[] = {{&S76361[66], 6}, {NULL}};
const Segment SGI210489[] = {{&S87127[34], 13}, {NULL}};
const Segment SGI210490[] = {{&S87127[27], 6}, {NULL}};
const Segment SGI210491[] = {{&S87127[100], 9}, {NULL}};
const Segment SGI210492[] = {{&S87127[73], 6}, {NULL}};
const Segment SGI210493[] = {{&S87131[34], 14}, {NULL}};
const Segment SGI210494[] = {{&S87131[27], 6}, {NULL}};
const Segment SGI210495[] = {{&S87131[101], 9}, {NULL}};
const Segment SGI210496[] = {{&S87131[74], 6}, {NULL}};
const Segment SGI210497[] = {{&S67793[34], 19}, {NULL}};
const Segment SGI210498[] = {{&S67793[27], 6}, {NULL}};
const Segment SGI210499[] = {{&S67793[105], 9}, {NULL}};
const Segment SGI210500[] = {{&S67793[78], 6}, {NULL}};
const Segment SGI210501[] = {{&S81918[44], 10}, {NULL}};
const Segment SGI210502[] = {{&S81918[44], 22}, {NULL}};
const Segment SGI210503[] = {{&S81918[30], 6}, {NULL}};
const Segment SGI210504[] = {{&S81918[113], 3}, {NULL}};
const Segment SGI210505[] = {{&S81918[80], 5}, {NULL}};
const Segment SGI210506[] = {{&S81918[117], 14}, {NULL}};
const Segment SGI210507[] = {{&S67795[34], 13}, {NULL}};
const Segment SGI210508[] = {{&S67795[27], 6}, {NULL}};
const Segment SGI210509[] = {{&S67795[99], 9}, {NULL}};
const Segment SGI210510[] = {{&S67795[72], 6}, {NULL}};
const Segment SGI210511[] = {{&S87146[44], 9}, {NULL}};
const Segment SGI210512[] = {{&S87146[44], 21}, {NULL}};
const Segment SGI210513[] = {{&S87146[30], 6}, {NULL}};
const Segment SGI210514[] = {{&S87146[31], 3}, {NULL}};
const Segment SGI210515[] = {{&S87146[79], 6}, {NULL}};
const Segment SGI210516[] = {{&S67809[34], 8}, {NULL}};
const Segment SGI210517[] = {{&S67809[27], 6}, {NULL}};
const Segment SGI210518[] = {{&S67809[94], 9}, {NULL}};
const Segment SGI210519[] = {{&S67809[67], 6}, {NULL}};
const Segment SGI210520[] = {{&S87154[34], 7}, {NULL}};
const Segment SGI210521[] = {{&S87154[27], 6}, {NULL}};
const Segment SGI210522[] = {{&S87154[90], 9}, {NULL}};
const Segment SGI210523[] = {{&S87154[63], 6}, {NULL}};
const Segment SGI210524[] = {{&S87162[34], 13}, {NULL}};
const Segment SGI210525[] = {{&S67825[34], 13}, {NULL}};
const Segment SGI210526[] = {{&S87162[27], 6}, {NULL}};
const Segment SGI210527[] = {{&S67825[27], 6}, {NULL}};
const Segment SGI210528[] = {{&S87162[100], 9}, {NULL}};
const Segment SGI210529[] = {{&S67825[99], 9}, {NULL}};
const Segment SGI210530[] = {{&S87162[73], 6}, {NULL}};
const Segment SGI210531[] = {{&S67825[72], 6}, {NULL}};
const Segment SGI210532[] = {{&S87163[34], 14}, {NULL}};
const Segment SGI210533[] = {{&S67826[34], 19}, {NULL}};
const Segment SGI210534[] = {{&S87163[27], 6}, {NULL}};
const Segment SGI210535[] = {{&S67826[27], 6}, {NULL}};
const Segment SGI210536[] = {{&S87163[101], 9}, {NULL}};
const Segment SGI210537[] = {{&S87163[74], 6}, {NULL}};
const Segment SGI210538[] = {{&S67826[105], 9}, {NULL}};
const Segment SGI210539[] = {{&S67826[78], 6}, {NULL}};
const Segment SGI210540[] = {{&S87170[34], 10}, {NULL}};
const Segment SGI210541[] = {{&S87170[27], 6}, {NULL}};
const Segment SGI210542[] = {{&S87170[97], 9}, {NULL}};
const Segment SGI210543[] = {{&S87170[70], 6}, {NULL}};
const Segment SGI210544[] = {{&S87171[34], 10}, {NULL}};
const Segment SGI210545[] = {{&S87171[27], 6}, {NULL}};
const Segment SGI210546[] = {{&S87171[97], 9}, {NULL}};
const Segment SGI210547[] = {{&S87171[70], 6}, {NULL}};
const Segment SGI210548[] = {{&S87172[34], 15}, {NULL}};
const Segment SGI210549[] = {{&S87172[27], 6}, {NULL}};
const Segment SGI210550[] = {{&S87172[102], 9}, {NULL}};
const Segment SGI210551[] = {{&S87172[75], 6}, {NULL}};
const Segment SGI210552[] = {{&S87175[34], 10}, {NULL}};
const Segment SGI210553[] = {{&S87175[27], 6}, {NULL}};
const Segment SGI210554[] = {{&S87175[97], 9}, {NULL}};
const Segment SGI210555[] = {{&S87175[70], 6}, {NULL}};
const Segment SGI210556[] = {{&S80293[32], 10}, {NULL}};
const Segment SGI210557[] = {{&S80293[27], 4}, {NULL}};
const Segment SGI210558[] = {{&S80293[95], 9}, {NULL}};
const Segment SGI210559[] = {{&S80293[68], 6}, {NULL}};
const Segment SGI210560[] = {{&S77313[34], 6}, {NULL}};
const Segment SGI210561[] = {{&S77313[27], 6}, {NULL}};
const Segment SGI210562[] = {{&S77313[93], 9}, {NULL}};
const Segment SGI210563[] = {{&S77313[66], 6}, {NULL}};
const Segment SGI210564[] = {{&S77314[34], 11}, {NULL}};
const Segment SGI210565[] = {{&S77314[27], 6}, {NULL}};
const Segment SGI210566[] = {{&S77314[98], 9}, {NULL}};
const Segment SGI210567[] = {{&S77314[71], 6}, {NULL}};
const Segment SGI210568[] = {{&S67856[44], 10}, {NULL}};
const Segment SGI210569[] = {{&S67856[44], 21}, {NULL}};
const Segment SGI210570[] = {{&S67856[30], 6}, {NULL}};
const Segment SGI210571[] = {{&S77316[34], 13}, {NULL}};
const Segment SGI210572[] = {{&S67856[132], 3}, {NULL}};
const Segment SGI210573[] = {{&S77316[27], 6}, {NULL}};
const Segment SGI210574[] = {{&S67856[97], 6}, {NULL}};
const Segment SGI210575[] = {{&S67856[67], 8}, {NULL}};
const Segment SGI210576[] = {{&S77316[100], 9}, {NULL}};
const Segment SGI210577[] = {{&S77316[73], 6}, {NULL}};
const Segment SGI210578[] = {{&S67857[44], 10}, {NULL}};
const Segment SGI210579[] = {{&S67857[44], 22}, {NULL}};
const Segment SGI210580[] = {{&S67857[30], 6}, {NULL}};
const Segment SGI210581[] = {{&S77317[34], 19}, {NULL}};
const Segment SGI210582[] = {{&S77317[27], 6}, {NULL}};
const Segment SGI210583[] = {{&S67857[133], 3}, {NULL}};
const Segment SGI210584[] = {{&S67857[98], 6}, {NULL}};
const Segment SGI210585[] = {{&S67857[68], 8}, {NULL}};
const Segment SGI210586[] = {{&S77317[106], 9}, {NULL}};
const Segment SGI210587[] = {{&S77317[79], 6}, {NULL}};
const Segment SGI210588[] = {{&S67858[44], 10}, {NULL}};
const Segment SGI210589[] = {{&S67858[44], 24}, {NULL}};
const Segment SGI210590[] = {{&S67858[30], 6}, {NULL}};
const Segment SGI210591[] = {{&S77318[44], 10}, {NULL}};
const Segment SGI210592[] = {{&S77318[44], 22}, {NULL}};
const Segment SGI210593[] = {{&S77318[30], 6}, {NULL}};
const Segment SGI210594[] = {{&S67858[141], 3}, {NULL}};
const Segment SGI210595[] = {{&S67858[106], 6}, {NULL}};
const Segment SGI210596[] = {{&S67858[70], 8}, {NULL}};
const Segment SGI210597[] = {{&S77318[80], 6}, {NULL}};
const Segment SGI210598[] = {{&S77318[107], 13}, {NULL}};
const Segment SGI210599[] = {{&S85954[42], 11}, {NULL}};
const Segment SGI210600[] = {{&S85954[42], 23}, {NULL}};
const Segment SGI210601[] = {{&S85954[30], 4}, {NULL}};
const Segment SGI210602[] = {{&S85954[114], 3}, {NULL}};
const Segment SGI210603[] = {{&S85954[79], 6}, {NULL}};
const Segment SGI210604[] = {{&S87206[44], 11}, {NULL}};
const Segment SGI210605[] = {{&S87206[44], 23}, {NULL}};
const Segment SGI210606[] = {{&S87206[30], 6}, {NULL}};
const Segment SGI210607[] = {{&S87206[31], 3}, {NULL}};
const Segment SGI210608[] = {{&S87206[81], 6}, {NULL}};
const Segment SGI210609[] = {{&S87207[44], 14}, {NULL}};
const Segment SGI210610[] = {{&S87207[44], 26}, {NULL}};
const Segment SGI210611[] = {{&S87207[30], 6}, {NULL}};
const Segment SGI210612[] = {{&S87207[31], 3}, {NULL}};
const Segment SGI210613[] = {{&S87207[84], 6}, {NULL}};
const Segment SGI210614[] = {{&S67874[44], 12}, {NULL}};
const Segment SGI210615[] = {{&S67874[44], 23}, {NULL}};
const Segment SGI210616[] = {{&S67874[30], 6}, {NULL}};
const Segment SGI210617[] = {{&S77336[34], 10}, {NULL}};
const Segment SGI210618[] = {{&S77336[27], 6}, {NULL}};
const Segment SGI210619[] = {{&S67874[116], 3}, {NULL}};
const Segment SGI210620[] = {{&S67874[81], 6}, {NULL}};
const Segment SGI210621[] = {{&S77336[97], 9}, {NULL}};
const Segment SGI210622[] = {{&S77336[70], 6}, {NULL}};
const Segment SGI210623[] = {{&S87211[44], 10}, {NULL}};
const Segment SGI210624[] = {{&S87211[44], 22}, {NULL}};
const Segment SGI210625[] = {{&S87211[30], 6}, {NULL}};
const Segment SGI210626[] = {{&S87211[31], 3}, {NULL}};
const Segment SGI210627[] = {{&S87211[80], 6}, {NULL}};
const Segment SGI210628[] = {{&S67876[44], 12}, {NULL}};
const Segment SGI210629[] = {{&S67876[44], 24}, {NULL}};
const Segment SGI210630[] = {{&S67876[30], 6}, {NULL}};
const Segment SGI210631[] = {{&S67876[117], 3}, {NULL}};
const Segment SGI210632[] = {{&S67876[82], 6}, {NULL}};
const Segment SGI210633[] = {{&S87213[59], 13}, {NULL}};
const Segment SGI210634[] = {{&S87213[59], 25}, {NULL}};
const Segment SGI210635[] = {{&S87213[45], 6}, {NULL}};
const Segment SGI210636[] = {{&S87213[46], 3}, {NULL}};
const Segment SGI210637[] = {{&S87213[98], 6}, {NULL}};
const Segment SGI210638[] = {{&S87213[0], 14}, {NULL}};
const Segment SGI210639[] = {{&S87214[34], 19}, {NULL}};
const Segment SGI210640[] = {{&S87214[27], 6}, {NULL}};
const Segment SGI210641[] = {{&S87214[106], 9}, {NULL}};
const Segment SGI210642[] = {{&S87214[79], 6}, {NULL}};
const Segment SGI210643[] = {{&S76475[34], 16}, {NULL}};
const Segment SGI210644[] = {{&S76475[27], 6}, {NULL}};
const Segment SGI210645[] = {{&S76475[103], 9}, {NULL}};
const Segment SGI210646[] = {{&S76475[76], 6}, {NULL}};
const Segment SGI210647[] = {{&S76476[34], 16}, {NULL}};
const Segment SGI210648[] = {{&S76476[27], 6}, {NULL}};
const Segment SGI210649[] = {{&S87219[34], 21}, {NULL}};
const Segment SGI210650[] = {{&S87219[27], 6}, {NULL}};
const Segment SGI210651[] = {{&S76476[103], 9}, {NULL}};
const Segment SGI210652[] = {{&S76476[76], 6}, {NULL}};
const Segment SGI210653[] = {{&S87219[108], 9}, {NULL}};
const Segment SGI210654[] = {{&S87219[81], 6}, {NULL}};
const Segment SGI210655[] = {{&S77350[34], 8}, {NULL}};
const Segment SGI210656[] = {{&S77350[27], 6}, {NULL}};
const Segment SGI210657[] = {{&S87222[44], 8}, {NULL}};
const Segment SGI210658[] = {{&S77350[95], 9}, {NULL}};
const Segment SGI210659[] = {{&S87222[44], 20}, {NULL}};
const Segment SGI210660[] = {{&S77350[68], 6}, {NULL}};
const Segment SGI210661[] = {{&S87222[30], 6}, {NULL}};
const Segment SGI210662[] = {{&S87222[31], 3}, {NULL}};
const Segment SGI210663[] = {{&S87222[78], 6}, {NULL}};
const Segment SGI210664[] = {{&S77351[34], 8}, {NULL}};
const Segment SGI210665[] = {{&S77351[27], 6}, {NULL}};
const Segment SGI210666[] = {{&S77351[102], 9}, {NULL}};
const Segment SGI210667[] = {{&S77351[75], 6}, {NULL}};
const Segment SGI210668[] = {{&S87223[44], 10}, {NULL}};
const Segment SGI210669[] = {{&S87223[44], 22}, {NULL}};
const Segment SGI210670[] = {{&S87223[30], 6}, {NULL}};
const Segment SGI210671[] = {{&S87223[31], 3}, {NULL}};
const Segment SGI210672[] = {{&S87223[80], 5}, {NULL}};
const Segment SGI210673[] = {{&S87223[117], 14}, {NULL}};
const Segment SGI210674[] = {{&S87224[34], 10}, {NULL}};
const Segment SGI210675[] = {{&S87224[27], 6}, {NULL}};
const Segment SGI210676[] = {{&S87224[97], 9}, {NULL}};
const Segment SGI210677[] = {{&S87224[70], 6}, {NULL}};
const Segment SGI210678[] = {{&S77358[44], 10}, {NULL}};
const Segment SGI210679[] = {{&S87227[34], 15}, {NULL}};
const Segment SGI210680[] = {{&S77358[44], 22}, {NULL}};
const Segment SGI210681[] = {{&S87227[27], 6}, {NULL}};
const Segment SGI210682[] = {{&S77358[30], 6}, {NULL}};
const Segment SGI210683[] = {{&S87227[102], 9}, {NULL}};
const Segment SGI210684[] = {{&S87227[75], 6}, {NULL}};
const Segment SGI210685[] = {{&S77358[80], 6}, {NULL}};
const Segment SGI210686[] = {{&S77358[107], 13}, {NULL}};
const Segment SGI210687[] = {{&S87231[34], 7}, {NULL}};
const Segment SGI210688[] = {{&S87231[27], 6}, {NULL}};
const Segment SGI210689[] = {{&S87231[94], 9}, {NULL}};
const Segment SGI210690[] = {{&S87231[67], 6}, {NULL}};
const Segment SGI210691[] = {{&S87233[34], 10}, {NULL}};
const Segment SGI210692[] = {{&S87233[27], 6}, {NULL}};
const Segment SGI210693[] = {{&S87233[97], 9}, {NULL}};
const Segment SGI210694[] = {{&S87233[70], 6}, {NULL}};
const Segment SGI210695[] = {{&S87244[34], 7}, {NULL}};
const Segment SGI210696[] = {{&S87244[27], 6}, {NULL}};
const Segment SGI210697[] = {{&S87244[98], 9}, {NULL}};
const Segment SGI210698[] = {{&S87244[71], 6}, {NULL}};
const Segment SGI210699[] = {{&S87251[34], 18}, {NULL}};
const Segment SGI210700[] = {{&S87251[27], 6}, {NULL}};
const Segment SGI210701[] = {{&S87251[105], 9}, {NULL}};
const Segment SGI210702[] = {{&S87251[78], 6}, {NULL}};
const Segment SGI210703[] = {{&S87252[34], 20}, {NULL}};
const Segment SGI210704[] = {{&S87252[27], 6}, {NULL}};
const Segment SGI210705[] = {{&S87252[107], 9}, {NULL}};
const Segment SGI210706[] = {{&S87252[80], 6}, {NULL}};
const Segment SGI210707[] = {{&S87253[34], 21}, {NULL}};
const Segment SGI210708[] = {{&S87253[27], 6}, {NULL}};
const Segment SGI210709[] = {{&S87253[108], 9}, {NULL}};
const Segment SGI210710[] = {{&S87253[81], 6}, {NULL}};
const Segment SGI210711[] = {{&S87257[34], 10}, {NULL}};
const Segment SGI210712[] = {{&S87257[27], 6}, {NULL}};
const Segment SGI210713[] = {{&S87257[97], 9}, {NULL}};
const Segment SGI210714[] = {{&S87257[70], 6}, {NULL}};
const Segment SGI210715[] = {{&S87259[52], 10}, {NULL}};
const Segment SGI210716[] = {{&S87259[52], 22}, {NULL}};
const Segment SGI210717[] = {{&S87259[38], 6}, {NULL}};
const Segment SGI210718[] = {{&S87259[88], 5}, {NULL}};
const Segment SGI210719[] = {{&S87259[0], 14}, {NULL}};
const Segment SGI210720[] = {{&S87260[34], 9}, {NULL}};
const Segment SGI210721[] = {{&S87260[27], 6}, {NULL}};
const Segment SGI210722[] = {{&S87260[96], 9}, {NULL}};
const Segment SGI210723[] = {{&S87260[69], 6}, {NULL}};
const Segment SGI210724[] = {{&S67937[44], 9}, {NULL}};
const Segment SGI210725[] = {{&S87261[34], 8}, {NULL}};
const Segment SGI210726[] = {{&S67937[44], 20}, {NULL}};
const Segment SGI210727[] = {{&S87261[27], 6}, {NULL}};
const Segment SGI210728[] = {{&S67937[30], 6}, {NULL}};
const Segment SGI210729[] = {{&S67937[142], 3}, {NULL}};
const Segment SGI210730[] = {{&S87261[94], 9}, {NULL}};
const Segment SGI210731[] = {{&S87261[67], 6}, {NULL}};
const Segment SGI210732[] = {{&S67937[107], 6}, {NULL}};
const Segment SGI210733[] = {{&S87262[34], 7}, {NULL}};
const Segment SGI210734[] = {{&S87262[27], 6}, {NULL}};
const Segment SGI210735[] = {{&S67938[44], 9}, {NULL}};
const Segment SGI210736[] = {{&S67938[44], 21}, {NULL}};
const Segment SGI210737[] = {{&S67938[30], 6}, {NULL}};
const Segment SGI210738[] = {{&S87262[97], 9}, {NULL}};
const Segment SGI210739[] = {{&S87262[70], 6}, {NULL}};
const Segment SGI210740[] = {{&S77408[34], 8}, {NULL}};
const Segment SGI210741[] = {{&S67938[114], 3}, {NULL}};
const Segment SGI210742[] = {{&S87262[138], 8}, {NULL}};
const Segment SGI210743[] = {{&S77408[27], 6}, {NULL}};
const Segment SGI210744[] = {{&S67938[79], 6}, {NULL}};
const Segment SGI210745[] = {{&S77408[99], 9}, {NULL}};
const Segment SGI210746[] = {{&S77408[72], 6}, {NULL}};
const Segment SGI210747[] = {{&S87268[34], 8}, {NULL}};
const Segment SGI210748[] = {{&S87268[27], 6}, {NULL}};
const Segment SGI210749[] = {{&S87268[95], 9}, {NULL}};
const Segment SGI210750[] = {{&S87268[68], 6}, {NULL}};
const Segment SGI210751[] = {{&S77415[34], 10}, {NULL}};
const Segment SGI210752[] = {{&S77415[27], 6}, {NULL}};
const Segment SGI210753[] = {{&S77415[97], 9}, {NULL}};
const Segment SGI210754[] = {{&S77415[70], 6}, {NULL}};
const Segment SGI210755[] = {{&S87269[34], 7}, {NULL}};
const Segment SGI210756[] = {{&S87269[27], 6}, {NULL}};
const Segment SGI210757[] = {{&S87269[99], 9}, {NULL}};
const Segment SGI210758[] = {{&S87269[72], 6}, {NULL}};
const Segment SGI210759[] = {{&S77417[34], 10}, {NULL}};
const Segment SGI210760[] = {{&S77417[27], 6}, {NULL}};
const Segment SGI210761[] = {{&S77417[97], 9}, {NULL}};
const Segment SGI210762[] = {{&S77417[70], 6}, {NULL}};
const Segment SGI210763[] = {{&S67952[34], 10}, {NULL}};
const Segment SGI210764[] = {{&S67952[27], 6}, {NULL}};
const Segment SGI210765[] = {{&S67952[97], 9}, {NULL}};
const Segment SGI210766[] = {{&S67952[70], 6}, {NULL}};
const Segment SGI210767[] = {{&S87281[34], 13}, {NULL}};
const Segment SGI210768[] = {{&S87281[27], 6}, {NULL}};
const Segment SGI210769[] = {{&S87281[100], 9}, {NULL}};
const Segment SGI210770[] = {{&S87281[73], 6}, {NULL}};
const Segment SGI210771[] = {{&S77428[44], 10}, {NULL}};
const Segment SGI210772[] = {{&S77428[44], 22}, {NULL}};
const Segment SGI210773[] = {{&S77428[30], 6}, {NULL}};
const Segment SGI210774[] = {{&S77428[131], 3}, {NULL}};
const Segment SGI210775[] = {{&S77428[96], 6}, {NULL}};
const Segment SGI210776[] = {{&S77428[68], 8}, {NULL}};
const Segment SGI210777[] = {{&S76568[44], 10}, {NULL}};
const Segment SGI210778[] = {{&S81990[34], 11}, {NULL}};
const Segment SGI210779[] = {{&S76568[44], 22}, {NULL}};
const Segment SGI210780[] = {{&S76568[30], 6}, {NULL}};
const Segment SGI210781[] = {{&S81990[27], 6}, {NULL}};
const Segment SGI210782[] = {{&S76568[115], 3}, {NULL}};
const Segment SGI210783[] = {{&S81990[98], 9}, {NULL}};
const Segment SGI210784[] = {{&S76568[80], 6}, {NULL}};
const Segment SGI210785[] = {{&S81990[71], 6}, {NULL}};
const Segment SGI210786[] = {{&S81991[44], 10}, {NULL}};
const Segment SGI210787[] = {{&S81991[44], 22}, {NULL}};
const Segment SGI210788[] = {{&S81991[30], 6}, {NULL}};
const Segment SGI210789[] = {{&S81991[115], 3}, {NULL}};
const Segment SGI210790[] = {{&S81991[80], 6}, {NULL}};
const Segment SGI210791[] = {{&S77433[34], 10}, {NULL}};
const Segment SGI210792[] = {{&S77433[27], 6}, {NULL}};
const Segment SGI210793[] = {{&S67972[34], 11}, {NULL}};
const Segment SGI210794[] = {{&S67972[27], 6}, {NULL}};
const Segment SGI210795[] = {{&S67972[97], 9}, {NULL}};
const Segment SGI210796[] = {{&S77433[97], 9}, {NULL}};
const Segment SGI210797[] = {{&S67972[70], 6}, {NULL}};
const Segment SGI210798[] = {{&S77433[70], 6}, {NULL}};
const Segment SGI210799[] = {{&S77437[44], 6}, {NULL}};
const Segment SGI210800[] = {{&S77437[44], 22}, {NULL}};
const Segment SGI210801[] = {{&S77437[30], 6}, {NULL}};
const Segment SGI210802[] = {{&S81998[34], 7}, {NULL}};
const Segment SGI210803[] = {{&S77437[115], 3}, {NULL}};
const Segment SGI210804[] = {{&S81998[27], 6}, {NULL}};
const Segment SGI210805[] = {{&S77437[80], 6}, {NULL}};
const Segment SGI210806[] = {{&S81998[94], 9}, {NULL}};
const Segment SGI210807[] = {{&S81998[67], 6}, {NULL}};
const Segment SGI210808[] = {{&S77438[38], 6}, {NULL}};
const Segment SGI210809[] = {{&S77438[31], 6}, {NULL}};
const Segment SGI210810[] = {{&S77439[44], 6}, {NULL}};
const Segment SGI210811[] = {{&S77439[44], 22}, {NULL}};
const Segment SGI210812[] = {{&S77439[30], 6}, {NULL}};
const Segment SGI210813[] = {{&S77439[115], 3}, {NULL}};
const Segment SGI210814[] = {{&S77439[80], 6}, {NULL}};
const Segment SGI210815[] = {{&S77440[44], 11}, {NULL}};
const Segment SGI210816[] = {{&S77440[44], 23}, {NULL}};
const Segment SGI210817[] = {{&S77440[30], 6}, {NULL}};
const Segment SGI210818[] = {{&S77440[116], 3}, {NULL}};
const Segment SGI210819[] = {{&S77440[81], 6}, {NULL}};
const Segment SGI210820[] = {{&S77441[38], 7}, {NULL}};
const Segment SGI210821[] = {{&S77441[31], 6}, {NULL}};
const Segment SGI210822[] = {{&S87313[44], 10}, {NULL}};
const Segment SGI210823[] = {{&S87313[44], 22}, {NULL}};
const Segment SGI210824[] = {{&S87313[30], 6}, {NULL}};
const Segment SGI210825[] = {{&S87313[31], 3}, {NULL}};
const Segment SGI210826[] = {{&S87313[80], 6}, {NULL}};
const Segment SGI210827[] = {{&S87313[150], 3}, {NULL}};
const Segment SGI210828[] = {{&S77442[44], 7}, {NULL}};
const Segment SGI210829[] = {{&S77442[44], 25}, {NULL}};
const Segment SGI210830[] = {{&S77442[30], 6}, {NULL}};
const Segment SGI210831[] = {{&S77442[118], 3}, {NULL}};
const Segment SGI210832[] = {{&S77442[83], 6}, {NULL}};
const Segment SGI210833[] = {{&S77444[44], 14}, {NULL}};
const Segment SGI210834[] = {{&S77444[44], 26}, {NULL}};
const Segment SGI210835[] = {{&S77444[30], 6}, {NULL}};
const Segment SGI210836[] = {{&S76594[34], 8}, {NULL}};
const Segment SGI210837[] = {{&S77444[135], 3}, {NULL}};
const Segment SGI210838[] = {{&S76594[27], 6}, {NULL}};
const Segment SGI210839[] = {{&S77444[100], 6}, {NULL}};
const Segment SGI210840[] = {{&S77444[72], 8}, {NULL}};
const Segment SGI210841[] = {{&S76594[95], 9}, {NULL}};
const Segment SGI210842[] = {{&S76594[68], 6}, {NULL}};
const Segment SGI210843[] = {{&S77445[34], 18}, {NULL}};
const Segment SGI210844[] = {{&S77445[27], 6}, {NULL}};
const Segment SGI210845[] = {{&S77445[105], 9}, {NULL}};
const Segment SGI210846[] = {{&S77445[78], 6}, {NULL}};
const Segment SGI210847[] = {{&S82016[44], 18}, {NULL}};
const Segment SGI210848[] = {{&S82016[44], 30}, {NULL}};
const Segment SGI210849[] = {{&S82016[30], 6}, {NULL}};
const Segment SGI210850[] = {{&S77446[34], 18}, {NULL}};
const Segment SGI210851[] = {{&S82016[88], 6}, {NULL}};
const Segment SGI210852[] = {{&S77446[27], 6}, {NULL}};
const Segment SGI210853[] = {{&S82016[115], 13}, {NULL}};
const Segment SGI210854[] = {{&S77446[105], 9}, {NULL}};
const Segment SGI210855[] = {{&S77446[78], 6}, {NULL}};
const Segment SGI210856[] = {{&S87327[44], 10}, {NULL}};
const Segment SGI210857[] = {{&S87327[44], 22}, {NULL}};
const Segment SGI210858[] = {{&S87327[30], 6}, {NULL}};
const Segment SGI210859[] = {{&S87327[31], 3}, {NULL}};
const Segment SGI210860[] = {{&S87327[80], 6}, {NULL}};
const Segment SGI210861[] = {{&S77450[34], 4}, {NULL}};
const Segment SGI210862[] = {{&S77450[27], 6}, {NULL}};
const Segment SGI210863[] = {{&S77450[91], 9}, {NULL}};
const Segment SGI210864[] = {{&S77450[64], 6}, {NULL}};
const Segment SGI210865[] = {{&S87331[34], 10}, {NULL}};
const Segment SGI210866[] = {{&S87331[27], 6}, {NULL}};
const Segment SGI210867[] = {{&S87331[97], 9}, {NULL}};
const Segment SGI210868[] = {{&S87331[70], 6}, {NULL}};
const Segment SGI210869[] = {{&S87332[59], 11}, {NULL}};
const Segment SGI210870[] = {{&S87332[59], 23}, {NULL}};
const Segment SGI210871[] = {{&S87332[45], 6}, {NULL}};
const Segment SGI210872[] = {{&S87332[46], 3}, {NULL}};
const Segment SGI210873[] = {{&S87332[96], 6}, {NULL}};
const Segment SGI210874[] = {{&S87332[0], 14}, {NULL}};
const Segment SGI210875[] = {{&S77454[34], 14}, {NULL}};
const Segment SGI210876[] = {{&S77454[27], 6}, {NULL}};
const Segment SGI210877[] = {{&S77454[111], 9}, {NULL}};
const Segment SGI210878[] = {{&S77454[84], 6}, {NULL}};
const Segment SGI210879[] = {{&S77454[152], 8}, {NULL}};
const Segment SGI210880[] = {{&S77455[34], 9}, {NULL}};
const Segment SGI210881[] = {{&S77455[27], 6}, {NULL}};
const Segment SGI210882[] = {{&S77455[96], 9}, {NULL}};
const Segment SGI210883[] = {{&S77455[69], 6}, {NULL}};
const Segment SGI210884[] = {{&S68016[34], 10}, {NULL}};
const Segment SGI210885[] = {{&S68016[27], 6}, {NULL}};
const Segment SGI210886[] = {{&S68016[96], 9}, {NULL}};
const Segment SGI210887[] = {{&S87339[34], 13}, {NULL}};
const Segment SGI210888[] = {{&S68016[69], 6}, {NULL}};
const Segment SGI210889[] = {{&S87339[27], 6}, {NULL}};
const Segment SGI210890[] = {{&S68016[137], 8}, {NULL}};
const Segment SGI210891[] = {{&S87339[113], 9}, {NULL}};
const Segment SGI210892[] = {{&S87339[86], 6}, {NULL}};
const Segment SGI210893[] = {{&S77456[34], 6}, {NULL}};
const Segment SGI210894[] = {{&S77456[27], 6}, {NULL}};
const Segment SGI210895[] = {{&S77456[93], 9}, {NULL}};
const Segment SGI210896[] = {{&S77456[66], 6}, {NULL}};
const Segment SGI210897[] = {{&S76622[44], 10}, {NULL}};
const Segment SGI210898[] = {{&S76622[44], 22}, {NULL}};
const Segment SGI210899[] = {{&S76622[30], 6}, {NULL}};
const Segment SGI210900[] = {{&S76622[133], 3}, {NULL}};
const Segment SGI210901[] = {{&S76622[98], 6}, {NULL}};
const Segment SGI210902[] = {{&S77458[34], 9}, {NULL}};
const Segment SGI210903[] = {{&S76622[68], 8}, {NULL}};
const Segment SGI210904[] = {{&S77458[27], 6}, {NULL}};
const Segment SGI210905[] = {{&S77458[96], 9}, {NULL}};
const Segment SGI210906[] = {{&S87345[34], 4}, {NULL}};
const Segment SGI210907[] = {{&S87345[27], 6}, {NULL}};
const Segment SGI210908[] = {{&S77458[69], 6}, {NULL}};
const Segment SGI210909[] = {{&S87345[88], 9}, {NULL}};
const Segment SGI210910[] = {{&S87345[61], 6}, {NULL}};
const Segment SGI210911[] = {{&S87346[34], 12}, {NULL}};
const Segment SGI210912[] = {{&S77459[34], 10}, {NULL}};
const Segment SGI210913[] = {{&S87346[27], 6}, {NULL}};
const Segment SGI210914[] = {{&S77459[27], 6}, {NULL}};
const Segment SGI210915[] = {{&S87346[96], 9}, {NULL}};
const Segment SGI210916[] = {{&S87346[69], 6}, {NULL}};
const Segment SGI210917[] = {{&S77459[97], 9}, {NULL}};
const Segment SGI210918[] = {{&S77459[70], 6}, {NULL}};
const Segment SGI210919[] = {{&S82050[44], 11}, {NULL}};
const Segment SGI210920[] = {{&S82050[44], 23}, {NULL}};
const Segment SGI210921[] = {{&S82050[30], 6}, {NULL}};
const Segment SGI210922[] = {{&S82050[116], 3}, {NULL}};
const Segment SGI210923[] = {{&S82050[81], 6}, {NULL}};
const Segment SGI210924[] = {{&S87352[44], 7}, {NULL}};
const Segment SGI210925[] = {{&S87352[44], 25}, {NULL}};
const Segment SGI210926[] = {{&S87352[30], 6}, {NULL}};
const Segment SGI210927[] = {{&S87352[83], 6}, {NULL}};
const Segment SGI210928[] = {{&S87352[110], 13}, {NULL}};
const Segment SGI210929[] = {{&S87354[44], 14}, {NULL}};
const Segment SGI210930[] = {{&S87354[44], 26}, {NULL}};
const Segment SGI210931[] = {{&S87354[30], 6}, {NULL}};
const Segment SGI210932[] = {{&S87354[31], 3}, {NULL}};
const Segment SGI210933[] = {{&S87354[84], 6}, {NULL}};
const Segment SGI210934[] = {{&S87355[34], 8}, {NULL}};
const Segment SGI210935[] = {{&S87355[27], 6}, {NULL}};
const Segment SGI210936[] = {{&S87355[95], 9}, {NULL}};
const Segment SGI210937[] = {{&S87355[68], 6}, {NULL}};
const Segment SGI210938[] = {{&S87359[44], 10}, {NULL}};
const Segment SGI210939[] = {{&S87359[44], 22}, {NULL}};
const Segment SGI210940[] = {{&S87359[30], 6}, {NULL}};
const Segment SGI210941[] = {{&S87359[31], 3}, {NULL}};
const Segment SGI210942[] = {{&S68043[38], 10}, {NULL}};
const Segment SGI210943[] = {{&S87359[80], 6}, {NULL}};
const Segment SGI210944[] = {{&S68043[31], 6}, {NULL}};
const Segment SGI210945[] = {{&S87359[140], 7}, {NULL}};
const Segment SGI210946[] = {{&S68044[44], 10}, {NULL}};
const Segment SGI210947[] = {{&S68044[44], 21}, {NULL}};
const Segment SGI210948[] = {{&S68044[30], 6}, {NULL}};
const Segment SGI210949[] = {{&S68044[114], 3}, {NULL}};
const Segment SGI210950[] = {{&S68044[79], 6}, {NULL}};
const Segment SGI210951[] = {{&S82066[44], 12}, {NULL}};
const Segment SGI210952[] = {{&S82066[44], 24}, {NULL}};
const Segment SGI210953[] = {{&S82066[30], 6}, {NULL}};
const Segment SGI210954[] = {{&S82066[117], 3}, {NULL}};
const Segment SGI210955[] = {{&S82066[82], 6}, {NULL}};
const Segment SGI210956[] = {{&S82068[44], 7}, {NULL}};
const Segment SGI210957[] = {{&S82068[44], 19}, {NULL}};
const Segment SGI210958[] = {{&S82068[30], 6}, {NULL}};
const Segment SGI210959[] = {{&S82068[112], 3}, {NULL}};
const Segment SGI210960[] = {{&S82068[77], 6}, {NULL}};
const Segment SGI210961[] = {{&S82072[34], 10}, {NULL}};
const Segment SGI210962[] = {{&S82072[27], 6}, {NULL}};
const Segment SGI210963[] = {{&S82072[97], 9}, {NULL}};
const Segment SGI210964[] = {{&S82072[70], 6}, {NULL}};
const Segment SGI210965[] = {{&S87377[34], 7}, {NULL}};
const Segment SGI210966[] = {{&S87377[27], 6}, {NULL}};
const Segment SGI210967[] = {{&S87377[99], 9}, {NULL}};
const Segment SGI210968[] = {{&S87377[72], 6}, {NULL}};
const Segment SGI210969[] = {{&S87377[140], 8}, {NULL}};
const Segment SGI210970[] = {{&S82079[34], 25}, {NULL}};
const Segment SGI210971[] = {{&S82079[27], 6}, {NULL}};
const Segment SGI210972[] = {{&S82079[112], 9}, {NULL}};
const Segment SGI210973[] = {{&S82079[85], 6}, {NULL}};
const Segment SGI210974[] = {{&S82080[44], 14}, {NULL}};
const Segment SGI210975[] = {{&S82080[44], 26}, {NULL}};
const Segment SGI210976[] = {{&S82080[30], 6}, {NULL}};
const Segment SGI210977[] = {{&S82080[119], 3}, {NULL}};
const Segment SGI210978[] = {{&S82080[84], 6}, {NULL}};
const Segment SGI210979[] = {{&S82084[34], 7}, {NULL}};
const Segment SGI210980[] = {{&S82084[27], 6}, {NULL}};
const Segment SGI210981[] = {{&S82084[94], 9}, {NULL}};
const Segment SGI210982[] = {{&S82084[67], 6}, {NULL}};
const Segment SGI210983[] = {{&S77496[34], 18}, {NULL}};
const Segment SGI210984[] = {{&S77496[27], 6}, {NULL}};
const Segment SGI210985[] = {{&S77496[105], 9}, {NULL}};
const Segment SGI210986[] = {{&S77496[78], 6}, {NULL}};
const Segment SGI210987[] = {{&S87390[34], 13}, {NULL}};
const Segment SGI210988[] = {{&S87390[27], 6}, {NULL}};
const Segment SGI210989[] = {{&S87390[100], 9}, {NULL}};
const Segment SGI210990[] = {{&S77497[34], 18}, {NULL}};
const Segment SGI210991[] = {{&S87390[73], 6}, {NULL}};
const Segment SGI210992[] = {{&S77497[27], 6}, {NULL}};
const Segment SGI210993[] = {{&S77497[105], 9}, {NULL}};
const Segment SGI210994[] = {{&S77497[78], 6}, {NULL}};
const Segment SGI210995[] = {{&S87391[34], 10}, {NULL}};
const Segment SGI210996[] = {{&S87391[27], 6}, {NULL}};
const Segment SGI210997[] = {{&S87391[97], 9}, {NULL}};
const Segment SGI210998[] = {{&S87391[70], 6}, {NULL}};
const Segment SGI210999[] = {{&S87392[34], 9}, {NULL}};
const Segment SGI211000[] = {{&S87392[27], 6}, {NULL}};
const Segment SGI211001[] = {{&S82093[34], 11}, {NULL}};
const Segment SGI211002[] = {{&S87392[96], 9}, {NULL}};
const Segment SGI211003[] = {{&S82093[27], 6}, {NULL}};
const Segment SGI211004[] = {{&S87392[69], 6}, {NULL}};
const Segment SGI211005[] = {{&S82093[98], 9}, {NULL}};
const Segment SGI211006[] = {{&S82093[71], 6}, {NULL}};
const Segment SGI211007[] = {{&S87393[34], 6}, {NULL}};
const Segment SGI211008[] = {{&S87393[27], 6}, {NULL}};
const Segment SGI211009[] = {{&S87393[93], 9}, {NULL}};
const Segment SGI211010[] = {{&S87393[66], 6}, {NULL}};
const Segment SGI211011[] = {{&S87397[34], 9}, {NULL}};
const Segment SGI211012[] = {{&S87397[27], 6}, {NULL}};
const Segment SGI211013[] = {{&S87397[93], 9}, {NULL}};
const Segment SGI211014[] = {{&S87397[66], 6}, {NULL}};
const Segment SGI211015[] = {{&S68099[44], 12}, {NULL}};
const Segment SGI211016[] = {{&S68099[44], 23}, {NULL}};
const Segment SGI211017[] = {{&S68099[30], 6}, {NULL}};
const Segment SGI211018[] = {{&S68099[116], 3}, {NULL}};
const Segment SGI211019[] = {{&S68099[81], 6}, {NULL}};
const Segment SGI211020[] = {{&S68100[44], 15}, {NULL}};
const Segment SGI211021[] = {{&S68100[44], 26}, {NULL}};
const Segment SGI211022[] = {{&S68100[30], 6}, {NULL}};
const Segment SGI211023[] = {{&S68100[119], 3}, {NULL}};
const Segment SGI211024[] = {{&S68100[84], 6}, {NULL}};
const Segment SGI211025[] = {{&S68106[34], 17}, {NULL}};
const Segment SGI211026[] = {{&S68106[27], 6}, {NULL}};
const Segment SGI211027[] = {{&S68106[103], 9}, {NULL}};
const Segment SGI211028[] = {{&S68106[76], 6}, {NULL}};
const Segment SGI211029[] = {{&S68107[34], 17}, {NULL}};
const Segment SGI211030[] = {{&S68107[27], 6}, {NULL}};
const Segment SGI211031[] = {{&S68107[103], 9}, {NULL}};
const Segment SGI211032[] = {{&S68107[76], 6}, {NULL}};
const Segment SGI211033[] = {{&S68108[34], 10}, {NULL}};
const Segment SGI211034[] = {{&S68108[27], 6}, {NULL}};
const Segment SGI211035[] = {{&S68108[96], 9}, {NULL}};
const Segment SGI211036[] = {{&S68108[69], 6}, {NULL}};
const Segment SGI211037[] = {{&S82132[44], 15}, {NULL}};
const Segment SGI211038[] = {{&S82132[44], 30}, {NULL}};
const Segment SGI211039[] = {{&S82132[30], 6}, {NULL}};
const Segment SGI211040[] = {{&S82132[123], 3}, {NULL}};
const Segment SGI211041[] = {{&S82132[88], 6}, {NULL}};
const Segment SGI211042[] = {{&S82132[148], 11}, {NULL}};
const Segment SGI211043[] = {{&S68119[34], 18}, {NULL}};
const Segment SGI211044[] = {{&S68119[27], 6}, {NULL}};
const Segment SGI211045[] = {{&S68119[104], 9}, {NULL}};
const Segment SGI211046[] = {{&S68119[77], 6}, {NULL}};
const Segment SGI211047[] = {{&S68120[34], 10}, {NULL}};
const Segment SGI211048[] = {{&S68120[27], 6}, {NULL}};
const Segment SGI211049[] = {{&S68120[96], 9}, {NULL}};
const Segment SGI211050[] = {{&S68120[69], 6}, {NULL}};
const Segment SGI211051[] = {{&S68121[44], 10}, {NULL}};
const Segment SGI211052[] = {{&S68121[44], 21}, {NULL}};
const Segment SGI211053[] = {{&S68121[30], 6}, {NULL}};
const Segment SGI211054[] = {{&S68121[114], 3}, {NULL}};
const Segment SGI211055[] = {{&S68121[79], 6}, {NULL}};
const Segment SGI211056[] = {{&S87440[34], 8}, {NULL}};
const Segment SGI211057[] = {{&S87440[27], 6}, {NULL}};
const Segment SGI211058[] = {{&S87440[95], 9}, {NULL}};
const Segment SGI211059[] = {{&S87440[68], 6}, {NULL}};
const Segment SGI211060[] = {{&S68122[44], 10}, {NULL}};
const Segment SGI211061[] = {{&S68122[44], 21}, {NULL}};
const Segment SGI211062[] = {{&S68122[30], 6}, {NULL}};
const Segment SGI211063[] = {{&S68122[82], 6}, {NULL}};
const Segment SGI211064[] = {{&S68123[38], 10}, {NULL}};
const Segment SGI211065[] = {{&S68123[31], 6}, {NULL}};
const Segment SGI211066[] = {{&S82148[44], 12}, {NULL}};
const Segment SGI211067[] = {{&S82148[44], 24}, {NULL}};
const Segment SGI211068[] = {{&S82148[30], 6}, {NULL}};
const Segment SGI211069[] = {{&S82148[117], 3}, {NULL}};
const Segment SGI211070[] = {{&S82148[82], 6}, {NULL}};
const Segment SGI211071[] = {{&S77543[44], 10}, {NULL}};
const Segment SGI211072[] = {{&S77543[44], 22}, {NULL}};
const Segment SGI211073[] = {{&S77543[30], 6}, {NULL}};
const Segment SGI211074[] = {{&S87444[34], 15}, {NULL}};
const Segment SGI211075[] = {{&S87444[27], 6}, {NULL}};
const Segment SGI211076[] = {{&S77543[113], 3}, {NULL}};
const Segment SGI211077[] = {{&S77543[80], 5}, {NULL}};
const Segment SGI211078[] = {{&S87444[102], 9}, {NULL}};
const Segment SGI211079[] = {{&S87444[75], 6}, {NULL}};
const Segment SGI211080[] = {{&S77543[117], 14}, {NULL}};
const Segment SGI211081[] = {{&S82156[34], 25}, {NULL}};
const Segment SGI211082[] = {{&S82156[27], 6}, {NULL}};
const Segment SGI211083[] = {{&S82156[112], 9}, {NULL}};
const Segment SGI211084[] = {{&S82156[85], 6}, {NULL}};
const Segment SGI211085[] = {{&S82157[34], 32}, {NULL}};
const Segment SGI211086[] = {{&S82157[27], 6}, {NULL}};
const Segment SGI211087[] = {{&S87450[34], 5}, {NULL}};
const Segment SGI211088[] = {{&S82157[119], 9}, {NULL}};
const Segment SGI211089[] = {{&S87450[27], 6}, {NULL}};
const Segment SGI211090[] = {{&S82157[92], 6}, {NULL}};
const Segment SGI211091[] = {{&S87450[92], 9}, {NULL}};
const Segment SGI211092[] = {{&S87450[65], 6}, {NULL}};
const Segment SGI211093[] = {{&S87451[44], 10}, {NULL}};
const Segment SGI211094[] = {{&S87451[44], 22}, {NULL}};
const Segment SGI211095[] = {{&S87451[30], 6}, {NULL}};
const Segment SGI211096[] = {{&S87451[31], 3}, {NULL}};
const Segment SGI211097[] = {{&S87451[80], 6}, {NULL}};
const Segment SGI211098[] = {{&S87453[45], 7}, {NULL}};
const Segment SGI211099[] = {{&S87453[30], 6}, {NULL}};
const Segment SGI211100[] = {{&S87453[31], 3}, {NULL}};
const Segment SGI211101[] = {{&S87453[84], 5}, {NULL}};
const Segment SGI211102[] = {{&S87453[142], 13}, {NULL}};
const Segment SGI211103[] = {{&S77556[34], 7}, {NULL}};
const Segment SGI211104[] = {{&S77556[27], 6}, {NULL}};
const Segment SGI211105[] = {{&S77556[99], 9}, {NULL}};
const Segment SGI211106[] = {{&S77556[72], 6}, {NULL}};
const Segment SGI211107[] = {{&S77556[140], 8}, {NULL}};
const Segment SGI211108[] = {{&S87463[34], 17}, {NULL}};
const Segment SGI211109[] = {{&S87463[27], 6}, {NULL}};
const Segment SGI211110[] = {{&S87463[112], 9}, {NULL}};
const Segment SGI211111[] = {{&S87463[85], 6}, {NULL}};
const Segment SGI211112[] = {{&S87464[34], 4}, {NULL}};
const Segment SGI211113[] = {{&S87464[27], 6}, {NULL}};
const Segment SGI211114[] = {{&S87464[91], 9}, {NULL}};
const Segment SGI211115[] = {{&S87464[64], 6}, {NULL}};
const Segment SGI211116[] = {{&S82177[34], 11}, {NULL}};
const Segment SGI211117[] = {{&S82177[27], 6}, {NULL}};
const Segment SGI211118[] = {{&S82177[98], 9}, {NULL}};
const Segment SGI211119[] = {{&S82177[71], 6}, {NULL}};
const Segment SGI211120[] = {{&S82177[139], 8}, {NULL}};
const Segment SGI211121[] = {{&S87467[45], 10}, {NULL}};
const Segment SGI211122[] = {{&S87467[30], 6}, {NULL}};
const Segment SGI211123[] = {{&S87467[31], 3}, {NULL}};
const Segment SGI211124[] = {{&S87467[81], 5}, {NULL}};
const Segment SGI211125[] = {{&S87467[139], 13}, {NULL}};
const Segment SGI211126[] = {{&S87469[44], 14}, {NULL}};
const Segment SGI211127[] = {{&S87469[44], 26}, {NULL}};
const Segment SGI211128[] = {{&S87469[30], 6}, {NULL}};
const Segment SGI211129[] = {{&S87469[31], 3}, {NULL}};
const Segment SGI211130[] = {{&S87469[84], 6}, {NULL}};
const Segment SGI211131[] = {{&S87471[34], 7}, {NULL}};
const Segment SGI211132[] = {{&S87471[27], 6}, {NULL}};
const Segment SGI211133[] = {{&S87471[104], 9}, {NULL}};
const Segment SGI211134[] = {{&S87471[77], 6}, {NULL}};
const Segment SGI211135[] = {{&S87472[34], 9}, {NULL}};
const Segment SGI211136[] = {{&S87472[27], 6}, {NULL}};
const Segment SGI211137[] = {{&S87472[93], 9}, {NULL}};
const Segment SGI211138[] = {{&S87472[66], 6}, {NULL}};
const Segment SGI211139[] = {{&S87474[34], 7}, {NULL}};
const Segment SGI211140[] = {{&S87474[27], 6}, {NULL}};
const Segment SGI211141[] = {{&S87474[94], 9}, {NULL}};
const Segment SGI211142[] = {{&S87474[67], 6}, {NULL}};
const Segment SGI211143[] = {{&S87482[44], 10}, {NULL}};
const Segment SGI211144[] = {{&S87482[44], 22}, {NULL}};
const Segment SGI211145[] = {{&S87482[30], 6}, {NULL}};
const Segment SGI211146[] = {{&S87482[80], 6}, {NULL}};
const Segment SGI211147[] = {{&S87482[107], 13}, {NULL}};
const Segment SGI211148[] = {{&S87483[44], 11}, {NULL}};
const Segment SGI211149[] = {{&S87483[44], 23}, {NULL}};
const Segment SGI211150[] = {{&S87483[30], 6}, {NULL}};
const Segment SGI211151[] = {{&S87483[81], 6}, {NULL}};
const Segment SGI211152[] = {{&S87483[108], 13}, {NULL}};
const Segment SGI211153[] = {{&S87484[44], 10}, {NULL}};
const Segment SGI211154[] = {{&S87484[44], 27}, {NULL}};
const Segment SGI211155[] = {{&S87484[30], 6}, {NULL}};
const Segment SGI211156[] = {{&S87484[85], 6}, {NULL}};
const Segment SGI211157[] = {{&S87484[112], 13}, {NULL}};
const Segment SGI211158[] = {{&S87487[44], 15}, {NULL}};
const Segment SGI211159[] = {{&S87487[44], 26}, {NULL}};
const Segment SGI211160[] = {{&S87487[30], 6}, {NULL}};
const Segment SGI211161[] = {{&S87487[31], 3}, {NULL}};
const Segment SGI211162[] = {{&S87487[84], 9}, {NULL}};
const Segment SGI211163[] = {{&S87488[34], 6}, {NULL}};
const Segment SGI211164[] = {{&S87488[27], 6}, {NULL}};
const Segment SGI211165[] = {{&S87488[93], 9}, {NULL}};
const Segment SGI211166[] = {{&S87488[66], 6}, {NULL}};
const Segment SGI211167[] = {{&S77602[44], 12}, {NULL}};
const Segment SGI211168[] = {{&S77602[44], 24}, {NULL}};
const Segment SGI211169[] = {{&S77602[30], 6}, {NULL}};
const Segment SGI211170[] = {{&S77602[133], 3}, {NULL}};
const Segment SGI211171[] = {{&S77602[98], 6}, {NULL}};
const Segment SGI211172[] = {{&S77602[70], 8}, {NULL}};
const Segment SGI211173[] = {{&S77603[44], 21}, {NULL}};
const Segment SGI211174[] = {{&S77603[44], 33}, {NULL}};
const Segment SGI211175[] = {{&S77603[30], 6}, {NULL}};
const Segment SGI211176[] = {{&S77603[142], 3}, {NULL}};
const Segment SGI211177[] = {{&S77603[107], 6}, {NULL}};
const Segment SGI211178[] = {{&S77603[79], 8}, {NULL}};
const Segment SGI211179[] = {{&S77604[44], 12}, {NULL}};
const Segment SGI211180[] = {{&S77604[44], 24}, {NULL}};
const Segment SGI211181[] = {{&S77604[30], 6}, {NULL}};
const Segment SGI211182[] = {{&S77604[133], 3}, {NULL}};
const Segment SGI211183[] = {{&S77604[98], 6}, {NULL}};
const Segment SGI211184[] = {{&S77604[70], 8}, {NULL}};
const Segment SGI211185[] = {{&S77615[34], 10}, {NULL}};
const Segment SGI211186[] = {{&S77615[27], 6}, {NULL}};
const Segment SGI211187[] = {{&S77615[97], 9}, {NULL}};
const Segment SGI211188[] = {{&S77615[70], 6}, {NULL}};
const Segment SGI211189[] = {{&S87501[34], 11}, {NULL}};
const Segment SGI211190[] = {{&S77616[34], 11}, {NULL}};
const Segment SGI211191[] = {{&S87501[27], 6}, {NULL}};
const Segment SGI211192[] = {{&S77616[27], 6}, {NULL}};
const Segment SGI211193[] = {{&S87501[98], 9}, {NULL}};
const Segment SGI211194[] = {{&S87501[71], 6}, {NULL}};
const Segment SGI211195[] = {{&S77616[98], 9}, {NULL}};
const Segment SGI211196[] = {{&S77616[71], 6}, {NULL}};
const Segment SGI211197[] = {{&S87501[139], 8}, {NULL}};
const Segment SGI211198[] = {{&S87503[34], 8}, {NULL}};
const Segment SGI211199[] = {{&S87503[27], 6}, {NULL}};
const Segment SGI211200[] = {{&S87503[95], 9}, {NULL}};
const Segment SGI211201[] = {{&S87503[68], 6}, {NULL}};
const Segment SGI211202[] = {{&S68235[34], 10}, {NULL}};
const Segment SGI211203[] = {{&S68235[27], 6}, {NULL}};
const Segment SGI211204[] = {{&S68235[97], 9}, {NULL}};
const Segment SGI211205[] = {{&S68235[70], 6}, {NULL}};
const Segment SGI211206[] = {{&S87515[34], 10}, {NULL}};
const Segment SGI211207[] = {{&S87515[27], 6}, {NULL}};
const Segment SGI211208[] = {{&S87515[97], 9}, {NULL}};
const Segment SGI211209[] = {{&S80478[42], 7}, {NULL}};
const Segment SGI211210[] = {{&S87515[70], 6}, {NULL}};
const Segment SGI211211[] = {{&S80478[42], 24}, {NULL}};
const Segment SGI211212[] = {{&S80478[30], 4}, {NULL}};
const Segment SGI211213[] = {{&S80478[115], 3}, {NULL}};
const Segment SGI211214[] = {{&S80478[80], 6}, {NULL}};
const Segment SGI211215[] = {{&S80482[32], 12}, {NULL}};
const Segment SGI211216[] = {{&S80482[27], 4}, {NULL}};
const Segment SGI211217[] = {{&S80482[97], 9}, {NULL}};
const Segment SGI211218[] = {{&S80482[70], 6}, {NULL}};
const Segment SGI211219[] = {{&S77644[34], 12}, {NULL}};
const Segment SGI211220[] = {{&S77644[27], 6}, {NULL}};
const Segment SGI211221[] = {{&S77644[99], 9}, {NULL}};
const Segment SGI211222[] = {{&S77644[72], 6}, {NULL}};
const Segment SGI211223[] = {{&S77663[34], 11}, {NULL}};
const Segment SGI211224[] = {{&S77663[27], 6}, {NULL}};
const Segment SGI211225[] = {{&S77663[98], 9}, {NULL}};
const Segment SGI211226[] = {{&S77663[71], 6}, {NULL}};
const Segment SGI211227[] = {{&S77664[34], 11}, {NULL}};
const Segment SGI211228[] = {{&S77664[27], 6}, {NULL}};
const Segment SGI211229[] = {{&S77664[98], 9}, {NULL}};
const Segment SGI211230[] = {{&S77664[71], 6}, {NULL}};
const Segment SGI211231[] = {{&S77665[34], 11}, {NULL}};
const Segment SGI211232[] = {{&S77665[27], 6}, {NULL}};
const Segment SGI211233[] = {{&S77665[98], 9}, {NULL}};
const Segment SGI211234[] = {{&S77665[71], 6}, {NULL}};
const Segment SGI211235[] = {{&S77668[34], 9}, {NULL}};
const Segment SGI211236[] = {{&S77668[27], 6}, {NULL}};
const Segment SGI211237[] = {{&S77668[96], 9}, {NULL}};
const Segment SGI211238[] = {{&S77668[69], 6}, {NULL}};
const Segment SGI211239[] = {{&S77669[34], 9}, {NULL}};
const Segment SGI211240[] = {{&S77669[27], 6}, {NULL}};
const Segment SGI211241[] = {{&S77669[96], 9}, {NULL}};
const Segment SGI211242[] = {{&S77669[69], 6}, {NULL}};
const Segment SGI211243[] = {{&S77670[34], 10}, {NULL}};
const Segment SGI211244[] = {{&S77670[27], 6}, {NULL}};
const Segment SGI211245[] = {{&S77670[97], 9}, {NULL}};
const Segment SGI211246[] = {{&S77670[70], 6}, {NULL}};
const Segment SGI211247[] = {{&S77674[44], 10}, {NULL}};
const Segment SGI211248[] = {{&S77674[44], 22}, {NULL}};
const Segment SGI211249[] = {{&S77674[30], 6}, {NULL}};
const Segment SGI211250[] = {{&S77674[80], 6}, {NULL}};
const Segment SGI211251[] = {{&S77674[107], 13}, {NULL}};
const Segment SGI211252[] = {{&S77676[34], 10}, {NULL}};
const Segment SGI211253[] = {{&S77676[27], 6}, {NULL}};
const Segment SGI211254[] = {{&S77676[97], 9}, {NULL}};
const Segment SGI211255[] = {{&S77676[70], 6}, {NULL}};
const Segment SGI211256[] = {{&S87582[34], 12}, {NULL}};
const Segment SGI211257[] = {{&S87582[27], 6}, {NULL}};
const Segment SGI211258[] = {{&S87582[99], 9}, {NULL}};
const Segment SGI211259[] = {{&S87582[72], 6}, {NULL}};
const Segment SGI211260[] = {{&S87584[34], 7}, {NULL}};
const Segment SGI211261[] = {{&S87584[27], 6}, {NULL}};
const Segment SGI211262[] = {{&S87584[94], 9}, {NULL}};
const Segment SGI211263[] = {{&S87584[67], 6}, {NULL}};
const Segment SGI211264[] = {{&S87590[44], 16}, {NULL}};
const Segment SGI211265[] = {{&S87590[44], 28}, {NULL}};
const Segment SGI211266[] = {{&S87590[30], 6}, {NULL}};
const Segment SGI211267[] = {{&S87590[31], 3}, {NULL}};
const Segment SGI211268[] = {{&S87590[86], 6}, {NULL}};
const Segment SGI211269[] = {{&S77696[59], 10}, {NULL}};
const Segment SGI211270[] = {{&S77696[59], 22}, {NULL}};
const Segment SGI211271[] = {{&S77696[45], 6}, {NULL}};
const Segment SGI211272[] = {{&S77696[130], 3}, {NULL}};
const Segment SGI211273[] = {{&S77696[95], 6}, {NULL}};
const Segment SGI211274[] = {{&S77696[0], 14}, {NULL}};
const Segment SGI211275[] = {{&S77697[59], 10}, {NULL}};
const Segment SGI211276[] = {{&S77697[59], 22}, {NULL}};
const Segment SGI211277[] = {{&S87594[34], 18}, {NULL}};
const Segment SGI211278[] = {{&S77697[45], 6}, {NULL}};
const Segment SGI211279[] = {{&S87594[27], 6}, {NULL}};
const Segment SGI211280[] = {{&S77697[130], 3}, {NULL}};
const Segment SGI211281[] = {{&S77697[95], 6}, {NULL}};
const Segment SGI211282[] = {{&S87594[105], 9}, {NULL}};
const Segment SGI211283[] = {{&S87594[78], 6}, {NULL}};
const Segment SGI211284[] = {{&S77697[0], 14}, {NULL}};
const Segment SGI211285[] = {{&S87595[34], 19}, {NULL}};
const Segment SGI211286[] = {{&S87595[27], 6}, {NULL}};
const Segment SGI211287[] = {{&S87595[106], 9}, {NULL}};
const Segment SGI211288[] = {{&S87595[79], 6}, {NULL}};
const Segment SGI211289[] = {{&S77701[44], 7}, {NULL}};
const Segment SGI211290[] = {{&S77701[44], 25}, {NULL}};
const Segment SGI211291[] = {{&S77701[30], 6}, {NULL}};
const Segment SGI211292[] = {{&S77701[118], 3}, {NULL}};
const Segment SGI211293[] = {{&S77701[83], 6}, {NULL}};
const Segment SGI211294[] = {{&S77702[44], 7}, {NULL}};
const Segment SGI211295[] = {{&S77702[44], 25}, {NULL}};
const Segment SGI211296[] = {{&S77702[30], 6}, {NULL}};
const Segment SGI211297[] = {{&S87599[45], 10}, {NULL}};
const Segment SGI211298[] = {{&S77702[118], 3}, {NULL}};
const Segment SGI211299[] = {{&S87599[30], 6}, {NULL}};
const Segment SGI211300[] = {{&S77702[83], 6}, {NULL}};
const Segment SGI211301[] = {{&S87599[31], 3}, {NULL}};
const Segment SGI211302[] = {{&S87599[81], 5}, {NULL}};
const Segment SGI211303[] = {{&S82209[44], 10}, {NULL}};
const Segment SGI211304[] = {{&S87599[139], 13}, {NULL}};
const Segment SGI211305[] = {{&S82209[44], 22}, {NULL}};
const Segment SGI211306[] = {{&S82209[30], 6}, {NULL}};
const Segment SGI211307[] = {{&S82209[115], 3}, {NULL}};
const Segment SGI211308[] = {{&S82209[80], 6}, {NULL}};
const Segment SGI211309[] = {{&S77704[34], 25}, {NULL}};
const Segment SGI211310[] = {{&S77704[27], 6}, {NULL}};
const Segment SGI211311[] = {{&S82210[44], 10}, {NULL}};
const Segment SGI211312[] = {{&S82210[44], 73}, {NULL}};
const Segment SGI211313[] = {{&S82210[30], 6}, {NULL}};
const Segment SGI211314[] = {{&S77704[112], 9}, {NULL}};
const Segment SGI211315[] = {{&S87601[34], 8}, {NULL}};
const Segment SGI211316[] = {{&S87601[27], 6}, {NULL}};
const Segment SGI211317[] = {{&S76953[34], 11}, {NULL}};
const Segment SGI211318[] = {{&S68334[34], 8}, {NULL}};
const Segment SGI211319[] = {{&S76953[27], 6}, {NULL}};
const Segment SGI211320[] = {{&S68334[27], 6}, {NULL}};
const Segment SGI211321[] = {{&S87601[100], 9}, {NULL}};
const Segment SGI211322[] = {{&S87601[73], 6}, {NULL}};
const Segment SGI211323[] = {{&S76953[98], 9}, {NULL}};
const Segment SGI211324[] = {{&S68334[94], 9}, {NULL}};
const Segment SGI211325[] = {{&S76953[71], 6}, {NULL}};
const Segment SGI211326[] = {{&S68334[67], 6}, {NULL}};
const Segment SGI211327[] = {{&S82211[38], 10}, {NULL}};
const Segment SGI211328[] = {{&S82211[31], 6}, {NULL}};
const Segment SGI211329[] = {{&S82215[44], 10}, {NULL}};
const Segment SGI211330[] = {{&S82215[44], 73}, {NULL}};
const Segment SGI211331[] = {{&S82215[30], 6}, {NULL}};
const Segment SGI211332[] = {{&S87606[44], 7}, {NULL}};
const Segment SGI211333[] = {{&S87606[44], 18}, {NULL}};
const Segment SGI211334[] = {{&S87606[30], 6}, {NULL}};
const Segment SGI211335[] = {{&S87606[76], 6}, {NULL}};
const Segment SGI211336[] = {{&S87606[103], 13}, {NULL}};
const Segment SGI211337[] = {{&S87608[44], 7}, {NULL}};
const Segment SGI211338[] = {{&S87608[44], 24}, {NULL}};
const Segment SGI211339[] = {{&S87608[30], 6}, {NULL}};
const Segment SGI211340[] = {{&S87608[82], 6}, {NULL}};
const Segment SGI211341[] = {{&S87608[109], 13}, {NULL}};
const Segment SGI211342[] = {{&S82219[34], 7}, {NULL}};
const Segment SGI211343[] = {{&S82219[27], 6}, {NULL}};
const Segment SGI211344[] = {{&S82219[94], 9}, {NULL}};
const Segment SGI211345[] = {{&S82219[67], 6}, {NULL}};
const Segment SGI211346[] = {{&S68346[44], 10}, {NULL}};
const Segment SGI211347[] = {{&S68346[44], 21}, {NULL}};
const Segment SGI211348[] = {{&S68346[30], 6}, {NULL}};
const Segment SGI211349[] = {{&S68346[132], 3}, {NULL}};
const Segment SGI211350[] = {{&S68346[97], 6}, {NULL}};
const Segment SGI211351[] = {{&S68346[67], 8}, {NULL}};
const Segment SGI211352[] = {{&S68347[44], 10}, {NULL}};
const Segment SGI211353[] = {{&S68347[44], 22}, {NULL}};
const Segment SGI211354[] = {{&S68347[30], 6}, {NULL}};
const Segment SGI211355[] = {{&S68347[133], 3}, {NULL}};
const Segment SGI211356[] = {{&S68347[98], 6}, {NULL}};
const Segment SGI211357[] = {{&S68347[68], 8}, {NULL}};
const Segment SGI211358[] = {{&S87616[34], 10}, {NULL}};
const Segment SGI211359[] = {{&S87616[27], 6}, {NULL}};
const Segment SGI211360[] = {{&S82226[44], 10}, {NULL}};
const Segment SGI211361[] = {{&S82226[44], 22}, {NULL}};
const Segment SGI211362[] = {{&S82226[30], 6}, {NULL}};
const Segment SGI211363[] = {{&S87616[97], 9}, {NULL}};
const Segment SGI211364[] = {{&S82226[115], 3}, {NULL}};
const Segment SGI211365[] = {{&S87616[70], 6}, {NULL}};
const Segment SGI211366[] = {{&S82226[80], 6}, {NULL}};
const Segment SGI211367[] = {{&S87617[34], 10}, {NULL}};
const Segment SGI211368[] = {{&S87617[27], 6}, {NULL}};
const Segment SGI211369[] = {{&S87617[97], 9}, {NULL}};
const Segment SGI211370[] = {{&S87617[70], 6}, {NULL}};
const Segment SGI211371[] = {{&S87618[34], 7}, {NULL}};
const Segment SGI211372[] = {{&S87618[27], 6}, {NULL}};
const Segment SGI211373[] = {{&S87618[94], 9}, {NULL}};
const Segment SGI211374[] = {{&S87618[67], 6}, {NULL}};
const Segment SGI211375[] = {{&S82230[34], 12}, {NULL}};
const Segment SGI211376[] = {{&S82230[27], 6}, {NULL}};
const Segment SGI211377[] = {{&S82230[99], 9}, {NULL}};
const Segment SGI211378[] = {{&S82230[72], 6}, {NULL}};
const Segment SGI211379[] = {{&S87621[44], 14}, {NULL}};
const Segment SGI211380[] = {{&S87621[44], 26}, {NULL}};
const Segment SGI211381[] = {{&S87621[30], 6}, {NULL}};
const Segment SGI211382[] = {{&S87621[31], 3}, {NULL}};
const Segment SGI211383[] = {{&S87621[84], 6}, {NULL}};
const Segment SGI211384[] = {{&S87626[44], 13}, {NULL}};
const Segment SGI211385[] = {{&S87626[44], 25}, {NULL}};
const Segment SGI211386[] = {{&S87626[30], 6}, {NULL}};
const Segment SGI211387[] = {{&S87626[83], 6}, {NULL}};
const Segment SGI211388[] = {{&S87626[110], 13}, {NULL}};
const Segment SGI211389[] = {{&S82238[34], 18}, {NULL}};
const Segment SGI211390[] = {{&S82238[27], 6}, {NULL}};
const Segment SGI211391[] = {{&S82238[128], 9}, {NULL}};
const Segment SGI211392[] = {{&S82238[101], 6}, {NULL}};
const Segment SGI211393[] = {{&S82239[34], 18}, {NULL}};
const Segment SGI211394[] = {{&S82239[27], 6}, {NULL}};
const Segment SGI211395[] = {{&S82239[105], 9}, {NULL}};
const Segment SGI211396[] = {{&S82239[78], 6}, {NULL}};
const Segment SGI211397[] = {{&S68368[44], 7}, {NULL}};
const Segment SGI211398[] = {{&S68368[44], 19}, {NULL}};
const Segment SGI211399[] = {{&S68368[30], 6}, {NULL}};
const Segment SGI211400[] = {{&S68368[130], 3}, {NULL}};
const Segment SGI211401[] = {{&S68368[95], 6}, {NULL}};
const Segment SGI211402[] = {{&S68368[65], 8}, {NULL}};
const Segment SGI211403[] = {{&S68369[44], 7}, {NULL}};
const Segment SGI211404[] = {{&S68369[44], 18}, {NULL}};
const Segment SGI211405[] = {{&S68369[30], 6}, {NULL}};
const Segment SGI211406[] = {{&S68369[139], 3}, {NULL}};
const Segment SGI211407[] = {{&S68369[104], 6}, {NULL}};
const Segment SGI211408[] = {{&S68369[64], 8}, {NULL}};
const Segment SGI211409[] = {{&S87634[34], 12}, {NULL}};
const Segment SGI211410[] = {{&S87634[27], 6}, {NULL}};
const Segment SGI211411[] = {{&S87634[99], 9}, {NULL}};
const Segment SGI211412[] = {{&S87634[72], 6}, {NULL}};
const Segment SGI211413[] = {{&S87634[140], 8}, {NULL}};
const Segment SGI211414[] = {{&S68372[34], 10}, {NULL}};
const Segment SGI211415[] = {{&S76996[34], 4}, {NULL}};
const Segment SGI211416[] = {{&S68372[27], 6}, {NULL}};
const Segment SGI211417[] = {{&S76996[27], 6}, {NULL}};
const Segment SGI211418[] = {{&S68372[97], 9}, {NULL}};
const Segment SGI211419[] = {{&S76996[91], 9}, {NULL}};
const Segment SGI211420[] = {{&S68372[70], 6}, {NULL}};
const Segment SGI211421[] = {{&S76996[64], 6}, {NULL}};
const Segment SGI211422[] = {{&S87637[34], 10}, {NULL}};
const Segment SGI211423[] = {{&S87637[27], 6}, {NULL}};
const Segment SGI211424[] = {{&S87637[97], 9}, {NULL}};
const Segment SGI211425[] = {{&S87637[70], 6}, {NULL}};
const Segment SGI211426[] = {{&S87637[138], 8}, {NULL}};
const Segment SGI211427[] = {{&S82254[34], 12}, {NULL}};
const Segment SGI211428[] = {{&S82254[27], 6}, {NULL}};
const Segment SGI211429[] = {{&S82254[99], 9}, {NULL}};
const Segment SGI211430[] = {{&S82254[72], 6}, {NULL}};
const Segment SGI211431[] = {{&S87649[34], 9}, {NULL}};
const Segment SGI211432[] = {{&S87649[27], 6}, {NULL}};
const Segment SGI211433[] = {{&S87649[96], 9}, {NULL}};
const Segment SGI211434[] = {{&S87649[69], 6}, {NULL}};
const Segment SGI211435[] = {{&S87651[44], 13}, {NULL}};
const Segment SGI211436[] = {{&S87651[44], 25}, {NULL}};
const Segment SGI211437[] = {{&S87651[30], 6}, {NULL}};
const Segment SGI211438[] = {{&S87651[83], 6}, {NULL}};
const Segment SGI211439[] = {{&S87651[110], 13}, {NULL}};
const Segment SGI211440[] = {{&S87652[34], 10}, {NULL}};
const Segment SGI211441[] = {{&S87652[27], 6}, {NULL}};
const Segment SGI211442[] = {{&S87652[97], 9}, {NULL}};
const Segment SGI211443[] = {{&S87652[70], 6}, {NULL}};
const Segment SGI211444[] = {{&S87652[138], 8}, {NULL}};
const Segment SGI211445[] = {{&S87653[34], 14}, {NULL}};
const Segment SGI211446[] = {{&S87653[27], 6}, {NULL}};
const Segment SGI211447[] = {{&S87653[101], 9}, {NULL}};
const Segment SGI211448[] = {{&S87653[74], 6}, {NULL}};
const Segment SGI211449[] = {{&S87653[142], 8}, {NULL}};
const Segment SGI211450[] = {{&S82286[34], 14}, {NULL}};
const Segment SGI211451[] = {{&S82286[27], 6}, {NULL}};
const Segment SGI211452[] = {{&S82286[101], 9}, {NULL}};
const Segment SGI211453[] = {{&S82286[74], 6}, {NULL}};
const Segment SGI211454[] = {{&S68429[34], 18}, {NULL}};
const Segment SGI211455[] = {{&S68429[27], 6}, {NULL}};
const Segment SGI211456[] = {{&S68429[105], 9}, {NULL}};
const Segment SGI211457[] = {{&S68429[78], 6}, {NULL}};
const Segment SGI211458[] = {{&S68430[34], 10}, {NULL}};
const Segment SGI211459[] = {{&S68430[27], 6}, {NULL}};
const Segment SGI211460[] = {{&S68430[96], 9}, {NULL}};
const Segment SGI211461[] = {{&S68430[69], 6}, {NULL}};
const Segment SGI211462[] = {{&S87688[44], 10}, {NULL}};
const Segment SGI211463[] = {{&S87688[44], 22}, {NULL}};
const Segment SGI211464[] = {{&S87688[30], 6}, {NULL}};
const Segment SGI211465[] = {{&S87688[31], 3}, {NULL}};
const Segment SGI211466[] = {{&S87688[95], 6}, {NULL}};
const Segment SGI211467[] = {{&S87688[68], 8}, {NULL}};
const Segment SGI211468[] = {{&S82312[44], 5}, {NULL}};
const Segment SGI211469[] = {{&S82312[44], 17}, {NULL}};
const Segment SGI211470[] = {{&S82312[30], 6}, {NULL}};
const Segment SGI211471[] = {{&S82312[110], 3}, {NULL}};
const Segment SGI211472[] = {{&S82312[75], 6}, {NULL}};
const Segment SGI211473[] = {{&S87694[34], 8}, {NULL}};
const Segment SGI211474[] = {{&S87694[27], 6}, {NULL}};
const Segment SGI211475[] = {{&S87694[95], 9}, {NULL}};
const Segment SGI211476[] = {{&S87694[68], 6}, {NULL}};
const Segment SGI211477[] = {{&S87698[34], 7}, {NULL}};
const Segment SGI211478[] = {{&S87698[27], 6}, {NULL}};
const Segment SGI211479[] = {{&S87698[94], 9}, {NULL}};
const Segment SGI211480[] = {{&S87698[67], 6}, {NULL}};
const Segment SGI211481[] = {{&S87699[34], 9}, {NULL}};
const Segment SGI211482[] = {{&S87699[27], 6}, {NULL}};
const Segment SGI211483[] = {{&S87699[96], 9}, {NULL}};
const Segment SGI211484[] = {{&S87699[69], 6}, {NULL}};
const Segment SGI211485[] = {{&S87700[34], 7}, {NULL}};
const Segment SGI211486[] = {{&S87700[27], 6}, {NULL}};
const Segment SGI211487[] = {{&S87700[94], 9}, {NULL}};
const Segment SGI211488[] = {{&S87700[67], 6}, {NULL}};
const Segment SGI211489[] = {{&S87702[34], 16}, {NULL}};
const Segment SGI211490[] = {{&S87702[27], 6}, {NULL}};
const Segment SGI211491[] = {{&S87702[101], 9}, {NULL}};
const Segment SGI211492[] = {{&S87702[74], 6}, {NULL}};
const Segment SGI211493[] = {{&S82328[34], 6}, {NULL}};
const Segment SGI211494[] = {{&S82328[27], 6}, {NULL}};
const Segment SGI211495[] = {{&S82328[93], 9}, {NULL}};
const Segment SGI211496[] = {{&S82328[66], 6}, {NULL}};
const Segment SGI211497[] = {{&S87706[44], 9}, {NULL}};
const Segment SGI211498[] = {{&S87706[44], 32}, {NULL}};
const Segment SGI211499[] = {{&S87706[30], 6}, {NULL}};
const Segment SGI211500[] = {{&S87706[31], 3}, {NULL}};
const Segment SGI211501[] = {{&S87706[90], 6}, {NULL}};
const Segment SGI211502[] = {{&S68464[44], 8}, {NULL}};
const Segment SGI211503[] = {{&S68464[44], 19}, {NULL}};
const Segment SGI211504[] = {{&S68464[30], 6}, {NULL}};
const Segment SGI211505[] = {{&S68464[112], 3}, {NULL}};
const Segment SGI211506[] = {{&S68464[77], 6}, {NULL}};
const Segment SGI211507[] = {{&S87710[34], 11}, {NULL}};
const Segment SGI211508[] = {{&S87710[27], 6}, {NULL}};
const Segment SGI211509[] = {{&S68465[44], 8}, {NULL}};
const Segment SGI211510[] = {{&S68465[44], 19}, {NULL}};
const Segment SGI211511[] = {{&S68465[30], 6}, {NULL}};
const Segment SGI211512[] = {{&S87710[98], 9}, {NULL}};
const Segment SGI211513[] = {{&S87710[71], 6}, {NULL}};
const Segment SGI211514[] = {{&S68465[112], 3}, {NULL}};
const Segment SGI211515[] = {{&S68465[77], 6}, {NULL}};
const Segment SGI211516[] = {{&S68466[44], 8}, {NULL}};
const Segment SGI211517[] = {{&S68466[44], 19}, {NULL}};
const Segment SGI211518[] = {{&S68466[30], 6}, {NULL}};
const Segment SGI211519[] = {{&S68466[112], 3}, {NULL}};
const Segment SGI211520[] = {{&S68466[77], 6}, {NULL}};
const Segment SGI211521[] = {{&S68467[44], 8}, {NULL}};
const Segment SGI211522[] = {{&S68467[44], 19}, {NULL}};
const Segment SGI211523[] = {{&S68467[30], 6}, {NULL}};
const Segment SGI211524[] = {{&S68467[112], 3}, {NULL}};
const Segment SGI211525[] = {{&S68467[77], 6}, {NULL}};
const Segment SGI211526[] = {{&S87715[34], 10}, {NULL}};
const Segment SGI211527[] = {{&S68468[38], 8}, {NULL}};
const Segment SGI211528[] = {{&S87715[27], 6}, {NULL}};
const Segment SGI211529[] = {{&S68468[31], 6}, {NULL}};
const Segment SGI211530[] = {{&S87715[105], 9}, {NULL}};
const Segment SGI211531[] = {{&S87715[78], 6}, {NULL}};
const Segment SGI211532[] = {{&S68469[34], 9}, {NULL}};
const Segment SGI211533[] = {{&S68469[27], 6}, {NULL}};
const Segment SGI211534[] = {{&S68469[95], 9}, {NULL}};
const Segment SGI211535[] = {{&S68469[68], 6}, {NULL}};
const Segment SGI211536[] = {{&S68469[136], 8}, {NULL}};
const Segment SGI211537[] = {{&S68470[34], 11}, {NULL}};
const Segment SGI211538[] = {{&S68470[27], 6}, {NULL}};
const Segment SGI211539[] = {{&S87718[34], 13}, {NULL}};
const Segment SGI211540[] = {{&S68470[97], 9}, {NULL}};
const Segment SGI211541[] = {{&S87718[27], 6}, {NULL}};
const Segment SGI211542[] = {{&S68470[70], 6}, {NULL}};
const Segment SGI211543[] = {{&S68470[138], 8}, {NULL}};
const Segment SGI211544[] = {{&S82348[34], 11}, {NULL}};
const Segment SGI211545[] = {{&S87718[100], 9}, {NULL}};
const Segment SGI211546[] = {{&S87718[73], 6}, {NULL}};
const Segment SGI211547[] = {{&S82348[27], 6}, {NULL}};
const Segment SGI211548[] = {{&S82348[98], 9}, {NULL}};
const Segment SGI211549[] = {{&S82348[71], 6}, {NULL}};
const Segment SGI211550[] = {{&S77106[34], 11}, {NULL}};
const Segment SGI211551[] = {{&S77106[27], 6}, {NULL}};
const Segment SGI211552[] = {{&S77106[98], 9}, {NULL}};
const Segment SGI211553[] = {{&S77106[71], 6}, {NULL}};
const Segment SGI211554[] = {{&S87727[27], 6}, {NULL}};
const Segment SGI211555[] = {{&S87727[103], 9}, {NULL}};
const Segment SGI211556[] = {{&S87727[76], 6}, {NULL}};
const Segment SGI211557[] = {{&S87728[44], 17}, {NULL}};
const Segment SGI211558[] = {{&S87728[44], 29}, {NULL}};
const Segment SGI211559[] = {{&S87728[30], 6}, {NULL}};
const Segment SGI211560[] = {{&S87728[31], 3}, {NULL}};
const Segment SGI211561[] = {{&S87728[87], 6}, {NULL}};
const Segment SGI211562[] = {{&S87734[34], 13}, {NULL}};
const Segment SGI211563[] = {{&S87734[27], 6}, {NULL}};
const Segment SGI211564[] = {{&S87734[100], 9}, {NULL}};
const Segment SGI211565[] = {{&S87734[73], 6}, {NULL}};
const Segment SGI211566[] = {{&S87735[34], 8}, {NULL}};
const Segment SGI211567[] = {{&S87735[27], 6}, {NULL}};
const Segment SGI211568[] = {{&S87735[99], 9}, {NULL}};
const Segment SGI211569[] = {{&S87735[72], 6}, {NULL}};
const Segment SGI211570[] = {{&S87735[140], 8}, {NULL}};
const Segment SGI211571[] = {{&S87741[34], 8}, {NULL}};
const Segment SGI211572[] = {{&S87741[27], 6}, {NULL}};
const Segment SGI211573[] = {{&S87741[101], 9}, {NULL}};
const Segment SGI211574[] = {{&S87741[74], 6}, {NULL}};
const Segment SGI211575[] = {{&S87742[44], 10}, {NULL}};
const Segment SGI211576[] = {{&S87742[44], 22}, {NULL}};
const Segment SGI211577[] = {{&S87742[30], 6}, {NULL}};
const Segment SGI211578[] = {{&S87742[31], 3}, {NULL}};
const Segment SGI211579[] = {{&S87742[80], 6}, {NULL}};
const Segment SGI211580[] = {{&S87743[34], 9}, {NULL}};
const Segment SGI211581[] = {{&S87743[27], 6}, {NULL}};
const Segment SGI211582[] = {{&S87743[96], 9}, {NULL}};
const Segment SGI211583[] = {{&S87743[69], 6}, {NULL}};
const Segment SGI211584[] = {{&S87743[137], 8}, {NULL}};
const Segment SGI211585[] = {{&S68507[44], 10}, {NULL}};
const Segment SGI211586[] = {{&S87744[34], 9}, {NULL}};
const Segment SGI211587[] = {{&S68507[44], 21}, {NULL}};
const Segment SGI211588[] = {{&S87744[27], 6}, {NULL}};
const Segment SGI211589[] = {{&S68507[30], 6}, {NULL}};
const Segment SGI211590[] = {{&S68507[132], 3}, {NULL}};
const Segment SGI211591[] = {{&S82383[44], 7}, {NULL}};
const Segment SGI211592[] = {{&S87744[96], 9}, {NULL}};
const Segment SGI211593[] = {{&S68507[97], 6}, {NULL}};
const Segment SGI211594[] = {{&S82383[44], 19}, {NULL}};
const Segment SGI211595[] = {{&S87744[69], 6}, {NULL}};
const Segment SGI211596[] = {{&S82383[30], 6}, {NULL}};
const Segment SGI211597[] = {{&S68507[67], 8}, {NULL}};
const Segment SGI211598[] = {{&S87744[137], 8}, {NULL}};
const Segment SGI211599[] = {{&S82383[128], 3}, {NULL}};
const Segment SGI211600[] = {{&S82383[93], 6}, {NULL}};
const Segment SGI211601[] = {{&S82383[65], 8}, {NULL}};
const Segment SGI211602[] = {{&S68508[44], 10}, {NULL}};
const Segment SGI211603[] = {{&S87745[34], 9}, {NULL}};
const Segment SGI211604[] = {{&S68508[44], 22}, {NULL}};
const Segment SGI211605[] = {{&S68508[30], 6}, {NULL}};
const Segment SGI211606[] = {{&S87745[27], 6}, {NULL}};
const Segment SGI211607[] = {{&S68508[133], 3}, {NULL}};
const Segment SGI211608[] = {{&S68508[98], 6}, {NULL}};
const Segment SGI211609[] = {{&S87745[96], 9}, {NULL}};
const Segment SGI211610[] = {{&S82384[44], 7}, {NULL}};
const Segment SGI211611[] = {{&S87745[69], 6}, {NULL}};
const Segment SGI211612[] = {{&S82384[44], 19}, {NULL}};
const Segment SGI211613[] = {{&S68508[68], 8}, {NULL}};
const Segment SGI211614[] = {{&S82384[30], 6}, {NULL}};
const Segment SGI211615[] = {{&S87745[137], 8}, {NULL}};
const Segment SGI211616[] = {{&S82384[128], 3}, {NULL}};
const Segment SGI211617[] = {{&S82384[93], 6}, {NULL}};
const Segment SGI211618[] = {{&S82384[65], 8}, {NULL}};
const Segment SGI211619[] = {{&S68512[34], 6}, {NULL}};
const Segment SGI211620[] = {{&S68512[27], 6}, {NULL}};
const Segment SGI211621[] = {{&S68512[92], 9}, {NULL}};
const Segment SGI211622[] = {{&S68512[65], 6}, {NULL}};
const Segment SGI211623[] = {{&S87754[34], 10}, {NULL}};
const Segment SGI211624[] = {{&S87754[27], 6}, {NULL}};
const Segment SGI211625[] = {{&S87754[97], 9}, {NULL}};
const Segment SGI211626[] = {{&S87754[70], 6}, {NULL}};
const Segment SGI211627[] = {{&S87758[34], 7}, {NULL}};
const Segment SGI211628[] = {{&S87758[27], 6}, {NULL}};
const Segment SGI211629[] = {{&S87758[99], 9}, {NULL}};
const Segment SGI211630[] = {{&S87758[72], 6}, {NULL}};
const Segment SGI211631[] = {{&S87764[44], 6}, {NULL}};
const Segment SGI211632[] = {{&S87764[44], 18}, {NULL}};
const Segment SGI211633[] = {{&S87764[30], 6}, {NULL}};
const Segment SGI211634[] = {{&S87764[76], 6}, {NULL}};
const Segment SGI211635[] = {{&S87764[103], 13}, {NULL}};
const Segment SGI211636[] = {{&S87777[34], 10}, {NULL}};
const Segment SGI211637[] = {{&S87777[27], 6}, {NULL}};
const Segment SGI211638[] = {{&S68543[34], 8}, {NULL}};
const Segment SGI211639[] = {{&S68543[27], 6}, {NULL}};
const Segment SGI211640[] = {{&S87777[97], 9}, {NULL}};
const Segment SGI211641[] = {{&S87777[70], 6}, {NULL}};
const Segment SGI211642[] = {{&S68543[94], 9}, {NULL}};
const Segment SGI211643[] = {{&S68543[67], 6}, {NULL}};
const Segment SGI211644[] = {{&S87781[34], 10}, {NULL}};
const Segment SGI211645[] = {{&S87781[27], 6}, {NULL}};
const Segment SGI211646[] = {{&S87781[97], 9}, {NULL}};
const Segment SGI211647[] = {{&S87781[70], 6}, {NULL}};
const Segment SGI211648[] = {{&S82425[34], 7}, {NULL}};
const Segment SGI211649[] = {{&S82425[27], 6}, {NULL}};
const Segment SGI211650[] = {{&S82425[110], 9}, {NULL}};
const Segment SGI211651[] = {{&S82425[83], 6}, {NULL}};
const Segment SGI211652[] = {{&S87782[34], 11}, {NULL}};
const Segment SGI211653[] = {{&S87782[27], 6}, {NULL}};
const Segment SGI211654[] = {{&S82425[151], 8}, {NULL}};
const Segment SGI211655[] = {{&S87782[98], 9}, {NULL}};
const Segment SGI211656[] = {{&S87782[71], 6}, {NULL}};
const Segment SGI211657[] = {{&S82432[44], 6}, {NULL}};
const Segment SGI211658[] = {{&S82432[44], 18}, {NULL}};
const Segment SGI211659[] = {{&S82432[30], 6}, {NULL}};
const Segment SGI211660[] = {{&S82432[76], 6}, {NULL}};
const Segment SGI211661[] = {{&S82432[103], 13}, {NULL}};
const Segment SGI211662[] = {{&S77704[85], 6}, {NULL}};
const Segment SGI211663[] = {{&S87811[44], 7}, {NULL}};
const Segment SGI211664[] = {{&S87811[44], 19}, {NULL}};
const Segment SGI211665[] = {{&S87811[30], 6}, {NULL}};
const Segment SGI211666[] = {{&S87811[31], 3}, {NULL}};
const Segment SGI211667[] = {{&S87811[96], 6}, {NULL}};
const Segment SGI211668[] = {{&S87811[65], 8}, {NULL}};
const Segment SGI211669[] = {{&S77216[34], 7}, {NULL}};
const Segment SGI211670[] = {{&S77216[27], 6}, {NULL}};
const Segment SGI211671[] = {{&S77216[94], 9}, {NULL}};
const Segment SGI211672[] = {{&S77216[67], 6}, {NULL}};
const Segment SGI211673[] = {{&S87812[44], 7}, {NULL}};
const Segment SGI211674[] = {{&S87812[44], 19}, {NULL}};
const Segment SGI211675[] = {{&S87812[30], 6}, {NULL}};
const Segment SGI211676[] = {{&S87812[31], 3}, {NULL}};
const Segment SGI211677[] = {{&S87812[92], 6}, {NULL}};
const Segment SGI211678[] = {{&S87812[65], 8}, {NULL}};
const Segment SGI211679[] = {{&S87814[45], 10}, {NULL}};
const Segment SGI211680[] = {{&S87814[30], 6}, {NULL}};
const Segment SGI211681[] = {{&S87814[31], 3}, {NULL}};
const Segment SGI211682[] = {{&S87814[81], 5}, {NULL}};
const Segment SGI211683[] = {{&S87814[139], 13}, {NULL}};
const Segment SGI211684[] = {{&S77711[34], 10}, {NULL}};
const Segment SGI211685[] = {{&S77711[27], 6}, {NULL}};
const Segment SGI211686[] = {{&S77711[97], 9}, {NULL}};
const Segment SGI211687[] = {{&S77711[70], 6}, {NULL}};
const Segment SGI211688[] = {{&S77713[34], 9}, {NULL}};
const Segment SGI211689[] = {{&S77713[27], 6}, {NULL}};
const Segment SGI211690[] = {{&S77713[96], 9}, {NULL}};
const Segment SGI211691[] = {{&S77713[69], 6}, {NULL}};
const Segment SGI211692[] = {{&S87823[45], 7}, {NULL}};
const Segment SGI211693[] = {{&S87823[30], 6}, {NULL}};
const Segment SGI211694[] = {{&S87823[31], 3}, {NULL}};
const Segment SGI211695[] = {{&S87823[83], 5}, {NULL}};
const Segment SGI211696[] = {{&S87823[141], 13}, {NULL}};
const Segment SGI211697[] = {{&S77719[44], 14}, {NULL}};
const Segment SGI211698[] = {{&S77719[44], 26}, {NULL}};
const Segment SGI211699[] = {{&S77719[30], 6}, {NULL}};
const Segment SGI211700[] = {{&S77719[119], 3}, {NULL}};
const Segment SGI211701[] = {{&S77719[84], 6}, {NULL}};
const Segment SGI211702[] = {{&S87827[34], 7}, {NULL}};
const Segment SGI211703[] = {{&S87827[27], 6}, {NULL}};
const Segment SGI211704[] = {{&S87827[99], 9}, {NULL}};
const Segment SGI211705[] = {{&S87827[72], 6}, {NULL}};
const Segment SGI211706[] = {{&S68619[44], 6}, {NULL}};
const Segment SGI211707[] = {{&S68619[44], 17}, {NULL}};
const Segment SGI211708[] = {{&S68619[30], 6}, {NULL}};
const Segment SGI211709[] = {{&S68619[166], 3}, {NULL}};
const Segment SGI211710[] = {{&S68619[131], 6}, {NULL}};
const Segment SGI211711[] = {{&S68619[63], 8}, {NULL}};
const Segment SGI211712[] = {{&S87847[34], 14}, {NULL}};
const Segment SGI211713[] = {{&S87847[27], 6}, {NULL}};
const Segment SGI211714[] = {{&S87847[101], 9}, {NULL}};
const Segment SGI211715[] = {{&S87847[74], 6}, {NULL}};
const Segment SGI211716[] = {{&S68621[44], 6}, {NULL}};
const Segment SGI211717[] = {{&S68621[44], 18}, {NULL}};
const Segment SGI211718[] = {{&S68621[30], 6}, {NULL}};
const Segment SGI211719[] = {{&S68621[167], 3}, {NULL}};
const Segment SGI211720[] = {{&S68621[132], 6}, {NULL}};
const Segment SGI211721[] = {{&S68621[64], 8}, {NULL}};
const Segment SGI211722[] = {{&S87850[44], 11}, {NULL}};
const Segment SGI211723[] = {{&S87850[44], 23}, {NULL}};
const Segment SGI211724[] = {{&S87850[30], 6}, {NULL}};
const Segment SGI211725[] = {{&S87850[31], 3}, {NULL}};
const Segment SGI211726[] = {{&S87850[81], 6}, {NULL}};
const Segment SGI211727[] = {{&S87850[144], 3}, {NULL}};
const Segment SGI211728[] = {{&S77753[34], 7}, {NULL}};
const Segment SGI211729[] = {{&S77753[27], 6}, {NULL}};
const Segment SGI211730[] = {{&S77753[99], 9}, {NULL}};
const Segment SGI211731[] = {{&S87851[44], 10}, {NULL}};
const Segment SGI211732[] = {{&S77753[72], 6}, {NULL}};
const Segment SGI211733[] = {{&S87851[44], 22}, {NULL}};
const Segment SGI211734[] = {{&S87851[30], 6}, {NULL}};
const Segment SGI211735[] = {{&S87851[31], 3}, {NULL}};
const Segment SGI211736[] = {{&S87851[80], 6}, {NULL}};
const Segment SGI211737[] = {{&S87851[143], 3}, {NULL}};
const Segment SGI211738[] = {{&S77754[34], 11}, {NULL}};
const Segment SGI211739[] = {{&S77754[27], 6}, {NULL}};
const Segment SGI211740[] = {{&S77754[98], 9}, {NULL}};
const Segment SGI211741[] = {{&S77754[71], 6}, {NULL}};
const Segment SGI211742[] = {{&S77755[44], 15}, {NULL}};
const Segment SGI211743[] = {{&S77755[44], 39}, {NULL}};
const Segment SGI211744[] = {{&S77755[30], 6}, {NULL}};
const Segment SGI211745[] = {{&S77755[132], 3}, {NULL}};
const Segment SGI211746[] = {{&S77755[97], 6}, {NULL}};
const Segment SGI211747[] = {{&S77755[157], 7}, {NULL}};
const Segment SGI211748[] = {{&S87854[34], 10}, {NULL}};
const Segment SGI211749[] = {{&S87854[27], 6}, {NULL}};
const Segment SGI211750[] = {{&S87854[108], 9}, {NULL}};
const Segment SGI211751[] = {{&S87854[81], 6}, {NULL}};
const Segment SGI211752[] = {{&S77756[44], 15}, {NULL}};
const Segment SGI211753[] = {{&S77756[44], 30}, {NULL}};
const Segment SGI211754[] = {{&S77756[30], 6}, {NULL}};
const Segment SGI211755[] = {{&S77756[123], 3}, {NULL}};
const Segment SGI211756[] = {{&S77756[88], 6}, {NULL}};
const Segment SGI211757[] = {{&S77756[148], 7}, {NULL}};
const Segment SGI211758[] = {{&S77757[44], 15}, {NULL}};
const Segment SGI211759[] = {{&S87856[34], 11}, {NULL}};
const Segment SGI211760[] = {{&S77757[44], 32}, {NULL}};
const Segment SGI211761[] = {{&S77757[30], 6}, {NULL}};
const Segment SGI211762[] = {{&S87856[27], 6}, {NULL}};
const Segment SGI211763[] = {{&S77757[125], 3}, {NULL}};
const Segment SGI211764[] = {{&S87856[98], 9}, {NULL}};
const Segment SGI211765[] = {{&S77757[90], 6}, {NULL}};
const Segment SGI211766[] = {{&S87856[71], 6}, {NULL}};
const Segment SGI211767[] = {{&S77757[150], 7}, {NULL}};
const Segment SGI211768[] = {{&S87857[34], 7}, {NULL}};
const Segment SGI211769[] = {{&S87857[27], 6}, {NULL}};
const Segment SGI211770[] = {{&S87857[99], 9}, {NULL}};
const Segment SGI211771[] = {{&S87857[72], 6}, {NULL}};
const Segment SGI211772[] = {{&S87858[34], 11}, {NULL}};
const Segment SGI211773[] = {{&S87858[27], 6}, {NULL}};
const Segment SGI211774[] = {{&S87858[98], 9}, {NULL}};
const Segment SGI211775[] = {{&S87858[71], 6}, {NULL}};
const Segment SGI211776[] = {{&S87860[44], 7}, {NULL}};
const Segment SGI211777[] = {{&S87860[44], 24}, {NULL}};
const Segment SGI211778[] = {{&S87860[30], 6}, {NULL}};
const Segment SGI211779[] = {{&S87860[82], 6}, {NULL}};
const Segment SGI211780[] = {{&S87860[109], 13}, {NULL}};
const Segment SGI211781[] = {{&S87862[34], 17}, {NULL}};
const Segment SGI211782[] = {{&S87862[27], 6}, {NULL}};
const Segment SGI211783[] = {{&S87862[111], 9}, {NULL}};
const Segment SGI211784[] = {{&S87862[84], 6}, {NULL}};
const Segment SGI211785[] = {{&S87863[34], 13}, {NULL}};
const Segment SGI211786[] = {{&S87863[27], 6}, {NULL}};
const Segment SGI211787[] = {{&S87863[100], 9}, {NULL}};
const Segment SGI211788[] = {{&S87863[73], 6}, {NULL}};
const Segment SGI211789[] = {{&S87864[34], 8}, {NULL}};
const Segment SGI211790[] = {{&S87864[27], 6}, {NULL}};
const Segment SGI211791[] = {{&S87864[100], 9}, {NULL}};
const Segment SGI211792[] = {{&S87864[73], 6}, {NULL}};
const Segment SGI211793[] = {{&S68651[34], 7}, {NULL}};
const Segment SGI211794[] = {{&S68651[27], 6}, {NULL}};
const Segment SGI211795[] = {{&S68651[94], 9}, {NULL}};
const Segment SGI211796[] = {{&S68651[67], 6}, {NULL}};
const Segment SGI211797[] = {{&S87866[34], 10}, {NULL}};
const Segment SGI211798[] = {{&S87866[27], 6}, {NULL}};
const Segment SGI211799[] = {{&S87866[108], 9}, {NULL}};
const Segment SGI211800[] = {{&S87866[81], 6}, {NULL}};
const Segment SGI211801[] = {{&S68652[34], 11}, {NULL}};
const Segment SGI211802[] = {{&S68652[27], 6}, {NULL}};
const Segment SGI211803[] = {{&S68652[97], 9}, {NULL}};
const Segment SGI211804[] = {{&S87867[34], 7}, {NULL}};
const Segment SGI211805[] = {{&S68652[70], 6}, {NULL}};
const Segment SGI211806[] = {{&S87867[27], 6}, {NULL}};
const Segment SGI211807[] = {{&S87867[97], 9}, {NULL}};
const Segment SGI211808[] = {{&S87867[70], 6}, {NULL}};
const Segment SGI211809[] = {{&S87867[138], 8}, {NULL}};
const Segment SGI211810[] = {{&S87869[27], 6}, {NULL}};
const Segment SGI211811[] = {{&S87869[103], 9}, {NULL}};
const Segment SGI211812[] = {{&S87869[76], 6}, {NULL}};
const Segment SGI211813[] = {{&S77782[34], 10}, {NULL}};
const Segment SGI211814[] = {{&S77782[27], 6}, {NULL}};
const Segment SGI211815[] = {{&S77782[97], 9}, {NULL}};
const Segment SGI211816[] = {{&S77782[70], 6}, {NULL}};
const Segment SGI211817[] = {{&S87885[34], 10}, {NULL}};
const Segment SGI211818[] = {{&S87885[27], 6}, {NULL}};
const Segment SGI211819[] = {{&S87885[97], 9}, {NULL}};
const Segment SGI211820[] = {{&S87885[70], 6}, {NULL}};
const Segment SGI211821[] = {{&S87886[34], 13}, {NULL}};
const Segment SGI211822[] = {{&S87886[27], 6}, {NULL}};
const Segment SGI211823[] = {{&S87886[100], 9}, {NULL}};
const Segment SGI211824[] = {{&S87886[73], 6}, {NULL}};
const Segment SGI211825[] = {{&S68681[34], 10}, {NULL}};
const Segment SGI211826[] = {{&S68681[27], 6}, {NULL}};
const Segment SGI211827[] = {{&S68681[96], 9}, {NULL}};
const Segment SGI211828[] = {{&S68681[69], 6}, {NULL}};
const Segment SGI211829[] = {{&S68682[34], 10}, {NULL}};
const Segment SGI211830[] = {{&S68682[27], 6}, {NULL}};
const Segment SGI211831[] = {{&S68682[96], 9}, {NULL}};
const Segment SGI211832[] = {{&S68682[69], 6}, {NULL}};
const Segment SGI211833[] = {{&S77343[34], 10}, {NULL}};
const Segment SGI211834[] = {{&S77343[27], 6}, {NULL}};
const Segment SGI211835[] = {{&S77343[97], 9}, {NULL}};
const Segment SGI211836[] = {{&S77343[70], 6}, {NULL}};
const Segment SGI211837[] = {{&S68698[41], 10}, {NULL}};
const Segment SGI211838[] = {{&S68698[41], 21}, {NULL}};
const Segment SGI211839[] = {{&S68698[27], 6}, {NULL}};
const Segment SGI211840[] = {{&S68698[111], 3}, {NULL}};
const Segment SGI211841[] = {{&S68698[115], 9}, {NULL}};
const Segment SGI211842[] = {{&S68698[76], 6}, {NULL}};
const Segment SGI211843[] = {{&S77835[44], 18}, {NULL}};
const Segment SGI211844[] = {{&S77835[44], 30}, {NULL}};
const Segment SGI211845[] = {{&S77835[30], 6}, {NULL}};
const Segment SGI211846[] = {{&S77835[123], 3}, {NULL}};
const Segment SGI211847[] = {{&S77835[88], 6}, {NULL}};
const Segment SGI211848[] = {{&S77844[44], 8}, {NULL}};
const Segment SGI211849[] = {{&S77844[44], 20}, {NULL}};
const Segment SGI211850[] = {{&S77844[30], 6}, {NULL}};
const Segment SGI211851[] = {{&S77844[113], 3}, {NULL}};
const Segment SGI211852[] = {{&S77844[78], 6}, {NULL}};
const Segment SGI211853[] = {{&S77846[34], 10}, {NULL}};
const Segment SGI211854[] = {{&S77846[27], 6}, {NULL}};
const Segment SGI211855[] = {{&S77846[97], 9}, {NULL}};
const Segment SGI211856[] = {{&S77846[70], 6}, {NULL}};
const Segment SGI211857[] = {{&S77847[34], 7}, {NULL}};
const Segment SGI211858[] = {{&S77847[27], 6}, {NULL}};
const Segment SGI211859[] = {{&S77847[99], 9}, {NULL}};
const Segment SGI211860[] = {{&S77847[72], 6}, {NULL}};
const Segment SGI211861[] = {{&S77848[34], 9}, {NULL}};
const Segment SGI211862[] = {{&S77848[27], 6}, {NULL}};
const Segment SGI211863[] = {{&S77848[96], 9}, {NULL}};
const Segment SGI211864[] = {{&S77848[69], 6}, {NULL}};
const Segment SGI211865[] = {{&S68722[44], 10}, {NULL}};
const Segment SGI211866[] = {{&S68722[44], 21}, {NULL}};
const Segment SGI211867[] = {{&S68722[30], 6}, {NULL}};
const Segment SGI211868[] = {{&S68722[114], 3}, {NULL}};
const Segment SGI211869[] = {{&S68722[79], 6}, {NULL}};
const Segment SGI211870[] = {{&S68722[139], 7}, {NULL}};
const Segment SGI211871[] = {{&S87944[34], 7}, {NULL}};
const Segment SGI211872[] = {{&S87944[27], 6}, {NULL}};
const Segment SGI211873[] = {{&S87944[94], 9}, {NULL}};
const Segment SGI211874[] = {{&S87944[67], 6}, {NULL}};
const Segment SGI211875[] = {{&S87946[34], 12}, {NULL}};
const Segment SGI211876[] = {{&S87946[27], 6}, {NULL}};
const Segment SGI211877[] = {{&S87946[97], 9}, {NULL}};
const Segment SGI211878[] = {{&S87946[70], 6}, {NULL}};
const Segment SGI211879[] = {{&S87952[34], 11}, {NULL}};
const Segment SGI211880[] = {{&S87952[27], 6}, {NULL}};
const Segment SGI211881[] = {{&S87952[98], 9}, {NULL}};
const Segment SGI211882[] = {{&S87952[71], 6}, {NULL}};
const Segment SGI211883[] = {{&S82479[44], 11}, {NULL}};
const Segment SGI211884[] = {{&S82479[44], 23}, {NULL}};
const Segment SGI211885[] = {{&S82479[30], 6}, {NULL}};
const Segment SGI211886[] = {{&S82479[116], 3}, {NULL}};
const Segment SGI211887[] = {{&S82479[81], 6}, {NULL}};
const Segment SGI211888[] = {{&S82483[44], 11}, {NULL}};
const Segment SGI211889[] = {{&S82483[44], 23}, {NULL}};
const Segment SGI211890[] = {{&S82483[30], 6}, {NULL}};
const Segment SGI211891[] = {{&S82483[116], 3}, {NULL}};
const Segment SGI211892[] = {{&S82483[81], 6}, {NULL}};
const Segment SGI211893[] = {{&S82485[44], 7}, {NULL}};
const Segment SGI211894[] = {{&S82485[44], 24}, {NULL}};
const Segment SGI211895[] = {{&S82485[30], 6}, {NULL}};
const Segment SGI211896[] = {{&S82485[117], 3}, {NULL}};
const Segment SGI211897[] = {{&S82485[82], 6}, {NULL}};
const Segment SGI211898[] = {{&S87967[44], 21}, {NULL}};
const Segment SGI211899[] = {{&S87967[44], 33}, {NULL}};
const Segment SGI211900[] = {{&S77404[44], 9}, {NULL}};
const Segment SGI211901[] = {{&S87967[30], 6}, {NULL}};
const Segment SGI211902[] = {{&S77404[44], 21}, {NULL}};
const Segment SGI211903[] = {{&S77404[30], 6}, {NULL}};
const Segment SGI211904[] = {{&S87967[31], 3}, {NULL}};
const Segment SGI211905[] = {{&S77404[114], 3}, {NULL}};
const Segment SGI211906[] = {{&S87967[91], 6}, {NULL}};
const Segment SGI211907[] = {{&S77404[79], 6}, {NULL}};
const Segment SGI211908[] = {{&S77405[44], 11}, {NULL}};
const Segment SGI211909[] = {{&S77405[44], 23}, {NULL}};
const Segment SGI211910[] = {{&S77405[30], 6}, {NULL}};
const Segment SGI211911[] = {{&S77405[116], 3}, {NULL}};
const Segment SGI211912[] = {{&S77405[81], 6}, {NULL}};
const Segment SGI211913[] = {{&S87971[44], 10}, {NULL}};
const Segment SGI211914[] = {{&S87971[44], 22}, {NULL}};
const Segment SGI211915[] = {{&S87971[30], 6}, {NULL}};
const Segment SGI211916[] = {{&S87971[31], 3}, {NULL}};
const Segment SGI211917[] = {{&S87971[80], 5}, {NULL}};
const Segment SGI211918[] = {{&S87971[117], 14}, {NULL}};
const Segment SGI211919[] = {{&S87972[44], 11}, {NULL}};
const Segment SGI211920[] = {{&S87972[44], 23}, {NULL}};
const Segment SGI211921[] = {{&S87972[30], 6}, {NULL}};
const Segment SGI211922[] = {{&S87972[31], 3}, {NULL}};
const Segment SGI211923[] = {{&S87972[81], 5}, {NULL}};
const Segment SGI211924[] = {{&S87972[118], 14}, {NULL}};
const Segment SGI211925[] = {{&S87974[34], 8}, {NULL}};
const Segment SGI211926[] = {{&S87974[27], 6}, {NULL}};
const Segment SGI211927[] = {{&S87974[95], 9}, {NULL}};
const Segment SGI211928[] = {{&S87974[68], 6}, {NULL}};
const Segment SGI211929[] = {{&S87978[34], 13}, {NULL}};
const Segment SGI211930[] = {{&S87978[27], 6}, {NULL}};
const Segment SGI211931[] = {{&S87978[100], 9}, {NULL}};
const Segment SGI211932[] = {{&S87978[73], 6}, {NULL}};
const Segment SGI211933[] = {{&S87979[44], 10}, {NULL}};
const Segment SGI211934[] = {{&S87979[44], 22}, {NULL}};
const Segment SGI211935[] = {{&S87979[30], 6}, {NULL}};
const Segment SGI211936[] = {{&S87979[80], 6}, {NULL}};
const Segment SGI211937[] = {{&S87979[107], 13}, {NULL}};
const Segment SGI211938[] = {{&S87980[44], 11}, {NULL}};
const Segment SGI211939[] = {{&S87980[44], 23}, {NULL}};
const Segment SGI211940[] = {{&S87980[30], 6}, {NULL}};
const Segment SGI211941[] = {{&S87980[81], 6}, {NULL}};
const Segment SGI211942[] = {{&S87980[108], 13}, {NULL}};
const Segment SGI211943[] = {{&S87983[34], 13}, {NULL}};
const Segment SGI211944[] = {{&S87983[27], 6}, {NULL}};
const Segment SGI211945[] = {{&S87983[100], 9}, {NULL}};
const Segment SGI211946[] = {{&S87983[73], 6}, {NULL}};
const Segment SGI211947[] = {{&S87985[34], 8}, {NULL}};
const Segment SGI211948[] = {{&S87985[27], 6}, {NULL}};
const Segment SGI211949[] = {{&S87985[100], 9}, {NULL}};
const Segment SGI211950[] = {{&S87985[73], 6}, {NULL}};
const Segment SGI211951[] = {{&S88007[44], 6}, {NULL}};
const Segment SGI211952[] = {{&S88007[44], 18}, {NULL}};
const Segment SGI211953[] = {{&S88007[30], 6}, {NULL}};
const Segment SGI211954[] = {{&S88007[76], 6}, {NULL}};
const Segment SGI211955[] = {{&S88007[103], 13}, {NULL}};
const Segment SGI211956[] = {{&S88013[34], 8}, {NULL}};
const Segment SGI211957[] = {{&S88013[27], 6}, {NULL}};
const Segment SGI211958[] = {{&S88013[95], 9}, {NULL}};
const Segment SGI211959[] = {{&S88013[68], 6}, {NULL}};
const Segment SGI211960[] = {{&S88022[34], 10}, {NULL}};
const Segment SGI211961[] = {{&S88022[27], 6}, {NULL}};
const Segment SGI211962[] = {{&S88022[97], 9}, {NULL}};
const Segment SGI211963[] = {{&S88022[70], 6}, {NULL}};
const Segment SGI211964[] = {{&S88025[44], 15}, {NULL}};
const Segment SGI211965[] = {{&S88025[44], 42}, {NULL}};
const Segment SGI211966[] = {{&S88025[30], 6}, {NULL}};
const Segment SGI211967[] = {{&S68827[34], 10}, {NULL}};
const Segment SGI211968[] = {{&S68827[27], 6}, {NULL}};
const Segment SGI211969[] = {{&S88025[100], 6}, {NULL}};
const Segment SGI211970[] = {{&S88025[127], 13}, {NULL}};
const Segment SGI211971[] = {{&S68827[97], 9}, {NULL}};
const Segment SGI211972[] = {{&S68827[70], 6}, {NULL}};
const Segment SGI211973[] = {{&S88033[34], 7}, {NULL}};
const Segment SGI211974[] = {{&S88033[27], 6}, {NULL}};
const Segment SGI211975[] = {{&S88033[90], 9}, {NULL}};
const Segment SGI211976[] = {{&S88033[63], 6}, {NULL}};
const Segment SGI211977[] = {{&S88034[34], 7}, {NULL}};
const Segment SGI211978[] = {{&S88034[27], 6}, {NULL}};
const Segment SGI211979[] = {{&S88034[94], 9}, {NULL}};
const Segment SGI211980[] = {{&S88034[67], 6}, {NULL}};
const Segment SGI211981[] = {{&S88038[34], 10}, {NULL}};
const Segment SGI211982[] = {{&S88038[27], 6}, {NULL}};
const Segment SGI211983[] = {{&S88038[97], 9}, {NULL}};
const Segment SGI211984[] = {{&S88038[70], 6}, {NULL}};
const Segment SGI211985[] = {{&S82575[44], 9}, {NULL}};
const Segment SGI211986[] = {{&S82575[44], 21}, {NULL}};
const Segment SGI211987[] = {{&S82575[30], 6}, {NULL}};
const Segment SGI211988[] = {{&S82575[114], 3}, {NULL}};
const Segment SGI211989[] = {{&S82575[79], 6}, {NULL}};
const Segment SGI211990[] = {{&S77851[34], 11}, {NULL}};
const Segment SGI211991[] = {{&S77851[27], 6}, {NULL}};
const Segment SGI211992[] = {{&S77851[98], 9}, {NULL}};
const Segment SGI211993[] = {{&S77851[71], 6}, {NULL}};
const Segment SGI211994[] = {{&S82579[44], 16}, {NULL}};
const Segment SGI211995[] = {{&S82579[44], 28}, {NULL}};
const Segment SGI211996[] = {{&S82579[30], 6}, {NULL}};
const Segment SGI211997[] = {{&S82579[121], 3}, {NULL}};
const Segment SGI211998[] = {{&S82579[86], 6}, {NULL}};
const Segment SGI211999[] = {{&S82580[45], 13}, {NULL}};
const Segment SGI212000[] = {{&S82580[30], 6}, {NULL}};
const Segment SGI212001[] = {{&S77856[34], 14}, {NULL}};
const Segment SGI212002[] = {{&S77856[27], 6}, {NULL}};
const Segment SGI212003[] = {{&S82580[118], 3}, {NULL}};
const Segment SGI212004[] = {{&S82580[84], 5}, {NULL}};
const Segment SGI212005[] = {{&S82580[142], 13}, {NULL}};
const Segment SGI212006[] = {{&S77856[101], 9}, {NULL}};
const Segment SGI212007[] = {{&S77856[74], 6}, {NULL}};
const Segment SGI212008[] = {{&S68853[34], 10}, {NULL}};
const Segment SGI212009[] = {{&S68853[27], 6}, {NULL}};
const Segment SGI212010[] = {{&S77857[34], 14}, {NULL}};
const Segment SGI212011[] = {{&S68853[96], 9}, {NULL}};
const Segment SGI212012[] = {{&S77857[27], 6}, {NULL}};
const Segment SGI212013[] = {{&S68853[69], 6}, {NULL}};
const Segment SGI212014[] = {{&S77857[101], 9}, {NULL}};
const Segment SGI212015[] = {{&S77857[74], 6}, {NULL}};
const Segment SGI212016[] = {{&S68854[34], 10}, {NULL}};
const Segment SGI212017[] = {{&S68854[27], 6}, {NULL}};
const Segment SGI212018[] = {{&S68854[96], 9}, {NULL}};
const Segment SGI212019[] = {{&S68854[69], 6}, {NULL}};
const Segment SGI212020[] = {{&S68855[34], 10}, {NULL}};
const Segment SGI212021[] = {{&S68855[27], 6}, {NULL}};
const Segment SGI212022[] = {{&S68855[96], 9}, {NULL}};
const Segment SGI212023[] = {{&S68855[69], 6}, {NULL}};
const Segment SGI212024[] = {{&S68862[44], 10}, {NULL}};
const Segment SGI212025[] = {{&S68862[44], 21}, {NULL}};
const Segment SGI212026[] = {{&S77514[34], 25}, {NULL}};
const Segment SGI212027[] = {{&S68862[30], 6}, {NULL}};
const Segment SGI212028[] = {{&S77514[27], 6}, {NULL}};
const Segment SGI212029[] = {{&S68862[114], 3}, {NULL}};
const Segment SGI212030[] = {{&S68862[79], 6}, {NULL}};
const Segment SGI212031[] = {{&S77514[112], 9}, {NULL}};
const Segment SGI212032[] = {{&S77514[85], 6}, {NULL}};
const Segment SGI212033[] = {{&S88069[44], 13}, {NULL}};
const Segment SGI212034[] = {{&S88069[44], 25}, {NULL}};
const Segment SGI212035[] = {{&S88069[30], 6}, {NULL}};
const Segment SGI212036[] = {{&S88069[31], 3}, {NULL}};
const Segment SGI212037[] = {{&S88069[83], 6}, {NULL}};
const Segment SGI212038[] = {{&S88070[45], 7}, {NULL}};
const Segment SGI212039[] = {{&S88070[30], 6}, {NULL}};
const Segment SGI212040[] = {{&S88070[31], 3}, {NULL}};
const Segment SGI212041[] = {{&S88070[83], 5}, {NULL}};
const Segment SGI212042[] = {{&S88070[141], 13}, {NULL}};
const Segment SGI212043[] = {{&S88072[44], 13}, {NULL}};
const Segment SGI212044[] = {{&S88072[44], 25}, {NULL}};
const Segment SGI212045[] = {{&S88072[30], 6}, {NULL}};
const Segment SGI212046[] = {{&S88072[31], 3}, {NULL}};
const Segment SGI212047[] = {{&S88072[98], 6}, {NULL}};
const Segment SGI212048[] = {{&S88072[71], 8}, {NULL}};
const Segment SGI212049[] = {{&S77885[44], 14}, {NULL}};
const Segment SGI212050[] = {{&S77885[44], 26}, {NULL}};
const Segment SGI212051[] = {{&S77885[30], 6}, {NULL}};
const Segment SGI212052[] = {{&S88073[34], 10}, {NULL}};
const Segment SGI212053[] = {{&S88073[27], 6}, {NULL}};
const Segment SGI212054[] = {{&S77885[119], 3}, {NULL}};
const Segment SGI212055[] = {{&S77885[84], 6}, {NULL}};
const Segment SGI212056[] = {{&S88073[97], 9}, {NULL}};
const Segment SGI212057[] = {{&S88073[70], 6}, {NULL}};
const Segment SGI212058[] = {{&S77886[44], 14}, {NULL}};
const Segment SGI212059[] = {{&S77886[44], 25}, {NULL}};
const Segment SGI212060[] = {{&S77886[30], 6}, {NULL}};
const Segment SGI212061[] = {{&S77886[118], 3}, {NULL}};
const Segment SGI212062[] = {{&S88074[45], 7}, {NULL}};
const Segment SGI212063[] = {{&S77886[83], 6}, {NULL}};
const Segment SGI212064[] = {{&S88074[30], 6}, {NULL}};
const Segment SGI212065[] = {{&S88074[31], 3}, {NULL}};
const Segment SGI212066[] = {{&S88074[83], 5}, {NULL}};
const Segment SGI212067[] = {{&S88074[141], 13}, {NULL}};
const Segment SGI212068[] = {{&S77888[44], 10}, {NULL}};
const Segment SGI212069[] = {{&S77888[44], 22}, {NULL}};
const Segment SGI212070[] = {{&S77888[30], 6}, {NULL}};
const Segment SGI212071[] = {{&S82616[34], 10}, {NULL}};
const Segment SGI212072[] = {{&S77888[131], 3}, {NULL}};
const Segment SGI212073[] = {{&S82616[27], 6}, {NULL}};
const Segment SGI212074[] = {{&S77888[96], 6}, {NULL}};
const Segment SGI212075[] = {{&S77888[68], 8}, {NULL}};
const Segment SGI212076[] = {{&S88076[44], 11}, {NULL}};
const Segment SGI212077[] = {{&S88076[44], 23}, {NULL}};
const Segment SGI212078[] = {{&S88076[30], 6}, {NULL}};
const Segment SGI212079[] = {{&S82616[97], 9}, {NULL}};
const Segment SGI212080[] = {{&S82616[70], 6}, {NULL}};
const Segment SGI212081[] = {{&S88076[31], 3}, {NULL}};
const Segment SGI212082[] = {{&S88076[81], 6}, {NULL}};
const Segment SGI212083[] = {{&S88076[140], 10}, {NULL}};
const Segment SGI212084[] = {{&S88079[45], 7}, {NULL}};
const Segment SGI212085[] = {{&S88079[30], 6}, {NULL}};
const Segment SGI212086[] = {{&S88079[31], 3}, {NULL}};
const Segment SGI212087[] = {{&S88079[83], 5}, {NULL}};
const Segment SGI212088[] = {{&S88079[141], 13}, {NULL}};
const Segment SGI212089[] = {{&S88081[45], 14}, {NULL}};
const Segment SGI212090[] = {{&S88081[30], 6}, {NULL}};
const Segment SGI212091[] = {{&S88081[31], 3}, {NULL}};
const Segment SGI212092[] = {{&S88081[85], 5}, {NULL}};
const Segment SGI212093[] = {{&S88081[143], 13}, {NULL}};
const Segment SGI212094[] = {{&S77899[34], 10}, {NULL}};
const Segment SGI212095[] = {{&S77899[27], 6}, {NULL}};
const Segment SGI212096[] = {{&S88084[44], 13}, {NULL}};
const Segment SGI212097[] = {{&S88084[44], 25}, {NULL}};
const Segment SGI212098[] = {{&S77899[97], 9}, {NULL}};
const Segment SGI212099[] = {{&S88084[30], 6}, {NULL}};
const Segment SGI212100[] = {{&S77899[70], 6}, {NULL}};
const Segment SGI212101[] = {{&S88084[31], 3}, {NULL}};
const Segment SGI212102[] = {{&S88084[83], 6}, {NULL}};
const Segment SGI212103[] = {{&S82633[34], 7}, {NULL}};
const Segment SGI212104[] = {{&S82633[27], 6}, {NULL}};
const Segment SGI212105[] = {{&S82633[94], 9}, {NULL}};
const Segment SGI212106[] = {{&S82633[67], 6}, {NULL}};
const Segment SGI212107[] = {{&S77559[44], 10}, {NULL}};
const Segment SGI212108[] = {{&S77559[44], 22}, {NULL}};
const Segment SGI212109[] = {{&S77559[30], 6}, {NULL}};
const Segment SGI212110[] = {{&S77559[115], 3}, {NULL}};
const Segment SGI212111[] = {{&S77559[80], 6}, {NULL}};
const Segment SGI212112[] = {{&S77912[34], 11}, {NULL}};
const Segment SGI212113[] = {{&S77912[27], 6}, {NULL}};
const Segment SGI212114[] = {{&S77912[98], 9}, {NULL}};
const Segment SGI212115[] = {{&S77912[71], 6}, {NULL}};
const Segment SGI212116[] = {{&S82641[44], 7}, {NULL}};
const Segment SGI212117[] = {{&S82641[44], 24}, {NULL}};
const Segment SGI212118[] = {{&S77912[139], 8}, {NULL}};
const Segment SGI212119[] = {{&S82641[30], 6}, {NULL}};
const Segment SGI212120[] = {{&S82641[82], 6}, {NULL}};
const Segment SGI212121[] = {{&S82641[109], 13}, {NULL}};
const Segment SGI212122[] = {{&S77913[34], 10}, {NULL}};
const Segment SGI212123[] = {{&S77913[27], 6}, {NULL}};
const Segment SGI212124[] = {{&S77913[97], 9}, {NULL}};
const Segment SGI212125[] = {{&S77913[70], 6}, {NULL}};
const Segment SGI212126[] = {{&S77913[138], 8}, {NULL}};
const Segment SGI212127[] = {{&S88100[34], 9}, {NULL}};
const Segment SGI212128[] = {{&S88100[27], 6}, {NULL}};
const Segment SGI212129[] = {{&S82643[34], 10}, {NULL}};
const Segment SGI212130[] = {{&S82643[27], 6}, {NULL}};
const Segment SGI212131[] = {{&S88100[103], 9}, {NULL}};
const Segment SGI212132[] = {{&S88100[76], 6}, {NULL}};
const Segment SGI212133[] = {{&S82643[97], 9}, {NULL}};
const Segment SGI212134[] = {{&S82643[70], 6}, {NULL}};
const Segment SGI212135[] = {{&S88102[44], 7}, {NULL}};
const Segment SGI212136[] = {{&S88102[44], 19}, {NULL}};
const Segment SGI212137[] = {{&S88102[30], 6}, {NULL}};
const Segment SGI212138[] = {{&S88102[31], 3}, {NULL}};
const Segment SGI212139[] = {{&S88102[77], 6}, {NULL}};
const Segment SGI212140[] = {{&S77920[34], 10}, {NULL}};
const Segment SGI212141[] = {{&S77920[27], 6}, {NULL}};
const Segment SGI212142[] = {{&S77920[97], 9}, {NULL}};
const Segment SGI212143[] = {{&S77920[70], 6}, {NULL}};
const Segment SGI212144[] = {{&S77921[34], 10}, {NULL}};
const Segment SGI212145[] = {{&S77921[27], 6}, {NULL}};
const Segment SGI212146[] = {{&S88107[44], 15}, {NULL}};
const Segment SGI212147[] = {{&S88107[44], 33}, {NULL}};
const Segment SGI212148[] = {{&S77921[97], 9}, {NULL}};
const Segment SGI212149[] = {{&S88107[30], 6}, {NULL}};
const Segment SGI212150[] = {{&S77921[70], 6}, {NULL}};
const Segment SGI212151[] = {{&S88107[31], 3}, {NULL}};
const Segment SGI212152[] = {{&S88107[91], 5}, {NULL}};
const Segment SGI212153[] = {{&S88107[128], 14}, {NULL}};
const Segment SGI212154[] = {{&S77922[34], 9}, {NULL}};
const Segment SGI212155[] = {{&S77922[27], 6}, {NULL}};
const Segment SGI212156[] = {{&S77922[96], 9}, {NULL}};
const Segment SGI212157[] = {{&S77922[69], 6}, {NULL}};
const Segment SGI212158[] = {{&S77923[34], 10}, {NULL}};
const Segment SGI212159[] = {{&S77923[27], 6}, {NULL}};
const Segment SGI212160[] = {{&S77923[97], 9}, {NULL}};
const Segment SGI212161[] = {{&S77923[70], 6}, {NULL}};
const Segment SGI212162[] = {{&S77924[34], 10}, {NULL}};
const Segment SGI212163[] = {{&S77924[27], 6}, {NULL}};
const Segment SGI212164[] = {{&S77924[97], 9}, {NULL}};
const Segment SGI212165[] = {{&S77924[70], 6}, {NULL}};
const Segment SGI212166[] = {{&S77926[34], 14}, {NULL}};
const Segment SGI212167[] = {{&S77926[27], 6}, {NULL}};
const Segment SGI212168[] = {{&S88115[44], 9}, {NULL}};
const Segment SGI212169[] = {{&S88115[44], 21}, {NULL}};
const Segment SGI212170[] = {{&S77926[106], 9}, {NULL}};
const Segment SGI212171[] = {{&S88115[30], 6}, {NULL}};
const Segment SGI212172[] = {{&S77926[79], 6}, {NULL}};
const Segment SGI212173[] = {{&S88115[79], 6}, {NULL}};
const Segment SGI212174[] = {{&S88115[106], 13}, {NULL}};
const Segment SGI212175[] = {{&S88116[34], 7}, {NULL}};
const Segment SGI212176[] = {{&S88116[27], 6}, {NULL}};
const Segment SGI212177[] = {{&S88116[99], 9}, {NULL}};
const Segment SGI212178[] = {{&S88116[72], 6}, {NULL}};
const Segment SGI212179[] = {{&S82663[34], 7}, {NULL}};
const Segment SGI212180[] = {{&S82663[27], 6}, {NULL}};
const Segment SGI212181[] = {{&S82663[94], 9}, {NULL}};
const Segment SGI212182[] = {{&S82663[67], 6}, {NULL}};
const Segment SGI212183[] = {{&S88118[34], 7}, {NULL}};
const Segment SGI212184[] = {{&S88118[27], 6}, {NULL}};
const Segment SGI212185[] = {{&S88118[99], 9}, {NULL}};
const Segment SGI212186[] = {{&S88118[72], 6}, {NULL}};
const Segment SGI212187[] = {{&S88119[44], 13}, {NULL}};
const Segment SGI212188[] = {{&S88119[44], 25}, {NULL}};
const Segment SGI212189[] = {{&S88119[30], 6}, {NULL}};
const Segment SGI212190[] = {{&S88119[83], 6}, {NULL}};
const Segment SGI212191[] = {{&S88119[110], 13}, {NULL}};
const Segment SGI212192[] = {{&S88123[34], 4}, {NULL}};
const Segment SGI212193[] = {{&S88123[27], 6}, {NULL}};
const Segment SGI212194[] = {{&S88123[106], 9}, {NULL}};
const Segment SGI212195[] = {{&S88123[79], 6}, {NULL}};
const Segment SGI212196[] = {{&S88124[44], 14}, {NULL}};
const Segment SGI212197[] = {{&S88124[44], 31}, {NULL}};
const Segment SGI212198[] = {{&S88124[30], 6}, {NULL}};
const Segment SGI212199[] = {{&S88124[89], 6}, {NULL}};
const Segment SGI212200[] = {{&S88124[116], 13}, {NULL}};
const Segment SGI212201[] = {{&S68863[38], 10}, {NULL}};
const Segment SGI212202[] = {{&S68863[31], 6}, {NULL}};
const Segment SGI212203[] = {{&S88131[44], 6}, {NULL}};
const Segment SGI212204[] = {{&S88131[44], 18}, {NULL}};
const Segment SGI212205[] = {{&S88131[30], 6}, {NULL}};
const Segment SGI212206[] = {{&S88131[76], 6}, {NULL}};
const Segment SGI212207[] = {{&S88131[103], 13}, {NULL}};
const Segment SGI212208[] = {{&S77611[44], 14}, {NULL}};
const Segment SGI212209[] = {{&S77611[44], 26}, {NULL}};
const Segment SGI212210[] = {{&S77611[30], 6}, {NULL}};
const Segment SGI212211[] = {{&S77611[119], 3}, {NULL}};
const Segment SGI212212[] = {{&S77611[84], 6}, {NULL}};
const Segment SGI212213[] = {{&S88132[44], 7}, {NULL}};
const Segment SGI212214[] = {{&S88132[44], 24}, {NULL}};
const Segment SGI212215[] = {{&S88132[30], 6}, {NULL}};
const Segment SGI212216[] = {{&S88132[82], 6}, {NULL}};
const Segment SGI212217[] = {{&S88132[109], 13}, {NULL}};
const Segment SGI212218[] = {{&S82686[44], 14}, {NULL}};
const Segment SGI212219[] = {{&S82686[44], 26}, {NULL}};
const Segment SGI212220[] = {{&S82686[30], 6}, {NULL}};
const Segment SGI212221[] = {{&S82686[148], 3}, {NULL}};
const Segment SGI212222[] = {{&S82686[113], 6}, {NULL}};
const Segment SGI212223[] = {{&S77613[44], 14}, {NULL}};
const Segment SGI212224[] = {{&S82686[72], 8}, {NULL}};
const Segment SGI212225[] = {{&S88133[34], 7}, {NULL}};
const Segment SGI212226[] = {{&S77613[44], 26}, {NULL}};
const Segment SGI212227[] = {{&S82686[84], 15}, {NULL}};
const Segment SGI212228[] = {{&S77613[30], 6}, {NULL}};
const Segment SGI212229[] = {{&S88133[27], 6}, {NULL}};
const Segment SGI212230[] = {{&S77613[119], 3}, {NULL}};
const Segment SGI212231[] = {{&S77613[84], 6}, {NULL}};
const Segment SGI212232[] = {{&S88133[93], 9}, {NULL}};
const Segment SGI212233[] = {{&S88133[66], 6}, {NULL}};
const Segment SGI212234[] = {{&S77613[144], 7}, {NULL}};
const Segment SGI212235[] = {{&S88133[134], 8}, {NULL}};
const Segment SGI212236[] = {{&S82687[44], 14}, {NULL}};
const Segment SGI212237[] = {{&S82687[44], 26}, {NULL}};
const Segment SGI212238[] = {{&S82687[30], 6}, {NULL}};
const Segment SGI212239[] = {{&S82687[148], 3}, {NULL}};
const Segment SGI212240[] = {{&S82687[113], 6}, {NULL}};
const Segment SGI212241[] = {{&S88134[34], 7}, {NULL}};
const Segment SGI212242[] = {{&S82687[72], 8}, {NULL}};
const Segment SGI212243[] = {{&S88134[27], 6}, {NULL}};
const Segment SGI212244[] = {{&S82687[84], 15}, {NULL}};
const Segment SGI212245[] = {{&S88134[98], 9}, {NULL}};
const Segment SGI212246[] = {{&S88134[71], 6}, {NULL}};
const Segment SGI212247[] = {{&S88134[139], 8}, {NULL}};
const Segment SGI212248[] = {{&S88136[34], 10}, {NULL}};
const Segment SGI212249[] = {{&S88136[27], 6}, {NULL}};
const Segment SGI212250[] = {{&S88136[97], 9}, {NULL}};
const Segment SGI212251[] = {{&S88136[70], 6}, {NULL}};
const Segment SGI212252[] = {{&S88136[131], 8}, {NULL}};
const Segment SGI212253[] = {{&S88137[34], 5}, {NULL}};
const Segment SGI212254[] = {{&S88137[27], 6}, {NULL}};
const Segment SGI212255[] = {{&S88137[92], 9}, {NULL}};
const Segment SGI212256[] = {{&S88137[65], 6}, {NULL}};
const Segment SGI212257[] = {{&S88138[34], 10}, {NULL}};
const Segment SGI212258[] = {{&S88138[27], 6}, {NULL}};
const Segment SGI212259[] = {{&S88138[97], 9}, {NULL}};
const Segment SGI212260[] = {{&S88138[70], 6}, {NULL}};
const Segment SGI212261[] = {{&S88139[44], 7}, {NULL}};
const Segment SGI212262[] = {{&S88139[44], 24}, {NULL}};
const Segment SGI212263[] = {{&S88139[30], 6}, {NULL}};
const Segment SGI212264[] = {{&S88139[82], 6}, {NULL}};
const Segment SGI212265[] = {{&S88139[109], 13}, {NULL}};
const Segment SGI212266[] = {{&S82698[44], 5}, {NULL}};
const Segment SGI212267[] = {{&S82698[44], 17}, {NULL}};
const Segment SGI212268[] = {{&S82698[30], 6}, {NULL}};
const Segment SGI212269[] = {{&S82698[110], 3}, {NULL}};
const Segment SGI212270[] = {{&S82698[75], 6}, {NULL}};
const Segment SGI212271[] = {{&S82698[135], 11}, {NULL}};
const Segment SGI212272[] = {{&S88142[44], 10}, {NULL}};
const Segment SGI212273[] = {{&S88142[44], 22}, {NULL}};
const Segment SGI212274[] = {{&S88142[30], 6}, {NULL}};
const Segment SGI212275[] = {{&S88142[31], 3}, {NULL}};
const Segment SGI212276[] = {{&S88142[80], 5}, {NULL}};
const Segment SGI212277[] = {{&S88142[117], 14}, {NULL}};
const Segment SGI212278[] = {{&S88143[34], 9}, {NULL}};
const Segment SGI212279[] = {{&S88143[27], 6}, {NULL}};
const Segment SGI212280[] = {{&S88143[96], 9}, {NULL}};
const Segment SGI212281[] = {{&S88143[69], 6}, {NULL}};
const Segment SGI212282[] = {{&S88144[34], 9}, {NULL}};
const Segment SGI212283[] = {{&S88144[27], 6}, {NULL}};
const Segment SGI212284[] = {{&S82704[44], 11}, {NULL}};
const Segment SGI212285[] = {{&S82704[44], 23}, {NULL}};
const Segment SGI212286[] = {{&S82704[30], 6}, {NULL}};
const Segment SGI212287[] = {{&S88144[93], 9}, {NULL}};
const Segment SGI212288[] = {{&S88144[66], 6}, {NULL}};
const Segment SGI212289[] = {{&S82704[116], 3}, {NULL}};
const Segment SGI212290[] = {{&S82704[81], 6}, {NULL}};
const Segment SGI212291[] = {{&S88145[34], 15}, {NULL}};
const Segment SGI212292[] = {{&S88145[27], 6}, {NULL}};
const Segment SGI212293[] = {{&S88145[102], 9}, {NULL}};
const Segment SGI212294[] = {{&S88145[75], 6}, {NULL}};
const Segment SGI212295[] = {{&S88147[34], 7}, {NULL}};
const Segment SGI212296[] = {{&S88147[27], 6}, {NULL}};
const Segment SGI212297[] = {{&S88147[99], 9}, {NULL}};
const Segment SGI212298[] = {{&S88147[72], 6}, {NULL}};
const Segment SGI212299[] = {{&S77651[34], 11}, {NULL}};
const Segment SGI212300[] = {{&S77651[27], 6}, {NULL}};
const Segment SGI212301[] = {{&S77651[98], 9}, {NULL}};
const Segment SGI212302[] = {{&S77651[71], 6}, {NULL}};
const Segment SGI212303[] = {{&S88161[34], 10}, {NULL}};
const Segment SGI212304[] = {{&S88161[27], 6}, {NULL}};
const Segment SGI212305[] = {{&S88161[97], 9}, {NULL}};
const Segment SGI212306[] = {{&S88161[70], 6}, {NULL}};
const Segment SGI212307[] = {{&S88161[138], 8}, {NULL}};
const Segment SGI212308[] = {{&S77980[44], 10}, {NULL}};
const Segment SGI212309[] = {{&S77980[44], 22}, {NULL}};
const Segment SGI212310[] = {{&S77980[30], 6}, {NULL}};
const Segment SGI212311[] = {{&S77980[131], 3}, {NULL}};
const Segment SGI212312[] = {{&S77980[96], 6}, {NULL}};
const Segment SGI212313[] = {{&S77980[68], 8}, {NULL}};
const Segment SGI212314[] = {{&S77982[44], 10}, {NULL}};
const Segment SGI212315[] = {{&S77982[44], 22}, {NULL}};
const Segment SGI212316[] = {{&S77982[30], 6}, {NULL}};
const Segment SGI212317[] = {{&S77982[115], 3}, {NULL}};
const Segment SGI212318[] = {{&S77982[80], 6}, {NULL}};
const Segment SGI212319[] = {{&S68929[34], 10}, {NULL}};
const Segment SGI212320[] = {{&S68929[27], 6}, {NULL}};
const Segment SGI212321[] = {{&S68929[96], 9}, {NULL}};
const Segment SGI212322[] = {{&S68929[69], 6}, {NULL}};
const Segment SGI212323[] = {{&S88192[44], 7}, {NULL}};
const Segment SGI212324[] = {{&S88192[44], 19}, {NULL}};
const Segment SGI212325[] = {{&S88192[30], 6}, {NULL}};
const Segment SGI212326[] = {{&S88192[31], 3}, {NULL}};
const Segment SGI212327[] = {{&S88192[77], 6}, {NULL}};
const Segment SGI212328[] = {{&S88194[44], 7}, {NULL}};
const Segment SGI212329[] = {{&S88194[44], 19}, {NULL}};
const Segment SGI212330[] = {{&S88194[30], 6}, {NULL}};
const Segment SGI212331[] = {{&S88194[31], 3}, {NULL}};
const Segment SGI212332[] = {{&S88194[77], 6}, {NULL}};
const Segment SGI212333[] = {{&S88195[44], 7}, {NULL}};
const Segment SGI212334[] = {{&S88195[44], 19}, {NULL}};
const Segment SGI212335[] = {{&S88195[30], 6}, {NULL}};
const Segment SGI212336[] = {{&S88195[31], 3}, {NULL}};
const Segment SGI212337[] = {{&S88195[77], 6}, {NULL}};
const Segment SGI212338[] = {{&S88196[44], 7}, {NULL}};
const Segment SGI212339[] = {{&S88196[44], 19}, {NULL}};
const Segment SGI212340[] = {{&S88196[30], 6}, {NULL}};
const Segment SGI212341[] = {{&S88196[31], 3}, {NULL}};
const Segment SGI212342[] = {{&S88196[77], 6}, {NULL}};
const Segment SGI212343[] = {{&S68949[34], 10}, {NULL}};
const Segment SGI212344[] = {{&S68949[27], 6}, {NULL}};
const Segment SGI212345[] = {{&S68949[96], 9}, {NULL}};
const Segment SGI212346[] = {{&S68949[69], 6}, {NULL}};
const Segment SGI212347[] = {{&S88200[45], 10}, {NULL}};
const Segment SGI212348[] = {{&S88200[30], 6}, {NULL}};
const Segment SGI212349[] = {{&S88200[31], 3}, {NULL}};
const Segment SGI212350[] = {{&S88200[81], 5}, {NULL}};
const Segment SGI212351[] = {{&S88200[139], 13}, {NULL}};
const Segment SGI212352[] = {{&S68954[34], 15}, {NULL}};
const Segment SGI212353[] = {{&S68954[27], 6}, {NULL}};
const Segment SGI212354[] = {{&S68954[108], 9}, {NULL}};
const Segment SGI212355[] = {{&S68954[81], 6}, {NULL}};
const Segment SGI212356[] = {{&S88213[34], 19}, {NULL}};
const Segment SGI212357[] = {{&S88213[27], 6}, {NULL}};
const Segment SGI212358[] = {{&S88213[106], 9}, {NULL}};
const Segment SGI212359[] = {{&S88213[79], 6}, {NULL}};
const Segment SGI212360[] = {{&S88218[34], 10}, {NULL}};
const Segment SGI212361[] = {{&S88218[27], 6}, {NULL}};
const Segment SGI212362[] = {{&S88218[97], 9}, {NULL}};
const Segment SGI212363[] = {{&S88218[70], 6}, {NULL}};
const Segment SGI212364[] = {{&S77715[44], 4}, {NULL}};
const Segment SGI212365[] = {{&S77715[44], 16}, {NULL}};
const Segment SGI212366[] = {{&S77715[30], 6}, {NULL}};
const Segment SGI212367[] = {{&S77715[109], 3}, {NULL}};
const Segment SGI212368[] = {{&S78026[34], 6}, {NULL}};
const Segment SGI212369[] = {{&S77715[74], 6}, {NULL}};
const Segment SGI212370[] = {{&S78026[27], 6}, {NULL}};
const Segment SGI212371[] = {{&S78026[93], 9}, {NULL}};
const Segment SGI212372[] = {{&S78026[66], 6}, {NULL}};
const Segment SGI212373[] = {{&S77716[38], 4}, {NULL}};
const Segment SGI212374[] = {{&S77716[31], 6}, {NULL}};
const Segment SGI212375[] = {{&S88221[34], 10}, {NULL}};
const Segment SGI212376[] = {{&S88221[27], 6}, {NULL}};
const Segment SGI212377[] = {{&S68974[44], 10}, {NULL}};
const Segment SGI212378[] = {{&S68974[44], 21}, {NULL}};
const Segment SGI212379[] = {{&S68974[30], 6}, {NULL}};
const Segment SGI212380[] = {{&S88221[97], 9}, {NULL}};
const Segment SGI212381[] = {{&S88221[70], 6}, {NULL}};
const Segment SGI212382[] = {{&S68974[114], 3}, {NULL}};
const Segment SGI212383[] = {{&S68974[79], 6}, {NULL}};
const Segment SGI212384[] = {{&S78029[34], 11}, {NULL}};
const Segment SGI212385[] = {{&S78029[27], 6}, {NULL}};
const Segment SGI212386[] = {{&S78029[98], 9}, {NULL}};
const Segment SGI212387[] = {{&S78029[71], 6}, {NULL}};
const Segment SGI212388[] = {{&S68975[44], 10}, {NULL}};
const Segment SGI212389[] = {{&S68975[44], 22}, {NULL}};
const Segment SGI212390[] = {{&S68975[30], 6}, {NULL}};
const Segment SGI212391[] = {{&S68975[115], 3}, {NULL}};
const Segment SGI212392[] = {{&S88223[34], 6}, {NULL}};
const Segment SGI212393[] = {{&S68975[80], 6}, {NULL}};
const Segment SGI212394[] = {{&S88223[27], 6}, {NULL}};
const Segment SGI212395[] = {{&S88223[93], 9}, {NULL}};
const Segment SGI212396[] = {{&S88223[66], 6}, {NULL}};
const Segment SGI212397[] = {{&S78030[34], 11}, {NULL}};
const Segment SGI212398[] = {{&S78030[27], 6}, {NULL}};
const Segment SGI212399[] = {{&S78030[98], 9}, {NULL}};
const Segment SGI212400[] = {{&S78030[71], 6}, {NULL}};
const Segment SGI212401[] = {{&S88224[34], 6}, {NULL}};
const Segment SGI212402[] = {{&S88224[27], 6}, {NULL}};
const Segment SGI212403[] = {{&S68977[38], 10}, {NULL}};
const Segment SGI212404[] = {{&S68977[31], 6}, {NULL}};
const Segment SGI212405[] = {{&S88224[93], 9}, {NULL}};
const Segment SGI212406[] = {{&S88224[66], 6}, {NULL}};
const Segment SGI212407[] = {{&S78032[34], 11}, {NULL}};
const Segment SGI212408[] = {{&S88225[34], 18}, {NULL}};
const Segment SGI212409[] = {{&S78032[27], 6}, {NULL}};
const Segment SGI212410[] = {{&S88225[27], 6}, {NULL}};
const Segment SGI212411[] = {{&S78032[98], 9}, {NULL}};
const Segment SGI212412[] = {{&S78032[71], 6}, {NULL}};
const Segment SGI212413[] = {{&S88225[105], 9}, {NULL}};
const Segment SGI212414[] = {{&S88225[78], 6}, {NULL}};
const Segment SGI212415[] = {{&S68980[34], 10}, {NULL}};
const Segment SGI212416[] = {{&S68980[27], 6}, {NULL}};
const Segment SGI212417[] = {{&S88226[34], 12}, {NULL}};
const Segment SGI212418[] = {{&S88226[27], 6}, {NULL}};
const Segment SGI212419[] = {{&S68980[96], 9}, {NULL}};
const Segment SGI212420[] = {{&S68980[69], 6}, {NULL}};
const Segment SGI212421[] = {{&S88226[99], 9}, {NULL}};
const Segment SGI212422[] = {{&S78034[34], 7}, {NULL}};
const Segment SGI212423[] = {{&S88226[72], 6}, {NULL}};
const Segment SGI212424[] = {{&S78034[27], 6}, {NULL}};
const Segment SGI212425[] = {{&S78034[100], 9}, {NULL}};
const Segment SGI212426[] = {{&S78034[73], 6}, {NULL}};
const Segment SGI212427[] = {{&S68981[34], 10}, {NULL}};
const Segment SGI212428[] = {{&S68981[27], 6}, {NULL}};
const Segment SGI212429[] = {{&S68981[96], 9}, {NULL}};
const Segment SGI212430[] = {{&S68981[69], 6}, {NULL}};
const Segment SGI212431[] = {{&S88230[44], 9}, {NULL}};
const Segment SGI212432[] = {{&S88230[44], 21}, {NULL}};
const Segment SGI212433[] = {{&S88230[30], 6}, {NULL}};
const Segment SGI212434[] = {{&S88230[31], 3}, {NULL}};
const Segment SGI212435[] = {{&S88230[79], 6}, {NULL}};
const Segment SGI212436[] = {{&S88239[44], 10}, {NULL}};
const Segment SGI212437[] = {{&S88239[44], 29}, {NULL}};
const Segment SGI212438[] = {{&S88239[30], 6}, {NULL}};
const Segment SGI212439[] = {{&S88239[87], 6}, {NULL}};
const Segment SGI212440[] = {{&S88239[114], 13}, {NULL}};
const Segment SGI212441[] = {{&S88242[34], 7}, {NULL}};
const Segment SGI212442[] = {{&S88242[27], 6}, {NULL}};
const Segment SGI212443[] = {{&S88242[94], 9}, {NULL}};
const Segment SGI212444[] = {{&S88242[67], 6}, {NULL}};
const Segment SGI212445[] = {{&S88244[34], 18}, {NULL}};
const Segment SGI212446[] = {{&S88244[27], 6}, {NULL}};
const Segment SGI212447[] = {{&S78054[44], 15}, {NULL}};
const Segment SGI212448[] = {{&S78054[44], 33}, {NULL}};
const Segment SGI212449[] = {{&S88244[105], 9}, {NULL}};
const Segment SGI212450[] = {{&S88244[78], 6}, {NULL}};
const Segment SGI212451[] = {{&S78054[30], 6}, {NULL}};
const Segment SGI212452[] = {{&S88244[146], 8}, {NULL}};
const Segment SGI212453[] = {{&S78054[91], 6}, {NULL}};
const Segment SGI212454[] = {{&S78054[118], 13}, {NULL}};
const Segment SGI212455[] = {{&S77758[34], 25}, {NULL}};
const Segment SGI212456[] = {{&S77758[27], 6}, {NULL}};
const Segment SGI212457[] = {{&S77758[112], 9}, {NULL}};
const Segment SGI212458[] = {{&S77758[85], 6}, {NULL}};
const Segment SGI212459[] = {{&S88253[44], 10}, {NULL}};
const Segment SGI212460[] = {{&S88253[44], 22}, {NULL}};
const Segment SGI212461[] = {{&S88253[30], 6}, {NULL}};
const Segment SGI212462[] = {{&S88253[31], 3}, {NULL}};
const Segment SGI212463[] = {{&S88253[80], 6}, {NULL}};
const Segment SGI212464[] = {{&S88258[34], 18}, {NULL}};
const Segment SGI212465[] = {{&S88258[27], 6}, {NULL}};
const Segment SGI212466[] = {{&S88258[105], 9}, {NULL}};
const Segment SGI212467[] = {{&S88258[78], 6}, {NULL}};
const Segment SGI212468[] = {{&S88258[146], 8}, {NULL}};
const Segment SGI212469[] = {{&S69016[34], 10}, {NULL}};
const Segment SGI212470[] = {{&S69016[27], 6}, {NULL}};
const Segment SGI212471[] = {{&S69016[96], 9}, {NULL}};
const Segment SGI212472[] = {{&S69016[69], 6}, {NULL}};
const Segment SGI212473[] = {{&S88261[34], 9}, {NULL}};
const Segment SGI212474[] = {{&S88261[27], 6}, {NULL}};
const Segment SGI212475[] = {{&S88261[96], 9}, {NULL}};
const Segment SGI212476[] = {{&S88261[69], 6}, {NULL}};
const Segment SGI212477[] = {{&S88261[137], 8}, {NULL}};
const Segment SGI212478[] = {{&S88262[34], 9}, {NULL}};
const Segment SGI212479[] = {{&S88262[27], 6}, {NULL}};
const Segment SGI212480[] = {{&S88262[96], 9}, {NULL}};
const Segment SGI212481[] = {{&S88262[69], 6}, {NULL}};
const Segment SGI212482[] = {{&S88262[137], 8}, {NULL}};
const Segment SGI212483[] = {{&S88265[44], 7}, {NULL}};
const Segment SGI212484[] = {{&S88265[44], 22}, {NULL}};
const Segment SGI212485[] = {{&S88265[30], 6}, {NULL}};
const Segment SGI212486[] = {{&S88265[80], 6}, {NULL}};
const Segment SGI212487[] = {{&S88265[107], 13}, {NULL}};
const Segment SGI212488[] = {{&S88266[44], 7}, {NULL}};
const Segment SGI212489[] = {{&S88266[44], 23}, {NULL}};
const Segment SGI212490[] = {{&S88266[30], 6}, {NULL}};
const Segment SGI212491[] = {{&S88266[81], 6}, {NULL}};
const Segment SGI212492[] = {{&S88266[108], 13}, {NULL}};
const Segment SGI212493[] = {{&S82848[44], 17}, {NULL}};
const Segment SGI212494[] = {{&S82848[44], 29}, {NULL}};
const Segment SGI212495[] = {{&S82848[30], 6}, {NULL}};
const Segment SGI212496[] = {{&S82848[122], 3}, {NULL}};
const Segment SGI212497[] = {{&S82848[87], 6}, {NULL}};
const Segment SGI212498[] = {{&S88274[27], 6}, {NULL}};
const Segment SGI212499[] = {{&S88274[74], 9}, {NULL}};
const Segment SGI212500[] = {{&S88274[47], 6}, {NULL}};
const Segment SGI212501[] = {{&S88281[34], 7}, {NULL}};
const Segment SGI212502[] = {{&S88281[27], 6}, {NULL}};
const Segment SGI212503[] = {{&S88281[94], 9}, {NULL}};
const Segment SGI212504[] = {{&S88281[67], 6}, {NULL}};
const Segment SGI212505[] = {{&S88310[34], 9}, {NULL}};
const Segment SGI212506[] = {{&S88310[27], 6}, {NULL}};
const Segment SGI212507[] = {{&S88310[95], 9}, {NULL}};
const Segment SGI212508[] = {{&S88310[68], 6}, {NULL}};
const Segment SGI212509[] = {{&S88320[44], 10}, {NULL}};
const Segment SGI212510[] = {{&S88320[44], 22}, {NULL}};
const Segment SGI212511[] = {{&S88320[30], 6}, {NULL}};
const Segment SGI212512[] = {{&S88320[31], 3}, {NULL}};
const Segment SGI212513[] = {{&S88320[80], 6}, {NULL}};
const Segment SGI212514[] = {{&S88321[38], 10}, {NULL}};
const Segment SGI212515[] = {{&S88321[31], 6}, {NULL}};
const Segment SGI212516[] = {{&S69091[44], 7}, {NULL}};
const Segment SGI212517[] = {{&S69091[44], 18}, {NULL}};
const Segment SGI212518[] = {{&S69091[30], 6}, {NULL}};
const Segment SGI212519[] = {{&S69091[129], 3}, {NULL}};
const Segment SGI212520[] = {{&S69091[94], 6}, {NULL}};
const Segment SGI212521[] = {{&S69091[64], 8}, {NULL}};
const Segment SGI212522[] = {{&S88323[34], 14}, {NULL}};
const Segment SGI212523[] = {{&S88323[27], 6}, {NULL}};
const Segment SGI212524[] = {{&S88323[117], 9}, {NULL}};
const Segment SGI212525[] = {{&S88323[90], 6}, {NULL}};
const Segment SGI212526[] = {{&S69093[44], 7}, {NULL}};
const Segment SGI212527[] = {{&S69093[44], 19}, {NULL}};
const Segment SGI212528[] = {{&S69093[30], 6}, {NULL}};
const Segment SGI212529[] = {{&S69093[112], 3}, {NULL}};
const Segment SGI212530[] = {{&S69093[77], 6}, {NULL}};
const Segment SGI212531[] = {{&S69093[137], 8}, {NULL}};
const Segment SGI212532[] = {{&S69094[44], 11}, {NULL}};
const Segment SGI212533[] = {{&S69094[44], 22}, {NULL}};
const Segment SGI212534[] = {{&S69094[30], 6}, {NULL}};
const Segment SGI212535[] = {{&S69094[115], 3}, {NULL}};
const Segment SGI212536[] = {{&S69094[80], 6}, {NULL}};
const Segment SGI212537[] = {{&S69095[38], 11}, {NULL}};
const Segment SGI212538[] = {{&S69095[31], 6}, {NULL}};
const Segment SGI212539[] = {{&S69096[44], 10}, {NULL}};
const Segment SGI212540[] = {{&S69096[44], 21}, {NULL}};
const Segment SGI212541[] = {{&S69096[30], 6}, {NULL}};
const Segment SGI212542[] = {{&S69096[114], 3}, {NULL}};
const Segment SGI212543[] = {{&S69096[79], 6}, {NULL}};
const Segment SGI212544[] = {{&S88330[34], 12}, {NULL}};
const Segment SGI212545[] = {{&S88330[27], 6}, {NULL}};
const Segment SGI212546[] = {{&S69097[38], 10}, {NULL}};
const Segment SGI212547[] = {{&S69097[31], 6}, {NULL}};
const Segment SGI212548[] = {{&S88330[99], 9}, {NULL}};
const Segment SGI212549[] = {{&S88330[72], 6}, {NULL}};
const Segment SGI212550[] = {{&S88331[34], 14}, {NULL}};
const Segment SGI212551[] = {{&S88331[27], 6}, {NULL}};
const Segment SGI212552[] = {{&S88331[101], 9}, {NULL}};
const Segment SGI212553[] = {{&S88331[74], 6}, {NULL}};
const Segment SGI212554[] = {{&S82920[44], 19}, {NULL}};
const Segment SGI212555[] = {{&S82920[44], 31}, {NULL}};
const Segment SGI212556[] = {{&S82920[30], 6}, {NULL}};
const Segment SGI212557[] = {{&S82920[124], 3}, {NULL}};
const Segment SGI212558[] = {{&S82920[89], 6}, {NULL}};
const Segment SGI212559[] = {{&S77859[34], 11}, {NULL}};
const Segment SGI212560[] = {{&S77859[27], 6}, {NULL}};
const Segment SGI212561[] = {{&S77859[98], 9}, {NULL}};
const Segment SGI212562[] = {{&S77859[71], 6}, {NULL}};
const Segment SGI212563[] = {{&S88342[34], 7}, {NULL}};
const Segment SGI212564[] = {{&S88342[27], 6}, {NULL}};
const Segment SGI212565[] = {{&S88342[94], 9}, {NULL}};
const Segment SGI212566[] = {{&S88342[67], 6}, {NULL}};
const Segment SGI212567[] = {{&S88343[44], 8}, {NULL}};
const Segment SGI212568[] = {{&S88343[44], 20}, {NULL}};
const Segment SGI212569[] = {{&S88343[30], 6}, {NULL}};
const Segment SGI212570[] = {{&S88343[31], 3}, {NULL}};
const Segment SGI212571[] = {{&S88343[78], 6}, {NULL}};
const Segment SGI212572[] = {{&S88344[45], 7}, {NULL}};
const Segment SGI212573[] = {{&S88344[30], 6}, {NULL}};
const Segment SGI212574[] = {{&S88344[31], 3}, {NULL}};
const Segment SGI212575[] = {{&S88344[81], 5}, {NULL}};
const Segment SGI212576[] = {{&S88344[139], 13}, {NULL}};
const Segment SGI212577[] = {{&S88346[34], 11}, {NULL}};
const Segment SGI212578[] = {{&S88346[27], 6}, {NULL}};
const Segment SGI212579[] = {{&S69118[44], 10}, {NULL}};
const Segment SGI212580[] = {{&S69118[44], 21}, {NULL}};
const Segment SGI212581[] = {{&S88346[98], 9}, {NULL}};
const Segment SGI212582[] = {{&S69118[30], 6}, {NULL}};
const Segment SGI212583[] = {{&S88346[71], 6}, {NULL}};
const Segment SGI212584[] = {{&S69118[114], 3}, {NULL}};
const Segment SGI212585[] = {{&S69118[79], 6}, {NULL}};
const Segment SGI212586[] = {{&S88347[34], 15}, {NULL}};
const Segment SGI212587[] = {{&S88347[27], 6}, {NULL}};
const Segment SGI212588[] = {{&S69119[44], 10}, {NULL}};
const Segment SGI212589[] = {{&S69119[44], 21}, {NULL}};
const Segment SGI212590[] = {{&S82937[44], 18}, {NULL}};
const Segment SGI212591[] = {{&S69119[30], 6}, {NULL}};
const Segment SGI212592[] = {{&S88347[102], 9}, {NULL}};
const Segment SGI212593[] = {{&S82937[44], 30}, {NULL}};
const Segment SGI212594[] = {{&S88347[75], 6}, {NULL}};
const Segment SGI212595[] = {{&S82937[30], 6}, {NULL}};
const Segment SGI212596[] = {{&S69119[114], 3}, {NULL}};
const Segment SGI212597[] = {{&S69119[79], 6}, {NULL}};
const Segment SGI212598[] = {{&S82937[123], 3}, {NULL}};
const Segment SGI212599[] = {{&S82937[88], 6}, {NULL}};
const Segment SGI212600[] = {{&S69120[44], 18}, {NULL}};
const Segment SGI212601[] = {{&S69120[44], 29}, {NULL}};
const Segment SGI212602[] = {{&S69120[30], 6}, {NULL}};
const Segment SGI212603[] = {{&S69120[122], 3}, {NULL}};
const Segment SGI212604[] = {{&S69120[87], 6}, {NULL}};
const Segment SGI212605[] = {{&S69121[38], 10}, {NULL}};
const Segment SGI212606[] = {{&S69121[31], 6}, {NULL}};
const Segment SGI212607[] = {{&S88351[44], 15}, {NULL}};
const Segment SGI212608[] = {{&S88351[44], 42}, {NULL}};
const Segment SGI212609[] = {{&S88351[30], 6}, {NULL}};
const Segment SGI212610[] = {{&S88351[100], 6}, {NULL}};
const Segment SGI212611[] = {{&S88351[127], 13}, {NULL}};
const Segment SGI212612[] = {{&S88353[44], 10}, {NULL}};
const Segment SGI212613[] = {{&S88353[44], 22}, {NULL}};
const Segment SGI212614[] = {{&S88353[30], 6}, {NULL}};
const Segment SGI212615[] = {{&S88353[31], 3}, {NULL}};
const Segment SGI212616[] = {{&S88353[80], 5}, {NULL}};
const Segment SGI212617[] = {{&S88353[117], 14}, {NULL}};
const Segment SGI212618[] = {{&S88371[34], 6}, {NULL}};
const Segment SGI212619[] = {{&S88371[27], 6}, {NULL}};
const Segment SGI212620[] = {{&S88371[93], 9}, {NULL}};
const Segment SGI212621[] = {{&S88371[66], 6}, {NULL}};
const Segment SGI212622[] = {{&S88372[34], 6}, {NULL}};
const Segment SGI212623[] = {{&S88372[27], 6}, {NULL}};
const Segment SGI212624[] = {{&S88372[93], 9}, {NULL}};
const Segment SGI212625[] = {{&S88372[66], 6}, {NULL}};
const Segment SGI212626[] = {{&S88376[34], 12}, {NULL}};
const Segment SGI212627[] = {{&S88376[27], 6}, {NULL}};
const Segment SGI212628[] = {{&S88376[109], 9}, {NULL}};
const Segment SGI212629[] = {{&S88376[82], 6}, {NULL}};
const Segment SGI212630[] = {{&S88382[44], 18}, {NULL}};
const Segment SGI212631[] = {{&S88382[44], 40}, {NULL}};
const Segment SGI212632[] = {{&S88382[30], 6}, {NULL}};
const Segment SGI212633[] = {{&S88382[31], 3}, {NULL}};
const Segment SGI212634[] = {{&S88382[98], 6}, {NULL}};
const Segment SGI212635[] = {{&S88386[34], 14}, {NULL}};
const Segment SGI212636[] = {{&S88386[27], 6}, {NULL}};
const Segment SGI212637[] = {{&S88386[101], 9}, {NULL}};
const Segment SGI212638[] = {{&S88386[74], 6}, {NULL}};
const Segment SGI212639[] = {{&S88389[34], 8}, {NULL}};
const Segment SGI212640[] = {{&S88389[27], 6}, {NULL}};
const Segment SGI212641[] = {{&S88389[95], 9}, {NULL}};
const Segment SGI212642[] = {{&S88389[68], 6}, {NULL}};
const Segment SGI212643[] = {{&S82994[44], 9}, {NULL}};
const Segment SGI212644[] = {{&S82994[44], 21}, {NULL}};
const Segment SGI212645[] = {{&S82994[30], 6}, {NULL}};
const Segment SGI212646[] = {{&S82994[114], 3}, {NULL}};
const Segment SGI212647[] = {{&S82994[79], 6}, {NULL}};
const Segment SGI212648[] = {{&S83001[34], 6}, {NULL}};
const Segment SGI212649[] = {{&S69179[34], 9}, {NULL}};
const Segment SGI212650[] = {{&S83001[27], 6}, {NULL}};
const Segment SGI212651[] = {{&S69179[27], 6}, {NULL}};
const Segment SGI212652[] = {{&S83001[93], 9}, {NULL}};
const Segment SGI212653[] = {{&S83001[66], 6}, {NULL}};
const Segment SGI212654[] = {{&S69179[95], 9}, {NULL}};
const Segment SGI212655[] = {{&S69179[68], 6}, {NULL}};
const Segment SGI212656[] = {{&S83005[34], 14}, {NULL}};
const Segment SGI212657[] = {{&S83005[27], 6}, {NULL}};
const Segment SGI212658[] = {{&S83005[101], 9}, {NULL}};
const Segment SGI212659[] = {{&S83005[74], 6}, {NULL}};
const Segment SGI212660[] = {{&S69183[34], 10}, {NULL}};
const Segment SGI212661[] = {{&S69183[27], 6}, {NULL}};
const Segment SGI212662[] = {{&S69183[96], 9}, {NULL}};
const Segment SGI212663[] = {{&S69183[69], 6}, {NULL}};
const Segment SGI212664[] = {{&S69185[34], 17}, {NULL}};
const Segment SGI212665[] = {{&S69185[27], 6}, {NULL}};
const Segment SGI212666[] = {{&S69185[103], 9}, {NULL}};
const Segment SGI212667[] = {{&S69185[76], 6}, {NULL}};
const Segment SGI212668[] = {{&S69198[34], 10}, {NULL}};
const Segment SGI212669[] = {{&S69198[27], 6}, {NULL}};
const Segment SGI212670[] = {{&S69198[96], 9}, {NULL}};
const Segment SGI212671[] = {{&S69198[69], 6}, {NULL}};
const Segment SGI212672[] = {{&S83028[59], 18}, {NULL}};
const Segment SGI212673[] = {{&S83028[59], 39}, {NULL}};
const Segment SGI212674[] = {{&S83028[45], 6}, {NULL}};
const Segment SGI212675[] = {{&S83028[147], 3}, {NULL}};
const Segment SGI212676[] = {{&S83028[112], 6}, {NULL}};
const Segment SGI212677[] = {{&S83028[0], 14}, {NULL}};
const Segment SGI212678[] = {{&S69229[44], 10}, {NULL}};
const Segment SGI212679[] = {{&S69229[44], 22}, {NULL}};
const Segment SGI212680[] = {{&S69229[30], 6}, {NULL}};
const Segment SGI212681[] = {{&S88459[44], 12}, {NULL}};
const Segment SGI212682[] = {{&S88459[30], 6}, {NULL}};
const Segment SGI212683[] = {{&S69229[135], 3}, {NULL}};
const Segment SGI212684[] = {{&S69229[100], 6}, {NULL}};
const Segment SGI212685[] = {{&S88459[31], 3}, {NULL}};
const Segment SGI212686[] = {{&S69229[68], 8}, {NULL}};
const Segment SGI212687[] = {{&S88459[84], 6}, {NULL}};
const Segment SGI212688[] = {{&S69230[44], 10}, {NULL}};
const Segment SGI212689[] = {{&S69230[44], 21}, {NULL}};
const Segment SGI212690[] = {{&S69230[30], 6}, {NULL}};
const Segment SGI212691[] = {{&S88460[44], 7}, {NULL}};
const Segment SGI212692[] = {{&S88460[44], 19}, {NULL}};
const Segment SGI212693[] = {{&S69230[132], 3}, {NULL}};
const Segment SGI212694[] = {{&S88460[30], 6}, {NULL}};
const Segment SGI212695[] = {{&S69230[97], 6}, {NULL}};
const Segment SGI212696[] = {{&S69230[67], 8}, {NULL}};
const Segment SGI212697[] = {{&S88460[108], 2}, {NULL}};
const Segment SGI212698[] = {{&S88460[77], 5}, {NULL}};
const Segment SGI212699[] = {{&S69231[44], 13}, {NULL}};
const Segment SGI212700[] = {{&S69231[44], 34}, {NULL}};
const Segment SGI212701[] = {{&S69231[30], 6}, {NULL}};
const Segment SGI212702[] = {{&S69231[145], 3}, {NULL}};
const Segment SGI212703[] = {{&S69231[110], 6}, {NULL}};
const Segment SGI212704[] = {{&S69231[80], 8}, {NULL}};
const Segment SGI212705[] = {{&S88468[34], 7}, {NULL}};
const Segment SGI212706[] = {{&S88468[27], 6}, {NULL}};
const Segment SGI212707[] = {{&S88468[98], 9}, {NULL}};
const Segment SGI212708[] = {{&S88468[71], 6}, {NULL}};
const Segment SGI212709[] = {{&S88469[34], 10}, {NULL}};
const Segment SGI212710[] = {{&S88469[27], 6}, {NULL}};
const Segment SGI212711[] = {{&S88469[97], 9}, {NULL}};
const Segment SGI212712[] = {{&S88469[70], 6}, {NULL}};
const Segment SGI212713[] = {{&S69250[44], 10}, {NULL}};
const Segment SGI212714[] = {{&S69250[44], 21}, {NULL}};
const Segment SGI212715[] = {{&S69250[30], 6}, {NULL}};
const Segment SGI212716[] = {{&S69250[114], 3}, {NULL}};
const Segment SGI212717[] = {{&S69250[79], 6}, {NULL}};
const Segment SGI212718[] = {{&S69250[149], 3}, {NULL}};
const Segment SGI212719[] = {{&S88481[44], 18}, {NULL}};
const Segment SGI212720[] = {{&S88481[44], 30}, {NULL}};
const Segment SGI212721[] = {{&S88481[30], 6}, {NULL}};
const Segment SGI212722[] = {{&S88481[88], 6}, {NULL}};
const Segment SGI212723[] = {{&S88481[115], 13}, {NULL}};
const Segment SGI212724[] = {{&S88482[34], 7}, {NULL}};
const Segment SGI212725[] = {{&S88482[27], 6}, {NULL}};
const Segment SGI212726[] = {{&S88482[94], 9}, {NULL}};
const Segment SGI212727[] = {{&S88482[67], 6}, {NULL}};
const Segment SGI212728[] = {{&S88483[34], 9}, {NULL}};
const Segment SGI212729[] = {{&S88483[27], 6}, {NULL}};
const Segment SGI212730[] = {{&S88483[96], 9}, {NULL}};
const Segment SGI212731[] = {{&S88483[69], 6}, {NULL}};
const Segment SGI212732[] = {{&S88487[44], 17}, {NULL}};
const Segment SGI212733[] = {{&S88487[44], 29}, {NULL}};
const Segment SGI212734[] = {{&S88487[30], 6}, {NULL}};
const Segment SGI212735[] = {{&S88487[31], 3}, {NULL}};
const Segment SGI212736[] = {{&S88487[87], 6}, {NULL}};
const Segment SGI212737[] = {{&S83093[34], 13}, {NULL}};
const Segment SGI212738[] = {{&S83093[27], 6}, {NULL}};
const Segment SGI212739[] = {{&S83093[114], 9}, {NULL}};
const Segment SGI212740[] = {{&S83093[87], 6}, {NULL}};
const Segment SGI212741[] = {{&S83094[34], 12}, {NULL}};
const Segment SGI212742[] = {{&S83094[27], 6}, {NULL}};
const Segment SGI212743[] = {{&S88489[34], 7}, {NULL}};
const Segment SGI212744[] = {{&S88489[27], 6}, {NULL}};
const Segment SGI212745[] = {{&S83094[112], 9}, {NULL}};
const Segment SGI212746[] = {{&S83094[85], 6}, {NULL}};
const Segment SGI212747[] = {{&S88489[99], 9}, {NULL}};
const Segment SGI212748[] = {{&S88489[72], 6}, {NULL}};
const Segment SGI212749[] = {{&S69266[44], 18}, {NULL}};
const Segment SGI212750[] = {{&S69266[44], 29}, {NULL}};
const Segment SGI212751[] = {{&S69266[30], 6}, {NULL}};
const Segment SGI212752[] = {{&S69266[122], 3}, {NULL}};
const Segment SGI212753[] = {{&S69266[87], 6}, {NULL}};
const Segment SGI212754[] = {{&S69267[38], 18}, {NULL}};
const Segment SGI212755[] = {{&S69267[31], 6}, {NULL}};
const Segment SGI212756[] = {{&S69268[44], 11}, {NULL}};
const Segment SGI212757[] = {{&S69268[44], 22}, {NULL}};
const Segment SGI212758[] = {{&S69268[30], 6}, {NULL}};
const Segment SGI212759[] = {{&S69268[115], 3}, {NULL}};
const Segment SGI212760[] = {{&S69268[80], 6}, {NULL}};
const Segment SGI212761[] = {{&S78046[34], 7}, {NULL}};
const Segment SGI212762[] = {{&S78046[27], 6}, {NULL}};
const Segment SGI212763[] = {{&S78046[94], 9}, {NULL}};
const Segment SGI212764[] = {{&S78046[67], 6}, {NULL}};
const Segment SGI212765[] = {{&S88497[44], 11}, {NULL}};
const Segment SGI212766[] = {{&S88497[44], 23}, {NULL}};
const Segment SGI212767[] = {{&S88497[30], 6}, {NULL}};
const Segment SGI212768[] = {{&S88497[31], 3}, {NULL}};
const Segment SGI212769[] = {{&S88497[81], 6}, {NULL}};
const Segment SGI212770[] = {{&S69276[34], 10}, {NULL}};
const Segment SGI212771[] = {{&S69276[27], 6}, {NULL}};
const Segment SGI212772[] = {{&S69276[96], 9}, {NULL}};
const Segment SGI212773[] = {{&S69276[69], 6}, {NULL}};
const Segment SGI212774[] = {{&S69279[34], 7}, {NULL}};
const Segment SGI212775[] = {{&S69279[27], 6}, {NULL}};
const Segment SGI212776[] = {{&S69279[100], 9}, {NULL}};
const Segment SGI212777[] = {{&S69279[73], 6}, {NULL}};
const Segment SGI212778[] = {{&S69280[34], 7}, {NULL}};
const Segment SGI212779[] = {{&S69280[27], 6}, {NULL}};
const Segment SGI212780[] = {{&S69280[100], 9}, {NULL}};
const Segment SGI212781[] = {{&S69280[73], 6}, {NULL}};
const Segment SGI212782[] = {{&S78060[34], 8}, {NULL}};
const Segment SGI212783[] = {{&S78060[27], 6}, {NULL}};
const Segment SGI212784[] = {{&S78060[95], 9}, {NULL}};
const Segment SGI212785[] = {{&S78060[68], 6}, {NULL}};
const Segment SGI212786[] = {{&S69309[44], 9}, {NULL}};
const Segment SGI212787[] = {{&S69309[44], 20}, {NULL}};
const Segment SGI212788[] = {{&S69309[30], 6}, {NULL}};
const Segment SGI212789[] = {{&S78103[44], 10}, {NULL}};
const Segment SGI212790[] = {{&S69309[113], 3}, {NULL}};
const Segment SGI212791[] = {{&S78103[44], 22}, {NULL}};
const Segment SGI212792[] = {{&S78103[30], 6}, {NULL}};
const Segment SGI212793[] = {{&S69309[78], 6}, {NULL}};
const Segment SGI212794[] = {{&S78103[80], 6}, {NULL}};
const Segment SGI212795[] = {{&S78103[107], 13}, {NULL}};
const Segment SGI212796[] = {{&S69322[34], 8}, {NULL}};
const Segment SGI212797[] = {{&S69322[27], 6}, {NULL}};
const Segment SGI212798[] = {{&S69322[103], 9}, {NULL}};
const Segment SGI212799[] = {{&S69322[76], 6}, {NULL}};
const Segment SGI212800[] = {{&S83171[34], 10}, {NULL}};
const Segment SGI212801[] = {{&S83171[27], 6}, {NULL}};
const Segment SGI212802[] = {{&S83171[97], 9}, {NULL}};
const Segment SGI212803[] = {{&S83171[70], 6}, {NULL}};
const Segment SGI212804[] = {{&S83173[34], 6}, {NULL}};
const Segment SGI212805[] = {{&S83173[27], 6}, {NULL}};
const Segment SGI212806[] = {{&S83173[93], 9}, {NULL}};
const Segment SGI212807[] = {{&S83173[66], 6}, {NULL}};
const Segment SGI212808[] = {{&S78138[44], 16}, {NULL}};
const Segment SGI212809[] = {{&S78138[44], 28}, {NULL}};
const Segment SGI212810[] = {{&S78138[30], 6}, {NULL}};
const Segment SGI212811[] = {{&S78138[121], 3}, {NULL}};
const Segment SGI212812[] = {{&S78138[86], 6}, {NULL}};
const Segment SGI212813[] = {{&S69342[34], 10}, {NULL}};
const Segment SGI212814[] = {{&S69342[27], 6}, {NULL}};
const Segment SGI212815[] = {{&S69342[97], 9}, {NULL}};
const Segment SGI212816[] = {{&S69342[70], 6}, {NULL}};
const Segment SGI212817[] = {{&S78141[43], 10}, {NULL}};
const Segment SGI212818[] = {{&S78141[43], 22}, {NULL}};
const Segment SGI212819[] = {{&S78141[30], 6}, {NULL}};
const Segment SGI212820[] = {{&S78141[114], 3}, {NULL}};
const Segment SGI212821[] = {{&S78141[79], 6}, {NULL}};
const Segment SGI212822[] = {{&S83183[44], 5}, {NULL}};
const Segment SGI212823[] = {{&S83183[44], 17}, {NULL}};
const Segment SGI212824[] = {{&S83183[30], 6}, {NULL}};
const Segment SGI212825[] = {{&S83183[110], 3}, {NULL}};
const Segment SGI212826[] = {{&S83183[75], 6}, {NULL}};
const Segment SGI212827[] = {{&S83183[135], 7}, {NULL}};
const Segment SGI212828[] = {{&S78148[44], 10}, {NULL}};
const Segment SGI212829[] = {{&S78148[44], 22}, {NULL}};
const Segment SGI212830[] = {{&S78148[30], 6}, {NULL}};
const Segment SGI212831[] = {{&S78148[80], 6}, {NULL}};
const Segment SGI212832[] = {{&S78148[107], 13}, {NULL}};
const Segment SGI212833[] = {{&S69352[44], 10}, {NULL}};
const Segment SGI212834[] = {{&S69352[44], 27}, {NULL}};
const Segment SGI212835[] = {{&S69352[30], 6}, {NULL}};
const Segment SGI212836[] = {{&S69352[119], 3}, {NULL}};
const Segment SGI212837[] = {{&S69352[85], 5}, {NULL}};
const Segment SGI212838[] = {{&S78150[34], 10}, {NULL}};
const Segment SGI212839[] = {{&S78150[27], 6}, {NULL}};
const Segment SGI212840[] = {{&S78150[97], 9}, {NULL}};
const Segment SGI212841[] = {{&S78150[70], 6}, {NULL}};
const Segment SGI212842[] = {{&S83188[34], 7}, {NULL}};
const Segment SGI212843[] = {{&S83188[27], 6}, {NULL}};
const Segment SGI212844[] = {{&S83188[110], 9}, {NULL}};
const Segment SGI212845[] = {{&S83188[83], 6}, {NULL}};
const Segment SGI212846[] = {{&S83188[151], 8}, {NULL}};
const Segment SGI212847[] = {{&S69356[38], 11}, {NULL}};
const Segment SGI212848[] = {{&S69356[31], 6}, {NULL}};
const Segment SGI212849[] = {{&S78153[44], 13}, {NULL}};
const Segment SGI212850[] = {{&S78153[44], 25}, {NULL}};
const Segment SGI212851[] = {{&S78153[30], 6}, {NULL}};
const Segment SGI212852[] = {{&S69357[44], 11}, {NULL}};
const Segment SGI212853[] = {{&S69357[44], 22}, {NULL}};
const Segment SGI212854[] = {{&S78153[83], 6}, {NULL}};
const Segment SGI212855[] = {{&S69357[30], 6}, {NULL}};
const Segment SGI212856[] = {{&S78153[110], 13}, {NULL}};
const Segment SGI212857[] = {{&S69357[115], 3}, {NULL}};
const Segment SGI212858[] = {{&S69357[80], 6}, {NULL}};
const Segment SGI212859[] = {{&S83225[34], 13}, {NULL}};
const Segment SGI212860[] = {{&S83225[27], 6}, {NULL}};
const Segment SGI212861[] = {{&S83225[100], 9}, {NULL}};
const Segment SGI212862[] = {{&S83225[73], 6}, {NULL}};
const Segment SGI212863[] = {{&S69403[34], 10}, {NULL}};
const Segment SGI212864[] = {{&S69403[27], 6}, {NULL}};
const Segment SGI212865[] = {{&S69403[96], 9}, {NULL}};
const Segment SGI212866[] = {{&S69403[69], 6}, {NULL}};
const Segment SGI212867[] = {{&S78197[44], 24}, {NULL}};
const Segment SGI212868[] = {{&S78197[44], 36}, {NULL}};
const Segment SGI212869[] = {{&S78197[30], 6}, {NULL}};
const Segment SGI212870[] = {{&S78197[147], 3}, {NULL}};
const Segment SGI212871[] = {{&S78197[112], 6}, {NULL}};
const Segment SGI212872[] = {{&S78197[82], 8}, {NULL}};
const Segment SGI212873[] = {{&S69405[34], 8}, {NULL}};
const Segment SGI212874[] = {{&S69405[27], 6}, {NULL}};
const Segment SGI212875[] = {{&S69405[103], 9}, {NULL}};
const Segment SGI212876[] = {{&S69405[76], 6}, {NULL}};
const Segment SGI212877[] = {{&S83239[6], 4}, {NULL}};
const Segment SGI212878[] = {{&S83239[81], 5}, {NULL}};
const Segment SGI212879[] = {{&S78206[44], 10}, {NULL}};
const Segment SGI212880[] = {{&S78206[44], 22}, {NULL}};
const Segment SGI212881[] = {{&S78206[30], 6}, {NULL}};
const Segment SGI212882[] = {{&S78206[133], 3}, {NULL}};
const Segment SGI212883[] = {{&S78206[98], 6}, {NULL}};
const Segment SGI212884[] = {{&S78206[68], 8}, {NULL}};
const Segment SGI212885[] = {{&S83246[34], 25}, {NULL}};
const Segment SGI212886[] = {{&S83246[27], 6}, {NULL}};
const Segment SGI212887[] = {{&S78213[34], 6}, {NULL}};
const Segment SGI212888[] = {{&S78213[27], 6}, {NULL}};
const Segment SGI212889[] = {{&S83246[112], 9}, {NULL}};
const Segment SGI212890[] = {{&S83246[85], 6}, {NULL}};
const Segment SGI212891[] = {{&S78213[93], 9}, {NULL}};
const Segment SGI212892[] = {{&S78213[66], 6}, {NULL}};
const Segment SGI212893[] = {{&S78208[44], 10}, {NULL}};
const Segment SGI212894[] = {{&S78208[44], 22}, {NULL}};
const Segment SGI212895[] = {{&S78208[30], 6}, {NULL}};
const Segment SGI212896[] = {{&S78208[133], 3}, {NULL}};
const Segment SGI212897[] = {{&S78208[98], 6}, {NULL}};
const Segment SGI212898[] = {{&S78208[68], 8}, {NULL}};
const Segment SGI212899[] = {{&S88497[151], 12}, {NULL}};
const Segment SGI212900[] = {{&S83252[34], 5}, {NULL}};
const Segment SGI212901[] = {{&S83252[27], 6}, {NULL}};
const Segment SGI212902[] = {{&S83252[92], 9}, {NULL}};
const Segment SGI212903[] = {{&S83252[65], 6}, {NULL}};
const Segment SGI212904[] = {{&S88503[34], 13}, {NULL}};
const Segment SGI212905[] = {{&S88503[27], 6}, {NULL}};
const Segment SGI212906[] = {{&S88503[100], 9}, {NULL}};
const Segment SGI212907[] = {{&S88503[73], 6}, {NULL}};
const Segment SGI212908[] = {{&S88506[34], 15}, {NULL}};
const Segment SGI212909[] = {{&S88506[27], 6}, {NULL}};
const Segment SGI212910[] = {{&S88506[102], 9}, {NULL}};
const Segment SGI212911[] = {{&S88506[75], 6}, {NULL}};
const Segment SGI212912[] = {{&S88511[34], 13}, {NULL}};
const Segment SGI212913[] = {{&S88511[27], 6}, {NULL}};
const Segment SGI212914[] = {{&S88511[100], 9}, {NULL}};
const Segment SGI212915[] = {{&S88511[73], 6}, {NULL}};
const Segment SGI212916[] = {{&S88513[34], 7}, {NULL}};
const Segment SGI212917[] = {{&S88513[27], 6}, {NULL}};
const Segment SGI212918[] = {{&S88513[99], 9}, {NULL}};
const Segment SGI212919[] = {{&S88513[72], 6}, {NULL}};
const Segment SGI212920[] = {{&S88514[34], 7}, {NULL}};
const Segment SGI212921[] = {{&S88514[27], 6}, {NULL}};
const Segment SGI212922[] = {{&S88514[100], 9}, {NULL}};
const Segment SGI212923[] = {{&S88514[73], 6}, {NULL}};
const Segment SGI212924[] = {{&S88514[141], 8}, {NULL}};
const Segment SGI212925[] = {{&S88516[34], 18}, {NULL}};
const Segment SGI212926[] = {{&S88516[27], 6}, {NULL}};
const Segment SGI212927[] = {{&S78241[44], 9}, {NULL}};
const Segment SGI212928[] = {{&S78241[44], 21}, {NULL}};
const Segment SGI212929[] = {{&S78241[30], 6}, {NULL}};
const Segment SGI212930[] = {{&S88516[105], 9}, {NULL}};
const Segment SGI212931[] = {{&S88516[78], 6}, {NULL}};
const Segment SGI212932[] = {{&S78241[130], 3}, {NULL}};
const Segment SGI212933[] = {{&S78241[95], 6}, {NULL}};
const Segment SGI212934[] = {{&S88516[146], 8}, {NULL}};
const Segment SGI212935[] = {{&S78241[67], 8}, {NULL}};
const Segment SGI212936[] = {{&S88517[44], 8}, {NULL}};
const Segment SGI212937[] = {{&S88517[44], 20}, {NULL}};
const Segment SGI212938[] = {{&S88517[30], 6}, {NULL}};
const Segment SGI212939[] = {{&S88517[31], 3}, {NULL}};
const Segment SGI212940[] = {{&S88517[78], 6}, {NULL}};
const Segment SGI212941[] = {{&S78244[34], 10}, {NULL}};
const Segment SGI212942[] = {{&S78244[27], 6}, {NULL}};
const Segment SGI212943[] = {{&S78244[97], 9}, {NULL}};
const Segment SGI212944[] = {{&S78244[70], 6}, {NULL}};
const Segment SGI212945[] = {{&S88520[44], 12}, {NULL}};
const Segment SGI212946[] = {{&S88520[44], 24}, {NULL}};
const Segment SGI212947[] = {{&S88520[30], 6}, {NULL}};
const Segment SGI212948[] = {{&S88520[82], 6}, {NULL}};
const Segment SGI212949[] = {{&S88520[109], 13}, {NULL}};
const Segment SGI212950[] = {{&S78249[45], 10}, {NULL}};
const Segment SGI212951[] = {{&S88521[34], 16}, {NULL}};
const Segment SGI212952[] = {{&S78249[30], 6}, {NULL}};
const Segment SGI212953[] = {{&S88521[27], 6}, {NULL}};
const Segment SGI212954[] = {{&S78249[115], 3}, {NULL}};
const Segment SGI212955[] = {{&S78249[81], 5}, {NULL}};
const Segment SGI212956[] = {{&S88521[116], 9}, {NULL}};
const Segment SGI212957[] = {{&S88521[89], 6}, {NULL}};
const Segment SGI212958[] = {{&S78249[139], 13}, {NULL}};
const Segment SGI212959[] = {{&S88522[34], 19}, {NULL}};
const Segment SGI212960[] = {{&S88522[27], 6}, {NULL}};
const Segment SGI212961[] = {{&S88522[119], 9}, {NULL}};
const Segment SGI212962[] = {{&S88522[92], 6}, {NULL}};
const Segment SGI212963[] = {{&S88527[44], 7}, {NULL}};
const Segment SGI212964[] = {{&S88527[44], 31}, {NULL}};
const Segment SGI212965[] = {{&S88527[30], 6}, {NULL}};
const Segment SGI212966[] = {{&S78256[34], 7}, {NULL}};
const Segment SGI212967[] = {{&S88527[31], 3}, {NULL}};
const Segment SGI212968[] = {{&S78256[27], 6}, {NULL}};
const Segment SGI212969[] = {{&S88527[89], 6}, {NULL}};
const Segment SGI212970[] = {{&S88527[149], 7}, {NULL}};
const Segment SGI212971[] = {{&S83290[34], 10}, {NULL}};
const Segment SGI212972[] = {{&S78256[104], 9}, {NULL}};
const Segment SGI212973[] = {{&S83290[27], 6}, {NULL}};
const Segment SGI212974[] = {{&S78256[77], 6}, {NULL}};
const Segment SGI212975[] = {{&S78256[145], 8}, {NULL}};
const Segment SGI212976[] = {{&S83290[97], 9}, {NULL}};
const Segment SGI212977[] = {{&S83290[70], 6}, {NULL}};
const Segment SGI212978[] = {{&S88533[44], 10}, {NULL}};
const Segment SGI212979[] = {{&S88533[44], 22}, {NULL}};
const Segment SGI212980[] = {{&S88533[30], 6}, {NULL}};
const Segment SGI212981[] = {{&S88533[31], 3}, {NULL}};
const Segment SGI212982[] = {{&S88533[80], 5}, {NULL}};
const Segment SGI212983[] = {{&S88533[117], 14}, {NULL}};
const Segment SGI212984[] = {{&S88534[34], 11}, {NULL}};
const Segment SGI212985[] = {{&S88534[27], 6}, {NULL}};
const Segment SGI212986[] = {{&S88534[95], 9}, {NULL}};
const Segment SGI212987[] = {{&S88534[68], 6}, {NULL}};
const Segment SGI212988[] = {{&S88539[34], 13}, {NULL}};
const Segment SGI212989[] = {{&S88539[27], 6}, {NULL}};
const Segment SGI212990[] = {{&S88539[100], 9}, {NULL}};
const Segment SGI212991[] = {{&S88539[73], 6}, {NULL}};
const Segment SGI212992[] = {{&S88558[44], 7}, {NULL}};
const Segment SGI212993[] = {{&S88558[44], 19}, {NULL}};
const Segment SGI212994[] = {{&S88558[30], 6}, {NULL}};
const Segment SGI212995[] = {{&S88558[31], 3}, {NULL}};
const Segment SGI212996[] = {{&S88558[77], 5}, {NULL}};
const Segment SGI212997[] = {{&S88558[114], 14}, {NULL}};
const Segment SGI212998[] = {{&S88559[44], 8}, {NULL}};
const Segment SGI212999[] = {{&S88559[44], 20}, {NULL}};
const Segment SGI213000[] = {{&S88559[30], 6}, {NULL}};
const Segment SGI213001[] = {{&S88559[31], 3}, {NULL}};
const Segment SGI213002[] = {{&S88559[78], 5}, {NULL}};
const Segment SGI213003[] = {{&S88559[115], 14}, {NULL}};
const Segment SGI213004[] = {{&S88560[34], 8}, {NULL}};
const Segment SGI213005[] = {{&S88560[27], 6}, {NULL}};
const Segment SGI213006[] = {{&S88560[95], 9}, {NULL}};
const Segment SGI213007[] = {{&S88560[68], 6}, {NULL}};
const Segment SGI213008[] = {{&S78302[34], 11}, {NULL}};
const Segment SGI213009[] = {{&S78302[27], 6}, {NULL}};
const Segment SGI213010[] = {{&S78302[98], 9}, {NULL}};
const Segment SGI213011[] = {{&S78302[71], 6}, {NULL}};
const Segment SGI213012[] = {{&S88561[34], 13}, {NULL}};
const Segment SGI213013[] = {{&S88561[27], 6}, {NULL}};
const Segment SGI213014[] = {{&S88561[100], 9}, {NULL}};
const Segment SGI213015[] = {{&S88561[73], 6}, {NULL}};
const Segment SGI213016[] = {{&S78303[34], 11}, {NULL}};
const Segment SGI213017[] = {{&S78303[27], 6}, {NULL}};
const Segment SGI213018[] = {{&S78303[98], 9}, {NULL}};
const Segment SGI213019[] = {{&S78303[71], 6}, {NULL}};
const Segment SGI213020[] = {{&S78304[34], 11}, {NULL}};
const Segment SGI213021[] = {{&S78304[27], 6}, {NULL}};
const Segment SGI213022[] = {{&S78304[98], 9}, {NULL}};
const Segment SGI213023[] = {{&S78304[71], 6}, {NULL}};
const Segment SGI213024[] = {{&S83341[34], 7}, {NULL}};
const Segment SGI213025[] = {{&S83341[27], 6}, {NULL}};
const Segment SGI213026[] = {{&S83341[99], 9}, {NULL}};
const Segment SGI213027[] = {{&S83341[72], 6}, {NULL}};
const Segment SGI213028[] = {{&S78308[44], 8}, {NULL}};
const Segment SGI213029[] = {{&S78308[44], 20}, {NULL}};
const Segment SGI213030[] = {{&S78308[30], 6}, {NULL}};
const Segment SGI213031[] = {{&S78308[113], 3}, {NULL}};
const Segment SGI213032[] = {{&S78308[78], 6}, {NULL}};
const Segment SGI213033[] = {{&S78316[44], 10}, {NULL}};
const Segment SGI213034[] = {{&S78316[44], 22}, {NULL}};
const Segment SGI213035[] = {{&S78316[30], 6}, {NULL}};
const Segment SGI213036[] = {{&S78316[80], 6}, {NULL}};
const Segment SGI213037[] = {{&S78316[107], 13}, {NULL}};
const Segment SGI213038[] = {{&S78320[34], 7}, {NULL}};
const Segment SGI213039[] = {{&S78320[27], 6}, {NULL}};
const Segment SGI213040[] = {{&S78320[94], 9}, {NULL}};
const Segment SGI213041[] = {{&S78320[67], 6}, {NULL}};
const Segment SGI213042[] = {{&S83358[34], 7}, {NULL}};
const Segment SGI213043[] = {{&S83358[27], 6}, {NULL}};
const Segment SGI213044[] = {{&S83358[94], 9}, {NULL}};
const Segment SGI213045[] = {{&S83358[67], 6}, {NULL}};
const Segment SGI213046[] = {{&S83364[34], 7}, {NULL}};
const Segment SGI213047[] = {{&S83364[27], 6}, {NULL}};
const Segment SGI213048[] = {{&S83364[94], 9}, {NULL}};
const Segment SGI213049[] = {{&S83364[67], 6}, {NULL}};
const Segment SGI213050[] = {{&S88599[34], 10}, {NULL}};
const Segment SGI213051[] = {{&S88599[27], 6}, {NULL}};
const Segment SGI213052[] = {{&S88599[97], 9}, {NULL}};
const Segment SGI213053[] = {{&S88599[70], 6}, {NULL}};
const Segment SGI213054[] = {{&S88602[34], 7}, {NULL}};
const Segment SGI213055[] = {{&S88602[27], 6}, {NULL}};
const Segment SGI213056[] = {{&S88602[94], 9}, {NULL}};
const Segment SGI213057[] = {{&S88602[67], 6}, {NULL}};
const Segment SGI213058[] = {{&S87882[43], 17}, {NULL}};
const Segment SGI213059[] = {{&S87882[30], 4}, {NULL}};
const Segment SGI213060[] = {{&S87882[129], 3}, {NULL}};
const Segment SGI213061[] = {{&S87882[95], 5}, {NULL}};
const Segment SGI213062[] = {{&S88616[44], 18}, {NULL}};
const Segment SGI213063[] = {{&S88616[44], 30}, {NULL}};
const Segment SGI213064[] = {{&S88616[30], 6}, {NULL}};
const Segment SGI213065[] = {{&S88616[31], 3}, {NULL}};
const Segment SGI213066[] = {{&S88616[88], 6}, {NULL}};
const Segment SGI213067[] = {{&S78359[34], 10}, {NULL}};
const Segment SGI213068[] = {{&S78359[27], 6}, {NULL}};
const Segment SGI213069[] = {{&S78359[97], 9}, {NULL}};
const Segment SGI213070[] = {{&S78359[70], 6}, {NULL}};
const Segment SGI213071[] = {{&S88618[34], 16}, {NULL}};
const Segment SGI213072[] = {{&S88618[27], 6}, {NULL}};
const Segment SGI213073[] = {{&S78360[34], 10}, {NULL}};
const Segment SGI213074[] = {{&S78360[27], 6}, {NULL}};
const Segment SGI213075[] = {{&S78360[97], 9}, {NULL}};
const Segment SGI213076[] = {{&S78360[70], 6}, {NULL}};
const Segment SGI213077[] = {{&S78361[34], 10}, {NULL}};
const Segment SGI213078[] = {{&S78361[27], 6}, {NULL}};
const Segment SGI213079[] = {{&S78361[97], 9}, {NULL}};
const Segment SGI213080[] = {{&S78361[70], 6}, {NULL}};
const Segment SGI213081[] = {{&S78371[44], 10}, {NULL}};
const Segment SGI213082[] = {{&S78371[44], 22}, {NULL}};
const Segment SGI213083[] = {{&S78371[30], 6}, {NULL}};
const Segment SGI213084[] = {{&S78371[115], 3}, {NULL}};
const Segment SGI213085[] = {{&S78371[80], 6}, {NULL}};
const Segment SGI213086[] = {{&S78371[140], 7}, {NULL}};
const Segment SGI213087[] = {{&S78374[34], 10}, {NULL}};
const Segment SGI213088[] = {{&S78374[27], 6}, {NULL}};
const Segment SGI213089[] = {{&S78374[97], 9}, {NULL}};
const Segment SGI213090[] = {{&S78374[70], 6}, {NULL}};
const Segment SGI213091[] = {{&S83432[34], 7}, {NULL}};
const Segment SGI213092[] = {{&S83432[27], 6}, {NULL}};
const Segment SGI213093[] = {{&S83432[99], 9}, {NULL}};
const Segment SGI213094[] = {{&S83432[72], 6}, {NULL}};
const Segment SGI213095[] = {{&S83433[34], 11}, {NULL}};
const Segment SGI213096[] = {{&S83433[27], 6}, {NULL}};
const Segment SGI213097[] = {{&S83433[98], 9}, {NULL}};
const Segment SGI213098[] = {{&S83433[71], 6}, {NULL}};
const Segment SGI213099[] = {{&S83434[34], 10}, {NULL}};
const Segment SGI213100[] = {{&S83434[27], 6}, {NULL}};
const Segment SGI213101[] = {{&S83434[97], 9}, {NULL}};
const Segment SGI213102[] = {{&S83434[70], 6}, {NULL}};
const Segment SGI213103[] = {{&S83438[44], 17}, {NULL}};
const Segment SGI213104[] = {{&S83438[44], 29}, {NULL}};
const Segment SGI213105[] = {{&S83438[30], 6}, {NULL}};
const Segment SGI213106[] = {{&S83438[122], 3}, {NULL}};
const Segment SGI213107[] = {{&S83438[87], 6}, {NULL}};
const Segment SGI213108[] = {{&S83439[44], 15}, {NULL}};
const Segment SGI213109[] = {{&S83439[44], 34}, {NULL}};
const Segment SGI213110[] = {{&S83439[30], 6}, {NULL}};
const Segment SGI213111[] = {{&S78406[44], 7}, {NULL}};
const Segment SGI213112[] = {{&S78406[44], 19}, {NULL}};
const Segment SGI213113[] = {{&S78406[30], 6}, {NULL}};
const Segment SGI213114[] = {{&S83439[127], 3}, {NULL}};
const Segment SGI213115[] = {{&S83439[92], 6}, {NULL}};
const Segment SGI213116[] = {{&S78406[112], 3}, {NULL}};
const Segment SGI213117[] = {{&S78406[77], 6}, {NULL}};
const Segment SGI213118[] = {{&S83440[44], 15}, {NULL}};
const Segment SGI213119[] = {{&S83440[44], 34}, {NULL}};
const Segment SGI213120[] = {{&S83440[30], 6}, {NULL}};
const Segment SGI213121[] = {{&S83440[127], 3}, {NULL}};
const Segment SGI213122[] = {{&S83440[92], 6}, {NULL}};
const Segment SGI213123[] = {{&S83441[44], 15}, {NULL}};
const Segment SGI213124[] = {{&S83441[44], 34}, {NULL}};
const Segment SGI213125[] = {{&S83441[30], 6}, {NULL}};
const Segment SGI213126[] = {{&S83441[127], 3}, {NULL}};
const Segment SGI213127[] = {{&S83441[92], 6}, {NULL}};
const Segment SGI213128[] = {{&S78405[44], 10}, {NULL}};
const Segment SGI213129[] = {{&S78405[44], 22}, {NULL}};
const Segment SGI213130[] = {{&S78405[30], 6}, {NULL}};
const Segment SGI213131[] = {{&S78405[115], 3}, {NULL}};
const Segment SGI213132[] = {{&S78405[80], 6}, {NULL}};
const Segment SGI213133[] = {{&S78405[140], 13}, {NULL}};
const Segment SGI213134[] = {{&S78410[34], 9}, {NULL}};
const Segment SGI213135[] = {{&S78410[27], 6}, {NULL}};
const Segment SGI213136[] = {{&S78410[96], 9}, {NULL}};
const Segment SGI213137[] = {{&S78410[69], 6}, {NULL}};
const Segment SGI213138[] = {{&S78411[34], 9}, {NULL}};
const Segment SGI213139[] = {{&S78411[27], 6}, {NULL}};
const Segment SGI213140[] = {{&S78411[96], 9}, {NULL}};
const Segment SGI213141[] = {{&S78411[69], 6}, {NULL}};
const Segment SGI213142[] = {{&S78412[34], 12}, {NULL}};
const Segment SGI213143[] = {{&S78412[27], 6}, {NULL}};
const Segment SGI213144[] = {{&S78412[99], 9}, {NULL}};
const Segment SGI213145[] = {{&S78412[72], 6}, {NULL}};
const Segment SGI213146[] = {{&S78413[34], 9}, {NULL}};
const Segment SGI213147[] = {{&S78413[27], 6}, {NULL}};
const Segment SGI213148[] = {{&S78413[96], 9}, {NULL}};
const Segment SGI213149[] = {{&S78413[69], 6}, {NULL}};
const Segment SGI213150[] = {{&S78414[34], 9}, {NULL}};
const Segment SGI213151[] = {{&S78414[27], 6}, {NULL}};
const Segment SGI213152[] = {{&S78414[96], 9}, {NULL}};
const Segment SGI213153[] = {{&S78414[69], 6}, {NULL}};
const Segment SGI213154[] = {{&S83468[34], 17}, {NULL}};
const Segment SGI213155[] = {{&S83468[27], 6}, {NULL}};
const Segment SGI213156[] = {{&S83468[104], 9}, {NULL}};
const Segment SGI213157[] = {{&S83468[77], 6}, {NULL}};
const Segment SGI213158[] = {{&S83470[44], 7}, {NULL}};
const Segment SGI213159[] = {{&S83470[44], 19}, {NULL}};
const Segment SGI213160[] = {{&S83470[30], 6}, {NULL}};
const Segment SGI213161[] = {{&S83470[112], 3}, {NULL}};
const Segment SGI213162[] = {{&S83470[77], 6}, {NULL}};
const Segment SGI213163[] = {{&S83476[34], 25}, {NULL}};
const Segment SGI213164[] = {{&S83476[27], 6}, {NULL}};
const Segment SGI213165[] = {{&S83476[112], 9}, {NULL}};
const Segment SGI213166[] = {{&S83476[85], 6}, {NULL}};
const Segment SGI213167[] = {{&S69521[34], 18}, {NULL}};
const Segment SGI213168[] = {{&S69521[27], 6}, {NULL}};
const Segment SGI213169[] = {{&S69521[104], 9}, {NULL}};
const Segment SGI213170[] = {{&S69521[77], 6}, {NULL}};
const Segment SGI213171[] = {{&S69522[34], 10}, {NULL}};
const Segment SGI213172[] = {{&S69522[27], 6}, {NULL}};
const Segment SGI213173[] = {{&S69522[96], 9}, {NULL}};
const Segment SGI213174[] = {{&S69522[69], 6}, {NULL}};
const Segment SGI213175[] = {{&S78473[34], 8}, {NULL}};
const Segment SGI213176[] = {{&S78473[27], 6}, {NULL}};
const Segment SGI213177[] = {{&S78473[95], 9}, {NULL}};
const Segment SGI213178[] = {{&S78473[68], 6}, {NULL}};
const Segment SGI213179[] = {{&S69532[34], 10}, {NULL}};
const Segment SGI213180[] = {{&S69532[27], 6}, {NULL}};
const Segment SGI213181[] = {{&S69532[96], 9}, {NULL}};
const Segment SGI213182[] = {{&S69532[69], 6}, {NULL}};
const Segment SGI213183[] = {{&S69533[34], 10}, {NULL}};
const Segment SGI213184[] = {{&S69533[27], 6}, {NULL}};
const Segment SGI213185[] = {{&S69533[96], 9}, {NULL}};
const Segment SGI213186[] = {{&S69533[69], 6}, {NULL}};
const Segment SGI213187[] = {{&S78468[34], 8}, {NULL}};
const Segment SGI213188[] = {{&S78468[27], 6}, {NULL}};
const Segment SGI213189[] = {{&S88618[111], 9}, {NULL}};
const Segment SGI213190[] = {{&S88618[84], 6}, {NULL}};
const Segment SGI213191[] = {{&S78468[95], 9}, {NULL}};
const Segment SGI213192[] = {{&S78468[68], 6}, {NULL}};
const Segment SGI213193[] = {{&S88622[44], 10}, {NULL}};
const Segment SGI213194[] = {{&S88622[44], 22}, {NULL}};
const Segment SGI213195[] = {{&S88622[30], 6}, {NULL}};
const Segment SGI213196[] = {{&S88622[31], 3}, {NULL}};
const Segment SGI213197[] = {{&S88622[80], 5}, {NULL}};
const Segment SGI213198[] = {{&S88622[117], 14}, {NULL}};
const Segment SGI213199[] = {{&S83513[34], 9}, {NULL}};
const Segment SGI213200[] = {{&S83513[27], 6}, {NULL}};
const Segment SGI213201[] = {{&S83513[96], 9}, {NULL}};
const Segment SGI213202[] = {{&S83513[69], 6}, {NULL}};
const Segment SGI213203[] = {{&S88628[34], 14}, {NULL}};
const Segment SGI213204[] = {{&S88628[27], 6}, {NULL}};
const Segment SGI213205[] = {{&S88628[101], 9}, {NULL}};
const Segment SGI213206[] = {{&S88628[74], 6}, {NULL}};
const Segment SGI213207[] = {{&S83517[34], 10}, {NULL}};
const Segment SGI213208[] = {{&S83517[27], 6}, {NULL}};
const Segment SGI213209[] = {{&S83517[97], 9}, {NULL}};
const Segment SGI213210[] = {{&S83517[70], 6}, {NULL}};
const Segment SGI213211[] = {{&S88633[34], 7}, {NULL}};
const Segment SGI213212[] = {{&S88633[27], 6}, {NULL}};
const Segment SGI213213[] = {{&S78501[34], 12}, {NULL}};
const Segment SGI213214[] = {{&S78501[27], 6}, {NULL}};
const Segment SGI213215[] = {{&S88633[99], 9}, {NULL}};
const Segment SGI213216[] = {{&S88633[72], 6}, {NULL}};
const Segment SGI213217[] = {{&S78501[99], 9}, {NULL}};
const Segment SGI213218[] = {{&S78501[72], 6}, {NULL}};
const Segment SGI213219[] = {{&S88635[44], 10}, {NULL}};
const Segment SGI213220[] = {{&S88635[44], 22}, {NULL}};
const Segment SGI213221[] = {{&S88635[30], 6}, {NULL}};
const Segment SGI213222[] = {{&S88635[31], 3}, {NULL}};
const Segment SGI213223[] = {{&S88635[80], 5}, {NULL}};
const Segment SGI213224[] = {{&S88635[117], 14}, {NULL}};
const Segment SGI213225[] = {{&S88636[44], 5}, {NULL}};
const Segment SGI213226[] = {{&S88636[44], 17}, {NULL}};
const Segment SGI213227[] = {{&S88636[30], 6}, {NULL}};
const Segment SGI213228[] = {{&S88636[75], 6}, {NULL}};
const Segment SGI213229[] = {{&S88636[102], 13}, {NULL}};
const Segment SGI213230[] = {{&S88638[44], 9}, {NULL}};
const Segment SGI213231[] = {{&S88638[44], 21}, {NULL}};
const Segment SGI213232[] = {{&S88638[30], 6}, {NULL}};
const Segment SGI213233[] = {{&S88638[79], 6}, {NULL}};
const Segment SGI213234[] = {{&S88638[106], 13}, {NULL}};
const Segment SGI213235[] = {{&S88641[34], 11}, {NULL}};
const Segment SGI213236[] = {{&S88641[27], 6}, {NULL}};
const Segment SGI213237[] = {{&S88641[98], 9}, {NULL}};
const Segment SGI213238[] = {{&S88641[71], 6}, {NULL}};
const Segment SGI213239[] = {{&S78517[44], 12}, {NULL}};
const Segment SGI213240[] = {{&S78517[44], 24}, {NULL}};
const Segment SGI213241[] = {{&S78517[30], 6}, {NULL}};
const Segment SGI213242[] = {{&S78517[117], 3}, {NULL}};
const Segment SGI213243[] = {{&S78517[82], 6}, {NULL}};
const Segment SGI213244[] = {{&S88650[44], 17}, {NULL}};
const Segment SGI213245[] = {{&S88650[44], 29}, {NULL}};
const Segment SGI213246[] = {{&S88650[30], 6}, {NULL}};
const Segment SGI213247[] = {{&S88650[31], 3}, {NULL}};
const Segment SGI213248[] = {{&S88650[87], 6}, {NULL}};
const Segment SGI213249[] = {{&S78514[44], 19}, {NULL}};
const Segment SGI213250[] = {{&S78514[44], 31}, {NULL}};
const Segment SGI213251[] = {{&S78514[30], 6}, {NULL}};
const Segment SGI213252[] = {{&S78514[124], 3}, {NULL}};
const Segment SGI213253[] = {{&S78514[89], 6}, {NULL}};
const Segment SGI213254[] = {{&S88657[44], 7}, {NULL}};
const Segment SGI213255[] = {{&S88657[44], 25}, {NULL}};
const Segment SGI213256[] = {{&S88657[30], 6}, {NULL}};
const Segment SGI213257[] = {{&S88657[83], 6}, {NULL}};
const Segment SGI213258[] = {{&S88657[110], 13}, {NULL}};
const Segment SGI213259[] = {{&S78516[34], 9}, {NULL}};
const Segment SGI213260[] = {{&S78516[27], 6}, {NULL}};
const Segment SGI213261[] = {{&S78516[96], 9}, {NULL}};
const Segment SGI213262[] = {{&S78516[69], 6}, {NULL}};
const Segment SGI213263[] = {{&S69602[34], 11}, {NULL}};
const Segment SGI213264[] = {{&S69602[27], 6}, {NULL}};
const Segment SGI213265[] = {{&S88669[34], 7}, {NULL}};
const Segment SGI213266[] = {{&S69602[97], 9}, {NULL}};
const Segment SGI213267[] = {{&S88669[27], 6}, {NULL}};
const Segment SGI213268[] = {{&S69602[70], 6}, {NULL}};
const Segment SGI213269[] = {{&S88669[93], 9}, {NULL}};
const Segment SGI213270[] = {{&S88669[66], 6}, {NULL}};
const Segment SGI213271[] = {{&S88669[134], 8}, {NULL}};
const Segment SGI213272[] = {{&S88670[34], 7}, {NULL}};
const Segment SGI213273[] = {{&S88670[27], 6}, {NULL}};
const Segment SGI213274[] = {{&S88670[99], 9}, {NULL}};
const Segment SGI213275[] = {{&S88670[72], 6}, {NULL}};
const Segment SGI213276[] = {{&S88670[140], 8}, {NULL}};
const Segment SGI213277[] = {{&S88671[34], 5}, {NULL}};
const Segment SGI213278[] = {{&S88671[27], 6}, {NULL}};
const Segment SGI213279[] = {{&S88671[92], 9}, {NULL}};
const Segment SGI213280[] = {{&S88671[65], 6}, {NULL}};
const Segment SGI213281[] = {{&S88671[133], 8}, {NULL}};
const Segment SGI213282[] = {{&S88674[44], 10}, {NULL}};
const Segment SGI213283[] = {{&S88674[44], 22}, {NULL}};
const Segment SGI213284[] = {{&S88674[30], 6}, {NULL}};
const Segment SGI213285[] = {{&S88674[80], 6}, {NULL}};
const Segment SGI213286[] = {{&S88674[107], 13}, {NULL}};
const Segment SGI213287[] = {{&S69614[34], 10}, {NULL}};
const Segment SGI213288[] = {{&S69614[27], 6}, {NULL}};
const Segment SGI213289[] = {{&S69614[96], 9}, {NULL}};
const Segment SGI213290[] = {{&S69614[69], 6}, {NULL}};
const Segment SGI213291[] = {{&S88676[34], 14}, {NULL}};
const Segment SGI213292[] = {{&S69615[34], 10}, {NULL}};
const Segment SGI213293[] = {{&S88676[27], 6}, {NULL}};
const Segment SGI213294[] = {{&S69615[27], 6}, {NULL}};
const Segment SGI213295[] = {{&S88676[101], 9}, {NULL}};
const Segment SGI213296[] = {{&S69615[96], 9}, {NULL}};
const Segment SGI213297[] = {{&S88676[74], 6}, {NULL}};
const Segment SGI213298[] = {{&S69615[69], 6}, {NULL}};
const Segment SGI213299[] = {{&S78551[34], 9}, {NULL}};
const Segment SGI213300[] = {{&S78551[27], 6}, {NULL}};
const Segment SGI213301[] = {{&S78551[96], 9}, {NULL}};
const Segment SGI213302[] = {{&S78551[69], 6}, {NULL}};
const Segment SGI213303[] = {{&S88685[34], 7}, {NULL}};
const Segment SGI213304[] = {{&S88685[27], 6}, {NULL}};
const Segment SGI213305[] = {{&S88685[99], 9}, {NULL}};
const Segment SGI213306[] = {{&S88685[72], 6}, {NULL}};
const Segment SGI213307[] = {{&S69626[44], 10}, {NULL}};
const Segment SGI213308[] = {{&S69626[44], 21}, {NULL}};
const Segment SGI213309[] = {{&S69626[30], 6}, {NULL}};
const Segment SGI213310[] = {{&S69626[114], 3}, {NULL}};
const Segment SGI213311[] = {{&S69626[79], 6}, {NULL}};
const Segment SGI213312[] = {{&S69626[149], 3}, {NULL}};
const Segment SGI213313[] = {{&S78610[34], 11}, {NULL}};
const Segment SGI213314[] = {{&S78610[27], 6}, {NULL}};
const Segment SGI213315[] = {{&S78610[98], 9}, {NULL}};
const Segment SGI213316[] = {{&S78610[71], 6}, {NULL}};
const Segment SGI213317[] = {{&S78589[34], 6}, {NULL}};
const Segment SGI213318[] = {{&S78589[27], 6}, {NULL}};
const Segment SGI213319[] = {{&S78589[93], 9}, {NULL}};
const Segment SGI213320[] = {{&S78589[66], 6}, {NULL}};
const Segment SGI213321[] = {{&S88723[44], 7}, {NULL}};
const Segment SGI213322[] = {{&S88723[44], 18}, {NULL}};
const Segment SGI213323[] = {{&S88723[30], 6}, {NULL}};
const Segment SGI213324[] = {{&S88723[31], 3}, {NULL}};
const Segment SGI213325[] = {{&S88723[76], 6}, {NULL}};
const Segment SGI213326[] = {{&S78606[34], 9}, {NULL}};
const Segment SGI213327[] = {{&S88737[34], 10}, {NULL}};
const Segment SGI213328[] = {{&S78606[27], 6}, {NULL}};
const Segment SGI213329[] = {{&S88737[27], 6}, {NULL}};
const Segment SGI213330[] = {{&S78606[96], 9}, {NULL}};
const Segment SGI213331[] = {{&S88737[97], 9}, {NULL}};
const Segment SGI213332[] = {{&S78606[69], 6}, {NULL}};
const Segment SGI213333[] = {{&S88737[70], 6}, {NULL}};
const Segment SGI213334[] = {{&S88740[59], 10}, {NULL}};
const Segment SGI213335[] = {{&S88740[59], 22}, {NULL}};
const Segment SGI213336[] = {{&S88740[45], 6}, {NULL}};
const Segment SGI213337[] = {{&S88740[46], 3}, {NULL}};
const Segment SGI213338[] = {{&S88740[95], 6}, {NULL}};
const Segment SGI213339[] = {{&S88740[0], 14}, {NULL}};
const Segment SGI213340[] = {{&S78611[44], 10}, {NULL}};
const Segment SGI213341[] = {{&S78611[44], 22}, {NULL}};
const Segment SGI213342[] = {{&S78611[30], 6}, {NULL}};
const Segment SGI213343[] = {{&S78611[115], 3}, {NULL}};
const Segment SGI213344[] = {{&S78611[80], 6}, {NULL}};
const Segment SGI213345[] = {{&S78611[135], 8}, {NULL}};
const Segment SGI213346[] = {{&S78611[144], 13}, {NULL}};
const Segment SGI213347[] = {{&S78612[45], 10}, {NULL}};
const Segment SGI213348[] = {{&S78612[30], 6}, {NULL}};
const Segment SGI213349[] = {{&S78612[115], 3}, {NULL}};
const Segment SGI213350[] = {{&S78612[81], 5}, {NULL}};
const Segment SGI213351[] = {{&S78612[139], 13}, {NULL}};
const Segment SGI213352[] = {{&S88745[44], 18}, {NULL}};
const Segment SGI213353[] = {{&S88745[44], 30}, {NULL}};
const Segment SGI213354[] = {{&S88745[30], 6}, {NULL}};
const Segment SGI213355[] = {{&S88745[31], 3}, {NULL}};
const Segment SGI213356[] = {{&S88745[88], 6}, {NULL}};
const Segment SGI213357[] = {{&S88745[148], 11}, {NULL}};
const Segment SGI213358[] = {{&S78614[34], 9}, {NULL}};
const Segment SGI213359[] = {{&S78614[27], 6}, {NULL}};
const Segment SGI213360[] = {{&S78614[96], 9}, {NULL}};
const Segment SGI213361[] = {{&S78614[69], 6}, {NULL}};
const Segment SGI213362[] = {{&S88752[44], 14}, {NULL}};
const Segment SGI213363[] = {{&S88752[44], 26}, {NULL}};
const Segment SGI213364[] = {{&S88752[30], 6}, {NULL}};
const Segment SGI213365[] = {{&S88752[31], 3}, {NULL}};
const Segment SGI213366[] = {{&S88752[84], 6}, {NULL}};
const Segment SGI213367[] = {{&S88757[44], 4}, {NULL}};
const Segment SGI213368[] = {{&S88757[44], 16}, {NULL}};
const Segment SGI213369[] = {{&S88757[30], 6}, {NULL}};
const Segment SGI213370[] = {{&S88757[31], 3}, {NULL}};
const Segment SGI213371[] = {{&S88757[74], 6}, {NULL}};
const Segment SGI213372[] = {{&S88758[44], 7}, {NULL}};
const Segment SGI213373[] = {{&S88758[44], 24}, {NULL}};
const Segment SGI213374[] = {{&S88758[30], 6}, {NULL}};
const Segment SGI213375[] = {{&S88758[82], 6}, {NULL}};
const Segment SGI213376[] = {{&S88758[109], 13}, {NULL}};
const Segment SGI213377[] = {{&S78663[34], 6}, {NULL}};
const Segment SGI213378[] = {{&S78663[27], 6}, {NULL}};
const Segment SGI213379[] = {{&S78663[93], 9}, {NULL}};
const Segment SGI213380[] = {{&S78663[66], 6}, {NULL}};
const Segment SGI213381[] = {{&S88761[34], 7}, {NULL}};
const Segment SGI213382[] = {{&S88761[27], 6}, {NULL}};
const Segment SGI213383[] = {{&S88761[94], 9}, {NULL}};
const Segment SGI213384[] = {{&S88761[67], 6}, {NULL}};
const Segment SGI213385[] = {{&S83540[34], 16}, {NULL}};
const Segment SGI213386[] = {{&S83540[27], 6}, {NULL}};
const Segment SGI213387[] = {{&S88762[34], 10}, {NULL}};
const Segment SGI213388[] = {{&S88762[27], 6}, {NULL}};
const Segment SGI213389[] = {{&S83540[103], 9}, {NULL}};
const Segment SGI213390[] = {{&S83540[76], 6}, {NULL}};
const Segment SGI213391[] = {{&S88762[97], 9}, {NULL}};
const Segment SGI213392[] = {{&S88762[70], 6}, {NULL}};
const Segment SGI213393[] = {{&S88763[44], 11}, {NULL}};
const Segment SGI213394[] = {{&S88763[44], 23}, {NULL}};
const Segment SGI213395[] = {{&S88763[30], 6}, {NULL}};
const Segment SGI213396[] = {{&S88763[31], 3}, {NULL}};
const Segment SGI213397[] = {{&S88763[81], 6}, {NULL}};
const Segment SGI213398[] = {{&S88766[44], 19}, {NULL}};
const Segment SGI213399[] = {{&S88766[44], 31}, {NULL}};
const Segment SGI213400[] = {{&S88766[30], 6}, {NULL}};
const Segment SGI213401[] = {{&S88766[31], 3}, {NULL}};
const Segment SGI213402[] = {{&S88766[89], 6}, {NULL}};
const Segment SGI213403[] = {{&S88768[45], 10}, {NULL}};
const Segment SGI213404[] = {{&S88768[30], 6}, {NULL}};
const Segment SGI213405[] = {{&S88768[31], 3}, {NULL}};
const Segment SGI213406[] = {{&S88768[81], 5}, {NULL}};
const Segment SGI213407[] = {{&S88768[139], 13}, {NULL}};
const Segment SGI213408[] = {{&S78652[34], 10}, {NULL}};
const Segment SGI213409[] = {{&S78652[27], 6}, {NULL}};
const Segment SGI213410[] = {{&S78652[97], 9}, {NULL}};
const Segment SGI213411[] = {{&S78652[70], 6}, {NULL}};
const Segment SGI213412[] = {{&S78653[34], 10}, {NULL}};
const Segment SGI213413[] = {{&S78653[27], 6}, {NULL}};
const Segment SGI213414[] = {{&S88774[34], 10}, {NULL}};
const Segment SGI213415[] = {{&S88774[27], 6}, {NULL}};
const Segment SGI213416[] = {{&S78653[97], 9}, {NULL}};
const Segment SGI213417[] = {{&S78653[70], 6}, {NULL}};
const Segment SGI213418[] = {{&S88774[97], 9}, {NULL}};
const Segment SGI213419[] = {{&S88774[70], 6}, {NULL}};
const Segment SGI213420[] = {{&S88774[138], 8}, {NULL}};
const Segment SGI213421[] = {{&S78654[34], 10}, {NULL}};
const Segment SGI213422[] = {{&S78654[27], 6}, {NULL}};
const Segment SGI213423[] = {{&S78654[97], 9}, {NULL}};
const Segment SGI213424[] = {{&S78654[70], 6}, {NULL}};
const Segment SGI213425[] = {{&S69738[34], 18}, {NULL}};
const Segment SGI213426[] = {{&S69738[27], 6}, {NULL}};
const Segment SGI213427[] = {{&S69738[105], 9}, {NULL}};
const Segment SGI213428[] = {{&S69738[78], 6}, {NULL}};
const Segment SGI213429[] = {{&S78695[34], 18}, {NULL}};
const Segment SGI213430[] = {{&S78695[27], 6}, {NULL}};
const Segment SGI213431[] = {{&S78662[44], 9}, {NULL}};
const Segment SGI213432[] = {{&S78662[44], 21}, {NULL}};
const Segment SGI213433[] = {{&S78695[105], 9}, {NULL}};
const Segment SGI213434[] = {{&S78662[30], 6}, {NULL}};
const Segment SGI213435[] = {{&S78695[78], 6}, {NULL}};
const Segment SGI213436[] = {{&S78662[114], 3}, {NULL}};
const Segment SGI213437[] = {{&S78662[79], 6}, {NULL}};
const Segment SGI213438[] = {{&S78664[34], 8}, {NULL}};
const Segment SGI213439[] = {{&S78664[27], 6}, {NULL}};
const Segment SGI213440[] = {{&S78664[100], 9}, {NULL}};
const Segment SGI213441[] = {{&S78664[73], 6}, {NULL}};
const Segment SGI213442[] = {{&S88791[52], 5}, {NULL}};
const Segment SGI213443[] = {{&S88791[52], 17}, {NULL}};
const Segment SGI213444[] = {{&S88791[38], 6}, {NULL}};
const Segment SGI213445[] = {{&S88791[83], 5}, {NULL}};
const Segment SGI213446[] = {{&S88791[0], 14}, {NULL}};
const Segment SGI213447[] = {{&S78713[34], 10}, {NULL}};
const Segment SGI213448[] = {{&S78713[27], 6}, {NULL}};
const Segment SGI213449[] = {{&S78713[97], 9}, {NULL}};
const Segment SGI213450[] = {{&S78713[70], 6}, {NULL}};
const Segment SGI213451[] = {{&S69760[34], 10}, {NULL}};
const Segment SGI213452[] = {{&S69760[27], 6}, {NULL}};
const Segment SGI213453[] = {{&S69760[96], 9}, {NULL}};
const Segment SGI213454[] = {{&S69760[69], 6}, {NULL}};
const Segment SGI213455[] = {{&S69761[34], 10}, {NULL}};
const Segment SGI213456[] = {{&S69761[27], 6}, {NULL}};
const Segment SGI213457[] = {{&S88804[34], 10}, {NULL}};
const Segment SGI213458[] = {{&S88804[27], 6}, {NULL}};
const Segment SGI213459[] = {{&S69761[96], 9}, {NULL}};
const Segment SGI213460[] = {{&S69761[69], 6}, {NULL}};
const Segment SGI213461[] = {{&S88804[97], 9}, {NULL}};
const Segment SGI213462[] = {{&S88804[70], 6}, {NULL}};
const Segment SGI213463[] = {{&S88809[34], 8}, {NULL}};
const Segment SGI213464[] = {{&S88809[27], 6}, {NULL}};
const Segment SGI213465[] = {{&S88809[95], 9}, {NULL}};
const Segment SGI213466[] = {{&S88809[68], 6}, {NULL}};
const Segment SGI213467[] = {{&S88813[34], 10}, {NULL}};
const Segment SGI213468[] = {{&S88813[27], 6}, {NULL}};
const Segment SGI213469[] = {{&S88813[97], 9}, {NULL}};
const Segment SGI213470[] = {{&S88813[70], 6}, {NULL}};
const Segment SGI213471[] = {{&S88814[34], 10}, {NULL}};
const Segment SGI213472[] = {{&S88814[27], 6}, {NULL}};
const Segment SGI213473[] = {{&S88814[97], 9}, {NULL}};
const Segment SGI213474[] = {{&S88814[70], 6}, {NULL}};
const Segment SGI213475[] = {{&S78698[44], 10}, {NULL}};
const Segment SGI213476[] = {{&S78698[44], 22}, {NULL}};
const Segment SGI213477[] = {{&S78698[30], 6}, {NULL}};
const Segment SGI213478[] = {{&S78698[115], 3}, {NULL}};
const Segment SGI213479[] = {{&S78698[80], 6}, {NULL}};
const Segment SGI213480[] = {{&S78698[140], 11}, {NULL}};
const Segment SGI213481[] = {{&S83609[44], 13}, {NULL}};
const Segment SGI213482[] = {{&S83609[44], 25}, {NULL}};
const Segment SGI213483[] = {{&S83609[30], 6}, {NULL}};
const Segment SGI213484[] = {{&S83609[118], 3}, {NULL}};
const Segment SGI213485[] = {{&S83609[83], 6}, {NULL}};
const Segment SGI213486[] = {{&S78710[34], 10}, {NULL}};
const Segment SGI213487[] = {{&S78710[27], 6}, {NULL}};
const Segment SGI213488[] = {{&S78710[97], 9}, {NULL}};
const Segment SGI213489[] = {{&S78710[70], 6}, {NULL}};
const Segment SGI213490[] = {{&S78749[44], 12}, {NULL}};
const Segment SGI213491[] = {{&S78749[44], 24}, {NULL}};
const Segment SGI213492[] = {{&S78749[30], 6}, {NULL}};
const Segment SGI213493[] = {{&S78749[117], 3}, {NULL}};
const Segment SGI213494[] = {{&S78749[82], 6}, {NULL}};
const Segment SGI213495[] = {{&S78750[44], 17}, {NULL}};
const Segment SGI213496[] = {{&S78750[44], 29}, {NULL}};
const Segment SGI213497[] = {{&S78750[30], 6}, {NULL}};
const Segment SGI213498[] = {{&S78750[122], 3}, {NULL}};
const Segment SGI213499[] = {{&S78750[87], 6}, {NULL}};
const Segment SGI213500[] = {{&S78715[34], 28}, {NULL}};
const Segment SGI213501[] = {{&S78715[27], 6}, {NULL}};
const Segment SGI213502[] = {{&S78715[115], 9}, {NULL}};
const Segment SGI213503[] = {{&S78715[88], 6}, {NULL}};
const Segment SGI213504[] = {{&S83637[59], 22}, {NULL}};
const Segment SGI213505[] = {{&S83637[59], 35}, {NULL}};
const Segment SGI213506[] = {{&S83637[45], 6}, {NULL}};
const Segment SGI213507[] = {{&S83637[143], 3}, {NULL}};
const Segment SGI213508[] = {{&S83637[108], 6}, {NULL}};
const Segment SGI213509[] = {{&S83637[0], 14}, {NULL}};
const Segment SGI213510[] = {{&S83642[34], 15}, {NULL}};
const Segment SGI213511[] = {{&S83642[27], 6}, {NULL}};
const Segment SGI213512[] = {{&S83642[102], 9}, {NULL}};
const Segment SGI213513[] = {{&S83642[75], 6}, {NULL}};
const Segment SGI213514[] = {{&S83642[143], 8}, {NULL}};
const Segment SGI213515[] = {{&S83644[34], 10}, {NULL}};
const Segment SGI213516[] = {{&S83644[27], 6}, {NULL}};
const Segment SGI213517[] = {{&S83644[97], 9}, {NULL}};
const Segment SGI213518[] = {{&S83644[70], 6}, {NULL}};
const Segment SGI213519[] = {{&S78745[6], 4}, {NULL}};
const Segment SGI213520[] = {{&S78745[82], 5}, {NULL}};
const Segment SGI213521[] = {{&S88861[45], 6}, {NULL}};
const Segment SGI213522[] = {{&S88861[30], 6}, {NULL}};
const Segment SGI213523[] = {{&S88861[31], 3}, {NULL}};
const Segment SGI213524[] = {{&S88861[77], 5}, {NULL}};
const Segment SGI213525[] = {{&S88861[135], 13}, {NULL}};
const Segment SGI213526[] = {{&S78753[34], 7}, {NULL}};
const Segment SGI213527[] = {{&S78753[27], 6}, {NULL}};
const Segment SGI213528[] = {{&S78753[100], 9}, {NULL}};
const Segment SGI213529[] = {{&S78753[73], 6}, {NULL}};
const Segment SGI213530[] = {{&S88868[34], 11}, {NULL}};
const Segment SGI213531[] = {{&S88868[27], 6}, {NULL}};
const Segment SGI213532[] = {{&S88868[98], 9}, {NULL}};
const Segment SGI213533[] = {{&S88868[71], 6}, {NULL}};
const Segment SGI213534[] = {{&S78754[34], 10}, {NULL}};
const Segment SGI213535[] = {{&S78754[27], 6}, {NULL}};
const Segment SGI213536[] = {{&S78754[97], 9}, {NULL}};
const Segment SGI213537[] = {{&S78754[70], 6}, {NULL}};
const Segment SGI213538[] = {{&S78754[138], 8}, {NULL}};
const Segment SGI213539[] = {{&S78755[34], 11}, {NULL}};
const Segment SGI213540[] = {{&S78755[27], 6}, {NULL}};
const Segment SGI213541[] = {{&S78755[98], 9}, {NULL}};
const Segment SGI213542[] = {{&S78755[71], 6}, {NULL}};
const Segment SGI213543[] = {{&S78755[139], 8}, {NULL}};
const Segment SGI213544[] = {{&S88872[34], 10}, {NULL}};
const Segment SGI213545[] = {{&S88872[27], 6}, {NULL}};
const Segment SGI213546[] = {{&S88872[97], 9}, {NULL}};
const Segment SGI213547[] = {{&S88872[70], 6}, {NULL}};
const Segment SGI213548[] = {{&S78757[34], 7}, {NULL}};
const Segment SGI213549[] = {{&S78757[27], 6}, {NULL}};
const Segment SGI213550[] = {{&S78757[99], 9}, {NULL}};
const Segment SGI213551[] = {{&S78757[72], 6}, {NULL}};
const Segment SGI213552[] = {{&S88877[34], 11}, {NULL}};
const Segment SGI213553[] = {{&S88877[27], 6}, {NULL}};
const Segment SGI213554[] = {{&S88877[98], 9}, {NULL}};
const Segment SGI213555[] = {{&S88877[71], 6}, {NULL}};
const Segment SGI213556[] = {{&S88878[34], 5}, {NULL}};
const Segment SGI213557[] = {{&S88878[27], 6}, {NULL}};
const Segment SGI213558[] = {{&S88878[92], 9}, {NULL}};
const Segment SGI213559[] = {{&S88878[65], 6}, {NULL}};
const Segment SGI213560[] = {{&S88879[34], 11}, {NULL}};
const Segment SGI213561[] = {{&S88879[27], 6}, {NULL}};
const Segment SGI213562[] = {{&S88879[98], 9}, {NULL}};
const Segment SGI213563[] = {{&S88879[71], 6}, {NULL}};
const Segment SGI213564[] = {{&S88880[44], 11}, {NULL}};
const Segment SGI213565[] = {{&S88880[44], 23}, {NULL}};
const Segment SGI213566[] = {{&S88880[30], 6}, {NULL}};
const Segment SGI213567[] = {{&S88880[31], 3}, {NULL}};
const Segment SGI213568[] = {{&S88880[81], 6}, {NULL}};
const Segment SGI213569[] = {{&S88881[44], 23}, {NULL}};
const Segment SGI213570[] = {{&S88881[44], 34}, {NULL}};
const Segment SGI213571[] = {{&S88881[30], 6}, {NULL}};
const Segment SGI213572[] = {{&S88881[31], 3}, {NULL}};
const Segment SGI213573[] = {{&S88881[92], 5}, {NULL}};
const Segment SGI213574[] = {{&S88882[45], 6}, {NULL}};
const Segment SGI213575[] = {{&S88882[30], 6}, {NULL}};
const Segment SGI213576[] = {{&S88882[31], 3}, {NULL}};
const Segment SGI213577[] = {{&S88882[77], 5}, {NULL}};
const Segment SGI213578[] = {{&S88882[135], 13}, {NULL}};
const Segment SGI213579[] = {{&S88888[34], 8}, {NULL}};
const Segment SGI213580[] = {{&S88888[27], 6}, {NULL}};
const Segment SGI213581[] = {{&S88888[95], 9}, {NULL}};
const Segment SGI213582[] = {{&S88888[68], 6}, {NULL}};
const Segment SGI213583[] = {{&S78785[34], 10}, {NULL}};
const Segment SGI213584[] = {{&S78785[27], 6}, {NULL}};
const Segment SGI213585[] = {{&S78785[97], 9}, {NULL}};
const Segment SGI213586[] = {{&S78785[70], 6}, {NULL}};
const Segment SGI213587[] = {{&S78786[34], 10}, {NULL}};
const Segment SGI213588[] = {{&S78786[27], 6}, {NULL}};
const Segment SGI213589[] = {{&S78786[97], 9}, {NULL}};
const Segment SGI213590[] = {{&S78786[70], 6}, {NULL}};
const Segment SGI213591[] = {{&S78787[34], 10}, {NULL}};
const Segment SGI213592[] = {{&S78787[27], 6}, {NULL}};
const Segment SGI213593[] = {{&S88893[44], 10}, {NULL}};
const Segment SGI213594[] = {{&S88893[44], 22}, {NULL}};
const Segment SGI213595[] = {{&S88893[30], 6}, {NULL}};
const Segment SGI213596[] = {{&S78787[97], 9}, {NULL}};
const Segment SGI213597[] = {{&S78787[70], 6}, {NULL}};
const Segment SGI213598[] = {{&S88893[31], 3}, {NULL}};
const Segment SGI213599[] = {{&S88893[80], 6}, {NULL}};
const Segment SGI213600[] = {{&S88893[133], 11}, {NULL}};
const Segment SGI213601[] = {{&S88895[59], 15}, {NULL}};
const Segment SGI213602[] = {{&S88895[59], 27}, {NULL}};
const Segment SGI213603[] = {{&S88895[45], 6}, {NULL}};
const Segment SGI213604[] = {{&S88895[46], 3}, {NULL}};
const Segment SGI213605[] = {{&S88895[100], 6}, {NULL}};
const Segment SGI213606[] = {{&S88895[0], 14}, {NULL}};
const Segment SGI213607[] = {{&S78792[44], 10}, {NULL}};
const Segment SGI213608[] = {{&S78792[44], 22}, {NULL}};
const Segment SGI213609[] = {{&S78792[30], 6}, {NULL}};
const Segment SGI213610[] = {{&S78792[113], 3}, {NULL}};
const Segment SGI213611[] = {{&S78792[80], 5}, {NULL}};
const Segment SGI213612[] = {{&S78793[44], 10}, {NULL}};
const Segment SGI213613[] = {{&S78793[44], 22}, {NULL}};
const Segment SGI213614[] = {{&S78793[30], 6}, {NULL}};
const Segment SGI213615[] = {{&S78793[113], 3}, {NULL}};
const Segment SGI213616[] = {{&S78793[80], 5}, {NULL}};
const Segment SGI213617[] = {{&S83704[44], 10}, {NULL}};
const Segment SGI213618[] = {{&S83704[44], 22}, {NULL}};
const Segment SGI213619[] = {{&S83704[30], 6}, {NULL}};
const Segment SGI213620[] = {{&S83704[115], 3}, {NULL}};
const Segment SGI213621[] = {{&S83704[80], 6}, {NULL}};
const Segment SGI213622[] = {{&S88900[44], 10}, {NULL}};
const Segment SGI213623[] = {{&S88900[44], 22}, {NULL}};
const Segment SGI213624[] = {{&S88900[30], 6}, {NULL}};
const Segment SGI213625[] = {{&S88900[31], 3}, {NULL}};
const Segment SGI213626[] = {{&S88900[80], 6}, {NULL}};
const Segment SGI213627[] = {{&S88901[34], 9}, {NULL}};
const Segment SGI213628[] = {{&S88901[27], 6}, {NULL}};
const Segment SGI213629[] = {{&S88901[96], 9}, {NULL}};
const Segment SGI213630[] = {{&S88901[69], 6}, {NULL}};
const Segment SGI213631[] = {{&S78799[44], 12}, {NULL}};
const Segment SGI213632[] = {{&S78799[44], 24}, {NULL}};
const Segment SGI213633[] = {{&S78799[30], 6}, {NULL}};
const Segment SGI213634[] = {{&S78799[132], 3}, {NULL}};
const Segment SGI213635[] = {{&S78799[98], 6}, {NULL}};
const Segment SGI213636[] = {{&S78799[70], 8}, {NULL}};
const Segment SGI213637[] = {{&S78800[44], 10}, {NULL}};
const Segment SGI213638[] = {{&S78800[44], 22}, {NULL}};
const Segment SGI213639[] = {{&S78800[30], 6}, {NULL}};
const Segment SGI213640[] = {{&S78800[115], 3}, {NULL}};
const Segment SGI213641[] = {{&S78800[80], 6}, {NULL}};
const Segment SGI213642[] = {{&S88906[34], 24}, {NULL}};
const Segment SGI213643[] = {{&S88906[27], 6}, {NULL}};
const Segment SGI213644[] = {{&S83713[34], 21}, {NULL}};
const Segment SGI213645[] = {{&S83713[27], 6}, {NULL}};
const Segment SGI213646[] = {{&S88906[111], 9}, {NULL}};
const Segment SGI213647[] = {{&S88906[84], 6}, {NULL}};
const Segment SGI213648[] = {{&S83713[108], 9}, {NULL}};
const Segment SGI213649[] = {{&S83713[81], 6}, {NULL}};
const Segment SGI213650[] = {{&S88907[34], 15}, {NULL}};
const Segment SGI213651[] = {{&S88907[27], 6}, {NULL}};
const Segment SGI213652[] = {{&S88907[102], 9}, {NULL}};
const Segment SGI213653[] = {{&S88907[75], 6}, {NULL}};
const Segment SGI213654[] = {{&S83718[44], 10}, {NULL}};
const Segment SGI213655[] = {{&S83718[44], 22}, {NULL}};
const Segment SGI213656[] = {{&S83718[30], 6}, {NULL}};
const Segment SGI213657[] = {{&S83718[113], 3}, {NULL}};
const Segment SGI213658[] = {{&S83718[80], 5}, {NULL}};
const Segment SGI213659[] = {{&S83720[34], 6}, {NULL}};
const Segment SGI213660[] = {{&S83720[27], 6}, {NULL}};
const Segment SGI213661[] = {{&S83720[93], 9}, {NULL}};
const Segment SGI213662[] = {{&S83720[66], 6}, {NULL}};
const Segment SGI213663[] = {{&S88913[44], 15}, {NULL}};
const Segment SGI213664[] = {{&S88913[44], 27}, {NULL}};
const Segment SGI213665[] = {{&S88913[30], 6}, {NULL}};
const Segment SGI213666[] = {{&S88913[85], 6}, {NULL}};
const Segment SGI213667[] = {{&S88913[112], 13}, {NULL}};
const Segment SGI213668[] = {{&S88914[44], 8}, {NULL}};
const Segment SGI213669[] = {{&S88914[44], 19}, {NULL}};
const Segment SGI213670[] = {{&S88914[30], 6}, {NULL}};
const Segment SGI213671[] = {{&S88914[31], 3}, {NULL}};
const Segment SGI213672[] = {{&S88914[77], 6}, {NULL}};
const Segment SGI213673[] = {{&S88915[44], 8}, {NULL}};
const Segment SGI213674[] = {{&S88915[44], 19}, {NULL}};
const Segment SGI213675[] = {{&S88915[30], 6}, {NULL}};
const Segment SGI213676[] = {{&S88915[31], 3}, {NULL}};
const Segment SGI213677[] = {{&S88915[77], 6}, {NULL}};
const Segment SGI213678[] = {{&S78820[34], 17}, {NULL}};
const Segment SGI213679[] = {{&S78820[27], 6}, {NULL}};
const Segment SGI213680[] = {{&S78820[104], 9}, {NULL}};
const Segment SGI213681[] = {{&S78820[77], 6}, {NULL}};
const Segment SGI213682[] = {{&S88921[44], 14}, {NULL}};
const Segment SGI213683[] = {{&S88921[44], 26}, {NULL}};
const Segment SGI213684[] = {{&S88921[30], 6}, {NULL}};
const Segment SGI213685[] = {{&S69915[44], 9}, {NULL}};
const Segment SGI213686[] = {{&S69915[44], 21}, {NULL}};
const Segment SGI213687[] = {{&S88921[31], 3}, {NULL}};
const Segment SGI213688[] = {{&S83734[34], 13}, {NULL}};
const Segment SGI213689[] = {{&S69915[30], 6}, {NULL}};
const Segment SGI213690[] = {{&S83734[27], 6}, {NULL}};
const Segment SGI213691[] = {{&S88921[84], 6}, {NULL}};
const Segment SGI213692[] = {{&S69915[114], 3}, {NULL}};
const Segment SGI213693[] = {{&S83734[100], 9}, {NULL}};
const Segment SGI213694[] = {{&S69915[79], 6}, {NULL}};
const Segment SGI213695[] = {{&S83734[73], 6}, {NULL}};
const Segment SGI213696[] = {{&S69915[139], 8}, {NULL}};
const Segment SGI213697[] = {{&S78828[34], 10}, {NULL}};
const Segment SGI213698[] = {{&S78828[27], 6}, {NULL}};
const Segment SGI213699[] = {{&S78828[97], 9}, {NULL}};
const Segment SGI213700[] = {{&S78828[70], 6}, {NULL}};
const Segment SGI213701[] = {{&S78829[34], 10}, {NULL}};
const Segment SGI213702[] = {{&S78829[27], 6}, {NULL}};
const Segment SGI213703[] = {{&S78829[97], 9}, {NULL}};
const Segment SGI213704[] = {{&S78829[70], 6}, {NULL}};
const Segment SGI213705[] = {{&S88929[44], 13}, {NULL}};
const Segment SGI213706[] = {{&S88929[44], 25}, {NULL}};
const Segment SGI213707[] = {{&S88929[30], 6}, {NULL}};
const Segment SGI213708[] = {{&S88929[83], 6}, {NULL}};
const Segment SGI213709[] = {{&S88929[110], 13}, {NULL}};
const Segment SGI213710[] = {{&S88934[44], 15}, {NULL}};
const Segment SGI213711[] = {{&S88934[30], 6}, {NULL}};
const Segment SGI213712[] = {{&S88934[31], 3}, {NULL}};
const Segment SGI213713[] = {{&S88934[87], 6}, {NULL}};
const Segment SGI213714[] = {{&S88938[34], 12}, {NULL}};
const Segment SGI213715[] = {{&S88938[27], 6}, {NULL}};
const Segment SGI213716[] = {{&S88938[99], 9}, {NULL}};
const Segment SGI213717[] = {{&S88938[72], 6}, {NULL}};
const Segment SGI213718[] = {{&S78847[34], 15}, {NULL}};
const Segment SGI213719[] = {{&S78847[27], 6}, {NULL}};
const Segment SGI213720[] = {{&S78847[102], 9}, {NULL}};
const Segment SGI213721[] = {{&S78847[75], 6}, {NULL}};
const Segment SGI213722[] = {{&S78847[143], 8}, {NULL}};
const Segment SGI213723[] = {{&S78849[34], 17}, {NULL}};
const Segment SGI213724[] = {{&S78849[27], 6}, {NULL}};
const Segment SGI213725[] = {{&S78849[104], 9}, {NULL}};
const Segment SGI213726[] = {{&S78849[77], 6}, {NULL}};
const Segment SGI213727[] = {{&S88943[44], 5}, {NULL}};
const Segment SGI213728[] = {{&S88943[44], 17}, {NULL}};
const Segment SGI213729[] = {{&S88943[30], 6}, {NULL}};
const Segment SGI213730[] = {{&S88943[75], 6}, {NULL}};
const Segment SGI213731[] = {{&S88943[102], 13}, {NULL}};
const Segment SGI213732[] = {{&S88946[34], 11}, {NULL}};
const Segment SGI213733[] = {{&S88946[27], 6}, {NULL}};
const Segment SGI213734[] = {{&S88946[98], 9}, {NULL}};
const Segment SGI213735[] = {{&S88946[71], 6}, {NULL}};
const Segment SGI213736[] = {{&S78859[34], 8}, {NULL}};
const Segment SGI213737[] = {{&S78859[27], 6}, {NULL}};
const Segment SGI213738[] = {{&S78859[94], 9}, {NULL}};
const Segment SGI213739[] = {{&S78859[67], 6}, {NULL}};
const Segment SGI213740[] = {{&S78860[44], 15}, {NULL}};
const Segment SGI213741[] = {{&S78860[44], 27}, {NULL}};
const Segment SGI213742[] = {{&S78860[30], 6}, {NULL}};
const Segment SGI213743[] = {{&S78860[85], 6}, {NULL}};
const Segment SGI213744[] = {{&S78860[112], 13}, {NULL}};
const Segment SGI213745[] = {{&S78861[34], 7}, {NULL}};
const Segment SGI213746[] = {{&S78861[27], 6}, {NULL}};
const Segment SGI213747[] = {{&S78861[99], 9}, {NULL}};
const Segment SGI213748[] = {{&S78861[72], 6}, {NULL}};
const Segment SGI213749[] = {{&S83777[34], 6}, {NULL}};
const Segment SGI213750[] = {{&S83777[27], 6}, {NULL}};
const Segment SGI213751[] = {{&S83777[93], 9}, {NULL}};
const Segment SGI213752[] = {{&S83777[66], 6}, {NULL}};
const Segment SGI213753[] = {{&S83784[44], 11}, {NULL}};
const Segment SGI213754[] = {{&S83784[44], 23}, {NULL}};
const Segment SGI213755[] = {{&S83784[30], 6}, {NULL}};
const Segment SGI213756[] = {{&S83784[116], 3}, {NULL}};
const Segment SGI213757[] = {{&S83784[81], 6}, {NULL}};
const Segment SGI213758[] = {{&S83785[44], 11}, {NULL}};
const Segment SGI213759[] = {{&S83785[44], 23}, {NULL}};
const Segment SGI213760[] = {{&S83785[30], 6}, {NULL}};
const Segment SGI213761[] = {{&S83785[116], 3}, {NULL}};
const Segment SGI213762[] = {{&S83785[81], 6}, {NULL}};
const Segment SGI213763[] = {{&S83792[34], 25}, {NULL}};
const Segment SGI213764[] = {{&S83792[27], 6}, {NULL}};
const Segment SGI213765[] = {{&S83792[112], 9}, {NULL}};
const Segment SGI213766[] = {{&S83792[85], 6}, {NULL}};
const Segment SGI213767[] = {{&S78823[44], 12}, {NULL}};
const Segment SGI213768[] = {{&S78823[44], 24}, {NULL}};
const Segment SGI213769[] = {{&S78823[30], 6}, {NULL}};
const Segment SGI213770[] = {{&S78823[117], 3}, {NULL}};
const Segment SGI213771[] = {{&S78823[82], 6}, {NULL}};
const Segment SGI213772[] = {{&S88993[34], 9}, {NULL}};
const Segment SGI213773[] = {{&S88993[27], 6}, {NULL}};
const Segment SGI213774[] = {{&S88993[96], 9}, {NULL}};
const Segment SGI213775[] = {{&S88993[69], 6}, {NULL}};
const Segment SGI213776[] = {{&S69989[44], 10}, {NULL}};
const Segment SGI213777[] = {{&S69989[44], 22}, {NULL}};
const Segment SGI213778[] = {{&S69989[30], 6}, {NULL}};
const Segment SGI213779[] = {{&S88995[34], 7}, {NULL}};
const Segment SGI213780[] = {{&S88995[27], 6}, {NULL}};
const Segment SGI213781[] = {{&S69989[115], 3}, {NULL}};
const Segment SGI213782[] = {{&S69989[80], 6}, {NULL}};
const Segment SGI213783[] = {{&S88995[94], 9}, {NULL}};
const Segment SGI213784[] = {{&S69989[140], 8}, {NULL}};
const Segment SGI213785[] = {{&S88995[67], 6}, {NULL}};
const Segment SGI213786[] = {{&S78908[34], 10}, {NULL}};
const Segment SGI213787[] = {{&S78908[27], 6}, {NULL}};
const Segment SGI213788[] = {{&S78908[97], 9}, {NULL}};
const Segment SGI213789[] = {{&S78908[70], 6}, {NULL}};
const Segment SGI213790[] = {{&S78911[34], 10}, {NULL}};
const Segment SGI213791[] = {{&S78911[27], 6}, {NULL}};
const Segment SGI213792[] = {{&S78911[97], 9}, {NULL}};
const Segment SGI213793[] = {{&S78911[70], 6}, {NULL}};
const Segment SGI213794[] = {{&S78912[34], 10}, {NULL}};
const Segment SGI213795[] = {{&S78912[27], 6}, {NULL}};
const Segment SGI213796[] = {{&S78912[97], 9}, {NULL}};
const Segment SGI213797[] = {{&S78912[70], 6}, {NULL}};
const Segment SGI213798[] = {{&S78913[34], 10}, {NULL}};
const Segment SGI213799[] = {{&S78913[27], 6}, {NULL}};
const Segment SGI213800[] = {{&S78913[97], 9}, {NULL}};
const Segment SGI213801[] = {{&S78913[70], 6}, {NULL}};
const Segment SGI213802[] = {{&S78915[44], 10}, {NULL}};
const Segment SGI213803[] = {{&S78915[44], 22}, {NULL}};
const Segment SGI213804[] = {{&S78915[30], 6}, {NULL}};
const Segment SGI213805[] = {{&S78915[115], 3}, {NULL}};
const Segment SGI213806[] = {{&S78915[80], 6}, {NULL}};
const Segment SGI213807[] = {{&S78916[44], 17}, {NULL}};
const Segment SGI213808[] = {{&S78916[44], 29}, {NULL}};
const Segment SGI213809[] = {{&S78916[30], 6}, {NULL}};
const Segment SGI213810[] = {{&S78916[122], 3}, {NULL}};
const Segment SGI213811[] = {{&S78916[87], 6}, {NULL}};
const Segment SGI213812[] = {{&S89015[44], 10}, {NULL}};
const Segment SGI213813[] = {{&S89015[44], 22}, {NULL}};
const Segment SGI213814[] = {{&S89015[30], 6}, {NULL}};
const Segment SGI213815[] = {{&S89015[31], 3}, {NULL}};
const Segment SGI213816[] = {{&S89015[80], 6}, {NULL}};
const Segment SGI213817[] = {{&S89018[34], 10}, {NULL}};
const Segment SGI213818[] = {{&S89018[27], 6}, {NULL}};
const Segment SGI213819[] = {{&S89018[97], 9}, {NULL}};
const Segment SGI213820[] = {{&S89018[70], 6}, {NULL}};
const Segment SGI213821[] = {{&S89019[44], 11}, {NULL}};
const Segment SGI213822[] = {{&S89019[44], 23}, {NULL}};
const Segment SGI213823[] = {{&S89019[30], 6}, {NULL}};
const Segment SGI213824[] = {{&S89019[31], 3}, {NULL}};
const Segment SGI213825[] = {{&S89019[100], 6}, {NULL}};
const Segment SGI213826[] = {{&S89019[69], 8}, {NULL}};
const Segment SGI213827[] = {{&S89020[44], 11}, {NULL}};
const Segment SGI213828[] = {{&S89020[44], 23}, {NULL}};
const Segment SGI213829[] = {{&S89020[30], 6}, {NULL}};
const Segment SGI213830[] = {{&S89020[31], 3}, {NULL}};
const Segment SGI213831[] = {{&S89020[96], 6}, {NULL}};
const Segment SGI213832[] = {{&S89020[69], 8}, {NULL}};
const Segment SGI213833[] = {{&S83854[34], 6}, {NULL}};
const Segment SGI213834[] = {{&S83854[27], 6}, {NULL}};
const Segment SGI213835[] = {{&S83854[93], 9}, {NULL}};
const Segment SGI213836[] = {{&S83854[66], 6}, {NULL}};
const Segment SGI213837[] = {{&S70030[34], 6}, {NULL}};
const Segment SGI213838[] = {{&S70030[27], 6}, {NULL}};
const Segment SGI213839[] = {{&S70030[92], 9}, {NULL}};
const Segment SGI213840[] = {{&S70030[65], 6}, {NULL}};
const Segment SGI213841[] = {{&S89030[34], 11}, {NULL}};
const Segment SGI213842[] = {{&S89030[27], 6}, {NULL}};
const Segment SGI213843[] = {{&S89030[98], 9}, {NULL}};
const Segment SGI213844[] = {{&S89030[71], 6}, {NULL}};
const Segment SGI213845[] = {{&S89033[34], 7}, {NULL}};
const Segment SGI213846[] = {{&S89033[27], 6}, {NULL}};
const Segment SGI213847[] = {{&S89033[94], 9}, {NULL}};
const Segment SGI213848[] = {{&S89033[67], 6}, {NULL}};
const Segment SGI213849[] = {{&S83869[44], 15}, {NULL}};
const Segment SGI213850[] = {{&S83869[44], 27}, {NULL}};
const Segment SGI213851[] = {{&S83869[30], 6}, {NULL}};
const Segment SGI213852[] = {{&S83869[120], 3}, {NULL}};
const Segment SGI213853[] = {{&S83869[85], 6}, {NULL}};
const Segment SGI213854[] = {{&S83869[145], 13}, {NULL}};
const Segment SGI213855[] = {{&S89041[34], 7}, {NULL}};
const Segment SGI213856[] = {{&S89041[27], 6}, {NULL}};
const Segment SGI213857[] = {{&S89041[94], 9}, {NULL}};
const Segment SGI213858[] = {{&S89041[67], 6}, {NULL}};
const Segment SGI213859[] = {{&S89042[34], 8}, {NULL}};
const Segment SGI213860[] = {{&S89042[27], 6}, {NULL}};
const Segment SGI213861[] = {{&S89042[95], 9}, {NULL}};
const Segment SGI213862[] = {{&S89042[68], 6}, {NULL}};
const Segment SGI213863[] = {{&S89043[61], 15}, {NULL}};
const Segment SGI213864[] = {{&S89043[61], 27}, {NULL}};
const Segment SGI213865[] = {{&S89043[47], 6}, {NULL}};
const Segment SGI213866[] = {{&S89043[48], 3}, {NULL}};
const Segment SGI213867[] = {{&S89043[102], 6}, {NULL}};
const Segment SGI213868[] = {{&S89043[0], 14}, {NULL}};
const Segment SGI213869[] = {{&S78960[34], 7}, {NULL}};
const Segment SGI213870[] = {{&S78960[27], 6}, {NULL}};
const Segment SGI213871[] = {{&S83877[34], 25}, {NULL}};
const Segment SGI213872[] = {{&S83877[27], 6}, {NULL}};
const Segment SGI213873[] = {{&S78960[94], 9}, {NULL}};
const Segment SGI213874[] = {{&S78960[67], 6}, {NULL}};
const Segment SGI213875[] = {{&S83877[112], 9}, {NULL}};
const Segment SGI213876[] = {{&S83877[85], 6}, {NULL}};
const Segment SGI213877[] = {{&S78961[34], 8}, {NULL}};
const Segment SGI213878[] = {{&S78961[27], 6}, {NULL}};
const Segment SGI213879[] = {{&S78961[95], 9}, {NULL}};
const Segment SGI213880[] = {{&S78961[68], 6}, {NULL}};
const Segment SGI213881[] = {{&S89050[34], 8}, {NULL}};
const Segment SGI213882[] = {{&S89050[27], 6}, {NULL}};
const Segment SGI213883[] = {{&S89050[95], 9}, {NULL}};
const Segment SGI213884[] = {{&S89050[68], 6}, {NULL}};
const Segment SGI213885[] = {{&S70062[34], 17}, {NULL}};
const Segment SGI213886[] = {{&S70062[27], 6}, {NULL}};
const Segment SGI213887[] = {{&S70062[103], 9}, {NULL}};
const Segment SGI213888[] = {{&S70062[76], 6}, {NULL}};
const Segment SGI213889[] = {{&S89053[34], 13}, {NULL}};
const Segment SGI213890[] = {{&S89053[27], 6}, {NULL}};
const Segment SGI213891[] = {{&S89053[100], 9}, {NULL}};
const Segment SGI213892[] = {{&S89053[73], 6}, {NULL}};
const Segment SGI213893[] = {{&S89055[34], 13}, {NULL}};
const Segment SGI213894[] = {{&S89055[27], 6}, {NULL}};
const Segment SGI213895[] = {{&S89055[100], 9}, {NULL}};
const Segment SGI213896[] = {{&S89055[73], 6}, {NULL}};
const Segment SGI213897[] = {{&S83892[34], 8}, {NULL}};
const Segment SGI213898[] = {{&S83892[27], 6}, {NULL}};
const Segment SGI213899[] = {{&S83892[95], 9}, {NULL}};
const Segment SGI213900[] = {{&S83892[68], 6}, {NULL}};
const Segment SGI213901[] = {{&S89060[34], 9}, {NULL}};
const Segment SGI213902[] = {{&S89060[27], 6}, {NULL}};
const Segment SGI213903[] = {{&S89060[96], 9}, {NULL}};
const Segment SGI213904[] = {{&S89060[69], 6}, {NULL}};
const Segment SGI213905[] = {{&S89060[137], 8}, {NULL}};
const Segment SGI213906[] = {{&S89066[34], 7}, {NULL}};
const Segment SGI213907[] = {{&S89066[27], 6}, {NULL}};
const Segment SGI213908[] = {{&S89066[99], 9}, {NULL}};
const Segment SGI213909[] = {{&S89066[72], 6}, {NULL}};
const Segment SGI213910[] = {{&S89069[44], 13}, {NULL}};
const Segment SGI213911[] = {{&S89069[44], 25}, {NULL}};
const Segment SGI213912[] = {{&S89069[30], 6}, {NULL}};
const Segment SGI213913[] = {{&S89069[31], 3}, {NULL}};
const Segment SGI213914[] = {{&S89069[83], 6}, {NULL}};
const Segment SGI213915[] = {{&S89077[34], 7}, {NULL}};
const Segment SGI213916[] = {{&S89077[27], 6}, {NULL}};
const Segment SGI213917[] = {{&S89077[94], 9}, {NULL}};
const Segment SGI213918[] = {{&S89077[67], 6}, {NULL}};
const Segment SGI213919[] = {{&S78948[44], 29}, {NULL}};
const Segment SGI213920[] = {{&S78948[44], 41}, {NULL}};
const Segment SGI213921[] = {{&S78948[30], 6}, {NULL}};
const Segment SGI213922[] = {{&S78948[152], 3}, {NULL}};
const Segment SGI213923[] = {{&S78948[117], 6}, {NULL}};
const Segment SGI213924[] = {{&S78948[87], 8}, {NULL}};
const Segment SGI213925[] = {{&S79005[34], 11}, {NULL}};
const Segment SGI213926[] = {{&S79005[27], 6}, {NULL}};
const Segment SGI213927[] = {{&S79005[98], 9}, {NULL}};
const Segment SGI213928[] = {{&S79005[71], 6}, {NULL}};
const Segment SGI213929[] = {{&S89085[34], 10}, {NULL}};
const Segment SGI213930[] = {{&S89085[27], 6}, {NULL}};
const Segment SGI213931[] = {{&S89085[97], 9}, {NULL}};
const Segment SGI213932[] = {{&S89085[70], 6}, {NULL}};
const Segment SGI213933[] = {{&S78956[44], 14}, {NULL}};
const Segment SGI213934[] = {{&S78956[44], 26}, {NULL}};
const Segment SGI213935[] = {{&S78956[30], 6}, {NULL}};
const Segment SGI213936[] = {{&S89086[34], 10}, {NULL}};
const Segment SGI213937[] = {{&S89086[27], 6}, {NULL}};
const Segment SGI213938[] = {{&S78956[84], 6}, {NULL}};
const Segment SGI213939[] = {{&S78956[121], 3}, {NULL}};
const Segment SGI213940[] = {{&S89086[97], 9}, {NULL}};
const Segment SGI213941[] = {{&S89086[70], 6}, {NULL}};
const Segment SGI213942[] = {{&S83933[34], 10}, {NULL}};
const Segment SGI213943[] = {{&S83933[27], 6}, {NULL}};
const Segment SGI213944[] = {{&S83933[97], 9}, {NULL}};
const Segment SGI213945[] = {{&S83933[70], 6}, {NULL}};
const Segment SGI213946[] = {{&S79013[44], 12}, {NULL}};
const Segment SGI213947[] = {{&S79013[44], 24}, {NULL}};
const Segment SGI213948[] = {{&S79013[30], 6}, {NULL}};
const Segment SGI213949[] = {{&S79013[117], 3}, {NULL}};
const Segment SGI213950[] = {{&S79013[82], 6}, {NULL}};
const Segment SGI213951[] = {{&S89090[44], 14}, {NULL}};
const Segment SGI213952[] = {{&S89090[44], 26}, {NULL}};
const Segment SGI213953[] = {{&S89090[30], 6}, {NULL}};
const Segment SGI213954[] = {{&S89090[84], 6}, {NULL}};
const Segment SGI213955[] = {{&S89090[111], 13}, {NULL}};
const Segment SGI213956[] = {{&S70115[44], 10}, {NULL}};
const Segment SGI213957[] = {{&S70115[44], 22}, {NULL}};
const Segment SGI213958[] = {{&S70115[30], 6}, {NULL}};
const Segment SGI213959[] = {{&S70115[115], 3}, {NULL}};
const Segment SGI213960[] = {{&S70115[80], 6}, {NULL}};
const Segment SGI213961[] = {{&S70115[140], 8}, {NULL}};
const Segment SGI213962[] = {{&S89092[44], 10}, {NULL}};
const Segment SGI213963[] = {{&S89092[44], 22}, {NULL}};
const Segment SGI213964[] = {{&S89092[30], 6}, {NULL}};
const Segment SGI213965[] = {{&S89092[31], 3}, {NULL}};
const Segment SGI213966[] = {{&S89092[80], 5}, {NULL}};
const Segment SGI213967[] = {{&S89092[117], 14}, {NULL}};
const Segment SGI213968[] = {{&S83946[34], 8}, {NULL}};
const Segment SGI213969[] = {{&S83946[27], 6}, {NULL}};
const Segment SGI213970[] = {{&S83946[95], 9}, {NULL}};
const Segment SGI213971[] = {{&S83946[68], 6}, {NULL}};
const Segment SGI213972[] = {{&S70130[34], 18}, {NULL}};
const Segment SGI213973[] = {{&S70130[27], 6}, {NULL}};
const Segment SGI213974[] = {{&S70130[104], 9}, {NULL}};
const Segment SGI213975[] = {{&S70130[77], 6}, {NULL}};
const Segment SGI213976[] = {{&S70131[34], 18}, {NULL}};
const Segment SGI213977[] = {{&S70131[27], 6}, {NULL}};
const Segment SGI213978[] = {{&S70131[104], 9}, {NULL}};
const Segment SGI213979[] = {{&S70131[77], 6}, {NULL}};
const Segment SGI213980[] = {{&S83959[34], 13}, {NULL}};
const Segment SGI213981[] = {{&S83959[27], 6}, {NULL}};
const Segment SGI213982[] = {{&S83959[100], 9}, {NULL}};
const Segment SGI213983[] = {{&S83959[73], 6}, {NULL}};
const Segment SGI213984[] = {{&S79060[34], 14}, {NULL}};
const Segment SGI213985[] = {{&S79060[27], 6}, {NULL}};
const Segment SGI213986[] = {{&S79060[106], 9}, {NULL}};
const Segment SGI213987[] = {{&S79060[79], 6}, {NULL}};
const Segment SGI213988[] = {{&S79061[34], 8}, {NULL}};
const Segment SGI213989[] = {{&S79061[27], 6}, {NULL}};
const Segment SGI213990[] = {{&S79061[94], 9}, {NULL}};
const Segment SGI213991[] = {{&S79061[67], 6}, {NULL}};
const Segment SGI213992[] = {{&S79062[34], 15}, {NULL}};
const Segment SGI213993[] = {{&S79062[27], 6}, {NULL}};
const Segment SGI213994[] = {{&S79062[102], 9}, {NULL}};
const Segment SGI213995[] = {{&S79062[75], 6}, {NULL}};
const Segment SGI213996[] = {{&S79063[34], 11}, {NULL}};
const Segment SGI213997[] = {{&S79063[27], 6}, {NULL}};
const Segment SGI213998[] = {{&S79063[98], 9}, {NULL}};
const Segment SGI213999[] = {{&S79063[71], 6}, {NULL}};
const Segment SGI214000[] = {{&S89142[44], 9}, {NULL}};
const Segment SGI214001[] = {{&S89142[44], 21}, {NULL}};
const Segment SGI214002[] = {{&S89142[30], 6}, {NULL}};
const Segment SGI214003[] = {{&S89142[31], 3}, {NULL}};
const Segment SGI214004[] = {{&S89142[79], 6}, {NULL}};
const Segment SGI214005[] = {{&S89143[44], 9}, {NULL}};
const Segment SGI214006[] = {{&S89143[44], 21}, {NULL}};
const Segment SGI214007[] = {{&S89143[30], 6}, {NULL}};
const Segment SGI214008[] = {{&S89143[31], 3}, {NULL}};
const Segment SGI214009[] = {{&S89143[79], 6}, {NULL}};
const Segment SGI214010[] = {{&S89146[59], 10}, {NULL}};
const Segment SGI214011[] = {{&S89146[59], 22}, {NULL}};
const Segment SGI214012[] = {{&S89146[45], 6}, {NULL}};
const Segment SGI214013[] = {{&S89146[46], 3}, {NULL}};
const Segment SGI214014[] = {{&S89146[95], 6}, {NULL}};
const Segment SGI214015[] = {{&S89146[0], 14}, {NULL}};
const Segment SGI214016[] = {{&S79070[44], 6}, {NULL}};
const Segment SGI214017[] = {{&S79070[44], 18}, {NULL}};
const Segment SGI214018[] = {{&S79070[30], 6}, {NULL}};
const Segment SGI214019[] = {{&S79070[111], 3}, {NULL}};
const Segment SGI214020[] = {{&S79070[76], 6}, {NULL}};
const Segment SGI214021[] = {{&S89156[34], 8}, {NULL}};
const Segment SGI214022[] = {{&S89156[27], 6}, {NULL}};
const Segment SGI214023[] = {{&S89156[95], 9}, {NULL}};
const Segment SGI214024[] = {{&S89156[68], 6}, {NULL}};
const Segment SGI214025[] = {{&S70186[34], 8}, {NULL}};
const Segment SGI214026[] = {{&S70186[27], 6}, {NULL}};
const Segment SGI214027[] = {{&S84011[59], 10}, {NULL}};
const Segment SGI214028[] = {{&S84011[59], 22}, {NULL}};
const Segment SGI214029[] = {{&S84011[45], 6}, {NULL}};
const Segment SGI214030[] = {{&S70186[94], 9}, {NULL}};
const Segment SGI214031[] = {{&S70186[67], 6}, {NULL}};
const Segment SGI214032[] = {{&S84011[130], 3}, {NULL}};
const Segment SGI214033[] = {{&S84011[95], 6}, {NULL}};
const Segment SGI214034[] = {{&S84011[0], 14}, {NULL}};
const Segment SGI214035[] = {{&S89158[27], 6}, {NULL}};
const Segment SGI214036[] = {{&S89158[103], 9}, {NULL}};
const Segment SGI214037[] = {{&S89158[76], 6}, {NULL}};
const Segment SGI214038[] = {{&S89160[44], 13}, {NULL}};
const Segment SGI214039[] = {{&S89160[44], 37}, {NULL}};
const Segment SGI214040[] = {{&S89160[30], 6}, {NULL}};
const Segment SGI214041[] = {{&S89160[31], 3}, {NULL}};
const Segment SGI214042[] = {{&S89160[95], 6}, {NULL}};
const Segment SGI214043[] = {{&S89161[44], 8}, {NULL}};
const Segment SGI214044[] = {{&S89161[44], 20}, {NULL}};
const Segment SGI214045[] = {{&S89161[30], 6}, {NULL}};
const Segment SGI214046[] = {{&S89161[31], 3}, {NULL}};
const Segment SGI214047[] = {{&S89161[78], 6}, {NULL}};
const Segment SGI214048[] = {{&S84021[34], 14}, {NULL}};
const Segment SGI214049[] = {{&S89163[34], 8}, {NULL}};
const Segment SGI214050[] = {{&S84021[27], 6}, {NULL}};
const Segment SGI214051[] = {{&S89163[27], 6}, {NULL}};
const Segment SGI214052[] = {{&S84021[101], 9}, {NULL}};
const Segment SGI214053[] = {{&S84021[74], 6}, {NULL}};
const Segment SGI214054[] = {{&S89163[100], 9}, {NULL}};
const Segment SGI214055[] = {{&S89163[73], 6}, {NULL}};
const Segment SGI214056[] = {{&S89165[34], 9}, {NULL}};
const Segment SGI214057[] = {{&S79096[44], 13}, {NULL}};
const Segment SGI214058[] = {{&S89165[27], 6}, {NULL}};
const Segment SGI214059[] = {{&S79096[44], 25}, {NULL}};
const Segment SGI214060[] = {{&S79096[30], 6}, {NULL}};
const Segment SGI214061[] = {{&S89165[96], 9}, {NULL}};
const Segment SGI214062[] = {{&S79096[134], 3}, {NULL}};
const Segment SGI214063[] = {{&S89165[69], 6}, {NULL}};
const Segment SGI214064[] = {{&S79096[99], 6}, {NULL}};
const Segment SGI214065[] = {{&S79096[71], 8}, {NULL}};
const Segment SGI214066[] = {{&S79097[44], 12}, {NULL}};
const Segment SGI214067[] = {{&S79097[44], 24}, {NULL}};
const Segment SGI214068[] = {{&S79097[30], 6}, {NULL}};
const Segment SGI214069[] = {{&S79097[133], 3}, {NULL}};
const Segment SGI214070[] = {{&S79097[98], 6}, {NULL}};
const Segment SGI214071[] = {{&S79097[70], 8}, {NULL}};
const Segment SGI214072[] = {{&S84028[34], 10}, {NULL}};
const Segment SGI214073[] = {{&S84028[27], 6}, {NULL}};
const Segment SGI214074[] = {{&S84028[102], 9}, {NULL}};
const Segment SGI214075[] = {{&S84028[75], 6}, {NULL}};
const Segment SGI214076[] = {{&S84033[34], 13}, {NULL}};
const Segment SGI214077[] = {{&S84033[27], 6}, {NULL}};
const Segment SGI214078[] = {{&S84033[100], 9}, {NULL}};
const Segment SGI214079[] = {{&S84033[73], 6}, {NULL}};
const Segment SGI214080[] = {{&S89175[44], 9}, {NULL}};
const Segment SGI214081[] = {{&S89175[44], 21}, {NULL}};
const Segment SGI214082[] = {{&S89175[30], 6}, {NULL}};
const Segment SGI214083[] = {{&S89175[31], 3}, {NULL}};
const Segment SGI214084[] = {{&S89175[79], 6}, {NULL}};
const Segment SGI214085[] = {{&S70211[44], 12}, {NULL}};
const Segment SGI214086[] = {{&S70211[44], 23}, {NULL}};
const Segment SGI214087[] = {{&S70211[30], 6}, {NULL}};
const Segment SGI214088[] = {{&S70211[134], 3}, {NULL}};
const Segment SGI214089[] = {{&S70211[99], 6}, {NULL}};
const Segment SGI214090[] = {{&S70211[69], 8}, {NULL}};
const Segment SGI214091[] = {{&S89177[34], 14}, {NULL}};
const Segment SGI214092[] = {{&S70212[44], 12}, {NULL}};
const Segment SGI214093[] = {{&S89177[27], 6}, {NULL}};
const Segment SGI214094[] = {{&S70212[44], 24}, {NULL}};
const Segment SGI214095[] = {{&S70212[30], 6}, {NULL}};
const Segment SGI214096[] = {{&S89177[101], 9}, {NULL}};
const Segment SGI214097[] = {{&S70212[135], 3}, {NULL}};
const Segment SGI214098[] = {{&S89177[74], 6}, {NULL}};
const Segment SGI214099[] = {{&S79109[34], 9}, {NULL}};
const Segment SGI214100[] = {{&S70212[100], 6}, {NULL}};
const Segment SGI214101[] = {{&S79109[27], 6}, {NULL}};
const Segment SGI214102[] = {{&S70212[70], 8}, {NULL}};
const Segment SGI214103[] = {{&S79109[96], 9}, {NULL}};
const Segment SGI214104[] = {{&S79109[69], 6}, {NULL}};
const Segment SGI214105[] = {{&S79109[137], 8}, {NULL}};
const Segment SGI214106[] = {{&S89181[34], 8}, {NULL}};
const Segment SGI214107[] = {{&S89181[27], 6}, {NULL}};
const Segment SGI214108[] = {{&S84042[34], 23}, {NULL}};
const Segment SGI214109[] = {{&S84042[27], 6}, {NULL}};
const Segment SGI214110[] = {{&S89181[101], 9}, {NULL}};
const Segment SGI214111[] = {{&S89181[74], 6}, {NULL}};
const Segment SGI214112[] = {{&S84042[110], 9}, {NULL}};
const Segment SGI214113[] = {{&S84042[83], 6}, {NULL}};
const Segment SGI214114[] = {{&S79114[34], 10}, {NULL}};
const Segment SGI214115[] = {{&S79114[27], 6}, {NULL}};
const Segment SGI214116[] = {{&S79114[97], 9}, {NULL}};
const Segment SGI214117[] = {{&S79114[70], 6}, {NULL}};
const Segment SGI214118[] = {{&S70219[34], 10}, {NULL}};
const Segment SGI214119[] = {{&S70219[27], 6}, {NULL}};
const Segment SGI214120[] = {{&S70219[97], 9}, {NULL}};
const Segment SGI214121[] = {{&S70219[70], 6}, {NULL}};
const Segment SGI214122[] = {{&S79115[34], 10}, {NULL}};
const Segment SGI214123[] = {{&S79115[27], 6}, {NULL}};
const Segment SGI214124[] = {{&S79115[97], 9}, {NULL}};
const Segment SGI214125[] = {{&S79115[70], 6}, {NULL}};
const Segment SGI214126[] = {{&S79116[34], 9}, {NULL}};
const Segment SGI214127[] = {{&S79116[27], 6}, {NULL}};
const Segment SGI214128[] = {{&S79116[96], 9}, {NULL}};
const Segment SGI214129[] = {{&S79116[69], 6}, {NULL}};
const Segment SGI214130[] = {{&S89189[44], 7}, {NULL}};
const Segment SGI214131[] = {{&S89189[44], 24}, {NULL}};
const Segment SGI214132[] = {{&S89189[30], 6}, {NULL}};
const Segment SGI214133[] = {{&S89189[82], 6}, {NULL}};
const Segment SGI214134[] = {{&S89189[109], 13}, {NULL}};
const Segment SGI214135[] = {{&S84052[34], 25}, {NULL}};
const Segment SGI214136[] = {{&S84052[27], 6}, {NULL}};
const Segment SGI214137[] = {{&S84052[112], 9}, {NULL}};
const Segment SGI214138[] = {{&S84052[85], 6}, {NULL}};
const Segment SGI214139[] = {{&S84052[146], 8}, {NULL}};
const Segment SGI214140[] = {{&S89199[44], 7}, {NULL}};
const Segment SGI214141[] = {{&S89199[44], 19}, {NULL}};
const Segment SGI214142[] = {{&S89199[30], 6}, {NULL}};
const Segment SGI214143[] = {{&S89199[31], 3}, {NULL}};
const Segment SGI214144[] = {{&S89199[77], 6}, {NULL}};
const Segment SGI214145[] = {{&S89201[44], 15}, {NULL}};
const Segment SGI214146[] = {{&S89201[44], 33}, {NULL}};
const Segment SGI214147[] = {{&S70243[34], 10}, {NULL}};
const Segment SGI214148[] = {{&S89201[30], 6}, {NULL}};
const Segment SGI214149[] = {{&S70243[27], 6}, {NULL}};
const Segment SGI214150[] = {{&S89201[91], 6}, {NULL}};
const Segment SGI214151[] = {{&S70243[96], 9}, {NULL}};
const Segment SGI214152[] = {{&S70243[69], 6}, {NULL}};
const Segment SGI214153[] = {{&S89201[118], 13}, {NULL}};
const Segment SGI214154[] = {{&S89202[44], 15}, {NULL}};
const Segment SGI214155[] = {{&S70244[34], 10}, {NULL}};
const Segment SGI214156[] = {{&S89202[44], 33}, {NULL}};
const Segment SGI214157[] = {{&S70244[27], 6}, {NULL}};
const Segment SGI214158[] = {{&S89202[30], 6}, {NULL}};
const Segment SGI214159[] = {{&S70244[97], 9}, {NULL}};
const Segment SGI214160[] = {{&S70244[70], 6}, {NULL}};
const Segment SGI214161[] = {{&S89202[91], 6}, {NULL}};
const Segment SGI214162[] = {{&S89202[118], 13}, {NULL}};
const Segment SGI214163[] = {{&S89209[34], 13}, {NULL}};
const Segment SGI214164[] = {{&S89209[27], 6}, {NULL}};
const Segment SGI214165[] = {{&S89209[100], 9}, {NULL}};
const Segment SGI214166[] = {{&S89209[73], 6}, {NULL}};
const Segment SGI214167[] = {{&S89212[44], 9}, {NULL}};
const Segment SGI214168[] = {{&S89212[44], 21}, {NULL}};
const Segment SGI214169[] = {{&S89212[30], 6}, {NULL}};
const Segment SGI214170[] = {{&S89212[79], 6}, {NULL}};
const Segment SGI214171[] = {{&S89212[106], 13}, {NULL}};
const Segment SGI214172[] = {{&S89213[44], 7}, {NULL}};
const Segment SGI214173[] = {{&S89213[44], 25}, {NULL}};
const Segment SGI214174[] = {{&S89213[30], 6}, {NULL}};
const Segment SGI214175[] = {{&S89213[83], 6}, {NULL}};
const Segment SGI214176[] = {{&S89213[110], 13}, {NULL}};
const Segment SGI214177[] = {{&S89216[34], 17}, {NULL}};
const Segment SGI214178[] = {{&S89216[27], 6}, {NULL}};
const Segment SGI214179[] = {{&S89216[111], 9}, {NULL}};
const Segment SGI214180[] = {{&S89216[84], 6}, {NULL}};
const Segment SGI214181[] = {{&S89217[34], 10}, {NULL}};
const Segment SGI214182[] = {{&S89217[27], 6}, {NULL}};
const Segment SGI214183[] = {{&S89217[97], 9}, {NULL}};
const Segment SGI214184[] = {{&S89217[70], 6}, {NULL}};
const Segment SGI214185[] = {{&S89254[34], 11}, {NULL}};
const Segment SGI214186[] = {{&S89254[27], 6}, {NULL}};
const Segment SGI214187[] = {{&S89254[98], 9}, {NULL}};
const Segment SGI214188[] = {{&S89254[71], 6}, {NULL}};
const Segment SGI214189[] = {{&S89260[34], 11}, {NULL}};
const Segment SGI214190[] = {{&S89260[27], 6}, {NULL}};
const Segment SGI214191[] = {{&S89260[98], 9}, {NULL}};
const Segment SGI214192[] = {{&S89260[71], 6}, {NULL}};
const Segment SGI214193[] = {{&S84144[34], 25}, {NULL}};
const Segment SGI214194[] = {{&S84144[27], 6}, {NULL}};
const Segment SGI214195[] = {{&S84144[112], 9}, {NULL}};
const Segment SGI214196[] = {{&S84144[85], 6}, {NULL}};
const Segment SGI214197[] = {{&S84146[44], 15}, {NULL}};
const Segment SGI214198[] = {{&S84146[44], 27}, {NULL}};
const Segment SGI214199[] = {{&S84146[30], 6}, {NULL}};
const Segment SGI214200[] = {{&S84146[120], 3}, {NULL}};
const Segment SGI214201[] = {{&S84146[85], 6}, {NULL}};
const Segment SGI214202[] = {{&S89267[59], 10}, {NULL}};
const Segment SGI214203[] = {{&S89267[59], 22}, {NULL}};
const Segment SGI214204[] = {{&S89267[45], 6}, {NULL}};
const Segment SGI214205[] = {{&S89267[95], 5}, {NULL}};
const Segment SGI214206[] = {{&S89267[0], 14}, {NULL}};
const Segment SGI214207[] = {{&S89268[44], 10}, {NULL}};
const Segment SGI214208[] = {{&S89268[44], 22}, {NULL}};
const Segment SGI214209[] = {{&S89268[30], 6}, {NULL}};
const Segment SGI214210[] = {{&S89268[31], 3}, {NULL}};
const Segment SGI214211[] = {{&S89268[80], 5}, {NULL}};
const Segment SGI214212[] = {{&S89268[125], 14}, {NULL}};
const Segment SGI214213[] = {{&S84150[34], 11}, {NULL}};
const Segment SGI214214[] = {{&S84150[27], 6}, {NULL}};
const Segment SGI214215[] = {{&S84150[98], 9}, {NULL}};
const Segment SGI214216[] = {{&S84150[71], 6}, {NULL}};
const Segment SGI214217[] = {{&S84150[132], 8}, {NULL}};
const Segment SGI214218[] = {{&S84151[34], 11}, {NULL}};
const Segment SGI214219[] = {{&S84151[27], 6}, {NULL}};
const Segment SGI214220[] = {{&S70323[34], 5}, {NULL}};
const Segment SGI214221[] = {{&S70323[27], 6}, {NULL}};
const Segment SGI214222[] = {{&S84151[98], 9}, {NULL}};
const Segment SGI214223[] = {{&S84151[71], 6}, {NULL}};
const Segment SGI214224[] = {{&S70323[92], 9}, {NULL}};
const Segment SGI214225[] = {{&S70323[65], 6}, {NULL}};
const Segment SGI214226[] = {{&S84151[139], 8}, {NULL}};
const Segment SGI214227[] = {{&S70324[34], 12}, {NULL}};
const Segment SGI214228[] = {{&S70324[27], 6}, {NULL}};
const Segment SGI214229[] = {{&S89273[34], 8}, {NULL}};
const Segment SGI214230[] = {{&S70324[99], 9}, {NULL}};
const Segment SGI214231[] = {{&S89273[27], 6}, {NULL}};
const Segment SGI214232[] = {{&S70324[72], 6}, {NULL}};
const Segment SGI214233[] = {{&S89273[100], 9}, {NULL}};
const Segment SGI214234[] = {{&S89273[73], 6}, {NULL}};
const Segment SGI214235[] = {{&S89274[34], 25}, {NULL}};
const Segment SGI214236[] = {{&S89274[27], 6}, {NULL}};
const Segment SGI214237[] = {{&S89274[112], 9}, {NULL}};
const Segment SGI214238[] = {{&S89274[85], 6}, {NULL}};
const Segment SGI214239[] = {{&S89275[34], 20}, {NULL}};
const Segment SGI214240[] = {{&S89275[27], 6}, {NULL}};
const Segment SGI214241[] = {{&S89275[107], 9}, {NULL}};
const Segment SGI214242[] = {{&S89275[80], 6}, {NULL}};
const Segment SGI214243[] = {{&S79065[34], 17}, {NULL}};
const Segment SGI214244[] = {{&S79065[27], 6}, {NULL}};
const Segment SGI214245[] = {{&S89278[34], 10}, {NULL}};
const Segment SGI214246[] = {{&S89278[27], 6}, {NULL}};
const Segment SGI214247[] = {{&S79065[104], 9}, {NULL}};
const Segment SGI214248[] = {{&S89278[97], 9}, {NULL}};
const Segment SGI214249[] = {{&S79065[77], 6}, {NULL}};
const Segment SGI214250[] = {{&S89278[70], 6}, {NULL}};
const Segment SGI214251[] = {{&S89280[45], 10}, {NULL}};
const Segment SGI214252[] = {{&S89280[30], 6}, {NULL}};
const Segment SGI214253[] = {{&S89280[31], 3}, {NULL}};
const Segment SGI214254[] = {{&S89280[81], 5}, {NULL}};
const Segment SGI214255[] = {{&S89280[139], 13}, {NULL}};
const Segment SGI214256[] = {{&S89284[34], 11}, {NULL}};
const Segment SGI214257[] = {{&S89284[27], 6}, {NULL}};
const Segment SGI214258[] = {{&S89284[98], 9}, {NULL}};
const Segment SGI214259[] = {{&S89284[71], 6}, {NULL}};
const Segment SGI214260[] = {{&S89286[34], 7}, {NULL}};
const Segment SGI214261[] = {{&S89286[27], 6}, {NULL}};
const Segment SGI214262[] = {{&S89286[99], 9}, {NULL}};
const Segment SGI214263[] = {{&S89286[72], 6}, {NULL}};
const Segment SGI214264[] = {{&S89287[34], 10}, {NULL}};
const Segment SGI214265[] = {{&S89287[27], 6}, {NULL}};
const Segment SGI214266[] = {{&S89287[97], 9}, {NULL}};
const Segment SGI214267[] = {{&S89287[70], 6}, {NULL}};
const Segment SGI214268[] = {{&S89289[34], 7}, {NULL}};
const Segment SGI214269[] = {{&S89289[27], 6}, {NULL}};
const Segment SGI214270[] = {{&S89289[94], 9}, {NULL}};
const Segment SGI214271[] = {{&S89289[67], 6}, {NULL}};
const Segment SGI214272[] = {{&S89290[34], 14}, {NULL}};
const Segment SGI214273[] = {{&S89290[27], 6}, {NULL}};
const Segment SGI214274[] = {{&S89290[114], 9}, {NULL}};
const Segment SGI214275[] = {{&S89290[87], 6}, {NULL}};
const Segment SGI214276[] = {{&S89291[34], 15}, {NULL}};
const Segment SGI214277[] = {{&S89291[27], 6}, {NULL}};
const Segment SGI214278[] = {{&S89291[99], 9}, {NULL}};
const Segment SGI214279[] = {{&S89291[72], 6}, {NULL}};
const Segment SGI214280[] = {{&S89299[44], 10}, {NULL}};
const Segment SGI214281[] = {{&S89299[44], 22}, {NULL}};
const Segment SGI214282[] = {{&S89299[30], 6}, {NULL}};
const Segment SGI214283[] = {{&S89299[31], 3}, {NULL}};
const Segment SGI214284[] = {{&S89299[80], 6}, {NULL}};
const Segment SGI214285[] = {{&S84200[44], 29}, {NULL}};
const Segment SGI214286[] = {{&S84200[44], 41}, {NULL}};
const Segment SGI214287[] = {{&S84200[30], 6}, {NULL}};
const Segment SGI214288[] = {{&S84200[134], 3}, {NULL}};
const Segment SGI214289[] = {{&S79101[44], 10}, {NULL}};
const Segment SGI214290[] = {{&S84200[99], 6}, {NULL}};
const Segment SGI214291[] = {{&S79101[44], 22}, {NULL}};
const Segment SGI214292[] = {{&S79101[30], 6}, {NULL}};
const Segment SGI214293[] = {{&S79101[115], 3}, {NULL}};
const Segment SGI214294[] = {{&S79101[80], 6}, {NULL}};
const Segment SGI214295[] = {{&S79102[38], 10}, {NULL}};
const Segment SGI214296[] = {{&S79102[31], 6}, {NULL}};
const Segment SGI214297[] = {{&S89307[44], 7}, {NULL}};
const Segment SGI214298[] = {{&S89307[44], 19}, {NULL}};
const Segment SGI214299[] = {{&S89307[30], 6}, {NULL}};
const Segment SGI214300[] = {{&S84204[44], 10}, {NULL}};
const Segment SGI214301[] = {{&S84204[44], 22}, {NULL}};
const Segment SGI214302[] = {{&S84204[30], 6}, {NULL}};
const Segment SGI214303[] = {{&S89307[31], 3}, {NULL}};
const Segment SGI214304[] = {{&S89307[77], 6}, {NULL}};
const Segment SGI214305[] = {{&S84204[131], 3}, {NULL}};
const Segment SGI214306[] = {{&S84204[96], 6}, {NULL}};
const Segment SGI214307[] = {{&S84204[68], 8}, {NULL}};
const Segment SGI214308[] = {{&S84205[44], 10}, {NULL}};
const Segment SGI214309[] = {{&S84205[44], 22}, {NULL}};
const Segment SGI214310[] = {{&S84205[30], 6}, {NULL}};
const Segment SGI214311[] = {{&S89309[34], 6}, {NULL}};
const Segment SGI214312[] = {{&S84205[131], 3}, {NULL}};
const Segment SGI214313[] = {{&S89309[27], 6}, {NULL}};
const Segment SGI214314[] = {{&S84205[96], 6}, {NULL}};
const Segment SGI214315[] = {{&S84205[68], 8}, {NULL}};
const Segment SGI214316[] = {{&S89309[93], 9}, {NULL}};
const Segment SGI214317[] = {{&S89309[66], 6}, {NULL}};
const Segment SGI214318[] = {{&S89310[34], 12}, {NULL}};
const Segment SGI214319[] = {{&S89310[27], 6}, {NULL}};
const Segment SGI214320[] = {{&S89310[99], 9}, {NULL}};
const Segment SGI214321[] = {{&S89310[72], 6}, {NULL}};
const Segment SGI214322[] = {{&S70384[44], 24}, {NULL}};
const Segment SGI214323[] = {{&S70384[44], 35}, {NULL}};
const Segment SGI214324[] = {{&S70384[30], 6}, {NULL}};
const Segment SGI214325[] = {{&S70384[146], 3}, {NULL}};
const Segment SGI214326[] = {{&S70384[111], 6}, {NULL}};
const Segment SGI214327[] = {{&S70384[81], 8}, {NULL}};
const Segment SGI214328[] = {{&S70385[44], 24}, {NULL}};
const Segment SGI214329[] = {{&S70385[44], 36}, {NULL}};
const Segment SGI214330[] = {{&S70385[30], 6}, {NULL}};
const Segment SGI214331[] = {{&S70385[147], 3}, {NULL}};
const Segment SGI214332[] = {{&S70385[112], 6}, {NULL}};
const Segment SGI214333[] = {{&S70385[82], 8}, {NULL}};
const Segment SGI214334[] = {{&S70386[44], 12}, {NULL}};
const Segment SGI214335[] = {{&S70386[44], 24}, {NULL}};
const Segment SGI214336[] = {{&S70386[30], 6}, {NULL}};
const Segment SGI214337[] = {{&S70386[117], 3}, {NULL}};
const Segment SGI214338[] = {{&S79119[44], 15}, {NULL}};
const Segment SGI214339[] = {{&S70386[82], 6}, {NULL}};
const Segment SGI214340[] = {{&S79119[44], 27}, {NULL}};
const Segment SGI214341[] = {{&S79119[30], 6}, {NULL}};
const Segment SGI214342[] = {{&S70386[135], 8}, {NULL}};
const Segment SGI214343[] = {{&S79119[85], 6}, {NULL}};
const Segment SGI214344[] = {{&S79119[122], 3}, {NULL}};
const Segment SGI214345[] = {{&S89324[34], 7}, {NULL}};
const Segment SGI214346[] = {{&S89324[27], 6}, {NULL}};
const Segment SGI214347[] = {{&S89324[94], 9}, {NULL}};
const Segment SGI214348[] = {{&S89324[67], 6}, {NULL}};
const Segment SGI214349[] = {{&S84223[44], 11}, {NULL}};
const Segment SGI214350[] = {{&S84223[44], 23}, {NULL}};
const Segment SGI214351[] = {{&S84223[30], 6}, {NULL}};
const Segment SGI214352[] = {{&S84223[116], 3}, {NULL}};
const Segment SGI214353[] = {{&S84223[81], 6}, {NULL}};
const Segment SGI214354[] = {{&S89328[34], 14}, {NULL}};
const Segment SGI214355[] = {{&S89328[27], 6}, {NULL}};
const Segment SGI214356[] = {{&S89328[101], 9}, {NULL}};
const Segment SGI214357[] = {{&S89328[74], 6}, {NULL}};
const Segment SGI214358[] = {{&S79157[34], 15}, {NULL}};
const Segment SGI214359[] = {{&S79157[27], 6}, {NULL}};
const Segment SGI214360[] = {{&S79157[102], 9}, {NULL}};
const Segment SGI214361[] = {{&S79157[75], 6}, {NULL}};
const Segment SGI214362[] = {{&S89329[44], 14}, {NULL}};
const Segment SGI214363[] = {{&S89329[44], 26}, {NULL}};
const Segment SGI214364[] = {{&S89329[30], 6}, {NULL}};
const Segment SGI214365[] = {{&S89329[31], 3}, {NULL}};
const Segment SGI214366[] = {{&S89329[84], 6}, {NULL}};
const Segment SGI214367[] = {{&S89332[34], 14}, {NULL}};
const Segment SGI214368[] = {{&S89332[27], 6}, {NULL}};
const Segment SGI214369[] = {{&S89332[106], 9}, {NULL}};
const Segment SGI214370[] = {{&S89332[79], 6}, {NULL}};
const Segment SGI214371[] = {{&S89334[34], 7}, {NULL}};
const Segment SGI214372[] = {{&S89334[27], 6}, {NULL}};
const Segment SGI214373[] = {{&S89334[99], 9}, {NULL}};
const Segment SGI214374[] = {{&S89334[72], 6}, {NULL}};
const Segment SGI214375[] = {{&S89334[140], 8}, {NULL}};
const Segment SGI214376[] = {{&S89337[34], 10}, {NULL}};
const Segment SGI214377[] = {{&S89337[27], 6}, {NULL}};
const Segment SGI214378[] = {{&S89337[97], 9}, {NULL}};
const Segment SGI214379[] = {{&S89337[70], 6}, {NULL}};
const Segment SGI214380[] = {{&S89338[34], 10}, {NULL}};
const Segment SGI214381[] = {{&S89338[27], 6}, {NULL}};
const Segment SGI214382[] = {{&S89338[97], 9}, {NULL}};
const Segment SGI214383[] = {{&S89338[70], 6}, {NULL}};
const Segment SGI214384[] = {{&S89343[44], 10}, {NULL}};
const Segment SGI214385[] = {{&S89343[44], 22}, {NULL}};
const Segment SGI214386[] = {{&S89343[30], 6}, {NULL}};
const Segment SGI214387[] = {{&S89343[31], 3}, {NULL}};
const Segment SGI214388[] = {{&S89343[80], 5}, {NULL}};
const Segment SGI214389[] = {{&S89343[117], 14}, {NULL}};
const Segment SGI214390[] = {{&S89349[52], 13}, {NULL}};
const Segment SGI214391[] = {{&S89349[52], 25}, {NULL}};
const Segment SGI214392[] = {{&S89349[38], 6}, {NULL}};
const Segment SGI214393[] = {{&S89349[91], 5}, {NULL}};
const Segment SGI214394[] = {{&S89349[0], 14}, {NULL}};
const Segment SGI214395[] = {{&S79161[44], 14}, {NULL}};
const Segment SGI214396[] = {{&S79161[44], 26}, {NULL}};
const Segment SGI214397[] = {{&S79161[30], 6}, {NULL}};
const Segment SGI214398[] = {{&S89350[34], 10}, {NULL}};
const Segment SGI214399[] = {{&S89350[27], 6}, {NULL}};
const Segment SGI214400[] = {{&S79161[119], 3}, {NULL}};
const Segment SGI214401[] = {{&S79161[84], 6}, {NULL}};
const Segment SGI214402[] = {{&S89350[97], 9}, {NULL}};
const Segment SGI214403[] = {{&S79161[144], 11}, {NULL}};
const Segment SGI214404[] = {{&S89350[70], 6}, {NULL}};
const Segment SGI214405[] = {{&S89350[138], 8}, {NULL}};
const Segment SGI214406[] = {{&S79169[34], 10}, {NULL}};
const Segment SGI214407[] = {{&S79169[27], 6}, {NULL}};
const Segment SGI214408[] = {{&S70437[44], 13}, {NULL}};
const Segment SGI214409[] = {{&S70437[44], 25}, {NULL}};
const Segment SGI214410[] = {{&S70437[30], 6}, {NULL}};
const Segment SGI214411[] = {{&S79169[97], 9}, {NULL}};
const Segment SGI214412[] = {{&S79169[70], 6}, {NULL}};
const Segment SGI214413[] = {{&S70437[136], 3}, {NULL}};
const Segment SGI214414[] = {{&S70437[101], 6}, {NULL}};
const Segment SGI214415[] = {{&S70437[71], 8}, {NULL}};
const Segment SGI214416[] = {{&S79170[44], 6}, {NULL}};
const Segment SGI214417[] = {{&S79170[44], 18}, {NULL}};
const Segment SGI214418[] = {{&S79170[30], 6}, {NULL}};
const Segment SGI214419[] = {{&S79170[111], 3}, {NULL}};
const Segment SGI214420[] = {{&S79170[76], 6}, {NULL}};
const Segment SGI214421[] = {{&S89365[34], 8}, {NULL}};
const Segment SGI214422[] = {{&S89365[27], 6}, {NULL}};
const Segment SGI214423[] = {{&S89365[95], 9}, {NULL}};
const Segment SGI214424[] = {{&S89365[68], 6}, {NULL}};
const Segment SGI214425[] = {{&S89366[34], 8}, {NULL}};
const Segment SGI214426[] = {{&S89366[27], 6}, {NULL}};
const Segment SGI214427[] = {{&S89366[94], 9}, {NULL}};
const Segment SGI214428[] = {{&S89366[67], 6}, {NULL}};
const Segment SGI214429[] = {{&S89367[34], 7}, {NULL}};
const Segment SGI214430[] = {{&S89367[27], 6}, {NULL}};
const Segment SGI214431[] = {{&S89367[99], 9}, {NULL}};
const Segment SGI214432[] = {{&S89367[72], 6}, {NULL}};
const Segment SGI214433[] = {{&S89370[44], 10}, {NULL}};
const Segment SGI214434[] = {{&S89370[44], 22}, {NULL}};
const Segment SGI214435[] = {{&S89370[30], 6}, {NULL}};
const Segment SGI214436[] = {{&S89370[80], 6}, {NULL}};
const Segment SGI214437[] = {{&S89370[107], 13}, {NULL}};
const Segment SGI214438[] = {{&S79210[34], 10}, {NULL}};
const Segment SGI214439[] = {{&S79210[27], 6}, {NULL}};
const Segment SGI214440[] = {{&S79210[97], 9}, {NULL}};
const Segment SGI214441[] = {{&S79210[70], 6}, {NULL}};
const Segment SGI214442[] = {{&S89374[44], 7}, {NULL}};
const Segment SGI214443[] = {{&S89374[44], 24}, {NULL}};
const Segment SGI214444[] = {{&S89374[30], 6}, {NULL}};
const Segment SGI214445[] = {{&S89374[82], 6}, {NULL}};
const Segment SGI214446[] = {{&S89374[109], 13}, {NULL}};
const Segment SGI214447[] = {{&S84297[34], 15}, {NULL}};
const Segment SGI214448[] = {{&S84297[27], 6}, {NULL}};
const Segment SGI214449[] = {{&S89379[34], 10}, {NULL}};
const Segment SGI214450[] = {{&S89379[27], 6}, {NULL}};
const Segment SGI214451[] = {{&S84297[109], 9}, {NULL}};
const Segment SGI214452[] = {{&S84297[82], 6}, {NULL}};
const Segment SGI214453[] = {{&S89379[97], 9}, {NULL}};
const Segment SGI214454[] = {{&S89379[70], 6}, {NULL}};
const Segment SGI214455[] = {{&S79221[52], 10}, {NULL}};
const Segment SGI214456[] = {{&S79221[52], 22}, {NULL}};
const Segment SGI214457[] = {{&S89381[34], 19}, {NULL}};
const Segment SGI214458[] = {{&S79221[38], 6}, {NULL}};
const Segment SGI214459[] = {{&S89381[27], 6}, {NULL}};
const Segment SGI214460[] = {{&S79221[88], 5}, {NULL}};
const Segment SGI214461[] = {{&S89381[106], 9}, {NULL}};
const Segment SGI214462[] = {{&S89381[79], 6}, {NULL}};
const Segment SGI214463[] = {{&S79221[11], 3}, {NULL}};
const Segment SGI214464[] = {{&S79222[52], 11}, {NULL}};
const Segment SGI214465[] = {{&S79222[52], 23}, {NULL}};
const Segment SGI214466[] = {{&S89382[34], 19}, {NULL}};
const Segment SGI214467[] = {{&S79222[38], 6}, {NULL}};
const Segment SGI214468[] = {{&S89382[27], 6}, {NULL}};
const Segment SGI214469[] = {{&S79222[89], 5}, {NULL}};
const Segment SGI214470[] = {{&S89382[106], 9}, {NULL}};
const Segment SGI214471[] = {{&S89382[79], 6}, {NULL}};
const Segment SGI214472[] = {{&S79222[11], 3}, {NULL}};
const Segment SGI214473[] = {{&S84307[34], 10}, {NULL}};
const Segment SGI214474[] = {{&S84307[27], 6}, {NULL}};
const Segment SGI214475[] = {{&S84307[104], 9}, {NULL}};
const Segment SGI214476[] = {{&S84307[77], 6}, {NULL}};
const Segment SGI214477[] = {{&S84308[34], 6}, {NULL}};
const Segment SGI214478[] = {{&S84308[27], 6}, {NULL}};
const Segment SGI214479[] = {{&S84308[93], 9}, {NULL}};
const Segment SGI214480[] = {{&S84308[66], 6}, {NULL}};
const Segment SGI214481[] = {{&S88869[55], 11}, {NULL}};
const Segment SGI214482[] = {{&S88869[55], 23}, {NULL}};
const Segment SGI214483[] = {{&S88869[43], 4}, {NULL}};
const Segment SGI214484[] = {{&S88869[127], 3}, {NULL}};
const Segment SGI214485[] = {{&S88869[92], 6}, {NULL}};
const Segment SGI214486[] = {{&S84313[34], 20}, {NULL}};
const Segment SGI214487[] = {{&S84313[27], 6}, {NULL}};
const Segment SGI214488[] = {{&S84313[107], 9}, {NULL}};
const Segment SGI214489[] = {{&S84313[80], 6}, {NULL}};
const Segment SGI214490[] = {{&S84317[34], 10}, {NULL}};
const Segment SGI214491[] = {{&S84317[27], 6}, {NULL}};
const Segment SGI214492[] = {{&S84317[104], 9}, {NULL}};
const Segment SGI214493[] = {{&S84317[77], 6}, {NULL}};
const Segment SGI214494[] = {{&S70490[34], 5}, {NULL}};
const Segment SGI214495[] = {{&S70490[27], 6}, {NULL}};
const Segment SGI214496[] = {{&S79243[34], 10}, {NULL}};
const Segment SGI214497[] = {{&S79243[27], 6}, {NULL}};
const Segment SGI214498[] = {{&S70490[92], 9}, {NULL}};
const Segment SGI214499[] = {{&S70490[65], 6}, {NULL}};
const Segment SGI214500[] = {{&S79243[97], 9}, {NULL}};
const Segment SGI214501[] = {{&S79243[70], 6}, {NULL}};
const Segment SGI214502[] = {{&S84323[44], 10}, {NULL}};
const Segment SGI214503[] = {{&S84323[44], 22}, {NULL}};
const Segment SGI214504[] = {{&S84323[30], 6}, {NULL}};
const Segment SGI214505[] = {{&S84323[113], 3}, {NULL}};
const Segment SGI214506[] = {{&S84323[80], 5}, {NULL}};
const Segment SGI214507[] = {{&S84323[117], 14}, {NULL}};
const Segment SGI214508[] = {{&S84325[44], 11}, {NULL}};
const Segment SGI214509[] = {{&S84325[44], 23}, {NULL}};
const Segment SGI214510[] = {{&S84325[30], 6}, {NULL}};
const Segment SGI214511[] = {{&S84325[116], 3}, {NULL}};
const Segment SGI214512[] = {{&S84325[81], 6}, {NULL}};
const Segment SGI214513[] = {{&S79252[34], 10}, {NULL}};
const Segment SGI214514[] = {{&S79252[27], 6}, {NULL}};
const Segment SGI214515[] = {{&S79252[97], 9}, {NULL}};
const Segment SGI214516[] = {{&S79252[70], 6}, {NULL}};
const Segment SGI214517[] = {{&S79253[34], 10}, {NULL}};
const Segment SGI214518[] = {{&S79253[27], 6}, {NULL}};
const Segment SGI214519[] = {{&S79253[97], 9}, {NULL}};
const Segment SGI214520[] = {{&S79253[70], 6}, {NULL}};
const Segment SGI214521[] = {{&S79253[131], 8}, {NULL}};
const Segment SGI214522[] = {{&S79257[44], 9}, {NULL}};
const Segment SGI214523[] = {{&S79257[44], 21}, {NULL}};
const Segment SGI214524[] = {{&S79257[30], 6}, {NULL}};
const Segment SGI214525[] = {{&S79257[114], 3}, {NULL}};
const Segment SGI214526[] = {{&S79257[79], 6}, {NULL}};
const Segment SGI214527[] = {{&S79257[138], 10}, {NULL}};
const Segment SGI214528[] = {{&S79258[44], 8}, {NULL}};
const Segment SGI214529[] = {{&S79258[44], 20}, {NULL}};
const Segment SGI214530[] = {{&S79258[30], 6}, {NULL}};
const Segment SGI214531[] = {{&S79258[113], 3}, {NULL}};
const Segment SGI214532[] = {{&S79258[78], 6}, {NULL}};
const Segment SGI214533[] = {{&S79258[137], 10}, {NULL}};
const Segment SGI214534[] = {{&S84340[34], 16}, {NULL}};
const Segment SGI214535[] = {{&S84340[27], 6}, {NULL}};
const Segment SGI214536[] = {{&S84340[103], 9}, {NULL}};
const Segment SGI214537[] = {{&S84340[76], 6}, {NULL}};
const Segment SGI214538[] = {{&S84342[34], 6}, {NULL}};
const Segment SGI214539[] = {{&S84342[27], 6}, {NULL}};
const Segment SGI214540[] = {{&S84342[93], 9}, {NULL}};
const Segment SGI214541[] = {{&S84342[66], 6}, {NULL}};
const Segment SGI214542[] = {{&S89436[34], 12}, {NULL}};
const Segment SGI214543[] = {{&S89436[27], 6}, {NULL}};
const Segment SGI214544[] = {{&S89436[99], 9}, {NULL}};
const Segment SGI214545[] = {{&S89436[72], 6}, {NULL}};
const Segment SGI214546[] = {{&S79267[34], 6}, {NULL}};
const Segment SGI214547[] = {{&S79267[27], 6}, {NULL}};
const Segment SGI214548[] = {{&S79267[93], 9}, {NULL}};
const Segment SGI214549[] = {{&S79267[66], 6}, {NULL}};
const Segment SGI214550[] = {{&S89452[44], 10}, {NULL}};
const Segment SGI214551[] = {{&S89452[44], 22}, {NULL}};
const Segment SGI214552[] = {{&S89452[30], 6}, {NULL}};
const Segment SGI214553[] = {{&S89452[31], 3}, {NULL}};
const Segment SGI214554[] = {{&S89452[95], 6}, {NULL}};
const Segment SGI214555[] = {{&S89452[68], 8}, {NULL}};
const Segment SGI214556[] = {{&S91736[34], 5}, {NULL}};
const Segment SGI214557[] = {{&S91736[27], 6}, {NULL}};
const Segment SGI214558[] = {{&S89456[44], 9}, {NULL}};
const Segment SGI214559[] = {{&S91736[65], 6}, {NULL}};
const Segment SGI214560[] = {{&S89456[44], 21}, {NULL}};
const Segment SGI214561[] = {{&S89456[30], 6}, {NULL}};
const Segment SGI214562[] = {{&S89456[31], 3}, {NULL}};
const Segment SGI214563[] = {{&S89456[79], 6}, {NULL}};
const Segment SGI214564[] = {{&S89458[34], 10}, {NULL}};
const Segment SGI214565[] = {{&S89458[27], 6}, {NULL}};
const Segment SGI214566[] = {{&S89458[97], 9}, {NULL}};
const Segment SGI214567[] = {{&S89458[70], 6}, {NULL}};
const Segment SGI214568[] = {{&S89459[34], 8}, {NULL}};
const Segment SGI214569[] = {{&S89459[27], 6}, {NULL}};
const Segment SGI214570[] = {{&S89459[95], 9}, {NULL}};
const Segment SGI214571[] = {{&S79293[34], 10}, {NULL}};
const Segment SGI214572[] = {{&S89459[68], 6}, {NULL}};
const Segment SGI214573[] = {{&S79293[27], 6}, {NULL}};
const Segment SGI214574[] = {{&S79293[97], 9}, {NULL}};
const Segment SGI214575[] = {{&S79293[70], 6}, {NULL}};
const Segment SGI214576[] = {{&S89460[34], 9}, {NULL}};
const Segment SGI214577[] = {{&S89460[27], 6}, {NULL}};
const Segment SGI214578[] = {{&S89460[96], 9}, {NULL}};
const Segment SGI214579[] = {{&S89460[69], 6}, {NULL}};
const Segment SGI214580[] = {{&S70553[34], 10}, {NULL}};
const Segment SGI214581[] = {{&S70553[27], 6}, {NULL}};
const Segment SGI214582[] = {{&S70553[96], 9}, {NULL}};
const Segment SGI214583[] = {{&S70553[69], 6}, {NULL}};
const Segment SGI214584[] = {{&S79296[34], 7}, {NULL}};
const Segment SGI214585[] = {{&S79296[27], 6}, {NULL}};
const Segment SGI214586[] = {{&S89462[31], 5}, {NULL}};
const Segment SGI214587[] = {{&S89462[51], 3}, {NULL}};
const Segment SGI214588[] = {{&S79296[100], 9}, {NULL}};
const Segment SGI214589[] = {{&S79296[73], 6}, {NULL}};
const Segment SGI214590[] = {{&S79288[34], 7}, {NULL}};
const Segment SGI214591[] = {{&S79288[27], 6}, {NULL}};
const Segment SGI214592[] = {{&S79288[94], 9}, {NULL}};
const Segment SGI214593[] = {{&S79288[67], 6}, {NULL}};
const Segment SGI214594[] = {{&S89464[56], 4}, {NULL}};
const Segment SGI214595[] = {{&S89464[56], 26}, {NULL}};
const Segment SGI214596[] = {{&S79298[34], 11}, {NULL}};
const Segment SGI214597[] = {{&S89464[42], 6}, {NULL}};
const Segment SGI214598[] = {{&S79298[27], 6}, {NULL}};
const Segment SGI214599[] = {{&S89464[43], 3}, {NULL}};
const Segment SGI214600[] = {{&S89464[96], 6}, {NULL}};
const Segment SGI214601[] = {{&S79298[98], 9}, {NULL}};
const Segment SGI214602[] = {{&S79298[71], 6}, {NULL}};
const Segment SGI214603[] = {{&S89464[0], 12}, {NULL}};
const Segment SGI214604[] = {{&S79299[34], 10}, {NULL}};
const Segment SGI214605[] = {{&S79299[27], 6}, {NULL}};
const Segment SGI214606[] = {{&S79299[97], 9}, {NULL}};
const Segment SGI214607[] = {{&S79299[70], 6}, {NULL}};
const Segment SGI214608[] = {{&S89477[34], 11}, {NULL}};
const Segment SGI214609[] = {{&S89477[27], 6}, {NULL}};
const Segment SGI214610[] = {{&S89477[98], 9}, {NULL}};
const Segment SGI214611[] = {{&S89477[71], 6}, {NULL}};
const Segment SGI214612[] = {{&S79314[44], 9}, {NULL}};
const Segment SGI214613[] = {{&S79314[44], 21}, {NULL}};
const Segment SGI214614[] = {{&S89478[34], 10}, {NULL}};
const Segment SGI214615[] = {{&S79314[30], 6}, {NULL}};
const Segment SGI214616[] = {{&S89478[27], 6}, {NULL}};
const Segment SGI214617[] = {{&S79314[134], 3}, {NULL}};
const Segment SGI214618[] = {{&S79314[99], 6}, {NULL}};
const Segment SGI214619[] = {{&S89478[97], 9}, {NULL}};
const Segment SGI214620[] = {{&S89478[70], 6}, {NULL}};
const Segment SGI214621[] = {{&S79314[67], 8}, {NULL}};
const Segment SGI214622[] = {{&S89479[34], 8}, {NULL}};
const Segment SGI214623[] = {{&S89479[27], 6}, {NULL}};
const Segment SGI214624[] = {{&S89479[101], 9}, {NULL}};
const Segment SGI214625[] = {{&S89479[74], 6}, {NULL}};
const Segment SGI214626[] = {{&S79319[45], 10}, {NULL}};
const Segment SGI214627[] = {{&S79319[30], 6}, {NULL}};
const Segment SGI214628[] = {{&S70581[34], 10}, {NULL}};
const Segment SGI214629[] = {{&S70581[27], 6}, {NULL}};
const Segment SGI214630[] = {{&S79319[115], 3}, {NULL}};
const Segment SGI214631[] = {{&S79319[81], 5}, {NULL}};
const Segment SGI214632[] = {{&S70581[97], 9}, {NULL}};
const Segment SGI214633[] = {{&S70581[70], 6}, {NULL}};
const Segment SGI214634[] = {{&S79319[139], 13}, {NULL}};
const Segment SGI214635[] = {{&S89484[44], 11}, {NULL}};
const Segment SGI214636[] = {{&S89484[44], 23}, {NULL}};
const Segment SGI214637[] = {{&S89484[30], 6}, {NULL}};
const Segment SGI214638[] = {{&S89484[31], 3}, {NULL}};
const Segment SGI214639[] = {{&S89484[81], 6}, {NULL}};
const Segment SGI214640[] = {{&S89485[34], 13}, {NULL}};
const Segment SGI214641[] = {{&S89485[27], 6}, {NULL}};
const Segment SGI214642[] = {{&S89485[100], 9}, {NULL}};
const Segment SGI214643[] = {{&S89485[73], 6}, {NULL}};
const Segment SGI214644[] = {{&S89486[34], 9}, {NULL}};
const Segment SGI214645[] = {{&S89486[27], 6}, {NULL}};
const Segment SGI214646[] = {{&S89486[96], 9}, {NULL}};
const Segment SGI214647[] = {{&S89486[69], 6}, {NULL}};
const Segment SGI214648[] = {{&S79325[44], 13}, {NULL}};
const Segment SGI214649[] = {{&S79325[44], 25}, {NULL}};
const Segment SGI214650[] = {{&S79325[30], 6}, {NULL}};
const Segment SGI214651[] = {{&S79325[118], 3}, {NULL}};
const Segment SGI214652[] = {{&S79325[83], 6}, {NULL}};
const Segment SGI214653[] = {{&S84417[34], 8}, {NULL}};
const Segment SGI214654[] = {{&S84417[27], 6}, {NULL}};
const Segment SGI214655[] = {{&S79326[44], 13}, {NULL}};
const Segment SGI214656[] = {{&S79326[44], 25}, {NULL}};
const Segment SGI214657[] = {{&S84417[95], 9}, {NULL}};
const Segment SGI214658[] = {{&S79326[30], 6}, {NULL}};
const Segment SGI214659[] = {{&S84417[68], 6}, {NULL}};
const Segment SGI214660[] = {{&S79326[86], 6}, {NULL}};
const Segment SGI214661[] = {{&S70592[34], 10}, {NULL}};
const Segment SGI214662[] = {{&S70592[27], 6}, {NULL}};
const Segment SGI214663[] = {{&S89490[44], 14}, {NULL}};
const Segment SGI214664[] = {{&S89490[44], 26}, {NULL}};
const Segment SGI214665[] = {{&S89490[30], 6}, {NULL}};
const Segment SGI214666[] = {{&S70592[97], 9}, {NULL}};
const Segment SGI214667[] = {{&S70592[70], 6}, {NULL}};
const Segment SGI214668[] = {{&S89490[31], 3}, {NULL}};
const Segment SGI214669[] = {{&S89490[84], 6}, {NULL}};
const Segment SGI214670[] = {{&S70593[44], 10}, {NULL}};
const Segment SGI214671[] = {{&S70593[44], 21}, {NULL}};
const Segment SGI214672[] = {{&S70593[30], 6}, {NULL}};
const Segment SGI214673[] = {{&S70593[79], 6}, {NULL}};
const Segment SGI214674[] = {{&S70593[106], 13}, {NULL}};
const Segment SGI214675[] = {{&S89494[34], 4}, {NULL}};
const Segment SGI214676[] = {{&S89494[27], 6}, {NULL}};
const Segment SGI214677[] = {{&S89494[91], 9}, {NULL}};
const Segment SGI214678[] = {{&S89494[64], 6}, {NULL}};
const Segment SGI214679[] = {{&S89495[34], 7}, {NULL}};
const Segment SGI214680[] = {{&S89495[27], 6}, {NULL}};
const Segment SGI214681[] = {{&S89495[94], 9}, {NULL}};
const Segment SGI214682[] = {{&S89495[67], 6}, {NULL}};
const Segment SGI214683[] = {{&S79337[34], 18}, {NULL}};
const Segment SGI214684[] = {{&S79337[27], 6}, {NULL}};
const Segment SGI214685[] = {{&S79337[105], 9}, {NULL}};
const Segment SGI214686[] = {{&S79337[78], 6}, {NULL}};
const Segment SGI214687[] = {{&S89500[34], 15}, {NULL}};
const Segment SGI214688[] = {{&S89500[27], 6}, {NULL}};
const Segment SGI214689[] = {{&S89500[102], 9}, {NULL}};
const Segment SGI214690[] = {{&S89500[75], 6}, {NULL}};
const Segment SGI214691[] = {{&S89502[34], 7}, {NULL}};
const Segment SGI214692[] = {{&S89502[27], 6}, {NULL}};
const Segment SGI214693[] = {{&S89502[93], 9}, {NULL}};
const Segment SGI214694[] = {{&S89502[66], 6}, {NULL}};
const Segment SGI214695[] = {{&S89502[134], 8}, {NULL}};
const Segment SGI214696[] = {{&S79344[44], 10}, {NULL}};
const Segment SGI214697[] = {{&S79344[44], 22}, {NULL}};
const Segment SGI214698[] = {{&S79344[30], 6}, {NULL}};
const Segment SGI214699[] = {{&S79344[80], 6}, {NULL}};
const Segment SGI214700[] = {{&S79344[107], 13}, {NULL}};
const Segment SGI214701[] = {{&S89503[34], 10}, {NULL}};
const Segment SGI214702[] = {{&S89503[27], 6}, {NULL}};
const Segment SGI214703[] = {{&S89503[97], 9}, {NULL}};
const Segment SGI214704[] = {{&S89503[70], 6}, {NULL}};
const Segment SGI214705[] = {{&S89504[44], 14}, {NULL}};
const Segment SGI214706[] = {{&S89504[44], 26}, {NULL}};
const Segment SGI214707[] = {{&S89504[30], 6}, {NULL}};
const Segment SGI214708[] = {{&S79346[44], 10}, {NULL}};
const Segment SGI214709[] = {{&S79346[44], 22}, {NULL}};
const Segment SGI214710[] = {{&S79346[30], 6}, {NULL}};
const Segment SGI214711[] = {{&S89504[31], 3}, {NULL}};
const Segment SGI214712[] = {{&S89504[84], 6}, {NULL}};
const Segment SGI214713[] = {{&S79346[80], 6}, {NULL}};
const Segment SGI214714[] = {{&S79346[107], 13}, {NULL}};
const Segment SGI214715[] = {{&S79349[44], 15}, {NULL}};
const Segment SGI214716[] = {{&S79349[44], 27}, {NULL}};
const Segment SGI214717[] = {{&S79349[30], 6}, {NULL}};
const Segment SGI214718[] = {{&S79349[120], 3}, {NULL}};
const Segment SGI214719[] = {{&S79349[85], 6}, {NULL}};
const Segment SGI214720[] = {{&S89506[44], 15}, {NULL}};
const Segment SGI214721[] = {{&S89506[44], 33}, {NULL}};
const Segment SGI214722[] = {{&S89506[30], 6}, {NULL}};
const Segment SGI214723[] = {{&S89506[91], 6}, {NULL}};
const Segment SGI214724[] = {{&S89506[118], 13}, {NULL}};
const Segment SGI214725[] = {{&S89507[44], 15}, {NULL}};
const Segment SGI214726[] = {{&S89507[44], 33}, {NULL}};
const Segment SGI214727[] = {{&S89507[30], 6}, {NULL}};
const Segment SGI214728[] = {{&S89507[91], 6}, {NULL}};
const Segment SGI214729[] = {{&S89507[118], 13}, {NULL}};
const Segment SGI214730[] = {{&S84443[44], 16}, {NULL}};
const Segment SGI214731[] = {{&S84443[44], 38}, {NULL}};
const Segment SGI214732[] = {{&S84443[30], 6}, {NULL}};
const Segment SGI214733[] = {{&S84443[131], 3}, {NULL}};
const Segment SGI214734[] = {{&S84443[96], 6}, {NULL}};
const Segment SGI214735[] = {{&S89513[34], 10}, {NULL}};
const Segment SGI214736[] = {{&S89513[27], 6}, {NULL}};
const Segment SGI214737[] = {{&S89513[97], 9}, {NULL}};
const Segment SGI214738[] = {{&S89513[70], 6}, {NULL}};
const Segment SGI214739[] = {{&S89514[34], 10}, {NULL}};
const Segment SGI214740[] = {{&S89514[27], 6}, {NULL}};
const Segment SGI214741[] = {{&S89514[97], 9}, {NULL}};
const Segment SGI214742[] = {{&S89514[70], 6}, {NULL}};
const Segment SGI214743[] = {{&S84452[34], 10}, {NULL}};
const Segment SGI214744[] = {{&S84452[27], 6}, {NULL}};
const Segment SGI214745[] = {{&S84452[104], 9}, {NULL}};
const Segment SGI214746[] = {{&S84452[77], 6}, {NULL}};
const Segment SGI214747[] = {{&S89521[44], 10}, {NULL}};
const Segment SGI214748[] = {{&S89521[44], 22}, {NULL}};
const Segment SGI214749[] = {{&S89521[30], 6}, {NULL}};
const Segment SGI214750[] = {{&S89521[80], 6}, {NULL}};
const Segment SGI214751[] = {{&S89521[107], 13}, {NULL}};
const Segment SGI214752[] = {{&S89525[52], 5}, {NULL}};
const Segment SGI214753[] = {{&S89525[52], 17}, {NULL}};
const Segment SGI214754[] = {{&S89525[38], 6}, {NULL}};
const Segment SGI214755[] = {{&S89525[83], 5}, {NULL}};
const Segment SGI214756[] = {{&S89525[0], 14}, {NULL}};
const Segment SGI214757[] = {{&S84465[34], 10}, {NULL}};
const Segment SGI214758[] = {{&S89530[34], 10}, {NULL}};
const Segment SGI214759[] = {{&S84465[27], 6}, {NULL}};
const Segment SGI214760[] = {{&S89530[27], 6}, {NULL}};
const Segment SGI214761[] = {{&S89530[97], 9}, {NULL}};
const Segment SGI214762[] = {{&S84465[104], 9}, {NULL}};
const Segment SGI214763[] = {{&S89530[70], 6}, {NULL}};
const Segment SGI214764[] = {{&S84465[77], 6}, {NULL}};
const Segment SGI214765[] = {{&S89537[44], 12}, {NULL}};
const Segment SGI214766[] = {{&S89537[44], 24}, {NULL}};
const Segment SGI214767[] = {{&S89537[30], 6}, {NULL}};
const Segment SGI214768[] = {{&S89537[82], 6}, {NULL}};
const Segment SGI214769[] = {{&S89537[109], 13}, {NULL}};
const Segment SGI214770[] = {{&S89539[44], 15}, {NULL}};
const Segment SGI214771[] = {{&S89539[44], 27}, {NULL}};
const Segment SGI214772[] = {{&S89539[30], 6}, {NULL}};
const Segment SGI214773[] = {{&S89539[85], 6}, {NULL}};
const Segment SGI214774[] = {{&S89539[112], 13}, {NULL}};
const Segment SGI214775[] = {{&S89541[44], 7}, {NULL}};
const Segment SGI214776[] = {{&S89541[44], 25}, {NULL}};
const Segment SGI214777[] = {{&S89541[30], 6}, {NULL}};
const Segment SGI214778[] = {{&S89541[83], 6}, {NULL}};
const Segment SGI214779[] = {{&S89541[110], 13}, {NULL}};
const Segment SGI214780[] = {{&S79403[34], 6}, {NULL}};
const Segment SGI214781[] = {{&S79403[27], 6}, {NULL}};
const Segment SGI214782[] = {{&S79403[95], 9}, {NULL}};
const Segment SGI214783[] = {{&S79403[68], 6}, {NULL}};
const Segment SGI214784[] = {{&S70704[44], 7}, {NULL}};
const Segment SGI214785[] = {{&S70704[44], 18}, {NULL}};
const Segment SGI214786[] = {{&S70704[30], 6}, {NULL}};
const Segment SGI214787[] = {{&S70704[111], 3}, {NULL}};
const Segment SGI214788[] = {{&S70704[76], 6}, {NULL}};
const Segment SGI214789[] = {{&S70705[38], 7}, {NULL}};
const Segment SGI214790[] = {{&S70705[31], 6}, {NULL}};
const Segment SGI214791[] = {{&S70706[44], 14}, {NULL}};
const Segment SGI214792[] = {{&S70706[44], 25}, {NULL}};
const Segment SGI214793[] = {{&S70706[30], 6}, {NULL}};
const Segment SGI214794[] = {{&S70706[118], 3}, {NULL}};
const Segment SGI214795[] = {{&S70706[83], 6}, {NULL}};
const Segment SGI214796[] = {{&S84529[44], 14}, {NULL}};
const Segment SGI214797[] = {{&S84529[44], 26}, {NULL}};
const Segment SGI214798[] = {{&S84529[30], 6}, {NULL}};
const Segment SGI214799[] = {{&S84529[119], 3}, {NULL}};
const Segment SGI214800[] = {{&S84529[84], 6}, {NULL}};
const Segment SGI214801[] = {{&S89602[34], 6}, {NULL}};
const Segment SGI214802[] = {{&S89602[27], 6}, {NULL}};
const Segment SGI214803[] = {{&S89602[94], 9}, {NULL}};
const Segment SGI214804[] = {{&S89602[67], 6}, {NULL}};
const Segment SGI214805[] = {{&S89604[44], 13}, {NULL}};
const Segment SGI214806[] = {{&S89604[44], 25}, {NULL}};
const Segment SGI214807[] = {{&S89604[30], 6}, {NULL}};
const Segment SGI214808[] = {{&S89604[31], 3}, {NULL}};
const Segment SGI214809[] = {{&S89604[83], 6}, {NULL}};
const Segment SGI214810[] = {{&S89613[59], 15}, {NULL}};
const Segment SGI214811[] = {{&S89613[59], 27}, {NULL}};
const Segment SGI214812[] = {{&S89613[45], 6}, {NULL}};
const Segment SGI214813[] = {{&S89613[46], 3}, {NULL}};
const Segment SGI214814[] = {{&S89613[100], 6}, {NULL}};
const Segment SGI214815[] = {{&S89613[0], 14}, {NULL}};
const Segment SGI214816[] = {{&S89616[45], 10}, {NULL}};
const Segment SGI214817[] = {{&S89616[30], 6}, {NULL}};
const Segment SGI214818[] = {{&S89616[31], 3}, {NULL}};
const Segment SGI214819[] = {{&S89616[81], 5}, {NULL}};
const Segment SGI214820[] = {{&S89616[139], 13}, {NULL}};
const Segment SGI214821[] = {{&S89617[45], 15}, {NULL}};
const Segment SGI214822[] = {{&S89617[30], 6}, {NULL}};
const Segment SGI214823[] = {{&S89617[31], 3}, {NULL}};
const Segment SGI214824[] = {{&S89617[86], 5}, {NULL}};
const Segment SGI214825[] = {{&S89617[144], 13}, {NULL}};
const Segment SGI214826[] = {{&S89618[45], 14}, {NULL}};
const Segment SGI214827[] = {{&S89618[30], 6}, {NULL}};
const Segment SGI214828[] = {{&S89618[31], 3}, {NULL}};
const Segment SGI214829[] = {{&S89618[85], 5}, {NULL}};
const Segment SGI214830[] = {{&S89618[143], 13}, {NULL}};
const Segment SGI214831[] = {{&S89619[34], 7}, {NULL}};
const Segment SGI214832[] = {{&S89619[27], 6}, {NULL}};
const Segment SGI214833[] = {{&S89619[94], 9}, {NULL}};
const Segment SGI214834[] = {{&S89619[67], 6}, {NULL}};
const Segment SGI214835[] = {{&S89628[34], 6}, {NULL}};
const Segment SGI214836[] = {{&S89628[27], 6}, {NULL}};
const Segment SGI214837[] = {{&S89628[93], 9}, {NULL}};
const Segment SGI214838[] = {{&S89628[66], 6}, {NULL}};
const Segment SGI214839[] = {{&S89629[34], 5}, {NULL}};
const Segment SGI214840[] = {{&S89629[27], 6}, {NULL}};
const Segment SGI214841[] = {{&S89629[92], 9}, {NULL}};
const Segment SGI214842[] = {{&S89629[65], 6}, {NULL}};
const Segment SGI214843[] = {{&S79348[34], 10}, {NULL}};
const Segment SGI214844[] = {{&S79348[27], 6}, {NULL}};
const Segment SGI214845[] = {{&S79348[97], 9}, {NULL}};
const Segment SGI214846[] = {{&S89633[59], 20}, {NULL}};
const Segment SGI214847[] = {{&S79348[70], 6}, {NULL}};
const Segment SGI214848[] = {{&S89633[59], 32}, {NULL}};
const Segment SGI214849[] = {{&S89633[45], 6}, {NULL}};
const Segment SGI214850[] = {{&S89633[46], 3}, {NULL}};
const Segment SGI214851[] = {{&S89633[105], 6}, {NULL}};
const Segment SGI214852[] = {{&S89633[0], 14}, {NULL}};
const Segment SGI214853[] = {{&S79353[44], 9}, {NULL}};
const Segment SGI214854[] = {{&S79353[44], 21}, {NULL}};
const Segment SGI214855[] = {{&S79353[30], 6}, {NULL}};
const Segment SGI214856[] = {{&S79353[114], 3}, {NULL}};
const Segment SGI214857[] = {{&S79353[79], 6}, {NULL}};
const Segment SGI214858[] = {{&S89639[34], 10}, {NULL}};
const Segment SGI214859[] = {{&S89639[27], 6}, {NULL}};
const Segment SGI214860[] = {{&S89639[97], 9}, {NULL}};
const Segment SGI214861[] = {{&S89639[70], 6}, {NULL}};
const Segment SGI214862[] = {{&S79355[44], 10}, {NULL}};
const Segment SGI214863[] = {{&S79355[44], 22}, {NULL}};
const Segment SGI214864[] = {{&S79355[30], 6}, {NULL}};
const Segment SGI214865[] = {{&S79355[80], 6}, {NULL}};
const Segment SGI214866[] = {{&S89640[34], 13}, {NULL}};
const Segment SGI214867[] = {{&S79355[107], 13}, {NULL}};
const Segment SGI214868[] = {{&S89640[27], 6}, {NULL}};
const Segment SGI214869[] = {{&S89640[100], 9}, {NULL}};
const Segment SGI214870[] = {{&S89640[73], 6}, {NULL}};
const Segment SGI214871[] = {{&S89644[44], 14}, {NULL}};
const Segment SGI214872[] = {{&S89644[44], 26}, {NULL}};
const Segment SGI214873[] = {{&S89644[30], 6}, {NULL}};
const Segment SGI214874[] = {{&S89644[84], 6}, {NULL}};
const Segment SGI214875[] = {{&S89644[111], 13}, {NULL}};
const Segment SGI214876[] = {{&S89646[34], 13}, {NULL}};
const Segment SGI214877[] = {{&S89646[27], 6}, {NULL}};
const Segment SGI214878[] = {{&S89646[100], 9}, {NULL}};
const Segment SGI214879[] = {{&S89646[73], 6}, {NULL}};
const Segment SGI214880[] = {{&S89647[34], 9}, {NULL}};
const Segment SGI214881[] = {{&S89647[27], 6}, {NULL}};
const Segment SGI214882[] = {{&S89647[96], 9}, {NULL}};
const Segment SGI214883[] = {{&S89647[69], 6}, {NULL}};
const Segment SGI214884[] = {{&S84609[44], 10}, {NULL}};
const Segment SGI214885[] = {{&S84609[44], 22}, {NULL}};
const Segment SGI214886[] = {{&S84609[30], 6}, {NULL}};
const Segment SGI214887[] = {{&S84609[115], 3}, {NULL}};
const Segment SGI214888[] = {{&S84609[80], 6}, {NULL}};
const Segment SGI214889[] = {{&S84609[150], 3}, {NULL}};
const Segment SGI214890[] = {{&S89656[44], 7}, {NULL}};
const Segment SGI214891[] = {{&S89656[44], 22}, {NULL}};
const Segment SGI214892[] = {{&S89656[30], 6}, {NULL}};
const Segment SGI214893[] = {{&S89656[80], 6}, {NULL}};
const Segment SGI214894[] = {{&S89656[107], 13}, {NULL}};
const Segment SGI214895[] = {{&S89657[44], 7}, {NULL}};
const Segment SGI214896[] = {{&S89657[44], 23}, {NULL}};
const Segment SGI214897[] = {{&S89657[30], 6}, {NULL}};
const Segment SGI214898[] = {{&S79378[44], 17}, {NULL}};
const Segment SGI214899[] = {{&S79378[44], 29}, {NULL}};
const Segment SGI214900[] = {{&S79378[30], 6}, {NULL}};
const Segment SGI214901[] = {{&S89657[81], 6}, {NULL}};
const Segment SGI214902[] = {{&S79378[143], 3}, {NULL}};
const Segment SGI214903[] = {{&S89657[108], 13}, {NULL}};
const Segment SGI214904[] = {{&S79378[108], 6}, {NULL}};
const Segment SGI214905[] = {{&S79379[44], 9}, {NULL}};
const Segment SGI214906[] = {{&S79379[44], 21}, {NULL}};
const Segment SGI214907[] = {{&S79379[30], 6}, {NULL}};
const Segment SGI214908[] = {{&S79379[135], 3}, {NULL}};
const Segment SGI214909[] = {{&S79379[100], 6}, {NULL}};
const Segment SGI214910[] = {{&S89659[44], 10}, {NULL}};
const Segment SGI214911[] = {{&S89659[44], 22}, {NULL}};
const Segment SGI214912[] = {{&S89659[30], 6}, {NULL}};
const Segment SGI214913[] = {{&S84619[34], 18}, {NULL}};
const Segment SGI214914[] = {{&S84619[27], 6}, {NULL}};
const Segment SGI214915[] = {{&S89659[80], 6}, {NULL}};
const Segment SGI214916[] = {{&S89659[107], 13}, {NULL}};
const Segment SGI214917[] = {{&S84619[105], 9}, {NULL}};
const Segment SGI214918[] = {{&S84619[78], 6}, {NULL}};
const Segment SGI214919[] = {{&S79380[44], 9}, {NULL}};
const Segment SGI214920[] = {{&S79380[44], 21}, {NULL}};
const Segment SGI214921[] = {{&S79380[30], 6}, {NULL}};
const Segment SGI214922[] = {{&S79380[135], 3}, {NULL}};
const Segment SGI214923[] = {{&S79380[100], 6}, {NULL}};
const Segment SGI214924[] = {{&S89660[44], 13}, {NULL}};
const Segment SGI214925[] = {{&S89660[44], 25}, {NULL}};
const Segment SGI214926[] = {{&S89660[30], 6}, {NULL}};
const Segment SGI214927[] = {{&S89660[83], 6}, {NULL}};
const Segment SGI214928[] = {{&S89660[110], 13}, {NULL}};
const Segment SGI214929[] = {{&S89662[52], 5}, {NULL}};
const Segment SGI214930[] = {{&S89662[52], 17}, {NULL}};
const Segment SGI214931[] = {{&S89662[38], 6}, {NULL}};
const Segment SGI214932[] = {{&S89662[83], 5}, {NULL}};
const Segment SGI214933[] = {{&S89662[0], 14}, {NULL}};
const Segment SGI214934[] = {{&S89665[52], 10}, {NULL}};
const Segment SGI214935[] = {{&S89665[52], 22}, {NULL}};
const Segment SGI214936[] = {{&S89665[38], 6}, {NULL}};
const Segment SGI214937[] = {{&S89665[88], 5}, {NULL}};
const Segment SGI214938[] = {{&S89665[0], 14}, {NULL}};
const Segment SGI214939[] = {{&S89667[34], 10}, {NULL}};
const Segment SGI214940[] = {{&S89667[27], 6}, {NULL}};
const Segment SGI214941[] = {{&S89667[97], 9}, {NULL}};
const Segment SGI214942[] = {{&S89667[70], 6}, {NULL}};
const Segment SGI214943[] = {{&S84634[34], 25}, {NULL}};
const Segment SGI214944[] = {{&S89672[34], 7}, {NULL}};
const Segment SGI214945[] = {{&S79395[34], 10}, {NULL}};
const Segment SGI214946[] = {{&S89672[27], 6}, {NULL}};
const Segment SGI214947[] = {{&S84634[27], 6}, {NULL}};
const Segment SGI214948[] = {{&S79395[27], 6}, {NULL}};
const Segment SGI214949[] = {{&S89672[94], 9}, {NULL}};
const Segment SGI214950[] = {{&S84634[112], 9}, {NULL}};
const Segment SGI214951[] = {{&S89672[67], 6}, {NULL}};
const Segment SGI214952[] = {{&S84634[85], 6}, {NULL}};
const Segment SGI214953[] = {{&S79395[97], 9}, {NULL}};
const Segment SGI214954[] = {{&S79395[70], 6}, {NULL}};
const Segment SGI214955[] = {{&S79395[138], 8}, {NULL}};
const Segment SGI214956[] = {{&S79396[34], 14}, {NULL}};
const Segment SGI214957[] = {{&S79396[27], 6}, {NULL}};
const Segment SGI214958[] = {{&S79396[101], 9}, {NULL}};
const Segment SGI214959[] = {{&S79396[74], 6}, {NULL}};
const Segment SGI214960[] = {{&S79396[142], 8}, {NULL}};
const Segment SGI214961[] = {{&S89675[44], 11}, {NULL}};
const Segment SGI214962[] = {{&S79397[34], 13}, {NULL}};
const Segment SGI214963[] = {{&S89675[44], 23}, {NULL}};
const Segment SGI214964[] = {{&S89675[30], 6}, {NULL}};
const Segment SGI214965[] = {{&S79397[27], 6}, {NULL}};
const Segment SGI214966[] = {{&S89675[81], 6}, {NULL}};
const Segment SGI214967[] = {{&S79397[100], 9}, {NULL}};
const Segment SGI214968[] = {{&S79397[73], 6}, {NULL}};
const Segment SGI214969[] = {{&S89675[108], 13}, {NULL}};
const Segment SGI214970[] = {{&S79397[141], 8}, {NULL}};
const Segment SGI214971[] = {{&S84639[44], 14}, {NULL}};
const Segment SGI214972[] = {{&S84639[44], 26}, {NULL}};
const Segment SGI214973[] = {{&S84639[30], 6}, {NULL}};
const Segment SGI214974[] = {{&S84639[84], 6}, {NULL}};
const Segment SGI214975[] = {{&S84639[111], 13}, {NULL}};
const Segment SGI214976[] = {{&S79399[34], 10}, {NULL}};
const Segment SGI214977[] = {{&S79399[27], 6}, {NULL}};
const Segment SGI214978[] = {{&S79399[97], 9}, {NULL}};
const Segment SGI214979[] = {{&S79399[70], 6}, {NULL}};
const Segment SGI214980[] = {{&S79402[34], 7}, {NULL}};
const Segment SGI214981[] = {{&S79402[27], 6}, {NULL}};
const Segment SGI214982[] = {{&S79402[100], 9}, {NULL}};
const Segment SGI214983[] = {{&S79402[73], 6}, {NULL}};
const Segment SGI214984[] = {{&S79402[141], 8}, {NULL}};
const Segment SGI214985[] = {{&S89684[34], 7}, {NULL}};
const Segment SGI214986[] = {{&S89684[27], 6}, {NULL}};
const Segment SGI214987[] = {{&S89684[95], 9}, {NULL}};
const Segment SGI214988[] = {{&S89684[68], 6}, {NULL}};
const Segment SGI214989[] = {{&S84646[34], 18}, {NULL}};
const Segment SGI214990[] = {{&S84646[27], 6}, {NULL}};
const Segment SGI214991[] = {{&S84646[114], 9}, {NULL}};
const Segment SGI214992[] = {{&S84646[87], 6}, {NULL}};
const Segment SGI214993[] = {{&S79405[44], 10}, {NULL}};
const Segment SGI214994[] = {{&S79405[44], 22}, {NULL}};
const Segment SGI214995[] = {{&S79405[30], 6}, {NULL}};
const Segment SGI214996[] = {{&S84646[155], 8}, {NULL}};
const Segment SGI214997[] = {{&S79405[80], 6}, {NULL}};
const Segment SGI214998[] = {{&S79405[107], 13}, {NULL}};
const Segment SGI214999[] = {{&S89686[34], 7}, {NULL}};
const Segment SGI215000[] = {{&S89686[27], 6}, {NULL}};
const Segment SGI215001[] = {{&S89686[99], 9}, {NULL}};
const Segment SGI215002[] = {{&S89686[72], 6}, {NULL}};
const Segment SGI215003[] = {{&S84650[34], 9}, {NULL}};
const Segment SGI215004[] = {{&S84650[27], 6}, {NULL}};
const Segment SGI215005[] = {{&S89689[34], 10}, {NULL}};
const Segment SGI215006[] = {{&S89689[27], 6}, {NULL}};
const Segment SGI215007[] = {{&S84650[96], 9}, {NULL}};
const Segment SGI215008[] = {{&S84650[69], 6}, {NULL}};
const Segment SGI215009[] = {{&S89689[97], 9}, {NULL}};
const Segment SGI215010[] = {{&S89689[70], 6}, {NULL}};
const Segment SGI215011[] = {{&S89689[138], 8}, {NULL}};
const Segment SGI215012[] = {{&S79424[34], 9}, {NULL}};
const Segment SGI215013[] = {{&S79424[27], 6}, {NULL}};
const Segment SGI215014[] = {{&S79424[96], 9}, {NULL}};
const Segment SGI215015[] = {{&S79424[69], 6}, {NULL}};
const Segment SGI215016[] = {{&S79425[34], 13}, {NULL}};
const Segment SGI215017[] = {{&S79425[27], 6}, {NULL}};
const Segment SGI215018[] = {{&S79425[100], 9}, {NULL}};
const Segment SGI215019[] = {{&S79425[73], 6}, {NULL}};
const Segment SGI215020[] = {{&S79426[34], 12}, {NULL}};
const Segment SGI215021[] = {{&S79426[27], 6}, {NULL}};
const Segment SGI215022[] = {{&S79426[99], 9}, {NULL}};
const Segment SGI215023[] = {{&S79426[72], 6}, {NULL}};
const Segment SGI215024[] = {{&S79610[34], 8}, {NULL}};
const Segment SGI215025[] = {{&S79610[27], 6}, {NULL}};
const Segment SGI215026[] = {{&S79610[95], 9}, {NULL}};
const Segment SGI215027[] = {{&S79610[68], 6}, {NULL}};
const Segment SGI215028[] = {{&S79453[44], 9}, {NULL}};
const Segment SGI215029[] = {{&S79453[44], 21}, {NULL}};
const Segment SGI215030[] = {{&S79453[30], 6}, {NULL}};
const Segment SGI215031[] = {{&S79453[114], 3}, {NULL}};
const Segment SGI215032[] = {{&S79453[79], 6}, {NULL}};
const Segment SGI215033[] = {{&S79456[34], 7}, {NULL}};
const Segment SGI215034[] = {{&S79456[27], 6}, {NULL}};
const Segment SGI215035[] = {{&S79456[99], 9}, {NULL}};
const Segment SGI215036[] = {{&S79456[72], 6}, {NULL}};
const Segment SGI215037[] = {{&S79456[140], 8}, {NULL}};
const Segment SGI215038[] = {{&S79460[34], 18}, {NULL}};
const Segment SGI215039[] = {{&S79460[27], 6}, {NULL}};
const Segment SGI215040[] = {{&S79460[105], 9}, {NULL}};
const Segment SGI215041[] = {{&S79460[78], 6}, {NULL}};
const Segment SGI215042[] = {{&S79461[34], 6}, {NULL}};
const Segment SGI215043[] = {{&S79461[27], 6}, {NULL}};
const Segment SGI215044[] = {{&S79461[93], 9}, {NULL}};
const Segment SGI215045[] = {{&S79461[66], 6}, {NULL}};
const Segment SGI215046[] = {{&S79463[34], 7}, {NULL}};
const Segment SGI215047[] = {{&S79463[27], 6}, {NULL}};
const Segment SGI215048[] = {{&S79463[94], 9}, {NULL}};
const Segment SGI215049[] = {{&S79463[67], 6}, {NULL}};
const Segment SGI215050[] = {{&S84726[44], 15}, {NULL}};
const Segment SGI215051[] = {{&S84726[44], 27}, {NULL}};
const Segment SGI215052[] = {{&S84726[30], 6}, {NULL}};
const Segment SGI215053[] = {{&S84726[120], 3}, {NULL}};
const Segment SGI215054[] = {{&S84726[85], 6}, {NULL}};
const Segment SGI215055[] = {{&S84731[34], 9}, {NULL}};
const Segment SGI215056[] = {{&S84731[27], 6}, {NULL}};
const Segment SGI215057[] = {{&S84731[96], 9}, {NULL}};
const Segment SGI215058[] = {{&S84731[69], 6}, {NULL}};
const Segment SGI215059[] = {{&S79494[44], 11}, {NULL}};
const Segment SGI215060[] = {{&S79494[44], 23}, {NULL}};
const Segment SGI215061[] = {{&S79494[30], 6}, {NULL}};
const Segment SGI215062[] = {{&S79494[132], 3}, {NULL}};
const Segment SGI215063[] = {{&S79494[97], 6}, {NULL}};
const Segment SGI215064[] = {{&S79494[69], 8}, {NULL}};
const Segment SGI215065[] = {{&S79505[44], 12}, {NULL}};
const Segment SGI215066[] = {{&S79505[44], 24}, {NULL}};
const Segment SGI215067[] = {{&S79505[30], 6}, {NULL}};
const Segment SGI215068[] = {{&S79505[117], 3}, {NULL}};
const Segment SGI215069[] = {{&S79505[82], 6}, {NULL}};
const Segment SGI215070[] = {{&S79513[34], 10}, {NULL}};
const Segment SGI215071[] = {{&S79513[27], 6}, {NULL}};
const Segment SGI215072[] = {{&S79513[97], 9}, {NULL}};
const Segment SGI215073[] = {{&S79513[70], 6}, {NULL}};
const Segment SGI215074[] = {{&S79516[34], 11}, {NULL}};
const Segment SGI215075[] = {{&S79516[27], 6}, {NULL}};
const Segment SGI215076[] = {{&S79516[98], 9}, {NULL}};
const Segment SGI215077[] = {{&S79516[71], 6}, {NULL}};
const Segment SGI215078[] = {{&S79707[44], 7}, {NULL}};
const Segment SGI215079[] = {{&S79707[44], 19}, {NULL}};
const Segment SGI215080[] = {{&S79707[30], 6}, {NULL}};
const Segment SGI215081[] = {{&S79707[112], 3}, {NULL}};
const Segment SGI215082[] = {{&S79707[77], 6}, {NULL}};
const Segment SGI215083[] = {{&S79520[34], 7}, {NULL}};
const Segment SGI215084[] = {{&S79520[27], 6}, {NULL}};
const Segment SGI215085[] = {{&S79520[99], 9}, {NULL}};
const Segment SGI215086[] = {{&S79520[72], 6}, {NULL}};
const Segment SGI215087[] = {{&S70867[34], 6}, {NULL}};
const Segment SGI215088[] = {{&S70867[27], 6}, {NULL}};
const Segment SGI215089[] = {{&S70867[92], 9}, {NULL}};
const Segment SGI215090[] = {{&S70867[65], 6}, {NULL}};
const Segment SGI215091[] = {{&S84803[34], 18}, {NULL}};
const Segment SGI215092[] = {{&S84803[27], 6}, {NULL}};
const Segment SGI215093[] = {{&S84803[105], 9}, {NULL}};
const Segment SGI215094[] = {{&S84803[78], 6}, {NULL}};
const Segment SGI215095[] = {{&S84811[45], 10}, {NULL}};
const Segment SGI215096[] = {{&S84811[30], 6}, {NULL}};
const Segment SGI215097[] = {{&S84811[115], 3}, {NULL}};
const Segment SGI215098[] = {{&S84811[81], 5}, {NULL}};
const Segment SGI215099[] = {{&S84811[139], 13}, {NULL}};
const Segment SGI215100[] = {{&S79555[44], 15}, {NULL}};
const Segment SGI215101[] = {{&S79555[44], 27}, {NULL}};
const Segment SGI215102[] = {{&S79555[30], 6}, {NULL}};
const Segment SGI215103[] = {{&S79555[120], 3}, {NULL}};
const Segment SGI215104[] = {{&S79555[85], 6}, {NULL}};
const Segment SGI215105[] = {{&S79555[152], 15}, {NULL}};
const Segment SGI215106[] = {{&S84814[44], 13}, {NULL}};
const Segment SGI215107[] = {{&S84814[44], 25}, {NULL}};
const Segment SGI215108[] = {{&S84814[30], 6}, {NULL}};
const Segment SGI215109[] = {{&S84814[118], 3}, {NULL}};
const Segment SGI215110[] = {{&S84814[83], 6}, {NULL}};
const Segment SGI215111[] = {{&S79556[44], 11}, {NULL}};
const Segment SGI215112[] = {{&S79556[44], 23}, {NULL}};
const Segment SGI215113[] = {{&S79556[30], 6}, {NULL}};
const Segment SGI215114[] = {{&S79556[116], 3}, {NULL}};
const Segment SGI215115[] = {{&S79556[81], 6}, {NULL}};
const Segment SGI215116[] = {{&S79556[148], 15}, {NULL}};
const Segment SGI215117[] = {{&S79557[34], 11}, {NULL}};
const Segment SGI215118[] = {{&S79557[27], 6}, {NULL}};
const Segment SGI215119[] = {{&S79557[98], 9}, {NULL}};
const Segment SGI215120[] = {{&S79557[71], 6}, {NULL}};
const Segment SGI215121[] = {{&S70893[34], 8}, {NULL}};
const Segment SGI215122[] = {{&S79559[34], 9}, {NULL}};
const Segment SGI215123[] = {{&S70893[27], 6}, {NULL}};
const Segment SGI215124[] = {{&S79559[27], 6}, {NULL}};
const Segment SGI215125[] = {{&S70893[94], 9}, {NULL}};
const Segment SGI215126[] = {{&S70893[67], 6}, {NULL}};
const Segment SGI215127[] = {{&S79559[96], 9}, {NULL}};
const Segment SGI215128[] = {{&S79559[69], 6}, {NULL}};
const Segment SGI215129[] = {{&S79560[44], 11}, {NULL}};
const Segment SGI215130[] = {{&S79560[44], 23}, {NULL}};
const Segment SGI215131[] = {{&S79560[30], 6}, {NULL}};
const Segment SGI215132[] = {{&S79560[116], 3}, {NULL}};
const Segment SGI215133[] = {{&S79560[81], 6}, {NULL}};
const Segment SGI215134[] = {{&S84823[34], 11}, {NULL}};
const Segment SGI215135[] = {{&S84823[27], 6}, {NULL}};
const Segment SGI215136[] = {{&S84823[98], 9}, {NULL}};
const Segment SGI215137[] = {{&S84823[71], 6}, {NULL}};
const Segment SGI215138[] = {{&S79562[34], 6}, {NULL}};
const Segment SGI215139[] = {{&S79562[27], 6}, {NULL}};
const Segment SGI215140[] = {{&S79562[93], 9}, {NULL}};
const Segment SGI215141[] = {{&S79562[66], 6}, {NULL}};
const Segment SGI215142[] = {{&S79565[34], 7}, {NULL}};
const Segment SGI215143[] = {{&S79565[27], 6}, {NULL}};
const Segment SGI215144[] = {{&S79565[98], 9}, {NULL}};
const Segment SGI215145[] = {{&S79565[71], 6}, {NULL}};
const Segment SGI215146[] = {{&S79566[34], 11}, {NULL}};
const Segment SGI215147[] = {{&S79566[27], 6}, {NULL}};
const Segment SGI215148[] = {{&S79566[98], 9}, {NULL}};
const Segment SGI215149[] = {{&S79566[71], 6}, {NULL}};
const Segment SGI215150[] = {{&S79567[34], 11}, {NULL}};
const Segment SGI215151[] = {{&S79567[27], 6}, {NULL}};
const Segment SGI215152[] = {{&S79567[98], 9}, {NULL}};
const Segment SGI215153[] = {{&S79567[71], 6}, {NULL}};
const Segment SGI215154[] = {{&S79568[34], 11}, {NULL}};
const Segment SGI215155[] = {{&S79568[27], 6}, {NULL}};
const Segment SGI215156[] = {{&S79568[98], 9}, {NULL}};
const Segment SGI215157[] = {{&S79568[71], 6}, {NULL}};
const Segment SGI215158[] = {{&S70911[34], 10}, {NULL}};
const Segment SGI215159[] = {{&S70911[27], 6}, {NULL}};
const Segment SGI215160[] = {{&S70911[96], 9}, {NULL}};
const Segment SGI215161[] = {{&S70911[69], 6}, {NULL}};
const Segment SGI215162[] = {{&S70912[34], 10}, {NULL}};
const Segment SGI215163[] = {{&S70912[27], 6}, {NULL}};
const Segment SGI215164[] = {{&S70912[96], 9}, {NULL}};
const Segment SGI215165[] = {{&S70912[69], 6}, {NULL}};
const Segment SGI215166[] = {{&S89756[44], 8}, {NULL}};
const Segment SGI215167[] = {{&S89756[44], 20}, {NULL}};
const Segment SGI215168[] = {{&S89756[30], 6}, {NULL}};
const Segment SGI215169[] = {{&S89756[31], 3}, {NULL}};
const Segment SGI215170[] = {{&S89756[78], 6}, {NULL}};
const Segment SGI215171[] = {{&S89763[45], 18}, {NULL}};
const Segment SGI215172[] = {{&S89763[30], 6}, {NULL}};
const Segment SGI215173[] = {{&S89763[31], 3}, {NULL}};
const Segment SGI215174[] = {{&S89763[89], 5}, {NULL}};
const Segment SGI215175[] = {{&S89763[147], 13}, {NULL}};
const Segment SGI215176[] = {{&S79804[44], 14}, {NULL}};
const Segment SGI215177[] = {{&S79804[44], 26}, {NULL}};
const Segment SGI215178[] = {{&S79804[30], 6}, {NULL}};
const Segment SGI215179[] = {{&S79804[84], 6}, {NULL}};
const Segment SGI215180[] = {{&S79804[121], 3}, {NULL}};
const Segment SGI215181[] = {{&S70943[44], 10}, {NULL}};
const Segment SGI215182[] = {{&S70943[44], 21}, {NULL}};
const Segment SGI215183[] = {{&S70943[30], 6}, {NULL}};
const Segment SGI215184[] = {{&S70943[114], 3}, {NULL}};
const Segment SGI215185[] = {{&S70943[79], 6}, {NULL}};
const Segment SGI215186[] = {{&S70944[44], 10}, {NULL}};
const Segment SGI215187[] = {{&S70944[44], 22}, {NULL}};
const Segment SGI215188[] = {{&S70944[30], 6}, {NULL}};
const Segment SGI215189[] = {{&S70944[115], 3}, {NULL}};
const Segment SGI215190[] = {{&S70944[80], 6}, {NULL}};
const Segment SGI215191[] = {{&S89786[34], 41}, {NULL}};
const Segment SGI215192[] = {{&S89786[27], 6}, {NULL}};
const Segment SGI215193[] = {{&S89786[138], 9}, {NULL}};
const Segment SGI215194[] = {{&S89786[111], 6}, {NULL}};
const Segment SGI215195[] = {{&S70946[44], 10}, {NULL}};
const Segment SGI215196[] = {{&S70946[44], 27}, {NULL}};
const Segment SGI215197[] = {{&S70946[30], 6}, {NULL}};
const Segment SGI215198[] = {{&S70946[120], 3}, {NULL}};
const Segment SGI215199[] = {{&S89787[34], 20}, {NULL}};
const Segment SGI215200[] = {{&S70946[85], 6}, {NULL}};
const Segment SGI215201[] = {{&S89787[27], 6}, {NULL}};
const Segment SGI215202[] = {{&S89787[117], 9}, {NULL}};
const Segment SGI215203[] = {{&S89787[90], 6}, {NULL}};
const Segment SGI215204[] = {{&S70947[44], 10}, {NULL}};
const Segment SGI215205[] = {{&S70947[44], 22}, {NULL}};
const Segment SGI215206[] = {{&S70947[30], 6}, {NULL}};
const Segment SGI215207[] = {{&S70947[115], 3}, {NULL}};
const Segment SGI215208[] = {{&S70947[80], 6}, {NULL}};
const Segment SGI215209[] = {{&S89790[34], 7}, {NULL}};
const Segment SGI215210[] = {{&S89790[27], 6}, {NULL}};
const Segment SGI215211[] = {{&S89790[94], 9}, {NULL}};
const Segment SGI215212[] = {{&S89790[67], 6}, {NULL}};
const Segment SGI215213[] = {{&S79614[44], 10}, {NULL}};
const Segment SGI215214[] = {{&S79614[44], 22}, {NULL}};
const Segment SGI215215[] = {{&S79614[30], 6}, {NULL}};
const Segment SGI215216[] = {{&S79614[80], 6}, {NULL}};
const Segment SGI215217[] = {{&S79614[107], 13}, {NULL}};
const Segment SGI215218[] = {{&S89792[34], 7}, {NULL}};
const Segment SGI215219[] = {{&S89792[27], 6}, {NULL}};
const Segment SGI215220[] = {{&S89792[112], 9}, {NULL}};
const Segment SGI215221[] = {{&S89792[85], 6}, {NULL}};
const Segment SGI215222[] = {{&S79615[34], 7}, {NULL}};
const Segment SGI215223[] = {{&S79615[27], 6}, {NULL}};
const Segment SGI215224[] = {{&S79615[99], 9}, {NULL}};
const Segment SGI215225[] = {{&S79615[72], 6}, {NULL}};
const Segment SGI215226[] = {{&S79615[140], 8}, {NULL}};
const Segment SGI215227[] = {{&S89793[34], 11}, {NULL}};
const Segment SGI215228[] = {{&S89793[27], 6}, {NULL}};
const Segment SGI215229[] = {{&S89793[98], 9}, {NULL}};
const Segment SGI215230[] = {{&S89793[71], 6}, {NULL}};
const Segment SGI215231[] = {{&S84890[34], 10}, {NULL}};
const Segment SGI215232[] = {{&S84890[27], 6}, {NULL}};
const Segment SGI215233[] = {{&S89794[34], 7}, {NULL}};
const Segment SGI215234[] = {{&S89794[27], 6}, {NULL}};
const Segment SGI215235[] = {{&S84890[97], 9}, {NULL}};
const Segment SGI215236[] = {{&S84890[70], 6}, {NULL}};
const Segment SGI215237[] = {{&S89794[98], 9}, {NULL}};
const Segment SGI215238[] = {{&S89794[71], 6}, {NULL}};
const Segment SGI215239[] = {{&S84891[34], 10}, {NULL}};
const Segment SGI215240[] = {{&S84891[27], 6}, {NULL}};
const Segment SGI215241[] = {{&S89795[34], 10}, {NULL}};
const Segment SGI215242[] = {{&S89795[27], 6}, {NULL}};
const Segment SGI215243[] = {{&S84891[97], 9}, {NULL}};
const Segment SGI215244[] = {{&S84891[70], 6}, {NULL}};
const Segment SGI215245[] = {{&S89795[97], 9}, {NULL}};
const Segment SGI215246[] = {{&S89795[70], 6}, {NULL}};
const Segment SGI215247[] = {{&S89797[34], 5}, {NULL}};
const Segment SGI215248[] = {{&S89797[27], 6}, {NULL}};
const Segment SGI215249[] = {{&S89797[92], 9}, {NULL}};
const Segment SGI215250[] = {{&S89797[65], 6}, {NULL}};
const Segment SGI215251[] = {{&S70961[44], 10}, {NULL}};
const Segment SGI215252[] = {{&S70961[44], 21}, {NULL}};
const Segment SGI215253[] = {{&S70961[30], 6}, {NULL}};
const Segment SGI215254[] = {{&S70961[114], 3}, {NULL}};
const Segment SGI215255[] = {{&S70961[79], 6}, {NULL}};
const Segment SGI215256[] = {{&S89798[34], 9}, {NULL}};
const Segment SGI215257[] = {{&S89798[27], 6}, {NULL}};
const Segment SGI215258[] = {{&S89798[96], 9}, {NULL}};
const Segment SGI215259[] = {{&S70962[38], 10}, {NULL}};
const Segment SGI215260[] = {{&S89798[69], 6}, {NULL}};
const Segment SGI215261[] = {{&S70962[31], 6}, {NULL}};
const Segment SGI215262[] = {{&S84898[44], 12}, {NULL}};
const Segment SGI215263[] = {{&S84898[44], 24}, {NULL}};
const Segment SGI215264[] = {{&S84898[30], 6}, {NULL}};
const Segment SGI215265[] = {{&S84898[117], 3}, {NULL}};
const Segment SGI215266[] = {{&S84898[82], 6}, {NULL}};
const Segment SGI215267[] = {{&S89806[44], 12}, {NULL}};
const Segment SGI215268[] = {{&S89806[44], 24}, {NULL}};
const Segment SGI215269[] = {{&S89806[30], 6}, {NULL}};
const Segment SGI215270[] = {{&S89806[31], 3}, {NULL}};
const Segment SGI215271[] = {{&S89806[82], 6}, {NULL}};
const Segment SGI215272[] = {{&S89808[34], 7}, {NULL}};
const Segment SGI215273[] = {{&S89808[27], 6}, {NULL}};
const Segment SGI215274[] = {{&S89808[94], 9}, {NULL}};
const Segment SGI215275[] = {{&S89808[67], 6}, {NULL}};
const Segment SGI215276[] = {{&S79862[34], 4}, {NULL}};
const Segment SGI215277[] = {{&S79862[27], 6}, {NULL}};
const Segment SGI215278[] = {{&S79862[91], 9}, {NULL}};
const Segment SGI215279[] = {{&S79862[64], 6}, {NULL}};
const Segment SGI215280[] = {{&S84917[34], 25}, {NULL}};
const Segment SGI215281[] = {{&S84917[27], 6}, {NULL}};
const Segment SGI215282[] = {{&S84917[112], 9}, {NULL}};
const Segment SGI215283[] = {{&S70983[34], 7}, {NULL}};
const Segment SGI215284[] = {{&S84917[85], 6}, {NULL}};
const Segment SGI215285[] = {{&S70983[27], 6}, {NULL}};
const Segment SGI215286[] = {{&S70983[100], 9}, {NULL}};
const Segment SGI215287[] = {{&S70983[73], 6}, {NULL}};
const Segment SGI215288[] = {{&S89820[34], 7}, {NULL}};
const Segment SGI215289[] = {{&S89820[27], 6}, {NULL}};
const Segment SGI215290[] = {{&S89820[94], 9}, {NULL}};
const Segment SGI215291[] = {{&S89820[67], 6}, {NULL}};
const Segment SGI215292[] = {{&S70984[34], 7}, {NULL}};
const Segment SGI215293[] = {{&S70984[27], 6}, {NULL}};
const Segment SGI215294[] = {{&S84919[44], 11}, {NULL}};
const Segment SGI215295[] = {{&S84919[44], 23}, {NULL}};
const Segment SGI215296[] = {{&S84919[30], 6}, {NULL}};
const Segment SGI215297[] = {{&S70984[100], 9}, {NULL}};
const Segment SGI215298[] = {{&S70984[73], 6}, {NULL}};
const Segment SGI215299[] = {{&S84919[116], 3}, {NULL}};
const Segment SGI215300[] = {{&S84919[81], 6}, {NULL}};
const Segment SGI215301[] = {{&S84919[148], 15}, {NULL}};
const Segment SGI215302[] = {{&S79652[34], 10}, {NULL}};
const Segment SGI215303[] = {{&S79652[27], 6}, {NULL}};
const Segment SGI215304[] = {{&S79652[97], 9}, {NULL}};
const Segment SGI215305[] = {{&S79652[70], 6}, {NULL}};
const Segment SGI215306[] = {{&S84921[44], 18}, {NULL}};
const Segment SGI215307[] = {{&S84921[44], 30}, {NULL}};
const Segment SGI215308[] = {{&S84921[30], 6}, {NULL}};
const Segment SGI215309[] = {{&S84921[123], 3}, {NULL}};
const Segment SGI215310[] = {{&S84921[88], 6}, {NULL}};
const Segment SGI215311[] = {{&S79657[44], 10}, {NULL}};
const Segment SGI215312[] = {{&S79657[44], 22}, {NULL}};
const Segment SGI215313[] = {{&S79657[30], 6}, {NULL}};
const Segment SGI215314[] = {{&S79657[115], 3}, {NULL}};
const Segment SGI215315[] = {{&S79657[80], 6}, {NULL}};
const Segment SGI215316[] = {{&S70993[44], 10}, {NULL}};
const Segment SGI215317[] = {{&S70993[44], 21}, {NULL}};
const Segment SGI215318[] = {{&S70993[30], 6}, {NULL}};
const Segment SGI215319[] = {{&S70993[79], 6}, {NULL}};
const Segment SGI215320[] = {{&S70993[106], 13}, {NULL}};
const Segment SGI215321[] = {{&S79663[34], 10}, {NULL}};
const Segment SGI215322[] = {{&S79663[27], 6}, {NULL}};
const Segment SGI215323[] = {{&S79663[97], 9}, {NULL}};
const Segment SGI215324[] = {{&S79663[70], 6}, {NULL}};
const Segment SGI215325[] = {{&S79664[34], 11}, {NULL}};
const Segment SGI215326[] = {{&S79664[27], 6}, {NULL}};
const Segment SGI215327[] = {{&S79664[98], 9}, {NULL}};
const Segment SGI215328[] = {{&S79664[71], 6}, {NULL}};
const Segment SGI215329[] = {{&S79665[34], 10}, {NULL}};
const Segment SGI215330[] = {{&S79665[27], 6}, {NULL}};
const Segment SGI215331[] = {{&S79665[97], 9}, {NULL}};
const Segment SGI215332[] = {{&S79665[70], 6}, {NULL}};
const Segment SGI215333[] = {{&S89842[44], 13}, {NULL}};
const Segment SGI215334[] = {{&S89842[44], 25}, {NULL}};
const Segment SGI215335[] = {{&S89842[30], 6}, {NULL}};
const Segment SGI215336[] = {{&S89842[83], 6}, {NULL}};
const Segment SGI215337[] = {{&S89842[110], 13}, {NULL}};
const Segment SGI215338[] = {{&S79667[34], 10}, {NULL}};
const Segment SGI215339[] = {{&S79667[27], 6}, {NULL}};
const Segment SGI215340[] = {{&S89843[44], 14}, {NULL}};
const Segment SGI215341[] = {{&S79667[97], 9}, {NULL}};
const Segment SGI215342[] = {{&S89843[44], 26}, {NULL}};
const Segment SGI215343[] = {{&S79667[70], 6}, {NULL}};
const Segment SGI215344[] = {{&S89843[30], 6}, {NULL}};
const Segment SGI215345[] = {{&S89843[84], 6}, {NULL}};
const Segment SGI215346[] = {{&S89843[111], 13}, {NULL}};
const Segment SGI215347[] = {{&S79668[34], 10}, {NULL}};
const Segment SGI215348[] = {{&S79668[27], 6}, {NULL}};
const Segment SGI215349[] = {{&S79668[97], 9}, {NULL}};
const Segment SGI215350[] = {{&S79668[70], 6}, {NULL}};
const Segment SGI215351[] = {{&S89844[44], 7}, {NULL}};
const Segment SGI215352[] = {{&S89844[44], 24}, {NULL}};
const Segment SGI215353[] = {{&S89844[30], 6}, {NULL}};
const Segment SGI215354[] = {{&S89844[82], 6}, {NULL}};
const Segment SGI215355[] = {{&S89844[109], 13}, {NULL}};
const Segment SGI215356[] = {{&S71009[34], 10}, {NULL}};
const Segment SGI215357[] = {{&S79669[34], 9}, {NULL}};
const Segment SGI215358[] = {{&S71009[27], 6}, {NULL}};
const Segment SGI215359[] = {{&S79669[27], 6}, {NULL}};
const Segment SGI215360[] = {{&S71009[97], 9}, {NULL}};
const Segment SGI215361[] = {{&S79669[96], 9}, {NULL}};
const Segment SGI215362[] = {{&S71009[70], 6}, {NULL}};
const Segment SGI215363[] = {{&S79669[69], 6}, {NULL}};
const Segment SGI215364[] = {{&S89846[44], 7}, {NULL}};
const Segment SGI215365[] = {{&S89846[44], 19}, {NULL}};
const Segment SGI215366[] = {{&S79670[34], 10}, {NULL}};
const Segment SGI215367[] = {{&S89846[30], 6}, {NULL}};
const Segment SGI215368[] = {{&S79670[27], 6}, {NULL}};
const Segment SGI215369[] = {{&S89846[77], 6}, {NULL}};
const Segment SGI215370[] = {{&S79670[97], 9}, {NULL}};
const Segment SGI215371[] = {{&S79670[70], 6}, {NULL}};
const Segment SGI215372[] = {{&S89846[104], 13}, {NULL}};
const Segment SGI215373[] = {{&S79672[34], 9}, {NULL}};
const Segment SGI215374[] = {{&S79672[27], 6}, {NULL}};
const Segment SGI215375[] = {{&S79672[96], 9}, {NULL}};
const Segment SGI215376[] = {{&S79672[69], 6}, {NULL}};
const Segment SGI215377[] = {{&S89849[34], 10}, {NULL}};
const Segment SGI215378[] = {{&S89849[27], 6}, {NULL}};
const Segment SGI215379[] = {{&S89849[97], 9}, {NULL}};
const Segment SGI215380[] = {{&S89849[70], 6}, {NULL}};
const Segment SGI215381[] = {{&S79676[34], 12}, {NULL}};
const Segment SGI215382[] = {{&S79676[27], 6}, {NULL}};
const Segment SGI215383[] = {{&S79676[99], 9}, {NULL}};
const Segment SGI215384[] = {{&S79676[72], 6}, {NULL}};
const Segment SGI215385[] = {{&S71027[44], 10}, {NULL}};
const Segment SGI215386[] = {{&S71027[44], 21}, {NULL}};
const Segment SGI215387[] = {{&S71027[30], 6}, {NULL}};
const Segment SGI215388[] = {{&S71027[79], 6}, {NULL}};
const Segment SGI215389[] = {{&S71027[106], 13}, {NULL}};
const Segment SGI215390[] = {{&S89876[44], 14}, {NULL}};
const Segment SGI215391[] = {{&S89876[44], 26}, {NULL}};
const Segment SGI215392[] = {{&S89876[30], 6}, {NULL}};
const Segment SGI215393[] = {{&S89876[31], 3}, {NULL}};
const Segment SGI215394[] = {{&S89876[113], 6}, {NULL}};
const Segment SGI215395[] = {{&S89876[72], 8}, {NULL}};
const Segment SGI215396[] = {{&S89876[84], 15}, {NULL}};
const Segment SGI215397[] = {{&S79930[44], 8}, {NULL}};
const Segment SGI215398[] = {{&S79930[44], 20}, {NULL}};
const Segment SGI215399[] = {{&S79930[30], 6}, {NULL}};
const Segment SGI215400[] = {{&S79930[113], 3}, {NULL}};
const Segment SGI215401[] = {{&S79930[78], 6}, {NULL}};
const Segment SGI215402[] = {{&S89877[44], 14}, {NULL}};
const Segment SGI215403[] = {{&S89877[44], 26}, {NULL}};
const Segment SGI215404[] = {{&S89877[30], 6}, {NULL}};
const Segment SGI215405[] = {{&S89877[31], 3}, {NULL}};
const Segment SGI215406[] = {{&S89877[113], 6}, {NULL}};
const Segment SGI215407[] = {{&S79931[44], 8}, {NULL}};
const Segment SGI215408[] = {{&S79931[44], 20}, {NULL}};
const Segment SGI215409[] = {{&S89877[72], 8}, {NULL}};
const Segment SGI215410[] = {{&S79931[30], 6}, {NULL}};
const Segment SGI215411[] = {{&S89877[84], 15}, {NULL}};
const Segment SGI215412[] = {{&S79931[113], 3}, {NULL}};
const Segment SGI215413[] = {{&S79931[78], 6}, {NULL}};
const Segment SGI215414[] = {{&S89890[34], 7}, {NULL}};
const Segment SGI215415[] = {{&S89890[27], 6}, {NULL}};
const Segment SGI215416[] = {{&S89890[94], 9}, {NULL}};
const Segment SGI215417[] = {{&S89890[67], 6}, {NULL}};
const Segment SGI215418[] = {{&S89892[34], 13}, {NULL}};
const Segment SGI215419[] = {{&S89892[27], 6}, {NULL}};
const Segment SGI215420[] = {{&S89892[100], 9}, {NULL}};
const Segment SGI215421[] = {{&S89892[73], 6}, {NULL}};
const Segment SGI215422[] = {{&S71062[34], 18}, {NULL}};
const Segment SGI215423[] = {{&S71062[27], 6}, {NULL}};
const Segment SGI215424[] = {{&S85002[44], 10}, {NULL}};
const Segment SGI215425[] = {{&S71062[104], 9}, {NULL}};
const Segment SGI215426[] = {{&S85002[44], 22}, {NULL}};
const Segment SGI215427[] = {{&S71062[77], 6}, {NULL}};
const Segment SGI215428[] = {{&S85002[30], 6}, {NULL}};
const Segment SGI215429[] = {{&S85002[115], 3}, {NULL}};
const Segment SGI215430[] = {{&S85002[80], 6}, {NULL}};
const Segment SGI215431[] = {{&S71063[34], 18}, {NULL}};
const Segment SGI215432[] = {{&S71063[27], 6}, {NULL}};
const Segment SGI215433[] = {{&S71063[104], 9}, {NULL}};
const Segment SGI215434[] = {{&S85003[34], 16}, {NULL}};
const Segment SGI215435[] = {{&S71063[77], 6}, {NULL}};
const Segment SGI215436[] = {{&S85003[27], 6}, {NULL}};
const Segment SGI215437[] = {{&S89899[34], 8}, {NULL}};
const Segment SGI215438[] = {{&S89899[27], 6}, {NULL}};
const Segment SGI215439[] = {{&S85003[113], 9}, {NULL}};
const Segment SGI215440[] = {{&S85003[86], 6}, {NULL}};
const Segment SGI215441[] = {{&S89899[94], 9}, {NULL}};
const Segment SGI215442[] = {{&S89899[67], 6}, {NULL}};
const Segment SGI215443[] = {{&S71064[34], 11}, {NULL}};
const Segment SGI215444[] = {{&S71064[27], 6}, {NULL}};
const Segment SGI215445[] = {{&S71064[97], 9}, {NULL}};
const Segment SGI215446[] = {{&S71064[70], 6}, {NULL}};
const Segment SGI215447[] = {{&S89901[34], 11}, {NULL}};
const Segment SGI215448[] = {{&S89901[27], 6}, {NULL}};
const Segment SGI215449[] = {{&S89901[98], 9}, {NULL}};
const Segment SGI215450[] = {{&S89901[71], 6}, {NULL}};
const Segment SGI215451[] = {{&S85008[34], 25}, {NULL}};
const Segment SGI215452[] = {{&S85008[27], 6}, {NULL}};
const Segment SGI215453[] = {{&S85008[112], 9}, {NULL}};
const Segment SGI215454[] = {{&S85008[85], 6}, {NULL}};
const Segment SGI215455[] = {{&S89908[34], 15}, {NULL}};
const Segment SGI215456[] = {{&S89908[27], 6}, {NULL}};
const Segment SGI215457[] = {{&S89908[102], 9}, {NULL}};
const Segment SGI215458[] = {{&S89908[75], 6}, {NULL}};
const Segment SGI215459[] = {{&S89910[34], 7}, {NULL}};
const Segment SGI215460[] = {{&S89910[27], 6}, {NULL}};
const Segment SGI215461[] = {{&S89910[94], 9}, {NULL}};
const Segment SGI215462[] = {{&S89910[67], 6}, {NULL}};
const Segment SGI215463[] = {{&S85018[44], 19}, {NULL}};
const Segment SGI215464[] = {{&S85018[44], 38}, {NULL}};
const Segment SGI215465[] = {{&S85018[30], 6}, {NULL}};
const Segment SGI215466[] = {{&S85018[131], 3}, {NULL}};
const Segment SGI215467[] = {{&S85018[96], 6}, {NULL}};
const Segment SGI215468[] = {{&S85019[44], 19}, {NULL}};
const Segment SGI215469[] = {{&S85019[44], 38}, {NULL}};
const Segment SGI215470[] = {{&S85019[30], 6}, {NULL}};
const Segment SGI215471[] = {{&S85019[131], 3}, {NULL}};
const Segment SGI215472[] = {{&S85019[96], 6}, {NULL}};
const Segment SGI215473[] = {{&S89917[34], 7}, {NULL}};
const Segment SGI215474[] = {{&S89917[27], 6}, {NULL}};
const Segment SGI215475[] = {{&S89917[99], 9}, {NULL}};
const Segment SGI215476[] = {{&S89917[72], 6}, {NULL}};
const Segment SGI215477[] = {{&S89920[34], 7}, {NULL}};
const Segment SGI215478[] = {{&S89920[27], 6}, {NULL}};
const Segment SGI215479[] = {{&S89920[94], 9}, {NULL}};
const Segment SGI215480[] = {{&S89920[67], 6}, {NULL}};
const Segment SGI215481[] = {{&S85027[44], 15}, {NULL}};
const Segment SGI215482[] = {{&S85027[44], 27}, {NULL}};
const Segment SGI215483[] = {{&S85027[30], 6}, {NULL}};
const Segment SGI215484[] = {{&S85027[85], 6}, {NULL}};
const Segment SGI215485[] = {{&S85027[112], 13}, {NULL}};
const Segment SGI215486[] = {{&S89925[45], 10}, {NULL}};
const Segment SGI215487[] = {{&S89925[30], 6}, {NULL}};
const Segment SGI215488[] = {{&S89925[31], 3}, {NULL}};
const Segment SGI215489[] = {{&S89925[81], 5}, {NULL}};
const Segment SGI215490[] = {{&S89925[139], 13}, {NULL}};
const Segment SGI215491[] = {{&S89929[44], 10}, {NULL}};
const Segment SGI215492[] = {{&S89929[44], 22}, {NULL}};
const Segment SGI215493[] = {{&S89929[30], 6}, {NULL}};
const Segment SGI215494[] = {{&S89929[31], 3}, {NULL}};
const Segment SGI215495[] = {{&S89929[80], 6}, {NULL}};
const Segment SGI215496[] = {{&S89937[44], 13}, {NULL}};
const Segment SGI215497[] = {{&S89937[44], 25}, {NULL}};
const Segment SGI215498[] = {{&S89937[30], 6}, {NULL}};
const Segment SGI215499[] = {{&S89937[31], 3}, {NULL}};
const Segment SGI215500[] = {{&S89937[83], 6}, {NULL}};
const Segment SGI215501[] = {{&S89938[34], 9}, {NULL}};
const Segment SGI215502[] = {{&S89938[27], 6}, {NULL}};
const Segment SGI215503[] = {{&S89938[96], 9}, {NULL}};
const Segment SGI215504[] = {{&S89938[69], 6}, {NULL}};
const Segment SGI215505[] = {{&S80010[44], 11}, {NULL}};
const Segment SGI215506[] = {{&S80010[44], 26}, {NULL}};
const Segment SGI215507[] = {{&S80010[30], 6}, {NULL}};
const Segment SGI215508[] = {{&S80010[137], 3}, {NULL}};
const Segment SGI215509[] = {{&S80010[102], 6}, {NULL}};
const Segment SGI215510[] = {{&S80010[72], 8}, {NULL}};
const Segment SGI215511[] = {{&S89942[34], 8}, {NULL}};
const Segment SGI215512[] = {{&S89942[27], 6}, {NULL}};
const Segment SGI215513[] = {{&S89942[95], 9}, {NULL}};
const Segment SGI215514[] = {{&S89942[68], 6}, {NULL}};
const Segment SGI215515[] = {{&S89943[34], 11}, {NULL}};
const Segment SGI215516[] = {{&S89943[27], 6}, {NULL}};
const Segment SGI215517[] = {{&S89943[98], 9}, {NULL}};
const Segment SGI215518[] = {{&S89943[71], 6}, {NULL}};
const Segment SGI215519[] = {{&S89944[34], 13}, {NULL}};
const Segment SGI215520[] = {{&S89944[27], 6}, {NULL}};
const Segment SGI215521[] = {{&S89944[100], 9}, {NULL}};
const Segment SGI215522[] = {{&S89944[73], 6}, {NULL}};
const Segment SGI215523[] = {{&S80022[44], 7}, {NULL}};
const Segment SGI215524[] = {{&S89950[34], 10}, {NULL}};
const Segment SGI215525[] = {{&S80022[44], 19}, {NULL}};
const Segment SGI215526[] = {{&S80022[30], 6}, {NULL}};
const Segment SGI215527[] = {{&S89950[27], 6}, {NULL}};
const Segment SGI215528[] = {{&S80022[112], 3}, {NULL}};
const Segment SGI215529[] = {{&S89950[97], 9}, {NULL}};
const Segment SGI215530[] = {{&S80022[77], 6}, {NULL}};
const Segment SGI215531[] = {{&S89950[70], 6}, {NULL}};
const Segment SGI215532[] = {{&S89951[34], 10}, {NULL}};
const Segment SGI215533[] = {{&S89951[27], 6}, {NULL}};
const Segment SGI215534[] = {{&S89951[97], 9}, {NULL}};
const Segment SGI215535[] = {{&S89951[70], 6}, {NULL}};
const Segment SGI215536[] = {{&S89952[34], 10}, {NULL}};
const Segment SGI215537[] = {{&S89952[27], 6}, {NULL}};
const Segment SGI215538[] = {{&S89952[97], 9}, {NULL}};
const Segment SGI215539[] = {{&S89952[70], 6}, {NULL}};
const Segment SGI215540[] = {{&S89955[44], 20}, {NULL}};
const Segment SGI215541[] = {{&S89955[44], 32}, {NULL}};
const Segment SGI215542[] = {{&S89955[30], 6}, {NULL}};
const Segment SGI215543[] = {{&S89955[31], 3}, {NULL}};
const Segment SGI215544[] = {{&S89955[90], 6}, {NULL}};
const Segment SGI215545[] = {{&S89958[34], 15}, {NULL}};
const Segment SGI215546[] = {{&S89958[27], 6}, {NULL}};
const Segment SGI215547[] = {{&S89958[102], 9}, {NULL}};
const Segment SGI215548[] = {{&S89958[75], 6}, {NULL}};
const Segment SGI215549[] = {{&S89960[34], 16}, {NULL}};
const Segment SGI215550[] = {{&S89960[27], 6}, {NULL}};
const Segment SGI215551[] = {{&S89960[103], 9}, {NULL}};
const Segment SGI215552[] = {{&S89960[76], 6}, {NULL}};
const Segment SGI215553[] = {{&S89965[34], 18}, {NULL}};
const Segment SGI215554[] = {{&S89965[27], 6}, {NULL}};
const Segment SGI215555[] = {{&S89965[105], 9}, {NULL}};
const Segment SGI215556[] = {{&S89965[78], 6}, {NULL}};
const Segment SGI215557[] = {{&S89965[146], 8}, {NULL}};
const Segment SGI215558[] = {{&S89971[44], 13}, {NULL}};
const Segment SGI215559[] = {{&S89971[44], 25}, {NULL}};
const Segment SGI215560[] = {{&S89971[30], 6}, {NULL}};
const Segment SGI215561[] = {{&S71156[44], 7}, {NULL}};
const Segment SGI215562[] = {{&S71156[44], 25}, {NULL}};
const Segment SGI215563[] = {{&S71156[30], 6}, {NULL}};
const Segment SGI215564[] = {{&S89971[31], 3}, {NULL}};
const Segment SGI215565[] = {{&S89971[83], 5}, {NULL}};
const Segment SGI215566[] = {{&S71156[118], 3}, {NULL}};
const Segment SGI215567[] = {{&S89971[120], 14}, {NULL}};
const Segment SGI215568[] = {{&S71156[83], 6}, {NULL}};
const Segment SGI215569[] = {{&S71157[38], 7}, {NULL}};
const Segment SGI215570[] = {{&S71157[31], 6}, {NULL}};
const Segment SGI215571[] = {{&S85098[44], 7}, {NULL}};
const Segment SGI215572[] = {{&S85098[44], 19}, {NULL}};
const Segment SGI215573[] = {{&S85098[30], 6}, {NULL}};
const Segment SGI215574[] = {{&S71158[44], 7}, {NULL}};
const Segment SGI215575[] = {{&S71158[44], 27}, {NULL}};
const Segment SGI215576[] = {{&S71158[30], 6}, {NULL}};
const Segment SGI215577[] = {{&S85098[112], 3}, {NULL}};
const Segment SGI215578[] = {{&S85098[77], 6}, {NULL}};
const Segment SGI215579[] = {{&S71158[120], 3}, {NULL}};
const Segment SGI215580[] = {{&S89975[44], 7}, {NULL}};
const Segment SGI215581[] = {{&S71158[85], 6}, {NULL}};
const Segment SGI215582[] = {{&S89975[44], 25}, {NULL}};
const Segment SGI215583[] = {{&S89975[30], 6}, {NULL}};
const Segment SGI215584[] = {{&S85099[38], 7}, {NULL}};
const Segment SGI215585[] = {{&S89975[83], 6}, {NULL}};
const Segment SGI215586[] = {{&S85099[31], 6}, {NULL}};
const Segment SGI215587[] = {{&S89975[110], 13}, {NULL}};
const Segment SGI215588[] = {{&S71163[34], 7}, {NULL}};
const Segment SGI215589[] = {{&S89979[44], 10}, {NULL}};
const Segment SGI215590[] = {{&S71163[27], 6}, {NULL}};
const Segment SGI215591[] = {{&S89979[44], 22}, {NULL}};
const Segment SGI215592[] = {{&S89979[30], 6}, {NULL}};
const Segment SGI215593[] = {{&S85104[34], 11}, {NULL}};
const Segment SGI215594[] = {{&S71163[94], 9}, {NULL}};
const Segment SGI215595[] = {{&S85104[27], 6}, {NULL}};
const Segment SGI215596[] = {{&S71163[67], 6}, {NULL}};
const Segment SGI215597[] = {{&S89979[80], 6}, {NULL}};
const Segment SGI215598[] = {{&S89979[107], 13}, {NULL}};
const Segment SGI215599[] = {{&S85104[98], 9}, {NULL}};
const Segment SGI215600[] = {{&S85104[71], 6}, {NULL}};
const Segment SGI215601[] = {{&S89980[44], 5}, {NULL}};
const Segment SGI215602[] = {{&S89980[44], 17}, {NULL}};
const Segment SGI215603[] = {{&S89980[30], 6}, {NULL}};
const Segment SGI215604[] = {{&S85105[34], 25}, {NULL}};
const Segment SGI215605[] = {{&S85105[27], 6}, {NULL}};
const Segment SGI215606[] = {{&S89980[75], 6}, {NULL}};
const Segment SGI215607[] = {{&S89980[102], 13}, {NULL}};
const Segment SGI215608[] = {{&S85105[112], 9}, {NULL}};
const Segment SGI215609[] = {{&S85105[85], 6}, {NULL}};
const Segment SGI215610[] = {{&S89989[34], 7}, {NULL}};
const Segment SGI215611[] = {{&S89989[27], 6}, {NULL}};
const Segment SGI215612[] = {{&S89989[99], 9}, {NULL}};
const Segment SGI215613[] = {{&S89989[72], 6}, {NULL}};
const Segment SGI215614[] = {{&S85116[34], 13}, {NULL}};
const Segment SGI215615[] = {{&S89990[34], 8}, {NULL}};
const Segment SGI215616[] = {{&S85116[27], 6}, {NULL}};
const Segment SGI215617[] = {{&S89990[27], 6}, {NULL}};
const Segment SGI215618[] = {{&S89990[101], 9}, {NULL}};
const Segment SGI215619[] = {{&S85116[114], 9}, {NULL}};
const Segment SGI215620[] = {{&S89990[74], 6}, {NULL}};
const Segment SGI215621[] = {{&S85116[87], 6}, {NULL}};
const Segment SGI215622[] = {{&S89992[44], 14}, {NULL}};
const Segment SGI215623[] = {{&S89992[44], 26}, {NULL}};
const Segment SGI215624[] = {{&S89992[30], 6}, {NULL}};
const Segment SGI215625[] = {{&S89992[84], 6}, {NULL}};
const Segment SGI215626[] = {{&S89992[111], 13}, {NULL}};
const Segment SGI215627[] = {{&S79711[34], 6}, {NULL}};
const Segment SGI215628[] = {{&S79711[27], 6}, {NULL}};
const Segment SGI215629[] = {{&S79711[93], 9}, {NULL}};
const Segment SGI215630[] = {{&S79711[66], 6}, {NULL}};
const Segment SGI215631[] = {{&S79716[44], 10}, {NULL}};
const Segment SGI215632[] = {{&S79716[44], 22}, {NULL}};
const Segment SGI215633[] = {{&S90000[44], 5}, {NULL}};
const Segment SGI215634[] = {{&S79716[30], 6}, {NULL}};
const Segment SGI215635[] = {{&S90000[44], 17}, {NULL}};
const Segment SGI215636[] = {{&S90000[30], 6}, {NULL}};
const Segment SGI215637[] = {{&S79716[80], 6}, {NULL}};
const Segment SGI215638[] = {{&S71188[34], 7}, {NULL}};
const Segment SGI215639[] = {{&S90000[75], 6}, {NULL}};
const Segment SGI215640[] = {{&S79716[107], 13}, {NULL}};
const Segment SGI215641[] = {{&S71188[27], 6}, {NULL}};
const Segment SGI215642[] = {{&S90000[102], 13}, {NULL}};
const Segment SGI215643[] = {{&S71188[93], 9}, {NULL}};
const Segment SGI215644[] = {{&S71188[66], 6}, {NULL}};
const Segment SGI215645[] = {{&S85129[34], 14}, {NULL}};
const Segment SGI215646[] = {{&S85129[27], 6}, {NULL}};
const Segment SGI215647[] = {{&S85129[101], 9}, {NULL}};
const Segment SGI215648[] = {{&S85129[74], 6}, {NULL}};
const Segment SGI215649[] = {{&S71189[34], 11}, {NULL}};
const Segment SGI215650[] = {{&S71189[27], 6}, {NULL}};
const Segment SGI215651[] = {{&S90002[44], 5}, {NULL}};
const Segment SGI215652[] = {{&S90002[44], 17}, {NULL}};
const Segment SGI215653[] = {{&S90002[30], 6}, {NULL}};
const Segment SGI215654[] = {{&S71189[97], 9}, {NULL}};
const Segment SGI215655[] = {{&S71189[70], 6}, {NULL}};
const Segment SGI215656[] = {{&S90002[75], 6}, {NULL}};
const Segment SGI215657[] = {{&S90002[102], 13}, {NULL}};
const Segment SGI215658[] = {{&S85131[34], 13}, {NULL}};
const Segment SGI215659[] = {{&S85131[27], 6}, {NULL}};
const Segment SGI215660[] = {{&S79720[45], 15}, {NULL}};
const Segment SGI215661[] = {{&S85131[114], 9}, {NULL}};
const Segment SGI215662[] = {{&S79720[30], 6}, {NULL}};
const Segment SGI215663[] = {{&S85131[87], 6}, {NULL}};
const Segment SGI215664[] = {{&S79720[120], 3}, {NULL}};
const Segment SGI215665[] = {{&S79720[86], 5}, {NULL}};
const Segment SGI215666[] = {{&S79720[144], 13}, {NULL}};
const Segment SGI215667[] = {{&S90004[34], 12}, {NULL}};
const Segment SGI215668[] = {{&S90004[27], 6}, {NULL}};
const Segment SGI215669[] = {{&S90004[96], 9}, {NULL}};
const Segment SGI215670[] = {{&S90004[69], 6}, {NULL}};
const Segment SGI215671[] = {{&S79721[44], 15}, {NULL}};
const Segment SGI215672[] = {{&S79721[44], 27}, {NULL}};
const Segment SGI215673[] = {{&S79721[30], 6}, {NULL}};
const Segment SGI215674[] = {{&S79721[120], 3}, {NULL}};
const Segment SGI215675[] = {{&S79721[85], 6}, {NULL}};
const Segment SGI215676[] = {{&S79721[140], 8}, {NULL}};
const Segment SGI215677[] = {{&S79721[149], 13}, {NULL}};
const Segment SGI215678[] = {{&S79732[34], 28}, {NULL}};
const Segment SGI215679[] = {{&S79732[27], 6}, {NULL}};
const Segment SGI215680[] = {{&S79732[115], 9}, {NULL}};
const Segment SGI215681[] = {{&S79732[88], 6}, {NULL}};
const Segment SGI215682[] = {{&S79738[34], 7}, {NULL}};
const Segment SGI215683[] = {{&S79738[27], 6}, {NULL}};
const Segment SGI215684[] = {{&S79738[94], 9}, {NULL}};
const Segment SGI215685[] = {{&S79738[67], 6}, {NULL}};
const Segment SGI215686[] = {{&S79739[34], 10}, {NULL}};
const Segment SGI215687[] = {{&S79739[27], 6}, {NULL}};
const Segment SGI215688[] = {{&S79739[97], 9}, {NULL}};
const Segment SGI215689[] = {{&S79739[70], 6}, {NULL}};
const Segment SGI215690[] = {{&S79742[44], 10}, {NULL}};
const Segment SGI215691[] = {{&S79742[44], 22}, {NULL}};
const Segment SGI215692[] = {{&S79742[30], 6}, {NULL}};
const Segment SGI215693[] = {{&S79742[113], 3}, {NULL}};
const Segment SGI215694[] = {{&S79742[80], 5}, {NULL}};
const Segment SGI215695[] = {{&S79742[117], 14}, {NULL}};
const Segment SGI215696[] = {{&S90054[34], 7}, {NULL}};
const Segment SGI215697[] = {{&S90054[27], 6}, {NULL}};
const Segment SGI215698[] = {{&S90054[94], 9}, {NULL}};
const Segment SGI215699[] = {{&S90054[67], 6}, {NULL}};
const Segment SGI215700[] = {{&S71245[44], 7}, {NULL}};
const Segment SGI215701[] = {{&S71245[44], 18}, {NULL}};
const Segment SGI215702[] = {{&S71245[30], 6}, {NULL}};
const Segment SGI215703[] = {{&S71245[111], 3}, {NULL}};
const Segment SGI215704[] = {{&S71245[76], 6}, {NULL}};
const Segment SGI215705[] = {{&S71246[38], 7}, {NULL}};
const Segment SGI215706[] = {{&S71246[31], 6}, {NULL}};
const Segment SGI215707[] = {{&S80160[34], 11}, {NULL}};
const Segment SGI215708[] = {{&S80160[27], 6}, {NULL}};
const Segment SGI215709[] = {{&S80160[98], 9}, {NULL}};
const Segment SGI215710[] = {{&S80160[71], 6}, {NULL}};
const Segment SGI215711[] = {{&S90062[27], 6}, {NULL}};
const Segment SGI215712[] = {{&S79770[44], 10}, {NULL}};
const Segment SGI215713[] = {{&S79770[44], 22}, {NULL}};
const Segment SGI215714[] = {{&S79770[30], 6}, {NULL}};
const Segment SGI215715[] = {{&S90062[108], 9}, {NULL}};
const Segment SGI215716[] = {{&S90062[81], 6}, {NULL}};
const Segment SGI215717[] = {{&S79770[131], 3}, {NULL}};
const Segment SGI215718[] = {{&S79770[96], 6}, {NULL}};
const Segment SGI215719[] = {{&S79770[68], 8}, {NULL}};
const Segment SGI215720[] = {{&S79771[44], 10}, {NULL}};
const Segment SGI215721[] = {{&S79771[44], 47}, {NULL}};
const Segment SGI215722[] = {{&S79771[30], 6}, {NULL}};
const Segment SGI215723[] = {{&S90064[34], 7}, {NULL}};
const Segment SGI215724[] = {{&S79771[156], 3}, {NULL}};
const Segment SGI215725[] = {{&S90064[27], 6}, {NULL}};
const Segment SGI215726[] = {{&S79771[121], 6}, {NULL}};
const Segment SGI215727[] = {{&S79771[93], 8}, {NULL}};
const Segment SGI215728[] = {{&S90064[93], 9}, {NULL}};
const Segment SGI215729[] = {{&S90064[66], 6}, {NULL}};
const Segment SGI215730[] = {{&S90066[34], 8}, {NULL}};
const Segment SGI215731[] = {{&S90066[27], 6}, {NULL}};
const Segment SGI215732[] = {{&S90066[94], 9}, {NULL}};
const Segment SGI215733[] = {{&S90066[67], 6}, {NULL}};
const Segment SGI215734[] = {{&S90068[34], 18}, {NULL}};
const Segment SGI215735[] = {{&S90068[27], 6}, {NULL}};
const Segment SGI215736[] = {{&S90068[105], 9}, {NULL}};
const Segment SGI215737[] = {{&S90068[78], 6}, {NULL}};
const Segment SGI215738[] = {{&S79781[34], 11}, {NULL}};
const Segment SGI215739[] = {{&S79781[27], 6}, {NULL}};
const Segment SGI215740[] = {{&S79781[98], 9}, {NULL}};
const Segment SGI215741[] = {{&S79781[71], 6}, {NULL}};
const Segment SGI215742[] = {{&S79782[34], 11}, {NULL}};
const Segment SGI215743[] = {{&S79782[27], 6}, {NULL}};
const Segment SGI215744[] = {{&S79782[98], 9}, {NULL}};
const Segment SGI215745[] = {{&S79782[71], 6}, {NULL}};
const Segment SGI215746[] = {{&S79783[34], 11}, {NULL}};
const Segment SGI215747[] = {{&S79783[27], 6}, {NULL}};
const Segment SGI215748[] = {{&S85210[44], 21}, {NULL}};
const Segment SGI215749[] = {{&S85210[44], 33}, {NULL}};
const Segment SGI215750[] = {{&S79783[98], 9}, {NULL}};
const Segment SGI215751[] = {{&S85210[30], 6}, {NULL}};
const Segment SGI215752[] = {{&S79783[71], 6}, {NULL}};
const Segment SGI215753[] = {{&S85210[126], 3}, {NULL}};
const Segment SGI215754[] = {{&S85210[91], 6}, {NULL}};
const Segment SGI215755[] = {{&S90078[34], 7}, {NULL}};
const Segment SGI215756[] = {{&S90078[27], 6}, {NULL}};
const Segment SGI215757[] = {{&S90078[98], 9}, {NULL}};
const Segment SGI215758[] = {{&S90078[71], 6}, {NULL}};
const Segment SGI215759[] = {{&S79785[44], 10}, {NULL}};
const Segment SGI215760[] = {{&S79785[44], 22}, {NULL}};
const Segment SGI215761[] = {{&S90079[34], 7}, {NULL}};
const Segment SGI215762[] = {{&S79785[30], 6}, {NULL}};
const Segment SGI215763[] = {{&S90079[27], 6}, {NULL}};
const Segment SGI215764[] = {{&S79785[115], 3}, {NULL}};
const Segment SGI215765[] = {{&S90079[99], 9}, {NULL}};
const Segment SGI215766[] = {{&S90079[72], 6}, {NULL}};
const Segment SGI215767[] = {{&S79785[80], 6}, {NULL}};
const Segment SGI215768[] = {{&S79785[143], 3}, {NULL}};
const Segment SGI215769[] = {{&S79787[44], 10}, {NULL}};
const Segment SGI215770[] = {{&S79787[44], 22}, {NULL}};
const Segment SGI215771[] = {{&S79787[30], 6}, {NULL}};
const Segment SGI215772[] = {{&S79787[115], 3}, {NULL}};
const Segment SGI215773[] = {{&S79787[80], 6}, {NULL}};
const Segment SGI215774[] = {{&S79787[150], 3}, {NULL}};
const Segment SGI215775[] = {{&S90085[44], 11}, {NULL}};
const Segment SGI215776[] = {{&S90085[44], 23}, {NULL}};
const Segment SGI215777[] = {{&S90085[30], 6}, {NULL}};
const Segment SGI215778[] = {{&S90085[31], 3}, {NULL}};
const Segment SGI215779[] = {{&S90085[81], 6}, {NULL}};
const Segment SGI215780[] = {{&S90085[134], 11}, {NULL}};
const Segment SGI215781[] = {{&S90086[44], 10}, {NULL}};
const Segment SGI215782[] = {{&S90086[44], 22}, {NULL}};
const Segment SGI215783[] = {{&S90086[30], 6}, {NULL}};
const Segment SGI215784[] = {{&S90086[31], 3}, {NULL}};
const Segment SGI215785[] = {{&S90086[80], 6}, {NULL}};
const Segment SGI215786[] = {{&S90086[133], 11}, {NULL}};
const Segment SGI215787[] = {{&S90087[44], 10}, {NULL}};
const Segment SGI215788[] = {{&S90087[44], 22}, {NULL}};
const Segment SGI215789[] = {{&S90087[30], 6}, {NULL}};
const Segment SGI215790[] = {{&S90087[31], 3}, {NULL}};
const Segment SGI215791[] = {{&S90087[80], 6}, {NULL}};
const Segment SGI215792[] = {{&S90087[134], 12}, {NULL}};
const Segment SGI215793[] = {{&S79797[44], 13}, {NULL}};
const Segment SGI215794[] = {{&S79797[44], 25}, {NULL}};
const Segment SGI215795[] = {{&S79797[30], 6}, {NULL}};
const Segment SGI215796[] = {{&S79797[83], 6}, {NULL}};
const Segment SGI215797[] = {{&S79797[110], 13}, {NULL}};
const Segment SGI215798[] = {{&S90093[34], 13}, {NULL}};
const Segment SGI215799[] = {{&S90093[27], 6}, {NULL}};
const Segment SGI215800[] = {{&S90093[100], 9}, {NULL}};
const Segment SGI215801[] = {{&S90093[73], 6}, {NULL}};
const Segment SGI215802[] = {{&S90097[34], 7}, {NULL}};
const Segment SGI215803[] = {{&S90097[27], 6}, {NULL}};
const Segment SGI215804[] = {{&S90097[94], 9}, {NULL}};
const Segment SGI215805[] = {{&S90097[67], 6}, {NULL}};
const Segment SGI215806[] = {{&S90098[34], 16}, {NULL}};
const Segment SGI215807[] = {{&S90098[27], 6}, {NULL}};
const Segment SGI215808[] = {{&S90098[116], 9}, {NULL}};
const Segment SGI215809[] = {{&S90098[89], 6}, {NULL}};
const Segment SGI215810[] = {{&S90099[34], 19}, {NULL}};
const Segment SGI215811[] = {{&S90099[27], 6}, {NULL}};
const Segment SGI215812[] = {{&S90099[119], 9}, {NULL}};
const Segment SGI215813[] = {{&S90099[92], 6}, {NULL}};
const Segment SGI215814[] = {{&S90101[44], 15}, {NULL}};
const Segment SGI215815[] = {{&S90101[44], 33}, {NULL}};
const Segment SGI215816[] = {{&S90101[30], 6}, {NULL}};
const Segment SGI215817[] = {{&S90101[91], 6}, {NULL}};
const Segment SGI215818[] = {{&S90101[118], 13}, {NULL}};
const Segment SGI215819[] = {{&S90102[44], 15}, {NULL}};
const Segment SGI215820[] = {{&S90102[44], 33}, {NULL}};
const Segment SGI215821[] = {{&S90102[30], 6}, {NULL}};
const Segment SGI215822[] = {{&S90102[91], 6}, {NULL}};
const Segment SGI215823[] = {{&S90102[118], 13}, {NULL}};
const Segment SGI215824[] = {{&S90104[34], 20}, {NULL}};
const Segment SGI215825[] = {{&S90104[27], 6}, {NULL}};
const Segment SGI215826[] = {{&S90104[107], 9}, {NULL}};
const Segment SGI215827[] = {{&S90104[80], 6}, {NULL}};
const Segment SGI215828[] = {{&S90105[34], 21}, {NULL}};
const Segment SGI215829[] = {{&S90105[27], 6}, {NULL}};
const Segment SGI215830[] = {{&S90105[108], 9}, {NULL}};
const Segment SGI215831[] = {{&S90105[81], 6}, {NULL}};
const Segment SGI215832[] = {{&S85263[44], 10}, {NULL}};
const Segment SGI215833[] = {{&S85263[44], 22}, {NULL}};
const Segment SGI215834[] = {{&S85263[30], 6}, {NULL}};
const Segment SGI215835[] = {{&S85263[135], 3}, {NULL}};
const Segment SGI215836[] = {{&S85263[100], 6}, {NULL}};
const Segment SGI215837[] = {{&S85263[68], 8}, {NULL}};
const Segment SGI215838[] = {{&S90114[34], 7}, {NULL}};
const Segment SGI215839[] = {{&S90114[27], 6}, {NULL}};
const Segment SGI215840[] = {{&S90114[99], 9}, {NULL}};
const Segment SGI215841[] = {{&S90114[72], 6}, {NULL}};
const Segment SGI215842[] = {{&S90114[140], 8}, {NULL}};
const Segment SGI215843[] = {{&S85268[34], 6}, {NULL}};
const Segment SGI215844[] = {{&S85268[27], 6}, {NULL}};
const Segment SGI215845[] = {{&S85268[93], 9}, {NULL}};
const Segment SGI215846[] = {{&S85268[66], 6}, {NULL}};
const Segment SGI215847[] = {{&S79840[44], 7}, {NULL}};
const Segment SGI215848[] = {{&S79840[44], 19}, {NULL}};
const Segment SGI215849[] = {{&S79840[30], 6}, {NULL}};
const Segment SGI215850[] = {{&S80243[34], 28}, {NULL}};
const Segment SGI215851[] = {{&S79840[128], 3}, {NULL}};
const Segment SGI215852[] = {{&S80243[27], 6}, {NULL}};
const Segment SGI215853[] = {{&S79840[93], 6}, {NULL}};
const Segment SGI215854[] = {{&S79840[65], 8}, {NULL}};
const Segment SGI215855[] = {{&S80243[115], 9}, {NULL}};
const Segment SGI215856[] = {{&S80243[88], 6}, {NULL}};
const Segment SGI215857[] = {{&S90119[44], 9}, {NULL}};
const Segment SGI215858[] = {{&S90119[44], 21}, {NULL}};
const Segment SGI215859[] = {{&S90119[30], 6}, {NULL}};
const Segment SGI215860[] = {{&S90119[31], 3}, {NULL}};
const Segment SGI215861[] = {{&S90119[79], 6}, {NULL}};
const Segment SGI215862[] = {{&S79847[44], 14}, {NULL}};
const Segment SGI215863[] = {{&S79847[44], 26}, {NULL}};
const Segment SGI215864[] = {{&S79847[30], 6}, {NULL}};
const Segment SGI215865[] = {{&S79847[135], 3}, {NULL}};
const Segment SGI215866[] = {{&S79847[100], 6}, {NULL}};
const Segment SGI215867[] = {{&S79847[72], 8}, {NULL}};
const Segment SGI215868[] = {{&S90126[34], 5}, {NULL}};
const Segment SGI215869[] = {{&S90126[27], 6}, {NULL}};
const Segment SGI215870[] = {{&S90126[92], 9}, {NULL}};
const Segment SGI215871[] = {{&S90126[65], 6}, {NULL}};
const Segment SGI215872[] = {{&S85278[44], 15}, {NULL}};
const Segment SGI215873[] = {{&S85278[44], 27}, {NULL}};
const Segment SGI215874[] = {{&S85278[30], 6}, {NULL}};
const Segment SGI215875[] = {{&S85278[120], 3}, {NULL}};
const Segment SGI215876[] = {{&S85278[85], 6}, {NULL}};
const Segment SGI215877[] = {{&S80254[44], 14}, {NULL}};
const Segment SGI215878[] = {{&S80254[44], 26}, {NULL}};
const Segment SGI215879[] = {{&S80254[30], 6}, {NULL}};
const Segment SGI215880[] = {{&S80254[119], 3}, {NULL}};
const Segment SGI215881[] = {{&S80254[84], 6}, {NULL}};
const Segment SGI215882[] = {{&S79857[44], 7}, {NULL}};
const Segment SGI215883[] = {{&S79857[44], 22}, {NULL}};
const Segment SGI215884[] = {{&S79857[30], 6}, {NULL}};
const Segment SGI215885[] = {{&S80258[44], 12}, {NULL}};
const Segment SGI215886[] = {{&S80258[44], 37}, {NULL}};
const Segment SGI215887[] = {{&S80258[30], 6}, {NULL}};
const Segment SGI215888[] = {{&S79857[115], 3}, {NULL}};
const Segment SGI215889[] = {{&S79857[80], 6}, {NULL}};
const Segment SGI215890[] = {{&S80258[130], 3}, {NULL}};
const Segment SGI215891[] = {{&S79857[140], 7}, {NULL}};
const Segment SGI215892[] = {{&S80258[95], 6}, {NULL}};
const Segment SGI215893[] = {{&S79858[44], 7}, {NULL}};
const Segment SGI215894[] = {{&S79858[44], 22}, {NULL}};
const Segment SGI215895[] = {{&S79858[30], 6}, {NULL}};
const Segment SGI215896[] = {{&S90136[34], 4}, {NULL}};
const Segment SGI215897[] = {{&S79858[115], 3}, {NULL}};
const Segment SGI215898[] = {{&S90136[27], 6}, {NULL}};
const Segment SGI215899[] = {{&S79858[80], 6}, {NULL}};
const Segment SGI215900[] = {{&S79858[140], 7}, {NULL}};
const Segment SGI215901[] = {{&S90136[91], 9}, {NULL}};
const Segment SGI215902[] = {{&S85287[44], 25}, {NULL}};
const Segment SGI215903[] = {{&S90136[64], 6}, {NULL}};
const Segment SGI215904[] = {{&S85287[44], 37}, {NULL}};
const Segment SGI215905[] = {{&S85287[30], 6}, {NULL}};
const Segment SGI215906[] = {{&S85287[130], 3}, {NULL}};
const Segment SGI215907[] = {{&S85287[95], 6}, {NULL}};
const Segment SGI215908[] = {{&S79859[44], 7}, {NULL}};
const Segment SGI215909[] = {{&S79859[44], 24}, {NULL}};
const Segment SGI215910[] = {{&S79859[30], 6}, {NULL}};
const Segment SGI215911[] = {{&S79859[117], 3}, {NULL}};
const Segment SGI215912[] = {{&S79859[82], 6}, {NULL}};
const Segment SGI215913[] = {{&S79859[142], 7}, {NULL}};
const Segment SGI215914[] = {{&S85291[44], 10}, {NULL}};
const Segment SGI215915[] = {{&S85291[44], 27}, {NULL}};
const Segment SGI215916[] = {{&S85291[30], 6}, {NULL}};
const Segment SGI215917[] = {{&S85291[85], 6}, {NULL}};
const Segment SGI215918[] = {{&S85291[112], 13}, {NULL}};
const Segment SGI215919[] = {{&S90144[34], 13}, {NULL}};
const Segment SGI215920[] = {{&S90144[27], 6}, {NULL}};
const Segment SGI215921[] = {{&S90144[100], 9}, {NULL}};
const Segment SGI215922[] = {{&S90144[73], 6}, {NULL}};
const Segment SGI215923[] = {{&S79869[34], 11}, {NULL}};
const Segment SGI215924[] = {{&S79869[27], 6}, {NULL}};
const Segment SGI215925[] = {{&S90146[44], 7}, {NULL}};
const Segment SGI215926[] = {{&S90146[44], 24}, {NULL}};
const Segment SGI215927[] = {{&S90146[30], 6}, {NULL}};
const Segment SGI215928[] = {{&S79869[98], 9}, {NULL}};
const Segment SGI215929[] = {{&S79869[71], 6}, {NULL}};
const Segment SGI215930[] = {{&S90146[82], 6}, {NULL}};
const Segment SGI215931[] = {{&S90146[109], 13}, {NULL}};
const Segment SGI215932[] = {{&S79870[34], 12}, {NULL}};
const Segment SGI215933[] = {{&S79870[27], 6}, {NULL}};
const Segment SGI215934[] = {{&S90147[44], 4}, {NULL}};
const Segment SGI215935[] = {{&S90147[44], 16}, {NULL}};
const Segment SGI215936[] = {{&S90147[30], 6}, {NULL}};
const Segment SGI215937[] = {{&S79870[99], 9}, {NULL}};
const Segment SGI215938[] = {{&S79870[72], 6}, {NULL}};
const Segment SGI215939[] = {{&S90147[74], 6}, {NULL}};
const Segment SGI215940[] = {{&S90147[101], 13}, {NULL}};
const Segment SGI215941[] = {{&S85305[34], 8}, {NULL}};
const Segment SGI215942[] = {{&S85305[27], 6}, {NULL}};
const Segment SGI215943[] = {{&S85305[95], 9}, {NULL}};
const Segment SGI215944[] = {{&S85305[68], 6}, {NULL}};
const Segment SGI215945[] = {{&S90158[34], 10}, {NULL}};
const Segment SGI215946[] = {{&S90158[27], 6}, {NULL}};
const Segment SGI215947[] = {{&S90158[108], 9}, {NULL}};
const Segment SGI215948[] = {{&S90158[81], 6}, {NULL}};
const Segment SGI215949[] = {{&S90162[44], 7}, {NULL}};
const Segment SGI215950[] = {{&S90162[44], 24}, {NULL}};
const Segment SGI215951[] = {{&S90162[30], 6}, {NULL}};
const Segment SGI215952[] = {{&S90162[82], 6}, {NULL}};
const Segment SGI215953[] = {{&S90162[109], 13}, {NULL}};
const Segment SGI215954[] = {{&S90163[34], 13}, {NULL}};
const Segment SGI215955[] = {{&S90163[27], 6}, {NULL}};
const Segment SGI215956[] = {{&S90163[100], 9}, {NULL}};
const Segment SGI215957[] = {{&S90163[73], 6}, {NULL}};
const Segment SGI215958[] = {{&S80294[34], 10}, {NULL}};
const Segment SGI215959[] = {{&S80294[27], 6}, {NULL}};
const Segment SGI215960[] = {{&S80294[97], 9}, {NULL}};
const Segment SGI215961[] = {{&S80294[70], 6}, {NULL}};
const Segment SGI215962[] = {{&S79914[44], 10}, {NULL}};
const Segment SGI215963[] = {{&S79914[44], 22}, {NULL}};
const Segment SGI215964[] = {{&S79914[30], 6}, {NULL}};
const Segment SGI215965[] = {{&S79914[113], 3}, {NULL}};
const Segment SGI215966[] = {{&S79914[80], 5}, {NULL}};
const Segment SGI215967[] = {{&S79918[44], 7}, {NULL}};
const Segment SGI215968[] = {{&S79918[44], 23}, {NULL}};
const Segment SGI215969[] = {{&S79918[30], 6}, {NULL}};
const Segment SGI215970[] = {{&S79918[116], 3}, {NULL}};
const Segment SGI215971[] = {{&S79918[81], 6}, {NULL}};
const Segment SGI215972[] = {{&S79919[44], 7}, {NULL}};
const Segment SGI215973[] = {{&S79919[44], 23}, {NULL}};
const Segment SGI215974[] = {{&S79919[30], 6}, {NULL}};
const Segment SGI215975[] = {{&S79919[116], 3}, {NULL}};
const Segment SGI215976[] = {{&S79919[81], 6}, {NULL}};
const Segment SGI215977[] = {{&S79920[38], 7}, {NULL}};
const Segment SGI215978[] = {{&S79920[31], 6}, {NULL}};
const Segment SGI215979[] = {{&S79921[44], 7}, {NULL}};
const Segment SGI215980[] = {{&S79921[44], 23}, {NULL}};
const Segment SGI215981[] = {{&S79921[30], 6}, {NULL}};
const Segment SGI215982[] = {{&S79921[116], 3}, {NULL}};
const Segment SGI215983[] = {{&S79921[81], 6}, {NULL}};
const Segment SGI215984[] = {{&S71285[44], 12}, {NULL}};
const Segment SGI215985[] = {{&S71285[44], 23}, {NULL}};
const Segment SGI215986[] = {{&S71285[30], 6}, {NULL}};
const Segment SGI215987[] = {{&S71285[134], 3}, {NULL}};
const Segment SGI215988[] = {{&S71285[99], 6}, {NULL}};
const Segment SGI215989[] = {{&S71285[69], 8}, {NULL}};
const Segment SGI215990[] = {{&S71286[44], 12}, {NULL}};
const Segment SGI215991[] = {{&S71286[44], 24}, {NULL}};
const Segment SGI215992[] = {{&S90207[44], 9}, {NULL}};
const Segment SGI215993[] = {{&S71286[30], 6}, {NULL}};
const Segment SGI215994[] = {{&S90207[44], 21}, {NULL}};
const Segment SGI215995[] = {{&S90207[30], 6}, {NULL}};
const Segment SGI215996[] = {{&S71286[135], 3}, {NULL}};
const Segment SGI215997[] = {{&S71286[100], 6}, {NULL}};
const Segment SGI215998[] = {{&S90207[31], 3}, {NULL}};
const Segment SGI215999[] = {{&S71286[70], 8}, {NULL}};
const Segment SGI216000[] = {{&S90207[94], 6}, {NULL}};
const Segment SGI216001[] = {{&S90207[67], 8}, {NULL}};
const Segment SGI216002[] = {{&S90208[44], 9}, {NULL}};
const Segment SGI216003[] = {{&S90208[44], 21}, {NULL}};
const Segment SGI216004[] = {{&S90208[30], 6}, {NULL}};
const Segment SGI216005[] = {{&S90208[31], 3}, {NULL}};
const Segment SGI216006[] = {{&S90208[94], 6}, {NULL}};
const Segment SGI216007[] = {{&S90208[67], 8}, {NULL}};
const Segment SGI216008[] = {{&S71289[34], 10}, {NULL}};
const Segment SGI216009[] = {{&S71289[27], 6}, {NULL}};
const Segment SGI216010[] = {{&S71289[96], 9}, {NULL}};
const Segment SGI216011[] = {{&S71289[69], 6}, {NULL}};
const Segment SGI216012[] = {{&S90209[44], 12}, {NULL}};
const Segment SGI216013[] = {{&S90209[44], 24}, {NULL}};
const Segment SGI216014[] = {{&S90209[30], 6}, {NULL}};
const Segment SGI216015[] = {{&S90209[31], 3}, {NULL}};
const Segment SGI216016[] = {{&S90209[97], 6}, {NULL}};
const Segment SGI216017[] = {{&S90209[70], 8}, {NULL}};
const Segment SGI216018[] = {{&S79933[34], 10}, {NULL}};
const Segment SGI216019[] = {{&S79933[27], 6}, {NULL}};
const Segment SGI216020[] = {{&S71291[34], 10}, {NULL}};
const Segment SGI216021[] = {{&S79933[97], 9}, {NULL}};
const Segment SGI216022[] = {{&S79933[70], 6}, {NULL}};
const Segment SGI216023[] = {{&S71291[27], 6}, {NULL}};
const Segment SGI216024[] = {{&S71291[96], 9}, {NULL}};
const Segment SGI216025[] = {{&S71291[69], 6}, {NULL}};
const Segment SGI216026[] = {{&S79937[34], 28}, {NULL}};
const Segment SGI216027[] = {{&S79937[27], 6}, {NULL}};
const Segment SGI216028[] = {{&S79937[115], 9}, {NULL}};
const Segment SGI216029[] = {{&S79937[88], 6}, {NULL}};
const Segment SGI216030[] = {{&S79938[34], 11}, {NULL}};
const Segment SGI216031[] = {{&S79938[27], 6}, {NULL}};
const Segment SGI216032[] = {{&S79938[98], 9}, {NULL}};
const Segment SGI216033[] = {{&S79938[71], 6}, {NULL}};
const Segment SGI216034[] = {{&S79939[34], 10}, {NULL}};
const Segment SGI216035[] = {{&S79939[27], 6}, {NULL}};
const Segment SGI216036[] = {{&S79939[97], 9}, {NULL}};
const Segment SGI216037[] = {{&S79939[70], 6}, {NULL}};
const Segment SGI216038[] = {{&S90224[44], 10}, {NULL}};
const Segment SGI216039[] = {{&S90224[44], 22}, {NULL}};
const Segment SGI216040[] = {{&S90224[30], 6}, {NULL}};
const Segment SGI216041[] = {{&S90224[31], 3}, {NULL}};
const Segment SGI216042[] = {{&S90224[80], 6}, {NULL}};
const Segment SGI216043[] = {{&S90224[139], 10}, {NULL}};
const Segment SGI216044[] = {{&S90225[44], 10}, {NULL}};
const Segment SGI216045[] = {{&S90225[44], 22}, {NULL}};
const Segment SGI216046[] = {{&S90225[30], 6}, {NULL}};
const Segment SGI216047[] = {{&S90225[31], 3}, {NULL}};
const Segment SGI216048[] = {{&S85385[34], 16}, {NULL}};
const Segment SGI216049[] = {{&S90225[80], 6}, {NULL}};
const Segment SGI216050[] = {{&S85385[27], 6}, {NULL}};
const Segment SGI216051[] = {{&S90225[139], 10}, {NULL}};
const Segment SGI216052[] = {{&S85385[103], 9}, {NULL}};
const Segment SGI216053[] = {{&S85385[76], 6}, {NULL}};
const Segment SGI216054[] = {{&S85388[34], 6}, {NULL}};
const Segment SGI216055[] = {{&S85388[27], 6}, {NULL}};
const Segment SGI216056[] = {{&S85388[93], 9}, {NULL}};
const Segment SGI216057[] = {{&S85388[66], 6}, {NULL}};
const Segment SGI216058[] = {{&S90230[59], 11}, {NULL}};
const Segment SGI216059[] = {{&S90230[59], 23}, {NULL}};
const Segment SGI216060[] = {{&S90230[45], 6}, {NULL}};
const Segment SGI216061[] = {{&S90230[46], 3}, {NULL}};
const Segment SGI216062[] = {{&S90230[96], 6}, {NULL}};
const Segment SGI216063[] = {{&S90230[0], 14}, {NULL}};
const Segment SGI216064[] = {{&S90233[44], 18}, {NULL}};
const Segment SGI216065[] = {{&S90233[44], 30}, {NULL}};
const Segment SGI216066[] = {{&S90233[30], 6}, {NULL}};
const Segment SGI216067[] = {{&S90233[31], 3}, {NULL}};
const Segment SGI216068[] = {{&S90233[103], 6}, {NULL}};
const Segment SGI216069[] = {{&S90233[76], 8}, {NULL}};
const Segment SGI216070[] = {{&S79962[44], 10}, {NULL}};
const Segment SGI216071[] = {{&S79962[44], 22}, {NULL}};
const Segment SGI216072[] = {{&S79962[30], 6}, {NULL}};
const Segment SGI216073[] = {{&S79962[115], 3}, {NULL}};
const Segment SGI216074[] = {{&S79962[80], 6}, {NULL}};
const Segment SGI216075[] = {{&S79962[140], 7}, {NULL}};
const Segment SGI216076[] = {{&S79963[44], 13}, {NULL}};
const Segment SGI216077[] = {{&S79963[44], 25}, {NULL}};
const Segment SGI216078[] = {{&S79963[30], 6}, {NULL}};
const Segment SGI216079[] = {{&S90239[34], 10}, {NULL}};
const Segment SGI216080[] = {{&S79963[118], 3}, {NULL}};
const Segment SGI216081[] = {{&S90239[27], 6}, {NULL}};
const Segment SGI216082[] = {{&S79963[83], 6}, {NULL}};
const Segment SGI216083[] = {{&S79963[143], 7}, {NULL}};
const Segment SGI216084[] = {{&S90239[97], 9}, {NULL}};
const Segment SGI216085[] = {{&S90239[70], 6}, {NULL}};
const Segment SGI216086[] = {{&S71329[34], 10}, {NULL}};
const Segment SGI216087[] = {{&S71329[27], 6}, {NULL}};
const Segment SGI216088[] = {{&S79964[44], 14}, {NULL}};
const Segment SGI216089[] = {{&S71329[96], 9}, {NULL}};
const Segment SGI216090[] = {{&S79964[44], 26}, {NULL}};
const Segment SGI216091[] = {{&S71329[69], 6}, {NULL}};
const Segment SGI216092[] = {{&S79964[30], 6}, {NULL}};
const Segment SGI216093[] = {{&S79964[119], 3}, {NULL}};
const Segment SGI216094[] = {{&S79964[84], 6}, {NULL}};
const Segment SGI216095[] = {{&S79964[144], 7}, {NULL}};
const Segment SGI216096[] = {{&S90243[45], 7}, {NULL}};
const Segment SGI216097[] = {{&S90243[30], 6}, {NULL}};
const Segment SGI216098[] = {{&S79967[44], 15}, {NULL}};
const Segment SGI216099[] = {{&S79967[44], 33}, {NULL}};
const Segment SGI216100[] = {{&S90243[31], 3}, {NULL}};
const Segment SGI216101[] = {{&S79967[30], 6}, {NULL}};
const Segment SGI216102[] = {{&S71333[34], 10}, {NULL}};
const Segment SGI216103[] = {{&S90243[83], 5}, {NULL}};
const Segment SGI216104[] = {{&S71333[27], 6}, {NULL}};
const Segment SGI216105[] = {{&S90243[141], 13}, {NULL}};
const Segment SGI216106[] = {{&S79967[91], 6}, {NULL}};
const Segment SGI216107[] = {{&S71333[96], 9}, {NULL}};
const Segment SGI216108[] = {{&S79967[118], 13}, {NULL}};
const Segment SGI216109[] = {{&S71333[69], 6}, {NULL}};
const Segment SGI216110[] = {{&S71333[137], 8}, {NULL}};
const Segment SGI216111[] = {{&S90244[34], 9}, {NULL}};
const Segment SGI216112[] = {{&S90244[27], 6}, {NULL}};
const Segment SGI216113[] = {{&S90244[96], 9}, {NULL}};
const Segment SGI216114[] = {{&S71334[44], 8}, {NULL}};
const Segment SGI216115[] = {{&S90244[69], 6}, {NULL}};
const Segment SGI216116[] = {{&S71334[44], 19}, {NULL}};
const Segment SGI216117[] = {{&S71334[30], 6}, {NULL}};
const Segment SGI216118[] = {{&S71334[112], 3}, {NULL}};
const Segment SGI216119[] = {{&S71334[77], 6}, {NULL}};
const Segment SGI216120[] = {{&S90248[45], 8}, {NULL}};
const Segment SGI216121[] = {{&S90248[30], 6}, {NULL}};
const Segment SGI216122[] = {{&S90248[31], 3}, {NULL}};
const Segment SGI216123[] = {{&S90248[79], 5}, {NULL}};
const Segment SGI216124[] = {{&S90248[137], 13}, {NULL}};
const Segment SGI216125[] = {{&S90249[59], 7}, {NULL}};
const Segment SGI216126[] = {{&S90249[59], 24}, {NULL}};
const Segment SGI216127[] = {{&S90249[45], 6}, {NULL}};
const Segment SGI216128[] = {{&S90249[46], 3}, {NULL}};
const Segment SGI216129[] = {{&S90249[97], 6}, {NULL}};
const Segment SGI216130[] = {{&S90249[0], 14}, {NULL}};
const Segment SGI216131[] = {{&S79978[34], 11}, {NULL}};
const Segment SGI216132[] = {{&S79978[27], 6}, {NULL}};
const Segment SGI216133[] = {{&S90250[59], 7}, {NULL}};
const Segment SGI216134[] = {{&S90250[59], 24}, {NULL}};
const Segment SGI216135[] = {{&S90250[45], 6}, {NULL}};
const Segment SGI216136[] = {{&S79978[98], 9}, {NULL}};
const Segment SGI216137[] = {{&S79978[71], 6}, {NULL}};
const Segment SGI216138[] = {{&S90250[46], 3}, {NULL}};
const Segment SGI216139[] = {{&S90250[97], 6}, {NULL}};
const Segment SGI216140[] = {{&S90250[0], 14}, {NULL}};
const Segment SGI216141[] = {{&S79979[34], 11}, {NULL}};
const Segment SGI216142[] = {{&S79979[27], 6}, {NULL}};
const Segment SGI216143[] = {{&S90251[34], 11}, {NULL}};
const Segment SGI216144[] = {{&S90251[27], 6}, {NULL}};
const Segment SGI216145[] = {{&S79979[98], 9}, {NULL}};
const Segment SGI216146[] = {{&S79979[71], 6}, {NULL}};
const Segment SGI216147[] = {{&S90251[98], 9}, {NULL}};
const Segment SGI216148[] = {{&S90251[71], 6}, {NULL}};
const Segment SGI216149[] = {{&S90252[44], 9}, {NULL}};
const Segment SGI216150[] = {{&S90252[44], 20}, {NULL}};
const Segment SGI216151[] = {{&S90252[30], 6}, {NULL}};
const Segment SGI216152[] = {{&S90252[31], 3}, {NULL}};
const Segment SGI216153[] = {{&S90252[78], 6}, {NULL}};
const Segment SGI216154[] = {{&S79987[34], 10}, {NULL}};
const Segment SGI216155[] = {{&S79987[27], 6}, {NULL}};
const Segment SGI216156[] = {{&S79987[97], 9}, {NULL}};
const Segment SGI216157[] = {{&S79987[70], 6}, {NULL}};
const Segment SGI216158[] = {{&S79987[131], 8}, {NULL}};
const Segment SGI216159[] = {{&S90266[34], 17}, {NULL}};
const Segment SGI216160[] = {{&S90266[27], 6}, {NULL}};
const Segment SGI216161[] = {{&S90266[111], 9}, {NULL}};
const Segment SGI216162[] = {{&S90266[84], 6}, {NULL}};
const Segment SGI216163[] = {{&S90268[34], 13}, {NULL}};
const Segment SGI216164[] = {{&S90268[27], 6}, {NULL}};
const Segment SGI216165[] = {{&S90268[100], 9}, {NULL}};
const Segment SGI216166[] = {{&S90268[73], 6}, {NULL}};
const Segment SGI216167[] = {{&S71374[34], 10}, {NULL}};
const Segment SGI216168[] = {{&S71374[27], 6}, {NULL}};
const Segment SGI216169[] = {{&S71374[96], 9}, {NULL}};
const Segment SGI216170[] = {{&S71374[69], 6}, {NULL}};
const Segment SGI216171[] = {{&S90272[44], 20}, {NULL}};
const Segment SGI216172[] = {{&S90272[44], 30}, {NULL}};
const Segment SGI216173[] = {{&S90272[30], 6}, {NULL}};
const Segment SGI216174[] = {{&S90272[31], 3}, {NULL}};
const Segment SGI216175[] = {{&S90272[88], 6}, {NULL}};
const Segment SGI216176[] = {{&S90273[44], 19}, {NULL}};
const Segment SGI216177[] = {{&S90273[44], 29}, {NULL}};
const Segment SGI216178[] = {{&S90273[30], 6}, {NULL}};
const Segment SGI216179[] = {{&S90273[31], 3}, {NULL}};
const Segment SGI216180[] = {{&S90273[87], 6}, {NULL}};
const Segment SGI216181[] = {{&S90275[34], 14}, {NULL}};
const Segment SGI216182[] = {{&S90275[27], 6}, {NULL}};
const Segment SGI216183[] = {{&S90275[101], 9}, {NULL}};
const Segment SGI216184[] = {{&S90275[74], 6}, {NULL}};
const Segment SGI216185[] = {{&S80020[34], 15}, {NULL}};
const Segment SGI216186[] = {{&S80020[27], 6}, {NULL}};
const Segment SGI216187[] = {{&S80020[102], 9}, {NULL}};
const Segment SGI216188[] = {{&S80020[75], 6}, {NULL}};
const Segment SGI216189[] = {{&S80020[143], 8}, {NULL}};
const Segment SGI216190[] = {{&S90284[34], 11}, {NULL}};
const Segment SGI216191[] = {{&S90284[27], 6}, {NULL}};
const Segment SGI216192[] = {{&S90284[98], 9}, {NULL}};
const Segment SGI216193[] = {{&S90284[71], 6}, {NULL}};
const Segment SGI216194[] = {{&S80026[34], 10}, {NULL}};
const Segment SGI216195[] = {{&S80026[27], 6}, {NULL}};
const Segment SGI216196[] = {{&S80026[97], 9}, {NULL}};
const Segment SGI216197[] = {{&S80026[70], 6}, {NULL}};
const Segment SGI216198[] = {{&S80026[138], 8}, {NULL}};
const Segment SGI216199[] = {{&S90292[34], 12}, {NULL}};
const Segment SGI216200[] = {{&S90292[27], 6}, {NULL}};
const Segment SGI216201[] = {{&S90292[112], 9}, {NULL}};
const Segment SGI216202[] = {{&S90292[85], 6}, {NULL}};
const Segment SGI216203[] = {{&S90296[44], 23}, {NULL}};
const Segment SGI216204[] = {{&S90296[44], 35}, {NULL}};
const Segment SGI216205[] = {{&S90296[30], 6}, {NULL}};
const Segment SGI216206[] = {{&S90296[31], 3}, {NULL}};
const Segment SGI216207[] = {{&S90296[93], 6}, {NULL}};
const Segment SGI216208[] = {{&S80334[34], 4}, {NULL}};
const Segment SGI216209[] = {{&S80334[27], 6}, {NULL}};
const Segment SGI216210[] = {{&S80334[91], 9}, {NULL}};
const Segment SGI216211[] = {{&S80334[64], 6}, {NULL}};
const Segment SGI216212[] = {{&S90298[34], 12}, {NULL}};
const Segment SGI216213[] = {{&S90298[27], 6}, {NULL}};
const Segment SGI216214[] = {{&S90298[99], 9}, {NULL}};
const Segment SGI216215[] = {{&S90298[72], 6}, {NULL}};
const Segment SGI216216[] = {{&S90298[140], 8}, {NULL}};
const Segment SGI216217[] = {{&S90299[34], 11}, {NULL}};
const Segment SGI216218[] = {{&S90299[27], 6}, {NULL}};
const Segment SGI216219[] = {{&S90299[95], 9}, {NULL}};
const Segment SGI216220[] = {{&S90299[68], 6}, {NULL}};
const Segment SGI216221[] = {{&S90306[34], 11}, {NULL}};
const Segment SGI216222[] = {{&S90306[27], 6}, {NULL}};
const Segment SGI216223[] = {{&S90306[98], 9}, {NULL}};
const Segment SGI216224[] = {{&S90306[71], 6}, {NULL}};
const Segment SGI216225[] = {{&S85503[44], 5}, {NULL}};
const Segment SGI216226[] = {{&S85503[44], 17}, {NULL}};
const Segment SGI216227[] = {{&S85503[30], 6}, {NULL}};
const Segment SGI216228[] = {{&S85503[110], 3}, {NULL}};
const Segment SGI216229[] = {{&S85503[75], 6}, {NULL}};
const Segment SGI216230[] = {{&S85503[134], 10}, {NULL}};
const Segment SGI216231[] = {{&S85504[34], 11}, {NULL}};
const Segment SGI216232[] = {{&S85504[27], 6}, {NULL}};
const Segment SGI216233[] = {{&S85504[98], 9}, {NULL}};
const Segment SGI216234[] = {{&S85504[71], 6}, {NULL}};
const Segment SGI216235[] = {{&S90317[44], 6}, {NULL}};
const Segment SGI216236[] = {{&S90317[44], 18}, {NULL}};
const Segment SGI216237[] = {{&S90317[30], 6}, {NULL}};
const Segment SGI216238[] = {{&S90317[76], 6}, {NULL}};
const Segment SGI216239[] = {{&S90317[103], 13}, {NULL}};
const Segment SGI216240[] = {{&S90325[34], 14}, {NULL}};
const Segment SGI216241[] = {{&S90325[27], 6}, {NULL}};
const Segment SGI216242[] = {{&S90325[106], 9}, {NULL}};
const Segment SGI216243[] = {{&S90325[79], 6}, {NULL}};
const Segment SGI216244[] = {{&S90325[147], 8}, {NULL}};
const Segment SGI216245[] = {{&S90326[34], 11}, {NULL}};
const Segment SGI216246[] = {{&S90326[27], 6}, {NULL}};
const Segment SGI216247[] = {{&S90326[98], 9}, {NULL}};
const Segment SGI216248[] = {{&S90326[71], 6}, {NULL}};
const Segment SGI216249[] = {{&S85529[34], 6}, {NULL}};
const Segment SGI216250[] = {{&S85529[27], 6}, {NULL}};
const Segment SGI216251[] = {{&S85529[93], 9}, {NULL}};
const Segment SGI216252[] = {{&S85529[66], 6}, {NULL}};
const Segment SGI216253[] = {{&S80379[34], 7}, {NULL}};
const Segment SGI216254[] = {{&S80379[27], 6}, {NULL}};
const Segment SGI216255[] = {{&S80379[94], 9}, {NULL}};
const Segment SGI216256[] = {{&S80379[67], 6}, {NULL}};
const Segment SGI216257[] = {{&S71457[34], 16}, {NULL}};
const Segment SGI216258[] = {{&S71457[27], 6}, {NULL}};
const Segment SGI216259[] = {{&S71457[102], 9}, {NULL}};
const Segment SGI216260[] = {{&S71457[75], 6}, {NULL}};
const Segment SGI216261[] = {{&S71458[34], 15}, {NULL}};
const Segment SGI216262[] = {{&S71458[27], 6}, {NULL}};
const Segment SGI216263[] = {{&S71458[101], 9}, {NULL}};
const Segment SGI216264[] = {{&S71458[74], 6}, {NULL}};
const Segment SGI216265[] = {{&S71460[11], 1}, {NULL}};
const Segment SGI216266[] = {{&S71460[27], 6}, {NULL}};
const Segment SGI216267[] = {{&S71460[88], 9}, {NULL}};
const Segment SGI216268[] = {{&S71460[61], 6}, {NULL}};
const Segment SGI216269[] = {{&S80100[34], 11}, {NULL}};
const Segment SGI216270[] = {{&S80100[27], 6}, {NULL}};
const Segment SGI216271[] = {{&S80100[98], 9}, {NULL}};
const Segment SGI216272[] = {{&S80100[71], 6}, {NULL}};
const Segment SGI216273[] = {{&S80102[34], 13}, {NULL}};
const Segment SGI216274[] = {{&S80102[27], 6}, {NULL}};
const Segment SGI216275[] = {{&S80102[100], 9}, {NULL}};
const Segment SGI216276[] = {{&S80102[73], 6}, {NULL}};
const Segment SGI216277[] = {{&S80108[44], 11}, {NULL}};
const Segment SGI216278[] = {{&S80108[44], 23}, {NULL}};
const Segment SGI216279[] = {{&S80108[30], 6}, {NULL}};
const Segment SGI216280[] = {{&S80108[116], 3}, {NULL}};
const Segment SGI216281[] = {{&S80108[81], 6}, {NULL}};
const Segment SGI216282[] = {{&S80109[34], 9}, {NULL}};
const Segment SGI216283[] = {{&S80109[27], 6}, {NULL}};
const Segment SGI216284[] = {{&S80109[96], 9}, {NULL}};
const Segment SGI216285[] = {{&S80109[69], 6}, {NULL}};
const Segment SGI216286[] = {{&S80110[34], 11}, {NULL}};
const Segment SGI216287[] = {{&S80110[27], 6}, {NULL}};
const Segment SGI216288[] = {{&S80110[98], 9}, {NULL}};
const Segment SGI216289[] = {{&S80110[71], 6}, {NULL}};
const Segment SGI216290[] = {{&S80112[34], 10}, {NULL}};
const Segment SGI216291[] = {{&S80112[27], 6}, {NULL}};
const Segment SGI216292[] = {{&S80112[97], 9}, {NULL}};
const Segment SGI216293[] = {{&S80112[70], 6}, {NULL}};
const Segment SGI216294[] = {{&S80112[131], 8}, {NULL}};
const Segment SGI216295[] = {{&S80113[34], 9}, {NULL}};
const Segment SGI216296[] = {{&S80113[27], 6}, {NULL}};
const Segment SGI216297[] = {{&S80113[96], 9}, {NULL}};
const Segment SGI216298[] = {{&S80113[69], 6}, {NULL}};
const Segment SGI216299[] = {{&S85591[34], 6}, {NULL}};
const Segment SGI216300[] = {{&S85591[27], 6}, {NULL}};
const Segment SGI216301[] = {{&S85591[93], 9}, {NULL}};
const Segment SGI216302[] = {{&S85591[66], 6}, {NULL}};
const Segment SGI216303[] = {{&S90395[44], 9}, {NULL}};
const Segment SGI216304[] = {{&S90395[44], 21}, {NULL}};
const Segment SGI216305[] = {{&S90395[30], 6}, {NULL}};
const Segment SGI216306[] = {{&S90395[31], 3}, {NULL}};
const Segment SGI216307[] = {{&S90395[98], 6}, {NULL}};
const Segment SGI216308[] = {{&S90395[67], 8}, {NULL}};
const Segment SGI216309[] = {{&S90396[44], 9}, {NULL}};
const Segment SGI216310[] = {{&S90396[44], 21}, {NULL}};
const Segment SGI216311[] = {{&S90396[30], 6}, {NULL}};
const Segment SGI216312[] = {{&S90396[31], 3}, {NULL}};
const Segment SGI216313[] = {{&S90396[98], 6}, {NULL}};
const Segment SGI216314[] = {{&S90396[67], 8}, {NULL}};
const Segment SGI216315[] = {{&S90397[44], 9}, {NULL}};
const Segment SGI216316[] = {{&S90397[44], 21}, {NULL}};
const Segment SGI216317[] = {{&S90397[30], 6}, {NULL}};
const Segment SGI216318[] = {{&S90397[31], 3}, {NULL}};
const Segment SGI216319[] = {{&S90397[94], 6}, {NULL}};
const Segment SGI216320[] = {{&S90397[67], 8}, {NULL}};
const Segment SGI216321[] = {{&S85598[34], 21}, {NULL}};
const Segment SGI216322[] = {{&S85598[27], 6}, {NULL}};
const Segment SGI216323[] = {{&S85598[108], 9}, {NULL}};
const Segment SGI216324[] = {{&S85598[81], 6}, {NULL}};
const Segment SGI216325[] = {{&S85603[44], 6}, {NULL}};
const Segment SGI216326[] = {{&S85603[44], 18}, {NULL}};
const Segment SGI216327[] = {{&S85603[30], 6}, {NULL}};
const Segment SGI216328[] = {{&S85603[109], 3}, {NULL}};
const Segment SGI216329[] = {{&S85603[76], 5}, {NULL}};
const Segment SGI216330[] = {{&S85603[113], 14}, {NULL}};
const Segment SGI216331[] = {{&S90407[44], 20}, {NULL}};
const Segment SGI216332[] = {{&S90407[44], 32}, {NULL}};
const Segment SGI216333[] = {{&S90407[30], 6}, {NULL}};
const Segment SGI216334[] = {{&S90407[31], 3}, {NULL}};
const Segment SGI216335[] = {{&S85604[44], 7}, {NULL}};
const Segment SGI216336[] = {{&S85604[44], 19}, {NULL}};
const Segment SGI216337[] = {{&S90407[90], 6}, {NULL}};
const Segment SGI216338[] = {{&S85604[30], 6}, {NULL}};
const Segment SGI216339[] = {{&S85604[110], 3}, {NULL}};
const Segment SGI216340[] = {{&S85604[77], 5}, {NULL}};
const Segment SGI216341[] = {{&S80150[44], 7}, {NULL}};
const Segment SGI216342[] = {{&S85604[114], 14}, {NULL}};
const Segment SGI216343[] = {{&S80150[44], 19}, {NULL}};
const Segment SGI216344[] = {{&S80150[30], 6}, {NULL}};
const Segment SGI216345[] = {{&S80150[132], 3}, {NULL}};
const Segment SGI216346[] = {{&S80150[97], 6}, {NULL}};
const Segment SGI216347[] = {{&S80150[65], 8}, {NULL}};
const Segment SGI216348[] = {{&S90410[59], 8}, {NULL}};
const Segment SGI216349[] = {{&S90410[59], 20}, {NULL}};
const Segment SGI216350[] = {{&S90410[45], 6}, {NULL}};
const Segment SGI216351[] = {{&S90410[46], 3}, {NULL}};
const Segment SGI216352[] = {{&S90410[93], 6}, {NULL}};
const Segment SGI216353[] = {{&S80152[34], 7}, {NULL}};
const Segment SGI216354[] = {{&S80152[27], 6}, {NULL}};
const Segment SGI216355[] = {{&S90410[0], 14}, {NULL}};
const Segment SGI216356[] = {{&S80152[94], 9}, {NULL}};
const Segment SGI216357[] = {{&S80152[67], 6}, {NULL}};
const Segment SGI216358[] = {{&S90414[34], 9}, {NULL}};
const Segment SGI216359[] = {{&S90414[27], 6}, {NULL}};
const Segment SGI216360[] = {{&S90414[96], 9}, {NULL}};
const Segment SGI216361[] = {{&S90414[69], 6}, {NULL}};
const Segment SGI216362[] = {{&S90417[34], 8}, {NULL}};
const Segment SGI216363[] = {{&S90417[27], 6}, {NULL}};
const Segment SGI216364[] = {{&S90417[95], 9}, {NULL}};
const Segment SGI216365[] = {{&S90417[68], 6}, {NULL}};
const Segment SGI216366[] = {{&S90421[34], 15}, {NULL}};
const Segment SGI216367[] = {{&S90421[27], 6}, {NULL}};
const Segment SGI216368[] = {{&S90421[102], 9}, {NULL}};
const Segment SGI216369[] = {{&S90421[75], 6}, {NULL}};
const Segment SGI216370[] = {{&S80480[34], 28}, {NULL}};
const Segment SGI216371[] = {{&S80168[34], 28}, {NULL}};
const Segment SGI216372[] = {{&S80480[27], 6}, {NULL}};
const Segment SGI216373[] = {{&S90425[34], 6}, {NULL}};
const Segment SGI216374[] = {{&S80168[27], 6}, {NULL}};
const Segment SGI216375[] = {{&S90425[27], 6}, {NULL}};
const Segment SGI216376[] = {{&S80480[115], 9}, {NULL}};
const Segment SGI216377[] = {{&S80480[88], 6}, {NULL}};
const Segment SGI216378[] = {{&S80168[115], 9}, {NULL}};
const Segment SGI216379[] = {{&S80168[88], 6}, {NULL}};
const Segment SGI216380[] = {{&S90425[93], 9}, {NULL}};
const Segment SGI216381[] = {{&S90425[66], 6}, {NULL}};
const Segment SGI216382[] = {{&S80481[34], 12}, {NULL}};
const Segment SGI216383[] = {{&S80481[27], 6}, {NULL}};
const Segment SGI216384[] = {{&S80481[99], 9}, {NULL}};
const Segment SGI216385[] = {{&S80481[72], 6}, {NULL}};
const Segment SGI216386[] = {{&S90433[59], 13}, {NULL}};
const Segment SGI216387[] = {{&S90433[59], 25}, {NULL}};
const Segment SGI216388[] = {{&S90433[45], 6}, {NULL}};
const Segment SGI216389[] = {{&S90433[46], 3}, {NULL}};
const Segment SGI216390[] = {{&S90433[98], 6}, {NULL}};
const Segment SGI216391[] = {{&S90433[0], 14}, {NULL}};
const Segment SGI216392[] = {{&S83957[32], 10}, {NULL}};
const Segment SGI216393[] = {{&S83957[27], 4}, {NULL}};
const Segment SGI216394[] = {{&S83957[95], 9}, {NULL}};
const Segment SGI216395[] = {{&S83957[68], 6}, {NULL}};
const Segment SGI216396[] = {{&S80180[34], 9}, {NULL}};
const Segment SGI216397[] = {{&S80180[27], 6}, {NULL}};
const Segment SGI216398[] = {{&S80180[96], 9}, {NULL}};
const Segment SGI216399[] = {{&S80180[69], 6}, {NULL}};
const Segment SGI216400[] = {{&S80181[44], 10}, {NULL}};
const Segment SGI216401[] = {{&S80494[44], 16}, {NULL}};
const Segment SGI216402[] = {{&S80181[44], 22}, {NULL}};
const Segment SGI216403[] = {{&S80494[44], 28}, {NULL}};
const Segment SGI216404[] = {{&S80181[30], 6}, {NULL}};
const Segment SGI216405[] = {{&S80494[30], 6}, {NULL}};
const Segment SGI216406[] = {{&S80494[121], 3}, {NULL}};
const Segment SGI216407[] = {{&S80181[80], 6}, {NULL}};
const Segment SGI216408[] = {{&S80494[86], 6}, {NULL}};
const Segment SGI216409[] = {{&S80181[107], 13}, {NULL}};
const Segment SGI216410[] = {{&S90442[34], 4}, {NULL}};
const Segment SGI216411[] = {{&S90442[27], 6}, {NULL}};
const Segment SGI216412[] = {{&S90442[91], 9}, {NULL}};
const Segment SGI216413[] = {{&S90442[64], 6}, {NULL}};
const Segment SGI216414[] = {{&S80184[34], 12}, {NULL}};
const Segment SGI216415[] = {{&S80184[27], 6}, {NULL}};
const Segment SGI216416[] = {{&S80184[99], 9}, {NULL}};
const Segment SGI216417[] = {{&S80184[72], 6}, {NULL}};
const Segment SGI216418[] = {{&S80498[44], 8}, {NULL}};
const Segment SGI216419[] = {{&S80498[44], 20}, {NULL}};
const Segment SGI216420[] = {{&S80498[30], 6}, {NULL}};
const Segment SGI216421[] = {{&S80498[113], 3}, {NULL}};
const Segment SGI216422[] = {{&S90443[44], 10}, {NULL}};
const Segment SGI216423[] = {{&S90443[44], 22}, {NULL}};
const Segment SGI216424[] = {{&S80498[78], 6}, {NULL}};
const Segment SGI216425[] = {{&S90443[30], 6}, {NULL}};
const Segment SGI216426[] = {{&S90443[80], 6}, {NULL}};
const Segment SGI216427[] = {{&S90443[107], 13}, {NULL}};
const Segment SGI216428[] = {{&S80501[34], 7}, {NULL}};
const Segment SGI216429[] = {{&S80501[27], 6}, {NULL}};
const Segment SGI216430[] = {{&S80501[94], 9}, {NULL}};
const Segment SGI216431[] = {{&S80501[67], 6}, {NULL}};
const Segment SGI216432[] = {{&S90451[34], 7}, {NULL}};
const Segment SGI216433[] = {{&S90451[27], 6}, {NULL}};
const Segment SGI216434[] = {{&S90451[94], 9}, {NULL}};
const Segment SGI216435[] = {{&S90451[67], 6}, {NULL}};
const Segment SGI216436[] = {{&S90458[44], 13}, {NULL}};
const Segment SGI216437[] = {{&S90458[44], 25}, {NULL}};
const Segment SGI216438[] = {{&S90458[30], 6}, {NULL}};
const Segment SGI216439[] = {{&S90458[31], 3}, {NULL}};
const Segment SGI216440[] = {{&S90458[83], 5}, {NULL}};
const Segment SGI216441[] = {{&S90458[120], 14}, {NULL}};
const Segment SGI216442[] = {{&S90459[44], 12}, {NULL}};
const Segment SGI216443[] = {{&S90459[44], 24}, {NULL}};
const Segment SGI216444[] = {{&S90459[30], 6}, {NULL}};
const Segment SGI216445[] = {{&S90459[31], 3}, {NULL}};
const Segment SGI216446[] = {{&S90459[82], 6}, {NULL}};
const Segment SGI216447[] = {{&S90461[44], 8}, {NULL}};
const Segment SGI216448[] = {{&S90461[44], 20}, {NULL}};
const Segment SGI216449[] = {{&S90461[30], 6}, {NULL}};
const Segment SGI216450[] = {{&S90461[78], 6}, {NULL}};
const Segment SGI216451[] = {{&S90461[105], 13}, {NULL}};
const Segment SGI216452[] = {{&S90464[44], 7}, {NULL}};
const Segment SGI216453[] = {{&S90464[44], 19}, {NULL}};
const Segment SGI216454[] = {{&S90464[30], 6}, {NULL}};
const Segment SGI216455[] = {{&S90464[31], 3}, {NULL}};
const Segment SGI216456[] = {{&S90464[77], 6}, {NULL}};
const Segment SGI216457[] = {{&S90466[34], 14}, {NULL}};
const Segment SGI216458[] = {{&S90466[27], 6}, {NULL}};
const Segment SGI216459[] = {{&S90466[101], 9}, {NULL}};
const Segment SGI216460[] = {{&S90466[74], 6}, {NULL}};
const Segment SGI216461[] = {{&S90467[34], 8}, {NULL}};
const Segment SGI216462[] = {{&S90467[27], 6}, {NULL}};
const Segment SGI216463[] = {{&S90467[100], 9}, {NULL}};
const Segment SGI216464[] = {{&S90467[73], 6}, {NULL}};
const Segment SGI216465[] = {{&S90468[44], 13}, {NULL}};
const Segment SGI216466[] = {{&S90468[44], 25}, {NULL}};
const Segment SGI216467[] = {{&S90468[30], 6}, {NULL}};
const Segment SGI216468[] = {{&S90468[31], 3}, {NULL}};
const Segment SGI216469[] = {{&S90468[83], 6}, {NULL}};
const Segment SGI216470[] = {{&S90469[34], 15}, {NULL}};
const Segment SGI216471[] = {{&S90469[27], 6}, {NULL}};
const Segment SGI216472[] = {{&S90469[102], 9}, {NULL}};
const Segment SGI216473[] = {{&S90469[75], 6}, {NULL}};
const Segment SGI216474[] = {{&S85691[34], 11}, {NULL}};
const Segment SGI216475[] = {{&S85691[27], 6}, {NULL}};
const Segment SGI216476[] = {{&S85691[98], 9}, {NULL}};
const Segment SGI216477[] = {{&S85691[71], 6}, {NULL}};
const Segment SGI216478[] = {{&S90473[34], 13}, {NULL}};
const Segment SGI216479[] = {{&S90473[27], 6}, {NULL}};
const Segment SGI216480[] = {{&S90473[100], 9}, {NULL}};
const Segment SGI216481[] = {{&S90473[73], 6}, {NULL}};
const Segment SGI216482[] = {{&S90476[34], 4}, {NULL}};
const Segment SGI216483[] = {{&S90476[27], 6}, {NULL}};
const Segment SGI216484[] = {{&S90476[91], 9}, {NULL}};
const Segment SGI216485[] = {{&S90476[64], 6}, {NULL}};
const Segment SGI216486[] = {{&S90478[44], 4}, {NULL}};
const Segment SGI216487[] = {{&S90478[44], 16}, {NULL}};
const Segment SGI216488[] = {{&S90478[30], 6}, {NULL}};
const Segment SGI216489[] = {{&S90478[74], 6}, {NULL}};
const Segment SGI216490[] = {{&S90478[101], 13}, {NULL}};
const Segment SGI216491[] = {{&S80240[44], 10}, {NULL}};
const Segment SGI216492[] = {{&S80240[44], 22}, {NULL}};
const Segment SGI216493[] = {{&S80240[30], 6}, {NULL}};
const Segment SGI216494[] = {{&S80240[135], 3}, {NULL}};
const Segment SGI216495[] = {{&S85702[44], 6}, {NULL}};
const Segment SGI216496[] = {{&S85702[44], 18}, {NULL}};
const Segment SGI216497[] = {{&S80240[100], 6}, {NULL}};
const Segment SGI216498[] = {{&S85702[30], 6}, {NULL}};
const Segment SGI216499[] = {{&S80240[68], 8}, {NULL}};
const Segment SGI216500[] = {{&S85702[109], 3}, {NULL}};
const Segment SGI216501[] = {{&S85702[76], 5}, {NULL}};
const Segment SGI216502[] = {{&S85702[113], 14}, {NULL}};
const Segment SGI216503[] = {{&S85703[45], 11}, {NULL}};
const Segment SGI216504[] = {{&S85703[30], 6}, {NULL}};
const Segment SGI216505[] = {{&S85703[116], 3}, {NULL}};
const Segment SGI216506[] = {{&S85703[82], 5}, {NULL}};
const Segment SGI216507[] = {{&S85703[140], 13}, {NULL}};
const Segment SGI216508[] = {{&S90485[34], 5}, {NULL}};
const Segment SGI216509[] = {{&S90485[27], 6}, {NULL}};
const Segment SGI216510[] = {{&S90485[92], 9}, {NULL}};
const Segment SGI216511[] = {{&S90485[65], 6}, {NULL}};
const Segment SGI216512[] = {{&S80246[44], 11}, {NULL}};
const Segment SGI216513[] = {{&S80246[44], 18}, {NULL}};
const Segment SGI216514[] = {{&S80246[30], 6}, {NULL}};
const Segment SGI216515[] = {{&S80246[111], 3}, {NULL}};
const Segment SGI216516[] = {{&S80246[76], 6}, {NULL}};
const Segment SGI216517[] = {{&S90487[34], 11}, {NULL}};
const Segment SGI216518[] = {{&S90487[27], 6}, {NULL}};
const Segment SGI216519[] = {{&S90487[95], 9}, {NULL}};
const Segment SGI216520[] = {{&S90487[68], 6}, {NULL}};
const Segment SGI216521[] = {{&S80247[44], 11}, {NULL}};
const Segment SGI216522[] = {{&S80247[44], 23}, {NULL}};
const Segment SGI216523[] = {{&S80247[30], 6}, {NULL}};
const Segment SGI216524[] = {{&S80247[116], 3}, {NULL}};
const Segment SGI216525[] = {{&S80247[81], 6}, {NULL}};
const Segment SGI216526[] = {{&S80563[44], 8}, {NULL}};
const Segment SGI216527[] = {{&S80563[44], 20}, {NULL}};
const Segment SGI216528[] = {{&S80563[30], 6}, {NULL}};
const Segment SGI216529[] = {{&S80563[113], 3}, {NULL}};
const Segment SGI216530[] = {{&S80563[78], 6}, {NULL}};
const Segment SGI216531[] = {{&S90491[34], 13}, {NULL}};
const Segment SGI216532[] = {{&S90491[27], 6}, {NULL}};
const Segment SGI216533[] = {{&S90491[100], 9}, {NULL}};
const Segment SGI216534[] = {{&S90491[73], 6}, {NULL}};
const Segment SGI216535[] = {{&S85714[42], 10}, {NULL}};
const Segment SGI216536[] = {{&S85714[30], 6}, {NULL}};
const Segment SGI216537[] = {{&S85714[113], 3}, {NULL}};
const Segment SGI216538[] = {{&S85714[78], 6}, {NULL}};
const Segment SGI216539[] = {{&S85714[130], 10}, {NULL}};
const Segment SGI216540[] = {{&S85715[44], 10}, {NULL}};
const Segment SGI216541[] = {{&S85715[44], 22}, {NULL}};
const Segment SGI216542[] = {{&S85715[30], 6}, {NULL}};
const Segment SGI216543[] = {{&S85715[115], 3}, {NULL}};
const Segment SGI216544[] = {{&S85715[80], 6}, {NULL}};
const Segment SGI216545[] = {{&S85715[132], 10}, {NULL}};
const Segment SGI216546[] = {{&S90497[34], 9}, {NULL}};
const Segment SGI216547[] = {{&S90497[27], 6}, {NULL}};
const Segment SGI216548[] = {{&S90497[98], 9}, {NULL}};
const Segment SGI216549[] = {{&S90497[71], 6}, {NULL}};
const Segment SGI216550[] = {{&S85716[44], 10}, {NULL}};
const Segment SGI216551[] = {{&S85716[44], 22}, {NULL}};
const Segment SGI216552[] = {{&S85716[30], 6}, {NULL}};
const Segment SGI216553[] = {{&S85716[115], 3}, {NULL}};
const Segment SGI216554[] = {{&S85716[80], 6}, {NULL}};
const Segment SGI216555[] = {{&S85716[133], 11}, {NULL}};
const Segment SGI216556[] = {{&S80259[34], 7}, {NULL}};
const Segment SGI216557[] = {{&S80259[27], 6}, {NULL}};
const Segment SGI216558[] = {{&S80259[104], 9}, {NULL}};
const Segment SGI216559[] = {{&S80259[77], 6}, {NULL}};
const Segment SGI216560[] = {{&S85717[42], 10}, {NULL}};
const Segment SGI216561[] = {{&S85717[30], 6}, {NULL}};
const Segment SGI216562[] = {{&S85717[113], 3}, {NULL}};
const Segment SGI216563[] = {{&S85717[78], 6}, {NULL}};
const Segment SGI216564[] = {{&S85717[131], 11}, {NULL}};
const Segment SGI216565[] = {{&S90508[44], 13}, {NULL}};
const Segment SGI216566[] = {{&S90508[44], 25}, {NULL}};
const Segment SGI216567[] = {{&S90508[30], 6}, {NULL}};
const Segment SGI216568[] = {{&S90508[118], 3}, {NULL}};
const Segment SGI216569[] = {{&S90508[83], 6}, {NULL}};
const Segment SGI216570[] = {{&S90508[138], 8}, {NULL}};
const Segment SGI216571[] = {{&S90508[147], 13}, {NULL}};
const Segment SGI216572[] = {{&S90511[44], 9}, {NULL}};
const Segment SGI216573[] = {{&S90511[44], 21}, {NULL}};
const Segment SGI216574[] = {{&S90511[30], 6}, {NULL}};
const Segment SGI216575[] = {{&S90511[79], 6}, {NULL}};
const Segment SGI216576[] = {{&S90511[106], 13}, {NULL}};
const Segment SGI216577[] = {{&S90519[34], 10}, {NULL}};
const Segment SGI216578[] = {{&S90519[27], 6}, {NULL}};
const Segment SGI216579[] = {{&S90519[97], 9}, {NULL}};
const Segment SGI216580[] = {{&S90519[70], 6}, {NULL}};
const Segment SGI216581[] = {{&S85741[44], 8}, {NULL}};
const Segment SGI216582[] = {{&S85741[44], 20}, {NULL}};
const Segment SGI216583[] = {{&S85741[30], 6}, {NULL}};
const Segment SGI216584[] = {{&S85741[113], 3}, {NULL}};
const Segment SGI216585[] = {{&S85741[78], 6}, {NULL}};
const Segment SGI216586[] = {{&S90522[34], 8}, {NULL}};
const Segment SGI216587[] = {{&S90522[27], 6}, {NULL}};
const Segment SGI216588[] = {{&S90522[95], 9}, {NULL}};
const Segment SGI216589[] = {{&S90522[68], 6}, {NULL}};
const Segment SGI216590[] = {{&S90527[44], 10}, {NULL}};
const Segment SGI216591[] = {{&S90527[44], 22}, {NULL}};
const Segment SGI216592[] = {{&S90527[30], 6}, {NULL}};
const Segment SGI216593[] = {{&S90527[80], 6}, {NULL}};
const Segment SGI216594[] = {{&S90527[107], 13}, {NULL}};
const Segment SGI216595[] = {{&S71564[34], 10}, {NULL}};
const Segment SGI216596[] = {{&S71564[27], 6}, {NULL}};
const Segment SGI216597[] = {{&S71564[96], 9}, {NULL}};
const Segment SGI216598[] = {{&S71564[69], 6}, {NULL}};
const Segment SGI216599[] = {{&S71565[34], 10}, {NULL}};
const Segment SGI216600[] = {{&S71565[27], 6}, {NULL}};
const Segment SGI216601[] = {{&S71565[96], 9}, {NULL}};
const Segment SGI216602[] = {{&S71565[69], 6}, {NULL}};
const Segment SGI216603[] = {{&S80642[44], 6}, {NULL}};
const Segment SGI216604[] = {{&S80642[44], 18}, {NULL}};
const Segment SGI216605[] = {{&S80642[30], 6}, {NULL}};
const Segment SGI216606[] = {{&S80642[111], 3}, {NULL}};
const Segment SGI216607[] = {{&S80642[76], 6}, {NULL}};
const Segment SGI216608[] = {{&S90570[44], 12}, {NULL}};
const Segment SGI216609[] = {{&S90570[44], 40}, {NULL}};
const Segment SGI216610[] = {{&S90570[30], 6}, {NULL}};
const Segment SGI216611[] = {{&S90570[31], 3}, {NULL}};
const Segment SGI216612[] = {{&S90570[98], 6}, {NULL}};
const Segment SGI216613[] = {{&S90570[168], 3}, {NULL}};
const Segment SGI216614[] = {{&S90576[34], 9}, {NULL}};
const Segment SGI216615[] = {{&S90576[27], 6}, {NULL}};
const Segment SGI216616[] = {{&S90576[96], 9}, {NULL}};
const Segment SGI216617[] = {{&S90576[69], 6}, {NULL}};
const Segment SGI216618[] = {{&S90581[34], 10}, {NULL}};
const Segment SGI216619[] = {{&S90581[27], 6}, {NULL}};
const Segment SGI216620[] = {{&S90581[102], 9}, {NULL}};
const Segment SGI216621[] = {{&S90581[75], 6}, {NULL}};
const Segment SGI216622[] = {{&S90582[44], 6}, {NULL}};
const Segment SGI216623[] = {{&S90582[44], 18}, {NULL}};
const Segment SGI216624[] = {{&S90582[30], 6}, {NULL}};
const Segment SGI216625[] = {{&S90582[107], 2}, {NULL}};
const Segment SGI216626[] = {{&S90582[76], 5}, {NULL}};
const Segment SGI216627[] = {{&S90583[44], 10}, {NULL}};
const Segment SGI216628[] = {{&S90583[44], 22}, {NULL}};
const Segment SGI216629[] = {{&S90583[30], 6}, {NULL}};
const Segment SGI216630[] = {{&S90583[111], 2}, {NULL}};
const Segment SGI216631[] = {{&S90583[80], 5}, {NULL}};
const Segment SGI216632[] = {{&S90588[34], 7}, {NULL}};
const Segment SGI216633[] = {{&S90588[27], 6}, {NULL}};
const Segment SGI216634[] = {{&S90588[94], 9}, {NULL}};
const Segment SGI216635[] = {{&S90588[67], 6}, {NULL}};
const Segment SGI216636[] = {{&S90589[34], 7}, {NULL}};
const Segment SGI216637[] = {{&S90589[27], 6}, {NULL}};
const Segment SGI216638[] = {{&S90589[94], 9}, {NULL}};
const Segment SGI216639[] = {{&S90589[67], 6}, {NULL}};
const Segment SGI216640[] = {{&S90592[45], 7}, {NULL}};
const Segment SGI216641[] = {{&S90592[30], 6}, {NULL}};
const Segment SGI216642[] = {{&S90592[31], 3}, {NULL}};
const Segment SGI216643[] = {{&S90592[84], 5}, {NULL}};
const Segment SGI216644[] = {{&S90592[142], 13}, {NULL}};
const Segment SGI216645[] = {{&S90594[45], 7}, {NULL}};
const Segment SGI216646[] = {{&S90594[30], 6}, {NULL}};
const Segment SGI216647[] = {{&S71613[34], 11}, {NULL}};
const Segment SGI216648[] = {{&S71613[27], 6}, {NULL}};
const Segment SGI216649[] = {{&S90594[31], 3}, {NULL}};
const Segment SGI216650[] = {{&S90594[83], 5}, {NULL}};
const Segment SGI216651[] = {{&S71613[97], 9}, {NULL}};
const Segment SGI216652[] = {{&S71613[70], 6}, {NULL}};
const Segment SGI216653[] = {{&S90594[141], 13}, {NULL}};
const Segment SGI216654[] = {{&S90595[34], 11}, {NULL}};
const Segment SGI216655[] = {{&S71614[34], 11}, {NULL}};
const Segment SGI216656[] = {{&S90595[27], 6}, {NULL}};
const Segment SGI216657[] = {{&S71614[27], 6}, {NULL}};
const Segment SGI216658[] = {{&S90595[98], 9}, {NULL}};
const Segment SGI216659[] = {{&S90595[71], 6}, {NULL}};
const Segment SGI216660[] = {{&S71614[97], 9}, {NULL}};
const Segment SGI216661[] = {{&S71614[70], 6}, {NULL}};
const Segment SGI216662[] = {{&S85834[44], 9}, {NULL}};
const Segment SGI216663[] = {{&S85834[44], 21}, {NULL}};
const Segment SGI216664[] = {{&S85834[30], 6}, {NULL}};
const Segment SGI216665[] = {{&S85834[114], 3}, {NULL}};
const Segment SGI216666[] = {{&S71615[44], 15}, {NULL}};
const Segment SGI216667[] = {{&S85834[79], 6}, {NULL}};
const Segment SGI216668[] = {{&S71615[44], 26}, {NULL}};
const Segment SGI216669[] = {{&S71615[30], 6}, {NULL}};
const Segment SGI216670[] = {{&S71615[119], 3}, {NULL}};
const Segment SGI216671[] = {{&S71615[84], 6}, {NULL}};
const Segment SGI216672[] = {{&S90598[34], 7}, {NULL}};
const Segment SGI216673[] = {{&S90598[27], 6}, {NULL}};
const Segment SGI216674[] = {{&S90598[94], 9}, {NULL}};
const Segment SGI216675[] = {{&S90598[67], 6}, {NULL}};
const Segment SGI216676[] = {{&S85837[34], 10}, {NULL}};
const Segment SGI216677[] = {{&S85837[27], 6}, {NULL}};
const Segment SGI216678[] = {{&S85837[97], 9}, {NULL}};
const Segment SGI216679[] = {{&S85837[70], 6}, {NULL}};
const Segment SGI216680[] = {{&S90599[34], 8}, {NULL}};
const Segment SGI216681[] = {{&S90599[27], 6}, {NULL}};
const Segment SGI216682[] = {{&S90599[95], 9}, {NULL}};
const Segment SGI216683[] = {{&S90599[68], 6}, {NULL}};
const Segment SGI216684[] = {{&S90600[34], 10}, {NULL}};
const Segment SGI216685[] = {{&S90600[27], 6}, {NULL}};
const Segment SGI216686[] = {{&S90600[97], 9}, {NULL}};
const Segment SGI216687[] = {{&S90600[70], 6}, {NULL}};
const Segment SGI216688[] = {{&S90600[132], 8}, {NULL}};
const Segment SGI216689[] = {{&S90601[34], 10}, {NULL}};
const Segment SGI216690[] = {{&S90601[27], 6}, {NULL}};
const Segment SGI216691[] = {{&S90601[97], 9}, {NULL}};
const Segment SGI216692[] = {{&S90601[70], 6}, {NULL}};
const Segment SGI216693[] = {{&S90601[131], 8}, {NULL}};
const Segment SGI216694[] = {{&S90610[34], 6}, {NULL}};
const Segment SGI216695[] = {{&S90610[27], 6}, {NULL}};
const Segment SGI216696[] = {{&S90610[93], 9}, {NULL}};
const Segment SGI216697[] = {{&S90610[66], 6}, {NULL}};
const Segment SGI216698[] = {{&S85854[44], 18}, {NULL}};
const Segment SGI216699[] = {{&S85854[44], 30}, {NULL}};
const Segment SGI216700[] = {{&S85854[30], 6}, {NULL}};
const Segment SGI216701[] = {{&S85854[31], 3}, {NULL}};
const Segment SGI216702[] = {{&S85854[88], 6}, {NULL}};
const Segment SGI216703[] = {{&S90611[44], 10}, {NULL}};
const Segment SGI216704[] = {{&S90611[44], 22}, {NULL}};
const Segment SGI216705[] = {{&S90611[30], 6}, {NULL}};
const Segment SGI216706[] = {{&S90611[80], 6}, {NULL}};
const Segment SGI216707[] = {{&S90611[107], 13}, {NULL}};
const Segment SGI216708[] = {{&S90613[34], 7}, {NULL}};
const Segment SGI216709[] = {{&S90613[27], 6}, {NULL}};
const Segment SGI216710[] = {{&S90613[94], 9}, {NULL}};
const Segment SGI216711[] = {{&S90613[67], 6}, {NULL}};
const Segment SGI216712[] = {{&S90614[34], 4}, {NULL}};
const Segment SGI216713[] = {{&S90614[27], 6}, {NULL}};
const Segment SGI216714[] = {{&S90614[91], 9}, {NULL}};
const Segment SGI216715[] = {{&S90614[64], 6}, {NULL}};
const Segment SGI216716[] = {{&S85861[34], 10}, {NULL}};
const Segment SGI216717[] = {{&S85861[27], 6}, {NULL}};
const Segment SGI216718[] = {{&S85861[97], 9}, {NULL}};
const Segment SGI216719[] = {{&S85861[70], 6}, {NULL}};
const Segment SGI216720[] = {{&S90620[44], 11}, {NULL}};
const Segment SGI216721[] = {{&S90620[44], 23}, {NULL}};
const Segment SGI216722[] = {{&S90620[30], 6}, {NULL}};
const Segment SGI216723[] = {{&S90620[31], 3}, {NULL}};
const Segment SGI216724[] = {{&S90620[81], 5}, {NULL}};
const Segment SGI216725[] = {{&S85866[44], 14}, {NULL}};
const Segment SGI216726[] = {{&S90620[118], 14}, {NULL}};
const Segment SGI216727[] = {{&S85866[44], 26}, {NULL}};
const Segment SGI216728[] = {{&S85866[30], 6}, {NULL}};
const Segment SGI216729[] = {{&S85866[84], 6}, {NULL}};
const Segment SGI216730[] = {{&S85866[111], 13}, {NULL}};
const Segment SGI216731[] = {{&S85872[44], 16}, {NULL}};
const Segment SGI216732[] = {{&S85872[44], 28}, {NULL}};
const Segment SGI216733[] = {{&S85872[30], 6}, {NULL}};
const Segment SGI216734[] = {{&S85872[121], 3}, {NULL}};
const Segment SGI216735[] = {{&S85872[86], 6}, {NULL}};
const Segment SGI216736[] = {{&S85874[34], 15}, {NULL}};
const Segment SGI216737[] = {{&S85874[27], 6}, {NULL}};
const Segment SGI216738[] = {{&S85874[102], 9}, {NULL}};
const Segment SGI216739[] = {{&S85874[75], 6}, {NULL}};
const Segment SGI216740[] = {{&S90634[44], 11}, {NULL}};
const Segment SGI216741[] = {{&S90634[44], 23}, {NULL}};
const Segment SGI216742[] = {{&S90634[30], 6}, {NULL}};
const Segment SGI216743[] = {{&S90634[31], 3}, {NULL}};
const Segment SGI216744[] = {{&S90634[81], 6}, {NULL}};
const Segment SGI216745[] = {{&S71666[34], 10}, {NULL}};
const Segment SGI216746[] = {{&S71666[27], 6}, {NULL}};
const Segment SGI216747[] = {{&S90635[44], 13}, {NULL}};
const Segment SGI216748[] = {{&S90635[44], 25}, {NULL}};
const Segment SGI216749[] = {{&S90635[30], 6}, {NULL}};
const Segment SGI216750[] = {{&S71666[97], 9}, {NULL}};
const Segment SGI216751[] = {{&S71666[70], 6}, {NULL}};
const Segment SGI216752[] = {{&S90635[31], 3}, {NULL}};
const Segment SGI216753[] = {{&S90635[83], 6}, {NULL}};
const Segment SGI216754[] = {{&S90639[34], 9}, {NULL}};
const Segment SGI216755[] = {{&S90639[27], 6}, {NULL}};
const Segment SGI216756[] = {{&S90639[96], 9}, {NULL}};
const Segment SGI216757[] = {{&S90639[69], 6}, {NULL}};
const Segment SGI216758[] = {{&S90641[34], 8}, {NULL}};
const Segment SGI216759[] = {{&S90641[27], 6}, {NULL}};
const Segment SGI216760[] = {{&S90641[95], 9}, {NULL}};
const Segment SGI216761[] = {{&S90641[68], 6}, {NULL}};
const Segment SGI216762[] = {{&S90651[44], 10}, {NULL}};
const Segment SGI216763[] = {{&S90651[44], 22}, {NULL}};
const Segment SGI216764[] = {{&S90651[30], 6}, {NULL}};
const Segment SGI216765[] = {{&S90651[80], 6}, {NULL}};
const Segment SGI216766[] = {{&S90651[107], 13}, {NULL}};
const Segment SGI216767[] = {{&S80306[44], 10}, {NULL}};
const Segment SGI216768[] = {{&S80306[44], 22}, {NULL}};
const Segment SGI216769[] = {{&S80306[30], 6}, {NULL}};
const Segment SGI216770[] = {{&S80306[131], 3}, {NULL}};
const Segment SGI216771[] = {{&S80306[96], 6}, {NULL}};
const Segment SGI216772[] = {{&S80306[68], 8}, {NULL}};
const Segment SGI216773[] = {{&S90657[44], 8}, {NULL}};
const Segment SGI216774[] = {{&S90657[44], 26}, {NULL}};
const Segment SGI216775[] = {{&S90657[30], 6}, {NULL}};
const Segment SGI216776[] = {{&S90657[31], 3}, {NULL}};
const Segment SGI216777[] = {{&S90657[84], 6}, {NULL}};
const Segment SGI216778[] = {{&S90661[44], 5}, {NULL}};
const Segment SGI216779[] = {{&S90661[44], 17}, {NULL}};
const Segment SGI216780[] = {{&S90661[30], 6}, {NULL}};
const Segment SGI216781[] = {{&S90661[75], 6}, {NULL}};
const Segment SGI216782[] = {{&S71696[34], 10}, {NULL}};
const Segment SGI216783[] = {{&S90661[102], 13}, {NULL}};
const Segment SGI216784[] = {{&S71696[27], 6}, {NULL}};
const Segment SGI216785[] = {{&S71696[96], 9}, {NULL}};
const Segment SGI216786[] = {{&S71696[69], 6}, {NULL}};
const Segment SGI216787[] = {{&S71697[34], 10}, {NULL}};
const Segment SGI216788[] = {{&S71697[27], 6}, {NULL}};
const Segment SGI216789[] = {{&S71697[96], 9}, {NULL}};
const Segment SGI216790[] = {{&S71697[69], 6}, {NULL}};
const Segment SGI216791[] = {{&S80321[52], 10}, {NULL}};
const Segment SGI216792[] = {{&S80321[52], 22}, {NULL}};
const Segment SGI216793[] = {{&S80321[38], 6}, {NULL}};
const Segment SGI216794[] = {{&S80321[88], 5}, {NULL}};
const Segment SGI216795[] = {{&S80321[11], 3}, {NULL}};
const Segment SGI216796[] = {{&S80324[44], 10}, {NULL}};
const Segment SGI216797[] = {{&S80324[44], 22}, {NULL}};
const Segment SGI216798[] = {{&S80324[30], 6}, {NULL}};
const Segment SGI216799[] = {{&S80324[115], 3}, {NULL}};
const Segment SGI216800[] = {{&S80324[80], 6}, {NULL}};
const Segment SGI216801[] = {{&S80325[44], 10}, {NULL}};
const Segment SGI216802[] = {{&S80325[44], 22}, {NULL}};
const Segment SGI216803[] = {{&S80325[30], 6}, {NULL}};
const Segment SGI216804[] = {{&S80325[83], 6}, {NULL}};
const Segment SGI216805[] = {{&S80326[38], 10}, {NULL}};
const Segment SGI216806[] = {{&S80326[31], 6}, {NULL}};
const Segment SGI216807[] = {{&S80327[34], 11}, {NULL}};
const Segment SGI216808[] = {{&S80327[27], 6}, {NULL}};
const Segment SGI216809[] = {{&S80327[98], 9}, {NULL}};
const Segment SGI216810[] = {{&S80327[71], 6}, {NULL}};
const Segment SGI216811[] = {{&S80328[34], 7}, {NULL}};
const Segment SGI216812[] = {{&S80328[27], 6}, {NULL}};
const Segment SGI216813[] = {{&S80328[104], 9}, {NULL}};
const Segment SGI216814[] = {{&S80328[77], 6}, {NULL}};
const Segment SGI216815[] = {{&S71719[34], 10}, {NULL}};
const Segment SGI216816[] = {{&S85941[34], 13}, {NULL}};
const Segment SGI216817[] = {{&S71719[27], 6}, {NULL}};
const Segment SGI216818[] = {{&S85941[27], 6}, {NULL}};
const Segment SGI216819[] = {{&S85941[95], 9}, {NULL}};
const Segment SGI216820[] = {{&S71719[96], 9}, {NULL}};
const Segment SGI216821[] = {{&S85941[68], 6}, {NULL}};
const Segment SGI216822[] = {{&S71719[69], 6}, {NULL}};
const Segment SGI216823[] = {{&S71719[138], 8}, {NULL}};
const Segment SGI216824[] = {{&S85942[34], 23}, {NULL}};
const Segment SGI216825[] = {{&S85942[27], 6}, {NULL}};
const Segment SGI216826[] = {{&S71720[34], 10}, {NULL}};
const Segment SGI216827[] = {{&S80335[34], 11}, {NULL}};
const Segment SGI216828[] = {{&S71720[27], 6}, {NULL}};
const Segment SGI216829[] = {{&S80335[27], 6}, {NULL}};
const Segment SGI216830[] = {{&S85942[110], 9}, {NULL}};
const Segment SGI216831[] = {{&S85942[83], 6}, {NULL}};
const Segment SGI216832[] = {{&S80335[98], 9}, {NULL}};
const Segment SGI216833[] = {{&S71720[97], 9}, {NULL}};
const Segment SGI216834[] = {{&S80335[71], 6}, {NULL}};
const Segment SGI216835[] = {{&S71720[70], 6}, {NULL}};
const Segment SGI216836[] = {{&S71720[138], 8}, {NULL}};
const Segment SGI216837[] = {{&S85965[44], 8}, {NULL}};
const Segment SGI216838[] = {{&S85965[44], 20}, {NULL}};
const Segment SGI216839[] = {{&S85965[30], 6}, {NULL}};
const Segment SGI216840[] = {{&S85965[113], 3}, {NULL}};
const Segment SGI216841[] = {{&S85965[78], 6}, {NULL}};
const Segment SGI216842[] = {{&S85966[44], 8}, {NULL}};
const Segment SGI216843[] = {{&S85966[44], 20}, {NULL}};
const Segment SGI216844[] = {{&S85966[30], 6}, {NULL}};
const Segment SGI216845[] = {{&S85966[113], 3}, {NULL}};
const Segment SGI216846[] = {{&S85966[78], 6}, {NULL}};
const Segment SGI216847[] = {{&S90710[34], 10}, {NULL}};
const Segment SGI216848[] = {{&S90710[27], 6}, {NULL}};
const Segment SGI216849[] = {{&S90710[97], 9}, {NULL}};
const Segment SGI216850[] = {{&S90710[70], 6}, {NULL}};
const Segment SGI216851[] = {{&S90710[138], 8}, {NULL}};
const Segment SGI216852[] = {{&S90714[44], 11}, {NULL}};
const Segment SGI216853[] = {{&S90714[44], 23}, {NULL}};
const Segment SGI216854[] = {{&S90714[30], 6}, {NULL}};
const Segment SGI216855[] = {{&S90714[31], 3}, {NULL}};
const Segment SGI216856[] = {{&S90714[81], 5}, {NULL}};
const Segment SGI216857[] = {{&S90714[118], 14}, {NULL}};
const Segment SGI216858[] = {{&S80374[44], 18}, {NULL}};
const Segment SGI216859[] = {{&S80374[44], 30}, {NULL}};
const Segment SGI216860[] = {{&S80374[30], 6}, {NULL}};
const Segment SGI216861[] = {{&S90717[34], 7}, {NULL}};
const Segment SGI216862[] = {{&S90717[27], 6}, {NULL}};
const Segment SGI216863[] = {{&S80374[123], 3}, {NULL}};
const Segment SGI216864[] = {{&S80374[88], 6}, {NULL}};
const Segment SGI216865[] = {{&S90717[94], 9}, {NULL}};
const Segment SGI216866[] = {{&S90717[67], 6}, {NULL}};
const Segment SGI216867[] = {{&S90720[44], 14}, {NULL}};
const Segment SGI216868[] = {{&S90720[44], 26}, {NULL}};
const Segment SGI216869[] = {{&S90720[30], 6}, {NULL}};
const Segment SGI216870[] = {{&S90720[84], 6}, {NULL}};
const Segment SGI216871[] = {{&S90720[111], 13}, {NULL}};
const Segment SGI216872[] = {{&S71767[44], 10}, {NULL}};
const Segment SGI216873[] = {{&S71767[44], 21}, {NULL}};
const Segment SGI216874[] = {{&S71767[30], 6}, {NULL}};
const Segment SGI216875[] = {{&S71767[132], 3}, {NULL}};
const Segment SGI216876[] = {{&S71767[97], 6}, {NULL}};
const Segment SGI216877[] = {{&S71767[67], 8}, {NULL}};
const Segment SGI216878[] = {{&S71768[44], 10}, {NULL}};
const Segment SGI216879[] = {{&S71768[44], 22}, {NULL}};
const Segment SGI216880[] = {{&S71768[30], 6}, {NULL}};
const Segment SGI216881[] = {{&S71768[135], 3}, {NULL}};
const Segment SGI216882[] = {{&S71768[100], 6}, {NULL}};
const Segment SGI216883[] = {{&S71768[68], 8}, {NULL}};
const Segment SGI216884[] = {{&S85991[44], 14}, {NULL}};
const Segment SGI216885[] = {{&S85991[44], 31}, {NULL}};
const Segment SGI216886[] = {{&S85991[30], 6}, {NULL}};
const Segment SGI216887[] = {{&S80385[34], 6}, {NULL}};
const Segment SGI216888[] = {{&S85991[122], 3}, {NULL}};
const Segment SGI216889[] = {{&S80385[27], 6}, {NULL}};
const Segment SGI216890[] = {{&S85991[89], 5}, {NULL}};
const Segment SGI216891[] = {{&S85991[126], 14}, {NULL}};
const Segment SGI216892[] = {{&S80385[93], 9}, {NULL}};
const Segment SGI216893[] = {{&S80385[66], 6}, {NULL}};
const Segment SGI216894[] = {{&S80863[34], 6}, {NULL}};
const Segment SGI216895[] = {{&S80863[27], 6}, {NULL}};
const Segment SGI216896[] = {{&S80863[93], 9}, {NULL}};
const Segment SGI216897[] = {{&S90730[34], 15}, {NULL}};
const Segment SGI216898[] = {{&S80863[66], 6}, {NULL}};
const Segment SGI216899[] = {{&S90730[27], 6}, {NULL}};
const Segment SGI216900[] = {{&S90730[117], 9}, {NULL}};
const Segment SGI216901[] = {{&S90730[90], 6}, {NULL}};
const Segment SGI216902[] = {{&S80416[44], 11}, {NULL}};
const Segment SGI216903[] = {{&S80416[44], 23}, {NULL}};
const Segment SGI216904[] = {{&S80416[30], 6}, {NULL}};
const Segment SGI216905[] = {{&S80416[132], 3}, {NULL}};
const Segment SGI216906[] = {{&S80416[97], 6}, {NULL}};
const Segment SGI216907[] = {{&S80416[69], 8}, {NULL}};
const Segment SGI216908[] = {{&S71818[44], 10}, {NULL}};
const Segment SGI216909[] = {{&S71818[44], 21}, {NULL}};
const Segment SGI216910[] = {{&S71818[30], 6}, {NULL}};
const Segment SGI216911[] = {{&S71818[114], 3}, {NULL}};
const Segment SGI216912[] = {{&S71818[79], 6}, {NULL}};
const Segment SGI216913[] = {{&S71819[38], 10}, {NULL}};
const Segment SGI216914[] = {{&S71819[31], 6}, {NULL}};
const Segment SGI216915[] = {{&S71820[34], 10}, {NULL}};
const Segment SGI216916[] = {{&S71820[27], 6}, {NULL}};
const Segment SGI216917[] = {{&S71820[96], 9}, {NULL}};
const Segment SGI216918[] = {{&S71820[69], 6}, {NULL}};
const Segment SGI216919[] = {{&S71821[34], 11}, {NULL}};
const Segment SGI216920[] = {{&S71821[27], 6}, {NULL}};
const Segment SGI216921[] = {{&S71821[97], 9}, {NULL}};
const Segment SGI216922[] = {{&S71821[70], 6}, {NULL}};
const Segment SGI216923[] = {{&S80929[34], 4}, {NULL}};
const Segment SGI216924[] = {{&S80929[27], 6}, {NULL}};
const Segment SGI216925[] = {{&S80929[91], 9}, {NULL}};
const Segment SGI216926[] = {{&S80929[64], 6}, {NULL}};
const Segment SGI216927[] = {{&S86070[44], 10}, {NULL}};
const Segment SGI216928[] = {{&S86070[44], 22}, {NULL}};
const Segment SGI216929[] = {{&S86070[30], 6}, {NULL}};
const Segment SGI216930[] = {{&S86070[136], 3}, {NULL}};
const Segment SGI216931[] = {{&S86070[101], 6}, {NULL}};
const Segment SGI216932[] = {{&S90808[44], 7}, {NULL}};
const Segment SGI216933[] = {{&S90808[44], 19}, {NULL}};
const Segment SGI216934[] = {{&S90808[30], 6}, {NULL}};
const Segment SGI216935[] = {{&S86072[34], 13}, {NULL}};
const Segment SGI216936[] = {{&S90808[31], 3}, {NULL}};
const Segment SGI216937[] = {{&S86072[27], 6}, {NULL}};
const Segment SGI216938[] = {{&S90808[92], 6}, {NULL}};
const Segment SGI216939[] = {{&S90808[65], 8}, {NULL}};
const Segment SGI216940[] = {{&S86072[100], 9}, {NULL}};
const Segment SGI216941[] = {{&S86072[73], 6}, {NULL}};
const Segment SGI216942[] = {{&S90811[34], 7}, {NULL}};
const Segment SGI216943[] = {{&S90811[27], 6}, {NULL}};
const Segment SGI216944[] = {{&S90811[90], 9}, {NULL}};
const Segment SGI216945[] = {{&S90811[63], 6}, {NULL}};
const Segment SGI216946[] = {{&S90812[44], 6}, {NULL}};
const Segment SGI216947[] = {{&S90812[44], 13}, {NULL}};
const Segment SGI216948[] = {{&S90812[30], 6}, {NULL}};
const Segment SGI216949[] = {{&S90812[31], 3}, {NULL}};
const Segment SGI216950[] = {{&S90812[71], 6}, {NULL}};
const Segment SGI216951[] = {{&S86078[34], 25}, {NULL}};
const Segment SGI216952[] = {{&S86078[27], 6}, {NULL}};
const Segment SGI216953[] = {{&S86078[112], 9}, {NULL}};
const Segment SGI216954[] = {{&S86078[85], 6}, {NULL}};
const Segment SGI216955[] = {{&S90813[34], 10}, {NULL}};
const Segment SGI216956[] = {{&S90813[27], 6}, {NULL}};
const Segment SGI216957[] = {{&S90813[97], 9}, {NULL}};
const Segment SGI216958[] = {{&S90813[70], 6}, {NULL}};
const Segment SGI216959[] = {{&S90815[34], 10}, {NULL}};
const Segment SGI216960[] = {{&S90815[27], 6}, {NULL}};
const Segment SGI216961[] = {{&S90815[97], 9}, {NULL}};
const Segment SGI216962[] = {{&S90815[70], 6}, {NULL}};
const Segment SGI216963[] = {{&S90817[48], 7}, {NULL}};
const Segment SGI216964[] = {{&S90817[48], 19}, {NULL}};
const Segment SGI216965[] = {{&S90817[30], 10}, {NULL}};
const Segment SGI216966[] = {{&S90817[31], 3}, {NULL}};
const Segment SGI216967[] = {{&S90817[95], 5}, {NULL}};
const Segment SGI216968[] = {{&S86085[44], 25}, {NULL}};
const Segment SGI216969[] = {{&S86085[44], 37}, {NULL}};
const Segment SGI216970[] = {{&S86085[30], 6}, {NULL}};
const Segment SGI216971[] = {{&S86085[130], 3}, {NULL}};
const Segment SGI216972[] = {{&S86085[95], 6}, {NULL}};
const Segment SGI216973[] = {{&S90820[34], 15}, {NULL}};
const Segment SGI216974[] = {{&S90820[27], 6}, {NULL}};
const Segment SGI216975[] = {{&S90820[102], 9}, {NULL}};
const Segment SGI216976[] = {{&S90820[75], 6}, {NULL}};
const Segment SGI216977[] = {{&S86090[34], 25}, {NULL}};
const Segment SGI216978[] = {{&S86090[27], 6}, {NULL}};
const Segment SGI216979[] = {{&S86090[112], 9}, {NULL}};
const Segment SGI216980[] = {{&S86090[85], 6}, {NULL}};
const Segment SGI216981[] = {{&S86091[34], 32}, {NULL}};
const Segment SGI216982[] = {{&S86091[27], 6}, {NULL}};
const Segment SGI216983[] = {{&S86091[119], 9}, {NULL}};
const Segment SGI216984[] = {{&S86091[92], 6}, {NULL}};
const Segment SGI216985[] = {{&S86094[34], 12}, {NULL}};
const Segment SGI216986[] = {{&S86094[27], 6}, {NULL}};
const Segment SGI216987[] = {{&S86094[99], 9}, {NULL}};
const Segment SGI216988[] = {{&S86094[72], 6}, {NULL}};
const Segment SGI216989[] = {{&S86094[140], 8}, {NULL}};
const Segment SGI216990[] = {{&S90832[44], 7}, {NULL}};
const Segment SGI216991[] = {{&S90832[44], 23}, {NULL}};
const Segment SGI216992[] = {{&S90832[30], 6}, {NULL}};
const Segment SGI216993[] = {{&S90832[81], 6}, {NULL}};
const Segment SGI216994[] = {{&S90832[108], 13}, {NULL}};
const Segment SGI216995[] = {{&S90833[34], 10}, {NULL}};
const Segment SGI216996[] = {{&S90833[27], 6}, {NULL}};
const Segment SGI216997[] = {{&S90833[108], 9}, {NULL}};
const Segment SGI216998[] = {{&S90833[81], 6}, {NULL}};
const Segment SGI216999[] = {{&S90840[45], 7}, {NULL}};
const Segment SGI217000[] = {{&S90840[30], 6}, {NULL}};
const Segment SGI217001[] = {{&S90840[31], 3}, {NULL}};
const Segment SGI217002[] = {{&S90840[77], 5}, {NULL}};
const Segment SGI217003[] = {{&S90840[135], 13}, {NULL}};
const Segment SGI217004[] = {{&S90842[34], 5}, {NULL}};
const Segment SGI217005[] = {{&S90842[27], 6}, {NULL}};
const Segment SGI217006[] = {{&S90842[92], 9}, {NULL}};
const Segment SGI217007[] = {{&S90842[65], 6}, {NULL}};
const Segment SGI217008[] = {{&S90843[34], 13}, {NULL}};
const Segment SGI217009[] = {{&S90843[27], 6}, {NULL}};
const Segment SGI217010[] = {{&S90843[100], 9}, {NULL}};
const Segment SGI217011[] = {{&S90843[73], 6}, {NULL}};
const Segment SGI217012[] = {{&S90845[44], 14}, {NULL}};
const Segment SGI217013[] = {{&S90845[44], 31}, {NULL}};
const Segment SGI217014[] = {{&S90845[30], 6}, {NULL}};
const Segment SGI217015[] = {{&S90845[89], 6}, {NULL}};
const Segment SGI217016[] = {{&S90845[116], 13}, {NULL}};
const Segment SGI217017[] = {{&S90846[34], 10}, {NULL}};
const Segment SGI217018[] = {{&S90846[27], 6}, {NULL}};
const Segment SGI217019[] = {{&S90846[97], 9}, {NULL}};
const Segment SGI217020[] = {{&S90846[70], 6}, {NULL}};
const Segment SGI217021[] = {{&S90852[34], 15}, {NULL}};
const Segment SGI217022[] = {{&S90852[27], 6}, {NULL}};
const Segment SGI217023[] = {{&S90852[99], 9}, {NULL}};
const Segment SGI217024[] = {{&S90852[72], 6}, {NULL}};
const Segment SGI217025[] = {{&S90854[34], 11}, {NULL}};
const Segment SGI217026[] = {{&S90854[27], 6}, {NULL}};
const Segment SGI217027[] = {{&S90854[98], 9}, {NULL}};
const Segment SGI217028[] = {{&S90854[71], 6}, {NULL}};
const Segment SGI217029[] = {{&S90858[44], 15}, {NULL}};
const Segment SGI217030[] = {{&S90858[44], 27}, {NULL}};
const Segment SGI217031[] = {{&S90858[30], 6}, {NULL}};
const Segment SGI217032[] = {{&S90858[31], 3}, {NULL}};
const Segment SGI217033[] = {{&S90858[85], 6}, {NULL}};
const Segment SGI217034[] = {{&S90858[145], 11}, {NULL}};
const Segment SGI217035[] = {{&S90859[44], 11}, {NULL}};
const Segment SGI217036[] = {{&S90859[44], 23}, {NULL}};
const Segment SGI217037[] = {{&S90859[30], 6}, {NULL}};
const Segment SGI217038[] = {{&S90859[31], 3}, {NULL}};
const Segment SGI217039[] = {{&S90859[96], 6}, {NULL}};
const Segment SGI217040[] = {{&S90859[69], 8}, {NULL}};
const Segment SGI217041[] = {{&S90860[44], 11}, {NULL}};
const Segment SGI217042[] = {{&S90860[44], 23}, {NULL}};
const Segment SGI217043[] = {{&S90860[30], 6}, {NULL}};
const Segment SGI217044[] = {{&S90860[31], 3}, {NULL}};
const Segment SGI217045[] = {{&S90860[96], 6}, {NULL}};
const Segment SGI217046[] = {{&S90860[69], 8}, {NULL}};
const Segment SGI217047[] = {{&S71928[44], 10}, {NULL}};
const Segment SGI217048[] = {{&S71928[44], 21}, {NULL}};
const Segment SGI217049[] = {{&S71928[30], 6}, {NULL}};
const Segment SGI217050[] = {{&S71928[114], 3}, {NULL}};
const Segment SGI217051[] = {{&S71928[79], 6}, {NULL}};
const Segment SGI217052[] = {{&S90862[44], 5}, {NULL}};
const Segment SGI217053[] = {{&S90862[44], 21}, {NULL}};
const Segment SGI217054[] = {{&S90862[30], 6}, {NULL}};
const Segment SGI217055[] = {{&S90862[79], 6}, {NULL}};
const Segment SGI217056[] = {{&S90862[106], 13}, {NULL}};
const Segment SGI217057[] = {{&S71929[44], 10}, {NULL}};
const Segment SGI217058[] = {{&S71929[44], 21}, {NULL}};
const Segment SGI217059[] = {{&S71929[30], 6}, {NULL}};
const Segment SGI217060[] = {{&S71929[114], 3}, {NULL}};
const Segment SGI217061[] = {{&S71929[79], 6}, {NULL}};
const Segment SGI217062[] = {{&S90863[34], 5}, {NULL}};
const Segment SGI217063[] = {{&S90863[27], 6}, {NULL}};
const Segment SGI217064[] = {{&S90863[92], 9}, {NULL}};
const Segment SGI217065[] = {{&S90863[65], 6}, {NULL}};
const Segment SGI217066[] = {{&S71930[38], 10}, {NULL}};
const Segment SGI217067[] = {{&S71930[31], 6}, {NULL}};
const Segment SGI217068[] = {{&S90864[34], 8}, {NULL}};
const Segment SGI217069[] = {{&S90864[27], 6}, {NULL}};
const Segment SGI217070[] = {{&S90864[95], 9}, {NULL}};
const Segment SGI217071[] = {{&S90864[68], 6}, {NULL}};
const Segment SGI217072[] = {{&S86149[44], 15}, {NULL}};
const Segment SGI217073[] = {{&S86149[44], 27}, {NULL}};
const Segment SGI217074[] = {{&S86149[30], 6}, {NULL}};
const Segment SGI217075[] = {{&S86149[120], 3}, {NULL}};
const Segment SGI217076[] = {{&S86149[85], 6}, {NULL}};
const Segment SGI217077[] = {{&S90868[34], 14}, {NULL}};
const Segment SGI217078[] = {{&S90868[27], 6}, {NULL}};
const Segment SGI217079[] = {{&S90868[101], 9}, {NULL}};
const Segment SGI217080[] = {{&S90868[74], 6}, {NULL}};
const Segment SGI217081[] = {{&S81036[44], 25}, {NULL}};
const Segment SGI217082[] = {{&S81036[44], 37}, {NULL}};
const Segment SGI217083[] = {{&S81036[30], 6}, {NULL}};
const Segment SGI217084[] = {{&S81036[31], 3}, {NULL}};
const Segment SGI217085[] = {{&S81036[95], 6}, {NULL}};
const Segment SGI217086[] = {{&S81037[44], 23}, {NULL}};
const Segment SGI217087[] = {{&S81037[44], 35}, {NULL}};
const Segment SGI217088[] = {{&S81037[30], 6}, {NULL}};
const Segment SGI217089[] = {{&S81037[31], 3}, {NULL}};
const Segment SGI217090[] = {{&S81037[93], 6}, {NULL}};
const Segment SGI217091[] = {{&S86157[44], 11}, {NULL}};
const Segment SGI217092[] = {{&S86157[44], 23}, {NULL}};
const Segment SGI217093[] = {{&S86157[30], 6}, {NULL}};
const Segment SGI217094[] = {{&S86157[116], 3}, {NULL}};
const Segment SGI217095[] = {{&S86157[81], 6}, {NULL}};
const Segment SGI217096[] = {{&S90879[34], 10}, {NULL}};
const Segment SGI217097[] = {{&S90879[27], 6}, {NULL}};
const Segment SGI217098[] = {{&S90879[97], 9}, {NULL}};
const Segment SGI217099[] = {{&S90879[70], 6}, {NULL}};
const Segment SGI217100[] = {{&S86169[44], 18}, {NULL}};
const Segment SGI217101[] = {{&S86169[44], 30}, {NULL}};
const Segment SGI217102[] = {{&S86169[30], 6}, {NULL}};
const Segment SGI217103[] = {{&S86169[123], 3}, {NULL}};
const Segment SGI217104[] = {{&S86169[88], 6}, {NULL}};
const Segment SGI217105[] = {{&S71958[34], 18}, {NULL}};
const Segment SGI217106[] = {{&S71958[27], 6}, {NULL}};
const Segment SGI217107[] = {{&S71958[104], 9}, {NULL}};
const Segment SGI217108[] = {{&S71958[77], 6}, {NULL}};
const Segment SGI217109[] = {{&S71959[34], 10}, {NULL}};
const Segment SGI217110[] = {{&S71959[27], 6}, {NULL}};
const Segment SGI217111[] = {{&S71959[96], 9}, {NULL}};
const Segment SGI217112[] = {{&S71959[69], 6}, {NULL}};
const Segment SGI217113[] = {{&S86176[34], 7}, {NULL}};
const Segment SGI217114[] = {{&S86176[27], 6}, {NULL}};
const Segment SGI217115[] = {{&S86176[98], 9}, {NULL}};
const Segment SGI217116[] = {{&S86176[71], 6}, {NULL}};
const Segment SGI217117[] = {{&S90895[44], 14}, {NULL}};
const Segment SGI217118[] = {{&S90895[44], 31}, {NULL}};
const Segment SGI217119[] = {{&S90895[30], 6}, {NULL}};
const Segment SGI217120[] = {{&S90895[89], 6}, {NULL}};
const Segment SGI217121[] = {{&S90895[116], 13}, {NULL}};
const Segment SGI217122[] = {{&S86182[44], 9}, {NULL}};
const Segment SGI217123[] = {{&S86182[44], 21}, {NULL}};
const Segment SGI217124[] = {{&S86182[30], 6}, {NULL}};
const Segment SGI217125[] = {{&S86182[114], 3}, {NULL}};
const Segment SGI217126[] = {{&S86182[79], 6}, {NULL}};
const Segment SGI217127[] = {{&S90900[44], 15}, {NULL}};
const Segment SGI217128[] = {{&S90900[44], 33}, {NULL}};
const Segment SGI217129[] = {{&S90900[30], 6}, {NULL}};
const Segment SGI217130[] = {{&S90900[91], 6}, {NULL}};
const Segment SGI217131[] = {{&S90900[118], 13}, {NULL}};
const Segment SGI217132[] = {{&S90901[44], 14}, {NULL}};
const Segment SGI217133[] = {{&S90901[44], 31}, {NULL}};
const Segment SGI217134[] = {{&S90901[30], 6}, {NULL}};
const Segment SGI217135[] = {{&S90901[89], 6}, {NULL}};
const Segment SGI217136[] = {{&S86188[34], 13}, {NULL}};
const Segment SGI217137[] = {{&S86188[27], 6}, {NULL}};
const Segment SGI217138[] = {{&S90901[116], 13}, {NULL}};
const Segment SGI217139[] = {{&S86188[100], 9}, {NULL}};
const Segment SGI217140[] = {{&S86188[73], 6}, {NULL}};
const Segment SGI217141[] = {{&S90905[34], 9}, {NULL}};
const Segment SGI217142[] = {{&S90905[27], 6}, {NULL}};
const Segment SGI217143[] = {{&S90905[96], 9}, {NULL}};
const Segment SGI217144[] = {{&S90905[69], 6}, {NULL}};
const Segment SGI217145[] = {{&S90906[34], 9}, {NULL}};
const Segment SGI217146[] = {{&S90906[27], 6}, {NULL}};
const Segment SGI217147[] = {{&S90906[96], 9}, {NULL}};
const Segment SGI217148[] = {{&S90906[69], 6}, {NULL}};
const Segment SGI217149[] = {{&S86195[34], 21}, {NULL}};
const Segment SGI217150[] = {{&S86195[27], 6}, {NULL}};
const Segment SGI217151[] = {{&S81084[44], 18}, {NULL}};
const Segment SGI217152[] = {{&S81084[44], 30}, {NULL}};
const Segment SGI217153[] = {{&S81084[30], 6}, {NULL}};
const Segment SGI217154[] = {{&S86195[108], 9}, {NULL}};
const Segment SGI217155[] = {{&S86195[81], 6}, {NULL}};
const Segment SGI217156[] = {{&S81084[123], 3}, {NULL}};
const Segment SGI217157[] = {{&S81084[88], 6}, {NULL}};
const Segment SGI217158[] = {{&S90910[44], 14}, {NULL}};
const Segment SGI217159[] = {{&S90910[44], 31}, {NULL}};
const Segment SGI217160[] = {{&S90910[30], 6}, {NULL}};
const Segment SGI217161[] = {{&S90910[89], 6}, {NULL}};
const Segment SGI217162[] = {{&S90910[116], 13}, {NULL}};
const Segment SGI217163[] = {{&S81088[44], 10}, {NULL}};
const Segment SGI217164[] = {{&S81088[44], 22}, {NULL}};
const Segment SGI217165[] = {{&S81088[30], 6}, {NULL}};
const Segment SGI217166[] = {{&S81088[133], 3}, {NULL}};
const Segment SGI217167[] = {{&S81088[98], 6}, {NULL}};
const Segment SGI217168[] = {{&S81088[68], 8}, {NULL}};
const Segment SGI217169[] = {{&S90915[34], 16}, {NULL}};
const Segment SGI217170[] = {{&S90915[27], 6}, {NULL}};
const Segment SGI217171[] = {{&S90915[110], 9}, {NULL}};
const Segment SGI217172[] = {{&S90915[83], 6}, {NULL}};
const Segment SGI217173[] = {{&S90915[151], 8}, {NULL}};
const Segment SGI217174[] = {{&S90922[34], 10}, {NULL}};
const Segment SGI217175[] = {{&S90922[27], 6}, {NULL}};
const Segment SGI217176[] = {{&S90922[97], 9}, {NULL}};
const Segment SGI217177[] = {{&S90922[70], 6}, {NULL}};
const Segment SGI217178[] = {{&S90924[44], 10}, {NULL}};
const Segment SGI217179[] = {{&S90924[44], 22}, {NULL}};
const Segment SGI217180[] = {{&S90924[30], 6}, {NULL}};
const Segment SGI217181[] = {{&S90924[80], 6}, {NULL}};
const Segment SGI217182[] = {{&S90924[107], 13}, {NULL}};
const Segment SGI217183[] = {{&S90925[44], 14}, {NULL}};
const Segment SGI217184[] = {{&S90925[44], 31}, {NULL}};
const Segment SGI217185[] = {{&S90925[30], 6}, {NULL}};
const Segment SGI217186[] = {{&S90925[89], 6}, {NULL}};
const Segment SGI217187[] = {{&S90925[116], 13}, {NULL}};
const Segment SGI217188[] = {{&S90927[44], 13}, {NULL}};
const Segment SGI217189[] = {{&S90927[44], 25}, {NULL}};
const Segment SGI217190[] = {{&S90927[30], 6}, {NULL}};
const Segment SGI217191[] = {{&S90927[83], 6}, {NULL}};
const Segment SGI217192[] = {{&S90927[110], 13}, {NULL}};
const Segment SGI217193[] = {{&S81111[34], 6}, {NULL}};
const Segment SGI217194[] = {{&S81111[27], 6}, {NULL}};
const Segment SGI217195[] = {{&S81111[95], 9}, {NULL}};
const Segment SGI217196[] = {{&S81111[68], 6}, {NULL}};
const Segment SGI217197[] = {{&S72018[34], 10}, {NULL}};
const Segment SGI217198[] = {{&S72018[27], 6}, {NULL}};
const Segment SGI217199[] = {{&S72018[97], 9}, {NULL}};
const Segment SGI217200[] = {{&S72018[70], 6}, {NULL}};
const Segment SGI217201[] = {{&S72018[138], 8}, {NULL}};
const Segment SGI217202[] = {{&S86251[34], 11}, {NULL}};
const Segment SGI217203[] = {{&S86251[27], 6}, {NULL}};
const Segment SGI217204[] = {{&S86251[98], 9}, {NULL}};
const Segment SGI217205[] = {{&S86251[71], 6}, {NULL}};
const Segment SGI217206[] = {{&S86256[44], 10}, {NULL}};
const Segment SGI217207[] = {{&S86256[44], 22}, {NULL}};
const Segment SGI217208[] = {{&S86256[30], 6}, {NULL}};
const Segment SGI217209[] = {{&S86256[114], 3}, {NULL}};
const Segment SGI217210[] = {{&S86256[80], 5}, {NULL}};
const Segment SGI217211[] = {{&S86256[125], 14}, {NULL}};
const Segment SGI217212[] = {{&S86265[34], 11}, {NULL}};
const Segment SGI217213[] = {{&S86265[27], 6}, {NULL}};
const Segment SGI217214[] = {{&S86265[98], 9}, {NULL}};
const Segment SGI217215[] = {{&S86265[71], 6}, {NULL}};
const Segment SGI217216[] = {{&S86266[34], 9}, {NULL}};
const Segment SGI217217[] = {{&S86266[27], 6}, {NULL}};
const Segment SGI217218[] = {{&S86266[96], 9}, {NULL}};
const Segment SGI217219[] = {{&S86266[69], 6}, {NULL}};
const Segment SGI217220[] = {{&S80425[44], 12}, {NULL}};
const Segment SGI217221[] = {{&S80425[44], 24}, {NULL}};
const Segment SGI217222[] = {{&S80425[30], 6}, {NULL}};
const Segment SGI217223[] = {{&S90979[34], 10}, {NULL}};
const Segment SGI217224[] = {{&S90979[27], 6}, {NULL}};
const Segment SGI217225[] = {{&S80425[133], 3}, {NULL}};
const Segment SGI217226[] = {{&S80425[98], 6}, {NULL}};
const Segment SGI217227[] = {{&S80425[70], 8}, {NULL}};
const Segment SGI217228[] = {{&S90979[97], 9}, {NULL}};
const Segment SGI217229[] = {{&S90979[70], 6}, {NULL}};
const Segment SGI217230[] = {{&S86269[44], 8}, {NULL}};
const Segment SGI217231[] = {{&S86269[44], 20}, {NULL}};
const Segment SGI217232[] = {{&S86269[30], 6}, {NULL}};
const Segment SGI217233[] = {{&S86269[113], 3}, {NULL}};
const Segment SGI217234[] = {{&S86269[78], 6}, {NULL}};
const Segment SGI217235[] = {{&S90980[34], 18}, {NULL}};
const Segment SGI217236[] = {{&S90980[27], 6}, {NULL}};
const Segment SGI217237[] = {{&S90980[105], 9}, {NULL}};
const Segment SGI217238[] = {{&S90980[78], 6}, {NULL}};
const Segment SGI217239[] = {{&S90985[44], 12}, {NULL}};
const Segment SGI217240[] = {{&S90985[44], 24}, {NULL}};
const Segment SGI217241[] = {{&S90985[30], 6}, {NULL}};
const Segment SGI217242[] = {{&S90985[31], 3}, {NULL}};
const Segment SGI217243[] = {{&S90985[82], 6}, {NULL}};
const Segment SGI217244[] = {{&S80435[44], 10}, {NULL}};
const Segment SGI217245[] = {{&S80435[44], 22}, {NULL}};
const Segment SGI217246[] = {{&S80435[30], 6}, {NULL}};
const Segment SGI217247[] = {{&S80435[115], 3}, {NULL}};
const Segment SGI217248[] = {{&S80435[80], 6}, {NULL}};
const Segment SGI217249[] = {{&S80436[44], 10}, {NULL}};
const Segment SGI217250[] = {{&S80436[44], 22}, {NULL}};
const Segment SGI217251[] = {{&S80436[30], 6}, {NULL}};
const Segment SGI217252[] = {{&S80436[83], 6}, {NULL}};
const Segment SGI217253[] = {{&S90989[45], 7}, {NULL}};
const Segment SGI217254[] = {{&S90989[30], 6}, {NULL}};
const Segment SGI217255[] = {{&S90989[31], 3}, {NULL}};
const Segment SGI217256[] = {{&S90989[83], 5}, {NULL}};
const Segment SGI217257[] = {{&S90989[141], 13}, {NULL}};
const Segment SGI217258[] = {{&S80438[44], 23}, {NULL}};
const Segment SGI217259[] = {{&S80438[44], 35}, {NULL}};
const Segment SGI217260[] = {{&S80438[30], 6}, {NULL}};
const Segment SGI217261[] = {{&S80438[128], 3}, {NULL}};
const Segment SGI217262[] = {{&S80438[93], 6}, {NULL}};
const Segment SGI217263[] = {{&S90991[34], 9}, {NULL}};
const Segment SGI217264[] = {{&S90991[27], 6}, {NULL}};
const Segment SGI217265[] = {{&S80439[34], 7}, {NULL}};
const Segment SGI217266[] = {{&S90991[96], 9}, {NULL}};
const Segment SGI217267[] = {{&S80439[27], 6}, {NULL}};
const Segment SGI217268[] = {{&S90991[69], 6}, {NULL}};
const Segment SGI217269[] = {{&S80439[102], 9}, {NULL}};
const Segment SGI217270[] = {{&S80439[75], 6}, {NULL}};
const Segment SGI217271[] = {{&S80440[34], 8}, {NULL}};
const Segment SGI217272[] = {{&S80440[27], 6}, {NULL}};
const Segment SGI217273[] = {{&S80440[103], 9}, {NULL}};
const Segment SGI217274[] = {{&S80440[76], 6}, {NULL}};
const Segment SGI217275[] = {{&S80441[34], 18}, {NULL}};
const Segment SGI217276[] = {{&S80441[27], 6}, {NULL}};
const Segment SGI217277[] = {{&S80441[113], 9}, {NULL}};
const Segment SGI217278[] = {{&S80441[86], 6}, {NULL}};
const Segment SGI217279[] = {{&S80442[44], 14}, {NULL}};
const Segment SGI217280[] = {{&S80442[44], 25}, {NULL}};
const Segment SGI217281[] = {{&S80442[30], 6}, {NULL}};
const Segment SGI217282[] = {{&S80442[118], 3}, {NULL}};
const Segment SGI217283[] = {{&S80442[83], 6}, {NULL}};
const Segment SGI217284[] = {{&S86294[34], 8}, {NULL}};
const Segment SGI217285[] = {{&S86294[27], 6}, {NULL}};
const Segment SGI217286[] = {{&S86294[95], 9}, {NULL}};
const Segment SGI217287[] = {{&S86294[68], 6}, {NULL}};
const Segment SGI217288[] = {{&S91003[34], 8}, {NULL}};
const Segment SGI217289[] = {{&S91003[27], 6}, {NULL}};
const Segment SGI217290[] = {{&S91003[95], 9}, {NULL}};
const Segment SGI217291[] = {{&S91003[68], 6}, {NULL}};
const Segment SGI217292[] = {{&S91004[34], 10}, {NULL}};
const Segment SGI217293[] = {{&S91004[27], 6}, {NULL}};
const Segment SGI217294[] = {{&S91004[97], 9}, {NULL}};
const Segment SGI217295[] = {{&S91004[70], 6}, {NULL}};
const Segment SGI217296[] = {{&S72101[34], 10}, {NULL}};
const Segment SGI217297[] = {{&S72101[27], 6}, {NULL}};
const Segment SGI217298[] = {{&S91005[34], 9}, {NULL}};
const Segment SGI217299[] = {{&S91005[27], 6}, {NULL}};
const Segment SGI217300[] = {{&S72101[96], 9}, {NULL}};
const Segment SGI217301[] = {{&S72101[69], 6}, {NULL}};
const Segment SGI217302[] = {{&S91005[96], 9}, {NULL}};
const Segment SGI217303[] = {{&S91005[69], 6}, {NULL}};
const Segment SGI217304[] = {{&S86302[44], 10}, {NULL}};
const Segment SGI217305[] = {{&S86302[44], 22}, {NULL}};
const Segment SGI217306[] = {{&S86302[30], 6}, {NULL}};
const Segment SGI217307[] = {{&S86302[115], 3}, {NULL}};
const Segment SGI217308[] = {{&S86302[80], 6}, {NULL}};
const Segment SGI217309[] = {{&S86303[34], 25}, {NULL}};
const Segment SGI217310[] = {{&S86303[27], 6}, {NULL}};
const Segment SGI217311[] = {{&S86303[112], 9}, {NULL}};
const Segment SGI217312[] = {{&S86303[85], 6}, {NULL}};
const Segment SGI217313[] = {{&S91008[44], 15}, {NULL}};
const Segment SGI217314[] = {{&S91008[44], 33}, {NULL}};
const Segment SGI217315[] = {{&S91008[30], 6}, {NULL}};
const Segment SGI217316[] = {{&S91008[31], 3}, {NULL}};
const Segment SGI217317[] = {{&S91008[91], 5}, {NULL}};
const Segment SGI217318[] = {{&S91008[128], 14}, {NULL}};
const Segment SGI217319[] = {{&S91009[44], 15}, {NULL}};
const Segment SGI217320[] = {{&S91009[44], 33}, {NULL}};
const Segment SGI217321[] = {{&S91009[30], 6}, {NULL}};
const Segment SGI217322[] = {{&S91009[31], 3}, {NULL}};
const Segment SGI217323[] = {{&S91009[91], 5}, {NULL}};
const Segment SGI217324[] = {{&S91009[128], 14}, {NULL}};
const Segment SGI217325[] = {{&S91013[34], 7}, {NULL}};
const Segment SGI217326[] = {{&S91013[27], 6}, {NULL}};
const Segment SGI217327[] = {{&S91013[98], 9}, {NULL}};
const Segment SGI217328[] = {{&S91013[71], 6}, {NULL}};
const Segment SGI217329[] = {{&S91020[34], 12}, {NULL}};
const Segment SGI217330[] = {{&S91020[27], 6}, {NULL}};
const Segment SGI217331[] = {{&S91020[99], 9}, {NULL}};
const Segment SGI217332[] = {{&S91020[72], 6}, {NULL}};
const Segment SGI217333[] = {{&S80489[34], 11}, {NULL}};
const Segment SGI217334[] = {{&S80489[27], 6}, {NULL}};
const Segment SGI217335[] = {{&S91030[34], 12}, {NULL}};
const Segment SGI217336[] = {{&S91030[27], 6}, {NULL}};
const Segment SGI217337[] = {{&S80489[98], 9}, {NULL}};
const Segment SGI217338[] = {{&S80489[71], 6}, {NULL}};
const Segment SGI217339[] = {{&S91030[99], 9}, {NULL}};
const Segment SGI217340[] = {{&S91030[72], 6}, {NULL}};
const Segment SGI217341[] = {{&S80490[34], 13}, {NULL}};
const Segment SGI217342[] = {{&S80490[27], 6}, {NULL}};
const Segment SGI217343[] = {{&S91031[34], 10}, {NULL}};
const Segment SGI217344[] = {{&S80490[100], 9}, {NULL}};
const Segment SGI217345[] = {{&S91031[27], 6}, {NULL}};
const Segment SGI217346[] = {{&S80490[73], 6}, {NULL}};
const Segment SGI217347[] = {{&S91031[97], 9}, {NULL}};
const Segment SGI217348[] = {{&S91031[70], 6}, {NULL}};
const Segment SGI217349[] = {{&S80491[44], 13}, {NULL}};
const Segment SGI217350[] = {{&S80491[44], 25}, {NULL}};
const Segment SGI217351[] = {{&S80491[30], 6}, {NULL}};
const Segment SGI217352[] = {{&S80491[118], 3}, {NULL}};
const Segment SGI217353[] = {{&S91032[44], 8}, {NULL}};
const Segment SGI217354[] = {{&S91032[44], 20}, {NULL}};
const Segment SGI217355[] = {{&S80491[83], 6}, {NULL}};
const Segment SGI217356[] = {{&S91032[30], 6}, {NULL}};
const Segment SGI217357[] = {{&S91032[31], 3}, {NULL}};
const Segment SGI217358[] = {{&S91032[78], 6}, {NULL}};
const Segment SGI217359[] = {{&S91035[44], 7}, {NULL}};
const Segment SGI217360[] = {{&S91035[44], 24}, {NULL}};
const Segment SGI217361[] = {{&S91035[30], 6}, {NULL}};
const Segment SGI217362[] = {{&S91035[82], 6}, {NULL}};
const Segment SGI217363[] = {{&S91035[109], 13}, {NULL}};
const Segment SGI217364[] = {{&S80496[44], 7}, {NULL}};
const Segment SGI217365[] = {{&S80496[44], 24}, {NULL}};
const Segment SGI217366[] = {{&S80496[30], 6}, {NULL}};
const Segment SGI217367[] = {{&S72142[34], 18}, {NULL}};
const Segment SGI217368[] = {{&S80496[82], 6}, {NULL}};
const Segment SGI217369[] = {{&S72142[27], 6}, {NULL}};
const Segment SGI217370[] = {{&S80496[109], 13}, {NULL}};
const Segment SGI217371[] = {{&S91036[34], 7}, {NULL}};
const Segment SGI217372[] = {{&S72142[104], 9}, {NULL}};
const Segment SGI217373[] = {{&S91036[27], 6}, {NULL}};
const Segment SGI217374[] = {{&S72142[77], 6}, {NULL}};
const Segment SGI217375[] = {{&S72142[145], 8}, {NULL}};
const Segment SGI217376[] = {{&S91036[94], 9}, {NULL}};
const Segment SGI217377[] = {{&S91036[67], 6}, {NULL}};
const Segment SGI217378[] = {{&S80497[44], 7}, {NULL}};
const Segment SGI217379[] = {{&S80497[44], 24}, {NULL}};
const Segment SGI217380[] = {{&S80497[30], 6}, {NULL}};
const Segment SGI217381[] = {{&S80497[82], 6}, {NULL}};
const Segment SGI217382[] = {{&S72143[34], 11}, {NULL}};
const Segment SGI217383[] = {{&S72143[27], 6}, {NULL}};
const Segment SGI217384[] = {{&S80497[109], 13}, {NULL}};
const Segment SGI217385[] = {{&S72143[98], 9}, {NULL}};
const Segment SGI217386[] = {{&S72143[71], 6}, {NULL}};
const Segment SGI217387[] = {{&S91039[34], 9}, {NULL}};
const Segment SGI217388[] = {{&S91039[27], 6}, {NULL}};
const Segment SGI217389[] = {{&S91039[96], 9}, {NULL}};
const Segment SGI217390[] = {{&S91039[69], 6}, {NULL}};
const Segment SGI217391[] = {{&S91041[34], 10}, {NULL}};
const Segment SGI217392[] = {{&S91041[27], 6}, {NULL}};
const Segment SGI217393[] = {{&S91041[97], 9}, {NULL}};
const Segment SGI217394[] = {{&S91041[70], 6}, {NULL}};
const Segment SGI217395[] = {{&S91050[34], 8}, {NULL}};
const Segment SGI217396[] = {{&S91050[27], 6}, {NULL}};
const Segment SGI217397[] = {{&S91050[95], 9}, {NULL}};
const Segment SGI217398[] = {{&S91050[68], 6}, {NULL}};
const Segment SGI217399[] = {{&S91053[34], 10}, {NULL}};
const Segment SGI217400[] = {{&S91053[27], 6}, {NULL}};
const Segment SGI217401[] = {{&S91053[104], 9}, {NULL}};
const Segment SGI217402[] = {{&S91053[77], 6}, {NULL}};
const Segment SGI217403[] = {{&S86367[34], 12}, {NULL}};
const Segment SGI217404[] = {{&S86367[27], 6}, {NULL}};
const Segment SGI217405[] = {{&S86367[99], 9}, {NULL}};
const Segment SGI217406[] = {{&S86367[72], 6}, {NULL}};
const Segment SGI217407[] = {{&S80527[45], 10}, {NULL}};
const Segment SGI217408[] = {{&S80527[30], 6}, {NULL}};
const Segment SGI217409[] = {{&S80527[115], 3}, {NULL}};
const Segment SGI217410[] = {{&S80527[81], 5}, {NULL}};
const Segment SGI217411[] = {{&S80527[139], 13}, {NULL}};
const Segment SGI217412[] = {{&S72172[34], 11}, {NULL}};
const Segment SGI217413[] = {{&S72172[27], 6}, {NULL}};
const Segment SGI217414[] = {{&S72172[97], 9}, {NULL}};
const Segment SGI217415[] = {{&S72172[70], 6}, {NULL}};
const Segment SGI217416[] = {{&S91067[34], 10}, {NULL}};
const Segment SGI217417[] = {{&S91067[27], 6}, {NULL}};
const Segment SGI217418[] = {{&S91067[97], 9}, {NULL}};
const Segment SGI217419[] = {{&S91067[70], 6}, {NULL}};
const Segment SGI217420[] = {{&S91067[138], 8}, {NULL}};
const Segment SGI217421[] = {{&S80538[59], 10}, {NULL}};
const Segment SGI217422[] = {{&S80538[59], 22}, {NULL}};
const Segment SGI217423[] = {{&S80538[45], 6}, {NULL}};
const Segment SGI217424[] = {{&S91070[34], 15}, {NULL}};
const Segment SGI217425[] = {{&S91070[27], 6}, {NULL}};
const Segment SGI217426[] = {{&S80538[130], 3}, {NULL}};
const Segment SGI217427[] = {{&S80538[95], 6}, {NULL}};
const Segment SGI217428[] = {{&S91070[99], 9}, {NULL}};
const Segment SGI217429[] = {{&S80538[11], 3}, {NULL}};
const Segment SGI217430[] = {{&S91070[72], 6}, {NULL}};
const Segment SGI217431[] = {{&S86380[34], 17}, {NULL}};
const Segment SGI217432[] = {{&S86380[27], 6}, {NULL}};
const Segment SGI217433[] = {{&S86380[104], 9}, {NULL}};
const Segment SGI217434[] = {{&S86380[77], 6}, {NULL}};
const Segment SGI217435[] = {{&S80540[59], 10}, {NULL}};
const Segment SGI217436[] = {{&S80540[59], 22}, {NULL}};
const Segment SGI217437[] = {{&S80540[45], 6}, {NULL}};
const Segment SGI217438[] = {{&S80540[130], 3}, {NULL}};
const Segment SGI217439[] = {{&S80540[95], 6}, {NULL}};
const Segment SGI217440[] = {{&S80540[11], 3}, {NULL}};
const Segment SGI217441[] = {{&S80542[57], 10}, {NULL}};
const Segment SGI217442[] = {{&S80542[45], 6}, {NULL}};
const Segment SGI217443[] = {{&S80542[128], 3}, {NULL}};
const Segment SGI217444[] = {{&S80542[93], 6}, {NULL}};
const Segment SGI217445[] = {{&S80542[11], 3}, {NULL}};
const Segment SGI217446[] = {{&S72190[44], 10}, {NULL}};
const Segment SGI217447[] = {{&S72190[44], 21}, {NULL}};
const Segment SGI217448[] = {{&S72190[30], 6}, {NULL}};
const Segment SGI217449[] = {{&S72190[114], 3}, {NULL}};
const Segment SGI217450[] = {{&S72190[79], 6}, {NULL}};
const Segment SGI217451[] = {{&S72191[44], 10}, {NULL}};
const Segment SGI217452[] = {{&S72191[44], 22}, {NULL}};
const Segment SGI217453[] = {{&S72191[30], 6}, {NULL}};
const Segment SGI217454[] = {{&S72191[115], 3}, {NULL}};
const Segment SGI217455[] = {{&S72191[80], 6}, {NULL}};
const Segment SGI217456[] = {{&S80549[44], 10}, {NULL}};
const Segment SGI217457[] = {{&S80549[44], 22}, {NULL}};
const Segment SGI217458[] = {{&S80549[30], 6}, {NULL}};
const Segment SGI217459[] = {{&S91083[34], 10}, {NULL}};
const Segment SGI217460[] = {{&S91083[27], 6}, {NULL}};
const Segment SGI217461[] = {{&S80549[80], 6}, {NULL}};
const Segment SGI217462[] = {{&S86391[44], 12}, {NULL}};
const Segment SGI217463[] = {{&S80549[107], 13}, {NULL}};
const Segment SGI217464[] = {{&S72192[38], 10}, {NULL}};
const Segment SGI217465[] = {{&S91083[97], 9}, {NULL}};
const Segment SGI217466[] = {{&S91083[70], 6}, {NULL}};
const Segment SGI217467[] = {{&S86391[44], 24}, {NULL}};
const Segment SGI217468[] = {{&S72192[31], 6}, {NULL}};
const Segment SGI217469[] = {{&S86391[30], 6}, {NULL}};
const Segment SGI217470[] = {{&S86391[117], 3}, {NULL}};
const Segment SGI217471[] = {{&S86391[82], 6}, {NULL}};
const Segment SGI217472[] = {{&S86391[142], 7}, {NULL}};
const Segment SGI217473[] = {{&S86392[44], 19}, {NULL}};
const Segment SGI217474[] = {{&S86392[44], 31}, {NULL}};
const Segment SGI217475[] = {{&S86392[30], 6}, {NULL}};
const Segment SGI217476[] = {{&S86392[124], 3}, {NULL}};
const Segment SGI217477[] = {{&S86392[89], 6}, {NULL}};
const Segment SGI217478[] = {{&S80557[34], 10}, {NULL}};
const Segment SGI217479[] = {{&S80557[27], 6}, {NULL}};
const Segment SGI217480[] = {{&S80557[97], 9}, {NULL}};
const Segment SGI217481[] = {{&S80557[70], 6}, {NULL}};
const Segment SGI217482[] = {{&S91091[34], 5}, {NULL}};
const Segment SGI217483[] = {{&S91091[27], 6}, {NULL}};
const Segment SGI217484[] = {{&S91091[92], 9}, {NULL}};
const Segment SGI217485[] = {{&S91091[65], 6}, {NULL}};
const Segment SGI217486[] = {{&S80558[34], 11}, {NULL}};
const Segment SGI217487[] = {{&S80558[27], 6}, {NULL}};
const Segment SGI217488[] = {{&S80558[98], 9}, {NULL}};
const Segment SGI217489[] = {{&S80558[71], 6}, {NULL}};
const Segment SGI217490[] = {{&S91092[34], 11}, {NULL}};
const Segment SGI217491[] = {{&S91092[27], 6}, {NULL}};
const Segment SGI217492[] = {{&S81292[34], 8}, {NULL}};
const Segment SGI217493[] = {{&S91092[98], 9}, {NULL}};
const Segment SGI217494[] = {{&S91092[71], 6}, {NULL}};
const Segment SGI217495[] = {{&S81292[27], 6}, {NULL}};
const Segment SGI217496[] = {{&S81292[95], 9}, {NULL}};
const Segment SGI217497[] = {{&S81292[68], 6}, {NULL}};
const Segment SGI217498[] = {{&S86401[34], 7}, {NULL}};
const Segment SGI217499[] = {{&S86401[27], 6}, {NULL}};
const Segment SGI217500[] = {{&S80560[34], 7}, {NULL}};
const Segment SGI217501[] = {{&S80560[27], 6}, {NULL}};
const Segment SGI217502[] = {{&S86401[94], 9}, {NULL}};
const Segment SGI217503[] = {{&S86401[67], 6}, {NULL}};
const Segment SGI217504[] = {{&S80560[98], 9}, {NULL}};
const Segment SGI217505[] = {{&S80560[71], 6}, {NULL}};
const Segment SGI217506[] = {{&S80566[44], 10}, {NULL}};
const Segment SGI217507[] = {{&S80566[44], 22}, {NULL}};
const Segment SGI217508[] = {{&S80566[30], 6}, {NULL}};
const Segment SGI217509[] = {{&S80566[115], 3}, {NULL}};
const Segment SGI217510[] = {{&S80566[80], 6}, {NULL}};
const Segment SGI217511[] = {{&S80566[143], 3}, {NULL}};
const Segment SGI217512[] = {{&S81303[34], 6}, {NULL}};
const Segment SGI217513[] = {{&S81303[27], 6}, {NULL}};
const Segment SGI217514[] = {{&S81303[93], 9}, {NULL}};
const Segment SGI217515[] = {{&S81303[66], 6}, {NULL}};
const Segment SGI217516[] = {{&S86412[44], 11}, {NULL}};
const Segment SGI217517[] = {{&S86412[44], 23}, {NULL}};
const Segment SGI217518[] = {{&S86412[30], 6}, {NULL}};
const Segment SGI217519[] = {{&S86412[81], 6}, {NULL}};
const Segment SGI217520[] = {{&S86412[108], 13}, {NULL}};
const Segment SGI217521[] = {{&S80601[34], 12}, {NULL}};
const Segment SGI217522[] = {{&S80601[27], 6}, {NULL}};
const Segment SGI217523[] = {{&S80601[99], 9}, {NULL}};
const Segment SGI217524[] = {{&S80601[72], 6}, {NULL}};
const Segment SGI217525[] = {{&S91142[44], 10}, {NULL}};
const Segment SGI217526[] = {{&S91142[44], 22}, {NULL}};
const Segment SGI217527[] = {{&S91142[30], 6}, {NULL}};
const Segment SGI217528[] = {{&S91142[31], 3}, {NULL}};
const Segment SGI217529[] = {{&S91142[80], 6}, {NULL}};
const Segment SGI217530[] = {{&S81343[44], 12}, {NULL}};
const Segment SGI217531[] = {{&S81343[44], 24}, {NULL}};
const Segment SGI217532[] = {{&S81343[30], 6}, {NULL}};
const Segment SGI217533[] = {{&S81343[117], 3}, {NULL}};
const Segment SGI217534[] = {{&S80610[44], 10}, {NULL}};
const Segment SGI217535[] = {{&S81343[82], 6}, {NULL}};
const Segment SGI217536[] = {{&S80610[44], 22}, {NULL}};
const Segment SGI217537[] = {{&S80610[30], 6}, {NULL}};
const Segment SGI217538[] = {{&S80610[131], 3}, {NULL}};
const Segment SGI217539[] = {{&S80610[96], 6}, {NULL}};
const Segment SGI217540[] = {{&S80610[68], 8}, {NULL}};
const Segment SGI217541[] = {{&S86466[44], 17}, {NULL}};
const Segment SGI217542[] = {{&S86466[44], 29}, {NULL}};
const Segment SGI217543[] = {{&S86466[30], 6}, {NULL}};
const Segment SGI217544[] = {{&S86466[122], 3}, {NULL}};
const Segment SGI217545[] = {{&S86466[87], 6}, {NULL}};
const Segment SGI217546[] = {{&S80631[44], 10}, {NULL}};
const Segment SGI217547[] = {{&S86474[34], 10}, {NULL}};
const Segment SGI217548[] = {{&S80631[44], 22}, {NULL}};
const Segment SGI217549[] = {{&S86474[27], 6}, {NULL}};
const Segment SGI217550[] = {{&S80631[30], 6}, {NULL}};
const Segment SGI217551[] = {{&S80631[133], 3}, {NULL}};
const Segment SGI217552[] = {{&S86474[97], 9}, {NULL}};
const Segment SGI217553[] = {{&S86474[70], 6}, {NULL}};
const Segment SGI217554[] = {{&S80631[98], 6}, {NULL}};
const Segment SGI217555[] = {{&S80631[68], 8}, {NULL}};
const Segment SGI217556[] = {{&S80632[44], 10}, {NULL}};
const Segment SGI217557[] = {{&S80632[44], 34}, {NULL}};
const Segment SGI217558[] = {{&S80632[30], 6}, {NULL}};
const Segment SGI217559[] = {{&S80632[143], 3}, {NULL}};
const Segment SGI217560[] = {{&S80632[108], 6}, {NULL}};
const Segment SGI217561[] = {{&S80632[80], 8}, {NULL}};
const Segment SGI217562[] = {{&S91172[34], 20}, {NULL}};
const Segment SGI217563[] = {{&S91172[27], 6}, {NULL}};
const Segment SGI217564[] = {{&S91172[107], 9}, {NULL}};
const Segment SGI217565[] = {{&S91172[80], 6}, {NULL}};
const Segment SGI217566[] = {{&S80633[44], 10}, {NULL}};
const Segment SGI217567[] = {{&S80633[44], 29}, {NULL}};
const Segment SGI217568[] = {{&S80633[30], 6}, {NULL}};
const Segment SGI217569[] = {{&S80633[138], 3}, {NULL}};
const Segment SGI217570[] = {{&S80633[103], 6}, {NULL}};
const Segment SGI217571[] = {{&S80633[75], 8}, {NULL}};
const Segment SGI217572[] = {{&S91173[34], 11}, {NULL}};
const Segment SGI217573[] = {{&S91173[27], 6}, {NULL}};
const Segment SGI217574[] = {{&S91173[98], 9}, {NULL}};
const Segment SGI217575[] = {{&S91173[71], 6}, {NULL}};
const Segment SGI217576[] = {{&S91175[59], 10}, {NULL}};
const Segment SGI217577[] = {{&S91175[59], 22}, {NULL}};
const Segment SGI217578[] = {{&S91175[45], 6}, {NULL}};
const Segment SGI217579[] = {{&S91175[46], 3}, {NULL}};
const Segment SGI217580[] = {{&S91175[95], 6}, {NULL}};
const Segment SGI217581[] = {{&S91175[0], 14}, {NULL}};
const Segment SGI217582[] = {{&S91181[34], 9}, {NULL}};
const Segment SGI217583[] = {{&S91181[27], 6}, {NULL}};
const Segment SGI217584[] = {{&S91181[128], 9}, {NULL}};
const Segment SGI217585[] = {{&S91181[101], 6}, {NULL}};
const Segment SGI217586[] = {{&S91186[34], 10}, {NULL}};
const Segment SGI217587[] = {{&S91186[27], 6}, {NULL}};
const Segment SGI217588[] = {{&S91186[97], 9}, {NULL}};
const Segment SGI217589[] = {{&S91186[70], 6}, {NULL}};
const Segment SGI217590[] = {{&S91188[44], 10}, {NULL}};
const Segment SGI217591[] = {{&S91188[44], 22}, {NULL}};
const Segment SGI217592[] = {{&S91188[30], 6}, {NULL}};
const Segment SGI217593[] = {{&S91188[31], 3}, {NULL}};
const Segment SGI217594[] = {{&S91188[80], 6}, {NULL}};
const Segment SGI217595[] = {{&S91188[133], 11}, {NULL}};
const Segment SGI217596[] = {{&S91189[44], 10}, {NULL}};
const Segment SGI217597[] = {{&S91189[44], 22}, {NULL}};
const Segment SGI217598[] = {{&S91189[30], 6}, {NULL}};
const Segment SGI217599[] = {{&S91189[31], 3}, {NULL}};
const Segment SGI217600[] = {{&S91189[80], 6}, {NULL}};
const Segment SGI217601[] = {{&S91189[133], 11}, {NULL}};
const Segment SGI217602[] = {{&S91190[44], 10}, {NULL}};
const Segment SGI217603[] = {{&S91190[44], 22}, {NULL}};
const Segment SGI217604[] = {{&S91190[30], 6}, {NULL}};
const Segment SGI217605[] = {{&S91190[31], 3}, {NULL}};
const Segment SGI217606[] = {{&S91190[80], 6}, {NULL}};
const Segment SGI217607[] = {{&S91190[134], 12}, {NULL}};
const Segment SGI217608[] = {{&S91191[45], 7}, {NULL}};
const Segment SGI217609[] = {{&S91191[30], 6}, {NULL}};
const Segment SGI217610[] = {{&S91191[31], 3}, {NULL}};
const Segment SGI217611[] = {{&S91191[82], 5}, {NULL}};
const Segment SGI217612[] = {{&S86507[34], 10}, {NULL}};
const Segment SGI217613[] = {{&S86507[27], 6}, {NULL}};
const Segment SGI217614[] = {{&S91191[140], 13}, {NULL}};
const Segment SGI217615[] = {{&S86507[97], 9}, {NULL}};
const Segment SGI217616[] = {{&S86507[70], 6}, {NULL}};
const Segment SGI217617[] = {{&S91192[34], 14}, {NULL}};
const Segment SGI217618[] = {{&S91192[27], 6}, {NULL}};
const Segment SGI217619[] = {{&S91192[101], 9}, {NULL}};
const Segment SGI217620[] = {{&S91192[74], 6}, {NULL}};
const Segment SGI217621[] = {{&S91193[44], 23}, {NULL}};
const Segment SGI217622[] = {{&S91193[44], 35}, {NULL}};
const Segment SGI217623[] = {{&S91193[30], 6}, {NULL}};
const Segment SGI217624[] = {{&S91193[31], 3}, {NULL}};
const Segment SGI217625[] = {{&S91193[93], 6}, {NULL}};
const Segment SGI217626[] = {{&S91194[44], 7}, {NULL}};
const Segment SGI217627[] = {{&S91194[44], 19}, {NULL}};
const Segment SGI217628[] = {{&S91194[30], 6}, {NULL}};
const Segment SGI217629[] = {{&S91194[31], 3}, {NULL}};
const Segment SGI217630[] = {{&S91194[77], 6}, {NULL}};
const Segment SGI217631[] = {{&S91197[34], 10}, {NULL}};
const Segment SGI217632[] = {{&S91197[27], 6}, {NULL}};
const Segment SGI217633[] = {{&S91197[97], 9}, {NULL}};
const Segment SGI217634[] = {{&S91197[70], 6}, {NULL}};
const Segment SGI217635[] = {{&S86519[34], 7}, {NULL}};
const Segment SGI217636[] = {{&S86519[27], 6}, {NULL}};
const Segment SGI217637[] = {{&S86519[94], 9}, {NULL}};
const Segment SGI217638[] = {{&S86519[67], 6}, {NULL}};
const Segment SGI217639[] = {{&S91202[44], 7}, {NULL}};
const Segment SGI217640[] = {{&S91202[44], 19}, {NULL}};
const Segment SGI217641[] = {{&S91202[30], 6}, {NULL}};
const Segment SGI217642[] = {{&S91202[31], 3}, {NULL}};
const Segment SGI217643[] = {{&S91202[77], 5}, {NULL}};
const Segment SGI217644[] = {{&S91202[114], 14}, {NULL}};
const Segment SGI217645[] = {{&S80683[34], 6}, {NULL}};
const Segment SGI217646[] = {{&S80683[27], 6}, {NULL}};
const Segment SGI217647[] = {{&S80683[93], 9}, {NULL}};
const Segment SGI217648[] = {{&S80683[66], 6}, {NULL}};
const Segment SGI217649[] = {{&S91204[44], 14}, {NULL}};
const Segment SGI217650[] = {{&S91204[44], 26}, {NULL}};
const Segment SGI217651[] = {{&S91204[30], 6}, {NULL}};
const Segment SGI217652[] = {{&S91204[31], 3}, {NULL}};
const Segment SGI217653[] = {{&S91204[84], 6}, {NULL}};
const Segment SGI217654[] = {{&S91207[44], 11}, {NULL}};
const Segment SGI217655[] = {{&S91207[44], 27}, {NULL}};
const Segment SGI217656[] = {{&S91207[30], 6}, {NULL}};
const Segment SGI217657[] = {{&S91207[31], 3}, {NULL}};
const Segment SGI217658[] = {{&S91207[85], 6}, {NULL}};
const Segment SGI217659[] = {{&S80691[34], 7}, {NULL}};
const Segment SGI217660[] = {{&S80691[27], 6}, {NULL}};
const Segment SGI217661[] = {{&S80691[94], 9}, {NULL}};
const Segment SGI217662[] = {{&S80691[67], 6}, {NULL}};
const Segment SGI217663[] = {{&S80692[34], 8}, {NULL}};
const Segment SGI217664[] = {{&S80692[27], 6}, {NULL}};
const Segment SGI217665[] = {{&S80692[95], 9}, {NULL}};
const Segment SGI217666[] = {{&S80692[68], 6}, {NULL}};
const Segment SGI217667[] = {{&S91211[34], 7}, {NULL}};
const Segment SGI217668[] = {{&S91211[27], 6}, {NULL}};
const Segment SGI217669[] = {{&S81421[34], 10}, {NULL}};
const Segment SGI217670[] = {{&S81421[27], 6}, {NULL}};
const Segment SGI217671[] = {{&S91211[98], 9}, {NULL}};
const Segment SGI217672[] = {{&S91211[71], 6}, {NULL}};
const Segment SGI217673[] = {{&S81421[97], 9}, {NULL}};
const Segment SGI217674[] = {{&S81421[70], 6}, {NULL}};
const Segment SGI217675[] = {{&S91214[34], 6}, {NULL}};
const Segment SGI217676[] = {{&S91214[27], 6}, {NULL}};
const Segment SGI217677[] = {{&S91214[93], 9}, {NULL}};
const Segment SGI217678[] = {{&S80697[34], 18}, {NULL}};
const Segment SGI217679[] = {{&S91214[66], 6}, {NULL}};
const Segment SGI217680[] = {{&S80697[27], 6}, {NULL}};
const Segment SGI217681[] = {{&S80697[105], 9}, {NULL}};
const Segment SGI217682[] = {{&S80697[78], 6}, {NULL}};
const Segment SGI217683[] = {{&S91215[34], 13}, {NULL}};
const Segment SGI217684[] = {{&S91215[27], 6}, {NULL}};
const Segment SGI217685[] = {{&S80698[34], 7}, {NULL}};
const Segment SGI217686[] = {{&S91215[100], 9}, {NULL}};
const Segment SGI217687[] = {{&S91215[73], 6}, {NULL}};
const Segment SGI217688[] = {{&S80698[27], 6}, {NULL}};
const Segment SGI217689[] = {{&S80698[99], 9}, {NULL}};
const Segment SGI217690[] = {{&S80698[72], 6}, {NULL}};
const Segment SGI217691[] = {{&S91217[34], 10}, {NULL}};
const Segment SGI217692[] = {{&S91217[27], 6}, {NULL}};
const Segment SGI217693[] = {{&S91217[97], 9}, {NULL}};
const Segment SGI217694[] = {{&S91217[70], 6}, {NULL}};
const Segment SGI217695[] = {{&S80709[34], 9}, {NULL}};
const Segment SGI217696[] = {{&S80709[27], 6}, {NULL}};
const Segment SGI217697[] = {{&S72235[34], 6}, {NULL}};
const Segment SGI217698[] = {{&S72235[27], 6}, {NULL}};
const Segment SGI217699[] = {{&S80709[96], 9}, {NULL}};
const Segment SGI217700[] = {{&S80709[69], 6}, {NULL}};
const Segment SGI217701[] = {{&S91227[44], 6}, {NULL}};
const Segment SGI217702[] = {{&S72235[92], 9}, {NULL}};
const Segment SGI217703[] = {{&S91227[44], 18}, {NULL}};
const Segment SGI217704[] = {{&S72235[65], 6}, {NULL}};
const Segment SGI217705[] = {{&S91227[30], 6}, {NULL}};
const Segment SGI217706[] = {{&S91227[76], 6}, {NULL}};
const Segment SGI217707[] = {{&S91227[103], 13}, {NULL}};
const Segment SGI217708[] = {{&S80710[34], 9}, {NULL}};
const Segment SGI217709[] = {{&S80710[27], 6}, {NULL}};
const Segment SGI217710[] = {{&S80710[96], 9}, {NULL}};
const Segment SGI217711[] = {{&S80710[69], 6}, {NULL}};
const Segment SGI217712[] = {{&S91232[44], 15}, {NULL}};
const Segment SGI217713[] = {{&S91232[44], 27}, {NULL}};
const Segment SGI217714[] = {{&S91232[30], 6}, {NULL}};
const Segment SGI217715[] = {{&S91232[31], 3}, {NULL}};
const Segment SGI217716[] = {{&S91232[85], 6}, {NULL}};
const Segment SGI217717[] = {{&S91234[34], 12}, {NULL}};
const Segment SGI217718[] = {{&S91234[27], 6}, {NULL}};
const Segment SGI217719[] = {{&S91234[99], 9}, {NULL}};
const Segment SGI217720[] = {{&S91234[72], 6}, {NULL}};
const Segment SGI217721[] = {{&S86570[34], 14}, {NULL}};
const Segment SGI217722[] = {{&S86570[27], 6}, {NULL}};
const Segment SGI217723[] = {{&S86570[101], 9}, {NULL}};
const Segment SGI217724[] = {{&S86570[74], 6}, {NULL}};
const Segment SGI217725[] = {{&S91235[34], 16}, {NULL}};
const Segment SGI217726[] = {{&S91235[27], 6}, {NULL}};
const Segment SGI217727[] = {{&S91235[103], 9}, {NULL}};
const Segment SGI217728[] = {{&S91235[76], 6}, {NULL}};
const Segment SGI217729[] = {{&S86577[34], 10}, {NULL}};
const Segment SGI217730[] = {{&S86577[27], 6}, {NULL}};
const Segment SGI217731[] = {{&S86577[97], 9}, {NULL}};
const Segment SGI217732[] = {{&S86577[70], 6}, {NULL}};
const Segment SGI217733[] = {{&S86583[34], 7}, {NULL}};
const Segment SGI217734[] = {{&S86583[27], 6}, {NULL}};
const Segment SGI217735[] = {{&S86583[99], 9}, {NULL}};
const Segment SGI217736[] = {{&S86583[72], 6}, {NULL}};
const Segment SGI217737[] = {{&S91249[34], 5}, {NULL}};
const Segment SGI217738[] = {{&S91249[27], 6}, {NULL}};
const Segment SGI217739[] = {{&S91249[92], 9}, {NULL}};
const Segment SGI217740[] = {{&S91249[65], 6}, {NULL}};
const Segment SGI217741[] = {{&S91249[133], 8}, {NULL}};
const Segment SGI217742[] = {{&S72262[34], 10}, {NULL}};
const Segment SGI217743[] = {{&S72262[27], 6}, {NULL}};
const Segment SGI217744[] = {{&S91250[34], 7}, {NULL}};
const Segment SGI217745[] = {{&S91250[27], 6}, {NULL}};
const Segment SGI217746[] = {{&S72262[96], 9}, {NULL}};
const Segment SGI217747[] = {{&S72262[69], 6}, {NULL}};
const Segment SGI217748[] = {{&S91250[99], 9}, {NULL}};
const Segment SGI217749[] = {{&S91250[72], 6}, {NULL}};
const Segment SGI217750[] = {{&S91250[140], 8}, {NULL}};
const Segment SGI217751[] = {{&S72263[34], 10}, {NULL}};
const Segment SGI217752[] = {{&S86587[44], 18}, {NULL}};
const Segment SGI217753[] = {{&S72263[27], 6}, {NULL}};
const Segment SGI217754[] = {{&S86587[44], 30}, {NULL}};
const Segment SGI217755[] = {{&S86587[30], 6}, {NULL}};
const Segment SGI217756[] = {{&S91251[34], 10}, {NULL}};
const Segment SGI217757[] = {{&S91251[27], 6}, {NULL}};
const Segment SGI217758[] = {{&S72263[96], 9}, {NULL}};
const Segment SGI217759[] = {{&S86587[123], 3}, {NULL}};
const Segment SGI217760[] = {{&S72263[69], 6}, {NULL}};
const Segment SGI217761[] = {{&S86587[88], 6}, {NULL}};
const Segment SGI217762[] = {{&S91251[97], 9}, {NULL}};
const Segment SGI217763[] = {{&S91251[70], 6}, {NULL}};
const Segment SGI217764[] = {{&S86588[34], 9}, {NULL}};
const Segment SGI217765[] = {{&S86588[27], 6}, {NULL}};
const Segment SGI217766[] = {{&S91252[34], 14}, {NULL}};
const Segment SGI217767[] = {{&S91252[27], 6}, {NULL}};
const Segment SGI217768[] = {{&S86588[96], 9}, {NULL}};
const Segment SGI217769[] = {{&S86588[69], 6}, {NULL}};
const Segment SGI217770[] = {{&S91252[101], 9}, {NULL}};
const Segment SGI217771[] = {{&S91252[74], 6}, {NULL}};
const Segment SGI217772[] = {{&S91253[34], 12}, {NULL}};
const Segment SGI217773[] = {{&S91253[27], 6}, {NULL}};
const Segment SGI217774[] = {{&S91253[99], 9}, {NULL}};
const Segment SGI217775[] = {{&S91253[72], 6}, {NULL}};
const Segment SGI217776[] = {{&S91258[34], 4}, {NULL}};
const Segment SGI217777[] = {{&S91258[27], 6}, {NULL}};
const Segment SGI217778[] = {{&S91258[91], 9}, {NULL}};
const Segment SGI217779[] = {{&S91258[64], 6}, {NULL}};
const Segment SGI217780[] = {{&S86597[34], 10}, {NULL}};
const Segment SGI217781[] = {{&S86597[27], 6}, {NULL}};
const Segment SGI217782[] = {{&S86597[97], 9}, {NULL}};
const Segment SGI217783[] = {{&S86597[70], 6}, {NULL}};
const Segment SGI217784[] = {{&S91261[34], 10}, {NULL}};
const Segment SGI217785[] = {{&S91261[27], 6}, {NULL}};
const Segment SGI217786[] = {{&S91261[97], 9}, {NULL}};
const Segment SGI217787[] = {{&S86599[34], 6}, {NULL}};
const Segment SGI217788[] = {{&S91261[70], 6}, {NULL}};
const Segment SGI217789[] = {{&S86599[27], 6}, {NULL}};
const Segment SGI217790[] = {{&S86599[93], 9}, {NULL}};
const Segment SGI217791[] = {{&S86599[66], 6}, {NULL}};
const Segment SGI217792[] = {{&S91267[34], 10}, {NULL}};
const Segment SGI217793[] = {{&S91267[27], 6}, {NULL}};
const Segment SGI217794[] = {{&S91267[97], 9}, {NULL}};
const Segment SGI217795[] = {{&S91267[70], 6}, {NULL}};
const Segment SGI217796[] = {{&S80763[38], 28}, {NULL}};
const Segment SGI217797[] = {{&S80763[31], 6}, {NULL}};
const Segment SGI217798[] = {{&S91268[34], 9}, {NULL}};
const Segment SGI217799[] = {{&S91268[27], 6}, {NULL}};
const Segment SGI217800[] = {{&S91268[96], 9}, {NULL}};
const Segment SGI217801[] = {{&S91268[69], 6}, {NULL}};
const Segment SGI217802[] = {{&S80764[44], 28}, {NULL}};
const Segment SGI217803[] = {{&S80764[44], 40}, {NULL}};
const Segment SGI217804[] = {{&S80764[30], 6}, {NULL}};
const Segment SGI217805[] = {{&S80764[133], 3}, {NULL}};
const Segment SGI217806[] = {{&S80764[98], 6}, {NULL}};
const Segment SGI217807[] = {{&S81494[34], 25}, {NULL}};
const Segment SGI217808[] = {{&S81494[27], 6}, {NULL}};
const Segment SGI217809[] = {{&S81494[112], 9}, {NULL}};
const Segment SGI217810[] = {{&S81494[85], 6}, {NULL}};
const Segment SGI217811[] = {{&S80766[34], 10}, {NULL}};
const Segment SGI217812[] = {{&S80766[27], 6}, {NULL}};
const Segment SGI217813[] = {{&S80766[97], 9}, {NULL}};
const Segment SGI217814[] = {{&S80766[70], 6}, {NULL}};
const Segment SGI217815[] = {{&S72293[34], 9}, {NULL}};
const Segment SGI217816[] = {{&S72293[27], 6}, {NULL}};
const Segment SGI217817[] = {{&S72293[95], 9}, {NULL}};
const Segment SGI217818[] = {{&S72293[68], 6}, {NULL}};
const Segment SGI217819[] = {{&S80772[34], 15}, {NULL}};
const Segment SGI217820[] = {{&S80772[27], 6}, {NULL}};
const Segment SGI217821[] = {{&S80772[102], 9}, {NULL}};
const Segment SGI217822[] = {{&S80772[75], 6}, {NULL}};
const Segment SGI217823[] = {{&S86618[44], 15}, {NULL}};
const Segment SGI217824[] = {{&S86618[44], 27}, {NULL}};
const Segment SGI217825[] = {{&S86618[30], 6}, {NULL}};
const Segment SGI217826[] = {{&S86618[120], 3}, {NULL}};
const Segment SGI217827[] = {{&S86618[85], 6}, {NULL}};
const Segment SGI217828[] = {{&S80773[34], 7}, {NULL}};
const Segment SGI217829[] = {{&S80773[27], 6}, {NULL}};
const Segment SGI217830[] = {{&S81504[44], 17}, {NULL}};
const Segment SGI217831[] = {{&S81504[44], 40}, {NULL}};
const Segment SGI217832[] = {{&S80773[99], 9}, {NULL}};
const Segment SGI217833[] = {{&S81504[30], 6}, {NULL}};
const Segment SGI217834[] = {{&S80773[72], 6}, {NULL}};
const Segment SGI217835[] = {{&S81504[133], 3}, {NULL}};
const Segment SGI217836[] = {{&S81504[98], 6}, {NULL}};
const Segment SGI217837[] = {{&S81509[44], 5}, {NULL}};
const Segment SGI217838[] = {{&S81509[44], 17}, {NULL}};
const Segment SGI217839[] = {{&S81509[30], 6}, {NULL}};
const Segment SGI217840[] = {{&S81509[110], 3}, {NULL}};
const Segment SGI217841[] = {{&S81509[75], 6}, {NULL}};
const Segment SGI217842[] = {{&S86628[11], 1}, {NULL}};
const Segment SGI217843[] = {{&S86628[27], 6}, {NULL}};
const Segment SGI217844[] = {{&S86628[88], 9}, {NULL}};
const Segment SGI217845[] = {{&S86628[61], 6}, {NULL}};
const Segment SGI217846[] = {{&S86628[122], 8}, {NULL}};
const Segment SGI217847[] = {{&S81513[34], 6}, {NULL}};
const Segment SGI217848[] = {{&S81513[27], 6}, {NULL}};
const Segment SGI217849[] = {{&S81513[93], 9}, {NULL}};
const Segment SGI217850[] = {{&S81513[66], 6}, {NULL}};
const Segment SGI217851[] = {{&S72310[34], 10}, {NULL}};
const Segment SGI217852[] = {{&S72310[27], 6}, {NULL}};
const Segment SGI217853[] = {{&S72310[97], 9}, {NULL}};
const Segment SGI217854[] = {{&S72310[70], 6}, {NULL}};
const Segment SGI217855[] = {{&S86632[34], 7}, {NULL}};
const Segment SGI217856[] = {{&S86632[27], 6}, {NULL}};
const Segment SGI217857[] = {{&S86632[94], 9}, {NULL}};
const Segment SGI217858[] = {{&S86632[67], 6}, {NULL}};
const Segment SGI217859[] = {{&S80788[34], 11}, {NULL}};
const Segment SGI217860[] = {{&S80788[27], 6}, {NULL}};
const Segment SGI217861[] = {{&S80788[98], 9}, {NULL}};
const Segment SGI217862[] = {{&S80788[71], 6}, {NULL}};
const Segment SGI217863[] = {{&S80789[34], 7}, {NULL}};
const Segment SGI217864[] = {{&S80789[27], 6}, {NULL}};
const Segment SGI217865[] = {{&S80789[94], 9}, {NULL}};
const Segment SGI217866[] = {{&S80789[67], 6}, {NULL}};
const Segment SGI217867[] = {{&S86635[44], 17}, {NULL}};
const Segment SGI217868[] = {{&S86635[44], 29}, {NULL}};
const Segment SGI217869[] = {{&S86635[30], 6}, {NULL}};
const Segment SGI217870[] = {{&S91304[34], 11}, {NULL}};
const Segment SGI217871[] = {{&S91304[27], 6}, {NULL}};
const Segment SGI217872[] = {{&S91304[98], 9}, {NULL}};
const Segment SGI217873[] = {{&S91304[71], 6}, {NULL}};
const Segment SGI217874[] = {{&S81543[34], 10}, {NULL}};
const Segment SGI217875[] = {{&S81543[27], 6}, {NULL}};
const Segment SGI217876[] = {{&S91328[44], 10}, {NULL}};
const Segment SGI217877[] = {{&S91328[44], 20}, {NULL}};
const Segment SGI217878[] = {{&S81543[97], 9}, {NULL}};
const Segment SGI217879[] = {{&S91328[30], 6}, {NULL}};
const Segment SGI217880[] = {{&S81543[70], 6}, {NULL}};
const Segment SGI217881[] = {{&S91328[31], 3}, {NULL}};
const Segment SGI217882[] = {{&S91328[78], 6}, {NULL}};
const Segment SGI217883[] = {{&S91338[34], 11}, {NULL}};
const Segment SGI217884[] = {{&S91338[27], 6}, {NULL}};
const Segment SGI217885[] = {{&S91338[98], 9}, {NULL}};
const Segment SGI217886[] = {{&S91338[71], 6}, {NULL}};
const Segment SGI217887[] = {{&S72356[34], 15}, {NULL}};
const Segment SGI217888[] = {{&S72356[27], 6}, {NULL}};
const Segment SGI217889[] = {{&S72356[102], 9}, {NULL}};
const Segment SGI217890[] = {{&S72356[75], 6}, {NULL}};
const Segment SGI217891[] = {{&S91344[34], 11}, {NULL}};
const Segment SGI217892[] = {{&S91344[27], 6}, {NULL}};
const Segment SGI217893[] = {{&S91344[98], 9}, {NULL}};
const Segment SGI217894[] = {{&S91344[71], 6}, {NULL}};
const Segment SGI217895[] = {{&S91345[34], 7}, {NULL}};
const Segment SGI217896[] = {{&S91345[27], 6}, {NULL}};
const Segment SGI217897[] = {{&S91345[100], 9}, {NULL}};
const Segment SGI217898[] = {{&S91345[73], 6}, {NULL}};
const Segment SGI217899[] = {{&S91347[44], 5}, {NULL}};
const Segment SGI217900[] = {{&S91347[44], 17}, {NULL}};
const Segment SGI217901[] = {{&S91347[30], 6}, {NULL}};
const Segment SGI217902[] = {{&S91347[31], 3}, {NULL}};
const Segment SGI217903[] = {{&S91347[75], 6}, {NULL}};
const Segment SGI217904[] = {{&S91348[40], 13}, {NULL}};
const Segment SGI217905[] = {{&S91348[27], 11}, {NULL}};
const Segment SGI217906[] = {{&S91348[106], 9}, {NULL}};
const Segment SGI217907[] = {{&S91348[79], 6}, {NULL}};
const Segment SGI217908[] = {{&S80841[34], 10}, {NULL}};
const Segment SGI217909[] = {{&S80841[27], 6}, {NULL}};
const Segment SGI217910[] = {{&S91349[34], 8}, {NULL}};
const Segment SGI217911[] = {{&S80841[97], 9}, {NULL}};
const Segment SGI217912[] = {{&S91349[27], 6}, {NULL}};
const Segment SGI217913[] = {{&S80841[70], 6}, {NULL}};
const Segment SGI217914[] = {{&S91349[95], 9}, {NULL}};
const Segment SGI217915[] = {{&S91349[68], 6}, {NULL}};
const Segment SGI217916[] = {{&S91350[34], 12}, {NULL}};
const Segment SGI217917[] = {{&S91350[27], 6}, {NULL}};
const Segment SGI217918[] = {{&S80843[34], 10}, {NULL}};
const Segment SGI217919[] = {{&S80843[27], 6}, {NULL}};
const Segment SGI217920[] = {{&S91350[112], 9}, {NULL}};
const Segment SGI217921[] = {{&S91350[85], 6}, {NULL}};
const Segment SGI217922[] = {{&S80843[97], 9}, {NULL}};
const Segment SGI217923[] = {{&S80843[70], 6}, {NULL}};
const Segment SGI217924[] = {{&S91351[44], 23}, {NULL}};
const Segment SGI217925[] = {{&S91351[44], 35}, {NULL}};
const Segment SGI217926[] = {{&S80844[34], 11}, {NULL}};
const Segment SGI217927[] = {{&S91351[30], 6}, {NULL}};
const Segment SGI217928[] = {{&S80844[27], 6}, {NULL}};
const Segment SGI217929[] = {{&S91351[31], 3}, {NULL}};
const Segment SGI217930[] = {{&S91351[93], 6}, {NULL}};
const Segment SGI217931[] = {{&S80844[98], 9}, {NULL}};
const Segment SGI217932[] = {{&S80844[71], 6}, {NULL}};
const Segment SGI217933[] = {{&S80845[34], 10}, {NULL}};
const Segment SGI217934[] = {{&S80845[27], 6}, {NULL}};
const Segment SGI217935[] = {{&S80845[97], 9}, {NULL}};
const Segment SGI217936[] = {{&S80845[70], 6}, {NULL}};
const Segment SGI217937[] = {{&S80847[44], 10}, {NULL}};
const Segment SGI217938[] = {{&S80847[44], 22}, {NULL}};
const Segment SGI217939[] = {{&S80847[30], 6}, {NULL}};
const Segment SGI217940[] = {{&S80847[115], 3}, {NULL}};
const Segment SGI217941[] = {{&S80847[80], 6}, {NULL}};
const Segment SGI217942[] = {{&S91359[44], 9}, {NULL}};
const Segment SGI217943[] = {{&S91359[44], 21}, {NULL}};
const Segment SGI217944[] = {{&S91359[30], 6}, {NULL}};
const Segment SGI217945[] = {{&S80850[34], 7}, {NULL}};
const Segment SGI217946[] = {{&S80850[27], 6}, {NULL}};
const Segment SGI217947[] = {{&S91359[31], 3}, {NULL}};
const Segment SGI217948[] = {{&S91359[79], 6}, {NULL}};
const Segment SGI217949[] = {{&S80850[98], 9}, {NULL}};
const Segment SGI217950[] = {{&S80850[71], 6}, {NULL}};
const Segment SGI217951[] = {{&S91360[45], 11}, {NULL}};
const Segment SGI217952[] = {{&S80851[34], 7}, {NULL}};
const Segment SGI217953[] = {{&S91360[30], 6}, {NULL}};
const Segment SGI217954[] = {{&S80851[27], 6}, {NULL}};
const Segment SGI217955[] = {{&S91360[31], 3}, {NULL}};
const Segment SGI217956[] = {{&S80851[98], 9}, {NULL}};
const Segment SGI217957[] = {{&S91360[82], 5}, {NULL}};
const Segment SGI217958[] = {{&S80851[71], 6}, {NULL}};
const Segment SGI217959[] = {{&S91360[140], 13}, {NULL}};
const Segment SGI217960[] = {{&S91362[34], 15}, {NULL}};
const Segment SGI217961[] = {{&S91362[27], 6}, {NULL}};
const Segment SGI217962[] = {{&S91362[102], 9}, {NULL}};
const Segment SGI217963[] = {{&S91362[75], 6}, {NULL}};
const Segment SGI217964[] = {{&S91363[34], 11}, {NULL}};
const Segment SGI217965[] = {{&S91363[27], 6}, {NULL}};
const Segment SGI217966[] = {{&S91363[98], 9}, {NULL}};
const Segment SGI217967[] = {{&S91363[71], 6}, {NULL}};
const Segment SGI217968[] = {{&S72392[44], 16}, {NULL}};
const Segment SGI217969[] = {{&S72392[44], 27}, {NULL}};
const Segment SGI217970[] = {{&S72392[30], 6}, {NULL}};
const Segment SGI217971[] = {{&S72392[120], 3}, {NULL}};
const Segment SGI217972[] = {{&S72392[85], 6}, {NULL}};
const Segment SGI217973[] = {{&S72393[38], 16}, {NULL}};
const Segment SGI217974[] = {{&S91365[44], 10}, {NULL}};
const Segment SGI217975[] = {{&S72393[31], 6}, {NULL}};
const Segment SGI217976[] = {{&S91365[44], 22}, {NULL}};
const Segment SGI217977[] = {{&S91365[30], 6}, {NULL}};
const Segment SGI217978[] = {{&S91365[83], 6}, {NULL}};
const Segment SGI217979[] = {{&S80860[44], 14}, {NULL}};
const Segment SGI217980[] = {{&S80860[44], 36}, {NULL}};
const Segment SGI217981[] = {{&S80860[30], 6}, {NULL}};
const Segment SGI217982[] = {{&S91366[44], 10}, {NULL}};
const Segment SGI217983[] = {{&S80860[129], 3}, {NULL}};
const Segment SGI217984[] = {{&S91366[44], 22}, {NULL}};
const Segment SGI217985[] = {{&S91366[30], 6}, {NULL}};
const Segment SGI217986[] = {{&S80860[94], 6}, {NULL}};
const Segment SGI217987[] = {{&S80860[154], 7}, {NULL}};
const Segment SGI217988[] = {{&S91366[31], 3}, {NULL}};
const Segment SGI217989[] = {{&S91366[80], 6}, {NULL}};
const Segment SGI217990[] = {{&S91367[38], 10}, {NULL}};
const Segment SGI217991[] = {{&S91367[31], 6}, {NULL}};
const Segment SGI217992[] = {{&S80862[44], 10}, {NULL}};
const Segment SGI217993[] = {{&S80862[44], 22}, {NULL}};
const Segment SGI217994[] = {{&S80862[30], 6}, {NULL}};
const Segment SGI217995[] = {{&S80862[149], 3}, {NULL}};
const Segment SGI217996[] = {{&S91368[44], 22}, {NULL}};
const Segment SGI217997[] = {{&S80862[114], 6}, {NULL}};
const Segment SGI217998[] = {{&S91368[30], 6}, {NULL}};
const Segment SGI217999[] = {{&S80862[68], 8}, {NULL}};
const Segment SGI218000[] = {{&S72400[34], 10}, {NULL}};
const Segment SGI218001[] = {{&S72400[27], 6}, {NULL}};
const Segment SGI218002[] = {{&S91369[44], 10}, {NULL}};
const Segment SGI218003[] = {{&S91369[44], 22}, {NULL}};
const Segment SGI218004[] = {{&S91369[30], 6}, {NULL}};
const Segment SGI218005[] = {{&S72400[96], 9}, {NULL}};
const Segment SGI218006[] = {{&S72400[69], 6}, {NULL}};
const Segment SGI218007[] = {{&S91369[124], 3}, {NULL}};
const Segment SGI218008[] = {{&S80864[34], 7}, {NULL}};
const Segment SGI218009[] = {{&S91369[89], 6}, {NULL}};
const Segment SGI218010[] = {{&S80864[27], 6}, {NULL}};
const Segment SGI218011[] = {{&S91369[144], 8}, {NULL}};
const Segment SGI218012[] = {{&S80864[100], 9}, {NULL}};
const Segment SGI218013[] = {{&S80864[73], 6}, {NULL}};
const Segment SGI218014[] = {{&S91375[34], 13}, {NULL}};
const Segment SGI218015[] = {{&S91375[27], 6}, {NULL}};
const Segment SGI218016[] = {{&S91375[100], 9}, {NULL}};
const Segment SGI218017[] = {{&S91375[73], 6}, {NULL}};
const Segment SGI218018[] = {{&S91376[34], 9}, {NULL}};
const Segment SGI218019[] = {{&S91376[27], 6}, {NULL}};
const Segment SGI218020[] = {{&S91376[96], 9}, {NULL}};
const Segment SGI218021[] = {{&S91376[69], 6}, {NULL}};
const Segment SGI218022[] = {{&S91377[44], 10}, {NULL}};
const Segment SGI218023[] = {{&S91377[44], 22}, {NULL}};
const Segment SGI218024[] = {{&S91377[30], 6}, {NULL}};
const Segment SGI218025[] = {{&S91377[31], 3}, {NULL}};
const Segment SGI218026[] = {{&S91377[80], 6}, {NULL}};
const Segment SGI218027[] = {{&S91385[44], 10}, {NULL}};
const Segment SGI218028[] = {{&S91385[44], 22}, {NULL}};
const Segment SGI218029[] = {{&S91385[30], 6}, {NULL}};
const Segment SGI218030[] = {{&S91385[31], 3}, {NULL}};
const Segment SGI218031[] = {{&S91385[80], 5}, {NULL}};
const Segment SGI218032[] = {{&S91385[117], 14}, {NULL}};
const Segment SGI218033[] = {{&S91388[44], 10}, {NULL}};
const Segment SGI218034[] = {{&S91388[44], 22}, {NULL}};
const Segment SGI218035[] = {{&S91388[30], 6}, {NULL}};
const Segment SGI218036[] = {{&S91388[80], 6}, {NULL}};
const Segment SGI218037[] = {{&S91388[107], 13}, {NULL}};
const Segment SGI218038[] = {{&S91390[34], 17}, {NULL}};
const Segment SGI218039[] = {{&S91390[27], 6}, {NULL}};
const Segment SGI218040[] = {{&S91390[104], 9}, {NULL}};
const Segment SGI218041[] = {{&S91390[77], 6}, {NULL}};
const Segment SGI218042[] = {{&S91392[34], 8}, {NULL}};
const Segment SGI218043[] = {{&S91392[27], 6}, {NULL}};
const Segment SGI218044[] = {{&S91392[100], 9}, {NULL}};
const Segment SGI218045[] = {{&S91392[73], 6}, {NULL}};
const Segment SGI218046[] = {{&S72434[44], 7}, {NULL}};
const Segment SGI218047[] = {{&S72434[44], 23}, {NULL}};
const Segment SGI218048[] = {{&S72434[30], 6}, {NULL}};
const Segment SGI218049[] = {{&S72434[116], 3}, {NULL}};
const Segment SGI218050[] = {{&S72434[81], 6}, {NULL}};
const Segment SGI218051[] = {{&S72435[44], 7}, {NULL}};
const Segment SGI218052[] = {{&S72435[44], 23}, {NULL}};
const Segment SGI218053[] = {{&S72435[30], 6}, {NULL}};
const Segment SGI218054[] = {{&S72435[116], 3}, {NULL}};
const Segment SGI218055[] = {{&S72435[81], 6}, {NULL}};
const Segment SGI218056[] = {{&S72436[38], 7}, {NULL}};
const Segment SGI218057[] = {{&S72436[31], 6}, {NULL}};
const Segment SGI218058[] = {{&S91398[34], 6}, {NULL}};
const Segment SGI218059[] = {{&S91398[27], 6}, {NULL}};
const Segment SGI218060[] = {{&S91398[93], 9}, {NULL}};
const Segment SGI218061[] = {{&S91398[66], 6}, {NULL}};
const Segment SGI218062[] = {{&S91399[34], 6}, {NULL}};
const Segment SGI218063[] = {{&S91399[27], 6}, {NULL}};
const Segment SGI218064[] = {{&S91399[93], 9}, {NULL}};
const Segment SGI218065[] = {{&S91399[66], 6}, {NULL}};
const Segment SGI218066[] = {{&S72438[34], 12}, {NULL}};
const Segment SGI218067[] = {{&S72438[27], 6}, {NULL}};
const Segment SGI218068[] = {{&S72438[99], 9}, {NULL}};
const Segment SGI218069[] = {{&S72438[72], 6}, {NULL}};
const Segment SGI218070[] = {{&S81648[43], 10}, {NULL}};
const Segment SGI218071[] = {{&S81648[43], 22}, {NULL}};
const Segment SGI218072[] = {{&S81648[30], 6}, {NULL}};
const Segment SGI218073[] = {{&S81648[114], 3}, {NULL}};
const Segment SGI218074[] = {{&S81648[79], 6}, {NULL}};
const Segment SGI218075[] = {{&S81649[44], 10}, {NULL}};
const Segment SGI218076[] = {{&S81649[44], 22}, {NULL}};
const Segment SGI218077[] = {{&S81649[30], 6}, {NULL}};
const Segment SGI218078[] = {{&S81649[115], 3}, {NULL}};
const Segment SGI218079[] = {{&S81649[80], 6}, {NULL}};
const Segment SGI218080[] = {{&S91410[44], 21}, {NULL}};
const Segment SGI218081[] = {{&S91410[44], 33}, {NULL}};
const Segment SGI218082[] = {{&S91410[30], 6}, {NULL}};
const Segment SGI218083[] = {{&S91410[31], 3}, {NULL}};
const Segment SGI218084[] = {{&S91410[91], 6}, {NULL}};
const Segment SGI218085[] = {{&S80919[34], 7}, {NULL}};
const Segment SGI218086[] = {{&S80919[27], 6}, {NULL}};
const Segment SGI218087[] = {{&S80919[99], 9}, {NULL}};
const Segment SGI218088[] = {{&S80919[72], 6}, {NULL}};
const Segment SGI218089[] = {{&S80919[140], 8}, {NULL}};
const Segment SGI218090[] = {{&S91412[44], 13}, {NULL}};
const Segment SGI218091[] = {{&S91412[44], 25}, {NULL}};
const Segment SGI218092[] = {{&S91412[30], 6}, {NULL}};
const Segment SGI218093[] = {{&S91412[83], 6}, {NULL}};
const Segment SGI218094[] = {{&S91412[110], 13}, {NULL}};
const Segment SGI218095[] = {{&S91414[44], 10}, {NULL}};
const Segment SGI218096[] = {{&S91414[44], 22}, {NULL}};
const Segment SGI218097[] = {{&S91414[30], 6}, {NULL}};
const Segment SGI218098[] = {{&S91414[80], 6}, {NULL}};
const Segment SGI218099[] = {{&S91414[107], 13}, {NULL}};
const Segment SGI218100[] = {{&S91422[45], 5}, {NULL}};
const Segment SGI218101[] = {{&S91422[30], 6}, {NULL}};
const Segment SGI218102[] = {{&S91422[31], 3}, {NULL}};
const Segment SGI218103[] = {{&S91422[76], 5}, {NULL}};
const Segment SGI218104[] = {{&S91422[134], 13}, {NULL}};
const Segment SGI218105[] = {{&S72467[44], 7}, {NULL}};
const Segment SGI218106[] = {{&S72467[44], 19}, {NULL}};
const Segment SGI218107[] = {{&S72467[30], 6}, {NULL}};
const Segment SGI218108[] = {{&S72467[130], 3}, {NULL}};
const Segment SGI218109[] = {{&S72467[95], 6}, {NULL}};
const Segment SGI218110[] = {{&S72467[65], 8}, {NULL}};
const Segment SGI218111[] = {{&S72470[34], 10}, {NULL}};
const Segment SGI218112[] = {{&S72470[27], 6}, {NULL}};
const Segment SGI218113[] = {{&S72470[97], 9}, {NULL}};
const Segment SGI218114[] = {{&S72470[70], 6}, {NULL}};
const Segment SGI218115[] = {{&S91429[34], 11}, {NULL}};
const Segment SGI218116[] = {{&S91429[27], 6}, {NULL}};
const Segment SGI218117[] = {{&S91429[98], 9}, {NULL}};
const Segment SGI218118[] = {{&S91429[71], 6}, {NULL}};
const Segment SGI218119[] = {{&S91429[139], 8}, {NULL}};
const Segment SGI218120[] = {{&S91430[34], 7}, {NULL}};
const Segment SGI218121[] = {{&S91430[27], 6}, {NULL}};
const Segment SGI218122[] = {{&S91430[99], 9}, {NULL}};
const Segment SGI218123[] = {{&S91430[72], 6}, {NULL}};
const Segment SGI218124[] = {{&S91430[140], 8}, {NULL}};
const Segment SGI218125[] = {{&S91431[34], 10}, {NULL}};
const Segment SGI218126[] = {{&S91431[27], 6}, {NULL}};
const Segment SGI218127[] = {{&S91431[97], 9}, {NULL}};
const Segment SGI218128[] = {{&S91431[70], 6}, {NULL}};
const Segment SGI218129[] = {{&S80951[34], 10}, {NULL}};
const Segment SGI218130[] = {{&S80951[27], 6}, {NULL}};
const Segment SGI218131[] = {{&S80951[97], 9}, {NULL}};
const Segment SGI218132[] = {{&S80951[70], 6}, {NULL}};
const Segment SGI218133[] = {{&S80953[34], 9}, {NULL}};
const Segment SGI218134[] = {{&S80953[27], 6}, {NULL}};
const Segment SGI218135[] = {{&S80953[96], 9}, {NULL}};
const Segment SGI218136[] = {{&S80953[69], 6}, {NULL}};
const Segment SGI218137[] = {{&S91441[34], 12}, {NULL}};
const Segment SGI218138[] = {{&S91441[27], 6}, {NULL}};
const Segment SGI218139[] = {{&S91441[112], 9}, {NULL}};
const Segment SGI218140[] = {{&S91441[85], 6}, {NULL}};
const Segment SGI218141[] = {{&S72490[44], 9}, {NULL}};
const Segment SGI218142[] = {{&S72490[44], 20}, {NULL}};
const Segment SGI218143[] = {{&S72490[30], 6}, {NULL}};
const Segment SGI218144[] = {{&S72490[136], 3}, {NULL}};
const Segment SGI218145[] = {{&S72490[101], 6}, {NULL}};
const Segment SGI218146[] = {{&S72491[44], 9}, {NULL}};
const Segment SGI218147[] = {{&S72491[44], 20}, {NULL}};
const Segment SGI218148[] = {{&S72491[30], 6}, {NULL}};
const Segment SGI218149[] = {{&S72491[136], 3}, {NULL}};
const Segment SGI218150[] = {{&S72491[101], 6}, {NULL}};
const Segment SGI218151[] = {{&S72491[73], 9}, {NULL}};
const Segment SGI218152[] = {{&S91446[34], 11}, {NULL}};
const Segment SGI218153[] = {{&S91446[27], 6}, {NULL}};
const Segment SGI218154[] = {{&S91446[98], 9}, {NULL}};
const Segment SGI218155[] = {{&S91446[71], 6}, {NULL}};
const Segment SGI218156[] = {{&S72492[44], 9}, {NULL}};
const Segment SGI218157[] = {{&S72492[44], 21}, {NULL}};
const Segment SGI218158[] = {{&S72492[30], 6}, {NULL}};
const Segment SGI218159[] = {{&S72492[137], 3}, {NULL}};
const Segment SGI218160[] = {{&S72492[102], 6}, {NULL}};
const Segment SGI218161[] = {{&S72492[74], 9}, {NULL}};
const Segment SGI218162[] = {{&S80966[34], 8}, {NULL}};
const Segment SGI218163[] = {{&S80966[27], 6}, {NULL}};
const Segment SGI218164[] = {{&S80966[105], 9}, {NULL}};
const Segment SGI218165[] = {{&S80966[78], 6}, {NULL}};
const Segment SGI218166[] = {{&S80966[146], 8}, {NULL}};
const Segment SGI218167[] = {{&S80967[44], 10}, {NULL}};
const Segment SGI218168[] = {{&S80967[44], 22}, {NULL}};
const Segment SGI218169[] = {{&S80967[30], 6}, {NULL}};
const Segment SGI218170[] = {{&S80967[80], 6}, {NULL}};
const Segment SGI218171[] = {{&S80967[107], 13}, {NULL}};
const Segment SGI218172[] = {{&S80970[44], 10}, {NULL}};
const Segment SGI218173[] = {{&S80970[44], 22}, {NULL}};
const Segment SGI218174[] = {{&S80970[30], 6}, {NULL}};
const Segment SGI218175[] = {{&S80970[115], 3}, {NULL}};
const Segment SGI218176[] = {{&S80970[80], 6}, {NULL}};
const Segment SGI218177[] = {{&S80970[147], 15}, {NULL}};
const Segment SGI218178[] = {{&S80971[44], 10}, {NULL}};
const Segment SGI218179[] = {{&S80971[44], 23}, {NULL}};
const Segment SGI218180[] = {{&S80971[30], 6}, {NULL}};
const Segment SGI218181[] = {{&S80971[116], 3}, {NULL}};
const Segment SGI218182[] = {{&S80971[81], 6}, {NULL}};
const Segment SGI218183[] = {{&S80971[148], 15}, {NULL}};
const Segment SGI218184[] = {{&S80973[34], 4}, {NULL}};
const Segment SGI218185[] = {{&S80973[27], 6}, {NULL}};
const Segment SGI218186[] = {{&S80973[91], 9}, {NULL}};
const Segment SGI218187[] = {{&S80973[64], 6}, {NULL}};
const Segment SGI218188[] = {{&S80976[34], 25}, {NULL}};
const Segment SGI218189[] = {{&S80976[27], 6}, {NULL}};
const Segment SGI218190[] = {{&S80976[112], 9}, {NULL}};
const Segment SGI218191[] = {{&S80976[85], 6}, {NULL}};
const Segment SGI218192[] = {{&S80980[34], 7}, {NULL}};
const Segment SGI218193[] = {{&S80980[27], 6}, {NULL}};
const Segment SGI218194[] = {{&S80980[94], 9}, {NULL}};
const Segment SGI218195[] = {{&S80980[67], 6}, {NULL}};
const Segment SGI218196[] = {{&S80982[34], 17}, {NULL}};
const Segment SGI218197[] = {{&S80982[27], 6}, {NULL}};
const Segment SGI218198[] = {{&S80982[104], 9}, {NULL}};
const Segment SGI218199[] = {{&S80982[77], 6}, {NULL}};
const Segment SGI218200[] = {{&S80985[34], 11}, {NULL}};
const Segment SGI218201[] = {{&S80985[27], 6}, {NULL}};
const Segment SGI218202[] = {{&S72526[34], 10}, {NULL}};
const Segment SGI218203[] = {{&S72526[27], 6}, {NULL}};
const Segment SGI218204[] = {{&S80985[98], 9}, {NULL}};
const Segment SGI218205[] = {{&S80985[71], 6}, {NULL}};
const Segment SGI218206[] = {{&S72526[96], 9}, {NULL}};
const Segment SGI218207[] = {{&S72526[69], 6}, {NULL}};
const Segment SGI218208[] = {{&S72527[34], 11}, {NULL}};
const Segment SGI218209[] = {{&S72527[27], 6}, {NULL}};
const Segment SGI218210[] = {{&S72527[97], 9}, {NULL}};
const Segment SGI218211[] = {{&S72527[70], 6}, {NULL}};
const Segment SGI218212[] = {{&S91511[44], 9}, {NULL}};
const Segment SGI218213[] = {{&S91511[44], 19}, {NULL}};
const Segment SGI218214[] = {{&S72549[44], 10}, {NULL}};
const Segment SGI218215[] = {{&S91511[30], 6}, {NULL}};
const Segment SGI218216[] = {{&S72549[44], 21}, {NULL}};
const Segment SGI218217[] = {{&S72549[30], 6}, {NULL}};
const Segment SGI218218[] = {{&S91511[31], 3}, {NULL}};
const Segment SGI218219[] = {{&S91511[77], 6}, {NULL}};
const Segment SGI218220[] = {{&S72549[114], 3}, {NULL}};
const Segment SGI218221[] = {{&S72549[79], 6}, {NULL}};
const Segment SGI218222[] = {{&S72549[149], 3}, {NULL}};
const Segment SGI218223[] = {{&S81766[44], 14}, {NULL}};
const Segment SGI218224[] = {{&S81766[44], 26}, {NULL}};
const Segment SGI218225[] = {{&S81766[30], 6}, {NULL}};
const Segment SGI218226[] = {{&S81766[119], 3}, {NULL}};
const Segment SGI218227[] = {{&S81766[84], 6}, {NULL}};
const Segment SGI218228[] = {{&S81766[143], 10}, {NULL}};
const Segment SGI218229[] = {{&S86635[122], 3}, {NULL}};
const Segment SGI218230[] = {{&S86635[87], 6}, {NULL}};
const Segment SGI218231[] = {{&S86636[44], 17}, {NULL}};
const Segment SGI218232[] = {{&S86636[44], 54}, {NULL}};
const Segment SGI218233[] = {{&S86636[30], 6}, {NULL}};
const Segment SGI218234[] = {{&S86636[147], 3}, {NULL}};
const Segment SGI218235[] = {{&S86636[112], 6}, {NULL}};
const Segment SGI218236[] = {{&S91524[44], 7}, {NULL}};
const Segment SGI218237[] = {{&S91524[44], 15}, {NULL}};
const Segment SGI218238[] = {{&S91524[30], 6}, {NULL}};
const Segment SGI218239[] = {{&S91524[31], 3}, {NULL}};
const Segment SGI218240[] = {{&S91524[73], 6}, {NULL}};
const Segment SGI218241[] = {{&S86637[34], 7}, {NULL}};
const Segment SGI218242[] = {{&S86637[27], 6}, {NULL}};
const Segment SGI218243[] = {{&S86637[94], 9}, {NULL}};
const Segment SGI218244[] = {{&S86637[67], 6}, {NULL}};
const Segment SGI218245[] = {{&S91530[34], 8}, {NULL}};
const Segment SGI218246[] = {{&S91530[27], 6}, {NULL}};
const Segment SGI218247[] = {{&S81030[34], 5}, {NULL}};
const Segment SGI218248[] = {{&S91530[94], 9}, {NULL}};
const Segment SGI218249[] = {{&S91530[67], 6}, {NULL}};
const Segment SGI218250[] = {{&S81030[27], 6}, {NULL}};
const Segment SGI218251[] = {{&S81030[92], 9}, {NULL}};
const Segment SGI218252[] = {{&S81030[65], 6}, {NULL}};
const Segment SGI218253[] = {{&S91534[34], 12}, {NULL}};
const Segment SGI218254[] = {{&S91534[27], 6}, {NULL}};
const Segment SGI218255[] = {{&S91534[99], 9}, {NULL}};
const Segment SGI218256[] = {{&S91534[72], 6}, {NULL}};
const Segment SGI218257[] = {{&S91538[34], 6}, {NULL}};
const Segment SGI218258[] = {{&S91538[27], 6}, {NULL}};
const Segment SGI218259[] = {{&S91538[93], 9}, {NULL}};
const Segment SGI218260[] = {{&S91538[66], 6}, {NULL}};
const Segment SGI218261[] = {{&S81041[34], 10}, {NULL}};
const Segment SGI218262[] = {{&S81041[27], 6}, {NULL}};
const Segment SGI218263[] = {{&S81041[97], 9}, {NULL}};
const Segment SGI218264[] = {{&S81041[70], 6}, {NULL}};
const Segment SGI218265[] = {{&S81042[34], 10}, {NULL}};
const Segment SGI218266[] = {{&S81042[27], 6}, {NULL}};
const Segment SGI218267[] = {{&S81042[97], 9}, {NULL}};
const Segment SGI218268[] = {{&S81042[70], 6}, {NULL}};
const Segment SGI218269[] = {{&S81048[34], 7}, {NULL}};
const Segment SGI218270[] = {{&S81048[27], 6}, {NULL}};
const Segment SGI218271[] = {{&S81048[94], 9}, {NULL}};
const Segment SGI218272[] = {{&S81048[67], 6}, {NULL}};
const Segment SGI218273[] = {{&S91550[44], 17}, {NULL}};
const Segment SGI218274[] = {{&S91550[44], 29}, {NULL}};
const Segment SGI218275[] = {{&S91550[30], 6}, {NULL}};
const Segment SGI218276[] = {{&S81049[34], 10}, {NULL}};
const Segment SGI218277[] = {{&S81049[27], 6}, {NULL}};
const Segment SGI218278[] = {{&S91550[31], 3}, {NULL}};
const Segment SGI218279[] = {{&S91550[106], 6}, {NULL}};
const Segment SGI218280[] = {{&S81049[97], 9}, {NULL}};
const Segment SGI218281[] = {{&S81049[70], 6}, {NULL}};
const Segment SGI218282[] = {{&S91550[75], 8}, {NULL}};
const Segment SGI218283[] = {{&S91557[34], 8}, {NULL}};
const Segment SGI218284[] = {{&S91557[27], 6}, {NULL}};
const Segment SGI218285[] = {{&S91557[95], 9}, {NULL}};
const Segment SGI218286[] = {{&S91557[68], 6}, {NULL}};
const Segment SGI218287[] = {{&S91558[34], 7}, {NULL}};
const Segment SGI218288[] = {{&S91558[27], 6}, {NULL}};
const Segment SGI218289[] = {{&S91558[98], 9}, {NULL}};
const Segment SGI218290[] = {{&S91558[71], 6}, {NULL}};
const Segment SGI218291[] = {{&S91561[59], 15}, {NULL}};
const Segment SGI218292[] = {{&S91561[59], 27}, {NULL}};
const Segment SGI218293[] = {{&S91561[45], 6}, {NULL}};
const Segment SGI218294[] = {{&S91561[46], 3}, {NULL}};
const Segment SGI218295[] = {{&S91561[100], 6}, {NULL}};
const Segment SGI218296[] = {{&S91561[0], 14}, {NULL}};
const Segment SGI218297[] = {{&S91563[34], 12}, {NULL}};
const Segment SGI218298[] = {{&S91563[27], 6}, {NULL}};
const Segment SGI218299[] = {{&S91563[99], 9}, {NULL}};
const Segment SGI218300[] = {{&S91563[72], 6}, {NULL}};
const Segment SGI218301[] = {{&S86694[59], 10}, {NULL}};
const Segment SGI218302[] = {{&S86694[59], 22}, {NULL}};
const Segment SGI218303[] = {{&S86694[45], 6}, {NULL}};
const Segment SGI218304[] = {{&S86694[130], 3}, {NULL}};
const Segment SGI218305[] = {{&S86694[95], 6}, {NULL}};
const Segment SGI218306[] = {{&S86694[0], 14}, {NULL}};
const Segment SGI218307[] = {{&S91567[34], 7}, {NULL}};
const Segment SGI218308[] = {{&S91567[27], 6}, {NULL}};
const Segment SGI218309[] = {{&S91567[99], 9}, {NULL}};
const Segment SGI218310[] = {{&S91567[72], 6}, {NULL}};
const Segment SGI218311[] = {{&S86696[34], 7}, {NULL}};
const Segment SGI218312[] = {{&S86696[27], 6}, {NULL}};
const Segment SGI218313[] = {{&S86696[94], 9}, {NULL}};
const Segment SGI218314[] = {{&S86696[67], 6}, {NULL}};
const Segment SGI218315[] = {{&S86697[34], 8}, {NULL}};
const Segment SGI218316[] = {{&S86697[27], 6}, {NULL}};
const Segment SGI218317[] = {{&S86697[95], 9}, {NULL}};
const Segment SGI218318[] = {{&S86697[68], 6}, {NULL}};
const Segment SGI218319[] = {{&S91572[34], 5}, {NULL}};
const Segment SGI218320[] = {{&S91572[27], 6}, {NULL}};
const Segment SGI218321[] = {{&S91572[92], 9}, {NULL}};
const Segment SGI218322[] = {{&S91572[65], 6}, {NULL}};
const Segment SGI218323[] = {{&S91574[34], 10}, {NULL}};
const Segment SGI218324[] = {{&S91574[27], 6}, {NULL}};
const Segment SGI218325[] = {{&S91574[97], 9}, {NULL}};
const Segment SGI218326[] = {{&S91574[70], 6}, {NULL}};
const Segment SGI218327[] = {{&S91576[34], 9}, {NULL}};
const Segment SGI218328[] = {{&S91576[27], 6}, {NULL}};
const Segment SGI218329[] = {{&S91576[96], 9}, {NULL}};
const Segment SGI218330[] = {{&S91576[69], 6}, {NULL}};
const Segment SGI218331[] = {{&S91577[34], 8}, {NULL}};
const Segment SGI218332[] = {{&S91577[27], 6}, {NULL}};
const Segment SGI218333[] = {{&S91577[95], 9}, {NULL}};
const Segment SGI218334[] = {{&S91577[68], 6}, {NULL}};
const Segment SGI218335[] = {{&S81087[44], 12}, {NULL}};
const Segment SGI218336[] = {{&S81087[44], 24}, {NULL}};
const Segment SGI218337[] = {{&S81087[30], 6}, {NULL}};
const Segment SGI218338[] = {{&S81087[133], 3}, {NULL}};
const Segment SGI218339[] = {{&S81087[98], 6}, {NULL}};
const Segment SGI218340[] = {{&S91578[34], 14}, {NULL}};
const Segment SGI218341[] = {{&S81087[70], 8}, {NULL}};
const Segment SGI218342[] = {{&S91578[27], 6}, {NULL}};
const Segment SGI218343[] = {{&S91578[101], 9}, {NULL}};
const Segment SGI218344[] = {{&S91578[74], 6}, {NULL}};
const Segment SGI218345[] = {{&S91579[34], 10}, {NULL}};
const Segment SGI218346[] = {{&S91579[27], 6}, {NULL}};
const Segment SGI218347[] = {{&S91579[97], 9}, {NULL}};
const Segment SGI218348[] = {{&S91579[70], 6}, {NULL}};
const Segment SGI218349[] = {{&S91582[34], 10}, {NULL}};
const Segment SGI218350[] = {{&S91582[27], 6}, {NULL}};
const Segment SGI218351[] = {{&S91582[97], 9}, {NULL}};
const Segment SGI218352[] = {{&S91582[70], 6}, {NULL}};
const Segment SGI218353[] = {{&S81093[44], 11}, {NULL}};
const Segment SGI218354[] = {{&S81093[44], 23}, {NULL}};
const Segment SGI218355[] = {{&S81093[30], 6}, {NULL}};
const Segment SGI218356[] = {{&S81093[116], 3}, {NULL}};
const Segment SGI218357[] = {{&S81093[81], 6}, {NULL}};
const Segment SGI218358[] = {{&S91583[34], 10}, {NULL}};
const Segment SGI218359[] = {{&S91583[27], 6}, {NULL}};
const Segment SGI218360[] = {{&S91583[97], 9}, {NULL}};
const Segment SGI218361[] = {{&S91583[70], 6}, {NULL}};
const Segment SGI218362[] = {{&S81094[44], 22}, {NULL}};
const Segment SGI218363[] = {{&S81094[44], 42}, {NULL}};
const Segment SGI218364[] = {{&S81094[30], 6}, {NULL}};
const Segment SGI218365[] = {{&S81094[155], 3}, {NULL}};
const Segment SGI218366[] = {{&S81094[120], 6}, {NULL}};
const Segment SGI218367[] = {{&S81094[88], 8}, {NULL}};
const Segment SGI218368[] = {{&S91585[44], 8}, {NULL}};
const Segment SGI218369[] = {{&S91585[44], 20}, {NULL}};
const Segment SGI218370[] = {{&S91585[30], 6}, {NULL}};
const Segment SGI218371[] = {{&S86722[34], 25}, {NULL}};
const Segment SGI218372[] = {{&S91585[31], 3}, {NULL}};
const Segment SGI218373[] = {{&S86722[27], 6}, {NULL}};
const Segment SGI218374[] = {{&S91585[78], 6}, {NULL}};
const Segment SGI218375[] = {{&S81095[44], 18}, {NULL}};
const Segment SGI218376[] = {{&S81095[44], 28}, {NULL}};
const Segment SGI218377[] = {{&S81095[30], 6}, {NULL}};
const Segment SGI218378[] = {{&S86722[112], 9}, {NULL}};
const Segment SGI218379[] = {{&S86722[85], 6}, {NULL}};
const Segment SGI218380[] = {{&S81095[141], 3}, {NULL}};
const Segment SGI218381[] = {{&S81095[106], 6}, {NULL}};
const Segment SGI218382[] = {{&S81095[74], 8}, {NULL}};
const Segment SGI218383[] = {{&S81096[44], 9}, {NULL}};
const Segment SGI218384[] = {{&S81096[44], 19}, {NULL}};
const Segment SGI218385[] = {{&S81096[30], 6}, {NULL}};
const Segment SGI218386[] = {{&S91587[44], 14}, {NULL}};
const Segment SGI218387[] = {{&S91587[44], 26}, {NULL}};
const Segment SGI218388[] = {{&S81096[128], 3}, {NULL}};
const Segment SGI218389[] = {{&S91587[30], 6}, {NULL}};
const Segment SGI218390[] = {{&S81096[93], 6}, {NULL}};
const Segment SGI218391[] = {{&S81096[65], 8}, {NULL}};
const Segment SGI218392[] = {{&S91587[84], 6}, {NULL}};
const Segment SGI218393[] = {{&S91587[111], 13}, {NULL}};
const Segment SGI218394[] = {{&S86726[44], 12}, {NULL}};
const Segment SGI218395[] = {{&S86726[44], 24}, {NULL}};
const Segment SGI218396[] = {{&S86726[30], 6}, {NULL}};
const Segment SGI218397[] = {{&S86726[117], 3}, {NULL}};
const Segment SGI218398[] = {{&S86726[82], 6}, {NULL}};
const Segment SGI218399[] = {{&S86726[141], 10}, {NULL}};
const Segment SGI218400[] = {{&S86731[44], 23}, {NULL}};
const Segment SGI218401[] = {{&S86731[44], 35}, {NULL}};
const Segment SGI218402[] = {{&S86731[30], 6}, {NULL}};
const Segment SGI218403[] = {{&S86731[128], 3}, {NULL}};
const Segment SGI218404[] = {{&S86731[93], 6}, {NULL}};
const Segment SGI218405[] = {{&S91595[34], 7}, {NULL}};
const Segment SGI218406[] = {{&S91595[27], 6}, {NULL}};
const Segment SGI218407[] = {{&S91595[99], 9}, {NULL}};
const Segment SGI218408[] = {{&S91595[72], 6}, {NULL}};
const Segment SGI218409[] = {{&S91595[140], 8}, {NULL}};
const Segment SGI218410[] = {{&S91596[34], 6}, {NULL}};
const Segment SGI218411[] = {{&S91596[27], 6}, {NULL}};
const Segment SGI218412[] = {{&S91596[93], 9}, {NULL}};
const Segment SGI218413[] = {{&S91596[66], 6}, {NULL}};
const Segment SGI218414[] = {{&S91596[134], 8}, {NULL}};
const Segment SGI218415[] = {{&S72666[44], 9}, {NULL}};
const Segment SGI218416[] = {{&S72666[44], 20}, {NULL}};
const Segment SGI218417[] = {{&S72666[30], 6}, {NULL}};
const Segment SGI218418[] = {{&S72666[131], 3}, {NULL}};
const Segment SGI218419[] = {{&S81109[52], 10}, {NULL}};
const Segment SGI218420[] = {{&S81109[52], 22}, {NULL}};
const Segment SGI218421[] = {{&S72666[96], 6}, {NULL}};
const Segment SGI218422[] = {{&S81109[38], 6}, {NULL}};
const Segment SGI218423[] = {{&S72666[66], 8}, {NULL}};
const Segment SGI218424[] = {{&S81109[88], 5}, {NULL}};
const Segment SGI218425[] = {{&S81109[11], 3}, {NULL}};
const Segment SGI218426[] = {{&S72667[44], 9}, {NULL}};
const Segment SGI218427[] = {{&S86737[34], 25}, {NULL}};
const Segment SGI218428[] = {{&S72667[44], 21}, {NULL}};
const Segment SGI218429[] = {{&S72667[30], 6}, {NULL}};
const Segment SGI218430[] = {{&S86737[27], 6}, {NULL}};
const Segment SGI218431[] = {{&S72667[132], 3}, {NULL}};
const Segment SGI218432[] = {{&S86737[112], 9}, {NULL}};
const Segment SGI218433[] = {{&S72667[97], 6}, {NULL}};
const Segment SGI218434[] = {{&S86737[85], 6}, {NULL}};
const Segment SGI218435[] = {{&S72667[67], 8}, {NULL}};
const Segment SGI218436[] = {{&S91600[44], 9}, {NULL}};
const Segment SGI218437[] = {{&S91600[44], 21}, {NULL}};
const Segment SGI218438[] = {{&S91600[30], 6}, {NULL}};
const Segment SGI218439[] = {{&S91600[79], 6}, {NULL}};
const Segment SGI218440[] = {{&S81112[34], 20}, {NULL}};
const Segment SGI218441[] = {{&S91600[106], 13}, {NULL}};
const Segment SGI218442[] = {{&S81112[27], 6}, {NULL}};
const Segment SGI218443[] = {{&S81112[107], 9}, {NULL}};
const Segment SGI218444[] = {{&S81112[80], 6}, {NULL}};
const Segment SGI218445[] = {{&S81113[44], 6}, {NULL}};
const Segment SGI218446[] = {{&S81113[44], 26}, {NULL}};
const Segment SGI218447[] = {{&S81113[30], 6}, {NULL}};
const Segment SGI218448[] = {{&S81113[84], 6}, {NULL}};
const Segment SGI218449[] = {{&S81113[111], 13}, {NULL}};
const Segment SGI218450[] = {{&S81114[44], 10}, {NULL}};
const Segment SGI218451[] = {{&S81114[44], 22}, {NULL}};
const Segment SGI218452[] = {{&S81114[30], 6}, {NULL}};
const Segment SGI218453[] = {{&S81114[80], 6}, {NULL}};
const Segment SGI218454[] = {{&S81114[107], 13}, {NULL}};
const Segment SGI218455[] = {{&S81115[44], 16}, {NULL}};
const Segment SGI218456[] = {{&S81115[44], 35}, {NULL}};
const Segment SGI218457[] = {{&S81115[30], 6}, {NULL}};
const Segment SGI218458[] = {{&S81115[93], 6}, {NULL}};
const Segment SGI218459[] = {{&S81115[120], 13}, {NULL}};
const Segment SGI218460[] = {{&S91608[44], 18}, {NULL}};
const Segment SGI218461[] = {{&S91608[44], 30}, {NULL}};
const Segment SGI218462[] = {{&S91608[30], 6}, {NULL}};
const Segment SGI218463[] = {{&S81116[34], 11}, {NULL}};
const Segment SGI218464[] = {{&S81116[27], 6}, {NULL}};
const Segment SGI218465[] = {{&S91608[31], 3}, {NULL}};
const Segment SGI218466[] = {{&S91608[88], 6}, {NULL}};
const Segment SGI218467[] = {{&S86749[44], 17}, {NULL}};
const Segment SGI218468[] = {{&S81116[98], 9}, {NULL}};
const Segment SGI218469[] = {{&S86749[44], 38}, {NULL}};
const Segment SGI218470[] = {{&S81116[71], 6}, {NULL}};
const Segment SGI218471[] = {{&S86749[30], 6}, {NULL}};
const Segment SGI218472[] = {{&S91608[148], 7}, {NULL}};
const Segment SGI218473[] = {{&S86749[131], 3}, {NULL}};
const Segment SGI218474[] = {{&S86749[96], 6}, {NULL}};
const Segment SGI218475[] = {{&S86751[34], 14}, {NULL}};
const Segment SGI218476[] = {{&S86751[27], 6}, {NULL}};
const Segment SGI218477[] = {{&S86751[101], 9}, {NULL}};
const Segment SGI218478[] = {{&S86751[74], 6}, {NULL}};
const Segment SGI218479[] = {{&S91614[44], 7}, {NULL}};
const Segment SGI218480[] = {{&S91614[44], 24}, {NULL}};
const Segment SGI218481[] = {{&S91614[30], 6}, {NULL}};
const Segment SGI218482[] = {{&S91614[82], 6}, {NULL}};
const Segment SGI218483[] = {{&S91614[109], 13}, {NULL}};
const Segment SGI218484[] = {{&S86763[44], 14}, {NULL}};
const Segment SGI218485[] = {{&S86763[44], 31}, {NULL}};
const Segment SGI218486[] = {{&S86763[30], 6}, {NULL}};
const Segment SGI218487[] = {{&S86763[122], 3}, {NULL}};
const Segment SGI218488[] = {{&S86763[89], 5}, {NULL}};
const Segment SGI218489[] = {{&S86763[126], 14}, {NULL}};
const Segment SGI218490[] = {{&S91626[44], 10}, {NULL}};
const Segment SGI218491[] = {{&S91626[44], 22}, {NULL}};
const Segment SGI218492[] = {{&S91626[30], 6}, {NULL}};
const Segment SGI218493[] = {{&S91626[31], 3}, {NULL}};
const Segment SGI218494[] = {{&S91626[80], 5}, {NULL}};
const Segment SGI218495[] = {{&S91626[117], 14}, {NULL}};
const Segment SGI218496[] = {{&S91634[34], 15}, {NULL}};
const Segment SGI218497[] = {{&S91634[27], 6}, {NULL}};
const Segment SGI218498[] = {{&S91634[102], 9}, {NULL}};
const Segment SGI218499[] = {{&S91634[75], 6}, {NULL}};
const Segment SGI218500[] = {{&S91640[44], 9}, {NULL}};
const Segment SGI218501[] = {{&S91640[44], 21}, {NULL}};
const Segment SGI218502[] = {{&S91640[30], 6}, {NULL}};
const Segment SGI218503[] = {{&S91640[79], 6}, {NULL}};
const Segment SGI218504[] = {{&S91640[106], 13}, {NULL}};
const Segment SGI218505[] = {{&S91645[44], 5}, {NULL}};
const Segment SGI218506[] = {{&S91645[44], 17}, {NULL}};
const Segment SGI218507[] = {{&S91645[30], 6}, {NULL}};
const Segment SGI218508[] = {{&S91645[75], 6}, {NULL}};
const Segment SGI218509[] = {{&S91645[102], 13}, {NULL}};
const Segment SGI218510[] = {{&S72749[44], 11}, {NULL}};
const Segment SGI218511[] = {{&S72749[44], 22}, {NULL}};
const Segment SGI218512[] = {{&S72749[30], 6}, {NULL}};
const Segment SGI218513[] = {{&S72749[115], 3}, {NULL}};
const Segment SGI218514[] = {{&S72749[80], 6}, {NULL}};
const Segment SGI218515[] = {{&S72750[44], 7}, {NULL}};
const Segment SGI218516[] = {{&S72750[44], 18}, {NULL}};
const Segment SGI218517[] = {{&S72750[30], 6}, {NULL}};
const Segment SGI218518[] = {{&S72750[111], 3}, {NULL}};
const Segment SGI218519[] = {{&S72750[76], 6}, {NULL}};
const Segment SGI218520[] = {{&S72751[38], 7}, {NULL}};
const Segment SGI218521[] = {{&S72751[31], 6}, {NULL}};
const Segment SGI218522[] = {{&S72752[44], 11}, {NULL}};
const Segment SGI218523[] = {{&S72752[44], 23}, {NULL}};
const Segment SGI218524[] = {{&S72752[30], 6}, {NULL}};
const Segment SGI218525[] = {{&S72752[116], 3}, {NULL}};
const Segment SGI218526[] = {{&S72752[81], 6}, {NULL}};
const Segment SGI218527[] = {{&S72753[44], 11}, {NULL}};
const Segment SGI218528[] = {{&S72753[44], 22}, {NULL}};
const Segment SGI218529[] = {{&S72753[30], 6}, {NULL}};
const Segment SGI218530[] = {{&S72753[115], 3}, {NULL}};
const Segment SGI218531[] = {{&S72753[80], 6}, {NULL}};
const Segment SGI218532[] = {{&S72765[34], 6}, {NULL}};
const Segment SGI218533[] = {{&S72765[27], 6}, {NULL}};
const Segment SGI218534[] = {{&S72765[93], 9}, {NULL}};
const Segment SGI218535[] = {{&S72765[66], 6}, {NULL}};
const Segment SGI218536[] = {{&S86846[44], 18}, {NULL}};
const Segment SGI218537[] = {{&S86846[44], 30}, {NULL}};
const Segment SGI218538[] = {{&S86846[30], 6}, {NULL}};
const Segment SGI218539[] = {{&S86846[121], 3}, {NULL}};
const Segment SGI218540[] = {{&S86846[88], 5}, {NULL}};
const Segment SGI218541[] = {{&S86860[34], 15}, {NULL}};
const Segment SGI218542[] = {{&S86860[27], 6}, {NULL}};
const Segment SGI218543[] = {{&S86860[102], 9}, {NULL}};
const Segment SGI218544[] = {{&S86860[75], 6}, {NULL}};
const Segment SGI218545[] = {{&S91722[34], 6}, {NULL}};
const Segment SGI218546[] = {{&S91722[27], 6}, {NULL}};
const Segment SGI218547[] = {{&S91722[93], 9}, {NULL}};
const Segment SGI218548[] = {{&S91722[66], 6}, {NULL}};
const Segment SGI218549[] = {{&S86864[45], 4}, {NULL}};
const Segment SGI218550[] = {{&S86864[30], 6}, {NULL}};
const Segment SGI218551[] = {{&S86864[110], 3}, {NULL}};
const Segment SGI218552[] = {{&S86864[76], 5}, {NULL}};
const Segment SGI218553[] = {{&S86864[134], 13}, {NULL}};
const Segment SGI218554[] = {{&S91726[59], 10}, {NULL}};
const Segment SGI218555[] = {{&S91726[59], 22}, {NULL}};
const Segment SGI218556[] = {{&S91726[45], 6}, {NULL}};
const Segment SGI218557[] = {{&S91726[46], 3}, {NULL}};
const Segment SGI218558[] = {{&S91726[95], 6}, {NULL}};
const Segment SGI218559[] = {{&S91726[0], 14}, {NULL}};
const Segment SGI218560[] = {{&S91732[34], 8}, {NULL}};
const Segment SGI218561[] = {{&S91732[27], 6}, {NULL}};
const Segment SGI218562[] = {{&S91732[95], 9}, {NULL}};
const Segment SGI218563[] = {{&S91732[68], 6}, {NULL}};
const Segment SGI218564[] = {{&S86874[34], 12}, {NULL}};
const Segment SGI218565[] = {{&S86874[27], 6}, {NULL}};
const Segment SGI218566[] = {{&S86874[99], 9}, {NULL}};
const Segment SGI218567[] = {{&S86874[72], 6}, {NULL}};
const Segment SGI218568[] = {{&S91735[34], 13}, {NULL}};
const Segment SGI218569[] = {{&S91735[27], 6}, {NULL}};
const Segment SGI218570[] = {{&S91735[100], 9}, {NULL}};
const Segment SGI218571[] = {{&S91735[73], 6}, {NULL}};
const Segment SGI218572[] = {{&S91741[34], 14}, {NULL}};
const Segment SGI218573[] = {{&S91741[27], 6}, {NULL}};
const Segment SGI218574[] = {{&S91741[101], 9}, {NULL}};
const Segment SGI218575[] = {{&S91741[74], 6}, {NULL}};
const Segment SGI218576[] = {{&S91742[34], 9}, {NULL}};
const Segment SGI218577[] = {{&S91742[27], 6}, {NULL}};
const Segment SGI218578[] = {{&S91742[96], 9}, {NULL}};
const Segment SGI218579[] = {{&S91742[69], 6}, {NULL}};
const Segment SGI218580[] = {{&S91745[44], 11}, {NULL}};
const Segment SGI218581[] = {{&S91745[44], 23}, {NULL}};
const Segment SGI218582[] = {{&S91745[30], 6}, {NULL}};
const Segment SGI218583[] = {{&S91745[31], 3}, {NULL}};
const Segment SGI218584[] = {{&S91745[81], 6}, {NULL}};
const Segment SGI218585[] = {{&S91745[141], 11}, {NULL}};
const Segment SGI218586[] = {{&S91746[44], 10}, {NULL}};
const Segment SGI218587[] = {{&S91746[44], 22}, {NULL}};
const Segment SGI218588[] = {{&S91746[30], 6}, {NULL}};
const Segment SGI218589[] = {{&S91746[83], 6}, {NULL}};
const Segment SGI218590[] = {{&S91747[38], 10}, {NULL}};
const Segment SGI218591[] = {{&S91747[31], 6}, {NULL}};
const Segment SGI218592[] = {{&S72837[44], 10}, {NULL}};
const Segment SGI218593[] = {{&S72837[44], 21}, {NULL}};
const Segment SGI218594[] = {{&S72837[30], 6}, {NULL}};
const Segment SGI218595[] = {{&S72837[114], 3}, {NULL}};
const Segment SGI218596[] = {{&S72837[79], 6}, {NULL}};
const Segment SGI218597[] = {{&S91753[44], 10}, {NULL}};
const Segment SGI218598[] = {{&S91753[44], 22}, {NULL}};
const Segment SGI218599[] = {{&S91753[30], 6}, {NULL}};
const Segment SGI218600[] = {{&S91753[111], 2}, {NULL}};
const Segment SGI218601[] = {{&S91753[80], 5}, {NULL}};
const Segment SGI218602[] = {{&S72838[44], 10}, {NULL}};
const Segment SGI218603[] = {{&S72838[44], 22}, {NULL}};
const Segment SGI218604[] = {{&S72838[30], 6}, {NULL}};
const Segment SGI218605[] = {{&S72838[115], 3}, {NULL}};
const Segment SGI218606[] = {{&S72838[80], 6}, {NULL}};
const Segment SGI218607[] = {{&S91754[44], 10}, {NULL}};
const Segment SGI218608[] = {{&S91754[44], 22}, {NULL}};
const Segment SGI218609[] = {{&S91754[30], 6}, {NULL}};
const Segment SGI218610[] = {{&S91754[111], 2}, {NULL}};
const Segment SGI218611[] = {{&S91754[80], 5}, {NULL}};
const Segment SGI218612[] = {{&S72841[34], 10}, {NULL}};
const Segment SGI218613[] = {{&S72841[27], 6}, {NULL}};
const Segment SGI218614[] = {{&S72841[96], 9}, {NULL}};
const Segment SGI218615[] = {{&S72841[69], 6}, {NULL}};
const Segment SGI218616[] = {{&S91756[44], 10}, {NULL}};
const Segment SGI218617[] = {{&S91756[44], 22}, {NULL}};
const Segment SGI218618[] = {{&S91756[30], 6}, {NULL}};
const Segment SGI218619[] = {{&S72842[34], 10}, {NULL}};
const Segment SGI218620[] = {{&S91756[31], 3}, {NULL}};
const Segment SGI218621[] = {{&S72842[27], 6}, {NULL}};
const Segment SGI218622[] = {{&S91756[80], 6}, {NULL}};
const Segment SGI218623[] = {{&S91756[143], 3}, {NULL}};
const Segment SGI218624[] = {{&S72842[96], 9}, {NULL}};
const Segment SGI218625[] = {{&S72842[69], 6}, {NULL}};
const Segment SGI218626[] = {{&S91758[59], 13}, {NULL}};
const Segment SGI218627[] = {{&S91758[59], 25}, {NULL}};
const Segment SGI218628[] = {{&S91758[45], 6}, {NULL}};
const Segment SGI218629[] = {{&S91758[46], 3}, {NULL}};
const Segment SGI218630[] = {{&S91758[98], 6}, {NULL}};
const Segment SGI218631[] = {{&S91758[0], 14}, {NULL}};
const Segment SGI218632[] = {{&S91759[44], 11}, {NULL}};
const Segment SGI218633[] = {{&S91759[44], 23}, {NULL}};
const Segment SGI218634[] = {{&S91759[30], 6}, {NULL}};
const Segment SGI218635[] = {{&S91759[31], 3}, {NULL}};
const Segment SGI218636[] = {{&S91759[81], 6}, {NULL}};
const Segment SGI218637[] = {{&S91759[140], 10}, {NULL}};
const Segment SGI218638[] = {{&S91761[34], 10}, {NULL}};
const Segment SGI218639[] = {{&S91761[27], 6}, {NULL}};
const Segment SGI218640[] = {{&S91761[97], 9}, {NULL}};
const Segment SGI218641[] = {{&S91761[70], 6}, {NULL}};
const Segment SGI218642[] = {{&S91762[44], 10}, {NULL}};
const Segment SGI218643[] = {{&S91762[44], 22}, {NULL}};
const Segment SGI218644[] = {{&S91762[30], 6}, {NULL}};
const Segment SGI218645[] = {{&S91762[80], 6}, {NULL}};
const Segment SGI218646[] = {{&S91762[107], 13}, {NULL}};
const Segment SGI218647[] = {{&S85468[42], 12}, {NULL}};
const Segment SGI218648[] = {{&S85468[42], 24}, {NULL}};
const Segment SGI218649[] = {{&S85468[30], 4}, {NULL}};
const Segment SGI218650[] = {{&S85468[115], 3}, {NULL}};
const Segment SGI218651[] = {{&S85468[80], 6}, {NULL}};
const Segment SGI218652[] = {{&S91764[34], 7}, {NULL}};
const Segment SGI218653[] = {{&S91764[27], 6}, {NULL}};
const Segment SGI218654[] = {{&S91764[94], 9}, {NULL}};
const Segment SGI218655[] = {{&S91764[67], 6}, {NULL}};
const Segment SGI218656[] = {{&S91765[34], 10}, {NULL}};
const Segment SGI218657[] = {{&S91765[27], 6}, {NULL}};
const Segment SGI218658[] = {{&S91765[97], 9}, {NULL}};
const Segment SGI218659[] = {{&S91765[70], 6}, {NULL}};
const Segment SGI218660[] = {{&S91772[34], 13}, {NULL}};
const Segment SGI218661[] = {{&S91772[27], 6}, {NULL}};
const Segment SGI218662[] = {{&S91772[100], 9}, {NULL}};
const Segment SGI218663[] = {{&S91772[73], 6}, {NULL}};
const Segment SGI218664[] = {{&S91781[34], 10}, {NULL}};
const Segment SGI218665[] = {{&S91781[27], 6}, {NULL}};
const Segment SGI218666[] = {{&S91781[97], 9}, {NULL}};
const Segment SGI218667[] = {{&S91781[70], 6}, {NULL}};
const Segment SGI218668[] = {{&S91783[34], 16}, {NULL}};
const Segment SGI218669[] = {{&S91783[27], 6}, {NULL}};
const Segment SGI218670[] = {{&S91783[113], 9}, {NULL}};
const Segment SGI218671[] = {{&S91783[86], 6}, {NULL}};
const Segment SGI218672[] = {{&S72879[34], 9}, {NULL}};
const Segment SGI218673[] = {{&S72879[27], 6}, {NULL}};
const Segment SGI218674[] = {{&S72879[95], 9}, {NULL}};
const Segment SGI218675[] = {{&S72879[68], 6}, {NULL}};
const Segment SGI218676[] = {{&S91787[34], 15}, {NULL}};
const Segment SGI218677[] = {{&S91787[27], 6}, {NULL}};
const Segment SGI218678[] = {{&S91787[102], 9}, {NULL}};
const Segment SGI218679[] = {{&S91787[75], 6}, {NULL}};
const Segment SGI218680[] = {{&S91788[34], 10}, {NULL}};
const Segment SGI218681[] = {{&S91788[27], 6}, {NULL}};
const Segment SGI218682[] = {{&S91788[102], 9}, {NULL}};
const Segment SGI218683[] = {{&S91788[75], 6}, {NULL}};
const Segment SGI218684[] = {{&S86956[34], 7}, {NULL}};
const Segment SGI218685[] = {{&S86956[27], 6}, {NULL}};
const Segment SGI218686[] = {{&S86956[94], 9}, {NULL}};
const Segment SGI218687[] = {{&S86956[67], 6}, {NULL}};
const Segment SGI218688[] = {{&S91796[44], 12}, {NULL}};
const Segment SGI218689[] = {{&S91796[44], 24}, {NULL}};
const Segment SGI218690[] = {{&S91796[30], 6}, {NULL}};
const Segment SGI218691[] = {{&S91796[31], 3}, {NULL}};
const Segment SGI218692[] = {{&S91796[82], 6}, {NULL}};
const Segment SGI218693[] = {{&S91798[34], 11}, {NULL}};
const Segment SGI218694[] = {{&S91798[27], 6}, {NULL}};
const Segment SGI218695[] = {{&S91798[98], 9}, {NULL}};
const Segment SGI218696[] = {{&S91798[71], 6}, {NULL}};
const Segment SGI218697[] = {{&S91799[34], 9}, {NULL}};
const Segment SGI218698[] = {{&S91799[27], 6}, {NULL}};
const Segment SGI218699[] = {{&S91799[96], 9}, {NULL}};
const Segment SGI218700[] = {{&S91799[69], 6}, {NULL}};
const Segment SGI218701[] = {{&S72905[44], 13}, {NULL}};
const Segment SGI218702[] = {{&S72905[44], 24}, {NULL}};
const Segment SGI218703[] = {{&S72905[30], 6}, {NULL}};
const Segment SGI218704[] = {{&S91805[34], 10}, {NULL}};
const Segment SGI218705[] = {{&S91805[27], 6}, {NULL}};
const Segment SGI218706[] = {{&S72905[135], 3}, {NULL}};
const Segment SGI218707[] = {{&S72905[100], 6}, {NULL}};
const Segment SGI218708[] = {{&S72905[70], 8}, {NULL}};
const Segment SGI218709[] = {{&S91805[97], 9}, {NULL}};
const Segment SGI218710[] = {{&S91805[70], 6}, {NULL}};
const Segment SGI218711[] = {{&S91805[138], 8}, {NULL}};
const Segment SGI218712[] = {{&S72906[44], 13}, {NULL}};
const Segment SGI218713[] = {{&S72906[44], 24}, {NULL}};
const Segment SGI218714[] = {{&S72906[30], 6}, {NULL}};
const Segment SGI218715[] = {{&S72906[135], 3}, {NULL}};
const Segment SGI218716[] = {{&S72906[100], 6}, {NULL}};
const Segment SGI218717[] = {{&S72906[70], 8}, {NULL}};
const Segment SGI218718[] = {{&S72907[44], 13}, {NULL}};
const Segment SGI218719[] = {{&S72907[44], 25}, {NULL}};
const Segment SGI218720[] = {{&S72907[30], 6}, {NULL}};
const Segment SGI218721[] = {{&S72907[118], 3}, {NULL}};
const Segment SGI218722[] = {{&S72907[83], 6}, {NULL}};
const Segment SGI218723[] = {{&S72907[143], 8}, {NULL}};
const Segment SGI218724[] = {{&S91813[45], 8}, {NULL}};
const Segment SGI218725[] = {{&S91813[30], 6}, {NULL}};
const Segment SGI218726[] = {{&S91813[31], 3}, {NULL}};
const Segment SGI218727[] = {{&S91813[75], 5}, {NULL}};
const Segment SGI218728[] = {{&S91813[133], 13}, {NULL}};
const Segment SGI218729[] = {{&S86982[34], 15}, {NULL}};
const Segment SGI218730[] = {{&S86982[27], 6}, {NULL}};
const Segment SGI218731[] = {{&S72913[34], 10}, {NULL}};
const Segment SGI218732[] = {{&S72913[27], 6}, {NULL}};
const Segment SGI218733[] = {{&S86982[102], 9}, {NULL}};
const Segment SGI218734[] = {{&S86982[75], 6}, {NULL}};
const Segment SGI218735[] = {{&S72913[96], 9}, {NULL}};
const Segment SGI218736[] = {{&S72913[69], 6}, {NULL}};
const Segment SGI218737[] = {{&S86983[34], 16}, {NULL}};
const Segment SGI218738[] = {{&S86983[27], 6}, {NULL}};
const Segment SGI218739[] = {{&S72914[44], 12}, {NULL}};
const Segment SGI218740[] = {{&S72914[44], 32}, {NULL}};
const Segment SGI218741[] = {{&S72914[30], 6}, {NULL}};
const Segment SGI218742[] = {{&S86983[103], 9}, {NULL}};
const Segment SGI218743[] = {{&S86983[76], 6}, {NULL}};
const Segment SGI218744[] = {{&S72914[125], 3}, {NULL}};
const Segment SGI218745[] = {{&S72914[90], 6}, {NULL}};
const Segment SGI218746[] = {{&S72915[44], 8}, {NULL}};
const Segment SGI218747[] = {{&S72915[44], 28}, {NULL}};
const Segment SGI218748[] = {{&S72915[30], 6}, {NULL}};
const Segment SGI218749[] = {{&S72915[121], 3}, {NULL}};
const Segment SGI218750[] = {{&S72915[86], 6}, {NULL}};
const Segment SGI218751[] = {{&S72916[38], 8}, {NULL}};
const Segment SGI218752[] = {{&S72916[31], 6}, {NULL}};
const Segment SGI218753[] = {{&S91821[44], 7}, {NULL}};
const Segment SGI218754[] = {{&S91821[44], 24}, {NULL}};
const Segment SGI218755[] = {{&S91821[30], 6}, {NULL}};
const Segment SGI218756[] = {{&S91821[82], 6}, {NULL}};
const Segment SGI218757[] = {{&S91821[109], 13}, {NULL}};
const Segment SGI218758[] = {{&S86988[44], 11}, {NULL}};
const Segment SGI218759[] = {{&S86988[44], 23}, {NULL}};
const Segment SGI218760[] = {{&S86988[30], 6}, {NULL}};
const Segment SGI218761[] = {{&S86988[116], 3}, {NULL}};
const Segment SGI218762[] = {{&S86988[81], 6}, {NULL}};
const Segment SGI218763[] = {{&S91825[44], 8}, {NULL}};
const Segment SGI218764[] = {{&S91825[44], 24}, {NULL}};
const Segment SGI218765[] = {{&S91825[30], 6}, {NULL}};
const Segment SGI218766[] = {{&S91825[31], 3}, {NULL}};
const Segment SGI218767[] = {{&S91825[82], 5}, {NULL}};
const Segment SGI218768[] = {{&S91825[119], 14}, {NULL}};
const Segment SGI218769[] = {{&S91826[44], 7}, {NULL}};
const Segment SGI218770[] = {{&S91826[44], 24}, {NULL}};
const Segment SGI218771[] = {{&S91826[30], 6}, {NULL}};
const Segment SGI218772[] = {{&S91826[31], 3}, {NULL}};
const Segment SGI218773[] = {{&S91826[82], 5}, {NULL}};
const Segment SGI218774[] = {{&S91826[119], 14}, {NULL}};
const Segment SGI218775[] = {{&S86996[34], 7}, {NULL}};
const Segment SGI218776[] = {{&S86996[27], 6}, {NULL}};
const Segment SGI218777[] = {{&S86996[94], 9}, {NULL}};
const Segment SGI218778[] = {{&S86996[67], 6}, {NULL}};
const Segment SGI218779[] = {{&S86996[128], 8}, {NULL}};
const Segment SGI218780[] = {{&S91830[34], 10}, {NULL}};
const Segment SGI218781[] = {{&S91830[27], 6}, {NULL}};
const Segment SGI218782[] = {{&S91830[97], 9}, {NULL}};
const Segment SGI218783[] = {{&S91830[70], 6}, {NULL}};
const Segment SGI218784[] = {{&S91830[131], 8}, {NULL}};
const Segment SGI218785[] = {{&S86997[34], 9}, {NULL}};
const Segment SGI218786[] = {{&S86997[27], 6}, {NULL}};
const Segment SGI218787[] = {{&S86997[96], 9}, {NULL}};
const Segment SGI218788[] = {{&S86997[69], 6}, {NULL}};
const Segment SGI218789[] = {{&S86997[130], 8}, {NULL}};
const Segment SGI218790[] = {{&S91831[34], 9}, {NULL}};
const Segment SGI218791[] = {{&S91831[27], 6}, {NULL}};
const Segment SGI218792[] = {{&S91831[96], 9}, {NULL}};
const Segment SGI218793[] = {{&S91831[69], 6}, {NULL}};
const Segment SGI218794[] = {{&S91831[137], 8}, {NULL}};
const Segment SGI218795[] = {{&S91836[34], 6}, {NULL}};
const Segment SGI218796[] = {{&S91836[27], 6}, {NULL}};
const Segment SGI218797[] = {{&S91836[93], 9}, {NULL}};
const Segment SGI218798[] = {{&S91836[66], 6}, {NULL}};
const Segment SGI218799[] = {{&S72941[34], 11}, {NULL}};
const Segment SGI218800[] = {{&S72941[27], 6}, {NULL}};
const Segment SGI218801[] = {{&S87007[44], 7}, {NULL}};
const Segment SGI218802[] = {{&S72941[97], 9}, {NULL}};
const Segment SGI218803[] = {{&S87007[44], 23}, {NULL}};
const Segment SGI218804[] = {{&S72941[70], 6}, {NULL}};
const Segment SGI218805[] = {{&S87007[30], 6}, {NULL}};
const Segment SGI218806[] = {{&S91839[44], 6}, {NULL}};
const Segment SGI218807[] = {{&S91839[44], 18}, {NULL}};
const Segment SGI218808[] = {{&S91839[30], 6}, {NULL}};
const Segment SGI218809[] = {{&S87007[81], 6}, {NULL}};
const Segment SGI218810[] = {{&S87007[108], 13}, {NULL}};
const Segment SGI218811[] = {{&S91839[76], 6}, {NULL}};
const Segment SGI218812[] = {{&S91839[103], 13}, {NULL}};
const Segment SGI218813[] = {{&S87009[34], 8}, {NULL}};
const Segment SGI218814[] = {{&S87009[27], 6}, {NULL}};
const Segment SGI218815[] = {{&S87009[95], 9}, {NULL}};
const Segment SGI218816[] = {{&S87009[68], 6}, {NULL}};
const Segment SGI218817[] = {{&S87013[34], 21}, {NULL}};
const Segment SGI218818[] = {{&S87013[27], 6}, {NULL}};
const Segment SGI218819[] = {{&S87013[108], 9}, {NULL}};
const Segment SGI218820[] = {{&S87013[81], 6}, {NULL}};
const Segment SGI218821[] = {{&S82060[34], 10}, {NULL}};
const Segment SGI218822[] = {{&S82060[27], 6}, {NULL}};
const Segment SGI218823[] = {{&S82060[97], 9}, {NULL}};
const Segment SGI218824[] = {{&S82060[70], 6}, {NULL}};
const Segment SGI218825[] = {{&S82061[34], 11}, {NULL}};
const Segment SGI218826[] = {{&S82061[27], 6}, {NULL}};
const Segment SGI218827[] = {{&S82061[98], 9}, {NULL}};
const Segment SGI218828[] = {{&S82061[71], 6}, {NULL}};
const Segment SGI218829[] = {{&S82062[34], 10}, {NULL}};
const Segment SGI218830[] = {{&S82062[27], 6}, {NULL}};
const Segment SGI218831[] = {{&S82062[97], 9}, {NULL}};
const Segment SGI218832[] = {{&S82062[70], 6}, {NULL}};
const Segment SGI218833[] = {{&S91854[44], 7}, {NULL}};
const Segment SGI218834[] = {{&S91854[44], 24}, {NULL}};
const Segment SGI218835[] = {{&S91854[30], 6}, {NULL}};
const Segment SGI218836[] = {{&S91854[82], 6}, {NULL}};
const Segment SGI218837[] = {{&S82063[34], 11}, {NULL}};
const Segment SGI218838[] = {{&S91854[109], 13}, {NULL}};
const Segment SGI218839[] = {{&S82063[27], 6}, {NULL}};
const Segment SGI218840[] = {{&S82063[98], 9}, {NULL}};
const Segment SGI218841[] = {{&S82063[71], 6}, {NULL}};
const Segment SGI218842[] = {{&S72970[34], 7}, {NULL}};
const Segment SGI218843[] = {{&S72970[27], 6}, {NULL}};
const Segment SGI218844[] = {{&S72970[93], 9}, {NULL}};
const Segment SGI218845[] = {{&S72970[66], 6}, {NULL}};
const Segment SGI218846[] = {{&S72971[34], 11}, {NULL}};
const Segment SGI218847[] = {{&S72971[27], 6}, {NULL}};
const Segment SGI218848[] = {{&S72971[97], 9}, {NULL}};
const Segment SGI218849[] = {{&S72971[70], 6}, {NULL}};
const Segment SGI218850[] = {{&S87086[34], 7}, {NULL}};
const Segment SGI218851[] = {{&S87086[27], 6}, {NULL}};
const Segment SGI218852[] = {{&S87086[94], 9}, {NULL}};
const Segment SGI218853[] = {{&S87086[67], 6}, {NULL}};
const Segment SGI218854[] = {{&S94766[34], 7}, {NULL}};
const Segment SGI218855[] = {{&S94766[27], 6}, {NULL}};
const Segment SGI218856[] = {{&S94766[67], 6}, {NULL}};
const Segment SGI218857[] = {{&S91920[44], 6}, {NULL}};
const Segment SGI218858[] = {{&S91920[44], 18}, {NULL}};
const Segment SGI218859[] = {{&S91920[30], 6}, {NULL}};
const Segment SGI218860[] = {{&S91920[31], 3}, {NULL}};
const Segment SGI218861[] = {{&S91920[91], 6}, {NULL}};
const Segment SGI218862[] = {{&S91920[64], 8}, {NULL}};
const Segment SGI218863[] = {{&S91927[34], 7}, {NULL}};
const Segment SGI218864[] = {{&S91927[27], 6}, {NULL}};
const Segment SGI218865[] = {{&S91927[90], 9}, {NULL}};
const Segment SGI218866[] = {{&S91927[63], 6}, {NULL}};
const Segment SGI218867[] = {{&S87106[44], 11}, {NULL}};
const Segment SGI218868[] = {{&S87106[44], 23}, {NULL}};
const Segment SGI218869[] = {{&S87106[30], 6}, {NULL}};
const Segment SGI218870[] = {{&S87106[116], 3}, {NULL}};
const Segment SGI218871[] = {{&S87106[81], 6}, {NULL}};
const Segment SGI218872[] = {{&S91939[34], 13}, {NULL}};
const Segment SGI218873[] = {{&S91939[27], 6}, {NULL}};
const Segment SGI218874[] = {{&S91939[100], 9}, {NULL}};
const Segment SGI218875[] = {{&S91939[73], 6}, {NULL}};
const Segment SGI218876[] = {{&S91940[45], 6}, {NULL}};
const Segment SGI218877[] = {{&S91940[30], 6}, {NULL}};
const Segment SGI218878[] = {{&S91940[31], 3}, {NULL}};
const Segment SGI218879[] = {{&S91940[77], 5}, {NULL}};
const Segment SGI218880[] = {{&S91940[135], 13}, {NULL}};
const Segment SGI218881[] = {{&S91943[34], 9}, {NULL}};
const Segment SGI218882[] = {{&S91943[27], 6}, {NULL}};
const Segment SGI218883[] = {{&S91943[96], 9}, {NULL}};
const Segment SGI218884[] = {{&S91943[69], 6}, {NULL}};
const Segment SGI218885[] = {{&S91946[34], 16}, {NULL}};
const Segment SGI218886[] = {{&S91946[27], 6}, {NULL}};
const Segment SGI218887[] = {{&S91946[113], 9}, {NULL}};
const Segment SGI218888[] = {{&S91946[86], 6}, {NULL}};
const Segment SGI218889[] = {{&S87123[44], 8}, {NULL}};
const Segment SGI218890[] = {{&S87123[44], 20}, {NULL}};
const Segment SGI218891[] = {{&S87123[30], 6}, {NULL}};
const Segment SGI218892[] = {{&S87123[113], 3}, {NULL}};
const Segment SGI218893[] = {{&S87123[78], 6}, {NULL}};
const Segment SGI218894[] = {{&S91953[31], 5}, {NULL}};
const Segment SGI218895[] = {{&S91953[51], 3}, {NULL}};
const Segment SGI218896[] = {{&S82164[44], 11}, {NULL}};
const Segment SGI218897[] = {{&S82164[44], 23}, {NULL}};
const Segment SGI218898[] = {{&S82164[30], 6}, {NULL}};
const Segment SGI218899[] = {{&S82164[116], 3}, {NULL}};
const Segment SGI218900[] = {{&S82164[81], 6}, {NULL}};
const Segment SGI218901[] = {{&S91954[44], 18}, {NULL}};
const Segment SGI218902[] = {{&S91954[44], 30}, {NULL}};
const Segment SGI218903[] = {{&S91954[30], 6}, {NULL}};
const Segment SGI218904[] = {{&S91954[31], 3}, {NULL}};
const Segment SGI218905[] = {{&S91954[88], 6}, {NULL}};
const Segment SGI218906[] = {{&S91954[148], 11}, {NULL}};
const Segment SGI218907[] = {{&S82165[44], 11}, {NULL}};
const Segment SGI218908[] = {{&S82165[44], 23}, {NULL}};
const Segment SGI218909[] = {{&S82165[30], 6}, {NULL}};
const Segment SGI218910[] = {{&S82165[84], 6}, {NULL}};
const Segment SGI218911[] = {{&S73066[34], 10}, {NULL}};
const Segment SGI218912[] = {{&S73066[27], 6}, {NULL}};
const Segment SGI218913[] = {{&S73066[96], 9}, {NULL}};
const Segment SGI218914[] = {{&S73066[69], 6}, {NULL}};
const Segment SGI218915[] = {{&S73066[137], 8}, {NULL}};
const Segment SGI218916[] = {{&S91959[45], 10}, {NULL}};
const Segment SGI218917[] = {{&S91959[30], 6}, {NULL}};
const Segment SGI218918[] = {{&S91959[31], 3}, {NULL}};
const Segment SGI218919[] = {{&S91959[81], 5}, {NULL}};
const Segment SGI218920[] = {{&S91959[139], 13}, {NULL}};
const Segment SGI218921[] = {{&S87138[34], 14}, {NULL}};
const Segment SGI218922[] = {{&S87138[27], 6}, {NULL}};
const Segment SGI218923[] = {{&S87138[101], 9}, {NULL}};
const Segment SGI218924[] = {{&S87138[74], 6}, {NULL}};
const Segment SGI218925[] = {{&S91965[34], 5}, {NULL}};
const Segment SGI218926[] = {{&S91965[27], 6}, {NULL}};
const Segment SGI218927[] = {{&S91965[92], 9}, {NULL}};
const Segment SGI218928[] = {{&S91965[65], 6}, {NULL}};
const Segment SGI218929[] = {{&S87150[44], 8}, {NULL}};
const Segment SGI218930[] = {{&S87150[44], 20}, {NULL}};
const Segment SGI218931[] = {{&S87150[30], 6}, {NULL}};
const Segment SGI218932[] = {{&S87150[113], 3}, {NULL}};
const Segment SGI218933[] = {{&S87150[78], 6}, {NULL}};
const Segment SGI218934[] = {{&S91976[34], 7}, {NULL}};
const Segment SGI218935[] = {{&S91976[27], 6}, {NULL}};
const Segment SGI218936[] = {{&S87152[34], 7}, {NULL}};
const Segment SGI218937[] = {{&S91976[93], 9}, {NULL}};
const Segment SGI218938[] = {{&S91976[66], 6}, {NULL}};
const Segment SGI218939[] = {{&S87152[27], 6}, {NULL}};
const Segment SGI218940[] = {{&S87152[94], 9}, {NULL}};
const Segment SGI218941[] = {{&S87152[67], 6}, {NULL}};
const Segment SGI218942[] = {{&S87173[34], 8}, {NULL}};
const Segment SGI218943[] = {{&S87173[27], 6}, {NULL}};
const Segment SGI218944[] = {{&S87173[95], 9}, {NULL}};
const Segment SGI218945[] = {{&S87173[68], 6}, {NULL}};
const Segment SGI218946[] = {{&S87174[34], 14}, {NULL}};
const Segment SGI218947[] = {{&S87174[27], 6}, {NULL}};
const Segment SGI218948[] = {{&S87174[101], 9}, {NULL}};
const Segment SGI218949[] = {{&S87174[74], 6}, {NULL}};
const Segment SGI218950[] = {{&S81143[34], 8}, {NULL}};
const Segment SGI218951[] = {{&S81143[27], 6}, {NULL}};
const Segment SGI218952[] = {{&S81143[95], 9}, {NULL}};
const Segment SGI218953[] = {{&S81143[68], 6}, {NULL}};
const Segment SGI218954[] = {{&S87179[34], 9}, {NULL}};
const Segment SGI218955[] = {{&S87179[27], 6}, {NULL}};
const Segment SGI218956[] = {{&S87179[96], 9}, {NULL}};
const Segment SGI218957[] = {{&S87179[69], 6}, {NULL}};
const Segment SGI218958[] = {{&S87180[34], 7}, {NULL}};
const Segment SGI218959[] = {{&S87180[27], 6}, {NULL}};
const Segment SGI218960[] = {{&S87180[94], 9}, {NULL}};
const Segment SGI218961[] = {{&S81146[34], 10}, {NULL}};
const Segment SGI218962[] = {{&S87180[67], 6}, {NULL}};
const Segment SGI218963[] = {{&S81146[27], 6}, {NULL}};
const Segment SGI218964[] = {{&S81146[97], 9}, {NULL}};
const Segment SGI218965[] = {{&S81146[70], 6}, {NULL}};
const Segment SGI218966[] = {{&S81150[34], 6}, {NULL}};
const Segment SGI218967[] = {{&S81150[27], 6}, {NULL}};
const Segment SGI218968[] = {{&S81150[93], 9}, {NULL}};
const Segment SGI218969[] = {{&S81150[66], 6}, {NULL}};
const Segment SGI218970[] = {{&S81151[44], 7}, {NULL}};
const Segment SGI218971[] = {{&S81151[44], 25}, {NULL}};
const Segment SGI218972[] = {{&S81151[30], 6}, {NULL}};
const Segment SGI218973[] = {{&S81151[83], 6}, {NULL}};
const Segment SGI218974[] = {{&S81151[110], 13}, {NULL}};
const Segment SGI218975[] = {{&S81152[44], 7}, {NULL}};
const Segment SGI218976[] = {{&S81152[44], 27}, {NULL}};
const Segment SGI218977[] = {{&S81152[30], 6}, {NULL}};
const Segment SGI218978[] = {{&S81152[85], 6}, {NULL}};
const Segment SGI218979[] = {{&S81152[112], 13}, {NULL}};
const Segment SGI218980[] = {{&S81158[44], 13}, {NULL}};
const Segment SGI218981[] = {{&S81158[44], 25}, {NULL}};
const Segment SGI218982[] = {{&S81158[30], 6}, {NULL}};
const Segment SGI218983[] = {{&S81158[118], 3}, {NULL}};
const Segment SGI218984[] = {{&S81158[83], 6}, {NULL}};
const Segment SGI218985[] = {{&S81161[34], 13}, {NULL}};
const Segment SGI218986[] = {{&S81161[27], 6}, {NULL}};
const Segment SGI218987[] = {{&S81161[100], 9}, {NULL}};
const Segment SGI218988[] = {{&S81161[73], 6}, {NULL}};
const Segment SGI218989[] = {{&S81162[34], 11}, {NULL}};
const Segment SGI218990[] = {{&S81162[27], 6}, {NULL}};
const Segment SGI218991[] = {{&S81162[98], 9}, {NULL}};
const Segment SGI218992[] = {{&S81162[71], 6}, {NULL}};
const Segment SGI218993[] = {{&S82251[44], 7}, {NULL}};
const Segment SGI218994[] = {{&S82251[44], 19}, {NULL}};
const Segment SGI218995[] = {{&S82251[30], 6}, {NULL}};
const Segment SGI218996[] = {{&S82251[77], 6}, {NULL}};
const Segment SGI218997[] = {{&S82251[114], 3}, {NULL}};
const Segment SGI218998[] = {{&S87229[44], 14}, {NULL}};
const Segment SGI218999[] = {{&S87229[44], 31}, {NULL}};
const Segment SGI219000[] = {{&S87229[30], 6}, {NULL}};
const Segment SGI219001[] = {{&S87229[89], 6}, {NULL}};
const Segment SGI219002[] = {{&S87229[116], 13}, {NULL}};
const Segment SGI219003[] = {{&S87240[44], 10}, {NULL}};
const Segment SGI219004[] = {{&S87240[44], 22}, {NULL}};
const Segment SGI219005[] = {{&S87240[30], 6}, {NULL}};
const Segment SGI219006[] = {{&S87240[115], 3}, {NULL}};
const Segment SGI219007[] = {{&S87240[80], 6}, {NULL}};
const Segment SGI219008[] = {{&S81202[34], 11}, {NULL}};
const Segment SGI219009[] = {{&S81202[27], 6}, {NULL}};
const Segment SGI219010[] = {{&S81202[98], 9}, {NULL}};
const Segment SGI219011[] = {{&S81202[71], 6}, {NULL}};
const Segment SGI219012[] = {{&S81203[34], 11}, {NULL}};
const Segment SGI219013[] = {{&S81203[27], 6}, {NULL}};
const Segment SGI219014[] = {{&S81203[98], 9}, {NULL}};
const Segment SGI219015[] = {{&S81203[71], 6}, {NULL}};
const Segment SGI219016[] = {{&S87250[44], 15}, {NULL}};
const Segment SGI219017[] = {{&S87250[44], 27}, {NULL}};
const Segment SGI219018[] = {{&S87250[30], 6}, {NULL}};
const Segment SGI219019[] = {{&S87250[85], 6}, {NULL}};
const Segment SGI219020[] = {{&S87250[112], 13}, {NULL}};
const Segment SGI219021[] = {{&S81207[44], 7}, {NULL}};
const Segment SGI219022[] = {{&S81207[44], 24}, {NULL}};
const Segment SGI219023[] = {{&S81207[30], 6}, {NULL}};
const Segment SGI219024[] = {{&S81207[82], 6}, {NULL}};
const Segment SGI219025[] = {{&S81207[109], 13}, {NULL}};
const Segment SGI219026[] = {{&S81208[44], 7}, {NULL}};
const Segment SGI219027[] = {{&S81208[44], 24}, {NULL}};
const Segment SGI219028[] = {{&S81208[30], 6}, {NULL}};
const Segment SGI219029[] = {{&S81208[82], 6}, {NULL}};
const Segment SGI219030[] = {{&S81208[109], 13}, {NULL}};
const Segment SGI219031[] = {{&S81212[34], 13}, {NULL}};
const Segment SGI219032[] = {{&S81212[27], 6}, {NULL}};
const Segment SGI219033[] = {{&S81212[100], 9}, {NULL}};
const Segment SGI219034[] = {{&S81212[73], 6}, {NULL}};
const Segment SGI219035[] = {{&S81213[44], 15}, {NULL}};
const Segment SGI219036[] = {{&S81213[44], 27}, {NULL}};
const Segment SGI219037[] = {{&S81213[30], 6}, {NULL}};
const Segment SGI219038[] = {{&S81213[120], 3}, {NULL}};
const Segment SGI219039[] = {{&S81213[85], 6}, {NULL}};
const Segment SGI219040[] = {{&S81213[145], 13}, {NULL}};
const Segment SGI219041[] = {{&S81215[34], 9}, {NULL}};
const Segment SGI219042[] = {{&S81215[27], 6}, {NULL}};
const Segment SGI219043[] = {{&S81215[96], 9}, {NULL}};
const Segment SGI219044[] = {{&S81215[69], 6}, {NULL}};
const Segment SGI219045[] = {{&S81216[34], 10}, {NULL}};
const Segment SGI219046[] = {{&S81216[27], 6}, {NULL}};
const Segment SGI219047[] = {{&S81216[97], 9}, {NULL}};
const Segment SGI219048[] = {{&S81216[70], 6}, {NULL}};
const Segment SGI219049[] = {{&S81219[44], 10}, {NULL}};
const Segment SGI219050[] = {{&S81219[44], 22}, {NULL}};
const Segment SGI219051[] = {{&S81219[30], 6}, {NULL}};
const Segment SGI219052[] = {{&S81219[115], 3}, {NULL}};
const Segment SGI219053[] = {{&S81219[80], 6}, {NULL}};
const Segment SGI219054[] = {{&S81225[34], 10}, {NULL}};
const Segment SGI219055[] = {{&S81225[27], 6}, {NULL}};
const Segment SGI219056[] = {{&S81225[97], 9}, {NULL}};
const Segment SGI219057[] = {{&S81225[70], 6}, {NULL}};
const Segment SGI219058[] = {{&S81227[34], 7}, {NULL}};
const Segment SGI219059[] = {{&S81227[27], 6}, {NULL}};
const Segment SGI219060[] = {{&S81227[94], 9}, {NULL}};
const Segment SGI219061[] = {{&S81227[67], 6}, {NULL}};
const Segment SGI219062[] = {{&S82343[34], 7}, {NULL}};
const Segment SGI219063[] = {{&S82343[27], 6}, {NULL}};
const Segment SGI219064[] = {{&S73248[34], 21}, {NULL}};
const Segment SGI219065[] = {{&S73248[27], 6}, {NULL}};
const Segment SGI219066[] = {{&S82343[94], 9}, {NULL}};
const Segment SGI219067[] = {{&S82343[67], 6}, {NULL}};
const Segment SGI219068[] = {{&S73248[108], 9}, {NULL}};
const Segment SGI219069[] = {{&S73248[81], 6}, {NULL}};
const Segment SGI219070[] = {{&S73249[34], 20}, {NULL}};
const Segment SGI219071[] = {{&S73249[27], 6}, {NULL}};
const Segment SGI219072[] = {{&S73249[106], 9}, {NULL}};
const Segment SGI219073[] = {{&S73249[79], 6}, {NULL}};
const Segment SGI219074[] = {{&S91981[34], 8}, {NULL}};
const Segment SGI219075[] = {{&S91981[27], 6}, {NULL}};
const Segment SGI219076[] = {{&S91981[95], 9}, {NULL}};
const Segment SGI219077[] = {{&S91981[68], 6}, {NULL}};
const Segment SGI219078[] = {{&S81266[59], 10}, {NULL}};
const Segment SGI219079[] = {{&S81266[59], 22}, {NULL}};
const Segment SGI219080[] = {{&S81266[45], 6}, {NULL}};
const Segment SGI219081[] = {{&S91982[34], 10}, {NULL}};
const Segment SGI219082[] = {{&S91982[27], 6}, {NULL}};
const Segment SGI219083[] = {{&S81266[130], 3}, {NULL}};
const Segment SGI219084[] = {{&S81266[95], 6}, {NULL}};
const Segment SGI219085[] = {{&S91982[97], 9}, {NULL}};
const Segment SGI219086[] = {{&S81266[11], 3}, {NULL}};
const Segment SGI219087[] = {{&S91982[70], 6}, {NULL}};
const Segment SGI219088[] = {{&S87323[44], 9}, {NULL}};
const Segment SGI219089[] = {{&S87323[44], 21}, {NULL}};
const Segment SGI219090[] = {{&S87323[30], 6}, {NULL}};
const Segment SGI219091[] = {{&S87323[114], 3}, {NULL}};
const Segment SGI219092[] = {{&S87323[79], 6}, {NULL}};
const Segment SGI219093[] = {{&S82360[44], 17}, {NULL}};
const Segment SGI219094[] = {{&S82360[44], 37}, {NULL}};
const Segment SGI219095[] = {{&S82360[30], 6}, {NULL}};
const Segment SGI219096[] = {{&S73264[44], 6}, {NULL}};
const Segment SGI219097[] = {{&S73264[44], 17}, {NULL}};
const Segment SGI219098[] = {{&S73264[30], 6}, {NULL}};
const Segment SGI219099[] = {{&S82360[95], 6}, {NULL}};
const Segment SGI219100[] = {{&S73264[110], 3}, {NULL}};
const Segment SGI219101[] = {{&S73264[75], 6}, {NULL}};
const Segment SGI219102[] = {{&S82360[132], 3}, {NULL}};
const Segment SGI219103[] = {{&S91986[34], 15}, {NULL}};
const Segment SGI219104[] = {{&S91986[27], 6}, {NULL}};
const Segment SGI219105[] = {{&S73265[44], 7}, {NULL}};
const Segment SGI219106[] = {{&S91986[99], 9}, {NULL}};
const Segment SGI219107[] = {{&S73265[44], 18}, {NULL}};
const Segment SGI219108[] = {{&S91986[72], 6}, {NULL}};
const Segment SGI219109[] = {{&S73265[30], 6}, {NULL}};
const Segment SGI219110[] = {{&S73265[111], 3}, {NULL}};
const Segment SGI219111[] = {{&S73265[76], 6}, {NULL}};
const Segment SGI219112[] = {{&S73266[34], 15}, {NULL}};
const Segment SGI219113[] = {{&S73266[27], 6}, {NULL}};
const Segment SGI219114[] = {{&S73266[102], 9}, {NULL}};
const Segment SGI219115[] = {{&S91988[45], 10}, {NULL}};
const Segment SGI219116[] = {{&S73266[75], 6}, {NULL}};
const Segment SGI219117[] = {{&S91988[30], 6}, {NULL}};
const Segment SGI219118[] = {{&S91988[31], 3}, {NULL}};
const Segment SGI219119[] = {{&S91988[81], 5}, {NULL}};
const Segment SGI219120[] = {{&S91988[139], 13}, {NULL}};
const Segment SGI219121[] = {{&S91990[34], 10}, {NULL}};
const Segment SGI219122[] = {{&S91990[27], 6}, {NULL}};
const Segment SGI219123[] = {{&S91990[97], 9}, {NULL}};
const Segment SGI219124[] = {{&S91990[70], 6}, {NULL}};
const Segment SGI219125[] = {{&S91991[34], 10}, {NULL}};
const Segment SGI219126[] = {{&S91991[27], 6}, {NULL}};
const Segment SGI219127[] = {{&S91991[97], 9}, {NULL}};
const Segment SGI219128[] = {{&S91991[70], 6}, {NULL}};
const Segment SGI219129[] = {{&S91997[44], 14}, {NULL}};
const Segment SGI219130[] = {{&S91997[44], 31}, {NULL}};
const Segment SGI219131[] = {{&S91997[30], 6}, {NULL}};
const Segment SGI219132[] = {{&S91997[31], 3}, {NULL}};
const Segment SGI219133[] = {{&S91997[89], 5}, {NULL}};
const Segment SGI219134[] = {{&S91997[126], 14}, {NULL}};
const Segment SGI219135[] = {{&S87344[44], 8}, {NULL}};
const Segment SGI219136[] = {{&S87344[44], 20}, {NULL}};
const Segment SGI219137[] = {{&S87344[30], 6}, {NULL}};
const Segment SGI219138[] = {{&S87344[113], 3}, {NULL}};
const Segment SGI219139[] = {{&S87344[78], 6}, {NULL}};
const Segment SGI219140[] = {{&S92000[44], 14}, {NULL}};
const Segment SGI219141[] = {{&S92000[44], 26}, {NULL}};
const Segment SGI219142[] = {{&S92000[30], 6}, {NULL}};
const Segment SGI219143[] = {{&S92000[84], 6}, {NULL}};
const Segment SGI219144[] = {{&S92000[111], 13}, {NULL}};
const Segment SGI219145[] = {{&S92003[44], 5}, {NULL}};
const Segment SGI219146[] = {{&S92003[44], 17}, {NULL}};
const Segment SGI219147[] = {{&S92003[30], 6}, {NULL}};
const Segment SGI219148[] = {{&S92003[31], 3}, {NULL}};
const Segment SGI219149[] = {{&S92003[75], 6}, {NULL}};
const Segment SGI219150[] = {{&S92003[135], 13}, {NULL}};
const Segment SGI219151[] = {{&S92006[44], 7}, {NULL}};
const Segment SGI219152[] = {{&S92006[44], 24}, {NULL}};
const Segment SGI219153[] = {{&S92006[30], 6}, {NULL}};
const Segment SGI219154[] = {{&S92006[82], 6}, {NULL}};
const Segment SGI219155[] = {{&S92006[109], 13}, {NULL}};
const Segment SGI219156[] = {{&S92010[44], 10}, {NULL}};
const Segment SGI219157[] = {{&S92010[44], 27}, {NULL}};
const Segment SGI219158[] = {{&S92010[30], 6}, {NULL}};
const Segment SGI219159[] = {{&S87358[34], 12}, {NULL}};
const Segment SGI219160[] = {{&S87358[27], 6}, {NULL}};
const Segment SGI219161[] = {{&S92010[85], 6}, {NULL}};
const Segment SGI219162[] = {{&S82398[45], 9}, {NULL}};
const Segment SGI219163[] = {{&S82398[30], 6}, {NULL}};
const Segment SGI219164[] = {{&S92010[112], 13}, {NULL}};
const Segment SGI219165[] = {{&S87358[99], 9}, {NULL}};
const Segment SGI219166[] = {{&S87358[72], 6}, {NULL}};
const Segment SGI219167[] = {{&S82398[114], 3}, {NULL}};
const Segment SGI219168[] = {{&S82398[80], 5}, {NULL}};
const Segment SGI219169[] = {{&S87358[140], 8}, {NULL}};
const Segment SGI219170[] = {{&S82398[148], 3}, {NULL}};
const Segment SGI219171[] = {{&S87362[44], 13}, {NULL}};
const Segment SGI219172[] = {{&S92015[52], 5}, {NULL}};
const Segment SGI219173[] = {{&S87362[44], 25}, {NULL}};
const Segment SGI219174[] = {{&S92015[52], 17}, {NULL}};
const Segment SGI219175[] = {{&S87362[30], 6}, {NULL}};
const Segment SGI219176[] = {{&S92015[38], 6}, {NULL}};
const Segment SGI219177[] = {{&S87362[118], 3}, {NULL}};
const Segment SGI219178[] = {{&S92015[83], 5}, {NULL}};
const Segment SGI219179[] = {{&S87362[83], 6}, {NULL}};
const Segment SGI219180[] = {{&S92015[0], 14}, {NULL}};
const Segment SGI219181[] = {{&S87363[44], 11}, {NULL}};
const Segment SGI219182[] = {{&S87363[44], 23}, {NULL}};
const Segment SGI219183[] = {{&S87363[30], 6}, {NULL}};
const Segment SGI219184[] = {{&S87363[116], 3}, {NULL}};
const Segment SGI219185[] = {{&S87363[81], 6}, {NULL}};
const Segment SGI219186[] = {{&S82407[34], 25}, {NULL}};
const Segment SGI219187[] = {{&S82407[27], 6}, {NULL}};
const Segment SGI219188[] = {{&S82407[112], 9}, {NULL}};
const Segment SGI219189[] = {{&S82407[85], 6}, {NULL}};
const Segment SGI219190[] = {{&S87366[34], 7}, {NULL}};
const Segment SGI219191[] = {{&S92020[45], 10}, {NULL}};
const Segment SGI219192[] = {{&S87366[27], 6}, {NULL}};
const Segment SGI219193[] = {{&S92020[30], 6}, {NULL}};
const Segment SGI219194[] = {{&S87366[94], 9}, {NULL}};
const Segment SGI219195[] = {{&S92020[31], 3}, {NULL}};
const Segment SGI219196[] = {{&S87366[67], 6}, {NULL}};
const Segment SGI219197[] = {{&S92020[88], 5}, {NULL}};
const Segment SGI219198[] = {{&S92020[146], 13}, {NULL}};
const Segment SGI219199[] = {{&S92022[34], 15}, {NULL}};
const Segment SGI219200[] = {{&S92022[27], 6}, {NULL}};
const Segment SGI219201[] = {{&S92022[102], 9}, {NULL}};
const Segment SGI219202[] = {{&S92022[75], 6}, {NULL}};
const Segment SGI219203[] = {{&S92022[143], 8}, {NULL}};
const Segment SGI219204[] = {{&S92023[34], 15}, {NULL}};
const Segment SGI219205[] = {{&S92023[27], 6}, {NULL}};
const Segment SGI219206[] = {{&S92023[102], 9}, {NULL}};
const Segment SGI219207[] = {{&S92023[75], 6}, {NULL}};
const Segment SGI219208[] = {{&S92023[143], 8}, {NULL}};
const Segment SGI219209[] = {{&S92029[34], 15}, {NULL}};
const Segment SGI219210[] = {{&S92029[27], 6}, {NULL}};
const Segment SGI219211[] = {{&S92029[102], 9}, {NULL}};
const Segment SGI219212[] = {{&S92029[75], 6}, {NULL}};
const Segment SGI219213[] = {{&S87379[44], 21}, {NULL}};
const Segment SGI219214[] = {{&S87379[44], 33}, {NULL}};
const Segment SGI219215[] = {{&S87379[30], 6}, {NULL}};
const Segment SGI219216[] = {{&S92030[34], 14}, {NULL}};
const Segment SGI219217[] = {{&S73326[34], 25}, {NULL}};
const Segment SGI219218[] = {{&S87379[31], 3}, {NULL}};
const Segment SGI219219[] = {{&S92030[27], 6}, {NULL}};
const Segment SGI219220[] = {{&S73326[27], 6}, {NULL}};
const Segment SGI219221[] = {{&S87379[91], 6}, {NULL}};
const Segment SGI219222[] = {{&S92030[101], 9}, {NULL}};
const Segment SGI219223[] = {{&S73326[112], 9}, {NULL}};
const Segment SGI219224[] = {{&S92030[74], 6}, {NULL}};
const Segment SGI219225[] = {{&S73326[85], 6}, {NULL}};
const Segment SGI219226[] = {{&S73327[34], 12}, {NULL}};
const Segment SGI219227[] = {{&S92031[34], 9}, {NULL}};
const Segment SGI219228[] = {{&S73327[27], 6}, {NULL}};
const Segment SGI219229[] = {{&S92031[27], 6}, {NULL}};
const Segment SGI219230[] = {{&S73327[99], 9}, {NULL}};
const Segment SGI219231[] = {{&S92031[96], 9}, {NULL}};
const Segment SGI219232[] = {{&S73327[72], 6}, {NULL}};
const Segment SGI219233[] = {{&S92031[69], 6}, {NULL}};
const Segment SGI219234[] = {{&S92036[34], 12}, {NULL}};
const Segment SGI219235[] = {{&S92036[27], 6}, {NULL}};
const Segment SGI219236[] = {{&S82434[44], 11}, {NULL}};
const Segment SGI219237[] = {{&S82434[44], 23}, {NULL}};
const Segment SGI219238[] = {{&S82434[30], 6}, {NULL}};
const Segment SGI219239[] = {{&S92036[99], 9}, {NULL}};
const Segment SGI219240[] = {{&S92036[72], 6}, {NULL}};
const Segment SGI219241[] = {{&S82434[116], 3}, {NULL}};
const Segment SGI219242[] = {{&S82434[81], 6}, {NULL}};
const Segment SGI219243[] = {{&S87394[34], 12}, {NULL}};
const Segment SGI219244[] = {{&S87394[27], 6}, {NULL}};
const Segment SGI219245[] = {{&S87394[112], 9}, {NULL}};
const Segment SGI219246[] = {{&S87394[85], 6}, {NULL}};
const Segment SGI219247[] = {{&S92044[44], 10}, {NULL}};
const Segment SGI219248[] = {{&S92044[44], 27}, {NULL}};
const Segment SGI219249[] = {{&S92044[30], 6}, {NULL}};
const Segment SGI219250[] = {{&S92044[85], 6}, {NULL}};
const Segment SGI219251[] = {{&S92044[112], 13}, {NULL}};
const Segment SGI219252[] = {{&S92046[34], 10}, {NULL}};
const Segment SGI219253[] = {{&S92046[27], 6}, {NULL}};
const Segment SGI219254[] = {{&S92046[104], 9}, {NULL}};
const Segment SGI219255[] = {{&S92046[77], 6}, {NULL}};
const Segment SGI219256[] = {{&S92046[145], 8}, {NULL}};
const Segment SGI219257[] = {{&S92056[34], 12}, {NULL}};
const Segment SGI219258[] = {{&S92056[27], 6}, {NULL}};
const Segment SGI219259[] = {{&S92056[99], 9}, {NULL}};
const Segment SGI219260[] = {{&S92056[72], 6}, {NULL}};
const Segment SGI219261[] = {{&S92059[34], 8}, {NULL}};
const Segment SGI219262[] = {{&S92059[27], 6}, {NULL}};
const Segment SGI219263[] = {{&S92059[95], 9}, {NULL}};
const Segment SGI219264[] = {{&S92059[68], 6}, {NULL}};
const Segment SGI219265[] = {{&S87447[44], 13}, {NULL}};
const Segment SGI219266[] = {{&S87447[44], 25}, {NULL}};
const Segment SGI219267[] = {{&S87447[30], 6}, {NULL}};
const Segment SGI219268[] = {{&S87447[118], 3}, {NULL}};
const Segment SGI219269[] = {{&S87447[83], 6}, {NULL}};
const Segment SGI219270[] = {{&S87448[44], 10}, {NULL}};
const Segment SGI219271[] = {{&S87448[44], 22}, {NULL}};
const Segment SGI219272[] = {{&S87448[30], 6}, {NULL}};
const Segment SGI219273[] = {{&S87448[114], 3}, {NULL}};
const Segment SGI219274[] = {{&S87448[80], 5}, {NULL}};
const Segment SGI219275[] = {{&S87448[125], 14}, {NULL}};
const Segment SGI219276[] = {{&S87452[44], 17}, {NULL}};
const Segment SGI219277[] = {{&S87452[44], 29}, {NULL}};
const Segment SGI219278[] = {{&S87452[30], 6}, {NULL}};
const Segment SGI219279[] = {{&S87452[122], 3}, {NULL}};
const Segment SGI219280[] = {{&S87452[87], 6}, {NULL}};
const Segment SGI219281[] = {{&S81272[45], 10}, {NULL}};
const Segment SGI219282[] = {{&S81272[30], 6}, {NULL}};
const Segment SGI219283[] = {{&S81272[115], 3}, {NULL}};
const Segment SGI219284[] = {{&S81272[81], 5}, {NULL}};
const Segment SGI219285[] = {{&S81272[139], 13}, {NULL}};
const Segment SGI219286[] = {{&S73420[34], 11}, {NULL}};
const Segment SGI219287[] = {{&S73420[27], 6}, {NULL}};
const Segment SGI219288[] = {{&S81283[34], 4}, {NULL}};
const Segment SGI219289[] = {{&S81283[27], 6}, {NULL}};
const Segment SGI219290[] = {{&S73420[97], 9}, {NULL}};
const Segment SGI219291[] = {{&S73420[70], 6}, {NULL}};
const Segment SGI219292[] = {{&S81283[91], 9}, {NULL}};
const Segment SGI219293[] = {{&S81283[64], 6}, {NULL}};
const Segment SGI219294[] = {{&S81289[34], 10}, {NULL}};
const Segment SGI219295[] = {{&S81289[27], 6}, {NULL}};
const Segment SGI219296[] = {{&S81289[97], 9}, {NULL}};
const Segment SGI219297[] = {{&S81289[70], 6}, {NULL}};
const Segment SGI219298[] = {{&S92136[34], 11}, {NULL}};
const Segment SGI219299[] = {{&S92136[27], 6}, {NULL}};
const Segment SGI219300[] = {{&S92136[98], 9}, {NULL}};
const Segment SGI219301[] = {{&S92136[71], 6}, {NULL}};
const Segment SGI219302[] = {{&S87490[44], 17}, {NULL}};
const Segment SGI219303[] = {{&S87490[44], 36}, {NULL}};
const Segment SGI219304[] = {{&S87490[30], 6}, {NULL}};
const Segment SGI219305[] = {{&S87490[129], 3}, {NULL}};
const Segment SGI219306[] = {{&S81300[44], 14}, {NULL}};
const Segment SGI219307[] = {{&S87490[94], 6}, {NULL}};
const Segment SGI219308[] = {{&S81300[44], 35}, {NULL}};
const Segment SGI219309[] = {{&S81300[30], 6}, {NULL}};
const Segment SGI219310[] = {{&S81300[147], 3}, {NULL}};
const Segment SGI219311[] = {{&S81300[112], 6}, {NULL}};
const Segment SGI219312[] = {{&S92141[34], 6}, {NULL}};
const Segment SGI219313[] = {{&S92141[27], 6}, {NULL}};
const Segment SGI219314[] = {{&S92141[93], 9}, {NULL}};
const Segment SGI219315[] = {{&S92141[66], 6}, {NULL}};
const Segment SGI219316[] = {{&S92144[34], 9}, {NULL}};
const Segment SGI219317[] = {{&S92144[27], 6}, {NULL}};
const Segment SGI219318[] = {{&S87496[44], 9}, {NULL}};
const Segment SGI219319[] = {{&S87496[44], 21}, {NULL}};
const Segment SGI219320[] = {{&S92144[96], 9}, {NULL}};
const Segment SGI219321[] = {{&S87496[30], 6}, {NULL}};
const Segment SGI219322[] = {{&S92144[69], 6}, {NULL}};
const Segment SGI219323[] = {{&S87496[114], 3}, {NULL}};
const Segment SGI219324[] = {{&S87496[79], 6}, {NULL}};
const Segment SGI219325[] = {{&S87498[52], 15}, {NULL}};
const Segment SGI219326[] = {{&S87498[52], 27}, {NULL}};
const Segment SGI219327[] = {{&S87498[38], 6}, {NULL}};
const Segment SGI219328[] = {{&S87498[93], 5}, {NULL}};
const Segment SGI219329[] = {{&S92147[34], 8}, {NULL}};
const Segment SGI219330[] = {{&S92147[27], 6}, {NULL}};
const Segment SGI219331[] = {{&S87498[0], 14}, {NULL}};
const Segment SGI219332[] = {{&S92147[94], 9}, {NULL}};
const Segment SGI219333[] = {{&S92147[67], 6}, {NULL}};
const Segment SGI219334[] = {{&S87502[34], 7}, {NULL}};
const Segment SGI219335[] = {{&S87502[27], 6}, {NULL}};
const Segment SGI219336[] = {{&S87502[94], 9}, {NULL}};
const Segment SGI219337[] = {{&S87502[67], 6}, {NULL}};
const Segment SGI219338[] = {{&S92153[34], 25}, {NULL}};
const Segment SGI219339[] = {{&S92153[27], 6}, {NULL}};
const Segment SGI219340[] = {{&S92153[112], 9}, {NULL}};
const Segment SGI219341[] = {{&S92153[85], 6}, {NULL}};
const Segment SGI219342[] = {{&S87504[34], 16}, {NULL}};
const Segment SGI219343[] = {{&S87504[27], 6}, {NULL}};
const Segment SGI219344[] = {{&S87504[103], 9}, {NULL}};
const Segment SGI219345[] = {{&S87504[76], 6}, {NULL}};
const Segment SGI219346[] = {{&S92156[45], 15}, {NULL}};
const Segment SGI219347[] = {{&S92156[30], 6}, {NULL}};
const Segment SGI219348[] = {{&S92156[31], 3}, {NULL}};
const Segment SGI219349[] = {{&S92156[86], 5}, {NULL}};
const Segment SGI219350[] = {{&S92156[144], 13}, {NULL}};
const Segment SGI219351[] = {{&S92158[44], 17}, {NULL}};
const Segment SGI219352[] = {{&S92158[44], 28}, {NULL}};
const Segment SGI219353[] = {{&S92158[30], 6}, {NULL}};
const Segment SGI219354[] = {{&S92158[31], 3}, {NULL}};
const Segment SGI219355[] = {{&S92158[101], 6}, {NULL}};
const Segment SGI219356[] = {{&S92158[74], 8}, {NULL}};
const Segment SGI219357[] = {{&S92160[34], 7}, {NULL}};
const Segment SGI219358[] = {{&S92160[27], 6}, {NULL}};
const Segment SGI219359[] = {{&S92160[94], 9}, {NULL}};
const Segment SGI219360[] = {{&S92160[67], 6}, {NULL}};
const Segment SGI219361[] = {{&S87517[34], 15}, {NULL}};
const Segment SGI219362[] = {{&S87517[27], 6}, {NULL}};
const Segment SGI219363[] = {{&S87517[102], 9}, {NULL}};
const Segment SGI219364[] = {{&S87517[75], 6}, {NULL}};
const Segment SGI219365[] = {{&S92167[34], 21}, {NULL}};
const Segment SGI219366[] = {{&S92167[27], 6}, {NULL}};
const Segment SGI219367[] = {{&S92167[108], 9}, {NULL}};
const Segment SGI219368[] = {{&S92167[81], 6}, {NULL}};
const Segment SGI219369[] = {{&S92169[59], 5}, {NULL}};
const Segment SGI219370[] = {{&S92169[59], 17}, {NULL}};
const Segment SGI219371[] = {{&S92169[45], 6}, {NULL}};
const Segment SGI219372[] = {{&S92169[46], 3}, {NULL}};
const Segment SGI219373[] = {{&S92169[90], 6}, {NULL}};
const Segment SGI219374[] = {{&S92169[0], 14}, {NULL}};
const Segment SGI219375[] = {{&S92171[34], 6}, {NULL}};
const Segment SGI219376[] = {{&S92171[27], 6}, {NULL}};
const Segment SGI219377[] = {{&S92171[105], 9}, {NULL}};
const Segment SGI219378[] = {{&S92171[78], 6}, {NULL}};
const Segment SGI219379[] = {{&S92172[44], 11}, {NULL}};
const Segment SGI219380[] = {{&S92172[44], 23}, {NULL}};
const Segment SGI219381[] = {{&S92172[30], 6}, {NULL}};
const Segment SGI219382[] = {{&S92172[31], 3}, {NULL}};
const Segment SGI219383[] = {{&S92172[81], 6}, {NULL}};
const Segment SGI219384[] = {{&S92173[34], 10}, {NULL}};
const Segment SGI219385[] = {{&S92173[27], 6}, {NULL}};
const Segment SGI219386[] = {{&S92173[97], 9}, {NULL}};
const Segment SGI219387[] = {{&S92173[70], 6}, {NULL}};
const Segment SGI219388[] = {{&S81348[44], 10}, {NULL}};
const Segment SGI219389[] = {{&S81348[44], 22}, {NULL}};
const Segment SGI219390[] = {{&S81348[30], 6}, {NULL}};
const Segment SGI219391[] = {{&S81348[115], 3}, {NULL}};
const Segment SGI219392[] = {{&S81348[80], 6}, {NULL}};
const Segment SGI219393[] = {{&S81348[133], 11}, {NULL}};
const Segment SGI219394[] = {{&S92175[34], 7}, {NULL}};
const Segment SGI219395[] = {{&S92175[27], 6}, {NULL}};
const Segment SGI219396[] = {{&S92175[99], 9}, {NULL}};
const Segment SGI219397[] = {{&S92175[72], 6}, {NULL}};
const Segment SGI219398[] = {{&S81351[44], 10}, {NULL}};
const Segment SGI219399[] = {{&S81351[44], 22}, {NULL}};
const Segment SGI219400[] = {{&S81351[30], 6}, {NULL}};
const Segment SGI219401[] = {{&S81351[115], 3}, {NULL}};
const Segment SGI219402[] = {{&S81351[80], 6}, {NULL}};
const Segment SGI219403[] = {{&S81351[134], 12}, {NULL}};
const Segment SGI219404[] = {{&S92178[34], 8}, {NULL}};
const Segment SGI219405[] = {{&S92178[27], 6}, {NULL}};
const Segment SGI219406[] = {{&S81352[34], 6}, {NULL}};
const Segment SGI219407[] = {{&S92178[95], 9}, {NULL}};
const Segment SGI219408[] = {{&S81352[27], 6}, {NULL}};
const Segment SGI219409[] = {{&S92178[68], 6}, {NULL}};
const Segment SGI219410[] = {{&S81352[93], 9}, {NULL}};
const Segment SGI219411[] = {{&S81352[66], 6}, {NULL}};
const Segment SGI219412[] = {{&S92179[34], 4}, {NULL}};
const Segment SGI219413[] = {{&S92179[27], 6}, {NULL}};
const Segment SGI219414[] = {{&S92179[88], 9}, {NULL}};
const Segment SGI219415[] = {{&S92179[61], 6}, {NULL}};
const Segment SGI219416[] = {{&S73501[34], 8}, {NULL}};
const Segment SGI219417[] = {{&S73501[27], 6}, {NULL}};
const Segment SGI219418[] = {{&S73501[95], 9}, {NULL}};
const Segment SGI219419[] = {{&S73501[68], 6}, {NULL}};
const Segment SGI219420[] = {{&S92180[34], 4}, {NULL}};
const Segment SGI219421[] = {{&S92180[27], 6}, {NULL}};
const Segment SGI219422[] = {{&S92180[91], 9}, {NULL}};
const Segment SGI219423[] = {{&S92180[64], 6}, {NULL}};
const Segment SGI219424[] = {{&S73502[34], 12}, {NULL}};
const Segment SGI219425[] = {{&S73502[27], 6}, {NULL}};
const Segment SGI219426[] = {{&S73502[98], 9}, {NULL}};
const Segment SGI219427[] = {{&S73502[71], 6}, {NULL}};
const Segment SGI219428[] = {{&S81360[34], 10}, {NULL}};
const Segment SGI219429[] = {{&S81360[27], 6}, {NULL}};
const Segment SGI219430[] = {{&S81360[97], 9}, {NULL}};
const Segment SGI219431[] = {{&S81360[70], 6}, {NULL}};
const Segment SGI219432[] = {{&S92186[34], 11}, {NULL}};
const Segment SGI219433[] = {{&S92186[27], 6}, {NULL}};
const Segment SGI219434[] = {{&S92186[98], 9}, {NULL}};
const Segment SGI219435[] = {{&S92186[71], 6}, {NULL}};
const Segment SGI219436[] = {{&S92187[34], 10}, {NULL}};
const Segment SGI219437[] = {{&S92187[27], 6}, {NULL}};
const Segment SGI219438[] = {{&S92187[104], 9}, {NULL}};
const Segment SGI219439[] = {{&S92187[77], 6}, {NULL}};
const Segment SGI219440[] = {{&S92188[34], 12}, {NULL}};
const Segment SGI219441[] = {{&S92188[27], 6}, {NULL}};
const Segment SGI219442[] = {{&S92188[99], 9}, {NULL}};
const Segment SGI219443[] = {{&S92188[72], 6}, {NULL}};
const Segment SGI219444[] = {{&S92189[34], 14}, {NULL}};
const Segment SGI219445[] = {{&S92189[27], 6}, {NULL}};
const Segment SGI219446[] = {{&S92189[101], 9}, {NULL}};
const Segment SGI219447[] = {{&S92189[74], 6}, {NULL}};
const Segment SGI219448[] = {{&S81368[44], 7}, {NULL}};
const Segment SGI219449[] = {{&S81368[44], 24}, {NULL}};
const Segment SGI219450[] = {{&S81368[30], 6}, {NULL}};
const Segment SGI219451[] = {{&S81368[82], 6}, {NULL}};
const Segment SGI219452[] = {{&S81368[109], 13}, {NULL}};
const Segment SGI219453[] = {{&S73520[34], 5}, {NULL}};
const Segment SGI219454[] = {{&S73520[27], 6}, {NULL}};
const Segment SGI219455[] = {{&S73520[92], 9}, {NULL}};
const Segment SGI219456[] = {{&S73520[65], 6}, {NULL}};
const Segment SGI219457[] = {{&S92197[45], 9}, {NULL}};
const Segment SGI219458[] = {{&S92197[30], 6}, {NULL}};
const Segment SGI219459[] = {{&S92197[31], 3}, {NULL}};
const Segment SGI219460[] = {{&S92197[80], 5}, {NULL}};
const Segment SGI219461[] = {{&S92197[138], 13}, {NULL}};
const Segment SGI219462[] = {{&S92200[44], 10}, {NULL}};
const Segment SGI219463[] = {{&S92200[44], 22}, {NULL}};
const Segment SGI219464[] = {{&S92200[30], 6}, {NULL}};
const Segment SGI219465[] = {{&S92200[80], 6}, {NULL}};
const Segment SGI219466[] = {{&S92200[107], 13}, {NULL}};
const Segment SGI219467[] = {{&S92204[34], 11}, {NULL}};
const Segment SGI219468[] = {{&S92204[27], 6}, {NULL}};
const Segment SGI219469[] = {{&S92204[98], 9}, {NULL}};
const Segment SGI219470[] = {{&S92204[71], 6}, {NULL}};
const Segment SGI219471[] = {{&S92205[34], 8}, {NULL}};
const Segment SGI219472[] = {{&S92205[27], 6}, {NULL}};
const Segment SGI219473[] = {{&S92205[95], 9}, {NULL}};
const Segment SGI219474[] = {{&S92205[68], 6}, {NULL}};
const Segment SGI219475[] = {{&S73545[44], 12}, {NULL}};
const Segment SGI219476[] = {{&S73545[44], 23}, {NULL}};
const Segment SGI219477[] = {{&S73545[30], 6}, {NULL}};
const Segment SGI219478[] = {{&S73545[116], 3}, {NULL}};
const Segment SGI219479[] = {{&S73545[81], 6}, {NULL}};
const Segment SGI219480[] = {{&S92215[44], 13}, {NULL}};
const Segment SGI219481[] = {{&S73546[38], 12}, {NULL}};
const Segment SGI219482[] = {{&S92215[44], 25}, {NULL}};
const Segment SGI219483[] = {{&S73546[31], 6}, {NULL}};
const Segment SGI219484[] = {{&S92215[30], 6}, {NULL}};
const Segment SGI219485[] = {{&S92215[83], 6}, {NULL}};
const Segment SGI219486[] = {{&S92215[110], 13}, {NULL}};
const Segment SGI219487[] = {{&S92216[44], 5}, {NULL}};
const Segment SGI219488[] = {{&S92216[44], 17}, {NULL}};
const Segment SGI219489[] = {{&S92216[30], 6}, {NULL}};
const Segment SGI219490[] = {{&S73548[34], 10}, {NULL}};
const Segment SGI219491[] = {{&S92216[31], 3}, {NULL}};
const Segment SGI219492[] = {{&S73548[27], 6}, {NULL}};
const Segment SGI219493[] = {{&S92216[75], 5}, {NULL}};
const Segment SGI219494[] = {{&S92216[112], 14}, {NULL}};
const Segment SGI219495[] = {{&S73548[96], 9}, {NULL}};
const Segment SGI219496[] = {{&S73548[69], 6}, {NULL}};
const Segment SGI219497[] = {{&S92217[34], 12}, {NULL}};
const Segment SGI219498[] = {{&S92217[27], 6}, {NULL}};
const Segment SGI219499[] = {{&S92217[99], 9}, {NULL}};
const Segment SGI219500[] = {{&S92217[72], 6}, {NULL}};
const Segment SGI219501[] = {{&S73550[34], 10}, {NULL}};
const Segment SGI219502[] = {{&S73550[27], 6}, {NULL}};
const Segment SGI219503[] = {{&S73550[96], 9}, {NULL}};
const Segment SGI219504[] = {{&S73550[69], 6}, {NULL}};
const Segment SGI219505[] = {{&S82662[34], 14}, {NULL}};
const Segment SGI219506[] = {{&S82662[27], 6}, {NULL}};
const Segment SGI219507[] = {{&S82662[101], 9}, {NULL}};
const Segment SGI219508[] = {{&S82662[74], 6}, {NULL}};
const Segment SGI219509[] = {{&S92223[34], 10}, {NULL}};
const Segment SGI219510[] = {{&S92223[27], 6}, {NULL}};
const Segment SGI219511[] = {{&S92223[97], 9}, {NULL}};
const Segment SGI219512[] = {{&S92223[70], 6}, {NULL}};
const Segment SGI219513[] = {{&S81410[44], 6}, {NULL}};
const Segment SGI219514[] = {{&S81410[44], 18}, {NULL}};
const Segment SGI219515[] = {{&S81410[30], 6}, {NULL}};
const Segment SGI219516[] = {{&S81410[111], 3}, {NULL}};
const Segment SGI219517[] = {{&S81410[76], 6}, {NULL}};
const Segment SGI219518[] = {{&S92226[38], 9}, {NULL}};
const Segment SGI219519[] = {{&S92226[31], 6}, {NULL}};
const Segment SGI219520[] = {{&S92227[44], 9}, {NULL}};
const Segment SGI219521[] = {{&S92227[44], 21}, {NULL}};
const Segment SGI219522[] = {{&S92227[30], 6}, {NULL}};
const Segment SGI219523[] = {{&S92227[31], 3}, {NULL}};
const Segment SGI219524[] = {{&S81414[34], 10}, {NULL}};
const Segment SGI219525[] = {{&S92227[79], 6}, {NULL}};
const Segment SGI219526[] = {{&S81414[27], 6}, {NULL}};
const Segment SGI219527[] = {{&S81414[97], 9}, {NULL}};
const Segment SGI219528[] = {{&S81414[70], 6}, {NULL}};
const Segment SGI219529[] = {{&S92232[44], 9}, {NULL}};
const Segment SGI219530[] = {{&S92232[44], 21}, {NULL}};
const Segment SGI219531[] = {{&S92232[30], 6}, {NULL}};
const Segment SGI219532[] = {{&S92232[79], 6}, {NULL}};
const Segment SGI219533[] = {{&S92232[106], 13}, {NULL}};
const Segment SGI219534[] = {{&S81424[34], 7}, {NULL}};
const Segment SGI219535[] = {{&S81424[27], 6}, {NULL}};
const Segment SGI219536[] = {{&S81424[99], 9}, {NULL}};
const Segment SGI219537[] = {{&S81424[72], 6}, {NULL}};
const Segment SGI219538[] = {{&S92238[52], 7}, {NULL}};
const Segment SGI219539[] = {{&S92238[52], 24}, {NULL}};
const Segment SGI219540[] = {{&S92238[38], 6}, {NULL}};
const Segment SGI219541[] = {{&S92238[90], 5}, {NULL}};
const Segment SGI219542[] = {{&S92238[0], 14}, {NULL}};
const Segment SGI219543[] = {{&S81430[34], 7}, {NULL}};
const Segment SGI219544[] = {{&S81430[27], 6}, {NULL}};
const Segment SGI219545[] = {{&S81430[94], 9}, {NULL}};
const Segment SGI219546[] = {{&S81430[67], 6}, {NULL}};
const Segment SGI219547[] = {{&S92245[34], 9}, {NULL}};
const Segment SGI219548[] = {{&S92245[27], 6}, {NULL}};
const Segment SGI219549[] = {{&S92245[96], 9}, {NULL}};
const Segment SGI219550[] = {{&S92245[69], 6}, {NULL}};
const Segment SGI219551[] = {{&S92245[137], 8}, {NULL}};
const Segment SGI219552[] = {{&S81435[34], 10}, {NULL}};
const Segment SGI219553[] = {{&S81435[27], 6}, {NULL}};
const Segment SGI219554[] = {{&S81435[97], 9}, {NULL}};
const Segment SGI219555[] = {{&S81435[70], 6}, {NULL}};
const Segment SGI219556[] = {{&S81438[34], 14}, {NULL}};
const Segment SGI219557[] = {{&S81438[27], 6}, {NULL}};
const Segment SGI219558[] = {{&S81438[111], 9}, {NULL}};
const Segment SGI219559[] = {{&S81438[84], 6}, {NULL}};
const Segment SGI219560[] = {{&S73595[34], 6}, {NULL}};
const Segment SGI219561[] = {{&S73595[27], 6}, {NULL}};
const Segment SGI219562[] = {{&S73595[92], 9}, {NULL}};
const Segment SGI219563[] = {{&S73595[65], 6}, {NULL}};
const Segment SGI219564[] = {{&S73601[34], 10}, {NULL}};
const Segment SGI219565[] = {{&S73601[27], 6}, {NULL}};
const Segment SGI219566[] = {{&S73601[96], 9}, {NULL}};
const Segment SGI219567[] = {{&S73601[69], 6}, {NULL}};
const Segment SGI219568[] = {{&S92271[34], 10}, {NULL}};
const Segment SGI219569[] = {{&S92271[27], 6}, {NULL}};
const Segment SGI219570[] = {{&S92271[97], 9}, {NULL}};
const Segment SGI219571[] = {{&S92271[70], 6}, {NULL}};
const Segment SGI219572[] = {{&S82726[34], 4}, {NULL}};
const Segment SGI219573[] = {{&S82726[27], 6}, {NULL}};
const Segment SGI219574[] = {{&S82726[91], 9}, {NULL}};
const Segment SGI219575[] = {{&S82726[64], 6}, {NULL}};
const Segment SGI219576[] = {{&S82730[34], 6}, {NULL}};
const Segment SGI219577[] = {{&S82730[27], 6}, {NULL}};
const Segment SGI219578[] = {{&S82730[93], 9}, {NULL}};
const Segment SGI219579[] = {{&S82730[66], 6}, {NULL}};
const Segment SGI219580[] = {{&S73626[44], 10}, {NULL}};
const Segment SGI219581[] = {{&S73626[44], 21}, {NULL}};
const Segment SGI219582[] = {{&S73626[30], 6}, {NULL}};
const Segment SGI219583[] = {{&S73626[79], 6}, {NULL}};
const Segment SGI219584[] = {{&S73626[106], 13}, {NULL}};
const Segment SGI219585[] = {{&S81476[44], 10}, {NULL}};
const Segment SGI219586[] = {{&S81476[44], 22}, {NULL}};
const Segment SGI219587[] = {{&S81476[30], 6}, {NULL}};
const Segment SGI219588[] = {{&S81476[80], 6}, {NULL}};
const Segment SGI219589[] = {{&S81477[44], 10}, {NULL}};
const Segment SGI219590[] = {{&S81477[44], 22}, {NULL}};
const Segment SGI219591[] = {{&S81477[30], 6}, {NULL}};
const Segment SGI219592[] = {{&S81477[115], 3}, {NULL}};
const Segment SGI219593[] = {{&S81477[80], 6}, {NULL}};
const Segment SGI219594[] = {{&S81478[38], 10}, {NULL}};
const Segment SGI219595[] = {{&S81478[31], 6}, {NULL}};
const Segment SGI219596[] = {{&S81493[34], 16}, {NULL}};
const Segment SGI219597[] = {{&S81493[27], 6}, {NULL}};
const Segment SGI219598[] = {{&S81493[110], 9}, {NULL}};
const Segment SGI219599[] = {{&S81493[83], 6}, {NULL}};
const Segment SGI219600[] = {{&S81496[34], 4}, {NULL}};
const Segment SGI219601[] = {{&S81496[27], 6}, {NULL}};
const Segment SGI219602[] = {{&S81496[91], 9}, {NULL}};
const Segment SGI219603[] = {{&S81496[64], 6}, {NULL}};
const Segment SGI219604[] = {{&S81497[44], 13}, {NULL}};
const Segment SGI219605[] = {{&S81497[44], 25}, {NULL}};
const Segment SGI219606[] = {{&S81497[30], 6}, {NULL}};
const Segment SGI219607[] = {{&S81497[118], 3}, {NULL}};
const Segment SGI219608[] = {{&S81497[83], 6}, {NULL}};
const Segment SGI219609[] = {{&S81498[44], 10}, {NULL}};
const Segment SGI219610[] = {{&S81498[44], 22}, {NULL}};
const Segment SGI219611[] = {{&S81498[30], 6}, {NULL}};
const Segment SGI219612[] = {{&S81498[80], 6}, {NULL}};
const Segment SGI219613[] = {{&S81498[107], 13}, {NULL}};
const Segment SGI219614[] = {{&S81499[34], 6}, {NULL}};
const Segment SGI219615[] = {{&S92324[44], 11}, {NULL}};
const Segment SGI219616[] = {{&S81499[27], 6}, {NULL}};
const Segment SGI219617[] = {{&S92324[44], 23}, {NULL}};
const Segment SGI219618[] = {{&S92324[30], 6}, {NULL}};
const Segment SGI219619[] = {{&S81499[93], 9}, {NULL}};
const Segment SGI219620[] = {{&S92324[31], 3}, {NULL}};
const Segment SGI219621[] = {{&S81499[66], 6}, {NULL}};
const Segment SGI219622[] = {{&S92324[96], 6}, {NULL}};
const Segment SGI219623[] = {{&S92324[69], 8}, {NULL}};
const Segment SGI219624[] = {{&S81500[34], 11}, {NULL}};
const Segment SGI219625[] = {{&S81500[27], 6}, {NULL}};
const Segment SGI219626[] = {{&S92325[44], 11}, {NULL}};
const Segment SGI219627[] = {{&S92325[44], 23}, {NULL}};
const Segment SGI219628[] = {{&S81500[98], 9}, {NULL}};
const Segment SGI219629[] = {{&S92325[30], 6}, {NULL}};
const Segment SGI219630[] = {{&S81500[71], 6}, {NULL}};
const Segment SGI219631[] = {{&S92325[31], 3}, {NULL}};
const Segment SGI219632[] = {{&S92325[100], 6}, {NULL}};
const Segment SGI219633[] = {{&S92325[69], 8}, {NULL}};
const Segment SGI219634[] = {{&S81501[34], 11}, {NULL}};
const Segment SGI219635[] = {{&S81501[27], 6}, {NULL}};
const Segment SGI219636[] = {{&S81501[98], 9}, {NULL}};
const Segment SGI219637[] = {{&S81501[71], 6}, {NULL}};
const Segment SGI219638[] = {{&S87586[34], 7}, {NULL}};
const Segment SGI219639[] = {{&S87586[27], 6}, {NULL}};
const Segment SGI219640[] = {{&S87586[94], 9}, {NULL}};
const Segment SGI219641[] = {{&S87586[67], 6}, {NULL}};
const Segment SGI219642[] = {{&S81506[44], 4}, {NULL}};
const Segment SGI219643[] = {{&S81506[44], 16}, {NULL}};
const Segment SGI219644[] = {{&S81506[30], 6}, {NULL}};
const Segment SGI219645[] = {{&S81506[109], 3}, {NULL}};
const Segment SGI219646[] = {{&S81506[74], 6}, {NULL}};
const Segment SGI219647[] = {{&S87588[59], 6}, {NULL}};
const Segment SGI219648[] = {{&S87588[59], 18}, {NULL}};
const Segment SGI219649[] = {{&S87588[45], 6}, {NULL}};
const Segment SGI219650[] = {{&S87588[126], 3}, {NULL}};
const Segment SGI219651[] = {{&S87588[91], 6}, {NULL}};
const Segment SGI219652[] = {{&S87588[0], 14}, {NULL}};
const Segment SGI219653[] = {{&S81510[44], 10}, {NULL}};
const Segment SGI219654[] = {{&S81510[44], 22}, {NULL}};
const Segment SGI219655[] = {{&S81510[30], 6}, {NULL}};
const Segment SGI219656[] = {{&S81510[115], 3}, {NULL}};
const Segment SGI219657[] = {{&S81510[80], 6}, {NULL}};
const Segment SGI219658[] = {{&S81511[44], 10}, {NULL}};
const Segment SGI219659[] = {{&S81511[44], 22}, {NULL}};
const Segment SGI219660[] = {{&S81511[30], 6}, {NULL}};
const Segment SGI219661[] = {{&S81511[83], 6}, {NULL}};
const Segment SGI219662[] = {{&S92339[34], 15}, {NULL}};
const Segment SGI219663[] = {{&S92339[27], 6}, {NULL}};
const Segment SGI219664[] = {{&S92339[102], 9}, {NULL}};
const Segment SGI219665[] = {{&S92339[75], 6}, {NULL}};
const Segment SGI219666[] = {{&S81512[34], 12}, {NULL}};
const Segment SGI219667[] = {{&S81512[27], 6}, {NULL}};
const Segment SGI219668[] = {{&S81512[99], 9}, {NULL}};
const Segment SGI219669[] = {{&S81512[72], 6}, {NULL}};
const Segment SGI219670[] = {{&S81512[140], 8}, {NULL}};
const Segment SGI219671[] = {{&S82795[34], 4}, {NULL}};
const Segment SGI219672[] = {{&S82795[27], 6}, {NULL}};
const Segment SGI219673[] = {{&S82795[91], 9}, {NULL}};
const Segment SGI219674[] = {{&S82795[64], 6}, {NULL}};
const Segment SGI219675[] = {{&S92347[34], 25}, {NULL}};
const Segment SGI219676[] = {{&S92347[27], 6}, {NULL}};
const Segment SGI219677[] = {{&S92347[112], 9}, {NULL}};
const Segment SGI219678[] = {{&S92347[85], 6}, {NULL}};
const Segment SGI219679[] = {{&S92349[44], 8}, {NULL}};
const Segment SGI219680[] = {{&S92349[44], 20}, {NULL}};
const Segment SGI219681[] = {{&S92349[30], 6}, {NULL}};
const Segment SGI219682[] = {{&S92349[31], 3}, {NULL}};
const Segment SGI219683[] = {{&S92349[97], 6}, {NULL}};
const Segment SGI219684[] = {{&S92349[66], 8}, {NULL}};
const Segment SGI219685[] = {{&S81528[34], 9}, {NULL}};
const Segment SGI219686[] = {{&S81528[27], 6}, {NULL}};
const Segment SGI219687[] = {{&S81528[96], 9}, {NULL}};
const Segment SGI219688[] = {{&S81528[69], 6}, {NULL}};
const Segment SGI219689[] = {{&S92355[44], 7}, {NULL}};
const Segment SGI219690[] = {{&S92355[44], 19}, {NULL}};
const Segment SGI219691[] = {{&S92355[30], 6}, {NULL}};
const Segment SGI219692[] = {{&S92355[31], 3}, {NULL}};
const Segment SGI219693[] = {{&S92355[77], 6}, {NULL}};
const Segment SGI219694[] = {{&S92356[34], 13}, {NULL}};
const Segment SGI219695[] = {{&S92356[27], 6}, {NULL}};
const Segment SGI219696[] = {{&S92356[100], 9}, {NULL}};
const Segment SGI219697[] = {{&S92356[73], 6}, {NULL}};
const Segment SGI219698[] = {{&S81548[34], 7}, {NULL}};
const Segment SGI219699[] = {{&S81548[27], 6}, {NULL}};
const Segment SGI219700[] = {{&S81548[95], 9}, {NULL}};
const Segment SGI219701[] = {{&S81548[68], 6}, {NULL}};
const Segment SGI219702[] = {{&S92373[44], 7}, {NULL}};
const Segment SGI219703[] = {{&S92373[44], 19}, {NULL}};
const Segment SGI219704[] = {{&S92373[30], 6}, {NULL}};
const Segment SGI219705[] = {{&S92373[31], 3}, {NULL}};
const Segment SGI219706[] = {{&S92373[77], 6}, {NULL}};
const Segment SGI219707[] = {{&S92375[45], 15}, {NULL}};
const Segment SGI219708[] = {{&S92375[30], 6}, {NULL}};
const Segment SGI219709[] = {{&S92375[31], 3}, {NULL}};
const Segment SGI219710[] = {{&S92375[101], 5}, {NULL}};
const Segment SGI219711[] = {{&S92375[159], 13}, {NULL}};
const Segment SGI219712[] = {{&S73723[44], 15}, {NULL}};
const Segment SGI219713[] = {{&S73723[44], 26}, {NULL}};
const Segment SGI219714[] = {{&S73723[30], 6}, {NULL}};
const Segment SGI219715[] = {{&S73723[119], 3}, {NULL}};
const Segment SGI219716[] = {{&S73723[84], 6}, {NULL}};
const Segment SGI219717[] = {{&S92376[45], 5}, {NULL}};
const Segment SGI219718[] = {{&S92376[30], 6}, {NULL}};
const Segment SGI219719[] = {{&S92376[31], 3}, {NULL}};
const Segment SGI219720[] = {{&S92376[76], 5}, {NULL}};
const Segment SGI219721[] = {{&S73724[38], 15}, {NULL}};
const Segment SGI219722[] = {{&S73724[31], 6}, {NULL}};
const Segment SGI219723[] = {{&S92376[134], 13}, {NULL}};
const Segment SGI219724[] = {{&S73725[44], 15}, {NULL}};
const Segment SGI219725[] = {{&S73725[44], 27}, {NULL}};
const Segment SGI219726[] = {{&S73725[30], 6}, {NULL}};
const Segment SGI219727[] = {{&S73725[120], 3}, {NULL}};
const Segment SGI219728[] = {{&S73725[85], 6}, {NULL}};
const Segment SGI219729[] = {{&S92378[34], 10}, {NULL}};
const Segment SGI219730[] = {{&S92378[27], 6}, {NULL}};
const Segment SGI219731[] = {{&S92378[97], 9}, {NULL}};
const Segment SGI219732[] = {{&S81558[44], 10}, {NULL}};
const Segment SGI219733[] = {{&S92378[70], 6}, {NULL}};
const Segment SGI219734[] = {{&S81558[44], 22}, {NULL}};
const Segment SGI219735[] = {{&S81558[30], 6}, {NULL}};
const Segment SGI219736[] = {{&S81558[113], 3}, {NULL}};
const Segment SGI219737[] = {{&S81558[80], 5}, {NULL}};
const Segment SGI219738[] = {{&S81559[44], 11}, {NULL}};
const Segment SGI219739[] = {{&S81559[44], 23}, {NULL}};
const Segment SGI219740[] = {{&S81559[30], 6}, {NULL}};
const Segment SGI219741[] = {{&S81559[114], 3}, {NULL}};
const Segment SGI219742[] = {{&S81559[81], 5}, {NULL}};
const Segment SGI219743[] = {{&S92382[44], 10}, {NULL}};
const Segment SGI219744[] = {{&S92382[44], 22}, {NULL}};
const Segment SGI219745[] = {{&S92382[30], 6}, {NULL}};
const Segment SGI219746[] = {{&S87646[34], 15}, {NULL}};
const Segment SGI219747[] = {{&S92382[31], 3}, {NULL}};
const Segment SGI219748[] = {{&S87646[27], 6}, {NULL}};
const Segment SGI219749[] = {{&S92382[80], 5}, {NULL}};
const Segment SGI219750[] = {{&S92382[117], 14}, {NULL}};
const Segment SGI219751[] = {{&S87646[102], 9}, {NULL}};
const Segment SGI219752[] = {{&S87646[75], 6}, {NULL}};
const Segment SGI219753[] = {{&S87648[34], 7}, {NULL}};
const Segment SGI219754[] = {{&S87648[27], 6}, {NULL}};
const Segment SGI219755[] = {{&S81564[44], 10}, {NULL}};
const Segment SGI219756[] = {{&S81564[44], 22}, {NULL}};
const Segment SGI219757[] = {{&S81564[30], 6}, {NULL}};
const Segment SGI219758[] = {{&S87648[94], 9}, {NULL}};
const Segment SGI219759[] = {{&S87648[67], 6}, {NULL}};
const Segment SGI219760[] = {{&S81564[115], 3}, {NULL}};
const Segment SGI219761[] = {{&S81564[80], 6}, {NULL}};
const Segment SGI219762[] = {{&S92386[34], 7}, {NULL}};
const Segment SGI219763[] = {{&S92386[27], 6}, {NULL}};
const Segment SGI219764[] = {{&S92386[94], 9}, {NULL}};
const Segment SGI219765[] = {{&S92386[67], 6}, {NULL}};
const Segment SGI219766[] = {{&S73737[34], 11}, {NULL}};
const Segment SGI219767[] = {{&S73737[27], 6}, {NULL}};
const Segment SGI219768[] = {{&S81567[34], 10}, {NULL}};
const Segment SGI219769[] = {{&S81567[27], 6}, {NULL}};
const Segment SGI219770[] = {{&S73737[98], 9}, {NULL}};
const Segment SGI219771[] = {{&S73737[71], 6}, {NULL}};
const Segment SGI219772[] = {{&S81567[97], 9}, {NULL}};
const Segment SGI219773[] = {{&S81567[70], 6}, {NULL}};
const Segment SGI219774[] = {{&S81568[44], 16}, {NULL}};
const Segment SGI219775[] = {{&S81568[44], 28}, {NULL}};
const Segment SGI219776[] = {{&S81568[30], 6}, {NULL}};
const Segment SGI219777[] = {{&S92390[34], 10}, {NULL}};
const Segment SGI219778[] = {{&S92390[27], 6}, {NULL}};
const Segment SGI219779[] = {{&S81568[121], 3}, {NULL}};
const Segment SGI219780[] = {{&S81568[86], 6}, {NULL}};
const Segment SGI219781[] = {{&S92390[97], 9}, {NULL}};
const Segment SGI219782[] = {{&S92390[70], 6}, {NULL}};
const Segment SGI219783[] = {{&S87656[44], 14}, {NULL}};
const Segment SGI219784[] = {{&S87656[44], 31}, {NULL}};
const Segment SGI219785[] = {{&S87656[30], 6}, {NULL}};
const Segment SGI219786[] = {{&S87656[89], 6}, {NULL}};
const Segment SGI219787[] = {{&S87656[116], 13}, {NULL}};
const Segment SGI219788[] = {{&S81572[34], 7}, {NULL}};
const Segment SGI219789[] = {{&S81572[27], 6}, {NULL}};
const Segment SGI219790[] = {{&S81572[99], 9}, {NULL}};
const Segment SGI219791[] = {{&S81572[72], 6}, {NULL}};
const Segment SGI219792[] = {{&S81572[140], 8}, {NULL}};
const Segment SGI219793[] = {{&S92395[34], 7}, {NULL}};
const Segment SGI219794[] = {{&S92395[27], 6}, {NULL}};
const Segment SGI219795[] = {{&S92395[94], 9}, {NULL}};
const Segment SGI219796[] = {{&S92395[67], 6}, {NULL}};
const Segment SGI219797[] = {{&S81573[34], 7}, {NULL}};
const Segment SGI219798[] = {{&S81573[27], 6}, {NULL}};
const Segment SGI219799[] = {{&S81573[94], 9}, {NULL}};
const Segment SGI219800[] = {{&S81573[67], 6}, {NULL}};
const Segment SGI219801[] = {{&S92398[34], 11}, {NULL}};
const Segment SGI219802[] = {{&S92398[27], 6}, {NULL}};
const Segment SGI219803[] = {{&S92398[98], 9}, {NULL}};
const Segment SGI219804[] = {{&S92398[71], 6}, {NULL}};
const Segment SGI219805[] = {{&S81576[34], 11}, {NULL}};
const Segment SGI219806[] = {{&S81576[27], 6}, {NULL}};
const Segment SGI219807[] = {{&S81576[98], 9}, {NULL}};
const Segment SGI219808[] = {{&S81576[71], 6}, {NULL}};
const Segment SGI219809[] = {{&S92399[44], 7}, {NULL}};
const Segment SGI219810[] = {{&S92399[44], 19}, {NULL}};
const Segment SGI219811[] = {{&S92399[30], 6}, {NULL}};
const Segment SGI219812[] = {{&S92399[31], 3}, {NULL}};
const Segment SGI219813[] = {{&S92399[96], 6}, {NULL}};
const Segment SGI219814[] = {{&S92399[65], 8}, {NULL}};
const Segment SGI219815[] = {{&S92401[44], 17}, {NULL}};
const Segment SGI219816[] = {{&S92401[44], 29}, {NULL}};
const Segment SGI219817[] = {{&S92401[30], 6}, {NULL}};
const Segment SGI219818[] = {{&S92401[31], 3}, {NULL}};
const Segment SGI219819[] = {{&S92401[87], 6}, {NULL}};
const Segment SGI219820[] = {{&S92412[34], 9}, {NULL}};
const Segment SGI219821[] = {{&S92412[27], 6}, {NULL}};
const Segment SGI219822[] = {{&S92412[96], 9}, {NULL}};
const Segment SGI219823[] = {{&S92412[69], 6}, {NULL}};
const Segment SGI219824[] = {{&S92421[59], 15}, {NULL}};
const Segment SGI219825[] = {{&S92421[59], 33}, {NULL}};
const Segment SGI219826[] = {{&S92421[45], 6}, {NULL}};
const Segment SGI219827[] = {{&S92421[46], 3}, {NULL}};
const Segment SGI219828[] = {{&S92421[106], 6}, {NULL}};
const Segment SGI219829[] = {{&S92421[0], 14}, {NULL}};
const Segment SGI219830[] = {{&S92435[34], 13}, {NULL}};
const Segment SGI219831[] = {{&S92435[27], 6}, {NULL}};
const Segment SGI219832[] = {{&S92435[100], 9}, {NULL}};
const Segment SGI219833[] = {{&S92435[73], 6}, {NULL}};
const Segment SGI219834[] = {{&S92436[44], 7}, {NULL}};
const Segment SGI219835[] = {{&S92436[44], 24}, {NULL}};
const Segment SGI219836[] = {{&S92436[30], 6}, {NULL}};
const Segment SGI219837[] = {{&S92436[82], 6}, {NULL}};
const Segment SGI219838[] = {{&S92436[109], 13}, {NULL}};
const Segment SGI219839[] = {{&S87708[34], 11}, {NULL}};
const Segment SGI219840[] = {{&S87708[27], 6}, {NULL}};
const Segment SGI219841[] = {{&S87708[120], 9}, {NULL}};
const Segment SGI219842[] = {{&S87708[93], 6}, {NULL}};
const Segment SGI219843[] = {{&S81621[44], 7}, {NULL}};
const Segment SGI219844[] = {{&S81621[44], 19}, {NULL}};
const Segment SGI219845[] = {{&S81621[30], 6}, {NULL}};
const Segment SGI219846[] = {{&S81621[128], 3}, {NULL}};
const Segment SGI219847[] = {{&S81621[93], 6}, {NULL}};
const Segment SGI219848[] = {{&S81621[65], 8}, {NULL}};
const Segment SGI219849[] = {{&S81624[34], 11}, {NULL}};
const Segment SGI219850[] = {{&S81624[27], 6}, {NULL}};
const Segment SGI219851[] = {{&S81624[98], 9}, {NULL}};
const Segment SGI219852[] = {{&S81624[71], 6}, {NULL}};
const Segment SGI219853[] = {{&S92444[34], 18}, {NULL}};
const Segment SGI219854[] = {{&S92444[27], 6}, {NULL}};
const Segment SGI219855[] = {{&S81625[34], 11}, {NULL}};
const Segment SGI219856[] = {{&S81625[27], 6}, {NULL}};
const Segment SGI219857[] = {{&S92444[105], 9}, {NULL}};
const Segment SGI219858[] = {{&S92444[78], 6}, {NULL}};
const Segment SGI219859[] = {{&S81625[98], 9}, {NULL}};
const Segment SGI219860[] = {{&S81625[71], 6}, {NULL}};
const Segment SGI219861[] = {{&S81626[34], 11}, {NULL}};
const Segment SGI219862[] = {{&S81626[27], 6}, {NULL}};
const Segment SGI219863[] = {{&S81626[98], 9}, {NULL}};
const Segment SGI219864[] = {{&S81626[71], 6}, {NULL}};
const Segment SGI219865[] = {{&S92453[44], 6}, {NULL}};
const Segment SGI219866[] = {{&S92453[44], 18}, {NULL}};
const Segment SGI219867[] = {{&S92453[30], 6}, {NULL}};
const Segment SGI219868[] = {{&S87723[44], 13}, {NULL}};
const Segment SGI219869[] = {{&S87723[44], 25}, {NULL}};
const Segment SGI219870[] = {{&S87723[30], 6}, {NULL}};
const Segment SGI219871[] = {{&S92453[76], 6}, {NULL}};
const Segment SGI219872[] = {{&S87723[118], 3}, {NULL}};
const Segment SGI219873[] = {{&S87723[83], 6}, {NULL}};
const Segment SGI219874[] = {{&S92453[103], 13}, {NULL}};
const Segment SGI219875[] = {{&S87724[44], 13}, {NULL}};
const Segment SGI219876[] = {{&S87724[44], 25}, {NULL}};
const Segment SGI219877[] = {{&S87724[30], 6}, {NULL}};
const Segment SGI219878[] = {{&S87724[118], 3}, {NULL}};
const Segment SGI219879[] = {{&S87724[83], 6}, {NULL}};
const Segment SGI219880[] = {{&S92458[34], 11}, {NULL}};
const Segment SGI219881[] = {{&S92458[27], 6}, {NULL}};
const Segment SGI219882[] = {{&S92458[98], 9}, {NULL}};
const Segment SGI219883[] = {{&S92458[71], 6}, {NULL}};
const Segment SGI219884[] = {{&S92460[34], 11}, {NULL}};
const Segment SGI219885[] = {{&S92460[27], 6}, {NULL}};
const Segment SGI219886[] = {{&S92460[98], 9}, {NULL}};
const Segment SGI219887[] = {{&S92460[71], 6}, {NULL}};
const Segment SGI219888[] = {{&S81641[34], 10}, {NULL}};
const Segment SGI219889[] = {{&S81641[27], 6}, {NULL}};
const Segment SGI219890[] = {{&S81641[97], 9}, {NULL}};
const Segment SGI219891[] = {{&S81641[70], 6}, {NULL}};
const Segment SGI219892[] = {{&S92461[34], 7}, {NULL}};
const Segment SGI219893[] = {{&S92461[27], 6}, {NULL}};
const Segment SGI219894[] = {{&S92461[98], 9}, {NULL}};
const Segment SGI219895[] = {{&S92461[71], 6}, {NULL}};
const Segment SGI219896[] = {{&S81642[34], 25}, {NULL}};
const Segment SGI219897[] = {{&S92461[139], 8}, {NULL}};
const Segment SGI219898[] = {{&S81642[27], 6}, {NULL}};
const Segment SGI219899[] = {{&S81642[112], 9}, {NULL}};
const Segment SGI219900[] = {{&S81642[85], 6}, {NULL}};
const Segment SGI219901[] = {{&S92462[34], 11}, {NULL}};
const Segment SGI219902[] = {{&S92462[27], 6}, {NULL}};
const Segment SGI219903[] = {{&S92462[98], 9}, {NULL}};
const Segment SGI219904[] = {{&S92462[71], 6}, {NULL}};
const Segment SGI219905[] = {{&S81651[34], 10}, {NULL}};
const Segment SGI219906[] = {{&S81651[27], 6}, {NULL}};
const Segment SGI219907[] = {{&S81651[97], 9}, {NULL}};
const Segment SGI219908[] = {{&S81651[70], 6}, {NULL}};
const Segment SGI219909[] = {{&S81651[138], 8}, {NULL}};
const Segment SGI219910[] = {{&S87746[34], 25}, {NULL}};
const Segment SGI219911[] = {{&S87746[27], 6}, {NULL}};
const Segment SGI219912[] = {{&S87746[112], 9}, {NULL}};
const Segment SGI219913[] = {{&S87746[85], 6}, {NULL}};
const Segment SGI219914[] = {{&S87747[34], 25}, {NULL}};
const Segment SGI219915[] = {{&S87747[27], 6}, {NULL}};
const Segment SGI219916[] = {{&S87747[112], 9}, {NULL}};
const Segment SGI219917[] = {{&S87747[85], 6}, {NULL}};
const Segment SGI219918[] = {{&S87748[34], 25}, {NULL}};
const Segment SGI219919[] = {{&S87748[27], 6}, {NULL}};
const Segment SGI219920[] = {{&S87748[112], 9}, {NULL}};
const Segment SGI219921[] = {{&S87748[85], 6}, {NULL}};
const Segment SGI219922[] = {{&S87763[44], 13}, {NULL}};
const Segment SGI219923[] = {{&S87763[44], 25}, {NULL}};
const Segment SGI219924[] = {{&S87763[30], 6}, {NULL}};
const Segment SGI219925[] = {{&S87763[83], 6}, {NULL}};
const Segment SGI219926[] = {{&S87763[110], 13}, {NULL}};
const Segment SGI219927[] = {{&S87773[34], 11}, {NULL}};
const Segment SGI219928[] = {{&S87773[27], 6}, {NULL}};
const Segment SGI219929[] = {{&S87773[98], 9}, {NULL}};
const Segment SGI219930[] = {{&S87773[71], 6}, {NULL}};
const Segment SGI219931[] = {{&S92513[44], 8}, {NULL}};
const Segment SGI219932[] = {{&S92513[44], 20}, {NULL}};
const Segment SGI219933[] = {{&S92513[30], 6}, {NULL}};
const Segment SGI219934[] = {{&S92513[31], 3}, {NULL}};
const Segment SGI219935[] = {{&S92513[78], 6}, {NULL}};
const Segment SGI219936[] = {{&S92531[44], 13}, {NULL}};
const Segment SGI219937[] = {{&S92531[44], 25}, {NULL}};
const Segment SGI219938[] = {{&S92531[30], 6}, {NULL}};
const Segment SGI219939[] = {{&S92531[31], 3}, {NULL}};
const Segment SGI219940[] = {{&S92531[137], 6}, {NULL}};
const Segment SGI219941[] = {{&S92531[71], 8}, {NULL}};
const Segment SGI219942[] = {{&S92532[34], 10}, {NULL}};
const Segment SGI219943[] = {{&S92532[27], 6}, {NULL}};
const Segment SGI219944[] = {{&S92532[97], 9}, {NULL}};
const Segment SGI219945[] = {{&S92532[70], 6}, {NULL}};
const Segment SGI219946[] = {{&S92537[44], 7}, {NULL}};
const Segment SGI219947[] = {{&S92537[44], 19}, {NULL}};
const Segment SGI219948[] = {{&S92537[30], 6}, {NULL}};
const Segment SGI219949[] = {{&S92537[108], 2}, {NULL}};
const Segment SGI219950[] = {{&S92537[77], 5}, {NULL}};
const Segment SGI219951[] = {{&S92539[34], 14}, {NULL}};
const Segment SGI219952[] = {{&S92539[27], 6}, {NULL}};
const Segment SGI219953[] = {{&S92539[101], 9}, {NULL}};
const Segment SGI219954[] = {{&S92539[74], 6}, {NULL}};
const Segment SGI219955[] = {{&S81717[44], 7}, {NULL}};
const Segment SGI219956[] = {{&S81717[44], 23}, {NULL}};
const Segment SGI219957[] = {{&S81717[30], 6}, {NULL}};
const Segment SGI219958[] = {{&S92541[34], 10}, {NULL}};
const Segment SGI219959[] = {{&S81717[116], 3}, {NULL}};
const Segment SGI219960[] = {{&S92541[27], 6}, {NULL}};
const Segment SGI219961[] = {{&S81717[81], 6}, {NULL}};
const Segment SGI219962[] = {{&S92541[97], 9}, {NULL}};
const Segment SGI219963[] = {{&S92541[70], 6}, {NULL}};
const Segment SGI219964[] = {{&S81718[44], 7}, {NULL}};
const Segment SGI219965[] = {{&S81718[44], 23}, {NULL}};
const Segment SGI219966[] = {{&S81718[30], 6}, {NULL}};
const Segment SGI219967[] = {{&S81718[116], 3}, {NULL}};
const Segment SGI219968[] = {{&S92542[34], 9}, {NULL}};
const Segment SGI219969[] = {{&S81718[81], 6}, {NULL}};
const Segment SGI219970[] = {{&S92542[27], 6}, {NULL}};
const Segment SGI219971[] = {{&S92542[96], 9}, {NULL}};
const Segment SGI219972[] = {{&S92542[69], 6}, {NULL}};
const Segment SGI219973[] = {{&S81719[38], 7}, {NULL}};
const Segment SGI219974[] = {{&S81719[31], 6}, {NULL}};
const Segment SGI219975[] = {{&S81721[34], 10}, {NULL}};
const Segment SGI219976[] = {{&S81721[27], 6}, {NULL}};
const Segment SGI219977[] = {{&S81721[97], 9}, {NULL}};
const Segment SGI219978[] = {{&S92545[34], 7}, {NULL}};
const Segment SGI219979[] = {{&S81721[70], 6}, {NULL}};
const Segment SGI219980[] = {{&S92545[27], 6}, {NULL}};
const Segment SGI219981[] = {{&S92545[94], 9}, {NULL}};
const Segment SGI219982[] = {{&S92545[67], 6}, {NULL}};
const Segment SGI219983[] = {{&S92546[44], 14}, {NULL}};
const Segment SGI219984[] = {{&S92546[44], 26}, {NULL}};
const Segment SGI219985[] = {{&S92546[30], 6}, {NULL}};
const Segment SGI219986[] = {{&S87820[44], 10}, {NULL}};
const Segment SGI219987[] = {{&S87820[44], 22}, {NULL}};
const Segment SGI219988[] = {{&S92546[31], 3}, {NULL}};
const Segment SGI219989[] = {{&S87820[30], 6}, {NULL}};
const Segment SGI219990[] = {{&S92546[84], 6}, {NULL}};
const Segment SGI219991[] = {{&S87820[144], 3}, {NULL}};
const Segment SGI219992[] = {{&S87820[109], 6}, {NULL}};
const Segment SGI219993[] = {{&S87820[68], 8}, {NULL}};
const Segment SGI219994[] = {{&S87820[80], 15}, {NULL}};
const Segment SGI219995[] = {{&S87821[44], 10}, {NULL}};
const Segment SGI219996[] = {{&S87821[44], 22}, {NULL}};
const Segment SGI219997[] = {{&S87821[30], 6}, {NULL}};
const Segment SGI219998[] = {{&S87821[144], 3}, {NULL}};
const Segment SGI219999[] = {{&S87821[109], 6}, {NULL}};
const Segment SGI220000[] = {{&S87821[68], 8}, {NULL}};
const Segment SGI220001[] = {{&S87821[80], 15}, {NULL}};
const Segment SGI220002[] = {{&S83036[34], 14}, {NULL}};
const Segment SGI220003[] = {{&S83036[27], 6}, {NULL}};
const Segment SGI220004[] = {{&S83036[101], 9}, {NULL}};
const Segment SGI220005[] = {{&S83036[74], 6}, {NULL}};
const Segment SGI220006[] = {{&S83036[142], 8}, {NULL}};
const Segment SGI220007[] = {{&S87822[44], 11}, {NULL}};
const Segment SGI220008[] = {{&S87822[44], 23}, {NULL}};
const Segment SGI220009[] = {{&S87822[30], 6}, {NULL}};
const Segment SGI220010[] = {{&S87822[145], 3}, {NULL}};
const Segment SGI220011[] = {{&S92550[44], 10}, {NULL}};
const Segment SGI220012[] = {{&S92550[44], 22}, {NULL}};
const Segment SGI220013[] = {{&S87822[110], 6}, {NULL}};
const Segment SGI220014[] = {{&S92550[30], 6}, {NULL}};
const Segment SGI220015[] = {{&S87822[69], 8}, {NULL}};
const Segment SGI220016[] = {{&S87822[81], 15}, {NULL}};
const Segment SGI220017[] = {{&S92550[31], 3}, {NULL}};
const Segment SGI220018[] = {{&S92550[80], 6}, {NULL}};
const Segment SGI220019[] = {{&S92550[140], 11}, {NULL}};
const Segment SGI220020[] = {{&S92552[45], 6}, {NULL}};
const Segment SGI220021[] = {{&S92552[30], 6}, {NULL}};
const Segment SGI220022[] = {{&S92552[31], 3}, {NULL}};
const Segment SGI220023[] = {{&S92552[77], 5}, {NULL}};
const Segment SGI220024[] = {{&S92552[135], 13}, {NULL}};
const Segment SGI220025[] = {{&S92556[59], 10}, {NULL}};
const Segment SGI220026[] = {{&S92556[59], 22}, {NULL}};
const Segment SGI220027[] = {{&S92556[45], 6}, {NULL}};
const Segment SGI220028[] = {{&S92556[46], 3}, {NULL}};
const Segment SGI220029[] = {{&S92556[95], 6}, {NULL}};
const Segment SGI220030[] = {{&S92556[0], 14}, {NULL}};
const Segment SGI220031[] = {{&S92562[45], 9}, {NULL}};
const Segment SGI220032[] = {{&S92562[30], 6}, {NULL}};
const Segment SGI220033[] = {{&S92562[31], 3}, {NULL}};
const Segment SGI220034[] = {{&S92562[80], 5}, {NULL}};
const Segment SGI220035[] = {{&S92562[138], 13}, {NULL}};
const Segment SGI220036[] = {{&S92566[34], 18}, {NULL}};
const Segment SGI220037[] = {{&S92566[27], 6}, {NULL}};
const Segment SGI220038[] = {{&S92566[105], 9}, {NULL}};
const Segment SGI220039[] = {{&S92566[78], 6}, {NULL}};
const Segment SGI220040[] = {{&S92568[34], 7}, {NULL}};
const Segment SGI220041[] = {{&S92568[27], 6}, {NULL}};
const Segment SGI220042[] = {{&S92568[94], 9}, {NULL}};
const Segment SGI220043[] = {{&S92568[67], 6}, {NULL}};
const Segment SGI220044[] = {{&S87848[34], 9}, {NULL}};
const Segment SGI220045[] = {{&S87848[27], 6}, {NULL}};
const Segment SGI220046[] = {{&S87848[96], 9}, {NULL}};
const Segment SGI220047[] = {{&S87848[69], 6}, {NULL}};
const Segment SGI220048[] = {{&S92572[34], 10}, {NULL}};
const Segment SGI220049[] = {{&S92572[27], 6}, {NULL}};
const Segment SGI220050[] = {{&S92572[97], 9}, {NULL}};
const Segment SGI220051[] = {{&S92572[70], 6}, {NULL}};
const Segment SGI220052[] = {{&S81762[59], 10}, {NULL}};
const Segment SGI220053[] = {{&S81762[59], 22}, {NULL}};
const Segment SGI220054[] = {{&S81762[45], 6}, {NULL}};
const Segment SGI220055[] = {{&S81762[130], 3}, {NULL}};
const Segment SGI220056[] = {{&S81762[95], 6}, {NULL}};
const Segment SGI220057[] = {{&S81762[0], 14}, {NULL}};
const Segment SGI220058[] = {{&S83079[38], 10}, {NULL}};
const Segment SGI220059[] = {{&S83079[31], 6}, {NULL}};
const Segment SGI220060[] = {{&S92581[34], 10}, {NULL}};
const Segment SGI220061[] = {{&S92581[27], 6}, {NULL}};
const Segment SGI220062[] = {{&S92581[97], 9}, {NULL}};
const Segment SGI220063[] = {{&S92581[70], 6}, {NULL}};
const Segment SGI220064[] = {{&S83080[44], 10}, {NULL}};
const Segment SGI220065[] = {{&S83080[44], 22}, {NULL}};
const Segment SGI220066[] = {{&S83080[30], 6}, {NULL}};
const Segment SGI220067[] = {{&S83080[115], 3}, {NULL}};
const Segment SGI220068[] = {{&S83080[80], 6}, {NULL}};
const Segment SGI220069[] = {{&S92582[34], 10}, {NULL}};
const Segment SGI220070[] = {{&S92582[27], 6}, {NULL}};
const Segment SGI220071[] = {{&S92582[97], 9}, {NULL}};
const Segment SGI220072[] = {{&S92582[70], 6}, {NULL}};
const Segment SGI220073[] = {{&S83081[44], 10}, {NULL}};
const Segment SGI220074[] = {{&S83081[44], 22}, {NULL}};
const Segment SGI220075[] = {{&S83081[30], 6}, {NULL}};
const Segment SGI220076[] = {{&S81770[34], 9}, {NULL}};
const Segment SGI220077[] = {{&S81770[27], 6}, {NULL}};
const Segment SGI220078[] = {{&S81770[96], 9}, {NULL}};
const Segment SGI220079[] = {{&S81770[69], 6}, {NULL}};
const Segment SGI220080[] = {{&S92583[44], 5}, {NULL}};
const Segment SGI220081[] = {{&S92583[44], 17}, {NULL}};
const Segment SGI220082[] = {{&S92583[30], 6}, {NULL}};
const Segment SGI220083[] = {{&S83082[44], 10}, {NULL}};
const Segment SGI220084[] = {{&S83082[44], 22}, {NULL}};
const Segment SGI220085[] = {{&S83082[30], 6}, {NULL}};
const Segment SGI220086[] = {{&S92583[31], 3}, {NULL}};
const Segment SGI220087[] = {{&S92583[75], 5}, {NULL}};
const Segment SGI220088[] = {{&S83082[83], 6}, {NULL}};
const Segment SGI220089[] = {{&S92583[112], 14}, {NULL}};
const Segment SGI220090[] = {{&S81771[34], 10}, {NULL}};
const Segment SGI220091[] = {{&S81771[27], 6}, {NULL}};
const Segment SGI220092[] = {{&S81771[97], 9}, {NULL}};
const Segment SGI220093[] = {{&S81771[70], 6}, {NULL}};
const Segment SGI220094[] = {{&S81772[34], 10}, {NULL}};
const Segment SGI220095[] = {{&S81772[27], 6}, {NULL}};
const Segment SGI220096[] = {{&S81772[97], 9}, {NULL}};
const Segment SGI220097[] = {{&S81772[70], 6}, {NULL}};
const Segment SGI220098[] = {{&S92586[34], 12}, {NULL}};
const Segment SGI220099[] = {{&S92586[27], 6}, {NULL}};
const Segment SGI220100[] = {{&S92586[99], 9}, {NULL}};
const Segment SGI220101[] = {{&S92586[72], 6}, {NULL}};
const Segment SGI220102[] = {{&S81776[44], 10}, {NULL}};
const Segment SGI220103[] = {{&S87872[34], 10}, {NULL}};
const Segment SGI220104[] = {{&S81776[44], 22}, {NULL}};
const Segment SGI220105[] = {{&S87872[27], 6}, {NULL}};
const Segment SGI220106[] = {{&S81776[30], 6}, {NULL}};
const Segment SGI220107[] = {{&S87872[97], 9}, {NULL}};
const Segment SGI220108[] = {{&S87872[70], 6}, {NULL}};
const Segment SGI220109[] = {{&S81776[80], 6}, {NULL}};
const Segment SGI220110[] = {{&S83090[44], 15}, {NULL}};
const Segment SGI220111[] = {{&S81776[107], 13}, {NULL}};
const Segment SGI220112[] = {{&S83090[44], 27}, {NULL}};
const Segment SGI220113[] = {{&S83090[30], 6}, {NULL}};
const Segment SGI220114[] = {{&S83090[120], 3}, {NULL}};
const Segment SGI220115[] = {{&S83090[85], 6}, {NULL}};
const Segment SGI220116[] = {{&S92591[34], 9}, {NULL}};
const Segment SGI220117[] = {{&S81777[34], 9}, {NULL}};
const Segment SGI220118[] = {{&S92591[27], 6}, {NULL}};
const Segment SGI220119[] = {{&S81777[27], 6}, {NULL}};
const Segment SGI220120[] = {{&S92591[96], 9}, {NULL}};
const Segment SGI220121[] = {{&S92591[69], 6}, {NULL}};
const Segment SGI220122[] = {{&S81777[96], 9}, {NULL}};
const Segment SGI220123[] = {{&S81777[69], 6}, {NULL}};
const Segment SGI220124[] = {{&S83092[44], 13}, {NULL}};
const Segment SGI220125[] = {{&S83092[44], 25}, {NULL}};
const Segment SGI220126[] = {{&S83092[30], 6}, {NULL}};
const Segment SGI220127[] = {{&S83092[118], 3}, {NULL}};
const Segment SGI220128[] = {{&S83092[83], 6}, {NULL}};
const Segment SGI220129[] = {{&S81779[34], 7}, {NULL}};
const Segment SGI220130[] = {{&S81779[27], 6}, {NULL}};
const Segment SGI220131[] = {{&S81779[104], 9}, {NULL}};
const Segment SGI220132[] = {{&S81779[77], 6}, {NULL}};
const Segment SGI220133[] = {{&S87881[45], 17}, {NULL}};
const Segment SGI220134[] = {{&S87881[30], 6}, {NULL}};
const Segment SGI220135[] = {{&S92597[44], 10}, {NULL}};
const Segment SGI220136[] = {{&S92597[44], 22}, {NULL}};
const Segment SGI220137[] = {{&S92597[30], 6}, {NULL}};
const Segment SGI220138[] = {{&S87881[131], 3}, {NULL}};
const Segment SGI220139[] = {{&S87881[97], 5}, {NULL}};
const Segment SGI220140[] = {{&S92597[80], 6}, {NULL}};
const Segment SGI220141[] = {{&S87881[155], 13}, {NULL}};
const Segment SGI220142[] = {{&S92597[107], 13}, {NULL}};
const Segment SGI220143[] = {{&S92599[34], 10}, {NULL}};
const Segment SGI220144[] = {{&S92599[27], 6}, {NULL}};
const Segment SGI220145[] = {{&S92599[97], 9}, {NULL}};
const Segment SGI220146[] = {{&S92599[70], 6}, {NULL}};
const Segment SGI220147[] = {{&S92599[138], 8}, {NULL}};
const Segment SGI220148[] = {{&S92600[34], 8}, {NULL}};
const Segment SGI220149[] = {{&S92600[27], 6}, {NULL}};
const Segment SGI220150[] = {{&S92600[106], 9}, {NULL}};
const Segment SGI220151[] = {{&S92600[79], 6}, {NULL}};
const Segment SGI220152[] = {{&S87891[44], 11}, {NULL}};
const Segment SGI220153[] = {{&S87891[44], 23}, {NULL}};
const Segment SGI220154[] = {{&S87891[30], 6}, {NULL}};
const Segment SGI220155[] = {{&S87891[81], 6}, {NULL}};
const Segment SGI220156[] = {{&S87891[108], 13}, {NULL}};
const Segment SGI220157[] = {{&S83109[34], 17}, {NULL}};
const Segment SGI220158[] = {{&S83109[27], 6}, {NULL}};
const Segment SGI220159[] = {{&S83109[104], 9}, {NULL}};
const Segment SGI220160[] = {{&S83109[77], 6}, {NULL}};
const Segment SGI220161[] = {{&S92607[44], 15}, {NULL}};
const Segment SGI220162[] = {{&S92607[44], 42}, {NULL}};
const Segment SGI220163[] = {{&S92607[30], 6}, {NULL}};
const Segment SGI220164[] = {{&S92607[100], 6}, {NULL}};
const Segment SGI220165[] = {{&S92607[127], 13}, {NULL}};
const Segment SGI220166[] = {{&S87896[34], 23}, {NULL}};
const Segment SGI220167[] = {{&S87896[27], 6}, {NULL}};
const Segment SGI220168[] = {{&S73809[34], 15}, {NULL}};
const Segment SGI220169[] = {{&S73809[27], 6}, {NULL}};
const Segment SGI220170[] = {{&S87896[110], 9}, {NULL}};
const Segment SGI220171[] = {{&S87896[83], 6}, {NULL}};
const Segment SGI220172[] = {{&S73809[108], 9}, {NULL}};
const Segment SGI220173[] = {{&S73809[81], 6}, {NULL}};
const Segment SGI220174[] = {{&S73810[34], 15}, {NULL}};
const Segment SGI220175[] = {{&S73810[27], 6}, {NULL}};
const Segment SGI220176[] = {{&S73810[108], 9}, {NULL}};
const Segment SGI220177[] = {{&S73810[81], 6}, {NULL}};
const Segment SGI220178[] = {{&S81817[45], 7}, {NULL}};
const Segment SGI220179[] = {{&S81817[30], 6}, {NULL}};
const Segment SGI220180[] = {{&S81817[117], 3}, {NULL}};
const Segment SGI220181[] = {{&S81817[83], 5}, {NULL}};
const Segment SGI220182[] = {{&S81817[141], 13}, {NULL}};
const Segment SGI220183[] = {{&S92628[34], 16}, {NULL}};
const Segment SGI220184[] = {{&S92628[27], 6}, {NULL}};
const Segment SGI220185[] = {{&S92628[113], 9}, {NULL}};
const Segment SGI220186[] = {{&S92628[86], 6}, {NULL}};
const Segment SGI220187[] = {{&S81823[44], 7}, {NULL}};
const Segment SGI220188[] = {{&S81823[44], 22}, {NULL}};
const Segment SGI220189[] = {{&S81823[30], 6}, {NULL}};
const Segment SGI220190[] = {{&S81823[115], 3}, {NULL}};
const Segment SGI220191[] = {{&S81823[80], 6}, {NULL}};
const Segment SGI220192[] = {{&S81823[139], 10}, {NULL}};
const Segment SGI220193[] = {{&S81824[44], 7}, {NULL}};
const Segment SGI220194[] = {{&S81824[44], 22}, {NULL}};
const Segment SGI220195[] = {{&S81824[30], 6}, {NULL}};
const Segment SGI220196[] = {{&S81824[115], 3}, {NULL}};
const Segment SGI220197[] = {{&S81824[80], 6}, {NULL}};
const Segment SGI220198[] = {{&S81824[139], 10}, {NULL}};
const Segment SGI220199[] = {{&S92640[44], 10}, {NULL}};
const Segment SGI220200[] = {{&S92640[44], 22}, {NULL}};
const Segment SGI220201[] = {{&S92640[30], 6}, {NULL}};
const Segment SGI220202[] = {{&S92640[80], 6}, {NULL}};
const Segment SGI220203[] = {{&S92640[107], 13}, {NULL}};
const Segment SGI220204[] = {{&S81830[34], 10}, {NULL}};
const Segment SGI220205[] = {{&S81830[27], 6}, {NULL}};
const Segment SGI220206[] = {{&S81830[97], 9}, {NULL}};
const Segment SGI220207[] = {{&S81830[70], 6}, {NULL}};
const Segment SGI220208[] = {{&S81830[138], 8}, {NULL}};
const Segment SGI220209[] = {{&S92644[34], 7}, {NULL}};
const Segment SGI220210[] = {{&S92644[27], 6}, {NULL}};
const Segment SGI220211[] = {{&S92644[100], 9}, {NULL}};
const Segment SGI220212[] = {{&S92644[73], 6}, {NULL}};
const Segment SGI220213[] = {{&S92644[141], 8}, {NULL}};
const Segment SGI220214[] = {{&S81835[34], 9}, {NULL}};
const Segment SGI220215[] = {{&S92648[34], 7}, {NULL}};
const Segment SGI220216[] = {{&S81835[27], 6}, {NULL}};
const Segment SGI220217[] = {{&S92648[27], 6}, {NULL}};
const Segment SGI220218[] = {{&S81835[96], 9}, {NULL}};
const Segment SGI220219[] = {{&S92648[94], 9}, {NULL}};
const Segment SGI220220[] = {{&S81835[69], 6}, {NULL}};
const Segment SGI220221[] = {{&S92648[67], 6}, {NULL}};
const Segment SGI220222[] = {{&S81835[137], 8}, {NULL}};
const Segment SGI220223[] = {{&S81836[34], 10}, {NULL}};
const Segment SGI220224[] = {{&S81836[27], 6}, {NULL}};
const Segment SGI220225[] = {{&S81836[97], 9}, {NULL}};
const Segment SGI220226[] = {{&S81836[70], 6}, {NULL}};
const Segment SGI220227[] = {{&S92652[44], 10}, {NULL}};
const Segment SGI220228[] = {{&S92652[44], 22}, {NULL}};
const Segment SGI220229[] = {{&S92652[30], 6}, {NULL}};
const Segment SGI220230[] = {{&S92652[80], 6}, {NULL}};
const Segment SGI220231[] = {{&S92652[107], 13}, {NULL}};
const Segment SGI220232[] = {{&S92653[34], 7}, {NULL}};
const Segment SGI220233[] = {{&S92653[27], 6}, {NULL}};
const Segment SGI220234[] = {{&S92653[99], 9}, {NULL}};
const Segment SGI220235[] = {{&S92653[72], 6}, {NULL}};
const Segment SGI220236[] = {{&S92653[140], 8}, {NULL}};
const Segment SGI220237[] = {{&S81842[34], 25}, {NULL}};
const Segment SGI220238[] = {{&S81842[27], 6}, {NULL}};
const Segment SGI220239[] = {{&S81842[112], 9}, {NULL}};
const Segment SGI220240[] = {{&S81842[85], 6}, {NULL}};
const Segment SGI220241[] = {{&S81843[34], 18}, {NULL}};
const Segment SGI220242[] = {{&S81843[27], 6}, {NULL}};
const Segment SGI220243[] = {{&S81843[105], 9}, {NULL}};
const Segment SGI220244[] = {{&S81843[78], 6}, {NULL}};
const Segment SGI220245[] = {{&S73872[34], 10}, {NULL}};
const Segment SGI220246[] = {{&S73872[27], 6}, {NULL}};
const Segment SGI220247[] = {{&S73872[96], 9}, {NULL}};
const Segment SGI220248[] = {{&S73872[69], 6}, {NULL}};
const Segment SGI220249[] = {{&S73872[137], 8}, {NULL}};
const Segment SGI220250[] = {{&S73873[34], 10}, {NULL}};
const Segment SGI220251[] = {{&S73873[27], 6}, {NULL}};
const Segment SGI220252[] = {{&S73873[97], 9}, {NULL}};
const Segment SGI220253[] = {{&S73873[70], 6}, {NULL}};
const Segment SGI220254[] = {{&S73874[34], 7}, {NULL}};
const Segment SGI220255[] = {{&S73874[27], 6}, {NULL}};
const Segment SGI220256[] = {{&S73874[94], 9}, {NULL}};
const Segment SGI220257[] = {{&S73874[67], 6}, {NULL}};
const Segment SGI220258[] = {{&S92679[11], 1}, {NULL}};
const Segment SGI220259[] = {{&S92679[44], 11}, {NULL}};
const Segment SGI220260[] = {{&S92679[30], 6}, {NULL}};
const Segment SGI220261[] = {{&S92679[31], 3}, {NULL}};
const Segment SGI220262[] = {{&S92679[69], 6}, {NULL}};
const Segment SGI220263[] = {{&S87973[34], 5}, {NULL}};
const Segment SGI220264[] = {{&S87973[27], 6}, {NULL}};
const Segment SGI220265[] = {{&S87973[92], 9}, {NULL}};
const Segment SGI220266[] = {{&S73878[34], 18}, {NULL}};
const Segment SGI220267[] = {{&S87973[65], 6}, {NULL}};
const Segment SGI220268[] = {{&S73878[27], 6}, {NULL}};
const Segment SGI220269[] = {{&S73878[104], 9}, {NULL}};
const Segment SGI220270[] = {{&S73878[77], 6}, {NULL}};
const Segment SGI220271[] = {{&S87984[34], 13}, {NULL}};
const Segment SGI220272[] = {{&S87984[27], 6}, {NULL}};
const Segment SGI220273[] = {{&S87984[114], 9}, {NULL}};
const Segment SGI220274[] = {{&S87984[87], 6}, {NULL}};
const Segment SGI220275[] = {{&S87993[44], 11}, {NULL}};
const Segment SGI220276[] = {{&S87993[44], 23}, {NULL}};
const Segment SGI220277[] = {{&S87993[30], 6}, {NULL}};
const Segment SGI220278[] = {{&S87993[116], 3}, {NULL}};
const Segment SGI220279[] = {{&S87993[81], 6}, {NULL}};
const Segment SGI220280[] = {{&S87997[34], 13}, {NULL}};
const Segment SGI220281[] = {{&S87997[27], 6}, {NULL}};
const Segment SGI220282[] = {{&S92710[34], 10}, {NULL}};
const Segment SGI220283[] = {{&S92710[27], 6}, {NULL}};
const Segment SGI220284[] = {{&S87997[100], 9}, {NULL}};
const Segment SGI220285[] = {{&S87997[73], 6}, {NULL}};
const Segment SGI220286[] = {{&S92710[97], 9}, {NULL}};
const Segment SGI220287[] = {{&S92710[70], 6}, {NULL}};
const Segment SGI220288[] = {{&S81897[38], 7}, {NULL}};
const Segment SGI220289[] = {{&S87999[34], 7}, {NULL}};
const Segment SGI220290[] = {{&S81897[31], 6}, {NULL}};
const Segment SGI220291[] = {{&S87999[27], 6}, {NULL}};
const Segment SGI220292[] = {{&S87999[94], 9}, {NULL}};
const Segment SGI220293[] = {{&S87999[67], 6}, {NULL}};
const Segment SGI220294[] = {{&S92713[45], 7}, {NULL}};
const Segment SGI220295[] = {{&S92713[30], 6}, {NULL}};
const Segment SGI220296[] = {{&S92713[31], 3}, {NULL}};
const Segment SGI220297[] = {{&S92713[83], 5}, {NULL}};
const Segment SGI220298[] = {{&S92713[141], 13}, {NULL}};
const Segment SGI220299[] = {{&S88000[34], 18}, {NULL}};
const Segment SGI220300[] = {{&S88000[27], 6}, {NULL}};
const Segment SGI220301[] = {{&S81899[44], 7}, {NULL}};
const Segment SGI220302[] = {{&S81899[44], 19}, {NULL}};
const Segment SGI220303[] = {{&S81899[30], 6}, {NULL}};
const Segment SGI220304[] = {{&S88000[105], 9}, {NULL}};
const Segment SGI220305[] = {{&S88000[78], 6}, {NULL}};
const Segment SGI220306[] = {{&S81899[112], 3}, {NULL}};
const Segment SGI220307[] = {{&S81899[77], 6}, {NULL}};
const Segment SGI220308[] = {{&S88003[44], 11}, {NULL}};
const Segment SGI220309[] = {{&S88003[44], 23}, {NULL}};
const Segment SGI220310[] = {{&S88003[30], 6}, {NULL}};
const Segment SGI220311[] = {{&S88003[81], 6}, {NULL}};
const Segment SGI220312[] = {{&S88003[108], 13}, {NULL}};
const Segment SGI220313[] = {{&S92719[34], 7}, {NULL}};
const Segment SGI220314[] = {{&S92719[27], 6}, {NULL}};
const Segment SGI220315[] = {{&S92719[90], 9}, {NULL}};
const Segment SGI220316[] = {{&S92719[63], 6}, {NULL}};
const Segment SGI220317[] = {{&S88005[34], 21}, {NULL}};
const Segment SGI220318[] = {{&S88005[27], 6}, {NULL}};
const Segment SGI220319[] = {{&S88005[108], 9}, {NULL}};
const Segment SGI220320[] = {{&S88005[81], 6}, {NULL}};
const Segment SGI220321[] = {{&S81906[34], 8}, {NULL}};
const Segment SGI220322[] = {{&S81906[27], 6}, {NULL}};
const Segment SGI220323[] = {{&S92721[34], 17}, {NULL}};
const Segment SGI220324[] = {{&S92721[27], 6}, {NULL}};
const Segment SGI220325[] = {{&S81906[94], 9}, {NULL}};
const Segment SGI220326[] = {{&S81906[67], 6}, {NULL}};
const Segment SGI220327[] = {{&S92721[104], 9}, {NULL}};
const Segment SGI220328[] = {{&S92721[77], 6}, {NULL}};
const Segment SGI220329[] = {{&S88006[44], 7}, {NULL}};
const Segment SGI220330[] = {{&S88006[44], 24}, {NULL}};
const Segment SGI220331[] = {{&S88006[30], 6}, {NULL}};
const Segment SGI220332[] = {{&S88006[82], 6}, {NULL}};
const Segment SGI220333[] = {{&S88006[109], 13}, {NULL}};
const Segment SGI220334[] = {{&S92722[44], 10}, {NULL}};
const Segment SGI220335[] = {{&S92722[44], 22}, {NULL}};
const Segment SGI220336[] = {{&S92722[30], 6}, {NULL}};
const Segment SGI220337[] = {{&S92722[31], 3}, {NULL}};
const Segment SGI220338[] = {{&S92722[80], 5}, {NULL}};
const Segment SGI220339[] = {{&S92722[117], 14}, {NULL}};
const Segment SGI220340[] = {{&S92723[34], 16}, {NULL}};
const Segment SGI220341[] = {{&S92723[27], 6}, {NULL}};
const Segment SGI220342[] = {{&S92723[103], 9}, {NULL}};
const Segment SGI220343[] = {{&S92723[76], 6}, {NULL}};
const Segment SGI220344[] = {{&S92724[34], 7}, {NULL}};
const Segment SGI220345[] = {{&S92724[27], 6}, {NULL}};
const Segment SGI220346[] = {{&S92724[98], 9}, {NULL}};
const Segment SGI220347[] = {{&S92724[71], 6}, {NULL}};
const Segment SGI220348[] = {{&S92727[34], 7}, {NULL}};
const Segment SGI220349[] = {{&S92727[27], 6}, {NULL}};
const Segment SGI220350[] = {{&S92727[112], 9}, {NULL}};
const Segment SGI220351[] = {{&S92727[85], 6}, {NULL}};
const Segment SGI220352[] = {{&S81919[44], 10}, {NULL}};
const Segment SGI220353[] = {{&S81919[44], 22}, {NULL}};
const Segment SGI220354[] = {{&S81919[30], 6}, {NULL}};
const Segment SGI220355[] = {{&S92728[34], 16}, {NULL}};
const Segment SGI220356[] = {{&S92728[27], 6}, {NULL}};
const Segment SGI220357[] = {{&S81919[113], 3}, {NULL}};
const Segment SGI220358[] = {{&S81919[80], 5}, {NULL}};
const Segment SGI220359[] = {{&S92728[113], 9}, {NULL}};
const Segment SGI220360[] = {{&S81919[117], 14}, {NULL}};
const Segment SGI220361[] = {{&S92728[86], 6}, {NULL}};
const Segment SGI220362[] = {{&S88017[44], 11}, {NULL}};
const Segment SGI220363[] = {{&S88017[44], 23}, {NULL}};
const Segment SGI220364[] = {{&S88017[30], 6}, {NULL}};
const Segment SGI220365[] = {{&S88017[116], 3}, {NULL}};
const Segment SGI220366[] = {{&S88017[81], 6}, {NULL}};
const Segment SGI220367[] = {{&S88017[141], 7}, {NULL}};
const Segment SGI220368[] = {{&S88018[44], 11}, {NULL}};
const Segment SGI220369[] = {{&S88018[44], 23}, {NULL}};
const Segment SGI220370[] = {{&S88018[30], 6}, {NULL}};
const Segment SGI220371[] = {{&S88018[116], 3}, {NULL}};
const Segment SGI220372[] = {{&S81923[34], 7}, {NULL}};
const Segment SGI220373[] = {{&S88018[81], 6}, {NULL}};
const Segment SGI220374[] = {{&S81923[27], 6}, {NULL}};
const Segment SGI220375[] = {{&S88018[141], 7}, {NULL}};
const Segment SGI220376[] = {{&S81923[94], 9}, {NULL}};
const Segment SGI220377[] = {{&S81923[67], 6}, {NULL}};
const Segment SGI220378[] = {{&S81924[44], 9}, {NULL}};
const Segment SGI220379[] = {{&S81924[44], 21}, {NULL}};
const Segment SGI220380[] = {{&S81924[30], 6}, {NULL}};
const Segment SGI220381[] = {{&S81924[114], 3}, {NULL}};
const Segment SGI220382[] = {{&S81924[79], 6}, {NULL}};
const Segment SGI220383[] = {{&S81925[34], 11}, {NULL}};
const Segment SGI220384[] = {{&S81925[27], 6}, {NULL}};
const Segment SGI220385[] = {{&S81925[98], 9}, {NULL}};
const Segment SGI220386[] = {{&S81925[71], 6}, {NULL}};
const Segment SGI220387[] = {{&S81927[34], 10}, {NULL}};
const Segment SGI220388[] = {{&S81927[27], 6}, {NULL}};
const Segment SGI220389[] = {{&S92738[34], 15}, {NULL}};
const Segment SGI220390[] = {{&S81927[97], 9}, {NULL}};
const Segment SGI220391[] = {{&S92738[27], 6}, {NULL}};
const Segment SGI220392[] = {{&S81927[70], 6}, {NULL}};
const Segment SGI220393[] = {{&S92738[102], 9}, {NULL}};
const Segment SGI220394[] = {{&S92738[75], 6}, {NULL}};
const Segment SGI220395[] = {{&S81929[34], 9}, {NULL}};
const Segment SGI220396[] = {{&S81929[27], 6}, {NULL}};
const Segment SGI220397[] = {{&S92740[34], 8}, {NULL}};
const Segment SGI220398[] = {{&S92740[27], 6}, {NULL}};
const Segment SGI220399[] = {{&S81929[96], 9}, {NULL}};
const Segment SGI220400[] = {{&S81929[69], 6}, {NULL}};
const Segment SGI220401[] = {{&S81929[137], 8}, {NULL}};
const Segment SGI220402[] = {{&S92740[100], 9}, {NULL}};
const Segment SGI220403[] = {{&S92740[73], 6}, {NULL}};
const Segment SGI220404[] = {{&S88030[34], 7}, {NULL}};
const Segment SGI220405[] = {{&S88030[27], 6}, {NULL}};
const Segment SGI220406[] = {{&S81930[34], 8}, {NULL}};
const Segment SGI220407[] = {{&S81930[27], 6}, {NULL}};
const Segment SGI220408[] = {{&S88030[94], 9}, {NULL}};
const Segment SGI220409[] = {{&S88030[67], 6}, {NULL}};
const Segment SGI220410[] = {{&S92741[34], 16}, {NULL}};
const Segment SGI220411[] = {{&S92741[27], 6}, {NULL}};
const Segment SGI220412[] = {{&S81930[95], 9}, {NULL}};
const Segment SGI220413[] = {{&S81930[68], 6}, {NULL}};
const Segment SGI220414[] = {{&S81930[136], 8}, {NULL}};
const Segment SGI220415[] = {{&S92741[113], 9}, {NULL}};
const Segment SGI220416[] = {{&S92741[86], 6}, {NULL}};
const Segment SGI220417[] = {{&S92742[34], 11}, {NULL}};
const Segment SGI220418[] = {{&S92742[27], 6}, {NULL}};
const Segment SGI220419[] = {{&S92742[98], 9}, {NULL}};
const Segment SGI220420[] = {{&S92742[71], 6}, {NULL}};
const Segment SGI220421[] = {{&S92743[45], 10}, {NULL}};
const Segment SGI220422[] = {{&S92743[30], 6}, {NULL}};
const Segment SGI220423[] = {{&S92743[31], 3}, {NULL}};
const Segment SGI220424[] = {{&S92743[81], 5}, {NULL}};
const Segment SGI220425[] = {{&S92743[139], 13}, {NULL}};
const Segment SGI220426[] = {{&S92745[34], 15}, {NULL}};
const Segment SGI220427[] = {{&S92745[27], 6}, {NULL}};
const Segment SGI220428[] = {{&S92745[102], 9}, {NULL}};
const Segment SGI220429[] = {{&S92745[75], 6}, {NULL}};
const Segment SGI220430[] = {{&S92747[34], 13}, {NULL}};
const Segment SGI220431[] = {{&S92747[27], 6}, {NULL}};
const Segment SGI220432[] = {{&S73963[44], 12}, {NULL}};
const Segment SGI220433[] = {{&S73963[44], 23}, {NULL}};
const Segment SGI220434[] = {{&S73963[30], 6}, {NULL}};
const Segment SGI220435[] = {{&S92747[100], 9}, {NULL}};
const Segment SGI220436[] = {{&S92747[73], 6}, {NULL}};
const Segment SGI220437[] = {{&S73963[116], 3}, {NULL}};
const Segment SGI220438[] = {{&S73963[81], 6}, {NULL}};
const Segment SGI220439[] = {{&S73964[38], 12}, {NULL}};
const Segment SGI220440[] = {{&S73964[31], 6}, {NULL}};
const Segment SGI220441[] = {{&S73965[34], 9}, {NULL}};
const Segment SGI220442[] = {{&S73965[27], 6}, {NULL}};
const Segment SGI220443[] = {{&S73965[95], 9}, {NULL}};
const Segment SGI220444[] = {{&S73965[68], 6}, {NULL}};
const Segment SGI220445[] = {{&S92754[34], 10}, {NULL}};
const Segment SGI220446[] = {{&S92754[27], 6}, {NULL}};
const Segment SGI220447[] = {{&S92754[97], 9}, {NULL}};
const Segment SGI220448[] = {{&S92754[70], 6}, {NULL}};
const Segment SGI220449[] = {{&S92755[44], 10}, {NULL}};
const Segment SGI220450[] = {{&S92755[44], 22}, {NULL}};
const Segment SGI220451[] = {{&S92755[30], 6}, {NULL}};
const Segment SGI220452[] = {{&S92755[31], 3}, {NULL}};
const Segment SGI220453[] = {{&S92755[80], 5}, {NULL}};
const Segment SGI220454[] = {{&S92755[117], 14}, {NULL}};
const Segment SGI220455[] = {{&S92762[34], 11}, {NULL}};
const Segment SGI220456[] = {{&S92762[27], 6}, {NULL}};
const Segment SGI220457[] = {{&S92762[98], 9}, {NULL}};
const Segment SGI220458[] = {{&S92762[71], 6}, {NULL}};
const Segment SGI220459[] = {{&S92764[34], 10}, {NULL}};
const Segment SGI220460[] = {{&S92764[27], 6}, {NULL}};
const Segment SGI220461[] = {{&S92764[97], 9}, {NULL}};
const Segment SGI220462[] = {{&S92764[70], 6}, {NULL}};
const Segment SGI220463[] = {{&S92765[34], 10}, {NULL}};
const Segment SGI220464[] = {{&S92765[27], 6}, {NULL}};
const Segment SGI220465[] = {{&S92765[97], 9}, {NULL}};
const Segment SGI220466[] = {{&S92765[70], 6}, {NULL}};
const Segment SGI220467[] = {{&S92770[34], 18}, {NULL}};
const Segment SGI220468[] = {{&S92770[27], 6}, {NULL}};
const Segment SGI220469[] = {{&S92770[105], 9}, {NULL}};
const Segment SGI220470[] = {{&S92770[78], 6}, {NULL}};
const Segment SGI220471[] = {{&S92771[34], 5}, {NULL}};
const Segment SGI220472[] = {{&S92771[27], 6}, {NULL}};
const Segment SGI220473[] = {{&S92771[92], 9}, {NULL}};
const Segment SGI220474[] = {{&S92771[65], 6}, {NULL}};
const Segment SGI220475[] = {{&S81979[34], 15}, {NULL}};
const Segment SGI220476[] = {{&S81979[27], 6}, {NULL}};
const Segment SGI220477[] = {{&S81979[102], 9}, {NULL}};
const Segment SGI220478[] = {{&S81979[75], 6}, {NULL}};
const Segment SGI220479[] = {{&S92777[34], 25}, {NULL}};
const Segment SGI220480[] = {{&S92777[27], 6}, {NULL}};
const Segment SGI220481[] = {{&S92777[112], 9}, {NULL}};
const Segment SGI220482[] = {{&S92777[85], 6}, {NULL}};
const Segment SGI220483[] = {{&S92778[34], 21}, {NULL}};
const Segment SGI220484[] = {{&S92778[27], 6}, {NULL}};
const Segment SGI220485[] = {{&S92778[108], 9}, {NULL}};
const Segment SGI220486[] = {{&S92778[81], 6}, {NULL}};
const Segment SGI220487[] = {{&S81989[34], 10}, {NULL}};
const Segment SGI220488[] = {{&S81989[27], 6}, {NULL}};
const Segment SGI220489[] = {{&S81989[97], 9}, {NULL}};
const Segment SGI220490[] = {{&S81989[70], 6}, {NULL}};
const Segment SGI220491[] = {{&S92785[44], 7}, {NULL}};
const Segment SGI220492[] = {{&S92785[44], 18}, {NULL}};
const Segment SGI220493[] = {{&S92785[30], 6}, {NULL}};
const Segment SGI220494[] = {{&S92785[31], 3}, {NULL}};
const Segment SGI220495[] = {{&S92785[76], 6}, {NULL}};
const Segment SGI220496[] = {{&S92785[136], 7}, {NULL}};
const Segment SGI220497[] = {{&S88096[44], 10}, {NULL}};
const Segment SGI220498[] = {{&S88096[44], 22}, {NULL}};
const Segment SGI220499[] = {{&S88096[30], 6}, {NULL}};
const Segment SGI220500[] = {{&S88096[115], 3}, {NULL}};
const Segment SGI220501[] = {{&S88096[80], 6}, {NULL}};
const Segment SGI220502[] = {{&S92789[44], 14}, {NULL}};
const Segment SGI220503[] = {{&S92789[44], 26}, {NULL}};
const Segment SGI220504[] = {{&S92789[30], 6}, {NULL}};
const Segment SGI220505[] = {{&S92789[84], 6}, {NULL}};
const Segment SGI220506[] = {{&S92789[111], 13}, {NULL}};
const Segment SGI220507[] = {{&S81997[34], 7}, {NULL}};
const Segment SGI220508[] = {{&S81997[27], 6}, {NULL}};
const Segment SGI220509[] = {{&S81997[104], 9}, {NULL}};
const Segment SGI220510[] = {{&S81997[77], 6}, {NULL}};
const Segment SGI220511[] = {{&S81999[34], 7}, {NULL}};
const Segment SGI220512[] = {{&S81999[27], 6}, {NULL}};
const Segment SGI220513[] = {{&S92794[44], 11}, {NULL}};
const Segment SGI220514[] = {{&S92794[44], 23}, {NULL}};
const Segment SGI220515[] = {{&S92794[30], 6}, {NULL}};
const Segment SGI220516[] = {{&S81999[94], 9}, {NULL}};
const Segment SGI220517[] = {{&S81999[67], 6}, {NULL}};
const Segment SGI220518[] = {{&S92794[31], 3}, {NULL}};
const Segment SGI220519[] = {{&S92794[81], 6}, {NULL}};
const Segment SGI220520[] = {{&S92794[141], 7}, {NULL}};
const Segment SGI220521[] = {{&S92795[44], 10}, {NULL}};
const Segment SGI220522[] = {{&S92795[44], 22}, {NULL}};
const Segment SGI220523[] = {{&S92795[30], 6}, {NULL}};
const Segment SGI220524[] = {{&S82001[44], 5}, {NULL}};
const Segment SGI220525[] = {{&S82001[44], 17}, {NULL}};
const Segment SGI220526[] = {{&S92795[80], 6}, {NULL}};
const Segment SGI220527[] = {{&S82001[30], 6}, {NULL}};
const Segment SGI220528[] = {{&S92795[107], 13}, {NULL}};
const Segment SGI220529[] = {{&S82001[110], 3}, {NULL}};
const Segment SGI220530[] = {{&S82001[75], 6}, {NULL}};
const Segment SGI220531[] = {{&S92797[44], 5}, {NULL}};
const Segment SGI220532[] = {{&S92797[44], 17}, {NULL}};
const Segment SGI220533[] = {{&S92797[30], 6}, {NULL}};
const Segment SGI220534[] = {{&S92797[75], 6}, {NULL}};
const Segment SGI220535[] = {{&S92797[102], 13}, {NULL}};
const Segment SGI220536[] = {{&S92799[34], 11}, {NULL}};
const Segment SGI220537[] = {{&S92799[27], 6}, {NULL}};
const Segment SGI220538[] = {{&S92799[98], 9}, {NULL}};
const Segment SGI220539[] = {{&S92799[71], 6}, {NULL}};
const Segment SGI220540[] = {{&S82007[34], 10}, {NULL}};
const Segment SGI220541[] = {{&S82007[27], 6}, {NULL}};
const Segment SGI220542[] = {{&S82007[97], 9}, {NULL}};
const Segment SGI220543[] = {{&S82007[70], 6}, {NULL}};
const Segment SGI220544[] = {{&S88114[34], 9}, {NULL}};
const Segment SGI220545[] = {{&S88114[27], 6}, {NULL}};
const Segment SGI220546[] = {{&S88114[96], 9}, {NULL}};
const Segment SGI220547[] = {{&S88114[69], 6}, {NULL}};
const Segment SGI220548[] = {{&S82014[44], 10}, {NULL}};
const Segment SGI220549[] = {{&S82014[44], 22}, {NULL}};
const Segment SGI220550[] = {{&S82014[30], 6}, {NULL}};
const Segment SGI220551[] = {{&S82014[80], 6}, {NULL}};
const Segment SGI220552[] = {{&S82014[107], 13}, {NULL}};
const Segment SGI220553[] = {{&S82015[44], 18}, {NULL}};
const Segment SGI220554[] = {{&S82015[44], 30}, {NULL}};
const Segment SGI220555[] = {{&S82015[30], 6}, {NULL}};
const Segment SGI220556[] = {{&S82015[88], 6}, {NULL}};
const Segment SGI220557[] = {{&S82015[115], 13}, {NULL}};
const Segment SGI220558[] = {{&S82017[44], 10}, {NULL}};
const Segment SGI220559[] = {{&S82017[44], 22}, {NULL}};
const Segment SGI220560[] = {{&S82017[30], 6}, {NULL}};
const Segment SGI220561[] = {{&S82017[80], 6}, {NULL}};
const Segment SGI220562[] = {{&S82017[107], 13}, {NULL}};
const Segment SGI220563[] = {{&S82051[44], 11}, {NULL}};
const Segment SGI220564[] = {{&S82051[44], 23}, {NULL}};
const Segment SGI220565[] = {{&S82051[30], 6}, {NULL}};
const Segment SGI220566[] = {{&S82051[116], 3}, {NULL}};
const Segment SGI220567[] = {{&S82051[81], 6}, {NULL}};
const Segment SGI220568[] = {{&S82052[44], 11}, {NULL}};
const Segment SGI220569[] = {{&S82052[44], 24}, {NULL}};
const Segment SGI220570[] = {{&S82052[30], 6}, {NULL}};
const Segment SGI220571[] = {{&S82052[117], 3}, {NULL}};
const Segment SGI220572[] = {{&S82052[82], 6}, {NULL}};
const Segment SGI220573[] = {{&S92860[44], 5}, {NULL}};
const Segment SGI220574[] = {{&S92860[44], 14}, {NULL}};
const Segment SGI220575[] = {{&S92860[30], 6}, {NULL}};
const Segment SGI220576[] = {{&S92860[103], 2}, {NULL}};
const Segment SGI220577[] = {{&S92860[72], 5}, {NULL}};
const Segment SGI220578[] = {{&S74092[34], 10}, {NULL}};
const Segment SGI220579[] = {{&S74092[27], 6}, {NULL}};
const Segment SGI220580[] = {{&S74092[96], 9}, {NULL}};
const Segment SGI220581[] = {{&S74092[69], 6}, {NULL}};
const Segment SGI220582[] = {{&S92867[44], 10}, {NULL}};
const Segment SGI220583[] = {{&S92867[44], 22}, {NULL}};
const Segment SGI220584[] = {{&S92867[30], 6}, {NULL}};
const Segment SGI220585[] = {{&S92867[31], 3}, {NULL}};
const Segment SGI220586[] = {{&S92867[80], 6}, {NULL}};
const Segment SGI220587[] = {{&S92867[140], 7}, {NULL}};
const Segment SGI220588[] = {{&S82071[34], 10}, {NULL}};
const Segment SGI220589[] = {{&S82071[27], 6}, {NULL}};
const Segment SGI220590[] = {{&S82071[97], 9}, {NULL}};
const Segment SGI220591[] = {{&S82071[70], 6}, {NULL}};
const Segment SGI220592[] = {{&S92874[44], 10}, {NULL}};
const Segment SGI220593[] = {{&S92874[44], 22}, {NULL}};
const Segment SGI220594[] = {{&S92874[30], 6}, {NULL}};
const Segment SGI220595[] = {{&S92874[31], 3}, {NULL}};
const Segment SGI220596[] = {{&S92874[87], 6}, {NULL}};
const Segment SGI220597[] = {{&S92874[66], 8}, {NULL}};
const Segment SGI220598[] = {{&S82078[34], 9}, {NULL}};
const Segment SGI220599[] = {{&S82078[27], 6}, {NULL}};
const Segment SGI220600[] = {{&S92875[44], 10}, {NULL}};
const Segment SGI220601[] = {{&S92875[44], 22}, {NULL}};
const Segment SGI220602[] = {{&S82078[96], 9}, {NULL}};
const Segment SGI220603[] = {{&S92875[30], 6}, {NULL}};
const Segment SGI220604[] = {{&S82078[69], 6}, {NULL}};
const Segment SGI220605[] = {{&S92875[31], 3}, {NULL}};
const Segment SGI220606[] = {{&S92875[80], 6}, {NULL}};
const Segment SGI220607[] = {{&S92875[140], 7}, {NULL}};
const Segment SGI220608[] = {{&S92877[44], 11}, {NULL}};
const Segment SGI220609[] = {{&S92877[44], 23}, {NULL}};
const Segment SGI220610[] = {{&S82081[34], 12}, {NULL}};
const Segment SGI220611[] = {{&S92877[30], 6}, {NULL}};
const Segment SGI220612[] = {{&S82081[27], 6}, {NULL}};
const Segment SGI220613[] = {{&S92877[31], 3}, {NULL}};
const Segment SGI220614[] = {{&S92877[96], 6}, {NULL}};
const Segment SGI220615[] = {{&S82081[99], 9}, {NULL}};
const Segment SGI220616[] = {{&S82081[72], 6}, {NULL}};
const Segment SGI220617[] = {{&S92877[69], 8}, {NULL}};
const Segment SGI220618[] = {{&S82082[34], 7}, {NULL}};
const Segment SGI220619[] = {{&S82082[27], 6}, {NULL}};
const Segment SGI220620[] = {{&S92878[44], 11}, {NULL}};
const Segment SGI220621[] = {{&S92878[44], 23}, {NULL}};
const Segment SGI220622[] = {{&S92878[30], 6}, {NULL}};
const Segment SGI220623[] = {{&S82082[99], 9}, {NULL}};
const Segment SGI220624[] = {{&S82082[72], 6}, {NULL}};
const Segment SGI220625[] = {{&S92878[31], 3}, {NULL}};
const Segment SGI220626[] = {{&S92878[100], 6}, {NULL}};
const Segment SGI220627[] = {{&S92878[69], 8}, {NULL}};
const Segment SGI220628[] = {{&S82083[34], 11}, {NULL}};
const Segment SGI220629[] = {{&S82083[27], 6}, {NULL}};
const Segment SGI220630[] = {{&S82083[98], 9}, {NULL}};
const Segment SGI220631[] = {{&S82083[71], 6}, {NULL}};
const Segment SGI220632[] = {{&S82086[44], 21}, {NULL}};
const Segment SGI220633[] = {{&S82086[30], 6}, {NULL}};
const Segment SGI220634[] = {{&S82086[116], 3}, {NULL}};
const Segment SGI220635[] = {{&S82086[81], 6}, {NULL}};
const Segment SGI220636[] = {{&S82087[44], 10}, {NULL}};
const Segment SGI220637[] = {{&S82087[44], 22}, {NULL}};
const Segment SGI220638[] = {{&S82087[30], 6}, {NULL}};
const Segment SGI220639[] = {{&S82087[115], 3}, {NULL}};
const Segment SGI220640[] = {{&S82087[80], 6}, {NULL}};
const Segment SGI220641[] = {{&S92886[59], 21}, {NULL}};
const Segment SGI220642[] = {{&S92886[59], 33}, {NULL}};
const Segment SGI220643[] = {{&S92886[45], 6}, {NULL}};
const Segment SGI220644[] = {{&S92886[46], 3}, {NULL}};
const Segment SGI220645[] = {{&S92886[106], 6}, {NULL}};
const Segment SGI220646[] = {{&S92886[0], 14}, {NULL}};
const Segment SGI220647[] = {{&S82088[44], 21}, {NULL}};
const Segment SGI220648[] = {{&S88207[44], 16}, {NULL}};
const Segment SGI220649[] = {{&S88207[44], 28}, {NULL}};
const Segment SGI220650[] = {{&S88207[30], 6}, {NULL}};
const Segment SGI220651[] = {{&S88207[137], 3}, {NULL}};
const Segment SGI220652[] = {{&S88207[102], 6}, {NULL}};
const Segment SGI220653[] = {{&S88207[74], 8}, {NULL}};
const Segment SGI220654[] = {{&S92893[34], 10}, {NULL}};
const Segment SGI220655[] = {{&S92893[27], 6}, {NULL}};
const Segment SGI220656[] = {{&S92893[96], 9}, {NULL}};
const Segment SGI220657[] = {{&S92893[69], 6}, {NULL}};
const Segment SGI220658[] = {{&S92895[34], 5}, {NULL}};
const Segment SGI220659[] = {{&S92895[27], 6}, {NULL}};
const Segment SGI220660[] = {{&S92895[92], 9}, {NULL}};
const Segment SGI220661[] = {{&S92895[65], 6}, {NULL}};
const Segment SGI220662[] = {{&S83256[44], 6}, {NULL}};
const Segment SGI220663[] = {{&S83256[44], 18}, {NULL}};
const Segment SGI220664[] = {{&S83256[30], 6}, {NULL}};
const Segment SGI220665[] = {{&S83256[76], 6}, {NULL}};
const Segment SGI220666[] = {{&S83256[113], 3}, {NULL}};
const Segment SGI220667[] = {{&S92904[34], 7}, {NULL}};
const Segment SGI220668[] = {{&S92904[27], 6}, {NULL}};
const Segment SGI220669[] = {{&S92904[94], 9}, {NULL}};
const Segment SGI220670[] = {{&S92904[67], 6}, {NULL}};
const Segment SGI220671[] = {{&S92907[34], 11}, {NULL}};
const Segment SGI220672[] = {{&S92907[27], 6}, {NULL}};
const Segment SGI220673[] = {{&S92907[98], 9}, {NULL}};
const Segment SGI220674[] = {{&S92907[71], 6}, {NULL}};
const Segment SGI220675[] = {{&S92909[45], 14}, {NULL}};
const Segment SGI220676[] = {{&S92909[30], 6}, {NULL}};
const Segment SGI220677[] = {{&S92909[31], 3}, {NULL}};
const Segment SGI220678[] = {{&S92909[85], 5}, {NULL}};
const Segment SGI220679[] = {{&S92909[143], 13}, {NULL}};
const Segment SGI220680[] = {{&S88231[44], 11}, {NULL}};
const Segment SGI220681[] = {{&S88231[44], 23}, {NULL}};
const Segment SGI220682[] = {{&S92910[34], 16}, {NULL}};
const Segment SGI220683[] = {{&S88231[30], 6}, {NULL}};
const Segment SGI220684[] = {{&S92910[27], 6}, {NULL}};
const Segment SGI220685[] = {{&S88231[116], 3}, {NULL}};
const Segment SGI220686[] = {{&S88231[81], 6}, {NULL}};
const Segment SGI220687[] = {{&S92910[103], 9}, {NULL}};
const Segment SGI220688[] = {{&S92910[76], 6}, {NULL}};
const Segment SGI220689[] = {{&S92911[34], 10}, {NULL}};
const Segment SGI220690[] = {{&S92911[27], 6}, {NULL}};
const Segment SGI220691[] = {{&S92911[97], 9}, {NULL}};
const Segment SGI220692[] = {{&S92911[70], 6}, {NULL}};
const Segment SGI220693[] = {{&S92912[34], 7}, {NULL}};
const Segment SGI220694[] = {{&S92912[27], 6}, {NULL}};
const Segment SGI220695[] = {{&S88235[34], 23}, {NULL}};
const Segment SGI220696[] = {{&S88235[27], 6}, {NULL}};
const Segment SGI220697[] = {{&S92912[94], 9}, {NULL}};
const Segment SGI220698[] = {{&S92912[67], 6}, {NULL}};
const Segment SGI220699[] = {{&S88235[110], 9}, {NULL}};
const Segment SGI220700[] = {{&S88235[83], 6}, {NULL}};
const Segment SGI220701[] = {{&S92915[34], 7}, {NULL}};
const Segment SGI220702[] = {{&S92915[27], 6}, {NULL}};
const Segment SGI220703[] = {{&S92915[94], 9}, {NULL}};
const Segment SGI220704[] = {{&S92915[67], 6}, {NULL}};
const Segment SGI220705[] = {{&S88241[34], 12}, {NULL}};
const Segment SGI220706[] = {{&S88241[27], 6}, {NULL}};
const Segment SGI220707[] = {{&S88241[99], 9}, {NULL}};
const Segment SGI220708[] = {{&S88241[72], 6}, {NULL}};
const Segment SGI220709[] = {{&S92924[45], 7}, {NULL}};
const Segment SGI220710[] = {{&S92924[30], 6}, {NULL}};
const Segment SGI220711[] = {{&S92924[31], 3}, {NULL}};
const Segment SGI220712[] = {{&S92924[84], 5}, {NULL}};
const Segment SGI220713[] = {{&S92924[142], 13}, {NULL}};
const Segment SGI220714[] = {{&S88251[44], 8}, {NULL}};
const Segment SGI220715[] = {{&S88251[44], 20}, {NULL}};
const Segment SGI220716[] = {{&S88251[30], 6}, {NULL}};
const Segment SGI220717[] = {{&S88251[113], 3}, {NULL}};
const Segment SGI220718[] = {{&S88251[78], 6}, {NULL}};
const Segment SGI220719[] = {{&S88252[38], 8}, {NULL}};
const Segment SGI220720[] = {{&S88252[31], 6}, {NULL}};
const Segment SGI220721[] = {{&S92928[34], 11}, {NULL}};
const Segment SGI220722[] = {{&S92928[27], 6}, {NULL}};
const Segment SGI220723[] = {{&S92928[98], 9}, {NULL}};
const Segment SGI220724[] = {{&S92928[71], 6}, {NULL}};
const Segment SGI220725[] = {{&S82088[30], 6}, {NULL}};
const Segment SGI220726[] = {{&S82088[116], 3}, {NULL}};
const Segment SGI220727[] = {{&S82088[81], 6}, {NULL}};
const Segment SGI220728[] = {{&S88255[44], 14}, {NULL}};
const Segment SGI220729[] = {{&S88255[44], 26}, {NULL}};
const Segment SGI220730[] = {{&S88255[30], 6}, {NULL}};
const Segment SGI220731[] = {{&S88255[84], 6}, {NULL}};
const Segment SGI220732[] = {{&S88255[111], 13}, {NULL}};
const Segment SGI220733[] = {{&S88256[44], 23}, {NULL}};
const Segment SGI220734[] = {{&S88256[44], 35}, {NULL}};
const Segment SGI220735[] = {{&S88256[30], 6}, {NULL}};
const Segment SGI220736[] = {{&S88256[93], 6}, {NULL}};
const Segment SGI220737[] = {{&S88256[120], 13}, {NULL}};
const Segment SGI220738[] = {{&S92934[45], 6}, {NULL}};
const Segment SGI220739[] = {{&S92934[30], 6}, {NULL}};
const Segment SGI220740[] = {{&S92934[31], 3}, {NULL}};
const Segment SGI220741[] = {{&S92934[77], 5}, {NULL}};
const Segment SGI220742[] = {{&S92934[135], 13}, {NULL}};
const Segment SGI220743[] = {{&S92936[34], 8}, {NULL}};
const Segment SGI220744[] = {{&S92936[27], 6}, {NULL}};
const Segment SGI220745[] = {{&S92936[95], 9}, {NULL}};
const Segment SGI220746[] = {{&S92936[68], 6}, {NULL}};
const Segment SGI220747[] = {{&S82097[44], 7}, {NULL}};
const Segment SGI220748[] = {{&S82097[44], 23}, {NULL}};
const Segment SGI220749[] = {{&S82097[30], 6}, {NULL}};
const Segment SGI220750[] = {{&S82097[81], 6}, {NULL}};
const Segment SGI220751[] = {{&S82097[108], 13}, {NULL}};
const Segment SGI220752[] = {{&S88263[34], 9}, {NULL}};
const Segment SGI220753[] = {{&S88263[27], 6}, {NULL}};
const Segment SGI220754[] = {{&S88263[96], 9}, {NULL}};
const Segment SGI220755[] = {{&S88263[69], 6}, {NULL}};
const Segment SGI220756[] = {{&S82098[44], 7}, {NULL}};
const Segment SGI220757[] = {{&S82098[44], 24}, {NULL}};
const Segment SGI220758[] = {{&S82098[30], 6}, {NULL}};
const Segment SGI220759[] = {{&S82098[82], 6}, {NULL}};
const Segment SGI220760[] = {{&S82098[109], 13}, {NULL}};
const Segment SGI220761[] = {{&S88277[34], 5}, {NULL}};
const Segment SGI220762[] = {{&S88277[27], 6}, {NULL}};
const Segment SGI220763[] = {{&S88277[92], 9}, {NULL}};
const Segment SGI220764[] = {{&S88277[65], 6}, {NULL}};
const Segment SGI220765[] = {{&S92963[44], 7}, {NULL}};
const Segment SGI220766[] = {{&S92963[44], 24}, {NULL}};
const Segment SGI220767[] = {{&S92963[30], 6}, {NULL}};
const Segment SGI220768[] = {{&S92963[31], 3}, {NULL}};
const Segment SGI220769[] = {{&S92963[82], 5}, {NULL}};
const Segment SGI220770[] = {{&S92963[119], 14}, {NULL}};
const Segment SGI220771[] = {{&S74216[34], 18}, {NULL}};
const Segment SGI220772[] = {{&S74216[27], 6}, {NULL}};
const Segment SGI220773[] = {{&S74216[105], 9}, {NULL}};
const Segment SGI220774[] = {{&S74216[78], 6}, {NULL}};
const Segment SGI220775[] = {{&S92968[44], 12}, {NULL}};
const Segment SGI220776[] = {{&S92968[44], 24}, {NULL}};
const Segment SGI220777[] = {{&S92968[30], 6}, {NULL}};
const Segment SGI220778[] = {{&S92968[31], 3}, {NULL}};
const Segment SGI220779[] = {{&S92968[82], 6}, {NULL}};
const Segment SGI220780[] = {{&S92969[38], 12}, {NULL}};
const Segment SGI220781[] = {{&S92969[31], 6}, {NULL}};
const Segment SGI220782[] = {{&S82133[44], 15}, {NULL}};
const Segment SGI220783[] = {{&S82133[44], 39}, {NULL}};
const Segment SGI220784[] = {{&S82133[30], 6}, {NULL}};
const Segment SGI220785[] = {{&S82133[132], 3}, {NULL}};
const Segment SGI220786[] = {{&S82133[97], 6}, {NULL}};
const Segment SGI220787[] = {{&S82133[157], 11}, {NULL}};
const Segment SGI220788[] = {{&S92978[34], 14}, {NULL}};
const Segment SGI220789[] = {{&S92978[27], 6}, {NULL}};
const Segment SGI220790[] = {{&S92978[101], 9}, {NULL}};
const Segment SGI220791[] = {{&S92978[74], 6}, {NULL}};
const Segment SGI220792[] = {{&S92980[34], 10}, {NULL}};
const Segment SGI220793[] = {{&S92980[27], 6}, {NULL}};
const Segment SGI220794[] = {{&S92980[97], 9}, {NULL}};
const Segment SGI220795[] = {{&S92980[70], 6}, {NULL}};
const Segment SGI220796[] = {{&S92982[34], 7}, {NULL}};
const Segment SGI220797[] = {{&S92982[27], 6}, {NULL}};
const Segment SGI220798[] = {{&S92982[99], 9}, {NULL}};
const Segment SGI220799[] = {{&S92982[72], 6}, {NULL}};
const Segment SGI220800[] = {{&S92989[34], 7}, {NULL}};
const Segment SGI220801[] = {{&S92989[27], 6}, {NULL}};
const Segment SGI220802[] = {{&S92989[95], 9}, {NULL}};
const Segment SGI220803[] = {{&S92989[68], 6}, {NULL}};
const Segment SGI220804[] = {{&S83370[44], 3}, {NULL}};
const Segment SGI220805[] = {{&S83370[44], 12}, {NULL}};
const Segment SGI220806[] = {{&S83370[30], 6}, {NULL}};
const Segment SGI220807[] = {{&S83370[70], 6}, {NULL}};
const Segment SGI220808[] = {{&S92996[44], 9}, {NULL}};
const Segment SGI220809[] = {{&S83370[107], 3}, {NULL}};
const Segment SGI220810[] = {{&S92996[44], 21}, {NULL}};
const Segment SGI220811[] = {{&S92996[30], 6}, {NULL}};
const Segment SGI220812[] = {{&S92996[31], 3}, {NULL}};
const Segment SGI220813[] = {{&S92996[108], 6}, {NULL}};
const Segment SGI220814[] = {{&S92996[67], 8}, {NULL}};
const Segment SGI220815[] = {{&S92996[79], 15}, {NULL}};
const Segment SGI220816[] = {{&S82163[34], 9}, {NULL}};
const Segment SGI220817[] = {{&S82163[27], 6}, {NULL}};
const Segment SGI220818[] = {{&S82163[96], 9}, {NULL}};
const Segment SGI220819[] = {{&S83373[44], 17}, {NULL}};
const Segment SGI220820[] = {{&S82163[69], 6}, {NULL}};
const Segment SGI220821[] = {{&S83373[44], 37}, {NULL}};
const Segment SGI220822[] = {{&S83373[30], 6}, {NULL}};
const Segment SGI220823[] = {{&S83373[95], 6}, {NULL}};
const Segment SGI220824[] = {{&S83373[132], 3}, {NULL}};
const Segment SGI220825[] = {{&S82167[34], 13}, {NULL}};
const Segment SGI220826[] = {{&S82167[27], 6}, {NULL}};
const Segment SGI220827[] = {{&S82167[100], 9}, {NULL}};
const Segment SGI220828[] = {{&S82167[73], 6}, {NULL}};
const Segment SGI220829[] = {{&S82168[34], 6}, {NULL}};
const Segment SGI220830[] = {{&S82168[27], 6}, {NULL}};
const Segment SGI220831[] = {{&S82168[104], 9}, {NULL}};
const Segment SGI220832[] = {{&S82168[77], 6}, {NULL}};
const Segment SGI220833[] = {{&S93006[44], 18}, {NULL}};
const Segment SGI220834[] = {{&S93006[44], 30}, {NULL}};
const Segment SGI220835[] = {{&S93006[30], 6}, {NULL}};
const Segment SGI220836[] = {{&S93006[31], 3}, {NULL}};
const Segment SGI220837[] = {{&S93006[88], 6}, {NULL}};
const Segment SGI220838[] = {{&S93006[148], 7}, {NULL}};
const Segment SGI220839[] = {{&S82169[34], 9}, {NULL}};
const Segment SGI220840[] = {{&S82169[27], 6}, {NULL}};
const Segment SGI220841[] = {{&S82169[96], 9}, {NULL}};
const Segment SGI220842[] = {{&S82169[69], 6}, {NULL}};
const Segment SGI220843[] = {{&S93007[52], 7}, {NULL}};
const Segment SGI220844[] = {{&S93007[52], 25}, {NULL}};
const Segment SGI220845[] = {{&S82169[137], 8}, {NULL}};
const Segment SGI220846[] = {{&S93007[38], 6}, {NULL}};
const Segment SGI220847[] = {{&S93007[91], 5}, {NULL}};
const Segment SGI220848[] = {{&S93007[0], 14}, {NULL}};
const Segment SGI220849[] = {{&S88349[34], 25}, {NULL}};
const Segment SGI220850[] = {{&S88349[27], 6}, {NULL}};
const Segment SGI220851[] = {{&S88349[112], 9}, {NULL}};
const Segment SGI220852[] = {{&S88349[85], 6}, {NULL}};
const Segment SGI220853[] = {{&S93017[52], 5}, {NULL}};
const Segment SGI220854[] = {{&S93017[52], 21}, {NULL}};
const Segment SGI220855[] = {{&S93017[38], 6}, {NULL}};
const Segment SGI220856[] = {{&S93017[87], 5}, {NULL}};
const Segment SGI220857[] = {{&S93017[0], 14}, {NULL}};
const Segment SGI220858[] = {{&S93022[34], 15}, {NULL}};
const Segment SGI220859[] = {{&S93022[27], 6}, {NULL}};
const Segment SGI220860[] = {{&S93022[102], 9}, {NULL}};
const Segment SGI220861[] = {{&S93022[75], 6}, {NULL}};
const Segment SGI220862[] = {{&S93024[34], 12}, {NULL}};
const Segment SGI220863[] = {{&S93024[27], 6}, {NULL}};
const Segment SGI220864[] = {{&S93024[99], 9}, {NULL}};
const Segment SGI220865[] = {{&S93024[72], 6}, {NULL}};
const Segment SGI220866[] = {{&S88362[34], 6}, {NULL}};
const Segment SGI220867[] = {{&S88362[27], 6}, {NULL}};
const Segment SGI220868[] = {{&S88362[93], 9}, {NULL}};
const Segment SGI220869[] = {{&S88362[66], 6}, {NULL}};
const Segment SGI220870[] = {{&S93028[44], 10}, {NULL}};
const Segment SGI220871[] = {{&S93028[44], 22}, {NULL}};
const Segment SGI220872[] = {{&S93028[30], 6}, {NULL}};
const Segment SGI220873[] = {{&S93028[80], 6}, {NULL}};
const Segment SGI220874[] = {{&S93028[107], 13}, {NULL}};
const Segment SGI220875[] = {{&S93036[34], 8}, {NULL}};
const Segment SGI220876[] = {{&S88373[34], 14}, {NULL}};
const Segment SGI220877[] = {{&S93036[27], 6}, {NULL}};
const Segment SGI220878[] = {{&S88373[27], 6}, {NULL}};
const Segment SGI220879[] = {{&S93036[95], 9}, {NULL}};
const Segment SGI220880[] = {{&S88373[101], 9}, {NULL}};
const Segment SGI220881[] = {{&S93036[68], 6}, {NULL}};
const Segment SGI220882[] = {{&S88373[74], 6}, {NULL}};
const Segment SGI220883[] = {{&S93036[136], 8}, {NULL}};
const Segment SGI220884[] = {{&S88377[34], 10}, {NULL}};
const Segment SGI220885[] = {{&S88377[27], 6}, {NULL}};
const Segment SGI220886[] = {{&S88377[97], 9}, {NULL}};
const Segment SGI220887[] = {{&S88377[70], 6}, {NULL}};
const Segment SGI220888[] = {{&S82212[44], 10}, {NULL}};
const Segment SGI220889[] = {{&S82212[44], 22}, {NULL}};
const Segment SGI220890[] = {{&S82212[30], 6}, {NULL}};
const Segment SGI220891[] = {{&S82212[115], 3}, {NULL}};
const Segment SGI220892[] = {{&S82212[80], 6}, {NULL}};
const Segment SGI220893[] = {{&S82213[45], 10}, {NULL}};
const Segment SGI220894[] = {{&S82213[45], 73}, {NULL}};
const Segment SGI220895[] = {{&S82213[31], 6}, {NULL}};
const Segment SGI220896[] = {{&S82214[38], 10}, {NULL}};
const Segment SGI220897[] = {{&S82214[31], 6}, {NULL}};
const Segment SGI220898[] = {{&S88390[34], 6}, {NULL}};
const Segment SGI220899[] = {{&S88390[27], 6}, {NULL}};
const Segment SGI220900[] = {{&S88390[93], 9}, {NULL}};
const Segment SGI220901[] = {{&S88390[66], 6}, {NULL}};
const Segment SGI220902[] = {{&S88390[134], 8}, {NULL}};
const Segment SGI220903[] = {{&S88391[34], 9}, {NULL}};
const Segment SGI220904[] = {{&S88391[27], 6}, {NULL}};
const Segment SGI220905[] = {{&S88391[96], 9}, {NULL}};
const Segment SGI220906[] = {{&S88391[69], 6}, {NULL}};
const Segment SGI220907[] = {{&S88391[130], 8}, {NULL}};
const Segment SGI220908[] = {{&S88392[34], 9}, {NULL}};
const Segment SGI220909[] = {{&S88392[27], 6}, {NULL}};
const Segment SGI220910[] = {{&S88392[96], 9}, {NULL}};
const Segment SGI220911[] = {{&S88392[69], 6}, {NULL}};
const Segment SGI220912[] = {{&S88392[137], 8}, {NULL}};
const Segment SGI220913[] = {{&S74334[44], 10}, {NULL}};
const Segment SGI220914[] = {{&S74334[44], 22}, {NULL}};
const Segment SGI220915[] = {{&S74334[30], 6}, {NULL}};
const Segment SGI220916[] = {{&S74334[115], 3}, {NULL}};
const Segment SGI220917[] = {{&S74334[80], 6}, {NULL}};
const Segment SGI220918[] = {{&S74334[140], 8}, {NULL}};
const Segment SGI220919[] = {{&S83455[34], 10}, {NULL}};
const Segment SGI220920[] = {{&S83455[27], 6}, {NULL}};
const Segment SGI220921[] = {{&S83455[97], 9}, {NULL}};
const Segment SGI220922[] = {{&S83455[70], 6}, {NULL}};
const Segment SGI220923[] = {{&S93086[34], 13}, {NULL}};
const Segment SGI220924[] = {{&S93086[27], 6}, {NULL}};
const Segment SGI220925[] = {{&S93086[100], 9}, {NULL}};
const Segment SGI220926[] = {{&S93086[73], 6}, {NULL}};
const Segment SGI220927[] = {{&S93092[57], 10}, {NULL}};
const Segment SGI220928[] = {{&S93092[57], 22}, {NULL}};
const Segment SGI220929[] = {{&S93092[43], 6}, {NULL}};
const Segment SGI220930[] = {{&S93092[44], 3}, {NULL}};
const Segment SGI220931[] = {{&S93092[93], 6}, {NULL}};
const Segment SGI220932[] = {{&S93092[0], 12}, {NULL}};
const Segment SGI220933[] = {{&S83477[34], 15}, {NULL}};
const Segment SGI220934[] = {{&S83477[27], 6}, {NULL}};
const Segment SGI220935[] = {{&S93094[55], 10}, {NULL}};
const Segment SGI220936[] = {{&S93094[43], 6}, {NULL}};
const Segment SGI220937[] = {{&S83477[102], 9}, {NULL}};
const Segment SGI220938[] = {{&S83477[75], 6}, {NULL}};
const Segment SGI220939[] = {{&S93094[44], 3}, {NULL}};
const Segment SGI220940[] = {{&S93094[91], 6}, {NULL}};
const Segment SGI220941[] = {{&S93094[0], 12}, {NULL}};
const Segment SGI220942[] = {{&S93112[44], 9}, {NULL}};
const Segment SGI220943[] = {{&S93112[44], 21}, {NULL}};
const Segment SGI220944[] = {{&S93112[30], 6}, {NULL}};
const Segment SGI220945[] = {{&S93112[31], 3}, {NULL}};
const Segment SGI220946[] = {{&S93112[79], 6}, {NULL}};
const Segment SGI220947[] = {{&S88447[44], 10}, {NULL}};
const Segment SGI220948[] = {{&S88447[44], 22}, {NULL}};
const Segment SGI220949[] = {{&S88447[30], 6}, {NULL}};
const Segment SGI220950[] = {{&S88447[115], 3}, {NULL}};
const Segment SGI220951[] = {{&S88447[80], 6}, {NULL}};
const Segment SGI220952[] = {{&S96246[44], 14}, {NULL}};
const Segment SGI220953[] = {{&S96246[44], 26}, {NULL}};
const Segment SGI220954[] = {{&S96246[30], 6}, {NULL}};
const Segment SGI220955[] = {{&S96246[84], 6}, {NULL}};
const Segment SGI220956[] = {{&S88453[34], 6}, {NULL}};
const Segment SGI220957[] = {{&S74393[34], 7}, {NULL}};
const Segment SGI220958[] = {{&S88453[27], 6}, {NULL}};
const Segment SGI220959[] = {{&S74393[27], 6}, {NULL}};
const Segment SGI220960[] = {{&S88453[93], 9}, {NULL}};
const Segment SGI220961[] = {{&S74393[93], 9}, {NULL}};
const Segment SGI220962[] = {{&S88453[66], 6}, {NULL}};
const Segment SGI220963[] = {{&S74393[66], 6}, {NULL}};
const Segment SGI220964[] = {{&S88464[44], 7}, {NULL}};
const Segment SGI220965[] = {{&S88464[44], 24}, {NULL}};
const Segment SGI220966[] = {{&S88464[30], 6}, {NULL}};
const Segment SGI220967[] = {{&S93137[34], 10}, {NULL}};
const Segment SGI220968[] = {{&S93137[27], 6}, {NULL}};
const Segment SGI220969[] = {{&S88464[117], 3}, {NULL}};
const Segment SGI220970[] = {{&S88464[82], 6}, {NULL}};
const Segment SGI220971[] = {{&S93137[97], 9}, {NULL}};
const Segment SGI220972[] = {{&S88464[142], 11}, {NULL}};
const Segment SGI220973[] = {{&S93137[70], 6}, {NULL}};
const Segment SGI220974[] = {{&S93138[34], 21}, {NULL}};
const Segment SGI220975[] = {{&S93138[27], 6}, {NULL}};
const Segment SGI220976[] = {{&S93138[108], 9}, {NULL}};
const Segment SGI220977[] = {{&S93138[81], 6}, {NULL}};
const Segment SGI220978[] = {{&S93139[34], 8}, {NULL}};
const Segment SGI220979[] = {{&S93139[27], 6}, {NULL}};
const Segment SGI220980[] = {{&S93139[95], 9}, {NULL}};
const Segment SGI220981[] = {{&S93139[68], 6}, {NULL}};
const Segment SGI220982[] = {{&S93141[34], 8}, {NULL}};
const Segment SGI220983[] = {{&S93141[27], 6}, {NULL}};
const Segment SGI220984[] = {{&S93141[95], 9}, {NULL}};
const Segment SGI220985[] = {{&S93141[68], 6}, {NULL}};
const Segment SGI220986[] = {{&S93142[34], 7}, {NULL}};
const Segment SGI220987[] = {{&S93142[27], 6}, {NULL}};
const Segment SGI220988[] = {{&S93142[98], 9}, {NULL}};
const Segment SGI220989[] = {{&S93142[71], 6}, {NULL}};
const Segment SGI220990[] = {{&S88473[44], 10}, {NULL}};
const Segment SGI220991[] = {{&S88473[44], 22}, {NULL}};
const Segment SGI220992[] = {{&S88473[30], 6}, {NULL}};
const Segment SGI220993[] = {{&S88473[115], 3}, {NULL}};
const Segment SGI220994[] = {{&S88473[80], 6}, {NULL}};
const Segment SGI220995[] = {{&S88473[133], 11}, {NULL}};
const Segment SGI220996[] = {{&S93144[34], 14}, {NULL}};
const Segment SGI220997[] = {{&S93144[27], 6}, {NULL}};
const Segment SGI220998[] = {{&S93144[114], 9}, {NULL}};
const Segment SGI220999[] = {{&S93144[87], 6}, {NULL}};
const Segment SGI221000[] = {{&S93148[34], 11}, {NULL}};
const Segment SGI221001[] = {{&S93148[27], 6}, {NULL}};
const Segment SGI221002[] = {{&S93148[98], 9}, {NULL}};
const Segment SGI221003[] = {{&S93148[71], 6}, {NULL}};
const Segment SGI221004[] = {{&S93149[34], 13}, {NULL}};
const Segment SGI221005[] = {{&S93149[27], 6}, {NULL}};
const Segment SGI221006[] = {{&S93149[100], 9}, {NULL}};
const Segment SGI221007[] = {{&S93149[73], 6}, {NULL}};
const Segment SGI221008[] = {{&S93151[34], 5}, {NULL}};
const Segment SGI221009[] = {{&S93151[27], 6}, {NULL}};
const Segment SGI221010[] = {{&S93151[92], 9}, {NULL}};
const Segment SGI221011[] = {{&S93151[65], 6}, {NULL}};
const Segment SGI221012[] = {{&S93153[34], 4}, {NULL}};
const Segment SGI221013[] = {{&S93153[27], 6}, {NULL}};
const Segment SGI221014[] = {{&S93153[91], 9}, {NULL}};
const Segment SGI221015[] = {{&S93153[64], 6}, {NULL}};
const Segment SGI221016[] = {{&S74433[44], 9}, {NULL}};
const Segment SGI221017[] = {{&S74433[44], 20}, {NULL}};
const Segment SGI221018[] = {{&S74433[30], 6}, {NULL}};
const Segment SGI221019[] = {{&S74433[113], 3}, {NULL}};
const Segment SGI221020[] = {{&S74433[78], 6}, {NULL}};
const Segment SGI221021[] = {{&S74433[145], 15}, {NULL}};
const Segment SGI221022[] = {{&S74434[44], 9}, {NULL}};
const Segment SGI221023[] = {{&S74434[44], 21}, {NULL}};
const Segment SGI221024[] = {{&S74434[30], 6}, {NULL}};
const Segment SGI221025[] = {{&S74434[114], 3}, {NULL}};
const Segment SGI221026[] = {{&S74434[79], 6}, {NULL}};
const Segment SGI221027[] = {{&S74434[146], 15}, {NULL}};
const Segment SGI221028[] = {{&S93158[44], 11}, {NULL}};
const Segment SGI221029[] = {{&S83555[34], 6}, {NULL}};
const Segment SGI221030[] = {{&S93158[44], 23}, {NULL}};
const Segment SGI221031[] = {{&S93158[30], 6}, {NULL}};
const Segment SGI221032[] = {{&S83555[27], 6}, {NULL}};
const Segment SGI221033[] = {{&S93158[31], 3}, {NULL}};
const Segment SGI221034[] = {{&S83555[93], 9}, {NULL}};
const Segment SGI221035[] = {{&S93158[81], 6}, {NULL}};
const Segment SGI221036[] = {{&S83555[66], 6}, {NULL}};
const Segment SGI221037[] = {{&S93160[34], 7}, {NULL}};
const Segment SGI221038[] = {{&S93160[27], 6}, {NULL}};
const Segment SGI221039[] = {{&S93160[94], 9}, {NULL}};
const Segment SGI221040[] = {{&S93160[67], 6}, {NULL}};
const Segment SGI221041[] = {{&S74440[34], 12}, {NULL}};
const Segment SGI221042[] = {{&S74440[27], 6}, {NULL}};
const Segment SGI221043[] = {{&S74440[98], 9}, {NULL}};
const Segment SGI221044[] = {{&S74440[71], 6}, {NULL}};
const Segment SGI221045[] = {{&S93166[34], 11}, {NULL}};
const Segment SGI221046[] = {{&S93166[27], 6}, {NULL}};
const Segment SGI221047[] = {{&S93166[98], 9}, {NULL}};
const Segment SGI221048[] = {{&S93166[71], 6}, {NULL}};
const Segment SGI221049[] = {{&S88507[34], 7}, {NULL}};
const Segment SGI221050[] = {{&S88507[27], 6}, {NULL}};
const Segment SGI221051[] = {{&S88507[94], 9}, {NULL}};
const Segment SGI221052[] = {{&S88507[67], 6}, {NULL}};
const Segment SGI221053[] = {{&S93172[44], 10}, {NULL}};
const Segment SGI221054[] = {{&S93172[44], 22}, {NULL}};
const Segment SGI221055[] = {{&S93172[30], 6}, {NULL}};
const Segment SGI221056[] = {{&S93172[31], 3}, {NULL}};
const Segment SGI221057[] = {{&S93172[80], 6}, {NULL}};
const Segment SGI221058[] = {{&S93180[34], 6}, {NULL}};
const Segment SGI221059[] = {{&S93180[27], 6}, {NULL}};
const Segment SGI221060[] = {{&S93180[93], 9}, {NULL}};
const Segment SGI221061[] = {{&S93180[66], 6}, {NULL}};
const Segment SGI221062[] = {{&S88518[44], 21}, {NULL}};
const Segment SGI221063[] = {{&S88518[44], 33}, {NULL}};
const Segment SGI221064[] = {{&S88518[30], 6}, {NULL}};
const Segment SGI221065[] = {{&S88518[126], 3}, {NULL}};
const Segment SGI221066[] = {{&S88518[91], 6}, {NULL}};
const Segment SGI221067[] = {{&S93182[34], 11}, {NULL}};
const Segment SGI221068[] = {{&S93182[27], 6}, {NULL}};
const Segment SGI221069[] = {{&S93182[98], 9}, {NULL}};
const Segment SGI221070[] = {{&S93182[71], 6}, {NULL}};
const Segment SGI221071[] = {{&S88523[34], 7}, {NULL}};
const Segment SGI221072[] = {{&S88523[27], 6}, {NULL}};
const Segment SGI221073[] = {{&S88523[94], 9}, {NULL}};
const Segment SGI221074[] = {{&S88523[67], 6}, {NULL}};
const Segment SGI221075[] = {{&S88524[34], 6}, {NULL}};
const Segment SGI221076[] = {{&S88524[27], 6}, {NULL}};
const Segment SGI221077[] = {{&S88524[93], 9}, {NULL}};
const Segment SGI221078[] = {{&S88524[66], 6}, {NULL}};
const Segment SGI221079[] = {{&S88536[44], 10}, {NULL}};
const Segment SGI221080[] = {{&S88536[30], 6}, {NULL}};
const Segment SGI221081[] = {{&S88536[103], 3}, {NULL}};
const Segment SGI221082[] = {{&S88536[68], 6}, {NULL}};
const Segment SGI221083[] = {{&S82247[44], 6}, {NULL}};
const Segment SGI221084[] = {{&S82247[44], 18}, {NULL}};
const Segment SGI221085[] = {{&S82247[30], 6}, {NULL}};
const Segment SGI221086[] = {{&S82247[76], 6}, {NULL}};
const Segment SGI221087[] = {{&S88546[34], 7}, {NULL}};
const Segment SGI221088[] = {{&S82247[103], 13}, {NULL}};
const Segment SGI221089[] = {{&S88546[27], 6}, {NULL}};
const Segment SGI221090[] = {{&S88546[99], 9}, {NULL}};
const Segment SGI221091[] = {{&S74495[64], 10}, {NULL}};
const Segment SGI221092[] = {{&S88546[72], 6}, {NULL}};
const Segment SGI221093[] = {{&S74495[64], 21}, {NULL}};
const Segment SGI221094[] = {{&S74495[50], 6}, {NULL}};
const Segment SGI221095[] = {{&S88546[140], 8}, {NULL}};
const Segment SGI221096[] = {{&S74496[44], 10}, {NULL}};
const Segment SGI221097[] = {{&S74496[44], 21}, {NULL}};
const Segment SGI221098[] = {{&S74496[30], 6}, {NULL}};
const Segment SGI221099[] = {{&S74496[114], 3}, {NULL}};
const Segment SGI221100[] = {{&S74496[79], 6}, {NULL}};
const Segment SGI221101[] = {{&S74497[38], 10}, {NULL}};
const Segment SGI221102[] = {{&S74497[31], 6}, {NULL}};
const Segment SGI221103[] = {{&S82253[34], 4}, {NULL}};
const Segment SGI221104[] = {{&S82253[27], 6}, {NULL}};
const Segment SGI221105[] = {{&S82253[91], 9}, {NULL}};
const Segment SGI221106[] = {{&S82253[64], 6}, {NULL}};
const Segment SGI221107[] = {{&S74505[34], 15}, {NULL}};
const Segment SGI221108[] = {{&S74505[27], 6}, {NULL}};
const Segment SGI221109[] = {{&S74505[108], 9}, {NULL}};
const Segment SGI221110[] = {{&S74505[81], 6}, {NULL}};
const Segment SGI221111[] = {{&S82287[34], 14}, {NULL}};
const Segment SGI221112[] = {{&S82287[27], 6}, {NULL}};
const Segment SGI221113[] = {{&S82287[101], 9}, {NULL}};
const Segment SGI221114[] = {{&S82287[74], 6}, {NULL}};
const Segment SGI221115[] = {{&S88608[44], 12}, {NULL}};
const Segment SGI221116[] = {{&S88608[44], 24}, {NULL}};
const Segment SGI221117[] = {{&S88608[30], 6}, {NULL}};
const Segment SGI221118[] = {{&S88608[146], 3}, {NULL}};
const Segment SGI221119[] = {{&S88608[111], 6}, {NULL}};
const Segment SGI221120[] = {{&S88608[70], 8}, {NULL}};
const Segment SGI221121[] = {{&S88608[82], 15}, {NULL}};
const Segment SGI221122[] = {{&S83690[34], 10}, {NULL}};
const Segment SGI221123[] = {{&S83690[27], 6}, {NULL}};
const Segment SGI221124[] = {{&S83690[97], 9}, {NULL}};
const Segment SGI221125[] = {{&S83690[70], 6}, {NULL}};
const Segment SGI221126[] = {{&S83698[34], 15}, {NULL}};
const Segment SGI221127[] = {{&S83698[27], 6}, {NULL}};
const Segment SGI221128[] = {{&S83698[102], 9}, {NULL}};
const Segment SGI221129[] = {{&S83698[75], 6}, {NULL}};
const Segment SGI221130[] = {{&S88624[59], 10}, {NULL}};
const Segment SGI221131[] = {{&S88624[59], 22}, {NULL}};
const Segment SGI221132[] = {{&S88624[45], 6}, {NULL}};
const Segment SGI221133[] = {{&S88624[130], 3}, {NULL}};
const Segment SGI221134[] = {{&S88624[95], 6}, {NULL}};
const Segment SGI221135[] = {{&S88624[0], 14}, {NULL}};
const Segment SGI221136[] = {{&S83701[44], 10}, {NULL}};
const Segment SGI221137[] = {{&S83701[44], 22}, {NULL}};
const Segment SGI221138[] = {{&S83701[30], 6}, {NULL}};
const Segment SGI221139[] = {{&S83701[115], 3}, {NULL}};
const Segment SGI221140[] = {{&S83701[80], 6}, {NULL}};
const Segment SGI221141[] = {{&S82324[34], 11}, {NULL}};
const Segment SGI221142[] = {{&S82324[27], 6}, {NULL}};
const Segment SGI221143[] = {{&S82324[98], 9}, {NULL}};
const Segment SGI221144[] = {{&S82324[71], 6}, {NULL}};
const Segment SGI221145[] = {{&S88629[34], 10}, {NULL}};
const Segment SGI221146[] = {{&S88629[27], 6}, {NULL}};
const Segment SGI221147[] = {{&S88629[97], 9}, {NULL}};
const Segment SGI221148[] = {{&S88629[70], 6}, {NULL}};
const Segment SGI221149[] = {{&S82325[34], 10}, {NULL}};
const Segment SGI221150[] = {{&S82325[27], 6}, {NULL}};
const Segment SGI221151[] = {{&S82325[97], 9}, {NULL}};
const Segment SGI221152[] = {{&S82325[70], 6}, {NULL}};
const Segment SGI221153[] = {{&S88640[34], 23}, {NULL}};
const Segment SGI221154[] = {{&S88640[27], 6}, {NULL}};
const Segment SGI221155[] = {{&S88640[110], 9}, {NULL}};
const Segment SGI221156[] = {{&S88640[83], 6}, {NULL}};
const Segment SGI221157[] = {{&S88643[34], 7}, {NULL}};
const Segment SGI221158[] = {{&S88643[27], 6}, {NULL}};
const Segment SGI221159[] = {{&S88643[94], 9}, {NULL}};
const Segment SGI221160[] = {{&S88643[67], 6}, {NULL}};
const Segment SGI221161[] = {{&S88644[44], 5}, {NULL}};
const Segment SGI221162[] = {{&S88644[44], 17}, {NULL}};
const Segment SGI221163[] = {{&S88644[30], 6}, {NULL}};
const Segment SGI221164[] = {{&S88644[110], 3}, {NULL}};
const Segment SGI221165[] = {{&S88644[75], 6}, {NULL}};
const Segment SGI221166[] = {{&S88645[34], 15}, {NULL}};
const Segment SGI221167[] = {{&S88645[27], 6}, {NULL}};
const Segment SGI221168[] = {{&S88645[102], 9}, {NULL}};
const Segment SGI221169[] = {{&S88645[75], 6}, {NULL}};
const Segment SGI221170[] = {{&S82347[34], 17}, {NULL}};
const Segment SGI221171[] = {{&S82347[27], 6}, {NULL}};
const Segment SGI221172[] = {{&S82347[104], 9}, {NULL}};
const Segment SGI221173[] = {{&S82347[77], 6}, {NULL}};
const Segment SGI221174[] = {{&S82349[34], 8}, {NULL}};
const Segment SGI221175[] = {{&S82349[27], 6}, {NULL}};
const Segment SGI221176[] = {{&S82349[100], 9}, {NULL}};
const Segment SGI221177[] = {{&S82349[73], 6}, {NULL}};
const Segment SGI221178[] = {{&S82355[44], 6}, {NULL}};
const Segment SGI221179[] = {{&S82355[44], 18}, {NULL}};
const Segment SGI221180[] = {{&S82355[30], 6}, {NULL}};
const Segment SGI221181[] = {{&S82355[76], 6}, {NULL}};
const Segment SGI221182[] = {{&S82355[103], 13}, {NULL}};
const Segment SGI221183[] = {{&S93185[34], 7}, {NULL}};
const Segment SGI221184[] = {{&S93185[27], 6}, {NULL}};
const Segment SGI221185[] = {{&S93185[94], 9}, {NULL}};
const Segment SGI221186[] = {{&S93185[67], 6}, {NULL}};
const Segment SGI221187[] = {{&S88677[34], 12}, {NULL}};
const Segment SGI221188[] = {{&S88677[27], 6}, {NULL}};
const Segment SGI221189[] = {{&S88677[99], 9}, {NULL}};
const Segment SGI221190[] = {{&S88677[72], 6}, {NULL}};
const Segment SGI221191[] = {{&S88679[34], 41}, {NULL}};
const Segment SGI221192[] = {{&S88679[27], 6}, {NULL}};
const Segment SGI221193[] = {{&S88679[128], 9}, {NULL}};
const Segment SGI221194[] = {{&S88679[101], 6}, {NULL}};
const Segment SGI221195[] = {{&S93203[44], 11}, {NULL}};
const Segment SGI221196[] = {{&S93203[44], 23}, {NULL}};
const Segment SGI221197[] = {{&S93203[30], 6}, {NULL}};
const Segment SGI221198[] = {{&S93203[81], 6}, {NULL}};
const Segment SGI221199[] = {{&S93203[108], 13}, {NULL}};
const Segment SGI221200[] = {{&S87107[32], 12}, {NULL}};
const Segment SGI221201[] = {{&S87107[27], 4}, {NULL}};
const Segment SGI221202[] = {{&S82385[44], 7}, {NULL}};
const Segment SGI221203[] = {{&S82385[44], 19}, {NULL}};
const Segment SGI221204[] = {{&S82385[30], 6}, {NULL}};
const Segment SGI221205[] = {{&S87107[97], 9}, {NULL}};
const Segment SGI221206[] = {{&S87107[70], 6}, {NULL}};
const Segment SGI221207[] = {{&S82385[128], 3}, {NULL}};
const Segment SGI221208[] = {{&S82385[93], 6}, {NULL}};
const Segment SGI221209[] = {{&S82385[65], 8}, {NULL}};
const Segment SGI221210[] = {{&S93209[34], 10}, {NULL}};
const Segment SGI221211[] = {{&S93209[27], 6}, {NULL}};
const Segment SGI221212[] = {{&S93209[97], 9}, {NULL}};
const Segment SGI221213[] = {{&S93209[70], 6}, {NULL}};
const Segment SGI221214[] = {{&S93210[52], 6}, {NULL}};
const Segment SGI221215[] = {{&S93210[52], 18}, {NULL}};
const Segment SGI221216[] = {{&S93210[38], 6}, {NULL}};
const Segment SGI221217[] = {{&S93210[84], 5}, {NULL}};
const Segment SGI221218[] = {{&S93210[0], 14}, {NULL}};
const Segment SGI221219[] = {{&S93213[34], 7}, {NULL}};
const Segment SGI221220[] = {{&S93213[27], 6}, {NULL}};
const Segment SGI221221[] = {{&S93213[95], 9}, {NULL}};
const Segment SGI221222[] = {{&S93213[68], 6}, {NULL}};
const Segment SGI221223[] = {{&S93218[34], 9}, {NULL}};
const Segment SGI221224[] = {{&S93218[27], 6}, {NULL}};
const Segment SGI221225[] = {{&S93218[96], 9}, {NULL}};
const Segment SGI221226[] = {{&S93218[69], 6}, {NULL}};
const Segment SGI221227[] = {{&S93228[44], 14}, {NULL}};
const Segment SGI221228[] = {{&S93228[44], 26}, {NULL}};
const Segment SGI221229[] = {{&S93228[30], 6}, {NULL}};
const Segment SGI221230[] = {{&S82414[44], 7}, {NULL}};
const Segment SGI221231[] = {{&S93228[84], 6}, {NULL}};
const Segment SGI221232[] = {{&S82414[44], 19}, {NULL}};
const Segment SGI221233[] = {{&S82414[30], 6}, {NULL}};
const Segment SGI221234[] = {{&S93228[111], 13}, {NULL}};
const Segment SGI221235[] = {{&S82414[112], 3}, {NULL}};
const Segment SGI221236[] = {{&S82414[77], 6}, {NULL}};
const Segment SGI221237[] = {{&S93231[34], 7}, {NULL}};
const Segment SGI221238[] = {{&S93231[27], 6}, {NULL}};
const Segment SGI221239[] = {{&S93231[99], 9}, {NULL}};
const Segment SGI221240[] = {{&S93231[72], 6}, {NULL}};
const Segment SGI221241[] = {{&S93231[140], 8}, {NULL}};
const Segment SGI221242[] = {{&S74680[34], 11}, {NULL}};
const Segment SGI221243[] = {{&S74680[27], 6}, {NULL}};
const Segment SGI221244[] = {{&S74680[97], 9}, {NULL}};
const Segment SGI221245[] = {{&S74680[70], 6}, {NULL}};
const Segment SGI221246[] = {{&S82433[34], 8}, {NULL}};
const Segment SGI221247[] = {{&S82433[27], 6}, {NULL}};
const Segment SGI221248[] = {{&S82433[100], 9}, {NULL}};
const Segment SGI221249[] = {{&S82433[73], 6}, {NULL}};
const Segment SGI221250[] = {{&S82435[34], 10}, {NULL}};
const Segment SGI221251[] = {{&S82435[27], 6}, {NULL}};
const Segment SGI221252[] = {{&S82435[97], 9}, {NULL}};
const Segment SGI221253[] = {{&S82435[70], 6}, {NULL}};
const Segment SGI221254[] = {{&S93250[34], 18}, {NULL}};
const Segment SGI221255[] = {{&S93250[27], 6}, {NULL}};
const Segment SGI221256[] = {{&S82436[34], 9}, {NULL}};
const Segment SGI221257[] = {{&S82436[27], 6}, {NULL}};
const Segment SGI221258[] = {{&S93250[105], 9}, {NULL}};
const Segment SGI221259[] = {{&S93250[78], 6}, {NULL}};
const Segment SGI221260[] = {{&S88741[59], 10}, {NULL}};
const Segment SGI221261[] = {{&S82436[96], 9}, {NULL}};
const Segment SGI221262[] = {{&S88741[59], 22}, {NULL}};
const Segment SGI221263[] = {{&S82436[69], 6}, {NULL}};
const Segment SGI221264[] = {{&S88741[45], 6}, {NULL}};
const Segment SGI221265[] = {{&S88741[130], 3}, {NULL}};
const Segment SGI221266[] = {{&S88741[95], 6}, {NULL}};
const Segment SGI221267[] = {{&S88741[0], 14}, {NULL}};
const Segment SGI221268[] = {{&S82442[34], 11}, {NULL}};
const Segment SGI221269[] = {{&S82442[27], 6}, {NULL}};
const Segment SGI221270[] = {{&S88746[44], 10}, {NULL}};
const Segment SGI221271[] = {{&S82442[98], 9}, {NULL}};
const Segment SGI221272[] = {{&S82442[71], 6}, {NULL}};
const Segment SGI221273[] = {{&S88746[44], 22}, {NULL}};
const Segment SGI221274[] = {{&S88746[30], 6}, {NULL}};
const Segment SGI221275[] = {{&S88746[158], 3}, {NULL}};
const Segment SGI221276[] = {{&S88746[124], 3}, {NULL}};
const Segment SGI221277[] = {{&S88746[89], 6}, {NULL}};
const Segment SGI221278[] = {{&S88746[144], 8}, {NULL}};
const Segment SGI221279[] = {{&S82443[34], 15}, {NULL}};
const Segment SGI221280[] = {{&S82443[27], 6}, {NULL}};
const Segment SGI221281[] = {{&S82443[102], 9}, {NULL}};
const Segment SGI221282[] = {{&S82443[75], 6}, {NULL}};
const Segment SGI221283[] = {{&S82444[34], 11}, {NULL}};
const Segment SGI221284[] = {{&S82444[27], 6}, {NULL}};
const Segment SGI221285[] = {{&S82444[98], 9}, {NULL}};
const Segment SGI221286[] = {{&S82444[71], 6}, {NULL}};
const Segment SGI221287[] = {{&S93624[42], 11}, {NULL}};
const Segment SGI221288[] = {{&S93624[42], 23}, {NULL}};
const Segment SGI221289[] = {{&S93624[30], 4}, {NULL}};
const Segment SGI221290[] = {{&S93624[114], 3}, {NULL}};
const Segment SGI221291[] = {{&S93624[79], 6}, {NULL}};
const Segment SGI221292[] = {{&S88764[34], 13}, {NULL}};
const Segment SGI221293[] = {{&S88764[27], 6}, {NULL}};
const Segment SGI221294[] = {{&S88764[100], 9}, {NULL}};
const Segment SGI221295[] = {{&S88764[73], 6}, {NULL}};
const Segment SGI221296[] = {{&S88769[34], 20}, {NULL}};
const Segment SGI221297[] = {{&S88769[27], 6}, {NULL}};
const Segment SGI221298[] = {{&S88769[107], 9}, {NULL}};
const Segment SGI221299[] = {{&S88769[80], 6}, {NULL}};
const Segment SGI221300[] = {{&S82481[34], 9}, {NULL}};
const Segment SGI221301[] = {{&S82481[27], 6}, {NULL}};
const Segment SGI221302[] = {{&S82481[96], 9}, {NULL}};
const Segment SGI221303[] = {{&S82481[69], 6}, {NULL}};
const Segment SGI221304[] = {{&S88783[34], 24}, {NULL}};
const Segment SGI221305[] = {{&S88783[27], 6}, {NULL}};
const Segment SGI221306[] = {{&S88783[111], 9}, {NULL}};
const Segment SGI221307[] = {{&S88783[84], 6}, {NULL}};
const Segment SGI221308[] = {{&S82486[38], 14}, {NULL}};
const Segment SGI221309[] = {{&S82486[31], 6}, {NULL}};
const Segment SGI221310[] = {{&S82494[34], 10}, {NULL}};
const Segment SGI221311[] = {{&S82494[27], 6}, {NULL}};
const Segment SGI221312[] = {{&S82494[97], 9}, {NULL}};
const Segment SGI221313[] = {{&S82494[70], 6}, {NULL}};
const Segment SGI221314[] = {{&S88795[34], 12}, {NULL}};
const Segment SGI221315[] = {{&S88795[27], 6}, {NULL}};
const Segment SGI221316[] = {{&S88795[112], 9}, {NULL}};
const Segment SGI221317[] = {{&S88795[85], 6}, {NULL}};
const Segment SGI221318[] = {{&S82498[44], 14}, {NULL}};
const Segment SGI221319[] = {{&S82498[44], 26}, {NULL}};
const Segment SGI221320[] = {{&S82498[30], 6}, {NULL}};
const Segment SGI221321[] = {{&S82498[119], 3}, {NULL}};
const Segment SGI221322[] = {{&S82498[84], 6}, {NULL}};
const Segment SGI221323[] = {{&S82499[44], 14}, {NULL}};
const Segment SGI221324[] = {{&S82499[44], 26}, {NULL}};
const Segment SGI221325[] = {{&S82499[30], 6}, {NULL}};
const Segment SGI221326[] = {{&S82499[119], 3}, {NULL}};
const Segment SGI221327[] = {{&S82499[84], 6}, {NULL}};
const Segment SGI221328[] = {{&S88800[34], 6}, {NULL}};
const Segment SGI221329[] = {{&S88800[27], 6}, {NULL}};
const Segment SGI221330[] = {{&S88800[93], 9}, {NULL}};
const Segment SGI221331[] = {{&S88800[66], 6}, {NULL}};
const Segment SGI221332[] = {{&S88812[34], 25}, {NULL}};
const Segment SGI221333[] = {{&S88812[27], 6}, {NULL}};
const Segment SGI221334[] = {{&S88812[112], 9}, {NULL}};
const Segment SGI221335[] = {{&S88812[85], 6}, {NULL}};
const Segment SGI221336[] = {{&S74786[34], 6}, {NULL}};
const Segment SGI221337[] = {{&S74786[27], 6}, {NULL}};
const Segment SGI221338[] = {{&S74786[92], 9}, {NULL}};
const Segment SGI221339[] = {{&S74786[65], 6}, {NULL}};
const Segment SGI221340[] = {{&S93331[59], 11}, {NULL}};
const Segment SGI221341[] = {{&S93331[59], 23}, {NULL}};
const Segment SGI221342[] = {{&S93331[45], 6}, {NULL}};
const Segment SGI221343[] = {{&S93331[46], 3}, {NULL}};
const Segment SGI221344[] = {{&S93331[96], 6}, {NULL}};
const Segment SGI221345[] = {{&S93331[0], 14}, {NULL}};
const Segment SGI221346[] = {{&S93339[45], 11}, {NULL}};
const Segment SGI221347[] = {{&S93339[30], 6}, {NULL}};
const Segment SGI221348[] = {{&S93339[31], 3}, {NULL}};
const Segment SGI221349[] = {{&S93339[82], 5}, {NULL}};
const Segment SGI221350[] = {{&S93339[140], 13}, {NULL}};
const Segment SGI221351[] = {{&S93341[31], 5}, {NULL}};
const Segment SGI221352[] = {{&S93341[51], 3}, {NULL}};
const Segment SGI221353[] = {{&S93346[34], 8}, {NULL}};
const Segment SGI221354[] = {{&S93346[27], 6}, {NULL}};
const Segment SGI221355[] = {{&S93346[95], 9}, {NULL}};
const Segment SGI221356[] = {{&S93346[68], 6}, {NULL}};
const Segment SGI221357[] = {{&S93347[34], 21}, {NULL}};
const Segment SGI221358[] = {{&S93347[27], 6}, {NULL}};
const Segment SGI221359[] = {{&S93347[108], 9}, {NULL}};
const Segment SGI221360[] = {{&S93347[81], 6}, {NULL}};
const Segment SGI221361[] = {{&S83881[44], 13}, {NULL}};
const Segment SGI221362[] = {{&S83881[44], 25}, {NULL}};
const Segment SGI221363[] = {{&S83881[30], 6}, {NULL}};
const Segment SGI221364[] = {{&S83881[118], 3}, {NULL}};
const Segment SGI221365[] = {{&S83881[83], 6}, {NULL}};
const Segment SGI221366[] = {{&S93351[34], 6}, {NULL}};
const Segment SGI221367[] = {{&S93351[27], 6}, {NULL}};
const Segment SGI221368[] = {{&S93351[93], 9}, {NULL}};
const Segment SGI221369[] = {{&S93351[66], 6}, {NULL}};
const Segment SGI221370[] = {{&S83883[38], 13}, {NULL}};
const Segment SGI221371[] = {{&S74845[44], 10}, {NULL}};
const Segment SGI221372[] = {{&S83883[31], 6}, {NULL}};
const Segment SGI221373[] = {{&S74845[44], 22}, {NULL}};
const Segment SGI221374[] = {{&S74845[30], 6}, {NULL}};
const Segment SGI221375[] = {{&S74845[115], 3}, {NULL}};
const Segment SGI221376[] = {{&S74845[80], 6}, {NULL}};
const Segment SGI221377[] = {{&S74845[140], 8}, {NULL}};
const Segment SGI221378[] = {{&S93353[59], 7}, {NULL}};
const Segment SGI221379[] = {{&S88874[44], 9}, {NULL}};
const Segment SGI221380[] = {{&S93353[59], 23}, {NULL}};
const Segment SGI221381[] = {{&S88874[44], 21}, {NULL}};
const Segment SGI221382[] = {{&S88874[30], 6}, {NULL}};
const Segment SGI221383[] = {{&S74846[34], 10}, {NULL}};
const Segment SGI221384[] = {{&S93353[45], 6}, {NULL}};
const Segment SGI221385[] = {{&S74846[27], 6}, {NULL}};
const Segment SGI221386[] = {{&S88874[114], 3}, {NULL}};
const Segment SGI221387[] = {{&S93353[46], 3}, {NULL}};
const Segment SGI221388[] = {{&S88874[79], 6}, {NULL}};
const Segment SGI221389[] = {{&S93353[96], 6}, {NULL}};
const Segment SGI221390[] = {{&S74846[97], 9}, {NULL}};
const Segment SGI221391[] = {{&S74846[70], 6}, {NULL}};
const Segment SGI221392[] = {{&S93353[0], 14}, {NULL}};
const Segment SGI221393[] = {{&S88875[34], 11}, {NULL}};
const Segment SGI221394[] = {{&S88875[27], 6}, {NULL}};
const Segment SGI221395[] = {{&S88875[98], 9}, {NULL}};
const Segment SGI221396[] = {{&S88875[71], 6}, {NULL}};
const Segment SGI221397[] = {{&S93357[34], 10}, {NULL}};
const Segment SGI221398[] = {{&S93357[27], 6}, {NULL}};
const Segment SGI221399[] = {{&S93357[97], 9}, {NULL}};
const Segment SGI221400[] = {{&S93357[70], 6}, {NULL}};
const Segment SGI221401[] = {{&S93364[45], 5}, {NULL}};
const Segment SGI221402[] = {{&S93364[30], 6}, {NULL}};
const Segment SGI221403[] = {{&S93364[31], 3}, {NULL}};
const Segment SGI221404[] = {{&S93364[76], 5}, {NULL}};
const Segment SGI221405[] = {{&S93364[134], 13}, {NULL}};
const Segment SGI221406[] = {{&S93366[59], 5}, {NULL}};
const Segment SGI221407[] = {{&S93366[59], 17}, {NULL}};
const Segment SGI221408[] = {{&S93366[45], 6}, {NULL}};
const Segment SGI221409[] = {{&S93366[46], 3}, {NULL}};
const Segment SGI221410[] = {{&S93366[90], 6}, {NULL}};
const Segment SGI221411[] = {{&S93366[0], 14}, {NULL}};
const Segment SGI221412[] = {{&S88890[44], 10}, {NULL}};
const Segment SGI221413[] = {{&S88890[44], 22}, {NULL}};
const Segment SGI221414[] = {{&S93367[59], 13}, {NULL}};
const Segment SGI221415[] = {{&S88890[30], 6}, {NULL}};
const Segment SGI221416[] = {{&S93367[59], 25}, {NULL}};
const Segment SGI221417[] = {{&S93367[45], 6}, {NULL}};
const Segment SGI221418[] = {{&S88890[115], 3}, {NULL}};
const Segment SGI221419[] = {{&S93367[46], 3}, {NULL}};
const Segment SGI221420[] = {{&S93367[98], 6}, {NULL}};
const Segment SGI221421[] = {{&S88890[80], 6}, {NULL}};
const Segment SGI221422[] = {{&S88890[143], 3}, {NULL}};
const Segment SGI221423[] = {{&S93367[0], 14}, {NULL}};
const Segment SGI221424[] = {{&S93368[44], 7}, {NULL}};
const Segment SGI221425[] = {{&S93368[44], 19}, {NULL}};
const Segment SGI221426[] = {{&S93368[30], 6}, {NULL}};
const Segment SGI221427[] = {{&S93368[31], 3}, {NULL}};
const Segment SGI221428[] = {{&S93368[77], 6}, {NULL}};
const Segment SGI221429[] = {{&S93370[34], 9}, {NULL}};
const Segment SGI221430[] = {{&S93370[27], 6}, {NULL}};
const Segment SGI221431[] = {{&S93370[132], 9}, {NULL}};
const Segment SGI221432[] = {{&S93370[105], 6}, {NULL}};
const Segment SGI221433[] = {{&S74876[34], 7}, {NULL}};
const Segment SGI221434[] = {{&S74876[27], 6}, {NULL}};
const Segment SGI221435[] = {{&S74876[93], 9}, {NULL}};
const Segment SGI221436[] = {{&S74876[66], 6}, {NULL}};
const Segment SGI221437[] = {{&S93373[34], 8}, {NULL}};
const Segment SGI221438[] = {{&S93373[27], 6}, {NULL}};
const Segment SGI221439[] = {{&S93373[95], 9}, {NULL}};
const Segment SGI221440[] = {{&S93373[68], 6}, {NULL}};
const Segment SGI221441[] = {{&S74878[34], 11}, {NULL}};
const Segment SGI221442[] = {{&S74878[27], 6}, {NULL}};
const Segment SGI221443[] = {{&S74878[97], 9}, {NULL}};
const Segment SGI221444[] = {{&S74878[70], 6}, {NULL}};
const Segment SGI221445[] = {{&S93374[34], 14}, {NULL}};
const Segment SGI221446[] = {{&S93374[27], 6}, {NULL}};
const Segment SGI221447[] = {{&S93374[101], 9}, {NULL}};
const Segment SGI221448[] = {{&S93374[74], 6}, {NULL}};
const Segment SGI221449[] = {{&S74880[34], 5}, {NULL}};
const Segment SGI221450[] = {{&S74880[27], 6}, {NULL}};
const Segment SGI221451[] = {{&S74880[92], 9}, {NULL}};
const Segment SGI221452[] = {{&S74880[65], 6}, {NULL}};
const Segment SGI221453[] = {{&S93376[34], 14}, {NULL}};
const Segment SGI221454[] = {{&S93376[27], 6}, {NULL}};
const Segment SGI221455[] = {{&S93376[101], 9}, {NULL}};
const Segment SGI221456[] = {{&S93376[74], 6}, {NULL}};
const Segment SGI221457[] = {{&S74882[44], 10}, {NULL}};
const Segment SGI221458[] = {{&S74882[44], 21}, {NULL}};
const Segment SGI221459[] = {{&S74882[30], 6}, {NULL}};
const Segment SGI221460[] = {{&S74882[132], 3}, {NULL}};
const Segment SGI221461[] = {{&S74882[97], 6}, {NULL}};
const Segment SGI221462[] = {{&S74882[67], 8}, {NULL}};
const Segment SGI221463[] = {{&S93379[44], 15}, {NULL}};
const Segment SGI221464[] = {{&S93379[44], 27}, {NULL}};
const Segment SGI221465[] = {{&S93379[30], 6}, {NULL}};
const Segment SGI221466[] = {{&S74883[44], 10}, {NULL}};
const Segment SGI221467[] = {{&S74883[44], 22}, {NULL}};
const Segment SGI221468[] = {{&S74883[30], 6}, {NULL}};
const Segment SGI221469[] = {{&S93379[31], 3}, {NULL}};
const Segment SGI221470[] = {{&S93379[85], 6}, {NULL}};
const Segment SGI221471[] = {{&S74883[135], 3}, {NULL}};
const Segment SGI221472[] = {{&S74883[100], 6}, {NULL}};
const Segment SGI221473[] = {{&S74883[68], 8}, {NULL}};
const Segment SGI221474[] = {{&S93382[34], 10}, {NULL}};
const Segment SGI221475[] = {{&S93382[27], 6}, {NULL}};
const Segment SGI221476[] = {{&S93382[97], 9}, {NULL}};
const Segment SGI221477[] = {{&S93382[70], 6}, {NULL}};
const Segment SGI221478[] = {{&S88916[44], 15}, {NULL}};
const Segment SGI221479[] = {{&S88916[44], 27}, {NULL}};
const Segment SGI221480[] = {{&S88916[30], 6}, {NULL}};
const Segment SGI221481[] = {{&S93386[44], 10}, {NULL}};
const Segment SGI221482[] = {{&S88916[118], 3}, {NULL}};
const Segment SGI221483[] = {{&S93386[44], 22}, {NULL}};
const Segment SGI221484[] = {{&S93386[30], 6}, {NULL}};
const Segment SGI221485[] = {{&S88916[85], 5}, {NULL}};
const Segment SGI221486[] = {{&S93386[31], 3}, {NULL}};
const Segment SGI221487[] = {{&S93386[80], 5}, {NULL}};
const Segment SGI221488[] = {{&S93386[117], 14}, {NULL}};
const Segment SGI221489[] = {{&S74893[34], 10}, {NULL}};
const Segment SGI221490[] = {{&S74893[27], 6}, {NULL}};
const Segment SGI221491[] = {{&S82507[34], 6}, {NULL}};
const Segment SGI221492[] = {{&S82507[27], 6}, {NULL}};
const Segment SGI221493[] = {{&S74893[96], 9}, {NULL}};
const Segment SGI221494[] = {{&S74893[69], 6}, {NULL}};
const Segment SGI221495[] = {{&S82507[93], 9}, {NULL}};
const Segment SGI221496[] = {{&S82507[66], 6}, {NULL}};
const Segment SGI221497[] = {{&S93388[34], 5}, {NULL}};
const Segment SGI221498[] = {{&S93388[27], 6}, {NULL}};
const Segment SGI221499[] = {{&S93388[92], 9}, {NULL}};
const Segment SGI221500[] = {{&S93388[65], 6}, {NULL}};
const Segment SGI221501[] = {{&S83951[34], 10}, {NULL}};
const Segment SGI221502[] = {{&S83951[27], 6}, {NULL}};
const Segment SGI221503[] = {{&S83951[97], 9}, {NULL}};
const Segment SGI221504[] = {{&S83951[70], 6}, {NULL}};
const Segment SGI221505[] = {{&S83956[34], 10}, {NULL}};
const Segment SGI221506[] = {{&S83956[27], 6}, {NULL}};
const Segment SGI221507[] = {{&S83956[97], 9}, {NULL}};
const Segment SGI221508[] = {{&S83956[70], 6}, {NULL}};
const Segment SGI221509[] = {{&S74951[34], 7}, {NULL}};
const Segment SGI221510[] = {{&S74951[27], 6}, {NULL}};
const Segment SGI221511[] = {{&S74951[100], 9}, {NULL}};
const Segment SGI221512[] = {{&S74951[73], 6}, {NULL}};
const Segment SGI221513[] = {{&S82587[44], 17}, {NULL}};
const Segment SGI221514[] = {{&S82587[44], 29}, {NULL}};
const Segment SGI221515[] = {{&S82587[30], 6}, {NULL}};
const Segment SGI221516[] = {{&S82587[122], 3}, {NULL}};
const Segment SGI221517[] = {{&S82587[87], 6}, {NULL}};
const Segment SGI221518[] = {{&S82589[44], 10}, {NULL}};
const Segment SGI221519[] = {{&S82589[44], 22}, {NULL}};
const Segment SGI221520[] = {{&S82589[30], 6}, {NULL}};
const Segment SGI221521[] = {{&S82589[115], 3}, {NULL}};
const Segment SGI221522[] = {{&S82589[80], 6}, {NULL}};
const Segment SGI221523[] = {{&S74996[34], 12}, {NULL}};
const Segment SGI221524[] = {{&S74996[27], 6}, {NULL}};
const Segment SGI221525[] = {{&S74996[98], 9}, {NULL}};
const Segment SGI221526[] = {{&S74996[71], 6}, {NULL}};
const Segment SGI221527[] = {{&S82596[34], 15}, {NULL}};
const Segment SGI221528[] = {{&S82596[27], 6}, {NULL}};
const Segment SGI221529[] = {{&S82596[102], 9}, {NULL}};
const Segment SGI221530[] = {{&S82596[75], 6}, {NULL}};
const Segment SGI221531[] = {{&S82597[34], 10}, {NULL}};
const Segment SGI221532[] = {{&S82597[27], 6}, {NULL}};
const Segment SGI221533[] = {{&S82597[97], 9}, {NULL}};
const Segment SGI221534[] = {{&S82597[70], 6}, {NULL}};
const Segment SGI221535[] = {{&S82598[34], 11}, {NULL}};
const Segment SGI221536[] = {{&S82598[27], 6}, {NULL}};
const Segment SGI221537[] = {{&S75001[34], 7}, {NULL}};
const Segment SGI221538[] = {{&S75001[27], 6}, {NULL}};
const Segment SGI221539[] = {{&S82598[98], 9}, {NULL}};
const Segment SGI221540[] = {{&S82598[71], 6}, {NULL}};
const Segment SGI221541[] = {{&S75001[100], 9}, {NULL}};
const Segment SGI221542[] = {{&S75001[73], 6}, {NULL}};
const Segment SGI221543[] = {{&S75002[34], 7}, {NULL}};
const Segment SGI221544[] = {{&S75002[27], 6}, {NULL}};
const Segment SGI221545[] = {{&S75002[100], 9}, {NULL}};
const Segment SGI221546[] = {{&S75002[73], 6}, {NULL}};
const Segment SGI221547[] = {{&S75005[44], 9}, {NULL}};
const Segment SGI221548[] = {{&S75005[44], 20}, {NULL}};
const Segment SGI221549[] = {{&S75005[30], 6}, {NULL}};
const Segment SGI221550[] = {{&S75005[113], 3}, {NULL}};
const Segment SGI221551[] = {{&S75005[78], 6}, {NULL}};
const Segment SGI221552[] = {{&S75005[138], 7}, {NULL}};
const Segment SGI221553[] = {{&S82615[34], 10}, {NULL}};
const Segment SGI221554[] = {{&S82615[27], 6}, {NULL}};
const Segment SGI221555[] = {{&S82615[97], 9}, {NULL}};
const Segment SGI221556[] = {{&S82615[70], 6}, {NULL}};
const Segment SGI221557[] = {{&S89039[34], 13}, {NULL}};
const Segment SGI221558[] = {{&S89039[27], 6}, {NULL}};
const Segment SGI221559[] = {{&S89039[100], 9}, {NULL}};
const Segment SGI221560[] = {{&S89039[73], 6}, {NULL}};
const Segment SGI221561[] = {{&S89040[34], 7}, {NULL}};
const Segment SGI221562[] = {{&S89040[27], 6}, {NULL}};
const Segment SGI221563[] = {{&S89040[94], 9}, {NULL}};
const Segment SGI221564[] = {{&S89040[67], 6}, {NULL}};
const Segment SGI221565[] = {{&S82627[34], 12}, {NULL}};
const Segment SGI221566[] = {{&S82627[27], 6}, {NULL}};
const Segment SGI221567[] = {{&S82627[99], 9}, {NULL}};
const Segment SGI221568[] = {{&S82627[72], 6}, {NULL}};
const Segment SGI221569[] = {{&S89047[44], 10}, {NULL}};
const Segment SGI221570[] = {{&S89047[44], 22}, {NULL}};
const Segment SGI221571[] = {{&S89047[30], 6}, {NULL}};
const Segment SGI221572[] = {{&S89047[115], 3}, {NULL}};
const Segment SGI221573[] = {{&S89047[80], 6}, {NULL}};
const Segment SGI221574[] = {{&S89047[147], 15}, {NULL}};
const Segment SGI221575[] = {{&S82632[34], 7}, {NULL}};
const Segment SGI221576[] = {{&S82632[27], 6}, {NULL}};
const Segment SGI221577[] = {{&S82632[94], 9}, {NULL}};
const Segment SGI221578[] = {{&S82632[67], 6}, {NULL}};
const Segment SGI221579[] = {{&S89049[34], 13}, {NULL}};
const Segment SGI221580[] = {{&S89049[27], 6}, {NULL}};
const Segment SGI221581[] = {{&S89049[114], 9}, {NULL}};
const Segment SGI221582[] = {{&S89049[87], 6}, {NULL}};
const Segment SGI221583[] = {{&S82637[34], 5}, {NULL}};
const Segment SGI221584[] = {{&S82637[27], 6}, {NULL}};
const Segment SGI221585[] = {{&S82637[92], 9}, {NULL}};
const Segment SGI221586[] = {{&S82637[65], 6}, {NULL}};
const Segment SGI221587[] = {{&S82639[34], 9}, {NULL}};
const Segment SGI221588[] = {{&S82639[27], 6}, {NULL}};
const Segment SGI221589[] = {{&S82639[96], 9}, {NULL}};
const Segment SGI221590[] = {{&S82639[69], 6}, {NULL}};
const Segment SGI221591[] = {{&S82640[44], 7}, {NULL}};
const Segment SGI221592[] = {{&S82640[44], 19}, {NULL}};
const Segment SGI221593[] = {{&S82640[30], 6}, {NULL}};
const Segment SGI221594[] = {{&S82640[112], 3}, {NULL}};
const Segment SGI221595[] = {{&S82640[77], 6}, {NULL}};
const Segment SGI221596[] = {{&S89061[44], 18}, {NULL}};
const Segment SGI221597[] = {{&S89061[44], 30}, {NULL}};
const Segment SGI221598[] = {{&S89061[30], 6}, {NULL}};
const Segment SGI221599[] = {{&S89061[123], 3}, {NULL}};
const Segment SGI221600[] = {{&S89061[88], 6}, {NULL}};
const Segment SGI221601[] = {{&S89062[38], 18}, {NULL}};
const Segment SGI221602[] = {{&S89062[31], 6}, {NULL}};
const Segment SGI221603[] = {{&S82644[34], 10}, {NULL}};
const Segment SGI221604[] = {{&S82644[27], 6}, {NULL}};
const Segment SGI221605[] = {{&S82644[97], 9}, {NULL}};
const Segment SGI221606[] = {{&S82644[70], 6}, {NULL}};
const Segment SGI221607[] = {{&S82645[34], 7}, {NULL}};
const Segment SGI221608[] = {{&S89064[34], 15}, {NULL}};
const Segment SGI221609[] = {{&S82645[27], 6}, {NULL}};
const Segment SGI221610[] = {{&S89064[27], 6}, {NULL}};
const Segment SGI221611[] = {{&S82645[99], 9}, {NULL}};
const Segment SGI221612[] = {{&S89064[102], 9}, {NULL}};
const Segment SGI221613[] = {{&S82645[72], 6}, {NULL}};
const Segment SGI221614[] = {{&S89064[75], 6}, {NULL}};
const Segment SGI221615[] = {{&S82646[44], 10}, {NULL}};
const Segment SGI221616[] = {{&S89065[44], 16}, {NULL}};
const Segment SGI221617[] = {{&S82646[44], 43}, {NULL}};
const Segment SGI221618[] = {{&S89065[44], 28}, {NULL}};
const Segment SGI221619[] = {{&S82646[30], 6}, {NULL}};
const Segment SGI221620[] = {{&S89065[30], 6}, {NULL}};
const Segment SGI221621[] = {{&S82646[136], 3}, {NULL}};
const Segment SGI221622[] = {{&S89065[121], 3}, {NULL}};
const Segment SGI221623[] = {{&S82646[101], 6}, {NULL}};
const Segment SGI221624[] = {{&S89065[86], 6}, {NULL}};
const Segment SGI221625[] = {{&S82647[44], 10}, {NULL}};
const Segment SGI221626[] = {{&S82647[44], 46}, {NULL}};
const Segment SGI221627[] = {{&S82647[30], 6}, {NULL}};
const Segment SGI221628[] = {{&S82647[139], 3}, {NULL}};
const Segment SGI221629[] = {{&S82647[104], 6}, {NULL}};
const Segment SGI221630[] = {{&S89067[44], 13}, {NULL}};
const Segment SGI221631[] = {{&S89067[44], 25}, {NULL}};
const Segment SGI221632[] = {{&S89067[30], 6}, {NULL}};
const Segment SGI221633[] = {{&S89067[83], 6}, {NULL}};
const Segment SGI221634[] = {{&S89067[110], 13}, {NULL}};
const Segment SGI221635[] = {{&S89068[44], 11}, {NULL}};
const Segment SGI221636[] = {{&S89068[44], 23}, {NULL}};
const Segment SGI221637[] = {{&S89068[30], 6}, {NULL}};
const Segment SGI221638[] = {{&S89068[116], 3}, {NULL}};
const Segment SGI221639[] = {{&S89068[81], 6}, {NULL}};
const Segment SGI221640[] = {{&S89074[44], 13}, {NULL}};
const Segment SGI221641[] = {{&S89074[44], 25}, {NULL}};
const Segment SGI221642[] = {{&S89074[30], 6}, {NULL}};
const Segment SGI221643[] = {{&S82655[34], 7}, {NULL}};
const Segment SGI221644[] = {{&S89074[118], 3}, {NULL}};
const Segment SGI221645[] = {{&S82655[27], 6}, {NULL}};
const Segment SGI221646[] = {{&S89074[83], 6}, {NULL}};
const Segment SGI221647[] = {{&S82655[107], 9}, {NULL}};
const Segment SGI221648[] = {{&S82655[80], 6}, {NULL}};
const Segment SGI221649[] = {{&S82655[148], 8}, {NULL}};
const Segment SGI221650[] = {{&S89076[34], 12}, {NULL}};
const Segment SGI221651[] = {{&S89076[27], 6}, {NULL}};
const Segment SGI221652[] = {{&S89076[99], 9}, {NULL}};
const Segment SGI221653[] = {{&S89076[72], 6}, {NULL}};
const Segment SGI221654[] = {{&S82657[34], 17}, {NULL}};
const Segment SGI221655[] = {{&S82657[27], 6}, {NULL}};
const Segment SGI221656[] = {{&S82657[104], 9}, {NULL}};
const Segment SGI221657[] = {{&S82657[77], 6}, {NULL}};
const Segment SGI221658[] = {{&S82658[34], 10}, {NULL}};
const Segment SGI221659[] = {{&S82658[27], 6}, {NULL}};
const Segment SGI221660[] = {{&S82658[97], 9}, {NULL}};
const Segment SGI221661[] = {{&S82658[70], 6}, {NULL}};
const Segment SGI221662[] = {{&S82658[138], 8}, {NULL}};
const Segment SGI221663[] = {{&S93415[34], 7}, {NULL}};
const Segment SGI221664[] = {{&S89081[44], 6}, {NULL}};
const Segment SGI221665[] = {{&S93415[27], 6}, {NULL}};
const Segment SGI221666[] = {{&S89081[44], 18}, {NULL}};
const Segment SGI221667[] = {{&S89081[30], 6}, {NULL}};
const Segment SGI221668[] = {{&S93415[95], 9}, {NULL}};
const Segment SGI221669[] = {{&S93415[68], 6}, {NULL}};
const Segment SGI221670[] = {{&S89081[76], 6}, {NULL}};
const Segment SGI221671[] = {{&S89081[103], 13}, {NULL}};
const Segment SGI221672[] = {{&S93417[34], 8}, {NULL}};
const Segment SGI221673[] = {{&S93417[27], 6}, {NULL}};
const Segment SGI221674[] = {{&S93417[95], 9}, {NULL}};
const Segment SGI221675[] = {{&S93417[68], 6}, {NULL}};
const Segment SGI221676[] = {{&S89088[34], 7}, {NULL}};
const Segment SGI221677[] = {{&S89088[27], 6}, {NULL}};
const Segment SGI221678[] = {{&S89088[94], 9}, {NULL}};
const Segment SGI221679[] = {{&S89088[67], 6}, {NULL}};
const Segment SGI221680[] = {{&S89093[44], 10}, {NULL}};
const Segment SGI221681[] = {{&S89093[44], 22}, {NULL}};
const Segment SGI221682[] = {{&S89093[30], 6}, {NULL}};
const Segment SGI221683[] = {{&S89093[80], 6}, {NULL}};
const Segment SGI221684[] = {{&S89093[107], 13}, {NULL}};
const Segment SGI221685[] = {{&S93430[34], 15}, {NULL}};
const Segment SGI221686[] = {{&S93430[27], 6}, {NULL}};
const Segment SGI221687[] = {{&S93430[102], 9}, {NULL}};
const Segment SGI221688[] = {{&S93430[75], 6}, {NULL}};
const Segment SGI221689[] = {{&S84148[34], 6}, {NULL}};
const Segment SGI221690[] = {{&S84148[27], 6}, {NULL}};
const Segment SGI221691[] = {{&S84148[93], 9}, {NULL}};
const Segment SGI221692[] = {{&S84148[66], 6}, {NULL}};
const Segment SGI221693[] = {{&S93431[34], 8}, {NULL}};
const Segment SGI221694[] = {{&S93431[27], 6}, {NULL}};
const Segment SGI221695[] = {{&S93431[95], 9}, {NULL}};
const Segment SGI221696[] = {{&S93431[68], 6}, {NULL}};
const Segment SGI221697[] = {{&S89100[34], 19}, {NULL}};
const Segment SGI221698[] = {{&S89100[27], 6}, {NULL}};
const Segment SGI221699[] = {{&S89100[106], 9}, {NULL}};
const Segment SGI221700[] = {{&S89100[79], 6}, {NULL}};
const Segment SGI221701[] = {{&S89102[34], 14}, {NULL}};
const Segment SGI221702[] = {{&S89102[27], 6}, {NULL}};
const Segment SGI221703[] = {{&S89102[101], 9}, {NULL}};
const Segment SGI221704[] = {{&S89102[74], 6}, {NULL}};
const Segment SGI221705[] = {{&S89102[142], 8}, {NULL}};
const Segment SGI221706[] = {{&S93438[44], 15}, {NULL}};
const Segment SGI221707[] = {{&S93438[44], 33}, {NULL}};
const Segment SGI221708[] = {{&S93438[30], 6}, {NULL}};
const Segment SGI221709[] = {{&S93438[31], 3}, {NULL}};
const Segment SGI221710[] = {{&S93438[91], 5}, {NULL}};
const Segment SGI221711[] = {{&S93438[128], 14}, {NULL}};
const Segment SGI221712[] = {{&S93441[44], 7}, {NULL}};
const Segment SGI221713[] = {{&S93441[44], 25}, {NULL}};
const Segment SGI221714[] = {{&S93441[30], 6}, {NULL}};
const Segment SGI221715[] = {{&S93441[83], 6}, {NULL}};
const Segment SGI221716[] = {{&S93441[110], 13}, {NULL}};
const Segment SGI221717[] = {{&S93446[44], 11}, {NULL}};
const Segment SGI221718[] = {{&S93446[44], 23}, {NULL}};
const Segment SGI221719[] = {{&S93446[30], 6}, {NULL}};
const Segment SGI221720[] = {{&S93446[81], 6}, {NULL}};
const Segment SGI221721[] = {{&S93446[108], 13}, {NULL}};
const Segment SGI221722[] = {{&S93451[52], 7}, {NULL}};
const Segment SGI221723[] = {{&S93451[52], 23}, {NULL}};
const Segment SGI221724[] = {{&S93451[38], 6}, {NULL}};
const Segment SGI221725[] = {{&S93451[89], 5}, {NULL}};
const Segment SGI221726[] = {{&S93451[0], 14}, {NULL}};
const Segment SGI221727[] = {{&S82706[44], 10}, {NULL}};
const Segment SGI221728[] = {{&S82706[44], 22}, {NULL}};
const Segment SGI221729[] = {{&S82706[30], 6}, {NULL}};
const Segment SGI221730[] = {{&S82706[136], 3}, {NULL}};
const Segment SGI221731[] = {{&S82706[101], 6}, {NULL}};
const Segment SGI221732[] = {{&S93454[44], 14}, {NULL}};
const Segment SGI221733[] = {{&S93454[44], 26}, {NULL}};
const Segment SGI221734[] = {{&S93454[30], 6}, {NULL}};
const Segment SGI221735[] = {{&S93454[84], 6}, {NULL}};
const Segment SGI221736[] = {{&S93454[111], 13}, {NULL}};
const Segment SGI221737[] = {{&S82707[44], 18}, {NULL}};
const Segment SGI221738[] = {{&S82707[44], 30}, {NULL}};
const Segment SGI221739[] = {{&S82707[30], 6}, {NULL}};
const Segment SGI221740[] = {{&S82707[151], 3}, {NULL}};
const Segment SGI221741[] = {{&S82707[116], 6}, {NULL}};
const Segment SGI221742[] = {{&S93456[34], 7}, {NULL}};
const Segment SGI221743[] = {{&S93456[27], 6}, {NULL}};
const Segment SGI221744[] = {{&S93456[94], 9}, {NULL}};
const Segment SGI221745[] = {{&S93456[67], 6}, {NULL}};
const Segment SGI221746[] = {{&S82710[34], 12}, {NULL}};
const Segment SGI221747[] = {{&S93457[34], 10}, {NULL}};
const Segment SGI221748[] = {{&S82710[27], 6}, {NULL}};
const Segment SGI221749[] = {{&S93457[27], 6}, {NULL}};
const Segment SGI221750[] = {{&S82710[99], 9}, {NULL}};
const Segment SGI221751[] = {{&S82710[72], 6}, {NULL}};
const Segment SGI221752[] = {{&S93457[97], 9}, {NULL}};
const Segment SGI221753[] = {{&S93457[70], 6}, {NULL}};
const Segment SGI221754[] = {{&S93458[34], 7}, {NULL}};
const Segment SGI221755[] = {{&S93458[27], 6}, {NULL}};
const Segment SGI221756[] = {{&S93458[112], 9}, {NULL}};
const Segment SGI221757[] = {{&S82712[34], 8}, {NULL}};
const Segment SGI221758[] = {{&S93458[85], 6}, {NULL}};
const Segment SGI221759[] = {{&S82712[27], 6}, {NULL}};
const Segment SGI221760[] = {{&S82712[95], 9}, {NULL}};
const Segment SGI221761[] = {{&S82712[68], 6}, {NULL}};
const Segment SGI221762[] = {{&S82713[34], 7}, {NULL}};
const Segment SGI221763[] = {{&S82713[27], 6}, {NULL}};
const Segment SGI221764[] = {{&S82713[94], 9}, {NULL}};
const Segment SGI221765[] = {{&S82713[67], 6}, {NULL}};
const Segment SGI221766[] = {{&S82714[34], 8}, {NULL}};
const Segment SGI221767[] = {{&S82714[27], 6}, {NULL}};
const Segment SGI221768[] = {{&S82714[95], 9}, {NULL}};
const Segment SGI221769[] = {{&S82714[68], 6}, {NULL}};
const Segment SGI221770[] = {{&S82716[45], 10}, {NULL}};
const Segment SGI221771[] = {{&S82716[30], 6}, {NULL}};
const Segment SGI221772[] = {{&S82716[115], 3}, {NULL}};
const Segment SGI221773[] = {{&S82716[81], 5}, {NULL}};
const Segment SGI221774[] = {{&S82716[139], 13}, {NULL}};
const Segment SGI221775[] = {{&S82717[44], 10}, {NULL}};
const Segment SGI221776[] = {{&S82717[44], 22}, {NULL}};
const Segment SGI221777[] = {{&S93467[44], 15}, {NULL}};
const Segment SGI221778[] = {{&S82717[30], 6}, {NULL}};
const Segment SGI221779[] = {{&S93467[44], 33}, {NULL}};
const Segment SGI221780[] = {{&S93467[30], 6}, {NULL}};
const Segment SGI221781[] = {{&S82717[115], 3}, {NULL}};
const Segment SGI221782[] = {{&S93467[31], 3}, {NULL}};
const Segment SGI221783[] = {{&S82717[80], 6}, {NULL}};
const Segment SGI221784[] = {{&S82717[135], 8}, {NULL}};
const Segment SGI221785[] = {{&S93467[91], 5}, {NULL}};
const Segment SGI221786[] = {{&S82717[144], 13}, {NULL}};
const Segment SGI221787[] = {{&S93467[128], 14}, {NULL}};
const Segment SGI221788[] = {{&S93468[44], 14}, {NULL}};
const Segment SGI221789[] = {{&S93468[44], 31}, {NULL}};
const Segment SGI221790[] = {{&S93468[30], 6}, {NULL}};
const Segment SGI221791[] = {{&S93468[31], 3}, {NULL}};
const Segment SGI221792[] = {{&S93468[89], 5}, {NULL}};
const Segment SGI221793[] = {{&S93468[126], 14}, {NULL}};
const Segment SGI221794[] = {{&S93469[44], 10}, {NULL}};
const Segment SGI221795[] = {{&S93469[44], 22}, {NULL}};
const Segment SGI221796[] = {{&S93469[30], 6}, {NULL}};
const Segment SGI221797[] = {{&S93469[80], 6}, {NULL}};
const Segment SGI221798[] = {{&S93469[107], 13}, {NULL}};
const Segment SGI221799[] = {{&S75159[34], 10}, {NULL}};
const Segment SGI221800[] = {{&S75159[27], 6}, {NULL}};
const Segment SGI221801[] = {{&S75159[96], 9}, {NULL}};
const Segment SGI221802[] = {{&S75159[69], 6}, {NULL}};
const Segment SGI221803[] = {{&S75159[137], 8}, {NULL}};
const Segment SGI221804[] = {{&S82731[34], 8}, {NULL}};
const Segment SGI221805[] = {{&S82731[27], 6}, {NULL}};
const Segment SGI221806[] = {{&S82731[101], 9}, {NULL}};
const Segment SGI221807[] = {{&S82731[74], 6}, {NULL}};
const Segment SGI221808[] = {{&S75164[34], 11}, {NULL}};
const Segment SGI221809[] = {{&S75164[27], 6}, {NULL}};
const Segment SGI221810[] = {{&S75164[97], 9}, {NULL}};
const Segment SGI221811[] = {{&S75164[70], 6}, {NULL}};
const Segment SGI221812[] = {{&S75165[34], 10}, {NULL}};
const Segment SGI221813[] = {{&S75165[27], 6}, {NULL}};
const Segment SGI221814[] = {{&S75165[96], 9}, {NULL}};
const Segment SGI221815[] = {{&S75165[69], 6}, {NULL}};
const Segment SGI221816[] = {{&S89172[34], 10}, {NULL}};
const Segment SGI221817[] = {{&S89172[27], 6}, {NULL}};
const Segment SGI221818[] = {{&S89172[97], 9}, {NULL}};
const Segment SGI221819[] = {{&S89172[70], 6}, {NULL}};
const Segment SGI221820[] = {{&S89173[34], 10}, {NULL}};
const Segment SGI221821[] = {{&S89173[27], 6}, {NULL}};
const Segment SGI221822[] = {{&S89173[97], 9}, {NULL}};
const Segment SGI221823[] = {{&S89173[70], 6}, {NULL}};
const Segment SGI221824[] = {{&S89174[34], 7}, {NULL}};
const Segment SGI221825[] = {{&S89174[27], 6}, {NULL}};
const Segment SGI221826[] = {{&S89174[94], 9}, {NULL}};
const Segment SGI221827[] = {{&S89174[67], 6}, {NULL}};
const Segment SGI221828[] = {{&S93490[44], 7}, {NULL}};
const Segment SGI221829[] = {{&S93490[44], 24}, {NULL}};
const Segment SGI221830[] = {{&S93490[30], 6}, {NULL}};
const Segment SGI221831[] = {{&S93490[82], 6}, {NULL}};
const Segment SGI221832[] = {{&S93490[109], 13}, {NULL}};
const Segment SGI221833[] = {{&S89178[44], 6}, {NULL}};
const Segment SGI221834[] = {{&S89178[44], 18}, {NULL}};
const Segment SGI221835[] = {{&S89178[30], 6}, {NULL}};
const Segment SGI221836[] = {{&S89178[76], 6}, {NULL}};
const Segment SGI221837[] = {{&S89178[103], 13}, {NULL}};
const Segment SGI221838[] = {{&S89184[44], 9}, {NULL}};
const Segment SGI221839[] = {{&S89184[44], 21}, {NULL}};
const Segment SGI221840[] = {{&S89184[30], 6}, {NULL}};
const Segment SGI221841[] = {{&S89184[114], 3}, {NULL}};
const Segment SGI221842[] = {{&S89184[79], 6}, {NULL}};
const Segment SGI221843[] = {{&S84247[45], 4}, {NULL}};
const Segment SGI221844[] = {{&S84247[30], 6}, {NULL}};
const Segment SGI221845[] = {{&S84247[109], 3}, {NULL}};
const Segment SGI221846[] = {{&S84247[75], 5}, {NULL}};
const Segment SGI221847[] = {{&S84247[143], 3}, {NULL}};
const Segment SGI221848[] = {{&S84248[44], 23}, {NULL}};
const Segment SGI221849[] = {{&S84248[44], 35}, {NULL}};
const Segment SGI221850[] = {{&S84248[30], 6}, {NULL}};
const Segment SGI221851[] = {{&S84248[128], 3}, {NULL}};
const Segment SGI221852[] = {{&S84248[93], 6}, {NULL}};
const Segment SGI221853[] = {{&S89197[44], 11}, {NULL}};
const Segment SGI221854[] = {{&S89197[44], 23}, {NULL}};
const Segment SGI221855[] = {{&S89197[30], 6}, {NULL}};
const Segment SGI221856[] = {{&S89197[81], 6}, {NULL}};
const Segment SGI221857[] = {{&S89197[108], 13}, {NULL}};
const Segment SGI221858[] = {{&S84257[34], 19}, {NULL}};
const Segment SGI221859[] = {{&S84257[27], 6}, {NULL}};
const Segment SGI221860[] = {{&S84257[106], 9}, {NULL}};
const Segment SGI221861[] = {{&S84257[79], 6}, {NULL}};
const Segment SGI221862[] = {{&S89204[34], 15}, {NULL}};
const Segment SGI221863[] = {{&S89204[27], 6}, {NULL}};
const Segment SGI221864[] = {{&S89204[102], 9}, {NULL}};
const Segment SGI221865[] = {{&S89204[75], 6}, {NULL}};
const Segment SGI221866[] = {{&S89207[34], 10}, {NULL}};
const Segment SGI221867[] = {{&S89207[27], 6}, {NULL}};
const Segment SGI221868[] = {{&S89207[97], 9}, {NULL}};
const Segment SGI221869[] = {{&S89207[70], 6}, {NULL}};
const Segment SGI221870[] = {{&S89223[34], 18}, {NULL}};
const Segment SGI221871[] = {{&S89223[27], 6}, {NULL}};
const Segment SGI221872[] = {{&S89223[114], 9}, {NULL}};
const Segment SGI221873[] = {{&S89223[87], 6}, {NULL}};
const Segment SGI221874[] = {{&S84295[34], 10}, {NULL}};
const Segment SGI221875[] = {{&S84295[27], 6}, {NULL}};
const Segment SGI221876[] = {{&S84295[97], 9}, {NULL}};
const Segment SGI221877[] = {{&S84295[70], 6}, {NULL}};
const Segment SGI221878[] = {{&S93570[34], 8}, {NULL}};
const Segment SGI221879[] = {{&S93570[27], 6}, {NULL}};
const Segment SGI221880[] = {{&S93570[94], 9}, {NULL}};
const Segment SGI221881[] = {{&S93570[67], 6}, {NULL}};
const Segment SGI221882[] = {{&S93586[34], 7}, {NULL}};
const Segment SGI221883[] = {{&S93586[27], 6}, {NULL}};
const Segment SGI221884[] = {{&S93586[103], 9}, {NULL}};
const Segment SGI221885[] = {{&S93586[76], 6}, {NULL}};
const Segment SGI221886[] = {{&S93588[34], 7}, {NULL}};
const Segment SGI221887[] = {{&S93588[27], 6}, {NULL}};
const Segment SGI221888[] = {{&S89270[44], 11}, {NULL}};
const Segment SGI221889[] = {{&S89270[44], 23}, {NULL}};
const Segment SGI221890[] = {{&S89270[30], 6}, {NULL}};
const Segment SGI221891[] = {{&S93588[90], 9}, {NULL}};
const Segment SGI221892[] = {{&S93588[63], 6}, {NULL}};
const Segment SGI221893[] = {{&S89270[132], 3}, {NULL}};
const Segment SGI221894[] = {{&S89270[97], 6}, {NULL}};
const Segment SGI221895[] = {{&S89270[69], 8}, {NULL}};
const Segment SGI221896[] = {{&S93592[45], 10}, {NULL}};
const Segment SGI221897[] = {{&S93592[30], 6}, {NULL}};
const Segment SGI221898[] = {{&S93592[31], 3}, {NULL}};
const Segment SGI221899[] = {{&S93592[81], 5}, {NULL}};
const Segment SGI221900[] = {{&S93592[139], 13}, {NULL}};
const Segment SGI221901[] = {{&S84334[34], 7}, {NULL}};
const Segment SGI221902[] = {{&S84334[27], 6}, {NULL}};
const Segment SGI221903[] = {{&S84334[94], 9}, {NULL}};
const Segment SGI221904[] = {{&S84334[67], 6}, {NULL}};
const Segment SGI221905[] = {{&S93596[34], 7}, {NULL}};
const Segment SGI221906[] = {{&S93596[27], 6}, {NULL}};
const Segment SGI221907[] = {{&S84338[44], 4}, {NULL}};
const Segment SGI221908[] = {{&S84338[44], 16}, {NULL}};
const Segment SGI221909[] = {{&S93596[90], 9}, {NULL}};
const Segment SGI221910[] = {{&S84338[30], 6}, {NULL}};
const Segment SGI221911[] = {{&S93596[63], 6}, {NULL}};
const Segment SGI221912[] = {{&S84338[74], 6}, {NULL}};
const Segment SGI221913[] = {{&S84338[111], 3}, {NULL}};
const Segment SGI221914[] = {{&S93597[34], 8}, {NULL}};
const Segment SGI221915[] = {{&S93597[27], 6}, {NULL}};
const Segment SGI221916[] = {{&S84339[34], 11}, {NULL}};
const Segment SGI221917[] = {{&S93597[95], 9}, {NULL}};
const Segment SGI221918[] = {{&S93597[68], 6}, {NULL}};
const Segment SGI221919[] = {{&S84339[27], 6}, {NULL}};
const Segment SGI221920[] = {{&S84339[98], 9}, {NULL}};
const Segment SGI221921[] = {{&S84339[71], 6}, {NULL}};
const Segment SGI221922[] = {{&S93598[31], 5}, {NULL}};
const Segment SGI221923[] = {{&S93598[52], 3}, {NULL}};
const Segment SGI221924[] = {{&S84341[34], 10}, {NULL}};
const Segment SGI221925[] = {{&S84341[27], 6}, {NULL}};
const Segment SGI221926[] = {{&S84341[99], 9}, {NULL}};
const Segment SGI221927[] = {{&S84341[72], 6}, {NULL}};
const Segment SGI221928[] = {{&S93605[34], 10}, {NULL}};
const Segment SGI221929[] = {{&S93605[27], 6}, {NULL}};
const Segment SGI221930[] = {{&S93605[102], 9}, {NULL}};
const Segment SGI221931[] = {{&S93605[75], 6}, {NULL}};
const Segment SGI221932[] = {{&S93617[44], 13}, {NULL}};
const Segment SGI221933[] = {{&S93617[44], 25}, {NULL}};
const Segment SGI221934[] = {{&S93617[30], 6}, {NULL}};
const Segment SGI221935[] = {{&S93617[31], 3}, {NULL}};
const Segment SGI221936[] = {{&S93617[83], 5}, {NULL}};
const Segment SGI221937[] = {{&S93617[120], 14}, {NULL}};
const Segment SGI221938[] = {{&S93621[44], 10}, {NULL}};
const Segment SGI221939[] = {{&S93621[44], 22}, {NULL}};
const Segment SGI221940[] = {{&S93621[30], 6}, {NULL}};
const Segment SGI221941[] = {{&S93621[80], 6}, {NULL}};
const Segment SGI221942[] = {{&S93621[107], 13}, {NULL}};
const Segment SGI221943[] = {{&S93622[34], 12}, {NULL}};
const Segment SGI221944[] = {{&S93622[27], 6}, {NULL}};
const Segment SGI221945[] = {{&S93622[112], 9}, {NULL}};
const Segment SGI221946[] = {{&S93622[85], 6}, {NULL}};
const Segment SGI221947[] = {{&S93623[44], 9}, {NULL}};
const Segment SGI221948[] = {{&S93623[44], 21}, {NULL}};
const Segment SGI221949[] = {{&S93623[30], 6}, {NULL}};
const Segment SGI221950[] = {{&S93623[31], 3}, {NULL}};
const Segment SGI221951[] = {{&S93623[79], 6}, {NULL}};
const Segment SGI221952[] = {{&S93626[44], 20}, {NULL}};
const Segment SGI221953[] = {{&S93626[44], 32}, {NULL}};
const Segment SGI221954[] = {{&S93626[30], 6}, {NULL}};
const Segment SGI221955[] = {{&S93626[31], 3}, {NULL}};
const Segment SGI221956[] = {{&S93626[90], 6}, {NULL}};
const Segment SGI221957[] = {{&S93629[34], 10}, {NULL}};
const Segment SGI221958[] = {{&S93629[27], 6}, {NULL}};
const Segment SGI221959[] = {{&S93629[97], 9}, {NULL}};
const Segment SGI221960[] = {{&S93629[70], 6}, {NULL}};
const Segment SGI221961[] = {{&S93631[34], 4}, {NULL}};
const Segment SGI221962[] = {{&S93631[27], 6}, {NULL}};
const Segment SGI221963[] = {{&S93631[91], 9}, {NULL}};
const Segment SGI221964[] = {{&S93631[64], 6}, {NULL}};
const Segment SGI221965[] = {{&S93632[34], 11}, {NULL}};
const Segment SGI221966[] = {{&S93632[27], 6}, {NULL}};
const Segment SGI221967[] = {{&S93632[98], 9}, {NULL}};
const Segment SGI221968[] = {{&S93632[71], 6}, {NULL}};
const Segment SGI221969[] = {{&S89333[34], 7}, {NULL}};
const Segment SGI221970[] = {{&S89333[27], 6}, {NULL}};
const Segment SGI221971[] = {{&S89333[98], 9}, {NULL}};
const Segment SGI221972[] = {{&S89333[71], 6}, {NULL}};
const Segment SGI221973[] = {{&S93634[34], 9}, {NULL}};
const Segment SGI221974[] = {{&S93634[27], 6}, {NULL}};
const Segment SGI221975[] = {{&S93634[96], 9}, {NULL}};
const Segment SGI221976[] = {{&S93634[69], 6}, {NULL}};
const Segment SGI221977[] = {{&S89336[34], 5}, {NULL}};
const Segment SGI221978[] = {{&S89336[27], 6}, {NULL}};
const Segment SGI221979[] = {{&S89336[92], 9}, {NULL}};
const Segment SGI221980[] = {{&S89336[65], 6}, {NULL}};
const Segment SGI221981[] = {{&S93639[34], 8}, {NULL}};
const Segment SGI221982[] = {{&S93639[27], 6}, {NULL}};
const Segment SGI221983[] = {{&S93639[101], 9}, {NULL}};
const Segment SGI221984[] = {{&S93639[74], 6}, {NULL}};
const Segment SGI221985[] = {{&S93642[44], 10}, {NULL}};
const Segment SGI221986[] = {{&S93642[44], 22}, {NULL}};
const Segment SGI221987[] = {{&S93642[30], 6}, {NULL}};
const Segment SGI221988[] = {{&S93642[31], 3}, {NULL}};
const Segment SGI221989[] = {{&S93642[80], 5}, {NULL}};
const Segment SGI221990[] = {{&S93642[117], 14}, {NULL}};
const Segment SGI221991[] = {{&S93643[34], 11}, {NULL}};
const Segment SGI221992[] = {{&S93643[27], 6}, {NULL}};
const Segment SGI221993[] = {{&S93643[98], 9}, {NULL}};
const Segment SGI221994[] = {{&S93643[71], 6}, {NULL}};
const Segment SGI221995[] = {{&S89347[44], 6}, {NULL}};
const Segment SGI221996[] = {{&S89347[44], 18}, {NULL}};
const Segment SGI221997[] = {{&S89347[30], 6}, {NULL}};
const Segment SGI221998[] = {{&S75348[34], 10}, {NULL}};
const Segment SGI221999[] = {{&S89347[76], 6}, {NULL}};
const Segment SGI222000[] = {{&S75348[27], 6}, {NULL}};
const Segment SGI222001[] = {{&S89347[103], 13}, {NULL}};
const Segment SGI222002[] = {{&S75348[96], 9}, {NULL}};
const Segment SGI222003[] = {{&S75348[69], 6}, {NULL}};
const Segment SGI222004[] = {{&S93647[44], 5}, {NULL}};
const Segment SGI222005[] = {{&S93647[44], 17}, {NULL}};
const Segment SGI222006[] = {{&S93647[30], 6}, {NULL}};
const Segment SGI222007[] = {{&S93647[31], 3}, {NULL}};
const Segment SGI222008[] = {{&S93647[75], 5}, {NULL}};
const Segment SGI222009[] = {{&S93647[112], 14}, {NULL}};
const Segment SGI222010[] = {{&S93650[34], 7}, {NULL}};
const Segment SGI222011[] = {{&S93650[27], 6}, {NULL}};
const Segment SGI222012[] = {{&S89355[34], 41}, {NULL}};
const Segment SGI222013[] = {{&S93650[94], 9}, {NULL}};
const Segment SGI222014[] = {{&S89355[27], 6}, {NULL}};
const Segment SGI222015[] = {{&S93650[67], 6}, {NULL}};
const Segment SGI222016[] = {{&S89355[128], 9}, {NULL}};
const Segment SGI222017[] = {{&S89355[101], 6}, {NULL}};
const Segment SGI222018[] = {{&S93654[45], 7}, {NULL}};
const Segment SGI222019[] = {{&S93654[30], 6}, {NULL}};
const Segment SGI222020[] = {{&S93654[31], 3}, {NULL}};
const Segment SGI222021[] = {{&S93654[83], 5}, {NULL}};
const Segment SGI222022[] = {{&S93654[141], 13}, {NULL}};
const Segment SGI222023[] = {{&S89364[34], 18}, {NULL}};
const Segment SGI222024[] = {{&S89364[27], 6}, {NULL}};
const Segment SGI222025[] = {{&S89364[105], 9}, {NULL}};
const Segment SGI222026[] = {{&S89364[78], 6}, {NULL}};
const Segment SGI222027[] = {{&S82781[34], 10}, {NULL}};
const Segment SGI222028[] = {{&S82781[27], 6}, {NULL}};
const Segment SGI222029[] = {{&S82781[97], 9}, {NULL}};
const Segment SGI222030[] = {{&S82781[70], 6}, {NULL}};
const Segment SGI222031[] = {{&S93662[44], 9}, {NULL}};
const Segment SGI222032[] = {{&S93662[44], 21}, {NULL}};
const Segment SGI222033[] = {{&S93662[30], 6}, {NULL}};
const Segment SGI222034[] = {{&S89368[34], 9}, {NULL}};
const Segment SGI222035[] = {{&S93662[31], 3}, {NULL}};
const Segment SGI222036[] = {{&S89368[27], 6}, {NULL}};
const Segment SGI222037[] = {{&S93662[79], 6}, {NULL}};
const Segment SGI222038[] = {{&S89368[96], 9}, {NULL}};
const Segment SGI222039[] = {{&S89368[69], 6}, {NULL}};
const Segment SGI222040[] = {{&S93663[34], 9}, {NULL}};
const Segment SGI222041[] = {{&S93663[27], 6}, {NULL}};
const Segment SGI222042[] = {{&S93663[96], 9}, {NULL}};
const Segment SGI222043[] = {{&S93663[69], 6}, {NULL}};
const Segment SGI222044[] = {{&S93664[34], 13}, {NULL}};
const Segment SGI222045[] = {{&S93664[27], 6}, {NULL}};
const Segment SGI222046[] = {{&S93664[100], 9}, {NULL}};
const Segment SGI222047[] = {{&S93664[73], 6}, {NULL}};
const Segment SGI222048[] = {{&S82787[34], 20}, {NULL}};
const Segment SGI222049[] = {{&S82787[27], 6}, {NULL}};
const Segment SGI222050[] = {{&S82787[107], 9}, {NULL}};
const Segment SGI222051[] = {{&S82787[80], 6}, {NULL}};
const Segment SGI222052[] = {{&S93665[34], 15}, {NULL}};
const Segment SGI222053[] = {{&S93665[27], 6}, {NULL}};
const Segment SGI222054[] = {{&S93665[102], 9}, {NULL}};
const Segment SGI222055[] = {{&S93665[75], 6}, {NULL}};
const Segment SGI222056[] = {{&S93666[34], 14}, {NULL}};
const Segment SGI222057[] = {{&S93666[27], 6}, {NULL}};
const Segment SGI222058[] = {{&S93666[114], 9}, {NULL}};
const Segment SGI222059[] = {{&S93666[87], 6}, {NULL}};
const Segment SGI222060[] = {{&S89380[34], 6}, {NULL}};
const Segment SGI222061[] = {{&S89380[27], 6}, {NULL}};
const Segment SGI222062[] = {{&S89380[93], 9}, {NULL}};
const Segment SGI222063[] = {{&S89380[66], 6}, {NULL}};
const Segment SGI222064[] = {{&S93670[34], 10}, {NULL}};
const Segment SGI222065[] = {{&S93670[27], 6}, {NULL}};
const Segment SGI222066[] = {{&S93670[97], 9}, {NULL}};
const Segment SGI222067[] = {{&S93670[70], 6}, {NULL}};
const Segment SGI222068[] = {{&S82796[34], 7}, {NULL}};
const Segment SGI222069[] = {{&S82796[27], 6}, {NULL}};
const Segment SGI222070[] = {{&S82796[107], 9}, {NULL}};
const Segment SGI222071[] = {{&S82796[80], 6}, {NULL}};
const Segment SGI222072[] = {{&S82796[148], 8}, {NULL}};
const Segment SGI222073[] = {{&S93672[45], 14}, {NULL}};
const Segment SGI222074[] = {{&S93672[30], 6}, {NULL}};
const Segment SGI222075[] = {{&S93672[31], 3}, {NULL}};
const Segment SGI222076[] = {{&S93672[85], 5}, {NULL}};
const Segment SGI222077[] = {{&S93672[143], 13}, {NULL}};
const Segment SGI222078[] = {{&S82801[34], 7}, {NULL}};
const Segment SGI222079[] = {{&S82801[27], 6}, {NULL}};
const Segment SGI222080[] = {{&S82801[99], 9}, {NULL}};
const Segment SGI222081[] = {{&S82801[72], 6}, {NULL}};
const Segment SGI222082[] = {{&S84457[44], 7}, {NULL}};
const Segment SGI222083[] = {{&S84457[44], 19}, {NULL}};
const Segment SGI222084[] = {{&S84457[30], 6}, {NULL}};
const Segment SGI222085[] = {{&S84457[112], 3}, {NULL}};
const Segment SGI222086[] = {{&S84457[77], 6}, {NULL}};
const Segment SGI222087[] = {{&S93682[44], 10}, {NULL}};
const Segment SGI222088[] = {{&S93682[44], 22}, {NULL}};
const Segment SGI222089[] = {{&S93682[30], 6}, {NULL}};
const Segment SGI222090[] = {{&S93682[31], 3}, {NULL}};
const Segment SGI222091[] = {{&S93682[80], 6}, {NULL}};
const Segment SGI222092[] = {{&S93682[140], 7}, {NULL}};
const Segment SGI222093[] = {{&S93685[34], 25}, {NULL}};
const Segment SGI222094[] = {{&S93685[27], 6}, {NULL}};
const Segment SGI222095[] = {{&S93685[112], 9}, {NULL}};
const Segment SGI222096[] = {{&S93685[85], 6}, {NULL}};
const Segment SGI222097[] = {{&S93686[34], 30}, {NULL}};
const Segment SGI222098[] = {{&S93686[27], 6}, {NULL}};
const Segment SGI222099[] = {{&S93686[117], 9}, {NULL}};
const Segment SGI222100[] = {{&S93686[90], 6}, {NULL}};
const Segment SGI222101[] = {{&S82815[34], 12}, {NULL}};
const Segment SGI222102[] = {{&S82815[27], 6}, {NULL}};
const Segment SGI222103[] = {{&S82815[99], 9}, {NULL}};
const Segment SGI222104[] = {{&S82815[72], 6}, {NULL}};
const Segment SGI222105[] = {{&S93688[34], 4}, {NULL}};
const Segment SGI222106[] = {{&S93688[27], 6}, {NULL}};
const Segment SGI222107[] = {{&S93688[91], 9}, {NULL}};
const Segment SGI222108[] = {{&S93688[64], 6}, {NULL}};
const Segment SGI222109[] = {{&S84471[44], 16}, {NULL}};
const Segment SGI222110[] = {{&S84471[44], 28}, {NULL}};
const Segment SGI222111[] = {{&S84471[30], 6}, {NULL}};
const Segment SGI222112[] = {{&S84471[86], 6}, {NULL}};
const Segment SGI222113[] = {{&S84471[123], 3}, {NULL}};
const Segment SGI222114[] = {{&S93692[34], 11}, {NULL}};
const Segment SGI222115[] = {{&S93692[27], 6}, {NULL}};
const Segment SGI222116[] = {{&S93692[98], 9}, {NULL}};
const Segment SGI222117[] = {{&S93692[71], 6}, {NULL}};
const Segment SGI222118[] = {{&S93698[34], 10}, {NULL}};
const Segment SGI222119[] = {{&S93698[27], 6}, {NULL}};
const Segment SGI222120[] = {{&S93698[101], 9}, {NULL}};
const Segment SGI222121[] = {{&S93698[74], 6}, {NULL}};
const Segment SGI222122[] = {{&S93698[142], 8}, {NULL}};
const Segment SGI222123[] = {{&S93702[43], 10}, {NULL}};
const Segment SGI222124[] = {{&S93702[43], 22}, {NULL}};
const Segment SGI222125[] = {{&S93702[30], 6}, {NULL}};
const Segment SGI222126[] = {{&S93702[31], 3}, {NULL}};
const Segment SGI222127[] = {{&S93702[79], 6}, {NULL}};
const Segment SGI222128[] = {{&S75435[41], 18}, {NULL}};
const Segment SGI222129[] = {{&S75435[41], 29}, {NULL}};
const Segment SGI222130[] = {{&S75435[27], 6}, {NULL}};
const Segment SGI222131[] = {{&S75435[119], 3}, {NULL}};
const Segment SGI222132[] = {{&S75435[123], 9}, {NULL}};
const Segment SGI222133[] = {{&S75435[84], 6}, {NULL}};
const Segment SGI222134[] = {{&S93703[34], 7}, {NULL}};
const Segment SGI222135[] = {{&S93703[27], 6}, {NULL}};
const Segment SGI222136[] = {{&S93703[94], 9}, {NULL}};
const Segment SGI222137[] = {{&S93703[67], 6}, {NULL}};
const Segment SGI222138[] = {{&S93707[44], 7}, {NULL}};
const Segment SGI222139[] = {{&S93707[44], 23}, {NULL}};
const Segment SGI222140[] = {{&S93707[30], 6}, {NULL}};
const Segment SGI222141[] = {{&S93707[81], 6}, {NULL}};
const Segment SGI222142[] = {{&S93707[108], 13}, {NULL}};
const Segment SGI222143[] = {{&S93708[44], 7}, {NULL}};
const Segment SGI222144[] = {{&S93708[44], 23}, {NULL}};
const Segment SGI222145[] = {{&S93708[30], 6}, {NULL}};
const Segment SGI222146[] = {{&S93708[81], 6}, {NULL}};
const Segment SGI222147[] = {{&S93708[108], 13}, {NULL}};
const Segment SGI222148[] = {{&S93712[44], 5}, {NULL}};
const Segment SGI222149[] = {{&S93712[44], 17}, {NULL}};
const Segment SGI222150[] = {{&S93712[30], 6}, {NULL}};
const Segment SGI222151[] = {{&S93712[75], 6}, {NULL}};
const Segment SGI222152[] = {{&S93712[102], 13}, {NULL}};
const Segment SGI222153[] = {{&S93713[34], 10}, {NULL}};
const Segment SGI222154[] = {{&S93713[27], 6}, {NULL}};
const Segment SGI222155[] = {{&S82852[34], 7}, {NULL}};
const Segment SGI222156[] = {{&S93713[97], 9}, {NULL}};
const Segment SGI222157[] = {{&S82852[27], 6}, {NULL}};
const Segment SGI222158[] = {{&S93713[70], 6}, {NULL}};
const Segment SGI222159[] = {{&S82852[94], 9}, {NULL}};
const Segment SGI222160[] = {{&S82852[67], 6}, {NULL}};
const Segment SGI222161[] = {{&S89454[44], 12}, {NULL}};
const Segment SGI222162[] = {{&S89454[44], 22}, {NULL}};
const Segment SGI222163[] = {{&S89454[30], 6}, {NULL}};
const Segment SGI222164[] = {{&S89454[115], 3}, {NULL}};
const Segment SGI222165[] = {{&S89454[80], 6}, {NULL}};
const Segment SGI222166[] = {{&S75459[34], 9}, {NULL}};
const Segment SGI222167[] = {{&S75459[27], 6}, {NULL}};
const Segment SGI222168[] = {{&S75459[95], 9}, {NULL}};
const Segment SGI222169[] = {{&S75459[68], 6}, {NULL}};
const Segment SGI222170[] = {{&S82862[44], 23}, {NULL}};
const Segment SGI222171[] = {{&S82862[44], 34}, {NULL}};
const Segment SGI222172[] = {{&S82862[30], 6}, {NULL}};
const Segment SGI222173[] = {{&S82862[127], 3}, {NULL}};
const Segment SGI222174[] = {{&S82862[92], 6}, {NULL}};
const Segment SGI222175[] = {{&S75467[34], 10}, {NULL}};
const Segment SGI222176[] = {{&S82863[34], 9}, {NULL}};
const Segment SGI222177[] = {{&S75467[27], 6}, {NULL}};
const Segment SGI222178[] = {{&S82863[27], 6}, {NULL}};
const Segment SGI222179[] = {{&S75467[96], 9}, {NULL}};
const Segment SGI222180[] = {{&S75467[69], 6}, {NULL}};
const Segment SGI222181[] = {{&S82863[96], 9}, {NULL}};
const Segment SGI222182[] = {{&S82863[69], 6}, {NULL}};
const Segment SGI222183[] = {{&S82864[34], 10}, {NULL}};
const Segment SGI222184[] = {{&S82864[27], 6}, {NULL}};
const Segment SGI222185[] = {{&S82864[97], 9}, {NULL}};
const Segment SGI222186[] = {{&S82864[70], 6}, {NULL}};
const Segment SGI222187[] = {{&S82867[34], 20}, {NULL}};
const Segment SGI222188[] = {{&S82867[27], 6}, {NULL}};
const Segment SGI222189[] = {{&S82867[107], 9}, {NULL}};
const Segment SGI222190[] = {{&S82867[80], 6}, {NULL}};
const Segment SGI222191[] = {{&S89472[44], 8}, {NULL}};
const Segment SGI222192[] = {{&S89472[44], 20}, {NULL}};
const Segment SGI222193[] = {{&S89472[30], 6}, {NULL}};
const Segment SGI222194[] = {{&S89472[113], 3}, {NULL}};
const Segment SGI222195[] = {{&S89472[78], 6}, {NULL}};
const Segment SGI222196[] = {{&S82872[44], 12}, {NULL}};
const Segment SGI222197[] = {{&S82872[44], 24}, {NULL}};
const Segment SGI222198[] = {{&S82872[30], 6}, {NULL}};
const Segment SGI222199[] = {{&S82872[117], 3}, {NULL}};
const Segment SGI222200[] = {{&S82872[82], 6}, {NULL}};
const Segment SGI222201[] = {{&S89493[34], 10}, {NULL}};
const Segment SGI222202[] = {{&S89493[27], 6}, {NULL}};
const Segment SGI222203[] = {{&S89493[97], 9}, {NULL}};
const Segment SGI222204[] = {{&S89493[70], 6}, {NULL}};
const Segment SGI222205[] = {{&S89496[34], 16}, {NULL}};
const Segment SGI222206[] = {{&S89496[27], 6}, {NULL}};
const Segment SGI222207[] = {{&S89496[103], 9}, {NULL}};
const Segment SGI222208[] = {{&S89496[76], 6}, {NULL}};
const Segment SGI222209[] = {{&S82910[44], 9}, {NULL}};
const Segment SGI222210[] = {{&S82910[44], 20}, {NULL}};
const Segment SGI222211[] = {{&S82910[30], 6}, {NULL}};
const Segment SGI222212[] = {{&S82910[129], 3}, {NULL}};
const Segment SGI222213[] = {{&S82910[94], 6}, {NULL}};
const Segment SGI222214[] = {{&S82910[66], 8}, {NULL}};
const Segment SGI222215[] = {{&S89508[44], 20}, {NULL}};
const Segment SGI222216[] = {{&S89508[44], 32}, {NULL}};
const Segment SGI222217[] = {{&S89508[30], 6}, {NULL}};
const Segment SGI222218[] = {{&S89508[125], 3}, {NULL}};
const Segment SGI222219[] = {{&S89508[90], 6}, {NULL}};
const Segment SGI222220[] = {{&S93792[44], 7}, {NULL}};
const Segment SGI222221[] = {{&S93792[44], 19}, {NULL}};
const Segment SGI222222[] = {{&S93792[30], 6}, {NULL}};
const Segment SGI222223[] = {{&S93792[31], 3}, {NULL}};
const Segment SGI222224[] = {{&S93792[77], 6}, {NULL}};
const Segment SGI222225[] = {{&S93793[38], 7}, {NULL}};
const Segment SGI222226[] = {{&S93793[31], 6}, {NULL}};
const Segment SGI222227[] = {{&S89517[44], 10}, {NULL}};
const Segment SGI222228[] = {{&S89517[44], 27}, {NULL}};
const Segment SGI222229[] = {{&S89517[30], 6}, {NULL}};
const Segment SGI222230[] = {{&S89517[119], 3}, {NULL}};
const Segment SGI222231[] = {{&S89517[85], 5}, {NULL}};
const Segment SGI222232[] = {{&S75529[34], 5}, {NULL}};
const Segment SGI222233[] = {{&S75529[27], 6}, {NULL}};
const Segment SGI222234[] = {{&S75529[92], 9}, {NULL}};
const Segment SGI222235[] = {{&S75529[65], 6}, {NULL}};
const Segment SGI222236[] = {{&S89526[34], 8}, {NULL}};
const Segment SGI222237[] = {{&S89526[27], 6}, {NULL}};
const Segment SGI222238[] = {{&S89526[95], 9}, {NULL}};
const Segment SGI222239[] = {{&S89526[68], 6}, {NULL}};
const Segment SGI222240[] = {{&S93808[44], 6}, {NULL}};
const Segment SGI222241[] = {{&S93808[44], 18}, {NULL}};
const Segment SGI222242[] = {{&S93808[30], 6}, {NULL}};
const Segment SGI222243[] = {{&S93808[31], 3}, {NULL}};
const Segment SGI222244[] = {{&S93808[91], 6}, {NULL}};
const Segment SGI222245[] = {{&S93808[64], 8}, {NULL}};
const Segment SGI222246[] = {{&S82944[34], 8}, {NULL}};
const Segment SGI222247[] = {{&S82944[27], 6}, {NULL}};
const Segment SGI222248[] = {{&S82944[103], 9}, {NULL}};
const Segment SGI222249[] = {{&S82944[76], 6}, {NULL}};
const Segment SGI222250[] = {{&S82944[137], 8}, {NULL}};
const Segment SGI222251[] = {{&S82947[34], 10}, {NULL}};
const Segment SGI222252[] = {{&S82947[27], 6}, {NULL}};
const Segment SGI222253[] = {{&S82947[97], 9}, {NULL}};
const Segment SGI222254[] = {{&S82947[70], 6}, {NULL}};
const Segment SGI222255[] = {{&S89543[44], 10}, {NULL}};
const Segment SGI222256[] = {{&S89543[44], 27}, {NULL}};
const Segment SGI222257[] = {{&S89543[30], 6}, {NULL}};
const Segment SGI222258[] = {{&S89543[85], 6}, {NULL}};
const Segment SGI222259[] = {{&S89543[112], 13}, {NULL}};
const Segment SGI222260[] = {{&S75558[34], 9}, {NULL}};
const Segment SGI222261[] = {{&S75558[27], 6}, {NULL}};
const Segment SGI222262[] = {{&S75558[95], 9}, {NULL}};
const Segment SGI222263[] = {{&S75558[68], 6}, {NULL}};
const Segment SGI222264[] = {{&S93828[34], 13}, {NULL}};
const Segment SGI222265[] = {{&S93828[27], 6}, {NULL}};
const Segment SGI222266[] = {{&S93828[100], 9}, {NULL}};
const Segment SGI222267[] = {{&S93828[73], 6}, {NULL}};
const Segment SGI222268[] = {{&S93829[45], 5}, {NULL}};
const Segment SGI222269[] = {{&S93829[30], 6}, {NULL}};
const Segment SGI222270[] = {{&S93829[31], 3}, {NULL}};
const Segment SGI222271[] = {{&S93829[76], 5}, {NULL}};
const Segment SGI222272[] = {{&S93829[134], 13}, {NULL}};
const Segment SGI222273[] = {{&S93832[34], 13}, {NULL}};
const Segment SGI222274[] = {{&S93832[27], 6}, {NULL}};
const Segment SGI222275[] = {{&S93832[123], 9}, {NULL}};
const Segment SGI222276[] = {{&S93832[96], 6}, {NULL}};
const Segment SGI222277[] = {{&S93838[44], 18}, {NULL}};
const Segment SGI222278[] = {{&S93838[44], 30}, {NULL}};
const Segment SGI222279[] = {{&S93838[30], 6}, {NULL}};
const Segment SGI222280[] = {{&S93838[31], 3}, {NULL}};
const Segment SGI222281[] = {{&S93838[88], 6}, {NULL}};
const Segment SGI222282[] = {{&S93841[34], 13}, {NULL}};
const Segment SGI222283[] = {{&S93841[27], 6}, {NULL}};
const Segment SGI222284[] = {{&S93841[100], 9}, {NULL}};
const Segment SGI222285[] = {{&S93841[73], 6}, {NULL}};
const Segment SGI222286[] = {{&S84640[34], 8}, {NULL}};
const Segment SGI222287[] = {{&S84640[27], 6}, {NULL}};
const Segment SGI222288[] = {{&S84640[95], 9}, {NULL}};
const Segment SGI222289[] = {{&S84640[68], 6}, {NULL}};
const Segment SGI222290[] = {{&S84640[129], 8}, {NULL}};
const Segment SGI222291[] = {{&S93848[34], 10}, {NULL}};
const Segment SGI222292[] = {{&S93848[27], 6}, {NULL}};
const Segment SGI222293[] = {{&S93848[97], 9}, {NULL}};
const Segment SGI222294[] = {{&S93848[70], 6}, {NULL}};
const Segment SGI222295[] = {{&S94347[42], 5}, {NULL}};
const Segment SGI222296[] = {{&S94347[42], 17}, {NULL}};
const Segment SGI222297[] = {{&S94347[30], 4}, {NULL}};
const Segment SGI222298[] = {{&S94347[108], 3}, {NULL}};
const Segment SGI222299[] = {{&S94347[73], 6}, {NULL}};
const Segment SGI222300[] = {{&S93854[59], 10}, {NULL}};
const Segment SGI222301[] = {{&S93854[59], 22}, {NULL}};
const Segment SGI222302[] = {{&S93854[45], 6}, {NULL}};
const Segment SGI222303[] = {{&S84652[44], 6}, {NULL}};
const Segment SGI222304[] = {{&S93854[46], 3}, {NULL}};
const Segment SGI222305[] = {{&S84652[44], 20}, {NULL}};
const Segment SGI222306[] = {{&S84652[30], 6}, {NULL}};
const Segment SGI222307[] = {{&S93854[95], 6}, {NULL}};
const Segment SGI222308[] = {{&S93854[0], 14}, {NULL}};
const Segment SGI222309[] = {{&S84652[78], 6}, {NULL}};
const Segment SGI222310[] = {{&S84652[115], 3}, {NULL}};
const Segment SGI222311[] = {{&S84657[44], 15}, {NULL}};
const Segment SGI222312[] = {{&S84657[44], 27}, {NULL}};
const Segment SGI222313[] = {{&S84657[30], 6}, {NULL}};
const Segment SGI222314[] = {{&S84657[85], 6}, {NULL}};
const Segment SGI222315[] = {{&S84657[122], 3}, {NULL}};
const Segment SGI222316[] = {{&S83007[34], 11}, {NULL}};
const Segment SGI222317[] = {{&S83007[27], 6}, {NULL}};
const Segment SGI222318[] = {{&S93855[34], 5}, {NULL}};
const Segment SGI222319[] = {{&S83007[98], 9}, {NULL}};
const Segment SGI222320[] = {{&S93855[27], 6}, {NULL}};
const Segment SGI222321[] = {{&S83007[71], 6}, {NULL}};
const Segment SGI222322[] = {{&S93855[92], 9}, {NULL}};
const Segment SGI222323[] = {{&S93855[65], 6}, {NULL}};
const Segment SGI222324[] = {{&S83008[34], 11}, {NULL}};
const Segment SGI222325[] = {{&S83008[27], 6}, {NULL}};
const Segment SGI222326[] = {{&S83008[98], 9}, {NULL}};
const Segment SGI222327[] = {{&S83008[71], 6}, {NULL}};
const Segment SGI222328[] = {{&S93856[34], 13}, {NULL}};
const Segment SGI222329[] = {{&S93856[27], 6}, {NULL}};
const Segment SGI222330[] = {{&S93856[100], 9}, {NULL}};
const Segment SGI222331[] = {{&S93856[73], 6}, {NULL}};
const Segment SGI222332[] = {{&S93859[34], 6}, {NULL}};
const Segment SGI222333[] = {{&S93859[27], 6}, {NULL}};
const Segment SGI222334[] = {{&S93859[105], 9}, {NULL}};
const Segment SGI222335[] = {{&S93859[78], 6}, {NULL}};
const Segment SGI222336[] = {{&S89605[34], 12}, {NULL}};
const Segment SGI222337[] = {{&S89605[27], 6}, {NULL}};
const Segment SGI222338[] = {{&S89605[99], 9}, {NULL}};
const Segment SGI222339[] = {{&S93861[44], 11}, {NULL}};
const Segment SGI222340[] = {{&S89605[72], 6}, {NULL}};
const Segment SGI222341[] = {{&S93861[44], 23}, {NULL}};
const Segment SGI222342[] = {{&S93861[30], 6}, {NULL}};
const Segment SGI222343[] = {{&S93861[31], 3}, {NULL}};
const Segment SGI222344[] = {{&S93861[96], 6}, {NULL}};
const Segment SGI222345[] = {{&S93861[69], 8}, {NULL}};
const Segment SGI222346[] = {{&S83020[34], 10}, {NULL}};
const Segment SGI222347[] = {{&S83020[27], 6}, {NULL}};
const Segment SGI222348[] = {{&S93863[34], 7}, {NULL}};
const Segment SGI222349[] = {{&S93863[27], 6}, {NULL}};
const Segment SGI222350[] = {{&S83020[97], 9}, {NULL}};
const Segment SGI222351[] = {{&S83020[70], 6}, {NULL}};
const Segment SGI222352[] = {{&S93863[112], 9}, {NULL}};
const Segment SGI222353[] = {{&S93863[85], 6}, {NULL}};
const Segment SGI222354[] = {{&S83021[34], 10}, {NULL}};
const Segment SGI222355[] = {{&S83021[27], 6}, {NULL}};
const Segment SGI222356[] = {{&S83021[97], 9}, {NULL}};
const Segment SGI222357[] = {{&S83021[70], 6}, {NULL}};
const Segment SGI222358[] = {{&S93866[34], 11}, {NULL}};
const Segment SGI222359[] = {{&S93866[27], 6}, {NULL}};
const Segment SGI222360[] = {{&S93866[98], 9}, {NULL}};
const Segment SGI222361[] = {{&S93866[71], 6}, {NULL}};
const Segment SGI222362[] = {{&S83024[34], 11}, {NULL}};
const Segment SGI222363[] = {{&S83024[27], 6}, {NULL}};
const Segment SGI222364[] = {{&S83024[98], 9}, {NULL}};
const Segment SGI222365[] = {{&S83024[71], 6}, {NULL}};
const Segment SGI222366[] = {{&S93867[34], 7}, {NULL}};
const Segment SGI222367[] = {{&S93867[27], 6}, {NULL}};
const Segment SGI222368[] = {{&S93867[99], 9}, {NULL}};
const Segment SGI222369[] = {{&S93867[72], 6}, {NULL}};
const Segment SGI222370[] = {{&S93869[34], 5}, {NULL}};
const Segment SGI222371[] = {{&S93869[27], 6}, {NULL}};
const Segment SGI222372[] = {{&S93869[92], 9}, {NULL}};
const Segment SGI222373[] = {{&S93869[65], 6}, {NULL}};
const Segment SGI222374[] = {{&S93871[34], 14}, {NULL}};
const Segment SGI222375[] = {{&S93871[27], 6}, {NULL}};
const Segment SGI222376[] = {{&S93871[101], 9}, {NULL}};
const Segment SGI222377[] = {{&S93871[74], 6}, {NULL}};
const Segment SGI222378[] = {{&S89621[34], 21}, {NULL}};
const Segment SGI222379[] = {{&S89621[27], 6}, {NULL}};
const Segment SGI222380[] = {{&S89621[108], 9}, {NULL}};
const Segment SGI222381[] = {{&S89621[81], 6}, {NULL}};
const Segment SGI222382[] = {{&S93872[45], 5}, {NULL}};
const Segment SGI222383[] = {{&S93872[30], 6}, {NULL}};
const Segment SGI222384[] = {{&S93872[31], 3}, {NULL}};
const Segment SGI222385[] = {{&S93872[76], 5}, {NULL}};
const Segment SGI222386[] = {{&S93872[134], 13}, {NULL}};
const Segment SGI222387[] = {{&S83035[34], 6}, {NULL}};
const Segment SGI222388[] = {{&S83035[27], 6}, {NULL}};
const Segment SGI222389[] = {{&S83035[104], 9}, {NULL}};
const Segment SGI222390[] = {{&S83035[77], 6}, {NULL}};
const Segment SGI222391[] = {{&S89625[44], 17}, {NULL}};
const Segment SGI222392[] = {{&S89625[44], 29}, {NULL}};
const Segment SGI222393[] = {{&S89625[30], 6}, {NULL}};
const Segment SGI222394[] = {{&S89625[122], 3}, {NULL}};
const Segment SGI222395[] = {{&S89625[87], 6}, {NULL}};
const Segment SGI222396[] = {{&S93875[44], 15}, {NULL}};
const Segment SGI222397[] = {{&S93875[44], 26}, {NULL}};
const Segment SGI222398[] = {{&S93875[30], 6}, {NULL}};
const Segment SGI222399[] = {{&S93875[31], 3}, {NULL}};
const Segment SGI222400[] = {{&S93875[84], 6}, {NULL}};
const Segment SGI222401[] = {{&S89627[34], 7}, {NULL}};
const Segment SGI222402[] = {{&S89627[27], 6}, {NULL}};
const Segment SGI222403[] = {{&S89627[94], 9}, {NULL}};
const Segment SGI222404[] = {{&S89627[67], 6}, {NULL}};
const Segment SGI222405[] = {{&S89631[34], 7}, {NULL}};
const Segment SGI222406[] = {{&S89631[27], 6}, {NULL}};
const Segment SGI222407[] = {{&S89631[99], 9}, {NULL}};
const Segment SGI222408[] = {{&S89631[72], 6}, {NULL}};
const Segment SGI222409[] = {{&S93882[44], 13}, {NULL}};
const Segment SGI222410[] = {{&S93882[44], 30}, {NULL}};
const Segment SGI222411[] = {{&S93882[30], 6}, {NULL}};
const Segment SGI222412[] = {{&S93882[31], 3}, {NULL}};
const Segment SGI222413[] = {{&S93882[88], 6}, {NULL}};
const Segment SGI222414[] = {{&S93884[34], 7}, {NULL}};
const Segment SGI222415[] = {{&S93884[27], 6}, {NULL}};
const Segment SGI222416[] = {{&S93884[95], 9}, {NULL}};
const Segment SGI222417[] = {{&S93884[68], 6}, {NULL}};
const Segment SGI222418[] = {{&S93885[34], 14}, {NULL}};
const Segment SGI222419[] = {{&S93885[27], 6}, {NULL}};
const Segment SGI222420[] = {{&S93885[101], 9}, {NULL}};
const Segment SGI222421[] = {{&S93885[74], 6}, {NULL}};
const Segment SGI222422[] = {{&S93888[34], 22}, {NULL}};
const Segment SGI222423[] = {{&S93888[27], 6}, {NULL}};
const Segment SGI222424[] = {{&S93888[109], 9}, {NULL}};
const Segment SGI222425[] = {{&S93888[82], 6}, {NULL}};
const Segment SGI222426[] = {{&S89642[34], 13}, {NULL}};
const Segment SGI222427[] = {{&S89642[27], 6}, {NULL}};
const Segment SGI222428[] = {{&S89642[100], 9}, {NULL}};
const Segment SGI222429[] = {{&S89642[73], 6}, {NULL}};
const Segment SGI222430[] = {{&S89648[44], 10}, {NULL}};
const Segment SGI222431[] = {{&S89648[44], 22}, {NULL}};
const Segment SGI222432[] = {{&S89648[30], 6}, {NULL}};
const Segment SGI222433[] = {{&S89648[115], 3}, {NULL}};
const Segment SGI222434[] = {{&S93896[44], 16}, {NULL}};
const Segment SGI222435[] = {{&S89648[80], 6}, {NULL}};
const Segment SGI222436[] = {{&S93896[44], 28}, {NULL}};
const Segment SGI222437[] = {{&S93896[30], 6}, {NULL}};
const Segment SGI222438[] = {{&S89648[143], 3}, {NULL}};
const Segment SGI222439[] = {{&S93896[31], 3}, {NULL}};
const Segment SGI222440[] = {{&S93896[86], 6}, {NULL}};
const Segment SGI222441[] = {{&S84728[45], 14}, {NULL}};
const Segment SGI222442[] = {{&S84728[30], 6}, {NULL}};
const Segment SGI222443[] = {{&S84728[119], 3}, {NULL}};
const Segment SGI222444[] = {{&S84728[85], 5}, {NULL}};
const Segment SGI222445[] = {{&S84728[153], 3}, {NULL}};
const Segment SGI222446[] = {{&S93900[34], 10}, {NULL}};
const Segment SGI222447[] = {{&S93900[27], 6}, {NULL}};
const Segment SGI222448[] = {{&S93900[97], 9}, {NULL}};
const Segment SGI222449[] = {{&S93900[70], 6}, {NULL}};
const Segment SGI222450[] = {{&S93900[131], 8}, {NULL}};
const Segment SGI222451[] = {{&S93905[34], 11}, {NULL}};
const Segment SGI222452[] = {{&S93905[27], 6}, {NULL}};
const Segment SGI222453[] = {{&S93905[98], 9}, {NULL}};
const Segment SGI222454[] = {{&S93905[71], 6}, {NULL}};
const Segment SGI222455[] = {{&S93907[34], 23}, {NULL}};
const Segment SGI222456[] = {{&S93907[27], 6}, {NULL}};
const Segment SGI222457[] = {{&S93907[114], 9}, {NULL}};
const Segment SGI222458[] = {{&S93907[87], 6}, {NULL}};
const Segment SGI222459[] = {{&S83086[34], 10}, {NULL}};
const Segment SGI222460[] = {{&S83086[27], 6}, {NULL}};
const Segment SGI222461[] = {{&S83086[97], 9}, {NULL}};
const Segment SGI222462[] = {{&S83086[70], 6}, {NULL}};
const Segment SGI222463[] = {{&S83087[34], 10}, {NULL}};
const Segment SGI222464[] = {{&S83087[27], 6}, {NULL}};
const Segment SGI222465[] = {{&S83087[96], 9}, {NULL}};
const Segment SGI222466[] = {{&S83087[69], 6}, {NULL}};
const Segment SGI222467[] = {{&S93913[44], 8}, {NULL}};
const Segment SGI222468[] = {{&S93913[44], 20}, {NULL}};
const Segment SGI222469[] = {{&S93913[30], 6}, {NULL}};
const Segment SGI222470[] = {{&S93913[31], 3}, {NULL}};
const Segment SGI222471[] = {{&S93913[78], 6}, {NULL}};
const Segment SGI222472[] = {{&S84752[34], 15}, {NULL}};
const Segment SGI222473[] = {{&S84752[27], 6}, {NULL}};
const Segment SGI222474[] = {{&S84752[102], 9}, {NULL}};
const Segment SGI222475[] = {{&S84752[75], 6}, {NULL}};
const Segment SGI222476[] = {{&S89680[34], 6}, {NULL}};
const Segment SGI222477[] = {{&S89680[27], 6}, {NULL}};
const Segment SGI222478[] = {{&S89680[93], 9}, {NULL}};
const Segment SGI222479[] = {{&S89680[66], 6}, {NULL}};
const Segment SGI222480[] = {{&S83097[34], 10}, {NULL}};
const Segment SGI222481[] = {{&S83097[27], 6}, {NULL}};
const Segment SGI222482[] = {{&S83097[97], 9}, {NULL}};
const Segment SGI222483[] = {{&S83097[70], 6}, {NULL}};
const Segment SGI222484[] = {{&S83097[138], 8}, {NULL}};
const Segment SGI222485[] = {{&S89682[44], 8}, {NULL}};
const Segment SGI222486[] = {{&S83098[34], 11}, {NULL}};
const Segment SGI222487[] = {{&S89682[44], 20}, {NULL}};
const Segment SGI222488[] = {{&S89682[30], 6}, {NULL}};
const Segment SGI222489[] = {{&S83098[27], 6}, {NULL}};
const Segment SGI222490[] = {{&S89682[113], 3}, {NULL}};
const Segment SGI222491[] = {{&S89682[78], 6}, {NULL}};
const Segment SGI222492[] = {{&S83098[98], 9}, {NULL}};
const Segment SGI222493[] = {{&S83098[71], 6}, {NULL}};
const Segment SGI222494[] = {{&S83098[139], 8}, {NULL}};
const Segment SGI222495[] = {{&S93929[52], 10}, {NULL}};
const Segment SGI222496[] = {{&S93929[52], 22}, {NULL}};
const Segment SGI222497[] = {{&S93929[38], 6}, {NULL}};
const Segment SGI222498[] = {{&S93929[88], 5}, {NULL}};
const Segment SGI222499[] = {{&S93929[0], 14}, {NULL}};
const Segment SGI222500[] = {{&S93931[52], 10}, {NULL}};
const Segment SGI222501[] = {{&S93931[52], 22}, {NULL}};
const Segment SGI222502[] = {{&S93931[38], 6}, {NULL}};
const Segment SGI222503[] = {{&S93931[88], 5}, {NULL}};
const Segment SGI222504[] = {{&S93931[0], 14}, {NULL}};
const Segment SGI222505[] = {{&S75718[34], 7}, {NULL}};
const Segment SGI222506[] = {{&S75718[27], 6}, {NULL}};
const Segment SGI222507[] = {{&S83110[34], 11}, {NULL}};
const Segment SGI222508[] = {{&S83110[27], 6}, {NULL}};
const Segment SGI222509[] = {{&S75718[100], 9}, {NULL}};
const Segment SGI222510[] = {{&S75718[73], 6}, {NULL}};
const Segment SGI222511[] = {{&S83110[98], 9}, {NULL}};
const Segment SGI222512[] = {{&S83110[71], 6}, {NULL}};
const Segment SGI222513[] = {{&S83110[139], 8}, {NULL}};
const Segment SGI222514[] = {{&S93938[34], 17}, {NULL}};
const Segment SGI222515[] = {{&S93938[27], 6}, {NULL}};
const Segment SGI222516[] = {{&S93938[111], 9}, {NULL}};
const Segment SGI222517[] = {{&S93938[84], 6}, {NULL}};
const Segment SGI222518[] = {{&S83116[34], 10}, {NULL}};
const Segment SGI222519[] = {{&S83116[27], 6}, {NULL}};
const Segment SGI222520[] = {{&S83116[97], 9}, {NULL}};
const Segment SGI222521[] = {{&S83116[70], 6}, {NULL}};
const Segment SGI222522[] = {{&S83117[34], 11}, {NULL}};
const Segment SGI222523[] = {{&S83117[27], 6}, {NULL}};
const Segment SGI222524[] = {{&S83117[98], 9}, {NULL}};
const Segment SGI222525[] = {{&S83117[71], 6}, {NULL}};
const Segment SGI222526[] = {{&S93951[34], 10}, {NULL}};
const Segment SGI222527[] = {{&S93951[27], 6}, {NULL}};
const Segment SGI222528[] = {{&S93951[97], 9}, {NULL}};
const Segment SGI222529[] = {{&S93951[70], 6}, {NULL}};
const Segment SGI222530[] = {{&S93951[131], 8}, {NULL}};
const Segment SGI222531[] = {{&S93953[34], 10}, {NULL}};
const Segment SGI222532[] = {{&S93953[27], 6}, {NULL}};
const Segment SGI222533[] = {{&S93953[97], 9}, {NULL}};
const Segment SGI222534[] = {{&S93953[70], 6}, {NULL}};
const Segment SGI222535[] = {{&S75748[34], 10}, {NULL}};
const Segment SGI222536[] = {{&S75748[27], 6}, {NULL}};
const Segment SGI222537[] = {{&S97378[34], 7}, {NULL}};
const Segment SGI222538[] = {{&S97378[27], 6}, {NULL}};
const Segment SGI222539[] = {{&S75748[96], 9}, {NULL}};
const Segment SGI222540[] = {{&S75748[69], 6}, {NULL}};
const Segment SGI222541[] = {{&S97378[67], 6}, {NULL}};
const Segment SGI222542[] = {{&S84807[34], 28}, {NULL}};
const Segment SGI222543[] = {{&S84807[27], 6}, {NULL}};
const Segment SGI222544[] = {{&S84807[115], 9}, {NULL}};
const Segment SGI222545[] = {{&S84807[88], 6}, {NULL}};
const Segment SGI222546[] = {{&S75761[34], 23}, {NULL}};
const Segment SGI222547[] = {{&S75761[27], 6}, {NULL}};
const Segment SGI222548[] = {{&S75761[110], 9}, {NULL}};
const Segment SGI222549[] = {{&S75761[83], 6}, {NULL}};
const Segment SGI222550[] = {{&S89761[34], 13}, {NULL}};
const Segment SGI222551[] = {{&S89761[27], 6}, {NULL}};
const Segment SGI222552[] = {{&S89761[100], 9}, {NULL}};
const Segment SGI222553[] = {{&S89761[73], 6}, {NULL}};
const Segment SGI222554[] = {{&S83170[34], 10}, {NULL}};
const Segment SGI222555[] = {{&S83170[27], 6}, {NULL}};
const Segment SGI222556[] = {{&S84832[44], 9}, {NULL}};
const Segment SGI222557[] = {{&S84832[44], 21}, {NULL}};
const Segment SGI222558[] = {{&S84832[30], 6}, {NULL}};
const Segment SGI222559[] = {{&S83170[97], 9}, {NULL}};
const Segment SGI222560[] = {{&S83170[70], 6}, {NULL}};
const Segment SGI222561[] = {{&S84832[114], 3}, {NULL}};
const Segment SGI222562[] = {{&S84832[79], 6}, {NULL}};
const Segment SGI222563[] = {{&S89766[44], 10}, {NULL}};
const Segment SGI222564[] = {{&S89766[44], 22}, {NULL}};
const Segment SGI222565[] = {{&S89766[30], 6}, {NULL}};
const Segment SGI222566[] = {{&S89766[115], 3}, {NULL}};
const Segment SGI222567[] = {{&S89766[80], 6}, {NULL}};
const Segment SGI222568[] = {{&S89766[143], 3}, {NULL}};
const Segment SGI222569[] = {{&S83180[34], 6}, {NULL}};
const Segment SGI222570[] = {{&S83180[27], 6}, {NULL}};
const Segment SGI222571[] = {{&S83180[104], 9}, {NULL}};
const Segment SGI222572[] = {{&S83180[77], 6}, {NULL}};
const Segment SGI222573[] = {{&S89775[44], 11}, {NULL}};
const Segment SGI222574[] = {{&S89775[44], 23}, {NULL}};
const Segment SGI222575[] = {{&S89775[30], 6}, {NULL}};
const Segment SGI222576[] = {{&S89775[81], 6}, {NULL}};
const Segment SGI222577[] = {{&S89775[108], 13}, {NULL}};
const Segment SGI222578[] = {{&S83187[34], 5}, {NULL}};
const Segment SGI222579[] = {{&S89777[34], 12}, {NULL}};
const Segment SGI222580[] = {{&S83187[27], 6}, {NULL}};
const Segment SGI222581[] = {{&S89777[27], 6}, {NULL}};
const Segment SGI222582[] = {{&S83187[92], 9}, {NULL}};
const Segment SGI222583[] = {{&S83187[65], 6}, {NULL}};
const Segment SGI222584[] = {{&S89777[99], 9}, {NULL}};
const Segment SGI222585[] = {{&S89777[72], 6}, {NULL}};
const Segment SGI222586[] = {{&S89778[34], 10}, {NULL}};
const Segment SGI222587[] = {{&S89778[27], 6}, {NULL}};
const Segment SGI222588[] = {{&S89778[97], 9}, {NULL}};
const Segment SGI222589[] = {{&S89778[70], 6}, {NULL}};
const Segment SGI222590[] = {{&S83189[34], 15}, {NULL}};
const Segment SGI222591[] = {{&S83189[27], 6}, {NULL}};
const Segment SGI222592[] = {{&S83189[102], 9}, {NULL}};
const Segment SGI222593[] = {{&S83189[75], 6}, {NULL}};
const Segment SGI222594[] = {{&S89782[34], 7}, {NULL}};
const Segment SGI222595[] = {{&S89782[27], 6}, {NULL}};
const Segment SGI222596[] = {{&S89782[99], 9}, {NULL}};
const Segment SGI222597[] = {{&S89782[72], 6}, {NULL}};
const Segment SGI222598[] = {{&S83198[34], 11}, {NULL}};
const Segment SGI222599[] = {{&S83198[27], 6}, {NULL}};
const Segment SGI222600[] = {{&S83198[98], 9}, {NULL}};
const Segment SGI222601[] = {{&S83198[71], 6}, {NULL}};
const Segment SGI222602[] = {{&S94031[44], 26}, {NULL}};
const Segment SGI222603[] = {{&S94031[44], 38}, {NULL}};
const Segment SGI222604[] = {{&S94031[30], 6}, {NULL}};
const Segment SGI222605[] = {{&S94031[31], 3}, {NULL}};
const Segment SGI222606[] = {{&S94031[111], 6}, {NULL}};
const Segment SGI222607[] = {{&S94031[84], 8}, {NULL}};
const Segment SGI222608[] = {{&S94032[44], 24}, {NULL}};
const Segment SGI222609[] = {{&S94032[44], 36}, {NULL}};
const Segment SGI222610[] = {{&S94032[30], 6}, {NULL}};
const Segment SGI222611[] = {{&S94032[31], 3}, {NULL}};
const Segment SGI222612[] = {{&S94032[109], 6}, {NULL}};
const Segment SGI222613[] = {{&S94032[82], 8}, {NULL}};
const Segment SGI222614[] = {{&S89813[34], 7}, {NULL}};
const Segment SGI222615[] = {{&S89813[27], 6}, {NULL}};
const Segment SGI222616[] = {{&S89813[94], 9}, {NULL}};
const Segment SGI222617[] = {{&S89813[67], 6}, {NULL}};
const Segment SGI222618[] = {{&S89814[34], 10}, {NULL}};
const Segment SGI222619[] = {{&S89814[27], 6}, {NULL}};
const Segment SGI222620[] = {{&S89814[97], 9}, {NULL}};
const Segment SGI222621[] = {{&S89814[70], 6}, {NULL}};
const Segment SGI222622[] = {{&S83230[34], 10}, {NULL}};
const Segment SGI222623[] = {{&S83230[27], 6}, {NULL}};
const Segment SGI222624[] = {{&S83230[97], 9}, {NULL}};
const Segment SGI222625[] = {{&S83230[70], 6}, {NULL}};
const Segment SGI222626[] = {{&S75846[44], 11}, {NULL}};
const Segment SGI222627[] = {{&S75846[44], 22}, {NULL}};
const Segment SGI222628[] = {{&S75846[30], 6}, {NULL}};
const Segment SGI222629[] = {{&S94053[34], 12}, {NULL}};
const Segment SGI222630[] = {{&S75846[133], 3}, {NULL}};
const Segment SGI222631[] = {{&S94053[27], 6}, {NULL}};
const Segment SGI222632[] = {{&S75846[98], 6}, {NULL}};
const Segment SGI222633[] = {{&S83233[34], 9}, {NULL}};
const Segment SGI222634[] = {{&S75846[68], 8}, {NULL}};
const Segment SGI222635[] = {{&S83233[27], 6}, {NULL}};
const Segment SGI222636[] = {{&S94053[99], 9}, {NULL}};
const Segment SGI222637[] = {{&S94053[72], 6}, {NULL}};
const Segment SGI222638[] = {{&S83233[96], 9}, {NULL}};
const Segment SGI222639[] = {{&S83233[69], 6}, {NULL}};
const Segment SGI222640[] = {{&S75848[44], 11}, {NULL}};
const Segment SGI222641[] = {{&S75848[44], 23}, {NULL}};
const Segment SGI222642[] = {{&S75848[30], 6}, {NULL}};
const Segment SGI222643[] = {{&S75848[116], 3}, {NULL}};
const Segment SGI222644[] = {{&S75848[81], 6}, {NULL}};
const Segment SGI222645[] = {{&S75848[141], 8}, {NULL}};
const Segment SGI222646[] = {{&S83236[34], 7}, {NULL}};
const Segment SGI222647[] = {{&S94056[34], 11}, {NULL}};
const Segment SGI222648[] = {{&S83236[27], 6}, {NULL}};
const Segment SGI222649[] = {{&S94056[27], 6}, {NULL}};
const Segment SGI222650[] = {{&S83236[99], 9}, {NULL}};
const Segment SGI222651[] = {{&S94056[98], 9}, {NULL}};
const Segment SGI222652[] = {{&S83236[72], 6}, {NULL}};
const Segment SGI222653[] = {{&S94056[71], 6}, {NULL}};
const Segment SGI222654[] = {{&S94057[34], 8}, {NULL}};
const Segment SGI222655[] = {{&S94057[27], 6}, {NULL}};
const Segment SGI222656[] = {{&S94057[95], 9}, {NULL}};
const Segment SGI222657[] = {{&S94057[68], 6}, {NULL}};
const Segment SGI222658[] = {{&S83238[6], 4}, {NULL}};
const Segment SGI222659[] = {{&S83238[81], 5}, {NULL}};
const Segment SGI222660[] = {{&S75852[44], 18}, {NULL}};
const Segment SGI222661[] = {{&S75852[44], 29}, {NULL}};
const Segment SGI222662[] = {{&S75852[30], 6}, {NULL}};
const Segment SGI222663[] = {{&S75852[122], 3}, {NULL}};
const Segment SGI222664[] = {{&S75852[87], 6}, {NULL}};
const Segment SGI222665[] = {{&S89829[34], 21}, {NULL}};
const Segment SGI222666[] = {{&S89829[27], 6}, {NULL}};
const Segment SGI222667[] = {{&S89829[108], 9}, {NULL}};
const Segment SGI222668[] = {{&S89829[81], 6}, {NULL}};
const Segment SGI222669[] = {{&S94059[44], 7}, {NULL}};
const Segment SGI222670[] = {{&S94059[44], 19}, {NULL}};
const Segment SGI222671[] = {{&S94059[30], 6}, {NULL}};
const Segment SGI222672[] = {{&S94059[31], 3}, {NULL}};
const Segment SGI222673[] = {{&S94059[77], 6}, {NULL}};
const Segment SGI222674[] = {{&S89830[34], 4}, {NULL}};
const Segment SGI222675[] = {{&S89830[27], 6}, {NULL}};
const Segment SGI222676[] = {{&S89830[91], 9}, {NULL}};
const Segment SGI222677[] = {{&S89830[64], 6}, {NULL}};
const Segment SGI222678[] = {{&S83243[44], 10}, {NULL}};
const Segment SGI222679[] = {{&S83243[44], 22}, {NULL}};
const Segment SGI222680[] = {{&S83243[30], 6}, {NULL}};
const Segment SGI222681[] = {{&S83243[115], 3}, {NULL}};
const Segment SGI222682[] = {{&S83243[80], 6}, {NULL}};
const Segment SGI222683[] = {{&S83243[139], 10}, {NULL}};
const Segment SGI222684[] = {{&S94062[34], 7}, {NULL}};
const Segment SGI222685[] = {{&S94062[27], 6}, {NULL}};
const Segment SGI222686[] = {{&S94062[94], 9}, {NULL}};
const Segment SGI222687[] = {{&S94062[67], 6}, {NULL}};
const Segment SGI222688[] = {{&S83247[34], 13}, {NULL}};
const Segment SGI222689[] = {{&S83247[27], 6}, {NULL}};
const Segment SGI222690[] = {{&S83247[100], 9}, {NULL}};
const Segment SGI222691[] = {{&S83247[73], 6}, {NULL}};
const Segment SGI222692[] = {{&S94069[44], 21}, {NULL}};
const Segment SGI222693[] = {{&S94069[44], 33}, {NULL}};
const Segment SGI222694[] = {{&S94069[30], 6}, {NULL}};
const Segment SGI222695[] = {{&S94069[31], 3}, {NULL}};
const Segment SGI222696[] = {{&S94069[112], 6}, {NULL}};
const Segment SGI222697[] = {{&S94069[79], 8}, {NULL}};
const Segment SGI222698[] = {{&S94070[34], 16}, {NULL}};
const Segment SGI222699[] = {{&S94070[27], 6}, {NULL}};
const Segment SGI222700[] = {{&S94070[103], 9}, {NULL}};
const Segment SGI222701[] = {{&S83254[34], 9}, {NULL}};
const Segment SGI222702[] = {{&S94070[76], 6}, {NULL}};
const Segment SGI222703[] = {{&S83254[27], 6}, {NULL}};
const Segment SGI222704[] = {{&S83254[96], 9}, {NULL}};
const Segment SGI222705[] = {{&S83254[69], 6}, {NULL}};
const Segment SGI222706[] = {{&S83254[130], 8}, {NULL}};
const Segment SGI222707[] = {{&S94071[45], 14}, {NULL}};
const Segment SGI222708[] = {{&S94071[30], 6}, {NULL}};
const Segment SGI222709[] = {{&S94071[31], 3}, {NULL}};
const Segment SGI222710[] = {{&S94071[85], 5}, {NULL}};
const Segment SGI222711[] = {{&S94071[143], 13}, {NULL}};
const Segment SGI222712[] = {{&S94073[45], 22}, {NULL}};
const Segment SGI222713[] = {{&S94073[30], 6}, {NULL}};
const Segment SGI222714[] = {{&S94073[31], 3}, {NULL}};
const Segment SGI222715[] = {{&S94073[93], 5}, {NULL}};
const Segment SGI222716[] = {{&S94073[151], 13}, {NULL}};
const Segment SGI222717[] = {{&S84932[34], 6}, {NULL}};
const Segment SGI222718[] = {{&S84932[27], 6}, {NULL}};
const Segment SGI222719[] = {{&S94078[34], 17}, {NULL}};
const Segment SGI222720[] = {{&S94078[27], 6}, {NULL}};
const Segment SGI222721[] = {{&S84932[93], 9}, {NULL}};
const Segment SGI222722[] = {{&S84932[66], 6}, {NULL}};
const Segment SGI222723[] = {{&S94078[104], 9}, {NULL}};
const Segment SGI222724[] = {{&S94078[77], 6}, {NULL}};
const Segment SGI222725[] = {{&S94081[34], 10}, {NULL}};
const Segment SGI222726[] = {{&S94081[27], 6}, {NULL}};
const Segment SGI222727[] = {{&S94081[97], 9}, {NULL}};
const Segment SGI222728[] = {{&S94081[70], 6}, {NULL}};
const Segment SGI222729[] = {{&S94082[34], 9}, {NULL}};
const Segment SGI222730[] = {{&S94082[27], 6}, {NULL}};
const Segment SGI222731[] = {{&S94082[93], 9}, {NULL}};
const Segment SGI222732[] = {{&S94082[66], 6}, {NULL}};
const Segment SGI222733[] = {{&S94087[34], 8}, {NULL}};
const Segment SGI222734[] = {{&S94087[27], 6}, {NULL}};
const Segment SGI222735[] = {{&S94087[95], 9}, {NULL}};
const Segment SGI222736[] = {{&S94087[68], 6}, {NULL}};
const Segment SGI222737[] = {{&S94088[34], 13}, {NULL}};
const Segment SGI222738[] = {{&S94088[27], 6}, {NULL}};
const Segment SGI222739[] = {{&S94088[100], 9}, {NULL}};
const Segment SGI222740[] = {{&S94088[73], 6}, {NULL}};
const Segment SGI222741[] = {{&S94090[34], 10}, {NULL}};
const Segment SGI222742[] = {{&S94090[27], 6}, {NULL}};
const Segment SGI222743[] = {{&S94090[97], 9}, {NULL}};
const Segment SGI222744[] = {{&S94090[70], 6}, {NULL}};
const Segment SGI222745[] = {{&S94094[44], 8}, {NULL}};
const Segment SGI222746[] = {{&S94094[44], 20}, {NULL}};
const Segment SGI222747[] = {{&S94094[30], 6}, {NULL}};
const Segment SGI222748[] = {{&S94094[31], 3}, {NULL}};
const Segment SGI222749[] = {{&S94094[78], 6}, {NULL}};
const Segment SGI222750[] = {{&S94095[44], 7}, {NULL}};
const Segment SGI222751[] = {{&S94095[44], 31}, {NULL}};
const Segment SGI222752[] = {{&S94095[30], 6}, {NULL}};
const Segment SGI222753[] = {{&S94095[31], 3}, {NULL}};
const Segment SGI222754[] = {{&S94095[89], 6}, {NULL}};
const Segment SGI222755[] = {{&S94095[149], 11}, {NULL}};
const Segment SGI222756[] = {{&S94096[34], 8}, {NULL}};
const Segment SGI222757[] = {{&S94096[27], 6}, {NULL}};
const Segment SGI222758[] = {{&S94096[95], 9}, {NULL}};
const Segment SGI222759[] = {{&S94096[68], 6}, {NULL}};
const Segment SGI222760[] = {{&S94097[34], 15}, {NULL}};
const Segment SGI222761[] = {{&S94097[27], 6}, {NULL}};
const Segment SGI222762[] = {{&S94097[109], 9}, {NULL}};
const Segment SGI222763[] = {{&S94097[82], 6}, {NULL}};
const Segment SGI222764[] = {{&S94098[34], 7}, {NULL}};
const Segment SGI222765[] = {{&S94098[27], 6}, {NULL}};
const Segment SGI222766[] = {{&S94098[94], 9}, {NULL}};
const Segment SGI222767[] = {{&S94098[67], 6}, {NULL}};
const Segment SGI222768[] = {{&S89896[44], 14}, {NULL}};
const Segment SGI222769[] = {{&S89896[44], 26}, {NULL}};
const Segment SGI222770[] = {{&S89896[30], 6}, {NULL}};
const Segment SGI222771[] = {{&S89896[119], 3}, {NULL}};
const Segment SGI222772[] = {{&S89896[84], 6}, {NULL}};
const Segment SGI222773[] = {{&S94102[34], 10}, {NULL}};
const Segment SGI222774[] = {{&S94102[27], 6}, {NULL}};
const Segment SGI222775[] = {{&S94102[97], 9}, {NULL}};
const Segment SGI222776[] = {{&S94102[70], 6}, {NULL}};
const Segment SGI222777[] = {{&S89897[44], 14}, {NULL}};
const Segment SGI222778[] = {{&S89897[44], 26}, {NULL}};
const Segment SGI222779[] = {{&S89897[30], 6}, {NULL}};
const Segment SGI222780[] = {{&S89897[119], 3}, {NULL}};
const Segment SGI222781[] = {{&S94103[34], 7}, {NULL}};
const Segment SGI222782[] = {{&S89897[84], 6}, {NULL}};
const Segment SGI222783[] = {{&S94103[27], 6}, {NULL}};
const Segment SGI222784[] = {{&S94103[94], 9}, {NULL}};
const Segment SGI222785[] = {{&S94103[67], 6}, {NULL}};
const Segment SGI222786[] = {{&S94106[34], 18}, {NULL}};
const Segment SGI222787[] = {{&S94106[27], 6}, {NULL}};
const Segment SGI222788[] = {{&S94106[105], 9}, {NULL}};
const Segment SGI222789[] = {{&S94106[78], 6}, {NULL}};
const Segment SGI222790[] = {{&S94108[44], 5}, {NULL}};
const Segment SGI222791[] = {{&S94108[44], 17}, {NULL}};
const Segment SGI222792[] = {{&S94108[30], 6}, {NULL}};
const Segment SGI222793[] = {{&S94108[31], 3}, {NULL}};
const Segment SGI222794[] = {{&S94108[75], 5}, {NULL}};
const Segment SGI222795[] = {{&S94108[112], 14}, {NULL}};
const Segment SGI222796[] = {{&S89907[45], 11}, {NULL}};
const Segment SGI222797[] = {{&S89907[30], 6}, {NULL}};
const Segment SGI222798[] = {{&S89907[116], 3}, {NULL}};
const Segment SGI222799[] = {{&S94110[59], 12}, {NULL}};
const Segment SGI222800[] = {{&S89907[82], 5}, {NULL}};
const Segment SGI222801[] = {{&S94110[59], 24}, {NULL}};
const Segment SGI222802[] = {{&S94110[45], 6}, {NULL}};
const Segment SGI222803[] = {{&S89907[140], 13}, {NULL}};
const Segment SGI222804[] = {{&S94110[46], 3}, {NULL}};
const Segment SGI222805[] = {{&S94110[97], 6}, {NULL}};
const Segment SGI222806[] = {{&S94110[0], 14}, {NULL}};
const Segment SGI222807[] = {{&S89909[34], 18}, {NULL}};
const Segment SGI222808[] = {{&S89909[27], 6}, {NULL}};
const Segment SGI222809[] = {{&S89909[105], 9}, {NULL}};
const Segment SGI222810[] = {{&S89909[78], 6}, {NULL}};
const Segment SGI222811[] = {{&S94112[45], 7}, {NULL}};
const Segment SGI222812[] = {{&S94112[30], 6}, {NULL}};
const Segment SGI222813[] = {{&S94112[31], 3}, {NULL}};
const Segment SGI222814[] = {{&S94112[83], 5}, {NULL}};
const Segment SGI222815[] = {{&S94112[141], 13}, {NULL}};
const Segment SGI222816[] = {{&S89913[44], 9}, {NULL}};
const Segment SGI222817[] = {{&S89913[44], 21}, {NULL}};
const Segment SGI222818[] = {{&S89913[30], 6}, {NULL}};
const Segment SGI222819[] = {{&S89913[114], 3}, {NULL}};
const Segment SGI222820[] = {{&S89913[79], 6}, {NULL}};
const Segment SGI222821[] = {{&S94120[34], 12}, {NULL}};
const Segment SGI222822[] = {{&S94120[27], 6}, {NULL}};
const Segment SGI222823[] = {{&S94120[99], 9}, {NULL}};
const Segment SGI222824[] = {{&S75954[34], 30}, {NULL}};
const Segment SGI222825[] = {{&S94120[72], 6}, {NULL}};
const Segment SGI222826[] = {{&S75954[27], 6}, {NULL}};
const Segment SGI222827[] = {{&S75954[117], 9}, {NULL}};
const Segment SGI222828[] = {{&S75954[90], 6}, {NULL}};
const Segment SGI222829[] = {{&S89921[44], 10}, {NULL}};
const Segment SGI222830[] = {{&S89921[44], 22}, {NULL}};
const Segment SGI222831[] = {{&S89921[30], 6}, {NULL}};
const Segment SGI222832[] = {{&S94122[34], 7}, {NULL}};
const Segment SGI222833[] = {{&S89921[131], 3}, {NULL}};
const Segment SGI222834[] = {{&S94122[27], 6}, {NULL}};
const Segment SGI222835[] = {{&S89921[96], 6}, {NULL}};
const Segment SGI222836[] = {{&S89921[68], 8}, {NULL}};
const Segment SGI222837[] = {{&S94122[99], 9}, {NULL}};
const Segment SGI222838[] = {{&S94122[72], 6}, {NULL}};
const Segment SGI222839[] = {{&S94122[140], 8}, {NULL}};
const Segment SGI222840[] = {{&S89922[44], 10}, {NULL}};
const Segment SGI222841[] = {{&S89922[44], 22}, {NULL}};
const Segment SGI222842[] = {{&S89922[30], 6}, {NULL}};
const Segment SGI222843[] = {{&S89922[131], 3}, {NULL}};
const Segment SGI222844[] = {{&S89922[96], 6}, {NULL}};
const Segment SGI222845[] = {{&S89922[68], 8}, {NULL}};
const Segment SGI222846[] = {{&S94124[34], 7}, {NULL}};
const Segment SGI222847[] = {{&S94124[27], 6}, {NULL}};
const Segment SGI222848[] = {{&S83334[34], 18}, {NULL}};
const Segment SGI222849[] = {{&S83334[27], 6}, {NULL}};
const Segment SGI222850[] = {{&S94124[94], 9}, {NULL}};
const Segment SGI222851[] = {{&S94124[67], 6}, {NULL}};
const Segment SGI222852[] = {{&S83334[105], 9}, {NULL}};
const Segment SGI222853[] = {{&S83334[78], 6}, {NULL}};
const Segment SGI222854[] = {{&S75960[34], 10}, {NULL}};
const Segment SGI222855[] = {{&S75960[27], 6}, {NULL}};
const Segment SGI222856[] = {{&S75960[97], 9}, {NULL}};
const Segment SGI222857[] = {{&S75960[70], 6}, {NULL}};
const Segment SGI222858[] = {{&S83337[44], 10}, {NULL}};
const Segment SGI222859[] = {{&S83337[44], 22}, {NULL}};
const Segment SGI222860[] = {{&S83337[30], 6}, {NULL}};
const Segment SGI222861[] = {{&S83337[135], 3}, {NULL}};
const Segment SGI222862[] = {{&S83337[100], 6}, {NULL}};
const Segment SGI222863[] = {{&S83337[68], 8}, {NULL}};
const Segment SGI222864[] = {{&S83338[44], 10}, {NULL}};
const Segment SGI222865[] = {{&S83338[44], 26}, {NULL}};
const Segment SGI222866[] = {{&S83338[30], 6}, {NULL}};
const Segment SGI222867[] = {{&S83338[134], 3}, {NULL}};
const Segment SGI222868[] = {{&S83338[99], 6}, {NULL}};
const Segment SGI222869[] = {{&S83338[72], 8}, {NULL}};
const Segment SGI222870[] = {{&S94132[44], 9}, {NULL}};
const Segment SGI222871[] = {{&S94132[44], 21}, {NULL}};
const Segment SGI222872[] = {{&S94132[30], 6}, {NULL}};
const Segment SGI222873[] = {{&S94132[31], 3}, {NULL}};
const Segment SGI222874[] = {{&S94132[79], 6}, {NULL}};
const Segment SGI222875[] = {{&S89934[34], 14}, {NULL}};
const Segment SGI222876[] = {{&S89934[27], 6}, {NULL}};
const Segment SGI222877[] = {{&S89934[101], 9}, {NULL}};
const Segment SGI222878[] = {{&S89934[74], 6}, {NULL}};
const Segment SGI222879[] = {{&S94136[44], 10}, {NULL}};
const Segment SGI222880[] = {{&S94136[44], 22}, {NULL}};
const Segment SGI222881[] = {{&S94136[30], 6}, {NULL}};
const Segment SGI222882[] = {{&S94136[31], 3}, {NULL}};
const Segment SGI222883[] = {{&S94136[80], 6}, {NULL}};
const Segment SGI222884[] = {{&S94136[140], 7}, {NULL}};
const Segment SGI222885[] = {{&S85022[44], 25}, {NULL}};
const Segment SGI222886[] = {{&S85022[44], 37}, {NULL}};
const Segment SGI222887[] = {{&S85022[30], 6}, {NULL}};
const Segment SGI222888[] = {{&S85022[31], 3}, {NULL}};
const Segment SGI222889[] = {{&S83349[44], 7}, {NULL}};
const Segment SGI222890[] = {{&S85022[95], 6}, {NULL}};
const Segment SGI222891[] = {{&S83349[44], 19}, {NULL}};
const Segment SGI222892[] = {{&S83349[30], 6}, {NULL}};
const Segment SGI222893[] = {{&S83349[112], 3}, {NULL}};
const Segment SGI222894[] = {{&S94139[44], 5}, {NULL}};
const Segment SGI222895[] = {{&S83349[77], 6}, {NULL}};
const Segment SGI222896[] = {{&S94139[44], 17}, {NULL}};
const Segment SGI222897[] = {{&S94139[30], 6}, {NULL}};
const Segment SGI222898[] = {{&S94139[75], 6}, {NULL}};
const Segment SGI222899[] = {{&S94139[102], 13}, {NULL}};
const Segment SGI222900[] = {{&S89941[34], 11}, {NULL}};
const Segment SGI222901[] = {{&S89941[27], 6}, {NULL}};
const Segment SGI222902[] = {{&S89941[98], 9}, {NULL}};
const Segment SGI222903[] = {{&S89941[71], 6}, {NULL}};
const Segment SGI222904[] = {{&S85026[34], 17}, {NULL}};
const Segment SGI222905[] = {{&S85026[27], 6}, {NULL}};
const Segment SGI222906[] = {{&S83352[34], 11}, {NULL}};
const Segment SGI222907[] = {{&S83352[27], 6}, {NULL}};
const Segment SGI222908[] = {{&S85026[104], 9}, {NULL}};
const Segment SGI222909[] = {{&S85026[77], 6}, {NULL}};
const Segment SGI222910[] = {{&S83352[98], 9}, {NULL}};
const Segment SGI222911[] = {{&S83352[71], 6}, {NULL}};
const Segment SGI222912[] = {{&S83352[139], 8}, {NULL}};
const Segment SGI222913[] = {{&S89946[44], 12}, {NULL}};
const Segment SGI222914[] = {{&S89946[44], 24}, {NULL}};
const Segment SGI222915[] = {{&S89946[30], 6}, {NULL}};
const Segment SGI222916[] = {{&S94145[44], 6}, {NULL}};
const Segment SGI222917[] = {{&S94145[44], 18}, {NULL}};
const Segment SGI222918[] = {{&S94145[30], 6}, {NULL}};
const Segment SGI222919[] = {{&S89946[117], 3}, {NULL}};
const Segment SGI222920[] = {{&S89946[82], 6}, {NULL}};
const Segment SGI222921[] = {{&S94145[76], 6}, {NULL}};
const Segment SGI222922[] = {{&S89946[142], 11}, {NULL}};
const Segment SGI222923[] = {{&S94145[103], 13}, {NULL}};
const Segment SGI222924[] = {{&S89953[34], 7}, {NULL}};
const Segment SGI222925[] = {{&S89953[27], 6}, {NULL}};
const Segment SGI222926[] = {{&S89953[98], 9}, {NULL}};
const Segment SGI222927[] = {{&S89953[71], 6}, {NULL}};
const Segment SGI222928[] = {{&S94152[44], 9}, {NULL}};
const Segment SGI222929[] = {{&S94152[44], 21}, {NULL}};
const Segment SGI222930[] = {{&S94152[30], 6}, {NULL}};
const Segment SGI222931[] = {{&S94152[79], 6}, {NULL}};
const Segment SGI222932[] = {{&S94152[106], 13}, {NULL}};
const Segment SGI222933[] = {{&S83365[34], 11}, {NULL}};
const Segment SGI222934[] = {{&S83365[27], 6}, {NULL}};
const Segment SGI222935[] = {{&S83365[98], 9}, {NULL}};
const Segment SGI222936[] = {{&S83365[71], 6}, {NULL}};
const Segment SGI222937[] = {{&S75997[34], 10}, {NULL}};
const Segment SGI222938[] = {{&S75997[27], 6}, {NULL}};
const Segment SGI222939[] = {{&S94153[44], 13}, {NULL}};
const Segment SGI222940[] = {{&S75997[96], 9}, {NULL}};
const Segment SGI222941[] = {{&S75997[69], 6}, {NULL}};
const Segment SGI222942[] = {{&S94153[44], 25}, {NULL}};
const Segment SGI222943[] = {{&S94153[30], 6}, {NULL}};
const Segment SGI222944[] = {{&S83366[34], 11}, {NULL}};
const Segment SGI222945[] = {{&S97502[34], 7}, {NULL}};
const Segment SGI222946[] = {{&S94153[83], 6}, {NULL}};
const Segment SGI222947[] = {{&S83366[27], 6}, {NULL}};
const Segment SGI222948[] = {{&S97502[27], 6}, {NULL}};
const Segment SGI222949[] = {{&S94153[110], 13}, {NULL}};
const Segment SGI222950[] = {{&S97502[67], 6}, {NULL}};
const Segment SGI222951[] = {{&S83366[98], 9}, {NULL}};
const Segment SGI222952[] = {{&S83366[71], 6}, {NULL}};
const Segment SGI222953[] = {{&S83367[34], 11}, {NULL}};
const Segment SGI222954[] = {{&S83367[27], 6}, {NULL}};
const Segment SGI222955[] = {{&S83367[98], 9}, {NULL}};
const Segment SGI222956[] = {{&S83367[71], 6}, {NULL}};
const Segment SGI222957[] = {{&S83368[34], 5}, {NULL}};
const Segment SGI222958[] = {{&S83368[27], 6}, {NULL}};
const Segment SGI222959[] = {{&S83368[92], 9}, {NULL}};
const Segment SGI222960[] = {{&S83368[65], 6}, {NULL}};
const Segment SGI222961[] = {{&S94159[34], 5}, {NULL}};
const Segment SGI222962[] = {{&S94159[27], 6}, {NULL}};
const Segment SGI222963[] = {{&S94159[92], 9}, {NULL}};
const Segment SGI222964[] = {{&S94159[65], 6}, {NULL}};
const Segment SGI222965[] = {{&S94160[34], 10}, {NULL}};
const Segment SGI222966[] = {{&S94160[27], 6}, {NULL}};
const Segment SGI222967[] = {{&S94160[104], 9}, {NULL}};
const Segment SGI222968[] = {{&S94160[77], 6}, {NULL}};
const Segment SGI222969[] = {{&S94161[34], 9}, {NULL}};
const Segment SGI222970[] = {{&S94161[27], 6}, {NULL}};
const Segment SGI222971[] = {{&S94161[96], 9}, {NULL}};
const Segment SGI222972[] = {{&S94161[69], 6}, {NULL}};
const Segment SGI222973[] = {{&S89976[34], 25}, {NULL}};
const Segment SGI222974[] = {{&S89976[27], 6}, {NULL}};
const Segment SGI222975[] = {{&S89976[112], 9}, {NULL}};
const Segment SGI222976[] = {{&S89976[85], 6}, {NULL}};
const Segment SGI222977[] = {{&S76018[44], 7}, {NULL}};
const Segment SGI222978[] = {{&S76018[44], 18}, {NULL}};
const Segment SGI222979[] = {{&S76018[30], 6}, {NULL}};
const Segment SGI222980[] = {{&S76018[129], 3}, {NULL}};
const Segment SGI222981[] = {{&S76018[94], 6}, {NULL}};
const Segment SGI222982[] = {{&S76018[64], 8}, {NULL}};
const Segment SGI222983[] = {{&S76019[44], 7}, {NULL}};
const Segment SGI222984[] = {{&S76019[44], 19}, {NULL}};
const Segment SGI222985[] = {{&S76019[30], 6}, {NULL}};
const Segment SGI222986[] = {{&S76019[112], 3}, {NULL}};
const Segment SGI222987[] = {{&S76019[77], 6}, {NULL}};
const Segment SGI222988[] = {{&S76019[137], 8}, {NULL}};
const Segment SGI222989[] = {{&S89983[44], 6}, {NULL}};
const Segment SGI222990[] = {{&S89983[44], 18}, {NULL}};
const Segment SGI222991[] = {{&S89983[30], 6}, {NULL}};
const Segment SGI222992[] = {{&S89983[111], 3}, {NULL}};
const Segment SGI222993[] = {{&S89983[76], 6}, {NULL}};
const Segment SGI222994[] = {{&S89983[136], 7}, {NULL}};
const Segment SGI222995[] = {{&S89984[44], 7}, {NULL}};
const Segment SGI222996[] = {{&S89984[44], 19}, {NULL}};
const Segment SGI222997[] = {{&S89984[30], 6}, {NULL}};
const Segment SGI222998[] = {{&S89984[112], 3}, {NULL}};
const Segment SGI222999[] = {{&S89984[77], 6}, {NULL}};
const Segment SGI223000[] = {{&S89984[137], 7}, {NULL}};
const Segment SGI223001[] = {{&S89993[44], 19}, {NULL}};
const Segment SGI223002[] = {{&S89993[44], 31}, {NULL}};
const Segment SGI223003[] = {{&S89993[30], 6}, {NULL}};
const Segment SGI223004[] = {{&S89993[124], 3}, {NULL}};
const Segment SGI223005[] = {{&S89993[89], 6}, {NULL}};
const Segment SGI223006[] = {{&S89994[44], 19}, {NULL}};
const Segment SGI223007[] = {{&S89994[44], 31}, {NULL}};
const Segment SGI223008[] = {{&S89994[30], 6}, {NULL}};
const Segment SGI223009[] = {{&S89994[124], 3}, {NULL}};
const Segment SGI223010[] = {{&S89994[89], 6}, {NULL}};
const Segment SGI223011[] = {{&S83419[44], 10}, {NULL}};
const Segment SGI223012[] = {{&S83419[44], 22}, {NULL}};
const Segment SGI223013[] = {{&S83419[30], 6}, {NULL}};
const Segment SGI223014[] = {{&S83419[136], 3}, {NULL}};
const Segment SGI223015[] = {{&S83419[101], 6}, {NULL}};
const Segment SGI223016[] = {{&S85094[34], 10}, {NULL}};
const Segment SGI223017[] = {{&S85094[27], 6}, {NULL}};
const Segment SGI223018[] = {{&S85094[97], 9}, {NULL}};
const Segment SGI223019[] = {{&S85094[70], 6}, {NULL}};
const Segment SGI223020[] = {{&S85095[34], 14}, {NULL}};
const Segment SGI223021[] = {{&S85095[27], 6}, {NULL}};
const Segment SGI223022[] = {{&S85095[114], 9}, {NULL}};
const Segment SGI223023[] = {{&S85095[87], 6}, {NULL}};
const Segment SGI223024[] = {{&S76059[34], 6}, {NULL}};
const Segment SGI223025[] = {{&S76059[27], 6}, {NULL}};
const Segment SGI223026[] = {{&S76059[93], 9}, {NULL}};
const Segment SGI223027[] = {{&S76059[66], 6}, {NULL}};
const Segment SGI223028[] = {{&S83436[34], 10}, {NULL}};
const Segment SGI223029[] = {{&S83436[27], 6}, {NULL}};
const Segment SGI223030[] = {{&S83436[97], 9}, {NULL}};
const Segment SGI223031[] = {{&S83436[70], 6}, {NULL}};
const Segment SGI223032[] = {{&S94225[34], 25}, {NULL}};
const Segment SGI223033[] = {{&S94225[27], 6}, {NULL}};
const Segment SGI223034[] = {{&S94225[111], 9}, {NULL}};
const Segment SGI223035[] = {{&S94225[84], 6}, {NULL}};
const Segment SGI223036[] = {{&S76074[34], 15}, {NULL}};
const Segment SGI223037[] = {{&S76074[27], 6}, {NULL}};
const Segment SGI223038[] = {{&S76074[101], 9}, {NULL}};
const Segment SGI223039[] = {{&S76074[74], 6}, {NULL}};
const Segment SGI223040[] = {{&S76080[44], 7}, {NULL}};
const Segment SGI223041[] = {{&S76080[44], 25}, {NULL}};
const Segment SGI223042[] = {{&S76080[30], 6}, {NULL}};
const Segment SGI223043[] = {{&S76080[118], 3}, {NULL}};
const Segment SGI223044[] = {{&S76080[83], 6}, {NULL}};
const Segment SGI223045[] = {{&S76081[44], 7}, {NULL}};
const Segment SGI223046[] = {{&S76081[44], 26}, {NULL}};
const Segment SGI223047[] = {{&S76081[30], 6}, {NULL}};
const Segment SGI223048[] = {{&S76081[119], 3}, {NULL}};
const Segment SGI223049[] = {{&S76081[84], 6}, {NULL}};
const Segment SGI223050[] = {{&S94242[34], 6}, {NULL}};
const Segment SGI223051[] = {{&S94242[27], 6}, {NULL}};
const Segment SGI223052[] = {{&S94242[93], 9}, {NULL}};
const Segment SGI223053[] = {{&S94242[66], 6}, {NULL}};
const Segment SGI223054[] = {{&S76082[44], 7}, {NULL}};
const Segment SGI223055[] = {{&S76082[44], 25}, {NULL}};
const Segment SGI223056[] = {{&S76082[30], 6}, {NULL}};
const Segment SGI223057[] = {{&S76082[118], 3}, {NULL}};
const Segment SGI223058[] = {{&S76082[83], 6}, {NULL}};
const Segment SGI223059[] = {{&S94243[59], 7}, {NULL}};
const Segment SGI223060[] = {{&S94243[59], 24}, {NULL}};
const Segment SGI223061[] = {{&S94243[45], 6}, {NULL}};
const Segment SGI223062[] = {{&S94243[46], 3}, {NULL}};
const Segment SGI223063[] = {{&S76083[38], 7}, {NULL}};
const Segment SGI223064[] = {{&S94243[97], 6}, {NULL}};
const Segment SGI223065[] = {{&S76083[31], 6}, {NULL}};
const Segment SGI223066[] = {{&S94243[0], 14}, {NULL}};
const Segment SGI223067[] = {{&S76084[44], 7}, {NULL}};
const Segment SGI223068[] = {{&S76084[44], 27}, {NULL}};
const Segment SGI223069[] = {{&S76084[30], 6}, {NULL}};
const Segment SGI223070[] = {{&S76084[120], 3}, {NULL}};
const Segment SGI223071[] = {{&S76084[85], 6}, {NULL}};
const Segment SGI223072[] = {{&S94246[34], 6}, {NULL}};
const Segment SGI223073[] = {{&S94246[27], 6}, {NULL}};
const Segment SGI223074[] = {{&S94246[93], 9}, {NULL}};
const Segment SGI223075[] = {{&S94246[66], 6}, {NULL}};
const Segment SGI223076[] = {{&S94248[34], 10}, {NULL}};
const Segment SGI223077[] = {{&S94248[27], 6}, {NULL}};
const Segment SGI223078[] = {{&S94248[97], 9}, {NULL}};
const Segment SGI223079[] = {{&S94248[70], 6}, {NULL}};
const Segment SGI223080[] = {{&S94251[34], 11}, {NULL}};
const Segment SGI223081[] = {{&S94251[27], 6}, {NULL}};
const Segment SGI223082[] = {{&S83473[34], 10}, {NULL}};
const Segment SGI223083[] = {{&S83473[27], 6}, {NULL}};
const Segment SGI223084[] = {{&S94251[98], 9}, {NULL}};
const Segment SGI223085[] = {{&S94251[71], 6}, {NULL}};
const Segment SGI223086[] = {{&S83473[97], 9}, {NULL}};
const Segment SGI223087[] = {{&S83473[70], 6}, {NULL}};
const Segment SGI223088[] = {{&S94255[34], 25}, {NULL}};
const Segment SGI223089[] = {{&S94255[27], 6}, {NULL}};
const Segment SGI223090[] = {{&S94255[112], 9}, {NULL}};
const Segment SGI223091[] = {{&S94255[85], 6}, {NULL}};
const Segment SGI223092[] = {{&S94259[44], 6}, {NULL}};
const Segment SGI223093[] = {{&S94259[44], 18}, {NULL}};
const Segment SGI223094[] = {{&S94259[30], 6}, {NULL}};
const Segment SGI223095[] = {{&S94259[31], 3}, {NULL}};
const Segment SGI223096[] = {{&S94259[97], 6}, {NULL}};
const Segment SGI223097[] = {{&S94259[64], 8}, {NULL}};
const Segment SGI223098[] = {{&S94260[44], 6}, {NULL}};
const Segment SGI223099[] = {{&S94260[44], 18}, {NULL}};
const Segment SGI223100[] = {{&S94260[30], 6}, {NULL}};
const Segment SGI223101[] = {{&S94260[31], 3}, {NULL}};
const Segment SGI223102[] = {{&S94260[97], 6}, {NULL}};
const Segment SGI223103[] = {{&S94260[64], 8}, {NULL}};
const Segment SGI223104[] = {{&S90077[44], 14}, {NULL}};
const Segment SGI223105[] = {{&S90077[44], 26}, {NULL}};
const Segment SGI223106[] = {{&S90077[30], 6}, {NULL}};
const Segment SGI223107[] = {{&S90077[119], 3}, {NULL}};
const Segment SGI223108[] = {{&S90077[84], 6}, {NULL}};
const Segment SGI223109[] = {{&S94265[34], 10}, {NULL}};
const Segment SGI223110[] = {{&S94265[27], 6}, {NULL}};
const Segment SGI223111[] = {{&S94265[97], 9}, {NULL}};
const Segment SGI223112[] = {{&S94265[70], 6}, {NULL}};
const Segment SGI223113[] = {{&S94266[59], 7}, {NULL}};
const Segment SGI223114[] = {{&S94266[59], 24}, {NULL}};
const Segment SGI223115[] = {{&S94266[45], 6}, {NULL}};
const Segment SGI223116[] = {{&S94266[46], 3}, {NULL}};
const Segment SGI223117[] = {{&S94266[97], 6}, {NULL}};
const Segment SGI223118[] = {{&S94266[0], 14}, {NULL}};
const Segment SGI223119[] = {{&S94268[45], 6}, {NULL}};
const Segment SGI223120[] = {{&S94268[30], 6}, {NULL}};
const Segment SGI223121[] = {{&S94268[31], 3}, {NULL}};
const Segment SGI223122[] = {{&S94268[77], 5}, {NULL}};
const Segment SGI223123[] = {{&S94268[135], 13}, {NULL}};
const Segment SGI223124[] = {{&S94269[34], 7}, {NULL}};
const Segment SGI223125[] = {{&S94269[27], 6}, {NULL}};
const Segment SGI223126[] = {{&S94269[90], 9}, {NULL}};
const Segment SGI223127[] = {{&S94269[63], 6}, {NULL}};
const Segment SGI223128[] = {{&S94271[45], 11}, {NULL}};
const Segment SGI223129[] = {{&S94271[30], 6}, {NULL}};
const Segment SGI223130[] = {{&S94271[31], 3}, {NULL}};
const Segment SGI223131[] = {{&S94271[82], 5}, {NULL}};
const Segment SGI223132[] = {{&S94271[140], 13}, {NULL}};
const Segment SGI223133[] = {{&S90095[44], 16}, {NULL}};
const Segment SGI223134[] = {{&S90095[44], 28}, {NULL}};
const Segment SGI223135[] = {{&S90095[30], 6}, {NULL}};
const Segment SGI223136[] = {{&S90095[121], 3}, {NULL}};
const Segment SGI223137[] = {{&S90095[86], 6}, {NULL}};
const Segment SGI223138[] = {{&S94273[34], 7}, {NULL}};
const Segment SGI223139[] = {{&S94273[27], 6}, {NULL}};
const Segment SGI223140[] = {{&S94273[94], 9}, {NULL}};
const Segment SGI223141[] = {{&S94273[67], 6}, {NULL}};
const Segment SGI223142[] = {{&S94278[45], 13}, {NULL}};
const Segment SGI223143[] = {{&S94278[30], 6}, {NULL}};
const Segment SGI223144[] = {{&S94278[31], 3}, {NULL}};
const Segment SGI223145[] = {{&S83514[34], 9}, {NULL}};
const Segment SGI223146[] = {{&S94278[84], 5}, {NULL}};
const Segment SGI223147[] = {{&S83514[27], 6}, {NULL}};
const Segment SGI223148[] = {{&S94278[142], 13}, {NULL}};
const Segment SGI223149[] = {{&S83514[96], 9}, {NULL}};
const Segment SGI223150[] = {{&S83514[69], 6}, {NULL}};
const Segment SGI223151[] = {{&S94279[44], 8}, {NULL}};
const Segment SGI223152[] = {{&S94279[44], 24}, {NULL}};
const Segment SGI223153[] = {{&S94279[30], 6}, {NULL}};
const Segment SGI223154[] = {{&S94279[82], 6}, {NULL}};
const Segment SGI223155[] = {{&S94279[109], 13}, {NULL}};
const Segment SGI223156[] = {{&S83516[34], 10}, {NULL}};
const Segment SGI223157[] = {{&S83516[27], 6}, {NULL}};
const Segment SGI223158[] = {{&S83516[97], 9}, {NULL}};
const Segment SGI223159[] = {{&S83516[70], 6}, {NULL}};
const Segment SGI223160[] = {{&S83518[34], 10}, {NULL}};
const Segment SGI223161[] = {{&S83518[27], 6}, {NULL}};
const Segment SGI223162[] = {{&S83518[97], 9}, {NULL}};
const Segment SGI223163[] = {{&S94283[44], 5}, {NULL}};
const Segment SGI223164[] = {{&S83518[70], 6}, {NULL}};
const Segment SGI223165[] = {{&S94283[44], 17}, {NULL}};
const Segment SGI223166[] = {{&S94283[30], 6}, {NULL}};
const Segment SGI223167[] = {{&S94283[31], 3}, {NULL}};
const Segment SGI223168[] = {{&S94283[75], 6}, {NULL}};
const Segment SGI223169[] = {{&S94285[34], 11}, {NULL}};
const Segment SGI223170[] = {{&S94285[27], 6}, {NULL}};
const Segment SGI223171[] = {{&S94285[98], 9}, {NULL}};
const Segment SGI223172[] = {{&S94285[71], 6}, {NULL}};
const Segment SGI223173[] = {{&S94286[34], 10}, {NULL}};
const Segment SGI223174[] = {{&S94286[27], 6}, {NULL}};
const Segment SGI223175[] = {{&S94286[97], 9}, {NULL}};
const Segment SGI223176[] = {{&S94286[70], 6}, {NULL}};
const Segment SGI223177[] = {{&S94287[34], 20}, {NULL}};
const Segment SGI223178[] = {{&S94287[27], 6}, {NULL}};
const Segment SGI223179[] = {{&S94287[107], 9}, {NULL}};
const Segment SGI223180[] = {{&S94287[80], 6}, {NULL}};
const Segment SGI223181[] = {{&S83527[34], 11}, {NULL}};
const Segment SGI223182[] = {{&S90118[44], 4}, {NULL}};
const Segment SGI223183[] = {{&S83527[27], 6}, {NULL}};
const Segment SGI223184[] = {{&S90118[44], 16}, {NULL}};
const Segment SGI223185[] = {{&S90118[30], 6}, {NULL}};
const Segment SGI223186[] = {{&S94289[34], 6}, {NULL}};
const Segment SGI223187[] = {{&S90118[109], 3}, {NULL}};
const Segment SGI223188[] = {{&S83527[98], 9}, {NULL}};
const Segment SGI223189[] = {{&S94289[27], 6}, {NULL}};
const Segment SGI223190[] = {{&S83527[71], 6}, {NULL}};
const Segment SGI223191[] = {{&S90118[74], 6}, {NULL}};
const Segment SGI223192[] = {{&S94289[93], 9}, {NULL}};
const Segment SGI223193[] = {{&S94289[66], 6}, {NULL}};
const Segment SGI223194[] = {{&S90120[45], 13}, {NULL}};
const Segment SGI223195[] = {{&S83529[34], 10}, {NULL}};
const Segment SGI223196[] = {{&S90120[30], 6}, {NULL}};
const Segment SGI223197[] = {{&S94291[34], 11}, {NULL}};
const Segment SGI223198[] = {{&S83529[27], 6}, {NULL}};
const Segment SGI223199[] = {{&S94291[27], 6}, {NULL}};
const Segment SGI223200[] = {{&S90120[118], 3}, {NULL}};
const Segment SGI223201[] = {{&S83529[97], 9}, {NULL}};
const Segment SGI223202[] = {{&S83529[70], 6}, {NULL}};
const Segment SGI223203[] = {{&S90120[84], 5}, {NULL}};
const Segment SGI223204[] = {{&S94291[98], 9}, {NULL}};
const Segment SGI223205[] = {{&S94291[71], 6}, {NULL}};
const Segment SGI223206[] = {{&S90120[142], 13}, {NULL}};
const Segment SGI223207[] = {{&S94292[44], 15}, {NULL}};
const Segment SGI223208[] = {{&S94292[44], 27}, {NULL}};
const Segment SGI223209[] = {{&S94292[30], 6}, {NULL}};
const Segment SGI223210[] = {{&S94292[31], 3}, {NULL}};
const Segment SGI223211[] = {{&S94292[85], 6}, {NULL}};
const Segment SGI223212[] = {{&S90123[34], 11}, {NULL}};
const Segment SGI223213[] = {{&S90123[27], 6}, {NULL}};
const Segment SGI223214[] = {{&S83533[59], 11}, {NULL}};
const Segment SGI223215[] = {{&S83533[59], 23}, {NULL}};
const Segment SGI223216[] = {{&S83533[45], 6}, {NULL}};
const Segment SGI223217[] = {{&S90123[98], 9}, {NULL}};
const Segment SGI223218[] = {{&S90123[71], 6}, {NULL}};
const Segment SGI223219[] = {{&S83533[131], 3}, {NULL}};
const Segment SGI223220[] = {{&S90123[139], 8}, {NULL}};
const Segment SGI223221[] = {{&S83533[96], 6}, {NULL}};
const Segment SGI223222[] = {{&S83533[0], 14}, {NULL}};
const Segment SGI223223[] = {{&S90124[34], 11}, {NULL}};
const Segment SGI223224[] = {{&S90124[27], 6}, {NULL}};
const Segment SGI223225[] = {{&S85218[44], 9}, {NULL}};
const Segment SGI223226[] = {{&S85218[44], 21}, {NULL}};
const Segment SGI223227[] = {{&S85218[30], 6}, {NULL}};
const Segment SGI223228[] = {{&S90124[98], 9}, {NULL}};
const Segment SGI223229[] = {{&S90124[71], 6}, {NULL}};
const Segment SGI223230[] = {{&S85218[114], 3}, {NULL}};
const Segment SGI223231[] = {{&S85218[79], 6}, {NULL}};
const Segment SGI223232[] = {{&S90124[132], 8}, {NULL}};
const Segment SGI223233[] = {{&S94297[34], 9}, {NULL}};
const Segment SGI223234[] = {{&S94297[27], 6}, {NULL}};
const Segment SGI223235[] = {{&S94297[96], 9}, {NULL}};
const Segment SGI223236[] = {{&S94297[69], 6}, {NULL}};
const Segment SGI223237[] = {{&S90127[44], 7}, {NULL}};
const Segment SGI223238[] = {{&S90127[44], 35}, {NULL}};
const Segment SGI223239[] = {{&S90127[30], 6}, {NULL}};
const Segment SGI223240[] = {{&S90127[128], 3}, {NULL}};
const Segment SGI223241[] = {{&S90127[93], 6}, {NULL}};
const Segment SGI223242[] = {{&S90127[153], 7}, {NULL}};
const Segment SGI223243[] = {{&S94301[34], 20}, {NULL}};
const Segment SGI223244[] = {{&S94301[27], 6}, {NULL}};
const Segment SGI223245[] = {{&S85224[44], 10}, {NULL}};
const Segment SGI223246[] = {{&S85224[44], 22}, {NULL}};
const Segment SGI223247[] = {{&S85224[30], 6}, {NULL}};
const Segment SGI223248[] = {{&S94301[107], 9}, {NULL}};
const Segment SGI223249[] = {{&S94301[80], 6}, {NULL}};
const Segment SGI223250[] = {{&S85224[115], 3}, {NULL}};
const Segment SGI223251[] = {{&S83542[34], 7}, {NULL}};
const Segment SGI223252[] = {{&S85224[80], 6}, {NULL}};
const Segment SGI223253[] = {{&S83542[27], 6}, {NULL}};
const Segment SGI223254[] = {{&S83542[99], 9}, {NULL}};
const Segment SGI223255[] = {{&S83542[72], 6}, {NULL}};
const Segment SGI223256[] = {{&S83542[140], 8}, {NULL}};
const Segment SGI223257[] = {{&S94303[34], 7}, {NULL}};
const Segment SGI223258[] = {{&S94303[27], 6}, {NULL}};
const Segment SGI223259[] = {{&S94303[90], 9}, {NULL}};
const Segment SGI223260[] = {{&S94303[63], 6}, {NULL}};
const Segment SGI223261[] = {{&S83546[34], 9}, {NULL}};
const Segment SGI223262[] = {{&S83546[27], 6}, {NULL}};
const Segment SGI223263[] = {{&S94305[34], 9}, {NULL}};
const Segment SGI223264[] = {{&S94305[27], 6}, {NULL}};
const Segment SGI223265[] = {{&S83546[96], 9}, {NULL}};
const Segment SGI223266[] = {{&S83546[69], 6}, {NULL}};
const Segment SGI223267[] = {{&S94305[96], 9}, {NULL}};
const Segment SGI223268[] = {{&S94305[69], 6}, {NULL}};
const Segment SGI223269[] = {{&S83549[34], 10}, {NULL}};
const Segment SGI223270[] = {{&S83549[27], 6}, {NULL}};
const Segment SGI223271[] = {{&S83549[97], 9}, {NULL}};
const Segment SGI223272[] = {{&S83549[70], 6}, {NULL}};
const Segment SGI223273[] = {{&S94309[34], 24}, {NULL}};
const Segment SGI223274[] = {{&S94309[27], 6}, {NULL}};
const Segment SGI223275[] = {{&S94309[111], 9}, {NULL}};
const Segment SGI223276[] = {{&S94309[84], 6}, {NULL}};
const Segment SGI223277[] = {{&S94309[152], 8}, {NULL}};
const Segment SGI223278[] = {{&S90145[44], 13}, {NULL}};
const Segment SGI223279[] = {{&S90145[44], 25}, {NULL}};
const Segment SGI223280[] = {{&S90145[30], 6}, {NULL}};
const Segment SGI223281[] = {{&S90145[83], 6}, {NULL}};
const Segment SGI223282[] = {{&S90145[110], 13}, {NULL}};
const Segment SGI223283[] = {{&S83556[44], 10}, {NULL}};
const Segment SGI223284[] = {{&S83556[44], 22}, {NULL}};
const Segment SGI223285[] = {{&S83556[30], 6}, {NULL}};
const Segment SGI223286[] = {{&S83556[80], 6}, {NULL}};
const Segment SGI223287[] = {{&S83556[107], 13}, {NULL}};
const Segment SGI223288[] = {{&S94317[44], 11}, {NULL}};
const Segment SGI223289[] = {{&S94317[44], 23}, {NULL}};
const Segment SGI223290[] = {{&S94317[30], 6}, {NULL}};
const Segment SGI223291[] = {{&S94317[31], 3}, {NULL}};
const Segment SGI223292[] = {{&S94317[81], 6}, {NULL}};
const Segment SGI223293[] = {{&S83560[34], 10}, {NULL}};
const Segment SGI223294[] = {{&S83560[27], 6}, {NULL}};
const Segment SGI223295[] = {{&S83560[97], 9}, {NULL}};
const Segment SGI223296[] = {{&S83560[70], 6}, {NULL}};
const Segment SGI223297[] = {{&S94322[34], 15}, {NULL}};
const Segment SGI223298[] = {{&S94322[27], 6}, {NULL}};
const Segment SGI223299[] = {{&S94322[109], 9}, {NULL}};
const Segment SGI223300[] = {{&S94322[82], 6}, {NULL}};
const Segment SGI223301[] = {{&S94323[34], 4}, {NULL}};
const Segment SGI223302[] = {{&S94323[27], 6}, {NULL}};
const Segment SGI223303[] = {{&S94323[91], 9}, {NULL}};
const Segment SGI223304[] = {{&S94323[64], 6}, {NULL}};
const Segment SGI223305[] = {{&S94324[34], 8}, {NULL}};
const Segment SGI223306[] = {{&S94324[27], 6}, {NULL}};
const Segment SGI223307[] = {{&S94324[95], 9}, {NULL}};
const Segment SGI223308[] = {{&S94324[68], 6}, {NULL}};
const Segment SGI223309[] = {{&S90165[44], 6}, {NULL}};
const Segment SGI223310[] = {{&S90165[44], 18}, {NULL}};
const Segment SGI223311[] = {{&S90165[30], 6}, {NULL}};
const Segment SGI223312[] = {{&S94329[44], 10}, {NULL}};
const Segment SGI223313[] = {{&S94329[44], 22}, {NULL}};
const Segment SGI223314[] = {{&S94329[30], 6}, {NULL}};
const Segment SGI223315[] = {{&S94329[31], 3}, {NULL}};
const Segment SGI223316[] = {{&S90165[76], 6}, {NULL}};
const Segment SGI223317[] = {{&S94329[80], 6}, {NULL}};
const Segment SGI223318[] = {{&S90165[103], 13}, {NULL}};
const Segment SGI223319[] = {{&S90166[34], 10}, {NULL}};
const Segment SGI223320[] = {{&S90166[27], 6}, {NULL}};
const Segment SGI223321[] = {{&S90166[97], 9}, {NULL}};
const Segment SGI223322[] = {{&S90166[70], 6}, {NULL}};
const Segment SGI223323[] = {{&S90168[34], 11}, {NULL}};
const Segment SGI223324[] = {{&S90168[27], 6}, {NULL}};
const Segment SGI223325[] = {{&S90168[98], 9}, {NULL}};
const Segment SGI223326[] = {{&S90168[71], 6}, {NULL}};
const Segment SGI223327[] = {{&S90169[34], 11}, {NULL}};
const Segment SGI223328[] = {{&S90169[27], 6}, {NULL}};
const Segment SGI223329[] = {{&S90169[98], 9}, {NULL}};
const Segment SGI223330[] = {{&S90169[71], 6}, {NULL}};
const Segment SGI223331[] = {{&S94336[44], 10}, {NULL}};
const Segment SGI223332[] = {{&S94336[44], 22}, {NULL}};
const Segment SGI223333[] = {{&S94336[30], 6}, {NULL}};
const Segment SGI223334[] = {{&S94336[80], 6}, {NULL}};
const Segment SGI223335[] = {{&S94336[107], 13}, {NULL}};
const Segment SGI223336[] = {{&S94339[44], 4}, {NULL}};
const Segment SGI223337[] = {{&S94339[44], 16}, {NULL}};
const Segment SGI223338[] = {{&S94339[30], 6}, {NULL}};
const Segment SGI223339[] = {{&S94339[74], 6}, {NULL}};
const Segment SGI223340[] = {{&S94339[101], 13}, {NULL}};
const Segment SGI223341[] = {{&S94340[59], 10}, {NULL}};
const Segment SGI223342[] = {{&S94340[59], 22}, {NULL}};
const Segment SGI223343[] = {{&S94340[45], 6}, {NULL}};
const Segment SGI223344[] = {{&S94340[46], 3}, {NULL}};
const Segment SGI223345[] = {{&S94340[95], 6}, {NULL}};
const Segment SGI223346[] = {{&S94340[0], 14}, {NULL}};
const Segment SGI223347[] = {{&S94342[45], 5}, {NULL}};
const Segment SGI223348[] = {{&S94342[30], 6}, {NULL}};
const Segment SGI223349[] = {{&S94342[31], 3}, {NULL}};
const Segment SGI223350[] = {{&S94342[76], 5}, {NULL}};
const Segment SGI223351[] = {{&S94342[134], 13}, {NULL}};
const Segment SGI223352[] = {{&S94345[44], 5}, {NULL}};
const Segment SGI223353[] = {{&S94345[44], 17}, {NULL}};
const Segment SGI223354[] = {{&S94345[30], 6}, {NULL}};
const Segment SGI223355[] = {{&S94345[75], 6}, {NULL}};
const Segment SGI223356[] = {{&S94345[102], 13}, {NULL}};
const Segment SGI223357[] = {{&S94350[52], 10}, {NULL}};
const Segment SGI223358[] = {{&S94350[52], 22}, {NULL}};
const Segment SGI223359[] = {{&S94350[38], 6}, {NULL}};
const Segment SGI223360[] = {{&S94350[88], 5}, {NULL}};
const Segment SGI223361[] = {{&S94350[0], 14}, {NULL}};
const Segment SGI223362[] = {{&S76250[44], 11}, {NULL}};
const Segment SGI223363[] = {{&S76250[44], 22}, {NULL}};
const Segment SGI223364[] = {{&S76250[30], 6}, {NULL}};
const Segment SGI223365[] = {{&S76250[115], 3}, {NULL}};
const Segment SGI223366[] = {{&S76250[80], 6}, {NULL}};
const Segment SGI223367[] = {{&S94355[34], 11}, {NULL}};
const Segment SGI223368[] = {{&S94355[27], 6}, {NULL}};
const Segment SGI223369[] = {{&S94355[107], 9}, {NULL}};
const Segment SGI223370[] = {{&S94355[80], 6}, {NULL}};
const Segment SGI223371[] = {{&S83614[34], 25}, {NULL}};
const Segment SGI223372[] = {{&S94358[34], 8}, {NULL}};
const Segment SGI223373[] = {{&S83614[27], 6}, {NULL}};
const Segment SGI223374[] = {{&S94358[27], 6}, {NULL}};
const Segment SGI223375[] = {{&S94358[95], 9}, {NULL}};
const Segment SGI223376[] = {{&S83614[112], 9}, {NULL}};
const Segment SGI223377[] = {{&S94358[68], 6}, {NULL}};
const Segment SGI223378[] = {{&S83614[85], 6}, {NULL}};
const Segment SGI223379[] = {{&S83617[44], 7}, {NULL}};
const Segment SGI223380[] = {{&S83617[44], 27}, {NULL}};
const Segment SGI223381[] = {{&S83617[30], 6}, {NULL}};
const Segment SGI223382[] = {{&S83617[120], 3}, {NULL}};
const Segment SGI223383[] = {{&S83617[85], 6}, {NULL}};
const Segment SGI223384[] = {{&S85308[44], 16}, {NULL}};
const Segment SGI223385[] = {{&S85308[44], 28}, {NULL}};
const Segment SGI223386[] = {{&S85308[30], 6}, {NULL}};
const Segment SGI223387[] = {{&S83618[44], 7}, {NULL}};
const Segment SGI223388[] = {{&S83618[44], 27}, {NULL}};
const Segment SGI223389[] = {{&S85308[86], 6}, {NULL}};
const Segment SGI223390[] = {{&S83618[30], 6}, {NULL}};
const Segment SGI223391[] = {{&S85308[123], 3}, {NULL}};
const Segment SGI223392[] = {{&S83618[120], 3}, {NULL}};
const Segment SGI223393[] = {{&S83618[85], 6}, {NULL}};
const Segment SGI223394[] = {{&S83619[44], 7}, {NULL}};
const Segment SGI223395[] = {{&S83619[44], 27}, {NULL}};
const Segment SGI223396[] = {{&S83619[30], 6}, {NULL}};
const Segment SGI223397[] = {{&S85310[44], 11}, {NULL}};
const Segment SGI223398[] = {{&S83619[120], 3}, {NULL}};
const Segment SGI223399[] = {{&S85310[44], 29}, {NULL}};
const Segment SGI223400[] = {{&S85310[30], 6}, {NULL}};
const Segment SGI223401[] = {{&S83619[85], 6}, {NULL}};
const Segment SGI223402[] = {{&S85310[122], 3}, {NULL}};
const Segment SGI223403[] = {{&S94367[44], 21}, {NULL}};
const Segment SGI223404[] = {{&S85310[87], 6}, {NULL}};
const Segment SGI223405[] = {{&S94367[44], 33}, {NULL}};
const Segment SGI223406[] = {{&S94367[30], 6}, {NULL}};
const Segment SGI223407[] = {{&S94367[31], 3}, {NULL}};
const Segment SGI223408[] = {{&S94367[91], 6}, {NULL}};
const Segment SGI223409[] = {{&S83620[44], 7}, {NULL}};
const Segment SGI223410[] = {{&S83620[44], 27}, {NULL}};
const Segment SGI223411[] = {{&S83620[30], 6}, {NULL}};
const Segment SGI223412[] = {{&S83620[120], 3}, {NULL}};
const Segment SGI223413[] = {{&S83620[85], 6}, {NULL}};
const Segment SGI223414[] = {{&S83621[44], 7}, {NULL}};
const Segment SGI223415[] = {{&S83621[44], 26}, {NULL}};
const Segment SGI223416[] = {{&S83621[30], 6}, {NULL}};
const Segment SGI223417[] = {{&S83621[119], 3}, {NULL}};
const Segment SGI223418[] = {{&S83621[84], 6}, {NULL}};
const Segment SGI223419[] = {{&S83622[44], 7}, {NULL}};
const Segment SGI223420[] = {{&S83622[44], 26}, {NULL}};
const Segment SGI223421[] = {{&S83622[30], 6}, {NULL}};
const Segment SGI223422[] = {{&S83622[119], 3}, {NULL}};
const Segment SGI223423[] = {{&S83622[84], 6}, {NULL}};
const Segment SGI223424[] = {{&S94372[44], 14}, {NULL}};
const Segment SGI223425[] = {{&S94372[44], 31}, {NULL}};
const Segment SGI223426[] = {{&S94372[30], 6}, {NULL}};
const Segment SGI223427[] = {{&S94372[31], 3}, {NULL}};
const Segment SGI223428[] = {{&S94372[89], 5}, {NULL}};
const Segment SGI223429[] = {{&S94372[126], 14}, {NULL}};
const Segment SGI223430[] = {{&S83623[44], 7}, {NULL}};
const Segment SGI223431[] = {{&S83623[44], 27}, {NULL}};
const Segment SGI223432[] = {{&S83623[30], 6}, {NULL}};
const Segment SGI223433[] = {{&S83623[120], 3}, {NULL}};
const Segment SGI223434[] = {{&S83623[85], 6}, {NULL}};
const Segment SGI223435[] = {{&S83624[44], 7}, {NULL}};
const Segment SGI223436[] = {{&S83624[44], 27}, {NULL}};
const Segment SGI223437[] = {{&S83624[30], 6}, {NULL}};
const Segment SGI223438[] = {{&S83624[120], 3}, {NULL}};
const Segment SGI223439[] = {{&S83624[85], 6}, {NULL}};
const Segment SGI223440[] = {{&S83625[44], 7}, {NULL}};
const Segment SGI223441[] = {{&S83625[44], 33}, {NULL}};
const Segment SGI223442[] = {{&S83625[30], 6}, {NULL}};
const Segment SGI223443[] = {{&S83625[126], 3}, {NULL}};
const Segment SGI223444[] = {{&S83625[91], 6}, {NULL}};
const Segment SGI223445[] = {{&S94378[34], 10}, {NULL}};
const Segment SGI223446[] = {{&S94378[27], 6}, {NULL}};
const Segment SGI223447[] = {{&S94378[97], 9}, {NULL}};
const Segment SGI223448[] = {{&S94378[70], 6}, {NULL}};
const Segment SGI223449[] = {{&S83626[38], 7}, {NULL}};
const Segment SGI223450[] = {{&S94378[138], 8}, {NULL}};
const Segment SGI223451[] = {{&S83626[31], 6}, {NULL}};
const Segment SGI223452[] = {{&S83628[34], 8}, {NULL}};
const Segment SGI223453[] = {{&S83628[27], 6}, {NULL}};
const Segment SGI223454[] = {{&S83628[103], 9}, {NULL}};
const Segment SGI223455[] = {{&S83628[76], 6}, {NULL}};
const Segment SGI223456[] = {{&S94384[34], 8}, {NULL}};
const Segment SGI223457[] = {{&S94384[27], 6}, {NULL}};
const Segment SGI223458[] = {{&S94384[95], 9}, {NULL}};
const Segment SGI223459[] = {{&S94384[68], 6}, {NULL}};
const Segment SGI223460[] = {{&S83634[44], 10}, {NULL}};
const Segment SGI223461[] = {{&S83634[44], 22}, {NULL}};
const Segment SGI223462[] = {{&S83634[30], 6}, {NULL}};
const Segment SGI223463[] = {{&S83634[113], 3}, {NULL}};
const Segment SGI223464[] = {{&S83634[80], 5}, {NULL}};
const Segment SGI223465[] = {{&S83634[117], 14}, {NULL}};
const Segment SGI223466[] = {{&S83635[44], 11}, {NULL}};
const Segment SGI223467[] = {{&S83635[44], 23}, {NULL}};
const Segment SGI223468[] = {{&S83635[30], 6}, {NULL}};
const Segment SGI223469[] = {{&S83635[114], 3}, {NULL}};
const Segment SGI223470[] = {{&S83635[81], 5}, {NULL}};
const Segment SGI223471[] = {{&S83635[118], 14}, {NULL}};
const Segment SGI223472[] = {{&S83636[34], 9}, {NULL}};
const Segment SGI223473[] = {{&S83636[27], 6}, {NULL}};
const Segment SGI223474[] = {{&S83636[96], 9}, {NULL}};
const Segment SGI223475[] = {{&S83636[69], 6}, {NULL}};
const Segment SGI223476[] = {{&S76298[34], 9}, {NULL}};
const Segment SGI223477[] = {{&S76298[27], 6}, {NULL}};
const Segment SGI223478[] = {{&S76298[95], 9}, {NULL}};
const Segment SGI223479[] = {{&S76298[68], 6}, {NULL}};
const Segment SGI223480[] = {{&S76306[34], 10}, {NULL}};
const Segment SGI223481[] = {{&S76306[27], 6}, {NULL}};
const Segment SGI223482[] = {{&S76306[97], 9}, {NULL}};
const Segment SGI223483[] = {{&S76306[70], 6}, {NULL}};
const Segment SGI223484[] = {{&S76307[34], 10}, {NULL}};
const Segment SGI223485[] = {{&S76307[27], 6}, {NULL}};
const Segment SGI223486[] = {{&S76307[97], 9}, {NULL}};
const Segment SGI223487[] = {{&S76307[70], 6}, {NULL}};
const Segment SGI223488[] = {{&S76308[34], 12}, {NULL}};
const Segment SGI223489[] = {{&S76308[27], 6}, {NULL}};
const Segment SGI223490[] = {{&S76308[99], 9}, {NULL}};
const Segment SGI223491[] = {{&S76308[72], 6}, {NULL}};
const Segment SGI223492[] = {{&S90262[34], 7}, {NULL}};
const Segment SGI223493[] = {{&S90262[27], 6}, {NULL}};
const Segment SGI223494[] = {{&S90262[94], 9}, {NULL}};
const Segment SGI223495[] = {{&S90262[67], 6}, {NULL}};
const Segment SGI223496[] = {{&S90264[34], 10}, {NULL}};
const Segment SGI223497[] = {{&S90264[27], 6}, {NULL}};
const Segment SGI223498[] = {{&S90264[97], 9}, {NULL}};
const Segment SGI223499[] = {{&S90264[70], 6}, {NULL}};
const Segment SGI223500[] = {{&S94430[38], 9}, {NULL}};
const Segment SGI223501[] = {{&S85374[44], 10}, {NULL}};
const Segment SGI223502[] = {{&S94430[31], 6}, {NULL}};
const Segment SGI223503[] = {{&S85374[44], 22}, {NULL}};
const Segment SGI223504[] = {{&S85374[30], 6}, {NULL}};
const Segment SGI223505[] = {{&S85374[133], 3}, {NULL}};
const Segment SGI223506[] = {{&S85374[98], 6}, {NULL}};
const Segment SGI223507[] = {{&S85374[68], 8}, {NULL}};
const Segment SGI223508[] = {{&S90280[34], 10}, {NULL}};
const Segment SGI223509[] = {{&S90280[27], 6}, {NULL}};
const Segment SGI223510[] = {{&S90280[97], 9}, {NULL}};
const Segment SGI223511[] = {{&S90280[70], 6}, {NULL}};
const Segment SGI223512[] = {{&S90282[34], 7}, {NULL}};
const Segment SGI223513[] = {{&S90282[27], 6}, {NULL}};
const Segment SGI223514[] = {{&S90282[94], 9}, {NULL}};
const Segment SGI223515[] = {{&S90282[67], 6}, {NULL}};
const Segment SGI223516[] = {{&S85396[34], 17}, {NULL}};
const Segment SGI223517[] = {{&S85396[27], 6}, {NULL}};
const Segment SGI223518[] = {{&S90290[44], 11}, {NULL}};
const Segment SGI223519[] = {{&S90290[44], 23}, {NULL}};
const Segment SGI223520[] = {{&S90290[30], 6}, {NULL}};
const Segment SGI223521[] = {{&S85396[104], 9}, {NULL}};
const Segment SGI223522[] = {{&S85396[77], 6}, {NULL}};
const Segment SGI223523[] = {{&S90290[116], 3}, {NULL}};
const Segment SGI223524[] = {{&S90290[81], 6}, {NULL}};
const Segment SGI223525[] = {{&S90293[34], 20}, {NULL}};
const Segment SGI223526[] = {{&S90293[27], 6}, {NULL}};
const Segment SGI223527[] = {{&S90293[107], 9}, {NULL}};
const Segment SGI223528[] = {{&S94459[45], 11}, {NULL}};
const Segment SGI223529[] = {{&S90293[80], 6}, {NULL}};
const Segment SGI223530[] = {{&S94459[30], 6}, {NULL}};
const Segment SGI223531[] = {{&S94459[31], 3}, {NULL}};
const Segment SGI223532[] = {{&S94459[82], 5}, {NULL}};
const Segment SGI223533[] = {{&S94459[140], 13}, {NULL}};
const Segment SGI223534[] = {{&S83645[34], 8}, {NULL}};
const Segment SGI223535[] = {{&S83645[27], 6}, {NULL}};
const Segment SGI223536[] = {{&S83645[100], 9}, {NULL}};
const Segment SGI223537[] = {{&S83645[73], 6}, {NULL}};
const Segment SGI223538[] = {{&S90297[34], 9}, {NULL}};
const Segment SGI223539[] = {{&S90297[27], 6}, {NULL}};
const Segment SGI223540[] = {{&S90297[96], 9}, {NULL}};
const Segment SGI223541[] = {{&S90297[69], 6}, {NULL}};
const Segment SGI223542[] = {{&S94465[34], 14}, {NULL}};
const Segment SGI223543[] = {{&S94465[27], 6}, {NULL}};
const Segment SGI223544[] = {{&S94465[101], 9}, {NULL}};
const Segment SGI223545[] = {{&S94465[74], 6}, {NULL}};
const Segment SGI223546[] = {{&S94466[34], 13}, {NULL}};
const Segment SGI223547[] = {{&S94466[27], 6}, {NULL}};
const Segment SGI223548[] = {{&S94466[100], 9}, {NULL}};
const Segment SGI223549[] = {{&S94466[73], 6}, {NULL}};
const Segment SGI223550[] = {{&S90304[34], 14}, {NULL}};
const Segment SGI223551[] = {{&S90304[27], 6}, {NULL}};
const Segment SGI223552[] = {{&S90304[101], 9}, {NULL}};
const Segment SGI223553[] = {{&S90304[74], 6}, {NULL}};
const Segment SGI223554[] = {{&S94469[34], 10}, {NULL}};
const Segment SGI223555[] = {{&S94469[27], 6}, {NULL}};
const Segment SGI223556[] = {{&S94469[96], 9}, {NULL}};
const Segment SGI223557[] = {{&S94469[69], 6}, {NULL}};
const Segment SGI223558[] = {{&S94471[34], 9}, {NULL}};
const Segment SGI223559[] = {{&S94471[27], 6}, {NULL}};
const Segment SGI223560[] = {{&S94471[96], 9}, {NULL}};
const Segment SGI223561[] = {{&S94471[69], 6}, {NULL}};
const Segment SGI223562[] = {{&S90309[34], 9}, {NULL}};
const Segment SGI223563[] = {{&S90309[27], 6}, {NULL}};
const Segment SGI223564[] = {{&S90309[96], 9}, {NULL}};
const Segment SGI223565[] = {{&S90309[69], 6}, {NULL}};
const Segment SGI223566[] = {{&S90309[130], 8}, {NULL}};
const Segment SGI223567[] = {{&S83663[34], 7}, {NULL}};
const Segment SGI223568[] = {{&S83663[27], 6}, {NULL}};
const Segment SGI223569[] = {{&S83663[94], 9}, {NULL}};
const Segment SGI223570[] = {{&S83663[67], 6}, {NULL}};
const Segment SGI223571[] = {{&S90310[34], 7}, {NULL}};
const Segment SGI223572[] = {{&S90310[27], 6}, {NULL}};
const Segment SGI223573[] = {{&S90310[94], 9}, {NULL}};
const Segment SGI223574[] = {{&S90310[67], 6}, {NULL}};
const Segment SGI223575[] = {{&S90310[128], 8}, {NULL}};
const Segment SGI223576[] = {{&S94476[34], 7}, {NULL}};
const Segment SGI223577[] = {{&S94476[27], 6}, {NULL}};
const Segment SGI223578[] = {{&S94476[94], 9}, {NULL}};
const Segment SGI223579[] = {{&S94476[67], 6}, {NULL}};
const Segment SGI223580[] = {{&S94477[34], 10}, {NULL}};
const Segment SGI223581[] = {{&S94477[27], 6}, {NULL}};
const Segment SGI223582[] = {{&S94477[95], 9}, {NULL}};
const Segment SGI223583[] = {{&S94477[68], 6}, {NULL}};
const Segment SGI223584[] = {{&S94478[34], 10}, {NULL}};
const Segment SGI223585[] = {{&S94478[27], 6}, {NULL}};
const Segment SGI223586[] = {{&S94478[97], 9}, {NULL}};
const Segment SGI223587[] = {{&S94478[70], 6}, {NULL}};
const Segment SGI223588[] = {{&S94479[34], 15}, {NULL}};
const Segment SGI223589[] = {{&S94479[27], 6}, {NULL}};
const Segment SGI223590[] = {{&S94479[103], 9}, {NULL}};
const Segment SGI223591[] = {{&S94479[76], 6}, {NULL}};
const Segment SGI223592[] = {{&S94480[34], 12}, {NULL}};
const Segment SGI223593[] = {{&S94480[27], 6}, {NULL}};
const Segment SGI223594[] = {{&S94480[99], 9}, {NULL}};
const Segment SGI223595[] = {{&S94480[72], 6}, {NULL}};
const Segment SGI223596[] = {{&S90328[34], 16}, {NULL}};
const Segment SGI223597[] = {{&S90328[27], 6}, {NULL}};
const Segment SGI223598[] = {{&S90328[103], 9}, {NULL}};
const Segment SGI223599[] = {{&S90328[76], 6}, {NULL}};
const Segment SGI223600[] = {{&S83689[44], 13}, {NULL}};
const Segment SGI223601[] = {{&S83689[44], 25}, {NULL}};
const Segment SGI223602[] = {{&S83689[30], 6}, {NULL}};
const Segment SGI223603[] = {{&S83689[134], 3}, {NULL}};
const Segment SGI223604[] = {{&S83689[99], 6}, {NULL}};
const Segment SGI223605[] = {{&S83689[71], 8}, {NULL}};
const Segment SGI223606[] = {{&S94496[44], 10}, {NULL}};
const Segment SGI223607[] = {{&S94496[44], 22}, {NULL}};
const Segment SGI223608[] = {{&S94496[30], 6}, {NULL}};
const Segment SGI223609[] = {{&S94496[31], 3}, {NULL}};
const Segment SGI223610[] = {{&S94496[80], 6}, {NULL}};
const Segment SGI223611[] = {{&S90339[34], 10}, {NULL}};
const Segment SGI223612[] = {{&S90339[27], 6}, {NULL}};
const Segment SGI223613[] = {{&S90339[97], 9}, {NULL}};
const Segment SGI223614[] = {{&S90339[70], 6}, {NULL}};
const Segment SGI223615[] = {{&S90340[34], 16}, {NULL}};
const Segment SGI223616[] = {{&S90340[27], 6}, {NULL}};
const Segment SGI223617[] = {{&S90340[103], 9}, {NULL}};
const Segment SGI223618[] = {{&S94501[34], 7}, {NULL}};
const Segment SGI223619[] = {{&S90340[76], 6}, {NULL}};
const Segment SGI223620[] = {{&S94501[27], 6}, {NULL}};
const Segment SGI223621[] = {{&S94501[94], 9}, {NULL}};
const Segment SGI223622[] = {{&S94501[67], 6}, {NULL}};
const Segment SGI223623[] = {{&S94502[34], 10}, {NULL}};
const Segment SGI223624[] = {{&S94502[27], 6}, {NULL}};
const Segment SGI223625[] = {{&S94502[97], 9}, {NULL}};
const Segment SGI223626[] = {{&S94502[70], 6}, {NULL}};
const Segment SGI223627[] = {{&S94503[34], 20}, {NULL}};
const Segment SGI223628[] = {{&S94503[27], 6}, {NULL}};
const Segment SGI223629[] = {{&S94503[107], 9}, {NULL}};
const Segment SGI223630[] = {{&S94503[80], 6}, {NULL}};
const Segment SGI223631[] = {{&S94504[34], 18}, {NULL}};
const Segment SGI223632[] = {{&S94504[27], 6}, {NULL}};
const Segment SGI223633[] = {{&S94504[105], 9}, {NULL}};
const Segment SGI223634[] = {{&S94504[78], 6}, {NULL}};
const Segment SGI223635[] = {{&S94505[34], 11}, {NULL}};
const Segment SGI223636[] = {{&S94505[27], 6}, {NULL}};
const Segment SGI223637[] = {{&S83708[34], 11}, {NULL}};
const Segment SGI223638[] = {{&S94505[98], 9}, {NULL}};
const Segment SGI223639[] = {{&S94505[71], 6}, {NULL}};
const Segment SGI223640[] = {{&S83708[27], 6}, {NULL}};
const Segment SGI223641[] = {{&S83708[98], 9}, {NULL}};
const Segment SGI223642[] = {{&S83708[71], 6}, {NULL}};
const Segment SGI223643[] = {{&S83709[34], 11}, {NULL}};
const Segment SGI223644[] = {{&S83709[27], 6}, {NULL}};
const Segment SGI223645[] = {{&S83709[98], 9}, {NULL}};
const Segment SGI223646[] = {{&S83709[71], 6}, {NULL}};
const Segment SGI223647[] = {{&S83710[34], 11}, {NULL}};
const Segment SGI223648[] = {{&S83710[27], 6}, {NULL}};
const Segment SGI223649[] = {{&S83710[98], 9}, {NULL}};
const Segment SGI223650[] = {{&S83710[71], 6}, {NULL}};
const Segment SGI223651[] = {{&S94512[34], 8}, {NULL}};
const Segment SGI223652[] = {{&S94512[27], 6}, {NULL}};
const Segment SGI223653[] = {{&S94512[95], 9}, {NULL}};
const Segment SGI223654[] = {{&S94512[68], 6}, {NULL}};
const Segment SGI223655[] = {{&S83715[34], 20}, {NULL}};
const Segment SGI223656[] = {{&S83715[27], 6}, {NULL}};
const Segment SGI223657[] = {{&S94514[34], 10}, {NULL}};
const Segment SGI223658[] = {{&S94514[27], 6}, {NULL}};
const Segment SGI223659[] = {{&S83715[107], 9}, {NULL}};
const Segment SGI223660[] = {{&S83715[80], 6}, {NULL}};
const Segment SGI223661[] = {{&S94514[97], 9}, {NULL}};
const Segment SGI223662[] = {{&S94514[70], 6}, {NULL}};
const Segment SGI223663[] = {{&S94515[34], 9}, {NULL}};
const Segment SGI223664[] = {{&S94515[27], 6}, {NULL}};
const Segment SGI223665[] = {{&S83717[44], 10}, {NULL}};
const Segment SGI223666[] = {{&S83717[44], 22}, {NULL}};
const Segment SGI223667[] = {{&S94515[96], 9}, {NULL}};
const Segment SGI223668[] = {{&S83717[30], 6}, {NULL}};
const Segment SGI223669[] = {{&S94515[69], 6}, {NULL}};
const Segment SGI223670[] = {{&S83717[113], 3}, {NULL}};
const Segment SGI223671[] = {{&S83717[80], 5}, {NULL}};
const Segment SGI223672[] = {{&S94517[44], 11}, {NULL}};
const Segment SGI223673[] = {{&S94517[44], 23}, {NULL}};
const Segment SGI223674[] = {{&S94517[30], 6}, {NULL}};
const Segment SGI223675[] = {{&S94517[31], 3}, {NULL}};
const Segment SGI223676[] = {{&S94517[81], 6}, {NULL}};
const Segment SGI223677[] = {{&S94517[151], 3}, {NULL}};
const Segment SGI223678[] = {{&S83721[34], 9}, {NULL}};
const Segment SGI223679[] = {{&S83721[27], 6}, {NULL}};
const Segment SGI223680[] = {{&S83721[96], 9}, {NULL}};
const Segment SGI223681[] = {{&S83721[69], 6}, {NULL}};
const Segment SGI223682[] = {{&S83721[137], 8}, {NULL}};
const Segment SGI223683[] = {{&S83722[44], 10}, {NULL}};
const Segment SGI223684[] = {{&S83722[44], 22}, {NULL}};
const Segment SGI223685[] = {{&S83722[30], 6}, {NULL}};
const Segment SGI223686[] = {{&S83722[115], 3}, {NULL}};
const Segment SGI223687[] = {{&S83722[80], 6}, {NULL}};
const Segment SGI223688[] = {{&S94522[34], 15}, {NULL}};
const Segment SGI223689[] = {{&S83723[38], 10}, {NULL}};
const Segment SGI223690[] = {{&S83723[31], 6}, {NULL}};
const Segment SGI223691[] = {{&S94522[27], 6}, {NULL}};
const Segment SGI223692[] = {{&S94522[102], 9}, {NULL}};
const Segment SGI223693[] = {{&S94522[75], 6}, {NULL}};
const Segment SGI223694[] = {{&S94523[34], 11}, {NULL}};
const Segment SGI223695[] = {{&S94523[27], 6}, {NULL}};
const Segment SGI223696[] = {{&S83725[44], 11}, {NULL}};
const Segment SGI223697[] = {{&S83725[44], 32}, {NULL}};
const Segment SGI223698[] = {{&S83725[30], 6}, {NULL}};
const Segment SGI223699[] = {{&S94523[98], 9}, {NULL}};
const Segment SGI223700[] = {{&S94523[71], 6}, {NULL}};
const Segment SGI223701[] = {{&S83725[125], 3}, {NULL}};
const Segment SGI223702[] = {{&S83725[90], 6}, {NULL}};
const Segment SGI223703[] = {{&S94524[34], 14}, {NULL}};
const Segment SGI223704[] = {{&S83726[44], 11}, {NULL}};
const Segment SGI223705[] = {{&S94524[27], 6}, {NULL}};
const Segment SGI223706[] = {{&S83726[44], 32}, {NULL}};
const Segment SGI223707[] = {{&S83726[30], 6}, {NULL}};
const Segment SGI223708[] = {{&S83726[125], 3}, {NULL}};
const Segment SGI223709[] = {{&S94524[101], 9}, {NULL}};
const Segment SGI223710[] = {{&S94524[74], 6}, {NULL}};
const Segment SGI223711[] = {{&S83726[90], 6}, {NULL}};
const Segment SGI223712[] = {{&S85500[34], 8}, {NULL}};
const Segment SGI223713[] = {{&S85500[27], 6}, {NULL}};
const Segment SGI223714[] = {{&S85500[95], 9}, {NULL}};
const Segment SGI223715[] = {{&S85500[68], 6}, {NULL}};
const Segment SGI223716[] = {{&S83735[34], 10}, {NULL}};
const Segment SGI223717[] = {{&S83735[27], 6}, {NULL}};
const Segment SGI223718[] = {{&S83735[97], 9}, {NULL}};
const Segment SGI223719[] = {{&S83735[70], 6}, {NULL}};
const Segment SGI223720[] = {{&S94533[44], 8}, {NULL}};
const Segment SGI223721[] = {{&S94533[44], 20}, {NULL}};
const Segment SGI223722[] = {{&S94533[30], 6}, {NULL}};
const Segment SGI223723[] = {{&S94533[31], 3}, {NULL}};
const Segment SGI223724[] = {{&S94533[113], 6}, {NULL}};
const Segment SGI223725[] = {{&S94533[66], 8}, {NULL}};
const Segment SGI223726[] = {{&S94534[44], 12}, {NULL}};
const Segment SGI223727[] = {{&S94534[44], 24}, {NULL}};
const Segment SGI223728[] = {{&S94534[30], 6}, {NULL}};
const Segment SGI223729[] = {{&S94534[31], 3}, {NULL}};
const Segment SGI223730[] = {{&S94534[82], 6}, {NULL}};
const Segment SGI223731[] = {{&S76468[44], 10}, {NULL}};
const Segment SGI223732[] = {{&S76468[44], 21}, {NULL}};
const Segment SGI223733[] = {{&S76468[30], 6}, {NULL}};
const Segment SGI223734[] = {{&S76468[132], 3}, {NULL}};
const Segment SGI223735[] = {{&S76468[97], 6}, {NULL}};
const Segment SGI223736[] = {{&S76468[67], 8}, {NULL}};
const Segment SGI223737[] = {{&S94536[34], 9}, {NULL}};
const Segment SGI223738[] = {{&S94536[27], 6}, {NULL}};
const Segment SGI223739[] = {{&S76469[44], 10}, {NULL}};
const Segment SGI223740[] = {{&S76469[44], 22}, {NULL}};
const Segment SGI223741[] = {{&S76469[30], 6}, {NULL}};
const Segment SGI223742[] = {{&S94536[96], 9}, {NULL}};
const Segment SGI223743[] = {{&S94536[69], 6}, {NULL}};
const Segment SGI223744[] = {{&S76469[115], 3}, {NULL}};
const Segment SGI223745[] = {{&S76469[80], 6}, {NULL}};
const Segment SGI223746[] = {{&S85512[44], 7}, {NULL}};
const Segment SGI223747[] = {{&S76469[140], 8}, {NULL}};
const Segment SGI223748[] = {{&S85512[44], 19}, {NULL}};
const Segment SGI223749[] = {{&S85512[30], 6}, {NULL}};
const Segment SGI223750[] = {{&S85512[112], 3}, {NULL}};
const Segment SGI223751[] = {{&S85512[77], 6}, {NULL}};
const Segment SGI223752[] = {{&S94537[34], 10}, {NULL}};
const Segment SGI223753[] = {{&S94537[27], 6}, {NULL}};
const Segment SGI223754[] = {{&S94537[97], 9}, {NULL}};
const Segment SGI223755[] = {{&S94537[70], 6}, {NULL}};
const Segment SGI223756[] = {{&S94540[44], 15}, {NULL}};
const Segment SGI223757[] = {{&S94540[44], 27}, {NULL}};
const Segment SGI223758[] = {{&S94540[30], 6}, {NULL}};
const Segment SGI223759[] = {{&S94540[31], 3}, {NULL}};
const Segment SGI223760[] = {{&S94540[85], 6}, {NULL}};
const Segment SGI223761[] = {{&S94542[34], 8}, {NULL}};
const Segment SGI223762[] = {{&S94542[27], 6}, {NULL}};
const Segment SGI223763[] = {{&S94542[95], 9}, {NULL}};
const Segment SGI223764[] = {{&S94542[68], 6}, {NULL}};
const Segment SGI223765[] = {{&S94544[34], 14}, {NULL}};
const Segment SGI223766[] = {{&S94544[27], 6}, {NULL}};
const Segment SGI223767[] = {{&S94544[101], 9}, {NULL}};
const Segment SGI223768[] = {{&S94544[74], 6}, {NULL}};
const Segment SGI223769[] = {{&S94545[34], 10}, {NULL}};
const Segment SGI223770[] = {{&S94545[27], 6}, {NULL}};
const Segment SGI223771[] = {{&S94545[97], 9}, {NULL}};
const Segment SGI223772[] = {{&S94545[70], 6}, {NULL}};
const Segment SGI223773[] = {{&S94545[138], 8}, {NULL}};
const Segment SGI223774[] = {{&S94546[34], 15}, {NULL}};
const Segment SGI223775[] = {{&S94546[27], 6}, {NULL}};
const Segment SGI223776[] = {{&S94546[102], 9}, {NULL}};
const Segment SGI223777[] = {{&S94546[75], 6}, {NULL}};
const Segment SGI223778[] = {{&S94550[34], 10}, {NULL}};
const Segment SGI223779[] = {{&S94550[27], 6}, {NULL}};
const Segment SGI223780[] = {{&S94550[97], 9}, {NULL}};
const Segment SGI223781[] = {{&S94550[70], 6}, {NULL}};
const Segment SGI223782[] = {{&S94551[34], 9}, {NULL}};
const Segment SGI223783[] = {{&S94551[27], 6}, {NULL}};
const Segment SGI223784[] = {{&S94551[96], 9}, {NULL}};
const Segment SGI223785[] = {{&S94551[69], 6}, {NULL}};
const Segment SGI223786[] = {{&S94556[34], 9}, {NULL}};
const Segment SGI223787[] = {{&S94556[27], 6}, {NULL}};
const Segment SGI223788[] = {{&S94556[103], 9}, {NULL}};
const Segment SGI223789[] = {{&S94556[76], 6}, {NULL}};
const Segment SGI223790[] = {{&S83778[44], 18}, {NULL}};
const Segment SGI223791[] = {{&S83778[44], 30}, {NULL}};
const Segment SGI223792[] = {{&S83778[30], 6}, {NULL}};
const Segment SGI223793[] = {{&S83778[139], 3}, {NULL}};
const Segment SGI223794[] = {{&S83778[104], 6}, {NULL}};
const Segment SGI223795[] = {{&S83778[76], 8}, {NULL}};
const Segment SGI223796[] = {{&S94560[34], 11}, {NULL}};
const Segment SGI223797[] = {{&S94560[27], 6}, {NULL}};
const Segment SGI223798[] = {{&S94560[98], 9}, {NULL}};
const Segment SGI223799[] = {{&S94560[71], 6}, {NULL}};
const Segment SGI223800[] = {{&S94562[44], 6}, {NULL}};
const Segment SGI223801[] = {{&S94562[44], 18}, {NULL}};
const Segment SGI223802[] = {{&S94562[30], 6}, {NULL}};
const Segment SGI223803[] = {{&S94562[31], 3}, {NULL}};
const Segment SGI223804[] = {{&S94562[76], 5}, {NULL}};
const Segment SGI223805[] = {{&S94562[113], 14}, {NULL}};
const Segment SGI223806[] = {{&S90436[44], 9}, {NULL}};
const Segment SGI223807[] = {{&S90436[44], 21}, {NULL}};
const Segment SGI223808[] = {{&S90436[30], 6}, {NULL}};
const Segment SGI223809[] = {{&S90436[114], 3}, {NULL}};
const Segment SGI223810[] = {{&S90436[79], 6}, {NULL}};
const Segment SGI223811[] = {{&S94565[34], 7}, {NULL}};
const Segment SGI223812[] = {{&S94565[27], 6}, {NULL}};
const Segment SGI223813[] = {{&S94565[99], 9}, {NULL}};
const Segment SGI223814[] = {{&S94565[72], 6}, {NULL}};
const Segment SGI223815[] = {{&S94565[140], 8}, {NULL}};
const Segment SGI223816[] = {{&S94569[44], 23}, {NULL}};
const Segment SGI223817[] = {{&S94569[44], 39}, {NULL}};
const Segment SGI223818[] = {{&S94569[30], 6}, {NULL}};
const Segment SGI223819[] = {{&S94569[31], 3}, {NULL}};
const Segment SGI223820[] = {{&S94569[97], 6}, {NULL}};
const Segment SGI223821[] = {{&S83796[34], 17}, {NULL}};
const Segment SGI223822[] = {{&S83796[27], 6}, {NULL}};
const Segment SGI223823[] = {{&S83796[104], 9}, {NULL}};
const Segment SGI223824[] = {{&S83796[77], 6}, {NULL}};
const Segment SGI223825[] = {{&S94579[44], 13}, {NULL}};
const Segment SGI223826[] = {{&S83802[34], 9}, {NULL}};
const Segment SGI223827[] = {{&S94579[44], 25}, {NULL}};
const Segment SGI223828[] = {{&S94579[30], 6}, {NULL}};
const Segment SGI223829[] = {{&S83802[27], 6}, {NULL}};
const Segment SGI223830[] = {{&S83802[96], 9}, {NULL}};
const Segment SGI223831[] = {{&S94579[83], 6}, {NULL}};
const Segment SGI223832[] = {{&S83802[69], 6}, {NULL}};
const Segment SGI223833[] = {{&S94579[110], 13}, {NULL}};
const Segment SGI223834[] = {{&S94583[34], 9}, {NULL}};
const Segment SGI223835[] = {{&S94583[27], 6}, {NULL}};
const Segment SGI223836[] = {{&S94583[96], 9}, {NULL}};
const Segment SGI223837[] = {{&S94583[69], 6}, {NULL}};
const Segment SGI223838[] = {{&S90460[44], 16}, {NULL}};
const Segment SGI223839[] = {{&S90460[44], 35}, {NULL}};
const Segment SGI223840[] = {{&S90460[30], 6}, {NULL}};
const Segment SGI223841[] = {{&S90460[128], 3}, {NULL}};
const Segment SGI223842[] = {{&S90460[93], 6}, {NULL}};
const Segment SGI223843[] = {{&S94585[34], 14}, {NULL}};
const Segment SGI223844[] = {{&S94585[27], 6}, {NULL}};
const Segment SGI223845[] = {{&S94585[101], 9}, {NULL}};
const Segment SGI223846[] = {{&S94585[74], 6}, {NULL}};
const Segment SGI223847[] = {{&S94588[34], 15}, {NULL}};
const Segment SGI223848[] = {{&S94588[27], 6}, {NULL}};
const Segment SGI223849[] = {{&S94588[102], 9}, {NULL}};
const Segment SGI223850[] = {{&S94588[75], 6}, {NULL}};
const Segment SGI223851[] = {{&S90474[34], 8}, {NULL}};
const Segment SGI223852[] = {{&S90474[27], 6}, {NULL}};
const Segment SGI223853[] = {{&S90474[95], 9}, {NULL}};
const Segment SGI223854[] = {{&S90474[68], 6}, {NULL}};
const Segment SGI223855[] = {{&S94600[44], 13}, {NULL}};
const Segment SGI223856[] = {{&S94600[44], 25}, {NULL}};
const Segment SGI223857[] = {{&S94600[30], 6}, {NULL}};
const Segment SGI223858[] = {{&S94600[31], 3}, {NULL}};
const Segment SGI223859[] = {{&S94600[83], 5}, {NULL}};
const Segment SGI223860[] = {{&S94600[120], 14}, {NULL}};
const Segment SGI223861[] = {{&S94602[34], 24}, {NULL}};
const Segment SGI223862[] = {{&S94602[27], 6}, {NULL}};
const Segment SGI223863[] = {{&S94602[111], 9}, {NULL}};
const Segment SGI223864[] = {{&S94602[84], 6}, {NULL}};
const Segment SGI223865[] = {{&S94602[152], 8}, {NULL}};
const Segment SGI223866[] = {{&S94605[34], 7}, {NULL}};
const Segment SGI223867[] = {{&S94605[27], 6}, {NULL}};
const Segment SGI223868[] = {{&S94605[99], 9}, {NULL}};
const Segment SGI223869[] = {{&S94605[72], 6}, {NULL}};
const Segment SGI223870[] = {{&S85610[34], 11}, {NULL}};
const Segment SGI223871[] = {{&S85610[27], 6}, {NULL}};
const Segment SGI223872[] = {{&S85610[98], 9}, {NULL}};
const Segment SGI223873[] = {{&S85610[71], 6}, {NULL}};
const Segment SGI223874[] = {{&S90502[59], 6}, {NULL}};
const Segment SGI223875[] = {{&S90502[59], 18}, {NULL}};
const Segment SGI223876[] = {{&S90502[45], 6}, {NULL}};
const Segment SGI223877[] = {{&S94621[34], 7}, {NULL}};
const Segment SGI223878[] = {{&S90502[126], 3}, {NULL}};
const Segment SGI223879[] = {{&S94621[27], 6}, {NULL}};
const Segment SGI223880[] = {{&S90502[91], 6}, {NULL}};
const Segment SGI223881[] = {{&S90502[0], 14}, {NULL}};
const Segment SGI223882[] = {{&S94621[93], 9}, {NULL}};
const Segment SGI223883[] = {{&S94621[66], 6}, {NULL}};
const Segment SGI223884[] = {{&S83859[44], 10}, {NULL}};
const Segment SGI223885[] = {{&S83859[44], 22}, {NULL}};
const Segment SGI223886[] = {{&S83859[30], 6}, {NULL}};
const Segment SGI223887[] = {{&S83859[115], 3}, {NULL}};
const Segment SGI223888[] = {{&S83859[80], 6}, {NULL}};
const Segment SGI223889[] = {{&S83859[133], 13}, {NULL}};
const Segment SGI223890[] = {{&S83861[44], 11}, {NULL}};
const Segment SGI223891[] = {{&S83861[44], 26}, {NULL}};
const Segment SGI223892[] = {{&S83861[30], 6}, {NULL}};
const Segment SGI223893[] = {{&S83861[119], 3}, {NULL}};
const Segment SGI223894[] = {{&S83861[84], 6}, {NULL}};
const Segment SGI223895[] = {{&S83861[151], 15}, {NULL}};
const Segment SGI223896[] = {{&S83862[44], 15}, {NULL}};
const Segment SGI223897[] = {{&S83862[44], 27}, {NULL}};
const Segment SGI223898[] = {{&S83862[30], 6}, {NULL}};
const Segment SGI223899[] = {{&S83862[120], 3}, {NULL}};
const Segment SGI223900[] = {{&S83862[85], 6}, {NULL}};
const Segment SGI223901[] = {{&S83862[152], 15}, {NULL}};
const Segment SGI223902[] = {{&S90514[44], 11}, {NULL}};
const Segment SGI223903[] = {{&S90514[44], 23}, {NULL}};
const Segment SGI223904[] = {{&S90514[30], 6}, {NULL}};
const Segment SGI223905[] = {{&S90514[81], 6}, {NULL}};
const Segment SGI223906[] = {{&S90514[108], 13}, {NULL}};
const Segment SGI223907[] = {{&S83874[44], 7}, {NULL}};
const Segment SGI223908[] = {{&S83874[44], 24}, {NULL}};
const Segment SGI223909[] = {{&S83874[30], 6}, {NULL}};
const Segment SGI223910[] = {{&S83874[117], 3}, {NULL}};
const Segment SGI223911[] = {{&S83874[82], 6}, {NULL}};
const Segment SGI223912[] = {{&S90525[34], 7}, {NULL}};
const Segment SGI223913[] = {{&S90525[27], 6}, {NULL}};
const Segment SGI223914[] = {{&S90525[94], 9}, {NULL}};
const Segment SGI223915[] = {{&S90525[67], 6}, {NULL}};
const Segment SGI223916[] = {{&S83886[34], 10}, {NULL}};
const Segment SGI223917[] = {{&S83886[27], 6}, {NULL}};
const Segment SGI223918[] = {{&S83886[97], 9}, {NULL}};
const Segment SGI223919[] = {{&S83886[70], 6}, {NULL}};
const Segment SGI223920[] = {{&S83887[34], 11}, {NULL}};
const Segment SGI223921[] = {{&S83887[27], 6}, {NULL}};
const Segment SGI223922[] = {{&S83887[98], 9}, {NULL}};
const Segment SGI223923[] = {{&S83887[71], 6}, {NULL}};
const Segment SGI223924[] = {{&S85692[34], 7}, {NULL}};
const Segment SGI223925[] = {{&S85692[27], 6}, {NULL}};
const Segment SGI223926[] = {{&S85692[94], 9}, {NULL}};
const Segment SGI223927[] = {{&S85692[67], 6}, {NULL}};
const Segment SGI223928[] = {{&S94691[44], 11}, {NULL}};
const Segment SGI223929[] = {{&S94691[44], 23}, {NULL}};
const Segment SGI223930[] = {{&S94691[30], 6}, {NULL}};
const Segment SGI223931[] = {{&S94691[31], 3}, {NULL}};
const Segment SGI223932[] = {{&S94691[96], 6}, {NULL}};
const Segment SGI223933[] = {{&S94691[69], 8}, {NULL}};
const Segment SGI223934[] = {{&S83934[34], 10}, {NULL}};
const Segment SGI223935[] = {{&S83934[27], 6}, {NULL}};
const Segment SGI223936[] = {{&S83934[97], 9}, {NULL}};
const Segment SGI223937[] = {{&S83934[70], 6}, {NULL}};
const Segment SGI223938[] = {{&S83944[34], 14}, {NULL}};
const Segment SGI223939[] = {{&S83944[27], 6}, {NULL}};
const Segment SGI223940[] = {{&S83944[106], 9}, {NULL}};
const Segment SGI223941[] = {{&S83944[79], 6}, {NULL}};
const Segment SGI223942[] = {{&S94712[34], 11}, {NULL}};
const Segment SGI223943[] = {{&S94712[27], 6}, {NULL}};
const Segment SGI223944[] = {{&S94712[98], 9}, {NULL}};
const Segment SGI223945[] = {{&S94712[71], 6}, {NULL}};
const Segment SGI223946[] = {{&S94714[59], 12}, {NULL}};
const Segment SGI223947[] = {{&S94714[59], 40}, {NULL}};
const Segment SGI223948[] = {{&S94714[45], 6}, {NULL}};
const Segment SGI223949[] = {{&S94714[46], 3}, {NULL}};
const Segment SGI223950[] = {{&S94714[113], 6}, {NULL}};
const Segment SGI223951[] = {{&S94714[0], 14}, {NULL}};
const Segment SGI223952[] = {{&S94715[59], 24}, {NULL}};
const Segment SGI223953[] = {{&S94715[45], 6}, {NULL}};
const Segment SGI223954[] = {{&S94715[46], 3}, {NULL}};
const Segment SGI223955[] = {{&S94715[116], 6}, {NULL}};
const Segment SGI223956[] = {{&S94715[0], 14}, {NULL}};
const Segment SGI223957[] = {{&S94723[34], 9}, {NULL}};
const Segment SGI223958[] = {{&S94723[27], 6}, {NULL}};
const Segment SGI223959[] = {{&S94723[95], 9}, {NULL}};
const Segment SGI223960[] = {{&S94723[68], 6}, {NULL}};
const Segment SGI223961[] = {{&S83963[34], 11}, {NULL}};
const Segment SGI223962[] = {{&S83963[27], 6}, {NULL}};
const Segment SGI223963[] = {{&S85732[34], 16}, {NULL}};
const Segment SGI223964[] = {{&S85732[27], 6}, {NULL}};
const Segment SGI223965[] = {{&S83963[98], 9}, {NULL}};
const Segment SGI223966[] = {{&S83963[71], 6}, {NULL}};
const Segment SGI223967[] = {{&S85732[103], 9}, {NULL}};
const Segment SGI223968[] = {{&S85732[76], 6}, {NULL}};
const Segment SGI223969[] = {{&S83964[34], 11}, {NULL}};
const Segment SGI223970[] = {{&S83964[27], 6}, {NULL}};
const Segment SGI223971[] = {{&S83964[98], 9}, {NULL}};
const Segment SGI223972[] = {{&S83964[71], 6}, {NULL}};
const Segment SGI223973[] = {{&S90618[34], 7}, {NULL}};
const Segment SGI223974[] = {{&S90618[27], 6}, {NULL}};
const Segment SGI223975[] = {{&S94727[59], 10}, {NULL}};
const Segment SGI223976[] = {{&S83965[34], 11}, {NULL}};
const Segment SGI223977[] = {{&S94727[59], 22}, {NULL}};
const Segment SGI223978[] = {{&S90618[94], 9}, {NULL}};
const Segment SGI223979[] = {{&S94727[45], 6}, {NULL}};
const Segment SGI223980[] = {{&S90618[67], 6}, {NULL}};
const Segment SGI223981[] = {{&S83965[27], 6}, {NULL}};
const Segment SGI223982[] = {{&S94727[46], 3}, {NULL}};
const Segment SGI223983[] = {{&S83965[98], 9}, {NULL}};
const Segment SGI223984[] = {{&S83965[71], 6}, {NULL}};
const Segment SGI223985[] = {{&S94727[95], 6}, {NULL}};
const Segment SGI223986[] = {{&S94727[0], 14}, {NULL}};
const Segment SGI223987[] = {{&S94732[59], 18}, {NULL}};
const Segment SGI223988[] = {{&S94732[59], 30}, {NULL}};
const Segment SGI223989[] = {{&S94732[45], 6}, {NULL}};
const Segment SGI223990[] = {{&S94732[46], 3}, {NULL}};
const Segment SGI223991[] = {{&S94732[103], 6}, {NULL}};
const Segment SGI223992[] = {{&S94732[0], 14}, {NULL}};
const Segment SGI223993[] = {{&S83972[34], 13}, {NULL}};
const Segment SGI223994[] = {{&S83972[27], 6}, {NULL}};
const Segment SGI223995[] = {{&S83972[100], 9}, {NULL}};
const Segment SGI223996[] = {{&S83972[73], 6}, {NULL}};
const Segment SGI223997[] = {{&S94736[34], 8}, {NULL}};
const Segment SGI223998[] = {{&S94736[27], 6}, {NULL}};
const Segment SGI223999[] = {{&S83975[34], 10}, {NULL}};
const Segment SGI224000[] = {{&S83975[27], 6}, {NULL}};
const Segment SGI224001[] = {{&S94736[100], 9}, {NULL}};
const Segment SGI224002[] = {{&S94736[73], 6}, {NULL}};
const Segment SGI224003[] = {{&S83975[97], 9}, {NULL}};
const Segment SGI224004[] = {{&S83975[70], 6}, {NULL}};
const Segment SGI224005[] = {{&S94737[34], 5}, {NULL}};
const Segment SGI224006[] = {{&S94737[27], 6}, {NULL}};
const Segment SGI224007[] = {{&S83976[34], 8}, {NULL}};
const Segment SGI224008[] = {{&S83976[27], 6}, {NULL}};
const Segment SGI224009[] = {{&S94737[92], 9}, {NULL}};
const Segment SGI224010[] = {{&S94737[65], 6}, {NULL}};
const Segment SGI224011[] = {{&S83976[101], 9}, {NULL}};
const Segment SGI224012[] = {{&S83976[74], 6}, {NULL}};
const Segment SGI224013[] = {{&S94739[44], 10}, {NULL}};
const Segment SGI224014[] = {{&S94739[44], 22}, {NULL}};
const Segment SGI224015[] = {{&S94739[30], 6}, {NULL}};
const Segment SGI224016[] = {{&S94739[31], 3}, {NULL}};
const Segment SGI224017[] = {{&S94739[80], 6}, {NULL}};
const Segment SGI224018[] = {{&S94739[132], 10}, {NULL}};
const Segment SGI224019[] = {{&S94742[44], 25}, {NULL}};
const Segment SGI224020[] = {{&S94742[44], 37}, {NULL}};
const Segment SGI224021[] = {{&S94742[30], 6}, {NULL}};
const Segment SGI224022[] = {{&S94742[31], 3}, {NULL}};
const Segment SGI224023[] = {{&S94742[95], 6}, {NULL}};
const Segment SGI224024[] = {{&S94744[45], 7}, {NULL}};
const Segment SGI224025[] = {{&S94744[30], 6}, {NULL}};
const Segment SGI224026[] = {{&S94744[31], 3}, {NULL}};
const Segment SGI224027[] = {{&S94744[77], 5}, {NULL}};
const Segment SGI224028[] = {{&S94744[135], 13}, {NULL}};
const Segment SGI224029[] = {{&S94745[45], 6}, {NULL}};
const Segment SGI224030[] = {{&S94745[30], 6}, {NULL}};
const Segment SGI224031[] = {{&S94745[31], 3}, {NULL}};
const Segment SGI224032[] = {{&S94745[77], 5}, {NULL}};
const Segment SGI224033[] = {{&S94745[135], 13}, {NULL}};
const Segment SGI224034[] = {{&S94749[34], 7}, {NULL}};
const Segment SGI224035[] = {{&S94749[27], 6}, {NULL}};
const Segment SGI224036[] = {{&S94749[90], 9}, {NULL}};
const Segment SGI224037[] = {{&S94749[63], 6}, {NULL}};
const Segment SGI224038[] = {{&S76593[34], 8}, {NULL}};
const Segment SGI224039[] = {{&S76593[27], 6}, {NULL}};
const Segment SGI224040[] = {{&S76593[94], 9}, {NULL}};
const Segment SGI224041[] = {{&S76593[67], 6}, {NULL}};
const Segment SGI224042[] = {{&S94755[44], 14}, {NULL}};
const Segment SGI224043[] = {{&S94755[44], 31}, {NULL}};
const Segment SGI224044[] = {{&S94755[30], 6}, {NULL}};
const Segment SGI224045[] = {{&S94755[31], 3}, {NULL}};
const Segment SGI224046[] = {{&S94755[89], 5}, {NULL}};
const Segment SGI224047[] = {{&S94755[126], 14}, {NULL}};
const Segment SGI224048[] = {{&S94757[34], 10}, {NULL}};
const Segment SGI224049[] = {{&S94757[27], 6}, {NULL}};
const Segment SGI224050[] = {{&S94757[97], 9}, {NULL}};
const Segment SGI224051[] = {{&S94757[70], 6}, {NULL}};
const Segment SGI224052[] = {{&S90665[44], 11}, {NULL}};
const Segment SGI224053[] = {{&S90665[44], 23}, {NULL}};
const Segment SGI224054[] = {{&S90665[30], 6}, {NULL}};
const Segment SGI224055[] = {{&S90665[116], 3}, {NULL}};
const Segment SGI224056[] = {{&S90665[81], 6}, {NULL}};
const Segment SGI224057[] = {{&S84012[59], 10}, {NULL}};
const Segment SGI224058[] = {{&S84012[59], 22}, {NULL}};
const Segment SGI224059[] = {{&S84012[45], 6}, {NULL}};
const Segment SGI224060[] = {{&S84012[130], 3}, {NULL}};
const Segment SGI224061[] = {{&S84012[95], 6}, {NULL}};
const Segment SGI224062[] = {{&S84012[0], 14}, {NULL}};
const Segment SGI224063[] = {{&S90667[34], 13}, {NULL}};
const Segment SGI224064[] = {{&S90667[27], 6}, {NULL}};
const Segment SGI224065[] = {{&S94761[34], 11}, {NULL}};
const Segment SGI224066[] = {{&S94761[27], 6}, {NULL}};
const Segment SGI224067[] = {{&S90667[100], 9}, {NULL}};
const Segment SGI224068[] = {{&S90667[73], 6}, {NULL}};
const Segment SGI224069[] = {{&S94761[98], 9}, {NULL}};
const Segment SGI224070[] = {{&S94761[71], 6}, {NULL}};
const Segment SGI224071[] = {{&S94765[34], 15}, {NULL}};
const Segment SGI224072[] = {{&S94765[27], 6}, {NULL}};
const Segment SGI224073[] = {{&S94765[109], 9}, {NULL}};
const Segment SGI224074[] = {{&S94765[82], 6}, {NULL}};
const Segment SGI224075[] = {{&S84020[34], 10}, {NULL}};
const Segment SGI224076[] = {{&S84020[27], 6}, {NULL}};
const Segment SGI224077[] = {{&S84020[97], 9}, {NULL}};
const Segment SGI224078[] = {{&S84020[70], 6}, {NULL}};
const Segment SGI224079[] = {{&S94767[34], 10}, {NULL}};
const Segment SGI224080[] = {{&S94767[27], 6}, {NULL}};
const Segment SGI224081[] = {{&S94767[102], 9}, {NULL}};
const Segment SGI224082[] = {{&S94767[75], 6}, {NULL}};
const Segment SGI224083[] = {{&S94769[34], 9}, {NULL}};
const Segment SGI224084[] = {{&S94769[27], 6}, {NULL}};
const Segment SGI224085[] = {{&S94769[96], 9}, {NULL}};
const Segment SGI224086[] = {{&S94769[69], 6}, {NULL}};
const Segment SGI224087[] = {{&S84025[44], 7}, {NULL}};
const Segment SGI224088[] = {{&S84025[44], 19}, {NULL}};
const Segment SGI224089[] = {{&S84025[30], 6}, {NULL}};
const Segment SGI224090[] = {{&S84025[112], 3}, {NULL}};
const Segment SGI224091[] = {{&S84025[77], 6}, {NULL}};
const Segment SGI224092[] = {{&S76623[44], 10}, {NULL}};
const Segment SGI224093[] = {{&S76623[44], 21}, {NULL}};
const Segment SGI224094[] = {{&S76623[30], 6}, {NULL}};
const Segment SGI224095[] = {{&S76623[132], 3}, {NULL}};
const Segment SGI224096[] = {{&S76623[97], 6}, {NULL}};
const Segment SGI224097[] = {{&S76623[67], 8}, {NULL}};
const Segment SGI224098[] = {{&S84026[34], 8}, {NULL}};
const Segment SGI224099[] = {{&S84026[27], 6}, {NULL}};
const Segment SGI224100[] = {{&S84026[101], 9}, {NULL}};
const Segment SGI224101[] = {{&S84026[74], 6}, {NULL}};
const Segment SGI224102[] = {{&S76624[44], 10}, {NULL}};
const Segment SGI224103[] = {{&S76624[44], 22}, {NULL}};
const Segment SGI224104[] = {{&S76624[30], 6}, {NULL}};
const Segment SGI224105[] = {{&S76624[133], 3}, {NULL}};
const Segment SGI224106[] = {{&S76624[98], 6}, {NULL}};
const Segment SGI224107[] = {{&S84027[34], 8}, {NULL}};
const Segment SGI224108[] = {{&S76624[68], 8}, {NULL}};
const Segment SGI224109[] = {{&S84027[27], 6}, {NULL}};
const Segment SGI224110[] = {{&S84027[103], 9}, {NULL}};
const Segment SGI224111[] = {{&S84027[76], 6}, {NULL}};
const Segment SGI224112[] = {{&S84030[34], 9}, {NULL}};
const Segment SGI224113[] = {{&S84030[27], 6}, {NULL}};
const Segment SGI224114[] = {{&S84030[96], 9}, {NULL}};
const Segment SGI224115[] = {{&S84030[69], 6}, {NULL}};
const Segment SGI224116[] = {{&S94779[59], 10}, {NULL}};
const Segment SGI224117[] = {{&S94779[59], 22}, {NULL}};
const Segment SGI224118[] = {{&S94779[45], 6}, {NULL}};
const Segment SGI224119[] = {{&S94779[46], 3}, {NULL}};
const Segment SGI224120[] = {{&S84031[34], 10}, {NULL}};
const Segment SGI224121[] = {{&S94779[95], 6}, {NULL}};
const Segment SGI224122[] = {{&S84031[27], 6}, {NULL}};
const Segment SGI224123[] = {{&S94779[0], 14}, {NULL}};
const Segment SGI224124[] = {{&S84031[97], 9}, {NULL}};
const Segment SGI224125[] = {{&S84031[70], 6}, {NULL}};
const Segment SGI224126[] = {{&S94781[44], 11}, {NULL}};
const Segment SGI224127[] = {{&S94781[44], 23}, {NULL}};
const Segment SGI224128[] = {{&S94781[30], 6}, {NULL}};
const Segment SGI224129[] = {{&S94781[81], 6}, {NULL}};
const Segment SGI224130[] = {{&S94781[108], 13}, {NULL}};
const Segment SGI224131[] = {{&S84034[44], 7}, {NULL}};
const Segment SGI224132[] = {{&S84034[44], 24}, {NULL}};
const Segment SGI224133[] = {{&S84034[30], 6}, {NULL}};
const Segment SGI224134[] = {{&S84034[82], 6}, {NULL}};
const Segment SGI224135[] = {{&S76635[34], 9}, {NULL}};
const Segment SGI224136[] = {{&S84034[109], 13}, {NULL}};
const Segment SGI224137[] = {{&S76635[27], 6}, {NULL}};
const Segment SGI224138[] = {{&S76635[95], 9}, {NULL}};
const Segment SGI224139[] = {{&S76635[68], 6}, {NULL}};
const Segment SGI224140[] = {{&S76635[136], 8}, {NULL}};
const Segment SGI224141[] = {{&S84035[44], 7}, {NULL}};
const Segment SGI224142[] = {{&S84035[44], 24}, {NULL}};
const Segment SGI224143[] = {{&S84035[30], 6}, {NULL}};
const Segment SGI224144[] = {{&S84035[82], 6}, {NULL}};
const Segment SGI224145[] = {{&S84035[109], 13}, {NULL}};
const Segment SGI224146[] = {{&S84036[44], 9}, {NULL}};
const Segment SGI224147[] = {{&S84036[44], 21}, {NULL}};
const Segment SGI224148[] = {{&S84036[30], 6}, {NULL}};
const Segment SGI224149[] = {{&S84036[79], 6}, {NULL}};
const Segment SGI224150[] = {{&S84036[106], 13}, {NULL}};
const Segment SGI224151[] = {{&S94787[34], 5}, {NULL}};
const Segment SGI224152[] = {{&S94787[27], 6}, {NULL}};
const Segment SGI224153[] = {{&S94787[92], 9}, {NULL}};
const Segment SGI224154[] = {{&S94787[65], 6}, {NULL}};
const Segment SGI224155[] = {{&S94787[133], 8}, {NULL}};
const Segment SGI224156[] = {{&S85824[44], 10}, {NULL}};
const Segment SGI224157[] = {{&S85824[44], 22}, {NULL}};
const Segment SGI224158[] = {{&S85824[30], 6}, {NULL}};
const Segment SGI224159[] = {{&S85824[115], 3}, {NULL}};
const Segment SGI224160[] = {{&S85824[80], 6}, {NULL}};
const Segment SGI224161[] = {{&S85824[143], 3}, {NULL}};
const Segment SGI224162[] = {{&S94790[34], 10}, {NULL}};
const Segment SGI224163[] = {{&S94790[27], 6}, {NULL}};
const Segment SGI224164[] = {{&S94790[97], 9}, {NULL}};
const Segment SGI224165[] = {{&S94790[70], 6}, {NULL}};
const Segment SGI224166[] = {{&S85829[44], 9}, {NULL}};
const Segment SGI224167[] = {{&S85829[44], 21}, {NULL}};
const Segment SGI224168[] = {{&S85829[30], 6}, {NULL}};
const Segment SGI224169[] = {{&S85829[114], 3}, {NULL}};
const Segment SGI224170[] = {{&S85829[79], 6}, {NULL}};
const Segment SGI224171[] = {{&S84045[34], 9}, {NULL}};
const Segment SGI224172[] = {{&S85829[131], 10}, {NULL}};
const Segment SGI224173[] = {{&S84045[27], 6}, {NULL}};
const Segment SGI224174[] = {{&S84045[96], 9}, {NULL}};
const Segment SGI224175[] = {{&S84045[69], 6}, {NULL}};
const Segment SGI224176[] = {{&S84046[34], 10}, {NULL}};
const Segment SGI224177[] = {{&S84046[27], 6}, {NULL}};
const Segment SGI224178[] = {{&S84046[97], 9}, {NULL}};
const Segment SGI224179[] = {{&S84046[70], 6}, {NULL}};
const Segment SGI224180[] = {{&S94798[34], 20}, {NULL}};
const Segment SGI224181[] = {{&S94798[27], 6}, {NULL}};
const Segment SGI224182[] = {{&S94798[107], 9}, {NULL}};
const Segment SGI224183[] = {{&S94798[80], 6}, {NULL}};
const Segment SGI224184[] = {{&S94799[34], 5}, {NULL}};
const Segment SGI224185[] = {{&S94799[27], 6}, {NULL}};
const Segment SGI224186[] = {{&S94799[92], 9}, {NULL}};
const Segment SGI224187[] = {{&S94799[65], 6}, {NULL}};
const Segment SGI224188[] = {{&S94801[34], 11}, {NULL}};
const Segment SGI224189[] = {{&S94801[27], 6}, {NULL}};
const Segment SGI224190[] = {{&S94801[98], 9}, {NULL}};
const Segment SGI224191[] = {{&S94801[71], 6}, {NULL}};
const Segment SGI224192[] = {{&S94812[44], 7}, {NULL}};
const Segment SGI224193[] = {{&S94812[44], 24}, {NULL}};
const Segment SGI224194[] = {{&S94812[30], 6}, {NULL}};
const Segment SGI224195[] = {{&S94812[82], 6}, {NULL}};
const Segment SGI224196[] = {{&S94812[109], 13}, {NULL}};
const Segment SGI224197[] = {{&S94818[44], 15}, {NULL}};
const Segment SGI224198[] = {{&S94818[44], 27}, {NULL}};
const Segment SGI224199[] = {{&S94818[30], 6}, {NULL}};
const Segment SGI224200[] = {{&S94818[31], 3}, {NULL}};
const Segment SGI224201[] = {{&S94818[85], 6}, {NULL}};
const Segment SGI224202[] = {{&S94825[52], 10}, {NULL}};
const Segment SGI224203[] = {{&S94825[52], 22}, {NULL}};
const Segment SGI224204[] = {{&S94825[38], 6}, {NULL}};
const Segment SGI224205[] = {{&S94825[88], 5}, {NULL}};
const Segment SGI224206[] = {{&S94825[0], 14}, {NULL}};
const Segment SGI224207[] = {{&S94836[34], 11}, {NULL}};
const Segment SGI224208[] = {{&S94836[27], 6}, {NULL}};
const Segment SGI224209[] = {{&S94836[98], 9}, {NULL}};
const Segment SGI224210[] = {{&S94836[71], 6}, {NULL}};
const Segment SGI224211[] = {{&S94838[34], 7}, {NULL}};
const Segment SGI224212[] = {{&S94838[27], 6}, {NULL}};
const Segment SGI224213[] = {{&S94838[99], 9}, {NULL}};
const Segment SGI224214[] = {{&S94838[72], 6}, {NULL}};
const Segment SGI224215[] = {{&S94839[34], 10}, {NULL}};
const Segment SGI224216[] = {{&S94839[27], 6}, {NULL}};
const Segment SGI224217[] = {{&S94839[97], 9}, {NULL}};
const Segment SGI224218[] = {{&S94839[70], 6}, {NULL}};
const Segment SGI224219[] = {{&S94840[27], 6}, {NULL}};
const Segment SGI224220[] = {{&S94840[103], 9}, {NULL}};
const Segment SGI224221[] = {{&S94840[76], 6}, {NULL}};
const Segment SGI224222[] = {{&S94841[34], 10}, {NULL}};
const Segment SGI224223[] = {{&S94841[27], 6}, {NULL}};
const Segment SGI224224[] = {{&S94841[97], 9}, {NULL}};
const Segment SGI224225[] = {{&S94841[70], 6}, {NULL}};
const Segment SGI224226[] = {{&S85893[34], 9}, {NULL}};
const Segment SGI224227[] = {{&S85893[27], 6}, {NULL}};
const Segment SGI224228[] = {{&S85893[96], 9}, {NULL}};
const Segment SGI224229[] = {{&S85893[69], 6}, {NULL}};
const Segment SGI224230[] = {{&S94844[34], 7}, {NULL}};
const Segment SGI224231[] = {{&S94844[27], 6}, {NULL}};
const Segment SGI224232[] = {{&S94844[94], 9}, {NULL}};
const Segment SGI224233[] = {{&S94844[67], 6}, {NULL}};
const Segment SGI224234[] = {{&S94846[34], 9}, {NULL}};
const Segment SGI224235[] = {{&S94846[27], 6}, {NULL}};
const Segment SGI224236[] = {{&S94846[102], 9}, {NULL}};
const Segment SGI224237[] = {{&S94846[75], 6}, {NULL}};
const Segment SGI224238[] = {{&S94852[52], 7}, {NULL}};
const Segment SGI224239[] = {{&S94852[52], 24}, {NULL}};
const Segment SGI224240[] = {{&S94852[38], 6}, {NULL}};
const Segment SGI224241[] = {{&S94852[90], 5}, {NULL}};
const Segment SGI224242[] = {{&S94852[0], 14}, {NULL}};
const Segment SGI224243[] = {{&S76725[34], 9}, {NULL}};
const Segment SGI224244[] = {{&S76725[27], 6}, {NULL}};
const Segment SGI224245[] = {{&S76725[96], 9}, {NULL}};
const Segment SGI224246[] = {{&S76725[69], 6}, {NULL}};
const Segment SGI224247[] = {{&S76731[34], 10}, {NULL}};
const Segment SGI224248[] = {{&S76731[27], 6}, {NULL}};
const Segment SGI224249[] = {{&S76731[96], 9}, {NULL}};
const Segment SGI224250[] = {{&S76731[69], 6}, {NULL}};
const Segment SGI224251[] = {{&S76732[34], 18}, {NULL}};
const Segment SGI224252[] = {{&S76732[27], 6}, {NULL}};
const Segment SGI224253[] = {{&S76732[105], 9}, {NULL}};
const Segment SGI224254[] = {{&S76732[78], 6}, {NULL}};
const Segment SGI224255[] = {{&S94863[44], 10}, {NULL}};
const Segment SGI224256[] = {{&S94863[44], 22}, {NULL}};
const Segment SGI224257[] = {{&S94863[30], 6}, {NULL}};
const Segment SGI224258[] = {{&S76733[34], 10}, {NULL}};
const Segment SGI224259[] = {{&S94863[115], 3}, {NULL}};
const Segment SGI224260[] = {{&S76733[27], 6}, {NULL}};
const Segment SGI224261[] = {{&S94863[80], 6}, {NULL}};
const Segment SGI224262[] = {{&S94863[135], 8}, {NULL}};
const Segment SGI224263[] = {{&S94863[144], 13}, {NULL}};
const Segment SGI224264[] = {{&S76733[97], 9}, {NULL}};
const Segment SGI224265[] = {{&S76733[70], 6}, {NULL}};
const Segment SGI224266[] = {{&S94864[44], 15}, {NULL}};
const Segment SGI224267[] = {{&S94864[44], 27}, {NULL}};
const Segment SGI224268[] = {{&S94864[30], 6}, {NULL}};
const Segment SGI224269[] = {{&S94864[120], 3}, {NULL}};
const Segment SGI224270[] = {{&S94864[85], 6}, {NULL}};
const Segment SGI224271[] = {{&S94864[140], 8}, {NULL}};
const Segment SGI224272[] = {{&S94864[149], 13}, {NULL}};
const Segment SGI224273[] = {{&S94865[44], 18}, {NULL}};
const Segment SGI224274[] = {{&S94865[44], 30}, {NULL}};
const Segment SGI224275[] = {{&S94865[30], 6}, {NULL}};
const Segment SGI224276[] = {{&S94865[88], 6}, {NULL}};
const Segment SGI224277[] = {{&S94865[115], 13}, {NULL}};
const Segment SGI224278[] = {{&S84162[34], 13}, {NULL}};
const Segment SGI224279[] = {{&S84162[27], 6}, {NULL}};
const Segment SGI224280[] = {{&S84162[100], 9}, {NULL}};
const Segment SGI224281[] = {{&S84162[73], 6}, {NULL}};
const Segment SGI224282[] = {{&S84163[34], 10}, {NULL}};
const Segment SGI224283[] = {{&S84163[27], 6}, {NULL}};
const Segment SGI224284[] = {{&S84163[97], 9}, {NULL}};
const Segment SGI224285[] = {{&S84163[70], 6}, {NULL}};
const Segment SGI224286[] = {{&S76793[34], 7}, {NULL}};
const Segment SGI224287[] = {{&S76793[27], 6}, {NULL}};
const Segment SGI224288[] = {{&S76793[94], 9}, {NULL}};
const Segment SGI224289[] = {{&S76793[67], 6}, {NULL}};
const Segment SGI224290[] = {{&S84203[44], 10}, {NULL}};
const Segment SGI224291[] = {{&S84203[44], 22}, {NULL}};
const Segment SGI224292[] = {{&S84203[30], 6}, {NULL}};
const Segment SGI224293[] = {{&S84203[131], 3}, {NULL}};
const Segment SGI224294[] = {{&S84203[96], 6}, {NULL}};
const Segment SGI224295[] = {{&S84203[68], 8}, {NULL}};
const Segment SGI224296[] = {{&S90841[34], 7}, {NULL}};
const Segment SGI224297[] = {{&S90841[27], 6}, {NULL}};
const Segment SGI224298[] = {{&S90841[94], 9}, {NULL}};
const Segment SGI224299[] = {{&S90841[67], 6}, {NULL}};
const Segment SGI224300[] = {{&S90844[34], 12}, {NULL}};
const Segment SGI224301[] = {{&S90844[27], 6}, {NULL}};
const Segment SGI224302[] = {{&S90844[99], 9}, {NULL}};
const Segment SGI224303[] = {{&S90844[72], 6}, {NULL}};
const Segment SGI224304[] = {{&S94930[45], 10}, {NULL}};
const Segment SGI224305[] = {{&S94930[30], 6}, {NULL}};
const Segment SGI224306[] = {{&S94930[31], 3}, {NULL}};
const Segment SGI224307[] = {{&S94930[81], 5}, {NULL}};
const Segment SGI224308[] = {{&S94930[139], 13}, {NULL}};
const Segment SGI224309[] = {{&S84213[34], 10}, {NULL}};
const Segment SGI224310[] = {{&S84213[27], 6}, {NULL}};
const Segment SGI224311[] = {{&S90847[44], 28}, {NULL}};
const Segment SGI224312[] = {{&S90847[44], 40}, {NULL}};
const Segment SGI224313[] = {{&S90847[30], 6}, {NULL}};
const Segment SGI224314[] = {{&S84213[97], 9}, {NULL}};
const Segment SGI224315[] = {{&S84213[70], 6}, {NULL}};
const Segment SGI224316[] = {{&S90847[133], 3}, {NULL}};
const Segment SGI224317[] = {{&S90847[98], 6}, {NULL}};
const Segment SGI224318[] = {{&S94936[34], 11}, {NULL}};
const Segment SGI224319[] = {{&S94936[27], 6}, {NULL}};
const Segment SGI224320[] = {{&S94936[98], 9}, {NULL}};
const Segment SGI224321[] = {{&S94936[71], 6}, {NULL}};
const Segment SGI224322[] = {{&S84220[34], 7}, {NULL}};
const Segment SGI224323[] = {{&S84220[27], 6}, {NULL}};
const Segment SGI224324[] = {{&S84220[94], 9}, {NULL}};
const Segment SGI224325[] = {{&S84220[67], 6}, {NULL}};
const Segment SGI224326[] = {{&S94941[34], 7}, {NULL}};
const Segment SGI224327[] = {{&S94941[27], 6}, {NULL}};
const Segment SGI224328[] = {{&S94941[96], 9}, {NULL}};
const Segment SGI224329[] = {{&S94941[69], 6}, {NULL}};
const Segment SGI224330[] = {{&S84229[44], 10}, {NULL}};
const Segment SGI224331[] = {{&S84229[44], 22}, {NULL}};
const Segment SGI224332[] = {{&S84229[30], 6}, {NULL}};
const Segment SGI224333[] = {{&S84229[115], 3}, {NULL}};
const Segment SGI224334[] = {{&S84229[80], 6}, {NULL}};
const Segment SGI224335[] = {{&S84229[134], 12}, {NULL}};
const Segment SGI224336[] = {{&S84230[44], 10}, {NULL}};
const Segment SGI224337[] = {{&S84230[44], 22}, {NULL}};
const Segment SGI224338[] = {{&S84230[30], 6}, {NULL}};
const Segment SGI224339[] = {{&S84230[115], 3}, {NULL}};
const Segment SGI224340[] = {{&S84230[80], 6}, {NULL}};
const Segment SGI224341[] = {{&S84230[133], 11}, {NULL}};
const Segment SGI224342[] = {{&S94950[34], 7}, {NULL}};
const Segment SGI224343[] = {{&S94950[27], 6}, {NULL}};
const Segment SGI224344[] = {{&S94950[94], 9}, {NULL}};
const Segment SGI224345[] = {{&S94950[67], 6}, {NULL}};
const Segment SGI224346[] = {{&S90870[34], 10}, {NULL}};
const Segment SGI224347[] = {{&S90870[27], 6}, {NULL}};
const Segment SGI224348[] = {{&S94951[34], 10}, {NULL}};
const Segment SGI224349[] = {{&S94951[27], 6}, {NULL}};
const Segment SGI224350[] = {{&S90870[97], 9}, {NULL}};
const Segment SGI224351[] = {{&S90870[70], 6}, {NULL}};
const Segment SGI224352[] = {{&S94951[97], 9}, {NULL}};
const Segment SGI224353[] = {{&S94951[70], 6}, {NULL}};
const Segment SGI224354[] = {{&S84237[44], 25}, {NULL}};
const Segment SGI224355[] = {{&S84237[44], 37}, {NULL}};
const Segment SGI224356[] = {{&S84237[30], 6}, {NULL}};
const Segment SGI224357[] = {{&S84237[130], 3}, {NULL}};
const Segment SGI224358[] = {{&S84237[95], 6}, {NULL}};
const Segment SGI224359[] = {{&S94954[34], 7}, {NULL}};
const Segment SGI224360[] = {{&S90873[44], 8}, {NULL}};
const Segment SGI224361[] = {{&S94954[27], 6}, {NULL}};
const Segment SGI224362[] = {{&S90873[44], 20}, {NULL}};
const Segment SGI224363[] = {{&S90873[30], 6}, {NULL}};
const Segment SGI224364[] = {{&S90873[113], 3}, {NULL}};
const Segment SGI224365[] = {{&S94954[94], 9}, {NULL}};
const Segment SGI224366[] = {{&S94954[67], 6}, {NULL}};
const Segment SGI224367[] = {{&S90873[78], 6}, {NULL}};
const Segment SGI224368[] = {{&S84238[38], 25}, {NULL}};
const Segment SGI224369[] = {{&S84238[31], 6}, {NULL}};
const Segment SGI224370[] = {{&S94955[34], 9}, {NULL}};
const Segment SGI224371[] = {{&S94955[27], 6}, {NULL}};
const Segment SGI224372[] = {{&S94955[96], 9}, {NULL}};
const Segment SGI224373[] = {{&S94955[69], 6}, {NULL}};
const Segment SGI224374[] = {{&S84240[34], 11}, {NULL}};
const Segment SGI224375[] = {{&S84240[27], 6}, {NULL}};
const Segment SGI224376[] = {{&S84240[98], 9}, {NULL}};
const Segment SGI224377[] = {{&S94956[34], 11}, {NULL}};
const Segment SGI224378[] = {{&S84240[71], 6}, {NULL}};
const Segment SGI224379[] = {{&S94956[27], 6}, {NULL}};
const Segment SGI224380[] = {{&S94956[98], 9}, {NULL}};
const Segment SGI224381[] = {{&S94956[71], 6}, {NULL}};
const Segment SGI224382[] = {{&S94957[34], 6}, {NULL}};
const Segment SGI224383[] = {{&S94957[27], 6}, {NULL}};
const Segment SGI224384[] = {{&S94957[93], 9}, {NULL}};
const Segment SGI224385[] = {{&S94957[66], 6}, {NULL}};
const Segment SGI224386[] = {{&S90880[59], 10}, {NULL}};
const Segment SGI224387[] = {{&S90880[59], 22}, {NULL}};
const Segment SGI224388[] = {{&S90880[45], 6}, {NULL}};
const Segment SGI224389[] = {{&S90880[130], 3}, {NULL}};
const Segment SGI224390[] = {{&S90880[95], 6}, {NULL}};
const Segment SGI224391[] = {{&S90880[0], 14}, {NULL}};
const Segment SGI224392[] = {{&S76846[34], 10}, {NULL}};
const Segment SGI224393[] = {{&S76846[27], 6}, {NULL}};
const Segment SGI224394[] = {{&S76846[96], 9}, {NULL}};
const Segment SGI224395[] = {{&S76846[69], 6}, {NULL}};
const Segment SGI224396[] = {{&S90883[44], 6}, {NULL}};
const Segment SGI224397[] = {{&S90883[44], 18}, {NULL}};
const Segment SGI224398[] = {{&S90883[30], 6}, {NULL}};
const Segment SGI224399[] = {{&S90883[109], 3}, {NULL}};
const Segment SGI224400[] = {{&S90883[76], 5}, {NULL}};
const Segment SGI224401[] = {{&S90883[113], 14}, {NULL}};
const Segment SGI224402[] = {{&S94963[45], 12}, {NULL}};
const Segment SGI224403[] = {{&S94963[30], 6}, {NULL}};
const Segment SGI224404[] = {{&S94963[31], 3}, {NULL}};
const Segment SGI224405[] = {{&S76848[34], 10}, {NULL}};
const Segment SGI224406[] = {{&S94963[99], 5}, {NULL}};
const Segment SGI224407[] = {{&S76848[27], 6}, {NULL}};
const Segment SGI224408[] = {{&S94963[157], 13}, {NULL}};
const Segment SGI224409[] = {{&S76848[96], 9}, {NULL}};
const Segment SGI224410[] = {{&S76848[69], 6}, {NULL}};
const Segment SGI224411[] = {{&S76848[137], 8}, {NULL}};
const Segment SGI224412[] = {{&S94964[34], 8}, {NULL}};
const Segment SGI224413[] = {{&S94964[27], 6}, {NULL}};
const Segment SGI224414[] = {{&S94964[95], 9}, {NULL}};
const Segment SGI224415[] = {{&S94964[68], 6}, {NULL}};
const Segment SGI224416[] = {{&S84252[44], 10}, {NULL}};
const Segment SGI224417[] = {{&S84252[44], 22}, {NULL}};
const Segment SGI224418[] = {{&S84252[30], 6}, {NULL}};
const Segment SGI224419[] = {{&S84252[115], 3}, {NULL}};
const Segment SGI224420[] = {{&S84252[80], 6}, {NULL}};
const Segment SGI224421[] = {{&S84252[133], 11}, {NULL}};
const Segment SGI224422[] = {{&S84253[44], 10}, {NULL}};
const Segment SGI224423[] = {{&S84253[44], 22}, {NULL}};
const Segment SGI224424[] = {{&S84253[30], 6}, {NULL}};
const Segment SGI224425[] = {{&S84253[115], 3}, {NULL}};
const Segment SGI224426[] = {{&S84253[80], 6}, {NULL}};
const Segment SGI224427[] = {{&S84253[134], 12}, {NULL}};
const Segment SGI224428[] = {{&S94969[45], 9}, {NULL}};
const Segment SGI224429[] = {{&S94969[30], 6}, {NULL}};
const Segment SGI224430[] = {{&S84255[34], 7}, {NULL}};
const Segment SGI224431[] = {{&S84255[27], 6}, {NULL}};
const Segment SGI224432[] = {{&S94969[31], 3}, {NULL}};
const Segment SGI224433[] = {{&S94969[80], 5}, {NULL}};
const Segment SGI224434[] = {{&S84255[94], 9}, {NULL}};
const Segment SGI224435[] = {{&S84255[67], 6}, {NULL}};
const Segment SGI224436[] = {{&S94969[138], 13}, {NULL}};
const Segment SGI224437[] = {{&S94970[44], 12}, {NULL}};
const Segment SGI224438[] = {{&S94970[44], 24}, {NULL}};
const Segment SGI224439[] = {{&S94970[30], 6}, {NULL}};
const Segment SGI224440[] = {{&S94970[31], 3}, {NULL}};
const Segment SGI224441[] = {{&S94970[82], 6}, {NULL}};
const Segment SGI224442[] = {{&S94971[34], 15}, {NULL}};
const Segment SGI224443[] = {{&S94971[27], 6}, {NULL}};
const Segment SGI224444[] = {{&S94971[102], 9}, {NULL}};
const Segment SGI224445[] = {{&S94971[75], 6}, {NULL}};
const Segment SGI224446[] = {{&S94975[44], 10}, {NULL}};
const Segment SGI224447[] = {{&S94975[44], 22}, {NULL}};
const Segment SGI224448[] = {{&S94975[30], 6}, {NULL}};
const Segment SGI224449[] = {{&S94975[31], 3}, {NULL}};
const Segment SGI224450[] = {{&S94975[80], 6}, {NULL}};
const Segment SGI224451[] = {{&S94975[143], 3}, {NULL}};
const Segment SGI224452[] = {{&S90904[34], 10}, {NULL}};
const Segment SGI224453[] = {{&S90904[27], 6}, {NULL}};
const Segment SGI224454[] = {{&S90904[97], 9}, {NULL}};
const Segment SGI224455[] = {{&S90904[70], 6}, {NULL}};
const Segment SGI224456[] = {{&S94982[34], 7}, {NULL}};
const Segment SGI224457[] = {{&S94982[27], 6}, {NULL}};
const Segment SGI224458[] = {{&S94982[94], 9}, {NULL}};
const Segment SGI224459[] = {{&S94982[67], 6}, {NULL}};
const Segment SGI224460[] = {{&S94984[45], 13}, {NULL}};
const Segment SGI224461[] = {{&S94984[30], 6}, {NULL}};
const Segment SGI224462[] = {{&S94984[31], 3}, {NULL}};
const Segment SGI224463[] = {{&S94984[84], 5}, {NULL}};
const Segment SGI224464[] = {{&S94984[142], 13}, {NULL}};
const Segment SGI224465[] = {{&S94989[34], 10}, {NULL}};
const Segment SGI224466[] = {{&S94989[27], 6}, {NULL}};
const Segment SGI224467[] = {{&S94989[97], 9}, {NULL}};
const Segment SGI224468[] = {{&S94989[70], 6}, {NULL}};
const Segment SGI224469[] = {{&S94991[34], 8}, {NULL}};
const Segment SGI224470[] = {{&S94991[27], 6}, {NULL}};
const Segment SGI224471[] = {{&S94991[95], 9}, {NULL}};
const Segment SGI224472[] = {{&S94991[68], 6}, {NULL}};
const Segment SGI224473[] = {{&S94992[34], 17}, {NULL}};
const Segment SGI224474[] = {{&S94992[27], 6}, {NULL}};
const Segment SGI224475[] = {{&S94992[104], 9}, {NULL}};
const Segment SGI224476[] = {{&S94992[77], 6}, {NULL}};
const Segment SGI224477[] = {{&S84298[34], 7}, {NULL}};
const Segment SGI224478[] = {{&S84298[27], 6}, {NULL}};
const Segment SGI224479[] = {{&S94997[34], 13}, {NULL}};
const Segment SGI224480[] = {{&S84298[94], 9}, {NULL}};
const Segment SGI224481[] = {{&S84298[67], 6}, {NULL}};
const Segment SGI224482[] = {{&S94997[27], 6}, {NULL}};
const Segment SGI224483[] = {{&S94997[100], 9}, {NULL}};
const Segment SGI224484[] = {{&S94997[73], 6}, {NULL}};
const Segment SGI224485[] = {{&S85958[6], 4}, {NULL}};
const Segment SGI224486[] = {{&S85958[83], 5}, {NULL}};
const Segment SGI224487[] = {{&S94999[44], 13}, {NULL}};
const Segment SGI224488[] = {{&S94999[44], 25}, {NULL}};
const Segment SGI224489[] = {{&S94999[30], 6}, {NULL}};
const Segment SGI224490[] = {{&S94999[31], 3}, {NULL}};
const Segment SGI224491[] = {{&S94999[83], 6}, {NULL}};
const Segment SGI224492[] = {{&S95003[34], 8}, {NULL}};
const Segment SGI224493[] = {{&S89021[42], 10}, {NULL}};
const Segment SGI224494[] = {{&S95003[27], 6}, {NULL}};
const Segment SGI224495[] = {{&S89021[42], 22}, {NULL}};
const Segment SGI224496[] = {{&S89021[30], 4}, {NULL}};
const Segment SGI224497[] = {{&S95003[95], 9}, {NULL}};
const Segment SGI224498[] = {{&S89021[113], 3}, {NULL}};
const Segment SGI224499[] = {{&S95003[68], 6}, {NULL}};
const Segment SGI224500[] = {{&S89021[78], 6}, {NULL}};
const Segment SGI224501[] = {{&S84309[44], 11}, {NULL}};
const Segment SGI224502[] = {{&S84309[44], 23}, {NULL}};
const Segment SGI224503[] = {{&S84309[30], 6}, {NULL}};
const Segment SGI224504[] = {{&S84309[116], 3}, {NULL}};
const Segment SGI224505[] = {{&S84309[81], 6}, {NULL}};
const Segment SGI224506[] = {{&S84309[148], 15}, {NULL}};
const Segment SGI224507[] = {{&S95006[59], 6}, {NULL}};
const Segment SGI224508[] = {{&S95006[59], 18}, {NULL}};
const Segment SGI224509[] = {{&S95006[45], 6}, {NULL}};
const Segment SGI224510[] = {{&S95006[46], 3}, {NULL}};
const Segment SGI224511[] = {{&S95006[91], 6}, {NULL}};
const Segment SGI224512[] = {{&S95006[0], 14}, {NULL}};
const Segment SGI224513[] = {{&S95010[34], 7}, {NULL}};
const Segment SGI224514[] = {{&S95010[27], 6}, {NULL}};
const Segment SGI224515[] = {{&S95010[90], 9}, {NULL}};
const Segment SGI224516[] = {{&S95010[63], 6}, {NULL}};
const Segment SGI224517[] = {{&S95011[34], 11}, {NULL}};
const Segment SGI224518[] = {{&S95011[27], 6}, {NULL}};
const Segment SGI224519[] = {{&S95011[98], 9}, {NULL}};
const Segment SGI224520[] = {{&S95011[71], 6}, {NULL}};
const Segment SGI224521[] = {{&S84319[34], 11}, {NULL}};
const Segment SGI224522[] = {{&S84319[27], 6}, {NULL}};
const Segment SGI224523[] = {{&S95013[34], 7}, {NULL}};
const Segment SGI224524[] = {{&S84319[98], 9}, {NULL}};
const Segment SGI224525[] = {{&S95013[27], 6}, {NULL}};
const Segment SGI224526[] = {{&S84319[71], 6}, {NULL}};
const Segment SGI224527[] = {{&S95013[94], 9}, {NULL}};
const Segment SGI224528[] = {{&S95013[67], 6}, {NULL}};
const Segment SGI224529[] = {{&S84322[44], 10}, {NULL}};
const Segment SGI224530[] = {{&S84322[44], 22}, {NULL}};
const Segment SGI224531[] = {{&S84322[30], 6}, {NULL}};
const Segment SGI224532[] = {{&S84322[113], 3}, {NULL}};
const Segment SGI224533[] = {{&S84322[80], 5}, {NULL}};
const Segment SGI224534[] = {{&S84322[117], 14}, {NULL}};
const Segment SGI224535[] = {{&S84324[44], 10}, {NULL}};
const Segment SGI224536[] = {{&S84324[44], 22}, {NULL}};
const Segment SGI224537[] = {{&S84324[30], 6}, {NULL}};
const Segment SGI224538[] = {{&S84324[115], 3}, {NULL}};
const Segment SGI224539[] = {{&S84324[80], 6}, {NULL}};
const Segment SGI224540[] = {{&S84324[140], 13}, {NULL}};
const Segment SGI224541[] = {{&S95022[34], 7}, {NULL}};
const Segment SGI224542[] = {{&S95022[27], 6}, {NULL}};
const Segment SGI224543[] = {{&S84326[44], 11}, {NULL}};
const Segment SGI224544[] = {{&S84326[44], 18}, {NULL}};
const Segment SGI224545[] = {{&S84326[30], 6}, {NULL}};
const Segment SGI224546[] = {{&S95022[94], 9}, {NULL}};
const Segment SGI224547[] = {{&S95022[67], 6}, {NULL}};
const Segment SGI224548[] = {{&S84326[111], 3}, {NULL}};
const Segment SGI224549[] = {{&S84326[76], 6}, {NULL}};
const Segment SGI224550[] = {{&S95024[34], 10}, {NULL}};
const Segment SGI224551[] = {{&S95024[27], 6}, {NULL}};
const Segment SGI224552[] = {{&S90970[44], 11}, {NULL}};
const Segment SGI224553[] = {{&S90970[44], 23}, {NULL}};
const Segment SGI224554[] = {{&S90970[30], 6}, {NULL}};
const Segment SGI224555[] = {{&S95024[97], 9}, {NULL}};
const Segment SGI224556[] = {{&S95024[70], 6}, {NULL}};
const Segment SGI224557[] = {{&S90970[145], 3}, {NULL}};
const Segment SGI224558[] = {{&S90970[110], 6}, {NULL}};
const Segment SGI224559[] = {{&S90970[69], 8}, {NULL}};
const Segment SGI224560[] = {{&S90970[81], 15}, {NULL}};
const Segment SGI224561[] = {{&S85995[44], 15}, {NULL}};
const Segment SGI224562[] = {{&S85995[44], 27}, {NULL}};
const Segment SGI224563[] = {{&S85995[30], 6}, {NULL}};
const Segment SGI224564[] = {{&S85995[31], 3}, {NULL}};
const Segment SGI224565[] = {{&S85995[85], 6}, {NULL}};
const Segment SGI224566[] = {{&S95028[34], 9}, {NULL}};
const Segment SGI224567[] = {{&S95028[27], 6}, {NULL}};
const Segment SGI224568[] = {{&S95028[96], 9}, {NULL}};
const Segment SGI224569[] = {{&S95028[69], 6}, {NULL}};
const Segment SGI224570[] = {{&S95028[137], 8}, {NULL}};
const Segment SGI224571[] = {{&S95030[34], 10}, {NULL}};
const Segment SGI224572[] = {{&S95030[27], 6}, {NULL}};
const Segment SGI224573[] = {{&S95030[97], 9}, {NULL}};
const Segment SGI224574[] = {{&S95030[70], 6}, {NULL}};
const Segment SGI224575[] = {{&S90977[44], 10}, {NULL}};
const Segment SGI224576[] = {{&S90977[44], 22}, {NULL}};
const Segment SGI224577[] = {{&S90977[30], 6}, {NULL}};
const Segment SGI224578[] = {{&S90977[115], 3}, {NULL}};
const Segment SGI224579[] = {{&S90977[80], 6}, {NULL}};
const Segment SGI224580[] = {{&S95036[34], 16}, {NULL}};
const Segment SGI224581[] = {{&S95036[27], 6}, {NULL}};
const Segment SGI224582[] = {{&S95036[113], 9}, {NULL}};
const Segment SGI224583[] = {{&S95036[86], 6}, {NULL}};
const Segment SGI224584[] = {{&S95037[34], 11}, {NULL}};
const Segment SGI224585[] = {{&S95037[27], 6}, {NULL}};
const Segment SGI224586[] = {{&S95037[98], 9}, {NULL}};
const Segment SGI224587[] = {{&S95037[71], 6}, {NULL}};
const Segment SGI224588[] = {{&S95037[139], 8}, {NULL}};
const Segment SGI224589[] = {{&S90987[44], 10}, {NULL}};
const Segment SGI224590[] = {{&S90987[44], 22}, {NULL}};
const Segment SGI224591[] = {{&S90987[30], 6}, {NULL}};
const Segment SGI224592[] = {{&S90987[113], 3}, {NULL}};
const Segment SGI224593[] = {{&S95039[34], 6}, {NULL}};
const Segment SGI224594[] = {{&S90987[80], 5}, {NULL}};
const Segment SGI224595[] = {{&S95039[27], 6}, {NULL}};
const Segment SGI224596[] = {{&S90987[117], 14}, {NULL}};
const Segment SGI224597[] = {{&S95039[93], 9}, {NULL}};
const Segment SGI224598[] = {{&S95039[66], 6}, {NULL}};
const Segment SGI224599[] = {{&S95041[44], 14}, {NULL}};
const Segment SGI224600[] = {{&S95041[44], 31}, {NULL}};
const Segment SGI224601[] = {{&S95041[30], 6}, {NULL}};
const Segment SGI224602[] = {{&S95041[31], 3}, {NULL}};
const Segment SGI224603[] = {{&S95041[89], 5}, {NULL}};
const Segment SGI224604[] = {{&S95041[126], 14}, {NULL}};
const Segment SGI224605[] = {{&S95044[44], 9}, {NULL}};
const Segment SGI224606[] = {{&S95044[44], 22}, {NULL}};
const Segment SGI224607[] = {{&S95044[30], 6}, {NULL}};
const Segment SGI224608[] = {{&S95044[31], 3}, {NULL}};
const Segment SGI224609[] = {{&S95044[80], 6}, {NULL}};
const Segment SGI224610[] = {{&S95054[34], 8}, {NULL}};
const Segment SGI224611[] = {{&S95054[27], 6}, {NULL}};
const Segment SGI224612[] = {{&S95054[95], 9}, {NULL}};
const Segment SGI224613[] = {{&S95054[68], 6}, {NULL}};
const Segment SGI224614[] = {{&S95056[44], 12}, {NULL}};
const Segment SGI224615[] = {{&S95056[44], 24}, {NULL}};
const Segment SGI224616[] = {{&S95056[30], 6}, {NULL}};
const Segment SGI224617[] = {{&S95056[31], 3}, {NULL}};
const Segment SGI224618[] = {{&S95056[82], 6}, {NULL}};
const Segment SGI224619[] = {{&S95056[142], 7}, {NULL}};
const Segment SGI224620[] = {{&S95057[44], 10}, {NULL}};
const Segment SGI224621[] = {{&S95057[44], 22}, {NULL}};
const Segment SGI224622[] = {{&S95057[30], 6}, {NULL}};
const Segment SGI224623[] = {{&S95057[31], 3}, {NULL}};
const Segment SGI224624[] = {{&S95057[80], 6}, {NULL}};
const Segment SGI224625[] = {{&S95057[140], 7}, {NULL}};
const Segment SGI224626[] = {{&S91015[34], 6}, {NULL}};
const Segment SGI224627[] = {{&S91015[27], 6}, {NULL}};
const Segment SGI224628[] = {{&S91015[93], 9}, {NULL}};
const Segment SGI224629[] = {{&S91015[66], 6}, {NULL}};
const Segment SGI224630[] = {{&S95062[44], 15}, {NULL}};
const Segment SGI224631[] = {{&S95062[44], 27}, {NULL}};
const Segment SGI224632[] = {{&S95062[30], 6}, {NULL}};
const Segment SGI224633[] = {{&S95062[31], 3}, {NULL}};
const Segment SGI224634[] = {{&S95062[85], 6}, {NULL}};
const Segment SGI224635[] = {{&S95062[145], 7}, {NULL}};
const Segment SGI224636[] = {{&S91019[34], 11}, {NULL}};
const Segment SGI224637[] = {{&S95063[34], 6}, {NULL}};
const Segment SGI224638[] = {{&S91019[27], 6}, {NULL}};
const Segment SGI224639[] = {{&S95063[27], 6}, {NULL}};
const Segment SGI224640[] = {{&S91019[98], 9}, {NULL}};
const Segment SGI224641[] = {{&S91019[71], 6}, {NULL}};
const Segment SGI224642[] = {{&S95063[93], 9}, {NULL}};
const Segment SGI224643[] = {{&S95063[66], 6}, {NULL}};
const Segment SGI224644[] = {{&S95068[52], 6}, {NULL}};
const Segment SGI224645[] = {{&S95068[52], 18}, {NULL}};
const Segment SGI224646[] = {{&S95068[38], 6}, {NULL}};
const Segment SGI224647[] = {{&S95068[84], 5}, {NULL}};
const Segment SGI224648[] = {{&S95068[0], 14}, {NULL}};
const Segment SGI224649[] = {{&S95069[44], 13}, {NULL}};
const Segment SGI224650[] = {{&S95069[44], 25}, {NULL}};
const Segment SGI224651[] = {{&S95069[30], 6}, {NULL}};
const Segment SGI224652[] = {{&S95069[83], 6}, {NULL}};
const Segment SGI224653[] = {{&S95069[110], 13}, {NULL}};
const Segment SGI224654[] = {{&S95070[52], 11}, {NULL}};
const Segment SGI224655[] = {{&S95070[52], 23}, {NULL}};
const Segment SGI224656[] = {{&S95070[38], 6}, {NULL}};
const Segment SGI224657[] = {{&S95070[89], 5}, {NULL}};
const Segment SGI224658[] = {{&S95070[0], 14}, {NULL}};
const Segment SGI224659[] = {{&S95075[52], 6}, {NULL}};
const Segment SGI224660[] = {{&S95075[52], 18}, {NULL}};
const Segment SGI224661[] = {{&S95075[38], 6}, {NULL}};
const Segment SGI224662[] = {{&S95075[84], 5}, {NULL}};
const Segment SGI224663[] = {{&S95075[0], 14}, {NULL}};
const Segment SGI224664[] = {{&S91037[34], 16}, {NULL}};
const Segment SGI224665[] = {{&S91037[27], 6}, {NULL}};
const Segment SGI224666[] = {{&S91037[103], 9}, {NULL}};
const Segment SGI224667[] = {{&S91037[76], 6}, {NULL}};
const Segment SGI224668[] = {{&S95085[34], 7}, {NULL}};
const Segment SGI224669[] = {{&S95085[27], 6}, {NULL}};
const Segment SGI224670[] = {{&S95085[93], 9}, {NULL}};
const Segment SGI224671[] = {{&S95085[66], 6}, {NULL}};
const Segment SGI224672[] = {{&S95086[34], 12}, {NULL}};
const Segment SGI224673[] = {{&S95086[27], 6}, {NULL}};
const Segment SGI224674[] = {{&S86084[44], 10}, {NULL}};
const Segment SGI224675[] = {{&S86084[44], 22}, {NULL}};
const Segment SGI224676[] = {{&S86084[30], 6}, {NULL}};
const Segment SGI224677[] = {{&S86084[115], 3}, {NULL}};
const Segment SGI224678[] = {{&S86084[80], 6}, {NULL}};
const Segment SGI224679[] = {{&S84429[34], 12}, {NULL}};
const Segment SGI224680[] = {{&S84429[27], 6}, {NULL}};
const Segment SGI224681[] = {{&S84429[99], 9}, {NULL}};
const Segment SGI224682[] = {{&S84429[72], 6}, {NULL}};
const Segment SGI224683[] = {{&S84430[34], 10}, {NULL}};
const Segment SGI224684[] = {{&S84430[27], 6}, {NULL}};
const Segment SGI224685[] = {{&S84430[97], 9}, {NULL}};
const Segment SGI224686[] = {{&S84430[70], 6}, {NULL}};
const Segment SGI224687[] = {{&S84431[34], 10}, {NULL}};
const Segment SGI224688[] = {{&S84431[27], 6}, {NULL}};
const Segment SGI224689[] = {{&S84431[97], 9}, {NULL}};
const Segment SGI224690[] = {{&S84431[70], 6}, {NULL}};
const Segment SGI224691[] = {{&S91061[44], 12}, {NULL}};
const Segment SGI224692[] = {{&S91061[44], 24}, {NULL}};
const Segment SGI224693[] = {{&S91061[30], 6}, {NULL}};
const Segment SGI224694[] = {{&S91061[82], 6}, {NULL}};
const Segment SGI224695[] = {{&S91061[109], 13}, {NULL}};
const Segment SGI224696[] = {{&S86092[44], 16}, {NULL}};
const Segment SGI224697[] = {{&S86092[44], 28}, {NULL}};
const Segment SGI224698[] = {{&S86092[30], 6}, {NULL}};
const Segment SGI224699[] = {{&S86092[121], 3}, {NULL}};
const Segment SGI224700[] = {{&S86092[86], 6}, {NULL}};
const Segment SGI224701[] = {{&S91064[34], 13}, {NULL}};
const Segment SGI224702[] = {{&S91064[27], 6}, {NULL}};
const Segment SGI224703[] = {{&S91064[100], 9}, {NULL}};
const Segment SGI224704[] = {{&S91064[73], 6}, {NULL}};
const Segment SGI224705[] = {{&S91064[141], 8}, {NULL}};
const Segment SGI224706[] = {{&S86096[34], 14}, {NULL}};
const Segment SGI224707[] = {{&S86096[27], 6}, {NULL}};
const Segment SGI224708[] = {{&S86096[101], 9}, {NULL}};
const Segment SGI224709[] = {{&S86096[74], 6}, {NULL}};
const Segment SGI224710[] = {{&S84440[59], 15}, {NULL}};
const Segment SGI224711[] = {{&S84440[59], 27}, {NULL}};
const Segment SGI224712[] = {{&S84440[45], 6}, {NULL}};
const Segment SGI224713[] = {{&S84440[135], 3}, {NULL}};
const Segment SGI224714[] = {{&S84440[100], 6}, {NULL}};
const Segment SGI224715[] = {{&S84440[11], 3}, {NULL}};
const Segment SGI224716[] = {{&S86104[44], 4}, {NULL}};
const Segment SGI224717[] = {{&S86104[44], 16}, {NULL}};
const Segment SGI224718[] = {{&S86104[30], 6}, {NULL}};
const Segment SGI224719[] = {{&S84447[52], 15}, {NULL}};
const Segment SGI224720[] = {{&S84447[52], 27}, {NULL}};
const Segment SGI224721[] = {{&S84447[38], 6}, {NULL}};
const Segment SGI224722[] = {{&S86104[74], 6}, {NULL}};
const Segment SGI224723[] = {{&S86104[111], 3}, {NULL}};
const Segment SGI224724[] = {{&S84447[93], 5}, {NULL}};
const Segment SGI224725[] = {{&S84447[11], 3}, {NULL}};
const Segment SGI224726[] = {{&S84466[34], 7}, {NULL}};
const Segment SGI224727[] = {{&S84466[27], 6}, {NULL}};
const Segment SGI224728[] = {{&S84466[95], 9}, {NULL}};
const Segment SGI224729[] = {{&S84466[68], 6}, {NULL}};
const Segment SGI224730[] = {{&S84469[34], 11}, {NULL}};
const Segment SGI224731[] = {{&S84469[27], 6}, {NULL}};
const Segment SGI224732[] = {{&S84469[98], 9}, {NULL}};
const Segment SGI224733[] = {{&S84469[71], 6}, {NULL}};
const Segment SGI224734[] = {{&S86152[44], 7}, {NULL}};
const Segment SGI224735[] = {{&S86152[44], 19}, {NULL}};
const Segment SGI224736[] = {{&S86152[30], 6}, {NULL}};
const Segment SGI224737[] = {{&S86152[130], 3}, {NULL}};
const Segment SGI224738[] = {{&S86152[95], 6}, {NULL}};
const Segment SGI224739[] = {{&S86152[65], 8}, {NULL}};
const Segment SGI224740[] = {{&S84520[44], 11}, {NULL}};
const Segment SGI224741[] = {{&S84520[44], 23}, {NULL}};
const Segment SGI224742[] = {{&S84520[30], 6}, {NULL}};
const Segment SGI224743[] = {{&S84520[132], 3}, {NULL}};
const Segment SGI224744[] = {{&S84520[97], 6}, {NULL}};
const Segment SGI224745[] = {{&S86171[44], 11}, {NULL}};
const Segment SGI224746[] = {{&S84520[69], 8}, {NULL}};
const Segment SGI224747[] = {{&S86171[44], 23}, {NULL}};
const Segment SGI224748[] = {{&S86171[30], 6}, {NULL}};
const Segment SGI224749[] = {{&S86171[116], 3}, {NULL}};
const Segment SGI224750[] = {{&S86171[81], 6}, {NULL}};
const Segment SGI224751[] = {{&S84521[44], 11}, {NULL}};
const Segment SGI224752[] = {{&S84521[44], 23}, {NULL}};
const Segment SGI224753[] = {{&S84521[30], 6}, {NULL}};
const Segment SGI224754[] = {{&S84521[132], 3}, {NULL}};
const Segment SGI224755[] = {{&S84521[97], 6}, {NULL}};
const Segment SGI224756[] = {{&S84521[69], 8}, {NULL}};
const Segment SGI224757[] = {{&S91168[59], 10}, {NULL}};
const Segment SGI224758[] = {{&S91168[59], 22}, {NULL}};
const Segment SGI224759[] = {{&S91168[45], 6}, {NULL}};
const Segment SGI224760[] = {{&S91168[130], 3}, {NULL}};
const Segment SGI224761[] = {{&S91168[95], 6}, {NULL}};
const Segment SGI224762[] = {{&S91168[0], 14}, {NULL}};
const Segment SGI224763[] = {{&S91171[34], 11}, {NULL}};
const Segment SGI224764[] = {{&S91171[27], 6}, {NULL}};
const Segment SGI224765[] = {{&S91171[98], 9}, {NULL}};
const Segment SGI224766[] = {{&S91171[71], 6}, {NULL}};
const Segment SGI224767[] = {{&S86193[34], 17}, {NULL}};
const Segment SGI224768[] = {{&S86193[27], 6}, {NULL}};
const Segment SGI224769[] = {{&S86193[104], 9}, {NULL}};
const Segment SGI224770[] = {{&S86193[77], 6}, {NULL}};
const Segment SGI224771[] = {{&S84546[34], 16}, {NULL}};
const Segment SGI224772[] = {{&S91183[34], 6}, {NULL}};
const Segment SGI224773[] = {{&S84546[27], 6}, {NULL}};
const Segment SGI224774[] = {{&S91183[27], 6}, {NULL}};
const Segment SGI224775[] = {{&S84546[110], 9}, {NULL}};
const Segment SGI224776[] = {{&S91183[93], 9}, {NULL}};
const Segment SGI224777[] = {{&S84546[83], 6}, {NULL}};
const Segment SGI224778[] = {{&S91183[66], 6}, {NULL}};
const Segment SGI224779[] = {{&S84557[34], 10}, {NULL}};
const Segment SGI224780[] = {{&S84557[27], 6}, {NULL}};
const Segment SGI224781[] = {{&S84557[97], 9}, {NULL}};
const Segment SGI224782[] = {{&S84557[70], 6}, {NULL}};
const Segment SGI224783[] = {{&S84558[34], 11}, {NULL}};
const Segment SGI224784[] = {{&S84558[27], 6}, {NULL}};
const Segment SGI224785[] = {{&S84558[98], 9}, {NULL}};
const Segment SGI224786[] = {{&S84558[71], 6}, {NULL}};
const Segment SGI224787[] = {{&S91196[34], 16}, {NULL}};
const Segment SGI224788[] = {{&S91196[27], 6}, {NULL}};
const Segment SGI224789[] = {{&S91196[103], 9}, {NULL}};
const Segment SGI224790[] = {{&S91196[76], 6}, {NULL}};
const Segment SGI224791[] = {{&S84559[44], 10}, {NULL}};
const Segment SGI224792[] = {{&S84559[44], 22}, {NULL}};
const Segment SGI224793[] = {{&S84559[30], 6}, {NULL}};
const Segment SGI224794[] = {{&S84559[80], 6}, {NULL}};
const Segment SGI224795[] = {{&S84559[107], 13}, {NULL}};
const Segment SGI224796[] = {{&S84560[44], 6}, {NULL}};
const Segment SGI224797[] = {{&S84560[44], 26}, {NULL}};
const Segment SGI224798[] = {{&S84560[30], 6}, {NULL}};
const Segment SGI224799[] = {{&S84560[84], 6}, {NULL}};
const Segment SGI224800[] = {{&S84560[111], 13}, {NULL}};
const Segment SGI224801[] = {{&S84561[44], 14}, {NULL}};
const Segment SGI224802[] = {{&S84561[44], 31}, {NULL}};
const Segment SGI224803[] = {{&S84561[30], 6}, {NULL}};
const Segment SGI224804[] = {{&S91201[34], 10}, {NULL}};
const Segment SGI224805[] = {{&S91201[27], 6}, {NULL}};
const Segment SGI224806[] = {{&S84561[89], 6}, {NULL}};
const Segment SGI224807[] = {{&S91201[97], 9}, {NULL}};
const Segment SGI224808[] = {{&S84561[116], 13}, {NULL}};
const Segment SGI224809[] = {{&S91201[70], 6}, {NULL}};
const Segment SGI224810[] = {{&S95086[112], 9}, {NULL}};
const Segment SGI224811[] = {{&S95086[85], 6}, {NULL}};
const Segment SGI224812[] = {{&S95088[34], 7}, {NULL}};
const Segment SGI224813[] = {{&S95088[27], 6}, {NULL}};
const Segment SGI224814[] = {{&S95088[99], 9}, {NULL}};
const Segment SGI224815[] = {{&S95088[72], 6}, {NULL}};
const Segment SGI224816[] = {{&S91218[44], 14}, {NULL}};
const Segment SGI224817[] = {{&S91218[44], 25}, {NULL}};
const Segment SGI224818[] = {{&S91218[30], 6}, {NULL}};
const Segment SGI224819[] = {{&S91218[118], 3}, {NULL}};
const Segment SGI224820[] = {{&S91218[83], 6}, {NULL}};
const Segment SGI224821[] = {{&S95090[34], 10}, {NULL}};
const Segment SGI224822[] = {{&S95090[27], 6}, {NULL}};
const Segment SGI224823[] = {{&S95090[108], 9}, {NULL}};
const Segment SGI224824[] = {{&S95090[81], 6}, {NULL}};
const Segment SGI224825[] = {{&S91229[44], 25}, {NULL}};
const Segment SGI224826[] = {{&S91229[44], 37}, {NULL}};
const Segment SGI224827[] = {{&S91229[30], 6}, {NULL}};
const Segment SGI224828[] = {{&S91229[95], 6}, {NULL}};
const Segment SGI224829[] = {{&S91229[122], 13}, {NULL}};
const Segment SGI224830[] = {{&S95107[34], 7}, {NULL}};
const Segment SGI224831[] = {{&S95107[27], 6}, {NULL}};
const Segment SGI224832[] = {{&S95107[99], 9}, {NULL}};
const Segment SGI224833[] = {{&S95107[72], 6}, {NULL}};
const Segment SGI224834[] = {{&S91236[44], 15}, {NULL}};
const Segment SGI224835[] = {{&S91236[44], 27}, {NULL}};
const Segment SGI224836[] = {{&S91236[30], 6}, {NULL}};
const Segment SGI224837[] = {{&S91236[85], 6}, {NULL}};
const Segment SGI224838[] = {{&S91236[112], 13}, {NULL}};
const Segment SGI224839[] = {{&S91248[34], 12}, {NULL}};
const Segment SGI224840[] = {{&S91248[27], 6}, {NULL}};
const Segment SGI224841[] = {{&S86267[34], 9}, {NULL}};
const Segment SGI224842[] = {{&S91248[99], 9}, {NULL}};
const Segment SGI224843[] = {{&S91248[72], 6}, {NULL}};
const Segment SGI224844[] = {{&S86267[27], 6}, {NULL}};
const Segment SGI224845[] = {{&S91248[133], 8}, {NULL}};
const Segment SGI224846[] = {{&S86267[96], 9}, {NULL}};
const Segment SGI224847[] = {{&S86267[69], 6}, {NULL}};
const Segment SGI224848[] = {{&S77086[34], 8}, {NULL}};
const Segment SGI224849[] = {{&S77086[27], 6}, {NULL}};
const Segment SGI224850[] = {{&S77086[94], 9}, {NULL}};
const Segment SGI224851[] = {{&S77086[67], 6}, {NULL}};
const Segment SGI224852[] = {{&S91262[34], 12}, {NULL}};
const Segment SGI224853[] = {{&S91262[27], 6}, {NULL}};
const Segment SGI224854[] = {{&S91262[99], 9}, {NULL}};
const Segment SGI224855[] = {{&S91262[72], 6}, {NULL}};
const Segment SGI224856[] = {{&S86288[44], 29}, {NULL}};
const Segment SGI224857[] = {{&S86288[44], 41}, {NULL}};
const Segment SGI224858[] = {{&S86288[30], 6}, {NULL}};
const Segment SGI224859[] = {{&S86288[134], 3}, {NULL}};
const Segment SGI224860[] = {{&S86288[99], 6}, {NULL}};
const Segment SGI224861[] = {{&S86289[44], 29}, {NULL}};
const Segment SGI224862[] = {{&S86289[44], 41}, {NULL}};
const Segment SGI224863[] = {{&S86289[30], 6}, {NULL}};
const Segment SGI224864[] = {{&S86289[134], 3}, {NULL}};
const Segment SGI224865[] = {{&S86289[99], 6}, {NULL}};
const Segment SGI224866[] = {{&S86289[152], 11}, {NULL}};
const Segment SGI224867[] = {{&S86296[44], 16}, {NULL}};
const Segment SGI224868[] = {{&S86296[44], 28}, {NULL}};
const Segment SGI224869[] = {{&S86296[30], 6}, {NULL}};
const Segment SGI224870[] = {{&S86296[86], 6}, {NULL}};
const Segment SGI224871[] = {{&S86296[123], 3}, {NULL}};
const Segment SGI224872[] = {{&S86297[34], 25}, {NULL}};
const Segment SGI224873[] = {{&S86297[27], 6}, {NULL}};
const Segment SGI224874[] = {{&S86297[112], 9}, {NULL}};
const Segment SGI224875[] = {{&S86297[85], 6}, {NULL}};
const Segment SGI224876[] = {{&S77107[34], 11}, {NULL}};
const Segment SGI224877[] = {{&S77107[27], 6}, {NULL}};
const Segment SGI224878[] = {{&S77107[97], 9}, {NULL}};
const Segment SGI224879[] = {{&S77107[70], 6}, {NULL}};
const Segment SGI224880[] = {{&S86301[44], 19}, {NULL}};
const Segment SGI224881[] = {{&S86301[44], 31}, {NULL}};
const Segment SGI224882[] = {{&S86301[30], 6}, {NULL}};
const Segment SGI224883[] = {{&S86301[124], 3}, {NULL}};
const Segment SGI224884[] = {{&S86301[89], 6}, {NULL}};
const Segment SGI224885[] = {{&S77119[34], 17}, {NULL}};
const Segment SGI224886[] = {{&S77119[27], 6}, {NULL}};
const Segment SGI224887[] = {{&S77119[103], 9}, {NULL}};
const Segment SGI224888[] = {{&S77119[76], 6}, {NULL}};
const Segment SGI224889[] = {{&S77122[34], 10}, {NULL}};
const Segment SGI224890[] = {{&S77122[27], 6}, {NULL}};
const Segment SGI224891[] = {{&S77122[96], 9}, {NULL}};
const Segment SGI224892[] = {{&S77122[69], 6}, {NULL}};
const Segment SGI224893[] = {{&S77123[34], 11}, {NULL}};
const Segment SGI224894[] = {{&S77123[27], 6}, {NULL}};
const Segment SGI224895[] = {{&S77123[97], 9}, {NULL}};
const Segment SGI224896[] = {{&S77123[70], 6}, {NULL}};
const Segment SGI224897[] = {{&S77124[34], 11}, {NULL}};
const Segment SGI224898[] = {{&S77124[27], 6}, {NULL}};
const Segment SGI224899[] = {{&S77124[97], 9}, {NULL}};
const Segment SGI224900[] = {{&S77124[70], 6}, {NULL}};
const Segment SGI224901[] = {{&S77125[34], 10}, {NULL}};
const Segment SGI224902[] = {{&S77125[27], 6}, {NULL}};
const Segment SGI224903[] = {{&S77125[96], 9}, {NULL}};
const Segment SGI224904[] = {{&S77125[69], 6}, {NULL}};
const Segment SGI224905[] = {{&S95109[34], 13}, {NULL}};
const Segment SGI224906[] = {{&S95109[27], 6}, {NULL}};
const Segment SGI224907[] = {{&S95109[100], 9}, {NULL}};
const Segment SGI224908[] = {{&S95109[73], 6}, {NULL}};
const Segment SGI224909[] = {{&S95113[34], 12}, {NULL}};
const Segment SGI224910[] = {{&S95113[27], 6}, {NULL}};
const Segment SGI224911[] = {{&S95113[99], 9}, {NULL}};
const Segment SGI224912[] = {{&S95113[72], 6}, {NULL}};
const Segment SGI224913[] = {{&S91381[44], 10}, {NULL}};
const Segment SGI224914[] = {{&S91381[44], 22}, {NULL}};
const Segment SGI224915[] = {{&S91381[30], 6}, {NULL}};
const Segment SGI224916[] = {{&S91381[113], 3}, {NULL}};
const Segment SGI224917[] = {{&S91381[80], 5}, {NULL}};
const Segment SGI224918[] = {{&S91381[117], 14}, {NULL}};
const Segment SGI224919[] = {{&S95115[34], 4}, {NULL}};
const Segment SGI224920[] = {{&S95115[27], 6}, {NULL}};
const Segment SGI224921[] = {{&S95115[91], 9}, {NULL}};
const Segment SGI224922[] = {{&S95115[64], 6}, {NULL}};
const Segment SGI224923[] = {{&S95116[44], 10}, {NULL}};
const Segment SGI224924[] = {{&S95116[44], 22}, {NULL}};
const Segment SGI224925[] = {{&S95116[30], 6}, {NULL}};
const Segment SGI224926[] = {{&S91384[34], 10}, {NULL}};
const Segment SGI224927[] = {{&S91384[27], 6}, {NULL}};
const Segment SGI224928[] = {{&S95116[80], 6}, {NULL}};
const Segment SGI224929[] = {{&S95116[107], 13}, {NULL}};
const Segment SGI224930[] = {{&S91384[97], 9}, {NULL}};
const Segment SGI224931[] = {{&S91384[70], 6}, {NULL}};
const Segment SGI224932[] = {{&S86402[34], 15}, {NULL}};
const Segment SGI224933[] = {{&S86402[27], 6}, {NULL}};
const Segment SGI224934[] = {{&S86402[102], 9}, {NULL}};
const Segment SGI224935[] = {{&S86402[75], 6}, {NULL}};
const Segment SGI224936[] = {{&S91394[34], 18}, {NULL}};
const Segment SGI224937[] = {{&S91394[27], 6}, {NULL}};
const Segment SGI224938[] = {{&S91394[105], 9}, {NULL}};
const Segment SGI224939[] = {{&S91394[78], 6}, {NULL}};
const Segment SGI224940[] = {{&S77208[44], 18}, {NULL}};
const Segment SGI224941[] = {{&S77208[44], 29}, {NULL}};
const Segment SGI224942[] = {{&S77208[30], 6}, {NULL}};
const Segment SGI224943[] = {{&S77208[122], 3}, {NULL}};
const Segment SGI224944[] = {{&S77208[87], 6}, {NULL}};
const Segment SGI224945[] = {{&S77208[147], 7}, {NULL}};
const Segment SGI224946[] = {{&S86409[34], 16}, {NULL}};
const Segment SGI224947[] = {{&S86409[27], 6}, {NULL}};
const Segment SGI224948[] = {{&S86409[103], 9}, {NULL}};
const Segment SGI224949[] = {{&S86409[76], 6}, {NULL}};
const Segment SGI224950[] = {{&S91401[44], 10}, {NULL}};
const Segment SGI224951[] = {{&S91401[44], 22}, {NULL}};
const Segment SGI224952[] = {{&S91401[30], 6}, {NULL}};
const Segment SGI224953[] = {{&S91401[115], 3}, {NULL}};
const Segment SGI224954[] = {{&S91401[80], 6}, {NULL}};
const Segment SGI224955[] = {{&S91402[34], 10}, {NULL}};
const Segment SGI224956[] = {{&S91402[27], 6}, {NULL}};
const Segment SGI224957[] = {{&S91402[97], 9}, {NULL}};
const Segment SGI224958[] = {{&S91402[70], 6}, {NULL}};
const Segment SGI224959[] = {{&S91411[44], 11}, {NULL}};
const Segment SGI224960[] = {{&S91411[44], 23}, {NULL}};
const Segment SGI224961[] = {{&S91411[30], 6}, {NULL}};
const Segment SGI224962[] = {{&S91411[116], 3}, {NULL}};
const Segment SGI224963[] = {{&S91411[81], 6}, {NULL}};
const Segment SGI224964[] = {{&S91416[44], 18}, {NULL}};
const Segment SGI224965[] = {{&S91416[44], 39}, {NULL}};
const Segment SGI224966[] = {{&S91416[30], 6}, {NULL}};
const Segment SGI224967[] = {{&S91416[97], 6}, {NULL}};
const Segment SGI224968[] = {{&S91416[124], 13}, {NULL}};
const Segment SGI224969[] = {{&S77234[44], 18}, {NULL}};
const Segment SGI224970[] = {{&S77234[44], 29}, {NULL}};
const Segment SGI224971[] = {{&S77234[30], 6}, {NULL}};
const Segment SGI224972[] = {{&S77234[140], 3}, {NULL}};
const Segment SGI224973[] = {{&S77234[105], 6}, {NULL}};
const Segment SGI224974[] = {{&S77234[75], 8}, {NULL}};
const Segment SGI224975[] = {{&S77235[44], 18}, {NULL}};
const Segment SGI224976[] = {{&S77235[44], 30}, {NULL}};
const Segment SGI224977[] = {{&S77235[30], 6}, {NULL}};
const Segment SGI224978[] = {{&S77235[141], 3}, {NULL}};
const Segment SGI224979[] = {{&S91423[44], 15}, {NULL}};
const Segment SGI224980[] = {{&S77235[106], 6}, {NULL}};
const Segment SGI224981[] = {{&S91423[44], 27}, {NULL}};
const Segment SGI224982[] = {{&S91423[30], 6}, {NULL}};
const Segment SGI224983[] = {{&S77235[76], 8}, {NULL}};
const Segment SGI224984[] = {{&S91423[120], 3}, {NULL}};
const Segment SGI224985[] = {{&S91423[85], 6}, {NULL}};
const Segment SGI224986[] = {{&S84637[34], 8}, {NULL}};
const Segment SGI224987[] = {{&S84637[27], 6}, {NULL}};
const Segment SGI224988[] = {{&S84637[94], 9}, {NULL}};
const Segment SGI224989[] = {{&S84637[67], 6}, {NULL}};
const Segment SGI224990[] = {{&S95173[44], 10}, {NULL}};
const Segment SGI224991[] = {{&S95173[44], 20}, {NULL}};
const Segment SGI224992[] = {{&S95173[30], 6}, {NULL}};
const Segment SGI224993[] = {{&S95173[31], 3}, {NULL}};
const Segment SGI224994[] = {{&S95173[78], 6}, {NULL}};
const Segment SGI224995[] = {{&S91428[44], 9}, {NULL}};
const Segment SGI224996[] = {{&S91428[44], 21}, {NULL}};
const Segment SGI224997[] = {{&S91428[30], 6}, {NULL}};
const Segment SGI224998[] = {{&S91428[114], 3}, {NULL}};
const Segment SGI224999[] = {{&S91428[79], 6}, {NULL}};
const Segment SGI225000[] = {{&S91428[139], 11}, {NULL}};
const Segment SGI225001[] = {{&S77246[44], 12}, {NULL}};
const Segment SGI225002[] = {{&S77246[44], 24}, {NULL}};
const Segment SGI225003[] = {{&S77246[30], 6}, {NULL}};
const Segment SGI225004[] = {{&S91433[34], 25}, {NULL}};
const Segment SGI225005[] = {{&S91433[27], 6}, {NULL}};
const Segment SGI225006[] = {{&S77246[117], 3}, {NULL}};
const Segment SGI225007[] = {{&S77246[82], 6}, {NULL}};
const Segment SGI225008[] = {{&S77246[142], 8}, {NULL}};
const Segment SGI225009[] = {{&S91433[112], 9}, {NULL}};
const Segment SGI225010[] = {{&S91433[85], 6}, {NULL}};
const Segment SGI225011[] = {{&S95188[44], 9}, {NULL}};
const Segment SGI225012[] = {{&S95188[44], 21}, {NULL}};
const Segment SGI225013[] = {{&S95188[30], 6}, {NULL}};
const Segment SGI225014[] = {{&S95188[31], 3}, {NULL}};
const Segment SGI225015[] = {{&S95188[108], 6}, {NULL}};
const Segment SGI225016[] = {{&S95188[67], 8}, {NULL}};
const Segment SGI225017[] = {{&S95188[79], 15}, {NULL}};
const Segment SGI225018[] = {{&S95189[44], 9}, {NULL}};
const Segment SGI225019[] = {{&S95189[44], 21}, {NULL}};
const Segment SGI225020[] = {{&S95189[30], 6}, {NULL}};
const Segment SGI225021[] = {{&S95189[31], 3}, {NULL}};
const Segment SGI225022[] = {{&S95189[108], 6}, {NULL}};
const Segment SGI225023[] = {{&S95189[67], 8}, {NULL}};
const Segment SGI225024[] = {{&S95189[79], 15}, {NULL}};
const Segment SGI225025[] = {{&S95212[59], 10}, {NULL}};
const Segment SGI225026[] = {{&S95212[59], 22}, {NULL}};
const Segment SGI225027[] = {{&S95212[45], 6}, {NULL}};
const Segment SGI225028[] = {{&S95212[46], 3}, {NULL}};
const Segment SGI225029[] = {{&S95212[95], 6}, {NULL}};
const Segment SGI225030[] = {{&S95212[0], 14}, {NULL}};
const Segment SGI225031[] = {{&S95213[59], 5}, {NULL}};
const Segment SGI225032[] = {{&S95213[59], 21}, {NULL}};
const Segment SGI225033[] = {{&S95213[45], 6}, {NULL}};
const Segment SGI225034[] = {{&S95213[46], 3}, {NULL}};
const Segment SGI225035[] = {{&S95213[94], 6}, {NULL}};
const Segment SGI225036[] = {{&S95213[0], 14}, {NULL}};
const Segment SGI225037[] = {{&S95231[34], 8}, {NULL}};
const Segment SGI225038[] = {{&S95231[27], 6}, {NULL}};
const Segment SGI225039[] = {{&S95231[95], 9}, {NULL}};
const Segment SGI225040[] = {{&S95231[68], 6}, {NULL}};
const Segment SGI225041[] = {{&S95232[34], 12}, {NULL}};
const Segment SGI225042[] = {{&S95232[27], 6}, {NULL}};
const Segment SGI225043[] = {{&S95232[112], 9}, {NULL}};
const Segment SGI225044[] = {{&S95232[85], 6}, {NULL}};
const Segment SGI225045[] = {{&S95234[34], 7}, {NULL}};
const Segment SGI225046[] = {{&S96095[55], 11}, {NULL}};
const Segment SGI225047[] = {{&S95234[27], 6}, {NULL}};
const Segment SGI225048[] = {{&S96095[55], 23}, {NULL}};
const Segment SGI225049[] = {{&S96095[43], 4}, {NULL}};
const Segment SGI225050[] = {{&S95234[94], 9}, {NULL}};
const Segment SGI225051[] = {{&S96095[127], 3}, {NULL}};
const Segment SGI225052[] = {{&S95234[67], 6}, {NULL}};
const Segment SGI225053[] = {{&S96095[92], 6}, {NULL}};
const Segment SGI225054[] = {{&S95236[34], 6}, {NULL}};
const Segment SGI225055[] = {{&S95236[27], 6}, {NULL}};
const Segment SGI225056[] = {{&S95236[93], 9}, {NULL}};
const Segment SGI225057[] = {{&S95236[66], 6}, {NULL}};
const Segment SGI225058[] = {{&S95237[44], 10}, {NULL}};
const Segment SGI225059[] = {{&S95237[44], 22}, {NULL}};
const Segment SGI225060[] = {{&S95237[30], 6}, {NULL}};
const Segment SGI225061[] = {{&S95237[80], 6}, {NULL}};
const Segment SGI225062[] = {{&S95237[107], 13}, {NULL}};
const Segment SGI225063[] = {{&S95239[34], 7}, {NULL}};
const Segment SGI225064[] = {{&S95239[27], 6}, {NULL}};
const Segment SGI225065[] = {{&S77311[34], 9}, {NULL}};
const Segment SGI225066[] = {{&S77311[27], 6}, {NULL}};
const Segment SGI225067[] = {{&S95239[93], 9}, {NULL}};
const Segment SGI225068[] = {{&S95239[66], 6}, {NULL}};
const Segment SGI225069[] = {{&S77311[96], 9}, {NULL}};
const Segment SGI225070[] = {{&S77311[69], 6}, {NULL}};
const Segment SGI225071[] = {{&S95240[34], 24}, {NULL}};
const Segment SGI225072[] = {{&S95240[27], 6}, {NULL}};
const Segment SGI225073[] = {{&S95240[111], 9}, {NULL}};
const Segment SGI225074[] = {{&S95240[84], 6}, {NULL}};
const Segment SGI225075[] = {{&S77315[34], 11}, {NULL}};
const Segment SGI225076[] = {{&S77315[27], 6}, {NULL}};
const Segment SGI225077[] = {{&S77315[97], 9}, {NULL}};
const Segment SGI225078[] = {{&S77315[70], 6}, {NULL}};
const Segment SGI225079[] = {{&S95248[34], 10}, {NULL}};
const Segment SGI225080[] = {{&S86523[34], 10}, {NULL}};
const Segment SGI225081[] = {{&S95248[27], 6}, {NULL}};
const Segment SGI225082[] = {{&S86523[27], 6}, {NULL}};
const Segment SGI225083[] = {{&S95248[97], 9}, {NULL}};
const Segment SGI225084[] = {{&S95248[70], 6}, {NULL}};
const Segment SGI225085[] = {{&S86523[97], 9}, {NULL}};
const Segment SGI225086[] = {{&S86523[70], 6}, {NULL}};
const Segment SGI225087[] = {{&S95251[34], 11}, {NULL}};
const Segment SGI225088[] = {{&S95251[27], 6}, {NULL}};
const Segment SGI225089[] = {{&S95251[98], 9}, {NULL}};
const Segment SGI225090[] = {{&S95251[71], 6}, {NULL}};
const Segment SGI225091[] = {{&S95258[34], 7}, {NULL}};
const Segment SGI225092[] = {{&S95258[27], 6}, {NULL}};
const Segment SGI225093[] = {{&S95258[94], 9}, {NULL}};
const Segment SGI225094[] = {{&S95258[67], 6}, {NULL}};
const Segment SGI225095[] = {{&S95260[34], 18}, {NULL}};
const Segment SGI225096[] = {{&S95260[27], 6}, {NULL}};
const Segment SGI225097[] = {{&S95260[105], 9}, {NULL}};
const Segment SGI225098[] = {{&S95260[78], 6}, {NULL}};
const Segment SGI225099[] = {{&S77342[34], 10}, {NULL}};
const Segment SGI225100[] = {{&S77342[27], 6}, {NULL}};
const Segment SGI225101[] = {{&S77342[96], 9}, {NULL}};
const Segment SGI225102[] = {{&S77342[69], 6}, {NULL}};
const Segment SGI225103[] = {{&S95263[34], 15}, {NULL}};
const Segment SGI225104[] = {{&S95263[27], 6}, {NULL}};
const Segment SGI225105[] = {{&S95263[102], 9}, {NULL}};
const Segment SGI225106[] = {{&S95263[75], 6}, {NULL}};
const Segment SGI225107[] = {{&S77344[34], 10}, {NULL}};
const Segment SGI225108[] = {{&S77344[27], 6}, {NULL}};
const Segment SGI225109[] = {{&S77344[96], 9}, {NULL}};
const Segment SGI225110[] = {{&S77344[69], 6}, {NULL}};
const Segment SGI225111[] = {{&S95264[34], 9}, {NULL}};
const Segment SGI225112[] = {{&S95264[27], 6}, {NULL}};
const Segment SGI225113[] = {{&S95264[103], 9}, {NULL}};
const Segment SGI225114[] = {{&S95264[76], 6}, {NULL}};
const Segment SGI225115[] = {{&S95265[44], 4}, {NULL}};
const Segment SGI225116[] = {{&S95265[44], 16}, {NULL}};
const Segment SGI225117[] = {{&S95265[30], 6}, {NULL}};
const Segment SGI225118[] = {{&S95265[31], 3}, {NULL}};
const Segment SGI225119[] = {{&S95265[74], 6}, {NULL}};
const Segment SGI225120[] = {{&S95267[34], 4}, {NULL}};
const Segment SGI225121[] = {{&S95267[27], 6}, {NULL}};
const Segment SGI225122[] = {{&S95267[91], 9}, {NULL}};
const Segment SGI225123[] = {{&S95267[64], 6}, {NULL}};
const Segment SGI225124[] = {{&S95268[34], 11}, {NULL}};
const Segment SGI225125[] = {{&S95268[27], 6}, {NULL}};
const Segment SGI225126[] = {{&S95268[98], 9}, {NULL}};
const Segment SGI225127[] = {{&S95268[71], 6}, {NULL}};
const Segment SGI225128[] = {{&S95269[34], 14}, {NULL}};
const Segment SGI225129[] = {{&S95269[27], 6}, {NULL}};
const Segment SGI225130[] = {{&S95269[106], 9}, {NULL}};
const Segment SGI225131[] = {{&S95269[79], 6}, {NULL}};
const Segment SGI225132[] = {{&S91551[44], 10}, {NULL}};
const Segment SGI225133[] = {{&S95269[147], 8}, {NULL}};
const Segment SGI225134[] = {{&S91551[44], 22}, {NULL}};
const Segment SGI225135[] = {{&S91551[30], 6}, {NULL}};
const Segment SGI225136[] = {{&S91551[113], 3}, {NULL}};
const Segment SGI225137[] = {{&S91551[80], 5}, {NULL}};
const Segment SGI225138[] = {{&S91551[117], 14}, {NULL}};
const Segment SGI225139[] = {{&S95270[34], 6}, {NULL}};
const Segment SGI225140[] = {{&S95270[27], 6}, {NULL}};
const Segment SGI225141[] = {{&S95270[93], 9}, {NULL}};
const Segment SGI225142[] = {{&S95270[66], 6}, {NULL}};
const Segment SGI225143[] = {{&S91556[34], 6}, {NULL}};
const Segment SGI225144[] = {{&S91556[27], 6}, {NULL}};
const Segment SGI225145[] = {{&S91556[93], 9}, {NULL}};
const Segment SGI225146[] = {{&S91556[66], 6}, {NULL}};
const Segment SGI225147[] = {{&S95275[44], 11}, {NULL}};
const Segment SGI225148[] = {{&S95275[44], 23}, {NULL}};
const Segment SGI225149[] = {{&S95275[30], 6}, {NULL}};
const Segment SGI225150[] = {{&S95275[31], 3}, {NULL}};
const Segment SGI225151[] = {{&S95275[81], 5}, {NULL}};
const Segment SGI225152[] = {{&S95276[34], 7}, {NULL}};
const Segment SGI225153[] = {{&S95276[27], 6}, {NULL}};
const Segment SGI225154[] = {{&S95276[94], 9}, {NULL}};
const Segment SGI225155[] = {{&S95276[67], 6}, {NULL}};
const Segment SGI225156[] = {{&S95278[44], 16}, {NULL}};
const Segment SGI225157[] = {{&S95278[30], 6}, {NULL}};
const Segment SGI225158[] = {{&S95278[31], 3}, {NULL}};
const Segment SGI225159[] = {{&S95278[88], 6}, {NULL}};
const Segment SGI225160[] = {{&S91566[34], 5}, {NULL}};
const Segment SGI225161[] = {{&S91566[27], 6}, {NULL}};
const Segment SGI225162[] = {{&S91566[92], 9}, {NULL}};
const Segment SGI225163[] = {{&S91566[65], 6}, {NULL}};
const Segment SGI225164[] = {{&S95283[34], 18}, {NULL}};
const Segment SGI225165[] = {{&S95283[27], 6}, {NULL}};
const Segment SGI225166[] = {{&S95283[105], 9}, {NULL}};
const Segment SGI225167[] = {{&S95283[78], 6}, {NULL}};
const Segment SGI225168[] = {{&S95284[44], 7}, {NULL}};
const Segment SGI225169[] = {{&S95284[44], 23}, {NULL}};
const Segment SGI225170[] = {{&S95284[30], 6}, {NULL}};
const Segment SGI225171[] = {{&S95284[31], 3}, {NULL}};
const Segment SGI225172[] = {{&S95284[81], 5}, {NULL}};
const Segment SGI225173[] = {{&S95284[118], 14}, {NULL}};
const Segment SGI225174[] = {{&S91575[34], 11}, {NULL}};
const Segment SGI225175[] = {{&S91575[27], 6}, {NULL}};
const Segment SGI225176[] = {{&S91575[98], 9}, {NULL}};
const Segment SGI225177[] = {{&S91575[71], 6}, {NULL}};
const Segment SGI225178[] = {{&S95291[34], 9}, {NULL}};
const Segment SGI225179[] = {{&S95291[27], 6}, {NULL}};
const Segment SGI225180[] = {{&S95291[96], 9}, {NULL}};
const Segment SGI225181[] = {{&S95291[69], 6}, {NULL}};
const Segment SGI225182[] = {{&S95295[34], 10}, {NULL}};
const Segment SGI225183[] = {{&S95295[27], 6}, {NULL}};
const Segment SGI225184[] = {{&S95295[105], 9}, {NULL}};
const Segment SGI225185[] = {{&S95295[78], 6}, {NULL}};
const Segment SGI225186[] = {{&S95297[59], 22}, {NULL}};
const Segment SGI225187[] = {{&S95297[59], 34}, {NULL}};
const Segment SGI225188[] = {{&S95297[45], 6}, {NULL}};
const Segment SGI225189[] = {{&S95297[46], 3}, {NULL}};
const Segment SGI225190[] = {{&S91588[44], 13}, {NULL}};
const Segment SGI225191[] = {{&S95297[107], 6}, {NULL}};
const Segment SGI225192[] = {{&S91588[44], 25}, {NULL}};
const Segment SGI225193[] = {{&S91588[30], 6}, {NULL}};
const Segment SGI225194[] = {{&S95297[0], 14}, {NULL}};
const Segment SGI225195[] = {{&S91588[83], 6}, {NULL}};
const Segment SGI225196[] = {{&S91588[110], 13}, {NULL}};
const Segment SGI225197[] = {{&S95298[34], 13}, {NULL}};
const Segment SGI225198[] = {{&S95298[27], 6}, {NULL}};
const Segment SGI225199[] = {{&S95298[100], 9}, {NULL}};
const Segment SGI225200[] = {{&S95298[73], 6}, {NULL}};
const Segment SGI225201[] = {{&S91589[34], 15}, {NULL}};
const Segment SGI225202[] = {{&S91589[27], 6}, {NULL}};
const Segment SGI225203[] = {{&S91589[102], 9}, {NULL}};
const Segment SGI225204[] = {{&S91589[75], 6}, {NULL}};
const Segment SGI225205[] = {{&S95299[34], 11}, {NULL}};
const Segment SGI225206[] = {{&S95299[27], 6}, {NULL}};
const Segment SGI225207[] = {{&S95299[98], 9}, {NULL}};
const Segment SGI225208[] = {{&S95299[71], 6}, {NULL}};
const Segment SGI225209[] = {{&S77403[44], 9}, {NULL}};
const Segment SGI225210[] = {{&S77403[44], 20}, {NULL}};
const Segment SGI225211[] = {{&S77403[30], 6}, {NULL}};
const Segment SGI225212[] = {{&S84725[44], 7}, {NULL}};
const Segment SGI225213[] = {{&S84725[44], 19}, {NULL}};
const Segment SGI225214[] = {{&S77403[113], 3}, {NULL}};
const Segment SGI225215[] = {{&S84725[30], 6}, {NULL}};
const Segment SGI225216[] = {{&S95300[34], 11}, {NULL}};
const Segment SGI225217[] = {{&S77403[78], 6}, {NULL}};
const Segment SGI225218[] = {{&S95300[27], 6}, {NULL}};
const Segment SGI225219[] = {{&S84725[112], 3}, {NULL}};
const Segment SGI225220[] = {{&S84725[77], 6}, {NULL}};
const Segment SGI225221[] = {{&S95300[98], 9}, {NULL}};
const Segment SGI225222[] = {{&S95300[71], 6}, {NULL}};
const Segment SGI225223[] = {{&S95301[34], 6}, {NULL}};
const Segment SGI225224[] = {{&S95301[27], 6}, {NULL}};
const Segment SGI225225[] = {{&S95301[93], 9}, {NULL}};
const Segment SGI225226[] = {{&S95301[66], 6}, {NULL}};
const Segment SGI225227[] = {{&S77406[44], 11}, {NULL}};
const Segment SGI225228[] = {{&S77406[44], 23}, {NULL}};
const Segment SGI225229[] = {{&S77406[30], 6}, {NULL}};
const Segment SGI225230[] = {{&S77406[116], 3}, {NULL}};
const Segment SGI225231[] = {{&S77406[81], 6}, {NULL}};
const Segment SGI225232[] = {{&S95308[44], 10}, {NULL}};
const Segment SGI225233[] = {{&S95308[44], 22}, {NULL}};
const Segment SGI225234[] = {{&S95308[30], 6}, {NULL}};
const Segment SGI225235[] = {{&S95308[31], 3}, {NULL}};
const Segment SGI225236[] = {{&S95308[80], 6}, {NULL}};
const Segment SGI225237[] = {{&S91602[34], 15}, {NULL}};
const Segment SGI225238[] = {{&S95308[140], 7}, {NULL}};
const Segment SGI225239[] = {{&S91602[27], 6}, {NULL}};
const Segment SGI225240[] = {{&S91602[102], 9}, {NULL}};
const Segment SGI225241[] = {{&S91602[75], 6}, {NULL}};
const Segment SGI225242[] = {{&S91602[143], 8}, {NULL}};
const Segment SGI225243[] = {{&S84739[34], 9}, {NULL}};
const Segment SGI225244[] = {{&S84739[27], 6}, {NULL}};
const Segment SGI225245[] = {{&S91606[44], 5}, {NULL}};
const Segment SGI225246[] = {{&S91606[44], 17}, {NULL}};
const Segment SGI225247[] = {{&S91606[30], 6}, {NULL}};
const Segment SGI225248[] = {{&S84739[96], 9}, {NULL}};
const Segment SGI225249[] = {{&S91606[110], 3}, {NULL}};
const Segment SGI225250[] = {{&S84739[69], 6}, {NULL}};
const Segment SGI225251[] = {{&S91606[75], 6}, {NULL}};
const Segment SGI225252[] = {{&S84740[34], 10}, {NULL}};
const Segment SGI225253[] = {{&S84740[27], 6}, {NULL}};
const Segment SGI225254[] = {{&S84740[97], 9}, {NULL}};
const Segment SGI225255[] = {{&S84740[70], 6}, {NULL}};
const Segment SGI225256[] = {{&S95317[44], 7}, {NULL}};
const Segment SGI225257[] = {{&S95317[44], 18}, {NULL}};
const Segment SGI225258[] = {{&S95317[30], 6}, {NULL}};
const Segment SGI225259[] = {{&S95317[31], 3}, {NULL}};
const Segment SGI225260[] = {{&S95317[76], 6}, {NULL}};
const Segment SGI225261[] = {{&S95317[136], 7}, {NULL}};
const Segment SGI225262[] = {{&S84742[44], 10}, {NULL}};
const Segment SGI225263[] = {{&S84742[44], 22}, {NULL}};
const Segment SGI225264[] = {{&S84742[30], 6}, {NULL}};
const Segment SGI225265[] = {{&S84742[131], 3}, {NULL}};
const Segment SGI225266[] = {{&S84742[96], 6}, {NULL}};
const Segment SGI225267[] = {{&S95318[44], 10}, {NULL}};
const Segment SGI225268[] = {{&S84742[68], 8}, {NULL}};
const Segment SGI225269[] = {{&S95318[44], 22}, {NULL}};
const Segment SGI225270[] = {{&S95318[30], 6}, {NULL}};
const Segment SGI225271[] = {{&S95318[31], 3}, {NULL}};
const Segment SGI225272[] = {{&S95318[80], 6}, {NULL}};
const Segment SGI225273[] = {{&S91620[44], 24}, {NULL}};
const Segment SGI225274[] = {{&S91620[44], 36}, {NULL}};
const Segment SGI225275[] = {{&S91620[30], 6}, {NULL}};
const Segment SGI225276[] = {{&S91620[128], 3}, {NULL}};
const Segment SGI225277[] = {{&S91620[94], 5}, {NULL}};
const Segment SGI225278[] = {{&S95328[34], 5}, {NULL}};
const Segment SGI225279[] = {{&S95328[27], 6}, {NULL}};
const Segment SGI225280[] = {{&S95328[92], 9}, {NULL}};
const Segment SGI225281[] = {{&S95328[65], 6}, {NULL}};
const Segment SGI225282[] = {{&S91621[44], 9}, {NULL}};
const Segment SGI225283[] = {{&S91621[44], 21}, {NULL}};
const Segment SGI225284[] = {{&S91621[30], 6}, {NULL}};
const Segment SGI225285[] = {{&S91621[114], 3}, {NULL}};
const Segment SGI225286[] = {{&S91621[79], 6}, {NULL}};
const Segment SGI225287[] = {{&S84754[34], 7}, {NULL}};
const Segment SGI225288[] = {{&S84754[27], 6}, {NULL}};
const Segment SGI225289[] = {{&S84754[94], 9}, {NULL}};
const Segment SGI225290[] = {{&S84754[67], 6}, {NULL}};
const Segment SGI225291[] = {{&S91625[44], 18}, {NULL}};
const Segment SGI225292[] = {{&S91625[44], 30}, {NULL}};
const Segment SGI225293[] = {{&S91625[30], 6}, {NULL}};
const Segment SGI225294[] = {{&S95334[34], 7}, {NULL}};
const Segment SGI225295[] = {{&S95334[27], 6}, {NULL}};
const Segment SGI225296[] = {{&S91625[123], 3}, {NULL}};
const Segment SGI225297[] = {{&S91625[88], 6}, {NULL}};
const Segment SGI225298[] = {{&S95334[95], 9}, {NULL}};
const Segment SGI225299[] = {{&S95334[68], 6}, {NULL}};
const Segment SGI225300[] = {{&S84760[34], 7}, {NULL}};
const Segment SGI225301[] = {{&S84760[27], 6}, {NULL}};
const Segment SGI225302[] = {{&S84760[104], 9}, {NULL}};
const Segment SGI225303[] = {{&S84760[77], 6}, {NULL}};
const Segment SGI225304[] = {{&S84760[145], 8}, {NULL}};
const Segment SGI225305[] = {{&S95341[34], 6}, {NULL}};
const Segment SGI225306[] = {{&S95341[27], 6}, {NULL}};
const Segment SGI225307[] = {{&S95341[93], 9}, {NULL}};
const Segment SGI225308[] = {{&S95341[66], 6}, {NULL}};
const Segment SGI225309[] = {{&S84763[44], 7}, {NULL}};
const Segment SGI225310[] = {{&S84763[44], 25}, {NULL}};
const Segment SGI225311[] = {{&S84763[30], 6}, {NULL}};
const Segment SGI225312[] = {{&S84763[83], 6}, {NULL}};
const Segment SGI225313[] = {{&S84763[110], 13}, {NULL}};
const Segment SGI225314[] = {{&S77453[34], 23}, {NULL}};
const Segment SGI225315[] = {{&S77453[27], 6}, {NULL}};
const Segment SGI225316[] = {{&S84764[44], 7}, {NULL}};
const Segment SGI225317[] = {{&S84764[44], 25}, {NULL}};
const Segment SGI225318[] = {{&S84764[30], 6}, {NULL}};
const Segment SGI225319[] = {{&S77453[110], 9}, {NULL}};
const Segment SGI225320[] = {{&S77453[83], 6}, {NULL}};
const Segment SGI225321[] = {{&S84764[83], 6}, {NULL}};
const Segment SGI225322[] = {{&S84764[110], 13}, {NULL}};
const Segment SGI225323[] = {{&S77503[34], 11}, {NULL}};
const Segment SGI225324[] = {{&S77503[27], 6}, {NULL}};
const Segment SGI225325[] = {{&S77503[98], 9}, {NULL}};
const Segment SGI225326[] = {{&S77503[71], 6}, {NULL}};
const Segment SGI225327[] = {{&S84812[45], 10}, {NULL}};
const Segment SGI225328[] = {{&S84812[30], 6}, {NULL}};
const Segment SGI225329[] = {{&S84812[115], 3}, {NULL}};
const Segment SGI225330[] = {{&S84812[81], 5}, {NULL}};
const Segment SGI225331[] = {{&S84812[139], 13}, {NULL}};
const Segment SGI225332[] = {{&S84828[34], 10}, {NULL}};
const Segment SGI225333[] = {{&S84828[27], 6}, {NULL}};
const Segment SGI225334[] = {{&S84828[97], 9}, {NULL}};
const Segment SGI225335[] = {{&S84828[70], 6}, {NULL}};
const Segment SGI225336[] = {{&S95433[57], 10}, {NULL}};
const Segment SGI225337[] = {{&S95433[57], 22}, {NULL}};
const Segment SGI225338[] = {{&S95433[43], 6}, {NULL}};
const Segment SGI225339[] = {{&S95433[44], 3}, {NULL}};
const Segment SGI225340[] = {{&S95433[93], 6}, {NULL}};
const Segment SGI225341[] = {{&S95433[0], 12}, {NULL}};
const Segment SGI225342[] = {{&S95449[45], 7}, {NULL}};
const Segment SGI225343[] = {{&S95449[30], 6}, {NULL}};
const Segment SGI225344[] = {{&S95449[31], 3}, {NULL}};
const Segment SGI225345[] = {{&S95449[83], 5}, {NULL}};
const Segment SGI225346[] = {{&S95449[141], 13}, {NULL}};
const Segment SGI225347[] = {{&S95450[45], 7}, {NULL}};
const Segment SGI225348[] = {{&S95450[30], 6}, {NULL}};
const Segment SGI225349[] = {{&S95450[31], 3}, {NULL}};
const Segment SGI225350[] = {{&S95450[83], 5}, {NULL}};
const Segment SGI225351[] = {{&S95450[141], 13}, {NULL}};
const Segment SGI225352[] = {{&S91751[44], 16}, {NULL}};
const Segment SGI225353[] = {{&S91751[44], 28}, {NULL}};
const Segment SGI225354[] = {{&S91751[30], 6}, {NULL}};
const Segment SGI225355[] = {{&S91751[121], 3}, {NULL}};
const Segment SGI225356[] = {{&S91751[86], 6}, {NULL}};
const Segment SGI225357[] = {{&S95454[44], 9}, {NULL}};
const Segment SGI225358[] = {{&S95454[44], 21}, {NULL}};
const Segment SGI225359[] = {{&S95454[30], 6}, {NULL}};
const Segment SGI225360[] = {{&S95454[31], 3}, {NULL}};
const Segment SGI225361[] = {{&S95454[79], 6}, {NULL}};
const Segment SGI225362[] = {{&S95456[44], 7}, {NULL}};
const Segment SGI225363[] = {{&S95456[44], 19}, {NULL}};
const Segment SGI225364[] = {{&S95456[30], 6}, {NULL}};
const Segment SGI225365[] = {{&S95456[31], 3}, {NULL}};
const Segment SGI225366[] = {{&S95456[77], 5}, {NULL}};
const Segment SGI225367[] = {{&S95456[122], 14}, {NULL}};
const Segment SGI225368[] = {{&S95458[59], 10}, {NULL}};
const Segment SGI225369[] = {{&S95458[59], 22}, {NULL}};
const Segment SGI225370[] = {{&S95458[45], 6}, {NULL}};
const Segment SGI225371[] = {{&S95458[46], 3}, {NULL}};
const Segment SGI225372[] = {{&S95458[95], 6}, {NULL}};
const Segment SGI225373[] = {{&S95458[0], 14}, {NULL}};
const Segment SGI225374[] = {{&S84879[34], 10}, {NULL}};
const Segment SGI225375[] = {{&S91763[34], 7}, {NULL}};
const Segment SGI225376[] = {{&S84879[27], 6}, {NULL}};
const Segment SGI225377[] = {{&S91763[27], 6}, {NULL}};
const Segment SGI225378[] = {{&S91763[94], 9}, {NULL}};
const Segment SGI225379[] = {{&S84879[97], 9}, {NULL}};
const Segment SGI225380[] = {{&S91763[67], 6}, {NULL}};
const Segment SGI225381[] = {{&S84879[70], 6}, {NULL}};
const Segment SGI225382[] = {{&S95463[44], 10}, {NULL}};
const Segment SGI225383[] = {{&S95463[44], 22}, {NULL}};
const Segment SGI225384[] = {{&S95463[30], 6}, {NULL}};
const Segment SGI225385[] = {{&S95463[31], 3}, {NULL}};
const Segment SGI225386[] = {{&S95463[80], 5}, {NULL}};
const Segment SGI225387[] = {{&S95463[125], 14}, {NULL}};
const Segment SGI225388[] = {{&S77560[44], 10}, {NULL}};
const Segment SGI225389[] = {{&S77560[44], 21}, {NULL}};
const Segment SGI225390[] = {{&S77560[30], 6}, {NULL}};
const Segment SGI225391[] = {{&S77560[114], 3}, {NULL}};
const Segment SGI225392[] = {{&S77560[79], 6}, {NULL}};
const Segment SGI225393[] = {{&S77560[138], 10}, {NULL}};
const Segment SGI225394[] = {{&S95467[59], 7}, {NULL}};
const Segment SGI225395[] = {{&S95467[59], 23}, {NULL}};
const Segment SGI225396[] = {{&S95467[45], 6}, {NULL}};
const Segment SGI225397[] = {{&S95467[46], 3}, {NULL}};
const Segment SGI225398[] = {{&S95467[96], 6}, {NULL}};
const Segment SGI225399[] = {{&S95467[0], 14}, {NULL}};
const Segment SGI225400[] = {{&S91771[34], 9}, {NULL}};
const Segment SGI225401[] = {{&S91771[27], 6}, {NULL}};
const Segment SGI225402[] = {{&S91771[96], 9}, {NULL}};
const Segment SGI225403[] = {{&S91771[69], 6}, {NULL}};
const Segment SGI225404[] = {{&S84889[34], 10}, {NULL}};
const Segment SGI225405[] = {{&S84889[27], 6}, {NULL}};
const Segment SGI225406[] = {{&S84889[97], 9}, {NULL}};
const Segment SGI225407[] = {{&S84889[70], 6}, {NULL}};
const Segment SGI225408[] = {{&S91774[44], 18}, {NULL}};
const Segment SGI225409[] = {{&S91774[44], 30}, {NULL}};
const Segment SGI225410[] = {{&S91774[30], 6}, {NULL}};
const Segment SGI225411[] = {{&S91774[123], 3}, {NULL}};
const Segment SGI225412[] = {{&S91774[88], 6}, {NULL}};
const Segment SGI225413[] = {{&S95472[34], 7}, {NULL}};
const Segment SGI225414[] = {{&S95472[27], 6}, {NULL}};
const Segment SGI225415[] = {{&S95472[94], 9}, {NULL}};
const Segment SGI225416[] = {{&S95472[67], 6}, {NULL}};
const Segment SGI225417[] = {{&S84892[34], 10}, {NULL}};
const Segment SGI225418[] = {{&S84892[27], 6}, {NULL}};
const Segment SGI225419[] = {{&S84892[97], 9}, {NULL}};
const Segment SGI225420[] = {{&S84892[70], 6}, {NULL}};
const Segment SGI225421[] = {{&S95473[34], 13}, {NULL}};
const Segment SGI225422[] = {{&S95473[27], 6}, {NULL}};
const Segment SGI225423[] = {{&S95473[100], 9}, {NULL}};
const Segment SGI225424[] = {{&S95473[73], 6}, {NULL}};
const Segment SGI225425[] = {{&S91784[34], 11}, {NULL}};
const Segment SGI225426[] = {{&S95479[59], 15}, {NULL}};
const Segment SGI225427[] = {{&S91784[27], 6}, {NULL}};
const Segment SGI225428[] = {{&S95479[59], 27}, {NULL}};
const Segment SGI225429[] = {{&S95479[45], 6}, {NULL}};
const Segment SGI225430[] = {{&S91784[98], 9}, {NULL}};
const Segment SGI225431[] = {{&S91784[71], 6}, {NULL}};
const Segment SGI225432[] = {{&S95479[46], 3}, {NULL}};
const Segment SGI225433[] = {{&S95479[100], 6}, {NULL}};
const Segment SGI225434[] = {{&S95479[0], 14}, {NULL}};
const Segment SGI225435[] = {{&S95480[45], 7}, {NULL}};
const Segment SGI225436[] = {{&S95480[30], 6}, {NULL}};
const Segment SGI225437[] = {{&S95480[31], 3}, {NULL}};
const Segment SGI225438[] = {{&S95480[83], 5}, {NULL}};
const Segment SGI225439[] = {{&S95480[141], 13}, {NULL}};
const Segment SGI225440[] = {{&S84905[44], 14}, {NULL}};
const Segment SGI225441[] = {{&S91789[34], 13}, {NULL}};
const Segment SGI225442[] = {{&S84905[44], 26}, {NULL}};
const Segment SGI225443[] = {{&S91789[27], 6}, {NULL}};
const Segment SGI225444[] = {{&S84905[30], 6}, {NULL}};
const Segment SGI225445[] = {{&S84905[119], 3}, {NULL}};
const Segment SGI225446[] = {{&S91789[100], 9}, {NULL}};
const Segment SGI225447[] = {{&S91789[73], 6}, {NULL}};
const Segment SGI225448[] = {{&S84905[84], 6}, {NULL}};
const Segment SGI225449[] = {{&S84905[151], 15}, {NULL}};
const Segment SGI225450[] = {{&S95483[44], 10}, {NULL}};
const Segment SGI225451[] = {{&S95483[44], 22}, {NULL}};
const Segment SGI225452[] = {{&S95483[30], 6}, {NULL}};
const Segment SGI225453[] = {{&S95483[158], 3}, {NULL}};
const Segment SGI225454[] = {{&S95483[124], 3}, {NULL}};
const Segment SGI225455[] = {{&S95483[89], 6}, {NULL}};
const Segment SGI225456[] = {{&S95483[144], 8}, {NULL}};
const Segment SGI225457[] = {{&S91792[44], 21}, {NULL}};
const Segment SGI225458[] = {{&S91792[44], 38}, {NULL}};
const Segment SGI225459[] = {{&S91792[30], 6}, {NULL}};
const Segment SGI225460[] = {{&S91792[137], 3}, {NULL}};
const Segment SGI225461[] = {{&S91792[103], 5}, {NULL}};
const Segment SGI225462[] = {{&S91792[82], 8}, {NULL}};
const Segment SGI225463[] = {{&S91793[44], 11}, {NULL}};
const Segment SGI225464[] = {{&S91793[44], 23}, {NULL}};
const Segment SGI225465[] = {{&S91793[30], 6}, {NULL}};
const Segment SGI225466[] = {{&S91793[84], 6}, {NULL}};
const Segment SGI225467[] = {{&S91794[44], 10}, {NULL}};
const Segment SGI225468[] = {{&S91794[30], 6}, {NULL}};
const Segment SGI225469[] = {{&S91794[103], 3}, {NULL}};
const Segment SGI225470[] = {{&S91794[68], 6}, {NULL}};
const Segment SGI225471[] = {{&S95488[34], 10}, {NULL}};
const Segment SGI225472[] = {{&S95488[27], 6}, {NULL}};
const Segment SGI225473[] = {{&S84913[44], 11}, {NULL}};
const Segment SGI225474[] = {{&S84913[44], 23}, {NULL}};
const Segment SGI225475[] = {{&S84913[30], 6}, {NULL}};
const Segment SGI225476[] = {{&S95488[97], 9}, {NULL}};
const Segment SGI225477[] = {{&S95488[70], 6}, {NULL}};
const Segment SGI225478[] = {{&S84913[132], 3}, {NULL}};
const Segment SGI225479[] = {{&S84913[97], 6}, {NULL}};
const Segment SGI225480[] = {{&S84913[69], 8}, {NULL}};
const Segment SGI225481[] = {{&S84914[44], 11}, {NULL}};
const Segment SGI225482[] = {{&S84914[44], 23}, {NULL}};
const Segment SGI225483[] = {{&S84914[30], 6}, {NULL}};
const Segment SGI225484[] = {{&S84914[132], 3}, {NULL}};
const Segment SGI225485[] = {{&S84914[97], 6}, {NULL}};
const Segment SGI225486[] = {{&S84914[69], 8}, {NULL}};
const Segment SGI225487[] = {{&S91797[44], 9}, {NULL}};
const Segment SGI225488[] = {{&S91797[44], 21}, {NULL}};
const Segment SGI225489[] = {{&S91797[30], 6}, {NULL}};
const Segment SGI225490[] = {{&S91797[114], 3}, {NULL}};
const Segment SGI225491[] = {{&S91797[79], 6}, {NULL}};
const Segment SGI225492[] = {{&S95493[44], 7}, {NULL}};
const Segment SGI225493[] = {{&S95493[44], 23}, {NULL}};
const Segment SGI225494[] = {{&S95493[30], 6}, {NULL}};
const Segment SGI225495[] = {{&S95493[31], 3}, {NULL}};
const Segment SGI225496[] = {{&S95493[81], 5}, {NULL}};
const Segment SGI225497[] = {{&S95493[118], 14}, {NULL}};
const Segment SGI225498[] = {{&S84920[34], 8}, {NULL}};
const Segment SGI225499[] = {{&S84920[27], 6}, {NULL}};
const Segment SGI225500[] = {{&S95495[44], 10}, {NULL}};
const Segment SGI225501[] = {{&S95495[44], 22}, {NULL}};
const Segment SGI225502[] = {{&S95495[30], 6}, {NULL}};
const Segment SGI225503[] = {{&S84920[95], 9}, {NULL}};
const Segment SGI225504[] = {{&S84920[68], 6}, {NULL}};
const Segment SGI225505[] = {{&S95495[31], 3}, {NULL}};
const Segment SGI225506[] = {{&S84920[129], 8}, {NULL}};
const Segment SGI225507[] = {{&S95495[80], 6}, {NULL}};
const Segment SGI225508[] = {{&S95495[140], 13}, {NULL}};
const Segment SGI225509[] = {{&S84922[34], 9}, {NULL}};
const Segment SGI225510[] = {{&S84922[27], 6}, {NULL}};
const Segment SGI225511[] = {{&S95497[34], 18}, {NULL}};
const Segment SGI225512[] = {{&S84922[96], 9}, {NULL}};
const Segment SGI225513[] = {{&S84922[69], 6}, {NULL}};
const Segment SGI225514[] = {{&S95497[27], 6}, {NULL}};
const Segment SGI225515[] = {{&S95497[105], 9}, {NULL}};
const Segment SGI225516[] = {{&S95497[78], 6}, {NULL}};
const Segment SGI225517[] = {{&S84923[44], 9}, {NULL}};
const Segment SGI225518[] = {{&S84923[44], 21}, {NULL}};
const Segment SGI225519[] = {{&S84923[30], 6}, {NULL}};
const Segment SGI225520[] = {{&S84923[130], 3}, {NULL}};
const Segment SGI225521[] = {{&S84923[95], 6}, {NULL}};
const Segment SGI225522[] = {{&S95498[34], 11}, {NULL}};
const Segment SGI225523[] = {{&S84923[67], 8}, {NULL}};
const Segment SGI225524[] = {{&S95498[27], 6}, {NULL}};
const Segment SGI225525[] = {{&S95498[98], 9}, {NULL}};
const Segment SGI225526[] = {{&S95498[71], 6}, {NULL}};
const Segment SGI225527[] = {{&S84924[34], 9}, {NULL}};
const Segment SGI225528[] = {{&S84924[27], 6}, {NULL}};
const Segment SGI225529[] = {{&S84924[96], 9}, {NULL}};
const Segment SGI225530[] = {{&S84924[69], 6}, {NULL}};
const Segment SGI225531[] = {{&S77612[44], 14}, {NULL}};
const Segment SGI225532[] = {{&S77612[44], 25}, {NULL}};
const Segment SGI225533[] = {{&S77612[30], 6}, {NULL}};
const Segment SGI225534[] = {{&S77612[118], 3}, {NULL}};
const Segment SGI225535[] = {{&S77612[83], 6}, {NULL}};
const Segment SGI225536[] = {{&S77612[143], 7}, {NULL}};
const Segment SGI225537[] = {{&S91812[34], 10}, {NULL}};
const Segment SGI225538[] = {{&S91812[27], 6}, {NULL}};
const Segment SGI225539[] = {{&S91812[102], 9}, {NULL}};
const Segment SGI225540[] = {{&S91812[75], 6}, {NULL}};
const Segment SGI225541[] = {{&S84927[34], 7}, {NULL}};
const Segment SGI225542[] = {{&S84927[27], 6}, {NULL}};
const Segment SGI225543[] = {{&S77614[34], 7}, {NULL}};
const Segment SGI225544[] = {{&S84927[94], 9}, {NULL}};
const Segment SGI225545[] = {{&S84927[67], 6}, {NULL}};
const Segment SGI225546[] = {{&S77614[27], 6}, {NULL}};
const Segment SGI225547[] = {{&S77614[100], 9}, {NULL}};
const Segment SGI225548[] = {{&S77614[73], 6}, {NULL}};
const Segment SGI225549[] = {{&S95503[44], 11}, {NULL}};
const Segment SGI225550[] = {{&S95503[44], 23}, {NULL}};
const Segment SGI225551[] = {{&S95503[30], 6}, {NULL}};
const Segment SGI225552[] = {{&S95503[81], 6}, {NULL}};
const Segment SGI225553[] = {{&S95503[108], 13}, {NULL}};
const Segment SGI225554[] = {{&S91815[59], 10}, {NULL}};
const Segment SGI225555[] = {{&S91815[59], 22}, {NULL}};
const Segment SGI225556[] = {{&S91815[45], 6}, {NULL}};
const Segment SGI225557[] = {{&S84929[34], 6}, {NULL}};
const Segment SGI225558[] = {{&S84929[27], 6}, {NULL}};
const Segment SGI225559[] = {{&S91815[130], 3}, {NULL}};
const Segment SGI225560[] = {{&S91815[95], 6}, {NULL}};
const Segment SGI225561[] = {{&S84929[93], 9}, {NULL}};
const Segment SGI225562[] = {{&S84929[66], 6}, {NULL}};
const Segment SGI225563[] = {{&S91815[0], 14}, {NULL}};
const Segment SGI225564[] = {{&S95505[34], 7}, {NULL}};
const Segment SGI225565[] = {{&S95505[27], 6}, {NULL}};
const Segment SGI225566[] = {{&S95505[95], 9}, {NULL}};
const Segment SGI225567[] = {{&S95505[68], 6}, {NULL}};
const Segment SGI225568[] = {{&S77618[34], 10}, {NULL}};
const Segment SGI225569[] = {{&S77618[27], 6}, {NULL}};
const Segment SGI225570[] = {{&S77618[96], 9}, {NULL}};
const Segment SGI225571[] = {{&S77618[69], 6}, {NULL}};
const Segment SGI225572[] = {{&S77618[137], 8}, {NULL}};
const Segment SGI225573[] = {{&S91818[44], 10}, {NULL}};
const Segment SGI225574[] = {{&S91818[44], 27}, {NULL}};
const Segment SGI225575[] = {{&S91818[30], 6}, {NULL}};
const Segment SGI225576[] = {{&S91818[85], 6}, {NULL}};
const Segment SGI225577[] = {{&S91818[112], 13}, {NULL}};
const Segment SGI225578[] = {{&S95507[34], 18}, {NULL}};
const Segment SGI225579[] = {{&S95507[27], 6}, {NULL}};
const Segment SGI225580[] = {{&S84934[34], 11}, {NULL}};
const Segment SGI225581[] = {{&S84934[27], 6}, {NULL}};
const Segment SGI225582[] = {{&S95507[105], 9}, {NULL}};
const Segment SGI225583[] = {{&S95507[78], 6}, {NULL}};
const Segment SGI225584[] = {{&S84934[98], 9}, {NULL}};
const Segment SGI225585[] = {{&S84934[71], 6}, {NULL}};
const Segment SGI225586[] = {{&S84935[34], 10}, {NULL}};
const Segment SGI225587[] = {{&S84935[27], 6}, {NULL}};
const Segment SGI225588[] = {{&S84935[97], 9}, {NULL}};
const Segment SGI225589[] = {{&S84935[70], 6}, {NULL}};
const Segment SGI225590[] = {{&S95512[34], 13}, {NULL}};
const Segment SGI225591[] = {{&S95512[27], 6}, {NULL}};
const Segment SGI225592[] = {{&S95512[100], 9}, {NULL}};
const Segment SGI225593[] = {{&S95512[73], 6}, {NULL}};
const Segment SGI225594[] = {{&S95514[34], 9}, {NULL}};
const Segment SGI225595[] = {{&S95514[27], 6}, {NULL}};
const Segment SGI225596[] = {{&S95514[96], 9}, {NULL}};
const Segment SGI225597[] = {{&S95514[69], 6}, {NULL}};
const Segment SGI225598[] = {{&S84943[34], 8}, {NULL}};
const Segment SGI225599[] = {{&S84943[27], 6}, {NULL}};
const Segment SGI225600[] = {{&S84943[95], 9}, {NULL}};
const Segment SGI225601[] = {{&S84943[68], 6}, {NULL}};
const Segment SGI225602[] = {{&S95516[34], 9}, {NULL}};
const Segment SGI225603[] = {{&S95516[27], 6}, {NULL}};
const Segment SGI225604[] = {{&S95516[96], 9}, {NULL}};
const Segment SGI225605[] = {{&S95516[69], 6}, {NULL}};
const Segment SGI225606[] = {{&S91834[34], 5}, {NULL}};
const Segment SGI225607[] = {{&S91834[27], 6}, {NULL}};
const Segment SGI225608[] = {{&S91834[92], 9}, {NULL}};
const Segment SGI225609[] = {{&S91834[65], 6}, {NULL}};
const Segment SGI225610[] = {{&S95521[34], 8}, {NULL}};
const Segment SGI225611[] = {{&S95521[27], 6}, {NULL}};
const Segment SGI225612[] = {{&S95521[95], 9}, {NULL}};
const Segment SGI225613[] = {{&S95521[68], 6}, {NULL}};
const Segment SGI225614[] = {{&S91845[44], 6}, {NULL}};
const Segment SGI225615[] = {{&S91845[44], 18}, {NULL}};
const Segment SGI225616[] = {{&S91845[30], 6}, {NULL}};
const Segment SGI225617[] = {{&S91845[111], 3}, {NULL}};
const Segment SGI225618[] = {{&S91845[76], 6}, {NULL}};
const Segment SGI225619[] = {{&S77650[34], 11}, {NULL}};
const Segment SGI225620[] = {{&S91846[44], 9}, {NULL}};
const Segment SGI225621[] = {{&S77650[27], 6}, {NULL}};
const Segment SGI225622[] = {{&S91846[44], 21}, {NULL}};
const Segment SGI225623[] = {{&S91846[30], 6}, {NULL}};
const Segment SGI225624[] = {{&S77650[97], 9}, {NULL}};
const Segment SGI225625[] = {{&S91846[114], 3}, {NULL}};
const Segment SGI225626[] = {{&S77650[70], 6}, {NULL}};
const Segment SGI225627[] = {{&S91846[79], 6}, {NULL}};
const Segment SGI225628[] = {{&S95531[34], 13}, {NULL}};
const Segment SGI225629[] = {{&S95531[27], 6}, {NULL}};
const Segment SGI225630[] = {{&S95531[100], 9}, {NULL}};
const Segment SGI225631[] = {{&S95531[73], 6}, {NULL}};
const Segment SGI225632[] = {{&S95532[59], 5}, {NULL}};
const Segment SGI225633[] = {{&S95532[59], 17}, {NULL}};
const Segment SGI225634[] = {{&S95532[45], 6}, {NULL}};
const Segment SGI225635[] = {{&S95532[46], 3}, {NULL}};
const Segment SGI225636[] = {{&S95532[90], 6}, {NULL}};
const Segment SGI225637[] = {{&S95532[0], 14}, {NULL}};
const Segment SGI225638[] = {{&S95534[34], 7}, {NULL}};
const Segment SGI225639[] = {{&S95534[27], 6}, {NULL}};
const Segment SGI225640[] = {{&S95534[98], 9}, {NULL}};
const Segment SGI225641[] = {{&S95534[71], 6}, {NULL}};
const Segment SGI225642[] = {{&S95534[139], 8}, {NULL}};
const Segment SGI225643[] = {{&S95535[34], 9}, {NULL}};
const Segment SGI225644[] = {{&S95535[27], 6}, {NULL}};
const Segment SGI225645[] = {{&S95535[102], 9}, {NULL}};
const Segment SGI225646[] = {{&S95535[75], 6}, {NULL}};
const Segment SGI225647[] = {{&S95538[34], 9}, {NULL}};
const Segment SGI225648[] = {{&S95538[27], 6}, {NULL}};
const Segment SGI225649[] = {{&S95538[96], 9}, {NULL}};
const Segment SGI225650[] = {{&S95538[69], 6}, {NULL}};
const Segment SGI225651[] = {{&S95539[44], 15}, {NULL}};
const Segment SGI225652[] = {{&S95539[44], 27}, {NULL}};
const Segment SGI225653[] = {{&S95539[30], 6}, {NULL}};
const Segment SGI225654[] = {{&S95539[120], 3}, {NULL}};
const Segment SGI225655[] = {{&S95539[85], 6}, {NULL}};
const Segment SGI225656[] = {{&S84981[38], 6}, {NULL}};
const Segment SGI225657[] = {{&S95539[140], 8}, {NULL}};
const Segment SGI225658[] = {{&S84981[31], 6}, {NULL}};
const Segment SGI225659[] = {{&S95539[149], 13}, {NULL}};
const Segment SGI225660[] = {{&S84982[44], 6}, {NULL}};
const Segment SGI225661[] = {{&S84982[44], 18}, {NULL}};
const Segment SGI225662[] = {{&S84982[30], 6}, {NULL}};
const Segment SGI225663[] = {{&S84982[111], 3}, {NULL}};
const Segment SGI225664[] = {{&S84982[76], 6}, {NULL}};
const Segment SGI225665[] = {{&S95542[34], 9}, {NULL}};
const Segment SGI225666[] = {{&S95542[27], 6}, {NULL}};
const Segment SGI225667[] = {{&S95542[96], 9}, {NULL}};
const Segment SGI225668[] = {{&S95542[69], 6}, {NULL}};
const Segment SGI225669[] = {{&S95545[34], 9}, {NULL}};
const Segment SGI225670[] = {{&S95545[27], 6}, {NULL}};
const Segment SGI225671[] = {{&S95545[96], 9}, {NULL}};
const Segment SGI225672[] = {{&S95545[69], 6}, {NULL}};
const Segment SGI225673[] = {{&S95546[34], 10}, {NULL}};
const Segment SGI225674[] = {{&S95546[27], 6}, {NULL}};
const Segment SGI225675[] = {{&S95546[97], 9}, {NULL}};
const Segment SGI225676[] = {{&S95546[70], 6}, {NULL}};
const Segment SGI225677[] = {{&S84991[44], 11}, {NULL}};
const Segment SGI225678[] = {{&S84991[44], 23}, {NULL}};
const Segment SGI225679[] = {{&S84991[30], 6}, {NULL}};
const Segment SGI225680[] = {{&S84991[116], 3}, {NULL}};
const Segment SGI225681[] = {{&S84991[81], 6}, {NULL}};
const Segment SGI225682[] = {{&S84991[151], 3}, {NULL}};
const Segment SGI225683[] = {{&S84992[34], 8}, {NULL}};
const Segment SGI225684[] = {{&S84992[27], 6}, {NULL}};
const Segment SGI225685[] = {{&S84992[101], 9}, {NULL}};
const Segment SGI225686[] = {{&S84992[74], 6}, {NULL}};
const Segment SGI225687[] = {{&S84993[34], 6}, {NULL}};
const Segment SGI225688[] = {{&S84993[27], 6}, {NULL}};
const Segment SGI225689[] = {{&S84993[93], 9}, {NULL}};
const Segment SGI225690[] = {{&S84993[66], 6}, {NULL}};
const Segment SGI225691[] = {{&S95553[34], 20}, {NULL}};
const Segment SGI225692[] = {{&S95553[27], 6}, {NULL}};
const Segment SGI225693[] = {{&S95553[107], 9}, {NULL}};
const Segment SGI225694[] = {{&S95553[80], 6}, {NULL}};
const Segment SGI225695[] = {{&S95556[34], 5}, {NULL}};
const Segment SGI225696[] = {{&S95556[27], 6}, {NULL}};
const Segment SGI225697[] = {{&S95556[92], 9}, {NULL}};
const Segment SGI225698[] = {{&S95556[65], 6}, {NULL}};
const Segment SGI225699[] = {{&S95558[34], 11}, {NULL}};
const Segment SGI225700[] = {{&S95558[27], 6}, {NULL}};
const Segment SGI225701[] = {{&S95558[98], 9}, {NULL}};
const Segment SGI225702[] = {{&S95558[71], 6}, {NULL}};
const Segment SGI225703[] = {{&S95560[34], 9}, {NULL}};
const Segment SGI225704[] = {{&S95560[27], 6}, {NULL}};
const Segment SGI225705[] = {{&S95560[96], 9}, {NULL}};
const Segment SGI225706[] = {{&S95560[69], 6}, {NULL}};
const Segment SGI225707[] = {{&S95560[137], 8}, {NULL}};
const Segment SGI225708[] = {{&S95563[34], 10}, {NULL}};
const Segment SGI225709[] = {{&S95563[27], 6}, {NULL}};
const Segment SGI225710[] = {{&S95563[97], 9}, {NULL}};
const Segment SGI225711[] = {{&S95563[70], 6}, {NULL}};
const Segment SGI225712[] = {{&S85011[34], 7}, {NULL}};
const Segment SGI225713[] = {{&S85011[27], 6}, {NULL}};
const Segment SGI225714[] = {{&S95566[34], 7}, {NULL}};
const Segment SGI225715[] = {{&S85011[94], 9}, {NULL}};
const Segment SGI225716[] = {{&S95566[27], 6}, {NULL}};
const Segment SGI225717[] = {{&S85011[67], 6}, {NULL}};
const Segment SGI225718[] = {{&S95566[99], 9}, {NULL}};
const Segment SGI225719[] = {{&S95566[72], 6}, {NULL}};
const Segment SGI225720[] = {{&S95570[34], 9}, {NULL}};
const Segment SGI225721[] = {{&S95570[27], 6}, {NULL}};
const Segment SGI225722[] = {{&S95570[96], 9}, {NULL}};
const Segment SGI225723[] = {{&S95570[69], 6}, {NULL}};
const Segment SGI225724[] = {{&S85020[52], 12}, {NULL}};
const Segment SGI225725[] = {{&S85020[52], 34}, {NULL}};
const Segment SGI225726[] = {{&S85020[38], 6}, {NULL}};
const Segment SGI225727[] = {{&S85020[100], 5}, {NULL}};
const Segment SGI225728[] = {{&S85020[11], 3}, {NULL}};
const Segment SGI225729[] = {{&S91918[44], 21}, {NULL}};
const Segment SGI225730[] = {{&S91918[44], 33}, {NULL}};
const Segment SGI225731[] = {{&S91918[30], 6}, {NULL}};
const Segment SGI225732[] = {{&S91918[126], 3}, {NULL}};
const Segment SGI225733[] = {{&S91918[91], 6}, {NULL}};
const Segment SGI225734[] = {{&S95584[44], 7}, {NULL}};
const Segment SGI225735[] = {{&S85030[34], 7}, {NULL}};
const Segment SGI225736[] = {{&S95584[44], 18}, {NULL}};
const Segment SGI225737[] = {{&S95584[30], 6}, {NULL}};
const Segment SGI225738[] = {{&S85030[27], 6}, {NULL}};
const Segment SGI225739[] = {{&S85030[94], 9}, {NULL}};
const Segment SGI225740[] = {{&S85030[67], 6}, {NULL}};
const Segment SGI225741[] = {{&S95584[76], 6}, {NULL}};
const Segment SGI225742[] = {{&S95584[103], 13}, {NULL}};
const Segment SGI225743[] = {{&S85031[34], 17}, {NULL}};
const Segment SGI225744[] = {{&S85031[27], 6}, {NULL}};
const Segment SGI225745[] = {{&S85031[104], 9}, {NULL}};
const Segment SGI225746[] = {{&S85031[77], 6}, {NULL}};
const Segment SGI225747[] = {{&S85032[34], 18}, {NULL}};
const Segment SGI225748[] = {{&S85032[27], 6}, {NULL}};
const Segment SGI225749[] = {{&S85032[105], 9}, {NULL}};
const Segment SGI225750[] = {{&S85032[78], 6}, {NULL}};
const Segment SGI225751[] = {{&S95589[44], 15}, {NULL}};
const Segment SGI225752[] = {{&S95589[44], 33}, {NULL}};
const Segment SGI225753[] = {{&S95589[30], 6}, {NULL}};
const Segment SGI225754[] = {{&S95589[31], 3}, {NULL}};
const Segment SGI225755[] = {{&S95589[91], 5}, {NULL}};
const Segment SGI225756[] = {{&S95589[128], 14}, {NULL}};
const Segment SGI225757[] = {{&S95590[44], 18}, {NULL}};
const Segment SGI225758[] = {{&S95590[44], 30}, {NULL}};
const Segment SGI225759[] = {{&S95590[30], 6}, {NULL}};
const Segment SGI225760[] = {{&S95590[31], 3}, {NULL}};
const Segment SGI225761[] = {{&S95590[88], 5}, {NULL}};
const Segment SGI225762[] = {{&S95590[125], 14}, {NULL}};
const Segment SGI225763[] = {{&S85037[34], 9}, {NULL}};
const Segment SGI225764[] = {{&S85037[27], 6}, {NULL}};
const Segment SGI225765[] = {{&S85037[96], 9}, {NULL}};
const Segment SGI225766[] = {{&S91935[38], 7}, {NULL}};
const Segment SGI225767[] = {{&S85037[69], 6}, {NULL}};
const Segment SGI225768[] = {{&S91935[31], 6}, {NULL}};
const Segment SGI225769[] = {{&S95591[34], 9}, {NULL}};
const Segment SGI225770[] = {{&S95591[27], 6}, {NULL}};
const Segment SGI225771[] = {{&S95591[96], 9}, {NULL}};
const Segment SGI225772[] = {{&S95591[69], 6}, {NULL}};
const Segment SGI225773[] = {{&S91936[44], 7}, {NULL}};
const Segment SGI225774[] = {{&S91936[44], 19}, {NULL}};
const Segment SGI225775[] = {{&S91936[30], 6}, {NULL}};
const Segment SGI225776[] = {{&S91936[112], 3}, {NULL}};
const Segment SGI225777[] = {{&S91936[77], 6}, {NULL}};
const Segment SGI225778[] = {{&S95592[44], 18}, {NULL}};
const Segment SGI225779[] = {{&S95592[44], 30}, {NULL}};
const Segment SGI225780[] = {{&S95592[30], 6}, {NULL}};
const Segment SGI225781[] = {{&S95592[31], 3}, {NULL}};
const Segment SGI225782[] = {{&S95592[88], 6}, {NULL}};
const Segment SGI225783[] = {{&S95592[148], 7}, {NULL}};
const Segment SGI225784[] = {{&S91942[34], 13}, {NULL}};
const Segment SGI225785[] = {{&S91942[27], 6}, {NULL}};
const Segment SGI225786[] = {{&S91942[100], 9}, {NULL}};
const Segment SGI225787[] = {{&S91942[73], 6}, {NULL}};
const Segment SGI225788[] = {{&S95601[44], 9}, {NULL}};
const Segment SGI225789[] = {{&S95601[44], 21}, {NULL}};
const Segment SGI225790[] = {{&S95601[30], 6}, {NULL}};
const Segment SGI225791[] = {{&S95601[79], 6}, {NULL}};
const Segment SGI225792[] = {{&S95601[106], 13}, {NULL}};
const Segment SGI225793[] = {{&S95602[44], 11}, {NULL}};
const Segment SGI225794[] = {{&S95602[44], 23}, {NULL}};
const Segment SGI225795[] = {{&S95602[30], 6}, {NULL}};
const Segment SGI225796[] = {{&S95602[81], 6}, {NULL}};
const Segment SGI225797[] = {{&S95602[108], 13}, {NULL}};
const Segment SGI225798[] = {{&S95605[34], 11}, {NULL}};
const Segment SGI225799[] = {{&S95605[27], 6}, {NULL}};
const Segment SGI225800[] = {{&S95605[98], 9}, {NULL}};
const Segment SGI225801[] = {{&S95605[71], 6}, {NULL}};
const Segment SGI225802[] = {{&S95609[34], 7}, {NULL}};
const Segment SGI225803[] = {{&S95609[27], 6}, {NULL}};
const Segment SGI225804[] = {{&S95609[90], 9}, {NULL}};
const Segment SGI225805[] = {{&S95609[63], 6}, {NULL}};
const Segment SGI225806[] = {{&S95610[34], 10}, {NULL}};
const Segment SGI225807[] = {{&S95610[27], 6}, {NULL}};
const Segment SGI225808[] = {{&S91960[34], 10}, {NULL}};
const Segment SGI225809[] = {{&S95610[97], 9}, {NULL}};
const Segment SGI225810[] = {{&S95610[70], 6}, {NULL}};
const Segment SGI225811[] = {{&S91960[27], 6}, {NULL}};
const Segment SGI225812[] = {{&S91960[97], 9}, {NULL}};
const Segment SGI225813[] = {{&S91960[70], 6}, {NULL}};
const Segment SGI225814[] = {{&S95613[34], 10}, {NULL}};
const Segment SGI225815[] = {{&S95613[27], 6}, {NULL}};
const Segment SGI225816[] = {{&S91963[44], 14}, {NULL}};
const Segment SGI225817[] = {{&S91963[44], 31}, {NULL}};
const Segment SGI225818[] = {{&S91963[30], 6}, {NULL}};
const Segment SGI225819[] = {{&S95613[97], 9}, {NULL}};
const Segment SGI225820[] = {{&S95613[70], 6}, {NULL}};
const Segment SGI225821[] = {{&S91963[122], 3}, {NULL}};
const Segment SGI225822[] = {{&S91963[89], 5}, {NULL}};
const Segment SGI225823[] = {{&S91963[126], 14}, {NULL}};
const Segment SGI225824[] = {{&S95614[34], 9}, {NULL}};
const Segment SGI225825[] = {{&S95614[27], 6}, {NULL}};
const Segment SGI225826[] = {{&S95614[96], 9}, {NULL}};
const Segment SGI225827[] = {{&S95614[69], 6}, {NULL}};
const Segment SGI225828[] = {{&S95616[34], 9}, {NULL}};
const Segment SGI225829[] = {{&S95616[27], 6}, {NULL}};
const Segment SGI225830[] = {{&S95616[96], 9}, {NULL}};
const Segment SGI225831[] = {{&S95616[69], 6}, {NULL}};
const Segment SGI225832[] = {{&S91975[44], 18}, {NULL}};
const Segment SGI225833[] = {{&S91975[44], 30}, {NULL}};
const Segment SGI225834[] = {{&S91975[30], 6}, {NULL}};
const Segment SGI225835[] = {{&S91975[123], 3}, {NULL}};
const Segment SGI225836[] = {{&S91975[88], 6}, {NULL}};
const Segment SGI225837[] = {{&S85085[44], 10}, {NULL}};
const Segment SGI225838[] = {{&S77788[34], 18}, {NULL}};
const Segment SGI225839[] = {{&S85085[44], 22}, {NULL}};
const Segment SGI225840[] = {{&S85085[30], 6}, {NULL}};
const Segment SGI225841[] = {{&S77788[27], 6}, {NULL}};
const Segment SGI225842[] = {{&S85085[115], 3}, {NULL}};
const Segment SGI225843[] = {{&S85085[80], 6}, {NULL}};
const Segment SGI225844[] = {{&S77788[105], 9}, {NULL}};
const Segment SGI225845[] = {{&S77788[78], 6}, {NULL}};
const Segment SGI225846[] = {{&S91977[34], 12}, {NULL}};
const Segment SGI225847[] = {{&S91977[27], 6}, {NULL}};
const Segment SGI225848[] = {{&S91977[99], 9}, {NULL}};
const Segment SGI225849[] = {{&S91977[72], 6}, {NULL}};
const Segment SGI225850[] = {{&S85086[38], 10}, {NULL}};
const Segment SGI225851[] = {{&S85086[31], 6}, {NULL}};
const Segment SGI225852[] = {{&S77789[34], 22}, {NULL}};
const Segment SGI225853[] = {{&S77789[27], 6}, {NULL}};
const Segment SGI225854[] = {{&S77789[109], 9}, {NULL}};
const Segment SGI225855[] = {{&S77789[82], 6}, {NULL}};
const Segment SGI225856[] = {{&S77790[34], 21}, {NULL}};
const Segment SGI225857[] = {{&S77790[27], 6}, {NULL}};
const Segment SGI225858[] = {{&S77790[108], 9}, {NULL}};
const Segment SGI225859[] = {{&S77790[81], 6}, {NULL}};
const Segment SGI225860[] = {{&S95631[44], 14}, {NULL}};
const Segment SGI225861[] = {{&S95631[44], 31}, {NULL}};
const Segment SGI225862[] = {{&S95631[30], 6}, {NULL}};
const Segment SGI225863[] = {{&S95631[31], 3}, {NULL}};
const Segment SGI225864[] = {{&S95631[89], 5}, {NULL}};
const Segment SGI225865[] = {{&S95631[126], 14}, {NULL}};
const Segment SGI225866[] = {{&S91983[34], 6}, {NULL}};
const Segment SGI225867[] = {{&S85092[34], 18}, {NULL}};
const Segment SGI225868[] = {{&S91983[27], 6}, {NULL}};
const Segment SGI225869[] = {{&S85092[27], 6}, {NULL}};
const Segment SGI225870[] = {{&S91983[93], 9}, {NULL}};
const Segment SGI225871[] = {{&S85092[105], 9}, {NULL}};
const Segment SGI225872[] = {{&S91983[66], 6}, {NULL}};
const Segment SGI225873[] = {{&S85092[78], 6}, {NULL}};
const Segment SGI225874[] = {{&S95634[44], 11}, {NULL}};
const Segment SGI225875[] = {{&S95634[44], 23}, {NULL}};
const Segment SGI225876[] = {{&S95634[30], 6}, {NULL}};
const Segment SGI225877[] = {{&S95634[31], 3}, {NULL}};
const Segment SGI225878[] = {{&S95634[81], 6}, {NULL}};
const Segment SGI225879[] = {{&S95634[141], 7}, {NULL}};
const Segment SGI225880[] = {{&S85100[34], 10}, {NULL}};
const Segment SGI225881[] = {{&S85100[27], 6}, {NULL}};
const Segment SGI225882[] = {{&S85100[97], 9}, {NULL}};
const Segment SGI225883[] = {{&S85100[70], 6}, {NULL}};
const Segment SGI225884[] = {{&S91992[34], 25}, {NULL}};
const Segment SGI225885[] = {{&S91992[27], 6}, {NULL}};
const Segment SGI225886[] = {{&S91992[112], 9}, {NULL}};
const Segment SGI225887[] = {{&S91992[85], 6}, {NULL}};
const Segment SGI225888[] = {{&S85101[34], 10}, {NULL}};
const Segment SGI225889[] = {{&S85101[27], 6}, {NULL}};
const Segment SGI225890[] = {{&S85101[97], 9}, {NULL}};
const Segment SGI225891[] = {{&S85101[70], 6}, {NULL}};
const Segment SGI225892[] = {{&S85102[34], 10}, {NULL}};
const Segment SGI225893[] = {{&S91994[34], 7}, {NULL}};
const Segment SGI225894[] = {{&S85102[27], 6}, {NULL}};
const Segment SGI225895[] = {{&S91994[27], 6}, {NULL}};
const Segment SGI225896[] = {{&S85102[97], 9}, {NULL}};
const Segment SGI225897[] = {{&S91994[94], 9}, {NULL}};
const Segment SGI225898[] = {{&S85102[70], 6}, {NULL}};
const Segment SGI225899[] = {{&S91994[67], 6}, {NULL}};
const Segment SGI225900[] = {{&S85107[77], 10}, {NULL}};
const Segment SGI225901[] = {{&S85107[77], 27}, {NULL}};
const Segment SGI225902[] = {{&S85107[30], 39}, {NULL}};
const Segment SGI225903[] = {{&S85107[152], 3}, {NULL}};
const Segment SGI225904[] = {{&S85107[118], 5}, {NULL}};
const Segment SGI225905[] = {{&S95652[44], 6}, {NULL}};
const Segment SGI225906[] = {{&S95652[44], 18}, {NULL}};
const Segment SGI225907[] = {{&S95652[30], 6}, {NULL}};
const Segment SGI225908[] = {{&S95652[76], 6}, {NULL}};
const Segment SGI225909[] = {{&S95652[103], 13}, {NULL}};
const Segment SGI225910[] = {{&S85108[34], 12}, {NULL}};
const Segment SGI225911[] = {{&S85108[27], 6}, {NULL}};
const Segment SGI225912[] = {{&S85108[99], 9}, {NULL}};
const Segment SGI225913[] = {{&S85108[72], 6}, {NULL}};
const Segment SGI225914[] = {{&S85109[34], 10}, {NULL}};
const Segment SGI225915[] = {{&S85109[27], 6}, {NULL}};
const Segment SGI225916[] = {{&S85109[97], 9}, {NULL}};
const Segment SGI225917[] = {{&S85109[70], 6}, {NULL}};
const Segment SGI225918[] = {{&S92007[44], 13}, {NULL}};
const Segment SGI225919[] = {{&S92007[44], 25}, {NULL}};
const Segment SGI225920[] = {{&S92007[30], 6}, {NULL}};
const Segment SGI225921[] = {{&S92007[83], 6}, {NULL}};
const Segment SGI225922[] = {{&S92007[110], 13}, {NULL}};
const Segment SGI225923[] = {{&S85114[44], 18}, {NULL}};
const Segment SGI225924[] = {{&S85114[44], 39}, {NULL}};
const Segment SGI225925[] = {{&S85114[30], 6}, {NULL}};
const Segment SGI225926[] = {{&S85114[97], 6}, {NULL}};
const Segment SGI225927[] = {{&S85114[124], 13}, {NULL}};
const Segment SGI225928[] = {{&S85115[44], 18}, {NULL}};
const Segment SGI225929[] = {{&S85115[44], 39}, {NULL}};
const Segment SGI225930[] = {{&S85115[30], 6}, {NULL}};
const Segment SGI225931[] = {{&S85115[97], 6}, {NULL}};
const Segment SGI225932[] = {{&S85115[124], 13}, {NULL}};
const Segment SGI225933[] = {{&S95663[34], 15}, {NULL}};
const Segment SGI225934[] = {{&S95663[27], 6}, {NULL}};
const Segment SGI225935[] = {{&S95663[102], 9}, {NULL}};
const Segment SGI225936[] = {{&S95663[75], 6}, {NULL}};
const Segment SGI225937[] = {{&S95663[143], 8}, {NULL}};
const Segment SGI225938[] = {{&S85117[34], 10}, {NULL}};
const Segment SGI225939[] = {{&S85117[27], 6}, {NULL}};
const Segment SGI225940[] = {{&S85117[97], 9}, {NULL}};
const Segment SGI225941[] = {{&S85117[70], 6}, {NULL}};
const Segment SGI225942[] = {{&S85117[138], 8}, {NULL}};
const Segment SGI225943[] = {{&S95665[34], 7}, {NULL}};
const Segment SGI225944[] = {{&S95665[27], 6}, {NULL}};
const Segment SGI225945[] = {{&S95665[96], 9}, {NULL}};
const Segment SGI225946[] = {{&S95665[69], 6}, {NULL}};
const Segment SGI225947[] = {{&S85130[34], 10}, {NULL}};
const Segment SGI225948[] = {{&S85130[27], 6}, {NULL}};
const Segment SGI225949[] = {{&S85130[97], 9}, {NULL}};
const Segment SGI225950[] = {{&S85130[70], 6}, {NULL}};
const Segment SGI225951[] = {{&S92035[44], 6}, {NULL}};
const Segment SGI225952[] = {{&S92035[44], 18}, {NULL}};
const Segment SGI225953[] = {{&S92035[30], 6}, {NULL}};
const Segment SGI225954[] = {{&S92035[109], 3}, {NULL}};
const Segment SGI225955[] = {{&S92035[76], 5}, {NULL}};
const Segment SGI225956[] = {{&S92035[113], 14}, {NULL}};
const Segment SGI225957[] = {{&S92060[34], 5}, {NULL}};
const Segment SGI225958[] = {{&S92060[27], 6}, {NULL}};
const Segment SGI225959[] = {{&S92060[92], 9}, {NULL}};
const Segment SGI225960[] = {{&S92060[65], 6}, {NULL}};
const Segment SGI225961[] = {{&S99271[34], 5}, {NULL}};
const Segment SGI225962[] = {{&S99271[27], 6}, {NULL}};
const Segment SGI225963[] = {{&S99271[65], 6}, {NULL}};
const Segment SGI225964[] = {{&S86717[44], 10}, {NULL}};
const Segment SGI225965[] = {{&S86717[44], 22}, {NULL}};
const Segment SGI225966[] = {{&S86717[30], 6}, {NULL}};
const Segment SGI225967[] = {{&S86717[133], 3}, {NULL}};
const Segment SGI225968[] = {{&S86717[98], 6}, {NULL}};
const Segment SGI225969[] = {{&S86717[68], 8}, {NULL}};
const Segment SGI225970[] = {{&S86719[34], 10}, {NULL}};
const Segment SGI225971[] = {{&S86719[27], 6}, {NULL}};
const Segment SGI225972[] = {{&S86719[97], 9}, {NULL}};
const Segment SGI225973[] = {{&S86719[70], 6}, {NULL}};
const Segment SGI225974[] = {{&S95746[45], 7}, {NULL}};
const Segment SGI225975[] = {{&S95746[30], 6}, {NULL}};
const Segment SGI225976[] = {{&S95746[31], 3}, {NULL}};
const Segment SGI225977[] = {{&S95746[83], 5}, {NULL}};
const Segment SGI225978[] = {{&S95746[141], 13}, {NULL}};
const Segment SGI225979[] = {{&S85201[34], 25}, {NULL}};
const Segment SGI225980[] = {{&S85201[27], 6}, {NULL}};
const Segment SGI225981[] = {{&S85201[112], 9}, {NULL}};
const Segment SGI225982[] = {{&S85201[85], 6}, {NULL}};
const Segment SGI225983[] = {{&S85204[44], 7}, {NULL}};
const Segment SGI225984[] = {{&S85204[44], 15}, {NULL}};
const Segment SGI225985[] = {{&S85204[30], 6}, {NULL}};
const Segment SGI225986[] = {{&S85204[108], 3}, {NULL}};
const Segment SGI225987[] = {{&S85204[73], 6}, {NULL}};
const Segment SGI225988[] = {{&S95756[59], 7}, {NULL}};
const Segment SGI225989[] = {{&S95756[59], 23}, {NULL}};
const Segment SGI225990[] = {{&S85208[34], 6}, {NULL}};
const Segment SGI225991[] = {{&S95756[45], 6}, {NULL}};
const Segment SGI225992[] = {{&S85208[27], 6}, {NULL}};
const Segment SGI225993[] = {{&S95756[46], 3}, {NULL}};
const Segment SGI225994[] = {{&S95756[96], 6}, {NULL}};
const Segment SGI225995[] = {{&S85208[104], 9}, {NULL}};
const Segment SGI225996[] = {{&S85208[77], 6}, {NULL}};
const Segment SGI225997[] = {{&S95756[0], 14}, {NULL}};
const Segment SGI225998[] = {{&S95757[44], 7}, {NULL}};
const Segment SGI225999[] = {{&S95757[44], 15}, {NULL}};
const Segment SGI226000[] = {{&S95757[30], 6}, {NULL}};
const Segment SGI226001[] = {{&S95757[31], 3}, {NULL}};
const Segment SGI226002[] = {{&S95757[73], 6}, {NULL}};
const Segment SGI226003[] = {{&S85211[34], 6}, {NULL}};
const Segment SGI226004[] = {{&S85211[27], 6}, {NULL}};
const Segment SGI226005[] = {{&S85211[93], 9}, {NULL}};
const Segment SGI226006[] = {{&S85211[66], 6}, {NULL}};
const Segment SGI226007[] = {{&S85212[34], 11}, {NULL}};
const Segment SGI226008[] = {{&S85212[27], 6}, {NULL}};
const Segment SGI226009[] = {{&S85212[98], 9}, {NULL}};
const Segment SGI226010[] = {{&S85212[71], 6}, {NULL}};
const Segment SGI226011[] = {{&S85215[44], 17}, {NULL}};
const Segment SGI226012[] = {{&S85215[44], 29}, {NULL}};
const Segment SGI226013[] = {{&S85215[30], 6}, {NULL}};
const Segment SGI226014[] = {{&S85215[122], 3}, {NULL}};
const Segment SGI226015[] = {{&S85215[87], 6}, {NULL}};
const Segment SGI226016[] = {{&S95765[44], 11}, {NULL}};
const Segment SGI226017[] = {{&S95765[44], 23}, {NULL}};
const Segment SGI226018[] = {{&S95765[30], 6}, {NULL}};
const Segment SGI226019[] = {{&S95765[31], 3}, {NULL}};
const Segment SGI226020[] = {{&S95765[81], 5}, {NULL}};
const Segment SGI226021[] = {{&S95765[126], 14}, {NULL}};
const Segment SGI226022[] = {{&S85217[34], 8}, {NULL}};
const Segment SGI226023[] = {{&S85217[27], 6}, {NULL}};
const Segment SGI226024[] = {{&S85217[100], 9}, {NULL}};
const Segment SGI226025[] = {{&S85217[73], 6}, {NULL}};
const Segment SGI226026[] = {{&S95766[34], 11}, {NULL}};
const Segment SGI226027[] = {{&S95766[27], 6}, {NULL}};
const Segment SGI226028[] = {{&S95766[98], 9}, {NULL}};
const Segment SGI226029[] = {{&S95766[71], 6}, {NULL}};
const Segment SGI226030[] = {{&S85219[44], 10}, {NULL}};
const Segment SGI226031[] = {{&S85219[44], 22}, {NULL}};
const Segment SGI226032[] = {{&S85219[30], 6}, {NULL}};
const Segment SGI226033[] = {{&S85219[115], 3}, {NULL}};
const Segment SGI226034[] = {{&S92126[44], 10}, {NULL}};
const Segment SGI226035[] = {{&S85219[80], 6}, {NULL}};
const Segment SGI226036[] = {{&S92126[44], 22}, {NULL}};
const Segment SGI226037[] = {{&S92126[30], 6}, {NULL}};
const Segment SGI226038[] = {{&S85219[140], 7}, {NULL}};
const Segment SGI226039[] = {{&S95768[34], 8}, {NULL}};
const Segment SGI226040[] = {{&S92126[131], 3}, {NULL}};
const Segment SGI226041[] = {{&S95768[27], 6}, {NULL}};
const Segment SGI226042[] = {{&S92126[96], 6}, {NULL}};
const Segment SGI226043[] = {{&S92126[68], 8}, {NULL}};
const Segment SGI226044[] = {{&S95768[95], 9}, {NULL}};
const Segment SGI226045[] = {{&S95768[68], 6}, {NULL}};
const Segment SGI226046[] = {{&S95770[34], 20}, {NULL}};
const Segment SGI226047[] = {{&S95770[27], 6}, {NULL}};
const Segment SGI226048[] = {{&S85223[34], 14}, {NULL}};
const Segment SGI226049[] = {{&S85223[27], 6}, {NULL}};
const Segment SGI226050[] = {{&S95770[107], 9}, {NULL}};
const Segment SGI226051[] = {{&S95770[80], 6}, {NULL}};
const Segment SGI226052[] = {{&S85223[101], 9}, {NULL}};
const Segment SGI226053[] = {{&S85223[74], 6}, {NULL}};
const Segment SGI226054[] = {{&S85225[34], 10}, {NULL}};
const Segment SGI226055[] = {{&S85225[27], 6}, {NULL}};
const Segment SGI226056[] = {{&S85225[97], 9}, {NULL}};
const Segment SGI226057[] = {{&S85225[70], 6}, {NULL}};
const Segment SGI226058[] = {{&S95779[34], 16}, {NULL}};
const Segment SGI226059[] = {{&S95779[27], 6}, {NULL}};
const Segment SGI226060[] = {{&S95779[103], 9}, {NULL}};
const Segment SGI226061[] = {{&S95779[76], 6}, {NULL}};
const Segment SGI226062[] = {{&S85232[34], 13}, {NULL}};
const Segment SGI226063[] = {{&S85232[27], 6}, {NULL}};
const Segment SGI226064[] = {{&S85232[100], 9}, {NULL}};
const Segment SGI226065[] = {{&S85232[73], 6}, {NULL}};
const Segment SGI226066[] = {{&S92146[44], 10}, {NULL}};
const Segment SGI226067[] = {{&S92146[44], 22}, {NULL}};
const Segment SGI226068[] = {{&S92146[30], 6}, {NULL}};
const Segment SGI226069[] = {{&S92146[144], 3}, {NULL}};
const Segment SGI226070[] = {{&S92146[109], 6}, {NULL}};
const Segment SGI226071[] = {{&S92146[68], 8}, {NULL}};
const Segment SGI226072[] = {{&S92146[80], 15}, {NULL}};
const Segment SGI226073[] = {{&S95785[34], 9}, {NULL}};
const Segment SGI226074[] = {{&S95785[27], 6}, {NULL}};
const Segment SGI226075[] = {{&S95785[96], 9}, {NULL}};
const Segment SGI226076[] = {{&S95785[69], 6}, {NULL}};
const Segment SGI226077[] = {{&S89966[72], 10}, {NULL}};
const Segment SGI226078[] = {{&S89966[72], 27}, {NULL}};
const Segment SGI226079[] = {{&S89966[30], 34}, {NULL}};
const Segment SGI226080[] = {{&S89966[147], 3}, {NULL}};
const Segment SGI226081[] = {{&S89966[113], 5}, {NULL}};
const Segment SGI226082[] = {{&S95796[44], 11}, {NULL}};
const Segment SGI226083[] = {{&S95796[44], 23}, {NULL}};
const Segment SGI226084[] = {{&S95796[30], 6}, {NULL}};
const Segment SGI226085[] = {{&S95796[81], 6}, {NULL}};
const Segment SGI226086[] = {{&S95796[108], 13}, {NULL}};
const Segment SGI226087[] = {{&S95802[34], 9}, {NULL}};
const Segment SGI226088[] = {{&S95802[27], 6}, {NULL}};
const Segment SGI226089[] = {{&S95802[96], 9}, {NULL}};
const Segment SGI226090[] = {{&S95802[69], 6}, {NULL}};
const Segment SGI226091[] = {{&S95803[34], 7}, {NULL}};
const Segment SGI226092[] = {{&S95803[27], 6}, {NULL}};
const Segment SGI226093[] = {{&S85264[44], 10}, {NULL}};
const Segment SGI226094[] = {{&S85264[44], 22}, {NULL}};
const Segment SGI226095[] = {{&S85264[30], 6}, {NULL}};
const Segment SGI226096[] = {{&S95803[94], 9}, {NULL}};
const Segment SGI226097[] = {{&S95803[67], 6}, {NULL}};
const Segment SGI226098[] = {{&S85264[135], 3}, {NULL}};
const Segment SGI226099[] = {{&S85264[100], 6}, {NULL}};
const Segment SGI226100[] = {{&S85264[68], 8}, {NULL}};
const Segment SGI226101[] = {{&S95804[34], 7}, {NULL}};
const Segment SGI226102[] = {{&S95804[27], 6}, {NULL}};
const Segment SGI226103[] = {{&S95804[94], 9}, {NULL}};
const Segment SGI226104[] = {{&S95804[67], 6}, {NULL}};
const Segment SGI226105[] = {{&S95805[34], 6}, {NULL}};
const Segment SGI226106[] = {{&S95805[27], 6}, {NULL}};
const Segment SGI226107[] = {{&S95805[93], 9}, {NULL}};
const Segment SGI226108[] = {{&S95805[66], 6}, {NULL}};
const Segment SGI226109[] = {{&S85269[34], 6}, {NULL}};
const Segment SGI226110[] = {{&S85269[27], 6}, {NULL}};
const Segment SGI226111[] = {{&S95806[44], 10}, {NULL}};
const Segment SGI226112[] = {{&S95806[44], 22}, {NULL}};
const Segment SGI226113[] = {{&S95806[30], 6}, {NULL}};
const Segment SGI226114[] = {{&S85269[93], 9}, {NULL}};
const Segment SGI226115[] = {{&S85269[66], 6}, {NULL}};
const Segment SGI226116[] = {{&S95806[31], 3}, {NULL}};
const Segment SGI226117[] = {{&S95806[95], 6}, {NULL}};
const Segment SGI226118[] = {{&S95806[68], 8}, {NULL}};
const Segment SGI226119[] = {{&S95808[34], 4}, {NULL}};
const Segment SGI226120[] = {{&S95808[27], 6}, {NULL}};
const Segment SGI226121[] = {{&S95808[91], 9}, {NULL}};
const Segment SGI226122[] = {{&S95808[64], 6}, {NULL}};
const Segment SGI226123[] = {{&S95809[34], 11}, {NULL}};
const Segment SGI226124[] = {{&S95809[27], 6}, {NULL}};
const Segment SGI226125[] = {{&S95809[98], 9}, {NULL}};
const Segment SGI226126[] = {{&S95809[71], 6}, {NULL}};
const Segment SGI226127[] = {{&S95810[34], 15}, {NULL}};
const Segment SGI226128[] = {{&S95810[27], 6}, {NULL}};
const Segment SGI226129[] = {{&S95810[102], 9}, {NULL}};
const Segment SGI226130[] = {{&S95810[75], 6}, {NULL}};
const Segment SGI226131[] = {{&S95811[34], 11}, {NULL}};
const Segment SGI226132[] = {{&S95811[27], 6}, {NULL}};
const Segment SGI226133[] = {{&S95811[98], 9}, {NULL}};
const Segment SGI226134[] = {{&S95811[71], 6}, {NULL}};
const Segment SGI226135[] = {{&S95812[34], 8}, {NULL}};
const Segment SGI226136[] = {{&S95812[27], 6}, {NULL}};
const Segment SGI226137[] = {{&S95812[95], 9}, {NULL}};
const Segment SGI226138[] = {{&S95812[68], 6}, {NULL}};
const Segment SGI226139[] = {{&S95816[44], 11}, {NULL}};
const Segment SGI226140[] = {{&S95816[44], 23}, {NULL}};
const Segment SGI226141[] = {{&S95816[30], 6}, {NULL}};
const Segment SGI226142[] = {{&S95816[31], 3}, {NULL}};
const Segment SGI226143[] = {{&S95816[96], 6}, {NULL}};
const Segment SGI226144[] = {{&S95816[69], 8}, {NULL}};
const Segment SGI226145[] = {{&S85289[34], 11}, {NULL}};
const Segment SGI226146[] = {{&S95817[59], 5}, {NULL}};
const Segment SGI226147[] = {{&S95817[59], 17}, {NULL}};
const Segment SGI226148[] = {{&S85289[27], 6}, {NULL}};
const Segment SGI226149[] = {{&S95817[45], 6}, {NULL}};
const Segment SGI226150[] = {{&S95817[46], 3}, {NULL}};
const Segment SGI226151[] = {{&S85289[98], 9}, {NULL}};
const Segment SGI226152[] = {{&S85289[71], 6}, {NULL}};
const Segment SGI226153[] = {{&S95817[90], 6}, {NULL}};
const Segment SGI226154[] = {{&S95817[0], 14}, {NULL}};
const Segment SGI226155[] = {{&S95819[34], 10}, {NULL}};
const Segment SGI226156[] = {{&S95819[27], 6}, {NULL}};
const Segment SGI226157[] = {{&S95819[97], 9}, {NULL}};
const Segment SGI226158[] = {{&S95819[70], 6}, {NULL}};
const Segment SGI226159[] = {{&S77849[44], 10}, {NULL}};
const Segment SGI226160[] = {{&S77849[44], 21}, {NULL}};
const Segment SGI226161[] = {{&S77849[30], 6}, {NULL}};
const Segment SGI226162[] = {{&S85294[34], 10}, {NULL}};
const Segment SGI226163[] = {{&S85294[27], 6}, {NULL}};
const Segment SGI226164[] = {{&S77849[114], 3}, {NULL}};
const Segment SGI226165[] = {{&S77849[79], 6}, {NULL}};
const Segment SGI226166[] = {{&S85294[97], 9}, {NULL}};
const Segment SGI226167[] = {{&S77849[139], 7}, {NULL}};
const Segment SGI226168[] = {{&S85294[70], 6}, {NULL}};
const Segment SGI226169[] = {{&S95821[34], 6}, {NULL}};
const Segment SGI226170[] = {{&S95821[27], 6}, {NULL}};
const Segment SGI226171[] = {{&S95821[93], 9}, {NULL}};
const Segment SGI226172[] = {{&S95821[66], 6}, {NULL}};
const Segment SGI226173[] = {{&S95822[44], 13}, {NULL}};
const Segment SGI226174[] = {{&S95822[44], 25}, {NULL}};
const Segment SGI226175[] = {{&S95822[30], 6}, {NULL}};
const Segment SGI226176[] = {{&S95822[31], 3}, {NULL}};
const Segment SGI226177[] = {{&S86852[34], 25}, {NULL}};
const Segment SGI226178[] = {{&S85297[34], 13}, {NULL}};
const Segment SGI226179[] = {{&S95822[83], 5}, {NULL}};
const Segment SGI226180[] = {{&S86852[27], 6}, {NULL}};
const Segment SGI226181[] = {{&S85297[27], 6}, {NULL}};
const Segment SGI226182[] = {{&S95822[120], 14}, {NULL}};
const Segment SGI226183[] = {{&S85297[100], 9}, {NULL}};
const Segment SGI226184[] = {{&S86852[112], 9}, {NULL}};
const Segment SGI226185[] = {{&S85297[73], 6}, {NULL}};
const Segment SGI226186[] = {{&S86852[85], 6}, {NULL}};
const Segment SGI226187[] = {{&S77853[34], 11}, {NULL}};
const Segment SGI226188[] = {{&S77853[27], 6}, {NULL}};
const Segment SGI226189[] = {{&S77853[97], 9}, {NULL}};
const Segment SGI226190[] = {{&S77853[70], 6}, {NULL}};
const Segment SGI226191[] = {{&S85298[34], 7}, {NULL}};
const Segment SGI226192[] = {{&S85298[27], 6}, {NULL}};
const Segment SGI226193[] = {{&S85298[102], 9}, {NULL}};
const Segment SGI226194[] = {{&S85298[75], 6}, {NULL}};
const Segment SGI226195[] = {{&S85298[143], 8}, {NULL}};
const Segment SGI226196[] = {{&S85299[34], 17}, {NULL}};
const Segment SGI226197[] = {{&S85299[27], 6}, {NULL}};
const Segment SGI226198[] = {{&S85299[112], 9}, {NULL}};
const Segment SGI226199[] = {{&S85299[85], 6}, {NULL}};
const Segment SGI226200[] = {{&S85300[44], 9}, {NULL}};
const Segment SGI226201[] = {{&S85300[44], 21}, {NULL}};
const Segment SGI226202[] = {{&S85300[30], 6}, {NULL}};
const Segment SGI226203[] = {{&S85300[130], 3}, {NULL}};
const Segment SGI226204[] = {{&S85300[95], 6}, {NULL}};
const Segment SGI226205[] = {{&S85300[67], 8}, {NULL}};
const Segment SGI226206[] = {{&S95829[34], 6}, {NULL}};
const Segment SGI226207[] = {{&S95829[27], 6}, {NULL}};
const Segment SGI226208[] = {{&S95829[93], 9}, {NULL}};
const Segment SGI226209[] = {{&S95829[66], 6}, {NULL}};
const Segment SGI226210[] = {{&S85301[44], 9}, {NULL}};
const Segment SGI226211[] = {{&S85301[44], 21}, {NULL}};
const Segment SGI226212[] = {{&S85301[30], 6}, {NULL}};
const Segment SGI226213[] = {{&S85301[130], 3}, {NULL}};
const Segment SGI226214[] = {{&S85301[95], 6}, {NULL}};
const Segment SGI226215[] = {{&S85301[67], 8}, {NULL}};
const Segment SGI226216[] = {{&S85306[44], 13}, {NULL}};
const Segment SGI226217[] = {{&S85306[44], 25}, {NULL}};
const Segment SGI226218[] = {{&S85306[30], 6}, {NULL}};
const Segment SGI226219[] = {{&S85306[83], 6}, {NULL}};
const Segment SGI226220[] = {{&S85306[110], 13}, {NULL}};
const Segment SGI226221[] = {{&S85307[44], 10}, {NULL}};
const Segment SGI226222[] = {{&S85307[44], 22}, {NULL}};
const Segment SGI226223[] = {{&S85307[30], 6}, {NULL}};
const Segment SGI226224[] = {{&S85307[115], 3}, {NULL}};
const Segment SGI226225[] = {{&S85307[80], 6}, {NULL}};
const Segment SGI226226[] = {{&S95840[34], 9}, {NULL}};
const Segment SGI226227[] = {{&S95840[27], 6}, {NULL}};
const Segment SGI226228[] = {{&S95840[96], 9}, {NULL}};
const Segment SGI226229[] = {{&S95840[69], 6}, {NULL}};
const Segment SGI226230[] = {{&S85309[34], 10}, {NULL}};
const Segment SGI226231[] = {{&S85309[27], 6}, {NULL}};
const Segment SGI226232[] = {{&S95841[34], 10}, {NULL}};
const Segment SGI226233[] = {{&S95841[27], 6}, {NULL}};
const Segment SGI226234[] = {{&S85309[97], 9}, {NULL}};
const Segment SGI226235[] = {{&S85309[70], 6}, {NULL}};
const Segment SGI226236[] = {{&S85309[138], 8}, {NULL}};
const Segment SGI226237[] = {{&S95841[97], 9}, {NULL}};
const Segment SGI226238[] = {{&S95841[70], 6}, {NULL}};
const Segment SGI226239[] = {{&S95842[44], 16}, {NULL}};
const Segment SGI226240[] = {{&S95842[44], 28}, {NULL}};
const Segment SGI226241[] = {{&S95842[30], 6}, {NULL}};
const Segment SGI226242[] = {{&S95842[31], 3}, {NULL}};
const Segment SGI226243[] = {{&S95842[86], 6}, {NULL}};
const Segment SGI226244[] = {{&S95845[44], 14}, {NULL}};
const Segment SGI226245[] = {{&S95845[44], 31}, {NULL}};
const Segment SGI226246[] = {{&S95845[30], 6}, {NULL}};
const Segment SGI226247[] = {{&S85316[34], 7}, {NULL}};
const Segment SGI226248[] = {{&S85316[27], 6}, {NULL}};
const Segment SGI226249[] = {{&S95845[31], 3}, {NULL}};
const Segment SGI226250[] = {{&S95845[89], 5}, {NULL}};
const Segment SGI226251[] = {{&S85316[94], 9}, {NULL}};
const Segment SGI226252[] = {{&S95845[126], 14}, {NULL}};
const Segment SGI226253[] = {{&S85316[67], 6}, {NULL}};
const Segment SGI226254[] = {{&S95846[34], 21}, {NULL}};
const Segment SGI226255[] = {{&S95846[27], 6}, {NULL}};
const Segment SGI226256[] = {{&S95846[108], 9}, {NULL}};
const Segment SGI226257[] = {{&S95846[81], 6}, {NULL}};
const Segment SGI226258[] = {{&S85322[34], 10}, {NULL}};
const Segment SGI226259[] = {{&S85322[27], 6}, {NULL}};
const Segment SGI226260[] = {{&S85322[97], 9}, {NULL}};
const Segment SGI226261[] = {{&S85322[70], 6}, {NULL}};
const Segment SGI226262[] = {{&S85324[44], 10}, {NULL}};
const Segment SGI226263[] = {{&S85324[44], 22}, {NULL}};
const Segment SGI226264[] = {{&S85324[30], 6}, {NULL}};
const Segment SGI226265[] = {{&S85324[80], 6}, {NULL}};
const Segment SGI226266[] = {{&S85324[107], 13}, {NULL}};
const Segment SGI226267[] = {{&S95855[34], 15}, {NULL}};
const Segment SGI226268[] = {{&S95855[27], 6}, {NULL}};
const Segment SGI226269[] = {{&S95855[102], 9}, {NULL}};
const Segment SGI226270[] = {{&S95855[75], 6}, {NULL}};
const Segment SGI226271[] = {{&S85326[34], 10}, {NULL}};
const Segment SGI226272[] = {{&S85326[27], 6}, {NULL}};
const Segment SGI226273[] = {{&S85326[97], 9}, {NULL}};
const Segment SGI226274[] = {{&S85326[70], 6}, {NULL}};
const Segment SGI226275[] = {{&S85326[138], 8}, {NULL}};
const Segment SGI226276[] = {{&S85327[34], 12}, {NULL}};
const Segment SGI226277[] = {{&S85327[27], 6}, {NULL}};
const Segment SGI226278[] = {{&S85327[109], 9}, {NULL}};
const Segment SGI226279[] = {{&S95858[34], 9}, {NULL}};
const Segment SGI226280[] = {{&S85327[82], 6}, {NULL}};
const Segment SGI226281[] = {{&S95858[27], 6}, {NULL}};
const Segment SGI226282[] = {{&S85327[150], 8}, {NULL}};
const Segment SGI226283[] = {{&S95858[96], 9}, {NULL}};
const Segment SGI226284[] = {{&S95858[69], 6}, {NULL}};
const Segment SGI226285[] = {{&S95867[44], 13}, {NULL}};
const Segment SGI226286[] = {{&S95867[44], 25}, {NULL}};
const Segment SGI226287[] = {{&S95867[30], 6}, {NULL}};
const Segment SGI226288[] = {{&S95867[31], 3}, {NULL}};
const Segment SGI226289[] = {{&S95867[83], 5}, {NULL}};
const Segment SGI226290[] = {{&S95867[120], 14}, {NULL}};
const Segment SGI226291[] = {{&S92161[34], 7}, {NULL}};
const Segment SGI226292[] = {{&S92161[27], 6}, {NULL}};
const Segment SGI226293[] = {{&S92161[94], 9}, {NULL}};
const Segment SGI226294[] = {{&S92161[67], 6}, {NULL}};
const Segment SGI226295[] = {{&S95870[44], 12}, {NULL}};
const Segment SGI226296[] = {{&S95870[44], 24}, {NULL}};
const Segment SGI226297[] = {{&S95870[30], 6}, {NULL}};
const Segment SGI226298[] = {{&S95870[82], 6}, {NULL}};
const Segment SGI226299[] = {{&S95870[109], 13}, {NULL}};
const Segment SGI226300[] = {{&S95876[34], 10}, {NULL}};
const Segment SGI226301[] = {{&S95876[27], 6}, {NULL}};
const Segment SGI226302[] = {{&S95876[97], 9}, {NULL}};
const Segment SGI226303[] = {{&S95876[70], 6}, {NULL}};
const Segment SGI226304[] = {{&S95877[44], 9}, {NULL}};
const Segment SGI226305[] = {{&S95877[44], 21}, {NULL}};
const Segment SGI226306[] = {{&S95877[30], 6}, {NULL}};
const Segment SGI226307[] = {{&S95877[31], 3}, {NULL}};
const Segment SGI226308[] = {{&S95877[79], 6}, {NULL}};
const Segment SGI226309[] = {{&S77919[34], 10}, {NULL}};
const Segment SGI226310[] = {{&S77919[27], 6}, {NULL}};
const Segment SGI226311[] = {{&S77919[97], 9}, {NULL}};
const Segment SGI226312[] = {{&S77919[70], 6}, {NULL}};
const Segment SGI226313[] = {{&S85360[44], 17}, {NULL}};
const Segment SGI226314[] = {{&S85360[44], 29}, {NULL}};
const Segment SGI226315[] = {{&S85360[30], 6}, {NULL}};
const Segment SGI226316[] = {{&S85360[122], 3}, {NULL}};
const Segment SGI226317[] = {{&S85360[87], 6}, {NULL}};
const Segment SGI226318[] = {{&S95897[34], 7}, {NULL}};
const Segment SGI226319[] = {{&S95897[27], 6}, {NULL}};
const Segment SGI226320[] = {{&S95897[94], 9}, {NULL}};
const Segment SGI226321[] = {{&S95897[67], 6}, {NULL}};
const Segment SGI226322[] = {{&S95901[34], 12}, {NULL}};
const Segment SGI226323[] = {{&S95901[27], 6}, {NULL}};
const Segment SGI226324[] = {{&S95901[99], 9}, {NULL}};
const Segment SGI226325[] = {{&S95901[72], 6}, {NULL}};
const Segment SGI226326[] = {{&S95903[34], 9}, {NULL}};
const Segment SGI226327[] = {{&S95903[27], 6}, {NULL}};
const Segment SGI226328[] = {{&S95903[96], 9}, {NULL}};
const Segment SGI226329[] = {{&S95903[69], 6}, {NULL}};
const Segment SGI226330[] = {{&S92207[34], 7}, {NULL}};
const Segment SGI226331[] = {{&S92207[27], 6}, {NULL}};
const Segment SGI226332[] = {{&S95911[34], 15}, {NULL}};
const Segment SGI226333[] = {{&S95911[27], 6}, {NULL}};
const Segment SGI226334[] = {{&S92207[94], 9}, {NULL}};
const Segment SGI226335[] = {{&S92207[67], 6}, {NULL}};
const Segment SGI226336[] = {{&S95911[102], 9}, {NULL}};
const Segment SGI226337[] = {{&S95911[75], 6}, {NULL}};
const Segment SGI226338[] = {{&S92208[34], 22}, {NULL}};
const Segment SGI226339[] = {{&S92208[27], 6}, {NULL}};
const Segment SGI226340[] = {{&S92208[109], 9}, {NULL}};
const Segment SGI226341[] = {{&S92208[82], 6}, {NULL}};
const Segment SGI226342[] = {{&S85394[44], 18}, {NULL}};
const Segment SGI226343[] = {{&S85394[44], 39}, {NULL}};
const Segment SGI226344[] = {{&S85394[30], 6}, {NULL}};
const Segment SGI226345[] = {{&S85394[130], 3}, {NULL}};
const Segment SGI226346[] = {{&S85394[97], 5}, {NULL}};
const Segment SGI226347[] = {{&S85394[134], 14}, {NULL}};
const Segment SGI226348[] = {{&S95915[34], 10}, {NULL}};
const Segment SGI226349[] = {{&S95915[27], 6}, {NULL}};
const Segment SGI226350[] = {{&S95915[97], 9}, {NULL}};
const Segment SGI226351[] = {{&S95915[70], 6}, {NULL}};
const Segment SGI226352[] = {{&S85395[44], 18}, {NULL}};
const Segment SGI226353[] = {{&S85395[44], 39}, {NULL}};
const Segment SGI226354[] = {{&S85395[30], 6}, {NULL}};
const Segment SGI226355[] = {{&S85395[130], 3}, {NULL}};
const Segment SGI226356[] = {{&S85395[97], 5}, {NULL}};
const Segment SGI226357[] = {{&S92211[44], 10}, {NULL}};
const Segment SGI226358[] = {{&S92211[44], 27}, {NULL}};
const Segment SGI226359[] = {{&S85395[134], 14}, {NULL}};
const Segment SGI226360[] = {{&S92211[30], 6}, {NULL}};
const Segment SGI226361[] = {{&S92211[126], 3}, {NULL}};
const Segment SGI226362[] = {{&S92211[92], 5}, {NULL}};
const Segment SGI226363[] = {{&S92211[71], 8}, {NULL}};
const Segment SGI226364[] = {{&S92213[34], 15}, {NULL}};
const Segment SGI226365[] = {{&S92213[27], 6}, {NULL}};
const Segment SGI226366[] = {{&S92213[109], 9}, {NULL}};
const Segment SGI226367[] = {{&S92213[82], 6}, {NULL}};
const Segment SGI226368[] = {{&S85404[34], 9}, {NULL}};
const Segment SGI226369[] = {{&S85404[27], 6}, {NULL}};
const Segment SGI226370[] = {{&S85404[96], 9}, {NULL}};
const Segment SGI226371[] = {{&S85404[69], 6}, {NULL}};
const Segment SGI226372[] = {{&S92224[34], 25}, {NULL}};
const Segment SGI226373[] = {{&S92224[27], 6}, {NULL}};
const Segment SGI226374[] = {{&S95932[34], 11}, {NULL}};
const Segment SGI226375[] = {{&S95932[27], 6}, {NULL}};
const Segment SGI226376[] = {{&S92224[112], 9}, {NULL}};
const Segment SGI226377[] = {{&S92224[85], 6}, {NULL}};
const Segment SGI226378[] = {{&S95932[98], 9}, {NULL}};
const Segment SGI226379[] = {{&S95932[71], 6}, {NULL}};
const Segment SGI226380[] = {{&S95933[44], 7}, {NULL}};
const Segment SGI226381[] = {{&S95933[44], 24}, {NULL}};
const Segment SGI226382[] = {{&S95933[30], 6}, {NULL}};
const Segment SGI226383[] = {{&S85413[34], 14}, {NULL}};
const Segment SGI226384[] = {{&S85413[27], 6}, {NULL}};
const Segment SGI226385[] = {{&S95933[82], 6}, {NULL}};
const Segment SGI226386[] = {{&S95933[109], 13}, {NULL}};
const Segment SGI226387[] = {{&S85413[101], 9}, {NULL}};
const Segment SGI226388[] = {{&S85413[74], 6}, {NULL}};
const Segment SGI226389[] = {{&S95939[44], 7}, {NULL}};
const Segment SGI226390[] = {{&S95939[44], 24}, {NULL}};
const Segment SGI226391[] = {{&S95939[30], 6}, {NULL}};
const Segment SGI226392[] = {{&S95939[82], 6}, {NULL}};
const Segment SGI226393[] = {{&S95939[109], 13}, {NULL}};
const Segment SGI226394[] = {{&S92233[44], 29}, {NULL}};
const Segment SGI226395[] = {{&S92233[30], 6}, {NULL}};
const Segment SGI226396[] = {{&S92233[136], 3}, {NULL}};
const Segment SGI226397[] = {{&S92233[101], 6}, {NULL}};
const Segment SGI226398[] = {{&S95940[34], 13}, {NULL}};
const Segment SGI226399[] = {{&S95940[27], 6}, {NULL}};
const Segment SGI226400[] = {{&S95940[100], 9}, {NULL}};
const Segment SGI226401[] = {{&S95940[73], 6}, {NULL}};
const Segment SGI226402[] = {{&S95941[34], 6}, {NULL}};
const Segment SGI226403[] = {{&S95941[27], 6}, {NULL}};
const Segment SGI226404[] = {{&S87001[34], 19}, {NULL}};
const Segment SGI226405[] = {{&S87001[27], 6}, {NULL}};
const Segment SGI226406[] = {{&S95941[93], 9}, {NULL}};
const Segment SGI226407[] = {{&S95941[66], 6}, {NULL}};
const Segment SGI226408[] = {{&S87001[106], 9}, {NULL}};
const Segment SGI226409[] = {{&S87001[79], 6}, {NULL}};
const Segment SGI226410[] = {{&S92241[34], 10}, {NULL}};
const Segment SGI226411[] = {{&S92241[27], 6}, {NULL}};
const Segment SGI226412[] = {{&S92241[97], 9}, {NULL}};
const Segment SGI226413[] = {{&S92241[70], 6}, {NULL}};
const Segment SGI226414[] = {{&S77993[44], 10}, {NULL}};
const Segment SGI226415[] = {{&S77993[44], 21}, {NULL}};
const Segment SGI226416[] = {{&S77993[30], 6}, {NULL}};
const Segment SGI226417[] = {{&S77993[112], 3}, {NULL}};
const Segment SGI226418[] = {{&S77993[79], 5}, {NULL}};
const Segment SGI226419[] = {{&S77994[59], 10}, {NULL}};
const Segment SGI226420[] = {{&S77994[59], 21}, {NULL}};
const Segment SGI226421[] = {{&S77994[45], 6}, {NULL}};
const Segment SGI226422[] = {{&S77994[129], 3}, {NULL}};
const Segment SGI226423[] = {{&S77994[94], 6}, {NULL}};
const Segment SGI226424[] = {{&S92266[34], 13}, {NULL}};
const Segment SGI226425[] = {{&S92266[27], 6}, {NULL}};
const Segment SGI226426[] = {{&S92266[100], 9}, {NULL}};
const Segment SGI226427[] = {{&S92266[73], 6}, {NULL}};
const Segment SGI226428[] = {{&S92267[34], 9}, {NULL}};
const Segment SGI226429[] = {{&S92267[27], 6}, {NULL}};
const Segment SGI226430[] = {{&S92267[96], 9}, {NULL}};
const Segment SGI226431[] = {{&S92267[69], 6}, {NULL}};
const Segment SGI226432[] = {{&S85485[34], 6}, {NULL}};
const Segment SGI226433[] = {{&S85485[27], 6}, {NULL}};
const Segment SGI226434[] = {{&S85485[93], 9}, {NULL}};
const Segment SGI226435[] = {{&S85485[66], 6}, {NULL}};
const Segment SGI226436[] = {{&S85490[59], 10}, {NULL}};
const Segment SGI226437[] = {{&S85490[59], 22}, {NULL}};
const Segment SGI226438[] = {{&S85490[45], 6}, {NULL}};
const Segment SGI226439[] = {{&S85490[130], 3}, {NULL}};
const Segment SGI226440[] = {{&S85490[95], 6}, {NULL}};
const Segment SGI226441[] = {{&S85490[0], 14}, {NULL}};
const Segment SGI226442[] = {{&S85497[44], 8}, {NULL}};
const Segment SGI226443[] = {{&S85497[44], 20}, {NULL}};
const Segment SGI226444[] = {{&S85497[30], 6}, {NULL}};
const Segment SGI226445[] = {{&S85497[113], 3}, {NULL}};
const Segment SGI226446[] = {{&S85497[78], 6}, {NULL}};
const Segment SGI226447[] = {{&S96017[34], 11}, {NULL}};
const Segment SGI226448[] = {{&S96017[27], 6}, {NULL}};
const Segment SGI226449[] = {{&S96017[98], 9}, {NULL}};
const Segment SGI226450[] = {{&S96017[71], 6}, {NULL}};
const Segment SGI226451[] = {{&S96018[44], 12}, {NULL}};
const Segment SGI226452[] = {{&S96018[44], 40}, {NULL}};
const Segment SGI226453[] = {{&S96018[30], 6}, {NULL}};
const Segment SGI226454[] = {{&S96018[31], 3}, {NULL}};
const Segment SGI226455[] = {{&S96018[105], 6}, {NULL}};
const Segment SGI226456[] = {{&S96018[84], 8}, {NULL}};
const Segment SGI226457[] = {{&S85502[34], 8}, {NULL}};
const Segment SGI226458[] = {{&S85502[27], 6}, {NULL}};
const Segment SGI226459[] = {{&S96019[44], 12}, {NULL}};
const Segment SGI226460[] = {{&S96019[44], 40}, {NULL}};
const Segment SGI226461[] = {{&S96019[30], 6}, {NULL}};
const Segment SGI226462[] = {{&S85502[95], 9}, {NULL}};
const Segment SGI226463[] = {{&S85502[68], 6}, {NULL}};
const Segment SGI226464[] = {{&S96019[31], 3}, {NULL}};
const Segment SGI226465[] = {{&S96019[98], 6}, {NULL}};
const Segment SGI226466[] = {{&S96019[158], 7}, {NULL}};
const Segment SGI226467[] = {{&S85507[34], 10}, {NULL}};
const Segment SGI226468[] = {{&S85507[27], 6}, {NULL}};
const Segment SGI226469[] = {{&S85507[97], 9}, {NULL}};
const Segment SGI226470[] = {{&S85507[70], 6}, {NULL}};
const Segment SGI226471[] = {{&S85511[34], 9}, {NULL}};
const Segment SGI226472[] = {{&S85511[27], 6}, {NULL}};
const Segment SGI226473[] = {{&S85511[96], 9}, {NULL}};
const Segment SGI226474[] = {{&S85511[69], 6}, {NULL}};
const Segment SGI226475[] = {{&S85513[34], 10}, {NULL}};
const Segment SGI226476[] = {{&S85513[27], 6}, {NULL}};
const Segment SGI226477[] = {{&S85513[97], 9}, {NULL}};
const Segment SGI226478[] = {{&S85513[70], 6}, {NULL}};
const Segment SGI226479[] = {{&S85514[34], 10}, {NULL}};
const Segment SGI226480[] = {{&S85514[27], 6}, {NULL}};
const Segment SGI226481[] = {{&S96033[59], 10}, {NULL}};
const Segment SGI226482[] = {{&S96033[59], 22}, {NULL}};
const Segment SGI226483[] = {{&S85514[97], 9}, {NULL}};
const Segment SGI226484[] = {{&S96033[45], 6}, {NULL}};
const Segment SGI226485[] = {{&S85514[70], 6}, {NULL}};
const Segment SGI226486[] = {{&S96033[46], 3}, {NULL}};
const Segment SGI226487[] = {{&S96033[95], 6}, {NULL}};
const Segment SGI226488[] = {{&S96033[0], 14}, {NULL}};
const Segment SGI226489[] = {{&S85515[34], 10}, {NULL}};
const Segment SGI226490[] = {{&S85515[27], 6}, {NULL}};
const Segment SGI226491[] = {{&S85515[97], 9}, {NULL}};
const Segment SGI226492[] = {{&S85515[70], 6}, {NULL}};
const Segment SGI226493[] = {{&S85516[44], 9}, {NULL}};
const Segment SGI226494[] = {{&S85516[44], 20}, {NULL}};
const Segment SGI226495[] = {{&S85516[30], 6}, {NULL}};
const Segment SGI226496[] = {{&S85516[113], 3}, {NULL}};
const Segment SGI226497[] = {{&S85516[78], 6}, {NULL}};
const Segment SGI226498[] = {{&S85517[44], 9}, {NULL}};
const Segment SGI226499[] = {{&S85517[44], 20}, {NULL}};
const Segment SGI226500[] = {{&S85517[30], 6}, {NULL}};
const Segment SGI226501[] = {{&S85517[113], 3}, {NULL}};
const Segment SGI226502[] = {{&S85517[78], 6}, {NULL}};
const Segment SGI226503[] = {{&S85518[44], 15}, {NULL}};
const Segment SGI226504[] = {{&S85518[44], 27}, {NULL}};
const Segment SGI226505[] = {{&S85518[30], 6}, {NULL}};
const Segment SGI226506[] = {{&S85518[85], 6}, {NULL}};
const Segment SGI226507[] = {{&S85518[112], 13}, {NULL}};
const Segment SGI226508[] = {{&S92346[34], 13}, {NULL}};
const Segment SGI226509[] = {{&S85524[34], 10}, {NULL}};
const Segment SGI226510[] = {{&S92346[27], 6}, {NULL}};
const Segment SGI226511[] = {{&S85524[27], 6}, {NULL}};
const Segment SGI226512[] = {{&S92346[100], 9}, {NULL}};
const Segment SGI226513[] = {{&S92346[73], 6}, {NULL}};
const Segment SGI226514[] = {{&S85524[97], 9}, {NULL}};
const Segment SGI226515[] = {{&S85524[70], 6}, {NULL}};
const Segment SGI226516[] = {{&S96055[34], 15}, {NULL}};
const Segment SGI226517[] = {{&S96055[27], 6}, {NULL}};
const Segment SGI226518[] = {{&S96055[102], 9}, {NULL}};
const Segment SGI226519[] = {{&S96055[75], 6}, {NULL}};
const Segment SGI226520[] = {{&S92359[34], 7}, {NULL}};
const Segment SGI226521[] = {{&S92359[27], 6}, {NULL}};
const Segment SGI226522[] = {{&S92359[94], 9}, {NULL}};
const Segment SGI226523[] = {{&S92359[67], 6}, {NULL}};
const Segment SGI226524[] = {{&S85539[34], 9}, {NULL}};
const Segment SGI226525[] = {{&S85539[27], 6}, {NULL}};
const Segment SGI226526[] = {{&S96059[34], 9}, {NULL}};
const Segment SGI226527[] = {{&S96059[27], 6}, {NULL}};
const Segment SGI226528[] = {{&S85539[96], 9}, {NULL}};
const Segment SGI226529[] = {{&S85539[69], 6}, {NULL}};
const Segment SGI226530[] = {{&S96059[102], 9}, {NULL}};
const Segment SGI226531[] = {{&S96059[75], 6}, {NULL}};
const Segment SGI226532[] = {{&S96062[59], 10}, {NULL}};
const Segment SGI226533[] = {{&S96062[59], 22}, {NULL}};
const Segment SGI226534[] = {{&S96062[45], 6}, {NULL}};
const Segment SGI226535[] = {{&S96062[46], 3}, {NULL}};
const Segment SGI226536[] = {{&S96062[95], 6}, {NULL}};
const Segment SGI226537[] = {{&S96062[0], 14}, {NULL}};
const Segment SGI226538[] = {{&S96063[44], 10}, {NULL}};
const Segment SGI226539[] = {{&S96063[44], 22}, {NULL}};
const Segment SGI226540[] = {{&S96063[30], 6}, {NULL}};
const Segment SGI226541[] = {{&S96063[31], 3}, {NULL}};
const Segment SGI226542[] = {{&S96063[80], 6}, {NULL}};
const Segment SGI226543[] = {{&S96063[132], 10}, {NULL}};
const Segment SGI226544[] = {{&S92371[44], 6}, {NULL}};
const Segment SGI226545[] = {{&S92371[44], 18}, {NULL}};
const Segment SGI226546[] = {{&S92371[30], 6}, {NULL}};
const Segment SGI226547[] = {{&S92371[133], 3}, {NULL}};
const Segment SGI226548[] = {{&S92371[98], 6}, {NULL}};
const Segment SGI226549[] = {{&S92371[64], 8}, {NULL}};
const Segment SGI226550[] = {{&S96067[44], 10}, {NULL}};
const Segment SGI226551[] = {{&S96067[44], 22}, {NULL}};
const Segment SGI226552[] = {{&S96067[30], 6}, {NULL}};
const Segment SGI226553[] = {{&S96067[31], 3}, {NULL}};
const Segment SGI226554[] = {{&S96067[80], 6}, {NULL}};
const Segment SGI226555[] = {{&S96067[133], 11}, {NULL}};
const Segment SGI226556[] = {{&S92380[34], 6}, {NULL}};
const Segment SGI226557[] = {{&S92380[27], 6}, {NULL}};
const Segment SGI226558[] = {{&S92380[93], 9}, {NULL}};
const Segment SGI226559[] = {{&S92380[66], 6}, {NULL}};
const Segment SGI226560[] = {{&S96078[44], 38}, {NULL}};
const Segment SGI226561[] = {{&S96078[44], 50}, {NULL}};
const Segment SGI226562[] = {{&S96078[30], 6}, {NULL}};
const Segment SGI226563[] = {{&S96078[31], 3}, {NULL}};
const Segment SGI226564[] = {{&S96078[108], 6}, {NULL}};
const Segment SGI226565[] = {{&S92384[34], 15}, {NULL}};
const Segment SGI226566[] = {{&S92384[27], 6}, {NULL}};
const Segment SGI226567[] = {{&S92384[102], 9}, {NULL}};
const Segment SGI226568[] = {{&S92384[75], 6}, {NULL}};
const Segment SGI226569[] = {{&S92385[34], 17}, {NULL}};
const Segment SGI226570[] = {{&S92385[27], 6}, {NULL}};
const Segment SGI226571[] = {{&S92385[104], 9}, {NULL}};
const Segment SGI226572[] = {{&S92385[77], 6}, {NULL}};
const Segment SGI226573[] = {{&S96082[45], 9}, {NULL}};
const Segment SGI226574[] = {{&S96082[30], 6}, {NULL}};
const Segment SGI226575[] = {{&S96082[31], 3}, {NULL}};
const Segment SGI226576[] = {{&S92387[44], 14}, {NULL}};
const Segment SGI226577[] = {{&S96082[80], 5}, {NULL}};
const Segment SGI226578[] = {{&S92387[44], 26}, {NULL}};
const Segment SGI226579[] = {{&S92387[30], 6}, {NULL}};
const Segment SGI226580[] = {{&S96082[138], 13}, {NULL}};
const Segment SGI226581[] = {{&S92387[119], 3}, {NULL}};
const Segment SGI226582[] = {{&S92387[84], 6}, {NULL}};
const Segment SGI226583[] = {{&S87155[34], 17}, {NULL}};
const Segment SGI226584[] = {{&S87155[27], 6}, {NULL}};
const Segment SGI226585[] = {{&S87155[104], 9}, {NULL}};
const Segment SGI226586[] = {{&S87155[77], 6}, {NULL}};
const Segment SGI226587[] = {{&S96083[45], 9}, {NULL}};
const Segment SGI226588[] = {{&S96083[30], 6}, {NULL}};
const Segment SGI226589[] = {{&S96083[31], 3}, {NULL}};
const Segment SGI226590[] = {{&S96083[80], 5}, {NULL}};
const Segment SGI226591[] = {{&S96083[138], 13}, {NULL}};
const Segment SGI226592[] = {{&S87157[34], 6}, {NULL}};
const Segment SGI226593[] = {{&S87157[27], 6}, {NULL}};
const Segment SGI226594[] = {{&S87157[93], 9}, {NULL}};
const Segment SGI226595[] = {{&S87157[66], 6}, {NULL}};
const Segment SGI226596[] = {{&S96085[34], 4}, {NULL}};
const Segment SGI226597[] = {{&S96085[27], 6}, {NULL}};
const Segment SGI226598[] = {{&S96085[91], 9}, {NULL}};
const Segment SGI226599[] = {{&S96085[64], 6}, {NULL}};
const Segment SGI226600[] = {{&S92392[44], 14}, {NULL}};
const Segment SGI226601[] = {{&S92392[44], 26}, {NULL}};
const Segment SGI226602[] = {{&S92392[30], 6}, {NULL}};
const Segment SGI226603[] = {{&S92392[119], 3}, {NULL}};
const Segment SGI226604[] = {{&S92392[84], 6}, {NULL}};
const Segment SGI226605[] = {{&S96086[34], 9}, {NULL}};
const Segment SGI226606[] = {{&S96086[27], 6}, {NULL}};
const Segment SGI226607[] = {{&S96086[96], 9}, {NULL}};
const Segment SGI226608[] = {{&S96086[69], 6}, {NULL}};
const Segment SGI226609[] = {{&S96087[34], 14}, {NULL}};
const Segment SGI226610[] = {{&S96087[27], 6}, {NULL}};
const Segment SGI226611[] = {{&S96087[101], 9}, {NULL}};
const Segment SGI226612[] = {{&S96087[74], 6}, {NULL}};
const Segment SGI226613[] = {{&S96091[34], 11}, {NULL}};
const Segment SGI226614[] = {{&S96091[27], 6}, {NULL}};
const Segment SGI226615[] = {{&S96091[98], 9}, {NULL}};
const Segment SGI226616[] = {{&S96091[71], 6}, {NULL}};
const Segment SGI226617[] = {{&S96101[45], 10}, {NULL}};
const Segment SGI226618[] = {{&S96101[30], 6}, {NULL}};
const Segment SGI226619[] = {{&S96101[31], 3}, {NULL}};
const Segment SGI226620[] = {{&S96101[86], 5}, {NULL}};
const Segment SGI226621[] = {{&S92413[44], 13}, {NULL}};
const Segment SGI226622[] = {{&S92413[44], 25}, {NULL}};
const Segment SGI226623[] = {{&S92413[30], 6}, {NULL}};
const Segment SGI226624[] = {{&S96101[144], 13}, {NULL}};
const Segment SGI226625[] = {{&S92413[118], 3}, {NULL}};
const Segment SGI226626[] = {{&S92413[83], 6}, {NULL}};
const Segment SGI226627[] = {{&S96102[34], 6}, {NULL}};
const Segment SGI226628[] = {{&S96102[27], 6}, {NULL}};
const Segment SGI226629[] = {{&S96102[93], 9}, {NULL}};
const Segment SGI226630[] = {{&S96102[66], 6}, {NULL}};
const Segment SGI226631[] = {{&S85605[44], 10}, {NULL}};
const Segment SGI226632[] = {{&S85605[44], 22}, {NULL}};
const Segment SGI226633[] = {{&S85605[30], 6}, {NULL}};
const Segment SGI226634[] = {{&S85605[115], 3}, {NULL}};
const Segment SGI226635[] = {{&S85605[80], 6}, {NULL}};
const Segment SGI226636[] = {{&S96103[45], 14}, {NULL}};
const Segment SGI226637[] = {{&S85605[140], 11}, {NULL}};
const Segment SGI226638[] = {{&S96103[30], 6}, {NULL}};
const Segment SGI226639[] = {{&S96103[31], 3}, {NULL}};
const Segment SGI226640[] = {{&S96103[85], 5}, {NULL}};
const Segment SGI226641[] = {{&S96103[143], 13}, {NULL}};
const Segment SGI226642[] = {{&S96104[44], 14}, {NULL}};
const Segment SGI226643[] = {{&S96104[44], 26}, {NULL}};
const Segment SGI226644[] = {{&S96104[30], 6}, {NULL}};
const Segment SGI226645[] = {{&S96104[84], 6}, {NULL}};
const Segment SGI226646[] = {{&S96104[111], 13}, {NULL}};
const Segment SGI226647[] = {{&S85609[44], 10}, {NULL}};
const Segment SGI226648[] = {{&S85609[44], 22}, {NULL}};
const Segment SGI226649[] = {{&S85609[30], 6}, {NULL}};
const Segment SGI226650[] = {{&S85609[113], 3}, {NULL}};
const Segment SGI226651[] = {{&S85609[80], 5}, {NULL}};
const Segment SGI226652[] = {{&S92423[44], 13}, {NULL}};
const Segment SGI226653[] = {{&S92423[44], 25}, {NULL}};
const Segment SGI226654[] = {{&S92423[30], 6}, {NULL}};
const Segment SGI226655[] = {{&S92423[118], 3}, {NULL}};
const Segment SGI226656[] = {{&S92423[83], 6}, {NULL}};
const Segment SGI226657[] = {{&S96108[34], 14}, {NULL}};
const Segment SGI226658[] = {{&S96108[27], 6}, {NULL}};
const Segment SGI226659[] = {{&S96108[101], 9}, {NULL}};
const Segment SGI226660[] = {{&S96108[74], 6}, {NULL}};
const Segment SGI226661[] = {{&S85616[34], 10}, {NULL}};
const Segment SGI226662[] = {{&S85616[27], 6}, {NULL}};
const Segment SGI226663[] = {{&S85616[97], 9}, {NULL}};
const Segment SGI226664[] = {{&S85616[70], 6}, {NULL}};
const Segment SGI226665[] = {{&S96112[59], 11}, {NULL}};
const Segment SGI226666[] = {{&S96112[59], 23}, {NULL}};
const Segment SGI226667[] = {{&S96112[45], 6}, {NULL}};
const Segment SGI226668[] = {{&S96112[46], 3}, {NULL}};
const Segment SGI226669[] = {{&S96112[96], 6}, {NULL}};
const Segment SGI226670[] = {{&S85617[34], 9}, {NULL}};
const Segment SGI226671[] = {{&S96112[0], 14}, {NULL}};
const Segment SGI226672[] = {{&S85617[27], 6}, {NULL}};
const Segment SGI226673[] = {{&S85617[96], 9}, {NULL}};
const Segment SGI226674[] = {{&S85617[69], 6}, {NULL}};
const Segment SGI226675[] = {{&S85618[34], 10}, {NULL}};
const Segment SGI226676[] = {{&S85618[27], 6}, {NULL}};
const Segment SGI226677[] = {{&S96114[44], 10}, {NULL}};
const Segment SGI226678[] = {{&S96114[44], 22}, {NULL}};
const Segment SGI226679[] = {{&S96114[30], 6}, {NULL}};
const Segment SGI226680[] = {{&S85618[97], 9}, {NULL}};
const Segment SGI226681[] = {{&S85618[70], 6}, {NULL}};
const Segment SGI226682[] = {{&S96114[31], 3}, {NULL}};
const Segment SGI226683[] = {{&S96114[80], 6}, {NULL}};
const Segment SGI226684[] = {{&S96114[140], 11}, {NULL}};
const Segment SGI226685[] = {{&S85619[44], 8}, {NULL}};
const Segment SGI226686[] = {{&S85619[44], 20}, {NULL}};
const Segment SGI226687[] = {{&S85619[30], 6}, {NULL}};
const Segment SGI226688[] = {{&S85619[113], 3}, {NULL}};
const Segment SGI226689[] = {{&S85619[78], 6}, {NULL}};
const Segment SGI226690[] = {{&S96117[34], 7}, {NULL}};
const Segment SGI226691[] = {{&S96117[27], 6}, {NULL}};
const Segment SGI226692[] = {{&S96117[94], 9}, {NULL}};
const Segment SGI226693[] = {{&S96117[67], 6}, {NULL}};
const Segment SGI226694[] = {{&S96119[44], 11}, {NULL}};
const Segment SGI226695[] = {{&S96119[44], 23}, {NULL}};
const Segment SGI226696[] = {{&S96119[30], 6}, {NULL}};
const Segment SGI226697[] = {{&S96119[31], 3}, {NULL}};
const Segment SGI226698[] = {{&S96119[81], 6}, {NULL}};
const Segment SGI226699[] = {{&S96123[45], 4}, {NULL}};
const Segment SGI226700[] = {{&S96123[30], 6}, {NULL}};
const Segment SGI226701[] = {{&S96123[31], 3}, {NULL}};
const Segment SGI226702[] = {{&S85629[34], 10}, {NULL}};
const Segment SGI226703[] = {{&S96123[75], 5}, {NULL}};
const Segment SGI226704[] = {{&S85629[27], 6}, {NULL}};
const Segment SGI226705[] = {{&S96123[133], 13}, {NULL}};
const Segment SGI226706[] = {{&S85629[97], 9}, {NULL}};
const Segment SGI226707[] = {{&S85629[70], 6}, {NULL}};
const Segment SGI226708[] = {{&S96125[44], 7}, {NULL}};
const Segment SGI226709[] = {{&S96125[44], 19}, {NULL}};
const Segment SGI226710[] = {{&S96125[30], 6}, {NULL}};
const Segment SGI226711[] = {{&S96125[31], 3}, {NULL}};
const Segment SGI226712[] = {{&S96125[77], 6}, {NULL}};
const Segment SGI226713[] = {{&S85632[34], 18}, {NULL}};
const Segment SGI226714[] = {{&S85632[27], 6}, {NULL}};
const Segment SGI226715[] = {{&S85632[105], 9}, {NULL}};
const Segment SGI226716[] = {{&S85632[78], 6}, {NULL}};
const Segment SGI226717[] = {{&S96127[44], 6}, {NULL}};
const Segment SGI226718[] = {{&S96127[44], 18}, {NULL}};
const Segment SGI226719[] = {{&S96127[30], 6}, {NULL}};
const Segment SGI226720[] = {{&S96127[31], 3}, {NULL}};
const Segment SGI226721[] = {{&S96127[76], 5}, {NULL}};
const Segment SGI226722[] = {{&S96127[113], 14}, {NULL}};
const Segment SGI226723[] = {{&S96131[34], 8}, {NULL}};
const Segment SGI226724[] = {{&S96131[27], 6}, {NULL}};
const Segment SGI226725[] = {{&S96131[95], 9}, {NULL}};
const Segment SGI226726[] = {{&S96131[68], 6}, {NULL}};
const Segment SGI226727[] = {{&S85640[44], 10}, {NULL}};
const Segment SGI226728[] = {{&S85640[44], 22}, {NULL}};
const Segment SGI226729[] = {{&S85640[30], 6}, {NULL}};
const Segment SGI226730[] = {{&S85640[80], 6}, {NULL}};
const Segment SGI226731[] = {{&S85640[107], 13}, {NULL}};
const Segment SGI226732[] = {{&S96134[44], 6}, {NULL}};
const Segment SGI226733[] = {{&S96134[44], 18}, {NULL}};
const Segment SGI226734[] = {{&S96134[30], 6}, {NULL}};
const Segment SGI226735[] = {{&S96134[31], 3}, {NULL}};
const Segment SGI226736[] = {{&S96134[76], 6}, {NULL}};
const Segment SGI226737[] = {{&S96139[34], 17}, {NULL}};
const Segment SGI226738[] = {{&S96139[27], 6}, {NULL}};
const Segment SGI226739[] = {{&S96139[104], 9}, {NULL}};
const Segment SGI226740[] = {{&S96139[77], 6}, {NULL}};
const Segment SGI226741[] = {{&S96141[34], 12}, {NULL}};
const Segment SGI226742[] = {{&S96141[27], 6}, {NULL}};
const Segment SGI226743[] = {{&S96141[97], 9}, {NULL}};
const Segment SGI226744[] = {{&S96141[70], 6}, {NULL}};
const Segment SGI226745[] = {{&S96142[34], 6}, {NULL}};
const Segment SGI226746[] = {{&S96142[27], 6}, {NULL}};
const Segment SGI226747[] = {{&S96142[93], 9}, {NULL}};
const Segment SGI226748[] = {{&S96142[66], 6}, {NULL}};
const Segment SGI226749[] = {{&S96149[34], 12}, {NULL}};
const Segment SGI226750[] = {{&S96149[27], 6}, {NULL}};
const Segment SGI226751[] = {{&S96149[97], 9}, {NULL}};
const Segment SGI226752[] = {{&S96149[70], 6}, {NULL}};
const Segment SGI226753[] = {{&S96150[27], 6}, {NULL}};
const Segment SGI226754[] = {{&S96150[99], 9}, {NULL}};
const Segment SGI226755[] = {{&S96150[72], 6}, {NULL}};
const Segment SGI226756[] = {{&S96151[44], 10}, {NULL}};
const Segment SGI226757[] = {{&S96151[44], 22}, {NULL}};
const Segment SGI226758[] = {{&S96151[30], 6}, {NULL}};
const Segment SGI226759[] = {{&S96151[31], 3}, {NULL}};
const Segment SGI226760[] = {{&S96151[80], 5}, {NULL}};
const Segment SGI226761[] = {{&S96151[117], 14}, {NULL}};
const Segment SGI226762[] = {{&S96152[34], 5}, {NULL}};
const Segment SGI226763[] = {{&S96152[27], 6}, {NULL}};
const Segment SGI226764[] = {{&S96152[96], 9}, {NULL}};
const Segment SGI226765[] = {{&S96152[69], 6}, {NULL}};
const Segment SGI226766[] = {{&S96153[34], 7}, {NULL}};
const Segment SGI226767[] = {{&S96153[27], 6}, {NULL}};
const Segment SGI226768[] = {{&S96153[94], 9}, {NULL}};
const Segment SGI226769[] = {{&S96153[67], 6}, {NULL}};
const Segment SGI226770[] = {{&S78098[34], 8}, {NULL}};
const Segment SGI226771[] = {{&S78098[27], 6}, {NULL}};
const Segment SGI226772[] = {{&S96154[34], 14}, {NULL}};
const Segment SGI226773[] = {{&S78098[94], 9}, {NULL}};
const Segment SGI226774[] = {{&S96154[27], 6}, {NULL}};
const Segment SGI226775[] = {{&S78098[67], 6}, {NULL}};
const Segment SGI226776[] = {{&S96154[106], 9}, {NULL}};
const Segment SGI226777[] = {{&S96154[79], 6}, {NULL}};
const Segment SGI226778[] = {{&S78099[34], 8}, {NULL}};
const Segment SGI226779[] = {{&S78099[27], 6}, {NULL}};
const Segment SGI226780[] = {{&S78099[94], 9}, {NULL}};
const Segment SGI226781[] = {{&S78099[67], 6}, {NULL}};
const Segment SGI226782[] = {{&S96156[34], 14}, {NULL}};
const Segment SGI226783[] = {{&S96156[27], 6}, {NULL}};
const Segment SGI226784[] = {{&S96156[101], 9}, {NULL}};
const Segment SGI226785[] = {{&S96156[74], 6}, {NULL}};
const Segment SGI226786[] = {{&S96158[34], 10}, {NULL}};
const Segment SGI226787[] = {{&S96158[27], 6}, {NULL}};
const Segment SGI226788[] = {{&S96158[97], 9}, {NULL}};
const Segment SGI226789[] = {{&S96158[70], 6}, {NULL}};
const Segment SGI226790[] = {{&S96159[34], 20}, {NULL}};
const Segment SGI226791[] = {{&S96159[27], 6}, {NULL}};
const Segment SGI226792[] = {{&S87271[34], 16}, {NULL}};
const Segment SGI226793[] = {{&S96159[107], 9}, {NULL}};
const Segment SGI226794[] = {{&S96159[80], 6}, {NULL}};
const Segment SGI226795[] = {{&S87271[27], 6}, {NULL}};
const Segment SGI226796[] = {{&S87271[103], 9}, {NULL}};
const Segment SGI226797[] = {{&S87271[76], 6}, {NULL}};
const Segment SGI226798[] = {{&S87271[144], 8}, {NULL}};
const Segment SGI226799[] = {{&S87272[34], 16}, {NULL}};
const Segment SGI226800[] = {{&S87272[27], 6}, {NULL}};
const Segment SGI226801[] = {{&S87272[103], 9}, {NULL}};
const Segment SGI226802[] = {{&S87272[76], 6}, {NULL}};
const Segment SGI226803[] = {{&S96165[44], 10}, {NULL}};
const Segment SGI226804[] = {{&S96165[44], 22}, {NULL}};
const Segment SGI226805[] = {{&S96165[30], 6}, {NULL}};
const Segment SGI226806[] = {{&S96165[31], 3}, {NULL}};
const Segment SGI226807[] = {{&S96165[80], 5}, {NULL}};
const Segment SGI226808[] = {{&S96165[117], 14}, {NULL}};
const Segment SGI226809[] = {{&S96166[44], 7}, {NULL}};
const Segment SGI226810[] = {{&S96166[44], 19}, {NULL}};
const Segment SGI226811[] = {{&S96166[30], 6}, {NULL}};
const Segment SGI226812[] = {{&S96166[31], 3}, {NULL}};
const Segment SGI226813[] = {{&S96166[77], 6}, {NULL}};
const Segment SGI226814[] = {{&S96167[34], 8}, {NULL}};
const Segment SGI226815[] = {{&S96167[27], 6}, {NULL}};
const Segment SGI226816[] = {{&S96167[95], 9}, {NULL}};
const Segment SGI226817[] = {{&S96167[68], 6}, {NULL}};
const Segment SGI226818[] = {{&S96169[34], 8}, {NULL}};
const Segment SGI226819[] = {{&S96169[27], 6}, {NULL}};
const Segment SGI226820[] = {{&S96169[95], 9}, {NULL}};
const Segment SGI226821[] = {{&S96169[68], 6}, {NULL}};
const Segment SGI226822[] = {{&S96169[136], 8}, {NULL}};
const Segment SGI226823[] = {{&S96171[34], 6}, {NULL}};
const Segment SGI226824[] = {{&S92524[44], 9}, {NULL}};
const Segment SGI226825[] = {{&S96171[27], 6}, {NULL}};
const Segment SGI226826[] = {{&S92524[44], 21}, {NULL}};
const Segment SGI226827[] = {{&S92524[30], 6}, {NULL}};
const Segment SGI226828[] = {{&S96171[93], 9}, {NULL}};
const Segment SGI226829[] = {{&S96171[66], 6}, {NULL}};
const Segment SGI226830[] = {{&S92524[135], 3}, {NULL}};
const Segment SGI226831[] = {{&S92524[100], 6}, {NULL}};
const Segment SGI226832[] = {{&S96173[34], 8}, {NULL}};
const Segment SGI226833[] = {{&S96173[27], 6}, {NULL}};
const Segment SGI226834[] = {{&S96173[95], 9}, {NULL}};
const Segment SGI226835[] = {{&S96173[68], 6}, {NULL}};
const Segment SGI226836[] = {{&S85706[34], 10}, {NULL}};
const Segment SGI226837[] = {{&S85706[27], 6}, {NULL}};
const Segment SGI226838[] = {{&S85706[97], 9}, {NULL}};
const Segment SGI226839[] = {{&S85706[70], 6}, {NULL}};
const Segment SGI226840[] = {{&S85707[34], 10}, {NULL}};
const Segment SGI226841[] = {{&S85707[27], 6}, {NULL}};
const Segment SGI226842[] = {{&S85707[97], 9}, {NULL}};
const Segment SGI226843[] = {{&S85707[70], 6}, {NULL}};
const Segment SGI226844[] = {{&S96176[34], 11}, {NULL}};
const Segment SGI226845[] = {{&S96176[27], 6}, {NULL}};
const Segment SGI226846[] = {{&S96176[98], 9}, {NULL}};
const Segment SGI226847[] = {{&S96176[71], 6}, {NULL}};
const Segment SGI226848[] = {{&S92534[34], 11}, {NULL}};
const Segment SGI226849[] = {{&S92534[27], 6}, {NULL}};
const Segment SGI226850[] = {{&S92534[98], 9}, {NULL}};
const Segment SGI226851[] = {{&S92534[71], 6}, {NULL}};
const Segment SGI226852[] = {{&S96181[34], 7}, {NULL}};
const Segment SGI226853[] = {{&S96181[27], 6}, {NULL}};
const Segment SGI226854[] = {{&S96181[94], 9}, {NULL}};
const Segment SGI226855[] = {{&S96181[67], 6}, {NULL}};
const Segment SGI226856[] = {{&S85718[44], 10}, {NULL}};
const Segment SGI226857[] = {{&S85718[44], 22}, {NULL}};
const Segment SGI226858[] = {{&S85718[30], 6}, {NULL}};
const Segment SGI226859[] = {{&S85718[115], 3}, {NULL}};
const Segment SGI226860[] = {{&S85718[80], 6}, {NULL}};
const Segment SGI226861[] = {{&S85718[133], 11}, {NULL}};
const Segment SGI226862[] = {{&S96187[44], 24}, {NULL}};
const Segment SGI226863[] = {{&S96187[44], 38}, {NULL}};
const Segment SGI226864[] = {{&S85719[44], 10}, {NULL}};
const Segment SGI226865[] = {{&S96187[30], 6}, {NULL}};
const Segment SGI226866[] = {{&S85719[44], 22}, {NULL}};
const Segment SGI226867[] = {{&S85719[30], 6}, {NULL}};
const Segment SGI226868[] = {{&S85719[115], 3}, {NULL}};
const Segment SGI226869[] = {{&S96187[96], 6}, {NULL}};
const Segment SGI226870[] = {{&S85719[80], 6}, {NULL}};
const Segment SGI226871[] = {{&S96187[123], 13}, {NULL}};
const Segment SGI226872[] = {{&S85719[134], 12}, {NULL}};
const Segment SGI226873[] = {{&S99799[34], 7}, {NULL}};
const Segment SGI226874[] = {{&S99799[27], 6}, {NULL}};
const Segment SGI226875[] = {{&S99799[67], 6}, {NULL}};
const Segment SGI226876[] = {{&S85723[34], 13}, {NULL}};
const Segment SGI226877[] = {{&S85723[27], 6}, {NULL}};
const Segment SGI226878[] = {{&S85723[100], 9}, {NULL}};
const Segment SGI226879[] = {{&S85723[73], 6}, {NULL}};
const Segment SGI226880[] = {{&S96192[34], 7}, {NULL}};
const Segment SGI226881[] = {{&S96192[27], 6}, {NULL}};
const Segment SGI226882[] = {{&S85724[34], 7}, {NULL}};
const Segment SGI226883[] = {{&S96192[94], 9}, {NULL}};
const Segment SGI226884[] = {{&S85724[27], 6}, {NULL}};
const Segment SGI226885[] = {{&S96192[67], 6}, {NULL}};
const Segment SGI226886[] = {{&S96192[135], 8}, {NULL}};
const Segment SGI226887[] = {{&S85724[94], 9}, {NULL}};
const Segment SGI226888[] = {{&S85724[67], 6}, {NULL}};
const Segment SGI226889[] = {{&S92554[34], 16}, {NULL}};
const Segment SGI226890[] = {{&S92554[27], 6}, {NULL}};
const Segment SGI226891[] = {{&S92554[103], 9}, {NULL}};
const Segment SGI226892[] = {{&S92554[76], 6}, {NULL}};
const Segment SGI226893[] = {{&S96195[44], 7}, {NULL}};
const Segment SGI226894[] = {{&S96195[44], 19}, {NULL}};
const Segment SGI226895[] = {{&S96195[30], 6}, {NULL}};
const Segment SGI226896[] = {{&S96195[31], 3}, {NULL}};
const Segment SGI226897[] = {{&S96195[77], 6}, {NULL}};
const Segment SGI226898[] = {{&S96195[129], 10}, {NULL}};
const Segment SGI226899[] = {{&S85728[34], 10}, {NULL}};
const Segment SGI226900[] = {{&S85728[27], 6}, {NULL}};
const Segment SGI226901[] = {{&S85728[97], 9}, {NULL}};
const Segment SGI226902[] = {{&S85728[70], 6}, {NULL}};
const Segment SGI226903[] = {{&S85729[34], 5}, {NULL}};
const Segment SGI226904[] = {{&S85729[27], 6}, {NULL}};
const Segment SGI226905[] = {{&S85729[92], 9}, {NULL}};
const Segment SGI226906[] = {{&S85729[65], 6}, {NULL}};
const Segment SGI226907[] = {{&S96199[44], 12}, {NULL}};
const Segment SGI226908[] = {{&S96199[44], 45}, {NULL}};
const Segment SGI226909[] = {{&S96199[30], 6}, {NULL}};
const Segment SGI226910[] = {{&S96199[31], 3}, {NULL}};
const Segment SGI226911[] = {{&S96199[103], 6}, {NULL}};
const Segment SGI226912[] = {{&S92561[34], 14}, {NULL}};
const Segment SGI226913[] = {{&S92561[27], 6}, {NULL}};
const Segment SGI226914[] = {{&S92561[101], 9}, {NULL}};
const Segment SGI226915[] = {{&S92561[74], 6}, {NULL}};
const Segment SGI226916[] = {{&S85733[44], 7}, {NULL}};
const Segment SGI226917[] = {{&S85733[44], 19}, {NULL}};
const Segment SGI226918[] = {{&S85733[30], 6}, {NULL}};
const Segment SGI226919[] = {{&S85733[77], 6}, {NULL}};
const Segment SGI226920[] = {{&S85733[104], 13}, {NULL}};
const Segment SGI226921[] = {{&S96201[44], 10}, {NULL}};
const Segment SGI226922[] = {{&S96201[44], 22}, {NULL}};
const Segment SGI226923[] = {{&S96201[30], 6}, {NULL}};
const Segment SGI226924[] = {{&S96201[31], 3}, {NULL}};
const Segment SGI226925[] = {{&S96201[95], 6}, {NULL}};
const Segment SGI226926[] = {{&S96201[68], 8}, {NULL}};
const Segment SGI226927[] = {{&S85734[44], 8}, {NULL}};
const Segment SGI226928[] = {{&S85734[44], 20}, {NULL}};
const Segment SGI226929[] = {{&S85734[30], 6}, {NULL}};
const Segment SGI226930[] = {{&S85734[78], 6}, {NULL}};
const Segment SGI226931[] = {{&S85734[105], 13}, {NULL}};
const Segment SGI226932[] = {{&S85737[34], 11}, {NULL}};
const Segment SGI226933[] = {{&S85737[27], 6}, {NULL}};
const Segment SGI226934[] = {{&S85737[107], 9}, {NULL}};
const Segment SGI226935[] = {{&S85737[80], 6}, {NULL}};
const Segment SGI226936[] = {{&S96207[44], 9}, {NULL}};
const Segment SGI226937[] = {{&S96207[44], 21}, {NULL}};
const Segment SGI226938[] = {{&S96207[30], 6}, {NULL}};
const Segment SGI226939[] = {{&S96207[31], 3}, {NULL}};
const Segment SGI226940[] = {{&S96207[79], 6}, {NULL}};
const Segment SGI226941[] = {{&S85740[44], 8}, {NULL}};
const Segment SGI226942[] = {{&S85740[44], 20}, {NULL}};
const Segment SGI226943[] = {{&S85740[30], 6}, {NULL}};
const Segment SGI226944[] = {{&S85740[113], 3}, {NULL}};
const Segment SGI226945[] = {{&S85740[78], 6}, {NULL}};
const Segment SGI226946[] = {{&S96211[34], 12}, {NULL}};
const Segment SGI226947[] = {{&S96211[27], 6}, {NULL}};
const Segment SGI226948[] = {{&S96211[99], 9}, {NULL}};
const Segment SGI226949[] = {{&S96211[72], 6}, {NULL}};
const Segment SGI226950[] = {{&S85744[34], 10}, {NULL}};
const Segment SGI226951[] = {{&S85744[27], 6}, {NULL}};
const Segment SGI226952[] = {{&S85744[97], 9}, {NULL}};
const Segment SGI226953[] = {{&S85744[70], 6}, {NULL}};
const Segment SGI226954[] = {{&S85746[34], 10}, {NULL}};
const Segment SGI226955[] = {{&S85746[27], 6}, {NULL}};
const Segment SGI226956[] = {{&S85746[97], 9}, {NULL}};
const Segment SGI226957[] = {{&S85746[70], 6}, {NULL}};
const Segment SGI226958[] = {{&S85747[44], 10}, {NULL}};
const Segment SGI226959[] = {{&S85747[44], 22}, {NULL}};
const Segment SGI226960[] = {{&S85747[30], 6}, {NULL}};
const Segment SGI226961[] = {{&S85747[115], 3}, {NULL}};
const Segment SGI226962[] = {{&S85747[80], 6}, {NULL}};
const Segment SGI226963[] = {{&S85747[135], 8}, {NULL}};
const Segment SGI226964[] = {{&S85747[144], 13}, {NULL}};
const Segment SGI226965[] = {{&S92587[34], 5}, {NULL}};
const Segment SGI226966[] = {{&S92587[27], 6}, {NULL}};
const Segment SGI226967[] = {{&S92587[92], 9}, {NULL}};
const Segment SGI226968[] = {{&S92587[65], 6}, {NULL}};
const Segment SGI226969[] = {{&S96221[44], 14}, {NULL}};
const Segment SGI226970[] = {{&S96221[44], 26}, {NULL}};
const Segment SGI226971[] = {{&S96221[30], 6}, {NULL}};
const Segment SGI226972[] = {{&S96221[84], 6}, {NULL}};
const Segment SGI226973[] = {{&S96221[111], 13}, {NULL}};
const Segment SGI226974[] = {{&S96223[44], 13}, {NULL}};
const Segment SGI226975[] = {{&S96223[44], 25}, {NULL}};
const Segment SGI226976[] = {{&S96223[30], 6}, {NULL}};
const Segment SGI226977[] = {{&S96223[83], 6}, {NULL}};
const Segment SGI226978[] = {{&S96223[110], 13}, {NULL}};
const Segment SGI226979[] = {{&S96225[44], 13}, {NULL}};
const Segment SGI226980[] = {{&S96225[44], 25}, {NULL}};
const Segment SGI226981[] = {{&S96225[30], 6}, {NULL}};
const Segment SGI226982[] = {{&S87361[34], 10}, {NULL}};
const Segment SGI226983[] = {{&S87361[27], 6}, {NULL}};
const Segment SGI226984[] = {{&S96225[83], 6}, {NULL}};
const Segment SGI226985[] = {{&S87361[97], 9}, {NULL}};
const Segment SGI226986[] = {{&S96225[110], 13}, {NULL}};
const Segment SGI226987[] = {{&S87361[70], 6}, {NULL}};
const Segment SGI226988[] = {{&S96227[34], 14}, {NULL}};
const Segment SGI226989[] = {{&S96227[27], 6}, {NULL}};
const Segment SGI226990[] = {{&S96227[101], 9}, {NULL}};
const Segment SGI226991[] = {{&S96227[74], 6}, {NULL}};
const Segment SGI226992[] = {{&S96227[142], 8}, {NULL}};
const Segment SGI226993[] = {{&S92604[44], 7}, {NULL}};
const Segment SGI226994[] = {{&S92604[44], 19}, {NULL}};
const Segment SGI226995[] = {{&S92604[30], 6}, {NULL}};
const Segment SGI226996[] = {{&S92604[112], 3}, {NULL}};
const Segment SGI226997[] = {{&S92604[77], 6}, {NULL}};
const Segment SGI226998[] = {{&S96234[34], 10}, {NULL}};
const Segment SGI226999[] = {{&S96234[27], 6}, {NULL}};
const Segment SGI227000[] = {{&S96234[97], 9}, {NULL}};
const Segment SGI227001[] = {{&S96234[70], 6}, {NULL}};
const Segment SGI227002[] = {{&S87372[44], 3}, {NULL}};
const Segment SGI227003[] = {{&S87372[44], 12}, {NULL}};
const Segment SGI227004[] = {{&S87372[30], 6}, {NULL}};
const Segment SGI227005[] = {{&S87372[105], 3}, {NULL}};
const Segment SGI227006[] = {{&S87372[70], 6}, {NULL}};
const Segment SGI227007[] = {{&S96235[34], 5}, {NULL}};
const Segment SGI227008[] = {{&S96235[27], 6}, {NULL}};
const Segment SGI227009[] = {{&S96235[92], 9}, {NULL}};
const Segment SGI227010[] = {{&S96235[65], 6}, {NULL}};
const Segment SGI227011[] = {{&S96242[34], 8}, {NULL}};
const Segment SGI227012[] = {{&S96242[27], 6}, {NULL}};
const Segment SGI227013[] = {{&S96242[108], 9}, {NULL}};
const Segment SGI227014[] = {{&S96242[81], 6}, {NULL}};
const Segment SGI227015[] = {{&S96244[52], 11}, {NULL}};
const Segment SGI227016[] = {{&S96244[52], 23}, {NULL}};
const Segment SGI227017[] = {{&S96244[38], 6}, {NULL}};
const Segment SGI227018[] = {{&S96244[89], 5}, {NULL}};
const Segment SGI227019[] = {{&S96244[0], 14}, {NULL}};
const Segment SGI227020[] = {{&S92627[34], 12}, {NULL}};
const Segment SGI227021[] = {{&S92627[27], 6}, {NULL}};
const Segment SGI227022[] = {{&S92627[99], 9}, {NULL}};
const Segment SGI227023[] = {{&S92627[72], 6}, {NULL}};
const Segment SGI227024[] = {{&S96253[34], 15}, {NULL}};
const Segment SGI227025[] = {{&S87395[34], 17}, {NULL}};
const Segment SGI227026[] = {{&S96253[27], 6}, {NULL}};
const Segment SGI227027[] = {{&S87395[27], 6}, {NULL}};
const Segment SGI227028[] = {{&S96253[102], 9}, {NULL}};
const Segment SGI227029[] = {{&S96253[75], 6}, {NULL}};
const Segment SGI227030[] = {{&S87395[104], 9}, {NULL}};
const Segment SGI227031[] = {{&S87395[77], 6}, {NULL}};
const Segment SGI227032[] = {{&S96254[44], 15}, {NULL}};
const Segment SGI227033[] = {{&S96254[44], 33}, {NULL}};
const Segment SGI227034[] = {{&S96254[30], 6}, {NULL}};
const Segment SGI227035[] = {{&S96254[31], 3}, {NULL}};
const Segment SGI227036[] = {{&S96254[91], 5}, {NULL}};
const Segment SGI227037[] = {{&S96254[128], 14}, {NULL}};
const Segment SGI227038[] = {{&S96256[34], 13}, {NULL}};
const Segment SGI227039[] = {{&S96256[27], 6}, {NULL}};
const Segment SGI227040[] = {{&S96256[113], 9}, {NULL}};
const Segment SGI227041[] = {{&S96256[86], 6}, {NULL}};
const Segment SGI227042[] = {{&S96256[154], 8}, {NULL}};
const Segment SGI227043[] = {{&S92639[44], 23}, {NULL}};
const Segment SGI227044[] = {{&S92639[44], 34}, {NULL}};
const Segment SGI227045[] = {{&S92639[30], 6}, {NULL}};
const Segment SGI227046[] = {{&S92639[127], 3}, {NULL}};
const Segment SGI227047[] = {{&S92639[92], 6}, {NULL}};
const Segment SGI227048[] = {{&S96259[52], 7}, {NULL}};
const Segment SGI227049[] = {{&S96259[52], 18}, {NULL}};
const Segment SGI227050[] = {{&S96259[38], 6}, {NULL}};
const Segment SGI227051[] = {{&S96259[84], 5}, {NULL}};
const Segment SGI227052[] = {{&S96259[0], 14}, {NULL}};
const Segment SGI227053[] = {{&S92641[44], 16}, {NULL}};
const Segment SGI227054[] = {{&S92641[44], 38}, {NULL}};
const Segment SGI227055[] = {{&S92641[30], 6}, {NULL}};
const Segment SGI227056[] = {{&S92641[96], 6}, {NULL}};
const Segment SGI227057[] = {{&S92641[123], 13}, {NULL}};
const Segment SGI227058[] = {{&S92649[34], 10}, {NULL}};
const Segment SGI227059[] = {{&S92649[27], 6}, {NULL}};
const Segment SGI227060[] = {{&S92649[102], 9}, {NULL}};
const Segment SGI227061[] = {{&S92649[75], 6}, {NULL}};
const Segment SGI227062[] = {{&S96271[34], 11}, {NULL}};
const Segment SGI227063[] = {{&S96271[27], 6}, {NULL}};
const Segment SGI227064[] = {{&S96271[98], 9}, {NULL}};
const Segment SGI227065[] = {{&S96271[71], 6}, {NULL}};
const Segment SGI227066[] = {{&S92655[34], 15}, {NULL}};
const Segment SGI227067[] = {{&S92655[27], 6}, {NULL}};
const Segment SGI227068[] = {{&S92655[102], 9}, {NULL}};
const Segment SGI227069[] = {{&S92655[75], 6}, {NULL}};
const Segment SGI227070[] = {{&S96276[44], 10}, {NULL}};
const Segment SGI227071[] = {{&S96276[44], 22}, {NULL}};
const Segment SGI227072[] = {{&S96276[30], 6}, {NULL}};
const Segment SGI227073[] = {{&S96276[80], 6}, {NULL}};
const Segment SGI227074[] = {{&S96276[107], 13}, {NULL}};
const Segment SGI227075[] = {{&S96278[34], 10}, {NULL}};
const Segment SGI227076[] = {{&S96278[27], 6}, {NULL}};
const Segment SGI227077[] = {{&S96278[97], 9}, {NULL}};
const Segment SGI227078[] = {{&S96278[70], 6}, {NULL}};
const Segment SGI227079[] = {{&S85836[34], 10}, {NULL}};
const Segment SGI227080[] = {{&S85836[27], 6}, {NULL}};
const Segment SGI227081[] = {{&S85836[97], 9}, {NULL}};
const Segment SGI227082[] = {{&S85836[70], 6}, {NULL}};
const Segment SGI227083[] = {{&S96279[34], 15}, {NULL}};
const Segment SGI227084[] = {{&S96279[27], 6}, {NULL}};
const Segment SGI227085[] = {{&S96279[99], 9}, {NULL}};
const Segment SGI227086[] = {{&S96279[72], 6}, {NULL}};
const Segment SGI227087[] = {{&S96283[34], 10}, {NULL}};
const Segment SGI227088[] = {{&S96283[27], 6}, {NULL}};
const Segment SGI227089[] = {{&S96283[97], 9}, {NULL}};
const Segment SGI227090[] = {{&S96283[70], 6}, {NULL}};
const Segment SGI227091[] = {{&S96284[34], 8}, {NULL}};
const Segment SGI227092[] = {{&S96284[27], 6}, {NULL}};
const Segment SGI227093[] = {{&S96284[100], 9}, {NULL}};
const Segment SGI227094[] = {{&S96284[73], 6}, {NULL}};
const Segment SGI227095[] = {{&S85848[34], 11}, {NULL}};
const Segment SGI227096[] = {{&S85848[27], 6}, {NULL}};
const Segment SGI227097[] = {{&S78198[44], 8}, {NULL}};
const Segment SGI227098[] = {{&S78198[44], 19}, {NULL}};
const Segment SGI227099[] = {{&S78198[30], 6}, {NULL}};
const Segment SGI227100[] = {{&S85848[98], 9}, {NULL}};
const Segment SGI227101[] = {{&S85848[71], 6}, {NULL}};
const Segment SGI227102[] = {{&S78198[130], 3}, {NULL}};
const Segment SGI227103[] = {{&S78198[95], 6}, {NULL}};
const Segment SGI227104[] = {{&S78198[65], 8}, {NULL}};
const Segment SGI227105[] = {{&S78199[44], 8}, {NULL}};
const Segment SGI227106[] = {{&S78199[44], 19}, {NULL}};
const Segment SGI227107[] = {{&S78199[30], 6}, {NULL}};
const Segment SGI227108[] = {{&S78199[130], 3}, {NULL}};
const Segment SGI227109[] = {{&S78199[95], 6}, {NULL}};
const Segment SGI227110[] = {{&S78199[65], 8}, {NULL}};
const Segment SGI227111[] = {{&S78200[44], 8}, {NULL}};
const Segment SGI227112[] = {{&S78200[44], 20}, {NULL}};
const Segment SGI227113[] = {{&S78200[30], 6}, {NULL}};
const Segment SGI227114[] = {{&S78200[131], 3}, {NULL}};
const Segment SGI227115[] = {{&S78200[96], 6}, {NULL}};
const Segment SGI227116[] = {{&S78200[66], 8}, {NULL}};
const Segment SGI227117[] = {{&S85860[34], 10}, {NULL}};
const Segment SGI227118[] = {{&S85860[27], 6}, {NULL}};
const Segment SGI227119[] = {{&S85860[97], 9}, {NULL}};
const Segment SGI227120[] = {{&S85860[70], 6}, {NULL}};
const Segment SGI227121[] = {{&S78207[44], 10}, {NULL}};
const Segment SGI227122[] = {{&S78207[44], 21}, {NULL}};
const Segment SGI227123[] = {{&S78207[30], 6}, {NULL}};
const Segment SGI227124[] = {{&S78207[132], 3}, {NULL}};
const Segment SGI227125[] = {{&S78207[97], 6}, {NULL}};
const Segment SGI227126[] = {{&S78207[67], 8}, {NULL}};
const Segment SGI227127[] = {{&S78209[44], 10}, {NULL}};
const Segment SGI227128[] = {{&S78209[44], 22}, {NULL}};
const Segment SGI227129[] = {{&S78209[30], 6}, {NULL}};
const Segment SGI227130[] = {{&S78209[133], 3}, {NULL}};
const Segment SGI227131[] = {{&S78209[98], 6}, {NULL}};
const Segment SGI227132[] = {{&S78209[68], 8}, {NULL}};
const Segment SGI227133[] = {{&S85871[44], 10}, {NULL}};
const Segment SGI227134[] = {{&S85871[44], 22}, {NULL}};
const Segment SGI227135[] = {{&S85871[30], 6}, {NULL}};
const Segment SGI227136[] = {{&S78215[34], 11}, {NULL}};
const Segment SGI227137[] = {{&S78215[27], 6}, {NULL}};
const Segment SGI227138[] = {{&S85871[80], 6}, {NULL}};
const Segment SGI227139[] = {{&S85871[107], 13}, {NULL}};
const Segment SGI227140[] = {{&S78215[97], 9}, {NULL}};
const Segment SGI227141[] = {{&S78215[70], 6}, {NULL}};
const Segment SGI227142[] = {{&S85883[44], 10}, {NULL}};
const Segment SGI227143[] = {{&S85883[44], 22}, {NULL}};
const Segment SGI227144[] = {{&S85883[30], 6}, {NULL}};
const Segment SGI227145[] = {{&S85883[113], 3}, {NULL}};
const Segment SGI227146[] = {{&S85883[80], 5}, {NULL}};
const Segment SGI227147[] = {{&S85883[117], 14}, {NULL}};
const Segment SGI227148[] = {{&S85884[44], 11}, {NULL}};
const Segment SGI227149[] = {{&S85884[44], 23}, {NULL}};
const Segment SGI227150[] = {{&S85884[30], 6}, {NULL}};
const Segment SGI227151[] = {{&S85884[114], 3}, {NULL}};
const Segment SGI227152[] = {{&S85884[81], 5}, {NULL}};
const Segment SGI227153[] = {{&S85884[118], 14}, {NULL}};
const Segment SGI227154[] = {{&S85887[34], 6}, {NULL}};
const Segment SGI227155[] = {{&S85887[27], 6}, {NULL}};
const Segment SGI227156[] = {{&S85887[93], 9}, {NULL}};
const Segment SGI227157[] = {{&S85887[66], 6}, {NULL}};
const Segment SGI227158[] = {{&S85888[34], 11}, {NULL}};
const Segment SGI227159[] = {{&S85888[27], 6}, {NULL}};
const Segment SGI227160[] = {{&S85888[98], 9}, {NULL}};
const Segment SGI227161[] = {{&S85888[71], 6}, {NULL}};
const Segment SGI227162[] = {{&S92725[44], 7}, {NULL}};
const Segment SGI227163[] = {{&S92725[44], 19}, {NULL}};
const Segment SGI227164[] = {{&S92725[30], 6}, {NULL}};
const Segment SGI227165[] = {{&S92725[112], 3}, {NULL}};
const Segment SGI227166[] = {{&S92725[77], 6}, {NULL}};
const Segment SGI227167[] = {{&S85890[34], 9}, {NULL}};
const Segment SGI227168[] = {{&S85890[27], 6}, {NULL}};
const Segment SGI227169[] = {{&S85890[96], 9}, {NULL}};
const Segment SGI227170[] = {{&S85890[69], 6}, {NULL}};
const Segment SGI227171[] = {{&S85891[34], 10}, {NULL}};
const Segment SGI227172[] = {{&S85891[27], 6}, {NULL}};
const Segment SGI227173[] = {{&S85891[97], 9}, {NULL}};
const Segment SGI227174[] = {{&S85891[70], 6}, {NULL}};
const Segment SGI227175[] = {{&S85891[138], 8}, {NULL}};
const Segment SGI227176[] = {{&S85892[34], 11}, {NULL}};
const Segment SGI227177[] = {{&S85892[27], 6}, {NULL}};
const Segment SGI227178[] = {{&S85892[98], 9}, {NULL}};
const Segment SGI227179[] = {{&S96348[34], 7}, {NULL}};
const Segment SGI227180[] = {{&S85892[71], 6}, {NULL}};
const Segment SGI227181[] = {{&S96348[27], 6}, {NULL}};
const Segment SGI227182[] = {{&S85892[139], 8}, {NULL}};
const Segment SGI227183[] = {{&S96348[90], 9}, {NULL}};
const Segment SGI227184[] = {{&S92730[44], 10}, {NULL}};
const Segment SGI227185[] = {{&S96348[63], 6}, {NULL}};
const Segment SGI227186[] = {{&S92730[44], 22}, {NULL}};
const Segment SGI227187[] = {{&S92730[30], 6}, {NULL}};
const Segment SGI227188[] = {{&S92730[115], 3}, {NULL}};
const Segment SGI227189[] = {{&S92730[80], 6}, {NULL}};
const Segment SGI227190[] = {{&S92730[132], 10}, {NULL}};
const Segment SGI227191[] = {{&S78245[34], 10}, {NULL}};
const Segment SGI227192[] = {{&S78245[27], 6}, {NULL}};
const Segment SGI227193[] = {{&S78245[96], 9}, {NULL}};
const Segment SGI227194[] = {{&S78245[69], 6}, {NULL}};
const Segment SGI227195[] = {{&S92733[34], 6}, {NULL}};
const Segment SGI227196[] = {{&S92733[27], 6}, {NULL}};
const Segment SGI227197[] = {{&S92733[98], 9}, {NULL}};
const Segment SGI227198[] = {{&S92733[71], 6}, {NULL}};
const Segment SGI227199[] = {{&S92735[44], 6}, {NULL}};
const Segment SGI227200[] = {{&S92735[44], 18}, {NULL}};
const Segment SGI227201[] = {{&S92735[30], 6}, {NULL}};
const Segment SGI227202[] = {{&S92735[109], 3}, {NULL}};
const Segment SGI227203[] = {{&S92735[76], 5}, {NULL}};
const Segment SGI227204[] = {{&S92735[113], 14}, {NULL}};
const Segment SGI227205[] = {{&S96359[34], 13}, {NULL}};
const Segment SGI227206[] = {{&S96359[27], 6}, {NULL}};
const Segment SGI227207[] = {{&S96359[100], 9}, {NULL}};
const Segment SGI227208[] = {{&S96359[73], 6}, {NULL}};
const Segment SGI227209[] = {{&S92749[34], 7}, {NULL}};
const Segment SGI227210[] = {{&S92749[27], 6}, {NULL}};
const Segment SGI227211[] = {{&S92749[94], 9}, {NULL}};
const Segment SGI227212[] = {{&S92749[67], 6}, {NULL}};
const Segment SGI227213[] = {{&S92750[34], 16}, {NULL}};
const Segment SGI227214[] = {{&S92750[27], 6}, {NULL}};
const Segment SGI227215[] = {{&S92750[103], 9}, {NULL}};
const Segment SGI227216[] = {{&S92750[76], 6}, {NULL}};
const Segment SGI227217[] = {{&S96373[45], 10}, {NULL}};
const Segment SGI227218[] = {{&S96373[30], 6}, {NULL}};
const Segment SGI227219[] = {{&S92751[44], 18}, {NULL}};
const Segment SGI227220[] = {{&S96373[31], 3}, {NULL}};
const Segment SGI227221[] = {{&S92751[44], 40}, {NULL}};
const Segment SGI227222[] = {{&S92751[30], 6}, {NULL}};
const Segment SGI227223[] = {{&S96373[81], 5}, {NULL}};
const Segment SGI227224[] = {{&S96373[139], 13}, {NULL}};
const Segment SGI227225[] = {{&S92751[133], 3}, {NULL}};
const Segment SGI227226[] = {{&S92751[98], 6}, {NULL}};
const Segment SGI227227[] = {{&S92756[34], 15}, {NULL}};
const Segment SGI227228[] = {{&S92756[27], 6}, {NULL}};
const Segment SGI227229[] = {{&S92756[102], 9}, {NULL}};
const Segment SGI227230[] = {{&S92756[75], 6}, {NULL}};
const Segment SGI227231[] = {{&S96380[34], 6}, {NULL}};
const Segment SGI227232[] = {{&S96380[27], 6}, {NULL}};
const Segment SGI227233[] = {{&S96380[93], 9}, {NULL}};
const Segment SGI227234[] = {{&S96380[66], 6}, {NULL}};
const Segment SGI227235[] = {{&S96381[44], 11}, {NULL}};
const Segment SGI227236[] = {{&S96381[44], 23}, {NULL}};
const Segment SGI227237[] = {{&S96381[30], 6}, {NULL}};
const Segment SGI227238[] = {{&S96381[31], 3}, {NULL}};
const Segment SGI227239[] = {{&S96381[81], 6}, {NULL}};
const Segment SGI227240[] = {{&S96381[141], 11}, {NULL}};
const Segment SGI227241[] = {{&S96387[34], 11}, {NULL}};
const Segment SGI227242[] = {{&S96387[27], 6}, {NULL}};
const Segment SGI227243[] = {{&S96387[98], 9}, {NULL}};
const Segment SGI227244[] = {{&S96387[71], 6}, {NULL}};
const Segment SGI227245[] = {{&S96389[44], 10}, {NULL}};
const Segment SGI227246[] = {{&S96389[44], 22}, {NULL}};
const Segment SGI227247[] = {{&S96389[30], 6}, {NULL}};
const Segment SGI227248[] = {{&S96389[31], 3}, {NULL}};
const Segment SGI227249[] = {{&S96389[95], 6}, {NULL}};
const Segment SGI227250[] = {{&S96389[68], 8}, {NULL}};
const Segment SGI227251[] = {{&S96391[44], 10}, {NULL}};
const Segment SGI227252[] = {{&S96391[44], 22}, {NULL}};
const Segment SGI227253[] = {{&S96391[30], 6}, {NULL}};
const Segment SGI227254[] = {{&S96391[31], 3}, {NULL}};
const Segment SGI227255[] = {{&S96391[111], 6}, {NULL}};
const Segment SGI227256[] = {{&S96391[68], 8}, {NULL}};
const Segment SGI227257[] = {{&S96399[44], 10}, {NULL}};
const Segment SGI227258[] = {{&S96399[44], 22}, {NULL}};
const Segment SGI227259[] = {{&S96399[30], 6}, {NULL}};
const Segment SGI227260[] = {{&S92787[44], 9}, {NULL}};
const Segment SGI227261[] = {{&S92787[44], 22}, {NULL}};
const Segment SGI227262[] = {{&S96399[31], 3}, {NULL}};
const Segment SGI227263[] = {{&S92787[30], 6}, {NULL}};
const Segment SGI227264[] = {{&S96399[80], 5}, {NULL}};
const Segment SGI227265[] = {{&S92787[115], 3}, {NULL}};
const Segment SGI227266[] = {{&S96399[125], 14}, {NULL}};
const Segment SGI227267[] = {{&S92787[80], 6}, {NULL}};
const Segment SGI227268[] = {{&S96402[34], 14}, {NULL}};
const Segment SGI227269[] = {{&S96402[27], 6}, {NULL}};
const Segment SGI227270[] = {{&S96402[112], 9}, {NULL}};
const Segment SGI227271[] = {{&S96402[85], 6}, {NULL}};
const Segment SGI227272[] = {{&S92791[44], 7}, {NULL}};
const Segment SGI227273[] = {{&S92791[44], 23}, {NULL}};
const Segment SGI227274[] = {{&S92791[30], 6}, {NULL}};
const Segment SGI227275[] = {{&S92791[81], 6}, {NULL}};
const Segment SGI227276[] = {{&S92791[108], 13}, {NULL}};
const Segment SGI227277[] = {{&S96403[44], 10}, {NULL}};
const Segment SGI227278[] = {{&S96403[44], 22}, {NULL}};
const Segment SGI227279[] = {{&S96403[30], 6}, {NULL}};
const Segment SGI227280[] = {{&S96403[31], 3}, {NULL}};
const Segment SGI227281[] = {{&S96403[80], 5}, {NULL}};
const Segment SGI227282[] = {{&S96403[117], 14}, {NULL}};
const Segment SGI227283[] = {{&S85968[34], 8}, {NULL}};
const Segment SGI227284[] = {{&S85968[27], 6}, {NULL}};
const Segment SGI227285[] = {{&S85968[95], 9}, {NULL}};
const Segment SGI227286[] = {{&S85968[68], 6}, {NULL}};
const Segment SGI227287[] = {{&S96406[45], 18}, {NULL}};
const Segment SGI227288[] = {{&S96406[30], 6}, {NULL}};
const Segment SGI227289[] = {{&S78319[44], 10}, {NULL}};
const Segment SGI227290[] = {{&S85969[34], 11}, {NULL}};
const Segment SGI227291[] = {{&S78319[44], 22}, {NULL}};
const Segment SGI227292[] = {{&S78319[30], 6}, {NULL}};
const Segment SGI227293[] = {{&S85969[27], 6}, {NULL}};
const Segment SGI227294[] = {{&S96406[31], 3}, {NULL}};
const Segment SGI227295[] = {{&S96406[89], 5}, {NULL}};
const Segment SGI227296[] = {{&S78319[115], 3}, {NULL}};
const Segment SGI227297[] = {{&S85969[98], 9}, {NULL}};
const Segment SGI227298[] = {{&S96406[147], 13}, {NULL}};
const Segment SGI227299[] = {{&S85969[71], 6}, {NULL}};
const Segment SGI227300[] = {{&S78319[80], 6}, {NULL}};
const Segment SGI227301[] = {{&S78319[140], 8}, {NULL}};
const Segment SGI227302[] = {{&S96407[45], 7}, {NULL}};
const Segment SGI227303[] = {{&S96407[30], 6}, {NULL}};
const Segment SGI227304[] = {{&S96407[31], 3}, {NULL}};
const Segment SGI227305[] = {{&S96407[84], 5}, {NULL}};
const Segment SGI227306[] = {{&S96407[142], 13}, {NULL}};
const Segment SGI227307[] = {{&S85972[34], 11}, {NULL}};
const Segment SGI227308[] = {{&S85972[27], 6}, {NULL}};
const Segment SGI227309[] = {{&S85972[98], 9}, {NULL}};
const Segment SGI227310[] = {{&S85972[71], 6}, {NULL}};
const Segment SGI227311[] = {{&S92801[34], 10}, {NULL}};
const Segment SGI227312[] = {{&S92801[27], 6}, {NULL}};
const Segment SGI227313[] = {{&S92801[102], 9}, {NULL}};
const Segment SGI227314[] = {{&S92801[75], 6}, {NULL}};
const Segment SGI227315[] = {{&S96409[34], 9}, {NULL}};
const Segment SGI227316[] = {{&S96409[27], 6}, {NULL}};
const Segment SGI227317[] = {{&S96409[96], 9}, {NULL}};
const Segment SGI227318[] = {{&S96409[69], 6}, {NULL}};
const Segment SGI227319[] = {{&S85974[44], 12}, {NULL}};
const Segment SGI227320[] = {{&S85974[44], 34}, {NULL}};
const Segment SGI227321[] = {{&S85974[30], 6}, {NULL}};
const Segment SGI227322[] = {{&S96410[34], 9}, {NULL}};
const Segment SGI227323[] = {{&S85974[92], 6}, {NULL}};
const Segment SGI227324[] = {{&S96410[27], 6}, {NULL}};
const Segment SGI227325[] = {{&S85974[119], 13}, {NULL}};
const Segment SGI227326[] = {{&S96410[96], 9}, {NULL}};
const Segment SGI227327[] = {{&S96410[69], 6}, {NULL}};
const Segment SGI227328[] = {{&S96411[34], 10}, {NULL}};
const Segment SGI227329[] = {{&S96411[27], 6}, {NULL}};
const Segment SGI227330[] = {{&S96411[102], 9}, {NULL}};
const Segment SGI227331[] = {{&S96411[75], 6}, {NULL}};
const Segment SGI227332[] = {{&S96413[59], 8}, {NULL}};
const Segment SGI227333[] = {{&S96413[59], 20}, {NULL}};
const Segment SGI227334[] = {{&S96413[45], 6}, {NULL}};
const Segment SGI227335[] = {{&S96413[46], 3}, {NULL}};
const Segment SGI227336[] = {{&S96413[93], 6}, {NULL}};
const Segment SGI227337[] = {{&S96413[0], 14}, {NULL}};
const Segment SGI227338[] = {{&S96414[44], 4}, {NULL}};
const Segment SGI227339[] = {{&S96414[44], 26}, {NULL}};
const Segment SGI227340[] = {{&S96414[30], 6}, {NULL}};
const Segment SGI227341[] = {{&S96414[31], 3}, {NULL}};
const Segment SGI227342[] = {{&S96414[84], 6}, {NULL}};
const Segment SGI227343[] = {{&S92812[44], 16}, {NULL}};
const Segment SGI227344[] = {{&S92812[44], 28}, {NULL}};
const Segment SGI227345[] = {{&S92812[30], 6}, {NULL}};
const Segment SGI227346[] = {{&S92812[121], 3}, {NULL}};
const Segment SGI227347[] = {{&S92812[86], 6}, {NULL}};
const Segment SGI227348[] = {{&S92813[44], 7}, {NULL}};
const Segment SGI227349[] = {{&S92813[44], 25}, {NULL}};
const Segment SGI227350[] = {{&S92813[30], 6}, {NULL}};
const Segment SGI227351[] = {{&S92813[118], 3}, {NULL}};
const Segment SGI227352[] = {{&S92813[83], 6}, {NULL}};
const Segment SGI227353[] = {{&S92815[44], 11}, {NULL}};
const Segment SGI227354[] = {{&S92815[44], 23}, {NULL}};
const Segment SGI227355[] = {{&S92815[30], 6}, {NULL}};
const Segment SGI227356[] = {{&S92815[116], 3}, {NULL}};
const Segment SGI227357[] = {{&S92815[81], 6}, {NULL}};
const Segment SGI227358[] = {{&S92816[44], 11}, {NULL}};
const Segment SGI227359[] = {{&S92816[44], 23}, {NULL}};
const Segment SGI227360[] = {{&S92816[30], 6}, {NULL}};
const Segment SGI227361[] = {{&S96422[34], 13}, {NULL}};
const Segment SGI227362[] = {{&S96422[27], 6}, {NULL}};
const Segment SGI227363[] = {{&S92816[116], 3}, {NULL}};
const Segment SGI227364[] = {{&S92816[81], 6}, {NULL}};
const Segment SGI227365[] = {{&S96422[100], 9}, {NULL}};
const Segment SGI227366[] = {{&S96422[73], 6}, {NULL}};
const Segment SGI227367[] = {{&S85992[44], 15}, {NULL}};
const Segment SGI227368[] = {{&S85992[44], 33}, {NULL}};
const Segment SGI227369[] = {{&S85992[30], 6}, {NULL}};
const Segment SGI227370[] = {{&S85992[124], 3}, {NULL}};
const Segment SGI227371[] = {{&S85992[91], 5}, {NULL}};
const Segment SGI227372[] = {{&S85992[128], 14}, {NULL}};
const Segment SGI227373[] = {{&S96425[34], 22}, {NULL}};
const Segment SGI227374[] = {{&S96425[27], 6}, {NULL}};
const Segment SGI227375[] = {{&S96425[109], 9}, {NULL}};
const Segment SGI227376[] = {{&S96425[82], 6}, {NULL}};
const Segment SGI227377[] = {{&S96426[34], 4}, {NULL}};
const Segment SGI227378[] = {{&S96426[27], 6}, {NULL}};
const Segment SGI227379[] = {{&S96426[91], 9}, {NULL}};
const Segment SGI227380[] = {{&S96426[64], 6}, {NULL}};
const Segment SGI227381[] = {{&S96430[44], 6}, {NULL}};
const Segment SGI227382[] = {{&S96430[44], 18}, {NULL}};
const Segment SGI227383[] = {{&S96430[30], 6}, {NULL}};
const Segment SGI227384[] = {{&S96430[31], 3}, {NULL}};
const Segment SGI227385[] = {{&S96430[76], 6}, {NULL}};
const Segment SGI227386[] = {{&S86008[34], 9}, {NULL}};
const Segment SGI227387[] = {{&S86008[27], 6}, {NULL}};
const Segment SGI227388[] = {{&S86008[96], 9}, {NULL}};
const Segment SGI227389[] = {{&S86008[69], 6}, {NULL}};
const Segment SGI227390[] = {{&S96436[34], 20}, {NULL}};
const Segment SGI227391[] = {{&S96436[27], 6}, {NULL}};
const Segment SGI227392[] = {{&S86009[34], 20}, {NULL}};
const Segment SGI227393[] = {{&S86009[27], 6}, {NULL}};
const Segment SGI227394[] = {{&S96436[107], 9}, {NULL}};
const Segment SGI227395[] = {{&S96436[80], 6}, {NULL}};
const Segment SGI227396[] = {{&S86009[107], 9}, {NULL}};
const Segment SGI227397[] = {{&S86009[80], 6}, {NULL}};
const Segment SGI227398[] = {{&S86012[34], 16}, {NULL}};
const Segment SGI227399[] = {{&S86012[27], 6}, {NULL}};
const Segment SGI227400[] = {{&S86012[103], 9}, {NULL}};
const Segment SGI227401[] = {{&S86012[76], 6}, {NULL}};
const Segment SGI227402[] = {{&S96440[34], 14}, {NULL}};
const Segment SGI227403[] = {{&S96440[27], 6}, {NULL}};
const Segment SGI227404[] = {{&S96440[101], 9}, {NULL}};
const Segment SGI227405[] = {{&S96440[74], 6}, {NULL}};
const Segment SGI227406[] = {{&S86013[34], 13}, {NULL}};
const Segment SGI227407[] = {{&S78370[34], 6}, {NULL}};
const Segment SGI227408[] = {{&S86013[27], 6}, {NULL}};
const Segment SGI227409[] = {{&S78370[27], 6}, {NULL}};
const Segment SGI227410[] = {{&S78370[93], 9}, {NULL}};
const Segment SGI227411[] = {{&S86013[100], 9}, {NULL}};
const Segment SGI227412[] = {{&S78370[66], 6}, {NULL}};
const Segment SGI227413[] = {{&S86013[73], 6}, {NULL}};
const Segment SGI227414[] = {{&S86014[44], 13}, {NULL}};
const Segment SGI227415[] = {{&S86014[44], 25}, {NULL}};
const Segment SGI227416[] = {{&S86014[30], 6}, {NULL}};
const Segment SGI227417[] = {{&S86014[83], 6}, {NULL}};
const Segment SGI227418[] = {{&S96442[34], 11}, {NULL}};
const Segment SGI227419[] = {{&S86014[110], 13}, {NULL}};
const Segment SGI227420[] = {{&S96442[27], 6}, {NULL}};
const Segment SGI227421[] = {{&S96442[98], 9}, {NULL}};
const Segment SGI227422[] = {{&S96442[71], 6}, {NULL}};
const Segment SGI227423[] = {{&S96443[34], 13}, {NULL}};
const Segment SGI227424[] = {{&S86016[34], 10}, {NULL}};
const Segment SGI227425[] = {{&S96443[27], 6}, {NULL}};
const Segment SGI227426[] = {{&S86016[27], 6}, {NULL}};
const Segment SGI227427[] = {{&S96443[100], 9}, {NULL}};
const Segment SGI227428[] = {{&S96443[73], 6}, {NULL}};
const Segment SGI227429[] = {{&S86016[97], 9}, {NULL}};
const Segment SGI227430[] = {{&S86016[70], 6}, {NULL}};
const Segment SGI227431[] = {{&S78375[34], 7}, {NULL}};
const Segment SGI227432[] = {{&S78375[27], 6}, {NULL}};
const Segment SGI227433[] = {{&S78375[94], 9}, {NULL}};
const Segment SGI227434[] = {{&S78375[67], 6}, {NULL}};
const Segment SGI227435[] = {{&S96445[44], 7}, {NULL}};
const Segment SGI227436[] = {{&S96445[44], 19}, {NULL}};
const Segment SGI227437[] = {{&S96445[30], 6}, {NULL}};
const Segment SGI227438[] = {{&S96445[31], 3}, {NULL}};
const Segment SGI227439[] = {{&S96445[77], 6}, {NULL}};
const Segment SGI227440[] = {{&S96447[34], 8}, {NULL}};
const Segment SGI227441[] = {{&S96447[27], 6}, {NULL}};
const Segment SGI227442[] = {{&S96447[95], 9}, {NULL}};
const Segment SGI227443[] = {{&S96447[68], 6}, {NULL}};
const Segment SGI227444[] = {{&S96449[34], 10}, {NULL}};
const Segment SGI227445[] = {{&S96449[27], 6}, {NULL}};
const Segment SGI227446[] = {{&S96449[97], 9}, {NULL}};
const Segment SGI227447[] = {{&S96449[70], 6}, {NULL}};
const Segment SGI227448[] = {{&S96453[44], 9}, {NULL}};
const Segment SGI227449[] = {{&S96453[44], 21}, {NULL}};
const Segment SGI227450[] = {{&S96453[30], 6}, {NULL}};
const Segment SGI227451[] = {{&S96453[31], 3}, {NULL}};
const Segment SGI227452[] = {{&S96453[79], 6}, {NULL}};
const Segment SGI227453[] = {{&S96458[34], 6}, {NULL}};
const Segment SGI227454[] = {{&S96458[27], 6}, {NULL}};
const Segment SGI227455[] = {{&S96458[93], 9}, {NULL}};
const Segment SGI227456[] = {{&S96458[66], 6}, {NULL}};
const Segment SGI227457[] = {{&S96461[34], 7}, {NULL}};
const Segment SGI227458[] = {{&S96461[27], 6}, {NULL}};
const Segment SGI227459[] = {{&S96461[94], 9}, {NULL}};
const Segment SGI227460[] = {{&S96461[67], 6}, {NULL}};
const Segment SGI227461[] = {{&S96462[34], 12}, {NULL}};
const Segment SGI227462[] = {{&S96462[27], 6}, {NULL}};
const Segment SGI227463[] = {{&S96462[112], 9}, {NULL}};
const Segment SGI227464[] = {{&S96462[85], 6}, {NULL}};
const Segment SGI227465[] = {{&S96467[44], 9}, {NULL}};
const Segment SGI227466[] = {{&S96467[44], 19}, {NULL}};
const Segment SGI227467[] = {{&S96467[30], 6}, {NULL}};
const Segment SGI227468[] = {{&S96467[31], 3}, {NULL}};
const Segment SGI227469[] = {{&S96467[77], 6}, {NULL}};
const Segment SGI227470[] = {{&S96468[59], 4}, {NULL}};
const Segment SGI227471[] = {{&S96468[59], 26}, {NULL}};
const Segment SGI227472[] = {{&S96468[45], 6}, {NULL}};
const Segment SGI227473[] = {{&S96468[46], 3}, {NULL}};
const Segment SGI227474[] = {{&S96468[99], 6}, {NULL}};
const Segment SGI227475[] = {{&S96468[0], 14}, {NULL}};
const Segment SGI227476[] = {{&S96469[34], 13}, {NULL}};
const Segment SGI227477[] = {{&S96469[27], 6}, {NULL}};
const Segment SGI227478[] = {{&S96469[100], 9}, {NULL}};
const Segment SGI227479[] = {{&S96469[73], 6}, {NULL}};
const Segment SGI227480[] = {{&S96472[44], 9}, {NULL}};
const Segment SGI227481[] = {{&S96472[44], 21}, {NULL}};
const Segment SGI227482[] = {{&S96472[30], 6}, {NULL}};
const Segment SGI227483[] = {{&S96472[31], 3}, {NULL}};
const Segment SGI227484[] = {{&S96472[79], 6}, {NULL}};
const Segment SGI227485[] = {{&S96473[44], 15}, {NULL}};
const Segment SGI227486[] = {{&S96473[44], 27}, {NULL}};
const Segment SGI227487[] = {{&S96473[30], 6}, {NULL}};
const Segment SGI227488[] = {{&S96473[31], 3}, {NULL}};
const Segment SGI227489[] = {{&S96473[85], 6}, {NULL}};
const Segment SGI227490[] = {{&S96474[34], 8}, {NULL}};
const Segment SGI227491[] = {{&S96474[27], 6}, {NULL}};
const Segment SGI227492[] = {{&S96474[95], 9}, {NULL}};
const Segment SGI227493[] = {{&S96474[68], 6}, {NULL}};
const Segment SGI227494[] = {{&S86065[34], 24}, {NULL}};
const Segment SGI227495[] = {{&S86065[27], 6}, {NULL}};
const Segment SGI227496[] = {{&S86065[111], 9}, {NULL}};
const Segment SGI227497[] = {{&S86065[84], 6}, {NULL}};
const Segment SGI227498[] = {{&S96475[34], 11}, {NULL}};
const Segment SGI227499[] = {{&S96475[27], 6}, {NULL}};
const Segment SGI227500[] = {{&S96475[98], 9}, {NULL}};
const Segment SGI227501[] = {{&S96475[71], 6}, {NULL}};
const Segment SGI227502[] = {{&S96477[34], 10}, {NULL}};
const Segment SGI227503[] = {{&S96477[27], 6}, {NULL}};
const Segment SGI227504[] = {{&S96477[97], 9}, {NULL}};
const Segment SGI227505[] = {{&S96477[70], 6}, {NULL}};
const Segment SGI227506[] = {{&S86069[44], 10}, {NULL}};
const Segment SGI227507[] = {{&S86069[44], 22}, {NULL}};
const Segment SGI227508[] = {{&S86069[30], 6}, {NULL}};
const Segment SGI227509[] = {{&S86069[136], 3}, {NULL}};
const Segment SGI227510[] = {{&S96478[34], 11}, {NULL}};
const Segment SGI227511[] = {{&S86069[101], 6}, {NULL}};
const Segment SGI227512[] = {{&S96478[27], 6}, {NULL}};
const Segment SGI227513[] = {{&S96478[98], 9}, {NULL}};
const Segment SGI227514[] = {{&S96478[71], 6}, {NULL}};
const Segment SGI227515[] = {{&S96483[34], 6}, {NULL}};
const Segment SGI227516[] = {{&S96483[27], 6}, {NULL}};
const Segment SGI227517[] = {{&S96483[105], 9}, {NULL}};
const Segment SGI227518[] = {{&S96483[78], 6}, {NULL}};
const Segment SGI227519[] = {{&S86076[34], 10}, {NULL}};
const Segment SGI227520[] = {{&S86076[27], 6}, {NULL}};
const Segment SGI227521[] = {{&S96484[34], 13}, {NULL}};
const Segment SGI227522[] = {{&S96484[27], 6}, {NULL}};
const Segment SGI227523[] = {{&S86076[97], 9}, {NULL}};
const Segment SGI227524[] = {{&S86076[70], 6}, {NULL}};
const Segment SGI227525[] = {{&S96484[113], 9}, {NULL}};
const Segment SGI227526[] = {{&S96484[86], 6}, {NULL}};
const Segment SGI227527[] = {{&S96484[154], 8}, {NULL}};
const Segment SGI227528[] = {{&S92918[44], 8}, {NULL}};
const Segment SGI227529[] = {{&S92918[44], 20}, {NULL}};
const Segment SGI227530[] = {{&S92918[30], 6}, {NULL}};
const Segment SGI227531[] = {{&S92918[113], 3}, {NULL}};
const Segment SGI227532[] = {{&S92918[78], 6}, {NULL}};
const Segment SGI227533[] = {{&S96487[34], 11}, {NULL}};
const Segment SGI227534[] = {{&S96487[27], 6}, {NULL}};
const Segment SGI227535[] = {{&S96487[98], 9}, {NULL}};
const Segment SGI227536[] = {{&S96487[71], 6}, {NULL}};
const Segment SGI227537[] = {{&S86081[44], 15}, {NULL}};
const Segment SGI227538[] = {{&S86081[44], 27}, {NULL}};
const Segment SGI227539[] = {{&S86081[30], 6}, {NULL}};
const Segment SGI227540[] = {{&S96488[34], 9}, {NULL}};
const Segment SGI227541[] = {{&S86081[118], 3}, {NULL}};
const Segment SGI227542[] = {{&S96488[27], 6}, {NULL}};
const Segment SGI227543[] = {{&S86081[85], 5}, {NULL}};
const Segment SGI227544[] = {{&S96488[96], 9}, {NULL}};
const Segment SGI227545[] = {{&S96488[69], 6}, {NULL}};
const Segment SGI227546[] = {{&S96488[130], 8}, {NULL}};
const Segment SGI227547[] = {{&S96491[34], 12}, {NULL}};
const Segment SGI227548[] = {{&S96491[27], 6}, {NULL}};
const Segment SGI227549[] = {{&S96491[120], 9}, {NULL}};
const Segment SGI227550[] = {{&S96491[93], 6}, {NULL}};
const Segment SGI227551[] = {{&S96492[34], 12}, {NULL}};
const Segment SGI227552[] = {{&S96492[27], 6}, {NULL}};
const Segment SGI227553[] = {{&S96492[99], 9}, {NULL}};
const Segment SGI227554[] = {{&S96492[72], 6}, {NULL}};
const Segment SGI227555[] = {{&S92933[34], 7}, {NULL}};
const Segment SGI227556[] = {{&S92933[27], 6}, {NULL}};
const Segment SGI227557[] = {{&S92933[94], 9}, {NULL}};
const Segment SGI227558[] = {{&S92933[67], 6}, {NULL}};
const Segment SGI227559[] = {{&S96499[44], 9}, {NULL}};
const Segment SGI227560[] = {{&S96499[44], 21}, {NULL}};
const Segment SGI227561[] = {{&S96499[30], 6}, {NULL}};
const Segment SGI227562[] = {{&S96499[31], 3}, {NULL}};
const Segment SGI227563[] = {{&S96499[79], 6}, {NULL}};
const Segment SGI227564[] = {{&S86100[34], 8}, {NULL}};
const Segment SGI227565[] = {{&S86100[27], 6}, {NULL}};
const Segment SGI227566[] = {{&S86100[95], 9}, {NULL}};
const Segment SGI227567[] = {{&S86100[68], 6}, {NULL}};
const Segment SGI227568[] = {{&S92939[34], 10}, {NULL}};
const Segment SGI227569[] = {{&S92939[27], 6}, {NULL}};
const Segment SGI227570[] = {{&S92939[97], 9}, {NULL}};
const Segment SGI227571[] = {{&S92939[70], 6}, {NULL}};
const Segment SGI227572[] = {{&S92943[44], 13}, {NULL}};
const Segment SGI227573[] = {{&S96510[44], 10}, {NULL}};
const Segment SGI227574[] = {{&S92943[44], 25}, {NULL}};
const Segment SGI227575[] = {{&S96510[44], 22}, {NULL}};
const Segment SGI227576[] = {{&S92943[30], 6}, {NULL}};
const Segment SGI227577[] = {{&S96510[30], 6}, {NULL}};
const Segment SGI227578[] = {{&S92943[118], 3}, {NULL}};
const Segment SGI227579[] = {{&S96510[31], 3}, {NULL}};
const Segment SGI227580[] = {{&S92943[83], 6}, {NULL}};
const Segment SGI227581[] = {{&S96510[80], 6}, {NULL}};
const Segment SGI227582[] = {{&S96510[140], 7}, {NULL}};
const Segment SGI227583[] = {{&S96511[34], 11}, {NULL}};
const Segment SGI227584[] = {{&S96511[27], 6}, {NULL}};
const Segment SGI227585[] = {{&S96511[98], 9}, {NULL}};
const Segment SGI227586[] = {{&S96511[71], 6}, {NULL}};
const Segment SGI227587[] = {{&S78477[34], 8}, {NULL}};
const Segment SGI227588[] = {{&S78477[27], 6}, {NULL}};
const Segment SGI227589[] = {{&S78477[94], 9}, {NULL}};
const Segment SGI227590[] = {{&S78477[67], 6}, {NULL}};
const Segment SGI227591[] = {{&S96516[44], 10}, {NULL}};
const Segment SGI227592[] = {{&S96516[44], 22}, {NULL}};
const Segment SGI227593[] = {{&S96516[30], 6}, {NULL}};
const Segment SGI227594[] = {{&S96516[31], 3}, {NULL}};
const Segment SGI227595[] = {{&S96516[80], 5}, {NULL}};
const Segment SGI227596[] = {{&S96516[117], 14}, {NULL}};
const Segment SGI227597[] = {{&S96520[52], 10}, {NULL}};
const Segment SGI227598[] = {{&S96520[52], 22}, {NULL}};
const Segment SGI227599[] = {{&S96520[38], 6}, {NULL}};
const Segment SGI227600[] = {{&S96520[88], 5}, {NULL}};
const Segment SGI227601[] = {{&S96520[0], 14}, {NULL}};
const Segment SGI227602[] = {{&S96521[44], 7}, {NULL}};
const Segment SGI227603[] = {{&S96521[44], 19}, {NULL}};
const Segment SGI227604[] = {{&S96521[30], 6}, {NULL}};
const Segment SGI227605[] = {{&S96521[31], 3}, {NULL}};
const Segment SGI227606[] = {{&S96521[77], 5}, {NULL}};
const Segment SGI227607[] = {{&S96521[114], 14}, {NULL}};
const Segment SGI227608[] = {{&S92959[44], 6}, {NULL}};
const Segment SGI227609[] = {{&S92959[44], 18}, {NULL}};
const Segment SGI227610[] = {{&S92959[30], 6}, {NULL}};
const Segment SGI227611[] = {{&S92959[76], 6}, {NULL}};
const Segment SGI227612[] = {{&S92959[103], 13}, {NULL}};
const Segment SGI227613[] = {{&S96522[44], 15}, {NULL}};
const Segment SGI227614[] = {{&S96522[44], 27}, {NULL}};
const Segment SGI227615[] = {{&S96522[30], 6}, {NULL}};
const Segment SGI227616[] = {{&S96522[85], 6}, {NULL}};
const Segment SGI227617[] = {{&S96522[112], 13}, {NULL}};
const Segment SGI227618[] = {{&S96524[44], 14}, {NULL}};
const Segment SGI227619[] = {{&S96524[44], 26}, {NULL}};
const Segment SGI227620[] = {{&S96524[30], 6}, {NULL}};
const Segment SGI227621[] = {{&S96524[84], 6}, {NULL}};
const Segment SGI227622[] = {{&S96524[111], 13}, {NULL}};
const Segment SGI227623[] = {{&S92965[34], 21}, {NULL}};
const Segment SGI227624[] = {{&S92965[27], 6}, {NULL}};
const Segment SGI227625[] = {{&S92965[108], 9}, {NULL}};
const Segment SGI227626[] = {{&S92965[81], 6}, {NULL}};
const Segment SGI227627[] = {{&S96528[34], 4}, {NULL}};
const Segment SGI227628[] = {{&S96528[27], 6}, {NULL}};
const Segment SGI227629[] = {{&S96528[91], 9}, {NULL}};
const Segment SGI227630[] = {{&S96528[64], 6}, {NULL}};
const Segment SGI227631[] = {{&S78502[34], 12}, {NULL}};
const Segment SGI227632[] = {{&S78502[27], 6}, {NULL}};
const Segment SGI227633[] = {{&S96529[34], 8}, {NULL}};
const Segment SGI227634[] = {{&S96529[27], 6}, {NULL}};
const Segment SGI227635[] = {{&S78502[98], 9}, {NULL}};
const Segment SGI227636[] = {{&S78502[71], 6}, {NULL}};
const Segment SGI227637[] = {{&S96529[95], 9}, {NULL}};
const Segment SGI227638[] = {{&S96529[68], 6}, {NULL}};
const Segment SGI227639[] = {{&S96530[34], 9}, {NULL}};
const Segment SGI227640[] = {{&S96530[27], 6}, {NULL}};
const Segment SGI227641[] = {{&S96530[96], 9}, {NULL}};
const Segment SGI227642[] = {{&S96530[69], 6}, {NULL}};
const Segment SGI227643[] = {{&S96533[34], 10}, {NULL}};
const Segment SGI227644[] = {{&S96533[27], 6}, {NULL}};
const Segment SGI227645[] = {{&S96533[97], 9}, {NULL}};
const Segment SGI227646[] = {{&S96533[70], 6}, {NULL}};
const Segment SGI227647[] = {{&S92977[34], 7}, {NULL}};
const Segment SGI227648[] = {{&S92977[27], 6}, {NULL}};
const Segment SGI227649[] = {{&S92977[94], 9}, {NULL}};
const Segment SGI227650[] = {{&S96536[34], 13}, {NULL}};
const Segment SGI227651[] = {{&S92977[67], 6}, {NULL}};
const Segment SGI227652[] = {{&S96536[27], 6}, {NULL}};
const Segment SGI227653[] = {{&S96536[100], 9}, {NULL}};
const Segment SGI227654[] = {{&S96536[73], 6}, {NULL}};
const Segment SGI227655[] = {{&S96539[34], 9}, {NULL}};
const Segment SGI227656[] = {{&S96539[27], 6}, {NULL}};
const Segment SGI227657[] = {{&S96539[96], 9}, {NULL}};
const Segment SGI227658[] = {{&S96539[69], 6}, {NULL}};
const Segment SGI227659[] = {{&S78521[44], 8}, {NULL}};
const Segment SGI227660[] = {{&S78521[44], 28}, {NULL}};
const Segment SGI227661[] = {{&S78521[30], 6}, {NULL}};
const Segment SGI227662[] = {{&S78521[121], 3}, {NULL}};
const Segment SGI227663[] = {{&S78521[86], 6}, {NULL}};
const Segment SGI227664[] = {{&S78522[38], 8}, {NULL}};
const Segment SGI227665[] = {{&S78522[31], 6}, {NULL}};
const Segment SGI227666[] = {{&S96547[34], 11}, {NULL}};
const Segment SGI227667[] = {{&S96547[27], 6}, {NULL}};
const Segment SGI227668[] = {{&S96547[98], 9}, {NULL}};
const Segment SGI227669[] = {{&S96547[71], 6}, {NULL}};
const Segment SGI227670[] = {{&S92997[44], 12}, {NULL}};
const Segment SGI227671[] = {{&S92997[44], 24}, {NULL}};
const Segment SGI227672[] = {{&S92997[30], 6}, {NULL}};
const Segment SGI227673[] = {{&S86166[34], 7}, {NULL}};
const Segment SGI227674[] = {{&S86166[27], 6}, {NULL}};
const Segment SGI227675[] = {{&S92997[82], 6}, {NULL}};
const Segment SGI227676[] = {{&S92997[109], 13}, {NULL}};
const Segment SGI227677[] = {{&S86166[94], 9}, {NULL}};
const Segment SGI227678[] = {{&S86166[67], 6}, {NULL}};
const Segment SGI227679[] = {{&S86167[34], 8}, {NULL}};
const Segment SGI227680[] = {{&S86167[27], 6}, {NULL}};
const Segment SGI227681[] = {{&S86167[95], 9}, {NULL}};
const Segment SGI227682[] = {{&S86167[68], 6}, {NULL}};
const Segment SGI227683[] = {{&S96564[34], 9}, {NULL}};
const Segment SGI227684[] = {{&S96564[27], 6}, {NULL}};
const Segment SGI227685[] = {{&S96564[96], 9}, {NULL}};
const Segment SGI227686[] = {{&S96564[69], 6}, {NULL}};
const Segment SGI227687[] = {{&S96564[137], 8}, {NULL}};
const Segment SGI227688[] = {{&S86178[34], 5}, {NULL}};
const Segment SGI227689[] = {{&S86178[27], 6}, {NULL}};
const Segment SGI227690[] = {{&S86178[92], 9}, {NULL}};
const Segment SGI227691[] = {{&S86178[65], 6}, {NULL}};
const Segment SGI227692[] = {{&S93013[34], 13}, {NULL}};
const Segment SGI227693[] = {{&S93013[27], 6}, {NULL}};
const Segment SGI227694[] = {{&S93013[100], 9}, {NULL}};
const Segment SGI227695[] = {{&S93013[73], 6}, {NULL}};
const Segment SGI227696[] = {{&S93014[44], 8}, {NULL}};
const Segment SGI227697[] = {{&S93014[44], 20}, {NULL}};
const Segment SGI227698[] = {{&S93014[30], 6}, {NULL}};
const Segment SGI227699[] = {{&S93014[113], 3}, {NULL}};
const Segment SGI227700[] = {{&S93014[78], 6}, {NULL}};
const Segment SGI227701[] = {{&S93014[138], 7}, {NULL}};
const Segment SGI227702[] = {{&S96576[44], 7}, {NULL}};
const Segment SGI227703[] = {{&S96576[44], 25}, {NULL}};
const Segment SGI227704[] = {{&S96576[30], 6}, {NULL}};
const Segment SGI227705[] = {{&S96576[83], 6}, {NULL}};
const Segment SGI227706[] = {{&S96576[110], 13}, {NULL}};
const Segment SGI227707[] = {{&S86190[44], 16}, {NULL}};
const Segment SGI227708[] = {{&S86190[44], 25}, {NULL}};
const Segment SGI227709[] = {{&S86190[30], 6}, {NULL}};
const Segment SGI227710[] = {{&S86190[118], 3}, {NULL}};
const Segment SGI227711[] = {{&S86190[83], 6}, {NULL}};
const Segment SGI227712[] = {{&S93023[34], 6}, {NULL}};
const Segment SGI227713[] = {{&S93023[27], 6}, {NULL}};
const Segment SGI227714[] = {{&S78562[34], 9}, {NULL}};
const Segment SGI227715[] = {{&S78562[27], 6}, {NULL}};
const Segment SGI227716[] = {{&S96577[34], 7}, {NULL}};
const Segment SGI227717[] = {{&S93023[93], 9}, {NULL}};
const Segment SGI227718[] = {{&S93023[66], 6}, {NULL}};
const Segment SGI227719[] = {{&S96577[27], 6}, {NULL}};
const Segment SGI227720[] = {{&S78562[95], 9}, {NULL}};
const Segment SGI227721[] = {{&S78562[68], 6}, {NULL}};
const Segment SGI227722[] = {{&S86191[44], 8}, {NULL}};
const Segment SGI227723[] = {{&S86191[44], 19}, {NULL}};
const Segment SGI227724[] = {{&S86191[30], 6}, {NULL}};
const Segment SGI227725[] = {{&S96577[112], 9}, {NULL}};
const Segment SGI227726[] = {{&S96577[85], 6}, {NULL}};
const Segment SGI227727[] = {{&S86191[112], 3}, {NULL}};
const Segment SGI227728[] = {{&S86191[77], 6}, {NULL}};
const Segment SGI227729[] = {{&S96579[44], 6}, {NULL}};
const Segment SGI227730[] = {{&S96579[44], 18}, {NULL}};
const Segment SGI227731[] = {{&S96579[30], 6}, {NULL}};
const Segment SGI227732[] = {{&S96579[76], 6}, {NULL}};
const Segment SGI227733[] = {{&S96579[103], 13}, {NULL}};
const Segment SGI227734[] = {{&S96580[34], 5}, {NULL}};
const Segment SGI227735[] = {{&S96580[27], 6}, {NULL}};
const Segment SGI227736[] = {{&S96580[92], 9}, {NULL}};
const Segment SGI227737[] = {{&S96580[65], 6}, {NULL}};
const Segment SGI227738[] = {{&S86197[34], 8}, {NULL}};
const Segment SGI227739[] = {{&S86197[27], 6}, {NULL}};
const Segment SGI227740[] = {{&S86197[100], 9}, {NULL}};
const Segment SGI227741[] = {{&S86197[73], 6}, {NULL}};
const Segment SGI227742[] = {{&S96582[34], 11}, {NULL}};
const Segment SGI227743[] = {{&S96582[27], 6}, {NULL}};
const Segment SGI227744[] = {{&S86198[34], 13}, {NULL}};
const Segment SGI227745[] = {{&S86198[27], 6}, {NULL}};
const Segment SGI227746[] = {{&S96582[107], 9}, {NULL}};
const Segment SGI227747[] = {{&S96582[80], 6}, {NULL}};
const Segment SGI227748[] = {{&S86198[100], 9}, {NULL}};
const Segment SGI227749[] = {{&S86198[73], 6}, {NULL}};
const Segment SGI227750[] = {{&S96583[34], 18}, {NULL}};
const Segment SGI227751[] = {{&S96583[27], 6}, {NULL}};
const Segment SGI227752[] = {{&S96583[102], 9}, {NULL}};
const Segment SGI227753[] = {{&S96583[75], 6}, {NULL}};
const Segment SGI227754[] = {{&S96586[27], 6}, {NULL}};
const Segment SGI227755[] = {{&S96586[103], 9}, {NULL}};
const Segment SGI227756[] = {{&S96586[76], 6}, {NULL}};
const Segment SGI227757[] = {{&S86250[34], 11}, {NULL}};
const Segment SGI227758[] = {{&S86250[27], 6}, {NULL}};
const Segment SGI227759[] = {{&S86250[98], 9}, {NULL}};
const Segment SGI227760[] = {{&S86250[71], 6}, {NULL}};
const Segment SGI227761[] = {{&S86257[44], 10}, {NULL}};
const Segment SGI227762[] = {{&S86257[44], 22}, {NULL}};
const Segment SGI227763[] = {{&S86257[30], 6}, {NULL}};
const Segment SGI227764[] = {{&S86257[114], 3}, {NULL}};
const Segment SGI227765[] = {{&S86257[80], 5}, {NULL}};
const Segment SGI227766[] = {{&S86257[125], 14}, {NULL}};
const Segment SGI227767[] = {{&S96648[44], 10}, {NULL}};
const Segment SGI227768[] = {{&S96648[44], 22}, {NULL}};
const Segment SGI227769[] = {{&S96648[30], 6}, {NULL}};
const Segment SGI227770[] = {{&S96648[31], 3}, {NULL}};
const Segment SGI227771[] = {{&S96648[80], 6}, {NULL}};
const Segment SGI227772[] = {{&S86264[34], 11}, {NULL}};
const Segment SGI227773[] = {{&S86264[27], 6}, {NULL}};
const Segment SGI227774[] = {{&S86264[98], 9}, {NULL}};
const Segment SGI227775[] = {{&S96649[44], 10}, {NULL}};
const Segment SGI227776[] = {{&S86264[71], 6}, {NULL}};
const Segment SGI227777[] = {{&S96649[44], 22}, {NULL}};
const Segment SGI227778[] = {{&S96649[30], 6}, {NULL}};
const Segment SGI227779[] = {{&S96649[31], 3}, {NULL}};
const Segment SGI227780[] = {{&S96649[80], 6}, {NULL}};
const Segment SGI227781[] = {{&S86274[34], 6}, {NULL}};
const Segment SGI227782[] = {{&S86274[27], 6}, {NULL}};
const Segment SGI227783[] = {{&S86274[94], 9}, {NULL}};
const Segment SGI227784[] = {{&S86274[67], 6}, {NULL}};
const Segment SGI227785[] = {{&S93115[34], 6}, {NULL}};
const Segment SGI227786[] = {{&S93115[27], 6}, {NULL}};
const Segment SGI227787[] = {{&S93115[93], 9}, {NULL}};
const Segment SGI227788[] = {{&S93115[66], 6}, {NULL}};
const Segment SGI227789[] = {{&S78659[44], 10}, {NULL}};
const Segment SGI227790[] = {{&S78659[44], 22}, {NULL}};
const Segment SGI227791[] = {{&S78659[30], 6}, {NULL}};
const Segment SGI227792[] = {{&S96660[27], 6}, {NULL}};
const Segment SGI227793[] = {{&S78659[133], 3}, {NULL}};
const Segment SGI227794[] = {{&S78659[98], 6}, {NULL}};
const Segment SGI227795[] = {{&S96660[111], 9}, {NULL}};
const Segment SGI227796[] = {{&S96660[84], 6}, {NULL}};
const Segment SGI227797[] = {{&S78659[68], 8}, {NULL}};
const Segment SGI227798[] = {{&S78660[44], 10}, {NULL}};
const Segment SGI227799[] = {{&S78660[44], 21}, {NULL}};
const Segment SGI227800[] = {{&S78660[30], 6}, {NULL}};
const Segment SGI227801[] = {{&S78660[132], 3}, {NULL}};
const Segment SGI227802[] = {{&S78660[97], 6}, {NULL}};
const Segment SGI227803[] = {{&S78660[67], 8}, {NULL}};
const Segment SGI227804[] = {{&S86282[44], 7}, {NULL}};
const Segment SGI227805[] = {{&S86282[44], 19}, {NULL}};
const Segment SGI227806[] = {{&S86282[30], 6}, {NULL}};
const Segment SGI227807[] = {{&S86282[112], 3}, {NULL}};
const Segment SGI227808[] = {{&S86282[77], 6}, {NULL}};
const Segment SGI227809[] = {{&S86283[44], 13}, {NULL}};
const Segment SGI227810[] = {{&S86283[44], 25}, {NULL}};
const Segment SGI227811[] = {{&S86283[30], 6}, {NULL}};
const Segment SGI227812[] = {{&S86283[83], 6}, {NULL}};
const Segment SGI227813[] = {{&S86283[110], 13}, {NULL}};
const Segment SGI227814[] = {{&S96670[34], 7}, {NULL}};
const Segment SGI227815[] = {{&S96670[27], 6}, {NULL}};
const Segment SGI227816[] = {{&S96670[90], 9}, {NULL}};
const Segment SGI227817[] = {{&S96670[63], 6}, {NULL}};
const Segment SGI227818[] = {{&S86287[34], 10}, {NULL}};
const Segment SGI227819[] = {{&S86287[27], 6}, {NULL}};
const Segment SGI227820[] = {{&S86287[97], 9}, {NULL}};
const Segment SGI227821[] = {{&S86287[70], 6}, {NULL}};
const Segment SGI227822[] = {{&S86295[34], 8}, {NULL}};
const Segment SGI227823[] = {{&S86295[27], 6}, {NULL}};
const Segment SGI227824[] = {{&S86295[100], 9}, {NULL}};
const Segment SGI227825[] = {{&S86295[73], 6}, {NULL}};
const Segment SGI227826[] = {{&S96684[34], 22}, {NULL}};
const Segment SGI227827[] = {{&S96684[27], 6}, {NULL}};
const Segment SGI227828[] = {{&S86299[44], 8}, {NULL}};
const Segment SGI227829[] = {{&S86299[44], 30}, {NULL}};
const Segment SGI227830[] = {{&S86299[30], 6}, {NULL}};
const Segment SGI227831[] = {{&S96684[109], 9}, {NULL}};
const Segment SGI227832[] = {{&S96684[82], 6}, {NULL}};
const Segment SGI227833[] = {{&S86299[123], 3}, {NULL}};
const Segment SGI227834[] = {{&S86299[88], 6}, {NULL}};
const Segment SGI227835[] = {{&S86299[148], 7}, {NULL}};
const Segment SGI227836[] = {{&S86300[44], 8}, {NULL}};
const Segment SGI227837[] = {{&S86300[44], 32}, {NULL}};
const Segment SGI227838[] = {{&S86300[30], 6}, {NULL}};
const Segment SGI227839[] = {{&S86300[125], 3}, {NULL}};
const Segment SGI227840[] = {{&S86300[90], 6}, {NULL}};
const Segment SGI227841[] = {{&S86300[150], 7}, {NULL}};
const Segment SGI227842[] = {{&S86305[44], 9}, {NULL}};
const Segment SGI227843[] = {{&S86305[44], 21}, {NULL}};
const Segment SGI227844[] = {{&S86305[30], 6}, {NULL}};
const Segment SGI227845[] = {{&S78696[34], 18}, {NULL}};
const Segment SGI227846[] = {{&S86305[79], 6}, {NULL}};
const Segment SGI227847[] = {{&S78696[27], 6}, {NULL}};
const Segment SGI227848[] = {{&S86305[106], 13}, {NULL}};
const Segment SGI227849[] = {{&S78696[104], 9}, {NULL}};
const Segment SGI227850[] = {{&S78696[77], 6}, {NULL}};
const Segment SGI227851[] = {{&S96701[44], 13}, {NULL}};
const Segment SGI227852[] = {{&S96701[44], 25}, {NULL}};
const Segment SGI227853[] = {{&S96701[30], 6}, {NULL}};
const Segment SGI227854[] = {{&S96701[31], 3}, {NULL}};
const Segment SGI227855[] = {{&S78706[34], 7}, {NULL}};
const Segment SGI227856[] = {{&S96701[83], 5}, {NULL}};
const Segment SGI227857[] = {{&S78706[27], 6}, {NULL}};
const Segment SGI227858[] = {{&S96701[120], 14}, {NULL}};
const Segment SGI227859[] = {{&S78706[93], 9}, {NULL}};
const Segment SGI227860[] = {{&S78706[66], 6}, {NULL}};
const Segment SGI227861[] = {{&S96702[45], 5}, {NULL}};
const Segment SGI227862[] = {{&S96702[30], 6}, {NULL}};
const Segment SGI227863[] = {{&S96702[31], 3}, {NULL}};
const Segment SGI227864[] = {{&S96702[80], 5}, {NULL}};
const Segment SGI227865[] = {{&S96702[138], 13}, {NULL}};
const Segment SGI227866[] = {{&S78708[44], 7}, {NULL}};
const Segment SGI227867[] = {{&S78708[44], 18}, {NULL}};
const Segment SGI227868[] = {{&S78708[30], 6}, {NULL}};
const Segment SGI227869[] = {{&S78708[111], 3}, {NULL}};
const Segment SGI227870[] = {{&S78708[76], 6}, {NULL}};
const Segment SGI227871[] = {{&S96704[45], 10}, {NULL}};
const Segment SGI227872[] = {{&S96704[30], 6}, {NULL}};
const Segment SGI227873[] = {{&S96704[31], 3}, {NULL}};
const Segment SGI227874[] = {{&S96704[81], 5}, {NULL}};
const Segment SGI227875[] = {{&S96704[139], 13}, {NULL}};
const Segment SGI227876[] = {{&S78712[34], 10}, {NULL}};
const Segment SGI227877[] = {{&S78712[27], 6}, {NULL}};
const Segment SGI227878[] = {{&S78712[97], 9}, {NULL}};
const Segment SGI227879[] = {{&S78712[70], 6}, {NULL}};
const Segment SGI227880[] = {{&S96709[45], 7}, {NULL}};
const Segment SGI227881[] = {{&S96709[30], 6}, {NULL}};
const Segment SGI227882[] = {{&S96709[31], 3}, {NULL}};
const Segment SGI227883[] = {{&S96709[83], 5}, {NULL}};
const Segment SGI227884[] = {{&S96709[141], 13}, {NULL}};
const Segment SGI227885[] = {{&S96710[34], 12}, {NULL}};
const Segment SGI227886[] = {{&S96710[27], 6}, {NULL}};
const Segment SGI227887[] = {{&S96710[99], 9}, {NULL}};
const Segment SGI227888[] = {{&S96710[72], 6}, {NULL}};
const Segment SGI227889[] = {{&S87454[34], 38}, {NULL}};
const Segment SGI227890[] = {{&S87454[27], 6}, {NULL}};
const Segment SGI227891[] = {{&S87454[125], 9}, {NULL}};
const Segment SGI227892[] = {{&S87454[98], 6}, {NULL}};
const Segment SGI227893[] = {{&S87455[34], 25}, {NULL}};
const Segment SGI227894[] = {{&S87455[27], 6}, {NULL}};
const Segment SGI227895[] = {{&S87455[112], 9}, {NULL}};
const Segment SGI227896[] = {{&S87455[85], 6}, {NULL}};
const Segment SGI227897[] = {{&S96723[34], 4}, {NULL}};
const Segment SGI227898[] = {{&S96723[27], 6}, {NULL}};
const Segment SGI227899[] = {{&S96723[91], 9}, {NULL}};
const Segment SGI227900[] = {{&S96723[64], 6}, {NULL}};
const Segment SGI227901[] = {{&S96724[27], 6}, {NULL}};
const Segment SGI227902[] = {{&S96724[111], 9}, {NULL}};
const Segment SGI227903[] = {{&S96724[84], 6}, {NULL}};
const Segment SGI227904[] = {{&S96725[27], 6}, {NULL}};
const Segment SGI227905[] = {{&S96725[99], 9}, {NULL}};
const Segment SGI227906[] = {{&S96725[72], 6}, {NULL}};
const Segment SGI227907[] = {{&S96727[34], 8}, {NULL}};
const Segment SGI227908[] = {{&S96727[27], 6}, {NULL}};
const Segment SGI227909[] = {{&S96727[95], 9}, {NULL}};
const Segment SGI227910[] = {{&S96727[68], 6}, {NULL}};
const Segment SGI227911[] = {{&S96728[34], 15}, {NULL}};
const Segment SGI227912[] = {{&S96728[27], 6}, {NULL}};
const Segment SGI227913[] = {{&S96728[102], 9}, {NULL}};
const Segment SGI227914[] = {{&S96728[75], 6}, {NULL}};
const Segment SGI227915[] = {{&S96729[34], 10}, {NULL}};
const Segment SGI227916[] = {{&S96729[27], 6}, {NULL}};
const Segment SGI227917[] = {{&S86360[34], 7}, {NULL}};
const Segment SGI227918[] = {{&S86360[27], 6}, {NULL}};
const Segment SGI227919[] = {{&S96729[97], 9}, {NULL}};
const Segment SGI227920[] = {{&S96729[70], 6}, {NULL}};
const Segment SGI227921[] = {{&S86360[94], 9}, {NULL}};
const Segment SGI227922[] = {{&S86360[67], 6}, {NULL}};
const Segment SGI227923[] = {{&S96731[59], 7}, {NULL}};
const Segment SGI227924[] = {{&S96731[59], 25}, {NULL}};
const Segment SGI227925[] = {{&S96731[45], 6}, {NULL}};
const Segment SGI227926[] = {{&S96731[46], 3}, {NULL}};
const Segment SGI227927[] = {{&S96731[98], 6}, {NULL}};
const Segment SGI227928[] = {{&S96731[0], 14}, {NULL}};
const Segment SGI227929[] = {{&S78751[34], 10}, {NULL}};
const Segment SGI227930[] = {{&S78751[27], 6}, {NULL}};
const Segment SGI227931[] = {{&S78751[96], 9}, {NULL}};
const Segment SGI227932[] = {{&S78751[69], 6}, {NULL}};
const Segment SGI227933[] = {{&S78751[137], 8}, {NULL}};
const Segment SGI227934[] = {{&S78756[44], 10}, {NULL}};
const Segment SGI227935[] = {{&S78756[44], 21}, {NULL}};
const Segment SGI227936[] = {{&S78756[30], 6}, {NULL}};
const Segment SGI227937[] = {{&S78756[112], 3}, {NULL}};
const Segment SGI227938[] = {{&S78756[79], 5}, {NULL}};
const Segment SGI227939[] = {{&S86372[34], 10}, {NULL}};
const Segment SGI227940[] = {{&S86372[27], 6}, {NULL}};
const Segment SGI227941[] = {{&S86372[97], 9}, {NULL}};
const Segment SGI227942[] = {{&S86372[70], 6}, {NULL}};
const Segment SGI227943[] = {{&S96741[44], 14}, {NULL}};
const Segment SGI227944[] = {{&S96741[44], 26}, {NULL}};
const Segment SGI227945[] = {{&S86373[44], 7}, {NULL}};
const Segment SGI227946[] = {{&S96741[30], 6}, {NULL}};
const Segment SGI227947[] = {{&S86373[44], 19}, {NULL}};
const Segment SGI227948[] = {{&S86373[30], 6}, {NULL}};
const Segment SGI227949[] = {{&S96741[31], 3}, {NULL}};
const Segment SGI227950[] = {{&S96741[84], 5}, {NULL}};
const Segment SGI227951[] = {{&S86373[77], 6}, {NULL}};
const Segment SGI227952[] = {{&S96741[121], 14}, {NULL}};
const Segment SGI227953[] = {{&S86373[104], 13}, {NULL}};
const Segment SGI227954[] = {{&S96742[34], 5}, {NULL}};
const Segment SGI227955[] = {{&S96742[27], 6}, {NULL}};
const Segment SGI227956[] = {{&S96742[92], 9}, {NULL}};
const Segment SGI227957[] = {{&S96742[65], 6}, {NULL}};
const Segment SGI227958[] = {{&S96745[44], 10}, {NULL}};
const Segment SGI227959[] = {{&S96745[44], 22}, {NULL}};
const Segment SGI227960[] = {{&S96745[30], 6}, {NULL}};
const Segment SGI227961[] = {{&S96745[31], 3}, {NULL}};
const Segment SGI227962[] = {{&S96745[80], 6}, {NULL}};
const Segment SGI227963[] = {{&S96745[140], 13}, {NULL}};
const Segment SGI227964[] = {{&S86383[44], 8}, {NULL}};
const Segment SGI227965[] = {{&S86383[44], 20}, {NULL}};
const Segment SGI227966[] = {{&S96748[31], 16}, {NULL}};
const Segment SGI227967[] = {{&S86383[30], 6}, {NULL}};
const Segment SGI227968[] = {{&S96748[62], 3}, {NULL}};
const Segment SGI227969[] = {{&S86383[113], 3}, {NULL}};
const Segment SGI227970[] = {{&S86383[78], 6}, {NULL}};
const Segment SGI227971[] = {{&S96749[34], 8}, {NULL}};
const Segment SGI227972[] = {{&S96749[27], 6}, {NULL}};
const Segment SGI227973[] = {{&S96749[95], 9}, {NULL}};
const Segment SGI227974[] = {{&S96749[68], 6}, {NULL}};
const Segment SGI227975[] = {{&S96753[34], 7}, {NULL}};
const Segment SGI227976[] = {{&S86389[34], 10}, {NULL}};
const Segment SGI227977[] = {{&S96753[27], 6}, {NULL}};
const Segment SGI227978[] = {{&S86389[27], 6}, {NULL}};
const Segment SGI227979[] = {{&S96753[94], 9}, {NULL}};
const Segment SGI227980[] = {{&S86389[97], 9}, {NULL}};
const Segment SGI227981[] = {{&S96753[67], 6}, {NULL}};
const Segment SGI227982[] = {{&S87508[34], 25}, {NULL}};
const Segment SGI227983[] = {{&S86389[70], 6}, {NULL}};
const Segment SGI227984[] = {{&S87508[27], 6}, {NULL}};
const Segment SGI227985[] = {{&S87508[112], 9}, {NULL}};
const Segment SGI227986[] = {{&S87508[85], 6}, {NULL}};
const Segment SGI227987[] = {{&S87509[34], 23}, {NULL}};
const Segment SGI227988[] = {{&S87509[27], 6}, {NULL}};
const Segment SGI227989[] = {{&S87509[110], 9}, {NULL}};
const Segment SGI227990[] = {{&S87509[83], 6}, {NULL}};
const Segment SGI227991[] = {{&S86395[44], 11}, {NULL}};
const Segment SGI227992[] = {{&S86395[44], 23}, {NULL}};
const Segment SGI227993[] = {{&S86395[30], 6}, {NULL}};
const Segment SGI227994[] = {{&S86395[116], 3}, {NULL}};
const Segment SGI227995[] = {{&S86395[81], 6}, {NULL}};
const Segment SGI227996[] = {{&S96760[44], 7}, {NULL}};
const Segment SGI227997[] = {{&S96760[44], 18}, {NULL}};
const Segment SGI227998[] = {{&S96760[30], 6}, {NULL}};
const Segment SGI227999[] = {{&S96760[31], 3}, {NULL}};
const Segment SGI228000[] = {{&S96760[76], 6}, {NULL}};
const Segment SGI228001[] = {{&S96760[136], 11}, {NULL}};
const Segment SGI228002[] = {{&S96761[34], 11}, {NULL}};
const Segment SGI228003[] = {{&S96761[27], 6}, {NULL}};
const Segment SGI228004[] = {{&S96761[98], 9}, {NULL}};
const Segment SGI228005[] = {{&S96761[71], 6}, {NULL}};
const Segment SGI228006[] = {{&S96762[34], 14}, {NULL}};
const Segment SGI228007[] = {{&S96762[27], 6}, {NULL}};
const Segment SGI228008[] = {{&S96763[34], 12}, {NULL}};
const Segment SGI228009[] = {{&S96763[27], 6}, {NULL}};
const Segment SGI228010[] = {{&S96763[99], 9}, {NULL}};
const Segment SGI228011[] = {{&S96763[72], 6}, {NULL}};
const Segment SGI228012[] = {{&S96766[44], 6}, {NULL}};
const Segment SGI228013[] = {{&S96766[44], 18}, {NULL}};
const Segment SGI228014[] = {{&S96766[30], 6}, {NULL}};
const Segment SGI228015[] = {{&S96766[31], 3}, {NULL}};
const Segment SGI228016[] = {{&S96766[76], 5}, {NULL}};
const Segment SGI228017[] = {{&S96766[113], 14}, {NULL}};
const Segment SGI228018[] = {{&S96768[44], 6}, {NULL}};
const Segment SGI228019[] = {{&S96768[44], 18}, {NULL}};
const Segment SGI228020[] = {{&S96768[30], 6}, {NULL}};
const Segment SGI228021[] = {{&S96768[76], 6}, {NULL}};
const Segment SGI228022[] = {{&S96768[103], 13}, {NULL}};
const Segment SGI228023[] = {{&S96770[34], 10}, {NULL}};
const Segment SGI228024[] = {{&S96770[27], 6}, {NULL}};
const Segment SGI228025[] = {{&S96770[97], 9}, {NULL}};
const Segment SGI228026[] = {{&S96770[70], 6}, {NULL}};
const Segment SGI228027[] = {{&S96774[59], 7}, {NULL}};
const Segment SGI228028[] = {{&S96774[59], 19}, {NULL}};
const Segment SGI228029[] = {{&S96774[45], 6}, {NULL}};
const Segment SGI228030[] = {{&S96774[46], 3}, {NULL}};
const Segment SGI228031[] = {{&S96774[92], 6}, {NULL}};
const Segment SGI228032[] = {{&S96774[0], 14}, {NULL}};
const Segment SGI228033[] = {{&S96776[34], 7}, {NULL}};
const Segment SGI228034[] = {{&S96776[27], 6}, {NULL}};
const Segment SGI228035[] = {{&S93147[34], 20}, {NULL}};
const Segment SGI228036[] = {{&S96776[90], 9}, {NULL}};
const Segment SGI228037[] = {{&S93147[27], 6}, {NULL}};
const Segment SGI228038[] = {{&S96776[63], 6}, {NULL}};
const Segment SGI228039[] = {{&S93147[107], 9}, {NULL}};
const Segment SGI228040[] = {{&S93147[80], 6}, {NULL}};
const Segment SGI228041[] = {{&S96778[34], 11}, {NULL}};
const Segment SGI228042[] = {{&S96778[27], 6}, {NULL}};
const Segment SGI228043[] = {{&S96778[98], 9}, {NULL}};
const Segment SGI228044[] = {{&S96778[71], 6}, {NULL}};
const Segment SGI228045[] = {{&S93150[34], 12}, {NULL}};
const Segment SGI228046[] = {{&S93150[27], 6}, {NULL}};
const Segment SGI228047[] = {{&S93150[99], 9}, {NULL}};
const Segment SGI228048[] = {{&S93150[72], 6}, {NULL}};
const Segment SGI228049[] = {{&S96784[45], 10}, {NULL}};
const Segment SGI228050[] = {{&S96784[30], 6}, {NULL}};
const Segment SGI228051[] = {{&S96784[31], 3}, {NULL}};
const Segment SGI228052[] = {{&S96784[81], 5}, {NULL}};
const Segment SGI228053[] = {{&S96784[139], 13}, {NULL}};
const Segment SGI228054[] = {{&S96787[44], 10}, {NULL}};
const Segment SGI228055[] = {{&S96787[44], 22}, {NULL}};
const Segment SGI228056[] = {{&S96787[30], 6}, {NULL}};
const Segment SGI228057[] = {{&S96787[31], 3}, {NULL}};
const Segment SGI228058[] = {{&S96787[80], 5}, {NULL}};
const Segment SGI228059[] = {{&S96787[117], 14}, {NULL}};
const Segment SGI228060[] = {{&S93162[44], 14}, {NULL}};
const Segment SGI228061[] = {{&S93162[44], 26}, {NULL}};
const Segment SGI228062[] = {{&S93162[30], 6}, {NULL}};
const Segment SGI228063[] = {{&S93162[119], 3}, {NULL}};
const Segment SGI228064[] = {{&S93162[84], 6}, {NULL}};
const Segment SGI228065[] = {{&S96798[34], 11}, {NULL}};
const Segment SGI228066[] = {{&S96798[27], 6}, {NULL}};
const Segment SGI228067[] = {{&S96798[98], 9}, {NULL}};
const Segment SGI228068[] = {{&S96798[71], 6}, {NULL}};
const Segment SGI228069[] = {{&S93170[34], 15}, {NULL}};
const Segment SGI228070[] = {{&S93170[27], 6}, {NULL}};
const Segment SGI228071[] = {{&S93170[109], 9}, {NULL}};
const Segment SGI228072[] = {{&S93170[82], 6}, {NULL}};
const Segment SGI228073[] = {{&S96801[34], 5}, {NULL}};
const Segment SGI228074[] = {{&S96801[27], 6}, {NULL}};
const Segment SGI228075[] = {{&S93171[34], 13}, {NULL}};
const Segment SGI228076[] = {{&S93171[27], 6}, {NULL}};
const Segment SGI228077[] = {{&S96801[92], 9}, {NULL}};
const Segment SGI228078[] = {{&S96801[65], 6}, {NULL}};
const Segment SGI228079[] = {{&S93171[100], 9}, {NULL}};
const Segment SGI228080[] = {{&S93171[73], 6}, {NULL}};
const Segment SGI228081[] = {{&S86465[44], 11}, {NULL}};
const Segment SGI228082[] = {{&S86465[44], 23}, {NULL}};
const Segment SGI228083[] = {{&S86465[30], 6}, {NULL}};
const Segment SGI228084[] = {{&S86465[136], 3}, {NULL}};
const Segment SGI228085[] = {{&S86465[101], 6}, {NULL}};
const Segment SGI228086[] = {{&S86465[69], 8}, {NULL}};
const Segment SGI228087[] = {{&S96817[44], 11}, {NULL}};
const Segment SGI228088[] = {{&S96817[44], 23}, {NULL}};
const Segment SGI228089[] = {{&S96817[30], 6}, {NULL}};
const Segment SGI228090[] = {{&S96817[81], 6}, {NULL}};
const Segment SGI228091[] = {{&S96817[108], 13}, {NULL}};
const Segment SGI228092[] = {{&S96819[34], 10}, {NULL}};
const Segment SGI228093[] = {{&S96819[27], 6}, {NULL}};
const Segment SGI228094[] = {{&S96819[97], 9}, {NULL}};
const Segment SGI228095[] = {{&S96819[70], 6}, {NULL}};
const Segment SGI228096[] = {{&S96822[44], 6}, {NULL}};
const Segment SGI228097[] = {{&S96822[44], 18}, {NULL}};
const Segment SGI228098[] = {{&S96822[30], 6}, {NULL}};
const Segment SGI228099[] = {{&S96822[76], 6}, {NULL}};
const Segment SGI228100[] = {{&S96822[103], 13}, {NULL}};
const Segment SGI228101[] = {{&S86477[34], 10}, {NULL}};
const Segment SGI228102[] = {{&S86477[27], 6}, {NULL}};
const Segment SGI228103[] = {{&S86477[97], 9}, {NULL}};
const Segment SGI228104[] = {{&S86477[70], 6}, {NULL}};
const Segment SGI228105[] = {{&S96829[44], 7}, {NULL}};
const Segment SGI228106[] = {{&S96829[44], 18}, {NULL}};
const Segment SGI228107[] = {{&S96829[30], 6}, {NULL}};
const Segment SGI228108[] = {{&S96829[76], 6}, {NULL}};
const Segment SGI228109[] = {{&S96829[103], 13}, {NULL}};
const Segment SGI228110[] = {{&S96831[34], 12}, {NULL}};
const Segment SGI228111[] = {{&S96831[27], 6}, {NULL}};
const Segment SGI228112[] = {{&S96831[99], 9}, {NULL}};
const Segment SGI228113[] = {{&S96831[72], 6}, {NULL}};
const Segment SGI228114[] = {{&S86487[34], 10}, {NULL}};
const Segment SGI228115[] = {{&S86487[27], 6}, {NULL}};
const Segment SGI228116[] = {{&S86487[97], 9}, {NULL}};
const Segment SGI228117[] = {{&S86487[70], 6}, {NULL}};
const Segment SGI228118[] = {{&S96836[34], 11}, {NULL}};
const Segment SGI228119[] = {{&S96836[27], 6}, {NULL}};
const Segment SGI228120[] = {{&S96836[98], 9}, {NULL}};
const Segment SGI228121[] = {{&S96836[71], 6}, {NULL}};
const Segment SGI228122[] = {{&S96839[34], 10}, {NULL}};
const Segment SGI228123[] = {{&S96839[27], 6}, {NULL}};
const Segment SGI228124[] = {{&S96839[102], 9}, {NULL}};
const Segment SGI228125[] = {{&S96839[75], 6}, {NULL}};
const Segment SGI228126[] = {{&S86495[34], 10}, {NULL}};
const Segment SGI228127[] = {{&S86495[27], 6}, {NULL}};
const Segment SGI228128[] = {{&S86495[97], 9}, {NULL}};
const Segment SGI228129[] = {{&S86495[70], 6}, {NULL}};
const Segment SGI228130[] = {{&S96841[34], 11}, {NULL}};
const Segment SGI228131[] = {{&S96841[27], 6}, {NULL}};
const Segment SGI228132[] = {{&S96841[98], 9}, {NULL}};
const Segment SGI228133[] = {{&S96841[71], 6}, {NULL}};
const Segment SGI228134[] = {{&S96842[52], 13}, {NULL}};
const Segment SGI228135[] = {{&S96842[52], 25}, {NULL}};
const Segment SGI228136[] = {{&S96842[38], 6}, {NULL}};
const Segment SGI228137[] = {{&S96842[91], 5}, {NULL}};
const Segment SGI228138[] = {{&S96842[0], 14}, {NULL}};
const Segment SGI228139[] = {{&S87627[44], 8}, {NULL}};
const Segment SGI228140[] = {{&S87627[44], 20}, {NULL}};
const Segment SGI228141[] = {{&S87627[30], 6}, {NULL}};
const Segment SGI228142[] = {{&S96843[44], 18}, {NULL}};
const Segment SGI228143[] = {{&S87627[113], 3}, {NULL}};
const Segment SGI228144[] = {{&S96843[44], 30}, {NULL}};
const Segment SGI228145[] = {{&S87627[78], 6}, {NULL}};
const Segment SGI228146[] = {{&S96843[30], 6}, {NULL}};
const Segment SGI228147[] = {{&S96843[31], 3}, {NULL}};
const Segment SGI228148[] = {{&S96843[88], 6}, {NULL}};
const Segment SGI228149[] = {{&S96843[158], 3}, {NULL}};
const Segment SGI228150[] = {{&S96845[44], 10}, {NULL}};
const Segment SGI228151[] = {{&S96845[44], 22}, {NULL}};
const Segment SGI228152[] = {{&S96845[30], 6}, {NULL}};
const Segment SGI228153[] = {{&S96845[80], 6}, {NULL}};
const Segment SGI228154[] = {{&S96845[107], 13}, {NULL}};
const Segment SGI228155[] = {{&S86511[34], 10}, {NULL}};
const Segment SGI228156[] = {{&S86511[27], 6}, {NULL}};
const Segment SGI228157[] = {{&S86511[97], 9}, {NULL}};
const Segment SGI228158[] = {{&S96852[52], 15}, {NULL}};
const Segment SGI228159[] = {{&S86511[70], 6}, {NULL}};
const Segment SGI228160[] = {{&S96852[52], 42}, {NULL}};
const Segment SGI228161[] = {{&S96852[38], 6}, {NULL}};
const Segment SGI228162[] = {{&S96852[108], 5}, {NULL}};
const Segment SGI228163[] = {{&S96852[0], 14}, {NULL}};
const Segment SGI228164[] = {{&S87639[34], 13}, {NULL}};
const Segment SGI228165[] = {{&S87639[27], 6}, {NULL}};
const Segment SGI228166[] = {{&S93232[34], 18}, {NULL}};
const Segment SGI228167[] = {{&S93232[27], 6}, {NULL}};
const Segment SGI228168[] = {{&S87639[100], 9}, {NULL}};
const Segment SGI228169[] = {{&S87639[73], 6}, {NULL}};
const Segment SGI228170[] = {{&S93232[114], 9}, {NULL}};
const Segment SGI228171[] = {{&S93232[87], 6}, {NULL}};
const Segment SGI228172[] = {{&S96854[34], 14}, {NULL}};
const Segment SGI228173[] = {{&S96854[27], 6}, {NULL}};
const Segment SGI228174[] = {{&S87640[34], 17}, {NULL}};
const Segment SGI228175[] = {{&S87640[27], 6}, {NULL}};
const Segment SGI228176[] = {{&S96854[101], 9}, {NULL}};
const Segment SGI228177[] = {{&S96854[74], 6}, {NULL}};
const Segment SGI228178[] = {{&S87640[104], 9}, {NULL}};
const Segment SGI228179[] = {{&S87640[77], 6}, {NULL}};
const Segment SGI228180[] = {{&S87641[34], 10}, {NULL}};
const Segment SGI228181[] = {{&S87641[27], 6}, {NULL}};
const Segment SGI228182[] = {{&S87641[99], 9}, {NULL}};
const Segment SGI228183[] = {{&S87641[72], 6}, {NULL}};
const Segment SGI228184[] = {{&S87644[34], 15}, {NULL}};
const Segment SGI228185[] = {{&S87644[27], 6}, {NULL}};
const Segment SGI228186[] = {{&S87644[102], 9}, {NULL}};
const Segment SGI228187[] = {{&S87644[75], 6}, {NULL}};
const Segment SGI228188[] = {{&S87644[143], 8}, {NULL}};
const Segment SGI228189[] = {{&S93245[44], 15}, {NULL}};
const Segment SGI228190[] = {{&S93245[44], 27}, {NULL}};
const Segment SGI228191[] = {{&S93245[30], 6}, {NULL}};
const Segment SGI228192[] = {{&S96866[44], 11}, {NULL}};
const Segment SGI228193[] = {{&S96866[44], 23}, {NULL}};
const Segment SGI228194[] = {{&S96866[30], 6}, {NULL}};
const Segment SGI228195[] = {{&S93245[85], 6}, {NULL}};
const Segment SGI228196[] = {{&S93245[112], 13}, {NULL}};
const Segment SGI228197[] = {{&S78797[34], 11}, {NULL}};
const Segment SGI228198[] = {{&S78797[27], 6}, {NULL}};
const Segment SGI228199[] = {{&S78797[97], 9}, {NULL}};
const Segment SGI228200[] = {{&S78797[70], 6}, {NULL}};
const Segment SGI228201[] = {{&S78824[44], 12}, {NULL}};
const Segment SGI228202[] = {{&S78824[44], 23}, {NULL}};
const Segment SGI228203[] = {{&S78824[30], 6}, {NULL}};
const Segment SGI228204[] = {{&S78824[116], 3}, {NULL}};
const Segment SGI228205[] = {{&S78824[81], 6}, {NULL}};
const Segment SGI228206[] = {{&S86569[34], 14}, {NULL}};
const Segment SGI228207[] = {{&S86569[27], 6}, {NULL}};
const Segment SGI228208[] = {{&S86569[101], 9}, {NULL}};
const Segment SGI228209[] = {{&S86569[74], 6}, {NULL}};
const Segment SGI228210[] = {{&S78834[44], 7}, {NULL}};
const Segment SGI228211[] = {{&S78834[44], 18}, {NULL}};
const Segment SGI228212[] = {{&S78834[30], 6}, {NULL}};
const Segment SGI228213[] = {{&S78834[129], 3}, {NULL}};
const Segment SGI228214[] = {{&S78834[94], 6}, {NULL}};
const Segment SGI228215[] = {{&S78834[64], 8}, {NULL}};
const Segment SGI228216[] = {{&S78835[44], 7}, {NULL}};
const Segment SGI228217[] = {{&S78835[44], 19}, {NULL}};
const Segment SGI228218[] = {{&S78835[30], 6}, {NULL}};
const Segment SGI228219[] = {{&S78835[130], 3}, {NULL}};
const Segment SGI228220[] = {{&S78835[95], 6}, {NULL}};
const Segment SGI228221[] = {{&S86575[44], 9}, {NULL}};
const Segment SGI228222[] = {{&S78835[65], 8}, {NULL}};
const Segment SGI228223[] = {{&S86575[44], 21}, {NULL}};
const Segment SGI228224[] = {{&S86575[30], 6}, {NULL}};
const Segment SGI228225[] = {{&S86575[114], 3}, {NULL}};
const Segment SGI228226[] = {{&S86575[79], 6}, {NULL}};
const Segment SGI228227[] = {{&S86578[34], 10}, {NULL}};
const Segment SGI228228[] = {{&S86578[27], 6}, {NULL}};
const Segment SGI228229[] = {{&S86578[97], 9}, {NULL}};
const Segment SGI228230[] = {{&S86578[70], 6}, {NULL}};
const Segment SGI228231[] = {{&S78843[44], 18}, {NULL}};
const Segment SGI228232[] = {{&S78843[44], 30}, {NULL}};
const Segment SGI228233[] = {{&S78843[30], 6}, {NULL}};
const Segment SGI228234[] = {{&S78843[123], 3}, {NULL}};
const Segment SGI228235[] = {{&S78843[88], 6}, {NULL}};
const Segment SGI228236[] = {{&S78843[155], 15}, {NULL}};
const Segment SGI228237[] = {{&S78844[44], 10}, {NULL}};
const Segment SGI228238[] = {{&S78844[44], 22}, {NULL}};
const Segment SGI228239[] = {{&S78844[30], 6}, {NULL}};
const Segment SGI228240[] = {{&S78844[115], 3}, {NULL}};
const Segment SGI228241[] = {{&S78844[80], 6}, {NULL}};
const Segment SGI228242[] = {{&S86584[34], 14}, {NULL}};
const Segment SGI228243[] = {{&S78844[147], 15}, {NULL}};
const Segment SGI228244[] = {{&S86584[27], 6}, {NULL}};
const Segment SGI228245[] = {{&S86584[101], 9}, {NULL}};
const Segment SGI228246[] = {{&S86584[74], 6}, {NULL}};
const Segment SGI228247[] = {{&S86596[34], 10}, {NULL}};
const Segment SGI228248[] = {{&S86596[27], 6}, {NULL}};
const Segment SGI228249[] = {{&S86596[97], 9}, {NULL}};
const Segment SGI228250[] = {{&S86596[70], 6}, {NULL}};
const Segment SGI228251[] = {{&S86598[34], 11}, {NULL}};
const Segment SGI228252[] = {{&S86598[27], 6}, {NULL}};
const Segment SGI228253[] = {{&S86598[98], 9}, {NULL}};
const Segment SGI228254[] = {{&S86598[71], 6}, {NULL}};
const Segment SGI228255[] = {{&S86604[34], 10}, {NULL}};
const Segment SGI228256[] = {{&S86604[27], 6}, {NULL}};
const Segment SGI228257[] = {{&S86604[97], 9}, {NULL}};
const Segment SGI228258[] = {{&S86604[70], 6}, {NULL}};
const Segment SGI228259[] = {{&S86609[34], 7}, {NULL}};
const Segment SGI228260[] = {{&S86609[27], 6}, {NULL}};
const Segment SGI228261[] = {{&S86609[98], 9}, {NULL}};
const Segment SGI228262[] = {{&S86609[71], 6}, {NULL}};
const Segment SGI228263[] = {{&S86611[44], 6}, {NULL}};
const Segment SGI228264[] = {{&S86611[44], 18}, {NULL}};
const Segment SGI228265[] = {{&S86611[30], 6}, {NULL}};
const Segment SGI228266[] = {{&S86611[76], 6}, {NULL}};
const Segment SGI228267[] = {{&S86611[103], 13}, {NULL}};
const Segment SGI228268[] = {{&S87749[34], 14}, {NULL}};
const Segment SGI228269[] = {{&S87749[27], 6}, {NULL}};
const Segment SGI228270[] = {{&S87749[101], 9}, {NULL}};
const Segment SGI228271[] = {{&S87749[74], 6}, {NULL}};
const Segment SGI228272[] = {{&S86623[44], 9}, {NULL}};
const Segment SGI228273[] = {{&S86623[44], 26}, {NULL}};
const Segment SGI228274[] = {{&S86623[30], 6}, {NULL}};
const Segment SGI228275[] = {{&S86623[119], 3}, {NULL}};
const Segment SGI228276[] = {{&S86623[84], 6}, {NULL}};
const Segment SGI228277[] = {{&S87759[45], 4}, {NULL}};
const Segment SGI228278[] = {{&S87759[30], 6}, {NULL}};
const Segment SGI228279[] = {{&S87759[109], 3}, {NULL}};
const Segment SGI228280[] = {{&S87759[75], 5}, {NULL}};
const Segment SGI228281[] = {{&S87759[143], 3}, {NULL}};
const Segment SGI228282[] = {{&S87760[45], 17}, {NULL}};
const Segment SGI228283[] = {{&S87760[30], 6}, {NULL}};
const Segment SGI228284[] = {{&S86629[34], 10}, {NULL}};
const Segment SGI228285[] = {{&S87760[122], 3}, {NULL}};
const Segment SGI228286[] = {{&S86629[27], 6}, {NULL}};
const Segment SGI228287[] = {{&S87760[88], 5}, {NULL}};
const Segment SGI228288[] = {{&S87760[156], 3}, {NULL}};
const Segment SGI228289[] = {{&S86629[97], 9}, {NULL}};
const Segment SGI228290[] = {{&S86629[70], 6}, {NULL}};
const Segment SGI228291[] = {{&S86629[138], 8}, {NULL}};
const Segment SGI228292[] = {{&S87761[34], 11}, {NULL}};
const Segment SGI228293[] = {{&S87761[27], 6}, {NULL}};
const Segment SGI228294[] = {{&S87761[98], 9}, {NULL}};
const Segment SGI228295[] = {{&S93371[34], 13}, {NULL}};
const Segment SGI228296[] = {{&S87761[71], 6}, {NULL}};
const Segment SGI228297[] = {{&S93371[27], 6}, {NULL}};
const Segment SGI228298[] = {{&S93371[100], 9}, {NULL}};
const Segment SGI228299[] = {{&S93371[73], 6}, {NULL}};
const Segment SGI228300[] = {{&S86631[34], 10}, {NULL}};
const Segment SGI228301[] = {{&S86631[27], 6}, {NULL}};
const Segment SGI228302[] = {{&S86631[97], 9}, {NULL}};
const Segment SGI228303[] = {{&S86631[70], 6}, {NULL}};
const Segment SGI228304[] = {{&S86640[34], 5}, {NULL}};
const Segment SGI228305[] = {{&S86640[27], 6}, {NULL}};
const Segment SGI228306[] = {{&S86640[92], 9}, {NULL}};
const Segment SGI228307[] = {{&S86640[65], 6}, {NULL}};
const Segment SGI228308[] = {{&S86641[44], 26}, {NULL}};
const Segment SGI228309[] = {{&S86641[44], 38}, {NULL}};
const Segment SGI228310[] = {{&S86641[30], 6}, {NULL}};
const Segment SGI228311[] = {{&S86641[131], 3}, {NULL}};
const Segment SGI228312[] = {{&S86641[96], 6}, {NULL}};
const Segment SGI228313[] = {{&S78917[44], 10}, {NULL}};
const Segment SGI228314[] = {{&S78917[44], 21}, {NULL}};
const Segment SGI228315[] = {{&S78917[30], 6}, {NULL}};
const Segment SGI228316[] = {{&S78917[79], 6}, {NULL}};
const Segment SGI228317[] = {{&S78917[106], 13}, {NULL}};
const Segment SGI228318[] = {{&S87779[34], 15}, {NULL}};
const Segment SGI228319[] = {{&S87779[27], 6}, {NULL}};
const Segment SGI228320[] = {{&S87779[102], 9}, {NULL}};
const Segment SGI228321[] = {{&S93389[34], 10}, {NULL}};
const Segment SGI228322[] = {{&S87779[75], 6}, {NULL}};
const Segment SGI228323[] = {{&S93389[27], 6}, {NULL}};
const Segment SGI228324[] = {{&S93389[97], 9}, {NULL}};
const Segment SGI228325[] = {{&S93389[70], 6}, {NULL}};
const Segment SGI228326[] = {{&S86648[34], 8}, {NULL}};
const Segment SGI228327[] = {{&S86648[27], 6}, {NULL}};
const Segment SGI228328[] = {{&S86648[95], 9}, {NULL}};
const Segment SGI228329[] = {{&S86648[68], 6}, {NULL}};
const Segment SGI228330[] = {{&S93392[44], 17}, {NULL}};
const Segment SGI228331[] = {{&S93392[44], 29}, {NULL}};
const Segment SGI228332[] = {{&S93392[30], 6}, {NULL}};
const Segment SGI228333[] = {{&S93392[122], 3}, {NULL}};
const Segment SGI228334[] = {{&S93392[87], 6}, {NULL}};
const Segment SGI228335[] = {{&S93393[34], 18}, {NULL}};
const Segment SGI228336[] = {{&S93393[27], 6}, {NULL}};
const Segment SGI228337[] = {{&S93393[105], 9}, {NULL}};
const Segment SGI228338[] = {{&S93393[78], 6}, {NULL}};
const Segment SGI228339[] = {{&S86654[34], 10}, {NULL}};
const Segment SGI228340[] = {{&S86654[27], 6}, {NULL}};
const Segment SGI228341[] = {{&S86654[97], 9}, {NULL}};
const Segment SGI228342[] = {{&S86654[70], 6}, {NULL}};
const Segment SGI228343[] = {{&S93402[44], 18}, {NULL}};
const Segment SGI228344[] = {{&S93402[44], 30}, {NULL}};
const Segment SGI228345[] = {{&S93402[30], 6}, {NULL}};
const Segment SGI228346[] = {{&S93402[123], 3}, {NULL}};
const Segment SGI228347[] = {{&S93402[88], 6}, {NULL}};
const Segment SGI228348[] = {{&S78949[44], 29}, {NULL}};
const Segment SGI228349[] = {{&S78949[44], 40}, {NULL}};
const Segment SGI228350[] = {{&S78949[30], 6}, {NULL}};
const Segment SGI228351[] = {{&S78949[151], 3}, {NULL}};
const Segment SGI228352[] = {{&S78949[116], 6}, {NULL}};
const Segment SGI228353[] = {{&S93414[34], 7}, {NULL}};
const Segment SGI228354[] = {{&S78949[86], 8}, {NULL}};
const Segment SGI228355[] = {{&S93414[27], 6}, {NULL}};
const Segment SGI228356[] = {{&S93414[94], 9}, {NULL}};
const Segment SGI228357[] = {{&S93414[67], 6}, {NULL}};
const Segment SGI228358[] = {{&S78950[44], 29}, {NULL}};
const Segment SGI228359[] = {{&S78950[44], 41}, {NULL}};
const Segment SGI228360[] = {{&S78950[30], 6}, {NULL}};
const Segment SGI228361[] = {{&S78950[134], 3}, {NULL}};
const Segment SGI228362[] = {{&S78950[99], 6}, {NULL}};
const Segment SGI228363[] = {{&S78950[152], 8}, {NULL}};
const Segment SGI228364[] = {{&S78955[44], 14}, {NULL}};
const Segment SGI228365[] = {{&S78955[44], 25}, {NULL}};
const Segment SGI228366[] = {{&S78955[30], 6}, {NULL}};
const Segment SGI228367[] = {{&S78955[83], 6}, {NULL}};
const Segment SGI228368[] = {{&S78955[110], 13}, {NULL}};
const Segment SGI228369[] = {{&S93424[34], 5}, {NULL}};
const Segment SGI228370[] = {{&S93424[27], 6}, {NULL}};
const Segment SGI228371[] = {{&S93424[92], 9}, {NULL}};
const Segment SGI228372[] = {{&S93424[65], 6}, {NULL}};
const Segment SGI228373[] = {{&S86690[34], 7}, {NULL}};
const Segment SGI228374[] = {{&S86690[27], 6}, {NULL}};
const Segment SGI228375[] = {{&S86690[94], 9}, {NULL}};
const Segment SGI228376[] = {{&S86690[67], 6}, {NULL}};
const Segment SGI228377[] = {{&S78962[34], 28}, {NULL}};
const Segment SGI228378[] = {{&S78962[27], 6}, {NULL}};
const Segment SGI228379[] = {{&S78962[115], 9}, {NULL}};
const Segment SGI228380[] = {{&S78962[88], 6}, {NULL}};
const Segment SGI228381[] = {{&S78963[34], 12}, {NULL}};
const Segment SGI228382[] = {{&S78963[27], 6}, {NULL}};
const Segment SGI228383[] = {{&S78963[99], 9}, {NULL}};
const Segment SGI228384[] = {{&S78963[72], 6}, {NULL}};
const Segment SGI228385[] = {{&S86693[34], 10}, {NULL}};
const Segment SGI228386[] = {{&S86693[27], 6}, {NULL}};
const Segment SGI228387[] = {{&S86693[97], 9}, {NULL}};
const Segment SGI228388[] = {{&S86693[70], 6}, {NULL}};
const Segment SGI228389[] = {{&S86695[59], 10}, {NULL}};
const Segment SGI228390[] = {{&S86695[59], 22}, {NULL}};
const Segment SGI228391[] = {{&S86695[45], 6}, {NULL}};
const Segment SGI228392[] = {{&S86695[130], 3}, {NULL}};
const Segment SGI228393[] = {{&S86695[95], 6}, {NULL}};
const Segment SGI228394[] = {{&S86695[11], 3}, {NULL}};
const Segment SGI228395[] = {{&S86700[44], 13}, {NULL}};
const Segment SGI228396[] = {{&S86700[44], 25}, {NULL}};
const Segment SGI228397[] = {{&S86700[30], 6}, {NULL}};
const Segment SGI228398[] = {{&S86700[139], 3}, {NULL}};
const Segment SGI228399[] = {{&S86700[104], 6}, {NULL}};
const Segment SGI228400[] = {{&S87839[34], 10}, {NULL}};
const Segment SGI228401[] = {{&S87839[27], 6}, {NULL}};
const Segment SGI228402[] = {{&S87839[97], 9}, {NULL}};
const Segment SGI228403[] = {{&S87839[70], 6}, {NULL}};
const Segment SGI228404[] = {{&S93450[44], 24}, {NULL}};
const Segment SGI228405[] = {{&S93450[44], 36}, {NULL}};
const Segment SGI228406[] = {{&S93450[30], 6}, {NULL}};
const Segment SGI228407[] = {{&S93450[31], 3}, {NULL}};
const Segment SGI228408[] = {{&S93450[94], 6}, {NULL}};
const Segment SGI228409[] = {{&S93462[34], 19}, {NULL}};
const Segment SGI228410[] = {{&S93462[27], 6}, {NULL}};
const Segment SGI228411[] = {{&S93462[106], 9}, {NULL}};
const Segment SGI228412[] = {{&S93462[79], 6}, {NULL}};
const Segment SGI228413[] = {{&S79004[34], 11}, {NULL}};
const Segment SGI228414[] = {{&S79004[27], 6}, {NULL}};
const Segment SGI228415[] = {{&S79004[97], 9}, {NULL}};
const Segment SGI228416[] = {{&S79004[70], 6}, {NULL}};
const Segment SGI228417[] = {{&S79007[34], 18}, {NULL}};
const Segment SGI228418[] = {{&S79007[27], 6}, {NULL}};
const Segment SGI228419[] = {{&S79007[104], 9}, {NULL}};
const Segment SGI228420[] = {{&S79007[77], 6}, {NULL}};
const Segment SGI228421[] = {{&S79007[145], 8}, {NULL}};
const Segment SGI228422[] = {{&S93470[44], 10}, {NULL}};
const Segment SGI228423[] = {{&S93470[44], 22}, {NULL}};
const Segment SGI228424[] = {{&S93470[30], 6}, {NULL}};
const Segment SGI228425[] = {{&S93470[115], 3}, {NULL}};
const Segment SGI228426[] = {{&S93470[80], 6}, {NULL}};
const Segment SGI228427[] = {{&S86742[44], 10}, {NULL}};
const Segment SGI228428[] = {{&S86742[44], 22}, {NULL}};
const Segment SGI228429[] = {{&S86742[30], 6}, {NULL}};
const Segment SGI228430[] = {{&S86742[80], 6}, {NULL}};
const Segment SGI228431[] = {{&S86742[107], 13}, {NULL}};
const Segment SGI228432[] = {{&S86743[44], 7}, {NULL}};
const Segment SGI228433[] = {{&S86743[44], 24}, {NULL}};
const Segment SGI228434[] = {{&S86743[30], 6}, {NULL}};
const Segment SGI228435[] = {{&S86743[82], 6}, {NULL}};
const Segment SGI228436[] = {{&S86743[109], 13}, {NULL}};
const Segment SGI228437[] = {{&S86744[44], 10}, {NULL}};
const Segment SGI228438[] = {{&S86744[44], 22}, {NULL}};
const Segment SGI228439[] = {{&S86744[30], 6}, {NULL}};
const Segment SGI228440[] = {{&S86744[115], 3}, {NULL}};
const Segment SGI228441[] = {{&S86744[80], 6}, {NULL}};
const Segment SGI228442[] = {{&S86744[140], 7}, {NULL}};
const Segment SGI228443[] = {{&S93480[59], 10}, {NULL}};
const Segment SGI228444[] = {{&S93480[59], 22}, {NULL}};
const Segment SGI228445[] = {{&S93480[45], 6}, {NULL}};
const Segment SGI228446[] = {{&S93480[130], 3}, {NULL}};
const Segment SGI228447[] = {{&S93480[95], 6}, {NULL}};
const Segment SGI228448[] = {{&S93480[0], 14}, {NULL}};
const Segment SGI228449[] = {{&S93481[59], 7}, {NULL}};
const Segment SGI228450[] = {{&S93481[59], 19}, {NULL}};
const Segment SGI228451[] = {{&S93481[45], 6}, {NULL}};
const Segment SGI228452[] = {{&S93481[127], 3}, {NULL}};
const Segment SGI228453[] = {{&S93481[92], 6}, {NULL}};
const Segment SGI228454[] = {{&S93481[0], 14}, {NULL}};
const Segment SGI228455[] = {{&S86748[44], 10}, {NULL}};
const Segment SGI228456[] = {{&S86748[44], 22}, {NULL}};
const Segment SGI228457[] = {{&S86748[30], 6}, {NULL}};
const Segment SGI228458[] = {{&S86748[115], 3}, {NULL}};
const Segment SGI228459[] = {{&S86748[80], 6}, {NULL}};
const Segment SGI228460[] = {{&S87884[55], 6}, {NULL}};
const Segment SGI228461[] = {{&S86748[150], 3}, {NULL}};
const Segment SGI228462[] = {{&S87884[117], 6}, {NULL}};
const Segment SGI228463[] = {{&S93486[44], 8}, {NULL}};
const Segment SGI228464[] = {{&S93486[44], 20}, {NULL}};
const Segment SGI228465[] = {{&S93486[30], 6}, {NULL}};
const Segment SGI228466[] = {{&S93486[113], 3}, {NULL}};
const Segment SGI228467[] = {{&S93486[78], 6}, {NULL}};
const Segment SGI228468[] = {{&S87888[44], 10}, {NULL}};
const Segment SGI228469[] = {{&S87888[44], 22}, {NULL}};
const Segment SGI228470[] = {{&S87888[30], 6}, {NULL}};
const Segment SGI228471[] = {{&S87888[80], 6}, {NULL}};
const Segment SGI228472[] = {{&S87888[117], 3}, {NULL}};
const Segment SGI228473[] = {{&S93488[44], 7}, {NULL}};
const Segment SGI228474[] = {{&S93488[44], 24}, {NULL}};
const Segment SGI228475[] = {{&S93488[30], 6}, {NULL}};
const Segment SGI228476[] = {{&S93488[82], 6}, {NULL}};
const Segment SGI228477[] = {{&S93488[109], 13}, {NULL}};
const Segment SGI228478[] = {{&S86758[34], 10}, {NULL}};
const Segment SGI228479[] = {{&S86758[27], 6}, {NULL}};
const Segment SGI228480[] = {{&S86758[97], 9}, {NULL}};
const Segment SGI228481[] = {{&S86758[70], 6}, {NULL}};
const Segment SGI228482[] = {{&S86762[44], 15}, {NULL}};
const Segment SGI228483[] = {{&S86762[44], 33}, {NULL}};
const Segment SGI228484[] = {{&S86762[30], 6}, {NULL}};
const Segment SGI228485[] = {{&S86762[124], 3}, {NULL}};
const Segment SGI228486[] = {{&S86762[91], 5}, {NULL}};
const Segment SGI228487[] = {{&S86762[128], 14}, {NULL}};
const Segment SGI228488[] = {{&S86766[34], 10}, {NULL}};
const Segment SGI228489[] = {{&S86766[27], 6}, {NULL}};
const Segment SGI228490[] = {{&S79048[34], 28}, {NULL}};
const Segment SGI228491[] = {{&S86766[97], 9}, {NULL}};
const Segment SGI228492[] = {{&S86766[70], 6}, {NULL}};
const Segment SGI228493[] = {{&S79048[27], 6}, {NULL}};
const Segment SGI228494[] = {{&S79048[115], 9}, {NULL}};
const Segment SGI228495[] = {{&S79048[88], 6}, {NULL}};
const Segment SGI228496[] = {{&S79049[34], 12}, {NULL}};
const Segment SGI228497[] = {{&S79049[27], 6}, {NULL}};
const Segment SGI228498[] = {{&S79049[99], 9}, {NULL}};
const Segment SGI228499[] = {{&S79049[72], 6}, {NULL}};
const Segment SGI228500[] = {{&S79050[44], 7}, {NULL}};
const Segment SGI228501[] = {{&S79050[44], 25}, {NULL}};
const Segment SGI228502[] = {{&S79050[30], 6}, {NULL}};
const Segment SGI228503[] = {{&S79050[118], 3}, {NULL}};
const Segment SGI228504[] = {{&S79050[83], 6}, {NULL}};
const Segment SGI228505[] = {{&S79051[42], 7}, {NULL}};
const Segment SGI228506[] = {{&S79051[30], 6}, {NULL}};
const Segment SGI228507[] = {{&S79051[116], 3}, {NULL}};
const Segment SGI228508[] = {{&S79051[81], 6}, {NULL}};
const Segment SGI228509[] = {{&S79052[44], 7}, {NULL}};
const Segment SGI228510[] = {{&S79052[44], 25}, {NULL}};
const Segment SGI228511[] = {{&S79052[30], 6}, {NULL}};
const Segment SGI228512[] = {{&S79052[118], 3}, {NULL}};
const Segment SGI228513[] = {{&S79052[83], 6}, {NULL}};
const Segment SGI228514[] = {{&S79053[44], 7}, {NULL}};
const Segment SGI228515[] = {{&S79053[44], 25}, {NULL}};
const Segment SGI228516[] = {{&S79053[30], 6}, {NULL}};
const Segment SGI228517[] = {{&S79053[118], 3}, {NULL}};
const Segment SGI228518[] = {{&S79053[83], 6}, {NULL}};
const Segment SGI228519[] = {{&S79054[38], 7}, {NULL}};
const Segment SGI228520[] = {{&S79054[31], 6}, {NULL}};
const Segment SGI228521[] = {{&S79055[44], 11}, {NULL}};
const Segment SGI228522[] = {{&S79055[44], 29}, {NULL}};
const Segment SGI228523[] = {{&S79055[30], 6}, {NULL}};
const Segment SGI228524[] = {{&S79055[122], 3}, {NULL}};
const Segment SGI228525[] = {{&S79055[87], 6}, {NULL}};
const Segment SGI228526[] = {{&S79056[44], 7}, {NULL}};
const Segment SGI228527[] = {{&S79056[44], 25}, {NULL}};
const Segment SGI228528[] = {{&S79056[30], 6}, {NULL}};
const Segment SGI228529[] = {{&S79056[118], 3}, {NULL}};
const Segment SGI228530[] = {{&S79056[83], 6}, {NULL}};
const Segment SGI228531[] = {{&S96866[81], 6}, {NULL}};
const Segment SGI228532[] = {{&S96866[108], 13}, {NULL}};
const Segment SGI228533[] = {{&S96867[34], 17}, {NULL}};
const Segment SGI228534[] = {{&S96867[27], 6}, {NULL}};
const Segment SGI228535[] = {{&S96867[104], 9}, {NULL}};
const Segment SGI228536[] = {{&S96867[77], 6}, {NULL}};
const Segment SGI228537[] = {{&S96867[145], 8}, {NULL}};
const Segment SGI228538[] = {{&S79066[34], 17}, {NULL}};
const Segment SGI228539[] = {{&S79066[27], 6}, {NULL}};
const Segment SGI228540[] = {{&S79066[103], 9}, {NULL}};
const Segment SGI228541[] = {{&S79066[76], 6}, {NULL}};
const Segment SGI228542[] = {{&S86808[34], 10}, {NULL}};
const Segment SGI228543[] = {{&S86808[27], 6}, {NULL}};
const Segment SGI228544[] = {{&S86808[97], 9}, {NULL}};
const Segment SGI228545[] = {{&S86808[70], 6}, {NULL}};
const Segment SGI228546[] = {{&S96887[44], 10}, {NULL}};
const Segment SGI228547[] = {{&S96887[44], 22}, {NULL}};
const Segment SGI228548[] = {{&S96887[30], 6}, {NULL}};
const Segment SGI228549[] = {{&S96887[80], 6}, {NULL}};
const Segment SGI228550[] = {{&S96887[107], 13}, {NULL}};
const Segment SGI228551[] = {{&S96888[34], 13}, {NULL}};
const Segment SGI228552[] = {{&S96888[27], 6}, {NULL}};
const Segment SGI228553[] = {{&S96888[100], 9}, {NULL}};
const Segment SGI228554[] = {{&S96888[73], 6}, {NULL}};
const Segment SGI228555[] = {{&S96889[34], 18}, {NULL}};
const Segment SGI228556[] = {{&S96889[27], 6}, {NULL}};
const Segment SGI228557[] = {{&S96889[105], 9}, {NULL}};
const Segment SGI228558[] = {{&S96889[78], 6}, {NULL}};
const Segment SGI228559[] = {{&S96889[146], 8}, {NULL}};
const Segment SGI228560[] = {{&S79091[11], 1}, {NULL}};
const Segment SGI228561[] = {{&S79091[27], 6}, {NULL}};
const Segment SGI228562[] = {{&S79091[88], 9}, {NULL}};
const Segment SGI228563[] = {{&S79091[61], 6}, {NULL}};
const Segment SGI228564[] = {{&S96897[34], 16}, {NULL}};
const Segment SGI228565[] = {{&S96897[27], 6}, {NULL}};
const Segment SGI228566[] = {{&S96897[113], 9}, {NULL}};
const Segment SGI228567[] = {{&S96897[86], 6}, {NULL}};
const Segment SGI228568[] = {{&S87957[34], 7}, {NULL}};
const Segment SGI228569[] = {{&S87957[27], 6}, {NULL}};
const Segment SGI228570[] = {{&S87957[94], 9}, {NULL}};
const Segment SGI228571[] = {{&S87957[67], 6}, {NULL}};
const Segment SGI228572[] = {{&S86828[34], 10}, {NULL}};
const Segment SGI228573[] = {{&S86828[27], 6}, {NULL}};
const Segment SGI228574[] = {{&S86828[97], 9}, {NULL}};
const Segment SGI228575[] = {{&S86828[70], 6}, {NULL}};
const Segment SGI228576[] = {{&S86832[34], 10}, {NULL}};
const Segment SGI228577[] = {{&S86832[27], 6}, {NULL}};
const Segment SGI228578[] = {{&S79103[44], 10}, {NULL}};
const Segment SGI228579[] = {{&S86832[97], 9}, {NULL}};
const Segment SGI228580[] = {{&S86832[70], 6}, {NULL}};
const Segment SGI228581[] = {{&S79103[44], 21}, {NULL}};
const Segment SGI228582[] = {{&S79103[30], 6}, {NULL}};
const Segment SGI228583[] = {{&S79103[114], 3}, {NULL}};
const Segment SGI228584[] = {{&S79103[79], 6}, {NULL}};
const Segment SGI228585[] = {{&S79104[38], 10}, {NULL}};
const Segment SGI228586[] = {{&S79104[31], 6}, {NULL}};
const Segment SGI228587[] = {{&S86834[34], 9}, {NULL}};
const Segment SGI228588[] = {{&S86834[27], 6}, {NULL}};
const Segment SGI228589[] = {{&S86834[96], 9}, {NULL}};
const Segment SGI228590[] = {{&S86834[69], 6}, {NULL}};
const Segment SGI228591[] = {{&S86847[44], 10}, {NULL}};
const Segment SGI228592[] = {{&S86847[44], 22}, {NULL}};
const Segment SGI228593[] = {{&S86847[30], 6}, {NULL}};
const Segment SGI228594[] = {{&S86847[113], 3}, {NULL}};
const Segment SGI228595[] = {{&S86847[80], 5}, {NULL}};
const Segment SGI228596[] = {{&S79118[34], 9}, {NULL}};
const Segment SGI228597[] = {{&S79118[27], 6}, {NULL}};
const Segment SGI228598[] = {{&S79118[95], 9}, {NULL}};
const Segment SGI228599[] = {{&S79118[68], 6}, {NULL}};
const Segment SGI228600[] = {{&S86859[44], 10}, {NULL}};
const Segment SGI228601[] = {{&S86859[44], 22}, {NULL}};
const Segment SGI228602[] = {{&S86859[30], 6}, {NULL}};
const Segment SGI228603[] = {{&S86859[115], 3}, {NULL}};
const Segment SGI228604[] = {{&S86859[80], 6}, {NULL}};
const Segment SGI228605[] = {{&S86859[150], 3}, {NULL}};
const Segment SGI228606[] = {{&S86870[44], 18}, {NULL}};
const Segment SGI228607[] = {{&S86870[44], 39}, {NULL}};
const Segment SGI228608[] = {{&S86870[30], 6}, {NULL}};
const Segment SGI228609[] = {{&S86870[130], 3}, {NULL}};
const Segment SGI228610[] = {{&S86870[97], 5}, {NULL}};
const Segment SGI228611[] = {{&S86870[134], 14}, {NULL}};
const Segment SGI228612[] = {{&S86871[44], 18}, {NULL}};
const Segment SGI228613[] = {{&S86871[44], 39}, {NULL}};
const Segment SGI228614[] = {{&S86871[30], 6}, {NULL}};
const Segment SGI228615[] = {{&S86871[130], 3}, {NULL}};
const Segment SGI228616[] = {{&S86871[97], 5}, {NULL}};
const Segment SGI228617[] = {{&S86871[134], 14}, {NULL}};
const Segment SGI228618[] = {{&S96955[44], 6}, {NULL}};
const Segment SGI228619[] = {{&S96955[44], 18}, {NULL}};
const Segment SGI228620[] = {{&S96955[30], 6}, {NULL}};
const Segment SGI228621[] = {{&S96955[107], 2}, {NULL}};
const Segment SGI228622[] = {{&S96955[76], 5}, {NULL}};
const Segment SGI228623[] = {{&S86876[34], 8}, {NULL}};
const Segment SGI228624[] = {{&S86876[27], 6}, {NULL}};
const Segment SGI228625[] = {{&S86876[95], 9}, {NULL}};
const Segment SGI228626[] = {{&S86876[68], 6}, {NULL}};
const Segment SGI228627[] = {{&S96958[44], 6}, {NULL}};
const Segment SGI228628[] = {{&S96958[44], 17}, {NULL}};
const Segment SGI228629[] = {{&S96958[30], 6}, {NULL}};
const Segment SGI228630[] = {{&S96958[106], 2}, {NULL}};
const Segment SGI228631[] = {{&S96958[75], 5}, {NULL}};
const Segment SGI228632[] = {{&S79160[44], 14}, {NULL}};
const Segment SGI228633[] = {{&S79160[44], 25}, {NULL}};
const Segment SGI228634[] = {{&S79160[30], 6}, {NULL}};
const Segment SGI228635[] = {{&S79160[118], 3}, {NULL}};
const Segment SGI228636[] = {{&S79160[83], 6}, {NULL}};
const Segment SGI228637[] = {{&S79160[143], 11}, {NULL}};
const Segment SGI228638[] = {{&S96979[44], 10}, {NULL}};
const Segment SGI228639[] = {{&S96979[44], 22}, {NULL}};
const Segment SGI228640[] = {{&S96979[30], 6}, {NULL}};
const Segment SGI228641[] = {{&S96979[31], 3}, {NULL}};
const Segment SGI228642[] = {{&S96979[109], 6}, {NULL}};
const Segment SGI228643[] = {{&S96979[68], 8}, {NULL}};
const Segment SGI228644[] = {{&S96979[80], 15}, {NULL}};
const Segment SGI228645[] = {{&S79175[41], 18}, {NULL}};
const Segment SGI228646[] = {{&S79175[41], 29}, {NULL}};
const Segment SGI228647[] = {{&S79175[27], 6}, {NULL}};
const Segment SGI228648[] = {{&S79175[119], 3}, {NULL}};
const Segment SGI228649[] = {{&S79175[123], 9}, {NULL}};
const Segment SGI228650[] = {{&S79175[84], 6}, {NULL}};
const Segment SGI228651[] = {{&S96986[34], 12}, {NULL}};
const Segment SGI228652[] = {{&S96986[27], 6}, {NULL}};
const Segment SGI228653[] = {{&S96986[97], 9}, {NULL}};
const Segment SGI228654[] = {{&S96986[70], 6}, {NULL}};
const Segment SGI228655[] = {{&S97007[59], 10}, {NULL}};
const Segment SGI228656[] = {{&S97007[59], 22}, {NULL}};
const Segment SGI228657[] = {{&S97007[45], 6}, {NULL}};
const Segment SGI228658[] = {{&S97007[46], 3}, {NULL}};
const Segment SGI228659[] = {{&S97007[95], 6}, {NULL}};
const Segment SGI228660[] = {{&S97007[0], 14}, {NULL}};
const Segment SGI228661[] = {{&S97010[34], 9}, {NULL}};
const Segment SGI228662[] = {{&S97010[27], 6}, {NULL}};
const Segment SGI228663[] = {{&S97010[96], 9}, {NULL}};
const Segment SGI228664[] = {{&S97010[69], 6}, {NULL}};
const Segment SGI228665[] = {{&S79215[44], 9}, {NULL}};
const Segment SGI228666[] = {{&S79215[44], 20}, {NULL}};
const Segment SGI228667[] = {{&S79215[30], 6}, {NULL}};
const Segment SGI228668[] = {{&S79215[78], 6}, {NULL}};
const Segment SGI228669[] = {{&S79215[105], 13}, {NULL}};
const Segment SGI228670[] = {{&S97027[34], 9}, {NULL}};
const Segment SGI228671[] = {{&S97027[27], 6}, {NULL}};
const Segment SGI228672[] = {{&S97027[96], 9}, {NULL}};
const Segment SGI228673[] = {{&S97027[69], 6}, {NULL}};
const Segment SGI228674[] = {{&S79219[34], 8}, {NULL}};
const Segment SGI228675[] = {{&S79219[27], 6}, {NULL}};
const Segment SGI228676[] = {{&S79219[94], 9}, {NULL}};
const Segment SGI228677[] = {{&S79219[67], 6}, {NULL}};
const Segment SGI228678[] = {{&S79220[34], 8}, {NULL}};
const Segment SGI228679[] = {{&S79220[27], 6}, {NULL}};
const Segment SGI228680[] = {{&S79220[94], 9}, {NULL}};
const Segment SGI228681[] = {{&S79220[67], 6}, {NULL}};
const Segment SGI228682[] = {{&S97031[34], 7}, {NULL}};
const Segment SGI228683[] = {{&S97031[27], 6}, {NULL}};
const Segment SGI228684[] = {{&S97031[95], 9}, {NULL}};
const Segment SGI228685[] = {{&S97031[68], 6}, {NULL}};
const Segment SGI228686[] = {{&S86960[34], 9}, {NULL}};
const Segment SGI228687[] = {{&S86960[27], 6}, {NULL}};
const Segment SGI228688[] = {{&S86960[96], 9}, {NULL}};
const Segment SGI228689[] = {{&S86960[69], 6}, {NULL}};
const Segment SGI228690[] = {{&S97033[34], 10}, {NULL}};
const Segment SGI228691[] = {{&S97033[27], 6}, {NULL}};
const Segment SGI228692[] = {{&S97033[97], 9}, {NULL}};
const Segment SGI228693[] = {{&S97033[70], 6}, {NULL}};
const Segment SGI228694[] = {{&S79227[44], 9}, {NULL}};
const Segment SGI228695[] = {{&S79227[44], 20}, {NULL}};
const Segment SGI228696[] = {{&S79227[30], 6}, {NULL}};
const Segment SGI228697[] = {{&S79227[113], 3}, {NULL}};
const Segment SGI228698[] = {{&S79227[78], 6}, {NULL}};
const Segment SGI228699[] = {{&S88104[44], 7}, {NULL}};
const Segment SGI228700[] = {{&S88104[44], 19}, {NULL}};
const Segment SGI228701[] = {{&S88104[30], 6}, {NULL}};
const Segment SGI228702[] = {{&S88104[130], 3}, {NULL}};
const Segment SGI228703[] = {{&S88104[95], 6}, {NULL}};
const Segment SGI228704[] = {{&S79228[38], 9}, {NULL}};
const Segment SGI228705[] = {{&S79228[31], 6}, {NULL}};
const Segment SGI228706[] = {{&S97037[44], 10}, {NULL}};
const Segment SGI228707[] = {{&S88104[65], 8}, {NULL}};
const Segment SGI228708[] = {{&S97037[44], 22}, {NULL}};
const Segment SGI228709[] = {{&S97037[30], 6}, {NULL}};
const Segment SGI228710[] = {{&S97037[31], 3}, {NULL}};
const Segment SGI228711[] = {{&S97037[95], 6}, {NULL}};
const Segment SGI228712[] = {{&S97037[68], 8}, {NULL}};
const Segment SGI228713[] = {{&S88105[44], 7}, {NULL}};
const Segment SGI228714[] = {{&S88105[44], 19}, {NULL}};
const Segment SGI228715[] = {{&S88105[30], 6}, {NULL}};
const Segment SGI228716[] = {{&S88105[130], 3}, {NULL}};
const Segment SGI228717[] = {{&S88105[95], 6}, {NULL}};
const Segment SGI228718[] = {{&S88105[65], 8}, {NULL}};
const Segment SGI228719[] = {{&S97038[44], 10}, {NULL}};
const Segment SGI228720[] = {{&S97038[44], 22}, {NULL}};
const Segment SGI228721[] = {{&S97038[30], 6}, {NULL}};
const Segment SGI228722[] = {{&S79230[34], 4}, {NULL}};
const Segment SGI228723[] = {{&S79230[27], 6}, {NULL}};
const Segment SGI228724[] = {{&S97038[31], 3}, {NULL}};
const Segment SGI228725[] = {{&S97038[95], 6}, {NULL}};
const Segment SGI228726[] = {{&S79230[90], 9}, {NULL}};
const Segment SGI228727[] = {{&S79230[63], 6}, {NULL}};
const Segment SGI228728[] = {{&S97038[68], 8}, {NULL}};
const Segment SGI228729[] = {{&S97039[44], 10}, {NULL}};
const Segment SGI228730[] = {{&S79231[34], 5}, {NULL}};
const Segment SGI228731[] = {{&S97039[44], 22}, {NULL}};
const Segment SGI228732[] = {{&S79231[27], 6}, {NULL}};
const Segment SGI228733[] = {{&S97039[30], 6}, {NULL}};
const Segment SGI228734[] = {{&S97039[31], 3}, {NULL}};
const Segment SGI228735[] = {{&S79231[92], 9}, {NULL}};
const Segment SGI228736[] = {{&S79231[65], 6}, {NULL}};
const Segment SGI228737[] = {{&S97039[95], 6}, {NULL}};
const Segment SGI228738[] = {{&S97039[68], 8}, {NULL}};
const Segment SGI228739[] = {{&S97040[44], 10}, {&S97040[190], 18}, {NULL}};
const Segment SGI228740[] = {{&S97040[44], 22}, {NULL}};
const Segment SGI228741[] = {{&S97040[30], 6}, {NULL}};
const Segment SGI228742[] = {{&S97040[31], 3}, {NULL}};
const Segment SGI228743[] = {{&S86975[34], 14}, {NULL}};
const Segment SGI228744[] = {{&S86975[27], 6}, {NULL}};
const Segment SGI228745[] = {{&S97040[95], 6}, {NULL}};
const Segment SGI228746[] = {{&S97040[68], 8}, {NULL}};
const Segment SGI228747[] = {{&S86975[101], 9}, {NULL}};
const Segment SGI228748[] = {{&S86975[74], 6}, {NULL}};
const Segment SGI228749[] = {{&S86977[44], 10}, {NULL}};
const Segment SGI228750[] = {{&S86977[44], 22}, {NULL}};
const Segment SGI228751[] = {{&S86977[30], 6}, {NULL}};
const Segment SGI228752[] = {{&S86977[113], 3}, {NULL}};
const Segment SGI228753[] = {{&S86977[80], 5}, {NULL}};
const Segment SGI228754[] = {{&S86977[117], 14}, {NULL}};
const Segment SGI228755[] = {{&S86980[34], 10}, {NULL}};
const Segment SGI228756[] = {{&S86980[27], 6}, {NULL}};
const Segment SGI228757[] = {{&S86980[97], 9}, {NULL}};
const Segment SGI228758[] = {{&S86980[70], 6}, {NULL}};
const Segment SGI228759[] = {{&S86981[34], 10}, {NULL}};
const Segment SGI228760[] = {{&S86981[27], 6}, {NULL}};
const Segment SGI228761[] = {{&S86981[97], 9}, {NULL}};
const Segment SGI228762[] = {{&S86981[70], 6}, {NULL}};
const Segment SGI228763[] = {{&S93579[44], 12}, {NULL}};
const Segment SGI228764[] = {{&S93579[44], 24}, {NULL}};
const Segment SGI228765[] = {{&S86981[131], 8}, {NULL}};
const Segment SGI228766[] = {{&S93579[30], 6}, {NULL}};
const Segment SGI228767[] = {{&S93579[117], 3}, {NULL}};
const Segment SGI228768[] = {{&S93579[82], 6}, {NULL}};
const Segment SGI228769[] = {{&S97053[44], 11}, {NULL}};
const Segment SGI228770[] = {{&S97053[44], 23}, {NULL}};
const Segment SGI228771[] = {{&S97053[30], 6}, {NULL}};
const Segment SGI228772[] = {{&S97053[81], 6}, {NULL}};
const Segment SGI228773[] = {{&S97053[108], 13}, {NULL}};
const Segment SGI228774[] = {{&S97054[34], 8}, {NULL}};
const Segment SGI228775[] = {{&S97054[27], 6}, {NULL}};
const Segment SGI228776[] = {{&S97054[95], 9}, {NULL}};
const Segment SGI228777[] = {{&S97054[68], 6}, {NULL}};
const Segment SGI228778[] = {{&S86990[44], 10}, {NULL}};
const Segment SGI228779[] = {{&S86990[44], 22}, {NULL}};
const Segment SGI228780[] = {{&S86990[30], 6}, {NULL}};
const Segment SGI228781[] = {{&S97055[34], 15}, {NULL}};
const Segment SGI228782[] = {{&S79254[34], 8}, {NULL}};
const Segment SGI228783[] = {{&S97055[27], 6}, {NULL}};
const Segment SGI228784[] = {{&S86990[113], 3}, {NULL}};
const Segment SGI228785[] = {{&S79254[27], 6}, {NULL}};
const Segment SGI228786[] = {{&S86990[80], 5}, {NULL}};
const Segment SGI228787[] = {{&S97055[102], 9}, {NULL}};
const Segment SGI228788[] = {{&S86990[117], 14}, {NULL}};
const Segment SGI228789[] = {{&S79254[94], 9}, {NULL}};
const Segment SGI228790[] = {{&S97055[75], 6}, {NULL}};
const Segment SGI228791[] = {{&S79254[67], 6}, {NULL}};
const Segment SGI228792[] = {{&S86991[59], 10}, {NULL}};
const Segment SGI228793[] = {{&S86991[59], 22}, {NULL}};
const Segment SGI228794[] = {{&S86991[45], 6}, {NULL}};
const Segment SGI228795[] = {{&S79255[44], 10}, {NULL}};
const Segment SGI228796[] = {{&S79255[44], 21}, {NULL}};
const Segment SGI228797[] = {{&S86991[130], 3}, {NULL}};
const Segment SGI228798[] = {{&S79255[30], 6}, {NULL}};
const Segment SGI228799[] = {{&S86991[95], 6}, {NULL}};
const Segment SGI228800[] = {{&S79255[114], 3}, {NULL}};
const Segment SGI228801[] = {{&S86991[0], 14}, {NULL}};
const Segment SGI228802[] = {{&S79255[79], 6}, {NULL}};
const Segment SGI228803[] = {{&S79255[139], 7}, {NULL}};
const Segment SGI228804[] = {{&S97061[34], 10}, {NULL}};
const Segment SGI228805[] = {{&S97061[27], 6}, {NULL}};
const Segment SGI228806[] = {{&S97061[97], 9}, {NULL}};
const Segment SGI228807[] = {{&S97061[70], 6}, {NULL}};
const Segment SGI228808[] = {{&S97062[34], 10}, {NULL}};
const Segment SGI228809[] = {{&S86999[34], 9}, {NULL}};
const Segment SGI228810[] = {{&S97062[27], 6}, {NULL}};
const Segment SGI228811[] = {{&S86999[27], 6}, {NULL}};
const Segment SGI228812[] = {{&S97062[97], 9}, {NULL}};
const Segment SGI228813[] = {{&S97062[70], 6}, {NULL}};
const Segment SGI228814[] = {{&S86999[96], 9}, {NULL}};
const Segment SGI228815[] = {{&S86999[69], 6}, {NULL}};
const Segment SGI228816[] = {{&S97063[34], 7}, {NULL}};
const Segment SGI228817[] = {{&S87000[34], 11}, {NULL}};
const Segment SGI228818[] = {{&S97063[27], 6}, {NULL}};
const Segment SGI228819[] = {{&S87000[27], 6}, {NULL}};
const Segment SGI228820[] = {{&S97063[94], 9}, {NULL}};
const Segment SGI228821[] = {{&S87000[98], 9}, {NULL}};
const Segment SGI228822[] = {{&S97063[67], 6}, {NULL}};
const Segment SGI228823[] = {{&S87000[71], 6}, {NULL}};
const Segment SGI228824[] = {{&S97064[34], 10}, {NULL}};
const Segment SGI228825[] = {{&S97064[27], 6}, {NULL}};
const Segment SGI228826[] = {{&S97064[97], 9}, {NULL}};
const Segment SGI228827[] = {{&S97064[70], 6}, {NULL}};
const Segment SGI228828[] = {{&S88146[34], 8}, {NULL}};
const Segment SGI228829[] = {{&S88146[27], 6}, {NULL}};
const Segment SGI228830[] = {{&S87003[34], 10}, {NULL}};
const Segment SGI228831[] = {{&S87003[27], 6}, {NULL}};
const Segment SGI228832[] = {{&S88146[95], 9}, {NULL}};
const Segment SGI228833[] = {{&S88146[68], 6}, {NULL}};
const Segment SGI228834[] = {{&S97065[34], 9}, {NULL}};
const Segment SGI228835[] = {{&S87003[97], 9}, {NULL}};
const Segment SGI228836[] = {{&S87003[70], 6}, {NULL}};
const Segment SGI228837[] = {{&S97065[27], 6}, {NULL}};
const Segment SGI228838[] = {{&S97065[96], 9}, {NULL}};
const Segment SGI228839[] = {{&S97065[69], 6}, {NULL}};
const Segment SGI228840[] = {{&S93608[45], 10}, {NULL}};
const Segment SGI228841[] = {{&S93608[30], 6}, {NULL}};
const Segment SGI228842[] = {{&S93608[120], 3}, {NULL}};
const Segment SGI228843[] = {{&S93608[86], 5}, {NULL}};
const Segment SGI228844[] = {{&S93608[144], 13}, {NULL}};
const Segment SGI228845[] = {{&S87008[44], 7}, {NULL}};
const Segment SGI228846[] = {{&S87008[44], 20}, {NULL}};
const Segment SGI228847[] = {{&S87008[30], 6}, {NULL}};
const Segment SGI228848[] = {{&S87008[113], 3}, {NULL}};
const Segment SGI228849[] = {{&S87008[78], 6}, {NULL}};
const Segment SGI228850[] = {{&S93611[34], 8}, {NULL}};
const Segment SGI228851[] = {{&S93611[27], 6}, {NULL}};
const Segment SGI228852[] = {{&S93611[95], 9}, {NULL}};
const Segment SGI228853[] = {{&S97071[34], 12}, {NULL}};
const Segment SGI228854[] = {{&S93611[68], 6}, {NULL}};
const Segment SGI228855[] = {{&S97071[27], 6}, {NULL}};
const Segment SGI228856[] = {{&S97071[99], 9}, {NULL}};
const Segment SGI228857[] = {{&S87010[44], 7}, {NULL}};
const Segment SGI228858[] = {{&S97071[72], 6}, {NULL}};
const Segment SGI228859[] = {{&S87010[44], 19}, {NULL}};
const Segment SGI228860[] = {{&S87010[30], 6}, {NULL}};
const Segment SGI228861[] = {{&S87010[77], 6}, {NULL}};
const Segment SGI228862[] = {{&S87010[104], 13}, {NULL}};
const Segment SGI228863[] = {{&S97072[34], 11}, {NULL}};
const Segment SGI228864[] = {{&S97072[27], 6}, {NULL}};
const Segment SGI228865[] = {{&S97072[98], 9}, {NULL}};
const Segment SGI228866[] = {{&S87011[34], 10}, {NULL}};
const Segment SGI228867[] = {{&S97072[71], 6}, {NULL}};
const Segment SGI228868[] = {{&S87011[27], 6}, {NULL}};
const Segment SGI228869[] = {{&S87011[97], 9}, {NULL}};
const Segment SGI228870[] = {{&S87011[70], 6}, {NULL}};
const Segment SGI228871[] = {{&S97074[34], 10}, {NULL}};
const Segment SGI228872[] = {{&S97074[27], 6}, {NULL}};
const Segment SGI228873[] = {{&S97074[97], 9}, {NULL}};
const Segment SGI228874[] = {{&S97074[70], 6}, {NULL}};
const Segment SGI228875[] = {{&S87014[34], 10}, {NULL}};
const Segment SGI228876[] = {{&S87014[27], 6}, {NULL}};
const Segment SGI228877[] = {{&S87014[96], 9}, {NULL}};
const Segment SGI228878[] = {{&S87014[69], 6}, {NULL}};
const Segment SGI228879[] = {{&S97077[45], 24}, {NULL}};
const Segment SGI228880[] = {{&S97077[30], 6}, {NULL}};
const Segment SGI228881[] = {{&S97077[31], 3}, {NULL}};
const Segment SGI228882[] = {{&S87017[34], 10}, {NULL}};
const Segment SGI228883[] = {{&S97077[97], 5}, {NULL}};
const Segment SGI228884[] = {{&S87017[27], 6}, {NULL}};
const Segment SGI228885[] = {{&S97077[155], 13}, {NULL}};
const Segment SGI228886[] = {{&S87017[97], 9}, {NULL}};
const Segment SGI228887[] = {{&S87017[70], 6}, {NULL}};
const Segment SGI228888[] = {{&S97078[34], 13}, {NULL}};
const Segment SGI228889[] = {{&S97078[27], 6}, {NULL}};
const Segment SGI228890[] = {{&S79289[34], 7}, {NULL}};
const Segment SGI228891[] = {{&S79289[27], 6}, {NULL}};
const Segment SGI228892[] = {{&S97078[100], 9}, {NULL}};
const Segment SGI228893[] = {{&S97078[73], 6}, {NULL}};
const Segment SGI228894[] = {{&S79289[93], 9}, {NULL}};
const Segment SGI228895[] = {{&S79289[66], 6}, {NULL}};
const Segment SGI228896[] = {{&S97081[34], 10}, {NULL}};
const Segment SGI228897[] = {{&S97081[27], 6}, {NULL}};
const Segment SGI228898[] = {{&S97081[97], 9}, {NULL}};
const Segment SGI228899[] = {{&S97081[70], 6}, {NULL}};
const Segment SGI228900[] = {{&S97083[34], 5}, {NULL}};
const Segment SGI228901[] = {{&S97083[27], 6}, {NULL}};
const Segment SGI228902[] = {{&S97083[96], 9}, {NULL}};
const Segment SGI228903[] = {{&S97083[69], 6}, {NULL}};
const Segment SGI228904[] = {{&S93635[34], 15}, {NULL}};
const Segment SGI228905[] = {{&S93635[27], 6}, {NULL}};
const Segment SGI228906[] = {{&S97086[34], 14}, {NULL}};
const Segment SGI228907[] = {{&S93635[102], 9}, {NULL}};
const Segment SGI228908[] = {{&S97086[27], 6}, {NULL}};
const Segment SGI228909[] = {{&S93635[75], 6}, {NULL}};
const Segment SGI228910[] = {{&S97086[101], 9}, {NULL}};
const Segment SGI228911[] = {{&S97086[74], 6}, {NULL}};
const Segment SGI228912[] = {{&S97087[44], 11}, {NULL}};
const Segment SGI228913[] = {{&S97087[44], 23}, {NULL}};
const Segment SGI228914[] = {{&S97087[30], 6}, {NULL}};
const Segment SGI228915[] = {{&S93637[44], 14}, {NULL}};
const Segment SGI228916[] = {{&S93637[44], 31}, {NULL}};
const Segment SGI228917[] = {{&S93637[30], 6}, {NULL}};
const Segment SGI228918[] = {{&S97087[81], 6}, {NULL}};
const Segment SGI228919[] = {{&S97087[108], 13}, {NULL}};
const Segment SGI228920[] = {{&S93637[122], 3}, {NULL}};
const Segment SGI228921[] = {{&S93637[89], 5}, {NULL}};
const Segment SGI228922[] = {{&S93637[126], 14}, {NULL}};
const Segment SGI228923[] = {{&S97090[44], 14}, {NULL}};
const Segment SGI228924[] = {{&S97090[44], 26}, {NULL}};
const Segment SGI228925[] = {{&S97090[30], 6}, {NULL}};
const Segment SGI228926[] = {{&S97090[84], 6}, {NULL}};
const Segment SGI228927[] = {{&S97090[111], 13}, {NULL}};
const Segment SGI228928[] = {{&S97092[44], 8}, {NULL}};
const Segment SGI228929[] = {{&S97092[44], 20}, {NULL}};
const Segment SGI228930[] = {{&S97092[30], 6}, {NULL}};
const Segment SGI228931[] = {{&S97092[31], 3}, {NULL}};
const Segment SGI228932[] = {{&S97092[78], 6}, {NULL}};
const Segment SGI228933[] = {{&S93651[34], 18}, {NULL}};
const Segment SGI228934[] = {{&S93651[27], 6}, {NULL}};
const Segment SGI228935[] = {{&S93651[114], 9}, {NULL}};
const Segment SGI228936[] = {{&S93651[87], 6}, {NULL}};
const Segment SGI228937[] = {{&S93651[155], 8}, {NULL}};
const Segment SGI228938[] = {{&S97103[44], 10}, {NULL}};
const Segment SGI228939[] = {{&S97103[44], 22}, {NULL}};
const Segment SGI228940[] = {{&S97103[30], 6}, {NULL}};
const Segment SGI228941[] = {{&S97103[31], 3}, {NULL}};
const Segment SGI228942[] = {{&S97103[80], 5}, {NULL}};
const Segment SGI228943[] = {{&S97103[117], 14}, {NULL}};
const Segment SGI228944[] = {{&S93657[44], 13}, {NULL}};
const Segment SGI228945[] = {{&S93657[44], 25}, {NULL}};
const Segment SGI228946[] = {{&S93657[30], 6}, {NULL}};
const Segment SGI228947[] = {{&S93657[117], 3}, {NULL}};
const Segment SGI228948[] = {{&S93657[83], 5}, {NULL}};
const Segment SGI228949[] = {{&S93657[128], 14}, {NULL}};
const Segment SGI228950[] = {{&S97109[34], 5}, {NULL}};
const Segment SGI228951[] = {{&S97109[27], 6}, {NULL}};
const Segment SGI228952[] = {{&S97109[92], 9}, {NULL}};
const Segment SGI228953[] = {{&S97109[65], 6}, {NULL}};
const Segment SGI228954[] = {{&S97112[34], 10}, {NULL}};
const Segment SGI228955[] = {{&S97112[27], 6}, {NULL}};
const Segment SGI228956[] = {{&S97112[97], 9}, {NULL}};
const Segment SGI228957[] = {{&S97112[70], 6}, {NULL}};
const Segment SGI228958[] = {{&S97114[34], 15}, {NULL}};
const Segment SGI228959[] = {{&S97114[27], 6}, {NULL}};
const Segment SGI228960[] = {{&S97114[102], 9}, {NULL}};
const Segment SGI228961[] = {{&S97114[75], 6}, {NULL}};
const Segment SGI228962[] = {{&S93667[34], 12}, {NULL}};
const Segment SGI228963[] = {{&S93667[27], 6}, {NULL}};
const Segment SGI228964[] = {{&S93667[99], 9}, {NULL}};
const Segment SGI228965[] = {{&S93667[72], 6}, {NULL}};
const Segment SGI228966[] = {{&S97115[34], 8}, {NULL}};
const Segment SGI228967[] = {{&S97115[27], 6}, {NULL}};
const Segment SGI228968[] = {{&S97115[94], 9}, {NULL}};
const Segment SGI228969[] = {{&S97115[67], 6}, {NULL}};
const Segment SGI228970[] = {{&S97118[34], 24}, {NULL}};
const Segment SGI228971[] = {{&S97118[27], 6}, {NULL}};
const Segment SGI228972[] = {{&S93671[44], 14}, {NULL}};
const Segment SGI228973[] = {{&S93671[44], 26}, {NULL}};
const Segment SGI228974[] = {{&S93671[30], 6}, {NULL}};
const Segment SGI228975[] = {{&S97118[111], 9}, {NULL}};
const Segment SGI228976[] = {{&S97118[84], 6}, {NULL}};
const Segment SGI228977[] = {{&S93671[84], 6}, {NULL}};
const Segment SGI228978[] = {{&S93671[111], 13}, {NULL}};
const Segment SGI228979[] = {{&S93674[44], 6}, {NULL}};
const Segment SGI228980[] = {{&S93674[44], 18}, {NULL}};
const Segment SGI228981[] = {{&S93674[30], 6}, {NULL}};
const Segment SGI228982[] = {{&S93674[109], 3}, {NULL}};
const Segment SGI228983[] = {{&S93674[76], 5}, {NULL}};
const Segment SGI228984[] = {{&S93674[113], 14}, {NULL}};
const Segment SGI228985[] = {{&S79352[44], 10}, {NULL}};
const Segment SGI228986[] = {{&S79352[44], 21}, {NULL}};
const Segment SGI228987[] = {{&S79352[30], 6}, {NULL}};
const Segment SGI228988[] = {{&S93678[34], 17}, {NULL}};
const Segment SGI228989[] = {{&S93678[27], 6}, {NULL}};
const Segment SGI228990[] = {{&S79352[79], 6}, {NULL}};
const Segment SGI228991[] = {{&S79352[106], 13}, {NULL}};
const Segment SGI228992[] = {{&S93678[104], 9}, {NULL}};
const Segment SGI228993[] = {{&S93678[77], 6}, {NULL}};
const Segment SGI228994[] = {{&S88243[34], 8}, {NULL}};
const Segment SGI228995[] = {{&S88243[27], 6}, {NULL}};
const Segment SGI228996[] = {{&S88243[95], 9}, {NULL}};
const Segment SGI228997[] = {{&S88243[68], 6}, {NULL}};
const Segment SGI228998[] = {{&S97142[59], 12}, {NULL}};
const Segment SGI228999[] = {{&S97142[59], 24}, {NULL}};
const Segment SGI229000[] = {{&S97142[45], 6}, {NULL}};
const Segment SGI229001[] = {{&S97142[46], 3}, {NULL}};
const Segment SGI229002[] = {{&S97142[97], 6}, {NULL}};
const Segment SGI229003[] = {{&S97142[0], 14}, {NULL}};
const Segment SGI229004[] = {{&S97149[34], 9}, {NULL}};
const Segment SGI229005[] = {{&S97149[27], 6}, {NULL}};
const Segment SGI229006[] = {{&S97149[96], 9}, {NULL}};
const Segment SGI229007[] = {{&S97149[69], 6}, {NULL}};
const Segment SGI229008[] = {{&S97152[34], 7}, {NULL}};
const Segment SGI229009[] = {{&S97152[27], 6}, {NULL}};
const Segment SGI229010[] = {{&S97152[99], 9}, {NULL}};
const Segment SGI229011[] = {{&S97152[72], 6}, {NULL}};
const Segment SGI229012[] = {{&S97153[44], 11}, {NULL}};
const Segment SGI229013[] = {{&S97153[44], 23}, {NULL}};
const Segment SGI229014[] = {{&S97153[30], 6}, {NULL}};
const Segment SGI229015[] = {{&S97153[31], 3}, {NULL}};
const Segment SGI229016[] = {{&S97153[81], 6}, {NULL}};
const Segment SGI229017[] = {{&S97153[141], 7}, {NULL}};
const Segment SGI229018[] = {{&S87121[44], 10}, {NULL}};
const Segment SGI229019[] = {{&S87121[44], 22}, {NULL}};
const Segment SGI229020[] = {{&S87121[30], 6}, {NULL}};
const Segment SGI229021[] = {{&S87121[131], 3}, {NULL}};
const Segment SGI229022[] = {{&S97154[34], 20}, {NULL}};
const Segment SGI229023[] = {{&S87121[96], 6}, {NULL}};
const Segment SGI229024[] = {{&S97154[27], 6}, {NULL}};
const Segment SGI229025[] = {{&S87121[68], 8}, {NULL}};
const Segment SGI229026[] = {{&S97154[107], 9}, {NULL}};
const Segment SGI229027[] = {{&S97154[80], 6}, {NULL}};
const Segment SGI229028[] = {{&S87122[44], 10}, {NULL}};
const Segment SGI229029[] = {{&S87122[44], 58}, {NULL}};
const Segment SGI229030[] = {{&S87122[30], 6}, {NULL}};
const Segment SGI229031[] = {{&S93710[34], 7}, {NULL}};
const Segment SGI229032[] = {{&S87122[167], 3}, {NULL}};
const Segment SGI229033[] = {{&S93710[27], 6}, {NULL}};
const Segment SGI229034[] = {{&S87122[132], 6}, {NULL}};
const Segment SGI229035[] = {{&S87122[104], 8}, {NULL}};
const Segment SGI229036[] = {{&S93710[94], 9}, {NULL}};
const Segment SGI229037[] = {{&S93710[67], 6}, {NULL}};
const Segment SGI229038[] = {{&S97163[44], 7}, {NULL}};
const Segment SGI229039[] = {{&S97163[44], 24}, {NULL}};
const Segment SGI229040[] = {{&S97163[30], 6}, {NULL}};
const Segment SGI229041[] = {{&S97163[82], 6}, {NULL}};
const Segment SGI229042[] = {{&S97163[109], 13}, {NULL}};
const Segment SGI229043[] = {{&S97164[44], 5}, {NULL}};
const Segment SGI229044[] = {{&S97164[44], 21}, {NULL}};
const Segment SGI229045[] = {{&S97164[30], 6}, {NULL}};
const Segment SGI229046[] = {{&S97164[79], 6}, {NULL}};
const Segment SGI229047[] = {{&S97164[106], 13}, {NULL}};
const Segment SGI229048[] = {{&S97165[34], 7}, {NULL}};
const Segment SGI229049[] = {{&S97165[27], 6}, {NULL}};
const Segment SGI229050[] = {{&S97165[94], 9}, {NULL}};
const Segment SGI229051[] = {{&S97165[67], 6}, {NULL}};
const Segment SGI229052[] = {{&S87136[34], 10}, {NULL}};
const Segment SGI229053[] = {{&S87136[27], 6}, {NULL}};
const Segment SGI229054[] = {{&S87136[97], 9}, {NULL}};
const Segment SGI229055[] = {{&S87136[70], 6}, {NULL}};
const Segment SGI229056[] = {{&S97167[34], 5}, {NULL}};
const Segment SGI229057[] = {{&S97167[27], 6}, {NULL}};
const Segment SGI229058[] = {{&S97167[92], 9}, {NULL}};
const Segment SGI229059[] = {{&S97167[65], 6}, {NULL}};
const Segment SGI229060[] = {{&S79400[34], 10}, {NULL}};
const Segment SGI229061[] = {{&S79400[27], 6}, {NULL}};
const Segment SGI229062[] = {{&S79400[97], 9}, {NULL}};
const Segment SGI229063[] = {{&S97169[34], 7}, {NULL}};
const Segment SGI229064[] = {{&S79400[70], 6}, {NULL}};
const Segment SGI229065[] = {{&S97169[27], 6}, {NULL}};
const Segment SGI229066[] = {{&S97169[94], 9}, {NULL}};
const Segment SGI229067[] = {{&S97169[67], 6}, {NULL}};
const Segment SGI229068[] = {{&S97170[34], 15}, {NULL}};
const Segment SGI229069[] = {{&S97170[27], 6}, {NULL}};
const Segment SGI229070[] = {{&S97170[102], 9}, {NULL}};
const Segment SGI229071[] = {{&S97170[75], 6}, {NULL}};
const Segment SGI229072[] = {{&S87142[34], 14}, {NULL}};
const Segment SGI229073[] = {{&S87142[27], 6}, {NULL}};
const Segment SGI229074[] = {{&S87142[106], 9}, {NULL}};
const Segment SGI229075[] = {{&S87142[79], 6}, {NULL}};
const Segment SGI229076[] = {{&S79404[34], 10}, {NULL}};
const Segment SGI229077[] = {{&S79404[27], 6}, {NULL}};
const Segment SGI229078[] = {{&S79404[97], 9}, {NULL}};
const Segment SGI229079[] = {{&S79404[70], 6}, {NULL}};
const Segment SGI229080[] = {{&S97172[34], 7}, {NULL}};
const Segment SGI229081[] = {{&S97172[27], 6}, {NULL}};
const Segment SGI229082[] = {{&S97172[90], 9}, {NULL}};
const Segment SGI229083[] = {{&S97172[63], 6}, {NULL}};
const Segment SGI229084[] = {{&S97174[34], 5}, {NULL}};
const Segment SGI229085[] = {{&S97174[27], 6}, {NULL}};
const Segment SGI229086[] = {{&S97174[92], 9}, {NULL}};
const Segment SGI229087[] = {{&S97174[65], 6}, {NULL}};
const Segment SGI229088[] = {{&S87148[34], 9}, {NULL}};
const Segment SGI229089[] = {{&S87148[27], 6}, {NULL}};
const Segment SGI229090[] = {{&S87148[96], 9}, {NULL}};
const Segment SGI229091[] = {{&S87148[69], 6}, {NULL}};
const Segment SGI229092[] = {{&S97175[34], 7}, {NULL}};
const Segment SGI229093[] = {{&S97175[27], 6}, {NULL}};
const Segment SGI229094[] = {{&S97175[99], 9}, {NULL}};
const Segment SGI229095[] = {{&S97175[72], 6}, {NULL}};
const Segment SGI229096[] = {{&S87149[44], 9}, {NULL}};
const Segment SGI229097[] = {{&S87149[44], 26}, {NULL}};
const Segment SGI229098[] = {{&S87149[30], 6}, {NULL}};
const Segment SGI229099[] = {{&S87149[84], 6}, {NULL}};
const Segment SGI229100[] = {{&S87149[111], 13}, {NULL}};
const Segment SGI229101[] = {{&S97176[34], 10}, {NULL}};
const Segment SGI229102[] = {{&S97176[27], 6}, {NULL}};
const Segment SGI229103[] = {{&S97176[97], 9}, {NULL}};
const Segment SGI229104[] = {{&S97176[70], 6}, {NULL}};
const Segment SGI229105[] = {{&S97178[34], 16}, {NULL}};
const Segment SGI229106[] = {{&S97178[27], 6}, {NULL}};
const Segment SGI229107[] = {{&S97178[116], 9}, {NULL}};
const Segment SGI229108[] = {{&S97178[89], 6}, {NULL}};
const Segment SGI229109[] = {{&S87153[44], 10}, {NULL}};
const Segment SGI229110[] = {{&S87153[44], 22}, {NULL}};
const Segment SGI229111[] = {{&S87153[30], 6}, {NULL}};
const Segment SGI229112[] = {{&S87153[80], 6}, {NULL}};
const Segment SGI229113[] = {{&S87153[107], 13}, {NULL}};
const Segment SGI229114[] = {{&S97183[34], 11}, {NULL}};
const Segment SGI229115[] = {{&S97183[27], 6}, {NULL}};
const Segment SGI229116[] = {{&S97183[98], 9}, {NULL}};
const Segment SGI229117[] = {{&S97183[71], 6}, {NULL}};
const Segment SGI229118[] = {{&S97184[44], 13}, {NULL}};
const Segment SGI229119[] = {{&S97184[44], 29}, {NULL}};
const Segment SGI229120[] = {{&S97184[30], 6}, {NULL}};
const Segment SGI229121[] = {{&S97184[31], 3}, {NULL}};
const Segment SGI229122[] = {{&S97184[87], 6}, {NULL}};
const Segment SGI229123[] = {{&S87160[34], 11}, {NULL}};
const Segment SGI229124[] = {{&S87160[27], 6}, {NULL}};
const Segment SGI229125[] = {{&S97184[147], 7}, {NULL}};
const Segment SGI229126[] = {{&S87160[98], 9}, {NULL}};
const Segment SGI229127[] = {{&S87160[71], 6}, {NULL}};
const Segment SGI229128[] = {{&S88332[34], 6}, {NULL}};
const Segment SGI229129[] = {{&S88332[27], 6}, {NULL}};
const Segment SGI229130[] = {{&S88332[93], 9}, {NULL}};
const Segment SGI229131[] = {{&S88332[66], 6}, {NULL}};
const Segment SGI229132[] = {{&S97198[44], 13}, {NULL}};
const Segment SGI229133[] = {{&S97198[44], 25}, {NULL}};
const Segment SGI229134[] = {{&S97198[30], 6}, {NULL}};
const Segment SGI229135[] = {{&S97198[83], 6}, {NULL}};
const Segment SGI229136[] = {{&S97198[110], 13}, {NULL}};
const Segment SGI229137[] = {{&S87177[34], 16}, {NULL}};
const Segment SGI229138[] = {{&S87177[27], 6}, {NULL}};
const Segment SGI229139[] = {{&S87177[103], 9}, {NULL}};
const Segment SGI229140[] = {{&S87177[76], 6}, {NULL}};
const Segment SGI229141[] = {{&S97205[44], 13}, {NULL}};
const Segment SGI229142[] = {{&S97205[44], 25}, {NULL}};
const Segment SGI229143[] = {{&S97205[30], 6}, {NULL}};
const Segment SGI229144[] = {{&S97205[118], 3}, {NULL}};
const Segment SGI229145[] = {{&S97205[83], 6}, {NULL}};
const Segment SGI229146[] = {{&S97205[138], 8}, {NULL}};
const Segment SGI229147[] = {{&S97205[147], 13}, {NULL}};
const Segment SGI229148[] = {{&S79454[34], 11}, {NULL}};
const Segment SGI229149[] = {{&S79454[27], 6}, {NULL}};
const Segment SGI229150[] = {{&S79454[98], 9}, {NULL}};
const Segment SGI229151[] = {{&S79454[71], 6}, {NULL}};
const Segment SGI229152[] = {{&S79458[34], 10}, {NULL}};
const Segment SGI229153[] = {{&S79458[27], 6}, {NULL}};
const Segment SGI229154[] = {{&S79458[96], 9}, {NULL}};
const Segment SGI229155[] = {{&S79458[69], 6}, {NULL}};
const Segment SGI229156[] = {{&S97216[44], 10}, {NULL}};
const Segment SGI229157[] = {{&S97216[44], 22}, {NULL}};
const Segment SGI229158[] = {{&S97216[30], 6}, {NULL}};
const Segment SGI229159[] = {{&S97216[115], 3}, {NULL}};
const Segment SGI229160[] = {{&S97216[80], 6}, {NULL}};
const Segment SGI229161[] = {{&S97216[135], 8}, {NULL}};
const Segment SGI229162[] = {{&S97216[144], 13}, {NULL}};
const Segment SGI229163[] = {{&S79464[34], 12}, {NULL}};
const Segment SGI229164[] = {{&S79464[27], 6}, {NULL}};
const Segment SGI229165[] = {{&S79464[98], 9}, {NULL}};
const Segment SGI229166[] = {{&S79464[71], 6}, {NULL}};
const Segment SGI229167[] = {{&S97225[34], 30}, {NULL}};
const Segment SGI229168[] = {{&S97225[27], 6}, {NULL}};
const Segment SGI229169[] = {{&S97225[117], 9}, {NULL}};
const Segment SGI229170[] = {{&S97225[90], 6}, {NULL}};
const Segment SGI229171[] = {{&S88369[34], 17}, {NULL}};
const Segment SGI229172[] = {{&S88369[27], 6}, {NULL}};
const Segment SGI229173[] = {{&S88369[104], 9}, {NULL}};
const Segment SGI229174[] = {{&S88369[77], 6}, {NULL}};
const Segment SGI229175[] = {{&S97227[34], 7}, {NULL}};
const Segment SGI229176[] = {{&S97227[27], 6}, {NULL}};
const Segment SGI229177[] = {{&S97227[99], 9}, {NULL}};
const Segment SGI229178[] = {{&S97227[72], 6}, {NULL}};
const Segment SGI229179[] = {{&S87212[34], 7}, {NULL}};
const Segment SGI229180[] = {{&S87212[27], 6}, {NULL}};
const Segment SGI229181[] = {{&S87212[94], 9}, {NULL}};
const Segment SGI229182[] = {{&S87212[67], 6}, {NULL}};
const Segment SGI229183[] = {{&S97229[34], 13}, {NULL}};
const Segment SGI229184[] = {{&S97229[27], 6}, {NULL}};
const Segment SGI229185[] = {{&S97229[100], 9}, {NULL}};
const Segment SGI229186[] = {{&S97229[73], 6}, {NULL}};
const Segment SGI229187[] = {{&S97231[44], 7}, {NULL}};
const Segment SGI229188[] = {{&S97231[44], 23}, {NULL}};
const Segment SGI229189[] = {{&S97231[30], 6}, {NULL}};
const Segment SGI229190[] = {{&S97231[81], 6}, {NULL}};
const Segment SGI229191[] = {{&S97231[108], 13}, {NULL}};
const Segment SGI229192[] = {{&S97232[34], 5}, {NULL}};
const Segment SGI229193[] = {{&S97232[27], 6}, {NULL}};
const Segment SGI229194[] = {{&S97232[92], 9}, {NULL}};
const Segment SGI229195[] = {{&S97232[65], 6}, {NULL}};
const Segment SGI229196[] = {{&S97237[34], 7}, {NULL}};
const Segment SGI229197[] = {{&S97237[27], 6}, {NULL}};
const Segment SGI229198[] = {{&S97237[106], 9}, {NULL}};
const Segment SGI229199[] = {{&S97237[79], 6}, {NULL}};
const Segment SGI229200[] = {{&S87230[45], 12}, {NULL}};
const Segment SGI229201[] = {{&S87230[30], 6}, {NULL}};
const Segment SGI229202[] = {{&S87230[127], 3}, {NULL}};
const Segment SGI229203[] = {{&S87230[93], 5}, {NULL}};
const Segment SGI229204[] = {{&S87230[151], 13}, {NULL}};
const Segment SGI229205[] = {{&S87234[34], 8}, {NULL}};
const Segment SGI229206[] = {{&S87234[27], 6}, {NULL}};
const Segment SGI229207[] = {{&S87234[101], 9}, {NULL}};
const Segment SGI229208[] = {{&S87234[74], 6}, {NULL}};
const Segment SGI229209[] = {{&S88400[34], 10}, {NULL}};
const Segment SGI229210[] = {{&S88400[27], 6}, {NULL}};
const Segment SGI229211[] = {{&S79506[44], 10}, {NULL}};
const Segment SGI229212[] = {{&S79506[44], 21}, {NULL}};
const Segment SGI229213[] = {{&S88400[97], 9}, {NULL}};
const Segment SGI229214[] = {{&S79506[30], 6}, {NULL}};
const Segment SGI229215[] = {{&S88400[70], 6}, {NULL}};
const Segment SGI229216[] = {{&S79506[137], 3}, {NULL}};
const Segment SGI229217[] = {{&S79506[102], 6}, {NULL}};
const Segment SGI229218[] = {{&S79506[74], 9}, {NULL}};
const Segment SGI229219[] = {{&S88401[44], 7}, {NULL}};
const Segment SGI229220[] = {{&S88401[44], 19}, {NULL}};
const Segment SGI229221[] = {{&S88401[30], 6}, {NULL}};
const Segment SGI229222[] = {{&S79507[44], 10}, {NULL}};
const Segment SGI229223[] = {{&S88401[112], 3}, {NULL}};
const Segment SGI229224[] = {{&S79507[44], 22}, {NULL}};
const Segment SGI229225[] = {{&S79507[30], 6}, {NULL}};
const Segment SGI229226[] = {{&S88401[77], 6}, {NULL}};
const Segment SGI229227[] = {{&S79507[138], 3}, {NULL}};
const Segment SGI229228[] = {{&S79507[103], 6}, {NULL}};
const Segment SGI229229[] = {{&S79507[75], 9}, {NULL}};
const Segment SGI229230[] = {{&S79508[44], 10}, {NULL}};
const Segment SGI229231[] = {{&S79508[44], 22}, {NULL}};
const Segment SGI229232[] = {{&S79508[30], 6}, {NULL}};
const Segment SGI229233[] = {{&S79508[138], 3}, {NULL}};
const Segment SGI229234[] = {{&S79508[103], 6}, {NULL}};
const Segment SGI229235[] = {{&S79508[75], 9}, {NULL}};
const Segment SGI229236[] = {{&S87243[34], 9}, {NULL}};
const Segment SGI229237[] = {{&S87243[27], 6}, {NULL}};
const Segment SGI229238[] = {{&S79509[44], 10}, {NULL}};
const Segment SGI229239[] = {{&S79509[44], 22}, {NULL}};
const Segment SGI229240[] = {{&S87243[96], 9}, {NULL}};
const Segment SGI229241[] = {{&S79509[30], 6}, {NULL}};
const Segment SGI229242[] = {{&S87243[69], 6}, {NULL}};
const Segment SGI229243[] = {{&S79509[138], 3}, {NULL}};
const Segment SGI229244[] = {{&S79509[103], 6}, {NULL}};
const Segment SGI229245[] = {{&S79509[75], 9}, {NULL}};
const Segment SGI229246[] = {{&S87254[34], 10}, {NULL}};
const Segment SGI229247[] = {{&S87254[27], 6}, {NULL}};
const Segment SGI229248[] = {{&S93862[34], 4}, {NULL}};
const Segment SGI229249[] = {{&S87254[97], 9}, {NULL}};
const Segment SGI229250[] = {{&S87254[70], 6}, {NULL}};
const Segment SGI229251[] = {{&S93862[27], 6}, {NULL}};
const Segment SGI229252[] = {{&S93862[91], 9}, {NULL}};
const Segment SGI229253[] = {{&S93862[64], 6}, {NULL}};
const Segment SGI229254[] = {{&S87255[44], 10}, {NULL}};
const Segment SGI229255[] = {{&S87255[44], 22}, {NULL}};
const Segment SGI229256[] = {{&S87255[30], 6}, {NULL}};
const Segment SGI229257[] = {{&S87255[115], 3}, {NULL}};
const Segment SGI229258[] = {{&S87255[80], 6}, {NULL}};
const Segment SGI229259[] = {{&S87255[143], 3}, {NULL}};
const Segment SGI229260[] = {{&S93864[34], 6}, {NULL}};
const Segment SGI229261[] = {{&S93864[27], 6}, {NULL}};
const Segment SGI229262[] = {{&S93864[93], 9}, {NULL}};
const Segment SGI229263[] = {{&S93864[66], 6}, {NULL}};
const Segment SGI229264[] = {{&S87258[34], 11}, {NULL}};
const Segment SGI229265[] = {{&S87258[27], 6}, {NULL}};
const Segment SGI229266[] = {{&S87258[98], 9}, {NULL}};
const Segment SGI229267[] = {{&S87258[71], 6}, {NULL}};
const Segment SGI229268[] = {{&S93870[34], 13}, {NULL}};
const Segment SGI229269[] = {{&S93870[27], 6}, {NULL}};
const Segment SGI229270[] = {{&S93870[100], 9}, {NULL}};
const Segment SGI229271[] = {{&S93870[73], 6}, {NULL}};
const Segment SGI229272[] = {{&S87263[34], 8}, {NULL}};
const Segment SGI229273[] = {{&S87263[27], 6}, {NULL}};
const Segment SGI229274[] = {{&S87263[95], 9}, {NULL}};
const Segment SGI229275[] = {{&S87263[68], 6}, {NULL}};
const Segment SGI229276[] = {{&S87264[34], 18}, {NULL}};
const Segment SGI229277[] = {{&S87264[27], 6}, {NULL}};
const Segment SGI229278[] = {{&S87264[105], 9}, {NULL}};
const Segment SGI229279[] = {{&S87264[78], 6}, {NULL}};
const Segment SGI229280[] = {{&S87264[146], 8}, {NULL}};
const Segment SGI229281[] = {{&S93876[44], 13}, {NULL}};
const Segment SGI229282[] = {{&S93876[44], 25}, {NULL}};
const Segment SGI229283[] = {{&S93876[30], 6}, {NULL}};
const Segment SGI229284[] = {{&S93876[118], 3}, {NULL}};
const Segment SGI229285[] = {{&S93876[83], 6}, {NULL}};
const Segment SGI229286[] = {{&S87267[34], 10}, {NULL}};
const Segment SGI229287[] = {{&S87267[27], 6}, {NULL}};
const Segment SGI229288[] = {{&S87267[97], 9}, {NULL}};
const Segment SGI229289[] = {{&S87267[70], 6}, {NULL}};
const Segment SGI229290[] = {{&S87267[138], 8}, {NULL}};
const Segment SGI229291[] = {{&S79549[34], 10}, {NULL}};
const Segment SGI229292[] = {{&S79549[27], 6}, {NULL}};
const Segment SGI229293[] = {{&S79549[96], 9}, {NULL}};
const Segment SGI229294[] = {{&S79549[69], 6}, {NULL}};
const Segment SGI229295[] = {{&S87282[34], 7}, {NULL}};
const Segment SGI229296[] = {{&S87282[27], 6}, {NULL}};
const Segment SGI229297[] = {{&S87282[94], 9}, {NULL}};
const Segment SGI229298[] = {{&S87282[67], 6}, {NULL}};
const Segment SGI229299[] = {{&S97312[34], 9}, {NULL}};
const Segment SGI229300[] = {{&S97312[27], 6}, {NULL}};
const Segment SGI229301[] = {{&S97312[96], 9}, {NULL}};
const Segment SGI229302[] = {{&S97312[69], 6}, {NULL}};
const Segment SGI229303[] = {{&S93901[34], 9}, {NULL}};
const Segment SGI229304[] = {{&S93901[27], 6}, {NULL}};
const Segment SGI229305[] = {{&S93901[96], 9}, {NULL}};
const Segment SGI229306[] = {{&S93901[69], 6}, {NULL}};
const Segment SGI229307[] = {{&S93904[34], 11}, {NULL}};
const Segment SGI229308[] = {{&S93904[27], 6}, {NULL}};
const Segment SGI229309[] = {{&S93904[98], 9}, {NULL}};
const Segment SGI229310[] = {{&S93904[71], 6}, {NULL}};
const Segment SGI229311[] = {{&S97330[34], 10}, {NULL}};
const Segment SGI229312[] = {{&S97330[27], 6}, {NULL}};
const Segment SGI229313[] = {{&S93911[44], 12}, {NULL}};
const Segment SGI229314[] = {{&S93911[44], 24}, {NULL}};
const Segment SGI229315[] = {{&S93911[30], 6}, {NULL}};
const Segment SGI229316[] = {{&S97330[97], 9}, {NULL}};
const Segment SGI229317[] = {{&S97330[70], 6}, {NULL}};
const Segment SGI229318[] = {{&S93911[117], 3}, {NULL}};
const Segment SGI229319[] = {{&S93911[82], 6}, {NULL}};
const Segment SGI229320[] = {{&S97338[34], 12}, {NULL}};
const Segment SGI229321[] = {{&S97338[27], 6}, {NULL}};
const Segment SGI229322[] = {{&S97338[99], 9}, {NULL}};
const Segment SGI229323[] = {{&S97338[72], 6}, {NULL}};
const Segment SGI229324[] = {{&S93919[44], 17}, {NULL}};
const Segment SGI229325[] = {{&S93919[44], 38}, {NULL}};
const Segment SGI229326[] = {{&S93919[30], 6}, {NULL}};
const Segment SGI229327[] = {{&S93919[130], 3}, {NULL}};
const Segment SGI229328[] = {{&S93919[96], 5}, {NULL}};
const Segment SGI229329[] = {{&S93922[44], 17}, {NULL}};
const Segment SGI229330[] = {{&S93922[44], 29}, {NULL}};
const Segment SGI229331[] = {{&S93922[30], 6}, {NULL}};
const Segment SGI229332[] = {{&S93922[122], 3}, {NULL}};
const Segment SGI229333[] = {{&S93922[87], 6}, {NULL}};
const Segment SGI229334[] = {{&S93925[44], 25}, {NULL}};
const Segment SGI229335[] = {{&S93925[44], 37}, {NULL}};
const Segment SGI229336[] = {{&S93925[30], 6}, {NULL}};
const Segment SGI229337[] = {{&S93925[31], 3}, {NULL}};
const Segment SGI229338[] = {{&S97347[44], 25}, {NULL}};
const Segment SGI229339[] = {{&S93925[95], 6}, {NULL}};
const Segment SGI229340[] = {{&S97347[44], 37}, {NULL}};
const Segment SGI229341[] = {{&S97347[30], 6}, {NULL}};
const Segment SGI229342[] = {{&S97347[31], 3}, {NULL}};
const Segment SGI229343[] = {{&S97347[95], 6}, {NULL}};
const Segment SGI229344[] = {{&S87330[34], 10}, {NULL}};
const Segment SGI229345[] = {{&S87330[27], 6}, {NULL}};
const Segment SGI229346[] = {{&S87330[97], 9}, {NULL}};
const Segment SGI229347[] = {{&S87330[70], 6}, {NULL}};
const Segment SGI229348[] = {{&S97353[34], 13}, {NULL}};
const Segment SGI229349[] = {{&S97353[27], 6}, {&S97353[42], 17}, {NULL}};
const Segment SGI229350[] = {{&S97353[100], 9}, {NULL}};
const Segment SGI229351[] = {{&S97353[73], 6}, {NULL}};
const Segment SGI229352[] = {{&S93935[34], 7}, {NULL}};
const Segment SGI229353[] = {{&S97355[34], 7}, {NULL}};
const Segment SGI229354[] = {{&S93935[27], 6}, {NULL}};
const Segment SGI229355[] = {{&S97355[27], 6}, {NULL}};
const Segment SGI229356[] = {{&S87337[34], 10}, {NULL}};
const Segment SGI229357[] = {{&S87337[27], 6}, {NULL}};
const Segment SGI229358[] = {{&S97355[90], 9}, {NULL}};
const Segment SGI229359[] = {{&S93935[94], 9}, {NULL}};
const Segment SGI229360[] = {{&S97355[63], 6}, {NULL}};
const Segment SGI229361[] = {{&S93935[67], 6}, {NULL}};
const Segment SGI229362[] = {{&S87337[97], 9}, {NULL}};
const Segment SGI229363[] = {{&S87337[70], 6}, {NULL}};
const Segment SGI229364[] = {{&S97357[45], 10}, {NULL}};
const Segment SGI229365[] = {{&S97357[30], 6}, {NULL}};
const Segment SGI229366[] = {{&S97357[31], 3}, {NULL}};
const Segment SGI229367[] = {{&S97357[81], 5}, {NULL}};
const Segment SGI229368[] = {{&S97357[139], 13}, {NULL}};
const Segment SGI229369[] = {{&S97358[59], 7}, {NULL}};
const Segment SGI229370[] = {{&S97358[59], 19}, {NULL}};
const Segment SGI229371[] = {{&S97358[45], 6}, {NULL}};
const Segment SGI229372[] = {{&S97358[46], 3}, {NULL}};
const Segment SGI229373[] = {{&S97358[92], 6}, {NULL}};
const Segment SGI229374[] = {{&S97358[0], 14}, {NULL}};
const Segment SGI229375[] = {{&S97360[31], 13}, {NULL}};
const Segment SGI229376[] = {{&S97360[59], 3}, {NULL}};
const Segment SGI229377[] = {{&S87347[34], 10}, {NULL}};
const Segment SGI229378[] = {{&S87347[27], 6}, {NULL}};
const Segment SGI229379[] = {{&S87347[97], 9}, {NULL}};
const Segment SGI229380[] = {{&S87347[70], 6}, {NULL}};
const Segment SGI229381[] = {{&S88492[34], 4}, {NULL}};
const Segment SGI229382[] = {{&S88492[27], 6}, {NULL}};
const Segment SGI229383[] = {{&S88492[91], 9}, {NULL}};
const Segment SGI229384[] = {{&S88492[64], 6}, {NULL}};
const Segment SGI229385[] = {{&S97372[34], 11}, {NULL}};
const Segment SGI229386[] = {{&S97372[27], 6}, {NULL}};
const Segment SGI229387[] = {{&S97372[98], 9}, {NULL}};
const Segment SGI229388[] = {{&S97372[71], 6}, {NULL}};
const Segment SGI229389[] = {{&S97373[45], 10}, {NULL}};
const Segment SGI229390[] = {{&S97373[30], 6}, {NULL}};
const Segment SGI229391[] = {{&S97373[31], 3}, {NULL}};
const Segment SGI229392[] = {{&S97373[81], 5}, {NULL}};
const Segment SGI229393[] = {{&S97373[139], 13}, {NULL}};
const Segment SGI229394[] = {{&S97374[45], 11}, {NULL}};
const Segment SGI229395[] = {{&S97374[30], 6}, {NULL}};
const Segment SGI229396[] = {{&S97374[31], 3}, {NULL}};
const Segment SGI229397[] = {{&S97374[82], 5}, {NULL}};
const Segment SGI229398[] = {{&S97374[140], 13}, {NULL}};
const Segment SGI229399[] = {{&S97377[34], 9}, {NULL}};
const Segment SGI229400[] = {{&S97377[27], 6}, {NULL}};
const Segment SGI229401[] = {{&S87367[44], 10}, {NULL}};
const Segment SGI229402[] = {{&S87367[44], 22}, {NULL}};
const Segment SGI229403[] = {{&S87367[30], 6}, {NULL}};
const Segment SGI229404[] = {{&S97377[102], 9}, {NULL}};
const Segment SGI229405[] = {{&S97377[75], 6}, {NULL}};
const Segment SGI229406[] = {{&S87367[115], 3}, {NULL}};
const Segment SGI229407[] = {{&S87367[80], 6}, {NULL}};
const Segment SGI229408[] = {{&S97379[34], 8}, {NULL}};
const Segment SGI229409[] = {{&S97379[27], 6}, {NULL}};
const Segment SGI229410[] = {{&S97379[100], 9}, {NULL}};
const Segment SGI229411[] = {{&S97379[73], 6}, {NULL}};
const Segment SGI229412[] = {{&S87371[45], 10}, {NULL}};
const Segment SGI229413[] = {{&S87371[30], 6}, {NULL}};
const Segment SGI229414[] = {{&S87371[115], 3}, {NULL}};
const Segment SGI229415[] = {{&S87371[81], 5}, {NULL}};
const Segment SGI229416[] = {{&S87371[139], 13}, {NULL}};
const Segment SGI229417[] = {{&S97382[34], 7}, {NULL}};
const Segment SGI229418[] = {{&S97382[27], 6}, {NULL}};
const Segment SGI229419[] = {{&S97382[90], 9}, {NULL}};
const Segment SGI229420[] = {{&S97382[63], 6}, {NULL}};
const Segment SGI229421[] = {{&S97383[34], 7}, {NULL}};
const Segment SGI229422[] = {{&S88515[44], 17}, {NULL}};
const Segment SGI229423[] = {{&S97383[27], 6}, {NULL}};
const Segment SGI229424[] = {{&S88515[44], 29}, {NULL}};
const Segment SGI229425[] = {{&S88515[30], 6}, {NULL}};
const Segment SGI229426[] = {{&S97383[94], 9}, {NULL}};
const Segment SGI229427[] = {{&S88515[122], 3}, {NULL}};
const Segment SGI229428[] = {{&S97383[67], 6}, {NULL}};
const Segment SGI229429[] = {{&S88515[87], 6}, {NULL}};
const Segment SGI229430[] = {{&S97385[34], 15}, {NULL}};
const Segment SGI229431[] = {{&S97385[27], 6}, {NULL}};
const Segment SGI229432[] = {{&S97385[102], 9}, {NULL}};
const Segment SGI229433[] = {{&S97385[75], 6}, {NULL}};
const Segment SGI229434[] = {{&S87378[34], 13}, {NULL}};
const Segment SGI229435[] = {{&S87378[27], 6}, {NULL}};
const Segment SGI229436[] = {{&S87378[100], 9}, {NULL}};
const Segment SGI229437[] = {{&S87378[73], 6}, {NULL}};
const Segment SGI229438[] = {{&S87378[141], 8}, {NULL}};
const Segment SGI229439[] = {{&S97387[44], 11}, {NULL}};
const Segment SGI229440[] = {{&S97387[44], 23}, {NULL}};
const Segment SGI229441[] = {{&S97387[30], 6}, {NULL}};
const Segment SGI229442[] = {{&S97387[81], 6}, {NULL}};
const Segment SGI229443[] = {{&S97387[108], 13}, {NULL}};
const Segment SGI229444[] = {{&S87381[34], 9}, {NULL}};
const Segment SGI229445[] = {{&S87381[27], 6}, {NULL}};
const Segment SGI229446[] = {{&S87381[96], 9}, {NULL}};
const Segment SGI229447[] = {{&S87381[69], 6}, {NULL}};
const Segment SGI229448[] = {{&S97390[34], 7}, {NULL}};
const Segment SGI229449[] = {{&S97390[27], 6}, {NULL}};
const Segment SGI229450[] = {{&S97390[94], 9}, {NULL}};
const Segment SGI229451[] = {{&S97390[67], 6}, {NULL}};
const Segment SGI229452[] = {{&S97391[34], 13}, {NULL}};
const Segment SGI229453[] = {{&S97391[27], 6}, {NULL}};
const Segment SGI229454[] = {{&S97391[100], 9}, {NULL}};
const Segment SGI229455[] = {{&S97391[73], 6}, {NULL}};
const Segment SGI229456[] = {{&S97393[34], 8}, {NULL}};
const Segment SGI229457[] = {{&S97393[27], 6}, {NULL}};
const Segment SGI229458[] = {{&S97393[95], 9}, {NULL}};
const Segment SGI229459[] = {{&S97393[68], 6}, {NULL}};
const Segment SGI229460[] = {{&S88532[34], 8}, {NULL}};
const Segment SGI229461[] = {{&S88532[27], 6}, {NULL}};
const Segment SGI229462[] = {{&S88532[95], 9}, {NULL}};
const Segment SGI229463[] = {{&S88532[68], 6}, {NULL}};
const Segment SGI229464[] = {{&S88537[45], 12}, {NULL}};
const Segment SGI229465[] = {{&S88537[30], 6}, {NULL}};
const Segment SGI229466[] = {{&S97400[34], 11}, {NULL}};
const Segment SGI229467[] = {{&S97400[27], 6}, {NULL}};
const Segment SGI229468[] = {{&S88537[117], 3}, {NULL}};
const Segment SGI229469[] = {{&S88537[83], 5}, {NULL}};
const Segment SGI229470[] = {{&S97400[98], 9}, {NULL}};
const Segment SGI229471[] = {{&S97400[71], 6}, {NULL}};
const Segment SGI229472[] = {{&S88537[151], 3}, {NULL}};
const Segment SGI229473[] = {{&S97402[34], 11}, {NULL}};
const Segment SGI229474[] = {{&S97402[27], 6}, {NULL}};
const Segment SGI229475[] = {{&S97402[98], 9}, {NULL}};
const Segment SGI229476[] = {{&S97402[71], 6}, {NULL}};
const Segment SGI229477[] = {{&S97404[44], 8}, {NULL}};
const Segment SGI229478[] = {{&S97404[44], 24}, {NULL}};
const Segment SGI229479[] = {{&S97404[30], 6}, {NULL}};
const Segment SGI229480[] = {{&S97404[31], 3}, {NULL}};
const Segment SGI229481[] = {{&S97404[82], 5}, {NULL}};
const Segment SGI229482[] = {{&S97404[119], 14}, {NULL}};
const Segment SGI229483[] = {{&S97405[44], 13}, {NULL}};
const Segment SGI229484[] = {{&S97405[44], 25}, {NULL}};
const Segment SGI229485[] = {{&S97405[30], 6}, {NULL}};
const Segment SGI229486[] = {{&S97405[83], 6}, {NULL}};
const Segment SGI229487[] = {{&S97405[110], 13}, {NULL}};
const Segment SGI229488[] = {{&S97406[34], 21}, {NULL}};
const Segment SGI229489[] = {{&S97406[27], 6}, {NULL}};
const Segment SGI229490[] = {{&S97406[108], 9}, {NULL}};
const Segment SGI229491[] = {{&S97406[81], 6}, {NULL}};
const Segment SGI229492[] = {{&S97407[34], 6}, {NULL}};
const Segment SGI229493[] = {{&S97407[27], 6}, {NULL}};
const Segment SGI229494[] = {{&S97407[93], 9}, {NULL}};
const Segment SGI229495[] = {{&S97407[66], 6}, {NULL}};
const Segment SGI229496[] = {{&S97414[34], 10}, {NULL}};
const Segment SGI229497[] = {{&S97414[27], 6}, {NULL}};
const Segment SGI229498[] = {{&S97414[97], 9}, {NULL}};
const Segment SGI229499[] = {{&S97414[70], 6}, {NULL}};
const Segment SGI229500[] = {{&S97415[45], 7}, {NULL}};
const Segment SGI229501[] = {{&S97415[30], 6}, {NULL}};
const Segment SGI229502[] = {{&S97415[31], 3}, {NULL}};
const Segment SGI229503[] = {{&S97415[83], 5}, {NULL}};
const Segment SGI229504[] = {{&S97415[141], 13}, {NULL}};
const Segment SGI229505[] = {{&S97417[44], 7}, {NULL}};
const Segment SGI229506[] = {{&S97417[44], 25}, {NULL}};
const Segment SGI229507[] = {{&S97417[30], 6}, {NULL}};
const Segment SGI229508[] = {{&S97417[83], 6}, {NULL}};
const Segment SGI229509[] = {{&S97417[110], 13}, {NULL}};
const Segment SGI229510[] = {{&S97420[34], 5}, {NULL}};
const Segment SGI229511[] = {{&S97420[27], 6}, {NULL}};
const Segment SGI229512[] = {{&S97420[92], 9}, {NULL}};
const Segment SGI229513[] = {{&S97420[65], 6}, {NULL}};
const Segment SGI229514[] = {{&S97421[45], 13}, {NULL}};
const Segment SGI229515[] = {{&S97421[30], 6}, {NULL}};
const Segment SGI229516[] = {{&S97421[31], 3}, {NULL}};
const Segment SGI229517[] = {{&S97421[84], 5}, {NULL}};
const Segment SGI229518[] = {{&S97421[142], 13}, {NULL}};
const Segment SGI229519[] = {{&S97423[34], 11}, {NULL}};
const Segment SGI229520[] = {{&S97423[27], 6}, {NULL}};
const Segment SGI229521[] = {{&S97423[93], 9}, {NULL}};
const Segment SGI229522[] = {{&S97423[66], 6}, {NULL}};
const Segment SGI229523[] = {{&S97424[34], 23}, {NULL}};
const Segment SGI229524[] = {{&S97424[27], 6}, {NULL}};
const Segment SGI229525[] = {{&S97424[110], 9}, {NULL}};
const Segment SGI229526[] = {{&S97424[83], 6}, {NULL}};
const Segment SGI229527[] = {{&S97426[34], 18}, {NULL}};
const Segment SGI229528[] = {{&S97426[27], 6}, {NULL}};
const Segment SGI229529[] = {{&S97426[112], 9}, {NULL}};
const Segment SGI229530[] = {{&S97426[85], 6}, {NULL}};
const Segment SGI229531[] = {{&S97427[45], 5}, {NULL}};
const Segment SGI229532[] = {{&S97427[30], 6}, {NULL}};
const Segment SGI229533[] = {{&S97427[31], 3}, {NULL}};
const Segment SGI229534[] = {{&S97427[76], 5}, {NULL}};
const Segment SGI229535[] = {{&S97427[134], 13}, {NULL}};
const Segment SGI229536[] = {{&S97430[27], 6}, {NULL}};
const Segment SGI229537[] = {{&S97430[101], 9}, {NULL}};
const Segment SGI229538[] = {{&S97430[74], 6}, {NULL}};
const Segment SGI229539[] = {{&S87449[44], 10}, {NULL}};
const Segment SGI229540[] = {{&S87449[44], 22}, {NULL}};
const Segment SGI229541[] = {{&S87449[30], 6}, {NULL}};
const Segment SGI229542[] = {{&S87449[114], 3}, {NULL}};
const Segment SGI229543[] = {{&S87449[80], 5}, {NULL}};
const Segment SGI229544[] = {{&S87449[125], 14}, {NULL}};
const Segment SGI229545[] = {{&S97438[44], 14}, {NULL}};
const Segment SGI229546[] = {{&S97438[44], 31}, {NULL}};
const Segment SGI229547[] = {{&S97438[30], 6}, {NULL}};
const Segment SGI229548[] = {{&S97438[89], 6}, {NULL}};
const Segment SGI229549[] = {{&S97438[116], 13}, {NULL}};
const Segment SGI229550[] = {{&S94065[34], 12}, {NULL}};
const Segment SGI229551[] = {{&S94065[27], 6}, {NULL}};
const Segment SGI229552[] = {{&S94065[99], 9}, {NULL}};
const Segment SGI229553[] = {{&S94065[72], 6}, {NULL}};
const Segment SGI229554[] = {{&S79608[44], 7}, {NULL}};
const Segment SGI229555[] = {{&S79608[44], 18}, {NULL}};
const Segment SGI229556[] = {{&S79608[30], 6}, {NULL}};
const Segment SGI229557[] = {{&S79608[111], 3}, {NULL}};
const Segment SGI229558[] = {{&S79608[76], 6}, {NULL}};
const Segment SGI229559[] = {{&S87462[34], 7}, {NULL}};
const Segment SGI229560[] = {{&S87462[27], 6}, {NULL}};
const Segment SGI229561[] = {{&S87462[94], 9}, {NULL}};
const Segment SGI229562[] = {{&S87462[67], 6}, {NULL}};
const Segment SGI229563[] = {{&S79609[44], 6}, {NULL}};
const Segment SGI229564[] = {{&S79609[44], 17}, {NULL}};
const Segment SGI229565[] = {{&S79609[30], 6}, {NULL}};
const Segment SGI229566[] = {{&S79609[110], 3}, {NULL}};
const Segment SGI229567[] = {{&S79609[75], 6}, {NULL}};
const Segment SGI229568[] = {{&S79611[44], 14}, {NULL}};
const Segment SGI229569[] = {{&S79611[44], 25}, {NULL}};
const Segment SGI229570[] = {{&S79611[30], 6}, {NULL}};
const Segment SGI229571[] = {{&S79611[118], 3}, {NULL}};
const Segment SGI229572[] = {{&S79611[83], 6}, {NULL}};
const Segment SGI229573[] = {{&S97450[34], 16}, {NULL}};
const Segment SGI229574[] = {{&S97450[27], 6}, {NULL}};
const Segment SGI229575[] = {{&S97450[103], 9}, {NULL}};
const Segment SGI229576[] = {{&S97450[76], 6}, {NULL}};
const Segment SGI229577[] = {{&S97450[144], 8}, {NULL}};
const Segment SGI229578[] = {{&S79612[44], 14}, {NULL}};
const Segment SGI229579[] = {{&S79612[44], 25}, {NULL}};
const Segment SGI229580[] = {{&S79612[30], 6}, {NULL}};
const Segment SGI229581[] = {{&S79612[118], 3}, {NULL}};
const Segment SGI229582[] = {{&S79612[83], 6}, {NULL}};
const Segment SGI229583[] = {{&S97451[34], 12}, {NULL}};
const Segment SGI229584[] = {{&S97451[27], 6}, {NULL}};
const Segment SGI229585[] = {{&S97451[99], 9}, {NULL}};
const Segment SGI229586[] = {{&S97451[72], 6}, {NULL}};
const Segment SGI229587[] = {{&S94077[34], 13}, {NULL}};
const Segment SGI229588[] = {{&S94077[27], 6}, {NULL}};
const Segment SGI229589[] = {{&S94077[114], 9}, {NULL}};
const Segment SGI229590[] = {{&S94077[87], 6}, {NULL}};
const Segment SGI229591[] = {{&S97452[34], 8}, {NULL}};
const Segment SGI229592[] = {{&S97452[27], 6}, {NULL}};
const Segment SGI229593[] = {{&S97452[95], 9}, {NULL}};
const Segment SGI229594[] = {{&S97452[68], 6}, {NULL}};
const Segment SGI229595[] = {{&S97456[34], 15}, {NULL}};
const Segment SGI229596[] = {{&S97456[27], 6}, {NULL}};
const Segment SGI229597[] = {{&S87476[34], 7}, {NULL}};
const Segment SGI229598[] = {{&S87476[27], 6}, {NULL}};
const Segment SGI229599[] = {{&S97456[102], 9}, {NULL}};
const Segment SGI229600[] = {{&S97456[75], 6}, {NULL}};
const Segment SGI229601[] = {{&S87476[104], 9}, {NULL}};
const Segment SGI229602[] = {{&S87476[77], 6}, {NULL}};
const Segment SGI229603[] = {{&S97457[34], 10}, {NULL}};
const Segment SGI229604[] = {{&S97457[27], 6}, {NULL}};
const Segment SGI229605[] = {{&S97457[97], 9}, {NULL}};
const Segment SGI229606[] = {{&S97457[70], 6}, {NULL}};
const Segment SGI229607[] = {{&S97458[34], 12}, {NULL}};
const Segment SGI229608[] = {{&S97458[27], 6}, {NULL}};
const Segment SGI229609[] = {{&S97458[99], 9}, {NULL}};
const Segment SGI229610[] = {{&S97458[72], 6}, {NULL}};
const Segment SGI229611[] = {{&S94089[34], 15}, {NULL}};
const Segment SGI229612[] = {{&S97459[34], 8}, {NULL}};
const Segment SGI229613[] = {{&S94089[27], 6}, {NULL}};
const Segment SGI229614[] = {{&S97459[27], 6}, {NULL}};
const Segment SGI229615[] = {{&S94089[102], 9}, {NULL}};
const Segment SGI229616[] = {{&S97459[95], 9}, {NULL}};
const Segment SGI229617[] = {{&S94089[75], 6}, {NULL}};
const Segment SGI229618[] = {{&S97459[68], 6}, {NULL}};
const Segment SGI229619[] = {{&S97460[34], 7}, {NULL}};
const Segment SGI229620[] = {{&S97460[27], 6}, {NULL}};
const Segment SGI229621[] = {{&S97460[94], 9}, {NULL}};
const Segment SGI229622[] = {{&S97460[67], 6}, {NULL}};
const Segment SGI229623[] = {{&S94092[44], 12}, {NULL}};
const Segment SGI229624[] = {{&S94092[44], 24}, {NULL}};
const Segment SGI229625[] = {{&S94092[30], 6}, {NULL}};
const Segment SGI229626[] = {{&S97461[34], 16}, {NULL}};
const Segment SGI229627[] = {{&S97461[27], 6}, {NULL}};
const Segment SGI229628[] = {{&S94092[117], 3}, {NULL}};
const Segment SGI229629[] = {{&S87486[44], 10}, {NULL}};
const Segment SGI229630[] = {{&S87486[44], 22}, {NULL}};
const Segment SGI229631[] = {{&S94092[82], 6}, {NULL}};
const Segment SGI229632[] = {{&S87486[30], 6}, {NULL}};
const Segment SGI229633[] = {{&S97461[110], 9}, {NULL}};
const Segment SGI229634[] = {{&S97461[83], 6}, {NULL}};
const Segment SGI229635[] = {{&S87486[80], 6}, {NULL}};
const Segment SGI229636[] = {{&S97461[151], 8}, {NULL}};
const Segment SGI229637[] = {{&S87486[107], 13}, {NULL}};
const Segment SGI229638[] = {{&S97462[34], 11}, {NULL}};
const Segment SGI229639[] = {{&S97462[27], 6}, {NULL}};
const Segment SGI229640[] = {{&S97462[98], 9}, {NULL}};
const Segment SGI229641[] = {{&S97462[71], 6}, {NULL}};
const Segment SGI229642[] = {{&S97462[139], 8}, {NULL}};
const Segment SGI229643[] = {{&S97466[34], 9}, {NULL}};
const Segment SGI229644[] = {{&S97466[27], 6}, {NULL}};
const Segment SGI229645[] = {{&S97466[96], 9}, {NULL}};
const Segment SGI229646[] = {{&S97466[69], 6}, {NULL}};
const Segment SGI229647[] = {{&S97466[137], 8}, {NULL}};
const Segment SGI229648[] = {{&S87495[44], 25}, {NULL}};
const Segment SGI229649[] = {{&S87495[44], 37}, {NULL}};
const Segment SGI229650[] = {{&S87495[30], 6}, {NULL}};
const Segment SGI229651[] = {{&S87495[130], 3}, {NULL}};
const Segment SGI229652[] = {{&S87495[95], 6}, {NULL}};
const Segment SGI229653[] = {{&S97470[45], 9}, {NULL}};
const Segment SGI229654[] = {{&S97470[30], 6}, {NULL}};
const Segment SGI229655[] = {{&S97470[31], 3}, {NULL}};
const Segment SGI229656[] = {{&S97470[80], 5}, {NULL}};
const Segment SGI229657[] = {{&S97470[138], 13}, {NULL}};
const Segment SGI229658[] = {{&S94111[44], 13}, {NULL}};
const Segment SGI229659[] = {{&S94111[44], 25}, {NULL}};
const Segment SGI229660[] = {{&S94111[30], 6}, {NULL}};
const Segment SGI229661[] = {{&S79653[44], 10}, {NULL}};
const Segment SGI229662[] = {{&S94111[118], 3}, {NULL}};
const Segment SGI229663[] = {{&S79653[44], 21}, {NULL}};
const Segment SGI229664[] = {{&S79653[30], 6}, {NULL}};
const Segment SGI229665[] = {{&S94111[83], 6}, {NULL}};
const Segment SGI229666[] = {{&S79653[143], 3}, {NULL}};
const Segment SGI229667[] = {{&S79653[108], 6}, {NULL}};
const Segment SGI229668[] = {{&S87505[34], 10}, {NULL}};
const Segment SGI229669[] = {{&S87505[27], 6}, {NULL}};
const Segment SGI229670[] = {{&S87505[97], 9}, {NULL}};
const Segment SGI229671[] = {{&S79654[44], 10}, {NULL}};
const Segment SGI229672[] = {{&S87505[70], 6}, {NULL}};
const Segment SGI229673[] = {{&S79654[44], 22}, {NULL}};
const Segment SGI229674[] = {{&S79654[30], 6}, {NULL}};
const Segment SGI229675[] = {{&S79654[115], 3}, {NULL}};
const Segment SGI229676[] = {{&S79654[80], 6}, {NULL}};
const Segment SGI229677[] = {{&S87506[34], 10}, {NULL}};
const Segment SGI229678[] = {{&S87506[27], 6}, {NULL}};
const Segment SGI229679[] = {{&S87506[97], 9}, {NULL}};
const Segment SGI229680[] = {{&S87506[70], 6}, {NULL}};
const Segment SGI229681[] = {{&S97482[59], 14}, {NULL}};
const Segment SGI229682[] = {{&S97482[59], 26}, {NULL}};
const Segment SGI229683[] = {{&S87507[34], 10}, {NULL}};
const Segment SGI229684[] = {{&S97482[45], 6}, {NULL}};
const Segment SGI229685[] = {{&S87507[27], 6}, {NULL}};
const Segment SGI229686[] = {{&S97482[46], 3}, {NULL}};
const Segment SGI229687[] = {{&S97482[99], 6}, {NULL}};
const Segment SGI229688[] = {{&S87507[97], 9}, {NULL}};
const Segment SGI229689[] = {{&S87507[70], 6}, {NULL}};
const Segment SGI229690[] = {{&S97482[0], 14}, {NULL}};
const Segment SGI229691[] = {{&S97483[59], 9}, {NULL}};
const Segment SGI229692[] = {{&S97483[59], 21}, {NULL}};
const Segment SGI229693[] = {{&S97483[45], 6}, {NULL}};
const Segment SGI229694[] = {{&S97483[46], 3}, {NULL}};
const Segment SGI229695[] = {{&S97483[94], 6}, {NULL}};
const Segment SGI229696[] = {{&S97483[0], 14}, {NULL}};
const Segment SGI229697[] = {{&S97488[34], 8}, {NULL}};
const Segment SGI229698[] = {{&S97488[27], 6}, {NULL}};
const Segment SGI229699[] = {{&S97488[95], 9}, {NULL}};
const Segment SGI229700[] = {{&S97488[68], 6}, {NULL}};
const Segment SGI229701[] = {{&S87518[34], 7}, {NULL}};
const Segment SGI229702[] = {{&S87518[27], 6}, {NULL}};
const Segment SGI229703[] = {{&S87518[94], 9}, {NULL}};
const Segment SGI229704[] = {{&S87518[67], 6}, {NULL}};
const Segment SGI229705[] = {{&S94133[44], 10}, {NULL}};
const Segment SGI229706[] = {{&S94133[44], 22}, {NULL}};
const Segment SGI229707[] = {{&S94133[30], 6}, {NULL}};
const Segment SGI229708[] = {{&S94133[115], 3}, {NULL}};
const Segment SGI229709[] = {{&S94133[80], 6}, {NULL}};
const Segment SGI229710[] = {{&S97496[44], 4}, {NULL}};
const Segment SGI229711[] = {{&S97496[44], 26}, {NULL}};
const Segment SGI229712[] = {{&S97496[30], 6}, {NULL}};
const Segment SGI229713[] = {{&S97496[84], 6}, {NULL}};
const Segment SGI229714[] = {{&S97496[111], 13}, {NULL}};
const Segment SGI229715[] = {{&S97497[44], 10}, {NULL}};
const Segment SGI229716[] = {{&S97497[44], 22}, {NULL}};
const Segment SGI229717[] = {{&S97497[30], 6}, {NULL}};
const Segment SGI229718[] = {{&S97497[80], 6}, {NULL}};
const Segment SGI229719[] = {{&S97497[107], 13}, {NULL}};
const Segment SGI229720[] = {{&S97498[44], 13}, {NULL}};
const Segment SGI229721[] = {{&S97498[44], 25}, {NULL}};
const Segment SGI229722[] = {{&S97498[30], 6}, {NULL}};
const Segment SGI229723[] = {{&S97498[83], 6}, {NULL}};
const Segment SGI229724[] = {{&S97498[110], 13}, {NULL}};
const Segment SGI229725[] = {{&S94140[34], 17}, {NULL}};
const Segment SGI229726[] = {{&S94140[27], 6}, {NULL}};
const Segment SGI229727[] = {{&S94140[104], 9}, {NULL}};
const Segment SGI229728[] = {{&S94140[77], 6}, {NULL}};
const Segment SGI229729[] = {{&S88564[44], 6}, {NULL}};
const Segment SGI229730[] = {{&S88564[44], 18}, {NULL}};
const Segment SGI229731[] = {{&S88564[30], 6}, {NULL}};
const Segment SGI229732[] = {{&S88564[76], 6}, {NULL}};
const Segment SGI229733[] = {{&S88564[113], 3}, {NULL}};
const Segment SGI229734[] = {{&S97508[45], 17}, {NULL}};
const Segment SGI229735[] = {{&S97508[30], 6}, {NULL}};
const Segment SGI229736[] = {{&S97508[31], 3}, {NULL}};
const Segment SGI229737[] = {{&S97508[88], 5}, {NULL}};
const Segment SGI229738[] = {{&S97508[146], 13}, {NULL}};
const Segment SGI229739[] = {{&S97510[44], 9}, {NULL}};
const Segment SGI229740[] = {{&S97510[44], 21}, {NULL}};
const Segment SGI229741[] = {{&S97510[30], 6}, {NULL}};
const Segment SGI229742[] = {{&S97510[79], 6}, {NULL}};
const Segment SGI229743[] = {{&S97510[106], 13}, {NULL}};
const Segment SGI229744[] = {{&S97511[44], 10}, {NULL}};
const Segment SGI229745[] = {{&S97511[44], 22}, {NULL}};
const Segment SGI229746[] = {{&S97511[30], 6}, {NULL}};
const Segment SGI229747[] = {{&S97511[80], 6}, {NULL}};
const Segment SGI229748[] = {{&S97511[107], 13}, {NULL}};
const Segment SGI229749[] = {{&S94156[34], 11}, {NULL}};
const Segment SGI229750[] = {{&S94156[27], 6}, {NULL}};
const Segment SGI229751[] = {{&S94156[98], 9}, {NULL}};
const Segment SGI229752[] = {{&S94156[71], 6}, {NULL}};
const Segment SGI229753[] = {{&S79708[44], 7}, {NULL}};
const Segment SGI229754[] = {{&S79708[44], 18}, {NULL}};
const Segment SGI229755[] = {{&S79708[30], 6}, {NULL}};
const Segment SGI229756[] = {{&S79708[111], 3}, {NULL}};
const Segment SGI229757[] = {{&S79708[76], 6}, {NULL}};
const Segment SGI229758[] = {{&S97520[34], 20}, {NULL}};
const Segment SGI229759[] = {{&S97520[27], 6}, {NULL}};
const Segment SGI229760[] = {{&S97520[107], 9}, {NULL}};
const Segment SGI229761[] = {{&S97520[80], 6}, {NULL}};
const Segment SGI229762[] = {{&S97522[34], 7}, {NULL}};
const Segment SGI229763[] = {{&S97522[27], 6}, {NULL}};
const Segment SGI229764[] = {{&S97522[94], 9}, {NULL}};
const Segment SGI229765[] = {{&S97522[67], 6}, {NULL}};
const Segment SGI229766[] = {{&S97525[34], 7}, {NULL}};
const Segment SGI229767[] = {{&S97525[27], 6}, {NULL}};
const Segment SGI229768[] = {{&S97525[94], 9}, {NULL}};
const Segment SGI229769[] = {{&S97525[67], 6}, {NULL}};
const Segment SGI229770[] = {{&S94176[44], 14}, {NULL}};
const Segment SGI229771[] = {{&S94176[44], 26}, {NULL}};
const Segment SGI229772[] = {{&S94176[30], 6}, {NULL}};
const Segment SGI229773[] = {{&S94176[84], 6}, {NULL}};
const Segment SGI229774[] = {{&S94176[111], 13}, {NULL}};
const Segment SGI229775[] = {{&S79727[34], 7}, {NULL}};
const Segment SGI229776[] = {{&S79727[27], 6}, {NULL}};
const Segment SGI229777[] = {{&S79727[100], 9}, {NULL}};
const Segment SGI229778[] = {{&S79727[73], 6}, {NULL}};
const Segment SGI229779[] = {{&S79728[34], 7}, {NULL}};
const Segment SGI229780[] = {{&S79728[27], 6}, {NULL}};
const Segment SGI229781[] = {{&S87581[34], 11}, {NULL}};
const Segment SGI229782[] = {{&S79728[102], 9}, {NULL}};
const Segment SGI229783[] = {{&S87581[27], 6}, {NULL}};
const Segment SGI229784[] = {{&S79728[75], 6}, {NULL}};
const Segment SGI229785[] = {{&S87581[98], 9}, {NULL}};
const Segment SGI229786[] = {{&S87581[71], 6}, {NULL}};
const Segment SGI229787[] = {{&S79733[44], 7}, {NULL}};
const Segment SGI229788[] = {{&S97539[34], 6}, {NULL}};
const Segment SGI229789[] = {{&S79733[44], 19}, {NULL}};
const Segment SGI229790[] = {{&S97539[27], 6}, {NULL}};
const Segment SGI229791[] = {{&S79733[30], 6}, {NULL}};
const Segment SGI229792[] = {{&S79733[112], 3}, {NULL}};
const Segment SGI229793[] = {{&S97539[93], 9}, {NULL}};
const Segment SGI229794[] = {{&S97539[66], 6}, {NULL}};
const Segment SGI229795[] = {{&S79733[77], 6}, {NULL}};
const Segment SGI229796[] = {{&S79733[137], 8}, {NULL}};
const Segment SGI229797[] = {{&S97541[44], 11}, {NULL}};
const Segment SGI229798[] = {{&S97541[44], 23}, {NULL}};
const Segment SGI229799[] = {{&S97541[30], 6}, {NULL}};
const Segment SGI229800[] = {{&S97541[31], 3}, {NULL}};
const Segment SGI229801[] = {{&S97541[81], 5}, {NULL}};
const Segment SGI229802[] = {{&S97541[118], 14}, {NULL}};
const Segment SGI229803[] = {{&S79737[44], 18}, {NULL}};
const Segment SGI229804[] = {{&S79737[44], 29}, {NULL}};
const Segment SGI229805[] = {{&S79737[30], 6}, {NULL}};
const Segment SGI229806[] = {{&S97542[44], 10}, {NULL}};
const Segment SGI229807[] = {{&S97542[44], 22}, {NULL}};
const Segment SGI229808[] = {{&S97542[30], 6}, {NULL}};
const Segment SGI229809[] = {{&S79737[122], 3}, {NULL}};
const Segment SGI229810[] = {{&S79737[87], 6}, {NULL}};
const Segment SGI229811[] = {{&S97542[80], 6}, {NULL}};
const Segment SGI229812[] = {{&S79737[146], 10}, {NULL}};
const Segment SGI229813[] = {{&S97542[107], 13}, {NULL}};
const Segment SGI229814[] = {{&S97543[59], 6}, {NULL}};
const Segment SGI229815[] = {{&S97543[59], 18}, {NULL}};
const Segment SGI229816[] = {{&S97543[45], 6}, {NULL}};
const Segment SGI229817[] = {{&S97543[46], 3}, {NULL}};
const Segment SGI229818[] = {{&S97543[91], 6}, {NULL}};
const Segment SGI229819[] = {{&S97543[0], 14}, {NULL}};
const Segment SGI229820[] = {{&S97549[34], 9}, {NULL}};
const Segment SGI229821[] = {{&S97549[27], 6}, {NULL}};
const Segment SGI229822[] = {{&S87602[34], 8}, {NULL}};
const Segment SGI229823[] = {{&S87602[27], 6}, {NULL}};
const Segment SGI229824[] = {{&S97549[96], 9}, {NULL}};
const Segment SGI229825[] = {{&S97549[69], 6}, {NULL}};
const Segment SGI229826[] = {{&S87602[95], 9}, {NULL}};
const Segment SGI229827[] = {{&S97549[137], 8}, {NULL}};
const Segment SGI229828[] = {{&S87602[68], 6}, {NULL}};
const Segment SGI229829[] = {{&S88626[44], 13}, {NULL}};
const Segment SGI229830[] = {{&S88626[44], 25}, {NULL}};
const Segment SGI229831[] = {{&S87612[44], 10}, {NULL}};
const Segment SGI229832[] = {{&S88626[30], 6}, {NULL}};
const Segment SGI229833[] = {{&S87612[44], 22}, {NULL}};
const Segment SGI229834[] = {{&S87612[30], 6}, {NULL}};
const Segment SGI229835[] = {{&S88626[118], 3}, {NULL}};
const Segment SGI229836[] = {{&S88626[83], 6}, {NULL}};
const Segment SGI229837[] = {{&S87612[80], 6}, {NULL}};
const Segment SGI229838[] = {{&S87612[107], 13}, {NULL}};
const Segment SGI229839[] = {{&S88627[44], 17}, {NULL}};
const Segment SGI229840[] = {{&S88627[44], 29}, {NULL}};
const Segment SGI229841[] = {{&S88627[30], 6}, {NULL}};
const Segment SGI229842[] = {{&S88627[122], 3}, {NULL}};
const Segment SGI229843[] = {{&S88627[87], 6}, {NULL}};
const Segment SGI229844[] = {{&S87615[34], 13}, {NULL}};
const Segment SGI229845[] = {{&S87615[27], 6}, {NULL}};
const Segment SGI229846[] = {{&S87615[100], 9}, {NULL}};
const Segment SGI229847[] = {{&S87615[73], 6}, {NULL}};
const Segment SGI229848[] = {{&S88630[34], 10}, {NULL}};
const Segment SGI229849[] = {{&S88630[27], 6}, {NULL}};
const Segment SGI229850[] = {{&S88630[97], 9}, {NULL}};
const Segment SGI229851[] = {{&S88630[70], 6}, {NULL}};
const Segment SGI229852[] = {{&S87622[45], 10}, {NULL}};
const Segment SGI229853[] = {{&S87622[30], 6}, {NULL}};
const Segment SGI229854[] = {{&S87622[115], 3}, {NULL}};
const Segment SGI229855[] = {{&S87622[81], 5}, {NULL}};
const Segment SGI229856[] = {{&S87622[139], 13}, {NULL}};
const Segment SGI229857[] = {{&S87623[44], 14}, {NULL}};
const Segment SGI229858[] = {{&S87623[44], 36}, {NULL}};
const Segment SGI229859[] = {{&S87623[30], 6}, {NULL}};
const Segment SGI229860[] = {{&S87623[129], 3}, {NULL}};
const Segment SGI229861[] = {{&S87623[94], 6}, {NULL}};
const Segment SGI229862[] = {{&S87623[153], 10}, {NULL}};
const Segment SGI229863[] = {{&S87624[34], 8}, {NULL}};
const Segment SGI229864[] = {{&S87624[27], 6}, {NULL}};
const Segment SGI229865[] = {{&S87624[94], 9}, {NULL}};
const Segment SGI229866[] = {{&S87624[67], 6}, {NULL}};
const Segment SGI229867[] = {{&S87628[34], 8}, {NULL}};
const Segment SGI229868[] = {{&S87628[27], 6}, {NULL}};
const Segment SGI229869[] = {{&S87628[100], 9}, {NULL}};
const Segment SGI229870[] = {{&S87628[73], 6}, {NULL}};
const Segment SGI229871[] = {{&S87629[44], 13}, {NULL}};
const Segment SGI229872[] = {{&S87629[44], 25}, {NULL}};
const Segment SGI229873[] = {{&S87629[30], 6}, {NULL}};
const Segment SGI229874[] = {{&S87629[83], 6}, {NULL}};
const Segment SGI229875[] = {{&S87629[110], 13}, {NULL}};
const Segment SGI229876[] = {{&S87630[44], 15}, {NULL}};
const Segment SGI229877[] = {{&S87630[44], 27}, {NULL}};
const Segment SGI229878[] = {{&S87630[30], 6}, {NULL}};
const Segment SGI229879[] = {{&S87630[85], 6}, {NULL}};
const Segment SGI229880[] = {{&S87630[112], 13}, {NULL}};
const Segment SGI229881[] = {{&S97588[44], 5}, {NULL}};
const Segment SGI229882[] = {{&S97588[44], 21}, {NULL}};
const Segment SGI229883[] = {{&S97588[30], 6}, {NULL}};
const Segment SGI229884[] = {{&S79792[34], 18}, {NULL}};
const Segment SGI229885[] = {{&S79792[27], 6}, {NULL}};
const Segment SGI229886[] = {{&S97588[79], 6}, {NULL}};
const Segment SGI229887[] = {{&S79792[105], 9}, {NULL}};
const Segment SGI229888[] = {{&S97588[106], 13}, {NULL}};
const Segment SGI229889[] = {{&S79792[78], 6}, {NULL}};
const Segment SGI229890[] = {{&S79793[34], 22}, {NULL}};
const Segment SGI229891[] = {{&S79793[27], 6}, {NULL}};
const Segment SGI229892[] = {{&S79793[109], 9}, {NULL}};
const Segment SGI229893[] = {{&S79793[82], 6}, {NULL}};
const Segment SGI229894[] = {{&S87636[44], 15}, {NULL}};
const Segment SGI229895[] = {{&S87636[44], 27}, {NULL}};
const Segment SGI229896[] = {{&S87636[30], 6}, {NULL}};
const Segment SGI229897[] = {{&S87636[120], 3}, {NULL}};
const Segment SGI229898[] = {{&S87636[85], 6}, {NULL}};
const Segment SGI229899[] = {{&S97591[44], 7}, {NULL}};
const Segment SGI229900[] = {{&S97591[44], 25}, {NULL}};
const Segment SGI229901[] = {{&S97591[30], 6}, {NULL}};
const Segment SGI229902[] = {{&S97591[83], 6}, {NULL}};
const Segment SGI229903[] = {{&S97591[110], 13}, {NULL}};
const Segment SGI229904[] = {{&S87643[44], 10}, {NULL}};
const Segment SGI229905[] = {{&S87643[44], 22}, {NULL}};
const Segment SGI229906[] = {{&S87643[30], 6}, {NULL}};
const Segment SGI229907[] = {{&S97597[44], 7}, {NULL}};
const Segment SGI229908[] = {{&S97597[44], 25}, {NULL}};
const Segment SGI229909[] = {{&S87643[80], 6}, {NULL}};
const Segment SGI229910[] = {{&S97597[30], 6}, {NULL}};
const Segment SGI229911[] = {{&S87643[107], 13}, {NULL}};
const Segment SGI229912[] = {{&S97597[83], 6}, {NULL}};
const Segment SGI229913[] = {{&S97597[110], 13}, {NULL}};
const Segment SGI229914[] = {{&S87645[34], 10}, {NULL}};
const Segment SGI229915[] = {{&S87645[27], 6}, {NULL}};
const Segment SGI229916[] = {{&S94258[34], 7}, {NULL}};
const Segment SGI229917[] = {{&S87645[97], 9}, {NULL}};
const Segment SGI229918[] = {{&S87645[70], 6}, {NULL}};
const Segment SGI229919[] = {{&S94258[27], 6}, {NULL}};
const Segment SGI229920[] = {{&S94258[94], 9}, {NULL}};
const Segment SGI229921[] = {{&S94258[67], 6}, {NULL}};
const Segment SGI229922[] = {{&S97602[34], 9}, {NULL}};
const Segment SGI229923[] = {{&S97602[27], 6}, {NULL}};
const Segment SGI229924[] = {{&S97602[103], 9}, {NULL}};
const Segment SGI229925[] = {{&S97602[76], 6}, {NULL}};
const Segment SGI229926[] = {{&S94264[34], 12}, {NULL}};
const Segment SGI229927[] = {{&S94264[27], 6}, {NULL}};
const Segment SGI229928[] = {{&S97605[34], 10}, {NULL}};
const Segment SGI229929[] = {{&S97605[27], 6}, {NULL}};
const Segment SGI229930[] = {{&S94264[99], 9}, {NULL}};
const Segment SGI229931[] = {{&S94264[72], 6}, {NULL}};
const Segment SGI229932[] = {{&S97605[97], 9}, {NULL}};
const Segment SGI229933[] = {{&S97605[70], 6}, {NULL}};
const Segment SGI229934[] = {{&S87654[34], 7}, {NULL}};
const Segment SGI229935[] = {{&S87654[27], 6}, {NULL}};
const Segment SGI229936[] = {{&S97606[34], 9}, {NULL}};
const Segment SGI229937[] = {{&S87654[99], 9}, {NULL}};
const Segment SGI229938[] = {{&S97606[27], 6}, {NULL}};
const Segment SGI229939[] = {{&S87654[72], 6}, {NULL}};
const Segment SGI229940[] = {{&S87654[140], 8}, {NULL}};
const Segment SGI229941[] = {{&S97606[96], 9}, {NULL}};
const Segment SGI229942[] = {{&S97606[69], 6}, {NULL}};
const Segment SGI229943[] = {{&S87655[44], 15}, {NULL}};
const Segment SGI229944[] = {{&S87655[44], 33}, {NULL}};
const Segment SGI229945[] = {{&S87655[30], 6}, {NULL}};
const Segment SGI229946[] = {{&S87655[91], 6}, {NULL}};
const Segment SGI229947[] = {{&S87655[118], 13}, {NULL}};
const Segment SGI229948[] = {{&S97608[34], 10}, {NULL}};
const Segment SGI229949[] = {{&S97608[27], 6}, {NULL}};
const Segment SGI229950[] = {{&S97608[97], 9}, {NULL}};
const Segment SGI229951[] = {{&S97608[70], 6}, {NULL}};
const Segment SGI229952[] = {{&S97612[44], 14}, {NULL}};
const Segment SGI229953[] = {{&S97612[44], 26}, {NULL}};
const Segment SGI229954[] = {{&S97612[30], 6}, {NULL}};
const Segment SGI229955[] = {{&S97612[84], 6}, {NULL}};
const Segment SGI229956[] = {{&S97612[111], 13}, {NULL}};
const Segment SGI229957[] = {{&S94295[34], 10}, {NULL}};
const Segment SGI229958[] = {{&S94295[27], 6}, {NULL}};
const Segment SGI229959[] = {{&S94295[97], 9}, {NULL}};
const Segment SGI229960[] = {{&S94295[70], 6}, {NULL}};
const Segment SGI229961[] = {{&S94308[34], 13}, {NULL}};
const Segment SGI229962[] = {{&S94308[27], 6}, {NULL}};
const Segment SGI229963[] = {{&S94308[100], 9}, {NULL}};
const Segment SGI229964[] = {{&S94308[73], 6}, {NULL}};
const Segment SGI229965[] = {{&S87712[34], 7}, {NULL}};
const Segment SGI229966[] = {{&S87712[27], 6}, {NULL}};
const Segment SGI229967[] = {{&S87712[100], 9}, {NULL}};
const Segment SGI229968[] = {{&S87712[73], 6}, {NULL}};
const Segment SGI229969[] = {{&S87719[34], 10}, {NULL}};
const Segment SGI229970[] = {{&S87719[27], 6}, {NULL}};
const Segment SGI229971[] = {{&S87719[97], 9}, {NULL}};
const Segment SGI229972[] = {{&S87719[70], 6}, {NULL}};
const Segment SGI229973[] = {{&S94341[34], 25}, {NULL}};
const Segment SGI229974[] = {{&S87730[34], 11}, {NULL}};
const Segment SGI229975[] = {{&S94341[27], 6}, {NULL}};
const Segment SGI229976[] = {{&S87730[27], 6}, {NULL}};
const Segment SGI229977[] = {{&S87730[98], 9}, {NULL}};
const Segment SGI229978[] = {{&S94341[112], 9}, {NULL}};
const Segment SGI229979[] = {{&S87730[71], 6}, {NULL}};
const Segment SGI229980[] = {{&S94341[85], 6}, {NULL}};
const Segment SGI229981[] = {{&S97686[44], 10}, {NULL}};
const Segment SGI229982[] = {{&S97686[44], 24}, {NULL}};
const Segment SGI229983[] = {{&S97686[30], 6}, {NULL}};
const Segment SGI229984[] = {{&S97686[31], 3}, {NULL}};
const Segment SGI229985[] = {{&S97686[82], 6}, {NULL}};
const Segment SGI229986[] = {{&S97691[44], 7}, {NULL}};
const Segment SGI229987[] = {{&S97691[44], 19}, {NULL}};
const Segment SGI229988[] = {{&S97691[30], 6}, {NULL}};
const Segment SGI229989[] = {{&S97691[31], 3}, {NULL}};
const Segment SGI229990[] = {{&S97691[92], 6}, {NULL}};
const Segment SGI229991[] = {{&S97691[65], 8}, {NULL}};
const Segment SGI229992[] = {{&S97693[34], 11}, {NULL}};
const Segment SGI229993[] = {{&S97693[27], 6}, {NULL}};
const Segment SGI229994[] = {{&S97693[98], 9}, {NULL}};
const Segment SGI229995[] = {{&S97693[71], 6}, {NULL}};
const Segment SGI229996[] = {{&S94352[44], 25}, {NULL}};
const Segment SGI229997[] = {{&S94352[44], 37}, {NULL}};
const Segment SGI229998[] = {{&S94352[30], 6}, {NULL}};
const Segment SGI229999[] = {{&S94352[95], 6}, {NULL}};
const Segment SGI230000[] = {{&S94352[122], 13}, {NULL}};
const Segment SGI230001[] = {{&S94353[44], 10}, {NULL}};
const Segment SGI230002[] = {{&S94353[44], 27}, {NULL}};
const Segment SGI230003[] = {{&S94353[30], 6}, {NULL}};
const Segment SGI230004[] = {{&S94353[85], 6}, {NULL}};
const Segment SGI230005[] = {{&S94353[112], 13}, {NULL}};
const Segment SGI230006[] = {{&S97697[57], 10}, {NULL}};
const Segment SGI230007[] = {{&S97697[57], 22}, {NULL}};
const Segment SGI230008[] = {{&S97697[43], 6}, {NULL}};
const Segment SGI230009[] = {{&S97697[44], 3}, {NULL}};
const Segment SGI230010[] = {{&S97697[93], 6}, {NULL}};
const Segment SGI230011[] = {{&S97697[0], 12}, {NULL}};
const Segment SGI230012[] = {{&S87751[44], 6}, {NULL}};
const Segment SGI230013[] = {{&S87751[44], 18}, {NULL}};
const Segment SGI230014[] = {{&S87751[30], 6}, {NULL}};
const Segment SGI230015[] = {{&S87751[76], 6}, {NULL}};
const Segment SGI230016[] = {{&S87751[103], 13}, {NULL}};
const Segment SGI230017[] = {{&S94370[34], 15}, {NULL}};
const Segment SGI230018[] = {{&S94370[27], 6}, {NULL}};
const Segment SGI230019[] = {{&S88798[44], 7}, {NULL}};
const Segment SGI230020[] = {{&S88798[44], 19}, {NULL}};
const Segment SGI230021[] = {{&S88798[30], 6}, {NULL}};
const Segment SGI230022[] = {{&S94370[102], 9}, {NULL}};
const Segment SGI230023[] = {{&S94370[75], 6}, {NULL}};
const Segment SGI230024[] = {{&S88798[110], 3}, {NULL}};
const Segment SGI230025[] = {{&S88798[77], 5}, {NULL}};
const Segment SGI230026[] = {{&S94370[136], 8}, {NULL}};
const Segment SGI230027[] = {{&S97714[45], 12}, {NULL}};
const Segment SGI230028[] = {{&S97714[30], 6}, {NULL}};
const Segment SGI230029[] = {{&S97714[31], 3}, {NULL}};
const Segment SGI230030[] = {{&S97714[99], 5}, {NULL}};
const Segment SGI230031[] = {{&S97714[157], 13}, {NULL}};
const Segment SGI230032[] = {{&S87765[59], 11}, {NULL}};
const Segment SGI230033[] = {{&S87765[59], 23}, {NULL}};
const Segment SGI230034[] = {{&S87765[45], 6}, {NULL}};
const Segment SGI230035[] = {{&S79794[34], 21}, {NULL}};
const Segment SGI230036[] = {{&S79794[27], 6}, {NULL}};
const Segment SGI230037[] = {{&S87765[131], 3}, {NULL}};
const Segment SGI230038[] = {{&S87765[96], 6}, {NULL}};
const Segment SGI230039[] = {{&S87765[0], 14}, {NULL}};
const Segment SGI230040[] = {{&S79794[108], 9}, {NULL}};
const Segment SGI230041[] = {{&S79794[81], 6}, {NULL}};
const Segment SGI230042[] = {{&S97715[30], 6}, {NULL}};
const Segment SGI230043[] = {{&S97715[31], 3}, {NULL}};
const Segment SGI230044[] = {{&S97715[95], 5}, {NULL}};
const Segment SGI230045[] = {{&S87766[44], 9}, {NULL}};
const Segment SGI230046[] = {{&S97715[153], 13}, {NULL}};
const Segment SGI230047[] = {{&S79795[34], 21}, {NULL}};
const Segment SGI230048[] = {{&S87766[44], 21}, {NULL}};
const Segment SGI230049[] = {{&S87766[30], 6}, {NULL}};
const Segment SGI230050[] = {{&S79795[27], 6}, {NULL}};
const Segment SGI230051[] = {{&S79795[108], 9}, {NULL}};
const Segment SGI230052[] = {{&S87766[79], 6}, {NULL}};
const Segment SGI230053[] = {{&S79795[81], 6}, {NULL}};
const Segment SGI230054[] = {{&S87766[106], 13}, {NULL}};
const Segment SGI230055[] = {{&S79801[34], 10}, {NULL}};
const Segment SGI230056[] = {{&S87772[34], 7}, {NULL}};
const Segment SGI230057[] = {{&S87772[27], 6}, {NULL}};
const Segment SGI230058[] = {{&S79801[27], 6}, {NULL}};
const Segment SGI230059[] = {{&S87772[94], 9}, {NULL}};
const Segment SGI230060[] = {{&S87772[67], 6}, {NULL}};
const Segment SGI230061[] = {{&S79801[96], 9}, {NULL}};
const Segment SGI230062[] = {{&S79801[69], 6}, {NULL}};
const Segment SGI230063[] = {{&S79801[137], 8}, {NULL}};
const Segment SGI230064[] = {{&S79803[44], 14}, {NULL}};
const Segment SGI230065[] = {{&S79803[44], 25}, {NULL}};
const Segment SGI230066[] = {{&S79803[30], 6}, {NULL}};
const Segment SGI230067[] = {{&S79803[83], 6}, {NULL}};
const Segment SGI230068[] = {{&S79803[110], 13}, {NULL}};
const Segment SGI230069[] = {{&S87776[34], 9}, {NULL}};
const Segment SGI230070[] = {{&S87776[27], 6}, {NULL}};
const Segment SGI230071[] = {{&S87776[96], 9}, {NULL}};
const Segment SGI230072[] = {{&S87776[69], 6}, {NULL}};
const Segment SGI230073[] = {{&S87778[34], 9}, {NULL}};
const Segment SGI230074[] = {{&S87778[27], 6}, {NULL}};
const Segment SGI230075[] = {{&S88816[44], 17}, {NULL}};
const Segment SGI230076[] = {{&S88816[44], 37}, {NULL}};
const Segment SGI230077[] = {{&S88816[30], 6}, {NULL}};
const Segment SGI230078[] = {{&S87778[96], 9}, {NULL}};
const Segment SGI230079[] = {{&S87778[69], 6}, {NULL}};
const Segment SGI230080[] = {{&S88816[128], 3}, {NULL}};
const Segment SGI230081[] = {{&S88816[95], 5}, {NULL}};
const Segment SGI230082[] = {{&S79809[34], 14}, {NULL}};
const Segment SGI230083[] = {{&S87780[44], 7}, {NULL}};
const Segment SGI230084[] = {{&S79809[27], 6}, {NULL}};
const Segment SGI230085[] = {{&S87780[44], 24}, {NULL}};
const Segment SGI230086[] = {{&S87780[30], 6}, {NULL}};
const Segment SGI230087[] = {{&S79809[100], 9}, {NULL}};
const Segment SGI230088[] = {{&S79809[73], 6}, {NULL}};
const Segment SGI230089[] = {{&S87780[82], 6}, {NULL}};
const Segment SGI230090[] = {{&S87780[109], 13}, {NULL}};
const Segment SGI230091[] = {{&S79810[34], 14}, {NULL}};
const Segment SGI230092[] = {{&S79810[27], 6}, {NULL}};
const Segment SGI230093[] = {{&S79810[100], 9}, {NULL}};
const Segment SGI230094[] = {{&S79810[73], 6}, {NULL}};
const Segment SGI230095[] = {{&S97738[34], 20}, {NULL}};
const Segment SGI230096[] = {{&S97738[27], 6}, {NULL}};
const Segment SGI230097[] = {{&S97738[107], 9}, {NULL}};
const Segment SGI230098[] = {{&S97738[80], 6}, {NULL}};
const Segment SGI230099[] = {{&S87783[34], 10}, {NULL}};
const Segment SGI230100[] = {{&S79811[44], 10}, {NULL}};
const Segment SGI230101[] = {{&S87783[27], 6}, {NULL}};
const Segment SGI230102[] = {{&S79811[44], 21}, {NULL}};
const Segment SGI230103[] = {{&S79811[30], 6}, {NULL}};
const Segment SGI230104[] = {{&S87783[97], 9}, {NULL}};
const Segment SGI230105[] = {{&S87783[70], 6}, {NULL}};
const Segment SGI230106[] = {{&S79811[79], 6}, {NULL}};
const Segment SGI230107[] = {{&S79811[106], 13}, {NULL}};
const Segment SGI230108[] = {{&S97741[45], 15}, {NULL}};
const Segment SGI230109[] = {{&S97741[30], 6}, {NULL}};
const Segment SGI230110[] = {{&S97741[31], 3}, {NULL}};
const Segment SGI230111[] = {{&S97741[86], 5}, {NULL}};
const Segment SGI230112[] = {{&S97741[144], 13}, {NULL}};
const Segment SGI230113[] = {{&S97743[34], 7}, {NULL}};
const Segment SGI230114[] = {{&S97743[27], 6}, {NULL}};
const Segment SGI230115[] = {{&S97743[93], 9}, {NULL}};
const Segment SGI230116[] = {{&S97743[66], 6}, {NULL}};
const Segment SGI230117[] = {{&S97745[44], 10}, {NULL}};
const Segment SGI230118[] = {{&S97745[44], 22}, {NULL}};
const Segment SGI230119[] = {{&S97745[30], 6}, {NULL}};
const Segment SGI230120[] = {{&S97745[111], 2}, {NULL}};
const Segment SGI230121[] = {{&S97745[80], 5}, {NULL}};
const Segment SGI230122[] = {{&S97746[44], 10}, {NULL}};
const Segment SGI230123[] = {{&S97746[44], 25}, {NULL}};
const Segment SGI230124[] = {{&S97746[30], 6}, {NULL}};
const Segment SGI230125[] = {{&S97746[114], 2}, {NULL}};
const Segment SGI230126[] = {{&S97746[83], 5}, {NULL}};
const Segment SGI230127[] = {{&S97747[44], 11}, {NULL}};
const Segment SGI230128[] = {{&S97747[44], 23}, {NULL}};
const Segment SGI230129[] = {{&S97747[30], 6}, {NULL}};
const Segment SGI230130[] = {{&S97747[112], 2}, {NULL}};
const Segment SGI230131[] = {{&S97747[81], 5}, {NULL}};
const Segment SGI230132[] = {{&S97751[44], 10}, {NULL}};
const Segment SGI230133[] = {{&S97751[44], 22}, {NULL}};
const Segment SGI230134[] = {{&S97751[30], 6}, {NULL}};
const Segment SGI230135[] = {{&S97751[31], 3}, {NULL}};
const Segment SGI230136[] = {{&S97751[80], 6}, {NULL}};
const Segment SGI230137[] = {{&S97751[132], 10}, {NULL}};
const Segment SGI230138[] = {{&S97757[34], 10}, {NULL}};
const Segment SGI230139[] = {{&S97757[27], 6}, {NULL}};
const Segment SGI230140[] = {{&S97757[97], 9}, {NULL}};
const Segment SGI230141[] = {{&S97757[70], 6}, {NULL}};
const Segment SGI230142[] = {{&S97762[34], 9}, {NULL}};
const Segment SGI230143[] = {{&S97762[27], 6}, {NULL}};
const Segment SGI230144[] = {{&S97762[96], 9}, {NULL}};
const Segment SGI230145[] = {{&S97762[69], 6}, {NULL}};
const Segment SGI230146[] = {{&S97763[34], 6}, {NULL}};
const Segment SGI230147[] = {{&S97763[27], 6}, {NULL}};
const Segment SGI230148[] = {{&S97763[93], 9}, {NULL}};
const Segment SGI230149[] = {{&S97763[66], 6}, {NULL}};
const Segment SGI230150[] = {{&S97772[34], 15}, {NULL}};
const Segment SGI230151[] = {{&S97772[27], 6}, {NULL}};
const Segment SGI230152[] = {{&S97772[102], 9}, {NULL}};
const Segment SGI230153[] = {{&S97772[75], 6}, {NULL}};
const Segment SGI230154[] = {{&S79864[34], 7}, {NULL}};
const Segment SGI230155[] = {{&S79864[27], 6}, {NULL}};
const Segment SGI230156[] = {{&S79864[94], 9}, {NULL}};
const Segment SGI230157[] = {{&S79864[67], 6}, {NULL}};
const Segment SGI230158[] = {{&S97779[34], 12}, {NULL}};
const Segment SGI230159[] = {{&S97779[27], 6}, {NULL}};
const Segment SGI230160[] = {{&S97779[99], 9}, {NULL}};
const Segment SGI230161[] = {{&S97779[72], 6}, {NULL}};
const Segment SGI230162[] = {{&S97781[34], 10}, {NULL}};
const Segment SGI230163[] = {{&S97781[27], 6}, {NULL}};
const Segment SGI230164[] = {{&S97781[97], 9}, {NULL}};
const Segment SGI230165[] = {{&S97781[70], 6}, {NULL}};
const Segment SGI230166[] = {{&S97782[34], 7}, {NULL}};
const Segment SGI230167[] = {{&S97782[27], 6}, {NULL}};
const Segment SGI230168[] = {{&S97782[90], 9}, {NULL}};
const Segment SGI230169[] = {{&S97782[63], 6}, {NULL}};
const Segment SGI230170[] = {{&S97783[34], 7}, {NULL}};
const Segment SGI230171[] = {{&S97783[27], 6}, {NULL}};
const Segment SGI230172[] = {{&S97783[94], 9}, {NULL}};
const Segment SGI230173[] = {{&S97783[67], 6}, {NULL}};
const Segment SGI230174[] = {{&S97785[34], 7}, {NULL}};
const Segment SGI230175[] = {{&S97785[27], 6}, {NULL}};
const Segment SGI230176[] = {{&S97785[98], 9}, {NULL}};
const Segment SGI230177[] = {{&S97785[71], 6}, {NULL}};
const Segment SGI230178[] = {{&S94472[34], 11}, {NULL}};
const Segment SGI230179[] = {{&S94472[27], 6}, {NULL}};
const Segment SGI230180[] = {{&S94472[98], 9}, {NULL}};
const Segment SGI230181[] = {{&S94472[71], 6}, {NULL}};
const Segment SGI230182[] = {{&S87849[34], 11}, {NULL}};
const Segment SGI230183[] = {{&S87849[27], 6}, {NULL}};
const Segment SGI230184[] = {{&S97788[34], 7}, {NULL}};
const Segment SGI230185[] = {{&S97788[27], 6}, {NULL}};
const Segment SGI230186[] = {{&S87849[98], 9}, {NULL}};
const Segment SGI230187[] = {{&S87849[71], 6}, {NULL}};
const Segment SGI230188[] = {{&S97788[95], 9}, {NULL}};
const Segment SGI230189[] = {{&S97788[68], 6}, {NULL}};
const Segment SGI230190[] = {{&S97791[44], 18}, {NULL}};
const Segment SGI230191[] = {{&S97791[44], 30}, {NULL}};
const Segment SGI230192[] = {{&S97791[30], 6}, {NULL}};
const Segment SGI230193[] = {{&S97791[31], 3}, {NULL}};
const Segment SGI230194[] = {{&S97791[88], 5}, {NULL}};
const Segment SGI230195[] = {{&S97791[125], 14}, {NULL}};
const Segment SGI230196[] = {{&S87855[34], 24}, {NULL}};
const Segment SGI230197[] = {{&S87855[27], 6}, {NULL}};
const Segment SGI230198[] = {{&S87855[111], 9}, {NULL}};
const Segment SGI230199[] = {{&S87855[84], 6}, {NULL}};
const Segment SGI230200[] = {{&S97799[44], 10}, {NULL}};
const Segment SGI230201[] = {{&S97799[44], 22}, {NULL}};
const Segment SGI230202[] = {{&S97799[30], 6}, {NULL}};
const Segment SGI230203[] = {{&S97799[31], 3}, {NULL}};
const Segment SGI230204[] = {{&S97799[80], 6}, {NULL}};
const Segment SGI230205[] = {{&S97799[140], 11}, {NULL}};
const Segment SGI230206[] = {{&S97801[34], 7}, {NULL}};
const Segment SGI230207[] = {{&S97801[27], 6}, {NULL}};
const Segment SGI230208[] = {{&S97801[90], 9}, {NULL}};
const Segment SGI230209[] = {{&S97801[63], 6}, {NULL}};
const Segment SGI230210[] = {{&S87865[44], 13}, {NULL}};
const Segment SGI230211[] = {{&S87865[44], 25}, {NULL}};
const Segment SGI230212[] = {{&S87865[30], 6}, {NULL}};
const Segment SGI230213[] = {{&S87865[83], 6}, {NULL}};
const Segment SGI230214[] = {{&S94494[45], 10}, {NULL}};
const Segment SGI230215[] = {{&S87865[110], 13}, {NULL}};
const Segment SGI230216[] = {{&S94494[30], 6}, {NULL}};
const Segment SGI230217[] = {{&S94494[115], 3}, {NULL}};
const Segment SGI230218[] = {{&S94494[81], 5}, {NULL}};
const Segment SGI230219[] = {{&S94494[139], 13}, {NULL}};
const Segment SGI230220[] = {{&S97812[34], 7}, {NULL}};
const Segment SGI230221[] = {{&S97812[27], 6}, {NULL}};
const Segment SGI230222[] = {{&S97812[90], 9}, {NULL}};
const Segment SGI230223[] = {{&S97812[63], 6}, {NULL}};
const Segment SGI230224[] = {{&S97815[44], 14}, {NULL}};
const Segment SGI230225[] = {{&S97815[44], 26}, {NULL}};
const Segment SGI230226[] = {{&S97815[30], 6}, {NULL}};
const Segment SGI230227[] = {{&S97815[31], 3}, {NULL}};
const Segment SGI230228[] = {{&S97815[84], 6}, {NULL}};
const Segment SGI230229[] = {{&S87880[34], 14}, {NULL}};
const Segment SGI230230[] = {{&S97816[44], 13}, {NULL}};
const Segment SGI230231[] = {{&S87880[27], 6}, {NULL}};
const Segment SGI230232[] = {{&S97816[44], 25}, {NULL}};
const Segment SGI230233[] = {{&S97816[30], 6}, {NULL}};
const Segment SGI230234[] = {{&S97816[31], 3}, {NULL}};
const Segment SGI230235[] = {{&S87880[101], 9}, {NULL}};
const Segment SGI230236[] = {{&S87880[74], 6}, {NULL}};
const Segment SGI230237[] = {{&S97816[83], 6}, {NULL}};
const Segment SGI230238[] = {{&S97820[34], 10}, {NULL}};
const Segment SGI230239[] = {{&S97820[27], 6}, {NULL}};
const Segment SGI230240[] = {{&S94513[34], 8}, {NULL}};
const Segment SGI230241[] = {{&S94513[27], 6}, {NULL}};
const Segment SGI230242[] = {{&S97820[97], 9}, {NULL}};
const Segment SGI230243[] = {{&S97820[70], 6}, {NULL}};
const Segment SGI230244[] = {{&S94513[95], 9}, {NULL}};
const Segment SGI230245[] = {{&S94513[68], 6}, {NULL}};
const Segment SGI230246[] = {{&S97822[34], 13}, {NULL}};
const Segment SGI230247[] = {{&S97822[27], 6}, {NULL}};
const Segment SGI230248[] = {{&S97822[100], 9}, {NULL}};
const Segment SGI230249[] = {{&S87889[44], 12}, {NULL}};
const Segment SGI230250[] = {{&S97822[73], 6}, {NULL}};
const Segment SGI230251[] = {{&S87889[44], 34}, {NULL}};
const Segment SGI230252[] = {{&S87889[30], 6}, {NULL}};
const Segment SGI230253[] = {{&S87889[92], 6}, {NULL}};
const Segment SGI230254[] = {{&S87889[119], 13}, {NULL}};
const Segment SGI230255[] = {{&S97823[44], 10}, {NULL}};
const Segment SGI230256[] = {{&S97823[44], 30}, {NULL}};
const Segment SGI230257[] = {{&S97823[30], 6}, {NULL}};
const Segment SGI230258[] = {{&S97823[31], 3}, {NULL}};
const Segment SGI230259[] = {{&S97823[88], 5}, {NULL}};
const Segment SGI230260[] = {{&S97823[125], 14}, {NULL}};
const Segment SGI230261[] = {{&S94521[34], 9}, {NULL}};
const Segment SGI230262[] = {{&S94521[27], 6}, {NULL}};
const Segment SGI230263[] = {{&S94521[96], 9}, {NULL}};
const Segment SGI230264[] = {{&S94521[69], 6}, {NULL}};
const Segment SGI230265[] = {{&S87895[34], 9}, {NULL}};
const Segment SGI230266[] = {{&S87895[27], 6}, {NULL}};
const Segment SGI230267[] = {{&S97827[34], 10}, {NULL}};
const Segment SGI230268[] = {{&S97827[27], 6}, {NULL}};
const Segment SGI230269[] = {{&S87895[96], 9}, {NULL}};
const Segment SGI230270[] = {{&S87895[69], 6}, {NULL}};
const Segment SGI230271[] = {{&S97827[97], 9}, {NULL}};
const Segment SGI230272[] = {{&S97827[70], 6}, {NULL}};
const Segment SGI230273[] = {{&S88947[34], 19}, {NULL}};
const Segment SGI230274[] = {{&S88947[27], 6}, {NULL}};
const Segment SGI230275[] = {{&S88947[106], 9}, {NULL}};
const Segment SGI230276[] = {{&S88947[79], 6}, {NULL}};
const Segment SGI230277[] = {{&S97838[34], 15}, {NULL}};
const Segment SGI230278[] = {{&S97838[27], 6}, {NULL}};
const Segment SGI230279[] = {{&S97838[102], 9}, {NULL}};
const Segment SGI230280[] = {{&S97838[75], 6}, {NULL}};
const Segment SGI230281[] = {{&S97838[143], 8}, {NULL}};
const Segment SGI230282[] = {{&S94539[34], 6}, {NULL}};
const Segment SGI230283[] = {{&S94539[27], 6}, {NULL}};
const Segment SGI230284[] = {{&S97840[34], 13}, {NULL}};
const Segment SGI230285[] = {{&S97840[27], 6}, {NULL}};
const Segment SGI230286[] = {{&S94539[93], 9}, {NULL}};
const Segment SGI230287[] = {{&S94539[66], 6}, {NULL}};
const Segment SGI230288[] = {{&S97840[100], 9}, {NULL}};
const Segment SGI230289[] = {{&S97840[73], 6}, {NULL}};
const Segment SGI230290[] = {{&S94541[59], 15}, {NULL}};
const Segment SGI230291[] = {{&S94541[59], 27}, {NULL}};
const Segment SGI230292[] = {{&S94541[45], 6}, {NULL}};
const Segment SGI230293[] = {{&S97842[34], 6}, {NULL}};
const Segment SGI230294[] = {{&S97842[27], 6}, {NULL}};
const Segment SGI230295[] = {{&S94541[135], 3}, {NULL}};
const Segment SGI230296[] = {{&S94541[100], 6}, {NULL}};
const Segment SGI230297[] = {{&S97842[93], 9}, {NULL}};
const Segment SGI230298[] = {{&S94541[0], 14}, {NULL}};
const Segment SGI230299[] = {{&S97842[66], 6}, {NULL}};
const Segment SGI230300[] = {{&S79955[44], 9}, {NULL}};
const Segment SGI230301[] = {{&S79955[44], 20}, {NULL}};
const Segment SGI230302[] = {{&S79955[30], 6}, {NULL}};
const Segment SGI230303[] = {{&S79955[131], 3}, {NULL}};
const Segment SGI230304[] = {{&S79955[96], 6}, {NULL}};
const Segment SGI230305[] = {{&S79955[66], 8}, {NULL}};
const Segment SGI230306[] = {{&S94543[34], 12}, {NULL}};
const Segment SGI230307[] = {{&S94543[27], 6}, {NULL}};
const Segment SGI230308[] = {{&S94543[99], 9}, {NULL}};
const Segment SGI230309[] = {{&S94543[72], 6}, {NULL}};
const Segment SGI230310[] = {{&S79956[44], 9}, {NULL}};
const Segment SGI230311[] = {{&S79956[44], 21}, {NULL}};
const Segment SGI230312[] = {{&S79956[30], 6}, {NULL}};
const Segment SGI230313[] = {{&S94543[140], 8}, {NULL}};
const Segment SGI230314[] = {{&S79956[134], 3}, {NULL}};
const Segment SGI230315[] = {{&S79956[99], 6}, {NULL}};
const Segment SGI230316[] = {{&S79956[67], 8}, {NULL}};
const Segment SGI230317[] = {{&S97848[34], 9}, {NULL}};
const Segment SGI230318[] = {{&S97848[27], 6}, {NULL}};
const Segment SGI230319[] = {{&S97848[96], 9}, {NULL}};
const Segment SGI230320[] = {{&S97848[69], 6}, {NULL}};
const Segment SGI230321[] = {{&S94547[57], 4}, {NULL}};
const Segment SGI230322[] = {{&S94547[57], 16}, {NULL}};
const Segment SGI230323[] = {{&S94547[43], 6}, {NULL}};
const Segment SGI230324[] = {{&S94547[122], 3}, {NULL}};
const Segment SGI230325[] = {{&S94547[87], 6}, {NULL}};
const Segment SGI230326[] = {{&S94563[44], 6}, {NULL}};
const Segment SGI230327[] = {{&S94563[44], 18}, {NULL}};
const Segment SGI230328[] = {{&S94563[30], 6}, {NULL}};
const Segment SGI230329[] = {{&S94563[76], 6}, {NULL}};
const Segment SGI230330[] = {{&S94563[103], 13}, {NULL}};
const Segment SGI230331[] = {{&S97870[34], 4}, {NULL}};
const Segment SGI230332[] = {{&S97870[27], 6}, {NULL}};
const Segment SGI230333[] = {{&S97870[91], 9}, {NULL}};
const Segment SGI230334[] = {{&S97870[64], 6}, {NULL}};
const Segment SGI230335[] = {{&S97872[45], 15}, {NULL}};
const Segment SGI230336[] = {{&S97872[30], 6}, {NULL}};
const Segment SGI230337[] = {{&S97872[31], 3}, {NULL}};
const Segment SGI230338[] = {{&S97872[86], 5}, {NULL}};
const Segment SGI230339[] = {{&S97872[144], 13}, {NULL}};
const Segment SGI230340[] = {{&S97875[59], 10}, {NULL}};
const Segment SGI230341[] = {{&S97875[59], 22}, {NULL}};
const Segment SGI230342[] = {{&S97875[45], 6}, {NULL}};
const Segment SGI230343[] = {{&S97875[46], 3}, {NULL}};
const Segment SGI230344[] = {{&S97875[95], 6}, {NULL}};
const Segment SGI230345[] = {{&S97875[0], 14}, {NULL}};
const Segment SGI230346[] = {{&S97877[44], 18}, {NULL}};
const Segment SGI230347[] = {{&S97877[44], 30}, {NULL}};
const Segment SGI230348[] = {{&S97877[30], 6}, {NULL}};
const Segment SGI230349[] = {{&S97877[31], 3}, {NULL}};
const Segment SGI230350[] = {{&S97877[88], 6}, {NULL}};
const Segment SGI230351[] = {{&S97877[147], 10}, {NULL}};
const Segment SGI230352[] = {{&S87966[44], 18}, {NULL}};
const Segment SGI230353[] = {{&S87966[44], 30}, {NULL}};
const Segment SGI230354[] = {{&S87966[30], 6}, {NULL}};
const Segment SGI230355[] = {{&S87966[139], 3}, {NULL}};
const Segment SGI230356[] = {{&S87966[104], 6}, {NULL}};
const Segment SGI230357[] = {{&S87966[76], 8}, {NULL}};
const Segment SGI230358[] = {{&S94587[34], 9}, {NULL}};
const Segment SGI230359[] = {{&S94587[27], 6}, {NULL}};
const Segment SGI230360[] = {{&S94587[96], 9}, {NULL}};
const Segment SGI230361[] = {{&S94587[69], 6}, {NULL}};
const Segment SGI230362[] = {{&S89016[34], 12}, {NULL}};
const Segment SGI230363[] = {{&S89016[27], 6}, {NULL}};
const Segment SGI230364[] = {{&S89016[102], 9}, {NULL}};
const Segment SGI230365[] = {{&S97884[44], 10}, {NULL}};
const Segment SGI230366[] = {{&S89016[75], 6}, {NULL}};
const Segment SGI230367[] = {{&S97884[44], 22}, {NULL}};
const Segment SGI230368[] = {{&S97884[30], 6}, {NULL}};
const Segment SGI230369[] = {{&S97884[115], 3}, {NULL}};
const Segment SGI230370[] = {{&S97884[80], 6}, {NULL}};
const Segment SGI230371[] = {{&S97884[135], 8}, {NULL}};
const Segment SGI230372[] = {{&S97884[144], 13}, {NULL}};
const Segment SGI230373[] = {{&S97885[44], 4}, {NULL}};
const Segment SGI230374[] = {{&S97885[44], 13}, {NULL}};
const Segment SGI230375[] = {{&S97885[30], 6}, {NULL}};
const Segment SGI230376[] = {{&S97885[71], 6}, {NULL}};
const Segment SGI230377[] = {{&S97885[98], 13}, {NULL}};
const Segment SGI230378[] = {{&S97887[44], 11}, {NULL}};
const Segment SGI230379[] = {{&S97887[44], 23}, {NULL}};
const Segment SGI230380[] = {{&S97887[30], 6}, {NULL}};
const Segment SGI230381[] = {{&S97887[31], 3}, {NULL}};
const Segment SGI230382[] = {{&S97887[81], 5}, {NULL}};
const Segment SGI230383[] = {{&S89022[38], 10}, {NULL}};
const Segment SGI230384[] = {{&S89022[31], 6}, {NULL}};
const Segment SGI230385[] = {{&S97887[118], 14}, {NULL}};
const Segment SGI230386[] = {{&S80011[44], 11}, {NULL}};
const Segment SGI230387[] = {{&S80011[44], 26}, {NULL}};
const Segment SGI230388[] = {{&S89023[47], 10}, {NULL}};
const Segment SGI230389[] = {{&S80011[30], 6}, {NULL}};
const Segment SGI230390[] = {{&S89023[47], 76}, {NULL}};
const Segment SGI230391[] = {{&S89023[33], 6}, {NULL}};
const Segment SGI230392[] = {{&S80011[137], 3}, {NULL}};
const Segment SGI230393[] = {{&S80011[102], 6}, {NULL}};
const Segment SGI230394[] = {{&S80011[72], 8}, {NULL}};
const Segment SGI230395[] = {{&S97890[44], 9}, {NULL}};
const Segment SGI230396[] = {{&S97890[44], 21}, {NULL}};
const Segment SGI230397[] = {{&S89024[44], 10}, {NULL}};
const Segment SGI230398[] = {{&S97890[30], 6}, {NULL}};
const Segment SGI230399[] = {{&S89024[44], 22}, {NULL}};
const Segment SGI230400[] = {{&S89024[30], 6}, {NULL}};
const Segment SGI230401[] = {{&S89024[115], 3}, {NULL}};
const Segment SGI230402[] = {{&S97890[79], 6}, {NULL}};
const Segment SGI230403[] = {{&S89024[80], 6}, {NULL}};
const Segment SGI230404[] = {{&S97890[106], 13}, {NULL}};
const Segment SGI230405[] = {{&S94601[34], 8}, {NULL}};
const Segment SGI230406[] = {{&S94601[27], 6}, {NULL}};
const Segment SGI230407[] = {{&S94601[95], 9}, {NULL}};
const Segment SGI230408[] = {{&S94601[68], 6}, {NULL}};
const Segment SGI230409[] = {{&S94601[129], 8}, {NULL}};
const Segment SGI230410[] = {{&S97893[44], 10}, {NULL}};
const Segment SGI230411[] = {{&S97893[44], 22}, {NULL}};
const Segment SGI230412[] = {{&S97893[30], 6}, {NULL}};
const Segment SGI230413[] = {{&S97893[31], 3}, {NULL}};
const Segment SGI230414[] = {{&S97893[80], 6}, {NULL}};
const Segment SGI230415[] = {{&S97893[140], 7}, {NULL}};
const Segment SGI230416[] = {{&S80019[34], 10}, {NULL}};
const Segment SGI230417[] = {{&S80019[27], 6}, {NULL}};
const Segment SGI230418[] = {{&S80019[96], 9}, {NULL}};
const Segment SGI230419[] = {{&S80019[69], 6}, {NULL}};
const Segment SGI230420[] = {{&S97898[44], 14}, {NULL}};
const Segment SGI230421[] = {{&S97898[44], 31}, {NULL}};
const Segment SGI230422[] = {{&S97898[30], 6}, {NULL}};
const Segment SGI230423[] = {{&S97898[31], 3}, {NULL}};
const Segment SGI230424[] = {{&S97898[89], 5}, {NULL}};
const Segment SGI230425[] = {{&S97898[126], 14}, {NULL}};
const Segment SGI230426[] = {{&S80023[34], 9}, {NULL}};
const Segment SGI230427[] = {{&S80023[27], 6}, {NULL}};
const Segment SGI230428[] = {{&S80023[95], 9}, {NULL}};
const Segment SGI230429[] = {{&S80023[68], 6}, {NULL}};
const Segment SGI230430[] = {{&S97900[44], 7}, {NULL}};
const Segment SGI230431[] = {{&S97900[44], 18}, {NULL}};
const Segment SGI230432[] = {{&S97900[30], 6}, {NULL}};
const Segment SGI230433[] = {{&S97900[31], 3}, {NULL}};
const Segment SGI230434[] = {{&S80025[34], 10}, {NULL}};
const Segment SGI230435[] = {{&S97900[76], 6}, {NULL}};
const Segment SGI230436[] = {{&S80025[27], 6}, {NULL}};
const Segment SGI230437[] = {{&S97900[136], 7}, {NULL}};
const Segment SGI230438[] = {{&S80025[96], 9}, {NULL}};
const Segment SGI230439[] = {{&S80025[69], 6}, {NULL}};
const Segment SGI230440[] = {{&S97905[44], 4}, {NULL}};
const Segment SGI230441[] = {{&S97905[44], 13}, {NULL}};
const Segment SGI230442[] = {{&S97905[30], 6}, {NULL}};
const Segment SGI230443[] = {{&S94619[34], 7}, {NULL}};
const Segment SGI230444[] = {{&S97905[31], 3}, {NULL}};
const Segment SGI230445[] = {{&S94619[27], 6}, {NULL}};
const Segment SGI230446[] = {{&S97905[71], 5}, {NULL}};
const Segment SGI230447[] = {{&S97905[108], 14}, {NULL}};
const Segment SGI230448[] = {{&S94619[94], 9}, {NULL}};
const Segment SGI230449[] = {{&S94619[67], 6}, {NULL}};
const Segment SGI230450[] = {{&S97907[44], 7}, {NULL}};
const Segment SGI230451[] = {{&S97907[44], 24}, {NULL}};
const Segment SGI230452[] = {{&S97907[30], 6}, {NULL}};
const Segment SGI230453[] = {{&S97907[82], 6}, {NULL}};
const Segment SGI230454[] = {{&S97907[109], 13}, {NULL}};
const Segment SGI230455[] = {{&S97910[44], 9}, {NULL}};
const Segment SGI230456[] = {{&S97910[44], 21}, {NULL}};
const Segment SGI230457[] = {{&S97910[30], 6}, {NULL}};
const Segment SGI230458[] = {{&S97910[79], 6}, {NULL}};
const Segment SGI230459[] = {{&S97910[106], 13}, {NULL}};
const Segment SGI230460[] = {{&S97912[44], 10}, {NULL}};
const Segment SGI230461[] = {{&S97912[44], 22}, {NULL}};
const Segment SGI230462[] = {{&S97912[30], 6}, {NULL}};
const Segment SGI230463[] = {{&S97912[80], 6}, {NULL}};
const Segment SGI230464[] = {{&S97912[107], 13}, {NULL}};
const Segment SGI230465[] = {{&S97925[34], 7}, {NULL}};
const Segment SGI230466[] = {{&S97925[27], 6}, {NULL}};
const Segment SGI230467[] = {{&S97925[98], 9}, {NULL}};
const Segment SGI230468[] = {{&S97925[71], 6}, {NULL}};
const Segment SGI230469[] = {{&S97926[34], 10}, {NULL}};
const Segment SGI230470[] = {{&S97926[27], 6}, {NULL}};
const Segment SGI230471[] = {{&S97926[97], 9}, {NULL}};
const Segment SGI230472[] = {{&S97926[70], 6}, {NULL}};
const Segment SGI230473[] = {{&S97929[34], 7}, {NULL}};
const Segment SGI230474[] = {{&S97929[27], 6}, {NULL}};
const Segment SGI230475[] = {{&S97929[94], 9}, {NULL}};
const Segment SGI230476[] = {{&S97929[67], 6}, {NULL}};
const Segment SGI230477[] = {{&S97933[34], 9}, {NULL}};
const Segment SGI230478[] = {{&S97933[27], 6}, {NULL}};
const Segment SGI230479[] = {{&S97933[96], 9}, {NULL}};
const Segment SGI230480[] = {{&S97933[69], 6}, {NULL}};
const Segment SGI230481[] = {{&S97934[45], 5}, {NULL}};
const Segment SGI230482[] = {{&S97934[30], 6}, {NULL}};
const Segment SGI230483[] = {{&S97934[31], 3}, {NULL}};
const Segment SGI230484[] = {{&S97934[76], 5}, {NULL}};
const Segment SGI230485[] = {{&S97934[134], 13}, {NULL}};
const Segment SGI230486[] = {{&S97938[44], 23}, {NULL}};
const Segment SGI230487[] = {{&S97938[44], 34}, {NULL}};
const Segment SGI230488[] = {{&S97938[30], 6}, {NULL}};
const Segment SGI230489[] = {{&S97938[31], 3}, {NULL}};
const Segment SGI230490[] = {{&S97938[92], 6}, {NULL}};
const Segment SGI230491[] = {{&S97940[44], 5}, {NULL}};
const Segment SGI230492[] = {{&S97940[44], 17}, {NULL}};
const Segment SGI230493[] = {{&S97940[30], 6}, {NULL}};
const Segment SGI230494[] = {{&S97940[31], 3}, {NULL}};
const Segment SGI230495[] = {{&S97940[75], 6}, {NULL}};
const Segment SGI230496[] = {{&S97940[135], 13}, {NULL}};
const Segment SGI230497[] = {{&S97942[34], 9}, {NULL}};
const Segment SGI230498[] = {{&S97942[27], 6}, {NULL}};
const Segment SGI230499[] = {{&S97942[96], 9}, {NULL}};
const Segment SGI230500[] = {{&S97942[69], 6}, {NULL}};
const Segment SGI230501[] = {{&S97954[44], 7}, {NULL}};
const Segment SGI230502[] = {{&S97954[44], 19}, {NULL}};
const Segment SGI230503[] = {{&S97954[30], 6}, {NULL}};
const Segment SGI230504[] = {{&S97954[31], 3}, {NULL}};
const Segment SGI230505[] = {{&S97954[77], 5}, {NULL}};
const Segment SGI230506[] = {{&S97954[114], 14}, {NULL}};
const Segment SGI230507[] = {{&S97955[44], 14}, {NULL}};
const Segment SGI230508[] = {{&S97955[44], 26}, {NULL}};
const Segment SGI230509[] = {{&S97955[30], 6}, {NULL}};
const Segment SGI230510[] = {{&S97955[31], 3}, {NULL}};
const Segment SGI230511[] = {{&S97955[84], 6}, {NULL}};
const Segment SGI230512[] = {{&S97955[144], 7}, {NULL}};
const Segment SGI230513[] = {{&S80103[34], 10}, {NULL}};
const Segment SGI230514[] = {{&S80103[27], 6}, {NULL}};
const Segment SGI230515[] = {{&S80103[96], 9}, {NULL}};
const Segment SGI230516[] = {{&S80103[69], 6}, {NULL}};
const Segment SGI230517[] = {{&S80103[137], 8}, {NULL}};
const Segment SGI230518[] = {{&S97966[44], 19}, {NULL}};
const Segment SGI230519[] = {{&S97966[44], 31}, {NULL}};
const Segment SGI230520[] = {{&S97966[30], 6}, {NULL}};
const Segment SGI230521[] = {{&S97966[31], 3}, {NULL}};
const Segment SGI230522[] = {{&S97966[89], 6}, {NULL}};
const Segment SGI230523[] = {{&S97972[34], 10}, {NULL}};
const Segment SGI230524[] = {{&S97972[27], 6}, {NULL}};
const Segment SGI230525[] = {{&S97972[97], 9}, {NULL}};
const Segment SGI230526[] = {{&S97972[70], 6}, {NULL}};
const Segment SGI230527[] = {{&S97972[138], 8}, {NULL}};
const Segment SGI230528[] = {{&S97978[34], 7}, {NULL}};
const Segment SGI230529[] = {{&S97978[27], 6}, {NULL}};
const Segment SGI230530[] = {{&S97978[94], 9}, {NULL}};
const Segment SGI230531[] = {{&S97978[67], 6}, {NULL}};
const Segment SGI230532[] = {{&S97985[34], 20}, {NULL}};
const Segment SGI230533[] = {{&S94730[34], 10}, {NULL}};
const Segment SGI230534[] = {{&S97985[27], 6}, {NULL}};
const Segment SGI230535[] = {{&S94730[27], 6}, {NULL}};
const Segment SGI230536[] = {{&S97985[107], 9}, {NULL}};
const Segment SGI230537[] = {{&S94730[97], 9}, {NULL}};
const Segment SGI230538[] = {{&S97985[80], 6}, {NULL}};
const Segment SGI230539[] = {{&S94730[70], 6}, {NULL}};
const Segment SGI230540[] = {{&S94734[59], 10}, {NULL}};
const Segment SGI230541[] = {{&S94734[59], 22}, {NULL}};
const Segment SGI230542[] = {{&S94734[45], 6}, {NULL}};
const Segment SGI230543[] = {{&S94734[130], 3}, {NULL}};
const Segment SGI230544[] = {{&S94734[95], 6}, {NULL}};
const Segment SGI230545[] = {{&S94734[0], 14}, {NULL}};
const Segment SGI230546[] = {{&S97995[44], 10}, {NULL}};
const Segment SGI230547[] = {{&S97995[44], 29}, {NULL}};
const Segment SGI230548[] = {{&S97995[30], 6}, {NULL}};
const Segment SGI230549[] = {{&S97995[87], 6}, {NULL}};
const Segment SGI230550[] = {{&S97995[114], 13}, {NULL}};
const Segment SGI230551[] = {{&S97997[44], 13}, {NULL}};
const Segment SGI230552[] = {{&S97997[44], 25}, {NULL}};
const Segment SGI230553[] = {{&S97997[30], 6}, {NULL}};
const Segment SGI230554[] = {{&S97997[83], 6}, {NULL}};
const Segment SGI230555[] = {{&S97997[110], 13}, {NULL}};
const Segment SGI230556[] = {{&S94746[34], 15}, {NULL}};
const Segment SGI230557[] = {{&S94746[27], 6}, {NULL}};
const Segment SGI230558[] = {{&S94746[102], 9}, {NULL}};
const Segment SGI230559[] = {{&S94746[75], 6}, {NULL}};
const Segment SGI230560[] = {{&S98002[34], 8}, {NULL}};
const Segment SGI230561[] = {{&S98002[27], 6}, {NULL}};
const Segment SGI230562[] = {{&S98002[95], 9}, {NULL}};
const Segment SGI230563[] = {{&S98002[68], 6}, {NULL}};
const Segment SGI230564[] = {{&S98003[34], 10}, {NULL}};
const Segment SGI230565[] = {{&S98003[27], 6}, {NULL}};
const Segment SGI230566[] = {{&S80159[34], 11}, {NULL}};
const Segment SGI230567[] = {{&S80159[27], 6}, {NULL}};
const Segment SGI230568[] = {{&S98003[97], 9}, {NULL}};
const Segment SGI230569[] = {{&S98003[70], 6}, {NULL}};
const Segment SGI230570[] = {{&S80159[97], 9}, {NULL}};
const Segment SGI230571[] = {{&S80159[70], 6}, {NULL}};
const Segment SGI230572[] = {{&S88001[44], 9}, {NULL}};
const Segment SGI230573[] = {{&S88001[44], 21}, {NULL}};
const Segment SGI230574[] = {{&S88001[30], 6}, {NULL}};
const Segment SGI230575[] = {{&S94750[34], 24}, {NULL}};
const Segment SGI230576[] = {{&S94750[27], 6}, {NULL}};
const Segment SGI230577[] = {{&S89171[34], 10}, {NULL}};
const Segment SGI230578[] = {{&S89171[27], 6}, {NULL}};
const Segment SGI230579[] = {{&S88001[79], 6}, {NULL}};
const Segment SGI230580[] = {{&S94750[111], 9}, {NULL}};
const Segment SGI230581[] = {{&S88001[106], 13}, {NULL}};
const Segment SGI230582[] = {{&S98004[44], 11}, {NULL}};
const Segment SGI230583[] = {{&S94750[84], 6}, {NULL}};
const Segment SGI230584[] = {{&S98004[44], 23}, {NULL}};
const Segment SGI230585[] = {{&S89171[97], 9}, {NULL}};
const Segment SGI230586[] = {{&S98004[30], 6}, {NULL}};
const Segment SGI230587[] = {{&S89171[70], 6}, {NULL}};
const Segment SGI230588[] = {{&S98004[81], 6}, {NULL}};
const Segment SGI230589[] = {{&S98004[108], 13}, {NULL}};
const Segment SGI230590[] = {{&S88002[44], 13}, {NULL}};
const Segment SGI230591[] = {{&S88002[44], 25}, {NULL}};
const Segment SGI230592[] = {{&S88002[30], 6}, {NULL}};
const Segment SGI230593[] = {{&S88002[134], 3}, {NULL}};
const Segment SGI230594[] = {{&S88002[99], 6}, {NULL}};
const Segment SGI230595[] = {{&S88002[71], 8}, {NULL}};
const Segment SGI230596[] = {{&S88004[44], 13}, {NULL}};
const Segment SGI230597[] = {{&S88004[44], 24}, {NULL}};
const Segment SGI230598[] = {{&S88004[30], 6}, {NULL}};
const Segment SGI230599[] = {{&S88004[117], 3}, {NULL}};
const Segment SGI230600[] = {{&S88004[82], 6}, {NULL}};
const Segment SGI230601[] = {{&S98011[34], 7}, {NULL}};
const Segment SGI230602[] = {{&S98011[27], 6}, {NULL}};
const Segment SGI230603[] = {{&S94758[34], 6}, {NULL}};
const Segment SGI230604[] = {{&S94758[27], 6}, {NULL}};
const Segment SGI230605[] = {{&S98011[94], 9}, {NULL}};
const Segment SGI230606[] = {{&S98011[67], 6}, {NULL}};
const Segment SGI230607[] = {{&S94758[93], 9}, {NULL}};
const Segment SGI230608[] = {{&S94758[66], 6}, {NULL}};
const Segment SGI230609[] = {{&S98012[44], 6}, {NULL}};
const Segment SGI230610[] = {{&S98012[44], 18}, {NULL}};
const Segment SGI230611[] = {{&S98012[30], 6}, {NULL}};
const Segment SGI230612[] = {{&S98012[76], 6}, {NULL}};
const Segment SGI230613[] = {{&S98012[103], 13}, {NULL}};
const Segment SGI230614[] = {{&S94760[34], 13}, {NULL}};
const Segment SGI230615[] = {{&S94760[27], 6}, {NULL}};
const Segment SGI230616[] = {{&S88010[34], 10}, {NULL}};
const Segment SGI230617[] = {{&S88010[27], 6}, {NULL}};
const Segment SGI230618[] = {{&S94760[114], 9}, {NULL}};
const Segment SGI230619[] = {{&S94760[87], 6}, {NULL}};
const Segment SGI230620[] = {{&S98014[34], 10}, {NULL}};
const Segment SGI230621[] = {{&S98014[27], 6}, {NULL}};
const Segment SGI230622[] = {{&S98014[104], 9}, {NULL}};
const Segment SGI230623[] = {{&S98014[77], 6}, {NULL}};
const Segment SGI230624[] = {{&S98015[34], 9}, {NULL}};
const Segment SGI230625[] = {{&S98015[27], 6}, {NULL}};
const Segment SGI230626[] = {{&S98015[102], 9}, {NULL}};
const Segment SGI230627[] = {{&S98015[75], 6}, {NULL}};
const Segment SGI230628[] = {{&S98019[34], 9}, {NULL}};
const Segment SGI230629[] = {{&S98019[27], 6}, {NULL}};
const Segment SGI230630[] = {{&S98019[96], 9}, {NULL}};
const Segment SGI230631[] = {{&S98019[69], 6}, {NULL}};
const Segment SGI230632[] = {{&S94770[44], 6}, {NULL}};
const Segment SGI230633[] = {{&S94770[44], 18}, {NULL}};
const Segment SGI230634[] = {{&S94770[30], 6}, {NULL}};
const Segment SGI230635[] = {{&S94770[109], 3}, {NULL}};
const Segment SGI230636[] = {{&S94770[76], 5}, {NULL}};
const Segment SGI230637[] = {{&S94770[113], 14}, {NULL}};
const Segment SGI230638[] = {{&S94774[34], 17}, {NULL}};
const Segment SGI230639[] = {{&S94774[27], 6}, {NULL}};
const Segment SGI230640[] = {{&S94774[115], 9}, {NULL}};
const Segment SGI230641[] = {{&S94774[88], 6}, {NULL}};
const Segment SGI230642[] = {{&S89200[34], 6}, {NULL}};
const Segment SGI230643[] = {{&S89200[27], 6}, {NULL}};
const Segment SGI230644[] = {{&S89200[93], 9}, {NULL}};
const Segment SGI230645[] = {{&S89200[66], 6}, {NULL}};
const Segment SGI230646[] = {{&S98033[34], 11}, {NULL}};
const Segment SGI230647[] = {{&S98033[27], 6}, {NULL}};
const Segment SGI230648[] = {{&S98033[98], 9}, {NULL}};
const Segment SGI230649[] = {{&S98033[71], 6}, {NULL}};
const Segment SGI230650[] = {{&S98035[34], 10}, {NULL}};
const Segment SGI230651[] = {{&S98035[27], 6}, {NULL}};
const Segment SGI230652[] = {{&S98035[97], 9}, {NULL}};
const Segment SGI230653[] = {{&S98035[70], 6}, {NULL}};
const Segment SGI230654[] = {{&S94788[34], 13}, {NULL}};
const Segment SGI230655[] = {{&S94788[27], 6}, {NULL}};
const Segment SGI230656[] = {{&S94788[100], 9}, {NULL}};
const Segment SGI230657[] = {{&S94788[73], 6}, {NULL}};
const Segment SGI230658[] = {{&S94788[141], 8}, {NULL}};
const Segment SGI230659[] = {{&S94789[34], 9}, {NULL}};
const Segment SGI230660[] = {{&S94789[27], 6}, {NULL}};
const Segment SGI230661[] = {{&S94789[91], 9}, {NULL}};
const Segment SGI230662[] = {{&S94789[64], 6}, {NULL}};
const Segment SGI230663[] = {{&S94795[34], 7}, {NULL}};
const Segment SGI230664[] = {{&S94795[27], 6}, {NULL}};
const Segment SGI230665[] = {{&S94795[94], 9}, {NULL}};
const Segment SGI230666[] = {{&S94795[67], 6}, {NULL}};
const Segment SGI230667[] = {{&S80241[34], 12}, {NULL}};
const Segment SGI230668[] = {{&S80241[27], 6}, {NULL}};
const Segment SGI230669[] = {{&S80241[98], 9}, {NULL}};
const Segment SGI230670[] = {{&S80241[71], 6}, {NULL}};
const Segment SGI230671[] = {{&S80242[34], 28}, {NULL}};
const Segment SGI230672[] = {{&S80242[27], 6}, {NULL}};
const Segment SGI230673[] = {{&S80242[115], 9}, {NULL}};
const Segment SGI230674[] = {{&S80242[88], 6}, {NULL}};
const Segment SGI230675[] = {{&S80249[44], 10}, {NULL}};
const Segment SGI230676[] = {{&S80249[44], 22}, {NULL}};
const Segment SGI230677[] = {{&S80249[30], 6}, {NULL}};
const Segment SGI230678[] = {{&S80249[115], 3}, {NULL}};
const Segment SGI230679[] = {{&S80249[80], 6}, {NULL}};
const Segment SGI230680[] = {{&S80250[38], 10}, {NULL}};
const Segment SGI230681[] = {{&S80250[31], 6}, {NULL}};
const Segment SGI230682[] = {{&S94843[34], 15}, {NULL}};
const Segment SGI230683[] = {{&S94843[27], 6}, {NULL}};
const Segment SGI230684[] = {{&S94843[102], 9}, {NULL}};
const Segment SGI230685[] = {{&S94843[75], 6}, {NULL}};
const Segment SGI230686[] = {{&S80260[34], 10}, {NULL}};
const Segment SGI230687[] = {{&S80260[27], 6}, {NULL}};
const Segment SGI230688[] = {{&S94849[34], 25}, {NULL}};
const Segment SGI230689[] = {{&S89282[34], 10}, {NULL}};
const Segment SGI230690[] = {{&S94849[27], 6}, {NULL}};
const Segment SGI230691[] = {{&S89282[27], 6}, {NULL}};
const Segment SGI230692[] = {{&S80260[96], 9}, {NULL}};
const Segment SGI230693[] = {{&S80260[69], 6}, {NULL}};
const Segment SGI230694[] = {{&S94849[112], 9}, {NULL}};
const Segment SGI230695[] = {{&S89282[97], 9}, {NULL}};
const Segment SGI230696[] = {{&S94849[85], 6}, {NULL}};
const Segment SGI230697[] = {{&S89282[70], 6}, {NULL}};
const Segment SGI230698[] = {{&S98120[44], 25}, {NULL}};
const Segment SGI230699[] = {{&S98120[44], 37}, {NULL}};
const Segment SGI230700[] = {{&S98120[30], 6}, {NULL}};
const Segment SGI230701[] = {{&S98120[31], 3}, {NULL}};
const Segment SGI230702[] = {{&S98120[114], 6}, {NULL}};
const Segment SGI230703[] = {{&S98120[83], 8}, {NULL}};
const Segment SGI230704[] = {{&S94861[34], 4}, {NULL}};
const Segment SGI230705[] = {{&S94861[27], 6}, {NULL}};
const Segment SGI230706[] = {{&S94861[91], 9}, {NULL}};
const Segment SGI230707[] = {{&S94861[64], 6}, {NULL}};
const Segment SGI230708[] = {{&S98128[44], 10}, {NULL}};
const Segment SGI230709[] = {{&S98128[44], 22}, {NULL}};
const Segment SGI230710[] = {{&S98128[30], 6}, {NULL}};
const Segment SGI230711[] = {{&S98128[31], 3}, {NULL}};
const Segment SGI230712[] = {{&S98128[80], 6}, {NULL}};
const Segment SGI230713[] = {{&S98128[150], 3}, {NULL}};
const Segment SGI230714[] = {{&S98130[44], 11}, {NULL}};
const Segment SGI230715[] = {{&S98130[44], 23}, {NULL}};
const Segment SGI230716[] = {{&S98130[30], 6}, {NULL}};
const Segment SGI230717[] = {{&S98130[31], 3}, {NULL}};
const Segment SGI230718[] = {{&S94868[44], 13}, {NULL}};
const Segment SGI230719[] = {{&S98130[81], 6}, {NULL}};
const Segment SGI230720[] = {{&S94868[44], 25}, {NULL}};
const Segment SGI230721[] = {{&S94868[30], 6}, {NULL}};
const Segment SGI230722[] = {{&S98130[141], 7}, {NULL}};
const Segment SGI230723[] = {{&S94868[83], 6}, {NULL}};
const Segment SGI230724[] = {{&S94868[110], 13}, {NULL}};
const Segment SGI230725[] = {{&S94870[34], 5}, {NULL}};
const Segment SGI230726[] = {{&S94870[27], 6}, {NULL}};
const Segment SGI230727[] = {{&S94870[92], 9}, {NULL}};
const Segment SGI230728[] = {{&S94870[65], 6}, {NULL}};
const Segment SGI230729[] = {{&S94870[133], 8}, {NULL}};
const Segment SGI230730[] = {{&S98139[45], 7}, {NULL}};
const Segment SGI230731[] = {{&S98139[30], 6}, {NULL}};
const Segment SGI230732[] = {{&S98139[31], 3}, {NULL}};
const Segment SGI230733[] = {{&S98139[83], 5}, {NULL}};
const Segment SGI230734[] = {{&S98139[141], 13}, {NULL}};
const Segment SGI230735[] = {{&S80295[34], 10}, {NULL}};
const Segment SGI230736[] = {{&S80295[27], 6}, {NULL}};
const Segment SGI230737[] = {{&S80295[96], 9}, {NULL}};
const Segment SGI230738[] = {{&S80295[69], 6}, {NULL}};
const Segment SGI230739[] = {{&S98148[45], 12}, {NULL}};
const Segment SGI230740[] = {{&S98148[30], 6}, {NULL}};
const Segment SGI230741[] = {{&S98148[31], 3}, {NULL}};
const Segment SGI230742[] = {{&S98148[99], 5}, {NULL}};
const Segment SGI230743[] = {{&S98148[157], 13}, {NULL}};
const Segment SGI230744[] = {{&S98149[45], 18}, {NULL}};
const Segment SGI230745[] = {{&S98149[30], 6}, {NULL}};
const Segment SGI230746[] = {{&S98149[31], 3}, {NULL}};
const Segment SGI230747[] = {{&S98149[89], 5}, {NULL}};
const Segment SGI230748[] = {{&S98149[147], 13}, {NULL}};
const Segment SGI230749[] = {{&S80305[34], 10}, {NULL}};
const Segment SGI230750[] = {{&S80305[27], 6}, {NULL}};
const Segment SGI230751[] = {{&S80305[96], 9}, {NULL}};
const Segment SGI230752[] = {{&S80305[69], 6}, {NULL}};
const Segment SGI230753[] = {{&S88010[97], 9}, {NULL}};
const Segment SGI230754[] = {{&S88010[70], 6}, {NULL}};
const Segment SGI230755[] = {{&S88014[34], 8}, {NULL}};
const Segment SGI230756[] = {{&S88014[27], 6}, {NULL}};
const Segment SGI230757[] = {{&S88014[95], 9}, {NULL}};
const Segment SGI230758[] = {{&S88014[68], 6}, {NULL}};
const Segment SGI230759[] = {{&S80313[44], 19}, {NULL}};
const Segment SGI230760[] = {{&S80313[44], 31}, {NULL}};
const Segment SGI230761[] = {{&S80313[30], 6}, {NULL}};
const Segment SGI230762[] = {{&S98165[44], 8}, {NULL}};
const Segment SGI230763[] = {{&S98165[44], 36}, {NULL}};
const Segment SGI230764[] = {{&S98165[30], 6}, {NULL}};
const Segment SGI230765[] = {{&S80313[142], 3}, {NULL}};
const Segment SGI230766[] = {{&S80313[107], 6}, {NULL}};
const Segment SGI230767[] = {{&S80313[77], 8}, {NULL}};
const Segment SGI230768[] = {{&S98165[31], 3}, {NULL}};
const Segment SGI230769[] = {{&S98165[94], 6}, {NULL}};
const Segment SGI230770[] = {{&S98165[154], 11}, {NULL}};
const Segment SGI230771[] = {{&S98865[42], 8}, {NULL}};
const Segment SGI230772[] = {{&S98865[42], 20}, {NULL}};
const Segment SGI230773[] = {{&S98865[30], 4}, {NULL}};
const Segment SGI230774[] = {{&S98865[111], 3}, {NULL}};
const Segment SGI230775[] = {{&S98865[76], 6}, {NULL}};
const Segment SGI230776[] = {{&S80320[44], 10}, {NULL}};
const Segment SGI230777[] = {{&S80320[44], 21}, {NULL}};
const Segment SGI230778[] = {{&S80320[30], 6}, {NULL}};
const Segment SGI230779[] = {{&S88023[44], 7}, {NULL}};
const Segment SGI230780[] = {{&S88023[44], 24}, {NULL}};
const Segment SGI230781[] = {{&S80320[114], 3}, {NULL}};
const Segment SGI230782[] = {{&S88023[30], 6}, {NULL}};
const Segment SGI230783[] = {{&S80320[79], 6}, {NULL}};
const Segment SGI230784[] = {{&S80320[139], 7}, {NULL}};
const Segment SGI230785[] = {{&S88023[82], 6}, {NULL}};
const Segment SGI230786[] = {{&S88023[109], 13}, {NULL}};
const Segment SGI230787[] = {{&S88026[44], 13}, {NULL}};
const Segment SGI230788[] = {{&S88026[44], 25}, {NULL}};
const Segment SGI230789[] = {{&S88026[30], 6}, {NULL}};
const Segment SGI230790[] = {{&S88026[118], 3}, {NULL}};
const Segment SGI230791[] = {{&S88026[83], 6}, {NULL}};
const Segment SGI230792[] = {{&S88026[138], 8}, {NULL}};
const Segment SGI230793[] = {{&S88026[147], 13}, {NULL}};
const Segment SGI230794[] = {{&S88027[45], 13}, {NULL}};
const Segment SGI230795[] = {{&S88027[30], 6}, {NULL}};
const Segment SGI230796[] = {{&S88027[118], 3}, {NULL}};
const Segment SGI230797[] = {{&S88027[84], 5}, {NULL}};
const Segment SGI230798[] = {{&S88027[142], 13}, {NULL}};
const Segment SGI230799[] = {{&S88029[44], 10}, {NULL}};
const Segment SGI230800[] = {{&S88029[44], 22}, {NULL}};
const Segment SGI230801[] = {{&S88029[30], 6}, {NULL}};
const Segment SGI230802[] = {{&S88029[115], 3}, {NULL}};
const Segment SGI230803[] = {{&S88029[80], 6}, {NULL}};
const Segment SGI230804[] = {{&S98183[34], 10}, {NULL}};
const Segment SGI230805[] = {{&S98183[27], 6}, {NULL}};
const Segment SGI230806[] = {{&S88031[34], 10}, {NULL}};
const Segment SGI230807[] = {{&S98183[97], 9}, {NULL}};
const Segment SGI230808[] = {{&S98183[70], 6}, {NULL}};
const Segment SGI230809[] = {{&S88031[27], 6}, {NULL}};
const Segment SGI230810[] = {{&S98183[132], 8}, {NULL}};
const Segment SGI230811[] = {{&S88031[97], 9}, {NULL}};
const Segment SGI230812[] = {{&S88031[70], 6}, {NULL}};
const Segment SGI230813[] = {{&S98184[34], 10}, {NULL}};
const Segment SGI230814[] = {{&S98184[27], 6}, {NULL}};
const Segment SGI230815[] = {{&S98184[97], 9}, {NULL}};
const Segment SGI230816[] = {{&S98184[70], 6}, {NULL}};
const Segment SGI230817[] = {{&S98184[131], 8}, {NULL}};
const Segment SGI230818[] = {{&S88035[34], 10}, {NULL}};
const Segment SGI230819[] = {{&S88035[27], 6}, {NULL}};
const Segment SGI230820[] = {{&S98186[59], 13}, {NULL}};
const Segment SGI230821[] = {{&S98186[59], 25}, {NULL}};
const Segment SGI230822[] = {{&S98186[45], 6}, {NULL}};
const Segment SGI230823[] = {{&S88035[97], 9}, {NULL}};
const Segment SGI230824[] = {{&S88035[70], 6}, {NULL}};
const Segment SGI230825[] = {{&S98186[46], 3}, {NULL}};
const Segment SGI230826[] = {{&S98186[98], 6}, {NULL}};
const Segment SGI230827[] = {{&S98186[0], 14}, {NULL}};
const Segment SGI230828[] = {{&S88037[34], 7}, {NULL}};
const Segment SGI230829[] = {{&S88037[27], 6}, {NULL}};
const Segment SGI230830[] = {{&S88037[99], 9}, {NULL}};
const Segment SGI230831[] = {{&S88037[72], 6}, {NULL}};
const Segment SGI230832[] = {{&S98189[34], 13}, {NULL}};
const Segment SGI230833[] = {{&S98189[27], 6}, {NULL}};
const Segment SGI230834[] = {{&S98189[112], 9}, {NULL}};
const Segment SGI230835[] = {{&S98189[85], 6}, {NULL}};
const Segment SGI230836[] = {{&S98190[30], 6}, {NULL}};
const Segment SGI230837[] = {{&S98190[31], 3}, {NULL}};
const Segment SGI230838[] = {{&S98190[95], 5}, {NULL}};
const Segment SGI230839[] = {{&S98190[153], 13}, {NULL}};
const Segment SGI230840[] = {{&S94949[34], 13}, {NULL}};
const Segment SGI230841[] = {{&S94949[27], 6}, {NULL}};
const Segment SGI230842[] = {{&S94949[100], 9}, {NULL}};
const Segment SGI230843[] = {{&S94949[73], 6}, {NULL}};
const Segment SGI230844[] = {{&S98200[34], 7}, {NULL}};
const Segment SGI230845[] = {{&S98200[27], 6}, {NULL}};
const Segment SGI230846[] = {{&S98200[95], 9}, {NULL}};
const Segment SGI230847[] = {{&S98200[68], 6}, {NULL}};
const Segment SGI230848[] = {{&S98201[34], 13}, {NULL}};
const Segment SGI230849[] = {{&S98201[27], 6}, {NULL}};
const Segment SGI230850[] = {{&S98201[100], 9}, {NULL}};
const Segment SGI230851[] = {{&S98201[73], 6}, {NULL}};
const Segment SGI230852[] = {{&S98212[34], 25}, {NULL}};
const Segment SGI230853[] = {{&S98212[27], 6}, {NULL}};
const Segment SGI230854[] = {{&S98212[112], 9}, {NULL}};
const Segment SGI230855[] = {{&S98212[85], 6}, {NULL}};
const Segment SGI230856[] = {{&S94972[34], 7}, {NULL}};
const Segment SGI230857[] = {{&S94972[27], 6}, {NULL}};
const Segment SGI230858[] = {{&S98213[34], 14}, {NULL}};
const Segment SGI230859[] = {{&S98213[27], 6}, {NULL}};
const Segment SGI230860[] = {{&S94972[94], 9}, {NULL}};
const Segment SGI230861[] = {{&S94972[67], 6}, {NULL}};
const Segment SGI230862[] = {{&S98213[101], 9}, {NULL}};
const Segment SGI230863[] = {{&S98213[74], 6}, {NULL}};
const Segment SGI230864[] = {{&S98216[34], 41}, {NULL}};
const Segment SGI230865[] = {{&S98216[27], 6}, {NULL}};
const Segment SGI230866[] = {{&S98216[138], 9}, {NULL}};
const Segment SGI230867[] = {{&S98216[111], 6}, {NULL}};
const Segment SGI230868[] = {{&S98217[34], 7}, {NULL}};
const Segment SGI230869[] = {{&S98217[27], 6}, {NULL}};
const Segment SGI230870[] = {{&S98217[90], 9}, {NULL}};
const Segment SGI230871[] = {{&S98217[63], 6}, {NULL}};
const Segment SGI230872[] = {{&S94985[59], 14}, {NULL}};
const Segment SGI230873[] = {{&S94985[59], 31}, {NULL}};
const Segment SGI230874[] = {{&S94985[45], 6}, {NULL}};
const Segment SGI230875[] = {{&S98224[34], 7}, {NULL}};
const Segment SGI230876[] = {{&S98224[27], 6}, {NULL}};
const Segment SGI230877[] = {{&S94985[139], 3}, {NULL}};
const Segment SGI230878[] = {{&S94985[104], 6}, {NULL}};
const Segment SGI230879[] = {{&S98224[94], 9}, {NULL}};
const Segment SGI230880[] = {{&S98224[67], 6}, {NULL}};
const Segment SGI230881[] = {{&S94985[0], 14}, {NULL}};
const Segment SGI230882[] = {{&S98227[34], 11}, {NULL}};
const Segment SGI230883[] = {{&S98227[27], 6}, {NULL}};
const Segment SGI230884[] = {{&S98227[98], 9}, {NULL}};
const Segment SGI230885[] = {{&S98227[71], 6}, {NULL}};
const Segment SGI230886[] = {{&S94993[34], 15}, {NULL}};
const Segment SGI230887[] = {{&S94993[27], 6}, {NULL}};
const Segment SGI230888[] = {{&S88094[34], 10}, {NULL}};
const Segment SGI230889[] = {{&S88094[27], 6}, {NULL}};
const Segment SGI230890[] = {{&S94993[109], 9}, {NULL}};
const Segment SGI230891[] = {{&S94993[82], 6}, {NULL}};
const Segment SGI230892[] = {{&S88094[97], 9}, {NULL}};
const Segment SGI230893[] = {{&S88094[70], 6}, {NULL}};
const Segment SGI230894[] = {{&S98231[59], 11}, {NULL}};
const Segment SGI230895[] = {{&S98231[59], 23}, {NULL}};
const Segment SGI230896[] = {{&S98231[45], 6}, {NULL}};
const Segment SGI230897[] = {{&S98231[46], 3}, {NULL}};
const Segment SGI230898[] = {{&S98231[96], 6}, {NULL}};
const Segment SGI230899[] = {{&S98231[0], 14}, {NULL}};
const Segment SGI230900[] = {{&S98233[44], 15}, {NULL}};
const Segment SGI230901[] = {{&S98233[44], 33}, {NULL}};
const Segment SGI230902[] = {{&S88097[38], 10}, {NULL}};
const Segment SGI230903[] = {{&S94996[44], 11}, {NULL}};
const Segment SGI230904[] = {{&S98233[30], 6}, {NULL}};
const Segment SGI230905[] = {{&S88097[31], 6}, {NULL}};
const Segment SGI230906[] = {{&S94996[44], 23}, {NULL}};
const Segment SGI230907[] = {{&S94996[30], 6}, {NULL}};
const Segment SGI230908[] = {{&S98233[31], 3}, {NULL}};
const Segment SGI230909[] = {{&S94996[116], 3}, {NULL}};
const Segment SGI230910[] = {{&S98233[91], 5}, {NULL}};
const Segment SGI230911[] = {{&S94996[81], 6}, {NULL}};
const Segment SGI230912[] = {{&S98233[128], 14}, {NULL}};
const Segment SGI230913[] = {{&S94996[140], 10}, {NULL}};
const Segment SGI230914[] = {{&S88098[44], 10}, {NULL}};
const Segment SGI230915[] = {{&S88098[44], 22}, {NULL}};
const Segment SGI230916[] = {{&S88098[30], 6}, {NULL}};
const Segment SGI230917[] = {{&S88098[115], 3}, {NULL}};
const Segment SGI230918[] = {{&S88098[80], 6}, {NULL}};
const Segment SGI230919[] = {{&S98234[34], 9}, {NULL}};
const Segment SGI230920[] = {{&S98234[27], 6}, {NULL}};
const Segment SGI230921[] = {{&S98234[96], 9}, {NULL}};
const Segment SGI230922[] = {{&S98234[69], 6}, {NULL}};
const Segment SGI230923[] = {{&S98235[34], 10}, {NULL}};
const Segment SGI230924[] = {{&S98235[27], 6}, {NULL}};
const Segment SGI230925[] = {{&S98235[94], 9}, {NULL}};
const Segment SGI230926[] = {{&S98235[67], 6}, {NULL}};
const Segment SGI230927[] = {{&S98242[34], 10}, {NULL}};
const Segment SGI230928[] = {{&S98242[27], 6}, {NULL}};
const Segment SGI230929[] = {{&S98242[97], 9}, {NULL}};
const Segment SGI230930[] = {{&S98242[70], 6}, {NULL}};
const Segment SGI230931[] = {{&S95009[34], 18}, {NULL}};
const Segment SGI230932[] = {{&S95009[27], 6}, {NULL}};
const Segment SGI230933[] = {{&S95009[114], 9}, {NULL}};
const Segment SGI230934[] = {{&S98243[34], 15}, {NULL}};
const Segment SGI230935[] = {{&S95009[87], 6}, {NULL}};
const Segment SGI230936[] = {{&S98243[27], 6}, {NULL}};
const Segment SGI230937[] = {{&S88110[44], 15}, {NULL}};
const Segment SGI230938[] = {{&S88110[44], 39}, {NULL}};
const Segment SGI230939[] = {{&S98243[102], 9}, {NULL}};
const Segment SGI230940[] = {{&S88110[30], 6}, {NULL}};
const Segment SGI230941[] = {{&S98243[75], 6}, {NULL}};
const Segment SGI230942[] = {{&S88110[132], 3}, {NULL}};
const Segment SGI230943[] = {{&S88110[97], 6}, {NULL}};
const Segment SGI230944[] = {{&S88110[156], 10}, {NULL}};
const Segment SGI230945[] = {{&S88111[44], 15}, {NULL}};
const Segment SGI230946[] = {{&S88111[44], 30}, {NULL}};
const Segment SGI230947[] = {{&S88111[30], 6}, {NULL}};
const Segment SGI230948[] = {{&S88111[123], 3}, {NULL}};
const Segment SGI230949[] = {{&S88111[88], 6}, {NULL}};
const Segment SGI230950[] = {{&S88111[147], 10}, {NULL}};
const Segment SGI230951[] = {{&S95015[44], 13}, {NULL}};
const Segment SGI230952[] = {{&S95015[44], 25}, {NULL}};
const Segment SGI230953[] = {{&S95015[30], 6}, {NULL}};
const Segment SGI230954[] = {{&S95015[118], 3}, {NULL}};
const Segment SGI230955[] = {{&S95015[83], 6}, {NULL}};
const Segment SGI230956[] = {{&S95017[44], 15}, {NULL}};
const Segment SGI230957[] = {{&S95017[44], 28}, {NULL}};
const Segment SGI230958[] = {{&S95017[30], 6}, {NULL}};
const Segment SGI230959[] = {{&S88117[44], 10}, {NULL}};
const Segment SGI230960[] = {{&S95017[121], 3}, {NULL}};
const Segment SGI230961[] = {{&S88117[44], 22}, {NULL}};
const Segment SGI230962[] = {{&S95017[86], 6}, {NULL}};
const Segment SGI230963[] = {{&S88117[30], 6}, {NULL}};
const Segment SGI230964[] = {{&S88117[115], 3}, {NULL}};
const Segment SGI230965[] = {{&S88117[80], 6}, {NULL}};
const Segment SGI230966[] = {{&S88117[150], 3}, {NULL}};
const Segment SGI230967[] = {{&S95020[34], 10}, {NULL}};
const Segment SGI230968[] = {{&S95020[27], 6}, {NULL}};
const Segment SGI230969[] = {{&S95020[97], 9}, {NULL}};
const Segment SGI230970[] = {{&S95020[70], 6}, {NULL}};
const Segment SGI230971[] = {{&S98260[44], 4}, {NULL}};
const Segment SGI230972[] = {{&S98260[44], 16}, {NULL}};
const Segment SGI230973[] = {{&S98260[30], 6}, {NULL}};
const Segment SGI230974[] = {{&S98260[31], 3}, {NULL}};
const Segment SGI230975[] = {{&S98260[74], 6}, {NULL}};
const Segment SGI230976[] = {{&S98261[59], 14}, {NULL}};
const Segment SGI230977[] = {{&S98261[59], 31}, {NULL}};
const Segment SGI230978[] = {{&S98261[45], 6}, {NULL}};
const Segment SGI230979[] = {{&S88125[34], 7}, {NULL}};
const Segment SGI230980[] = {{&S98261[46], 3}, {NULL}};
const Segment SGI230981[] = {{&S88125[27], 6}, {NULL}};
const Segment SGI230982[] = {{&S98261[104], 6}, {NULL}};
const Segment SGI230983[] = {{&S98261[0], 14}, {NULL}};
const Segment SGI230984[] = {{&S88125[102], 9}, {NULL}};
const Segment SGI230985[] = {{&S88125[75], 6}, {NULL}};
const Segment SGI230986[] = {{&S80447[34], 11}, {NULL}};
const Segment SGI230987[] = {{&S80447[27], 6}, {NULL}};
const Segment SGI230988[] = {{&S80447[97], 9}, {NULL}};
const Segment SGI230989[] = {{&S80447[70], 6}, {NULL}};
const Segment SGI230990[] = {{&S98265[45], 10}, {NULL}};
const Segment SGI230991[] = {{&S98265[30], 6}, {NULL}};
const Segment SGI230992[] = {{&S98265[31], 3}, {NULL}};
const Segment SGI230993[] = {{&S98265[81], 5}, {NULL}};
const Segment SGI230994[] = {{&S98265[139], 13}, {NULL}};
const Segment SGI230995[] = {{&S95032[34], 20}, {NULL}};
const Segment SGI230996[] = {{&S95032[27], 6}, {NULL}};
const Segment SGI230997[] = {{&S98266[34], 16}, {NULL}};
const Segment SGI230998[] = {{&S98266[27], 6}, {NULL}};
const Segment SGI230999[] = {{&S95032[107], 9}, {NULL}};
const Segment SGI231000[] = {{&S95032[80], 6}, {NULL}};
const Segment SGI231001[] = {{&S98266[116], 9}, {NULL}};
const Segment SGI231002[] = {{&S98266[89], 6}, {NULL}};
const Segment SGI231003[] = {{&S88135[34], 7}, {NULL}};
const Segment SGI231004[] = {{&S88135[27], 6}, {NULL}};
const Segment SGI231005[] = {{&S88135[94], 9}, {NULL}};
const Segment SGI231006[] = {{&S98269[34], 9}, {NULL}};
const Segment SGI231007[] = {{&S88135[67], 6}, {NULL}};
const Segment SGI231008[] = {{&S98269[27], 6}, {NULL}};
const Segment SGI231009[] = {{&S98269[96], 9}, {NULL}};
const Segment SGI231010[] = {{&S98269[69], 6}, {NULL}};
const Segment SGI231011[] = {{&S95042[34], 8}, {NULL}};
const Segment SGI231012[] = {{&S95042[27], 6}, {NULL}};
const Segment SGI231013[] = {{&S98275[34], 11}, {NULL}};
const Segment SGI231014[] = {{&S95042[95], 9}, {NULL}};
const Segment SGI231015[] = {{&S98275[27], 6}, {NULL}};
const Segment SGI231016[] = {{&S95042[68], 6}, {NULL}};
const Segment SGI231017[] = {{&S95042[129], 8}, {NULL}};
const Segment SGI231018[] = {{&S98275[98], 9}, {NULL}};
const Segment SGI231019[] = {{&S98275[71], 6}, {NULL}};
const Segment SGI231020[] = {{&S98277[44], 14}, {NULL}};
const Segment SGI231021[] = {{&S98277[44], 31}, {NULL}};
const Segment SGI231022[] = {{&S98277[30], 6}, {NULL}};
const Segment SGI231023[] = {{&S98277[31], 3}, {NULL}};
const Segment SGI231024[] = {{&S98277[89], 5}, {NULL}};
const Segment SGI231025[] = {{&S98277[126], 14}, {NULL}};
const Segment SGI231026[] = {{&S98278[34], 12}, {NULL}};
const Segment SGI231027[] = {{&S98278[27], 6}, {NULL}};
const Segment SGI231028[] = {{&S98278[99], 9}, {NULL}};
const Segment SGI231029[] = {{&S98278[72], 6}, {NULL}};
const Segment SGI231030[] = {{&S98280[34], 11}, {NULL}};
const Segment SGI231031[] = {{&S98280[27], 6}, {NULL}};
const Segment SGI231032[] = {{&S98280[98], 9}, {NULL}};
const Segment SGI231033[] = {{&S98280[71], 6}, {NULL}};
const Segment SGI231034[] = {{&S98281[34], 10}, {NULL}};
const Segment SGI231035[] = {{&S98281[27], 6}, {NULL}};
const Segment SGI231036[] = {{&S98281[97], 9}, {NULL}};
const Segment SGI231037[] = {{&S98281[70], 6}, {NULL}};
const Segment SGI231038[] = {{&S88156[34], 16}, {NULL}};
const Segment SGI231039[] = {{&S88156[27], 6}, {NULL}};
const Segment SGI231040[] = {{&S88156[103], 9}, {NULL}};
const Segment SGI231041[] = {{&S88156[76], 6}, {NULL}};
const Segment SGI231042[] = {{&S98283[44], 7}, {NULL}};
const Segment SGI231043[] = {{&S98283[44], 26}, {NULL}};
const Segment SGI231044[] = {{&S98283[30], 6}, {NULL}};
const Segment SGI231045[] = {{&S98283[84], 6}, {NULL}};
const Segment SGI231046[] = {{&S98283[111], 13}, {NULL}};
const Segment SGI231047[] = {{&S98285[44], 8}, {NULL}};
const Segment SGI231048[] = {{&S98285[44], 20}, {NULL}};
const Segment SGI231049[] = {{&S98285[30], 6}, {NULL}};
const Segment SGI231050[] = {{&S98285[31], 3}, {NULL}};
const Segment SGI231051[] = {{&S98285[78], 6}, {NULL}};
const Segment SGI231052[] = {{&S98287[34], 15}, {NULL}};
const Segment SGI231053[] = {{&S98287[27], 6}, {NULL}};
const Segment SGI231054[] = {{&S98287[102], 9}, {NULL}};
const Segment SGI231055[] = {{&S98287[75], 6}, {NULL}};
const Segment SGI231056[] = {{&S98287[136], 8}, {NULL}};
const Segment SGI231057[] = {{&S98290[34], 9}, {NULL}};
const Segment SGI231058[] = {{&S98290[27], 6}, {NULL}};
const Segment SGI231059[] = {{&S98290[96], 9}, {NULL}};
const Segment SGI231060[] = {{&S98290[69], 6}, {NULL}};
const Segment SGI231061[] = {{&S98291[34], 12}, {NULL}};
const Segment SGI231062[] = {{&S98291[27], 6}, {NULL}};
const Segment SGI231063[] = {{&S98291[99], 9}, {NULL}};
const Segment SGI231064[] = {{&S98291[72], 6}, {NULL}};
const Segment SGI231065[] = {{&S98293[34], 15}, {NULL}};
const Segment SGI231066[] = {{&S98293[27], 6}, {NULL}};
const Segment SGI231067[] = {{&S98293[102], 9}, {NULL}};
const Segment SGI231068[] = {{&S98293[75], 6}, {NULL}};
const Segment SGI231069[] = {{&S95074[44], 18}, {NULL}};
const Segment SGI231070[] = {{&S95074[44], 39}, {NULL}};
const Segment SGI231071[] = {{&S98294[34], 8}, {NULL}};
const Segment SGI231072[] = {{&S95074[30], 6}, {NULL}};
const Segment SGI231073[] = {{&S98294[27], 6}, {NULL}};
const Segment SGI231074[] = {{&S98294[94], 9}, {NULL}};
const Segment SGI231075[] = {{&S98294[67], 6}, {NULL}};
const Segment SGI231076[] = {{&S95074[97], 6}, {NULL}};
const Segment SGI231077[] = {{&S95074[124], 13}, {NULL}};
const Segment SGI231078[] = {{&S98297[34], 8}, {NULL}};
const Segment SGI231079[] = {{&S98297[27], 6}, {NULL}};
const Segment SGI231080[] = {{&S98297[94], 9}, {NULL}};
const Segment SGI231081[] = {{&S98297[67], 6}, {NULL}};
const Segment SGI231082[] = {{&S95080[34], 25}, {NULL}};
const Segment SGI231083[] = {{&S95080[27], 6}, {NULL}};
const Segment SGI231084[] = {{&S98299[34], 12}, {NULL}};
const Segment SGI231085[] = {{&S98299[27], 6}, {NULL}};
const Segment SGI231086[] = {{&S95080[112], 9}, {NULL}};
const Segment SGI231087[] = {{&S95080[85], 6}, {NULL}};
const Segment SGI231088[] = {{&S98299[99], 9}, {NULL}};
const Segment SGI231089[] = {{&S98299[72], 6}, {NULL}};
const Segment SGI231090[] = {{&S98299[140], 8}, {NULL}};
const Segment SGI231091[] = {{&S98300[34], 9}, {NULL}};
const Segment SGI231092[] = {{&S98300[27], 6}, {NULL}};
const Segment SGI231093[] = {{&S98300[96], 9}, {NULL}};
const Segment SGI231094[] = {{&S98300[69], 6}, {NULL}};
const Segment SGI231095[] = {{&S95083[34], 12}, {NULL}};
const Segment SGI231096[] = {{&S95083[27], 6}, {NULL}};
const Segment SGI231097[] = {{&S95083[99], 9}, {NULL}};
const Segment SGI231098[] = {{&S95083[72], 6}, {NULL}};
const Segment SGI231099[] = {{&S98304[34], 15}, {NULL}};
const Segment SGI231100[] = {{&S98304[27], 6}, {NULL}};
const Segment SGI231101[] = {{&S98304[102], 9}, {NULL}};
const Segment SGI231102[] = {{&S98304[75], 6}, {NULL}};
const Segment SGI231103[] = {{&S98305[34], 7}, {NULL}};
const Segment SGI231104[] = {{&S98305[27], 6}, {NULL}};
const Segment SGI231105[] = {{&S98305[98], 9}, {NULL}};
const Segment SGI231106[] = {{&S98305[71], 6}, {NULL}};
const Segment SGI231107[] = {{&S98308[34], 6}, {NULL}};
const Segment SGI231108[] = {{&S98308[27], 6}, {NULL}};
const Segment SGI231109[] = {{&S98308[93], 9}, {NULL}};
const Segment SGI231110[] = {{&S98308[66], 6}, {NULL}};
const Segment SGI231111[] = {{&S98309[34], 11}, {NULL}};
const Segment SGI231112[] = {{&S98309[27], 6}, {NULL}};
const Segment SGI231113[] = {{&S98309[98], 9}, {NULL}};
const Segment SGI231114[] = {{&S98309[71], 6}, {NULL}};
const Segment SGI231115[] = {{&S98309[139], 8}, {NULL}};
const Segment SGI231116[] = {{&S98310[34], 7}, {NULL}};
const Segment SGI231117[] = {{&S98310[27], 6}, {NULL}};
const Segment SGI231118[] = {{&S98310[94], 9}, {NULL}};
const Segment SGI231119[] = {{&S98310[67], 6}, {NULL}};
const Segment SGI231120[] = {{&S98314[27], 6}, {NULL}};
const Segment SGI231121[] = {{&S95089[34], 7}, {NULL}};
const Segment SGI231122[] = {{&S98314[111], 9}, {NULL}};
const Segment SGI231123[] = {{&S95089[27], 6}, {NULL}};
const Segment SGI231124[] = {{&S98314[84], 6}, {NULL}};
const Segment SGI231125[] = {{&S88216[45], 10}, {NULL}};
const Segment SGI231126[] = {{&S95089[94], 9}, {NULL}};
const Segment SGI231127[] = {{&S95089[67], 6}, {NULL}};
const Segment SGI231128[] = {{&S88216[30], 6}, {NULL}};
const Segment SGI231129[] = {{&S88216[115], 3}, {NULL}};
const Segment SGI231130[] = {{&S88216[81], 5}, {NULL}};
const Segment SGI231131[] = {{&S88216[139], 13}, {NULL}};
const Segment SGI231132[] = {{&S88217[44], 9}, {NULL}};
const Segment SGI231133[] = {{&S88217[44], 21}, {NULL}};
const Segment SGI231134[] = {{&S88217[30], 6}, {NULL}};
const Segment SGI231135[] = {{&S98317[44], 10}, {NULL}};
const Segment SGI231136[] = {{&S98317[44], 20}, {NULL}};
const Segment SGI231137[] = {{&S88217[136], 3}, {NULL}};
const Segment SGI231138[] = {{&S98317[30], 6}, {NULL}};
const Segment SGI231139[] = {{&S88217[101], 6}, {NULL}};
const Segment SGI231140[] = {{&S98317[31], 3}, {NULL}};
const Segment SGI231141[] = {{&S88217[67], 8}, {NULL}};
const Segment SGI231142[] = {{&S98317[78], 6}, {NULL}};
const Segment SGI231143[] = {{&S98321[34], 8}, {NULL}};
const Segment SGI231144[] = {{&S98321[27], 6}, {NULL}};
const Segment SGI231145[] = {{&S98321[95], 9}, {NULL}};
const Segment SGI231146[] = {{&S98321[68], 6}, {NULL}};
const Segment SGI231147[] = {{&S88222[34], 10}, {NULL}};
const Segment SGI231148[] = {{&S88222[27], 6}, {NULL}};
const Segment SGI231149[] = {{&S88222[97], 9}, {NULL}};
const Segment SGI231150[] = {{&S88222[70], 6}, {NULL}};
const Segment SGI231151[] = {{&S98326[34], 15}, {NULL}};
const Segment SGI231152[] = {{&S98326[27], 6}, {NULL}};
const Segment SGI231153[] = {{&S98326[104], 9}, {NULL}};
const Segment SGI231154[] = {{&S98326[77], 6}, {NULL}};
const Segment SGI231155[] = {{&S98328[44], 14}, {NULL}};
const Segment SGI231156[] = {{&S98328[44], 31}, {NULL}};
const Segment SGI231157[] = {{&S98328[30], 6}, {NULL}};
const Segment SGI231158[] = {{&S98328[89], 6}, {NULL}};
const Segment SGI231159[] = {{&S98328[116], 13}, {NULL}};
const Segment SGI231160[] = {{&S95108[34], 17}, {NULL}};
const Segment SGI231161[] = {{&S95108[27], 6}, {NULL}};
const Segment SGI231162[] = {{&S95108[104], 9}, {NULL}};
const Segment SGI231163[] = {{&S95108[77], 6}, {NULL}};
const Segment SGI231164[] = {{&S98334[44], 11}, {NULL}};
const Segment SGI231165[] = {{&S98334[44], 23}, {NULL}};
const Segment SGI231166[] = {{&S98334[30], 6}, {NULL}};
const Segment SGI231167[] = {{&S98334[159], 3}, {NULL}};
const Segment SGI231168[] = {{&S95112[34], 13}, {NULL}};
const Segment SGI231169[] = {{&S95112[27], 6}, {NULL}};
const Segment SGI231170[] = {{&S98334[125], 3}, {NULL}};
const Segment SGI231171[] = {{&S98334[90], 6}, {NULL}};
const Segment SGI231172[] = {{&S98334[145], 8}, {NULL}};
const Segment SGI231173[] = {{&S95112[100], 9}, {NULL}};
const Segment SGI231174[] = {{&S95112[73], 6}, {NULL}};
const Segment SGI231175[] = {{&S98336[44], 13}, {NULL}};
const Segment SGI231176[] = {{&S98336[44], 25}, {NULL}};
const Segment SGI231177[] = {{&S98336[30], 6}, {NULL}};
const Segment SGI231178[] = {{&S98336[31], 3}, {NULL}};
const Segment SGI231179[] = {{&S98336[83], 6}, {NULL}};
const Segment SGI231180[] = {{&S88246[44], 13}, {NULL}};
const Segment SGI231181[] = {{&S88246[44], 25}, {NULL}};
const Segment SGI231182[] = {{&S88246[30], 6}, {NULL}};
const Segment SGI231183[] = {{&S88246[83], 6}, {NULL}};
const Segment SGI231184[] = {{&S88246[110], 13}, {NULL}};
const Segment SGI231185[] = {{&S88247[44], 13}, {NULL}};
const Segment SGI231186[] = {{&S88247[44], 25}, {NULL}};
const Segment SGI231187[] = {{&S88247[30], 6}, {NULL}};
const Segment SGI231188[] = {{&S88247[83], 6}, {NULL}};
const Segment SGI231189[] = {{&S88247[110], 13}, {NULL}};
const Segment SGI231190[] = {{&S98347[34], 16}, {NULL}};
const Segment SGI231191[] = {{&S98347[27], 6}, {NULL}};
const Segment SGI231192[] = {{&S98347[111], 9}, {NULL}};
const Segment SGI231193[] = {{&S98347[84], 6}, {NULL}};
const Segment SGI231194[] = {{&S98348[44], 7}, {NULL}};
const Segment SGI231195[] = {{&S98348[44], 19}, {NULL}};
const Segment SGI231196[] = {{&S98348[30], 6}, {NULL}};
const Segment SGI231197[] = {{&S98348[31], 3}, {NULL}};
const Segment SGI231198[] = {{&S98348[77], 6}, {NULL}};
const Segment SGI231199[] = {{&S98350[44], 18}, {NULL}};
const Segment SGI231200[] = {{&S98350[44], 30}, {NULL}};
const Segment SGI231201[] = {{&S98350[30], 6}, {NULL}};
const Segment SGI231202[] = {{&S98350[31], 3}, {NULL}};
const Segment SGI231203[] = {{&S98350[88], 6}, {NULL}};
const Segment SGI231204[] = {{&S98350[147], 10}, {NULL}};
const Segment SGI231205[] = {{&S98357[44], 10}, {NULL}};
const Segment SGI231206[] = {{&S98357[44], 22}, {NULL}};
const Segment SGI231207[] = {{&S98357[30], 6}, {NULL}};
const Segment SGI231208[] = {{&S98357[80], 6}, {NULL}};
const Segment SGI231209[] = {{&S98357[107], 13}, {NULL}};
const Segment SGI231210[] = {{&S98366[34], 7}, {NULL}};
const Segment SGI231211[] = {{&S98366[27], 6}, {NULL}};
const Segment SGI231212[] = {{&S98366[94], 9}, {NULL}};
const Segment SGI231213[] = {{&S98366[67], 6}, {NULL}};
const Segment SGI231214[] = {{&S88279[34], 11}, {NULL}};
const Segment SGI231215[] = {{&S88279[27], 6}, {NULL}};
const Segment SGI231216[] = {{&S88279[98], 9}, {NULL}};
const Segment SGI231217[] = {{&S88279[71], 6}, {NULL}};
const Segment SGI231218[] = {{&S98369[34], 10}, {NULL}};
const Segment SGI231219[] = {{&S98369[27], 6}, {NULL}};
const Segment SGI231220[] = {{&S98369[97], 9}, {NULL}};
const Segment SGI231221[] = {{&S98369[70], 6}, {NULL}};
const Segment SGI231222[] = {{&S88280[34], 10}, {NULL}};
const Segment SGI231223[] = {{&S88280[27], 6}, {NULL}};
const Segment SGI231224[] = {{&S88280[97], 9}, {NULL}};
const Segment SGI231225[] = {{&S88280[70], 6}, {NULL}};
const Segment SGI231226[] = {{&S88280[131], 8}, {NULL}};
const Segment SGI231227[] = {{&S88282[34], 9}, {NULL}};
const Segment SGI231228[] = {{&S88282[27], 6}, {NULL}};
const Segment SGI231229[] = {{&S88282[96], 9}, {NULL}};
const Segment SGI231230[] = {{&S98373[34], 7}, {NULL}};
const Segment SGI231231[] = {{&S88282[69], 6}, {NULL}};
const Segment SGI231232[] = {{&S98373[27], 6}, {NULL}};
const Segment SGI231233[] = {{&S98373[94], 9}, {NULL}};
const Segment SGI231234[] = {{&S98373[67], 6}, {NULL}};
const Segment SGI231235[] = {{&S98374[44], 10}, {NULL}};
const Segment SGI231236[] = {{&S98374[44], 22}, {NULL}};
const Segment SGI231237[] = {{&S98374[30], 6}, {NULL}};
const Segment SGI231238[] = {{&S98374[80], 6}, {NULL}};
const Segment SGI231239[] = {{&S89536[34], 11}, {NULL}};
const Segment SGI231240[] = {{&S98374[107], 13}, {NULL}};
const Segment SGI231241[] = {{&S89536[27], 6}, {NULL}};
const Segment SGI231242[] = {{&S89536[98], 9}, {NULL}};
const Segment SGI231243[] = {{&S89536[71], 6}, {NULL}};
const Segment SGI231244[] = {{&S98375[44], 10}, {NULL}};
const Segment SGI231245[] = {{&S98375[44], 22}, {NULL}};
const Segment SGI231246[] = {{&S98375[30], 6}, {NULL}};
const Segment SGI231247[] = {{&S98375[80], 6}, {NULL}};
const Segment SGI231248[] = {{&S98375[107], 13}, {NULL}};
const Segment SGI231249[] = {{&S98378[44], 14}, {NULL}};
const Segment SGI231250[] = {{&S98378[44], 26}, {NULL}};
const Segment SGI231251[] = {{&S98378[30], 6}, {NULL}};
const Segment SGI231252[] = {{&S98378[31], 3}, {NULL}};
const Segment SGI231253[] = {{&S98378[84], 5}, {NULL}};
const Segment SGI231254[] = {{&S98378[121], 14}, {NULL}};
const Segment SGI231255[] = {{&S98383[44], 9}, {NULL}};
const Segment SGI231256[] = {{&S98383[44], 21}, {NULL}};
const Segment SGI231257[] = {{&S98383[30], 6}, {NULL}};
const Segment SGI231258[] = {{&S98383[79], 6}, {NULL}};
const Segment SGI231259[] = {{&S98383[106], 13}, {NULL}};
const Segment SGI231260[] = {{&S98389[34], 5}, {NULL}};
const Segment SGI231261[] = {{&S98389[27], 6}, {NULL}};
const Segment SGI231262[] = {{&S98389[92], 9}, {NULL}};
const Segment SGI231263[] = {{&S98389[65], 6}, {NULL}};
const Segment SGI231264[] = {{&S98389[133], 8}, {NULL}};
const Segment SGI231265[] = {{&S98394[44], 10}, {NULL}};
const Segment SGI231266[] = {{&S98394[44], 22}, {NULL}};
const Segment SGI231267[] = {{&S98394[30], 6}, {NULL}};
const Segment SGI231268[] = {{&S98394[80], 6}, {NULL}};
const Segment SGI231269[] = {{&S98394[107], 13}, {NULL}};
const Segment SGI231270[] = {{&S98397[34], 11}, {NULL}};
const Segment SGI231271[] = {{&S98397[27], 6}, {NULL}};
const Segment SGI231272[] = {{&S98397[98], 9}, {NULL}};
const Segment SGI231273[] = {{&S98397[71], 6}, {NULL}};
const Segment SGI231274[] = {{&S98399[34], 16}, {NULL}};
const Segment SGI231275[] = {{&S98399[27], 6}, {NULL}};
const Segment SGI231276[] = {{&S98399[103], 9}, {NULL}};
const Segment SGI231277[] = {{&S98399[76], 6}, {NULL}};
const Segment SGI231278[] = {{&S98400[34], 11}, {NULL}};
const Segment SGI231279[] = {{&S98400[27], 6}, {NULL}};
const Segment SGI231280[] = {{&S98400[98], 9}, {NULL}};
const Segment SGI231281[] = {{&S98400[71], 6}, {NULL}};
const Segment SGI231282[] = {{&S98412[59], 11}, {NULL}};
const Segment SGI231283[] = {{&S98412[59], 23}, {NULL}};
const Segment SGI231284[] = {{&S98412[45], 6}, {NULL}};
const Segment SGI231285[] = {{&S98412[46], 3}, {NULL}};
const Segment SGI231286[] = {{&S98412[96], 6}, {NULL}};
const Segment SGI231287[] = {{&S98412[0], 14}, {NULL}};
const Segment SGI231288[] = {{&S98413[59], 4}, {NULL}};
const Segment SGI231289[] = {{&S98413[59], 16}, {NULL}};
const Segment SGI231290[] = {{&S98413[45], 6}, {NULL}};
const Segment SGI231291[] = {{&S98413[46], 3}, {NULL}};
const Segment SGI231292[] = {{&S98413[89], 6}, {NULL}};
const Segment SGI231293[] = {{&S98413[0], 14}, {NULL}};
const Segment SGI231294[] = {{&S88339[44], 12}, {NULL}};
const Segment SGI231295[] = {{&S88339[44], 24}, {NULL}};
const Segment SGI231296[] = {{&S88339[30], 6}, {NULL}};
const Segment SGI231297[] = {{&S88339[117], 3}, {NULL}};
const Segment SGI231298[] = {{&S88339[82], 6}, {NULL}};
const Segment SGI231299[] = {{&S98422[44], 10}, {NULL}};
const Segment SGI231300[] = {{&S98422[44], 22}, {NULL}};
const Segment SGI231301[] = {{&S98422[30], 6}, {NULL}};
const Segment SGI231302[] = {{&S98422[80], 6}, {NULL}};
const Segment SGI231303[] = {{&S98422[107], 13}, {NULL}};
const Segment SGI231304[] = {{&S95242[45], 11}, {NULL}};
const Segment SGI231305[] = {{&S95242[30], 6}, {NULL}};
const Segment SGI231306[] = {{&S95242[116], 3}, {NULL}};
const Segment SGI231307[] = {{&S95242[82], 5}, {NULL}};
const Segment SGI231308[] = {{&S95242[140], 13}, {NULL}};
const Segment SGI231309[] = {{&S98437[34], 10}, {NULL}};
const Segment SGI231310[] = {{&S98437[27], 6}, {NULL}};
const Segment SGI231311[] = {{&S98437[97], 9}, {NULL}};
const Segment SGI231312[] = {{&S98437[70], 6}, {NULL}};
const Segment SGI231313[] = {{&S98437[131], 8}, {NULL}};
const Segment SGI231314[] = {{&S95247[34], 9}, {NULL}};
const Segment SGI231315[] = {{&S95247[27], 6}, {NULL}};
const Segment SGI231316[] = {{&S95247[96], 9}, {NULL}};
const Segment SGI231317[] = {{&S95247[69], 6}, {NULL}};
const Segment SGI231318[] = {{&S80562[34], 7}, {NULL}};
const Segment SGI231319[] = {{&S80562[27], 6}, {NULL}};
const Segment SGI231320[] = {{&S80562[94], 9}, {NULL}};
const Segment SGI231321[] = {{&S80562[67], 6}, {NULL}};
const Segment SGI231322[] = {{&S98442[52], 14}, {NULL}};
const Segment SGI231323[] = {{&S95250[44], 4}, {NULL}};
const Segment SGI231324[] = {{&S98442[52], 26}, {NULL}};
const Segment SGI231325[] = {{&S95250[44], 16}, {NULL}};
const Segment SGI231326[] = {{&S98442[38], 6}, {NULL}};
const Segment SGI231327[] = {{&S95250[30], 6}, {NULL}};
const Segment SGI231328[] = {{&S95250[109], 3}, {NULL}};
const Segment SGI231329[] = {{&S98442[92], 5}, {NULL}};
const Segment SGI231330[] = {{&S95250[74], 6}, {NULL}};
const Segment SGI231331[] = {{&S98442[0], 14}, {NULL}};
const Segment SGI231332[] = {{&S88378[34], 10}, {NULL}};
const Segment SGI231333[] = {{&S88378[27], 6}, {NULL}};
const Segment SGI231334[] = {{&S88378[97], 9}, {NULL}};
const Segment SGI231335[] = {{&S88378[70], 6}, {NULL}};
const Segment SGI231336[] = {{&S98449[44], 14}, {NULL}};
const Segment SGI231337[] = {{&S98449[44], 26}, {NULL}};
const Segment SGI231338[] = {{&S98449[30], 6}, {NULL}};
const Segment SGI231339[] = {{&S98449[84], 6}, {NULL}};
const Segment SGI231340[] = {{&S98449[111], 13}, {NULL}};
const Segment SGI231341[] = {{&S88379[34], 11}, {NULL}};
const Segment SGI231342[] = {{&S88379[27], 6}, {NULL}};
const Segment SGI231343[] = {{&S88379[98], 9}, {NULL}};
const Segment SGI231344[] = {{&S88379[71], 6}, {NULL}};
const Segment SGI231345[] = {{&S98451[44], 14}, {NULL}};
const Segment SGI231346[] = {{&S98451[44], 31}, {NULL}};
const Segment SGI231347[] = {{&S98451[30], 6}, {NULL}};
const Segment SGI231348[] = {{&S98451[89], 6}, {NULL}};
const Segment SGI231349[] = {{&S98451[116], 13}, {NULL}};
const Segment SGI231350[] = {{&S98454[34], 16}, {NULL}};
const Segment SGI231351[] = {{&S98454[27], 6}, {NULL}};
const Segment SGI231352[] = {{&S98454[103], 9}, {NULL}};
const Segment SGI231353[] = {{&S98454[76], 6}, {NULL}};
const Segment SGI231354[] = {{&S88387[45], 17}, {NULL}};
const Segment SGI231355[] = {{&S88387[30], 6}, {NULL}};
const Segment SGI231356[] = {{&S88387[122], 3}, {NULL}};
const Segment SGI231357[] = {{&S88387[88], 5}, {NULL}};
const Segment SGI231358[] = {{&S88387[146], 13}, {NULL}};
const Segment SGI231359[] = {{&S98457[52], 13}, {NULL}};
const Segment SGI231360[] = {{&S98457[52], 29}, {NULL}};
const Segment SGI231361[] = {{&S98457[38], 6}, {NULL}};
const Segment SGI231362[] = {{&S98457[95], 5}, {NULL}};
const Segment SGI231363[] = {{&S98457[0], 14}, {NULL}};
const Segment SGI231364[] = {{&S98458[44], 14}, {NULL}};
const Segment SGI231365[] = {{&S98458[44], 26}, {NULL}};
const Segment SGI231366[] = {{&S98458[30], 6}, {NULL}};
const Segment SGI231367[] = {{&S98458[84], 6}, {NULL}};
const Segment SGI231368[] = {{&S98458[111], 13}, {NULL}};
const Segment SGI231369[] = {{&S98459[52], 6}, {NULL}};
const Segment SGI231370[] = {{&S98459[52], 18}, {NULL}};
const Segment SGI231371[] = {{&S98459[38], 6}, {NULL}};
const Segment SGI231372[] = {{&S95273[44], 9}, {NULL}};
const Segment SGI231373[] = {{&S98459[84], 5}, {NULL}};
const Segment SGI231374[] = {{&S95273[30], 6}, {NULL}};
const Segment SGI231375[] = {{&S98459[0], 14}, {NULL}};
const Segment SGI231376[] = {{&S95273[116], 3}, {NULL}};
const Segment SGI231377[] = {{&S95273[81], 6}, {NULL}};
const Segment SGI231378[] = {{&S88393[44], 8}, {NULL}};
const Segment SGI231379[] = {{&S88393[44], 19}, {NULL}};
const Segment SGI231380[] = {{&S88393[30], 6}, {NULL}};
const Segment SGI231381[] = {{&S88393[112], 3}, {NULL}};
const Segment SGI231382[] = {{&S88393[77], 6}, {NULL}};
const Segment SGI231383[] = {{&S88394[44], 9}, {NULL}};
const Segment SGI231384[] = {{&S88394[44], 21}, {NULL}};
const Segment SGI231385[] = {{&S88394[30], 6}, {NULL}};
const Segment SGI231386[] = {{&S88394[114], 3}, {NULL}};
const Segment SGI231387[] = {{&S88394[79], 6}, {NULL}};
const Segment SGI231388[] = {{&S98463[34], 11}, {NULL}};
const Segment SGI231389[] = {{&S95277[44], 7}, {NULL}};
const Segment SGI231390[] = {{&S98463[27], 6}, {NULL}};
const Segment SGI231391[] = {{&S95277[44], 19}, {NULL}};
const Segment SGI231392[] = {{&S95277[30], 6}, {NULL}};
const Segment SGI231393[] = {{&S98463[98], 9}, {NULL}};
const Segment SGI231394[] = {{&S95277[112], 3}, {NULL}};
const Segment SGI231395[] = {{&S98463[71], 6}, {NULL}};
const Segment SGI231396[] = {{&S95277[77], 6}, {NULL}};
const Segment SGI231397[] = {{&S98466[44], 18}, {NULL}};
const Segment SGI231398[] = {{&S98466[44], 30}, {NULL}};
const Segment SGI231399[] = {{&S98466[30], 6}, {NULL}};
const Segment SGI231400[] = {{&S98466[31], 3}, {NULL}};
const Segment SGI231401[] = {{&S98466[88], 5}, {NULL}};
const Segment SGI231402[] = {{&S98466[125], 14}, {NULL}};
const Segment SGI231403[] = {{&S98467[44], 10}, {NULL}};
const Segment SGI231404[] = {{&S98467[44], 22}, {NULL}};
const Segment SGI231405[] = {{&S98467[30], 6}, {NULL}};
const Segment SGI231406[] = {{&S88402[34], 10}, {NULL}};
const Segment SGI231407[] = {{&S98467[31], 3}, {NULL}};
const Segment SGI231408[] = {{&S88402[27], 6}, {NULL}};
const Segment SGI231409[] = {{&S98467[80], 5}, {NULL}};
const Segment SGI231410[] = {{&S98467[117], 14}, {NULL}};
const Segment SGI231411[] = {{&S88402[97], 9}, {NULL}};
const Segment SGI231412[] = {{&S88402[70], 6}, {NULL}};
const Segment SGI231413[] = {{&S98473[34], 9}, {NULL}};
const Segment SGI231414[] = {{&S98473[27], 6}, {NULL}};
const Segment SGI231415[] = {{&S98473[96], 9}, {NULL}};
const Segment SGI231416[] = {{&S98473[69], 6}, {NULL}};
const Segment SGI231417[] = {{&S95292[34], 4}, {NULL}};
const Segment SGI231418[] = {{&S95292[27], 6}, {NULL}};
const Segment SGI231419[] = {{&S98474[44], 5}, {NULL}};
const Segment SGI231420[] = {{&S98474[44], 17}, {NULL}};
const Segment SGI231421[] = {{&S98474[30], 6}, {NULL}};
const Segment SGI231422[] = {{&S95292[91], 9}, {NULL}};
const Segment SGI231423[] = {{&S95292[64], 6}, {NULL}};
const Segment SGI231424[] = {{&S98474[110], 3}, {NULL}};
const Segment SGI231425[] = {{&S98474[75], 6}, {NULL}};
const Segment SGI231426[] = {{&S98474[130], 8}, {NULL}};
const Segment SGI231427[] = {{&S98474[139], 13}, {NULL}};
const Segment SGI231428[] = {{&S95304[44], 11}, {NULL}};
const Segment SGI231429[] = {{&S95304[44], 23}, {NULL}};
const Segment SGI231430[] = {{&S95304[30], 6}, {NULL}};
const Segment SGI231431[] = {{&S95304[116], 3}, {NULL}};
const Segment SGI231432[] = {{&S95304[81], 6}, {NULL}};
const Segment SGI231433[] = {{&S80634[34], 10}, {NULL}};
const Segment SGI231434[] = {{&S80634[27], 6}, {NULL}};
const Segment SGI231435[] = {{&S80634[97], 9}, {NULL}};
const Segment SGI231436[] = {{&S80634[70], 6}, {NULL}};
const Segment SGI231437[] = {{&S98497[34], 7}, {NULL}};
const Segment SGI231438[] = {{&S98497[27], 6}, {NULL}};
const Segment SGI231439[] = {{&S98497[94], 9}, {NULL}};
const Segment SGI231440[] = {{&S98497[67], 6}, {NULL}};
const Segment SGI231441[] = {{&S80637[44], 10}, {NULL}};
const Segment SGI231442[] = {{&S80637[44], 21}, {NULL}};
const Segment SGI231443[] = {{&S80637[30], 6}, {NULL}};
const Segment SGI231444[] = {{&S80637[114], 3}, {NULL}};
const Segment SGI231445[] = {{&S98498[34], 14}, {NULL}};
const Segment SGI231446[] = {{&S80637[79], 6}, {NULL}};
const Segment SGI231447[] = {{&S95316[44], 6}, {NULL}};
const Segment SGI231448[] = {{&S98498[27], 6}, {NULL}};
const Segment SGI231449[] = {{&S95316[44], 18}, {NULL}};
const Segment SGI231450[] = {{&S80637[138], 10}, {NULL}};
const Segment SGI231451[] = {{&S95316[30], 6}, {NULL}};
const Segment SGI231452[] = {{&S98498[101], 9}, {NULL}};
const Segment SGI231453[] = {{&S95316[111], 3}, {NULL}};
const Segment SGI231454[] = {{&S98498[74], 6}, {NULL}};
const Segment SGI231455[] = {{&S95316[76], 6}, {NULL}};
const Segment SGI231456[] = {{&S98499[44], 13}, {NULL}};
const Segment SGI231457[] = {{&S98499[44], 25}, {NULL}};
const Segment SGI231458[] = {{&S98499[30], 6}, {NULL}};
const Segment SGI231459[] = {{&S80639[34], 11}, {NULL}};
const Segment SGI231460[] = {{&S80639[27], 6}, {NULL}};
const Segment SGI231461[] = {{&S98499[83], 6}, {NULL}};
const Segment SGI231462[] = {{&S98499[110], 13}, {NULL}};
const Segment SGI231463[] = {{&S80639[97], 9}, {NULL}};
const Segment SGI231464[] = {{&S80639[70], 6}, {NULL}};
const Segment SGI231465[] = {{&S80639[138], 8}, {NULL}};
const Segment SGI231466[] = {{&S95321[44], 16}, {NULL}};
const Segment SGI231467[] = {{&S95321[44], 28}, {NULL}};
const Segment SGI231468[] = {{&S95321[30], 6}, {NULL}};
const Segment SGI231469[] = {{&S95321[86], 6}, {NULL}};
const Segment SGI231470[] = {{&S95321[113], 13}, {NULL}};
const Segment SGI231471[] = {{&S88448[38], 10}, {NULL}};
const Segment SGI231472[] = {{&S88448[31], 6}, {NULL}};
const Segment SGI231473[] = {{&S98505[34], 10}, {NULL}};
const Segment SGI231474[] = {{&S98505[27], 6}, {NULL}};
const Segment SGI231475[] = {{&S98505[97], 9}, {NULL}};
const Segment SGI231476[] = {{&S98505[70], 6}, {NULL}};
const Segment SGI231477[] = {{&S98506[34], 7}, {NULL}};
const Segment SGI231478[] = {{&S98506[27], 6}, {NULL}};
const Segment SGI231479[] = {{&S98506[96], 9}, {NULL}};
const Segment SGI231480[] = {{&S98506[69], 6}, {NULL}};
const Segment SGI231481[] = {{&S95329[34], 21}, {NULL}};
const Segment SGI231482[] = {{&S88454[34], 6}, {NULL}};
const Segment SGI231483[] = {{&S88454[27], 6}, {NULL}};
const Segment SGI231484[] = {{&S95329[27], 6}, {NULL}};
const Segment SGI231485[] = {{&S88454[93], 9}, {NULL}};
const Segment SGI231486[] = {{&S95329[108], 9}, {NULL}};
const Segment SGI231487[] = {{&S88454[66], 6}, {NULL}};
const Segment SGI231488[] = {{&S95329[81], 6}, {NULL}};
const Segment SGI231489[] = {{&S88455[44], 12}, {NULL}};
const Segment SGI231490[] = {{&S88455[44], 24}, {NULL}};
const Segment SGI231491[] = {{&S88455[30], 6}, {NULL}};
const Segment SGI231492[] = {{&S88455[133], 3}, {NULL}};
const Segment SGI231493[] = {{&S88455[98], 6}, {NULL}};
const Segment SGI231494[] = {{&S88455[70], 8}, {NULL}};
const Segment SGI231495[] = {{&S95333[34], 13}, {NULL}};
const Segment SGI231496[] = {{&S95333[27], 6}, {NULL}};
const Segment SGI231497[] = {{&S95333[100], 9}, {NULL}};
const Segment SGI231498[] = {{&S95333[73], 6}, {NULL}};
const Segment SGI231499[] = {{&S95335[34], 22}, {NULL}};
const Segment SGI231500[] = {{&S95335[27], 6}, {NULL}};
const Segment SGI231501[] = {{&S95335[109], 9}, {NULL}};
const Segment SGI231502[] = {{&S95335[82], 6}, {NULL}};
const Segment SGI231503[] = {{&S88462[44], 9}, {NULL}};
const Segment SGI231504[] = {{&S88462[44], 21}, {NULL}};
const Segment SGI231505[] = {{&S88462[30], 6}, {NULL}};
const Segment SGI231506[] = {{&S88462[130], 3}, {NULL}};
const Segment SGI231507[] = {{&S88462[95], 6}, {NULL}};
const Segment SGI231508[] = {{&S88462[67], 8}, {NULL}};
const Segment SGI231509[] = {{&S88465[44], 7}, {NULL}};
const Segment SGI231510[] = {{&S88465[44], 22}, {NULL}};
const Segment SGI231511[] = {{&S88465[30], 6}, {NULL}};
const Segment SGI231512[] = {{&S88465[115], 3}, {NULL}};
const Segment SGI231513[] = {{&S88465[80], 6}, {NULL}};
const Segment SGI231514[] = {{&S88465[140], 11}, {NULL}};
const Segment SGI231515[] = {{&S88471[34], 7}, {NULL}};
const Segment SGI231516[] = {{&S88471[27], 6}, {NULL}};
const Segment SGI231517[] = {{&S88471[94], 9}, {NULL}};
const Segment SGI231518[] = {{&S88471[67], 6}, {NULL}};
const Segment SGI231519[] = {{&S88472[44], 10}, {NULL}};
const Segment SGI231520[] = {{&S88472[44], 22}, {NULL}};
const Segment SGI231521[] = {{&S88472[30], 6}, {NULL}};
const Segment SGI231522[] = {{&S88472[115], 3}, {NULL}};
const Segment SGI231523[] = {{&S88472[80], 6}, {NULL}};
const Segment SGI231524[] = {{&S88472[133], 11}, {NULL}};
const Segment SGI231525[] = {{&S88479[44], 10}, {NULL}};
const Segment SGI231526[] = {{&S88479[44], 20}, {NULL}};
const Segment SGI231527[] = {{&S88479[30], 6}, {NULL}};
const Segment SGI231528[] = {{&S88479[113], 3}, {NULL}};
const Segment SGI231529[] = {{&S88479[78], 6}, {NULL}};
const Segment SGI231530[] = {{&S88488[44], 9}, {NULL}};
const Segment SGI231531[] = {{&S88488[44], 21}, {NULL}};
const Segment SGI231532[] = {{&S88488[30], 6}, {NULL}};
const Segment SGI231533[] = {{&S88488[114], 3}, {NULL}};
const Segment SGI231534[] = {{&S88488[79], 6}, {NULL}};
const Segment SGI231535[] = {{&S88491[34], 8}, {NULL}};
const Segment SGI231536[] = {{&S88491[27], 6}, {NULL}};
const Segment SGI231537[] = {{&S88491[103], 9}, {NULL}};
const Segment SGI231538[] = {{&S88491[76], 6}, {NULL}};
const Segment SGI231539[] = {{&S88498[34], 10}, {NULL}};
const Segment SGI231540[] = {{&S88498[27], 6}, {NULL}};
const Segment SGI231541[] = {{&S88498[97], 9}, {NULL}};
const Segment SGI231542[] = {{&S88498[70], 6}, {NULL}};
const Segment SGI231543[] = {{&S88499[34], 10}, {NULL}};
const Segment SGI231544[] = {{&S88499[27], 6}, {NULL}};
const Segment SGI231545[] = {{&S88499[97], 9}, {NULL}};
const Segment SGI231546[] = {{&S88499[70], 6}, {NULL}};
const Segment SGI231547[] = {{&S88500[34], 8}, {NULL}};
const Segment SGI231548[] = {{&S88500[27], 6}, {NULL}};
const Segment SGI231549[] = {{&S88500[95], 9}, {NULL}};
const Segment SGI231550[] = {{&S88500[68], 6}, {NULL}};
const Segment SGI231551[] = {{&S88501[44], 11}, {NULL}};
const Segment SGI231552[] = {{&S88501[44], 23}, {NULL}};
const Segment SGI231553[] = {{&S88501[30], 6}, {NULL}};
const Segment SGI231554[] = {{&S88501[132], 3}, {NULL}};
const Segment SGI231555[] = {{&S88501[97], 6}, {NULL}};
const Segment SGI231556[] = {{&S88501[69], 8}, {NULL}};
const Segment SGI231557[] = {{&S89649[34], 6}, {NULL}};
const Segment SGI231558[] = {{&S89649[27], 6}, {NULL}};
const Segment SGI231559[] = {{&S89649[93], 9}, {NULL}};
const Segment SGI231560[] = {{&S89649[66], 6}, {NULL}};
const Segment SGI231561[] = {{&S80711[44], 12}, {NULL}};
const Segment SGI231562[] = {{&S80711[44], 44}, {NULL}};
const Segment SGI231563[] = {{&S80711[30], 6}, {NULL}};
const Segment SGI231564[] = {{&S80711[137], 3}, {NULL}};
const Segment SGI231565[] = {{&S80711[102], 6}, {NULL}};
const Segment SGI231566[] = {{&S80712[44], 12}, {NULL}};
const Segment SGI231567[] = {{&S80712[44], 44}, {NULL}};
const Segment SGI231568[] = {{&S80712[30], 6}, {NULL}};
const Segment SGI231569[] = {{&S80712[137], 3}, {NULL}};
const Segment SGI231570[] = {{&S80712[102], 6}, {NULL}};
const Segment SGI231571[] = {{&S88512[34], 10}, {NULL}};
const Segment SGI231572[] = {{&S88512[27], 6}, {NULL}};
const Segment SGI231573[] = {{&S88512[97], 9}, {NULL}};
const Segment SGI231574[] = {{&S88512[70], 6}, {NULL}};
const Segment SGI231575[] = {{&S80720[34], 10}, {NULL}};
const Segment SGI231576[] = {{&S80720[27], 6}, {NULL}};
const Segment SGI231577[] = {{&S80720[97], 9}, {NULL}};
const Segment SGI231578[] = {{&S80720[70], 6}, {NULL}};
const Segment SGI231579[] = {{&S89670[34], 7}, {NULL}};
const Segment SGI231580[] = {{&S89670[27], 6}, {NULL}};
const Segment SGI231581[] = {{&S89670[94], 9}, {NULL}};
const Segment SGI231582[] = {{&S89670[67], 6}, {NULL}};
const Segment SGI231583[] = {{&S88530[44], 10}, {NULL}};
const Segment SGI231584[] = {{&S88530[44], 22}, {NULL}};
const Segment SGI231585[] = {{&S88530[30], 6}, {NULL}};
const Segment SGI231586[] = {{&S88530[115], 3}, {NULL}};
const Segment SGI231587[] = {{&S88530[80], 6}, {NULL}};
const Segment SGI231588[] = {{&S88530[140], 7}, {NULL}};
const Segment SGI231589[] = {{&S88531[44], 10}, {NULL}};
const Segment SGI231590[] = {{&S88531[44], 22}, {NULL}};
const Segment SGI231591[] = {{&S88531[30], 6}, {NULL}};
const Segment SGI231592[] = {{&S88531[115], 3}, {NULL}};
const Segment SGI231593[] = {{&S88531[80], 6}, {NULL}};
const Segment SGI231594[] = {{&S88531[140], 13}, {NULL}};
const Segment SGI231595[] = {{&S98613[59], 10}, {NULL}};
const Segment SGI231596[] = {{&S98613[59], 22}, {NULL}};
const Segment SGI231597[] = {{&S98613[45], 6}, {NULL}};
const Segment SGI231598[] = {{&S98613[46], 3}, {NULL}};
const Segment SGI231599[] = {{&S98613[95], 6}, {NULL}};
const Segment SGI231600[] = {{&S98613[0], 14}, {NULL}};
const Segment SGI231601[] = {{&S89683[44], 14}, {NULL}};
const Segment SGI231602[] = {{&S89683[44], 26}, {NULL}};
const Segment SGI231603[] = {{&S89683[30], 6}, {NULL}};
const Segment SGI231604[] = {{&S89683[84], 6}, {NULL}};
const Segment SGI231605[] = {{&S89683[121], 3}, {NULL}};
const Segment SGI231606[] = {{&S88550[34], 12}, {NULL}};
const Segment SGI231607[] = {{&S88550[27], 6}, {NULL}};
const Segment SGI231608[] = {{&S88550[99], 9}, {NULL}};
const Segment SGI231609[] = {{&S88550[72], 6}, {NULL}};
const Segment SGI231610[] = {{&S88550[140], 8}, {NULL}};
const Segment SGI231611[] = {{&S88551[34], 11}, {NULL}};
const Segment SGI231612[] = {{&S88551[27], 6}, {NULL}};
const Segment SGI231613[] = {{&S88551[98], 9}, {NULL}};
const Segment SGI231614[] = {{&S88551[71], 6}, {NULL}};
const Segment SGI231615[] = {{&S88552[34], 17}, {NULL}};
const Segment SGI231616[] = {{&S88552[27], 6}, {NULL}};
const Segment SGI231617[] = {{&S88552[104], 9}, {NULL}};
const Segment SGI231618[] = {{&S88552[77], 6}, {NULL}};
const Segment SGI231619[] = {{&S98633[45], 7}, {NULL}};
const Segment SGI231620[] = {{&S98633[30], 6}, {NULL}};
const Segment SGI231621[] = {{&S98633[31], 3}, {NULL}};
const Segment SGI231622[] = {{&S98633[83], 5}, {NULL}};
const Segment SGI231623[] = {{&S88553[34], 18}, {NULL}};
const Segment SGI231624[] = {{&S98633[141], 13}, {NULL}};
const Segment SGI231625[] = {{&S88553[27], 6}, {NULL}};
const Segment SGI231626[] = {{&S88553[105], 9}, {NULL}};
const Segment SGI231627[] = {{&S88553[78], 6}, {NULL}};
const Segment SGI231628[] = {{&S88555[44], 13}, {NULL}};
const Segment SGI231629[] = {{&S88555[44], 25}, {NULL}};
const Segment SGI231630[] = {{&S88555[30], 6}, {NULL}};
const Segment SGI231631[] = {{&S88555[83], 6}, {NULL}};
const Segment SGI231632[] = {{&S88555[110], 13}, {NULL}};
const Segment SGI231633[] = {{&S80770[41], 10}, {NULL}};
const Segment SGI231634[] = {{&S80770[41], 21}, {NULL}};
const Segment SGI231635[] = {{&S80770[27], 6}, {NULL}};
const Segment SGI231636[] = {{&S80770[111], 3}, {NULL}};
const Segment SGI231637[] = {{&S80770[117], 9}, {NULL}};
const Segment SGI231638[] = {{&S80770[76], 6}, {NULL}};
const Segment SGI231639[] = {{&S80771[41], 18}, {NULL}};
const Segment SGI231640[] = {{&S80771[41], 29}, {NULL}};
const Segment SGI231641[] = {{&S80771[27], 6}, {NULL}};
const Segment SGI231642[] = {{&S80771[119], 3}, {NULL}};
const Segment SGI231643[] = {{&S80771[125], 9}, {NULL}};
const Segment SGI231644[] = {{&S80771[84], 6}, {NULL}};
const Segment SGI231645[] = {{&S95461[59], 10}, {NULL}};
const Segment SGI231646[] = {{&S95461[59], 22}, {NULL}};
const Segment SGI231647[] = {{&S95461[45], 6}, {NULL}};
const Segment SGI231648[] = {{&S95461[130], 3}, {NULL}};
const Segment SGI231649[] = {{&S95461[95], 6}, {NULL}};
const Segment SGI231650[] = {{&S95461[0], 14}, {NULL}};
const Segment SGI231651[] = {{&S88562[34], 10}, {NULL}};
const Segment SGI231652[] = {{&S88562[27], 6}, {NULL}};
const Segment SGI231653[] = {{&S88562[97], 9}, {NULL}};
const Segment SGI231654[] = {{&S88562[70], 6}, {NULL}};
const Segment SGI231655[] = {{&S98644[44], 10}, {NULL}};
const Segment SGI231656[] = {{&S98644[44], 22}, {NULL}};
const Segment SGI231657[] = {{&S98644[30], 6}, {NULL}};
const Segment SGI231658[] = {{&S98644[80], 6}, {NULL}};
const Segment SGI231659[] = {{&S98644[107], 13}, {NULL}};
const Segment SGI231660[] = {{&S98645[44], 10}, {NULL}};
const Segment SGI231661[] = {{&S98645[44], 22}, {NULL}};
const Segment SGI231662[] = {{&S98645[30], 6}, {NULL}};
const Segment SGI231663[] = {{&S98645[31], 3}, {NULL}};
const Segment SGI231664[] = {{&S98645[80], 6}, {NULL}};
const Segment SGI231665[] = {{&S98645[133], 11}, {NULL}};
const Segment SGI231666[] = {{&S95470[44], 6}, {NULL}};
const Segment SGI231667[] = {{&S95470[44], 18}, {NULL}};
const Segment SGI231668[] = {{&S95470[30], 6}, {NULL}};
const Segment SGI231669[] = {{&S95470[111], 3}, {NULL}};
const Segment SGI231670[] = {{&S95470[76], 6}, {NULL}};
const Segment SGI231671[] = {{&S80784[44], 8}, {NULL}};
const Segment SGI231672[] = {{&S80784[44], 19}, {NULL}};
const Segment SGI231673[] = {{&S80784[30], 6}, {NULL}};
const Segment SGI231674[] = {{&S80784[112], 3}, {NULL}};
const Segment SGI231675[] = {{&S80784[77], 6}, {NULL}};
const Segment SGI231676[] = {{&S80785[44], 8}, {NULL}};
const Segment SGI231677[] = {{&S80785[44], 19}, {NULL}};
const Segment SGI231678[] = {{&S80785[30], 6}, {NULL}};
const Segment SGI231679[] = {{&S80785[112], 3}, {NULL}};
const Segment SGI231680[] = {{&S80785[77], 6}, {NULL}};
const Segment SGI231681[] = {{&S98652[31], 5}, {NULL}};
const Segment SGI231682[] = {{&S98652[51], 3}, {NULL}};
const Segment SGI231683[] = {{&S98657[34], 11}, {NULL}};
const Segment SGI231684[] = {{&S98657[27], 6}, {NULL}};
const Segment SGI231685[] = {{&S98657[96], 9}, {NULL}};
const Segment SGI231686[] = {{&S98657[69], 6}, {NULL}};
const Segment SGI231687[] = {{&S95482[34], 25}, {NULL}};
const Segment SGI231688[] = {{&S95482[27], 6}, {NULL}};
const Segment SGI231689[] = {{&S95482[112], 9}, {NULL}};
const Segment SGI231690[] = {{&S95482[85], 6}, {NULL}};
const Segment SGI231691[] = {{&S98659[44], 10}, {NULL}};
const Segment SGI231692[] = {{&S98659[44], 22}, {NULL}};
const Segment SGI231693[] = {{&S98659[30], 6}, {NULL}};
const Segment SGI231694[] = {{&S98659[31], 3}, {NULL}};
const Segment SGI231695[] = {{&S98659[80], 5}, {NULL}};
const Segment SGI231696[] = {{&S98659[125], 14}, {NULL}};
const Segment SGI231697[] = {{&S98670[45], 7}, {NULL}};
const Segment SGI231698[] = {{&S98670[30], 6}, {NULL}};
const Segment SGI231699[] = {{&S98670[31], 3}, {NULL}};
const Segment SGI231700[] = {{&S98670[83], 5}, {NULL}};
const Segment SGI231701[] = {{&S98670[141], 13}, {NULL}};
const Segment SGI231702[] = {{&S95499[34], 15}, {NULL}};
const Segment SGI231703[] = {{&S95499[27], 6}, {NULL}};
const Segment SGI231704[] = {{&S95499[102], 9}, {NULL}};
const Segment SGI231705[] = {{&S95499[75], 6}, {NULL}};
const Segment SGI231706[] = {{&S98676[34], 11}, {NULL}};
const Segment SGI231707[] = {{&S98676[27], 6}, {NULL}};
const Segment SGI231708[] = {{&S98676[98], 9}, {NULL}};
const Segment SGI231709[] = {{&S98676[71], 6}, {NULL}};
const Segment SGI231710[] = {{&S98679[34], 12}, {NULL}};
const Segment SGI231711[] = {{&S98679[27], 6}, {NULL}};
const Segment SGI231712[] = {{&S98679[99], 9}, {NULL}};
const Segment SGI231713[] = {{&S98679[72], 6}, {NULL}};
const Segment SGI231714[] = {{&S98680[34], 11}, {NULL}};
const Segment SGI231715[] = {{&S98680[27], 6}, {NULL}};
const Segment SGI231716[] = {{&S98680[98], 9}, {NULL}};
const Segment SGI231717[] = {{&S98680[71], 6}, {NULL}};
const Segment SGI231718[] = {{&S98682[45], 11}, {NULL}};
const Segment SGI231719[] = {{&S98682[30], 6}, {NULL}};
const Segment SGI231720[] = {{&S98682[31], 3}, {NULL}};
const Segment SGI231721[] = {{&S98682[82], 5}, {NULL}};
const Segment SGI231722[] = {{&S98682[140], 13}, {NULL}};
const Segment SGI231723[] = {{&S98683[34], 12}, {NULL}};
const Segment SGI231724[] = {{&S98683[27], 6}, {NULL}};
const Segment SGI231725[] = {{&S98683[97], 9}, {NULL}};
const Segment SGI231726[] = {{&S98683[70], 6}, {NULL}};
const Segment SGI231727[] = {{&S98684[34], 10}, {NULL}};
const Segment SGI231728[] = {{&S98684[27], 6}, {NULL}};
const Segment SGI231729[] = {{&S98684[97], 9}, {NULL}};
const Segment SGI231730[] = {{&S98684[70], 6}, {NULL}};
const Segment SGI231731[] = {{&S95517[44], 7}, {NULL}};
const Segment SGI231732[] = {{&S95517[44], 19}, {NULL}};
const Segment SGI231733[] = {{&S95517[30], 6}, {NULL}};
const Segment SGI231734[] = {{&S95517[128], 3}, {NULL}};
const Segment SGI231735[] = {{&S95517[93], 6}, {NULL}};
const Segment SGI231736[] = {{&S95517[65], 8}, {NULL}};
const Segment SGI231737[] = {{&S98685[30], 6}, {NULL}};
const Segment SGI231738[] = {{&S98685[31], 3}, {NULL}};
const Segment SGI231739[] = {{&S98685[95], 5}, {NULL}};
const Segment SGI231740[] = {{&S98685[153], 13}, {NULL}};
const Segment SGI231741[] = {{&S95524[34], 15}, {NULL}};
const Segment SGI231742[] = {{&S95524[27], 6}, {NULL}};
const Segment SGI231743[] = {{&S95524[102], 9}, {NULL}};
const Segment SGI231744[] = {{&S95524[75], 6}, {NULL}};
const Segment SGI231745[] = {{&S98693[34], 25}, {NULL}};
const Segment SGI231746[] = {{&S98693[27], 6}, {NULL}};
const Segment SGI231747[] = {{&S98693[112], 9}, {NULL}};
const Segment SGI231748[] = {{&S98693[85], 6}, {NULL}};
const Segment SGI231749[] = {{&S95529[44], 11}, {NULL}};
const Segment SGI231750[] = {{&S95529[44], 23}, {NULL}};
const Segment SGI231751[] = {{&S95529[30], 6}, {NULL}};
const Segment SGI231752[] = {{&S95529[116], 3}, {NULL}};
const Segment SGI231753[] = {{&S95529[81], 6}, {NULL}};
const Segment SGI231754[] = {{&S98700[34], 10}, {NULL}};
const Segment SGI231755[] = {{&S98700[27], 6}, {NULL}};
const Segment SGI231756[] = {{&S98700[97], 9}, {NULL}};
const Segment SGI231757[] = {{&S98700[70], 6}, {NULL}};
const Segment SGI231758[] = {{&S88646[34], 17}, {NULL}};
const Segment SGI231759[] = {{&S88646[27], 6}, {NULL}};
const Segment SGI231760[] = {{&S88646[104], 9}, {NULL}};
const Segment SGI231761[] = {{&S88646[77], 6}, {NULL}};
const Segment SGI231762[] = {{&S88647[34], 18}, {NULL}};
const Segment SGI231763[] = {{&S88647[27], 6}, {NULL}};
const Segment SGI231764[] = {{&S95537[34], 17}, {NULL}};
const Segment SGI231765[] = {{&S95537[27], 6}, {NULL}};
const Segment SGI231766[] = {{&S88647[105], 9}, {NULL}};
const Segment SGI231767[] = {{&S88647[78], 6}, {NULL}};
const Segment SGI231768[] = {{&S95537[104], 9}, {NULL}};
const Segment SGI231769[] = {{&S95537[77], 6}, {NULL}};
const Segment SGI231770[] = {{&S88648[34], 11}, {NULL}};
const Segment SGI231771[] = {{&S88648[27], 6}, {NULL}};
const Segment SGI231772[] = {{&S88648[98], 9}, {NULL}};
const Segment SGI231773[] = {{&S88648[71], 6}, {NULL}};
const Segment SGI231774[] = {{&S88648[139], 8}, {NULL}};
const Segment SGI231775[] = {{&S89810[34], 7}, {NULL}};
const Segment SGI231776[] = {{&S89810[27], 6}, {NULL}};
const Segment SGI231777[] = {{&S89810[94], 9}, {NULL}};
const Segment SGI231778[] = {{&S89810[67], 6}, {NULL}};
const Segment SGI231779[] = {{&S98709[34], 10}, {NULL}};
const Segment SGI231780[] = {{&S98709[27], 6}, {NULL}};
const Segment SGI231781[] = {{&S88652[44], 18}, {NULL}};
const Segment SGI231782[] = {{&S88652[44], 30}, {NULL}};
const Segment SGI231783[] = {{&S88652[30], 6}, {NULL}};
const Segment SGI231784[] = {{&S98709[97], 9}, {NULL}};
const Segment SGI231785[] = {{&S98709[70], 6}, {NULL}};
const Segment SGI231786[] = {{&S88652[123], 3}, {NULL}};
const Segment SGI231787[] = {{&S88652[88], 6}, {NULL}};
const Segment SGI231788[] = {{&S88653[44], 10}, {NULL}};
const Segment SGI231789[] = {{&S88653[44], 22}, {NULL}};
const Segment SGI231790[] = {{&S88653[30], 6}, {NULL}};
const Segment SGI231791[] = {{&S88653[115], 3}, {NULL}};
const Segment SGI231792[] = {{&S88653[80], 6}, {NULL}};
const Segment SGI231793[] = {{&S88655[59], 17}, {NULL}};
const Segment SGI231794[] = {{&S88655[59], 29}, {NULL}};
const Segment SGI231795[] = {{&S88655[45], 6}, {NULL}};
const Segment SGI231796[] = {{&S88655[137], 3}, {NULL}};
const Segment SGI231797[] = {{&S88655[102], 6}, {NULL}};
const Segment SGI231798[] = {{&S88655[0], 14}, {NULL}};
const Segment SGI231799[] = {{&S95549[34], 11}, {NULL}};
const Segment SGI231800[] = {{&S95549[27], 6}, {NULL}};
const Segment SGI231801[] = {{&S95549[98], 9}, {NULL}};
const Segment SGI231802[] = {{&S95549[71], 6}, {NULL}};
const Segment SGI231803[] = {{&S88656[59], 18}, {NULL}};
const Segment SGI231804[] = {{&S88656[59], 30}, {NULL}};
const Segment SGI231805[] = {{&S88656[45], 6}, {NULL}};
const Segment SGI231806[] = {{&S88656[138], 3}, {NULL}};
const Segment SGI231807[] = {{&S88656[103], 6}, {NULL}};
const Segment SGI231808[] = {{&S98716[45], 10}, {NULL}};
const Segment SGI231809[] = {{&S88656[0], 14}, {NULL}};
const Segment SGI231810[] = {{&S98716[30], 6}, {NULL}};
const Segment SGI231811[] = {{&S98716[31], 3}, {NULL}};
const Segment SGI231812[] = {{&S98716[81], 5}, {NULL}};
const Segment SGI231813[] = {{&S98716[139], 13}, {NULL}};
const Segment SGI231814[] = {{&S95552[34], 10}, {NULL}};
const Segment SGI231815[] = {{&S95552[27], 6}, {NULL}};
const Segment SGI231816[] = {{&S95552[97], 9}, {NULL}};
const Segment SGI231817[] = {{&S95552[70], 6}, {NULL}};
const Segment SGI231818[] = {{&S88659[34], 10}, {NULL}};
const Segment SGI231819[] = {{&S88659[27], 6}, {NULL}};
const Segment SGI231820[] = {{&S88659[97], 9}, {NULL}};
const Segment SGI231821[] = {{&S88659[70], 6}, {NULL}};
const Segment SGI231822[] = {{&S95554[34], 18}, {NULL}};
const Segment SGI231823[] = {{&S88660[34], 9}, {NULL}};
const Segment SGI231824[] = {{&S95554[27], 6}, {NULL}};
const Segment SGI231825[] = {{&S88660[27], 6}, {NULL}};
const Segment SGI231826[] = {{&S95554[105], 9}, {NULL}};
const Segment SGI231827[] = {{&S88660[96], 9}, {NULL}};
const Segment SGI231828[] = {{&S95554[78], 6}, {NULL}};
const Segment SGI231829[] = {{&S88660[69], 6}, {NULL}};
const Segment SGI231830[] = {{&S89828[44], 19}, {NULL}};
const Segment SGI231831[] = {{&S89828[44], 31}, {NULL}};
const Segment SGI231832[] = {{&S89828[30], 6}, {NULL}};
const Segment SGI231833[] = {{&S89828[124], 3}, {NULL}};
const Segment SGI231834[] = {{&S89828[89], 6}, {NULL}};
const Segment SGI231835[] = {{&S98725[34], 20}, {NULL}};
const Segment SGI231836[] = {{&S98725[27], 6}, {NULL}};
const Segment SGI231837[] = {{&S98725[107], 9}, {NULL}};
const Segment SGI231838[] = {{&S98725[80], 6}, {NULL}};
const Segment SGI231839[] = {{&S95559[34], 6}, {NULL}};
const Segment SGI231840[] = {{&S95559[27], 6}, {NULL}};
const Segment SGI231841[] = {{&S95559[93], 9}, {NULL}};
const Segment SGI231842[] = {{&S95559[66], 6}, {NULL}};
const Segment SGI231843[] = {{&S95559[134], 8}, {NULL}};
const Segment SGI231844[] = {{&S88668[52], 10}, {NULL}};
const Segment SGI231845[] = {{&S88668[52], 22}, {NULL}};
const Segment SGI231846[] = {{&S88668[38], 6}, {NULL}};
const Segment SGI231847[] = {{&S88668[88], 5}, {NULL}};
const Segment SGI231848[] = {{&S88668[11], 3}, {NULL}};
const Segment SGI231849[] = {{&S98732[44], 14}, {NULL}};
const Segment SGI231850[] = {{&S98732[44], 31}, {NULL}};
const Segment SGI231851[] = {{&S98732[30], 6}, {NULL}};
const Segment SGI231852[] = {{&S88672[34], 9}, {NULL}};
const Segment SGI231853[] = {{&S88672[27], 6}, {NULL}};
const Segment SGI231854[] = {{&S98732[89], 6}, {NULL}};
const Segment SGI231855[] = {{&S88672[96], 9}, {NULL}};
const Segment SGI231856[] = {{&S88672[69], 6}, {NULL}};
const Segment SGI231857[] = {{&S98732[116], 13}, {NULL}};
const Segment SGI231858[] = {{&S89840[44], 10}, {NULL}};
const Segment SGI231859[] = {{&S89840[44], 24}, {NULL}};
const Segment SGI231860[] = {{&S89840[30], 6}, {NULL}};
const Segment SGI231861[] = {{&S89840[82], 6}, {NULL}};
const Segment SGI231862[] = {{&S89840[119], 3}, {NULL}};
const Segment SGI231863[] = {{&S98737[34], 7}, {NULL}};
const Segment SGI231864[] = {{&S98737[27], 6}, {NULL}};
const Segment SGI231865[] = {{&S95572[34], 12}, {NULL}};
const Segment SGI231866[] = {{&S98737[99], 9}, {NULL}};
const Segment SGI231867[] = {{&S98737[72], 6}, {NULL}};
const Segment SGI231868[] = {{&S95572[27], 6}, {NULL}};
const Segment SGI231869[] = {{&S95572[99], 9}, {NULL}};
const Segment SGI231870[] = {{&S95572[72], 6}, {NULL}};
const Segment SGI231871[] = {{&S98738[34], 14}, {NULL}};
const Segment SGI231872[] = {{&S98738[27], 6}, {NULL}};
const Segment SGI231873[] = {{&S98738[112], 9}, {NULL}};
const Segment SGI231874[] = {{&S98738[85], 6}, {NULL}};
const Segment SGI231875[] = {{&S95574[44], 18}, {NULL}};
const Segment SGI231876[] = {{&S95574[44], 30}, {NULL}};
const Segment SGI231877[] = {{&S95574[30], 6}, {NULL}};
const Segment SGI231878[] = {{&S95574[123], 3}, {NULL}};
const Segment SGI231879[] = {{&S98739[34], 6}, {NULL}};
const Segment SGI231880[] = {{&S95574[88], 6}, {NULL}};
const Segment SGI231881[] = {{&S98739[27], 6}, {NULL}};
const Segment SGI231882[] = {{&S88683[44], 13}, {NULL}};
const Segment SGI231883[] = {{&S98739[93], 9}, {NULL}};
const Segment SGI231884[] = {{&S88683[44], 25}, {NULL}};
const Segment SGI231885[] = {{&S98739[66], 6}, {NULL}};
const Segment SGI231886[] = {{&S88683[30], 6}, {NULL}};
const Segment SGI231887[] = {{&S88683[83], 6}, {NULL}};
const Segment SGI231888[] = {{&S88683[110], 13}, {NULL}};
const Segment SGI231889[] = {{&S88684[44], 15}, {NULL}};
const Segment SGI231890[] = {{&S88684[44], 27}, {NULL}};
const Segment SGI231891[] = {{&S88684[30], 6}, {NULL}};
const Segment SGI231892[] = {{&S98741[34], 12}, {NULL}};
const Segment SGI231893[] = {{&S98741[27], 6}, {NULL}};
const Segment SGI231894[] = {{&S88684[85], 6}, {NULL}};
const Segment SGI231895[] = {{&S88684[112], 13}, {NULL}};
const Segment SGI231896[] = {{&S98741[99], 9}, {NULL}};
const Segment SGI231897[] = {{&S98741[72], 6}, {NULL}};
const Segment SGI231898[] = {{&S88687[34], 7}, {NULL}};
const Segment SGI231899[] = {{&S88687[27], 6}, {NULL}};
const Segment SGI231900[] = {{&S98743[34], 7}, {NULL}};
const Segment SGI231901[] = {{&S88687[94], 9}, {NULL}};
const Segment SGI231902[] = {{&S88687[67], 6}, {NULL}};
const Segment SGI231903[] = {{&S98743[27], 6}, {NULL}};
const Segment SGI231904[] = {{&S98743[94], 9}, {NULL}};
const Segment SGI231905[] = {{&S98743[67], 6}, {NULL}};
const Segment SGI231906[] = {{&S88688[34], 9}, {NULL}};
const Segment SGI231907[] = {{&S88688[27], 6}, {NULL}};
const Segment SGI231908[] = {{&S88688[101], 9}, {NULL}};
const Segment SGI231909[] = {{&S98744[34], 11}, {NULL}};
const Segment SGI231910[] = {{&S88688[74], 6}, {NULL}};
const Segment SGI231911[] = {{&S98744[27], 6}, {NULL}};
const Segment SGI231912[] = {{&S98744[98], 9}, {NULL}};
const Segment SGI231913[] = {{&S98744[71], 6}, {NULL}};
const Segment SGI231914[] = {{&S80921[44], 7}, {NULL}};
const Segment SGI231915[] = {{&S80921[44], 19}, {NULL}};
const Segment SGI231916[] = {{&S80921[30], 6}, {NULL}};
const Segment SGI231917[] = {{&S80921[112], 3}, {NULL}};
const Segment SGI231918[] = {{&S80921[77], 6}, {NULL}};
const Segment SGI231919[] = {{&S80921[137], 8}, {NULL}};
const Segment SGI231920[] = {{&S98746[44], 4}, {NULL}};
const Segment SGI231921[] = {{&S98746[44], 16}, {NULL}};
const Segment SGI231922[] = {{&S98746[30], 6}, {NULL}};
const Segment SGI231923[] = {{&S98746[31], 3}, {NULL}};
const Segment SGI231924[] = {{&S98746[74], 5}, {NULL}};
const Segment SGI231925[] = {{&S98746[111], 14}, {NULL}};
const Segment SGI231926[] = {{&S98748[34], 10}, {NULL}};
const Segment SGI231927[] = {{&S98748[27], 6}, {NULL}};
const Segment SGI231928[] = {{&S98748[97], 9}, {NULL}};
const Segment SGI231929[] = {{&S98748[70], 6}, {NULL}};
const Segment SGI231930[] = {{&S98748[131], 8}, {NULL}};
const Segment SGI231931[] = {{&S98750[34], 10}, {NULL}};
const Segment SGI231932[] = {{&S98750[27], 6}, {NULL}};
const Segment SGI231933[] = {{&S98750[97], 9}, {NULL}};
const Segment SGI231934[] = {{&S80930[34], 12}, {NULL}};
const Segment SGI231935[] = {{&S98750[70], 6}, {NULL}};
const Segment SGI231936[] = {{&S80930[27], 6}, {NULL}};
const Segment SGI231937[] = {{&S80930[99], 9}, {NULL}};
const Segment SGI231938[] = {{&S80930[72], 6}, {NULL}};
const Segment SGI231939[] = {{&S95594[45], 10}, {NULL}};
const Segment SGI231940[] = {{&S95594[30], 6}, {NULL}};
const Segment SGI231941[] = {{&S95594[115], 3}, {NULL}};
const Segment SGI231942[] = {{&S95594[81], 5}, {NULL}};
const Segment SGI231943[] = {{&S95594[139], 13}, {NULL}};
const Segment SGI231944[] = {{&S80932[34], 11}, {NULL}};
const Segment SGI231945[] = {{&S80932[27], 6}, {NULL}};
const Segment SGI231946[] = {{&S80932[97], 9}, {NULL}};
const Segment SGI231947[] = {{&S80932[70], 6}, {NULL}};
const Segment SGI231948[] = {{&S80932[138], 8}, {NULL}};
const Segment SGI231949[] = {{&S98754[34], 10}, {NULL}};
const Segment SGI231950[] = {{&S98754[27], 6}, {NULL}};
const Segment SGI231951[] = {{&S98754[97], 9}, {NULL}};
const Segment SGI231952[] = {{&S98754[70], 6}, {NULL}};
const Segment SGI231953[] = {{&S98755[34], 7}, {NULL}};
const Segment SGI231954[] = {{&S98755[27], 6}, {NULL}};
const Segment SGI231955[] = {{&S98755[94], 9}, {NULL}};
const Segment SGI231956[] = {{&S98755[67], 6}, {NULL}};
const Segment SGI231957[] = {{&S98763[34], 10}, {NULL}};
const Segment SGI231958[] = {{&S98763[27], 6}, {NULL}};
const Segment SGI231959[] = {{&S98763[97], 9}, {NULL}};
const Segment SGI231960[] = {{&S98763[70], 6}, {NULL}};
const Segment SGI231961[] = {{&S95608[34], 4}, {NULL}};
const Segment SGI231962[] = {{&S95608[27], 6}, {NULL}};
const Segment SGI231963[] = {{&S95608[91], 9}, {NULL}};
const Segment SGI231964[] = {{&S95608[64], 6}, {NULL}};
const Segment SGI231965[] = {{&S98765[34], 11}, {NULL}};
const Segment SGI231966[] = {{&S98765[27], 6}, {NULL}};
const Segment SGI231967[] = {{&S98765[98], 9}, {NULL}};
const Segment SGI231968[] = {{&S98765[71], 6}, {NULL}};
const Segment SGI231969[] = {{&S98768[34], 10}, {NULL}};
const Segment SGI231970[] = {{&S98768[27], 6}, {NULL}};
const Segment SGI231971[] = {{&S98768[97], 9}, {NULL}};
const Segment SGI231972[] = {{&S98768[70], 6}, {NULL}};
const Segment SGI231973[] = {{&S95615[34], 12}, {NULL}};
const Segment SGI231974[] = {{&S95615[27], 6}, {NULL}};
const Segment SGI231975[] = {{&S95615[99], 9}, {NULL}};
const Segment SGI231976[] = {{&S95615[72], 6}, {NULL}};
const Segment SGI231977[] = {{&S98771[34], 11}, {NULL}};
const Segment SGI231978[] = {{&S98771[27], 6}, {NULL}};
const Segment SGI231979[] = {{&S98771[98], 9}, {NULL}};
const Segment SGI231980[] = {{&S98771[71], 6}, {NULL}};
const Segment SGI231981[] = {{&S98775[34], 4}, {NULL}};
const Segment SGI231982[] = {{&S98775[27], 6}, {NULL}};
const Segment SGI231983[] = {{&S98775[88], 9}, {NULL}};
const Segment SGI231984[] = {{&S98775[61], 6}, {NULL}};
const Segment SGI231985[] = {{&S95624[44], 5}, {NULL}};
const Segment SGI231986[] = {{&S95624[44], 17}, {NULL}};
const Segment SGI231987[] = {{&S95624[30], 6}, {NULL}};
const Segment SGI231988[] = {{&S95624[109], 3}, {NULL}};
const Segment SGI231989[] = {{&S95624[75], 5}, {NULL}};
const Segment SGI231990[] = {{&S88742[59], 10}, {NULL}};
const Segment SGI231991[] = {{&S88742[59], 22}, {NULL}};
const Segment SGI231992[] = {{&S88742[45], 6}, {NULL}};
const Segment SGI231993[] = {{&S88742[130], 3}, {NULL}};
const Segment SGI231994[] = {{&S88742[95], 6}, {NULL}};
const Segment SGI231995[] = {{&S88742[0], 14}, {NULL}};
const Segment SGI231996[] = {{&S98785[34], 10}, {NULL}};
const Segment SGI231997[] = {{&S98785[27], 6}, {NULL}};
const Segment SGI231998[] = {{&S98785[97], 9}, {NULL}};
const Segment SGI231999[] = {{&S98785[70], 6}, {NULL}};
const Segment SGI232000[] = {{&S98788[44], 7}, {NULL}};
const Segment SGI232001[] = {{&S98788[44], 23}, {NULL}};
const Segment SGI232002[] = {{&S98788[30], 6}, {NULL}};
const Segment SGI232003[] = {{&S98788[31], 3}, {NULL}};
const Segment SGI232004[] = {{&S98788[81], 5}, {NULL}};
const Segment SGI232005[] = {{&S98788[118], 14}, {NULL}};
const Segment SGI232006[] = {{&S95636[52], 14}, {NULL}};
const Segment SGI232007[] = {{&S95636[52], 26}, {NULL}};
const Segment SGI232008[] = {{&S95636[38], 6}, {NULL}};
const Segment SGI232009[] = {{&S95636[92], 5}, {NULL}};
const Segment SGI232010[] = {{&S95636[0], 14}, {NULL}};
const Segment SGI232011[] = {{&S98791[44], 7}, {NULL}};
const Segment SGI232012[] = {{&S98791[44], 19}, {NULL}};
const Segment SGI232013[] = {{&S98791[30], 6}, {NULL}};
const Segment SGI232014[] = {{&S98791[77], 6}, {NULL}};
const Segment SGI232015[] = {{&S98791[104], 13}, {NULL}};
const Segment SGI232016[] = {{&S88756[34], 11}, {NULL}};
const Segment SGI232017[] = {{&S88756[27], 6}, {NULL}};
const Segment SGI232018[] = {{&S88756[98], 9}, {NULL}};
const Segment SGI232019[] = {{&S88756[71], 6}, {NULL}};
const Segment SGI232020[] = {{&S80983[34], 28}, {NULL}};
const Segment SGI232021[] = {{&S80983[27], 6}, {NULL}};
const Segment SGI232022[] = {{&S80983[115], 9}, {NULL}};
const Segment SGI232023[] = {{&S80983[88], 6}, {NULL}};
const Segment SGI232024[] = {{&S98795[59], 15}, {NULL}};
const Segment SGI232025[] = {{&S98795[59], 33}, {NULL}};
const Segment SGI232026[] = {{&S98795[45], 6}, {NULL}};
const Segment SGI232027[] = {{&S98795[46], 3}, {NULL}};
const Segment SGI232028[] = {{&S98795[106], 6}, {NULL}};
const Segment SGI232029[] = {{&S88760[34], 7}, {NULL}};
const Segment SGI232030[] = {{&S88760[27], 6}, {NULL}};
const Segment SGI232031[] = {{&S98795[0], 14}, {NULL}};
const Segment SGI232032[] = {{&S88760[94], 9}, {NULL}};
const Segment SGI232033[] = {{&S88760[67], 6}, {NULL}};
const Segment SGI232034[] = {{&S98801[34], 20}, {NULL}};
const Segment SGI232035[] = {{&S98801[27], 6}, {NULL}};
const Segment SGI232036[] = {{&S98801[107], 9}, {NULL}};
const Segment SGI232037[] = {{&S98801[80], 6}, {NULL}};
const Segment SGI232038[] = {{&S98802[44], 10}, {NULL}};
const Segment SGI232039[] = {{&S98802[44], 22}, {NULL}};
const Segment SGI232040[] = {{&S98802[30], 6}, {NULL}};
const Segment SGI232041[] = {{&S98802[31], 3}, {NULL}};
const Segment SGI232042[] = {{&S98802[80], 6}, {NULL}};
const Segment SGI232043[] = {{&S98802[139], 10}, {NULL}};
const Segment SGI232044[] = {{&S95655[34], 4}, {NULL}};
const Segment SGI232045[] = {{&S95655[27], 6}, {NULL}};
const Segment SGI232046[] = {{&S98803[34], 10}, {NULL}};
const Segment SGI232047[] = {{&S95655[91], 9}, {NULL}};
const Segment SGI232048[] = {{&S95655[64], 6}, {NULL}};
const Segment SGI232049[] = {{&S98803[27], 6}, {NULL}};
const Segment SGI232050[] = {{&S98803[97], 9}, {NULL}};
const Segment SGI232051[] = {{&S98803[70], 6}, {NULL}};
const Segment SGI232052[] = {{&S88770[34], 23}, {NULL}};
const Segment SGI232053[] = {{&S88770[27], 6}, {NULL}};
const Segment SGI232054[] = {{&S88770[110], 9}, {NULL}};
const Segment SGI232055[] = {{&S88770[83], 6}, {NULL}};
const Segment SGI232056[] = {{&S88773[34], 7}, {NULL}};
const Segment SGI232057[] = {{&S88773[27], 6}, {NULL}};
const Segment SGI232058[] = {{&S98807[44], 11}, {NULL}};
const Segment SGI232059[] = {{&S98807[44], 23}, {NULL}};
const Segment SGI232060[] = {{&S88773[99], 9}, {NULL}};
const Segment SGI232061[] = {{&S98807[30], 6}, {NULL}};
const Segment SGI232062[] = {{&S88773[72], 6}, {NULL}};
const Segment SGI232063[] = {{&S95660[34], 17}, {NULL}};
const Segment SGI232064[] = {{&S95660[27], 6}, {NULL}};
const Segment SGI232065[] = {{&S98807[81], 6}, {NULL}};
const Segment SGI232066[] = {{&S98807[108], 13}, {NULL}};
const Segment SGI232067[] = {{&S95660[115], 9}, {NULL}};
const Segment SGI232068[] = {{&S95660[88], 6}, {NULL}};
const Segment SGI232069[] = {{&S88778[34], 10}, {NULL}};
const Segment SGI232070[] = {{&S88778[27], 6}, {NULL}};
const Segment SGI232071[] = {{&S88778[97], 9}, {NULL}};
const Segment SGI232072[] = {{&S88778[70], 6}, {NULL}};
const Segment SGI232073[] = {{&S98815[34], 30}, {NULL}};
const Segment SGI232074[] = {{&S98815[27], 6}, {NULL}};
const Segment SGI232075[] = {{&S98815[117], 9}, {NULL}};
const Segment SGI232076[] = {{&S98815[90], 6}, {NULL}};
const Segment SGI232077[] = {{&S98816[44], 5}, {NULL}};
const Segment SGI232078[] = {{&S98816[44], 21}, {NULL}};
const Segment SGI232079[] = {{&S98816[30], 6}, {NULL}};
const Segment SGI232080[] = {{&S98816[31], 3}, {NULL}};
const Segment SGI232081[] = {{&S98816[79], 5}, {NULL}};
const Segment SGI232082[] = {{&S98816[116], 14}, {NULL}};
const Segment SGI232083[] = {{&S98817[44], 14}, {NULL}};
const Segment SGI232084[] = {{&S98817[44], 26}, {NULL}};
const Segment SGI232085[] = {{&S98817[30], 6}, {NULL}};
const Segment SGI232086[] = {{&S98817[31], 3}, {NULL}};
const Segment SGI232087[] = {{&S98817[84], 5}, {NULL}};
const Segment SGI232088[] = {{&S88785[44], 13}, {NULL}};
const Segment SGI232089[] = {{&S98817[121], 14}, {NULL}};
const Segment SGI232090[] = {{&S88785[44], 25}, {NULL}};
const Segment SGI232091[] = {{&S88785[30], 6}, {NULL}};
const Segment SGI232092[] = {{&S88785[116], 3}, {NULL}};
const Segment SGI232093[] = {{&S88785[83], 5}, {NULL}};
const Segment SGI232094[] = {{&S88785[120], 14}, {NULL}};
const Segment SGI232095[] = {{&S88786[44], 15}, {NULL}};
const Segment SGI232096[] = {{&S88786[44], 27}, {NULL}};
const Segment SGI232097[] = {{&S88786[30], 6}, {NULL}};
const Segment SGI232098[] = {{&S98819[44], 10}, {NULL}};
const Segment SGI232099[] = {{&S98819[44], 30}, {NULL}};
const Segment SGI232100[] = {{&S98819[30], 6}, {NULL}};
const Segment SGI232101[] = {{&S88786[118], 3}, {NULL}};
const Segment SGI232102[] = {{&S88786[85], 5}, {NULL}};
const Segment SGI232103[] = {{&S98819[31], 3}, {NULL}};
const Segment SGI232104[] = {{&S88786[122], 14}, {NULL}};
const Segment SGI232105[] = {{&S98819[88], 5}, {NULL}};
const Segment SGI232106[] = {{&S98819[125], 14}, {NULL}};
const Segment SGI232107[] = {{&S88787[44], 13}, {NULL}};
const Segment SGI232108[] = {{&S88787[44], 25}, {NULL}};
const Segment SGI232109[] = {{&S88787[30], 6}, {NULL}};
const Segment SGI232110[] = {{&S88787[116], 3}, {NULL}};
const Segment SGI232111[] = {{&S88787[83], 5}, {NULL}};
const Segment SGI232112[] = {{&S88787[120], 14}, {NULL}};
const Segment SGI232113[] = {{&S98821[34], 6}, {NULL}};
const Segment SGI232114[] = {{&S98821[27], 6}, {NULL}};
const Segment SGI232115[] = {{&S98821[93], 9}, {NULL}};
const Segment SGI232116[] = {{&S98821[66], 6}, {NULL}};
const Segment SGI232117[] = {{&S88788[44], 14}, {NULL}};
const Segment SGI232118[] = {{&S88788[44], 26}, {NULL}};
const Segment SGI232119[] = {{&S88788[30], 6}, {NULL}};
const Segment SGI232120[] = {{&S81025[44], 10}, {NULL}};
const Segment SGI232121[] = {{&S88788[117], 3}, {NULL}};
const Segment SGI232122[] = {{&S81025[44], 21}, {NULL}};
const Segment SGI232123[] = {{&S81025[30], 6}, {NULL}};
const Segment SGI232124[] = {{&S88788[84], 5}, {NULL}};
const Segment SGI232125[] = {{&S88788[121], 14}, {NULL}};
const Segment SGI232126[] = {{&S81025[137], 3}, {NULL}};
const Segment SGI232127[] = {{&S81025[102], 6}, {NULL}};
const Segment SGI232128[] = {{&S81025[74], 9}, {NULL}};
const Segment SGI232129[] = {{&S88789[44], 9}, {NULL}};
const Segment SGI232130[] = {{&S98823[44], 5}, {NULL}};
const Segment SGI232131[] = {{&S88789[44], 21}, {NULL}};
const Segment SGI232132[] = {{&S98823[44], 17}, {NULL}};
const Segment SGI232133[] = {{&S88789[30], 6}, {NULL}};
const Segment SGI232134[] = {{&S98823[30], 6}, {NULL}};
const Segment SGI232135[] = {{&S88789[114], 3}, {NULL}};
const Segment SGI232136[] = {{&S81026[44], 10}, {NULL}};
const Segment SGI232137[] = {{&S88789[79], 6}, {NULL}};
const Segment SGI232138[] = {{&S81026[44], 22}, {NULL}};
const Segment SGI232139[] = {{&S98823[75], 6}, {NULL}};
const Segment SGI232140[] = {{&S81026[30], 6}, {NULL}};
const Segment SGI232141[] = {{&S98823[102], 13}, {NULL}};
const Segment SGI232142[] = {{&S81026[138], 3}, {NULL}};
const Segment SGI232143[] = {{&S81026[103], 6}, {NULL}};
const Segment SGI232144[] = {{&S81026[75], 9}, {NULL}};
const Segment SGI232145[] = {{&S88790[44], 10}, {NULL}};
const Segment SGI232146[] = {{&S88790[44], 22}, {NULL}};
const Segment SGI232147[] = {{&S88790[30], 6}, {NULL}};
const Segment SGI232148[] = {{&S88790[80], 6}, {NULL}};
const Segment SGI232149[] = {{&S88790[107], 13}, {NULL}};
const Segment SGI232150[] = {{&S98828[34], 9}, {NULL}};
const Segment SGI232151[] = {{&S98828[27], 6}, {NULL}};
const Segment SGI232152[] = {{&S88793[44], 15}, {NULL}};
const Segment SGI232153[] = {{&S88793[44], 27}, {NULL}};
const Segment SGI232154[] = {{&S98828[96], 9}, {NULL}};
const Segment SGI232155[] = {{&S88793[30], 6}, {NULL}};
const Segment SGI232156[] = {{&S98828[69], 6}, {NULL}};
const Segment SGI232157[] = {{&S88793[120], 3}, {NULL}};
const Segment SGI232158[] = {{&S88793[85], 6}, {NULL}};
const Segment SGI232159[] = {{&S88794[44], 16}, {NULL}};
const Segment SGI232160[] = {{&S88794[44], 29}, {NULL}};
const Segment SGI232161[] = {{&S88794[30], 6}, {NULL}};
const Segment SGI232162[] = {{&S88794[122], 3}, {NULL}};
const Segment SGI232163[] = {{&S88794[87], 6}, {NULL}};
const Segment SGI232164[] = {{&S81035[44], 10}, {NULL}};
const Segment SGI232165[] = {{&S81035[44], 21}, {NULL}};
const Segment SGI232166[] = {{&S81035[30], 6}, {NULL}};
const Segment SGI232167[] = {{&S81035[79], 6}, {NULL}};
const Segment SGI232168[] = {{&S81035[106], 13}, {NULL}};
const Segment SGI232169[] = {{&S88796[34], 10}, {NULL}};
const Segment SGI232170[] = {{&S88796[27], 6}, {NULL}};
const Segment SGI232171[] = {{&S88796[97], 9}, {NULL}};
const Segment SGI232172[] = {{&S88796[70], 6}, {NULL}};
const Segment SGI232173[] = {{&S88796[131], 8}, {NULL}};
const Segment SGI232174[] = {{&S98835[44], 24}, {NULL}};
const Segment SGI232175[] = {{&S98835[44], 36}, {NULL}};
const Segment SGI232176[] = {{&S98835[30], 6}, {NULL}};
const Segment SGI232177[] = {{&S98835[94], 6}, {NULL}};
const Segment SGI232178[] = {{&S81038[44], 19}, {NULL}};
const Segment SGI232179[] = {{&S81038[44], 30}, {NULL}};
const Segment SGI232180[] = {{&S98835[121], 13}, {NULL}};
const Segment SGI232181[] = {{&S81038[30], 6}, {NULL}};
const Segment SGI232182[] = {{&S81038[123], 3}, {NULL}};
const Segment SGI232183[] = {{&S81038[88], 6}, {NULL}};
const Segment SGI232184[] = {{&S81039[44], 17}, {NULL}};
const Segment SGI232185[] = {{&S81039[44], 28}, {NULL}};
const Segment SGI232186[] = {{&S81039[30], 6}, {NULL}};
const Segment SGI232187[] = {{&S81039[121], 3}, {NULL}};
const Segment SGI232188[] = {{&S81039[86], 6}, {NULL}};
const Segment SGI232189[] = {{&S81040[44], 17}, {NULL}};
const Segment SGI232190[] = {{&S81040[44], 28}, {NULL}};
const Segment SGI232191[] = {{&S81040[30], 6}, {NULL}};
const Segment SGI232192[] = {{&S98839[44], 10}, {NULL}};
const Segment SGI232193[] = {{&S98839[44], 22}, {NULL}};
const Segment SGI232194[] = {{&S81040[121], 3}, {NULL}};
const Segment SGI232195[] = {{&S98839[30], 6}, {NULL}};
const Segment SGI232196[] = {{&S81040[86], 6}, {NULL}};
const Segment SGI232197[] = {{&S98839[80], 6}, {NULL}};
const Segment SGI232198[] = {{&S98839[107], 13}, {NULL}};
const Segment SGI232199[] = {{&S81046[34], 17}, {NULL}};
const Segment SGI232200[] = {{&S81046[27], 6}, {NULL}};
const Segment SGI232201[] = {{&S81046[103], 9}, {NULL}};
const Segment SGI232202[] = {{&S81046[76], 6}, {NULL}};
const Segment SGI232203[] = {{&S81047[34], 17}, {NULL}};
const Segment SGI232204[] = {{&S81047[27], 6}, {NULL}};
const Segment SGI232205[] = {{&S81047[103], 9}, {NULL}};
const Segment SGI232206[] = {{&S81047[76], 6}, {NULL}};
const Segment SGI232207[] = {{&S98848[34], 18}, {NULL}};
const Segment SGI232208[] = {{&S98848[27], 6}, {NULL}};
const Segment SGI232209[] = {{&S98848[105], 9}, {NULL}};
const Segment SGI232210[] = {{&S98848[78], 6}, {NULL}};
const Segment SGI232211[] = {{&S98848[146], 8}, {NULL}};
const Segment SGI232212[] = {{&S98857[34], 10}, {NULL}};
const Segment SGI232213[] = {{&S98857[27], 6}, {NULL}};
const Segment SGI232214[] = {{&S98857[97], 9}, {NULL}};
const Segment SGI232215[] = {{&S98857[70], 6}, {NULL}};
const Segment SGI232216[] = {{&S98861[34], 15}, {NULL}};
const Segment SGI232217[] = {{&S98861[27], 6}, {NULL}};
const Segment SGI232218[] = {{&S98861[102], 9}, {NULL}};
const Segment SGI232219[] = {{&S98861[75], 6}, {NULL}};
const Segment SGI232220[] = {{&S103421[38], 10}, {NULL}};
const Segment SGI232221[] = {{&S103421[31], 6}, {NULL}};
const Segment SGI232222[] = {{&S103423[44], 10}, {NULL}};
const Segment SGI232223[] = {{&S103423[44], 22}, {NULL}};
const Segment SGI232224[] = {{&S103423[30], 6}, {NULL}};
const Segment SGI232225[] = {{&S103423[31], 3}, {NULL}};
const Segment SGI232226[] = {{&S103423[80], 6}, {NULL}};
const Segment SGI232227[] = {{&S98868[44], 23}, {NULL}};
const Segment SGI232228[] = {{&S98868[44], 35}, {NULL}};
const Segment SGI232229[] = {{&S98868[30], 6}, {NULL}};
const Segment SGI232230[] = {{&S98868[31], 3}, {NULL}};
const Segment SGI232231[] = {{&S98868[93], 6}, {NULL}};
const Segment SGI232232[] = {{&S89911[34], 7}, {NULL}};
const Segment SGI232233[] = {{&S89911[27], 6}, {NULL}};
const Segment SGI232234[] = {{&S89911[94], 9}, {NULL}};
const Segment SGI232235[] = {{&S89911[67], 6}, {NULL}};
const Segment SGI232236[] = {{&S98877[44], 20}, {NULL}};
const Segment SGI232237[] = {{&S98877[44], 32}, {NULL}};
const Segment SGI232238[] = {{&S98877[30], 6}, {NULL}};
const Segment SGI232239[] = {{&S98877[90], 6}, {NULL}};
const Segment SGI232240[] = {{&S98877[117], 13}, {NULL}};
const Segment SGI232241[] = {{&S98878[52], 12}, {NULL}};
const Segment SGI232242[] = {{&S98878[52], 42}, {NULL}};
const Segment SGI232243[] = {{&S98878[38], 6}, {NULL}};
const Segment SGI232244[] = {{&S98878[108], 5}, {NULL}};
const Segment SGI232245[] = {{&S98878[0], 14}, {NULL}};
const Segment SGI232246[] = {{&S81085[44], 18}, {NULL}};
const Segment SGI232247[] = {{&S81085[44], 30}, {NULL}};
const Segment SGI232248[] = {{&S81085[30], 6}, {NULL}};
const Segment SGI232249[] = {{&S81085[123], 3}, {NULL}};
const Segment SGI232250[] = {{&S81085[88], 6}, {NULL}};
const Segment SGI232251[] = {{&S98881[44], 14}, {NULL}};
const Segment SGI232252[] = {{&S98881[44], 26}, {NULL}};
const Segment SGI232253[] = {{&S98881[30], 6}, {NULL}};
const Segment SGI232254[] = {{&S98881[84], 6}, {NULL}};
const Segment SGI232255[] = {{&S98881[111], 13}, {NULL}};
const Segment SGI232256[] = {{&S98883[44], 14}, {NULL}};
const Segment SGI232257[] = {{&S98883[44], 26}, {NULL}};
const Segment SGI232258[] = {{&S81089[44], 10}, {NULL}};
const Segment SGI232259[] = {{&S98883[30], 6}, {NULL}};
const Segment SGI232260[] = {{&S81089[44], 21}, {NULL}};
const Segment SGI232261[] = {{&S81089[30], 6}, {NULL}};
const Segment SGI232262[] = {{&S81089[132], 3}, {NULL}};
const Segment SGI232263[] = {{&S98883[84], 6}, {NULL}};
const Segment SGI232264[] = {{&S81089[97], 6}, {NULL}};
const Segment SGI232265[] = {{&S98883[111], 13}, {NULL}};
const Segment SGI232266[] = {{&S81089[67], 8}, {NULL}};
const Segment SGI232267[] = {{&S81090[44], 10}, {NULL}};
const Segment SGI232268[] = {{&S81090[44], 22}, {NULL}};
const Segment SGI232269[] = {{&S81090[30], 6}, {NULL}};
const Segment SGI232270[] = {{&S81090[133], 3}, {NULL}};
const Segment SGI232271[] = {{&S98885[44], 15}, {NULL}};
const Segment SGI232272[] = {{&S81090[98], 6}, {NULL}};
const Segment SGI232273[] = {{&S98885[44], 27}, {NULL}};
const Segment SGI232274[] = {{&S98885[30], 6}, {NULL}};
const Segment SGI232275[] = {{&S81090[68], 8}, {NULL}};
const Segment SGI232276[] = {{&S98885[85], 6}, {NULL}};
const Segment SGI232277[] = {{&S98885[112], 13}, {NULL}};
const Segment SGI232278[] = {{&S81091[44], 10}, {NULL}};
const Segment SGI232279[] = {{&S81091[44], 21}, {NULL}};
const Segment SGI232280[] = {{&S81091[30], 6}, {NULL}};
const Segment SGI232281[] = {{&S81091[137], 3}, {NULL}};
const Segment SGI232282[] = {{&S81091[102], 6}, {NULL}};
const Segment SGI232283[] = {{&S98886[44], 6}, {NULL}};
const Segment SGI232284[] = {{&S98886[44], 18}, {NULL}};
const Segment SGI232285[] = {{&S98886[30], 6}, {NULL}};
const Segment SGI232286[] = {{&S98886[76], 6}, {NULL}};
const Segment SGI232287[] = {{&S98886[103], 13}, {NULL}};
const Segment SGI232288[] = {{&S81092[44], 10}, {NULL}};
const Segment SGI232289[] = {{&S81092[44], 22}, {NULL}};
const Segment SGI232290[] = {{&S81092[30], 6}, {NULL}};
const Segment SGI232291[] = {{&S81092[138], 3}, {NULL}};
const Segment SGI232292[] = {{&S81092[103], 6}, {NULL}};
const Segment SGI232293[] = {{&S81092[75], 9}, {NULL}};
const Segment SGI232294[] = {{&S98888[34], 13}, {NULL}};
const Segment SGI232295[] = {{&S98888[27], 6}, {NULL}};
const Segment SGI232296[] = {{&S98888[104], 9}, {NULL}};
const Segment SGI232297[] = {{&S98888[77], 6}, {NULL}};
const Segment SGI232298[] = {{&S98888[145], 8}, {NULL}};
const Segment SGI232299[] = {{&S81098[44], 7}, {NULL}};
const Segment SGI232300[] = {{&S81098[44], 18}, {NULL}};
const Segment SGI232301[] = {{&S81098[30], 6}, {NULL}};
const Segment SGI232302[] = {{&S81098[142], 3}, {NULL}};
const Segment SGI232303[] = {{&S81098[107], 6}, {NULL}};
const Segment SGI232304[] = {{&S81099[44], 7}, {NULL}};
const Segment SGI232305[] = {{&S81099[44], 19}, {NULL}};
const Segment SGI232306[] = {{&S81099[30], 6}, {NULL}};
const Segment SGI232307[] = {{&S81099[141], 3}, {NULL}};
const Segment SGI232308[] = {{&S81099[106], 6}, {NULL}};
const Segment SGI232309[] = {{&S81108[44], 12}, {NULL}};
const Segment SGI232310[] = {{&S81108[44], 23}, {NULL}};
const Segment SGI232311[] = {{&S81108[30], 6}, {NULL}};
const Segment SGI232312[] = {{&S81108[116], 3}, {NULL}};
const Segment SGI232313[] = {{&S81108[81], 6}, {NULL}};
const Segment SGI232314[] = {{&S98907[34], 9}, {NULL}};
const Segment SGI232315[] = {{&S98907[27], 6}, {NULL}};
const Segment SGI232316[] = {{&S98907[96], 9}, {NULL}};
const Segment SGI232317[] = {{&S98907[69], 6}, {NULL}};
const Segment SGI232318[] = {{&S81110[34], 7}, {NULL}};
const Segment SGI232319[] = {{&S81110[27], 6}, {NULL}};
const Segment SGI232320[] = {{&S81110[94], 9}, {NULL}};
const Segment SGI232321[] = {{&S81110[67], 6}, {NULL}};
const Segment SGI232322[] = {{&S88887[34], 5}, {NULL}};
const Segment SGI232323[] = {{&S88887[27], 6}, {NULL}};
const Segment SGI232324[] = {{&S88887[92], 9}, {NULL}};
const Segment SGI232325[] = {{&S88887[65], 6}, {NULL}};
const Segment SGI232326[] = {{&S98913[44], 6}, {NULL}};
const Segment SGI232327[] = {{&S98913[44], 18}, {NULL}};
const Segment SGI232328[] = {{&S98913[30], 6}, {NULL}};
const Segment SGI232329[] = {{&S98913[31], 3}, {NULL}};
const Segment SGI232330[] = {{&S98913[76], 6}, {NULL}};
const Segment SGI232331[] = {{&S98913[136], 13}, {NULL}};
const Segment SGI232332[] = {{&S88891[44], 10}, {NULL}};
const Segment SGI232333[] = {{&S88891[44], 22}, {NULL}};
const Segment SGI232334[] = {{&S88891[30], 6}, {NULL}};
const Segment SGI232335[] = {{&S88891[115], 3}, {NULL}};
const Segment SGI232336[] = {{&S98915[44], 11}, {NULL}};
const Segment SGI232337[] = {{&S88891[80], 6}, {NULL}};
const Segment SGI232338[] = {{&S98915[44], 23}, {NULL}};
const Segment SGI232339[] = {{&S98915[30], 6}, {NULL}};
const Segment SGI232340[] = {{&S88891[143], 3}, {NULL}};
const Segment SGI232341[] = {{&S98915[31], 3}, {NULL}};
const Segment SGI232342[] = {{&S98915[81], 6}, {NULL}};
const Segment SGI232343[] = {{&S98915[141], 7}, {NULL}};
const Segment SGI232344[] = {{&S98917[44], 7}, {NULL}};
const Segment SGI232345[] = {{&S98917[44], 19}, {NULL}};
const Segment SGI232346[] = {{&S98917[30], 6}, {NULL}};
const Segment SGI232347[] = {{&S98917[31], 3}, {NULL}};
const Segment SGI232348[] = {{&S98917[77], 6}, {NULL}};
const Segment SGI232349[] = {{&S98917[131], 12}, {NULL}};
const Segment SGI232350[] = {{&S98918[52], 11}, {NULL}};
const Segment SGI232351[] = {{&S98918[52], 23}, {NULL}};
const Segment SGI232352[] = {{&S98918[38], 6}, {NULL}};
const Segment SGI232353[] = {{&S98918[89], 5}, {NULL}};
const Segment SGI232354[] = {{&S98918[0], 14}, {NULL}};
const Segment SGI232355[] = {{&S98923[34], 9}, {NULL}};
const Segment SGI232356[] = {{&S98923[27], 6}, {NULL}};
const Segment SGI232357[] = {{&S88903[34], 10}, {NULL}};
const Segment SGI232358[] = {{&S88903[27], 6}, {NULL}};
const Segment SGI232359[] = {{&S98923[96], 9}, {NULL}};
const Segment SGI232360[] = {{&S98923[69], 6}, {NULL}};
const Segment SGI232361[] = {{&S98923[130], 8}, {NULL}};
const Segment SGI232362[] = {{&S88903[97], 9}, {NULL}};
const Segment SGI232363[] = {{&S88903[70], 6}, {NULL}};
const Segment SGI232364[] = {{&S88904[34], 13}, {NULL}};
const Segment SGI232365[] = {{&S88904[27], 6}, {NULL}};
const Segment SGI232366[] = {{&S88904[100], 9}, {NULL}};
const Segment SGI232367[] = {{&S88904[73], 6}, {NULL}};
const Segment SGI232368[] = {{&S98926[44], 14}, {NULL}};
const Segment SGI232369[] = {{&S98926[44], 26}, {NULL}};
const Segment SGI232370[] = {{&S98926[30], 6}, {NULL}};
const Segment SGI232371[] = {{&S98926[84], 6}, {NULL}};
const Segment SGI232372[] = {{&S98926[111], 13}, {NULL}};
const Segment SGI232373[] = {{&S88909[34], 11}, {NULL}};
const Segment SGI232374[] = {{&S88909[27], 6}, {NULL}};
const Segment SGI232375[] = {{&S88909[98], 9}, {NULL}};
const Segment SGI232376[] = {{&S88909[71], 6}, {NULL}};
const Segment SGI232377[] = {{&S89991[34], 10}, {NULL}};
const Segment SGI232378[] = {{&S89991[27], 6}, {NULL}};
const Segment SGI232379[] = {{&S89991[97], 9}, {NULL}};
const Segment SGI232380[] = {{&S89991[70], 6}, {NULL}};
const Segment SGI232381[] = {{&S98934[52], 7}, {NULL}};
const Segment SGI232382[] = {{&S88912[44], 11}, {NULL}};
const Segment SGI232383[] = {{&S98934[52], 24}, {NULL}};
const Segment SGI232384[] = {{&S88912[44], 23}, {NULL}};
const Segment SGI232385[] = {{&S98934[38], 6}, {NULL}};
const Segment SGI232386[] = {{&S88912[30], 6}, {NULL}};
const Segment SGI232387[] = {{&S88912[116], 3}, {NULL}};
const Segment SGI232388[] = {{&S98934[90], 5}, {NULL}};
const Segment SGI232389[] = {{&S88912[81], 6}, {NULL}};
const Segment SGI232390[] = {{&S98934[0], 14}, {NULL}};
const Segment SGI232391[] = {{&S98937[34], 12}, {NULL}};
const Segment SGI232392[] = {{&S98937[27], 6}, {NULL}};
const Segment SGI232393[] = {{&S98937[99], 9}, {NULL}};
const Segment SGI232394[] = {{&S98937[72], 6}, {NULL}};
const Segment SGI232395[] = {{&S98938[44], 14}, {NULL}};
const Segment SGI232396[] = {{&S98938[44], 26}, {NULL}};
const Segment SGI232397[] = {{&S98938[30], 6}, {NULL}};
const Segment SGI232398[] = {{&S98938[31], 3}, {NULL}};
const Segment SGI232399[] = {{&S98938[84], 5}, {NULL}};
const Segment SGI232400[] = {{&S98938[121], 14}, {NULL}};
const Segment SGI232401[] = {{&S88919[44], 9}, {NULL}};
const Segment SGI232402[] = {{&S88919[44], 26}, {NULL}};
const Segment SGI232403[] = {{&S88919[30], 6}, {NULL}};
const Segment SGI232404[] = {{&S88919[119], 3}, {NULL}};
const Segment SGI232405[] = {{&S88919[84], 6}, {NULL}};
const Segment SGI232406[] = {{&S88920[34], 9}, {NULL}};
const Segment SGI232407[] = {{&S88920[27], 6}, {NULL}};
const Segment SGI232408[] = {{&S88920[96], 9}, {NULL}};
const Segment SGI232409[] = {{&S88920[69], 6}, {NULL}};
const Segment SGI232410[] = {{&S88922[34], 24}, {NULL}};
const Segment SGI232411[] = {{&S88922[27], 6}, {NULL}};
const Segment SGI232412[] = {{&S88922[111], 9}, {NULL}};
const Segment SGI232413[] = {{&S88922[84], 6}, {NULL}};
const Segment SGI232414[] = {{&S81160[34], 11}, {NULL}};
const Segment SGI232415[] = {{&S81160[27], 6}, {NULL}};
const Segment SGI232416[] = {{&S81160[97], 9}, {NULL}};
const Segment SGI232417[] = {{&S81160[70], 6}, {NULL}};
const Segment SGI232418[] = {{&S81160[138], 8}, {NULL}};
const Segment SGI232419[] = {{&S88930[34], 10}, {NULL}};
const Segment SGI232420[] = {{&S88930[27], 6}, {NULL}};
const Segment SGI232421[] = {{&S88930[97], 9}, {NULL}};
const Segment SGI232422[] = {{&S88930[70], 6}, {NULL}};
const Segment SGI232423[] = {{&S88930[138], 8}, {NULL}};
const Segment SGI232424[] = {{&S88932[44], 10}, {NULL}};
const Segment SGI232425[] = {{&S88932[44], 22}, {NULL}};
const Segment SGI232426[] = {{&S88932[30], 6}, {NULL}};
const Segment SGI232427[] = {{&S88932[113], 3}, {NULL}};
const Segment SGI232428[] = {{&S88932[80], 5}, {NULL}};
const Segment SGI232429[] = {{&S88932[117], 14}, {NULL}};
const Segment SGI232430[] = {{&S98958[34], 9}, {NULL}};
const Segment SGI232431[] = {{&S98958[27], 6}, {NULL}};
const Segment SGI232432[] = {{&S98958[96], 9}, {NULL}};
const Segment SGI232433[] = {{&S98958[69], 6}, {NULL}};
const Segment SGI232434[] = {{&S98960[34], 5}, {NULL}};
const Segment SGI232435[] = {{&S98960[27], 6}, {NULL}};
const Segment SGI232436[] = {{&S98960[92], 9}, {NULL}};
const Segment SGI232437[] = {{&S98960[65], 6}, {NULL}};
const Segment SGI232438[] = {{&S88936[34], 9}, {NULL}};
const Segment SGI232439[] = {{&S88936[27], 6}, {NULL}};
const Segment SGI232440[] = {{&S88936[96], 9}, {NULL}};
const Segment SGI232441[] = {{&S88936[69], 6}, {NULL}};
const Segment SGI232442[] = {{&S88937[34], 18}, {NULL}};
const Segment SGI232443[] = {{&S88937[27], 6}, {NULL}};
const Segment SGI232444[] = {{&S88937[105], 9}, {NULL}};
const Segment SGI232445[] = {{&S88937[78], 6}, {NULL}};
const Segment SGI232446[] = {{&S88937[146], 8}, {NULL}};
const Segment SGI232447[] = {{&S88940[44], 10}, {NULL}};
const Segment SGI232448[] = {{&S88940[44], 22}, {NULL}};
const Segment SGI232449[] = {{&S88940[30], 6}, {NULL}};
const Segment SGI232450[] = {{&S88940[80], 6}, {NULL}};
const Segment SGI232451[] = {{&S88940[107], 13}, {NULL}};
const Segment SGI232452[] = {{&S98967[34], 15}, {NULL}};
const Segment SGI232453[] = {{&S98967[27], 6}, {NULL}};
const Segment SGI232454[] = {{&S98967[102], 9}, {NULL}};
const Segment SGI232455[] = {{&S98967[75], 6}, {NULL}};
const Segment SGI232456[] = {{&S95771[34], 18}, {NULL}};
const Segment SGI232457[] = {{&S95771[27], 6}, {NULL}};
const Segment SGI232458[] = {{&S95771[105], 9}, {NULL}};
const Segment SGI232459[] = {{&S95771[78], 6}, {NULL}};
const Segment SGI232460[] = {{&S98982[44], 14}, {NULL}};
const Segment SGI232461[] = {{&S98982[44], 31}, {NULL}};
const Segment SGI232462[] = {{&S98982[30], 6}, {NULL}};
const Segment SGI232463[] = {{&S98982[89], 6}, {NULL}};
const Segment SGI232464[] = {{&S98982[116], 13}, {NULL}};
const Segment SGI232465[] = {{&S98984[34], 10}, {NULL}};
const Segment SGI232466[] = {{&S98984[27], 6}, {NULL}};
const Segment SGI232467[] = {{&S98984[97], 9}, {NULL}};
const Segment SGI232468[] = {{&S98984[70], 6}, {NULL}};
const Segment SGI232469[] = {{&S95795[34], 10}, {NULL}};
const Segment SGI232470[] = {{&S95795[27], 6}, {NULL}};
const Segment SGI232471[] = {{&S95795[97], 9}, {NULL}};
const Segment SGI232472[] = {{&S95795[70], 6}, {NULL}};
const Segment SGI232473[] = {{&S98992[34], 11}, {NULL}};
const Segment SGI232474[] = {{&S98992[27], 6}, {NULL}};
const Segment SGI232475[] = {{&S81210[34], 9}, {NULL}};
const Segment SGI232476[] = {{&S81210[27], 6}, {NULL}};
const Segment SGI232477[] = {{&S98992[98], 9}, {NULL}};
const Segment SGI232478[] = {{&S98992[71], 6}, {NULL}};
const Segment SGI232479[] = {{&S81210[96], 9}, {NULL}};
const Segment SGI232480[] = {{&S81210[69], 6}, {NULL}};
const Segment SGI232481[] = {{&S98992[139], 8}, {NULL}};
const Segment SGI232482[] = {{&S98996[34], 18}, {NULL}};
const Segment SGI232483[] = {{&S81214[34], 10}, {NULL}};
const Segment SGI232484[] = {{&S98996[27], 6}, {NULL}};
const Segment SGI232485[] = {{&S81214[27], 6}, {NULL}};
const Segment SGI232486[] = {{&S98996[105], 9}, {NULL}};
const Segment SGI232487[] = {{&S81214[96], 9}, {NULL}};
const Segment SGI232488[] = {{&S98996[78], 6}, {NULL}};
const Segment SGI232489[] = {{&S81214[69], 6}, {NULL}};
const Segment SGI232490[] = {{&S99000[34], 7}, {NULL}};
const Segment SGI232491[] = {{&S99000[27], 6}, {NULL}};
const Segment SGI232492[] = {{&S99000[100], 9}, {NULL}};
const Segment SGI232493[] = {{&S99000[73], 6}, {NULL}};
const Segment SGI232494[] = {{&S81224[34], 7}, {NULL}};
const Segment SGI232495[] = {{&S81224[27], 6}, {NULL}};
const Segment SGI232496[] = {{&S81224[94], 9}, {NULL}};
const Segment SGI232497[] = {{&S81224[67], 6}, {NULL}};
const Segment SGI232498[] = {{&S90082[44], 10}, {NULL}};
const Segment SGI232499[] = {{&S90082[44], 22}, {NULL}};
const Segment SGI232500[] = {{&S90082[30], 6}, {NULL}};
const Segment SGI232501[] = {{&S90082[133], 3}, {NULL}};
const Segment SGI232502[] = {{&S90082[98], 6}, {NULL}};
const Segment SGI232503[] = {{&S90082[68], 8}, {NULL}};
const Segment SGI232504[] = {{&S90083[44], 10}, {NULL}};
const Segment SGI232505[] = {{&S90083[44], 22}, {NULL}};
const Segment SGI232506[] = {{&S90083[30], 6}, {NULL}};
const Segment SGI232507[] = {{&S90083[133], 3}, {NULL}};
const Segment SGI232508[] = {{&S90083[98], 6}, {NULL}};
const Segment SGI232509[] = {{&S90083[68], 8}, {NULL}};
const Segment SGI232510[] = {{&S95825[34], 7}, {NULL}};
const Segment SGI232511[] = {{&S90092[44], 14}, {NULL}};
const Segment SGI232512[] = {{&S95825[27], 6}, {NULL}};
const Segment SGI232513[] = {{&S90092[44], 26}, {NULL}};
const Segment SGI232514[] = {{&S90092[30], 6}, {NULL}};
const Segment SGI232515[] = {{&S95825[94], 9}, {NULL}};
const Segment SGI232516[] = {{&S95825[67], 6}, {NULL}};
const Segment SGI232517[] = {{&S90092[84], 6}, {NULL}};
const Segment SGI232518[] = {{&S90092[121], 3}, {NULL}};
const Segment SGI232519[] = {{&S95832[34], 7}, {NULL}};
const Segment SGI232520[] = {{&S95832[27], 6}, {NULL}};
const Segment SGI232521[] = {{&S95832[99], 9}, {NULL}};
const Segment SGI232522[] = {{&S95832[72], 6}, {NULL}};
const Segment SGI232523[] = {{&S95850[34], 20}, {NULL}};
const Segment SGI232524[] = {{&S95850[27], 6}, {NULL}};
const Segment SGI232525[] = {{&S95850[107], 9}, {NULL}};
const Segment SGI232526[] = {{&S95850[80], 6}, {NULL}};
const Segment SGI232527[] = {{&S89034[34], 7}, {NULL}};
const Segment SGI232528[] = {{&S89034[27], 6}, {NULL}};
const Segment SGI232529[] = {{&S89034[94], 9}, {NULL}};
const Segment SGI232530[] = {{&S89034[67], 6}, {NULL}};
const Segment SGI232531[] = {{&S95852[34], 15}, {NULL}};
const Segment SGI232532[] = {{&S95852[27], 6}, {NULL}};
const Segment SGI232533[] = {{&S95852[102], 9}, {NULL}};
const Segment SGI232534[] = {{&S95852[75], 6}, {NULL}};
const Segment SGI232535[] = {{&S95857[34], 19}, {NULL}};
const Segment SGI232536[] = {{&S95857[27], 6}, {NULL}};
const Segment SGI232537[] = {{&S95857[106], 9}, {NULL}};
const Segment SGI232538[] = {{&S95857[79], 6}, {NULL}};
const Segment SGI232539[] = {{&S90130[44], 9}, {NULL}};
const Segment SGI232540[] = {{&S90130[44], 21}, {NULL}};
const Segment SGI232541[] = {{&S90130[30], 6}, {NULL}};
const Segment SGI232542[] = {{&S90130[114], 3}, {NULL}};
const Segment SGI232543[] = {{&S95859[34], 11}, {NULL}};
const Segment SGI232544[] = {{&S90130[79], 6}, {NULL}};
const Segment SGI232545[] = {{&S95859[27], 6}, {NULL}};
const Segment SGI232546[] = {{&S95859[98], 9}, {NULL}};
const Segment SGI232547[] = {{&S95859[71], 6}, {NULL}};
const Segment SGI232548[] = {{&S89046[44], 18}, {NULL}};
const Segment SGI232549[] = {{&S89046[44], 30}, {NULL}};
const Segment SGI232550[] = {{&S89046[30], 6}, {NULL}};
const Segment SGI232551[] = {{&S89046[123], 3}, {NULL}};
const Segment SGI232552[] = {{&S89046[88], 6}, {NULL}};
const Segment SGI232553[] = {{&S89046[155], 15}, {NULL}};
const Segment SGI232554[] = {{&S90135[44], 15}, {NULL}};
const Segment SGI232555[] = {{&S90135[44], 27}, {NULL}};
const Segment SGI232556[] = {{&S90135[30], 6}, {NULL}};
const Segment SGI232557[] = {{&S90135[85], 6}, {NULL}};
const Segment SGI232558[] = {{&S90135[122], 3}, {NULL}};
const Segment SGI232559[] = {{&S95866[44], 14}, {NULL}};
const Segment SGI232560[] = {{&S95866[44], 26}, {NULL}};
const Segment SGI232561[] = {{&S95866[30], 6}, {NULL}};
const Segment SGI232562[] = {{&S95866[119], 3}, {NULL}};
const Segment SGI232563[] = {{&S95866[84], 6}, {NULL}};
const Segment SGI232564[] = {{&S90138[44], 4}, {NULL}};
const Segment SGI232565[] = {{&S90138[44], 16}, {NULL}};
const Segment SGI232566[] = {{&S90138[30], 6}, {NULL}};
const Segment SGI232567[] = {{&S90138[74], 6}, {NULL}};
const Segment SGI232568[] = {{&S90138[111], 3}, {NULL}};
const Segment SGI232569[] = {{&S95868[44], 18}, {NULL}};
const Segment SGI232570[] = {{&S95868[44], 39}, {NULL}};
const Segment SGI232571[] = {{&S95868[30], 6}, {NULL}};
const Segment SGI232572[] = {{&S89054[34], 10}, {NULL}};
const Segment SGI232573[] = {{&S95868[130], 3}, {NULL}};
const Segment SGI232574[] = {{&S89054[27], 6}, {NULL}};
const Segment SGI232575[] = {{&S95868[97], 5}, {NULL}};
const Segment SGI232576[] = {{&S89054[97], 9}, {NULL}};
const Segment SGI232577[] = {{&S95868[134], 14}, {NULL}};
const Segment SGI232578[] = {{&S89054[70], 6}, {NULL}};
const Segment SGI232579[] = {{&S81299[44], 7}, {NULL}};
const Segment SGI232580[] = {{&S81299[44], 19}, {NULL}};
const Segment SGI232581[] = {{&S81299[30], 6}, {NULL}};
const Segment SGI232582[] = {{&S81299[112], 3}, {NULL}};
const Segment SGI232583[] = {{&S81299[77], 6}, {NULL}};
const Segment SGI232584[] = {{&S95873[34], 17}, {NULL}};
const Segment SGI232585[] = {{&S95873[27], 6}, {NULL}};
const Segment SGI232586[] = {{&S95873[104], 9}, {NULL}};
const Segment SGI232587[] = {{&S95873[77], 6}, {NULL}};
const Segment SGI232588[] = {{&S89063[44], 7}, {NULL}};
const Segment SGI232589[] = {{&S89063[44], 25}, {NULL}};
const Segment SGI232590[] = {{&S89063[30], 6}, {NULL}};
const Segment SGI232591[] = {{&S89063[83], 6}, {NULL}};
const Segment SGI232592[] = {{&S89063[110], 13}, {NULL}};
const Segment SGI232593[] = {{&S90151[44], 7}, {NULL}};
const Segment SGI232594[] = {{&S90151[44], 19}, {NULL}};
const Segment SGI232595[] = {{&S90151[30], 6}, {NULL}};
const Segment SGI232596[] = {{&S90151[112], 3}, {NULL}};
const Segment SGI232597[] = {{&S90151[77], 6}, {NULL}};
const Segment SGI232598[] = {{&S95878[44], 42}, {NULL}};
const Segment SGI232599[] = {{&S95878[44], 54}, {NULL}};
const Segment SGI232600[] = {{&S95878[30], 6}, {NULL}};
const Segment SGI232601[] = {{&S81306[44], 11}, {NULL}};
const Segment SGI232602[] = {{&S81306[44], 22}, {NULL}};
const Segment SGI232603[] = {{&S81306[30], 6}, {NULL}};
const Segment SGI232604[] = {{&S95878[147], 3}, {NULL}};
const Segment SGI232605[] = {{&S95878[112], 6}, {NULL}};
const Segment SGI232606[] = {{&S81306[115], 3}, {NULL}};
const Segment SGI232607[] = {{&S81306[80], 6}, {NULL}};
const Segment SGI232608[] = {{&S99096[34], 8}, {NULL}};
const Segment SGI232609[] = {{&S99096[27], 6}, {NULL}};
const Segment SGI232610[] = {{&S99096[95], 9}, {NULL}};
const Segment SGI232611[] = {{&S99096[68], 6}, {NULL}};
const Segment SGI232612[] = {{&S81307[44], 12}, {NULL}};
const Segment SGI232613[] = {{&S81307[44], 23}, {NULL}};
const Segment SGI232614[] = {{&S81307[30], 6}, {NULL}};
const Segment SGI232615[] = {{&S81307[116], 3}, {NULL}};
const Segment SGI232616[] = {{&S81307[81], 6}, {NULL}};
const Segment SGI232617[] = {{&S95883[34], 21}, {NULL}};
const Segment SGI232618[] = {{&S95883[27], 6}, {NULL}};
const Segment SGI232619[] = {{&S95883[108], 9}, {NULL}};
const Segment SGI232620[] = {{&S95883[81], 6}, {NULL}};
const Segment SGI232621[] = {{&S89078[34], 7}, {NULL}};
const Segment SGI232622[] = {{&S89078[27], 6}, {NULL}};
const Segment SGI232623[] = {{&S89078[95], 9}, {NULL}};
const Segment SGI232624[] = {{&S89078[68], 6}, {NULL}};
const Segment SGI232625[] = {{&S99114[45], 18}, {NULL}};
const Segment SGI232626[] = {{&S99114[30], 6}, {NULL}};
const Segment SGI232627[] = {{&S99114[31], 3}, {NULL}};
const Segment SGI232628[] = {{&S99114[89], 5}, {NULL}};
const Segment SGI232629[] = {{&S99114[147], 13}, {NULL}};
const Segment SGI232630[] = {{&S89094[44], 10}, {NULL}};
const Segment SGI232631[] = {{&S89094[44], 22}, {NULL}};
const Segment SGI232632[] = {{&S89094[30], 6}, {NULL}};
const Segment SGI232633[] = {{&S89094[115], 3}, {NULL}};
const Segment SGI232634[] = {{&S89094[80], 6}, {NULL}};
const Segment SGI232635[] = {{&S89094[135], 8}, {NULL}};
const Segment SGI232636[] = {{&S89094[144], 13}, {NULL}};
const Segment SGI232637[] = {{&S99125[44], 10}, {NULL}};
const Segment SGI232638[] = {{&S99125[44], 22}, {NULL}};
const Segment SGI232639[] = {{&S95907[44], 11}, {NULL}};
const Segment SGI232640[] = {{&S99125[30], 6}, {NULL}};
const Segment SGI232641[] = {{&S95907[44], 23}, {NULL}};
const Segment SGI232642[] = {{&S95907[30], 6}, {NULL}};
const Segment SGI232643[] = {{&S99125[146], 4}, {NULL}};
const Segment SGI232644[] = {{&S99125[172], 3}, {NULL}};
const Segment SGI232645[] = {{&S95907[115], 3}, {NULL}};
const Segment SGI232646[] = {{&S95907[81], 5}, {NULL}};
const Segment SGI232647[] = {{&S99125[31], 3}, {&S99125[221], 5}, {NULL}};
const Segment SGI232648[] = {{&S99125[80], 6}, {NULL}};
const Segment SGI232649[] = {{&S89099[34], 8}, {NULL}};
const Segment SGI232650[] = {{&S89099[27], 6}, {NULL}};
const Segment SGI232651[] = {{&S89099[95], 9}, {NULL}};
const Segment SGI232652[] = {{&S89099[68], 6}, {NULL}};
const Segment SGI232653[] = {{&S99129[34], 7}, {NULL}};
const Segment SGI232654[] = {{&S99129[27], 6}, {NULL}};
const Segment SGI232655[] = {{&S99129[94], 9}, {NULL}};
const Segment SGI232656[] = {{&S99129[67], 6}, {NULL}};
const Segment SGI232657[] = {{&S81344[44], 12}, {NULL}};
const Segment SGI232658[] = {{&S81344[44], 23}, {NULL}};
const Segment SGI232659[] = {{&S81344[30], 6}, {NULL}};
const Segment SGI232660[] = {{&S81344[116], 3}, {NULL}};
const Segment SGI232661[] = {{&S81344[81], 6}, {NULL}};
const Segment SGI232662[] = {{&S89103[34], 10}, {NULL}};
const Segment SGI232663[] = {{&S89103[27], 6}, {NULL}};
const Segment SGI232664[] = {{&S89103[97], 9}, {NULL}};
const Segment SGI232665[] = {{&S89103[70], 6}, {NULL}};
const Segment SGI232666[] = {{&S81345[44], 12}, {NULL}};
const Segment SGI232667[] = {{&S81345[44], 24}, {NULL}};
const Segment SGI232668[] = {{&S81345[30], 6}, {NULL}};
const Segment SGI232669[] = {{&S81345[117], 3}, {NULL}};
const Segment SGI232670[] = {{&S81345[82], 6}, {NULL}};
const Segment SGI232671[] = {{&S99144[44], 10}, {NULL}};
const Segment SGI232672[] = {{&S99144[44], 22}, {NULL}};
const Segment SGI232673[] = {{&S99144[30], 6}, {NULL}};
const Segment SGI232674[] = {{&S99144[31], 3}, {NULL}};
const Segment SGI232675[] = {{&S99144[99], 6}, {NULL}};
const Segment SGI232676[] = {{&S99144[68], 8}, {NULL}};
const Segment SGI232677[] = {{&S99145[44], 10}, {NULL}};
const Segment SGI232678[] = {{&S99145[44], 22}, {NULL}};
const Segment SGI232679[] = {{&S99145[30], 6}, {NULL}};
const Segment SGI232680[] = {{&S99145[31], 3}, {NULL}};
const Segment SGI232681[] = {{&S99145[115], 6}, {NULL}};
const Segment SGI232682[] = {{&S99145[68], 8}, {NULL}};
const Segment SGI232683[] = {{&S81363[44], 9}, {NULL}};
const Segment SGI232684[] = {{&S81363[44], 20}, {NULL}};
const Segment SGI232685[] = {{&S81363[30], 6}, {NULL}};
const Segment SGI232686[] = {{&S81363[131], 3}, {NULL}};
const Segment SGI232687[] = {{&S81363[96], 6}, {NULL}};
const Segment SGI232688[] = {{&S81363[66], 8}, {NULL}};
const Segment SGI232689[] = {{&S81364[44], 9}, {NULL}};
const Segment SGI232690[] = {{&S81364[44], 21}, {NULL}};
const Segment SGI232691[] = {{&S81364[30], 6}, {NULL}};
const Segment SGI232692[] = {{&S81364[132], 3}, {NULL}};
const Segment SGI232693[] = {{&S81364[97], 6}, {NULL}};
const Segment SGI232694[] = {{&S81364[67], 8}, {NULL}};
const Segment SGI232695[] = {{&S99152[44], 10}, {NULL}};
const Segment SGI232696[] = {{&S99152[44], 22}, {NULL}};
const Segment SGI232697[] = {{&S99152[30], 6}, {NULL}};
const Segment SGI232698[] = {{&S99152[31], 3}, {NULL}};
const Segment SGI232699[] = {{&S99152[80], 6}, {NULL}};
const Segment SGI232700[] = {{&S99152[132], 10}, {NULL}};
const Segment SGI232701[] = {{&S99165[44], 7}, {NULL}};
const Segment SGI232702[] = {{&S99165[44], 19}, {NULL}};
const Segment SGI232703[] = {{&S99165[30], 6}, {NULL}};
const Segment SGI232704[] = {{&S99165[31], 3}, {NULL}};
const Segment SGI232705[] = {{&S99165[92], 6}, {NULL}};
const Segment SGI232706[] = {{&S99165[65], 8}, {NULL}};
const Segment SGI232707[] = {{&S90240[45], 9}, {NULL}};
const Segment SGI232708[] = {{&S90240[30], 6}, {NULL}};
const Segment SGI232709[] = {{&S90240[114], 3}, {NULL}};
const Segment SGI232710[] = {{&S90240[80], 5}, {NULL}};
const Segment SGI232711[] = {{&S90240[148], 3}, {NULL}};
const Segment SGI232712[] = {{&S99179[44], 14}, {NULL}};
const Segment SGI232713[] = {{&S99179[44], 31}, {NULL}};
const Segment SGI232714[] = {{&S99179[30], 6}, {NULL}};
const Segment SGI232715[] = {{&S99179[89], 6}, {NULL}};
const Segment SGI232716[] = {{&S99179[116], 13}, {NULL}};
const Segment SGI232717[] = {{&S99180[34], 7}, {NULL}};
const Segment SGI232718[] = {{&S99180[27], 6}, {NULL}};
const Segment SGI232719[] = {{&S99180[94], 9}, {NULL}};
const Segment SGI232720[] = {{&S99180[67], 6}, {NULL}};
const Segment SGI232721[] = {{&S99187[59], 13}, {NULL}};
const Segment SGI232722[] = {{&S99187[59], 25}, {NULL}};
const Segment SGI232723[] = {{&S99187[45], 6}, {NULL}};
const Segment SGI232724[] = {{&S99187[46], 3}, {NULL}};
const Segment SGI232725[] = {{&S99187[98], 6}, {NULL}};
const Segment SGI232726[] = {{&S99187[0], 14}, {NULL}};
const Segment SGI232727[] = {{&S99198[34], 18}, {NULL}};
const Segment SGI232728[] = {{&S99198[27], 6}, {NULL}};
const Segment SGI232729[] = {{&S99198[105], 9}, {NULL}};
const Segment SGI232730[] = {{&S99198[78], 6}, {NULL}};
const Segment SGI232731[] = {{&S81420[34], 10}, {NULL}};
const Segment SGI232732[] = {{&S81420[27], 6}, {NULL}};
const Segment SGI232733[] = {{&S81420[96], 9}, {NULL}};
const Segment SGI232734[] = {{&S81420[69], 6}, {NULL}};
const Segment SGI232735[] = {{&S89185[34], 10}, {NULL}};
const Segment SGI232736[] = {{&S89185[27], 6}, {NULL}};
const Segment SGI232737[] = {{&S89185[97], 9}, {NULL}};
const Segment SGI232738[] = {{&S89185[70], 6}, {NULL}};
const Segment SGI232739[] = {{&S89186[44], 13}, {NULL}};
const Segment SGI232740[] = {{&S89186[44], 29}, {NULL}};
const Segment SGI232741[] = {{&S89186[30], 6}, {NULL}};
const Segment SGI232742[] = {{&S89186[120], 3}, {NULL}};
const Segment SGI232743[] = {{&S89186[87], 5}, {NULL}};
const Segment SGI232744[] = {{&S89186[124], 14}, {NULL}};
const Segment SGI232745[] = {{&S81429[34], 10}, {NULL}};
const Segment SGI232746[] = {{&S81429[27], 6}, {NULL}};
const Segment SGI232747[] = {{&S81429[97], 9}, {NULL}};
const Segment SGI232748[] = {{&S81429[70], 6}, {NULL}};
const Segment SGI232749[] = {{&S89190[44], 18}, {NULL}};
const Segment SGI232750[] = {{&S89190[44], 34}, {NULL}};
const Segment SGI232751[] = {{&S89190[30], 6}, {NULL}};
const Segment SGI232752[] = {{&S89190[127], 3}, {NULL}};
const Segment SGI232753[] = {{&S89190[92], 6}, {NULL}};
const Segment SGI232754[] = {{&S99214[44], 9}, {NULL}};
const Segment SGI232755[] = {{&S99214[44], 21}, {NULL}};
const Segment SGI232756[] = {{&S99214[30], 6}, {NULL}};
const Segment SGI232757[] = {{&S99214[31], 3}, {NULL}};
const Segment SGI232758[] = {{&S99214[94], 6}, {NULL}};
const Segment SGI232759[] = {{&S99214[67], 8}, {NULL}};
const Segment SGI232760[] = {{&S89191[34], 25}, {NULL}};
const Segment SGI232761[] = {{&S89191[27], 6}, {NULL}};
const Segment SGI232762[] = {{&S89191[112], 9}, {NULL}};
const Segment SGI232763[] = {{&S89191[85], 6}, {NULL}};
const Segment SGI232764[] = {{&S89192[44], 11}, {NULL}};
const Segment SGI232765[] = {{&S89192[44], 23}, {NULL}};
const Segment SGI232766[] = {{&S89192[30], 6}, {NULL}};
const Segment SGI232767[] = {{&S89192[114], 3}, {NULL}};
const Segment SGI232768[] = {{&S89192[81], 5}, {NULL}};
const Segment SGI232769[] = {{&S89192[118], 14}, {NULL}};
const Segment SGI232770[] = {{&S89193[44], 10}, {NULL}};
const Segment SGI232771[] = {{&S89193[44], 22}, {NULL}};
const Segment SGI232772[] = {{&S89193[30], 6}, {NULL}};
const Segment SGI232773[] = {{&S89193[113], 3}, {NULL}};
const Segment SGI232774[] = {{&S89193[80], 5}, {NULL}};
const Segment SGI232775[] = {{&S89193[117], 14}, {NULL}};
const Segment SGI232776[] = {{&S81439[34], 8}, {NULL}};
const Segment SGI232777[] = {{&S99222[45], 8}, {NULL}};
const Segment SGI232778[] = {{&S81439[27], 6}, {NULL}};
const Segment SGI232779[] = {{&S99222[30], 6}, {NULL}};
const Segment SGI232780[] = {{&S81439[103], 9}, {NULL}};
const Segment SGI232781[] = {{&S99222[31], 3}, {NULL}};
const Segment SGI232782[] = {{&S81439[76], 6}, {NULL}};
const Segment SGI232783[] = {{&S99222[79], 5}, {NULL}};
const Segment SGI232784[] = {{&S99222[137], 13}, {NULL}};
const Segment SGI232785[] = {{&S99228[45], 14}, {NULL}};
const Segment SGI232786[] = {{&S99228[30], 6}, {NULL}};
const Segment SGI232787[] = {{&S99228[31], 3}, {NULL}};
const Segment SGI232788[] = {{&S99228[85], 5}, {NULL}};
const Segment SGI232789[] = {{&S99228[143], 13}, {NULL}};
const Segment SGI232790[] = {{&S89205[34], 10}, {NULL}};
const Segment SGI232791[] = {{&S89205[27], 6}, {NULL}};
const Segment SGI232792[] = {{&S89205[97], 9}, {NULL}};
const Segment SGI232793[] = {{&S89205[70], 6}, {NULL}};
const Segment SGI232794[] = {{&S89205[138], 8}, {NULL}};
const Segment SGI232795[] = {{&S99232[34], 8}, {NULL}};
const Segment SGI232796[] = {{&S99232[27], 6}, {NULL}};
const Segment SGI232797[] = {{&S99232[95], 9}, {NULL}};
const Segment SGI232798[] = {{&S99232[68], 6}, {NULL}};
const Segment SGI232799[] = {{&S89208[34], 10}, {NULL}};
const Segment SGI232800[] = {{&S89208[27], 6}, {NULL}};
const Segment SGI232801[] = {{&S99235[34], 13}, {NULL}};
const Segment SGI232802[] = {{&S89208[97], 9}, {NULL}};
const Segment SGI232803[] = {{&S89208[70], 6}, {NULL}};
const Segment SGI232804[] = {{&S99235[27], 6}, {NULL}};
const Segment SGI232805[] = {{&S89208[138], 8}, {NULL}};
const Segment SGI232806[] = {{&S99235[100], 9}, {NULL}};
const Segment SGI232807[] = {{&S99235[73], 6}, {NULL}};
const Segment SGI232808[] = {{&S99236[34], 16}, {NULL}};
const Segment SGI232809[] = {{&S99236[27], 6}, {NULL}};
const Segment SGI232810[] = {{&S99236[103], 9}, {NULL}};
const Segment SGI232811[] = {{&S99236[76], 6}, {NULL}};
const Segment SGI232812[] = {{&S90321[34], 10}, {NULL}};
const Segment SGI232813[] = {{&S90321[27], 6}, {NULL}};
const Segment SGI232814[] = {{&S90321[96], 9}, {NULL}};
const Segment SGI232815[] = {{&S90321[70], 5}, {NULL}};
const Segment SGI232816[] = {{&S89214[44], 13}, {NULL}};
const Segment SGI232817[] = {{&S89214[44], 25}, {NULL}};
const Segment SGI232818[] = {{&S89214[30], 6}, {NULL}};
const Segment SGI232819[] = {{&S89214[83], 6}, {NULL}};
const Segment SGI232820[] = {{&S89214[110], 13}, {NULL}};
const Segment SGI232821[] = {{&S90324[34], 7}, {NULL}};
const Segment SGI232822[] = {{&S90324[27], 6}, {NULL}};
const Segment SGI232823[] = {{&S90324[94], 9}, {NULL}};
const Segment SGI232824[] = {{&S90324[67], 6}, {NULL}};
const Segment SGI232825[] = {{&S99243[34], 15}, {NULL}};
const Segment SGI232826[] = {{&S99243[27], 6}, {NULL}};
const Segment SGI232827[] = {{&S99243[102], 9}, {NULL}};
const Segment SGI232828[] = {{&S99243[75], 6}, {NULL}};
const Segment SGI232829[] = {{&S99244[44], 10}, {NULL}};
const Segment SGI232830[] = {{&S99244[44], 22}, {NULL}};
const Segment SGI232831[] = {{&S99244[30], 6}, {NULL}};
const Segment SGI232832[] = {{&S99244[80], 6}, {NULL}};
const Segment SGI232833[] = {{&S99244[107], 13}, {NULL}};
const Segment SGI232834[] = {{&S99247[34], 10}, {NULL}};
const Segment SGI232835[] = {{&S99247[27], 6}, {NULL}};
const Segment SGI232836[] = {{&S99247[97], 9}, {NULL}};
const Segment SGI232837[] = {{&S99247[70], 6}, {NULL}};
const Segment SGI232838[] = {{&S99249[34], 7}, {NULL}};
const Segment SGI232839[] = {{&S99249[27], 6}, {NULL}};
const Segment SGI232840[] = {{&S99249[94], 9}, {NULL}};
const Segment SGI232841[] = {{&S99249[67], 6}, {NULL}};
const Segment SGI232842[] = {{&S99257[34], 9}, {NULL}};
const Segment SGI232843[] = {{&S99257[27], 6}, {NULL}};
const Segment SGI232844[] = {{&S99257[96], 9}, {NULL}};
const Segment SGI232845[] = {{&S99257[69], 6}, {NULL}};
const Segment SGI232846[] = {{&S95934[34], 18}, {NULL}};
const Segment SGI232847[] = {{&S95934[27], 6}, {NULL}};
const Segment SGI232848[] = {{&S95934[114], 9}, {NULL}};
const Segment SGI232849[] = {{&S99262[34], 10}, {NULL}};
const Segment SGI232850[] = {{&S95934[87], 6}, {NULL}};
const Segment SGI232851[] = {{&S99262[27], 6}, {NULL}};
const Segment SGI232852[] = {{&S99262[97], 9}, {NULL}};
const Segment SGI232853[] = {{&S99262[70], 6}, {NULL}};
const Segment SGI232854[] = {{&S99262[138], 8}, {NULL}};
const Segment SGI232855[] = {{&S95936[34], 12}, {NULL}};
const Segment SGI232856[] = {{&S95936[27], 6}, {NULL}};
const Segment SGI232857[] = {{&S95936[99], 9}, {NULL}};
const Segment SGI232858[] = {{&S95936[72], 6}, {NULL}};
const Segment SGI232859[] = {{&S99267[34], 7}, {NULL}};
const Segment SGI232860[] = {{&S99267[27], 6}, {NULL}};
const Segment SGI232861[] = {{&S99267[95], 9}, {NULL}};
const Segment SGI232862[] = {{&S99267[68], 6}, {NULL}};
const Segment SGI232863[] = {{&S99270[34], 25}, {NULL}};
const Segment SGI232864[] = {{&S99270[27], 6}, {NULL}};
const Segment SGI232865[] = {{&S99270[112], 9}, {NULL}};
const Segment SGI232866[] = {{&S99270[85], 6}, {NULL}};
const Segment SGI232867[] = {{&S99274[34], 8}, {NULL}};
const Segment SGI232868[] = {{&S99274[27], 6}, {NULL}};
const Segment SGI232869[] = {{&S99274[95], 9}, {NULL}};
const Segment SGI232870[] = {{&S99274[68], 6}, {NULL}};
const Segment SGI232871[] = {{&S81502[44], 9}, {NULL}};
const Segment SGI232872[] = {{&S81502[44], 20}, {NULL}};
const Segment SGI232873[] = {{&S81502[30], 6}, {NULL}};
const Segment SGI232874[] = {{&S81502[113], 3}, {NULL}};
const Segment SGI232875[] = {{&S81502[78], 6}, {NULL}};
const Segment SGI232876[] = {{&S81502[138], 7}, {NULL}};
const Segment SGI232877[] = {{&S89271[44], 11}, {NULL}};
const Segment SGI232878[] = {{&S89271[44], 21}, {NULL}};
const Segment SGI232879[] = {{&S89271[30], 6}, {NULL}};
const Segment SGI232880[] = {{&S89271[134], 3}, {NULL}};
const Segment SGI232881[] = {{&S89271[99], 6}, {NULL}};
const Segment SGI232882[] = {{&S89271[67], 8}, {NULL}};
const Segment SGI232883[] = {{&S99290[34], 9}, {NULL}};
const Segment SGI232884[] = {{&S99290[27], 6}, {NULL}};
const Segment SGI232885[] = {{&S99290[96], 9}, {NULL}};
const Segment SGI232886[] = {{&S99290[69], 6}, {NULL}};
const Segment SGI232887[] = {{&S89285[44], 7}, {NULL}};
const Segment SGI232888[] = {{&S89285[44], 19}, {NULL}};
const Segment SGI232889[] = {{&S89285[30], 6}, {NULL}};
const Segment SGI232890[] = {{&S89285[128], 3}, {NULL}};
const Segment SGI232891[] = {{&S89285[93], 6}, {NULL}};
const Segment SGI232892[] = {{&S89285[65], 8}, {NULL}};
const Segment SGI232893[] = {{&S99300[44], 6}, {NULL}};
const Segment SGI232894[] = {{&S99300[44], 18}, {NULL}};
const Segment SGI232895[] = {{&S99300[30], 6}, {NULL}};
const Segment SGI232896[] = {{&S99300[76], 6}, {NULL}};
const Segment SGI232897[] = {{&S99300[103], 13}, {NULL}};
const Segment SGI232898[] = {{&S99301[34], 25}, {NULL}};
const Segment SGI232899[] = {{&S99301[27], 6}, {NULL}};
const Segment SGI232900[] = {{&S99301[112], 9}, {NULL}};
const Segment SGI232901[] = {{&S99301[85], 6}, {NULL}};
const Segment SGI232902[] = {{&S89292[34], 7}, {NULL}};
const Segment SGI232903[] = {{&S99303[44], 7}, {NULL}};
const Segment SGI232904[] = {{&S89292[27], 6}, {NULL}};
const Segment SGI232905[] = {{&S99303[44], 31}, {NULL}};
const Segment SGI232906[] = {{&S99303[30], 6}, {NULL}};
const Segment SGI232907[] = {{&S89292[100], 9}, {NULL}};
const Segment SGI232908[] = {{&S99303[31], 3}, {NULL}};
const Segment SGI232909[] = {{&S89292[73], 6}, {NULL}};
const Segment SGI232910[] = {{&S99303[89], 6}, {NULL}};
const Segment SGI232911[] = {{&S99303[148], 10}, {NULL}};
const Segment SGI232912[] = {{&S89293[34], 10}, {NULL}};
const Segment SGI232913[] = {{&S89293[27], 6}, {NULL}};
const Segment SGI232914[] = {{&S89293[97], 9}, {NULL}};
const Segment SGI232915[] = {{&S89293[70], 6}, {NULL}};
const Segment SGI232916[] = {{&S99307[34], 15}, {NULL}};
const Segment SGI232917[] = {{&S99307[27], 6}, {NULL}};
const Segment SGI232918[] = {{&S99307[102], 9}, {NULL}};
const Segment SGI232919[] = {{&S99307[75], 6}, {NULL}};
const Segment SGI232920[] = {{&S99311[44], 8}, {NULL}};
const Segment SGI232921[] = {{&S99311[44], 24}, {NULL}};
const Segment SGI232922[] = {{&S99311[30], 6}, {NULL}};
const Segment SGI232923[] = {{&S99311[31], 3}, {NULL}};
const Segment SGI232924[] = {{&S99311[82], 5}, {NULL}};
const Segment SGI232925[] = {{&S99311[119], 14}, {NULL}};
const Segment SGI232926[] = {{&S81542[34], 10}, {NULL}};
const Segment SGI232927[] = {{&S81542[27], 6}, {NULL}};
const Segment SGI232928[] = {{&S81542[96], 9}, {NULL}};
const Segment SGI232929[] = {{&S81542[69], 6}, {NULL}};
const Segment SGI232930[] = {{&S99316[44], 6}, {NULL}};
const Segment SGI232931[] = {{&S99316[44], 18}, {NULL}};
const Segment SGI232932[] = {{&S99316[30], 6}, {NULL}};
const Segment SGI232933[] = {{&S99316[31], 3}, {NULL}};
const Segment SGI232934[] = {{&S99316[76], 5}, {NULL}};
const Segment SGI232935[] = {{&S99316[113], 14}, {NULL}};
const Segment SGI232936[] = {{&S89311[34], 5}, {NULL}};
const Segment SGI232937[] = {{&S89311[27], 6}, {NULL}};
const Segment SGI232938[] = {{&S89311[92], 9}, {NULL}};
const Segment SGI232939[] = {{&S89311[65], 6}, {NULL}};
const Segment SGI232940[] = {{&S99319[34], 21}, {NULL}};
const Segment SGI232941[] = {{&S99319[27], 6}, {NULL}};
const Segment SGI232942[] = {{&S99319[108], 9}, {NULL}};
const Segment SGI232943[] = {{&S99319[81], 6}, {NULL}};
const Segment SGI232944[] = {{&S99320[34], 11}, {NULL}};
const Segment SGI232945[] = {{&S99320[27], 6}, {NULL}};
const Segment SGI232946[] = {{&S99320[98], 9}, {NULL}};
const Segment SGI232947[] = {{&S99320[71], 6}, {NULL}};
const Segment SGI232948[] = {{&S99321[34], 10}, {NULL}};
const Segment SGI232949[] = {{&S89315[61], 10}, {NULL}};
const Segment SGI232950[] = {{&S89315[61], 22}, {NULL}};
const Segment SGI232951[] = {{&S99321[27], 6}, {NULL}};
const Segment SGI232952[] = {{&S89315[47], 6}, {NULL}};
const Segment SGI232953[] = {{&S99321[94], 9}, {NULL}};
const Segment SGI232954[] = {{&S89315[132], 3}, {NULL}};
const Segment SGI232955[] = {{&S99321[67], 6}, {NULL}};
const Segment SGI232956[] = {{&S89315[97], 6}, {NULL}};
const Segment SGI232957[] = {{&S89315[11], 3}, {NULL}};
const Segment SGI232958[] = {{&S96022[44], 10}, {NULL}};
const Segment SGI232959[] = {{&S96022[44], 22}, {NULL}};
const Segment SGI232960[] = {{&S96022[30], 6}, {NULL}};
const Segment SGI232961[] = {{&S96022[144], 3}, {NULL}};
const Segment SGI232962[] = {{&S96022[109], 6}, {NULL}};
const Segment SGI232963[] = {{&S96022[68], 8}, {NULL}};
const Segment SGI232964[] = {{&S96022[80], 15}, {NULL}};
const Segment SGI232965[] = {{&S96023[44], 10}, {NULL}};
const Segment SGI232966[] = {{&S96023[44], 22}, {NULL}};
const Segment SGI232967[] = {{&S96023[30], 6}, {NULL}};
const Segment SGI232968[] = {{&S96023[144], 3}, {NULL}};
const Segment SGI232969[] = {{&S96023[109], 6}, {NULL}};
const Segment SGI232970[] = {{&S96023[68], 8}, {NULL}};
const Segment SGI232971[] = {{&S96023[80], 15}, {NULL}};
const Segment SGI232972[] = {{&S81565[44], 7}, {NULL}};
const Segment SGI232973[] = {{&S81565[44], 18}, {NULL}};
const Segment SGI232974[] = {{&S81565[30], 6}, {NULL}};
const Segment SGI232975[] = {{&S81565[111], 3}, {NULL}};
const Segment SGI232976[] = {{&S81565[76], 6}, {NULL}};
const Segment SGI232977[] = {{&S81566[38], 7}, {NULL}};
const Segment SGI232978[] = {{&S81566[31], 6}, {NULL}};
const Segment SGI232979[] = {{&S99334[44], 14}, {NULL}};
const Segment SGI232980[] = {{&S99334[44], 26}, {NULL}};
const Segment SGI232981[] = {{&S99334[30], 6}, {NULL}};
const Segment SGI232982[] = {{&S99334[84], 6}, {NULL}};
const Segment SGI232983[] = {{&S99334[111], 13}, {NULL}};
const Segment SGI232984[] = {{&S89330[44], 10}, {NULL}};
const Segment SGI232985[] = {{&S89330[44], 22}, {NULL}};
const Segment SGI232986[] = {{&S89330[30], 6}, {NULL}};
const Segment SGI232987[] = {{&S89330[115], 3}, {NULL}};
const Segment SGI232988[] = {{&S89330[80], 6}, {NULL}};
const Segment SGI232989[] = {{&S99336[44], 5}, {NULL}};
const Segment SGI232990[] = {{&S99336[44], 17}, {NULL}};
const Segment SGI232991[] = {{&S99336[30], 6}, {NULL}};
const Segment SGI232992[] = {{&S99336[75], 6}, {NULL}};
const Segment SGI232993[] = {{&S99336[102], 13}, {NULL}};
const Segment SGI232994[] = {{&S89340[44], 10}, {NULL}};
const Segment SGI232995[] = {{&S89340[44], 22}, {NULL}};
const Segment SGI232996[] = {{&S89340[30], 6}, {NULL}};
const Segment SGI232997[] = {{&S89340[115], 3}, {NULL}};
const Segment SGI232998[] = {{&S89340[80], 6}, {NULL}};
const Segment SGI232999[] = {{&S89340[140], 7}, {NULL}};
const Segment SGI233000[] = {{&S99345[34], 10}, {NULL}};
const Segment SGI233001[] = {{&S99345[27], 6}, {NULL}};
const Segment SGI233002[] = {{&S99345[97], 9}, {NULL}};
const Segment SGI233003[] = {{&S99345[70], 6}, {NULL}};
const Segment SGI233004[] = {{&S89341[44], 10}, {NULL}};
const Segment SGI233005[] = {{&S89341[44], 22}, {NULL}};
const Segment SGI233006[] = {{&S89341[30], 6}, {NULL}};
const Segment SGI233007[] = {{&S89341[115], 3}, {NULL}};
const Segment SGI233008[] = {{&S89341[80], 6}, {NULL}};
const Segment SGI233009[] = {{&S99346[34], 11}, {NULL}};
const Segment SGI233010[] = {{&S99346[27], 6}, {NULL}};
const Segment SGI233011[] = {{&S99346[98], 9}, {NULL}};
const Segment SGI233012[] = {{&S99346[71], 6}, {NULL}};
const Segment SGI233013[] = {{&S99347[34], 10}, {NULL}};
const Segment SGI233014[] = {{&S99347[27], 6}, {NULL}};
const Segment SGI233015[] = {{&S99347[97], 9}, {NULL}};
const Segment SGI233016[] = {{&S99347[70], 6}, {NULL}};
const Segment SGI233017[] = {{&S89345[34], 8}, {NULL}};
const Segment SGI233018[] = {{&S89345[27], 6}, {NULL}};
const Segment SGI233019[] = {{&S99348[34], 8}, {NULL}};
const Segment SGI233020[] = {{&S89345[100], 9}, {NULL}};
const Segment SGI233021[] = {{&S89345[73], 6}, {NULL}};
const Segment SGI233022[] = {{&S99348[27], 6}, {NULL}};
const Segment SGI233023[] = {{&S99348[94], 9}, {NULL}};
const Segment SGI233024[] = {{&S99348[67], 6}, {NULL}};
const Segment SGI233025[] = {{&S99349[44], 10}, {NULL}};
const Segment SGI233026[] = {{&S99349[44], 22}, {NULL}};
const Segment SGI233027[] = {{&S99349[30], 6}, {NULL}};
const Segment SGI233028[] = {{&S99349[80], 6}, {NULL}};
const Segment SGI233029[] = {{&S99349[107], 13}, {NULL}};
const Segment SGI233030[] = {{&S89348[44], 6}, {NULL}};
const Segment SGI233031[] = {{&S89348[44], 18}, {NULL}};
const Segment SGI233032[] = {{&S89348[30], 6}, {NULL}};
const Segment SGI233033[] = {{&S89348[111], 3}, {NULL}};
const Segment SGI233034[] = {{&S89348[76], 6}, {NULL}};
const Segment SGI233035[] = {{&S96065[44], 10}, {NULL}};
const Segment SGI233036[] = {{&S96065[44], 22}, {NULL}};
const Segment SGI233037[] = {{&S96065[30], 6}, {NULL}};
const Segment SGI233038[] = {{&S96065[115], 3}, {NULL}};
const Segment SGI233039[] = {{&S96065[80], 6}, {NULL}};
const Segment SGI233040[] = {{&S96065[132], 10}, {NULL}};
const Segment SGI233041[] = {{&S99357[45], 10}, {NULL}};
const Segment SGI233042[] = {{&S99357[30], 6}, {NULL}};
const Segment SGI233043[] = {{&S89356[34], 11}, {NULL}};
const Segment SGI233044[] = {{&S89356[27], 6}, {NULL}};
const Segment SGI233045[] = {{&S99357[31], 3}, {NULL}};
const Segment SGI233046[] = {{&S99357[81], 5}, {NULL}};
const Segment SGI233047[] = {{&S99357[139], 13}, {NULL}};
const Segment SGI233048[] = {{&S89356[98], 9}, {NULL}};
const Segment SGI233049[] = {{&S89356[71], 6}, {NULL}};
const Segment SGI233050[] = {{&S90488[34], 7}, {NULL}};
const Segment SGI233051[] = {{&S90488[27], 6}, {NULL}};
const Segment SGI233052[] = {{&S90488[99], 9}, {NULL}};
const Segment SGI233053[] = {{&S90488[72], 6}, {NULL}};
const Segment SGI233054[] = {{&S99362[34], 25}, {NULL}};
const Segment SGI233055[] = {{&S99362[27], 6}, {NULL}};
const Segment SGI233056[] = {{&S99362[112], 9}, {NULL}};
const Segment SGI233057[] = {{&S99362[85], 6}, {NULL}};
const Segment SGI233058[] = {{&S89363[44], 11}, {NULL}};
const Segment SGI233059[] = {{&S89363[44], 23}, {NULL}};
const Segment SGI233060[] = {{&S89363[30], 6}, {NULL}};
const Segment SGI233061[] = {{&S89363[116], 3}, {NULL}};
const Segment SGI233062[] = {{&S89363[81], 6}, {NULL}};
const Segment SGI233063[] = {{&S96075[61], 21}, {NULL}};
const Segment SGI233064[] = {{&S96075[27], 33}, {NULL}};
const Segment SGI233065[] = {{&S96075[135], 9}, {NULL}};
const Segment SGI233066[] = {{&S96075[108], 6}, {NULL}};
const Segment SGI233067[] = {{&S99364[34], 14}, {NULL}};
const Segment SGI233068[] = {{&S99364[27], 6}, {NULL}};
const Segment SGI233069[] = {{&S99364[101], 9}, {NULL}};
const Segment SGI233070[] = {{&S99364[74], 6}, {NULL}};
const Segment SGI233071[] = {{&S99368[44], 24}, {NULL}};
const Segment SGI233072[] = {{&S99368[44], 38}, {NULL}};
const Segment SGI233073[] = {{&S99368[30], 6}, {NULL}};
const Segment SGI233074[] = {{&S99368[96], 6}, {NULL}};
const Segment SGI233075[] = {{&S99368[123], 13}, {NULL}};
const Segment SGI233076[] = {{&S89371[34], 10}, {NULL}};
const Segment SGI233077[] = {{&S89371[27], 6}, {NULL}};
const Segment SGI233078[] = {{&S89371[97], 9}, {NULL}};
const Segment SGI233079[] = {{&S89371[70], 6}, {NULL}};
const Segment SGI233080[] = {{&S89373[34], 15}, {NULL}};
const Segment SGI233081[] = {{&S89373[27], 6}, {NULL}};
const Segment SGI233082[] = {{&S89373[105], 9}, {NULL}};
const Segment SGI233083[] = {{&S89373[78], 6}, {NULL}};
const Segment SGI233084[] = {{&S89373[146], 8}, {NULL}};
const Segment SGI233085[] = {{&S89375[44], 10}, {NULL}};
const Segment SGI233086[] = {{&S89375[44], 22}, {NULL}};
const Segment SGI233087[] = {{&S89375[30], 6}, {NULL}};
const Segment SGI233088[] = {{&S89375[113], 3}, {NULL}};
const Segment SGI233089[] = {{&S89375[80], 5}, {NULL}};
const Segment SGI233090[] = {{&S89375[117], 14}, {NULL}};
const Segment SGI233091[] = {{&S99375[34], 8}, {NULL}};
const Segment SGI233092[] = {{&S99375[27], 6}, {NULL}};
const Segment SGI233093[] = {{&S99375[108], 9}, {NULL}};
const Segment SGI233094[] = {{&S99375[81], 6}, {NULL}};
const Segment SGI233095[] = {{&S89376[34], 11}, {NULL}};
const Segment SGI233096[] = {{&S89376[27], 6}, {NULL}};
const Segment SGI233097[] = {{&S89376[98], 9}, {NULL}};
const Segment SGI233098[] = {{&S89376[71], 6}, {NULL}};
const Segment SGI233099[] = {{&S99376[34], 11}, {NULL}};
const Segment SGI233100[] = {{&S99376[27], 6}, {NULL}};
const Segment SGI233101[] = {{&S99376[98], 9}, {NULL}};
const Segment SGI233102[] = {{&S99376[71], 6}, {NULL}};
const Segment SGI233103[] = {{&S99377[34], 6}, {NULL}};
const Segment SGI233104[] = {{&S99377[27], 6}, {NULL}};
const Segment SGI233105[] = {{&S99377[93], 9}, {NULL}};
const Segment SGI233106[] = {{&S99377[66], 6}, {NULL}};
const Segment SGI233107[] = {{&S99381[34], 7}, {NULL}};
const Segment SGI233108[] = {{&S99381[27], 6}, {NULL}};
const Segment SGI233109[] = {{&S99381[95], 9}, {NULL}};
const Segment SGI233110[] = {{&S99381[68], 6}, {NULL}};
const Segment SGI233111[] = {{&S99383[44], 8}, {NULL}};
const Segment SGI233112[] = {{&S99383[44], 20}, {NULL}};
const Segment SGI233113[] = {{&S99383[30], 6}, {NULL}};
const Segment SGI233114[] = {{&S99383[78], 6}, {NULL}};
const Segment SGI233115[] = {{&S99383[105], 13}, {NULL}};
const Segment SGI233116[] = {{&S99385[45], 6}, {NULL}};
const Segment SGI233117[] = {{&S99385[30], 6}, {NULL}};
const Segment SGI233118[] = {{&S99385[31], 3}, {NULL}};
const Segment SGI233119[] = {{&S99385[77], 5}, {NULL}};
const Segment SGI233120[] = {{&S99385[135], 13}, {NULL}};
const Segment SGI233121[] = {{&S99390[44], 22}, {NULL}};
const Segment SGI233122[] = {{&S99390[44], 34}, {NULL}};
const Segment SGI233123[] = {{&S99390[30], 6}, {NULL}};
const Segment SGI233124[] = {{&S96113[34], 11}, {NULL}};
const Segment SGI233125[] = {{&S96113[27], 6}, {NULL}};
const Segment SGI233126[] = {{&S99390[31], 3}, {NULL}};
const Segment SGI233127[] = {{&S99390[92], 6}, {NULL}};
const Segment SGI233128[] = {{&S96113[98], 9}, {NULL}};
const Segment SGI233129[] = {{&S96113[71], 6}, {NULL}};
const Segment SGI233130[] = {{&S81652[44], 10}, {NULL}};
const Segment SGI233131[] = {{&S81652[44], 21}, {NULL}};
const Segment SGI233132[] = {{&S81652[30], 6}, {NULL}};
const Segment SGI233133[] = {{&S81652[79], 6}, {NULL}};
const Segment SGI233134[] = {{&S81652[106], 13}, {NULL}};
const Segment SGI233135[] = {{&S99395[44], 10}, {NULL}};
const Segment SGI233136[] = {{&S99395[44], 22}, {NULL}};
const Segment SGI233137[] = {{&S99395[30], 6}, {NULL}};
const Segment SGI233138[] = {{&S99395[31], 3}, {NULL}};
const Segment SGI233139[] = {{&S99395[80], 5}, {NULL}};
const Segment SGI233140[] = {{&S99395[117], 14}, {NULL}};
const Segment SGI233141[] = {{&S99398[52], 9}, {NULL}};
const Segment SGI233142[] = {{&S99398[52], 21}, {NULL}};
const Segment SGI233143[] = {{&S99398[38], 6}, {NULL}};
const Segment SGI233144[] = {{&S99398[87], 5}, {NULL}};
const Segment SGI233145[] = {{&S99398[0], 14}, {NULL}};
const Segment SGI233146[] = {{&S96126[59], 18}, {NULL}};
const Segment SGI233147[] = {{&S96126[59], 39}, {NULL}};
const Segment SGI233148[] = {{&S96126[45], 6}, {NULL}};
const Segment SGI233149[] = {{&S96126[147], 3}, {NULL}};
const Segment SGI233150[] = {{&S96126[112], 6}, {NULL}};
const Segment SGI233151[] = {{&S96126[0], 14}, {NULL}};
const Segment SGI233152[] = {{&S99405[44], 13}, {NULL}};
const Segment SGI233153[] = {{&S99405[44], 25}, {NULL}};
const Segment SGI233154[] = {{&S99405[30], 6}, {NULL}};
const Segment SGI233155[] = {{&S99405[83], 6}, {NULL}};
const Segment SGI233156[] = {{&S99405[110], 13}, {NULL}};
const Segment SGI233157[] = {{&S99411[59], 13}, {NULL}};
const Segment SGI233158[] = {{&S99411[59], 32}, {NULL}};
const Segment SGI233159[] = {{&S99411[45], 6}, {NULL}};
const Segment SGI233160[] = {{&S99411[46], 3}, {NULL}};
const Segment SGI233161[] = {{&S99411[105], 6}, {NULL}};
const Segment SGI233162[] = {{&S99411[0], 14}, {NULL}};
const Segment SGI233163[] = {{&S99413[44], 10}, {NULL}};
const Segment SGI233164[] = {{&S99413[44], 22}, {NULL}};
const Segment SGI233165[] = {{&S99413[30], 6}, {NULL}};
const Segment SGI233166[] = {{&S99413[31], 3}, {NULL}};
const Segment SGI233167[] = {{&S99413[80], 6}, {NULL}};
const Segment SGI233168[] = {{&S96144[34], 7}, {NULL}};
const Segment SGI233169[] = {{&S96144[27], 6}, {NULL}};
const Segment SGI233170[] = {{&S96144[98], 9}, {NULL}};
const Segment SGI233171[] = {{&S96144[71], 6}, {NULL}};
const Segment SGI233172[] = {{&S96147[34], 7}, {NULL}};
const Segment SGI233173[] = {{&S96147[27], 6}, {NULL}};
const Segment SGI233174[] = {{&S96147[94], 9}, {NULL}};
const Segment SGI233175[] = {{&S96147[67], 6}, {NULL}};
const Segment SGI233176[] = {{&S81697[44], 9}, {NULL}};
const Segment SGI233177[] = {{&S81697[44], 20}, {NULL}};
const Segment SGI233178[] = {{&S81697[30], 6}, {NULL}};
const Segment SGI233179[] = {{&S81697[113], 3}, {NULL}};
const Segment SGI233180[] = {{&S81697[78], 6}, {NULL}};
const Segment SGI233181[] = {{&S81697[138], 11}, {NULL}};
const Segment SGI233182[] = {{&S89450[44], 7}, {NULL}};
const Segment SGI233183[] = {{&S89450[44], 19}, {NULL}};
const Segment SGI233184[] = {{&S89450[30], 6}, {NULL}};
const Segment SGI233185[] = {{&S89450[112], 3}, {NULL}};
const Segment SGI233186[] = {{&S89450[77], 6}, {NULL}};
const Segment SGI233187[] = {{&S99432[44], 6}, {NULL}};
const Segment SGI233188[] = {{&S99432[44], 18}, {NULL}};
const Segment SGI233189[] = {{&S99432[30], 6}, {NULL}};
const Segment SGI233190[] = {{&S99432[31], 3}, {NULL}};
const Segment SGI233191[] = {{&S99432[76], 6}, {NULL}};
const Segment SGI233192[] = {{&S99432[136], 11}, {NULL}};
const Segment SGI233193[] = {{&S81702[44], 10}, {NULL}};
const Segment SGI233194[] = {{&S81702[44], 21}, {NULL}};
const Segment SGI233195[] = {{&S81702[30], 6}, {NULL}};
const Segment SGI233196[] = {{&S81702[137], 3}, {NULL}};
const Segment SGI233197[] = {{&S81702[102], 6}, {NULL}};
const Segment SGI233198[] = {{&S89453[44], 12}, {NULL}};
const Segment SGI233199[] = {{&S89453[44], 22}, {NULL}};
const Segment SGI233200[] = {{&S89453[30], 6}, {NULL}};
const Segment SGI233201[] = {{&S89453[115], 3}, {NULL}};
const Segment SGI233202[] = {{&S89453[80], 6}, {NULL}};
const Segment SGI233203[] = {{&S90586[34], 10}, {NULL}};
const Segment SGI233204[] = {{&S90586[27], 6}, {NULL}};
const Segment SGI233205[] = {{&S81703[44], 10}, {NULL}};
const Segment SGI233206[] = {{&S81703[44], 21}, {NULL}};
const Segment SGI233207[] = {{&S81703[30], 6}, {NULL}};
const Segment SGI233208[] = {{&S90586[97], 9}, {NULL}};
const Segment SGI233209[] = {{&S90586[70], 6}, {NULL}};
const Segment SGI233210[] = {{&S81703[137], 3}, {NULL}};
const Segment SGI233211[] = {{&S81703[102], 6}, {NULL}};
const Segment SGI233212[] = {{&S81703[74], 9}, {NULL}};
const Segment SGI233213[] = {{&S81704[44], 10}, {NULL}};
const Segment SGI233214[] = {{&S81704[44], 22}, {NULL}};
const Segment SGI233215[] = {{&S81704[30], 6}, {NULL}};
const Segment SGI233216[] = {{&S81704[138], 3}, {NULL}};
const Segment SGI233217[] = {{&S81704[103], 6}, {NULL}};
const Segment SGI233218[] = {{&S81704[75], 9}, {NULL}};
const Segment SGI233219[] = {{&S96170[34], 19}, {NULL}};
const Segment SGI233220[] = {{&S96170[27], 6}, {NULL}};
const Segment SGI233221[] = {{&S96170[106], 9}, {NULL}};
const Segment SGI233222[] = {{&S96170[79], 6}, {NULL}};
const Segment SGI233223[] = {{&S99445[44], 8}, {NULL}};
const Segment SGI233224[] = {{&S99445[44], 20}, {NULL}};
const Segment SGI233225[] = {{&S99445[30], 6}, {NULL}};
const Segment SGI233226[] = {{&S99445[31], 3}, {NULL}};
const Segment SGI233227[] = {{&S99445[78], 6}, {NULL}};
const Segment SGI233228[] = {{&S99445[138], 7}, {NULL}};
const Segment SGI233229[] = {{&S99446[45], 10}, {NULL}};
const Segment SGI233230[] = {{&S99446[30], 6}, {NULL}};
const Segment SGI233231[] = {{&S99446[31], 3}, {NULL}};
const Segment SGI233232[] = {{&S99446[81], 5}, {NULL}};
const Segment SGI233233[] = {{&S99446[139], 13}, {NULL}};
const Segment SGI233234[] = {{&S89469[34], 6}, {NULL}};
const Segment SGI233235[] = {{&S89469[27], 6}, {NULL}};
const Segment SGI233236[] = {{&S89469[93], 9}, {NULL}};
const Segment SGI233237[] = {{&S89469[66], 6}, {NULL}};
const Segment SGI233238[] = {{&S99454[34], 6}, {NULL}};
const Segment SGI233239[] = {{&S99454[27], 6}, {NULL}};
const Segment SGI233240[] = {{&S81724[44], 9}, {NULL}};
const Segment SGI233241[] = {{&S81724[44], 20}, {NULL}};
const Segment SGI233242[] = {{&S81724[30], 6}, {NULL}};
const Segment SGI233243[] = {{&S99454[93], 9}, {NULL}};
const Segment SGI233244[] = {{&S99454[66], 6}, {NULL}};
const Segment SGI233245[] = {{&S81724[78], 6}, {NULL}};
const Segment SGI233246[] = {{&S81724[105], 13}, {NULL}};
const Segment SGI233247[] = {{&S81727[34], 8}, {NULL}};
const Segment SGI233248[] = {{&S81727[27], 6}, {NULL}};
const Segment SGI233249[] = {{&S99458[44], 6}, {NULL}};
const Segment SGI233250[] = {{&S99458[44], 18}, {NULL}};
const Segment SGI233251[] = {{&S81727[94], 9}, {NULL}};
const Segment SGI233252[] = {{&S99458[30], 6}, {NULL}};
const Segment SGI233253[] = {{&S81727[67], 6}, {NULL}};
const Segment SGI233254[] = {{&S99458[31], 3}, {NULL}};
const Segment SGI233255[] = {{&S99458[76], 5}, {NULL}};
const Segment SGI233256[] = {{&S99458[113], 14}, {NULL}};
const Segment SGI233257[] = {{&S99463[34], 7}, {NULL}};
const Segment SGI233258[] = {{&S99463[27], 6}, {NULL}};
const Segment SGI233259[] = {{&S99463[93], 9}, {NULL}};
const Segment SGI233260[] = {{&S99463[66], 6}, {NULL}};
const Segment SGI233261[] = {{&S96200[34], 12}, {NULL}};
const Segment SGI233262[] = {{&S99466[34], 25}, {NULL}};
const Segment SGI233263[] = {{&S96200[27], 6}, {NULL}};
const Segment SGI233264[] = {{&S99466[27], 6}, {NULL}};
const Segment SGI233265[] = {{&S99466[112], 9}, {NULL}};
const Segment SGI233266[] = {{&S96200[99], 9}, {NULL}};
const Segment SGI233267[] = {{&S99466[85], 6}, {NULL}};
const Segment SGI233268[] = {{&S96200[72], 6}, {NULL}};
const Segment SGI233269[] = {{&S99469[34], 9}, {NULL}};
const Segment SGI233270[] = {{&S99469[27], 6}, {NULL}};
const Segment SGI233271[] = {{&S89497[34], 8}, {NULL}};
const Segment SGI233272[] = {{&S89497[27], 6}, {NULL}};
const Segment SGI233273[] = {{&S99469[96], 9}, {NULL}};
const Segment SGI233274[] = {{&S99469[69], 6}, {NULL}};
const Segment SGI233275[] = {{&S89497[95], 9}, {NULL}};
const Segment SGI233276[] = {{&S89497[68], 6}, {NULL}};
const Segment SGI233277[] = {{&S99470[44], 18}, {NULL}};
const Segment SGI233278[] = {{&S99470[44], 30}, {NULL}};
const Segment SGI233279[] = {{&S99470[30], 6}, {NULL}};
const Segment SGI233280[] = {{&S99470[31], 3}, {NULL}};
const Segment SGI233281[] = {{&S99470[103], 6}, {NULL}};
const Segment SGI233282[] = {{&S89499[34], 8}, {NULL}};
const Segment SGI233283[] = {{&S99470[76], 8}, {NULL}};
const Segment SGI233284[] = {{&S89499[27], 6}, {NULL}};
const Segment SGI233285[] = {{&S89499[95], 9}, {NULL}};
const Segment SGI233286[] = {{&S89499[68], 6}, {NULL}};
const Segment SGI233287[] = {{&S99472[34], 7}, {NULL}};
const Segment SGI233288[] = {{&S99472[27], 6}, {NULL}};
const Segment SGI233289[] = {{&S99472[94], 9}, {NULL}};
const Segment SGI233290[] = {{&S89501[34], 12}, {NULL}};
const Segment SGI233291[] = {{&S99472[67], 6}, {NULL}};
const Segment SGI233292[] = {{&S89501[27], 6}, {NULL}};
const Segment SGI233293[] = {{&S89501[99], 9}, {NULL}};
const Segment SGI233294[] = {{&S89501[72], 6}, {NULL}};
const Segment SGI233295[] = {{&S89501[140], 8}, {NULL}};
const Segment SGI233296[] = {{&S99474[52], 10}, {NULL}};
const Segment SGI233297[] = {{&S96212[44], 10}, {NULL}};
const Segment SGI233298[] = {{&S99474[52], 22}, {NULL}};
const Segment SGI233299[] = {{&S99474[38], 6}, {NULL}};
const Segment SGI233300[] = {{&S96212[44], 22}, {NULL}};
const Segment SGI233301[] = {{&S96212[30], 6}, {NULL}};
const Segment SGI233302[] = {{&S96212[115], 3}, {NULL}};
const Segment SGI233303[] = {{&S99474[88], 5}, {NULL}};
const Segment SGI233304[] = {{&S96212[80], 6}, {NULL}};
const Segment SGI233305[] = {{&S99474[0], 14}, {NULL}};
const Segment SGI233306[] = {{&S96212[133], 7}, {NULL}};
const Segment SGI233307[] = {{&S90643[34], 8}, {NULL}};
const Segment SGI233308[] = {{&S90643[27], 6}, {NULL}};
const Segment SGI233309[] = {{&S90643[95], 9}, {NULL}};
const Segment SGI233310[] = {{&S90643[68], 6}, {NULL}};
const Segment SGI233311[] = {{&S90643[129], 8}, {NULL}};
const Segment SGI233312[] = {{&S99480[34], 6}, {NULL}};
const Segment SGI233313[] = {{&S99480[27], 6}, {NULL}};
const Segment SGI233314[] = {{&S99480[93], 9}, {NULL}};
const Segment SGI233315[] = {{&S99480[66], 6}, {NULL}};
const Segment SGI233316[] = {{&S81765[44], 14}, {NULL}};
const Segment SGI233317[] = {{&S81765[44], 25}, {NULL}};
const Segment SGI233318[] = {{&S81765[30], 6}, {NULL}};
const Segment SGI233319[] = {{&S81765[118], 3}, {NULL}};
const Segment SGI233320[] = {{&S81765[83], 6}, {NULL}};
const Segment SGI233321[] = {{&S81765[142], 10}, {NULL}};
const Segment SGI233322[] = {{&S89524[44], 10}, {NULL}};
const Segment SGI233323[] = {{&S89524[44], 22}, {NULL}};
const Segment SGI233324[] = {{&S89524[30], 6}, {NULL}};
const Segment SGI233325[] = {{&S89524[80], 6}, {NULL}};
const Segment SGI233326[] = {{&S99491[44], 13}, {NULL}};
const Segment SGI233327[] = {{&S89524[107], 13}, {NULL}};
const Segment SGI233328[] = {{&S99491[44], 25}, {NULL}};
const Segment SGI233329[] = {{&S99491[30], 6}, {NULL}};
const Segment SGI233330[] = {{&S99491[83], 6}, {NULL}};
const Segment SGI233331[] = {{&S99491[110], 13}, {NULL}};
const Segment SGI233332[] = {{&S89527[34], 7}, {NULL}};
const Segment SGI233333[] = {{&S89527[27], 6}, {NULL}};
const Segment SGI233334[] = {{&S89527[94], 9}, {NULL}};
const Segment SGI233335[] = {{&S89527[67], 6}, {NULL}};
const Segment SGI233336[] = {{&S90663[44], 19}, {NULL}};
const Segment SGI233337[] = {{&S90663[44], 31}, {NULL}};
const Segment SGI233338[] = {{&S90663[30], 6}, {NULL}};
const Segment SGI233339[] = {{&S90663[124], 3}, {NULL}};
const Segment SGI233340[] = {{&S90663[89], 6}, {NULL}};
const Segment SGI233341[] = {{&S89529[34], 21}, {NULL}};
const Segment SGI233342[] = {{&S96237[34], 7}, {NULL}};
const Segment SGI233343[] = {{&S89529[27], 6}, {NULL}};
const Segment SGI233344[] = {{&S96237[27], 6}, {NULL}};
const Segment SGI233345[] = {{&S89529[116], 3}, {NULL}};
const Segment SGI233346[] = {{&S89529[81], 6}, {NULL}};
const Segment SGI233347[] = {{&S96237[94], 9}, {NULL}};
const Segment SGI233348[] = {{&S96237[67], 6}, {NULL}};
const Segment SGI233349[] = {{&S81778[34], 10}, {NULL}};
const Segment SGI233350[] = {{&S81778[27], 6}, {NULL}};
const Segment SGI233351[] = {{&S81778[96], 9}, {NULL}};
const Segment SGI233352[] = {{&S81778[69], 6}, {NULL}};
const Segment SGI233353[] = {{&S89534[34], 9}, {NULL}};
const Segment SGI233354[] = {{&S89534[27], 6}, {NULL}};
const Segment SGI233355[] = {{&S89534[101], 9}, {NULL}};
const Segment SGI233356[] = {{&S89534[74], 6}, {NULL}};
const Segment SGI233357[] = {{&S99503[44], 12}, {NULL}};
const Segment SGI233358[] = {{&S99503[44], 24}, {NULL}};
const Segment SGI233359[] = {{&S99503[30], 6}, {NULL}};
const Segment SGI233360[] = {{&S99503[82], 6}, {NULL}};
const Segment SGI233361[] = {{&S99503[109], 13}, {NULL}};
const Segment SGI233362[] = {{&S99510[44], 10}, {NULL}};
const Segment SGI233363[] = {{&S99510[44], 22}, {NULL}};
const Segment SGI233364[] = {{&S99510[30], 6}, {NULL}};
const Segment SGI233365[] = {{&S99510[80], 6}, {NULL}};
const Segment SGI233366[] = {{&S99510[107], 13}, {NULL}};
const Segment SGI233367[] = {{&S89546[34], 9}, {NULL}};
const Segment SGI233368[] = {{&S89546[27], 6}, {NULL}};
const Segment SGI233369[] = {{&S89546[96], 9}, {NULL}};
const Segment SGI233370[] = {{&S89546[69], 6}, {NULL}};
const Segment SGI233371[] = {{&S89546[137], 8}, {NULL}};
const Segment SGI233372[] = {{&S89547[34], 9}, {NULL}};
const Segment SGI233373[] = {{&S89547[27], 6}, {NULL}};
const Segment SGI233374[] = {{&S89547[96], 9}, {NULL}};
const Segment SGI233375[] = {{&S89547[69], 6}, {NULL}};
const Segment SGI233376[] = {{&S89547[137], 8}, {NULL}};
const Segment SGI233377[] = {{&S99517[34], 9}, {NULL}};
const Segment SGI233378[] = {{&S99517[27], 6}, {NULL}};
const Segment SGI233379[] = {{&S99517[96], 9}, {NULL}};
const Segment SGI233380[] = {{&S99517[69], 6}, {NULL}};
const Segment SGI233381[] = {{&S99517[130], 8}, {NULL}};
const Segment SGI233382[] = {{&S96262[44], 15}, {NULL}};
const Segment SGI233383[] = {{&S96262[44], 27}, {NULL}};
const Segment SGI233384[] = {{&S96262[30], 6}, {NULL}};
const Segment SGI233385[] = {{&S96262[120], 3}, {NULL}};
const Segment SGI233386[] = {{&S96262[85], 6}, {NULL}};
const Segment SGI233387[] = {{&S96262[140], 8}, {NULL}};
const Segment SGI233388[] = {{&S96262[149], 13}, {NULL}};
const Segment SGI233389[] = {{&S99519[34], 7}, {NULL}};
const Segment SGI233390[] = {{&S99519[27], 6}, {NULL}};
const Segment SGI233391[] = {{&S99519[99], 9}, {NULL}};
const Segment SGI233392[] = {{&S99519[72], 6}, {NULL}};
const Segment SGI233393[] = {{&S99519[140], 8}, {NULL}};
const Segment SGI233394[] = {{&S96281[44], 10}, {NULL}};
const Segment SGI233395[] = {{&S96281[44], 27}, {NULL}};
const Segment SGI233396[] = {{&S96281[30], 6}, {NULL}};
const Segment SGI233397[] = {{&S96281[85], 6}, {NULL}};
const Segment SGI233398[] = {{&S96281[112], 13}, {NULL}};
const Segment SGI233399[] = {{&S90716[34], 11}, {NULL}};
const Segment SGI233400[] = {{&S90716[27], 6}, {NULL}};
const Segment SGI233401[] = {{&S90716[98], 9}, {NULL}};
const Segment SGI233402[] = {{&S90716[71], 6}, {NULL}};
const Segment SGI233403[] = {{&S81837[34], 10}, {NULL}};
const Segment SGI233404[] = {{&S81837[27], 6}, {NULL}};
const Segment SGI233405[] = {{&S81837[97], 9}, {NULL}};
const Segment SGI233406[] = {{&S81837[70], 6}, {NULL}};
const Segment SGI233407[] = {{&S81839[44], 10}, {NULL}};
const Segment SGI233408[] = {{&S81839[44], 21}, {NULL}};
const Segment SGI233409[] = {{&S81839[30], 6}, {NULL}};
const Segment SGI233410[] = {{&S81839[79], 6}, {NULL}};
const Segment SGI233411[] = {{&S81839[106], 13}, {NULL}};
const Segment SGI233412[] = {{&S89594[34], 14}, {NULL}};
const Segment SGI233413[] = {{&S89594[27], 6}, {NULL}};
const Segment SGI233414[] = {{&S89594[101], 9}, {NULL}};
const Segment SGI233415[] = {{&S89594[74], 6}, {NULL}};
const Segment SGI233416[] = {{&S89620[34], 24}, {NULL}};
const Segment SGI233417[] = {{&S89620[27], 6}, {NULL}};
const Segment SGI233418[] = {{&S89620[111], 9}, {NULL}};
const Segment SGI233419[] = {{&S89620[84], 6}, {NULL}};
const Segment SGI233420[] = {{&S89626[34], 7}, {NULL}};
const Segment SGI233421[] = {{&S89626[27], 6}, {NULL}};
const Segment SGI233422[] = {{&S89626[100], 9}, {NULL}};
const Segment SGI233423[] = {{&S89626[73], 6}, {NULL}};
const Segment SGI233424[] = {{&S89630[34], 8}, {NULL}};
const Segment SGI233425[] = {{&S89630[27], 6}, {NULL}};
const Segment SGI233426[] = {{&S89630[95], 9}, {NULL}};
const Segment SGI233427[] = {{&S89630[68], 6}, {NULL}};
const Segment SGI233428[] = {{&S89634[45], 10}, {NULL}};
const Segment SGI233429[] = {{&S89634[30], 6}, {NULL}};
const Segment SGI233430[] = {{&S89634[115], 3}, {NULL}};
const Segment SGI233431[] = {{&S89634[81], 5}, {NULL}};
const Segment SGI233432[] = {{&S89634[139], 13}, {NULL}};
const Segment SGI233433[] = {{&S89638[34], 11}, {NULL}};
const Segment SGI233434[] = {{&S89638[27], 6}, {NULL}};
const Segment SGI233435[] = {{&S89638[98], 9}, {NULL}};
const Segment SGI233436[] = {{&S89638[71], 6}, {NULL}};
const Segment SGI233437[] = {{&S89650[34], 19}, {NULL}};
const Segment SGI233438[] = {{&S89650[27], 6}, {NULL}};
const Segment SGI233439[] = {{&S89650[106], 9}, {NULL}};
const Segment SGI233440[] = {{&S89650[79], 6}, {NULL}};
const Segment SGI233441[] = {{&S89653[44], 18}, {NULL}};
const Segment SGI233442[] = {{&S89653[44], 30}, {NULL}};
const Segment SGI233443[] = {{&S89653[30], 6}, {NULL}};
const Segment SGI233444[] = {{&S89653[139], 3}, {NULL}};
const Segment SGI233445[] = {{&S89653[104], 6}, {NULL}};
const Segment SGI233446[] = {{&S89653[76], 8}, {NULL}};
const Segment SGI233447[] = {{&S89653[171], 15}, {NULL}};
const Segment SGI233448[] = {{&S89654[45], 19}, {NULL}};
const Segment SGI233449[] = {{&S89654[30], 6}, {NULL}};
const Segment SGI233450[] = {{&S99635[44], 15}, {NULL}};
const Segment SGI233451[] = {{&S99635[44], 27}, {NULL}};
const Segment SGI233452[] = {{&S99635[30], 6}, {NULL}};
const Segment SGI233453[] = {{&S89654[124], 3}, {NULL}};
const Segment SGI233454[] = {{&S89654[90], 5}, {NULL}};
const Segment SGI233455[] = {{&S99635[31], 3}, {NULL}};
const Segment SGI233456[] = {{&S89654[148], 13}, {NULL}};
const Segment SGI233457[] = {{&S99635[85], 6}, {NULL}};
const Segment SGI233458[] = {{&S89661[44], 7}, {NULL}};
const Segment SGI233459[] = {{&S89661[44], 19}, {NULL}};
const Segment SGI233460[] = {{&S89661[30], 6}, {NULL}};
const Segment SGI233461[] = {{&S89661[112], 3}, {NULL}};
const Segment SGI233462[] = {{&S89661[77], 6}, {NULL}};
const Segment SGI233463[] = {{&S89663[44], 10}, {NULL}};
const Segment SGI233464[] = {{&S89663[44], 22}, {NULL}};
const Segment SGI233465[] = {{&S89663[30], 6}, {NULL}};
const Segment SGI233466[] = {{&S89663[115], 3}, {NULL}};
const Segment SGI233467[] = {{&S89663[80], 6}, {NULL}};
const Segment SGI233468[] = {{&S81928[44], 7}, {NULL}};
const Segment SGI233469[] = {{&S81928[44], 18}, {NULL}};
const Segment SGI233470[] = {{&S81928[30], 6}, {NULL}};
const Segment SGI233471[] = {{&S81928[111], 3}, {NULL}};
const Segment SGI233472[] = {{&S81928[76], 6}, {NULL}};
const Segment SGI233473[] = {{&S89666[52], 10}, {NULL}};
const Segment SGI233474[] = {{&S89666[52], 22}, {NULL}};
const Segment SGI233475[] = {{&S89666[38], 6}, {NULL}};
const Segment SGI233476[] = {{&S89666[88], 5}, {NULL}};
const Segment SGI233477[] = {{&S89666[11], 3}, {NULL}};
const Segment SGI233478[] = {{&S89669[34], 8}, {NULL}};
const Segment SGI233479[] = {{&S89669[27], 6}, {NULL}};
const Segment SGI233480[] = {{&S99655[44], 10}, {NULL}};
const Segment SGI233481[] = {{&S99655[44], 22}, {NULL}};
const Segment SGI233482[] = {{&S89669[102], 9}, {NULL}};
const Segment SGI233483[] = {{&S99655[30], 6}, {NULL}};
const Segment SGI233484[] = {{&S89669[75], 6}, {NULL}};
const Segment SGI233485[] = {{&S99655[31], 3}, {NULL}};
const Segment SGI233486[] = {{&S99655[80], 6}, {NULL}};
const Segment SGI233487[] = {{&S89671[44], 10}, {NULL}};
const Segment SGI233488[] = {{&S89671[44], 22}, {NULL}};
const Segment SGI233489[] = {{&S89671[30], 6}, {NULL}};
const Segment SGI233490[] = {{&S89671[115], 3}, {NULL}};
const Segment SGI233491[] = {{&S89671[80], 6}, {NULL}};
const Segment SGI233492[] = {{&S89688[34], 7}, {NULL}};
const Segment SGI233493[] = {{&S89688[27], 6}, {NULL}};
const Segment SGI233494[] = {{&S89688[97], 9}, {NULL}};
const Segment SGI233495[] = {{&S89688[70], 6}, {NULL}};
const Segment SGI233496[] = {{&S89688[138], 8}, {NULL}};
const Segment SGI233497[] = {{&S99678[34], 7}, {NULL}};
const Segment SGI233498[] = {{&S99678[27], 6}, {NULL}};
const Segment SGI233499[] = {{&S89691[34], 10}, {NULL}};
const Segment SGI233500[] = {{&S89691[27], 6}, {NULL}};
const Segment SGI233501[] = {{&S99678[90], 9}, {NULL}};
const Segment SGI233502[] = {{&S99678[63], 6}, {NULL}};
const Segment SGI233503[] = {{&S89691[97], 9}, {NULL}};
const Segment SGI233504[] = {{&S89691[70], 6}, {NULL}};
const Segment SGI233505[] = {{&S99684[44], 14}, {NULL}};
const Segment SGI233506[] = {{&S99684[44], 39}, {NULL}};
const Segment SGI233507[] = {{&S99684[30], 6}, {NULL}};
const Segment SGI233508[] = {{&S99684[31], 3}, {NULL}};
const Segment SGI233509[] = {{&S99684[97], 6}, {NULL}};
const Segment SGI233510[] = {{&S99684[156], 10}, {NULL}};
const Segment SGI233511[] = {{&S90856[34], 7}, {NULL}};
const Segment SGI233512[] = {{&S90856[27], 6}, {NULL}};
const Segment SGI233513[] = {{&S90856[94], 9}, {NULL}};
const Segment SGI233514[] = {{&S90856[67], 6}, {NULL}};
const Segment SGI233515[] = {{&S99702[45], 10}, {NULL}};
const Segment SGI233516[] = {{&S99702[30], 6}, {NULL}};
const Segment SGI233517[] = {{&S99702[31], 3}, {NULL}};
const Segment SGI233518[] = {{&S99702[81], 5}, {NULL}};
const Segment SGI233519[] = {{&S99702[139], 13}, {NULL}};
const Segment SGI233520[] = {{&S99704[44], 10}, {NULL}};
const Segment SGI233521[] = {{&S99704[44], 22}, {NULL}};
const Segment SGI233522[] = {{&S99704[30], 6}, {NULL}};
const Segment SGI233523[] = {{&S99704[31], 3}, {NULL}};
const Segment SGI233524[] = {{&S99704[80], 6}, {NULL}};
const Segment SGI233525[] = {{&S81995[44], 10}, {NULL}};
const Segment SGI233526[] = {{&S99704[132], 10}, {NULL}};
const Segment SGI233527[] = {{&S81995[44], 21}, {NULL}};
const Segment SGI233528[] = {{&S81995[30], 6}, {NULL}};
const Segment SGI233529[] = {{&S81995[114], 3}, {NULL}};
const Segment SGI233530[] = {{&S81995[79], 6}, {NULL}};
const Segment SGI233531[] = {{&S81996[44], 10}, {NULL}};
const Segment SGI233532[] = {{&S81996[44], 22}, {NULL}};
const Segment SGI233533[] = {{&S81996[30], 6}, {NULL}};
const Segment SGI233534[] = {{&S81996[115], 3}, {NULL}};
const Segment SGI233535[] = {{&S81996[80], 6}, {NULL}};
const Segment SGI233536[] = {{&S90882[34], 7}, {NULL}};
const Segment SGI233537[] = {{&S90882[27], 6}, {NULL}};
const Segment SGI233538[] = {{&S90882[94], 9}, {NULL}};
const Segment SGI233539[] = {{&S90882[67], 6}, {NULL}};
const Segment SGI233540[] = {{&S90884[44], 26}, {NULL}};
const Segment SGI233541[] = {{&S90884[44], 38}, {NULL}};
const Segment SGI233542[] = {{&S90884[30], 6}, {NULL}};
const Segment SGI233543[] = {{&S90884[131], 3}, {NULL}};
const Segment SGI233544[] = {{&S90884[96], 6}, {NULL}};
const Segment SGI233545[] = {{&S90885[44], 13}, {NULL}};
const Segment SGI233546[] = {{&S90885[44], 25}, {NULL}};
const Segment SGI233547[] = {{&S90885[30], 6}, {NULL}};
const Segment SGI233548[] = {{&S90885[118], 3}, {NULL}};
const Segment SGI233549[] = {{&S90885[83], 6}, {NULL}};
const Segment SGI233550[] = {{&S99719[45], 16}, {NULL}};
const Segment SGI233551[] = {{&S99719[30], 6}, {NULL}};
const Segment SGI233552[] = {{&S99719[31], 3}, {NULL}};
const Segment SGI233553[] = {{&S99719[87], 5}, {NULL}};
const Segment SGI233554[] = {{&S99719[145], 13}, {NULL}};
const Segment SGI233555[] = {{&S82013[34], 12}, {NULL}};
const Segment SGI233556[] = {{&S99721[34], 7}, {NULL}};
const Segment SGI233557[] = {{&S82013[27], 6}, {NULL}};
const Segment SGI233558[] = {{&S99721[27], 6}, {NULL}};
const Segment SGI233559[] = {{&S82013[98], 9}, {NULL}};
const Segment SGI233560[] = {{&S82013[71], 6}, {NULL}};
const Segment SGI233561[] = {{&S99721[94], 9}, {NULL}};
const Segment SGI233562[] = {{&S99721[67], 6}, {NULL}};
const Segment SGI233563[] = {{&S99728[34], 7}, {NULL}};
const Segment SGI233564[] = {{&S99728[27], 6}, {NULL}};
const Segment SGI233565[] = {{&S99728[103], 9}, {NULL}};
const Segment SGI233566[] = {{&S99728[76], 6}, {NULL}};
const Segment SGI233567[] = {{&S99732[34], 9}, {NULL}};
const Segment SGI233568[] = {{&S99732[27], 6}, {NULL}};
const Segment SGI233569[] = {{&S99732[96], 9}, {NULL}};
const Segment SGI233570[] = {{&S99732[69], 6}, {NULL}};
const Segment SGI233571[] = {{&S99733[34], 10}, {NULL}};
const Segment SGI233572[] = {{&S99733[27], 6}, {NULL}};
const Segment SGI233573[] = {{&S99733[97], 9}, {NULL}};
const Segment SGI233574[] = {{&S99733[70], 6}, {NULL}};
const Segment SGI233575[] = {{&S96349[44], 14}, {NULL}};
const Segment SGI233576[] = {{&S89767[44], 10}, {NULL}};
const Segment SGI233577[] = {{&S96349[44], 26}, {NULL}};
const Segment SGI233578[] = {{&S96349[30], 6}, {NULL}};
const Segment SGI233579[] = {{&S89767[44], 22}, {NULL}};
const Segment SGI233580[] = {{&S89767[30], 6}, {NULL}};
const Segment SGI233581[] = {{&S96349[119], 3}, {NULL}};
const Segment SGI233582[] = {{&S96349[84], 6}, {NULL}};
const Segment SGI233583[] = {{&S89767[115], 3}, {NULL}};
const Segment SGI233584[] = {{&S89767[80], 6}, {NULL}};
const Segment SGI233585[] = {{&S99739[34], 8}, {NULL}};
const Segment SGI233586[] = {{&S99739[27], 6}, {NULL}};
const Segment SGI233587[] = {{&S89767[143], 3}, {NULL}};
const Segment SGI233588[] = {{&S99739[95], 9}, {NULL}};
const Segment SGI233589[] = {{&S99739[68], 6}, {NULL}};
const Segment SGI233590[] = {{&S96354[34], 18}, {NULL}};
const Segment SGI233591[] = {{&S96354[27], 6}, {NULL}};
const Segment SGI233592[] = {{&S96354[105], 9}, {NULL}};
const Segment SGI233593[] = {{&S96354[78], 6}, {NULL}};
const Segment SGI233594[] = {{&S89776[61], 10}, {NULL}};
const Segment SGI233595[] = {{&S89776[61], 22}, {NULL}};
const Segment SGI233596[] = {{&S89776[47], 6}, {NULL}};
const Segment SGI233597[] = {{&S89776[132], 3}, {NULL}};
const Segment SGI233598[] = {{&S99748[44], 14}, {NULL}};
const Segment SGI233599[] = {{&S89776[97], 6}, {NULL}};
const Segment SGI233600[] = {{&S99748[44], 31}, {NULL}};
const Segment SGI233601[] = {{&S99748[30], 6}, {NULL}};
const Segment SGI233602[] = {{&S89776[11], 3}, {NULL}};
const Segment SGI233603[] = {{&S99748[31], 3}, {NULL}};
const Segment SGI233604[] = {{&S99748[89], 5}, {NULL}};
const Segment SGI233605[] = {{&S99748[126], 14}, {NULL}};
const Segment SGI233606[] = {{&S99749[44], 18}, {NULL}};
const Segment SGI233607[] = {{&S99749[44], 30}, {NULL}};
const Segment SGI233608[] = {{&S99749[30], 6}, {NULL}};
const Segment SGI233609[] = {{&S99749[31], 3}, {NULL}};
const Segment SGI233610[] = {{&S99749[88], 6}, {NULL}};
const Segment SGI233611[] = {{&S99749[148], 11}, {NULL}};
const Segment SGI233612[] = {{&S96366[45], 14}, {NULL}};
const Segment SGI233613[] = {{&S96366[30], 6}, {NULL}};
const Segment SGI233614[] = {{&S96366[119], 3}, {NULL}};
const Segment SGI233615[] = {{&S96366[85], 5}, {NULL}};
const Segment SGI233616[] = {{&S96366[143], 13}, {NULL}};
const Segment SGI233617[] = {{&S89788[44], 9}, {NULL}};
const Segment SGI233618[] = {{&S89788[44], 21}, {NULL}};
const Segment SGI233619[] = {{&S89788[30], 6}, {NULL}};
const Segment SGI233620[] = {{&S89788[114], 3}, {NULL}};
const Segment SGI233621[] = {{&S104411[34], 10}, {NULL}};
const Segment SGI233622[] = {{&S89788[79], 6}, {NULL}};
const Segment SGI233623[] = {{&S104411[27], 6}, {NULL}};
const Segment SGI233624[] = {{&S104411[70], 6}, {NULL}};
const Segment SGI233625[] = {{&S89788[139], 11}, {NULL}};
const Segment SGI233626[] = {{&S82064[34], 10}, {NULL}};
const Segment SGI233627[] = {{&S82064[27], 6}, {NULL}};
const Segment SGI233628[] = {{&S82064[96], 9}, {NULL}};
const Segment SGI233629[] = {{&S82064[69], 6}, {NULL}};
const Segment SGI233630[] = {{&S96375[34], 15}, {NULL}};
const Segment SGI233631[] = {{&S96375[27], 6}, {NULL}};
const Segment SGI233632[] = {{&S82065[34], 10}, {NULL}};
const Segment SGI233633[] = {{&S82065[27], 6}, {NULL}};
const Segment SGI233634[] = {{&S96375[102], 9}, {NULL}};
const Segment SGI233635[] = {{&S99764[45], 10}, {NULL}};
const Segment SGI233636[] = {{&S96375[75], 6}, {NULL}};
const Segment SGI233637[] = {{&S99764[30], 6}, {NULL}};
const Segment SGI233638[] = {{&S82065[96], 9}, {NULL}};
const Segment SGI233639[] = {{&S82065[69], 6}, {NULL}};
const Segment SGI233640[] = {{&S99764[31], 3}, {NULL}};
const Segment SGI233641[] = {{&S99764[81], 5}, {NULL}};
const Segment SGI233642[] = {{&S99764[139], 13}, {NULL}};
const Segment SGI233643[] = {{&S99766[45], 14}, {NULL}};
const Segment SGI233644[] = {{&S99766[30], 6}, {NULL}};
const Segment SGI233645[] = {{&S99766[31], 3}, {NULL}};
const Segment SGI233646[] = {{&S99766[85], 5}, {NULL}};
const Segment SGI233647[] = {{&S99766[143], 13}, {NULL}};
const Segment SGI233648[] = {{&S82075[44], 18}, {NULL}};
const Segment SGI233649[] = {{&S82075[44], 29}, {NULL}};
const Segment SGI233650[] = {{&S82075[30], 6}, {NULL}};
const Segment SGI233651[] = {{&S82075[157], 3}, {NULL}};
const Segment SGI233652[] = {{&S82075[122], 6}, {NULL}};
const Segment SGI233653[] = {{&S82075[75], 8}, {NULL}};
const Segment SGI233654[] = {{&S82076[44], 18}, {NULL}};
const Segment SGI233655[] = {{&S82076[44], 53}, {NULL}};
const Segment SGI233656[] = {{&S89805[34], 9}, {NULL}};
const Segment SGI233657[] = {{&S82076[30], 6}, {NULL}};
const Segment SGI233658[] = {{&S89805[27], 6}, {NULL}};
const Segment SGI233659[] = {{&S82076[166], 3}, {NULL}};
const Segment SGI233660[] = {{&S89805[96], 9}, {NULL}};
const Segment SGI233661[] = {{&S82076[131], 6}, {NULL}};
const Segment SGI233662[] = {{&S89805[69], 6}, {NULL}};
const Segment SGI233663[] = {{&S82076[99], 8}, {NULL}};
const Segment SGI233664[] = {{&S82077[44], 18}, {NULL}};
const Segment SGI233665[] = {{&S82077[44], 32}, {NULL}};
const Segment SGI233666[] = {{&S82077[30], 6}, {NULL}};
const Segment SGI233667[] = {{&S82077[125], 3}, {NULL}};
const Segment SGI233668[] = {{&S82077[90], 6}, {NULL}};
const Segment SGI233669[] = {{&S82077[150], 8}, {NULL}};
const Segment SGI233670[] = {{&S96392[44], 10}, {NULL}};
const Segment SGI233671[] = {{&S96392[44], 22}, {NULL}};
const Segment SGI233672[] = {{&S96392[30], 6}, {NULL}};
const Segment SGI233673[] = {{&S96392[115], 3}, {NULL}};
const Segment SGI233674[] = {{&S96392[80], 6}, {NULL}};
const Segment SGI233675[] = {{&S96392[133], 13}, {NULL}};
const Segment SGI233676[] = {{&S99781[34], 14}, {NULL}};
const Segment SGI233677[] = {{&S99781[27], 6}, {NULL}};
const Segment SGI233678[] = {{&S99781[101], 9}, {NULL}};
const Segment SGI233679[] = {{&S99781[74], 6}, {NULL}};
const Segment SGI233680[] = {{&S96395[45], 10}, {NULL}};
const Segment SGI233681[] = {{&S96395[30], 6}, {NULL}};
const Segment SGI233682[] = {{&S96395[115], 3}, {NULL}};
const Segment SGI233683[] = {{&S96395[81], 5}, {NULL}};
const Segment SGI233684[] = {{&S96395[139], 13}, {NULL}};
const Segment SGI233685[] = {{&S89815[34], 18}, {NULL}};
const Segment SGI233686[] = {{&S89815[27], 6}, {NULL}};
const Segment SGI233687[] = {{&S99784[34], 14}, {NULL}};
const Segment SGI233688[] = {{&S99784[27], 6}, {NULL}};
const Segment SGI233689[] = {{&S89815[105], 9}, {NULL}};
const Segment SGI233690[] = {{&S89815[78], 6}, {NULL}};
const Segment SGI233691[] = {{&S99784[101], 9}, {NULL}};
const Segment SGI233692[] = {{&S99784[74], 6}, {NULL}};
const Segment SGI233693[] = {{&S82089[44], 9}, {NULL}};
const Segment SGI233694[] = {{&S82089[44], 20}, {NULL}};
const Segment SGI233695[] = {{&S82089[30], 6}, {NULL}};
const Segment SGI233696[] = {{&S82089[113], 3}, {NULL}};
const Segment SGI233697[] = {{&S82089[78], 6}, {NULL}};
const Segment SGI233698[] = {{&S99787[44], 7}, {NULL}};
const Segment SGI233699[] = {{&S99787[44], 19}, {NULL}};
const Segment SGI233700[] = {{&S82089[138], 11}, {NULL}};
const Segment SGI233701[] = {{&S99787[30], 6}, {NULL}};
const Segment SGI233702[] = {{&S99787[31], 3}, {NULL}};
const Segment SGI233703[] = {{&S99787[77], 5}, {NULL}};
const Segment SGI233704[] = {{&S99787[114], 14}, {NULL}};
const Segment SGI233705[] = {{&S89821[34], 10}, {NULL}};
const Segment SGI233706[] = {{&S89821[27], 6}, {NULL}};
const Segment SGI233707[] = {{&S89821[96], 9}, {NULL}};
const Segment SGI233708[] = {{&S89821[69], 6}, {NULL}};
const Segment SGI233709[] = {{&S89824[44], 13}, {NULL}};
const Segment SGI233710[] = {{&S89824[44], 25}, {NULL}};
const Segment SGI233711[] = {{&S89824[30], 6}, {NULL}};
const Segment SGI233712[] = {{&S89824[134], 3}, {NULL}};
const Segment SGI233713[] = {{&S89824[99], 6}, {NULL}};
const Segment SGI233714[] = {{&S96408[34], 12}, {NULL}};
const Segment SGI233715[] = {{&S89824[71], 8}, {NULL}};
const Segment SGI233716[] = {{&S96408[27], 6}, {NULL}};
const Segment SGI233717[] = {{&S96408[99], 9}, {NULL}};
const Segment SGI233718[] = {{&S96408[72], 6}, {NULL}};
const Segment SGI233719[] = {{&S89825[52], 7}, {NULL}};
const Segment SGI233720[] = {{&S89825[52], 24}, {NULL}};
const Segment SGI233721[] = {{&S89825[38], 6}, {NULL}};
const Segment SGI233722[] = {{&S89825[90], 5}, {NULL}};
const Segment SGI233723[] = {{&S99795[34], 14}, {NULL}};
const Segment SGI233724[] = {{&S89825[11], 3}, {NULL}};
const Segment SGI233725[] = {{&S99795[27], 6}, {NULL}};
const Segment SGI233726[] = {{&S99795[101], 9}, {NULL}};
const Segment SGI233727[] = {{&S99795[74], 6}, {NULL}};
const Segment SGI233728[] = {{&S89826[52], 7}, {NULL}};
const Segment SGI233729[] = {{&S89826[52], 24}, {NULL}};
const Segment SGI233730[] = {{&S89826[38], 6}, {NULL}};
const Segment SGI233731[] = {{&S89826[90], 5}, {NULL}};
const Segment SGI233732[] = {{&S89826[11], 3}, {NULL}};
const Segment SGI233733[] = {{&S99797[34], 14}, {NULL}};
const Segment SGI233734[] = {{&S99797[27], 6}, {NULL}};
const Segment SGI233735[] = {{&S99797[101], 9}, {NULL}};
const Segment SGI233736[] = {{&S99797[74], 6}, {NULL}};
const Segment SGI233737[] = {{&S89831[44], 9}, {NULL}};
const Segment SGI233738[] = {{&S89831[44], 21}, {NULL}};
const Segment SGI233739[] = {{&S89831[30], 6}, {NULL}};
const Segment SGI233740[] = {{&S99800[34], 24}, {NULL}};
const Segment SGI233741[] = {{&S89831[114], 3}, {NULL}};
const Segment SGI233742[] = {{&S99800[27], 6}, {NULL}};
const Segment SGI233743[] = {{&S89831[79], 6}, {NULL}};
const Segment SGI233744[] = {{&S89831[139], 7}, {NULL}};
const Segment SGI233745[] = {{&S99800[111], 9}, {NULL}};
const Segment SGI233746[] = {{&S99800[84], 6}, {NULL}};
const Segment SGI233747[] = {{&S96423[34], 22}, {NULL}};
const Segment SGI233748[] = {{&S96423[27], 6}, {NULL}};
const Segment SGI233749[] = {{&S96423[109], 9}, {NULL}};
const Segment SGI233750[] = {{&S96423[82], 6}, {NULL}};
const Segment SGI233751[] = {{&S99806[34], 10}, {NULL}};
const Segment SGI233752[] = {{&S89838[34], 6}, {NULL}};
const Segment SGI233753[] = {{&S99806[27], 6}, {NULL}};
const Segment SGI233754[] = {{&S89838[27], 6}, {NULL}};
const Segment SGI233755[] = {{&S99806[97], 9}, {NULL}};
const Segment SGI233756[] = {{&S89838[94], 9}, {NULL}};
const Segment SGI233757[] = {{&S89838[67], 6}, {NULL}};
const Segment SGI233758[] = {{&S99806[70], 6}, {NULL}};
const Segment SGI233759[] = {{&S99810[44], 7}, {NULL}};
const Segment SGI233760[] = {{&S99810[44], 25}, {NULL}};
const Segment SGI233761[] = {{&S99810[30], 6}, {NULL}};
const Segment SGI233762[] = {{&S99810[83], 6}, {NULL}};
const Segment SGI233763[] = {{&S99810[110], 13}, {NULL}};
const Segment SGI233764[] = {{&S99811[34], 15}, {NULL}};
const Segment SGI233765[] = {{&S99811[27], 6}, {NULL}};
const Segment SGI233766[] = {{&S99811[102], 9}, {NULL}};
const Segment SGI233767[] = {{&S99811[75], 6}, {NULL}};
const Segment SGI233768[] = {{&S99813[34], 10}, {NULL}};
const Segment SGI233769[] = {{&S99813[27], 6}, {NULL}};
const Segment SGI233770[] = {{&S99813[97], 9}, {NULL}};
const Segment SGI233771[] = {{&S99813[70], 6}, {NULL}};
const Segment SGI233772[] = {{&S99814[34], 15}, {NULL}};
const Segment SGI233773[] = {{&S99814[27], 6}, {NULL}};
const Segment SGI233774[] = {{&S99814[102], 9}, {NULL}};
const Segment SGI233775[] = {{&S99814[75], 6}, {NULL}};
const Segment SGI233776[] = {{&S99821[34], 7}, {NULL}};
const Segment SGI233777[] = {{&S99821[27], 6}, {NULL}};
const Segment SGI233778[] = {{&S99821[99], 9}, {NULL}};
const Segment SGI233779[] = {{&S99821[72], 6}, {NULL}};
const Segment SGI233780[] = {{&S99821[140], 8}, {NULL}};
const Segment SGI233781[] = {{&S99822[44], 7}, {NULL}};
const Segment SGI233782[] = {{&S99822[44], 23}, {NULL}};
const Segment SGI233783[] = {{&S99822[30], 6}, {NULL}};
const Segment SGI233784[] = {{&S99822[31], 3}, {NULL}};
const Segment SGI233785[] = {{&S99822[81], 5}, {NULL}};
const Segment SGI233786[] = {{&S99822[118], 14}, {NULL}};
const Segment SGI233787[] = {{&S99823[34], 8}, {NULL}};
const Segment SGI233788[] = {{&S99823[27], 6}, {NULL}};
const Segment SGI233789[] = {{&S99823[106], 9}, {NULL}};
const Segment SGI233790[] = {{&S99823[79], 6}, {NULL}};
const Segment SGI233791[] = {{&S99823[147], 8}, {NULL}};
const Segment SGI233792[] = {{&S82146[34], 23}, {NULL}};
const Segment SGI233793[] = {{&S82146[27], 6}, {NULL}};
const Segment SGI233794[] = {{&S82146[110], 9}, {NULL}};
const Segment SGI233795[] = {{&S82146[83], 6}, {NULL}};
const Segment SGI233796[] = {{&S99826[34], 7}, {NULL}};
const Segment SGI233797[] = {{&S99826[27], 6}, {NULL}};
const Segment SGI233798[] = {{&S99826[94], 9}, {NULL}};
const Segment SGI233799[] = {{&S99826[67], 6}, {NULL}};
const Segment SGI233800[] = {{&S99826[135], 8}, {NULL}};
const Segment SGI233801[] = {{&S82149[34], 13}, {NULL}};
const Segment SGI233802[] = {{&S99827[34], 4}, {NULL}};
const Segment SGI233803[] = {{&S82149[27], 6}, {NULL}};
const Segment SGI233804[] = {{&S99827[27], 6}, {NULL}};
const Segment SGI233805[] = {{&S82149[100], 9}, {NULL}};
const Segment SGI233806[] = {{&S82149[73], 6}, {NULL}};
const Segment SGI233807[] = {{&S99827[91], 9}, {NULL}};
const Segment SGI233808[] = {{&S99827[64], 6}, {NULL}};
const Segment SGI233809[] = {{&S99827[132], 8}, {NULL}};
const Segment SGI233810[] = {{&S82150[34], 12}, {NULL}};
const Segment SGI233811[] = {{&S82150[27], 6}, {NULL}};
const Segment SGI233812[] = {{&S82150[99], 9}, {NULL}};
const Segment SGI233813[] = {{&S82150[72], 6}, {NULL}};
const Segment SGI233814[] = {{&S82151[34], 9}, {NULL}};
const Segment SGI233815[] = {{&S82151[27], 6}, {NULL}};
const Segment SGI233816[] = {{&S82151[95], 9}, {NULL}};
const Segment SGI233817[] = {{&S82151[68], 6}, {NULL}};
const Segment SGI233818[] = {{&S99832[44], 10}, {NULL}};
const Segment SGI233819[] = {{&S99832[44], 22}, {NULL}};
const Segment SGI233820[] = {{&S99832[30], 6}, {NULL}};
const Segment SGI233821[] = {{&S99832[31], 3}, {NULL}};
const Segment SGI233822[] = {{&S99832[80], 5}, {NULL}};
const Segment SGI233823[] = {{&S99832[117], 14}, {NULL}};
const Segment SGI233824[] = {{&S99835[34], 10}, {NULL}};
const Segment SGI233825[] = {{&S99835[27], 6}, {NULL}};
const Segment SGI233826[] = {{&S99835[97], 9}, {NULL}};
const Segment SGI233827[] = {{&S99835[70], 6}, {NULL}};
const Segment SGI233828[] = {{&S99836[34], 8}, {NULL}};
const Segment SGI233829[] = {{&S99836[27], 6}, {NULL}};
const Segment SGI233830[] = {{&S99836[95], 9}, {NULL}};
const Segment SGI233831[] = {{&S99836[68], 6}, {NULL}};
const Segment SGI233832[] = {{&S82166[44], 10}, {NULL}};
const Segment SGI233833[] = {{&S82166[44], 21}, {NULL}};
const Segment SGI233834[] = {{&S82166[30], 6}, {NULL}};
const Segment SGI233835[] = {{&S82166[114], 3}, {NULL}};
const Segment SGI233836[] = {{&S82166[79], 6}, {NULL}};
const Segment SGI233837[] = {{&S96480[34], 7}, {NULL}};
const Segment SGI233838[] = {{&S96480[27], 6}, {NULL}};
const Segment SGI233839[] = {{&S96480[94], 9}, {NULL}};
const Segment SGI233840[] = {{&S96480[67], 6}, {NULL}};
const Segment SGI233841[] = {{&S82175[34], 14}, {NULL}};
const Segment SGI233842[] = {{&S82175[27], 6}, {NULL}};
const Segment SGI233843[] = {{&S82175[100], 9}, {NULL}};
const Segment SGI233844[] = {{&S82175[73], 6}, {NULL}};
const Segment SGI233845[] = {{&S99853[44], 9}, {NULL}};
const Segment SGI233846[] = {{&S99853[44], 21}, {NULL}};
const Segment SGI233847[] = {{&S99853[30], 6}, {NULL}};
const Segment SGI233848[] = {{&S82176[34], 14}, {NULL}};
const Segment SGI233849[] = {{&S99853[31], 3}, {NULL}};
const Segment SGI233850[] = {{&S82176[27], 6}, {NULL}};
const Segment SGI233851[] = {{&S99853[120], 6}, {NULL}};
const Segment SGI233852[] = {{&S99853[67], 8}, {NULL}};
const Segment SGI233853[] = {{&S82176[100], 9}, {NULL}};
const Segment SGI233854[] = {{&S82176[73], 6}, {NULL}};
const Segment SGI233855[] = {{&S99863[44], 7}, {NULL}};
const Segment SGI233856[] = {{&S99863[44], 19}, {NULL}};
const Segment SGI233857[] = {{&S99863[30], 6}, {NULL}};
const Segment SGI233858[] = {{&S96494[34], 10}, {NULL}};
const Segment SGI233859[] = {{&S99863[31], 3}, {NULL}};
const Segment SGI233860[] = {{&S96494[27], 6}, {NULL}};
const Segment SGI233861[] = {{&S99863[77], 5}, {NULL}};
const Segment SGI233862[] = {{&S99863[114], 14}, {NULL}};
const Segment SGI233863[] = {{&S96494[97], 9}, {NULL}};
const Segment SGI233864[] = {{&S96494[70], 6}, {NULL}};
const Segment SGI233865[] = {{&S99864[45], 18}, {NULL}};
const Segment SGI233866[] = {{&S99864[30], 6}, {NULL}};
const Segment SGI233867[] = {{&S99864[31], 3}, {NULL}};
const Segment SGI233868[] = {{&S99864[89], 5}, {NULL}};
const Segment SGI233869[] = {{&S99864[147], 13}, {NULL}};
const Segment SGI233870[] = {{&S99865[34], 7}, {NULL}};
const Segment SGI233871[] = {{&S99865[27], 6}, {NULL}};
const Segment SGI233872[] = {{&S99865[94], 9}, {NULL}};
const Segment SGI233873[] = {{&S99865[67], 6}, {NULL}};
const Segment SGI233874[] = {{&S99866[34], 21}, {NULL}};
const Segment SGI233875[] = {{&S99866[27], 6}, {NULL}};
const Segment SGI233876[] = {{&S99866[118], 9}, {NULL}};
const Segment SGI233877[] = {{&S99866[91], 6}, {NULL}};
const Segment SGI233878[] = {{&S89923[44], 10}, {NULL}};
const Segment SGI233879[] = {{&S89923[44], 22}, {NULL}};
const Segment SGI233880[] = {{&S89923[30], 6}, {NULL}};
const Segment SGI233881[] = {{&S89923[131], 3}, {NULL}};
const Segment SGI233882[] = {{&S89923[96], 6}, {NULL}};
const Segment SGI233883[] = {{&S89923[68], 8}, {NULL}};
const Segment SGI233884[] = {{&S96501[44], 8}, {NULL}};
const Segment SGI233885[] = {{&S96501[44], 20}, {NULL}};
const Segment SGI233886[] = {{&S96501[30], 6}, {NULL}};
const Segment SGI233887[] = {{&S96501[113], 3}, {NULL}};
const Segment SGI233888[] = {{&S96501[78], 6}, {NULL}};
const Segment SGI233889[] = {{&S96501[145], 15}, {NULL}};
const Segment SGI233890[] = {{&S99868[34], 11}, {NULL}};
const Segment SGI233891[] = {{&S99868[27], 6}, {NULL}};
const Segment SGI233892[] = {{&S89924[44], 10}, {&S89924[191], 18}, {NULL}};
const Segment SGI233893[] = {{&S99868[98], 9}, {NULL}};
const Segment SGI233894[] = {{&S89924[44], 22}, {NULL}};
const Segment SGI233895[] = {{&S99868[71], 6}, {NULL}};
const Segment SGI233896[] = {{&S89924[30], 6}, {NULL}};
const Segment SGI233897[] = {{&S89924[131], 3}, {NULL}};
const Segment SGI233898[] = {{&S89924[96], 6}, {NULL}};
const Segment SGI233899[] = {{&S89924[68], 8}, {NULL}};
const Segment SGI233900[] = {{&S96506[34], 10}, {NULL}};
const Segment SGI233901[] = {{&S96506[27], 6}, {NULL}};
const Segment SGI233902[] = {{&S96506[97], 9}, {NULL}};
const Segment SGI233903[] = {{&S96506[70], 6}, {NULL}};
const Segment SGI233904[] = {{&S89928[34], 10}, {NULL}};
const Segment SGI233905[] = {{&S89928[27], 6}, {NULL}};
const Segment SGI233906[] = {{&S89928[97], 9}, {NULL}};
const Segment SGI233907[] = {{&S89928[70], 6}, {NULL}};
const Segment SGI233908[] = {{&S99875[44], 18}, {NULL}};
const Segment SGI233909[] = {{&S99875[44], 30}, {NULL}};
const Segment SGI233910[] = {{&S99875[30], 6}, {NULL}};
const Segment SGI233911[] = {{&S99875[31], 3}, {NULL}};
const Segment SGI233912[] = {{&S99875[88], 5}, {NULL}};
const Segment SGI233913[] = {{&S99875[125], 14}, {NULL}};
const Segment SGI233914[] = {{&S99877[44], 14}, {NULL}};
const Segment SGI233915[] = {{&S99877[44], 26}, {NULL}};
const Segment SGI233916[] = {{&S99877[30], 6}, {NULL}};
const Segment SGI233917[] = {{&S99877[31], 3}, {NULL}};
const Segment SGI233918[] = {{&S99877[84], 5}, {NULL}};
const Segment SGI233919[] = {{&S99877[121], 14}, {NULL}};
const Segment SGI233920[] = {{&S99883[34], 14}, {NULL}};
const Segment SGI233921[] = {{&S99883[27], 6}, {NULL}};
const Segment SGI233922[] = {{&S89940[44], 14}, {NULL}};
const Segment SGI233923[] = {{&S99883[101], 9}, {NULL}};
const Segment SGI233924[] = {{&S99883[74], 6}, {NULL}};
const Segment SGI233925[] = {{&S89940[44], 36}, {NULL}};
const Segment SGI233926[] = {{&S89940[30], 6}, {NULL}};
const Segment SGI233927[] = {{&S99883[142], 8}, {NULL}};
const Segment SGI233928[] = {{&S89940[129], 3}, {NULL}};
const Segment SGI233929[] = {{&S89940[94], 6}, {NULL}};
const Segment SGI233930[] = {{&S89940[154], 11}, {NULL}};
const Segment SGI233931[] = {{&S99887[44], 10}, {NULL}};
const Segment SGI233932[] = {{&S99887[44], 22}, {NULL}};
const Segment SGI233933[] = {{&S99887[30], 6}, {NULL}};
const Segment SGI233934[] = {{&S99887[80], 6}, {NULL}};
const Segment SGI233935[] = {{&S99887[107], 13}, {NULL}};
const Segment SGI233936[] = {{&S96525[44], 11}, {NULL}};
const Segment SGI233937[] = {{&S96525[44], 23}, {NULL}};
const Segment SGI233938[] = {{&S96525[30], 6}, {NULL}};
const Segment SGI233939[] = {{&S96525[81], 6}, {NULL}};
const Segment SGI233940[] = {{&S99888[34], 4}, {NULL}};
const Segment SGI233941[] = {{&S96525[108], 13}, {NULL}};
const Segment SGI233942[] = {{&S99888[27], 6}, {NULL}};
const Segment SGI233943[] = {{&S99888[91], 9}, {NULL}};
const Segment SGI233944[] = {{&S99888[64], 6}, {NULL}};
const Segment SGI233945[] = {{&S89949[34], 11}, {NULL}};
const Segment SGI233946[] = {{&S89949[27], 6}, {NULL}};
const Segment SGI233947[] = {{&S89949[98], 9}, {NULL}};
const Segment SGI233948[] = {{&S89949[71], 6}, {NULL}};
const Segment SGI233949[] = {{&S89954[44], 9}, {NULL}};
const Segment SGI233950[] = {{&S89954[44], 21}, {NULL}};
const Segment SGI233951[] = {{&S89954[30], 6}, {NULL}};
const Segment SGI233952[] = {{&S89954[79], 6}, {NULL}};
const Segment SGI233953[] = {{&S89954[106], 13}, {NULL}};
const Segment SGI233954[] = {{&S99898[34], 9}, {NULL}};
const Segment SGI233955[] = {{&S99898[27], 6}, {NULL}};
const Segment SGI233956[] = {{&S99898[96], 9}, {NULL}};
const Segment SGI233957[] = {{&S99898[69], 6}, {NULL}};
const Segment SGI233958[] = {{&S99898[137], 8}, {NULL}};
const Segment SGI233959[] = {{&S96538[34], 13}, {NULL}};
const Segment SGI233960[] = {{&S96538[27], 6}, {NULL}};
const Segment SGI233961[] = {{&S99899[44], 11}, {NULL}};
const Segment SGI233962[] = {{&S99899[44], 23}, {NULL}};
const Segment SGI233963[] = {{&S89957[34], 15}, {NULL}};
const Segment SGI233964[] = {{&S99899[30], 6}, {NULL}};
const Segment SGI233965[] = {{&S89957[27], 6}, {NULL}};
const Segment SGI233966[] = {{&S96538[100], 9}, {NULL}};
const Segment SGI233967[] = {{&S96538[73], 6}, {NULL}};
const Segment SGI233968[] = {{&S99899[81], 6}, {NULL}};
const Segment SGI233969[] = {{&S89957[102], 9}, {NULL}};
const Segment SGI233970[] = {{&S89957[75], 6}, {NULL}};
const Segment SGI233971[] = {{&S99899[108], 13}, {NULL}};
const Segment SGI233972[] = {{&S82240[34], 7}, {NULL}};
const Segment SGI233973[] = {{&S82240[27], 6}, {NULL}};
const Segment SGI233974[] = {{&S82240[100], 9}, {NULL}};
const Segment SGI233975[] = {{&S82240[73], 6}, {NULL}};
const Segment SGI233976[] = {{&S99902[34], 12}, {NULL}};
const Segment SGI233977[] = {{&S99902[27], 6}, {NULL}};
const Segment SGI233978[] = {{&S99902[112], 9}, {NULL}};
const Segment SGI233979[] = {{&S99902[85], 6}, {NULL}};
const Segment SGI233980[] = {{&S99903[34], 10}, {NULL}};
const Segment SGI233981[] = {{&S99903[27], 6}, {NULL}};
const Segment SGI233982[] = {{&S89962[34], 7}, {NULL}};
const Segment SGI233983[] = {{&S89962[27], 6}, {NULL}};
const Segment SGI233984[] = {{&S99903[97], 9}, {NULL}};
const Segment SGI233985[] = {{&S99903[70], 6}, {NULL}};
const Segment SGI233986[] = {{&S89962[94], 9}, {NULL}};
const Segment SGI233987[] = {{&S89962[67], 6}, {NULL}};
const Segment SGI233988[] = {{&S82243[44], 11}, {NULL}};
const Segment SGI233989[] = {{&S82243[44], 23}, {NULL}};
const Segment SGI233990[] = {{&S82243[30], 6}, {NULL}};
const Segment SGI233991[] = {{&S82243[116], 3}, {NULL}};
const Segment SGI233992[] = {{&S82243[81], 6}, {NULL}};
const Segment SGI233993[] = {{&S96546[34], 18}, {NULL}};
const Segment SGI233994[] = {{&S82243[141], 8}, {NULL}};
const Segment SGI233995[] = {{&S96546[27], 6}, {NULL}};
const Segment SGI233996[] = {{&S96546[114], 9}, {NULL}};
const Segment SGI233997[] = {{&S96546[87], 6}, {NULL}};
const Segment SGI233998[] = {{&S89964[34], 11}, {NULL}};
const Segment SGI233999[] = {{&S89964[27], 6}, {NULL}};
const Segment SGI234000[] = {{&S89964[98], 9}, {NULL}};
const Segment SGI234001[] = {{&S89964[71], 6}, {NULL}};
const Segment SGI234002[] = {{&S82250[44], 4}, {NULL}};
const Segment SGI234003[] = {{&S82250[44], 15}, {NULL}};
const Segment SGI234004[] = {{&S82250[30], 6}, {NULL}};
const Segment SGI234005[] = {{&S82250[73], 6}, {NULL}};
const Segment SGI234006[] = {{&S82250[100], 13}, {NULL}};
const Segment SGI234007[] = {{&S99915[44], 15}, {NULL}};
const Segment SGI234008[] = {{&S99915[44], 27}, {NULL}};
const Segment SGI234009[] = {{&S99915[30], 6}, {NULL}};
const Segment SGI234010[] = {{&S99915[85], 6}, {NULL}};
const Segment SGI234011[] = {{&S99915[112], 13}, {NULL}};
const Segment SGI234012[] = {{&S82255[34], 8}, {NULL}};
const Segment SGI234013[] = {{&S82255[27], 6}, {NULL}};
const Segment SGI234014[] = {{&S82255[103], 9}, {NULL}};
const Segment SGI234015[] = {{&S82255[76], 6}, {NULL}};
const Segment SGI234016[] = {{&S99919[34], 23}, {NULL}};
const Segment SGI234017[] = {{&S99919[27], 6}, {NULL}};
const Segment SGI234018[] = {{&S99919[110], 9}, {NULL}};
const Segment SGI234019[] = {{&S99919[83], 6}, {NULL}};
const Segment SGI234020[] = {{&S99926[44], 7}, {NULL}};
const Segment SGI234021[] = {{&S99926[44], 25}, {NULL}};
const Segment SGI234022[] = {{&S99926[30], 6}, {NULL}};
const Segment SGI234023[] = {{&S89985[34], 9}, {NULL}};
const Segment SGI234024[] = {{&S89985[27], 6}, {NULL}};
const Segment SGI234025[] = {{&S99926[83], 6}, {NULL}};
const Segment SGI234026[] = {{&S89985[96], 9}, {NULL}};
const Segment SGI234027[] = {{&S99926[110], 13}, {NULL}};
const Segment SGI234028[] = {{&S89985[69], 6}, {NULL}};
const Segment SGI234029[] = {{&S89986[34], 9}, {NULL}};
const Segment SGI234030[] = {{&S89986[27], 6}, {NULL}};
const Segment SGI234031[] = {{&S89986[96], 9}, {NULL}};
const Segment SGI234032[] = {{&S89986[69], 6}, {NULL}};
const Segment SGI234033[] = {{&S89987[44], 13}, {NULL}};
const Segment SGI234034[] = {{&S89987[44], 25}, {NULL}};
const Segment SGI234035[] = {{&S89987[30], 6}, {NULL}};
const Segment SGI234036[] = {{&S89987[83], 6}, {NULL}};
const Segment SGI234037[] = {{&S89987[110], 13}, {NULL}};
const Segment SGI234038[] = {{&S89988[44], 14}, {NULL}};
const Segment SGI234039[] = {{&S89988[44], 26}, {NULL}};
const Segment SGI234040[] = {{&S89988[30], 6}, {NULL}};
const Segment SGI234041[] = {{&S99932[34], 7}, {NULL}};
const Segment SGI234042[] = {{&S99932[27], 6}, {NULL}};
const Segment SGI234043[] = {{&S89988[84], 6}, {NULL}};
const Segment SGI234044[] = {{&S99932[94], 9}, {NULL}};
const Segment SGI234045[] = {{&S89988[111], 13}, {NULL}};
const Segment SGI234046[] = {{&S99932[67], 6}, {NULL}};
const Segment SGI234047[] = {{&S89998[34], 7}, {NULL}};
const Segment SGI234048[] = {{&S89998[27], 6}, {NULL}};
const Segment SGI234049[] = {{&S89998[93], 9}, {NULL}};
const Segment SGI234050[] = {{&S89998[66], 6}, {NULL}};
const Segment SGI234051[] = {{&S96587[34], 14}, {NULL}};
const Segment SGI234052[] = {{&S99944[44], 7}, {NULL}};
const Segment SGI234053[] = {{&S96587[27], 6}, {NULL}};
const Segment SGI234054[] = {{&S99944[44], 19}, {NULL}};
const Segment SGI234055[] = {{&S99944[30], 6}, {NULL}};
const Segment SGI234056[] = {{&S96587[101], 9}, {NULL}};
const Segment SGI234057[] = {{&S96587[74], 6}, {NULL}};
const Segment SGI234058[] = {{&S99944[77], 6}, {NULL}};
const Segment SGI234059[] = {{&S99944[104], 13}, {NULL}};
const Segment SGI234060[] = {{&S99945[44], 10}, {NULL}};
const Segment SGI234061[] = {{&S99945[44], 22}, {NULL}};
const Segment SGI234062[] = {{&S99945[30], 6}, {NULL}};
const Segment SGI234063[] = {{&S99945[31], 3}, {NULL}};
const Segment SGI234064[] = {{&S99945[80], 5}, {NULL}};
const Segment SGI234065[] = {{&S99945[117], 14}, {NULL}};
const Segment SGI234066[] = {{&S99946[44], 14}, {NULL}};
const Segment SGI234067[] = {{&S99946[44], 26}, {NULL}};
const Segment SGI234068[] = {{&S99946[30], 6}, {NULL}};
const Segment SGI234069[] = {{&S99946[84], 6}, {NULL}};
const Segment SGI234070[] = {{&S99946[111], 13}, {NULL}};
const Segment SGI234071[] = {{&S99950[52], 5}, {NULL}};
const Segment SGI234072[] = {{&S99950[52], 17}, {NULL}};
const Segment SGI234073[] = {{&S99950[38], 6}, {NULL}};
const Segment SGI234074[] = {{&S90009[34], 7}, {NULL}};
const Segment SGI234075[] = {{&S99950[83], 5}, {NULL}};
const Segment SGI234076[] = {{&S90009[27], 6}, {NULL}};
const Segment SGI234077[] = {{&S99950[0], 14}, {NULL}};
const Segment SGI234078[] = {{&S90009[110], 9}, {NULL}};
const Segment SGI234079[] = {{&S90009[83], 6}, {NULL}};
const Segment SGI234080[] = {{&S99952[44], 10}, {NULL}};
const Segment SGI234081[] = {{&S99952[44], 22}, {NULL}};
const Segment SGI234082[] = {{&S99952[30], 6}, {NULL}};
const Segment SGI234083[] = {{&S99952[80], 6}, {NULL}};
const Segment SGI234084[] = {{&S99952[107], 13}, {NULL}};
const Segment SGI234085[] = {{&S99953[44], 7}, {NULL}};
const Segment SGI234086[] = {{&S99953[44], 32}, {NULL}};
const Segment SGI234087[] = {{&S99953[30], 6}, {NULL}};
const Segment SGI234088[] = {{&S99953[90], 6}, {NULL}};
const Segment SGI234089[] = {{&S99953[117], 13}, {NULL}};
const Segment SGI234090[] = {{&S99956[44], 8}, {NULL}};
const Segment SGI234091[] = {{&S99956[44], 20}, {NULL}};
const Segment SGI234092[] = {{&S99956[30], 6}, {NULL}};
const Segment SGI234093[] = {{&S99956[78], 6}, {NULL}};
const Segment SGI234094[] = {{&S99956[105], 13}, {NULL}};
const Segment SGI234095[] = {{&S99958[44], 14}, {NULL}};
const Segment SGI234096[] = {{&S99958[44], 26}, {NULL}};
const Segment SGI234097[] = {{&S99958[30], 6}, {NULL}};
const Segment SGI234098[] = {{&S99958[84], 6}, {NULL}};
const Segment SGI234099[] = {{&S99958[111], 13}, {NULL}};
const Segment SGI234100[] = {{&S99960[34], 9}, {NULL}};
const Segment SGI234101[] = {{&S99960[27], 6}, {NULL}};
const Segment SGI234102[] = {{&S99960[96], 9}, {NULL}};
const Segment SGI234103[] = {{&S99960[69], 6}, {NULL}};
const Segment SGI234104[] = {{&S99970[44], 15}, {NULL}};
const Segment SGI234105[] = {{&S99970[44], 27}, {NULL}};
const Segment SGI234106[] = {{&S99970[30], 6}, {NULL}};
const Segment SGI234107[] = {{&S99970[85], 6}, {NULL}};
const Segment SGI234108[] = {{&S99970[112], 13}, {NULL}};
const Segment SGI234109[] = {{&S99973[34], 7}, {NULL}};
const Segment SGI234110[] = {{&S99973[27], 6}, {NULL}};
const Segment SGI234111[] = {{&S99973[94], 9}, {NULL}};
const Segment SGI234112[] = {{&S82329[34], 10}, {NULL}};
const Segment SGI234113[] = {{&S99973[67], 6}, {NULL}};
const Segment SGI234114[] = {{&S82329[27], 6}, {NULL}};
const Segment SGI234115[] = {{&S82329[96], 9}, {NULL}};
const Segment SGI234116[] = {{&S82329[69], 6}, {NULL}};
const Segment SGI234117[] = {{&S94231[32], 7}, {NULL}};
const Segment SGI234118[] = {{&S94231[27], 4}, {NULL}};
const Segment SGI234119[] = {{&S94231[97], 9}, {NULL}};
const Segment SGI234120[] = {{&S94231[70], 6}, {NULL}};
const Segment SGI234121[] = {{&S82340[34], 11}, {NULL}};
const Segment SGI234122[] = {{&S82340[27], 6}, {NULL}};
const Segment SGI234123[] = {{&S82340[97], 9}, {NULL}};
const Segment SGI234124[] = {{&S82340[70], 6}, {NULL}};
const Segment SGI234125[] = {{&S99987[59], 13}, {NULL}};
const Segment SGI234126[] = {{&S99987[59], 25}, {NULL}};
const Segment SGI234127[] = {{&S99987[45], 6}, {NULL}};
const Segment SGI234128[] = {{&S99987[46], 3}, {NULL}};
const Segment SGI234129[] = {{&S99987[98], 6}, {NULL}};
const Segment SGI234130[] = {{&S99987[0], 14}, {NULL}};
const Segment SGI234131[] = {{&S99994[44], 10}, {NULL}};
const Segment SGI234132[] = {{&S99994[44], 22}, {NULL}};
const Segment SGI234133[] = {{&S99994[30], 6}, {NULL}};
const Segment SGI234134[] = {{&S99994[31], 3}, {NULL}};
const Segment SGI234135[] = {{&S99994[80], 6}, {NULL}};
const Segment SGI234136[] = {{&S82353[44], 8}, {NULL}};
const Segment SGI234137[] = {{&S82353[44], 19}, {NULL}};
const Segment SGI234138[] = {{&S82353[30], 6}, {NULL}};
const Segment SGI234139[] = {{&S90067[44], 10}, {NULL}};
const Segment SGI234140[] = {{&S90067[44], 22}, {NULL}};
const Segment SGI234141[] = {{&S90067[30], 6}, {NULL}};
const Segment SGI234142[] = {{&S82353[112], 3}, {NULL}};
const Segment SGI234143[] = {{&S82353[77], 6}, {NULL}};
const Segment SGI234144[] = {{&S90067[131], 3}, {NULL}};
const Segment SGI234145[] = {{&S90067[96], 6}, {NULL}};
const Segment SGI234146[] = {{&S90067[68], 8}, {NULL}};
const Segment SGI234147[] = {{&S99998[44], 7}, {NULL}};
const Segment SGI234148[] = {{&S82354[44], 8}, {NULL}};
const Segment SGI234149[] = {{&S99998[44], 24}, {NULL}};
const Segment SGI234150[] = {{&S82354[44], 19}, {NULL}};
const Segment SGI234151[] = {{&S99998[30], 6}, {NULL}};
const Segment SGI234152[] = {{&S82354[30], 6}, {NULL}};
const Segment SGI234153[] = {{&S82354[112], 3}, {NULL}};
const Segment SGI234154[] = {{&S99998[31], 3}, {NULL}};
const Segment SGI234155[] = {{&S82354[77], 6}, {NULL}};
const Segment SGI234156[] = {{&S99998[82], 5}, {NULL}};
const Segment SGI234157[] = {{&S99998[119], 14}, {NULL}};
const Segment SGI234158[] = {{&S82356[34], 9}, {NULL}};
const Segment SGI234159[] = {{&S82356[27], 6}, {NULL}};
const Segment SGI234160[] = {{&S82356[96], 9}, {NULL}};
const Segment SGI234161[] = {{&S82356[69], 6}, {NULL}};
const Segment SGI234162[] = {{&S82357[44], 15}, {NULL}};
const Segment SGI234163[] = {{&S82357[44], 33}, {NULL}};
const Segment SGI234164[] = {{&S82357[30], 6}, {NULL}};
const Segment SGI234165[] = {{&S82357[91], 6}, {NULL}};
const Segment SGI234166[] = {{&S82357[118], 13}, {NULL}};
const Segment SGI234167[] = {{&S82358[44], 4}, {NULL}};
const Segment SGI234168[] = {{&S82358[44], 15}, {NULL}};
const Segment SGI234169[] = {{&S82358[30], 6}, {NULL}};
const Segment SGI234170[] = {{&S82358[73], 6}, {NULL}};
const Segment SGI234171[] = {{&S82358[100], 13}, {NULL}};
const Segment SGI234172[] = {{&S82359[44], 15}, {NULL}};
const Segment SGI234173[] = {{&S82359[44], 33}, {NULL}};
const Segment SGI234174[] = {{&S82359[30], 6}, {NULL}};
const Segment SGI234175[] = {{&S82359[91], 6}, {NULL}};
const Segment SGI234176[] = {{&S82359[118], 13}, {NULL}};
const Segment SGI234177[] = {{&S100007[44], 21}, {NULL}};
const Segment SGI234178[] = {{&S100007[44], 33}, {NULL}};
const Segment SGI234179[] = {{&S100007[30], 6}, {NULL}};
const Segment SGI234180[] = {{&S100007[91], 6}, {NULL}};
const Segment SGI234181[] = {{&S100007[118], 13}, {NULL}};
const Segment SGI234182[] = {{&S90080[34], 6}, {NULL}};
const Segment SGI234183[] = {{&S90080[27], 6}, {NULL}};
const Segment SGI234184[] = {{&S90080[93], 9}, {NULL}};
const Segment SGI234185[] = {{&S90080[66], 6}, {NULL}};
const Segment SGI234186[] = {{&S100010[44], 7}, {NULL}};
const Segment SGI234187[] = {{&S100010[44], 24}, {NULL}};
const Segment SGI234188[] = {{&S100010[30], 6}, {NULL}};
const Segment SGI234189[] = {{&S100010[31], 3}, {NULL}};
const Segment SGI234190[] = {{&S100010[82], 6}, {NULL}};
const Segment SGI234191[] = {{&S100010[142], 13}, {NULL}};
const Segment SGI234192[] = {{&S100025[34], 24}, {NULL}};
const Segment SGI234193[] = {{&S100025[27], 6}, {NULL}};
const Segment SGI234194[] = {{&S100025[111], 9}, {NULL}};
const Segment SGI234195[] = {{&S100025[84], 6}, {NULL}};
const Segment SGI234196[] = {{&S100025[152], 8}, {NULL}};
const Segment SGI234197[] = {{&S90100[34], 8}, {NULL}};
const Segment SGI234198[] = {{&S90100[27], 6}, {NULL}};
const Segment SGI234199[] = {{&S90100[95], 9}, {NULL}};
const Segment SGI234200[] = {{&S90100[68], 6}, {NULL}};
const Segment SGI234201[] = {{&S100030[34], 7}, {NULL}};
const Segment SGI234202[] = {{&S100030[27], 6}, {NULL}};
const Segment SGI234203[] = {{&S100030[95], 9}, {NULL}};
const Segment SGI234204[] = {{&S100030[68], 6}, {NULL}};
const Segment SGI234205[] = {{&S96698[44], 10}, {NULL}};
const Segment SGI234206[] = {{&S96698[44], 22}, {NULL}};
const Segment SGI234207[] = {{&S100031[34], 6}, {NULL}};
const Segment SGI234208[] = {{&S96698[30], 6}, {NULL}};
const Segment SGI234209[] = {{&S100031[27], 6}, {NULL}};
const Segment SGI234210[] = {{&S96698[80], 6}, {NULL}};
const Segment SGI234211[] = {{&S100031[105], 9}, {NULL}};
const Segment SGI234212[] = {{&S100031[78], 6}, {NULL}};
const Segment SGI234213[] = {{&S96698[107], 13}, {NULL}};
const Segment SGI234214[] = {{&S90106[44], 12}, {NULL}};
const Segment SGI234215[] = {{&S90106[44], 34}, {NULL}};
const Segment SGI234216[] = {{&S90106[30], 6}, {NULL}};
const Segment SGI234217[] = {{&S90106[125], 3}, {NULL}};
const Segment SGI234218[] = {{&S90106[92], 5}, {NULL}};
const Segment SGI234219[] = {{&S100034[44], 10}, {NULL}};
const Segment SGI234220[] = {{&S100034[44], 22}, {NULL}};
const Segment SGI234221[] = {{&S100034[30], 6}, {NULL}};
const Segment SGI234222[] = {{&S100034[31], 3}, {NULL}};
const Segment SGI234223[] = {{&S100034[80], 6}, {NULL}};
const Segment SGI234224[] = {{&S100034[150], 3}, {NULL}};
const Segment SGI234225[] = {{&S82397[45], 9}, {NULL}};
const Segment SGI234226[] = {{&S82397[30], 6}, {NULL}};
const Segment SGI234227[] = {{&S82397[113], 3}, {NULL}};
const Segment SGI234228[] = {{&S82397[79], 5}, {NULL}};
const Segment SGI234229[] = {{&S82397[137], 13}, {NULL}};
const Segment SGI234230[] = {{&S90110[44], 15}, {NULL}};
const Segment SGI234231[] = {{&S100035[34], 25}, {NULL}};
const Segment SGI234232[] = {{&S90110[44], 33}, {NULL}};
const Segment SGI234233[] = {{&S90110[30], 6}, {NULL}};
const Segment SGI234234[] = {{&S100035[27], 6}, {NULL}};
const Segment SGI234235[] = {{&S90110[126], 3}, {NULL}};
const Segment SGI234236[] = {{&S100035[112], 9}, {NULL}};
const Segment SGI234237[] = {{&S90110[91], 6}, {NULL}};
const Segment SGI234238[] = {{&S100035[85], 6}, {NULL}};
const Segment SGI234239[] = {{&S96706[44], 10}, {NULL}};
const Segment SGI234240[] = {{&S96706[44], 22}, {NULL}};
const Segment SGI234241[] = {{&S96706[30], 6}, {NULL}};
const Segment SGI234242[] = {{&S96706[144], 3}, {NULL}};
const Segment SGI234243[] = {{&S96706[109], 6}, {NULL}};
const Segment SGI234244[] = {{&S96706[68], 8}, {NULL}};
const Segment SGI234245[] = {{&S96706[80], 15}, {NULL}};
const Segment SGI234246[] = {{&S90113[34], 10}, {NULL}};
const Segment SGI234247[] = {{&S90113[27], 6}, {NULL}};
const Segment SGI234248[] = {{&S90113[97], 9}, {NULL}};
const Segment SGI234249[] = {{&S90113[70], 6}, {NULL}};
const Segment SGI234250[] = {{&S82406[34], 38}, {NULL}};
const Segment SGI234251[] = {{&S82406[27], 6}, {NULL}};
const Segment SGI234252[] = {{&S82406[124], 9}, {NULL}};
const Segment SGI234253[] = {{&S82406[97], 6}, {NULL}};
const Segment SGI234254[] = {{&S82408[34], 7}, {NULL}};
const Segment SGI234255[] = {{&S82408[27], 6}, {NULL}};
const Segment SGI234256[] = {{&S82408[93], 9}, {NULL}};
const Segment SGI234257[] = {{&S82408[66], 6}, {NULL}};
const Segment SGI234258[] = {{&S100046[34], 17}, {NULL}};
const Segment SGI234259[] = {{&S100046[27], 6}, {NULL}};
const Segment SGI234260[] = {{&S100046[104], 9}, {NULL}};
const Segment SGI234261[] = {{&S100046[77], 6}, {NULL}};
const Segment SGI234262[] = {{&S90122[34], 11}, {NULL}};
const Segment SGI234263[] = {{&S90122[27], 6}, {NULL}};
const Segment SGI234264[] = {{&S90122[98], 9}, {NULL}};
const Segment SGI234265[] = {{&S90122[71], 6}, {NULL}};
const Segment SGI234266[] = {{&S90125[34], 20}, {NULL}};
const Segment SGI234267[] = {{&S90125[27], 6}, {NULL}};
const Segment SGI234268[] = {{&S90125[107], 9}, {NULL}};
const Segment SGI234269[] = {{&S90125[80], 6}, {NULL}};
const Segment SGI234270[] = {{&S100051[52], 7}, {NULL}};
const Segment SGI234271[] = {{&S100051[52], 18}, {NULL}};
const Segment SGI234272[] = {{&S100051[38], 6}, {NULL}};
const Segment SGI234273[] = {{&S100051[84], 5}, {NULL}};
const Segment SGI234274[] = {{&S100051[0], 14}, {NULL}};
const Segment SGI234275[] = {{&S100052[52], 5}, {NULL}};
const Segment SGI234276[] = {{&S100052[52], 17}, {NULL}};
const Segment SGI234277[] = {{&S100052[38], 6}, {NULL}};
const Segment SGI234278[] = {{&S100052[83], 5}, {NULL}};
const Segment SGI234279[] = {{&S100052[0], 14}, {NULL}};
const Segment SGI234280[] = {{&S90129[44], 7}, {NULL}};
const Segment SGI234281[] = {{&S90129[44], 19}, {NULL}};
const Segment SGI234282[] = {{&S90129[30], 6}, {NULL}};
const Segment SGI234283[] = {{&S90129[112], 3}, {NULL}};
const Segment SGI234284[] = {{&S90129[77], 6}, {NULL}};
const Segment SGI234285[] = {{&S100057[44], 7}, {NULL}};
const Segment SGI234286[] = {{&S100057[44], 19}, {NULL}};
const Segment SGI234287[] = {{&S100057[30], 6}, {NULL}};
const Segment SGI234288[] = {{&S100057[31], 3}, {NULL}};
const Segment SGI234289[] = {{&S100057[77], 5}, {NULL}};
const Segment SGI234290[] = {{&S100057[114], 14}, {NULL}};
const Segment SGI234291[] = {{&S96730[34], 20}, {NULL}};
const Segment SGI234292[] = {{&S96730[27], 6}, {NULL}};
const Segment SGI234293[] = {{&S96730[107], 9}, {NULL}};
const Segment SGI234294[] = {{&S96730[80], 6}, {NULL}};
const Segment SGI234295[] = {{&S90137[34], 10}, {NULL}};
const Segment SGI234296[] = {{&S90137[27], 6}, {NULL}};
const Segment SGI234297[] = {{&S90137[97], 9}, {NULL}};
const Segment SGI234298[] = {{&S90137[70], 6}, {NULL}};
const Segment SGI234299[] = {{&S90141[34], 13}, {NULL}};
const Segment SGI234300[] = {{&S90141[27], 6}, {NULL}};
const Segment SGI234301[] = {{&S90141[100], 9}, {NULL}};
const Segment SGI234302[] = {{&S90141[73], 6}, {NULL}};
const Segment SGI234303[] = {{&S82437[44], 6}, {NULL}};
const Segment SGI234304[] = {{&S82437[44], 19}, {NULL}};
const Segment SGI234305[] = {{&S82437[30], 6}, {NULL}};
const Segment SGI234306[] = {{&S82437[112], 3}, {NULL}};
const Segment SGI234307[] = {{&S82437[77], 6}, {NULL}};
const Segment SGI234308[] = {{&S82438[44], 6}, {NULL}};
const Segment SGI234309[] = {{&S82438[44], 19}, {NULL}};
const Segment SGI234310[] = {{&S82438[30], 6}, {NULL}};
const Segment SGI234311[] = {{&S82438[112], 3}, {NULL}};
const Segment SGI234312[] = {{&S82438[77], 6}, {NULL}};
const Segment SGI234313[] = {{&S100072[44], 11}, {NULL}};
const Segment SGI234314[] = {{&S82439[38], 6}, {NULL}};
const Segment SGI234315[] = {{&S100072[44], 23}, {NULL}};
const Segment SGI234316[] = {{&S82439[31], 6}, {NULL}};
const Segment SGI234317[] = {{&S100072[30], 6}, {NULL}};
const Segment SGI234318[] = {{&S100072[81], 6}, {NULL}};
const Segment SGI234319[] = {{&S100072[108], 13}, {NULL}};
const Segment SGI234320[] = {{&S100075[34], 15}, {NULL}};
const Segment SGI234321[] = {{&S100075[27], 6}, {NULL}};
const Segment SGI234322[] = {{&S100075[109], 9}, {NULL}};
const Segment SGI234323[] = {{&S100075[82], 6}, {NULL}};
const Segment SGI234324[] = {{&S100077[34], 10}, {NULL}};
const Segment SGI234325[] = {{&S100077[27], 6}, {NULL}};
const Segment SGI234326[] = {{&S100077[102], 9}, {NULL}};
const Segment SGI234327[] = {{&S100077[75], 6}, {NULL}};
const Segment SGI234328[] = {{&S90157[44], 16}, {NULL}};
const Segment SGI234329[] = {{&S90157[44], 35}, {NULL}};
const Segment SGI234330[] = {{&S90157[30], 6}, {NULL}};
const Segment SGI234331[] = {{&S90157[93], 6}, {NULL}};
const Segment SGI234332[] = {{&S90157[120], 13}, {NULL}};
const Segment SGI234333[] = {{&S100081[34], 10}, {NULL}};
const Segment SGI234334[] = {{&S100081[27], 6}, {NULL}};
const Segment SGI234335[] = {{&S100081[97], 9}, {NULL}};
const Segment SGI234336[] = {{&S100081[70], 6}, {NULL}};
const Segment SGI234337[] = {{&S100082[44], 7}, {NULL}};
const Segment SGI234338[] = {{&S100082[44], 24}, {NULL}};
const Segment SGI234339[] = {{&S100082[30], 6}, {NULL}};
const Segment SGI234340[] = {{&S100082[82], 6}, {NULL}};
const Segment SGI234341[] = {{&S100082[109], 13}, {NULL}};
const Segment SGI234342[] = {{&S90164[34], 11}, {NULL}};
const Segment SGI234343[] = {{&S100085[34], 10}, {NULL}};
const Segment SGI234344[] = {{&S90164[27], 6}, {NULL}};
const Segment SGI234345[] = {{&S100085[27], 6}, {NULL}};
const Segment SGI234346[] = {{&S90164[98], 9}, {NULL}};
const Segment SGI234347[] = {{&S100085[97], 9}, {NULL}};
const Segment SGI234348[] = {{&S90164[71], 6}, {NULL}};
const Segment SGI234349[] = {{&S100085[70], 6}, {NULL}};
const Segment SGI234350[] = {{&S100089[34], 10}, {NULL}};
const Segment SGI234351[] = {{&S100089[27], 6}, {NULL}};
const Segment SGI234352[] = {{&S100089[97], 9}, {NULL}};
const Segment SGI234353[] = {{&S100089[70], 6}, {NULL}};
const Segment SGI234354[] = {{&S100090[34], 15}, {NULL}};
const Segment SGI234355[] = {{&S100090[27], 6}, {NULL}};
const Segment SGI234356[] = {{&S100090[102], 9}, {NULL}};
const Segment SGI234357[] = {{&S100090[75], 6}, {NULL}};
const Segment SGI234358[] = {{&S100091[34], 18}, {NULL}};
const Segment SGI234359[] = {{&S100091[27], 6}, {NULL}};
const Segment SGI234360[] = {{&S100091[105], 9}, {NULL}};
const Segment SGI234361[] = {{&S100091[78], 6}, {NULL}};
const Segment SGI234362[] = {{&S100091[146], 8}, {NULL}};
const Segment SGI234363[] = {{&S100092[44], 17}, {NULL}};
const Segment SGI234364[] = {{&S100092[44], 29}, {NULL}};
const Segment SGI234365[] = {{&S100092[30], 6}, {NULL}};
const Segment SGI234366[] = {{&S100092[87], 6}, {NULL}};
const Segment SGI234367[] = {{&S100092[114], 13}, {NULL}};
const Segment SGI234368[] = {{&S100094[44], 14}, {NULL}};
const Segment SGI234369[] = {{&S100094[44], 26}, {NULL}};
const Segment SGI234370[] = {{&S100094[30], 6}, {NULL}};
const Segment SGI234371[] = {{&S100094[84], 6}, {NULL}};
const Segment SGI234372[] = {{&S100094[111], 13}, {NULL}};
const Segment SGI234373[] = {{&S100096[34], 11}, {NULL}};
const Segment SGI234374[] = {{&S100096[27], 6}, {NULL}};
const Segment SGI234375[] = {{&S100096[98], 9}, {NULL}};
const Segment SGI234376[] = {{&S100096[71], 6}, {NULL}};
const Segment SGI234377[] = {{&S96779[34], 14}, {NULL}};
const Segment SGI234378[] = {{&S96779[27], 6}, {NULL}};
const Segment SGI234379[] = {{&S96779[101], 9}, {NULL}};
const Segment SGI234380[] = {{&S96779[74], 6}, {NULL}};
const Segment SGI234381[] = {{&S100098[34], 12}, {NULL}};
const Segment SGI234382[] = {{&S100098[27], 6}, {NULL}};
const Segment SGI234383[] = {{&S100098[99], 9}, {NULL}};
const Segment SGI234384[] = {{&S100098[72], 6}, {NULL}};
const Segment SGI234385[] = {{&S100098[140], 8}, {NULL}};
const Segment SGI234386[] = {{&S96782[44], 7}, {NULL}};
const Segment SGI234387[] = {{&S96782[44], 23}, {NULL}};
const Segment SGI234388[] = {{&S96782[30], 6}, {NULL}};
const Segment SGI234389[] = {{&S96782[81], 6}, {NULL}};
const Segment SGI234390[] = {{&S100100[34], 7}, {NULL}};
const Segment SGI234391[] = {{&S96782[108], 13}, {NULL}};
const Segment SGI234392[] = {{&S100100[27], 6}, {NULL}};
const Segment SGI234393[] = {{&S100100[100], 9}, {NULL}};
const Segment SGI234394[] = {{&S100100[73], 6}, {NULL}};
const Segment SGI234395[] = {{&S100100[141], 8}, {NULL}};
const Segment SGI234396[] = {{&S100102[34], 7}, {NULL}};
const Segment SGI234397[] = {{&S100102[27], 6}, {NULL}};
const Segment SGI234398[] = {{&S100102[112], 9}, {NULL}};
const Segment SGI234399[] = {{&S100102[85], 6}, {NULL}};
const Segment SGI234400[] = {{&S100102[153], 8}, {NULL}};
const Segment SGI234401[] = {{&S96791[44], 25}, {NULL}};
const Segment SGI234402[] = {{&S96791[44], 37}, {NULL}};
const Segment SGI234403[] = {{&S96791[30], 6}, {NULL}};
const Segment SGI234404[] = {{&S96791[130], 3}, {NULL}};
const Segment SGI234405[] = {{&S96791[95], 6}, {NULL}};
const Segment SGI234406[] = {{&S96809[52], 7}, {NULL}};
const Segment SGI234407[] = {{&S96809[52], 24}, {NULL}};
const Segment SGI234408[] = {{&S96809[38], 6}, {NULL}};
const Segment SGI234409[] = {{&S96809[90], 5}, {NULL}};
const Segment SGI234410[] = {{&S96809[0], 14}, {NULL}};
const Segment SGI234411[] = {{&S96824[44], 5}, {NULL}};
const Segment SGI234412[] = {{&S96824[44], 17}, {NULL}};
const Segment SGI234413[] = {{&S96824[30], 6}, {NULL}};
const Segment SGI234414[] = {{&S96824[75], 6}, {NULL}};
const Segment SGI234415[] = {{&S96824[102], 13}, {NULL}};
const Segment SGI234416[] = {{&S96833[34], 18}, {NULL}};
const Segment SGI234417[] = {{&S96833[27], 6}, {NULL}};
const Segment SGI234418[] = {{&S96833[105], 9}, {NULL}};
const Segment SGI234419[] = {{&S96833[78], 6}, {NULL}};
const Segment SGI234420[] = {{&S96838[34], 17}, {NULL}};
const Segment SGI234421[] = {{&S96838[27], 6}, {NULL}};
const Segment SGI234422[] = {{&S96838[115], 9}, {NULL}};
const Segment SGI234423[] = {{&S96838[88], 6}, {NULL}};
const Segment SGI234424[] = {{&S96851[44], 7}, {NULL}};
const Segment SGI234425[] = {{&S96851[44], 19}, {NULL}};
const Segment SGI234426[] = {{&S96851[30], 6}, {NULL}};
const Segment SGI234427[] = {{&S96851[112], 3}, {NULL}};
const Segment SGI234428[] = {{&S96851[77], 6}, {NULL}};
const Segment SGI234429[] = {{&S96857[44], 18}, {NULL}};
const Segment SGI234430[] = {{&S96857[44], 39}, {NULL}};
const Segment SGI234431[] = {{&S96857[30], 6}, {NULL}};
const Segment SGI234432[] = {{&S96857[97], 6}, {NULL}};
const Segment SGI234433[] = {{&S96857[124], 13}, {NULL}};
const Segment SGI234434[] = {{&S90279[44], 9}, {NULL}};
const Segment SGI234435[] = {{&S90279[44], 21}, {NULL}};
const Segment SGI234436[] = {{&S90279[30], 6}, {NULL}};
const Segment SGI234437[] = {{&S90279[114], 3}, {NULL}};
const Segment SGI234438[] = {{&S90279[79], 6}, {NULL}};
const Segment SGI234439[] = {{&S90279[146], 15}, {NULL}};
const Segment SGI234440[] = {{&S82582[44], 10}, {NULL}};
const Segment SGI234441[] = {{&S82582[44], 21}, {NULL}};
const Segment SGI234442[] = {{&S82582[30], 6}, {NULL}};
const Segment SGI234443[] = {{&S82582[137], 3}, {NULL}};
const Segment SGI234444[] = {{&S82582[102], 6}, {NULL}};
const Segment SGI234445[] = {{&S90281[34], 10}, {NULL}};
const Segment SGI234446[] = {{&S82582[74], 9}, {NULL}};
const Segment SGI234447[] = {{&S90281[27], 6}, {NULL}};
const Segment SGI234448[] = {{&S90281[97], 9}, {NULL}};
const Segment SGI234449[] = {{&S90281[70], 6}, {NULL}};
const Segment SGI234450[] = {{&S82583[44], 10}, {NULL}};
const Segment SGI234451[] = {{&S82583[44], 22}, {NULL}};
const Segment SGI234452[] = {{&S82583[30], 6}, {NULL}};
const Segment SGI234453[] = {{&S82583[138], 3}, {NULL}};
const Segment SGI234454[] = {{&S82583[103], 6}, {NULL}};
const Segment SGI234455[] = {{&S82583[75], 9}, {NULL}};
const Segment SGI234456[] = {{&S90283[34], 7}, {NULL}};
const Segment SGI234457[] = {{&S90283[27], 6}, {NULL}};
const Segment SGI234458[] = {{&S82584[44], 10}, {NULL}};
const Segment SGI234459[] = {{&S90283[100], 9}, {NULL}};
const Segment SGI234460[] = {{&S90283[73], 6}, {NULL}};
const Segment SGI234461[] = {{&S82584[44], 21}, {NULL}};
const Segment SGI234462[] = {{&S82584[30], 6}, {NULL}};
const Segment SGI234463[] = {{&S96870[34], 7}, {NULL}};
const Segment SGI234464[] = {{&S82584[137], 3}, {NULL}};
const Segment SGI234465[] = {{&S96870[27], 6}, {NULL}};
const Segment SGI234466[] = {{&S82584[102], 6}, {NULL}};
const Segment SGI234467[] = {{&S96870[94], 9}, {NULL}};
const Segment SGI234468[] = {{&S96870[67], 6}, {NULL}};
const Segment SGI234469[] = {{&S82591[34], 12}, {NULL}};
const Segment SGI234470[] = {{&S82591[27], 6}, {NULL}};
const Segment SGI234471[] = {{&S82591[99], 9}, {NULL}};
const Segment SGI234472[] = {{&S82591[72], 6}, {NULL}};
const Segment SGI234473[] = {{&S82592[34], 24}, {NULL}};
const Segment SGI234474[] = {{&S82592[27], 6}, {NULL}};
const Segment SGI234475[] = {{&S94491[32], 12}, {NULL}};
const Segment SGI234476[] = {{&S94491[27], 4}, {NULL}};
const Segment SGI234477[] = {{&S82592[111], 9}, {NULL}};
const Segment SGI234478[] = {{&S82592[84], 6}, {NULL}};
const Segment SGI234479[] = {{&S94491[97], 9}, {NULL}};
const Segment SGI234480[] = {{&S94491[70], 6}, {NULL}};
const Segment SGI234481[] = {{&S82593[34], 10}, {NULL}};
const Segment SGI234482[] = {{&S82593[27], 6}, {NULL}};
const Segment SGI234483[] = {{&S82593[97], 9}, {NULL}};
const Segment SGI234484[] = {{&S82593[70], 6}, {NULL}};
const Segment SGI234485[] = {{&S90303[34], 10}, {NULL}};
const Segment SGI234486[] = {{&S90303[27], 6}, {NULL}};
const Segment SGI234487[] = {{&S90303[97], 9}, {NULL}};
const Segment SGI234488[] = {{&S90303[70], 6}, {NULL}};
const Segment SGI234489[] = {{&S90303[138], 8}, {NULL}};
const Segment SGI234490[] = {{&S90311[34], 7}, {NULL}};
const Segment SGI234491[] = {{&S90311[27], 6}, {NULL}};
const Segment SGI234492[] = {{&S90311[98], 9}, {NULL}};
const Segment SGI234493[] = {{&S90311[71], 6}, {NULL}};
const Segment SGI234494[] = {{&S90312[44], 10}, {NULL}};
const Segment SGI234495[] = {{&S90312[44], 22}, {NULL}};
const Segment SGI234496[] = {{&S90312[30], 6}, {NULL}};
const Segment SGI234497[] = {{&S100206[44], 9}, {NULL}};
const Segment SGI234498[] = {{&S100206[44], 21}, {NULL}};
const Segment SGI234499[] = {{&S90312[113], 3}, {NULL}};
const Segment SGI234500[] = {{&S100206[30], 6}, {NULL}};
const Segment SGI234501[] = {{&S90312[80], 5}, {NULL}};
const Segment SGI234502[] = {{&S100206[31], 3}, {NULL}};
const Segment SGI234503[] = {{&S90312[117], 14}, {NULL}};
const Segment SGI234504[] = {{&S100206[94], 6}, {NULL}};
const Segment SGI234505[] = {{&S100206[67], 8}, {NULL}};
const Segment SGI234506[] = {{&S90313[34], 10}, {NULL}};
const Segment SGI234507[] = {{&S90313[27], 6}, {NULL}};
const Segment SGI234508[] = {{&S90313[97], 9}, {NULL}};
const Segment SGI234509[] = {{&S90313[70], 6}, {NULL}};
const Segment SGI234510[] = {{&S90318[44], 10}, {NULL}};
const Segment SGI234511[] = {{&S94520[32], 12}, {NULL}};
const Segment SGI234512[] = {{&S90318[44], 22}, {NULL}};
const Segment SGI234513[] = {{&S94520[27], 4}, {NULL}};
const Segment SGI234514[] = {{&S90318[30], 6}, {NULL}};
const Segment SGI234515[] = {{&S94520[97], 9}, {NULL}};
const Segment SGI234516[] = {{&S94520[70], 6}, {NULL}};
const Segment SGI234517[] = {{&S90318[80], 6}, {NULL}};
const Segment SGI234518[] = {{&S90318[107], 13}, {NULL}};
const Segment SGI234519[] = {{&S90319[44], 10}, {NULL}};
const Segment SGI234520[] = {{&S90319[44], 22}, {NULL}};
const Segment SGI234521[] = {{&S90319[30], 6}, {NULL}};
const Segment SGI234522[] = {{&S90319[131], 3}, {NULL}};
const Segment SGI234523[] = {{&S90319[96], 6}, {NULL}};
const Segment SGI234524[] = {{&S90319[68], 8}, {NULL}};
const Segment SGI234525[] = {{&S82628[44], 7}, {NULL}};
const Segment SGI234526[] = {{&S82628[44], 18}, {NULL}};
const Segment SGI234527[] = {{&S82628[30], 6}, {NULL}};
const Segment SGI234528[] = {{&S82628[140], 3}, {NULL}};
const Segment SGI234529[] = {{&S82628[105], 6}, {NULL}};
const Segment SGI234530[] = {{&S90320[34], 14}, {NULL}};
const Segment SGI234531[] = {{&S90320[27], 6}, {NULL}};
const Segment SGI234532[] = {{&S90320[101], 9}, {NULL}};
const Segment SGI234533[] = {{&S90320[74], 6}, {NULL}};
const Segment SGI234534[] = {{&S82629[44], 7}, {NULL}};
const Segment SGI234535[] = {{&S82629[44], 19}, {NULL}};
const Segment SGI234536[] = {{&S82629[30], 6}, {NULL}};
const Segment SGI234537[] = {{&S90320[142], 8}, {NULL}};
const Segment SGI234538[] = {{&S82629[141], 3}, {NULL}};
const Segment SGI234539[] = {{&S82629[106], 6}, {NULL}};
const Segment SGI234540[] = {{&S100220[44], 9}, {NULL}};
const Segment SGI234541[] = {{&S100220[44], 21}, {NULL}};
const Segment SGI234542[] = {{&S100220[30], 6}, {NULL}};
const Segment SGI234543[] = {{&S100220[31], 3}, {NULL}};
const Segment SGI234544[] = {{&S100220[94], 6}, {NULL}};
const Segment SGI234545[] = {{&S100220[67], 8}, {NULL}};
const Segment SGI234546[] = {{&S90323[34], 10}, {NULL}};
const Segment SGI234547[] = {{&S90323[27], 6}, {NULL}};
const Segment SGI234548[] = {{&S90323[97], 9}, {NULL}};
const Segment SGI234549[] = {{&S90323[70], 6}, {NULL}};
const Segment SGI234550[] = {{&S100221[44], 9}, {NULL}};
const Segment SGI234551[] = {{&S90323[138], 8}, {NULL}};
const Segment SGI234552[] = {{&S100221[44], 21}, {NULL}};
const Segment SGI234553[] = {{&S100221[30], 6}, {NULL}};
const Segment SGI234554[] = {{&S100221[31], 3}, {NULL}};
const Segment SGI234555[] = {{&S100221[98], 6}, {NULL}};
const Segment SGI234556[] = {{&S100221[67], 8}, {NULL}};
const Segment SGI234557[] = {{&S100222[44], 9}, {NULL}};
const Segment SGI234558[] = {{&S100222[44], 21}, {NULL}};
const Segment SGI234559[] = {{&S100222[30], 6}, {NULL}};
const Segment SGI234560[] = {{&S100222[31], 3}, {NULL}};
const Segment SGI234561[] = {{&S100222[98], 6}, {NULL}};
const Segment SGI234562[] = {{&S100222[67], 8}, {NULL}};
const Segment SGI234563[] = {{&S90327[44], 28}, {NULL}};
const Segment SGI234564[] = {{&S90327[44], 40}, {NULL}};
const Segment SGI234565[] = {{&S100223[44], 9}, {NULL}};
const Segment SGI234566[] = {{&S90327[30], 6}, {NULL}};
const Segment SGI234567[] = {{&S100223[44], 21}, {NULL}};
const Segment SGI234568[] = {{&S100223[30], 6}, {NULL}};
const Segment SGI234569[] = {{&S90327[133], 3}, {NULL}};
const Segment SGI234570[] = {{&S100223[31], 3}, {NULL}};
const Segment SGI234571[] = {{&S90327[98], 6}, {NULL}};
const Segment SGI234572[] = {{&S100223[98], 6}, {NULL}};
const Segment SGI234573[] = {{&S100223[67], 8}, {NULL}};
const Segment SGI234574[] = {{&S90329[44], 10}, {NULL}};
const Segment SGI234575[] = {{&S90329[44], 22}, {NULL}};
const Segment SGI234576[] = {{&S90329[30], 6}, {NULL}};
const Segment SGI234577[] = {{&S90329[115], 3}, {NULL}};
const Segment SGI234578[] = {{&S90329[80], 6}, {NULL}};
const Segment SGI234579[] = {{&S90329[135], 8}, {NULL}};
const Segment SGI234580[] = {{&S90329[144], 13}, {NULL}};
const Segment SGI234581[] = {{&S90330[45], 10}, {NULL}};
const Segment SGI234582[] = {{&S90330[30], 6}, {NULL}};
const Segment SGI234583[] = {{&S90330[115], 3}, {NULL}};
const Segment SGI234584[] = {{&S90330[81], 5}, {NULL}};
const Segment SGI234585[] = {{&S90330[139], 13}, {NULL}};
const Segment SGI234586[] = {{&S90336[34], 14}, {NULL}};
const Segment SGI234587[] = {{&S90336[27], 6}, {NULL}};
const Segment SGI234588[] = {{&S90336[101], 9}, {NULL}};
const Segment SGI234589[] = {{&S90336[74], 6}, {NULL}};
const Segment SGI234590[] = {{&S90341[44], 11}, {NULL}};
const Segment SGI234591[] = {{&S90341[44], 23}, {NULL}};
const Segment SGI234592[] = {{&S90341[30], 6}, {NULL}};
const Segment SGI234593[] = {{&S90341[114], 3}, {NULL}};
const Segment SGI234594[] = {{&S90341[81], 5}, {NULL}};
const Segment SGI234595[] = {{&S90341[118], 14}, {NULL}};
const Segment SGI234596[] = {{&S90342[44], 10}, {NULL}};
const Segment SGI234597[] = {{&S90342[44], 22}, {NULL}};
const Segment SGI234598[] = {{&S90342[30], 6}, {NULL}};
const Segment SGI234599[] = {{&S90342[113], 3}, {NULL}};
const Segment SGI234600[] = {{&S90342[80], 5}, {NULL}};
const Segment SGI234601[] = {{&S90342[117], 14}, {NULL}};
const Segment SGI234602[] = {{&S100249[44], 10}, {NULL}};
const Segment SGI234603[] = {{&S100249[44], 22}, {NULL}};
const Segment SGI234604[] = {{&S100249[30], 6}, {NULL}};
const Segment SGI234605[] = {{&S90346[34], 14}, {NULL}};
const Segment SGI234606[] = {{&S90346[27], 6}, {NULL}};
const Segment SGI234607[] = {{&S100249[31], 3}, {NULL}};
const Segment SGI234608[] = {{&S90346[101], 9}, {NULL}};
const Segment SGI234609[] = {{&S100249[80], 6}, {NULL}};
const Segment SGI234610[] = {{&S90346[74], 6}, {NULL}};
const Segment SGI234611[] = {{&S100249[143], 3}, {NULL}};
const Segment SGI234612[] = {{&S90352[34], 18}, {NULL}};
const Segment SGI234613[] = {{&S90352[27], 6}, {NULL}};
const Segment SGI234614[] = {{&S90352[105], 9}, {NULL}};
const Segment SGI234615[] = {{&S90352[78], 6}, {NULL}};
const Segment SGI234616[] = {{&S100265[59], 12}, {NULL}};
const Segment SGI234617[] = {{&S100265[59], 40}, {NULL}};
const Segment SGI234618[] = {{&S100265[45], 6}, {NULL}};
const Segment SGI234619[] = {{&S100265[46], 3}, {NULL}};
const Segment SGI234620[] = {{&S100265[113], 6}, {NULL}};
const Segment SGI234621[] = {{&S100265[0], 14}, {NULL}};
const Segment SGI234622[] = {{&S96987[44], 9}, {NULL}};
const Segment SGI234623[] = {{&S96987[44], 21}, {NULL}};
const Segment SGI234624[] = {{&S96987[30], 6}, {NULL}};
const Segment SGI234625[] = {{&S96987[143], 3}, {NULL}};
const Segment SGI234626[] = {{&S96987[108], 6}, {NULL}};
const Segment SGI234627[] = {{&S96987[67], 8}, {NULL}};
const Segment SGI234628[] = {{&S96987[79], 15}, {NULL}};
const Segment SGI234629[] = {{&S96988[44], 9}, {NULL}};
const Segment SGI234630[] = {{&S96988[44], 21}, {NULL}};
const Segment SGI234631[] = {{&S96988[30], 6}, {NULL}};
const Segment SGI234632[] = {{&S96988[143], 3}, {NULL}};
const Segment SGI234633[] = {{&S96988[108], 6}, {NULL}};
const Segment SGI234634[] = {{&S96988[67], 8}, {NULL}};
const Segment SGI234635[] = {{&S96988[79], 15}, {NULL}};
const Segment SGI234636[] = {{&S100278[45], 10}, {NULL}};
const Segment SGI234637[] = {{&S100278[30], 6}, {NULL}};
const Segment SGI234638[] = {{&S100278[31], 3}, {NULL}};
const Segment SGI234639[] = {{&S100278[81], 5}, {NULL}};
const Segment SGI234640[] = {{&S100278[139], 13}, {NULL}};
const Segment SGI234641[] = {{&S100282[45], 10}, {NULL}};
const Segment SGI234642[] = {{&S100282[30], 6}, {NULL}};
const Segment SGI234643[] = {{&S100282[31], 3}, {NULL}};
const Segment SGI234644[] = {{&S100282[81], 5}, {NULL}};
const Segment SGI234645[] = {{&S100282[139], 13}, {NULL}};
const Segment SGI234646[] = {{&S100283[34], 10}, {NULL}};
const Segment SGI234647[] = {{&S100283[27], 6}, {NULL}};
const Segment SGI234648[] = {{&S100283[97], 9}, {NULL}};
const Segment SGI234649[] = {{&S100283[70], 6}, {NULL}};
const Segment SGI234650[] = {{&S82718[44], 7}, {NULL}};
const Segment SGI234651[] = {{&S82718[44], 18}, {NULL}};
const Segment SGI234652[] = {{&S82718[30], 6}, {NULL}};
const Segment SGI234653[] = {{&S82718[111], 3}, {NULL}};
const Segment SGI234654[] = {{&S82718[76], 6}, {NULL}};
const Segment SGI234655[] = {{&S100295[34], 13}, {NULL}};
const Segment SGI234656[] = {{&S100295[27], 6}, {NULL}};
const Segment SGI234657[] = {{&S100295[100], 9}, {NULL}};
const Segment SGI234658[] = {{&S100295[73], 6}, {NULL}};
const Segment SGI234659[] = {{&S82719[44], 9}, {NULL}};
const Segment SGI234660[] = {{&S82719[44], 21}, {NULL}};
const Segment SGI234661[] = {{&S82719[30], 6}, {NULL}};
const Segment SGI234662[] = {{&S82719[114], 3}, {NULL}};
const Segment SGI234663[] = {{&S82719[79], 6}, {NULL}};
const Segment SGI234664[] = {{&S82720[44], 15}, {NULL}};
const Segment SGI234665[] = {{&S82720[44], 26}, {NULL}};
const Segment SGI234666[] = {{&S82720[30], 6}, {NULL}};
const Segment SGI234667[] = {{&S82720[119], 3}, {NULL}};
const Segment SGI234668[] = {{&S82720[84], 6}, {NULL}};
const Segment SGI234669[] = {{&S82724[44], 10}, {NULL}};
const Segment SGI234670[] = {{&S82724[44], 21}, {NULL}};
const Segment SGI234671[] = {{&S82724[30], 6}, {NULL}};
const Segment SGI234672[] = {{&S82724[114], 3}, {NULL}};
const Segment SGI234673[] = {{&S82724[79], 6}, {NULL}};
const Segment SGI234674[] = {{&S82724[139], 7}, {NULL}};
const Segment SGI234675[] = {{&S100303[44], 7}, {NULL}};
const Segment SGI234676[] = {{&S100303[44], 23}, {NULL}};
const Segment SGI234677[] = {{&S100303[30], 6}, {NULL}};
const Segment SGI234678[] = {{&S100303[31], 3}, {NULL}};
const Segment SGI234679[] = {{&S100303[81], 6}, {NULL}};
const Segment SGI234680[] = {{&S90409[34], 10}, {NULL}};
const Segment SGI234681[] = {{&S100303[141], 11}, {NULL}};
const Segment SGI234682[] = {{&S90409[27], 6}, {NULL}};
const Segment SGI234683[] = {{&S90409[97], 9}, {NULL}};
const Segment SGI234684[] = {{&S90409[70], 6}, {NULL}};
const Segment SGI234685[] = {{&S100306[34], 10}, {NULL}};
const Segment SGI234686[] = {{&S100306[27], 6}, {NULL}};
const Segment SGI234687[] = {{&S100306[97], 9}, {NULL}};
const Segment SGI234688[] = {{&S100306[70], 6}, {NULL}};
const Segment SGI234689[] = {{&S100307[34], 10}, {NULL}};
const Segment SGI234690[] = {{&S100307[27], 6}, {NULL}};
const Segment SGI234691[] = {{&S100307[97], 9}, {NULL}};
const Segment SGI234692[] = {{&S100307[70], 6}, {NULL}};
const Segment SGI234693[] = {{&S90415[34], 10}, {NULL}};
const Segment SGI234694[] = {{&S90415[27], 6}, {NULL}};
const Segment SGI234695[] = {{&S90415[97], 9}, {NULL}};
const Segment SGI234696[] = {{&S90415[70], 6}, {NULL}};
const Segment SGI234697[] = {{&S100309[59], 7}, {NULL}};
const Segment SGI234698[] = {{&S100309[59], 24}, {NULL}};
const Segment SGI234699[] = {{&S100309[45], 6}, {NULL}};
const Segment SGI234700[] = {{&S100309[46], 3}, {NULL}};
const Segment SGI234701[] = {{&S100309[97], 6}, {NULL}};
const Segment SGI234702[] = {{&S100309[0], 14}, {NULL}};
const Segment SGI234703[] = {{&S90420[34], 10}, {NULL}};
const Segment SGI234704[] = {{&S100313[34], 10}, {NULL}};
const Segment SGI234705[] = {{&S90420[27], 6}, {NULL}};
const Segment SGI234706[] = {{&S100313[27], 6}, {NULL}};
const Segment SGI234707[] = {{&S90420[97], 9}, {NULL}};
const Segment SGI234708[] = {{&S100313[96], 9}, {NULL}};
const Segment SGI234709[] = {{&S90420[70], 6}, {NULL}};
const Segment SGI234710[] = {{&S100313[69], 6}, {NULL}};
const Segment SGI234711[] = {{&S100320[44], 10}, {NULL}};
const Segment SGI234712[] = {{&S100320[44], 22}, {NULL}};
const Segment SGI234713[] = {{&S100320[30], 6}, {NULL}};
const Segment SGI234714[] = {{&S100320[31], 3}, {NULL}};
const Segment SGI234715[] = {{&S100320[95], 6}, {NULL}};
const Segment SGI234716[] = {{&S100320[68], 8}, {NULL}};
const Segment SGI234717[] = {{&S97051[34], 11}, {NULL}};
const Segment SGI234718[] = {{&S97051[27], 6}, {NULL}};
const Segment SGI234719[] = {{&S97051[98], 9}, {NULL}};
const Segment SGI234720[] = {{&S97051[71], 6}, {NULL}};
const Segment SGI234721[] = {{&S90437[34], 8}, {NULL}};
const Segment SGI234722[] = {{&S90437[27], 6}, {NULL}};
const Segment SGI234723[] = {{&S90437[95], 9}, {NULL}};
const Segment SGI234724[] = {{&S90437[68], 6}, {NULL}};
const Segment SGI234725[] = {{&S90438[44], 29}, {NULL}};
const Segment SGI234726[] = {{&S90438[44], 41}, {NULL}};
const Segment SGI234727[] = {{&S90438[30], 6}, {NULL}};
const Segment SGI234728[] = {{&S100330[45], 7}, {NULL}};
const Segment SGI234729[] = {{&S100330[30], 6}, {NULL}};
const Segment SGI234730[] = {{&S90438[154], 3}, {NULL}};
const Segment SGI234731[] = {{&S90438[119], 6}, {NULL}};
const Segment SGI234732[] = {{&S90438[87], 8}, {NULL}};
const Segment SGI234733[] = {{&S100330[31], 3}, {NULL}};
const Segment SGI234734[] = {{&S100330[83], 5}, {NULL}};
const Segment SGI234735[] = {{&S100330[141], 13}, {NULL}};
const Segment SGI234736[] = {{&S90439[44], 10}, {NULL}};
const Segment SGI234737[] = {{&S90439[44], 20}, {NULL}};
const Segment SGI234738[] = {{&S90439[30], 6}, {NULL}};
const Segment SGI234739[] = {{&S90439[113], 3}, {NULL}};
const Segment SGI234740[] = {{&S90439[78], 6}, {NULL}};
const Segment SGI234741[] = {{&S90440[44], 10}, {NULL}};
const Segment SGI234742[] = {{&S90440[44], 32}, {NULL}};
const Segment SGI234743[] = {{&S90440[30], 6}, {NULL}};
const Segment SGI234744[] = {{&S90440[125], 3}, {NULL}};
const Segment SGI234745[] = {{&S90440[90], 6}, {NULL}};
const Segment SGI234746[] = {{&S90444[34], 9}, {NULL}};
const Segment SGI234747[] = {{&S90444[27], 6}, {NULL}};
const Segment SGI234748[] = {{&S90444[96], 9}, {NULL}};
const Segment SGI234749[] = {{&S90444[69], 6}, {NULL}};
const Segment SGI234750[] = {{&S100341[34], 13}, {NULL}};
const Segment SGI234751[] = {{&S100341[27], 6}, {NULL}};
const Segment SGI234752[] = {{&S100341[100], 9}, {NULL}};
const Segment SGI234753[] = {{&S100341[73], 6}, {NULL}};
const Segment SGI234754[] = {{&S90449[59], 11}, {NULL}};
const Segment SGI234755[] = {{&S90449[59], 23}, {NULL}};
const Segment SGI234756[] = {{&S90449[45], 6}, {NULL}};
const Segment SGI234757[] = {{&S90449[131], 3}, {NULL}};
const Segment SGI234758[] = {{&S90449[96], 6}, {NULL}};
const Segment SGI234759[] = {{&S90449[0], 14}, {NULL}};
const Segment SGI234760[] = {{&S90450[34], 5}, {NULL}};
const Segment SGI234761[] = {{&S90450[27], 6}, {NULL}};
const Segment SGI234762[] = {{&S90450[92], 9}, {NULL}};
const Segment SGI234763[] = {{&S90450[65], 6}, {NULL}};
const Segment SGI234764[] = {{&S90452[34], 10}, {NULL}};
const Segment SGI234765[] = {{&S90452[27], 6}, {NULL}};
const Segment SGI234766[] = {{&S90452[97], 9}, {NULL}};
const Segment SGI234767[] = {{&S90452[70], 6}, {NULL}};
const Segment SGI234768[] = {{&S90453[34], 13}, {NULL}};
const Segment SGI234769[] = {{&S90453[27], 6}, {NULL}};
const Segment SGI234770[] = {{&S90453[100], 9}, {NULL}};
const Segment SGI234771[] = {{&S90453[73], 6}, {NULL}};
const Segment SGI234772[] = {{&S100350[34], 6}, {NULL}};
const Segment SGI234773[] = {{&S100350[27], 6}, {NULL}};
const Segment SGI234774[] = {{&S100350[93], 9}, {NULL}};
const Segment SGI234775[] = {{&S100350[66], 6}, {NULL}};
const Segment SGI234776[] = {{&S97085[34], 9}, {NULL}};
const Segment SGI234777[] = {{&S97085[27], 6}, {NULL}};
const Segment SGI234778[] = {{&S97085[96], 9}, {NULL}};
const Segment SGI234779[] = {{&S97085[69], 6}, {NULL}};
const Segment SGI234780[] = {{&S100354[45], 21}, {NULL}};
const Segment SGI234781[] = {{&S100354[30], 6}, {NULL}};
const Segment SGI234782[] = {{&S100354[31], 3}, {NULL}};
const Segment SGI234783[] = {{&S100354[92], 5}, {NULL}};
const Segment SGI234784[] = {{&S100354[150], 13}, {NULL}};
const Segment SGI234785[] = {{&S90463[44], 7}, {NULL}};
const Segment SGI234786[] = {{&S90463[44], 19}, {NULL}};
const Segment SGI234787[] = {{&S90463[30], 6}, {NULL}};
const Segment SGI234788[] = {{&S90463[132], 3}, {NULL}};
const Segment SGI234789[] = {{&S90463[97], 6}, {NULL}};
const Segment SGI234790[] = {{&S90463[65], 8}, {NULL}};
const Segment SGI234791[] = {{&S100359[34], 7}, {NULL}};
const Segment SGI234792[] = {{&S100359[27], 6}, {NULL}};
const Segment SGI234793[] = {{&S100359[90], 9}, {NULL}};
const Segment SGI234794[] = {{&S100359[63], 6}, {NULL}};
const Segment SGI234795[] = {{&S97094[34], 9}, {NULL}};
const Segment SGI234796[] = {{&S97094[27], 6}, {NULL}};
const Segment SGI234797[] = {{&S97094[96], 9}, {NULL}};
const Segment SGI234798[] = {{&S97094[69], 6}, {NULL}};
const Segment SGI234799[] = {{&S100361[34], 9}, {NULL}};
const Segment SGI234800[] = {{&S100361[27], 6}, {NULL}};
const Segment SGI234801[] = {{&S90470[44], 9}, {NULL}};
const Segment SGI234802[] = {{&S100361[96], 9}, {NULL}};
const Segment SGI234803[] = {{&S90470[44], 21}, {NULL}};
const Segment SGI234804[] = {{&S100361[69], 6}, {NULL}};
const Segment SGI234805[] = {{&S90470[30], 6}, {NULL}};
const Segment SGI234806[] = {{&S90470[114], 3}, {NULL}};
const Segment SGI234807[] = {{&S90470[79], 6}, {NULL}};
const Segment SGI234808[] = {{&S100362[34], 7}, {NULL}};
const Segment SGI234809[] = {{&S100362[27], 6}, {NULL}};
const Segment SGI234810[] = {{&S90471[44], 9}, {NULL}};
const Segment SGI234811[] = {{&S90471[44], 21}, {NULL}};
const Segment SGI234812[] = {{&S90471[30], 6}, {NULL}};
const Segment SGI234813[] = {{&S100362[94], 9}, {NULL}};
const Segment SGI234814[] = {{&S100362[67], 6}, {NULL}};
const Segment SGI234815[] = {{&S90471[114], 3}, {NULL}};
const Segment SGI234816[] = {{&S90471[79], 6}, {NULL}};
const Segment SGI234817[] = {{&S90475[34], 7}, {NULL}};
const Segment SGI234818[] = {{&S90475[27], 6}, {NULL}};
const Segment SGI234819[] = {{&S100366[34], 5}, {NULL}};
const Segment SGI234820[] = {{&S100366[27], 6}, {NULL}};
const Segment SGI234821[] = {{&S90475[94], 9}, {NULL}};
const Segment SGI234822[] = {{&S90475[67], 6}, {NULL}};
const Segment SGI234823[] = {{&S100366[96], 9}, {NULL}};
const Segment SGI234824[] = {{&S100366[69], 6}, {NULL}};
const Segment SGI234825[] = {{&S100369[34], 14}, {NULL}};
const Segment SGI234826[] = {{&S100369[27], 6}, {NULL}};
const Segment SGI234827[] = {{&S90479[34], 7}, {NULL}};
const Segment SGI234828[] = {{&S100369[101], 9}, {NULL}};
const Segment SGI234829[] = {{&S90479[27], 6}, {NULL}};
const Segment SGI234830[] = {{&S100369[74], 6}, {NULL}};
const Segment SGI234831[] = {{&S90479[94], 9}, {NULL}};
const Segment SGI234832[] = {{&S90479[67], 6}, {NULL}};
const Segment SGI234833[] = {{&S90480[34], 7}, {NULL}};
const Segment SGI234834[] = {{&S90480[27], 6}, {NULL}};
const Segment SGI234835[] = {{&S90480[95], 9}, {NULL}};
const Segment SGI234836[] = {{&S90480[68], 6}, {NULL}};
const Segment SGI234837[] = {{&S100371[34], 21}, {NULL}};
const Segment SGI234838[] = {{&S100371[27], 6}, {NULL}};
const Segment SGI234839[] = {{&S100371[108], 9}, {NULL}};
const Segment SGI234840[] = {{&S100371[81], 6}, {NULL}};
const Segment SGI234841[] = {{&S90481[34], 10}, {NULL}};
const Segment SGI234842[] = {{&S90481[27], 6}, {NULL}};
const Segment SGI234843[] = {{&S90481[97], 9}, {NULL}};
const Segment SGI234844[] = {{&S90481[70], 6}, {NULL}};
const Segment SGI234845[] = {{&S90481[138], 8}, {NULL}};
const Segment SGI234846[] = {{&S90482[34], 10}, {NULL}};
const Segment SGI234847[] = {{&S90482[27], 6}, {NULL}};
const Segment SGI234848[] = {{&S90482[97], 9}, {NULL}};
const Segment SGI234849[] = {{&S90482[70], 6}, {NULL}};
const Segment SGI234850[] = {{&S90482[131], 8}, {NULL}};
const Segment SGI234851[] = {{&S97117[34], 22}, {NULL}};
const Segment SGI234852[] = {{&S97117[27], 6}, {NULL}};
const Segment SGI234853[] = {{&S97117[109], 9}, {NULL}};
const Segment SGI234854[] = {{&S97117[82], 6}, {NULL}};
const Segment SGI234855[] = {{&S90484[34], 11}, {NULL}};
const Segment SGI234856[] = {{&S90484[27], 6}, {NULL}};
const Segment SGI234857[] = {{&S100377[34], 12}, {NULL}};
const Segment SGI234858[] = {{&S100377[27], 6}, {NULL}};
const Segment SGI234859[] = {{&S90484[98], 9}, {NULL}};
const Segment SGI234860[] = {{&S90484[71], 6}, {NULL}};
const Segment SGI234861[] = {{&S90484[139], 8}, {NULL}};
const Segment SGI234862[] = {{&S100377[99], 9}, {NULL}};
const Segment SGI234863[] = {{&S100377[72], 6}, {NULL}};
const Segment SGI234864[] = {{&S100378[34], 10}, {NULL}};
const Segment SGI234865[] = {{&S100378[27], 6}, {NULL}};
const Segment SGI234866[] = {{&S100378[96], 9}, {NULL}};
const Segment SGI234867[] = {{&S100378[69], 6}, {NULL}};
const Segment SGI234868[] = {{&S90492[44], 14}, {NULL}};
const Segment SGI234869[] = {{&S90492[44], 26}, {NULL}};
const Segment SGI234870[] = {{&S90492[30], 6}, {NULL}};
const Segment SGI234871[] = {{&S90492[119], 3}, {NULL}};
const Segment SGI234872[] = {{&S90492[84], 6}, {NULL}};
const Segment SGI234873[] = {{&S97129[44], 10}, {NULL}};
const Segment SGI234874[] = {{&S97129[44], 22}, {NULL}};
const Segment SGI234875[] = {{&S97129[30], 6}, {NULL}};
const Segment SGI234876[] = {{&S97129[80], 6}, {NULL}};
const Segment SGI234877[] = {{&S97129[107], 13}, {NULL}};
const Segment SGI234878[] = {{&S90495[44], 7}, {NULL}};
const Segment SGI234879[] = {{&S90495[44], 18}, {NULL}};
const Segment SGI234880[] = {{&S90495[30], 6}, {NULL}};
const Segment SGI234881[] = {{&S90495[111], 3}, {NULL}};
const Segment SGI234882[] = {{&S90495[76], 6}, {NULL}};
const Segment SGI234883[] = {{&S100389[44], 11}, {NULL}};
const Segment SGI234884[] = {{&S100389[44], 23}, {NULL}};
const Segment SGI234885[] = {{&S100389[30], 6}, {NULL}};
const Segment SGI234886[] = {{&S100389[31], 3}, {NULL}};
const Segment SGI234887[] = {{&S100389[81], 6}, {NULL}};
const Segment SGI234888[] = {{&S90498[44], 6}, {NULL}};
const Segment SGI234889[] = {{&S90498[44], 18}, {NULL}};
const Segment SGI234890[] = {{&S100390[34], 13}, {NULL}};
const Segment SGI234891[] = {{&S90498[30], 6}, {NULL}};
const Segment SGI234892[] = {{&S100390[27], 6}, {NULL}};
const Segment SGI234893[] = {{&S90498[76], 6}, {NULL}};
const Segment SGI234894[] = {{&S100390[100], 9}, {NULL}};
const Segment SGI234895[] = {{&S100390[73], 6}, {NULL}};
const Segment SGI234896[] = {{&S90498[103], 13}, {NULL}};
const Segment SGI234897[] = {{&S90503[59], 11}, {NULL}};
const Segment SGI234898[] = {{&S90503[59], 23}, {NULL}};
const Segment SGI234899[] = {{&S90503[45], 6}, {NULL}};
const Segment SGI234900[] = {{&S90503[131], 3}, {NULL}};
const Segment SGI234901[] = {{&S100395[34], 21}, {NULL}};
const Segment SGI234902[] = {{&S100395[27], 6}, {NULL}};
const Segment SGI234903[] = {{&S90503[96], 6}, {NULL}};
const Segment SGI234904[] = {{&S90503[0], 14}, {NULL}};
const Segment SGI234905[] = {{&S100395[118], 9}, {NULL}};
const Segment SGI234906[] = {{&S100395[91], 6}, {NULL}};
const Segment SGI234907[] = {{&S90504[34], 10}, {NULL}};
const Segment SGI234908[] = {{&S90504[27], 6}, {NULL}};
const Segment SGI234909[] = {{&S90504[97], 9}, {NULL}};
const Segment SGI234910[] = {{&S90504[70], 6}, {NULL}};
const Segment SGI234911[] = {{&S100396[34], 7}, {NULL}};
const Segment SGI234912[] = {{&S100396[27], 6}, {NULL}};
const Segment SGI234913[] = {{&S100396[94], 9}, {NULL}};
const Segment SGI234914[] = {{&S100396[67], 6}, {NULL}};
const Segment SGI234915[] = {{&S90505[34], 9}, {NULL}};
const Segment SGI234916[] = {{&S90505[27], 6}, {NULL}};
const Segment SGI234917[] = {{&S90505[96], 9}, {NULL}};
const Segment SGI234918[] = {{&S90505[69], 6}, {NULL}};
const Segment SGI234919[] = {{&S97147[34], 16}, {NULL}};
const Segment SGI234920[] = {{&S100399[34], 12}, {NULL}};
const Segment SGI234921[] = {{&S97147[27], 6}, {NULL}};
const Segment SGI234922[] = {{&S100399[27], 6}, {NULL}};
const Segment SGI234923[] = {{&S97147[103], 9}, {NULL}};
const Segment SGI234924[] = {{&S97147[76], 6}, {NULL}};
const Segment SGI234925[] = {{&S100399[99], 9}, {NULL}};
const Segment SGI234926[] = {{&S100399[72], 6}, {NULL}};
const Segment SGI234927[] = {{&S90510[34], 11}, {NULL}};
const Segment SGI234928[] = {{&S90510[27], 6}, {NULL}};
const Segment SGI234929[] = {{&S90510[98], 9}, {NULL}};
const Segment SGI234930[] = {{&S90510[71], 6}, {NULL}};
const Segment SGI234931[] = {{&S90516[44], 9}, {NULL}};
const Segment SGI234932[] = {{&S90516[44], 21}, {NULL}};
const Segment SGI234933[] = {{&S97156[44], 18}, {NULL}};
const Segment SGI234934[] = {{&S90516[30], 6}, {NULL}};
const Segment SGI234935[] = {{&S97156[44], 39}, {NULL}};
const Segment SGI234936[] = {{&S97156[30], 6}, {NULL}};
const Segment SGI234937[] = {{&S90516[114], 3}, {NULL}};
const Segment SGI234938[] = {{&S90516[79], 6}, {NULL}};
const Segment SGI234939[] = {{&S97156[130], 3}, {NULL}};
const Segment SGI234940[] = {{&S97156[97], 5}, {NULL}};
const Segment SGI234941[] = {{&S97156[134], 14}, {NULL}};
const Segment SGI234942[] = {{&S90518[34], 10}, {NULL}};
const Segment SGI234943[] = {{&S90518[27], 6}, {NULL}};
const Segment SGI234944[] = {{&S90518[97], 9}, {NULL}};
const Segment SGI234945[] = {{&S90518[70], 6}, {NULL}};
const Segment SGI234946[] = {{&S90524[34], 24}, {NULL}};
const Segment SGI234947[] = {{&S90524[27], 6}, {NULL}};
const Segment SGI234948[] = {{&S90524[111], 9}, {NULL}};
const Segment SGI234949[] = {{&S90524[84], 6}, {NULL}};
const Segment SGI234950[] = {{&S100418[34], 12}, {NULL}};
const Segment SGI234951[] = {{&S100418[27], 6}, {NULL}};
const Segment SGI234952[] = {{&S100418[99], 9}, {NULL}};
const Segment SGI234953[] = {{&S100418[72], 6}, {NULL}};
const Segment SGI234954[] = {{&S97168[34], 15}, {NULL}};
const Segment SGI234955[] = {{&S97168[27], 6}, {NULL}};
const Segment SGI234956[] = {{&S97168[102], 9}, {NULL}};
const Segment SGI234957[] = {{&S97168[75], 6}, {NULL}};
const Segment SGI234958[] = {{&S100421[34], 21}, {NULL}};
const Segment SGI234959[] = {{&S100421[27], 6}, {NULL}};
const Segment SGI234960[] = {{&S100421[118], 9}, {NULL}};
const Segment SGI234961[] = {{&S100421[91], 6}, {NULL}};
const Segment SGI234962[] = {{&S90529[44], 10}, {NULL}};
const Segment SGI234963[] = {{&S90529[44], 22}, {NULL}};
const Segment SGI234964[] = {{&S90529[30], 6}, {NULL}};
const Segment SGI234965[] = {{&S90529[113], 3}, {NULL}};
const Segment SGI234966[] = {{&S90529[80], 5}, {NULL}};
const Segment SGI234967[] = {{&S100422[34], 12}, {NULL}};
const Segment SGI234968[] = {{&S90529[117], 14}, {NULL}};
const Segment SGI234969[] = {{&S100422[27], 6}, {NULL}};
const Segment SGI234970[] = {{&S100422[99], 9}, {NULL}};
const Segment SGI234971[] = {{&S100422[72], 6}, {NULL}};
const Segment SGI234972[] = {{&S90530[34], 11}, {NULL}};
const Segment SGI234973[] = {{&S90530[27], 6}, {NULL}};
const Segment SGI234974[] = {{&S90530[98], 9}, {NULL}};
const Segment SGI234975[] = {{&S90530[71], 6}, {NULL}};
const Segment SGI234976[] = {{&S100425[44], 14}, {NULL}};
const Segment SGI234977[] = {{&S100425[44], 31}, {NULL}};
const Segment SGI234978[] = {{&S100425[30], 6}, {NULL}};
const Segment SGI234979[] = {{&S100425[31], 3}, {NULL}};
const Segment SGI234980[] = {{&S100425[89], 5}, {NULL}};
const Segment SGI234981[] = {{&S100425[126], 14}, {NULL}};
const Segment SGI234982[] = {{&S100427[44], 6}, {NULL}};
const Segment SGI234983[] = {{&S100427[44], 18}, {NULL}};
const Segment SGI234984[] = {{&S100427[30], 6}, {NULL}};
const Segment SGI234985[] = {{&S100427[31], 3}, {NULL}};
const Segment SGI234986[] = {{&S100427[76], 5}, {NULL}};
const Segment SGI234987[] = {{&S100427[113], 14}, {NULL}};
const Segment SGI234988[] = {{&S97182[34], 8}, {NULL}};
const Segment SGI234989[] = {{&S97182[27], 6}, {NULL}};
const Segment SGI234990[] = {{&S97182[95], 9}, {NULL}};
const Segment SGI234991[] = {{&S97182[68], 6}, {NULL}};
const Segment SGI234992[] = {{&S100433[34], 9}, {NULL}};
const Segment SGI234993[] = {{&S100433[27], 6}, {NULL}};
const Segment SGI234994[] = {{&S100433[96], 9}, {NULL}};
const Segment SGI234995[] = {{&S100433[69], 6}, {NULL}};
const Segment SGI234996[] = {{&S97187[44], 12}, {NULL}};
const Segment SGI234997[] = {{&S97187[44], 24}, {NULL}};
const Segment SGI234998[] = {{&S97187[30], 6}, {NULL}};
const Segment SGI234999[] = {{&S97187[82], 6}, {NULL}};
const Segment SGI235000[] = {{&S97187[109], 13}, {NULL}};
const Segment SGI235001[] = {{&S91090[34], 10}, {NULL}};
const Segment SGI235002[] = {{&S91090[27], 6}, {NULL}};
const Segment SGI235003[] = {{&S91090[97], 9}, {NULL}};
const Segment SGI235004[] = {{&S91090[70], 6}, {NULL}};
const Segment SGI235005[] = {{&S97192[44], 18}, {NULL}};
const Segment SGI235006[] = {{&S97192[44], 39}, {NULL}};
const Segment SGI235007[] = {{&S97192[30], 6}, {NULL}};
const Segment SGI235008[] = {{&S97192[130], 3}, {NULL}};
const Segment SGI235009[] = {{&S97192[97], 5}, {NULL}};
const Segment SGI235010[] = {{&S97192[134], 14}, {NULL}};
const Segment SGI235011[] = {{&S100447[34], 24}, {NULL}};
const Segment SGI235012[] = {{&S100447[27], 6}, {NULL}};
const Segment SGI235013[] = {{&S100447[111], 9}, {NULL}};
const Segment SGI235014[] = {{&S100447[84], 6}, {NULL}};
const Segment SGI235015[] = {{&S100447[152], 8}, {NULL}};
const Segment SGI235016[] = {{&S100451[34], 17}, {NULL}};
const Segment SGI235017[] = {{&S100451[27], 6}, {NULL}};
const Segment SGI235018[] = {{&S100451[104], 9}, {NULL}};
const Segment SGI235019[] = {{&S100451[77], 6}, {NULL}};
const Segment SGI235020[] = {{&S100451[138], 8}, {NULL}};
const Segment SGI235021[] = {{&S82799[44], 10}, {NULL}};
const Segment SGI235022[] = {{&S82799[44], 21}, {NULL}};
const Segment SGI235023[] = {{&S100453[34], 12}, {NULL}};
const Segment SGI235024[] = {{&S82799[30], 6}, {NULL}};
const Segment SGI235025[] = {{&S100453[27], 6}, {NULL}};
const Segment SGI235026[] = {{&S82799[114], 3}, {NULL}};
const Segment SGI235027[] = {{&S100453[99], 9}, {NULL}};
const Segment SGI235028[] = {{&S100453[72], 6}, {NULL}};
const Segment SGI235029[] = {{&S82799[79], 6}, {NULL}};
const Segment SGI235030[] = {{&S82799[138], 10}, {NULL}};
const Segment SGI235031[] = {{&S82803[34], 10}, {NULL}};
const Segment SGI235032[] = {{&S82803[27], 6}, {NULL}};
const Segment SGI235033[] = {{&S82803[97], 9}, {NULL}};
const Segment SGI235034[] = {{&S82803[70], 6}, {NULL}};
const Segment SGI235035[] = {{&S100462[34], 7}, {NULL}};
const Segment SGI235036[] = {{&S100462[27], 6}, {NULL}};
const Segment SGI235037[] = {{&S100462[94], 9}, {NULL}};
const Segment SGI235038[] = {{&S100462[67], 6}, {NULL}};
const Segment SGI235039[] = {{&S90584[34], 10}, {NULL}};
const Segment SGI235040[] = {{&S100463[44], 11}, {NULL}};
const Segment SGI235041[] = {{&S90584[27], 6}, {NULL}};
const Segment SGI235042[] = {{&S100463[44], 23}, {NULL}};
const Segment SGI235043[] = {{&S100463[30], 6}, {NULL}};
const Segment SGI235044[] = {{&S90584[97], 9}, {NULL}};
const Segment SGI235045[] = {{&S100463[31], 3}, {NULL}};
const Segment SGI235046[] = {{&S90584[70], 6}, {NULL}};
const Segment SGI235047[] = {{&S100463[81], 5}, {NULL}};
const Segment SGI235048[] = {{&S100463[118], 14}, {NULL}};
const Segment SGI235049[] = {{&S97222[44], 7}, {NULL}};
const Segment SGI235050[] = {{&S97222[44], 23}, {NULL}};
const Segment SGI235051[] = {{&S97222[30], 6}, {NULL}};
const Segment SGI235052[] = {{&S97222[81], 6}, {NULL}};
const Segment SGI235053[] = {{&S97222[108], 13}, {NULL}};
const Segment SGI235054[] = {{&S100466[45], 7}, {NULL}};
const Segment SGI235055[] = {{&S100466[30], 6}, {NULL}};
const Segment SGI235056[] = {{&S100466[31], 3}, {NULL}};
const Segment SGI235057[] = {{&S100466[85], 5}, {NULL}};
const Segment SGI235058[] = {{&S100466[143], 13}, {NULL}};
const Segment SGI235059[] = {{&S100473[44], 7}, {NULL}};
const Segment SGI235060[] = {{&S100473[44], 24}, {NULL}};
const Segment SGI235061[] = {{&S100473[30], 6}, {NULL}};
const Segment SGI235062[] = {{&S100473[82], 6}, {NULL}};
const Segment SGI235063[] = {{&S100473[109], 13}, {NULL}};
const Segment SGI235064[] = {{&S100483[34], 7}, {NULL}};
const Segment SGI235065[] = {{&S100483[27], 6}, {NULL}};
const Segment SGI235066[] = {{&S100483[95], 9}, {NULL}};
const Segment SGI235067[] = {{&S100483[68], 6}, {NULL}};
const Segment SGI235068[] = {{&S100484[44], 7}, {NULL}};
const Segment SGI235069[] = {{&S100484[44], 19}, {NULL}};
const Segment SGI235070[] = {{&S100484[30], 6}, {NULL}};
const Segment SGI235071[] = {{&S100484[31], 3}, {NULL}};
const Segment SGI235072[] = {{&S100484[77], 5}, {NULL}};
const Segment SGI235073[] = {{&S100484[114], 14}, {NULL}};
const Segment SGI235074[] = {{&S90615[34], 7}, {NULL}};
const Segment SGI235075[] = {{&S90615[27], 6}, {NULL}};
const Segment SGI235076[] = {{&S90615[94], 9}, {NULL}};
const Segment SGI235077[] = {{&S90615[67], 6}, {NULL}};
const Segment SGI235078[] = {{&S100489[44], 11}, {NULL}};
const Segment SGI235079[] = {{&S100489[44], 23}, {NULL}};
const Segment SGI235080[] = {{&S100489[30], 6}, {NULL}};
const Segment SGI235081[] = {{&S90617[34], 7}, {NULL}};
const Segment SGI235082[] = {{&S90617[27], 6}, {NULL}};
const Segment SGI235083[] = {{&S100489[81], 6}, {NULL}};
const Segment SGI235084[] = {{&S100489[108], 13}, {NULL}};
const Segment SGI235085[] = {{&S90617[94], 9}, {NULL}};
const Segment SGI235086[] = {{&S90617[67], 6}, {NULL}};
const Segment SGI235087[] = {{&S90619[34], 8}, {NULL}};
const Segment SGI235088[] = {{&S90619[27], 6}, {NULL}};
const Segment SGI235089[] = {{&S90619[95], 9}, {NULL}};
const Segment SGI235090[] = {{&S90619[68], 6}, {NULL}};
const Segment SGI235091[] = {{&S100499[45], 5}, {NULL}};
const Segment SGI235092[] = {{&S100499[30], 6}, {NULL}};
const Segment SGI235093[] = {{&S100499[31], 3}, {NULL}};
const Segment SGI235094[] = {{&S100499[76], 5}, {NULL}};
const Segment SGI235095[] = {{&S100499[134], 13}, {NULL}};
const Segment SGI235096[] = {{&S100501[44], 11}, {NULL}};
const Segment SGI235097[] = {{&S100501[44], 23}, {NULL}};
const Segment SGI235098[] = {{&S100501[30], 6}, {NULL}};
const Segment SGI235099[] = {{&S100501[81], 6}, {NULL}};
const Segment SGI235100[] = {{&S100501[108], 13}, {NULL}};
const Segment SGI235101[] = {{&S100507[34], 4}, {NULL}};
const Segment SGI235102[] = {{&S100507[27], 6}, {NULL}};
const Segment SGI235103[] = {{&S100507[91], 9}, {NULL}};
const Segment SGI235104[] = {{&S100507[64], 6}, {NULL}};
const Segment SGI235105[] = {{&S100511[34], 16}, {NULL}};
const Segment SGI235106[] = {{&S90642[34], 11}, {NULL}};
const Segment SGI235107[] = {{&S100511[27], 6}, {NULL}};
const Segment SGI235108[] = {{&S90642[27], 6}, {NULL}};
const Segment SGI235109[] = {{&S100511[103], 9}, {NULL}};
const Segment SGI235110[] = {{&S90642[98], 9}, {NULL}};
const Segment SGI235111[] = {{&S100511[76], 6}, {NULL}};
const Segment SGI235112[] = {{&S90642[71], 6}, {NULL}};
const Segment SGI235113[] = {{&S82870[34], 7}, {NULL}};
const Segment SGI235114[] = {{&S82870[27], 6}, {NULL}};
const Segment SGI235115[] = {{&S82870[94], 9}, {NULL}};
const Segment SGI235116[] = {{&S82870[67], 6}, {NULL}};
const Segment SGI235117[] = {{&S100515[44], 7}, {NULL}};
const Segment SGI235118[] = {{&S100515[44], 19}, {NULL}};
const Segment SGI235119[] = {{&S100515[30], 6}, {NULL}};
const Segment SGI235120[] = {{&S100515[31], 3}, {NULL}};
const Segment SGI235121[] = {{&S100515[77], 5}, {NULL}};
const Segment SGI235122[] = {{&S100515[114], 14}, {NULL}};
const Segment SGI235123[] = {{&S100521[44], 15}, {NULL}};
const Segment SGI235124[] = {{&S100521[44], 27}, {NULL}};
const Segment SGI235125[] = {{&S100521[30], 6}, {NULL}};
const Segment SGI235126[] = {{&S100521[31], 3}, {NULL}};
const Segment SGI235127[] = {{&S100521[85], 6}, {NULL}};
const Segment SGI235128[] = {{&S100525[44], 7}, {NULL}};
const Segment SGI235129[] = {{&S100525[44], 25}, {NULL}};
const Segment SGI235130[] = {{&S100525[30], 6}, {NULL}};
const Segment SGI235131[] = {{&S100525[31], 3}, {NULL}};
const Segment SGI235132[] = {{&S100525[83], 5}, {NULL}};
const Segment SGI235133[] = {{&S100525[120], 14}, {NULL}};
const Segment SGI235134[] = {{&S90660[34], 25}, {NULL}};
const Segment SGI235135[] = {{&S90660[27], 6}, {NULL}};
const Segment SGI235136[] = {{&S90660[112], 9}, {NULL}};
const Segment SGI235137[] = {{&S90660[85], 6}, {NULL}};
const Segment SGI235138[] = {{&S90664[34], 6}, {NULL}};
const Segment SGI235139[] = {{&S90664[27], 6}, {NULL}};
const Segment SGI235140[] = {{&S90664[93], 9}, {NULL}};
const Segment SGI235141[] = {{&S90664[66], 6}, {NULL}};
const Segment SGI235142[] = {{&S90664[134], 8}, {NULL}};
const Segment SGI235143[] = {{&S100533[44], 11}, {NULL}};
const Segment SGI235144[] = {{&S100533[44], 23}, {NULL}};
const Segment SGI235145[] = {{&S100533[30], 6}, {NULL}};
const Segment SGI235146[] = {{&S100533[31], 3}, {NULL}};
const Segment SGI235147[] = {{&S100533[81], 6}, {NULL}};
const Segment SGI235148[] = {{&S100533[151], 3}, {NULL}};
const Segment SGI235149[] = {{&S100535[44], 15}, {NULL}};
const Segment SGI235150[] = {{&S100535[44], 33}, {NULL}};
const Segment SGI235151[] = {{&S100535[30], 6}, {NULL}};
const Segment SGI235152[] = {{&S100535[91], 6}, {NULL}};
const Segment SGI235153[] = {{&S100535[118], 13}, {NULL}};
const Segment SGI235154[] = {{&S100537[44], 10}, {NULL}};
const Segment SGI235155[] = {{&S100537[44], 22}, {NULL}};
const Segment SGI235156[] = {{&S100537[30], 6}, {NULL}};
const Segment SGI235157[] = {{&S100537[158], 3}, {NULL}};
const Segment SGI235158[] = {{&S100537[124], 3}, {NULL}};
const Segment SGI235159[] = {{&S100537[89], 6}, {NULL}};
const Segment SGI235160[] = {{&S100537[144], 8}, {NULL}};
const Segment SGI235161[] = {{&S100538[44], 11}, {NULL}};
const Segment SGI235162[] = {{&S100538[44], 35}, {NULL}};
const Segment SGI235163[] = {{&S100538[30], 6}, {NULL}};
const Segment SGI235164[] = {{&S100538[31], 3}, {NULL}};
const Segment SGI235165[] = {{&S100538[93], 6}, {NULL}};
const Segment SGI235166[] = {{&S100544[45], 13}, {NULL}};
const Segment SGI235167[] = {{&S100544[30], 6}, {NULL}};
const Segment SGI235168[] = {{&S100544[31], 3}, {NULL}};
const Segment SGI235169[] = {{&S100544[84], 5}, {NULL}};
const Segment SGI235170[] = {{&S100544[142], 13}, {NULL}};
const Segment SGI235171[] = {{&S91240[34], 15}, {NULL}};
const Segment SGI235172[] = {{&S91240[27], 6}, {NULL}};
const Segment SGI235173[] = {{&S91240[102], 9}, {NULL}};
const Segment SGI235174[] = {{&S91240[75], 6}, {NULL}};
const Segment SGI235175[] = {{&S100549[34], 10}, {NULL}};
const Segment SGI235176[] = {{&S100549[27], 6}, {NULL}};
const Segment SGI235177[] = {{&S97345[34], 11}, {NULL}};
const Segment SGI235178[] = {{&S100549[97], 9}, {NULL}};
const Segment SGI235179[] = {{&S100549[70], 6}, {NULL}};
const Segment SGI235180[] = {{&S97345[27], 6}, {NULL}};
const Segment SGI235181[] = {{&S97345[98], 9}, {NULL}};
const Segment SGI235182[] = {{&S97345[71], 6}, {NULL}};
const Segment SGI235183[] = {{&S100550[44], 7}, {NULL}};
const Segment SGI235184[] = {{&S100550[44], 19}, {NULL}};
const Segment SGI235185[] = {{&S100550[30], 6}, {NULL}};
const Segment SGI235186[] = {{&S100550[77], 6}, {NULL}};
const Segment SGI235187[] = {{&S100550[104], 13}, {NULL}};
const Segment SGI235188[] = {{&S82923[44], 7}, {NULL}};
const Segment SGI235189[] = {{&S82923[44], 18}, {NULL}};
const Segment SGI235190[] = {{&S82923[30], 6}, {NULL}};
const Segment SGI235191[] = {{&S82923[140], 3}, {NULL}};
const Segment SGI235192[] = {{&S82923[105], 6}, {NULL}};
const Segment SGI235193[] = {{&S100552[44], 6}, {NULL}};
const Segment SGI235194[] = {{&S100552[44], 18}, {NULL}};
const Segment SGI235195[] = {{&S100552[30], 6}, {NULL}};
const Segment SGI235196[] = {{&S100552[76], 6}, {NULL}};
const Segment SGI235197[] = {{&S100552[103], 13}, {NULL}};
const Segment SGI235198[] = {{&S100558[44], 13}, {NULL}};
const Segment SGI235199[] = {{&S100558[44], 25}, {NULL}};
const Segment SGI235200[] = {{&S100558[30], 6}, {NULL}};
const Segment SGI235201[] = {{&S100558[83], 6}, {NULL}};
const Segment SGI235202[] = {{&S100558[110], 13}, {NULL}};
const Segment SGI235203[] = {{&S91255[34], 10}, {NULL}};
const Segment SGI235204[] = {{&S91255[27], 6}, {NULL}};
const Segment SGI235205[] = {{&S91255[110], 9}, {NULL}};
const Segment SGI235206[] = {{&S91255[83], 6}, {NULL}};
const Segment SGI235207[] = {{&S100565[34], 10}, {NULL}};
const Segment SGI235208[] = {{&S82940[34], 7}, {NULL}};
const Segment SGI235209[] = {{&S100565[27], 6}, {NULL}};
const Segment SGI235210[] = {{&S82940[27], 6}, {NULL}};
const Segment SGI235211[] = {{&S100565[94], 9}, {NULL}};
const Segment SGI235212[] = {{&S82940[93], 9}, {NULL}};
const Segment SGI235213[] = {{&S100565[67], 6}, {NULL}};
const Segment SGI235214[] = {{&S82940[66], 6}, {NULL}};
const Segment SGI235215[] = {{&S90713[44], 10}, {NULL}};
const Segment SGI235216[] = {{&S90713[44], 22}, {NULL}};
const Segment SGI235217[] = {{&S90713[30], 6}, {NULL}};
const Segment SGI235218[] = {{&S90713[113], 3}, {NULL}};
const Segment SGI235219[] = {{&S90713[80], 5}, {NULL}};
const Segment SGI235220[] = {{&S90713[117], 14}, {NULL}};
const Segment SGI235221[] = {{&S100573[52], 10}, {NULL}};
const Segment SGI235222[] = {{&S100573[52], 22}, {NULL}};
const Segment SGI235223[] = {{&S100573[38], 6}, {NULL}};
const Segment SGI235224[] = {{&S100573[88], 5}, {NULL}};
const Segment SGI235225[] = {{&S100573[0], 14}, {NULL}};
const Segment SGI235226[] = {{&S100576[44], 13}, {NULL}};
const Segment SGI235227[] = {{&S100576[44], 25}, {NULL}};
const Segment SGI235228[] = {{&S100576[30], 6}, {NULL}};
const Segment SGI235229[] = {{&S100576[83], 6}, {NULL}};
const Segment SGI235230[] = {{&S100576[110], 13}, {NULL}};
const Segment SGI235231[] = {{&S97381[34], 12}, {NULL}};
const Segment SGI235232[] = {{&S97381[27], 6}, {NULL}};
const Segment SGI235233[] = {{&S90721[44], 15}, {NULL}};
const Segment SGI235234[] = {{&S90721[44], 27}, {NULL}};
const Segment SGI235235[] = {{&S90721[30], 6}, {NULL}};
const Segment SGI235236[] = {{&S97381[99], 9}, {NULL}};
const Segment SGI235237[] = {{&S97381[72], 6}, {NULL}};
const Segment SGI235238[] = {{&S90721[118], 3}, {NULL}};
const Segment SGI235239[] = {{&S90721[85], 5}, {NULL}};
const Segment SGI235240[] = {{&S90721[122], 14}, {NULL}};
const Segment SGI235241[] = {{&S90722[44], 13}, {NULL}};
const Segment SGI235242[] = {{&S90722[44], 25}, {NULL}};
const Segment SGI235243[] = {{&S90722[30], 6}, {NULL}};
const Segment SGI235244[] = {{&S90722[116], 3}, {NULL}};
const Segment SGI235245[] = {{&S90722[83], 5}, {NULL}};
const Segment SGI235246[] = {{&S90722[120], 14}, {NULL}};
const Segment SGI235247[] = {{&S100588[34], 11}, {NULL}};
const Segment SGI235248[] = {{&S100588[27], 6}, {NULL}};
const Segment SGI235249[] = {{&S100588[98], 9}, {NULL}};
const Segment SGI235250[] = {{&S100588[71], 6}, {NULL}};
const Segment SGI235251[] = {{&S100589[34], 11}, {NULL}};
const Segment SGI235252[] = {{&S97394[34], 7}, {NULL}};
const Segment SGI235253[] = {{&S100589[27], 6}, {NULL}};
const Segment SGI235254[] = {{&S97394[27], 6}, {NULL}};
const Segment SGI235255[] = {{&S100589[98], 9}, {NULL}};
const Segment SGI235256[] = {{&S100589[71], 6}, {NULL}};
const Segment SGI235257[] = {{&S97394[94], 9}, {NULL}};
const Segment SGI235258[] = {{&S97394[67], 6}, {NULL}};
const Segment SGI235259[] = {{&S90734[34], 11}, {NULL}};
const Segment SGI235260[] = {{&S90734[27], 6}, {NULL}};
const Segment SGI235261[] = {{&S90734[98], 9}, {NULL}};
const Segment SGI235262[] = {{&S90734[71], 6}, {NULL}};
const Segment SGI235263[] = {{&S97395[34], 15}, {NULL}};
const Segment SGI235264[] = {{&S97395[27], 6}, {NULL}};
const Segment SGI235265[] = {{&S97395[102], 9}, {NULL}};
const Segment SGI235266[] = {{&S97395[75], 6}, {NULL}};
const Segment SGI235267[] = {{&S90735[34], 10}, {NULL}};
const Segment SGI235268[] = {{&S90735[27], 6}, {NULL}};
const Segment SGI235269[] = {{&S90735[97], 9}, {NULL}};
const Segment SGI235270[] = {{&S90735[70], 6}, {NULL}};
const Segment SGI235271[] = {{&S100593[34], 9}, {NULL}};
const Segment SGI235272[] = {{&S100593[27], 6}, {NULL}};
const Segment SGI235273[] = {{&S100593[96], 9}, {NULL}};
const Segment SGI235274[] = {{&S100593[69], 6}, {NULL}};
const Segment SGI235275[] = {{&S100610[34], 15}, {NULL}};
const Segment SGI235276[] = {{&S100610[27], 6}, {NULL}};
const Segment SGI235277[] = {{&S100610[108], 9}, {NULL}};
const Segment SGI235278[] = {{&S100610[81], 6}, {NULL}};
const Segment SGI235279[] = {{&S100613[44], 14}, {NULL}};
const Segment SGI235280[] = {{&S100613[44], 26}, {NULL}};
const Segment SGI235281[] = {{&S100613[30], 6}, {NULL}};
const Segment SGI235282[] = {{&S100613[31], 3}, {NULL}};
const Segment SGI235283[] = {{&S100613[84], 6}, {NULL}};
const Segment SGI235284[] = {{&S100613[154], 12}, {NULL}};
const Segment SGI235285[] = {{&S100621[44], 15}, {NULL}};
const Segment SGI235286[] = {{&S100621[44], 42}, {NULL}};
const Segment SGI235287[] = {{&S100621[30], 6}, {NULL}};
const Segment SGI235288[] = {{&S100621[31], 3}, {NULL}};
const Segment SGI235289[] = {{&S100621[100], 5}, {NULL}};
const Segment SGI235290[] = {{&S100621[137], 14}, {NULL}};
const Segment SGI235291[] = {{&S100628[44], 9}, {NULL}};
const Segment SGI235292[] = {{&S100628[44], 21}, {NULL}};
const Segment SGI235293[] = {{&S100628[30], 6}, {NULL}};
const Segment SGI235294[] = {{&S100628[79], 6}, {NULL}};
const Segment SGI235295[] = {{&S100628[106], 13}, {NULL}};
const Segment SGI235296[] = {{&S97440[34], 7}, {NULL}};
const Segment SGI235297[] = {{&S97440[27], 6}, {NULL}};
const Segment SGI235298[] = {{&S97440[94], 9}, {NULL}};
const Segment SGI235299[] = {{&S97440[67], 6}, {NULL}};
const Segment SGI235300[] = {{&S83019[44], 9}, {NULL}};
const Segment SGI235301[] = {{&S83019[44], 20}, {NULL}};
const Segment SGI235302[] = {{&S83019[30], 6}, {NULL}};
const Segment SGI235303[] = {{&S83019[78], 6}, {NULL}};
const Segment SGI235304[] = {{&S83019[105], 13}, {NULL}};
const Segment SGI235305[] = {{&S97442[34], 15}, {NULL}};
const Segment SGI235306[] = {{&S97442[27], 6}, {NULL}};
const Segment SGI235307[] = {{&S100634[52], 6}, {NULL}};
const Segment SGI235308[] = {{&S100634[52], 18}, {NULL}};
const Segment SGI235309[] = {{&S97442[102], 9}, {NULL}};
const Segment SGI235310[] = {{&S100634[38], 6}, {NULL}};
const Segment SGI235311[] = {{&S97442[75], 6}, {NULL}};
const Segment SGI235312[] = {{&S100634[84], 5}, {NULL}};
const Segment SGI235313[] = {{&S100634[0], 14}, {NULL}};
const Segment SGI235314[] = {{&S97443[34], 7}, {NULL}};
const Segment SGI235315[] = {{&S97443[27], 6}, {NULL}};
const Segment SGI235316[] = {{&S100635[52], 6}, {NULL}};
const Segment SGI235317[] = {{&S100635[52], 18}, {NULL}};
const Segment SGI235318[] = {{&S100635[38], 6}, {NULL}};
const Segment SGI235319[] = {{&S97443[94], 9}, {NULL}};
const Segment SGI235320[] = {{&S97443[67], 6}, {NULL}};
const Segment SGI235321[] = {{&S100635[84], 5}, {NULL}};
const Segment SGI235322[] = {{&S100635[0], 14}, {NULL}};
const Segment SGI235323[] = {{&S83037[34], 14}, {NULL}};
const Segment SGI235324[] = {{&S83037[27], 6}, {NULL}};
const Segment SGI235325[] = {{&S83037[100], 9}, {NULL}};
const Segment SGI235326[] = {{&S83037[73], 6}, {NULL}};
const Segment SGI235327[] = {{&S83037[141], 8}, {NULL}};
const Segment SGI235328[] = {{&S100669[34], 15}, {NULL}};
const Segment SGI235329[] = {{&S100669[27], 6}, {NULL}};
const Segment SGI235330[] = {{&S100669[117], 9}, {NULL}};
const Segment SGI235331[] = {{&S100669[90], 6}, {NULL}};
const Segment SGI235332[] = {{&S100669[158], 8}, {NULL}};
const Segment SGI235333[] = {{&S97484[44], 6}, {NULL}};
const Segment SGI235334[] = {{&S97484[44], 18}, {NULL}};
const Segment SGI235335[] = {{&S97484[30], 6}, {NULL}};
const Segment SGI235336[] = {{&S97484[111], 3}, {NULL}};
const Segment SGI235337[] = {{&S97484[76], 6}, {NULL}};
const Segment SGI235338[] = {{&S97486[59], 4}, {NULL}};
const Segment SGI235339[] = {{&S97486[59], 16}, {NULL}};
const Segment SGI235340[] = {{&S97486[45], 6}, {NULL}};
const Segment SGI235341[] = {{&S97486[124], 3}, {NULL}};
const Segment SGI235342[] = {{&S97486[89], 6}, {NULL}};
const Segment SGI235343[] = {{&S97486[0], 14}, {NULL}};
const Segment SGI235344[] = {{&S100679[44], 7}, {NULL}};
const Segment SGI235345[] = {{&S100679[44], 24}, {NULL}};
const Segment SGI235346[] = {{&S100679[30], 6}, {NULL}};
const Segment SGI235347[] = {{&S100679[117], 3}, {NULL}};
const Segment SGI235348[] = {{&S100679[82], 6}, {NULL}};
const Segment SGI235349[] = {{&S100679[137], 8}, {NULL}};
const Segment SGI235350[] = {{&S100679[146], 13}, {NULL}};
const Segment SGI235351[] = {{&S97492[44], 19}, {NULL}};
const Segment SGI235352[] = {{&S97492[44], 31}, {NULL}};
const Segment SGI235353[] = {{&S97492[30], 6}, {NULL}};
const Segment SGI235354[] = {{&S91408[34], 8}, {NULL}};
const Segment SGI235355[] = {{&S91408[27], 6}, {NULL}};
const Segment SGI235356[] = {{&S97492[124], 3}, {NULL}};
const Segment SGI235357[] = {{&S97492[89], 6}, {NULL}};
const Segment SGI235358[] = {{&S91408[95], 9}, {NULL}};
const Segment SGI235359[] = {{&S91408[68], 6}, {NULL}};
const Segment SGI235360[] = {{&S100689[34], 10}, {NULL}};
const Segment SGI235361[] = {{&S100689[27], 6}, {NULL}};
const Segment SGI235362[] = {{&S100689[97], 9}, {NULL}};
const Segment SGI235363[] = {{&S100689[70], 6}, {NULL}};
const Segment SGI235364[] = {{&S90850[34], 7}, {NULL}};
const Segment SGI235365[] = {{&S90850[27], 6}, {NULL}};
const Segment SGI235366[] = {{&S90850[104], 9}, {NULL}};
const Segment SGI235367[] = {{&S90850[77], 6}, {NULL}};
const Segment SGI235368[] = {{&S100690[34], 5}, {NULL}};
const Segment SGI235369[] = {{&S100690[27], 6}, {NULL}};
const Segment SGI235370[] = {{&S100690[92], 9}, {NULL}};
const Segment SGI235371[] = {{&S100690[65], 6}, {NULL}};
const Segment SGI235372[] = {{&S83083[44], 10}, {NULL}};
const Segment SGI235373[] = {{&S83083[44], 21}, {NULL}};
const Segment SGI235374[] = {{&S83083[30], 6}, {NULL}};
const Segment SGI235375[] = {{&S100691[44], 6}, {NULL}};
const Segment SGI235376[] = {{&S100691[44], 18}, {NULL}};
const Segment SGI235377[] = {{&S100691[30], 6}, {NULL}};
const Segment SGI235378[] = {{&S83083[114], 3}, {NULL}};
const Segment SGI235379[] = {{&S83083[79], 6}, {NULL}};
const Segment SGI235380[] = {{&S100691[76], 6}, {NULL}};
const Segment SGI235381[] = {{&S100691[103], 13}, {NULL}};
const Segment SGI235382[] = {{&S83084[38], 10}, {NULL}};
const Segment SGI235383[] = {{&S83084[31], 6}, {NULL}};
const Segment SGI235384[] = {{&S100692[44], 13}, {NULL}};
const Segment SGI235385[] = {{&S91418[44], 10}, {NULL}};
const Segment SGI235386[] = {{&S100692[44], 25}, {NULL}};
const Segment SGI235387[] = {{&S91418[44], 22}, {NULL}};
const Segment SGI235388[] = {{&S100692[30], 6}, {NULL}};
const Segment SGI235389[] = {{&S91418[30], 6}, {NULL}};
const Segment SGI235390[] = {{&S91418[115], 3}, {NULL}};
const Segment SGI235391[] = {{&S100692[83], 6}, {NULL}};
const Segment SGI235392[] = {{&S91418[80], 6}, {NULL}};
const Segment SGI235393[] = {{&S100692[110], 13}, {NULL}};
const Segment SGI235394[] = {{&S91419[34], 17}, {NULL}};
const Segment SGI235395[] = {{&S91419[27], 6}, {NULL}};
const Segment SGI235396[] = {{&S91419[104], 9}, {NULL}};
const Segment SGI235397[] = {{&S91419[77], 6}, {NULL}};
const Segment SGI235398[] = {{&S91420[34], 12}, {NULL}};
const Segment SGI235399[] = {{&S91420[27], 6}, {NULL}};
const Segment SGI235400[] = {{&S100695[44], 15}, {NULL}};
const Segment SGI235401[] = {{&S91420[99], 9}, {NULL}};
const Segment SGI235402[] = {{&S100695[44], 27}, {NULL}};
const Segment SGI235403[] = {{&S91420[72], 6}, {NULL}};
const Segment SGI235404[] = {{&S100695[30], 6}, {NULL}};
const Segment SGI235405[] = {{&S100695[85], 6}, {NULL}};
const Segment SGI235406[] = {{&S100695[112], 13}, {NULL}};
const Segment SGI235407[] = {{&S100696[44], 5}, {NULL}};
const Segment SGI235408[] = {{&S83091[44], 17}, {NULL}};
const Segment SGI235409[] = {{&S100696[44], 17}, {NULL}};
const Segment SGI235410[] = {{&S100696[30], 6}, {NULL}};
const Segment SGI235411[] = {{&S83091[44], 38}, {NULL}};
const Segment SGI235412[] = {{&S83091[30], 6}, {NULL}};
const Segment SGI235413[] = {{&S100696[75], 6}, {NULL}};
const Segment SGI235414[] = {{&S100696[102], 13}, {NULL}};
const Segment SGI235415[] = {{&S90869[34], 10}, {NULL}};
const Segment SGI235416[] = {{&S90869[27], 6}, {NULL}};
const Segment SGI235417[] = {{&S90869[97], 9}, {NULL}};
const Segment SGI235418[] = {{&S90869[70], 6}, {NULL}};
const Segment SGI235419[] = {{&S100702[44], 4}, {NULL}};
const Segment SGI235420[] = {{&S100702[44], 16}, {NULL}};
const Segment SGI235421[] = {{&S100702[30], 6}, {NULL}};
const Segment SGI235422[] = {{&S100702[74], 6}, {NULL}};
const Segment SGI235423[] = {{&S100702[101], 13}, {NULL}};
const Segment SGI235424[] = {{&S90871[34], 10}, {NULL}};
const Segment SGI235425[] = {{&S90871[27], 6}, {NULL}};
const Segment SGI235426[] = {{&S90871[97], 9}, {NULL}};
const Segment SGI235427[] = {{&S90871[70], 6}, {NULL}};
const Segment SGI235428[] = {{&S100709[44], 6}, {NULL}};
const Segment SGI235429[] = {{&S100709[44], 18}, {NULL}};
const Segment SGI235430[] = {{&S100709[30], 6}, {NULL}};
const Segment SGI235431[] = {{&S100709[76], 6}, {NULL}};
const Segment SGI235432[] = {{&S90876[34], 9}, {NULL}};
const Segment SGI235433[] = {{&S100709[103], 13}, {NULL}};
const Segment SGI235434[] = {{&S90876[27], 6}, {NULL}};
const Segment SGI235435[] = {{&S90876[96], 9}, {NULL}};
const Segment SGI235436[] = {{&S90876[69], 6}, {NULL}};
const Segment SGI235437[] = {{&S100711[44], 10}, {NULL}};
const Segment SGI235438[] = {{&S100711[44], 22}, {NULL}};
const Segment SGI235439[] = {{&S100711[30], 6}, {NULL}};
const Segment SGI235440[] = {{&S90878[34], 7}, {NULL}};
const Segment SGI235441[] = {{&S90878[27], 6}, {NULL}};
const Segment SGI235442[] = {{&S100711[80], 6}, {NULL}};
const Segment SGI235443[] = {{&S100711[107], 13}, {NULL}};
const Segment SGI235444[] = {{&S90878[98], 9}, {NULL}};
const Segment SGI235445[] = {{&S90878[71], 6}, {NULL}};
const Segment SGI235446[] = {{&S90878[139], 8}, {NULL}};
const Segment SGI235447[] = {{&S90881[59], 10}, {NULL}};
const Segment SGI235448[] = {{&S90881[59], 22}, {NULL}};
const Segment SGI235449[] = {{&S90881[45], 6}, {NULL}};
const Segment SGI235450[] = {{&S100715[34], 7}, {NULL}};
const Segment SGI235451[] = {{&S100715[27], 6}, {NULL}};
const Segment SGI235452[] = {{&S90881[130], 3}, {NULL}};
const Segment SGI235453[] = {{&S90881[95], 6}, {NULL}};
const Segment SGI235454[] = {{&S100715[99], 9}, {NULL}};
const Segment SGI235455[] = {{&S90881[0], 14}, {NULL}};
const Segment SGI235456[] = {{&S100715[72], 6}, {NULL}};
const Segment SGI235457[] = {{&S100715[140], 8}, {NULL}};
const Segment SGI235458[] = {{&S100718[34], 7}, {NULL}};
const Segment SGI235459[] = {{&S100718[27], 6}, {NULL}};
const Segment SGI235460[] = {{&S100718[100], 9}, {NULL}};
const Segment SGI235461[] = {{&S100718[73], 6}, {NULL}};
const Segment SGI235462[] = {{&S90886[44], 18}, {NULL}};
const Segment SGI235463[] = {{&S90886[44], 30}, {NULL}};
const Segment SGI235464[] = {{&S90886[30], 6}, {NULL}};
const Segment SGI235465[] = {{&S90886[123], 3}, {NULL}};
const Segment SGI235466[] = {{&S90886[88], 6}, {NULL}};
const Segment SGI235467[] = {{&S90886[148], 7}, {NULL}};
const Segment SGI235468[] = {{&S90892[34], 10}, {NULL}};
const Segment SGI235469[] = {{&S90892[27], 6}, {NULL}};
const Segment SGI235470[] = {{&S90892[96], 9}, {NULL}};
const Segment SGI235471[] = {{&S90892[69], 6}, {NULL}};
const Segment SGI235472[] = {{&S97562[34], 10}, {NULL}};
const Segment SGI235473[] = {{&S97562[27], 6}, {NULL}};
const Segment SGI235474[] = {{&S97562[97], 9}, {NULL}};
const Segment SGI235475[] = {{&S97562[70], 6}, {NULL}};
const Segment SGI235476[] = {{&S90911[44], 14}, {NULL}};
const Segment SGI235477[] = {{&S90911[44], 31}, {NULL}};
const Segment SGI235478[] = {{&S90911[30], 6}, {NULL}};
const Segment SGI235479[] = {{&S90911[89], 6}, {NULL}};
const Segment SGI235480[] = {{&S90911[116], 13}, {NULL}};
const Segment SGI235481[] = {{&S97569[44], 14}, {NULL}};
const Segment SGI235482[] = {{&S97569[44], 33}, {NULL}};
const Segment SGI235483[] = {{&S97569[30], 6}, {NULL}};
const Segment SGI235484[] = {{&S97569[126], 3}, {NULL}};
const Segment SGI235485[] = {{&S97569[91], 6}, {NULL}};
const Segment SGI235486[] = {{&S90919[44], 10}, {NULL}};
const Segment SGI235487[] = {{&S90919[44], 22}, {NULL}};
const Segment SGI235488[] = {{&S90919[30], 6}, {NULL}};
const Segment SGI235489[] = {{&S90919[113], 3}, {NULL}};
const Segment SGI235490[] = {{&S90919[80], 5}, {NULL}};
const Segment SGI235491[] = {{&S90919[117], 14}, {NULL}};
const Segment SGI235492[] = {{&S90920[44], 10}, {NULL}};
const Segment SGI235493[] = {{&S90920[44], 22}, {NULL}};
const Segment SGI235494[] = {{&S90920[30], 6}, {NULL}};
const Segment SGI235495[] = {{&S90920[113], 3}, {NULL}};
const Segment SGI235496[] = {{&S90920[80], 5}, {NULL}};
const Segment SGI235497[] = {{&S90920[117], 14}, {NULL}};
const Segment SGI235498[] = {{&S97589[34], 13}, {NULL}};
const Segment SGI235499[] = {{&S97589[27], 6}, {NULL}};
const Segment SGI235500[] = {{&S97589[100], 9}, {NULL}};
const Segment SGI235501[] = {{&S97589[73], 6}, {NULL}};
const Segment SGI235502[] = {{&S100797[59], 10}, {NULL}};
const Segment SGI235503[] = {{&S100797[59], 22}, {NULL}};
const Segment SGI235504[] = {{&S100797[45], 6}, {NULL}};
const Segment SGI235505[] = {{&S100797[46], 3}, {NULL}};
const Segment SGI235506[] = {{&S100797[95], 6}, {NULL}};
const Segment SGI235507[] = {{&S100797[0], 14}, {NULL}};
const Segment SGI235508[] = {{&S97609[34], 16}, {NULL}};
const Segment SGI235509[] = {{&S97609[27], 6}, {NULL}};
const Segment SGI235510[] = {{&S97609[113], 9}, {NULL}};
const Segment SGI235511[] = {{&S97609[86], 6}, {NULL}};
const Segment SGI235512[] = {{&S91541[34], 11}, {NULL}};
const Segment SGI235513[] = {{&S91541[27], 6}, {NULL}};
const Segment SGI235514[] = {{&S91541[98], 9}, {NULL}};
const Segment SGI235515[] = {{&S91541[71], 6}, {NULL}};
const Segment SGI235516[] = {{&S91544[34], 12}, {NULL}};
const Segment SGI235517[] = {{&S91544[27], 6}, {NULL}};
const Segment SGI235518[] = {{&S91544[99], 9}, {NULL}};
const Segment SGI235519[] = {{&S91544[72], 6}, {NULL}};
const Segment SGI235520[] = {{&S97621[34], 18}, {NULL}};
const Segment SGI235521[] = {{&S97621[27], 6}, {NULL}};
const Segment SGI235522[] = {{&S97621[114], 9}, {NULL}};
const Segment SGI235523[] = {{&S97621[87], 6}, {NULL}};
const Segment SGI235524[] = {{&S97622[34], 9}, {NULL}};
const Segment SGI235525[] = {{&S97622[27], 6}, {NULL}};
const Segment SGI235526[] = {{&S97622[96], 9}, {NULL}};
const Segment SGI235527[] = {{&S97622[69], 6}, {NULL}};
const Segment SGI235528[] = {{&S91554[44], 10}, {NULL}};
const Segment SGI235529[] = {{&S91554[44], 22}, {NULL}};
const Segment SGI235530[] = {{&S91554[30], 6}, {NULL}};
const Segment SGI235531[] = {{&S91554[144], 3}, {NULL}};
const Segment SGI235532[] = {{&S91554[109], 6}, {NULL}};
const Segment SGI235533[] = {{&S91559[44], 10}, {NULL}};
const Segment SGI235534[] = {{&S91559[44], 22}, {NULL}};
const Segment SGI235535[] = {{&S91559[30], 6}, {NULL}};
const Segment SGI235536[] = {{&S91559[133], 3}, {NULL}};
const Segment SGI235537[] = {{&S91559[98], 6}, {NULL}};
const Segment SGI235538[] = {{&S91559[68], 8}, {NULL}};
const Segment SGI235539[] = {{&S100835[34], 9}, {NULL}};
const Segment SGI235540[] = {{&S100835[27], 6}, {NULL}};
const Segment SGI235541[] = {{&S100835[96], 9}, {NULL}};
const Segment SGI235542[] = {{&S100835[69], 6}, {NULL}};
const Segment SGI235543[] = {{&S100836[30], 6}, {NULL}};
const Segment SGI235544[] = {{&S100836[31], 3}, {NULL}};
const Segment SGI235545[] = {{&S100836[95], 5}, {NULL}};
const Segment SGI235546[] = {{&S100836[153], 13}, {NULL}};
const Segment SGI235547[] = {{&S90992[34], 11}, {NULL}};
const Segment SGI235548[] = {{&S90992[27], 6}, {NULL}};
const Segment SGI235549[] = {{&S90992[98], 9}, {NULL}};
const Segment SGI235550[] = {{&S90992[71], 6}, {NULL}};
const Segment SGI235551[] = {{&S91000[45], 10}, {NULL}};
const Segment SGI235552[] = {{&S91000[30], 6}, {NULL}};
const Segment SGI235553[] = {{&S91000[115], 3}, {NULL}};
const Segment SGI235554[] = {{&S100848[34], 15}, {NULL}};
const Segment SGI235555[] = {{&S91000[81], 5}, {NULL}};
const Segment SGI235556[] = {{&S100848[27], 6}, {NULL}};
const Segment SGI235557[] = {{&S91000[139], 13}, {NULL}};
const Segment SGI235558[] = {{&S100848[102], 9}, {NULL}};
const Segment SGI235559[] = {{&S100848[75], 6}, {NULL}};
const Segment SGI235560[] = {{&S91001[44], 10}, {NULL}};
const Segment SGI235561[] = {{&S91001[44], 22}, {NULL}};
const Segment SGI235562[] = {{&S91001[30], 6}, {NULL}};
const Segment SGI235563[] = {{&S91001[115], 3}, {NULL}};
const Segment SGI235564[] = {{&S100849[34], 8}, {NULL}};
const Segment SGI235565[] = {{&S100849[27], 6}, {NULL}};
const Segment SGI235566[] = {{&S91001[80], 6}, {NULL}};
const Segment SGI235567[] = {{&S91001[135], 8}, {NULL}};
const Segment SGI235568[] = {{&S91001[144], 13}, {NULL}};
const Segment SGI235569[] = {{&S100849[95], 9}, {NULL}};
const Segment SGI235570[] = {{&S100849[68], 6}, {NULL}};
const Segment SGI235571[] = {{&S91002[59], 10}, {NULL}};
const Segment SGI235572[] = {{&S91002[59], 22}, {NULL}};
const Segment SGI235573[] = {{&S91002[45], 6}, {NULL}};
const Segment SGI235574[] = {{&S91002[130], 3}, {NULL}};
const Segment SGI235575[] = {{&S91002[95], 6}, {NULL}};
const Segment SGI235576[] = {{&S91002[11], 3}, {NULL}};
const Segment SGI235577[] = {{&S100856[45], 10}, {NULL}};
const Segment SGI235578[] = {{&S100856[30], 6}, {NULL}};
const Segment SGI235579[] = {{&S100856[31], 3}, {NULL}};
const Segment SGI235580[] = {{&S100856[81], 5}, {NULL}};
const Segment SGI235581[] = {{&S100856[139], 13}, {NULL}};
const Segment SGI235582[] = {{&S91016[34], 6}, {NULL}};
const Segment SGI235583[] = {{&S91016[27], 6}, {NULL}};
const Segment SGI235584[] = {{&S91016[104], 9}, {NULL}};
const Segment SGI235585[] = {{&S91016[77], 6}, {NULL}};
const Segment SGI235586[] = {{&S91021[45], 10}, {NULL}};
const Segment SGI235587[] = {{&S91021[30], 6}, {NULL}};
const Segment SGI235588[] = {{&S91021[115], 3}, {NULL}};
const Segment SGI235589[] = {{&S91021[81], 5}, {NULL}};
const Segment SGI235590[] = {{&S91021[139], 13}, {NULL}};
const Segment SGI235591[] = {{&S91023[44], 7}, {NULL}};
const Segment SGI235592[] = {{&S91607[34], 10}, {NULL}};
const Segment SGI235593[] = {{&S91023[44], 25}, {NULL}};
const Segment SGI235594[] = {{&S91023[30], 6}, {NULL}};
const Segment SGI235595[] = {{&S91607[27], 6}, {NULL}};
const Segment SGI235596[] = {{&S91607[97], 9}, {NULL}};
const Segment SGI235597[] = {{&S91023[83], 6}, {NULL}};
const Segment SGI235598[] = {{&S91607[70], 6}, {NULL}};
const Segment SGI235599[] = {{&S91023[110], 13}, {NULL}};
const Segment SGI235600[] = {{&S91024[44], 7}, {NULL}};
const Segment SGI235601[] = {{&S91024[44], 27}, {NULL}};
const Segment SGI235602[] = {{&S91024[30], 6}, {NULL}};
const Segment SGI235603[] = {{&S91024[85], 6}, {NULL}};
const Segment SGI235604[] = {{&S91024[112], 13}, {NULL}};
const Segment SGI235605[] = {{&S100888[44], 12}, {NULL}};
const Segment SGI235606[] = {{&S100888[44], 24}, {NULL}};
const Segment SGI235607[] = {{&S100888[30], 6}, {NULL}};
const Segment SGI235608[] = {{&S100888[31], 3}, {NULL}};
const Segment SGI235609[] = {{&S100888[97], 6}, {NULL}};
const Segment SGI235610[] = {{&S100888[70], 8}, {NULL}};
const Segment SGI235611[] = {{&S91040[34], 10}, {NULL}};
const Segment SGI235612[] = {{&S97703[45], 9}, {NULL}};
const Segment SGI235613[] = {{&S91040[27], 6}, {NULL}};
const Segment SGI235614[] = {{&S97703[30], 6}, {NULL}};
const Segment SGI235615[] = {{&S91040[97], 9}, {NULL}};
const Segment SGI235616[] = {{&S97703[114], 3}, {NULL}};
const Segment SGI235617[] = {{&S91040[70], 6}, {NULL}};
const Segment SGI235618[] = {{&S97703[80], 5}, {NULL}};
const Segment SGI235619[] = {{&S97703[138], 13}, {NULL}};
const Segment SGI235620[] = {{&S91042[34], 14}, {NULL}};
const Segment SGI235621[] = {{&S91042[27], 6}, {NULL}};
const Segment SGI235622[] = {{&S91042[101], 9}, {NULL}};
const Segment SGI235623[] = {{&S91042[74], 6}, {NULL}};
const Segment SGI235624[] = {{&S91042[142], 8}, {NULL}};
const Segment SGI235625[] = {{&S100895[34], 13}, {NULL}};
const Segment SGI235626[] = {{&S100895[27], 6}, {NULL}};
const Segment SGI235627[] = {{&S100895[100], 9}, {NULL}};
const Segment SGI235628[] = {{&S100895[73], 6}, {NULL}};
const Segment SGI235629[] = {{&S91637[34], 17}, {NULL}};
const Segment SGI235630[] = {{&S91637[27], 6}, {NULL}};
const Segment SGI235631[] = {{&S91637[104], 9}, {NULL}};
const Segment SGI235632[] = {{&S91637[77], 6}, {NULL}};
const Segment SGI235633[] = {{&S91051[34], 6}, {NULL}};
const Segment SGI235634[] = {{&S91051[27], 6}, {NULL}};
const Segment SGI235635[] = {{&S91051[93], 9}, {NULL}};
const Segment SGI235636[] = {{&S91051[66], 6}, {NULL}};
const Segment SGI235637[] = {{&S100903[59], 7}, {NULL}};
const Segment SGI235638[] = {{&S100903[59], 24}, {NULL}};
const Segment SGI235639[] = {{&S100903[45], 6}, {NULL}};
const Segment SGI235640[] = {{&S100903[46], 3}, {NULL}};
const Segment SGI235641[] = {{&S100903[97], 6}, {NULL}};
const Segment SGI235642[] = {{&S100903[0], 14}, {NULL}};
const Segment SGI235643[] = {{&S91052[44], 9}, {NULL}};
const Segment SGI235644[] = {{&S91052[44], 26}, {NULL}};
const Segment SGI235645[] = {{&S91052[30], 6}, {NULL}};
const Segment SGI235646[] = {{&S91052[84], 6}, {NULL}};
const Segment SGI235647[] = {{&S91052[111], 13}, {NULL}};
const Segment SGI235648[] = {{&S91056[44], 18}, {NULL}};
const Segment SGI235649[] = {{&S91056[44], 30}, {NULL}};
const Segment SGI235650[] = {{&S91056[30], 6}, {NULL}};
const Segment SGI235651[] = {{&S91056[143], 3}, {NULL}};
const Segment SGI235652[] = {{&S91056[108], 6}, {NULL}};
const Segment SGI235653[] = {{&S91056[76], 8}, {NULL}};
const Segment SGI235654[] = {{&S91056[175], 15}, {NULL}};
const Segment SGI235655[] = {{&S100911[45], 13}, {NULL}};
const Segment SGI235656[] = {{&S100911[30], 6}, {NULL}};
const Segment SGI235657[] = {{&S100911[31], 3}, {NULL}};
const Segment SGI235658[] = {{&S100911[84], 5}, {NULL}};
const Segment SGI235659[] = {{&S100911[142], 13}, {NULL}};
const Segment SGI235660[] = {{&S91060[44], 10}, {NULL}};
const Segment SGI235661[] = {{&S91060[44], 22}, {NULL}};
const Segment SGI235662[] = {{&S91060[30], 6}, {NULL}};
const Segment SGI235663[] = {{&S91060[115], 3}, {NULL}};
const Segment SGI235664[] = {{&S91060[80], 6}, {NULL}};
const Segment SGI235665[] = {{&S100913[45], 8}, {NULL}};
const Segment SGI235666[] = {{&S100913[30], 6}, {NULL}};
const Segment SGI235667[] = {{&S91060[147], 15}, {NULL}};
const Segment SGI235668[] = {{&S100913[31], 3}, {NULL}};
const Segment SGI235669[] = {{&S100913[83], 5}, {NULL}};
const Segment SGI235670[] = {{&S100913[141], 13}, {NULL}};
const Segment SGI235671[] = {{&S91062[44], 18}, {NULL}};
const Segment SGI235672[] = {{&S91062[44], 30}, {NULL}};
const Segment SGI235673[] = {{&S91062[30], 6}, {NULL}};
const Segment SGI235674[] = {{&S91062[123], 3}, {NULL}};
const Segment SGI235675[] = {{&S91062[88], 6}, {NULL}};
const Segment SGI235676[] = {{&S91062[148], 7}, {NULL}};
const Segment SGI235677[] = {{&S91063[44], 7}, {NULL}};
const Segment SGI235678[] = {{&S91063[44], 25}, {NULL}};
const Segment SGI235679[] = {{&S91063[30], 6}, {NULL}};
const Segment SGI235680[] = {{&S91063[83], 6}, {NULL}};
const Segment SGI235681[] = {{&S91063[110], 13}, {NULL}};
const Segment SGI235682[] = {{&S100918[45], 11}, {NULL}};
const Segment SGI235683[] = {{&S100918[30], 6}, {NULL}};
const Segment SGI235684[] = {{&S100918[31], 3}, {NULL}};
const Segment SGI235685[] = {{&S100918[82], 5}, {NULL}};
const Segment SGI235686[] = {{&S100918[140], 13}, {NULL}};
const Segment SGI235687[] = {{&S91066[34], 9}, {NULL}};
const Segment SGI235688[] = {{&S91066[27], 6}, {NULL}};
const Segment SGI235689[] = {{&S91066[96], 9}, {NULL}};
const Segment SGI235690[] = {{&S91066[69], 6}, {NULL}};
const Segment SGI235691[] = {{&S100921[34], 18}, {NULL}};
const Segment SGI235692[] = {{&S100921[27], 6}, {NULL}};
const Segment SGI235693[] = {{&S100921[105], 9}, {NULL}};
const Segment SGI235694[] = {{&S100921[78], 6}, {NULL}};
const Segment SGI235695[] = {{&S100923[34], 25}, {NULL}};
const Segment SGI235696[] = {{&S100923[27], 6}, {NULL}};
const Segment SGI235697[] = {{&S100923[112], 9}, {NULL}};
const Segment SGI235698[] = {{&S100923[85], 6}, {NULL}};
const Segment SGI235699[] = {{&S91072[34], 24}, {NULL}};
const Segment SGI235700[] = {{&S91072[27], 6}, {NULL}};
const Segment SGI235701[] = {{&S91072[111], 9}, {NULL}};
const Segment SGI235702[] = {{&S91072[84], 6}, {NULL}};
const Segment SGI235703[] = {{&S100927[34], 14}, {NULL}};
const Segment SGI235704[] = {{&S100927[27], 6}, {NULL}};
const Segment SGI235705[] = {{&S100927[101], 9}, {NULL}};
const Segment SGI235706[] = {{&S100927[74], 6}, {NULL}};
const Segment SGI235707[] = {{&S91077[44], 15}, {NULL}};
const Segment SGI235708[] = {{&S91077[44], 27}, {NULL}};
const Segment SGI235709[] = {{&S91077[30], 6}, {NULL}};
const Segment SGI235710[] = {{&S91077[85], 6}, {NULL}};
const Segment SGI235711[] = {{&S91077[112], 13}, {NULL}};
const Segment SGI235712[] = {{&S100932[34], 5}, {NULL}};
const Segment SGI235713[] = {{&S100932[27], 6}, {NULL}};
const Segment SGI235714[] = {{&S100932[92], 9}, {NULL}};
const Segment SGI235715[] = {{&S100932[65], 6}, {NULL}};
const Segment SGI235716[] = {{&S91080[44], 10}, {NULL}};
const Segment SGI235717[] = {{&S91080[44], 22}, {NULL}};
const Segment SGI235718[] = {{&S91080[30], 6}, {NULL}};
const Segment SGI235719[] = {{&S91080[80], 6}, {NULL}};
const Segment SGI235720[] = {{&S91080[107], 13}, {NULL}};
const Segment SGI235721[] = {{&S100934[34], 9}, {NULL}};
const Segment SGI235722[] = {{&S100934[27], 6}, {NULL}};
const Segment SGI235723[] = {{&S100934[96], 9}, {NULL}};
const Segment SGI235724[] = {{&S100934[69], 6}, {NULL}};
const Segment SGI235725[] = {{&S91085[44], 10}, {NULL}};
const Segment SGI235726[] = {{&S91085[44], 22}, {NULL}};
const Segment SGI235727[] = {{&S91085[30], 6}, {NULL}};
const Segment SGI235728[] = {{&S91085[80], 6}, {NULL}};
const Segment SGI235729[] = {{&S91085[107], 13}, {NULL}};
const Segment SGI235730[] = {{&S83091[131], 3}, {NULL}};
const Segment SGI235731[] = {{&S83091[96], 6}, {NULL}};
const Segment SGI235732[] = {{&S83095[34], 10}, {NULL}};
const Segment SGI235733[] = {{&S83095[27], 6}, {NULL}};
const Segment SGI235734[] = {{&S83095[97], 9}, {NULL}};
const Segment SGI235735[] = {{&S83095[70], 6}, {NULL}};
const Segment SGI235736[] = {{&S83096[34], 10}, {NULL}};
const Segment SGI235737[] = {{&S83096[27], 6}, {NULL}};
const Segment SGI235738[] = {{&S83096[97], 9}, {NULL}};
const Segment SGI235739[] = {{&S83096[70], 6}, {NULL}};
const Segment SGI235740[] = {{&S97784[34], 13}, {NULL}};
const Segment SGI235741[] = {{&S97784[27], 6}, {NULL}};
const Segment SGI235742[] = {{&S97784[100], 9}, {NULL}};
const Segment SGI235743[] = {{&S97784[73], 6}, {NULL}};
const Segment SGI235744[] = {{&S97786[34], 7}, {NULL}};
const Segment SGI235745[] = {{&S97786[27], 6}, {NULL}};
const Segment SGI235746[] = {{&S97786[94], 9}, {NULL}};
const Segment SGI235747[] = {{&S97786[67], 6}, {NULL}};
const Segment SGI235748[] = {{&S83114[38], 10}, {NULL}};
const Segment SGI235749[] = {{&S83114[31], 6}, {NULL}};
const Segment SGI235750[] = {{&S83115[44], 10}, {NULL}};
const Segment SGI235751[] = {{&S83115[44], 21}, {NULL}};
const Segment SGI235752[] = {{&S83115[30], 6}, {NULL}};
const Segment SGI235753[] = {{&S83115[114], 3}, {NULL}};
const Segment SGI235754[] = {{&S83115[79], 6}, {NULL}};
const Segment SGI235755[] = {{&S97790[34], 4}, {NULL}};
const Segment SGI235756[] = {{&S97790[27], 6}, {NULL}};
const Segment SGI235757[] = {{&S97790[91], 9}, {NULL}};
const Segment SGI235758[] = {{&S97790[64], 6}, {NULL}};
const Segment SGI235759[] = {{&S97792[34], 13}, {NULL}};
const Segment SGI235760[] = {{&S97792[27], 6}, {NULL}};
const Segment SGI235761[] = {{&S97792[100], 9}, {NULL}};
const Segment SGI235762[] = {{&S97792[73], 6}, {NULL}};
const Segment SGI235763[] = {{&S83120[34], 17}, {NULL}};
const Segment SGI235764[] = {{&S83120[27], 6}, {NULL}};
const Segment SGI235765[] = {{&S83120[103], 9}, {NULL}};
const Segment SGI235766[] = {{&S83120[76], 6}, {NULL}};
const Segment SGI235767[] = {{&S83121[34], 17}, {NULL}};
const Segment SGI235768[] = {{&S83121[27], 6}, {NULL}};
const Segment SGI235769[] = {{&S83121[103], 9}, {NULL}};
const Segment SGI235770[] = {{&S83121[76], 6}, {NULL}};
const Segment SGI235771[] = {{&S97828[34], 6}, {NULL}};
const Segment SGI235772[] = {{&S97828[27], 6}, {NULL}};
const Segment SGI235773[] = {{&S97828[93], 9}, {NULL}};
const Segment SGI235774[] = {{&S97828[66], 6}, {NULL}};
const Segment SGI235775[] = {{&S97832[34], 6}, {NULL}};
const Segment SGI235776[] = {{&S97832[27], 6}, {NULL}};
const Segment SGI235777[] = {{&S97832[93], 9}, {NULL}};
const Segment SGI235778[] = {{&S97832[66], 6}, {NULL}};
const Segment SGI235779[] = {{&S91169[59], 10}, {NULL}};
const Segment SGI235780[] = {{&S91169[59], 22}, {NULL}};
const Segment SGI235781[] = {{&S91169[45], 6}, {NULL}};
const Segment SGI235782[] = {{&S91169[130], 3}, {NULL}};
const Segment SGI235783[] = {{&S91169[95], 6}, {NULL}};
const Segment SGI235784[] = {{&S91169[0], 14}, {NULL}};
const Segment SGI235785[] = {{&S101031[44], 8}, {NULL}};
const Segment SGI235786[] = {{&S101031[44], 19}, {NULL}};
const Segment SGI235787[] = {{&S101031[30], 6}, {NULL}};
const Segment SGI235788[] = {{&S101031[108], 2}, {NULL}};
const Segment SGI235789[] = {{&S101031[77], 5}, {NULL}};
const Segment SGI235790[] = {{&S97839[44], 10}, {NULL}};
const Segment SGI235791[] = {{&S97839[44], 22}, {NULL}};
const Segment SGI235792[] = {{&S97839[30], 6}, {NULL}};
const Segment SGI235793[] = {{&S97839[80], 6}, {NULL}};
const Segment SGI235794[] = {{&S97839[107], 13}, {NULL}};
const Segment SGI235795[] = {{&S91176[44], 10}, {NULL}};
const Segment SGI235796[] = {{&S91176[44], 20}, {NULL}};
const Segment SGI235797[] = {{&S91176[30], 6}, {NULL}};
const Segment SGI235798[] = {{&S91176[113], 3}, {NULL}};
const Segment SGI235799[] = {{&S91176[78], 6}, {NULL}};
const Segment SGI235800[] = {{&S97841[34], 12}, {NULL}};
const Segment SGI235801[] = {{&S97841[27], 6}, {NULL}};
const Segment SGI235802[] = {{&S97841[112], 9}, {NULL}};
const Segment SGI235803[] = {{&S97841[85], 6}, {NULL}};
const Segment SGI235804[] = {{&S83174[34], 11}, {NULL}};
const Segment SGI235805[] = {{&S83174[27], 6}, {NULL}};
const Segment SGI235806[] = {{&S83174[97], 9}, {NULL}};
const Segment SGI235807[] = {{&S83174[70], 6}, {NULL}};
const Segment SGI235808[] = {{&S97843[44], 14}, {NULL}};
const Segment SGI235809[] = {{&S97843[44], 26}, {NULL}};
const Segment SGI235810[] = {{&S97843[30], 6}, {NULL}};
const Segment SGI235811[] = {{&S97843[119], 3}, {NULL}};
const Segment SGI235812[] = {{&S97843[84], 6}, {NULL}};
const Segment SGI235813[] = {{&S83178[44], 10}, {NULL}};
const Segment SGI235814[] = {{&S83178[44], 21}, {NULL}};
const Segment SGI235815[] = {{&S83178[30], 6}, {NULL}};
const Segment SGI235816[] = {{&S83178[114], 3}, {NULL}};
const Segment SGI235817[] = {{&S83178[79], 6}, {NULL}};
const Segment SGI235818[] = {{&S97847[34], 16}, {NULL}};
const Segment SGI235819[] = {{&S97847[27], 6}, {NULL}};
const Segment SGI235820[] = {{&S97847[113], 9}, {NULL}};
const Segment SGI235821[] = {{&S97847[86], 6}, {NULL}};
const Segment SGI235822[] = {{&S83179[44], 10}, {NULL}};
const Segment SGI235823[] = {{&S83179[44], 22}, {NULL}};
const Segment SGI235824[] = {{&S83179[30], 6}, {NULL}};
const Segment SGI235825[] = {{&S83179[115], 3}, {NULL}};
const Segment SGI235826[] = {{&S83179[80], 6}, {NULL}};
const Segment SGI235827[] = {{&S97852[34], 13}, {NULL}};
const Segment SGI235828[] = {{&S97852[27], 6}, {NULL}};
const Segment SGI235829[] = {{&S97852[100], 9}, {NULL}};
const Segment SGI235830[] = {{&S97852[73], 6}, {NULL}};
const Segment SGI235831[] = {{&S97856[44], 24}, {NULL}};
const Segment SGI235832[] = {{&S97856[44], 36}, {NULL}};
const Segment SGI235833[] = {{&S97856[30], 6}, {NULL}};
const Segment SGI235834[] = {{&S97856[129], 3}, {NULL}};
const Segment SGI235835[] = {{&S97856[94], 6}, {NULL}};
const Segment SGI235836[] = {{&S101065[44], 10}, {NULL}};
const Segment SGI235837[] = {{&S101065[44], 22}, {NULL}};
const Segment SGI235838[] = {{&S101065[30], 6}, {NULL}};
const Segment SGI235839[] = {{&S101065[31], 3}, {NULL}};
const Segment SGI235840[] = {{&S101065[80], 6}, {NULL}};
const Segment SGI235841[] = {{&S101065[132], 10}, {NULL}};
const Segment SGI235842[] = {{&S91806[34], 4}, {NULL}};
const Segment SGI235843[] = {{&S91806[27], 6}, {NULL}};
const Segment SGI235844[] = {{&S91806[91], 9}, {NULL}};
const Segment SGI235845[] = {{&S91806[64], 6}, {NULL}};
const Segment SGI235846[] = {{&S83196[34], 10}, {NULL}};
const Segment SGI235847[] = {{&S83196[27], 6}, {NULL}};
const Segment SGI235848[] = {{&S97863[34], 10}, {NULL}};
const Segment SGI235849[] = {{&S97863[27], 6}, {NULL}};
const Segment SGI235850[] = {{&S83196[97], 9}, {NULL}};
const Segment SGI235851[] = {{&S83196[70], 6}, {NULL}};
const Segment SGI235852[] = {{&S97863[102], 9}, {NULL}};
const Segment SGI235853[] = {{&S97863[75], 6}, {NULL}};
const Segment SGI235854[] = {{&S91807[34], 7}, {NULL}};
const Segment SGI235855[] = {{&S91807[27], 6}, {NULL}};
const Segment SGI235856[] = {{&S91807[94], 9}, {NULL}};
const Segment SGI235857[] = {{&S91807[67], 6}, {NULL}};
const Segment SGI235858[] = {{&S101069[44], 10}, {NULL}};
const Segment SGI235859[] = {{&S101069[44], 22}, {NULL}};
const Segment SGI235860[] = {{&S101069[30], 6}, {NULL}};
const Segment SGI235861[] = {{&S101069[111], 2}, {NULL}};
const Segment SGI235862[] = {{&S101069[80], 5}, {NULL}};
const Segment SGI235863[] = {{&S91810[34], 4}, {NULL}};
const Segment SGI235864[] = {{&S91810[27], 6}, {NULL}};
const Segment SGI235865[] = {{&S91810[91], 9}, {NULL}};
const Segment SGI235866[] = {{&S91810[64], 6}, {NULL}};
const Segment SGI235867[] = {{&S101074[44], 10}, {NULL}};
const Segment SGI235868[] = {{&S101074[44], 22}, {NULL}};
const Segment SGI235869[] = {{&S91213[34], 10}, {NULL}};
const Segment SGI235870[] = {{&S101074[30], 6}, {NULL}};
const Segment SGI235871[] = {{&S91213[27], 6}, {NULL}};
const Segment SGI235872[] = {{&S101074[31], 3}, {NULL}};
const Segment SGI235873[] = {{&S101074[80], 6}, {NULL}};
const Segment SGI235874[] = {{&S91213[97], 9}, {NULL}};
const Segment SGI235875[] = {{&S101074[133], 13}, {NULL}};
const Segment SGI235876[] = {{&S91213[70], 6}, {NULL}};
const Segment SGI235877[] = {{&S91213[138], 8}, {NULL}};
const Segment SGI235878[] = {{&S91216[34], 10}, {NULL}};
const Segment SGI235879[] = {{&S91216[27], 6}, {NULL}};
const Segment SGI235880[] = {{&S91216[97], 9}, {NULL}};
const Segment SGI235881[] = {{&S91216[70], 6}, {NULL}};
const Segment SGI235882[] = {{&S97876[45], 13}, {NULL}};
const Segment SGI235883[] = {{&S97876[30], 6}, {NULL}};
const Segment SGI235884[] = {{&S97876[118], 3}, {NULL}};
const Segment SGI235885[] = {{&S97876[84], 5}, {NULL}};
const Segment SGI235886[] = {{&S97876[142], 13}, {NULL}};
const Segment SGI235887[] = {{&S91822[44], 6}, {NULL}};
const Segment SGI235888[] = {{&S91822[44], 20}, {NULL}};
const Segment SGI235889[] = {{&S91822[30], 6}, {NULL}};
const Segment SGI235890[] = {{&S91822[78], 6}, {NULL}};
const Segment SGI235891[] = {{&S91822[115], 3}, {NULL}};
const Segment SGI235892[] = {{&S91824[44], 8}, {NULL}};
const Segment SGI235893[] = {{&S91824[44], 20}, {NULL}};
const Segment SGI235894[] = {{&S91824[30], 6}, {NULL}};
const Segment SGI235895[] = {{&S91824[113], 3}, {NULL}};
const Segment SGI235896[] = {{&S91824[78], 6}, {NULL}};
const Segment SGI235897[] = {{&S91223[34], 18}, {NULL}};
const Segment SGI235898[] = {{&S91223[27], 6}, {NULL}};
const Segment SGI235899[] = {{&S101088[34], 9}, {NULL}};
const Segment SGI235900[] = {{&S101088[27], 6}, {NULL}};
const Segment SGI235901[] = {{&S91223[105], 9}, {NULL}};
const Segment SGI235902[] = {{&S91223[78], 6}, {NULL}};
const Segment SGI235903[] = {{&S101088[96], 9}, {NULL}};
const Segment SGI235904[] = {{&S91223[146], 8}, {NULL}};
const Segment SGI235905[] = {{&S101088[69], 6}, {NULL}};
const Segment SGI235906[] = {{&S101092[34], 9}, {NULL}};
const Segment SGI235907[] = {{&S101092[27], 6}, {NULL}};
const Segment SGI235908[] = {{&S101092[96], 9}, {NULL}};
const Segment SGI235909[] = {{&S101092[69], 6}, {NULL}};
const Segment SGI235910[] = {{&S91231[44], 10}, {NULL}};
const Segment SGI235911[] = {{&S91231[44], 22}, {NULL}};
const Segment SGI235912[] = {{&S91231[30], 6}, {NULL}};
const Segment SGI235913[] = {{&S91231[80], 6}, {NULL}};
const Segment SGI235914[] = {{&S91231[107], 13}, {NULL}};
const Segment SGI235915[] = {{&S83235[44], 11}, {NULL}};
const Segment SGI235916[] = {{&S83235[44], 22}, {NULL}};
const Segment SGI235917[] = {{&S83235[30], 6}, {NULL}};
const Segment SGI235918[] = {{&S83235[133], 3}, {NULL}};
const Segment SGI235919[] = {{&S83235[98], 6}, {NULL}};
const Segment SGI235920[] = {{&S83235[68], 8}, {NULL}};
const Segment SGI235921[] = {{&S91242[52], 10}, {NULL}};
const Segment SGI235922[] = {{&S91242[52], 22}, {NULL}};
const Segment SGI235923[] = {{&S91242[38], 6}, {NULL}};
const Segment SGI235924[] = {{&S91242[88], 5}, {NULL}};
const Segment SGI235925[] = {{&S91242[11], 3}, {NULL}};
const Segment SGI235926[] = {{&S97906[44], 9}, {NULL}};
const Segment SGI235927[] = {{&S97906[44], 21}, {NULL}};
const Segment SGI235928[] = {{&S97906[30], 6}, {NULL}};
const Segment SGI235929[] = {{&S97906[112], 3}, {NULL}};
const Segment SGI235930[] = {{&S97906[79], 5}, {NULL}};
const Segment SGI235931[] = {{&S97906[116], 14}, {NULL}};
const Segment SGI235932[] = {{&S101109[45], 7}, {NULL}};
const Segment SGI235933[] = {{&S101109[30], 6}, {NULL}};
const Segment SGI235934[] = {{&S101109[31], 3}, {NULL}};
const Segment SGI235935[] = {{&S101109[83], 5}, {NULL}};
const Segment SGI235936[] = {{&S101109[141], 13}, {NULL}};
const Segment SGI235937[] = {{&S101113[45], 9}, {NULL}};
const Segment SGI235938[] = {{&S97913[44], 7}, {NULL}};
const Segment SGI235939[] = {{&S101113[30], 6}, {NULL}};
const Segment SGI235940[] = {{&S97913[44], 19}, {NULL}};
const Segment SGI235941[] = {{&S97913[30], 6}, {NULL}};
const Segment SGI235942[] = {{&S101113[31], 3}, {NULL}};
const Segment SGI235943[] = {{&S101113[80], 5}, {NULL}};
const Segment SGI235944[] = {{&S97913[77], 6}, {NULL}};
const Segment SGI235945[] = {{&S101113[138], 13}, {NULL}};
const Segment SGI235946[] = {{&S97913[104], 13}, {NULL}};
const Segment SGI235947[] = {{&S101114[34], 7}, {NULL}};
const Segment SGI235948[] = {{&S101114[27], 6}, {NULL}};
const Segment SGI235949[] = {{&S101114[93], 9}, {NULL}};
const Segment SGI235950[] = {{&S101114[66], 6}, {NULL}};
const Segment SGI235951[] = {{&S83255[44], 9}, {NULL}};
const Segment SGI235952[] = {{&S83255[44], 20}, {NULL}};
const Segment SGI235953[] = {{&S83255[30], 6}, {NULL}};
const Segment SGI235954[] = {{&S83255[78], 6}, {NULL}};
const Segment SGI235955[] = {{&S101116[34], 7}, {NULL}};
const Segment SGI235956[] = {{&S101116[27], 6}, {NULL}};
const Segment SGI235957[] = {{&S83255[105], 13}, {NULL}};
const Segment SGI235958[] = {{&S101116[95], 9}, {NULL}};
const Segment SGI235959[] = {{&S101116[68], 6}, {NULL}};
const Segment SGI235960[] = {{&S101117[34], 7}, {NULL}};
const Segment SGI235961[] = {{&S101117[27], 6}, {NULL}};
const Segment SGI235962[] = {{&S101117[94], 9}, {NULL}};
const Segment SGI235963[] = {{&S101117[67], 6}, {NULL}};
const Segment SGI235964[] = {{&S97921[44], 6}, {NULL}};
const Segment SGI235965[] = {{&S97921[44], 18}, {NULL}};
const Segment SGI235966[] = {{&S97921[30], 6}, {NULL}};
const Segment SGI235967[] = {{&S101118[34], 7}, {NULL}};
const Segment SGI235968[] = {{&S101118[27], 6}, {NULL}};
const Segment SGI235969[] = {{&S97921[76], 6}, {NULL}};
const Segment SGI235970[] = {{&S97921[103], 13}, {NULL}};
const Segment SGI235971[] = {{&S101118[95], 9}, {NULL}};
const Segment SGI235972[] = {{&S101118[68], 6}, {NULL}};
const Segment SGI235973[] = {{&S101120[34], 12}, {NULL}};
const Segment SGI235974[] = {{&S101120[27], 6}, {NULL}};
const Segment SGI235975[] = {{&S101120[112], 9}, {NULL}};
const Segment SGI235976[] = {{&S101120[85], 6}, {NULL}};
const Segment SGI235977[] = {{&S101121[34], 11}, {NULL}};
const Segment SGI235978[] = {{&S101121[27], 6}, {NULL}};
const Segment SGI235979[] = {{&S101121[98], 9}, {NULL}};
const Segment SGI235980[] = {{&S101121[71], 6}, {NULL}};
const Segment SGI235981[] = {{&S101123[34], 9}, {NULL}};
const Segment SGI235982[] = {{&S101123[27], 6}, {NULL}};
const Segment SGI235983[] = {{&S101123[96], 9}, {NULL}};
const Segment SGI235984[] = {{&S101123[69], 6}, {NULL}};
const Segment SGI235985[] = {{&S97930[34], 13}, {NULL}};
const Segment SGI235986[] = {{&S97930[27], 6}, {NULL}};
const Segment SGI235987[] = {{&S97930[100], 9}, {NULL}};
const Segment SGI235988[] = {{&S97930[73], 6}, {NULL}};
const Segment SGI235989[] = {{&S97937[44], 17}, {NULL}};
const Segment SGI235990[] = {{&S97937[44], 38}, {NULL}};
const Segment SGI235991[] = {{&S97937[30], 6}, {NULL}};
const Segment SGI235992[] = {{&S97937[131], 3}, {NULL}};
const Segment SGI235993[] = {{&S97937[96], 6}, {NULL}};
const Segment SGI235994[] = {{&S97941[34], 15}, {NULL}};
const Segment SGI235995[] = {{&S97941[27], 6}, {NULL}};
const Segment SGI235996[] = {{&S97941[102], 9}, {NULL}};
const Segment SGI235997[] = {{&S97941[75], 6}, {NULL}};
const Segment SGI235998[] = {{&S97953[34], 5}, {NULL}};
const Segment SGI235999[] = {{&S97953[27], 6}, {NULL}};
const Segment SGI236000[] = {{&S97953[92], 9}, {NULL}};
const Segment SGI236001[] = {{&S97953[65], 6}, {NULL}};
const Segment SGI236002[] = {{&S97953[126], 8}, {NULL}};
const Segment SGI236003[] = {{&S97964[44], 6}, {NULL}};
const Segment SGI236004[] = {{&S97964[44], 18}, {NULL}};
const Segment SGI236005[] = {{&S97964[30], 6}, {NULL}};
const Segment SGI236006[] = {{&S97964[76], 6}, {NULL}};
const Segment SGI236007[] = {{&S97964[103], 13}, {NULL}};
const Segment SGI236008[] = {{&S97965[34], 10}, {NULL}};
const Segment SGI236009[] = {{&S97965[27], 6}, {NULL}};
const Segment SGI236010[] = {{&S97965[97], 9}, {NULL}};
const Segment SGI236011[] = {{&S97965[70], 6}, {NULL}};
const Segment SGI236012[] = {{&S97975[44], 13}, {NULL}};
const Segment SGI236013[] = {{&S97975[44], 25}, {NULL}};
const Segment SGI236014[] = {{&S97975[30], 6}, {NULL}};
const Segment SGI236015[] = {{&S97975[83], 6}, {NULL}};
const Segment SGI236016[] = {{&S97975[110], 13}, {NULL}};
const Segment SGI236017[] = {{&S97976[44], 12}, {NULL}};
const Segment SGI236018[] = {{&S97976[44], 24}, {NULL}};
const Segment SGI236019[] = {{&S97976[30], 6}, {NULL}};
const Segment SGI236020[] = {{&S97976[117], 3}, {NULL}};
const Segment SGI236021[] = {{&S97976[82], 6}, {NULL}};
const Segment SGI236022[] = {{&S83332[44], 10}, {NULL}};
const Segment SGI236023[] = {{&S83332[44], 21}, {NULL}};
const Segment SGI236024[] = {{&S83332[30], 6}, {NULL}};
const Segment SGI236025[] = {{&S83332[132], 3}, {NULL}};
const Segment SGI236026[] = {{&S83332[97], 6}, {NULL}};
const Segment SGI236027[] = {{&S83332[67], 8}, {NULL}};
const Segment SGI236028[] = {{&S83333[44], 10}, {NULL}};
const Segment SGI236029[] = {{&S83333[44], 22}, {NULL}};
const Segment SGI236030[] = {{&S83333[30], 6}, {NULL}};
const Segment SGI236031[] = {{&S83333[115], 3}, {NULL}};
const Segment SGI236032[] = {{&S83333[80], 6}, {NULL}};
const Segment SGI236033[] = {{&S83333[140], 8}, {NULL}};
const Segment SGI236034[] = {{&S83335[34], 10}, {NULL}};
const Segment SGI236035[] = {{&S83335[27], 6}, {NULL}};
const Segment SGI236036[] = {{&S83335[96], 9}, {NULL}};
const Segment SGI236037[] = {{&S83335[69], 6}, {NULL}};
const Segment SGI236038[] = {{&S91342[34], 18}, {NULL}};
const Segment SGI236039[] = {{&S91342[27], 6}, {NULL}};
const Segment SGI236040[] = {{&S91342[105], 9}, {NULL}};
const Segment SGI236041[] = {{&S91342[78], 6}, {NULL}};
const Segment SGI236042[] = {{&S83342[34], 10}, {NULL}};
const Segment SGI236043[] = {{&S83342[27], 6}, {NULL}};
const Segment SGI236044[] = {{&S83342[97], 9}, {NULL}};
const Segment SGI236045[] = {{&S83342[70], 6}, {NULL}};
const Segment SGI236046[] = {{&S98007[34], 13}, {NULL}};
const Segment SGI236047[] = {{&S98007[27], 6}, {NULL}};
const Segment SGI236048[] = {{&S98007[100], 9}, {NULL}};
const Segment SGI236049[] = {{&S98007[73], 6}, {NULL}};
const Segment SGI236050[] = {{&S83351[34], 9}, {NULL}};
const Segment SGI236051[] = {{&S83351[27], 6}, {NULL}};
const Segment SGI236052[] = {{&S101218[44], 10}, {NULL}};
const Segment SGI236053[] = {{&S101218[44], 22}, {NULL}};
const Segment SGI236054[] = {{&S83351[96], 9}, {NULL}};
const Segment SGI236055[] = {{&S101218[30], 6}, {NULL}};
const Segment SGI236056[] = {{&S83351[69], 6}, {NULL}};
const Segment SGI236057[] = {{&S101218[31], 3}, {NULL}};
const Segment SGI236058[] = {{&S101218[109], 6}, {NULL}};
const Segment SGI236059[] = {{&S101218[68], 8}, {NULL}};
const Segment SGI236060[] = {{&S101218[80], 15}, {NULL}};
const Segment SGI236061[] = {{&S83354[34], 10}, {NULL}};
const Segment SGI236062[] = {{&S83354[27], 6}, {NULL}};
const Segment SGI236063[] = {{&S83354[96], 9}, {NULL}};
const Segment SGI236064[] = {{&S83354[69], 6}, {NULL}};
const Segment SGI236065[] = {{&S83360[34], 12}, {NULL}};
const Segment SGI236066[] = {{&S83360[27], 6}, {NULL}};
const Segment SGI236067[] = {{&S83360[99], 9}, {NULL}};
const Segment SGI236068[] = {{&S83360[72], 6}, {NULL}};
const Segment SGI236069[] = {{&S83361[34], 24}, {NULL}};
const Segment SGI236070[] = {{&S83361[27], 6}, {NULL}};
const Segment SGI236071[] = {{&S83361[111], 9}, {NULL}};
const Segment SGI236072[] = {{&S83361[84], 6}, {NULL}};
const Segment SGI236073[] = {{&S101236[44], 10}, {NULL}};
const Segment SGI236074[] = {{&S101236[44], 27}, {NULL}};
const Segment SGI236075[] = {{&S101236[30], 6}, {NULL}};
const Segment SGI236076[] = {{&S91378[44], 11}, {NULL}};
const Segment SGI236077[] = {{&S91378[44], 23}, {NULL}};
const Segment SGI236078[] = {{&S91378[30], 6}, {NULL}};
const Segment SGI236079[] = {{&S101236[116], 2}, {NULL}};
const Segment SGI236080[] = {{&S101236[85], 5}, {NULL}};
const Segment SGI236081[] = {{&S91378[116], 3}, {NULL}};
const Segment SGI236082[] = {{&S91378[81], 6}, {NULL}};
const Segment SGI236083[] = {{&S101237[44], 10}, {NULL}};
const Segment SGI236084[] = {{&S101237[44], 22}, {NULL}};
const Segment SGI236085[] = {{&S101237[30], 6}, {NULL}};
const Segment SGI236086[] = {{&S91379[44], 9}, {NULL}};
const Segment SGI236087[] = {{&S91379[44], 20}, {NULL}};
const Segment SGI236088[] = {{&S101237[111], 2}, {NULL}};
const Segment SGI236089[] = {{&S91379[30], 6}, {NULL}};
const Segment SGI236090[] = {{&S101237[80], 5}, {NULL}};
const Segment SGI236091[] = {{&S91379[129], 3}, {NULL}};
const Segment SGI236092[] = {{&S91379[94], 6}, {NULL}};
const Segment SGI236093[] = {{&S83371[44], 15}, {NULL}};
const Segment SGI236094[] = {{&S83371[44], 33}, {NULL}};
const Segment SGI236095[] = {{&S91379[66], 8}, {NULL}};
const Segment SGI236096[] = {{&S83371[30], 6}, {NULL}};
const Segment SGI236097[] = {{&S83371[91], 6}, {NULL}};
const Segment SGI236098[] = {{&S83371[118], 13}, {NULL}};
const Segment SGI236099[] = {{&S101238[44], 11}, {NULL}};
const Segment SGI236100[] = {{&S101238[44], 23}, {NULL}};
const Segment SGI236101[] = {{&S101238[30], 6}, {NULL}};
const Segment SGI236102[] = {{&S101238[112], 2}, {NULL}};
const Segment SGI236103[] = {{&S101238[81], 5}, {NULL}};
const Segment SGI236104[] = {{&S83372[44], 4}, {NULL}};
const Segment SGI236105[] = {{&S83372[44], 15}, {NULL}};
const Segment SGI236106[] = {{&S83372[30], 6}, {NULL}};
const Segment SGI236107[] = {{&S83372[73], 6}, {NULL}};
const Segment SGI236108[] = {{&S83372[100], 13}, {NULL}};
const Segment SGI236109[] = {{&S91382[44], 10}, {NULL}};
const Segment SGI236110[] = {{&S91382[44], 22}, {NULL}};
const Segment SGI236111[] = {{&S91382[30], 6}, {NULL}};
const Segment SGI236112[] = {{&S91382[113], 3}, {NULL}};
const Segment SGI236113[] = {{&S91382[80], 5}, {NULL}};
const Segment SGI236114[] = {{&S91382[117], 14}, {NULL}};
const Segment SGI236115[] = {{&S91383[34], 10}, {NULL}};
const Segment SGI236116[] = {{&S91383[27], 6}, {NULL}};
const Segment SGI236117[] = {{&S91383[97], 9}, {NULL}};
const Segment SGI236118[] = {{&S91383[70], 6}, {NULL}};
const Segment SGI236119[] = {{&S91389[34], 7}, {NULL}};
const Segment SGI236120[] = {{&S91389[27], 6}, {NULL}};
const Segment SGI236121[] = {{&S91389[94], 9}, {NULL}};
const Segment SGI236122[] = {{&S91389[67], 6}, {NULL}};
const Segment SGI236123[] = {{&S101249[44], 10}, {NULL}};
const Segment SGI236124[] = {{&S101249[44], 22}, {NULL}};
const Segment SGI236125[] = {{&S101249[30], 6}, {NULL}};
const Segment SGI236126[] = {{&S101249[31], 3}, {NULL}};
const Segment SGI236127[] = {{&S101249[80], 6}, {NULL}};
const Segment SGI236128[] = {{&S101249[143], 3}, {NULL}};
const Segment SGI236129[] = {{&S101254[44], 10}, {NULL}};
const Segment SGI236130[] = {{&S101254[44], 22}, {NULL}};
const Segment SGI236131[] = {{&S101254[30], 6}, {NULL}};
const Segment SGI236132[] = {{&S101254[31], 3}, {NULL}};
const Segment SGI236133[] = {{&S101254[80], 6}, {NULL}};
const Segment SGI236134[] = {{&S101254[132], 10}, {NULL}};
const Segment SGI236135[] = {{&S101260[44], 10}, {NULL}};
const Segment SGI236136[] = {{&S101260[44], 22}, {NULL}};
const Segment SGI236137[] = {{&S101260[30], 6}, {NULL}};
const Segment SGI236138[] = {{&S101260[31], 3}, {NULL}};
const Segment SGI236139[] = {{&S91407[34], 8}, {NULL}};
const Segment SGI236140[] = {{&S101260[80], 5}, {NULL}};
const Segment SGI236141[] = {{&S91407[27], 6}, {NULL}};
const Segment SGI236142[] = {{&S101260[117], 14}, {NULL}};
const Segment SGI236143[] = {{&S91407[95], 9}, {NULL}};
const Segment SGI236144[] = {{&S91407[68], 6}, {NULL}};
const Segment SGI236145[] = {{&S101265[44], 7}, {NULL}};
const Segment SGI236146[] = {{&S101265[44], 19}, {NULL}};
const Segment SGI236147[] = {{&S101265[30], 6}, {NULL}};
const Segment SGI236148[] = {{&S101265[31], 3}, {NULL}};
const Segment SGI236149[] = {{&S101265[92], 6}, {NULL}};
const Segment SGI236150[] = {{&S101265[65], 8}, {NULL}};
const Segment SGI236151[] = {{&S101268[44], 9}, {NULL}};
const Segment SGI236152[] = {{&S101268[44], 21}, {NULL}};
const Segment SGI236153[] = {{&S101268[30], 6}, {NULL}};
const Segment SGI236154[] = {{&S101268[31], 3}, {NULL}};
const Segment SGI236155[] = {{&S101268[94], 6}, {NULL}};
const Segment SGI236156[] = {{&S101268[67], 8}, {NULL}};
const Segment SGI236157[] = {{&S91417[52], 13}, {NULL}};
const Segment SGI236158[] = {{&S91417[52], 25}, {NULL}};
const Segment SGI236159[] = {{&S91417[38], 6}, {NULL}};
const Segment SGI236160[] = {{&S91417[91], 5}, {NULL}};
const Segment SGI236161[] = {{&S91417[11], 3}, {NULL}};
const Segment SGI236162[] = {{&S101273[44], 14}, {NULL}};
const Segment SGI236163[] = {{&S101273[44], 26}, {NULL}};
const Segment SGI236164[] = {{&S101273[30], 6}, {NULL}};
const Segment SGI236165[] = {{&S101273[31], 3}, {NULL}};
const Segment SGI236166[] = {{&S101273[113], 6}, {NULL}};
const Segment SGI236167[] = {{&S101273[72], 8}, {NULL}};
const Segment SGI236168[] = {{&S91424[44], 11}, {NULL}};
const Segment SGI236169[] = {{&S91424[44], 23}, {NULL}};
const Segment SGI236170[] = {{&S91424[30], 6}, {NULL}};
const Segment SGI236171[] = {{&S91424[116], 3}, {NULL}};
const Segment SGI236172[] = {{&S91424[81], 6}, {NULL}};
const Segment SGI236173[] = {{&S101276[45], 18}, {NULL}};
const Segment SGI236174[] = {{&S101276[30], 6}, {NULL}};
const Segment SGI236175[] = {{&S101276[31], 3}, {NULL}};
const Segment SGI236176[] = {{&S101276[89], 5}, {NULL}};
const Segment SGI236177[] = {{&S101276[147], 13}, {NULL}};
const Segment SGI236178[] = {{&S91426[52], 7}, {NULL}};
const Segment SGI236179[] = {{&S91426[52], 24}, {NULL}};
const Segment SGI236180[] = {{&S91426[38], 6}, {NULL}};
const Segment SGI236181[] = {{&S91426[90], 5}, {NULL}};
const Segment SGI236182[] = {{&S91426[11], 3}, {NULL}};
const Segment SGI236183[] = {{&S91427[52], 7}, {NULL}};
const Segment SGI236184[] = {{&S91427[52], 24}, {NULL}};
const Segment SGI236185[] = {{&S91427[38], 6}, {NULL}};
const Segment SGI236186[] = {{&S91427[90], 5}, {NULL}};
const Segment SGI236187[] = {{&S91427[11], 3}, {NULL}};
const Segment SGI236188[] = {{&S91434[52], 10}, {NULL}};
const Segment SGI236189[] = {{&S91434[52], 22}, {NULL}};
const Segment SGI236190[] = {{&S91434[38], 6}, {NULL}};
const Segment SGI236191[] = {{&S91434[88], 5}, {NULL}};
const Segment SGI236192[] = {{&S91434[11], 3}, {NULL}};
const Segment SGI236193[] = {{&S91438[44], 10}, {NULL}};
const Segment SGI236194[] = {{&S91438[44], 22}, {NULL}};
const Segment SGI236195[] = {{&S91438[30], 6}, {NULL}};
const Segment SGI236196[] = {{&S91438[80], 6}, {NULL}};
const Segment SGI236197[] = {{&S83443[34], 10}, {NULL}};
const Segment SGI236198[] = {{&S83443[27], 6}, {NULL}};
const Segment SGI236199[] = {{&S91438[107], 13}, {NULL}};
const Segment SGI236200[] = {{&S83443[96], 9}, {NULL}};
const Segment SGI236201[] = {{&S83443[69], 6}, {NULL}};
const Segment SGI236202[] = {{&S91439[34], 9}, {NULL}};
const Segment SGI236203[] = {{&S91439[27], 6}, {NULL}};
const Segment SGI236204[] = {{&S83444[34], 11}, {NULL}};
const Segment SGI236205[] = {{&S91439[96], 9}, {NULL}};
const Segment SGI236206[] = {{&S83444[27], 6}, {NULL}};
const Segment SGI236207[] = {{&S91439[69], 6}, {NULL}};
const Segment SGI236208[] = {{&S91439[130], 8}, {NULL}};
const Segment SGI236209[] = {{&S83444[97], 9}, {NULL}};
const Segment SGI236210[] = {{&S83444[70], 6}, {NULL}};
const Segment SGI236211[] = {{&S83448[44], 10}, {NULL}};
const Segment SGI236212[] = {{&S83448[44], 21}, {NULL}};
const Segment SGI236213[] = {{&S83448[30], 6}, {NULL}};
const Segment SGI236214[] = {{&S83448[132], 3}, {NULL}};
const Segment SGI236215[] = {{&S83448[97], 6}, {NULL}};
const Segment SGI236216[] = {{&S83448[67], 8}, {NULL}};
const Segment SGI236217[] = {{&S83449[44], 10}, {NULL}};
const Segment SGI236218[] = {{&S83449[44], 22}, {NULL}};
const Segment SGI236219[] = {{&S83449[30], 6}, {NULL}};
const Segment SGI236220[] = {{&S83449[133], 3}, {NULL}};
const Segment SGI236221[] = {{&S83449[98], 6}, {NULL}};
const Segment SGI236222[] = {{&S91447[34], 10}, {NULL}};
const Segment SGI236223[] = {{&S83449[68], 8}, {NULL}};
const Segment SGI236224[] = {{&S91447[27], 6}, {NULL}};
const Segment SGI236225[] = {{&S91447[97], 9}, {NULL}};
const Segment SGI236226[] = {{&S91447[70], 6}, {NULL}};
const Segment SGI236227[] = {{&S91447[131], 8}, {NULL}};
const Segment SGI236228[] = {{&S91448[34], 10}, {NULL}};
const Segment SGI236229[] = {{&S91448[27], 6}, {NULL}};
const Segment SGI236230[] = {{&S91448[97], 9}, {NULL}};
const Segment SGI236231[] = {{&S91448[70], 6}, {NULL}};
const Segment SGI236232[] = {{&S83454[34], 9}, {NULL}};
const Segment SGI236233[] = {{&S83454[27], 6}, {NULL}};
const Segment SGI236234[] = {{&S83454[96], 9}, {NULL}};
const Segment SGI236235[] = {{&S83454[69], 6}, {NULL}};
const Segment SGI236236[] = {{&S83469[44], 8}, {NULL}};
const Segment SGI236237[] = {{&S83469[44], 19}, {NULL}};
const Segment SGI236238[] = {{&S83469[30], 6}, {NULL}};
const Segment SGI236239[] = {{&S83469[112], 3}, {NULL}};
const Segment SGI236240[] = {{&S83469[77], 6}, {NULL}};
const Segment SGI236241[] = {{&S91980[34], 7}, {NULL}};
const Segment SGI236242[] = {{&S91980[27], 6}, {NULL}};
const Segment SGI236243[] = {{&S91980[94], 9}, {NULL}};
const Segment SGI236244[] = {{&S91980[67], 6}, {NULL}};
const Segment SGI236245[] = {{&S91987[34], 6}, {NULL}};
const Segment SGI236246[] = {{&S91987[27], 6}, {NULL}};
const Segment SGI236247[] = {{&S91987[95], 9}, {NULL}};
const Segment SGI236248[] = {{&S91987[68], 6}, {NULL}};
const Segment SGI236249[] = {{&S98021[34], 22}, {NULL}};
const Segment SGI236250[] = {{&S98021[27], 6}, {NULL}};
const Segment SGI236251[] = {{&S98021[109], 9}, {NULL}};
const Segment SGI236252[] = {{&S98021[82], 6}, {NULL}};
const Segment SGI236253[] = {{&S98023[34], 15}, {NULL}};
const Segment SGI236254[] = {{&S98023[27], 6}, {NULL}};
const Segment SGI236255[] = {{&S98023[102], 9}, {NULL}};
const Segment SGI236256[] = {{&S98023[75], 6}, {NULL}};
const Segment SGI236257[] = {{&S98023[143], 8}, {NULL}};
const Segment SGI236258[] = {{&S101358[44], 18}, {NULL}};
const Segment SGI236259[] = {{&S101358[44], 30}, {NULL}};
const Segment SGI236260[] = {{&S101358[30], 6}, {NULL}};
const Segment SGI236261[] = {{&S101358[31], 3}, {NULL}};
const Segment SGI236262[] = {{&S101358[117], 6}, {NULL}};
const Segment SGI236263[] = {{&S101358[76], 8}, {NULL}};
const Segment SGI236264[] = {{&S101358[88], 15}, {NULL}};
const Segment SGI236265[] = {{&S101365[44], 8}, {NULL}};
const Segment SGI236266[] = {{&S101365[44], 20}, {NULL}};
const Segment SGI236267[] = {{&S101365[30], 6}, {NULL}};
const Segment SGI236268[] = {{&S101365[31], 3}, {NULL}};
const Segment SGI236269[] = {{&S101365[78], 6}, {NULL}};
const Segment SGI236270[] = {{&S101374[59], 18}, {NULL}};
const Segment SGI236271[] = {{&S101374[59], 30}, {NULL}};
const Segment SGI236272[] = {{&S101374[45], 6}, {NULL}};
const Segment SGI236273[] = {{&S101374[46], 3}, {NULL}};
const Segment SGI236274[] = {{&S101374[103], 6}, {NULL}};
const Segment SGI236275[] = {{&S101374[0], 14}, {NULL}};
const Segment SGI236276[] = {{&S92027[52], 16}, {NULL}};
const Segment SGI236277[] = {{&S92027[52], 28}, {NULL}};
const Segment SGI236278[] = {{&S92027[38], 6}, {NULL}};
const Segment SGI236279[] = {{&S92027[94], 5}, {NULL}};
const Segment SGI236280[] = {{&S101391[59], 11}, {NULL}};
const Segment SGI236281[] = {{&S101391[59], 23}, {NULL}};
const Segment SGI236282[] = {{&S101391[45], 6}, {NULL}};
const Segment SGI236283[] = {{&S101391[46], 3}, {NULL}};
const Segment SGI236284[] = {{&S101391[96], 6}, {NULL}};
const Segment SGI236285[] = {{&S101391[0], 14}, {NULL}};
const Segment SGI236286[] = {{&S92038[44], 14}, {NULL}};
const Segment SGI236287[] = {{&S92038[44], 26}, {NULL}};
const Segment SGI236288[] = {{&S92038[30], 6}, {NULL}};
const Segment SGI236289[] = {{&S101392[59], 10}, {NULL}};
const Segment SGI236290[] = {{&S92038[84], 6}, {NULL}};
const Segment SGI236291[] = {{&S101392[59], 22}, {NULL}};
const Segment SGI236292[] = {{&S101392[45], 6}, {NULL}};
const Segment SGI236293[] = {{&S92038[121], 3}, {NULL}};
const Segment SGI236294[] = {{&S101392[46], 3}, {NULL}};
const Segment SGI236295[] = {{&S101392[95], 6}, {NULL}};
const Segment SGI236296[] = {{&S101392[0], 14}, {NULL}};
const Segment SGI236297[] = {{&S101393[44], 11}, {NULL}};
const Segment SGI236298[] = {{&S101393[44], 23}, {NULL}};
const Segment SGI236299[] = {{&S101393[30], 6}, {NULL}};
const Segment SGI236300[] = {{&S101393[31], 3}, {NULL}};
const Segment SGI236301[] = {{&S101393[81], 6}, {NULL}};
const Segment SGI236302[] = {{&S101393[133], 10}, {NULL}};
const Segment SGI236303[] = {{&S91545[34], 8}, {NULL}};
const Segment SGI236304[] = {{&S91545[27], 6}, {NULL}};
const Segment SGI236305[] = {{&S101394[44], 10}, {NULL}};
const Segment SGI236306[] = {{&S101394[44], 22}, {NULL}};
const Segment SGI236307[] = {{&S101394[30], 6}, {NULL}};
const Segment SGI236308[] = {{&S91545[95], 9}, {NULL}};
const Segment SGI236309[] = {{&S91545[68], 6}, {NULL}};
const Segment SGI236310[] = {{&S101394[31], 3}, {NULL}};
const Segment SGI236311[] = {{&S101394[80], 6}, {NULL}};
const Segment SGI236312[] = {{&S101394[132], 10}, {NULL}};
const Segment SGI236313[] = {{&S101395[44], 11}, {NULL}};
const Segment SGI236314[] = {{&S101395[44], 23}, {NULL}};
const Segment SGI236315[] = {{&S101395[30], 6}, {NULL}};
const Segment SGI236316[] = {{&S91547[34], 7}, {NULL}};
const Segment SGI236317[] = {{&S91547[27], 6}, {NULL}};
const Segment SGI236318[] = {{&S101395[31], 3}, {NULL}};
const Segment SGI236319[] = {{&S101395[81], 6}, {NULL}};
const Segment SGI236320[] = {{&S101395[133], 10}, {NULL}};
const Segment SGI236321[] = {{&S91547[94], 9}, {NULL}};
const Segment SGI236322[] = {{&S91547[67], 6}, {NULL}};
const Segment SGI236323[] = {{&S101396[44], 10}, {NULL}};
const Segment SGI236324[] = {{&S101396[44], 22}, {NULL}};
const Segment SGI236325[] = {{&S101396[30], 6}, {NULL}};
const Segment SGI236326[] = {{&S101396[111], 2}, {NULL}};
const Segment SGI236327[] = {{&S101396[80], 5}, {NULL}};
const Segment SGI236328[] = {{&S92052[34], 6}, {NULL}};
const Segment SGI236329[] = {{&S92052[27], 6}, {NULL}};
const Segment SGI236330[] = {{&S92052[95], 9}, {NULL}};
const Segment SGI236331[] = {{&S92052[68], 6}, {NULL}};
const Segment SGI236332[] = {{&S92052[136], 8}, {NULL}};
const Segment SGI236333[] = {{&S101403[45], 10}, {NULL}};
const Segment SGI236334[] = {{&S101403[30], 6}, {NULL}};
const Segment SGI236335[] = {{&S101403[31], 3}, {NULL}};
const Segment SGI236336[] = {{&S101403[81], 5}, {NULL}};
const Segment SGI236337[] = {{&S101403[139], 13}, {NULL}};
const Segment SGI236338[] = {{&S101405[45], 10}, {NULL}};
const Segment SGI236339[] = {{&S101405[30], 6}, {NULL}};
const Segment SGI236340[] = {{&S101405[31], 3}, {NULL}};
const Segment SGI236341[] = {{&S101405[81], 5}, {NULL}};
const Segment SGI236342[] = {{&S101405[139], 13}, {NULL}};
const Segment SGI236343[] = {{&S91562[59], 10}, {NULL}};
const Segment SGI236344[] = {{&S91562[59], 22}, {NULL}};
const Segment SGI236345[] = {{&S91562[45], 6}, {NULL}};
const Segment SGI236346[] = {{&S91562[130], 3}, {NULL}};
const Segment SGI236347[] = {{&S91562[95], 6}, {NULL}};
const Segment SGI236348[] = {{&S91562[0], 14}, {NULL}};
const Segment SGI236349[] = {{&S91564[34], 7}, {NULL}};
const Segment SGI236350[] = {{&S91564[27], 6}, {NULL}};
const Segment SGI236351[] = {{&S91564[94], 9}, {NULL}};
const Segment SGI236352[] = {{&S91564[67], 6}, {NULL}};
const Segment SGI236353[] = {{&S91565[34], 8}, {NULL}};
const Segment SGI236354[] = {{&S91565[27], 6}, {NULL}};
const Segment SGI236355[] = {{&S91565[95], 9}, {NULL}};
const Segment SGI236356[] = {{&S91565[68], 6}, {NULL}};
const Segment SGI236357[] = {{&S91571[34], 6}, {NULL}};
const Segment SGI236358[] = {{&S91571[27], 6}, {NULL}};
const Segment SGI236359[] = {{&S91571[97], 9}, {NULL}};
const Segment SGI236360[] = {{&S91571[70], 6}, {NULL}};
const Segment SGI236361[] = {{&S98125[44], 10}, {NULL}};
const Segment SGI236362[] = {{&S98125[44], 22}, {NULL}};
const Segment SGI236363[] = {{&S98125[30], 6}, {NULL}};
const Segment SGI236364[] = {{&S98125[144], 3}, {NULL}};
const Segment SGI236365[] = {{&S98125[109], 6}, {NULL}};
const Segment SGI236366[] = {{&S98125[68], 8}, {NULL}};
const Segment SGI236367[] = {{&S98125[80], 15}, {NULL}};
const Segment SGI236368[] = {{&S98126[44], 18}, {NULL}};
const Segment SGI236369[] = {{&S98126[44], 30}, {NULL}};
const Segment SGI236370[] = {{&S98126[30], 6}, {NULL}};
const Segment SGI236371[] = {{&S98126[152], 3}, {NULL}};
const Segment SGI236372[] = {{&S98126[117], 6}, {NULL}};
const Segment SGI236373[] = {{&S98126[76], 8}, {NULL}};
const Segment SGI236374[] = {{&S98126[88], 15}, {NULL}};
const Segment SGI236375[] = {{&S91586[44], 10}, {NULL}};
const Segment SGI236376[] = {{&S91586[44], 22}, {NULL}};
const Segment SGI236377[] = {{&S91586[30], 6}, {NULL}};
const Segment SGI236378[] = {{&S91586[113], 3}, {NULL}};
const Segment SGI236379[] = {{&S91586[80], 5}, {NULL}};
const Segment SGI236380[] = {{&S91586[117], 14}, {NULL}};
const Segment SGI236381[] = {{&S91591[44], 20}, {NULL}};
const Segment SGI236382[] = {{&S91591[44], 32}, {NULL}};
const Segment SGI236383[] = {{&S91591[30], 6}, {NULL}};
const Segment SGI236384[] = {{&S91591[125], 3}, {NULL}};
const Segment SGI236385[] = {{&S91591[90], 6}, {NULL}};
const Segment SGI236386[] = {{&S91603[34], 11}, {NULL}};
const Segment SGI236387[] = {{&S91603[27], 6}, {NULL}};
const Segment SGI236388[] = {{&S91603[98], 9}, {NULL}};
const Segment SGI236389[] = {{&S91603[71], 6}, {NULL}};
const Segment SGI236390[] = {{&S98151[45], 10}, {NULL}};
const Segment SGI236391[] = {{&S98151[30], 6}, {NULL}};
const Segment SGI236392[] = {{&S98151[115], 3}, {NULL}};
const Segment SGI236393[] = {{&S98151[81], 5}, {NULL}};
const Segment SGI236394[] = {{&S98151[139], 13}, {NULL}};
const Segment SGI236395[] = {{&S91613[44], 12}, {NULL}};
const Segment SGI236396[] = {{&S91613[44], 34}, {NULL}};
const Segment SGI236397[] = {{&S91613[30], 6}, {NULL}};
const Segment SGI236398[] = {{&S91613[127], 3}, {NULL}};
const Segment SGI236399[] = {{&S91613[92], 6}, {NULL}};
const Segment SGI236400[] = {{&S91613[162], 3}, {NULL}};
const Segment SGI236401[] = {{&S91618[34], 9}, {NULL}};
const Segment SGI236402[] = {{&S91618[27], 6}, {NULL}};
const Segment SGI236403[] = {{&S91618[96], 9}, {NULL}};
const Segment SGI236404[] = {{&S91618[69], 6}, {NULL}};
const Segment SGI236405[] = {{&S91618[137], 8}, {NULL}};
const Segment SGI236406[] = {{&S98164[44], 14}, {NULL}};
const Segment SGI236407[] = {{&S98164[44], 26}, {NULL}};
const Segment SGI236408[] = {{&S98164[30], 6}, {NULL}};
const Segment SGI236409[] = {{&S98164[119], 3}, {NULL}};
const Segment SGI236410[] = {{&S98164[84], 6}, {NULL}};
const Segment SGI236411[] = {{&S98164[144], 11}, {NULL}};
const Segment SGI236412[] = {{&S91622[44], 9}, {NULL}};
const Segment SGI236413[] = {{&S91622[44], 21}, {NULL}};
const Segment SGI236414[] = {{&S91622[30], 6}, {NULL}};
const Segment SGI236415[] = {{&S91622[114], 3}, {NULL}};
const Segment SGI236416[] = {{&S91622[79], 6}, {NULL}};
const Segment SGI236417[] = {{&S91631[34], 6}, {NULL}};
const Segment SGI236418[] = {{&S91631[27], 6}, {NULL}};
const Segment SGI236419[] = {{&S91631[93], 9}, {NULL}};
const Segment SGI236420[] = {{&S91631[66], 6}, {NULL}};
const Segment SGI236421[] = {{&S91633[34], 19}, {NULL}};
const Segment SGI236422[] = {{&S91633[27], 6}, {NULL}};
const Segment SGI236423[] = {{&S91633[106], 9}, {NULL}};
const Segment SGI236424[] = {{&S91633[79], 6}, {NULL}};
const Segment SGI236425[] = {{&S91635[34], 10}, {NULL}};
const Segment SGI236426[] = {{&S91635[27], 6}, {NULL}};
const Segment SGI236427[] = {{&S91635[97], 9}, {NULL}};
const Segment SGI236428[] = {{&S91635[70], 6}, {NULL}};
const Segment SGI236429[] = {{&S98182[44], 10}, {NULL}};
const Segment SGI236430[] = {{&S98182[44], 22}, {NULL}};
const Segment SGI236431[] = {{&S98182[30], 6}, {NULL}};
const Segment SGI236432[] = {{&S98182[80], 6}, {NULL}};
const Segment SGI236433[] = {{&S98182[107], 13}, {NULL}};
const Segment SGI236434[] = {{&S91646[44], 13}, {NULL}};
const Segment SGI236435[] = {{&S91646[44], 25}, {NULL}};
const Segment SGI236436[] = {{&S91646[30], 6}, {NULL}};
const Segment SGI236437[] = {{&S91646[83], 6}, {NULL}};
const Segment SGI236438[] = {{&S91646[110], 13}, {NULL}};
const Segment SGI236439[] = {{&S83541[44], 10}, {NULL}};
const Segment SGI236440[] = {{&S83541[44], 21}, {NULL}};
const Segment SGI236441[] = {{&S83541[30], 6}, {NULL}};
const Segment SGI236442[] = {{&S83541[114], 3}, {NULL}};
const Segment SGI236443[] = {{&S83541[79], 6}, {NULL}};
const Segment SGI236444[] = {{&S91647[44], 10}, {NULL}};
const Segment SGI236445[] = {{&S91647[44], 22}, {NULL}};
const Segment SGI236446[] = {{&S91647[30], 6}, {NULL}};
const Segment SGI236447[] = {{&S91647[115], 3}, {NULL}};
const Segment SGI236448[] = {{&S91647[80], 6}, {NULL}};
const Segment SGI236449[] = {{&S91647[135], 8}, {NULL}};
const Segment SGI236450[] = {{&S91647[144], 13}, {NULL}};
const Segment SGI236451[] = {{&S91648[44], 13}, {NULL}};
const Segment SGI236452[] = {{&S91648[44], 29}, {NULL}};
const Segment SGI236453[] = {{&S91648[30], 6}, {NULL}};
const Segment SGI236454[] = {{&S91648[87], 6}, {NULL}};
const Segment SGI236455[] = {{&S91648[114], 13}, {NULL}};
const Segment SGI236456[] = {{&S83545[44], 7}, {NULL}};
const Segment SGI236457[] = {{&S83545[44], 19}, {NULL}};
const Segment SGI236458[] = {{&S83545[30], 6}, {NULL}};
const Segment SGI236459[] = {{&S83545[132], 3}, {NULL}};
const Segment SGI236460[] = {{&S83545[97], 6}, {NULL}};
const Segment SGI236461[] = {{&S83545[65], 8}, {NULL}};
const Segment SGI236462[] = {{&S91655[44], 6}, {NULL}};
const Segment SGI236463[] = {{&S91655[44], 18}, {NULL}};
const Segment SGI236464[] = {{&S91655[30], 6}, {NULL}};
const Segment SGI236465[] = {{&S91655[76], 6}, {NULL}};
const Segment SGI236466[] = {{&S91655[103], 13}, {NULL}};
const Segment SGI236467[] = {{&S101532[44], 10}, {NULL}};
const Segment SGI236468[] = {{&S101532[44], 22}, {NULL}};
const Segment SGI236469[] = {{&S83552[34], 10}, {NULL}};
const Segment SGI236470[] = {{&S101532[30], 6}, {NULL}};
const Segment SGI236471[] = {{&S83552[27], 6}, {NULL}};
const Segment SGI236472[] = {{&S101532[111], 2}, {NULL}};
const Segment SGI236473[] = {{&S83552[97], 9}, {NULL}};
const Segment SGI236474[] = {{&S101532[80], 5}, {NULL}};
const Segment SGI236475[] = {{&S83552[70], 6}, {NULL}};
const Segment SGI236476[] = {{&S98206[44], 7}, {NULL}};
const Segment SGI236477[] = {{&S98206[44], 35}, {NULL}};
const Segment SGI236478[] = {{&S98206[30], 6}, {NULL}};
const Segment SGI236479[] = {{&S98206[128], 3}, {NULL}};
const Segment SGI236480[] = {{&S98206[93], 6}, {NULL}};
const Segment SGI236481[] = {{&S98206[153], 11}, {NULL}};
const Segment SGI236482[] = {{&S91659[34], 9}, {NULL}};
const Segment SGI236483[] = {{&S91659[27], 6}, {NULL}};
const Segment SGI236484[] = {{&S91659[96], 9}, {NULL}};
const Segment SGI236485[] = {{&S91659[69], 6}, {NULL}};
const Segment SGI236486[] = {{&S91660[34], 10}, {NULL}};
const Segment SGI236487[] = {{&S91660[27], 6}, {NULL}};
const Segment SGI236488[] = {{&S91660[97], 9}, {NULL}};
const Segment SGI236489[] = {{&S91660[70], 6}, {NULL}};
const Segment SGI236490[] = {{&S91660[138], 8}, {NULL}};
const Segment SGI236491[] = {{&S98220[34], 13}, {NULL}};
const Segment SGI236492[] = {{&S98220[27], 6}, {NULL}};
const Segment SGI236493[] = {{&S98220[100], 9}, {NULL}};
const Segment SGI236494[] = {{&S98220[73], 6}, {NULL}};
const Segment SGI236495[] = {{&S101560[45], 10}, {NULL}};
const Segment SGI236496[] = {{&S101560[30], 6}, {NULL}};
const Segment SGI236497[] = {{&S101560[31], 3}, {NULL}};
const Segment SGI236498[] = {{&S101560[81], 5}, {NULL}};
const Segment SGI236499[] = {{&S101560[139], 13}, {NULL}};
const Segment SGI236500[] = {{&S101563[44], 10}, {NULL}};
const Segment SGI236501[] = {{&S101563[44], 22}, {NULL}};
const Segment SGI236502[] = {{&S101563[30], 6}, {NULL}};
const Segment SGI236503[] = {{&S101563[31], 3}, {NULL}};
const Segment SGI236504[] = {{&S101563[95], 6}, {NULL}};
const Segment SGI236505[] = {{&S101563[158], 3}, {NULL}};
const Segment SGI236506[] = {{&S101563[68], 8}, {NULL}};
const Segment SGI236507[] = {{&S101564[44], 11}, {NULL}};
const Segment SGI236508[] = {{&S101564[44], 23}, {NULL}};
const Segment SGI236509[] = {{&S101564[30], 6}, {NULL}};
const Segment SGI236510[] = {{&S101564[31], 3}, {NULL}};
const Segment SGI236511[] = {{&S101564[96], 6}, {NULL}};
const Segment SGI236512[] = {{&S101564[159], 3}, {NULL}};
const Segment SGI236513[] = {{&S101564[69], 8}, {NULL}};
const Segment SGI236514[] = {{&S101565[44], 10}, {NULL}};
const Segment SGI236515[] = {{&S101565[44], 22}, {NULL}};
const Segment SGI236516[] = {{&S101565[30], 6}, {NULL}};
const Segment SGI236517[] = {{&S101565[31], 3}, {NULL}};
const Segment SGI236518[] = {{&S101565[99], 6}, {NULL}};
const Segment SGI236519[] = {{&S101565[162], 3}, {NULL}};
const Segment SGI236520[] = {{&S101565[68], 8}, {NULL}};
const Segment SGI236521[] = {{&S101575[59], 10}, {NULL}};
const Segment SGI236522[] = {{&S101575[59], 22}, {NULL}};
const Segment SGI236523[] = {{&S101575[45], 6}, {NULL}};
const Segment SGI236524[] = {{&S101575[46], 3}, {NULL}};
const Segment SGI236525[] = {{&S101575[95], 6}, {NULL}};
const Segment SGI236526[] = {{&S101575[0], 14}, {NULL}};
const Segment SGI236527[] = {{&S101578[45], 10}, {NULL}};
const Segment SGI236528[] = {{&S101578[30], 6}, {NULL}};
const Segment SGI236529[] = {{&S101578[31], 3}, {NULL}};
const Segment SGI236530[] = {{&S101578[81], 5}, {NULL}};
const Segment SGI236531[] = {{&S101578[139], 13}, {NULL}};
const Segment SGI236532[] = {{&S101582[44], 10}, {NULL}};
const Segment SGI236533[] = {{&S101582[44], 22}, {NULL}};
const Segment SGI236534[] = {{&S101582[30], 6}, {NULL}};
const Segment SGI236535[] = {{&S101582[31], 3}, {NULL}};
const Segment SGI236536[] = {{&S101582[99], 6}, {NULL}};
const Segment SGI236537[] = {{&S101582[68], 8}, {NULL}};
const Segment SGI236538[] = {{&S101583[44], 10}, {NULL}};
const Segment SGI236539[] = {{&S101583[44], 22}, {NULL}};
const Segment SGI236540[] = {{&S101583[30], 6}, {NULL}};
const Segment SGI236541[] = {{&S101583[31], 3}, {NULL}};
const Segment SGI236542[] = {{&S101583[109], 6}, {NULL}};
const Segment SGI236543[] = {{&S101583[68], 8}, {NULL}};
const Segment SGI236544[] = {{&S101583[80], 15}, {NULL}};
const Segment SGI236545[] = {{&S101587[44], 10}, {NULL}};
const Segment SGI236546[] = {{&S101587[44], 22}, {NULL}};
const Segment SGI236547[] = {{&S101587[30], 6}, {NULL}};
const Segment SGI236548[] = {{&S101587[31], 3}, {NULL}};
const Segment SGI236549[] = {{&S101587[99], 6}, {NULL}};
const Segment SGI236550[] = {{&S101587[68], 8}, {NULL}};
const Segment SGI236551[] = {{&S98284[34], 7}, {NULL}};
const Segment SGI236552[] = {{&S98284[27], 6}, {NULL}};
const Segment SGI236553[] = {{&S98284[94], 9}, {NULL}};
const Segment SGI236554[] = {{&S98284[67], 6}, {NULL}};
const Segment SGI236555[] = {{&S98288[34], 9}, {NULL}};
const Segment SGI236556[] = {{&S98288[27], 6}, {NULL}};
const Segment SGI236557[] = {{&S98288[96], 9}, {NULL}};
const Segment SGI236558[] = {{&S98288[69], 6}, {NULL}};
const Segment SGI236559[] = {{&S98288[137], 8}, {NULL}};
const Segment SGI236560[] = {{&S91750[34], 10}, {NULL}};
const Segment SGI236561[] = {{&S91750[27], 6}, {NULL}};
const Segment SGI236562[] = {{&S91750[97], 9}, {NULL}};
const Segment SGI236563[] = {{&S91750[70], 6}, {NULL}};
const Segment SGI236564[] = {{&S98302[34], 7}, {NULL}};
const Segment SGI236565[] = {{&S98302[27], 6}, {NULL}};
const Segment SGI236566[] = {{&S98302[94], 9}, {NULL}};
const Segment SGI236567[] = {{&S98302[67], 6}, {NULL}};
const Segment SGI236568[] = {{&S98306[34], 11}, {NULL}};
const Segment SGI236569[] = {{&S98306[27], 6}, {NULL}};
const Segment SGI236570[] = {{&S98306[98], 9}, {NULL}};
const Segment SGI236571[] = {{&S98306[71], 6}, {NULL}};
const Segment SGI236572[] = {{&S91766[34], 7}, {NULL}};
const Segment SGI236573[] = {{&S91766[27], 6}, {NULL}};
const Segment SGI236574[] = {{&S91766[94], 9}, {NULL}};
const Segment SGI236575[] = {{&S91766[67], 6}, {NULL}};
const Segment SGI236576[] = {{&S98311[34], 12}, {NULL}};
const Segment SGI236577[] = {{&S98311[27], 6}, {NULL}};
const Segment SGI236578[] = {{&S98311[99], 9}, {NULL}};
const Segment SGI236579[] = {{&S98311[72], 6}, {NULL}};
const Segment SGI236580[] = {{&S98312[34], 25}, {NULL}};
const Segment SGI236581[] = {{&S98312[27], 6}, {NULL}};
const Segment SGI236582[] = {{&S98312[112], 9}, {NULL}};
const Segment SGI236583[] = {{&S98312[85], 6}, {NULL}};
const Segment SGI236584[] = {{&S91775[59], 10}, {NULL}};
const Segment SGI236585[] = {{&S91775[59], 22}, {NULL}};
const Segment SGI236586[] = {{&S91775[45], 6}, {NULL}};
const Segment SGI236587[] = {{&S91775[130], 3}, {NULL}};
const Segment SGI236588[] = {{&S91775[95], 6}, {NULL}};
const Segment SGI236589[] = {{&S91775[11], 3}, {NULL}};
const Segment SGI236590[] = {{&S91782[34], 10}, {NULL}};
const Segment SGI236591[] = {{&S91782[27], 6}, {NULL}};
const Segment SGI236592[] = {{&S91782[97], 9}, {NULL}};
const Segment SGI236593[] = {{&S91782[70], 6}, {NULL}};
const Segment SGI236594[] = {{&S83641[34], 7}, {NULL}};
const Segment SGI236595[] = {{&S83641[27], 6}, {NULL}};
const Segment SGI236596[] = {{&S92218[34], 10}, {NULL}};
const Segment SGI236597[] = {{&S92218[27], 6}, {NULL}};
const Segment SGI236598[] = {{&S83641[94], 9}, {NULL}};
const Segment SGI236599[] = {{&S83641[67], 6}, {NULL}};
const Segment SGI236600[] = {{&S92218[97], 9}, {NULL}};
const Segment SGI236601[] = {{&S92218[70], 6}, {NULL}};
const Segment SGI236602[] = {{&S98342[44], 35}, {NULL}};
const Segment SGI236603[] = {{&S98342[44], 47}, {NULL}};
const Segment SGI236604[] = {{&S98342[30], 6}, {NULL}};
const Segment SGI236605[] = {{&S98342[140], 3}, {NULL}};
const Segment SGI236606[] = {{&S98342[105], 6}, {NULL}};
const Segment SGI236607[] = {{&S92231[34], 11}, {NULL}};
const Segment SGI236608[] = {{&S92231[27], 6}, {NULL}};
const Segment SGI236609[] = {{&S92231[98], 9}, {NULL}};
const Segment SGI236610[] = {{&S92231[71], 6}, {NULL}};
const Segment SGI236611[] = {{&S91800[34], 9}, {NULL}};
const Segment SGI236612[] = {{&S91800[27], 6}, {NULL}};
const Segment SGI236613[] = {{&S91800[101], 9}, {NULL}};
const Segment SGI236614[] = {{&S91800[74], 6}, {NULL}};
const Segment SGI236615[] = {{&S91801[44], 7}, {NULL}};
const Segment SGI236616[] = {{&S91801[44], 24}, {NULL}};
const Segment SGI236617[] = {{&S91801[30], 6}, {NULL}};
const Segment SGI236618[] = {{&S91801[82], 6}, {NULL}};
const Segment SGI236619[] = {{&S91801[109], 13}, {NULL}};
const Segment SGI236620[] = {{&S91802[44], 7}, {NULL}};
const Segment SGI236621[] = {{&S91802[44], 24}, {NULL}};
const Segment SGI236622[] = {{&S91802[30], 6}, {NULL}};
const Segment SGI236623[] = {{&S91802[82], 6}, {NULL}};
const Segment SGI236624[] = {{&S91802[109], 13}, {NULL}};
const Segment SGI236625[] = {{&S98349[44], 18}, {NULL}};
const Segment SGI236626[] = {{&S98349[44], 30}, {NULL}};
const Segment SGI236627[] = {{&S98349[30], 6}, {NULL}};
const Segment SGI236628[] = {{&S98349[123], 3}, {NULL}};
const Segment SGI236629[] = {{&S98349[88], 6}, {NULL}};
const Segment SGI236630[] = {{&S91803[34], 14}, {NULL}};
const Segment SGI236631[] = {{&S91803[27], 6}, {NULL}};
const Segment SGI236632[] = {{&S91803[101], 9}, {NULL}};
const Segment SGI236633[] = {{&S91803[74], 6}, {NULL}};
const Segment SGI236634[] = {{&S91804[34], 11}, {NULL}};
const Segment SGI236635[] = {{&S91804[27], 6}, {NULL}};
const Segment SGI236636[] = {{&S91804[98], 9}, {NULL}};
const Segment SGI236637[] = {{&S91804[71], 6}, {NULL}};
const Segment SGI236638[] = {{&S91809[34], 19}, {NULL}};
const Segment SGI236639[] = {{&S91809[27], 6}, {NULL}};
const Segment SGI236640[] = {{&S91809[106], 9}, {NULL}};
const Segment SGI236641[] = {{&S91809[79], 6}, {NULL}};
const Segment SGI236642[] = {{&S91811[45], 13}, {NULL}};
const Segment SGI236643[] = {{&S91811[30], 6}, {NULL}};
const Segment SGI236644[] = {{&S91811[118], 3}, {NULL}};
const Segment SGI236645[] = {{&S91811[84], 5}, {NULL}};
const Segment SGI236646[] = {{&S91811[142], 13}, {NULL}};
const Segment SGI236647[] = {{&S98370[34], 15}, {NULL}};
const Segment SGI236648[] = {{&S98370[27], 6}, {NULL}};
const Segment SGI236649[] = {{&S98370[102], 9}, {NULL}};
const Segment SGI236650[] = {{&S98370[75], 6}, {NULL}};
const Segment SGI236651[] = {{&S98370[136], 8}, {NULL}};
const Segment SGI236652[] = {{&S101706[44], 10}, {NULL}};
const Segment SGI236653[] = {{&S101706[44], 20}, {NULL}};
const Segment SGI236654[] = {{&S101706[30], 6}, {NULL}};
const Segment SGI236655[] = {{&S98371[34], 15}, {NULL}};
const Segment SGI236656[] = {{&S98371[27], 6}, {NULL}};
const Segment SGI236657[] = {{&S101706[31], 3}, {NULL}};
const Segment SGI236658[] = {{&S101706[78], 6}, {NULL}};
const Segment SGI236659[] = {{&S98371[102], 9}, {NULL}};
const Segment SGI236660[] = {{&S98371[75], 6}, {NULL}};
const Segment SGI236661[] = {{&S98371[136], 8}, {NULL}};
const Segment SGI236662[] = {{&S101707[44], 25}, {NULL}};
const Segment SGI236663[] = {{&S101707[44], 36}, {NULL}};
const Segment SGI236664[] = {{&S101707[30], 6}, {NULL}};
const Segment SGI236665[] = {{&S101707[125], 2}, {NULL}};
const Segment SGI236666[] = {{&S101707[94], 5}, {NULL}};
const Segment SGI236667[] = {{&S83691[34], 10}, {NULL}};
const Segment SGI236668[] = {{&S83691[27], 6}, {NULL}};
const Segment SGI236669[] = {{&S83691[96], 9}, {NULL}};
const Segment SGI236670[] = {{&S83691[69], 6}, {NULL}};
const Segment SGI236671[] = {{&S98377[44], 11}, {NULL}};
const Segment SGI236672[] = {{&S98377[44], 23}, {NULL}};
const Segment SGI236673[] = {{&S98377[30], 6}, {NULL}};
const Segment SGI236674[] = {{&S98377[81], 6}, {NULL}};
const Segment SGI236675[] = {{&S98377[108], 13}, {NULL}};
const Segment SGI236676[] = {{&S91833[34], 11}, {NULL}};
const Segment SGI236677[] = {{&S91833[27], 6}, {NULL}};
const Segment SGI236678[] = {{&S91833[98], 9}, {NULL}};
const Segment SGI236679[] = {{&S91833[71], 6}, {NULL}};
const Segment SGI236680[] = {{&S98379[44], 13}, {NULL}};
const Segment SGI236681[] = {{&S98379[44], 25}, {NULL}};
const Segment SGI236682[] = {{&S98379[30], 6}, {NULL}};
const Segment SGI236683[] = {{&S98379[83], 6}, {NULL}};
const Segment SGI236684[] = {{&S98379[110], 13}, {NULL}};
const Segment SGI236685[] = {{&S83699[34], 11}, {NULL}};
const Segment SGI236686[] = {{&S83699[27], 6}, {NULL}};
const Segment SGI236687[] = {{&S98381[44], 11}, {NULL}};
const Segment SGI236688[] = {{&S91837[34], 12}, {NULL}};
const Segment SGI236689[] = {{&S98381[44], 23}, {NULL}};
const Segment SGI236690[] = {{&S98381[30], 6}, {NULL}};
const Segment SGI236691[] = {{&S91837[27], 6}, {NULL}};
const Segment SGI236692[] = {{&S83699[97], 9}, {NULL}};
const Segment SGI236693[] = {{&S83699[70], 6}, {NULL}};
const Segment SGI236694[] = {{&S91837[99], 9}, {NULL}};
const Segment SGI236695[] = {{&S98381[81], 6}, {NULL}};
const Segment SGI236696[] = {{&S91837[72], 6}, {NULL}};
const Segment SGI236697[] = {{&S98381[108], 13}, {NULL}};
const Segment SGI236698[] = {{&S83700[34], 15}, {NULL}};
const Segment SGI236699[] = {{&S83700[27], 6}, {NULL}};
const Segment SGI236700[] = {{&S91838[34], 16}, {NULL}};
const Segment SGI236701[] = {{&S91838[27], 6}, {NULL}};
const Segment SGI236702[] = {{&S83700[101], 9}, {NULL}};
const Segment SGI236703[] = {{&S83700[74], 6}, {NULL}};
const Segment SGI236704[] = {{&S91838[103], 9}, {NULL}};
const Segment SGI236705[] = {{&S91838[76], 6}, {NULL}};
const Segment SGI236706[] = {{&S98384[44], 11}, {NULL}};
const Segment SGI236707[] = {{&S98384[44], 23}, {NULL}};
const Segment SGI236708[] = {{&S98384[30], 6}, {NULL}};
const Segment SGI236709[] = {{&S83702[44], 10}, {NULL}};
const Segment SGI236710[] = {{&S83702[44], 32}, {NULL}};
const Segment SGI236711[] = {{&S83702[30], 6}, {NULL}};
const Segment SGI236712[] = {{&S98384[81], 6}, {NULL}};
const Segment SGI236713[] = {{&S98384[108], 13}, {NULL}};
const Segment SGI236714[] = {{&S83702[125], 3}, {NULL}};
const Segment SGI236715[] = {{&S83702[90], 6}, {NULL}};
const Segment SGI236716[] = {{&S91841[44], 10}, {NULL}};
const Segment SGI236717[] = {{&S91841[44], 22}, {NULL}};
const Segment SGI236718[] = {{&S91841[30], 6}, {NULL}};
const Segment SGI236719[] = {{&S83703[44], 10}, {NULL}};
const Segment SGI236720[] = {{&S83703[44], 21}, {NULL}};
const Segment SGI236721[] = {{&S91841[80], 6}, {NULL}};
const Segment SGI236722[] = {{&S83703[30], 6}, {NULL}};
const Segment SGI236723[] = {{&S91841[107], 13}, {NULL}};
const Segment SGI236724[] = {{&S83703[114], 3}, {NULL}};
const Segment SGI236725[] = {{&S83703[79], 6}, {NULL}};
const Segment SGI236726[] = {{&S91842[44], 9}, {NULL}};
const Segment SGI236727[] = {{&S91842[44], 26}, {NULL}};
const Segment SGI236728[] = {{&S91842[30], 6}, {NULL}};
const Segment SGI236729[] = {{&S101728[34], 10}, {NULL}};
const Segment SGI236730[] = {{&S101728[27], 6}, {NULL}};
const Segment SGI236731[] = {{&S91842[84], 6}, {NULL}};
const Segment SGI236732[] = {{&S91842[111], 13}, {NULL}};
const Segment SGI236733[] = {{&S101728[97], 9}, {NULL}};
const Segment SGI236734[] = {{&S101728[70], 6}, {NULL}};
const Segment SGI236735[] = {{&S91847[44], 14}, {NULL}};
const Segment SGI236736[] = {{&S91847[44], 26}, {NULL}};
const Segment SGI236737[] = {{&S91847[30], 6}, {NULL}};
const Segment SGI236738[] = {{&S91847[135], 3}, {NULL}};
const Segment SGI236739[] = {{&S91847[100], 6}, {NULL}};
const Segment SGI236740[] = {{&S91847[72], 8}, {NULL}};
const Segment SGI236741[] = {{&S101742[34], 20}, {NULL}};
const Segment SGI236742[] = {{&S101742[27], 6}, {NULL}};
const Segment SGI236743[] = {{&S101742[107], 9}, {NULL}};
const Segment SGI236744[] = {{&S101742[80], 6}, {NULL}};
const Segment SGI236745[] = {{&S91858[34], 15}, {NULL}};
const Segment SGI236746[] = {{&S91858[27], 6}, {NULL}};
const Segment SGI236747[] = {{&S91858[102], 9}, {NULL}};
const Segment SGI236748[] = {{&S91858[75], 6}, {NULL}};
const Segment SGI236749[] = {{&S91858[143], 8}, {NULL}};
const Segment SGI236750[] = {{&S101746[44], 10}, {NULL}};
const Segment SGI236751[] = {{&S101746[44], 22}, {NULL}};
const Segment SGI236752[] = {{&S101746[30], 6}, {NULL}};
const Segment SGI236753[] = {{&S101746[31], 3}, {NULL}};
const Segment SGI236754[] = {{&S101746[80], 6}, {NULL}};
const Segment SGI236755[] = {{&S101746[133], 14}, {NULL}};
const Segment SGI236756[] = {{&S101746[140], 7}, {NULL}};
const Segment SGI236757[] = {{&S101747[44], 10}, {NULL}};
const Segment SGI236758[] = {{&S101747[44], 22}, {NULL}};
const Segment SGI236759[] = {{&S101747[30], 6}, {NULL}};
const Segment SGI236760[] = {{&S83727[34], 15}, {NULL}};
const Segment SGI236761[] = {{&S101747[31], 3}, {NULL}};
const Segment SGI236762[] = {{&S83727[27], 6}, {NULL}};
const Segment SGI236763[] = {{&S101747[87], 6}, {NULL}};
const Segment SGI236764[] = {{&S101747[66], 8}, {NULL}};
const Segment SGI236765[] = {{&S83727[101], 9}, {NULL}};
const Segment SGI236766[] = {{&S83727[74], 6}, {NULL}};
const Segment SGI236767[] = {{&S83729[34], 7}, {NULL}};
const Segment SGI236768[] = {{&S83729[27], 6}, {NULL}};
const Segment SGI236769[] = {{&S83729[94], 9}, {NULL}};
const Segment SGI236770[] = {{&S83729[67], 6}, {NULL}};
const Segment SGI236771[] = {{&S101751[58], 10}, {NULL}};
const Segment SGI236772[] = {{&S101751[58], 22}, {NULL}};
const Segment SGI236773[] = {{&S101751[44], 6}, {NULL}};
const Segment SGI236774[] = {{&S101751[45], 3}, {NULL}};
const Segment SGI236775[] = {{&S101751[94], 6}, {NULL}};
const Segment SGI236776[] = {{&S98421[52], 10}, {NULL}};
const Segment SGI236777[] = {{&S98421[52], 22}, {NULL}};
const Segment SGI236778[] = {{&S98421[38], 6}, {NULL}};
const Segment SGI236779[] = {{&S101754[44], 10}, {NULL}};
const Segment SGI236780[] = {{&S98421[88], 5}, {NULL}};
const Segment SGI236781[] = {{&S101754[44], 22}, {NULL}};
const Segment SGI236782[] = {{&S101754[30], 6}, {NULL}};
const Segment SGI236783[] = {{&S101754[159], 3}, {NULL}};
const Segment SGI236784[] = {{&S98421[0], 14}, {NULL}};
const Segment SGI236785[] = {{&S101754[125], 3}, {NULL}};
const Segment SGI236786[] = {{&S101754[90], 6}, {NULL}};
const Segment SGI236787[] = {{&S101754[145], 8}, {NULL}};
const Segment SGI236788[] = {{&S101759[44], 10}, {NULL}};
const Segment SGI236789[] = {{&S101759[44], 22}, {NULL}};
const Segment SGI236790[] = {{&S101759[30], 6}, {NULL}};
const Segment SGI236791[] = {{&S101759[31], 3}, {NULL}};
const Segment SGI236792[] = {{&S101759[80], 6}, {NULL}};
const Segment SGI236793[] = {{&S101759[132], 10}, {NULL}};
const Segment SGI236794[] = {{&S98431[34], 22}, {NULL}};
const Segment SGI236795[] = {{&S98431[27], 6}, {NULL}};
const Segment SGI236796[] = {{&S98431[110], 9}, {NULL}};
const Segment SGI236797[] = {{&S98431[83], 6}, {NULL}};
const Segment SGI236798[] = {{&S98432[44], 8}, {NULL}};
const Segment SGI236799[] = {{&S98432[44], 20}, {NULL}};
const Segment SGI236800[] = {{&S98432[30], 6}, {NULL}};
const Segment SGI236801[] = {{&S98432[113], 3}, {NULL}};
const Segment SGI236802[] = {{&S98432[78], 6}, {NULL}};
const Segment SGI236803[] = {{&S98435[44], 12}, {NULL}};
const Segment SGI236804[] = {{&S98435[44], 24}, {NULL}};
const Segment SGI236805[] = {{&S98435[30], 6}, {NULL}};
const Segment SGI236806[] = {{&S98435[82], 6}, {NULL}};
const Segment SGI236807[] = {{&S98435[109], 13}, {NULL}};
const Segment SGI236808[] = {{&S98436[45], 4}, {NULL}};
const Segment SGI236809[] = {{&S98436[30], 6}, {NULL}};
const Segment SGI236810[] = {{&S98436[110], 3}, {NULL}};
const Segment SGI236811[] = {{&S98436[76], 5}, {NULL}};
const Segment SGI236812[] = {{&S98436[134], 13}, {NULL}};
const Segment SGI236813[] = {{&S92362[44], 10}, {NULL}};
const Segment SGI236814[] = {{&S92362[44], 22}, {NULL}};
const Segment SGI236815[] = {{&S92362[30], 6}, {NULL}};
const Segment SGI236816[] = {{&S92362[133], 3}, {NULL}};
const Segment SGI236817[] = {{&S92362[98], 6}, {NULL}};
const Segment SGI236818[] = {{&S92362[68], 8}, {NULL}};
const Segment SGI236819[] = {{&S92363[44], 10}, {NULL}};
const Segment SGI236820[] = {{&S92363[44], 22}, {NULL}};
const Segment SGI236821[] = {{&S92363[30], 6}, {NULL}};
const Segment SGI236822[] = {{&S92363[133], 3}, {NULL}};
const Segment SGI236823[] = {{&S92363[98], 6}, {NULL}};
const Segment SGI236824[] = {{&S92363[68], 8}, {NULL}};
const Segment SGI236825[] = {{&S98455[44], 10}, {NULL}};
const Segment SGI236826[] = {{&S98455[44], 22}, {NULL}};
const Segment SGI236827[] = {{&S98455[30], 6}, {NULL}};
const Segment SGI236828[] = {{&S98455[115], 3}, {NULL}};
const Segment SGI236829[] = {{&S98455[80], 6}, {NULL}};
const Segment SGI236830[] = {{&S92369[38], 10}, {NULL}};
const Segment SGI236831[] = {{&S92369[31], 6}, {NULL}};
const Segment SGI236832[] = {{&S98461[45], 13}, {NULL}};
const Segment SGI236833[] = {{&S98461[30], 6}, {NULL}};
const Segment SGI236834[] = {{&S98461[118], 3}, {NULL}};
const Segment SGI236835[] = {{&S98461[84], 5}, {NULL}};
const Segment SGI236836[] = {{&S98461[142], 13}, {NULL}};
const Segment SGI236837[] = {{&S83787[34], 7}, {NULL}};
const Segment SGI236838[] = {{&S83787[27], 6}, {NULL}};
const Segment SGI236839[] = {{&S83787[93], 9}, {NULL}};
const Segment SGI236840[] = {{&S83787[66], 6}, {NULL}};
const Segment SGI236841[] = {{&S91933[44], 7}, {NULL}};
const Segment SGI236842[] = {{&S91933[44], 19}, {NULL}};
const Segment SGI236843[] = {{&S91933[30], 6}, {NULL}};
const Segment SGI236844[] = {{&S91933[133], 3}, {NULL}};
const Segment SGI236845[] = {{&S91933[98], 6}, {NULL}};
const Segment SGI236846[] = {{&S91934[44], 11}, {NULL}};
const Segment SGI236847[] = {{&S91934[44], 23}, {NULL}};
const Segment SGI236848[] = {{&S91934[30], 6}, {NULL}};
const Segment SGI236849[] = {{&S91934[137], 3}, {NULL}};
const Segment SGI236850[] = {{&S91934[102], 6}, {NULL}};
const Segment SGI236851[] = {{&S83799[34], 10}, {NULL}};
const Segment SGI236852[] = {{&S83799[27], 6}, {NULL}};
const Segment SGI236853[] = {{&S83799[96], 9}, {NULL}};
const Segment SGI236854[] = {{&S83799[69], 6}, {NULL}};
const Segment SGI236855[] = {{&S83801[34], 7}, {NULL}};
const Segment SGI236856[] = {{&S83801[27], 6}, {NULL}};
const Segment SGI236857[] = {{&S83801[93], 9}, {NULL}};
const Segment SGI236858[] = {{&S83801[66], 6}, {NULL}};
const Segment SGI236859[] = {{&S98477[34], 15}, {NULL}};
const Segment SGI236860[] = {{&S98477[27], 6}, {NULL}};
const Segment SGI236861[] = {{&S98477[102], 9}, {NULL}};
const Segment SGI236862[] = {{&S98477[75], 6}, {NULL}};
const Segment SGI236863[] = {{&S98490[34], 7}, {NULL}};
const Segment SGI236864[] = {{&S98490[27], 6}, {NULL}};
const Segment SGI236865[] = {{&S98490[94], 9}, {NULL}};
const Segment SGI236866[] = {{&S98490[67], 6}, {NULL}};
const Segment SGI236867[] = {{&S98492[34], 10}, {NULL}};
const Segment SGI236868[] = {{&S98492[27], 6}, {NULL}};
const Segment SGI236869[] = {{&S98492[97], 9}, {NULL}};
const Segment SGI236870[] = {{&S98492[70], 6}, {NULL}};
const Segment SGI236871[] = {{&S92414[34], 6}, {NULL}};
const Segment SGI236872[] = {{&S92414[27], 6}, {NULL}};
const Segment SGI236873[] = {{&S92414[93], 9}, {NULL}};
const Segment SGI236874[] = {{&S92414[66], 6}, {NULL}};
const Segment SGI236875[] = {{&S91964[34], 9}, {NULL}};
const Segment SGI236876[] = {{&S91964[27], 6}, {NULL}};
const Segment SGI236877[] = {{&S91964[96], 9}, {NULL}};
const Segment SGI236878[] = {{&S91964[69], 6}, {NULL}};
const Segment SGI236879[] = {{&S91966[34], 5}, {NULL}};
const Segment SGI236880[] = {{&S91966[27], 6}, {NULL}};
const Segment SGI236881[] = {{&S91966[92], 9}, {NULL}};
const Segment SGI236882[] = {{&S91966[65], 6}, {NULL}};
const Segment SGI236883[] = {{&S91971[34], 11}, {NULL}};
const Segment SGI236884[] = {{&S91971[27], 6}, {NULL}};
const Segment SGI236885[] = {{&S91971[98], 9}, {NULL}};
const Segment SGI236886[] = {{&S91971[71], 6}, {NULL}};
const Segment SGI236887[] = {{&S83860[44], 18}, {NULL}};
const Segment SGI236888[] = {{&S83860[44], 29}, {NULL}};
const Segment SGI236889[] = {{&S83860[30], 6}, {NULL}};
const Segment SGI236890[] = {{&S83860[122], 3}, {NULL}};
const Segment SGI236891[] = {{&S83860[87], 6}, {NULL}};
const Segment SGI236892[] = {{&S91998[44], 13}, {NULL}};
const Segment SGI236893[] = {{&S91998[44], 25}, {NULL}};
const Segment SGI236894[] = {{&S91998[30], 6}, {NULL}};
const Segment SGI236895[] = {{&S91998[118], 3}, {NULL}};
const Segment SGI236896[] = {{&S91998[83], 6}, {NULL}};
const Segment SGI236897[] = {{&S91999[44], 13}, {NULL}};
const Segment SGI236898[] = {{&S91999[44], 29}, {NULL}};
const Segment SGI236899[] = {{&S91999[30], 6}, {NULL}};
const Segment SGI236900[] = {{&S91999[120], 3}, {NULL}};
const Segment SGI236901[] = {{&S91999[87], 5}, {NULL}};
const Segment SGI236902[] = {{&S91999[124], 14}, {NULL}};
const Segment SGI236903[] = {{&S92004[34], 18}, {NULL}};
const Segment SGI236904[] = {{&S92004[27], 6}, {NULL}};
const Segment SGI236905[] = {{&S92004[105], 9}, {NULL}};
const Segment SGI236906[] = {{&S92004[78], 6}, {NULL}};
const Segment SGI236907[] = {{&S83878[34], 10}, {NULL}};
const Segment SGI236908[] = {{&S83878[27], 6}, {NULL}};
const Segment SGI236909[] = {{&S83878[96], 9}, {NULL}};
const Segment SGI236910[] = {{&S83878[69], 6}, {NULL}};
const Segment SGI236911[] = {{&S83879[44], 14}, {NULL}};
const Segment SGI236912[] = {{&S83879[44], 26}, {NULL}};
const Segment SGI236913[] = {{&S83879[30], 6}, {NULL}};
const Segment SGI236914[] = {{&S83879[119], 3}, {NULL}};
const Segment SGI236915[] = {{&S83879[84], 6}, {NULL}};
const Segment SGI236916[] = {{&S83879[144], 8}, {NULL}};
const Segment SGI236917[] = {{&S83880[44], 14}, {NULL}};
const Segment SGI236918[] = {{&S83880[44], 25}, {NULL}};
const Segment SGI236919[] = {{&S83880[30], 6}, {NULL}};
const Segment SGI236920[] = {{&S83880[136], 3}, {NULL}};
const Segment SGI236921[] = {{&S83880[101], 6}, {NULL}};
const Segment SGI236922[] = {{&S83880[71], 8}, {NULL}};
const Segment SGI236923[] = {{&S83882[44], 13}, {NULL}};
const Segment SGI236924[] = {{&S83882[44], 25}, {NULL}};
const Segment SGI236925[] = {{&S83882[30], 6}, {NULL}};
const Segment SGI236926[] = {{&S83882[118], 3}, {NULL}};
const Segment SGI236927[] = {{&S83882[83], 6}, {NULL}};
const Segment SGI236928[] = {{&S92018[44], 11}, {NULL}};
const Segment SGI236929[] = {{&S92018[44], 23}, {NULL}};
const Segment SGI236930[] = {{&S92018[30], 6}, {NULL}};
const Segment SGI236931[] = {{&S92018[116], 3}, {NULL}};
const Segment SGI236932[] = {{&S92018[81], 6}, {NULL}};
const Segment SGI236933[] = {{&S92019[52], 10}, {NULL}};
const Segment SGI236934[] = {{&S92019[52], 22}, {NULL}};
const Segment SGI236935[] = {{&S92019[38], 6}, {NULL}};
const Segment SGI236936[] = {{&S92019[88], 5}, {NULL}};
const Segment SGI236937[] = {{&S92019[11], 3}, {NULL}};
const Segment SGI236938[] = {{&S92025[34], 6}, {NULL}};
const Segment SGI236939[] = {{&S92025[27], 6}, {NULL}};
const Segment SGI236940[] = {{&S92025[93], 9}, {NULL}};
const Segment SGI236941[] = {{&S92025[66], 6}, {NULL}};
const Segment SGI236942[] = {{&S92025[134], 8}, {NULL}};
const Segment SGI236943[] = {{&S92028[34], 9}, {NULL}};
const Segment SGI236944[] = {{&S92028[27], 6}, {NULL}};
const Segment SGI236945[] = {{&S92028[96], 9}, {NULL}};
const Segment SGI236946[] = {{&S92028[69], 6}, {NULL}};
const Segment SGI236947[] = {{&S92028[137], 8}, {NULL}};
const Segment SGI236948[] = {{&S101761[45], 10}, {NULL}};
const Segment SGI236949[] = {{&S101761[30], 6}, {NULL}};
const Segment SGI236950[] = {{&S101761[31], 3}, {NULL}};
const Segment SGI236951[] = {{&S101761[81], 5}, {NULL}};
const Segment SGI236952[] = {{&S101761[139], 13}, {NULL}};
const Segment SGI236953[] = {{&S101762[44], 10}, {NULL}};
const Segment SGI236954[] = {{&S101762[44], 22}, {NULL}};
const Segment SGI236955[] = {{&S101762[30], 6}, {NULL}};
const Segment SGI236956[] = {{&S101762[31], 3}, {NULL}};
const Segment SGI236957[] = {{&S101762[80], 6}, {NULL}};
const Segment SGI236958[] = {{&S101762[143], 3}, {NULL}};
const Segment SGI236959[] = {{&S101765[44], 10}, {NULL}};
const Segment SGI236960[] = {{&S101765[44], 22}, {NULL}};
const Segment SGI236961[] = {{&S101765[30], 6}, {NULL}};
const Segment SGI236962[] = {{&S101765[31], 3}, {NULL}};
const Segment SGI236963[] = {{&S101765[80], 6}, {NULL}};
const Segment SGI236964[] = {{&S101765[143], 3}, {NULL}};
const Segment SGI236965[] = {{&S101777[44], 13}, {NULL}};
const Segment SGI236966[] = {{&S101777[44], 25}, {NULL}};
const Segment SGI236967[] = {{&S101777[30], 6}, {NULL}};
const Segment SGI236968[] = {{&S101777[31], 3}, {NULL}};
const Segment SGI236969[] = {{&S101777[83], 6}, {NULL}};
const Segment SGI236970[] = {{&S92037[44], 14}, {NULL}};
const Segment SGI236971[] = {{&S92037[44], 26}, {NULL}};
const Segment SGI236972[] = {{&S92037[30], 6}, {NULL}};
const Segment SGI236973[] = {{&S92037[119], 3}, {NULL}};
const Segment SGI236974[] = {{&S92037[84], 6}, {NULL}};
const Segment SGI236975[] = {{&S92039[44], 10}, {NULL}};
const Segment SGI236976[] = {{&S92039[44], 22}, {NULL}};
const Segment SGI236977[] = {{&S92039[30], 6}, {NULL}};
const Segment SGI236978[] = {{&S92039[80], 6}, {NULL}};
const Segment SGI236979[] = {{&S92039[107], 13}, {NULL}};
const Segment SGI236980[] = {{&S98611[45], 14}, {NULL}};
const Segment SGI236981[] = {{&S98611[30], 6}, {NULL}};
const Segment SGI236982[] = {{&S98611[119], 3}, {NULL}};
const Segment SGI236983[] = {{&S98611[85], 5}, {NULL}};
const Segment SGI236984[] = {{&S98611[143], 13}, {NULL}};
const Segment SGI236985[] = {{&S83950[34], 10}, {NULL}};
const Segment SGI236986[] = {{&S83950[27], 6}, {NULL}};
const Segment SGI236987[] = {{&S83950[96], 9}, {NULL}};
const Segment SGI236988[] = {{&S83950[69], 6}, {NULL}};
const Segment SGI236989[] = {{&S92055[44], 6}, {NULL}};
const Segment SGI236990[] = {{&S92055[44], 18}, {NULL}};
const Segment SGI236991[] = {{&S92055[30], 6}, {NULL}};
const Segment SGI236992[] = {{&S92055[76], 6}, {NULL}};
const Segment SGI236993[] = {{&S92055[103], 13}, {NULL}};
const Segment SGI236994[] = {{&S83955[34], 10}, {NULL}};
const Segment SGI236995[] = {{&S83955[27], 6}, {NULL}};
const Segment SGI236996[] = {{&S83955[96], 9}, {NULL}};
const Segment SGI236997[] = {{&S83955[69], 6}, {NULL}};
const Segment SGI236998[] = {{&S83984[44], 10}, {NULL}};
const Segment SGI236999[] = {{&S83984[44], 21}, {NULL}};
const Segment SGI237000[] = {{&S83984[30], 6}, {NULL}};
const Segment SGI237001[] = {{&S83984[112], 3}, {NULL}};
const Segment SGI237002[] = {{&S83984[79], 5}, {NULL}};
const Segment SGI237003[] = {{&S83985[34], 11}, {NULL}};
const Segment SGI237004[] = {{&S83985[27], 6}, {NULL}};
const Segment SGI237005[] = {{&S83985[98], 9}, {NULL}};
const Segment SGI237006[] = {{&S83985[71], 6}, {NULL}};
const Segment SGI237007[] = {{&S98661[45], 10}, {NULL}};
const Segment SGI237008[] = {{&S98661[30], 6}, {NULL}};
const Segment SGI237009[] = {{&S98661[115], 3}, {NULL}};
const Segment SGI237010[] = {{&S98661[81], 5}, {NULL}};
const Segment SGI237011[] = {{&S98661[139], 13}, {NULL}};
const Segment SGI237012[] = {{&S98674[34], 13}, {NULL}};
const Segment SGI237013[] = {{&S98674[27], 6}, {NULL}};
const Segment SGI237014[] = {{&S98674[114], 9}, {NULL}};
const Segment SGI237015[] = {{&S98674[87], 6}, {NULL}};
const Segment SGI237016[] = {{&S84023[34], 10}, {NULL}};
const Segment SGI237017[] = {{&S84023[27], 6}, {NULL}};
const Segment SGI237018[] = {{&S84023[96], 9}, {NULL}};
const Segment SGI237019[] = {{&S84023[69], 6}, {NULL}};
const Segment SGI237020[] = {{&S101890[44], 10}, {NULL}};
const Segment SGI237021[] = {{&S101890[44], 22}, {NULL}};
const Segment SGI237022[] = {{&S101890[30], 6}, {NULL}};
const Segment SGI237023[] = {{&S101890[31], 3}, {NULL}};
const Segment SGI237024[] = {{&S101890[95], 6}, {NULL}};
const Segment SGI237025[] = {{&S101890[68], 8}, {NULL}};
const Segment SGI237026[] = {{&S101895[45], 10}, {NULL}};
const Segment SGI237027[] = {{&S101895[30], 6}, {NULL}};
const Segment SGI237028[] = {{&S101895[31], 3}, {NULL}};
const Segment SGI237029[] = {{&S101895[81], 5}, {NULL}};
const Segment SGI237030[] = {{&S101895[139], 13}, {NULL}};
const Segment SGI237031[] = {{&S98704[44], 11}, {NULL}};
const Segment SGI237032[] = {{&S98704[44], 23}, {NULL}};
const Segment SGI237033[] = {{&S98704[30], 6}, {NULL}};
const Segment SGI237034[] = {{&S98704[116], 3}, {NULL}};
const Segment SGI237035[] = {{&S101896[45], 10}, {NULL}};
const Segment SGI237036[] = {{&S98704[81], 6}, {NULL}};
const Segment SGI237037[] = {{&S101896[30], 6}, {NULL}};
const Segment SGI237038[] = {{&S101896[31], 3}, {NULL}};
const Segment SGI237039[] = {{&S101896[81], 5}, {NULL}};
const Segment SGI237040[] = {{&S101896[139], 13}, {NULL}};
const Segment SGI237041[] = {{&S84044[52], 10}, {NULL}};
const Segment SGI237042[] = {{&S84044[52], 21}, {NULL}};
const Segment SGI237043[] = {{&S84044[38], 6}, {NULL}};
const Segment SGI237044[] = {{&S84044[87], 5}, {NULL}};
const Segment SGI237045[] = {{&S101905[44], 10}, {NULL}};
const Segment SGI237046[] = {{&S101905[44], 22}, {NULL}};
const Segment SGI237047[] = {{&S101905[30], 6}, {NULL}};
const Segment SGI237048[] = {{&S101905[31], 3}, {NULL}};
const Segment SGI237049[] = {{&S101905[80], 6}, {NULL}};
const Segment SGI237050[] = {{&S101905[133], 11}, {NULL}};
const Segment SGI237051[] = {{&S98724[44], 18}, {NULL}};
const Segment SGI237052[] = {{&S98724[44], 39}, {NULL}};
const Segment SGI237053[] = {{&S98724[30], 6}, {NULL}};
const Segment SGI237054[] = {{&S98724[97], 6}, {NULL}};
const Segment SGI237055[] = {{&S98724[124], 13}, {NULL}};
const Segment SGI237056[] = {{&S92168[44], 12}, {NULL}};
const Segment SGI237057[] = {{&S98731[44], 6}, {NULL}};
const Segment SGI237058[] = {{&S98731[44], 18}, {NULL}};
const Segment SGI237059[] = {{&S98731[30], 6}, {NULL}};
const Segment SGI237060[] = {{&S98731[76], 6}, {NULL}};
const Segment SGI237061[] = {{&S98731[103], 13}, {NULL}};
const Segment SGI237062[] = {{&S92168[44], 29}, {NULL}};
const Segment SGI237063[] = {{&S92168[30], 6}, {NULL}};
const Segment SGI237064[] = {{&S92168[122], 3}, {NULL}};
const Segment SGI237065[] = {{&S92168[87], 6}, {NULL}};
const Segment SGI237066[] = {{&S92181[34], 10}, {NULL}};
const Segment SGI237067[] = {{&S92181[27], 6}, {NULL}};
const Segment SGI237068[] = {{&S92181[97], 9}, {NULL}};
const Segment SGI237069[] = {{&S92181[70], 6}, {NULL}};
const Segment SGI237070[] = {{&S92183[34], 10}, {NULL}};
const Segment SGI237071[] = {{&S92183[27], 6}, {NULL}};
const Segment SGI237072[] = {{&S92183[97], 9}, {NULL}};
const Segment SGI237073[] = {{&S92183[70], 6}, {NULL}};
const Segment SGI237074[] = {{&S92193[45], 7}, {NULL}};
const Segment SGI237075[] = {{&S92193[30], 6}, {NULL}};
const Segment SGI237076[] = {{&S92193[118], 3}, {NULL}};
const Segment SGI237077[] = {{&S92193[84], 5}, {NULL}};
const Segment SGI237078[] = {{&S92193[142], 13}, {NULL}};
const Segment SGI237079[] = {{&S98769[34], 12}, {NULL}};
const Segment SGI237080[] = {{&S98769[27], 6}, {NULL}};
const Segment SGI237081[] = {{&S98769[99], 9}, {NULL}};
const Segment SGI237082[] = {{&S98769[72], 6}, {NULL}};
const Segment SGI237083[] = {{&S98769[140], 8}, {NULL}};
const Segment SGI237084[] = {{&S92201[34], 11}, {NULL}};
const Segment SGI237085[] = {{&S92201[27], 6}, {NULL}};
const Segment SGI237086[] = {{&S92201[98], 9}, {NULL}};
const Segment SGI237087[] = {{&S92201[71], 6}, {NULL}};
const Segment SGI237088[] = {{&S98774[44], 15}, {NULL}};
const Segment SGI237089[] = {{&S98774[44], 27}, {NULL}};
const Segment SGI237090[] = {{&S98774[30], 6}, {NULL}};
const Segment SGI237091[] = {{&S98774[120], 3}, {NULL}};
const Segment SGI237092[] = {{&S98774[85], 6}, {NULL}};
const Segment SGI237093[] = {{&S92206[34], 10}, {NULL}};
const Segment SGI237094[] = {{&S92206[27], 6}, {NULL}};
const Segment SGI237095[] = {{&S92206[96], 9}, {NULL}};
const Segment SGI237096[] = {{&S92206[69], 6}, {NULL}};
const Segment SGI237097[] = {{&S92219[34], 9}, {NULL}};
const Segment SGI237098[] = {{&S92219[27], 6}, {NULL}};
const Segment SGI237099[] = {{&S92219[101], 9}, {NULL}};
const Segment SGI237100[] = {{&S92219[74], 6}, {NULL}};
const Segment SGI237101[] = {{&S92220[34], 9}, {NULL}};
const Segment SGI237102[] = {{&S92220[27], 6}, {NULL}};
const Segment SGI237103[] = {{&S92220[96], 9}, {NULL}};
const Segment SGI237104[] = {{&S92220[69], 6}, {NULL}};
const Segment SGI237105[] = {{&S92220[130], 8}, {NULL}};
const Segment SGI237106[] = {{&S98794[44], 7}, {NULL}};
const Segment SGI237107[] = {{&S98794[44], 24}, {NULL}};
const Segment SGI237108[] = {{&S98794[30], 6}, {NULL}};
const Segment SGI237109[] = {{&S98794[117], 3}, {NULL}};
const Segment SGI237110[] = {{&S98794[82], 6}, {NULL}};
const Segment SGI237111[] = {{&S92221[34], 11}, {NULL}};
const Segment SGI237112[] = {{&S92221[27], 6}, {NULL}};
const Segment SGI237113[] = {{&S92221[98], 9}, {NULL}};
const Segment SGI237114[] = {{&S92221[71], 6}, {NULL}};
const Segment SGI237115[] = {{&S92221[139], 8}, {NULL}};
const Segment SGI237116[] = {{&S98796[44], 7}, {NULL}};
const Segment SGI237117[] = {{&S92222[34], 11}, {NULL}};
const Segment SGI237118[] = {{&S98796[44], 19}, {NULL}};
const Segment SGI237119[] = {{&S92222[27], 6}, {NULL}};
const Segment SGI237120[] = {{&S98796[30], 6}, {NULL}};
const Segment SGI237121[] = {{&S98796[112], 3}, {NULL}};
const Segment SGI237122[] = {{&S92222[98], 9}, {NULL}};
const Segment SGI237123[] = {{&S92222[71], 6}, {NULL}};
const Segment SGI237124[] = {{&S98796[77], 6}, {NULL}};
const Segment SGI237125[] = {{&S98798[44], 14}, {NULL}};
const Segment SGI237126[] = {{&S98798[44], 26}, {NULL}};
const Segment SGI237127[] = {{&S98798[30], 6}, {NULL}};
const Segment SGI237128[] = {{&S98798[119], 3}, {NULL}};
const Segment SGI237129[] = {{&S98798[84], 6}, {NULL}};
const Segment SGI237130[] = {{&S92225[34], 8}, {NULL}};
const Segment SGI237131[] = {{&S92225[27], 6}, {NULL}};
const Segment SGI237132[] = {{&S92225[100], 9}, {NULL}};
const Segment SGI237133[] = {{&S92225[73], 6}, {NULL}};
const Segment SGI237134[] = {{&S84145[44], 7}, {NULL}};
const Segment SGI237135[] = {{&S84145[44], 19}, {NULL}};
const Segment SGI237136[] = {{&S84145[30], 6}, {NULL}};
const Segment SGI237137[] = {{&S84145[140], 3}, {NULL}};
const Segment SGI237138[] = {{&S84145[105], 6}, {NULL}};
const Segment SGI237139[] = {{&S84145[72], 9}, {NULL}};
const Segment SGI237140[] = {{&S84149[44], 11}, {NULL}};
const Segment SGI237141[] = {{&S84149[44], 22}, {NULL}};
const Segment SGI237142[] = {{&S84149[30], 6}, {NULL}};
const Segment SGI237143[] = {{&S84149[115], 3}, {NULL}};
const Segment SGI237144[] = {{&S84149[80], 6}, {NULL}};
const Segment SGI237145[] = {{&S84149[150], 3}, {NULL}};
const Segment SGI237146[] = {{&S92242[34], 10}, {NULL}};
const Segment SGI237147[] = {{&S92242[27], 6}, {NULL}};
const Segment SGI237148[] = {{&S92242[97], 9}, {NULL}};
const Segment SGI237149[] = {{&S92242[70], 6}, {NULL}};
const Segment SGI237150[] = {{&S92243[44], 19}, {NULL}};
const Segment SGI237151[] = {{&S92243[44], 31}, {NULL}};
const Segment SGI237152[] = {{&S92243[30], 6}, {NULL}};
const Segment SGI237153[] = {{&S98818[44], 25}, {NULL}};
const Segment SGI237154[] = {{&S98818[44], 37}, {NULL}};
const Segment SGI237155[] = {{&S98818[30], 6}, {NULL}};
const Segment SGI237156[] = {{&S92243[124], 3}, {NULL}};
const Segment SGI237157[] = {{&S92243[89], 6}, {NULL}};
const Segment SGI237158[] = {{&S98818[95], 6}, {NULL}};
const Segment SGI237159[] = {{&S98818[122], 13}, {NULL}};
const Segment SGI237160[] = {{&S92246[34], 10}, {NULL}};
const Segment SGI237161[] = {{&S92246[27], 6}, {NULL}};
const Segment SGI237162[] = {{&S92246[97], 9}, {NULL}};
const Segment SGI237163[] = {{&S92246[70], 6}, {NULL}};
const Segment SGI237164[] = {{&S92246[138], 8}, {NULL}};
const Segment SGI237165[] = {{&S92258[44], 18}, {NULL}};
const Segment SGI237166[] = {{&S92258[44], 39}, {NULL}};
const Segment SGI237167[] = {{&S92258[30], 6}, {NULL}};
const Segment SGI237168[] = {{&S92258[97], 6}, {NULL}};
const Segment SGI237169[] = {{&S92258[124], 13}, {NULL}};
const Segment SGI237170[] = {{&S98840[44], 18}, {NULL}};
const Segment SGI237171[] = {{&S98840[44], 39}, {NULL}};
const Segment SGI237172[] = {{&S98840[30], 6}, {NULL}};
const Segment SGI237173[] = {{&S98840[97], 6}, {NULL}};
const Segment SGI237174[] = {{&S98840[124], 13}, {NULL}};
const Segment SGI237175[] = {{&S92272[34], 9}, {NULL}};
const Segment SGI237176[] = {{&S92272[27], 6}, {NULL}};
const Segment SGI237177[] = {{&S92272[96], 9}, {NULL}};
const Segment SGI237178[] = {{&S92272[69], 6}, {NULL}};
const Segment SGI237179[] = {{&S102056[45], 7}, {NULL}};
const Segment SGI237180[] = {{&S102056[30], 6}, {NULL}};
const Segment SGI237181[] = {{&S102056[31], 3}, {NULL}};
const Segment SGI237182[] = {{&S102056[83], 5}, {NULL}};
const Segment SGI237183[] = {{&S102056[141], 13}, {NULL}};
const Segment SGI237184[] = {{&S102078[44], 10}, {NULL}};
const Segment SGI237185[] = {{&S102078[44], 22}, {NULL}};
const Segment SGI237186[] = {{&S102078[30], 6}, {NULL}};
const Segment SGI237187[] = {{&S102078[31], 3}, {NULL}};
const Segment SGI237188[] = {{&S102078[109], 6}, {NULL}};
const Segment SGI237189[] = {{&S102078[68], 8}, {NULL}};
const Segment SGI237190[] = {{&S102078[80], 15}, {NULL}};
const Segment SGI237191[] = {{&S98890[44], 7}, {NULL}};
const Segment SGI237192[] = {{&S98890[44], 24}, {NULL}};
const Segment SGI237193[] = {{&S98890[30], 6}, {NULL}};
const Segment SGI237194[] = {{&S98890[117], 3}, {NULL}};
const Segment SGI237195[] = {{&S98890[82], 6}, {NULL}};
const Segment SGI237196[] = {{&S98890[137], 8}, {NULL}};
const Segment SGI237197[] = {{&S98890[146], 13}, {NULL}};
const Segment SGI237198[] = {{&S92620[44], 17}, {NULL}};
const Segment SGI237199[] = {{&S92620[44], 29}, {NULL}};
const Segment SGI237200[] = {{&S92620[30], 6}, {NULL}};
const Segment SGI237201[] = {{&S92620[122], 3}, {NULL}};
const Segment SGI237202[] = {{&S92620[87], 6}, {NULL}};
const Segment SGI237203[] = {{&S98900[44], 10}, {NULL}};
const Segment SGI237204[] = {{&S98900[44], 22}, {NULL}};
const Segment SGI237205[] = {{&S98900[30], 6}, {NULL}};
const Segment SGI237206[] = {{&S98900[115], 3}, {NULL}};
const Segment SGI237207[] = {{&S98900[80], 6}, {NULL}};
const Segment SGI237208[] = {{&S98900[133], 7}, {NULL}};
const Segment SGI237209[] = {{&S102104[62], 10}, {NULL}};
const Segment SGI237210[] = {{&S102104[55], 6}, {NULL}};
const Segment SGI237211[] = {{&S102107[44], 10}, {NULL}};
const Segment SGI237212[] = {{&S102107[44], 22}, {NULL}};
const Segment SGI237213[] = {{&S102107[30], 6}, {NULL}};
const Segment SGI237214[] = {{&S92348[34], 18}, {NULL}};
const Segment SGI237215[] = {{&S92348[27], 6}, {NULL}};
const Segment SGI237216[] = {{&S102107[31], 3}, {NULL}};
const Segment SGI237217[] = {{&S102107[80], 6}, {NULL}};
const Segment SGI237218[] = {{&S102107[132], 10}, {NULL}};
const Segment SGI237219[] = {{&S92348[105], 9}, {NULL}};
const Segment SGI237220[] = {{&S92348[78], 6}, {NULL}};
const Segment SGI237221[] = {{&S102109[44], 10}, {NULL}};
const Segment SGI237222[] = {{&S102109[44], 22}, {NULL}};
const Segment SGI237223[] = {{&S102109[30], 6}, {NULL}};
const Segment SGI237224[] = {{&S92351[34], 9}, {NULL}};
const Segment SGI237225[] = {{&S102109[31], 3}, {NULL}};
const Segment SGI237226[] = {{&S92351[27], 6}, {NULL}};
const Segment SGI237227[] = {{&S102109[80], 6}, {NULL}};
const Segment SGI237228[] = {{&S102109[143], 3}, {NULL}};
const Segment SGI237229[] = {{&S92351[96], 9}, {NULL}};
const Segment SGI237230[] = {{&S92351[69], 6}, {NULL}};
const Segment SGI237231[] = {{&S102110[44], 10}, {NULL}};
const Segment SGI237232[] = {{&S102110[44], 22}, {NULL}};
const Segment SGI237233[] = {{&S102110[30], 6}, {NULL}};
const Segment SGI237234[] = {{&S102110[139], 4}, {NULL}};
const Segment SGI237235[] = {{&S102110[165], 3}, {NULL}};
const Segment SGI237236[] = {{&S102110[31], 3}, {&S102110[214], 5}, {NULL}};
const Segment SGI237237[] = {{&S102110[80], 6}, {NULL}};
const Segment SGI237238[] = {{&S98924[34], 14}, {NULL}};
const Segment SGI237239[] = {{&S98924[27], 6}, {NULL}};
const Segment SGI237240[] = {{&S98924[101], 9}, {NULL}};
const Segment SGI237241[] = {{&S98924[74], 6}, {NULL}};
const Segment SGI237242[] = {{&S102111[42], 10}, {NULL}};
const Segment SGI237243[] = {{&S98924[142], 8}, {NULL}};
const Segment SGI237244[] = {{&S102111[30], 6}, {NULL}};
const Segment SGI237245[] = {{&S102111[137], 4}, {NULL}};
const Segment SGI237246[] = {{&S102111[163], 3}, {NULL}};
const Segment SGI237247[] = {{&S102111[31], 3}, {&S102111[212], 5}, {NULL}};
const Segment SGI237248[] = {{&S102111[78], 6}, {NULL}};
const Segment SGI237249[] = {{&S102112[59], 10}, {NULL}};
const Segment SGI237250[] = {{&S102112[59], 22}, {NULL}};
const Segment SGI237251[] = {{&S102112[45], 6}, {NULL}};
const Segment SGI237252[] = {{&S102112[46], 3}, {NULL}};
const Segment SGI237253[] = {{&S102112[95], 6}, {NULL}};
const Segment SGI237254[] = {{&S102112[0], 14}, {NULL}};
const Segment SGI237255[] = {{&S98929[34], 7}, {NULL}};
const Segment SGI237256[] = {{&S98929[27], 6}, {NULL}};
const Segment SGI237257[] = {{&S98929[94], 9}, {NULL}};
const Segment SGI237258[] = {{&S98929[67], 6}, {NULL}};
const Segment SGI237259[] = {{&S102114[31], 8}, {NULL}};
const Segment SGI237260[] = {{&S102114[54], 3}, {NULL}};
const Segment SGI237261[] = {{&S102115[39], 8}, {NULL}};
const Segment SGI237262[] = {{&S102115[103], 3}, {NULL}};
const Segment SGI237263[] = {{&S98931[44], 6}, {NULL}};
const Segment SGI237264[] = {{&S98931[44], 18}, {NULL}};
const Segment SGI237265[] = {{&S98931[30], 6}, {NULL}};
const Segment SGI237266[] = {{&S102115[92], 3}, {NULL}};
const Segment SGI237267[] = {{&S102115[69], 6}, {NULL}};
const Segment SGI237268[] = {{&S98931[110], 3}, {NULL}};
const Segment SGI237269[] = {{&S98931[76], 5}, {NULL}};
const Segment SGI237270[] = {{&S84296[34], 10}, {NULL}};
const Segment SGI237271[] = {{&S84296[27], 6}, {NULL}};
const Segment SGI237272[] = {{&S84296[96], 9}, {NULL}};
const Segment SGI237273[] = {{&S84296[69], 6}, {NULL}};
const Segment SGI237274[] = {{&S92372[44], 6}, {NULL}};
const Segment SGI237275[] = {{&S92372[44], 18}, {NULL}};
const Segment SGI237276[] = {{&S92372[30], 6}, {NULL}};
const Segment SGI237277[] = {{&S102122[45], 10}, {NULL}};
const Segment SGI237278[] = {{&S102122[30], 6}, {NULL}};
const Segment SGI237279[] = {{&S92372[133], 3}, {NULL}};
const Segment SGI237280[] = {{&S92372[98], 6}, {NULL}};
const Segment SGI237281[] = {{&S102122[31], 3}, {NULL}};
const Segment SGI237282[] = {{&S92372[64], 8}, {NULL}};
const Segment SGI237283[] = {{&S102122[81], 5}, {NULL}};
const Segment SGI237284[] = {{&S102122[139], 13}, {NULL}};
const Segment SGI237285[] = {{&S98949[44], 4}, {NULL}};
const Segment SGI237286[] = {{&S98949[44], 17}, {NULL}};
const Segment SGI237287[] = {{&S98949[30], 6}, {NULL}};
const Segment SGI237288[] = {{&S98949[75], 6}, {NULL}};
const Segment SGI237289[] = {{&S98949[102], 13}, {NULL}};
const Segment SGI237290[] = {{&S84310[44], 5}, {NULL}};
const Segment SGI237291[] = {{&S84310[44], 17}, {NULL}};
const Segment SGI237292[] = {{&S84310[30], 6}, {NULL}};
const Segment SGI237293[] = {{&S84310[142], 3}, {NULL}};
const Segment SGI237294[] = {{&S84310[107], 6}, {NULL}};
const Segment SGI237295[] = {{&S84310[70], 9}, {NULL}};
const Segment SGI237296[] = {{&S84311[44], 9}, {NULL}};
const Segment SGI237297[] = {{&S84311[44], 21}, {NULL}};
const Segment SGI237298[] = {{&S84311[30], 6}, {NULL}};
const Segment SGI237299[] = {{&S84311[145], 3}, {NULL}};
const Segment SGI237300[] = {{&S84311[110], 6}, {NULL}};
const Segment SGI237301[] = {{&S84311[74], 9}, {NULL}};
const Segment SGI237302[] = {{&S92389[34], 10}, {NULL}};
const Segment SGI237303[] = {{&S92389[27], 6}, {NULL}};
const Segment SGI237304[] = {{&S92389[97], 9}, {NULL}};
const Segment SGI237305[] = {{&S92389[70], 6}, {NULL}};
const Segment SGI237306[] = {{&S98961[44], 11}, {NULL}};
const Segment SGI237307[] = {{&S98961[44], 23}, {NULL}};
const Segment SGI237308[] = {{&S98961[30], 6}, {NULL}};
const Segment SGI237309[] = {{&S98961[81], 6}, {NULL}};
const Segment SGI237310[] = {{&S98961[108], 13}, {NULL}};
const Segment SGI237311[] = {{&S92393[44], 11}, {NULL}};
const Segment SGI237312[] = {{&S92393[44], 23}, {NULL}};
const Segment SGI237313[] = {{&S92393[30], 6}, {NULL}};
const Segment SGI237314[] = {{&S92393[132], 3}, {NULL}};
const Segment SGI237315[] = {{&S92393[97], 6}, {NULL}};
const Segment SGI237316[] = {{&S92393[69], 8}, {NULL}};
const Segment SGI237317[] = {{&S92397[44], 10}, {NULL}};
const Segment SGI237318[] = {{&S92397[44], 22}, {NULL}};
const Segment SGI237319[] = {{&S92397[30], 6}, {NULL}};
const Segment SGI237320[] = {{&S92397[80], 6}, {NULL}};
const Segment SGI237321[] = {{&S92397[107], 13}, {NULL}};
const Segment SGI237322[] = {{&S92404[44], 7}, {NULL}};
const Segment SGI237323[] = {{&S92404[44], 19}, {NULL}};
const Segment SGI237324[] = {{&S92404[30], 6}, {NULL}};
const Segment SGI237325[] = {{&S92404[188], 3}, {NULL}};
const Segment SGI237326[] = {{&S92404[153], 6}, {NULL}};
const Segment SGI237327[] = {{&S92404[65], 8}, {NULL}};
const Segment SGI237328[] = {{&S92405[44], 10}, {NULL}};
const Segment SGI237329[] = {{&S92405[44], 22}, {NULL}};
const Segment SGI237330[] = {{&S92405[30], 6}, {NULL}};
const Segment SGI237331[] = {{&S92405[135], 3}, {NULL}};
const Segment SGI237332[] = {{&S84336[34], 9}, {NULL}};
const Segment SGI237333[] = {{&S92405[100], 6}, {NULL}};
const Segment SGI237334[] = {{&S84336[27], 6}, {NULL}};
const Segment SGI237335[] = {{&S92405[68], 8}, {NULL}};
const Segment SGI237336[] = {{&S84336[96], 9}, {NULL}};
const Segment SGI237337[] = {{&S84336[69], 6}, {NULL}};
const Segment SGI237338[] = {{&S84337[34], 7}, {NULL}};
const Segment SGI237339[] = {{&S84337[27], 6}, {NULL}};
const Segment SGI237340[] = {{&S84337[94], 9}, {NULL}};
const Segment SGI237341[] = {{&S92407[34], 13}, {NULL}};
const Segment SGI237342[] = {{&S84337[67], 6}, {NULL}};
const Segment SGI237343[] = {{&S92407[27], 6}, {NULL}};
const Segment SGI237344[] = {{&S92407[100], 9}, {NULL}};
const Segment SGI237345[] = {{&S92407[73], 6}, {NULL}};
const Segment SGI237346[] = {{&S92422[44], 13}, {NULL}};
const Segment SGI237347[] = {{&S92422[44], 25}, {NULL}};
const Segment SGI237348[] = {{&S92422[30], 6}, {NULL}};
const Segment SGI237349[] = {{&S92422[134], 3}, {NULL}};
const Segment SGI237350[] = {{&S92422[99], 6}, {NULL}};
const Segment SGI237351[] = {{&S92422[71], 8}, {NULL}};
const Segment SGI237352[] = {{&S92425[44], 13}, {NULL}};
const Segment SGI237353[] = {{&S92425[44], 25}, {NULL}};
const Segment SGI237354[] = {{&S92425[30], 6}, {NULL}};
const Segment SGI237355[] = {{&S92425[118], 3}, {NULL}};
const Segment SGI237356[] = {{&S92425[83], 6}, {NULL}};
const Segment SGI237357[] = {{&S92431[34], 10}, {NULL}};
const Segment SGI237358[] = {{&S92431[27], 6}, {NULL}};
const Segment SGI237359[] = {{&S92431[97], 9}, {NULL}};
const Segment SGI237360[] = {{&S92431[70], 6}, {NULL}};
const Segment SGI237361[] = {{&S98993[44], 6}, {NULL}};
const Segment SGI237362[] = {{&S98993[44], 18}, {NULL}};
const Segment SGI237363[] = {{&S98993[30], 6}, {NULL}};
const Segment SGI237364[] = {{&S98993[109], 3}, {NULL}};
const Segment SGI237365[] = {{&S98993[76], 5}, {NULL}};
const Segment SGI237366[] = {{&S98993[113], 14}, {NULL}};
const Segment SGI237367[] = {{&S92434[34], 9}, {NULL}};
const Segment SGI237368[] = {{&S92434[27], 6}, {NULL}};
const Segment SGI237369[] = {{&S92434[96], 9}, {NULL}};
const Segment SGI237370[] = {{&S92434[69], 6}, {NULL}};
const Segment SGI237371[] = {{&S98998[34], 16}, {NULL}};
const Segment SGI237372[] = {{&S98998[27], 6}, {NULL}};
const Segment SGI237373[] = {{&S98998[112], 9}, {NULL}};
const Segment SGI237374[] = {{&S98998[85], 6}, {NULL}};
const Segment SGI237375[] = {{&S92445[34], 10}, {NULL}};
const Segment SGI237376[] = {{&S92445[27], 6}, {NULL}};
const Segment SGI237377[] = {{&S92445[97], 9}, {NULL}};
const Segment SGI237378[] = {{&S92445[70], 6}, {NULL}};
const Segment SGI237379[] = {{&S92446[44], 9}, {NULL}};
const Segment SGI237380[] = {{&S92446[44], 21}, {NULL}};
const Segment SGI237381[] = {{&S92446[30], 6}, {NULL}};
const Segment SGI237382[] = {{&S92446[79], 6}, {NULL}};
const Segment SGI237383[] = {{&S92446[106], 13}, {NULL}};
const Segment SGI237384[] = {{&S92447[37], 4}, {NULL}};
const Segment SGI237385[] = {{&S92447[30], 6}, {NULL}};
const Segment SGI237386[] = {{&S92447[102], 3}, {NULL}};
const Segment SGI237387[] = {{&S92447[67], 6}, {NULL}};
const Segment SGI237388[] = {{&S92450[34], 10}, {NULL}};
const Segment SGI237389[] = {{&S92450[27], 6}, {NULL}};
const Segment SGI237390[] = {{&S92450[97], 9}, {NULL}};
const Segment SGI237391[] = {{&S92450[70], 6}, {NULL}};
const Segment SGI237392[] = {{&S92451[44], 7}, {NULL}};
const Segment SGI237393[] = {{&S92451[44], 23}, {NULL}};
const Segment SGI237394[] = {{&S92451[30], 6}, {NULL}};
const Segment SGI237395[] = {{&S92451[116], 3}, {NULL}};
const Segment SGI237396[] = {{&S92451[81], 6}, {NULL}};
const Segment SGI237397[] = {{&S92452[44], 7}, {NULL}};
const Segment SGI237398[] = {{&S92452[44], 23}, {NULL}};
const Segment SGI237399[] = {{&S92452[30], 6}, {NULL}};
const Segment SGI237400[] = {{&S92452[116], 3}, {NULL}};
const Segment SGI237401[] = {{&S92452[81], 6}, {NULL}};
const Segment SGI237402[] = {{&S92457[34], 7}, {NULL}};
const Segment SGI237403[] = {{&S92457[27], 6}, {NULL}};
const Segment SGI237404[] = {{&S92457[94], 9}, {NULL}};
const Segment SGI237405[] = {{&S92457[67], 6}, {NULL}};
const Segment SGI237406[] = {{&S102262[44], 18}, {NULL}};
const Segment SGI237407[] = {{&S102262[44], 30}, {NULL}};
const Segment SGI237408[] = {{&S102262[30], 6}, {NULL}};
const Segment SGI237409[] = {{&S102262[31], 3}, {NULL}};
const Segment SGI237410[] = {{&S102262[117], 6}, {NULL}};
const Segment SGI237411[] = {{&S102262[76], 8}, {NULL}};
const Segment SGI237412[] = {{&S102262[88], 15}, {NULL}};
const Segment SGI237413[] = {{&S102269[34], 7}, {NULL}};
const Segment SGI237414[] = {{&S102269[27], 6}, {NULL}};
const Segment SGI237415[] = {{&S102269[94], 9}, {NULL}};
const Segment SGI237416[] = {{&S102269[67], 6}, {NULL}};
const Segment SGI237417[] = {{&S84437[34], 11}, {NULL}};
const Segment SGI237418[] = {{&S84437[27], 6}, {NULL}};
const Segment SGI237419[] = {{&S84437[98], 9}, {NULL}};
const Segment SGI237420[] = {{&S84437[71], 6}, {NULL}};
const Segment SGI237421[] = {{&S84444[44], 10}, {NULL}};
const Segment SGI237422[] = {{&S84444[44], 21}, {NULL}};
const Segment SGI237423[] = {{&S84444[30], 6}, {NULL}};
const Segment SGI237424[] = {{&S84444[112], 3}, {NULL}};
const Segment SGI237425[] = {{&S84444[79], 5}, {NULL}};
const Segment SGI237426[] = {{&S102282[45], 11}, {NULL}};
const Segment SGI237427[] = {{&S102282[30], 6}, {NULL}};
const Segment SGI237428[] = {{&S102282[31], 3}, {NULL}};
const Segment SGI237429[] = {{&S102282[82], 5}, {NULL}};
const Segment SGI237430[] = {{&S102282[140], 13}, {NULL}};
const Segment SGI237431[] = {{&S102283[45], 10}, {NULL}};
const Segment SGI237432[] = {{&S102283[30], 6}, {NULL}};
const Segment SGI237433[] = {{&S102283[31], 3}, {NULL}};
const Segment SGI237434[] = {{&S102283[81], 5}, {NULL}};
const Segment SGI237435[] = {{&S102283[139], 13}, {NULL}};
const Segment SGI237436[] = {{&S84455[38], 10}, {NULL}};
const Segment SGI237437[] = {{&S84455[31], 6}, {NULL}};
const Segment SGI237438[] = {{&S84456[44], 10}, {NULL}};
const Segment SGI237439[] = {{&S84456[44], 21}, {NULL}};
const Segment SGI237440[] = {{&S84456[30], 6}, {NULL}};
const Segment SGI237441[] = {{&S84456[114], 3}, {NULL}};
const Segment SGI237442[] = {{&S102288[44], 10}, {NULL}};
const Segment SGI237443[] = {{&S84456[79], 6}, {NULL}};
const Segment SGI237444[] = {{&S102288[44], 22}, {NULL}};
const Segment SGI237445[] = {{&S102288[30], 6}, {NULL}};
const Segment SGI237446[] = {{&S102288[31], 3}, {NULL}};
const Segment SGI237447[] = {{&S102288[80], 6}, {NULL}};
const Segment SGI237448[] = {{&S102288[132], 10}, {NULL}};
const Segment SGI237449[] = {{&S84459[34], 13}, {NULL}};
const Segment SGI237450[] = {{&S84459[27], 6}, {NULL}};
const Segment SGI237451[] = {{&S84459[100], 9}, {NULL}};
const Segment SGI237452[] = {{&S84459[73], 6}, {NULL}};
const Segment SGI237453[] = {{&S84460[34], 9}, {NULL}};
const Segment SGI237454[] = {{&S84460[27], 6}, {NULL}};
const Segment SGI237455[] = {{&S84460[95], 9}, {NULL}};
const Segment SGI237456[] = {{&S84460[68], 6}, {NULL}};
const Segment SGI237457[] = {{&S92525[44], 9}, {NULL}};
const Segment SGI237458[] = {{&S84461[34], 12}, {NULL}};
const Segment SGI237459[] = {{&S92525[44], 21}, {NULL}};
const Segment SGI237460[] = {{&S92525[30], 6}, {NULL}};
const Segment SGI237461[] = {{&S84461[27], 6}, {NULL}};
const Segment SGI237462[] = {{&S92525[135], 3}, {NULL}};
const Segment SGI237463[] = {{&S84461[99], 9}, {NULL}};
const Segment SGI237464[] = {{&S92525[100], 6}, {NULL}};
const Segment SGI237465[] = {{&S84461[72], 6}, {NULL}};
const Segment SGI237466[] = {{&S84468[34], 7}, {NULL}};
const Segment SGI237467[] = {{&S84468[27], 6}, {NULL}};
const Segment SGI237468[] = {{&S92533[34], 11}, {NULL}};
const Segment SGI237469[] = {{&S92533[27], 6}, {NULL}};
const Segment SGI237470[] = {{&S84468[95], 9}, {NULL}};
const Segment SGI237471[] = {{&S84468[68], 6}, {NULL}};
const Segment SGI237472[] = {{&S92533[98], 9}, {NULL}};
const Segment SGI237473[] = {{&S92533[71], 6}, {NULL}};
const Segment SGI237474[] = {{&S92731[44], 10}, {NULL}};
const Segment SGI237475[] = {{&S92731[44], 22}, {NULL}};
const Segment SGI237476[] = {{&S92731[30], 6}, {NULL}};
const Segment SGI237477[] = {{&S84470[34], 10}, {NULL}};
const Segment SGI237478[] = {{&S92731[133], 3}, {NULL}};
const Segment SGI237479[] = {{&S84470[27], 6}, {NULL}};
const Segment SGI237480[] = {{&S92731[98], 6}, {NULL}};
const Segment SGI237481[] = {{&S92731[68], 8}, {NULL}};
const Segment SGI237482[] = {{&S84470[97], 9}, {NULL}};
const Segment SGI237483[] = {{&S84470[70], 6}, {NULL}};
const Segment SGI237484[] = {{&S102307[34], 10}, {NULL}};
const Segment SGI237485[] = {{&S102307[27], 6}, {NULL}};
const Segment SGI237486[] = {{&S102307[97], 9}, {NULL}};
const Segment SGI237487[] = {{&S102307[70], 6}, {NULL}};
const Segment SGI237488[] = {{&S102307[132], 8}, {NULL}};
const Segment SGI237489[] = {{&S102313[11], 1}, {NULL}};
const Segment SGI237490[] = {{&S102313[44], 13}, {NULL}};
const Segment SGI237491[] = {{&S102313[30], 6}, {NULL}};
const Segment SGI237492[] = {{&S102313[31], 3}, {NULL}};
const Segment SGI237493[] = {{&S102313[100], 6}, {NULL}};
const Segment SGI237494[] = {{&S102313[59], 8}, {NULL}};
const Segment SGI237495[] = {{&S92547[34], 10}, {NULL}};
const Segment SGI237496[] = {{&S102313[71], 15}, {NULL}};
const Segment SGI237497[] = {{&S92547[27], 6}, {NULL}};
const Segment SGI237498[] = {{&S92547[97], 9}, {NULL}};
const Segment SGI237499[] = {{&S92547[70], 6}, {NULL}};
const Segment SGI237500[] = {{&S99115[45], 10}, {NULL}};
const Segment SGI237501[] = {{&S99115[30], 6}, {NULL}};
const Segment SGI237502[] = {{&S99115[120], 3}, {NULL}};
const Segment SGI237503[] = {{&S99115[86], 5}, {NULL}};
const Segment SGI237504[] = {{&S99115[144], 13}, {NULL}};
const Segment SGI237505[] = {{&S92553[34], 10}, {NULL}};
const Segment SGI237506[] = {{&S92553[27], 6}, {NULL}};
const Segment SGI237507[] = {{&S92553[97], 9}, {NULL}};
const Segment SGI237508[] = {{&S92553[70], 6}, {NULL}};
const Segment SGI237509[] = {{&S92757[34], 7}, {NULL}};
const Segment SGI237510[] = {{&S92757[27], 6}, {NULL}};
const Segment SGI237511[] = {{&S92757[94], 9}, {NULL}};
const Segment SGI237512[] = {{&S92757[67], 6}, {NULL}};
const Segment SGI237513[] = {{&S99132[44], 12}, {NULL}};
const Segment SGI237514[] = {{&S99132[44], 24}, {NULL}};
const Segment SGI237515[] = {{&S99132[30], 6}, {NULL}};
const Segment SGI237516[] = {{&S99132[146], 3}, {NULL}};
const Segment SGI237517[] = {{&S99132[111], 6}, {NULL}};
const Segment SGI237518[] = {{&S99132[70], 8}, {NULL}};
const Segment SGI237519[] = {{&S99132[82], 15}, {NULL}};
const Segment SGI237520[] = {{&S92574[34], 10}, {NULL}};
const Segment SGI237521[] = {{&S92574[27], 6}, {NULL}};
const Segment SGI237522[] = {{&S92574[97], 9}, {NULL}};
const Segment SGI237523[] = {{&S92574[70], 6}, {NULL}};
const Segment SGI237524[] = {{&S92773[34], 7}, {NULL}};
const Segment SGI237525[] = {{&S92773[27], 6}, {NULL}};
const Segment SGI237526[] = {{&S92773[94], 9}, {NULL}};
const Segment SGI237527[] = {{&S92773[67], 6}, {NULL}};
const Segment SGI237528[] = {{&S92775[34], 11}, {NULL}};
const Segment SGI237529[] = {{&S92775[27], 6}, {NULL}};
const Segment SGI237530[] = {{&S92775[98], 9}, {NULL}};
const Segment SGI237531[] = {{&S92775[71], 6}, {NULL}};
const Segment SGI237532[] = {{&S92779[44], 8}, {NULL}};
const Segment SGI237533[] = {{&S92779[44], 20}, {NULL}};
const Segment SGI237534[] = {{&S92779[30], 6}, {NULL}};
const Segment SGI237535[] = {{&S92779[113], 3}, {NULL}};
const Segment SGI237536[] = {{&S92779[78], 6}, {NULL}};
const Segment SGI237537[] = {{&S92588[34], 7}, {NULL}};
const Segment SGI237538[] = {{&S92588[27], 6}, {NULL}};
const Segment SGI237539[] = {{&S92588[94], 9}, {NULL}};
const Segment SGI237540[] = {{&S92588[67], 6}, {NULL}};
const Segment SGI237541[] = {{&S92786[34], 16}, {NULL}};
const Segment SGI237542[] = {{&S92589[34], 7}, {NULL}};
const Segment SGI237543[] = {{&S92786[27], 6}, {NULL}};
const Segment SGI237544[] = {{&S92589[27], 6}, {NULL}};
const Segment SGI237545[] = {{&S92786[103], 9}, {NULL}};
const Segment SGI237546[] = {{&S92589[94], 9}, {NULL}};
const Segment SGI237547[] = {{&S92786[76], 6}, {NULL}};
const Segment SGI237548[] = {{&S92589[67], 6}, {NULL}};
const Segment SGI237549[] = {{&S92595[44], 9}, {NULL}};
const Segment SGI237550[] = {{&S92595[44], 21}, {NULL}};
const Segment SGI237551[] = {{&S92595[30], 6}, {NULL}};
const Segment SGI237552[] = {{&S92595[79], 6}, {NULL}};
const Segment SGI237553[] = {{&S92595[106], 13}, {NULL}};
const Segment SGI237554[] = {{&S84542[34], 11}, {NULL}};
const Segment SGI237555[] = {{&S84542[27], 6}, {NULL}};
const Segment SGI237556[] = {{&S92798[44], 16}, {NULL}};
const Segment SGI237557[] = {{&S92798[44], 28}, {NULL}};
const Segment SGI237558[] = {{&S92798[30], 6}, {NULL}};
const Segment SGI237559[] = {{&S84542[97], 9}, {NULL}};
const Segment SGI237560[] = {{&S84542[70], 6}, {NULL}};
const Segment SGI237561[] = {{&S92798[121], 3}, {NULL}};
const Segment SGI237562[] = {{&S92798[86], 6}, {NULL}};
const Segment SGI237563[] = {{&S92798[156], 3}, {NULL}};
const Segment SGI237564[] = {{&S92602[44], 10}, {NULL}};
const Segment SGI237565[] = {{&S92602[44], 22}, {NULL}};
const Segment SGI237566[] = {{&S92602[30], 6}, {NULL}};
const Segment SGI237567[] = {{&S92602[113], 3}, {NULL}};
const Segment SGI237568[] = {{&S92602[80], 5}, {NULL}};
const Segment SGI237569[] = {{&S92602[117], 14}, {NULL}};
const Segment SGI237570[] = {{&S92605[44], 8}, {NULL}};
const Segment SGI237571[] = {{&S92605[44], 20}, {NULL}};
const Segment SGI237572[] = {{&S92605[30], 6}, {NULL}};
const Segment SGI237573[] = {{&S92605[113], 3}, {NULL}};
const Segment SGI237574[] = {{&S92605[78], 6}, {NULL}};
const Segment SGI237575[] = {{&S84556[44], 18}, {NULL}};
const Segment SGI237576[] = {{&S92811[34], 8}, {NULL}};
const Segment SGI237577[] = {{&S84556[44], 29}, {NULL}};
const Segment SGI237578[] = {{&S84556[30], 6}, {NULL}};
const Segment SGI237579[] = {{&S92811[27], 6}, {NULL}};
const Segment SGI237580[] = {{&S84556[87], 6}, {NULL}};
const Segment SGI237581[] = {{&S92811[95], 9}, {NULL}};
const Segment SGI237582[] = {{&S92811[68], 6}, {NULL}};
const Segment SGI237583[] = {{&S84556[114], 13}, {NULL}};
const Segment SGI237584[] = {{&S92811[129], 8}, {NULL}};
const Segment SGI237585[] = {{&S92614[44], 26}, {NULL}};
const Segment SGI237586[] = {{&S92614[44], 38}, {NULL}};
const Segment SGI237587[] = {{&S92614[30], 6}, {NULL}};
const Segment SGI237588[] = {{&S92614[131], 3}, {NULL}};
const Segment SGI237589[] = {{&S92614[96], 6}, {NULL}};
const Segment SGI237590[] = {{&S92619[34], 9}, {NULL}};
const Segment SGI237591[] = {{&S92619[27], 6}, {NULL}};
const Segment SGI237592[] = {{&S92619[96], 9}, {NULL}};
const Segment SGI237593[] = {{&S92619[69], 6}, {NULL}};
const Segment SGI237594[] = {{&S92626[34], 9}, {NULL}};
const Segment SGI237595[] = {{&S92626[27], 6}, {NULL}};
const Segment SGI237596[] = {{&S92626[96], 9}, {NULL}};
const Segment SGI237597[] = {{&S92626[69], 6}, {NULL}};
const Segment SGI237598[] = {{&S92630[44], 9}, {NULL}};
const Segment SGI237599[] = {{&S92630[44], 21}, {NULL}};
const Segment SGI237600[] = {{&S92630[30], 6}, {NULL}};
const Segment SGI237601[] = {{&S92630[79], 6}, {NULL}};
const Segment SGI237602[] = {{&S92630[106], 13}, {NULL}};
const Segment SGI237603[] = {{&S92636[34], 12}, {NULL}};
const Segment SGI237604[] = {{&S92636[27], 6}, {NULL}};
const Segment SGI237605[] = {{&S92636[99], 9}, {NULL}};
const Segment SGI237606[] = {{&S92636[72], 6}, {NULL}};
const Segment SGI237607[] = {{&S92642[44], 6}, {NULL}};
const Segment SGI237608[] = {{&S92642[44], 18}, {NULL}};
const Segment SGI237609[] = {{&S92642[30], 6}, {NULL}};
const Segment SGI237610[] = {{&S92642[76], 6}, {NULL}};
const Segment SGI237611[] = {{&S92642[103], 13}, {NULL}};
const Segment SGI237612[] = {{&S99230[34], 11}, {NULL}};
const Segment SGI237613[] = {{&S99230[27], 6}, {NULL}};
const Segment SGI237614[] = {{&S99230[98], 9}, {NULL}};
const Segment SGI237615[] = {{&S99230[71], 6}, {NULL}};
const Segment SGI237616[] = {{&S84618[34], 7}, {NULL}};
const Segment SGI237617[] = {{&S84618[27], 6}, {NULL}};
const Segment SGI237618[] = {{&S84618[93], 9}, {NULL}};
const Segment SGI237619[] = {{&S84618[66], 6}, {NULL}};
const Segment SGI237620[] = {{&S99248[34], 13}, {NULL}};
const Segment SGI237621[] = {{&S99248[27], 6}, {NULL}};
const Segment SGI237622[] = {{&S99248[100], 9}, {NULL}};
const Segment SGI237623[] = {{&S99248[73], 6}, {NULL}};
const Segment SGI237624[] = {{&S99252[34], 10}, {NULL}};
const Segment SGI237625[] = {{&S99252[27], 6}, {NULL}};
const Segment SGI237626[] = {{&S99252[97], 9}, {NULL}};
const Segment SGI237627[] = {{&S99252[70], 6}, {NULL}};
const Segment SGI237628[] = {{&S99252[131], 8}, {NULL}};
const Segment SGI237629[] = {{&S99259[34], 15}, {NULL}};
const Segment SGI237630[] = {{&S99259[27], 6}, {NULL}};
const Segment SGI237631[] = {{&S99259[102], 9}, {NULL}};
const Segment SGI237632[] = {{&S99259[75], 6}, {NULL}};
const Segment SGI237633[] = {{&S99259[143], 8}, {NULL}};
const Segment SGI237634[] = {{&S99261[34], 6}, {NULL}};
const Segment SGI237635[] = {{&S99261[27], 6}, {NULL}};
const Segment SGI237636[] = {{&S99261[93], 9}, {NULL}};
const Segment SGI237637[] = {{&S99261[66], 6}, {NULL}};
const Segment SGI237638[] = {{&S99272[34], 7}, {NULL}};
const Segment SGI237639[] = {{&S99272[27], 6}, {NULL}};
const Segment SGI237640[] = {{&S99272[99], 9}, {NULL}};
const Segment SGI237641[] = {{&S99272[72], 6}, {NULL}};
const Segment SGI237642[] = {{&S99272[140], 8}, {NULL}};
const Segment SGI237643[] = {{&S99275[34], 18}, {NULL}};
const Segment SGI237644[] = {{&S99275[27], 6}, {NULL}};
const Segment SGI237645[] = {{&S99275[105], 9}, {NULL}};
const Segment SGI237646[] = {{&S99275[78], 6}, {NULL}};
const Segment SGI237647[] = {{&S102502[59], 10}, {NULL}};
const Segment SGI237648[] = {{&S102502[59], 22}, {NULL}};
const Segment SGI237649[] = {{&S102502[45], 6}, {NULL}};
const Segment SGI237650[] = {{&S84643[44], 10}, {NULL}};
const Segment SGI237651[] = {{&S84643[44], 21}, {NULL}};
const Segment SGI237652[] = {{&S84643[30], 6}, {NULL}};
const Segment SGI237653[] = {{&S102502[46], 3}, {NULL}};
const Segment SGI237654[] = {{&S84643[114], 3}, {NULL}};
const Segment SGI237655[] = {{&S102502[95], 6}, {NULL}};
const Segment SGI237656[] = {{&S84643[79], 6}, {NULL}};
const Segment SGI237657[] = {{&S102502[0], 14}, {NULL}};
const Segment SGI237658[] = {{&S84644[44], 21}, {NULL}};
const Segment SGI237659[] = {{&S84644[30], 6}, {NULL}};
const Segment SGI237660[] = {{&S84644[116], 3}, {NULL}};
const Segment SGI237661[] = {{&S84644[81], 6}, {NULL}};
const Segment SGI237662[] = {{&S102509[44], 10}, {NULL}};
const Segment SGI237663[] = {{&S102509[44], 22}, {NULL}};
const Segment SGI237664[] = {{&S102509[30], 6}, {NULL}};
const Segment SGI237665[] = {{&S102509[31], 3}, {NULL}};
const Segment SGI237666[] = {{&S102509[80], 6}, {NULL}};
const Segment SGI237667[] = {{&S102509[133], 11}, {NULL}};
const Segment SGI237668[] = {{&S84654[44], 7}, {NULL}};
const Segment SGI237669[] = {{&S84654[44], 20}, {NULL}};
const Segment SGI237670[] = {{&S84654[30], 6}, {NULL}};
const Segment SGI237671[] = {{&S84654[113], 3}, {NULL}};
const Segment SGI237672[] = {{&S84654[78], 6}, {NULL}};
const Segment SGI237673[] = {{&S102513[45], 10}, {NULL}};
const Segment SGI237674[] = {{&S102513[30], 6}, {NULL}};
const Segment SGI237675[] = {{&S102513[31], 3}, {NULL}};
const Segment SGI237676[] = {{&S102513[81], 5}, {NULL}};
const Segment SGI237677[] = {{&S102513[139], 13}, {NULL}};
const Segment SGI237678[] = {{&S84659[34], 10}, {NULL}};
const Segment SGI237679[] = {{&S84659[27], 6}, {NULL}};
const Segment SGI237680[] = {{&S84659[96], 9}, {NULL}};
const Segment SGI237681[] = {{&S84659[69], 6}, {NULL}};
const Segment SGI237682[] = {{&S99298[44], 15}, {NULL}};
const Segment SGI237683[] = {{&S99298[44], 34}, {NULL}};
const Segment SGI237684[] = {{&S84659[137], 8}, {NULL}};
const Segment SGI237685[] = {{&S99298[30], 6}, {NULL}};
const Segment SGI237686[] = {{&S99298[127], 3}, {NULL}};
const Segment SGI237687[] = {{&S99298[92], 6}, {NULL}};
const Segment SGI237688[] = {{&S84660[34], 10}, {NULL}};
const Segment SGI237689[] = {{&S84660[27], 6}, {NULL}};
const Segment SGI237690[] = {{&S102517[31], 8}, {NULL}};
const Segment SGI237691[] = {{&S102517[55], 3}, {NULL}};
const Segment SGI237692[] = {{&S84660[96], 9}, {NULL}};
const Segment SGI237693[] = {{&S84660[69], 6}, {NULL}};
const Segment SGI237694[] = {{&S99299[59], 11}, {NULL}};
const Segment SGI237695[] = {{&S99299[59], 23}, {NULL}};
const Segment SGI237696[] = {{&S99299[45], 6}, {NULL}};
const Segment SGI237697[] = {{&S99299[131], 3}, {NULL}};
const Segment SGI237698[] = {{&S99299[96], 6}, {NULL}};
const Segment SGI237699[] = {{&S99299[0], 14}, {NULL}};
const Segment SGI237700[] = {{&S102518[45], 10}, {NULL}};
const Segment SGI237701[] = {{&S102518[30], 6}, {NULL}};
const Segment SGI237702[] = {{&S102518[31], 3}, {NULL}};
const Segment SGI237703[] = {{&S102518[81], 5}, {NULL}};
const Segment SGI237704[] = {{&S102518[139], 13}, {NULL}};
const Segment SGI237705[] = {{&S99324[44], 7}, {NULL}};
const Segment SGI237706[] = {{&S99324[44], 19}, {NULL}};
const Segment SGI237707[] = {{&S99324[30], 6}, {NULL}};
const Segment SGI237708[] = {{&S99324[110], 3}, {NULL}};
const Segment SGI237709[] = {{&S99324[77], 5}, {NULL}};
const Segment SGI237710[] = {{&S99324[114], 14}, {NULL}};
const Segment SGI237711[] = {{&S99326[44], 11}, {NULL}};
const Segment SGI237712[] = {{&S99326[44], 23}, {NULL}};
const Segment SGI237713[] = {{&S99326[30], 6}, {NULL}};
const Segment SGI237714[] = {{&S99326[114], 3}, {NULL}};
const Segment SGI237715[] = {{&S99326[81], 5}, {NULL}};
const Segment SGI237716[] = {{&S99326[118], 14}, {NULL}};
const Segment SGI237717[] = {{&S99327[34], 13}, {NULL}};
const Segment SGI237718[] = {{&S99327[27], 6}, {NULL}};
const Segment SGI237719[] = {{&S99327[100], 9}, {NULL}};
const Segment SGI237720[] = {{&S99327[73], 6}, {NULL}};
const Segment SGI237721[] = {{&S99328[44], 11}, {NULL}};
const Segment SGI237722[] = {{&S99328[44], 23}, {NULL}};
const Segment SGI237723[] = {{&S99328[30], 6}, {NULL}};
const Segment SGI237724[] = {{&S99328[114], 3}, {NULL}};
const Segment SGI237725[] = {{&S99328[81], 5}, {NULL}};
const Segment SGI237726[] = {{&S99328[118], 14}, {NULL}};
const Segment SGI237727[] = {{&S99331[44], 11}, {NULL}};
const Segment SGI237728[] = {{&S99331[44], 23}, {NULL}};
const Segment SGI237729[] = {{&S99331[30], 6}, {NULL}};
const Segment SGI237730[] = {{&S99331[81], 6}, {NULL}};
const Segment SGI237731[] = {{&S99331[108], 13}, {NULL}};
const Segment SGI237732[] = {{&S99332[34], 16}, {NULL}};
const Segment SGI237733[] = {{&S99332[27], 6}, {NULL}};
const Segment SGI237734[] = {{&S99332[113], 9}, {NULL}};
const Segment SGI237735[] = {{&S99332[86], 6}, {NULL}};
const Segment SGI237736[] = {{&S92985[34], 21}, {NULL}};
const Segment SGI237737[] = {{&S92985[27], 6}, {NULL}};
const Segment SGI237738[] = {{&S99344[34], 7}, {NULL}};
const Segment SGI237739[] = {{&S92985[108], 9}, {NULL}};
const Segment SGI237740[] = {{&S99344[27], 6}, {NULL}};
const Segment SGI237741[] = {{&S92985[81], 6}, {NULL}};
const Segment SGI237742[] = {{&S99344[94], 9}, {NULL}};
const Segment SGI237743[] = {{&S99344[67], 6}, {NULL}};
const Segment SGI237744[] = {{&S84727[45], 14}, {NULL}};
const Segment SGI237745[] = {{&S84727[30], 6}, {NULL}};
const Segment SGI237746[] = {{&S84727[118], 3}, {NULL}};
const Segment SGI237747[] = {{&S84727[84], 5}, {NULL}};
const Segment SGI237748[] = {{&S84727[142], 13}, {NULL}};
const Segment SGI237749[] = {{&S99354[44], 9}, {NULL}};
const Segment SGI237750[] = {{&S99354[44], 21}, {NULL}};
const Segment SGI237751[] = {{&S99354[30], 6}, {NULL}};
const Segment SGI237752[] = {{&S99354[114], 3}, {NULL}};
const Segment SGI237753[] = {{&S99354[79], 6}, {NULL}};
const Segment SGI237754[] = {{&S93000[44], 11}, {NULL}};
const Segment SGI237755[] = {{&S93000[44], 23}, {NULL}};
const Segment SGI237756[] = {{&S93000[30], 6}, {NULL}};
const Segment SGI237757[] = {{&S93000[114], 3}, {NULL}};
const Segment SGI237758[] = {{&S93000[81], 5}, {NULL}};
const Segment SGI237759[] = {{&S92711[34], 13}, {NULL}};
const Segment SGI237760[] = {{&S92711[27], 6}, {NULL}};
const Segment SGI237761[] = {{&S92711[100], 9}, {NULL}};
const Segment SGI237762[] = {{&S92711[73], 6}, {NULL}};
const Segment SGI237763[] = {{&S84743[34], 11}, {NULL}};
const Segment SGI237764[] = {{&S84743[27], 6}, {NULL}};
const Segment SGI237765[] = {{&S93009[52], 15}, {NULL}};
const Segment SGI237766[] = {{&S93009[52], 27}, {NULL}};
const Segment SGI237767[] = {{&S84743[97], 9}, {NULL}};
const Segment SGI237768[] = {{&S93009[38], 6}, {NULL}};
const Segment SGI237769[] = {{&S84743[70], 6}, {NULL}};
const Segment SGI237770[] = {{&S99369[44], 10}, {NULL}};
const Segment SGI237771[] = {{&S93009[93], 5}, {NULL}};
const Segment SGI237772[] = {{&S99369[44], 27}, {NULL}};
const Segment SGI237773[] = {{&S99369[30], 6}, {NULL}};
const Segment SGI237774[] = {{&S99369[85], 6}, {NULL}};
const Segment SGI237775[] = {{&S99369[112], 13}, {NULL}};
const Segment SGI237776[] = {{&S92720[34], 10}, {NULL}};
const Segment SGI237777[] = {{&S92720[27], 6}, {NULL}};
const Segment SGI237778[] = {{&S84745[44], 10}, {NULL}};
const Segment SGI237779[] = {{&S84745[44], 21}, {NULL}};
const Segment SGI237780[] = {{&S84745[30], 6}, {NULL}};
const Segment SGI237781[] = {{&S92720[97], 9}, {NULL}};
const Segment SGI237782[] = {{&S92720[70], 6}, {NULL}};
const Segment SGI237783[] = {{&S84745[132], 3}, {NULL}};
const Segment SGI237784[] = {{&S93011[44], 11}, {NULL}};
const Segment SGI237785[] = {{&S93011[44], 23}, {NULL}};
const Segment SGI237786[] = {{&S84745[97], 6}, {NULL}};
const Segment SGI237787[] = {{&S93011[30], 6}, {NULL}};
const Segment SGI237788[] = {{&S84745[67], 8}, {NULL}};
const Segment SGI237789[] = {{&S93011[116], 3}, {NULL}};
const Segment SGI237790[] = {{&S93011[81], 6}, {NULL}};
const Segment SGI237791[] = {{&S84746[44], 10}, {NULL}};
const Segment SGI237792[] = {{&S84746[44], 22}, {NULL}};
const Segment SGI237793[] = {{&S84746[30], 6}, {NULL}};
const Segment SGI237794[] = {{&S84746[115], 3}, {NULL}};
const Segment SGI237795[] = {{&S84746[80], 6}, {NULL}};
const Segment SGI237796[] = {{&S84746[140], 8}, {NULL}};
const Segment SGI237797[] = {{&S84749[34], 7}, {NULL}};
const Segment SGI237798[] = {{&S84749[27], 6}, {NULL}};
const Segment SGI237799[] = {{&S92726[45], 9}, {NULL}};
const Segment SGI237800[] = {{&S92726[30], 6}, {NULL}};
const Segment SGI237801[] = {{&S84749[94], 9}, {NULL}};
const Segment SGI237802[] = {{&S84749[67], 6}, {NULL}};
const Segment SGI237803[] = {{&S93020[34], 8}, {NULL}};
const Segment SGI237804[] = {{&S93020[27], 6}, {NULL}};
const Segment SGI237805[] = {{&S93020[95], 9}, {NULL}};
const Segment SGI237806[] = {{&S93020[68], 6}, {NULL}};
const Segment SGI237807[] = {{&S99389[44], 7}, {NULL}};
const Segment SGI237808[] = {{&S99389[44], 19}, {NULL}};
const Segment SGI237809[] = {{&S99389[30], 6}, {NULL}};
const Segment SGI237810[] = {{&S99389[112], 3}, {NULL}};
const Segment SGI237811[] = {{&S99389[77], 6}, {NULL}};
const Segment SGI237812[] = {{&S99391[44], 9}, {NULL}};
const Segment SGI237813[] = {{&S99391[44], 21}, {NULL}};
const Segment SGI237814[] = {{&S99391[30], 6}, {NULL}};
const Segment SGI237815[] = {{&S99391[114], 3}, {NULL}};
const Segment SGI237816[] = {{&S99391[79], 6}, {NULL}};
const Segment SGI237817[] = {{&S84766[34], 7}, {NULL}};
const Segment SGI237818[] = {{&S84766[27], 6}, {NULL}};
const Segment SGI237819[] = {{&S84766[94], 9}, {NULL}};
const Segment SGI237820[] = {{&S84766[67], 6}, {NULL}};
const Segment SGI237821[] = {{&S93044[34], 14}, {NULL}};
const Segment SGI237822[] = {{&S93044[27], 6}, {NULL}};
const Segment SGI237823[] = {{&S93044[101], 9}, {NULL}};
const Segment SGI237824[] = {{&S93044[74], 6}, {NULL}};
const Segment SGI237825[] = {{&S102645[44], 7}, {NULL}};
const Segment SGI237826[] = {{&S102645[44], 17}, {NULL}};
const Segment SGI237827[] = {{&S102645[30], 6}, {NULL}};
const Segment SGI237828[] = {{&S102645[31], 3}, {NULL}};
const Segment SGI237829[] = {{&S102645[75], 6}, {NULL}};
const Segment SGI237830[] = {{&S99412[44], 18}, {NULL}};
const Segment SGI237831[] = {{&S99412[44], 39}, {NULL}};
const Segment SGI237832[] = {{&S99412[30], 6}, {NULL}};
const Segment SGI237833[] = {{&S99412[130], 3}, {NULL}};
const Segment SGI237834[] = {{&S99412[97], 5}, {NULL}};
const Segment SGI237835[] = {{&S99412[134], 14}, {NULL}};
const Segment SGI237836[] = {{&S84806[34], 12}, {NULL}};
const Segment SGI237837[] = {{&S84806[27], 6}, {NULL}};
const Segment SGI237838[] = {{&S84806[98], 9}, {NULL}};
const Segment SGI237839[] = {{&S84806[71], 6}, {NULL}};
const Segment SGI237840[] = {{&S84815[44], 7}, {NULL}};
const Segment SGI237841[] = {{&S84815[44], 25}, {NULL}};
const Segment SGI237842[] = {{&S84815[30], 6}, {NULL}};
const Segment SGI237843[] = {{&S84815[118], 3}, {NULL}};
const Segment SGI237844[] = {{&S84815[83], 6}, {NULL}};
const Segment SGI237845[] = {{&S84816[38], 7}, {NULL}};
const Segment SGI237846[] = {{&S84816[31], 6}, {NULL}};
const Segment SGI237847[] = {{&S84826[59], 10}, {NULL}};
const Segment SGI237848[] = {{&S84826[59], 21}, {NULL}};
const Segment SGI237849[] = {{&S84826[45], 6}, {NULL}};
const Segment SGI237850[] = {{&S84826[129], 3}, {NULL}};
const Segment SGI237851[] = {{&S84826[94], 6}, {NULL}};
const Segment SGI237852[] = {{&S102694[45], 10}, {NULL}};
const Segment SGI237853[] = {{&S102694[30], 6}, {NULL}};
const Segment SGI237854[] = {{&S102694[31], 3}, {NULL}};
const Segment SGI237855[] = {{&S102694[81], 5}, {NULL}};
const Segment SGI237856[] = {{&S102694[139], 13}, {NULL}};
const Segment SGI237857[] = {{&S102695[45], 10}, {NULL}};
const Segment SGI237858[] = {{&S102695[30], 6}, {NULL}};
const Segment SGI237859[] = {{&S102695[31], 3}, {NULL}};
const Segment SGI237860[] = {{&S102695[81], 5}, {NULL}};
const Segment SGI237861[] = {{&S102695[139], 13}, {NULL}};
const Segment SGI237862[] = {{&S104018[42], 9}, {NULL}};
const Segment SGI237863[] = {{&S104018[42], 21}, {NULL}};
const Segment SGI237864[] = {{&S104018[30], 4}, {NULL}};
const Segment SGI237865[] = {{&S104018[112], 3}, {NULL}};
const Segment SGI237866[] = {{&S104018[77], 6}, {NULL}};
const Segment SGI237867[] = {{&S104018[140], 3}, {NULL}};
const Segment SGI237868[] = {{&S102697[44], 10}, {NULL}};
const Segment SGI237869[] = {{&S102697[44], 22}, {NULL}};
const Segment SGI237870[] = {{&S84836[34], 7}, {NULL}};
const Segment SGI237871[] = {{&S102697[30], 6}, {NULL}};
const Segment SGI237872[] = {{&S84836[27], 6}, {NULL}};
const Segment SGI237873[] = {{&S102697[139], 4}, {NULL}};
const Segment SGI237874[] = {{&S84836[93], 9}, {NULL}};
const Segment SGI237875[] = {{&S102697[31], 3}, {&S102697[192], 5}, {NULL}};
const Segment SGI237876[] = {{&S84836[66], 6}, {NULL}};
const Segment SGI237877[] = {{&S102697[80], 6}, {NULL}};
const Segment SGI237878[] = {{&S102702[45], 10}, {NULL}};
const Segment SGI237879[] = {{&S102702[30], 6}, {NULL}};
const Segment SGI237880[] = {{&S102702[31], 3}, {NULL}};
const Segment SGI237881[] = {{&S102702[81], 5}, {NULL}};
const Segment SGI237882[] = {{&S102702[139], 13}, {NULL}};
const Segment SGI237883[] = {{&S102703[44], 10}, {NULL}};
const Segment SGI237884[] = {{&S102703[44], 22}, {NULL}};
const Segment SGI237885[] = {{&S102703[30], 6}, {NULL}};
const Segment SGI237886[] = {{&S102703[111], 2}, {NULL}};
const Segment SGI237887[] = {{&S102703[80], 5}, {NULL}};
const Segment SGI237888[] = {{&S102706[44], 10}, {NULL}};
const Segment SGI237889[] = {{&S102706[44], 22}, {NULL}};
const Segment SGI237890[] = {{&S102706[30], 6}, {NULL}};
const Segment SGI237891[] = {{&S102706[31], 3}, {NULL}};
const Segment SGI237892[] = {{&S102706[80], 6}, {NULL}};
const Segment SGI237893[] = {{&S102706[143], 3}, {NULL}};
const Segment SGI237894[] = {{&S102709[44], 10}, {NULL}};
const Segment SGI237895[] = {{&S102709[44], 22}, {NULL}};
const Segment SGI237896[] = {{&S102709[30], 6}, {NULL}};
const Segment SGI237897[] = {{&S102709[31], 3}, {NULL}};
const Segment SGI237898[] = {{&S102709[80], 6}, {NULL}};
const Segment SGI237899[] = {{&S102709[133], 13}, {NULL}};
const Segment SGI237900[] = {{&S102714[44], 10}, {NULL}};
const Segment SGI237901[] = {{&S102714[44], 22}, {NULL}};
const Segment SGI237902[] = {{&S102714[30], 6}, {NULL}};
const Segment SGI237903[] = {{&S102714[80], 6}, {NULL}};
const Segment SGI237904[] = {{&S102714[107], 13}, {NULL}};
const Segment SGI237905[] = {{&S92726[114], 3}, {NULL}};
const Segment SGI237906[] = {{&S92726[80], 5}, {NULL}};
const Segment SGI237907[] = {{&S92726[138], 13}, {NULL}};
const Segment SGI237908[] = {{&S92734[34], 12}, {NULL}};
const Segment SGI237909[] = {{&S92734[27], 6}, {NULL}};
const Segment SGI237910[] = {{&S92734[99], 9}, {NULL}};
const Segment SGI237911[] = {{&S92734[72], 6}, {NULL}};
const Segment SGI237912[] = {{&S99514[44], 6}, {NULL}};
const Segment SGI237913[] = {{&S99514[44], 18}, {NULL}};
const Segment SGI237914[] = {{&S99514[30], 6}, {NULL}};
const Segment SGI237915[] = {{&S99514[76], 6}, {NULL}};
const Segment SGI237916[] = {{&S99514[103], 13}, {NULL}};
const Segment SGI237917[] = {{&S92744[44], 8}, {NULL}};
const Segment SGI237918[] = {{&S92744[44], 30}, {NULL}};
const Segment SGI237919[] = {{&S92744[30], 6}, {NULL}};
const Segment SGI237920[] = {{&S92744[123], 3}, {NULL}};
const Segment SGI237921[] = {{&S92744[88], 6}, {NULL}};
const Segment SGI237922[] = {{&S92744[148], 11}, {NULL}};
const Segment SGI237923[] = {{&S92746[34], 10}, {NULL}};
const Segment SGI237924[] = {{&S92746[27], 6}, {NULL}};
const Segment SGI237925[] = {{&S92746[97], 9}, {NULL}};
const Segment SGI237926[] = {{&S92746[70], 6}, {NULL}};
const Segment SGI237927[] = {{&S92748[34], 7}, {NULL}};
const Segment SGI237928[] = {{&S92748[27], 6}, {NULL}};
const Segment SGI237929[] = {{&S99521[34], 6}, {NULL}};
const Segment SGI237930[] = {{&S99521[27], 6}, {NULL}};
const Segment SGI237931[] = {{&S92748[102], 9}, {NULL}};
const Segment SGI237932[] = {{&S92748[75], 6}, {NULL}};
const Segment SGI237933[] = {{&S99521[93], 9}, {NULL}};
const Segment SGI237934[] = {{&S99521[66], 6}, {NULL}};
const Segment SGI237935[] = {{&S99522[34], 12}, {NULL}};
const Segment SGI237936[] = {{&S99522[27], 6}, {NULL}};
const Segment SGI237937[] = {{&S99522[99], 9}, {NULL}};
const Segment SGI237938[] = {{&S99522[72], 6}, {NULL}};
const Segment SGI237939[] = {{&S99522[140], 8}, {NULL}};
const Segment SGI237940[] = {{&S92761[34], 9}, {NULL}};
const Segment SGI237941[] = {{&S92761[27], 6}, {NULL}};
const Segment SGI237942[] = {{&S92761[96], 9}, {NULL}};
const Segment SGI237943[] = {{&S92761[69], 6}, {NULL}};
const Segment SGI237944[] = {{&S92761[137], 8}, {NULL}};
const Segment SGI237945[] = {{&S92766[44], 14}, {NULL}};
const Segment SGI237946[] = {{&S92766[44], 31}, {NULL}};
const Segment SGI237947[] = {{&S92766[30], 6}, {NULL}};
const Segment SGI237948[] = {{&S92766[89], 6}, {NULL}};
const Segment SGI237949[] = {{&S92766[116], 13}, {NULL}};
const Segment SGI237950[] = {{&S84925[34], 12}, {NULL}};
const Segment SGI237951[] = {{&S84925[27], 6}, {NULL}};
const Segment SGI237952[] = {{&S84925[98], 9}, {NULL}};
const Segment SGI237953[] = {{&S84925[71], 6}, {NULL}};
const Segment SGI237954[] = {{&S93207[44], 29}, {NULL}};
const Segment SGI237955[] = {{&S93207[44], 41}, {NULL}};
const Segment SGI237956[] = {{&S93207[30], 6}, {NULL}};
const Segment SGI237957[] = {{&S93207[134], 3}, {NULL}};
const Segment SGI237958[] = {{&S93207[99], 6}, {NULL}};
const Segment SGI237959[] = {{&S92781[34], 17}, {NULL}};
const Segment SGI237960[] = {{&S92781[27], 6}, {NULL}};
const Segment SGI237961[] = {{&S92781[112], 9}, {NULL}};
const Segment SGI237962[] = {{&S92781[85], 6}, {NULL}};
const Segment SGI237963[] = {{&S92788[44], 11}, {NULL}};
const Segment SGI237964[] = {{&S92788[44], 23}, {NULL}};
const Segment SGI237965[] = {{&S92788[30], 6}, {NULL}};
const Segment SGI237966[] = {{&S92788[114], 3}, {NULL}};
const Segment SGI237967[] = {{&S92788[81], 5}, {NULL}};
const Segment SGI237968[] = {{&S92788[118], 14}, {NULL}};
const Segment SGI237969[] = {{&S92803[45], 9}, {NULL}};
const Segment SGI237970[] = {{&S92803[30], 6}, {NULL}};
const Segment SGI237971[] = {{&S92803[114], 3}, {NULL}};
const Segment SGI237972[] = {{&S92803[80], 5}, {NULL}};
const Segment SGI237973[] = {{&S92803[138], 13}, {NULL}};
const Segment SGI237974[] = {{&S92804[44], 10}, {NULL}};
const Segment SGI237975[] = {{&S92804[44], 29}, {NULL}};
const Segment SGI237976[] = {{&S92804[30], 6}, {NULL}};
const Segment SGI237977[] = {{&S92804[122], 3}, {NULL}};
const Segment SGI237978[] = {{&S92804[87], 6}, {NULL}};
const Segment SGI237979[] = {{&S93237[34], 5}, {NULL}};
const Segment SGI237980[] = {{&S93237[27], 6}, {NULL}};
const Segment SGI237981[] = {{&S92805[44], 10}, {NULL}};
const Segment SGI237982[] = {{&S92805[44], 22}, {NULL}};
const Segment SGI237983[] = {{&S92805[30], 6}, {NULL}};
const Segment SGI237984[] = {{&S93237[92], 9}, {NULL}};
const Segment SGI237985[] = {{&S93237[65], 6}, {NULL}};
const Segment SGI237986[] = {{&S92805[115], 3}, {NULL}};
const Segment SGI237987[] = {{&S92805[80], 6}, {NULL}};
const Segment SGI237988[] = {{&S93241[38], 6}, {NULL}};
const Segment SGI237989[] = {{&S93241[31], 6}, {NULL}};
const Segment SGI237990[] = {{&S93242[44], 6}, {NULL}};
const Segment SGI237991[] = {{&S93242[44], 18}, {NULL}};
const Segment SGI237992[] = {{&S93242[30], 6}, {NULL}};
const Segment SGI237993[] = {{&S93242[111], 3}, {NULL}};
const Segment SGI237994[] = {{&S93242[76], 6}, {NULL}};
const Segment SGI237995[] = {{&S102837[44], 9}, {NULL}};
const Segment SGI237996[] = {{&S102837[44], 21}, {NULL}};
const Segment SGI237997[] = {{&S102837[30], 6}, {NULL}};
const Segment SGI237998[] = {{&S102837[110], 2}, {NULL}};
const Segment SGI237999[] = {{&S102837[79], 5}, {NULL}};
const Segment SGI238000[] = {{&S92817[44], 9}, {NULL}};
const Segment SGI238001[] = {{&S84989[34], 7}, {NULL}};
const Segment SGI238002[] = {{&S84989[27], 6}, {NULL}};
const Segment SGI238003[] = {{&S84989[93], 9}, {NULL}};
const Segment SGI238004[] = {{&S84989[66], 6}, {NULL}};
const Segment SGI238005[] = {{&S85010[34], 10}, {NULL}};
const Segment SGI238006[] = {{&S85010[27], 6}, {NULL}};
const Segment SGI238007[] = {{&S85010[96], 9}, {NULL}};
const Segment SGI238008[] = {{&S85010[69], 6}, {NULL}};
const Segment SGI238009[] = {{&S85010[137], 8}, {NULL}};
const Segment SGI238010[] = {{&S85021[34], 10}, {NULL}};
const Segment SGI238011[] = {{&S85021[27], 6}, {NULL}};
const Segment SGI238012[] = {{&S85021[96], 9}, {NULL}};
const Segment SGI238013[] = {{&S85021[69], 6}, {NULL}};
const Segment SGI238014[] = {{&S102902[44], 10}, {NULL}};
const Segment SGI238015[] = {{&S102902[44], 22}, {NULL}};
const Segment SGI238016[] = {{&S102902[30], 6}, {NULL}};
const Segment SGI238017[] = {{&S102902[31], 3}, {NULL}};
const Segment SGI238018[] = {{&S102902[109], 6}, {NULL}};
const Segment SGI238019[] = {{&S102902[68], 8}, {NULL}};
const Segment SGI238020[] = {{&S102902[80], 15}, {NULL}};
const Segment SGI238021[] = {{&S102905[44], 10}, {NULL}};
const Segment SGI238022[] = {{&S102905[44], 22}, {NULL}};
const Segment SGI238023[] = {{&S102905[30], 6}, {NULL}};
const Segment SGI238024[] = {{&S102905[31], 3}, {NULL}};
const Segment SGI238025[] = {{&S102905[111], 6}, {NULL}};
const Segment SGI238026[] = {{&S102905[68], 8}, {NULL}};
const Segment SGI238027[] = {{&S99692[44], 9}, {NULL}};
const Segment SGI238028[] = {{&S99692[44], 16}, {NULL}};
const Segment SGI238029[] = {{&S99692[30], 6}, {NULL}};
const Segment SGI238030[] = {{&S99692[109], 3}, {NULL}};
const Segment SGI238031[] = {{&S99692[74], 6}, {NULL}};
const Segment SGI238032[] = {{&S93348[34], 10}, {NULL}};
const Segment SGI238033[] = {{&S93348[27], 6}, {NULL}};
const Segment SGI238034[] = {{&S93348[97], 9}, {NULL}};
const Segment SGI238035[] = {{&S93348[70], 6}, {NULL}};
const Segment SGI238036[] = {{&S102917[45], 10}, {NULL}};
const Segment SGI238037[] = {{&S102917[30], 6}, {NULL}};
const Segment SGI238038[] = {{&S102917[31], 3}, {NULL}};
const Segment SGI238039[] = {{&S102917[81], 5}, {NULL}};
const Segment SGI238040[] = {{&S102917[139], 13}, {NULL}};
const Segment SGI238041[] = {{&S102919[45], 10}, {NULL}};
const Segment SGI238042[] = {{&S102919[30], 6}, {NULL}};
const Segment SGI238043[] = {{&S102919[31], 3}, {NULL}};
const Segment SGI238044[] = {{&S102919[81], 5}, {NULL}};
const Segment SGI238045[] = {{&S102919[139], 13}, {NULL}};
const Segment SGI238046[] = {{&S102920[45], 11}, {NULL}};
const Segment SGI238047[] = {{&S102920[30], 6}, {NULL}};
const Segment SGI238048[] = {{&S102920[31], 3}, {NULL}};
const Segment SGI238049[] = {{&S102920[82], 5}, {NULL}};
const Segment SGI238050[] = {{&S102920[140], 13}, {NULL}};
const Segment SGI238051[] = {{&S102921[34], 10}, {NULL}};
const Segment SGI238052[] = {{&S102921[27], 6}, {NULL}};
const Segment SGI238053[] = {{&S102921[97], 9}, {NULL}};
const Segment SGI238054[] = {{&S102921[70], 6}, {NULL}};
const Segment SGI238055[] = {{&S99711[34], 10}, {NULL}};
const Segment SGI238056[] = {{&S99711[27], 6}, {NULL}};
const Segment SGI238057[] = {{&S99711[97], 9}, {NULL}};
const Segment SGI238058[] = {{&S99711[70], 6}, {NULL}};
const Segment SGI238059[] = {{&S102925[45], 10}, {NULL}};
const Segment SGI238060[] = {{&S102925[30], 6}, {NULL}};
const Segment SGI238061[] = {{&S102925[31], 3}, {NULL}};
const Segment SGI238062[] = {{&S102925[86], 5}, {NULL}};
const Segment SGI238063[] = {{&S102925[144], 13}, {NULL}};
const Segment SGI238064[] = {{&S102926[34], 10}, {NULL}};
const Segment SGI238065[] = {{&S102926[27], 6}, {NULL}};
const Segment SGI238066[] = {{&S102926[95], 9}, {NULL}};
const Segment SGI238067[] = {{&S102926[68], 6}, {NULL}};
const Segment SGI238068[] = {{&S92817[44], 21}, {NULL}};
const Segment SGI238069[] = {{&S92817[30], 6}, {NULL}};
const Segment SGI238070[] = {{&S92817[79], 6}, {NULL}};
const Segment SGI238071[] = {{&S92817[106], 13}, {NULL}};
const Segment SGI238072[] = {{&S92820[34], 9}, {NULL}};
const Segment SGI238073[] = {{&S92820[27], 6}, {NULL}};
const Segment SGI238074[] = {{&S92820[96], 9}, {NULL}};
const Segment SGI238075[] = {{&S92820[69], 6}, {NULL}};
const Segment SGI238076[] = {{&S99750[34], 17}, {NULL}};
const Segment SGI238077[] = {{&S99750[27], 6}, {NULL}};
const Segment SGI238078[] = {{&S99750[104], 9}, {NULL}};
const Segment SGI238079[] = {{&S99750[77], 6}, {NULL}};
const Segment SGI238080[] = {{&S93423[34], 6}, {NULL}};
const Segment SGI238081[] = {{&S93423[27], 6}, {NULL}};
const Segment SGI238082[] = {{&S93423[93], 9}, {NULL}};
const Segment SGI238083[] = {{&S93423[66], 6}, {NULL}};
const Segment SGI238084[] = {{&S93433[44], 9}, {NULL}};
const Segment SGI238085[] = {{&S93433[44], 21}, {NULL}};
const Segment SGI238086[] = {{&S93433[30], 6}, {NULL}};
const Segment SGI238087[] = {{&S93433[114], 3}, {NULL}};
const Segment SGI238088[] = {{&S93433[79], 6}, {NULL}};
const Segment SGI238089[] = {{&S93440[44], 4}, {NULL}};
const Segment SGI238090[] = {{&S93440[44], 16}, {NULL}};
const Segment SGI238091[] = {{&S93440[30], 6}, {NULL}};
const Segment SGI238092[] = {{&S93440[74], 6}, {NULL}};
const Segment SGI238093[] = {{&S93440[111], 3}, {NULL}};
const Segment SGI238094[] = {{&S93447[34], 12}, {NULL}};
const Segment SGI238095[] = {{&S93447[27], 6}, {NULL}};
const Segment SGI238096[] = {{&S93447[99], 9}, {NULL}};
const Segment SGI238097[] = {{&S93447[72], 6}, {NULL}};
const Segment SGI238098[] = {{&S93448[34], 17}, {NULL}};
const Segment SGI238099[] = {{&S93448[27], 6}, {NULL}};
const Segment SGI238100[] = {{&S93448[104], 9}, {NULL}};
const Segment SGI238101[] = {{&S93448[77], 6}, {NULL}};
const Segment SGI238102[] = {{&S99801[34], 18}, {NULL}};
const Segment SGI238103[] = {{&S99801[27], 6}, {NULL}};
const Segment SGI238104[] = {{&S99801[105], 9}, {NULL}};
const Segment SGI238105[] = {{&S99801[78], 6}, {NULL}};
const Segment SGI238106[] = {{&S85093[34], 10}, {NULL}};
const Segment SGI238107[] = {{&S85093[27], 6}, {NULL}};
const Segment SGI238108[] = {{&S85093[96], 9}, {NULL}};
const Segment SGI238109[] = {{&S85093[69], 6}, {NULL}};
const Segment SGI238110[] = {{&S85106[34], 7}, {NULL}};
const Segment SGI238111[] = {{&S85106[27], 6}, {NULL}};
const Segment SGI238112[] = {{&S85106[94], 9}, {NULL}};
const Segment SGI238113[] = {{&S85106[67], 6}, {NULL}};
const Segment SGI238114[] = {{&S92929[44], 11}, {NULL}};
const Segment SGI238115[] = {{&S92929[44], 23}, {NULL}};
const Segment SGI238116[] = {{&S92929[30], 6}, {NULL}};
const Segment SGI238117[] = {{&S92929[114], 3}, {NULL}};
const Segment SGI238118[] = {{&S92929[81], 5}, {NULL}};
const Segment SGI238119[] = {{&S92929[118], 14}, {NULL}};
const Segment SGI238120[] = {{&S85113[34], 18}, {NULL}};
const Segment SGI238121[] = {{&S85113[27], 6}, {NULL}};
const Segment SGI238122[] = {{&S85113[105], 9}, {NULL}};
const Segment SGI238123[] = {{&S85113[78], 6}, {NULL}};
const Segment SGI238124[] = {{&S92935[34], 11}, {NULL}};
const Segment SGI238125[] = {{&S92935[27], 6}, {NULL}};
const Segment SGI238126[] = {{&S92935[98], 9}, {NULL}};
const Segment SGI238127[] = {{&S92935[71], 6}, {NULL}};
const Segment SGI238128[] = {{&S99844[34], 10}, {NULL}};
const Segment SGI238129[] = {{&S99844[27], 6}, {NULL}};
const Segment SGI238130[] = {{&S99844[102], 9}, {NULL}};
const Segment SGI238131[] = {{&S99844[75], 6}, {NULL}};
const Segment SGI238132[] = {{&S99856[31], 8}, {NULL}};
const Segment SGI238133[] = {{&S99856[55], 3}, {NULL}};
const Segment SGI238134[] = {{&S103090[44], 7}, {NULL}};
const Segment SGI238135[] = {{&S103090[44], 19}, {NULL}};
const Segment SGI238136[] = {{&S103090[30], 6}, {NULL}};
const Segment SGI238137[] = {{&S103090[31], 3}, {NULL}};
const Segment SGI238138[] = {{&S103090[77], 6}, {NULL}};
const Segment SGI238139[] = {{&S99874[44], 6}, {NULL}};
const Segment SGI238140[] = {{&S99874[44], 18}, {NULL}};
const Segment SGI238141[] = {{&S99874[30], 6}, {NULL}};
const Segment SGI238142[] = {{&S103106[44], 12}, {NULL}};
const Segment SGI238143[] = {{&S103106[44], 24}, {NULL}};
const Segment SGI238144[] = {{&S103106[30], 6}, {NULL}};
const Segment SGI238145[] = {{&S99874[76], 6}, {NULL}};
const Segment SGI238146[] = {{&S99874[103], 13}, {NULL}};
const Segment SGI238147[] = {{&S103106[31], 3}, {NULL}};
const Segment SGI238148[] = {{&S103106[111], 6}, {NULL}};
const Segment SGI238149[] = {{&S103106[70], 8}, {NULL}};
const Segment SGI238150[] = {{&S103106[82], 15}, {NULL}};
const Segment SGI238151[] = {{&S103107[44], 12}, {NULL}};
const Segment SGI238152[] = {{&S103107[44], 24}, {NULL}};
const Segment SGI238153[] = {{&S103107[30], 6}, {NULL}};
const Segment SGI238154[] = {{&S103107[31], 3}, {NULL}};
const Segment SGI238155[] = {{&S103107[111], 6}, {NULL}};
const Segment SGI238156[] = {{&S103107[70], 8}, {NULL}};
const Segment SGI238157[] = {{&S103107[82], 15}, {NULL}};
const Segment SGI238158[] = {{&S92976[34], 11}, {NULL}};
const Segment SGI238159[] = {{&S92976[27], 6}, {NULL}};
const Segment SGI238160[] = {{&S92976[98], 9}, {NULL}};
const Segment SGI238161[] = {{&S92976[71], 6}, {NULL}};
const Segment SGI238162[] = {{&S92981[34], 7}, {NULL}};
const Segment SGI238163[] = {{&S92981[27], 6}, {NULL}};
const Segment SGI238164[] = {{&S92981[94], 9}, {NULL}};
const Segment SGI238165[] = {{&S92981[67], 6}, {NULL}};
const Segment SGI238166[] = {{&S99893[34], 41}, {NULL}};
const Segment SGI238167[] = {{&S99893[27], 6}, {NULL}};
const Segment SGI238168[] = {{&S99893[128], 9}, {NULL}};
const Segment SGI238169[] = {{&S99893[101], 6}, {NULL}};
const Segment SGI238170[] = {{&S92990[44], 6}, {NULL}};
const Segment SGI238171[] = {{&S92990[44], 18}, {NULL}};
const Segment SGI238172[] = {{&S92990[30], 6}, {NULL}};
const Segment SGI238173[] = {{&S92990[76], 6}, {NULL}};
const Segment SGI238174[] = {{&S92990[103], 13}, {NULL}};
const Segment SGI238175[] = {{&S92992[44], 10}, {NULL}};
const Segment SGI238176[] = {{&S92992[44], 22}, {NULL}};
const Segment SGI238177[] = {{&S92992[30], 6}, {NULL}};
const Segment SGI238178[] = {{&S92992[113], 3}, {NULL}};
const Segment SGI238179[] = {{&S92992[80], 5}, {NULL}};
const Segment SGI238180[] = {{&S92992[117], 14}, {NULL}};
const Segment SGI238181[] = {{&S103130[44], 13}, {NULL}};
const Segment SGI238182[] = {{&S103130[44], 29}, {NULL}};
const Segment SGI238183[] = {{&S103130[30], 6}, {NULL}};
const Segment SGI238184[] = {{&S103130[31], 3}, {NULL}};
const Segment SGI238185[] = {{&S103130[87], 6}, {NULL}};
const Segment SGI238186[] = {{&S92993[44], 12}, {NULL}};
const Segment SGI238187[] = {{&S92993[44], 34}, {NULL}};
const Segment SGI238188[] = {{&S103130[146], 10}, {NULL}};
const Segment SGI238189[] = {{&S92993[30], 6}, {NULL}};
const Segment SGI238190[] = {{&S92993[92], 6}, {NULL}};
const Segment SGI238191[] = {{&S92993[119], 13}, {NULL}};
const Segment SGI238192[] = {{&S99905[34], 24}, {NULL}};
const Segment SGI238193[] = {{&S99905[27], 6}, {NULL}};
const Segment SGI238194[] = {{&S99905[111], 9}, {NULL}};
const Segment SGI238195[] = {{&S99905[84], 6}, {NULL}};
const Segment SGI238196[] = {{&S92999[44], 11}, {NULL}};
const Segment SGI238197[] = {{&S92999[44], 23}, {NULL}};
const Segment SGI238198[] = {{&S92999[30], 6}, {NULL}};
const Segment SGI238199[] = {{&S92999[81], 6}, {NULL}};
const Segment SGI238200[] = {{&S92999[108], 13}, {NULL}};
const Segment SGI238201[] = {{&S93003[44], 10}, {NULL}};
const Segment SGI238202[] = {{&S99913[44], 11}, {NULL}};
const Segment SGI238203[] = {{&S93003[44], 22}, {NULL}};
const Segment SGI238204[] = {{&S99913[44], 23}, {NULL}};
const Segment SGI238205[] = {{&S93003[30], 6}, {NULL}};
const Segment SGI238206[] = {{&S99913[30], 6}, {NULL}};
const Segment SGI238207[] = {{&S99913[116], 3}, {NULL}};
const Segment SGI238208[] = {{&S93003[80], 6}, {NULL}};
const Segment SGI238209[] = {{&S99913[81], 6}, {NULL}};
const Segment SGI238210[] = {{&S93003[107], 13}, {NULL}};
const Segment SGI238211[] = {{&S93005[34], 6}, {NULL}};
const Segment SGI238212[] = {{&S93005[27], 6}, {NULL}};
const Segment SGI238213[] = {{&S93005[93], 9}, {NULL}};
const Segment SGI238214[] = {{&S93005[66], 6}, {NULL}};
const Segment SGI238215[] = {{&S99921[44], 11}, {NULL}};
const Segment SGI238216[] = {{&S99921[44], 23}, {NULL}};
const Segment SGI238217[] = {{&S99921[30], 6}, {NULL}};
const Segment SGI238218[] = {{&S99921[81], 6}, {NULL}};
const Segment SGI238219[] = {{&S99921[108], 13}, {NULL}};
const Segment SGI238220[] = {{&S103155[44], 10}, {NULL}};
const Segment SGI238221[] = {{&S103155[44], 22}, {NULL}};
const Segment SGI238222[] = {{&S103155[30], 6}, {NULL}};
const Segment SGI238223[] = {{&S103155[139], 4}, {NULL}};
const Segment SGI238224[] = {{&S103155[165], 3}, {NULL}};
const Segment SGI238225[] = {{&S103155[31], 3}, {&S103155[214], 5}, {NULL}};
const Segment SGI238226[] = {{&S103155[80], 6}, {NULL}};
const Segment SGI238227[] = {{&S93021[34], 10}, {NULL}};
const Segment SGI238228[] = {{&S93021[27], 6}, {NULL}};
const Segment SGI238229[] = {{&S85220[44], 14}, {NULL}};
const Segment SGI238230[] = {{&S85220[44], 25}, {NULL}};
const Segment SGI238231[] = {{&S93021[97], 9}, {NULL}};
const Segment SGI238232[] = {{&S85220[30], 6}, {NULL}};
const Segment SGI238233[] = {{&S93021[70], 6}, {NULL}};
const Segment SGI238234[] = {{&S85220[118], 3}, {NULL}};
const Segment SGI238235[] = {{&S85220[83], 6}, {NULL}};
const Segment SGI238236[] = {{&S85220[150], 15}, {NULL}};
const Segment SGI238237[] = {{&S103159[44], 10}, {NULL}};
const Segment SGI238238[] = {{&S103159[44], 22}, {NULL}};
const Segment SGI238239[] = {{&S103159[30], 6}, {NULL}};
const Segment SGI238240[] = {{&S103159[31], 3}, {NULL}};
const Segment SGI238241[] = {{&S103159[80], 5}, {NULL}};
const Segment SGI238242[] = {{&S103159[125], 14}, {NULL}};
const Segment SGI238243[] = {{&S93029[44], 11}, {NULL}};
const Segment SGI238244[] = {{&S93029[44], 23}, {NULL}};
const Segment SGI238245[] = {{&S93029[30], 6}, {NULL}};
const Segment SGI238246[] = {{&S93029[116], 3}, {NULL}};
const Segment SGI238247[] = {{&S99941[59], 4}, {NULL}};
const Segment SGI238248[] = {{&S93029[81], 6}, {NULL}};
const Segment SGI238249[] = {{&S99941[59], 16}, {NULL}};
const Segment SGI238250[] = {{&S99941[45], 6}, {NULL}};
const Segment SGI238251[] = {{&S99941[124], 3}, {NULL}};
const Segment SGI238252[] = {{&S99941[89], 6}, {NULL}};
const Segment SGI238253[] = {{&S99941[0], 14}, {NULL}};
const Segment SGI238254[] = {{&S103166[45], 10}, {NULL}};
const Segment SGI238255[] = {{&S103166[30], 6}, {NULL}};
const Segment SGI238256[] = {{&S103166[31], 3}, {NULL}};
const Segment SGI238257[] = {{&S103166[81], 5}, {NULL}};
const Segment SGI238258[] = {{&S103166[139], 13}, {NULL}};
const Segment SGI238259[] = {{&S93034[44], 12}, {NULL}};
const Segment SGI238260[] = {{&S93034[44], 24}, {NULL}};
const Segment SGI238261[] = {{&S93034[30], 6}, {NULL}};
const Segment SGI238262[] = {{&S93034[117], 3}, {NULL}};
const Segment SGI238263[] = {{&S85234[44], 10}, {NULL}};
const Segment SGI238264[] = {{&S93034[82], 6}, {NULL}};
const Segment SGI238265[] = {{&S85234[44], 21}, {NULL}};
const Segment SGI238266[] = {{&S85234[30], 6}, {NULL}};
const Segment SGI238267[] = {{&S85234[79], 6}, {NULL}};
const Segment SGI238268[] = {{&S85234[106], 13}, {NULL}};
const Segment SGI238269[] = {{&S85236[34], 10}, {NULL}};
const Segment SGI238270[] = {{&S85236[27], 6}, {NULL}};
const Segment SGI238271[] = {{&S85236[96], 9}, {NULL}};
const Segment SGI238272[] = {{&S85236[69], 6}, {NULL}};
const Segment SGI238273[] = {{&S99951[45], 10}, {NULL}};
const Segment SGI238274[] = {{&S99951[30], 6}, {NULL}};
const Segment SGI238275[] = {{&S99951[115], 3}, {NULL}};
const Segment SGI238276[] = {{&S99951[81], 5}, {NULL}};
const Segment SGI238277[] = {{&S99951[139], 13}, {NULL}};
const Segment SGI238278[] = {{&S93043[34], 7}, {NULL}};
const Segment SGI238279[] = {{&S93043[27], 6}, {NULL}};
const Segment SGI238280[] = {{&S93043[99], 9}, {NULL}};
const Segment SGI238281[] = {{&S93043[72], 6}, {NULL}};
const Segment SGI238282[] = {{&S93043[140], 8}, {NULL}};
const Segment SGI238283[] = {{&S93046[44], 7}, {NULL}};
const Segment SGI238284[] = {{&S93046[44], 25}, {NULL}};
const Segment SGI238285[] = {{&S93046[30], 6}, {NULL}};
const Segment SGI238286[] = {{&S103180[44], 10}, {NULL}};
const Segment SGI238287[] = {{&S103180[44], 22}, {NULL}};
const Segment SGI238288[] = {{&S103180[30], 6}, {NULL}};
const Segment SGI238289[] = {{&S93046[83], 6}, {NULL}};
const Segment SGI238290[] = {{&S93046[110], 13}, {NULL}};
const Segment SGI238291[] = {{&S103180[31], 3}, {NULL}};
const Segment SGI238292[] = {{&S103180[80], 6}, {NULL}};
const Segment SGI238293[] = {{&S103180[143], 3}, {NULL}};
const Segment SGI238294[] = {{&S93047[44], 7}, {NULL}};
const Segment SGI238295[] = {{&S93047[44], 25}, {NULL}};
const Segment SGI238296[] = {{&S93047[30], 6}, {NULL}};
const Segment SGI238297[] = {{&S93047[83], 6}, {NULL}};
const Segment SGI238298[] = {{&S93047[110], 13}, {NULL}};
const Segment SGI238299[] = {{&S93048[34], 7}, {NULL}};
const Segment SGI238300[] = {{&S93048[27], 6}, {NULL}};
const Segment SGI238301[] = {{&S93048[98], 9}, {NULL}};
const Segment SGI238302[] = {{&S93048[71], 6}, {NULL}};
const Segment SGI238303[] = {{&S103186[44], 10}, {NULL}};
const Segment SGI238304[] = {{&S103186[44], 22}, {NULL}};
const Segment SGI238305[] = {{&S103186[30], 6}, {NULL}};
const Segment SGI238306[] = {{&S103186[80], 6}, {NULL}};
const Segment SGI238307[] = {{&S103186[107], 13}, {NULL}};
const Segment SGI238308[] = {{&S93612[34], 10}, {NULL}};
const Segment SGI238309[] = {{&S93612[27], 6}, {NULL}};
const Segment SGI238310[] = {{&S93612[97], 9}, {NULL}};
const Segment SGI238311[] = {{&S93612[70], 6}, {NULL}};
const Segment SGI238312[] = {{&S99967[44], 10}, {NULL}};
const Segment SGI238313[] = {{&S99967[44], 22}, {NULL}};
const Segment SGI238314[] = {{&S99967[30], 6}, {NULL}};
const Segment SGI238315[] = {{&S99967[115], 3}, {NULL}};
const Segment SGI238316[] = {{&S99967[80], 6}, {NULL}};
const Segment SGI238317[] = {{&S99969[44], 18}, {NULL}};
const Segment SGI238318[] = {{&S99969[44], 39}, {NULL}};
const Segment SGI238319[] = {{&S99969[30], 6}, {NULL}};
const Segment SGI238320[] = {{&S99969[97], 6}, {NULL}};
const Segment SGI238321[] = {{&S99969[124], 13}, {NULL}};
const Segment SGI238322[] = {{&S99972[44], 11}, {NULL}};
const Segment SGI238323[] = {{&S99972[44], 23}, {NULL}};
const Segment SGI238324[] = {{&S99972[30], 6}, {NULL}};
const Segment SGI238325[] = {{&S99972[116], 3}, {NULL}};
const Segment SGI238326[] = {{&S99972[81], 6}, {NULL}};
const Segment SGI238327[] = {{&S99972[151], 3}, {NULL}};
const Segment SGI238328[] = {{&S85295[34], 12}, {NULL}};
const Segment SGI238329[] = {{&S85295[27], 6}, {NULL}};
const Segment SGI238330[] = {{&S85295[99], 9}, {NULL}};
const Segment SGI238331[] = {{&S85295[72], 6}, {NULL}};
const Segment SGI238332[] = {{&S93649[34], 17}, {NULL}};
const Segment SGI238333[] = {{&S93649[27], 6}, {NULL}};
const Segment SGI238334[] = {{&S93649[104], 9}, {NULL}};
const Segment SGI238335[] = {{&S93649[77], 6}, {NULL}};
const Segment SGI238336[] = {{&S99991[44], 10}, {NULL}};
const Segment SGI238337[] = {{&S99991[44], 22}, {NULL}};
const Segment SGI238338[] = {{&S99991[30], 6}, {NULL}};
const Segment SGI238339[] = {{&S99991[115], 3}, {NULL}};
const Segment SGI238340[] = {{&S99991[80], 6}, {NULL}};
const Segment SGI238341[] = {{&S93102[44], 7}, {NULL}};
const Segment SGI238342[] = {{&S93102[44], 19}, {NULL}};
const Segment SGI238343[] = {{&S93102[30], 6}, {NULL}};
const Segment SGI238344[] = {{&S93102[128], 3}, {NULL}};
const Segment SGI238345[] = {{&S93102[93], 6}, {NULL}};
const Segment SGI238346[] = {{&S93102[65], 8}, {NULL}};
const Segment SGI238347[] = {{&S85304[44], 10}, {NULL}};
const Segment SGI238348[] = {{&S85304[44], 21}, {NULL}};
const Segment SGI238349[] = {{&S85304[30], 6}, {NULL}};
const Segment SGI238350[] = {{&S99995[44], 11}, {NULL}};
const Segment SGI238351[] = {{&S99995[44], 23}, {NULL}};
const Segment SGI238352[] = {{&S99995[30], 6}, {NULL}};
const Segment SGI238353[] = {{&S85304[79], 6}, {NULL}};
const Segment SGI238354[] = {{&S99995[116], 3}, {NULL}};
const Segment SGI238355[] = {{&S85304[106], 13}, {NULL}};
const Segment SGI238356[] = {{&S99995[81], 6}, {NULL}};
const Segment SGI238357[] = {{&S99995[148], 15}, {NULL}};
const Segment SGI238358[] = {{&S99999[44], 6}, {NULL}};
const Segment SGI238359[] = {{&S99999[44], 18}, {NULL}};
const Segment SGI238360[] = {{&S99999[30], 6}, {NULL}};
const Segment SGI238361[] = {{&S99999[111], 3}, {NULL}};
const Segment SGI238362[] = {{&S99999[76], 6}, {NULL}};
const Segment SGI238363[] = {{&S99999[143], 15}, {NULL}};
const Segment SGI238364[] = {{&S104393[57], 9}, {NULL}};
const Segment SGI238365[] = {{&S104393[57], 21}, {NULL}};
const Segment SGI238366[] = {{&S104393[45], 4}, {NULL}};
const Segment SGI238367[] = {{&S93120[38], 11}, {NULL}};
const Segment SGI238368[] = {{&S93120[31], 6}, {NULL}};
const Segment SGI238369[] = {{&S104393[127], 3}, {NULL}};
const Segment SGI238370[] = {{&S104393[92], 6}, {NULL}};
const Segment SGI238371[] = {{&S93121[38], 10}, {NULL}};
const Segment SGI238372[] = {{&S93121[31], 6}, {NULL}};
const Segment SGI238373[] = {{&S85328[34], 18}, {NULL}};
const Segment SGI238374[] = {{&S85328[27], 6}, {NULL}};
const Segment SGI238375[] = {{&S85328[104], 9}, {NULL}};
const Segment SGI238376[] = {{&S85328[77], 6}, {NULL}};
const Segment SGI238377[] = {{&S85328[145], 8}, {NULL}};
const Segment SGI238378[] = {{&S93699[44], 11}, {NULL}};
const Segment SGI238379[] = {{&S93699[44], 23}, {NULL}};
const Segment SGI238380[] = {{&S93699[30], 6}, {NULL}};
const Segment SGI238381[] = {{&S93699[114], 3}, {NULL}};
const Segment SGI238382[] = {{&S93699[81], 5}, {NULL}};
const Segment SGI238383[] = {{&S93700[34], 10}, {NULL}};
const Segment SGI238384[] = {{&S93700[27], 6}, {NULL}};
const Segment SGI238385[] = {{&S93146[34], 10}, {NULL}};
const Segment SGI238386[] = {{&S93146[27], 6}, {NULL}};
const Segment SGI238387[] = {{&S93700[110], 9}, {NULL}};
const Segment SGI238388[] = {{&S93700[83], 6}, {NULL}};
const Segment SGI238389[] = {{&S93146[97], 9}, {NULL}};
const Segment SGI238390[] = {{&S93146[70], 6}, {NULL}};
const Segment SGI238391[] = {{&S100038[44], 8}, {NULL}};
const Segment SGI238392[] = {{&S100038[44], 20}, {NULL}};
const Segment SGI238393[] = {{&S100038[30], 6}, {NULL}};
const Segment SGI238394[] = {{&S100038[113], 3}, {NULL}};
const Segment SGI238395[] = {{&S100038[78], 6}, {NULL}};
const Segment SGI238396[] = {{&S93706[44], 6}, {NULL}};
const Segment SGI238397[] = {{&S93706[44], 20}, {NULL}};
const Segment SGI238398[] = {{&S93706[30], 6}, {NULL}};
const Segment SGI238399[] = {{&S93706[78], 6}, {NULL}};
const Segment SGI238400[] = {{&S93706[115], 3}, {NULL}};
const Segment SGI238401[] = {{&S100040[34], 4}, {NULL}};
const Segment SGI238402[] = {{&S100040[27], 6}, {NULL}};
const Segment SGI238403[] = {{&S100040[91], 9}, {NULL}};
const Segment SGI238404[] = {{&S100040[64], 6}, {NULL}};
const Segment SGI238405[] = {{&S100043[44], 25}, {NULL}};
const Segment SGI238406[] = {{&S100043[44], 37}, {NULL}};
const Segment SGI238407[] = {{&S100043[30], 6}, {NULL}};
const Segment SGI238408[] = {{&S100043[130], 3}, {NULL}};
const Segment SGI238409[] = {{&S100043[95], 6}, {NULL}};
const Segment SGI238410[] = {{&S100043[149], 12}, {NULL}};
const Segment SGI238411[] = {{&S100044[34], 12}, {NULL}};
const Segment SGI238412[] = {{&S100044[27], 6}, {NULL}};
const Segment SGI238413[] = {{&S100044[99], 9}, {NULL}};
const Segment SGI238414[] = {{&S100044[72], 6}, {NULL}};
const Segment SGI238415[] = {{&S93168[34], 7}, {NULL}};
const Segment SGI238416[] = {{&S93168[27], 6}, {NULL}};
const Segment SGI238417[] = {{&S93168[100], 9}, {NULL}};
const Segment SGI238418[] = {{&S93168[73], 6}, {NULL}};
const Segment SGI238419[] = {{&S85375[44], 10}, {NULL}};
const Segment SGI238420[] = {{&S85375[44], 21}, {NULL}};
const Segment SGI238421[] = {{&S85375[30], 6}, {NULL}};
const Segment SGI238422[] = {{&S85375[132], 3}, {NULL}};
const Segment SGI238423[] = {{&S85375[97], 6}, {NULL}};
const Segment SGI238424[] = {{&S85375[67], 8}, {NULL}};
const Segment SGI238425[] = {{&S93175[44], 18}, {NULL}};
const Segment SGI238426[] = {{&S93175[44], 39}, {NULL}};
const Segment SGI238427[] = {{&S93175[30], 6}, {NULL}};
const Segment SGI238428[] = {{&S93175[130], 3}, {NULL}};
const Segment SGI238429[] = {{&S93175[97], 5}, {NULL}};
const Segment SGI238430[] = {{&S93175[134], 14}, {NULL}};
const Segment SGI238431[] = {{&S93176[44], 18}, {NULL}};
const Segment SGI238432[] = {{&S93176[44], 39}, {NULL}};
const Segment SGI238433[] = {{&S93176[30], 6}, {NULL}};
const Segment SGI238434[] = {{&S93176[130], 3}, {NULL}};
const Segment SGI238435[] = {{&S93176[97], 5}, {NULL}};
const Segment SGI238436[] = {{&S93176[134], 14}, {NULL}};
const Segment SGI238437[] = {{&S93183[34], 7}, {NULL}};
const Segment SGI238438[] = {{&S93183[27], 6}, {NULL}};
const Segment SGI238439[] = {{&S93183[104], 9}, {NULL}};
const Segment SGI238440[] = {{&S93183[77], 6}, {NULL}};
const Segment SGI238441[] = {{&S93183[145], 8}, {NULL}};
const Segment SGI238442[] = {{&S93187[34], 11}, {NULL}};
const Segment SGI238443[] = {{&S93187[27], 6}, {NULL}};
const Segment SGI238444[] = {{&S93187[98], 9}, {NULL}};
const Segment SGI238445[] = {{&S93187[71], 6}, {NULL}};
const Segment SGI238446[] = {{&S93190[59], 10}, {NULL}};
const Segment SGI238447[] = {{&S93190[59], 22}, {NULL}};
const Segment SGI238448[] = {{&S93190[45], 6}, {NULL}};
const Segment SGI238449[] = {{&S93190[130], 3}, {NULL}};
const Segment SGI238450[] = {{&S93190[95], 6}, {NULL}};
const Segment SGI238451[] = {{&S93190[0], 14}, {NULL}};
const Segment SGI238452[] = {{&S93191[59], 18}, {NULL}};
const Segment SGI238453[] = {{&S93191[59], 39}, {NULL}};
const Segment SGI238454[] = {{&S93191[45], 6}, {NULL}};
const Segment SGI238455[] = {{&S93191[147], 3}, {NULL}};
const Segment SGI238456[] = {{&S93191[112], 6}, {NULL}};
const Segment SGI238457[] = {{&S93191[0], 14}, {NULL}};
const Segment SGI238458[] = {{&S100083[44], 13}, {NULL}};
const Segment SGI238459[] = {{&S100083[44], 25}, {NULL}};
const Segment SGI238460[] = {{&S100083[30], 6}, {NULL}};
const Segment SGI238461[] = {{&S100083[83], 6}, {NULL}};
const Segment SGI238462[] = {{&S100083[110], 13}, {NULL}};
const Segment SGI238463[] = {{&S93194[44], 10}, {NULL}};
const Segment SGI238464[] = {{&S93194[44], 22}, {NULL}};
const Segment SGI238465[] = {{&S93194[30], 6}, {NULL}};
const Segment SGI238466[] = {{&S93194[113], 3}, {NULL}};
const Segment SGI238467[] = {{&S93194[80], 5}, {NULL}};
const Segment SGI238468[] = {{&S93194[117], 14}, {NULL}};
const Segment SGI238469[] = {{&S93195[44], 11}, {NULL}};
const Segment SGI238470[] = {{&S93195[44], 23}, {NULL}};
const Segment SGI238471[] = {{&S93195[30], 6}, {NULL}};
const Segment SGI238472[] = {{&S93195[114], 3}, {NULL}};
const Segment SGI238473[] = {{&S93195[81], 5}, {NULL}};
const Segment SGI238474[] = {{&S93195[118], 14}, {NULL}};
const Segment SGI238475[] = {{&S93201[44], 15}, {NULL}};
const Segment SGI238476[] = {{&S93201[44], 27}, {NULL}};
const Segment SGI238477[] = {{&S93201[30], 6}, {NULL}};
const Segment SGI238478[] = {{&S93201[85], 6}, {NULL}};
const Segment SGI238479[] = {{&S93201[112], 13}, {NULL}};
const Segment SGI238480[] = {{&S85406[44], 10}, {NULL}};
const Segment SGI238481[] = {{&S85406[44], 22}, {NULL}};
const Segment SGI238482[] = {{&S85406[30], 6}, {NULL}};
const Segment SGI238483[] = {{&S85406[115], 3}, {NULL}};
const Segment SGI238484[] = {{&S85406[80], 6}, {NULL}};
const Segment SGI238485[] = {{&S85406[140], 8}, {NULL}};
const Segment SGI238486[] = {{&S93208[44], 9}, {NULL}};
const Segment SGI238487[] = {{&S93208[44], 21}, {NULL}};
const Segment SGI238488[] = {{&S93208[30], 6}, {NULL}};
const Segment SGI238489[] = {{&S93208[79], 6}, {NULL}};
const Segment SGI238490[] = {{&S93208[106], 13}, {NULL}};
const Segment SGI238491[] = {{&S103370[44], 12}, {NULL}};
const Segment SGI238492[] = {{&S103370[44], 22}, {NULL}};
const Segment SGI238493[] = {{&S103370[30], 6}, {NULL}};
const Segment SGI238494[] = {{&S103370[31], 3}, {NULL}};
const Segment SGI238495[] = {{&S103370[80], 6}, {NULL}};
const Segment SGI238496[] = {{&S85416[34], 10}, {NULL}};
const Segment SGI238497[] = {{&S85416[27], 6}, {NULL}};
const Segment SGI238498[] = {{&S85416[96], 9}, {NULL}};
const Segment SGI238499[] = {{&S85416[69], 6}, {NULL}};
const Segment SGI238500[] = {{&S85416[137], 8}, {NULL}};
const Segment SGI238501[] = {{&S93215[34], 13}, {NULL}};
const Segment SGI238502[] = {{&S93215[27], 6}, {NULL}};
const Segment SGI238503[] = {{&S93215[100], 9}, {NULL}};
const Segment SGI238504[] = {{&S93215[73], 6}, {NULL}};
const Segment SGI238505[] = {{&S103376[44], 10}, {NULL}};
const Segment SGI238506[] = {{&S103376[44], 22}, {NULL}};
const Segment SGI238507[] = {{&S103376[30], 6}, {NULL}};
const Segment SGI238508[] = {{&S93216[34], 11}, {NULL}};
const Segment SGI238509[] = {{&S103376[31], 3}, {NULL}};
const Segment SGI238510[] = {{&S93216[27], 6}, {NULL}};
const Segment SGI238511[] = {{&S103376[80], 6}, {NULL}};
const Segment SGI238512[] = {{&S103376[150], 3}, {NULL}};
const Segment SGI238513[] = {{&S93216[98], 9}, {NULL}};
const Segment SGI238514[] = {{&S93216[71], 6}, {NULL}};
const Segment SGI238515[] = {{&S93217[34], 10}, {NULL}};
const Segment SGI238516[] = {{&S93217[27], 6}, {NULL}};
const Segment SGI238517[] = {{&S93217[97], 9}, {NULL}};
const Segment SGI238518[] = {{&S93217[70], 6}, {NULL}};
const Segment SGI238519[] = {{&S93219[34], 7}, {NULL}};
const Segment SGI238520[] = {{&S93219[27], 6}, {NULL}};
const Segment SGI238521[] = {{&S93219[94], 9}, {NULL}};
const Segment SGI238522[] = {{&S93219[67], 6}, {NULL}};
const Segment SGI238523[] = {{&S93224[34], 10}, {NULL}};
const Segment SGI238524[] = {{&S93224[27], 6}, {NULL}};
const Segment SGI238525[] = {{&S93224[97], 9}, {NULL}};
const Segment SGI238526[] = {{&S93224[70], 6}, {NULL}};
const Segment SGI238527[] = {{&S93224[138], 8}, {NULL}};
const Segment SGI238528[] = {{&S93225[44], 11}, {NULL}};
const Segment SGI238529[] = {{&S93225[44], 23}, {NULL}};
const Segment SGI238530[] = {{&S93225[30], 6}, {NULL}};
const Segment SGI238531[] = {{&S93225[114], 3}, {NULL}};
const Segment SGI238532[] = {{&S93225[81], 5}, {NULL}};
const Segment SGI238533[] = {{&S93225[118], 14}, {NULL}};
const Segment SGI238534[] = {{&S93226[44], 7}, {NULL}};
const Segment SGI238535[] = {{&S93226[44], 19}, {NULL}};
const Segment SGI238536[] = {{&S93226[30], 6}, {NULL}};
const Segment SGI238537[] = {{&S93226[112], 3}, {NULL}};
const Segment SGI238538[] = {{&S93226[77], 6}, {NULL}};
const Segment SGI238539[] = {{&S93233[44], 7}, {NULL}};
const Segment SGI238540[] = {{&S93233[44], 24}, {NULL}};
const Segment SGI238541[] = {{&S93233[30], 6}, {NULL}};
const Segment SGI238542[] = {{&S93233[117], 3}, {NULL}};
const Segment SGI238543[] = {{&S93233[82], 6}, {NULL}};
const Segment SGI238544[] = {{&S93238[34], 10}, {NULL}};
const Segment SGI238545[] = {{&S93238[27], 6}, {NULL}};
const Segment SGI238546[] = {{&S93238[97], 9}, {NULL}};
const Segment SGI238547[] = {{&S93238[70], 6}, {NULL}};
const Segment SGI238548[] = {{&S93239[34], 10}, {NULL}};
const Segment SGI238549[] = {{&S93239[27], 6}, {NULL}};
const Segment SGI238550[] = {{&S93239[97], 9}, {NULL}};
const Segment SGI238551[] = {{&S93239[70], 6}, {NULL}};
const Segment SGI238552[] = {{&S93240[34], 10}, {NULL}};
const Segment SGI238553[] = {{&S93240[27], 6}, {NULL}};
const Segment SGI238554[] = {{&S93240[97], 9}, {NULL}};
const Segment SGI238555[] = {{&S93240[70], 6}, {NULL}};
const Segment SGI238556[] = {{&S85469[44], 12}, {NULL}};
const Segment SGI238557[] = {{&S85469[44], 23}, {NULL}};
const Segment SGI238558[] = {{&S85469[30], 6}, {NULL}};
const Segment SGI238559[] = {{&S85469[116], 3}, {NULL}};
const Segment SGI238560[] = {{&S85469[81], 6}, {NULL}};
const Segment SGI238561[] = {{&S93251[34], 8}, {NULL}};
const Segment SGI238562[] = {{&S93251[27], 6}, {NULL}};
const Segment SGI238563[] = {{&S93251[100], 9}, {NULL}};
const Segment SGI238564[] = {{&S93251[73], 6}, {NULL}};
const Segment SGI238565[] = {{&S103442[62], 10}, {NULL}};
const Segment SGI238566[] = {{&S103442[55], 6}, {NULL}};
const Segment SGI238567[] = {{&S103443[62], 11}, {NULL}};
const Segment SGI238568[] = {{&S103443[55], 6}, {NULL}};
const Segment SGI238569[] = {{&S85498[34], 12}, {NULL}};
const Segment SGI238570[] = {{&S85498[27], 6}, {NULL}};
const Segment SGI238571[] = {{&S85498[98], 9}, {NULL}};
const Segment SGI238572[] = {{&S85498[71], 6}, {NULL}};
const Segment SGI238573[] = {{&S85499[34], 19}, {NULL}};
const Segment SGI238574[] = {{&S85499[27], 6}, {NULL}};
const Segment SGI238575[] = {{&S85499[105], 9}, {NULL}};
const Segment SGI238576[] = {{&S85499[78], 6}, {NULL}};
const Segment SGI238577[] = {{&S103458[44], 10}, {NULL}};
const Segment SGI238578[] = {{&S85508[30], 6}, {NULL}};
const Segment SGI238579[] = {{&S103458[44], 22}, {NULL}};
const Segment SGI238580[] = {{&S103458[30], 6}, {NULL}};
const Segment SGI238581[] = {{&S85508[85], 9}, {NULL}};
const Segment SGI238582[] = {{&S85508[57], 6}, {NULL}};
const Segment SGI238583[] = {{&S103458[31], 3}, {NULL}};
const Segment SGI238584[] = {{&S103458[80], 6}, {NULL}};
const Segment SGI238585[] = {{&S103458[133], 11}, {NULL}};
const Segment SGI238586[] = {{&S103463[44], 10}, {NULL}};
const Segment SGI238587[] = {{&S103463[44], 22}, {NULL}};
const Segment SGI238588[] = {{&S103463[30], 6}, {NULL}};
const Segment SGI238589[] = {{&S103463[80], 6}, {NULL}};
const Segment SGI238590[] = {{&S103463[107], 13}, {NULL}};
const Segment SGI238591[] = {{&S85519[44], 10}, {NULL}};
const Segment SGI238592[] = {{&S85519[44], 21}, {NULL}};
const Segment SGI238593[] = {{&S85519[30], 6}, {NULL}};
const Segment SGI238594[] = {{&S85519[79], 6}, {NULL}};
const Segment SGI238595[] = {{&S85519[106], 13}, {NULL}};
const Segment SGI238596[] = {{&S85538[34], 8}, {NULL}};
const Segment SGI238597[] = {{&S85538[27], 6}, {NULL}};
const Segment SGI238598[] = {{&S85538[103], 9}, {NULL}};
const Segment SGI238599[] = {{&S85538[76], 6}, {NULL}};
const Segment SGI238600[] = {{&S100101[34], 7}, {NULL}};
const Segment SGI238601[] = {{&S100101[27], 6}, {NULL}};
const Segment SGI238602[] = {{&S100101[110], 9}, {NULL}};
const Segment SGI238603[] = {{&S100101[83], 6}, {NULL}};
const Segment SGI238604[] = {{&S100101[151], 8}, {NULL}};
const Segment SGI238605[] = {{&S93356[34], 10}, {NULL}};
const Segment SGI238606[] = {{&S93356[27], 6}, {NULL}};
const Segment SGI238607[] = {{&S93356[97], 9}, {NULL}};
const Segment SGI238608[] = {{&S93356[70], 6}, {NULL}};
const Segment SGI238609[] = {{&S93936[34], 10}, {NULL}};
const Segment SGI238610[] = {{&S93936[27], 6}, {NULL}};
const Segment SGI238611[] = {{&S93936[97], 9}, {NULL}};
const Segment SGI238612[] = {{&S93936[70], 6}, {NULL}};
const Segment SGI238613[] = {{&S93375[44], 10}, {NULL}};
const Segment SGI238614[] = {{&S93375[44], 22}, {NULL}};
const Segment SGI238615[] = {{&S93375[30], 6}, {NULL}};
const Segment SGI238616[] = {{&S93375[80], 6}, {NULL}};
const Segment SGI238617[] = {{&S93375[107], 13}, {NULL}};
const Segment SGI238618[] = {{&S93385[59], 10}, {NULL}};
const Segment SGI238619[] = {{&S93385[59], 22}, {NULL}};
const Segment SGI238620[] = {{&S93385[45], 6}, {NULL}};
const Segment SGI238621[] = {{&S93385[130], 3}, {NULL}};
const Segment SGI238622[] = {{&S93385[95], 6}, {NULL}};
const Segment SGI238623[] = {{&S93385[0], 14}, {NULL}};
const Segment SGI238624[] = {{&S93387[44], 10}, {NULL}};
const Segment SGI238625[] = {{&S93387[44], 22}, {NULL}};
const Segment SGI238626[] = {{&S93387[30], 6}, {NULL}};
const Segment SGI238627[] = {{&S93387[115], 3}, {NULL}};
const Segment SGI238628[] = {{&S93387[80], 6}, {NULL}};
const Segment SGI238629[] = {{&S93387[140], 11}, {NULL}};
const Segment SGI238630[] = {{&S85607[44], 7}, {NULL}};
const Segment SGI238631[] = {{&S85607[44], 18}, {NULL}};
const Segment SGI238632[] = {{&S85607[30], 6}, {NULL}};
const Segment SGI238633[] = {{&S93394[34], 10}, {NULL}};
const Segment SGI238634[] = {{&S85607[111], 3}, {NULL}};
const Segment SGI238635[] = {{&S93394[27], 6}, {NULL}};
const Segment SGI238636[] = {{&S85607[76], 6}, {NULL}};
const Segment SGI238637[] = {{&S93394[97], 9}, {NULL}};
const Segment SGI238638[] = {{&S93394[70], 6}, {NULL}};
const Segment SGI238639[] = {{&S85608[44], 7}, {NULL}};
const Segment SGI238640[] = {{&S85608[44], 19}, {NULL}};
const Segment SGI238641[] = {{&S85608[30], 6}, {NULL}};
const Segment SGI238642[] = {{&S93395[34], 11}, {NULL}};
const Segment SGI238643[] = {{&S85608[112], 3}, {NULL}};
const Segment SGI238644[] = {{&S93395[27], 6}, {NULL}};
const Segment SGI238645[] = {{&S85608[77], 6}, {NULL}};
const Segment SGI238646[] = {{&S93395[98], 9}, {NULL}};
const Segment SGI238647[] = {{&S93395[71], 6}, {NULL}};
const Segment SGI238648[] = {{&S85611[34], 10}, {NULL}};
const Segment SGI238649[] = {{&S85611[27], 6}, {NULL}};
const Segment SGI238650[] = {{&S85611[97], 9}, {NULL}};
const Segment SGI238651[] = {{&S85611[70], 6}, {NULL}};
const Segment SGI238652[] = {{&S93399[44], 7}, {NULL}};
const Segment SGI238653[] = {{&S93399[44], 22}, {NULL}};
const Segment SGI238654[] = {{&S93399[30], 6}, {NULL}};
const Segment SGI238655[] = {{&S93399[113], 3}, {NULL}};
const Segment SGI238656[] = {{&S93399[80], 5}, {NULL}};
const Segment SGI238657[] = {{&S93418[34], 7}, {NULL}};
const Segment SGI238658[] = {{&S93418[27], 6}, {NULL}};
const Segment SGI238659[] = {{&S93418[99], 9}, {NULL}};
const Segment SGI238660[] = {{&S93418[72], 6}, {NULL}};
const Segment SGI238661[] = {{&S85636[34], 7}, {NULL}};
const Segment SGI238662[] = {{&S85636[27], 6}, {NULL}};
const Segment SGI238663[] = {{&S85636[94], 9}, {NULL}};
const Segment SGI238664[] = {{&S85636[67], 6}, {NULL}};
const Segment SGI238665[] = {{&S93432[44], 11}, {NULL}};
const Segment SGI238666[] = {{&S93432[44], 37}, {NULL}};
const Segment SGI238667[] = {{&S93432[30], 6}, {NULL}};
const Segment SGI238668[] = {{&S93432[74], 3}, {NULL}};
const Segment SGI238669[] = {{&S93432[95], 6}, {NULL}};
const Segment SGI238670[] = {{&S93459[34], 7}, {NULL}};
const Segment SGI238671[] = {{&S93459[27], 6}, {NULL}};
const Segment SGI238672[] = {{&S93459[94], 9}, {NULL}};
const Segment SGI238673[] = {{&S93459[67], 6}, {NULL}};
const Segment SGI238674[] = {{&S93461[34], 10}, {NULL}};
const Segment SGI238675[] = {{&S93461[27], 6}, {NULL}};
const Segment SGI238676[] = {{&S93461[97], 9}, {NULL}};
const Segment SGI238677[] = {{&S93461[70], 6}, {NULL}};
const Segment SGI238678[] = {{&S85693[34], 7}, {NULL}};
const Segment SGI238679[] = {{&S93472[34], 9}, {NULL}};
const Segment SGI238680[] = {{&S85693[27], 6}, {NULL}};
const Segment SGI238681[] = {{&S93472[27], 6}, {NULL}};
const Segment SGI238682[] = {{&S85693[93], 9}, {NULL}};
const Segment SGI238683[] = {{&S93472[96], 9}, {NULL}};
const Segment SGI238684[] = {{&S85693[66], 6}, {NULL}};
const Segment SGI238685[] = {{&S93472[69], 6}, {NULL}};
const Segment SGI238686[] = {{&S93474[44], 13}, {NULL}};
const Segment SGI238687[] = {{&S93474[44], 29}, {NULL}};
const Segment SGI238688[] = {{&S93474[30], 6}, {NULL}};
const Segment SGI238689[] = {{&S93474[120], 3}, {NULL}};
const Segment SGI238690[] = {{&S93474[87], 5}, {NULL}};
const Segment SGI238691[] = {{&S93474[124], 14}, {NULL}};
const Segment SGI238692[] = {{&S93477[34], 9}, {NULL}};
const Segment SGI238693[] = {{&S93477[27], 6}, {NULL}};
const Segment SGI238694[] = {{&S93477[96], 9}, {NULL}};
const Segment SGI238695[] = {{&S93477[69], 6}, {NULL}};
const Segment SGI238696[] = {{&S103659[44], 10}, {NULL}};
const Segment SGI238697[] = {{&S103659[44], 22}, {NULL}};
const Segment SGI238698[] = {{&S103659[30], 6}, {NULL}};
const Segment SGI238699[] = {{&S103659[111], 2}, {NULL}};
const Segment SGI238700[] = {{&S103659[80], 5}, {NULL}};
const Segment SGI238701[] = {{&S103660[44], 10}, {NULL}};
const Segment SGI238702[] = {{&S103660[44], 22}, {NULL}};
const Segment SGI238703[] = {{&S103660[30], 6}, {NULL}};
const Segment SGI238704[] = {{&S103660[111], 2}, {NULL}};
const Segment SGI238705[] = {{&S103660[80], 5}, {NULL}};
const Segment SGI238706[] = {{&S93482[44], 10}, {NULL}};
const Segment SGI238707[] = {{&S93482[44], 22}, {NULL}};
const Segment SGI238708[] = {{&S103662[44], 10}, {NULL}};
const Segment SGI238709[] = {{&S93482[30], 6}, {NULL}};
const Segment SGI238710[] = {{&S103662[44], 22}, {NULL}};
const Segment SGI238711[] = {{&S103662[30], 6}, {NULL}};
const Segment SGI238712[] = {{&S103662[111], 2}, {NULL}};
const Segment SGI238713[] = {{&S93482[80], 6}, {NULL}};
const Segment SGI238714[] = {{&S103662[80], 5}, {NULL}};
const Segment SGI238715[] = {{&S93482[107], 13}, {NULL}};
const Segment SGI238716[] = {{&S93483[34], 12}, {NULL}};
const Segment SGI238717[] = {{&S93483[27], 6}, {NULL}};
const Segment SGI238718[] = {{&S93483[99], 9}, {NULL}};
const Segment SGI238719[] = {{&S93483[72], 6}, {NULL}};
const Segment SGI238720[] = {{&S93484[34], 7}, {NULL}};
const Segment SGI238721[] = {{&S93484[27], 6}, {NULL}};
const Segment SGI238722[] = {{&S93484[94], 9}, {NULL}};
const Segment SGI238723[] = {{&S93484[67], 6}, {NULL}};
const Segment SGI238724[] = {{&S100258[44], 14}, {NULL}};
const Segment SGI238725[] = {{&S100258[44], 26}, {NULL}};
const Segment SGI238726[] = {{&S100258[30], 6}, {NULL}};
const Segment SGI238727[] = {{&S100258[115], 2}, {NULL}};
const Segment SGI238728[] = {{&S100258[84], 5}, {NULL}};
const Segment SGI238729[] = {{&S93489[34], 10}, {NULL}};
const Segment SGI238730[] = {{&S93489[27], 6}, {NULL}};
const Segment SGI238731[] = {{&S93489[97], 9}, {NULL}};
const Segment SGI238732[] = {{&S93489[70], 6}, {NULL}};
const Segment SGI238733[] = {{&S100266[45], 10}, {NULL}};
const Segment SGI238734[] = {{&S100266[30], 6}, {NULL}};
const Segment SGI238735[] = {{&S100266[120], 3}, {NULL}};
const Segment SGI238736[] = {{&S100266[86], 5}, {NULL}};
const Segment SGI238737[] = {{&S100266[144], 13}, {NULL}};
const Segment SGI238738[] = {{&S94099[34], 8}, {NULL}};
const Segment SGI238739[] = {{&S94099[27], 6}, {NULL}};
const Segment SGI238740[] = {{&S93495[34], 6}, {NULL}};
const Segment SGI238741[] = {{&S93495[27], 6}, {NULL}};
const Segment SGI238742[] = {{&S94099[95], 9}, {NULL}};
const Segment SGI238743[] = {{&S94099[68], 6}, {NULL}};
const Segment SGI238744[] = {{&S93495[93], 9}, {NULL}};
const Segment SGI238745[] = {{&S93495[66], 6}, {NULL}};
const Segment SGI238746[] = {{&S103699[44], 9}, {NULL}};
const Segment SGI238747[] = {{&S103699[44], 19}, {NULL}};
const Segment SGI238748[] = {{&S103699[30], 6}, {NULL}};
const Segment SGI238749[] = {{&S103699[31], 3}, {NULL}};
const Segment SGI238750[] = {{&S103699[77], 6}, {NULL}};
const Segment SGI238751[] = {{&S94117[34], 7}, {NULL}};
const Segment SGI238752[] = {{&S94117[27], 6}, {NULL}};
const Segment SGI238753[] = {{&S94117[94], 9}, {NULL}};
const Segment SGI238754[] = {{&S94117[67], 6}, {NULL}};
const Segment SGI238755[] = {{&S94118[34], 14}, {NULL}};
const Segment SGI238756[] = {{&S94118[27], 6}, {NULL}};
const Segment SGI238757[] = {{&S94118[101], 9}, {NULL}};
const Segment SGI238758[] = {{&S94118[74], 6}, {NULL}};
const Segment SGI238759[] = {{&S100291[45], 7}, {NULL}};
const Segment SGI238760[] = {{&S100291[30], 6}, {NULL}};
const Segment SGI238761[] = {{&S100291[117], 3}, {NULL}};
const Segment SGI238762[] = {{&S100291[83], 5}, {NULL}};
const Segment SGI238763[] = {{&S100291[141], 13}, {NULL}};
const Segment SGI238764[] = {{&S100292[34], 10}, {NULL}};
const Segment SGI238765[] = {{&S100292[27], 6}, {NULL}};
const Segment SGI238766[] = {{&S100292[97], 9}, {NULL}};
const Segment SGI238767[] = {{&S100292[70], 6}, {NULL}};
const Segment SGI238768[] = {{&S103715[45], 7}, {NULL}};
const Segment SGI238769[] = {{&S103715[30], 6}, {NULL}};
const Segment SGI238770[] = {{&S103715[31], 3}, {NULL}};
const Segment SGI238771[] = {{&S94131[34], 16}, {NULL}};
const Segment SGI238772[] = {{&S103715[83], 5}, {NULL}};
const Segment SGI238773[] = {{&S94131[27], 6}, {NULL}};
const Segment SGI238774[] = {{&S103715[141], 13}, {NULL}};
const Segment SGI238775[] = {{&S94131[103], 9}, {NULL}};
const Segment SGI238776[] = {{&S94131[76], 6}, {NULL}};
const Segment SGI238777[] = {{&S103718[45], 7}, {NULL}};
const Segment SGI238778[] = {{&S103718[30], 6}, {NULL}};
const Segment SGI238779[] = {{&S103718[31], 3}, {NULL}};
const Segment SGI238780[] = {{&S103718[83], 5}, {NULL}};
const Segment SGI238781[] = {{&S103718[141], 13}, {NULL}};
const Segment SGI238782[] = {{&S103746[44], 10}, {NULL}};
const Segment SGI238783[] = {{&S103746[44], 22}, {NULL}};
const Segment SGI238784[] = {{&S103746[30], 6}, {NULL}};
const Segment SGI238785[] = {{&S103746[31], 3}, {NULL}};
const Segment SGI238786[] = {{&S103746[80], 6}, {NULL}};
const Segment SGI238787[] = {{&S103746[143], 3}, {NULL}};
const Segment SGI238788[] = {{&S103748[59], 10}, {NULL}};
const Segment SGI238789[] = {{&S103748[59], 22}, {NULL}};
const Segment SGI238790[] = {{&S103748[45], 6}, {NULL}};
const Segment SGI238791[] = {{&S103748[95], 5}, {NULL}};
const Segment SGI238792[] = {{&S103748[0], 14}, {NULL}};
const Segment SGI238793[] = {{&S103750[44], 10}, {NULL}};
const Segment SGI238794[] = {{&S103750[44], 22}, {NULL}};
const Segment SGI238795[] = {{&S103750[30], 6}, {NULL}};
const Segment SGI238796[] = {{&S103750[31], 3}, {NULL}};
const Segment SGI238797[] = {{&S103750[80], 5}, {NULL}};
const Segment SGI238798[] = {{&S103750[125], 14}, {NULL}};
const Segment SGI238799[] = {{&S94177[34], 8}, {NULL}};
const Segment SGI238800[] = {{&S94177[27], 6}, {NULL}};
const Segment SGI238801[] = {{&S103754[45], 10}, {NULL}};
const Segment SGI238802[] = {{&S103754[30], 6}, {NULL}};
const Segment SGI238803[] = {{&S94177[95], 9}, {NULL}};
const Segment SGI238804[] = {{&S94177[68], 6}, {NULL}};
const Segment SGI238805[] = {{&S103754[31], 3}, {NULL}};
const Segment SGI238806[] = {{&S100346[45], 12}, {NULL}};
const Segment SGI238807[] = {{&S103754[81], 5}, {NULL}};
const Segment SGI238808[] = {{&S100346[30], 6}, {NULL}};
const Segment SGI238809[] = {{&S103754[139], 13}, {NULL}};
const Segment SGI238810[] = {{&S100346[117], 3}, {NULL}};
const Segment SGI238811[] = {{&S100346[83], 5}, {NULL}};
const Segment SGI238812[] = {{&S100346[141], 13}, {NULL}};
const Segment SGI238813[] = {{&S93580[44], 12}, {NULL}};
const Segment SGI238814[] = {{&S93580[44], 24}, {NULL}};
const Segment SGI238815[] = {{&S93580[30], 6}, {NULL}};
const Segment SGI238816[] = {{&S93580[117], 3}, {NULL}};
const Segment SGI238817[] = {{&S103756[59], 10}, {NULL}};
const Segment SGI238818[] = {{&S93580[82], 6}, {NULL}};
const Segment SGI238819[] = {{&S103756[59], 22}, {NULL}};
const Segment SGI238820[] = {{&S103756[45], 6}, {NULL}};
const Segment SGI238821[] = {{&S103756[46], 3}, {NULL}};
const Segment SGI238822[] = {{&S103756[95], 6}, {NULL}};
const Segment SGI238823[] = {{&S103756[0], 14}, {NULL}};
const Segment SGI238824[] = {{&S103757[59], 10}, {NULL}};
const Segment SGI238825[] = {{&S103757[59], 22}, {NULL}};
const Segment SGI238826[] = {{&S103757[45], 6}, {NULL}};
const Segment SGI238827[] = {{&S103757[46], 3}, {NULL}};
const Segment SGI238828[] = {{&S103757[95], 6}, {NULL}};
const Segment SGI238829[] = {{&S103757[0], 14}, {NULL}};
const Segment SGI238830[] = {{&S103761[44], 10}, {NULL}};
const Segment SGI238831[] = {{&S103761[44], 22}, {NULL}};
const Segment SGI238832[] = {{&S103761[30], 6}, {NULL}};
const Segment SGI238833[] = {{&S103761[139], 4}, {NULL}};
const Segment SGI238834[] = {{&S103761[31], 3}, {&S103761[192], 5}, {NULL}};
const Segment SGI238835[] = {{&S103761[80], 6}, {NULL}};
const Segment SGI238836[] = {{&S103764[44], 10}, {NULL}};
const Segment SGI238837[] = {{&S103764[44], 22}, {NULL}};
const Segment SGI238838[] = {{&S103764[30], 6}, {NULL}};
const Segment SGI238839[] = {{&S103764[31], 3}, {NULL}};
const Segment SGI238840[] = {{&S103764[80], 6}, {NULL}};
const Segment SGI238841[] = {{&S103764[133], 11}, {NULL}};
const Segment SGI238842[] = {{&S85825[44], 10}, {NULL}};
const Segment SGI238843[] = {{&S85825[44], 21}, {NULL}};
const Segment SGI238844[] = {{&S85825[30], 6}, {NULL}};
const Segment SGI238845[] = {{&S103767[31], 8}, {NULL}};
const Segment SGI238846[] = {{&S103767[55], 3}, {NULL}};
const Segment SGI238847[] = {{&S85825[114], 3}, {NULL}};
const Segment SGI238848[] = {{&S85825[79], 6}, {NULL}};
const Segment SGI238849[] = {{&S85825[142], 3}, {NULL}};
const Segment SGI238850[] = {{&S85830[44], 9}, {NULL}};
const Segment SGI238851[] = {{&S85830[44], 20}, {NULL}};
const Segment SGI238852[] = {{&S85830[30], 6}, {NULL}};
const Segment SGI238853[] = {{&S85830[113], 3}, {NULL}};
const Segment SGI238854[] = {{&S100372[34], 11}, {NULL}};
const Segment SGI238855[] = {{&S85830[78], 6}, {NULL}};
const Segment SGI238856[] = {{&S100372[27], 6}, {NULL}};
const Segment SGI238857[] = {{&S85830[130], 10}, {NULL}};
const Segment SGI238858[] = {{&S100372[98], 9}, {NULL}};
const Segment SGI238859[] = {{&S100372[71], 6}, {NULL}};
const Segment SGI238860[] = {{&S103777[43], 10}, {NULL}};
const Segment SGI238861[] = {{&S103777[43], 22}, {NULL}};
const Segment SGI238862[] = {{&S103777[29], 6}, {NULL}};
const Segment SGI238863[] = {{&S103777[79], 6}, {NULL}};
const Segment SGI238864[] = {{&S103777[106], 13}, {NULL}};
const Segment SGI238865[] = {{&S103779[44], 10}, {NULL}};
const Segment SGI238866[] = {{&S103779[44], 22}, {NULL}};
const Segment SGI238867[] = {{&S103779[30], 6}, {NULL}};
const Segment SGI238868[] = {{&S103779[31], 3}, {NULL}};
const Segment SGI238869[] = {{&S103779[80], 5}, {NULL}};
const Segment SGI238870[] = {{&S103779[117], 14}, {NULL}};
const Segment SGI238871[] = {{&S100393[34], 6}, {NULL}};
const Segment SGI238872[] = {{&S100393[27], 6}, {NULL}};
const Segment SGI238873[] = {{&S100393[93], 9}, {NULL}};
const Segment SGI238874[] = {{&S100393[66], 6}, {NULL}};
const Segment SGI238875[] = {{&S103795[45], 10}, {NULL}};
const Segment SGI238876[] = {{&S103795[30], 6}, {NULL}};
const Segment SGI238877[] = {{&S103795[31], 3}, {NULL}};
const Segment SGI238878[] = {{&S103795[81], 5}, {NULL}};
const Segment SGI238879[] = {{&S103795[139], 13}, {NULL}};
const Segment SGI238880[] = {{&S100398[34], 10}, {NULL}};
const Segment SGI238881[] = {{&S100398[27], 6}, {NULL}};
const Segment SGI238882[] = {{&S100398[97], 9}, {NULL}};
const Segment SGI238883[] = {{&S100398[70], 6}, {NULL}};
const Segment SGI238884[] = {{&S85869[34], 10}, {NULL}};
const Segment SGI238885[] = {{&S85869[27], 6}, {NULL}};
const Segment SGI238886[] = {{&S85869[96], 9}, {NULL}};
const Segment SGI238887[] = {{&S85869[69], 6}, {NULL}};
const Segment SGI238888[] = {{&S85876[34], 19}, {NULL}};
const Segment SGI238889[] = {{&S85876[27], 6}, {NULL}};
const Segment SGI238890[] = {{&S100415[44], 6}, {NULL}};
const Segment SGI238891[] = {{&S100415[44], 18}, {NULL}};
const Segment SGI238892[] = {{&S100415[30], 6}, {NULL}};
const Segment SGI238893[] = {{&S85876[106], 9}, {NULL}};
const Segment SGI238894[] = {{&S85876[79], 6}, {NULL}};
const Segment SGI238895[] = {{&S100415[109], 3}, {NULL}};
const Segment SGI238896[] = {{&S100415[76], 5}, {NULL}};
const Segment SGI238897[] = {{&S100415[113], 14}, {NULL}};
const Segment SGI238898[] = {{&S85885[34], 9}, {NULL}};
const Segment SGI238899[] = {{&S85885[27], 6}, {NULL}};
const Segment SGI238900[] = {{&S85885[95], 9}, {NULL}};
const Segment SGI238901[] = {{&S85885[68], 6}, {NULL}};
const Segment SGI238902[] = {{&S100432[34], 13}, {NULL}};
const Segment SGI238903[] = {{&S100432[27], 6}, {NULL}};
const Segment SGI238904[] = {{&S100432[100], 9}, {NULL}};
const Segment SGI238905[] = {{&S100432[73], 6}, {NULL}};
const Segment SGI238906[] = {{&S100437[34], 9}, {NULL}};
const Segment SGI238907[] = {{&S100437[27], 6}, {NULL}};
const Segment SGI238908[] = {{&S100437[96], 9}, {NULL}};
const Segment SGI238909[] = {{&S100437[69], 6}, {NULL}};
const Segment SGI238910[] = {{&S100437[130], 8}, {NULL}};
const Segment SGI238911[] = {{&S100444[34], 4}, {NULL}};
const Segment SGI238912[] = {{&S100444[27], 6}, {NULL}};
const Segment SGI238913[] = {{&S100444[91], 9}, {NULL}};
const Segment SGI238914[] = {{&S100444[64], 6}, {NULL}};
const Segment SGI238915[] = {{&S100471[44], 6}, {NULL}};
const Segment SGI238916[] = {{&S100471[44], 18}, {NULL}};
const Segment SGI238917[] = {{&S100471[30], 6}, {NULL}};
const Segment SGI238918[] = {{&S100471[111], 3}, {NULL}};
const Segment SGI238919[] = {{&S100471[76], 6}, {NULL}};
const Segment SGI238920[] = {{&S100471[136], 11}, {NULL}};
const Segment SGI238921[] = {{&S100472[34], 8}, {NULL}};
const Segment SGI238922[] = {{&S100472[27], 6}, {NULL}};
const Segment SGI238923[] = {{&S100472[95], 9}, {NULL}};
const Segment SGI238924[] = {{&S100472[68], 6}, {NULL}};
const Segment SGI238925[] = {{&S94332[34], 8}, {NULL}};
const Segment SGI238926[] = {{&S94332[27], 6}, {NULL}};
const Segment SGI238927[] = {{&S94332[95], 9}, {NULL}};
const Segment SGI238928[] = {{&S94332[68], 6}, {NULL}};
const Segment SGI238929[] = {{&S85959[6], 4}, {NULL}};
const Segment SGI238930[] = {{&S85959[86], 5}, {NULL}};
const Segment SGI238931[] = {{&S100492[34], 11}, {NULL}};
const Segment SGI238932[] = {{&S100492[27], 6}, {NULL}};
const Segment SGI238933[] = {{&S100492[98], 9}, {NULL}};
const Segment SGI238934[] = {{&S100492[71], 6}, {NULL}};
const Segment SGI238935[] = {{&S94344[34], 10}, {NULL}};
const Segment SGI238936[] = {{&S94344[27], 6}, {NULL}};
const Segment SGI238937[] = {{&S94344[93], 9}, {NULL}};
const Segment SGI238938[] = {{&S94344[66], 6}, {NULL}};
const Segment SGI238939[] = {{&S100504[34], 7}, {NULL}};
const Segment SGI238940[] = {{&S100504[27], 6}, {NULL}};
const Segment SGI238941[] = {{&S100504[94], 9}, {NULL}};
const Segment SGI238942[] = {{&S100504[67], 6}, {NULL}};
const Segment SGI238943[] = {{&S100504[135], 8}, {NULL}};
const Segment SGI238944[] = {{&S85982[34], 10}, {NULL}};
const Segment SGI238945[] = {{&S85982[27], 6}, {NULL}};
const Segment SGI238946[] = {{&S85982[96], 9}, {NULL}};
const Segment SGI238947[] = {{&S85982[69], 6}, {NULL}};
const Segment SGI238948[] = {{&S94357[34], 14}, {NULL}};
const Segment SGI238949[] = {{&S94357[27], 6}, {NULL}};
const Segment SGI238950[] = {{&S94357[101], 9}, {NULL}};
const Segment SGI238951[] = {{&S94357[74], 6}, {NULL}};
const Segment SGI238952[] = {{&S85996[44], 7}, {NULL}};
const Segment SGI238953[] = {{&S85996[44], 18}, {NULL}};
const Segment SGI238954[] = {{&S85996[30], 6}, {NULL}};
const Segment SGI238955[] = {{&S85996[31], 3}, {NULL}};
const Segment SGI238956[] = {{&S85996[76], 6}, {NULL}};
const Segment SGI238957[] = {{&S85997[44], 7}, {NULL}};
const Segment SGI238958[] = {{&S85997[44], 18}, {NULL}};
const Segment SGI238959[] = {{&S85997[30], 6}, {NULL}};
const Segment SGI238960[] = {{&S85997[111], 3}, {NULL}};
const Segment SGI238961[] = {{&S85997[76], 6}, {NULL}};
const Segment SGI238962[] = {{&S100531[34], 7}, {NULL}};
const Segment SGI238963[] = {{&S100531[27], 6}, {NULL}};
const Segment SGI238964[] = {{&S100531[94], 9}, {NULL}};
const Segment SGI238965[] = {{&S100531[67], 6}, {NULL}};
const Segment SGI238966[] = {{&S93627[34], 10}, {NULL}};
const Segment SGI238967[] = {{&S93627[27], 6}, {NULL}};
const Segment SGI238968[] = {{&S93627[97], 9}, {NULL}};
const Segment SGI238969[] = {{&S93627[70], 6}, {NULL}};
const Segment SGI238970[] = {{&S86010[34], 11}, {NULL}};
const Segment SGI238971[] = {{&S93630[34], 10}, {NULL}};
const Segment SGI238972[] = {{&S86010[27], 6}, {NULL}};
const Segment SGI238973[] = {{&S93630[27], 6}, {NULL}};
const Segment SGI238974[] = {{&S86010[97], 9}, {NULL}};
const Segment SGI238975[] = {{&S86010[70], 6}, {NULL}};
const Segment SGI238976[] = {{&S93630[97], 9}, {NULL}};
const Segment SGI238977[] = {{&S93630[70], 6}, {NULL}};
const Segment SGI238978[] = {{&S100542[44], 9}, {NULL}};
const Segment SGI238979[] = {{&S100542[44], 21}, {NULL}};
const Segment SGI238980[] = {{&S100542[30], 6}, {NULL}};
const Segment SGI238981[] = {{&S100542[114], 3}, {NULL}};
const Segment SGI238982[] = {{&S100542[79], 6}, {NULL}};
const Segment SGI238983[] = {{&S100543[34], 7}, {NULL}};
const Segment SGI238984[] = {{&S100543[27], 6}, {NULL}};
const Segment SGI238985[] = {{&S100543[94], 9}, {NULL}};
const Segment SGI238986[] = {{&S100543[67], 6}, {NULL}};
const Segment SGI238987[] = {{&S93640[44], 12}, {NULL}};
const Segment SGI238988[] = {{&S93640[44], 24}, {NULL}};
const Segment SGI238989[] = {{&S93640[30], 6}, {NULL}};
const Segment SGI238990[] = {{&S103971[44], 13}, {NULL}};
const Segment SGI238991[] = {{&S93640[133], 3}, {NULL}};
const Segment SGI238992[] = {{&S103971[44], 25}, {NULL}};
const Segment SGI238993[] = {{&S93640[98], 6}, {NULL}};
const Segment SGI238994[] = {{&S103971[30], 6}, {NULL}};
const Segment SGI238995[] = {{&S93640[70], 8}, {NULL}};
const Segment SGI238996[] = {{&S103971[31], 3}, {NULL}};
const Segment SGI238997[] = {{&S103971[102], 6}, {NULL}};
const Segment SGI238998[] = {{&S103971[71], 8}, {NULL}};
const Segment SGI238999[] = {{&S103972[44], 13}, {NULL}};
const Segment SGI239000[] = {{&S103972[44], 25}, {NULL}};
const Segment SGI239001[] = {{&S103972[30], 6}, {NULL}};
const Segment SGI239002[] = {{&S103972[31], 3}, {NULL}};
const Segment SGI239003[] = {{&S103972[98], 6}, {NULL}};
const Segment SGI239004[] = {{&S103972[71], 8}, {NULL}};
const Segment SGI239005[] = {{&S93648[34], 10}, {NULL}};
const Segment SGI239006[] = {{&S93648[27], 6}, {NULL}};
const Segment SGI239007[] = {{&S93648[97], 9}, {NULL}};
const Segment SGI239008[] = {{&S93648[70], 6}, {NULL}};
const Segment SGI239009[] = {{&S100560[34], 15}, {NULL}};
const Segment SGI239010[] = {{&S100560[27], 6}, {NULL}};
const Segment SGI239011[] = {{&S100560[102], 9}, {NULL}};
const Segment SGI239012[] = {{&S100560[75], 6}, {NULL}};
const Segment SGI239013[] = {{&S100562[44], 8}, {NULL}};
const Segment SGI239014[] = {{&S100562[44], 20}, {NULL}};
const Segment SGI239015[] = {{&S100562[30], 6}, {NULL}};
const Segment SGI239016[] = {{&S100562[113], 3}, {NULL}};
const Segment SGI239017[] = {{&S100562[78], 6}, {NULL}};
const Segment SGI239018[] = {{&S93668[44], 13}, {NULL}};
const Segment SGI239019[] = {{&S93668[44], 25}, {NULL}};
const Segment SGI239020[] = {{&S93668[30], 6}, {NULL}};
const Segment SGI239021[] = {{&S100574[44], 6}, {NULL}};
const Segment SGI239022[] = {{&S100574[44], 18}, {NULL}};
const Segment SGI239023[] = {{&S100574[30], 6}, {NULL}};
const Segment SGI239024[] = {{&S93668[116], 3}, {NULL}};
const Segment SGI239025[] = {{&S93668[83], 5}, {NULL}};
const Segment SGI239026[] = {{&S100574[76], 6}, {NULL}};
const Segment SGI239027[] = {{&S100574[103], 13}, {NULL}};
const Segment SGI239028[] = {{&S93669[34], 9}, {NULL}};
const Segment SGI239029[] = {{&S93669[27], 6}, {NULL}};
const Segment SGI239030[] = {{&S93669[96], 9}, {NULL}};
const Segment SGI239031[] = {{&S93669[69], 6}, {NULL}};
const Segment SGI239032[] = {{&S104008[44], 12}, {NULL}};
const Segment SGI239033[] = {{&S104008[44], 24}, {NULL}};
const Segment SGI239034[] = {{&S104008[30], 6}, {NULL}};
const Segment SGI239035[] = {{&S104008[113], 2}, {NULL}};
const Segment SGI239036[] = {{&S104008[82], 5}, {NULL}};
const Segment SGI239037[] = {{&S104012[44], 9}, {NULL}};
const Segment SGI239038[] = {{&S104012[44], 21}, {NULL}};
const Segment SGI239039[] = {{&S104012[30], 6}, {NULL}};
const Segment SGI239040[] = {{&S104012[31], 3}, {NULL}};
const Segment SGI239041[] = {{&S104012[108], 6}, {NULL}};
const Segment SGI239042[] = {{&S104012[67], 8}, {NULL}};
const Segment SGI239043[] = {{&S104012[79], 15}, {NULL}};
const Segment SGI239044[] = {{&S86074[34], 12}, {NULL}};
const Segment SGI239045[] = {{&S86074[27], 6}, {NULL}};
const Segment SGI239046[] = {{&S86074[99], 9}, {NULL}};
const Segment SGI239047[] = {{&S86074[72], 6}, {NULL}};
const Segment SGI239048[] = {{&S100596[44], 12}, {NULL}};
const Segment SGI239049[] = {{&S93691[34], 10}, {NULL}};
const Segment SGI239050[] = {{&S100596[44], 24}, {NULL}};
const Segment SGI239051[] = {{&S93691[27], 6}, {NULL}};
const Segment SGI239052[] = {{&S100596[30], 6}, {NULL}};
const Segment SGI239053[] = {{&S93691[97], 9}, {NULL}};
const Segment SGI239054[] = {{&S93691[70], 6}, {NULL}};
const Segment SGI239055[] = {{&S100596[82], 6}, {NULL}};
const Segment SGI239056[] = {{&S100596[109], 13}, {NULL}};
const Segment SGI239057[] = {{&S86080[34], 11}, {NULL}};
const Segment SGI239058[] = {{&S86080[27], 6}, {NULL}};
const Segment SGI239059[] = {{&S86080[97], 9}, {NULL}};
const Segment SGI239060[] = {{&S86080[70], 6}, {NULL}};
const Segment SGI239061[] = {{&S93696[44], 23}, {NULL}};
const Segment SGI239062[] = {{&S93696[44], 34}, {NULL}};
const Segment SGI239063[] = {{&S93696[30], 6}, {NULL}};
const Segment SGI239064[] = {{&S93696[127], 3}, {NULL}};
const Segment SGI239065[] = {{&S93696[92], 6}, {NULL}};
const Segment SGI239066[] = {{&S104022[44], 10}, {NULL}};
const Segment SGI239067[] = {{&S104022[44], 22}, {NULL}};
const Segment SGI239068[] = {{&S104022[30], 6}, {NULL}};
const Segment SGI239069[] = {{&S104022[31], 3}, {NULL}};
const Segment SGI239070[] = {{&S104022[80], 6}, {NULL}};
const Segment SGI239071[] = {{&S104023[34], 10}, {NULL}};
const Segment SGI239072[] = {{&S104023[27], 6}, {NULL}};
const Segment SGI239073[] = {{&S104023[97], 9}, {NULL}};
const Segment SGI239074[] = {{&S104023[70], 6}, {NULL}};
const Segment SGI239075[] = {{&S86086[34], 7}, {NULL}};
const Segment SGI239076[] = {{&S86086[27], 6}, {NULL}};
const Segment SGI239077[] = {{&S86086[94], 9}, {NULL}};
const Segment SGI239078[] = {{&S86086[67], 6}, {NULL}};
const Segment SGI239079[] = {{&S93701[34], 10}, {NULL}};
const Segment SGI239080[] = {{&S93701[27], 6}, {NULL}};
const Segment SGI239081[] = {{&S93701[97], 9}, {NULL}};
const Segment SGI239082[] = {{&S93701[70], 6}, {NULL}};
const Segment SGI239083[] = {{&S104027[44], 13}, {NULL}};
const Segment SGI239084[] = {{&S104027[44], 29}, {NULL}};
const Segment SGI239085[] = {{&S104027[30], 6}, {NULL}};
const Segment SGI239086[] = {{&S104027[31], 3}, {NULL}};
const Segment SGI239087[] = {{&S104027[87], 6}, {NULL}};
const Segment SGI239088[] = {{&S104027[147], 11}, {NULL}};
const Segment SGI239089[] = {{&S94473[34], 6}, {NULL}};
const Segment SGI239090[] = {{&S94473[27], 6}, {NULL}};
const Segment SGI239091[] = {{&S94473[93], 9}, {NULL}};
const Segment SGI239092[] = {{&S94473[66], 6}, {NULL}};
const Segment SGI239093[] = {{&S100614[44], 10}, {NULL}};
const Segment SGI239094[] = {{&S100614[44], 22}, {NULL}};
const Segment SGI239095[] = {{&S100614[30], 6}, {NULL}};
const Segment SGI239096[] = {{&S100614[80], 6}, {NULL}};
const Segment SGI239097[] = {{&S100614[107], 13}, {NULL}};
const Segment SGI239098[] = {{&S94475[34], 10}, {NULL}};
const Segment SGI239099[] = {{&S94475[27], 6}, {NULL}};
const Segment SGI239100[] = {{&S94475[97], 9}, {NULL}};
const Segment SGI239101[] = {{&S94475[70], 6}, {NULL}};
const Segment SGI239102[] = {{&S86101[34], 7}, {NULL}};
const Segment SGI239103[] = {{&S86101[27], 6}, {NULL}};
const Segment SGI239104[] = {{&S86101[95], 9}, {NULL}};
const Segment SGI239105[] = {{&S86101[68], 6}, {NULL}};
const Segment SGI239106[] = {{&S86103[34], 8}, {NULL}};
const Segment SGI239107[] = {{&S86103[27], 6}, {NULL}};
const Segment SGI239108[] = {{&S86103[94], 9}, {NULL}};
const Segment SGI239109[] = {{&S86103[67], 6}, {NULL}};
const Segment SGI239110[] = {{&S104042[44], 10}, {NULL}};
const Segment SGI239111[] = {{&S104042[44], 22}, {NULL}};
const Segment SGI239112[] = {{&S104042[30], 6}, {NULL}};
const Segment SGI239113[] = {{&S104042[31], 3}, {NULL}};
const Segment SGI239114[] = {{&S104042[87], 6}, {NULL}};
const Segment SGI239115[] = {{&S104042[66], 8}, {NULL}};
const Segment SGI239116[] = {{&S94492[34], 10}, {NULL}};
const Segment SGI239117[] = {{&S94492[27], 6}, {NULL}};
const Segment SGI239118[] = {{&S94492[97], 9}, {NULL}};
const Segment SGI239119[] = {{&S94492[70], 6}, {NULL}};
const Segment SGI239120[] = {{&S100633[44], 6}, {NULL}};
const Segment SGI239121[] = {{&S100633[44], 18}, {NULL}};
const Segment SGI239122[] = {{&S100633[30], 6}, {NULL}};
const Segment SGI239123[] = {{&S100633[111], 3}, {NULL}};
const Segment SGI239124[] = {{&S100633[76], 6}, {NULL}};
const Segment SGI239125[] = {{&S104055[45], 10}, {NULL}};
const Segment SGI239126[] = {{&S104055[30], 6}, {NULL}};
const Segment SGI239127[] = {{&S104055[31], 3}, {NULL}};
const Segment SGI239128[] = {{&S104055[81], 5}, {NULL}};
const Segment SGI239129[] = {{&S104055[139], 13}, {NULL}};
const Segment SGI239130[] = {{&S104057[59], 10}, {NULL}};
const Segment SGI239131[] = {{&S104057[59], 22}, {NULL}};
const Segment SGI239132[] = {{&S104057[45], 6}, {NULL}};
const Segment SGI239133[] = {{&S104057[46], 3}, {NULL}};
const Segment SGI239134[] = {{&S104057[95], 6}, {NULL}};
const Segment SGI239135[] = {{&S104057[0], 14}, {NULL}};
const Segment SGI239136[] = {{&S104060[45], 10}, {NULL}};
const Segment SGI239137[] = {{&S104060[30], 6}, {NULL}};
const Segment SGI239138[] = {{&S104060[31], 3}, {NULL}};
const Segment SGI239139[] = {{&S104060[81], 5}, {NULL}};
const Segment SGI239140[] = {{&S104060[139], 13}, {NULL}};
const Segment SGI239141[] = {{&S100656[44], 18}, {NULL}};
const Segment SGI239142[] = {{&S100656[44], 39}, {NULL}};
const Segment SGI239143[] = {{&S100656[30], 6}, {NULL}};
const Segment SGI239144[] = {{&S100656[97], 6}, {NULL}};
const Segment SGI239145[] = {{&S100656[124], 13}, {NULL}};
const Segment SGI239146[] = {{&S104067[59], 10}, {NULL}};
const Segment SGI239147[] = {{&S104067[59], 22}, {NULL}};
const Segment SGI239148[] = {{&S104067[45], 6}, {NULL}};
const Segment SGI239149[] = {{&S104067[46], 3}, {NULL}};
const Segment SGI239150[] = {{&S104067[95], 6}, {NULL}};
const Segment SGI239151[] = {{&S104067[0], 14}, {NULL}};
const Segment SGI239152[] = {{&S100660[44], 18}, {NULL}};
const Segment SGI239153[] = {{&S100660[44], 39}, {NULL}};
const Segment SGI239154[] = {{&S100660[30], 6}, {NULL}};
const Segment SGI239155[] = {{&S100660[130], 3}, {NULL}};
const Segment SGI239156[] = {{&S100660[97], 5}, {NULL}};
const Segment SGI239157[] = {{&S100660[134], 14}, {NULL}};
const Segment SGI239158[] = {{&S104073[59], 10}, {NULL}};
const Segment SGI239159[] = {{&S104073[59], 22}, {NULL}};
const Segment SGI239160[] = {{&S104073[45], 6}, {NULL}};
const Segment SGI239161[] = {{&S104073[46], 3}, {NULL}};
const Segment SGI239162[] = {{&S104073[95], 6}, {NULL}};
const Segment SGI239163[] = {{&S104073[0], 14}, {NULL}};
const Segment SGI239164[] = {{&S104078[44], 10}, {NULL}};
const Segment SGI239165[] = {{&S104078[44], 22}, {NULL}};
const Segment SGI239166[] = {{&S104078[30], 6}, {NULL}};
const Segment SGI239167[] = {{&S104078[31], 3}, {NULL}};
const Segment SGI239168[] = {{&S100671[34], 25}, {NULL}};
const Segment SGI239169[] = {{&S104078[95], 6}, {NULL}};
const Segment SGI239170[] = {{&S100671[27], 6}, {NULL}};
const Segment SGI239171[] = {{&S104078[158], 3}, {NULL}};
const Segment SGI239172[] = {{&S104078[68], 8}, {NULL}};
const Segment SGI239173[] = {{&S100671[112], 9}, {NULL}};
const Segment SGI239174[] = {{&S100671[85], 6}, {NULL}};
const Segment SGI239175[] = {{&S100671[153], 8}, {NULL}};
const Segment SGI239176[] = {{&S104095[44], 10}, {NULL}};
const Segment SGI239177[] = {{&S104095[44], 22}, {NULL}};
const Segment SGI239178[] = {{&S104095[30], 6}, {NULL}};
const Segment SGI239179[] = {{&S104095[31], 3}, {NULL}};
const Segment SGI239180[] = {{&S104095[80], 5}, {NULL}};
const Segment SGI239181[] = {{&S104095[117], 14}, {NULL}};
const Segment SGI239182[] = {{&S104102[44], 10}, {NULL}};
const Segment SGI239183[] = {{&S104102[44], 22}, {NULL}};
const Segment SGI239184[] = {{&S104102[30], 6}, {NULL}};
const Segment SGI239185[] = {{&S104102[31], 3}, {NULL}};
const Segment SGI239186[] = {{&S104102[80], 6}, {NULL}};
const Segment SGI239187[] = {{&S104102[133], 13}, {NULL}};
const Segment SGI239188[] = {{&S104105[45], 10}, {NULL}};
const Segment SGI239189[] = {{&S104105[30], 6}, {NULL}};
const Segment SGI239190[] = {{&S104105[31], 3}, {NULL}};
const Segment SGI239191[] = {{&S104105[81], 5}, {NULL}};
const Segment SGI239192[] = {{&S104105[139], 13}, {NULL}};
const Segment SGI239193[] = {{&S104106[44], 10}, {NULL}};
const Segment SGI239194[] = {{&S104106[44], 22}, {NULL}};
const Segment SGI239195[] = {{&S104106[30], 6}, {NULL}};
const Segment SGI239196[] = {{&S104106[115], 3}, {NULL}};
const Segment SGI239197[] = {{&S104106[80], 6}, {NULL}};
const Segment SGI239198[] = {{&S104106[135], 8}, {NULL}};
const Segment SGI239199[] = {{&S100706[44], 11}, {NULL}};
const Segment SGI239200[] = {{&S104106[144], 13}, {NULL}};
const Segment SGI239201[] = {{&S100706[44], 23}, {NULL}};
const Segment SGI239202[] = {{&S100706[30], 6}, {NULL}};
const Segment SGI239203[] = {{&S100706[81], 6}, {NULL}};
const Segment SGI239204[] = {{&S100706[108], 13}, {NULL}};
const Segment SGI239205[] = {{&S104107[44], 10}, {NULL}};
const Segment SGI239206[] = {{&S104107[44], 22}, {NULL}};
const Segment SGI239207[] = {{&S104107[30], 6}, {NULL}};
const Segment SGI239208[] = {{&S104107[31], 3}, {NULL}};
const Segment SGI239209[] = {{&S104107[80], 6}, {NULL}};
const Segment SGI239210[] = {{&S104107[133], 11}, {NULL}};
const Segment SGI239211[] = {{&S94570[44], 13}, {NULL}};
const Segment SGI239212[] = {{&S94570[44], 25}, {NULL}};
const Segment SGI239213[] = {{&S94570[30], 6}, {NULL}};
const Segment SGI239214[] = {{&S94570[118], 3}, {NULL}};
const Segment SGI239215[] = {{&S104108[44], 10}, {NULL}};
const Segment SGI239216[] = {{&S104108[44], 22}, {NULL}};
const Segment SGI239217[] = {{&S94570[83], 6}, {NULL}};
const Segment SGI239218[] = {{&S104108[30], 6}, {NULL}};
const Segment SGI239219[] = {{&S104108[80], 6}, {NULL}};
const Segment SGI239220[] = {{&S104108[107], 13}, {NULL}};
const Segment SGI239221[] = {{&S104109[44], 10}, {NULL}};
const Segment SGI239222[] = {{&S104109[44], 22}, {NULL}};
const Segment SGI239223[] = {{&S104109[30], 6}, {NULL}};
const Segment SGI239224[] = {{&S104109[80], 6}, {NULL}};
const Segment SGI239225[] = {{&S104109[107], 13}, {NULL}};
const Segment SGI239226[] = {{&S104110[44], 10}, {NULL}};
const Segment SGI239227[] = {{&S104110[44], 22}, {NULL}};
const Segment SGI239228[] = {{&S104110[30], 6}, {NULL}};
const Segment SGI239229[] = {{&S94574[34], 13}, {NULL}};
const Segment SGI239230[] = {{&S94574[27], 6}, {NULL}};
const Segment SGI239231[] = {{&S104110[80], 6}, {NULL}};
const Segment SGI239232[] = {{&S94574[100], 9}, {NULL}};
const Segment SGI239233[] = {{&S104110[107], 13}, {NULL}};
const Segment SGI239234[] = {{&S94574[73], 6}, {NULL}};
const Segment SGI239235[] = {{&S104115[34], 10}, {NULL}};
const Segment SGI239236[] = {{&S104115[27], 6}, {NULL}};
const Segment SGI239237[] = {{&S104115[97], 9}, {NULL}};
const Segment SGI239238[] = {{&S104115[70], 6}, {NULL}};
const Segment SGI239239[] = {{&S104115[132], 8}, {NULL}};
const Segment SGI239240[] = {{&S94581[44], 10}, {NULL}};
const Segment SGI239241[] = {{&S94581[44], 22}, {NULL}};
const Segment SGI239242[] = {{&S94581[30], 6}, {NULL}};
const Segment SGI239243[] = {{&S94581[80], 6}, {NULL}};
const Segment SGI239244[] = {{&S94581[117], 3}, {NULL}};
const Segment SGI239245[] = {{&S104148[34], 10}, {NULL}};
const Segment SGI239246[] = {{&S104148[27], 6}, {NULL}};
const Segment SGI239247[] = {{&S104148[104], 9}, {NULL}};
const Segment SGI239248[] = {{&S104148[77], 6}, {NULL}};
const Segment SGI239249[] = {{&S104148[145], 8}, {NULL}};
const Segment SGI239250[] = {{&S94613[44], 6}, {NULL}};
const Segment SGI239251[] = {{&S94613[44], 20}, {NULL}};
const Segment SGI239252[] = {{&S94613[30], 6}, {NULL}};
const Segment SGI239253[] = {{&S94613[78], 6}, {NULL}};
const Segment SGI239254[] = {{&S94613[115], 3}, {NULL}};
const Segment SGI239255[] = {{&S93865[34], 8}, {NULL}};
const Segment SGI239256[] = {{&S93865[27], 6}, {NULL}};
const Segment SGI239257[] = {{&S93865[95], 9}, {NULL}};
const Segment SGI239258[] = {{&S93865[68], 6}, {NULL}};
const Segment SGI239259[] = {{&S93868[34], 14}, {NULL}};
const Segment SGI239260[] = {{&S93868[27], 6}, {NULL}};
const Segment SGI239261[] = {{&S93868[101], 9}, {NULL}};
const Segment SGI239262[] = {{&S93868[74], 6}, {NULL}};
const Segment SGI239263[] = {{&S93874[44], 10}, {NULL}};
const Segment SGI239264[] = {{&S93874[44], 22}, {NULL}};
const Segment SGI239265[] = {{&S93874[30], 6}, {NULL}};
const Segment SGI239266[] = {{&S93874[131], 3}, {NULL}};
const Segment SGI239267[] = {{&S93874[96], 6}, {NULL}};
const Segment SGI239268[] = {{&S93874[68], 8}, {NULL}};
const Segment SGI239269[] = {{&S93877[34], 10}, {NULL}};
const Segment SGI239270[] = {{&S93877[27], 6}, {NULL}};
const Segment SGI239271[] = {{&S93877[97], 9}, {NULL}};
const Segment SGI239272[] = {{&S93877[70], 6}, {NULL}};
const Segment SGI239273[] = {{&S93878[34], 10}, {NULL}};
const Segment SGI239274[] = {{&S93878[27], 6}, {NULL}};
const Segment SGI239275[] = {{&S93878[97], 9}, {NULL}};
const Segment SGI239276[] = {{&S93878[70], 6}, {NULL}};
const Segment SGI239277[] = {{&S93881[44], 11}, {NULL}};
const Segment SGI239278[] = {{&S93881[44], 23}, {NULL}};
const Segment SGI239279[] = {{&S93881[30], 6}, {NULL}};
const Segment SGI239280[] = {{&S93881[116], 3}, {NULL}};
const Segment SGI239281[] = {{&S93881[81], 6}, {NULL}};
const Segment SGI239282[] = {{&S93890[59], 13}, {NULL}};
const Segment SGI239283[] = {{&S93890[59], 33}, {NULL}};
const Segment SGI239284[] = {{&S93890[45], 6}, {NULL}};
const Segment SGI239285[] = {{&S93890[160], 3}, {NULL}};
const Segment SGI239286[] = {{&S93890[125], 6}, {NULL}};
const Segment SGI239287[] = {{&S93890[0], 14}, {NULL}};
const Segment SGI239288[] = {{&S93899[44], 10}, {NULL}};
const Segment SGI239289[] = {{&S93899[44], 22}, {NULL}};
const Segment SGI239290[] = {{&S93899[30], 6}, {NULL}};
const Segment SGI239291[] = {{&S93899[115], 3}, {NULL}};
const Segment SGI239292[] = {{&S93899[80], 6}, {NULL}};
const Segment SGI239293[] = {{&S93899[135], 8}, {NULL}};
const Segment SGI239294[] = {{&S93899[144], 13}, {NULL}};
const Segment SGI239295[] = {{&S93903[34], 7}, {NULL}};
const Segment SGI239296[] = {{&S93903[27], 6}, {NULL}};
const Segment SGI239297[] = {{&S93903[100], 9}, {NULL}};
const Segment SGI239298[] = {{&S93903[73], 6}, {NULL}};
const Segment SGI239299[] = {{&S93903[141], 8}, {NULL}};
const Segment SGI239300[] = {{&S86153[44], 7}, {NULL}};
const Segment SGI239301[] = {{&S86153[44], 19}, {NULL}};
const Segment SGI239302[] = {{&S86153[30], 6}, {NULL}};
const Segment SGI239303[] = {{&S86153[130], 3}, {NULL}};
const Segment SGI239304[] = {{&S86153[95], 6}, {NULL}};
const Segment SGI239305[] = {{&S86153[65], 8}, {NULL}};
const Segment SGI239306[] = {{&S93915[44], 7}, {NULL}};
const Segment SGI239307[] = {{&S93915[44], 35}, {NULL}};
const Segment SGI239308[] = {{&S93915[30], 6}, {NULL}};
const Segment SGI239309[] = {{&S93915[128], 3}, {NULL}};
const Segment SGI239310[] = {{&S93915[93], 6}, {NULL}};
const Segment SGI239311[] = {{&S93915[153], 7}, {NULL}};
const Segment SGI239312[] = {{&S93918[59], 11}, {NULL}};
const Segment SGI239313[] = {{&S93918[59], 23}, {NULL}};
const Segment SGI239314[] = {{&S93918[45], 6}, {NULL}};
const Segment SGI239315[] = {{&S93918[131], 3}, {NULL}};
const Segment SGI239316[] = {{&S93918[96], 6}, {NULL}};
const Segment SGI239317[] = {{&S93918[0], 14}, {NULL}};
const Segment SGI239318[] = {{&S86170[44], 11}, {NULL}};
const Segment SGI239319[] = {{&S86170[44], 22}, {NULL}};
const Segment SGI239320[] = {{&S86170[30], 6}, {NULL}};
const Segment SGI239321[] = {{&S93920[34], 10}, {NULL}};
const Segment SGI239322[] = {{&S93920[27], 6}, {NULL}};
const Segment SGI239323[] = {{&S86170[115], 3}, {NULL}};
const Segment SGI239324[] = {{&S86170[80], 6}, {NULL}};
const Segment SGI239325[] = {{&S93920[97], 9}, {NULL}};
const Segment SGI239326[] = {{&S93920[70], 6}, {NULL}};
const Segment SGI239327[] = {{&S93921[44], 11}, {NULL}};
const Segment SGI239328[] = {{&S93921[44], 23}, {NULL}};
const Segment SGI239329[] = {{&S93921[30], 6}, {NULL}};
const Segment SGI239330[] = {{&S93921[116], 3}, {NULL}};
const Segment SGI239331[] = {{&S86172[44], 10}, {NULL}};
const Segment SGI239332[] = {{&S93921[81], 6}, {NULL}};
const Segment SGI239333[] = {{&S86172[44], 21}, {NULL}};
const Segment SGI239334[] = {{&S86172[30], 6}, {NULL}};
const Segment SGI239335[] = {{&S93921[141], 7}, {NULL}};
const Segment SGI239336[] = {{&S86172[112], 3}, {NULL}};
const Segment SGI239337[] = {{&S86172[79], 5}, {NULL}};
const Segment SGI239338[] = {{&S93927[44], 13}, {NULL}};
const Segment SGI239339[] = {{&S93927[44], 25}, {NULL}};
const Segment SGI239340[] = {{&S93927[30], 6}, {NULL}};
const Segment SGI239341[] = {{&S93927[118], 3}, {NULL}};
const Segment SGI239342[] = {{&S93927[83], 6}, {NULL}};
const Segment SGI239343[] = {{&S93927[143], 13}, {NULL}};
const Segment SGI239344[] = {{&S93928[44], 11}, {NULL}};
const Segment SGI239345[] = {{&S93928[44], 23}, {NULL}};
const Segment SGI239346[] = {{&S93928[30], 6}, {NULL}};
const Segment SGI239347[] = {{&S93928[116], 3}, {NULL}};
const Segment SGI239348[] = {{&S86180[34], 24}, {NULL}};
const Segment SGI239349[] = {{&S86180[27], 6}, {NULL}};
const Segment SGI239350[] = {{&S93928[81], 6}, {NULL}};
const Segment SGI239351[] = {{&S93928[141], 7}, {NULL}};
const Segment SGI239352[] = {{&S86180[111], 9}, {NULL}};
const Segment SGI239353[] = {{&S86180[84], 6}, {NULL}};
const Segment SGI239354[] = {{&S86181[34], 12}, {NULL}};
const Segment SGI239355[] = {{&S86181[27], 6}, {NULL}};
const Segment SGI239356[] = {{&S86181[99], 9}, {NULL}};
const Segment SGI239357[] = {{&S86181[72], 6}, {NULL}};
const Segment SGI239358[] = {{&S93934[34], 10}, {NULL}};
const Segment SGI239359[] = {{&S93934[27], 6}, {NULL}};
const Segment SGI239360[] = {{&S93934[97], 9}, {NULL}};
const Segment SGI239361[] = {{&S93934[70], 6}, {NULL}};
const Segment SGI239362[] = {{&S86189[44], 7}, {NULL}};
const Segment SGI239363[] = {{&S86189[44], 19}, {NULL}};
const Segment SGI239364[] = {{&S86189[30], 6}, {NULL}};
const Segment SGI239365[] = {{&S86189[112], 3}, {NULL}};
const Segment SGI239366[] = {{&S86189[77], 6}, {NULL}};
const Segment SGI239367[] = {{&S86189[137], 8}, {NULL}};
const Segment SGI239368[] = {{&S86192[44], 10}, {NULL}};
const Segment SGI239369[] = {{&S86192[44], 21}, {NULL}};
const Segment SGI239370[] = {{&S86192[30], 6}, {NULL}};
const Segment SGI239371[] = {{&S86192[79], 6}, {NULL}};
const Segment SGI239372[] = {{&S86192[106], 13}, {NULL}};
const Segment SGI239373[] = {{&S93944[34], 9}, {NULL}};
const Segment SGI239374[] = {{&S93944[27], 6}, {NULL}};
const Segment SGI239375[] = {{&S93944[96], 9}, {NULL}};
const Segment SGI239376[] = {{&S93944[69], 6}, {NULL}};
const Segment SGI239377[] = {{&S86196[34], 18}, {NULL}};
const Segment SGI239378[] = {{&S86196[27], 6}, {NULL}};
const Segment SGI239379[] = {{&S86196[105], 9}, {NULL}};
const Segment SGI239380[] = {{&S86196[78], 6}, {NULL}};
const Segment SGI239381[] = {{&S93950[44], 10}, {NULL}};
const Segment SGI239382[] = {{&S93950[44], 22}, {NULL}};
const Segment SGI239383[] = {{&S86199[34], 9}, {NULL}};
const Segment SGI239384[] = {{&S93950[30], 6}, {NULL}};
const Segment SGI239385[] = {{&S86199[27], 6}, {NULL}};
const Segment SGI239386[] = {{&S93950[115], 3}, {NULL}};
const Segment SGI239387[] = {{&S86199[96], 9}, {NULL}};
const Segment SGI239388[] = {{&S93950[80], 6}, {NULL}};
const Segment SGI239389[] = {{&S86199[69], 6}, {NULL}};
const Segment SGI239390[] = {{&S93950[135], 8}, {NULL}};
const Segment SGI239391[] = {{&S93950[144], 13}, {NULL}};
const Segment SGI239392[] = {{&S93952[34], 10}, {NULL}};
const Segment SGI239393[] = {{&S93952[27], 6}, {NULL}};
const Segment SGI239394[] = {{&S93952[97], 9}, {NULL}};
const Segment SGI239395[] = {{&S93952[70], 6}, {NULL}};
const Segment SGI239396[] = {{&S93954[34], 10}, {NULL}};
const Segment SGI239397[] = {{&S93954[27], 6}, {NULL}};
const Segment SGI239398[] = {{&S93954[97], 9}, {NULL}};
const Segment SGI239399[] = {{&S93954[70], 6}, {NULL}};
const Segment SGI239400[] = {{&S93954[138], 8}, {NULL}};
const Segment SGI239401[] = {{&S100792[44], 10}, {NULL}};
const Segment SGI239402[] = {{&S100792[44], 22}, {NULL}};
const Segment SGI239403[] = {{&S100792[30], 6}, {NULL}};
const Segment SGI239404[] = {{&S100792[144], 3}, {NULL}};
const Segment SGI239405[] = {{&S100792[109], 6}, {NULL}};
const Segment SGI239406[] = {{&S100792[68], 8}, {NULL}};
const Segment SGI239407[] = {{&S100792[80], 15}, {NULL}};
const Segment SGI239408[] = {{&S104342[44], 12}, {NULL}};
const Segment SGI239409[] = {{&S104342[44], 24}, {NULL}};
const Segment SGI239410[] = {{&S104342[30], 6}, {NULL}};
const Segment SGI239411[] = {{&S104342[31], 3}, {NULL}};
const Segment SGI239412[] = {{&S104342[101], 6}, {NULL}};
const Segment SGI239413[] = {{&S104342[70], 8}, {NULL}};
const Segment SGI239414[] = {{&S104343[44], 12}, {NULL}};
const Segment SGI239415[] = {{&S104343[44], 24}, {NULL}};
const Segment SGI239416[] = {{&S104343[30], 6}, {NULL}};
const Segment SGI239417[] = {{&S104343[31], 3}, {NULL}};
const Segment SGI239418[] = {{&S104343[101], 6}, {NULL}};
const Segment SGI239419[] = {{&S104343[70], 8}, {NULL}};
const Segment SGI239420[] = {{&S104345[44], 12}, {NULL}};
const Segment SGI239421[] = {{&S104345[44], 24}, {NULL}};
const Segment SGI239422[] = {{&S104345[30], 6}, {NULL}};
const Segment SGI239423[] = {{&S104345[31], 3}, {NULL}};
const Segment SGI239424[] = {{&S104345[82], 6}, {NULL}};
const Segment SGI239425[] = {{&S100806[44], 10}, {NULL}};
const Segment SGI239426[] = {{&S100806[44], 22}, {NULL}};
const Segment SGI239427[] = {{&S100806[30], 6}, {NULL}};
const Segment SGI239428[] = {{&S100806[144], 3}, {NULL}};
const Segment SGI239429[] = {{&S100806[109], 6}, {NULL}};
const Segment SGI239430[] = {{&S100806[68], 8}, {NULL}};
const Segment SGI239431[] = {{&S100806[80], 15}, {NULL}};
const Segment SGI239432[] = {{&S86268[34], 9}, {NULL}};
const Segment SGI239433[] = {{&S94793[34], 8}, {NULL}};
const Segment SGI239434[] = {{&S86268[27], 6}, {NULL}};
const Segment SGI239435[] = {{&S94793[27], 6}, {NULL}};
const Segment SGI239436[] = {{&S86268[96], 9}, {NULL}};
const Segment SGI239437[] = {{&S94793[95], 9}, {NULL}};
const Segment SGI239438[] = {{&S86268[69], 6}, {NULL}};
const Segment SGI239439[] = {{&S94793[68], 6}, {NULL}};
const Segment SGI239440[] = {{&S86306[34], 10}, {NULL}};
const Segment SGI239441[] = {{&S86306[27], 6}, {NULL}};
const Segment SGI239442[] = {{&S86306[97], 9}, {NULL}};
const Segment SGI239443[] = {{&S86306[70], 6}, {NULL}};
const Segment SGI239444[] = {{&S100859[57], 10}, {NULL}};
const Segment SGI239445[] = {{&S100859[45], 6}, {NULL}};
const Segment SGI239446[] = {{&S100859[128], 3}, {NULL}};
const Segment SGI239447[] = {{&S100859[93], 6}, {NULL}};
const Segment SGI239448[] = {{&S100859[0], 14}, {NULL}};
const Segment SGI239449[] = {{&S94075[34], 10}, {NULL}};
const Segment SGI239450[] = {{&S94075[27], 6}, {NULL}};
const Segment SGI239451[] = {{&S94075[97], 9}, {NULL}};
const Segment SGI239452[] = {{&S94075[70], 6}, {NULL}};
const Segment SGI239453[] = {{&S94850[44], 25}, {NULL}};
const Segment SGI239454[] = {{&S100860[59], 10}, {NULL}};
const Segment SGI239455[] = {{&S94850[44], 37}, {NULL}};
const Segment SGI239456[] = {{&S94850[30], 6}, {NULL}};
const Segment SGI239457[] = {{&S100860[59], 22}, {NULL}};
const Segment SGI239458[] = {{&S100860[45], 6}, {NULL}};
const Segment SGI239459[] = {{&S94850[130], 3}, {NULL}};
const Segment SGI239460[] = {{&S100860[130], 3}, {NULL}};
const Segment SGI239461[] = {{&S94850[95], 6}, {NULL}};
const Segment SGI239462[] = {{&S100860[95], 6}, {NULL}};
const Segment SGI239463[] = {{&S100860[0], 14}, {NULL}};
const Segment SGI239464[] = {{&S94076[34], 10}, {NULL}};
const Segment SGI239465[] = {{&S94076[27], 6}, {NULL}};
const Segment SGI239466[] = {{&S94076[97], 9}, {NULL}};
const Segment SGI239467[] = {{&S94076[70], 6}, {NULL}};
const Segment SGI239468[] = {{&S94080[44], 10}, {NULL}};
const Segment SGI239469[] = {{&S94080[44], 22}, {NULL}};
const Segment SGI239470[] = {{&S94080[30], 6}, {NULL}};
const Segment SGI239471[] = {{&S94080[80], 6}, {NULL}};
const Segment SGI239472[] = {{&S94080[107], 13}, {NULL}};
const Segment SGI239473[] = {{&S104410[34], 10}, {NULL}};
const Segment SGI239474[] = {{&S104410[27], 6}, {NULL}};
const Segment SGI239475[] = {{&S104410[97], 9}, {NULL}};
const Segment SGI239476[] = {{&S104410[70], 6}, {NULL}};
const Segment SGI239477[] = {{&S94085[34], 10}, {NULL}};
const Segment SGI239478[] = {{&S94085[27], 6}, {NULL}};
const Segment SGI239479[] = {{&S94085[97], 9}, {NULL}};
const Segment SGI239480[] = {{&S94085[70], 6}, {NULL}};
const Segment SGI239481[] = {{&S94862[34], 13}, {NULL}};
const Segment SGI239482[] = {{&S94862[27], 6}, {NULL}};
const Segment SGI239483[] = {{&S94862[100], 9}, {NULL}};
const Segment SGI239484[] = {{&S94862[73], 6}, {NULL}};
const Segment SGI239485[] = {{&S104426[59], 10}, {NULL}};
const Segment SGI239486[] = {{&S104426[59], 22}, {NULL}};
const Segment SGI239487[] = {{&S104426[45], 6}, {NULL}};
const Segment SGI239488[] = {{&S104426[46], 3}, {NULL}};
const Segment SGI239489[] = {{&S104426[95], 6}, {NULL}};
const Segment SGI239490[] = {{&S104426[0], 14}, {NULL}};
const Segment SGI239491[] = {{&S94101[34], 7}, {NULL}};
const Segment SGI239492[] = {{&S94101[27], 6}, {NULL}};
const Segment SGI239493[] = {{&S94101[99], 9}, {NULL}};
const Segment SGI239494[] = {{&S94101[72], 6}, {NULL}};
const Segment SGI239495[] = {{&S104428[59], 10}, {NULL}};
const Segment SGI239496[] = {{&S104428[59], 22}, {NULL}};
const Segment SGI239497[] = {{&S104428[45], 6}, {NULL}};
const Segment SGI239498[] = {{&S104428[46], 3}, {NULL}};
const Segment SGI239499[] = {{&S104428[95], 6}, {NULL}};
const Segment SGI239500[] = {{&S104428[0], 14}, {NULL}};
const Segment SGI239501[] = {{&S104430[58], 10}, {NULL}};
const Segment SGI239502[] = {{&S104430[58], 22}, {NULL}};
const Segment SGI239503[] = {{&S104430[44], 6}, {NULL}};
const Segment SGI239504[] = {{&S104430[45], 3}, {NULL}};
const Segment SGI239505[] = {{&S104430[94], 6}, {NULL}};
const Segment SGI239506[] = {{&S104432[45], 10}, {NULL}};
const Segment SGI239507[] = {{&S104432[30], 6}, {NULL}};
const Segment SGI239508[] = {{&S104432[31], 3}, {NULL}};
const Segment SGI239509[] = {{&S104432[81], 5}, {NULL}};
const Segment SGI239510[] = {{&S104432[139], 13}, {NULL}};
const Segment SGI239511[] = {{&S94109[34], 9}, {NULL}};
const Segment SGI239512[] = {{&S94109[27], 6}, {NULL}};
const Segment SGI239513[] = {{&S94109[96], 9}, {NULL}};
const Segment SGI239514[] = {{&S94109[69], 6}, {NULL}};
const Segment SGI239515[] = {{&S104435[44], 10}, {NULL}};
const Segment SGI239516[] = {{&S104435[44], 22}, {NULL}};
const Segment SGI239517[] = {{&S104435[30], 6}, {NULL}};
const Segment SGI239518[] = {{&S104435[31], 3}, {NULL}};
const Segment SGI239519[] = {{&S104435[80], 5}, {NULL}};
const Segment SGI239520[] = {{&S104435[125], 14}, {NULL}};
const Segment SGI239521[] = {{&S100908[45], 11}, {NULL}};
const Segment SGI239522[] = {{&S100908[30], 6}, {NULL}};
const Segment SGI239523[] = {{&S100908[116], 3}, {NULL}};
const Segment SGI239524[] = {{&S100908[82], 5}, {NULL}};
const Segment SGI239525[] = {{&S104441[44], 10}, {NULL}};
const Segment SGI239526[] = {{&S100908[140], 13}, {NULL}};
const Segment SGI239527[] = {{&S104441[44], 27}, {NULL}};
const Segment SGI239528[] = {{&S104441[30], 6}, {NULL}};
const Segment SGI239529[] = {{&S94119[34], 18}, {NULL}};
const Segment SGI239530[] = {{&S104441[31], 3}, {NULL}};
const Segment SGI239531[] = {{&S94119[27], 6}, {NULL}};
const Segment SGI239532[] = {{&S104441[85], 6}, {NULL}};
const Segment SGI239533[] = {{&S104441[148], 3}, {NULL}};
const Segment SGI239534[] = {{&S94119[105], 9}, {NULL}};
const Segment SGI239535[] = {{&S94119[78], 6}, {NULL}};
const Segment SGI239536[] = {{&S94119[146], 8}, {NULL}};
const Segment SGI239537[] = {{&S94121[34], 11}, {NULL}};
const Segment SGI239538[] = {{&S94121[27], 6}, {NULL}};
const Segment SGI239539[] = {{&S94121[98], 9}, {NULL}};
const Segment SGI239540[] = {{&S94121[71], 6}, {NULL}};
const Segment SGI239541[] = {{&S94123[34], 7}, {NULL}};
const Segment SGI239542[] = {{&S94123[27], 6}, {NULL}};
const Segment SGI239543[] = {{&S94123[99], 9}, {NULL}};
const Segment SGI239544[] = {{&S94123[72], 6}, {NULL}};
const Segment SGI239545[] = {{&S94123[140], 8}, {NULL}};
const Segment SGI239546[] = {{&S104447[59], 10}, {NULL}};
const Segment SGI239547[] = {{&S104447[59], 22}, {NULL}};
const Segment SGI239548[] = {{&S104447[45], 6}, {NULL}};
const Segment SGI239549[] = {{&S104447[46], 3}, {NULL}};
const Segment SGI239550[] = {{&S104447[95], 6}, {NULL}};
const Segment SGI239551[] = {{&S104447[0], 14}, {NULL}};
const Segment SGI239552[] = {{&S94125[34], 7}, {NULL}};
const Segment SGI239553[] = {{&S94125[27], 6}, {NULL}};
const Segment SGI239554[] = {{&S94125[94], 9}, {NULL}};
const Segment SGI239555[] = {{&S94125[67], 6}, {NULL}};
const Segment SGI239556[] = {{&S104449[44], 10}, {NULL}};
const Segment SGI239557[] = {{&S104449[44], 22}, {NULL}};
const Segment SGI239558[] = {{&S104449[30], 6}, {NULL}};
const Segment SGI239559[] = {{&S104449[158], 3}, {NULL}};
const Segment SGI239560[] = {{&S94127[34], 10}, {NULL}};
const Segment SGI239561[] = {{&S104449[124], 3}, {NULL}};
const Segment SGI239562[] = {{&S94127[27], 6}, {NULL}};
const Segment SGI239563[] = {{&S104449[89], 6}, {NULL}};
const Segment SGI239564[] = {{&S104449[144], 8}, {NULL}};
const Segment SGI239565[] = {{&S94127[97], 9}, {NULL}};
const Segment SGI239566[] = {{&S94127[70], 6}, {NULL}};
const Segment SGI239567[] = {{&S104451[59], 10}, {NULL}};
const Segment SGI239568[] = {{&S104451[59], 22}, {NULL}};
const Segment SGI239569[] = {{&S104451[45], 6}, {NULL}};
const Segment SGI239570[] = {{&S104451[95], 5}, {NULL}};
const Segment SGI239571[] = {{&S100924[44], 11}, {NULL}};
const Segment SGI239572[] = {{&S100924[44], 23}, {NULL}};
const Segment SGI239573[] = {{&S104451[0], 14}, {NULL}};
const Segment SGI239574[] = {{&S100924[30], 6}, {NULL}};
const Segment SGI239575[] = {{&S100924[116], 3}, {NULL}};
const Segment SGI239576[] = {{&S100924[81], 6}, {NULL}};
const Segment SGI239577[] = {{&S100924[151], 3}, {NULL}};
const Segment SGI239578[] = {{&S104452[44], 10}, {NULL}};
const Segment SGI239579[] = {{&S104452[44], 22}, {NULL}};
const Segment SGI239580[] = {{&S104452[30], 6}, {NULL}};
const Segment SGI239581[] = {{&S104452[31], 3}, {NULL}};
const Segment SGI239582[] = {{&S104452[80], 5}, {NULL}};
const Segment SGI239583[] = {{&S104452[125], 14}, {NULL}};
const Segment SGI239584[] = {{&S86394[44], 9}, {NULL}};
const Segment SGI239585[] = {{&S86394[44], 20}, {NULL}};
const Segment SGI239586[] = {{&S86394[30], 6}, {NULL}};
const Segment SGI239587[] = {{&S86394[78], 6}, {NULL}};
const Segment SGI239588[] = {{&S86394[105], 13}, {NULL}};
const Segment SGI239589[] = {{&S100929[44], 11}, {NULL}};
const Segment SGI239590[] = {{&S100929[44], 23}, {NULL}};
const Segment SGI239591[] = {{&S100929[30], 6}, {NULL}};
const Segment SGI239592[] = {{&S100929[114], 3}, {NULL}};
const Segment SGI239593[] = {{&S104456[62], 10}, {NULL}};
const Segment SGI239594[] = {{&S100929[81], 5}, {NULL}};
const Segment SGI239595[] = {{&S104456[55], 6}, {NULL}};
const Segment SGI239596[] = {{&S100929[118], 14}, {NULL}};
const Segment SGI239597[] = {{&S94138[44], 10}, {NULL}};
const Segment SGI239598[] = {{&S94138[44], 22}, {NULL}};
const Segment SGI239599[] = {{&S94138[30], 6}, {NULL}};
const Segment SGI239600[] = {{&S94138[113], 3}, {NULL}};
const Segment SGI239601[] = {{&S94138[80], 5}, {NULL}};
const Segment SGI239602[] = {{&S94138[117], 14}, {NULL}};
const Segment SGI239603[] = {{&S104458[57], 10}, {NULL}};
const Segment SGI239604[] = {{&S104458[45], 6}, {NULL}};
const Segment SGI239605[] = {{&S104458[46], 3}, {NULL}};
const Segment SGI239606[] = {{&S104458[93], 6}, {NULL}};
const Segment SGI239607[] = {{&S104458[0], 14}, {NULL}};
const Segment SGI239608[] = {{&S94141[44], 17}, {NULL}};
const Segment SGI239609[] = {{&S94141[44], 29}, {NULL}};
const Segment SGI239610[] = {{&S94141[30], 6}, {NULL}};
const Segment SGI239611[] = {{&S104459[59], 10}, {NULL}};
const Segment SGI239612[] = {{&S104459[59], 22}, {NULL}};
const Segment SGI239613[] = {{&S104459[45], 6}, {NULL}};
const Segment SGI239614[] = {{&S94141[122], 3}, {NULL}};
const Segment SGI239615[] = {{&S94141[87], 6}, {NULL}};
const Segment SGI239616[] = {{&S104459[46], 3}, {NULL}};
const Segment SGI239617[] = {{&S104459[95], 6}, {NULL}};
const Segment SGI239618[] = {{&S104459[0], 14}, {NULL}};
const Segment SGI239619[] = {{&S94142[34], 11}, {NULL}};
const Segment SGI239620[] = {{&S94142[27], 6}, {NULL}};
const Segment SGI239621[] = {{&S94142[98], 9}, {NULL}};
const Segment SGI239622[] = {{&S94142[71], 6}, {NULL}};
const Segment SGI239623[] = {{&S94144[52], 14}, {NULL}};
const Segment SGI239624[] = {{&S94144[52], 36}, {NULL}};
const Segment SGI239625[] = {{&S94144[38], 6}, {NULL}};
const Segment SGI239626[] = {{&S94144[102], 5}, {NULL}};
const Segment SGI239627[] = {{&S94144[11], 3}, {NULL}};
const Segment SGI239628[] = {{&S94147[44], 10}, {NULL}};
const Segment SGI239629[] = {{&S94147[44], 22}, {NULL}};
const Segment SGI239630[] = {{&S94147[30], 6}, {NULL}};
const Segment SGI239631[] = {{&S94147[115], 3}, {NULL}};
const Segment SGI239632[] = {{&S94147[80], 6}, {NULL}};
const Segment SGI239633[] = {{&S94147[140], 13}, {NULL}};
const Segment SGI239634[] = {{&S94151[44], 13}, {NULL}};
const Segment SGI239635[] = {{&S94151[44], 25}, {NULL}};
const Segment SGI239636[] = {{&S94151[30], 6}, {NULL}};
const Segment SGI239637[] = {{&S94151[83], 6}, {NULL}};
const Segment SGI239638[] = {{&S94151[110], 13}, {NULL}};
const Segment SGI239639[] = {{&S104471[45], 10}, {NULL}};
const Segment SGI239640[] = {{&S104471[30], 6}, {NULL}};
const Segment SGI239641[] = {{&S104471[31], 3}, {NULL}};
const Segment SGI239642[] = {{&S104471[81], 5}, {NULL}};
const Segment SGI239643[] = {{&S104471[139], 13}, {NULL}};
const Segment SGI239644[] = {{&S104473[44], 10}, {NULL}};
const Segment SGI239645[] = {{&S104473[44], 22}, {NULL}};
const Segment SGI239646[] = {{&S104473[30], 6}, {NULL}};
const Segment SGI239647[] = {{&S104473[31], 3}, {NULL}};
const Segment SGI239648[] = {{&S104473[80], 6}, {NULL}};
const Segment SGI239649[] = {{&S104473[133], 11}, {NULL}};
const Segment SGI239650[] = {{&S104480[44], 10}, {NULL}};
const Segment SGI239651[] = {{&S104480[44], 22}, {NULL}};
const Segment SGI239652[] = {{&S104480[30], 6}, {NULL}};
const Segment SGI239653[] = {{&S104480[80], 6}, {NULL}};
const Segment SGI239654[] = {{&S104480[107], 13}, {NULL}};
const Segment SGI239655[] = {{&S94165[34], 10}, {NULL}};
const Segment SGI239656[] = {{&S94165[27], 6}, {NULL}};
const Segment SGI239657[] = {{&S94165[97], 9}, {NULL}};
const Segment SGI239658[] = {{&S94165[70], 6}, {NULL}};
const Segment SGI239659[] = {{&S94165[138], 8}, {NULL}};
const Segment SGI239660[] = {{&S94167[34], 10}, {NULL}};
const Segment SGI239661[] = {{&S94167[27], 6}, {NULL}};
const Segment SGI239662[] = {{&S104483[44], 10}, {NULL}};
const Segment SGI239663[] = {{&S104483[44], 22}, {NULL}};
const Segment SGI239664[] = {{&S104483[30], 6}, {NULL}};
const Segment SGI239665[] = {{&S94167[97], 9}, {NULL}};
const Segment SGI239666[] = {{&S94167[70], 6}, {NULL}};
const Segment SGI239667[] = {{&S104483[31], 3}, {NULL}};
const Segment SGI239668[] = {{&S104483[80], 6}, {NULL}};
const Segment SGI239669[] = {{&S104483[132], 10}, {NULL}};
const Segment SGI239670[] = {{&S94168[44], 10}, {NULL}};
const Segment SGI239671[] = {{&S94168[44], 22}, {NULL}};
const Segment SGI239672[] = {{&S94168[30], 6}, {NULL}};
const Segment SGI239673[] = {{&S94168[115], 3}, {NULL}};
const Segment SGI239674[] = {{&S94168[80], 6}, {NULL}};
const Segment SGI239675[] = {{&S94168[135], 8}, {NULL}};
const Segment SGI239676[] = {{&S94168[144], 13}, {NULL}};
const Segment SGI239677[] = {{&S104490[44], 10}, {NULL}};
const Segment SGI239678[] = {{&S104490[44], 22}, {NULL}};
const Segment SGI239679[] = {{&S94174[44], 12}, {NULL}};
const Segment SGI239680[] = {{&S104490[30], 6}, {NULL}};
const Segment SGI239681[] = {{&S94174[44], 34}, {NULL}};
const Segment SGI239682[] = {{&S94174[30], 6}, {NULL}};
const Segment SGI239683[] = {{&S104490[31], 3}, {NULL}};
const Segment SGI239684[] = {{&S104490[80], 6}, {NULL}};
const Segment SGI239685[] = {{&S94174[92], 6}, {NULL}};
const Segment SGI239686[] = {{&S104490[132], 10}, {NULL}};
const Segment SGI239687[] = {{&S94174[119], 13}, {NULL}};
const Segment SGI239688[] = {{&S104491[44], 10}, {NULL}};
const Segment SGI239689[] = {{&S104491[44], 27}, {NULL}};
const Segment SGI239690[] = {{&S104491[30], 6}, {NULL}};
const Segment SGI239691[] = {{&S104491[31], 3}, {NULL}};
const Segment SGI239692[] = {{&S104491[85], 6}, {NULL}};
const Segment SGI239693[] = {{&S104491[138], 13}, {NULL}};
const Segment SGI239694[] = {{&S104492[44], 10}, {NULL}};
const Segment SGI239695[] = {{&S104492[44], 22}, {NULL}};
const Segment SGI239696[] = {{&S104492[30], 6}, {NULL}};
const Segment SGI239697[] = {{&S104492[31], 3}, {NULL}};
const Segment SGI239698[] = {{&S104492[80], 6}, {NULL}};
const Segment SGI239699[] = {{&S104492[140], 13}, {NULL}};
const Segment SGI239700[] = {{&S94180[34], 14}, {NULL}};
const Segment SGI239701[] = {{&S94180[27], 6}, {NULL}};
const Segment SGI239702[] = {{&S94180[101], 9}, {NULL}};
const Segment SGI239703[] = {{&S94180[74], 6}, {NULL}};
const Segment SGI239704[] = {{&S104498[44], 10}, {NULL}};
const Segment SGI239705[] = {{&S104498[44], 22}, {NULL}};
const Segment SGI239706[] = {{&S104498[30], 6}, {NULL}};
const Segment SGI239707[] = {{&S104498[31], 3}, {NULL}};
const Segment SGI239708[] = {{&S104498[95], 6}, {NULL}};
const Segment SGI239709[] = {{&S104498[148], 11}, {NULL}};
const Segment SGI239710[] = {{&S104498[68], 8}, {NULL}};
const Segment SGI239711[] = {{&S104500[44], 10}, {NULL}};
const Segment SGI239712[] = {{&S104500[44], 22}, {NULL}};
const Segment SGI239713[] = {{&S104500[30], 6}, {NULL}};
const Segment SGI239714[] = {{&S104500[31], 3}, {NULL}};
const Segment SGI239715[] = {{&S104500[80], 6}, {NULL}};
const Segment SGI239716[] = {{&S104500[143], 3}, {NULL}};
const Segment SGI239717[] = {{&S86478[44], 17}, {NULL}};
const Segment SGI239718[] = {{&S86478[44], 28}, {NULL}};
const Segment SGI239719[] = {{&S86478[30], 6}, {NULL}};
const Segment SGI239720[] = {{&S86478[139], 3}, {NULL}};
const Segment SGI239721[] = {{&S86478[104], 6}, {NULL}};
const Segment SGI239722[] = {{&S86478[74], 8}, {NULL}};
const Segment SGI239723[] = {{&S86479[44], 17}, {NULL}};
const Segment SGI239724[] = {{&S86479[44], 28}, {NULL}};
const Segment SGI239725[] = {{&S104513[44], 10}, {NULL}};
const Segment SGI239726[] = {{&S86479[30], 6}, {NULL}};
const Segment SGI239727[] = {{&S104513[44], 22}, {NULL}};
const Segment SGI239728[] = {{&S104513[30], 6}, {NULL}};
const Segment SGI239729[] = {{&S86479[141], 3}, {NULL}};
const Segment SGI239730[] = {{&S86479[106], 6}, {NULL}};
const Segment SGI239731[] = {{&S104513[31], 3}, {NULL}};
const Segment SGI239732[] = {{&S86479[74], 8}, {NULL}};
const Segment SGI239733[] = {{&S104513[80], 6}, {NULL}};
const Segment SGI239734[] = {{&S104513[133], 11}, {NULL}};
const Segment SGI239735[] = {{&S95012[34], 8}, {NULL}};
const Segment SGI239736[] = {{&S95012[27], 6}, {NULL}};
const Segment SGI239737[] = {{&S95012[95], 9}, {NULL}};
const Segment SGI239738[] = {{&S104515[44], 10}, {NULL}};
const Segment SGI239739[] = {{&S95012[68], 6}, {NULL}};
const Segment SGI239740[] = {{&S104515[44], 22}, {NULL}};
const Segment SGI239741[] = {{&S104515[30], 6}, {NULL}};
const Segment SGI239742[] = {{&S104515[31], 3}, {NULL}};
const Segment SGI239743[] = {{&S104515[80], 6}, {NULL}};
const Segment SGI239744[] = {{&S104515[157], 3}, {NULL}};
const Segment SGI239745[] = {{&S104517[44], 10}, {NULL}};
const Segment SGI239746[] = {{&S104517[44], 22}, {NULL}};
const Segment SGI239747[] = {{&S104517[30], 6}, {NULL}};
const Segment SGI239748[] = {{&S104517[31], 3}, {NULL}};
const Segment SGI239749[] = {{&S104517[80], 6}, {NULL}};
const Segment SGI239750[] = {{&S104517[133], 13}, {NULL}};
const Segment SGI239751[] = {{&S86505[34], 10}, {NULL}};
const Segment SGI239752[] = {{&S86505[27], 6}, {NULL}};
const Segment SGI239753[] = {{&S86505[97], 9}, {NULL}};
const Segment SGI239754[] = {{&S86505[70], 6}, {NULL}};
const Segment SGI239755[] = {{&S101053[44], 5}, {NULL}};
const Segment SGI239756[] = {{&S101053[44], 17}, {NULL}};
const Segment SGI239757[] = {{&S101053[30], 6}, {NULL}};
const Segment SGI239758[] = {{&S101053[139], 3}, {NULL}};
const Segment SGI239759[] = {{&S101053[104], 6}, {NULL}};
const Segment SGI239760[] = {{&S101053[63], 8}, {NULL}};
const Segment SGI239761[] = {{&S101053[75], 15}, {NULL}};
const Segment SGI239762[] = {{&S86521[34], 12}, {NULL}};
const Segment SGI239763[] = {{&S86521[27], 6}, {NULL}};
const Segment SGI239764[] = {{&S86521[98], 9}, {NULL}};
const Segment SGI239765[] = {{&S86521[71], 6}, {NULL}};
const Segment SGI239766[] = {{&S86522[34], 7}, {NULL}};
const Segment SGI239767[] = {{&S86522[27], 6}, {NULL}};
const Segment SGI239768[] = {{&S86522[93], 9}, {NULL}};
const Segment SGI239769[] = {{&S86522[66], 6}, {NULL}};
const Segment SGI239770[] = {{&S104558[34], 13}, {NULL}};
const Segment SGI239771[] = {{&S104558[27], 6}, {NULL}};
const Segment SGI239772[] = {{&S104558[100], 9}, {NULL}};
const Segment SGI239773[] = {{&S104558[73], 6}, {NULL}};
const Segment SGI239774[] = {{&S104558[141], 8}, {NULL}};
const Segment SGI239775[] = {{&S104562[34], 10}, {NULL}};
const Segment SGI239776[] = {{&S104562[27], 6}, {NULL}};
const Segment SGI239777[] = {{&S104562[97], 9}, {NULL}};
const Segment SGI239778[] = {{&S104562[70], 6}, {NULL}};
const Segment SGI239779[] = {{&S104562[138], 8}, {NULL}};
const Segment SGI239780[] = {{&S101076[45], 10}, {NULL}};
const Segment SGI239781[] = {{&S101076[30], 6}, {NULL}};
const Segment SGI239782[] = {{&S101076[115], 3}, {NULL}};
const Segment SGI239783[] = {{&S101076[81], 5}, {NULL}};
const Segment SGI239784[] = {{&S101076[139], 13}, {NULL}};
const Segment SGI239785[] = {{&S95076[44], 10}, {NULL}};
const Segment SGI239786[] = {{&S95076[44], 24}, {NULL}};
const Segment SGI239787[] = {{&S95076[30], 6}, {NULL}};
const Segment SGI239788[] = {{&S95076[82], 6}, {NULL}};
const Segment SGI239789[] = {{&S95076[119], 3}, {NULL}};
const Segment SGI239790[] = {{&S94288[34], 8}, {NULL}};
const Segment SGI239791[] = {{&S94288[27], 6}, {NULL}};
const Segment SGI239792[] = {{&S95082[34], 11}, {NULL}};
const Segment SGI239793[] = {{&S94288[95], 9}, {NULL}};
const Segment SGI239794[] = {{&S94288[68], 6}, {NULL}};
const Segment SGI239795[] = {{&S95082[27], 6}, {NULL}};
const Segment SGI239796[] = {{&S95082[98], 9}, {NULL}};
const Segment SGI239797[] = {{&S95082[71], 6}, {NULL}};
const Segment SGI239798[] = {{&S94294[34], 8}, {NULL}};
const Segment SGI239799[] = {{&S94294[27], 6}, {NULL}};
const Segment SGI239800[] = {{&S94294[95], 9}, {NULL}};
const Segment SGI239801[] = {{&S94294[68], 6}, {NULL}};
const Segment SGI239802[] = {{&S95092[34], 15}, {NULL}};
const Segment SGI239803[] = {{&S95092[27], 6}, {NULL}};
const Segment SGI239804[] = {{&S95092[102], 9}, {NULL}};
const Segment SGI239805[] = {{&S95092[75], 6}, {NULL}};
const Segment SGI239806[] = {{&S94298[61], 10}, {NULL}};
const Segment SGI239807[] = {{&S94298[61], 22}, {NULL}};
const Segment SGI239808[] = {{&S94298[47], 6}, {NULL}};
const Segment SGI239809[] = {{&S94298[132], 3}, {NULL}};
const Segment SGI239810[] = {{&S94298[97], 6}, {NULL}};
const Segment SGI239811[] = {{&S94298[11], 3}, {NULL}};
const Segment SGI239812[] = {{&S94304[44], 10}, {NULL}};
const Segment SGI239813[] = {{&S94304[44], 22}, {NULL}};
const Segment SGI239814[] = {{&S94304[30], 6}, {NULL}};
const Segment SGI239815[] = {{&S95101[44], 5}, {NULL}};
const Segment SGI239816[] = {{&S95101[44], 17}, {NULL}};
const Segment SGI239817[] = {{&S95101[30], 6}, {NULL}};
const Segment SGI239818[] = {{&S94304[115], 3}, {NULL}};
const Segment SGI239819[] = {{&S94304[80], 6}, {NULL}};
const Segment SGI239820[] = {{&S94304[135], 8}, {NULL}};
const Segment SGI239821[] = {{&S95101[110], 3}, {NULL}};
const Segment SGI239822[] = {{&S94304[144], 13}, {NULL}};
const Segment SGI239823[] = {{&S95101[75], 6}, {NULL}};
const Segment SGI239824[] = {{&S94325[34], 10}, {NULL}};
const Segment SGI239825[] = {{&S94325[27], 6}, {NULL}};
const Segment SGI239826[] = {{&S94325[97], 9}, {NULL}};
const Segment SGI239827[] = {{&S94325[70], 6}, {NULL}};
const Segment SGI239828[] = {{&S94327[34], 10}, {NULL}};
const Segment SGI239829[] = {{&S94327[27], 6}, {NULL}};
const Segment SGI239830[] = {{&S94327[97], 9}, {NULL}};
const Segment SGI239831[] = {{&S94327[70], 6}, {NULL}};
const Segment SGI239832[] = {{&S94327[138], 8}, {NULL}};
const Segment SGI239833[] = {{&S94328[44], 10}, {NULL}};
const Segment SGI239834[] = {{&S94328[44], 22}, {NULL}};
const Segment SGI239835[] = {{&S94328[30], 6}, {NULL}};
const Segment SGI239836[] = {{&S94328[131], 3}, {NULL}};
const Segment SGI239837[] = {{&S94328[96], 6}, {NULL}};
const Segment SGI239838[] = {{&S94328[68], 8}, {NULL}};
const Segment SGI239839[] = {{&S94333[34], 7}, {NULL}};
const Segment SGI239840[] = {{&S94333[27], 6}, {NULL}};
const Segment SGI239841[] = {{&S94333[94], 9}, {NULL}};
const Segment SGI239842[] = {{&S94333[67], 6}, {NULL}};
const Segment SGI239843[] = {{&S94334[44], 15}, {NULL}};
const Segment SGI239844[] = {{&S94334[44], 27}, {NULL}};
const Segment SGI239845[] = {{&S94334[30], 6}, {NULL}};
const Segment SGI239846[] = {{&S94334[120], 3}, {NULL}};
const Segment SGI239847[] = {{&S94334[85], 6}, {NULL}};
const Segment SGI239848[] = {{&S94334[145], 7}, {NULL}};
const Segment SGI239849[] = {{&S94346[44], 10}, {NULL}};
const Segment SGI239850[] = {{&S94346[44], 22}, {NULL}};
const Segment SGI239851[] = {{&S94346[30], 6}, {NULL}};
const Segment SGI239852[] = {{&S94346[80], 6}, {NULL}};
const Segment SGI239853[] = {{&S94346[107], 13}, {NULL}};
const Segment SGI239854[] = {{&S86639[44], 10}, {NULL}};
const Segment SGI239855[] = {{&S86639[44], 22}, {NULL}};
const Segment SGI239856[] = {{&S86639[30], 6}, {NULL}};
const Segment SGI239857[] = {{&S86639[115], 3}, {NULL}};
const Segment SGI239858[] = {{&S86639[80], 6}, {NULL}};
const Segment SGI239859[] = {{&S86639[147], 15}, {NULL}};
const Segment SGI239860[] = {{&S86643[44], 18}, {NULL}};
const Segment SGI239861[] = {{&S86643[44], 30}, {NULL}};
const Segment SGI239862[] = {{&S86643[30], 6}, {NULL}};
const Segment SGI239863[] = {{&S94359[34], 9}, {NULL}};
const Segment SGI239864[] = {{&S86643[123], 3}, {NULL}};
const Segment SGI239865[] = {{&S94359[27], 6}, {NULL}};
const Segment SGI239866[] = {{&S86643[88], 6}, {NULL}};
const Segment SGI239867[] = {{&S94359[96], 9}, {NULL}};
const Segment SGI239868[] = {{&S94359[69], 6}, {NULL}};
const Segment SGI239869[] = {{&S94364[44], 18}, {NULL}};
const Segment SGI239870[] = {{&S94364[44], 30}, {NULL}};
const Segment SGI239871[] = {{&S94364[30], 6}, {NULL}};
const Segment SGI239872[] = {{&S94364[123], 3}, {NULL}};
const Segment SGI239873[] = {{&S94364[88], 6}, {NULL}};
const Segment SGI239874[] = {{&S94364[148], 7}, {NULL}};
const Segment SGI239875[] = {{&S94365[34], 11}, {NULL}};
const Segment SGI239876[] = {{&S94365[27], 6}, {NULL}};
const Segment SGI239877[] = {{&S94365[98], 9}, {NULL}};
const Segment SGI239878[] = {{&S94365[71], 6}, {NULL}};
const Segment SGI239879[] = {{&S94366[52], 6}, {NULL}};
const Segment SGI239880[] = {{&S94366[52], 18}, {NULL}};
const Segment SGI239881[] = {{&S94366[38], 6}, {NULL}};
const Segment SGI239882[] = {{&S94366[84], 5}, {NULL}};
const Segment SGI239883[] = {{&S94366[11], 3}, {NULL}};
const Segment SGI239884[] = {{&S94371[34], 10}, {NULL}};
const Segment SGI239885[] = {{&S94371[27], 6}, {NULL}};
const Segment SGI239886[] = {{&S94371[97], 9}, {NULL}};
const Segment SGI239887[] = {{&S94371[70], 6}, {NULL}};
const Segment SGI239888[] = {{&S94371[138], 8}, {NULL}};
const Segment SGI239889[] = {{&S86718[44], 10}, {NULL}};
const Segment SGI239890[] = {{&S86718[44], 21}, {NULL}};
const Segment SGI239891[] = {{&S86718[30], 6}, {NULL}};
const Segment SGI239892[] = {{&S86718[132], 3}, {NULL}};
const Segment SGI239893[] = {{&S86718[97], 6}, {NULL}};
const Segment SGI239894[] = {{&S86718[67], 8}, {NULL}};
const Segment SGI239895[] = {{&S86720[34], 10}, {NULL}};
const Segment SGI239896[] = {{&S86720[27], 6}, {NULL}};
const Segment SGI239897[] = {{&S86720[96], 9}, {NULL}};
const Segment SGI239898[] = {{&S86720[69], 6}, {NULL}};
const Segment SGI239899[] = {{&S86728[34], 11}, {NULL}};
const Segment SGI239900[] = {{&S86728[27], 6}, {NULL}};
const Segment SGI239901[] = {{&S86728[97], 9}, {NULL}};
const Segment SGI239902[] = {{&S86728[70], 6}, {NULL}};
const Segment SGI239903[] = {{&S86745[44], 14}, {NULL}};
const Segment SGI239904[] = {{&S86745[44], 25}, {NULL}};
const Segment SGI239905[] = {{&S86745[30], 6}, {NULL}};
const Segment SGI239906[] = {{&S86745[118], 3}, {NULL}};
const Segment SGI239907[] = {{&S86745[83], 6}, {NULL}};
const Segment SGI239908[] = {{&S104814[45], 7}, {NULL}};
const Segment SGI239909[] = {{&S104814[30], 6}, {NULL}};
const Segment SGI239910[] = {{&S104814[31], 3}, {NULL}};
const Segment SGI239911[] = {{&S104814[83], 5}, {NULL}};
const Segment SGI239912[] = {{&S104814[141], 13}, {NULL}};
const Segment SGI239913[] = {{&S86765[34], 8}, {NULL}};
const Segment SGI239914[] = {{&S86765[27], 6}, {NULL}};
const Segment SGI239915[] = {{&S86765[103], 9}, {NULL}};
const Segment SGI239916[] = {{&S86765[76], 6}, {NULL}};
const Segment SGI239917[] = {{&S95314[34], 13}, {NULL}};
const Segment SGI239918[] = {{&S95314[27], 6}, {NULL}};
const Segment SGI239919[] = {{&S95314[100], 9}, {NULL}};
const Segment SGI239920[] = {{&S95314[73], 6}, {NULL}};
const Segment SGI239921[] = {{&S95330[34], 17}, {NULL}};
const Segment SGI239922[] = {{&S95330[27], 6}, {NULL}};
const Segment SGI239923[] = {{&S95330[104], 9}, {NULL}};
const Segment SGI239924[] = {{&S95330[77], 6}, {NULL}};
const Segment SGI239925[] = {{&S95354[34], 7}, {NULL}};
const Segment SGI239926[] = {{&S95354[27], 6}, {NULL}};
const Segment SGI239927[] = {{&S104867[34], 10}, {NULL}};
const Segment SGI239928[] = {{&S104867[27], 6}, {NULL}};
const Segment SGI239929[] = {{&S95354[94], 9}, {NULL}};
const Segment SGI239930[] = {{&S95354[67], 6}, {NULL}};
const Segment SGI239931[] = {{&S104867[122], 9}, {NULL}};
const Segment SGI239932[] = {{&S104867[95], 6}, {NULL}};
const Segment SGI239933[] = {{&S101381[34], 9}, {NULL}};
const Segment SGI239934[] = {{&S101381[27], 6}, {NULL}};
const Segment SGI239935[] = {{&S101381[105], 9}, {NULL}};
const Segment SGI239936[] = {{&S101381[78], 6}, {NULL}};
const Segment SGI239937[] = {{&S86823[34], 7}, {NULL}};
const Segment SGI239938[] = {{&S104873[57], 10}, {NULL}};
const Segment SGI239939[] = {{&S104873[57], 22}, {NULL}};
const Segment SGI239940[] = {{&S104873[43], 6}, {NULL}};
const Segment SGI239941[] = {{&S104873[44], 3}, {NULL}};
const Segment SGI239942[] = {{&S104873[93], 6}, {NULL}};
const Segment SGI239943[] = {{&S104873[0], 12}, {NULL}};
const Segment SGI239944[] = {{&S104880[56], 10}, {NULL}};
const Segment SGI239945[] = {{&S104880[56], 22}, {NULL}};
const Segment SGI239946[] = {{&S104880[42], 6}, {NULL}};
const Segment SGI239947[] = {{&S104880[43], 3}, {NULL}};
const Segment SGI239948[] = {{&S104880[92], 6}, {NULL}};
const Segment SGI239949[] = {{&S104880[0], 12}, {NULL}};
const Segment SGI239950[] = {{&S104883[45], 10}, {NULL}};
const Segment SGI239951[] = {{&S104883[30], 6}, {NULL}};
const Segment SGI239952[] = {{&S104883[31], 3}, {NULL}};
const Segment SGI239953[] = {{&S104883[81], 5}, {NULL}};
const Segment SGI239954[] = {{&S104883[139], 13}, {NULL}};
const Segment SGI239955[] = {{&S104884[31], 8}, {NULL}};
const Segment SGI239956[] = {{&S104884[55], 3}, {NULL}};
const Segment SGI239957[] = {{&S104893[62], 10}, {NULL}};
const Segment SGI239958[] = {{&S104893[55], 6}, {NULL}};
const Segment SGI239959[] = {{&S104894[59], 10}, {NULL}};
const Segment SGI239960[] = {{&S104894[59], 22}, {NULL}};
const Segment SGI239961[] = {{&S104894[45], 6}, {NULL}};
const Segment SGI239962[] = {{&S104894[46], 3}, {NULL}};
const Segment SGI239963[] = {{&S104894[95], 6}, {NULL}};
const Segment SGI239964[] = {{&S104894[0], 14}, {NULL}};
const Segment SGI239965[] = {{&S104898[59], 10}, {NULL}};
const Segment SGI239966[] = {{&S104898[59], 22}, {NULL}};
const Segment SGI239967[] = {{&S104898[45], 6}, {NULL}};
const Segment SGI239968[] = {{&S104898[46], 3}, {NULL}};
const Segment SGI239969[] = {{&S104898[95], 6}, {NULL}};
const Segment SGI239970[] = {{&S104898[0], 14}, {NULL}};
const Segment SGI239971[] = {{&S104901[59], 10}, {NULL}};
const Segment SGI239972[] = {{&S104901[59], 22}, {NULL}};
const Segment SGI239973[] = {{&S104901[45], 6}, {NULL}};
const Segment SGI239974[] = {{&S104901[95], 5}, {NULL}};
const Segment SGI239975[] = {{&S104901[0], 14}, {NULL}};
const Segment SGI239976[] = {{&S104902[44], 10}, {NULL}};
const Segment SGI239977[] = {{&S104902[44], 22}, {NULL}};
const Segment SGI239978[] = {{&S104902[30], 6}, {NULL}};
const Segment SGI239979[] = {{&S104902[31], 3}, {NULL}};
const Segment SGI239980[] = {{&S104902[80], 5}, {NULL}};
const Segment SGI239981[] = {{&S104902[125], 14}, {NULL}};
const Segment SGI239982[] = {{&S104904[59], 10}, {NULL}};
const Segment SGI239983[] = {{&S104904[59], 22}, {NULL}};
const Segment SGI239984[] = {{&S104904[45], 6}, {NULL}};
const Segment SGI239985[] = {{&S104904[46], 3}, {NULL}};
const Segment SGI239986[] = {{&S104904[95], 6}, {NULL}};
const Segment SGI239987[] = {{&S104904[0], 14}, {NULL}};
const Segment SGI239988[] = {{&S104905[62], 10}, {NULL}};
const Segment SGI239989[] = {{&S104905[55], 6}, {NULL}};
const Segment SGI239990[] = {{&S104907[59], 10}, {NULL}};
const Segment SGI239991[] = {{&S104907[59], 22}, {NULL}};
const Segment SGI239992[] = {{&S104907[45], 6}, {NULL}};
const Segment SGI239993[] = {{&S104907[95], 5}, {NULL}};
const Segment SGI239994[] = {{&S104907[0], 14}, {NULL}};
const Segment SGI239995[] = {{&S94495[45], 10}, {NULL}};
const Segment SGI239996[] = {{&S94495[30], 6}, {NULL}};
const Segment SGI239997[] = {{&S94495[115], 3}, {NULL}};
const Segment SGI239998[] = {{&S94495[81], 5}, {NULL}};
const Segment SGI239999[] = {{&S94495[139], 13}, {NULL}};
const Segment SGI240000[] = {{&S104919[57], 10}, {NULL}};
const Segment SGI240001[] = {{&S104919[45], 6}, {NULL}};
const Segment SGI240002[] = {{&S104919[46], 3}, {NULL}};
const Segment SGI240003[] = {{&S104919[93], 6}, {NULL}};
const Segment SGI240004[] = {{&S104919[0], 14}, {NULL}};
const Segment SGI240005[] = {{&S104920[59], 10}, {NULL}};
const Segment SGI240006[] = {{&S104920[59], 22}, {NULL}};
const Segment SGI240007[] = {{&S104920[45], 6}, {NULL}};
const Segment SGI240008[] = {{&S104920[46], 3}, {NULL}};
const Segment SGI240009[] = {{&S104920[95], 6}, {NULL}};
const Segment SGI240010[] = {{&S104920[0], 14}, {NULL}};
const Segment SGI240011[] = {{&S104923[45], 10}, {NULL}};
const Segment SGI240012[] = {{&S104923[30], 6}, {NULL}};
const Segment SGI240013[] = {{&S104923[31], 3}, {NULL}};
const Segment SGI240014[] = {{&S104923[81], 5}, {NULL}};
const Segment SGI240015[] = {{&S104923[139], 13}, {NULL}};
const Segment SGI240016[] = {{&S94506[44], 11}, {NULL}};
const Segment SGI240017[] = {{&S94506[44], 23}, {NULL}};
const Segment SGI240018[] = {{&S94506[30], 6}, {NULL}};
const Segment SGI240019[] = {{&S104924[44], 10}, {NULL}};
const Segment SGI240020[] = {{&S104924[44], 22}, {NULL}};
const Segment SGI240021[] = {{&S104924[30], 6}, {NULL}};
const Segment SGI240022[] = {{&S94506[81], 6}, {NULL}};
const Segment SGI240023[] = {{&S94506[108], 13}, {NULL}};
const Segment SGI240024[] = {{&S104924[31], 3}, {NULL}};
const Segment SGI240025[] = {{&S104924[80], 6}, {NULL}};
const Segment SGI240026[] = {{&S104924[132], 10}, {NULL}};
const Segment SGI240027[] = {{&S94511[34], 7}, {NULL}};
const Segment SGI240028[] = {{&S94511[27], 6}, {NULL}};
const Segment SGI240029[] = {{&S94511[94], 9}, {NULL}};
const Segment SGI240030[] = {{&S94511[67], 6}, {NULL}};
const Segment SGI240031[] = {{&S104935[44], 10}, {NULL}};
const Segment SGI240032[] = {{&S104935[44], 22}, {NULL}};
const Segment SGI240033[] = {{&S104935[30], 6}, {NULL}};
const Segment SGI240034[] = {{&S104935[31], 3}, {NULL}};
const Segment SGI240035[] = {{&S104935[99], 6}, {NULL}};
const Segment SGI240036[] = {{&S104935[151], 10}, {NULL}};
const Segment SGI240037[] = {{&S104935[68], 8}, {NULL}};
const Segment SGI240038[] = {{&S104936[59], 10}, {NULL}};
const Segment SGI240039[] = {{&S104936[59], 22}, {NULL}};
const Segment SGI240040[] = {{&S104936[45], 6}, {NULL}};
const Segment SGI240041[] = {{&S104936[46], 3}, {NULL}};
const Segment SGI240042[] = {{&S104936[95], 6}, {NULL}};
const Segment SGI240043[] = {{&S104936[0], 14}, {NULL}};
const Segment SGI240044[] = {{&S104937[59], 10}, {NULL}};
const Segment SGI240045[] = {{&S104937[59], 22}, {NULL}};
const Segment SGI240046[] = {{&S104937[45], 6}, {NULL}};
const Segment SGI240047[] = {{&S104937[46], 3}, {NULL}};
const Segment SGI240048[] = {{&S104937[95], 6}, {NULL}};
const Segment SGI240049[] = {{&S104937[0], 14}, {NULL}};
const Segment SGI240050[] = {{&S94527[34], 7}, {NULL}};
const Segment SGI240051[] = {{&S94527[27], 6}, {NULL}};
const Segment SGI240052[] = {{&S94527[94], 9}, {NULL}};
const Segment SGI240053[] = {{&S94527[67], 6}, {NULL}};
const Segment SGI240054[] = {{&S104943[44], 10}, {NULL}};
const Segment SGI240055[] = {{&S104943[44], 22}, {NULL}};
const Segment SGI240056[] = {{&S104943[30], 6}, {NULL}};
const Segment SGI240057[] = {{&S104943[139], 4}, {NULL}};
const Segment SGI240058[] = {{&S104943[165], 3}, {NULL}};
const Segment SGI240059[] = {{&S104943[31], 3}, {&S104943[214], 5}, {NULL}};
const Segment SGI240060[] = {{&S104943[80], 6}, {NULL}};
const Segment SGI240061[] = {{&S104945[59], 10}, {NULL}};
const Segment SGI240062[] = {{&S104945[59], 22}, {NULL}};
const Segment SGI240063[] = {{&S104945[45], 6}, {NULL}};
const Segment SGI240064[] = {{&S104945[46], 3}, {NULL}};
const Segment SGI240065[] = {{&S104945[110], 6}, {NULL}};
const Segment SGI240066[] = {{&S104945[83], 8}, {NULL}};
const Segment SGI240067[] = {{&S104945[0], 14}, {NULL}};
const Segment SGI240068[] = {{&S95469[34], 7}, {NULL}};
const Segment SGI240069[] = {{&S95469[27], 6}, {NULL}};
const Segment SGI240070[] = {{&S95469[94], 9}, {NULL}};
const Segment SGI240071[] = {{&S95469[67], 6}, {NULL}};
const Segment SGI240072[] = {{&S104950[44], 10}, {NULL}};
const Segment SGI240073[] = {{&S104950[44], 22}, {NULL}};
const Segment SGI240074[] = {{&S104950[30], 6}, {NULL}};
const Segment SGI240075[] = {{&S104950[158], 3}, {NULL}};
const Segment SGI240076[] = {{&S104950[124], 3}, {NULL}};
const Segment SGI240077[] = {{&S104950[89], 6}, {NULL}};
const Segment SGI240078[] = {{&S104950[144], 8}, {NULL}};
const Segment SGI240079[] = {{&S104952[44], 10}, {NULL}};
const Segment SGI240080[] = {{&S104952[44], 22}, {NULL}};
const Segment SGI240081[] = {{&S104952[30], 6}, {NULL}};
const Segment SGI240082[] = {{&S104952[158], 3}, {NULL}};
const Segment SGI240083[] = {{&S104952[124], 3}, {NULL}};
const Segment SGI240084[] = {{&S104952[89], 6}, {NULL}};
const Segment SGI240085[] = {{&S104952[144], 8}, {NULL}};
const Segment SGI240086[] = {{&S86823[27], 6}, {NULL}};
const Segment SGI240087[] = {{&S86823[93], 9}, {NULL}};
const Segment SGI240088[] = {{&S86823[66], 6}, {NULL}};
const Segment SGI240089[] = {{&S104956[44], 10}, {NULL}};
const Segment SGI240090[] = {{&S104956[44], 22}, {NULL}};
const Segment SGI240091[] = {{&S104956[30], 6}, {NULL}};
const Segment SGI240092[] = {{&S104956[31], 3}, {NULL}};
const Segment SGI240093[] = {{&S104956[80], 5}, {NULL}};
const Segment SGI240094[] = {{&S104956[117], 14}, {NULL}};
const Segment SGI240095[] = {{&S94554[34], 7}, {NULL}};
const Segment SGI240096[] = {{&S94554[27], 6}, {NULL}};
const Segment SGI240097[] = {{&S94554[94], 9}, {NULL}};
const Segment SGI240098[] = {{&S94554[67], 6}, {NULL}};
const Segment SGI240099[] = {{&S104957[44], 10}, {NULL}};
const Segment SGI240100[] = {{&S104957[44], 22}, {NULL}};
const Segment SGI240101[] = {{&S104957[30], 6}, {NULL}};
const Segment SGI240102[] = {{&S104957[31], 3}, {NULL}};
const Segment SGI240103[] = {{&S104957[80], 5}, {NULL}};
const Segment SGI240104[] = {{&S104957[117], 14}, {NULL}};
const Segment SGI240105[] = {{&S94557[34], 9}, {NULL}};
const Segment SGI240106[] = {{&S94557[27], 6}, {NULL}};
const Segment SGI240107[] = {{&S94557[96], 9}, {NULL}};
const Segment SGI240108[] = {{&S94557[69], 6}, {NULL}};
const Segment SGI240109[] = {{&S86833[34], 7}, {NULL}};
const Segment SGI240110[] = {{&S86833[27], 6}, {NULL}};
const Segment SGI240111[] = {{&S86833[93], 9}, {NULL}};
const Segment SGI240112[] = {{&S86833[66], 6}, {NULL}};
const Segment SGI240113[] = {{&S104965[44], 10}, {NULL}};
const Segment SGI240114[] = {{&S104965[44], 22}, {NULL}};
const Segment SGI240115[] = {{&S104965[30], 6}, {NULL}};
const Segment SGI240116[] = {{&S104965[31], 3}, {NULL}};
const Segment SGI240117[] = {{&S104965[80], 6}, {NULL}};
const Segment SGI240118[] = {{&S86838[34], 9}, {NULL}};
const Segment SGI240119[] = {{&S86838[27], 6}, {NULL}};
const Segment SGI240120[] = {{&S104965[133], 13}, {NULL}};
const Segment SGI240121[] = {{&S86838[95], 9}, {NULL}};
const Segment SGI240122[] = {{&S86838[68], 6}, {NULL}};
const Segment SGI240123[] = {{&S104966[44], 10}, {NULL}};
const Segment SGI240124[] = {{&S104966[44], 22}, {NULL}};
const Segment SGI240125[] = {{&S104966[30], 6}, {NULL}};
const Segment SGI240126[] = {{&S86839[34], 13}, {NULL}};
const Segment SGI240127[] = {{&S86839[27], 6}, {NULL}};
const Segment SGI240128[] = {{&S104966[80], 6}, {NULL}};
const Segment SGI240129[] = {{&S104966[107], 13}, {NULL}};
const Segment SGI240130[] = {{&S86839[99], 9}, {NULL}};
const Segment SGI240131[] = {{&S86839[72], 6}, {NULL}};
const Segment SGI240132[] = {{&S94575[52], 10}, {NULL}};
const Segment SGI240133[] = {{&S94575[52], 22}, {NULL}};
const Segment SGI240134[] = {{&S94575[38], 6}, {NULL}};
const Segment SGI240135[] = {{&S94575[88], 5}, {NULL}};
const Segment SGI240136[] = {{&S94575[11], 3}, {NULL}};
const Segment SGI240137[] = {{&S104977[44], 10}, {NULL}};
const Segment SGI240138[] = {{&S104977[44], 22}, {NULL}};
const Segment SGI240139[] = {{&S104977[30], 6}, {NULL}};
const Segment SGI240140[] = {{&S104977[31], 3}, {NULL}};
const Segment SGI240141[] = {{&S104977[80], 5}, {NULL}};
const Segment SGI240142[] = {{&S104977[117], 14}, {NULL}};
const Segment SGI240143[] = {{&S94584[34], 10}, {NULL}};
const Segment SGI240144[] = {{&S86857[34], 7}, {NULL}};
const Segment SGI240145[] = {{&S94584[27], 6}, {NULL}};
const Segment SGI240146[] = {{&S86857[27], 6}, {NULL}};
const Segment SGI240147[] = {{&S94584[97], 9}, {NULL}};
const Segment SGI240148[] = {{&S94584[70], 6}, {NULL}};
const Segment SGI240149[] = {{&S86857[94], 9}, {NULL}};
const Segment SGI240150[] = {{&S86857[67], 6}, {NULL}};
const Segment SGI240151[] = {{&S86858[34], 9}, {NULL}};
const Segment SGI240152[] = {{&S86858[27], 6}, {NULL}};
const Segment SGI240153[] = {{&S86858[96], 9}, {NULL}};
const Segment SGI240154[] = {{&S86858[69], 6}, {NULL}};
const Segment SGI240155[] = {{&S86862[34], 9}, {NULL}};
const Segment SGI240156[] = {{&S86862[27], 6}, {NULL}};
const Segment SGI240157[] = {{&S86862[96], 9}, {NULL}};
const Segment SGI240158[] = {{&S86862[69], 6}, {NULL}};
const Segment SGI240159[] = {{&S94592[34], 9}, {NULL}};
const Segment SGI240160[] = {{&S94592[27], 6}, {NULL}};
const Segment SGI240161[] = {{&S94592[96], 9}, {NULL}};
const Segment SGI240162[] = {{&S94592[69], 6}, {NULL}};
const Segment SGI240163[] = {{&S94596[44], 10}, {NULL}};
const Segment SGI240164[] = {{&S94596[44], 22}, {NULL}};
const Segment SGI240165[] = {{&S94596[30], 6}, {NULL}};
const Segment SGI240166[] = {{&S94596[115], 3}, {NULL}};
const Segment SGI240167[] = {{&S94596[80], 6}, {NULL}};
const Segment SGI240168[] = {{&S94596[140], 13}, {NULL}};
const Segment SGI240169[] = {{&S105000[44], 10}, {NULL}};
const Segment SGI240170[] = {{&S105000[44], 22}, {NULL}};
const Segment SGI240171[] = {{&S105000[30], 6}, {NULL}};
const Segment SGI240172[] = {{&S105000[80], 6}, {NULL}};
const Segment SGI240173[] = {{&S105000[107], 13}, {NULL}};
const Segment SGI240174[] = {{&S94615[34], 16}, {NULL}};
const Segment SGI240175[] = {{&S94615[27], 6}, {NULL}};
const Segment SGI240176[] = {{&S94615[103], 9}, {NULL}};
const Segment SGI240177[] = {{&S94615[76], 6}, {NULL}};
const Segment SGI240178[] = {{&S105014[59], 10}, {NULL}};
const Segment SGI240179[] = {{&S105014[59], 22}, {NULL}};
const Segment SGI240180[] = {{&S105014[45], 6}, {NULL}};
const Segment SGI240181[] = {{&S105014[46], 3}, {NULL}};
const Segment SGI240182[] = {{&S105014[95], 6}, {NULL}};
const Segment SGI240183[] = {{&S105014[0], 14}, {NULL}};
const Segment SGI240184[] = {{&S105015[44], 10}, {NULL}};
const Segment SGI240185[] = {{&S105015[44], 22}, {NULL}};
const Segment SGI240186[] = {{&S105015[30], 6}, {NULL}};
const Segment SGI240187[] = {{&S105015[31], 3}, {NULL}};
const Segment SGI240188[] = {{&S105015[99], 6}, {NULL}};
const Segment SGI240189[] = {{&S105015[169], 3}, {NULL}};
const Segment SGI240190[] = {{&S105015[68], 8}, {NULL}};
const Segment SGI240191[] = {{&S94620[34], 14}, {NULL}};
const Segment SGI240192[] = {{&S94620[27], 6}, {NULL}};
const Segment SGI240193[] = {{&S94620[101], 9}, {NULL}};
const Segment SGI240194[] = {{&S94620[74], 6}, {NULL}};
const Segment SGI240195[] = {{&S105017[44], 10}, {NULL}};
const Segment SGI240196[] = {{&S105017[44], 22}, {NULL}};
const Segment SGI240197[] = {{&S105017[30], 6}, {NULL}};
const Segment SGI240198[] = {{&S105017[31], 3}, {NULL}};
const Segment SGI240199[] = {{&S105017[80], 6}, {NULL}};
const Segment SGI240200[] = {{&S105017[133], 11}, {NULL}};
const Segment SGI240201[] = {{&S105018[59], 10}, {NULL}};
const Segment SGI240202[] = {{&S105018[59], 22}, {NULL}};
const Segment SGI240203[] = {{&S105018[45], 6}, {NULL}};
const Segment SGI240204[] = {{&S105018[46], 3}, {NULL}};
const Segment SGI240205[] = {{&S105018[95], 6}, {NULL}};
const Segment SGI240206[] = {{&S105018[0], 14}, {NULL}};
const Segment SGI240207[] = {{&S105022[44], 10}, {NULL}};
const Segment SGI240208[] = {{&S105022[44], 22}, {NULL}};
const Segment SGI240209[] = {{&S105022[30], 6}, {NULL}};
const Segment SGI240210[] = {{&S105022[115], 3}, {NULL}};
const Segment SGI240211[] = {{&S105022[80], 6}, {NULL}};
const Segment SGI240212[] = {{&S105022[135], 8}, {NULL}};
const Segment SGI240213[] = {{&S105022[144], 13}, {NULL}};
const Segment SGI240214[] = {{&S105073[34], 7}, {NULL}};
const Segment SGI240215[] = {{&S105073[27], 6}, {NULL}};
const Segment SGI240216[] = {{&S105073[100], 9}, {NULL}};
const Segment SGI240217[] = {{&S105073[73], 6}, {NULL}};
const Segment SGI240218[] = {{&S105073[141], 8}, {NULL}};
const Segment SGI240219[] = {{&S105075[34], 12}, {NULL}};
const Segment SGI240220[] = {{&S105075[27], 6}, {NULL}};
const Segment SGI240221[] = {{&S105075[99], 9}, {NULL}};
const Segment SGI240222[] = {{&S105075[72], 6}, {NULL}};
const Segment SGI240223[] = {{&S105075[140], 8}, {NULL}};
const Segment SGI240224[] = {{&S105076[34], 5}, {NULL}};
const Segment SGI240225[] = {{&S105076[27], 6}, {NULL}};
const Segment SGI240226[] = {{&S105076[92], 9}, {NULL}};
const Segment SGI240227[] = {{&S105076[65], 6}, {NULL}};
const Segment SGI240228[] = {{&S105076[126], 8}, {NULL}};
const Segment SGI240229[] = {{&S95523[34], 8}, {NULL}};
const Segment SGI240230[] = {{&S95523[27], 6}, {NULL}};
const Segment SGI240231[] = {{&S95523[95], 9}, {NULL}};
const Segment SGI240232[] = {{&S95523[68], 6}, {NULL}};
const Segment SGI240233[] = {{&S86974[34], 10}, {NULL}};
const Segment SGI240234[] = {{&S86974[27], 6}, {NULL}};
const Segment SGI240235[] = {{&S86974[96], 9}, {NULL}};
const Segment SGI240236[] = {{&S86974[69], 6}, {NULL}};
const Segment SGI240237[] = {{&S94728[34], 10}, {NULL}};
const Segment SGI240238[] = {{&S94728[27], 6}, {NULL}};
const Segment SGI240239[] = {{&S94728[97], 9}, {NULL}};
const Segment SGI240240[] = {{&S94728[70], 6}, {NULL}};
const Segment SGI240241[] = {{&S86995[44], 10}, {NULL}};
const Segment SGI240242[] = {{&S86995[44], 22}, {NULL}};
const Segment SGI240243[] = {{&S86995[30], 6}, {NULL}};
const Segment SGI240244[] = {{&S86995[115], 3}, {NULL}};
const Segment SGI240245[] = {{&S86995[80], 6}, {NULL}};
const Segment SGI240246[] = {{&S86995[140], 8}, {NULL}};
const Segment SGI240247[] = {{&S94747[34], 10}, {NULL}};
const Segment SGI240248[] = {{&S94747[27], 6}, {NULL}};
const Segment SGI240249[] = {{&S94747[97], 9}, {NULL}};
const Segment SGI240250[] = {{&S94747[70], 6}, {NULL}};
const Segment SGI240251[] = {{&S94748[44], 10}, {NULL}};
const Segment SGI240252[] = {{&S94748[44], 22}, {NULL}};
const Segment SGI240253[] = {{&S94748[30], 6}, {NULL}};
const Segment SGI240254[] = {{&S94748[80], 6}, {NULL}};
const Segment SGI240255[] = {{&S94748[107], 13}, {NULL}};
const Segment SGI240256[] = {{&S101733[34], 10}, {NULL}};
const Segment SGI240257[] = {{&S101733[27], 6}, {NULL}};
const Segment SGI240258[] = {{&S101733[97], 9}, {NULL}};
const Segment SGI240259[] = {{&S101733[70], 6}, {NULL}};
const Segment SGI240260[] = {{&S95580[44], 15}, {NULL}};
const Segment SGI240261[] = {{&S101734[44], 10}, {NULL}};
const Segment SGI240262[] = {{&S95580[44], 27}, {NULL}};
const Segment SGI240263[] = {{&S101734[44], 22}, {NULL}};
const Segment SGI240264[] = {{&S95580[30], 6}, {NULL}};
const Segment SGI240265[] = {{&S101734[30], 6}, {NULL}};
const Segment SGI240266[] = {{&S101734[115], 3}, {NULL}};
const Segment SGI240267[] = {{&S101734[80], 6}, {NULL}};
const Segment SGI240268[] = {{&S95580[85], 6}, {NULL}};
const Segment SGI240269[] = {{&S95580[122], 3}, {NULL}};
const Segment SGI240270[] = {{&S87006[34], 10}, {NULL}};
const Segment SGI240271[] = {{&S87006[27], 6}, {NULL}};
const Segment SGI240272[] = {{&S87006[97], 9}, {NULL}};
const Segment SGI240273[] = {{&S87006[70], 6}, {NULL}};
const Segment SGI240274[] = {{&S101740[44], 9}, {NULL}};
const Segment SGI240275[] = {{&S101740[44], 30}, {NULL}};
const Segment SGI240276[] = {{&S101740[30], 6}, {NULL}};
const Segment SGI240277[] = {{&S101740[123], 3}, {NULL}};
const Segment SGI240278[] = {{&S101740[88], 6}, {NULL}};
const Segment SGI240279[] = {{&S94763[34], 8}, {NULL}};
const Segment SGI240280[] = {{&S94763[27], 6}, {NULL}};
const Segment SGI240281[] = {{&S94763[95], 9}, {NULL}};
const Segment SGI240282[] = {{&S94763[68], 6}, {NULL}};
const Segment SGI240283[] = {{&S94782[34], 11}, {NULL}};
const Segment SGI240284[] = {{&S94782[27], 6}, {NULL}};
const Segment SGI240285[] = {{&S94782[98], 9}, {NULL}};
const Segment SGI240286[] = {{&S94782[71], 6}, {NULL}};
const Segment SGI240287[] = {{&S95620[34], 15}, {NULL}};
const Segment SGI240288[] = {{&S95620[27], 6}, {NULL}};
const Segment SGI240289[] = {{&S95620[102], 9}, {NULL}};
const Segment SGI240290[] = {{&S95620[75], 6}, {NULL}};
const Segment SGI240291[] = {{&S94791[34], 10}, {NULL}};
const Segment SGI240292[] = {{&S94791[27], 6}, {NULL}};
const Segment SGI240293[] = {{&S94791[97], 9}, {NULL}};
const Segment SGI240294[] = {{&S94791[70], 6}, {NULL}};
const Segment SGI240295[] = {{&S101772[45], 10}, {NULL}};
const Segment SGI240296[] = {{&S101772[30], 6}, {NULL}};
const Segment SGI240297[] = {{&S101772[115], 3}, {NULL}};
const Segment SGI240298[] = {{&S101772[81], 5}, {NULL}};
const Segment SGI240299[] = {{&S101772[139], 13}, {NULL}};
const Segment SGI240300[] = {{&S94794[34], 14}, {NULL}};
const Segment SGI240301[] = {{&S94794[27], 6}, {NULL}};
const Segment SGI240302[] = {{&S94794[101], 9}, {NULL}};
const Segment SGI240303[] = {{&S94794[74], 6}, {NULL}};
const Segment SGI240304[] = {{&S95635[34], 15}, {NULL}};
const Segment SGI240305[] = {{&S95635[27], 6}, {NULL}};
const Segment SGI240306[] = {{&S95635[102], 9}, {NULL}};
const Segment SGI240307[] = {{&S95635[75], 6}, {NULL}};
const Segment SGI240308[] = {{&S94807[34], 7}, {NULL}};
const Segment SGI240309[] = {{&S94807[27], 6}, {NULL}};
const Segment SGI240310[] = {{&S94807[100], 9}, {NULL}};
const Segment SGI240311[] = {{&S94807[73], 6}, {NULL}};
const Segment SGI240312[] = {{&S94809[34], 11}, {NULL}};
const Segment SGI240313[] = {{&S94809[27], 6}, {NULL}};
const Segment SGI240314[] = {{&S94809[98], 9}, {NULL}};
const Segment SGI240315[] = {{&S94809[71], 6}, {NULL}};
const Segment SGI240316[] = {{&S94810[59], 20}, {NULL}};
const Segment SGI240317[] = {{&S94810[59], 32}, {NULL}};
const Segment SGI240318[] = {{&S94810[45], 6}, {NULL}};
const Segment SGI240319[] = {{&S94810[140], 3}, {NULL}};
const Segment SGI240320[] = {{&S94810[105], 6}, {NULL}};
const Segment SGI240321[] = {{&S94810[0], 14}, {NULL}};
const Segment SGI240322[] = {{&S94811[44], 10}, {NULL}};
const Segment SGI240323[] = {{&S94811[44], 22}, {NULL}};
const Segment SGI240324[] = {{&S94811[30], 6}, {NULL}};
const Segment SGI240325[] = {{&S94811[115], 3}, {NULL}};
const Segment SGI240326[] = {{&S94811[80], 6}, {NULL}};
const Segment SGI240327[] = {{&S94811[140], 7}, {NULL}};
const Segment SGI240328[] = {{&S94815[44], 9}, {NULL}};
const Segment SGI240329[] = {{&S94815[44], 21}, {NULL}};
const Segment SGI240330[] = {{&S94815[30], 6}, {NULL}};
const Segment SGI240331[] = {{&S95651[34], 7}, {NULL}};
const Segment SGI240332[] = {{&S95651[27], 6}, {NULL}};
const Segment SGI240333[] = {{&S94815[79], 6}, {NULL}};
const Segment SGI240334[] = {{&S95651[94], 9}, {NULL}};
const Segment SGI240335[] = {{&S94815[106], 13}, {NULL}};
const Segment SGI240336[] = {{&S95651[67], 6}, {NULL}};
const Segment SGI240337[] = {{&S94821[45], 4}, {NULL}};
const Segment SGI240338[] = {{&S94821[30], 6}, {NULL}};
const Segment SGI240339[] = {{&S94821[109], 3}, {NULL}};
const Segment SGI240340[] = {{&S94821[75], 5}, {NULL}};
const Segment SGI240341[] = {{&S94821[133], 13}, {NULL}};
const Segment SGI240342[] = {{&S94823[44], 10}, {NULL}};
const Segment SGI240343[] = {{&S94823[44], 22}, {NULL}};
const Segment SGI240344[] = {{&S94823[30], 6}, {NULL}};
const Segment SGI240345[] = {{&S94823[113], 3}, {NULL}};
const Segment SGI240346[] = {{&S94823[80], 5}, {NULL}};
const Segment SGI240347[] = {{&S94823[117], 14}, {NULL}};
const Segment SGI240348[] = {{&S87145[44], 8}, {NULL}};
const Segment SGI240349[] = {{&S87145[44], 20}, {NULL}};
const Segment SGI240350[] = {{&S87145[30], 6}, {NULL}};
const Segment SGI240351[] = {{&S87145[113], 3}, {NULL}};
const Segment SGI240352[] = {{&S87145[78], 6}, {NULL}};
const Segment SGI240353[] = {{&S87145[138], 8}, {NULL}};
const Segment SGI240354[] = {{&S101879[44], 9}, {NULL}};
const Segment SGI240355[] = {{&S101879[44], 21}, {NULL}};
const Segment SGI240356[] = {{&S101879[30], 6}, {NULL}};
const Segment SGI240357[] = {{&S101879[134], 3}, {NULL}};
const Segment SGI240358[] = {{&S101879[99], 6}, {NULL}};
const Segment SGI240359[] = {{&S101879[162], 3}, {NULL}};
const Segment SGI240360[] = {{&S101879[67], 8}, {NULL}};
const Segment SGI240361[] = {{&S87158[44], 10}, {NULL}};
const Segment SGI240362[] = {{&S87158[44], 22}, {NULL}};
const Segment SGI240363[] = {{&S87158[30], 6}, {NULL}};
const Segment SGI240364[] = {{&S87158[115], 3}, {NULL}};
const Segment SGI240365[] = {{&S87158[80], 6}, {NULL}};
const Segment SGI240366[] = {{&S87158[140], 8}, {NULL}};
const Segment SGI240367[] = {{&S87161[34], 10}, {NULL}};
const Segment SGI240368[] = {{&S87161[27], 6}, {NULL}};
const Segment SGI240369[] = {{&S87161[96], 9}, {NULL}};
const Segment SGI240370[] = {{&S87161[69], 6}, {NULL}};
const Segment SGI240371[] = {{&S87169[34], 10}, {NULL}};
const Segment SGI240372[] = {{&S87169[27], 6}, {NULL}};
const Segment SGI240373[] = {{&S87169[97], 9}, {NULL}};
const Segment SGI240374[] = {{&S87169[70], 6}, {NULL}};
const Segment SGI240375[] = {{&S101901[45], 10}, {NULL}};
const Segment SGI240376[] = {{&S101901[30], 6}, {NULL}};
const Segment SGI240377[] = {{&S101901[115], 3}, {NULL}};
const Segment SGI240378[] = {{&S101901[81], 5}, {NULL}};
const Segment SGI240379[] = {{&S101901[139], 13}, {NULL}};
const Segment SGI240380[] = {{&S87178[34], 10}, {NULL}};
const Segment SGI240381[] = {{&S87178[27], 6}, {NULL}};
const Segment SGI240382[] = {{&S87178[96], 9}, {NULL}};
const Segment SGI240383[] = {{&S87178[69], 6}, {NULL}};
const Segment SGI240384[] = {{&S95814[34], 10}, {NULL}};
const Segment SGI240385[] = {{&S95814[27], 6}, {NULL}};
const Segment SGI240386[] = {{&S95814[97], 9}, {NULL}};
const Segment SGI240387[] = {{&S95814[70], 6}, {NULL}};
const Segment SGI240388[] = {{&S94828[44], 19}, {NULL}};
const Segment SGI240389[] = {{&S94828[44], 31}, {NULL}};
const Segment SGI240390[] = {{&S94828[30], 6}, {NULL}};
const Segment SGI240391[] = {{&S94828[124], 3}, {NULL}};
const Segment SGI240392[] = {{&S94828[89], 6}, {NULL}};
const Segment SGI240393[] = {{&S94829[52], 10}, {NULL}};
const Segment SGI240394[] = {{&S94829[52], 22}, {NULL}};
const Segment SGI240395[] = {{&S94829[38], 6}, {NULL}};
const Segment SGI240396[] = {{&S94829[88], 5}, {NULL}};
const Segment SGI240397[] = {{&S94829[11], 3}, {NULL}};
const Segment SGI240398[] = {{&S94833[34], 11}, {NULL}};
const Segment SGI240399[] = {{&S94833[27], 6}, {NULL}};
const Segment SGI240400[] = {{&S94833[98], 9}, {NULL}};
const Segment SGI240401[] = {{&S94833[71], 6}, {NULL}};
const Segment SGI240402[] = {{&S94835[34], 11}, {NULL}};
const Segment SGI240403[] = {{&S94835[27], 6}, {NULL}};
const Segment SGI240404[] = {{&S94835[98], 9}, {NULL}};
const Segment SGI240405[] = {{&S94835[71], 6}, {NULL}};
const Segment SGI240406[] = {{&S87249[44], 10}, {NULL}};
const Segment SGI240407[] = {{&S87249[44], 21}, {NULL}};
const Segment SGI240408[] = {{&S87249[30], 6}, {NULL}};
const Segment SGI240409[] = {{&S87249[79], 6}, {NULL}};
const Segment SGI240410[] = {{&S87249[106], 13}, {NULL}};
const Segment SGI240411[] = {{&S94837[34], 10}, {NULL}};
const Segment SGI240412[] = {{&S94837[27], 6}, {NULL}};
const Segment SGI240413[] = {{&S94837[97], 9}, {NULL}};
const Segment SGI240414[] = {{&S94837[70], 6}, {NULL}};
const Segment SGI240415[] = {{&S94842[34], 8}, {NULL}};
const Segment SGI240416[] = {{&S94842[27], 6}, {NULL}};
const Segment SGI240417[] = {{&S94842[95], 9}, {NULL}};
const Segment SGI240418[] = {{&S94842[68], 6}, {NULL}};
const Segment SGI240419[] = {{&S94855[34], 5}, {NULL}};
const Segment SGI240420[] = {{&S94855[27], 6}, {NULL}};
const Segment SGI240421[] = {{&S94855[92], 9}, {NULL}};
const Segment SGI240422[] = {{&S94855[65], 6}, {NULL}};
const Segment SGI240423[] = {{&S87274[34], 10}, {NULL}};
const Segment SGI240424[] = {{&S87274[27], 6}, {NULL}};
const Segment SGI240425[] = {{&S87274[96], 9}, {NULL}};
const Segment SGI240426[] = {{&S87274[69], 6}, {NULL}};
const Segment SGI240427[] = {{&S87274[137], 8}, {NULL}};
const Segment SGI240428[] = {{&S94871[34], 12}, {NULL}};
const Segment SGI240429[] = {{&S94871[27], 6}, {NULL}};
const Segment SGI240430[] = {{&S94871[99], 9}, {NULL}};
const Segment SGI240431[] = {{&S94871[72], 6}, {NULL}};
const Segment SGI240432[] = {{&S94871[140], 8}, {NULL}};
const Segment SGI240433[] = {{&S94873[34], 9}, {NULL}};
const Segment SGI240434[] = {{&S94873[27], 6}, {NULL}};
const Segment SGI240435[] = {{&S94873[96], 9}, {NULL}};
const Segment SGI240436[] = {{&S94873[69], 6}, {NULL}};
const Segment SGI240437[] = {{&S105489[44], 12}, {NULL}};
const Segment SGI240438[] = {{&S105489[44], 22}, {NULL}};
const Segment SGI240439[] = {{&S105489[30], 6}, {NULL}};
const Segment SGI240440[] = {{&S105489[31], 3}, {NULL}};
const Segment SGI240441[] = {{&S105489[80], 6}, {NULL}};
const Segment SGI240442[] = {{&S87338[34], 12}, {NULL}};
const Segment SGI240443[] = {{&S87338[27], 6}, {NULL}};
const Segment SGI240444[] = {{&S87338[98], 9}, {NULL}};
const Segment SGI240445[] = {{&S87338[71], 6}, {NULL}};
const Segment SGI240446[] = {{&S87348[44], 9}, {NULL}};
const Segment SGI240447[] = {{&S87348[44], 20}, {NULL}};
const Segment SGI240448[] = {{&S87348[30], 6}, {NULL}};
const Segment SGI240449[] = {{&S87348[113], 3}, {NULL}};
const Segment SGI240450[] = {{&S87348[78], 6}, {NULL}};
const Segment SGI240451[] = {{&S87348[137], 10}, {NULL}};
const Segment SGI240452[] = {{&S105521[45], 7}, {NULL}};
const Segment SGI240453[] = {{&S105521[30], 6}, {NULL}};
const Segment SGI240454[] = {{&S105521[31], 3}, {NULL}};
const Segment SGI240455[] = {{&S105521[83], 5}, {NULL}};
const Segment SGI240456[] = {{&S105521[141], 13}, {NULL}};
const Segment SGI240457[] = {{&S102073[44], 10}, {NULL}};
const Segment SGI240458[] = {{&S102073[44], 22}, {NULL}};
const Segment SGI240459[] = {{&S102073[30], 6}, {NULL}};
const Segment SGI240460[] = {{&S102073[144], 3}, {NULL}};
const Segment SGI240461[] = {{&S102073[109], 6}, {NULL}};
const Segment SGI240462[] = {{&S102073[68], 8}, {NULL}};
const Segment SGI240463[] = {{&S102073[80], 15}, {NULL}};
const Segment SGI240464[] = {{&S87360[34], 10}, {NULL}};
const Segment SGI240465[] = {{&S87360[27], 6}, {NULL}};
const Segment SGI240466[] = {{&S87360[97], 9}, {NULL}};
const Segment SGI240467[] = {{&S87360[70], 6}, {NULL}};
const Segment SGI240468[] = {{&S105531[44], 10}, {NULL}};
const Segment SGI240469[] = {{&S105531[44], 22}, {NULL}};
const Segment SGI240470[] = {{&S105531[30], 6}, {NULL}};
const Segment SGI240471[] = {{&S102077[44], 9}, {NULL}};
const Segment SGI240472[] = {{&S105531[31], 3}, {NULL}};
const Segment SGI240473[] = {{&S102077[44], 21}, {NULL}};
const Segment SGI240474[] = {{&S102077[30], 6}, {NULL}};
const Segment SGI240475[] = {{&S105531[109], 6}, {NULL}};
const Segment SGI240476[] = {{&S105531[68], 8}, {NULL}};
const Segment SGI240477[] = {{&S102077[143], 3}, {NULL}};
const Segment SGI240478[] = {{&S102077[108], 6}, {NULL}};
const Segment SGI240479[] = {{&S105531[80], 15}, {NULL}};
const Segment SGI240480[] = {{&S102077[67], 8}, {NULL}};
const Segment SGI240481[] = {{&S102077[79], 15}, {NULL}};
const Segment SGI240482[] = {{&S87368[34], 9}, {NULL}};
const Segment SGI240483[] = {{&S87368[27], 6}, {NULL}};
const Segment SGI240484[] = {{&S87368[95], 9}, {NULL}};
const Segment SGI240485[] = {{&S87368[68], 6}, {NULL}};
const Segment SGI240486[] = {{&S94958[34], 9}, {NULL}};
const Segment SGI240487[] = {{&S94958[27], 6}, {NULL}};
const Segment SGI240488[] = {{&S94958[96], 9}, {NULL}};
const Segment SGI240489[] = {{&S94958[69], 6}, {NULL}};
const Segment SGI240490[] = {{&S105538[44], 15}, {NULL}};
const Segment SGI240491[] = {{&S105538[44], 26}, {NULL}};
const Segment SGI240492[] = {{&S105538[30], 6}, {NULL}};
const Segment SGI240493[] = {{&S105538[31], 3}, {NULL}};
const Segment SGI240494[] = {{&S105538[99], 6}, {NULL}};
const Segment SGI240495[] = {{&S105538[72], 8}, {NULL}};
const Segment SGI240496[] = {{&S105540[34], 12}, {NULL}};
const Segment SGI240497[] = {{&S105540[27], 6}, {NULL}};
const Segment SGI240498[] = {{&S105540[99], 9}, {NULL}};
const Segment SGI240499[] = {{&S105540[72], 6}, {NULL}};
const Segment SGI240500[] = {{&S94968[44], 10}, {NULL}};
const Segment SGI240501[] = {{&S94968[44], 22}, {NULL}};
const Segment SGI240502[] = {{&S94968[30], 6}, {NULL}};
const Segment SGI240503[] = {{&S94968[142], 3}, {NULL}};
const Segment SGI240504[] = {{&S94968[107], 6}, {NULL}};
const Segment SGI240505[] = {{&S94968[68], 8}, {NULL}};
const Segment SGI240506[] = {{&S94979[34], 8}, {NULL}};
const Segment SGI240507[] = {{&S94979[27], 6}, {NULL}};
const Segment SGI240508[] = {{&S94979[95], 9}, {NULL}};
const Segment SGI240509[] = {{&S94979[68], 6}, {NULL}};
const Segment SGI240510[] = {{&S87398[44], 9}, {NULL}};
const Segment SGI240511[] = {{&S87398[44], 20}, {NULL}};
const Segment SGI240512[] = {{&S87398[30], 6}, {NULL}};
const Segment SGI240513[] = {{&S87398[78], 6}, {NULL}};
const Segment SGI240514[] = {{&S87398[105], 13}, {NULL}};
const Segment SGI240515[] = {{&S87400[34], 18}, {NULL}};
const Segment SGI240516[] = {{&S87400[27], 6}, {NULL}};
const Segment SGI240517[] = {{&S87400[104], 9}, {NULL}};
const Segment SGI240518[] = {{&S87400[77], 6}, {NULL}};
const Segment SGI240519[] = {{&S102119[44], 10}, {NULL}};
const Segment SGI240520[] = {{&S102119[44], 22}, {NULL}};
const Segment SGI240521[] = {{&S102119[30], 6}, {NULL}};
const Segment SGI240522[] = {{&S102119[115], 3}, {NULL}};
const Segment SGI240523[] = {{&S102119[80], 6}, {NULL}};
const Segment SGI240524[] = {{&S102119[132], 10}, {NULL}};
const Segment SGI240525[] = {{&S105580[44], 13}, {NULL}};
const Segment SGI240526[] = {{&S105580[44], 29}, {NULL}};
const Segment SGI240527[] = {{&S105580[30], 6}, {NULL}};
const Segment SGI240528[] = {{&S105580[31], 3}, {NULL}};
const Segment SGI240529[] = {{&S105580[87], 5}, {NULL}};
const Segment SGI240530[] = {{&S105580[124], 14}, {NULL}};
const Segment SGI240531[] = {{&S105590[45], 10}, {NULL}};
const Segment SGI240532[] = {{&S105590[30], 6}, {NULL}};
const Segment SGI240533[] = {{&S105590[31], 3}, {NULL}};
const Segment SGI240534[] = {{&S105590[86], 5}, {NULL}};
const Segment SGI240535[] = {{&S105590[144], 13}, {NULL}};
const Segment SGI240536[] = {{&S105592[44], 9}, {NULL}};
const Segment SGI240537[] = {{&S105592[44], 21}, {NULL}};
const Segment SGI240538[] = {{&S105592[30], 6}, {NULL}};
const Segment SGI240539[] = {{&S105592[31], 3}, {NULL}};
const Segment SGI240540[] = {{&S105592[79], 6}, {NULL}};
const Segment SGI240541[] = {{&S95018[44], 7}, {NULL}};
const Segment SGI240542[] = {{&S95018[44], 19}, {NULL}};
const Segment SGI240543[] = {{&S95018[30], 6}, {NULL}};
const Segment SGI240544[] = {{&S95018[128], 3}, {NULL}};
const Segment SGI240545[] = {{&S95018[93], 6}, {NULL}};
const Segment SGI240546[] = {{&S95018[65], 8}, {NULL}};
const Segment SGI240547[] = {{&S95018[160], 15}, {NULL}};
const Segment SGI240548[] = {{&S105602[44], 10}, {NULL}};
const Segment SGI240549[] = {{&S95025[34], 8}, {NULL}};
const Segment SGI240550[] = {{&S105602[44], 22}, {NULL}};
const Segment SGI240551[] = {{&S105602[30], 6}, {NULL}};
const Segment SGI240552[] = {{&S95025[27], 6}, {NULL}};
const Segment SGI240553[] = {{&S105602[31], 3}, {NULL}};
const Segment SGI240554[] = {{&S95025[95], 9}, {NULL}};
const Segment SGI240555[] = {{&S95025[68], 6}, {NULL}};
const Segment SGI240556[] = {{&S105602[80], 6}, {NULL}};
const Segment SGI240557[] = {{&S105602[143], 3}, {NULL}};
const Segment SGI240558[] = {{&S95029[34], 7}, {NULL}};
const Segment SGI240559[] = {{&S95029[27], 6}, {NULL}};
const Segment SGI240560[] = {{&S95029[93], 9}, {NULL}};
const Segment SGI240561[] = {{&S95029[66], 6}, {NULL}};
const Segment SGI240562[] = {{&S95031[34], 7}, {NULL}};
const Segment SGI240563[] = {{&S95031[27], 6}, {NULL}};
const Segment SGI240564[] = {{&S105609[59], 10}, {NULL}};
const Segment SGI240565[] = {{&S105609[59], 22}, {NULL}};
const Segment SGI240566[] = {{&S95031[94], 9}, {NULL}};
const Segment SGI240567[] = {{&S105609[45], 6}, {NULL}};
const Segment SGI240568[] = {{&S95031[67], 6}, {NULL}};
const Segment SGI240569[] = {{&S105609[46], 3}, {NULL}};
const Segment SGI240570[] = {{&S105609[95], 6}, {NULL}};
const Segment SGI240571[] = {{&S105609[0], 14}, {NULL}};
const Segment SGI240572[] = {{&S95035[34], 10}, {NULL}};
const Segment SGI240573[] = {{&S95035[27], 6}, {NULL}};
const Segment SGI240574[] = {{&S95035[97], 9}, {NULL}};
const Segment SGI240575[] = {{&S95035[70], 6}, {NULL}};
const Segment SGI240576[] = {{&S105613[44], 10}, {NULL}};
const Segment SGI240577[] = {{&S105613[44], 22}, {NULL}};
const Segment SGI240578[] = {{&S105613[30], 6}, {NULL}};
const Segment SGI240579[] = {{&S95035[138], 8}, {NULL}};
const Segment SGI240580[] = {{&S105613[111], 2}, {NULL}};
const Segment SGI240581[] = {{&S105613[80], 5}, {NULL}};
const Segment SGI240582[] = {{&S105615[59], 10}, {NULL}};
const Segment SGI240583[] = {{&S105615[59], 22}, {NULL}};
const Segment SGI240584[] = {{&S105615[45], 6}, {NULL}};
const Segment SGI240585[] = {{&S95038[34], 10}, {NULL}};
const Segment SGI240586[] = {{&S95038[27], 6}, {NULL}};
const Segment SGI240587[] = {{&S105615[95], 5}, {NULL}};
const Segment SGI240588[] = {{&S95038[97], 9}, {NULL}};
const Segment SGI240589[] = {{&S105615[0], 14}, {NULL}};
const Segment SGI240590[] = {{&S95038[70], 6}, {NULL}};
const Segment SGI240591[] = {{&S105616[57], 10}, {NULL}};
const Segment SGI240592[] = {{&S105616[57], 22}, {NULL}};
const Segment SGI240593[] = {{&S105616[43], 6}, {NULL}};
const Segment SGI240594[] = {{&S105616[44], 3}, {NULL}};
const Segment SGI240595[] = {{&S105616[93], 6}, {NULL}};
const Segment SGI240596[] = {{&S105616[0], 12}, {NULL}};
const Segment SGI240597[] = {{&S105619[34], 10}, {NULL}};
const Segment SGI240598[] = {{&S105619[27], 6}, {NULL}};
const Segment SGI240599[] = {{&S105619[97], 9}, {NULL}};
const Segment SGI240600[] = {{&S105619[70], 6}, {NULL}};
const Segment SGI240601[] = {{&S95047[44], 4}, {NULL}};
const Segment SGI240602[] = {{&S95047[44], 16}, {NULL}};
const Segment SGI240603[] = {{&S95047[30], 6}, {NULL}};
const Segment SGI240604[] = {{&S105622[57], 10}, {NULL}};
const Segment SGI240605[] = {{&S95047[109], 3}, {NULL}};
const Segment SGI240606[] = {{&S105622[57], 22}, {NULL}};
const Segment SGI240607[] = {{&S105622[43], 6}, {NULL}};
const Segment SGI240608[] = {{&S95047[74], 6}, {NULL}};
const Segment SGI240609[] = {{&S105622[44], 3}, {NULL}};
const Segment SGI240610[] = {{&S105622[93], 6}, {NULL}};
const Segment SGI240611[] = {{&S105622[0], 12}, {NULL}};
const Segment SGI240612[] = {{&S87475[34], 9}, {NULL}};
const Segment SGI240613[] = {{&S87475[27], 6}, {NULL}};
const Segment SGI240614[] = {{&S87475[96], 9}, {NULL}};
const Segment SGI240615[] = {{&S87475[69], 6}, {NULL}};
const Segment SGI240616[] = {{&S95050[44], 10}, {NULL}};
const Segment SGI240617[] = {{&S95050[44], 22}, {NULL}};
const Segment SGI240618[] = {{&S95050[30], 6}, {NULL}};
const Segment SGI240619[] = {{&S95050[113], 3}, {NULL}};
const Segment SGI240620[] = {{&S95050[80], 5}, {NULL}};
const Segment SGI240621[] = {{&S95050[117], 14}, {NULL}};
const Segment SGI240622[] = {{&S105626[45], 10}, {NULL}};
const Segment SGI240623[] = {{&S105626[30], 6}, {NULL}};
const Segment SGI240624[] = {{&S105626[31], 3}, {NULL}};
const Segment SGI240625[] = {{&S105626[81], 5}, {NULL}};
const Segment SGI240626[] = {{&S105626[139], 13}, {NULL}};
const Segment SGI240627[] = {{&S95051[44], 10}, {NULL}};
const Segment SGI240628[] = {{&S95051[44], 22}, {NULL}};
const Segment SGI240629[] = {{&S95051[30], 6}, {NULL}};
const Segment SGI240630[] = {{&S95051[80], 6}, {NULL}};
const Segment SGI240631[] = {{&S95051[107], 13}, {NULL}};
const Segment SGI240632[] = {{&S105632[44], 10}, {NULL}};
const Segment SGI240633[] = {{&S105632[44], 22}, {NULL}};
const Segment SGI240634[] = {{&S105632[30], 6}, {NULL}};
const Segment SGI240635[] = {{&S105632[111], 2}, {NULL}};
const Segment SGI240636[] = {{&S105632[80], 5}, {NULL}};
const Segment SGI240637[] = {{&S105633[44], 10}, {NULL}};
const Segment SGI240638[] = {{&S105633[44], 22}, {NULL}};
const Segment SGI240639[] = {{&S105633[30], 6}, {NULL}};
const Segment SGI240640[] = {{&S105633[31], 3}, {NULL}};
const Segment SGI240641[] = {{&S95059[44], 9}, {NULL}};
const Segment SGI240642[] = {{&S105633[80], 6}, {NULL}};
const Segment SGI240643[] = {{&S95059[44], 42}, {NULL}};
const Segment SGI240644[] = {{&S105633[143], 3}, {NULL}};
const Segment SGI240645[] = {{&S95059[30], 6}, {NULL}};
const Segment SGI240646[] = {{&S95059[135], 3}, {NULL}};
const Segment SGI240647[] = {{&S95059[100], 6}, {NULL}};
const Segment SGI240648[] = {{&S105635[157], 10}, {NULL}};
const Segment SGI240649[] = {{&S105635[39], 14}, {&S105635[157], 22}, {NULL}};
const Segment SGI240650[] = {{&S105635[143], 6}, {NULL}};
const Segment SGI240651[] = {{&S105635[144], 3}, {NULL}};
const Segment SGI240652[] = {{&S105635[193], 6}, {NULL}};
const Segment SGI240653[] = {{&S87499[44], 13}, {NULL}};
const Segment SGI240654[] = {{&S87499[44], 24}, {NULL}};
const Segment SGI240655[] = {{&S87499[30], 6}, {NULL}};
const Segment SGI240656[] = {{&S87499[117], 3}, {NULL}};
const Segment SGI240657[] = {{&S105640[44], 10}, {NULL}};
const Segment SGI240658[] = {{&S87499[82], 6}, {NULL}};
const Segment SGI240659[] = {{&S105640[44], 22}, {NULL}};
const Segment SGI240660[] = {{&S105640[30], 6}, {NULL}};
const Segment SGI240661[] = {{&S105640[31], 3}, {NULL}};
const Segment SGI240662[] = {{&S105640[87], 6}, {NULL}};
const Segment SGI240663[] = {{&S105640[66], 8}, {NULL}};
const Segment SGI240664[] = {{&S105648[59], 10}, {NULL}};
const Segment SGI240665[] = {{&S105648[59], 22}, {NULL}};
const Segment SGI240666[] = {{&S105648[45], 6}, {NULL}};
const Segment SGI240667[] = {{&S105648[46], 3}, {NULL}};
const Segment SGI240668[] = {{&S95079[44], 9}, {NULL}};
const Segment SGI240669[] = {{&S95079[44], 26}, {NULL}};
const Segment SGI240670[] = {{&S105648[95], 6}, {NULL}};
const Segment SGI240671[] = {{&S95079[30], 6}, {NULL}};
const Segment SGI240672[] = {{&S105648[0], 14}, {NULL}};
const Segment SGI240673[] = {{&S95079[84], 6}, {NULL}};
const Segment SGI240674[] = {{&S95079[111], 13}, {NULL}};
const Segment SGI240675[] = {{&S105650[44], 10}, {NULL}};
const Segment SGI240676[] = {{&S105650[44], 22}, {NULL}};
const Segment SGI240677[] = {{&S105650[30], 6}, {NULL}};
const Segment SGI240678[] = {{&S105650[31], 3}, {NULL}};
const Segment SGI240679[] = {{&S105650[87], 6}, {NULL}};
const Segment SGI240680[] = {{&S105650[66], 8}, {NULL}};
const Segment SGI240681[] = {{&S95084[34], 16}, {NULL}};
const Segment SGI240682[] = {{&S95084[27], 6}, {NULL}};
const Segment SGI240683[] = {{&S105652[45], 10}, {NULL}};
const Segment SGI240684[] = {{&S95084[103], 9}, {NULL}};
const Segment SGI240685[] = {{&S95084[76], 6}, {NULL}};
const Segment SGI240686[] = {{&S105652[30], 6}, {NULL}};
const Segment SGI240687[] = {{&S105652[31], 3}, {NULL}};
const Segment SGI240688[] = {{&S105652[81], 5}, {NULL}};
const Segment SGI240689[] = {{&S105652[139], 13}, {NULL}};
const Segment SGI240690[] = {{&S95087[34], 16}, {NULL}};
const Segment SGI240691[] = {{&S95087[27], 6}, {NULL}};
const Segment SGI240692[] = {{&S95087[103], 9}, {NULL}};
const Segment SGI240693[] = {{&S95087[76], 6}, {NULL}};
const Segment SGI240694[] = {{&S105657[59], 10}, {NULL}};
const Segment SGI240695[] = {{&S105657[59], 22}, {NULL}};
const Segment SGI240696[] = {{&S105657[45], 6}, {NULL}};
const Segment SGI240697[] = {{&S105657[46], 3}, {NULL}};
const Segment SGI240698[] = {{&S105657[95], 6}, {NULL}};
const Segment SGI240699[] = {{&S105657[0], 14}, {NULL}};
const Segment SGI240700[] = {{&S95096[52], 7}, {NULL}};
const Segment SGI240701[] = {{&S105661[59], 10}, {NULL}};
const Segment SGI240702[] = {{&S95096[52], 22}, {NULL}};
const Segment SGI240703[] = {{&S95096[38], 6}, {NULL}};
const Segment SGI240704[] = {{&S105661[59], 22}, {NULL}};
const Segment SGI240705[] = {{&S105661[45], 6}, {NULL}};
const Segment SGI240706[] = {{&S105661[46], 3}, {NULL}};
const Segment SGI240707[] = {{&S95096[88], 5}, {NULL}};
const Segment SGI240708[] = {{&S105661[95], 6}, {NULL}};
const Segment SGI240709[] = {{&S95096[11], 3}, {NULL}};
const Segment SGI240710[] = {{&S105661[0], 14}, {NULL}};
const Segment SGI240711[] = {{&S105663[58], 10}, {NULL}};
const Segment SGI240712[] = {{&S105663[58], 22}, {NULL}};
const Segment SGI240713[] = {{&S105663[44], 6}, {NULL}};
const Segment SGI240714[] = {{&S105663[45], 3}, {NULL}};
const Segment SGI240715[] = {{&S95099[34], 9}, {NULL}};
const Segment SGI240716[] = {{&S105663[94], 6}, {NULL}};
const Segment SGI240717[] = {{&S95099[27], 6}, {NULL}};
const Segment SGI240718[] = {{&S95099[96], 9}, {NULL}};
const Segment SGI240719[] = {{&S95099[69], 6}, {NULL}};
const Segment SGI240720[] = {{&S95099[130], 8}, {NULL}};
const Segment SGI240721[] = {{&S95102[34], 11}, {NULL}};
const Segment SGI240722[] = {{&S95102[27], 6}, {NULL}};
const Segment SGI240723[] = {{&S95102[98], 9}, {NULL}};
const Segment SGI240724[] = {{&S95102[71], 6}, {NULL}};
const Segment SGI240725[] = {{&S105667[44], 10}, {NULL}};
const Segment SGI240726[] = {{&S105667[44], 22}, {NULL}};
const Segment SGI240727[] = {{&S105667[30], 6}, {NULL}};
const Segment SGI240728[] = {{&S105667[31], 3}, {NULL}};
const Segment SGI240729[] = {{&S105667[80], 6}, {NULL}};
const Segment SGI240730[] = {{&S105667[143], 3}, {NULL}};
const Segment SGI240731[] = {{&S95103[34], 7}, {NULL}};
const Segment SGI240732[] = {{&S95103[27], 6}, {NULL}};
const Segment SGI240733[] = {{&S95103[104], 9}, {NULL}};
const Segment SGI240734[] = {{&S95103[77], 6}, {NULL}};
const Segment SGI240735[] = {{&S95103[145], 8}, {NULL}};
const Segment SGI240736[] = {{&S105669[59], 10}, {NULL}};
const Segment SGI240737[] = {{&S105669[59], 22}, {NULL}};
const Segment SGI240738[] = {{&S95104[44], 15}, {NULL}};
const Segment SGI240739[] = {{&S105669[45], 6}, {NULL}};
const Segment SGI240740[] = {{&S95104[44], 27}, {NULL}};
const Segment SGI240741[] = {{&S95104[30], 6}, {NULL}};
const Segment SGI240742[] = {{&S102261[44], 11}, {NULL}};
const Segment SGI240743[] = {{&S105669[46], 3}, {NULL}};
const Segment SGI240744[] = {{&S102261[44], 23}, {NULL}};
const Segment SGI240745[] = {{&S102261[30], 6}, {NULL}};
const Segment SGI240746[] = {{&S95104[120], 3}, {NULL}};
const Segment SGI240747[] = {{&S105669[114], 6}, {NULL}};
const Segment SGI240748[] = {{&S95104[85], 6}, {NULL}};
const Segment SGI240749[] = {{&S105669[83], 8}, {NULL}};
const Segment SGI240750[] = {{&S95104[140], 8}, {NULL}};
const Segment SGI240751[] = {{&S105669[0], 14}, {NULL}};
const Segment SGI240752[] = {{&S102261[145], 3}, {NULL}};
const Segment SGI240753[] = {{&S95104[149], 13}, {NULL}};
const Segment SGI240754[] = {{&S102261[110], 6}, {NULL}};
const Segment SGI240755[] = {{&S102261[69], 8}, {NULL}};
const Segment SGI240756[] = {{&S102261[81], 15}, {NULL}};
const Segment SGI240757[] = {{&S105670[45], 10}, {NULL}};
const Segment SGI240758[] = {{&S105670[30], 6}, {NULL}};
const Segment SGI240759[] = {{&S105670[31], 3}, {NULL}};
const Segment SGI240760[] = {{&S105670[81], 5}, {NULL}};
const Segment SGI240761[] = {{&S105670[139], 13}, {NULL}};
const Segment SGI240762[] = {{&S105671[57], 10}, {NULL}};
const Segment SGI240763[] = {{&S105671[57], 22}, {NULL}};
const Segment SGI240764[] = {{&S105671[43], 6}, {NULL}};
const Segment SGI240765[] = {{&S105671[44], 3}, {NULL}};
const Segment SGI240766[] = {{&S105671[93], 6}, {NULL}};
const Segment SGI240767[] = {{&S105671[0], 12}, {NULL}};
const Segment SGI240768[] = {{&S102268[34], 21}, {NULL}};
const Segment SGI240769[] = {{&S102268[27], 6}, {NULL}};
const Segment SGI240770[] = {{&S102268[108], 9}, {NULL}};
const Segment SGI240771[] = {{&S102268[81], 6}, {NULL}};
const Segment SGI240772[] = {{&S105682[62], 10}, {NULL}};
const Segment SGI240773[] = {{&S105682[55], 6}, {NULL}};
const Segment SGI240774[] = {{&S105686[31], 8}, {NULL}};
const Segment SGI240775[] = {{&S105686[55], 3}, {NULL}};
const Segment SGI240776[] = {{&S95889[44], 7}, {NULL}};
const Segment SGI240777[] = {{&S95889[44], 19}, {NULL}};
const Segment SGI240778[] = {{&S95889[30], 6}, {NULL}};
const Segment SGI240779[] = {{&S95889[77], 6}, {NULL}};
const Segment SGI240780[] = {{&S95889[114], 3}, {NULL}};
const Segment SGI240781[] = {{&S105692[31], 8}, {NULL}};
const Segment SGI240782[] = {{&S105692[55], 3}, {NULL}};
const Segment SGI240783[] = {{&S105695[44], 10}, {NULL}};
const Segment SGI240784[] = {{&S105695[44], 22}, {NULL}};
const Segment SGI240785[] = {{&S105695[30], 6}, {NULL}};
const Segment SGI240786[] = {{&S105695[139], 4}, {NULL}};
const Segment SGI240787[] = {{&S105695[31], 3}, {&S105695[192], 5}, {NULL}};
const Segment SGI240788[] = {{&S105695[80], 6}, {NULL}};
const Segment SGI240789[] = {{&S102299[59], 10}, {NULL}};
const Segment SGI240790[] = {{&S102299[59], 22}, {NULL}};
const Segment SGI240791[] = {{&S102299[45], 6}, {NULL}};
const Segment SGI240792[] = {{&S102299[130], 3}, {NULL}};
const Segment SGI240793[] = {{&S102299[95], 6}, {NULL}};
const Segment SGI240794[] = {{&S102299[0], 14}, {NULL}};
const Segment SGI240795[] = {{&S102302[44], 10}, {NULL}};
const Segment SGI240796[] = {{&S102302[44], 22}, {NULL}};
const Segment SGI240797[] = {{&S102302[30], 6}, {NULL}};
const Segment SGI240798[] = {{&S102302[115], 3}, {NULL}};
const Segment SGI240799[] = {{&S102302[80], 6}, {NULL}};
const Segment SGI240800[] = {{&S102302[133], 11}, {NULL}};
const Segment SGI240801[] = {{&S105702[44], 10}, {NULL}};
const Segment SGI240802[] = {{&S105702[44], 22}, {NULL}};
const Segment SGI240803[] = {{&S105702[30], 6}, {NULL}};
const Segment SGI240804[] = {{&S105702[31], 3}, {NULL}};
const Segment SGI240805[] = {{&S105702[80], 6}, {NULL}};
const Segment SGI240806[] = {{&S105702[143], 3}, {NULL}};
const Segment SGI240807[] = {{&S102304[44], 10}, {NULL}};
const Segment SGI240808[] = {{&S102304[44], 22}, {NULL}};
const Segment SGI240809[] = {{&S102304[30], 6}, {NULL}};
const Segment SGI240810[] = {{&S102304[115], 3}, {NULL}};
const Segment SGI240811[] = {{&S102304[80], 6}, {NULL}};
const Segment SGI240812[] = {{&S102304[157], 3}, {NULL}};
const Segment SGI240813[] = {{&S102305[44], 10}, {NULL}};
const Segment SGI240814[] = {{&S105705[44], 10}, {NULL}};
const Segment SGI240815[] = {{&S102305[44], 22}, {NULL}};
const Segment SGI240816[] = {{&S105705[44], 22}, {NULL}};
const Segment SGI240817[] = {{&S102305[30], 6}, {NULL}};
const Segment SGI240818[] = {{&S105705[30], 6}, {NULL}};
const Segment SGI240819[] = {{&S105705[159], 3}, {NULL}};
const Segment SGI240820[] = {{&S102305[115], 3}, {NULL}};
const Segment SGI240821[] = {{&S102305[80], 6}, {NULL}};
const Segment SGI240822[] = {{&S105705[125], 3}, {NULL}};
const Segment SGI240823[] = {{&S102305[133], 13}, {NULL}};
const Segment SGI240824[] = {{&S105705[90], 6}, {NULL}};
const Segment SGI240825[] = {{&S105705[145], 8}, {NULL}};
const Segment SGI240826[] = {{&S95919[44], 7}, {NULL}};
const Segment SGI240827[] = {{&S95919[44], 19}, {NULL}};
const Segment SGI240828[] = {{&S95919[30], 6}, {NULL}};
const Segment SGI240829[] = {{&S105711[38], 10}, {NULL}};
const Segment SGI240830[] = {{&S105711[31], 6}, {NULL}};
const Segment SGI240831[] = {{&S95919[77], 6}, {NULL}};
const Segment SGI240832[] = {{&S95919[114], 3}, {NULL}};
const Segment SGI240833[] = {{&S95922[34], 21}, {NULL}};
const Segment SGI240834[] = {{&S95922[27], 6}, {NULL}};
const Segment SGI240835[] = {{&S95922[108], 9}, {NULL}};
const Segment SGI240836[] = {{&S95922[81], 6}, {NULL}};
const Segment SGI240837[] = {{&S105718[44], 10}, {NULL}};
const Segment SGI240838[] = {{&S105718[44], 22}, {NULL}};
const Segment SGI240839[] = {{&S105718[30], 6}, {NULL}};
const Segment SGI240840[] = {{&S105718[31], 3}, {NULL}};
const Segment SGI240841[] = {{&S105718[95], 6}, {NULL}};
const Segment SGI240842[] = {{&S105718[165], 3}, {NULL}};
const Segment SGI240843[] = {{&S105718[68], 8}, {NULL}};
const Segment SGI240844[] = {{&S105721[44], 10}, {NULL}};
const Segment SGI240845[] = {{&S105721[44], 22}, {NULL}};
const Segment SGI240846[] = {{&S105721[30], 6}, {NULL}};
const Segment SGI240847[] = {{&S105721[31], 3}, {NULL}};
const Segment SGI240848[] = {{&S105721[111], 6}, {NULL}};
const Segment SGI240849[] = {{&S105721[174], 3}, {NULL}};
const Segment SGI240850[] = {{&S105721[68], 8}, {NULL}};
const Segment SGI240851[] = {{&S105723[59], 10}, {NULL}};
const Segment SGI240852[] = {{&S105723[59], 22}, {NULL}};
const Segment SGI240853[] = {{&S105723[45], 6}, {NULL}};
const Segment SGI240854[] = {{&S105723[46], 3}, {NULL}};
const Segment SGI240855[] = {{&S105723[95], 6}, {NULL}};
const Segment SGI240856[] = {{&S105723[0], 14}, {NULL}};
const Segment SGI240857[] = {{&S105726[59], 10}, {NULL}};
const Segment SGI240858[] = {{&S105726[59], 22}, {NULL}};
const Segment SGI240859[] = {{&S105726[45], 6}, {NULL}};
const Segment SGI240860[] = {{&S105726[46], 3}, {NULL}};
const Segment SGI240861[] = {{&S105726[95], 6}, {NULL}};
const Segment SGI240862[] = {{&S105726[0], 14}, {NULL}};
const Segment SGI240863[] = {{&S105730[44], 10}, {NULL}};
const Segment SGI240864[] = {{&S105730[44], 22}, {NULL}};
const Segment SGI240865[] = {{&S105730[30], 6}, {NULL}};
const Segment SGI240866[] = {{&S105730[31], 3}, {NULL}};
const Segment SGI240867[] = {{&S105730[80], 5}, {NULL}};
const Segment SGI240868[] = {{&S105730[125], 14}, {NULL}};
const Segment SGI240869[] = {{&S105734[59], 10}, {NULL}};
const Segment SGI240870[] = {{&S105734[59], 22}, {NULL}};
const Segment SGI240871[] = {{&S105734[45], 6}, {NULL}};
const Segment SGI240872[] = {{&S105734[46], 3}, {NULL}};
const Segment SGI240873[] = {{&S105734[95], 6}, {NULL}};
const Segment SGI240874[] = {{&S105734[0], 14}, {NULL}};
const Segment SGI240875[] = {{&S87510[44], 7}, {NULL}};
const Segment SGI240876[] = {{&S87510[44], 18}, {NULL}};
const Segment SGI240877[] = {{&S87510[30], 6}, {NULL}};
const Segment SGI240878[] = {{&S87510[111], 3}, {NULL}};
const Segment SGI240879[] = {{&S105735[59], 10}, {NULL}};
const Segment SGI240880[] = {{&S87510[76], 6}, {NULL}};
const Segment SGI240881[] = {{&S105735[59], 27}, {NULL}};
const Segment SGI240882[] = {{&S105735[45], 6}, {NULL}};
const Segment SGI240883[] = {{&S105735[46], 3}, {NULL}};
const Segment SGI240884[] = {{&S105735[100], 6}, {NULL}};
const Segment SGI240885[] = {{&S105735[0], 14}, {NULL}};
const Segment SGI240886[] = {{&S87511[44], 7}, {NULL}};
const Segment SGI240887[] = {{&S87511[44], 19}, {NULL}};
const Segment SGI240888[] = {{&S87511[30], 6}, {NULL}};
const Segment SGI240889[] = {{&S87511[112], 3}, {NULL}};
const Segment SGI240890[] = {{&S87511[77], 6}, {NULL}};
const Segment SGI240891[] = {{&S105736[44], 10}, {NULL}};
const Segment SGI240892[] = {{&S105736[44], 22}, {NULL}};
const Segment SGI240893[] = {{&S105736[30], 6}, {NULL}};
const Segment SGI240894[] = {{&S105736[31], 3}, {NULL}};
const Segment SGI240895[] = {{&S105736[80], 6}, {NULL}};
const Segment SGI240896[] = {{&S105736[133], 11}, {NULL}};
const Segment SGI240897[] = {{&S87514[34], 10}, {NULL}};
const Segment SGI240898[] = {{&S87514[27], 6}, {NULL}};
const Segment SGI240899[] = {{&S87514[96], 9}, {NULL}};
const Segment SGI240900[] = {{&S87514[69], 6}, {NULL}};
const Segment SGI240901[] = {{&S87516[34], 12}, {NULL}};
const Segment SGI240902[] = {{&S87516[27], 6}, {NULL}};
const Segment SGI240903[] = {{&S87516[98], 9}, {NULL}};
const Segment SGI240904[] = {{&S87516[71], 6}, {NULL}};
const Segment SGI240905[] = {{&S87516[139], 8}, {NULL}};
const Segment SGI240906[] = {{&S105742[45], 10}, {NULL}};
const Segment SGI240907[] = {{&S105742[30], 6}, {NULL}};
const Segment SGI240908[] = {{&S105742[31], 3}, {NULL}};
const Segment SGI240909[] = {{&S105742[81], 5}, {NULL}};
const Segment SGI240910[] = {{&S105742[139], 13}, {NULL}};
const Segment SGI240911[] = {{&S105754[44], 10}, {NULL}};
const Segment SGI240912[] = {{&S105754[44], 22}, {NULL}};
const Segment SGI240913[] = {{&S105754[30], 6}, {NULL}};
const Segment SGI240914[] = {{&S105754[31], 3}, {NULL}};
const Segment SGI240915[] = {{&S105754[80], 6}, {NULL}};
const Segment SGI240916[] = {{&S105754[133], 13}, {NULL}};
const Segment SGI240917[] = {{&S105755[62], 10}, {NULL}};
const Segment SGI240918[] = {{&S105755[55], 6}, {NULL}};
const Segment SGI240919[] = {{&S105756[44], 10}, {NULL}};
const Segment SGI240920[] = {{&S105756[44], 22}, {NULL}};
const Segment SGI240921[] = {{&S105756[30], 6}, {NULL}};
const Segment SGI240922[] = {{&S105756[80], 6}, {NULL}};
const Segment SGI240923[] = {{&S105756[107], 13}, {NULL}};
const Segment SGI240924[] = {{&S105757[44], 10}, {NULL}};
const Segment SGI240925[] = {{&S105757[44], 22}, {NULL}};
const Segment SGI240926[] = {{&S105757[30], 6}, {NULL}};
const Segment SGI240927[] = {{&S105757[31], 3}, {NULL}};
const Segment SGI240928[] = {{&S105757[99], 6}, {NULL}};
const Segment SGI240929[] = {{&S105757[162], 3}, {NULL}};
const Segment SGI240930[] = {{&S105757[68], 8}, {NULL}};
const Segment SGI240931[] = {{&S105758[59], 10}, {NULL}};
const Segment SGI240932[] = {{&S105758[59], 22}, {NULL}};
const Segment SGI240933[] = {{&S105758[45], 6}, {NULL}};
const Segment SGI240934[] = {{&S105758[46], 3}, {NULL}};
const Segment SGI240935[] = {{&S105758[95], 6}, {NULL}};
const Segment SGI240936[] = {{&S105758[0], 14}, {NULL}};
const Segment SGI240937[] = {{&S105759[59], 10}, {NULL}};
const Segment SGI240938[] = {{&S105759[59], 22}, {NULL}};
const Segment SGI240939[] = {{&S105759[45], 6}, {NULL}};
const Segment SGI240940[] = {{&S105759[46], 3}, {NULL}};
const Segment SGI240941[] = {{&S105759[95], 6}, {NULL}};
const Segment SGI240942[] = {{&S105759[0], 14}, {NULL}};
const Segment SGI240943[] = {{&S105760[44], 10}, {NULL}};
const Segment SGI240944[] = {{&S105760[44], 22}, {NULL}};
const Segment SGI240945[] = {{&S105760[30], 6}, {NULL}};
const Segment SGI240946[] = {{&S105760[31], 3}, {NULL}};
const Segment SGI240947[] = {{&S105760[80], 5}, {NULL}};
const Segment SGI240948[] = {{&S105760[117], 14}, {NULL}};
const Segment SGI240949[] = {{&S105765[44], 10}, {NULL}};
const Segment SGI240950[] = {{&S105765[44], 22}, {NULL}};
const Segment SGI240951[] = {{&S105765[30], 6}, {NULL}};
const Segment SGI240952[] = {{&S105765[31], 3}, {NULL}};
const Segment SGI240953[] = {{&S105765[99], 6}, {NULL}};
const Segment SGI240954[] = {{&S105765[152], 11}, {NULL}};
const Segment SGI240955[] = {{&S105765[68], 8}, {NULL}};
const Segment SGI240956[] = {{&S105770[44], 10}, {NULL}};
const Segment SGI240957[] = {{&S105770[44], 22}, {NULL}};
const Segment SGI240958[] = {{&S105770[30], 6}, {NULL}};
const Segment SGI240959[] = {{&S105770[80], 6}, {NULL}};
const Segment SGI240960[] = {{&S105770[107], 13}, {NULL}};
const Segment SGI240961[] = {{&S105771[44], 10}, {NULL}};
const Segment SGI240962[] = {{&S105771[44], 22}, {NULL}};
const Segment SGI240963[] = {{&S105771[30], 6}, {NULL}};
const Segment SGI240964[] = {{&S105771[31], 3}, {NULL}};
const Segment SGI240965[] = {{&S105771[87], 6}, {NULL}};
const Segment SGI240966[] = {{&S105771[66], 8}, {NULL}};
const Segment SGI240967[] = {{&S105782[59], 10}, {NULL}};
const Segment SGI240968[] = {{&S105782[59], 22}, {NULL}};
const Segment SGI240969[] = {{&S105782[45], 6}, {NULL}};
const Segment SGI240970[] = {{&S105782[46], 3}, {NULL}};
const Segment SGI240971[] = {{&S105782[95], 6}, {NULL}};
const Segment SGI240972[] = {{&S105782[0], 14}, {NULL}};
const Segment SGI240973[] = {{&S105793[44], 10}, {NULL}};
const Segment SGI240974[] = {{&S105793[44], 22}, {NULL}};
const Segment SGI240975[] = {{&S105793[30], 6}, {NULL}};
const Segment SGI240976[] = {{&S105793[31], 3}, {NULL}};
const Segment SGI240977[] = {{&S105793[80], 6}, {NULL}};
const Segment SGI240978[] = {{&S105793[133], 11}, {NULL}};
const Segment SGI240979[] = {{&S105797[44], 10}, {NULL}};
const Segment SGI240980[] = {{&S105797[44], 22}, {NULL}};
const Segment SGI240981[] = {{&S105797[30], 6}, {NULL}};
const Segment SGI240982[] = {{&S105797[31], 3}, {NULL}};
const Segment SGI240983[] = {{&S105797[80], 6}, {NULL}};
const Segment SGI240984[] = {{&S105797[133], 13}, {NULL}};
const Segment SGI240985[] = {{&S105800[44], 10}, {NULL}};
const Segment SGI240986[] = {{&S105800[44], 22}, {NULL}};
const Segment SGI240987[] = {{&S105800[30], 6}, {NULL}};
const Segment SGI240988[] = {{&S105800[31], 3}, {NULL}};
const Segment SGI240989[] = {{&S105800[80], 5}, {NULL}};
const Segment SGI240990[] = {{&S105800[117], 14}, {NULL}};
const Segment SGI240991[] = {{&S105805[44], 10}, {NULL}};
const Segment SGI240992[] = {{&S105805[44], 22}, {NULL}};
const Segment SGI240993[] = {{&S105805[30], 6}, {NULL}};
const Segment SGI240994[] = {{&S105805[31], 3}, {NULL}};
const Segment SGI240995[] = {{&S105805[80], 6}, {NULL}};
const Segment SGI240996[] = {{&S105805[133], 13}, {NULL}};
const Segment SGI240997[] = {{&S87619[34], 11}, {NULL}};
const Segment SGI240998[] = {{&S87619[27], 6}, {NULL}};
const Segment SGI240999[] = {{&S87619[97], 9}, {NULL}};
const Segment SGI241000[] = {{&S87619[70], 6}, {NULL}};
const Segment SGI241001[] = {{&S96068[38], 10}, {NULL}};
const Segment SGI241002[] = {{&S96068[31], 6}, {NULL}};
const Segment SGI241003[] = {{&S105813[59], 10}, {NULL}};
const Segment SGI241004[] = {{&S105813[59], 22}, {NULL}};
const Segment SGI241005[] = {{&S105813[45], 6}, {NULL}};
const Segment SGI241006[] = {{&S105813[46], 3}, {NULL}};
const Segment SGI241007[] = {{&S105813[95], 6}, {NULL}};
const Segment SGI241008[] = {{&S95209[34], 10}, {NULL}};
const Segment SGI241009[] = {{&S105813[0], 14}, {NULL}};
const Segment SGI241010[] = {{&S95209[27], 6}, {NULL}};
const Segment SGI241011[] = {{&S95209[97], 9}, {NULL}};
const Segment SGI241012[] = {{&S95209[70], 6}, {NULL}};
const Segment SGI241013[] = {{&S105818[44], 10}, {NULL}};
const Segment SGI241014[] = {{&S105818[44], 22}, {NULL}};
const Segment SGI241015[] = {{&S95215[44], 7}, {NULL}};
const Segment SGI241016[] = {{&S105818[30], 6}, {NULL}};
const Segment SGI241017[] = {{&S95215[44], 15}, {NULL}};
const Segment SGI241018[] = {{&S105818[159], 3}, {NULL}};
const Segment SGI241019[] = {{&S95215[30], 6}, {NULL}};
const Segment SGI241020[] = {{&S105818[125], 3}, {NULL}};
const Segment SGI241021[] = {{&S95215[108], 3}, {NULL}};
const Segment SGI241022[] = {{&S95215[73], 6}, {NULL}};
const Segment SGI241023[] = {{&S105818[90], 6}, {NULL}};
const Segment SGI241024[] = {{&S105818[145], 8}, {NULL}};
const Segment SGI241025[] = {{&S105821[44], 10}, {NULL}};
const Segment SGI241026[] = {{&S105821[44], 22}, {NULL}};
const Segment SGI241027[] = {{&S105821[30], 6}, {NULL}};
const Segment SGI241028[] = {{&S105821[31], 3}, {NULL}};
const Segment SGI241029[] = {{&S105821[80], 6}, {NULL}};
const Segment SGI241030[] = {{&S105821[133], 11}, {NULL}};
const Segment SGI241031[] = {{&S87635[34], 8}, {NULL}};
const Segment SGI241032[] = {{&S87635[27], 6}, {NULL}};
const Segment SGI241033[] = {{&S87635[94], 9}, {NULL}};
const Segment SGI241034[] = {{&S87635[67], 6}, {NULL}};
const Segment SGI241035[] = {{&S87635[128], 8}, {NULL}};
const Segment SGI241036[] = {{&S105822[44], 10}, {NULL}};
const Segment SGI241037[] = {{&S105822[44], 22}, {NULL}};
const Segment SGI241038[] = {{&S105822[30], 6}, {NULL}};
const Segment SGI241039[] = {{&S105822[159], 3}, {NULL}};
const Segment SGI241040[] = {{&S105822[125], 3}, {NULL}};
const Segment SGI241041[] = {{&S105822[90], 6}, {NULL}};
const Segment SGI241042[] = {{&S105822[145], 8}, {NULL}};
const Segment SGI241043[] = {{&S87638[44], 10}, {NULL}};
const Segment SGI241044[] = {{&S105824[44], 10}, {NULL}};
const Segment SGI241045[] = {{&S87638[44], 22}, {NULL}};
const Segment SGI241046[] = {{&S105824[44], 22}, {NULL}};
const Segment SGI241047[] = {{&S87638[30], 6}, {NULL}};
const Segment SGI241048[] = {{&S105824[30], 6}, {NULL}};
const Segment SGI241049[] = {{&S95226[34], 10}, {NULL}};
const Segment SGI241050[] = {{&S87638[115], 3}, {NULL}};
const Segment SGI241051[] = {{&S95226[27], 6}, {NULL}};
const Segment SGI241052[] = {{&S105824[80], 6}, {NULL}};
const Segment SGI241053[] = {{&S105824[107], 13}, {NULL}};
const Segment SGI241054[] = {{&S95226[97], 9}, {NULL}};
const Segment SGI241055[] = {{&S95226[70], 6}, {NULL}};
const Segment SGI241056[] = {{&S105825[44], 10}, {NULL}};
const Segment SGI241057[] = {{&S105825[44], 22}, {NULL}};
const Segment SGI241058[] = {{&S105825[30], 6}, {NULL}};
const Segment SGI241059[] = {{&S105825[31], 3}, {NULL}};
const Segment SGI241060[] = {{&S105825[99], 6}, {NULL}};
const Segment SGI241061[] = {{&S105825[152], 13}, {NULL}};
const Segment SGI241062[] = {{&S105825[68], 8}, {NULL}};
const Segment SGI241063[] = {{&S105830[44], 10}, {NULL}};
const Segment SGI241064[] = {{&S105830[44], 22}, {NULL}};
const Segment SGI241065[] = {{&S105830[30], 6}, {NULL}};
const Segment SGI241066[] = {{&S105830[31], 3}, {NULL}};
const Segment SGI241067[] = {{&S105830[80], 6}, {NULL}};
const Segment SGI241068[] = {{&S105830[133], 13}, {NULL}};
const Segment SGI241069[] = {{&S105831[44], 10}, {NULL}};
const Segment SGI241070[] = {{&S105831[44], 22}, {NULL}};
const Segment SGI241071[] = {{&S105831[30], 6}, {NULL}};
const Segment SGI241072[] = {{&S105831[31], 3}, {NULL}};
const Segment SGI241073[] = {{&S105831[80], 5}, {NULL}};
const Segment SGI241074[] = {{&S105831[117], 14}, {NULL}};
const Segment SGI241075[] = {{&S95238[34], 5}, {NULL}};
const Segment SGI241076[] = {{&S95238[27], 6}, {NULL}};
const Segment SGI241077[] = {{&S95238[92], 9}, {NULL}};
const Segment SGI241078[] = {{&S95238[65], 6}, {NULL}};
const Segment SGI241079[] = {{&S102516[44], 10}, {NULL}};
const Segment SGI241080[] = {{&S102516[44], 22}, {NULL}};
const Segment SGI241081[] = {{&S102516[30], 6}, {NULL}};
const Segment SGI241082[] = {{&S102516[131], 3}, {NULL}};
const Segment SGI241083[] = {{&S102516[96], 6}, {NULL}};
const Segment SGI241084[] = {{&S102516[159], 3}, {NULL}};
const Segment SGI241085[] = {{&S102516[68], 8}, {NULL}};
const Segment SGI241086[] = {{&S105835[34], 10}, {NULL}};
const Segment SGI241087[] = {{&S105835[27], 6}, {NULL}};
const Segment SGI241088[] = {{&S95241[44], 16}, {NULL}};
const Segment SGI241089[] = {{&S95241[44], 43}, {NULL}};
const Segment SGI241090[] = {{&S95241[30], 6}, {NULL}};
const Segment SGI241091[] = {{&S105835[97], 9}, {NULL}};
const Segment SGI241092[] = {{&S105835[70], 6}, {NULL}};
const Segment SGI241093[] = {{&S95241[136], 3}, {NULL}};
const Segment SGI241094[] = {{&S95241[101], 6}, {NULL}};
const Segment SGI241095[] = {{&S102524[59], 10}, {NULL}};
const Segment SGI241096[] = {{&S102524[59], 22}, {NULL}};
const Segment SGI241097[] = {{&S102524[45], 6}, {NULL}};
const Segment SGI241098[] = {{&S102524[130], 3}, {NULL}};
const Segment SGI241099[] = {{&S102524[95], 6}, {NULL}};
const Segment SGI241100[] = {{&S102524[0], 14}, {NULL}};
const Segment SGI241101[] = {{&S105845[45], 10}, {NULL}};
const Segment SGI241102[] = {{&S102526[44], 10}, {NULL}};
const Segment SGI241103[] = {{&S102526[44], 22}, {NULL}};
const Segment SGI241104[] = {{&S105845[30], 6}, {NULL}};
const Segment SGI241105[] = {{&S102526[30], 6}, {NULL}};
const Segment SGI241106[] = {{&S95252[44], 10}, {NULL}};
const Segment SGI241107[] = {{&S95252[44], 22}, {NULL}};
const Segment SGI241108[] = {{&S95252[30], 6}, {NULL}};
const Segment SGI241109[] = {{&S102526[113], 3}, {NULL}};
const Segment SGI241110[] = {{&S105845[31], 3}, {NULL}};
const Segment SGI241111[] = {{&S102526[80], 5}, {NULL}};
const Segment SGI241112[] = {{&S105845[81], 5}, {NULL}};
const Segment SGI241113[] = {{&S95252[115], 3}, {NULL}};
const Segment SGI241114[] = {{&S105845[139], 13}, {NULL}};
const Segment SGI241115[] = {{&S102526[117], 14}, {NULL}};
const Segment SGI241116[] = {{&S95252[80], 6}, {NULL}};
const Segment SGI241117[] = {{&S102527[44], 11}, {NULL}};
const Segment SGI241118[] = {{&S105846[45], 10}, {NULL}};
const Segment SGI241119[] = {{&S102527[44], 23}, {NULL}};
const Segment SGI241120[] = {{&S102527[30], 6}, {NULL}};
const Segment SGI241121[] = {{&S105846[30], 6}, {NULL}};
const Segment SGI241122[] = {{&S102527[114], 3}, {NULL}};
const Segment SGI241123[] = {{&S105846[31], 3}, {NULL}};
const Segment SGI241124[] = {{&S102527[81], 5}, {NULL}};
const Segment SGI241125[] = {{&S105846[81], 5}, {NULL}};
const Segment SGI241126[] = {{&S102527[118], 14}, {NULL}};
const Segment SGI241127[] = {{&S105846[139], 13}, {NULL}};
const Segment SGI241128[] = {{&S95255[34], 13}, {NULL}};
const Segment SGI241129[] = {{&S95255[27], 6}, {NULL}};
const Segment SGI241130[] = {{&S95255[100], 9}, {NULL}};
const Segment SGI241131[] = {{&S95255[73], 6}, {NULL}};
const Segment SGI241132[] = {{&S95256[34], 10}, {NULL}};
const Segment SGI241133[] = {{&S95256[27], 6}, {NULL}};
const Segment SGI241134[] = {{&S95256[97], 9}, {NULL}};
const Segment SGI241135[] = {{&S95256[70], 6}, {NULL}};
const Segment SGI241136[] = {{&S95262[44], 15}, {NULL}};
const Segment SGI241137[] = {{&S95262[44], 27}, {NULL}};
const Segment SGI241138[] = {{&S95262[30], 6}, {NULL}};
const Segment SGI241139[] = {{&S95262[136], 3}, {NULL}};
const Segment SGI241140[] = {{&S95262[101], 6}, {NULL}};
const Segment SGI241141[] = {{&S95262[73], 8}, {NULL}};
const Segment SGI241142[] = {{&S105859[44], 10}, {NULL}};
const Segment SGI241143[] = {{&S105859[44], 22}, {NULL}};
const Segment SGI241144[] = {{&S105859[30], 6}, {NULL}};
const Segment SGI241145[] = {{&S105859[31], 3}, {NULL}};
const Segment SGI241146[] = {{&S105859[80], 6}, {NULL}};
const Segment SGI241147[] = {{&S105859[143], 3}, {NULL}};
const Segment SGI241148[] = {{&S105871[44], 11}, {NULL}};
const Segment SGI241149[] = {{&S105871[44], 23}, {NULL}};
const Segment SGI241150[] = {{&S105871[30], 6}, {NULL}};
const Segment SGI241151[] = {{&S105871[31], 3}, {NULL}};
const Segment SGI241152[] = {{&S105871[81], 6}, {NULL}};
const Segment SGI241153[] = {{&S105871[134], 11}, {NULL}};
const Segment SGI241154[] = {{&S95281[44], 10}, {NULL}};
const Segment SGI241155[] = {{&S95281[44], 22}, {NULL}};
const Segment SGI241156[] = {{&S95281[30], 6}, {NULL}};
const Segment SGI241157[] = {{&S95281[115], 3}, {NULL}};
const Segment SGI241158[] = {{&S95281[80], 6}, {NULL}};
const Segment SGI241159[] = {{&S105872[44], 10}, {NULL}};
const Segment SGI241160[] = {{&S95281[147], 15}, {NULL}};
const Segment SGI241161[] = {{&S105872[44], 22}, {NULL}};
const Segment SGI241162[] = {{&S105872[30], 6}, {NULL}};
const Segment SGI241163[] = {{&S105872[31], 3}, {NULL}};
const Segment SGI241164[] = {{&S105872[80], 6}, {NULL}};
const Segment SGI241165[] = {{&S105872[132], 10}, {NULL}};
const Segment SGI241166[] = {{&S95289[34], 10}, {NULL}};
const Segment SGI241167[] = {{&S95289[27], 6}, {NULL}};
const Segment SGI241168[] = {{&S95289[97], 9}, {NULL}};
const Segment SGI241169[] = {{&S95289[70], 6}, {NULL}};
const Segment SGI241170[] = {{&S105880[59], 10}, {NULL}};
const Segment SGI241171[] = {{&S105880[59], 22}, {NULL}};
const Segment SGI241172[] = {{&S105880[45], 6}, {NULL}};
const Segment SGI241173[] = {{&S105880[46], 3}, {NULL}};
const Segment SGI241174[] = {{&S105880[114], 6}, {NULL}};
const Segment SGI241175[] = {{&S105880[83], 8}, {NULL}};
const Segment SGI241176[] = {{&S105880[0], 14}, {NULL}};
const Segment SGI241177[] = {{&S95293[34], 10}, {NULL}};
const Segment SGI241178[] = {{&S95293[27], 6}, {NULL}};
const Segment SGI241179[] = {{&S95293[97], 9}, {NULL}};
const Segment SGI241180[] = {{&S95293[70], 6}, {NULL}};
const Segment SGI241181[] = {{&S95294[34], 10}, {NULL}};
const Segment SGI241182[] = {{&S95294[27], 6}, {NULL}};
const Segment SGI241183[] = {{&S95294[97], 9}, {NULL}};
const Segment SGI241184[] = {{&S95294[70], 6}, {NULL}};
const Segment SGI241185[] = {{&S95294[138], 8}, {NULL}};
const Segment SGI241186[] = {{&S105885[44], 10}, {NULL}};
const Segment SGI241187[] = {{&S105885[44], 22}, {NULL}};
const Segment SGI241188[] = {{&S105885[30], 6}, {NULL}};
const Segment SGI241189[] = {{&S105885[31], 3}, {NULL}};
const Segment SGI241190[] = {{&S105885[80], 6}, {NULL}};
const Segment SGI241191[] = {{&S105885[143], 3}, {NULL}};
const Segment SGI241192[] = {{&S105887[44], 10}, {NULL}};
const Segment SGI241193[] = {{&S105887[44], 22}, {NULL}};
const Segment SGI241194[] = {{&S105887[30], 6}, {NULL}};
const Segment SGI241195[] = {{&S105887[31], 3}, {NULL}};
const Segment SGI241196[] = {{&S105887[80], 6}, {NULL}};
const Segment SGI241197[] = {{&S105887[157], 3}, {NULL}};
const Segment SGI241198[] = {{&S105891[44], 10}, {NULL}};
const Segment SGI241199[] = {{&S105891[44], 22}, {NULL}};
const Segment SGI241200[] = {{&S105891[30], 6}, {NULL}};
const Segment SGI241201[] = {{&S105891[115], 3}, {NULL}};
const Segment SGI241202[] = {{&S105891[80], 6}, {NULL}};
const Segment SGI241203[] = {{&S105891[135], 8}, {NULL}};
const Segment SGI241204[] = {{&S105891[144], 13}, {NULL}};
const Segment SGI241205[] = {{&S105892[59], 10}, {NULL}};
const Segment SGI241206[] = {{&S105892[59], 22}, {NULL}};
const Segment SGI241207[] = {{&S105892[45], 6}, {NULL}};
const Segment SGI241208[] = {{&S95307[44], 10}, {NULL}};
const Segment SGI241209[] = {{&S105892[46], 3}, {NULL}};
const Segment SGI241210[] = {{&S95307[44], 22}, {NULL}};
const Segment SGI241211[] = {{&S95307[30], 6}, {NULL}};
const Segment SGI241212[] = {{&S105892[95], 6}, {NULL}};
const Segment SGI241213[] = {{&S105892[0], 14}, {NULL}};
const Segment SGI241214[] = {{&S95307[115], 3}, {NULL}};
const Segment SGI241215[] = {{&S95307[80], 6}, {NULL}};
const Segment SGI241216[] = {{&S95307[135], 8}, {NULL}};
const Segment SGI241217[] = {{&S95307[144], 13}, {NULL}};
const Segment SGI241218[] = {{&S95309[44], 7}, {NULL}};
const Segment SGI241219[] = {{&S95309[44], 24}, {NULL}};
const Segment SGI241220[] = {{&S95309[30], 6}, {NULL}};
const Segment SGI241221[] = {{&S95309[82], 6}, {NULL}};
const Segment SGI241222[] = {{&S95309[109], 13}, {NULL}};
const Segment SGI241223[] = {{&S96183[44], 10}, {NULL}};
const Segment SGI241224[] = {{&S96183[44], 22}, {NULL}};
const Segment SGI241225[] = {{&S96183[30], 6}, {NULL}};
const Segment SGI241226[] = {{&S96183[115], 3}, {NULL}};
const Segment SGI241227[] = {{&S96183[80], 6}, {NULL}};
const Segment SGI241228[] = {{&S105898[44], 10}, {NULL}};
const Segment SGI241229[] = {{&S105898[44], 22}, {NULL}};
const Segment SGI241230[] = {{&S105898[30], 6}, {NULL}};
const Segment SGI241231[] = {{&S95312[59], 13}, {NULL}};
const Segment SGI241232[] = {{&S95312[59], 25}, {NULL}};
const Segment SGI241233[] = {{&S95312[45], 6}, {NULL}};
const Segment SGI241234[] = {{&S105898[31], 3}, {NULL}};
const Segment SGI241235[] = {{&S105898[80], 6}, {NULL}};
const Segment SGI241236[] = {{&S95312[133], 3}, {NULL}};
const Segment SGI241237[] = {{&S105898[133], 11}, {NULL}};
const Segment SGI241238[] = {{&S95312[98], 6}, {NULL}};
const Segment SGI241239[] = {{&S95312[0], 14}, {NULL}};
const Segment SGI241240[] = {{&S105902[34], 15}, {NULL}};
const Segment SGI241241[] = {{&S105902[27], 6}, {NULL}};
const Segment SGI241242[] = {{&S105902[102], 9}, {NULL}};
const Segment SGI241243[] = {{&S105902[75], 6}, {NULL}};
const Segment SGI241244[] = {{&S95319[44], 10}, {NULL}};
const Segment SGI241245[] = {{&S95319[44], 22}, {NULL}};
const Segment SGI241246[] = {{&S95319[30], 6}, {NULL}};
const Segment SGI241247[] = {{&S95319[115], 3}, {NULL}};
const Segment SGI241248[] = {{&S95319[80], 6}, {NULL}};
const Segment SGI241249[] = {{&S95319[140], 7}, {NULL}};
const Segment SGI241250[] = {{&S105911[44], 15}, {NULL}};
const Segment SGI241251[] = {{&S105911[44], 27}, {NULL}};
const Segment SGI241252[] = {{&S105911[30], 6}, {NULL}};
const Segment SGI241253[] = {{&S105911[31], 3}, {NULL}};
const Segment SGI241254[] = {{&S105911[85], 6}, {NULL}};
const Segment SGI241255[] = {{&S105911[145], 13}, {NULL}};
const Segment SGI241256[] = {{&S105912[44], 19}, {NULL}};
const Segment SGI241257[] = {{&S105912[30], 6}, {NULL}};
const Segment SGI241258[] = {{&S105912[31], 3}, {NULL}};
const Segment SGI241259[] = {{&S105912[91], 6}, {NULL}};
const Segment SGI241260[] = {{&S95331[34], 7}, {NULL}};
const Segment SGI241261[] = {{&S95331[27], 6}, {NULL}};
const Segment SGI241262[] = {{&S95331[94], 9}, {NULL}};
const Segment SGI241263[] = {{&S95331[67], 6}, {NULL}};
const Segment SGI241264[] = {{&S96206[34], 7}, {NULL}};
const Segment SGI241265[] = {{&S105916[34], 7}, {NULL}};
const Segment SGI241266[] = {{&S96206[27], 6}, {NULL}};
const Segment SGI241267[] = {{&S105916[27], 6}, {NULL}};
const Segment SGI241268[] = {{&S95332[34], 11}, {NULL}};
const Segment SGI241269[] = {{&S95332[27], 6}, {NULL}};
const Segment SGI241270[] = {{&S96206[99], 9}, {NULL}};
const Segment SGI241271[] = {{&S105916[94], 9}, {NULL}};
const Segment SGI241272[] = {{&S96206[72], 6}, {NULL}};
const Segment SGI241273[] = {{&S105916[67], 6}, {NULL}};
const Segment SGI241274[] = {{&S95332[98], 9}, {NULL}};
const Segment SGI241275[] = {{&S95332[71], 6}, {NULL}};
const Segment SGI241276[] = {{&S96209[34], 14}, {NULL}};
const Segment SGI241277[] = {{&S96209[27], 6}, {NULL}};
const Segment SGI241278[] = {{&S96209[101], 9}, {NULL}};
const Segment SGI241279[] = {{&S96209[74], 6}, {NULL}};
const Segment SGI241280[] = {{&S95339[44], 9}, {NULL}};
const Segment SGI241281[] = {{&S95339[44], 21}, {NULL}};
const Segment SGI241282[] = {{&S95339[30], 6}, {NULL}};
const Segment SGI241283[] = {{&S95339[114], 3}, {NULL}};
const Segment SGI241284[] = {{&S95339[79], 6}, {NULL}};
const Segment SGI241285[] = {{&S95339[139], 7}, {NULL}};
const Segment SGI241286[] = {{&S95342[34], 7}, {NULL}};
const Segment SGI241287[] = {{&S95342[27], 6}, {NULL}};
const Segment SGI241288[] = {{&S87638[80], 6}, {NULL}};
const Segment SGI241289[] = {{&S95342[99], 9}, {NULL}};
const Segment SGI241290[] = {{&S95342[72], 6}, {NULL}};
const Segment SGI241291[] = {{&S95342[140], 8}, {NULL}};
const Segment SGI241292[] = {{&S95345[44], 10}, {NULL}};
const Segment SGI241293[] = {{&S95345[44], 22}, {NULL}};
const Segment SGI241294[] = {{&S95345[30], 6}, {NULL}};
const Segment SGI241295[] = {{&S95345[113], 3}, {NULL}};
const Segment SGI241296[] = {{&S95345[80], 5}, {NULL}};
const Segment SGI241297[] = {{&S95345[117], 14}, {NULL}};
const Segment SGI241298[] = {{&S95350[45], 16}, {NULL}};
const Segment SGI241299[] = {{&S95350[30], 6}, {NULL}};
const Segment SGI241300[] = {{&S95350[121], 3}, {NULL}};
const Segment SGI241301[] = {{&S95350[87], 5}, {NULL}};
const Segment SGI241302[] = {{&S95350[145], 13}, {NULL}};
const Segment SGI241303[] = {{&S95357[34], 13}, {NULL}};
const Segment SGI241304[] = {{&S95357[27], 6}, {NULL}};
const Segment SGI241305[] = {{&S95357[100], 9}, {NULL}};
const Segment SGI241306[] = {{&S95357[73], 6}, {NULL}};
const Segment SGI241307[] = {{&S95358[44], 9}, {NULL}};
const Segment SGI241308[] = {{&S95358[44], 21}, {NULL}};
const Segment SGI241309[] = {{&S95358[30], 6}, {NULL}};
const Segment SGI241310[] = {{&S95358[79], 6}, {NULL}};
const Segment SGI241311[] = {{&S95358[106], 13}, {NULL}};
const Segment SGI241312[] = {{&S95360[34], 11}, {NULL}};
const Segment SGI241313[] = {{&S95360[27], 6}, {NULL}};
const Segment SGI241314[] = {{&S95360[98], 9}, {NULL}};
const Segment SGI241315[] = {{&S95360[71], 6}, {NULL}};
const Segment SGI241316[] = {{&S95360[139], 8}, {NULL}};
const Segment SGI241317[] = {{&S96247[45], 17}, {NULL}};
const Segment SGI241318[] = {{&S96247[30], 6}, {NULL}};
const Segment SGI241319[] = {{&S96247[122], 3}, {NULL}};
const Segment SGI241320[] = {{&S96247[88], 5}, {NULL}};
const Segment SGI241321[] = {{&S96247[156], 3}, {NULL}};
const Segment SGI241322[] = {{&S96267[34], 17}, {NULL}};
const Segment SGI241323[] = {{&S96267[27], 6}, {NULL}};
const Segment SGI241324[] = {{&S96267[104], 9}, {NULL}};
const Segment SGI241325[] = {{&S96267[77], 6}, {NULL}};
const Segment SGI241326[] = {{&S105987[34], 7}, {NULL}};
const Segment SGI241327[] = {{&S105987[27], 6}, {NULL}};
const Segment SGI241328[] = {{&S105987[111], 9}, {NULL}};
const Segment SGI241329[] = {{&S105987[84], 6}, {NULL}};
const Segment SGI241330[] = {{&S105987[152], 8}, {NULL}};
const Segment SGI241331[] = {{&S105988[34], 11}, {NULL}};
const Segment SGI241332[] = {{&S105988[27], 6}, {NULL}};
const Segment SGI241333[] = {{&S105988[98], 9}, {NULL}};
const Segment SGI241334[] = {{&S105988[71], 6}, {NULL}};
const Segment SGI241335[] = {{&S105988[132], 8}, {NULL}};
const Segment SGI241336[] = {{&S105989[34], 4}, {NULL}};
const Segment SGI241337[] = {{&S105989[27], 6}, {NULL}};
const Segment SGI241338[] = {{&S105989[91], 9}, {NULL}};
const Segment SGI241339[] = {{&S105989[64], 6}, {NULL}};
const Segment SGI241340[] = {{&S105989[132], 8}, {NULL}};
const Segment SGI241341[] = {{&S95455[34], 6}, {NULL}};
const Segment SGI241342[] = {{&S95455[27], 6}, {NULL}};
const Segment SGI241343[] = {{&S95455[93], 9}, {NULL}};
const Segment SGI241344[] = {{&S95455[66], 6}, {NULL}};
const Segment SGI241345[] = {{&S95510[44], 10}, {NULL}};
const Segment SGI241346[] = {{&S95510[44], 22}, {NULL}};
const Segment SGI241347[] = {{&S95510[30], 6}, {NULL}};
const Segment SGI241348[] = {{&S95510[115], 3}, {NULL}};
const Segment SGI241349[] = {{&S95510[80], 6}, {NULL}};
const Segment SGI241350[] = {{&S95510[140], 11}, {NULL}};
const Segment SGI241351[] = {{&S95511[34], 10}, {NULL}};
const Segment SGI241352[] = {{&S95511[27], 6}, {NULL}};
const Segment SGI241353[] = {{&S95511[97], 9}, {NULL}};
const Segment SGI241354[] = {{&S95511[70], 6}, {NULL}};
const Segment SGI241355[] = {{&S87650[34], 10}, {NULL}};
const Segment SGI241356[] = {{&S87650[27], 6}, {NULL}};
const Segment SGI241357[] = {{&S87650[96], 9}, {NULL}};
const Segment SGI241358[] = {{&S87650[69], 6}, {NULL}};
const Segment SGI241359[] = {{&S87650[137], 8}, {NULL}};
const Segment SGI241360[] = {{&S96394[34], 10}, {NULL}};
const Segment SGI241361[] = {{&S96394[27], 6}, {NULL}};
const Segment SGI241362[] = {{&S96394[97], 9}, {NULL}};
const Segment SGI241363[] = {{&S96394[70], 6}, {NULL}};
const Segment SGI241364[] = {{&S96401[34], 10}, {NULL}};
const Segment SGI241365[] = {{&S96401[27], 6}, {NULL}};
const Segment SGI241366[] = {{&S96401[97], 9}, {NULL}};
const Segment SGI241367[] = {{&S96401[70], 6}, {NULL}};
const Segment SGI241368[] = {{&S102906[54], 10}, {NULL}};
const Segment SGI241369[] = {{&S102906[47], 6}, {NULL}};
const Segment SGI241370[] = {{&S102908[44], 10}, {NULL}};
const Segment SGI241371[] = {{&S102908[44], 22}, {NULL}};
const Segment SGI241372[] = {{&S102908[30], 6}, {NULL}};
const Segment SGI241373[] = {{&S102908[115], 3}, {NULL}};
const Segment SGI241374[] = {{&S102908[80], 6}, {NULL}};
const Segment SGI241375[] = {{&S102922[57], 10}, {NULL}};
const Segment SGI241376[] = {{&S102922[45], 6}, {NULL}};
const Segment SGI241377[] = {{&S102922[128], 3}, {NULL}};
const Segment SGI241378[] = {{&S102922[93], 6}, {NULL}};
const Segment SGI241379[] = {{&S102922[0], 14}, {NULL}};
const Segment SGI241380[] = {{&S102923[59], 10}, {NULL}};
const Segment SGI241381[] = {{&S102923[59], 22}, {NULL}};
const Segment SGI241382[] = {{&S102923[45], 6}, {NULL}};
const Segment SGI241383[] = {{&S102923[130], 3}, {NULL}};
const Segment SGI241384[] = {{&S102923[95], 6}, {NULL}};
const Segment SGI241385[] = {{&S102923[0], 14}, {NULL}};
const Segment SGI241386[] = {{&S102924[59], 10}, {NULL}};
const Segment SGI241387[] = {{&S102924[59], 22}, {NULL}};
const Segment SGI241388[] = {{&S102924[45], 6}, {NULL}};
const Segment SGI241389[] = {{&S102924[130], 3}, {NULL}};
const Segment SGI241390[] = {{&S102924[95], 6}, {NULL}};
const Segment SGI241391[] = {{&S102924[0], 14}, {NULL}};
const Segment SGI241392[] = {{&S87755[34], 17}, {NULL}};
const Segment SGI241393[] = {{&S87755[27], 6}, {NULL}};
const Segment SGI241394[] = {{&S87755[103], 9}, {NULL}};
const Segment SGI241395[] = {{&S87755[76], 6}, {NULL}};
const Segment SGI241396[] = {{&S95525[34], 11}, {NULL}};
const Segment SGI241397[] = {{&S95525[27], 6}, {NULL}};
const Segment SGI241398[] = {{&S87756[34], 17}, {NULL}};
const Segment SGI241399[] = {{&S87756[27], 6}, {NULL}};
const Segment SGI241400[] = {{&S95525[98], 9}, {NULL}};
const Segment SGI241401[] = {{&S95525[71], 6}, {NULL}};
const Segment SGI241402[] = {{&S87756[103], 9}, {NULL}};
const Segment SGI241403[] = {{&S87756[76], 6}, {NULL}};
const Segment SGI241404[] = {{&S95527[44], 13}, {NULL}};
const Segment SGI241405[] = {{&S95527[44], 25}, {NULL}};
const Segment SGI241406[] = {{&S95527[30], 6}, {NULL}};
const Segment SGI241407[] = {{&S95527[83], 6}, {NULL}};
const Segment SGI241408[] = {{&S95527[110], 13}, {NULL}};
const Segment SGI241409[] = {{&S95528[44], 9}, {NULL}};
const Segment SGI241410[] = {{&S95528[44], 19}, {NULL}};
const Segment SGI241411[] = {{&S95528[30], 6}, {NULL}};
const Segment SGI241412[] = {{&S95528[112], 3}, {NULL}};
const Segment SGI241413[] = {{&S95528[77], 6}, {NULL}};
const Segment SGI241414[] = {{&S95530[44], 14}, {NULL}};
const Segment SGI241415[] = {{&S95530[44], 36}, {NULL}};
const Segment SGI241416[] = {{&S95530[30], 6}, {NULL}};
const Segment SGI241417[] = {{&S95530[127], 3}, {NULL}};
const Segment SGI241418[] = {{&S95530[94], 5}, {NULL}};
const Segment SGI241419[] = {{&S87768[44], 10}, {NULL}};
const Segment SGI241420[] = {{&S87768[44], 21}, {NULL}};
const Segment SGI241421[] = {{&S87768[30], 6}, {NULL}};
const Segment SGI241422[] = {{&S95533[34], 9}, {NULL}};
const Segment SGI241423[] = {{&S95533[27], 6}, {NULL}};
const Segment SGI241424[] = {{&S87768[79], 6}, {NULL}};
const Segment SGI241425[] = {{&S87768[106], 13}, {NULL}};
const Segment SGI241426[] = {{&S95533[96], 9}, {NULL}};
const Segment SGI241427[] = {{&S95533[69], 6}, {NULL}};
const Segment SGI241428[] = {{&S87774[34], 8}, {NULL}};
const Segment SGI241429[] = {{&S87774[27], 6}, {NULL}};
const Segment SGI241430[] = {{&S87774[94], 9}, {NULL}};
const Segment SGI241431[] = {{&S87774[67], 6}, {NULL}};
const Segment SGI241432[] = {{&S95543[44], 13}, {NULL}};
const Segment SGI241433[] = {{&S95543[44], 25}, {NULL}};
const Segment SGI241434[] = {{&S95543[30], 6}, {NULL}};
const Segment SGI241435[] = {{&S95543[116], 3}, {NULL}};
const Segment SGI241436[] = {{&S95543[83], 5}, {NULL}};
const Segment SGI241437[] = {{&S95543[120], 14}, {NULL}};
const Segment SGI241438[] = {{&S95544[34], 7}, {NULL}};
const Segment SGI241439[] = {{&S95544[27], 6}, {NULL}};
const Segment SGI241440[] = {{&S95544[100], 9}, {NULL}};
const Segment SGI241441[] = {{&S95544[73], 6}, {NULL}};
const Segment SGI241442[] = {{&S95551[34], 11}, {NULL}};
const Segment SGI241443[] = {{&S95551[27], 6}, {NULL}};
const Segment SGI241444[] = {{&S95551[98], 9}, {NULL}};
const Segment SGI241445[] = {{&S95551[71], 6}, {NULL}};
const Segment SGI241446[] = {{&S95551[139], 8}, {NULL}};
const Segment SGI241447[] = {{&S95555[34], 7}, {NULL}};
const Segment SGI241448[] = {{&S95555[27], 6}, {NULL}};
const Segment SGI241449[] = {{&S95555[94], 9}, {NULL}};
const Segment SGI241450[] = {{&S95555[67], 6}, {NULL}};
const Segment SGI241451[] = {{&S95557[44], 7}, {NULL}};
const Segment SGI241452[] = {{&S95557[44], 24}, {NULL}};
const Segment SGI241453[] = {{&S95557[30], 6}, {NULL}};
const Segment SGI241454[] = {{&S95557[82], 6}, {NULL}};
const Segment SGI241455[] = {{&S95557[109], 13}, {NULL}};
const Segment SGI241456[] = {{&S95561[34], 13}, {NULL}};
const Segment SGI241457[] = {{&S95561[27], 6}, {NULL}};
const Segment SGI241458[] = {{&S95561[100], 9}, {NULL}};
const Segment SGI241459[] = {{&S95561[73], 6}, {NULL}};
const Segment SGI241460[] = {{&S96531[34], 25}, {NULL}};
const Segment SGI241461[] = {{&S96531[27], 6}, {NULL}};
const Segment SGI241462[] = {{&S95567[34], 10}, {NULL}};
const Segment SGI241463[] = {{&S96531[112], 9}, {NULL}};
const Segment SGI241464[] = {{&S96531[85], 6}, {NULL}};
const Segment SGI241465[] = {{&S95567[27], 6}, {NULL}};
const Segment SGI241466[] = {{&S95567[97], 9}, {NULL}};
const Segment SGI241467[] = {{&S95567[70], 6}, {NULL}};
const Segment SGI241468[] = {{&S95567[138], 8}, {NULL}};
const Segment SGI241469[] = {{&S95568[34], 8}, {NULL}};
const Segment SGI241470[] = {{&S95568[27], 6}, {NULL}};
const Segment SGI241471[] = {{&S95568[100], 9}, {NULL}};
const Segment SGI241472[] = {{&S95568[73], 6}, {NULL}};
const Segment SGI241473[] = {{&S96534[34], 9}, {NULL}};
const Segment SGI241474[] = {{&S96534[27], 6}, {NULL}};
const Segment SGI241475[] = {{&S96534[96], 9}, {NULL}};
const Segment SGI241476[] = {{&S96534[69], 6}, {NULL}};
const Segment SGI241477[] = {{&S96552[44], 13}, {NULL}};
const Segment SGI241478[] = {{&S96552[44], 29}, {NULL}};
const Segment SGI241479[] = {{&S96552[30], 6}, {NULL}};
const Segment SGI241480[] = {{&S96552[122], 3}, {NULL}};
const Segment SGI241481[] = {{&S96552[87], 6}, {NULL}};
const Segment SGI241482[] = {{&S87840[34], 10}, {NULL}};
const Segment SGI241483[] = {{&S87840[27], 6}, {NULL}};
const Segment SGI241484[] = {{&S87840[96], 9}, {NULL}};
const Segment SGI241485[] = {{&S87840[69], 6}, {NULL}};
const Segment SGI241486[] = {{&S96568[45], 17}, {NULL}};
const Segment SGI241487[] = {{&S96568[30], 6}, {NULL}};
const Segment SGI241488[] = {{&S96568[122], 3}, {NULL}};
const Segment SGI241489[] = {{&S96568[88], 5}, {NULL}};
const Segment SGI241490[] = {{&S96568[156], 3}, {NULL}};
const Segment SGI241491[] = {{&S96585[34], 19}, {NULL}};
const Segment SGI241492[] = {{&S96585[27], 6}, {NULL}};
const Segment SGI241493[] = {{&S96585[106], 9}, {NULL}};
const Segment SGI241494[] = {{&S96585[79], 6}, {NULL}};
const Segment SGI241495[] = {{&S87868[44], 9}, {NULL}};
const Segment SGI241496[] = {{&S87868[44], 20}, {NULL}};
const Segment SGI241497[] = {{&S87868[30], 6}, {NULL}};
const Segment SGI241498[] = {{&S87868[113], 3}, {NULL}};
const Segment SGI241499[] = {{&S87868[78], 6}, {NULL}};
const Segment SGI241500[] = {{&S87868[148], 3}, {NULL}};
const Segment SGI241501[] = {{&S87871[44], 7}, {NULL}};
const Segment SGI241502[] = {{&S87871[44], 19}, {NULL}};
const Segment SGI241503[] = {{&S87871[30], 6}, {NULL}};
const Segment SGI241504[] = {{&S87871[112], 3}, {NULL}};
const Segment SGI241505[] = {{&S87871[77], 6}, {NULL}};
const Segment SGI241506[] = {{&S87873[34], 7}, {NULL}};
const Segment SGI241507[] = {{&S87873[27], 6}, {NULL}};
const Segment SGI241508[] = {{&S87873[100], 9}, {NULL}};
const Segment SGI241509[] = {{&S87873[73], 6}, {NULL}};
const Segment SGI241510[] = {{&S87873[141], 8}, {NULL}};
const Segment SGI241511[] = {{&S87879[44], 10}, {NULL}};
const Segment SGI241512[] = {{&S87879[44], 21}, {NULL}};
const Segment SGI241513[] = {{&S87879[30], 6}, {NULL}};
const Segment SGI241514[] = {{&S87879[114], 3}, {NULL}};
const Segment SGI241515[] = {{&S87879[79], 6}, {NULL}};
const Segment SGI241516[] = {{&S87879[139], 7}, {NULL}};
const Segment SGI241517[] = {{&S87887[34], 8}, {NULL}};
const Segment SGI241518[] = {{&S87887[27], 6}, {NULL}};
const Segment SGI241519[] = {{&S87887[94], 9}, {NULL}};
const Segment SGI241520[] = {{&S87887[67], 6}, {NULL}};
const Segment SGI241521[] = {{&S87890[44], 10}, {NULL}};
const Segment SGI241522[] = {{&S87890[44], 21}, {NULL}};
const Segment SGI241523[] = {{&S87890[30], 6}, {NULL}};
const Segment SGI241524[] = {{&S87890[79], 6}, {NULL}};
const Segment SGI241525[] = {{&S87890[106], 13}, {NULL}};
const Segment SGI241526[] = {{&S95571[44], 18}, {NULL}};
const Segment SGI241527[] = {{&S95571[44], 39}, {NULL}};
const Segment SGI241528[] = {{&S95571[30], 6}, {NULL}};
const Segment SGI241529[] = {{&S95571[97], 6}, {NULL}};
const Segment SGI241530[] = {{&S95571[124], 13}, {NULL}};
const Segment SGI241531[] = {{&S103111[44], 10}, {NULL}};
const Segment SGI241532[] = {{&S103111[44], 22}, {NULL}};
const Segment SGI241533[] = {{&S103111[30], 6}, {NULL}};
const Segment SGI241534[] = {{&S103111[144], 3}, {NULL}};
const Segment SGI241535[] = {{&S95578[34], 10}, {NULL}};
const Segment SGI241536[] = {{&S103111[109], 6}, {NULL}};
const Segment SGI241537[] = {{&S95578[27], 6}, {NULL}};
const Segment SGI241538[] = {{&S103111[68], 8}, {NULL}};
const Segment SGI241539[] = {{&S103111[80], 15}, {NULL}};
const Segment SGI241540[] = {{&S95578[97], 9}, {NULL}};
const Segment SGI241541[] = {{&S95578[70], 6}, {NULL}};
const Segment SGI241542[] = {{&S103112[44], 10}, {NULL}};
const Segment SGI241543[] = {{&S103112[44], 22}, {NULL}};
const Segment SGI241544[] = {{&S103112[30], 6}, {NULL}};
const Segment SGI241545[] = {{&S103112[144], 3}, {NULL}};
const Segment SGI241546[] = {{&S103112[109], 6}, {NULL}};
const Segment SGI241547[] = {{&S103112[68], 8}, {NULL}};
const Segment SGI241548[] = {{&S103112[80], 15}, {NULL}};
const Segment SGI241549[] = {{&S95581[44], 14}, {NULL}};
const Segment SGI241550[] = {{&S95581[44], 31}, {NULL}};
const Segment SGI241551[] = {{&S95581[30], 6}, {NULL}};
const Segment SGI241552[] = {{&S95581[89], 6}, {NULL}};
const Segment SGI241553[] = {{&S95581[116], 13}, {NULL}};
const Segment SGI241554[] = {{&S95585[59], 15}, {NULL}};
const Segment SGI241555[] = {{&S95585[59], 27}, {NULL}};
const Segment SGI241556[] = {{&S95585[45], 6}, {NULL}};
const Segment SGI241557[] = {{&S95585[135], 3}, {NULL}};
const Segment SGI241558[] = {{&S95585[100], 6}, {NULL}};
const Segment SGI241559[] = {{&S95585[11], 3}, {NULL}};
const Segment SGI241560[] = {{&S95586[34], 10}, {NULL}};
const Segment SGI241561[] = {{&S95586[27], 6}, {NULL}};
const Segment SGI241562[] = {{&S95586[97], 9}, {NULL}};
const Segment SGI241563[] = {{&S95586[70], 6}, {NULL}};
const Segment SGI241564[] = {{&S95606[34], 7}, {NULL}};
const Segment SGI241565[] = {{&S95606[27], 6}, {NULL}};
const Segment SGI241566[] = {{&S95606[93], 9}, {NULL}};
const Segment SGI241567[] = {{&S95606[66], 6}, {NULL}};
const Segment SGI241568[] = {{&S103148[45], 10}, {NULL}};
const Segment SGI241569[] = {{&S103148[30], 6}, {NULL}};
const Segment SGI241570[] = {{&S103148[115], 3}, {NULL}};
const Segment SGI241571[] = {{&S103148[81], 5}, {NULL}};
const Segment SGI241572[] = {{&S103148[139], 13}, {NULL}};
const Segment SGI241573[] = {{&S103149[44], 10}, {NULL}};
const Segment SGI241574[] = {{&S103149[44], 22}, {NULL}};
const Segment SGI241575[] = {{&S103149[30], 6}, {NULL}};
const Segment SGI241576[] = {{&S103149[144], 3}, {NULL}};
const Segment SGI241577[] = {{&S103149[109], 6}, {NULL}};
const Segment SGI241578[] = {{&S103149[68], 8}, {NULL}};
const Segment SGI241579[] = {{&S103149[80], 15}, {NULL}};
const Segment SGI241580[] = {{&S96677[44], 9}, {NULL}};
const Segment SGI241581[] = {{&S96677[44], 21}, {NULL}};
const Segment SGI241582[] = {{&S96677[30], 6}, {NULL}};
const Segment SGI241583[] = {{&S96677[143], 3}, {NULL}};
const Segment SGI241584[] = {{&S103150[44], 10}, {NULL}};
const Segment SGI241585[] = {{&S96677[108], 6}, {NULL}};
const Segment SGI241586[] = {{&S103150[44], 22}, {NULL}};
const Segment SGI241587[] = {{&S103150[30], 6}, {NULL}};
const Segment SGI241588[] = {{&S103150[144], 3}, {NULL}};
const Segment SGI241589[] = {{&S103150[109], 6}, {NULL}};
const Segment SGI241590[] = {{&S103150[68], 8}, {NULL}};
const Segment SGI241591[] = {{&S103150[80], 15}, {NULL}};
const Segment SGI241592[] = {{&S95628[34], 10}, {NULL}};
const Segment SGI241593[] = {{&S95628[27], 6}, {NULL}};
const Segment SGI241594[] = {{&S95628[97], 9}, {NULL}};
const Segment SGI241595[] = {{&S95628[70], 6}, {NULL}};
const Segment SGI241596[] = {{&S87958[34], 7}, {NULL}};
const Segment SGI241597[] = {{&S87958[27], 6}, {NULL}};
const Segment SGI241598[] = {{&S87958[93], 9}, {NULL}};
const Segment SGI241599[] = {{&S87958[66], 6}, {NULL}};
const Segment SGI241600[] = {{&S95633[34], 10}, {NULL}};
const Segment SGI241601[] = {{&S95633[27], 6}, {NULL}};
const Segment SGI241602[] = {{&S95633[97], 9}, {NULL}};
const Segment SGI241603[] = {{&S95633[70], 6}, {NULL}};
const Segment SGI241604[] = {{&S103165[45], 10}, {NULL}};
const Segment SGI241605[] = {{&S103165[30], 6}, {NULL}};
const Segment SGI241606[] = {{&S103165[115], 3}, {NULL}};
const Segment SGI241607[] = {{&S103165[81], 5}, {NULL}};
const Segment SGI241608[] = {{&S103165[139], 13}, {NULL}};
const Segment SGI241609[] = {{&S95637[44], 17}, {NULL}};
const Segment SGI241610[] = {{&S95637[44], 29}, {NULL}};
const Segment SGI241611[] = {{&S95637[30], 6}, {NULL}};
const Segment SGI241612[] = {{&S95637[122], 3}, {NULL}};
const Segment SGI241613[] = {{&S95637[87], 6}, {NULL}};
const Segment SGI241614[] = {{&S95650[44], 9}, {NULL}};
const Segment SGI241615[] = {{&S95650[44], 21}, {NULL}};
const Segment SGI241616[] = {{&S95650[30], 6}, {NULL}};
const Segment SGI241617[] = {{&S95650[79], 6}, {NULL}};
const Segment SGI241618[] = {{&S95650[106], 13}, {NULL}};
const Segment SGI241619[] = {{&S103181[45], 10}, {NULL}};
const Segment SGI241620[] = {{&S103181[30], 6}, {NULL}};
const Segment SGI241621[] = {{&S103181[115], 3}, {NULL}};
const Segment SGI241622[] = {{&S103181[81], 5}, {NULL}};
const Segment SGI241623[] = {{&S103181[139], 13}, {NULL}};
const Segment SGI241624[] = {{&S95656[44], 10}, {NULL}};
const Segment SGI241625[] = {{&S95656[44], 22}, {NULL}};
const Segment SGI241626[] = {{&S95656[30], 6}, {NULL}};
const Segment SGI241627[] = {{&S95656[80], 6}, {NULL}};
const Segment SGI241628[] = {{&S95656[107], 13}, {NULL}};
const Segment SGI241629[] = {{&S88015[44], 10}, {NULL}};
const Segment SGI241630[] = {{&S88015[44], 21}, {NULL}};
const Segment SGI241631[] = {{&S88015[30], 6}, {NULL}};
const Segment SGI241632[] = {{&S88015[132], 3}, {NULL}};
const Segment SGI241633[] = {{&S88015[97], 6}, {NULL}};
const Segment SGI241634[] = {{&S88015[67], 8}, {NULL}};
const Segment SGI241635[] = {{&S88016[44], 10}, {NULL}};
const Segment SGI241636[] = {{&S88016[44], 22}, {NULL}};
const Segment SGI241637[] = {{&S88016[30], 6}, {NULL}};
const Segment SGI241638[] = {{&S88016[115], 3}, {NULL}};
const Segment SGI241639[] = {{&S88016[80], 6}, {NULL}};
const Segment SGI241640[] = {{&S88016[133], 8}, {NULL}};
const Segment SGI241641[] = {{&S88028[34], 10}, {NULL}};
const Segment SGI241642[] = {{&S88028[27], 6}, {NULL}};
const Segment SGI241643[] = {{&S88028[96], 9}, {NULL}};
const Segment SGI241644[] = {{&S88028[69], 6}, {NULL}};
const Segment SGI241645[] = {{&S88028[137], 8}, {NULL}};
const Segment SGI241646[] = {{&S95761[34], 10}, {NULL}};
const Segment SGI241647[] = {{&S95761[27], 6}, {NULL}};
const Segment SGI241648[] = {{&S95761[97], 9}, {NULL}};
const Segment SGI241649[] = {{&S95761[70], 6}, {NULL}};
const Segment SGI241650[] = {{&S88093[34], 11}, {NULL}};
const Segment SGI241651[] = {{&S88093[27], 6}, {NULL}};
const Segment SGI241652[] = {{&S88093[97], 9}, {NULL}};
const Segment SGI241653[] = {{&S88093[70], 6}, {NULL}};
const Segment SGI241654[] = {{&S95774[34], 5}, {NULL}};
const Segment SGI241655[] = {{&S95774[27], 6}, {NULL}};
const Segment SGI241656[] = {{&S95774[92], 9}, {NULL}};
const Segment SGI241657[] = {{&S95774[65], 6}, {NULL}};
const Segment SGI241658[] = {{&S88122[34], 10}, {NULL}};
const Segment SGI241659[] = {{&S88122[27], 6}, {NULL}};
const Segment SGI241660[] = {{&S95801[34], 10}, {NULL}};
const Segment SGI241661[] = {{&S95801[27], 6}, {NULL}};
const Segment SGI241662[] = {{&S88122[97], 9}, {NULL}};
const Segment SGI241663[] = {{&S88122[70], 6}, {NULL}};
const Segment SGI241664[] = {{&S95801[97], 9}, {NULL}};
const Segment SGI241665[] = {{&S95801[70], 6}, {NULL}};
const Segment SGI241666[] = {{&S95813[34], 15}, {NULL}};
const Segment SGI241667[] = {{&S95813[27], 6}, {NULL}};
const Segment SGI241668[] = {{&S95813[102], 9}, {NULL}};
const Segment SGI241669[] = {{&S95813[75], 6}, {NULL}};
const Segment SGI241670[] = {{&S95815[34], 11}, {NULL}};
const Segment SGI241671[] = {{&S95815[27], 6}, {NULL}};
const Segment SGI241672[] = {{&S95815[98], 9}, {NULL}};
const Segment SGI241673[] = {{&S95815[71], 6}, {NULL}};
const Segment SGI241674[] = {{&S95823[34], 8}, {NULL}};
const Segment SGI241675[] = {{&S95823[27], 6}, {NULL}};
const Segment SGI241676[] = {{&S95823[101], 9}, {NULL}};
const Segment SGI241677[] = {{&S95823[74], 6}, {NULL}};
const Segment SGI241678[] = {{&S95824[34], 12}, {NULL}};
const Segment SGI241679[] = {{&S95824[27], 6}, {NULL}};
const Segment SGI241680[] = {{&S95824[99], 9}, {NULL}};
const Segment SGI241681[] = {{&S95824[72], 6}, {NULL}};
const Segment SGI241682[] = {{&S95826[44], 10}, {NULL}};
const Segment SGI241683[] = {{&S95826[44], 22}, {NULL}};
const Segment SGI241684[] = {{&S95826[30], 6}, {NULL}};
const Segment SGI241685[] = {{&S95826[115], 3}, {NULL}};
const Segment SGI241686[] = {{&S96694[44], 10}, {NULL}};
const Segment SGI241687[] = {{&S96694[44], 22}, {NULL}};
const Segment SGI241688[] = {{&S95826[80], 6}, {NULL}};
const Segment SGI241689[] = {{&S88154[34], 12}, {NULL}};
const Segment SGI241690[] = {{&S96694[30], 6}, {NULL}};
const Segment SGI241691[] = {{&S88154[27], 6}, {NULL}};
const Segment SGI241692[] = {{&S95826[147], 15}, {NULL}};
const Segment SGI241693[] = {{&S88154[98], 9}, {NULL}};
const Segment SGI241694[] = {{&S96694[133], 3}, {NULL}};
const Segment SGI241695[] = {{&S88154[71], 6}, {NULL}};
const Segment SGI241696[] = {{&S96694[98], 6}, {NULL}};
const Segment SGI241697[] = {{&S96694[161], 3}, {NULL}};
const Segment SGI241698[] = {{&S96694[68], 8}, {NULL}};
const Segment SGI241699[] = {{&S96695[44], 10}, {NULL}};
const Segment SGI241700[] = {{&S96695[44], 22}, {NULL}};
const Segment SGI241701[] = {{&S96695[30], 6}, {NULL}};
const Segment SGI241702[] = {{&S96695[133], 3}, {NULL}};
const Segment SGI241703[] = {{&S96695[98], 6}, {NULL}};
const Segment SGI241704[] = {{&S96695[161], 3}, {NULL}};
const Segment SGI241705[] = {{&S96695[68], 8}, {NULL}};
const Segment SGI241706[] = {{&S88157[44], 10}, {NULL}};
const Segment SGI241707[] = {{&S88157[44], 21}, {NULL}};
const Segment SGI241708[] = {{&S88157[30], 6}, {NULL}};
const Segment SGI241709[] = {{&S88157[79], 6}, {NULL}};
const Segment SGI241710[] = {{&S88157[106], 13}, {NULL}};
const Segment SGI241711[] = {{&S95847[34], 11}, {NULL}};
const Segment SGI241712[] = {{&S95847[27], 6}, {NULL}};
const Segment SGI241713[] = {{&S95847[98], 9}, {NULL}};
const Segment SGI241714[] = {{&S95847[71], 6}, {NULL}};
const Segment SGI241715[] = {{&S95849[34], 9}, {NULL}};
const Segment SGI241716[] = {{&S95849[27], 6}, {NULL}};
const Segment SGI241717[] = {{&S95849[96], 9}, {NULL}};
const Segment SGI241718[] = {{&S95849[69], 6}, {NULL}};
const Segment SGI241719[] = {{&S95851[34], 10}, {NULL}};
const Segment SGI241720[] = {{&S95851[27], 6}, {NULL}};
const Segment SGI241721[] = {{&S95851[97], 9}, {NULL}};
const Segment SGI241722[] = {{&S95851[70], 6}, {NULL}};
const Segment SGI241723[] = {{&S95854[34], 14}, {NULL}};
const Segment SGI241724[] = {{&S95854[27], 6}, {NULL}};
const Segment SGI241725[] = {{&S95854[100], 9}, {NULL}};
const Segment SGI241726[] = {{&S95854[73], 6}, {NULL}};
const Segment SGI241727[] = {{&S95856[34], 7}, {NULL}};
const Segment SGI241728[] = {{&S95856[27], 6}, {NULL}};
const Segment SGI241729[] = {{&S95856[99], 9}, {NULL}};
const Segment SGI241730[] = {{&S95856[72], 6}, {NULL}};
const Segment SGI241731[] = {{&S95884[44], 13}, {NULL}};
const Segment SGI241732[] = {{&S95884[44], 25}, {NULL}};
const Segment SGI241733[] = {{&S95884[30], 6}, {NULL}};
const Segment SGI241734[] = {{&S95884[116], 3}, {NULL}};
const Segment SGI241735[] = {{&S95884[83], 5}, {NULL}};
const Segment SGI241736[] = {{&S95884[120], 14}, {NULL}};
const Segment SGI241737[] = {{&S96756[44], 10}, {NULL}};
const Segment SGI241738[] = {{&S96756[44], 22}, {NULL}};
const Segment SGI241739[] = {{&S96756[30], 6}, {NULL}};
const Segment SGI241740[] = {{&S95885[44], 7}, {NULL}};
const Segment SGI241741[] = {{&S95885[44], 24}, {NULL}};
const Segment SGI241742[] = {{&S95885[30], 6}, {NULL}};
const Segment SGI241743[] = {{&S96756[133], 3}, {NULL}};
const Segment SGI241744[] = {{&S96756[98], 6}, {NULL}};
const Segment SGI241745[] = {{&S96756[68], 8}, {NULL}};
const Segment SGI241746[] = {{&S95885[117], 3}, {NULL}};
const Segment SGI241747[] = {{&S95885[82], 6}, {NULL}};
const Segment SGI241748[] = {{&S95885[142], 7}, {NULL}};
const Segment SGI241749[] = {{&S95887[52], 10}, {NULL}};
const Segment SGI241750[] = {{&S95887[52], 22}, {NULL}};
const Segment SGI241751[] = {{&S95887[38], 6}, {NULL}};
const Segment SGI241752[] = {{&S95887[88], 5}, {NULL}};
const Segment SGI241753[] = {{&S95887[11], 3}, {NULL}};
const Segment SGI241754[] = {{&S95888[44], 10}, {NULL}};
const Segment SGI241755[] = {{&S95888[44], 22}, {NULL}};
const Segment SGI241756[] = {{&S95888[30], 6}, {NULL}};
const Segment SGI241757[] = {{&S95888[115], 3}, {NULL}};
const Segment SGI241758[] = {{&S95888[80], 6}, {NULL}};
const Segment SGI241759[] = {{&S95888[140], 7}, {NULL}};
const Segment SGI241760[] = {{&S95898[34], 9}, {NULL}};
const Segment SGI241761[] = {{&S95898[27], 6}, {NULL}};
const Segment SGI241762[] = {{&S95898[96], 9}, {NULL}};
const Segment SGI241763[] = {{&S95898[69], 6}, {NULL}};
const Segment SGI241764[] = {{&S95899[34], 10}, {NULL}};
const Segment SGI241765[] = {{&S95899[27], 6}, {NULL}};
const Segment SGI241766[] = {{&S95899[97], 9}, {NULL}};
const Segment SGI241767[] = {{&S95899[70], 6}, {NULL}};
const Segment SGI241768[] = {{&S95900[34], 10}, {NULL}};
const Segment SGI241769[] = {{&S95900[27], 6}, {NULL}};
const Segment SGI241770[] = {{&S95900[97], 9}, {NULL}};
const Segment SGI241771[] = {{&S95900[70], 6}, {NULL}};
const Segment SGI241772[] = {{&S88259[34], 9}, {NULL}};
const Segment SGI241773[] = {{&S88259[27], 6}, {NULL}};
const Segment SGI241774[] = {{&S88259[96], 9}, {NULL}};
const Segment SGI241775[] = {{&S88259[69], 6}, {NULL}};
const Segment SGI241776[] = {{&S95917[34], 9}, {NULL}};
const Segment SGI241777[] = {{&S95917[27], 6}, {NULL}};
const Segment SGI241778[] = {{&S95917[96], 9}, {NULL}};
const Segment SGI241779[] = {{&S95917[69], 6}, {NULL}};
const Segment SGI241780[] = {{&S95920[44], 7}, {NULL}};
const Segment SGI241781[] = {{&S95920[44], 19}, {NULL}};
const Segment SGI241782[] = {{&S95920[30], 6}, {NULL}};
const Segment SGI241783[] = {{&S95920[110], 3}, {NULL}};
const Segment SGI241784[] = {{&S95920[77], 5}, {NULL}};
const Segment SGI241785[] = {{&S95920[114], 14}, {NULL}};
const Segment SGI241786[] = {{&S88271[34], 8}, {NULL}};
const Segment SGI241787[] = {{&S88271[27], 6}, {NULL}};
const Segment SGI241788[] = {{&S88271[103], 9}, {NULL}};
const Segment SGI241789[] = {{&S88271[76], 6}, {NULL}};
const Segment SGI241790[] = {{&S88271[137], 8}, {NULL}};
const Segment SGI241791[] = {{&S95928[44], 7}, {NULL}};
const Segment SGI241792[] = {{&S95928[44], 19}, {NULL}};
const Segment SGI241793[] = {{&S95928[30], 6}, {NULL}};
const Segment SGI241794[] = {{&S88272[34], 7}, {NULL}};
const Segment SGI241795[] = {{&S95928[77], 6}, {NULL}};
const Segment SGI241796[] = {{&S88272[27], 6}, {NULL}};
const Segment SGI241797[] = {{&S95928[104], 13}, {NULL}};
const Segment SGI241798[] = {{&S88272[94], 9}, {NULL}};
const Segment SGI241799[] = {{&S88272[67], 6}, {NULL}};
const Segment SGI241800[] = {{&S88275[34], 7}, {NULL}};
const Segment SGI241801[] = {{&S88275[27], 6}, {NULL}};
const Segment SGI241802[] = {{&S88275[94], 9}, {NULL}};
const Segment SGI241803[] = {{&S88275[67], 6}, {NULL}};
const Segment SGI241804[] = {{&S88276[34], 10}, {NULL}};
const Segment SGI241805[] = {{&S88276[27], 6}, {NULL}};
const Segment SGI241806[] = {{&S88276[96], 9}, {NULL}};
const Segment SGI241807[] = {{&S88276[69], 6}, {NULL}};
const Segment SGI241808[] = {{&S88278[34], 10}, {NULL}};
const Segment SGI241809[] = {{&S88278[27], 6}, {NULL}};
const Segment SGI241810[] = {{&S88278[96], 9}, {NULL}};
const Segment SGI241811[] = {{&S88278[69], 6}, {NULL}};
const Segment SGI241812[] = {{&S88278[137], 8}, {NULL}};
const Segment SGI241813[] = {{&S95942[34], 10}, {NULL}};
const Segment SGI241814[] = {{&S95942[27], 6}, {NULL}};
const Segment SGI241815[] = {{&S95942[97], 9}, {NULL}};
const Segment SGI241816[] = {{&S95942[70], 6}, {NULL}};
const Segment SGI241817[] = {{&S95943[34], 10}, {NULL}};
const Segment SGI241818[] = {{&S95943[27], 6}, {NULL}};
const Segment SGI241819[] = {{&S95943[97], 9}, {NULL}};
const Segment SGI241820[] = {{&S95943[70], 6}, {NULL}};
const Segment SGI241821[] = {{&S95943[138], 8}, {NULL}};
const Segment SGI241822[] = {{&S95944[34], 10}, {NULL}};
const Segment SGI241823[] = {{&S95944[27], 6}, {NULL}};
const Segment SGI241824[] = {{&S95944[97], 9}, {NULL}};
const Segment SGI241825[] = {{&S95944[70], 6}, {NULL}};
const Segment SGI241826[] = {{&S95945[34], 18}, {NULL}};
const Segment SGI241827[] = {{&S95945[27], 6}, {NULL}};
const Segment SGI241828[] = {{&S95945[105], 9}, {NULL}};
const Segment SGI241829[] = {{&S95945[78], 6}, {NULL}};
const Segment SGI241830[] = {{&S96834[34], 8}, {NULL}};
const Segment SGI241831[] = {{&S96834[27], 6}, {NULL}};
const Segment SGI241832[] = {{&S96834[95], 9}, {NULL}};
const Segment SGI241833[] = {{&S96834[68], 6}, {NULL}};
const Segment SGI241834[] = {{&S103418[44], 14}, {NULL}};
const Segment SGI241835[] = {{&S103418[44], 26}, {NULL}};
const Segment SGI241836[] = {{&S103418[30], 6}, {NULL}};
const Segment SGI241837[] = {{&S103418[148], 3}, {NULL}};
const Segment SGI241838[] = {{&S103418[113], 6}, {NULL}};
const Segment SGI241839[] = {{&S103418[72], 8}, {NULL}};
const Segment SGI241840[] = {{&S103418[84], 15}, {NULL}};
const Segment SGI241841[] = {{&S88333[34], 6}, {NULL}};
const Segment SGI241842[] = {{&S88333[27], 6}, {NULL}};
const Segment SGI241843[] = {{&S88333[92], 9}, {NULL}};
const Segment SGI241844[] = {{&S88333[65], 6}, {NULL}};
const Segment SGI241845[] = {{&S103436[31], 8}, {NULL}};
const Segment SGI241846[] = {{&S103436[54], 3}, {NULL}};
const Segment SGI241847[] = {{&S103437[39], 8}, {NULL}};
const Segment SGI241848[] = {{&S103437[103], 3}, {NULL}};
const Segment SGI241849[] = {{&S103437[92], 3}, {NULL}};
const Segment SGI241850[] = {{&S103437[69], 6}, {NULL}};
const Segment SGI241851[] = {{&S103445[45], 10}, {NULL}};
const Segment SGI241852[] = {{&S103445[30], 6}, {NULL}};
const Segment SGI241853[] = {{&S103445[115], 3}, {NULL}};
const Segment SGI241854[] = {{&S103445[81], 5}, {NULL}};
const Segment SGI241855[] = {{&S103445[139], 13}, {NULL}};
const Segment SGI241856[] = {{&S103447[44], 10}, {NULL}};
const Segment SGI241857[] = {{&S103447[44], 22}, {NULL}};
const Segment SGI241858[] = {{&S103447[30], 6}, {NULL}};
const Segment SGI241859[] = {{&S103447[115], 3}, {NULL}};
const Segment SGI241860[] = {{&S103447[80], 6}, {NULL}};
const Segment SGI241861[] = {{&S103447[132], 10}, {NULL}};
const Segment SGI241862[] = {{&S88355[34], 9}, {NULL}};
const Segment SGI241863[] = {{&S88355[27], 6}, {NULL}};
const Segment SGI241864[] = {{&S88355[95], 9}, {NULL}};
const Segment SGI241865[] = {{&S88355[68], 6}, {NULL}};
const Segment SGI241866[] = {{&S88356[34], 9}, {NULL}};
const Segment SGI241867[] = {{&S88356[27], 6}, {NULL}};
const Segment SGI241868[] = {{&S88356[96], 9}, {NULL}};
const Segment SGI241869[] = {{&S88356[69], 6}, {NULL}};
const Segment SGI241870[] = {{&S88357[34], 10}, {NULL}};
const Segment SGI241871[] = {{&S88357[27], 6}, {NULL}};
const Segment SGI241872[] = {{&S88357[96], 9}, {NULL}};
const Segment SGI241873[] = {{&S88357[69], 6}, {NULL}};
const Segment SGI241874[] = {{&S103454[44], 10}, {NULL}};
const Segment SGI241875[] = {{&S103454[44], 22}, {NULL}};
const Segment SGI241876[] = {{&S103454[30], 6}, {NULL}};
const Segment SGI241877[] = {{&S103454[131], 3}, {NULL}};
const Segment SGI241878[] = {{&S103454[96], 6}, {NULL}};
const Segment SGI241879[] = {{&S103454[159], 3}, {NULL}};
const Segment SGI241880[] = {{&S103454[68], 8}, {NULL}};
const Segment SGI241881[] = {{&S103455[45], 10}, {NULL}};
const Segment SGI241882[] = {{&S103455[30], 6}, {NULL}};
const Segment SGI241883[] = {{&S103455[115], 3}, {NULL}};
const Segment SGI241884[] = {{&S103455[81], 5}, {NULL}};
const Segment SGI241885[] = {{&S103455[139], 13}, {NULL}};
const Segment SGI241886[] = {{&S103460[45], 10}, {NULL}};
const Segment SGI241887[] = {{&S103460[30], 6}, {NULL}};
const Segment SGI241888[] = {{&S103460[115], 3}, {NULL}};
const Segment SGI241889[] = {{&S103460[81], 5}, {NULL}};
const Segment SGI241890[] = {{&S103460[139], 13}, {NULL}};
const Segment SGI241891[] = {{&S103464[44], 10}, {NULL}};
const Segment SGI241892[] = {{&S103464[44], 22}, {NULL}};
const Segment SGI241893[] = {{&S103464[30], 6}, {NULL}};
const Segment SGI241894[] = {{&S103464[115], 3}, {NULL}};
const Segment SGI241895[] = {{&S103464[80], 6}, {NULL}};
const Segment SGI241896[] = {{&S103464[135], 8}, {NULL}};
const Segment SGI241897[] = {{&S103464[144], 13}, {NULL}};
const Segment SGI241898[] = {{&S103466[44], 10}, {NULL}};
const Segment SGI241899[] = {{&S103466[44], 22}, {NULL}};
const Segment SGI241900[] = {{&S103466[30], 6}, {NULL}};
const Segment SGI241901[] = {{&S103466[115], 3}, {NULL}};
const Segment SGI241902[] = {{&S103466[80], 6}, {NULL}};
const Segment SGI241903[] = {{&S103466[157], 3}, {NULL}};
const Segment SGI241904[] = {{&S88381[34], 8}, {NULL}};
const Segment SGI241905[] = {{&S88381[27], 6}, {NULL}};
const Segment SGI241906[] = {{&S88381[94], 9}, {NULL}};
const Segment SGI241907[] = {{&S88381[67], 6}, {NULL}};
const Segment SGI241908[] = {{&S88398[34], 12}, {NULL}};
const Segment SGI241909[] = {{&S88398[27], 6}, {NULL}};
const Segment SGI241910[] = {{&S88398[99], 9}, {NULL}};
const Segment SGI241911[] = {{&S88398[72], 6}, {NULL}};
const Segment SGI241912[] = {{&S96038[34], 7}, {NULL}};
const Segment SGI241913[] = {{&S96038[27], 6}, {NULL}};
const Segment SGI241914[] = {{&S96038[94], 9}, {NULL}};
const Segment SGI241915[] = {{&S96038[67], 6}, {NULL}};
const Segment SGI241916[] = {{&S96088[34], 7}, {NULL}};
const Segment SGI241917[] = {{&S96088[27], 6}, {NULL}};
const Segment SGI241918[] = {{&S96088[94], 9}, {NULL}};
const Segment SGI241919[] = {{&S96088[67], 6}, {NULL}};
const Segment SGI241920[] = {{&S96090[34], 10}, {NULL}};
const Segment SGI241921[] = {{&S96090[27], 6}, {NULL}};
const Segment SGI241922[] = {{&S96090[97], 9}, {NULL}};
const Segment SGI241923[] = {{&S96090[70], 6}, {NULL}};
const Segment SGI241924[] = {{&S96093[34], 8}, {NULL}};
const Segment SGI241925[] = {{&S96093[27], 6}, {NULL}};
const Segment SGI241926[] = {{&S96093[102], 9}, {NULL}};
const Segment SGI241927[] = {{&S96093[75], 6}, {NULL}};
const Segment SGI241928[] = {{&S97073[34], 7}, {NULL}};
const Segment SGI241929[] = {{&S97073[27], 6}, {NULL}};
const Segment SGI241930[] = {{&S97073[94], 9}, {NULL}};
const Segment SGI241931[] = {{&S97073[67], 6}, {NULL}};
const Segment SGI241932[] = {{&S96107[34], 10}, {NULL}};
const Segment SGI241933[] = {{&S96107[27], 6}, {NULL}};
const Segment SGI241934[] = {{&S96107[96], 9}, {NULL}};
const Segment SGI241935[] = {{&S96107[69], 6}, {NULL}};
const Segment SGI241936[] = {{&S96109[34], 10}, {NULL}};
const Segment SGI241937[] = {{&S96109[27], 6}, {NULL}};
const Segment SGI241938[] = {{&S96109[97], 9}, {NULL}};
const Segment SGI241939[] = {{&S96109[70], 6}, {NULL}};
const Segment SGI241940[] = {{&S96115[34], 9}, {NULL}};
const Segment SGI241941[] = {{&S96115[27], 6}, {NULL}};
const Segment SGI241942[] = {{&S96115[96], 9}, {NULL}};
const Segment SGI241943[] = {{&S96115[69], 6}, {NULL}};
const Segment SGI241944[] = {{&S96148[34], 6}, {NULL}};
const Segment SGI241945[] = {{&S96148[27], 6}, {NULL}};
const Segment SGI241946[] = {{&S96148[94], 9}, {NULL}};
const Segment SGI241947[] = {{&S96148[67], 6}, {NULL}};
const Segment SGI241948[] = {{&S96163[34], 5}, {NULL}};
const Segment SGI241949[] = {{&S96163[27], 6}, {NULL}};
const Segment SGI241950[] = {{&S96163[92], 9}, {NULL}};
const Segment SGI241951[] = {{&S96163[65], 6}, {NULL}};
const Segment SGI241952[] = {{&S96168[34], 10}, {NULL}};
const Segment SGI241953[] = {{&S96168[27], 6}, {NULL}};
const Segment SGI241954[] = {{&S96168[97], 9}, {NULL}};
const Segment SGI241955[] = {{&S96168[70], 6}, {NULL}};
const Segment SGI241956[] = {{&S97145[34], 7}, {NULL}};
const Segment SGI241957[] = {{&S97145[27], 6}, {NULL}};
const Segment SGI241958[] = {{&S97145[94], 9}, {NULL}};
const Segment SGI241959[] = {{&S97145[67], 6}, {NULL}};
const Segment SGI241960[] = {{&S96172[34], 11}, {NULL}};
const Segment SGI241961[] = {{&S96172[27], 6}, {NULL}};
const Segment SGI241962[] = {{&S96172[98], 9}, {NULL}};
const Segment SGI241963[] = {{&S96172[71], 6}, {NULL}};
const Segment SGI241964[] = {{&S96177[34], 7}, {NULL}};
const Segment SGI241965[] = {{&S96177[27], 6}, {NULL}};
const Segment SGI241966[] = {{&S96177[94], 9}, {NULL}};
const Segment SGI241967[] = {{&S96177[67], 6}, {NULL}};
const Segment SGI241968[] = {{&S96180[34], 11}, {NULL}};
const Segment SGI241969[] = {{&S96180[27], 6}, {NULL}};
const Segment SGI241970[] = {{&S96180[98], 9}, {NULL}};
const Segment SGI241971[] = {{&S96180[71], 6}, {NULL}};
const Segment SGI241972[] = {{&S97157[44], 7}, {NULL}};
const Segment SGI241973[] = {{&S97157[44], 19}, {NULL}};
const Segment SGI241974[] = {{&S97157[30], 6}, {NULL}};
const Segment SGI241975[] = {{&S97157[110], 3}, {NULL}};
const Segment SGI241976[] = {{&S97157[77], 5}, {NULL}};
const Segment SGI241977[] = {{&S96182[44], 10}, {NULL}};
const Segment SGI241978[] = {{&S96182[44], 22}, {NULL}};
const Segment SGI241979[] = {{&S96182[30], 6}, {NULL}};
const Segment SGI241980[] = {{&S96182[113], 3}, {NULL}};
const Segment SGI241981[] = {{&S96182[80], 5}, {NULL}};
const Segment SGI241982[] = {{&S96182[117], 14}, {NULL}};
const Segment SGI241983[] = {{&S96185[44], 10}, {NULL}};
const Segment SGI241984[] = {{&S96185[44], 22}, {NULL}};
const Segment SGI241985[] = {{&S96185[30], 6}, {NULL}};
const Segment SGI241986[] = {{&S96185[113], 3}, {NULL}};
const Segment SGI241987[] = {{&S96185[80], 5}, {NULL}};
const Segment SGI241988[] = {{&S96186[34], 10}, {NULL}};
const Segment SGI241989[] = {{&S96186[27], 6}, {NULL}};
const Segment SGI241990[] = {{&S96186[97], 9}, {NULL}};
const Segment SGI241991[] = {{&S96186[70], 6}, {NULL}};
const Segment SGI241992[] = {{&S103719[44], 18}, {NULL}};
const Segment SGI241993[] = {{&S103719[44], 30}, {NULL}};
const Segment SGI241994[] = {{&S103719[30], 6}, {NULL}};
const Segment SGI241995[] = {{&S96189[34], 13}, {NULL}};
const Segment SGI241996[] = {{&S96189[27], 6}, {NULL}};
const Segment SGI241997[] = {{&S103719[152], 3}, {NULL}};
const Segment SGI241998[] = {{&S103719[117], 6}, {NULL}};
const Segment SGI241999[] = {{&S96189[100], 9}, {NULL}};
const Segment SGI242000[] = {{&S96189[73], 6}, {NULL}};
const Segment SGI242001[] = {{&S103719[76], 8}, {NULL}};
const Segment SGI242002[] = {{&S103719[88], 15}, {NULL}};
const Segment SGI242003[] = {{&S96197[59], 10}, {NULL}};
const Segment SGI242004[] = {{&S96197[59], 22}, {NULL}};
const Segment SGI242005[] = {{&S96197[45], 6}, {NULL}};
const Segment SGI242006[] = {{&S96197[130], 3}, {NULL}};
const Segment SGI242007[] = {{&S96197[95], 6}, {NULL}};
const Segment SGI242008[] = {{&S96197[0], 14}, {NULL}};
const Segment SGI242009[] = {{&S96198[34], 10}, {NULL}};
const Segment SGI242010[] = {{&S96198[27], 6}, {NULL}};
const Segment SGI242011[] = {{&S96198[97], 9}, {NULL}};
const Segment SGI242012[] = {{&S96198[70], 6}, {NULL}};
const Segment SGI242013[] = {{&S96204[44], 14}, {NULL}};
const Segment SGI242014[] = {{&S96204[44], 26}, {NULL}};
const Segment SGI242015[] = {{&S96204[30], 6}, {NULL}};
const Segment SGI242016[] = {{&S96204[84], 6}, {NULL}};
const Segment SGI242017[] = {{&S96204[111], 13}, {NULL}};
const Segment SGI242018[] = {{&S96208[44], 11}, {NULL}};
const Segment SGI242019[] = {{&S96208[44], 23}, {NULL}};
const Segment SGI242020[] = {{&S96208[30], 6}, {NULL}};
const Segment SGI242021[] = {{&S96208[116], 3}, {NULL}};
const Segment SGI242022[] = {{&S96208[81], 6}, {NULL}};
const Segment SGI242023[] = {{&S96208[141], 7}, {NULL}};
const Segment SGI242024[] = {{&S97195[44], 16}, {NULL}};
const Segment SGI242025[] = {{&S97195[44], 28}, {NULL}};
const Segment SGI242026[] = {{&S97195[30], 6}, {NULL}};
const Segment SGI242027[] = {{&S97195[121], 3}, {NULL}};
const Segment SGI242028[] = {{&S97195[86], 6}, {NULL}};
const Segment SGI242029[] = {{&S97195[141], 8}, {NULL}};
const Segment SGI242030[] = {{&S97195[160], 3}, {NULL}};
const Segment SGI242031[] = {{&S88631[34], 10}, {NULL}};
const Segment SGI242032[] = {{&S88631[27], 6}, {NULL}};
const Segment SGI242033[] = {{&S97217[44], 7}, {NULL}};
const Segment SGI242034[] = {{&S97217[44], 19}, {NULL}};
const Segment SGI242035[] = {{&S97217[30], 6}, {NULL}};
const Segment SGI242036[] = {{&S88631[96], 9}, {NULL}};
const Segment SGI242037[] = {{&S88631[69], 6}, {NULL}};
const Segment SGI242038[] = {{&S97217[77], 6}, {NULL}};
const Segment SGI242039[] = {{&S97217[114], 3}, {NULL}};
const Segment SGI242040[] = {{&S103771[44], 10}, {NULL}};
const Segment SGI242041[] = {{&S103771[44], 22}, {NULL}};
const Segment SGI242042[] = {{&S103771[30], 6}, {NULL}};
const Segment SGI242043[] = {{&S103771[114], 3}, {NULL}};
const Segment SGI242044[] = {{&S103771[80], 5}, {NULL}};
const Segment SGI242045[] = {{&S103771[125], 14}, {NULL}};
const Segment SGI242046[] = {{&S103772[59], 10}, {NULL}};
const Segment SGI242047[] = {{&S103772[59], 22}, {NULL}};
const Segment SGI242048[] = {{&S103772[45], 6}, {NULL}};
const Segment SGI242049[] = {{&S103772[95], 5}, {NULL}};
const Segment SGI242050[] = {{&S103772[0], 14}, {NULL}};
const Segment SGI242051[] = {{&S88642[34], 7}, {NULL}};
const Segment SGI242052[] = {{&S88642[27], 6}, {NULL}};
const Segment SGI242053[] = {{&S88642[94], 9}, {NULL}};
const Segment SGI242054[] = {{&S88642[67], 6}, {NULL}};
const Segment SGI242055[] = {{&S103780[44], 10}, {NULL}};
const Segment SGI242056[] = {{&S103780[44], 22}, {NULL}};
const Segment SGI242057[] = {{&S103780[30], 6}, {NULL}};
const Segment SGI242058[] = {{&S103780[113], 3}, {NULL}};
const Segment SGI242059[] = {{&S103780[80], 5}, {NULL}};
const Segment SGI242060[] = {{&S103780[117], 14}, {NULL}};
const Segment SGI242061[] = {{&S103785[44], 10}, {NULL}};
const Segment SGI242062[] = {{&S103785[44], 22}, {NULL}};
const Segment SGI242063[] = {{&S103785[30], 6}, {NULL}};
const Segment SGI242064[] = {{&S103785[115], 3}, {NULL}};
const Segment SGI242065[] = {{&S103785[80], 6}, {NULL}};
const Segment SGI242066[] = {{&S103785[133], 13}, {NULL}};
const Segment SGI242067[] = {{&S103790[44], 10}, {NULL}};
const Segment SGI242068[] = {{&S103790[44], 22}, {NULL}};
const Segment SGI242069[] = {{&S103790[30], 6}, {NULL}};
const Segment SGI242070[] = {{&S103790[80], 6}, {NULL}};
const Segment SGI242071[] = {{&S103790[107], 13}, {NULL}};
const Segment SGI242072[] = {{&S103793[44], 10}, {NULL}};
const Segment SGI242073[] = {{&S103793[44], 22}, {NULL}};
const Segment SGI242074[] = {{&S103793[30], 6}, {NULL}};
const Segment SGI242075[] = {{&S103793[115], 3}, {NULL}};
const Segment SGI242076[] = {{&S103793[80], 6}, {NULL}};
const Segment SGI242077[] = {{&S88667[44], 10}, {NULL}};
const Segment SGI242078[] = {{&S88667[44], 21}, {NULL}};
const Segment SGI242079[] = {{&S88667[30], 6}, {NULL}};
const Segment SGI242080[] = {{&S103793[157], 3}, {NULL}};
const Segment SGI242081[] = {{&S88667[79], 6}, {NULL}};
const Segment SGI242082[] = {{&S88667[106], 13}, {NULL}};
const Segment SGI242083[] = {{&S88744[34], 6}, {NULL}};
const Segment SGI242084[] = {{&S88744[27], 6}, {NULL}};
const Segment SGI242085[] = {{&S88744[92], 9}, {NULL}};
const Segment SGI242086[] = {{&S88744[65], 6}, {NULL}};
const Segment SGI242087[] = {{&S97343[34], 9}, {NULL}};
const Segment SGI242088[] = {{&S97343[27], 6}, {NULL}};
const Segment SGI242089[] = {{&S97343[96], 9}, {NULL}};
const Segment SGI242090[] = {{&S97343[69], 6}, {NULL}};
const Segment SGI242091[] = {{&S88753[44], 7}, {NULL}};
const Segment SGI242092[] = {{&S88753[44], 18}, {NULL}};
const Segment SGI242093[] = {{&S88753[30], 6}, {NULL}};
const Segment SGI242094[] = {{&S88753[129], 3}, {NULL}};
const Segment SGI242095[] = {{&S88753[94], 6}, {NULL}};
const Segment SGI242096[] = {{&S88753[64], 8}, {NULL}};
const Segment SGI242097[] = {{&S88754[44], 7}, {NULL}};
const Segment SGI242098[] = {{&S88754[44], 19}, {NULL}};
const Segment SGI242099[] = {{&S88754[30], 6}, {NULL}};
const Segment SGI242100[] = {{&S88754[132], 3}, {NULL}};
const Segment SGI242101[] = {{&S88754[97], 6}, {NULL}};
const Segment SGI242102[] = {{&S88754[65], 8}, {NULL}};
const Segment SGI242103[] = {{&S88782[44], 10}, {NULL}};
const Segment SGI242104[] = {{&S88782[44], 22}, {NULL}};
const Segment SGI242105[] = {{&S88782[30], 6}, {NULL}};
const Segment SGI242106[] = {{&S88782[144], 3}, {NULL}};
const Segment SGI242107[] = {{&S88782[109], 6}, {NULL}};
const Segment SGI242108[] = {{&S88797[44], 7}, {NULL}};
const Segment SGI242109[] = {{&S88797[44], 18}, {NULL}};
const Segment SGI242110[] = {{&S88797[30], 6}, {NULL}};
const Segment SGI242111[] = {{&S88797[109], 3}, {NULL}};
const Segment SGI242112[] = {{&S88797[76], 5}, {NULL}};
const Segment SGI242113[] = {{&S96224[44], 10}, {NULL}};
const Segment SGI242114[] = {{&S96224[44], 22}, {NULL}};
const Segment SGI242115[] = {{&S96224[30], 6}, {NULL}};
const Segment SGI242116[] = {{&S96224[80], 6}, {NULL}};
const Segment SGI242117[] = {{&S88801[44], 10}, {NULL}};
const Segment SGI242118[] = {{&S96224[107], 13}, {NULL}};
const Segment SGI242119[] = {{&S88801[44], 21}, {NULL}};
const Segment SGI242120[] = {{&S88801[30], 6}, {NULL}};
const Segment SGI242121[] = {{&S88801[79], 6}, {NULL}};
const Segment SGI242122[] = {{&S88801[106], 13}, {NULL}};
const Segment SGI242123[] = {{&S96226[44], 10}, {NULL}};
const Segment SGI242124[] = {{&S96226[44], 22}, {NULL}};
const Segment SGI242125[] = {{&S96226[30], 6}, {NULL}};
const Segment SGI242126[] = {{&S96226[80], 6}, {NULL}};
const Segment SGI242127[] = {{&S96226[107], 13}, {NULL}};
const Segment SGI242128[] = {{&S88805[44], 10}, {NULL}};
const Segment SGI242129[] = {{&S88805[44], 21}, {NULL}};
const Segment SGI242130[] = {{&S88805[30], 6}, {NULL}};
const Segment SGI242131[] = {{&S88805[114], 3}, {NULL}};
const Segment SGI242132[] = {{&S88805[79], 6}, {NULL}};
const Segment SGI242133[] = {{&S88805[149], 3}, {NULL}};
const Segment SGI242134[] = {{&S96230[44], 10}, {NULL}};
const Segment SGI242135[] = {{&S96230[44], 22}, {NULL}};
const Segment SGI242136[] = {{&S96230[30], 6}, {NULL}};
const Segment SGI242137[] = {{&S96230[80], 6}, {NULL}};
const Segment SGI242138[] = {{&S96230[107], 13}, {NULL}};
const Segment SGI242139[] = {{&S96236[34], 10}, {NULL}};
const Segment SGI242140[] = {{&S96236[27], 6}, {NULL}};
const Segment SGI242141[] = {{&S96236[97], 9}, {NULL}};
const Segment SGI242142[] = {{&S96236[70], 6}, {NULL}};
const Segment SGI242143[] = {{&S88815[44], 7}, {NULL}};
const Segment SGI242144[] = {{&S88815[44], 18}, {NULL}};
const Segment SGI242145[] = {{&S88815[30], 6}, {NULL}};
const Segment SGI242146[] = {{&S88815[109], 3}, {NULL}};
const Segment SGI242147[] = {{&S88815[76], 5}, {NULL}};
const Segment SGI242148[] = {{&S96239[52], 6}, {NULL}};
const Segment SGI242149[] = {{&S96239[52], 18}, {NULL}};
const Segment SGI242150[] = {{&S96239[38], 6}, {NULL}};
const Segment SGI242151[] = {{&S96239[84], 5}, {NULL}};
const Segment SGI242152[] = {{&S96239[11], 3}, {NULL}};
const Segment SGI242153[] = {{&S88817[44], 15}, {NULL}};
const Segment SGI242154[] = {{&S88817[44], 33}, {NULL}};
const Segment SGI242155[] = {{&S88817[30], 6}, {NULL}};
const Segment SGI242156[] = {{&S88817[124], 3}, {NULL}};
const Segment SGI242157[] = {{&S88817[91], 5}, {NULL}};
const Segment SGI242158[] = {{&S96251[34], 11}, {NULL}};
const Segment SGI242159[] = {{&S96251[27], 6}, {NULL}};
const Segment SGI242160[] = {{&S96251[98], 9}, {NULL}};
const Segment SGI242161[] = {{&S96251[71], 6}, {NULL}};
const Segment SGI242162[] = {{&S96264[44], 10}, {NULL}};
const Segment SGI242163[] = {{&S96264[44], 22}, {NULL}};
const Segment SGI242164[] = {{&S96264[30], 6}, {NULL}};
const Segment SGI242165[] = {{&S96264[115], 3}, {NULL}};
const Segment SGI242166[] = {{&S96264[80], 6}, {NULL}};
const Segment SGI242167[] = {{&S96264[135], 8}, {NULL}};
const Segment SGI242168[] = {{&S96264[144], 13}, {NULL}};
const Segment SGI242169[] = {{&S96275[34], 8}, {NULL}};
const Segment SGI242170[] = {{&S96275[27], 6}, {NULL}};
const Segment SGI242171[] = {{&S96275[94], 9}, {NULL}};
const Segment SGI242172[] = {{&S96275[67], 6}, {NULL}};
const Segment SGI242173[] = {{&S104014[44], 9}, {NULL}};
const Segment SGI242174[] = {{&S104014[44], 21}, {NULL}};
const Segment SGI242175[] = {{&S104014[30], 6}, {NULL}};
const Segment SGI242176[] = {{&S104014[114], 3}, {NULL}};
const Segment SGI242177[] = {{&S104014[79], 6}, {NULL}};
const Segment SGI242178[] = {{&S104014[131], 10}, {NULL}};
const Segment SGI242179[] = {{&S96282[34], 9}, {NULL}};
const Segment SGI242180[] = {{&S96282[27], 6}, {NULL}};
const Segment SGI242181[] = {{&S96282[96], 9}, {NULL}};
const Segment SGI242182[] = {{&S96282[69], 6}, {NULL}};
const Segment SGI242183[] = {{&S88894[34], 10}, {NULL}};
const Segment SGI242184[] = {{&S88894[27], 6}, {NULL}};
const Segment SGI242185[] = {{&S88894[97], 9}, {NULL}};
const Segment SGI242186[] = {{&S88894[70], 6}, {NULL}};
const Segment SGI242187[] = {{&S88902[34], 10}, {NULL}};
const Segment SGI242188[] = {{&S88902[27], 6}, {NULL}};
const Segment SGI242189[] = {{&S88902[97], 9}, {NULL}};
const Segment SGI242190[] = {{&S88902[70], 6}, {NULL}};
const Segment SGI242191[] = {{&S97521[34], 10}, {NULL}};
const Segment SGI242192[] = {{&S97521[27], 6}, {NULL}};
const Segment SGI242193[] = {{&S97521[97], 9}, {NULL}};
const Segment SGI242194[] = {{&S97521[70], 6}, {NULL}};
const Segment SGI242195[] = {{&S88918[44], 10}, {NULL}};
const Segment SGI242196[] = {{&S88918[44], 21}, {NULL}};
const Segment SGI242197[] = {{&S88918[30], 6}, {NULL}};
const Segment SGI242198[] = {{&S88918[114], 3}, {NULL}};
const Segment SGI242199[] = {{&S88918[79], 6}, {NULL}};
const Segment SGI242200[] = {{&S104080[44], 10}, {NULL}};
const Segment SGI242201[] = {{&S104080[44], 22}, {NULL}};
const Segment SGI242202[] = {{&S104080[30], 6}, {NULL}};
const Segment SGI242203[] = {{&S104080[115], 3}, {NULL}};
const Segment SGI242204[] = {{&S104080[80], 6}, {NULL}};
const Segment SGI242205[] = {{&S104080[132], 10}, {NULL}};
const Segment SGI242206[] = {{&S104082[57], 10}, {NULL}};
const Segment SGI242207[] = {{&S104082[57], 22}, {NULL}};
const Segment SGI242208[] = {{&S104082[43], 6}, {NULL}};
const Segment SGI242209[] = {{&S104082[128], 3}, {NULL}};
const Segment SGI242210[] = {{&S104082[93], 6}, {NULL}};
const Segment SGI242211[] = {{&S104084[44], 10}, {NULL}};
const Segment SGI242212[] = {{&S104084[44], 22}, {NULL}};
const Segment SGI242213[] = {{&S104084[30], 6}, {NULL}};
const Segment SGI242214[] = {{&S104084[115], 3}, {NULL}};
const Segment SGI242215[] = {{&S104084[80], 6}, {NULL}};
const Segment SGI242216[] = {{&S104084[133], 11}, {NULL}};
const Segment SGI242217[] = {{&S104085[44], 11}, {NULL}};
const Segment SGI242218[] = {{&S104085[44], 23}, {NULL}};
const Segment SGI242219[] = {{&S104085[30], 6}, {NULL}};
const Segment SGI242220[] = {{&S104085[116], 3}, {NULL}};
const Segment SGI242221[] = {{&S104085[81], 6}, {NULL}};
const Segment SGI242222[] = {{&S104085[134], 11}, {NULL}};
const Segment SGI242223[] = {{&S104086[45], 10}, {NULL}};
const Segment SGI242224[] = {{&S104086[30], 6}, {NULL}};
const Segment SGI242225[] = {{&S104086[115], 3}, {NULL}};
const Segment SGI242226[] = {{&S104086[81], 5}, {NULL}};
const Segment SGI242227[] = {{&S104086[139], 13}, {NULL}};
const Segment SGI242228[] = {{&S104087[45], 10}, {NULL}};
const Segment SGI242229[] = {{&S104087[30], 6}, {NULL}};
const Segment SGI242230[] = {{&S104087[115], 3}, {NULL}};
const Segment SGI242231[] = {{&S104087[81], 5}, {NULL}};
const Segment SGI242232[] = {{&S104087[139], 13}, {NULL}};
const Segment SGI242233[] = {{&S104090[59], 10}, {NULL}};
const Segment SGI242234[] = {{&S104090[59], 22}, {NULL}};
const Segment SGI242235[] = {{&S104090[45], 6}, {NULL}};
const Segment SGI242236[] = {{&S104090[130], 3}, {NULL}};
const Segment SGI242237[] = {{&S104090[95], 6}, {NULL}};
const Segment SGI242238[] = {{&S88945[34], 10}, {NULL}};
const Segment SGI242239[] = {{&S88945[27], 6}, {NULL}};
const Segment SGI242240[] = {{&S104090[0], 14}, {NULL}};
const Segment SGI242241[] = {{&S88945[97], 9}, {NULL}};
const Segment SGI242242[] = {{&S88945[70], 6}, {NULL}};
const Segment SGI242243[] = {{&S88949[44], 14}, {NULL}};
const Segment SGI242244[] = {{&S88949[44], 25}, {NULL}};
const Segment SGI242245[] = {{&S88949[30], 6}, {NULL}};
const Segment SGI242246[] = {{&S88949[118], 3}, {NULL}};
const Segment SGI242247[] = {{&S88949[83], 6}, {NULL}};
const Segment SGI242248[] = {{&S88950[34], 10}, {NULL}};
const Segment SGI242249[] = {{&S88950[27], 6}, {NULL}};
const Segment SGI242250[] = {{&S104097[45], 10}, {NULL}};
const Segment SGI242251[] = {{&S88950[96], 9}, {NULL}};
const Segment SGI242252[] = {{&S104097[30], 6}, {NULL}};
const Segment SGI242253[] = {{&S88950[69], 6}, {NULL}};
const Segment SGI242254[] = {{&S104097[115], 3}, {NULL}};
const Segment SGI242255[] = {{&S104097[81], 5}, {NULL}};
const Segment SGI242256[] = {{&S104097[139], 13}, {NULL}};
const Segment SGI242257[] = {{&S104098[44], 10}, {NULL}};
const Segment SGI242258[] = {{&S104098[44], 22}, {NULL}};
const Segment SGI242259[] = {{&S104098[30], 6}, {NULL}};
const Segment SGI242260[] = {{&S104098[115], 3}, {NULL}};
const Segment SGI242261[] = {{&S104098[80], 6}, {NULL}};
const Segment SGI242262[] = {{&S104098[132], 10}, {NULL}};
const Segment SGI242263[] = {{&S104099[44], 10}, {NULL}};
const Segment SGI242264[] = {{&S104099[44], 22}, {NULL}};
const Segment SGI242265[] = {{&S104099[30], 6}, {NULL}};
const Segment SGI242266[] = {{&S97581[44], 17}, {NULL}};
const Segment SGI242267[] = {{&S97581[44], 29}, {NULL}};
const Segment SGI242268[] = {{&S97581[30], 6}, {NULL}};
const Segment SGI242269[] = {{&S104099[80], 6}, {NULL}};
const Segment SGI242270[] = {{&S104099[107], 13}, {NULL}};
const Segment SGI242271[] = {{&S97581[122], 3}, {NULL}};
const Segment SGI242272[] = {{&S97581[87], 6}, {NULL}};
const Segment SGI242273[] = {{&S104104[44], 10}, {NULL}};
const Segment SGI242274[] = {{&S104104[44], 22}, {NULL}};
const Segment SGI242275[] = {{&S104104[30], 6}, {NULL}};
const Segment SGI242276[] = {{&S104104[115], 3}, {NULL}};
const Segment SGI242277[] = {{&S104104[80], 6}, {NULL}};
const Segment SGI242278[] = {{&S104104[133], 13}, {NULL}};
const Segment SGI242279[] = {{&S104112[44], 10}, {NULL}};
const Segment SGI242280[] = {{&S104112[44], 22}, {NULL}};
const Segment SGI242281[] = {{&S104112[30], 6}, {NULL}};
const Segment SGI242282[] = {{&S104112[80], 6}, {NULL}};
const Segment SGI242283[] = {{&S104112[107], 13}, {NULL}};
const Segment SGI242284[] = {{&S97617[34], 17}, {NULL}};
const Segment SGI242285[] = {{&S97617[27], 6}, {NULL}};
const Segment SGI242286[] = {{&S97617[104], 9}, {NULL}};
const Segment SGI242287[] = {{&S97617[77], 6}, {NULL}};
const Segment SGI242288[] = {{&S89025[44], 10}, {NULL}};
const Segment SGI242289[] = {{&S89025[44], 21}, {NULL}};
const Segment SGI242290[] = {{&S89025[30], 6}, {NULL}};
const Segment SGI242291[] = {{&S89025[114], 3}, {NULL}};
const Segment SGI242292[] = {{&S89025[79], 6}, {NULL}};
const Segment SGI242293[] = {{&S89026[38], 10}, {NULL}};
const Segment SGI242294[] = {{&S89026[31], 6}, {NULL}};
const Segment SGI242295[] = {{&S89044[34], 10}, {NULL}};
const Segment SGI242296[] = {{&S89044[27], 6}, {NULL}};
const Segment SGI242297[] = {{&S89044[97], 9}, {NULL}};
const Segment SGI242298[] = {{&S89044[70], 6}, {NULL}};
const Segment SGI242299[] = {{&S89051[44], 10}, {NULL}};
const Segment SGI242300[] = {{&S89051[44], 21}, {NULL}};
const Segment SGI242301[] = {{&S89051[30], 6}, {NULL}};
const Segment SGI242302[] = {{&S89051[114], 3}, {NULL}};
const Segment SGI242303[] = {{&S89051[79], 6}, {NULL}};
const Segment SGI242304[] = {{&S89051[133], 12}, {NULL}};
const Segment SGI242305[] = {{&S89052[44], 10}, {NULL}};
const Segment SGI242306[] = {{&S89052[44], 21}, {NULL}};
const Segment SGI242307[] = {{&S89052[30], 6}, {NULL}};
const Segment SGI242308[] = {{&S89052[114], 3}, {NULL}};
const Segment SGI242309[] = {{&S89052[79], 6}, {NULL}};
const Segment SGI242310[] = {{&S89052[132], 11}, {NULL}};
const Segment SGI242311[] = {{&S96350[44], 14}, {NULL}};
const Segment SGI242312[] = {{&S96350[44], 26}, {NULL}};
const Segment SGI242313[] = {{&S96350[30], 6}, {NULL}};
const Segment SGI242314[] = {{&S96350[119], 3}, {NULL}};
const Segment SGI242315[] = {{&S96350[84], 6}, {NULL}};
const Segment SGI242316[] = {{&S89071[34], 9}, {NULL}};
const Segment SGI242317[] = {{&S89071[27], 6}, {NULL}};
const Segment SGI242318[] = {{&S89071[95], 9}, {NULL}};
const Segment SGI242319[] = {{&S89071[68], 6}, {NULL}};
const Segment SGI242320[] = {{&S89073[34], 11}, {NULL}};
const Segment SGI242321[] = {{&S89073[27], 6}, {NULL}};
const Segment SGI242322[] = {{&S89073[97], 9}, {NULL}};
const Segment SGI242323[] = {{&S89073[70], 6}, {NULL}};
const Segment SGI242324[] = {{&S96369[34], 10}, {NULL}};
const Segment SGI242325[] = {{&S96369[27], 6}, {NULL}};
const Segment SGI242326[] = {{&S96369[97], 9}, {NULL}};
const Segment SGI242327[] = {{&S96369[70], 6}, {NULL}};
const Segment SGI242328[] = {{&S96393[44], 10}, {NULL}};
const Segment SGI242329[] = {{&S96393[44], 22}, {NULL}};
const Segment SGI242330[] = {{&S96393[30], 6}, {NULL}};
const Segment SGI242331[] = {{&S96393[115], 3}, {NULL}};
const Segment SGI242332[] = {{&S96393[80], 6}, {NULL}};
const Segment SGI242333[] = {{&S96393[133], 13}, {NULL}};
const Segment SGI242334[] = {{&S96397[38], 9}, {NULL}};
const Segment SGI242335[] = {{&S96397[31], 6}, {NULL}};
const Segment SGI242336[] = {{&S96419[34], 7}, {NULL}};
const Segment SGI242337[] = {{&S96419[27], 6}, {NULL}};
const Segment SGI242338[] = {{&S96419[94], 9}, {NULL}};
const Segment SGI242339[] = {{&S96419[67], 6}, {NULL}};
const Segment SGI242340[] = {{&S96424[34], 7}, {NULL}};
const Segment SGI242341[] = {{&S96424[27], 6}, {NULL}};
const Segment SGI242342[] = {{&S96424[94], 9}, {NULL}};
const Segment SGI242343[] = {{&S96424[67], 6}, {NULL}};
const Segment SGI242344[] = {{&S96431[38], 25}, {NULL}};
const Segment SGI242345[] = {{&S96431[31], 6}, {NULL}};
const Segment SGI242346[] = {{&S89097[34], 10}, {NULL}};
const Segment SGI242347[] = {{&S97804[34], 8}, {NULL}};
const Segment SGI242348[] = {{&S97804[27], 6}, {NULL}};
const Segment SGI242349[] = {{&S89097[27], 6}, {NULL}};
const Segment SGI242350[] = {{&S97804[95], 9}, {NULL}};
const Segment SGI242351[] = {{&S97804[68], 6}, {NULL}};
const Segment SGI242352[] = {{&S89097[96], 9}, {NULL}};
const Segment SGI242353[] = {{&S89097[69], 6}, {NULL}};
const Segment SGI242354[] = {{&S89098[34], 10}, {NULL}};
const Segment SGI242355[] = {{&S89098[27], 6}, {NULL}};
const Segment SGI242356[] = {{&S89098[97], 9}, {NULL}};
const Segment SGI242357[] = {{&S89098[70], 6}, {NULL}};
const Segment SGI242358[] = {{&S96455[34], 9}, {NULL}};
const Segment SGI242359[] = {{&S96455[27], 6}, {NULL}};
const Segment SGI242360[] = {{&S96455[96], 9}, {NULL}};
const Segment SGI242361[] = {{&S96455[69], 6}, {NULL}};
const Segment SGI242362[] = {{&S104392[59], 9}, {NULL}};
const Segment SGI242363[] = {{&S104392[59], 21}, {NULL}};
const Segment SGI242364[] = {{&S104392[45], 6}, {NULL}};
const Segment SGI242365[] = {{&S104392[129], 3}, {NULL}};
const Segment SGI242366[] = {{&S104392[94], 6}, {NULL}};
const Segment SGI242367[] = {{&S104392[0], 14}, {NULL}};
const Segment SGI242368[] = {{&S96485[34], 10}, {NULL}};
const Segment SGI242369[] = {{&S96485[27], 6}, {NULL}};
const Segment SGI242370[] = {{&S96485[97], 9}, {NULL}};
const Segment SGI242371[] = {{&S96485[70], 6}, {NULL}};
const Segment SGI242372[] = {{&S96485[138], 8}, {NULL}};
const Segment SGI242373[] = {{&S96486[34], 6}, {NULL}};
const Segment SGI242374[] = {{&S96486[27], 6}, {NULL}};
const Segment SGI242375[] = {{&S96486[93], 9}, {NULL}};
const Segment SGI242376[] = {{&S96486[66], 6}, {NULL}};
const Segment SGI242377[] = {{&S96486[134], 8}, {NULL}};
const Segment SGI242378[] = {{&S89170[34], 10}, {NULL}};
const Segment SGI242379[] = {{&S89170[27], 6}, {NULL}};
const Segment SGI242380[] = {{&S89170[96], 9}, {NULL}};
const Segment SGI242381[] = {{&S89170[69], 6}, {NULL}};
const Segment SGI242382[] = {{&S89182[44], 7}, {NULL}};
const Segment SGI242383[] = {{&S89182[44], 18}, {NULL}};
const Segment SGI242384[] = {{&S89182[30], 6}, {NULL}};
const Segment SGI242385[] = {{&S89182[111], 3}, {NULL}};
const Segment SGI242386[] = {{&S89182[76], 6}, {NULL}};
const Segment SGI242387[] = {{&S96513[44], 11}, {NULL}};
const Segment SGI242388[] = {{&S96513[44], 23}, {NULL}};
const Segment SGI242389[] = {{&S96513[30], 6}, {NULL}};
const Segment SGI242390[] = {{&S96513[114], 3}, {NULL}};
const Segment SGI242391[] = {{&S96513[81], 5}, {NULL}};
const Segment SGI242392[] = {{&S96513[118], 14}, {NULL}};
const Segment SGI242393[] = {{&S89187[44], 14}, {NULL}};
const Segment SGI242394[] = {{&S89187[44], 26}, {NULL}};
const Segment SGI242395[] = {{&S89187[30], 6}, {NULL}};
const Segment SGI242396[] = {{&S89187[137], 3}, {NULL}};
const Segment SGI242397[] = {{&S89187[102], 6}, {NULL}};
const Segment SGI242398[] = {{&S96515[44], 8}, {NULL}};
const Segment SGI242399[] = {{&S96515[44], 20}, {NULL}};
const Segment SGI242400[] = {{&S89187[72], 8}, {NULL}};
const Segment SGI242401[] = {{&S96515[30], 6}, {NULL}};
const Segment SGI242402[] = {{&S96515[113], 3}, {NULL}};
const Segment SGI242403[] = {{&S96515[78], 6}, {NULL}};
const Segment SGI242404[] = {{&S96519[44], 10}, {NULL}};
const Segment SGI242405[] = {{&S96519[44], 22}, {NULL}};
const Segment SGI242406[] = {{&S96519[30], 6}, {NULL}};
const Segment SGI242407[] = {{&S96519[113], 3}, {NULL}};
const Segment SGI242408[] = {{&S96519[80], 5}, {NULL}};
const Segment SGI242409[] = {{&S96519[117], 14}, {NULL}};
const Segment SGI242410[] = {{&S96527[34], 10}, {NULL}};
const Segment SGI242411[] = {{&S96527[27], 6}, {NULL}};
const Segment SGI242412[] = {{&S96527[97], 9}, {NULL}};
const Segment SGI242413[] = {{&S96527[70], 6}, {NULL}};
const Segment SGI242414[] = {{&S89206[34], 12}, {NULL}};
const Segment SGI242415[] = {{&S89206[27], 6}, {NULL}};
const Segment SGI242416[] = {{&S89206[98], 9}, {NULL}};
const Segment SGI242417[] = {{&S89206[71], 6}, {NULL}};
const Segment SGI242418[] = {{&S96537[34], 7}, {NULL}};
const Segment SGI242419[] = {{&S96537[27], 6}, {NULL}};
const Segment SGI242420[] = {{&S96537[94], 9}, {NULL}};
const Segment SGI242421[] = {{&S96537[67], 6}, {NULL}};
const Segment SGI242422[] = {{&S96540[34], 6}, {NULL}};
const Segment SGI242423[] = {{&S96540[27], 6}, {NULL}};
const Segment SGI242424[] = {{&S96540[94], 9}, {NULL}};
const Segment SGI242425[] = {{&S96540[67], 6}, {NULL}};
const Segment SGI242426[] = {{&S96559[34], 10}, {NULL}};
const Segment SGI242427[] = {{&S96559[27], 6}, {NULL}};
const Segment SGI242428[] = {{&S96559[97], 9}, {NULL}};
const Segment SGI242429[] = {{&S96559[70], 6}, {NULL}};
const Segment SGI242430[] = {{&S96561[52], 9}, {NULL}};
const Segment SGI242431[] = {{&S96561[52], 21}, {NULL}};
const Segment SGI242432[] = {{&S96561[38], 6}, {NULL}};
const Segment SGI242433[] = {{&S96561[87], 5}, {NULL}};
const Segment SGI242434[] = {{&S96561[11], 3}, {NULL}};
const Segment SGI242435[] = {{&S104481[59], 10}, {NULL}};
const Segment SGI242436[] = {{&S104481[59], 22}, {NULL}};
const Segment SGI242437[] = {{&S104481[45], 6}, {NULL}};
const Segment SGI242438[] = {{&S104481[130], 3}, {NULL}};
const Segment SGI242439[] = {{&S104481[95], 6}, {NULL}};
const Segment SGI242440[] = {{&S104481[0], 14}, {NULL}};
const Segment SGI242441[] = {{&S104482[59], 10}, {NULL}};
const Segment SGI242442[] = {{&S104482[59], 22}, {NULL}};
const Segment SGI242443[] = {{&S104482[45], 6}, {NULL}};
const Segment SGI242444[] = {{&S104482[130], 3}, {NULL}};
const Segment SGI242445[] = {{&S104482[95], 6}, {NULL}};
const Segment SGI242446[] = {{&S104482[0], 14}, {NULL}};
const Segment SGI242447[] = {{&S96567[34], 9}, {NULL}};
const Segment SGI242448[] = {{&S96567[27], 6}, {NULL}};
const Segment SGI242449[] = {{&S96567[96], 9}, {NULL}};
const Segment SGI242450[] = {{&S96567[69], 6}, {NULL}};
const Segment SGI242451[] = {{&S104493[44], 10}, {NULL}};
const Segment SGI242452[] = {{&S104493[44], 22}, {NULL}};
const Segment SGI242453[] = {{&S104493[30], 6}, {NULL}};
const Segment SGI242454[] = {{&S104493[113], 3}, {NULL}};
const Segment SGI242455[] = {{&S104493[80], 5}, {NULL}};
const Segment SGI242456[] = {{&S104493[117], 14}, {NULL}};
const Segment SGI242457[] = {{&S97851[34], 15}, {NULL}};
const Segment SGI242458[] = {{&S97851[27], 6}, {NULL}};
const Segment SGI242459[] = {{&S97851[102], 9}, {NULL}};
const Segment SGI242460[] = {{&S97851[75], 6}, {NULL}};
const Segment SGI242461[] = {{&S96589[34], 7}, {NULL}};
const Segment SGI242462[] = {{&S96589[27], 6}, {NULL}};
const Segment SGI242463[] = {{&S96589[99], 9}, {NULL}};
const Segment SGI242464[] = {{&S96589[72], 6}, {NULL}};
const Segment SGI242465[] = {{&S104505[44], 10}, {NULL}};
const Segment SGI242466[] = {{&S104505[44], 22}, {NULL}};
const Segment SGI242467[] = {{&S104505[30], 6}, {NULL}};
const Segment SGI242468[] = {{&S104505[113], 3}, {NULL}};
const Segment SGI242469[] = {{&S104505[80], 5}, {NULL}};
const Segment SGI242470[] = {{&S104505[117], 14}, {NULL}};
const Segment SGI242471[] = {{&S89283[34], 10}, {NULL}};
const Segment SGI242472[] = {{&S89283[27], 6}, {NULL}};
const Segment SGI242473[] = {{&S89283[96], 9}, {NULL}};
const Segment SGI242474[] = {{&S89283[69], 6}, {NULL}};
const Segment SGI242475[] = {{&S89312[34], 23}, {NULL}};
const Segment SGI242476[] = {{&S89312[27], 6}, {NULL}};
const Segment SGI242477[] = {{&S89312[110], 9}, {NULL}};
const Segment SGI242478[] = {{&S89312[83], 6}, {NULL}};
const Segment SGI242479[] = {{&S89316[34], 11}, {NULL}};
const Segment SGI242480[] = {{&S89316[27], 6}, {NULL}};
const Segment SGI242481[] = {{&S89316[97], 9}, {NULL}};
const Segment SGI242482[] = {{&S89316[70], 6}, {NULL}};
const Segment SGI242483[] = {{&S97908[34], 17}, {NULL}};
const Segment SGI242484[] = {{&S97908[27], 6}, {NULL}};
const Segment SGI242485[] = {{&S97908[104], 9}, {NULL}};
const Segment SGI242486[] = {{&S97908[77], 6}, {NULL}};
const Segment SGI242487[] = {{&S89318[44], 10}, {NULL}};
const Segment SGI242488[] = {{&S89318[44], 21}, {NULL}};
const Segment SGI242489[] = {{&S89318[30], 6}, {NULL}};
const Segment SGI242490[] = {{&S89318[114], 3}, {NULL}};
const Segment SGI242491[] = {{&S89318[79], 6}, {NULL}};
const Segment SGI242492[] = {{&S89323[34], 8}, {NULL}};
const Segment SGI242493[] = {{&S89323[27], 6}, {NULL}};
const Segment SGI242494[] = {{&S89323[95], 9}, {NULL}};
const Segment SGI242495[] = {{&S89323[68], 6}, {NULL}};
const Segment SGI242496[] = {{&S96664[45], 9}, {NULL}};
const Segment SGI242497[] = {{&S96664[30], 6}, {NULL}};
const Segment SGI242498[] = {{&S96664[114], 3}, {NULL}};
const Segment SGI242499[] = {{&S96664[80], 5}, {NULL}};
const Segment SGI242500[] = {{&S96664[138], 13}, {NULL}};
const Segment SGI242501[] = {{&S89351[44], 5}, {NULL}};
const Segment SGI242502[] = {{&S89351[44], 17}, {NULL}};
const Segment SGI242503[] = {{&S89351[30], 6}, {NULL}};
const Segment SGI242504[] = {{&S89351[110], 3}, {NULL}};
const Segment SGI242505[] = {{&S89351[75], 6}, {NULL}};
const Segment SGI242506[] = {{&S89351[142], 15}, {NULL}};
const Segment SGI242507[] = {{&S97943[44], 7}, {NULL}};
const Segment SGI242508[] = {{&S97943[44], 19}, {NULL}};
const Segment SGI242509[] = {{&S97943[30], 6}, {NULL}};
const Segment SGI242510[] = {{&S97943[77], 6}, {NULL}};
const Segment SGI242511[] = {{&S97943[114], 3}, {NULL}};
const Segment SGI242512[] = {{&S96699[44], 10}, {NULL}};
const Segment SGI242513[] = {{&S96699[44], 22}, {NULL}};
const Segment SGI242514[] = {{&S96699[30], 6}, {NULL}};
const Segment SGI242515[] = {{&S96699[115], 3}, {NULL}};
const Segment SGI242516[] = {{&S96699[80], 6}, {NULL}};
const Segment SGI242517[] = {{&S96699[135], 8}, {NULL}};
const Segment SGI242518[] = {{&S96699[144], 13}, {NULL}};
const Segment SGI242519[] = {{&S89377[34], 15}, {NULL}};
const Segment SGI242520[] = {{&S89377[27], 6}, {NULL}};
const Segment SGI242521[] = {{&S89377[108], 9}, {NULL}};
const Segment SGI242522[] = {{&S89377[81], 6}, {NULL}};
const Segment SGI242523[] = {{&S97968[44], 15}, {NULL}};
const Segment SGI242524[] = {{&S97968[44], 27}, {NULL}};
const Segment SGI242525[] = {{&S97968[30], 6}, {NULL}};
const Segment SGI242526[] = {{&S97968[120], 3}, {NULL}};
const Segment SGI242527[] = {{&S97968[85], 6}, {NULL}};
const Segment SGI242528[] = {{&S97968[155], 3}, {NULL}};
const Segment SGI242529[] = {{&S96711[34], 10}, {NULL}};
const Segment SGI242530[] = {{&S96711[27], 6}, {NULL}};
const Segment SGI242531[] = {{&S96711[97], 9}, {NULL}};
const Segment SGI242532[] = {{&S96711[70], 6}, {NULL}};
const Segment SGI242533[] = {{&S97983[34], 11}, {NULL}};
const Segment SGI242534[] = {{&S97983[27], 6}, {NULL}};
const Segment SGI242535[] = {{&S97983[98], 9}, {NULL}};
const Segment SGI242536[] = {{&S97983[71], 6}, {NULL}};
const Segment SGI242537[] = {{&S96722[34], 6}, {NULL}};
const Segment SGI242538[] = {{&S96722[27], 6}, {NULL}};
const Segment SGI242539[] = {{&S96722[93], 9}, {NULL}};
const Segment SGI242540[] = {{&S96722[66], 6}, {NULL}};
const Segment SGI242541[] = {{&S96726[34], 10}, {NULL}};
const Segment SGI242542[] = {{&S96726[27], 6}, {NULL}};
const Segment SGI242543[] = {{&S96726[97], 9}, {NULL}};
const Segment SGI242544[] = {{&S96726[70], 6}, {NULL}};
const Segment SGI242545[] = {{&S97993[44], 16}, {NULL}};
const Segment SGI242546[] = {{&S97993[44], 28}, {NULL}};
const Segment SGI242547[] = {{&S97993[30], 6}, {NULL}};
const Segment SGI242548[] = {{&S97993[121], 3}, {NULL}};
const Segment SGI242549[] = {{&S97993[86], 6}, {NULL}};
const Segment SGI242550[] = {{&S97993[141], 8}, {NULL}};
const Segment SGI242551[] = {{&S97993[160], 3}, {NULL}};
const Segment SGI242552[] = {{&S97999[34], 25}, {NULL}};
const Segment SGI242553[] = {{&S97999[27], 6}, {NULL}};
const Segment SGI242554[] = {{&S97999[112], 9}, {NULL}};
const Segment SGI242555[] = {{&S97999[85], 6}, {NULL}};
const Segment SGI242556[] = {{&S104519[44], 10}, {NULL}};
const Segment SGI242557[] = {{&S104519[44], 22}, {NULL}};
const Segment SGI242558[] = {{&S104519[30], 6}, {NULL}};
const Segment SGI242559[] = {{&S104519[115], 3}, {NULL}};
const Segment SGI242560[] = {{&S104519[80], 6}, {NULL}};
const Segment SGI242561[] = {{&S104519[133], 13}, {NULL}};
const Segment SGI242562[] = {{&S96746[34], 7}, {NULL}};
const Segment SGI242563[] = {{&S96746[27], 6}, {NULL}};
const Segment SGI242564[] = {{&S96746[98], 9}, {NULL}};
const Segment SGI242565[] = {{&S96746[71], 6}, {NULL}};
const Segment SGI242566[] = {{&S96750[44], 18}, {NULL}};
const Segment SGI242567[] = {{&S96750[44], 30}, {NULL}};
const Segment SGI242568[] = {{&S96750[30], 6}, {NULL}};
const Segment SGI242569[] = {{&S96750[123], 3}, {NULL}};
const Segment SGI242570[] = {{&S96750[88], 6}, {NULL}};
const Segment SGI242571[] = {{&S98020[34], 7}, {NULL}};
const Segment SGI242572[] = {{&S98020[27], 6}, {NULL}};
const Segment SGI242573[] = {{&S98020[94], 9}, {NULL}};
const Segment SGI242574[] = {{&S98020[67], 6}, {NULL}};
const Segment SGI242575[] = {{&S96759[34], 10}, {NULL}};
const Segment SGI242576[] = {{&S96759[27], 6}, {NULL}};
const Segment SGI242577[] = {{&S96759[97], 9}, {NULL}};
const Segment SGI242578[] = {{&S96759[70], 6}, {NULL}};
const Segment SGI242579[] = {{&S98031[34], 8}, {NULL}};
const Segment SGI242580[] = {{&S98031[27], 6}, {NULL}};
const Segment SGI242581[] = {{&S98031[95], 9}, {NULL}};
const Segment SGI242582[] = {{&S98031[68], 6}, {NULL}};
const Segment SGI242583[] = {{&S98031[129], 8}, {NULL}};
const Segment SGI242584[] = {{&S96771[34], 8}, {NULL}};
const Segment SGI242585[] = {{&S96771[27], 6}, {NULL}};
const Segment SGI242586[] = {{&S96771[95], 9}, {NULL}};
const Segment SGI242587[] = {{&S96771[68], 6}, {NULL}};
const Segment SGI242588[] = {{&S96775[34], 10}, {NULL}};
const Segment SGI242589[] = {{&S96775[27], 6}, {NULL}};
const Segment SGI242590[] = {{&S96775[97], 9}, {NULL}};
const Segment SGI242591[] = {{&S96775[70], 6}, {NULL}};
const Segment SGI242592[] = {{&S96777[34], 7}, {NULL}};
const Segment SGI242593[] = {{&S96777[27], 6}, {NULL}};
const Segment SGI242594[] = {{&S96777[100], 9}, {NULL}};
const Segment SGI242595[] = {{&S96777[73], 6}, {NULL}};
const Segment SGI242596[] = {{&S96780[34], 9}, {NULL}};
const Segment SGI242597[] = {{&S96780[27], 6}, {NULL}};
const Segment SGI242598[] = {{&S96780[96], 9}, {NULL}};
const Segment SGI242599[] = {{&S96780[69], 6}, {NULL}};
const Segment SGI242600[] = {{&S96792[59], 10}, {NULL}};
const Segment SGI242601[] = {{&S96792[59], 22}, {NULL}};
const Segment SGI242602[] = {{&S96792[45], 6}, {NULL}};
const Segment SGI242603[] = {{&S96792[130], 3}, {NULL}};
const Segment SGI242604[] = {{&S96792[95], 6}, {NULL}};
const Segment SGI242605[] = {{&S96792[0], 14}, {NULL}};
const Segment SGI242606[] = {{&S96804[44], 7}, {NULL}};
const Segment SGI242607[] = {{&S96804[44], 29}, {NULL}};
const Segment SGI242608[] = {{&S96804[30], 6}, {NULL}};
const Segment SGI242609[] = {{&S96804[122], 3}, {NULL}};
const Segment SGI242610[] = {{&S96804[87], 6}, {NULL}};
const Segment SGI242611[] = {{&S96804[147], 7}, {NULL}};
const Segment SGI242612[] = {{&S96806[34], 12}, {NULL}};
const Segment SGI242613[] = {{&S96806[27], 6}, {NULL}};
const Segment SGI242614[] = {{&S96806[98], 9}, {NULL}};
const Segment SGI242615[] = {{&S96806[71], 6}, {NULL}};
const Segment SGI242616[] = {{&S96807[34], 10}, {NULL}};
const Segment SGI242617[] = {{&S96807[27], 6}, {NULL}};
const Segment SGI242618[] = {{&S96807[97], 9}, {NULL}};
const Segment SGI242619[] = {{&S96807[70], 6}, {NULL}};
const Segment SGI242620[] = {{&S96807[138], 8}, {NULL}};
const Segment SGI242621[] = {{&S96811[34], 7}, {NULL}};
const Segment SGI242622[] = {{&S96811[27], 6}, {NULL}};
const Segment SGI242623[] = {{&S96811[93], 9}, {NULL}};
const Segment SGI242624[] = {{&S96811[66], 6}, {NULL}};
const Segment SGI242625[] = {{&S96818[44], 16}, {NULL}};
const Segment SGI242626[] = {{&S96818[44], 35}, {NULL}};
const Segment SGI242627[] = {{&S96818[30], 6}, {NULL}};
const Segment SGI242628[] = {{&S96818[93], 6}, {NULL}};
const Segment SGI242629[] = {{&S96818[120], 13}, {NULL}};
const Segment SGI242630[] = {{&S96830[44], 9}, {NULL}};
const Segment SGI242631[] = {{&S96830[44], 21}, {NULL}};
const Segment SGI242632[] = {{&S96830[30], 6}, {NULL}};
const Segment SGI242633[] = {{&S96830[79], 6}, {NULL}};
const Segment SGI242634[] = {{&S96830[106], 13}, {NULL}};
const Segment SGI242635[] = {{&S96840[34], 7}, {NULL}};
const Segment SGI242636[] = {{&S96840[27], 6}, {NULL}};
const Segment SGI242637[] = {{&S96840[94], 9}, {NULL}};
const Segment SGI242638[] = {{&S96840[67], 6}, {NULL}};
const Segment SGI242639[] = {{&S96844[44], 10}, {NULL}};
const Segment SGI242640[] = {{&S96844[44], 22}, {NULL}};
const Segment SGI242641[] = {{&S96844[30], 6}, {NULL}};
const Segment SGI242642[] = {{&S96844[115], 3}, {NULL}};
const Segment SGI242643[] = {{&S96844[80], 6}, {NULL}};
const Segment SGI242644[] = {{&S96844[150], 3}, {NULL}};
const Segment SGI242645[] = {{&S96849[44], 10}, {NULL}};
const Segment SGI242646[] = {{&S96849[44], 22}, {NULL}};
const Segment SGI242647[] = {{&S96849[30], 6}, {NULL}};
const Segment SGI242648[] = {{&S96849[191], 3}, {NULL}};
const Segment SGI242649[] = {{&S96849[156], 6}, {NULL}};
const Segment SGI242650[] = {{&S96849[68], 8}, {NULL}};
const Segment SGI242651[] = {{&S96850[44], 10}, {NULL}};
const Segment SGI242652[] = {{&S96850[44], 43}, {NULL}};
const Segment SGI242653[] = {{&S96850[30], 6}, {NULL}};
const Segment SGI242654[] = {{&S96850[136], 3}, {NULL}};
const Segment SGI242655[] = {{&S96850[101], 6}, {NULL}};
const Segment SGI242656[] = {{&S96855[34], 25}, {NULL}};
const Segment SGI242657[] = {{&S96855[27], 6}, {NULL}};
const Segment SGI242658[] = {{&S96855[112], 9}, {NULL}};
const Segment SGI242659[] = {{&S96855[85], 6}, {NULL}};
const Segment SGI242660[] = {{&S96858[44], 10}, {NULL}};
const Segment SGI242661[] = {{&S96858[44], 22}, {NULL}};
const Segment SGI242662[] = {{&S96858[30], 6}, {NULL}};
const Segment SGI242663[] = {{&S96858[115], 3}, {NULL}};
const Segment SGI242664[] = {{&S96858[80], 6}, {NULL}};
const Segment SGI242665[] = {{&S96858[150], 3}, {NULL}};
const Segment SGI242666[] = {{&S96864[44], 10}, {NULL}};
const Segment SGI242667[] = {{&S96864[44], 22}, {NULL}};
const Segment SGI242668[] = {{&S96864[30], 6}, {NULL}};
const Segment SGI242669[] = {{&S96864[113], 3}, {NULL}};
const Segment SGI242670[] = {{&S96864[80], 5}, {NULL}};
const Segment SGI242671[] = {{&S96864[117], 14}, {NULL}};
const Segment SGI242672[] = {{&S96869[44], 10}, {NULL}};
const Segment SGI242673[] = {{&S96869[44], 22}, {NULL}};
const Segment SGI242674[] = {{&S96869[30], 6}, {NULL}};
const Segment SGI242675[] = {{&S96869[115], 3}, {NULL}};
const Segment SGI242676[] = {{&S96869[80], 6}, {NULL}};
const Segment SGI242677[] = {{&S96873[44], 6}, {NULL}};
const Segment SGI242678[] = {{&S96873[44], 18}, {NULL}};
const Segment SGI242679[] = {{&S96873[30], 6}, {NULL}};
const Segment SGI242680[] = {{&S96873[111], 3}, {NULL}};
const Segment SGI242681[] = {{&S96873[76], 6}, {NULL}};
const Segment SGI242682[] = {{&S96876[44], 7}, {NULL}};
const Segment SGI242683[] = {{&S96876[44], 24}, {NULL}};
const Segment SGI242684[] = {{&S96876[30], 6}, {NULL}};
const Segment SGI242685[] = {{&S96876[82], 6}, {NULL}};
const Segment SGI242686[] = {{&S96876[109], 13}, {NULL}};
const Segment SGI242687[] = {{&S96880[44], 10}, {NULL}};
const Segment SGI242688[] = {{&S96880[44], 22}, {NULL}};
const Segment SGI242689[] = {{&S96880[30], 6}, {NULL}};
const Segment SGI242690[] = {{&S96880[115], 3}, {NULL}};
const Segment SGI242691[] = {{&S96880[80], 6}, {NULL}};
const Segment SGI242692[] = {{&S96899[44], 10}, {NULL}};
const Segment SGI242693[] = {{&S96899[44], 22}, {NULL}};
const Segment SGI242694[] = {{&S96899[30], 6}, {NULL}};
const Segment SGI242695[] = {{&S96899[113], 3}, {NULL}};
const Segment SGI242696[] = {{&S96899[80], 5}, {NULL}};
const Segment SGI242697[] = {{&S96899[117], 14}, {NULL}};
const Segment SGI242698[] = {{&S96900[34], 9}, {NULL}};
const Segment SGI242699[] = {{&S96900[27], 6}, {NULL}};
const Segment SGI242700[] = {{&S96900[96], 9}, {NULL}};
const Segment SGI242701[] = {{&S96900[69], 6}, {NULL}};
const Segment SGI242702[] = {{&S96900[130], 8}, {NULL}};
const Segment SGI242703[] = {{&S96902[34], 10}, {NULL}};
const Segment SGI242704[] = {{&S96902[27], 6}, {NULL}};
const Segment SGI242705[] = {{&S96902[97], 9}, {NULL}};
const Segment SGI242706[] = {{&S96902[70], 6}, {NULL}};
const Segment SGI242707[] = {{&S96903[34], 10}, {NULL}};
const Segment SGI242708[] = {{&S96903[27], 6}, {NULL}};
const Segment SGI242709[] = {{&S96903[97], 9}, {NULL}};
const Segment SGI242710[] = {{&S96903[70], 6}, {NULL}};
const Segment SGI242711[] = {{&S96904[34], 9}, {NULL}};
const Segment SGI242712[] = {{&S96904[27], 6}, {NULL}};
const Segment SGI242713[] = {{&S96904[96], 9}, {NULL}};
const Segment SGI242714[] = {{&S96904[69], 6}, {NULL}};
const Segment SGI242715[] = {{&S89515[34], 7}, {NULL}};
const Segment SGI242716[] = {{&S89515[27], 6}, {NULL}};
const Segment SGI242717[] = {{&S89515[94], 9}, {NULL}};
const Segment SGI242718[] = {{&S89515[67], 6}, {NULL}};
const Segment SGI242719[] = {{&S89518[44], 11}, {NULL}};
const Segment SGI242720[] = {{&S89518[44], 28}, {NULL}};
const Segment SGI242721[] = {{&S89518[30], 6}, {NULL}};
const Segment SGI242722[] = {{&S89518[120], 3}, {NULL}};
const Segment SGI242723[] = {{&S89518[86], 5}, {NULL}};
const Segment SGI242724[] = {{&S89520[44], 10}, {NULL}};
const Segment SGI242725[] = {{&S89520[44], 21}, {NULL}};
const Segment SGI242726[] = {{&S89520[30], 6}, {NULL}};
const Segment SGI242727[] = {{&S89520[112], 3}, {NULL}};
const Segment SGI242728[] = {{&S89520[79], 5}, {NULL}};
const Segment SGI242729[] = {{&S89540[44], 10}, {NULL}};
const Segment SGI242730[] = {{&S89540[44], 21}, {NULL}};
const Segment SGI242731[] = {{&S89540[30], 6}, {NULL}};
const Segment SGI242732[] = {{&S89540[79], 6}, {NULL}};
const Segment SGI242733[] = {{&S89540[106], 13}, {NULL}};
const Segment SGI242734[] = {{&S89542[34], 10}, {NULL}};
const Segment SGI242735[] = {{&S89542[27], 6}, {NULL}};
const Segment SGI242736[] = {{&S89542[96], 9}, {NULL}};
const Segment SGI242737[] = {{&S89542[69], 6}, {NULL}};
const Segment SGI242738[] = {{&S97009[44], 7}, {NULL}};
const Segment SGI242739[] = {{&S97009[44], 19}, {NULL}};
const Segment SGI242740[] = {{&S97009[30], 6}, {NULL}};
const Segment SGI242741[] = {{&S97009[112], 3}, {NULL}};
const Segment SGI242742[] = {{&S97009[77], 6}, {NULL}};
const Segment SGI242743[] = {{&S97022[34], 18}, {NULL}};
const Segment SGI242744[] = {{&S97022[27], 6}, {NULL}};
const Segment SGI242745[] = {{&S97022[105], 9}, {NULL}};
const Segment SGI242746[] = {{&S97022[78], 6}, {NULL}};
const Segment SGI242747[] = {{&S104890[44], 10}, {NULL}};
const Segment SGI242748[] = {{&S104890[44], 22}, {NULL}};
const Segment SGI242749[] = {{&S104890[30], 6}, {NULL}};
const Segment SGI242750[] = {{&S104890[131], 3}, {NULL}};
const Segment SGI242751[] = {{&S104890[96], 6}, {NULL}};
const Segment SGI242752[] = {{&S104890[159], 3}, {NULL}};
const Segment SGI242753[] = {{&S104890[68], 8}, {NULL}};
const Segment SGI242754[] = {{&S89643[34], 11}, {NULL}};
const Segment SGI242755[] = {{&S89643[27], 6}, {NULL}};
const Segment SGI242756[] = {{&S89643[97], 9}, {NULL}};
const Segment SGI242757[] = {{&S89643[70], 6}, {NULL}};
const Segment SGI242758[] = {{&S89643[138], 8}, {NULL}};
const Segment SGI242759[] = {{&S89655[44], 10}, {NULL}};
const Segment SGI242760[] = {{&S89655[44], 21}, {NULL}};
const Segment SGI242761[] = {{&S89655[30], 6}, {NULL}};
const Segment SGI242762[] = {{&S89655[79], 6}, {NULL}};
const Segment SGI242763[] = {{&S89655[106], 13}, {NULL}};
const Segment SGI242764[] = {{&S97075[44], 10}, {NULL}};
const Segment SGI242765[] = {{&S97075[44], 22}, {NULL}};
const Segment SGI242766[] = {{&S97075[30], 6}, {NULL}};
const Segment SGI242767[] = {{&S97075[135], 3}, {NULL}};
const Segment SGI242768[] = {{&S97075[100], 6}, {NULL}};
const Segment SGI242769[] = {{&S97075[68], 8}, {NULL}};
const Segment SGI242770[] = {{&S89664[34], 12}, {NULL}};
const Segment SGI242771[] = {{&S89664[27], 6}, {NULL}};
const Segment SGI242772[] = {{&S89664[99], 9}, {NULL}};
const Segment SGI242773[] = {{&S89664[72], 6}, {NULL}};
const Segment SGI242774[] = {{&S97084[34], 16}, {NULL}};
const Segment SGI242775[] = {{&S97084[27], 6}, {NULL}};
const Segment SGI242776[] = {{&S97084[103], 9}, {NULL}};
const Segment SGI242777[] = {{&S97084[76], 6}, {NULL}};
const Segment SGI242778[] = {{&S104921[44], 10}, {NULL}};
const Segment SGI242779[] = {{&S104921[44], 22}, {NULL}};
const Segment SGI242780[] = {{&S104921[30], 6}, {NULL}};
const Segment SGI242781[] = {{&S104921[115], 3}, {NULL}};
const Segment SGI242782[] = {{&S104921[80], 6}, {NULL}};
const Segment SGI242783[] = {{&S104921[143], 3}, {NULL}};
const Segment SGI242784[] = {{&S89679[34], 7}, {NULL}};
const Segment SGI242785[] = {{&S89679[27], 6}, {NULL}};
const Segment SGI242786[] = {{&S89679[94], 9}, {NULL}};
const Segment SGI242787[] = {{&S89679[67], 6}, {NULL}};
const Segment SGI242788[] = {{&S89692[34], 12}, {NULL}};
const Segment SGI242789[] = {{&S89692[27], 6}, {NULL}};
const Segment SGI242790[] = {{&S89692[98], 9}, {NULL}};
const Segment SGI242791[] = {{&S89692[71], 6}, {NULL}};
const Segment SGI242792[] = {{&S104972[44], 10}, {NULL}};
const Segment SGI242793[] = {{&S104972[44], 22}, {NULL}};
const Segment SGI242794[] = {{&S104972[30], 6}, {NULL}};
const Segment SGI242795[] = {{&S104972[115], 3}, {NULL}};
const Segment SGI242796[] = {{&S104972[80], 6}, {NULL}};
const Segment SGI242797[] = {{&S104972[133], 11}, {NULL}};
const Segment SGI242798[] = {{&S104975[31], 8}, {NULL}};
const Segment SGI242799[] = {{&S104975[55], 3}, {NULL}};
const Segment SGI242800[] = {{&S104976[39], 8}, {NULL}};
const Segment SGI242801[] = {{&S104976[104], 3}, {NULL}};
const Segment SGI242802[] = {{&S104976[93], 3}, {NULL}};
const Segment SGI242803[] = {{&S104976[70], 6}, {NULL}};
const Segment SGI242804[] = {{&S104981[31], 8}, {NULL}};
const Segment SGI242805[] = {{&S104981[55], 3}, {NULL}};
const Segment SGI242806[] = {{&S104995[34], 10}, {NULL}};
const Segment SGI242807[] = {{&S104995[27], 6}, {NULL}};
const Segment SGI242808[] = {{&S104995[97], 9}, {NULL}};
const Segment SGI242809[] = {{&S104995[70], 6}, {NULL}};
const Segment SGI242810[] = {{&S104998[59], 10}, {NULL}};
const Segment SGI242811[] = {{&S104998[59], 22}, {NULL}};
const Segment SGI242812[] = {{&S104998[45], 6}, {NULL}};
const Segment SGI242813[] = {{&S104998[130], 3}, {NULL}};
const Segment SGI242814[] = {{&S104998[95], 6}, {NULL}};
const Segment SGI242815[] = {{&S104998[0], 14}, {NULL}};
const Segment SGI242816[] = {{&S89774[30], 6}, {NULL}};
const Segment SGI242817[] = {{&S89774[85], 9}, {NULL}};
const Segment SGI242818[] = {{&S89774[57], 6}, {NULL}};
const Segment SGI242819[] = {{&S89779[44], 10}, {NULL}};
const Segment SGI242820[] = {{&S89779[44], 21}, {NULL}};
const Segment SGI242821[] = {{&S89779[30], 6}, {NULL}};
const Segment SGI242822[] = {{&S89779[132], 3}, {NULL}};
const Segment SGI242823[] = {{&S89779[97], 6}, {NULL}};
const Segment SGI242824[] = {{&S89779[67], 8}, {NULL}};
const Segment SGI242825[] = {{&S89780[44], 10}, {NULL}};
const Segment SGI242826[] = {{&S89780[44], 22}, {NULL}};
const Segment SGI242827[] = {{&S89780[30], 6}, {NULL}};
const Segment SGI242828[] = {{&S89780[115], 3}, {NULL}};
const Segment SGI242829[] = {{&S89780[80], 6}, {NULL}};
const Segment SGI242830[] = {{&S89780[133], 8}, {NULL}};
const Segment SGI242831[] = {{&S105009[59], 10}, {NULL}};
const Segment SGI242832[] = {{&S105009[59], 22}, {NULL}};
const Segment SGI242833[] = {{&S105009[45], 6}, {NULL}};
const Segment SGI242834[] = {{&S105009[130], 3}, {NULL}};
const Segment SGI242835[] = {{&S105009[95], 6}, {NULL}};
const Segment SGI242836[] = {{&S105009[0], 14}, {NULL}};
const Segment SGI242837[] = {{&S105016[44], 10}, {NULL}};
const Segment SGI242838[] = {{&S105016[44], 22}, {NULL}};
const Segment SGI242839[] = {{&S105016[30], 6}, {NULL}};
const Segment SGI242840[] = {{&S105016[115], 3}, {NULL}};
const Segment SGI242841[] = {{&S105016[80], 6}, {NULL}};
const Segment SGI242842[] = {{&S105016[157], 3}, {NULL}};
const Segment SGI242843[] = {{&S89799[44], 10}, {NULL}};
const Segment SGI242844[] = {{&S89799[44], 21}, {NULL}};
const Segment SGI242845[] = {{&S89799[30], 6}, {NULL}};
const Segment SGI242846[] = {{&S89799[114], 3}, {NULL}};
const Segment SGI242847[] = {{&S89799[79], 6}, {NULL}};
const Segment SGI242848[] = {{&S89799[142], 3}, {NULL}};
const Segment SGI242849[] = {{&S105025[44], 10}, {NULL}};
const Segment SGI242850[] = {{&S105025[44], 22}, {NULL}};
const Segment SGI242851[] = {{&S105025[30], 6}, {NULL}};
const Segment SGI242852[] = {{&S105025[113], 3}, {NULL}};
const Segment SGI242853[] = {{&S105025[80], 5}, {NULL}};
const Segment SGI242854[] = {{&S105025[117], 14}, {NULL}};
const Segment SGI242855[] = {{&S89809[34], 10}, {NULL}};
const Segment SGI242856[] = {{&S89809[27], 6}, {NULL}};
const Segment SGI242857[] = {{&S89809[97], 9}, {NULL}};
const Segment SGI242858[] = {{&S89809[70], 6}, {NULL}};
const Segment SGI242859[] = {{&S89811[34], 10}, {NULL}};
const Segment SGI242860[] = {{&S89811[27], 6}, {NULL}};
const Segment SGI242861[] = {{&S89811[96], 9}, {NULL}};
const Segment SGI242862[] = {{&S89811[69], 6}, {NULL}};
const Segment SGI242863[] = {{&S97095[44], 12}, {NULL}};
const Segment SGI242864[] = {{&S97095[44], 24}, {NULL}};
const Segment SGI242865[] = {{&S97095[30], 6}, {NULL}};
const Segment SGI242866[] = {{&S97095[117], 3}, {NULL}};
const Segment SGI242867[] = {{&S97095[82], 6}, {NULL}};
const Segment SGI242868[] = {{&S97101[38], 9}, {NULL}};
const Segment SGI242869[] = {{&S97101[31], 6}, {NULL}};
const Segment SGI242870[] = {{&S98403[34], 17}, {NULL}};
const Segment SGI242871[] = {{&S97107[34], 12}, {NULL}};
const Segment SGI242872[] = {{&S98403[27], 6}, {NULL}};
const Segment SGI242873[] = {{&S97107[27], 6}, {NULL}};
const Segment SGI242874[] = {{&S97107[99], 9}, {NULL}};
const Segment SGI242875[] = {{&S98403[104], 9}, {NULL}};
const Segment SGI242876[] = {{&S97107[72], 6}, {NULL}};
const Segment SGI242877[] = {{&S98403[77], 6}, {NULL}};
const Segment SGI242878[] = {{&S98415[34], 15}, {NULL}};
const Segment SGI242879[] = {{&S98415[27], 6}, {NULL}};
const Segment SGI242880[] = {{&S98415[102], 9}, {NULL}};
const Segment SGI242881[] = {{&S98415[75], 6}, {NULL}};
const Segment SGI242882[] = {{&S98416[34], 10}, {NULL}};
const Segment SGI242883[] = {{&S89845[44], 10}, {NULL}};
const Segment SGI242884[] = {{&S97121[34], 25}, {NULL}};
const Segment SGI242885[] = {{&S89845[44], 21}, {NULL}};
const Segment SGI242886[] = {{&S98416[27], 6}, {NULL}};
const Segment SGI242887[] = {{&S97121[27], 6}, {NULL}};
const Segment SGI242888[] = {{&S89845[30], 6}, {NULL}};
const Segment SGI242889[] = {{&S98416[97], 9}, {NULL}};
const Segment SGI242890[] = {{&S97121[112], 9}, {NULL}};
const Segment SGI242891[] = {{&S98416[70], 6}, {NULL}};
const Segment SGI242892[] = {{&S89845[79], 6}, {NULL}};
const Segment SGI242893[] = {{&S97121[85], 6}, {NULL}};
const Segment SGI242894[] = {{&S89845[106], 13}, {NULL}};
const Segment SGI242895[] = {{&S97122[34], 7}, {NULL}};
const Segment SGI242896[] = {{&S97122[27], 6}, {NULL}};
const Segment SGI242897[] = {{&S97122[94], 9}, {NULL}};
const Segment SGI242898[] = {{&S97122[67], 6}, {NULL}};
const Segment SGI242899[] = {{&S89847[34], 10}, {NULL}};
const Segment SGI242900[] = {{&S89847[27], 6}, {NULL}};
const Segment SGI242901[] = {{&S89847[96], 9}, {NULL}};
const Segment SGI242902[] = {{&S89847[69], 6}, {NULL}};
const Segment SGI242903[] = {{&S97123[34], 8}, {NULL}};
const Segment SGI242904[] = {{&S89847[137], 8}, {NULL}};
const Segment SGI242905[] = {{&S97123[27], 6}, {NULL}};
const Segment SGI242906[] = {{&S97123[95], 9}, {NULL}};
const Segment SGI242907[] = {{&S97123[68], 6}, {NULL}};
const Segment SGI242908[] = {{&S105074[34], 13}, {NULL}};
const Segment SGI242909[] = {{&S105074[27], 6}, {NULL}};
const Segment SGI242910[] = {{&S105074[100], 9}, {NULL}};
const Segment SGI242911[] = {{&S105074[73], 6}, {NULL}};
const Segment SGI242912[] = {{&S105074[141], 8}, {NULL}};
const Segment SGI242913[] = {{&S97126[34], 8}, {NULL}};
const Segment SGI242914[] = {{&S97126[27], 6}, {NULL}};
const Segment SGI242915[] = {{&S97126[101], 9}, {NULL}};
const Segment SGI242916[] = {{&S97126[74], 6}, {NULL}};
const Segment SGI242917[] = {{&S97126[135], 8}, {NULL}};
const Segment SGI242918[] = {{&S105077[34], 25}, {NULL}};
const Segment SGI242919[] = {{&S105077[27], 6}, {NULL}};
const Segment SGI242920[] = {{&S105077[112], 9}, {NULL}};
const Segment SGI242921[] = {{&S105077[85], 6}, {NULL}};
const Segment SGI242922[] = {{&S105077[153], 8}, {NULL}};
const Segment SGI242923[] = {{&S98427[34], 9}, {NULL}};
const Segment SGI242924[] = {{&S98427[27], 6}, {NULL}};
const Segment SGI242925[] = {{&S98427[96], 9}, {NULL}};
const Segment SGI242926[] = {{&S98427[69], 6}, {NULL}};
const Segment SGI242927[] = {{&S97130[34], 8}, {NULL}};
const Segment SGI242928[] = {{&S97130[27], 6}, {NULL}};
const Segment SGI242929[] = {{&S97130[95], 9}, {NULL}};
const Segment SGI242930[] = {{&S97130[68], 6}, {NULL}};
const Segment SGI242931[] = {{&S97133[44], 9}, {NULL}};
const Segment SGI242932[] = {{&S97133[44], 21}, {NULL}};
const Segment SGI242933[] = {{&S97133[30], 6}, {NULL}};
const Segment SGI242934[] = {{&S97133[114], 3}, {NULL}};
const Segment SGI242935[] = {{&S97133[79], 6}, {NULL}};
const Segment SGI242936[] = {{&S97133[146], 15}, {NULL}};
const Segment SGI242937[] = {{&S97135[44], 10}, {NULL}};
const Segment SGI242938[] = {{&S97135[44], 22}, {NULL}};
const Segment SGI242939[] = {{&S97135[30], 6}, {NULL}};
const Segment SGI242940[] = {{&S97135[80], 6}, {NULL}};
const Segment SGI242941[] = {{&S97135[107], 13}, {NULL}};
const Segment SGI242942[] = {{&S97150[44], 5}, {NULL}};
const Segment SGI242943[] = {{&S97150[44], 17}, {NULL}};
const Segment SGI242944[] = {{&S97150[30], 6}, {NULL}};
const Segment SGI242945[] = {{&S97150[75], 6}, {NULL}};
const Segment SGI242946[] = {{&S97150[102], 13}, {NULL}};
const Segment SGI242947[] = {{&S97160[44], 9}, {NULL}};
const Segment SGI242948[] = {{&S97160[44], 21}, {NULL}};
const Segment SGI242949[] = {{&S97160[30], 6}, {NULL}};
const Segment SGI242950[] = {{&S97160[79], 6}, {NULL}};
const Segment SGI242951[] = {{&S97160[106], 13}, {NULL}};
const Segment SGI242952[] = {{&S97166[44], 10}, {NULL}};
const Segment SGI242953[] = {{&S97166[44], 22}, {NULL}};
const Segment SGI242954[] = {{&S97166[30], 6}, {NULL}};
const Segment SGI242955[] = {{&S97166[115], 3}, {NULL}};
const Segment SGI242956[] = {{&S97166[80], 6}, {NULL}};
const Segment SGI242957[] = {{&S97166[135], 8}, {NULL}};
const Segment SGI242958[] = {{&S97166[144], 13}, {NULL}};
const Segment SGI242959[] = {{&S89912[34], 7}, {NULL}};
const Segment SGI242960[] = {{&S89912[27], 6}, {NULL}};
const Segment SGI242961[] = {{&S89912[93], 9}, {NULL}};
const Segment SGI242962[] = {{&S89912[66], 6}, {NULL}};
const Segment SGI242963[] = {{&S97179[34], 10}, {NULL}};
const Segment SGI242964[] = {{&S97179[27], 6}, {NULL}};
const Segment SGI242965[] = {{&S97179[97], 9}, {NULL}};
const Segment SGI242966[] = {{&S97179[70], 6}, {NULL}};
const Segment SGI242967[] = {{&S89916[34], 10}, {NULL}};
const Segment SGI242968[] = {{&S89916[27], 6}, {NULL}};
const Segment SGI242969[] = {{&S89916[96], 9}, {NULL}};
const Segment SGI242970[] = {{&S89916[69], 6}, {NULL}};
const Segment SGI242971[] = {{&S98491[34], 14}, {NULL}};
const Segment SGI242972[] = {{&S98491[27], 6}, {NULL}};
const Segment SGI242973[] = {{&S98491[101], 9}, {NULL}};
const Segment SGI242974[] = {{&S98491[74], 6}, {NULL}};
const Segment SGI242975[] = {{&S98494[34], 10}, {NULL}};
const Segment SGI242976[] = {{&S98494[27], 6}, {NULL}};
const Segment SGI242977[] = {{&S98494[97], 9}, {NULL}};
const Segment SGI242978[] = {{&S98494[70], 6}, {NULL}};
const Segment SGI242979[] = {{&S98500[34], 8}, {NULL}};
const Segment SGI242980[] = {{&S98500[27], 6}, {NULL}};
const Segment SGI242981[] = {{&S98500[95], 9}, {NULL}};
const Segment SGI242982[] = {{&S98500[68], 6}, {NULL}};
const Segment SGI242983[] = {{&S97197[44], 14}, {NULL}};
const Segment SGI242984[] = {{&S97197[44], 26}, {NULL}};
const Segment SGI242985[] = {{&S97197[30], 6}, {NULL}};
const Segment SGI242986[] = {{&S97197[84], 6}, {NULL}};
const Segment SGI242987[] = {{&S97197[111], 13}, {NULL}};
const Segment SGI242988[] = {{&S89933[34], 9}, {NULL}};
const Segment SGI242989[] = {{&S89933[27], 6}, {NULL}};
const Segment SGI242990[] = {{&S89933[96], 9}, {NULL}};
const Segment SGI242991[] = {{&S89933[69], 6}, {NULL}};
const Segment SGI242992[] = {{&S97200[34], 9}, {NULL}};
const Segment SGI242993[] = {{&S97200[27], 6}, {NULL}};
const Segment SGI242994[] = {{&S97200[96], 9}, {NULL}};
const Segment SGI242995[] = {{&S97200[69], 6}, {NULL}};
const Segment SGI242996[] = {{&S97200[130], 8}, {NULL}};
const Segment SGI242997[] = {{&S98507[34], 16}, {NULL}};
const Segment SGI242998[] = {{&S98507[27], 6}, {NULL}};
const Segment SGI242999[] = {{&S98507[103], 9}, {NULL}};
const Segment SGI243000[] = {{&S98507[76], 6}, {NULL}};
const Segment SGI243001[] = {{&S98507[144], 8}, {NULL}};
const Segment SGI243002[] = {{&S97202[44], 13}, {NULL}};
const Segment SGI243003[] = {{&S97202[44], 29}, {NULL}};
const Segment SGI243004[] = {{&S97202[30], 6}, {NULL}};
const Segment SGI243005[] = {{&S97202[120], 3}, {NULL}};
const Segment SGI243006[] = {{&S97202[87], 5}, {NULL}};
const Segment SGI243007[] = {{&S97202[124], 14}, {NULL}};
const Segment SGI243008[] = {{&S97204[34], 16}, {NULL}};
const Segment SGI243009[] = {{&S97204[27], 6}, {NULL}};
const Segment SGI243010[] = {{&S97204[100], 9}, {NULL}};
const Segment SGI243011[] = {{&S97204[73], 6}, {NULL}};
const Segment SGI243012[] = {{&S97206[34], 10}, {NULL}};
const Segment SGI243013[] = {{&S97206[27], 6}, {NULL}};
const Segment SGI243014[] = {{&S97206[97], 9}, {NULL}};
const Segment SGI243015[] = {{&S97206[70], 6}, {NULL}};
const Segment SGI243016[] = {{&S97206[138], 8}, {NULL}};
const Segment SGI243017[] = {{&S97212[44], 10}, {NULL}};
const Segment SGI243018[] = {{&S97212[44], 22}, {NULL}};
const Segment SGI243019[] = {{&S97212[30], 6}, {NULL}};
const Segment SGI243020[] = {{&S97212[115], 3}, {NULL}};
const Segment SGI243021[] = {{&S97212[80], 6}, {NULL}};
const Segment SGI243022[] = {{&S89961[34], 12}, {NULL}};
const Segment SGI243023[] = {{&S89961[27], 6}, {NULL}};
const Segment SGI243024[] = {{&S97221[44], 13}, {NULL}};
const Segment SGI243025[] = {{&S97221[44], 25}, {NULL}};
const Segment SGI243026[] = {{&S97221[30], 6}, {NULL}};
const Segment SGI243027[] = {{&S89961[98], 9}, {NULL}};
const Segment SGI243028[] = {{&S89961[71], 6}, {NULL}};
const Segment SGI243029[] = {{&S97221[83], 6}, {NULL}};
const Segment SGI243030[] = {{&S97221[110], 13}, {NULL}};
const Segment SGI243031[] = {{&S2273[13], 6}, {NULL}};
const Segment SGI243032[] = {{&S2273[35], 3}, {NULL}};
const Segment SGI243033[] = {{&S2273[66], 6}, {NULL}};
const Segment SGI243034[] = {{&S2273[100], 8}, {NULL}};
const Segment SGI243035[] = {{&S2274[13], 6}, {NULL}};
const Segment SGI243036[] = {{&S2274[35], 3}, {NULL}};
const Segment SGI243037[] = {{&S2274[66], 6}, {NULL}};
const Segment SGI243038[] = {{&S2274[100], 8}, {NULL}};
const Segment SGI243039[] = {{&S97224[44], 10}, {NULL}};
const Segment SGI243040[] = {{&S2275[13], 6}, {NULL}};
const Segment SGI243041[] = {{&S97224[44], 22}, {NULL}};
const Segment SGI243042[] = {{&S2275[35], 3}, {NULL}};
const Segment SGI243043[] = {{&S97224[30], 6}, {NULL}};
const Segment SGI243044[] = {{&S2275[66], 6}, {NULL}};
const Segment SGI243045[] = {{&S2275[100], 8}, {NULL}};
const Segment SGI243046[] = {{&S97224[80], 6}, {NULL}};
const Segment SGI243047[] = {{&S97224[107], 13}, {NULL}};
const Segment SGI243048[] = {{&S2276[13], 6}, {NULL}};
const Segment SGI243049[] = {{&S2276[35], 3}, {NULL}};
const Segment SGI243050[] = {{&S2276[66], 6}, {NULL}};
const Segment SGI243051[] = {{&S2277[13], 6}, {NULL}};
const Segment SGI243052[] = {{&S2277[35], 3}, {NULL}};
const Segment SGI243053[] = {{&S2277[66], 6}, {NULL}};
const Segment SGI243054[] = {{&S2277[100], 6}, {NULL}};
const Segment SGI243055[] = {{&S2278[13], 6}, {NULL}};
const Segment SGI243056[] = {{&S2278[35], 3}, {NULL}};
const Segment SGI243057[] = {{&S2278[66], 6}, {NULL}};
const Segment SGI243058[] = {{&S2278[100], 6}, {NULL}};
const Segment SGI243059[] = {{&S2279[13], 6}, {NULL}};
const Segment SGI243060[] = {{&S2279[35], 3}, {NULL}};
const Segment SGI243061[] = {{&S2279[66], 6}, {NULL}};
const Segment SGI243062[] = {{&S2279[100], 6}, {NULL}};
const Segment SGI243063[] = {{&S2280[13], 6}, {NULL}};
const Segment SGI243064[] = {{&S2280[35], 3}, {NULL}};
const Segment SGI243065[] = {{&S2280[66], 6}, {NULL}};
const Segment SGI243066[] = {{&S2280[100], 6}, {NULL}};
const Segment SGI243067[] = {{&S2281[13], 6}, {NULL}};
const Segment SGI243068[] = {{&S2281[38], 3}, {NULL}};
const Segment SGI243069[] = {{&S97233[34], 19}, {NULL}};
const Segment SGI243070[] = {{&S97233[27], 6}, {NULL}};
const Segment SGI243071[] = {{&S97233[106], 9}, {NULL}};
const Segment SGI243072[] = {{&S97233[79], 6}, {NULL}};
const Segment SGI243073[] = {{&S2282[13], 6}, {NULL}};
const Segment SGI243074[] = {{&S2282[42], 3}, {NULL}};
const Segment SGI243075[] = {{&S2282[123], 5}, {NULL}};
const Segment SGI243076[] = {{&S2283[13], 6}, {NULL}};
const Segment SGI243077[] = {{&S2283[31], 3}, {NULL}};
const Segment SGI243078[] = {{&S97235[34], 11}, {NULL}};
const Segment SGI243079[] = {{&S97235[27], 6}, {NULL}};
const Segment SGI243080[] = {{&S2284[13], 6}, {NULL}};
const Segment SGI243081[] = {{&S2284[42], 3}, {NULL}};
const Segment SGI243082[] = {{&S97235[98], 9}, {NULL}};
const Segment SGI243083[] = {{&S97235[71], 6}, {NULL}};
const Segment SGI243084[] = {{&S2284[123], 5}, {NULL}};
const Segment SGI243085[] = {{&S2285[13], 6}, {NULL}};
const Segment SGI243086[] = {{&S2285[35], 5}, {NULL}};
const Segment SGI243087[] = {{&S2285[68], 6}, {NULL}};
const Segment SGI243088[] = {{&S97236[34], 10}, {NULL}};
const Segment SGI243089[] = {{&S97236[27], 6}, {NULL}};
const Segment SGI243090[] = {{&S97236[97], 9}, {NULL}};
const Segment SGI243091[] = {{&S97236[70], 6}, {NULL}};
const Segment SGI243092[] = {{&S2286[13], 6}, {NULL}};
const Segment SGI243093[] = {{&S2286[42], 3}, {NULL}};
const Segment SGI243094[] = {{&S2286[123], 5}, {NULL}};
const Segment SGI243095[] = {{&S2287[13], 6}, {NULL}};
const Segment SGI243096[] = {{&S97238[34], 10}, {NULL}};
const Segment SGI243097[] = {{&S2287[35], 5}, {NULL}};
const Segment SGI243098[] = {{&S2287[68], 6}, {NULL}};
const Segment SGI243099[] = {{&S97238[27], 6}, {NULL}};
const Segment SGI243100[] = {{&S2287[102], 6}, {NULL}};
const Segment SGI243101[] = {{&S97238[97], 9}, {NULL}};
const Segment SGI243102[] = {{&S97238[70], 6}, {NULL}};
const Segment SGI243103[] = {{&S2288[13], 6}, {&S2288[107], 4}, {NULL}};
const Segment SGI243104[] = {{&S2288[35], 3}, {NULL}};
const Segment SGI243105[] = {{&S2288[66], 6}, {NULL}};
const Segment SGI243106[] = {{&S2288[100], 6}, {NULL}};
const Segment SGI243107[] = {{&S2289[13], 6}, {NULL}};
const Segment SGI243108[] = {{&S2289[35], 5}, {NULL}};
const Segment SGI243109[] = {{&S2289[68], 6}, {NULL}};
const Segment SGI243110[] = {{&S2289[102], 6}, {NULL}};
const Segment SGI243111[] = {{&S2290[44], 6}, {NULL}};
const Segment SGI243112[] = {{&S2290[69], 3}, {NULL}};
const Segment SGI243113[] = {{&S2290[107], 6}, {NULL}};
const Segment SGI243114[] = {{&S2291[13], 6}, {NULL}};
const Segment SGI243115[] = {{&S2291[38], 5}, {NULL}};
const Segment SGI243116[] = {{&S2292[13], 6}, {NULL}};
const Segment SGI243117[] = {{&S2292[42], 3}, {NULL}};
const Segment SGI243118[] = {{&S2292[123], 5}, {NULL}};
const Segment SGI243119[] = {{&S2293[13], 6}, {NULL}};
const Segment SGI243120[] = {{&S2293[35], 5}, {NULL}};
const Segment SGI243121[] = {{&S2293[68], 6}, {NULL}};
const Segment SGI243122[] = {{&S2293[102], 6}, {NULL}};
const Segment SGI243123[] = {{&S2294[13], 6}, {NULL}};
const Segment SGI243124[] = {{&S2294[35], 5}, {NULL}};
const Segment SGI243125[] = {{&S2294[68], 6}, {NULL}};
const Segment SGI243126[] = {{&S2295[13], 6}, {NULL}};
const Segment SGI243127[] = {{&S2295[42], 5}, {NULL}};
const Segment SGI243128[] = {{&S2295[125], 5}, {NULL}};
const Segment SGI243129[] = {{&S89995[34], 8}, {NULL}};
const Segment SGI243130[] = {{&S89995[27], 6}, {NULL}};
const Segment SGI243131[] = {{&S89995[94], 9}, {NULL}};
const Segment SGI243132[] = {{&S89995[67], 6}, {NULL}};
const Segment SGI243133[] = {{&S2296[13], 6}, {NULL}};
const Segment SGI243134[] = {{&S2296[35], 5}, {NULL}};
const Segment SGI243135[] = {{&S2296[68], 6}, {NULL}};
const Segment SGI243136[] = {{&S2296[102], 6}, {NULL}};
const Segment SGI243137[] = {{&S2297[15], 6}, {NULL}};
const Segment SGI243138[] = {{&S2297[33], 3}, {NULL}};
const Segment SGI243139[] = {{&S2297[0], 7}, {NULL}};
const Segment SGI243140[] = {{&S2298[13], 6}, {NULL}};
const Segment SGI243141[] = {{&S2298[38], 5}, {NULL}};
const Segment SGI243142[] = {{&S2299[13], 6}, {NULL}};
const Segment SGI243143[] = {{&S2299[35], 5}, {NULL}};
const Segment SGI243144[] = {{&S2299[68], 6}, {NULL}};
const Segment SGI243145[] = {{&S2299[102], 6}, {NULL}};
const Segment SGI243146[] = {{&S2300[13], 6}, {NULL}};
const Segment SGI243147[] = {{&S2300[42], 5}, {NULL}};
const Segment SGI243148[] = {{&S2300[125], 5}, {NULL}};
const Segment SGI243149[] = {{&S90003[44], 10}, {NULL}};
const Segment SGI243150[] = {{&S2301[13], 6}, {NULL}};
const Segment SGI243151[] = {{&S90003[44], 21}, {NULL}};
const Segment SGI243152[] = {{&S2301[42], 3}, {NULL}};
const Segment SGI243153[] = {{&S90003[30], 6}, {NULL}};
const Segment SGI243154[] = {{&S2301[123], 5}, {NULL}};
const Segment SGI243155[] = {{&S90003[79], 6}, {NULL}};
const Segment SGI243156[] = {{&S90003[106], 13}, {NULL}};
const Segment SGI243157[] = {{&S2302[13], 6}, {&S2302[109], 4}, {NULL}};
const Segment SGI243158[] = {{&S2302[35], 5}, {NULL}};
const Segment SGI243159[] = {{&S2302[68], 6}, {NULL}};
const Segment SGI243160[] = {{&S2302[102], 6}, {NULL}};
const Segment SGI243161[] = {{&S2303[13], 6}, {NULL}};
const Segment SGI243162[] = {{&S2303[42], 5}, {NULL}};
const Segment SGI243163[] = {{&S2303[125], 5}, {NULL}};
const Segment SGI243164[] = {{&S2304[13], 6}, {NULL}};
const Segment SGI243165[] = {{&S2304[42], 5}, {NULL}};
const Segment SGI243166[] = {{&S2304[125], 5}, {NULL}};
const Segment SGI243167[] = {{&S90008[34], 12}, {NULL}};
const Segment SGI243168[] = {{&S90008[27], 6}, {NULL}};
const Segment SGI243169[] = {{&S2305[13], 6}, {NULL}};
const Segment SGI243170[] = {{&S2305[28], 3}, {NULL}};
const Segment SGI243171[] = {{&S2305[59], 6}, {NULL}};
const Segment SGI243172[] = {{&S90008[98], 9}, {NULL}};
const Segment SGI243173[] = {{&S2305[93], 6}, {NULL}};
const Segment SGI243174[] = {{&S90008[71], 6}, {NULL}};
const Segment SGI243175[] = {{&S2305[107], 7}, {NULL}};
const Segment SGI243176[] = {{&S90008[139], 8}, {NULL}};
const Segment SGI243177[] = {{&S2306[17], 6}, {NULL}};
const Segment SGI243178[] = {{&S2306[27], 5}, {NULL}};
const Segment SGI243179[] = {{&S2307[13], 6}, {NULL}};
const Segment SGI243180[] = {{&S2307[35], 3}, {NULL}};
const Segment SGI243181[] = {{&S2307[66], 6}, {NULL}};
const Segment SGI243182[] = {{&S2307[100], 5}, {NULL}};
const Segment SGI243183[] = {{&S2308[13], 6}, {NULL}};
const Segment SGI243184[] = {{&S2308[35], 5}, {NULL}};
const Segment SGI243185[] = {{&S2308[68], 6}, {NULL}};
const Segment SGI243186[] = {{&S2308[102], 6}, {NULL}};
const Segment SGI243187[] = {{&S2308[116], 9}, {NULL}};
const Segment SGI243188[] = {{&S2309[13], 6}, {NULL}};
const Segment SGI243189[] = {{&S2309[42], 5}, {NULL}};
const Segment SGI243190[] = {{&S2309[125], 5}, {NULL}};
const Segment SGI243191[] = {{&S2310[44], 6}, {NULL}};
const Segment SGI243192[] = {{&S2310[69], 5}, {NULL}};
const Segment SGI243193[] = {{&S2310[109], 6}, {NULL}};
const Segment SGI243194[] = {{&S2311[17], 6}, {&S2311[34], 4}, {NULL}};
const Segment SGI243195[] = {{&S2311[27], 5}, {NULL}};
const Segment SGI243196[] = {{&S2312[13], 6}, {&S2312[13], 6}, {NULL}};
const Segment SGI243197[] = {{&S2312[35], 5}, {&S2312[131], 5}, {NULL}};
const Segment SGI243198[] = {{&S2312[68], 6}, {NULL}};
const Segment SGI243199[] = {{&S2312[102], 6}, {NULL}};
const Segment SGI243200[] = {{&S2313[13], 6}, {NULL}};
const Segment SGI243201[] = {{&S2313[42], 5}, {NULL}};
const Segment SGI243202[] = {{&S2313[125], 5}, {NULL}};
const Segment SGI243203[] = {{&S2314[15], 6}, {NULL}};
const Segment SGI243204[] = {{&S2314[33], 5}, {NULL}};
const Segment SGI243205[] = {{&S2314[0], 7}, {NULL}};
const Segment SGI243206[] = {{&S2315[13], 6}, {NULL}};
const Segment SGI243207[] = {{&S2315[35], 5}, {NULL}};
const Segment SGI243208[] = {{&S2315[68], 6}, {NULL}};
const Segment SGI243209[] = {{&S2315[102], 6}, {NULL}};
const Segment SGI243210[] = {{&S2316[13], 6}, {NULL}};
const Segment SGI243211[] = {{&S2316[35], 5}, {NULL}};
const Segment SGI243212[] = {{&S2316[68], 6}, {NULL}};
const Segment SGI243213[] = {{&S2317[13], 6}, {NULL}};
const Segment SGI243214[] = {{&S2317[67], 6}, {NULL}};
const Segment SGI243215[] = {{&S2317[101], 6}, {NULL}};
const Segment SGI243216[] = {{&S2318[13], 6}, {NULL}};
const Segment SGI243217[] = {{&S2318[38], 5}, {NULL}};
const Segment SGI243218[] = {{&S2319[13], 6}, {NULL}};
const Segment SGI243219[] = {{&S2319[35], 5}, {NULL}};
const Segment SGI243220[] = {{&S2319[68], 6}, {NULL}};
const Segment SGI243221[] = {{&S2319[102], 5}, {NULL}};
const Segment SGI243222[] = {{&S2320[13], 6}, {&S2320[13], 6}, {NULL}};
const Segment SGI243223[] = {{&S2320[35], 5}, {&S2320[131], 5}, {NULL}};
const Segment SGI243224[] = {{&S2320[68], 6}, {NULL}};
const Segment SGI243225[] = {{&S2320[102], 6}, {NULL}};
const Segment SGI243226[] = {{&S2321[13], 6}, {&S2321[109], 4}, {NULL}};
const Segment SGI243227[] = {{&S2321[35], 5}, {NULL}};
const Segment SGI243228[] = {{&S2321[68], 6}, {NULL}};
const Segment SGI243229[] = {{&S2321[102], 6}, {NULL}};
const Segment SGI243230[] = {{&S2322[13], 6}, {NULL}};
const Segment SGI243231[] = {{&S2322[133], 5}, {NULL}};
const Segment SGI243232[] = {{&S2322[35], 5}, {NULL}};
const Segment SGI243233[] = {{&S2322[68], 6}, {NULL}};
const Segment SGI243234[] = {{&S2322[102], 6}, {NULL}};
const Segment SGI243235[] = {{&S2323[13], 6}, {NULL}};
const Segment SGI243236[] = {{&S2323[133], 5}, {NULL}};
const Segment SGI243237[] = {{&S2323[35], 5}, {NULL}};
const Segment SGI243238[] = {{&S2323[68], 6}, {NULL}};
const Segment SGI243239[] = {{&S2323[102], 6}, {NULL}};
const Segment SGI243240[] = {{&S2324[13], 6}, {NULL}};
const Segment SGI243241[] = {{&S2324[67], 6}, {NULL}};
const Segment SGI243242[] = {{&S2324[101], 6}, {NULL}};
const Segment SGI243243[] = {{&S2325[13], 6}, {NULL}};
const Segment SGI243244[] = {{&S2325[35], 5}, {NULL}};
const Segment SGI243245[] = {{&S2325[68], 6}, {NULL}};
const Segment SGI243246[] = {{&S2325[102], 6}, {NULL}};
const Segment SGI243247[] = {{&S2325[116], 9}, {NULL}};
const Segment SGI243248[] = {{&S2326[15], 6}, {NULL}};
const Segment SGI243249[] = {{&S2326[33], 5}, {NULL}};
const Segment SGI243250[] = {{&S2326[0], 7}, {NULL}};
const Segment SGI243251[] = {{&S2327[44], 6}, {NULL}};
const Segment SGI243252[] = {{&S2327[69], 5}, {NULL}};
const Segment SGI243253[] = {{&S2327[109], 6}, {NULL}};
const Segment SGI243254[] = {{&S2328[13], 6}, {NULL}};
const Segment SGI243255[] = {{&S2328[35], 5}, {NULL}};
const Segment SGI243256[] = {{&S2328[68], 6}, {NULL}};
const Segment SGI243257[] = {{&S2328[102], 6}, {NULL}};
const Segment SGI243258[] = {{&S2329[13], 6}, {NULL}};
const Segment SGI243259[] = {{&S2329[42], 5}, {NULL}};
const Segment SGI243260[] = {{&S2329[125], 5}, {NULL}};
const Segment SGI243261[] = {{&S2330[13], 6}, {NULL}};
const Segment SGI243262[] = {{&S2330[42], 5}, {NULL}};
const Segment SGI243263[] = {{&S2330[125], 5}, {NULL}};
const Segment SGI243264[] = {{&S2331[13], 6}, {NULL}};
const Segment SGI243265[] = {{&S2331[35], 5}, {NULL}};
const Segment SGI243266[] = {{&S2331[68], 6}, {NULL}};
const Segment SGI243267[] = {{&S2332[13], 6}, {NULL}};
const Segment SGI243268[] = {{&S2332[35], 5}, {NULL}};
const Segment SGI243269[] = {{&S2332[68], 6}, {NULL}};
const Segment SGI243270[] = {{&S2332[102], 6}, {NULL}};
const Segment SGI243271[] = {{&S2333[17], 6}, {NULL}};
const Segment SGI243272[] = {{&S2333[27], 5}, {NULL}};
const Segment SGI243273[] = {{&S2334[13], 6}, {&S2334[13], 6}, {NULL}};
const Segment SGI243274[] = {{&S2334[75], 6}, {NULL}};
const Segment SGI243275[] = {{&S2334[109], 4}, {NULL}};
const Segment SGI243276[] = {{&S2335[13], 6}, {&S2335[128], 4}, {NULL}};
const Segment SGI243277[] = {{&S2335[75], 6}, {NULL}};
const Segment SGI243278[] = {{&S2335[109], 4}, {NULL}};
const Segment SGI243279[] = {{&S2336[17], 6}, {NULL}};
const Segment SGI243280[] = {{&S2336[27], 5}, {NULL}};
const Segment SGI243281[] = {{&S2337[13], 6}, {NULL}};
const Segment SGI243282[] = {{&S2337[38], 5}, {NULL}};
const Segment SGI243283[] = {{&S2338[13], 6}, {&S2338[13], 6}, {NULL}};
const Segment SGI243284[] = {{&S2338[35], 5}, {&S2338[131], 5}, {NULL}};
const Segment SGI243285[] = {{&S2338[68], 6}, {NULL}};
const Segment SGI243286[] = {{&S2338[102], 6}, {NULL}};
const Segment SGI243287[] = {{&S2339[13], 6}, {NULL}};
const Segment SGI243288[] = {{&S2339[35], 3}, {NULL}};
const Segment SGI243289[] = {{&S2339[66], 6}, {NULL}};
const Segment SGI243290[] = {{&S2339[100], 6}, {NULL}};
const Segment SGI243291[] = {{&S2340[13], 6}, {NULL}};
const Segment SGI243292[] = {{&S2340[38], 5}, {NULL}};
const Segment SGI243293[] = {{&S2340[71], 6}, {NULL}};
const Segment SGI243294[] = {{&S2340[105], 6}, {NULL}};
const Segment SGI243295[] = {{&S2340[119], 7}, {NULL}};
const Segment SGI243296[] = {{&S2341[44], 6}, {NULL}};
const Segment SGI243297[] = {{&S2341[69], 5}, {NULL}};
const Segment SGI243298[] = {{&S2341[109], 6}, {NULL}};
const Segment SGI243299[] = {{&S2342[13], 6}, {NULL}};
const Segment SGI243300[] = {{&S2342[35], 3}, {NULL}};
const Segment SGI243301[] = {{&S2342[66], 6}, {NULL}};
const Segment SGI243302[] = {{&S2342[100], 6}, {NULL}};
const Segment SGI243303[] = {{&S2342[114], 9}, {NULL}};
const Segment SGI243304[] = {{&S2343[15], 6}, {NULL}};
const Segment SGI243305[] = {{&S2343[33], 5}, {NULL}};
const Segment SGI243306[] = {{&S2343[0], 7}, {NULL}};
const Segment SGI243307[] = {{&S2344[13], 6}, {&S2344[142], 4}, {&S2344[142], 4}, {NULL}};
const Segment SGI243308[] = {{&S90074[34], 7}, {NULL}};
const Segment SGI243309[] = {{&S2344[35], 5}, {NULL}};
const Segment SGI243310[] = {{&S90074[27], 6}, {NULL}};
const Segment SGI243311[] = {{&S2344[68], 6}, {NULL}};
const Segment SGI243312[] = {{&S2344[102], 6}, {NULL}};
const Segment SGI243313[] = {{&S90074[93], 9}, {NULL}};
const Segment SGI243314[] = {{&S90074[66], 6}, {NULL}};
const Segment SGI243315[] = {{&S2345[13], 6}, {NULL}};
const Segment SGI243316[] = {{&S2345[35], 5}, {NULL}};
const Segment SGI243317[] = {{&S2345[68], 6}, {NULL}};
const Segment SGI243318[] = {{&S2345[102], 6}, {NULL}};
const Segment SGI243319[] = {{&S2345[116], 9}, {NULL}};
const Segment SGI243320[] = {{&S2346[13], 6}, {&S2346[152], 4}, {&S2346[152], 4}, {NULL}};
const Segment SGI243321[] = {{&S2346[35], 5}, {NULL}};
const Segment SGI243322[] = {{&S2346[68], 6}, {NULL}};
const Segment SGI243323[] = {{&S2346[102], 6}, {NULL}};
const Segment SGI243324[] = {{&S2347[13], 6}, {NULL}};
const Segment SGI243325[] = {{&S2347[42], 5}, {NULL}};
const Segment SGI243326[] = {{&S2347[125], 5}, {NULL}};
const Segment SGI243327[] = {{&S2348[13], 6}, {NULL}};
const Segment SGI243328[] = {{&S2348[42], 5}, {NULL}};
const Segment SGI243329[] = {{&S2348[125], 5}, {NULL}};
const Segment SGI243330[] = {{&S2349[13], 6}, {NULL}};
const Segment SGI243331[] = {{&S2349[42], 5}, {NULL}};
const Segment SGI243332[] = {{&S2349[125], 5}, {NULL}};
const Segment SGI243333[] = {{&S2350[13], 6}, {NULL}};
const Segment SGI243334[] = {{&S2350[42], 5}, {NULL}};
const Segment SGI243335[] = {{&S2350[125], 5}, {NULL}};
const Segment SGI243336[] = {{&S90084[44], 10}, {NULL}};
const Segment SGI243337[] = {{&S90084[44], 21}, {NULL}};
const Segment SGI243338[] = {{&S90084[30], 6}, {NULL}};
const Segment SGI243339[] = {{&S90084[132], 3}, {NULL}};
const Segment SGI243340[] = {{&S90084[97], 6}, {NULL}};
const Segment SGI243341[] = {{&S90084[67], 8}, {NULL}};
const Segment SGI243342[] = {{&S2351[17], 6}, {&S2351[34], 4}, {NULL}};
const Segment SGI243343[] = {{&S2351[27], 5}, {NULL}};
const Segment SGI243344[] = {{&S2352[13], 6}, {NULL}};
const Segment SGI243345[] = {{&S2352[42], 5}, {NULL}};
const Segment SGI243346[] = {{&S2352[125], 5}, {NULL}};
const Segment SGI243347[] = {{&S2354[13], 6}, {NULL}};
const Segment SGI243348[] = {{&S2354[133], 5}, {NULL}};
const Segment SGI243349[] = {{&S2354[35], 5}, {NULL}};
const Segment SGI243350[] = {{&S2354[68], 6}, {NULL}};
const Segment SGI243351[] = {{&S2354[102], 6}, {NULL}};
const Segment SGI243352[] = {{&S2355[13], 6}, {&S2355[13], 6}, {NULL}};
const Segment SGI243353[] = {{&S2355[35], 5}, {NULL}};
const Segment SGI243354[] = {{&S2355[68], 6}, {NULL}};
const Segment SGI243355[] = {{&S2355[102], 6}, {NULL}};
const Segment SGI243356[] = {{&S90091[44], 14}, {NULL}};
const Segment SGI243357[] = {{&S90091[44], 25}, {NULL}};
const Segment SGI243358[] = {{&S90091[30], 6}, {NULL}};
const Segment SGI243359[] = {{&S2356[13], 6}, {&S2356[13], 6}, {NULL}};
const Segment SGI243360[] = {{&S2356[35], 5}, {&S2356[35], 5}, {NULL}};
const Segment SGI243361[] = {{&S90091[83], 6}, {NULL}};
const Segment SGI243362[] = {{&S2356[68], 6}, {&S2356[68], 6}, {NULL}};
const Segment SGI243363[] = {{&S90091[110], 13}, {NULL}};
const Segment SGI243364[] = {{&S2356[102], 6}, {&S2356[102], 6}, {NULL}};
const Segment SGI243365[] = {{&S2357[13], 6}, {NULL}};
const Segment SGI243366[] = {{&S2357[38], 5}, {NULL}};
const Segment SGI243367[] = {{&S2357[78], 6}, {NULL}};
const Segment SGI243368[] = {{&S2357[112], 4}, {NULL}};
const Segment SGI243369[] = {{&S2359[13], 6}, {&S2359[13], 6}, {NULL}};
const Segment SGI243370[] = {{&S2359[35], 5}, {NULL}};
const Segment SGI243371[] = {{&S2359[68], 6}, {NULL}};
const Segment SGI243372[] = {{&S2359[102], 6}, {NULL}};
const Segment SGI243373[] = {{&S2360[13], 6}, {NULL}};
const Segment SGI243374[] = {{&S2360[133], 5}, {NULL}};
const Segment SGI243375[] = {{&S2360[35], 5}, {NULL}};
const Segment SGI243376[] = {{&S2360[68], 6}, {NULL}};
const Segment SGI243377[] = {{&S2360[102], 6}, {NULL}};
const Segment SGI243378[] = {{&S2361[13], 6}, {NULL}};
const Segment SGI243379[] = {{&S2361[42], 5}, {NULL}};
const Segment SGI243380[] = {{&S2361[125], 5}, {NULL}};
const Segment SGI243381[] = {{&S2362[13], 6}, {NULL}};
const Segment SGI243382[] = {{&S2362[133], 5}, {NULL}};
const Segment SGI243383[] = {{&S2362[35], 5}, {NULL}};
const Segment SGI243384[] = {{&S2362[68], 6}, {NULL}};
const Segment SGI243385[] = {{&S2362[102], 6}, {NULL}};
const Segment SGI243386[] = {{&S2363[13], 6}, {NULL}};
const Segment SGI243387[] = {{&S2363[133], 5}, {NULL}};
const Segment SGI243388[] = {{&S2363[35], 5}, {NULL}};
const Segment SGI243389[] = {{&S2363[68], 6}, {NULL}};
const Segment SGI243390[] = {{&S2363[102], 6}, {NULL}};
const Segment SGI243391[] = {{&S90103[34], 10}, {NULL}};
const Segment SGI243392[] = {{&S90103[27], 6}, {NULL}};
const Segment SGI243393[] = {{&S90103[96], 9}, {NULL}};
const Segment SGI243394[] = {{&S90103[69], 6}, {NULL}};
const Segment SGI243395[] = {{&S2364[13], 6}, {NULL}};
const Segment SGI243396[] = {{&S2364[35], 5}, {NULL}};
const Segment SGI243397[] = {{&S2364[68], 6}, {NULL}};
const Segment SGI243398[] = {{&S2364[102], 6}, {NULL}};
const Segment SGI243399[] = {{&S2364[116], 7}, {NULL}};
const Segment SGI243400[] = {{&S90116[34], 9}, {NULL}};
const Segment SGI243401[] = {{&S90116[27], 6}, {NULL}};
const Segment SGI243402[] = {{&S90116[96], 9}, {NULL}};
const Segment SGI243403[] = {{&S90116[69], 6}, {NULL}};
const Segment SGI243404[] = {{&S90117[34], 10}, {NULL}};
const Segment SGI243405[] = {{&S90117[27], 6}, {NULL}};
const Segment SGI243406[] = {{&S90117[97], 9}, {NULL}};
const Segment SGI243407[] = {{&S90117[70], 6}, {NULL}};
const Segment SGI243408[] = {{&S90140[52], 10}, {NULL}};
const Segment SGI243409[] = {{&S90140[52], 21}, {NULL}};
const Segment SGI243410[] = {{&S90140[38], 6}, {NULL}};
const Segment SGI243411[] = {{&S90140[87], 5}, {NULL}};
const Segment SGI243412[] = {{&S90142[34], 9}, {NULL}};
const Segment SGI243413[] = {{&S90142[27], 6}, {NULL}};
const Segment SGI243414[] = {{&S90142[95], 9}, {NULL}};
const Segment SGI243415[] = {{&S90142[68], 6}, {NULL}};
const Segment SGI243416[] = {{&S90143[34], 4}, {NULL}};
const Segment SGI243417[] = {{&S90143[27], 6}, {NULL}};
const Segment SGI243418[] = {{&S90143[90], 9}, {NULL}};
const Segment SGI243419[] = {{&S90143[63], 6}, {NULL}};
const Segment SGI243420[] = {{&S90156[44], 10}, {NULL}};
const Segment SGI243421[] = {{&S90156[44], 21}, {NULL}};
const Segment SGI243422[] = {{&S90156[30], 6}, {NULL}};
const Segment SGI243423[] = {{&S90156[112], 3}, {NULL}};
const Segment SGI243424[] = {{&S90156[79], 5}, {NULL}};
const Segment SGI243425[] = {{&S97380[44], 7}, {NULL}};
const Segment SGI243426[] = {{&S97380[44], 19}, {NULL}};
const Segment SGI243427[] = {{&S97380[30], 6}, {NULL}};
const Segment SGI243428[] = {{&S97380[77], 6}, {NULL}};
const Segment SGI243429[] = {{&S90167[34], 10}, {NULL}};
const Segment SGI243430[] = {{&S90167[27], 6}, {NULL}};
const Segment SGI243431[] = {{&S97380[104], 13}, {NULL}};
const Segment SGI243432[] = {{&S90167[96], 9}, {NULL}};
const Segment SGI243433[] = {{&S90167[69], 6}, {NULL}};
const Segment SGI243434[] = {{&S90167[137], 8}, {NULL}};
const Segment SGI243435[] = {{&S97386[34], 7}, {NULL}};
const Segment SGI243436[] = {{&S97386[27], 6}, {NULL}};
const Segment SGI243437[] = {{&S97386[90], 9}, {NULL}};
const Segment SGI243438[] = {{&S97386[63], 6}, {NULL}};
const Segment SGI243439[] = {{&S97389[34], 7}, {NULL}};
const Segment SGI243440[] = {{&S97389[27], 6}, {NULL}};
const Segment SGI243441[] = {{&S97389[94], 9}, {NULL}};
const Segment SGI243442[] = {{&S97389[67], 6}, {NULL}};
const Segment SGI243443[] = {{&S97392[34], 10}, {NULL}};
const Segment SGI243444[] = {{&S97392[27], 6}, {NULL}};
const Segment SGI243445[] = {{&S97392[97], 9}, {NULL}};
const Segment SGI243446[] = {{&S97392[70], 6}, {NULL}};
const Segment SGI243447[] = {{&S97396[34], 14}, {NULL}};
const Segment SGI243448[] = {{&S97396[27], 6}, {NULL}};
const Segment SGI243449[] = {{&S97396[106], 9}, {NULL}};
const Segment SGI243450[] = {{&S97396[79], 6}, {NULL}};
const Segment SGI243451[] = {{&S97428[34], 14}, {NULL}};
const Segment SGI243452[] = {{&S97428[27], 6}, {NULL}};
const Segment SGI243453[] = {{&S97428[100], 9}, {NULL}};
const Segment SGI243454[] = {{&S97428[74], 5}, {NULL}};
const Segment SGI243455[] = {{&S97431[34], 7}, {NULL}};
const Segment SGI243456[] = {{&S97431[27], 6}, {NULL}};
const Segment SGI243457[] = {{&S97431[94], 9}, {NULL}};
const Segment SGI243458[] = {{&S97431[67], 6}, {NULL}};
const Segment SGI243459[] = {{&S97433[44], 7}, {NULL}};
const Segment SGI243460[] = {{&S97433[44], 19}, {NULL}};
const Segment SGI243461[] = {{&S97433[30], 6}, {NULL}};
const Segment SGI243462[] = {{&S97433[112], 3}, {NULL}};
const Segment SGI243463[] = {{&S97433[77], 6}, {NULL}};
const Segment SGI243464[] = {{&S97435[45], 10}, {NULL}};
const Segment SGI243465[] = {{&S97435[30], 6}, {NULL}};
const Segment SGI243466[] = {{&S97435[115], 3}, {NULL}};
const Segment SGI243467[] = {{&S97435[81], 5}, {NULL}};
const Segment SGI243468[] = {{&S97435[139], 13}, {NULL}};
const Segment SGI243469[] = {{&S105541[44], 10}, {NULL}};
const Segment SGI243470[] = {{&S105541[44], 22}, {NULL}};
const Segment SGI243471[] = {{&S105541[30], 6}, {NULL}};
const Segment SGI243472[] = {{&S105541[144], 3}, {NULL}};
const Segment SGI243473[] = {{&S105541[109], 6}, {NULL}};
const Segment SGI243474[] = {{&S105541[68], 8}, {NULL}};
const Segment SGI243475[] = {{&S105541[80], 15}, {NULL}};
const Segment SGI243476[] = {{&S90254[34], 13}, {NULL}};
const Segment SGI243477[] = {{&S90254[27], 6}, {NULL}};
const Segment SGI243478[] = {{&S90254[99], 9}, {NULL}};
const Segment SGI243479[] = {{&S90254[72], 6}, {NULL}};
const Segment SGI243480[] = {{&S90255[34], 12}, {NULL}};
const Segment SGI243481[] = {{&S90255[27], 6}, {NULL}};
const Segment SGI243482[] = {{&S90255[98], 9}, {NULL}};
const Segment SGI243483[] = {{&S90255[71], 6}, {NULL}};
const Segment SGI243484[] = {{&S105545[44], 20}, {NULL}};
const Segment SGI243485[] = {{&S105545[44], 32}, {NULL}};
const Segment SGI243486[] = {{&S105545[30], 6}, {NULL}};
const Segment SGI243487[] = {{&S105545[148], 3}, {NULL}};
const Segment SGI243488[] = {{&S105545[113], 6}, {NULL}};
const Segment SGI243489[] = {{&S105545[78], 8}, {NULL}};
const Segment SGI243490[] = {{&S105545[92], 8}, {NULL}};
const Segment SGI243491[] = {{&S97464[34], 15}, {NULL}};
const Segment SGI243492[] = {{&S97464[27], 6}, {NULL}};
const Segment SGI243493[] = {{&S97464[102], 9}, {NULL}};
const Segment SGI243494[] = {{&S97464[75], 6}, {NULL}};
const Segment SGI243495[] = {{&S105546[44], 20}, {NULL}};
const Segment SGI243496[] = {{&S105546[44], 32}, {NULL}};
const Segment SGI243497[] = {{&S105546[30], 6}, {NULL}};
const Segment SGI243498[] = {{&S105546[125], 3}, {NULL}};
const Segment SGI243499[] = {{&S105546[90], 6}, {NULL}};
const Segment SGI243500[] = {{&S105546[160], 3}, {NULL}};
const Segment SGI243501[] = {{&S97468[44], 10}, {NULL}};
const Segment SGI243502[] = {{&S97468[44], 22}, {NULL}};
const Segment SGI243503[] = {{&S97468[30], 6}, {NULL}};
const Segment SGI243504[] = {{&S97468[115], 3}, {NULL}};
const Segment SGI243505[] = {{&S97468[80], 6}, {NULL}};
const Segment SGI243506[] = {{&S105548[44], 22}, {NULL}};
const Segment SGI243507[] = {{&S97468[140], 13}, {NULL}};
const Segment SGI243508[] = {{&S105548[30], 6}, {NULL}};
const Segment SGI243509[] = {{&S105548[125], 2}, {NULL}};
const Segment SGI243510[] = {{&S105548[94], 5}, {NULL}};
const Segment SGI243511[] = {{&S105549[44], 9}, {NULL}};
const Segment SGI243512[] = {{&S105549[44], 73}, {NULL}};
const Segment SGI243513[] = {{&S105549[30], 6}, {NULL}};
const Segment SGI243514[] = {{&S90263[34], 10}, {NULL}};
const Segment SGI243515[] = {{&S90263[27], 6}, {NULL}};
const Segment SGI243516[] = {{&S90263[96], 9}, {NULL}};
const Segment SGI243517[] = {{&S90263[69], 6}, {NULL}};
const Segment SGI243518[] = {{&S97478[44], 9}, {NULL}};
const Segment SGI243519[] = {{&S97478[44], 21}, {NULL}};
const Segment SGI243520[] = {{&S97478[30], 6}, {NULL}};
const Segment SGI243521[] = {{&S97478[134], 3}, {NULL}};
const Segment SGI243522[] = {{&S97478[99], 6}, {NULL}};
const Segment SGI243523[] = {{&S97478[67], 8}, {NULL}};
const Segment SGI243524[] = {{&S97493[34], 11}, {NULL}};
const Segment SGI243525[] = {{&S97493[27], 6}, {NULL}};
const Segment SGI243526[] = {{&S97493[98], 9}, {NULL}};
const Segment SGI243527[] = {{&S97493[71], 6}, {NULL}};
const Segment SGI243528[] = {{&S97503[44], 9}, {NULL}};
const Segment SGI243529[] = {{&S97503[44], 21}, {NULL}};
const Segment SGI243530[] = {{&S97503[30], 6}, {NULL}};
const Segment SGI243531[] = {{&S97503[79], 6}, {NULL}};
const Segment SGI243532[] = {{&S97503[106], 13}, {NULL}};
const Segment SGI243533[] = {{&S97504[44], 7}, {NULL}};
const Segment SGI243534[] = {{&S97504[44], 24}, {NULL}};
const Segment SGI243535[] = {{&S90301[44], 9}, {NULL}};
const Segment SGI243536[] = {{&S97504[30], 6}, {NULL}};
const Segment SGI243537[] = {{&S90301[44], 21}, {NULL}};
const Segment SGI243538[] = {{&S90301[30], 6}, {NULL}};
const Segment SGI243539[] = {{&S97504[115], 3}, {NULL}};
const Segment SGI243540[] = {{&S90301[114], 3}, {NULL}};
const Segment SGI243541[] = {{&S97504[82], 5}, {NULL}};
const Segment SGI243542[] = {{&S90301[79], 6}, {NULL}};
const Segment SGI243543[] = {{&S97504[119], 14}, {NULL}};
const Segment SGI243544[] = {{&S97506[34], 9}, {NULL}};
const Segment SGI243545[] = {{&S97506[27], 6}, {NULL}};
const Segment SGI243546[] = {{&S97506[96], 9}, {NULL}};
const Segment SGI243547[] = {{&S97506[69], 6}, {NULL}};
const Segment SGI243548[] = {{&S97506[130], 8}, {NULL}};
const Segment SGI243549[] = {{&S97509[44], 14}, {NULL}};
const Segment SGI243550[] = {{&S97509[44], 26}, {NULL}};
const Segment SGI243551[] = {{&S97509[30], 6}, {NULL}};
const Segment SGI243552[] = {{&S97509[84], 6}, {NULL}};
const Segment SGI243553[] = {{&S97509[111], 13}, {NULL}};
const Segment SGI243554[] = {{&S90314[38], 10}, {NULL}};
const Segment SGI243555[] = {{&S90314[31], 6}, {NULL}};
const Segment SGI243556[] = {{&S90315[44], 10}, {NULL}};
const Segment SGI243557[] = {{&S90315[44], 21}, {NULL}};
const Segment SGI243558[] = {{&S90315[30], 6}, {NULL}};
const Segment SGI243559[] = {{&S90315[114], 3}, {NULL}};
const Segment SGI243560[] = {{&S90315[79], 6}, {NULL}};
const Segment SGI243561[] = {{&S90316[44], 10}, {NULL}};
const Segment SGI243562[] = {{&S90316[44], 21}, {NULL}};
const Segment SGI243563[] = {{&S90316[30], 6}, {NULL}};
const Segment SGI243564[] = {{&S90316[79], 6}, {NULL}};
const Segment SGI243565[] = {{&S90316[106], 13}, {NULL}};
const Segment SGI243566[] = {{&S97523[34], 9}, {NULL}};
const Segment SGI243567[] = {{&S97523[27], 6}, {NULL}};
const Segment SGI243568[] = {{&S97523[96], 9}, {NULL}};
const Segment SGI243569[] = {{&S97523[69], 6}, {NULL}};
const Segment SGI243570[] = {{&S97526[34], 10}, {NULL}};
const Segment SGI243571[] = {{&S97526[27], 6}, {NULL}};
const Segment SGI243572[] = {{&S97526[97], 9}, {NULL}};
const Segment SGI243573[] = {{&S97526[70], 6}, {NULL}};
const Segment SGI243574[] = {{&S97529[34], 10}, {NULL}};
const Segment SGI243575[] = {{&S97529[27], 6}, {NULL}};
const Segment SGI243576[] = {{&S97529[97], 9}, {NULL}};
const Segment SGI243577[] = {{&S97529[70], 6}, {NULL}};
const Segment SGI243578[] = {{&S97530[34], 11}, {NULL}};
const Segment SGI243579[] = {{&S97530[27], 6}, {NULL}};
const Segment SGI243580[] = {{&S97530[98], 9}, {NULL}};
const Segment SGI243581[] = {{&S97530[71], 6}, {NULL}};
const Segment SGI243582[] = {{&S90331[34], 8}, {NULL}};
const Segment SGI243583[] = {{&S90331[27], 6}, {NULL}};
const Segment SGI243584[] = {{&S90331[94], 9}, {NULL}};
const Segment SGI243585[] = {{&S90331[67], 6}, {NULL}};
const Segment SGI243586[] = {{&S97534[34], 10}, {NULL}};
const Segment SGI243587[] = {{&S97534[27], 6}, {NULL}};
const Segment SGI243588[] = {{&S97534[97], 9}, {NULL}};
const Segment SGI243589[] = {{&S97534[70], 6}, {NULL}};
const Segment SGI243590[] = {{&S105627[44], 10}, {NULL}};
const Segment SGI243591[] = {{&S105627[44], 22}, {NULL}};
const Segment SGI243592[] = {{&S105627[30], 6}, {NULL}};
const Segment SGI243593[] = {{&S105627[115], 3}, {NULL}};
const Segment SGI243594[] = {{&S105627[80], 6}, {NULL}};
const Segment SGI243595[] = {{&S105627[143], 3}, {NULL}};
const Segment SGI243596[] = {{&S97540[34], 7}, {NULL}};
const Segment SGI243597[] = {{&S97540[27], 6}, {NULL}};
const Segment SGI243598[] = {{&S97540[99], 9}, {NULL}};
const Segment SGI243599[] = {{&S97540[72], 6}, {NULL}};
const Segment SGI243600[] = {{&S97545[44], 9}, {NULL}};
const Segment SGI243601[] = {{&S97545[44], 21}, {NULL}};
const Segment SGI243602[] = {{&S97545[30], 6}, {NULL}};
const Segment SGI243603[] = {{&S97545[114], 3}, {NULL}};
const Segment SGI243604[] = {{&S97545[79], 6}, {NULL}};
const Segment SGI243605[] = {{&S97550[34], 10}, {NULL}};
const Segment SGI243606[] = {{&S97550[27], 6}, {NULL}};
const Segment SGI243607[] = {{&S97550[97], 9}, {NULL}};
const Segment SGI243608[] = {{&S97550[70], 6}, {NULL}};
const Segment SGI243609[] = {{&S97554[44], 9}, {NULL}};
const Segment SGI243610[] = {{&S97554[44], 21}, {NULL}};
const Segment SGI243611[] = {{&S97554[30], 6}, {NULL}};
const Segment SGI243612[] = {{&S97554[114], 3}, {NULL}};
const Segment SGI243613[] = {{&S97554[79], 6}, {NULL}};
const Segment SGI243614[] = {{&S97554[139], 7}, {NULL}};
const Segment SGI243615[] = {{&S97559[52], 15}, {NULL}};
const Segment SGI243616[] = {{&S97559[52], 30}, {NULL}};
const Segment SGI243617[] = {{&S97559[38], 6}, {NULL}};
const Segment SGI243618[] = {{&S97559[96], 5}, {NULL}};
const Segment SGI243619[] = {{&S97559[11], 3}, {NULL}};
const Segment SGI243620[] = {{&S98834[44], 13}, {NULL}};
const Segment SGI243621[] = {{&S98834[44], 25}, {NULL}};
const Segment SGI243622[] = {{&S98834[30], 6}, {NULL}};
const Segment SGI243623[] = {{&S98834[116], 3}, {NULL}};
const Segment SGI243624[] = {{&S98834[83], 5}, {NULL}};
const Segment SGI243625[] = {{&S97564[44], 10}, {NULL}};
const Segment SGI243626[] = {{&S97564[44], 22}, {NULL}};
const Segment SGI243627[] = {{&S97564[30], 6}, {NULL}};
const Segment SGI243628[] = {{&S97564[80], 6}, {NULL}};
const Segment SGI243629[] = {{&S97564[107], 13}, {NULL}};
const Segment SGI243630[] = {{&S97565[34], 7}, {NULL}};
const Segment SGI243631[] = {{&S97565[27], 6}, {NULL}};
const Segment SGI243632[] = {{&S97565[99], 9}, {NULL}};
const Segment SGI243633[] = {{&S97565[72], 6}, {NULL}};
const Segment SGI243634[] = {{&S97565[140], 8}, {NULL}};
const Segment SGI243635[] = {{&S97570[44], 6}, {NULL}};
const Segment SGI243636[] = {{&S97570[44], 18}, {NULL}};
const Segment SGI243637[] = {{&S97570[30], 6}, {NULL}};
const Segment SGI243638[] = {{&S97570[76], 6}, {NULL}};
const Segment SGI243639[] = {{&S97570[103], 13}, {NULL}};
const Segment SGI243640[] = {{&S97572[44], 9}, {NULL}};
const Segment SGI243641[] = {{&S97572[44], 21}, {NULL}};
const Segment SGI243642[] = {{&S97572[30], 6}, {NULL}};
const Segment SGI243643[] = {{&S97572[114], 3}, {NULL}};
const Segment SGI243644[] = {{&S97572[79], 6}, {NULL}};
const Segment SGI243645[] = {{&S97576[52], 10}, {NULL}};
const Segment SGI243646[] = {{&S97576[52], 22}, {NULL}};
const Segment SGI243647[] = {{&S97576[38], 6}, {NULL}};
const Segment SGI243648[] = {{&S97576[88], 5}, {NULL}};
const Segment SGI243649[] = {{&S97576[11], 3}, {NULL}};
const Segment SGI243650[] = {{&S97580[34], 10}, {NULL}};
const Segment SGI243651[] = {{&S97580[27], 6}, {NULL}};
const Segment SGI243652[] = {{&S97580[97], 9}, {NULL}};
const Segment SGI243653[] = {{&S97580[70], 6}, {NULL}};
const Segment SGI243654[] = {{&S97584[44], 10}, {NULL}};
const Segment SGI243655[] = {{&S97584[44], 22}, {NULL}};
const Segment SGI243656[] = {{&S97584[30], 6}, {NULL}};
const Segment SGI243657[] = {{&S97584[115], 3}, {NULL}};
const Segment SGI243658[] = {{&S97584[80], 6}, {NULL}};
const Segment SGI243659[] = {{&S97584[135], 8}, {NULL}};
const Segment SGI243660[] = {{&S97584[144], 13}, {NULL}};
const Segment SGI243661[] = {{&S97594[44], 13}, {NULL}};
const Segment SGI243662[] = {{&S97594[44], 25}, {NULL}};
const Segment SGI243663[] = {{&S97594[30], 6}, {NULL}};
const Segment SGI243664[] = {{&S97594[83], 6}, {NULL}};
const Segment SGI243665[] = {{&S97594[110], 13}, {NULL}};
const Segment SGI243666[] = {{&S97596[44], 10}, {NULL}};
const Segment SGI243667[] = {{&S97596[44], 22}, {NULL}};
const Segment SGI243668[] = {{&S97596[30], 6}, {NULL}};
const Segment SGI243669[] = {{&S97596[80], 6}, {NULL}};
const Segment SGI243670[] = {{&S97596[107], 13}, {NULL}};
const Segment SGI243671[] = {{&S97598[34], 10}, {NULL}};
const Segment SGI243672[] = {{&S97598[27], 6}, {NULL}};
const Segment SGI243673[] = {{&S97598[97], 9}, {NULL}};
const Segment SGI243674[] = {{&S97598[70], 6}, {NULL}};
const Segment SGI243675[] = {{&S97611[44], 10}, {NULL}};
const Segment SGI243676[] = {{&S97611[44], 22}, {NULL}};
const Segment SGI243677[] = {{&S97611[30], 6}, {NULL}};
const Segment SGI243678[] = {{&S97611[80], 6}, {NULL}};
const Segment SGI243679[] = {{&S97611[107], 13}, {NULL}};
const Segment SGI243680[] = {{&S97618[34], 9}, {NULL}};
const Segment SGI243681[] = {{&S105708[45], 10}, {NULL}};
const Segment SGI243682[] = {{&S97618[27], 6}, {NULL}};
const Segment SGI243683[] = {{&S105708[30], 6}, {NULL}};
const Segment SGI243684[] = {{&S97618[96], 9}, {NULL}};
const Segment SGI243685[] = {{&S105708[115], 3}, {NULL}};
const Segment SGI243686[] = {{&S97618[69], 6}, {NULL}};
const Segment SGI243687[] = {{&S105708[81], 5}, {NULL}};
const Segment SGI243688[] = {{&S105708[139], 13}, {NULL}};
const Segment SGI243689[] = {{&S97619[34], 10}, {NULL}};
const Segment SGI243690[] = {{&S97619[27], 6}, {NULL}};
const Segment SGI243691[] = {{&S105709[44], 10}, {NULL}};
const Segment SGI243692[] = {{&S105709[44], 22}, {NULL}};
const Segment SGI243693[] = {{&S105709[30], 6}, {NULL}};
const Segment SGI243694[] = {{&S97619[97], 9}, {NULL}};
const Segment SGI243695[] = {{&S97619[70], 6}, {NULL}};
const Segment SGI243696[] = {{&S105709[111], 2}, {NULL}};
const Segment SGI243697[] = {{&S105709[80], 5}, {NULL}};
const Segment SGI243698[] = {{&S97620[34], 7}, {NULL}};
const Segment SGI243699[] = {{&S97620[27], 6}, {NULL}};
const Segment SGI243700[] = {{&S97620[107], 9}, {NULL}};
const Segment SGI243701[] = {{&S97620[80], 6}, {NULL}};
const Segment SGI243702[] = {{&S105717[59], 10}, {NULL}};
const Segment SGI243703[] = {{&S105717[59], 22}, {NULL}};
const Segment SGI243704[] = {{&S105717[45], 6}, {NULL}};
const Segment SGI243705[] = {{&S105717[130], 3}, {NULL}};
const Segment SGI243706[] = {{&S105717[95], 6}, {NULL}};
const Segment SGI243707[] = {{&S105717[0], 14}, {NULL}};
const Segment SGI243708[] = {{&S90455[34], 7}, {NULL}};
const Segment SGI243709[] = {{&S90455[27], 6}, {NULL}};
const Segment SGI243710[] = {{&S90455[102], 9}, {NULL}};
const Segment SGI243711[] = {{&S90455[75], 6}, {NULL}};
const Segment SGI243712[] = {{&S90477[44], 11}, {NULL}};
const Segment SGI243713[] = {{&S90477[44], 22}, {NULL}};
const Segment SGI243714[] = {{&S90477[30], 6}, {NULL}};
const Segment SGI243715[] = {{&S90477[133], 3}, {NULL}};
const Segment SGI243716[] = {{&S90477[98], 6}, {NULL}};
const Segment SGI243717[] = {{&S90477[68], 8}, {NULL}};
const Segment SGI243718[] = {{&S105745[44], 10}, {NULL}};
const Segment SGI243719[] = {{&S105745[44], 22}, {NULL}};
const Segment SGI243720[] = {{&S105745[30], 6}, {NULL}};
const Segment SGI243721[] = {{&S105745[115], 3}, {NULL}};
const Segment SGI243722[] = {{&S105745[80], 6}, {NULL}};
const Segment SGI243723[] = {{&S105745[135], 8}, {NULL}};
const Segment SGI243724[] = {{&S105745[144], 13}, {NULL}};
const Segment SGI243725[] = {{&S98864[34], 19}, {NULL}};
const Segment SGI243726[] = {{&S98864[27], 6}, {NULL}};
const Segment SGI243727[] = {{&S98864[106], 9}, {NULL}};
const Segment SGI243728[] = {{&S98864[79], 6}, {NULL}};
const Segment SGI243729[] = {{&S105750[44], 10}, {NULL}};
const Segment SGI243730[] = {{&S105750[44], 22}, {NULL}};
const Segment SGI243731[] = {{&S105750[30], 6}, {NULL}};
const Segment SGI243732[] = {{&S105750[114], 3}, {NULL}};
const Segment SGI243733[] = {{&S105750[80], 5}, {NULL}};
const Segment SGI243734[] = {{&S105750[125], 14}, {NULL}};
const Segment SGI243735[] = {{&S98867[42], 14}, {NULL}};
const Segment SGI243736[] = {{&S98867[30], 6}, {NULL}};
const Segment SGI243737[] = {{&S98867[130], 3}, {NULL}};
const Segment SGI243738[] = {{&S90489[34], 10}, {NULL}};
const Segment SGI243739[] = {{&S98867[95], 6}, {NULL}};
const Segment SGI243740[] = {{&S90489[27], 6}, {NULL}};
const Segment SGI243741[] = {{&S90489[97], 9}, {NULL}};
const Segment SGI243742[] = {{&S90489[70], 6}, {NULL}};
const Segment SGI243743[] = {{&S90490[34], 10}, {NULL}};
const Segment SGI243744[] = {{&S90490[27], 6}, {NULL}};
const Segment SGI243745[] = {{&S90490[97], 9}, {NULL}};
const Segment SGI243746[] = {{&S90490[70], 6}, {NULL}};
const Segment SGI243747[] = {{&S98875[44], 7}, {NULL}};
const Segment SGI243748[] = {{&S98875[44], 19}, {NULL}};
const Segment SGI243749[] = {{&S98875[30], 6}, {NULL}};
const Segment SGI243750[] = {{&S98875[77], 6}, {NULL}};
const Segment SGI243751[] = {{&S98875[114], 3}, {NULL}};
const Segment SGI243752[] = {{&S98879[34], 10}, {NULL}};
const Segment SGI243753[] = {{&S98879[27], 6}, {NULL}};
const Segment SGI243754[] = {{&S98879[97], 9}, {NULL}};
const Segment SGI243755[] = {{&S98879[70], 6}, {NULL}};
const Segment SGI243756[] = {{&S90515[34], 4}, {NULL}};
const Segment SGI243757[] = {{&S90515[27], 6}, {NULL}};
const Segment SGI243758[] = {{&S105776[59], 10}, {NULL}};
const Segment SGI243759[] = {{&S105776[59], 22}, {NULL}};
const Segment SGI243760[] = {{&S105776[45], 6}, {NULL}};
const Segment SGI243761[] = {{&S90515[90], 9}, {NULL}};
const Segment SGI243762[] = {{&S90515[63], 6}, {NULL}};
const Segment SGI243763[] = {{&S105776[130], 3}, {NULL}};
const Segment SGI243764[] = {{&S97704[45], 9}, {NULL}};
const Segment SGI243765[] = {{&S105776[95], 6}, {NULL}};
const Segment SGI243766[] = {{&S97704[30], 6}, {NULL}};
const Segment SGI243767[] = {{&S105776[0], 14}, {NULL}};
const Segment SGI243768[] = {{&S97704[114], 3}, {NULL}};
const Segment SGI243769[] = {{&S97704[80], 5}, {NULL}};
const Segment SGI243770[] = {{&S97704[138], 13}, {NULL}};
const Segment SGI243771[] = {{&S105780[44], 10}, {NULL}};
const Segment SGI243772[] = {{&S105780[44], 22}, {NULL}};
const Segment SGI243773[] = {{&S105780[30], 6}, {NULL}};
const Segment SGI243774[] = {{&S105780[131], 3}, {NULL}};
const Segment SGI243775[] = {{&S105780[96], 6}, {NULL}};
const Segment SGI243776[] = {{&S105780[148], 10}, {NULL}};
const Segment SGI243777[] = {{&S105780[68], 8}, {NULL}};
const Segment SGI243778[] = {{&S105781[44], 10}, {NULL}};
const Segment SGI243779[] = {{&S105781[44], 22}, {NULL}};
const Segment SGI243780[] = {{&S105781[30], 6}, {NULL}};
const Segment SGI243781[] = {{&S105781[113], 3}, {NULL}};
const Segment SGI243782[] = {{&S105781[80], 5}, {NULL}};
const Segment SGI243783[] = {{&S105781[117], 14}, {NULL}};
const Segment SGI243784[] = {{&S2685[13], 6}, {NULL}};
const Segment SGI243785[] = {{&S2685[38], 3}, {NULL}};
const Segment SGI243786[] = {{&S2686[13], 6}, {NULL}};
const Segment SGI243787[] = {{&S2686[35], 3}, {NULL}};
const Segment SGI243788[] = {{&S2686[66], 6}, {NULL}};
const Segment SGI243789[] = {{&S2686[100], 8}, {NULL}};
const Segment SGI243790[] = {{&S2687[13], 6}, {NULL}};
const Segment SGI243791[] = {{&S2687[35], 3}, {NULL}};
const Segment SGI243792[] = {{&S2687[66], 6}, {NULL}};
const Segment SGI243793[] = {{&S2687[100], 8}, {NULL}};
const Segment SGI243794[] = {{&S2688[13], 6}, {NULL}};
const Segment SGI243795[] = {{&S2688[35], 3}, {NULL}};
const Segment SGI243796[] = {{&S2688[66], 6}, {NULL}};
const Segment SGI243797[] = {{&S2689[13], 6}, {NULL}};
const Segment SGI243798[] = {{&S2689[35], 3}, {NULL}};
const Segment SGI243799[] = {{&S2689[66], 6}, {NULL}};
const Segment SGI243800[] = {{&S2689[100], 8}, {NULL}};
const Segment SGI243801[] = {{&S2690[13], 6}, {NULL}};
const Segment SGI243802[] = {{&S2690[35], 3}, {NULL}};
const Segment SGI243803[] = {{&S2690[66], 6}, {NULL}};
const Segment SGI243804[] = {{&S2690[100], 6}, {NULL}};
const Segment SGI243805[] = {{&S105790[59], 10}, {NULL}};
const Segment SGI243806[] = {{&S105790[59], 22}, {NULL}};
const Segment SGI243807[] = {{&S105790[45], 6}, {NULL}};
const Segment SGI243808[] = {{&S105790[130], 3}, {NULL}};
const Segment SGI243809[] = {{&S2691[13], 6}, {NULL}};
const Segment SGI243810[] = {{&S2691[35], 3}, {NULL}};
const Segment SGI243811[] = {{&S105790[95], 6}, {NULL}};
const Segment SGI243812[] = {{&S2691[66], 6}, {NULL}};
const Segment SGI243813[] = {{&S2691[100], 6}, {NULL}};
const Segment SGI243814[] = {{&S105790[0], 14}, {NULL}};
const Segment SGI243815[] = {{&S2692[13], 6}, {&S2692[13], 6}, {NULL}};
const Segment SGI243816[] = {{&S2692[73], 6}, {NULL}};
const Segment SGI243817[] = {{&S2692[107], 4}, {NULL}};
const Segment SGI243818[] = {{&S2693[13], 6}, {&S2693[107], 4}, {NULL}};
const Segment SGI243819[] = {{&S2693[35], 3}, {NULL}};
const Segment SGI243820[] = {{&S2693[66], 6}, {NULL}};
const Segment SGI243821[] = {{&S2693[100], 6}, {NULL}};
const Segment SGI243822[] = {{&S2694[17], 6}, {NULL}};
const Segment SGI243823[] = {{&S2694[27], 3}, {NULL}};
const Segment SGI243824[] = {{&S2695[13], 6}, {NULL}};
const Segment SGI243825[] = {{&S2695[28], 3}, {NULL}};
const Segment SGI243826[] = {{&S2695[59], 6}, {NULL}};
const Segment SGI243827[] = {{&S2695[93], 6}, {NULL}};
const Segment SGI243828[] = {{&S2695[107], 7}, {NULL}};
const Segment SGI243829[] = {{&S105795[44], 10}, {NULL}};
const Segment SGI243830[] = {{&S105795[44], 22}, {NULL}};
const Segment SGI243831[] = {{&S105795[30], 6}, {NULL}};
const Segment SGI243832[] = {{&S105795[115], 3}, {NULL}};
const Segment SGI243833[] = {{&S105795[80], 6}, {NULL}};
const Segment SGI243834[] = {{&S105795[143], 3}, {NULL}};
const Segment SGI243835[] = {{&S2696[13], 6}, {NULL}};
const Segment SGI243836[] = {{&S2696[41], 3}, {NULL}};
const Segment SGI243837[] = {{&S2696[122], 5}, {NULL}};
const Segment SGI243838[] = {{&S2697[44], 6}, {NULL}};
const Segment SGI243839[] = {{&S2697[69], 3}, {NULL}};
const Segment SGI243840[] = {{&S2697[107], 6}, {NULL}};
const Segment SGI243841[] = {{&S2698[13], 6}, {NULL}};
const Segment SGI243842[] = {{&S2698[35], 3}, {NULL}};
const Segment SGI243843[] = {{&S2698[66], 6}, {NULL}};
const Segment SGI243844[] = {{&S2698[100], 6}, {NULL}};
const Segment SGI243845[] = {{&S2699[13], 6}, {NULL}};
const Segment SGI243846[] = {{&S2699[40], 3}, {NULL}};
const Segment SGI243847[] = {{&S2699[121], 5}, {NULL}};
const Segment SGI243848[] = {{&S2700[13], 6}, {NULL}};
const Segment SGI243849[] = {{&S2700[40], 3}, {NULL}};
const Segment SGI243850[] = {{&S2700[121], 5}, {NULL}};
const Segment SGI243851[] = {{&S2701[13], 6}, {NULL}};
const Segment SGI243852[] = {{&S2701[28], 3}, {NULL}};
const Segment SGI243853[] = {{&S2701[59], 6}, {NULL}};
const Segment SGI243854[] = {{&S2701[93], 6}, {NULL}};
const Segment SGI243855[] = {{&S2701[107], 7}, {NULL}};
const Segment SGI243856[] = {{&S2702[17], 6}, {&S2702[32], 4}, {NULL}};
const Segment SGI243857[] = {{&S2702[27], 3}, {NULL}};
const Segment SGI243858[] = {{&S2703[13], 6}, {NULL}};
const Segment SGI243859[] = {{&S2703[131], 3}, {NULL}};
const Segment SGI243860[] = {{&S2703[35], 3}, {NULL}};
const Segment SGI243861[] = {{&S2703[66], 6}, {NULL}};
const Segment SGI243862[] = {{&S2703[100], 6}, {NULL}};
const Segment SGI243863[] = {{&S2704[13], 6}, {NULL}};
const Segment SGI243864[] = {{&S2704[35], 5}, {NULL}};
const Segment SGI243865[] = {{&S2704[68], 6}, {NULL}};
const Segment SGI243866[] = {{&S2705[13], 6}, {NULL}};
const Segment SGI243867[] = {{&S2705[35], 5}, {NULL}};
const Segment SGI243868[] = {{&S2705[68], 6}, {NULL}};
const Segment SGI243869[] = {{&S2705[102], 6}, {NULL}};
const Segment SGI243870[] = {{&S2706[13], 6}, {NULL}};
const Segment SGI243871[] = {{&S2706[65], 6}, {NULL}};
const Segment SGI243872[] = {{&S2706[99], 6}, {NULL}};
const Segment SGI243873[] = {{&S2707[13], 6}, {NULL}};
const Segment SGI243874[] = {{&S2707[35], 5}, {NULL}};
const Segment SGI243875[] = {{&S2707[68], 6}, {NULL}};
const Segment SGI243876[] = {{&S2707[102], 6}, {NULL}};
const Segment SGI243877[] = {{&S2708[13], 6}, {NULL}};
const Segment SGI243878[] = {{&S2708[44], 3}, {NULL}};
const Segment SGI243879[] = {{&S2708[125], 5}, {NULL}};
const Segment SGI243880[] = {{&S105812[44], 10}, {NULL}};
const Segment SGI243881[] = {{&S105812[44], 22}, {NULL}};
const Segment SGI243882[] = {{&S105812[30], 6}, {NULL}};
const Segment SGI243883[] = {{&S105812[159], 3}, {NULL}};
const Segment SGI243884[] = {{&S105812[125], 3}, {NULL}};
const Segment SGI243885[] = {{&S2709[13], 6}, {NULL}};
const Segment SGI243886[] = {{&S105812[90], 6}, {NULL}};
const Segment SGI243887[] = {{&S2709[44], 3}, {NULL}};
const Segment SGI243888[] = {{&S105812[145], 8}, {NULL}};
const Segment SGI243889[] = {{&S2709[125], 5}, {NULL}};
const Segment SGI243890[] = {{&S2710[13], 6}, {NULL}};
const Segment SGI243891[] = {{&S2710[38], 5}, {NULL}};
const Segment SGI243892[] = {{&S2711[13], 6}, {NULL}};
const Segment SGI243893[] = {{&S2711[28], 3}, {NULL}};
const Segment SGI243894[] = {{&S2711[59], 6}, {NULL}};
const Segment SGI243895[] = {{&S2711[93], 6}, {NULL}};
const Segment SGI243896[] = {{&S2711[107], 7}, {NULL}};
const Segment SGI243897[] = {{&S2712[13], 6}, {NULL}};
const Segment SGI243898[] = {{&S2712[35], 3}, {NULL}};
const Segment SGI243899[] = {{&S2712[66], 6}, {NULL}};
const Segment SGI243900[] = {{&S2712[100], 6}, {NULL}};
const Segment SGI243901[] = {{&S2713[13], 6}, {NULL}};
const Segment SGI243902[] = {{&S2713[28], 3}, {NULL}};
const Segment SGI243903[] = {{&S2713[59], 6}, {NULL}};
const Segment SGI243904[] = {{&S2713[93], 6}, {NULL}};
const Segment SGI243905[] = {{&S2713[107], 7}, {NULL}};
const Segment SGI243906[] = {{&S2714[13], 6}, {&S2714[109], 4}, {NULL}};
const Segment SGI243907[] = {{&S2714[35], 5}, {NULL}};
const Segment SGI243908[] = {{&S2714[68], 6}, {NULL}};
const Segment SGI243909[] = {{&S2714[102], 6}, {NULL}};
const Segment SGI243910[] = {{&S2715[13], 6}, {NULL}};
const Segment SGI243911[] = {{&S2715[35], 5}, {NULL}};
const Segment SGI243912[] = {{&S2715[68], 6}, {NULL}};
const Segment SGI243913[] = {{&S2715[102], 7}, {NULL}};
const Segment SGI243914[] = {{&S2716[13], 6}, {NULL}};
const Segment SGI243915[] = {{&S2716[35], 5}, {NULL}};
const Segment SGI243916[] = {{&S2716[68], 6}, {NULL}};
const Segment SGI243917[] = {{&S2716[102], 6}, {NULL}};
const Segment SGI243918[] = {{&S2716[116], 9}, {NULL}};
const Segment SGI243919[] = {{&S97765[59], 10}, {NULL}};
const Segment SGI243920[] = {{&S97765[59], 22}, {NULL}};
const Segment SGI243921[] = {{&S97765[45], 6}, {NULL}};
const Segment SGI243922[] = {{&S2717[13], 6}, {NULL}};
const Segment SGI243923[] = {{&S2717[133], 3}, {NULL}};
const Segment SGI243924[] = {{&S97765[130], 3}, {NULL}};
const Segment SGI243925[] = {{&S2717[35], 5}, {NULL}};
const Segment SGI243926[] = {{&S97765[95], 6}, {NULL}};
const Segment SGI243927[] = {{&S2717[68], 6}, {NULL}};
const Segment SGI243928[] = {{&S2717[102], 6}, {NULL}};
const Segment SGI243929[] = {{&S97765[11], 3}, {NULL}};
const Segment SGI243930[] = {{&S2718[13], 6}, {NULL}};
const Segment SGI243931[] = {{&S2718[40], 5}, {NULL}};
const Segment SGI243932[] = {{&S2718[123], 5}, {NULL}};
const Segment SGI243933[] = {{&S2719[13], 6}, {NULL}};
const Segment SGI243934[] = {{&S105827[34], 10}, {NULL}};
const Segment SGI243935[] = {{&S2719[67], 6}, {NULL}};
const Segment SGI243936[] = {{&S2719[101], 6}, {NULL}};
const Segment SGI243937[] = {{&S105827[27], 6}, {NULL}};
const Segment SGI243938[] = {{&S105827[97], 9}, {NULL}};
const Segment SGI243939[] = {{&S105827[70], 6}, {NULL}};
const Segment SGI243940[] = {{&S105827[132], 8}, {NULL}};
const Segment SGI243941[] = {{&S2720[13], 6}, {NULL}};
const Segment SGI243942[] = {{&S2720[44], 5}, {NULL}};
const Segment SGI243943[] = {{&S2720[127], 5}, {NULL}};
const Segment SGI243944[] = {{&S2721[15], 6}, {NULL}};
const Segment SGI243945[] = {{&S2721[25], 5}, {NULL}};
const Segment SGI243946[] = {{&S2722[13], 6}, {NULL}};
const Segment SGI243947[] = {{&S2722[35], 5}, {NULL}};
const Segment SGI243948[] = {{&S2722[68], 6}, {NULL}};
const Segment SGI243949[] = {{&S2722[102], 6}, {NULL}};
const Segment SGI243950[] = {{&S2722[116], 9}, {NULL}};
const Segment SGI243951[] = {{&S2723[13], 6}, {&S2723[13], 6}, {NULL}};
const Segment SGI243952[] = {{&S2723[35], 5}, {&S2723[131], 5}, {NULL}};
const Segment SGI243953[] = {{&S2723[68], 6}, {NULL}};
const Segment SGI243954[] = {{&S97774[44], 10}, {NULL}};
const Segment SGI243955[] = {{&S2723[102], 6}, {NULL}};
const Segment SGI243956[] = {{&S97774[44], 22}, {NULL}};
const Segment SGI243957[] = {{&S97774[30], 6}, {NULL}};
const Segment SGI243958[] = {{&S97774[115], 3}, {NULL}};
const Segment SGI243959[] = {{&S97774[80], 6}, {NULL}};
const Segment SGI243960[] = {{&S97774[150], 3}, {NULL}};
const Segment SGI243961[] = {{&S2724[13], 6}, {&S2724[128], 4}, {NULL}};
const Segment SGI243962[] = {{&S2724[75], 6}, {NULL}};
const Segment SGI243963[] = {{&S2724[109], 4}, {NULL}};
const Segment SGI243964[] = {{&S2725[13], 6}, {NULL}};
const Segment SGI243965[] = {{&S2725[35], 5}, {NULL}};
const Segment SGI243966[] = {{&S2725[68], 6}, {NULL}};
const Segment SGI243967[] = {{&S2725[102], 6}, {NULL}};
const Segment SGI243968[] = {{&S97777[34], 5}, {NULL}};
const Segment SGI243969[] = {{&S97777[27], 6}, {NULL}};
const Segment SGI243970[] = {{&S97777[92], 9}, {NULL}};
const Segment SGI243971[] = {{&S97777[65], 6}, {NULL}};
const Segment SGI243972[] = {{&S2726[13], 6}, {&S2726[146], 4}, {&S2726[146], 4}, {NULL}};
const Segment SGI243973[] = {{&S2726[35], 5}, {NULL}};
const Segment SGI243974[] = {{&S2726[68], 6}, {NULL}};
const Segment SGI243975[] = {{&S2726[102], 6}, {NULL}};
const Segment SGI243976[] = {{&S2727[13], 6}, {NULL}};
const Segment SGI243977[] = {{&S2727[35], 5}, {NULL}};
const Segment SGI243978[] = {{&S2727[68], 6}, {NULL}};
const Segment SGI243979[] = {{&S2727[102], 7}, {NULL}};
const Segment SGI243980[] = {{&S97780[34], 12}, {NULL}};
const Segment SGI243981[] = {{&S97780[27], 6}, {NULL}};
const Segment SGI243982[] = {{&S97780[97], 9}, {NULL}};
const Segment SGI243983[] = {{&S2728[13], 6}, {NULL}};
const Segment SGI243984[] = {{&S97780[70], 6}, {NULL}};
const Segment SGI243985[] = {{&S2728[38], 5}, {NULL}};
const Segment SGI243986[] = {{&S105841[44], 10}, {NULL}};
const Segment SGI243987[] = {{&S2729[13], 6}, {NULL}};
const Segment SGI243988[] = {{&S105841[44], 22}, {NULL}};
const Segment SGI243989[] = {{&S2729[35], 5}, {NULL}};
const Segment SGI243990[] = {{&S105841[30], 6}, {NULL}};
const Segment SGI243991[] = {{&S2729[68], 6}, {NULL}};
const Segment SGI243992[] = {{&S2729[102], 6}, {NULL}};
const Segment SGI243993[] = {{&S105841[80], 6}, {NULL}};
const Segment SGI243994[] = {{&S105841[107], 13}, {NULL}};
const Segment SGI243995[] = {{&S2730[13], 6}, {NULL}};
const Segment SGI243996[] = {{&S2730[35], 5}, {NULL}};
const Segment SGI243997[] = {{&S2730[68], 6}, {NULL}};
const Segment SGI243998[] = {{&S105842[59], 10}, {NULL}};
const Segment SGI243999[] = {{&S105842[59], 22}, {NULL}};
const Segment SGI244000[] = {{&S105842[45], 6}, {NULL}};
const Segment SGI244001[] = {{&S2731[13], 6}, {&S2731[13], 6}, {NULL}};
const Segment SGI244002[] = {{&S2731[35], 5}, {&S2731[131], 5}, {NULL}};
const Segment SGI244003[] = {{&S105842[130], 3}, {NULL}};
const Segment SGI244004[] = {{&S2731[68], 6}, {NULL}};
const Segment SGI244005[] = {{&S105842[95], 6}, {NULL}};
const Segment SGI244006[] = {{&S2731[102], 6}, {NULL}};
const Segment SGI244007[] = {{&S105842[0], 14}, {NULL}};
const Segment SGI244008[] = {{&S2732[13], 6}, {NULL}};
const Segment SGI244009[] = {{&S2732[35], 5}, {NULL}};
const Segment SGI244010[] = {{&S2732[68], 6}, {NULL}};
const Segment SGI244011[] = {{&S2732[102], 6}, {NULL}};
const Segment SGI244012[] = {{&S97787[34], 7}, {NULL}};
const Segment SGI244013[] = {{&S97787[27], 6}, {NULL}};
const Segment SGI244014[] = {{&S97787[94], 9}, {NULL}};
const Segment SGI244015[] = {{&S97787[67], 6}, {NULL}};
const Segment SGI244016[] = {{&S2733[13], 6}, {NULL}};
const Segment SGI244017[] = {{&S2733[35], 5}, {NULL}};
const Segment SGI244018[] = {{&S2733[68], 6}, {NULL}};
const Segment SGI244019[] = {{&S2733[102], 6}, {NULL}};
const Segment SGI244020[] = {{&S2733[116], 9}, {&S2733[116], 9}, {NULL}};
const Segment SGI244021[] = {{&S2734[13], 6}, {NULL}};
const Segment SGI244022[] = {{&S2734[28], 3}, {NULL}};
const Segment SGI244023[] = {{&S2734[59], 6}, {NULL}};
const Segment SGI244024[] = {{&S2734[93], 6}, {NULL}};
const Segment SGI244025[] = {{&S2734[107], 7}, {NULL}};
const Segment SGI244026[] = {{&S2735[44], 6}, {NULL}};
const Segment SGI244027[] = {{&S2735[69], 5}, {NULL}};
const Segment SGI244028[] = {{&S2735[109], 6}, {NULL}};
const Segment SGI244029[] = {{&S2736[13], 6}, {NULL}};
const Segment SGI244030[] = {{&S2736[42], 5}, {NULL}};
const Segment SGI244031[] = {{&S2736[125], 5}, {NULL}};
const Segment SGI244032[] = {{&S97793[34], 7}, {NULL}};
const Segment SGI244033[] = {{&S97793[27], 6}, {NULL}};
const Segment SGI244034[] = {{&S2737[15], 6}, {NULL}};
const Segment SGI244035[] = {{&S2737[33], 5}, {NULL}};
const Segment SGI244036[] = {{&S97793[94], 9}, {NULL}};
const Segment SGI244037[] = {{&S97793[67], 6}, {NULL}};
const Segment SGI244038[] = {{&S2737[0], 7}, {NULL}};
const Segment SGI244039[] = {{&S105851[59], 10}, {NULL}};
const Segment SGI244040[] = {{&S105851[59], 22}, {NULL}};
const Segment SGI244041[] = {{&S2738[13], 6}, {NULL}};
const Segment SGI244042[] = {{&S105851[45], 6}, {NULL}};
const Segment SGI244043[] = {{&S2738[35], 5}, {NULL}};
const Segment SGI244044[] = {{&S2738[68], 6}, {NULL}};
const Segment SGI244045[] = {{&S2738[102], 6}, {NULL}};
const Segment SGI244046[] = {{&S105851[130], 3}, {NULL}};
const Segment SGI244047[] = {{&S2738[116], 9}, {NULL}};
const Segment SGI244048[] = {{&S105851[95], 6}, {NULL}};
const Segment SGI244049[] = {{&S105851[0], 14}, {NULL}};
const Segment SGI244050[] = {{&S2739[13], 6}, {NULL}};
const Segment SGI244051[] = {{&S2739[42], 5}, {NULL}};
const Segment SGI244052[] = {{&S2739[125], 5}, {NULL}};
const Segment SGI244053[] = {{&S2740[13], 6}, {NULL}};
const Segment SGI244054[] = {{&S2740[42], 5}, {NULL}};
const Segment SGI244055[] = {{&S105853[34], 10}, {NULL}};
const Segment SGI244056[] = {{&S2740[125], 5}, {NULL}};
const Segment SGI244057[] = {{&S105853[27], 6}, {NULL}};
const Segment SGI244058[] = {{&S105853[97], 9}, {NULL}};
const Segment SGI244059[] = {{&S105853[70], 6}, {NULL}};
const Segment SGI244060[] = {{&S105853[132], 8}, {NULL}};
const Segment SGI244061[] = {{&S2741[13], 6}, {NULL}};
const Segment SGI244062[] = {{&S2741[42], 5}, {NULL}};
const Segment SGI244063[] = {{&S2741[125], 5}, {NULL}};
const Segment SGI244064[] = {{&S2742[13], 6}, {NULL}};
const Segment SGI244065[] = {{&S2742[42], 5}, {NULL}};
const Segment SGI244066[] = {{&S2742[125], 5}, {NULL}};
const Segment SGI244067[] = {{&S97800[44], 18}, {NULL}};
const Segment SGI244068[] = {{&S97800[44], 30}, {NULL}};
const Segment SGI244069[] = {{&S97800[30], 6}, {NULL}};
const Segment SGI244070[] = {{&S97800[123], 3}, {NULL}};
const Segment SGI244071[] = {{&S97800[88], 6}, {NULL}};
const Segment SGI244072[] = {{&S97800[148], 11}, {NULL}};
const Segment SGI244073[] = {{&S2743[44], 6}, {NULL}};
const Segment SGI244074[] = {{&S2743[69], 5}, {NULL}};
const Segment SGI244075[] = {{&S2743[109], 6}, {NULL}};
const Segment SGI244076[] = {{&S2744[13], 6}, {&S2744[13], 6}, {NULL}};
const Segment SGI244077[] = {{&S2744[35], 5}, {&S2744[131], 5}, {NULL}};
const Segment SGI244078[] = {{&S2744[68], 6}, {NULL}};
const Segment SGI244079[] = {{&S2744[102], 6}, {NULL}};
const Segment SGI244080[] = {{&S2745[17], 6}, {&S2745[34], 4}, {NULL}};
const Segment SGI244081[] = {{&S2745[27], 5}, {NULL}};
const Segment SGI244082[] = {{&S2746[15], 6}, {NULL}};
const Segment SGI244083[] = {{&S2746[33], 5}, {NULL}};
const Segment SGI244084[] = {{&S2746[0], 7}, {NULL}};
const Segment SGI244085[] = {{&S90624[44], 9}, {NULL}};
const Segment SGI244086[] = {{&S90624[44], 20}, {NULL}};
const Segment SGI244087[] = {{&S90624[30], 6}, {NULL}};
const Segment SGI244088[] = {{&S90624[131], 3}, {NULL}};
const Segment SGI244089[] = {{&S90624[96], 6}, {NULL}};
const Segment SGI244090[] = {{&S2747[13], 6}, {NULL}};
const Segment SGI244091[] = {{&S2747[35], 3}, {NULL}};
const Segment SGI244092[] = {{&S90624[66], 8}, {NULL}};
const Segment SGI244093[] = {{&S2747[66], 6}, {NULL}};
const Segment SGI244094[] = {{&S2747[100], 6}, {NULL}};
const Segment SGI244095[] = {{&S2747[114], 9}, {NULL}};
const Segment SGI244096[] = {{&S90625[44], 9}, {NULL}};
const Segment SGI244097[] = {{&S90625[44], 21}, {NULL}};
const Segment SGI244098[] = {{&S2748[13], 6}, {NULL}};
const Segment SGI244099[] = {{&S90625[30], 6}, {NULL}};
const Segment SGI244100[] = {{&S2748[35], 5}, {NULL}};
const Segment SGI244101[] = {{&S2748[68], 6}, {NULL}};
const Segment SGI244102[] = {{&S2748[102], 6}, {NULL}};
const Segment SGI244103[] = {{&S2748[116], 9}, {NULL}};
const Segment SGI244104[] = {{&S90625[114], 3}, {NULL}};
const Segment SGI244105[] = {{&S90625[79], 6}, {NULL}};
const Segment SGI244106[] = {{&S90625[139], 8}, {NULL}};
const Segment SGI244107[] = {{&S97809[44], 10}, {NULL}};
const Segment SGI244108[] = {{&S97809[44], 22}, {NULL}};
const Segment SGI244109[] = {{&S97809[30], 6}, {NULL}};
const Segment SGI244110[] = {{&S97809[115], 3}, {NULL}};
const Segment SGI244111[] = {{&S2749[13], 6}, {NULL}};
const Segment SGI244112[] = {{&S97809[80], 6}, {NULL}};
const Segment SGI244113[] = {{&S2749[133], 5}, {NULL}};
const Segment SGI244114[] = {{&S2749[35], 5}, {NULL}};
const Segment SGI244115[] = {{&S97809[150], 3}, {NULL}};
const Segment SGI244116[] = {{&S2749[68], 6}, {NULL}};
const Segment SGI244117[] = {{&S2749[102], 6}, {NULL}};
const Segment SGI244118[] = {{&S2750[13], 6}, {&S2750[109], 4}, {NULL}};
const Segment SGI244119[] = {{&S2750[35], 5}, {NULL}};
const Segment SGI244120[] = {{&S2750[68], 6}, {NULL}};
const Segment SGI244121[] = {{&S2750[102], 6}, {NULL}};
const Segment SGI244122[] = {{&S2751[15], 6}, {NULL}};
const Segment SGI244123[] = {{&S2751[33], 3}, {NULL}};
const Segment SGI244124[] = {{&S2751[0], 7}, {NULL}};
const Segment SGI244125[] = {{&S2752[13], 6}, {NULL}};
const Segment SGI244126[] = {{&S2752[35], 5}, {NULL}};
const Segment SGI244127[] = {{&S2752[68], 6}, {NULL}};
const Segment SGI244128[] = {{&S2752[102], 5}, {NULL}};
const Segment SGI244129[] = {{&S2753[13], 6}, {&S2753[13], 6}, {NULL}};
const Segment SGI244130[] = {{&S2753[35], 5}, {NULL}};
const Segment SGI244131[] = {{&S2753[68], 6}, {NULL}};
const Segment SGI244132[] = {{&S2753[102], 6}, {NULL}};
const Segment SGI244133[] = {{&S2754[15], 6}, {NULL}};
const Segment SGI244134[] = {{&S2754[25], 5}, {NULL}};
const Segment SGI244135[] = {{&S2755[13], 6}, {NULL}};
const Segment SGI244136[] = {{&S2755[42], 5}, {NULL}};
const Segment SGI244137[] = {{&S2755[125], 5}, {NULL}};
const Segment SGI244138[] = {{&S97819[44], 12}, {NULL}};
const Segment SGI244139[] = {{&S97819[44], 24}, {NULL}};
const Segment SGI244140[] = {{&S97819[30], 6}, {NULL}};
const Segment SGI244141[] = {{&S2756[13], 6}, {NULL}};
const Segment SGI244142[] = {{&S2756[37], 5}, {NULL}};
const Segment SGI244143[] = {{&S2756[70], 6}, {NULL}};
const Segment SGI244144[] = {{&S97819[133], 3}, {NULL}};
const Segment SGI244145[] = {{&S2756[104], 6}, {NULL}};
const Segment SGI244146[] = {{&S2756[118], 7}, {NULL}};
const Segment SGI244147[] = {{&S97819[98], 6}, {NULL}};
const Segment SGI244148[] = {{&S97819[70], 8}, {NULL}};
const Segment SGI244149[] = {{&S98925[34], 7}, {NULL}};
const Segment SGI244150[] = {{&S98925[27], 6}, {NULL}};
const Segment SGI244151[] = {{&S98925[94], 9}, {NULL}};
const Segment SGI244152[] = {{&S98925[67], 6}, {NULL}};
const Segment SGI244153[] = {{&S98925[135], 8}, {NULL}};
const Segment SGI244154[] = {{&S90650[34], 15}, {NULL}};
const Segment SGI244155[] = {{&S90650[27], 6}, {NULL}};
const Segment SGI244156[] = {{&S90650[101], 9}, {NULL}};
const Segment SGI244157[] = {{&S90650[74], 6}, {NULL}};
const Segment SGI244158[] = {{&S97834[44], 15}, {NULL}};
const Segment SGI244159[] = {{&S97834[44], 39}, {NULL}};
const Segment SGI244160[] = {{&S97834[30], 6}, {NULL}};
const Segment SGI244161[] = {{&S97834[130], 3}, {NULL}};
const Segment SGI244162[] = {{&S97834[97], 5}, {NULL}};
const Segment SGI244163[] = {{&S97835[34], 10}, {NULL}};
const Segment SGI244164[] = {{&S97835[27], 6}, {NULL}};
const Segment SGI244165[] = {{&S97835[97], 9}, {NULL}};
const Segment SGI244166[] = {{&S97835[70], 6}, {NULL}};
const Segment SGI244167[] = {{&S97835[138], 8}, {NULL}};
const Segment SGI244168[] = {{&S90656[34], 10}, {NULL}};
const Segment SGI244169[] = {{&S90656[27], 6}, {NULL}};
const Segment SGI244170[] = {{&S90656[96], 9}, {NULL}};
const Segment SGI244171[] = {{&S90656[69], 6}, {NULL}};
const Segment SGI244172[] = {{&S90666[44], 12}, {NULL}};
const Segment SGI244173[] = {{&S90666[44], 24}, {NULL}};
const Segment SGI244174[] = {{&S90666[30], 6}, {NULL}};
const Segment SGI244175[] = {{&S90666[117], 3}, {NULL}};
const Segment SGI244176[] = {{&S90666[82], 6}, {NULL}};
const Segment SGI244177[] = {{&S90666[135], 8}, {NULL}};
const Segment SGI244178[] = {{&S97854[44], 9}, {NULL}};
const Segment SGI244179[] = {{&S97854[44], 20}, {NULL}};
const Segment SGI244180[] = {{&S97854[30], 6}, {NULL}};
const Segment SGI244181[] = {{&S97854[129], 3}, {NULL}};
const Segment SGI244182[] = {{&S97854[94], 6}, {NULL}};
const Segment SGI244183[] = {{&S97854[66], 8}, {NULL}};
const Segment SGI244184[] = {{&S98955[44], 10}, {NULL}};
const Segment SGI244185[] = {{&S98955[44], 22}, {NULL}};
const Segment SGI244186[] = {{&S98955[30], 6}, {NULL}};
const Segment SGI244187[] = {{&S98955[80], 6}, {NULL}};
const Segment SGI244188[] = {{&S98955[117], 3}, {NULL}};
const Segment SGI244189[] = {{&S105863[44], 10}, {NULL}};
const Segment SGI244190[] = {{&S105863[44], 22}, {NULL}};
const Segment SGI244191[] = {{&S105863[30], 6}, {NULL}};
const Segment SGI244192[] = {{&S105863[80], 6}, {NULL}};
const Segment SGI244193[] = {{&S105863[107], 13}, {NULL}};
const Segment SGI244194[] = {{&S97861[34], 10}, {NULL}};
const Segment SGI244195[] = {{&S97861[27], 6}, {NULL}};
const Segment SGI244196[] = {{&S97861[97], 9}, {NULL}};
const Segment SGI244197[] = {{&S97861[70], 6}, {NULL}};
const Segment SGI244198[] = {{&S2789[13], 4}, {NULL}};
const Segment SGI244199[] = {{&S2789[26], 3}, {NULL}};
const Segment SGI244200[] = {{&S2789[57], 6}, {NULL}};
const Segment SGI244201[] = {{&S2789[91], 8}, {NULL}};
const Segment SGI244202[] = {{&S2790[13], 4}, {NULL}};
const Segment SGI244203[] = {{&S2790[26], 3}, {NULL}};
const Segment SGI244204[] = {{&S97864[34], 16}, {NULL}};
const Segment SGI244205[] = {{&S2790[57], 6}, {NULL}};
const Segment SGI244206[] = {{&S97864[27], 6}, {NULL}};
const Segment SGI244207[] = {{&S2790[91], 8}, {NULL}};
const Segment SGI244208[] = {{&S97864[103], 9}, {NULL}};
const Segment SGI244209[] = {{&S97864[76], 6}, {NULL}};
const Segment SGI244210[] = {{&S2791[13], 4}, {NULL}};
const Segment SGI244211[] = {{&S2791[26], 3}, {NULL}};
const Segment SGI244212[] = {{&S2791[57], 6}, {NULL}};
const Segment SGI244213[] = {{&S2791[91], 8}, {NULL}};
const Segment SGI244214[] = {{&S97865[34], 12}, {NULL}};
const Segment SGI244215[] = {{&S97865[27], 6}, {NULL}};
const Segment SGI244216[] = {{&S105870[44], 10}, {NULL}};
const Segment SGI244217[] = {{&S105870[44], 22}, {NULL}};
const Segment SGI244218[] = {{&S105870[30], 6}, {NULL}};
const Segment SGI244219[] = {{&S97865[99], 9}, {NULL}};
const Segment SGI244220[] = {{&S2792[13], 4}, {NULL}};
const Segment SGI244221[] = {{&S97865[72], 6}, {NULL}};
const Segment SGI244222[] = {{&S2792[26], 3}, {NULL}};
const Segment SGI244223[] = {{&S2792[57], 6}, {NULL}};
const Segment SGI244224[] = {{&S97865[140], 8}, {NULL}};
const Segment SGI244225[] = {{&S105870[80], 6}, {NULL}};
const Segment SGI244226[] = {{&S105870[107], 13}, {NULL}};
const Segment SGI244227[] = {{&S2793[13], 4}, {NULL}};
const Segment SGI244228[] = {{&S2793[26], 3}, {NULL}};
const Segment SGI244229[] = {{&S98968[52], 6}, {NULL}};
const Segment SGI244230[] = {{&S2793[57], 6}, {NULL}};
const Segment SGI244231[] = {{&S98968[52], 18}, {NULL}};
const Segment SGI244232[] = {{&S2793[91], 6}, {NULL}};
const Segment SGI244233[] = {{&S98968[38], 6}, {NULL}};
const Segment SGI244234[] = {{&S98968[84], 5}, {NULL}};
const Segment SGI244235[] = {{&S2794[13], 4}, {NULL}};
const Segment SGI244236[] = {{&S2794[26], 3}, {NULL}};
const Segment SGI244237[] = {{&S2794[57], 6}, {NULL}};
const Segment SGI244238[] = {{&S2794[91], 6}, {NULL}};
const Segment SGI244239[] = {{&S2794[105], 9}, {NULL}};
const Segment SGI244240[] = {{&S2795[13], 4}, {&S2795[13], 4}, {&S2795[13], 4}, {NULL}};
const Segment SGI244241[] = {{&S105874[44], 10}, {NULL}};
const Segment SGI244242[] = {{&S2795[26], 3}, {NULL}};
const Segment SGI244243[] = {{&S105874[44], 22}, {NULL}};
const Segment SGI244244[] = {{&S2795[57], 6}, {NULL}};
const Segment SGI244245[] = {{&S105874[30], 6}, {NULL}};
const Segment SGI244246[] = {{&S2795[91], 6}, {NULL}};
const Segment SGI244247[] = {{&S105874[115], 3}, {NULL}};
const Segment SGI244248[] = {{&S105874[80], 6}, {NULL}};
const Segment SGI244249[] = {{&S105874[133], 11}, {NULL}};
const Segment SGI244250[] = {{&S2796[13], 4}, {&S2796[13], 4}, {NULL}};
const Segment SGI244251[] = {{&S2796[38], 3}, {NULL}};
const Segment SGI244252[] = {{&S2796[119], 5}, {NULL}};
const Segment SGI244253[] = {{&S2797[13], 4}, {NULL}};
const Segment SGI244254[] = {{&S2797[26], 3}, {NULL}};
const Segment SGI244255[] = {{&S2797[57], 6}, {NULL}};
const Segment SGI244256[] = {{&S2797[91], 6}, {NULL}};
const Segment SGI244257[] = {{&S2797[105], 9}, {&S2797[105], 9}, {NULL}};
const Segment SGI244258[] = {{&S2798[13], 4}, {&S2798[13], 4}, {NULL}};
const Segment SGI244259[] = {{&S2798[38], 3}, {NULL}};
const Segment SGI244260[] = {{&S2798[119], 4}, {NULL}};
const Segment SGI244261[] = {{&S2799[13], 4}, {NULL}};
const Segment SGI244262[] = {{&S2799[26], 3}, {NULL}};
const Segment SGI244263[] = {{&S2799[57], 6}, {NULL}};
const Segment SGI244264[] = {{&S2799[91], 5}, {NULL}};
const Segment SGI244265[] = {{&S2800[13], 4}, {NULL}};
const Segment SGI244266[] = {{&S2800[26], 3}, {NULL}};
const Segment SGI244267[] = {{&S2800[57], 6}, {NULL}};
const Segment SGI244268[] = {{&S2800[91], 6}, {NULL}};
const Segment SGI244269[] = {{&S2800[105], 7}, {NULL}};
const Segment SGI244270[] = {{&S105881[44], 10}, {NULL}};
const Segment SGI244271[] = {{&S105881[44], 22}, {NULL}};
const Segment SGI244272[] = {{&S105881[30], 6}, {NULL}};
const Segment SGI244273[] = {{&S2801[13], 4}, {NULL}};
const Segment SGI244274[] = {{&S105881[115], 3}, {NULL}};
const Segment SGI244275[] = {{&S2801[26], 3}, {NULL}};
const Segment SGI244276[] = {{&S2801[57], 6}, {NULL}};
const Segment SGI244277[] = {{&S105881[80], 6}, {NULL}};
const Segment SGI244278[] = {{&S2801[91], 6}, {&S2801[28], 1}, {NULL}};
const Segment SGI244279[] = {{&S105881[143], 3}, {NULL}};
const Segment SGI244280[] = {{&S97880[44], 10}, {NULL}};
const Segment SGI244281[] = {{&S97880[44], 22}, {NULL}};
const Segment SGI244282[] = {{&S97880[30], 6}, {NULL}};
const Segment SGI244283[] = {{&S2802[13], 4}, {NULL}};
const Segment SGI244284[] = {{&S97880[115], 3}, {NULL}};
const Segment SGI244285[] = {{&S2802[26], 3}, {NULL}};
const Segment SGI244286[] = {{&S97880[80], 6}, {NULL}};
const Segment SGI244287[] = {{&S2802[57], 6}, {NULL}};
const Segment SGI244288[] = {{&S97880[135], 8}, {NULL}};
const Segment SGI244289[] = {{&S2802[91], 6}, {NULL}};
const Segment SGI244290[] = {{&S105882[44], 10}, {NULL}};
const Segment SGI244291[] = {{&S2802[105], 9}, {NULL}};
const Segment SGI244292[] = {{&S97880[144], 13}, {NULL}};
const Segment SGI244293[] = {{&S105882[44], 22}, {NULL}};
const Segment SGI244294[] = {{&S105882[30], 6}, {NULL}};
const Segment SGI244295[] = {{&S105882[80], 6}, {NULL}};
const Segment SGI244296[] = {{&S105882[107], 13}, {NULL}};
const Segment SGI244297[] = {{&S2803[13], 4}, {&S2803[13], 4}, {&S2803[13], 4}, {NULL}};
const Segment SGI244298[] = {{&S2803[36], 3}, {NULL}};
const Segment SGI244299[] = {{&S2804[13], 4}, {&S2804[13], 4}, {NULL}};
const Segment SGI244300[] = {{&S2804[26], 3}, {NULL}};
const Segment SGI244301[] = {{&S2804[57], 6}, {NULL}};
const Segment SGI244302[] = {{&S2804[91], 6}, {NULL}};
const Segment SGI244303[] = {{&S98987[44], 10}, {NULL}};
const Segment SGI244304[] = {{&S2805[13], 4}, {NULL}};
const Segment SGI244305[] = {{&S98987[44], 24}, {NULL}};
const Segment SGI244306[] = {{&S98987[30], 6}, {NULL}};
const Segment SGI244307[] = {{&S2805[26], 5}, {NULL}};
const Segment SGI244308[] = {{&S2805[59], 6}, {NULL}};
const Segment SGI244309[] = {{&S2805[93], 6}, {NULL}};
const Segment SGI244310[] = {{&S98987[82], 6}, {NULL}};
const Segment SGI244311[] = {{&S98987[119], 3}, {NULL}};
const Segment SGI244312[] = {{&S2806[13], 4}, {NULL}};
const Segment SGI244313[] = {{&S2806[26], 5}, {NULL}};
const Segment SGI244314[] = {{&S2806[59], 6}, {NULL}};
const Segment SGI244315[] = {{&S2806[93], 6}, {NULL}};
const Segment SGI244316[] = {{&S2807[13], 4}, {NULL}};
const Segment SGI244317[] = {{&S2807[26], 5}, {NULL}};
const Segment SGI244318[] = {{&S2807[59], 6}, {NULL}};
const Segment SGI244319[] = {{&S2808[13], 4}, {NULL}};
const Segment SGI244320[] = {{&S2808[26], 5}, {NULL}};
const Segment SGI244321[] = {{&S2808[59], 6}, {NULL}};
const Segment SGI244322[] = {{&S2808[93], 5}, {NULL}};
const Segment SGI244323[] = {{&S105890[44], 10}, {NULL}};
const Segment SGI244324[] = {{&S105890[44], 22}, {NULL}};
const Segment SGI244325[] = {{&S105890[30], 6}, {NULL}};
const Segment SGI244326[] = {{&S105890[80], 6}, {NULL}};
const Segment SGI244327[] = {{&S105890[107], 13}, {NULL}};
const Segment SGI244328[] = {{&S97892[44], 10}, {NULL}};
const Segment SGI244329[] = {{&S97892[44], 22}, {NULL}};
const Segment SGI244330[] = {{&S97892[30], 6}, {NULL}};
const Segment SGI244331[] = {{&S97892[80], 6}, {NULL}};
const Segment SGI244332[] = {{&S97892[107], 13}, {NULL}};
const Segment SGI244333[] = {{&S97914[44], 7}, {NULL}};
const Segment SGI244334[] = {{&S97914[44], 23}, {NULL}};
const Segment SGI244335[] = {{&S97914[30], 6}, {NULL}};
const Segment SGI244336[] = {{&S97914[81], 6}, {NULL}};
const Segment SGI244337[] = {{&S97914[108], 13}, {NULL}};
const Segment SGI244338[] = {{&S97918[44], 10}, {NULL}};
const Segment SGI244339[] = {{&S97918[44], 22}, {NULL}};
const Segment SGI244340[] = {{&S97918[30], 6}, {NULL}};
const Segment SGI244341[] = {{&S97918[80], 6}, {NULL}};
const Segment SGI244342[] = {{&S97918[107], 13}, {NULL}};
const Segment SGI244343[] = {{&S97924[44], 9}, {NULL}};
const Segment SGI244344[] = {{&S97924[44], 21}, {NULL}};
const Segment SGI244345[] = {{&S97924[30], 6}, {NULL}};
const Segment SGI244346[] = {{&S97924[79], 6}, {NULL}};
const Segment SGI244347[] = {{&S97924[106], 13}, {NULL}};
const Segment SGI244348[] = {{&S90711[44], 10}, {NULL}};
const Segment SGI244349[] = {{&S90711[44], 22}, {NULL}};
const Segment SGI244350[] = {{&S90711[30], 6}, {NULL}};
const Segment SGI244351[] = {{&S90711[115], 3}, {NULL}};
const Segment SGI244352[] = {{&S90711[80], 6}, {NULL}};
const Segment SGI244353[] = {{&S90711[140], 8}, {NULL}};
const Segment SGI244354[] = {{&S90712[44], 10}, {NULL}};
const Segment SGI244355[] = {{&S90712[44], 21}, {NULL}};
const Segment SGI244356[] = {{&S90712[30], 6}, {NULL}};
const Segment SGI244357[] = {{&S90712[132], 3}, {NULL}};
const Segment SGI244358[] = {{&S90712[97], 6}, {NULL}};
const Segment SGI244359[] = {{&S90712[67], 8}, {NULL}};
const Segment SGI244360[] = {{&S90718[34], 10}, {NULL}};
const Segment SGI244361[] = {{&S90718[27], 6}, {NULL}};
const Segment SGI244362[] = {{&S97944[34], 16}, {NULL}};
const Segment SGI244363[] = {{&S97944[27], 6}, {NULL}};
const Segment SGI244364[] = {{&S90718[97], 9}, {NULL}};
const Segment SGI244365[] = {{&S90718[70], 6}, {NULL}};
const Segment SGI244366[] = {{&S97944[103], 9}, {NULL}};
const Segment SGI244367[] = {{&S97944[76], 6}, {NULL}};
const Segment SGI244368[] = {{&S90719[34], 15}, {NULL}};
const Segment SGI244369[] = {{&S90719[27], 6}, {NULL}};
const Segment SGI244370[] = {{&S90719[101], 9}, {NULL}};
const Segment SGI244371[] = {{&S90719[74], 6}, {NULL}};
const Segment SGI244372[] = {{&S90723[34], 10}, {NULL}};
const Segment SGI244373[] = {{&S90723[27], 6}, {NULL}};
const Segment SGI244374[] = {{&S90723[96], 9}, {NULL}};
const Segment SGI244375[] = {{&S90723[69], 6}, {NULL}};
const Segment SGI244376[] = {{&S97952[44], 10}, {NULL}};
const Segment SGI244377[] = {{&S97952[44], 22}, {NULL}};
const Segment SGI244378[] = {{&S97952[30], 6}, {NULL}};
const Segment SGI244379[] = {{&S97952[113], 3}, {NULL}};
const Segment SGI244380[] = {{&S97952[80], 5}, {NULL}};
const Segment SGI244381[] = {{&S97952[117], 14}, {NULL}};
const Segment SGI244382[] = {{&S97958[52], 10}, {NULL}};
const Segment SGI244383[] = {{&S97958[52], 22}, {NULL}};
const Segment SGI244384[] = {{&S97958[38], 6}, {NULL}};
const Segment SGI244385[] = {{&S97958[88], 5}, {NULL}};
const Segment SGI244386[] = {{&S97958[11], 3}, {NULL}};
const Segment SGI244387[] = {{&S97959[34], 14}, {NULL}};
const Segment SGI244388[] = {{&S97959[27], 6}, {NULL}};
const Segment SGI244389[] = {{&S97959[101], 9}, {NULL}};
const Segment SGI244390[] = {{&S97959[74], 6}, {NULL}};
const Segment SGI244391[] = {{&S97961[44], 10}, {NULL}};
const Segment SGI244392[] = {{&S97961[44], 22}, {NULL}};
const Segment SGI244393[] = {{&S97961[30], 6}, {NULL}};
const Segment SGI244394[] = {{&S97961[80], 6}, {NULL}};
const Segment SGI244395[] = {{&S97961[107], 13}, {NULL}};
const Segment SGI244396[] = {{&S97962[44], 9}, {NULL}};
const Segment SGI244397[] = {{&S97962[44], 21}, {NULL}};
const Segment SGI244398[] = {{&S97962[30], 6}, {NULL}};
const Segment SGI244399[] = {{&S97962[79], 6}, {NULL}};
const Segment SGI244400[] = {{&S97962[106], 13}, {NULL}};
const Segment SGI244401[] = {{&S97963[34], 9}, {NULL}};
const Segment SGI244402[] = {{&S97963[27], 6}, {NULL}};
const Segment SGI244403[] = {{&S97963[96], 9}, {NULL}};
const Segment SGI244404[] = {{&S97963[69], 6}, {NULL}};
const Segment SGI244405[] = {{&S97963[137], 8}, {NULL}};
const Segment SGI244406[] = {{&S97970[44], 11}, {NULL}};
const Segment SGI244407[] = {{&S97970[44], 23}, {NULL}};
const Segment SGI244408[] = {{&S97970[30], 6}, {NULL}};
const Segment SGI244409[] = {{&S97970[81], 6}, {NULL}};
const Segment SGI244410[] = {{&S97970[108], 13}, {NULL}};
const Segment SGI244411[] = {{&S97971[44], 10}, {NULL}};
const Segment SGI244412[] = {{&S97971[44], 22}, {NULL}};
const Segment SGI244413[] = {{&S97971[30], 6}, {NULL}};
const Segment SGI244414[] = {{&S97971[113], 3}, {NULL}};
const Segment SGI244415[] = {{&S97971[80], 5}, {NULL}};
const Segment SGI244416[] = {{&S97971[117], 14}, {NULL}};
const Segment SGI244417[] = {{&S97974[44], 9}, {NULL}};
const Segment SGI244418[] = {{&S97974[44], 21}, {NULL}};
const Segment SGI244419[] = {{&S97974[30], 6}, {NULL}};
const Segment SGI244420[] = {{&S97974[79], 6}, {NULL}};
const Segment SGI244421[] = {{&S97974[106], 13}, {NULL}};
const Segment SGI244422[] = {{&S97980[52], 15}, {NULL}};
const Segment SGI244423[] = {{&S97980[52], 27}, {NULL}};
const Segment SGI244424[] = {{&S97980[38], 6}, {NULL}};
const Segment SGI244425[] = {{&S97980[93], 5}, {NULL}};
const Segment SGI244426[] = {{&S97980[11], 3}, {NULL}};
const Segment SGI244427[] = {{&S2809[13], 4}, {NULL}};
const Segment SGI244428[] = {{&S2809[26], 5}, {NULL}};
const Segment SGI244429[] = {{&S2809[59], 6}, {NULL}};
const Segment SGI244430[] = {{&S2809[93], 6}, {NULL}};
const Segment SGI244431[] = {{&S2809[107], 9}, {NULL}};
const Segment SGI244432[] = {{&S2810[13], 4}, {&S2810[13], 4}, {&S2810[13], 4}, {NULL}};
const Segment SGI244433[] = {{&S2810[36], 5}, {NULL}};
const Segment SGI244434[] = {{&S2811[13], 4}, {&S2811[13], 4}, {NULL}};
const Segment SGI244435[] = {{&S2811[38], 3}, {NULL}};
const Segment SGI244436[] = {{&S2811[119], 6}, {NULL}};
const Segment SGI244437[] = {{&S2812[13], 4}, {NULL}};
const Segment SGI244438[] = {{&S2812[26], 5}, {NULL}};
const Segment SGI244439[] = {{&S2812[59], 6}, {NULL}};
const Segment SGI244440[] = {{&S2812[93], 6}, {&S2812[28], 1}, {NULL}};
const Segment SGI244441[] = {{&S2813[13], 4}, {NULL}};
const Segment SGI244442[] = {{&S2813[26], 5}, {NULL}};
const Segment SGI244443[] = {{&S2813[59], 6}, {NULL}};
const Segment SGI244444[] = {{&S2813[93], 6}, {NULL}};
const Segment SGI244445[] = {{&S2814[13], 4}, {&S2814[13], 4}, {&S2814[13], 4}, {NULL}};
const Segment SGI244446[] = {{&S2814[26], 5}, {NULL}};
const Segment SGI244447[] = {{&S2814[59], 6}, {NULL}};
const Segment SGI244448[] = {{&S2814[93], 6}, {NULL}};
const Segment SGI244449[] = {{&S2815[13], 4}, {&S2815[13], 4}, {&S2815[13], 4}, {NULL}};
const Segment SGI244450[] = {{&S2815[26], 5}, {NULL}};
const Segment SGI244451[] = {{&S2815[59], 6}, {NULL}};
const Segment SGI244452[] = {{&S2815[93], 6}, {NULL}};
const Segment SGI244453[] = {{&S2816[13], 4}, {NULL}};
const Segment SGI244454[] = {{&S2816[26], 3}, {NULL}};
const Segment SGI244455[] = {{&S2816[57], 6}, {NULL}};
const Segment SGI244456[] = {{&S2816[91], 6}, {NULL}};
const Segment SGI244457[] = {{&S2816[105], 7}, {NULL}};
const Segment SGI244458[] = {{&S2817[13], 4}, {&S2817[13], 4}, {NULL}};
const Segment SGI244459[] = {{&S2817[38], 5}, {NULL}};
const Segment SGI244460[] = {{&S2817[121], 5}, {NULL}};
const Segment SGI244461[] = {{&S2818[13], 4}, {NULL}};
const Segment SGI244462[] = {{&S2818[26], 5}, {NULL}};
const Segment SGI244463[] = {{&S2818[59], 6}, {NULL}};
const Segment SGI244464[] = {{&S2818[93], 6}, {NULL}};
const Segment SGI244465[] = {{&S2818[107], 9}, {&S2818[107], 9}, {NULL}};
const Segment SGI244466[] = {{&S2819[13], 4}, {NULL}};
const Segment SGI244467[] = {{&S2819[26], 3}, {NULL}};
const Segment SGI244468[] = {{&S2819[57], 6}, {NULL}};
const Segment SGI244469[] = {{&S2819[91], 6}, {NULL}};
const Segment SGI244470[] = {{&S2819[105], 7}, {NULL}};
const Segment SGI244471[] = {{&S2820[13], 4}, {NULL}};
const Segment SGI244472[] = {{&S2820[26], 5}, {NULL}};
const Segment SGI244473[] = {{&S2820[59], 6}, {NULL}};
const Segment SGI244474[] = {{&S2820[93], 6}, {NULL}};
const Segment SGI244475[] = {{&S2820[107], 7}, {NULL}};
const Segment SGI244476[] = {{&S2821[13], 4}, {NULL}};
const Segment SGI244477[] = {{&S2821[26], 5}, {NULL}};
const Segment SGI244478[] = {{&S2821[59], 6}, {NULL}};
const Segment SGI244479[] = {{&S2821[93], 7}, {NULL}};
const Segment SGI244480[] = {{&S2822[13], 4}, {&S2822[13], 4}, {NULL}};
const Segment SGI244481[] = {{&S2822[26], 5}, {NULL}};
const Segment SGI244482[] = {{&S2822[59], 6}, {NULL}};
const Segment SGI244483[] = {{&S2822[93], 6}, {NULL}};
const Segment SGI244484[] = {{&S2823[13], 4}, {&S2823[13], 4}, {NULL}};
const Segment SGI244485[] = {{&S2823[38], 5}, {NULL}};
const Segment SGI244486[] = {{&S2823[121], 4}, {NULL}};
const Segment SGI244487[] = {{&S2824[13], 4}, {&S2824[13], 4}, {NULL}};
const Segment SGI244488[] = {{&S2824[38], 5}, {NULL}};
const Segment SGI244489[] = {{&S2824[121], 6}, {NULL}};
const Segment SGI244490[] = {{&S2825[13], 4}, {NULL}};
const Segment SGI244491[] = {{&S2825[26], 5}, {NULL}};
const Segment SGI244492[] = {{&S2825[59], 6}, {NULL}};
const Segment SGI244493[] = {{&S2825[93], 6}, {NULL}};
const Segment SGI244494[] = {{&S2825[107], 9}, {NULL}};
const Segment SGI244495[] = {{&S2826[13], 4}, {NULL}};
const Segment SGI244496[] = {{&S2826[26], 5}, {NULL}};
const Segment SGI244497[] = {{&S2826[59], 6}, {NULL}};
const Segment SGI244498[] = {{&S2827[13], 4}, {NULL}};
const Segment SGI244499[] = {{&S2827[26], 5}, {NULL}};
const Segment SGI244500[] = {{&S2827[59], 6}, {NULL}};
const Segment SGI244501[] = {{&S2827[93], 6}, {NULL}};
const Segment SGI244502[] = {{&S2827[107], 9}, {&S2827[107], 9}, {&S2827[107], 9}, {NULL}};
const Segment SGI244503[] = {{&S98006[34], 10}, {NULL}};
const Segment SGI244504[] = {{&S98006[27], 6}, {NULL}};
const Segment SGI244505[] = {{&S98006[97], 9}, {NULL}};
const Segment SGI244506[] = {{&S2828[13], 4}, {&S2828[13], 4}, {NULL}};
const Segment SGI244507[] = {{&S98006[70], 6}, {NULL}};
const Segment SGI244508[] = {{&S2828[38], 5}, {NULL}};
const Segment SGI244509[] = {{&S2828[121], 6}, {NULL}};
const Segment SGI244510[] = {{&S2829[13], 4}, {NULL}};
const Segment SGI244511[] = {{&S2829[26], 5}, {NULL}};
const Segment SGI244512[] = {{&S2829[59], 6}, {NULL}};
const Segment SGI244513[] = {{&S2829[93], 6}, {NULL}};
const Segment SGI244514[] = {{&S2830[13], 4}, {NULL}};
const Segment SGI244515[] = {{&S2830[26], 5}, {NULL}};
const Segment SGI244516[] = {{&S2830[59], 6}, {NULL}};
const Segment SGI244517[] = {{&S2830[93], 6}, {NULL}};
const Segment SGI244518[] = {{&S2830[107], 9}, {NULL}};
const Segment SGI244519[] = {{&S2831[13], 4}, {&S2831[13], 4}, {NULL}};
const Segment SGI244520[] = {{&S2831[38], 5}, {NULL}};
const Segment SGI244521[] = {{&S2831[121], 5}, {NULL}};
const Segment SGI244522[] = {{&S2832[13], 4}, {NULL}};
const Segment SGI244523[] = {{&S2832[26], 3}, {NULL}};
const Segment SGI244524[] = {{&S2832[57], 6}, {NULL}};
const Segment SGI244525[] = {{&S2832[91], 6}, {NULL}};
const Segment SGI244526[] = {{&S2832[105], 7}, {NULL}};
const Segment SGI244527[] = {{&S2833[13], 4}, {NULL}};
const Segment SGI244528[] = {{&S2833[26], 5}, {NULL}};
const Segment SGI244529[] = {{&S2833[59], 6}, {NULL}};
const Segment SGI244530[] = {{&S2833[93], 6}, {NULL}};
const Segment SGI244531[] = {{&S2833[107], 9}, {&S2833[107], 9}, {NULL}};
const Segment SGI244532[] = {{&S2834[13], 4}, {NULL}};
const Segment SGI244533[] = {{&S2834[26], 5}, {NULL}};
const Segment SGI244534[] = {{&S2834[59], 6}, {NULL}};
const Segment SGI244535[] = {{&S2834[93], 6}, {&S2834[28], 1}, {NULL}};
const Segment SGI244536[] = {{&S98016[34], 7}, {NULL}};
const Segment SGI244537[] = {{&S98016[27], 6}, {NULL}};
const Segment SGI244538[] = {{&S98016[94], 9}, {NULL}};
const Segment SGI244539[] = {{&S98016[67], 6}, {NULL}};
const Segment SGI244540[] = {{&S2835[13], 4}, {&S2835[13], 4}, {NULL}};
const Segment SGI244541[] = {{&S2835[26], 5}, {NULL}};
const Segment SGI244542[] = {{&S2835[59], 6}, {NULL}};
const Segment SGI244543[] = {{&S2835[93], 6}, {NULL}};
const Segment SGI244544[] = {{&S2836[13], 4}, {NULL}};
const Segment SGI244545[] = {{&S2836[26], 5}, {NULL}};
const Segment SGI244546[] = {{&S2836[59], 6}, {NULL}};
const Segment SGI244547[] = {{&S2836[93], 5}, {NULL}};
const Segment SGI244548[] = {{&S99157[34], 10}, {NULL}};
const Segment SGI244549[] = {{&S99157[27], 6}, {NULL}};
const Segment SGI244550[] = {{&S2837[13], 4}, {NULL}};
const Segment SGI244551[] = {{&S2837[26], 5}, {NULL}};
const Segment SGI244552[] = {{&S99157[97], 9}, {NULL}};
const Segment SGI244553[] = {{&S2837[59], 6}, {NULL}};
const Segment SGI244554[] = {{&S99157[70], 6}, {NULL}};
const Segment SGI244555[] = {{&S2837[93], 6}, {&S2837[28], 1}, {NULL}};
const Segment SGI244556[] = {{&S99158[34], 10}, {NULL}};
const Segment SGI244557[] = {{&S99158[27], 6}, {NULL}};
const Segment SGI244558[] = {{&S2838[13], 4}, {&S2838[13], 4}, {&S2838[13], 4}, {NULL}};
const Segment SGI244559[] = {{&S2838[36], 5}, {NULL}};
const Segment SGI244560[] = {{&S99158[97], 9}, {NULL}};
const Segment SGI244561[] = {{&S99158[70], 6}, {NULL}};
const Segment SGI244562[] = {{&S98022[34], 14}, {NULL}};
const Segment SGI244563[] = {{&S98022[27], 6}, {NULL}};
const Segment SGI244564[] = {{&S98022[111], 9}, {NULL}};
const Segment SGI244565[] = {{&S98022[84], 6}, {NULL}};
const Segment SGI244566[] = {{&S2839[13], 4}, {NULL}};
const Segment SGI244567[] = {{&S2839[29], 3}, {NULL}};
const Segment SGI244568[] = {{&S98022[152], 8}, {NULL}};
const Segment SGI244569[] = {{&S2840[13], 4}, {NULL}};
const Segment SGI244570[] = {{&S2840[26], 3}, {NULL}};
const Segment SGI244571[] = {{&S2840[57], 6}, {NULL}};
const Segment SGI244572[] = {{&S2840[91], 5}, {NULL}};
const Segment SGI244573[] = {{&S98024[34], 9}, {NULL}};
const Segment SGI244574[] = {{&S2841[15], 4}, {NULL}};
const Segment SGI244575[] = {{&S2841[31], 5}, {NULL}};
const Segment SGI244576[] = {{&S98024[27], 6}, {NULL}};
const Segment SGI244577[] = {{&S2841[0], 7}, {NULL}};
const Segment SGI244578[] = {{&S98024[96], 9}, {NULL}};
const Segment SGI244579[] = {{&S98024[69], 6}, {NULL}};
const Segment SGI244580[] = {{&S98024[137], 8}, {NULL}};
const Segment SGI244581[] = {{&S2842[13], 4}, {&S2842[13], 4}, {NULL}};
const Segment SGI244582[] = {{&S2842[26], 5}, {NULL}};
const Segment SGI244583[] = {{&S2842[59], 6}, {NULL}};
const Segment SGI244584[] = {{&S2842[93], 6}, {NULL}};
const Segment SGI244585[] = {{&S2843[13], 4}, {&S2843[13], 4}, {&S2843[13], 4}, {NULL}};
const Segment SGI244586[] = {{&S2843[26], 5}, {NULL}};
const Segment SGI244587[] = {{&S2843[59], 6}, {NULL}};
const Segment SGI244588[] = {{&S2843[93], 6}, {NULL}};
const Segment SGI244589[] = {{&S98027[34], 10}, {NULL}};
const Segment SGI244590[] = {{&S98027[27], 6}, {NULL}};
const Segment SGI244591[] = {{&S98027[97], 9}, {NULL}};
const Segment SGI244592[] = {{&S98027[70], 6}, {NULL}};
const Segment SGI244593[] = {{&S98027[131], 8}, {NULL}};
const Segment SGI244594[] = {{&S2844[15], 4}, {NULL}};
const Segment SGI244595[] = {{&S2844[31], 3}, {NULL}};
const Segment SGI244596[] = {{&S2844[0], 7}, {NULL}};
const Segment SGI244597[] = {{&S98028[34], 7}, {NULL}};
const Segment SGI244598[] = {{&S2845[13], 4}, {NULL}};
const Segment SGI244599[] = {{&S98028[27], 6}, {NULL}};
const Segment SGI244600[] = {{&S2845[26], 5}, {NULL}};
const Segment SGI244601[] = {{&S2845[59], 6}, {NULL}};
const Segment SGI244602[] = {{&S2845[93], 5}, {NULL}};
const Segment SGI244603[] = {{&S98028[98], 9}, {NULL}};
const Segment SGI244604[] = {{&S98028[71], 6}, {NULL}};
const Segment SGI244605[] = {{&S98028[139], 8}, {NULL}};
const Segment SGI244606[] = {{&S99170[44], 7}, {NULL}};
const Segment SGI244607[] = {{&S2846[15], 4}, {NULL}};
const Segment SGI244608[] = {{&S99170[44], 19}, {NULL}};
const Segment SGI244609[] = {{&S2846[31], 5}, {NULL}};
const Segment SGI244610[] = {{&S99170[30], 6}, {NULL}};
const Segment SGI244611[] = {{&S99170[130], 3}, {NULL}};
const Segment SGI244612[] = {{&S99170[95], 6}, {NULL}};
const Segment SGI244613[] = {{&S99170[65], 8}, {NULL}};
const Segment SGI244614[] = {{&S2847[15], 4}, {NULL}};
const Segment SGI244615[] = {{&S2847[31], 5}, {NULL}};
const Segment SGI244616[] = {{&S2847[0], 7}, {NULL}};
const Segment SGI244617[] = {{&S2848[13], 4}, {&S2848[13], 4}, {NULL}};
const Segment SGI244618[] = {{&S2848[38], 3}, {NULL}};
const Segment SGI244619[] = {{&S2848[119], 5}, {NULL}};
const Segment SGI244620[] = {{&S98032[34], 7}, {NULL}};
const Segment SGI244621[] = {{&S98032[27], 6}, {NULL}};
const Segment SGI244622[] = {{&S2849[13], 4}, {NULL}};
const Segment SGI244623[] = {{&S2849[26], 3}, {NULL}};
const Segment SGI244624[] = {{&S2849[57], 6}, {NULL}};
const Segment SGI244625[] = {{&S2849[91], 6}, {NULL}};
const Segment SGI244626[] = {{&S2849[105], 6}, {NULL}};
const Segment SGI244627[] = {{&S98032[94], 9}, {NULL}};
const Segment SGI244628[] = {{&S98032[67], 6}, {NULL}};
const Segment SGI244629[] = {{&S2850[13], 4}, {&S2850[13], 4}, {NULL}};
const Segment SGI244630[] = {{&S2850[38], 3}, {NULL}};
const Segment SGI244631[] = {{&S2850[119], 6}, {NULL}};
const Segment SGI244632[] = {{&S99176[34], 12}, {NULL}};
const Segment SGI244633[] = {{&S99176[27], 6}, {NULL}};
const Segment SGI244634[] = {{&S99176[99], 9}, {NULL}};
const Segment SGI244635[] = {{&S99176[72], 6}, {NULL}};
const Segment SGI244636[] = {{&S2851[13], 4}, {&S2851[13], 4}, {NULL}};
const Segment SGI244637[] = {{&S2851[38], 5}, {NULL}};
const Segment SGI244638[] = {{&S2851[121], 6}, {NULL}};
const Segment SGI244639[] = {{&S2852[13], 4}, {&S2852[13], 4}, {NULL}};
const Segment SGI244640[] = {{&S2852[38], 5}, {NULL}};
const Segment SGI244641[] = {{&S2852[121], 4}, {NULL}};
const Segment SGI244642[] = {{&S2853[13], 4}, {NULL}};
const Segment SGI244643[] = {{&S2853[29], 5}, {NULL}};
const Segment SGI244644[] = {{&S98038[34], 7}, {NULL}};
const Segment SGI244645[] = {{&S98038[27], 6}, {NULL}};
const Segment SGI244646[] = {{&S98038[94], 9}, {NULL}};
const Segment SGI244647[] = {{&S98038[67], 6}, {NULL}};
const Segment SGI244648[] = {{&S2854[13], 4}, {&S2854[13], 4}, {NULL}};
const Segment SGI244649[] = {{&S2854[38], 5}, {NULL}};
const Segment SGI244650[] = {{&S2854[121], 5}, {NULL}};
const Segment SGI244651[] = {{&S2855[13], 4}, {&S2855[13], 4}, {NULL}};
const Segment SGI244652[] = {{&S2855[38], 5}, {NULL}};
const Segment SGI244653[] = {{&S2855[121], 6}, {NULL}};
const Segment SGI244654[] = {{&S2856[13], 4}, {NULL}};
const Segment SGI244655[] = {{&S2856[26], 5}, {NULL}};
const Segment SGI244656[] = {{&S2856[59], 6}, {NULL}};
const Segment SGI244657[] = {{&S2856[93], 6}, {NULL}};
const Segment SGI244658[] = {{&S2857[13], 4}, {&S2857[13], 4}, {NULL}};
const Segment SGI244659[] = {{&S2857[38], 5}, {NULL}};
const Segment SGI244660[] = {{&S2857[121], 5}, {NULL}};
const Segment SGI244661[] = {{&S2858[13], 4}, {&S2858[13], 4}, {NULL}};
const Segment SGI244662[] = {{&S2858[38], 5}, {NULL}};
const Segment SGI244663[] = {{&S2858[121], 5}, {NULL}};
const Segment SGI244664[] = {{&S2859[13], 4}, {&S2859[112], 6}, {&S2859[13], 4}, {NULL}};
const Segment SGI244665[] = {{&S2859[26], 5}, {&S2859[122], 5}, {NULL}};
const Segment SGI244666[] = {{&S2859[59], 6}, {NULL}};
const Segment SGI244667[] = {{&S2859[93], 6}, {NULL}};
const Segment SGI244668[] = {{&S2860[13], 4}, {&S2860[13], 4}, {NULL}};
const Segment SGI244669[] = {{&S2860[38], 5}, {NULL}};
const Segment SGI244670[] = {{&S2860[121], 6}, {NULL}};
const Segment SGI244671[] = {{&S90855[34], 7}, {NULL}};
const Segment SGI244672[] = {{&S90855[27], 6}, {NULL}};
const Segment SGI244673[] = {{&S90855[94], 9}, {NULL}};
const Segment SGI244674[] = {{&S90855[67], 6}, {NULL}};
const Segment SGI244675[] = {{&S2861[13], 4}, {&S2861[13], 4}, {NULL}};
const Segment SGI244676[] = {{&S2861[38], 5}, {NULL}};
const Segment SGI244677[] = {{&S2861[121], 4}, {NULL}};
const Segment SGI244678[] = {{&S2862[13], 4}, {&S2862[13], 4}, {NULL}};
const Segment SGI244679[] = {{&S2862[38], 5}, {NULL}};
const Segment SGI244680[] = {{&S2862[121], 6}, {NULL}};
const Segment SGI244681[] = {{&S2863[13], 4}, {NULL}};
const Segment SGI244682[] = {{&S2863[26], 5}, {NULL}};
const Segment SGI244683[] = {{&S2863[59], 6}, {NULL}};
const Segment SGI244684[] = {{&S2863[93], 6}, {NULL}};
const Segment SGI244685[] = {{&S2863[107], 6}, {NULL}};
const Segment SGI244686[] = {{&S2864[13], 4}, {NULL}};
const Segment SGI244687[] = {{&S2864[26], 5}, {NULL}};
const Segment SGI244688[] = {{&S2864[59], 3}, {NULL}};
const Segment SGI244689[] = {{&S2864[90], 6}, {NULL}};
const Segment SGI244690[] = {{&S2865[13], 4}, {&S2865[13], 4}, {NULL}};
const Segment SGI244691[] = {{&S2865[38], 5}, {NULL}};
const Segment SGI244692[] = {{&S2865[121], 5}, {NULL}};
const Segment SGI244693[] = {{&S2866[13], 4}, {NULL}};
const Segment SGI244694[] = {{&S2866[26], 5}, {NULL}};
const Segment SGI244695[] = {{&S2866[59], 3}, {NULL}};
const Segment SGI244696[] = {{&S2866[90], 6}, {NULL}};
const Segment SGI244697[] = {{&S2867[13], 4}, {&S2867[13], 4}, {NULL}};
const Segment SGI244698[] = {{&S2867[38], 5}, {NULL}};
const Segment SGI244699[] = {{&S2867[121], 6}, {NULL}};
const Segment SGI244700[] = {{&S90866[44], 7}, {NULL}};
const Segment SGI244701[] = {{&S90866[44], 19}, {NULL}};
const Segment SGI244702[] = {{&S90866[30], 6}, {NULL}};
const Segment SGI244703[] = {{&S90866[132], 3}, {NULL}};
const Segment SGI244704[] = {{&S2868[13], 4}, {NULL}};
const Segment SGI244705[] = {{&S90866[97], 6}, {NULL}};
const Segment SGI244706[] = {{&S2868[26], 5}, {NULL}};
const Segment SGI244707[] = {{&S2868[59], 6}, {NULL}};
const Segment SGI244708[] = {{&S90866[65], 8}, {NULL}};
const Segment SGI244709[] = {{&S2868[93], 6}, {NULL}};
const Segment SGI244710[] = {{&S2868[107], 9}, {&S2868[107], 9}, {&S2868[107], 9}, {NULL}};
const Segment SGI244711[] = {{&S2869[13], 4}, {NULL}};
const Segment SGI244712[] = {{&S2869[26], 5}, {NULL}};
const Segment SGI244713[] = {{&S2869[59], 3}, {NULL}};
const Segment SGI244714[] = {{&S2869[90], 6}, {NULL}};
const Segment SGI244715[] = {{&S2870[13], 4}, {NULL}};
const Segment SGI244716[] = {{&S2870[26], 5}, {NULL}};
const Segment SGI244717[] = {{&S2870[59], 6}, {NULL}};
const Segment SGI244718[] = {{&S2870[107], 6}, {NULL}};
const Segment SGI244719[] = {{&S2870[121], 7}, {NULL}};
const Segment SGI244720[] = {{&S2871[13], 4}, {NULL}};
const Segment SGI244721[] = {{&S2871[26], 5}, {NULL}};
const Segment SGI244722[] = {{&S2871[59], 6}, {NULL}};
const Segment SGI244723[] = {{&S2871[93], 6}, {NULL}};
const Segment SGI244724[] = {{&S2871[107], 9}, {NULL}};
const Segment SGI244725[] = {{&S2872[13], 4}, {NULL}};
const Segment SGI244726[] = {{&S2872[58], 6}, {NULL}};
const Segment SGI244727[] = {{&S2872[92], 6}, {NULL}};
const Segment SGI244728[] = {{&S2873[13], 4}, {NULL}};
const Segment SGI244729[] = {{&S2873[26], 5}, {NULL}};
const Segment SGI244730[] = {{&S2873[59], 3}, {NULL}};
const Segment SGI244731[] = {{&S2873[90], 6}, {NULL}};
const Segment SGI244732[] = {{&S2874[13], 4}, {NULL}};
const Segment SGI244733[] = {{&S2874[26], 5}, {NULL}};
const Segment SGI244734[] = {{&S2874[59], 6}, {NULL}};
const Segment SGI244735[] = {{&S2874[100], 2}, {NULL}};
const Segment SGI244736[] = {{&S99226[44], 12}, {NULL}};
const Segment SGI244737[] = {{&S99226[44], 27}, {NULL}};
const Segment SGI244738[] = {{&S99226[30], 6}, {NULL}};
const Segment SGI244739[] = {{&S99226[119], 3}, {NULL}};
const Segment SGI244740[] = {{&S99226[85], 5}, {NULL}};
const Segment SGI244741[] = {{&S90891[44], 25}, {NULL}};
const Segment SGI244742[] = {{&S90891[44], 36}, {NULL}};
const Segment SGI244743[] = {{&S90891[30], 6}, {NULL}};
const Segment SGI244744[] = {{&S90891[152], 3}, {NULL}};
const Segment SGI244745[] = {{&S90891[117], 6}, {NULL}};
const Segment SGI244746[] = {{&S90891[82], 8}, {NULL}};
const Segment SGI244747[] = {{&S90903[44], 7}, {NULL}};
const Segment SGI244748[] = {{&S90903[44], 19}, {NULL}};
const Segment SGI244749[] = {{&S90903[30], 6}, {NULL}};
const Segment SGI244750[] = {{&S90903[140], 3}, {NULL}};
const Segment SGI244751[] = {{&S90903[105], 6}, {NULL}};
const Segment SGI244752[] = {{&S90903[72], 9}, {NULL}};
const Segment SGI244753[] = {{&S2923[13], 6}, {NULL}};
const Segment SGI244754[] = {{&S2923[35], 3}, {NULL}};
const Segment SGI244755[] = {{&S2923[66], 6}, {NULL}};
const Segment SGI244756[] = {{&S2924[13], 6}, {NULL}};
const Segment SGI244757[] = {{&S2924[35], 3}, {NULL}};
const Segment SGI244758[] = {{&S2924[66], 6}, {NULL}};
const Segment SGI244759[] = {{&S2924[100], 5}, {NULL}};
const Segment SGI244760[] = {{&S2925[13], 6}, {NULL}};
const Segment SGI244761[] = {{&S2925[38], 3}, {NULL}};
const Segment SGI244762[] = {{&S2926[27], 6}, {NULL}};
const Segment SGI244763[] = {{&S2926[52], 3}, {NULL}};
const Segment SGI244764[] = {{&S2926[90], 6}, {NULL}};
const Segment SGI244765[] = {{&S2927[13], 6}, {NULL}};
const Segment SGI244766[] = {{&S90995[34], 10}, {NULL}};
const Segment SGI244767[] = {{&S99322[34], 19}, {NULL}};
const Segment SGI244768[] = {{&S2927[38], 5}, {NULL}};
const Segment SGI244769[] = {{&S90995[27], 6}, {NULL}};
const Segment SGI244770[] = {{&S99322[27], 6}, {NULL}};
const Segment SGI244771[] = {{&S90995[96], 9}, {NULL}};
const Segment SGI244772[] = {{&S99322[106], 9}, {NULL}};
const Segment SGI244773[] = {{&S90995[69], 6}, {NULL}};
const Segment SGI244774[] = {{&S99322[79], 6}, {NULL}};
const Segment SGI244775[] = {{&S2928[13], 6}, {NULL}};
const Segment SGI244776[] = {{&S2928[35], 5}, {NULL}};
const Segment SGI244777[] = {{&S2928[68], 6}, {NULL}};
const Segment SGI244778[] = {{&S2928[102], 5}, {NULL}};
const Segment SGI244779[] = {{&S90996[34], 10}, {NULL}};
const Segment SGI244780[] = {{&S90996[27], 6}, {NULL}};
const Segment SGI244781[] = {{&S90996[96], 9}, {NULL}};
const Segment SGI244782[] = {{&S90996[69], 6}, {NULL}};
const Segment SGI244783[] = {{&S2929[13], 6}, {NULL}};
const Segment SGI244784[] = {{&S2929[35], 3}, {NULL}};
const Segment SGI244785[] = {{&S2929[66], 6}, {NULL}};
const Segment SGI244786[] = {{&S2929[100], 5}, {NULL}};
const Segment SGI244787[] = {{&S2930[13], 6}, {NULL}};
const Segment SGI244788[] = {{&S2930[35], 5}, {NULL}};
const Segment SGI244789[] = {{&S2930[68], 6}, {NULL}};
const Segment SGI244790[] = {{&S2931[13], 6}, {NULL}};
const Segment SGI244791[] = {{&S2931[35], 3}, {NULL}};
const Segment SGI244792[] = {{&S2931[66], 6}, {NULL}};
const Segment SGI244793[] = {{&S2931[100], 5}, {NULL}};
const Segment SGI244794[] = {{&S2932[13], 6}, {NULL}};
const Segment SGI244795[] = {{&S2932[35], 3}, {NULL}};
const Segment SGI244796[] = {{&S2932[66], 6}, {NULL}};
const Segment SGI244797[] = {{&S2932[100], 5}, {NULL}};
const Segment SGI244798[] = {{&S2933[13], 6}, {&S2933[106], 4}, {NULL}};
const Segment SGI244799[] = {{&S2933[35], 3}, {NULL}};
const Segment SGI244800[] = {{&S2933[66], 6}, {NULL}};
const Segment SGI244801[] = {{&S2933[100], 5}, {NULL}};
const Segment SGI244802[] = {{&S2934[13], 6}, {NULL}};
const Segment SGI244803[] = {{&S2934[39], 3}, {NULL}};
const Segment SGI244804[] = {{&S2934[119], 5}, {NULL}};
const Segment SGI244805[] = {{&S2935[13], 6}, {NULL}};
const Segment SGI244806[] = {{&S2935[130], 3}, {NULL}};
const Segment SGI244807[] = {{&S2935[35], 3}, {NULL}};
const Segment SGI244808[] = {{&S2935[66], 6}, {NULL}};
const Segment SGI244809[] = {{&S91006[44], 9}, {NULL}};
const Segment SGI244810[] = {{&S2935[100], 5}, {NULL}};
const Segment SGI244811[] = {{&S91006[44], 20}, {NULL}};
const Segment SGI244812[] = {{&S91006[30], 6}, {NULL}};
const Segment SGI244813[] = {{&S91006[113], 3}, {NULL}};
const Segment SGI244814[] = {{&S91006[78], 6}, {NULL}};
const Segment SGI244815[] = {{&S91006[148], 3}, {NULL}};
const Segment SGI244816[] = {{&S2936[13], 6}, {&S2936[108], 4}, {NULL}};
const Segment SGI244817[] = {{&S2936[35], 5}, {NULL}};
const Segment SGI244818[] = {{&S2936[68], 6}, {NULL}};
const Segment SGI244819[] = {{&S2936[102], 5}, {NULL}};
const Segment SGI244820[] = {{&S2937[13], 6}, {NULL}};
const Segment SGI244821[] = {{&S2937[35], 5}, {NULL}};
const Segment SGI244822[] = {{&S2937[68], 6}, {NULL}};
const Segment SGI244823[] = {{&S2937[102], 5}, {NULL}};
const Segment SGI244824[] = {{&S2937[115], 9}, {NULL}};
const Segment SGI244825[] = {{&S2938[13], 6}, {NULL}};
const Segment SGI244826[] = {{&S2938[35], 5}, {NULL}};
const Segment SGI244827[] = {{&S2938[68], 6}, {NULL}};
const Segment SGI244828[] = {{&S2938[102], 5}, {NULL}};
const Segment SGI244829[] = {{&S2939[13], 6}, {NULL}};
const Segment SGI244830[] = {{&S2939[35], 5}, {NULL}};
const Segment SGI244831[] = {{&S2939[68], 6}, {NULL}};
const Segment SGI244832[] = {{&S2939[102], 5}, {NULL}};
const Segment SGI244833[] = {{&S91014[34], 10}, {NULL}};
const Segment SGI244834[] = {{&S2941[13], 6}, {NULL}};
const Segment SGI244835[] = {{&S2941[132], 3}, {NULL}};
const Segment SGI244836[] = {{&S91014[27], 6}, {NULL}};
const Segment SGI244837[] = {{&S2941[35], 5}, {NULL}};
const Segment SGI244838[] = {{&S2941[68], 6}, {NULL}};
const Segment SGI244839[] = {{&S2941[102], 5}, {NULL}};
const Segment SGI244840[] = {{&S91014[96], 9}, {NULL}};
const Segment SGI244841[] = {{&S91014[69], 6}, {NULL}};
const Segment SGI244842[] = {{&S2942[13], 6}, {NULL}};
const Segment SGI244843[] = {{&S2942[35], 5}, {NULL}};
const Segment SGI244844[] = {{&S2942[68], 6}, {NULL}};
const Segment SGI244845[] = {{&S2942[102], 5}, {NULL}};
const Segment SGI244846[] = {{&S2943[13], 6}, {NULL}};
const Segment SGI244847[] = {{&S2943[35], 5}, {NULL}};
const Segment SGI244848[] = {{&S2943[68], 6}, {NULL}};
const Segment SGI244849[] = {{&S2943[102], 5}, {NULL}};
const Segment SGI244850[] = {{&S91017[34], 12}, {NULL}};
const Segment SGI244851[] = {{&S2943[115], 6}, {NULL}};
const Segment SGI244852[] = {{&S91017[27], 6}, {NULL}};
const Segment SGI244853[] = {{&S91017[102], 9}, {NULL}};
const Segment SGI244854[] = {{&S91017[75], 6}, {NULL}};
const Segment SGI244855[] = {{&S2944[13], 6}, {NULL}};
const Segment SGI244856[] = {{&S2944[44], 5}, {NULL}};
const Segment SGI244857[] = {{&S2944[126], 5}, {NULL}};
const Segment SGI244858[] = {{&S2945[13], 6}, {NULL}};
const Segment SGI244859[] = {{&S2945[41], 5}, {NULL}};
const Segment SGI244860[] = {{&S2945[124], 5}, {NULL}};
const Segment SGI244861[] = {{&S2946[13], 6}, {NULL}};
const Segment SGI244862[] = {{&S2946[28], 3}, {NULL}};
const Segment SGI244863[] = {{&S2946[59], 6}, {NULL}};
const Segment SGI244864[] = {{&S2946[93], 5}, {NULL}};
const Segment SGI244865[] = {{&S2946[106], 7}, {NULL}};
const Segment SGI244866[] = {{&S2947[44], 6}, {NULL}};
const Segment SGI244867[] = {{&S2947[69], 5}, {NULL}};
const Segment SGI244868[] = {{&S2947[109], 6}, {NULL}};
const Segment SGI244869[] = {{&S2948[13], 6}, {&S2948[13], 6}, {NULL}};
const Segment SGI244870[] = {{&S2948[75], 6}, {NULL}};
const Segment SGI244871[] = {{&S2948[109], 4}, {NULL}};
const Segment SGI244872[] = {{&S2949[13], 6}, {NULL}};
const Segment SGI244873[] = {{&S2949[35], 5}, {NULL}};
const Segment SGI244874[] = {{&S2949[68], 6}, {NULL}};
const Segment SGI244875[] = {{&S2949[102], 5}, {NULL}};
const Segment SGI244876[] = {{&S2950[13], 6}, {NULL}};
const Segment SGI244877[] = {{&S2950[40], 5}, {NULL}};
const Segment SGI244878[] = {{&S2950[123], 5}, {NULL}};
const Segment SGI244879[] = {{&S2951[13], 6}, {&S2951[128], 4}, {NULL}};
const Segment SGI244880[] = {{&S2951[75], 6}, {NULL}};
const Segment SGI244881[] = {{&S2951[109], 4}, {NULL}};
const Segment SGI244882[] = {{&S2952[13], 6}, {&S2952[13], 6}, {NULL}};
const Segment SGI244883[] = {{&S2952[35], 5}, {&S2952[130], 5}, {NULL}};
const Segment SGI244884[] = {{&S2952[68], 6}, {NULL}};
const Segment SGI244885[] = {{&S2952[102], 5}, {NULL}};
const Segment SGI244886[] = {{&S2953[13], 6}, {NULL}};
const Segment SGI244887[] = {{&S2953[35], 5}, {NULL}};
const Segment SGI244888[] = {{&S2953[68], 6}, {NULL}};
const Segment SGI244889[] = {{&S2953[102], 5}, {NULL}};
const Segment SGI244890[] = {{&S2954[15], 6}, {NULL}};
const Segment SGI244891[] = {{&S2954[33], 5}, {NULL}};
const Segment SGI244892[] = {{&S2954[0], 7}, {NULL}};
const Segment SGI244893[] = {{&S2955[13], 6}, {NULL}};
const Segment SGI244894[] = {{&S2955[132], 5}, {NULL}};
const Segment SGI244895[] = {{&S2955[35], 5}, {NULL}};
const Segment SGI244896[] = {{&S2955[68], 6}, {NULL}};
const Segment SGI244897[] = {{&S2955[102], 5}, {NULL}};
const Segment SGI244898[] = {{&S2956[15], 6}, {NULL}};
const Segment SGI244899[] = {{&S2956[33], 3}, {NULL}};
const Segment SGI244900[] = {{&S2956[0], 7}, {NULL}};
const Segment SGI244901[] = {{&S2957[17], 6}, {NULL}};
const Segment SGI244902[] = {{&S2957[35], 5}, {NULL}};
const Segment SGI244903[] = {{&S2958[13], 6}, {&S2958[13], 6}, {NULL}};
const Segment SGI244904[] = {{&S2958[35], 5}, {NULL}};
const Segment SGI244905[] = {{&S2958[68], 6}, {NULL}};
const Segment SGI244906[] = {{&S2958[102], 5}, {NULL}};
const Segment SGI244907[] = {{&S2959[13], 6}, {NULL}};
const Segment SGI244908[] = {{&S2959[35], 5}, {NULL}};
const Segment SGI244909[] = {{&S2959[68], 6}, {NULL}};
const Segment SGI244910[] = {{&S2959[102], 5}, {NULL}};
const Segment SGI244911[] = {{&S2959[115], 9}, {NULL}};
const Segment SGI244912[] = {{&S2960[13], 6}, {&S2960[151], 4}, {&S2960[151], 4}, {NULL}};
const Segment SGI244913[] = {{&S2960[35], 5}, {NULL}};
const Segment SGI244914[] = {{&S2960[68], 6}, {NULL}};
const Segment SGI244915[] = {{&S2960[102], 5}, {NULL}};
const Segment SGI244916[] = {{&S99392[44], 11}, {NULL}};
const Segment SGI244917[] = {{&S99392[44], 23}, {NULL}};
const Segment SGI244918[] = {{&S99392[30], 6}, {NULL}};
const Segment SGI244919[] = {{&S99392[116], 3}, {NULL}};
const Segment SGI244920[] = {{&S99392[81], 6}, {NULL}};
const Segment SGI244921[] = {{&S99402[34], 10}, {NULL}};
const Segment SGI244922[] = {{&S99402[27], 6}, {NULL}};
const Segment SGI244923[] = {{&S99402[97], 9}, {NULL}};
const Segment SGI244924[] = {{&S99402[70], 6}, {NULL}};
const Segment SGI244925[] = {{&S99440[34], 6}, {NULL}};
const Segment SGI244926[] = {{&S99440[27], 6}, {NULL}};
const Segment SGI244927[] = {{&S99440[95], 9}, {NULL}};
const Segment SGI244928[] = {{&S99440[68], 6}, {NULL}};
const Segment SGI244929[] = {{&S99452[34], 10}, {NULL}};
const Segment SGI244930[] = {{&S99452[27], 6}, {NULL}};
const Segment SGI244931[] = {{&S99452[97], 9}, {NULL}};
const Segment SGI244932[] = {{&S99452[70], 6}, {NULL}};
const Segment SGI244933[] = {{&S98181[44], 10}, {NULL}};
const Segment SGI244934[] = {{&S98181[44], 22}, {NULL}};
const Segment SGI244935[] = {{&S98181[30], 6}, {NULL}};
const Segment SGI244936[] = {{&S98181[80], 6}, {NULL}};
const Segment SGI244937[] = {{&S98181[107], 13}, {NULL}};
const Segment SGI244938[] = {{&S98205[34], 9}, {NULL}};
const Segment SGI244939[] = {{&S98205[27], 6}, {NULL}};
const Segment SGI244940[] = {{&S98205[96], 9}, {NULL}};
const Segment SGI244941[] = {{&S98205[69], 6}, {NULL}};
const Segment SGI244942[] = {{&S98214[34], 13}, {NULL}};
const Segment SGI244943[] = {{&S98214[27], 6}, {NULL}};
const Segment SGI244944[] = {{&S98214[100], 9}, {NULL}};
const Segment SGI244945[] = {{&S98214[73], 6}, {NULL}};
const Segment SGI244946[] = {{&S98221[44], 10}, {NULL}};
const Segment SGI244947[] = {{&S98221[44], 22}, {NULL}};
const Segment SGI244948[] = {{&S98221[30], 6}, {NULL}};
const Segment SGI244949[] = {{&S98221[115], 3}, {NULL}};
const Segment SGI244950[] = {{&S98221[80], 6}, {NULL}};
const Segment SGI244951[] = {{&S99512[34], 7}, {NULL}};
const Segment SGI244952[] = {{&S99512[27], 6}, {NULL}};
const Segment SGI244953[] = {{&S99512[94], 9}, {NULL}};
const Segment SGI244954[] = {{&S99512[67], 6}, {NULL}};
const Segment SGI244955[] = {{&S104557[32], 9}, {NULL}};
const Segment SGI244956[] = {{&S104557[27], 4}, {NULL}};
const Segment SGI244957[] = {{&S104557[94], 9}, {NULL}};
const Segment SGI244958[] = {{&S104557[67], 6}, {NULL}};
const Segment SGI244959[] = {{&S104557[135], 8}, {NULL}};
const Segment SGI244960[] = {{&S98245[34], 7}, {NULL}};
const Segment SGI244961[] = {{&S98245[27], 6}, {NULL}};
const Segment SGI244962[] = {{&S98245[94], 9}, {NULL}};
const Segment SGI244963[] = {{&S98245[67], 6}, {NULL}};
const Segment SGI244964[] = {{&S98253[34], 7}, {NULL}};
const Segment SGI244965[] = {{&S98253[27], 6}, {NULL}};
const Segment SGI244966[] = {{&S98253[94], 9}, {NULL}};
const Segment SGI244967[] = {{&S98253[67], 6}, {NULL}};
const Segment SGI244968[] = {{&S91071[44], 10}, {NULL}};
const Segment SGI244969[] = {{&S98296[34], 7}, {NULL}};
const Segment SGI244970[] = {{&S91071[44], 22}, {NULL}};
const Segment SGI244971[] = {{&S91071[30], 6}, {NULL}};
const Segment SGI244972[] = {{&S98296[27], 6}, {NULL}};
const Segment SGI244973[] = {{&S91071[115], 3}, {NULL}};
const Segment SGI244974[] = {{&S91071[80], 6}, {NULL}};
const Segment SGI244975[] = {{&S98296[110], 9}, {NULL}};
const Segment SGI244976[] = {{&S98296[83], 6}, {NULL}};
const Segment SGI244977[] = {{&S98296[151], 8}, {NULL}};
const Segment SGI244978[] = {{&S98301[34], 9}, {NULL}};
const Segment SGI244979[] = {{&S98301[27], 6}, {NULL}};
const Segment SGI244980[] = {{&S98301[96], 9}, {NULL}};
const Segment SGI244981[] = {{&S98301[69], 6}, {NULL}};
const Segment SGI244982[] = {{&S98301[137], 8}, {NULL}};
const Segment SGI244983[] = {{&S98303[34], 10}, {NULL}};
const Segment SGI244984[] = {{&S98303[27], 6}, {NULL}};
const Segment SGI244985[] = {{&S98303[97], 9}, {NULL}};
const Segment SGI244986[] = {{&S98303[70], 6}, {NULL}};
const Segment SGI244987[] = {{&S91086[34], 10}, {NULL}};
const Segment SGI244988[] = {{&S91086[27], 6}, {NULL}};
const Segment SGI244989[] = {{&S91086[96], 9}, {NULL}};
const Segment SGI244990[] = {{&S91086[69], 6}, {NULL}};
const Segment SGI244991[] = {{&S98313[34], 10}, {NULL}};
const Segment SGI244992[] = {{&S98313[27], 6}, {NULL}};
const Segment SGI244993[] = {{&S98313[97], 9}, {NULL}};
const Segment SGI244994[] = {{&S98313[70], 6}, {NULL}};
const Segment SGI244995[] = {{&S98313[131], 8}, {NULL}};
const Segment SGI244996[] = {{&S98327[34], 13}, {NULL}};
const Segment SGI244997[] = {{&S98327[27], 6}, {NULL}};
const Segment SGI244998[] = {{&S98327[100], 9}, {NULL}};
const Segment SGI244999[] = {{&S98327[73], 6}, {NULL}};
const Segment SGI245000[] = {{&S98337[44], 15}, {NULL}};
const Segment SGI245001[] = {{&S98337[44], 27}, {NULL}};
const Segment SGI245002[] = {{&S98337[30], 6}, {NULL}};
const Segment SGI245003[] = {{&S98337[136], 3}, {NULL}};
const Segment SGI245004[] = {{&S98337[101], 6}, {NULL}};
const Segment SGI245005[] = {{&S98337[73], 8}, {NULL}};
const Segment SGI245006[] = {{&S98341[44], 9}, {NULL}};
const Segment SGI245007[] = {{&S98341[44], 21}, {NULL}};
const Segment SGI245008[] = {{&S98341[30], 6}, {NULL}};
const Segment SGI245009[] = {{&S98341[114], 3}, {NULL}};
const Segment SGI245010[] = {{&S98341[79], 6}, {NULL}};
const Segment SGI245011[] = {{&S98341[146], 15}, {NULL}};
const Segment SGI245012[] = {{&S98351[44], 10}, {NULL}};
const Segment SGI245013[] = {{&S98351[44], 22}, {NULL}};
const Segment SGI245014[] = {{&S98351[30], 6}, {NULL}};
const Segment SGI245015[] = {{&S98351[113], 3}, {NULL}};
const Segment SGI245016[] = {{&S98351[80], 5}, {NULL}};
const Segment SGI245017[] = {{&S98351[117], 14}, {NULL}};
const Segment SGI245018[] = {{&S98354[34], 10}, {NULL}};
const Segment SGI245019[] = {{&S98354[27], 6}, {NULL}};
const Segment SGI245020[] = {{&S98354[97], 9}, {NULL}};
const Segment SGI245021[] = {{&S98354[70], 6}, {NULL}};
const Segment SGI245022[] = {{&S98356[34], 10}, {NULL}};
const Segment SGI245023[] = {{&S98356[27], 6}, {NULL}};
const Segment SGI245024[] = {{&S98356[97], 9}, {NULL}};
const Segment SGI245025[] = {{&S98356[70], 6}, {NULL}};
const Segment SGI245026[] = {{&S98359[44], 10}, {NULL}};
const Segment SGI245027[] = {{&S98359[44], 22}, {NULL}};
const Segment SGI245028[] = {{&S98359[30], 6}, {NULL}};
const Segment SGI245029[] = {{&S98359[113], 3}, {NULL}};
const Segment SGI245030[] = {{&S98359[80], 5}, {NULL}};
const Segment SGI245031[] = {{&S98363[34], 10}, {NULL}};
const Segment SGI245032[] = {{&S98363[27], 6}, {NULL}};
const Segment SGI245033[] = {{&S98363[97], 9}, {NULL}};
const Segment SGI245034[] = {{&S98363[70], 6}, {NULL}};
const Segment SGI245035[] = {{&S98364[44], 7}, {NULL}};
const Segment SGI245036[] = {{&S98364[44], 25}, {NULL}};
const Segment SGI245037[] = {{&S98364[30], 6}, {NULL}};
const Segment SGI245038[] = {{&S98364[118], 3}, {NULL}};
const Segment SGI245039[] = {{&S98364[83], 6}, {NULL}};
const Segment SGI245040[] = {{&S98364[143], 7}, {NULL}};
const Segment SGI245041[] = {{&S98365[44], 14}, {NULL}};
const Segment SGI245042[] = {{&S98365[44], 31}, {NULL}};
const Segment SGI245043[] = {{&S98365[30], 6}, {NULL}};
const Segment SGI245044[] = {{&S98365[122], 3}, {NULL}};
const Segment SGI245045[] = {{&S98365[89], 5}, {NULL}};
const Segment SGI245046[] = {{&S98365[126], 14}, {NULL}};
const Segment SGI245047[] = {{&S98372[34], 10}, {NULL}};
const Segment SGI245048[] = {{&S98372[27], 6}, {NULL}};
const Segment SGI245049[] = {{&S98372[97], 9}, {NULL}};
const Segment SGI245050[] = {{&S98372[70], 6}, {NULL}};
const Segment SGI245051[] = {{&S98380[44], 11}, {NULL}};
const Segment SGI245052[] = {{&S98380[44], 23}, {NULL}};
const Segment SGI245053[] = {{&S98380[30], 6}, {NULL}};
const Segment SGI245054[] = {{&S98380[81], 6}, {NULL}};
const Segment SGI245055[] = {{&S98380[108], 13}, {NULL}};
const Segment SGI245056[] = {{&S98385[34], 25}, {NULL}};
const Segment SGI245057[] = {{&S98385[27], 6}, {NULL}};
const Segment SGI245058[] = {{&S98385[112], 9}, {NULL}};
const Segment SGI245059[] = {{&S98385[85], 6}, {NULL}};
const Segment SGI245060[] = {{&S91184[34], 18}, {NULL}};
const Segment SGI245061[] = {{&S91184[27], 6}, {NULL}};
const Segment SGI245062[] = {{&S91184[105], 9}, {NULL}};
const Segment SGI245063[] = {{&S91184[78], 6}, {NULL}};
const Segment SGI245064[] = {{&S91185[34], 9}, {NULL}};
const Segment SGI245065[] = {{&S91185[27], 6}, {NULL}};
const Segment SGI245066[] = {{&S91185[96], 9}, {NULL}};
const Segment SGI245067[] = {{&S91185[69], 6}, {NULL}};
const Segment SGI245068[] = {{&S99685[34], 13}, {NULL}};
const Segment SGI245069[] = {{&S99685[27], 6}, {NULL}};
const Segment SGI245070[] = {{&S99685[100], 9}, {NULL}};
const Segment SGI245071[] = {{&S99685[73], 6}, {NULL}};
const Segment SGI245072[] = {{&S99687[34], 13}, {NULL}};
const Segment SGI245073[] = {{&S99687[27], 6}, {NULL}};
const Segment SGI245074[] = {{&S99687[100], 9}, {NULL}};
const Segment SGI245075[] = {{&S99687[73], 6}, {NULL}};
const Segment SGI245076[] = {{&S91206[44], 7}, {NULL}};
const Segment SGI245077[] = {{&S91206[44], 20}, {NULL}};
const Segment SGI245078[] = {{&S91206[30], 6}, {NULL}};
const Segment SGI245079[] = {{&S91206[131], 3}, {NULL}};
const Segment SGI245080[] = {{&S91206[96], 6}, {NULL}};
const Segment SGI245081[] = {{&S91206[66], 8}, {NULL}};
const Segment SGI245082[] = {{&S98418[34], 16}, {NULL}};
const Segment SGI245083[] = {{&S98418[27], 6}, {NULL}};
const Segment SGI245084[] = {{&S98418[103], 9}, {NULL}};
const Segment SGI245085[] = {{&S98418[76], 6}, {NULL}};
const Segment SGI245086[] = {{&S98419[52], 13}, {NULL}};
const Segment SGI245087[] = {{&S98419[52], 25}, {NULL}};
const Segment SGI245088[] = {{&S98419[38], 6}, {NULL}};
const Segment SGI245089[] = {{&S98419[91], 5}, {NULL}};
const Segment SGI245090[] = {{&S98419[11], 3}, {NULL}};
const Segment SGI245091[] = {{&S98420[52], 7}, {NULL}};
const Segment SGI245092[] = {{&S98420[52], 25}, {NULL}};
const Segment SGI245093[] = {{&S98420[38], 6}, {NULL}};
const Segment SGI245094[] = {{&S98420[91], 5}, {NULL}};
const Segment SGI245095[] = {{&S98420[11], 3}, {NULL}};
const Segment SGI245096[] = {{&S99706[45], 10}, {NULL}};
const Segment SGI245097[] = {{&S99706[45], 72}, {NULL}};
const Segment SGI245098[] = {{&S99706[31], 6}, {NULL}};
const Segment SGI245099[] = {{&S99707[44], 10}, {NULL}};
const Segment SGI245100[] = {{&S99707[44], 22}, {NULL}};
const Segment SGI245101[] = {{&S99707[30], 6}, {NULL}};
const Segment SGI245102[] = {{&S99707[115], 3}, {NULL}};
const Segment SGI245103[] = {{&S99707[80], 6}, {NULL}};
const Segment SGI245104[] = {{&S98424[44], 11}, {NULL}};
const Segment SGI245105[] = {{&S98424[44], 23}, {NULL}};
const Segment SGI245106[] = {{&S98424[30], 6}, {NULL}};
const Segment SGI245107[] = {{&S98424[81], 6}, {NULL}};
const Segment SGI245108[] = {{&S98424[108], 13}, {NULL}};
const Segment SGI245109[] = {{&S91222[34], 10}, {NULL}};
const Segment SGI245110[] = {{&S91222[27], 6}, {NULL}};
const Segment SGI245111[] = {{&S91222[97], 9}, {NULL}};
const Segment SGI245112[] = {{&S91222[70], 6}, {NULL}};
const Segment SGI245113[] = {{&S91237[44], 8}, {NULL}};
const Segment SGI245114[] = {{&S91237[44], 19}, {NULL}};
const Segment SGI245115[] = {{&S91237[30], 6}, {NULL}};
const Segment SGI245116[] = {{&S91237[130], 3}, {NULL}};
const Segment SGI245117[] = {{&S91237[95], 6}, {NULL}};
const Segment SGI245118[] = {{&S91237[65], 8}, {NULL}};
const Segment SGI245119[] = {{&S91238[44], 8}, {NULL}};
const Segment SGI245120[] = {{&S91238[44], 20}, {NULL}};
const Segment SGI245121[] = {{&S91238[30], 6}, {NULL}};
const Segment SGI245122[] = {{&S91238[131], 3}, {NULL}};
const Segment SGI245123[] = {{&S91238[96], 6}, {NULL}};
const Segment SGI245124[] = {{&S91238[66], 8}, {NULL}};
const Segment SGI245125[] = {{&S98444[52], 9}, {NULL}};
const Segment SGI245126[] = {{&S98444[52], 21}, {NULL}};
const Segment SGI245127[] = {{&S98444[38], 6}, {NULL}};
const Segment SGI245128[] = {{&S98444[87], 5}, {NULL}};
const Segment SGI245129[] = {{&S98444[11], 3}, {NULL}};
const Segment SGI245130[] = {{&S98446[34], 6}, {NULL}};
const Segment SGI245131[] = {{&S98446[27], 6}, {NULL}};
const Segment SGI245132[] = {{&S98446[93], 9}, {NULL}};
const Segment SGI245133[] = {{&S98446[66], 6}, {NULL}};
const Segment SGI245134[] = {{&S98446[134], 8}, {NULL}};
const Segment SGI245135[] = {{&S98447[44], 11}, {NULL}};
const Segment SGI245136[] = {{&S98447[44], 23}, {NULL}};
const Segment SGI245137[] = {{&S98447[30], 6}, {NULL}};
const Segment SGI245138[] = {{&S98447[81], 6}, {NULL}};
const Segment SGI245139[] = {{&S98447[108], 13}, {NULL}};
const Segment SGI245140[] = {{&S98453[44], 10}, {NULL}};
const Segment SGI245141[] = {{&S98453[44], 22}, {NULL}};
const Segment SGI245142[] = {{&S98453[30], 6}, {NULL}};
const Segment SGI245143[] = {{&S98453[80], 6}, {NULL}};
const Segment SGI245144[] = {{&S98453[107], 13}, {NULL}};
const Segment SGI245145[] = {{&S91269[34], 10}, {NULL}};
const Segment SGI245146[] = {{&S91269[27], 6}, {NULL}};
const Segment SGI245147[] = {{&S91269[96], 9}, {NULL}};
const Segment SGI245148[] = {{&S91269[69], 6}, {NULL}};
const Segment SGI245149[] = {{&S98472[44], 13}, {NULL}};
const Segment SGI245150[] = {{&S98472[44], 25}, {NULL}};
const Segment SGI245151[] = {{&S98472[30], 6}, {NULL}};
const Segment SGI245152[] = {{&S98472[118], 3}, {NULL}};
const Segment SGI245153[] = {{&S98472[83], 6}, {NULL}};
const Segment SGI245154[] = {{&S98472[138], 8}, {NULL}};
const Segment SGI245155[] = {{&S98472[147], 13}, {NULL}};
const Segment SGI245156[] = {{&S98478[34], 9}, {NULL}};
const Segment SGI245157[] = {{&S98478[27], 6}, {NULL}};
const Segment SGI245158[] = {{&S98478[96], 9}, {NULL}};
const Segment SGI245159[] = {{&S98478[69], 6}, {NULL}};
const Segment SGI245160[] = {{&S98484[44], 10}, {NULL}};
const Segment SGI245161[] = {{&S98484[44], 22}, {NULL}};
const Segment SGI245162[] = {{&S98484[30], 6}, {NULL}};
const Segment SGI245163[] = {{&S98484[115], 3}, {NULL}};
const Segment SGI245164[] = {{&S98484[80], 6}, {NULL}};
const Segment SGI245165[] = {{&S98484[135], 8}, {NULL}};
const Segment SGI245166[] = {{&S98484[144], 13}, {NULL}};
const Segment SGI245167[] = {{&S98485[34], 25}, {NULL}};
const Segment SGI245168[] = {{&S98485[27], 6}, {NULL}};
const Segment SGI245169[] = {{&S98485[112], 9}, {NULL}};
const Segment SGI245170[] = {{&S98485[85], 6}, {NULL}};
const Segment SGI245171[] = {{&S98489[44], 9}, {NULL}};
const Segment SGI245172[] = {{&S98489[44], 21}, {NULL}};
const Segment SGI245173[] = {{&S98489[30], 6}, {NULL}};
const Segment SGI245174[] = {{&S98489[79], 6}, {NULL}};
const Segment SGI245175[] = {{&S98489[106], 13}, {NULL}};
const Segment SGI245176[] = {{&S98496[34], 10}, {NULL}};
const Segment SGI245177[] = {{&S98496[27], 6}, {NULL}};
const Segment SGI245178[] = {{&S98496[97], 9}, {NULL}};
const Segment SGI245179[] = {{&S98496[70], 6}, {NULL}};
const Segment SGI245180[] = {{&S98501[34], 10}, {NULL}};
const Segment SGI245181[] = {{&S98501[27], 6}, {NULL}};
const Segment SGI245182[] = {{&S98501[97], 9}, {NULL}};
const Segment SGI245183[] = {{&S98501[70], 6}, {NULL}};
const Segment SGI245184[] = {{&S98501[138], 8}, {NULL}};
const Segment SGI245185[] = {{&S98502[34], 7}, {NULL}};
const Segment SGI245186[] = {{&S98502[27], 6}, {NULL}};
const Segment SGI245187[] = {{&S98502[100], 9}, {NULL}};
const Segment SGI245188[] = {{&S98502[73], 6}, {NULL}};
const Segment SGI245189[] = {{&S98502[141], 8}, {NULL}};
const Segment SGI245190[] = {{&S98504[34], 10}, {NULL}};
const Segment SGI245191[] = {{&S98504[27], 6}, {NULL}};
const Segment SGI245192[] = {{&S98504[97], 9}, {NULL}};
const Segment SGI245193[] = {{&S98504[70], 6}, {NULL}};
const Segment SGI245194[] = {{&S98504[138], 8}, {NULL}};
const Segment SGI245195[] = {{&S98508[34], 14}, {NULL}};
const Segment SGI245196[] = {{&S98508[27], 6}, {NULL}};
const Segment SGI245197[] = {{&S98508[101], 9}, {NULL}};
const Segment SGI245198[] = {{&S98508[74], 6}, {NULL}};
const Segment SGI245199[] = {{&S98509[34], 12}, {NULL}};
const Segment SGI245200[] = {{&S98509[27], 6}, {NULL}};
const Segment SGI245201[] = {{&S98509[99], 9}, {NULL}};
const Segment SGI245202[] = {{&S98509[72], 6}, {NULL}};
const Segment SGI245203[] = {{&S99838[34], 10}, {NULL}};
const Segment SGI245204[] = {{&S99838[27], 6}, {NULL}};
const Segment SGI245205[] = {{&S99838[97], 9}, {NULL}};
const Segment SGI245206[] = {{&S99838[70], 6}, {NULL}};
const Segment SGI245207[] = {{&S99848[34], 11}, {NULL}};
const Segment SGI245208[] = {{&S99848[27], 6}, {NULL}};
const Segment SGI245209[] = {{&S99848[98], 9}, {NULL}};
const Segment SGI245210[] = {{&S99848[71], 6}, {NULL}};
const Segment SGI245211[] = {{&S99895[34], 25}, {NULL}};
const Segment SGI245212[] = {{&S99895[27], 6}, {NULL}};
const Segment SGI245213[] = {{&S99895[112], 9}, {NULL}};
const Segment SGI245214[] = {{&S99895[85], 6}, {NULL}};
const Segment SGI245215[] = {{&S98635[44], 18}, {NULL}};
const Segment SGI245216[] = {{&S98635[44], 30}, {NULL}};
const Segment SGI245217[] = {{&S98635[30], 6}, {NULL}};
const Segment SGI245218[] = {{&S98635[123], 3}, {NULL}};
const Segment SGI245219[] = {{&S98635[88], 6}, {NULL}};
const Segment SGI245220[] = {{&S98636[34], 8}, {NULL}};
const Segment SGI245221[] = {{&S98636[27], 6}, {NULL}};
const Segment SGI245222[] = {{&S98636[95], 9}, {NULL}};
const Segment SGI245223[] = {{&S98636[68], 6}, {NULL}};
const Segment SGI245224[] = {{&S98637[45], 9}, {NULL}};
const Segment SGI245225[] = {{&S98637[30], 6}, {NULL}};
const Segment SGI245226[] = {{&S98637[114], 3}, {NULL}};
const Segment SGI245227[] = {{&S98637[80], 5}, {NULL}};
const Segment SGI245228[] = {{&S98637[138], 13}, {NULL}};
const Segment SGI245229[] = {{&S98668[44], 10}, {NULL}};
const Segment SGI245230[] = {{&S98668[44], 22}, {NULL}};
const Segment SGI245231[] = {{&S98668[30], 6}, {NULL}};
const Segment SGI245232[] = {{&S98668[131], 3}, {NULL}};
const Segment SGI245233[] = {{&S98668[96], 6}, {NULL}};
const Segment SGI245234[] = {{&S98668[68], 8}, {NULL}};
const Segment SGI245235[] = {{&S98672[34], 7}, {NULL}};
const Segment SGI245236[] = {{&S98672[27], 6}, {NULL}};
const Segment SGI245237[] = {{&S98672[102], 9}, {NULL}};
const Segment SGI245238[] = {{&S98672[75], 6}, {NULL}};
const Segment SGI245239[] = {{&S98673[34], 7}, {NULL}};
const Segment SGI245240[] = {{&S98673[27], 6}, {NULL}};
const Segment SGI245241[] = {{&S98673[102], 9}, {NULL}};
const Segment SGI245242[] = {{&S98673[75], 6}, {NULL}};
const Segment SGI245243[] = {{&S91364[34], 12}, {NULL}};
const Segment SGI245244[] = {{&S91364[27], 6}, {NULL}};
const Segment SGI245245[] = {{&S91364[102], 9}, {NULL}};
const Segment SGI245246[] = {{&S91364[75], 6}, {NULL}};
const Segment SGI245247[] = {{&S98677[34], 12}, {NULL}};
const Segment SGI245248[] = {{&S98677[27], 6}, {NULL}};
const Segment SGI245249[] = {{&S98677[99], 9}, {NULL}};
const Segment SGI245250[] = {{&S98677[72], 6}, {NULL}};
const Segment SGI245251[] = {{&S98678[34], 9}, {NULL}};
const Segment SGI245252[] = {{&S98678[27], 6}, {NULL}};
const Segment SGI245253[] = {{&S98678[96], 9}, {NULL}};
const Segment SGI245254[] = {{&S98678[69], 6}, {NULL}};
const Segment SGI245255[] = {{&S98701[34], 9}, {NULL}};
const Segment SGI245256[] = {{&S98701[27], 6}, {NULL}};
const Segment SGI245257[] = {{&S98701[101], 9}, {NULL}};
const Segment SGI245258[] = {{&S98701[74], 6}, {NULL}};
const Segment SGI245259[] = {{&S98708[44], 10}, {NULL}};
const Segment SGI245260[] = {{&S98708[44], 22}, {NULL}};
const Segment SGI245261[] = {{&S98708[30], 6}, {NULL}};
const Segment SGI245262[] = {{&S98708[131], 3}, {NULL}};
const Segment SGI245263[] = {{&S98708[96], 6}, {NULL}};
const Segment SGI245264[] = {{&S98708[68], 8}, {NULL}};
const Segment SGI245265[] = {{&S98717[44], 10}, {NULL}};
const Segment SGI245266[] = {{&S98717[44], 22}, {NULL}};
const Segment SGI245267[] = {{&S98717[30], 6}, {NULL}};
const Segment SGI245268[] = {{&S98717[115], 3}, {NULL}};
const Segment SGI245269[] = {{&S98717[80], 6}, {NULL}};
const Segment SGI245270[] = {{&S98728[34], 8}, {NULL}};
const Segment SGI245271[] = {{&S98728[27], 6}, {NULL}};
const Segment SGI245272[] = {{&S98728[95], 9}, {NULL}};
const Segment SGI245273[] = {{&S98728[68], 6}, {NULL}};
const Segment SGI245274[] = {{&S98733[34], 10}, {NULL}};
const Segment SGI245275[] = {{&S98733[27], 6}, {NULL}};
const Segment SGI245276[] = {{&S98733[97], 9}, {NULL}};
const Segment SGI245277[] = {{&S98733[70], 6}, {NULL}};
const Segment SGI245278[] = {{&S98735[34], 8}, {NULL}};
const Segment SGI245279[] = {{&S98735[27], 6}, {NULL}};
const Segment SGI245280[] = {{&S98735[95], 9}, {NULL}};
const Segment SGI245281[] = {{&S98735[68], 6}, {NULL}};
const Segment SGI245282[] = {{&S100021[44], 5}, {NULL}};
const Segment SGI245283[] = {{&S100021[44], 17}, {NULL}};
const Segment SGI245284[] = {{&S100021[30], 6}, {NULL}};
const Segment SGI245285[] = {{&S100021[110], 3}, {NULL}};
const Segment SGI245286[] = {{&S100021[75], 6}, {NULL}};
const Segment SGI245287[] = {{&S100029[44], 7}, {NULL}};
const Segment SGI245288[] = {{&S100029[44], 19}, {NULL}};
const Segment SGI245289[] = {{&S100029[30], 6}, {NULL}};
const Segment SGI245290[] = {{&S98749[44], 10}, {NULL}};
const Segment SGI245291[] = {{&S98749[44], 22}, {NULL}};
const Segment SGI245292[] = {{&S98749[30], 6}, {NULL}};
const Segment SGI245293[] = {{&S100029[110], 3}, {NULL}};
const Segment SGI245294[] = {{&S100029[77], 5}, {NULL}};
const Segment SGI245295[] = {{&S98749[113], 3}, {NULL}};
const Segment SGI245296[] = {{&S98749[80], 5}, {NULL}};
const Segment SGI245297[] = {{&S91442[34], 10}, {NULL}};
const Segment SGI245298[] = {{&S91442[27], 6}, {NULL}};
const Segment SGI245299[] = {{&S91442[97], 9}, {NULL}};
const Segment SGI245300[] = {{&S91442[70], 6}, {NULL}};
const Segment SGI245301[] = {{&S98756[34], 10}, {NULL}};
const Segment SGI245302[] = {{&S98756[27], 6}, {NULL}};
const Segment SGI245303[] = {{&S98756[96], 9}, {NULL}};
const Segment SGI245304[] = {{&S98756[69], 6}, {NULL}};
const Segment SGI245305[] = {{&S91449[34], 18}, {NULL}};
const Segment SGI245306[] = {{&S98757[34], 9}, {NULL}};
const Segment SGI245307[] = {{&S91449[27], 6}, {NULL}};
const Segment SGI245308[] = {{&S98757[27], 6}, {NULL}};
const Segment SGI245309[] = {{&S91449[104], 9}, {NULL}};
const Segment SGI245310[] = {{&S91449[77], 6}, {NULL}};
const Segment SGI245311[] = {{&S98757[96], 9}, {NULL}};
const Segment SGI245312[] = {{&S98757[69], 6}, {NULL}};
const Segment SGI245313[] = {{&S98757[137], 8}, {NULL}};
const Segment SGI245314[] = {{&S98758[44], 14}, {NULL}};
const Segment SGI245315[] = {{&S98758[44], 26}, {NULL}};
const Segment SGI245316[] = {{&S98758[30], 6}, {NULL}};
const Segment SGI245317[] = {{&S98758[84], 6}, {NULL}};
const Segment SGI245318[] = {{&S98758[111], 13}, {NULL}};
const Segment SGI245319[] = {{&S98760[44], 16}, {NULL}};
const Segment SGI245320[] = {{&S98760[44], 35}, {NULL}};
const Segment SGI245321[] = {{&S98760[30], 6}, {NULL}};
const Segment SGI245322[] = {{&S98760[93], 6}, {NULL}};
const Segment SGI245323[] = {{&S98760[120], 13}, {NULL}};
const Segment SGI245324[] = {{&S100049[44], 19}, {NULL}};
const Segment SGI245325[] = {{&S100049[44], 31}, {NULL}};
const Segment SGI245326[] = {{&S100049[30], 6}, {NULL}};
const Segment SGI245327[] = {{&S100049[124], 3}, {NULL}};
const Segment SGI245328[] = {{&S100049[89], 6}, {NULL}};
const Segment SGI245329[] = {{&S98764[34], 10}, {NULL}};
const Segment SGI245330[] = {{&S98764[27], 6}, {NULL}};
const Segment SGI245331[] = {{&S98764[97], 9}, {NULL}};
const Segment SGI245332[] = {{&S98764[70], 6}, {NULL}};
const Segment SGI245333[] = {{&S98766[34], 8}, {NULL}};
const Segment SGI245334[] = {{&S98766[27], 6}, {NULL}};
const Segment SGI245335[] = {{&S98766[100], 9}, {NULL}};
const Segment SGI245336[] = {{&S98766[73], 6}, {NULL}};
const Segment SGI245337[] = {{&S98766[141], 8}, {NULL}};
const Segment SGI245338[] = {{&S98767[34], 10}, {NULL}};
const Segment SGI245339[] = {{&S98767[27], 6}, {NULL}};
const Segment SGI245340[] = {{&S98767[97], 9}, {NULL}};
const Segment SGI245341[] = {{&S98767[70], 6}, {NULL}};
const Segment SGI245342[] = {{&S98772[38], 6}, {NULL}};
const Segment SGI245343[] = {{&S98772[31], 6}, {NULL}};
const Segment SGI245344[] = {{&S98782[34], 10}, {NULL}};
const Segment SGI245345[] = {{&S100071[34], 15}, {NULL}};
const Segment SGI245346[] = {{&S98782[27], 6}, {NULL}};
const Segment SGI245347[] = {{&S100071[27], 6}, {NULL}};
const Segment SGI245348[] = {{&S98782[97], 9}, {NULL}};
const Segment SGI245349[] = {{&S98782[70], 6}, {NULL}};
const Segment SGI245350[] = {{&S100071[102], 9}, {NULL}};
const Segment SGI245351[] = {{&S100071[75], 6}, {NULL}};
const Segment SGI245352[] = {{&S98786[44], 10}, {NULL}};
const Segment SGI245353[] = {{&S98786[44], 22}, {NULL}};
const Segment SGI245354[] = {{&S98786[30], 6}, {NULL}};
const Segment SGI245355[] = {{&S98786[113], 3}, {NULL}};
const Segment SGI245356[] = {{&S98786[80], 5}, {NULL}};
const Segment SGI245357[] = {{&S98789[34], 10}, {NULL}};
const Segment SGI245358[] = {{&S98789[27], 6}, {NULL}};
const Segment SGI245359[] = {{&S98789[97], 9}, {NULL}};
const Segment SGI245360[] = {{&S98789[70], 6}, {NULL}};
const Segment SGI245361[] = {{&S100080[34], 9}, {NULL}};
const Segment SGI245362[] = {{&S100080[27], 6}, {NULL}};
const Segment SGI245363[] = {{&S100080[96], 9}, {NULL}};
const Segment SGI245364[] = {{&S100080[69], 6}, {NULL}};
const Segment SGI245365[] = {{&S98805[44], 10}, {NULL}};
const Segment SGI245366[] = {{&S98805[44], 29}, {NULL}};
const Segment SGI245367[] = {{&S98805[30], 6}, {NULL}};
const Segment SGI245368[] = {{&S98805[138], 3}, {NULL}};
const Segment SGI245369[] = {{&S98805[103], 6}, {NULL}};
const Segment SGI245370[] = {{&S98805[75], 8}, {NULL}};
const Segment SGI245371[] = {{&S98805[170], 15}, {NULL}};
const Segment SGI245372[] = {{&S98806[44], 10}, {NULL}};
const Segment SGI245373[] = {{&S98806[44], 22}, {NULL}};
const Segment SGI245374[] = {{&S98806[30], 6}, {NULL}};
const Segment SGI245375[] = {{&S98806[115], 3}, {NULL}};
const Segment SGI245376[] = {{&S98806[80], 6}, {NULL}};
const Segment SGI245377[] = {{&S98806[134], 12}, {NULL}};
const Segment SGI245378[] = {{&S98822[44], 10}, {NULL}};
const Segment SGI245379[] = {{&S98822[44], 22}, {NULL}};
const Segment SGI245380[] = {{&S98822[30], 6}, {NULL}};
const Segment SGI245381[] = {{&S98822[80], 6}, {NULL}};
const Segment SGI245382[] = {{&S98822[107], 13}, {NULL}};
const Segment SGI245383[] = {{&S98827[44], 10}, {NULL}};
const Segment SGI245384[] = {{&S98827[44], 22}, {NULL}};
const Segment SGI245385[] = {{&S98827[30], 6}, {NULL}};
const Segment SGI245386[] = {{&S98827[113], 3}, {NULL}};
const Segment SGI245387[] = {{&S98827[80], 5}, {NULL}};
const Segment SGI245388[] = {{&S98827[117], 14}, {NULL}};
const Segment SGI245389[] = {{&S98829[44], 8}, {NULL}};
const Segment SGI245390[] = {{&S98829[44], 25}, {NULL}};
const Segment SGI245391[] = {{&S98829[30], 6}, {NULL}};
const Segment SGI245392[] = {{&S98829[83], 6}, {NULL}};
const Segment SGI245393[] = {{&S98829[110], 13}, {NULL}};
const Segment SGI245394[] = {{&S98837[44], 13}, {NULL}};
const Segment SGI245395[] = {{&S98837[44], 29}, {NULL}};
const Segment SGI245396[] = {{&S98837[30], 6}, {NULL}};
const Segment SGI245397[] = {{&S98837[87], 6}, {NULL}};
const Segment SGI245398[] = {{&S98837[114], 13}, {NULL}};
const Segment SGI245399[] = {{&S98841[44], 17}, {NULL}};
const Segment SGI245400[] = {{&S98841[44], 37}, {NULL}};
const Segment SGI245401[] = {{&S98841[30], 6}, {NULL}};
const Segment SGI245402[] = {{&S98841[128], 3}, {NULL}};
const Segment SGI245403[] = {{&S98841[95], 5}, {NULL}};
const Segment SGI245404[] = {{&S98841[132], 14}, {NULL}};
const Segment SGI245405[] = {{&S98842[44], 11}, {NULL}};
const Segment SGI245406[] = {{&S98842[44], 23}, {NULL}};
const Segment SGI245407[] = {{&S98842[30], 6}, {NULL}};
const Segment SGI245408[] = {{&S98842[81], 6}, {NULL}};
const Segment SGI245409[] = {{&S98842[108], 13}, {NULL}};
const Segment SGI245410[] = {{&S98844[44], 9}, {NULL}};
const Segment SGI245411[] = {{&S98844[44], 21}, {NULL}};
const Segment SGI245412[] = {{&S98844[30], 6}, {NULL}};
const Segment SGI245413[] = {{&S98844[79], 6}, {NULL}};
const Segment SGI245414[] = {{&S98844[106], 13}, {NULL}};
const Segment SGI245415[] = {{&S91560[44], 10}, {NULL}};
const Segment SGI245416[] = {{&S91560[44], 21}, {NULL}};
const Segment SGI245417[] = {{&S91560[30], 6}, {NULL}};
const Segment SGI245418[] = {{&S91560[132], 3}, {NULL}};
const Segment SGI245419[] = {{&S91560[97], 6}, {NULL}};
const Segment SGI245420[] = {{&S91560[67], 8}, {NULL}};
const Segment SGI245421[] = {{&S98860[34], 8}, {NULL}};
const Segment SGI245422[] = {{&S98860[27], 6}, {NULL}};
const Segment SGI245423[] = {{&S98860[95], 9}, {NULL}};
const Segment SGI245424[] = {{&S98860[68], 6}, {NULL}};
const Segment SGI245425[] = {{&S98870[45], 10}, {NULL}};
const Segment SGI245426[] = {{&S98870[30], 6}, {NULL}};
const Segment SGI245427[] = {{&S98870[115], 3}, {NULL}};
const Segment SGI245428[] = {{&S98870[81], 5}, {NULL}};
const Segment SGI245429[] = {{&S98870[139], 13}, {NULL}};
const Segment SGI245430[] = {{&S91604[34], 13}, {NULL}};
const Segment SGI245431[] = {{&S91604[27], 6}, {NULL}};
const Segment SGI245432[] = {{&S91604[100], 9}, {NULL}};
const Segment SGI245433[] = {{&S91604[73], 6}, {NULL}};
const Segment SGI245434[] = {{&S91605[44], 7}, {NULL}};
const Segment SGI245435[] = {{&S91605[44], 18}, {NULL}};
const Segment SGI245436[] = {{&S91605[30], 6}, {NULL}};
const Segment SGI245437[] = {{&S91605[111], 3}, {NULL}};
const Segment SGI245438[] = {{&S91605[76], 6}, {NULL}};
const Segment SGI245439[] = {{&S91610[59], 10}, {NULL}};
const Segment SGI245440[] = {{&S91610[59], 21}, {NULL}};
const Segment SGI245441[] = {{&S91610[45], 6}, {NULL}};
const Segment SGI245442[] = {{&S91610[129], 3}, {NULL}};
const Segment SGI245443[] = {{&S91610[94], 6}, {NULL}};
const Segment SGI245444[] = {{&S91636[34], 12}, {NULL}};
const Segment SGI245445[] = {{&S91636[27], 6}, {NULL}};
const Segment SGI245446[] = {{&S91636[98], 9}, {NULL}};
const Segment SGI245447[] = {{&S91636[71], 6}, {NULL}};
const Segment SGI245448[] = {{&S91639[34], 8}, {NULL}};
const Segment SGI245449[] = {{&S91639[27], 6}, {NULL}};
const Segment SGI245450[] = {{&S91639[94], 9}, {NULL}};
const Segment SGI245451[] = {{&S91639[67], 6}, {NULL}};
const Segment SGI245452[] = {{&S91661[44], 10}, {NULL}};
const Segment SGI245453[] = {{&S91661[44], 21}, {NULL}};
const Segment SGI245454[] = {{&S91661[30], 6}, {NULL}};
const Segment SGI245455[] = {{&S91661[112], 3}, {NULL}};
const Segment SGI245456[] = {{&S91661[79], 5}, {NULL}};
const Segment SGI245457[] = {{&S98891[44], 10}, {NULL}};
const Segment SGI245458[] = {{&S98891[44], 22}, {NULL}};
const Segment SGI245459[] = {{&S98891[30], 6}, {NULL}};
const Segment SGI245460[] = {{&S98891[115], 3}, {NULL}};
const Segment SGI245461[] = {{&S98891[80], 6}, {NULL}};
const Segment SGI245462[] = {{&S98891[143], 3}, {NULL}};
const Segment SGI245463[] = {{&S98892[34], 8}, {NULL}};
const Segment SGI245464[] = {{&S98892[27], 6}, {NULL}};
const Segment SGI245465[] = {{&S98892[101], 9}, {NULL}};
const Segment SGI245466[] = {{&S98892[74], 6}, {NULL}};
const Segment SGI245467[] = {{&S98892[135], 8}, {NULL}};
const Segment SGI245468[] = {{&S98893[38], 12}, {NULL}};
const Segment SGI245469[] = {{&S98893[31], 6}, {NULL}};
const Segment SGI245470[] = {{&S98903[44], 10}, {NULL}};
const Segment SGI245471[] = {{&S98903[44], 22}, {NULL}};
const Segment SGI245472[] = {{&S98903[30], 6}, {NULL}};
const Segment SGI245473[] = {{&S98903[115], 3}, {NULL}};
const Segment SGI245474[] = {{&S98903[80], 6}, {NULL}};
const Segment SGI245475[] = {{&S98903[140], 7}, {NULL}};
const Segment SGI245476[] = {{&S98904[34], 17}, {NULL}};
const Segment SGI245477[] = {{&S98904[27], 6}, {NULL}};
const Segment SGI245478[] = {{&S98904[104], 9}, {NULL}};
const Segment SGI245479[] = {{&S98904[77], 6}, {NULL}};
const Segment SGI245480[] = {{&S98905[44], 10}, {NULL}};
const Segment SGI245481[] = {{&S98905[44], 22}, {NULL}};
const Segment SGI245482[] = {{&S98905[30], 6}, {NULL}};
const Segment SGI245483[] = {{&S98905[113], 3}, {NULL}};
const Segment SGI245484[] = {{&S98905[80], 5}, {NULL}};
const Segment SGI245485[] = {{&S98905[117], 14}, {NULL}};
const Segment SGI245486[] = {{&S98910[44], 10}, {NULL}};
const Segment SGI245487[] = {{&S98910[44], 22}, {NULL}};
const Segment SGI245488[] = {{&S98910[30], 6}, {NULL}};
const Segment SGI245489[] = {{&S98910[131], 3}, {NULL}};
const Segment SGI245490[] = {{&S98910[96], 6}, {NULL}};
const Segment SGI245491[] = {{&S98910[68], 8}, {NULL}};
const Segment SGI245492[] = {{&S100339[34], 12}, {NULL}};
const Segment SGI245493[] = {{&S100339[27], 6}, {NULL}};
const Segment SGI245494[] = {{&S100339[99], 9}, {NULL}};
const Segment SGI245495[] = {{&S100339[72], 6}, {NULL}};
const Segment SGI245496[] = {{&S98912[59], 11}, {NULL}};
const Segment SGI245497[] = {{&S98912[59], 23}, {NULL}};
const Segment SGI245498[] = {{&S98912[45], 6}, {NULL}};
const Segment SGI245499[] = {{&S98912[131], 3}, {NULL}};
const Segment SGI245500[] = {{&S98912[96], 6}, {NULL}};
const Segment SGI245501[] = {{&S98912[0], 14}, {NULL}};
const Segment SGI245502[] = {{&S98919[44], 5}, {NULL}};
const Segment SGI245503[] = {{&S98919[44], 17}, {NULL}};
const Segment SGI245504[] = {{&S98919[30], 6}, {NULL}};
const Segment SGI245505[] = {{&S98919[110], 3}, {NULL}};
const Segment SGI245506[] = {{&S98919[75], 6}, {NULL}};
const Segment SGI245507[] = {{&S98920[44], 6}, {NULL}};
const Segment SGI245508[] = {{&S98920[44], 18}, {NULL}};
const Segment SGI245509[] = {{&S98920[30], 6}, {NULL}};
const Segment SGI245510[] = {{&S98920[111], 3}, {NULL}};
const Segment SGI245511[] = {{&S98920[76], 6}, {NULL}};
const Segment SGI245512[] = {{&S98920[136], 13}, {NULL}};
const Segment SGI245513[] = {{&S98921[34], 6}, {NULL}};
const Segment SGI245514[] = {{&S98921[27], 6}, {NULL}};
const Segment SGI245515[] = {{&S98921[94], 9}, {NULL}};
const Segment SGI245516[] = {{&S98921[67], 6}, {NULL}};
const Segment SGI245517[] = {{&S98922[44], 10}, {NULL}};
const Segment SGI245518[] = {{&S98922[44], 22}, {NULL}};
const Segment SGI245519[] = {{&S98922[30], 6}, {NULL}};
const Segment SGI245520[] = {{&S98922[80], 6}, {NULL}};
const Segment SGI245521[] = {{&S98922[107], 13}, {NULL}};
const Segment SGI245522[] = {{&S98941[34], 9}, {NULL}};
const Segment SGI245523[] = {{&S98941[27], 6}, {NULL}};
const Segment SGI245524[] = {{&S98941[96], 9}, {NULL}};
const Segment SGI245525[] = {{&S98941[69], 6}, {NULL}};
const Segment SGI245526[] = {{&S98941[137], 8}, {NULL}};
const Segment SGI245527[] = {{&S98942[34], 10}, {NULL}};
const Segment SGI245528[] = {{&S98942[27], 6}, {NULL}};
const Segment SGI245529[] = {{&S98942[97], 9}, {NULL}};
const Segment SGI245530[] = {{&S98942[70], 6}, {NULL}};
const Segment SGI245531[] = {{&S98942[138], 8}, {NULL}};
const Segment SGI245532[] = {{&S98948[44], 9}, {NULL}};
const Segment SGI245533[] = {{&S98948[44], 21}, {NULL}};
const Segment SGI245534[] = {{&S98948[30], 6}, {NULL}};
const Segment SGI245535[] = {{&S98948[79], 6}, {NULL}};
const Segment SGI245536[] = {{&S98948[106], 13}, {NULL}};
const Segment SGI245537[] = {{&S98952[44], 14}, {NULL}};
const Segment SGI245538[] = {{&S98952[44], 31}, {NULL}};
const Segment SGI245539[] = {{&S98952[30], 6}, {NULL}};
const Segment SGI245540[] = {{&S98952[89], 6}, {NULL}};
const Segment SGI245541[] = {{&S98952[116], 13}, {NULL}};
const Segment SGI245542[] = {{&S98956[34], 9}, {NULL}};
const Segment SGI245543[] = {{&S98956[27], 6}, {NULL}};
const Segment SGI245544[] = {{&S98956[96], 9}, {NULL}};
const Segment SGI245545[] = {{&S98956[69], 6}, {NULL}};
const Segment SGI245546[] = {{&S98963[44], 9}, {NULL}};
const Segment SGI245547[] = {{&S98963[44], 21}, {NULL}};
const Segment SGI245548[] = {{&S98963[30], 6}, {NULL}};
const Segment SGI245549[] = {{&S98963[79], 6}, {NULL}};
const Segment SGI245550[] = {{&S98963[106], 13}, {NULL}};
const Segment SGI245551[] = {{&S98977[44], 10}, {NULL}};
const Segment SGI245552[] = {{&S98977[44], 22}, {NULL}};
const Segment SGI245553[] = {{&S98977[30], 6}, {NULL}};
const Segment SGI245554[] = {{&S98977[115], 3}, {NULL}};
const Segment SGI245555[] = {{&S98977[80], 6}, {NULL}};
const Segment SGI245556[] = {{&S98977[135], 8}, {NULL}};
const Segment SGI245557[] = {{&S98977[144], 13}, {NULL}};
const Segment SGI245558[] = {{&S98986[34], 11}, {NULL}};
const Segment SGI245559[] = {{&S98986[27], 6}, {NULL}};
const Segment SGI245560[] = {{&S98986[98], 9}, {NULL}};
const Segment SGI245561[] = {{&S98986[71], 6}, {NULL}};
const Segment SGI245562[] = {{&S98989[34], 10}, {NULL}};
const Segment SGI245563[] = {{&S98989[27], 6}, {NULL}};
const Segment SGI245564[] = {{&S98989[97], 9}, {NULL}};
const Segment SGI245565[] = {{&S98989[70], 6}, {NULL}};
const Segment SGI245566[] = {{&S98989[138], 8}, {NULL}};
const Segment SGI245567[] = {{&S98990[44], 11}, {NULL}};
const Segment SGI245568[] = {{&S98990[44], 23}, {NULL}};
const Segment SGI245569[] = {{&S98990[30], 6}, {NULL}};
const Segment SGI245570[] = {{&S98990[81], 6}, {NULL}};
const Segment SGI245571[] = {{&S98990[108], 13}, {NULL}};
const Segment SGI245572[] = {{&S98991[34], 7}, {NULL}};
const Segment SGI245573[] = {{&S98991[27], 6}, {NULL}};
const Segment SGI245574[] = {{&S98991[107], 9}, {NULL}};
const Segment SGI245575[] = {{&S98991[80], 6}, {NULL}};
const Segment SGI245576[] = {{&S98991[148], 8}, {NULL}};
const Segment SGI245577[] = {{&S98994[34], 9}, {NULL}};
const Segment SGI245578[] = {{&S98994[27], 6}, {NULL}};
const Segment SGI245579[] = {{&S98994[96], 9}, {NULL}};
const Segment SGI245580[] = {{&S98994[69], 6}, {NULL}};
const Segment SGI245581[] = {{&S91835[34], 7}, {NULL}};
const Segment SGI245582[] = {{&S98997[34], 11}, {NULL}};
const Segment SGI245583[] = {{&S91835[27], 6}, {NULL}};
const Segment SGI245584[] = {{&S98997[27], 6}, {NULL}};
const Segment SGI245585[] = {{&S91835[94], 9}, {NULL}};
const Segment SGI245586[] = {{&S98997[98], 9}, {NULL}};
const Segment SGI245587[] = {{&S91835[67], 6}, {NULL}};
const Segment SGI245588[] = {{&S98997[71], 6}, {NULL}};
const Segment SGI245589[] = {{&S98999[34], 10}, {NULL}};
const Segment SGI245590[] = {{&S98999[27], 6}, {NULL}};
const Segment SGI245591[] = {{&S98999[97], 9}, {NULL}};
const Segment SGI245592[] = {{&S98999[70], 6}, {NULL}};
const Segment SGI245593[] = {{&S91850[34], 10}, {NULL}};
const Segment SGI245594[] = {{&S91850[27], 6}, {NULL}};
const Segment SGI245595[] = {{&S91850[97], 9}, {NULL}};
const Segment SGI245596[] = {{&S91850[70], 6}, {NULL}};
const Segment SGI245597[] = {{&S91856[34], 10}, {NULL}};
const Segment SGI245598[] = {{&S91856[27], 6}, {NULL}};
const Segment SGI245599[] = {{&S91856[97], 9}, {NULL}};
const Segment SGI245600[] = {{&S91856[70], 6}, {NULL}};
const Segment SGI245601[] = {{&S91856[138], 8}, {NULL}};
const Segment SGI245602[] = {{&S91857[44], 10}, {NULL}};
const Segment SGI245603[] = {{&S91857[44], 21}, {NULL}};
const Segment SGI245604[] = {{&S91857[30], 6}, {NULL}};
const Segment SGI245605[] = {{&S91857[79], 6}, {NULL}};
const Segment SGI245606[] = {{&S91857[106], 13}, {NULL}};
const Segment SGI245607[] = {{&S91861[34], 11}, {NULL}};
const Segment SGI245608[] = {{&S91861[27], 6}, {NULL}};
const Segment SGI245609[] = {{&S91861[97], 9}, {NULL}};
const Segment SGI245610[] = {{&S91861[70], 6}, {NULL}};
const Segment SGI245611[] = {{&S91862[34], 7}, {NULL}};
const Segment SGI245612[] = {{&S91862[27], 6}, {NULL}};
const Segment SGI245613[] = {{&S91862[100], 9}, {NULL}};
const Segment SGI245614[] = {{&S91862[73], 6}, {NULL}};
const Segment SGI245615[] = {{&S99137[44], 7}, {NULL}};
const Segment SGI245616[] = {{&S99137[44], 19}, {NULL}};
const Segment SGI245617[] = {{&S99137[30], 6}, {NULL}};
const Segment SGI245618[] = {{&S99137[128], 3}, {NULL}};
const Segment SGI245619[] = {{&S99137[93], 6}, {NULL}};
const Segment SGI245620[] = {{&S99137[65], 8}, {NULL}};
const Segment SGI245621[] = {{&S99162[34], 10}, {NULL}};
const Segment SGI245622[] = {{&S99162[27], 6}, {NULL}};
const Segment SGI245623[] = {{&S99162[95], 9}, {NULL}};
const Segment SGI245624[] = {{&S99162[68], 6}, {NULL}};
const Segment SGI245625[] = {{&S91993[34], 19}, {NULL}};
const Segment SGI245626[] = {{&S91993[27], 6}, {NULL}};
const Segment SGI245627[] = {{&S91993[106], 9}, {NULL}};
const Segment SGI245628[] = {{&S91993[79], 6}, {NULL}};
const Segment SGI245629[] = {{&S91995[34], 10}, {NULL}};
const Segment SGI245630[] = {{&S91995[27], 6}, {NULL}};
const Segment SGI245631[] = {{&S91995[97], 9}, {NULL}};
const Segment SGI245632[] = {{&S91995[70], 6}, {NULL}};
const Segment SGI245633[] = {{&S91996[44], 9}, {NULL}};
const Segment SGI245634[] = {{&S91996[44], 20}, {NULL}};
const Segment SGI245635[] = {{&S91996[30], 6}, {NULL}};
const Segment SGI245636[] = {{&S91996[113], 3}, {NULL}};
const Segment SGI245637[] = {{&S91996[78], 6}, {NULL}};
const Segment SGI245638[] = {{&S91996[131], 7}, {NULL}};
const Segment SGI245639[] = {{&S99181[34], 10}, {NULL}};
const Segment SGI245640[] = {{&S99181[27], 6}, {NULL}};
const Segment SGI245641[] = {{&S99181[95], 9}, {NULL}};
const Segment SGI245642[] = {{&S99181[68], 6}, {NULL}};
const Segment SGI245643[] = {{&S100545[44], 13}, {NULL}};
const Segment SGI245644[] = {{&S100545[44], 25}, {NULL}};
const Segment SGI245645[] = {{&S100545[30], 6}, {NULL}};
const Segment SGI245646[] = {{&S100545[117], 3}, {NULL}};
const Segment SGI245647[] = {{&S100545[83], 5}, {NULL}};
const Segment SGI245648[] = {{&S99193[45], 15}, {NULL}};
const Segment SGI245649[] = {{&S99193[30], 6}, {NULL}};
const Segment SGI245650[] = {{&S99193[132], 3}, {NULL}};
const Segment SGI245651[] = {{&S99193[98], 5}, {NULL}};
const Segment SGI245652[] = {{&S99193[156], 13}, {NULL}};
const Segment SGI245653[] = {{&S100556[34], 25}, {NULL}};
const Segment SGI245654[] = {{&S100556[27], 6}, {NULL}};
const Segment SGI245655[] = {{&S100556[112], 9}, {NULL}};
const Segment SGI245656[] = {{&S100556[85], 6}, {NULL}};
const Segment SGI245657[] = {{&S99199[44], 15}, {NULL}};
const Segment SGI245658[] = {{&S99199[44], 39}, {NULL}};
const Segment SGI245659[] = {{&S99199[30], 6}, {NULL}};
const Segment SGI245660[] = {{&S99199[97], 6}, {NULL}};
const Segment SGI245661[] = {{&S99199[124], 13}, {NULL}};
const Segment SGI245662[] = {{&S100561[44], 15}, {NULL}};
const Segment SGI245663[] = {{&S100561[44], 27}, {NULL}};
const Segment SGI245664[] = {{&S100561[30], 6}, {NULL}};
const Segment SGI245665[] = {{&S100561[120], 3}, {NULL}};
const Segment SGI245666[] = {{&S100561[85], 6}, {NULL}};
const Segment SGI245667[] = {{&S92032[44], 12}, {NULL}};
const Segment SGI245668[] = {{&S92032[44], 24}, {NULL}};
const Segment SGI245669[] = {{&S92032[30], 6}, {NULL}};
const Segment SGI245670[] = {{&S92032[117], 3}, {NULL}};
const Segment SGI245671[] = {{&S92032[82], 6}, {NULL}};
const Segment SGI245672[] = {{&S100569[34], 16}, {NULL}};
const Segment SGI245673[] = {{&S100569[27], 6}, {NULL}};
const Segment SGI245674[] = {{&S100569[103], 9}, {NULL}};
const Segment SGI245675[] = {{&S100569[76], 6}, {NULL}};
const Segment SGI245676[] = {{&S100570[34], 12}, {NULL}};
const Segment SGI245677[] = {{&S100570[27], 6}, {NULL}};
const Segment SGI245678[] = {{&S100570[99], 9}, {NULL}};
const Segment SGI245679[] = {{&S100570[72], 6}, {NULL}};
const Segment SGI245680[] = {{&S92040[34], 9}, {NULL}};
const Segment SGI245681[] = {{&S92040[27], 6}, {NULL}};
const Segment SGI245682[] = {{&S92040[95], 9}, {NULL}};
const Segment SGI245683[] = {{&S92040[68], 6}, {NULL}};
const Segment SGI245684[] = {{&S99216[44], 7}, {NULL}};
const Segment SGI245685[] = {{&S99216[44], 19}, {NULL}};
const Segment SGI245686[] = {{&S99216[30], 6}, {NULL}};
const Segment SGI245687[] = {{&S99216[128], 3}, {NULL}};
const Segment SGI245688[] = {{&S99216[93], 6}, {NULL}};
const Segment SGI245689[] = {{&S92041[44], 10}, {NULL}};
const Segment SGI245690[] = {{&S92041[44], 21}, {NULL}};
const Segment SGI245691[] = {{&S99216[65], 8}, {NULL}};
const Segment SGI245692[] = {{&S92041[30], 6}, {NULL}};
const Segment SGI245693[] = {{&S92041[112], 3}, {NULL}};
const Segment SGI245694[] = {{&S92041[79], 5}, {NULL}};
const Segment SGI245695[] = {{&S92042[44], 10}, {NULL}};
const Segment SGI245696[] = {{&S92042[44], 21}, {NULL}};
const Segment SGI245697[] = {{&S92042[30], 6}, {NULL}};
const Segment SGI245698[] = {{&S92042[112], 3}, {NULL}};
const Segment SGI245699[] = {{&S92042[79], 5}, {NULL}};
const Segment SGI245700[] = {{&S99220[34], 10}, {NULL}};
const Segment SGI245701[] = {{&S99220[27], 6}, {NULL}};
const Segment SGI245702[] = {{&S99220[97], 9}, {NULL}};
const Segment SGI245703[] = {{&S99220[70], 6}, {NULL}};
const Segment SGI245704[] = {{&S99233[44], 7}, {NULL}};
const Segment SGI245705[] = {{&S99233[44], 25}, {NULL}};
const Segment SGI245706[] = {{&S99233[30], 6}, {NULL}};
const Segment SGI245707[] = {{&S99233[118], 3}, {NULL}};
const Segment SGI245708[] = {{&S99233[83], 6}, {NULL}};
const Segment SGI245709[] = {{&S99233[143], 11}, {NULL}};
const Segment SGI245710[] = {{&S99237[34], 10}, {NULL}};
const Segment SGI245711[] = {{&S99237[27], 6}, {NULL}};
const Segment SGI245712[] = {{&S99237[97], 9}, {NULL}};
const Segment SGI245713[] = {{&S99237[70], 6}, {NULL}};
const Segment SGI245714[] = {{&S99246[44], 7}, {NULL}};
const Segment SGI245715[] = {{&S99246[44], 24}, {NULL}};
const Segment SGI245716[] = {{&S99246[30], 6}, {NULL}};
const Segment SGI245717[] = {{&S99246[115], 3}, {NULL}};
const Segment SGI245718[] = {{&S99246[82], 5}, {NULL}};
const Segment SGI245719[] = {{&S99250[34], 7}, {NULL}};
const Segment SGI245720[] = {{&S99250[27], 6}, {NULL}};
const Segment SGI245721[] = {{&S99250[94], 9}, {NULL}};
const Segment SGI245722[] = {{&S99250[67], 6}, {NULL}};
const Segment SGI245723[] = {{&S99251[44], 10}, {NULL}};
const Segment SGI245724[] = {{&S99251[44], 22}, {NULL}};
const Segment SGI245725[] = {{&S99251[30], 6}, {NULL}};
const Segment SGI245726[] = {{&S99251[80], 6}, {NULL}};
const Segment SGI245727[] = {{&S99251[107], 13}, {NULL}};
const Segment SGI245728[] = {{&S99256[34], 8}, {NULL}};
const Segment SGI245729[] = {{&S99256[27], 6}, {NULL}};
const Segment SGI245730[] = {{&S99256[95], 9}, {NULL}};
const Segment SGI245731[] = {{&S99256[68], 6}, {NULL}};
const Segment SGI245732[] = {{&S99266[44], 10}, {NULL}};
const Segment SGI245733[] = {{&S99266[44], 22}, {NULL}};
const Segment SGI245734[] = {{&S99266[30], 6}, {NULL}};
const Segment SGI245735[] = {{&S99266[115], 3}, {NULL}};
const Segment SGI245736[] = {{&S99266[80], 6}, {NULL}};
const Segment SGI245737[] = {{&S99266[140], 13}, {NULL}};
const Segment SGI245738[] = {{&S99268[44], 8}, {NULL}};
const Segment SGI245739[] = {{&S99268[44], 30}, {NULL}};
const Segment SGI245740[] = {{&S99268[30], 6}, {NULL}};
const Segment SGI245741[] = {{&S99268[123], 3}, {NULL}};
const Segment SGI245742[] = {{&S99268[88], 6}, {NULL}};
const Segment SGI245743[] = {{&S99268[147], 10}, {NULL}};
const Segment SGI245744[] = {{&S99276[34], 6}, {NULL}};
const Segment SGI245745[] = {{&S99276[27], 6}, {NULL}};
const Segment SGI245746[] = {{&S99276[93], 9}, {NULL}};
const Segment SGI245747[] = {{&S99276[66], 6}, {NULL}};
const Segment SGI245748[] = {{&S100657[44], 4}, {NULL}};
const Segment SGI245749[] = {{&S100657[44], 16}, {NULL}};
const Segment SGI245750[] = {{&S100657[30], 6}, {NULL}};
const Segment SGI245751[] = {{&S100657[74], 6}, {NULL}};
const Segment SGI245752[] = {{&S99292[59], 11}, {NULL}};
const Segment SGI245753[] = {{&S99292[59], 23}, {NULL}};
const Segment SGI245754[] = {{&S100657[111], 3}, {NULL}};
const Segment SGI245755[] = {{&S99292[45], 6}, {NULL}};
const Segment SGI245756[] = {{&S99292[131], 3}, {NULL}};
const Segment SGI245757[] = {{&S99292[96], 6}, {NULL}};
const Segment SGI245758[] = {{&S99292[0], 14}, {NULL}};
const Segment SGI245759[] = {{&S100688[34], 13}, {NULL}};
const Segment SGI245760[] = {{&S100688[27], 6}, {NULL}};
const Segment SGI245761[] = {{&S100688[100], 9}, {NULL}};
const Segment SGI245762[] = {{&S100688[73], 6}, {NULL}};
const Segment SGI245763[] = {{&S100700[34], 17}, {NULL}};
const Segment SGI245764[] = {{&S100700[27], 6}, {NULL}};
const Segment SGI245765[] = {{&S100700[104], 9}, {NULL}};
const Segment SGI245766[] = {{&S100700[77], 6}, {NULL}};
const Segment SGI245767[] = {{&S99352[44], 10}, {NULL}};
const Segment SGI245768[] = {{&S99352[44], 27}, {NULL}};
const Segment SGI245769[] = {{&S99352[30], 6}, {NULL}};
const Segment SGI245770[] = {{&S99352[119], 3}, {NULL}};
const Segment SGI245771[] = {{&S99352[85], 5}, {NULL}};
const Segment SGI245772[] = {{&S92190[34], 9}, {NULL}};
const Segment SGI245773[] = {{&S92190[27], 6}, {NULL}};
const Segment SGI245774[] = {{&S92190[96], 9}, {NULL}};
const Segment SGI245775[] = {{&S92190[69], 6}, {NULL}};
const Segment SGI245776[] = {{&S92209[34], 15}, {NULL}};
const Segment SGI245777[] = {{&S92209[27], 6}, {NULL}};
const Segment SGI245778[] = {{&S92209[108], 9}, {NULL}};
const Segment SGI245779[] = {{&S92209[81], 6}, {NULL}};
const Segment SGI245780[] = {{&S99384[34], 12}, {NULL}};
const Segment SGI245781[] = {{&S99384[27], 6}, {NULL}};
const Segment SGI245782[] = {{&S99384[99], 9}, {NULL}};
const Segment SGI245783[] = {{&S99384[72], 6}, {NULL}};
const Segment SGI245784[] = {{&S92247[34], 19}, {NULL}};
const Segment SGI245785[] = {{&S92247[27], 6}, {NULL}};
const Segment SGI245786[] = {{&S92247[105], 9}, {NULL}};
const Segment SGI245787[] = {{&S92247[78], 6}, {NULL}};
const Segment SGI245788[] = {{&S92248[34], 10}, {NULL}};
const Segment SGI245789[] = {{&S92248[27], 6}, {NULL}};
const Segment SGI245790[] = {{&S92248[96], 9}, {NULL}};
const Segment SGI245791[] = {{&S92248[69], 6}, {NULL}};
const Segment SGI245792[] = {{&S92255[52], 10}, {NULL}};
const Segment SGI245793[] = {{&S92255[52], 21}, {NULL}};
const Segment SGI245794[] = {{&S92255[38], 6}, {NULL}};
const Segment SGI245795[] = {{&S92255[87], 5}, {NULL}};
const Segment SGI245796[] = {{&S92263[44], 10}, {NULL}};
const Segment SGI245797[] = {{&S92263[44], 22}, {NULL}};
const Segment SGI245798[] = {{&S92263[30], 6}, {NULL}};
const Segment SGI245799[] = {{&S92263[115], 3}, {NULL}};
const Segment SGI245800[] = {{&S92263[80], 6}, {NULL}};
const Segment SGI245801[] = {{&S92263[140], 8}, {NULL}};
const Segment SGI245802[] = {{&S92270[34], 9}, {NULL}};
const Segment SGI245803[] = {{&S92270[27], 6}, {NULL}};
const Segment SGI245804[] = {{&S92270[95], 9}, {NULL}};
const Segment SGI245805[] = {{&S92270[68], 6}, {NULL}};
const Segment SGI245806[] = {{&S100863[34], 10}, {NULL}};
const Segment SGI245807[] = {{&S100863[27], 6}, {NULL}};
const Segment SGI245808[] = {{&S100863[97], 9}, {NULL}};
const Segment SGI245809[] = {{&S100863[70], 6}, {NULL}};
const Segment SGI245810[] = {{&S99396[44], 10}, {NULL}};
const Segment SGI245811[] = {{&S99396[44], 22}, {NULL}};
const Segment SGI245812[] = {{&S99396[30], 6}, {NULL}};
const Segment SGI245813[] = {{&S99396[80], 6}, {NULL}};
const Segment SGI245814[] = {{&S99396[107], 13}, {NULL}};
const Segment SGI245815[] = {{&S99400[44], 9}, {NULL}};
const Segment SGI245816[] = {{&S99400[44], 21}, {NULL}};
const Segment SGI245817[] = {{&S99400[30], 6}, {NULL}};
const Segment SGI245818[] = {{&S99400[79], 6}, {NULL}};
const Segment SGI245819[] = {{&S99400[106], 13}, {NULL}};
const Segment SGI245820[] = {{&S99409[44], 9}, {NULL}};
const Segment SGI245821[] = {{&S99409[44], 21}, {NULL}};
const Segment SGI245822[] = {{&S99409[30], 6}, {NULL}};
const Segment SGI245823[] = {{&S99409[130], 3}, {NULL}};
const Segment SGI245824[] = {{&S99409[95], 6}, {NULL}};
const Segment SGI245825[] = {{&S99409[67], 8}, {NULL}};
const Segment SGI245826[] = {{&S99410[44], 10}, {NULL}};
const Segment SGI245827[] = {{&S99410[44], 22}, {NULL}};
const Segment SGI245828[] = {{&S99410[30], 6}, {NULL}};
const Segment SGI245829[] = {{&S99410[131], 3}, {NULL}};
const Segment SGI245830[] = {{&S99410[96], 6}, {NULL}};
const Segment SGI245831[] = {{&S99410[68], 8}, {NULL}};
const Segment SGI245832[] = {{&S99419[34], 9}, {NULL}};
const Segment SGI245833[] = {{&S99419[27], 6}, {NULL}};
const Segment SGI245834[] = {{&S99419[96], 9}, {NULL}};
const Segment SGI245835[] = {{&S99419[69], 6}, {NULL}};
const Segment SGI245836[] = {{&S99430[59], 10}, {NULL}};
const Segment SGI245837[] = {{&S99430[59], 22}, {NULL}};
const Segment SGI245838[] = {{&S99430[45], 6}, {NULL}};
const Segment SGI245839[] = {{&S99430[130], 3}, {NULL}};
const Segment SGI245840[] = {{&S99430[95], 6}, {NULL}};
const Segment SGI245841[] = {{&S99430[0], 14}, {NULL}};
const Segment SGI245842[] = {{&S92370[44], 10}, {NULL}};
const Segment SGI245843[] = {{&S92370[44], 21}, {NULL}};
const Segment SGI245844[] = {{&S92370[30], 6}, {NULL}};
const Segment SGI245845[] = {{&S92370[114], 3}, {NULL}};
const Segment SGI245846[] = {{&S92370[79], 6}, {NULL}};
const Segment SGI245847[] = {{&S99433[34], 9}, {NULL}};
const Segment SGI245848[] = {{&S99433[27], 6}, {NULL}};
const Segment SGI245849[] = {{&S99433[96], 9}, {NULL}};
const Segment SGI245850[] = {{&S99433[69], 6}, {NULL}};
const Segment SGI245851[] = {{&S99443[52], 9}, {NULL}};
const Segment SGI245852[] = {{&S99443[52], 26}, {NULL}};
const Segment SGI245853[] = {{&S99443[38], 6}, {NULL}};
const Segment SGI245854[] = {{&S99443[92], 5}, {NULL}};
const Segment SGI245855[] = {{&S99443[11], 3}, {NULL}};
const Segment SGI245856[] = {{&S92383[44], 12}, {NULL}};
const Segment SGI245857[] = {{&S92383[44], 23}, {NULL}};
const Segment SGI245858[] = {{&S92383[30], 6}, {NULL}};
const Segment SGI245859[] = {{&S92383[116], 3}, {NULL}};
const Segment SGI245860[] = {{&S92383[81], 6}, {NULL}};
const Segment SGI245861[] = {{&S100936[34], 7}, {NULL}};
const Segment SGI245862[] = {{&S100936[27], 6}, {NULL}};
const Segment SGI245863[] = {{&S100936[94], 9}, {NULL}};
const Segment SGI245864[] = {{&S100936[67], 6}, {NULL}};
const Segment SGI245865[] = {{&S99450[52], 11}, {NULL}};
const Segment SGI245866[] = {{&S99450[52], 23}, {NULL}};
const Segment SGI245867[] = {{&S99450[38], 6}, {NULL}};
const Segment SGI245868[] = {{&S99450[89], 5}, {NULL}};
const Segment SGI245869[] = {{&S99450[11], 3}, {NULL}};
const Segment SGI245870[] = {{&S99451[44], 10}, {NULL}};
const Segment SGI245871[] = {{&S99451[44], 22}, {NULL}};
const Segment SGI245872[] = {{&S99451[30], 6}, {NULL}};
const Segment SGI245873[] = {{&S99451[80], 6}, {NULL}};
const Segment SGI245874[] = {{&S99451[107], 13}, {NULL}};
const Segment SGI245875[] = {{&S99453[45], 16}, {NULL}};
const Segment SGI245876[] = {{&S99453[30], 6}, {NULL}};
const Segment SGI245877[] = {{&S99453[121], 3}, {NULL}};
const Segment SGI245878[] = {{&S99453[87], 5}, {NULL}};
const Segment SGI245879[] = {{&S99453[145], 13}, {NULL}};
const Segment SGI245880[] = {{&S99455[44], 6}, {NULL}};
const Segment SGI245881[] = {{&S99455[44], 18}, {NULL}};
const Segment SGI245882[] = {{&S99455[30], 6}, {NULL}};
const Segment SGI245883[] = {{&S99455[111], 3}, {NULL}};
const Segment SGI245884[] = {{&S99455[76], 6}, {NULL}};
const Segment SGI245885[] = {{&S99455[131], 8}, {NULL}};
const Segment SGI245886[] = {{&S99455[140], 13}, {NULL}};
const Segment SGI245887[] = {{&S99471[34], 10}, {NULL}};
const Segment SGI245888[] = {{&S99471[27], 6}, {NULL}};
const Segment SGI245889[] = {{&S99471[97], 9}, {NULL}};
const Segment SGI245890[] = {{&S99471[70], 6}, {NULL}};
const Segment SGI245891[] = {{&S99475[34], 10}, {NULL}};
const Segment SGI245892[] = {{&S99475[27], 6}, {NULL}};
const Segment SGI245893[] = {{&S99475[97], 9}, {NULL}};
const Segment SGI245894[] = {{&S99475[70], 6}, {NULL}};
const Segment SGI245895[] = {{&S99477[44], 10}, {NULL}};
const Segment SGI245896[] = {{&S99477[44], 22}, {NULL}};
const Segment SGI245897[] = {{&S99477[30], 6}, {NULL}};
const Segment SGI245898[] = {{&S99477[115], 3}, {NULL}};
const Segment SGI245899[] = {{&S99477[80], 6}, {NULL}};
const Segment SGI245900[] = {{&S99477[135], 8}, {NULL}};
const Segment SGI245901[] = {{&S99477[144], 13}, {NULL}};
const Segment SGI245902[] = {{&S92429[34], 10}, {NULL}};
const Segment SGI245903[] = {{&S92429[27], 6}, {NULL}};
const Segment SGI245904[] = {{&S92429[97], 9}, {NULL}};
const Segment SGI245905[] = {{&S92429[70], 6}, {NULL}};
const Segment SGI245906[] = {{&S92429[138], 8}, {NULL}};
const Segment SGI245907[] = {{&S92433[34], 7}, {NULL}};
const Segment SGI245908[] = {{&S92433[27], 6}, {NULL}};
const Segment SGI245909[] = {{&S92433[93], 9}, {NULL}};
const Segment SGI245910[] = {{&S92433[66], 6}, {NULL}};
const Segment SGI245911[] = {{&S92437[44], 26}, {NULL}};
const Segment SGI245912[] = {{&S99497[44], 7}, {NULL}};
const Segment SGI245913[] = {{&S92437[44], 38}, {NULL}};
const Segment SGI245914[] = {{&S99497[44], 24}, {NULL}};
const Segment SGI245915[] = {{&S92437[30], 6}, {NULL}};
const Segment SGI245916[] = {{&S99497[30], 6}, {NULL}};
const Segment SGI245917[] = {{&S92437[149], 3}, {NULL}};
const Segment SGI245918[] = {{&S92437[114], 6}, {NULL}};
const Segment SGI245919[] = {{&S99497[82], 6}, {NULL}};
const Segment SGI245920[] = {{&S92437[84], 8}, {NULL}};
const Segment SGI245921[] = {{&S99497[109], 13}, {NULL}};
const Segment SGI245922[] = {{&S92438[44], 6}, {NULL}};
const Segment SGI245923[] = {{&S92438[44], 18}, {NULL}};
const Segment SGI245924[] = {{&S92438[30], 6}, {NULL}};
const Segment SGI245925[] = {{&S92438[111], 3}, {NULL}};
const Segment SGI245926[] = {{&S92438[76], 6}, {NULL}};
const Segment SGI245927[] = {{&S92438[129], 8}, {NULL}};
const Segment SGI245928[] = {{&S99506[44], 10}, {NULL}};
const Segment SGI245929[] = {{&S99506[44], 22}, {NULL}};
const Segment SGI245930[] = {{&S99506[30], 6}, {NULL}};
const Segment SGI245931[] = {{&S99506[80], 6}, {NULL}};
const Segment SGI245932[] = {{&S99506[107], 13}, {NULL}};
const Segment SGI245933[] = {{&S99508[34], 11}, {NULL}};
const Segment SGI245934[] = {{&S99508[27], 6}, {NULL}};
const Segment SGI245935[] = {{&S99508[98], 9}, {NULL}};
const Segment SGI245936[] = {{&S99508[71], 6}, {NULL}};
const Segment SGI245937[] = {{&S99509[44], 9}, {NULL}};
const Segment SGI245938[] = {{&S99509[44], 26}, {NULL}};
const Segment SGI245939[] = {{&S99509[30], 6}, {NULL}};
const Segment SGI245940[] = {{&S99509[84], 6}, {NULL}};
const Segment SGI245941[] = {{&S99509[111], 13}, {NULL}};
const Segment SGI245942[] = {{&S99513[34], 10}, {NULL}};
const Segment SGI245943[] = {{&S99513[27], 6}, {NULL}};
const Segment SGI245944[] = {{&S99513[97], 9}, {NULL}};
const Segment SGI245945[] = {{&S99513[70], 6}, {NULL}};
const Segment SGI245946[] = {{&S99515[34], 16}, {NULL}};
const Segment SGI245947[] = {{&S99515[27], 6}, {NULL}};
const Segment SGI245948[] = {{&S99515[100], 9}, {NULL}};
const Segment SGI245949[] = {{&S99515[73], 6}, {NULL}};
const Segment SGI245950[] = {{&S99520[34], 9}, {NULL}};
const Segment SGI245951[] = {{&S99520[27], 6}, {NULL}};
const Segment SGI245952[] = {{&S99520[96], 9}, {NULL}};
const Segment SGI245953[] = {{&S99520[69], 6}, {NULL}};
const Segment SGI245954[] = {{&S99523[34], 7}, {NULL}};
const Segment SGI245955[] = {{&S99523[27], 6}, {NULL}};
const Segment SGI245956[] = {{&S99523[100], 9}, {NULL}};
const Segment SGI245957[] = {{&S99523[73], 6}, {NULL}};
const Segment SGI245958[] = {{&S4255[13], 4}, {&S4255[13], 4}, {&S4255[13], 4}, {NULL}};
const Segment SGI245959[] = {{&S4255[36], 3}, {NULL}};
const Segment SGI245960[] = {{&S4256[13], 4}, {NULL}};
const Segment SGI245961[] = {{&S4256[26], 3}, {NULL}};
const Segment SGI245962[] = {{&S4256[57], 6}, {NULL}};
const Segment SGI245963[] = {{&S4256[91], 8}, {NULL}};
const Segment SGI245964[] = {{&S4257[13], 4}, {NULL}};
const Segment SGI245965[] = {{&S4257[26], 3}, {NULL}};
const Segment SGI245966[] = {{&S4257[57], 6}, {NULL}};
const Segment SGI245967[] = {{&S4257[91], 8}, {NULL}};
const Segment SGI245968[] = {{&S4258[13], 4}, {NULL}};
const Segment SGI245969[] = {{&S4258[26], 3}, {NULL}};
const Segment SGI245970[] = {{&S4258[57], 6}, {NULL}};
const Segment SGI245971[] = {{&S4259[13], 4}, {NULL}};
const Segment SGI245972[] = {{&S4259[26], 3}, {NULL}};
const Segment SGI245973[] = {{&S4259[57], 6}, {NULL}};
const Segment SGI245974[] = {{&S4259[91], 8}, {NULL}};
const Segment SGI245975[] = {{&S4260[13], 4}, {NULL}};
const Segment SGI245976[] = {{&S4260[26], 3}, {NULL}};
const Segment SGI245977[] = {{&S4260[57], 6}, {NULL}};
const Segment SGI245978[] = {{&S4260[91], 6}, {NULL}};
const Segment SGI245979[] = {{&S4261[13], 4}, {&S4261[13], 4}, {&S4261[13], 4}, {NULL}};
const Segment SGI245980[] = {{&S4261[26], 3}, {NULL}};
const Segment SGI245981[] = {{&S4261[57], 6}, {NULL}};
const Segment SGI245982[] = {{&S4261[91], 6}, {NULL}};
const Segment SGI245983[] = {{&S4262[13], 4}, {&S4262[13], 4}, {NULL}};
const Segment SGI245984[] = {{&S4262[26], 3}, {NULL}};
const Segment SGI245985[] = {{&S4262[57], 6}, {NULL}};
const Segment SGI245986[] = {{&S4262[91], 6}, {NULL}};
const Segment SGI245987[] = {{&S4263[13], 4}, {NULL}};
const Segment SGI245988[] = {{&S4263[26], 3}, {NULL}};
const Segment SGI245989[] = {{&S4263[57], 6}, {NULL}};
const Segment SGI245990[] = {{&S4263[91], 6}, {NULL}};
const Segment SGI245991[] = {{&S4263[105], 7}, {NULL}};
const Segment SGI245992[] = {{&S4264[13], 4}, {NULL}};
const Segment SGI245993[] = {{&S4264[26], 3}, {NULL}};
const Segment SGI245994[] = {{&S4264[57], 6}, {NULL}};
const Segment SGI245995[] = {{&S4264[91], 6}, {NULL}};
const Segment SGI245996[] = {{&S4264[105], 7}, {NULL}};
const Segment SGI245997[] = {{&S4265[13], 4}, {&S4265[13], 4}, {NULL}};
const Segment SGI245998[] = {{&S4265[37], 3}, {NULL}};
const Segment SGI245999[] = {{&S4265[118], 4}, {NULL}};
const Segment SGI246000[] = {{&S4266[13], 4}, {&S4266[13], 4}, {NULL}};
const Segment SGI246001[] = {{&S4266[36], 3}, {NULL}};
const Segment SGI246002[] = {{&S4266[117], 5}, {NULL}};
const Segment SGI246003[] = {{&S4267[13], 4}, {NULL}};
const Segment SGI246004[] = {{&S4267[26], 3}, {NULL}};
const Segment SGI246005[] = {{&S4267[57], 6}, {NULL}};
const Segment SGI246006[] = {{&S4267[91], 6}, {NULL}};
const Segment SGI246007[] = {{&S4268[13], 4}, {NULL}};
const Segment SGI246008[] = {{&S4268[26], 5}, {NULL}};
const Segment SGI246009[] = {{&S4268[59], 6}, {NULL}};
const Segment SGI246010[] = {{&S4269[13], 4}, {NULL}};
const Segment SGI246011[] = {{&S4269[26], 3}, {NULL}};
const Segment SGI246012[] = {{&S4269[57], 6}, {NULL}};
const Segment SGI246013[] = {{&S4269[91], 6}, {NULL}};
const Segment SGI246014[] = {{&S4269[105], 9}, {NULL}};
const Segment SGI246015[] = {{&S4270[13], 4}, {NULL}};
const Segment SGI246016[] = {{&S4270[26], 5}, {NULL}};
const Segment SGI246017[] = {{&S4270[59], 6}, {NULL}};
const Segment SGI246018[] = {{&S4270[93], 6}, {NULL}};
const Segment SGI246019[] = {{&S4271[13], 4}, {NULL}};
const Segment SGI246020[] = {{&S4271[26], 3}, {NULL}};
const Segment SGI246021[] = {{&S4271[57], 6}, {NULL}};
const Segment SGI246022[] = {{&S4271[91], 6}, {NULL}};
const Segment SGI246023[] = {{&S92555[34], 12}, {NULL}};
const Segment SGI246024[] = {{&S92555[27], 6}, {NULL}};
const Segment SGI246025[] = {{&S92555[98], 9}, {NULL}};
const Segment SGI246026[] = {{&S92555[71], 6}, {NULL}};
const Segment SGI246027[] = {{&S4272[13], 4}, {NULL}};
const Segment SGI246028[] = {{&S4272[26], 3}, {NULL}};
const Segment SGI246029[] = {{&S4272[57], 6}, {NULL}};
const Segment SGI246030[] = {{&S4272[91], 6}, {NULL}};
const Segment SGI246031[] = {{&S4272[105], 9}, {&S4272[105], 9}, {NULL}};
const Segment SGI246032[] = {{&S4273[13], 4}, {&S4273[13], 4}, {&S4273[13], 4}, {NULL}};
const Segment SGI246033[] = {{&S4273[36], 5}, {NULL}};
const Segment SGI246034[] = {{&S4274[27], 4}, {NULL}};
const Segment SGI246035[] = {{&S4274[40], 3}, {NULL}};
const Segment SGI246036[] = {{&S4274[71], 6}, {NULL}};
const Segment SGI246037[] = {{&S4274[105], 6}, {NULL}};
const Segment SGI246038[] = {{&S4274[0], 14}, {NULL}};
const Segment SGI246039[] = {{&S4275[13], 4}, {NULL}};
const Segment SGI246040[] = {{&S4275[26], 5}, {NULL}};
const Segment SGI246041[] = {{&S4275[59], 6}, {NULL}};
const Segment SGI246042[] = {{&S4275[93], 6}, {NULL}};
const Segment SGI246043[] = {{&S4275[107], 9}, {NULL}};
const Segment SGI246044[] = {{&S4276[13], 4}, {NULL}};
const Segment SGI246045[] = {{&S4276[26], 5}, {NULL}};
const Segment SGI246046[] = {{&S4276[59], 6}, {NULL}};
const Segment SGI246047[] = {{&S4276[93], 6}, {NULL}};
const Segment SGI246048[] = {{&S4276[107], 9}, {&S4276[107], 9}, {NULL}};
const Segment SGI246049[] = {{&S4277[13], 4}, {NULL}};
const Segment SGI246050[] = {{&S4277[26], 5}, {NULL}};
const Segment SGI246051[] = {{&S4277[59], 6}, {NULL}};
const Segment SGI246052[] = {{&S4277[93], 7}, {NULL}};
const Segment SGI246053[] = {{&S4278[13], 4}, {NULL}};
const Segment SGI246054[] = {{&S4278[26], 5}, {NULL}};
const Segment SGI246055[] = {{&S4278[59], 6}, {NULL}};
const Segment SGI246056[] = {{&S4278[93], 6}, {&S4278[30], 1}, {NULL}};
const Segment SGI246057[] = {{&S4279[27], 4}, {NULL}};
const Segment SGI246058[] = {{&S4279[43], 3}, {NULL}};
const Segment SGI246059[] = {{&S4280[13], 4}, {NULL}};
const Segment SGI246060[] = {{&S4280[26], 5}, {NULL}};
const Segment SGI246061[] = {{&S4280[59], 6}, {NULL}};
const Segment SGI246062[] = {{&S4280[93], 5}, {NULL}};
const Segment SGI246063[] = {{&S4281[13], 4}, {NULL}};
const Segment SGI246064[] = {{&S4281[26], 5}, {NULL}};
const Segment SGI246065[] = {{&S4281[59], 6}, {NULL}};
const Segment SGI246066[] = {{&S4281[93], 6}, {NULL}};
const Segment SGI246067[] = {{&S4281[107], 7}, {NULL}};
const Segment SGI246068[] = {{&S92571[34], 7}, {NULL}};
const Segment SGI246069[] = {{&S92571[27], 6}, {NULL}};
const Segment SGI246070[] = {{&S92571[93], 9}, {NULL}};
const Segment SGI246071[] = {{&S4282[13], 4}, {NULL}};
const Segment SGI246072[] = {{&S92571[66], 6}, {NULL}};
const Segment SGI246073[] = {{&S4282[26], 5}, {NULL}};
const Segment SGI246074[] = {{&S4282[59], 6}, {NULL}};
const Segment SGI246075[] = {{&S4282[93], 6}, {NULL}};
const Segment SGI246076[] = {{&S4282[107], 6}, {NULL}};
const Segment SGI246077[] = {{&S4283[13], 4}, {NULL}};
const Segment SGI246078[] = {{&S4283[26], 5}, {NULL}};
const Segment SGI246079[] = {{&S4283[59], 6}, {NULL}};
const Segment SGI246080[] = {{&S4283[93], 6}, {NULL}};
const Segment SGI246081[] = {{&S4283[107], 9}, {&S4283[107], 9}, {&S4283[107], 9}, {NULL}};
const Segment SGI246082[] = {{&S4284[13], 4}, {&S4284[13], 4}, {NULL}};
const Segment SGI246083[] = {{&S4284[26], 3}, {NULL}};
const Segment SGI246084[] = {{&S4284[57], 6}, {NULL}};
const Segment SGI246085[] = {{&S4284[91], 6}, {NULL}};
const Segment SGI246086[] = {{&S4285[13], 4}, {NULL}};
const Segment SGI246087[] = {{&S4285[26], 3}, {NULL}};
const Segment SGI246088[] = {{&S4285[57], 6}, {NULL}};
const Segment SGI246089[] = {{&S4285[91], 6}, {NULL}};
const Segment SGI246090[] = {{&S4285[105], 7}, {NULL}};
const Segment SGI246091[] = {{&S4286[13], 4}, {&S4286[13], 4}, {NULL}};
const Segment SGI246092[] = {{&S4286[26], 5}, {NULL}};
const Segment SGI246093[] = {{&S4286[59], 6}, {NULL}};
const Segment SGI246094[] = {{&S4286[93], 6}, {NULL}};
const Segment SGI246095[] = {{&S4287[13], 4}, {NULL}};
const Segment SGI246096[] = {{&S4287[26], 3}, {NULL}};
const Segment SGI246097[] = {{&S4287[57], 6}, {NULL}};
const Segment SGI246098[] = {{&S4287[91], 5}, {NULL}};
const Segment SGI246099[] = {{&S4288[13], 4}, {NULL}};
const Segment SGI246100[] = {{&S4288[26], 5}, {NULL}};
const Segment SGI246101[] = {{&S4288[59], 6}, {NULL}};
const Segment SGI246102[] = {{&S4288[93], 6}, {NULL}};
const Segment SGI246103[] = {{&S4288[107], 9}, {NULL}};
const Segment SGI246104[] = {{&S4289[13], 4}, {NULL}};
const Segment SGI246105[] = {{&S4289[26], 3}, {NULL}};
const Segment SGI246106[] = {{&S4289[57], 6}, {NULL}};
const Segment SGI246107[] = {{&S4289[91], 6}, {NULL}};
const Segment SGI246108[] = {{&S4289[105], 9}, {NULL}};
const Segment SGI246109[] = {{&S92585[34], 9}, {NULL}};
const Segment SGI246110[] = {{&S92585[27], 6}, {NULL}};
const Segment SGI246111[] = {{&S4290[26], 4}, {NULL}};
const Segment SGI246112[] = {{&S4290[42], 5}, {NULL}};
const Segment SGI246113[] = {{&S92585[96], 9}, {NULL}};
const Segment SGI246114[] = {{&S92585[69], 6}, {NULL}};
const Segment SGI246115[] = {{&S4291[13], 4}, {&S4291[13], 4}, {NULL}};
const Segment SGI246116[] = {{&S4291[26], 5}, {NULL}};
const Segment SGI246117[] = {{&S4291[59], 6}, {NULL}};
const Segment SGI246118[] = {{&S4291[93], 7}, {NULL}};
const Segment SGI246119[] = {{&S4292[13], 4}, {NULL}};
const Segment SGI246120[] = {{&S4292[26], 5}, {NULL}};
const Segment SGI246121[] = {{&S4292[59], 6}, {NULL}};
const Segment SGI246122[] = {{&S4293[13], 4}, {&S4293[13], 4}, {NULL}};
const Segment SGI246123[] = {{&S4293[26], 5}, {NULL}};
const Segment SGI246124[] = {{&S4293[59], 6}, {NULL}};
const Segment SGI246125[] = {{&S4293[93], 6}, {NULL}};
const Segment SGI246126[] = {{&S4294[13], 4}, {&S4294[13], 4}, {&S4294[13], 4}, {NULL}};
const Segment SGI246127[] = {{&S4294[26], 5}, {NULL}};
const Segment SGI246128[] = {{&S4294[59], 6}, {NULL}};
const Segment SGI246129[] = {{&S4294[93], 6}, {NULL}};
const Segment SGI246130[] = {{&S4295[13], 4}, {&S4295[13], 4}, {&S4295[13], 4}, {NULL}};
const Segment SGI246131[] = {{&S4295[26], 3}, {NULL}};
const Segment SGI246132[] = {{&S4295[57], 6}, {NULL}};
const Segment SGI246133[] = {{&S4295[91], 6}, {NULL}};
const Segment SGI246134[] = {{&S4296[13], 4}, {&S4296[13], 4}, {&S4296[13], 4}, {NULL}};
const Segment SGI246135[] = {{&S4296[26], 3}, {NULL}};
const Segment SGI246136[] = {{&S4296[57], 6}, {NULL}};
const Segment SGI246137[] = {{&S4296[91], 6}, {NULL}};
const Segment SGI246138[] = {{&S4297[13], 4}, {&S4297[13], 4}, {&S4297[13], 4}, {NULL}};
const Segment SGI246139[] = {{&S4297[26], 5}, {NULL}};
const Segment SGI246140[] = {{&S4297[59], 6}, {NULL}};
const Segment SGI246141[] = {{&S4297[93], 7}, {NULL}};
const Segment SGI246142[] = {{&S4298[13], 4}, {&S4298[13], 4}, {&S4298[13], 4}, {NULL}};
const Segment SGI246143[] = {{&S4298[26], 5}, {NULL}};
const Segment SGI246144[] = {{&S4298[59], 6}, {NULL}};
const Segment SGI246145[] = {{&S4298[93], 7}, {NULL}};
const Segment SGI246146[] = {{&S4299[13], 4}, {NULL}};
const Segment SGI246147[] = {{&S4299[26], 5}, {NULL}};
const Segment SGI246148[] = {{&S4299[59], 6}, {NULL}};
const Segment SGI246149[] = {{&S4299[93], 6}, {NULL}};
const Segment SGI246150[] = {{&S4300[13], 4}, {&S4300[13], 4}, {&S4300[13], 4}, {NULL}};
const Segment SGI246151[] = {{&S4300[26], 5}, {NULL}};
const Segment SGI246152[] = {{&S4300[59], 6}, {NULL}};
const Segment SGI246153[] = {{&S4300[93], 6}, {NULL}};
const Segment SGI246154[] = {{&S4301[13], 4}, {NULL}};
const Segment SGI246155[] = {{&S4301[26], 5}, {NULL}};
const Segment SGI246156[] = {{&S4301[59], 6}, {NULL}};
const Segment SGI246157[] = {{&S4301[93], 7}, {NULL}};
const Segment SGI246158[] = {{&S4302[27], 4}, {NULL}};
const Segment SGI246159[] = {{&S4302[40], 5}, {NULL}};
const Segment SGI246160[] = {{&S4302[73], 6}, {NULL}};
const Segment SGI246161[] = {{&S4302[107], 6}, {NULL}};
const Segment SGI246162[] = {{&S4302[0], 14}, {NULL}};
const Segment SGI246163[] = {{&S92608[34], 7}, {NULL}};
const Segment SGI246164[] = {{&S92608[27], 6}, {NULL}};
const Segment SGI246165[] = {{&S92608[95], 9}, {NULL}};
const Segment SGI246166[] = {{&S92608[68], 6}, {NULL}};
const Segment SGI246167[] = {{&S4303[13], 4}, {&S4303[13], 4}, {NULL}};
const Segment SGI246168[] = {{&S4303[26], 5}, {NULL}};
const Segment SGI246169[] = {{&S4303[59], 6}, {NULL}};
const Segment SGI246170[] = {{&S4303[93], 7}, {NULL}};
const Segment SGI246171[] = {{&S4304[13], 4}, {&S4304[13], 4}, {NULL}};
const Segment SGI246172[] = {{&S4304[38], 5}, {NULL}};
const Segment SGI246173[] = {{&S4304[121], 5}, {NULL}};
const Segment SGI246174[] = {{&S4305[13], 4}, {&S4305[13], 4}, {NULL}};
const Segment SGI246175[] = {{&S4305[38], 5}, {NULL}};
const Segment SGI246176[] = {{&S4305[121], 4}, {NULL}};
const Segment SGI246177[] = {{&S4306[13], 4}, {&S4306[13], 4}, {NULL}};
const Segment SGI246178[] = {{&S4306[38], 5}, {NULL}};
const Segment SGI246179[] = {{&S4306[121], 6}, {NULL}};
const Segment SGI246180[] = {{&S4307[13], 4}, {&S4307[13], 4}, {NULL}};
const Segment SGI246181[] = {{&S4307[38], 5}, {NULL}};
const Segment SGI246182[] = {{&S4307[121], 6}, {NULL}};
const Segment SGI246183[] = {{&S4308[13], 4}, {NULL}};
const Segment SGI246184[] = {{&S4308[26], 3}, {NULL}};
const Segment SGI246185[] = {{&S4308[57], 6}, {NULL}};
const Segment SGI246186[] = {{&S4308[91], 6}, {NULL}};
const Segment SGI246187[] = {{&S92617[52], 10}, {NULL}};
const Segment SGI246188[] = {{&S4308[105], 7}, {NULL}};
const Segment SGI246189[] = {{&S92617[52], 21}, {NULL}};
const Segment SGI246190[] = {{&S92617[38], 6}, {NULL}};
const Segment SGI246191[] = {{&S92617[87], 5}, {NULL}};
const Segment SGI246192[] = {{&S4309[13], 4}, {&S4309[13], 4}, {&S4309[13], 4}, {NULL}};
const Segment SGI246193[] = {{&S4309[36], 5}, {NULL}};
const Segment SGI246194[] = {{&S4310[15], 4}, {NULL}};
const Segment SGI246195[] = {{&S4310[31], 5}, {NULL}};
const Segment SGI246196[] = {{&S4310[0], 7}, {NULL}};
const Segment SGI246197[] = {{&S4311[13], 4}, {NULL}};
const Segment SGI246198[] = {{&S4311[26], 5}, {NULL}};
const Segment SGI246199[] = {{&S4311[59], 6}, {NULL}};
const Segment SGI246200[] = {{&S4311[93], 5}, {NULL}};
const Segment SGI246201[] = {{&S4312[13], 4}, {NULL}};
const Segment SGI246202[] = {{&S4312[26], 3}, {NULL}};
const Segment SGI246203[] = {{&S92622[52], 18}, {NULL}};
const Segment SGI246204[] = {{&S4312[57], 9}, {NULL}};
const Segment SGI246205[] = {{&S92622[52], 29}, {NULL}};
const Segment SGI246206[] = {{&S92622[38], 6}, {NULL}};
const Segment SGI246207[] = {{&S4312[94], 6}, {NULL}};
const Segment SGI246208[] = {{&S92622[95], 5}, {NULL}};
const Segment SGI246209[] = {{&S4313[13], 4}, {NULL}};
const Segment SGI246210[] = {{&S4313[26], 5}, {NULL}};
const Segment SGI246211[] = {{&S4313[59], 6}, {NULL}};
const Segment SGI246212[] = {{&S4313[93], 7}, {NULL}};
const Segment SGI246213[] = {{&S4313[108], 9}, {NULL}};
const Segment SGI246214[] = {{&S4314[13], 4}, {NULL}};
const Segment SGI246215[] = {{&S4314[26], 5}, {NULL}};
const Segment SGI246216[] = {{&S92624[44], 10}, {NULL}};
const Segment SGI246217[] = {{&S4314[59], 6}, {NULL}};
const Segment SGI246218[] = {{&S92624[44], 22}, {NULL}};
const Segment SGI246219[] = {{&S92624[30], 6}, {NULL}};
const Segment SGI246220[] = {{&S4314[93], 7}, {&S4314[30], 1}, {NULL}};
const Segment SGI246221[] = {{&S92624[146], 3}, {NULL}};
const Segment SGI246222[] = {{&S92624[111], 6}, {NULL}};
const Segment SGI246223[] = {{&S92624[75], 9}, {NULL}};
const Segment SGI246224[] = {{&S4315[13], 4}, {NULL}};
const Segment SGI246225[] = {{&S4315[26], 5}, {NULL}};
const Segment SGI246226[] = {{&S4315[59], 6}, {NULL}};
const Segment SGI246227[] = {{&S4315[93], 7}, {NULL}};
const Segment SGI246228[] = {{&S4316[13], 4}, {&S4316[13], 4}, {NULL}};
const Segment SGI246229[] = {{&S4316[38], 5}, {NULL}};
const Segment SGI246230[] = {{&S4316[121], 5}, {NULL}};
const Segment SGI246231[] = {{&S4317[13], 4}, {NULL}};
const Segment SGI246232[] = {{&S4317[26], 5}, {NULL}};
const Segment SGI246233[] = {{&S4317[59], 6}, {NULL}};
const Segment SGI246234[] = {{&S4317[93], 7}, {NULL}};
const Segment SGI246235[] = {{&S4318[13], 4}, {&S4318[13], 4}, {NULL}};
const Segment SGI246236[] = {{&S4318[38], 5}, {NULL}};
const Segment SGI246237[] = {{&S4318[121], 6}, {NULL}};
const Segment SGI246238[] = {{&S4319[13], 4}, {NULL}};
const Segment SGI246239[] = {{&S4319[26], 5}, {NULL}};
const Segment SGI246240[] = {{&S4319[59], 6}, {NULL}};
const Segment SGI246241[] = {{&S4319[93], 7}, {&S4319[93], 1}, {NULL}};
const Segment SGI246242[] = {{&S4320[13], 4}, {NULL}};
const Segment SGI246243[] = {{&S4320[26], 3}, {NULL}};
const Segment SGI246244[] = {{&S4320[57], 6}, {NULL}};
const Segment SGI246245[] = {{&S4320[91], 6}, {NULL}};
const Segment SGI246246[] = {{&S4320[105], 6}, {NULL}};
const Segment SGI246247[] = {{&S4321[13], 4}, {&S4321[13], 4}, {NULL}};
const Segment SGI246248[] = {{&S4321[26], 3}, {NULL}};
const Segment SGI246249[] = {{&S4321[57], 6}, {NULL}};
const Segment SGI246250[] = {{&S4321[91], 6}, {NULL}};
const Segment SGI246251[] = {{&S92633[34], 10}, {NULL}};
const Segment SGI246252[] = {{&S92633[27], 6}, {NULL}};
const Segment SGI246253[] = {{&S92633[96], 9}, {NULL}};
const Segment SGI246254[] = {{&S92633[69], 6}, {NULL}};
const Segment SGI246255[] = {{&S92633[130], 8}, {NULL}};
const Segment SGI246256[] = {{&S4322[13], 4}, {NULL}};
const Segment SGI246257[] = {{&S4322[26], 5}, {NULL}};
const Segment SGI246258[] = {{&S4322[59], 6}, {NULL}};
const Segment SGI246259[] = {{&S4322[93], 6}, {NULL}};
const Segment SGI246260[] = {{&S4322[107], 9}, {&S4322[107], 9}, {&S4322[107], 9}, {&S4322[107], 9}, {NULL}};
const Segment SGI246261[] = {{&S4323[13], 4}, {NULL}};
const Segment SGI246262[] = {{&S92635[34], 13}, {NULL}};
const Segment SGI246263[] = {{&S4323[26], 3}, {NULL}};
const Segment SGI246264[] = {{&S92635[27], 6}, {NULL}};
const Segment SGI246265[] = {{&S4323[57], 6}, {NULL}};
const Segment SGI246266[] = {{&S4323[91], 6}, {&S4323[91], 1}, {NULL}};
const Segment SGI246267[] = {{&S92635[100], 9}, {NULL}};
const Segment SGI246268[] = {{&S92635[73], 6}, {NULL}};
const Segment SGI246269[] = {{&S4324[13], 4}, {NULL}};
const Segment SGI246270[] = {{&S4324[26], 5}, {NULL}};
const Segment SGI246271[] = {{&S4324[59], 6}, {NULL}};
const Segment SGI246272[] = {{&S4324[93], 6}, {NULL}};
const Segment SGI246273[] = {{&S4324[107], 6}, {NULL}};
const Segment SGI246274[] = {{&S4325[13], 4}, {NULL}};
const Segment SGI246275[] = {{&S4325[26], 5}, {NULL}};
const Segment SGI246276[] = {{&S4325[59], 6}, {NULL}};
const Segment SGI246277[] = {{&S4325[93], 6}, {NULL}};
const Segment SGI246278[] = {{&S4325[107], 9}, {NULL}};
const Segment SGI246279[] = {{&S92650[34], 7}, {NULL}};
const Segment SGI246280[] = {{&S92650[27], 6}, {NULL}};
const Segment SGI246281[] = {{&S92650[94], 9}, {NULL}};
const Segment SGI246282[] = {{&S92650[67], 6}, {NULL}};
const Segment SGI246283[] = {{&S92654[34], 6}, {NULL}};
const Segment SGI246284[] = {{&S92654[27], 6}, {NULL}};
const Segment SGI246285[] = {{&S92654[92], 9}, {NULL}};
const Segment SGI246286[] = {{&S92654[65], 6}, {NULL}};
const Segment SGI246287[] = {{&S92732[34], 10}, {NULL}};
const Segment SGI246288[] = {{&S92732[27], 6}, {NULL}};
const Segment SGI246289[] = {{&S92732[96], 9}, {NULL}};
const Segment SGI246290[] = {{&S92732[69], 6}, {NULL}};
const Segment SGI246291[] = {{&S92760[34], 10}, {NULL}};
const Segment SGI246292[] = {{&S92760[27], 6}, {NULL}};
const Segment SGI246293[] = {{&S92760[96], 9}, {NULL}};
const Segment SGI246294[] = {{&S92760[69], 6}, {NULL}};
const Segment SGI246295[] = {{&S99710[34], 10}, {NULL}};
const Segment SGI246296[] = {{&S99710[27], 6}, {NULL}};
const Segment SGI246297[] = {{&S99710[97], 9}, {NULL}};
const Segment SGI246298[] = {{&S99710[70], 6}, {NULL}};
const Segment SGI246299[] = {{&S99712[34], 10}, {NULL}};
const Segment SGI246300[] = {{&S99712[27], 6}, {NULL}};
const Segment SGI246301[] = {{&S99712[97], 9}, {NULL}};
const Segment SGI246302[] = {{&S99712[70], 6}, {NULL}};
const Segment SGI246303[] = {{&S92776[34], 10}, {NULL}};
const Segment SGI246304[] = {{&S92776[27], 6}, {NULL}};
const Segment SGI246305[] = {{&S92776[97], 9}, {NULL}};
const Segment SGI246306[] = {{&S92776[70], 6}, {NULL}};
const Segment SGI246307[] = {{&S99718[34], 6}, {NULL}};
const Segment SGI246308[] = {{&S99718[27], 6}, {NULL}};
const Segment SGI246309[] = {{&S99718[93], 9}, {NULL}};
const Segment SGI246310[] = {{&S99718[66], 6}, {NULL}};
const Segment SGI246311[] = {{&S99730[34], 7}, {NULL}};
const Segment SGI246312[] = {{&S99730[27], 6}, {NULL}};
const Segment SGI246313[] = {{&S99730[95], 9}, {NULL}};
const Segment SGI246314[] = {{&S99730[68], 6}, {NULL}};
const Segment SGI246315[] = {{&S99731[44], 12}, {NULL}};
const Segment SGI246316[] = {{&S99731[44], 34}, {NULL}};
const Segment SGI246317[] = {{&S99731[30], 6}, {NULL}};
const Segment SGI246318[] = {{&S99731[127], 3}, {NULL}};
const Segment SGI246319[] = {{&S99731[92], 6}, {NULL}};
const Segment SGI246320[] = {{&S99731[162], 3}, {NULL}};
const Segment SGI246321[] = {{&S99741[34], 10}, {NULL}};
const Segment SGI246322[] = {{&S99741[27], 6}, {NULL}};
const Segment SGI246323[] = {{&S99741[97], 9}, {NULL}};
const Segment SGI246324[] = {{&S99741[70], 6}, {NULL}};
const Segment SGI246325[] = {{&S99744[34], 10}, {NULL}};
const Segment SGI246326[] = {{&S99744[27], 6}, {NULL}};
const Segment SGI246327[] = {{&S99744[97], 9}, {NULL}};
const Segment SGI246328[] = {{&S99744[70], 6}, {NULL}};
const Segment SGI246329[] = {{&S92814[34], 10}, {NULL}};
const Segment SGI246330[] = {{&S92814[27], 6}, {NULL}};
const Segment SGI246331[] = {{&S92814[97], 9}, {NULL}};
const Segment SGI246332[] = {{&S99745[34], 7}, {NULL}};
const Segment SGI246333[] = {{&S92814[70], 6}, {NULL}};
const Segment SGI246334[] = {{&S99745[27], 6}, {NULL}};
const Segment SGI246335[] = {{&S99745[94], 9}, {NULL}};
const Segment SGI246336[] = {{&S99745[67], 6}, {NULL}};
const Segment SGI246337[] = {{&S92821[34], 10}, {NULL}};
const Segment SGI246338[] = {{&S92821[27], 6}, {NULL}};
const Segment SGI246339[] = {{&S92821[96], 9}, {NULL}};
const Segment SGI246340[] = {{&S92821[69], 6}, {NULL}};
const Segment SGI246341[] = {{&S99804[44], 10}, {NULL}};
const Segment SGI246342[] = {{&S99804[44], 22}, {NULL}};
const Segment SGI246343[] = {{&S99804[30], 6}, {NULL}};
const Segment SGI246344[] = {{&S99804[80], 6}, {NULL}};
const Segment SGI246345[] = {{&S99804[107], 13}, {NULL}};
const Segment SGI246346[] = {{&S99817[61], 10}, {NULL}};
const Segment SGI246347[] = {{&S99817[61], 22}, {NULL}};
const Segment SGI246348[] = {{&S99817[47], 6}, {NULL}};
const Segment SGI246349[] = {{&S99817[132], 3}, {NULL}};
const Segment SGI246350[] = {{&S99817[97], 6}, {NULL}};
const Segment SGI246351[] = {{&S99817[11], 3}, {NULL}};
const Segment SGI246352[] = {{&S99818[44], 7}, {NULL}};
const Segment SGI246353[] = {{&S99818[44], 24}, {NULL}};
const Segment SGI246354[] = {{&S99818[30], 6}, {NULL}};
const Segment SGI246355[] = {{&S99818[82], 6}, {NULL}};
const Segment SGI246356[] = {{&S99818[109], 13}, {NULL}};
const Segment SGI246357[] = {{&S99828[34], 17}, {NULL}};
const Segment SGI246358[] = {{&S99828[27], 6}, {NULL}};
const Segment SGI246359[] = {{&S99828[104], 9}, {NULL}};
const Segment SGI246360[] = {{&S99828[77], 6}, {NULL}};
const Segment SGI246361[] = {{&S99831[34], 6}, {NULL}};
const Segment SGI246362[] = {{&S99831[27], 6}, {NULL}};
const Segment SGI246363[] = {{&S99831[104], 9}, {NULL}};
const Segment SGI246364[] = {{&S99831[77], 6}, {NULL}};
const Segment SGI246365[] = {{&S99833[34], 10}, {NULL}};
const Segment SGI246366[] = {{&S99833[27], 6}, {NULL}};
const Segment SGI246367[] = {{&S99833[97], 9}, {NULL}};
const Segment SGI246368[] = {{&S99833[70], 6}, {NULL}};
const Segment SGI246369[] = {{&S99845[34], 8}, {NULL}};
const Segment SGI246370[] = {{&S99845[27], 6}, {NULL}};
const Segment SGI246371[] = {{&S99845[103], 9}, {NULL}};
const Segment SGI246372[] = {{&S99845[76], 6}, {NULL}};
const Segment SGI246373[] = {{&S99845[137], 8}, {NULL}};
const Segment SGI246374[] = {{&S99846[34], 9}, {NULL}};
const Segment SGI246375[] = {{&S99846[27], 6}, {NULL}};
const Segment SGI246376[] = {{&S99846[96], 9}, {NULL}};
const Segment SGI246377[] = {{&S99846[69], 6}, {NULL}};
const Segment SGI246378[] = {{&S99846[137], 8}, {NULL}};
const Segment SGI246379[] = {{&S99869[34], 10}, {NULL}};
const Segment SGI246380[] = {{&S99869[27], 6}, {NULL}};
const Segment SGI246381[] = {{&S99869[97], 9}, {NULL}};
const Segment SGI246382[] = {{&S99869[70], 6}, {NULL}};
const Segment SGI246383[] = {{&S99878[44], 11}, {NULL}};
const Segment SGI246384[] = {{&S99878[44], 23}, {NULL}};
const Segment SGI246385[] = {{&S99878[30], 6}, {NULL}};
const Segment SGI246386[] = {{&S99878[114], 3}, {NULL}};
const Segment SGI246387[] = {{&S99878[81], 5}, {NULL}};
const Segment SGI246388[] = {{&S99878[118], 14}, {NULL}};
const Segment SGI246389[] = {{&S99889[34], 10}, {NULL}};
const Segment SGI246390[] = {{&S99889[27], 6}, {NULL}};
const Segment SGI246391[] = {{&S99889[97], 9}, {NULL}};
const Segment SGI246392[] = {{&S99889[70], 6}, {NULL}};
const Segment SGI246393[] = {{&S99894[44], 11}, {NULL}};
const Segment SGI246394[] = {{&S99894[44], 23}, {NULL}};
const Segment SGI246395[] = {{&S99894[30], 6}, {NULL}};
const Segment SGI246396[] = {{&S99894[114], 3}, {NULL}};
const Segment SGI246397[] = {{&S99894[81], 5}, {NULL}};
const Segment SGI246398[] = {{&S99894[118], 14}, {NULL}};
const Segment SGI246399[] = {{&S92945[34], 6}, {NULL}};
const Segment SGI246400[] = {{&S92945[27], 6}, {NULL}};
const Segment SGI246401[] = {{&S92945[92], 9}, {NULL}};
const Segment SGI246402[] = {{&S92945[65], 6}, {NULL}};
const Segment SGI246403[] = {{&S99904[34], 10}, {NULL}};
const Segment SGI246404[] = {{&S99904[27], 6}, {NULL}};
const Segment SGI246405[] = {{&S99904[97], 9}, {NULL}};
const Segment SGI246406[] = {{&S99904[70], 6}, {NULL}};
const Segment SGI246407[] = {{&S92962[34], 11}, {NULL}};
const Segment SGI246408[] = {{&S92962[27], 6}, {NULL}};
const Segment SGI246409[] = {{&S92962[97], 9}, {NULL}};
const Segment SGI246410[] = {{&S92962[70], 6}, {NULL}};
const Segment SGI246411[] = {{&S92973[44], 10}, {NULL}};
const Segment SGI246412[] = {{&S92973[44], 27}, {NULL}};
const Segment SGI246413[] = {{&S92973[30], 6}, {NULL}};
const Segment SGI246414[] = {{&S92973[126], 3}, {NULL}};
const Segment SGI246415[] = {{&S92973[92], 5}, {NULL}};
const Segment SGI246416[] = {{&S92979[34], 10}, {NULL}};
const Segment SGI246417[] = {{&S92979[27], 6}, {NULL}};
const Segment SGI246418[] = {{&S92979[97], 9}, {NULL}};
const Segment SGI246419[] = {{&S92979[70], 6}, {NULL}};
const Segment SGI246420[] = {{&S99938[34], 11}, {NULL}};
const Segment SGI246421[] = {{&S99938[27], 6}, {NULL}};
const Segment SGI246422[] = {{&S99938[98], 9}, {NULL}};
const Segment SGI246423[] = {{&S99938[71], 6}, {NULL}};
const Segment SGI246424[] = {{&S92984[34], 12}, {NULL}};
const Segment SGI246425[] = {{&S92984[27], 6}, {NULL}};
const Segment SGI246426[] = {{&S92984[99], 9}, {NULL}};
const Segment SGI246427[] = {{&S92984[72], 6}, {NULL}};
const Segment SGI246428[] = {{&S99943[44], 10}, {NULL}};
const Segment SGI246429[] = {{&S99943[44], 22}, {NULL}};
const Segment SGI246430[] = {{&S99943[30], 6}, {NULL}};
const Segment SGI246431[] = {{&S99943[113], 3}, {NULL}};
const Segment SGI246432[] = {{&S99943[80], 5}, {NULL}};
const Segment SGI246433[] = {{&S99943[117], 14}, {NULL}};
const Segment SGI246434[] = {{&S92988[34], 7}, {NULL}};
const Segment SGI246435[] = {{&S92988[27], 6}, {NULL}};
const Segment SGI246436[] = {{&S92988[94], 9}, {NULL}};
const Segment SGI246437[] = {{&S92988[67], 6}, {NULL}};
const Segment SGI246438[] = {{&S99947[44], 11}, {NULL}};
const Segment SGI246439[] = {{&S99947[44], 23}, {NULL}};
const Segment SGI246440[] = {{&S99947[30], 6}, {NULL}};
const Segment SGI246441[] = {{&S99947[116], 3}, {NULL}};
const Segment SGI246442[] = {{&S99947[81], 6}, {NULL}};
const Segment SGI246443[] = {{&S99947[141], 7}, {NULL}};
const Segment SGI246444[] = {{&S92998[44], 11}, {NULL}};
const Segment SGI246445[] = {{&S92998[44], 22}, {NULL}};
const Segment SGI246446[] = {{&S92998[30], 6}, {NULL}};
const Segment SGI246447[] = {{&S92998[115], 3}, {NULL}};
const Segment SGI246448[] = {{&S92998[80], 6}, {NULL}};
const Segment SGI246449[] = {{&S92998[140], 7}, {NULL}};
const Segment SGI246450[] = {{&S101397[44], 10}, {NULL}};
const Segment SGI246451[] = {{&S101397[44], 22}, {NULL}};
const Segment SGI246452[] = {{&S101397[30], 6}, {NULL}};
const Segment SGI246453[] = {{&S101397[133], 3}, {NULL}};
const Segment SGI246454[] = {{&S101397[98], 6}, {NULL}};
const Segment SGI246455[] = {{&S101397[161], 3}, {NULL}};
const Segment SGI246456[] = {{&S101397[68], 8}, {NULL}};
const Segment SGI246457[] = {{&S101398[44], 10}, {NULL}};
const Segment SGI246458[] = {{&S101398[44], 22}, {NULL}};
const Segment SGI246459[] = {{&S101398[30], 6}, {NULL}};
const Segment SGI246460[] = {{&S101398[133], 3}, {NULL}};
const Segment SGI246461[] = {{&S101398[98], 6}, {NULL}};
const Segment SGI246462[] = {{&S101398[161], 3}, {NULL}};
const Segment SGI246463[] = {{&S101398[68], 8}, {NULL}};
const Segment SGI246464[] = {{&S99971[44], 10}, {NULL}};
const Segment SGI246465[] = {{&S99971[44], 22}, {NULL}};
const Segment SGI246466[] = {{&S99971[30], 6}, {NULL}};
const Segment SGI246467[] = {{&S99971[131], 3}, {NULL}};
const Segment SGI246468[] = {{&S99971[96], 6}, {NULL}};
const Segment SGI246469[] = {{&S99971[68], 8}, {NULL}};
const Segment SGI246470[] = {{&S93015[52], 10}, {NULL}};
const Segment SGI246471[] = {{&S99975[44], 10}, {NULL}};
const Segment SGI246472[] = {{&S93015[52], 21}, {NULL}};
const Segment SGI246473[] = {{&S99975[44], 22}, {NULL}};
const Segment SGI246474[] = {{&S93015[38], 6}, {NULL}};
const Segment SGI246475[] = {{&S99975[30], 6}, {NULL}};
const Segment SGI246476[] = {{&S93015[87], 5}, {NULL}};
const Segment SGI246477[] = {{&S99975[80], 6}, {NULL}};
const Segment SGI246478[] = {{&S99975[107], 13}, {NULL}};
const Segment SGI246479[] = {{&S99993[59], 11}, {NULL}};
const Segment SGI246480[] = {{&S99993[59], 23}, {NULL}};
const Segment SGI246481[] = {{&S99993[45], 6}, {NULL}};
const Segment SGI246482[] = {{&S99993[131], 3}, {NULL}};
const Segment SGI246483[] = {{&S99993[96], 6}, {NULL}};
const Segment SGI246484[] = {{&S99993[0], 14}, {NULL}};
const Segment SGI246485[] = {{&S93040[34], 11}, {NULL}};
const Segment SGI246486[] = {{&S93040[27], 6}, {NULL}};
const Segment SGI246487[] = {{&S93040[97], 9}, {NULL}};
const Segment SGI246488[] = {{&S93040[70], 6}, {NULL}};
const Segment SGI246489[] = {{&S93040[138], 8}, {NULL}};
const Segment SGI246490[] = {{&S100000[44], 9}, {NULL}};
const Segment SGI246491[] = {{&S100000[44], 21}, {NULL}};
const Segment SGI246492[] = {{&S100000[30], 6}, {NULL}};
const Segment SGI246493[] = {{&S100000[79], 6}, {NULL}};
const Segment SGI246494[] = {{&S100000[106], 13}, {NULL}};
const Segment SGI246495[] = {{&S100017[44], 14}, {NULL}};
const Segment SGI246496[] = {{&S100017[44], 31}, {NULL}};
const Segment SGI246497[] = {{&S100017[30], 6}, {NULL}};
const Segment SGI246498[] = {{&S100017[122], 3}, {NULL}};
const Segment SGI246499[] = {{&S100017[89], 5}, {NULL}};
const Segment SGI246500[] = {{&S100017[126], 14}, {NULL}};
const Segment SGI246501[] = {{&S100026[34], 7}, {NULL}};
const Segment SGI246502[] = {{&S100026[27], 6}, {NULL}};
const Segment SGI246503[] = {{&S100026[100], 9}, {NULL}};
const Segment SGI246504[] = {{&S100026[73], 6}, {NULL}};
const Segment SGI246505[] = {{&S100026[141], 8}, {NULL}};
const Segment SGI246506[] = {{&S100032[34], 10}, {NULL}};
const Segment SGI246507[] = {{&S100032[27], 6}, {NULL}};
const Segment SGI246508[] = {{&S100032[97], 9}, {NULL}};
const Segment SGI246509[] = {{&S100032[70], 6}, {NULL}};
const Segment SGI246510[] = {{&S100036[44], 14}, {NULL}};
const Segment SGI246511[] = {{&S100036[44], 26}, {NULL}};
const Segment SGI246512[] = {{&S100036[30], 6}, {NULL}};
const Segment SGI246513[] = {{&S100036[119], 3}, {NULL}};
const Segment SGI246514[] = {{&S100036[84], 6}, {NULL}};
const Segment SGI246515[] = {{&S100050[44], 7}, {NULL}};
const Segment SGI246516[] = {{&S100050[44], 19}, {NULL}};
const Segment SGI246517[] = {{&S100050[30], 6}, {NULL}};
const Segment SGI246518[] = {{&S100050[112], 3}, {NULL}};
const Segment SGI246519[] = {{&S100050[77], 6}, {NULL}};
const Segment SGI246520[] = {{&S100050[144], 15}, {NULL}};
const Segment SGI246521[] = {{&S100053[34], 7}, {NULL}};
const Segment SGI246522[] = {{&S100053[27], 6}, {NULL}};
const Segment SGI246523[] = {{&S100053[99], 9}, {NULL}};
const Segment SGI246524[] = {{&S100053[72], 6}, {NULL}};
const Segment SGI246525[] = {{&S100060[44], 10}, {NULL}};
const Segment SGI246526[] = {{&S100060[44], 22}, {NULL}};
const Segment SGI246527[] = {{&S100060[30], 6}, {NULL}};
const Segment SGI246528[] = {{&S100060[80], 6}, {NULL}};
const Segment SGI246529[] = {{&S100060[107], 13}, {NULL}};
const Segment SGI246530[] = {{&S4870[13], 4}, {NULL}};
const Segment SGI246531[] = {{&S4870[106], 5}, {NULL}};
const Segment SGI246532[] = {{&S4870[29], 5}, {NULL}};
const Segment SGI246533[] = {{&S4870[69], 8}, {NULL}};
const Segment SGI246534[] = {{&S4870[119], 3}, {NULL}};
const Segment SGI246535[] = {{&S4870[130], 9}, {NULL}};
const Segment SGI246536[] = {{&S93126[45], 9}, {NULL}};
const Segment SGI246537[] = {{&S93126[30], 6}, {NULL}};
const Segment SGI246538[] = {{&S93126[113], 3}, {NULL}};
const Segment SGI246539[] = {{&S93126[79], 5}, {NULL}};
const Segment SGI246540[] = {{&S93126[137], 13}, {NULL}};
const Segment SGI246541[] = {{&S4871[13], 4}, {NULL}};
const Segment SGI246542[] = {{&S4871[104], 5}, {NULL}};
const Segment SGI246543[] = {{&S4871[29], 3}, {NULL}};
const Segment SGI246544[] = {{&S4871[67], 8}, {NULL}};
const Segment SGI246545[] = {{&S4871[117], 5}, {NULL}};
const Segment SGI246546[] = {{&S4871[130], 9}, {NULL}};
const Segment SGI246547[] = {{&S100074[34], 9}, {NULL}};
const Segment SGI246548[] = {{&S100074[27], 6}, {NULL}};
const Segment SGI246549[] = {{&S100074[96], 9}, {NULL}};
const Segment SGI246550[] = {{&S93127[34], 12}, {NULL}};
const Segment SGI246551[] = {{&S100074[69], 6}, {NULL}};
const Segment SGI246552[] = {{&S93127[27], 6}, {NULL}};
const Segment SGI246553[] = {{&S4872[13], 4}, {NULL}};
const Segment SGI246554[] = {{&S4872[104], 5}, {NULL}};
const Segment SGI246555[] = {{&S4872[29], 3}, {NULL}};
const Segment SGI246556[] = {{&S93127[99], 9}, {NULL}};
const Segment SGI246557[] = {{&S93127[72], 6}, {NULL}};
const Segment SGI246558[] = {{&S4872[67], 8}, {NULL}};
const Segment SGI246559[] = {{&S4872[117], 5}, {NULL}};
const Segment SGI246560[] = {{&S4872[130], 9}, {NULL}};
const Segment SGI246561[] = {{&S4873[13], 4}, {NULL}};
const Segment SGI246562[] = {{&S4873[106], 5}, {NULL}};
const Segment SGI246563[] = {{&S4873[29], 5}, {NULL}};
const Segment SGI246564[] = {{&S4873[69], 8}, {NULL}};
const Segment SGI246565[] = {{&S4873[119], 3}, {NULL}};
const Segment SGI246566[] = {{&S4873[130], 9}, {NULL}};
const Segment SGI246567[] = {{&S4874[13], 4}, {NULL}};
const Segment SGI246568[] = {{&S4874[104], 5}, {NULL}};
const Segment SGI246569[] = {{&S4874[29], 3}, {NULL}};
const Segment SGI246570[] = {{&S4874[67], 8}, {NULL}};
const Segment SGI246571[] = {{&S4874[117], 5}, {NULL}};
const Segment SGI246572[] = {{&S4874[130], 9}, {NULL}};
const Segment SGI246573[] = {{&S4875[13], 4}, {NULL}};
const Segment SGI246574[] = {{&S4875[106], 5}, {NULL}};
const Segment SGI246575[] = {{&S4875[29], 5}, {NULL}};
const Segment SGI246576[] = {{&S4875[69], 8}, {NULL}};
const Segment SGI246577[] = {{&S4875[119], 3}, {NULL}};
const Segment SGI246578[] = {{&S4875[130], 9}, {NULL}};
const Segment SGI246579[] = {{&S4876[13], 4}, {NULL}};
const Segment SGI246580[] = {{&S4876[106], 5}, {NULL}};
const Segment SGI246581[] = {{&S4876[29], 5}, {NULL}};
const Segment SGI246582[] = {{&S4876[69], 8}, {NULL}};
const Segment SGI246583[] = {{&S4876[119], 3}, {NULL}};
const Segment SGI246584[] = {{&S4876[130], 9}, {NULL}};
const Segment SGI246585[] = {{&S4877[13], 4}, {NULL}};
const Segment SGI246586[] = {{&S4877[106], 5}, {NULL}};
const Segment SGI246587[] = {{&S4877[29], 5}, {NULL}};
const Segment SGI246588[] = {{&S4877[69], 8}, {NULL}};
const Segment SGI246589[] = {{&S4877[119], 3}, {NULL}};
const Segment SGI246590[] = {{&S4877[130], 9}, {NULL}};
const Segment SGI246591[] = {{&S4878[13], 4}, {NULL}};
const Segment SGI246592[] = {{&S4878[106], 5}, {NULL}};
const Segment SGI246593[] = {{&S4878[29], 5}, {NULL}};
const Segment SGI246594[] = {{&S4878[69], 8}, {NULL}};
const Segment SGI246595[] = {{&S4878[119], 5}, {NULL}};
const Segment SGI246596[] = {{&S4878[132], 9}, {NULL}};
const Segment SGI246597[] = {{&S4879[13], 4}, {NULL}};
const Segment SGI246598[] = {{&S4879[106], 5}, {NULL}};
const Segment SGI246599[] = {{&S4879[29], 5}, {NULL}};
const Segment SGI246600[] = {{&S4879[69], 8}, {NULL}};
const Segment SGI246601[] = {{&S4879[119], 5}, {NULL}};
const Segment SGI246602[] = {{&S100086[34], 15}, {NULL}};
const Segment SGI246603[] = {{&S4879[132], 9}, {NULL}};
const Segment SGI246604[] = {{&S100086[27], 6}, {NULL}};
const Segment SGI246605[] = {{&S100086[108], 9}, {NULL}};
const Segment SGI246606[] = {{&S100086[81], 6}, {NULL}};
const Segment SGI246607[] = {{&S4880[13], 4}, {NULL}};
const Segment SGI246608[] = {{&S4880[106], 5}, {NULL}};
const Segment SGI246609[] = {{&S4880[29], 5}, {NULL}};
const Segment SGI246610[] = {{&S4880[69], 8}, {NULL}};
const Segment SGI246611[] = {{&S4880[119], 3}, {NULL}};
const Segment SGI246612[] = {{&S4880[130], 9}, {NULL}};
const Segment SGI246613[] = {{&S100087[34], 5}, {NULL}};
const Segment SGI246614[] = {{&S100087[27], 6}, {NULL}};
const Segment SGI246615[] = {{&S100087[92], 9}, {NULL}};
const Segment SGI246616[] = {{&S100087[65], 6}, {NULL}};
const Segment SGI246617[] = {{&S4881[13], 4}, {NULL}};
const Segment SGI246618[] = {{&S4881[106], 5}, {NULL}};
const Segment SGI246619[] = {{&S4881[29], 5}, {NULL}};
const Segment SGI246620[] = {{&S4881[69], 8}, {NULL}};
const Segment SGI246621[] = {{&S4881[119], 3}, {NULL}};
const Segment SGI246622[] = {{&S4881[130], 9}, {NULL}};
const Segment SGI246623[] = {{&S100088[34], 11}, {NULL}};
const Segment SGI246624[] = {{&S100088[27], 6}, {NULL}};
const Segment SGI246625[] = {{&S4882[13], 4}, {NULL}};
const Segment SGI246626[] = {{&S4882[106], 5}, {NULL}};
const Segment SGI246627[] = {{&S100088[98], 9}, {NULL}};
const Segment SGI246628[] = {{&S4882[29], 5}, {NULL}};
const Segment SGI246629[] = {{&S100088[71], 6}, {NULL}};
const Segment SGI246630[] = {{&S4882[69], 8}, {NULL}};
const Segment SGI246631[] = {{&S4882[119], 5}, {NULL}};
const Segment SGI246632[] = {{&S4882[132], 9}, {NULL}};
const Segment SGI246633[] = {{&S4883[13], 4}, {NULL}};
const Segment SGI246634[] = {{&S4883[8], 3}, {NULL}};
const Segment SGI246635[] = {{&S4883[26], 5}, {NULL}};
const Segment SGI246636[] = {{&S4883[59], 8}, {NULL}};
const Segment SGI246637[] = {{&S4883[107], 3}, {NULL}};
const Segment SGI246638[] = {{&S4883[118], 9}, {NULL}};
const Segment SGI246639[] = {{&S4884[13], 4}, {NULL}};
const Segment SGI246640[] = {{&S4884[106], 5}, {NULL}};
const Segment SGI246641[] = {{&S4884[29], 5}, {NULL}};
const Segment SGI246642[] = {{&S4884[69], 8}, {NULL}};
const Segment SGI246643[] = {{&S4884[119], 5}, {NULL}};
const Segment SGI246644[] = {{&S4884[132], 9}, {NULL}};
const Segment SGI246645[] = {{&S4885[13], 4}, {NULL}};
const Segment SGI246646[] = {{&S4885[104], 5}, {NULL}};
const Segment SGI246647[] = {{&S4885[29], 3}, {NULL}};
const Segment SGI246648[] = {{&S4885[67], 8}, {NULL}};
const Segment SGI246649[] = {{&S4885[117], 5}, {NULL}};
const Segment SGI246650[] = {{&S4885[130], 9}, {NULL}};
const Segment SGI246651[] = {{&S4886[13], 4}, {NULL}};
const Segment SGI246652[] = {{&S4886[106], 5}, {NULL}};
const Segment SGI246653[] = {{&S4886[29], 5}, {NULL}};
const Segment SGI246654[] = {{&S4886[69], 8}, {NULL}};
const Segment SGI246655[] = {{&S4886[119], 5}, {NULL}};
const Segment SGI246656[] = {{&S4886[132], 9}, {NULL}};
const Segment SGI246657[] = {{&S100095[34], 10}, {NULL}};
const Segment SGI246658[] = {{&S100095[27], 6}, {NULL}};
const Segment SGI246659[] = {{&S100095[97], 9}, {NULL}};
const Segment SGI246660[] = {{&S100095[70], 6}, {NULL}};
const Segment SGI246661[] = {{&S100095[138], 8}, {NULL}};
const Segment SGI246662[] = {{&S4887[13], 4}, {NULL}};
const Segment SGI246663[] = {{&S4887[8], 3}, {NULL}};
const Segment SGI246664[] = {{&S4887[26], 5}, {NULL}};
const Segment SGI246665[] = {{&S4887[59], 8}, {NULL}};
const Segment SGI246666[] = {{&S4887[107], 3}, {NULL}};
const Segment SGI246667[] = {{&S4887[118], 9}, {NULL}};
const Segment SGI246668[] = {{&S4888[13], 4}, {NULL}};
const Segment SGI246669[] = {{&S4888[115], 5}, {NULL}};
const Segment SGI246670[] = {{&S4888[29], 3}, {NULL}};
const Segment SGI246671[] = {{&S4888[67], 8}, {NULL}};
const Segment SGI246672[] = {{&S4888[103], 3}, {NULL}};
const Segment SGI246673[] = {{&S4888[128], 9}, {NULL}};
const Segment SGI246674[] = {{&S4889[13], 4}, {NULL}};
const Segment SGI246675[] = {{&S4889[106], 5}, {NULL}};
const Segment SGI246676[] = {{&S4889[29], 5}, {NULL}};
const Segment SGI246677[] = {{&S4889[69], 8}, {NULL}};
const Segment SGI246678[] = {{&S4889[119], 5}, {NULL}};
const Segment SGI246679[] = {{&S4889[132], 9}, {NULL}};
const Segment SGI246680[] = {{&S4890[13], 4}, {&S4890[13], 4}, {NULL}};
const Segment SGI246681[] = {{&S4890[118], 5}, {NULL}};
const Segment SGI246682[] = {{&S4890[29], 3}, {NULL}};
const Segment SGI246683[] = {{&S4890[81], 8}, {NULL}};
const Segment SGI246684[] = {{&S4890[131], 5}, {NULL}};
const Segment SGI246685[] = {{&S4890[144], 9}, {NULL}};
const Segment SGI246686[] = {{&S4891[13], 4}, {NULL}};
const Segment SGI246687[] = {{&S4891[107], 5}, {NULL}};
const Segment SGI246688[] = {{&S4891[29], 5}, {NULL}};
const Segment SGI246689[] = {{&S4891[69], 9}, {NULL}};
const Segment SGI246690[] = {{&S4891[120], 3}, {NULL}};
const Segment SGI246691[] = {{&S4891[131], 9}, {NULL}};
const Segment SGI246692[] = {{&S4892[13], 4}, {NULL}};
const Segment SGI246693[] = {{&S4892[106], 5}, {NULL}};
const Segment SGI246694[] = {{&S4892[29], 5}, {NULL}};
const Segment SGI246695[] = {{&S4892[69], 8}, {NULL}};
const Segment SGI246696[] = {{&S100104[34], 11}, {NULL}};
const Segment SGI246697[] = {{&S4892[119], 5}, {NULL}};
const Segment SGI246698[] = {{&S4892[132], 9}, {NULL}};
const Segment SGI246699[] = {{&S100104[27], 6}, {NULL}};
const Segment SGI246700[] = {{&S100104[98], 9}, {NULL}};
const Segment SGI246701[] = {{&S100104[71], 6}, {NULL}};
const Segment SGI246702[] = {{&S4893[13], 4}, {NULL}};
const Segment SGI246703[] = {{&S4893[107], 5}, {NULL}};
const Segment SGI246704[] = {{&S4893[29], 5}, {NULL}};
const Segment SGI246705[] = {{&S4893[69], 9}, {NULL}};
const Segment SGI246706[] = {{&S4893[120], 3}, {NULL}};
const Segment SGI246707[] = {{&S4893[131], 9}, {NULL}};
const Segment SGI246708[] = {{&S4894[13], 4}, {NULL}};
const Segment SGI246709[] = {{&S4894[104], 5}, {NULL}};
const Segment SGI246710[] = {{&S4894[29], 3}, {NULL}};
const Segment SGI246711[] = {{&S4894[67], 8}, {NULL}};
const Segment SGI246712[] = {{&S4894[117], 5}, {NULL}};
const Segment SGI246713[] = {{&S4894[130], 9}, {NULL}};
const Segment SGI246714[] = {{&S4895[13], 4}, {NULL}};
const Segment SGI246715[] = {{&S4895[29], 5}, {NULL}};
const Segment SGI246716[] = {{&S4895[106], 5}, {NULL}};
const Segment SGI246717[] = {{&S4895[119], 9}, {NULL}};
const Segment SGI246718[] = {{&S4896[13], 4}, {NULL}};
const Segment SGI246719[] = {{&S4896[29], 5}, {NULL}};
const Segment SGI246720[] = {{&S4896[106], 5}, {NULL}};
const Segment SGI246721[] = {{&S4896[119], 9}, {NULL}};
const Segment SGI246722[] = {{&S4897[13], 4}, {NULL}};
const Segment SGI246723[] = {{&S4897[107], 5}, {NULL}};
const Segment SGI246724[] = {{&S4897[29], 5}, {NULL}};
const Segment SGI246725[] = {{&S4897[69], 9}, {NULL}};
const Segment SGI246726[] = {{&S4897[120], 3}, {NULL}};
const Segment SGI246727[] = {{&S4897[131], 9}, {NULL}};
const Segment SGI246728[] = {{&S93169[34], 10}, {NULL}};
const Segment SGI246729[] = {{&S4898[13], 4}, {NULL}};
const Segment SGI246730[] = {{&S4898[105], 5}, {NULL}};
const Segment SGI246731[] = {{&S93169[27], 6}, {NULL}};
const Segment SGI246732[] = {{&S4898[29], 3}, {NULL}};
const Segment SGI246733[] = {{&S4898[67], 9}, {NULL}};
const Segment SGI246734[] = {{&S4898[118], 3}, {NULL}};
const Segment SGI246735[] = {{&S4898[129], 9}, {NULL}};
const Segment SGI246736[] = {{&S93169[96], 9}, {NULL}};
const Segment SGI246737[] = {{&S93169[69], 6}, {NULL}};
const Segment SGI246738[] = {{&S4899[13], 4}, {NULL}};
const Segment SGI246739[] = {{&S4899[104], 5}, {NULL}};
const Segment SGI246740[] = {{&S4899[29], 3}, {NULL}};
const Segment SGI246741[] = {{&S4899[67], 8}, {NULL}};
const Segment SGI246742[] = {{&S4899[117], 7}, {NULL}};
const Segment SGI246743[] = {{&S4899[132], 9}, {NULL}};
const Segment SGI246744[] = {{&S4900[13], 4}, {NULL}};
const Segment SGI246745[] = {{&S4900[106], 5}, {NULL}};
const Segment SGI246746[] = {{&S4900[29], 5}, {NULL}};
const Segment SGI246747[] = {{&S4900[69], 8}, {NULL}};
const Segment SGI246748[] = {{&S4900[119], 3}, {NULL}};
const Segment SGI246749[] = {{&S4900[130], 9}, {NULL}};
const Segment SGI246750[] = {{&S4901[13], 4}, {NULL}};
const Segment SGI246751[] = {{&S4901[106], 5}, {NULL}};
const Segment SGI246752[] = {{&S4901[29], 5}, {NULL}};
const Segment SGI246753[] = {{&S4901[69], 8}, {NULL}};
const Segment SGI246754[] = {{&S4901[119], 3}, {NULL}};
const Segment SGI246755[] = {{&S4901[130], 9}, {NULL}};
const Segment SGI246756[] = {{&S4902[13], 4}, {NULL}};
const Segment SGI246757[] = {{&S4902[103], 5}, {NULL}};
const Segment SGI246758[] = {{&S4902[29], 3}, {NULL}};
const Segment SGI246759[] = {{&S4902[67], 8}, {NULL}};
const Segment SGI246760[] = {{&S4902[116], 3}, {NULL}};
const Segment SGI246761[] = {{&S4902[127], 9}, {NULL}};
const Segment SGI246762[] = {{&S4903[13], 4}, {NULL}};
const Segment SGI246763[] = {{&S4903[103], 5}, {NULL}};
const Segment SGI246764[] = {{&S4903[29], 3}, {NULL}};
const Segment SGI246765[] = {{&S4903[67], 8}, {NULL}};
const Segment SGI246766[] = {{&S4903[116], 3}, {NULL}};
const Segment SGI246767[] = {{&S4903[127], 9}, {NULL}};
const Segment SGI246768[] = {{&S93177[44], 7}, {NULL}};
const Segment SGI246769[] = {{&S93177[44], 19}, {NULL}};
const Segment SGI246770[] = {{&S93177[30], 6}, {NULL}};
const Segment SGI246771[] = {{&S93177[143], 3}, {NULL}};
const Segment SGI246772[] = {{&S4904[13], 4}, {NULL}};
const Segment SGI246773[] = {{&S93177[108], 6}, {NULL}};
const Segment SGI246774[] = {{&S4904[103], 5}, {NULL}};
const Segment SGI246775[] = {{&S4904[29], 3}, {NULL}};
const Segment SGI246776[] = {{&S4904[67], 8}, {NULL}};
const Segment SGI246777[] = {{&S93177[72], 9}, {NULL}};
const Segment SGI246778[] = {{&S4904[116], 3}, {NULL}};
const Segment SGI246779[] = {{&S4904[127], 9}, {NULL}};
const Segment SGI246780[] = {{&S4905[13], 4}, {NULL}};
const Segment SGI246781[] = {{&S4905[103], 5}, {NULL}};
const Segment SGI246782[] = {{&S4905[29], 3}, {NULL}};
const Segment SGI246783[] = {{&S4905[67], 8}, {NULL}};
const Segment SGI246784[] = {{&S4905[116], 3}, {NULL}};
const Segment SGI246785[] = {{&S4905[127], 9}, {NULL}};
const Segment SGI246786[] = {{&S4906[13], 4}, {NULL}};
const Segment SGI246787[] = {{&S4906[103], 5}, {NULL}};
const Segment SGI246788[] = {{&S4906[29], 3}, {NULL}};
const Segment SGI246789[] = {{&S4906[67], 8}, {NULL}};
const Segment SGI246790[] = {{&S4906[116], 3}, {NULL}};
const Segment SGI246791[] = {{&S4906[127], 9}, {NULL}};
const Segment SGI246792[] = {{&S4907[13], 4}, {NULL}};
const Segment SGI246793[] = {{&S4907[103], 5}, {NULL}};
const Segment SGI246794[] = {{&S4907[29], 3}, {NULL}};
const Segment SGI246795[] = {{&S4907[67], 8}, {NULL}};
const Segment SGI246796[] = {{&S4907[116], 3}, {NULL}};
const Segment SGI246797[] = {{&S4907[127], 9}, {NULL}};
const Segment SGI246798[] = {{&S4908[13], 4}, {&S4908[13], 4}, {NULL}};
const Segment SGI246799[] = {{&S4908[111], 5}, {NULL}};
const Segment SGI246800[] = {{&S4908[34], 5}, {NULL}};
const Segment SGI246801[] = {{&S4908[74], 8}, {NULL}};
const Segment SGI246802[] = {{&S4908[124], 3}, {NULL}};
const Segment SGI246803[] = {{&S4908[135], 9}, {NULL}};
const Segment SGI246804[] = {{&S4909[13], 4}, {NULL}};
const Segment SGI246805[] = {{&S4909[104], 4}, {NULL}};
const Segment SGI246806[] = {{&S4909[29], 3}, {NULL}};
const Segment SGI246807[] = {{&S4909[67], 8}, {NULL}};
const Segment SGI246808[] = {{&S4909[129], 3}, {NULL}};
const Segment SGI246809[] = {{&S4909[140], 9}, {NULL}};
const Segment SGI246810[] = {{&S93186[34], 7}, {NULL}};
const Segment SGI246811[] = {{&S93186[27], 6}, {NULL}};
const Segment SGI246812[] = {{&S93186[94], 9}, {NULL}};
const Segment SGI246813[] = {{&S93186[67], 6}, {NULL}};
const Segment SGI246814[] = {{&S93206[44], 10}, {NULL}};
const Segment SGI246815[] = {{&S93206[44], 21}, {NULL}};
const Segment SGI246816[] = {{&S93206[30], 6}, {NULL}};
const Segment SGI246817[] = {{&S93206[114], 3}, {NULL}};
const Segment SGI246818[] = {{&S93206[79], 6}, {NULL}};
const Segment SGI246819[] = {{&S93206[138], 10}, {NULL}};
const Segment SGI246820[] = {{&S93246[34], 8}, {NULL}};
const Segment SGI246821[] = {{&S93246[27], 6}, {NULL}};
const Segment SGI246822[] = {{&S93246[94], 9}, {NULL}};
const Segment SGI246823[] = {{&S93246[67], 6}, {NULL}};
const Segment SGI246824[] = {{&S93246[128], 8}, {NULL}};
const Segment SGI246825[] = {{&S93252[34], 10}, {NULL}};
const Segment SGI246826[] = {{&S93252[27], 6}, {NULL}};
const Segment SGI246827[] = {{&S93252[96], 9}, {NULL}};
const Segment SGI246828[] = {{&S93252[69], 6}, {NULL}};
const Segment SGI246829[] = {{&S93253[34], 10}, {NULL}};
const Segment SGI246830[] = {{&S93253[27], 6}, {NULL}};
const Segment SGI246831[] = {{&S93253[96], 9}, {NULL}};
const Segment SGI246832[] = {{&S93253[69], 6}, {NULL}};
const Segment SGI246833[] = {{&S93372[34], 10}, {NULL}};
const Segment SGI246834[] = {{&S93372[27], 6}, {NULL}};
const Segment SGI246835[] = {{&S93372[96], 9}, {NULL}};
const Segment SGI246836[] = {{&S93372[69], 6}, {NULL}};
const Segment SGI246837[] = {{&S100333[34], 9}, {NULL}};
const Segment SGI246838[] = {{&S100333[27], 6}, {NULL}};
const Segment SGI246839[] = {{&S100333[95], 9}, {NULL}};
const Segment SGI246840[] = {{&S100333[68], 6}, {NULL}};
const Segment SGI246841[] = {{&S101576[34], 10}, {NULL}};
const Segment SGI246842[] = {{&S101576[27], 6}, {NULL}};
const Segment SGI246843[] = {{&S101576[97], 9}, {NULL}};
const Segment SGI246844[] = {{&S101576[70], 6}, {NULL}};
const Segment SGI246845[] = {{&S100338[44], 8}, {NULL}};
const Segment SGI246846[] = {{&S100338[44], 20}, {NULL}};
const Segment SGI246847[] = {{&S100338[30], 6}, {NULL}};
const Segment SGI246848[] = {{&S100338[129], 3}, {NULL}};
const Segment SGI246849[] = {{&S100338[94], 6}, {NULL}};
const Segment SGI246850[] = {{&S100338[66], 8}, {NULL}};
const Segment SGI246851[] = {{&S93381[34], 10}, {NULL}};
const Segment SGI246852[] = {{&S93381[27], 6}, {NULL}};
const Segment SGI246853[] = {{&S93381[96], 9}, {NULL}};
const Segment SGI246854[] = {{&S93381[69], 6}, {NULL}};
const Segment SGI246855[] = {{&S100356[44], 17}, {NULL}};
const Segment SGI246856[] = {{&S100356[44], 25}, {NULL}};
const Segment SGI246857[] = {{&S100356[30], 6}, {NULL}};
const Segment SGI246858[] = {{&S100356[134], 3}, {NULL}};
const Segment SGI246859[] = {{&S100356[99], 6}, {NULL}};
const Segment SGI246860[] = {{&S100356[71], 8}, {NULL}};
const Segment SGI246861[] = {{&S100357[34], 8}, {NULL}};
const Segment SGI246862[] = {{&S100357[27], 6}, {NULL}};
const Segment SGI246863[] = {{&S100357[101], 9}, {NULL}};
const Segment SGI246864[] = {{&S100357[74], 6}, {NULL}};
const Segment SGI246865[] = {{&S100360[34], 8}, {NULL}};
const Segment SGI246866[] = {{&S100360[27], 6}, {NULL}};
const Segment SGI246867[] = {{&S100360[95], 9}, {NULL}};
const Segment SGI246868[] = {{&S100360[68], 6}, {NULL}};
const Segment SGI246869[] = {{&S100364[44], 10}, {NULL}};
const Segment SGI246870[] = {{&S100364[44], 22}, {NULL}};
const Segment SGI246871[] = {{&S100364[30], 6}, {NULL}};
const Segment SGI246872[] = {{&S100364[115], 3}, {NULL}};
const Segment SGI246873[] = {{&S100364[80], 6}, {NULL}};
const Segment SGI246874[] = {{&S100364[143], 3}, {NULL}};
const Segment SGI246875[] = {{&S93413[34], 12}, {NULL}};
const Segment SGI246876[] = {{&S93413[27], 6}, {NULL}};
const Segment SGI246877[] = {{&S93413[99], 9}, {NULL}};
const Segment SGI246878[] = {{&S93413[72], 6}, {NULL}};
const Segment SGI246879[] = {{&S100367[34], 10}, {NULL}};
const Segment SGI246880[] = {{&S100367[27], 6}, {NULL}};
const Segment SGI246881[] = {{&S100367[97], 9}, {NULL}};
const Segment SGI246882[] = {{&S100367[70], 6}, {NULL}};
const Segment SGI246883[] = {{&S93416[34], 18}, {NULL}};
const Segment SGI246884[] = {{&S93416[27], 6}, {NULL}};
const Segment SGI246885[] = {{&S93416[105], 9}, {NULL}};
const Segment SGI246886[] = {{&S93416[78], 6}, {NULL}};
const Segment SGI246887[] = {{&S93419[34], 9}, {NULL}};
const Segment SGI246888[] = {{&S93419[27], 6}, {NULL}};
const Segment SGI246889[] = {{&S93419[95], 9}, {NULL}};
const Segment SGI246890[] = {{&S93419[68], 6}, {NULL}};
const Segment SGI246891[] = {{&S93419[136], 8}, {NULL}};
const Segment SGI246892[] = {{&S100373[34], 11}, {NULL}};
const Segment SGI246893[] = {{&S100373[27], 6}, {NULL}};
const Segment SGI246894[] = {{&S100373[98], 9}, {NULL}};
const Segment SGI246895[] = {{&S100373[71], 6}, {NULL}};
const Segment SGI246896[] = {{&S100374[34], 10}, {NULL}};
const Segment SGI246897[] = {{&S100374[27], 6}, {NULL}};
const Segment SGI246898[] = {{&S93422[34], 12}, {NULL}};
const Segment SGI246899[] = {{&S93422[27], 6}, {NULL}};
const Segment SGI246900[] = {{&S100374[97], 9}, {NULL}};
const Segment SGI246901[] = {{&S100374[70], 6}, {NULL}};
const Segment SGI246902[] = {{&S93422[98], 9}, {NULL}};
const Segment SGI246903[] = {{&S93422[71], 6}, {NULL}};
const Segment SGI246904[] = {{&S93425[34], 12}, {NULL}};
const Segment SGI246905[] = {{&S93425[27], 6}, {NULL}};
const Segment SGI246906[] = {{&S93425[99], 9}, {NULL}};
const Segment SGI246907[] = {{&S93425[72], 6}, {NULL}};
const Segment SGI246908[] = {{&S93427[44], 10}, {NULL}};
const Segment SGI246909[] = {{&S93427[44], 21}, {NULL}};
const Segment SGI246910[] = {{&S93427[30], 6}, {NULL}};
const Segment SGI246911[] = {{&S100381[34], 6}, {NULL}};
const Segment SGI246912[] = {{&S100381[27], 6}, {NULL}};
const Segment SGI246913[] = {{&S93427[132], 3}, {NULL}};
const Segment SGI246914[] = {{&S93427[97], 6}, {NULL}};
const Segment SGI246915[] = {{&S93427[67], 8}, {NULL}};
const Segment SGI246916[] = {{&S100381[93], 9}, {NULL}};
const Segment SGI246917[] = {{&S100381[66], 6}, {NULL}};
const Segment SGI246918[] = {{&S100384[34], 9}, {NULL}};
const Segment SGI246919[] = {{&S100384[27], 6}, {NULL}};
const Segment SGI246920[] = {{&S100384[96], 9}, {NULL}};
const Segment SGI246921[] = {{&S100384[69], 6}, {NULL}};
const Segment SGI246922[] = {{&S100385[34], 10}, {NULL}};
const Segment SGI246923[] = {{&S100385[27], 6}, {NULL}};
const Segment SGI246924[] = {{&S100385[97], 9}, {NULL}};
const Segment SGI246925[] = {{&S100385[70], 6}, {NULL}};
const Segment SGI246926[] = {{&S100385[138], 8}, {NULL}};
const Segment SGI246927[] = {{&S93439[44], 11}, {NULL}};
const Segment SGI246928[] = {{&S93439[44], 22}, {NULL}};
const Segment SGI246929[] = {{&S93439[30], 6}, {NULL}};
const Segment SGI246930[] = {{&S93439[115], 3}, {NULL}};
const Segment SGI246931[] = {{&S93439[80], 6}, {NULL}};
const Segment SGI246932[] = {{&S93439[139], 10}, {NULL}};
const Segment SGI246933[] = {{&S100397[34], 10}, {NULL}};
const Segment SGI246934[] = {{&S100397[27], 6}, {NULL}};
const Segment SGI246935[] = {{&S100397[97], 9}, {NULL}};
const Segment SGI246936[] = {{&S100397[70], 6}, {NULL}};
const Segment SGI246937[] = {{&S100397[138], 8}, {NULL}};
const Segment SGI246938[] = {{&S93455[34], 10}, {NULL}};
const Segment SGI246939[] = {{&S93455[27], 6}, {NULL}};
const Segment SGI246940[] = {{&S93455[97], 9}, {NULL}};
const Segment SGI246941[] = {{&S93455[70], 6}, {NULL}};
const Segment SGI246942[] = {{&S93460[34], 10}, {NULL}};
const Segment SGI246943[] = {{&S93460[27], 6}, {NULL}};
const Segment SGI246944[] = {{&S93460[97], 9}, {NULL}};
const Segment SGI246945[] = {{&S93460[70], 6}, {NULL}};
const Segment SGI246946[] = {{&S100417[34], 11}, {NULL}};
const Segment SGI246947[] = {{&S100417[27], 6}, {NULL}};
const Segment SGI246948[] = {{&S100417[98], 9}, {NULL}};
const Segment SGI246949[] = {{&S100417[71], 6}, {NULL}};
const Segment SGI246950[] = {{&S100424[61], 15}, {NULL}};
const Segment SGI246951[] = {{&S100424[61], 27}, {NULL}};
const Segment SGI246952[] = {{&S100424[47], 6}, {NULL}};
const Segment SGI246953[] = {{&S100424[137], 3}, {NULL}};
const Segment SGI246954[] = {{&S93471[44], 10}, {NULL}};
const Segment SGI246955[] = {{&S100424[102], 6}, {NULL}};
const Segment SGI246956[] = {{&S93471[44], 21}, {NULL}};
const Segment SGI246957[] = {{&S93471[30], 6}, {NULL}};
const Segment SGI246958[] = {{&S100424[11], 3}, {NULL}};
const Segment SGI246959[] = {{&S93471[114], 3}, {NULL}};
const Segment SGI246960[] = {{&S93471[79], 6}, {NULL}};
const Segment SGI246961[] = {{&S93475[34], 10}, {NULL}};
const Segment SGI246962[] = {{&S93475[27], 6}, {NULL}};
const Segment SGI246963[] = {{&S93475[96], 9}, {NULL}};
const Segment SGI246964[] = {{&S93475[69], 6}, {NULL}};
const Segment SGI246965[] = {{&S93475[130], 8}, {NULL}};
const Segment SGI246966[] = {{&S100431[34], 10}, {NULL}};
const Segment SGI246967[] = {{&S100431[27], 6}, {NULL}};
const Segment SGI246968[] = {{&S100431[97], 9}, {NULL}};
const Segment SGI246969[] = {{&S100431[70], 6}, {NULL}};
const Segment SGI246970[] = {{&S93479[41], 84}, {NULL}};
const Segment SGI246971[] = {{&S93479[41], 95}, {NULL}};
const Segment SGI246972[] = {{&S93479[27], 6}, {NULL}};
const Segment SGI246973[] = {{&S100434[34], 18}, {NULL}};
const Segment SGI246974[] = {{&S100434[27], 6}, {NULL}};
const Segment SGI246975[] = {{&S93479[62], 3}, {NULL}};
const Segment SGI246976[] = {{&S100434[105], 9}, {NULL}};
const Segment SGI246977[] = {{&S93479[190], 9}, {NULL}};
const Segment SGI246978[] = {{&S100434[78], 6}, {NULL}};
const Segment SGI246979[] = {{&S93479[118], 6}, {NULL}};
const Segment SGI246980[] = {{&S93485[44], 18}, {NULL}};
const Segment SGI246981[] = {{&S93485[44], 29}, {NULL}};
const Segment SGI246982[] = {{&S93485[30], 6}, {NULL}};
const Segment SGI246983[] = {{&S93485[87], 6}, {NULL}};
const Segment SGI246984[] = {{&S93485[114], 13}, {NULL}};
const Segment SGI246985[] = {{&S100446[44], 10}, {NULL}};
const Segment SGI246986[] = {{&S100446[44], 22}, {NULL}};
const Segment SGI246987[] = {{&S100446[30], 6}, {NULL}};
const Segment SGI246988[] = {{&S100446[113], 3}, {NULL}};
const Segment SGI246989[] = {{&S100446[80], 5}, {NULL}};
const Segment SGI246990[] = {{&S100446[117], 14}, {NULL}};
const Segment SGI246991[] = {{&S100448[44], 10}, {NULL}};
const Segment SGI246992[] = {{&S100448[44], 22}, {NULL}};
const Segment SGI246993[] = {{&S100448[30], 6}, {NULL}};
const Segment SGI246994[] = {{&S100448[80], 6}, {NULL}};
const Segment SGI246995[] = {{&S100448[107], 13}, {NULL}};
const Segment SGI246996[] = {{&S100449[34], 18}, {NULL}};
const Segment SGI246997[] = {{&S100449[27], 6}, {NULL}};
const Segment SGI246998[] = {{&S100449[105], 9}, {NULL}};
const Segment SGI246999[] = {{&S100449[78], 6}, {NULL}};
const Segment SGI247000[] = {{&S100449[146], 8}, {NULL}};
const Segment SGI247001[] = {{&S5214[13], 6}, {NULL}};
const Segment SGI247002[] = {{&S5214[35], 3}, {NULL}};
const Segment SGI247003[] = {{&S5214[66], 6}, {NULL}};
const Segment SGI247004[] = {{&S5214[100], 8}, {NULL}};
const Segment SGI247005[] = {{&S5215[13], 6}, {NULL}};
const Segment SGI247006[] = {{&S5215[35], 3}, {NULL}};
const Segment SGI247007[] = {{&S5215[66], 6}, {NULL}};
const Segment SGI247008[] = {{&S5216[13], 6}, {NULL}};
const Segment SGI247009[] = {{&S5216[35], 3}, {NULL}};
const Segment SGI247010[] = {{&S5216[66], 6}, {NULL}};
const Segment SGI247011[] = {{&S5216[100], 6}, {NULL}};
const Segment SGI247012[] = {{&S5217[13], 6}, {NULL}};
const Segment SGI247013[] = {{&S5217[35], 3}, {NULL}};
const Segment SGI247014[] = {{&S5217[66], 6}, {NULL}};
const Segment SGI247015[] = {{&S5217[100], 6}, {NULL}};
const Segment SGI247016[] = {{&S5218[13], 6}, {NULL}};
const Segment SGI247017[] = {{&S5218[35], 3}, {NULL}};
const Segment SGI247018[] = {{&S5218[66], 6}, {NULL}};
const Segment SGI247019[] = {{&S5218[100], 8}, {NULL}};
const Segment SGI247020[] = {{&S5219[13], 6}, {&S5219[13], 6}, {NULL}};
const Segment SGI247021[] = {{&S5219[73], 6}, {NULL}};
const Segment SGI247022[] = {{&S5219[107], 4}, {NULL}};
const Segment SGI247023[] = {{&S5220[13], 6}, {NULL}};
const Segment SGI247024[] = {{&S5220[35], 3}, {NULL}};
const Segment SGI247025[] = {{&S5220[66], 6}, {NULL}};
const Segment SGI247026[] = {{&S100460[44], 9}, {NULL}};
const Segment SGI247027[] = {{&S100460[44], 21}, {NULL}};
const Segment SGI247028[] = {{&S100460[30], 6}, {NULL}};
const Segment SGI247029[] = {{&S100460[145], 3}, {NULL}};
const Segment SGI247030[] = {{&S5221[13], 6}, {NULL}};
const Segment SGI247031[] = {{&S5221[35], 3}, {NULL}};
const Segment SGI247032[] = {{&S100460[110], 6}, {NULL}};
const Segment SGI247033[] = {{&S5221[66], 6}, {NULL}};
const Segment SGI247034[] = {{&S5221[100], 8}, {NULL}};
const Segment SGI247035[] = {{&S100460[67], 8}, {NULL}};
const Segment SGI247036[] = {{&S5222[13], 6}, {NULL}};
const Segment SGI247037[] = {{&S5222[35], 3}, {NULL}};
const Segment SGI247038[] = {{&S5222[66], 6}, {NULL}};
const Segment SGI247039[] = {{&S5222[100], 6}, {NULL}};
const Segment SGI247040[] = {{&S5223[13], 6}, {NULL}};
const Segment SGI247041[] = {{&S5223[35], 3}, {NULL}};
const Segment SGI247042[] = {{&S5223[66], 6}, {NULL}};
const Segment SGI247043[] = {{&S5223[100], 8}, {NULL}};
const Segment SGI247044[] = {{&S5224[13], 6}, {NULL}};
const Segment SGI247045[] = {{&S5224[35], 3}, {NULL}};
const Segment SGI247046[] = {{&S5224[66], 8}, {NULL}};
const Segment SGI247047[] = {{&S5225[13], 6}, {NULL}};
const Segment SGI247048[] = {{&S5225[35], 3}, {NULL}};
const Segment SGI247049[] = {{&S5225[66], 8}, {NULL}};
const Segment SGI247050[] = {{&S5225[102], 8}, {NULL}};
const Segment SGI247051[] = {{&S5226[13], 6}, {NULL}};
const Segment SGI247052[] = {{&S5226[35], 3}, {NULL}};
const Segment SGI247053[] = {{&S5226[66], 8}, {NULL}};
const Segment SGI247054[] = {{&S5226[102], 6}, {NULL}};
const Segment SGI247055[] = {{&S5227[13], 6}, {NULL}};
const Segment SGI247056[] = {{&S5227[35], 3}, {NULL}};
const Segment SGI247057[] = {{&S5227[66], 8}, {NULL}};
const Segment SGI247058[] = {{&S5227[102], 8}, {NULL}};
const Segment SGI247059[] = {{&S5228[13], 6}, {NULL}};
const Segment SGI247060[] = {{&S5228[35], 3}, {NULL}};
const Segment SGI247061[] = {{&S5228[66], 8}, {NULL}};
const Segment SGI247062[] = {{&S5228[102], 6}, {NULL}};
const Segment SGI247063[] = {{&S5229[13], 6}, {NULL}};
const Segment SGI247064[] = {{&S5229[35], 3}, {NULL}};
const Segment SGI247065[] = {{&S5229[66], 8}, {NULL}};
const Segment SGI247066[] = {{&S5229[102], 8}, {NULL}};
const Segment SGI247067[] = {{&S5230[13], 6}, {NULL}};
const Segment SGI247068[] = {{&S5230[35], 3}, {NULL}};
const Segment SGI247069[] = {{&S5230[66], 8}, {NULL}};
const Segment SGI247070[] = {{&S5230[102], 8}, {NULL}};
const Segment SGI247071[] = {{&S5231[13], 6}, {NULL}};
const Segment SGI247072[] = {{&S5231[35], 3}, {NULL}};
const Segment SGI247073[] = {{&S5231[66], 8}, {NULL}};
const Segment SGI247074[] = {{&S5231[102], 6}, {NULL}};
const Segment SGI247075[] = {{&S5232[13], 6}, {NULL}};
const Segment SGI247076[] = {{&S5232[35], 3}, {NULL}};
const Segment SGI247077[] = {{&S5232[66], 8}, {NULL}};
const Segment SGI247078[] = {{&S5232[102], 6}, {NULL}};
const Segment SGI247079[] = {{&S5233[13], 6}, {NULL}};
const Segment SGI247080[] = {{&S5233[35], 3}, {NULL}};
const Segment SGI247081[] = {{&S5233[66], 8}, {NULL}};
const Segment SGI247082[] = {{&S5233[102], 6}, {NULL}};
const Segment SGI247083[] = {{&S5234[13], 6}, {NULL}};
const Segment SGI247084[] = {{&S5234[35], 3}, {NULL}};
const Segment SGI247085[] = {{&S5234[66], 8}, {NULL}};
const Segment SGI247086[] = {{&S5234[102], 6}, {NULL}};
const Segment SGI247087[] = {{&S5235[13], 6}, {NULL}};
const Segment SGI247088[] = {{&S5235[35], 3}, {NULL}};
const Segment SGI247089[] = {{&S5235[66], 8}, {NULL}};
const Segment SGI247090[] = {{&S5235[102], 6}, {NULL}};
const Segment SGI247091[] = {{&S5236[13], 6}, {NULL}};
const Segment SGI247092[] = {{&S5236[35], 5}, {NULL}};
const Segment SGI247093[] = {{&S5236[68], 8}, {NULL}};
const Segment SGI247094[] = {{&S5236[104], 7}, {NULL}};
const Segment SGI247095[] = {{&S5237[13], 6}, {NULL}};
const Segment SGI247096[] = {{&S5237[42], 3}, {NULL}};
const Segment SGI247097[] = {{&S5237[123], 5}, {NULL}};
const Segment SGI247098[] = {{&S5238[13], 6}, {NULL}};
const Segment SGI247099[] = {{&S5238[42], 3}, {NULL}};
const Segment SGI247100[] = {{&S5238[123], 5}, {NULL}};
const Segment SGI247101[] = {{&S5239[13], 6}, {NULL}};
const Segment SGI247102[] = {{&S5239[42], 3}, {NULL}};
const Segment SGI247103[] = {{&S5239[123], 5}, {NULL}};
const Segment SGI247104[] = {{&S5240[13], 6}, {NULL}};
const Segment SGI247105[] = {{&S5240[35], 5}, {NULL}};
const Segment SGI247106[] = {{&S5240[68], 8}, {NULL}};
const Segment SGI247107[] = {{&S5241[13], 6}, {NULL}};
const Segment SGI247108[] = {{&S5241[35], 5}, {NULL}};
const Segment SGI247109[] = {{&S5241[68], 8}, {NULL}};
const Segment SGI247110[] = {{&S5242[13], 6}, {NULL}};
const Segment SGI247111[] = {{&S5242[38], 3}, {NULL}};
const Segment SGI247112[] = {{&S5243[13], 6}, {&S5243[13], 6}, {NULL}};
const Segment SGI247113[] = {{&S5243[35], 3}, {&S5243[131], 3}, {NULL}};
const Segment SGI247114[] = {{&S5243[66], 8}, {NULL}};
const Segment SGI247115[] = {{&S5243[102], 6}, {NULL}};
const Segment SGI247116[] = {{&S5244[13], 6}, {NULL}};
const Segment SGI247117[] = {{&S5244[35], 3}, {NULL}};
const Segment SGI247118[] = {{&S5244[66], 8}, {NULL}};
const Segment SGI247119[] = {{&S5244[102], 6}, {NULL}};
const Segment SGI247120[] = {{&S5244[116], 9}, {NULL}};
const Segment SGI247121[] = {{&S5245[13], 6}, {NULL}};
const Segment SGI247122[] = {{&S5245[35], 5}, {NULL}};
const Segment SGI247123[] = {{&S5245[68], 8}, {NULL}};
const Segment SGI247124[] = {{&S5245[104], 6}, {NULL}};
const Segment SGI247125[] = {{&S5246[13], 6}, {NULL}};
const Segment SGI247126[] = {{&S5246[35], 5}, {NULL}};
const Segment SGI247127[] = {{&S5246[68], 8}, {NULL}};
const Segment SGI247128[] = {{&S5246[104], 6}, {NULL}};
const Segment SGI247129[] = {{&S5247[13], 6}, {NULL}};
const Segment SGI247130[] = {{&S5247[42], 5}, {NULL}};
const Segment SGI247131[] = {{&S5247[125], 5}, {NULL}};
const Segment SGI247132[] = {{&S5248[13], 6}, {NULL}};
const Segment SGI247133[] = {{&S5248[42], 5}, {NULL}};
const Segment SGI247134[] = {{&S5248[125], 5}, {NULL}};
const Segment SGI247135[] = {{&S5249[13], 6}, {NULL}};
const Segment SGI247136[] = {{&S5249[42], 5}, {NULL}};
const Segment SGI247137[] = {{&S5249[125], 5}, {NULL}};
const Segment SGI247138[] = {{&S93613[34], 10}, {NULL}};
const Segment SGI247139[] = {{&S93613[27], 6}, {NULL}};
const Segment SGI247140[] = {{&S93613[96], 9}, {NULL}};
const Segment SGI247141[] = {{&S93613[69], 6}, {NULL}};
const Segment SGI247142[] = {{&S93620[44], 11}, {NULL}};
const Segment SGI247143[] = {{&S93620[44], 22}, {NULL}};
const Segment SGI247144[] = {{&S93620[30], 6}, {NULL}};
const Segment SGI247145[] = {{&S93620[144], 3}, {NULL}};
const Segment SGI247146[] = {{&S93620[109], 6}, {NULL}};
const Segment SGI247147[] = {{&S5323[13], 4}, {NULL}};
const Segment SGI247148[] = {{&S5323[26], 3}, {NULL}};
const Segment SGI247149[] = {{&S5323[57], 6}, {NULL}};
const Segment SGI247150[] = {{&S5323[91], 7}, {NULL}};
const Segment SGI247151[] = {{&S5324[13], 4}, {&S5324[13], 4}, {&S5324[13], 4}, {NULL}};
const Segment SGI247152[] = {{&S5324[36], 3}, {NULL}};
const Segment SGI247153[] = {{&S5325[13], 4}, {&S5325[13], 4}, {NULL}};
const Segment SGI247154[] = {{&S5325[26], 3}, {NULL}};
const Segment SGI247155[] = {{&S5325[57], 6}, {NULL}};
const Segment SGI247156[] = {{&S5325[91], 5}, {NULL}};
const Segment SGI247157[] = {{&S5326[13], 4}, {&S5326[13], 4}, {&S5326[13], 4}, {NULL}};
const Segment SGI247158[] = {{&S5326[36], 5}, {NULL}};
const Segment SGI247159[] = {{&S93658[44], 9}, {NULL}};
const Segment SGI247160[] = {{&S93658[44], 20}, {NULL}};
const Segment SGI247161[] = {{&S93658[30], 6}, {NULL}};
const Segment SGI247162[] = {{&S93658[131], 3}, {NULL}};
const Segment SGI247163[] = {{&S93658[96], 6}, {NULL}};
const Segment SGI247164[] = {{&S93658[66], 8}, {NULL}};
const Segment SGI247165[] = {{&S93659[44], 9}, {NULL}};
const Segment SGI247166[] = {{&S93659[44], 21}, {NULL}};
const Segment SGI247167[] = {{&S93659[30], 6}, {NULL}};
const Segment SGI247168[] = {{&S93659[114], 3}, {NULL}};
const Segment SGI247169[] = {{&S93659[79], 6}, {NULL}};
const Segment SGI247170[] = {{&S93659[139], 8}, {NULL}};
const Segment SGI247171[] = {{&S93661[44], 12}, {NULL}};
const Segment SGI247172[] = {{&S93661[44], 23}, {NULL}};
const Segment SGI247173[] = {{&S93661[30], 6}, {NULL}};
const Segment SGI247174[] = {{&S93661[145], 3}, {NULL}};
const Segment SGI247175[] = {{&S93661[110], 6}, {NULL}};
const Segment SGI247176[] = {{&S100491[34], 11}, {NULL}};
const Segment SGI247177[] = {{&S100491[27], 6}, {NULL}};
const Segment SGI247178[] = {{&S100491[98], 9}, {NULL}};
const Segment SGI247179[] = {{&S100491[71], 6}, {NULL}};
const Segment SGI247180[] = {{&S100495[44], 9}, {NULL}};
const Segment SGI247181[] = {{&S100495[44], 21}, {NULL}};
const Segment SGI247182[] = {{&S100495[30], 6}, {NULL}};
const Segment SGI247183[] = {{&S100495[114], 3}, {NULL}};
const Segment SGI247184[] = {{&S5327[13], 4}, {NULL}};
const Segment SGI247185[] = {{&S100495[79], 6}, {NULL}};
const Segment SGI247186[] = {{&S5327[26], 3}, {NULL}};
const Segment SGI247187[] = {{&S5327[57], 6}, {NULL}};
const Segment SGI247188[] = {{&S100495[149], 3}, {NULL}};
const Segment SGI247189[] = {{&S5327[91], 5}, {NULL}};
const Segment SGI247190[] = {{&S5328[13], 4}, {NULL}};
const Segment SGI247191[] = {{&S5328[26], 5}, {NULL}};
const Segment SGI247192[] = {{&S5328[59], 6}, {NULL}};
const Segment SGI247193[] = {{&S5328[93], 5}, {NULL}};
const Segment SGI247194[] = {{&S5329[13], 4}, {NULL}};
const Segment SGI247195[] = {{&S100497[44], 10}, {NULL}};
const Segment SGI247196[] = {{&S5329[26], 3}, {NULL}};
const Segment SGI247197[] = {{&S100497[44], 22}, {NULL}};
const Segment SGI247198[] = {{&S5329[57], 6}, {NULL}};
const Segment SGI247199[] = {{&S100497[30], 6}, {NULL}};
const Segment SGI247200[] = {{&S5329[91], 4}, {NULL}};
const Segment SGI247201[] = {{&S100497[115], 3}, {NULL}};
const Segment SGI247202[] = {{&S100497[80], 6}, {NULL}};
const Segment SGI247203[] = {{&S100497[143], 3}, {NULL}};
const Segment SGI247204[] = {{&S5330[13], 4}, {NULL}};
const Segment SGI247205[] = {{&S5330[26], 5}, {NULL}};
const Segment SGI247206[] = {{&S5330[59], 6}, {NULL}};
const Segment SGI247207[] = {{&S5331[13], 4}, {NULL}};
const Segment SGI247208[] = {{&S5331[26], 3}, {NULL}};
const Segment SGI247209[] = {{&S5331[57], 6}, {NULL}};
const Segment SGI247210[] = {{&S5332[13], 4}, {&S5332[13], 4}, {NULL}};
const Segment SGI247211[] = {{&S5332[26], 5}, {NULL}};
const Segment SGI247212[] = {{&S5332[59], 6}, {NULL}};
const Segment SGI247213[] = {{&S5332[93], 5}, {NULL}};
const Segment SGI247214[] = {{&S5333[27], 4}, {NULL}};
const Segment SGI247215[] = {{&S5333[40], 5}, {NULL}};
const Segment SGI247216[] = {{&S5333[73], 6}, {NULL}};
const Segment SGI247217[] = {{&S5333[107], 5}, {NULL}};
const Segment SGI247218[] = {{&S5333[120], 9}, {NULL}};
const Segment SGI247219[] = {{&S5333[0], 14}, {NULL}};
const Segment SGI247220[] = {{&S5334[13], 4}, {NULL}};
const Segment SGI247221[] = {{&S5334[26], 3}, {NULL}};
const Segment SGI247222[] = {{&S5334[57], 6}, {NULL}};
const Segment SGI247223[] = {{&S5334[91], 5}, {NULL}};
const Segment SGI247224[] = {{&S93684[34], 10}, {NULL}};
const Segment SGI247225[] = {{&S93684[27], 6}, {NULL}};
const Segment SGI247226[] = {{&S93684[97], 9}, {NULL}};
const Segment SGI247227[] = {{&S93684[70], 6}, {NULL}};
const Segment SGI247228[] = {{&S5335[13], 4}, {NULL}};
const Segment SGI247229[] = {{&S5335[26], 5}, {NULL}};
const Segment SGI247230[] = {{&S5335[59], 6}, {NULL}};
const Segment SGI247231[] = {{&S5335[93], 5}, {NULL}};
const Segment SGI247232[] = {{&S5335[106], 6}, {NULL}};
const Segment SGI247233[] = {{&S5336[13], 4}, {NULL}};
const Segment SGI247234[] = {{&S5336[26], 3}, {NULL}};
const Segment SGI247235[] = {{&S5336[57], 6}, {NULL}};
const Segment SGI247236[] = {{&S5336[91], 5}, {NULL}};
const Segment SGI247237[] = {{&S5336[104], 6}, {NULL}};
const Segment SGI247238[] = {{&S5337[13], 4}, {NULL}};
const Segment SGI247239[] = {{&S5337[26], 5}, {NULL}};
const Segment SGI247240[] = {{&S5337[59], 6}, {NULL}};
const Segment SGI247241[] = {{&S5337[93], 5}, {NULL}};
const Segment SGI247242[] = {{&S5337[106], 9}, {NULL}};
const Segment SGI247243[] = {{&S5338[27], 4}, {NULL}};
const Segment SGI247244[] = {{&S5338[40], 5}, {NULL}};
const Segment SGI247245[] = {{&S93689[34], 10}, {NULL}};
const Segment SGI247246[] = {{&S5338[73], 6}, {NULL}};
const Segment SGI247247[] = {{&S5338[107], 5}, {NULL}};
const Segment SGI247248[] = {{&S93689[27], 6}, {NULL}};
const Segment SGI247249[] = {{&S5338[0], 14}, {NULL}};
const Segment SGI247250[] = {{&S93689[97], 9}, {NULL}};
const Segment SGI247251[] = {{&S93689[70], 6}, {NULL}};
const Segment SGI247252[] = {{&S5339[13], 4}, {NULL}};
const Segment SGI247253[] = {{&S5339[26], 3}, {NULL}};
const Segment SGI247254[] = {{&S5339[57], 6}, {NULL}};
const Segment SGI247255[] = {{&S5339[91], 5}, {NULL}};
const Segment SGI247256[] = {{&S5339[104], 7}, {NULL}};
const Segment SGI247257[] = {{&S5340[13], 4}, {&S5340[13], 4}, {NULL}};
const Segment SGI247258[] = {{&S5340[36], 5}, {NULL}};
const Segment SGI247259[] = {{&S5340[118], 5}, {NULL}};
const Segment SGI247260[] = {{&S5341[30], 4}, {NULL}};
const Segment SGI247261[] = {{&S5341[43], 3}, {NULL}};
const Segment SGI247262[] = {{&S5341[74], 6}, {NULL}};
const Segment SGI247263[] = {{&S5341[108], 5}, {NULL}};
const Segment SGI247264[] = {{&S5341[121], 9}, {NULL}};
const Segment SGI247265[] = {{&S5341[0], 14}, {NULL}};
const Segment SGI247266[] = {{&S5342[13], 4}, {NULL}};
const Segment SGI247267[] = {{&S5342[26], 5}, {NULL}};
const Segment SGI247268[] = {{&S5342[59], 6}, {NULL}};
const Segment SGI247269[] = {{&S5342[93], 5}, {NULL}};
const Segment SGI247270[] = {{&S5343[13], 4}, {&S5343[13], 4}, {NULL}};
const Segment SGI247271[] = {{&S5343[37], 5}, {NULL}};
const Segment SGI247272[] = {{&S5343[119], 4}, {NULL}};
const Segment SGI247273[] = {{&S5344[13], 4}, {&S5344[13], 4}, {&S5344[13], 4}, {NULL}};
const Segment SGI247274[] = {{&S5344[26], 3}, {NULL}};
const Segment SGI247275[] = {{&S5344[57], 6}, {NULL}};
const Segment SGI247276[] = {{&S5344[91], 5}, {NULL}};
const Segment SGI247277[] = {{&S5345[13], 4}, {NULL}};
const Segment SGI247278[] = {{&S5345[26], 3}, {NULL}};
const Segment SGI247279[] = {{&S5345[57], 6}, {NULL}};
const Segment SGI247280[] = {{&S5345[91], 5}, {NULL}};
const Segment SGI247281[] = {{&S5345[104], 7}, {NULL}};
const Segment SGI247282[] = {{&S100523[34], 7}, {NULL}};
const Segment SGI247283[] = {{&S100523[27], 6}, {NULL}};
const Segment SGI247284[] = {{&S100523[94], 9}, {NULL}};
const Segment SGI247285[] = {{&S100523[67], 6}, {NULL}};
const Segment SGI247286[] = {{&S5346[13], 4}, {NULL}};
const Segment SGI247287[] = {{&S5346[26], 5}, {NULL}};
const Segment SGI247288[] = {{&S5346[59], 6}, {NULL}};
const Segment SGI247289[] = {{&S5346[93], 5}, {NULL}};
const Segment SGI247290[] = {{&S5346[106], 9}, {NULL}};
const Segment SGI247291[] = {{&S5347[30], 4}, {NULL}};
const Segment SGI247292[] = {{&S5347[43], 5}, {NULL}};
const Segment SGI247293[] = {{&S5347[76], 6}, {NULL}};
const Segment SGI247294[] = {{&S5347[110], 5}, {NULL}};
const Segment SGI247295[] = {{&S5347[123], 9}, {NULL}};
const Segment SGI247296[] = {{&S5347[0], 14}, {NULL}};
const Segment SGI247297[] = {{&S5348[13], 4}, {NULL}};
const Segment SGI247298[] = {{&S5348[26], 5}, {NULL}};
const Segment SGI247299[] = {{&S5348[59], 6}, {NULL}};
const Segment SGI247300[] = {{&S5348[93], 5}, {NULL}};
const Segment SGI247301[] = {{&S5349[13], 4}, {NULL}};
const Segment SGI247302[] = {{&S5349[26], 5}, {NULL}};
const Segment SGI247303[] = {{&S5349[59], 6}, {NULL}};
const Segment SGI247304[] = {{&S93709[34], 8}, {NULL}};
const Segment SGI247305[] = {{&S5349[93], 5}, {NULL}};
const Segment SGI247306[] = {{&S93709[27], 6}, {NULL}};
const Segment SGI247307[] = {{&S100528[44], 14}, {NULL}};
const Segment SGI247308[] = {{&S100528[44], 26}, {NULL}};
const Segment SGI247309[] = {{&S100528[30], 6}, {NULL}};
const Segment SGI247310[] = {{&S93709[103], 9}, {NULL}};
const Segment SGI247311[] = {{&S93709[76], 6}, {NULL}};
const Segment SGI247312[] = {{&S93709[137], 8}, {NULL}};
const Segment SGI247313[] = {{&S100528[84], 6}, {NULL}};
const Segment SGI247314[] = {{&S5350[13], 4}, {NULL}};
const Segment SGI247315[] = {{&S100528[111], 13}, {NULL}};
const Segment SGI247316[] = {{&S5350[26], 5}, {NULL}};
const Segment SGI247317[] = {{&S5350[59], 6}, {NULL}};
const Segment SGI247318[] = {{&S5350[93], 5}, {NULL}};
const Segment SGI247319[] = {{&S5350[106], 9}, {&S5350[106], 9}, {NULL}};
const Segment SGI247320[] = {{&S5351[13], 4}, {NULL}};
const Segment SGI247321[] = {{&S5351[26], 5}, {NULL}};
const Segment SGI247322[] = {{&S5351[59], 6}, {NULL}};
const Segment SGI247323[] = {{&S5351[93], 5}, {&S5351[28], 1}, {NULL}};
const Segment SGI247324[] = {{&S93711[52], 10}, {NULL}};
const Segment SGI247325[] = {{&S93711[52], 21}, {NULL}};
const Segment SGI247326[] = {{&S93711[38], 6}, {NULL}};
const Segment SGI247327[] = {{&S93711[87], 5}, {NULL}};
const Segment SGI247328[] = {{&S5352[13], 4}, {NULL}};
const Segment SGI247329[] = {{&S5352[26], 3}, {NULL}};
const Segment SGI247330[] = {{&S5352[57], 6}, {NULL}};
const Segment SGI247331[] = {{&S5352[91], 5}, {NULL}};
const Segment SGI247332[] = {{&S5352[104], 9}, {NULL}};
const Segment SGI247333[] = {{&S5353[13], 4}, {&S5353[13], 4}, {&S5353[13], 4}, {NULL}};
const Segment SGI247334[] = {{&S5353[26], 5}, {NULL}};
const Segment SGI247335[] = {{&S5353[59], 6}, {NULL}};
const Segment SGI247336[] = {{&S5353[93], 5}, {NULL}};
const Segment SGI247337[] = {{&S100534[44], 16}, {NULL}};
const Segment SGI247338[] = {{&S100534[44], 35}, {NULL}};
const Segment SGI247339[] = {{&S100534[30], 6}, {NULL}};
const Segment SGI247340[] = {{&S5354[13], 4}, {NULL}};
const Segment SGI247341[] = {{&S5354[26], 5}, {NULL}};
const Segment SGI247342[] = {{&S5354[59], 6}, {NULL}};
const Segment SGI247343[] = {{&S5354[93], 5}, {NULL}};
const Segment SGI247344[] = {{&S100534[126], 3}, {NULL}};
const Segment SGI247345[] = {{&S100534[93], 5}, {NULL}};
const Segment SGI247346[] = {{&S100534[130], 14}, {NULL}};
const Segment SGI247347[] = {{&S5355[15], 4}, {NULL}};
const Segment SGI247348[] = {{&S5355[31], 5}, {NULL}};
const Segment SGI247349[] = {{&S5355[0], 7}, {NULL}};
const Segment SGI247350[] = {{&S5356[15], 4}, {NULL}};
const Segment SGI247351[] = {{&S5356[31], 5}, {NULL}};
const Segment SGI247352[] = {{&S5357[13], 4}, {&S5357[13], 4}, {NULL}};
const Segment SGI247353[] = {{&S5357[38], 5}, {NULL}};
const Segment SGI247354[] = {{&S5357[121], 5}, {NULL}};
const Segment SGI247355[] = {{&S5358[13], 4}, {NULL}};
const Segment SGI247356[] = {{&S5358[26], 3}, {NULL}};
const Segment SGI247357[] = {{&S5358[57], 6}, {NULL}};
const Segment SGI247358[] = {{&S5358[91], 5}, {NULL}};
const Segment SGI247359[] = {{&S5359[13], 4}, {NULL}};
const Segment SGI247360[] = {{&S5359[26], 5}, {NULL}};
const Segment SGI247361[] = {{&S5359[59], 6}, {NULL}};
const Segment SGI247362[] = {{&S5359[93], 5}, {NULL}};
const Segment SGI247363[] = {{&S5359[106], 6}, {NULL}};
const Segment SGI247364[] = {{&S100548[44], 15}, {NULL}};
const Segment SGI247365[] = {{&S100548[44], 27}, {NULL}};
const Segment SGI247366[] = {{&S100548[30], 6}, {NULL}};
const Segment SGI247367[] = {{&S100548[120], 3}, {NULL}};
const Segment SGI247368[] = {{&S100548[85], 6}, {NULL}};
const Segment SGI247369[] = {{&S100553[44], 10}, {NULL}};
const Segment SGI247370[] = {{&S100553[44], 22}, {NULL}};
const Segment SGI247371[] = {{&S100553[30], 6}, {NULL}};
const Segment SGI247372[] = {{&S100553[80], 6}, {NULL}};
const Segment SGI247373[] = {{&S100553[107], 13}, {NULL}};
const Segment SGI247374[] = {{&S100566[44], 7}, {NULL}};
const Segment SGI247375[] = {{&S100566[44], 19}, {NULL}};
const Segment SGI247376[] = {{&S100566[30], 6}, {NULL}};
const Segment SGI247377[] = {{&S100566[112], 3}, {NULL}};
const Segment SGI247378[] = {{&S100566[77], 6}, {NULL}};
const Segment SGI247379[] = {{&S100571[52], 10}, {NULL}};
const Segment SGI247380[] = {{&S100571[52], 22}, {NULL}};
const Segment SGI247381[] = {{&S100571[38], 6}, {NULL}};
const Segment SGI247382[] = {{&S100571[88], 5}, {NULL}};
const Segment SGI247383[] = {{&S100571[11], 3}, {NULL}};
const Segment SGI247384[] = {{&S100575[52], 7}, {NULL}};
const Segment SGI247385[] = {{&S100575[52], 24}, {NULL}};
const Segment SGI247386[] = {{&S100575[38], 6}, {NULL}};
const Segment SGI247387[] = {{&S100575[90], 5}, {NULL}};
const Segment SGI247388[] = {{&S100575[11], 3}, {NULL}};
const Segment SGI247389[] = {{&S100579[44], 10}, {NULL}};
const Segment SGI247390[] = {{&S100579[44], 22}, {NULL}};
const Segment SGI247391[] = {{&S100579[30], 6}, {NULL}};
const Segment SGI247392[] = {{&S100579[80], 6}, {NULL}};
const Segment SGI247393[] = {{&S100579[107], 13}, {NULL}};
const Segment SGI247394[] = {{&S5398[13], 6}, {NULL}};
const Segment SGI247395[] = {{&S5398[101], 3}, {NULL}};
const Segment SGI247396[] = {{&S5398[35], 3}, {NULL}};
const Segment SGI247397[] = {{&S5398[66], 6}, {NULL}};
const Segment SGI247398[] = {{&S5398[112], 8}, {NULL}};
const Segment SGI247399[] = {{&S5398[128], 7}, {NULL}};
const Segment SGI247400[] = {{&S5399[13], 6}, {NULL}};
const Segment SGI247401[] = {{&S5399[101], 3}, {NULL}};
const Segment SGI247402[] = {{&S5399[35], 3}, {NULL}};
const Segment SGI247403[] = {{&S5399[66], 6}, {NULL}};
const Segment SGI247404[] = {{&S5399[112], 6}, {NULL}};
const Segment SGI247405[] = {{&S5399[126], 7}, {NULL}};
const Segment SGI247406[] = {{&S5400[13], 6}, {NULL}};
const Segment SGI247407[] = {{&S5400[101], 3}, {NULL}};
const Segment SGI247408[] = {{&S5400[35], 3}, {NULL}};
const Segment SGI247409[] = {{&S5400[66], 6}, {NULL}};
const Segment SGI247410[] = {{&S5400[112], 8}, {NULL}};
const Segment SGI247411[] = {{&S5400[128], 7}, {NULL}};
const Segment SGI247412[] = {{&S100584[44], 18}, {NULL}};
const Segment SGI247413[] = {{&S100584[44], 30}, {NULL}};
const Segment SGI247414[] = {{&S100584[30], 6}, {NULL}};
const Segment SGI247415[] = {{&S5401[13], 6}, {NULL}};
const Segment SGI247416[] = {{&S5401[101], 3}, {NULL}};
const Segment SGI247417[] = {{&S5401[35], 3}, {NULL}};
const Segment SGI247418[] = {{&S5401[66], 6}, {NULL}};
const Segment SGI247419[] = {{&S5401[112], 8}, {NULL}};
const Segment SGI247420[] = {{&S5401[128], 7}, {NULL}};
const Segment SGI247421[] = {{&S5402[13], 6}, {NULL}};
const Segment SGI247422[] = {{&S5402[101], 3}, {NULL}};
const Segment SGI247423[] = {{&S5402[35], 3}, {NULL}};
const Segment SGI247424[] = {{&S5402[66], 6}, {NULL}};
const Segment SGI247425[] = {{&S5402[112], 6}, {NULL}};
const Segment SGI247426[] = {{&S5402[126], 7}, {NULL}};
const Segment SGI247427[] = {{&S5403[13], 6}, {NULL}};
const Segment SGI247428[] = {{&S5403[101], 3}, {NULL}};
const Segment SGI247429[] = {{&S5403[35], 3}, {NULL}};
const Segment SGI247430[] = {{&S5403[66], 6}, {NULL}};
const Segment SGI247431[] = {{&S5403[112], 6}, {NULL}};
const Segment SGI247432[] = {{&S5403[126], 7}, {NULL}};
const Segment SGI247433[] = {{&S5404[13], 6}, {NULL}};
const Segment SGI247434[] = {{&S5404[101], 3}, {NULL}};
const Segment SGI247435[] = {{&S5404[35], 3}, {NULL}};
const Segment SGI247436[] = {{&S5404[66], 6}, {NULL}};
const Segment SGI247437[] = {{&S5404[112], 6}, {NULL}};
const Segment SGI247438[] = {{&S5404[126], 7}, {NULL}};
const Segment SGI247439[] = {{&S5405[13], 6}, {NULL}};
const Segment SGI247440[] = {{&S5405[101], 3}, {NULL}};
const Segment SGI247441[] = {{&S5405[35], 3}, {NULL}};
const Segment SGI247442[] = {{&S5405[66], 6}, {NULL}};
const Segment SGI247443[] = {{&S5405[112], 6}, {NULL}};
const Segment SGI247444[] = {{&S5405[126], 7}, {NULL}};
const Segment SGI247445[] = {{&S5406[13], 6}, {NULL}};
const Segment SGI247446[] = {{&S5406[101], 3}, {NULL}};
const Segment SGI247447[] = {{&S5406[35], 3}, {NULL}};
const Segment SGI247448[] = {{&S5406[66], 6}, {NULL}};
const Segment SGI247449[] = {{&S5406[112], 6}, {NULL}};
const Segment SGI247450[] = {{&S5406[126], 7}, {NULL}};
const Segment SGI247451[] = {{&S5407[13], 6}, {NULL}};
const Segment SGI247452[] = {{&S5407[101], 3}, {NULL}};
const Segment SGI247453[] = {{&S5407[35], 3}, {NULL}};
const Segment SGI247454[] = {{&S5407[66], 6}, {NULL}};
const Segment SGI247455[] = {{&S5407[112], 6}, {NULL}};
const Segment SGI247456[] = {{&S5407[126], 7}, {NULL}};
const Segment SGI247457[] = {{&S5408[13], 6}, {NULL}};
const Segment SGI247458[] = {{&S5408[101], 3}, {NULL}};
const Segment SGI247459[] = {{&S5408[35], 3}, {NULL}};
const Segment SGI247460[] = {{&S5408[66], 6}, {NULL}};
const Segment SGI247461[] = {{&S5408[112], 6}, {NULL}};
const Segment SGI247462[] = {{&S5408[126], 7}, {NULL}};
const Segment SGI247463[] = {{&S5409[13], 6}, {NULL}};
const Segment SGI247464[] = {{&S5409[101], 3}, {NULL}};
const Segment SGI247465[] = {{&S100584[139], 3}, {NULL}};
const Segment SGI247466[] = {{&S100584[104], 6}, {NULL}};
const Segment SGI247467[] = {{&S100584[76], 8}, {NULL}};
const Segment SGI247468[] = {{&S100584[171], 15}, {NULL}};
const Segment SGI247469[] = {{&S100587[44], 10}, {NULL}};
const Segment SGI247470[] = {{&S100587[44], 22}, {NULL}};
const Segment SGI247471[] = {{&S100587[30], 6}, {NULL}};
const Segment SGI247472[] = {{&S100587[115], 3}, {NULL}};
const Segment SGI247473[] = {{&S100587[80], 6}, {NULL}};
const Segment SGI247474[] = {{&S100587[135], 8}, {NULL}};
const Segment SGI247475[] = {{&S100587[144], 13}, {NULL}};
const Segment SGI247476[] = {{&S100590[34], 7}, {NULL}};
const Segment SGI247477[] = {{&S100590[27], 6}, {NULL}};
const Segment SGI247478[] = {{&S100590[94], 9}, {NULL}};
const Segment SGI247479[] = {{&S100590[67], 6}, {NULL}};
const Segment SGI247480[] = {{&S100595[44], 10}, {NULL}};
const Segment SGI247481[] = {{&S100595[44], 22}, {NULL}};
const Segment SGI247482[] = {{&S100595[30], 6}, {NULL}};
const Segment SGI247483[] = {{&S100595[113], 3}, {NULL}};
const Segment SGI247484[] = {{&S100595[80], 5}, {NULL}};
const Segment SGI247485[] = {{&S100595[117], 14}, {NULL}};
const Segment SGI247486[] = {{&S100597[34], 10}, {NULL}};
const Segment SGI247487[] = {{&S100597[27], 6}, {NULL}};
const Segment SGI247488[] = {{&S100597[97], 9}, {NULL}};
const Segment SGI247489[] = {{&S100597[70], 6}, {NULL}};
const Segment SGI247490[] = {{&S100598[34], 11}, {NULL}};
const Segment SGI247491[] = {{&S100598[27], 6}, {NULL}};
const Segment SGI247492[] = {{&S100598[98], 9}, {NULL}};
const Segment SGI247493[] = {{&S100598[71], 6}, {NULL}};
const Segment SGI247494[] = {{&S100604[34], 10}, {NULL}};
const Segment SGI247495[] = {{&S100604[27], 6}, {NULL}};
const Segment SGI247496[] = {{&S100604[97], 9}, {NULL}};
const Segment SGI247497[] = {{&S100604[70], 6}, {NULL}};
const Segment SGI247498[] = {{&S93840[34], 11}, {NULL}};
const Segment SGI247499[] = {{&S93840[27], 6}, {NULL}};
const Segment SGI247500[] = {{&S93840[97], 9}, {NULL}};
const Segment SGI247501[] = {{&S93840[70], 6}, {NULL}};
const Segment SGI247502[] = {{&S93842[34], 10}, {NULL}};
const Segment SGI247503[] = {{&S93842[27], 6}, {NULL}};
const Segment SGI247504[] = {{&S93842[96], 9}, {NULL}};
const Segment SGI247505[] = {{&S93842[69], 6}, {NULL}};
const Segment SGI247506[] = {{&S100623[45], 10}, {NULL}};
const Segment SGI247507[] = {{&S100623[30], 6}, {NULL}};
const Segment SGI247508[] = {{&S100623[115], 3}, {NULL}};
const Segment SGI247509[] = {{&S100623[81], 5}, {NULL}};
const Segment SGI247510[] = {{&S100623[139], 13}, {NULL}};
const Segment SGI247511[] = {{&S100626[34], 10}, {NULL}};
const Segment SGI247512[] = {{&S100626[27], 6}, {NULL}};
const Segment SGI247513[] = {{&S100626[97], 9}, {NULL}};
const Segment SGI247514[] = {{&S100626[70], 6}, {NULL}};
const Segment SGI247515[] = {{&S100643[44], 9}, {NULL}};
const Segment SGI247516[] = {{&S100643[44], 21}, {NULL}};
const Segment SGI247517[] = {{&S100643[30], 6}, {NULL}};
const Segment SGI247518[] = {{&S100643[79], 6}, {NULL}};
const Segment SGI247519[] = {{&S100643[106], 13}, {NULL}};
const Segment SGI247520[] = {{&S100648[44], 6}, {NULL}};
const Segment SGI247521[] = {{&S100648[44], 18}, {NULL}};
const Segment SGI247522[] = {{&S100648[30], 6}, {NULL}};
const Segment SGI247523[] = {{&S100648[111], 3}, {NULL}};
const Segment SGI247524[] = {{&S100648[76], 6}, {NULL}};
const Segment SGI247525[] = {{&S100648[136], 7}, {NULL}};
const Segment SGI247526[] = {{&S100649[34], 9}, {NULL}};
const Segment SGI247527[] = {{&S100649[27], 6}, {NULL}};
const Segment SGI247528[] = {{&S100649[96], 9}, {NULL}};
const Segment SGI247529[] = {{&S100649[69], 6}, {NULL}};
const Segment SGI247530[] = {{&S100649[137], 8}, {NULL}};
const Segment SGI247531[] = {{&S93887[38], 9}, {NULL}};
const Segment SGI247532[] = {{&S93887[31], 6}, {NULL}};
const Segment SGI247533[] = {{&S100668[44], 10}, {NULL}};
const Segment SGI247534[] = {{&S100668[44], 22}, {NULL}};
const Segment SGI247535[] = {{&S100668[30], 6}, {NULL}};
const Segment SGI247536[] = {{&S100668[115], 3}, {NULL}};
const Segment SGI247537[] = {{&S100668[80], 6}, {NULL}};
const Segment SGI247538[] = {{&S100668[135], 8}, {NULL}};
const Segment SGI247539[] = {{&S100668[144], 13}, {NULL}};
const Segment SGI247540[] = {{&S100670[44], 10}, {NULL}};
const Segment SGI247541[] = {{&S100670[44], 22}, {NULL}};
const Segment SGI247542[] = {{&S100670[30], 6}, {NULL}};
const Segment SGI247543[] = {{&S100670[80], 6}, {NULL}};
const Segment SGI247544[] = {{&S100670[107], 13}, {NULL}};
const Segment SGI247545[] = {{&S93902[44], 9}, {NULL}};
const Segment SGI247546[] = {{&S93902[44], 20}, {NULL}};
const Segment SGI247547[] = {{&S93902[30], 6}, {NULL}};
const Segment SGI247548[] = {{&S93902[113], 3}, {NULL}};
const Segment SGI247549[] = {{&S93902[78], 6}, {NULL}};
const Segment SGI247550[] = {{&S5409[35], 3}, {NULL}};
const Segment SGI247551[] = {{&S5409[66], 6}, {NULL}};
const Segment SGI247552[] = {{&S5409[112], 6}, {NULL}};
const Segment SGI247553[] = {{&S5409[126], 7}, {NULL}};
const Segment SGI247554[] = {{&S5410[13], 6}, {NULL}};
const Segment SGI247555[] = {{&S5410[101], 3}, {NULL}};
const Segment SGI247556[] = {{&S5410[35], 3}, {NULL}};
const Segment SGI247557[] = {{&S5410[66], 6}, {NULL}};
const Segment SGI247558[] = {{&S5410[112], 6}, {NULL}};
const Segment SGI247559[] = {{&S5410[126], 7}, {NULL}};
const Segment SGI247560[] = {{&S5411[13], 6}, {NULL}};
const Segment SGI247561[] = {{&S5411[100], 3}, {NULL}};
const Segment SGI247562[] = {{&S5411[65], 6}, {NULL}};
const Segment SGI247563[] = {{&S5411[111], 6}, {NULL}};
const Segment SGI247564[] = {{&S5411[125], 7}, {NULL}};
const Segment SGI247565[] = {{&S5412[13], 6}, {NULL}};
const Segment SGI247566[] = {{&S5412[101], 3}, {NULL}};
const Segment SGI247567[] = {{&S5412[35], 3}, {NULL}};
const Segment SGI247568[] = {{&S5412[66], 6}, {NULL}};
const Segment SGI247569[] = {{&S5412[112], 12}, {NULL}};
const Segment SGI247570[] = {{&S5412[125], 7}, {NULL}};
const Segment SGI247571[] = {{&S93912[44], 18}, {NULL}};
const Segment SGI247572[] = {{&S93912[44], 29}, {NULL}};
const Segment SGI247573[] = {{&S93912[30], 6}, {NULL}};
const Segment SGI247574[] = {{&S5413[13], 6}, {NULL}};
const Segment SGI247575[] = {{&S5413[101], 3}, {NULL}};
const Segment SGI247576[] = {{&S5413[35], 3}, {NULL}};
const Segment SGI247577[] = {{&S5413[66], 6}, {NULL}};
const Segment SGI247578[] = {{&S5413[112], 12}, {NULL}};
const Segment SGI247579[] = {{&S5413[125], 7}, {NULL}};
const Segment SGI247580[] = {{&S93912[87], 6}, {NULL}};
const Segment SGI247581[] = {{&S93912[114], 13}, {NULL}};
const Segment SGI247582[] = {{&S5414[13], 6}, {NULL}};
const Segment SGI247583[] = {{&S5414[100], 3}, {NULL}};
const Segment SGI247584[] = {{&S5414[65], 6}, {NULL}};
const Segment SGI247585[] = {{&S5414[111], 6}, {NULL}};
const Segment SGI247586[] = {{&S5414[125], 7}, {NULL}};
const Segment SGI247587[] = {{&S5415[13], 6}, {NULL}};
const Segment SGI247588[] = {{&S5415[103], 3}, {NULL}};
const Segment SGI247589[] = {{&S5415[35], 5}, {NULL}};
const Segment SGI247590[] = {{&S5415[68], 6}, {NULL}};
const Segment SGI247591[] = {{&S5415[114], 6}, {NULL}};
const Segment SGI247592[] = {{&S5415[128], 7}, {NULL}};
const Segment SGI247593[] = {{&S5416[13], 6}, {NULL}};
const Segment SGI247594[] = {{&S5416[101], 3}, {NULL}};
const Segment SGI247595[] = {{&S5416[35], 3}, {NULL}};
const Segment SGI247596[] = {{&S5416[66], 6}, {NULL}};
const Segment SGI247597[] = {{&S5416[112], 5}, {NULL}};
const Segment SGI247598[] = {{&S5416[125], 7}, {NULL}};
const Segment SGI247599[] = {{&S5417[13], 6}, {NULL}};
const Segment SGI247600[] = {{&S5417[103], 3}, {NULL}};
const Segment SGI247601[] = {{&S5417[35], 5}, {NULL}};
const Segment SGI247602[] = {{&S5417[68], 6}, {NULL}};
const Segment SGI247603[] = {{&S5417[114], 6}, {NULL}};
const Segment SGI247604[] = {{&S5417[128], 7}, {NULL}};
const Segment SGI247605[] = {{&S5418[13], 6}, {NULL}};
const Segment SGI247606[] = {{&S5418[101], 3}, {NULL}};
const Segment SGI247607[] = {{&S5418[35], 3}, {NULL}};
const Segment SGI247608[] = {{&S5418[66], 6}, {NULL}};
const Segment SGI247609[] = {{&S5418[112], 6}, {NULL}};
const Segment SGI247610[] = {{&S5418[126], 7}, {NULL}};
const Segment SGI247611[] = {{&S5419[13], 6}, {NULL}};
const Segment SGI247612[] = {{&S5419[102], 3}, {NULL}};
const Segment SGI247613[] = {{&S5419[67], 6}, {NULL}};
const Segment SGI247614[] = {{&S5419[113], 6}, {NULL}};
const Segment SGI247615[] = {{&S5419[127], 7}, {NULL}};
const Segment SGI247616[] = {{&S5420[13], 6}, {NULL}};
const Segment SGI247617[] = {{&S5420[103], 3}, {NULL}};
const Segment SGI247618[] = {{&S5420[35], 5}, {NULL}};
const Segment SGI247619[] = {{&S5420[68], 6}, {NULL}};
const Segment SGI247620[] = {{&S5420[114], 6}, {NULL}};
const Segment SGI247621[] = {{&S5420[128], 7}, {NULL}};
const Segment SGI247622[] = {{&S93924[44], 14}, {NULL}};
const Segment SGI247623[] = {{&S93924[44], 25}, {NULL}};
const Segment SGI247624[] = {{&S93924[30], 6}, {NULL}};
const Segment SGI247625[] = {{&S5421[13], 6}, {NULL}};
const Segment SGI247626[] = {{&S5421[103], 3}, {NULL}};
const Segment SGI247627[] = {{&S5421[35], 5}, {NULL}};
const Segment SGI247628[] = {{&S93924[118], 3}, {NULL}};
const Segment SGI247629[] = {{&S5421[68], 6}, {NULL}};
const Segment SGI247630[] = {{&S93924[83], 6}, {NULL}};
const Segment SGI247631[] = {{&S5421[114], 12}, {NULL}};
const Segment SGI247632[] = {{&S5421[127], 7}, {NULL}};
const Segment SGI247633[] = {{&S5422[13], 6}, {NULL}};
const Segment SGI247634[] = {{&S5422[102], 3}, {NULL}};
const Segment SGI247635[] = {{&S5422[67], 6}, {NULL}};
const Segment SGI247636[] = {{&S5422[113], 6}, {NULL}};
const Segment SGI247637[] = {{&S5422[127], 7}, {NULL}};
const Segment SGI247638[] = {{&S5423[13], 6}, {NULL}};
const Segment SGI247639[] = {{&S5423[103], 3}, {NULL}};
const Segment SGI247640[] = {{&S5423[35], 5}, {NULL}};
const Segment SGI247641[] = {{&S5423[68], 6}, {NULL}};
const Segment SGI247642[] = {{&S5423[114], 6}, {NULL}};
const Segment SGI247643[] = {{&S5423[128], 7}, {NULL}};
const Segment SGI247644[] = {{&S5424[13], 6}, {NULL}};
const Segment SGI247645[] = {{&S5424[103], 3}, {NULL}};
const Segment SGI247646[] = {{&S5424[35], 5}, {NULL}};
const Segment SGI247647[] = {{&S5424[68], 6}, {NULL}};
const Segment SGI247648[] = {{&S5424[114], 12}, {NULL}};
const Segment SGI247649[] = {{&S5424[127], 7}, {NULL}};
const Segment SGI247650[] = {{&S5425[13], 6}, {NULL}};
const Segment SGI247651[] = {{&S5425[110], 5}, {NULL}};
const Segment SGI247652[] = {{&S5425[38], 3}, {NULL}};
const Segment SGI247653[] = {{&S5425[76], 5}, {NULL}};
const Segment SGI247654[] = {{&S5425[123], 5}, {NULL}};
const Segment SGI247655[] = {{&S5425[136], 9}, {NULL}};
const Segment SGI247656[] = {{&S5426[13], 6}, {NULL}};
const Segment SGI247657[] = {{&S5426[103], 3}, {NULL}};
const Segment SGI247658[] = {{&S5426[35], 5}, {NULL}};
const Segment SGI247659[] = {{&S5426[68], 6}, {NULL}};
const Segment SGI247660[] = {{&S5426[114], 6}, {NULL}};
const Segment SGI247661[] = {{&S5426[128], 7}, {NULL}};
const Segment SGI247662[] = {{&S5427[13], 6}, {NULL}};
const Segment SGI247663[] = {{&S5427[103], 3}, {NULL}};
const Segment SGI247664[] = {{&S5427[35], 5}, {NULL}};
const Segment SGI247665[] = {{&S5427[68], 6}, {NULL}};
const Segment SGI247666[] = {{&S5427[114], 6}, {NULL}};
const Segment SGI247667[] = {{&S5427[128], 7}, {NULL}};
const Segment SGI247668[] = {{&S5428[13], 6}, {NULL}};
const Segment SGI247669[] = {{&S5428[103], 3}, {NULL}};
const Segment SGI247670[] = {{&S5428[35], 5}, {NULL}};
const Segment SGI247671[] = {{&S5428[68], 6}, {NULL}};
const Segment SGI247672[] = {{&S5428[114], 5}, {NULL}};
const Segment SGI247673[] = {{&S5428[127], 7}, {NULL}};
const Segment SGI247674[] = {{&S5429[13], 6}, {NULL}};
const Segment SGI247675[] = {{&S5429[103], 3}, {NULL}};
const Segment SGI247676[] = {{&S5429[35], 5}, {NULL}};
const Segment SGI247677[] = {{&S5429[68], 6}, {NULL}};
const Segment SGI247678[] = {{&S5429[114], 6}, {NULL}};
const Segment SGI247679[] = {{&S93937[34], 10}, {NULL}};
const Segment SGI247680[] = {{&S5429[128], 7}, {NULL}};
const Segment SGI247681[] = {{&S93937[27], 6}, {NULL}};
const Segment SGI247682[] = {{&S93937[96], 9}, {NULL}};
const Segment SGI247683[] = {{&S93937[69], 6}, {NULL}};
const Segment SGI247684[] = {{&S100693[44], 13}, {NULL}};
const Segment SGI247685[] = {{&S100693[44], 29}, {NULL}};
const Segment SGI247686[] = {{&S100693[30], 6}, {NULL}};
const Segment SGI247687[] = {{&S5430[13], 6}, {NULL}};
const Segment SGI247688[] = {{&S5430[103], 3}, {NULL}};
const Segment SGI247689[] = {{&S5430[35], 5}, {NULL}};
const Segment SGI247690[] = {{&S5430[68], 6}, {NULL}};
const Segment SGI247691[] = {{&S5430[114], 6}, {NULL}};
const Segment SGI247692[] = {{&S5430[128], 7}, {NULL}};
const Segment SGI247693[] = {{&S100693[87], 6}, {NULL}};
const Segment SGI247694[] = {{&S100693[114], 13}, {NULL}};
const Segment SGI247695[] = {{&S5431[13], 6}, {NULL}};
const Segment SGI247696[] = {{&S5431[101], 3}, {NULL}};
const Segment SGI247697[] = {{&S5431[35], 3}, {NULL}};
const Segment SGI247698[] = {{&S5431[66], 6}, {NULL}};
const Segment SGI247699[] = {{&S5431[112], 6}, {NULL}};
const Segment SGI247700[] = {{&S5431[126], 7}, {NULL}};
const Segment SGI247701[] = {{&S93940[34], 10}, {NULL}};
const Segment SGI247702[] = {{&S93940[27], 6}, {NULL}};
const Segment SGI247703[] = {{&S5432[13], 6}, {NULL}};
const Segment SGI247704[] = {{&S5432[103], 3}, {NULL}};
const Segment SGI247705[] = {{&S5432[35], 5}, {NULL}};
const Segment SGI247706[] = {{&S93940[96], 9}, {NULL}};
const Segment SGI247707[] = {{&S93940[69], 6}, {NULL}};
const Segment SGI247708[] = {{&S5432[68], 6}, {NULL}};
const Segment SGI247709[] = {{&S5432[114], 6}, {NULL}};
const Segment SGI247710[] = {{&S5432[128], 7}, {NULL}};
const Segment SGI247711[] = {{&S5433[13], 6}, {NULL}};
const Segment SGI247712[] = {{&S5433[103], 3}, {NULL}};
const Segment SGI247713[] = {{&S5433[35], 5}, {NULL}};
const Segment SGI247714[] = {{&S5433[68], 6}, {NULL}};
const Segment SGI247715[] = {{&S5433[114], 12}, {NULL}};
const Segment SGI247716[] = {{&S5433[127], 7}, {NULL}};
const Segment SGI247717[] = {{&S5434[13], 6}, {NULL}};
const Segment SGI247718[] = {{&S5434[102], 3}, {NULL}};
const Segment SGI247719[] = {{&S5434[67], 6}, {NULL}};
const Segment SGI247720[] = {{&S5434[113], 6}, {NULL}};
const Segment SGI247721[] = {{&S5434[127], 7}, {NULL}};
const Segment SGI247722[] = {{&S100699[34], 10}, {NULL}};
const Segment SGI247723[] = {{&S100699[27], 6}, {NULL}};
const Segment SGI247724[] = {{&S100699[96], 9}, {NULL}};
const Segment SGI247725[] = {{&S100699[69], 6}, {NULL}};
const Segment SGI247726[] = {{&S5435[13], 6}, {NULL}};
const Segment SGI247727[] = {{&S5435[101], 3}, {NULL}};
const Segment SGI247728[] = {{&S5435[35], 3}, {NULL}};
const Segment SGI247729[] = {{&S5435[66], 6}, {NULL}};
const Segment SGI247730[] = {{&S5435[112], 6}, {NULL}};
const Segment SGI247731[] = {{&S5435[126], 7}, {NULL}};
const Segment SGI247732[] = {{&S5436[13], 6}, {NULL}};
const Segment SGI247733[] = {{&S5436[103], 3}, {NULL}};
const Segment SGI247734[] = {{&S5436[35], 5}, {NULL}};
const Segment SGI247735[] = {{&S5436[68], 6}, {NULL}};
const Segment SGI247736[] = {{&S5436[114], 6}, {NULL}};
const Segment SGI247737[] = {{&S5436[128], 7}, {NULL}};
const Segment SGI247738[] = {{&S5437[13], 6}, {NULL}};
const Segment SGI247739[] = {{&S5437[103], 3}, {NULL}};
const Segment SGI247740[] = {{&S5437[35], 5}, {NULL}};
const Segment SGI247741[] = {{&S5437[68], 6}, {NULL}};
const Segment SGI247742[] = {{&S5437[114], 6}, {NULL}};
const Segment SGI247743[] = {{&S5437[128], 7}, {NULL}};
const Segment SGI247744[] = {{&S5438[13], 6}, {NULL}};
const Segment SGI247745[] = {{&S5438[103], 3}, {NULL}};
const Segment SGI247746[] = {{&S5438[35], 5}, {NULL}};
const Segment SGI247747[] = {{&S5438[68], 6}, {NULL}};
const Segment SGI247748[] = {{&S5438[114], 12}, {NULL}};
const Segment SGI247749[] = {{&S5438[127], 7}, {NULL}};
const Segment SGI247750[] = {{&S100704[44], 9}, {NULL}};
const Segment SGI247751[] = {{&S100704[44], 21}, {NULL}};
const Segment SGI247752[] = {{&S100704[30], 6}, {NULL}};
const Segment SGI247753[] = {{&S100704[79], 6}, {NULL}};
const Segment SGI247754[] = {{&S5439[13], 6}, {NULL}};
const Segment SGI247755[] = {{&S5439[102], 3}, {NULL}};
const Segment SGI247756[] = {{&S100704[106], 13}, {NULL}};
const Segment SGI247757[] = {{&S5439[67], 6}, {NULL}};
const Segment SGI247758[] = {{&S5439[113], 6}, {NULL}};
const Segment SGI247759[] = {{&S5439[127], 7}, {NULL}};
const Segment SGI247760[] = {{&S5440[13], 6}, {NULL}};
const Segment SGI247761[] = {{&S5440[101], 3}, {NULL}};
const Segment SGI247762[] = {{&S5440[35], 3}, {NULL}};
const Segment SGI247763[] = {{&S5440[66], 6}, {NULL}};
const Segment SGI247764[] = {{&S5440[112], 6}, {NULL}};
const Segment SGI247765[] = {{&S5440[126], 7}, {NULL}};
const Segment SGI247766[] = {{&S5441[13], 6}, {NULL}};
const Segment SGI247767[] = {{&S5441[103], 3}, {NULL}};
const Segment SGI247768[] = {{&S5441[35], 5}, {NULL}};
const Segment SGI247769[] = {{&S5441[68], 6}, {NULL}};
const Segment SGI247770[] = {{&S93955[34], 18}, {NULL}};
const Segment SGI247771[] = {{&S5441[114], 5}, {NULL}};
const Segment SGI247772[] = {{&S5441[127], 7}, {NULL}};
const Segment SGI247773[] = {{&S93955[27], 6}, {NULL}};
const Segment SGI247774[] = {{&S93955[104], 9}, {NULL}};
const Segment SGI247775[] = {{&S93955[77], 6}, {NULL}};
const Segment SGI247776[] = {{&S100708[44], 14}, {NULL}};
const Segment SGI247777[] = {{&S100708[44], 31}, {NULL}};
const Segment SGI247778[] = {{&S100708[30], 6}, {NULL}};
const Segment SGI247779[] = {{&S5442[13], 6}, {NULL}};
const Segment SGI247780[] = {{&S5442[103], 3}, {NULL}};
const Segment SGI247781[] = {{&S5442[35], 5}, {NULL}};
const Segment SGI247782[] = {{&S5442[68], 6}, {NULL}};
const Segment SGI247783[] = {{&S5442[114], 6}, {NULL}};
const Segment SGI247784[] = {{&S100708[89], 6}, {NULL}};
const Segment SGI247785[] = {{&S5442[128], 7}, {NULL}};
const Segment SGI247786[] = {{&S100708[116], 13}, {NULL}};
const Segment SGI247787[] = {{&S5443[13], 6}, {NULL}};
const Segment SGI247788[] = {{&S5443[103], 3}, {NULL}};
const Segment SGI247789[] = {{&S5443[35], 5}, {NULL}};
const Segment SGI247790[] = {{&S93957[34], 10}, {NULL}};
const Segment SGI247791[] = {{&S5443[68], 6}, {NULL}};
const Segment SGI247792[] = {{&S93957[27], 6}, {NULL}};
const Segment SGI247793[] = {{&S5443[114], 6}, {NULL}};
const Segment SGI247794[] = {{&S5443[128], 7}, {NULL}};
const Segment SGI247795[] = {{&S93957[96], 9}, {NULL}};
const Segment SGI247796[] = {{&S93957[69], 6}, {NULL}};
const Segment SGI247797[] = {{&S5444[13], 6}, {NULL}};
const Segment SGI247798[] = {{&S5444[103], 3}, {NULL}};
const Segment SGI247799[] = {{&S5444[35], 5}, {NULL}};
const Segment SGI247800[] = {{&S5444[68], 6}, {NULL}};
const Segment SGI247801[] = {{&S5444[114], 6}, {NULL}};
const Segment SGI247802[] = {{&S5444[128], 7}, {NULL}};
const Segment SGI247803[] = {{&S93958[34], 11}, {NULL}};
const Segment SGI247804[] = {{&S93958[27], 6}, {NULL}};
const Segment SGI247805[] = {{&S93958[97], 9}, {NULL}};
const Segment SGI247806[] = {{&S93958[70], 6}, {NULL}};
const Segment SGI247807[] = {{&S5445[13], 6}, {NULL}};
const Segment SGI247808[] = {{&S5445[103], 3}, {NULL}};
const Segment SGI247809[] = {{&S5445[35], 5}, {NULL}};
const Segment SGI247810[] = {{&S5445[68], 6}, {NULL}};
const Segment SGI247811[] = {{&S5445[114], 6}, {NULL}};
const Segment SGI247812[] = {{&S5445[128], 7}, {NULL}};
const Segment SGI247813[] = {{&S5446[13], 6}, {NULL}};
const Segment SGI247814[] = {{&S5446[103], 3}, {NULL}};
const Segment SGI247815[] = {{&S5446[35], 5}, {NULL}};
const Segment SGI247816[] = {{&S5446[68], 6}, {NULL}};
const Segment SGI247817[] = {{&S5446[114], 6}, {NULL}};
const Segment SGI247818[] = {{&S5446[128], 7}, {NULL}};
const Segment SGI247819[] = {{&S100719[34], 10}, {NULL}};
const Segment SGI247820[] = {{&S100719[27], 6}, {NULL}};
const Segment SGI247821[] = {{&S100719[97], 9}, {NULL}};
const Segment SGI247822[] = {{&S100719[70], 6}, {NULL}};
const Segment SGI247823[] = {{&S94074[34], 10}, {NULL}};
const Segment SGI247824[] = {{&S94074[27], 6}, {NULL}};
const Segment SGI247825[] = {{&S94074[96], 9}, {NULL}};
const Segment SGI247826[] = {{&S94074[69], 6}, {NULL}};
const Segment SGI247827[] = {{&S94105[34], 6}, {NULL}};
const Segment SGI247828[] = {{&S94105[27], 6}, {NULL}};
const Segment SGI247829[] = {{&S94105[93], 9}, {NULL}};
const Segment SGI247830[] = {{&S94105[66], 6}, {NULL}};
const Segment SGI247831[] = {{&S100897[34], 9}, {NULL}};
const Segment SGI247832[] = {{&S100897[27], 6}, {NULL}};
const Segment SGI247833[] = {{&S100897[96], 9}, {NULL}};
const Segment SGI247834[] = {{&S100897[69], 6}, {NULL}};
const Segment SGI247835[] = {{&S94157[34], 10}, {NULL}};
const Segment SGI247836[] = {{&S94157[27], 6}, {NULL}};
const Segment SGI247837[] = {{&S94157[97], 9}, {NULL}};
const Segment SGI247838[] = {{&S102116[44], 10}, {NULL}};
const Segment SGI247839[] = {{&S102116[44], 22}, {NULL}};
const Segment SGI247840[] = {{&S102116[30], 6}, {NULL}};
const Segment SGI247841[] = {{&S102116[133], 3}, {NULL}};
const Segment SGI247842[] = {{&S102116[98], 6}, {NULL}};
const Segment SGI247843[] = {{&S102116[161], 3}, {NULL}};
const Segment SGI247844[] = {{&S102116[68], 8}, {NULL}};
const Segment SGI247845[] = {{&S102117[44], 10}, {NULL}};
const Segment SGI247846[] = {{&S102117[44], 22}, {NULL}};
const Segment SGI247847[] = {{&S102117[30], 6}, {NULL}};
const Segment SGI247848[] = {{&S102117[133], 3}, {NULL}};
const Segment SGI247849[] = {{&S102117[98], 6}, {NULL}};
const Segment SGI247850[] = {{&S102117[161], 3}, {NULL}};
const Segment SGI247851[] = {{&S102117[68], 8}, {NULL}};
const Segment SGI247852[] = {{&S100922[34], 10}, {NULL}};
const Segment SGI247853[] = {{&S100922[27], 6}, {NULL}};
const Segment SGI247854[] = {{&S100922[97], 9}, {NULL}};
const Segment SGI247855[] = {{&S100922[70], 6}, {NULL}};
const Segment SGI247856[] = {{&S94157[70], 6}, {NULL}};
const Segment SGI247857[] = {{&S94158[34], 9}, {NULL}};
const Segment SGI247858[] = {{&S94158[27], 6}, {NULL}};
const Segment SGI247859[] = {{&S94158[96], 9}, {NULL}};
const Segment SGI247860[] = {{&S94158[69], 6}, {NULL}};
const Segment SGI247861[] = {{&S100926[34], 11}, {NULL}};
const Segment SGI247862[] = {{&S100926[27], 6}, {NULL}};
const Segment SGI247863[] = {{&S100926[98], 9}, {NULL}};
const Segment SGI247864[] = {{&S100926[71], 6}, {NULL}};
const Segment SGI247865[] = {{&S100930[44], 10}, {NULL}};
const Segment SGI247866[] = {{&S100930[44], 22}, {NULL}};
const Segment SGI247867[] = {{&S100930[30], 6}, {NULL}};
const Segment SGI247868[] = {{&S100930[115], 3}, {NULL}};
const Segment SGI247869[] = {{&S100930[80], 6}, {NULL}};
const Segment SGI247870[] = {{&S100930[140], 11}, {NULL}};
const Segment SGI247871[] = {{&S94173[34], 8}, {NULL}};
const Segment SGI247872[] = {{&S94173[27], 6}, {NULL}};
const Segment SGI247873[] = {{&S94173[103], 9}, {NULL}};
const Segment SGI247874[] = {{&S94173[76], 6}, {NULL}};
const Segment SGI247875[] = {{&S5663[13], 6}, {NULL}};
const Segment SGI247876[] = {{&S5663[113], 5}, {NULL}};
const Segment SGI247877[] = {{&S5663[38], 3}, {NULL}};
const Segment SGI247878[] = {{&S5663[76], 8}, {NULL}};
const Segment SGI247879[] = {{&S5663[126], 5}, {NULL}};
const Segment SGI247880[] = {{&S5663[139], 9}, {NULL}};
const Segment SGI247881[] = {{&S5664[13], 6}, {NULL}};
const Segment SGI247882[] = {{&S5664[115], 5}, {NULL}};
const Segment SGI247883[] = {{&S5664[38], 5}, {NULL}};
const Segment SGI247884[] = {{&S5664[78], 8}, {NULL}};
const Segment SGI247885[] = {{&S5664[128], 3}, {NULL}};
const Segment SGI247886[] = {{&S5664[139], 9}, {NULL}};
const Segment SGI247887[] = {{&S5665[13], 6}, {NULL}};
const Segment SGI247888[] = {{&S5665[115], 5}, {NULL}};
const Segment SGI247889[] = {{&S5665[38], 5}, {NULL}};
const Segment SGI247890[] = {{&S5665[78], 8}, {NULL}};
const Segment SGI247891[] = {{&S5665[128], 5}, {NULL}};
const Segment SGI247892[] = {{&S5665[141], 9}, {NULL}};
const Segment SGI247893[] = {{&S5666[13], 6}, {NULL}};
const Segment SGI247894[] = {{&S5666[115], 5}, {NULL}};
const Segment SGI247895[] = {{&S5666[38], 5}, {NULL}};
const Segment SGI247896[] = {{&S5666[78], 8}, {NULL}};
const Segment SGI247897[] = {{&S5666[128], 3}, {NULL}};
const Segment SGI247898[] = {{&S5666[139], 9}, {NULL}};
const Segment SGI247899[] = {{&S5667[13], 6}, {NULL}};
const Segment SGI247900[] = {{&S5667[115], 5}, {NULL}};
const Segment SGI247901[] = {{&S5667[38], 5}, {NULL}};
const Segment SGI247902[] = {{&S5667[78], 8}, {NULL}};
const Segment SGI247903[] = {{&S5667[128], 3}, {NULL}};
const Segment SGI247904[] = {{&S5667[139], 9}, {NULL}};
const Segment SGI247905[] = {{&S5668[13], 6}, {NULL}};
const Segment SGI247906[] = {{&S5668[115], 5}, {NULL}};
const Segment SGI247907[] = {{&S5668[38], 5}, {NULL}};
const Segment SGI247908[] = {{&S5668[78], 8}, {NULL}};
const Segment SGI247909[] = {{&S5668[128], 4}, {NULL}};
const Segment SGI247910[] = {{&S5668[140], 9}, {NULL}};
const Segment SGI247911[] = {{&S5669[13], 6}, {NULL}};
const Segment SGI247912[] = {{&S5669[113], 5}, {NULL}};
const Segment SGI247913[] = {{&S5669[38], 3}, {NULL}};
const Segment SGI247914[] = {{&S5669[76], 8}, {NULL}};
const Segment SGI247915[] = {{&S5669[126], 4}, {NULL}};
const Segment SGI247916[] = {{&S5669[138], 9}, {NULL}};
const Segment SGI247917[] = {{&S5670[13], 6}, {NULL}};
const Segment SGI247918[] = {{&S5670[115], 5}, {NULL}};
const Segment SGI247919[] = {{&S5670[38], 5}, {NULL}};
const Segment SGI247920[] = {{&S5670[78], 8}, {NULL}};
const Segment SGI247921[] = {{&S5670[128], 4}, {NULL}};
const Segment SGI247922[] = {{&S5670[140], 9}, {NULL}};
const Segment SGI247923[] = {{&S5671[13], 6}, {NULL}};
const Segment SGI247924[] = {{&S5671[115], 5}, {NULL}};
const Segment SGI247925[] = {{&S5671[38], 5}, {NULL}};
const Segment SGI247926[] = {{&S5671[78], 8}, {NULL}};
const Segment SGI247927[] = {{&S5671[128], 4}, {NULL}};
const Segment SGI247928[] = {{&S5671[140], 9}, {NULL}};
const Segment SGI247929[] = {{&S5672[13], 6}, {NULL}};
const Segment SGI247930[] = {{&S5672[115], 5}, {NULL}};
const Segment SGI247931[] = {{&S5672[38], 5}, {NULL}};
const Segment SGI247932[] = {{&S5672[78], 8}, {NULL}};
const Segment SGI247933[] = {{&S5672[128], 3}, {NULL}};
const Segment SGI247934[] = {{&S5672[139], 9}, {NULL}};
const Segment SGI247935[] = {{&S5673[13], 6}, {NULL}};
const Segment SGI247936[] = {{&S5673[115], 5}, {NULL}};
const Segment SGI247937[] = {{&S5673[38], 5}, {NULL}};
const Segment SGI247938[] = {{&S5673[78], 8}, {NULL}};
const Segment SGI247939[] = {{&S5673[128], 3}, {NULL}};
const Segment SGI247940[] = {{&S5673[139], 9}, {NULL}};
const Segment SGI247941[] = {{&S5674[13], 6}, {NULL}};
const Segment SGI247942[] = {{&S5674[123], 5}, {NULL}};
const Segment SGI247943[] = {{&S5674[48], 3}, {NULL}};
const Segment SGI247944[] = {{&S5674[86], 8}, {NULL}};
const Segment SGI247945[] = {{&S5674[136], 5}, {NULL}};
const Segment SGI247946[] = {{&S5674[149], 9}, {NULL}};
const Segment SGI247947[] = {{&S5675[13], 6}, {NULL}};
const Segment SGI247948[] = {{&S5675[115], 5}, {NULL}};
const Segment SGI247949[] = {{&S5675[38], 5}, {NULL}};
const Segment SGI247950[] = {{&S5675[78], 8}, {NULL}};
const Segment SGI247951[] = {{&S5675[128], 3}, {NULL}};
const Segment SGI247952[] = {{&S5675[139], 9}, {NULL}};
const Segment SGI247953[] = {{&S5678[13], 6}, {NULL}};
const Segment SGI247954[] = {{&S5678[115], 5}, {NULL}};
const Segment SGI247955[] = {{&S5678[38], 5}, {NULL}};
const Segment SGI247956[] = {{&S5678[78], 8}, {NULL}};
const Segment SGI247957[] = {{&S5678[128], 5}, {NULL}};
const Segment SGI247958[] = {{&S5678[141], 9}, {NULL}};
const Segment SGI247959[] = {{&S5681[13], 6}, {NULL}};
const Segment SGI247960[] = {{&S5681[115], 5}, {NULL}};
const Segment SGI247961[] = {{&S5681[38], 5}, {NULL}};
const Segment SGI247962[] = {{&S5681[78], 8}, {NULL}};
const Segment SGI247963[] = {{&S5681[128], 5}, {NULL}};
const Segment SGI247964[] = {{&S5681[141], 9}, {NULL}};
const Segment SGI247965[] = {{&S5682[13], 6}, {NULL}};
const Segment SGI247966[] = {{&S5682[115], 5}, {NULL}};
const Segment SGI247967[] = {{&S5682[38], 5}, {NULL}};
const Segment SGI247968[] = {{&S5682[78], 8}, {NULL}};
const Segment SGI247969[] = {{&S5682[128], 3}, {NULL}};
const Segment SGI247970[] = {{&S5682[139], 9}, {NULL}};
const Segment SGI247971[] = {{&S5684[13], 6}, {NULL}};
const Segment SGI247972[] = {{&S5684[115], 5}, {NULL}};
const Segment SGI247973[] = {{&S5684[38], 5}, {NULL}};
const Segment SGI247974[] = {{&S5684[78], 8}, {NULL}};
const Segment SGI247975[] = {{&S5684[128], 3}, {NULL}};
const Segment SGI247976[] = {{&S5684[139], 9}, {NULL}};
const Segment SGI247977[] = {{&S5685[13], 6}, {NULL}};
const Segment SGI247978[] = {{&S5685[116], 5}, {NULL}};
const Segment SGI247979[] = {{&S5685[38], 5}, {NULL}};
const Segment SGI247980[] = {{&S5685[78], 9}, {NULL}};
const Segment SGI247981[] = {{&S5685[129], 6}, {NULL}};
const Segment SGI247982[] = {{&S5685[143], 9}, {NULL}};
const Segment SGI247983[] = {{&S5686[13], 6}, {NULL}};
const Segment SGI247984[] = {{&S5686[38], 5}, {NULL}};
const Segment SGI247985[] = {{&S5686[115], 5}, {NULL}};
const Segment SGI247986[] = {{&S5686[128], 9}, {NULL}};
const Segment SGI247987[] = {{&S5687[13], 6}, {NULL}};
const Segment SGI247988[] = {{&S5687[113], 5}, {NULL}};
const Segment SGI247989[] = {{&S5687[38], 3}, {NULL}};
const Segment SGI247990[] = {{&S5687[76], 8}, {NULL}};
const Segment SGI247991[] = {{&S5687[126], 7}, {NULL}};
const Segment SGI247992[] = {{&S5687[141], 9}, {NULL}};
const Segment SGI247993[] = {{&S5688[13], 6}, {NULL}};
const Segment SGI247994[] = {{&S5688[116], 5}, {NULL}};
const Segment SGI247995[] = {{&S5688[38], 5}, {NULL}};
const Segment SGI247996[] = {{&S5688[78], 9}, {NULL}};
const Segment SGI247997[] = {{&S5688[129], 6}, {NULL}};
const Segment SGI247998[] = {{&S5688[143], 9}, {NULL}};
const Segment SGI247999[] = {{&S5689[13], 6}, {NULL}};
const Segment SGI248000[] = {{&S5689[115], 5}, {NULL}};
const Segment SGI248001[] = {{&S5689[38], 5}, {NULL}};
const Segment SGI248002[] = {{&S5689[78], 8}, {NULL}};
const Segment SGI248003[] = {{&S5689[128], 3}, {NULL}};
const Segment SGI248004[] = {{&S5689[139], 9}, {NULL}};
const Segment SGI248005[] = {{&S5690[13], 6}, {NULL}};
const Segment SGI248006[] = {{&S5690[38], 5}, {NULL}};
const Segment SGI248007[] = {{&S5690[115], 5}, {NULL}};
const Segment SGI248008[] = {{&S5690[128], 9}, {NULL}};
const Segment SGI248009[] = {{&S5691[13], 6}, {NULL}};
const Segment SGI248010[] = {{&S5691[115], 5}, {NULL}};
const Segment SGI248011[] = {{&S5691[38], 5}, {NULL}};
const Segment SGI248012[] = {{&S5691[78], 8}, {NULL}};
const Segment SGI248013[] = {{&S5691[128], 5}, {NULL}};
const Segment SGI248014[] = {{&S5691[141], 9}, {NULL}};
const Segment SGI248015[] = {{&S5692[13], 6}, {NULL}};
const Segment SGI248016[] = {{&S5692[8], 3}, {NULL}};
const Segment SGI248017[] = {{&S5692[28], 5}, {NULL}};
const Segment SGI248018[] = {{&S5692[61], 8}, {NULL}};
const Segment SGI248019[] = {{&S5692[109], 3}, {NULL}};
const Segment SGI248020[] = {{&S5692[120], 9}, {NULL}};
const Segment SGI248021[] = {{&S5693[13], 6}, {NULL}};
const Segment SGI248022[] = {{&S5693[113], 5}, {NULL}};
const Segment SGI248023[] = {{&S5693[38], 3}, {NULL}};
const Segment SGI248024[] = {{&S5693[76], 8}, {NULL}};
const Segment SGI248025[] = {{&S5693[126], 5}, {NULL}};
const Segment SGI248026[] = {{&S5693[139], 9}, {NULL}};
const Segment SGI248027[] = {{&S5694[13], 6}, {NULL}};
const Segment SGI248028[] = {{&S5694[115], 5}, {NULL}};
const Segment SGI248029[] = {{&S5694[38], 5}, {NULL}};
const Segment SGI248030[] = {{&S5694[78], 8}, {NULL}};
const Segment SGI248031[] = {{&S5694[128], 5}, {NULL}};
const Segment SGI248032[] = {{&S5694[141], 9}, {NULL}};
const Segment SGI248033[] = {{&S5695[13], 6}, {NULL}};
const Segment SGI248034[] = {{&S5695[112], 5}, {NULL}};
const Segment SGI248035[] = {{&S5695[38], 5}, {NULL}};
const Segment SGI248036[] = {{&S5695[78], 5}, {NULL}};
const Segment SGI248037[] = {{&S5695[125], 3}, {NULL}};
const Segment SGI248038[] = {{&S5695[136], 9}, {NULL}};
const Segment SGI248039[] = {{&S5696[13], 6}, {NULL}};
const Segment SGI248040[] = {{&S5696[111], 5}, {NULL}};
const Segment SGI248041[] = {{&S5696[38], 2}, {NULL}};
const Segment SGI248042[] = {{&S5696[75], 8}, {NULL}};
const Segment SGI248043[] = {{&S5696[124], 5}, {NULL}};
const Segment SGI248044[] = {{&S5696[137], 9}, {NULL}};
const Segment SGI248045[] = {{&S5697[13], 6}, {NULL}};
const Segment SGI248046[] = {{&S5697[112], 5}, {NULL}};
const Segment SGI248047[] = {{&S5697[38], 3}, {NULL}};
const Segment SGI248048[] = {{&S5697[76], 8}, {NULL}};
const Segment SGI248049[] = {{&S5697[125], 3}, {NULL}};
const Segment SGI248050[] = {{&S5697[136], 9}, {NULL}};
const Segment SGI248051[] = {{&S5698[13], 6}, {NULL}};
const Segment SGI248052[] = {{&S5698[112], 5}, {NULL}};
const Segment SGI248053[] = {{&S5698[38], 3}, {NULL}};
const Segment SGI248054[] = {{&S5698[76], 8}, {NULL}};
const Segment SGI248055[] = {{&S5698[125], 3}, {NULL}};
const Segment SGI248056[] = {{&S5698[136], 9}, {NULL}};
const Segment SGI248057[] = {{&S5699[13], 6}, {NULL}};
const Segment SGI248058[] = {{&S5699[112], 5}, {NULL}};
const Segment SGI248059[] = {{&S5699[38], 3}, {NULL}};
const Segment SGI248060[] = {{&S5699[76], 8}, {NULL}};
const Segment SGI248061[] = {{&S5699[125], 3}, {NULL}};
const Segment SGI248062[] = {{&S5699[136], 9}, {NULL}};
const Segment SGI248063[] = {{&S5700[13], 6}, {NULL}};
const Segment SGI248064[] = {{&S5700[112], 5}, {NULL}};
const Segment SGI248065[] = {{&S5700[38], 3}, {NULL}};
const Segment SGI248066[] = {{&S5700[76], 8}, {NULL}};
const Segment SGI248067[] = {{&S5700[125], 3}, {NULL}};
const Segment SGI248068[] = {{&S5700[136], 9}, {NULL}};
const Segment SGI248069[] = {{&S5701[13], 6}, {NULL}};
const Segment SGI248070[] = {{&S5701[112], 5}, {NULL}};
const Segment SGI248071[] = {{&S5701[38], 3}, {NULL}};
const Segment SGI248072[] = {{&S5701[76], 8}, {NULL}};
const Segment SGI248073[] = {{&S5701[125], 3}, {NULL}};
const Segment SGI248074[] = {{&S5701[136], 9}, {NULL}};
const Segment SGI248075[] = {{&S5702[13], 6}, {NULL}};
const Segment SGI248076[] = {{&S5702[115], 5}, {NULL}};
const Segment SGI248077[] = {{&S5702[38], 5}, {NULL}};
const Segment SGI248078[] = {{&S5702[78], 8}, {NULL}};
const Segment SGI248079[] = {{&S5702[128], 5}, {NULL}};
const Segment SGI248080[] = {{&S5702[141], 9}, {NULL}};
const Segment SGI248081[] = {{&S5703[13], 6}, {NULL}};
const Segment SGI248082[] = {{&S5703[106], 4}, {NULL}};
const Segment SGI248083[] = {{&S5703[31], 3}, {NULL}};
const Segment SGI248084[] = {{&S5703[69], 8}, {NULL}};
const Segment SGI248085[] = {{&S5703[131], 3}, {NULL}};
const Segment SGI248086[] = {{&S5703[142], 9}, {NULL}};
const Segment SGI248087[] = {{&S5705[13], 6}, {NULL}};
const Segment SGI248088[] = {{&S5705[115], 5}, {NULL}};
const Segment SGI248089[] = {{&S5705[38], 5}, {NULL}};
const Segment SGI248090[] = {{&S5705[78], 8}, {NULL}};
const Segment SGI248091[] = {{&S5705[128], 3}, {NULL}};
const Segment SGI248092[] = {{&S5705[139], 9}, {NULL}};
const Segment SGI248093[] = {{&S5706[13], 6}, {NULL}};
const Segment SGI248094[] = {{&S5706[113], 5}, {NULL}};
const Segment SGI248095[] = {{&S5706[38], 3}, {NULL}};
const Segment SGI248096[] = {{&S5706[76], 8}, {NULL}};
const Segment SGI248097[] = {{&S5706[126], 6}, {NULL}};
const Segment SGI248098[] = {{&S5706[140], 9}, {NULL}};
const Segment SGI248099[] = {{&S5707[13], 6}, {NULL}};
const Segment SGI248100[] = {{&S5707[115], 5}, {NULL}};
const Segment SGI248101[] = {{&S5707[38], 5}, {NULL}};
const Segment SGI248102[] = {{&S5707[78], 8}, {NULL}};
const Segment SGI248103[] = {{&S5707[128], 6}, {NULL}};
const Segment SGI248104[] = {{&S5707[142], 9}, {NULL}};
const Segment SGI248105[] = {{&S101060[34], 10}, {NULL}};
const Segment SGI248106[] = {{&S101060[27], 6}, {NULL}};
const Segment SGI248107[] = {{&S101060[97], 9}, {NULL}};
const Segment SGI248108[] = {{&S101060[70], 6}, {NULL}};
const Segment SGI248109[] = {{&S102120[44], 10}, {NULL}};
const Segment SGI248110[] = {{&S102120[44], 22}, {NULL}};
const Segment SGI248111[] = {{&S102120[30], 6}, {NULL}};
const Segment SGI248112[] = {{&S102120[133], 3}, {NULL}};
const Segment SGI248113[] = {{&S102120[98], 6}, {NULL}};
const Segment SGI248114[] = {{&S102120[161], 3}, {NULL}};
const Segment SGI248115[] = {{&S102120[68], 8}, {NULL}};
const Segment SGI248116[] = {{&S101075[45], 10}, {NULL}};
const Segment SGI248117[] = {{&S101075[30], 6}, {NULL}};
const Segment SGI248118[] = {{&S101075[115], 3}, {NULL}};
const Segment SGI248119[] = {{&S101075[81], 5}, {NULL}};
const Segment SGI248120[] = {{&S101075[139], 13}, {NULL}};
const Segment SGI248121[] = {{&S101094[34], 12}, {NULL}};
const Segment SGI248122[] = {{&S101094[27], 6}, {NULL}};
const Segment SGI248123[] = {{&S101094[99], 9}, {NULL}};
const Segment SGI248124[] = {{&S101094[72], 6}, {NULL}};
const Segment SGI248125[] = {{&S101129[34], 10}, {NULL}};
const Segment SGI248126[] = {{&S101129[27], 6}, {NULL}};
const Segment SGI248127[] = {{&S101129[97], 9}, {NULL}};
const Segment SGI248128[] = {{&S101129[70], 6}, {NULL}};
const Segment SGI248129[] = {{&S94320[34], 10}, {NULL}};
const Segment SGI248130[] = {{&S94320[27], 6}, {NULL}};
const Segment SGI248131[] = {{&S94320[97], 9}, {NULL}};
const Segment SGI248132[] = {{&S94320[70], 6}, {NULL}};
const Segment SGI248133[] = {{&S94321[34], 10}, {NULL}};
const Segment SGI248134[] = {{&S94321[27], 6}, {NULL}};
const Segment SGI248135[] = {{&S94321[97], 9}, {NULL}};
const Segment SGI248136[] = {{&S94321[70], 6}, {NULL}};
const Segment SGI248137[] = {{&S94321[138], 8}, {NULL}};
const Segment SGI248138[] = {{&S102298[34], 10}, {NULL}};
const Segment SGI248139[] = {{&S102298[27], 6}, {NULL}};
const Segment SGI248140[] = {{&S102298[97], 9}, {NULL}};
const Segment SGI248141[] = {{&S102298[70], 6}, {NULL}};
const Segment SGI248142[] = {{&S102301[34], 10}, {NULL}};
const Segment SGI248143[] = {{&S102301[27], 6}, {NULL}};
const Segment SGI248144[] = {{&S102301[97], 9}, {NULL}};
const Segment SGI248145[] = {{&S102301[70], 6}, {NULL}};
const Segment SGI248146[] = {{&S94382[34], 9}, {NULL}};
const Segment SGI248147[] = {{&S94382[27], 6}, {NULL}};
const Segment SGI248148[] = {{&S94382[95], 9}, {NULL}};
const Segment SGI248149[] = {{&S94382[68], 6}, {NULL}};
const Segment SGI248150[] = {{&S94382[129], 8}, {NULL}};
const Segment SGI248151[] = {{&S94383[44], 10}, {NULL}};
const Segment SGI248152[] = {{&S94383[44], 21}, {NULL}};
const Segment SGI248153[] = {{&S94383[30], 6}, {NULL}};
const Segment SGI248154[] = {{&S94383[79], 6}, {NULL}};
const Segment SGI248155[] = {{&S94383[106], 13}, {NULL}};
const Segment SGI248156[] = {{&S94493[34], 10}, {NULL}};
const Segment SGI248157[] = {{&S94493[27], 6}, {NULL}};
const Segment SGI248158[] = {{&S94493[96], 9}, {NULL}};
const Segment SGI248159[] = {{&S94493[69], 6}, {NULL}};
const Segment SGI248160[] = {{&S102521[44], 10}, {NULL}};
const Segment SGI248161[] = {{&S102521[44], 22}, {NULL}};
const Segment SGI248162[] = {{&S102521[30], 6}, {NULL}};
const Segment SGI248163[] = {{&S102521[133], 3}, {NULL}};
const Segment SGI248164[] = {{&S102521[98], 6}, {NULL}};
const Segment SGI248165[] = {{&S102521[161], 3}, {NULL}};
const Segment SGI248166[] = {{&S102521[68], 8}, {NULL}};
const Segment SGI248167[] = {{&S102522[44], 10}, {NULL}};
const Segment SGI248168[] = {{&S102522[44], 22}, {NULL}};
const Segment SGI248169[] = {{&S102522[30], 6}, {NULL}};
const Segment SGI248170[] = {{&S102522[133], 3}, {NULL}};
const Segment SGI248171[] = {{&S102522[98], 6}, {NULL}};
const Segment SGI248172[] = {{&S102522[161], 3}, {NULL}};
const Segment SGI248173[] = {{&S102522[68], 8}, {NULL}};
const Segment SGI248174[] = {{&S94558[34], 11}, {NULL}};
const Segment SGI248175[] = {{&S94558[27], 6}, {NULL}};
const Segment SGI248176[] = {{&S94558[97], 9}, {NULL}};
const Segment SGI248177[] = {{&S94558[70], 6}, {NULL}};
const Segment SGI248178[] = {{&S94561[44], 10}, {NULL}};
const Segment SGI248179[] = {{&S94561[44], 21}, {NULL}};
const Segment SGI248180[] = {{&S94561[30], 6}, {NULL}};
const Segment SGI248181[] = {{&S94561[112], 3}, {NULL}};
const Segment SGI248182[] = {{&S94561[79], 5}, {NULL}};
const Segment SGI248183[] = {{&S94566[44], 10}, {NULL}};
const Segment SGI248184[] = {{&S94566[44], 21}, {NULL}};
const Segment SGI248185[] = {{&S94566[30], 6}, {NULL}};
const Segment SGI248186[] = {{&S94566[79], 6}, {NULL}};
const Segment SGI248187[] = {{&S94566[106], 13}, {NULL}};
const Segment SGI248188[] = {{&S94578[59], 10}, {NULL}};
const Segment SGI248189[] = {{&S94578[59], 21}, {NULL}};
const Segment SGI248190[] = {{&S94578[45], 6}, {NULL}};
const Segment SGI248191[] = {{&S94578[129], 3}, {NULL}};
const Segment SGI248192[] = {{&S94578[94], 6}, {NULL}};
const Segment SGI248193[] = {{&S94589[34], 8}, {NULL}};
const Segment SGI248194[] = {{&S94589[27], 6}, {NULL}};
const Segment SGI248195[] = {{&S94589[94], 9}, {NULL}};
const Segment SGI248196[] = {{&S94589[67], 6}, {NULL}};
const Segment SGI248197[] = {{&S94609[34], 10}, {NULL}};
const Segment SGI248198[] = {{&S94609[27], 6}, {NULL}};
const Segment SGI248199[] = {{&S94609[96], 9}, {NULL}};
const Segment SGI248200[] = {{&S94609[69], 6}, {NULL}};
const Segment SGI248201[] = {{&S101581[34], 10}, {NULL}};
const Segment SGI248202[] = {{&S101581[27], 6}, {NULL}};
const Segment SGI248203[] = {{&S101581[97], 9}, {NULL}};
const Segment SGI248204[] = {{&S101581[70], 6}, {NULL}};
const Segment SGI248205[] = {{&S94616[44], 10}, {NULL}};
const Segment SGI248206[] = {{&S94616[44], 21}, {NULL}};
const Segment SGI248207[] = {{&S94616[30], 6}, {NULL}};
const Segment SGI248208[] = {{&S94616[79], 6}, {NULL}};
const Segment SGI248209[] = {{&S94616[106], 13}, {NULL}};
const Segment SGI248210[] = {{&S94617[34], 9}, {NULL}};
const Segment SGI248211[] = {{&S94617[27], 6}, {NULL}};
const Segment SGI248212[] = {{&S94617[96], 9}, {NULL}};
const Segment SGI248213[] = {{&S94617[69], 6}, {NULL}};
const Segment SGI248214[] = {{&S94780[44], 9}, {NULL}};
const Segment SGI248215[] = {{&S94780[44], 20}, {NULL}};
const Segment SGI248216[] = {{&S94780[30], 6}, {NULL}};
const Segment SGI248217[] = {{&S94780[113], 3}, {NULL}};
const Segment SGI248218[] = {{&S94780[78], 6}, {NULL}};
const Segment SGI248219[] = {{&S101773[44], 11}, {NULL}};
const Segment SGI248220[] = {{&S101773[44], 23}, {NULL}};
const Segment SGI248221[] = {{&S101773[30], 6}, {NULL}};
const Segment SGI248222[] = {{&S101773[81], 6}, {NULL}};
const Segment SGI248223[] = {{&S101773[108], 13}, {NULL}};
const Segment SGI248224[] = {{&S94792[34], 7}, {NULL}};
const Segment SGI248225[] = {{&S94792[27], 6}, {NULL}};
const Segment SGI248226[] = {{&S94792[100], 9}, {NULL}};
const Segment SGI248227[] = {{&S94792[73], 6}, {NULL}};
const Segment SGI248228[] = {{&S94808[44], 10}, {NULL}};
const Segment SGI248229[] = {{&S94808[44], 21}, {NULL}};
const Segment SGI248230[] = {{&S94808[30], 6}, {NULL}};
const Segment SGI248231[] = {{&S94808[114], 3}, {NULL}};
const Segment SGI248232[] = {{&S94808[79], 6}, {NULL}};
const Segment SGI248233[] = {{&S94808[149], 3}, {NULL}};
const Segment SGI248234[] = {{&S94816[34], 7}, {NULL}};
const Segment SGI248235[] = {{&S94816[27], 6}, {NULL}};
const Segment SGI248236[] = {{&S94816[98], 9}, {NULL}};
const Segment SGI248237[] = {{&S94816[71], 6}, {NULL}};
const Segment SGI248238[] = {{&S94819[44], 9}, {NULL}};
const Segment SGI248239[] = {{&S94819[44], 21}, {NULL}};
const Segment SGI248240[] = {{&S94819[30], 6}, {NULL}};
const Segment SGI248241[] = {{&S94819[114], 3}, {NULL}};
const Segment SGI248242[] = {{&S94819[79], 6}, {NULL}};
const Segment SGI248243[] = {{&S94845[34], 10}, {NULL}};
const Segment SGI248244[] = {{&S94845[27], 6}, {NULL}};
const Segment SGI248245[] = {{&S94845[96], 9}, {NULL}};
const Segment SGI248246[] = {{&S94845[69], 6}, {NULL}};
const Segment SGI248247[] = {{&S94856[34], 10}, {NULL}};
const Segment SGI248248[] = {{&S94856[27], 6}, {NULL}};
const Segment SGI248249[] = {{&S94856[96], 9}, {NULL}};
const Segment SGI248250[] = {{&S94856[69], 6}, {NULL}};
const Segment SGI248251[] = {{&S94856[130], 8}, {NULL}};
const Segment SGI248252[] = {{&S94858[34], 10}, {NULL}};
const Segment SGI248253[] = {{&S94858[27], 6}, {NULL}};
const Segment SGI248254[] = {{&S94858[96], 9}, {NULL}};
const Segment SGI248255[] = {{&S94858[69], 6}, {NULL}};
const Segment SGI248256[] = {{&S94858[137], 8}, {NULL}};
const Segment SGI248257[] = {{&S94869[34], 10}, {NULL}};
const Segment SGI248258[] = {{&S94869[27], 6}, {NULL}};
const Segment SGI248259[] = {{&S94869[97], 9}, {NULL}};
const Segment SGI248260[] = {{&S94869[70], 6}, {NULL}};
const Segment SGI248261[] = {{&S94874[34], 8}, {NULL}};
const Segment SGI248262[] = {{&S94874[27], 6}, {NULL}};
const Segment SGI248263[] = {{&S94874[94], 9}, {NULL}};
const Segment SGI248264[] = {{&S94874[67], 6}, {NULL}};
const Segment SGI248265[] = {{&S101880[44], 9}, {NULL}};
const Segment SGI248266[] = {{&S101880[44], 21}, {NULL}};
const Segment SGI248267[] = {{&S101880[30], 6}, {NULL}};
const Segment SGI248268[] = {{&S101880[134], 3}, {NULL}};
const Segment SGI248269[] = {{&S101880[99], 6}, {NULL}};
const Segment SGI248270[] = {{&S101880[162], 3}, {NULL}};
const Segment SGI248271[] = {{&S101880[67], 8}, {NULL}};
const Segment SGI248272[] = {{&S95019[34], 10}, {NULL}};
const Segment SGI248273[] = {{&S95019[27], 6}, {NULL}};
const Segment SGI248274[] = {{&S95019[97], 9}, {NULL}};
const Segment SGI248275[] = {{&S95019[70], 6}, {NULL}};
const Segment SGI248276[] = {{&S95023[34], 10}, {NULL}};
const Segment SGI248277[] = {{&S95023[27], 6}, {NULL}};
const Segment SGI248278[] = {{&S95023[97], 9}, {NULL}};
const Segment SGI248279[] = {{&S95023[70], 6}, {NULL}};
const Segment SGI248280[] = {{&S95027[34], 12}, {NULL}};
const Segment SGI248281[] = {{&S95027[27], 6}, {NULL}};
const Segment SGI248282[] = {{&S95027[99], 9}, {NULL}};
const Segment SGI248283[] = {{&S95027[72], 6}, {NULL}};
const Segment SGI248284[] = {{&S95033[34], 12}, {NULL}};
const Segment SGI248285[] = {{&S95033[27], 6}, {NULL}};
const Segment SGI248286[] = {{&S95033[98], 9}, {NULL}};
const Segment SGI248287[] = {{&S95033[71], 6}, {NULL}};
const Segment SGI248288[] = {{&S95034[34], 12}, {NULL}};
const Segment SGI248289[] = {{&S95034[27], 6}, {NULL}};
const Segment SGI248290[] = {{&S95034[98], 9}, {NULL}};
const Segment SGI248291[] = {{&S95034[71], 6}, {NULL}};
const Segment SGI248292[] = {{&S95093[44], 9}, {NULL}};
const Segment SGI248293[] = {{&S95093[44], 21}, {NULL}};
const Segment SGI248294[] = {{&S95093[30], 6}, {NULL}};
const Segment SGI248295[] = {{&S102928[44], 10}, {NULL}};
const Segment SGI248296[] = {{&S95093[114], 3}, {NULL}};
const Segment SGI248297[] = {{&S102928[44], 22}, {NULL}};
const Segment SGI248298[] = {{&S102928[30], 6}, {NULL}};
const Segment SGI248299[] = {{&S95093[79], 6}, {NULL}};
const Segment SGI248300[] = {{&S95093[139], 8}, {NULL}};
const Segment SGI248301[] = {{&S102928[133], 3}, {NULL}};
const Segment SGI248302[] = {{&S102928[98], 6}, {NULL}};
const Segment SGI248303[] = {{&S102928[161], 3}, {NULL}};
const Segment SGI248304[] = {{&S102928[68], 8}, {NULL}};
const Segment SGI248305[] = {{&S95095[34], 9}, {NULL}};
const Segment SGI248306[] = {{&S95095[27], 6}, {NULL}};
const Segment SGI248307[] = {{&S95095[95], 9}, {NULL}};
const Segment SGI248308[] = {{&S95095[68], 6}, {NULL}};
const Segment SGI248309[] = {{&S95095[129], 8}, {NULL}};
const Segment SGI248310[] = {{&S102091[34], 7}, {NULL}};
const Segment SGI248311[] = {{&S102091[27], 6}, {NULL}};
const Segment SGI248312[] = {{&S102091[99], 9}, {NULL}};
const Segment SGI248313[] = {{&S102091[72], 6}, {NULL}};
const Segment SGI248314[] = {{&S95097[44], 5}, {NULL}};
const Segment SGI248315[] = {{&S95097[44], 14}, {NULL}};
const Segment SGI248316[] = {{&S95097[30], 6}, {NULL}};
const Segment SGI248317[] = {{&S95097[107], 3}, {NULL}};
const Segment SGI248318[] = {{&S95097[72], 6}, {NULL}};
const Segment SGI248319[] = {{&S102092[34], 11}, {NULL}};
const Segment SGI248320[] = {{&S102092[27], 6}, {NULL}};
const Segment SGI248321[] = {{&S102092[98], 9}, {NULL}};
const Segment SGI248322[] = {{&S102092[71], 6}, {NULL}};
const Segment SGI248323[] = {{&S102125[34], 7}, {NULL}};
const Segment SGI248324[] = {{&S102125[27], 6}, {NULL}};
const Segment SGI248325[] = {{&S102125[102], 9}, {NULL}};
const Segment SGI248326[] = {{&S102125[75], 6}, {NULL}};
const Segment SGI248327[] = {{&S95249[34], 7}, {NULL}};
const Segment SGI248328[] = {{&S95249[27], 6}, {NULL}};
const Segment SGI248329[] = {{&S95249[93], 9}, {NULL}};
const Segment SGI248330[] = {{&S95249[66], 6}, {NULL}};
const Segment SGI248331[] = {{&S95254[34], 10}, {NULL}};
const Segment SGI248332[] = {{&S95254[27], 6}, {NULL}};
const Segment SGI248333[] = {{&S95254[96], 9}, {NULL}};
const Segment SGI248334[] = {{&S95254[69], 6}, {NULL}};
const Segment SGI248335[] = {{&S95310[34], 18}, {NULL}};
const Segment SGI248336[] = {{&S95310[27], 6}, {NULL}};
const Segment SGI248337[] = {{&S95310[105], 9}, {NULL}};
const Segment SGI248338[] = {{&S95310[78], 6}, {NULL}};
const Segment SGI248339[] = {{&S95310[146], 8}, {NULL}};
const Segment SGI248340[] = {{&S102300[59], 10}, {NULL}};
const Segment SGI248341[] = {{&S102300[59], 22}, {NULL}};
const Segment SGI248342[] = {{&S102300[45], 6}, {NULL}};
const Segment SGI248343[] = {{&S102300[130], 3}, {NULL}};
const Segment SGI248344[] = {{&S102300[95], 6}, {NULL}};
const Segment SGI248345[] = {{&S102300[11], 3}, {NULL}};
const Segment SGI248346[] = {{&S102306[34], 10}, {NULL}};
const Segment SGI248347[] = {{&S102306[27], 6}, {NULL}};
const Segment SGI248348[] = {{&S102306[97], 9}, {NULL}};
const Segment SGI248349[] = {{&S102306[70], 6}, {NULL}};
const Segment SGI248350[] = {{&S103178[44], 10}, {NULL}};
const Segment SGI248351[] = {{&S103178[44], 22}, {NULL}};
const Segment SGI248352[] = {{&S103178[30], 6}, {NULL}};
const Segment SGI248353[] = {{&S103178[133], 3}, {NULL}};
const Segment SGI248354[] = {{&S103178[98], 6}, {NULL}};
const Segment SGI248355[] = {{&S103178[68], 8}, {NULL}};
const Segment SGI248356[] = {{&S95338[34], 10}, {NULL}};
const Segment SGI248357[] = {{&S95338[27], 6}, {NULL}};
const Segment SGI248358[] = {{&S95338[97], 9}, {NULL}};
const Segment SGI248359[] = {{&S95338[70], 6}, {NULL}};
const Segment SGI248360[] = {{&S95343[34], 10}, {NULL}};
const Segment SGI248361[] = {{&S95343[27], 6}, {NULL}};
const Segment SGI248362[] = {{&S95343[96], 9}, {NULL}};
const Segment SGI248363[] = {{&S95343[69], 6}, {NULL}};
const Segment SGI248364[] = {{&S95343[130], 8}, {NULL}};
const Segment SGI248365[] = {{&S95344[44], 10}, {NULL}};
const Segment SGI248366[] = {{&S95344[44], 22}, {NULL}};
const Segment SGI248367[] = {{&S95344[30], 6}, {NULL}};
const Segment SGI248368[] = {{&S95344[115], 3}, {NULL}};
const Segment SGI248369[] = {{&S95344[80], 6}, {NULL}};
const Segment SGI248370[] = {{&S95346[44], 10}, {NULL}};
const Segment SGI248371[] = {{&S95346[44], 22}, {NULL}};
const Segment SGI248372[] = {{&S95346[30], 6}, {NULL}};
const Segment SGI248373[] = {{&S95346[115], 3}, {NULL}};
const Segment SGI248374[] = {{&S95346[80], 6}, {NULL}};
const Segment SGI248375[] = {{&S95346[147], 15}, {NULL}};
const Segment SGI248376[] = {{&S95509[34], 7}, {NULL}};
const Segment SGI248377[] = {{&S95509[27], 6}, {NULL}};
const Segment SGI248378[] = {{&S95509[93], 9}, {NULL}};
const Segment SGI248379[] = {{&S95509[66], 6}, {NULL}};
const Segment SGI248380[] = {{&S95515[34], 10}, {NULL}};
const Segment SGI248381[] = {{&S95515[27], 6}, {NULL}};
const Segment SGI248382[] = {{&S102515[44], 10}, {NULL}};
const Segment SGI248383[] = {{&S95515[96], 9}, {NULL}};
const Segment SGI248384[] = {{&S95515[69], 6}, {NULL}};
const Segment SGI248385[] = {{&S102515[44], 22}, {NULL}};
const Segment SGI248386[] = {{&S102515[30], 6}, {NULL}};
const Segment SGI248387[] = {{&S102515[134], 3}, {NULL}};
const Segment SGI248388[] = {{&S102515[99], 6}, {NULL}};
const Segment SGI248389[] = {{&S102515[162], 3}, {NULL}};
const Segment SGI248390[] = {{&S102515[68], 8}, {NULL}};
const Segment SGI248391[] = {{&S102525[59], 10}, {NULL}};
const Segment SGI248392[] = {{&S102525[59], 22}, {NULL}};
const Segment SGI248393[] = {{&S102525[45], 6}, {NULL}};
const Segment SGI248394[] = {{&S102525[130], 3}, {NULL}};
const Segment SGI248395[] = {{&S102525[95], 6}, {NULL}};
const Segment SGI248396[] = {{&S102525[0], 14}, {NULL}};
const Segment SGI248397[] = {{&S102528[44], 10}, {NULL}};
const Segment SGI248398[] = {{&S102528[44], 22}, {NULL}};
const Segment SGI248399[] = {{&S102528[30], 6}, {NULL}};
const Segment SGI248400[] = {{&S102528[113], 3}, {NULL}};
const Segment SGI248401[] = {{&S102528[80], 5}, {NULL}};
const Segment SGI248402[] = {{&S102528[117], 14}, {NULL}};
const Segment SGI248403[] = {{&S102531[44], 11}, {NULL}};
const Segment SGI248404[] = {{&S102531[44], 23}, {NULL}};
const Segment SGI248405[] = {{&S102531[30], 6}, {NULL}};
const Segment SGI248406[] = {{&S102531[114], 3}, {NULL}};
const Segment SGI248407[] = {{&S102531[81], 5}, {NULL}};
const Segment SGI248408[] = {{&S102531[118], 14}, {NULL}};
const Segment SGI248409[] = {{&S103424[44], 10}, {NULL}};
const Segment SGI248410[] = {{&S103424[44], 22}, {NULL}};
const Segment SGI248411[] = {{&S103424[30], 6}, {NULL}};
const Segment SGI248412[] = {{&S103424[115], 3}, {NULL}};
const Segment SGI248413[] = {{&S103424[80], 6}, {NULL}};
const Segment SGI248414[] = {{&S103424[150], 3}, {NULL}};
const Segment SGI248415[] = {{&S95573[34], 9}, {NULL}};
const Segment SGI248416[] = {{&S95573[27], 6}, {NULL}};
const Segment SGI248417[] = {{&S95573[95], 9}, {NULL}};
const Segment SGI248418[] = {{&S95573[68], 6}, {NULL}};
const Segment SGI248419[] = {{&S95595[44], 11}, {NULL}};
const Segment SGI248420[] = {{&S95595[44], 23}, {NULL}};
const Segment SGI248421[] = {{&S95595[30], 6}, {NULL}};
const Segment SGI248422[] = {{&S103465[34], 10}, {NULL}};
const Segment SGI248423[] = {{&S95595[116], 3}, {NULL}};
const Segment SGI248424[] = {{&S103465[27], 6}, {NULL}};
const Segment SGI248425[] = {{&S95595[81], 6}, {NULL}};
const Segment SGI248426[] = {{&S95595[141], 8}, {NULL}};
const Segment SGI248427[] = {{&S103465[97], 9}, {NULL}};
const Segment SGI248428[] = {{&S103465[70], 6}, {NULL}};
const Segment SGI248429[] = {{&S95604[34], 10}, {NULL}};
const Segment SGI248430[] = {{&S95604[27], 6}, {NULL}};
const Segment SGI248431[] = {{&S95604[97], 9}, {NULL}};
const Segment SGI248432[] = {{&S95604[70], 6}, {NULL}};
const Segment SGI248433[] = {{&S95607[34], 7}, {NULL}};
const Segment SGI248434[] = {{&S95607[27], 6}, {NULL}};
const Segment SGI248435[] = {{&S95607[98], 9}, {NULL}};
const Segment SGI248436[] = {{&S95607[71], 6}, {NULL}};
const Segment SGI248437[] = {{&S95612[34], 10}, {NULL}};
const Segment SGI248438[] = {{&S95612[27], 6}, {NULL}};
const Segment SGI248439[] = {{&S95612[97], 9}, {NULL}};
const Segment SGI248440[] = {{&S95612[70], 6}, {NULL}};
const Segment SGI248441[] = {{&S95617[34], 10}, {NULL}};
const Segment SGI248442[] = {{&S95617[27], 6}, {NULL}};
const Segment SGI248443[] = {{&S95617[96], 9}, {NULL}};
const Segment SGI248444[] = {{&S95617[69], 6}, {NULL}};
const Segment SGI248445[] = {{&S95619[34], 9}, {NULL}};
const Segment SGI248446[] = {{&S95619[27], 6}, {NULL}};
const Segment SGI248447[] = {{&S95619[96], 9}, {NULL}};
const Segment SGI248448[] = {{&S95619[69], 6}, {NULL}};
const Segment SGI248449[] = {{&S95626[44], 10}, {NULL}};
const Segment SGI248450[] = {{&S95626[44], 22}, {NULL}};
const Segment SGI248451[] = {{&S95626[30], 6}, {NULL}};
const Segment SGI248452[] = {{&S95626[115], 3}, {NULL}};
const Segment SGI248453[] = {{&S95626[80], 6}, {NULL}};
const Segment SGI248454[] = {{&S95627[52], 10}, {NULL}};
const Segment SGI248455[] = {{&S95627[52], 21}, {NULL}};
const Segment SGI248456[] = {{&S95627[38], 6}, {NULL}};
const Segment SGI248457[] = {{&S95627[87], 5}, {NULL}};
const Segment SGI248458[] = {{&S95639[34], 18}, {NULL}};
const Segment SGI248459[] = {{&S95639[27], 6}, {NULL}};
const Segment SGI248460[] = {{&S95639[104], 9}, {NULL}};
const Segment SGI248461[] = {{&S95639[77], 6}, {NULL}};
const Segment SGI248462[] = {{&S95639[145], 8}, {NULL}};
const Segment SGI248463[] = {{&S95641[44], 6}, {NULL}};
const Segment SGI248464[] = {{&S95641[44], 17}, {NULL}};
const Segment SGI248465[] = {{&S95641[30], 6}, {NULL}};
const Segment SGI248466[] = {{&S95641[110], 3}, {NULL}};
const Segment SGI248467[] = {{&S95641[75], 6}, {NULL}};
const Segment SGI248468[] = {{&S95653[34], 4}, {NULL}};
const Segment SGI248469[] = {{&S95653[27], 6}, {NULL}};
const Segment SGI248470[] = {{&S95653[90], 9}, {NULL}};
const Segment SGI248471[] = {{&S95653[63], 6}, {NULL}};
const Segment SGI248472[] = {{&S95659[34], 10}, {NULL}};
const Segment SGI248473[] = {{&S95659[27], 6}, {NULL}};
const Segment SGI248474[] = {{&S95659[97], 9}, {NULL}};
const Segment SGI248475[] = {{&S95659[70], 6}, {NULL}};
const Segment SGI248476[] = {{&S95797[34], 6}, {NULL}};
const Segment SGI248477[] = {{&S95797[27], 6}, {NULL}};
const Segment SGI248478[] = {{&S95797[92], 9}, {NULL}};
const Segment SGI248479[] = {{&S95797[65], 6}, {NULL}};
const Segment SGI248480[] = {{&S95853[44], 11}, {NULL}};
const Segment SGI248481[] = {{&S95853[44], 23}, {NULL}};
const Segment SGI248482[] = {{&S95853[30], 6}, {NULL}};
const Segment SGI248483[] = {{&S95853[139], 3}, {NULL}};
const Segment SGI248484[] = {{&S95853[104], 6}, {NULL}};
const Segment SGI248485[] = {{&S95853[76], 9}, {NULL}};
const Segment SGI248486[] = {{&S95862[34], 9}, {NULL}};
const Segment SGI248487[] = {{&S95862[27], 6}, {NULL}};
const Segment SGI248488[] = {{&S95862[96], 9}, {NULL}};
const Segment SGI248489[] = {{&S95862[69], 6}, {NULL}};
const Segment SGI248490[] = {{&S103781[34], 10}, {NULL}};
const Segment SGI248491[] = {{&S103781[27], 6}, {NULL}};
const Segment SGI248492[] = {{&S103781[97], 9}, {NULL}};
const Segment SGI248493[] = {{&S103781[70], 6}, {NULL}};
const Segment SGI248494[] = {{&S103783[44], 10}, {NULL}};
const Segment SGI248495[] = {{&S103783[44], 22}, {NULL}};
const Segment SGI248496[] = {{&S103783[30], 6}, {NULL}};
const Segment SGI248497[] = {{&S103783[133], 3}, {NULL}};
const Segment SGI248498[] = {{&S103783[98], 6}, {NULL}};
const Segment SGI248499[] = {{&S103783[161], 3}, {NULL}};
const Segment SGI248500[] = {{&S103783[68], 8}, {NULL}};
const Segment SGI248501[] = {{&S95890[38], 12}, {NULL}};
const Segment SGI248502[] = {{&S95890[31], 6}, {NULL}};
const Segment SGI248503[] = {{&S103786[34], 10}, {NULL}};
const Segment SGI248504[] = {{&S103786[27], 6}, {NULL}};
const Segment SGI248505[] = {{&S103786[97], 9}, {NULL}};
const Segment SGI248506[] = {{&S103786[70], 6}, {NULL}};
const Segment SGI248507[] = {{&S95895[34], 7}, {NULL}};
const Segment SGI248508[] = {{&S95895[27], 6}, {NULL}};
const Segment SGI248509[] = {{&S95895[93], 9}, {NULL}};
const Segment SGI248510[] = {{&S95895[66], 6}, {NULL}};
const Segment SGI248511[] = {{&S95902[34], 10}, {NULL}};
const Segment SGI248512[] = {{&S95902[27], 6}, {NULL}};
const Segment SGI248513[] = {{&S95902[97], 9}, {NULL}};
const Segment SGI248514[] = {{&S95902[70], 6}, {NULL}};
const Segment SGI248515[] = {{&S95910[44], 10}, {NULL}};
const Segment SGI248516[] = {{&S95910[44], 21}, {NULL}};
const Segment SGI248517[] = {{&S95910[30], 6}, {NULL}};
const Segment SGI248518[] = {{&S95910[114], 3}, {NULL}};
const Segment SGI248519[] = {{&S95910[79], 6}, {NULL}};
const Segment SGI248520[] = {{&S95910[139], 13}, {NULL}};
const Segment SGI248521[] = {{&S95913[44], 12}, {NULL}};
const Segment SGI248522[] = {{&S95913[44], 23}, {NULL}};
const Segment SGI248523[] = {{&S95913[30], 6}, {NULL}};
const Segment SGI248524[] = {{&S95913[114], 3}, {NULL}};
const Segment SGI248525[] = {{&S95913[81], 5}, {NULL}};
const Segment SGI248526[] = {{&S95918[44], 10}, {NULL}};
const Segment SGI248527[] = {{&S95918[44], 21}, {NULL}};
const Segment SGI248528[] = {{&S95918[30], 6}, {NULL}};
const Segment SGI248529[] = {{&S95918[112], 3}, {NULL}};
const Segment SGI248530[] = {{&S95918[79], 5}, {NULL}};
const Segment SGI248531[] = {{&S95931[44], 10}, {NULL}};
const Segment SGI248532[] = {{&S95931[44], 21}, {NULL}};
const Segment SGI248533[] = {{&S95931[30], 6}, {NULL}};
const Segment SGI248534[] = {{&S95931[79], 6}, {NULL}};
const Segment SGI248535[] = {{&S95931[106], 13}, {NULL}};
const Segment SGI248536[] = {{&S95937[34], 12}, {NULL}};
const Segment SGI248537[] = {{&S95937[27], 6}, {NULL}};
const Segment SGI248538[] = {{&S95937[99], 9}, {NULL}};
const Segment SGI248539[] = {{&S95937[72], 6}, {NULL}};
const Segment SGI248540[] = {{&S96069[44], 10}, {NULL}};
const Segment SGI248541[] = {{&S96069[44], 21}, {NULL}};
const Segment SGI248542[] = {{&S96069[30], 6}, {NULL}};
const Segment SGI248543[] = {{&S96069[114], 3}, {NULL}};
const Segment SGI248544[] = {{&S96069[79], 6}, {NULL}};
const Segment SGI248545[] = {{&S96089[34], 10}, {NULL}};
const Segment SGI248546[] = {{&S96089[27], 6}, {NULL}};
const Segment SGI248547[] = {{&S96089[96], 9}, {NULL}};
const Segment SGI248548[] = {{&S96089[69], 6}, {NULL}};
const Segment SGI248549[] = {{&S102907[38], 10}, {NULL}};
const Segment SGI248550[] = {{&S102907[31], 6}, {NULL}};
const Segment SGI248551[] = {{&S96122[34], 12}, {NULL}};
const Segment SGI248552[] = {{&S96122[27], 6}, {NULL}};
const Segment SGI248553[] = {{&S96122[98], 9}, {NULL}};
const Segment SGI248554[] = {{&S96122[71], 6}, {NULL}};
const Segment SGI248555[] = {{&S102918[44], 10}, {NULL}};
const Segment SGI248556[] = {{&S102918[44], 22}, {NULL}};
const Segment SGI248557[] = {{&S102918[30], 6}, {NULL}};
const Segment SGI248558[] = {{&S102918[131], 3}, {NULL}};
const Segment SGI248559[] = {{&S102918[96], 6}, {NULL}};
const Segment SGI248560[] = {{&S102918[159], 3}, {NULL}};
const Segment SGI248561[] = {{&S102918[68], 8}, {NULL}};
const Segment SGI248562[] = {{&S96145[34], 25}, {NULL}};
const Segment SGI248563[] = {{&S96145[27], 6}, {NULL}};
const Segment SGI248564[] = {{&S96145[112], 9}, {NULL}};
const Segment SGI248565[] = {{&S96145[85], 6}, {NULL}};
const Segment SGI248566[] = {{&S96157[44], 10}, {NULL}};
const Segment SGI248567[] = {{&S96157[44], 21}, {NULL}};
const Segment SGI248568[] = {{&S96157[30], 6}, {NULL}};
const Segment SGI248569[] = {{&S96157[112], 3}, {NULL}};
const Segment SGI248570[] = {{&S96157[79], 5}, {NULL}};
const Segment SGI248571[] = {{&S96174[44], 9}, {NULL}};
const Segment SGI248572[] = {{&S96174[44], 21}, {NULL}};
const Segment SGI248573[] = {{&S96174[30], 6}, {NULL}};
const Segment SGI248574[] = {{&S96174[137], 3}, {NULL}};
const Segment SGI248575[] = {{&S96174[102], 6}, {NULL}};
const Segment SGI248576[] = {{&S96174[74], 9}, {NULL}};
const Segment SGI248577[] = {{&S96179[34], 17}, {NULL}};
const Segment SGI248578[] = {{&S96179[27], 6}, {NULL}};
const Segment SGI248579[] = {{&S96179[103], 9}, {NULL}};
const Segment SGI248580[] = {{&S96179[76], 6}, {NULL}};
const Segment SGI248581[] = {{&S96188[44], 12}, {NULL}};
const Segment SGI248582[] = {{&S96188[44], 23}, {NULL}};
const Segment SGI248583[] = {{&S96188[30], 6}, {NULL}};
const Segment SGI248584[] = {{&S96188[114], 3}, {NULL}};
const Segment SGI248585[] = {{&S96188[81], 5}, {NULL}};
const Segment SGI248586[] = {{&S96216[44], 10}, {NULL}};
const Segment SGI248587[] = {{&S96216[44], 21}, {NULL}};
const Segment SGI248588[] = {{&S96216[30], 6}, {NULL}};
const Segment SGI248589[] = {{&S96216[112], 3}, {NULL}};
const Segment SGI248590[] = {{&S96216[79], 5}, {NULL}};
const Segment SGI248591[] = {{&S96229[44], 10}, {NULL}};
const Segment SGI248592[] = {{&S96229[44], 21}, {NULL}};
const Segment SGI248593[] = {{&S96229[30], 6}, {NULL}};
const Segment SGI248594[] = {{&S96229[79], 6}, {NULL}};
const Segment SGI248595[] = {{&S96229[106], 13}, {NULL}};
const Segment SGI248596[] = {{&S96241[44], 7}, {NULL}};
const Segment SGI248597[] = {{&S96241[44], 18}, {NULL}};
const Segment SGI248598[] = {{&S96241[30], 6}, {NULL}};
const Segment SGI248599[] = {{&S96241[111], 3}, {NULL}};
const Segment SGI248600[] = {{&S96241[76], 6}, {NULL}};
const Segment SGI248601[] = {{&S96257[44], 15}, {NULL}};
const Segment SGI248602[] = {{&S96257[44], 33}, {NULL}};
const Segment SGI248603[] = {{&S96257[30], 6}, {NULL}};
const Segment SGI248604[] = {{&S96257[91], 6}, {NULL}};
const Segment SGI248605[] = {{&S96257[118], 13}, {NULL}};
const Segment SGI248606[] = {{&S96261[41], 10}, {NULL}};
const Segment SGI248607[] = {{&S96261[41], 21}, {NULL}};
const Segment SGI248608[] = {{&S96261[27], 6}, {NULL}};
const Segment SGI248609[] = {{&S96261[111], 3}, {NULL}};
const Segment SGI248610[] = {{&S96261[115], 9}, {NULL}};
const Segment SGI248611[] = {{&S96261[76], 6}, {NULL}};
const Segment SGI248612[] = {{&S96421[44], 9}, {NULL}};
const Segment SGI248613[] = {{&S96421[44], 20}, {NULL}};
const Segment SGI248614[] = {{&S96421[30], 6}, {NULL}};
const Segment SGI248615[] = {{&S96421[78], 6}, {NULL}};
const Segment SGI248616[] = {{&S96421[105], 13}, {NULL}};
const Segment SGI248617[] = {{&S96482[34], 18}, {NULL}};
const Segment SGI248618[] = {{&S96482[27], 6}, {NULL}};
const Segment SGI248619[] = {{&S96482[104], 9}, {NULL}};
const Segment SGI248620[] = {{&S96482[77], 6}, {NULL}};
const Segment SGI248621[] = {{&S96497[44], 11}, {NULL}};
const Segment SGI248622[] = {{&S96497[44], 22}, {NULL}};
const Segment SGI248623[] = {{&S96497[30], 6}, {NULL}};
const Segment SGI248624[] = {{&S96497[115], 3}, {NULL}};
const Segment SGI248625[] = {{&S96497[80], 6}, {NULL}};
const Segment SGI248626[] = {{&S96507[34], 24}, {NULL}};
const Segment SGI248627[] = {{&S96507[27], 6}, {NULL}};
const Segment SGI248628[] = {{&S96507[110], 9}, {NULL}};
const Segment SGI248629[] = {{&S96507[83], 6}, {NULL}};
const Segment SGI248630[] = {{&S96518[44], 10}, {NULL}};
const Segment SGI248631[] = {{&S96518[44], 21}, {NULL}};
const Segment SGI248632[] = {{&S96518[30], 6}, {NULL}};
const Segment SGI248633[] = {{&S96518[112], 3}, {NULL}};
const Segment SGI248634[] = {{&S96518[79], 5}, {NULL}};
const Segment SGI248635[] = {{&S96532[34], 10}, {NULL}};
const Segment SGI248636[] = {{&S96532[27], 6}, {NULL}};
const Segment SGI248637[] = {{&S96532[96], 9}, {NULL}};
const Segment SGI248638[] = {{&S96532[69], 6}, {NULL}};
const Segment SGI248639[] = {{&S96535[34], 19}, {NULL}};
const Segment SGI248640[] = {{&S96535[27], 6}, {NULL}};
const Segment SGI248641[] = {{&S96535[105], 9}, {NULL}};
const Segment SGI248642[] = {{&S96535[78], 6}, {NULL}};
const Segment SGI248643[] = {{&S96541[44], 9}, {NULL}};
const Segment SGI248644[] = {{&S96541[44], 20}, {NULL}};
const Segment SGI248645[] = {{&S96541[30], 6}, {NULL}};
const Segment SGI248646[] = {{&S96541[113], 3}, {NULL}};
const Segment SGI248647[] = {{&S96541[78], 6}, {NULL}};
const Segment SGI248648[] = {{&S96541[148], 3}, {NULL}};
const Segment SGI248649[] = {{&S96544[38], 9}, {NULL}};
const Segment SGI248650[] = {{&S96544[31], 6}, {NULL}};
const Segment SGI248651[] = {{&S96556[34], 10}, {NULL}};
const Segment SGI248652[] = {{&S96556[27], 6}, {NULL}};
const Segment SGI248653[] = {{&S96556[96], 9}, {NULL}};
const Segment SGI248654[] = {{&S96556[69], 6}, {NULL}};
const Segment SGI248655[] = {{&S96560[34], 10}, {NULL}};
const Segment SGI248656[] = {{&S96560[27], 6}, {NULL}};
const Segment SGI248657[] = {{&S96560[96], 9}, {NULL}};
const Segment SGI248658[] = {{&S96560[69], 6}, {NULL}};
const Segment SGI248659[] = {{&S96560[137], 8}, {NULL}};
const Segment SGI248660[] = {{&S96562[44], 18}, {NULL}};
const Segment SGI248661[] = {{&S96562[44], 29}, {NULL}};
const Segment SGI248662[] = {{&S96562[30], 6}, {NULL}};
const Segment SGI248663[] = {{&S96562[122], 3}, {NULL}};
const Segment SGI248664[] = {{&S96562[87], 6}, {NULL}};
const Segment SGI248665[] = {{&S96562[147], 7}, {NULL}};
const Segment SGI248666[] = {{&S96563[44], 10}, {NULL}};
const Segment SGI248667[] = {{&S96563[44], 21}, {NULL}};
const Segment SGI248668[] = {{&S96563[30], 6}, {NULL}};
const Segment SGI248669[] = {{&S96563[112], 3}, {NULL}};
const Segment SGI248670[] = {{&S96563[79], 5}, {NULL}};
const Segment SGI248671[] = {{&S96588[34], 10}, {NULL}};
const Segment SGI248672[] = {{&S96588[27], 6}, {NULL}};
const Segment SGI248673[] = {{&S96588[96], 9}, {NULL}};
const Segment SGI248674[] = {{&S96588[69], 6}, {NULL}};
const Segment SGI248675[] = {{&S103194[44], 11}, {NULL}};
const Segment SGI248676[] = {{&S103194[44], 23}, {NULL}};
const Segment SGI248677[] = {{&S103194[30], 6}, {NULL}};
const Segment SGI248678[] = {{&S103194[115], 3}, {NULL}};
const Segment SGI248679[] = {{&S103194[81], 5}, {NULL}};
const Segment SGI248680[] = {{&S103194[126], 14}, {NULL}};
const Segment SGI248681[] = {{&S104468[44], 10}, {NULL}};
const Segment SGI248682[] = {{&S104468[44], 22}, {NULL}};
const Segment SGI248683[] = {{&S104468[30], 6}, {NULL}};
const Segment SGI248684[] = {{&S103195[59], 11}, {NULL}};
const Segment SGI248685[] = {{&S103195[59], 23}, {NULL}};
const Segment SGI248686[] = {{&S103195[45], 6}, {NULL}};
const Segment SGI248687[] = {{&S104468[115], 3}, {NULL}};
const Segment SGI248688[] = {{&S104468[80], 6}, {NULL}};
const Segment SGI248689[] = {{&S104468[132], 10}, {NULL}};
const Segment SGI248690[] = {{&S103195[96], 5}, {NULL}};
const Segment SGI248691[] = {{&S103195[0], 14}, {NULL}};
const Segment SGI248692[] = {{&S104469[44], 10}, {NULL}};
const Segment SGI248693[] = {{&S104469[44], 22}, {NULL}};
const Segment SGI248694[] = {{&S104469[30], 6}, {NULL}};
const Segment SGI248695[] = {{&S104469[115], 3}, {NULL}};
const Segment SGI248696[] = {{&S104469[80], 6}, {NULL}};
const Segment SGI248697[] = {{&S104469[139], 10}, {NULL}};
const Segment SGI248698[] = {{&S96696[44], 10}, {NULL}};
const Segment SGI248699[] = {{&S96696[44], 21}, {NULL}};
const Segment SGI248700[] = {{&S96696[30], 6}, {NULL}};
const Segment SGI248701[] = {{&S96696[132], 3}, {NULL}};
const Segment SGI248702[] = {{&S96696[97], 6}, {NULL}};
const Segment SGI248703[] = {{&S96696[167], 3}, {NULL}};
const Segment SGI248704[] = {{&S96696[67], 8}, {NULL}};
const Segment SGI248705[] = {{&S96697[44], 10}, {NULL}};
const Segment SGI248706[] = {{&S96697[44], 21}, {NULL}};
const Segment SGI248707[] = {{&S96697[30], 6}, {NULL}};
const Segment SGI248708[] = {{&S96697[132], 3}, {NULL}};
const Segment SGI248709[] = {{&S96697[97], 6}, {NULL}};
const Segment SGI248710[] = {{&S96697[160], 3}, {NULL}};
const Segment SGI248711[] = {{&S96697[67], 8}, {NULL}};
const Segment SGI248712[] = {{&S96712[34], 13}, {NULL}};
const Segment SGI248713[] = {{&S96712[27], 6}, {NULL}};
const Segment SGI248714[] = {{&S96712[100], 9}, {NULL}};
const Segment SGI248715[] = {{&S96712[73], 6}, {NULL}};
const Segment SGI248716[] = {{&S104501[44], 10}, {NULL}};
const Segment SGI248717[] = {{&S104501[44], 22}, {NULL}};
const Segment SGI248718[] = {{&S104501[30], 6}, {NULL}};
const Segment SGI248719[] = {{&S104501[133], 3}, {NULL}};
const Segment SGI248720[] = {{&S104501[98], 6}, {NULL}};
const Segment SGI248721[] = {{&S104501[68], 8}, {NULL}};
const Segment SGI248722[] = {{&S104501[150], 10}, {NULL}};
const Segment SGI248723[] = {{&S104509[44], 10}, {NULL}};
const Segment SGI248724[] = {{&S104509[44], 22}, {NULL}};
const Segment SGI248725[] = {{&S104509[30], 6}, {NULL}};
const Segment SGI248726[] = {{&S104509[133], 3}, {NULL}};
const Segment SGI248727[] = {{&S104509[98], 6}, {NULL}};
const Segment SGI248728[] = {{&S104509[161], 3}, {NULL}};
const Segment SGI248729[] = {{&S104509[68], 8}, {NULL}};
const Segment SGI248730[] = {{&S96736[34], 9}, {NULL}};
const Segment SGI248731[] = {{&S96736[27], 6}, {NULL}};
const Segment SGI248732[] = {{&S96736[95], 9}, {NULL}};
const Segment SGI248733[] = {{&S96736[68], 6}, {NULL}};
const Segment SGI248734[] = {{&S96781[34], 15}, {NULL}};
const Segment SGI248735[] = {{&S96781[27], 6}, {NULL}};
const Segment SGI248736[] = {{&S96781[102], 9}, {NULL}};
const Segment SGI248737[] = {{&S96781[75], 6}, {NULL}};
const Segment SGI248738[] = {{&S96781[143], 8}, {NULL}};
const Segment SGI248739[] = {{&S96799[44], 18}, {NULL}};
const Segment SGI248740[] = {{&S96799[44], 29}, {NULL}};
const Segment SGI248741[] = {{&S96799[30], 6}, {NULL}};
const Segment SGI248742[] = {{&S96799[122], 3}, {NULL}};
const Segment SGI248743[] = {{&S96799[87], 6}, {NULL}};
const Segment SGI248744[] = {{&S96799[157], 3}, {NULL}};
const Segment SGI248745[] = {{&S96800[44], 14}, {NULL}};
const Segment SGI248746[] = {{&S96800[44], 25}, {NULL}};
const Segment SGI248747[] = {{&S96800[30], 6}, {NULL}};
const Segment SGI248748[] = {{&S96800[118], 3}, {NULL}};
const Segment SGI248749[] = {{&S96800[83], 6}, {NULL}};
const Segment SGI248750[] = {{&S96803[34], 13}, {NULL}};
const Segment SGI248751[] = {{&S96803[27], 6}, {NULL}};
const Segment SGI248752[] = {{&S96803[99], 9}, {NULL}};
const Segment SGI248753[] = {{&S96803[72], 6}, {NULL}};
const Segment SGI248754[] = {{&S96808[44], 10}, {NULL}};
const Segment SGI248755[] = {{&S96808[44], 21}, {NULL}};
const Segment SGI248756[] = {{&S96808[30], 6}, {NULL}};
const Segment SGI248757[] = {{&S96808[114], 3}, {NULL}};
const Segment SGI248758[] = {{&S96808[79], 6}, {NULL}};
const Segment SGI248759[] = {{&S96808[139], 7}, {NULL}};
const Segment SGI248760[] = {{&S96826[52], 14}, {NULL}};
const Segment SGI248761[] = {{&S96826[52], 25}, {NULL}};
const Segment SGI248762[] = {{&S96826[38], 6}, {NULL}};
const Segment SGI248763[] = {{&S96826[91], 5}, {NULL}};
const Segment SGI248764[] = {{&S103426[44], 10}, {NULL}};
const Segment SGI248765[] = {{&S103426[44], 22}, {NULL}};
const Segment SGI248766[] = {{&S103426[30], 6}, {NULL}};
const Segment SGI248767[] = {{&S103426[115], 3}, {NULL}};
const Segment SGI248768[] = {{&S103426[80], 6}, {NULL}};
const Segment SGI248769[] = {{&S96835[34], 10}, {NULL}};
const Segment SGI248770[] = {{&S96835[27], 6}, {NULL}};
const Segment SGI248771[] = {{&S96835[97], 9}, {NULL}};
const Segment SGI248772[] = {{&S96835[70], 6}, {NULL}};
const Segment SGI248773[] = {{&S96837[34], 10}, {NULL}};
const Segment SGI248774[] = {{&S96837[27], 6}, {NULL}};
const Segment SGI248775[] = {{&S96837[96], 9}, {NULL}};
const Segment SGI248776[] = {{&S96837[69], 6}, {NULL}};
const Segment SGI248777[] = {{&S103438[31], 8}, {NULL}};
const Segment SGI248778[] = {{&S103438[54], 3}, {NULL}};
const Segment SGI248779[] = {{&S103444[45], 10}, {NULL}};
const Segment SGI248780[] = {{&S103444[30], 6}, {NULL}};
const Segment SGI248781[] = {{&S103444[115], 3}, {NULL}};
const Segment SGI248782[] = {{&S103444[81], 5}, {NULL}};
const Segment SGI248783[] = {{&S103444[139], 13}, {NULL}};
const Segment SGI248784[] = {{&S103452[44], 10}, {NULL}};
const Segment SGI248785[] = {{&S103452[44], 22}, {NULL}};
const Segment SGI248786[] = {{&S103452[30], 6}, {NULL}};
const Segment SGI248787[] = {{&S103452[131], 3}, {NULL}};
const Segment SGI248788[] = {{&S103452[96], 6}, {NULL}};
const Segment SGI248789[] = {{&S103452[159], 3}, {NULL}};
const Segment SGI248790[] = {{&S103452[68], 8}, {NULL}};
const Segment SGI248791[] = {{&S103453[44], 10}, {NULL}};
const Segment SGI248792[] = {{&S103453[44], 22}, {NULL}};
const Segment SGI248793[] = {{&S103453[30], 6}, {NULL}};
const Segment SGI248794[] = {{&S103453[131], 3}, {NULL}};
const Segment SGI248795[] = {{&S103453[96], 6}, {NULL}};
const Segment SGI248796[] = {{&S103453[159], 3}, {NULL}};
const Segment SGI248797[] = {{&S103453[68], 8}, {NULL}};
const Segment SGI248798[] = {{&S103457[44], 10}, {NULL}};
const Segment SGI248799[] = {{&S103457[44], 22}, {NULL}};
const Segment SGI248800[] = {{&S103457[30], 6}, {NULL}};
const Segment SGI248801[] = {{&S103457[135], 3}, {NULL}};
const Segment SGI248802[] = {{&S103457[100], 6}, {NULL}};
const Segment SGI248803[] = {{&S103457[163], 3}, {NULL}};
const Segment SGI248804[] = {{&S103457[68], 8}, {NULL}};
const Segment SGI248805[] = {{&S96872[44], 21}, {NULL}};
const Segment SGI248806[] = {{&S96872[44], 32}, {NULL}};
const Segment SGI248807[] = {{&S96872[30], 6}, {NULL}};
const Segment SGI248808[] = {{&S96872[125], 3}, {NULL}};
const Segment SGI248809[] = {{&S96872[90], 6}, {NULL}};
const Segment SGI248810[] = {{&S96883[34], 7}, {NULL}};
const Segment SGI248811[] = {{&S96883[27], 6}, {NULL}};
const Segment SGI248812[] = {{&S96883[98], 9}, {NULL}};
const Segment SGI248813[] = {{&S96883[71], 6}, {NULL}};
const Segment SGI248814[] = {{&S96885[34], 6}, {NULL}};
const Segment SGI248815[] = {{&S96885[27], 6}, {NULL}};
const Segment SGI248816[] = {{&S96885[93], 9}, {NULL}};
const Segment SGI248817[] = {{&S96885[66], 6}, {NULL}};
const Segment SGI248818[] = {{&S96886[34], 19}, {NULL}};
const Segment SGI248819[] = {{&S96886[27], 6}, {NULL}};
const Segment SGI248820[] = {{&S96886[105], 9}, {NULL}};
const Segment SGI248821[] = {{&S96886[78], 6}, {NULL}};
const Segment SGI248822[] = {{&S97079[34], 11}, {NULL}};
const Segment SGI248823[] = {{&S97079[27], 6}, {NULL}};
const Segment SGI248824[] = {{&S97079[97], 9}, {NULL}};
const Segment SGI248825[] = {{&S97079[70], 6}, {NULL}};
const Segment SGI248826[] = {{&S104934[44], 10}, {NULL}};
const Segment SGI248827[] = {{&S104934[44], 22}, {NULL}};
const Segment SGI248828[] = {{&S104934[30], 6}, {NULL}};
const Segment SGI248829[] = {{&S104934[115], 3}, {NULL}};
const Segment SGI248830[] = {{&S104934[80], 6}, {NULL}};
const Segment SGI248831[] = {{&S104934[143], 3}, {NULL}};
const Segment SGI248832[] = {{&S104955[31], 8}, {NULL}};
const Segment SGI248833[] = {{&S104955[54], 3}, {NULL}};
const Segment SGI248834[] = {{&S104958[44], 10}, {NULL}};
const Segment SGI248835[] = {{&S104958[44], 22}, {NULL}};
const Segment SGI248836[] = {{&S104958[30], 6}, {NULL}};
const Segment SGI248837[] = {{&S104958[133], 3}, {NULL}};
const Segment SGI248838[] = {{&S104958[98], 6}, {NULL}};
const Segment SGI248839[] = {{&S104958[68], 8}, {NULL}};
const Segment SGI248840[] = {{&S104958[151], 11}, {NULL}};
const Segment SGI248841[] = {{&S97162[44], 10}, {NULL}};
const Segment SGI248842[] = {{&S97162[44], 21}, {NULL}};
const Segment SGI248843[] = {{&S97162[30], 6}, {NULL}};
const Segment SGI248844[] = {{&S97162[79], 6}, {NULL}};
const Segment SGI248845[] = {{&S97162[106], 13}, {NULL}};
const Segment SGI248846[] = {{&S97171[44], 12}, {NULL}};
const Segment SGI248847[] = {{&S97171[44], 24}, {NULL}};
const Segment SGI248848[] = {{&S97171[30], 6}, {NULL}};
const Segment SGI248849[] = {{&S97171[148], 3}, {NULL}};
const Segment SGI248850[] = {{&S97171[113], 6}, {NULL}};
const Segment SGI248851[] = {{&S97171[77], 9}, {NULL}};
const Segment SGI248852[] = {{&S97173[34], 11}, {NULL}};
const Segment SGI248853[] = {{&S97173[27], 6}, {NULL}};
const Segment SGI248854[] = {{&S97173[97], 9}, {NULL}};
const Segment SGI248855[] = {{&S97173[70], 6}, {NULL}};
const Segment SGI248856[] = {{&S97188[44], 10}, {NULL}};
const Segment SGI248857[] = {{&S97188[44], 22}, {NULL}};
const Segment SGI248858[] = {{&S97188[30], 6}, {NULL}};
const Segment SGI248859[] = {{&S97188[115], 3}, {NULL}};
const Segment SGI248860[] = {{&S97188[80], 6}, {NULL}};
const Segment SGI248861[] = {{&S97189[44], 8}, {NULL}};
const Segment SGI248862[] = {{&S97189[44], 19}, {NULL}};
const Segment SGI248863[] = {{&S97189[30], 6}, {NULL}};
const Segment SGI248864[] = {{&S97189[112], 3}, {NULL}};
const Segment SGI248865[] = {{&S97189[77], 6}, {NULL}};
const Segment SGI248866[] = {{&S97190[34], 12}, {NULL}};
const Segment SGI248867[] = {{&S97190[27], 6}, {NULL}};
const Segment SGI248868[] = {{&S97190[99], 9}, {NULL}};
const Segment SGI248869[] = {{&S97190[72], 6}, {NULL}};
const Segment SGI248870[] = {{&S97207[44], 10}, {NULL}};
const Segment SGI248871[] = {{&S97207[44], 21}, {NULL}};
const Segment SGI248872[] = {{&S97207[30], 6}, {NULL}};
const Segment SGI248873[] = {{&S97207[114], 3}, {NULL}};
const Segment SGI248874[] = {{&S97207[79], 6}, {NULL}};
const Segment SGI248875[] = {{&S103799[44], 11}, {NULL}};
const Segment SGI248876[] = {{&S103799[44], 23}, {NULL}};
const Segment SGI248877[] = {{&S103799[30], 6}, {NULL}};
const Segment SGI248878[] = {{&S103799[116], 3}, {NULL}};
const Segment SGI248879[] = {{&S103799[81], 6}, {NULL}};
const Segment SGI248880[] = {{&S103799[151], 3}, {NULL}};
const Segment SGI248881[] = {{&S103800[34], 11}, {NULL}};
const Segment SGI248882[] = {{&S103800[27], 6}, {NULL}};
const Segment SGI248883[] = {{&S103800[98], 9}, {NULL}};
const Segment SGI248884[] = {{&S103800[71], 6}, {NULL}};
const Segment SGI248885[] = {{&S103809[34], 10}, {NULL}};
const Segment SGI248886[] = {{&S103809[27], 6}, {NULL}};
const Segment SGI248887[] = {{&S103809[97], 9}, {NULL}};
const Segment SGI248888[] = {{&S103809[70], 6}, {NULL}};
const Segment SGI248889[] = {{&S104991[44], 10}, {NULL}};
const Segment SGI248890[] = {{&S104991[44], 22}, {NULL}};
const Segment SGI248891[] = {{&S104991[30], 6}, {NULL}};
const Segment SGI248892[] = {{&S104991[133], 3}, {NULL}};
const Segment SGI248893[] = {{&S104991[98], 6}, {NULL}};
const Segment SGI248894[] = {{&S104991[68], 8}, {NULL}};
const Segment SGI248895[] = {{&S104991[151], 11}, {NULL}};
const Segment SGI248896[] = {{&S97399[34], 10}, {NULL}};
const Segment SGI248897[] = {{&S97399[27], 6}, {NULL}};
const Segment SGI248898[] = {{&S97399[96], 9}, {NULL}};
const Segment SGI248899[] = {{&S97399[69], 6}, {NULL}};
const Segment SGI248900[] = {{&S105021[45], 10}, {NULL}};
const Segment SGI248901[] = {{&S105021[30], 6}, {NULL}};
const Segment SGI248902[] = {{&S105021[115], 3}, {NULL}};
const Segment SGI248903[] = {{&S105021[81], 5}, {NULL}};
const Segment SGI248904[] = {{&S105021[149], 3}, {NULL}};
const Segment SGI248905[] = {{&S105024[44], 10}, {NULL}};
const Segment SGI248906[] = {{&S105024[44], 22}, {NULL}};
const Segment SGI248907[] = {{&S105024[30], 6}, {NULL}};
const Segment SGI248908[] = {{&S105024[152], 3}, {NULL}};
const Segment SGI248909[] = {{&S105024[117], 6}, {NULL}};
const Segment SGI248910[] = {{&S105024[68], 8}, {NULL}};
const Segment SGI248911[] = {{&S97441[34], 18}, {NULL}};
const Segment SGI248912[] = {{&S97441[27], 6}, {NULL}};
const Segment SGI248913[] = {{&S97441[104], 9}, {NULL}};
const Segment SGI248914[] = {{&S97441[77], 6}, {NULL}};
const Segment SGI248915[] = {{&S97444[44], 10}, {NULL}};
const Segment SGI248916[] = {{&S97444[44], 21}, {NULL}};
const Segment SGI248917[] = {{&S97444[30], 6}, {NULL}};
const Segment SGI248918[] = {{&S97444[114], 3}, {NULL}};
const Segment SGI248919[] = {{&S97444[79], 6}, {NULL}};
const Segment SGI248920[] = {{&S97444[139], 11}, {NULL}};
const Segment SGI248921[] = {{&S97447[34], 19}, {NULL}};
const Segment SGI248922[] = {{&S97447[27], 6}, {NULL}};
const Segment SGI248923[] = {{&S97447[105], 9}, {NULL}};
const Segment SGI248924[] = {{&S97447[78], 6}, {NULL}};
const Segment SGI248925[] = {{&S97447[139], 8}, {NULL}};
const Segment SGI248926[] = {{&S97454[34], 10}, {NULL}};
const Segment SGI248927[] = {{&S97454[27], 6}, {NULL}};
const Segment SGI248928[] = {{&S97454[96], 9}, {NULL}};
const Segment SGI248929[] = {{&S97454[69], 6}, {NULL}};
const Segment SGI248930[] = {{&S97455[44], 9}, {NULL}};
const Segment SGI248931[] = {{&S97455[44], 20}, {NULL}};
const Segment SGI248932[] = {{&S97455[30], 6}, {NULL}};
const Segment SGI248933[] = {{&S97455[113], 3}, {NULL}};
const Segment SGI248934[] = {{&S97455[78], 6}, {NULL}};
const Segment SGI248935[] = {{&S97455[131], 13}, {NULL}};
const Segment SGI248936[] = {{&S97465[34], 17}, {NULL}};
const Segment SGI248937[] = {{&S97465[27], 6}, {NULL}};
const Segment SGI248938[] = {{&S97465[104], 9}, {NULL}};
const Segment SGI248939[] = {{&S97465[77], 6}, {NULL}};
const Segment SGI248940[] = {{&S97467[34], 8}, {NULL}};
const Segment SGI248941[] = {{&S97467[27], 6}, {NULL}};
const Segment SGI248942[] = {{&S97467[103], 9}, {NULL}};
const Segment SGI248943[] = {{&S97467[76], 6}, {NULL}};
const Segment SGI248944[] = {{&S104015[44], 9}, {NULL}};
const Segment SGI248945[] = {{&S104015[44], 21}, {NULL}};
const Segment SGI248946[] = {{&S104015[30], 6}, {NULL}};
const Segment SGI248947[] = {{&S104015[114], 3}, {NULL}};
const Segment SGI248948[] = {{&S104015[79], 6}, {NULL}};
const Segment SGI248949[] = {{&S104015[131], 10}, {NULL}};
const Segment SGI248950[] = {{&S97489[6], 4}, {NULL}};
const Segment SGI248951[] = {{&S97489[82], 5}, {NULL}};
const Segment SGI248952[] = {{&S97495[34], 10}, {NULL}};
const Segment SGI248953[] = {{&S97495[27], 6}, {NULL}};
const Segment SGI248954[] = {{&S97495[97], 9}, {NULL}};
const Segment SGI248955[] = {{&S97495[70], 6}, {NULL}};
const Segment SGI248956[] = {{&S97512[44], 10}, {NULL}};
const Segment SGI248957[] = {{&S97512[44], 22}, {NULL}};
const Segment SGI248958[] = {{&S97512[30], 6}, {NULL}};
const Segment SGI248959[] = {{&S97512[115], 3}, {NULL}};
const Segment SGI248960[] = {{&S97512[80], 6}, {NULL}};
const Segment SGI248961[] = {{&S97512[140], 8}, {NULL}};
const Segment SGI248962[] = {{&S97544[44], 11}, {NULL}};
const Segment SGI248963[] = {{&S97544[44], 22}, {NULL}};
const Segment SGI248964[] = {{&S97544[30], 6}, {NULL}};
const Segment SGI248965[] = {{&S97544[115], 3}, {NULL}};
const Segment SGI248966[] = {{&S97544[80], 6}, {NULL}};
const Segment SGI248967[] = {{&S97547[44], 10}, {NULL}};
const Segment SGI248968[] = {{&S97547[44], 22}, {NULL}};
const Segment SGI248969[] = {{&S97547[30], 6}, {NULL}};
const Segment SGI248970[] = {{&S97547[115], 3}, {NULL}};
const Segment SGI248971[] = {{&S97547[80], 6}, {NULL}};
const Segment SGI248972[] = {{&S104081[44], 10}, {NULL}};
const Segment SGI248973[] = {{&S104081[44], 22}, {NULL}};
const Segment SGI248974[] = {{&S104081[30], 6}, {NULL}};
const Segment SGI248975[] = {{&S104081[115], 3}, {NULL}};
const Segment SGI248976[] = {{&S104081[80], 6}, {NULL}};
const Segment SGI248977[] = {{&S104081[132], 10}, {NULL}};
const Segment SGI248978[] = {{&S97561[44], 10}, {NULL}};
const Segment SGI248979[] = {{&S97561[44], 21}, {NULL}};
const Segment SGI248980[] = {{&S97561[30], 6}, {NULL}};
const Segment SGI248981[] = {{&S97561[79], 6}, {NULL}};
const Segment SGI248982[] = {{&S97561[106], 13}, {NULL}};
const Segment SGI248983[] = {{&S104093[44], 10}, {NULL}};
const Segment SGI248984[] = {{&S104093[44], 22}, {NULL}};
const Segment SGI248985[] = {{&S104093[30], 6}, {NULL}};
const Segment SGI248986[] = {{&S104093[131], 3}, {NULL}};
const Segment SGI248987[] = {{&S104093[96], 6}, {NULL}};
const Segment SGI248988[] = {{&S104093[159], 3}, {NULL}};
const Segment SGI248989[] = {{&S104093[68], 8}, {NULL}};
const Segment SGI248990[] = {{&S97583[34], 10}, {NULL}};
const Segment SGI248991[] = {{&S97583[27], 6}, {NULL}};
const Segment SGI248992[] = {{&S97583[96], 9}, {NULL}};
const Segment SGI248993[] = {{&S97583[69], 6}, {NULL}};
const Segment SGI248994[] = {{&S97583[130], 8}, {NULL}};
const Segment SGI248995[] = {{&S104113[44], 10}, {NULL}};
const Segment SGI248996[] = {{&S104113[44], 22}, {NULL}};
const Segment SGI248997[] = {{&S104113[30], 6}, {NULL}};
const Segment SGI248998[] = {{&S104113[80], 6}, {NULL}};
const Segment SGI248999[] = {{&S104113[107], 13}, {NULL}};
const Segment SGI249000[] = {{&S97595[34], 10}, {NULL}};
const Segment SGI249001[] = {{&S97595[27], 6}, {NULL}};
const Segment SGI249002[] = {{&S97595[96], 9}, {NULL}};
const Segment SGI249003[] = {{&S97595[69], 6}, {NULL}};
const Segment SGI249004[] = {{&S104118[34], 10}, {NULL}};
const Segment SGI249005[] = {{&S104118[27], 6}, {NULL}};
const Segment SGI249006[] = {{&S104118[97], 9}, {NULL}};
const Segment SGI249007[] = {{&S104118[70], 6}, {NULL}};
const Segment SGI249008[] = {{&S97607[34], 10}, {NULL}};
const Segment SGI249009[] = {{&S97607[27], 6}, {NULL}};
const Segment SGI249010[] = {{&S97607[97], 9}, {NULL}};
const Segment SGI249011[] = {{&S97607[70], 6}, {NULL}};
const Segment SGI249012[] = {{&S97613[34], 14}, {NULL}};
const Segment SGI249013[] = {{&S97613[27], 6}, {NULL}};
const Segment SGI249014[] = {{&S97613[101], 9}, {NULL}};
const Segment SGI249015[] = {{&S97613[74], 6}, {NULL}};
const Segment SGI249016[] = {{&S8960[13], 6}, {NULL}};
const Segment SGI249017[] = {{&S8960[38], 3}, {NULL}};
const Segment SGI249018[] = {{&S8960[76], 8}, {NULL}};
const Segment SGI249019[] = {{&S8960[112], 5}, {NULL}};
const Segment SGI249020[] = {{&S8961[13], 6}, {NULL}};
const Segment SGI249021[] = {{&S8961[38], 5}, {NULL}};
const Segment SGI249022[] = {{&S8961[78], 8}, {NULL}};
const Segment SGI249023[] = {{&S8961[114], 3}, {NULL}};
const Segment SGI249024[] = {{&S8962[13], 6}, {NULL}};
const Segment SGI249025[] = {{&S8962[38], 5}, {NULL}};
const Segment SGI249026[] = {{&S8962[78], 8}, {NULL}};
const Segment SGI249027[] = {{&S8963[13], 6}, {NULL}};
const Segment SGI249028[] = {{&S8963[38], 3}, {NULL}};
const Segment SGI249029[] = {{&S8963[76], 8}, {NULL}};
const Segment SGI249030[] = {{&S8963[112], 5}, {NULL}};
const Segment SGI249031[] = {{&S8964[13], 6}, {NULL}};
const Segment SGI249032[] = {{&S8964[38], 5}, {NULL}};
const Segment SGI249033[] = {{&S8964[78], 8}, {NULL}};
const Segment SGI249034[] = {{&S8964[114], 3}, {NULL}};
const Segment SGI249035[] = {{&S8965[13], 6}, {NULL}};
const Segment SGI249036[] = {{&S8965[38], 5}, {NULL}};
const Segment SGI249037[] = {{&S8965[78], 8}, {NULL}};
const Segment SGI249038[] = {{&S8965[114], 3}, {NULL}};
const Segment SGI249039[] = {{&S8966[13], 6}, {NULL}};
const Segment SGI249040[] = {{&S8966[38], 5}, {NULL}};
const Segment SGI249041[] = {{&S8966[78], 8}, {NULL}};
const Segment SGI249042[] = {{&S8966[114], 3}, {NULL}};
const Segment SGI249043[] = {{&S8967[13], 6}, {NULL}};
const Segment SGI249044[] = {{&S8967[38], 5}, {NULL}};
const Segment SGI249045[] = {{&S8967[78], 8}, {NULL}};
const Segment SGI249046[] = {{&S8968[13], 6}, {NULL}};
const Segment SGI249047[] = {{&S8968[38], 5}, {NULL}};
const Segment SGI249048[] = {{&S8968[78], 8}, {NULL}};
const Segment SGI249049[] = {{&S8968[114], 3}, {NULL}};
const Segment SGI249050[] = {{&S8969[13], 6}, {NULL}};
const Segment SGI249051[] = {{&S8969[38], 5}, {NULL}};
const Segment SGI249052[] = {{&S8969[78], 8}, {NULL}};
const Segment SGI249053[] = {{&S8970[13], 6}, {NULL}};
const Segment SGI249054[] = {{&S8970[38], 5}, {NULL}};
const Segment SGI249055[] = {{&S8970[78], 8}, {NULL}};
const Segment SGI249056[] = {{&S8970[114], 3}, {NULL}};
const Segment SGI249057[] = {{&S8971[13], 6}, {NULL}};
const Segment SGI249058[] = {{&S8971[38], 3}, {NULL}};
const Segment SGI249059[] = {{&S8971[76], 8}, {NULL}};
const Segment SGI249060[] = {{&S8972[13], 6}, {NULL}};
const Segment SGI249061[] = {{&S8972[38], 5}, {NULL}};
const Segment SGI249062[] = {{&S8972[78], 8}, {NULL}};
const Segment SGI249063[] = {{&S8972[114], 3}, {NULL}};
const Segment SGI249064[] = {{&S8973[13], 6}, {NULL}};
const Segment SGI249065[] = {{&S8973[38], 5}, {NULL}};
const Segment SGI249066[] = {{&S8973[78], 8}, {NULL}};
const Segment SGI249067[] = {{&S8974[13], 6}, {NULL}};
const Segment SGI249068[] = {{&S8974[38], 5}, {NULL}};
const Segment SGI249069[] = {{&S8974[78], 8}, {NULL}};
const Segment SGI249070[] = {{&S8974[114], 3}, {NULL}};
const Segment SGI249071[] = {{&S8975[13], 6}, {NULL}};
const Segment SGI249072[] = {{&S8975[38], 5}, {NULL}};
const Segment SGI249073[] = {{&S8975[78], 8}, {NULL}};
const Segment SGI249074[] = {{&S8976[12], 6}, {NULL}};
const Segment SGI249075[] = {{&S8976[66], 8}, {NULL}};
const Segment SGI249076[] = {{&S8976[98], 3}, {NULL}};
const Segment SGI249077[] = {{&S8977[13], 6}, {NULL}};
const Segment SGI249078[] = {{&S8977[38], 3}, {NULL}};
const Segment SGI249079[] = {{&S8977[76], 8}, {NULL}};
const Segment SGI249080[] = {{&S8978[13], 6}, {NULL}};
const Segment SGI249081[] = {{&S8978[38], 5}, {NULL}};
const Segment SGI249082[] = {{&S8978[78], 8}, {NULL}};
const Segment SGI249083[] = {{&S8979[13], 6}, {NULL}};
const Segment SGI249084[] = {{&S8979[38], 5}, {NULL}};
const Segment SGI249085[] = {{&S8979[78], 8}, {NULL}};
const Segment SGI249086[] = {{&S8980[13], 6}, {NULL}};
const Segment SGI249087[] = {{&S8980[48], 3}, {NULL}};
const Segment SGI249088[] = {{&S8980[86], 8}, {NULL}};
const Segment SGI249089[] = {{&S8980[122], 5}, {NULL}};
const Segment SGI249090[] = {{&S8981[13], 6}, {NULL}};
const Segment SGI249091[] = {{&S8981[38], 5}, {NULL}};
const Segment SGI249092[] = {{&S8981[78], 8}, {NULL}};
const Segment SGI249093[] = {{&S8981[114], 3}, {NULL}};
const Segment SGI249094[] = {{&S8982[13], 6}, {NULL}};
const Segment SGI249095[] = {{&S8982[38], 3}, {NULL}};
const Segment SGI249096[] = {{&S8982[76], 8}, {NULL}};
const Segment SGI249097[] = {{&S8982[112], 5}, {NULL}};
const Segment SGI249098[] = {{&S8983[13], 6}, {NULL}};
const Segment SGI249099[] = {{&S8983[38], 5}, {NULL}};
const Segment SGI249100[] = {{&S8983[78], 8}, {NULL}};
const Segment SGI249101[] = {{&S8983[114], 3}, {NULL}};
const Segment SGI249102[] = {{&S8984[13], 6}, {NULL}};
const Segment SGI249103[] = {{&S8984[38], 5}, {NULL}};
const Segment SGI249104[] = {{&S8984[78], 8}, {NULL}};
const Segment SGI249105[] = {{&S8984[114], 5}, {NULL}};
const Segment SGI249106[] = {{&S8985[13], 6}, {NULL}};
const Segment SGI249107[] = {{&S8985[38], 5}, {NULL}};
const Segment SGI249108[] = {{&S8985[78], 8}, {NULL}};
const Segment SGI249109[] = {{&S8985[114], 3}, {NULL}};
const Segment SGI249110[] = {{&S8986[13], 6}, {NULL}};
const Segment SGI249111[] = {{&S8986[38], 5}, {NULL}};
const Segment SGI249112[] = {{&S8986[78], 8}, {NULL}};
const Segment SGI249113[] = {{&S8986[114], 3}, {NULL}};
const Segment SGI249114[] = {{&S8987[15], 6}, {NULL}};
const Segment SGI249115[] = {{&S8987[33], 5}, {NULL}};
const Segment SGI249116[] = {{&S8987[0], 7}, {NULL}};
const Segment SGI249117[] = {{&S8988[15], 6}, {NULL}};
const Segment SGI249118[] = {{&S8988[33], 5}, {NULL}};
const Segment SGI249119[] = {{&S8988[0], 7}, {NULL}};
const Segment SGI249120[] = {{&S8989[27], 6}, {NULL}};
const Segment SGI249121[] = {{&S8989[52], 5}, {NULL}};
const Segment SGI249122[] = {{&S8989[92], 6}, {NULL}};
const Segment SGI249123[] = {{&S8990[13], 6}, {NULL}};
const Segment SGI249124[] = {{&S8990[38], 5}, {NULL}};
const Segment SGI249125[] = {{&S8990[78], 8}, {NULL}};
const Segment SGI249126[] = {{&S8990[114], 3}, {NULL}};
const Segment SGI249127[] = {{&S8991[13], 6}, {NULL}};
const Segment SGI249128[] = {{&S8991[38], 5}, {NULL}};
const Segment SGI249129[] = {{&S8992[13], 6}, {NULL}};
const Segment SGI249130[] = {{&S8992[38], 3}, {NULL}};
const Segment SGI249131[] = {{&S8993[13], 6}, {NULL}};
const Segment SGI249132[] = {{&S8993[38], 5}, {NULL}};
const Segment SGI249133[] = {{&S8994[13], 6}, {NULL}};
const Segment SGI249134[] = {{&S8994[38], 5}, {NULL}};
const Segment SGI249135[] = {{&S8994[78], 8}, {NULL}};
const Segment SGI249136[] = {{&S8994[114], 5}, {NULL}};
const Segment SGI249137[] = {{&S8995[13], 6}, {NULL}};
const Segment SGI249138[] = {{&S8995[38], 5}, {NULL}};
const Segment SGI249139[] = {{&S8995[78], 8}, {NULL}};
const Segment SGI249140[] = {{&S8995[114], 5}, {NULL}};
const Segment SGI249141[] = {{&S8996[13], 6}, {NULL}};
const Segment SGI249142[] = {{&S8996[38], 5}, {NULL}};
const Segment SGI249143[] = {{&S8996[78], 8}, {NULL}};
const Segment SGI249144[] = {{&S8996[114], 3}, {NULL}};
const Segment SGI249145[] = {{&S8997[13], 6}, {NULL}};
const Segment SGI249146[] = {{&S8997[38], 5}, {NULL}};
const Segment SGI249147[] = {{&S8998[13], 6}, {NULL}};
const Segment SGI249148[] = {{&S8998[38], 5}, {NULL}};
const Segment SGI249149[] = {{&S8999[13], 6}, {NULL}};
const Segment SGI249150[] = {{&S8999[38], 5}, {NULL}};
const Segment SGI249151[] = {{&S9000[27], 6}, {NULL}};
const Segment SGI249152[] = {{&S9000[52], 5}, {NULL}};
const Segment SGI249153[] = {{&S9000[92], 6}, {NULL}};
const Segment SGI249154[] = {{&S9003[13], 6}, {NULL}};
const Segment SGI249155[] = {{&S9003[142], 3}, {NULL}};
const Segment SGI249156[] = {{&S9003[38], 5}, {NULL}};
const Segment SGI249157[] = {{&S9003[78], 8}, {NULL}};
const Segment SGI249158[] = {{&S9003[114], 3}, {NULL}};
const Segment SGI249159[] = {{&S9004[13], 6}, {&S9004[118], 4}, {NULL}};
const Segment SGI249160[] = {{&S9004[38], 5}, {NULL}};
const Segment SGI249161[] = {{&S9004[78], 8}, {NULL}};
const Segment SGI249162[] = {{&S9004[114], 3}, {NULL}};
const Segment SGI249163[] = {{&S9005[13], 6}, {&S9005[13], 6}, {NULL}};
const Segment SGI249164[] = {{&S9005[75], 6}, {NULL}};
const Segment SGI249165[] = {{&S9005[109], 4}, {NULL}};
const Segment SGI249166[] = {{&S9006[13], 6}, {&S9006[118], 4}, {NULL}};
const Segment SGI249167[] = {{&S9006[38], 5}, {NULL}};
const Segment SGI249168[] = {{&S9006[78], 8}, {NULL}};
const Segment SGI249169[] = {{&S9006[114], 3}, {NULL}};
const Segment SGI249170[] = {{&S9007[13], 6}, {NULL}};
const Segment SGI249171[] = {{&S9007[38], 5}, {NULL}};
const Segment SGI249172[] = {{&S9007[78], 5}, {NULL}};
const Segment SGI249173[] = {{&S9007[111], 3}, {NULL}};
const Segment SGI249174[] = {{&S9008[13], 6}, {NULL}};
const Segment SGI249175[] = {{&S9008[38], 3}, {NULL}};
const Segment SGI249176[] = {{&S9008[76], 5}, {NULL}};
const Segment SGI249177[] = {{&S9008[109], 5}, {NULL}};
const Segment SGI249178[] = {{&S9009[13], 6}, {&S9009[128], 4}, {NULL}};
const Segment SGI249179[] = {{&S9009[75], 6}, {NULL}};
const Segment SGI249180[] = {{&S9009[109], 4}, {NULL}};
const Segment SGI249181[] = {{&S9010[13], 6}, {NULL}};
const Segment SGI249182[] = {{&S9010[38], 5}, {NULL}};
const Segment SGI249183[] = {{&S9010[78], 8}, {NULL}};
const Segment SGI249184[] = {{&S9010[114], 5}, {NULL}};
const Segment SGI249185[] = {{&S9011[13], 6}, {NULL}};
const Segment SGI249186[] = {{&S9011[38], 5}, {NULL}};
const Segment SGI249187[] = {{&S9011[78], 5}, {NULL}};
const Segment SGI249188[] = {{&S9011[111], 3}, {NULL}};
const Segment SGI249189[] = {{&S9012[13], 6}, {NULL}};
const Segment SGI249190[] = {{&S9012[38], 3}, {NULL}};
const Segment SGI249191[] = {{&S9012[76], 5}, {NULL}};
const Segment SGI249192[] = {{&S97850[34], 4}, {NULL}};
const Segment SGI249193[] = {{&S97850[27], 6}, {NULL}};
const Segment SGI249194[] = {{&S97850[90], 9}, {NULL}};
const Segment SGI249195[] = {{&S97850[63], 6}, {NULL}};
const Segment SGI249196[] = {{&S97853[34], 10}, {NULL}};
const Segment SGI249197[] = {{&S97853[27], 6}, {NULL}};
const Segment SGI249198[] = {{&S97853[97], 9}, {NULL}};
const Segment SGI249199[] = {{&S97853[70], 6}, {NULL}};
const Segment SGI249200[] = {{&S104406[34], 7}, {NULL}};
const Segment SGI249201[] = {{&S104406[27], 6}, {NULL}};
const Segment SGI249202[] = {{&S104406[100], 9}, {NULL}};
const Segment SGI249203[] = {{&S104406[73], 6}, {NULL}};
const Segment SGI249204[] = {{&S104407[44], 10}, {NULL}};
const Segment SGI249205[] = {{&S104407[44], 22}, {NULL}};
const Segment SGI249206[] = {{&S104407[30], 6}, {NULL}};
const Segment SGI249207[] = {{&S104407[113], 3}, {NULL}};
const Segment SGI249208[] = {{&S104407[80], 5}, {NULL}};
const Segment SGI249209[] = {{&S104407[117], 14}, {NULL}};
const Segment SGI249210[] = {{&S97888[44], 17}, {NULL}};
const Segment SGI249211[] = {{&S97888[44], 29}, {NULL}};
const Segment SGI249212[] = {{&S97888[30], 6}, {NULL}};
const Segment SGI249213[] = {{&S97888[140], 3}, {NULL}};
const Segment SGI249214[] = {{&S97888[105], 6}, {NULL}};
const Segment SGI249215[] = {{&S97888[75], 8}, {NULL}};
const Segment SGI249216[] = {{&S104464[44], 10}, {NULL}};
const Segment SGI249217[] = {{&S104464[44], 22}, {NULL}};
const Segment SGI249218[] = {{&S104464[30], 6}, {NULL}};
const Segment SGI249219[] = {{&S104464[131], 3}, {NULL}};
const Segment SGI249220[] = {{&S104464[96], 6}, {NULL}};
const Segment SGI249221[] = {{&S104464[159], 3}, {NULL}};
const Segment SGI249222[] = {{&S104464[68], 8}, {NULL}};
const Segment SGI249223[] = {{&S104465[44], 10}, {NULL}};
const Segment SGI249224[] = {{&S104465[44], 22}, {NULL}};
const Segment SGI249225[] = {{&S104465[30], 6}, {NULL}};
const Segment SGI249226[] = {{&S104465[115], 3}, {NULL}};
const Segment SGI249227[] = {{&S104465[80], 6}, {NULL}};
const Segment SGI249228[] = {{&S104465[143], 3}, {NULL}};
const Segment SGI249229[] = {{&S104466[44], 10}, {NULL}};
const Segment SGI249230[] = {{&S104466[44], 22}, {NULL}};
const Segment SGI249231[] = {{&S104466[30], 6}, {NULL}};
const Segment SGI249232[] = {{&S104466[115], 3}, {NULL}};
const Segment SGI249233[] = {{&S104466[80], 6}, {NULL}};
const Segment SGI249234[] = {{&S104466[143], 3}, {NULL}};
const Segment SGI249235[] = {{&S104467[44], 10}, {NULL}};
const Segment SGI249236[] = {{&S104467[44], 22}, {NULL}};
const Segment SGI249237[] = {{&S104467[30], 6}, {NULL}};
const Segment SGI249238[] = {{&S104467[131], 3}, {NULL}};
const Segment SGI249239[] = {{&S104467[96], 6}, {NULL}};
const Segment SGI249240[] = {{&S104467[159], 3}, {NULL}};
const Segment SGI249241[] = {{&S104467[68], 8}, {NULL}};
const Segment SGI249242[] = {{&S97927[34], 7}, {NULL}};
const Segment SGI249243[] = {{&S97927[27], 6}, {NULL}};
const Segment SGI249244[] = {{&S97927[94], 9}, {NULL}};
const Segment SGI249245[] = {{&S97927[67], 6}, {NULL}};
const Segment SGI249246[] = {{&S104494[44], 10}, {NULL}};
const Segment SGI249247[] = {{&S104494[44], 22}, {NULL}};
const Segment SGI249248[] = {{&S104494[30], 6}, {NULL}};
const Segment SGI249249[] = {{&S104494[113], 3}, {NULL}};
const Segment SGI249250[] = {{&S104494[80], 5}, {NULL}};
const Segment SGI249251[] = {{&S104494[117], 14}, {NULL}};
const Segment SGI249252[] = {{&S104506[44], 10}, {NULL}};
const Segment SGI249253[] = {{&S104506[44], 22}, {NULL}};
const Segment SGI249254[] = {{&S104506[30], 6}, {NULL}};
const Segment SGI249255[] = {{&S104506[113], 3}, {NULL}};
const Segment SGI249256[] = {{&S104506[80], 5}, {NULL}};
const Segment SGI249257[] = {{&S104506[117], 14}, {NULL}};
const Segment SGI249258[] = {{&S104508[44], 10}, {NULL}};
const Segment SGI249259[] = {{&S104508[44], 22}, {NULL}};
const Segment SGI249260[] = {{&S104508[30], 6}, {NULL}};
const Segment SGI249261[] = {{&S104508[111], 2}, {NULL}};
const Segment SGI249262[] = {{&S104508[80], 5}, {NULL}};
const Segment SGI249263[] = {{&S97949[34], 10}, {NULL}};
const Segment SGI249264[] = {{&S97949[27], 6}, {NULL}};
const Segment SGI249265[] = {{&S97949[97], 9}, {NULL}};
const Segment SGI249266[] = {{&S97949[70], 6}, {NULL}};
const Segment SGI249267[] = {{&S97951[44], 20}, {NULL}};
const Segment SGI249268[] = {{&S97951[44], 32}, {NULL}};
const Segment SGI249269[] = {{&S97951[30], 6}, {NULL}};
const Segment SGI249270[] = {{&S97951[125], 3}, {NULL}};
const Segment SGI249271[] = {{&S97951[90], 6}, {NULL}};
const Segment SGI249272[] = {{&S104520[59], 11}, {NULL}};
const Segment SGI249273[] = {{&S104520[59], 23}, {NULL}};
const Segment SGI249274[] = {{&S104520[45], 6}, {NULL}};
const Segment SGI249275[] = {{&S104520[131], 3}, {NULL}};
const Segment SGI249276[] = {{&S104520[96], 6}, {NULL}};
const Segment SGI249277[] = {{&S104520[0], 14}, {NULL}};
const Segment SGI249278[] = {{&S104523[34], 7}, {NULL}};
const Segment SGI249279[] = {{&S104523[27], 6}, {NULL}};
const Segment SGI249280[] = {{&S104523[102], 9}, {NULL}};
const Segment SGI249281[] = {{&S104523[75], 6}, {NULL}};
const Segment SGI249282[] = {{&S104524[34], 14}, {NULL}};
const Segment SGI249283[] = {{&S104524[27], 6}, {NULL}};
const Segment SGI249284[] = {{&S104524[106], 9}, {NULL}};
const Segment SGI249285[] = {{&S104524[79], 6}, {NULL}};
const Segment SGI249286[] = {{&S97977[44], 10}, {NULL}};
const Segment SGI249287[] = {{&S97977[44], 21}, {NULL}};
const Segment SGI249288[] = {{&S97977[30], 6}, {NULL}};
const Segment SGI249289[] = {{&S97977[112], 3}, {NULL}};
const Segment SGI249290[] = {{&S97977[79], 5}, {NULL}};
const Segment SGI249291[] = {{&S97986[44], 7}, {NULL}};
const Segment SGI249292[] = {{&S97986[44], 19}, {NULL}};
const Segment SGI249293[] = {{&S97986[30], 6}, {NULL}};
const Segment SGI249294[] = {{&S97986[112], 3}, {NULL}};
const Segment SGI249295[] = {{&S97986[77], 6}, {NULL}};
const Segment SGI249296[] = {{&S104554[34], 7}, {NULL}};
const Segment SGI249297[] = {{&S104554[27], 6}, {NULL}};
const Segment SGI249298[] = {{&S104554[97], 9}, {NULL}};
const Segment SGI249299[] = {{&S104554[70], 6}, {NULL}};
const Segment SGI249300[] = {{&S97998[44], 10}, {NULL}};
const Segment SGI249301[] = {{&S97998[44], 21}, {NULL}};
const Segment SGI249302[] = {{&S97998[30], 6}, {NULL}};
const Segment SGI249303[] = {{&S97998[79], 6}, {NULL}};
const Segment SGI249304[] = {{&S97998[106], 13}, {NULL}};
const Segment SGI249305[] = {{&S98000[44], 14}, {NULL}};
const Segment SGI249306[] = {{&S98000[44], 25}, {NULL}};
const Segment SGI249307[] = {{&S98000[30], 6}, {NULL}};
const Segment SGI249308[] = {{&S98000[83], 6}, {NULL}};
const Segment SGI249309[] = {{&S98000[110], 13}, {NULL}};
const Segment SGI249310[] = {{&S104559[34], 5}, {NULL}};
const Segment SGI249311[] = {{&S104559[27], 6}, {NULL}};
const Segment SGI249312[] = {{&S104559[92], 9}, {NULL}};
const Segment SGI249313[] = {{&S104559[65], 6}, {NULL}};
const Segment SGI249314[] = {{&S104559[133], 8}, {NULL}};
const Segment SGI249315[] = {{&S98036[34], 15}, {NULL}};
const Segment SGI249316[] = {{&S98036[27], 6}, {NULL}};
const Segment SGI249317[] = {{&S98036[101], 9}, {NULL}};
const Segment SGI249318[] = {{&S98036[74], 6}, {NULL}};
const Segment SGI249319[] = {{&S105710[44], 10}, {NULL}};
const Segment SGI249320[] = {{&S105710[44], 22}, {NULL}};
const Segment SGI249321[] = {{&S105710[30], 6}, {NULL}};
const Segment SGI249322[] = {{&S105710[133], 3}, {NULL}};
const Segment SGI249323[] = {{&S105710[98], 6}, {NULL}};
const Segment SGI249324[] = {{&S105710[68], 8}, {NULL}};
const Segment SGI249325[] = {{&S105740[44], 10}, {NULL}};
const Segment SGI249326[] = {{&S105740[44], 22}, {NULL}};
const Segment SGI249327[] = {{&S105740[30], 6}, {NULL}};
const Segment SGI249328[] = {{&S105740[152], 3}, {NULL}};
const Segment SGI249329[] = {{&S105740[117], 6}, {NULL}};
const Segment SGI249330[] = {{&S105740[68], 8}, {NULL}};
const Segment SGI249331[] = {{&S105743[45], 10}, {NULL}};
const Segment SGI249332[] = {{&S105743[30], 6}, {NULL}};
const Segment SGI249333[] = {{&S105743[115], 3}, {NULL}};
const Segment SGI249334[] = {{&S105743[81], 5}, {NULL}};
const Segment SGI249335[] = {{&S105743[149], 3}, {NULL}};
const Segment SGI249336[] = {{&S105764[45], 10}, {NULL}};
const Segment SGI249337[] = {{&S105764[30], 6}, {NULL}};
const Segment SGI249338[] = {{&S105764[115], 3}, {NULL}};
const Segment SGI249339[] = {{&S105764[81], 5}, {NULL}};
const Segment SGI249340[] = {{&S105764[149], 3}, {NULL}};
const Segment SGI249341[] = {{&S105766[59], 10}, {NULL}};
const Segment SGI249342[] = {{&S105766[59], 22}, {NULL}};
const Segment SGI249343[] = {{&S105766[45], 6}, {NULL}};
const Segment SGI249344[] = {{&S105766[148], 3}, {NULL}};
const Segment SGI249345[] = {{&S105766[113], 6}, {NULL}};
const Segment SGI249346[] = {{&S105766[83], 8}, {NULL}};
const Segment SGI249347[] = {{&S105769[44], 10}, {NULL}};
const Segment SGI249348[] = {{&S105769[44], 22}, {NULL}};
const Segment SGI249349[] = {{&S105769[30], 6}, {NULL}};
const Segment SGI249350[] = {{&S105769[133], 3}, {NULL}};
const Segment SGI249351[] = {{&S105769[98], 6}, {NULL}};
const Segment SGI249352[] = {{&S105769[161], 3}, {NULL}};
const Segment SGI249353[] = {{&S105769[68], 8}, {NULL}};
const Segment SGI249354[] = {{&S105773[45], 10}, {NULL}};
const Segment SGI249355[] = {{&S105773[30], 6}, {NULL}};
const Segment SGI249356[] = {{&S105773[115], 3}, {NULL}};
const Segment SGI249357[] = {{&S105773[81], 5}, {NULL}};
const Segment SGI249358[] = {{&S105773[149], 3}, {NULL}};
const Segment SGI249359[] = {{&S105791[44], 10}, {NULL}};
const Segment SGI249360[] = {{&S105791[44], 22}, {NULL}};
const Segment SGI249361[] = {{&S105791[30], 6}, {NULL}};
const Segment SGI249362[] = {{&S105791[133], 3}, {NULL}};
const Segment SGI249363[] = {{&S105791[98], 6}, {NULL}};
const Segment SGI249364[] = {{&S105791[68], 8}, {NULL}};
const Segment SGI249365[] = {{&S105791[151], 11}, {NULL}};
const Segment SGI249366[] = {{&S105792[44], 10}, {NULL}};
const Segment SGI249367[] = {{&S105792[44], 22}, {NULL}};
const Segment SGI249368[] = {{&S105792[30], 6}, {NULL}};
const Segment SGI249369[] = {{&S105792[115], 3}, {NULL}};
const Segment SGI249370[] = {{&S105792[80], 6}, {NULL}};
const Segment SGI249371[] = {{&S105792[143], 3}, {NULL}};
const Segment SGI249372[] = {{&S105806[44], 10}, {NULL}};
const Segment SGI249373[] = {{&S105806[44], 22}, {NULL}};
const Segment SGI249374[] = {{&S105806[30], 6}, {NULL}};
const Segment SGI249375[] = {{&S105806[133], 3}, {NULL}};
const Segment SGI249376[] = {{&S105806[98], 6}, {NULL}};
const Segment SGI249377[] = {{&S105806[68], 8}, {NULL}};
const Segment SGI249378[] = {{&S105806[151], 11}, {NULL}};
const Segment SGI249379[] = {{&S105808[44], 10}, {NULL}};
const Segment SGI249380[] = {{&S105808[44], 22}, {NULL}};
const Segment SGI249381[] = {{&S105808[30], 6}, {NULL}};
const Segment SGI249382[] = {{&S105808[133], 3}, {NULL}};
const Segment SGI249383[] = {{&S105808[98], 6}, {NULL}};
const Segment SGI249384[] = {{&S105808[161], 3}, {NULL}};
const Segment SGI249385[] = {{&S105808[68], 8}, {NULL}};
const Segment SGI249386[] = {{&S105809[44], 10}, {NULL}};
const Segment SGI249387[] = {{&S105809[44], 22}, {NULL}};
const Segment SGI249388[] = {{&S105809[30], 6}, {NULL}};
const Segment SGI249389[] = {{&S105809[133], 3}, {NULL}};
const Segment SGI249390[] = {{&S105809[98], 6}, {NULL}};
const Segment SGI249391[] = {{&S105809[68], 8}, {NULL}};
const Segment SGI249392[] = {{&S105809[150], 10}, {NULL}};
const Segment SGI249393[] = {{&S98204[34], 10}, {NULL}};
const Segment SGI249394[] = {{&S98204[27], 6}, {NULL}};
const Segment SGI249395[] = {{&S98204[96], 9}, {NULL}};
const Segment SGI249396[] = {{&S98204[69], 6}, {NULL}};
const Segment SGI249397[] = {{&S98226[44], 5}, {NULL}};
const Segment SGI249398[] = {{&S98226[44], 17}, {NULL}};
const Segment SGI249399[] = {{&S98226[30], 6}, {NULL}};
const Segment SGI249400[] = {{&S98226[139], 3}, {NULL}};
const Segment SGI249401[] = {{&S98226[104], 6}, {NULL}};
const Segment SGI249402[] = {{&S105848[38], 10}, {NULL}};
const Segment SGI249403[] = {{&S105848[31], 6}, {NULL}};
const Segment SGI249404[] = {{&S105856[34], 10}, {NULL}};
const Segment SGI249405[] = {{&S105856[27], 6}, {NULL}};
const Segment SGI249406[] = {{&S105856[97], 9}, {NULL}};
const Segment SGI249407[] = {{&S105856[70], 6}, {NULL}};
const Segment SGI249408[] = {{&S105856[132], 8}, {NULL}};
const Segment SGI249409[] = {{&S105879[34], 10}, {NULL}};
const Segment SGI249410[] = {{&S105879[27], 6}, {NULL}};
const Segment SGI249411[] = {{&S105879[97], 9}, {NULL}};
const Segment SGI249412[] = {{&S105879[70], 6}, {NULL}};
const Segment SGI249413[] = {{&S104864[34], 10}, {NULL}};
const Segment SGI249414[] = {{&S104864[27], 6}, {NULL}};
const Segment SGI249415[] = {{&S104864[97], 9}, {NULL}};
const Segment SGI249416[] = {{&S104864[70], 6}, {NULL}};
const Segment SGI249417[] = {{&S105889[44], 10}, {NULL}};
const Segment SGI249418[] = {{&S105889[44], 22}, {NULL}};
const Segment SGI249419[] = {{&S105889[30], 6}, {NULL}};
const Segment SGI249420[] = {{&S105889[115], 3}, {NULL}};
const Segment SGI249421[] = {{&S105889[80], 6}, {NULL}};
const Segment SGI249422[] = {{&S105894[34], 10}, {NULL}};
const Segment SGI249423[] = {{&S105894[27], 6}, {NULL}};
const Segment SGI249424[] = {{&S105894[97], 9}, {NULL}};
const Segment SGI249425[] = {{&S105894[70], 6}, {NULL}};
const Segment SGI249426[] = {{&S98298[34], 10}, {NULL}};
const Segment SGI249427[] = {{&S98298[27], 6}, {NULL}};
const Segment SGI249428[] = {{&S98298[97], 9}, {NULL}};
const Segment SGI249429[] = {{&S98298[70], 6}, {NULL}};
const Segment SGI249430[] = {{&S98298[138], 8}, {NULL}};
const Segment SGI249431[] = {{&S104889[44], 10}, {NULL}};
const Segment SGI249432[] = {{&S104889[44], 22}, {NULL}};
const Segment SGI249433[] = {{&S104889[30], 6}, {NULL}};
const Segment SGI249434[] = {{&S104889[131], 3}, {NULL}};
const Segment SGI249435[] = {{&S104889[96], 6}, {NULL}};
const Segment SGI249436[] = {{&S104889[159], 3}, {NULL}};
const Segment SGI249437[] = {{&S104889[68], 8}, {NULL}};
const Segment SGI249438[] = {{&S98322[44], 20}, {NULL}};
const Segment SGI249439[] = {{&S98322[44], 31}, {NULL}};
const Segment SGI249440[] = {{&S98322[30], 6}, {NULL}};
const Segment SGI249441[] = {{&S98322[142], 3}, {NULL}};
const Segment SGI249442[] = {{&S98322[107], 6}, {NULL}};
const Segment SGI249443[] = {{&S98322[77], 8}, {NULL}};
const Segment SGI249444[] = {{&S98331[44], 13}, {NULL}};
const Segment SGI249445[] = {{&S98331[44], 25}, {NULL}};
const Segment SGI249446[] = {{&S98331[30], 6}, {NULL}};
const Segment SGI249447[] = {{&S98331[118], 3}, {NULL}};
const Segment SGI249448[] = {{&S98331[83], 6}, {NULL}};
const Segment SGI249449[] = {{&S104925[44], 10}, {NULL}};
const Segment SGI249450[] = {{&S104925[44], 22}, {NULL}};
const Segment SGI249451[] = {{&S104925[30], 6}, {NULL}};
const Segment SGI249452[] = {{&S104925[134], 3}, {NULL}};
const Segment SGI249453[] = {{&S104925[99], 6}, {NULL}};
const Segment SGI249454[] = {{&S104925[68], 8}, {NULL}};
const Segment SGI249455[] = {{&S104925[151], 10}, {NULL}};
const Segment SGI249456[] = {{&S98362[44], 18}, {NULL}};
const Segment SGI249457[] = {{&S98362[44], 29}, {NULL}};
const Segment SGI249458[] = {{&S98362[30], 6}, {NULL}};
const Segment SGI249459[] = {{&S98362[122], 3}, {NULL}};
const Segment SGI249460[] = {{&S98362[87], 6}, {NULL}};
const Segment SGI249461[] = {{&S98362[147], 11}, {NULL}};
const Segment SGI249462[] = {{&S105971[34], 14}, {NULL}};
const Segment SGI249463[] = {{&S105971[27], 6}, {NULL}};
const Segment SGI249464[] = {{&S105971[101], 9}, {NULL}};
const Segment SGI249465[] = {{&S105971[74], 6}, {NULL}};
const Segment SGI249466[] = {{&S105980[34], 15}, {NULL}};
const Segment SGI249467[] = {{&S105980[27], 6}, {NULL}};
const Segment SGI249468[] = {{&S105980[102], 9}, {NULL}};
const Segment SGI249469[] = {{&S105980[75], 6}, {NULL}};
const Segment SGI249470[] = {{&S105980[143], 8}, {NULL}};
const Segment SGI249471[] = {{&S105984[34], 13}, {NULL}};
const Segment SGI249472[] = {{&S105984[27], 6}, {NULL}};
const Segment SGI249473[] = {{&S105984[100], 9}, {NULL}};
const Segment SGI249474[] = {{&S105984[73], 6}, {NULL}};
const Segment SGI249475[] = {{&S105984[141], 8}, {NULL}};
const Segment SGI249476[] = {{&S98396[34], 12}, {NULL}};
const Segment SGI249477[] = {{&S98396[27], 6}, {NULL}};
const Segment SGI249478[] = {{&S98396[98], 9}, {NULL}};
const Segment SGI249479[] = {{&S98396[71], 6}, {NULL}};
const Segment SGI249480[] = {{&S104979[44], 10}, {NULL}};
const Segment SGI249481[] = {{&S104979[44], 22}, {NULL}};
const Segment SGI249482[] = {{&S104979[30], 6}, {NULL}};
const Segment SGI249483[] = {{&S104979[115], 3}, {NULL}};
const Segment SGI249484[] = {{&S104979[80], 6}, {NULL}};
const Segment SGI249485[] = {{&S98398[34], 6}, {NULL}};
const Segment SGI249486[] = {{&S104979[143], 3}, {NULL}};
const Segment SGI249487[] = {{&S98398[27], 6}, {NULL}};
const Segment SGI249488[] = {{&S98398[93], 9}, {NULL}};
const Segment SGI249489[] = {{&S98398[66], 6}, {NULL}};
const Segment SGI249490[] = {{&S98402[34], 14}, {NULL}};
const Segment SGI249491[] = {{&S98402[27], 6}, {NULL}};
const Segment SGI249492[] = {{&S98402[101], 9}, {NULL}};
const Segment SGI249493[] = {{&S98402[74], 6}, {NULL}};
const Segment SGI249494[] = {{&S98404[34], 17}, {NULL}};
const Segment SGI249495[] = {{&S98404[27], 6}, {NULL}};
const Segment SGI249496[] = {{&S98404[104], 9}, {NULL}};
const Segment SGI249497[] = {{&S98404[77], 6}, {NULL}};
const Segment SGI249498[] = {{&S98411[44], 9}, {NULL}};
const Segment SGI249499[] = {{&S98411[44], 20}, {NULL}};
const Segment SGI249500[] = {{&S98411[30], 6}, {NULL}};
const Segment SGI249501[] = {{&S98411[113], 3}, {NULL}};
const Segment SGI249502[] = {{&S104993[59], 10}, {NULL}};
const Segment SGI249503[] = {{&S98411[78], 6}, {NULL}};
const Segment SGI249504[] = {{&S104993[59], 22}, {NULL}};
const Segment SGI249505[] = {{&S104993[45], 6}, {NULL}};
const Segment SGI249506[] = {{&S104993[146], 3}, {NULL}};
const Segment SGI249507[] = {{&S104993[111], 6}, {NULL}};
const Segment SGI249508[] = {{&S104993[83], 8}, {NULL}};
const Segment SGI249509[] = {{&S104993[11], 3}, {NULL}};
const Segment SGI249510[] = {{&S9728[13], 4}, {NULL}};
const Segment SGI249511[] = {{&S9728[92], 3}, {NULL}};
const Segment SGI249512[] = {{&S9728[26], 3}, {NULL}};
const Segment SGI249513[] = {{&S9728[57], 6}, {NULL}};
const Segment SGI249514[] = {{&S9728[103], 8}, {NULL}};
const Segment SGI249515[] = {{&S9728[119], 7}, {NULL}};
const Segment SGI249516[] = {{&S9729[13], 4}, {NULL}};
const Segment SGI249517[] = {{&S9729[92], 3}, {NULL}};
const Segment SGI249518[] = {{&S9729[26], 3}, {NULL}};
const Segment SGI249519[] = {{&S9729[57], 6}, {NULL}};
const Segment SGI249520[] = {{&S9729[103], 8}, {NULL}};
const Segment SGI249521[] = {{&S9729[119], 7}, {NULL}};
const Segment SGI249522[] = {{&S9730[13], 4}, {NULL}};
const Segment SGI249523[] = {{&S9730[92], 3}, {NULL}};
const Segment SGI249524[] = {{&S9730[26], 3}, {NULL}};
const Segment SGI249525[] = {{&S9730[57], 6}, {NULL}};
const Segment SGI249526[] = {{&S9730[103], 6}, {NULL}};
const Segment SGI249527[] = {{&S98433[44], 13}, {NULL}};
const Segment SGI249528[] = {{&S9730[117], 7}, {NULL}};
const Segment SGI249529[] = {{&S98433[44], 24}, {NULL}};
const Segment SGI249530[] = {{&S98433[30], 6}, {NULL}};
const Segment SGI249531[] = {{&S98433[117], 3}, {NULL}};
const Segment SGI249532[] = {{&S98433[82], 6}, {NULL}};
const Segment SGI249533[] = {{&S9731[13], 4}, {NULL}};
const Segment SGI249534[] = {{&S9731[92], 3}, {NULL}};
const Segment SGI249535[] = {{&S9731[26], 3}, {NULL}};
const Segment SGI249536[] = {{&S9731[57], 6}, {NULL}};
const Segment SGI249537[] = {{&S9731[103], 6}, {NULL}};
const Segment SGI249538[] = {{&S9731[117], 7}, {NULL}};
const Segment SGI249539[] = {{&S9732[13], 4}, {NULL}};
const Segment SGI249540[] = {{&S9732[92], 3}, {NULL}};
const Segment SGI249541[] = {{&S9732[26], 3}, {NULL}};
const Segment SGI249542[] = {{&S9732[57], 6}, {NULL}};
const Segment SGI249543[] = {{&S9732[103], 12}, {NULL}};
const Segment SGI249544[] = {{&S9732[116], 7}, {NULL}};
const Segment SGI249545[] = {{&S9733[13], 4}, {NULL}};
const Segment SGI249546[] = {{&S9733[91], 3}, {NULL}};
const Segment SGI249547[] = {{&S9733[56], 6}, {NULL}};
const Segment SGI249548[] = {{&S9733[102], 6}, {NULL}};
const Segment SGI249549[] = {{&S9733[116], 7}, {NULL}};
const Segment SGI249550[] = {{&S9734[13], 4}, {NULL}};
const Segment SGI249551[] = {{&S9734[92], 3}, {NULL}};
const Segment SGI249552[] = {{&S9734[26], 3}, {NULL}};
const Segment SGI249553[] = {{&S9734[57], 6}, {NULL}};
const Segment SGI249554[] = {{&S9734[103], 6}, {NULL}};
const Segment SGI249555[] = {{&S9734[117], 7}, {NULL}};
const Segment SGI249556[] = {{&S9735[13], 4}, {NULL}};
const Segment SGI249557[] = {{&S9735[92], 3}, {NULL}};
const Segment SGI249558[] = {{&S9735[26], 3}, {NULL}};
const Segment SGI249559[] = {{&S9735[57], 6}, {NULL}};
const Segment SGI249560[] = {{&S9735[103], 6}, {NULL}};
const Segment SGI249561[] = {{&S9735[117], 7}, {NULL}};
const Segment SGI249562[] = {{&S9736[13], 4}, {NULL}};
const Segment SGI249563[] = {{&S9736[94], 3}, {NULL}};
const Segment SGI249564[] = {{&S9736[26], 5}, {NULL}};
const Segment SGI249565[] = {{&S9736[59], 6}, {NULL}};
const Segment SGI249566[] = {{&S9736[105], 6}, {NULL}};
const Segment SGI249567[] = {{&S9736[119], 7}, {NULL}};
const Segment SGI249568[] = {{&S9737[13], 4}, {NULL}};
const Segment SGI249569[] = {{&S9737[94], 3}, {NULL}};
const Segment SGI249570[] = {{&S9737[26], 5}, {NULL}};
const Segment SGI249571[] = {{&S9737[59], 6}, {NULL}};
const Segment SGI249572[] = {{&S9737[105], 7}, {NULL}};
const Segment SGI249573[] = {{&S9737[120], 7}, {NULL}};
const Segment SGI249574[] = {{&S9738[13], 4}, {NULL}};
const Segment SGI249575[] = {{&S9738[92], 3}, {NULL}};
const Segment SGI249576[] = {{&S9738[26], 3}, {NULL}};
const Segment SGI249577[] = {{&S9738[57], 6}, {NULL}};
const Segment SGI249578[] = {{&S9738[103], 6}, {NULL}};
const Segment SGI249579[] = {{&S9738[117], 7}, {NULL}};
const Segment SGI249580[] = {{&S9739[13], 4}, {NULL}};
const Segment SGI249581[] = {{&S9739[94], 3}, {NULL}};
const Segment SGI249582[] = {{&S9739[26], 5}, {NULL}};
const Segment SGI249583[] = {{&S9739[59], 6}, {NULL}};
const Segment SGI249584[] = {{&S9739[105], 7}, {NULL}};
const Segment SGI249585[] = {{&S9739[120], 7}, {NULL}};
const Segment SGI249586[] = {{&S98448[34], 10}, {NULL}};
const Segment SGI249587[] = {{&S98448[27], 6}, {NULL}};
const Segment SGI249588[] = {{&S9740[13], 4}, {NULL}};
const Segment SGI249589[] = {{&S9740[94], 3}, {NULL}};
const Segment SGI249590[] = {{&S9740[26], 5}, {NULL}};
const Segment SGI249591[] = {{&S9740[59], 6}, {NULL}};
const Segment SGI249592[] = {{&S9740[105], 12}, {NULL}};
const Segment SGI249593[] = {{&S9740[118], 7}, {NULL}};
const Segment SGI249594[] = {{&S98448[96], 9}, {NULL}};
const Segment SGI249595[] = {{&S98448[69], 6}, {NULL}};
const Segment SGI249596[] = {{&S98448[137], 8}, {NULL}};
const Segment SGI249597[] = {{&S9741[13], 4}, {NULL}};
const Segment SGI249598[] = {{&S9741[94], 3}, {NULL}};
const Segment SGI249599[] = {{&S9741[26], 5}, {NULL}};
const Segment SGI249600[] = {{&S9741[59], 6}, {NULL}};
const Segment SGI249601[] = {{&S9741[105], 7}, {NULL}};
const Segment SGI249602[] = {{&S9741[120], 7}, {NULL}};
const Segment SGI249603[] = {{&S105032[34], 7}, {NULL}};
const Segment SGI249604[] = {{&S105032[27], 6}, {NULL}};
const Segment SGI249605[] = {{&S105032[99], 9}, {NULL}};
const Segment SGI249606[] = {{&S105032[72], 6}, {NULL}};
const Segment SGI249607[] = {{&S9742[13], 4}, {NULL}};
const Segment SGI249608[] = {{&S9742[93], 3}, {NULL}};
const Segment SGI249609[] = {{&S9742[58], 6}, {NULL}};
const Segment SGI249610[] = {{&S9742[104], 6}, {NULL}};
const Segment SGI249611[] = {{&S9742[118], 7}, {NULL}};
const Segment SGI249612[] = {{&S9743[13], 4}, {NULL}};
const Segment SGI249613[] = {{&S9743[94], 3}, {NULL}};
const Segment SGI249614[] = {{&S9743[26], 5}, {NULL}};
const Segment SGI249615[] = {{&S9743[59], 6}, {NULL}};
const Segment SGI249616[] = {{&S9743[105], 7}, {NULL}};
const Segment SGI249617[] = {{&S9743[120], 7}, {NULL}};
const Segment SGI249618[] = {{&S9744[13], 4}, {NULL}};
const Segment SGI249619[] = {{&S9744[94], 3}, {NULL}};
const Segment SGI249620[] = {{&S9744[26], 5}, {NULL}};
const Segment SGI249621[] = {{&S9744[59], 6}, {NULL}};
const Segment SGI249622[] = {{&S9744[105], 7}, {NULL}};
const Segment SGI249623[] = {{&S9744[120], 7}, {NULL}};
const Segment SGI249624[] = {{&S9745[13], 4}, {NULL}};
const Segment SGI249625[] = {{&S9745[94], 3}, {NULL}};
const Segment SGI249626[] = {{&S9745[26], 5}, {NULL}};
const Segment SGI249627[] = {{&S9745[59], 6}, {NULL}};
const Segment SGI249628[] = {{&S9745[105], 6}, {NULL}};
const Segment SGI249629[] = {{&S9745[119], 7}, {NULL}};
const Segment SGI249630[] = {{&S9746[13], 4}, {NULL}};
const Segment SGI249631[] = {{&S9746[92], 3}, {NULL}};
const Segment SGI249632[] = {{&S9746[26], 3}, {NULL}};
const Segment SGI249633[] = {{&S9746[57], 6}, {NULL}};
const Segment SGI249634[] = {{&S9746[103], 5}, {NULL}};
const Segment SGI249635[] = {{&S9746[116], 9}, {NULL}};
const Segment SGI249636[] = {{&S9747[13], 4}, {NULL}};
const Segment SGI249637[] = {{&S9747[92], 3}, {NULL}};
const Segment SGI249638[] = {{&S9747[26], 3}, {NULL}};
const Segment SGI249639[] = {{&S9747[57], 6}, {NULL}};
const Segment SGI249640[] = {{&S9747[103], 7}, {NULL}};
const Segment SGI249641[] = {{&S9747[118], 7}, {NULL}};
const Segment SGI249642[] = {{&S9748[13], 4}, {NULL}};
const Segment SGI249643[] = {{&S9748[92], 3}, {NULL}};
const Segment SGI249644[] = {{&S9748[26], 3}, {NULL}};
const Segment SGI249645[] = {{&S9748[57], 6}, {NULL}};
const Segment SGI249646[] = {{&S9748[103], 6}, {NULL}};
const Segment SGI249647[] = {{&S9748[117], 7}, {NULL}};
const Segment SGI249648[] = {{&S98462[34], 12}, {NULL}};
const Segment SGI249649[] = {{&S98462[27], 6}, {NULL}};
const Segment SGI249650[] = {{&S98462[99], 9}, {NULL}};
const Segment SGI249651[] = {{&S9749[13], 4}, {NULL}};
const Segment SGI249652[] = {{&S98462[72], 6}, {NULL}};
const Segment SGI249653[] = {{&S9749[94], 3}, {NULL}};
const Segment SGI249654[] = {{&S9749[26], 5}, {NULL}};
const Segment SGI249655[] = {{&S9749[59], 6}, {NULL}};
const Segment SGI249656[] = {{&S9749[105], 6}, {NULL}};
const Segment SGI249657[] = {{&S9749[119], 7}, {NULL}};
const Segment SGI249658[] = {{&S9750[13], 4}, {NULL}};
const Segment SGI249659[] = {{&S9750[92], 3}, {NULL}};
const Segment SGI249660[] = {{&S9750[26], 3}, {NULL}};
const Segment SGI249661[] = {{&S9750[57], 6}, {NULL}};
const Segment SGI249662[] = {{&S9750[103], 7}, {NULL}};
const Segment SGI249663[] = {{&S9750[118], 7}, {NULL}};
const Segment SGI249664[] = {{&S9751[13], 4}, {NULL}};
const Segment SGI249665[] = {{&S9751[92], 3}, {NULL}};
const Segment SGI249666[] = {{&S9751[26], 3}, {NULL}};
const Segment SGI249667[] = {{&S9751[57], 6}, {NULL}};
const Segment SGI249668[] = {{&S9751[103], 7}, {NULL}};
const Segment SGI249669[] = {{&S9751[118], 7}, {NULL}};
const Segment SGI249670[] = {{&S9752[12], 4}, {NULL}};
const Segment SGI249671[] = {{&S9752[106], 3}, {NULL}};
const Segment SGI249672[] = {{&S9752[35], 2}, {NULL}};
const Segment SGI249673[] = {{&S9752[72], 6}, {NULL}};
const Segment SGI249674[] = {{&S9752[117], 8}, {NULL}};
const Segment SGI249675[] = {{&S9752[133], 7}, {NULL}};
const Segment SGI249676[] = {{&S9753[13], 4}, {NULL}};
const Segment SGI249677[] = {{&S9753[94], 3}, {NULL}};
const Segment SGI249678[] = {{&S9753[26], 5}, {NULL}};
const Segment SGI249679[] = {{&S9753[59], 6}, {NULL}};
const Segment SGI249680[] = {{&S9753[105], 6}, {NULL}};
const Segment SGI249681[] = {{&S9753[119], 7}, {NULL}};
const Segment SGI249682[] = {{&S9754[13], 4}, {NULL}};
const Segment SGI249683[] = {{&S9754[92], 3}, {NULL}};
const Segment SGI249684[] = {{&S9754[26], 3}, {NULL}};
const Segment SGI249685[] = {{&S9754[57], 6}, {NULL}};
const Segment SGI249686[] = {{&S9754[103], 6}, {NULL}};
const Segment SGI249687[] = {{&S9754[117], 7}, {NULL}};
const Segment SGI249688[] = {{&S9755[13], 4}, {NULL}};
const Segment SGI249689[] = {{&S9755[94], 3}, {NULL}};
const Segment SGI249690[] = {{&S9755[26], 5}, {NULL}};
const Segment SGI249691[] = {{&S9755[59], 6}, {NULL}};
const Segment SGI249692[] = {{&S9755[105], 6}, {NULL}};
const Segment SGI249693[] = {{&S9755[119], 7}, {NULL}};
const Segment SGI249694[] = {{&S9756[13], 4}, {NULL}};
const Segment SGI249695[] = {{&S9756[94], 3}, {NULL}};
const Segment SGI249696[] = {{&S9756[26], 5}, {NULL}};
const Segment SGI249697[] = {{&S9756[59], 6}, {NULL}};
const Segment SGI249698[] = {{&S9756[105], 5}, {NULL}};
const Segment SGI249699[] = {{&S9756[118], 7}, {NULL}};
const Segment SGI249700[] = {{&S9757[13], 4}, {NULL}};
const Segment SGI249701[] = {{&S9757[94], 3}, {NULL}};
const Segment SGI249702[] = {{&S9757[26], 5}, {NULL}};
const Segment SGI249703[] = {{&S9757[59], 6}, {NULL}};
const Segment SGI249704[] = {{&S9757[105], 5}, {NULL}};
const Segment SGI249705[] = {{&S9757[118], 7}, {NULL}};
const Segment SGI249706[] = {{&S9758[13], 4}, {NULL}};
const Segment SGI249707[] = {{&S9758[94], 3}, {NULL}};
const Segment SGI249708[] = {{&S9758[26], 5}, {NULL}};
const Segment SGI249709[] = {{&S9758[59], 6}, {NULL}};
const Segment SGI249710[] = {{&S9758[105], 6}, {NULL}};
const Segment SGI249711[] = {{&S9758[119], 7}, {NULL}};
const Segment SGI249712[] = {{&S9759[13], 4}, {NULL}};
const Segment SGI249713[] = {{&S9759[94], 3}, {NULL}};
const Segment SGI249714[] = {{&S9759[26], 5}, {NULL}};
const Segment SGI249715[] = {{&S9759[59], 6}, {NULL}};
const Segment SGI249716[] = {{&S9759[105], 7}, {NULL}};
const Segment SGI249717[] = {{&S9759[120], 7}, {NULL}};
const Segment SGI249718[] = {{&S9760[13], 4}, {NULL}};
const Segment SGI249719[] = {{&S9760[94], 3}, {NULL}};
const Segment SGI249720[] = {{&S9760[26], 5}, {NULL}};
const Segment SGI249721[] = {{&S9760[59], 6}, {NULL}};
const Segment SGI249722[] = {{&S9760[105], 6}, {NULL}};
const Segment SGI249723[] = {{&S9760[119], 7}, {NULL}};
const Segment SGI249724[] = {{&S98483[44], 11}, {NULL}};
const Segment SGI249725[] = {{&S98483[44], 22}, {NULL}};
const Segment SGI249726[] = {{&S98483[30], 6}, {NULL}};
const Segment SGI249727[] = {{&S98483[115], 3}, {NULL}};
const Segment SGI249728[] = {{&S98483[80], 6}, {NULL}};
const Segment SGI249729[] = {{&S105068[34], 9}, {NULL}};
const Segment SGI249730[] = {{&S105068[27], 6}, {NULL}};
const Segment SGI249731[] = {{&S105068[96], 9}, {NULL}};
const Segment SGI249732[] = {{&S105068[69], 6}, {NULL}};
const Segment SGI249733[] = {{&S98493[34], 8}, {NULL}};
const Segment SGI249734[] = {{&S98493[27], 6}, {NULL}};
const Segment SGI249735[] = {{&S98493[94], 9}, {NULL}};
const Segment SGI249736[] = {{&S98493[67], 6}, {NULL}};
const Segment SGI249737[] = {{&S105078[34], 9}, {NULL}};
const Segment SGI249738[] = {{&S105078[27], 6}, {NULL}};
const Segment SGI249739[] = {{&S105078[96], 9}, {NULL}};
const Segment SGI249740[] = {{&S105078[69], 6}, {NULL}};
const Segment SGI249741[] = {{&S105078[137], 8}, {NULL}};
const Segment SGI249742[] = {{&S105084[34], 7}, {NULL}};
const Segment SGI249743[] = {{&S105084[27], 6}, {NULL}};
const Segment SGI249744[] = {{&S105084[97], 9}, {NULL}};
const Segment SGI249745[] = {{&S105084[70], 6}, {NULL}};
const Segment SGI249746[] = {{&S9813[13], 6}, {NULL}};
const Segment SGI249747[] = {{&S9813[35], 3}, {NULL}};
const Segment SGI249748[] = {{&S9813[66], 6}, {NULL}};
const Segment SGI249749[] = {{&S9813[100], 7}, {NULL}};
const Segment SGI249750[] = {{&S9814[13], 6}, {NULL}};
const Segment SGI249751[] = {{&S9814[35], 3}, {NULL}};
const Segment SGI249752[] = {{&S9814[66], 6}, {NULL}};
const Segment SGI249753[] = {{&S9814[100], 7}, {NULL}};
const Segment SGI249754[] = {{&S9815[13], 6}, {NULL}};
const Segment SGI249755[] = {{&S9815[35], 3}, {NULL}};
const Segment SGI249756[] = {{&S9815[66], 6}, {NULL}};
const Segment SGI249757[] = {{&S9815[100], 7}, {NULL}};
const Segment SGI249758[] = {{&S9816[13], 6}, {NULL}};
const Segment SGI249759[] = {{&S9816[35], 3}, {NULL}};
const Segment SGI249760[] = {{&S9816[66], 6}, {NULL}};
const Segment SGI249761[] = {{&S9817[13], 6}, {NULL}};
const Segment SGI249762[] = {{&S9817[35], 3}, {NULL}};
const Segment SGI249763[] = {{&S9817[66], 6}, {NULL}};
const Segment SGI249764[] = {{&S9817[100], 5}, {NULL}};
const Segment SGI249765[] = {{&S9818[13], 6}, {NULL}};
const Segment SGI249766[] = {{&S9818[35], 5}, {NULL}};
const Segment SGI249767[] = {{&S9818[68], 6}, {NULL}};
const Segment SGI249768[] = {{&S9819[13], 6}, {NULL}};
const Segment SGI249769[] = {{&S9819[35], 5}, {NULL}};
const Segment SGI249770[] = {{&S9819[68], 6}, {NULL}};
const Segment SGI249771[] = {{&S9819[102], 5}, {NULL}};
const Segment SGI249772[] = {{&S9820[13], 6}, {NULL}};
const Segment SGI249773[] = {{&S9820[35], 5}, {NULL}};
const Segment SGI249774[] = {{&S9820[68], 6}, {NULL}};
const Segment SGI249775[] = {{&S9820[102], 5}, {NULL}};
const Segment SGI249776[] = {{&S9821[13], 6}, {NULL}};
const Segment SGI249777[] = {{&S9821[35], 5}, {NULL}};
const Segment SGI249778[] = {{&S9821[68], 6}, {NULL}};
const Segment SGI249779[] = {{&S9821[102], 5}, {NULL}};
const Segment SGI249780[] = {{&S9822[15], 6}, {NULL}};
const Segment SGI249781[] = {{&S9822[33], 3}, {NULL}};
const Segment SGI249782[] = {{&S9822[0], 7}, {NULL}};
const Segment SGI249783[] = {{&S9823[13], 6}, {NULL}};
const Segment SGI249784[] = {{&S9823[130], 3}, {NULL}};
const Segment SGI249785[] = {{&S9823[35], 3}, {NULL}};
const Segment SGI249786[] = {{&S9823[66], 6}, {NULL}};
const Segment SGI249787[] = {{&S9823[100], 5}, {NULL}};
const Segment SGI249788[] = {{&S9824[27], 6}, {NULL}};
const Segment SGI249789[] = {{&S9824[52], 3}, {NULL}};
const Segment SGI249790[] = {{&S9824[90], 6}, {NULL}};
const Segment SGI249791[] = {{&S9825[13], 6}, {NULL}};
const Segment SGI249792[] = {{&S9825[35], 5}, {NULL}};
const Segment SGI249793[] = {{&S9825[68], 6}, {NULL}};
const Segment SGI249794[] = {{&S9825[102], 4}, {NULL}};
const Segment SGI249795[] = {{&S9826[13], 6}, {NULL}};
const Segment SGI249796[] = {{&S9826[38], 5}, {NULL}};
const Segment SGI249797[] = {{&S9827[13], 6}, {&S9827[108], 4}, {NULL}};
const Segment SGI249798[] = {{&S9827[35], 5}, {NULL}};
const Segment SGI249799[] = {{&S9827[68], 6}, {NULL}};
const Segment SGI249800[] = {{&S9827[102], 5}, {NULL}};
const Segment SGI249801[] = {{&S9828[13], 6}, {NULL}};
const Segment SGI249802[] = {{&S9828[132], 3}, {NULL}};
const Segment SGI249803[] = {{&S9828[35], 5}, {NULL}};
const Segment SGI249804[] = {{&S9828[68], 6}, {NULL}};
const Segment SGI249805[] = {{&S9828[102], 5}, {NULL}};
const Segment SGI249806[] = {{&S9829[13], 6}, {NULL}};
const Segment SGI249807[] = {{&S9829[38], 3}, {NULL}};
const Segment SGI249808[] = {{&S9830[27], 6}, {NULL}};
const Segment SGI249809[] = {{&S9830[52], 5}, {NULL}};
const Segment SGI249810[] = {{&S9830[92], 6}, {NULL}};
const Segment SGI249811[] = {{&S98690[34], 11}, {NULL}};
const Segment SGI249812[] = {{&S98690[27], 6}, {NULL}};
const Segment SGI249813[] = {{&S98690[97], 9}, {NULL}};
const Segment SGI249814[] = {{&S98690[70], 6}, {NULL}};
const Segment SGI249815[] = {{&S9831[13], 6}, {NULL}};
const Segment SGI249816[] = {{&S9831[35], 5}, {NULL}};
const Segment SGI249817[] = {{&S9831[68], 6}, {NULL}};
const Segment SGI249818[] = {{&S9831[102], 5}, {NULL}};
const Segment SGI249819[] = {{&S98691[34], 12}, {NULL}};
const Segment SGI249820[] = {{&S98691[27], 6}, {NULL}};
const Segment SGI249821[] = {{&S98691[98], 9}, {NULL}};
const Segment SGI249822[] = {{&S98691[71], 6}, {NULL}};
const Segment SGI249823[] = {{&S9833[13], 6}, {NULL}};
const Segment SGI249824[] = {{&S9833[28], 3}, {NULL}};
const Segment SGI249825[] = {{&S9833[59], 6}, {NULL}};
const Segment SGI249826[] = {{&S9833[93], 7}, {NULL}};
const Segment SGI249827[] = {{&S9834[13], 6}, {NULL}};
const Segment SGI249828[] = {{&S9834[35], 5}, {NULL}};
const Segment SGI249829[] = {{&S9834[68], 6}, {NULL}};
const Segment SGI249830[] = {{&S9834[102], 5}, {NULL}};
const Segment SGI249831[] = {{&S9835[13], 6}, {NULL}};
const Segment SGI249832[] = {{&S9835[28], 3}, {NULL}};
const Segment SGI249833[] = {{&S9835[59], 6}, {NULL}};
const Segment SGI249834[] = {{&S9835[93], 5}, {NULL}};
const Segment SGI249835[] = {{&S9835[106], 7}, {NULL}};
const Segment SGI249836[] = {{&S9836[13], 6}, {NULL}};
const Segment SGI249837[] = {{&S9836[28], 3}, {NULL}};
const Segment SGI249838[] = {{&S9836[59], 6}, {NULL}};
const Segment SGI249839[] = {{&S9836[93], 5}, {NULL}};
const Segment SGI249840[] = {{&S9836[106], 7}, {NULL}};
const Segment SGI249841[] = {{&S9837[13], 6}, {&S9837[13], 6}, {NULL}};
const Segment SGI249842[] = {{&S9837[75], 6}, {NULL}};
const Segment SGI249843[] = {{&S9837[109], 4}, {NULL}};
const Segment SGI249844[] = {{&S9838[13], 6}, {NULL}};
const Segment SGI249845[] = {{&S9838[35], 3}, {NULL}};
const Segment SGI249846[] = {{&S9838[66], 6}, {NULL}};
const Segment SGI249847[] = {{&S9838[100], 6}, {NULL}};
const Segment SGI249848[] = {{&S9839[13], 6}, {NULL}};
const Segment SGI249849[] = {{&S9839[38], 3}, {NULL}};
const Segment SGI249850[] = {{&S9839[69], 6}, {NULL}};
const Segment SGI249851[] = {{&S9839[103], 5}, {NULL}};
const Segment SGI249852[] = {{&S9839[116], 9}, {NULL}};
const Segment SGI249853[] = {{&S9840[15], 6}, {NULL}};
const Segment SGI249854[] = {{&S9840[33], 5}, {NULL}};
const Segment SGI249855[] = {{&S9840[0], 7}, {NULL}};
const Segment SGI249856[] = {{&S9841[13], 6}, {NULL}};
const Segment SGI249857[] = {{&S9841[35], 3}, {NULL}};
const Segment SGI249858[] = {{&S9841[66], 6}, {NULL}};
const Segment SGI249859[] = {{&S9841[99], 5}, {NULL}};
const Segment SGI249860[] = {{&S9842[13], 6}, {NULL}};
const Segment SGI249861[] = {{&S9842[35], 3}, {NULL}};
const Segment SGI249862[] = {{&S9842[66], 6}, {NULL}};
const Segment SGI249863[] = {{&S9842[100], 6}, {NULL}};
const Segment SGI249864[] = {{&S9966[13], 6}, {NULL}};
const Segment SGI249865[] = {{&S98729[34], 10}, {NULL}};
const Segment SGI249866[] = {{&S9966[101], 3}, {NULL}};
const Segment SGI249867[] = {{&S98729[27], 6}, {NULL}};
const Segment SGI249868[] = {{&S9966[35], 3}, {NULL}};
const Segment SGI249869[] = {{&S9966[66], 6}, {NULL}};
const Segment SGI249870[] = {{&S9966[112], 8}, {NULL}};
const Segment SGI249871[] = {{&S9966[128], 7}, {NULL}};
const Segment SGI249872[] = {{&S98729[96], 9}, {NULL}};
const Segment SGI249873[] = {{&S98729[69], 6}, {NULL}};
const Segment SGI249874[] = {{&S9967[13], 6}, {NULL}};
const Segment SGI249875[] = {{&S9967[101], 3}, {NULL}};
const Segment SGI249876[] = {{&S9967[35], 3}, {NULL}};
const Segment SGI249877[] = {{&S9967[66], 6}, {NULL}};
const Segment SGI249878[] = {{&S9967[112], 6}, {NULL}};
const Segment SGI249879[] = {{&S9967[126], 7}, {NULL}};
const Segment SGI249880[] = {{&S9968[13], 6}, {NULL}};
const Segment SGI249881[] = {{&S9968[101], 3}, {NULL}};
const Segment SGI249882[] = {{&S9968[35], 3}, {NULL}};
const Segment SGI249883[] = {{&S9968[66], 6}, {NULL}};
const Segment SGI249884[] = {{&S9968[112], 6}, {NULL}};
const Segment SGI249885[] = {{&S9968[126], 7}, {NULL}};
const Segment SGI249886[] = {{&S9969[13], 6}, {NULL}};
const Segment SGI249887[] = {{&S9969[101], 3}, {NULL}};
const Segment SGI249888[] = {{&S9969[35], 3}, {NULL}};
const Segment SGI249889[] = {{&S9969[66], 6}, {NULL}};
const Segment SGI249890[] = {{&S9969[112], 6}, {NULL}};
const Segment SGI249891[] = {{&S9969[126], 7}, {NULL}};
const Segment SGI249892[] = {{&S9970[13], 6}, {NULL}};
const Segment SGI249893[] = {{&S9970[101], 3}, {NULL}};
const Segment SGI249894[] = {{&S9970[35], 3}, {NULL}};
const Segment SGI249895[] = {{&S9970[66], 6}, {NULL}};
const Segment SGI249896[] = {{&S9970[112], 6}, {NULL}};
const Segment SGI249897[] = {{&S9970[126], 7}, {NULL}};
const Segment SGI249898[] = {{&S9971[13], 6}, {NULL}};
const Segment SGI249899[] = {{&S9971[100], 3}, {NULL}};
const Segment SGI249900[] = {{&S9971[65], 6}, {NULL}};
const Segment SGI249901[] = {{&S9971[111], 6}, {NULL}};
const Segment SGI249902[] = {{&S9971[125], 7}, {NULL}};
const Segment SGI249903[] = {{&S9972[13], 6}, {NULL}};
const Segment SGI249904[] = {{&S9972[101], 5}, {&S9972[129], 3}, {NULL}};
const Segment SGI249905[] = {{&S9972[35], 3}, {NULL}};
const Segment SGI249906[] = {{&S9972[66], 6}, {NULL}};
const Segment SGI249907[] = {{&S9972[114], 6}, {NULL}};
const Segment SGI249908[] = {{&S9972[140], 7}, {&S9972[155], 9}, {NULL}};
const Segment SGI249909[] = {{&S9973[13], 6}, {NULL}};
const Segment SGI249910[] = {{&S9973[101], 3}, {NULL}};
const Segment SGI249911[] = {{&S9973[35], 3}, {NULL}};
const Segment SGI249912[] = {{&S9973[66], 6}, {NULL}};
const Segment SGI249913[] = {{&S9973[112], 12}, {NULL}};
const Segment SGI249914[] = {{&S98740[44], 18}, {NULL}};
const Segment SGI249915[] = {{&S9973[125], 7}, {NULL}};
const Segment SGI249916[] = {{&S98740[44], 29}, {NULL}};
const Segment SGI249917[] = {{&S98740[30], 6}, {NULL}};
const Segment SGI249918[] = {{&S98740[122], 3}, {NULL}};
const Segment SGI249919[] = {{&S98740[87], 6}, {NULL}};
const Segment SGI249920[] = {{&S9974[13], 6}, {NULL}};
const Segment SGI249921[] = {{&S9974[101], 5}, {&S9974[129], 3}, {NULL}};
const Segment SGI249922[] = {{&S9974[35], 3}, {NULL}};
const Segment SGI249923[] = {{&S9974[66], 6}, {NULL}};
const Segment SGI249924[] = {{&S9974[114], 6}, {NULL}};
const Segment SGI249925[] = {{&S9974[140], 7}, {&S9974[155], 9}, {NULL}};
const Segment SGI249926[] = {{&S9975[13], 6}, {NULL}};
const Segment SGI249927[] = {{&S9975[103], 3}, {NULL}};
const Segment SGI249928[] = {{&S9975[35], 5}, {NULL}};
const Segment SGI249929[] = {{&S9975[68], 6}, {NULL}};
const Segment SGI249930[] = {{&S9975[114], 6}, {NULL}};
const Segment SGI249931[] = {{&S9975[128], 7}, {NULL}};
const Segment SGI249932[] = {{&S9976[13], 6}, {NULL}};
const Segment SGI249933[] = {{&S9976[103], 3}, {NULL}};
const Segment SGI249934[] = {{&S9976[35], 5}, {NULL}};
const Segment SGI249935[] = {{&S9976[68], 6}, {NULL}};
const Segment SGI249936[] = {{&S9976[114], 6}, {NULL}};
const Segment SGI249937[] = {{&S9976[128], 7}, {NULL}};
const Segment SGI249938[] = {{&S9977[13], 6}, {NULL}};
const Segment SGI249939[] = {{&S9977[103], 3}, {NULL}};
const Segment SGI249940[] = {{&S9977[35], 5}, {NULL}};
const Segment SGI249941[] = {{&S9977[68], 6}, {NULL}};
const Segment SGI249942[] = {{&S9977[114], 6}, {NULL}};
const Segment SGI249943[] = {{&S9977[128], 7}, {NULL}};
const Segment SGI249944[] = {{&S9978[13], 6}, {NULL}};
const Segment SGI249945[] = {{&S9978[101], 3}, {NULL}};
const Segment SGI249946[] = {{&S9978[35], 3}, {NULL}};
const Segment SGI249947[] = {{&S9978[66], 6}, {NULL}};
const Segment SGI249948[] = {{&S9978[112], 6}, {NULL}};
const Segment SGI249949[] = {{&S9978[126], 7}, {NULL}};
const Segment SGI249950[] = {{&S9979[13], 6}, {NULL}};
const Segment SGI249951[] = {{&S9979[101], 3}, {NULL}};
const Segment SGI249952[] = {{&S9979[35], 3}, {NULL}};
const Segment SGI249953[] = {{&S9979[66], 6}, {NULL}};
const Segment SGI249954[] = {{&S9979[112], 6}, {NULL}};
const Segment SGI249955[] = {{&S9979[126], 7}, {NULL}};
const Segment SGI249956[] = {{&S9980[13], 6}, {NULL}};
const Segment SGI249957[] = {{&S9980[101], 3}, {NULL}};
const Segment SGI249958[] = {{&S9980[35], 3}, {NULL}};
const Segment SGI249959[] = {{&S9980[66], 6}, {NULL}};
const Segment SGI249960[] = {{&S9980[112], 6}, {NULL}};
const Segment SGI249961[] = {{&S9980[126], 7}, {NULL}};
const Segment SGI249962[] = {{&S9981[13], 6}, {NULL}};
const Segment SGI249963[] = {{&S9981[102], 3}, {NULL}};
const Segment SGI249964[] = {{&S9981[67], 6}, {NULL}};
const Segment SGI249965[] = {{&S9981[113], 6}, {NULL}};
const Segment SGI249966[] = {{&S9981[127], 7}, {NULL}};
const Segment SGI249967[] = {{&S9982[13], 6}, {NULL}};
const Segment SGI249968[] = {{&S9982[103], 3}, {NULL}};
const Segment SGI249969[] = {{&S9982[35], 5}, {NULL}};
const Segment SGI249970[] = {{&S9982[68], 6}, {NULL}};
const Segment SGI249971[] = {{&S9982[114], 12}, {NULL}};
const Segment SGI249972[] = {{&S9982[127], 7}, {NULL}};
const Segment SGI249973[] = {{&S9983[13], 6}, {NULL}};
const Segment SGI249974[] = {{&S9983[103], 5}, {&S9983[131], 3}, {NULL}};
const Segment SGI249975[] = {{&S9983[35], 5}, {NULL}};
const Segment SGI249976[] = {{&S9983[68], 6}, {NULL}};
const Segment SGI249977[] = {{&S9983[116], 6}, {NULL}};
const Segment SGI249978[] = {{&S9983[142], 7}, {&S9983[157], 9}, {NULL}};
const Segment SGI249979[] = {{&S9984[13], 6}, {NULL}};
const Segment SGI249980[] = {{&S9984[103], 5}, {&S9984[131], 3}, {NULL}};
const Segment SGI249981[] = {{&S9984[35], 5}, {NULL}};
const Segment SGI249982[] = {{&S9984[68], 6}, {NULL}};
const Segment SGI249983[] = {{&S9984[116], 6}, {NULL}};
const Segment SGI249984[] = {{&S9984[142], 7}, {&S9984[157], 9}, {NULL}};
const Segment SGI249985[] = {{&S9985[13], 6}, {NULL}};
const Segment SGI249986[] = {{&S9985[103], 3}, {NULL}};
const Segment SGI249987[] = {{&S9985[35], 5}, {NULL}};
const Segment SGI249988[] = {{&S9985[68], 6}, {NULL}};
const Segment SGI249989[] = {{&S9985[114], 6}, {NULL}};
const Segment SGI249990[] = {{&S9985[128], 7}, {NULL}};
const Segment SGI249991[] = {{&S9986[13], 6}, {NULL}};
const Segment SGI249992[] = {{&S9986[102], 3}, {NULL}};
const Segment SGI249993[] = {{&S9986[67], 6}, {NULL}};
const Segment SGI249994[] = {{&S9986[113], 6}, {NULL}};
const Segment SGI249995[] = {{&S9986[127], 7}, {NULL}};
const Segment SGI249996[] = {{&S9987[13], 6}, {NULL}};
const Segment SGI249997[] = {{&S9987[103], 3}, {NULL}};
const Segment SGI249998[] = {{&S9987[35], 5}, {NULL}};
const Segment SGI249999[] = {{&S9987[68], 6}, {NULL}};
const Segment SGI250000[] = {{&S9987[114], 12}, {NULL}};
const Segment SGI250001[] = {{&S9987[127], 7}, {NULL}};
const Segment SGI250002[] = {{&S9988[13], 6}, {NULL}};
const Segment SGI250003[] = {{&S9988[103], 5}, {&S9988[131], 3}, {NULL}};
const Segment SGI250004[] = {{&S9988[35], 5}, {NULL}};
const Segment SGI250005[] = {{&S9988[68], 6}, {NULL}};
const Segment SGI250006[] = {{&S9988[116], 6}, {NULL}};
const Segment SGI250007[] = {{&S9988[142], 7}, {&S9988[157], 9}, {NULL}};
const Segment SGI250008[] = {{&S9989[13], 6}, {NULL}};
const Segment SGI250009[] = {{&S9989[102], 3}, {NULL}};
const Segment SGI250010[] = {{&S9989[67], 6}, {NULL}};
const Segment SGI250011[] = {{&S9989[113], 6}, {NULL}};
const Segment SGI250012[] = {{&S9989[127], 7}, {NULL}};
const Segment SGI250013[] = {{&S9990[13], 6}, {NULL}};
const Segment SGI250014[] = {{&S9990[103], 3}, {NULL}};
const Segment SGI250015[] = {{&S9990[35], 5}, {NULL}};
const Segment SGI250016[] = {{&S9990[68], 6}, {NULL}};
const Segment SGI250017[] = {{&S9990[114], 12}, {NULL}};
const Segment SGI250018[] = {{&S9990[127], 7}, {NULL}};
const Segment SGI250019[] = {{&S9991[13], 6}, {NULL}};
const Segment SGI250020[] = {{&S9991[94], 3}, {NULL}};
const Segment SGI250021[] = {{&S9991[28], 3}, {NULL}};
const Segment SGI250022[] = {{&S9991[59], 6}, {NULL}};
const Segment SGI250023[] = {{&S9991[105], 8}, {NULL}};
const Segment SGI250024[] = {{&S9991[121], 7}, {NULL}};
const Segment SGI250025[] = {{&S9992[13], 6}, {NULL}};
const Segment SGI250026[] = {{&S9992[101], 3}, {NULL}};
const Segment SGI250027[] = {{&S9992[35], 3}, {NULL}};
const Segment SGI250028[] = {{&S9992[66], 6}, {NULL}};
const Segment SGI250029[] = {{&S9992[112], 6}, {NULL}};
const Segment SGI250030[] = {{&S9992[126], 7}, {NULL}};
const Segment SGI250031[] = {{&S9993[13], 6}, {NULL}};
const Segment SGI250032[] = {{&S9993[103], 3}, {NULL}};
const Segment SGI250033[] = {{&S9993[35], 5}, {NULL}};
const Segment SGI250034[] = {{&S9993[68], 6}, {NULL}};
const Segment SGI250035[] = {{&S9993[114], 6}, {NULL}};
const Segment SGI250036[] = {{&S9993[128], 7}, {NULL}};
const Segment SGI250037[] = {{&S9994[13], 6}, {NULL}};
const Segment SGI250038[] = {{&S9994[103], 3}, {NULL}};
const Segment SGI250039[] = {{&S9994[35], 5}, {NULL}};
const Segment SGI250040[] = {{&S9994[68], 6}, {NULL}};
const Segment SGI250041[] = {{&S9994[114], 5}, {NULL}};
const Segment SGI250042[] = {{&S9994[127], 7}, {NULL}};
const Segment SGI250043[] = {{&S9995[13], 6}, {NULL}};
const Segment SGI250044[] = {{&S9995[103], 3}, {NULL}};
const Segment SGI250045[] = {{&S9995[35], 5}, {NULL}};
const Segment SGI250046[] = {{&S9995[68], 6}, {NULL}};
const Segment SGI250047[] = {{&S9995[114], 6}, {NULL}};
const Segment SGI250048[] = {{&S9995[128], 7}, {NULL}};
const Segment SGI250049[] = {{&S9996[13], 6}, {NULL}};
const Segment SGI250050[] = {{&S9996[101], 3}, {NULL}};
const Segment SGI250051[] = {{&S9996[35], 3}, {NULL}};
const Segment SGI250052[] = {{&S9996[66], 6}, {NULL}};
const Segment SGI250053[] = {{&S9996[112], 5}, {NULL}};
const Segment SGI250054[] = {{&S9996[125], 9}, {NULL}};
const Segment SGI250055[] = {{&S9997[13], 6}, {NULL}};
const Segment SGI250056[] = {{&S9997[101], 3}, {NULL}};
const Segment SGI250057[] = {{&S9997[35], 3}, {NULL}};
const Segment SGI250058[] = {{&S9997[66], 6}, {NULL}};
const Segment SGI250059[] = {{&S9997[112], 6}, {NULL}};
const Segment SGI250060[] = {{&S9997[126], 7}, {NULL}};
const Segment SGI250061[] = {{&S9998[13], 6}, {NULL}};
const Segment SGI250062[] = {{&S9998[103], 3}, {NULL}};
const Segment SGI250063[] = {{&S9998[35], 5}, {NULL}};
const Segment SGI250064[] = {{&S9998[68], 6}, {NULL}};
const Segment SGI250065[] = {{&S9998[114], 6}, {NULL}};
const Segment SGI250066[] = {{&S9998[128], 7}, {NULL}};
const Segment SGI250067[] = {{&S9999[13], 6}, {NULL}};
const Segment SGI250068[] = {{&S9999[101], 3}, {NULL}};
const Segment SGI250069[] = {{&S9999[35], 3}, {NULL}};
const Segment SGI250070[] = {{&S9999[66], 6}, {NULL}};
const Segment SGI250071[] = {{&S9999[112], 5}, {NULL}};
const Segment SGI250072[] = {{&S9999[125], 7}, {NULL}};
const Segment SGI250073[] = {{&S10000[13], 6}, {NULL}};
const Segment SGI250074[] = {{&S10000[103], 3}, {NULL}};
const Segment SGI250075[] = {{&S10000[35], 5}, {NULL}};
const Segment SGI250076[] = {{&S10000[68], 6}, {NULL}};
const Segment SGI250077[] = {{&S10000[114], 5}, {NULL}};
const Segment SGI250078[] = {{&S10000[127], 7}, {NULL}};
const Segment SGI250079[] = {{&S10001[13], 6}, {NULL}};
const Segment SGI250080[] = {{&S10001[130], 3}, {NULL}};
const Segment SGI250081[] = {{&S10001[35], 5}, {NULL}};
const Segment SGI250082[] = {{&S10001[68], 5}, {NULL}};
const Segment SGI250083[] = {{&S10001[141], 6}, {NULL}};
const Segment SGI250084[] = {{&S10001[155], 7}, {NULL}};
const Segment SGI250085[] = {{&S10002[13], 6}, {NULL}};
const Segment SGI250086[] = {{&S10002[103], 3}, {NULL}};
const Segment SGI250087[] = {{&S10002[35], 5}, {NULL}};
const Segment SGI250088[] = {{&S10002[68], 6}, {NULL}};
const Segment SGI250089[] = {{&S10002[114], 6}, {NULL}};
const Segment SGI250090[] = {{&S10002[128], 7}, {NULL}};
const Segment SGI250091[] = {{&S10003[13], 6}, {NULL}};
const Segment SGI250092[] = {{&S10003[103], 3}, {NULL}};
const Segment SGI250093[] = {{&S10003[35], 5}, {NULL}};
const Segment SGI250094[] = {{&S10003[68], 6}, {NULL}};
const Segment SGI250095[] = {{&S10003[114], 6}, {NULL}};
const Segment SGI250096[] = {{&S10003[128], 7}, {NULL}};
const Segment SGI250097[] = {{&S10004[13], 6}, {NULL}};
const Segment SGI250098[] = {{&S10004[101], 3}, {NULL}};
const Segment SGI250099[] = {{&S10004[35], 3}, {NULL}};
const Segment SGI250100[] = {{&S10004[66], 6}, {NULL}};
const Segment SGI250101[] = {{&S10004[112], 6}, {NULL}};
const Segment SGI250102[] = {{&S10004[126], 7}, {NULL}};
const Segment SGI250103[] = {{&S10005[13], 6}, {NULL}};
const Segment SGI250104[] = {{&S10005[100], 3}, {NULL}};
const Segment SGI250105[] = {{&S10005[65], 6}, {NULL}};
const Segment SGI250106[] = {{&S10005[111], 6}, {NULL}};
const Segment SGI250107[] = {{&S10005[125], 7}, {NULL}};
const Segment SGI250108[] = {{&S10006[13], 6}, {NULL}};
const Segment SGI250109[] = {{&S10006[103], 3}, {NULL}};
const Segment SGI250110[] = {{&S10006[35], 5}, {NULL}};
const Segment SGI250111[] = {{&S10006[68], 6}, {NULL}};
const Segment SGI250112[] = {{&S10006[114], 6}, {NULL}};
const Segment SGI250113[] = {{&S10006[128], 7}, {NULL}};
const Segment SGI250114[] = {{&S98753[44], 9}, {NULL}};
const Segment SGI250115[] = {{&S98753[44], 20}, {NULL}};
const Segment SGI250116[] = {{&S98753[30], 6}, {NULL}};
const Segment SGI250117[] = {{&S98753[111], 3}, {NULL}};
const Segment SGI250118[] = {{&S98753[78], 5}, {NULL}};
const Segment SGI250119[] = {{&S105552[34], 9}, {NULL}};
const Segment SGI250120[] = {{&S105552[27], 6}, {NULL}};
const Segment SGI250121[] = {{&S105552[96], 9}, {NULL}};
const Segment SGI250122[] = {{&S105552[69], 6}, {NULL}};
const Segment SGI250123[] = {{&S98762[44], 14}, {NULL}};
const Segment SGI250124[] = {{&S98762[44], 25}, {NULL}};
const Segment SGI250125[] = {{&S98762[30], 6}, {NULL}};
const Segment SGI250126[] = {{&S98762[116], 3}, {NULL}};
const Segment SGI250127[] = {{&S98762[83], 5}, {NULL}};
const Segment SGI250128[] = {{&S105553[59], 9}, {NULL}};
const Segment SGI250129[] = {{&S105553[59], 21}, {NULL}};
const Segment SGI250130[] = {{&S105553[45], 6}, {NULL}};
const Segment SGI250131[] = {{&S105553[94], 5}, {NULL}};
const Segment SGI250132[] = {{&S105553[0], 14}, {NULL}};
const Segment SGI250133[] = {{&S105567[44], 11}, {NULL}};
const Segment SGI250134[] = {{&S105567[44], 24}, {NULL}};
const Segment SGI250135[] = {{&S105567[30], 6}, {NULL}};
const Segment SGI250136[] = {{&S98777[44], 10}, {NULL}};
const Segment SGI250137[] = {{&S98777[44], 21}, {NULL}};
const Segment SGI250138[] = {{&S98777[30], 6}, {NULL}};
const Segment SGI250139[] = {{&S105567[117], 3}, {NULL}};
const Segment SGI250140[] = {{&S105567[82], 6}, {NULL}};
const Segment SGI250141[] = {{&S98777[134], 3}, {NULL}};
const Segment SGI250142[] = {{&S98777[99], 6}, {NULL}};
const Segment SGI250143[] = {{&S98777[67], 8}, {NULL}};
const Segment SGI250144[] = {{&S105568[34], 10}, {NULL}};
const Segment SGI250145[] = {{&S105568[27], 6}, {NULL}};
const Segment SGI250146[] = {{&S105568[97], 9}, {NULL}};
const Segment SGI250147[] = {{&S105568[70], 6}, {NULL}};
const Segment SGI250148[] = {{&S105569[34], 18}, {NULL}};
const Segment SGI250149[] = {{&S105569[27], 6}, {NULL}};
const Segment SGI250150[] = {{&S98780[34], 11}, {NULL}};
const Segment SGI250151[] = {{&S98780[27], 6}, {NULL}};
const Segment SGI250152[] = {{&S105569[105], 9}, {NULL}};
const Segment SGI250153[] = {{&S105569[78], 6}, {NULL}};
const Segment SGI250154[] = {{&S98780[97], 9}, {NULL}};
const Segment SGI250155[] = {{&S98780[70], 6}, {NULL}};
const Segment SGI250156[] = {{&S98799[44], 7}, {NULL}};
const Segment SGI250157[] = {{&S98799[44], 19}, {NULL}};
const Segment SGI250158[] = {{&S98799[30], 6}, {NULL}};
const Segment SGI250159[] = {{&S98799[112], 3}, {NULL}};
const Segment SGI250160[] = {{&S98799[77], 6}, {NULL}};
const Segment SGI250161[] = {{&S105591[44], 10}, {NULL}};
const Segment SGI250162[] = {{&S105591[44], 22}, {NULL}};
const Segment SGI250163[] = {{&S105591[30], 6}, {NULL}};
const Segment SGI250164[] = {{&S105591[115], 3}, {NULL}};
const Segment SGI250165[] = {{&S105591[80], 6}, {NULL}};
const Segment SGI250166[] = {{&S105591[150], 3}, {NULL}};
const Segment SGI250167[] = {{&S98825[34], 10}, {NULL}};
const Segment SGI250168[] = {{&S98825[27], 6}, {NULL}};
const Segment SGI250169[] = {{&S98825[96], 9}, {NULL}};
const Segment SGI250170[] = {{&S98825[69], 6}, {NULL}};
const Segment SGI250171[] = {{&S98825[137], 8}, {NULL}};
const Segment SGI250172[] = {{&S105618[44], 10}, {NULL}};
const Segment SGI250173[] = {{&S105618[44], 22}, {NULL}};
const Segment SGI250174[] = {{&S105618[30], 6}, {NULL}};
const Segment SGI250175[] = {{&S105618[135], 3}, {NULL}};
const Segment SGI250176[] = {{&S105618[100], 6}, {NULL}};
const Segment SGI250177[] = {{&S105618[163], 3}, {NULL}};
const Segment SGI250178[] = {{&S105618[68], 8}, {NULL}};
const Segment SGI250179[] = {{&S98855[44], 10}, {NULL}};
const Segment SGI250180[] = {{&S98855[44], 21}, {NULL}};
const Segment SGI250181[] = {{&S98855[30], 6}, {NULL}};
const Segment SGI250182[] = {{&S98855[112], 3}, {NULL}};
const Segment SGI250183[] = {{&S98855[79], 5}, {NULL}};
const Segment SGI250184[] = {{&S98858[34], 10}, {NULL}};
const Segment SGI250185[] = {{&S98858[27], 6}, {NULL}};
const Segment SGI250186[] = {{&S98858[97], 9}, {NULL}};
const Segment SGI250187[] = {{&S98858[70], 6}, {NULL}};
const Segment SGI250188[] = {{&S98862[34], 6}, {NULL}};
const Segment SGI250189[] = {{&S98862[27], 6}, {NULL}};
const Segment SGI250190[] = {{&S98862[92], 9}, {NULL}};
const Segment SGI250191[] = {{&S98862[65], 6}, {NULL}};
const Segment SGI250192[] = {{&S105662[44], 10}, {NULL}};
const Segment SGI250193[] = {{&S105662[44], 22}, {NULL}};
const Segment SGI250194[] = {{&S105662[30], 6}, {NULL}};
const Segment SGI250195[] = {{&S105662[131], 3}, {NULL}};
const Segment SGI250196[] = {{&S105662[96], 6}, {NULL}};
const Segment SGI250197[] = {{&S105662[68], 8}, {NULL}};
const Segment SGI250198[] = {{&S98889[44], 10}, {NULL}};
const Segment SGI250199[] = {{&S98889[44], 22}, {NULL}};
const Segment SGI250200[] = {{&S98889[30], 6}, {NULL}};
const Segment SGI250201[] = {{&S98889[115], 3}, {NULL}};
const Segment SGI250202[] = {{&S98889[80], 6}, {NULL}};
const Segment SGI250203[] = {{&S105678[44], 10}, {NULL}};
const Segment SGI250204[] = {{&S105678[44], 73}, {NULL}};
const Segment SGI250205[] = {{&S105678[30], 6}, {NULL}};
const Segment SGI250206[] = {{&S98897[44], 10}, {NULL}};
const Segment SGI250207[] = {{&S98897[44], 27}, {NULL}};
const Segment SGI250208[] = {{&S98897[30], 6}, {NULL}};
const Segment SGI250209[] = {{&S98897[119], 3}, {NULL}};
const Segment SGI250210[] = {{&S98897[85], 5}, {NULL}};
const Segment SGI250211[] = {{&S98908[44], 10}, {NULL}};
const Segment SGI250212[] = {{&S98908[44], 21}, {NULL}};
const Segment SGI250213[] = {{&S98908[30], 6}, {NULL}};
const Segment SGI250214[] = {{&S98908[114], 3}, {NULL}};
const Segment SGI250215[] = {{&S98908[79], 6}, {NULL}};
const Segment SGI250216[] = {{&S98908[134], 8}, {NULL}};
const Segment SGI250217[] = {{&S98908[143], 13}, {NULL}};
const Segment SGI250218[] = {{&S98909[34], 15}, {NULL}};
const Segment SGI250219[] = {{&S98909[27], 6}, {NULL}};
const Segment SGI250220[] = {{&S98909[102], 9}, {NULL}};
const Segment SGI250221[] = {{&S98909[75], 6}, {NULL}};
const Segment SGI250222[] = {{&S98909[143], 8}, {NULL}};
const Segment SGI250223[] = {{&S105700[44], 10}, {NULL}};
const Segment SGI250224[] = {{&S105700[44], 22}, {NULL}};
const Segment SGI250225[] = {{&S105700[30], 6}, {NULL}};
const Segment SGI250226[] = {{&S105700[131], 3}, {NULL}};
const Segment SGI250227[] = {{&S105700[96], 6}, {NULL}};
const Segment SGI250228[] = {{&S105700[68], 8}, {NULL}};
const Segment SGI250229[] = {{&S105700[148], 10}, {NULL}};
const Segment SGI250230[] = {{&S98916[44], 28}, {NULL}};
const Segment SGI250231[] = {{&S98916[44], 40}, {NULL}};
const Segment SGI250232[] = {{&S98916[30], 6}, {NULL}};
const Segment SGI250233[] = {{&S98916[162], 3}, {NULL}};
const Segment SGI250234[] = {{&S98916[127], 6}, {NULL}};
const Segment SGI250235[] = {{&S105713[59], 10}, {NULL}};
const Segment SGI250236[] = {{&S105713[59], 22}, {NULL}};
const Segment SGI250237[] = {{&S105713[45], 6}, {NULL}};
const Segment SGI250238[] = {{&S105713[146], 3}, {NULL}};
const Segment SGI250239[] = {{&S105713[111], 6}, {NULL}};
const Segment SGI250240[] = {{&S105713[83], 8}, {NULL}};
const Segment SGI250241[] = {{&S105713[0], 14}, {NULL}};
const Segment SGI250242[] = {{&S98932[44], 10}, {NULL}};
const Segment SGI250243[] = {{&S98932[44], 21}, {NULL}};
const Segment SGI250244[] = {{&S98932[30], 6}, {NULL}};
const Segment SGI250245[] = {{&S98932[114], 3}, {NULL}};
const Segment SGI250246[] = {{&S98932[79], 6}, {NULL}};
const Segment SGI250247[] = {{&S98932[139], 13}, {NULL}};
const Segment SGI250248[] = {{&S105724[44], 10}, {NULL}};
const Segment SGI250249[] = {{&S105724[44], 22}, {NULL}};
const Segment SGI250250[] = {{&S105724[30], 6}, {NULL}};
const Segment SGI250251[] = {{&S105724[131], 3}, {NULL}};
const Segment SGI250252[] = {{&S105724[96], 6}, {NULL}};
const Segment SGI250253[] = {{&S105724[159], 3}, {NULL}};
const Segment SGI250254[] = {{&S105724[68], 8}, {NULL}};
const Segment SGI250255[] = {{&S105751[44], 10}, {NULL}};
const Segment SGI250256[] = {{&S105751[44], 22}, {NULL}};
const Segment SGI250257[] = {{&S105751[30], 6}, {NULL}};
const Segment SGI250258[] = {{&S105751[131], 3}, {NULL}};
const Segment SGI250259[] = {{&S105751[96], 6}, {NULL}};
const Segment SGI250260[] = {{&S105751[68], 8}, {NULL}};
const Segment SGI250261[] = {{&S105751[148], 10}, {NULL}};
const Segment SGI250262[] = {{&S105774[45], 10}, {NULL}};
const Segment SGI250263[] = {{&S105774[30], 6}, {NULL}};
const Segment SGI250264[] = {{&S105774[115], 3}, {NULL}};
const Segment SGI250265[] = {{&S105774[81], 5}, {NULL}};
const Segment SGI250266[] = {{&S105774[139], 13}, {NULL}};
const Segment SGI250267[] = {{&S105786[44], 10}, {NULL}};
const Segment SGI250268[] = {{&S105786[44], 22}, {NULL}};
const Segment SGI250269[] = {{&S105786[30], 6}, {NULL}};
const Segment SGI250270[] = {{&S105786[131], 3}, {NULL}};
const Segment SGI250271[] = {{&S105786[96], 6}, {NULL}};
const Segment SGI250272[] = {{&S105786[159], 3}, {NULL}};
const Segment SGI250273[] = {{&S105786[68], 8}, {NULL}};
const Segment SGI250274[] = {{&S105817[59], 10}, {NULL}};
const Segment SGI250275[] = {{&S105817[59], 22}, {NULL}};
const Segment SGI250276[] = {{&S105817[45], 6}, {NULL}};
const Segment SGI250277[] = {{&S98951[44], 10}, {NULL}};
const Segment SGI250278[] = {{&S105817[150], 3}, {NULL}};
const Segment SGI250279[] = {{&S98951[44], 34}, {NULL}};
const Segment SGI250280[] = {{&S98951[30], 6}, {NULL}};
const Segment SGI250281[] = {{&S105817[115], 6}, {NULL}};
const Segment SGI250282[] = {{&S105817[83], 8}, {NULL}};
const Segment SGI250283[] = {{&S105817[11], 3}, {NULL}};
const Segment SGI250284[] = {{&S98951[126], 3}, {NULL}};
const Segment SGI250285[] = {{&S98951[92], 5}, {NULL}};
const Segment SGI250286[] = {{&S98953[44], 10}, {NULL}};
const Segment SGI250287[] = {{&S98953[44], 21}, {NULL}};
const Segment SGI250288[] = {{&S98953[30], 6}, {NULL}};
const Segment SGI250289[] = {{&S98953[114], 3}, {NULL}};
const Segment SGI250290[] = {{&S98953[79], 6}, {NULL}};
const Segment SGI250291[] = {{&S98953[134], 8}, {NULL}};
const Segment SGI250292[] = {{&S98953[143], 13}, {NULL}};
const Segment SGI250293[] = {{&S98959[34], 12}, {NULL}};
const Segment SGI250294[] = {{&S98959[27], 6}, {NULL}};
const Segment SGI250295[] = {{&S98959[98], 9}, {NULL}};
const Segment SGI250296[] = {{&S98959[71], 6}, {NULL}};
const Segment SGI250297[] = {{&S105829[59], 10}, {NULL}};
const Segment SGI250298[] = {{&S105829[59], 22}, {NULL}};
const Segment SGI250299[] = {{&S105829[45], 6}, {NULL}};
const Segment SGI250300[] = {{&S105829[146], 3}, {NULL}};
const Segment SGI250301[] = {{&S105829[111], 6}, {NULL}};
const Segment SGI250302[] = {{&S105829[83], 8}, {NULL}};
const Segment SGI250303[] = {{&S105829[0], 14}, {NULL}};
const Segment SGI250304[] = {{&S98969[44], 7}, {NULL}};
const Segment SGI250305[] = {{&S98969[44], 19}, {NULL}};
const Segment SGI250306[] = {{&S98969[30], 6}, {NULL}};
const Segment SGI250307[] = {{&S98969[112], 3}, {NULL}};
const Segment SGI250308[] = {{&S98969[77], 6}, {NULL}};
const Segment SGI250309[] = {{&S105847[44], 10}, {NULL}};
const Segment SGI250310[] = {{&S105847[44], 22}, {NULL}};
const Segment SGI250311[] = {{&S105847[30], 6}, {NULL}};
const Segment SGI250312[] = {{&S105847[115], 3}, {NULL}};
const Segment SGI250313[] = {{&S105847[80], 6}, {NULL}};
const Segment SGI250314[] = {{&S105847[132], 10}, {NULL}};
const Segment SGI250315[] = {{&S98983[34], 10}, {NULL}};
const Segment SGI250316[] = {{&S98983[27], 6}, {NULL}};
const Segment SGI250317[] = {{&S98983[96], 9}, {NULL}};
const Segment SGI250318[] = {{&S98983[69], 6}, {NULL}};
const Segment SGI250319[] = {{&S105868[34], 10}, {NULL}};
const Segment SGI250320[] = {{&S105868[27], 6}, {NULL}};
const Segment SGI250321[] = {{&S105868[97], 9}, {NULL}};
const Segment SGI250322[] = {{&S105868[70], 6}, {NULL}};
const Segment SGI250323[] = {{&S105868[132], 8}, {NULL}};
const Segment SGI250324[] = {{&S105895[44], 10}, {NULL}};
const Segment SGI250325[] = {{&S105895[44], 22}, {NULL}};
const Segment SGI250326[] = {{&S105895[30], 6}, {NULL}};
const Segment SGI250327[] = {{&S105895[115], 3}, {NULL}};
const Segment SGI250328[] = {{&S105895[80], 6}, {NULL}};
const Segment SGI250329[] = {{&S105895[143], 3}, {NULL}};
const Segment SGI250330[] = {{&S105903[44], 11}, {NULL}};
const Segment SGI250331[] = {{&S105903[44], 23}, {NULL}};
const Segment SGI250332[] = {{&S105903[30], 6}, {NULL}};
const Segment SGI250333[] = {{&S105903[81], 6}, {NULL}};
const Segment SGI250334[] = {{&S105903[108], 13}, {NULL}};
const Segment SGI250335[] = {{&S105905[44], 10}, {NULL}};
const Segment SGI250336[] = {{&S105905[44], 22}, {NULL}};
const Segment SGI250337[] = {{&S105905[30], 6}, {NULL}};
const Segment SGI250338[] = {{&S105905[115], 3}, {NULL}};
const Segment SGI250339[] = {{&S105905[80], 6}, {NULL}};
const Segment SGI250340[] = {{&S105905[150], 3}, {NULL}};
const Segment SGI250341[] = {{&S105909[44], 7}, {NULL}};
const Segment SGI250342[] = {{&S105909[44], 27}, {NULL}};
const Segment SGI250343[] = {{&S105909[30], 6}, {NULL}};
const Segment SGI250344[] = {{&S105909[118], 3}, {NULL}};
const Segment SGI250345[] = {{&S105909[85], 5}, {NULL}};
const Segment SGI250346[] = {{&S105909[122], 14}, {NULL}};
const Segment SGI250347[] = {{&S105910[44], 7}, {NULL}};
const Segment SGI250348[] = {{&S105910[44], 27}, {NULL}};
const Segment SGI250349[] = {{&S105910[30], 6}, {NULL}};
const Segment SGI250350[] = {{&S105910[118], 3}, {NULL}};
const Segment SGI250351[] = {{&S105910[85], 5}, {NULL}};
const Segment SGI250352[] = {{&S105910[122], 14}, {NULL}};
const Segment SGI250353[] = {{&S105914[34], 11}, {NULL}};
const Segment SGI250354[] = {{&S105914[27], 6}, {NULL}};
const Segment SGI250355[] = {{&S105914[98], 9}, {NULL}};
const Segment SGI250356[] = {{&S105914[71], 6}, {NULL}};
const Segment SGI250357[] = {{&S105915[44], 10}, {NULL}};
const Segment SGI250358[] = {{&S105915[44], 22}, {NULL}};
const Segment SGI250359[] = {{&S105915[30], 6}, {NULL}};
const Segment SGI250360[] = {{&S105915[115], 3}, {NULL}};
const Segment SGI250361[] = {{&S105915[80], 6}, {NULL}};
const Segment SGI250362[] = {{&S105968[34], 15}, {NULL}};
const Segment SGI250363[] = {{&S105968[27], 6}, {NULL}};
const Segment SGI250364[] = {{&S105968[105], 9}, {NULL}};
const Segment SGI250365[] = {{&S105968[78], 6}, {NULL}};
const Segment SGI250366[] = {{&S105982[34], 6}, {NULL}};
const Segment SGI250367[] = {{&S105982[27], 6}, {NULL}};
const Segment SGI250368[] = {{&S105982[93], 9}, {NULL}};
const Segment SGI250369[] = {{&S105982[66], 6}, {NULL}};
const Segment SGI250370[] = {{&S105982[127], 8}, {NULL}};
const Segment SGI250371[] = {{&S105985[34], 7}, {NULL}};
const Segment SGI250372[] = {{&S105985[27], 6}, {NULL}};
const Segment SGI250373[] = {{&S105985[98], 9}, {NULL}};
const Segment SGI250374[] = {{&S105985[71], 6}, {NULL}};
const Segment SGI250375[] = {{&S105985[139], 8}, {NULL}};
const Segment SGI250376[] = {{&S105990[34], 10}, {NULL}};
const Segment SGI250377[] = {{&S105990[27], 6}, {NULL}};
const Segment SGI250378[] = {{&S105990[97], 9}, {NULL}};
const Segment SGI250379[] = {{&S105990[70], 6}, {NULL}};
const Segment SGI250380[] = {{&S105990[138], 8}, {NULL}};
const Segment SGI250381[] = {{&S105995[34], 10}, {NULL}};
const Segment SGI250382[] = {{&S105995[27], 6}, {NULL}};
const Segment SGI250383[] = {{&S105995[97], 9}, {NULL}};
const Segment SGI250384[] = {{&S105995[70], 6}, {NULL}};
const Segment SGI250385[] = {{&S105995[138], 8}, {NULL}};
const Segment SGI250386[] = {{&S106002[34], 10}, {NULL}};
const Segment SGI250387[] = {{&S106002[27], 6}, {NULL}};
const Segment SGI250388[] = {{&S106002[97], 9}, {NULL}};
const Segment SGI250389[] = {{&S106002[70], 6}, {NULL}};
const Segment SGI250390[] = {{&S106002[138], 8}, {NULL}};
const Segment SGI250391[] = {{&S99159[34], 10}, {NULL}};
const Segment SGI250392[] = {{&S99159[27], 6}, {NULL}};
const Segment SGI250393[] = {{&S99159[96], 9}, {NULL}};
const Segment SGI250394[] = {{&S99159[69], 6}, {NULL}};
const Segment SGI250395[] = {{&S99245[34], 7}, {NULL}};
const Segment SGI250396[] = {{&S99245[27], 6}, {NULL}};
const Segment SGI250397[] = {{&S99245[94], 9}, {NULL}};
const Segment SGI250398[] = {{&S99245[67], 6}, {NULL}};
const Segment SGI250399[] = {{&S99277[34], 7}, {NULL}};
const Segment SGI250400[] = {{&S99277[27], 6}, {NULL}};
const Segment SGI250401[] = {{&S99277[95], 9}, {NULL}};
const Segment SGI250402[] = {{&S99277[68], 6}, {NULL}};
const Segment SGI250403[] = {{&S99278[41], 18}, {NULL}};
const Segment SGI250404[] = {{&S99278[41], 29}, {NULL}};
const Segment SGI250405[] = {{&S99278[27], 6}, {NULL}};
const Segment SGI250406[] = {{&S99278[119], 3}, {NULL}};
const Segment SGI250407[] = {{&S99278[123], 9}, {NULL}};
const Segment SGI250408[] = {{&S99278[84], 6}, {NULL}};
const Segment SGI250409[] = {{&S99317[34], 10}, {NULL}};
const Segment SGI250410[] = {{&S99317[27], 6}, {NULL}};
const Segment SGI250411[] = {{&S99317[96], 9}, {NULL}};
const Segment SGI250412[] = {{&S99317[69], 6}, {NULL}};
const Segment SGI250413[] = {{&S99341[44], 9}, {NULL}};
const Segment SGI250414[] = {{&S99341[44], 20}, {NULL}};
const Segment SGI250415[] = {{&S99341[30], 6}, {NULL}};
const Segment SGI250416[] = {{&S99341[78], 6}, {NULL}};
const Segment SGI250417[] = {{&S99341[105], 13}, {NULL}};
const Segment SGI250418[] = {{&S99342[34], 10}, {NULL}};
const Segment SGI250419[] = {{&S99342[27], 6}, {NULL}};
const Segment SGI250420[] = {{&S99342[97], 9}, {NULL}};
const Segment SGI250421[] = {{&S99342[70], 6}, {NULL}};
const Segment SGI250422[] = {{&S99342[138], 8}, {NULL}};
const Segment SGI250423[] = {{&S99401[34], 10}, {NULL}};
const Segment SGI250424[] = {{&S99401[27], 6}, {NULL}};
const Segment SGI250425[] = {{&S99401[96], 9}, {NULL}};
const Segment SGI250426[] = {{&S99401[69], 6}, {NULL}};
const Segment SGI250427[] = {{&S99404[34], 6}, {NULL}};
const Segment SGI250428[] = {{&S99404[27], 6}, {NULL}};
const Segment SGI250429[] = {{&S99404[94], 9}, {NULL}};
const Segment SGI250430[] = {{&S99404[67], 6}, {NULL}};
const Segment SGI250431[] = {{&S99418[34], 8}, {NULL}};
const Segment SGI250432[] = {{&S99418[27], 6}, {NULL}};
const Segment SGI250433[] = {{&S99418[94], 9}, {NULL}};
const Segment SGI250434[] = {{&S99418[67], 6}, {NULL}};
const Segment SGI250435[] = {{&S99420[52], 9}, {NULL}};
const Segment SGI250436[] = {{&S99420[52], 20}, {NULL}};
const Segment SGI250437[] = {{&S99420[38], 6}, {NULL}};
const Segment SGI250438[] = {{&S99420[86], 5}, {NULL}};
const Segment SGI250439[] = {{&S99422[34], 10}, {NULL}};
const Segment SGI250440[] = {{&S99422[27], 6}, {NULL}};
const Segment SGI250441[] = {{&S99422[96], 9}, {NULL}};
const Segment SGI250442[] = {{&S99422[69], 6}, {NULL}};
const Segment SGI250443[] = {{&S99423[34], 10}, {NULL}};
const Segment SGI250444[] = {{&S99423[27], 6}, {NULL}};
const Segment SGI250445[] = {{&S99423[96], 9}, {NULL}};
const Segment SGI250446[] = {{&S99423[69], 6}, {NULL}};
const Segment SGI250447[] = {{&S99447[44], 13}, {NULL}};
const Segment SGI250448[] = {{&S99447[44], 24}, {NULL}};
const Segment SGI250449[] = {{&S99447[30], 6}, {NULL}};
const Segment SGI250450[] = {{&S99447[117], 3}, {NULL}};
const Segment SGI250451[] = {{&S99447[82], 6}, {NULL}};
const Segment SGI250452[] = {{&S99464[44], 9}, {NULL}};
const Segment SGI250453[] = {{&S99464[44], 20}, {NULL}};
const Segment SGI250454[] = {{&S99464[30], 6}, {NULL}};
const Segment SGI250455[] = {{&S99464[113], 3}, {NULL}};
const Segment SGI250456[] = {{&S99464[78], 6}, {NULL}};
const Segment SGI250457[] = {{&S99479[44], 10}, {NULL}};
const Segment SGI250458[] = {{&S99479[44], 21}, {NULL}};
const Segment SGI250459[] = {{&S99479[30], 6}, {NULL}};
const Segment SGI250460[] = {{&S99479[79], 6}, {NULL}};
const Segment SGI250461[] = {{&S99479[106], 13}, {NULL}};
const Segment SGI250462[] = {{&S99485[59], 10}, {NULL}};
const Segment SGI250463[] = {{&S99485[59], 34}, {NULL}};
const Segment SGI250464[] = {{&S99485[45], 6}, {NULL}};
const Segment SGI250465[] = {{&S99485[142], 3}, {NULL}};
const Segment SGI250466[] = {{&S99485[107], 6}, {NULL}};
const Segment SGI250467[] = {{&S99486[44], 11}, {NULL}};
const Segment SGI250468[] = {{&S99486[44], 22}, {NULL}};
const Segment SGI250469[] = {{&S99486[30], 6}, {NULL}};
const Segment SGI250470[] = {{&S99486[115], 3}, {NULL}};
const Segment SGI250471[] = {{&S99486[80], 6}, {NULL}};
const Segment SGI250472[] = {{&S99486[140], 7}, {NULL}};
const Segment SGI250473[] = {{&S99495[44], 10}, {NULL}};
const Segment SGI250474[] = {{&S99495[44], 21}, {NULL}};
const Segment SGI250475[] = {{&S99495[30], 6}, {NULL}};
const Segment SGI250476[] = {{&S99495[79], 6}, {NULL}};
const Segment SGI250477[] = {{&S99495[106], 13}, {NULL}};
const Segment SGI250478[] = {{&S99499[34], 10}, {NULL}};
const Segment SGI250479[] = {{&S99499[27], 6}, {NULL}};
const Segment SGI250480[] = {{&S99499[96], 9}, {NULL}};
const Segment SGI250481[] = {{&S99499[69], 6}, {NULL}};
const Segment SGI250482[] = {{&S99518[44], 10}, {NULL}};
const Segment SGI250483[] = {{&S99518[44], 21}, {NULL}};
const Segment SGI250484[] = {{&S99518[30], 6}, {NULL}};
const Segment SGI250485[] = {{&S99518[112], 3}, {NULL}};
const Segment SGI250486[] = {{&S99518[79], 5}, {NULL}};
const Segment SGI250487[] = {{&S11077[13], 4}, {NULL}};
const Segment SGI250488[] = {{&S11077[36], 3}, {NULL}};
const Segment SGI250489[] = {{&S11077[74], 8}, {NULL}};
const Segment SGI250490[] = {{&S11078[13], 4}, {NULL}};
const Segment SGI250491[] = {{&S11078[36], 5}, {NULL}};
const Segment SGI250492[] = {{&S11078[76], 8}, {NULL}};
const Segment SGI250493[] = {{&S11078[112], 4}, {NULL}};
const Segment SGI250494[] = {{&S11079[13], 4}, {NULL}};
const Segment SGI250495[] = {{&S11079[29], 3}, {NULL}};
const Segment SGI250496[] = {{&S11079[67], 8}, {NULL}};
const Segment SGI250497[] = {{&S11080[13], 4}, {NULL}};
const Segment SGI250498[] = {{&S11080[29], 5}, {NULL}};
const Segment SGI250499[] = {{&S11080[69], 8}, {NULL}};
const Segment SGI250500[] = {{&S11080[105], 3}, {NULL}};
const Segment SGI250501[] = {{&S11081[13], 4}, {NULL}};
const Segment SGI250502[] = {{&S11081[29], 5}, {NULL}};
const Segment SGI250503[] = {{&S11081[69], 8}, {NULL}};
const Segment SGI250504[] = {{&S11082[13], 4}, {NULL}};
const Segment SGI250505[] = {{&S11082[29], 5}, {NULL}};
const Segment SGI250506[] = {{&S11082[69], 8}, {NULL}};
const Segment SGI250507[] = {{&S11082[105], 3}, {NULL}};
const Segment SGI250508[] = {{&S11083[13], 4}, {NULL}};
const Segment SGI250509[] = {{&S11083[29], 3}, {NULL}};
const Segment SGI250510[] = {{&S11083[67], 8}, {NULL}};
const Segment SGI250511[] = {{&S11083[103], 5}, {NULL}};
const Segment SGI250512[] = {{&S11084[13], 4}, {NULL}};
const Segment SGI250513[] = {{&S11084[29], 5}, {NULL}};
const Segment SGI250514[] = {{&S11084[69], 8}, {NULL}};
const Segment SGI250515[] = {{&S11085[13], 4}, {NULL}};
const Segment SGI250516[] = {{&S11085[29], 3}, {NULL}};
const Segment SGI250517[] = {{&S11085[67], 8}, {NULL}};
const Segment SGI250518[] = {{&S11085[103], 5}, {NULL}};
const Segment SGI250519[] = {{&S11086[13], 4}, {NULL}};
const Segment SGI250520[] = {{&S11086[29], 5}, {NULL}};
const Segment SGI250521[] = {{&S11086[69], 8}, {NULL}};
const Segment SGI250522[] = {{&S11086[105], 3}, {NULL}};
const Segment SGI250523[] = {{&S11087[13], 4}, {NULL}};
const Segment SGI250524[] = {{&S11087[29], 5}, {NULL}};
const Segment SGI250525[] = {{&S11087[69], 8}, {NULL}};
const Segment SGI250526[] = {{&S11087[105], 3}, {NULL}};
const Segment SGI250527[] = {{&S11088[13], 4}, {NULL}};
const Segment SGI250528[] = {{&S11088[29], 5}, {NULL}};
const Segment SGI250529[] = {{&S11088[69], 8}, {NULL}};
const Segment SGI250530[] = {{&S11089[13], 4}, {NULL}};
const Segment SGI250531[] = {{&S11089[29], 5}, {NULL}};
const Segment SGI250532[] = {{&S11089[69], 8}, {NULL}};
const Segment SGI250533[] = {{&S11089[105], 3}, {NULL}};
const Segment SGI250534[] = {{&S11090[13], 4}, {NULL}};
const Segment SGI250535[] = {{&S11090[29], 5}, {NULL}};
const Segment SGI250536[] = {{&S11090[69], 8}, {NULL}};
const Segment SGI250537[] = {{&S11090[105], 3}, {NULL}};
const Segment SGI250538[] = {{&S11091[13], 4}, {NULL}};
const Segment SGI250539[] = {{&S11091[29], 5}, {NULL}};
const Segment SGI250540[] = {{&S11091[69], 8}, {NULL}};
const Segment SGI250541[] = {{&S11091[105], 3}, {NULL}};
const Segment SGI250542[] = {{&S11091[116], 6}, {NULL}};
const Segment SGI250543[] = {{&S11092[13], 4}, {NULL}};
const Segment SGI250544[] = {{&S11092[29], 5}, {NULL}};
const Segment SGI250545[] = {{&S11092[69], 8}, {NULL}};
const Segment SGI250546[] = {{&S11093[13], 4}, {NULL}};
const Segment SGI250547[] = {{&S11093[29], 5}, {NULL}};
const Segment SGI250548[] = {{&S11093[69], 8}, {NULL}};
const Segment SGI250549[] = {{&S11093[105], 3}, {NULL}};
const Segment SGI250550[] = {{&S11094[13], 4}, {NULL}};
const Segment SGI250551[] = {{&S11094[29], 5}, {NULL}};
const Segment SGI250552[] = {{&S11094[69], 8}, {NULL}};
const Segment SGI250553[] = {{&S11094[105], 3}, {NULL}};
const Segment SGI250554[] = {{&S11095[13], 4}, {NULL}};
const Segment SGI250555[] = {{&S11095[29], 5}, {NULL}};
const Segment SGI250556[] = {{&S11095[69], 8}, {NULL}};
const Segment SGI250557[] = {{&S11096[13], 4}, {NULL}};
const Segment SGI250558[] = {{&S11096[29], 5}, {NULL}};
const Segment SGI250559[] = {{&S11096[69], 8}, {NULL}};
const Segment SGI250560[] = {{&S11096[105], 3}, {NULL}};
const Segment SGI250561[] = {{&S11097[27], 4}, {NULL}};
const Segment SGI250562[] = {{&S11097[43], 5}, {NULL}};
const Segment SGI250563[] = {{&S11097[83], 8}, {NULL}};
const Segment SGI250564[] = {{&S11097[119], 3}, {NULL}};
const Segment SGI250565[] = {{&S11097[130], 9}, {NULL}};
const Segment SGI250566[] = {{&S11097[0], 14}, {NULL}};
const Segment SGI250567[] = {{&S11098[27], 4}, {NULL}};
const Segment SGI250568[] = {{&S11098[43], 5}, {NULL}};
const Segment SGI250569[] = {{&S11098[83], 8}, {NULL}};
const Segment SGI250570[] = {{&S11098[119], 3}, {NULL}};
const Segment SGI250571[] = {{&S11098[130], 9}, {NULL}};
const Segment SGI250572[] = {{&S11098[0], 14}, {NULL}};
const Segment SGI250573[] = {{&S11099[30], 4}, {NULL}};
const Segment SGI250574[] = {{&S11099[46], 5}, {NULL}};
const Segment SGI250575[] = {{&S11099[86], 8}, {NULL}};
const Segment SGI250576[] = {{&S11099[122], 3}, {NULL}};
const Segment SGI250577[] = {{&S11099[133], 9}, {NULL}};
const Segment SGI250578[] = {{&S11099[0], 14}, {NULL}};
const Segment SGI250579[] = {{&S11100[13], 4}, {NULL}};
const Segment SGI250580[] = {{&S11100[29], 3}, {NULL}};
const Segment SGI250581[] = {{&S11100[67], 8}, {NULL}};
const Segment SGI250582[] = {{&S11100[103], 5}, {NULL}};
const Segment SGI250583[] = {{&S11100[116], 6}, {NULL}};
const Segment SGI250584[] = {{&S11101[13], 4}, {NULL}};
const Segment SGI250585[] = {{&S11101[29], 3}, {NULL}};
const Segment SGI250586[] = {{&S11101[67], 8}, {NULL}};
const Segment SGI250587[] = {{&S11101[103], 5}, {NULL}};
const Segment SGI250588[] = {{&S99708[44], 10}, {NULL}};
const Segment SGI250589[] = {{&S99708[44], 21}, {NULL}};
const Segment SGI250590[] = {{&S99708[30], 6}, {NULL}};
const Segment SGI250591[] = {{&S99708[114], 3}, {NULL}};
const Segment SGI250592[] = {{&S11102[13], 4}, {NULL}};
const Segment SGI250593[] = {{&S11102[29], 3}, {NULL}};
const Segment SGI250594[] = {{&S99708[79], 6}, {NULL}};
const Segment SGI250595[] = {{&S11102[67], 8}, {NULL}};
const Segment SGI250596[] = {{&S11102[103], 5}, {NULL}};
const Segment SGI250597[] = {{&S11103[13], 4}, {NULL}};
const Segment SGI250598[] = {{&S99709[38], 10}, {NULL}};
const Segment SGI250599[] = {{&S11103[29], 5}, {NULL}};
const Segment SGI250600[] = {{&S99709[31], 6}, {NULL}};
const Segment SGI250601[] = {{&S11103[69], 8}, {NULL}};
const Segment SGI250602[] = {{&S11103[105], 3}, {NULL}};
const Segment SGI250603[] = {{&S11104[13], 4}, {NULL}};
const Segment SGI250604[] = {{&S11104[29], 5}, {NULL}};
const Segment SGI250605[] = {{&S11104[69], 8}, {NULL}};
const Segment SGI250606[] = {{&S11104[105], 3}, {NULL}};
const Segment SGI250607[] = {{&S11105[13], 4}, {NULL}};
const Segment SGI250608[] = {{&S11105[29], 5}, {NULL}};
const Segment SGI250609[] = {{&S11105[69], 8}, {NULL}};
const Segment SGI250610[] = {{&S11105[105], 3}, {NULL}};
const Segment SGI250611[] = {{&S11106[13], 4}, {NULL}};
const Segment SGI250612[] = {{&S11106[29], 5}, {NULL}};
const Segment SGI250613[] = {{&S11106[69], 8}, {NULL}};
const Segment SGI250614[] = {{&S11106[105], 3}, {NULL}};
const Segment SGI250615[] = {{&S11106[116], 6}, {NULL}};
const Segment SGI250616[] = {{&S11107[13], 4}, {NULL}};
const Segment SGI250617[] = {{&S11107[29], 5}, {NULL}};
const Segment SGI250618[] = {{&S11107[69], 8}, {NULL}};
const Segment SGI250619[] = {{&S11107[105], 3}, {NULL}};
const Segment SGI250620[] = {{&S11107[116], 6}, {NULL}};
const Segment SGI250621[] = {{&S11108[13], 4}, {NULL}};
const Segment SGI250622[] = {{&S11108[29], 5}, {NULL}};
const Segment SGI250623[] = {{&S11108[69], 8}, {NULL}};
const Segment SGI250624[] = {{&S11108[105], 3}, {NULL}};
const Segment SGI250625[] = {{&S11109[13], 4}, {&S11109[13], 4}, {&S11109[13], 4}, {NULL}};
const Segment SGI250626[] = {{&S11109[36], 5}, {NULL}};
const Segment SGI250627[] = {{&S11110[13], 4}, {&S11110[13], 4}, {&S11110[13], 4}, {NULL}};
const Segment SGI250628[] = {{&S11110[36], 5}, {NULL}};
const Segment SGI250629[] = {{&S11111[13], 4}, {&S11111[13], 4}, {&S11111[13], 4}, {NULL}};
const Segment SGI250630[] = {{&S11111[36], 5}, {NULL}};
const Segment SGI250631[] = {{&S11112[13], 4}, {&S11112[13], 4}, {&S11112[13], 4}, {NULL}};
const Segment SGI250632[] = {{&S11112[36], 3}, {NULL}};
const Segment SGI250633[] = {{&S11113[13], 4}, {&S11113[13], 4}, {&S11113[13], 4}, {NULL}};
const Segment SGI250634[] = {{&S11113[36], 5}, {NULL}};
const Segment SGI250635[] = {{&S11114[13], 4}, {NULL}};
const Segment SGI250636[] = {{&S11114[29], 5}, {NULL}};
const Segment SGI250637[] = {{&S11114[69], 8}, {NULL}};
const Segment SGI250638[] = {{&S11114[105], 2}, {NULL}};
const Segment SGI250639[] = {{&S11115[13], 4}, {NULL}};
const Segment SGI250640[] = {{&S11115[29], 5}, {NULL}};
const Segment SGI250641[] = {{&S11115[69], 8}, {NULL}};
const Segment SGI250642[] = {{&S11115[105], 2}, {NULL}};
const Segment SGI250643[] = {{&S99740[34], 12}, {NULL}};
const Segment SGI250644[] = {{&S99740[27], 6}, {NULL}};
const Segment SGI250645[] = {{&S99740[98], 9}, {NULL}};
const Segment SGI250646[] = {{&S99740[71], 6}, {NULL}};
const Segment SGI250647[] = {{&S11116[13], 4}, {NULL}};
const Segment SGI250648[] = {{&S11116[29], 5}, {NULL}};
const Segment SGI250649[] = {{&S11116[69], 8}, {NULL}};
const Segment SGI250650[] = {{&S11116[105], 2}, {NULL}};
const Segment SGI250651[] = {{&S11117[13], 4}, {NULL}};
const Segment SGI250652[] = {{&S11117[29], 5}, {NULL}};
const Segment SGI250653[] = {{&S11117[69], 8}, {NULL}};
const Segment SGI250654[] = {{&S11117[105], 3}, {NULL}};
const Segment SGI250655[] = {{&S11118[13], 4}, {&S11118[13], 4}, {&S11118[13], 4}, {NULL}};
const Segment SGI250656[] = {{&S11118[36], 5}, {NULL}};
const Segment SGI250657[] = {{&S11119[13], 4}, {NULL}};
const Segment SGI250658[] = {{&S11119[29], 3}, {NULL}};
const Segment SGI250659[] = {{&S11119[67], 8}, {NULL}};
const Segment SGI250660[] = {{&S11119[103], 5}, {NULL}};
const Segment SGI250661[] = {{&S11120[30], 4}, {NULL}};
const Segment SGI250662[] = {{&S11120[46], 5}, {NULL}};
const Segment SGI250663[] = {{&S11120[86], 8}, {NULL}};
const Segment SGI250664[] = {{&S11120[122], 3}, {NULL}};
const Segment SGI250665[] = {{&S11120[133], 9}, {NULL}};
const Segment SGI250666[] = {{&S11120[0], 14}, {NULL}};
const Segment SGI250667[] = {{&S11121[27], 4}, {NULL}};
const Segment SGI250668[] = {{&S11121[43], 5}, {NULL}};
const Segment SGI250669[] = {{&S11121[83], 8}, {NULL}};
const Segment SGI250670[] = {{&S11121[119], 3}, {NULL}};
const Segment SGI250671[] = {{&S11121[130], 9}, {NULL}};
const Segment SGI250672[] = {{&S11121[0], 14}, {NULL}};
const Segment SGI250673[] = {{&S11122[13], 4}, {NULL}};
const Segment SGI250674[] = {{&S11122[29], 5}, {NULL}};
const Segment SGI250675[] = {{&S11122[69], 8}, {NULL}};
const Segment SGI250676[] = {{&S11122[105], 3}, {NULL}};
const Segment SGI250677[] = {{&S11122[116], 6}, {NULL}};
const Segment SGI250678[] = {{&S11123[13], 4}, {&S11123[13], 4}, {NULL}};
const Segment SGI250679[] = {{&S11123[29], 5}, {NULL}};
const Segment SGI250680[] = {{&S11123[69], 8}, {NULL}};
const Segment SGI250681[] = {{&S11123[105], 3}, {NULL}};
const Segment SGI250682[] = {{&S11124[13], 4}, {&S11124[13], 4}, {NULL}};
const Segment SGI250683[] = {{&S11124[29], 5}, {NULL}};
const Segment SGI250684[] = {{&S11124[69], 8}, {NULL}};
const Segment SGI250685[] = {{&S11124[105], 3}, {NULL}};
const Segment SGI250686[] = {{&S11125[13], 4}, {NULL}};
const Segment SGI250687[] = {{&S11125[29], 5}, {NULL}};
const Segment SGI250688[] = {{&S11125[69], 8}, {NULL}};
const Segment SGI250689[] = {{&S11125[105], 3}, {NULL}};
const Segment SGI250690[] = {{&S11125[116], 6}, {NULL}};
const Segment SGI250691[] = {{&S11126[27], 4}, {NULL}};
const Segment SGI250692[] = {{&S11126[43], 5}, {NULL}};
const Segment SGI250693[] = {{&S11126[83], 8}, {NULL}};
const Segment SGI250694[] = {{&S11126[119], 3}, {NULL}};
const Segment SGI250695[] = {{&S11126[130], 9}, {NULL}};
const Segment SGI250696[] = {{&S11126[0], 14}, {NULL}};
const Segment SGI250697[] = {{&S11127[13], 4}, {&S11127[13], 4}, {NULL}};
const Segment SGI250698[] = {{&S11127[29], 5}, {NULL}};
const Segment SGI250699[] = {{&S11127[69], 8}, {NULL}};
const Segment SGI250700[] = {{&S11127[105], 3}, {NULL}};
const Segment SGI250701[] = {{&S11128[13], 4}, {NULL}};
const Segment SGI250702[] = {{&S11128[29], 5}, {NULL}};
const Segment SGI250703[] = {{&S11128[69], 8}, {NULL}};
const Segment SGI250704[] = {{&S11128[105], 5}, {NULL}};
const Segment SGI250705[] = {{&S11129[13], 4}, {&S11129[13], 4}, {NULL}};
const Segment SGI250706[] = {{&S11129[29], 3}, {NULL}};
const Segment SGI250707[] = {{&S11129[67], 8}, {NULL}};
const Segment SGI250708[] = {{&S11129[103], 5}, {NULL}};
const Segment SGI250709[] = {{&S11130[13], 4}, {&S11130[13], 4}, {&S11130[13], 4}, {NULL}};
const Segment SGI250710[] = {{&S11130[29], 5}, {NULL}};
const Segment SGI250711[] = {{&S11130[69], 8}, {NULL}};
const Segment SGI250712[] = {{&S11130[105], 3}, {NULL}};
const Segment SGI250713[] = {{&S11131[13], 4}, {&S11131[13], 4}, {&S11131[13], 4}, {NULL}};
const Segment SGI250714[] = {{&S11131[29], 3}, {NULL}};
const Segment SGI250715[] = {{&S11131[67], 8}, {NULL}};
const Segment SGI250716[] = {{&S11131[103], 5}, {NULL}};
const Segment SGI250717[] = {{&S11132[13], 4}, {&S11132[13], 4}, {&S11132[13], 4}, {NULL}};
const Segment SGI250718[] = {{&S11132[29], 5}, {NULL}};
const Segment SGI250719[] = {{&S11132[69], 8}, {NULL}};
const Segment SGI250720[] = {{&S11132[105], 3}, {NULL}};
const Segment SGI250721[] = {{&S11133[13], 4}, {NULL}};
const Segment SGI250722[] = {{&S11133[29], 5}, {NULL}};
const Segment SGI250723[] = {{&S11133[69], 8}, {NULL}};
const Segment SGI250724[] = {{&S11133[105], 3}, {NULL}};
const Segment SGI250725[] = {{&S11133[116], 6}, {NULL}};
const Segment SGI250726[] = {{&S11134[13], 4}, {NULL}};
const Segment SGI250727[] = {{&S11134[26], 3}, {NULL}};
const Segment SGI250728[] = {{&S11134[57], 6}, {NULL}};
const Segment SGI250729[] = {{&S11134[91], 3}, {NULL}};
const Segment SGI250730[] = {{&S11134[102], 7}, {NULL}};
const Segment SGI250731[] = {{&S11135[13], 4}, {&S11135[13], 4}, {&S11135[13], 4}, {NULL}};
const Segment SGI250732[] = {{&S11135[29], 5}, {NULL}};
const Segment SGI250733[] = {{&S11135[69], 8}, {NULL}};
const Segment SGI250734[] = {{&S11135[105], 3}, {NULL}};
const Segment SGI250735[] = {{&S11136[13], 4}, {&S11136[13], 4}, {&S11136[13], 4}, {NULL}};
const Segment SGI250736[] = {{&S11136[29], 5}, {NULL}};
const Segment SGI250737[] = {{&S11136[69], 8}, {NULL}};
const Segment SGI250738[] = {{&S11136[105], 3}, {NULL}};
const Segment SGI250739[] = {{&S11137[13], 4}, {&S11137[13], 4}, {&S11137[13], 4}, {NULL}};
const Segment SGI250740[] = {{&S11137[29], 5}, {NULL}};
const Segment SGI250741[] = {{&S11137[69], 8}, {NULL}};
const Segment SGI250742[] = {{&S11137[105], 3}, {NULL}};
const Segment SGI250743[] = {{&S99834[34], 11}, {NULL}};
const Segment SGI250744[] = {{&S99834[27], 6}, {NULL}};
const Segment SGI250745[] = {{&S99834[98], 9}, {NULL}};
const Segment SGI250746[] = {{&S99834[71], 6}, {NULL}};
const Segment SGI250747[] = {{&S11138[13], 4}, {&S11138[13], 4}, {&S11138[13], 4}, {NULL}};
const Segment SGI250748[] = {{&S11138[29], 5}, {NULL}};
const Segment SGI250749[] = {{&S11138[69], 8}, {NULL}};
const Segment SGI250750[] = {{&S11138[105], 3}, {NULL}};
const Segment SGI250751[] = {{&S11139[13], 4}, {&S11139[13], 4}, {&S11139[13], 4}, {NULL}};
const Segment SGI250752[] = {{&S11139[29], 3}, {NULL}};
const Segment SGI250753[] = {{&S11139[67], 8}, {NULL}};
const Segment SGI250754[] = {{&S11139[103], 5}, {NULL}};
const Segment SGI250755[] = {{&S11140[13], 4}, {&S11140[13], 4}, {&S11140[13], 4}, {NULL}};
const Segment SGI250756[] = {{&S11140[29], 5}, {NULL}};
const Segment SGI250757[] = {{&S11140[69], 8}, {NULL}};
const Segment SGI250758[] = {{&S11140[105], 3}, {NULL}};
const Segment SGI250759[] = {{&S11141[13], 4}, {NULL}};
const Segment SGI250760[] = {{&S11141[26], 5}, {NULL}};
const Segment SGI250761[] = {{&S11141[59], 6}, {NULL}};
const Segment SGI250762[] = {{&S11141[93], 5}, {NULL}};
const Segment SGI250763[] = {{&S11142[13], 4}, {NULL}};
const Segment SGI250764[] = {{&S11142[26], 5}, {NULL}};
const Segment SGI250765[] = {{&S11142[59], 6}, {NULL}};
const Segment SGI250766[] = {{&S11142[93], 5}, {NULL}};
const Segment SGI250767[] = {{&S11143[13], 4}, {NULL}};
const Segment SGI250768[] = {{&S11143[29], 3}, {NULL}};
const Segment SGI250769[] = {{&S11143[67], 8}, {NULL}};
const Segment SGI250770[] = {{&S11143[102], 3}, {NULL}};
const Segment SGI250771[] = {{&S11144[13], 4}, {NULL}};
const Segment SGI250772[] = {{&S11144[29], 3}, {NULL}};
const Segment SGI250773[] = {{&S11144[67], 8}, {NULL}};
const Segment SGI250774[] = {{&S11144[102], 3}, {NULL}};
const Segment SGI250775[] = {{&S99897[44], 12}, {NULL}};
const Segment SGI250776[] = {{&S99897[44], 23}, {NULL}};
const Segment SGI250777[] = {{&S99897[30], 6}, {NULL}};
const Segment SGI250778[] = {{&S99897[81], 6}, {NULL}};
const Segment SGI250779[] = {{&S99897[108], 13}, {NULL}};
const Segment SGI250780[] = {{&S99918[44], 10}, {NULL}};
const Segment SGI250781[] = {{&S99918[44], 21}, {NULL}};
const Segment SGI250782[] = {{&S99918[30], 6}, {NULL}};
const Segment SGI250783[] = {{&S99918[79], 6}, {NULL}};
const Segment SGI250784[] = {{&S99918[106], 13}, {NULL}};
const Segment SGI250785[] = {{&S99933[34], 9}, {NULL}};
const Segment SGI250786[] = {{&S99933[27], 6}, {NULL}};
const Segment SGI250787[] = {{&S99933[96], 9}, {NULL}};
const Segment SGI250788[] = {{&S99933[69], 6}, {NULL}};
const Segment SGI250789[] = {{&S99955[34], 12}, {NULL}};
const Segment SGI250790[] = {{&S99955[27], 6}, {NULL}};
const Segment SGI250791[] = {{&S99955[99], 9}, {NULL}};
const Segment SGI250792[] = {{&S99955[72], 6}, {NULL}};
const Segment SGI250793[] = {{&S99968[44], 10}, {NULL}};
const Segment SGI250794[] = {{&S99968[44], 21}, {NULL}};
const Segment SGI250795[] = {{&S99968[30], 6}, {NULL}};
const Segment SGI250796[] = {{&S99968[114], 3}, {NULL}};
const Segment SGI250797[] = {{&S99968[79], 6}, {NULL}};
const Segment SGI250798[] = {{&S99968[139], 11}, {NULL}};
const Segment SGI250799[] = {{&S99974[34], 7}, {NULL}};
const Segment SGI250800[] = {{&S99974[27], 6}, {NULL}};
const Segment SGI250801[] = {{&S99974[93], 9}, {NULL}};
const Segment SGI250802[] = {{&S99974[66], 6}, {NULL}};
const Segment SGI250803[] = {{&S99981[34], 10}, {NULL}};
const Segment SGI250804[] = {{&S99981[27], 6}, {NULL}};
const Segment SGI250805[] = {{&S99981[96], 9}, {NULL}};
const Segment SGI250806[] = {{&S99981[69], 6}, {NULL}};
const Segment SGI250807[] = {{&S99981[137], 8}, {NULL}};
const Segment SGI250808[] = {{&S100006[44], 14}, {NULL}};
const Segment SGI250809[] = {{&S100006[44], 26}, {NULL}};
const Segment SGI250810[] = {{&S100006[30], 6}, {NULL}};
const Segment SGI250811[] = {{&S100006[139], 3}, {NULL}};
const Segment SGI250812[] = {{&S100006[104], 6}, {NULL}};
const Segment SGI250813[] = {{&S100006[72], 8}, {NULL}};
const Segment SGI250814[] = {{&S1141[0], 7}, {NULL}};
const Segment SGI250815[] = {{&S1141[0], 23}, {NULL}};
const Segment SGI250816[] = {{&S1142[0], 10}, {&S1142[11], 35}, {NULL}};
const Segment SGI250817[] = {{&S1142[0], 46}, {NULL}};
const Segment SGI250818[] = {{&S1143[0], 11}, {&S1143[12], 35}, {NULL}};
const Segment SGI250819[] = {{&S1143[0], 47}, {NULL}};
const Segment SGI250820[] = {{&S1144[0], 53}, {NULL}};
const Segment SGI250821[] = {{&S1145[0], 17}, {NULL}};
const Segment SGI250822[] = {{&S1146[0], 3}, {NULL}};
const Segment SGI250823[] = {{&S1147[0], 13}, {&S1147[14], 36}, {NULL}};
const Segment SGI250824[] = {{&S1147[0], 50}, {NULL}};
const Segment SGI250825[] = {{&S1148[47], 35}, {NULL}};
const Segment SGI250826[] = {{&S1148[0], 82}, {NULL}};
const Segment SGI250827[] = {{&S1149[0], 42}, {NULL}};
const Segment SGI250828[] = {{&S1150[0], 8}, {NULL}};
const Segment SGI250829[] = {{&S1151[0], 15}, {&S1151[16], 48}, {NULL}};
const Segment SGI250830[] = {{&S1151[0], 64}, {NULL}};
const Segment SGI250831[] = {{&S1152[14], 7}, {NULL}};
const Segment SGI250832[] = {{&S1152[0], 90}, {NULL}};
const Segment SGI250833[] = {{&S1153[57], 7}, {NULL}};
const Segment SGI250834[] = {{&S1153[0], 75}, {NULL}};
const Segment SGI250835[] = {{&S1154[10], 6}, {NULL}};
const Segment SGI250836[] = {{&S1154[0], 71}, {NULL}};
const Segment SGI250837[] = {{&S1155[53], 34}, {NULL}};
const Segment SGI250838[] = {{&S1155[0], 87}, {NULL}};
const Segment SGI250839[] = {{&S1156[6], 3}, {NULL}};
const Segment SGI250840[] = {{&S1156[0], 13}, {NULL}};
const Segment SGI250841[] = {{&S1157[0], 22}, {&S1157[23], 35}, {NULL}};
const Segment SGI250842[] = {{&S1157[0], 58}, {NULL}};
const Segment SGI250843[] = {{&S1158[27], 5}, {NULL}};
const Segment SGI250844[] = {{&S1158[0], 32}, {NULL}};
const Segment SGI250845[] = {{&S1159[0], 9}, {NULL}};
const Segment SGI250846[] = {{&S1160[25], 13}, {NULL}};
const Segment SGI250847[] = {{&S1160[0], 77}, {NULL}};
const Segment SGI250848[] = {{&S1161[16], 7}, {NULL}};
const Segment SGI250849[] = {{&S1161[0], 40}, {NULL}};
const Segment SGI250850[] = {{&S1162[28], 6}, {NULL}};
const Segment SGI250851[] = {{&S1162[0], 84}, {NULL}};
const Segment SGI250852[] = {{&S1163[0], 59}, {NULL}};
const Segment SGI250853[] = {{&S1164[52], 6}, {NULL}};
const Segment SGI250854[] = {{&S1164[0], 58}, {NULL}};
const Segment SGI250855[] = {{&S1165[25], 11}, {&S1165[37], 32}, {NULL}};
const Segment SGI250856[] = {{&S1165[0], 69}, {NULL}};
const Segment SGI250857[] = {{&S1166[0], 5}, {&S1166[6], 28}, {NULL}};
const Segment SGI250858[] = {{&S1166[0], 34}, {NULL}};
const Segment SGI250859[] = {{&S1167[25], 8}, {NULL}};
const Segment SGI250860[] = {{&S1167[0], 81}, {NULL}};
const Segment SGI250861[] = {{&S1168[25], 15}, {&S1168[41], 38}, {NULL}};
const Segment SGI250862[] = {{&S1168[0], 79}, {NULL}};
const Segment SGI250863[] = {{&S1169[25], 14}, {&S1169[40], 45}, {NULL}};
const Segment SGI250864[] = {{&S1169[0], 87}, {NULL}};
const Segment SGI250865[] = {{&S1170[0], 16}, {&S1170[42], 57}, {NULL}};
const Segment SGI250866[] = {{&S1170[0], 99}, {NULL}};
const Segment SGI250867[] = {{&S1171[0], 16}, {&S1171[17], 35}, {NULL}};
const Segment SGI250868[] = {{&S1171[0], 52}, {NULL}};
const Segment SGI250869[] = {{&S1172[0], 39}, {NULL}};
const Segment SGI250870[] = {{&S100073[34], 10}, {NULL}};
const Segment SGI250871[] = {{&S100073[27], 6}, {NULL}};
const Segment SGI250872[] = {{&S100073[97], 9}, {NULL}};
const Segment SGI250873[] = {{&S100073[70], 6}, {NULL}};
const Segment SGI250874[] = {{&S1173[0], 48}, {NULL}};
const Segment SGI250875[] = {{&S1174[25], 18}, {&S1174[44], 33}, {NULL}};
const Segment SGI250876[] = {{&S1174[0], 77}, {NULL}};
const Segment SGI250877[] = {{&S1175[25], 8}, {NULL}};
const Segment SGI250878[] = {{&S1175[0], 52}, {NULL}};
const Segment SGI250879[] = {{&S1176[0], 50}, {NULL}};
const Segment SGI250880[] = {{&S1177[25], 12}, {NULL}};
const Segment SGI250881[] = {{&S1177[0], 68}, {NULL}};
const Segment SGI250882[] = {{&S1178[25], 13}, {NULL}};
const Segment SGI250883[] = {{&S1178[0], 62}, {NULL}};
const Segment SGI250884[] = {{&S1179[15], 39}, {NULL}};
const Segment SGI250885[] = {{&S1179[0], 54}, {NULL}};
const Segment SGI250886[] = {{&S1180[35], 10}, {NULL}};
const Segment SGI250887[] = {{&S1180[0], 45}, {NULL}};
const Segment SGI250888[] = {{&S1181[25], 6}, {&S1181[37], 31}, {NULL}};
const Segment SGI250889[] = {{&S1181[0], 68}, {NULL}};
const Segment SGI250890[] = {{&S1184[0], 26}, {NULL}};
const Segment SGI250891[] = {{&S1185[25], 13}, {&S1185[39], 42}, {NULL}};
const Segment SGI250892[] = {{&S1185[0], 81}, {NULL}};
const Segment SGI250893[] = {{&S1186[0], 6}, {NULL}};
const Segment SGI250894[] = {{&S1187[25], 19}, {NULL}};
const Segment SGI250895[] = {{&S1187[0], 68}, {NULL}};
const Segment SGI250896[] = {{&S1188[25], 18}, {NULL}};
const Segment SGI250897[] = {{&S1188[0], 71}, {NULL}};
const Segment SGI250898[] = {{&S1189[25], 17}, {NULL}};
const Segment SGI250899[] = {{&S1189[0], 67}, {NULL}};
const Segment SGI250900[] = {{&S1190[25], 13}, {&S1190[39], 44}, {NULL}};
const Segment SGI250901[] = {{&S1190[0], 85}, {NULL}};
const Segment SGI250902[] = {{&S1191[0], 12}, {&S1191[13], 47}, {NULL}};
const Segment SGI250903[] = {{&S1191[0], 60}, {NULL}};
const Segment SGI250904[] = {{&S1192[0], 15}, {NULL}};
const Segment SGI250905[] = {{&S1192[0], 80}, {NULL}};
const Segment SGI250906[] = {{&S1193[0], 9}, {&S1193[10], 36}, {NULL}};
const Segment SGI250907[] = {{&S1193[0], 46}, {NULL}};
const Segment SGI250908[] = {{&S1194[40], 3}, {NULL}};
const Segment SGI250909[] = {{&S1194[0], 49}, {NULL}};
const Segment SGI250910[] = {{&S1195[7], 7}, {NULL}};
const Segment SGI250911[] = {{&S1195[0], 18}, {NULL}};
const Segment SGI250912[] = {{&S1196[39], 7}, {NULL}};
const Segment SGI250913[] = {{&S1196[0], 56}, {NULL}};
const Segment SGI250914[] = {{&S100103[34], 10}, {NULL}};
const Segment SGI250915[] = {{&S100103[27], 6}, {NULL}};
const Segment SGI250916[] = {{&S1197[0], 61}, {NULL}};
const Segment SGI250917[] = {{&S100103[96], 9}, {NULL}};
const Segment SGI250918[] = {{&S100103[69], 6}, {NULL}};
const Segment SGI250919[] = {{&S100103[130], 8}, {NULL}};
const Segment SGI250920[] = {{&S1198[25], 14}, {&S1198[40], 24}, {NULL}};
const Segment SGI250921[] = {{&S1198[0], 64}, {NULL}};
const Segment SGI250922[] = {{&S1199[43], 24}, {NULL}};
const Segment SGI250923[] = {{&S1199[0], 67}, {NULL}};
const Segment SGI250924[] = {{&S1200[0], 7}, {&S1200[10], 37}, {NULL}};
const Segment SGI250925[] = {{&S1200[0], 47}, {NULL}};
const Segment SGI250926[] = {{&S1201[33], 7}, {&S1201[46], 37}, {&S1201[33], 7}, {NULL}};
const Segment SGI250927[] = {{&S1201[0], 104}, {NULL}};
const Segment SGI250928[] = {{&S1202[12], 77}, {NULL}};
const Segment SGI250929[] = {{&S1202[0], 89}, {NULL}};
const Segment SGI250930[] = {{&S1203[25], 11}, {&S1203[37], 57}, {NULL}};
const Segment SGI250931[] = {{&S1203[0], 94}, {NULL}};
const Segment SGI250932[] = {{&S1204[0], 9}, {NULL}};
const Segment SGI250933[] = {{&S1205[0], 14}, {&S1205[15], 38}, {NULL}};
const Segment SGI250934[] = {{&S1205[0], 53}, {NULL}};
const Segment SGI250935[] = {{&S1206[0], 34}, {NULL}};
const Segment SGI250936[] = {{&S1207[22], 36}, {NULL}};
const Segment SGI250937[] = {{&S1207[0], 58}, {NULL}};
const Segment SGI250938[] = {{&S1208[0], 12}, {NULL}};
const Segment SGI250939[] = {{&S1209[25], 13}, {&S1209[39], 31}, {NULL}};
const Segment SGI250940[] = {{&S1209[0], 70}, {NULL}};
const Segment SGI250941[] = {{&S1210[38], 32}, {NULL}};
const Segment SGI250942[] = {{&S1210[0], 70}, {NULL}};
const Segment SGI250943[] = {{&S1211[0], 10}, {&S1211[11], 32}, {NULL}};
const Segment SGI250944[] = {{&S1211[0], 43}, {NULL}};
const Segment SGI250945[] = {{&S1212[0], 7}, {NULL}};
const Segment SGI250946[] = {{&S1213[8], 7}, {NULL}};
const Segment SGI250947[] = {{&S1213[0], 43}, {NULL}};
const Segment SGI250948[] = {{&S1214[12], 5}, {NULL}};
const Segment SGI250949[] = {{&S1214[0], 46}, {NULL}};
const Segment SGI250950[] = {{&S1215[25], 17}, {&S1215[43], 38}, {NULL}};
const Segment SGI250951[] = {{&S1215[0], 81}, {NULL}};
const Segment SGI250952[] = {{&S1216[0], 11}, {&S1216[12], 38}, {NULL}};
const Segment SGI250953[] = {{&S1216[0], 50}, {NULL}};
const Segment SGI250954[] = {{&S1217[7], 3}, {NULL}};
const Segment SGI250955[] = {{&S1217[0], 10}, {NULL}};
const Segment SGI250956[] = {{&S1218[0], 8}, {NULL}};
const Segment SGI250957[] = {{&S1219[0], 9}, {NULL}};
const Segment SGI250958[] = {{&S1220[35], 41}, {NULL}};
const Segment SGI250959[] = {{&S1220[0], 76}, {NULL}};
const Segment SGI250960[] = {{&S1221[41], 10}, {NULL}};
const Segment SGI250961[] = {{&S1221[0], 103}, {NULL}};
const Segment SGI250962[] = {{&S1222[0], 13}, {&S1222[14], 24}, {NULL}};
const Segment SGI250963[] = {{&S1222[0], 38}, {NULL}};
const Segment SGI250964[] = {{&S1223[0], 41}, {NULL}};
const Segment SGI250965[] = {{&S11511[13], 6}, {NULL}};
const Segment SGI250966[] = {{&S11511[115], 5}, {NULL}};
const Segment SGI250967[] = {{&S11511[38], 5}, {NULL}};
const Segment SGI250968[] = {{&S11511[78], 8}, {NULL}};
const Segment SGI250969[] = {{&S11511[128], 6}, {NULL}};
const Segment SGI250970[] = {{&S11511[142], 9}, {NULL}};
const Segment SGI250971[] = {{&S11512[13], 6}, {NULL}};
const Segment SGI250972[] = {{&S11512[115], 5}, {NULL}};
const Segment SGI250973[] = {{&S11512[38], 5}, {NULL}};
const Segment SGI250974[] = {{&S11512[78], 8}, {NULL}};
const Segment SGI250975[] = {{&S11512[128], 5}, {NULL}};
const Segment SGI250976[] = {{&S11512[141], 9}, {NULL}};
const Segment SGI250977[] = {{&S11514[13], 6}, {NULL}};
const Segment SGI250978[] = {{&S11514[115], 5}, {NULL}};
const Segment SGI250979[] = {{&S11514[38], 5}, {NULL}};
const Segment SGI250980[] = {{&S11514[78], 8}, {NULL}};
const Segment SGI250981[] = {{&S11514[128], 5}, {NULL}};
const Segment SGI250982[] = {{&S11514[141], 9}, {NULL}};
const Segment SGI250983[] = {{&S11515[13], 6}, {NULL}};
const Segment SGI250984[] = {{&S11515[115], 5}, {NULL}};
const Segment SGI250985[] = {{&S11515[38], 5}, {NULL}};
const Segment SGI250986[] = {{&S11515[78], 8}, {NULL}};
const Segment SGI250987[] = {{&S11515[128], 4}, {NULL}};
const Segment SGI250988[] = {{&S11515[140], 9}, {NULL}};
const Segment SGI250989[] = {{&S11516[13], 6}, {NULL}};
const Segment SGI250990[] = {{&S11516[115], 5}, {NULL}};
const Segment SGI250991[] = {{&S11516[38], 5}, {NULL}};
const Segment SGI250992[] = {{&S11516[78], 8}, {NULL}};
const Segment SGI250993[] = {{&S11516[128], 3}, {NULL}};
const Segment SGI250994[] = {{&S11516[139], 9}, {NULL}};
const Segment SGI250995[] = {{&S11517[13], 6}, {NULL}};
const Segment SGI250996[] = {{&S11517[113], 5}, {NULL}};
const Segment SGI250997[] = {{&S11517[38], 3}, {NULL}};
const Segment SGI250998[] = {{&S11517[76], 8}, {NULL}};
const Segment SGI250999[] = {{&S11517[126], 5}, {NULL}};
const Segment SGI251000[] = {{&S11517[139], 9}, {NULL}};
const Segment SGI251001[] = {{&S11518[13], 6}, {NULL}};
const Segment SGI251002[] = {{&S11518[115], 5}, {NULL}};
const Segment SGI251003[] = {{&S11518[38], 5}, {NULL}};
const Segment SGI251004[] = {{&S11518[78], 8}, {NULL}};
const Segment SGI251005[] = {{&S11518[128], 5}, {NULL}};
const Segment SGI251006[] = {{&S11518[141], 9}, {NULL}};
const Segment SGI251007[] = {{&S11519[13], 6}, {NULL}};
const Segment SGI251008[] = {{&S11519[115], 5}, {NULL}};
const Segment SGI251009[] = {{&S11519[38], 5}, {NULL}};
const Segment SGI251010[] = {{&S11519[78], 8}, {NULL}};
const Segment SGI251011[] = {{&S11519[128], 5}, {NULL}};
const Segment SGI251012[] = {{&S11519[141], 9}, {NULL}};
const Segment SGI251013[] = {{&S11520[13], 6}, {NULL}};
const Segment SGI251014[] = {{&S11520[98], 5}, {NULL}};
const Segment SGI251015[] = {{&S11520[31], 5}, {NULL}};
const Segment SGI251016[] = {{&S11520[64], 5}, {NULL}};
const Segment SGI251017[] = {{&S11520[111], 7}, {NULL}};
const Segment SGI251018[] = {{&S11520[126], 9}, {NULL}};
const Segment SGI251019[] = {{&S11521[13], 6}, {NULL}};
const Segment SGI251020[] = {{&S11521[115], 5}, {NULL}};
const Segment SGI251021[] = {{&S11521[38], 5}, {NULL}};
const Segment SGI251022[] = {{&S11521[78], 8}, {NULL}};
const Segment SGI251023[] = {{&S11521[128], 5}, {NULL}};
const Segment SGI251024[] = {{&S11521[141], 9}, {NULL}};
const Segment SGI251025[] = {{&S11522[13], 6}, {NULL}};
const Segment SGI251026[] = {{&S11522[115], 5}, {NULL}};
const Segment SGI251027[] = {{&S11522[38], 5}, {NULL}};
const Segment SGI251028[] = {{&S11522[78], 8}, {NULL}};
const Segment SGI251029[] = {{&S11522[128], 5}, {NULL}};
const Segment SGI251030[] = {{&S11522[141], 9}, {NULL}};
const Segment SGI251031[] = {{&S11523[13], 6}, {NULL}};
const Segment SGI251032[] = {{&S11523[115], 5}, {NULL}};
const Segment SGI251033[] = {{&S11523[38], 5}, {NULL}};
const Segment SGI251034[] = {{&S11523[78], 8}, {NULL}};
const Segment SGI251035[] = {{&S11523[128], 5}, {NULL}};
const Segment SGI251036[] = {{&S11523[141], 9}, {NULL}};
const Segment SGI251037[] = {{&S11524[13], 6}, {NULL}};
const Segment SGI251038[] = {{&S11524[115], 5}, {NULL}};
const Segment SGI251039[] = {{&S11524[38], 5}, {NULL}};
const Segment SGI251040[] = {{&S11524[78], 8}, {NULL}};
const Segment SGI251041[] = {{&S11524[128], 5}, {NULL}};
const Segment SGI251042[] = {{&S11524[141], 9}, {NULL}};
const Segment SGI251043[] = {{&S11525[13], 6}, {NULL}};
const Segment SGI251044[] = {{&S11525[116], 5}, {NULL}};
const Segment SGI251045[] = {{&S11525[38], 6}, {NULL}};
const Segment SGI251046[] = {{&S11525[79], 8}, {NULL}};
const Segment SGI251047[] = {{&S11525[129], 5}, {NULL}};
const Segment SGI251048[] = {{&S11525[142], 9}, {NULL}};
const Segment SGI251049[] = {{&S11527[13], 6}, {NULL}};
const Segment SGI251050[] = {{&S11527[38], 5}, {NULL}};
const Segment SGI251051[] = {{&S11527[115], 5}, {NULL}};
const Segment SGI251052[] = {{&S11527[128], 9}, {NULL}};
const Segment SGI251053[] = {{&S11528[13], 6}, {NULL}};
const Segment SGI251054[] = {{&S11528[38], 5}, {NULL}};
const Segment SGI251055[] = {{&S11528[78], 8}, {NULL}};
const Segment SGI251056[] = {{&S11528[114], 5}, {NULL}};
const Segment SGI251057[] = {{&S11528[127], 9}, {NULL}};
const Segment SGI251058[] = {{&S11529[13], 6}, {NULL}};
const Segment SGI251059[] = {{&S11529[115], 5}, {NULL}};
const Segment SGI251060[] = {{&S11529[38], 5}, {NULL}};
const Segment SGI251061[] = {{&S11529[78], 8}, {NULL}};
const Segment SGI251062[] = {{&S11529[128], 5}, {NULL}};
const Segment SGI251063[] = {{&S11529[141], 9}, {NULL}};
const Segment SGI251064[] = {{&S11530[13], 6}, {NULL}};
const Segment SGI251065[] = {{&S11530[116], 5}, {NULL}};
const Segment SGI251066[] = {{&S11530[38], 5}, {NULL}};
const Segment SGI251067[] = {{&S11530[78], 9}, {NULL}};
const Segment SGI251068[] = {{&S11530[129], 6}, {NULL}};
const Segment SGI251069[] = {{&S11530[143], 9}, {NULL}};
const Segment SGI251070[] = {{&S11531[13], 6}, {NULL}};
const Segment SGI251071[] = {{&S11531[115], 5}, {NULL}};
const Segment SGI251072[] = {{&S11531[38], 5}, {NULL}};
const Segment SGI251073[] = {{&S11531[78], 8}, {NULL}};
const Segment SGI251074[] = {{&S11531[128], 5}, {NULL}};
const Segment SGI251075[] = {{&S11531[141], 9}, {NULL}};
const Segment SGI251076[] = {{&S11533[13], 6}, {NULL}};
const Segment SGI251077[] = {{&S11533[116], 5}, {NULL}};
const Segment SGI251078[] = {{&S11533[38], 5}, {NULL}};
const Segment SGI251079[] = {{&S11533[78], 9}, {NULL}};
const Segment SGI251080[] = {{&S11533[129], 6}, {NULL}};
const Segment SGI251081[] = {{&S11533[143], 9}, {NULL}};
const Segment SGI251082[] = {{&S1224[41], 10}, {NULL}};
const Segment SGI251083[] = {{&S1224[0], 129}, {NULL}};
const Segment SGI251084[] = {{&S1225[0], 11}, {&S1225[12], 37}, {NULL}};
const Segment SGI251085[] = {{&S1225[0], 49}, {NULL}};
const Segment SGI251086[] = {{&S1226[0], 10}, {&S1226[11], 33}, {NULL}};
const Segment SGI251087[] = {{&S1226[0], 44}, {NULL}};
const Segment SGI251088[] = {{&S1227[22], 31}, {NULL}};
const Segment SGI251089[] = {{&S1227[0], 53}, {NULL}};
const Segment SGI251090[] = {{&S1228[25], 16}, {&S1228[42], 32}, {NULL}};
const Segment SGI251091[] = {{&S1228[0], 74}, {NULL}};
const Segment SGI251092[] = {{&S1229[0], 14}, {NULL}};
const Segment SGI251093[] = {{&S1230[0], 15}, {NULL}};
const Segment SGI251094[] = {{&S1230[0], 98}, {NULL}};
const Segment SGI251095[] = {{&S1231[49], 24}, {NULL}};
const Segment SGI251096[] = {{&S1231[0], 73}, {NULL}};
const Segment SGI251097[] = {{&S1232[0], 21}, {&S1232[22], 35}, {NULL}};
const Segment SGI251098[] = {{&S1232[0], 57}, {NULL}};
const Segment SGI251099[] = {{&S1233[0], 12}, {NULL}};
const Segment SGI251100[] = {{&S1234[0], 12}, {&S1234[13], 35}, {NULL}};
const Segment SGI251101[] = {{&S1234[0], 48}, {NULL}};
const Segment SGI251102[] = {{&S1235[0], 12}, {NULL}};
const Segment SGI251103[] = {{&S1237[0], 15}, {NULL}};
const Segment SGI251104[] = {{&S1238[0], 13}, {&S1238[14], 24}, {NULL}};
const Segment SGI251105[] = {{&S1238[0], 38}, {NULL}};
const Segment SGI251106[] = {{&S1239[0], 8}, {NULL}};
const Segment SGI251107[] = {{&S1240[25], 15}, {&S1240[41], 38}, {NULL}};
const Segment SGI251108[] = {{&S1240[0], 79}, {NULL}};
const Segment SGI251109[] = {{&S1241[25], 15}, {&S1241[41], 48}, {NULL}};
const Segment SGI251110[] = {{&S1241[0], 89}, {NULL}};
const Segment SGI251111[] = {{&S1242[0], 16}, {NULL}};
const Segment SGI251112[] = {{&S1243[0], 15}, {NULL}};
const Segment SGI251113[] = {{&S1244[0], 58}, {NULL}};
const Segment SGI251114[] = {{&S1244[0], 87}, {NULL}};
const Segment SGI251115[] = {{&S1245[46], 24}, {NULL}};
const Segment SGI251116[] = {{&S1245[0], 70}, {NULL}};
const Segment SGI251117[] = {{&S1246[10], 7}, {NULL}};
const Segment SGI251118[] = {{&S1246[0], 64}, {NULL}};
const Segment SGI251119[] = {{&S1247[0], 11}, {&S1247[21], 7}, {NULL}};
const Segment SGI251120[] = {{&S1247[0], 57}, {NULL}};
const Segment SGI251121[] = {{&S1248[0], 34}, {NULL}};
const Segment SGI251122[] = {{&S1249[0], 17}, {NULL}};
const Segment SGI251123[] = {{&S1250[0], 10}, {NULL}};
const Segment SGI251124[] = {{&S1251[11], 27}, {NULL}};
const Segment SGI251125[] = {{&S1251[0], 38}, {NULL}};
const Segment SGI251126[] = {{&S1252[15], 5}, {NULL}};
const Segment SGI251127[] = {{&S1252[0], 20}, {NULL}};
const Segment SGI251128[] = {{&S1253[0], 12}, {NULL}};
const Segment SGI251129[] = {{&S1254[25], 13}, {&S1254[39], 11}, {&S1254[51], 34}, {NULL}};
const Segment SGI251130[] = {{&S1254[0], 85}, {NULL}};
const Segment SGI251131[] = {{&S1255[0], 12}, {NULL}};
const Segment SGI251132[] = {{&S1256[25], 13}, {&S1256[39], 34}, {NULL}};
const Segment SGI251133[] = {{&S1256[0], 73}, {NULL}};
const Segment SGI251134[] = {{&S1257[84], 17}, {NULL}};
const Segment SGI251135[] = {{&S1257[0], 123}, {NULL}};
const Segment SGI251136[] = {{&S1258[7], 3}, {NULL}};
const Segment SGI251137[] = {{&S1258[0], 50}, {NULL}};
const Segment SGI251138[] = {{&S1259[13], 7}, {NULL}};
const Segment SGI251139[] = {{&S1259[0], 48}, {NULL}};
const Segment SGI251140[] = {{&S1260[10], 7}, {NULL}};
const Segment SGI251141[] = {{&S1260[0], 42}, {NULL}};
const Segment SGI251142[] = {{&S1261[14], 36}, {NULL}};
const Segment SGI251143[] = {{&S1261[0], 50}, {NULL}};
const Segment SGI251144[] = {{&S1262[10], 3}, {NULL}};
const Segment SGI251145[] = {{&S1262[0], 13}, {NULL}};
const Segment SGI251146[] = {{&S1264[17], 7}, {&S1264[26], 26}, {NULL}};
const Segment SGI251147[] = {{&S1264[0], 52}, {NULL}};
const Segment SGI251148[] = {{&S1265[16], 7}, {&S1265[30], 17}, {NULL}};
const Segment SGI251149[] = {{&S1265[0], 47}, {NULL}};
const Segment SGI251150[] = {{&S1266[0], 10}, {NULL}};
const Segment SGI251151[] = {{&S1266[0], 30}, {NULL}};
const Segment SGI251152[] = {{&S1267[25], 10}, {&S1267[36], 31}, {NULL}};
const Segment SGI251153[] = {{&S1267[0], 69}, {NULL}};
const Segment SGI251154[] = {{&S1268[0], 12}, {NULL}};
const Segment SGI251155[] = {{&S1269[52], 35}, {NULL}};
const Segment SGI251156[] = {{&S1269[0], 87}, {NULL}};
const Segment SGI251157[] = {{&S1270[25], 14}, {&S1270[40], 32}, {NULL}};
const Segment SGI251158[] = {{&S1270[0], 72}, {NULL}};
const Segment SGI251159[] = {{&S1271[0], 13}, {&S1271[14], 32}, {NULL}};
const Segment SGI251160[] = {{&S1271[0], 46}, {NULL}};
const Segment SGI251161[] = {{&S1272[0], 19}, {NULL}};
const Segment SGI251162[] = {{&S1273[37], 21}, {NULL}};
const Segment SGI251163[] = {{&S1273[0], 58}, {NULL}};
const Segment SGI251164[] = {{&S1274[0], 17}, {&S1274[18], 34}, {NULL}};
const Segment SGI251165[] = {{&S1274[0], 52}, {NULL}};
const Segment SGI251166[] = {{&S1275[0], 16}, {&S1275[17], 19}, {NULL}};
const Segment SGI251167[] = {{&S1275[0], 36}, {NULL}};
const Segment SGI251168[] = {{&S1276[25], 8}, {&S1276[42], 54}, {NULL}};
const Segment SGI251169[] = {{&S1276[0], 96}, {NULL}};
const Segment SGI251170[] = {{&S1277[25], 16}, {&S1277[42], 41}, {NULL}};
const Segment SGI251171[] = {{&S1277[0], 83}, {NULL}};
const Segment SGI251172[] = {{&S1278[0], 7}, {&S1278[8], 7}, {&S1278[25], 24}, {NULL}};
const Segment SGI251173[] = {{&S1278[0], 49}, {NULL}};
const Segment SGI251174[] = {{&S1279[39], 31}, {NULL}};
const Segment SGI251175[] = {{&S1279[0], 70}, {NULL}};
const Segment SGI251176[] = {{&S1280[25], 12}, {&S1280[38], 23}, {NULL}};
const Segment SGI251177[] = {{&S1280[0], 61}, {NULL}};
const Segment SGI251178[] = {{&S1281[38], 3}, {NULL}};
const Segment SGI251179[] = {{&S1281[0], 72}, {NULL}};
const Segment SGI251180[] = {{&S1282[0], 10}, {NULL}};
const Segment SGI251181[] = {{&S1283[25], 17}, {&S1283[43], 32}, {NULL}};
const Segment SGI251182[] = {{&S1283[0], 75}, {NULL}};
const Segment SGI251183[] = {{&S1284[25], 11}, {&S1284[37], 27}, {NULL}};
const Segment SGI251184[] = {{&S1284[0], 64}, {NULL}};
const Segment SGI251185[] = {{&S1285[19], 6}, {NULL}};
const Segment SGI251186[] = {{&S1285[0], 30}, {NULL}};
const Segment SGI251187[] = {{&S1286[25], 12}, {NULL}};
const Segment SGI251188[] = {{&S1286[0], 74}, {NULL}};
const Segment SGI251189[] = {{&S1287[0], 12}, {&S1287[13], 40}, {NULL}};
const Segment SGI251190[] = {{&S1287[0], 53}, {NULL}};
const Segment SGI251191[] = {{&S1288[0], 5}, {NULL}};
const Segment SGI251192[] = {{&S1289[43], 24}, {NULL}};
const Segment SGI251193[] = {{&S1289[0], 67}, {NULL}};
const Segment SGI251194[] = {{&S1290[0], 14}, {NULL}};
const Segment SGI251195[] = {{&S1291[0], 20}, {NULL}};
const Segment SGI251196[] = {{&S1291[0], 60}, {NULL}};
const Segment SGI251197[] = {{&S1292[25], 12}, {&S1292[38], 23}, {NULL}};
const Segment SGI251198[] = {{&S1292[0], 61}, {NULL}};
const Segment SGI251199[] = {{&S1293[25], 12}, {&S1293[38], 23}, {NULL}};
const Segment SGI251200[] = {{&S1293[0], 61}, {NULL}};
const Segment SGI251201[] = {{&S1294[41], 24}, {NULL}};
const Segment SGI251202[] = {{&S1294[0], 65}, {NULL}};
const Segment SGI251203[] = {{&S1295[0], 10}, {NULL}};
const Segment SGI251204[] = {{&S1296[0], 9}, {NULL}};
const Segment SGI251205[] = {{&S1297[25], 12}, {&S1297[38], 46}, {NULL}};
const Segment SGI251206[] = {{&S1297[0], 110}, {NULL}};
const Segment SGI251207[] = {{&S1298[0], 11}, {&S1298[12], 35}, {NULL}};
const Segment SGI251208[] = {{&S1298[0], 47}, {NULL}};
const Segment SGI251209[] = {{&S1299[25], 21}, {&S1299[47], 34}, {NULL}};
const Segment SGI251210[] = {{&S1299[0], 81}, {NULL}};
const Segment SGI251211[] = {{&S1300[25], 14}, {&S1300[40], 34}, {NULL}};
const Segment SGI251212[] = {{&S1300[0], 103}, {NULL}};
const Segment SGI251213[] = {{&S1301[19], 7}, {NULL}};
const Segment SGI251214[] = {{&S1301[0], 26}, {NULL}};
const Segment SGI251215[] = {{&S1302[19], 6}, {NULL}};
const Segment SGI251216[] = {{&S1302[0], 30}, {NULL}};
const Segment SGI251217[] = {{&S1303[33], 36}, {NULL}};
const Segment SGI251218[] = {{&S1303[0], 69}, {NULL}};
const Segment SGI251219[] = {{&S1304[0], 17}, {&S1304[39], 64}, {NULL}};
const Segment SGI251220[] = {{&S1304[0], 136}, {NULL}};
const Segment SGI251221[] = {{&S1305[7], 3}, {NULL}};
const Segment SGI251222[] = {{&S1305[0], 10}, {NULL}};
const Segment SGI251223[] = {{&S1306[0], 13}, {NULL}};
const Segment SGI251224[] = {{&S1306[0], 42}, {NULL}};
const Segment SGI251225[] = {{&S1307[25], 14}, {&S1307[40], 35}, {NULL}};
const Segment SGI251226[] = {{&S1307[0], 75}, {NULL}};
const Segment SGI251227[] = {{&S1308[0], 15}, {&S1308[16], 42}, {NULL}};
const Segment SGI251228[] = {{&S1308[0], 58}, {NULL}};
const Segment SGI251229[] = {{&S1309[12], 7}, {NULL}};
const Segment SGI251230[] = {{&S1309[0], 86}, {NULL}};
const Segment SGI251231[] = {{&S1310[29], 13}, {&S1310[43], 44}, {NULL}};
const Segment SGI251232[] = {{&S1310[0], 89}, {NULL}};
const Segment SGI251233[] = {{&S1311[25], 15}, {&S1311[41], 38}, {NULL}};
const Segment SGI251234[] = {{&S1311[0], 79}, {NULL}};
const Segment SGI251235[] = {{&S1312[25], 14}, {&S1312[40], 31}, {NULL}};
const Segment SGI251236[] = {{&S1312[0], 71}, {NULL}};
const Segment SGI251237[] = {{&S1313[0], 10}, {&S1313[36], 7}, {NULL}};
const Segment SGI251238[] = {{&S1313[0], 60}, {NULL}};
const Segment SGI251239[] = {{&S1314[35], 26}, {NULL}};
const Segment SGI251240[] = {{&S1314[0], 61}, {NULL}};
const Segment SGI251241[] = {{&S1315[19], 6}, {NULL}};
const Segment SGI251242[] = {{&S1315[0], 30}, {NULL}};
const Segment SGI251243[] = {{&S11534[13], 6}, {NULL}};
const Segment SGI251244[] = {{&S11534[8], 3}, {NULL}};
const Segment SGI251245[] = {{&S1316[0], 11}, {NULL}};
const Segment SGI251246[] = {{&S1316[0], 51}, {NULL}};
const Segment SGI251247[] = {{&S11534[28], 5}, {NULL}};
const Segment SGI251248[] = {{&S11534[61], 8}, {NULL}};
const Segment SGI251249[] = {{&S11534[109], 5}, {NULL}};
const Segment SGI251250[] = {{&S11534[122], 9}, {NULL}};
const Segment SGI251251[] = {{&S1317[12], 69}, {NULL}};
const Segment SGI251252[] = {{&S1317[0], 81}, {NULL}};
const Segment SGI251253[] = {{&S11535[13], 6}, {NULL}};
const Segment SGI251254[] = {{&S11535[116], 5}, {NULL}};
const Segment SGI251255[] = {{&S11535[38], 5}, {NULL}};
const Segment SGI251256[] = {{&S11535[78], 9}, {NULL}};
const Segment SGI251257[] = {{&S11535[129], 6}, {NULL}};
const Segment SGI251258[] = {{&S11535[143], 9}, {NULL}};
const Segment SGI251259[] = {{&S1318[33], 7}, {&S1318[42], 28}, {NULL}};
const Segment SGI251260[] = {{&S1318[0], 70}, {NULL}};
const Segment SGI251261[] = {{&S11536[13], 6}, {NULL}};
const Segment SGI251262[] = {{&S11536[48], 3}, {NULL}};
const Segment SGI251263[] = {{&S11536[86], 8}, {NULL}};
const Segment SGI251264[] = {{&S11536[122], 5}, {NULL}};
const Segment SGI251265[] = {{&S1319[19], 6}, {NULL}};
const Segment SGI251266[] = {{&S1319[0], 30}, {NULL}};
const Segment SGI251267[] = {{&S11536[135], 9}, {NULL}};
const Segment SGI251268[] = {{&S1320[0], 60}, {NULL}};
const Segment SGI251269[] = {{&S11537[13], 6}, {NULL}};
const Segment SGI251270[] = {{&S11537[112], 5}, {NULL}};
const Segment SGI251271[] = {{&S11537[38], 3}, {NULL}};
const Segment SGI251272[] = {{&S11537[76], 8}, {NULL}};
const Segment SGI251273[] = {{&S11537[125], 5}, {NULL}};
const Segment SGI251274[] = {{&S11537[138], 9}, {NULL}};
const Segment SGI251275[] = {{&S1321[0], 53}, {NULL}};
const Segment SGI251276[] = {{&S1322[0], 13}, {&S1322[14], 22}, {NULL}};
const Segment SGI251277[] = {{&S1322[0], 36}, {NULL}};
const Segment SGI251278[] = {{&S11538[13], 6}, {NULL}};
const Segment SGI251279[] = {{&S100304[34], 11}, {NULL}};
const Segment SGI251280[] = {{&S11538[112], 5}, {NULL}};
const Segment SGI251281[] = {{&S11538[38], 3}, {NULL}};
const Segment SGI251282[] = {{&S100304[27], 6}, {NULL}};
const Segment SGI251283[] = {{&S11538[76], 8}, {NULL}};
const Segment SGI251284[] = {{&S11538[125], 5}, {NULL}};
const Segment SGI251285[] = {{&S11538[138], 9}, {NULL}};
const Segment SGI251286[] = {{&S100304[97], 9}, {NULL}};
const Segment SGI251287[] = {{&S100304[70], 6}, {NULL}};
const Segment SGI251288[] = {{&S1323[41], 7}, {NULL}};
const Segment SGI251289[] = {{&S1323[0], 68}, {NULL}};
const Segment SGI251290[] = {{&S11539[13], 6}, {NULL}};
const Segment SGI251291[] = {{&S11539[112], 5}, {NULL}};
const Segment SGI251292[] = {{&S11539[38], 3}, {NULL}};
const Segment SGI251293[] = {{&S11539[76], 8}, {NULL}};
const Segment SGI251294[] = {{&S11539[125], 6}, {NULL}};
const Segment SGI251295[] = {{&S11539[139], 9}, {NULL}};
const Segment SGI251296[] = {{&S1324[21], 30}, {NULL}};
const Segment SGI251297[] = {{&S1324[0], 51}, {NULL}};
const Segment SGI251298[] = {{&S11540[13], 6}, {NULL}};
const Segment SGI251299[] = {{&S11540[112], 5}, {NULL}};
const Segment SGI251300[] = {{&S11540[38], 3}, {NULL}};
const Segment SGI251301[] = {{&S11540[76], 8}, {NULL}};
const Segment SGI251302[] = {{&S11540[125], 5}, {NULL}};
const Segment SGI251303[] = {{&S11540[138], 9}, {NULL}};
const Segment SGI251304[] = {{&S1325[25], 11}, {&S1325[37], 34}, {NULL}};
const Segment SGI251305[] = {{&S1325[0], 71}, {NULL}};
const Segment SGI251306[] = {{&S1326[8], 7}, {NULL}};
const Segment SGI251307[] = {{&S1326[0], 15}, {NULL}};
const Segment SGI251308[] = {{&S1327[0], 19}, {NULL}};
const Segment SGI251309[] = {{&S1327[0], 24}, {NULL}};
const Segment SGI251310[] = {{&S1328[41], 6}, {&S1328[48], 24}, {NULL}};
const Segment SGI251311[] = {{&S1328[0], 72}, {NULL}};
const Segment SGI251312[] = {{&S1329[8], 7}, {NULL}};
const Segment SGI251313[] = {{&S1329[0], 36}, {NULL}};
const Segment SGI251314[] = {{&S1330[0], 93}, {NULL}};
const Segment SGI251315[] = {{&S1331[13], 31}, {NULL}};
const Segment SGI251316[] = {{&S1331[0], 44}, {NULL}};
const Segment SGI251317[] = {{&S1332[0], 10}, {NULL}};
const Segment SGI251318[] = {{&S1332[0], 52}, {NULL}};
const Segment SGI251319[] = {{&S1333[19], 6}, {NULL}};
const Segment SGI251320[] = {{&S1333[0], 30}, {NULL}};
const Segment SGI251321[] = {{&S1334[38], 7}, {NULL}};
const Segment SGI251322[] = {{&S1334[0], 58}, {NULL}};
const Segment SGI251323[] = {{&S1335[25], 12}, {&S1335[38], 35}, {NULL}};
const Segment SGI251324[] = {{&S1335[0], 73}, {NULL}};
const Segment SGI251325[] = {{&S1336[25], 16}, {&S1336[42], 33}, {NULL}};
const Segment SGI251326[] = {{&S1336[0], 75}, {NULL}};
const Segment SGI251327[] = {{&S1337[0], 9}, {NULL}};
const Segment SGI251328[] = {{&S1338[0], 10}, {&S1338[11], 36}, {NULL}};
const Segment SGI251329[] = {{&S1338[0], 78}, {NULL}};
const Segment SGI251330[] = {{&S1339[0], 19}, {&S1339[20], 8}, {&S1339[58], 48}, {NULL}};
const Segment SGI251331[] = {{&S1339[0], 106}, {NULL}};
const Segment SGI251332[] = {{&S1340[33], 7}, {NULL}};
const Segment SGI251333[] = {{&S1340[0], 58}, {NULL}};
const Segment SGI251334[] = {{&S1341[0], 9}, {&S1341[10], 38}, {NULL}};
const Segment SGI251335[] = {{&S1341[0], 48}, {NULL}};
const Segment SGI251336[] = {{&S1342[0], 11}, {NULL}};
const Segment SGI251337[] = {{&S1342[0], 46}, {NULL}};
const Segment SGI251338[] = {{&S1343[0], 17}, {NULL}};
const Segment SGI251339[] = {{&S1343[0], 46}, {NULL}};
const Segment SGI251340[] = {{&S1344[42], 24}, {NULL}};
const Segment SGI251341[] = {{&S1344[0], 66}, {NULL}};
const Segment SGI251342[] = {{&S1345[45], 24}, {NULL}};
const Segment SGI251343[] = {{&S1345[0], 69}, {NULL}};
const Segment SGI251344[] = {{&S1346[0], 12}, {NULL}};
const Segment SGI251345[] = {{&S1346[0], 55}, {NULL}};
const Segment SGI251346[] = {{&S1347[25], 12}, {&S1347[38], 31}, {NULL}};
const Segment SGI251347[] = {{&S1347[0], 69}, {NULL}};
const Segment SGI251348[] = {{&S1348[0], 4}, {NULL}};
const Segment SGI251349[] = {{&S1349[0], 18}, {&S1349[28], 26}, {NULL}};
const Segment SGI251350[] = {{&S1349[0], 54}, {NULL}};
const Segment SGI251351[] = {{&S1350[81], 7}, {NULL}};
const Segment SGI251352[] = {{&S1350[0], 99}, {NULL}};
const Segment SGI251353[] = {{&S1351[0], 12}, {NULL}};
const Segment SGI251354[] = {{&S1352[0], 12}, {&S1352[13], 29}, {NULL}};
const Segment SGI251355[] = {{&S1352[0], 42}, {NULL}};
const Segment SGI251356[] = {{&S1353[0], 18}, {NULL}};
const Segment SGI251357[] = {{&S1353[0], 86}, {NULL}};
const Segment SGI251358[] = {{&S1354[25], 14}, {NULL}};
const Segment SGI251359[] = {{&S1354[0], 72}, {NULL}};
const Segment SGI251360[] = {{&S1355[0], 11}, {NULL}};
const Segment SGI251361[] = {{&S1356[43], 6}, {&S1356[53], 24}, {NULL}};
const Segment SGI251362[] = {{&S1356[0], 87}, {NULL}};
const Segment SGI251363[] = {{&S1357[25], 6}, {&S1357[37], 31}, {NULL}};
const Segment SGI251364[] = {{&S1357[0], 75}, {NULL}};
const Segment SGI251365[] = {{&S1358[0], 21}, {&S1358[22], 20}, {NULL}};
const Segment SGI251366[] = {{&S1358[0], 64}, {NULL}};
const Segment SGI251367[] = {{&S1359[0], 14}, {NULL}};
const Segment SGI251368[] = {{&S1359[0], 37}, {NULL}};
const Segment SGI251369[] = {{&S1360[25], 14}, {NULL}};
const Segment SGI251370[] = {{&S1360[0], 59}, {NULL}};
const Segment SGI251371[] = {{&S1361[0], 11}, {NULL}};
const Segment SGI251372[] = {{&S1362[13], 7}, {NULL}};
const Segment SGI251373[] = {{&S1362[0], 31}, {NULL}};
const Segment SGI251374[] = {{&S1363[0], 24}, {&S1363[25], 44}, {NULL}};
const Segment SGI251375[] = {{&S1363[0], 69}, {NULL}};
const Segment SGI251376[] = {{&S1364[25], 24}, {&S1364[50], 48}, {NULL}};
const Segment SGI251377[] = {{&S1364[0], 98}, {NULL}};
const Segment SGI251378[] = {{&S1365[19], 6}, {NULL}};
const Segment SGI251379[] = {{&S1365[0], 30}, {NULL}};
const Segment SGI251380[] = {{&S1366[25], 10}, {&S1366[36], 32}, {NULL}};
const Segment SGI251381[] = {{&S1366[0], 68}, {NULL}};
const Segment SGI251382[] = {{&S1367[12], 16}, {&S1367[29], 36}, {NULL}};
const Segment SGI251383[] = {{&S1367[0], 65}, {NULL}};
const Segment SGI251384[] = {{&S1368[40], 3}, {NULL}};
const Segment SGI251385[] = {{&S1368[0], 49}, {NULL}};
const Segment SGI251386[] = {{&S1369[0], 40}, {NULL}};
const Segment SGI251387[] = {{&S1370[16], 7}, {&S1370[45], 22}, {NULL}};
const Segment SGI251388[] = {{&S1370[0], 67}, {NULL}};
const Segment SGI251389[] = {{&S1371[0], 12}, {NULL}};
const Segment SGI251390[] = {{&S1372[0], 32}, {NULL}};
const Segment SGI251391[] = {{&S1373[0], 11}, {NULL}};
const Segment SGI251392[] = {{&S1373[0], 61}, {NULL}};
const Segment SGI251393[] = {{&S1374[0], 10}, {NULL}};
const Segment SGI251394[] = {{&S100358[44], 10}, {NULL}};
const Segment SGI251395[] = {{&S100358[44], 21}, {NULL}};
const Segment SGI251396[] = {{&S1375[0], 9}, {NULL}};
const Segment SGI251397[] = {{&S100358[30], 6}, {NULL}};
const Segment SGI251398[] = {{&S1375[0], 45}, {NULL}};
const Segment SGI251399[] = {{&S100358[114], 3}, {NULL}};
const Segment SGI251400[] = {{&S100358[79], 6}, {NULL}};
const Segment SGI251401[] = {{&S1376[0], 11}, {&S1376[12], 30}, {NULL}};
const Segment SGI251402[] = {{&S1376[0], 42}, {NULL}};
const Segment SGI251403[] = {{&S1377[25], 16}, {NULL}};
const Segment SGI251404[] = {{&S1377[0], 100}, {NULL}};
const Segment SGI251405[] = {{&S1378[15], 9}, {NULL}};
const Segment SGI251406[] = {{&S1378[0], 24}, {NULL}};
const Segment SGI251407[] = {{&S1379[0], 13}, {NULL}};
const Segment SGI251408[] = {{&S1380[36], 6}, {NULL}};
const Segment SGI251409[] = {{&S1380[0], 68}, {NULL}};
const Segment SGI251410[] = {{&S1381[0], 34}, {NULL}};
const Segment SGI251411[] = {{&S1382[0], 22}, {NULL}};
const Segment SGI251412[] = {{&S1383[25], 21}, {&S1383[47], 40}, {NULL}};
const Segment SGI251413[] = {{&S1383[0], 87}, {NULL}};
const Segment SGI251414[] = {{&S1384[0], 8}, {NULL}};
const Segment SGI251415[] = {{&S1385[0], 12}, {NULL}};
const Segment SGI251416[] = {{&S1385[0], 20}, {NULL}};
const Segment SGI251417[] = {{&S100382[44], 18}, {NULL}};
const Segment SGI251418[] = {{&S100382[44], 29}, {NULL}};
const Segment SGI251419[] = {{&S100382[30], 6}, {NULL}};
const Segment SGI251420[] = {{&S100382[87], 6}, {NULL}};
const Segment SGI251421[] = {{&S100382[114], 13}, {NULL}};
const Segment SGI251422[] = {{&S100387[34], 14}, {NULL}};
const Segment SGI251423[] = {{&S100387[27], 6}, {NULL}};
const Segment SGI251424[] = {{&S100387[100], 9}, {NULL}};
const Segment SGI251425[] = {{&S100387[73], 6}, {NULL}};
const Segment SGI251426[] = {{&S11669[13], 4}, {NULL}};
const Segment SGI251427[] = {{&S11669[92], 3}, {NULL}};
const Segment SGI251428[] = {{&S11669[26], 3}, {NULL}};
const Segment SGI251429[] = {{&S11669[57], 6}, {NULL}};
const Segment SGI251430[] = {{&S11669[103], 8}, {NULL}};
const Segment SGI251431[] = {{&S11669[119], 7}, {NULL}};
const Segment SGI251432[] = {{&S11670[13], 4}, {NULL}};
const Segment SGI251433[] = {{&S11670[92], 3}, {NULL}};
const Segment SGI251434[] = {{&S11670[26], 3}, {NULL}};
const Segment SGI251435[] = {{&S11670[57], 6}, {NULL}};
const Segment SGI251436[] = {{&S11670[103], 6}, {NULL}};
const Segment SGI251437[] = {{&S11670[117], 7}, {NULL}};
const Segment SGI251438[] = {{&S11671[13], 4}, {NULL}};
const Segment SGI251439[] = {{&S11671[92], 3}, {NULL}};
const Segment SGI251440[] = {{&S11671[26], 3}, {NULL}};
const Segment SGI251441[] = {{&S11671[57], 6}, {NULL}};
const Segment SGI251442[] = {{&S11671[103], 8}, {NULL}};
const Segment SGI251443[] = {{&S11671[119], 7}, {NULL}};
const Segment SGI251444[] = {{&S11672[13], 4}, {NULL}};
const Segment SGI251445[] = {{&S11672[92], 3}, {NULL}};
const Segment SGI251446[] = {{&S11672[26], 3}, {NULL}};
const Segment SGI251447[] = {{&S11672[57], 6}, {NULL}};
const Segment SGI251448[] = {{&S11672[103], 8}, {NULL}};
const Segment SGI251449[] = {{&S11672[119], 7}, {NULL}};
const Segment SGI251450[] = {{&S11673[13], 4}, {NULL}};
const Segment SGI251451[] = {{&S11673[92], 3}, {NULL}};
const Segment SGI251452[] = {{&S11673[26], 3}, {NULL}};
const Segment SGI251453[] = {{&S11673[57], 6}, {NULL}};
const Segment SGI251454[] = {{&S11673[103], 6}, {NULL}};
const Segment SGI251455[] = {{&S11673[117], 7}, {NULL}};
const Segment SGI251456[] = {{&S11674[13], 4}, {NULL}};
const Segment SGI251457[] = {{&S11674[92], 3}, {NULL}};
const Segment SGI251458[] = {{&S11674[26], 3}, {NULL}};
const Segment SGI251459[] = {{&S11674[57], 6}, {NULL}};
const Segment SGI251460[] = {{&S11674[103], 6}, {NULL}};
const Segment SGI251461[] = {{&S11674[117], 7}, {NULL}};
const Segment SGI251462[] = {{&S11675[13], 4}, {NULL}};
const Segment SGI251463[] = {{&S11675[92], 3}, {NULL}};
const Segment SGI251464[] = {{&S11675[26], 3}, {NULL}};
const Segment SGI251465[] = {{&S11675[57], 6}, {NULL}};
const Segment SGI251466[] = {{&S11675[103], 6}, {NULL}};
const Segment SGI251467[] = {{&S11675[117], 7}, {NULL}};
const Segment SGI251468[] = {{&S11676[13], 4}, {NULL}};
const Segment SGI251469[] = {{&S11676[91], 3}, {NULL}};
const Segment SGI251470[] = {{&S11676[56], 6}, {NULL}};
const Segment SGI251471[] = {{&S11676[102], 6}, {NULL}};
const Segment SGI251472[] = {{&S11676[116], 7}, {NULL}};
const Segment SGI251473[] = {{&S11677[13], 4}, {NULL}};
const Segment SGI251474[] = {{&S11677[94], 3}, {NULL}};
const Segment SGI251475[] = {{&S11677[26], 5}, {NULL}};
const Segment SGI251476[] = {{&S11677[59], 6}, {NULL}};
const Segment SGI251477[] = {{&S11677[105], 6}, {NULL}};
const Segment SGI251478[] = {{&S11677[119], 7}, {NULL}};
const Segment SGI251479[] = {{&S11678[13], 4}, {NULL}};
const Segment SGI251480[] = {{&S11678[94], 3}, {NULL}};
const Segment SGI251481[] = {{&S11678[26], 5}, {NULL}};
const Segment SGI251482[] = {{&S11678[59], 6}, {NULL}};
const Segment SGI251483[] = {{&S11678[105], 6}, {NULL}};
const Segment SGI251484[] = {{&S11678[119], 7}, {NULL}};
const Segment SGI251485[] = {{&S11679[13], 4}, {NULL}};
const Segment SGI251486[] = {{&S11679[92], 3}, {NULL}};
const Segment SGI251487[] = {{&S11679[26], 3}, {NULL}};
const Segment SGI251488[] = {{&S11679[57], 6}, {NULL}};
const Segment SGI251489[] = {{&S11679[103], 12}, {NULL}};
const Segment SGI251490[] = {{&S11679[116], 7}, {NULL}};
const Segment SGI251491[] = {{&S11680[13], 4}, {NULL}};
const Segment SGI251492[] = {{&S11680[94], 3}, {NULL}};
const Segment SGI251493[] = {{&S11680[26], 5}, {NULL}};
const Segment SGI251494[] = {{&S11680[59], 6}, {NULL}};
const Segment SGI251495[] = {{&S11680[105], 6}, {NULL}};
const Segment SGI251496[] = {{&S11680[119], 7}, {NULL}};
const Segment SGI251497[] = {{&S100445[44], 10}, {NULL}};
const Segment SGI251498[] = {{&S11681[13], 4}, {NULL}};
const Segment SGI251499[] = {{&S100445[44], 21}, {NULL}};
const Segment SGI251500[] = {{&S11681[94], 3}, {NULL}};
const Segment SGI251501[] = {{&S100445[30], 6}, {NULL}};
const Segment SGI251502[] = {{&S11681[26], 5}, {NULL}};
const Segment SGI251503[] = {{&S11681[59], 6}, {NULL}};
const Segment SGI251504[] = {{&S11681[105], 7}, {NULL}};
const Segment SGI251505[] = {{&S11681[120], 7}, {NULL}};
const Segment SGI251506[] = {{&S100445[79], 6}, {NULL}};
const Segment SGI251507[] = {{&S100445[106], 13}, {NULL}};
const Segment SGI251508[] = {{&S11682[13], 4}, {NULL}};
const Segment SGI251509[] = {{&S11682[92], 3}, {NULL}};
const Segment SGI251510[] = {{&S11682[26], 3}, {NULL}};
const Segment SGI251511[] = {{&S11682[57], 6}, {NULL}};
const Segment SGI251512[] = {{&S11682[103], 5}, {NULL}};
const Segment SGI251513[] = {{&S11682[116], 7}, {NULL}};
const Segment SGI251514[] = {{&S11683[13], 4}, {NULL}};
const Segment SGI251515[] = {{&S11683[93], 3}, {NULL}};
const Segment SGI251516[] = {{&S11683[58], 6}, {NULL}};
const Segment SGI251517[] = {{&S11683[104], 6}, {NULL}};
const Segment SGI251518[] = {{&S11683[118], 7}, {NULL}};
const Segment SGI251519[] = {{&S11684[13], 4}, {NULL}};
const Segment SGI251520[] = {{&S11684[94], 3}, {NULL}};
const Segment SGI251521[] = {{&S11684[26], 5}, {NULL}};
const Segment SGI251522[] = {{&S11684[59], 6}, {NULL}};
const Segment SGI251523[] = {{&S11684[105], 12}, {NULL}};
const Segment SGI251524[] = {{&S11684[118], 7}, {NULL}};
const Segment SGI251525[] = {{&S11685[13], 4}, {NULL}};
const Segment SGI251526[] = {{&S11685[94], 3}, {NULL}};
const Segment SGI251527[] = {{&S11685[26], 5}, {NULL}};
const Segment SGI251528[] = {{&S11685[59], 6}, {NULL}};
const Segment SGI251529[] = {{&S11685[105], 5}, {NULL}};
const Segment SGI251530[] = {{&S11685[118], 7}, {NULL}};
const Segment SGI251531[] = {{&S100452[44], 10}, {NULL}};
const Segment SGI251532[] = {{&S100452[44], 22}, {NULL}};
const Segment SGI251533[] = {{&S100452[30], 6}, {NULL}};
const Segment SGI251534[] = {{&S11686[13], 4}, {NULL}};
const Segment SGI251535[] = {{&S11686[94], 3}, {NULL}};
const Segment SGI251536[] = {{&S11686[26], 5}, {NULL}};
const Segment SGI251537[] = {{&S11686[59], 6}, {NULL}};
const Segment SGI251538[] = {{&S11686[105], 6}, {NULL}};
const Segment SGI251539[] = {{&S100452[80], 6}, {NULL}};
const Segment SGI251540[] = {{&S11686[119], 7}, {NULL}};
const Segment SGI251541[] = {{&S100452[107], 13}, {NULL}};
const Segment SGI251542[] = {{&S11687[13], 4}, {NULL}};
const Segment SGI251543[] = {{&S11687[93], 3}, {NULL}};
const Segment SGI251544[] = {{&S11687[58], 6}, {NULL}};
const Segment SGI251545[] = {{&S11687[104], 6}, {NULL}};
const Segment SGI251546[] = {{&S11687[118], 7}, {NULL}};
const Segment SGI251547[] = {{&S11688[13], 4}, {NULL}};
const Segment SGI251548[] = {{&S11688[94], 3}, {NULL}};
const Segment SGI251549[] = {{&S11688[26], 5}, {NULL}};
const Segment SGI251550[] = {{&S11688[59], 6}, {NULL}};
const Segment SGI251551[] = {{&S11688[105], 12}, {NULL}};
const Segment SGI251552[] = {{&S11688[118], 7}, {NULL}};
const Segment SGI251553[] = {{&S11689[13], 4}, {NULL}};
const Segment SGI251554[] = {{&S11689[94], 3}, {NULL}};
const Segment SGI251555[] = {{&S11689[26], 5}, {NULL}};
const Segment SGI251556[] = {{&S11689[59], 6}, {NULL}};
const Segment SGI251557[] = {{&S11689[105], 6}, {NULL}};
const Segment SGI251558[] = {{&S11689[119], 7}, {NULL}};
const Segment SGI251559[] = {{&S11690[13], 4}, {NULL}};
const Segment SGI251560[] = {{&S11690[92], 3}, {NULL}};
const Segment SGI251561[] = {{&S11690[26], 3}, {NULL}};
const Segment SGI251562[] = {{&S11690[57], 6}, {NULL}};
const Segment SGI251563[] = {{&S11690[103], 6}, {NULL}};
const Segment SGI251564[] = {{&S11690[117], 7}, {NULL}};
const Segment SGI251565[] = {{&S11691[13], 4}, {NULL}};
const Segment SGI251566[] = {{&S11691[92], 3}, {NULL}};
const Segment SGI251567[] = {{&S11691[26], 3}, {NULL}};
const Segment SGI251568[] = {{&S11691[57], 6}, {NULL}};
const Segment SGI251569[] = {{&S11691[103], 6}, {NULL}};
const Segment SGI251570[] = {{&S11691[117], 7}, {NULL}};
const Segment SGI251571[] = {{&S11692[13], 4}, {NULL}};
const Segment SGI251572[] = {{&S11692[94], 3}, {NULL}};
const Segment SGI251573[] = {{&S11692[26], 5}, {NULL}};
const Segment SGI251574[] = {{&S11692[59], 6}, {NULL}};
const Segment SGI251575[] = {{&S11692[105], 6}, {NULL}};
const Segment SGI251576[] = {{&S11692[119], 7}, {NULL}};
const Segment SGI251577[] = {{&S11693[13], 4}, {NULL}};
const Segment SGI251578[] = {{&S11693[94], 3}, {NULL}};
const Segment SGI251579[] = {{&S11693[26], 5}, {NULL}};
const Segment SGI251580[] = {{&S11693[59], 6}, {NULL}};
const Segment SGI251581[] = {{&S11693[105], 6}, {NULL}};
const Segment SGI251582[] = {{&S11693[119], 7}, {NULL}};
const Segment SGI251583[] = {{&S11694[13], 4}, {NULL}};
const Segment SGI251584[] = {{&S11694[94], 3}, {NULL}};
const Segment SGI251585[] = {{&S11694[26], 5}, {NULL}};
const Segment SGI251586[] = {{&S11694[59], 6}, {NULL}};
const Segment SGI251587[] = {{&S11694[105], 6}, {NULL}};
const Segment SGI251588[] = {{&S11694[119], 7}, {NULL}};
const Segment SGI251589[] = {{&S11695[13], 4}, {NULL}};
const Segment SGI251590[] = {{&S11695[94], 3}, {NULL}};
const Segment SGI251591[] = {{&S11695[26], 5}, {NULL}};
const Segment SGI251592[] = {{&S11695[59], 6}, {NULL}};
const Segment SGI251593[] = {{&S11695[105], 6}, {NULL}};
const Segment SGI251594[] = {{&S11695[119], 7}, {NULL}};
const Segment SGI251595[] = {{&S11696[13], 4}, {NULL}};
const Segment SGI251596[] = {{&S11696[94], 3}, {NULL}};
const Segment SGI251597[] = {{&S11696[26], 5}, {NULL}};
const Segment SGI251598[] = {{&S11696[59], 6}, {NULL}};
const Segment SGI251599[] = {{&S11696[105], 6}, {NULL}};
const Segment SGI251600[] = {{&S11696[119], 7}, {NULL}};
const Segment SGI251601[] = {{&S11697[13], 4}, {NULL}};
const Segment SGI251602[] = {{&S11697[92], 3}, {NULL}};
const Segment SGI251603[] = {{&S11697[26], 3}, {NULL}};
const Segment SGI251604[] = {{&S11697[57], 6}, {NULL}};
const Segment SGI251605[] = {{&S11697[103], 6}, {NULL}};
const Segment SGI251606[] = {{&S11697[117], 7}, {NULL}};
const Segment SGI251607[] = {{&S100488[44], 7}, {NULL}};
const Segment SGI251608[] = {{&S100488[44], 18}, {NULL}};
const Segment SGI251609[] = {{&S100488[30], 6}, {NULL}};
const Segment SGI251610[] = {{&S100488[76], 6}, {NULL}};
const Segment SGI251611[] = {{&S100488[103], 13}, {NULL}};
const Segment SGI251612[] = {{&S100496[44], 18}, {NULL}};
const Segment SGI251613[] = {{&S100496[44], 29}, {NULL}};
const Segment SGI251614[] = {{&S100496[30], 6}, {NULL}};
const Segment SGI251615[] = {{&S100496[122], 3}, {NULL}};
const Segment SGI251616[] = {{&S100496[87], 6}, {NULL}};
const Segment SGI251617[] = {{&S100496[157], 3}, {NULL}};
const Segment SGI251618[] = {{&S11720[12], 6}, {NULL}};
const Segment SGI251619[] = {{&S11720[76], 5}, {NULL}};
const Segment SGI251620[] = {{&S100502[44], 10}, {NULL}};
const Segment SGI251621[] = {{&S100502[44], 21}, {NULL}};
const Segment SGI251622[] = {{&S100502[30], 6}, {NULL}};
const Segment SGI251623[] = {{&S100502[114], 3}, {NULL}};
const Segment SGI251624[] = {{&S100502[79], 6}, {NULL}};
const Segment SGI251625[] = {{&S100502[149], 3}, {NULL}};
const Segment SGI251626[] = {{&S1386[25], 15}, {&S1386[41], 38}, {NULL}};
const Segment SGI251627[] = {{&S1386[0], 79}, {NULL}};
const Segment SGI251628[] = {{&S1387[11], 6}, {NULL}};
const Segment SGI251629[] = {{&S1387[0], 71}, {NULL}};
const Segment SGI251630[] = {{&S1388[25], 12}, {&S1388[40], 40}, {NULL}};
const Segment SGI251631[] = {{&S1388[0], 80}, {NULL}};
const Segment SGI251632[] = {{&S1389[39], 42}, {NULL}};
const Segment SGI251633[] = {{&S1389[0], 81}, {NULL}};
const Segment SGI251634[] = {{&S1390[25], 13}, {&S1390[39], 42}, {NULL}};
const Segment SGI251635[] = {{&S1390[0], 81}, {NULL}};
const Segment SGI251636[] = {{&S1391[0], 7}, {NULL}};
const Segment SGI251637[] = {{&S1391[0], 44}, {NULL}};
const Segment SGI251638[] = {{&S1392[14], 3}, {NULL}};
const Segment SGI251639[] = {{&S1392[0], 44}, {NULL}};
const Segment SGI251640[] = {{&S1393[6], 7}, {&S1393[14], 37}, {NULL}};
const Segment SGI251641[] = {{&S1393[0], 51}, {NULL}};
const Segment SGI251642[] = {{&S1394[16], 7}, {NULL}};
const Segment SGI251643[] = {{&S1394[0], 27}, {NULL}};
const Segment SGI251644[] = {{&S1395[7], 6}, {NULL}};
const Segment SGI251645[] = {{&S1395[0], 13}, {NULL}};
const Segment SGI251646[] = {{&S1396[0], 17}, {&S1396[18], 32}, {NULL}};
const Segment SGI251647[] = {{&S1396[0], 50}, {NULL}};
const Segment SGI251648[] = {{&S1397[0], 12}, {NULL}};
const Segment SGI251649[] = {{&S1397[0], 38}, {NULL}};
const Segment SGI251650[] = {{&S1398[16], 7}, {&S1398[30], 17}, {NULL}};
const Segment SGI251651[] = {{&S1398[0], 47}, {NULL}};
const Segment SGI251652[] = {{&S1399[0], 13}, {NULL}};
const Segment SGI251653[] = {{&S100555[44], 10}, {NULL}};
const Segment SGI251654[] = {{&S1400[0], 16}, {NULL}};
const Segment SGI251655[] = {{&S100555[44], 21}, {NULL}};
const Segment SGI251656[] = {{&S100555[30], 6}, {NULL}};
const Segment SGI251657[] = {{&S100555[146], 3}, {NULL}};
const Segment SGI251658[] = {{&S100555[111], 6}, {NULL}};
const Segment SGI251659[] = {{&S1401[0], 13}, {NULL}};
const Segment SGI251660[] = {{&S1401[0], 33}, {NULL}};
const Segment SGI251661[] = {{&S100555[74], 9}, {NULL}};
const Segment SGI251662[] = {{&S1402[25], 15}, {&S1402[41], 38}, {NULL}};
const Segment SGI251663[] = {{&S1402[0], 79}, {NULL}};
const Segment SGI251664[] = {{&S1403[25], 9}, {&S1403[35], 30}, {NULL}};
const Segment SGI251665[] = {{&S1403[0], 65}, {NULL}};
const Segment SGI251666[] = {{&S1404[16], 7}, {&S1404[35], 23}, {NULL}};
const Segment SGI251667[] = {{&S1404[0], 98}, {NULL}};
const Segment SGI251668[] = {{&S1405[0], 13}, {NULL}};
const Segment SGI251669[] = {{&S1405[0], 28}, {NULL}};
const Segment SGI251670[] = {{&S1406[0], 67}, {NULL}};
const Segment SGI251671[] = {{&S1407[25], 13}, {&S1407[39], 35}, {NULL}};
const Segment SGI251672[] = {{&S1407[0], 74}, {NULL}};
const Segment SGI251673[] = {{&S1408[30], 6}, {NULL}};
const Segment SGI251674[] = {{&S1408[0], 36}, {NULL}};
const Segment SGI251675[] = {{&S1409[0], 15}, {&S1409[16], 42}, {NULL}};
const Segment SGI251676[] = {{&S1409[0], 58}, {NULL}};
const Segment SGI251677[] = {{&S1410[40], 6}, {NULL}};
const Segment SGI251678[] = {{&S1410[0], 46}, {NULL}};
const Segment SGI251679[] = {{&S1411[0], 28}, {&S1411[29], 30}, {NULL}};
const Segment SGI251680[] = {{&S1411[0], 59}, {NULL}};
const Segment SGI251681[] = {{&S100567[34], 10}, {NULL}};
const Segment SGI251682[] = {{&S1412[25], 7}, {NULL}};
const Segment SGI251683[] = {{&S1412[0], 42}, {NULL}};
const Segment SGI251684[] = {{&S100567[27], 6}, {NULL}};
const Segment SGI251685[] = {{&S100567[96], 9}, {NULL}};
const Segment SGI251686[] = {{&S100567[69], 6}, {NULL}};
const Segment SGI251687[] = {{&S1413[0], 12}, {&S1413[52], 35}, {NULL}};
const Segment SGI251688[] = {{&S1413[0], 87}, {NULL}};
const Segment SGI251689[] = {{&S1414[25], 12}, {NULL}};
const Segment SGI251690[] = {{&S1414[0], 37}, {NULL}};
const Segment SGI251691[] = {{&S1415[0], 22}, {&S1415[25], 7}, {NULL}};
const Segment SGI251692[] = {{&S1415[0], 68}, {NULL}};
const Segment SGI251693[] = {{&S1416[0], 9}, {NULL}};
const Segment SGI251694[] = {{&S1417[25], 12}, {NULL}};
const Segment SGI251695[] = {{&S1417[0], 56}, {NULL}};
const Segment SGI251696[] = {{&S1418[13], 3}, {NULL}};
const Segment SGI251697[] = {{&S1418[0], 71}, {NULL}};
const Segment SGI251698[] = {{&S1419[11], 6}, {NULL}};
const Segment SGI251699[] = {{&S1419[0], 72}, {NULL}};
const Segment SGI251700[] = {{&S1420[25], 11}, {&S1420[37], 7}, {NULL}};
const Segment SGI251701[] = {{&S1420[0], 56}, {NULL}};
const Segment SGI251702[] = {{&S1421[0], 12}, {NULL}};
const Segment SGI251703[] = {{&S1422[22], 29}, {NULL}};
const Segment SGI251704[] = {{&S1422[0], 51}, {NULL}};
const Segment SGI251705[] = {{&S100578[44], 7}, {NULL}};
const Segment SGI251706[] = {{&S100578[44], 19}, {NULL}};
const Segment SGI251707[] = {{&S100578[30], 6}, {NULL}};
const Segment SGI251708[] = {{&S1423[0], 19}, {NULL}};
const Segment SGI251709[] = {{&S100578[112], 3}, {NULL}};
const Segment SGI251710[] = {{&S100578[77], 6}, {NULL}};
const Segment SGI251711[] = {{&S100578[137], 8}, {NULL}};
const Segment SGI251712[] = {{&S1424[0], 42}, {NULL}};
const Segment SGI251713[] = {{&S1425[25], 16}, {&S1425[42], 28}, {NULL}};
const Segment SGI251714[] = {{&S1425[0], 70}, {NULL}};
const Segment SGI251715[] = {{&S1426[0], 10}, {&S1426[11], 28}, {NULL}};
const Segment SGI251716[] = {{&S1426[0], 39}, {NULL}};
const Segment SGI251717[] = {{&S1427[25], 14}, {NULL}};
const Segment SGI251718[] = {{&S1427[0], 63}, {NULL}};
const Segment SGI251719[] = {{&S1428[16], 56}, {NULL}};
const Segment SGI251720[] = {{&S1428[0], 72}, {NULL}};
const Segment SGI251721[] = {{&S1429[0], 4}, {NULL}};
const Segment SGI251722[] = {{&S100583[44], 9}, {NULL}};
const Segment SGI251723[] = {{&S100583[44], 21}, {NULL}};
const Segment SGI251724[] = {{&S100583[30], 6}, {NULL}};
const Segment SGI251725[] = {{&S100583[114], 3}, {NULL}};
const Segment SGI251726[] = {{&S1430[0], 13}, {&S1430[14], 31}, {NULL}};
const Segment SGI251727[] = {{&S1430[0], 45}, {NULL}};
const Segment SGI251728[] = {{&S100583[79], 6}, {NULL}};
const Segment SGI251729[] = {{&S100583[139], 8}, {NULL}};
const Segment SGI251730[] = {{&S1431[0], 20}, {NULL}};
const Segment SGI251731[] = {{&S1432[0], 9}, {NULL}};
const Segment SGI251732[] = {{&S1433[0], 13}, {NULL}};
const Segment SGI251733[] = {{&S1434[25], 18}, {NULL}};
const Segment SGI251734[] = {{&S1434[0], 43}, {NULL}};
const Segment SGI251735[] = {{&S1435[25], 12}, {&S1435[38], 38}, {NULL}};
const Segment SGI251736[] = {{&S1435[0], 76}, {NULL}};
const Segment SGI251737[] = {{&S1436[25], 14}, {&S1436[40], 33}, {NULL}};
const Segment SGI251738[] = {{&S1436[0], 73}, {NULL}};
const Segment SGI251739[] = {{&S1437[25], 12}, {&S1437[38], 35}, {NULL}};
const Segment SGI251740[] = {{&S1437[0], 73}, {NULL}};
const Segment SGI251741[] = {{&S1438[19], 6}, {NULL}};
const Segment SGI251742[] = {{&S1438[0], 30}, {NULL}};
const Segment SGI251743[] = {{&S1439[3], 7}, {NULL}};
const Segment SGI251744[] = {{&S1439[0], 29}, {NULL}};
const Segment SGI251745[] = {{&S1440[25], 5}, {NULL}};
const Segment SGI251746[] = {{&S1440[0], 30}, {NULL}};
const Segment SGI251747[] = {{&S1441[0], 6}, {NULL}};
const Segment SGI251748[] = {{&S1441[0], 63}, {NULL}};
const Segment SGI251749[] = {{&S1442[16], 7}, {NULL}};
const Segment SGI251750[] = {{&S1442[0], 72}, {NULL}};
const Segment SGI251751[] = {{&S100594[44], 10}, {NULL}};
const Segment SGI251752[] = {{&S100594[44], 21}, {NULL}};
const Segment SGI251753[] = {{&S100594[30], 6}, {NULL}};
const Segment SGI251754[] = {{&S100594[114], 3}, {NULL}};
const Segment SGI251755[] = {{&S100594[79], 6}, {NULL}};
const Segment SGI251756[] = {{&S100594[139], 13}, {NULL}};
const Segment SGI251757[] = {{&S1443[36], 34}, {NULL}};
const Segment SGI251758[] = {{&S1443[0], 70}, {NULL}};
const Segment SGI251759[] = {{&S1444[16], 3}, {NULL}};
const Segment SGI251760[] = {{&S1444[0], 23}, {NULL}};
const Segment SGI251761[] = {{&S1445[25], 10}, {&S1445[36], 23}, {NULL}};
const Segment SGI251762[] = {{&S1445[0], 59}, {NULL}};
const Segment SGI251763[] = {{&S1446[0], 5}, {NULL}};
const Segment SGI251764[] = {{&S1446[0], 80}, {NULL}};
const Segment SGI251765[] = {{&S1447[0], 12}, {NULL}};
const Segment SGI251766[] = {{&S1447[0], 39}, {NULL}};
const Segment SGI251767[] = {{&S1448[25], 10}, {NULL}};
const Segment SGI251768[] = {{&S1448[0], 35}, {NULL}};
const Segment SGI251769[] = {{&S1449[0], 14}, {&S1449[15], 28}, {NULL}};
const Segment SGI251770[] = {{&S1449[0], 43}, {NULL}};
const Segment SGI251771[] = {{&S1450[25], 12}, {&S1450[38], 33}, {NULL}};
const Segment SGI251772[] = {{&S1450[0], 71}, {NULL}};
const Segment SGI251773[] = {{&S1451[25], 10}, {&S1451[27], 3}, {NULL}};
const Segment SGI251774[] = {{&S1451[0], 59}, {NULL}};
const Segment SGI251775[] = {{&S1452[25], 37}, {NULL}};
const Segment SGI251776[] = {{&S1452[0], 62}, {NULL}};
const Segment SGI251777[] = {{&S1453[0], 17}, {&S1453[18], 24}, {&S1453[43], 32}, {NULL}};
const Segment SGI251778[] = {{&S1453[0], 75}, {NULL}};
const Segment SGI251779[] = {{&S1454[0], 9}, {NULL}};
const Segment SGI251780[] = {{&S1454[0], 28}, {NULL}};
const Segment SGI251781[] = {{&S1455[10], 3}, {NULL}};
const Segment SGI251782[] = {{&S1455[0], 13}, {NULL}};
const Segment SGI251783[] = {{&S1456[25], 11}, {NULL}};
const Segment SGI251784[] = {{&S1456[0], 70}, {NULL}};
const Segment SGI251785[] = {{&S1457[25], 15}, {NULL}};
const Segment SGI251786[] = {{&S1457[0], 65}, {NULL}};
const Segment SGI251787[] = {{&S1458[6], 6}, {NULL}};
const Segment SGI251788[] = {{&S1458[0], 32}, {NULL}};
const Segment SGI251789[] = {{&S1459[12], 9}, {&S1459[27], 31}, {NULL}};
const Segment SGI251790[] = {{&S1459[0], 58}, {NULL}};
const Segment SGI251791[] = {{&S1460[0], 9}, {NULL}};
const Segment SGI251792[] = {{&S1461[0], 20}, {NULL}};
const Segment SGI251793[] = {{&S1462[0], 16}, {NULL}};
const Segment SGI251794[] = {{&S1463[5], 6}, {NULL}};
const Segment SGI251795[] = {{&S1463[0], 11}, {NULL}};
const Segment SGI251796[] = {{&S1464[0], 7}, {NULL}};
const Segment SGI251797[] = {{&S1465[25], 16}, {&S1465[42], 30}, {NULL}};
const Segment SGI251798[] = {{&S1465[0], 72}, {NULL}};
const Segment SGI251799[] = {{&S1466[0], 18}, {&S1466[19], 41}, {NULL}};
const Segment SGI251800[] = {{&S1466[0], 60}, {NULL}};
const Segment SGI251801[] = {{&S1467[32], 3}, {NULL}};
const Segment SGI251802[] = {{&S1467[0], 59}, {NULL}};
const Segment SGI251803[] = {{&S1468[50], 24}, {NULL}};
const Segment SGI251804[] = {{&S1468[0], 74}, {NULL}};
const Segment SGI251805[] = {{&S1469[0], 9}, {NULL}};
const Segment SGI251806[] = {{&S100620[44], 11}, {NULL}};
const Segment SGI251807[] = {{&S100620[44], 22}, {NULL}};
const Segment SGI251808[] = {{&S100620[30], 6}, {NULL}};
const Segment SGI251809[] = {{&S100620[115], 3}, {NULL}};
const Segment SGI251810[] = {{&S1470[0], 3}, {NULL}};
const Segment SGI251811[] = {{&S1470[0], 30}, {NULL}};
const Segment SGI251812[] = {{&S100620[80], 6}, {NULL}};
const Segment SGI251813[] = {{&S1471[25], 10}, {NULL}};
const Segment SGI251814[] = {{&S1471[0], 37}, {NULL}};
const Segment SGI251815[] = {{&S1472[25], 13}, {NULL}};
const Segment SGI251816[] = {{&S1472[0], 62}, {NULL}};
const Segment SGI251817[] = {{&S1473[3], 7}, {NULL}};
const Segment SGI251818[] = {{&S1473[0], 63}, {NULL}};
const Segment SGI251819[] = {{&S1474[25], 16}, {NULL}};
const Segment SGI251820[] = {{&S1474[0], 65}, {NULL}};
const Segment SGI251821[] = {{&S1475[35], 6}, {NULL}};
const Segment SGI251822[] = {{&S1475[0], 60}, {NULL}};
const Segment SGI251823[] = {{&S1476[11], 32}, {NULL}};
const Segment SGI251824[] = {{&S1476[0], 43}, {NULL}};
const Segment SGI251825[] = {{&S1477[27], 37}, {NULL}};
const Segment SGI251826[] = {{&S1477[0], 64}, {NULL}};
const Segment SGI251827[] = {{&S1478[10], 31}, {NULL}};
const Segment SGI251828[] = {{&S1478[0], 41}, {NULL}};
const Segment SGI251829[] = {{&S100630[59], 12}, {NULL}};
const Segment SGI251830[] = {{&S100630[59], 23}, {NULL}};
const Segment SGI251831[] = {{&S100630[45], 6}, {NULL}};
const Segment SGI251832[] = {{&S1479[0], 14}, {NULL}};
const Segment SGI251833[] = {{&S100630[131], 3}, {NULL}};
const Segment SGI251834[] = {{&S100630[96], 6}, {NULL}};
const Segment SGI251835[] = {{&S1480[0], 19}, {NULL}};
const Segment SGI251836[] = {{&S1480[0], 61}, {NULL}};
const Segment SGI251837[] = {{&S1481[0], 13}, {NULL}};
const Segment SGI251838[] = {{&S1482[26], 6}, {NULL}};
const Segment SGI251839[] = {{&S1482[0], 65}, {NULL}};
const Segment SGI251840[] = {{&S1484[25], 15}, {NULL}};
const Segment SGI251841[] = {{&S1484[0], 65}, {NULL}};
const Segment SGI251842[] = {{&S1485[0], 19}, {&S1485[20], 35}, {NULL}};
const Segment SGI251843[] = {{&S1485[0], 55}, {NULL}};
const Segment SGI251844[] = {{&S1486[11], 7}, {NULL}};
const Segment SGI251845[] = {{&S1486[0], 18}, {NULL}};
const Segment SGI251846[] = {{&S1487[0], 16}, {&S1487[17], 35}, {NULL}};
const Segment SGI251847[] = {{&S1487[0], 52}, {NULL}};
const Segment SGI251848[] = {{&S1488[0], 14}, {NULL}};
const Segment SGI251849[] = {{&S1489[0], 16}, {&S1489[17], 35}, {NULL}};
const Segment SGI251850[] = {{&S1489[0], 52}, {NULL}};
const Segment SGI251851[] = {{&S1490[0], 7}, {NULL}};
const Segment SGI251852[] = {{&S1491[16], 40}, {NULL}};
const Segment SGI251853[] = {{&S1491[0], 56}, {NULL}};
const Segment SGI251854[] = {{&S1492[0], 12}, {NULL}};
const Segment SGI251855[] = {{&S1493[0], 13}, {NULL}};
const Segment SGI251856[] = {{&S1494[22], 7}, {NULL}};
const Segment SGI251857[] = {{&S1494[0], 29}, {NULL}};
const Segment SGI251858[] = {{&S1495[0], 58}, {NULL}};
const Segment SGI251859[] = {{&S1496[5], 7}, {NULL}};
const Segment SGI251860[] = {{&S1496[0], 103}, {NULL}};
const Segment SGI251861[] = {{&S1497[0], 16}, {NULL}};
const Segment SGI251862[] = {{&S1497[0], 86}, {NULL}};
const Segment SGI251863[] = {{&S1498[0], 17}, {NULL}};
const Segment SGI251864[] = {{&S1498[0], 109}, {NULL}};
const Segment SGI251865[] = {{&S100652[34], 15}, {NULL}};
const Segment SGI251866[] = {{&S1499[0], 10}, {&S1499[11], 7}, {&S1499[28], 23}, {NULL}};
const Segment SGI251867[] = {{&S1499[0], 51}, {NULL}};
const Segment SGI251868[] = {{&S100652[27], 6}, {NULL}};
const Segment SGI251869[] = {{&S100652[101], 9}, {NULL}};
const Segment SGI251870[] = {{&S1500[0], 9}, {NULL}};
const Segment SGI251871[] = {{&S100652[74], 6}, {NULL}};
const Segment SGI251872[] = {{&S1501[44], 24}, {NULL}};
const Segment SGI251873[] = {{&S1501[0], 68}, {NULL}};
const Segment SGI251874[] = {{&S100654[44], 11}, {NULL}};
const Segment SGI251875[] = {{&S1502[14], 35}, {NULL}};
const Segment SGI251876[] = {{&S1502[0], 49}, {NULL}};
const Segment SGI251877[] = {{&S100654[44], 23}, {NULL}};
const Segment SGI251878[] = {{&S100654[30], 6}, {NULL}};
const Segment SGI251879[] = {{&S100654[116], 3}, {NULL}};
const Segment SGI251880[] = {{&S1503[0], 16}, {NULL}};
const Segment SGI251881[] = {{&S100654[81], 6}, {NULL}};
const Segment SGI251882[] = {{&S1504[0], 9}, {NULL}};
const Segment SGI251883[] = {{&S1504[0], 33}, {NULL}};
const Segment SGI251884[] = {{&S1505[0], 7}, {&S1505[8], 7}, {&S1505[31], 31}, {NULL}};
const Segment SGI251885[] = {{&S1505[0], 62}, {NULL}};
const Segment SGI251886[] = {{&S1506[0], 17}, {NULL}};
const Segment SGI251887[] = {{&S1506[0], 45}, {NULL}};
const Segment SGI251888[] = {{&S1507[0], 19}, {NULL}};
const Segment SGI251889[] = {{&S1508[0], 16}, {NULL}};
const Segment SGI251890[] = {{&S1509[25], 14}, {NULL}};
const Segment SGI251891[] = {{&S1509[0], 61}, {NULL}};
const Segment SGI251892[] = {{&S1510[0], 17}, {&S1510[18], 36}, {NULL}};
const Segment SGI251893[] = {{&S1510[0], 54}, {NULL}};
const Segment SGI251894[] = {{&S1511[0], 18}, {&S1511[19], 35}, {NULL}};
const Segment SGI251895[] = {{&S1511[0], 54}, {NULL}};
const Segment SGI251896[] = {{&S1512[6], 6}, {NULL}};
const Segment SGI251897[] = {{&S1512[0], 31}, {NULL}};
const Segment SGI251898[] = {{&S1513[0], 4}, {&S1513[22], 22}, {NULL}};
const Segment SGI251899[] = {{&S1513[0], 44}, {NULL}};
const Segment SGI251900[] = {{&S1514[12], 3}, {NULL}};
const Segment SGI251901[] = {{&S1514[0], 15}, {NULL}};
const Segment SGI251902[] = {{&S1515[19], 6}, {NULL}};
const Segment SGI251903[] = {{&S1515[0], 30}, {NULL}};
const Segment SGI251904[] = {{&S1516[0], 17}, {NULL}};
const Segment SGI251905[] = {{&S1516[0], 65}, {NULL}};
const Segment SGI251906[] = {{&S1517[0], 13}, {&S1517[14], 15}, {NULL}};
const Segment SGI251907[] = {{&S1517[0], 52}, {NULL}};
const Segment SGI251908[] = {{&S1518[0], 10}, {NULL}};
const Segment SGI251909[] = {{&S1518[0], 17}, {NULL}};
const Segment SGI251910[] = {{&S1519[0], 18}, {NULL}};
const Segment SGI251911[] = {{&S1520[25], 18}, {NULL}};
const Segment SGI251912[] = {{&S1520[0], 70}, {NULL}};
const Segment SGI251913[] = {{&S1521[0], 12}, {NULL}};
const Segment SGI251914[] = {{&S1521[0], 16}, {NULL}};
const Segment SGI251915[] = {{&S1522[42], 7}, {&S1522[55], 62}, {NULL}};
const Segment SGI251916[] = {{&S1522[0], 117}, {NULL}};
const Segment SGI251917[] = {{&S1523[0], 9}, {NULL}};
const Segment SGI251918[] = {{&S1524[0], 22}, {NULL}};
const Segment SGI251919[] = {{&S1524[0], 69}, {NULL}};
const Segment SGI251920[] = {{&S1525[0], 13}, {NULL}};
const Segment SGI251921[] = {{&S1526[9], 11}, {NULL}};
const Segment SGI251922[] = {{&S1526[0], 93}, {NULL}};
const Segment SGI251923[] = {{&S1527[0], 15}, {NULL}};
const Segment SGI251924[] = {{&S1528[0], 73}, {NULL}};
const Segment SGI251925[] = {{&S100681[44], 10}, {NULL}};
const Segment SGI251926[] = {{&S100681[44], 21}, {NULL}};
const Segment SGI251927[] = {{&S100681[30], 6}, {NULL}};
const Segment SGI251928[] = {{&S100681[79], 6}, {NULL}};
const Segment SGI251929[] = {{&S1529[46], 24}, {NULL}};
const Segment SGI251930[] = {{&S1529[0], 70}, {NULL}};
const Segment SGI251931[] = {{&S100681[106], 13}, {NULL}};
const Segment SGI251932[] = {{&S1530[10], 3}, {NULL}};
const Segment SGI251933[] = {{&S1530[0], 49}, {NULL}};
const Segment SGI251934[] = {{&S1531[0], 16}, {NULL}};
const Segment SGI251935[] = {{&S1531[0], 84}, {NULL}};
const Segment SGI251936[] = {{&S100686[34], 6}, {NULL}};
const Segment SGI251937[] = {{&S100686[27], 6}, {NULL}};
const Segment SGI251938[] = {{&S100686[94], 9}, {NULL}};
const Segment SGI251939[] = {{&S1532[43], 6}, {&S1532[53], 24}, {NULL}};
const Segment SGI251940[] = {{&S100686[67], 6}, {NULL}};
const Segment SGI251941[] = {{&S1532[0], 87}, {NULL}};
const Segment SGI251942[] = {{&S1533[25], 19}, {&S1533[45], 32}, {NULL}};
const Segment SGI251943[] = {{&S1533[0], 77}, {NULL}};
const Segment SGI251944[] = {{&S100687[44], 15}, {NULL}};
const Segment SGI251945[] = {{&S100687[44], 33}, {NULL}};
const Segment SGI251946[] = {{&S100687[30], 6}, {NULL}};
const Segment SGI251947[] = {{&S1534[3], 7}, {NULL}};
const Segment SGI251948[] = {{&S1534[0], 10}, {NULL}};
const Segment SGI251949[] = {{&S100687[91], 6}, {NULL}};
const Segment SGI251950[] = {{&S100687[118], 13}, {NULL}};
const Segment SGI251951[] = {{&S1535[16], 7}, {NULL}};
const Segment SGI251952[] = {{&S1535[0], 73}, {NULL}};
const Segment SGI251953[] = {{&S1536[43], 13}, {NULL}};
const Segment SGI251954[] = {{&S1536[0], 56}, {NULL}};
const Segment SGI251955[] = {{&S1537[0], 11}, {NULL}};
const Segment SGI251956[] = {{&S1537[0], 36}, {NULL}};
const Segment SGI251957[] = {{&S1538[0], 13}, {&S1538[14], 41}, {NULL}};
const Segment SGI251958[] = {{&S1538[0], 55}, {NULL}};
const Segment SGI251959[] = {{&S1539[0], 10}, {NULL}};
const Segment SGI251960[] = {{&S100694[44], 14}, {NULL}};
const Segment SGI251961[] = {{&S1540[0], 16}, {NULL}};
const Segment SGI251962[] = {{&S1540[0], 76}, {NULL}};
const Segment SGI251963[] = {{&S100694[44], 25}, {NULL}};
const Segment SGI251964[] = {{&S100694[30], 6}, {NULL}};
const Segment SGI251965[] = {{&S100694[83], 6}, {NULL}};
const Segment SGI251966[] = {{&S100694[110], 13}, {NULL}};
const Segment SGI251967[] = {{&S1541[15], 3}, {&S1541[44], 12}, {&S1541[105], 3}, {NULL}};
const Segment SGI251968[] = {{&S1541[0], 120}, {NULL}};
const Segment SGI251969[] = {{&S1542[25], 15}, {NULL}};
const Segment SGI251970[] = {{&S1542[0], 40}, {NULL}};
const Segment SGI251971[] = {{&S1543[13], 34}, {NULL}};
const Segment SGI251972[] = {{&S1543[0], 47}, {NULL}};
const Segment SGI251973[] = {{&S100697[34], 10}, {NULL}};
const Segment SGI251974[] = {{&S100697[27], 6}, {NULL}};
const Segment SGI251975[] = {{&S1544[7], 3}, {NULL}};
const Segment SGI251976[] = {{&S1544[0], 10}, {NULL}};
const Segment SGI251977[] = {{&S100697[96], 9}, {NULL}};
const Segment SGI251978[] = {{&S100697[69], 6}, {NULL}};
const Segment SGI251979[] = {{&S1545[25], 10}, {&S1545[41], 27}, {NULL}};
const Segment SGI251980[] = {{&S1545[0], 68}, {NULL}};
const Segment SGI251981[] = {{&S1546[25], 13}, {&S1546[39], 31}, {NULL}};
const Segment SGI251982[] = {{&S1546[0], 70}, {NULL}};
const Segment SGI251983[] = {{&S1547[0], 14}, {NULL}};
const Segment SGI251984[] = {{&S1547[0], 49}, {NULL}};
const Segment SGI251985[] = {{&S1548[0], 5}, {NULL}};
const Segment SGI251986[] = {{&S1548[0], 27}, {NULL}};
const Segment SGI251987[] = {{&S1549[34], 44}, {NULL}};
const Segment SGI251988[] = {{&S1549[0], 78}, {NULL}};
const Segment SGI251989[] = {{&S1550[38], 22}, {NULL}};
const Segment SGI251990[] = {{&S1550[0], 60}, {NULL}};
const Segment SGI251991[] = {{&S1551[0], 16}, {&S1551[17], 36}, {NULL}};
const Segment SGI251992[] = {{&S1551[0], 53}, {NULL}};
const Segment SGI251993[] = {{&S100705[34], 9}, {NULL}};
const Segment SGI251994[] = {{&S1552[0], 12}, {&S1552[13], 23}, {NULL}};
const Segment SGI251995[] = {{&S1552[0], 36}, {NULL}};
const Segment SGI251996[] = {{&S100705[27], 6}, {NULL}};
const Segment SGI251997[] = {{&S100705[96], 9}, {NULL}};
const Segment SGI251998[] = {{&S100705[69], 6}, {NULL}};
const Segment SGI251999[] = {{&S1553[48], 48}, {NULL}};
const Segment SGI252000[] = {{&S1553[0], 96}, {NULL}};
const Segment SGI252001[] = {{&S1554[25], 26}, {NULL}};
const Segment SGI252002[] = {{&S1554[0], 87}, {NULL}};
const Segment SGI252003[] = {{&S1555[0], 8}, {NULL}};
const Segment SGI252004[] = {{&S1556[14], 3}, {&S1556[35], 17}, {NULL}};
const Segment SGI252005[] = {{&S1556[0], 52}, {NULL}};
const Segment SGI252006[] = {{&S1557[0], 15}, {NULL}};
const Segment SGI252007[] = {{&S1557[0], 37}, {NULL}};
const Segment SGI252008[] = {{&S1558[0], 17}, {NULL}};
const Segment SGI252009[] = {{&S100713[34], 7}, {NULL}};
const Segment SGI252010[] = {{&S100713[27], 6}, {NULL}};
const Segment SGI252011[] = {{&S100713[93], 9}, {NULL}};
const Segment SGI252012[] = {{&S100713[66], 6}, {NULL}};
const Segment SGI252013[] = {{&S1562[25], 16}, {NULL}};
const Segment SGI252014[] = {{&S1562[0], 93}, {NULL}};
const Segment SGI252015[] = {{&S1563[20], 11}, {NULL}};
const Segment SGI252016[] = {{&S1563[0], 35}, {NULL}};
const Segment SGI252017[] = {{&S100717[34], 9}, {NULL}};
const Segment SGI252018[] = {{&S1564[30], 3}, {NULL}};
const Segment SGI252019[] = {{&S100717[27], 6}, {NULL}};
const Segment SGI252020[] = {{&S1564[0], 39}, {NULL}};
const Segment SGI252021[] = {{&S100717[95], 9}, {NULL}};
const Segment SGI252022[] = {{&S100717[68], 6}, {NULL}};
const Segment SGI252023[] = {{&S1565[25], 14}, {&S1565[40], 33}, {NULL}};
const Segment SGI252024[] = {{&S1565[0], 73}, {NULL}};
const Segment SGI252025[] = {{&S1567[0], 12}, {&S1567[13], 35}, {NULL}};
const Segment SGI252026[] = {{&S1567[0], 48}, {NULL}};
const Segment SGI252027[] = {{&S1568[25], 15}, {&S1568[41], 38}, {NULL}};
const Segment SGI252028[] = {{&S1568[0], 79}, {NULL}};
const Segment SGI252029[] = {{&S1569[9], 3}, {NULL}};
const Segment SGI252030[] = {{&S1569[0], 12}, {NULL}};
const Segment SGI252031[] = {{&S1570[0], 14}, {&S1570[15], 63}, {NULL}};
const Segment SGI252032[] = {{&S1570[0], 78}, {NULL}};
const Segment SGI252033[] = {{&S1571[0], 7}, {&S1571[13], 18}, {NULL}};
const Segment SGI252034[] = {{&S1571[0], 31}, {NULL}};
const Segment SGI252035[] = {{&S1573[0], 13}, {NULL}};
const Segment SGI252036[] = {{&S1574[12], 7}, {&S1574[20], 37}, {NULL}};
const Segment SGI252037[] = {{&S1574[0], 57}, {NULL}};
const Segment SGI252038[] = {{&S1575[11], 3}, {NULL}};
const Segment SGI252039[] = {{&S1575[0], 14}, {NULL}};
const Segment SGI252040[] = {{&S1576[0], 9}, {&S1576[44], 9}, {&S1576[58], 30}, {NULL}};
const Segment SGI252041[] = {{&S1576[0], 88}, {NULL}};
const Segment SGI252042[] = {{&S1577[25], 21}, {&S1577[47], 48}, {NULL}};
const Segment SGI252043[] = {{&S1577[0], 95}, {NULL}};
const Segment SGI252044[] = {{&S1578[17], 28}, {NULL}};
const Segment SGI252045[] = {{&S1578[0], 45}, {NULL}};
const Segment SGI252046[] = {{&S1579[8], 3}, {NULL}};
const Segment SGI252047[] = {{&S1579[0], 11}, {NULL}};
const Segment SGI252048[] = {{&S1580[0], 19}, {NULL}};
const Segment SGI252049[] = {{&S1580[0], 54}, {NULL}};
const Segment SGI252050[] = {{&S1581[0], 12}, {NULL}};
const Segment SGI252051[] = {{&S1582[0], 15}, {&S1582[16], 36}, {NULL}};
const Segment SGI252052[] = {{&S1582[0], 52}, {NULL}};
const Segment SGI252053[] = {{&S1583[0], 41}, {NULL}};
const Segment SGI252054[] = {{&S1584[25], 14}, {&S1584[56], 24}, {NULL}};
const Segment SGI252055[] = {{&S1584[0], 80}, {NULL}};
const Segment SGI252056[] = {{&S1585[25], 18}, {&S1585[44], 33}, {NULL}};
const Segment SGI252057[] = {{&S1585[0], 103}, {NULL}};
const Segment SGI252058[] = {{&S1586[0], 56}, {NULL}};
const Segment SGI252059[] = {{&S1587[0], 83}, {NULL}};
const Segment SGI252060[] = {{&S1588[0], 12}, {&S1588[13], 28}, {NULL}};
const Segment SGI252061[] = {{&S1588[0], 41}, {NULL}};
const Segment SGI252062[] = {{&S1589[42], 24}, {NULL}};
const Segment SGI252063[] = {{&S1589[0], 66}, {NULL}};
const Segment SGI252064[] = {{&S1590[43], 6}, {&S1590[53], 24}, {NULL}};
const Segment SGI252065[] = {{&S1590[0], 87}, {NULL}};
const Segment SGI252066[] = {{&S1591[0], 11}, {NULL}};
const Segment SGI252067[] = {{&S1592[0], 10}, {NULL}};
const Segment SGI252068[] = {{&S1593[25], 20}, {&S1593[46], 44}, {NULL}};
const Segment SGI252069[] = {{&S1593[0], 90}, {NULL}};
const Segment SGI252070[] = {{&S1594[0], 12}, {NULL}};
const Segment SGI252071[] = {{&S1595[7], 6}, {&S1595[19], 5}, {&S1595[27], 25}, {NULL}};
const Segment SGI252072[] = {{&S1595[0], 52}, {NULL}};
const Segment SGI252073[] = {{&S1596[0], 5}, {NULL}};
const Segment SGI252074[] = {{&S1596[0], 19}, {NULL}};
const Segment SGI252075[] = {{&S1597[13], 34}, {NULL}};
const Segment SGI252076[] = {{&S1597[0], 47}, {NULL}};
const Segment SGI252077[] = {{&S1598[0], 7}, {NULL}};
const Segment SGI252078[] = {{&S1599[25], 24}, {&S1599[50], 33}, {NULL}};
const Segment SGI252079[] = {{&S1599[0], 83}, {NULL}};
const Segment SGI252080[] = {{&S1600[10], 7}, {NULL}};
const Segment SGI252081[] = {{&S1600[0], 17}, {NULL}};
const Segment SGI252082[] = {{&S1601[25], 14}, {&S1601[40], 38}, {NULL}};
const Segment SGI252083[] = {{&S1601[0], 78}, {NULL}};
const Segment SGI252084[] = {{&S1602[27], 10}, {&S1602[38], 6}, {NULL}};
const Segment SGI252085[] = {{&S1602[0], 85}, {NULL}};
const Segment SGI252086[] = {{&S1603[25], 7}, {&S1603[35], 41}, {NULL}};
const Segment SGI252087[] = {{&S1603[0], 76}, {NULL}};
const Segment SGI252088[] = {{&S1604[0], 11}, {NULL}};
const Segment SGI252089[] = {{&S1605[0], 10}, {NULL}};
const Segment SGI252090[] = {{&S1606[12], 20}, {&S1606[33], 35}, {NULL}};
const Segment SGI252091[] = {{&S1606[0], 68}, {NULL}};
const Segment SGI252092[] = {{&S1607[12], 12}, {NULL}};
const Segment SGI252093[] = {{&S1607[0], 24}, {NULL}};
const Segment SGI252094[] = {{&S1608[25], 13}, {&S1608[39], 48}, {NULL}};
const Segment SGI252095[] = {{&S1608[0], 87}, {NULL}};
const Segment SGI252096[] = {{&S1609[0], 47}, {NULL}};
const Segment SGI252097[] = {{&S1610[25], 11}, {&S1610[67], 7}, {NULL}};
const Segment SGI252098[] = {{&S1610[0], 92}, {NULL}};
const Segment SGI252099[] = {{&S100935[34], 9}, {NULL}};
const Segment SGI252100[] = {{&S100935[27], 6}, {NULL}};
const Segment SGI252101[] = {{&S100935[96], 9}, {NULL}};
const Segment SGI252102[] = {{&S100935[69], 6}, {NULL}};
const Segment SGI252103[] = {{&S1611[0], 12}, {&S1611[13], 20}, {NULL}};
const Segment SGI252104[] = {{&S1611[0], 33}, {NULL}};
const Segment SGI252105[] = {{&S1612[25], 10}, {&S1612[52], 32}, {NULL}};
const Segment SGI252106[] = {{&S1612[0], 84}, {NULL}};
const Segment SGI252107[] = {{&S1613[0], 23}, {&S1613[35], 7}, {NULL}};
const Segment SGI252108[] = {{&S1613[0], 43}, {NULL}};
const Segment SGI252109[] = {{&S1614[0], 11}, {&S1614[12], 35}, {NULL}};
const Segment SGI252110[] = {{&S1614[0], 52}, {NULL}};
const Segment SGI252111[] = {{&S1615[25], 10}, {NULL}};
const Segment SGI252112[] = {{&S1615[0], 74}, {NULL}};
const Segment SGI252113[] = {{&S1616[0], 10}, {NULL}};
const Segment SGI252114[] = {{&S1616[0], 42}, {NULL}};
const Segment SGI252115[] = {{&S1617[0], 10}, {&S1617[11], 35}, {NULL}};
const Segment SGI252116[] = {{&S1617[0], 46}, {NULL}};
const Segment SGI252117[] = {{&S1618[8], 3}, {&S1618[12], 28}, {&S1618[36], 3}, {NULL}};
const Segment SGI252118[] = {{&S1618[0], 57}, {NULL}};
const Segment SGI252119[] = {{&S1619[25], 10}, {NULL}};
const Segment SGI252120[] = {{&S1619[0], 60}, {NULL}};
const Segment SGI252121[] = {{&S1620[25], 16}, {&S1620[42], 41}, {NULL}};
const Segment SGI252122[] = {{&S1620[0], 109}, {NULL}};
const Segment SGI252123[] = {{&S1621[25], 16}, {&S1621[42], 41}, {NULL}};
const Segment SGI252124[] = {{&S1621[0], 154}, {NULL}};
const Segment SGI252125[] = {{&S1622[0], 19}, {&S1622[20], 44}, {NULL}};
const Segment SGI252126[] = {{&S1622[0], 64}, {NULL}};
const Segment SGI252127[] = {{&S1623[32], 7}, {NULL}};
const Segment SGI252128[] = {{&S1623[0], 63}, {NULL}};
const Segment SGI252129[] = {{&S1624[25], 33}, {NULL}};
const Segment SGI252130[] = {{&S1624[0], 60}, {NULL}};
const Segment SGI252131[] = {{&S1625[0], 8}, {NULL}};
const Segment SGI252132[] = {{&S1626[66], 61}, {NULL}};
const Segment SGI252133[] = {{&S1626[0], 127}, {NULL}};
const Segment SGI252134[] = {{&S1627[0], 15}, {NULL}};
const Segment SGI252135[] = {{&S1628[10], 26}, {NULL}};
const Segment SGI252136[] = {{&S1628[0], 66}, {NULL}};
const Segment SGI252137[] = {{&S1629[6], 7}, {NULL}};
const Segment SGI252138[] = {{&S1629[0], 13}, {NULL}};
const Segment SGI252139[] = {{&S1630[25], 11}, {NULL}};
const Segment SGI252140[] = {{&S1630[0], 66}, {NULL}};
const Segment SGI252141[] = {{&S1631[0], 10}, {NULL}};
const Segment SGI252142[] = {{&S1631[0], 40}, {NULL}};
const Segment SGI252143[] = {{&S1632[19], 6}, {NULL}};
const Segment SGI252144[] = {{&S1632[0], 30}, {NULL}};
const Segment SGI252145[] = {{&S1633[19], 6}, {NULL}};
const Segment SGI252146[] = {{&S1633[0], 30}, {NULL}};
const Segment SGI252147[] = {{&S1634[0], 7}, {NULL}};
const Segment SGI252148[] = {{&S1634[0], 10}, {NULL}};
const Segment SGI252149[] = {{&S1635[0], 19}, {NULL}};
const Segment SGI252150[] = {{&S1636[7], 6}, {&S1636[19], 5}, {&S1636[27], 25}, {NULL}};
const Segment SGI252151[] = {{&S1636[0], 52}, {NULL}};
const Segment SGI252152[] = {{&S1637[10], 7}, {NULL}};
const Segment SGI252153[] = {{&S1637[0], 22}, {NULL}};
const Segment SGI252154[] = {{&S1639[0], 11}, {NULL}};
const Segment SGI252155[] = {{&S1639[0], 58}, {NULL}};
const Segment SGI252156[] = {{&S1640[0], 20}, {NULL}};
const Segment SGI252157[] = {{&S1641[0], 8}, {&S1641[9], 47}, {NULL}};
const Segment SGI252158[] = {{&S1641[0], 56}, {NULL}};
const Segment SGI252159[] = {{&S1642[16], 7}, {NULL}};
const Segment SGI252160[] = {{&S1642[0], 55}, {NULL}};
const Segment SGI252161[] = {{&S1643[28], 34}, {NULL}};
const Segment SGI252162[] = {{&S1643[0], 111}, {NULL}};
const Segment SGI252163[] = {{&S1644[0], 3}, {&S1644[16], 30}, {NULL}};
const Segment SGI252164[] = {{&S1644[0], 46}, {NULL}};
const Segment SGI252165[] = {{&S1645[0], 18}, {NULL}};
const Segment SGI252166[] = {{&S1645[0], 66}, {NULL}};
const Segment SGI252167[] = {{&S1646[19], 6}, {NULL}};
const Segment SGI252168[] = {{&S1646[0], 30}, {NULL}};
const Segment SGI252169[] = {{&S1647[25], 17}, {&S1647[43], 32}, {NULL}};
const Segment SGI252170[] = {{&S1647[0], 75}, {NULL}};
const Segment SGI252171[] = {{&S1648[25], 11}, {&S1648[37], 41}, {NULL}};
const Segment SGI252172[] = {{&S1648[0], 78}, {NULL}};
const Segment SGI252173[] = {{&S1649[0], 10}, {NULL}};
const Segment SGI252174[] = {{&S1649[0], 38}, {NULL}};
const Segment SGI252175[] = {{&S1650[39], 28}, {NULL}};
const Segment SGI252176[] = {{&S1650[0], 67}, {NULL}};
const Segment SGI252177[] = {{&S1651[34], 3}, {NULL}};
const Segment SGI252178[] = {{&S1651[0], 37}, {NULL}};
const Segment SGI252179[] = {{&S1652[0], 17}, {NULL}};
const Segment SGI252180[] = {{&S1653[25], 15}, {NULL}};
const Segment SGI252181[] = {{&S1653[0], 40}, {NULL}};
const Segment SGI252182[] = {{&S1654[11], 27}, {NULL}};
const Segment SGI252183[] = {{&S1654[0], 38}, {NULL}};
const Segment SGI252184[] = {{&S1655[5], 6}, {NULL}};
const Segment SGI252185[] = {{&S1655[0], 11}, {NULL}};
const Segment SGI252186[] = {{&S1656[0], 14}, {&S1656[15], 38}, {NULL}};
const Segment SGI252187[] = {{&S1656[0], 53}, {NULL}};
const Segment SGI252188[] = {{&S1657[35], 35}, {NULL}};
const Segment SGI252189[] = {{&S1657[0], 78}, {NULL}};
const Segment SGI252190[] = {{&S1658[24], 3}, {&S1658[37], 28}, {NULL}};
const Segment SGI252191[] = {{&S1658[0], 65}, {NULL}};
const Segment SGI252192[] = {{&S1659[0], 3}, {NULL}};
const Segment SGI252193[] = {{&S1659[0], 7}, {NULL}};
const Segment SGI252194[] = {{&S1660[25], 13}, {&S1660[39], 40}, {NULL}};
const Segment SGI252195[] = {{&S1660[0], 79}, {NULL}};
const Segment SGI252196[] = {{&S1661[25], 12}, {&S1661[38], 31}, {NULL}};
const Segment SGI252197[] = {{&S1661[0], 71}, {NULL}};
const Segment SGI252198[] = {{&S1662[0], 17}, {NULL}};
const Segment SGI252199[] = {{&S1663[31], 14}, {NULL}};
const Segment SGI252200[] = {{&S1663[0], 89}, {NULL}};
const Segment SGI252201[] = {{&S1664[22], 40}, {NULL}};
const Segment SGI252202[] = {{&S1664[0], 82}, {NULL}};
const Segment SGI252203[] = {{&S1665[25], 16}, {NULL}};
const Segment SGI252204[] = {{&S1665[0], 81}, {NULL}};
const Segment SGI252205[] = {{&S1666[0], 6}, {NULL}};
const Segment SGI252206[] = {{&S1667[35], 28}, {NULL}};
const Segment SGI252207[] = {{&S1667[0], 63}, {NULL}};
const Segment SGI252208[] = {{&S1668[0], 17}, {NULL}};
const Segment SGI252209[] = {{&S1669[39], 37}, {&S1669[60], 7}, {NULL}};
const Segment SGI252210[] = {{&S1669[0], 98}, {NULL}};
const Segment SGI252211[] = {{&S1670[0], 12}, {&S1670[37], 28}, {NULL}};
const Segment SGI252212[] = {{&S1670[0], 65}, {NULL}};
const Segment SGI252213[] = {{&S1671[4], 22}, {&S1671[44], 3}, {&S1671[142], 7}, {&S1671[167], 6}, {&S1671[187], 26}, {&S1671[167], 6}, {NULL}};
const Segment SGI252214[] = {{&S1671[0], 233}, {NULL}};
const Segment SGI252215[] = {{&S1672[25], 10}, {&S1672[31], 3}, {&S1672[48], 46}, {NULL}};
const Segment SGI252216[] = {{&S1672[0], 120}, {NULL}};
const Segment SGI252217[] = {{&S1673[0], 14}, {NULL}};
const Segment SGI252218[] = {{&S1674[0], 18}, {&S1674[31], 7}, {&S1674[44], 30}, {&S1674[66], 7}, {NULL}};
const Segment SGI252219[] = {{&S1674[0], 94}, {NULL}};
const Segment SGI252220[] = {{&S1675[12], 7}, {NULL}};
const Segment SGI252221[] = {{&S1675[0], 66}, {NULL}};
const Segment SGI252222[] = {{&S1676[25], 14}, {NULL}};
const Segment SGI252223[] = {{&S1676[0], 71}, {NULL}};
const Segment SGI252224[] = {{&S1677[0], 12}, {NULL}};
const Segment SGI252225[] = {{&S1678[25], 14}, {NULL}};
const Segment SGI252226[] = {{&S1678[0], 59}, {NULL}};
const Segment SGI252227[] = {{&S1679[0], 6}, {NULL}};
const Segment SGI252228[] = {{&S1680[16], 3}, {NULL}};
const Segment SGI252229[] = {{&S1680[0], 53}, {NULL}};
const Segment SGI252230[] = {{&S1681[0], 15}, {NULL}};
const Segment SGI252231[] = {{&S1682[25], 21}, {NULL}};
const Segment SGI252232[] = {{&S1682[0], 124}, {NULL}};
const Segment SGI252233[] = {{&S1683[25], 19}, {&S1683[45], 32}, {NULL}};
const Segment SGI252234[] = {{&S1683[0], 77}, {NULL}};
const Segment SGI252235[] = {{&S1684[19], 6}, {NULL}};
const Segment SGI252236[] = {{&S1684[0], 30}, {NULL}};
const Segment SGI252237[] = {{&S1685[16], 36}, {NULL}};
const Segment SGI252238[] = {{&S1685[0], 52}, {NULL}};
const Segment SGI252239[] = {{&S1686[12], 29}, {NULL}};
const Segment SGI252240[] = {{&S1686[0], 41}, {NULL}};
const Segment SGI252241[] = {{&S1687[0], 5}, {NULL}};
const Segment SGI252242[] = {{&S1688[42], 37}, {NULL}};
const Segment SGI252243[] = {{&S1688[0], 79}, {NULL}};
const Segment SGI252244[] = {{&S1689[0], 27}, {NULL}};
const Segment SGI252245[] = {{&S1690[25], 14}, {&S1690[40], 35}, {NULL}};
const Segment SGI252246[] = {{&S1690[0], 75}, {NULL}};
const Segment SGI252247[] = {{&S1691[0], 11}, {NULL}};
const Segment SGI252248[] = {{&S1692[0], 12}, {NULL}};
const Segment SGI252249[] = {{&S1692[0], 58}, {NULL}};
const Segment SGI252250[] = {{&S1693[25], 19}, {&S1693[45], 36}, {NULL}};
const Segment SGI252251[] = {{&S1693[0], 81}, {NULL}};
const Segment SGI252252[] = {{&S1694[0], 6}, {NULL}};
const Segment SGI252253[] = {{&S1694[0], 10}, {NULL}};
const Segment SGI252254[] = {{&S1695[0], 14}, {NULL}};
const Segment SGI252255[] = {{&S1696[25], 17}, {&S1696[43], 51}, {NULL}};
const Segment SGI252256[] = {{&S1696[0], 94}, {NULL}};
const Segment SGI252257[] = {{&S1697[7], 6}, {&S1697[19], 5}, {&S1697[27], 27}, {NULL}};
const Segment SGI252258[] = {{&S1697[0], 54}, {NULL}};
const Segment SGI252259[] = {{&S1698[63], 3}, {NULL}};
const Segment SGI252260[] = {{&S1698[0], 83}, {NULL}};
const Segment SGI252261[] = {{&S1699[19], 6}, {NULL}};
const Segment SGI252262[] = {{&S1699[0], 30}, {NULL}};
const Segment SGI252263[] = {{&S101112[44], 9}, {NULL}};
const Segment SGI252264[] = {{&S101112[44], 20}, {NULL}};
const Segment SGI252265[] = {{&S101112[30], 6}, {NULL}};
const Segment SGI252266[] = {{&S1700[19], 6}, {NULL}};
const Segment SGI252267[] = {{&S1700[0], 30}, {NULL}};
const Segment SGI252268[] = {{&S101112[113], 3}, {NULL}};
const Segment SGI252269[] = {{&S101112[78], 6}, {NULL}};
const Segment SGI252270[] = {{&S101112[155], 3}, {NULL}};
const Segment SGI252271[] = {{&S1701[32], 42}, {NULL}};
const Segment SGI252272[] = {{&S1701[0], 91}, {NULL}};
const Segment SGI252273[] = {{&S1702[22], 9}, {&S1702[32], 25}, {&S1702[50], 6}, {NULL}};
const Segment SGI252274[] = {{&S1702[0], 76}, {NULL}};
const Segment SGI252275[] = {{&S1703[35], 41}, {NULL}};
const Segment SGI252276[] = {{&S1703[0], 76}, {NULL}};
const Segment SGI252277[] = {{&S1704[25], 10}, {&S1704[36], 36}, {NULL}};
const Segment SGI252278[] = {{&S1704[0], 72}, {NULL}};
const Segment SGI252279[] = {{&S1705[0], 12}, {&S1705[18], 39}, {NULL}};
const Segment SGI252280[] = {{&S1705[0], 108}, {NULL}};
const Segment SGI252281[] = {{&S1706[0], 14}, {NULL}};
const Segment SGI252282[] = {{&S1706[0], 31}, {NULL}};
const Segment SGI252283[] = {{&S1707[22], 41}, {NULL}};
const Segment SGI252284[] = {{&S1707[0], 63}, {NULL}};
const Segment SGI252285[] = {{&S1708[12], 21}, {NULL}};
const Segment SGI252286[] = {{&S1708[0], 78}, {NULL}};
const Segment SGI252287[] = {{&S1709[0], 6}, {NULL}};
const Segment SGI252288[] = {{&S101124[34], 18}, {NULL}};
const Segment SGI252289[] = {{&S101124[27], 6}, {NULL}};
const Segment SGI252290[] = {{&S101124[104], 9}, {NULL}};
const Segment SGI252291[] = {{&S101124[77], 6}, {NULL}};
const Segment SGI252292[] = {{&S1710[13], 7}, {NULL}};
const Segment SGI252293[] = {{&S1710[0], 56}, {NULL}};
const Segment SGI252294[] = {{&S1711[43], 6}, {&S1711[53], 24}, {NULL}};
const Segment SGI252295[] = {{&S1711[0], 77}, {NULL}};
const Segment SGI252296[] = {{&S101126[34], 14}, {NULL}};
const Segment SGI252297[] = {{&S101126[27], 6}, {NULL}};
const Segment SGI252298[] = {{&S101126[100], 9}, {NULL}};
const Segment SGI252299[] = {{&S1712[19], 6}, {NULL}};
const Segment SGI252300[] = {{&S1712[0], 30}, {NULL}};
const Segment SGI252301[] = {{&S101126[73], 6}, {NULL}};
const Segment SGI252302[] = {{&S1713[25], 11}, {NULL}};
const Segment SGI252303[] = {{&S1713[0], 67}, {NULL}};
const Segment SGI252304[] = {{&S101128[34], 10}, {NULL}};
const Segment SGI252305[] = {{&S1714[19], 3}, {NULL}};
const Segment SGI252306[] = {{&S101128[27], 6}, {NULL}};
const Segment SGI252307[] = {{&S1714[0], 28}, {NULL}};
const Segment SGI252308[] = {{&S101128[96], 9}, {NULL}};
const Segment SGI252309[] = {{&S101128[69], 6}, {NULL}};
const Segment SGI252310[] = {{&S1715[7], 5}, {NULL}};
const Segment SGI252311[] = {{&S1715[0], 12}, {NULL}};
const Segment SGI252312[] = {{&S1716[25], 15}, {NULL}};
const Segment SGI252313[] = {{&S1716[0], 79}, {NULL}};
const Segment SGI252314[] = {{&S101130[34], 16}, {NULL}};
const Segment SGI252315[] = {{&S101130[27], 6}, {NULL}};
const Segment SGI252316[] = {{&S1717[8], 7}, {NULL}};
const Segment SGI252317[] = {{&S1717[0], 15}, {NULL}};
const Segment SGI252318[] = {{&S101130[102], 9}, {NULL}};
const Segment SGI252319[] = {{&S101130[75], 6}, {NULL}};
const Segment SGI252320[] = {{&S1718[16], 7}, {NULL}};
const Segment SGI252321[] = {{&S1718[0], 38}, {NULL}};
const Segment SGI252322[] = {{&S1719[25], 13}, {NULL}};
const Segment SGI252323[] = {{&S1719[0], 69}, {NULL}};
const Segment SGI252324[] = {{&S1720[18], 7}, {&S1720[26], 37}, {NULL}};
const Segment SGI252325[] = {{&S1720[0], 63}, {NULL}};
const Segment SGI252326[] = {{&S1721[34], 7}, {NULL}};
const Segment SGI252327[] = {{&S1721[0], 96}, {NULL}};
const Segment SGI252328[] = {{&S1722[25], 14}, {&S1722[40], 30}, {NULL}};
const Segment SGI252329[] = {{&S1722[0], 70}, {NULL}};
const Segment SGI252330[] = {{&S1723[41], 3}, {&S1723[51], 53}, {NULL}};
const Segment SGI252331[] = {{&S1723[0], 104}, {NULL}};
const Segment SGI252332[] = {{&S1724[0], 41}, {NULL}};
const Segment SGI252333[] = {{&S1725[25], 3}, {NULL}};
const Segment SGI252334[] = {{&S1725[0], 68}, {NULL}};
const Segment SGI252335[] = {{&S1726[25], 22}, {NULL}};
const Segment SGI252336[] = {{&S1726[0], 87}, {NULL}};
const Segment SGI252337[] = {{&S1727[25], 23}, {&S1727[49], 27}, {NULL}};
const Segment SGI252338[] = {{&S1727[0], 94}, {NULL}};
const Segment SGI252339[] = {{&S1728[33], 7}, {NULL}};
const Segment SGI252340[] = {{&S1728[0], 86}, {NULL}};
const Segment SGI252341[] = {{&S1729[25], 13}, {&S1729[39], 37}, {NULL}};
const Segment SGI252342[] = {{&S1729[0], 76}, {NULL}};
const Segment SGI252343[] = {{&S1730[41], 25}, {NULL}};
const Segment SGI252344[] = {{&S1730[0], 66}, {NULL}};
const Segment SGI252345[] = {{&S1731[35], 32}, {NULL}};
const Segment SGI252346[] = {{&S1731[0], 67}, {NULL}};
const Segment SGI252347[] = {{&S1732[0], 12}, {NULL}};
const Segment SGI252348[] = {{&S1733[12], 26}, {NULL}};
const Segment SGI252349[] = {{&S1733[0], 80}, {NULL}};
const Segment SGI252350[] = {{&S1734[25], 16}, {NULL}};
const Segment SGI252351[] = {{&S1734[0], 71}, {NULL}};
const Segment SGI252352[] = {{&S1735[25], 12}, {&S1735[38], 24}, {NULL}};
const Segment SGI252353[] = {{&S1735[0], 62}, {NULL}};
const Segment SGI252354[] = {{&S1736[25], 14}, {NULL}};
const Segment SGI252355[] = {{&S1736[0], 64}, {NULL}};
const Segment SGI252356[] = {{&S1737[25], 11}, {&S1737[37], 32}, {NULL}};
const Segment SGI252357[] = {{&S1737[0], 69}, {NULL}};
const Segment SGI252358[] = {{&S1738[25], 10}, {NULL}};
const Segment SGI252359[] = {{&S1738[0], 56}, {NULL}};
const Segment SGI252360[] = {{&S1739[31], 30}, {NULL}};
const Segment SGI252361[] = {{&S1739[0], 92}, {NULL}};
const Segment SGI252362[] = {{&S1740[23], 12}, {&S1740[47], 9}, {&S1740[78], 25}, {NULL}};
const Segment SGI252363[] = {{&S1740[0], 103}, {NULL}};
const Segment SGI252364[] = {{&S1741[36], 6}, {NULL}};
const Segment SGI252365[] = {{&S1741[0], 94}, {NULL}};
const Segment SGI252366[] = {{&S1742[12], 34}, {NULL}};
const Segment SGI252367[] = {{&S1742[0], 104}, {NULL}};
const Segment SGI252368[] = {{&S1743[25], 15}, {&S1743[41], 33}, {NULL}};
const Segment SGI252369[] = {{&S1743[0], 74}, {NULL}};
const Segment SGI252370[] = {{&S1744[25], 27}, {NULL}};
const Segment SGI252371[] = {{&S1744[0], 52}, {NULL}};
const Segment SGI252372[] = {{&S1745[25], 16}, {&S1745[42], 37}, {NULL}};
const Segment SGI252373[] = {{&S1745[0], 79}, {NULL}};
const Segment SGI252374[] = {{&S1746[25], 23}, {&S1746[49], 27}, {NULL}};
const Segment SGI252375[] = {{&S1746[0], 76}, {NULL}};
const Segment SGI252376[] = {{&S1747[12], 76}, {NULL}};
const Segment SGI252377[] = {{&S1747[0], 88}, {NULL}};
const Segment SGI252378[] = {{&S1748[40], 10}, {NULL}};
const Segment SGI252379[] = {{&S1748[0], 102}, {NULL}};
const Segment SGI252380[] = {{&S1749[25], 12}, {&S1749[38], 24}, {NULL}};
const Segment SGI252381[] = {{&S1749[0], 62}, {NULL}};
const Segment SGI252382[] = {{&S1750[25], 12}, {&S1750[38], 29}, {NULL}};
const Segment SGI252383[] = {{&S1750[0], 67}, {NULL}};
const Segment SGI252384[] = {{&S1751[12], 25}, {NULL}};
const Segment SGI252385[] = {{&S1751[0], 37}, {NULL}};
const Segment SGI252386[] = {{&S1752[31], 34}, {NULL}};
const Segment SGI252387[] = {{&S1752[0], 114}, {NULL}};
const Segment SGI252388[] = {{&S1753[25], 13}, {NULL}};
const Segment SGI252389[] = {{&S1753[0], 38}, {NULL}};
const Segment SGI252390[] = {{&S1754[25], 30}, {NULL}};
const Segment SGI252391[] = {{&S1754[0], 57}, {NULL}};
const Segment SGI252392[] = {{&S1755[25], 14}, {&S1755[40], 31}, {NULL}};
const Segment SGI252393[] = {{&S1755[0], 71}, {NULL}};
const Segment SGI252394[] = {{&S1756[34], 3}, {&S1756[39], 32}, {NULL}};
const Segment SGI252395[] = {{&S1756[0], 71}, {NULL}};
const Segment SGI252396[] = {{&S1757[25], 18}, {NULL}};
const Segment SGI252397[] = {{&S1757[0], 82}, {NULL}};
const Segment SGI252398[] = {{&S1758[25], 11}, {NULL}};
const Segment SGI252399[] = {{&S1758[0], 66}, {NULL}};
const Segment SGI252400[] = {{&S1759[0], 82}, {NULL}};
const Segment SGI252401[] = {{&S1760[0], 12}, {NULL}};
const Segment SGI252402[] = {{&S1760[0], 75}, {NULL}};
const Segment SGI252403[] = {{&S1761[11], 6}, {NULL}};
const Segment SGI252404[] = {{&S1761[0], 17}, {NULL}};
const Segment SGI252405[] = {{&S1762[25], 17}, {NULL}};
const Segment SGI252406[] = {{&S1762[0], 61}, {NULL}};
const Segment SGI252407[] = {{&S1763[0], 10}, {&S1763[11], 32}, {NULL}};
const Segment SGI252408[] = {{&S1763[0], 74}, {NULL}};
const Segment SGI252409[] = {{&S1764[0], 17}, {NULL}};
const Segment SGI252410[] = {{&S1765[25], 11}, {&S1765[37], 19}, {NULL}};
const Segment SGI252411[] = {{&S1765[0], 56}, {NULL}};
const Segment SGI252412[] = {{&S1766[0], 17}, {NULL}};
const Segment SGI252413[] = {{&S1767[25], 17}, {&S1767[43], 41}, {NULL}};
const Segment SGI252414[] = {{&S1767[0], 84}, {NULL}};
const Segment SGI252415[] = {{&S1768[0], 17}, {NULL}};
const Segment SGI252416[] = {{&S1769[0], 20}, {NULL}};
const Segment SGI252417[] = {{&S1769[0], 88}, {NULL}};
const Segment SGI252418[] = {{&S1770[0], 3}, {NULL}};
const Segment SGI252419[] = {{&S1770[0], 50}, {NULL}};
const Segment SGI252420[] = {{&S1771[0], 34}, {NULL}};
const Segment SGI252421[] = {{&S1773[6], 14}, {NULL}};
const Segment SGI252422[] = {{&S1773[0], 50}, {NULL}};
const Segment SGI252423[] = {{&S1774[43], 38}, {NULL}};
const Segment SGI252424[] = {{&S1774[0], 81}, {NULL}};
const Segment SGI252425[] = {{&S1775[24], 15}, {&S1775[40], 36}, {NULL}};
const Segment SGI252426[] = {{&S1775[0], 76}, {NULL}};
const Segment SGI252427[] = {{&S1776[10], 6}, {NULL}};
const Segment SGI252428[] = {{&S1776[0], 20}, {NULL}};
const Segment SGI252429[] = {{&S1777[0], 20}, {NULL}};
const Segment SGI252430[] = {{&S1777[0], 38}, {NULL}};
const Segment SGI252431[] = {{&S1778[25], 16}, {&S1778[42], 33}, {NULL}};
const Segment SGI252432[] = {{&S1778[0], 75}, {NULL}};
const Segment SGI252433[] = {{&S1779[0], 12}, {&S1779[13], 35}, {&S1779[49], 28}, {NULL}};
const Segment SGI252434[] = {{&S1779[0], 77}, {NULL}};
const Segment SGI252435[] = {{&S1780[12], 7}, {NULL}};
const Segment SGI252436[] = {{&S1780[0], 19}, {NULL}};
const Segment SGI252437[] = {{&S1781[0], 57}, {NULL}};
const Segment SGI252438[] = {{&S1782[0], 7}, {NULL}};
const Segment SGI252439[] = {{&S1783[0], 19}, {&S1783[20], 12}, {&S1783[49], 35}, {NULL}};
const Segment SGI252440[] = {{&S1783[0], 84}, {NULL}};
const Segment SGI252441[] = {{&S1784[0], 10}, {&S1784[11], 8}, {&S1784[20], 22}, {&S1784[43], 18}, {NULL}};
const Segment SGI252442[] = {{&S1784[0], 61}, {NULL}};
const Segment SGI252443[] = {{&S1785[0], 20}, {&S1785[67], 42}, {NULL}};
const Segment SGI252444[] = {{&S1785[0], 132}, {NULL}};
const Segment SGI252445[] = {{&S1786[40], 10}, {NULL}};
const Segment SGI252446[] = {{&S1786[0], 102}, {NULL}};
const Segment SGI252447[] = {{&S1787[25], 10}, {&S1787[36], 35}, {NULL}};
const Segment SGI252448[] = {{&S1787[0], 71}, {NULL}};
const Segment SGI252449[] = {{&S1788[25], 15}, {NULL}};
const Segment SGI252450[] = {{&S1788[0], 68}, {NULL}};
const Segment SGI252451[] = {{&S1789[25], 15}, {&S1789[41], 37}, {NULL}};
const Segment SGI252452[] = {{&S1789[0], 78}, {NULL}};
const Segment SGI252453[] = {{&S1790[44], 10}, {NULL}};
const Segment SGI252454[] = {{&S1790[0], 106}, {NULL}};
const Segment SGI252455[] = {{&S1791[25], 16}, {&S1791[42], 41}, {NULL}};
const Segment SGI252456[] = {{&S1791[0], 155}, {NULL}};
const Segment SGI252457[] = {{&S1792[0], 17}, {NULL}};
const Segment SGI252458[] = {{&S1793[25], 13}, {&S1793[46], 24}, {NULL}};
const Segment SGI252459[] = {{&S1793[0], 70}, {NULL}};
const Segment SGI252460[] = {{&S1794[25], 10}, {NULL}};
const Segment SGI252461[] = {{&S1794[0], 35}, {NULL}};
const Segment SGI252462[] = {{&S1795[25], 14}, {NULL}};
const Segment SGI252463[] = {{&S1795[0], 66}, {NULL}};
const Segment SGI252464[] = {{&S1796[6], 3}, {NULL}};
const Segment SGI252465[] = {{&S1796[0], 15}, {NULL}};
const Segment SGI252466[] = {{&S1797[19], 6}, {NULL}};
const Segment SGI252467[] = {{&S1797[0], 30}, {NULL}};
const Segment SGI252468[] = {{&S1798[25], 21}, {NULL}};
const Segment SGI252469[] = {{&S1798[0], 46}, {NULL}};
const Segment SGI252470[] = {{&S1799[0], 16}, {&S1799[43], 55}, {NULL}};
const Segment SGI252471[] = {{&S1799[0], 98}, {NULL}};
const Segment SGI252472[] = {{&S1800[0], 11}, {NULL}};
const Segment SGI252473[] = {{&S1800[0], 62}, {NULL}};
const Segment SGI252474[] = {{&S1801[0], 7}, {NULL}};
const Segment SGI252475[] = {{&S1801[0], 54}, {NULL}};
const Segment SGI252476[] = {{&S1802[19], 3}, {NULL}};
const Segment SGI252477[] = {{&S1802[0], 28}, {NULL}};
const Segment SGI252478[] = {{&S1803[31], 30}, {NULL}};
const Segment SGI252479[] = {{&S1803[0], 92}, {NULL}};
const Segment SGI252480[] = {{&S1804[25], 14}, {&S1804[40], 31}, {NULL}};
const Segment SGI252481[] = {{&S1804[0], 71}, {NULL}};
const Segment SGI252482[] = {{&S1805[0], 17}, {&S1805[39], 7}, {NULL}};
const Segment SGI252483[] = {{&S1805[0], 58}, {NULL}};
const Segment SGI252484[] = {{&S1806[25], 18}, {NULL}};
const Segment SGI252485[] = {{&S1806[0], 61}, {NULL}};
const Segment SGI252486[] = {{&S1807[8], 3}, {NULL}};
const Segment SGI252487[] = {{&S1807[0], 36}, {NULL}};
const Segment SGI252488[] = {{&S1808[35], 28}, {&S1808[36], 7}, {NULL}};
const Segment SGI252489[] = {{&S1808[0], 91}, {NULL}};
const Segment SGI252490[] = {{&S1809[0], 13}, {NULL}};
const Segment SGI252491[] = {{&S1809[0], 58}, {NULL}};
const Segment SGI252492[] = {{&S1810[25], 16}, {&S1810[42], 37}, {NULL}};
const Segment SGI252493[] = {{&S1810[0], 79}, {NULL}};
const Segment SGI252494[] = {{&S1811[0], 3}, {NULL}};
const Segment SGI252495[] = {{&S1811[0], 44}, {NULL}};
const Segment SGI252496[] = {{&S1812[25], 15}, {&S1812[41], 36}, {NULL}};
const Segment SGI252497[] = {{&S1812[0], 77}, {NULL}};
const Segment SGI252498[] = {{&S1813[25], 17}, {NULL}};
const Segment SGI252499[] = {{&S1813[0], 66}, {NULL}};
const Segment SGI252500[] = {{&S1814[17], 7}, {NULL}};
const Segment SGI252501[] = {{&S1814[0], 59}, {NULL}};
const Segment SGI252502[] = {{&S1815[42], 7}, {NULL}};
const Segment SGI252503[] = {{&S1815[0], 71}, {NULL}};
const Segment SGI252504[] = {{&S1816[0], 16}, {NULL}};
const Segment SGI252505[] = {{&S1816[0], 41}, {NULL}};
const Segment SGI252506[] = {{&S1817[0], 14}, {NULL}};
const Segment SGI252507[] = {{&S1818[48], 3}, {NULL}};
const Segment SGI252508[] = {{&S1818[0], 72}, {NULL}};
const Segment SGI252509[] = {{&S1819[19], 6}, {NULL}};
const Segment SGI252510[] = {{&S1819[0], 30}, {NULL}};
const Segment SGI252511[] = {{&S1820[19], 6}, {NULL}};
const Segment SGI252512[] = {{&S1820[0], 30}, {NULL}};
const Segment SGI252513[] = {{&S1821[25], 17}, {&S1821[43], 30}, {NULL}};
const Segment SGI252514[] = {{&S1821[0], 73}, {NULL}};
const Segment SGI252515[] = {{&S1822[0], 23}, {NULL}};
const Segment SGI252516[] = {{&S1822[0], 43}, {NULL}};
const Segment SGI252517[] = {{&S1823[16], 7}, {NULL}};
const Segment SGI252518[] = {{&S1823[0], 38}, {NULL}};
const Segment SGI252519[] = {{&S1824[85], 8}, {NULL}};
const Segment SGI252520[] = {{&S1824[0], 93}, {NULL}};
const Segment SGI252521[] = {{&S1825[7], 13}, {NULL}};
const Segment SGI252522[] = {{&S1825[0], 80}, {NULL}};
const Segment SGI252523[] = {{&S1826[25], 6}, {&S1826[37], 46}, {NULL}};
const Segment SGI252524[] = {{&S1826[0], 83}, {NULL}};
const Segment SGI252525[] = {{&S1828[0], 8}, {NULL}};
const Segment SGI252526[] = {{&S1828[0], 12}, {NULL}};
const Segment SGI252527[] = {{&S1829[0], 16}, {&S1829[17], 32}, {NULL}};
const Segment SGI252528[] = {{&S1829[0], 91}, {NULL}};
const Segment SGI252529[] = {{&S1830[0], 8}, {NULL}};
const Segment SGI252530[] = {{&S1830[0], 40}, {NULL}};
const Segment SGI252531[] = {{&S1831[0], 12}, {&S1831[13], 28}, {NULL}};
const Segment SGI252532[] = {{&S1831[0], 41}, {NULL}};
const Segment SGI252533[] = {{&S1832[6], 3}, {NULL}};
const Segment SGI252534[] = {{&S1832[0], 32}, {NULL}};
const Segment SGI252535[] = {{&S1833[19], 6}, {NULL}};
const Segment SGI252536[] = {{&S1833[0], 56}, {NULL}};
const Segment SGI252537[] = {{&S1834[25], 14}, {NULL}};
const Segment SGI252538[] = {{&S1834[0], 39}, {NULL}};
const Segment SGI252539[] = {{&S1835[0], 11}, {&S1835[12], 41}, {NULL}};
const Segment SGI252540[] = {{&S1835[0], 53}, {NULL}};
const Segment SGI252541[] = {{&S1836[0], 7}, {NULL}};
const Segment SGI252542[] = {{&S1837[4], 22}, {&S1837[32], 3}, {&S1837[86], 26}, {&S1837[93], 6}, {NULL}};
const Segment SGI252543[] = {{&S1837[0], 132}, {NULL}};
const Segment SGI252544[] = {{&S1838[4], 22}, {&S1838[32], 3}, {&S1838[86], 26}, {&S1838[93], 6}, {NULL}};
const Segment SGI252545[] = {{&S1838[0], 134}, {NULL}};
const Segment SGI252546[] = {{&S1839[0], 19}, {NULL}};
const Segment SGI252547[] = {{&S1840[15], 6}, {NULL}};
const Segment SGI252548[] = {{&S1840[0], 52}, {NULL}};
const Segment SGI252549[] = {{&S1841[25], 10}, {&S1841[36], 31}, {NULL}};
const Segment SGI252550[] = {{&S1841[0], 69}, {NULL}};
const Segment SGI252551[] = {{&S1842[25], 16}, {NULL}};
const Segment SGI252552[] = {{&S1842[0], 41}, {NULL}};
const Segment SGI252553[] = {{&S1843[14], 7}, {NULL}};
const Segment SGI252554[] = {{&S1843[0], 49}, {NULL}};
const Segment SGI252555[] = {{&S1844[25], 17}, {NULL}};
const Segment SGI252556[] = {{&S1844[0], 67}, {NULL}};
const Segment SGI252557[] = {{&S1845[0], 10}, {NULL}};
const Segment SGI252558[] = {{&S1845[0], 40}, {NULL}};
const Segment SGI252559[] = {{&S1846[25], 21}, {NULL}};
const Segment SGI252560[] = {{&S1846[0], 81}, {NULL}};
const Segment SGI252561[] = {{&S1847[25], 16}, {&S1847[42], 32}, {NULL}};
const Segment SGI252562[] = {{&S1847[0], 117}, {NULL}};
const Segment SGI252563[] = {{&S1848[25], 16}, {NULL}};
const Segment SGI252564[] = {{&S1848[0], 67}, {NULL}};
const Segment SGI252565[] = {{&S1849[25], 14}, {NULL}};
const Segment SGI252566[] = {{&S1849[0], 39}, {NULL}};
const Segment SGI252567[] = {{&S1850[31], 3}, {NULL}};
const Segment SGI252568[] = {{&S1850[0], 39}, {NULL}};
const Segment SGI252569[] = {{&S1851[25], 18}, {NULL}};
const Segment SGI252570[] = {{&S1851[0], 66}, {NULL}};
const Segment SGI252571[] = {{&S1852[15], 6}, {NULL}};
const Segment SGI252572[] = {{&S1852[0], 52}, {NULL}};
const Segment SGI252573[] = {{&S1853[7], 3}, {NULL}};
const Segment SGI252574[] = {{&S1853[0], 15}, {NULL}};
const Segment SGI252575[] = {{&S1854[0], 10}, {NULL}};
const Segment SGI252576[] = {{&S1855[0], 17}, {NULL}};
const Segment SGI252577[] = {{&S1855[0], 56}, {NULL}};
const Segment SGI252578[] = {{&S1856[15], 3}, {&S1856[44], 12}, {&S1856[105], 3}, {NULL}};
const Segment SGI252579[] = {{&S1856[0], 120}, {NULL}};
const Segment SGI252580[] = {{&S1857[0], 59}, {NULL}};
const Segment SGI252581[] = {{&S1858[43], 6}, {&S1858[53], 25}, {NULL}};
const Segment SGI252582[] = {{&S1858[0], 86}, {NULL}};
const Segment SGI252583[] = {{&S1859[0], 11}, {NULL}};
const Segment SGI252584[] = {{&S1860[0], 9}, {NULL}};
const Segment SGI252585[] = {{&S1861[25], 11}, {&S1861[37], 33}, {NULL}};
const Segment SGI252586[] = {{&S1861[0], 70}, {NULL}};
const Segment SGI252587[] = {{&S1862[0], 10}, {&S1862[11], 40}, {NULL}};
const Segment SGI252588[] = {{&S1862[0], 51}, {NULL}};
const Segment SGI252589[] = {{&S1863[0], 6}, {NULL}};
const Segment SGI252590[] = {{&S1864[0], 6}, {NULL}};
const Segment SGI252591[] = {{&S1864[0], 9}, {NULL}};
const Segment SGI252592[] = {{&S1865[0], 12}, {NULL}};
const Segment SGI252593[] = {{&S1866[25], 16}, {&S1866[42], 41}, {NULL}};
const Segment SGI252594[] = {{&S1866[0], 155}, {NULL}};
const Segment SGI252595[] = {{&S1867[11], 27}, {NULL}};
const Segment SGI252596[] = {{&S1867[0], 38}, {NULL}};
const Segment SGI252597[] = {{&S1868[0], 15}, {NULL}};
const Segment SGI252598[] = {{&S1869[0], 13}, {NULL}};
const Segment SGI252599[] = {{&S1869[0], 43}, {NULL}};
const Segment SGI252600[] = {{&S1870[62], 3}, {NULL}};
const Segment SGI252601[] = {{&S1870[0], 82}, {NULL}};
const Segment SGI252602[] = {{&S1871[25], 12}, {NULL}};
const Segment SGI252603[] = {{&S1871[0], 60}, {NULL}};
const Segment SGI252604[] = {{&S1872[0], 24}, {&S1872[25], 38}, {NULL}};
const Segment SGI252605[] = {{&S1872[0], 63}, {NULL}};
const Segment SGI252606[] = {{&S1873[25], 10}, {NULL}};
const Segment SGI252607[] = {{&S1873[0], 63}, {NULL}};
const Segment SGI252608[] = {{&S1874[0], 5}, {NULL}};
const Segment SGI252609[] = {{&S1875[19], 15}, {NULL}};
const Segment SGI252610[] = {{&S1875[0], 70}, {NULL}};
const Segment SGI252611[] = {{&S1876[25], 15}, {&S1876[41], 37}, {NULL}};
const Segment SGI252612[] = {{&S1876[0], 78}, {NULL}};
const Segment SGI252613[] = {{&S1877[43], 38}, {NULL}};
const Segment SGI252614[] = {{&S1877[0], 81}, {NULL}};
const Segment SGI252615[] = {{&S1878[25], 13}, {NULL}};
const Segment SGI252616[] = {{&S1878[0], 101}, {NULL}};
const Segment SGI252617[] = {{&S1879[12], 7}, {NULL}};
const Segment SGI252618[] = {{&S1879[0], 19}, {NULL}};
const Segment SGI252619[] = {{&S1880[13], 7}, {NULL}};
const Segment SGI252620[] = {{&S1880[0], 24}, {NULL}};
const Segment SGI252621[] = {{&S1881[0], 7}, {NULL}};
const Segment SGI252622[] = {{&S1882[0], 14}, {&S1882[15], 39}, {NULL}};
const Segment SGI252623[] = {{&S1882[0], 54}, {NULL}};
const Segment SGI252624[] = {{&S101399[44], 10}, {NULL}};
const Segment SGI252625[] = {{&S101399[44], 21}, {NULL}};
const Segment SGI252626[] = {{&S101399[30], 6}, {NULL}};
const Segment SGI252627[] = {{&S101399[132], 3}, {NULL}};
const Segment SGI252628[] = {{&S101399[97], 6}, {NULL}};
const Segment SGI252629[] = {{&S101399[160], 3}, {NULL}};
const Segment SGI252630[] = {{&S101399[67], 8}, {NULL}};
const Segment SGI252631[] = {{&S1883[61], 22}, {NULL}};
const Segment SGI252632[] = {{&S1883[0], 83}, {NULL}};
const Segment SGI252633[] = {{&S1884[25], 7}, {&S1884[25], 7}, {NULL}};
const Segment SGI252634[] = {{&S1884[0], 62}, {NULL}};
const Segment SGI252635[] = {{&S1885[4], 3}, {NULL}};
const Segment SGI252636[] = {{&S1885[0], 11}, {NULL}};
const Segment SGI252637[] = {{&S1886[0], 16}, {NULL}};
const Segment SGI252638[] = {{&S1886[0], 41}, {NULL}};
const Segment SGI252639[] = {{&S1887[0], 9}, {NULL}};
const Segment SGI252640[] = {{&S1888[9], 7}, {NULL}};
const Segment SGI252641[] = {{&S1888[0], 21}, {NULL}};
const Segment SGI252642[] = {{&S1889[29], 3}, {NULL}};
const Segment SGI252643[] = {{&S1889[0], 54}, {NULL}};
const Segment SGI252644[] = {{&S1890[35], 38}, {NULL}};
const Segment SGI252645[] = {{&S1890[0], 81}, {NULL}};
const Segment SGI252646[] = {{&S1891[25], 20}, {&S1891[46], 53}, {NULL}};
const Segment SGI252647[] = {{&S1891[0], 99}, {NULL}};
const Segment SGI252648[] = {{&S1892[38], 43}, {NULL}};
const Segment SGI252649[] = {{&S1892[0], 81}, {NULL}};
const Segment SGI252650[] = {{&S1893[0], 6}, {NULL}};
const Segment SGI252651[] = {{&S1893[0], 32}, {NULL}};
const Segment SGI252652[] = {{&S1894[25], 17}, {NULL}};
const Segment SGI252653[] = {{&S1894[0], 66}, {NULL}};
const Segment SGI252654[] = {{&S1895[25], 7}, {NULL}};
const Segment SGI252655[] = {{&S1895[0], 32}, {NULL}};
const Segment SGI252656[] = {{&S1896[25], 15}, {&S1896[41], 38}, {NULL}};
const Segment SGI252657[] = {{&S1896[0], 151}, {NULL}};
const Segment SGI252658[] = {{&S1897[25], 17}, {NULL}};
const Segment SGI252659[] = {{&S1897[0], 112}, {NULL}};
const Segment SGI252660[] = {{&S1898[23], 12}, {NULL}};
const Segment SGI252661[] = {{&S1898[0], 35}, {NULL}};
const Segment SGI252662[] = {{&S1899[0], 7}, {NULL}};
const Segment SGI252663[] = {{&S1899[0], 54}, {NULL}};
const Segment SGI252664[] = {{&S1900[10], 6}, {NULL}};
const Segment SGI252665[] = {{&S1900[0], 21}, {NULL}};
const Segment SGI252666[] = {{&S1901[0], 15}, {NULL}};
const Segment SGI252667[] = {{&S1901[0], 45}, {NULL}};
const Segment SGI252668[] = {{&S1902[61], 18}, {NULL}};
const Segment SGI252669[] = {{&S1902[0], 79}, {NULL}};
const Segment SGI252670[] = {{&S1903[12], 3}, {NULL}};
const Segment SGI252671[] = {{&S1903[0], 20}, {NULL}};
const Segment SGI252672[] = {{&S1904[0], 11}, {NULL}};
const Segment SGI252673[] = {{&S1905[13], 7}, {NULL}};
const Segment SGI252674[] = {{&S1905[0], 87}, {NULL}};
const Segment SGI252675[] = {{&S1906[10], 35}, {NULL}};
const Segment SGI252676[] = {{&S1906[0], 45}, {NULL}};
const Segment SGI252677[] = {{&S1907[35], 3}, {NULL}};
const Segment SGI252678[] = {{&S1907[0], 43}, {NULL}};
const Segment SGI252679[] = {{&S1908[10], 3}, {NULL}};
const Segment SGI252680[] = {{&S1908[0], 13}, {NULL}};
const Segment SGI252681[] = {{&S1909[0], 10}, {&S1909[72], 18}, {NULL}};
const Segment SGI252682[] = {{&S1909[0], 90}, {NULL}};
const Segment SGI252683[] = {{&S1910[8], 7}, {&S1910[16], 32}, {NULL}};
const Segment SGI252684[] = {{&S1910[0], 48}, {NULL}};
const Segment SGI252685[] = {{&S1911[10], 6}, {NULL}};
const Segment SGI252686[] = {{&S1911[0], 23}, {NULL}};
const Segment SGI252687[] = {{&S1912[51], 7}, {NULL}};
const Segment SGI252688[] = {{&S1912[0], 75}, {NULL}};
const Segment SGI252689[] = {{&S1913[0], 10}, {&S1913[72], 22}, {NULL}};
const Segment SGI252690[] = {{&S1913[0], 94}, {NULL}};
const Segment SGI252691[] = {{&S1914[12], 25}, {&S1914[38], 24}, {NULL}};
const Segment SGI252692[] = {{&S1914[0], 62}, {NULL}};
const Segment SGI252693[] = {{&S1915[25], 9}, {&S1915[35], 30}, {NULL}};
const Segment SGI252694[] = {{&S1915[0], 65}, {NULL}};
const Segment SGI252695[] = {{&S1916[13], 7}, {&S1916[27], 28}, {NULL}};
const Segment SGI252696[] = {{&S1916[0], 55}, {NULL}};
const Segment SGI252697[] = {{&S1917[25], 17}, {NULL}};
const Segment SGI252698[] = {{&S1917[0], 68}, {NULL}};
const Segment SGI252699[] = {{&S1918[31], 7}, {NULL}};
const Segment SGI252700[] = {{&S1918[0], 50}, {NULL}};
const Segment SGI252701[] = {{&S1919[25], 13}, {NULL}};
const Segment SGI252702[] = {{&S1919[0], 38}, {NULL}};
const Segment SGI252703[] = {{&S1920[13], 7}, {NULL}};
const Segment SGI252704[] = {{&S1920[0], 46}, {NULL}};
const Segment SGI252705[] = {{&S1921[25], 16}, {&S1921[42], 31}, {NULL}};
const Segment SGI252706[] = {{&S1921[0], 73}, {NULL}};
const Segment SGI252707[] = {{&S1922[0], 17}, {NULL}};
const Segment SGI252708[] = {{&S1923[25], 12}, {&S1923[38], 30}, {NULL}};
const Segment SGI252709[] = {{&S1923[0], 68}, {NULL}};
const Segment SGI252710[] = {{&S1924[13], 9}, {NULL}};
const Segment SGI252711[] = {{&S1924[0], 22}, {NULL}};
const Segment SGI252712[] = {{&S1925[0], 14}, {&S1925[15], 37}, {NULL}};
const Segment SGI252713[] = {{&S1925[0], 52}, {NULL}};
const Segment SGI252714[] = {{&S1926[0], 10}, {NULL}};
const Segment SGI252715[] = {{&S1927[11], 45}, {NULL}};
const Segment SGI252716[] = {{&S1927[0], 56}, {NULL}};
const Segment SGI252717[] = {{&S1928[0], 19}, {NULL}};
const Segment SGI252718[] = {{&S1928[0], 60}, {NULL}};
const Segment SGI252719[] = {{&S1929[0], 12}, {&S1929[26], 15}, {&S1929[42], 44}, {NULL}};
const Segment SGI252720[] = {{&S1929[0], 86}, {NULL}};
const Segment SGI252721[] = {{&S1930[0], 6}, {NULL}};
const Segment SGI252722[] = {{&S1930[0], 11}, {NULL}};
const Segment SGI252723[] = {{&S1931[19], 3}, {NULL}};
const Segment SGI252724[] = {{&S1931[0], 28}, {NULL}};
const Segment SGI252725[] = {{&S1932[0], 11}, {&S1932[12], 42}, {NULL}};
const Segment SGI252726[] = {{&S1932[0], 54}, {NULL}};
const Segment SGI252727[] = {{&S1934[10], 6}, {NULL}};
const Segment SGI252728[] = {{&S1934[0], 22}, {NULL}};
const Segment SGI252729[] = {{&S1935[48], 3}, {NULL}};
const Segment SGI252730[] = {{&S1935[0], 100}, {NULL}};
const Segment SGI252731[] = {{&S1936[25], 20}, {NULL}};
const Segment SGI252732[] = {{&S1936[0], 78}, {NULL}};
const Segment SGI252733[] = {{&S1937[12], 7}, {NULL}};
const Segment SGI252734[] = {{&S1937[0], 19}, {NULL}};
const Segment SGI252735[] = {{&S1938[25], 15}, {&S1938[41], 27}, {NULL}};
const Segment SGI252736[] = {{&S1938[0], 68}, {NULL}};
const Segment SGI252737[] = {{&S1939[0], 34}, {NULL}};
const Segment SGI252738[] = {{&S1939[0], 38}, {NULL}};
const Segment SGI252739[] = {{&S12767[13], 4}, {NULL}};
const Segment SGI252740[] = {{&S1940[13], 7}, {&S1940[27], 28}, {NULL}};
const Segment SGI252741[] = {{&S12767[33], 3}, {NULL}};
const Segment SGI252742[] = {{&S1940[0], 55}, {NULL}};
const Segment SGI252743[] = {{&S12767[64], 6}, {NULL}};
const Segment SGI252744[] = {{&S12767[98], 6}, {NULL}};
const Segment SGI252745[] = {{&S12768[13], 4}, {NULL}};
const Segment SGI252746[] = {{&S1941[25], 8}, {NULL}};
const Segment SGI252747[] = {{&S1941[0], 79}, {NULL}};
const Segment SGI252748[] = {{&S12768[33], 3}, {NULL}};
const Segment SGI252749[] = {{&S12768[64], 6}, {NULL}};
const Segment SGI252750[] = {{&S12768[98], 6}, {NULL}};
const Segment SGI252751[] = {{&S1942[6], 3}, {NULL}};
const Segment SGI252752[] = {{&S1942[0], 18}, {NULL}};
const Segment SGI252753[] = {{&S12769[13], 4}, {&S12769[13], 4}, {NULL}};
const Segment SGI252754[] = {{&S12769[33], 3}, {NULL}};
const Segment SGI252755[] = {{&S12769[64], 6}, {NULL}};
const Segment SGI252756[] = {{&S1943[20], 3}, {NULL}};
const Segment SGI252757[] = {{&S1943[0], 23}, {NULL}};
const Segment SGI252758[] = {{&S12769[98], 6}, {NULL}};
const Segment SGI252759[] = {{&S1944[13], 7}, {&S1944[25], 28}, {NULL}};
const Segment SGI252760[] = {{&S1944[0], 53}, {NULL}};
const Segment SGI252761[] = {{&S12770[13], 4}, {&S12770[13], 4}, {&S12770[13], 4}, {NULL}};
const Segment SGI252762[] = {{&S1945[0], 12}, {&S1945[13], 35}, {&S1945[49], 34}, {NULL}};
const Segment SGI252763[] = {{&S1945[0], 83}, {NULL}};
const Segment SGI252764[] = {{&S12770[42], 3}, {NULL}};
const Segment SGI252765[] = {{&S1946[13], 7}, {&S1946[27], 28}, {NULL}};
const Segment SGI252766[] = {{&S1946[0], 55}, {NULL}};
const Segment SGI252767[] = {{&S12771[13], 4}, {&S12771[13], 4}, {&S12771[13], 4}, {NULL}};
const Segment SGI252768[] = {{&S1947[11], 6}, {&S1947[23], 31}, {NULL}};
const Segment SGI252769[] = {{&S1947[0], 54}, {NULL}};
const Segment SGI252770[] = {{&S12771[33], 3}, {NULL}};
const Segment SGI252771[] = {{&S12771[64], 6}, {NULL}};
const Segment SGI252772[] = {{&S12771[98], 6}, {NULL}};
const Segment SGI252773[] = {{&S1948[0], 67}, {NULL}};
const Segment SGI252774[] = {{&S12772[13], 4}, {&S12772[13], 4}, {&S12772[13], 4}, {NULL}};
const Segment SGI252775[] = {{&S1949[0], 15}, {&S1949[21], 35}, {NULL}};
const Segment SGI252776[] = {{&S1949[0], 56}, {NULL}};
const Segment SGI252777[] = {{&S12772[33], 3}, {NULL}};
const Segment SGI252778[] = {{&S12772[64], 6}, {NULL}};
const Segment SGI252779[] = {{&S12772[98], 6}, {NULL}};
const Segment SGI252780[] = {{&S1950[0], 35}, {NULL}};
const Segment SGI252781[] = {{&S12773[13], 4}, {NULL}};
const Segment SGI252782[] = {{&S12773[33], 3}, {NULL}};
const Segment SGI252783[] = {{&S1951[19], 6}, {NULL}};
const Segment SGI252784[] = {{&S12773[64], 6}, {NULL}};
const Segment SGI252785[] = {{&S1951[0], 30}, {NULL}};
const Segment SGI252786[] = {{&S12774[13], 6}, {&S12774[107], 4}, {NULL}};
const Segment SGI252787[] = {{&S12774[35], 3}, {NULL}};
const Segment SGI252788[] = {{&S1952[25], 57}, {NULL}};
const Segment SGI252789[] = {{&S1952[0], 82}, {NULL}};
const Segment SGI252790[] = {{&S12774[66], 6}, {NULL}};
const Segment SGI252791[] = {{&S12774[100], 6}, {NULL}};
const Segment SGI252792[] = {{&S1953[13], 7}, {&S1953[27], 28}, {NULL}};
const Segment SGI252793[] = {{&S1953[0], 55}, {NULL}};
const Segment SGI252794[] = {{&S12775[13], 4}, {NULL}};
const Segment SGI252795[] = {{&S12775[33], 5}, {NULL}};
const Segment SGI252796[] = {{&S12775[66], 6}, {NULL}};
const Segment SGI252797[] = {{&S12775[100], 6}, {NULL}};
const Segment SGI252798[] = {{&S1954[13], 7}, {&S1954[27], 28}, {NULL}};
const Segment SGI252799[] = {{&S1954[0], 55}, {NULL}};
const Segment SGI252800[] = {{&S12776[13], 4}, {&S12776[13], 4}, {NULL}};
const Segment SGI252801[] = {{&S12776[33], 5}, {NULL}};
const Segment SGI252802[] = {{&S1955[13], 7}, {&S1955[27], 28}, {NULL}};
const Segment SGI252803[] = {{&S12776[66], 6}, {NULL}};
const Segment SGI252804[] = {{&S1955[0], 55}, {NULL}};
const Segment SGI252805[] = {{&S12776[100], 6}, {NULL}};
const Segment SGI252806[] = {{&S1956[57], 11}, {NULL}};
const Segment SGI252807[] = {{&S1956[0], 68}, {NULL}};
const Segment SGI252808[] = {{&S12777[13], 4}, {&S12777[13], 4}, {&S12777[13], 4}, {NULL}};
const Segment SGI252809[] = {{&S12777[33], 5}, {NULL}};
const Segment SGI252810[] = {{&S12777[66], 6}, {NULL}};
const Segment SGI252811[] = {{&S12777[100], 6}, {NULL}};
const Segment SGI252812[] = {{&S1957[13], 7}, {&S1957[27], 28}, {NULL}};
const Segment SGI252813[] = {{&S1957[0], 55}, {NULL}};
const Segment SGI252814[] = {{&S1958[7], 7}, {NULL}};
const Segment SGI252815[] = {{&S1958[0], 16}, {NULL}};
const Segment SGI252816[] = {{&S12778[13], 6}, {&S12778[109], 4}, {NULL}};
const Segment SGI252817[] = {{&S12778[35], 5}, {NULL}};
const Segment SGI252818[] = {{&S12778[68], 6}, {NULL}};
const Segment SGI252819[] = {{&S12778[102], 6}, {NULL}};
const Segment SGI252820[] = {{&S1959[25], 10}, {NULL}};
const Segment SGI252821[] = {{&S1959[0], 71}, {NULL}};
const Segment SGI252822[] = {{&S12779[13], 4}, {NULL}};
const Segment SGI252823[] = {{&S12779[33], 5}, {NULL}};
const Segment SGI252824[] = {{&S12779[66], 6}, {NULL}};
const Segment SGI252825[] = {{&S1960[19], 6}, {NULL}};
const Segment SGI252826[] = {{&S1960[0], 30}, {NULL}};
const Segment SGI252827[] = {{&S1961[16], 29}, {NULL}};
const Segment SGI252828[] = {{&S1961[0], 45}, {NULL}};
const Segment SGI252829[] = {{&S12780[13], 4}, {&S12780[13], 4}, {&S12780[13], 4}, {NULL}};
const Segment SGI252830[] = {{&S12780[42], 5}, {NULL}};
const Segment SGI252831[] = {{&S1962[0], 9}, {NULL}};
const Segment SGI252832[] = {{&S12781[13], 4}, {NULL}};
const Segment SGI252833[] = {{&S12781[40], 5}, {NULL}};
const Segment SGI252834[] = {{&S1963[25], 13}, {NULL}};
const Segment SGI252835[] = {{&S1963[0], 63}, {NULL}};
const Segment SGI252836[] = {{&S12781[123], 5}, {NULL}};
const Segment SGI252837[] = {{&S12782[13], 4}, {NULL}};
const Segment SGI252838[] = {{&S12782[33], 5}, {NULL}};
const Segment SGI252839[] = {{&S12782[66], 6}, {NULL}};
const Segment SGI252840[] = {{&S1964[25], 23}, {NULL}};
const Segment SGI252841[] = {{&S1964[0], 75}, {NULL}};
const Segment SGI252842[] = {{&S12782[100], 6}, {NULL}};
const Segment SGI252843[] = {{&S1965[20], 9}, {NULL}};
const Segment SGI252844[] = {{&S1965[0], 29}, {NULL}};
const Segment SGI252845[] = {{&S12783[13], 4}, {&S12783[13], 4}, {NULL}};
const Segment SGI252846[] = {{&S12783[33], 5}, {NULL}};
const Segment SGI252847[] = {{&S12783[66], 6}, {NULL}};
const Segment SGI252848[] = {{&S12783[100], 6}, {NULL}};
const Segment SGI252849[] = {{&S1966[34], 46}, {NULL}};
const Segment SGI252850[] = {{&S1966[0], 106}, {NULL}};
const Segment SGI252851[] = {{&S12784[13], 6}, {&S12784[109], 4}, {NULL}};
const Segment SGI252852[] = {{&S12784[35], 5}, {NULL}};
const Segment SGI252853[] = {{&S12784[68], 6}, {NULL}};
const Segment SGI252854[] = {{&S12784[102], 6}, {NULL}};
const Segment SGI252855[] = {{&S1967[25], 14}, {NULL}};
const Segment SGI252856[] = {{&S1967[0], 59}, {NULL}};
const Segment SGI252857[] = {{&S12785[13], 4}, {&S12785[13], 4}, {&S12785[13], 4}, {NULL}};
const Segment SGI252858[] = {{&S12785[42], 5}, {NULL}};
const Segment SGI252859[] = {{&S1968[18], 3}, {&S1968[72], 36}, {&S1968[101], 6}, {NULL}};
const Segment SGI252860[] = {{&S1968[0], 134}, {NULL}};
const Segment SGI252861[] = {{&S12786[13], 4}, {NULL}};
const Segment SGI252862[] = {{&S12786[33], 5}, {NULL}};
const Segment SGI252863[] = {{&S12786[66], 6}, {NULL}};
const Segment SGI252864[] = {{&S12787[13], 4}, {&S12787[23], 6}, {&S12787[13], 4}, {NULL}};
const Segment SGI252865[] = {{&S12787[33], 5}, {&S12787[129], 5}, {NULL}};
const Segment SGI252866[] = {{&S12787[66], 6}, {NULL}};
const Segment SGI252867[] = {{&S12787[100], 6}, {NULL}};
const Segment SGI252868[] = {{&S1971[0], 15}, {NULL}};
const Segment SGI252869[] = {{&S12788[13], 6}, {&S12788[128], 4}, {NULL}};
const Segment SGI252870[] = {{&S12788[75], 6}, {NULL}};
const Segment SGI252871[] = {{&S12788[109], 4}, {NULL}};
const Segment SGI252872[] = {{&S1972[15], 6}, {NULL}};
const Segment SGI252873[] = {{&S1972[0], 53}, {NULL}};
const Segment SGI252874[] = {{&S12789[13], 4}, {NULL}};
const Segment SGI252875[] = {{&S12789[35], 5}, {NULL}};
const Segment SGI252876[] = {{&S1973[26], 3}, {NULL}};
const Segment SGI252877[] = {{&S1973[0], 43}, {NULL}};
const Segment SGI252878[] = {{&S1974[13], 7}, {&S1974[27], 28}, {NULL}};
const Segment SGI252879[] = {{&S1974[0], 55}, {NULL}};
const Segment SGI252880[] = {{&S12790[13], 4}, {NULL}};
const Segment SGI252881[] = {{&S12790[33], 3}, {NULL}};
const Segment SGI252882[] = {{&S12790[64], 6}, {NULL}};
const Segment SGI252883[] = {{&S12790[98], 8}, {NULL}};
const Segment SGI252884[] = {{&S1975[25], 16}, {&S1975[42], 33}, {NULL}};
const Segment SGI252885[] = {{&S1975[0], 75}, {NULL}};
const Segment SGI252886[] = {{&S12791[13], 4}, {NULL}};
const Segment SGI252887[] = {{&S12791[129], 5}, {NULL}};
const Segment SGI252888[] = {{&S12791[33], 3}, {NULL}};
const Segment SGI252889[] = {{&S12791[64], 6}, {NULL}};
const Segment SGI252890[] = {{&S12791[98], 6}, {NULL}};
const Segment SGI252891[] = {{&S1976[25], 7}, {NULL}};
const Segment SGI252892[] = {{&S1976[0], 62}, {NULL}};
const Segment SGI252893[] = {{&S12792[13], 4}, {NULL}};
const Segment SGI252894[] = {{&S12792[33], 5}, {NULL}};
const Segment SGI252895[] = {{&S12792[66], 6}, {NULL}};
const Segment SGI252896[] = {{&S12792[100], 6}, {NULL}};
const Segment SGI252897[] = {{&S12792[114], 7}, {NULL}};
const Segment SGI252898[] = {{&S1977[25], 15}, {NULL}};
const Segment SGI252899[] = {{&S1977[0], 66}, {NULL}};
const Segment SGI252900[] = {{&S12793[13], 4}, {NULL}};
const Segment SGI252901[] = {{&S12793[40], 5}, {NULL}};
const Segment SGI252902[] = {{&S1978[25], 10}, {&S1978[36], 30}, {NULL}};
const Segment SGI252903[] = {{&S12793[123], 5}, {NULL}};
const Segment SGI252904[] = {{&S1978[0], 66}, {NULL}};
const Segment SGI252905[] = {{&S1979[0], 12}, {&S1979[13], 35}, {&S1979[49], 36}, {NULL}};
const Segment SGI252906[] = {{&S1979[0], 85}, {NULL}};
const Segment SGI252907[] = {{&S12794[13], 4}, {&S12794[13], 4}, {&S12794[13], 4}, {NULL}};
const Segment SGI252908[] = {{&S12794[33], 5}, {NULL}};
const Segment SGI252909[] = {{&S12794[66], 6}, {NULL}};
const Segment SGI252910[] = {{&S12794[100], 6}, {NULL}};
const Segment SGI252911[] = {{&S1980[25], 11}, {NULL}};
const Segment SGI252912[] = {{&S1980[0], 65}, {NULL}};
const Segment SGI252913[] = {{&S12795[13], 4}, {NULL}};
const Segment SGI252914[] = {{&S12795[35], 3}, {NULL}};
const Segment SGI252915[] = {{&S1981[0], 35}, {NULL}};
const Segment SGI252916[] = {{&S12796[13], 4}, {NULL}};
const Segment SGI252917[] = {{&S12796[40], 5}, {NULL}};
const Segment SGI252918[] = {{&S12796[123], 5}, {NULL}};
const Segment SGI252919[] = {{&S1982[0], 16}, {NULL}};
const Segment SGI252920[] = {{&S12797[13], 4}, {NULL}};
const Segment SGI252921[] = {{&S1982[0], 123}, {NULL}};
const Segment SGI252922[] = {{&S12797[131], 5}, {NULL}};
const Segment SGI252923[] = {{&S12797[33], 5}, {NULL}};
const Segment SGI252924[] = {{&S12797[66], 6}, {NULL}};
const Segment SGI252925[] = {{&S12797[100], 6}, {NULL}};
const Segment SGI252926[] = {{&S1983[0], 35}, {NULL}};
const Segment SGI252927[] = {{&S12798[13], 4}, {NULL}};
const Segment SGI252928[] = {{&S1984[0], 15}, {NULL}};
const Segment SGI252929[] = {{&S12798[131], 5}, {NULL}};
const Segment SGI252930[] = {{&S12798[33], 5}, {NULL}};
const Segment SGI252931[] = {{&S12798[66], 6}, {NULL}};
const Segment SGI252932[] = {{&S12798[100], 6}, {NULL}};
const Segment SGI252933[] = {{&S1985[19], 6}, {NULL}};
const Segment SGI252934[] = {{&S1985[0], 30}, {NULL}};
const Segment SGI252935[] = {{&S1986[0], 12}, {&S1986[13], 26}, {NULL}};
const Segment SGI252936[] = {{&S1986[0], 39}, {NULL}};
const Segment SGI252937[] = {{&S1987[0], 12}, {NULL}};
const Segment SGI252938[] = {{&S1988[0], 7}, {NULL}};
const Segment SGI252939[] = {{&S1988[0], 54}, {NULL}};
const Segment SGI252940[] = {{&S1989[0], 18}, {NULL}};
const Segment SGI252941[] = {{&S1990[30], 3}, {NULL}};
const Segment SGI252942[] = {{&S1990[0], 47}, {NULL}};
const Segment SGI252943[] = {{&S1991[0], 3}, {NULL}};
const Segment SGI252944[] = {{&S1991[0], 20}, {NULL}};
const Segment SGI252945[] = {{&S1992[0], 11}, {NULL}};
const Segment SGI252946[] = {{&S1993[0], 7}, {&S1993[8], 33}, {NULL}};
const Segment SGI252947[] = {{&S1993[0], 41}, {NULL}};
const Segment SGI252948[] = {{&S1994[0], 16}, {NULL}};
const Segment SGI252949[] = {{&S1995[61], 18}, {NULL}};
const Segment SGI252950[] = {{&S1995[0], 79}, {NULL}};
const Segment SGI252951[] = {{&S1996[36], 3}, {&S1996[45], 31}, {NULL}};
const Segment SGI252952[] = {{&S1996[0], 76}, {NULL}};
const Segment SGI252953[] = {{&S1997[15], 3}, {NULL}};
const Segment SGI252954[] = {{&S1997[0], 54}, {NULL}};
const Segment SGI252955[] = {{&S1998[25], 12}, {NULL}};
const Segment SGI252956[] = {{&S1998[0], 65}, {NULL}};
const Segment SGI252957[] = {{&S2000[0], 36}, {NULL}};
const Segment SGI252958[] = {{&S2001[25], 8}, {NULL}};
const Segment SGI252959[] = {{&S2001[0], 58}, {NULL}};
const Segment SGI252960[] = {{&S2002[19], 6}, {NULL}};
const Segment SGI252961[] = {{&S2002[0], 30}, {NULL}};
const Segment SGI252962[] = {{&S2003[0], 13}, {NULL}};
const Segment SGI252963[] = {{&S2004[0], 17}, {&S2004[20], 31}, {NULL}};
const Segment SGI252964[] = {{&S2004[0], 53}, {NULL}};
const Segment SGI252965[] = {{&S2005[25], 17}, {NULL}};
const Segment SGI252966[] = {{&S2005[0], 80}, {NULL}};
const Segment SGI252967[] = {{&S2006[0], 27}, {&S2006[16], 7}, {NULL}};
const Segment SGI252968[] = {{&S2006[0], 55}, {NULL}};
const Segment SGI252969[] = {{&S2007[0], 17}, {NULL}};
const Segment SGI252970[] = {{&S2008[0], 10}, {&S2008[7], 3}, {NULL}};
const Segment SGI252971[] = {{&S2008[0], 61}, {NULL}};
const Segment SGI252972[] = {{&S2009[47], 39}, {NULL}};
const Segment SGI252973[] = {{&S2009[0], 86}, {NULL}};
const Segment SGI252974[] = {{&S2010[0], 18}, {NULL}};
const Segment SGI252975[] = {{&S2010[0], 38}, {NULL}};
const Segment SGI252976[] = {{&S2011[27], 3}, {NULL}};
const Segment SGI252977[] = {{&S2011[0], 30}, {NULL}};
const Segment SGI252978[] = {{&S2012[25], 11}, {NULL}};
const Segment SGI252979[] = {{&S2012[0], 36}, {NULL}};
const Segment SGI252980[] = {{&S2013[0], 7}, {NULL}};
const Segment SGI252981[] = {{&S2013[0], 49}, {NULL}};
const Segment SGI252982[] = {{&S2014[25], 10}, {NULL}};
const Segment SGI252983[] = {{&S2014[0], 35}, {NULL}};
const Segment SGI252984[] = {{&S101598[34], 10}, {NULL}};
const Segment SGI252985[] = {{&S101598[27], 6}, {NULL}};
const Segment SGI252986[] = {{&S2015[0], 10}, {NULL}};
const Segment SGI252987[] = {{&S101598[96], 9}, {NULL}};
const Segment SGI252988[] = {{&S101598[69], 6}, {NULL}};
const Segment SGI252989[] = {{&S2016[43], 6}, {&S2016[53], 25}, {NULL}};
const Segment SGI252990[] = {{&S2016[0], 86}, {NULL}};
const Segment SGI252991[] = {{&S101599[34], 10}, {NULL}};
const Segment SGI252992[] = {{&S101599[27], 6}, {NULL}};
const Segment SGI252993[] = {{&S101599[96], 9}, {NULL}};
const Segment SGI252994[] = {{&S101599[69], 6}, {NULL}};
const Segment SGI252995[] = {{&S2017[25], 6}, {&S2017[43], 25}, {NULL}};
const Segment SGI252996[] = {{&S2017[0], 68}, {NULL}};
const Segment SGI252997[] = {{&S2018[25], 5}, {NULL}};
const Segment SGI252998[] = {{&S2018[0], 30}, {NULL}};
const Segment SGI252999[] = {{&S2019[25], 17}, {&S2019[43], 28}, {NULL}};
const Segment SGI253000[] = {{&S2019[0], 71}, {NULL}};
const Segment SGI253001[] = {{&S2020[25], 23}, {NULL}};
const Segment SGI253002[] = {{&S2020[0], 48}, {NULL}};
const Segment SGI253003[] = {{&S2021[0], 3}, {NULL}};
const Segment SGI253004[] = {{&S2021[0], 27}, {NULL}};
const Segment SGI253005[] = {{&S2022[0], 10}, {NULL}};
const Segment SGI253006[] = {{&S2022[0], 28}, {NULL}};
const Segment SGI253007[] = {{&S2023[10], 7}, {&S2023[24], 39}, {NULL}};
const Segment SGI253008[] = {{&S2023[0], 63}, {NULL}};
const Segment SGI253009[] = {{&S2024[0], 13}, {NULL}};
const Segment SGI253010[] = {{&S2024[0], 32}, {NULL}};
const Segment SGI253011[] = {{&S2025[0], 12}, {&S2025[13], 35}, {&S2025[49], 43}, {NULL}};
const Segment SGI253012[] = {{&S2025[0], 92}, {NULL}};
const Segment SGI253013[] = {{&S2026[0], 10}, {NULL}};
const Segment SGI253014[] = {{&S2027[11], 7}, {NULL}};
const Segment SGI253015[] = {{&S2027[0], 39}, {NULL}};
const Segment SGI253016[] = {{&S2028[8], 3}, {NULL}};
const Segment SGI253017[] = {{&S2028[0], 11}, {NULL}};
const Segment SGI253018[] = {{&S2029[25], 10}, {&S2029[57], 7}, {NULL}};
const Segment SGI253019[] = {{&S2029[0], 82}, {NULL}};
const Segment SGI253020[] = {{&S2030[0], 13}, {NULL}};
const Segment SGI253021[] = {{&S2030[0], 28}, {NULL}};
const Segment SGI253022[] = {{&S2031[25], 14}, {&S2031[40], 31}, {&S2031[62], 3}, {NULL}};
const Segment SGI253023[] = {{&S2031[0], 87}, {NULL}};
const Segment SGI253024[] = {{&S2032[25], 13}, {&S2032[39], 30}, {NULL}};
const Segment SGI253025[] = {{&S2032[0], 69}, {NULL}};
const Segment SGI253026[] = {{&S2034[0], 9}, {NULL}};
const Segment SGI253027[] = {{&S2035[0], 15}, {&S2035[18], 42}, {NULL}};
const Segment SGI253028[] = {{&S2035[0], 60}, {NULL}};
const Segment SGI253029[] = {{&S2036[0], 8}, {NULL}};
const Segment SGI253030[] = {{&S2037[0], 11}, {NULL}};
const Segment SGI253031[] = {{&S2038[12], 23}, {NULL}};
const Segment SGI253032[] = {{&S2038[0], 35}, {NULL}};
const Segment SGI253033[] = {{&S2039[0], 6}, {NULL}};
const Segment SGI253034[] = {{&S2040[0], 12}, {&S2040[13], 35}, {&S2040[49], 43}, {NULL}};
const Segment SGI253035[] = {{&S2040[0], 92}, {NULL}};
const Segment SGI253036[] = {{&S2041[0], 12}, {&S2041[13], 35}, {&S2041[49], 45}, {NULL}};
const Segment SGI253037[] = {{&S2041[0], 94}, {NULL}};
const Segment SGI253038[] = {{&S2042[0], 12}, {&S2042[13], 35}, {&S2042[49], 43}, {NULL}};
const Segment SGI253039[] = {{&S2042[0], 92}, {NULL}};
const Segment SGI253040[] = {{&S2043[25], 21}, {NULL}};
const Segment SGI253041[] = {{&S2043[0], 85}, {NULL}};
const Segment SGI253042[] = {{&S2044[25], 13}, {NULL}};
const Segment SGI253043[] = {{&S2044[0], 85}, {NULL}};
const Segment SGI253044[] = {{&S2045[0], 11}, {&S2045[12], 7}, {&S2045[29], 29}, {NULL}};
const Segment SGI253045[] = {{&S2045[0], 58}, {NULL}};
const Segment SGI253046[] = {{&S2046[0], 13}, {&S2046[14], 39}, {NULL}};
const Segment SGI253047[] = {{&S2046[0], 53}, {NULL}};
const Segment SGI253048[] = {{&S2047[12], 15}, {NULL}};
const Segment SGI253049[] = {{&S2047[0], 27}, {NULL}};
const Segment SGI253050[] = {{&S2048[0], 11}, {NULL}};
const Segment SGI253051[] = {{&S2049[0], 13}, {&S2049[14], 24}, {NULL}};
const Segment SGI253052[] = {{&S2049[0], 38}, {NULL}};
const Segment SGI253053[] = {{&S2050[25], 18}, {&S2050[44], 33}, {NULL}};
const Segment SGI253054[] = {{&S2050[0], 77}, {NULL}};
const Segment SGI253055[] = {{&S2051[0], 20}, {NULL}};
const Segment SGI253056[] = {{&S2051[0], 104}, {NULL}};
const Segment SGI253057[] = {{&S2052[13], 34}, {NULL}};
const Segment SGI253058[] = {{&S2052[0], 47}, {NULL}};
const Segment SGI253059[] = {{&S2053[25], 15}, {&S2053[41], 36}, {NULL}};
const Segment SGI253060[] = {{&S2053[0], 77}, {NULL}};
const Segment SGI253061[] = {{&S2054[25], 16}, {NULL}};
const Segment SGI253062[] = {{&S2054[0], 67}, {NULL}};
const Segment SGI253063[] = {{&S2055[60], 7}, {NULL}};
const Segment SGI253064[] = {{&S2055[0], 67}, {NULL}};
const Segment SGI253065[] = {{&S2056[44], 24}, {NULL}};
const Segment SGI253066[] = {{&S2056[0], 68}, {NULL}};
const Segment SGI253067[] = {{&S2057[0], 12}, {&S2057[13], 36}, {NULL}};
const Segment SGI253068[] = {{&S2057[0], 49}, {NULL}};
const Segment SGI253069[] = {{&S2058[10], 15}, {NULL}};
const Segment SGI253070[] = {{&S2058[0], 25}, {NULL}};
const Segment SGI253071[] = {{&S2059[0], 14}, {&S2059[15], 35}, {NULL}};
const Segment SGI253072[] = {{&S2059[0], 50}, {NULL}};
const Segment SGI253073[] = {{&S2060[8], 29}, {NULL}};
const Segment SGI253074[] = {{&S2060[0], 37}, {NULL}};
const Segment SGI253075[] = {{&S2061[21], 8}, {&S2061[30], 49}, {NULL}};
const Segment SGI253076[] = {{&S2061[0], 79}, {NULL}};
const Segment SGI253077[] = {{&S2062[25], 17}, {NULL}};
const Segment SGI253078[] = {{&S2062[0], 111}, {NULL}};
const Segment SGI253079[] = {{&S2063[8], 3}, {NULL}};
const Segment SGI253080[] = {{&S2063[0], 11}, {NULL}};
const Segment SGI253081[] = {{&S2064[15], 9}, {NULL}};
const Segment SGI253082[] = {{&S2064[0], 24}, {NULL}};
const Segment SGI253083[] = {{&S2065[18], 3}, {NULL}};
const Segment SGI253084[] = {{&S2065[0], 54}, {NULL}};
const Segment SGI253085[] = {{&S2066[0], 7}, {NULL}};
const Segment SGI253086[] = {{&S2066[0], 11}, {NULL}};
const Segment SGI253087[] = {{&S2067[25], 16}, {&S2067[42], 28}, {NULL}};
const Segment SGI253088[] = {{&S2067[0], 70}, {NULL}};
const Segment SGI253089[] = {{&S2068[9], 19}, {NULL}};
const Segment SGI253090[] = {{&S2068[0], 63}, {NULL}};
const Segment SGI253091[] = {{&S2069[0], 12}, {&S2069[13], 35}, {&S2069[49], 43}, {NULL}};
const Segment SGI253092[] = {{&S2069[0], 92}, {NULL}};
const Segment SGI253093[] = {{&S2070[25], 9}, {&S2070[40], 32}, {NULL}};
const Segment SGI253094[] = {{&S2070[0], 72}, {NULL}};
const Segment SGI253095[] = {{&S2071[0], 12}, {&S2071[13], 35}, {&S2071[49], 43}, {NULL}};
const Segment SGI253096[] = {{&S2071[0], 92}, {NULL}};
const Segment SGI253097[] = {{&S2072[0], 16}, {NULL}};
const Segment SGI253098[] = {{&S2072[0], 44}, {NULL}};
const Segment SGI253099[] = {{&S2073[0], 10}, {NULL}};
const Segment SGI253100[] = {{&S2073[0], 14}, {NULL}};
const Segment SGI253101[] = {{&S2074[25], 3}, {NULL}};
const Segment SGI253102[] = {{&S2074[0], 29}, {NULL}};
const Segment SGI253103[] = {{&S2075[25], 17}, {NULL}};
const Segment SGI253104[] = {{&S2075[0], 68}, {NULL}};
const Segment SGI253105[] = {{&S2077[25], 15}, {&S2077[41], 32}, {NULL}};
const Segment SGI253106[] = {{&S2077[0], 73}, {NULL}};
const Segment SGI253107[] = {{&S2078[25], 15}, {&S2078[41], 36}, {NULL}};
const Segment SGI253108[] = {{&S2078[0], 77}, {NULL}};
const Segment SGI253109[] = {{&S2079[25], 13}, {&S2079[39], 30}, {NULL}};
const Segment SGI253110[] = {{&S2079[0], 69}, {NULL}};
const Segment SGI253111[] = {{&S2080[0], 9}, {NULL}};
const Segment SGI253112[] = {{&S2081[25], 17}, {NULL}};
const Segment SGI253113[] = {{&S2081[0], 67}, {NULL}};
const Segment SGI253114[] = {{&S2082[0], 14}, {NULL}};
const Segment SGI253115[] = {{&S2083[10], 3}, {NULL}};
const Segment SGI253116[] = {{&S2083[0], 13}, {NULL}};
const Segment SGI253117[] = {{&S2084[25], 11}, {NULL}};
const Segment SGI253118[] = {{&S2084[0], 66}, {NULL}};
const Segment SGI253119[] = {{&S2085[0], 77}, {NULL}};
const Segment SGI253120[] = {{&S2087[25], 16}, {&S2087[42], 51}, {NULL}};
const Segment SGI253121[] = {{&S2087[0], 122}, {NULL}};
const Segment SGI253122[] = {{&S2088[57], 3}, {NULL}};
const Segment SGI253123[] = {{&S2088[0], 85}, {NULL}};
const Segment SGI253124[] = {{&S2089[25], 17}, {&S2089[43], 43}, {NULL}};
const Segment SGI253125[] = {{&S2089[0], 86}, {NULL}};
const Segment SGI253126[] = {{&S2090[25], 14}, {&S2090[40], 27}, {NULL}};
const Segment SGI253127[] = {{&S2090[0], 67}, {NULL}};
const Segment SGI253128[] = {{&S2091[25], 9}, {NULL}};
const Segment SGI253129[] = {{&S2091[0], 79}, {NULL}};
const Segment SGI253130[] = {{&S2092[0], 12}, {&S2092[13], 35}, {&S2092[49], 43}, {NULL}};
const Segment SGI253131[] = {{&S2092[0], 92}, {NULL}};
const Segment SGI253132[] = {{&S2093[0], 12}, {&S2093[13], 35}, {&S2093[49], 43}, {NULL}};
const Segment SGI253133[] = {{&S2093[0], 92}, {NULL}};
const Segment SGI253134[] = {{&S2094[0], 12}, {&S2094[13], 35}, {&S2094[49], 43}, {NULL}};
const Segment SGI253135[] = {{&S2094[0], 92}, {NULL}};
const Segment SGI253136[] = {{&S2095[0], 12}, {&S2095[13], 35}, {&S2095[49], 43}, {NULL}};
const Segment SGI253137[] = {{&S2095[0], 92}, {NULL}};
const Segment SGI253138[] = {{&S2096[0], 12}, {&S2096[13], 35}, {&S2096[49], 43}, {NULL}};
const Segment SGI253139[] = {{&S2096[0], 92}, {NULL}};
const Segment SGI253140[] = {{&S2097[0], 12}, {&S2097[13], 35}, {&S2097[49], 43}, {NULL}};
const Segment SGI253141[] = {{&S2097[0], 92}, {NULL}};
const Segment SGI253142[] = {{&S2098[0], 12}, {&S2098[13], 35}, {&S2098[49], 43}, {NULL}};
const Segment SGI253143[] = {{&S2098[0], 92}, {NULL}};
const Segment SGI253144[] = {{&S2099[0], 8}, {NULL}};
const Segment SGI253145[] = {{&S2099[0], 12}, {NULL}};
const Segment SGI253146[] = {{&S2100[5], 14}, {NULL}};
const Segment SGI253147[] = {{&S2100[0], 34}, {NULL}};
const Segment SGI253148[] = {{&S2102[0], 7}, {NULL}};
const Segment SGI253149[] = {{&S2103[25], 15}, {NULL}};
const Segment SGI253150[] = {{&S2103[0], 66}, {NULL}};
const Segment SGI253151[] = {{&S2104[0], 5}, {NULL}};
const Segment SGI253152[] = {{&S2104[0], 66}, {NULL}};
const Segment SGI253153[] = {{&S2105[11], 6}, {NULL}};
const Segment SGI253154[] = {{&S2105[0], 72}, {NULL}};
const Segment SGI253155[] = {{&S2106[0], 4}, {NULL}};
const Segment SGI253156[] = {{&S2106[0], 11}, {NULL}};
const Segment SGI253157[] = {{&S2107[0], 5}, {&S2107[87], 3}, {NULL}};
const Segment SGI253158[] = {{&S2107[0], 104}, {NULL}};
const Segment SGI253159[] = {{&S2108[8], 6}, {&S2108[15], 27}, {NULL}};
const Segment SGI253160[] = {{&S2108[0], 42}, {NULL}};
const Segment SGI253161[] = {{&S2109[15], 7}, {NULL}};
const Segment SGI253162[] = {{&S2109[0], 22}, {NULL}};
const Segment SGI253163[] = {{&S2110[25], 9}, {&S2110[35], 35}, {NULL}};
const Segment SGI253164[] = {{&S2110[0], 70}, {NULL}};
const Segment SGI253165[] = {{&S2111[40], 3}, {NULL}};
const Segment SGI253166[] = {{&S2111[0], 44}, {NULL}};
const Segment SGI253167[] = {{&S2112[0], 21}, {NULL}};
const Segment SGI253168[] = {{&S2113[25], 11}, {&S2113[37], 23}, {NULL}};
const Segment SGI253169[] = {{&S2113[0], 60}, {NULL}};
const Segment SGI253170[] = {{&S2114[23], 8}, {NULL}};
const Segment SGI253171[] = {{&S2114[0], 51}, {NULL}};
const Segment SGI253172[] = {{&S2115[5], 3}, {NULL}};
const Segment SGI253173[] = {{&S2115[0], 8}, {NULL}};
const Segment SGI253174[] = {{&S2116[0], 10}, {NULL}};
const Segment SGI253175[] = {{&S2117[0], 12}, {&S2117[13], 35}, {&S2117[49], 43}, {NULL}};
const Segment SGI253176[] = {{&S2117[0], 92}, {NULL}};
const Segment SGI253177[] = {{&S2118[0], 12}, {&S2118[13], 35}, {&S2118[49], 43}, {NULL}};
const Segment SGI253178[] = {{&S2118[0], 92}, {NULL}};
const Segment SGI253179[] = {{&S2119[0], 12}, {&S2119[13], 35}, {&S2119[49], 43}, {NULL}};
const Segment SGI253180[] = {{&S2119[0], 92}, {NULL}};
const Segment SGI253181[] = {{&S2120[25], 12}, {&S2120[38], 30}, {NULL}};
const Segment SGI253182[] = {{&S2120[0], 68}, {NULL}};
const Segment SGI253183[] = {{&S2121[25], 11}, {NULL}};
const Segment SGI253184[] = {{&S2121[0], 70}, {NULL}};
const Segment SGI253185[] = {{&S2122[25], 61}, {NULL}};
const Segment SGI253186[] = {{&S2122[0], 86}, {NULL}};
const Segment SGI253187[] = {{&S2123[25], 15}, {&S2123[41], 31}, {NULL}};
const Segment SGI253188[] = {{&S2123[0], 72}, {NULL}};
const Segment SGI253189[] = {{&S2124[0], 20}, {NULL}};
const Segment SGI253190[] = {{&S2125[0], 35}, {NULL}};
const Segment SGI253191[] = {{&S2126[35], 7}, {NULL}};
const Segment SGI253192[] = {{&S2126[0], 72}, {NULL}};
const Segment SGI253193[] = {{&S2127[25], 19}, {NULL}};
const Segment SGI253194[] = {{&S2127[0], 70}, {NULL}};
const Segment SGI253195[] = {{&S2128[8], 7}, {NULL}};
const Segment SGI253196[] = {{&S2128[0], 15}, {NULL}};
const Segment SGI253197[] = {{&S2129[0], 12}, {&S2129[13], 35}, {&S2129[49], 43}, {NULL}};
const Segment SGI253198[] = {{&S2129[0], 92}, {NULL}};
const Segment SGI253199[] = {{&S2130[0], 12}, {&S2130[13], 35}, {&S2130[49], 43}, {NULL}};
const Segment SGI253200[] = {{&S2130[0], 92}, {NULL}};
const Segment SGI253201[] = {{&S2131[0], 12}, {&S2131[13], 35}, {&S2131[49], 43}, {NULL}};
const Segment SGI253202[] = {{&S2131[0], 92}, {NULL}};
const Segment SGI253203[] = {{&S2132[42], 3}, {NULL}};
const Segment SGI253204[] = {{&S2132[0], 50}, {NULL}};
const Segment SGI253205[] = {{&S2133[129], 18}, {NULL}};
const Segment SGI253206[] = {{&S2133[0], 147}, {NULL}};
const Segment SGI253207[] = {{&S2134[25], 17}, {NULL}};
const Segment SGI253208[] = {{&S2134[0], 42}, {NULL}};
const Segment SGI253209[] = {{&S2135[9], 6}, {NULL}};
const Segment SGI253210[] = {{&S2135[0], 21}, {NULL}};
const Segment SGI253211[] = {{&S2136[0], 35}, {NULL}};
const Segment SGI253212[] = {{&S2137[25], 13}, {NULL}};
const Segment SGI253213[] = {{&S2137[0], 62}, {NULL}};
const Segment SGI253214[] = {{&S2138[25], 16}, {&S2138[42], 42}, {NULL}};
const Segment SGI253215[] = {{&S2138[0], 84}, {NULL}};
const Segment SGI253216[] = {{&S2139[0], 16}, {NULL}};
const Segment SGI253217[] = {{&S2139[0], 55}, {NULL}};
const Segment SGI253218[] = {{&S2140[0], 11}, {NULL}};
const Segment SGI253219[] = {{&S2141[26], 15}, {&S2141[42], 37}, {NULL}};
const Segment SGI253220[] = {{&S2141[0], 79}, {NULL}};
const Segment SGI253221[] = {{&S2142[0], 7}, {NULL}};
const Segment SGI253222[] = {{&S2143[25], 15}, {&S2143[41], 31}, {NULL}};
const Segment SGI253223[] = {{&S2143[0], 72}, {NULL}};
const Segment SGI253224[] = {{&S2144[25], 13}, {NULL}};
const Segment SGI253225[] = {{&S2144[0], 85}, {NULL}};
const Segment SGI253226[] = {{&S2145[40], 36}, {NULL}};
const Segment SGI253227[] = {{&S2145[0], 76}, {NULL}};
const Segment SGI253228[] = {{&S2146[11], 7}, {&S2146[23], 24}, {NULL}};
const Segment SGI253229[] = {{&S2146[0], 47}, {NULL}};
const Segment SGI253230[] = {{&S2147[16], 7}, {NULL}};
const Segment SGI253231[] = {{&S2147[0], 60}, {NULL}};
const Segment SGI253232[] = {{&S2148[12], 7}, {NULL}};
const Segment SGI253233[] = {{&S2148[0], 35}, {NULL}};
const Segment SGI253234[] = {{&S2149[0], 12}, {&S2149[13], 35}, {&S2149[49], 43}, {NULL}};
const Segment SGI253235[] = {{&S2149[0], 92}, {NULL}};
const Segment SGI253236[] = {{&S2150[0], 12}, {&S2150[13], 35}, {&S2150[49], 43}, {NULL}};
const Segment SGI253237[] = {{&S2150[0], 92}, {NULL}};
const Segment SGI253238[] = {{&S2151[0], 11}, {NULL}};
const Segment SGI253239[] = {{&S2151[0], 34}, {NULL}};
const Segment SGI253240[] = {{&S2152[18], 20}, {NULL}};
const Segment SGI253241[] = {{&S2152[0], 38}, {NULL}};
const Segment SGI253242[] = {{&S2153[0], 27}, {NULL}};
const Segment SGI253243[] = {{&S2154[0], 7}, {NULL}};
const Segment SGI253244[] = {{&S2154[0], 54}, {NULL}};
const Segment SGI253245[] = {{&S2155[0], 8}, {NULL}};
const Segment SGI253246[] = {{&S2156[25], 12}, {NULL}};
const Segment SGI253247[] = {{&S2156[0], 66}, {NULL}};
const Segment SGI253248[] = {{&S2157[0], 16}, {&S2157[17], 41}, {NULL}};
const Segment SGI253249[] = {{&S2157[0], 58}, {NULL}};
const Segment SGI253250[] = {{&S2158[8], 7}, {NULL}};
const Segment SGI253251[] = {{&S2158[0], 15}, {NULL}};
const Segment SGI253252[] = {{&S2159[25], 15}, {NULL}};
const Segment SGI253253[] = {{&S2159[0], 65}, {NULL}};
const Segment SGI253254[] = {{&S2160[19], 9}, {NULL}};
const Segment SGI253255[] = {{&S2160[0], 28}, {NULL}};
const Segment SGI253256[] = {{&S2161[0], 12}, {&S2161[13], 35}, {&S2161[49], 43}, {NULL}};
const Segment SGI253257[] = {{&S2161[0], 92}, {NULL}};
const Segment SGI253258[] = {{&S2162[0], 12}, {&S2162[13], 35}, {&S2162[49], 43}, {NULL}};
const Segment SGI253259[] = {{&S2162[0], 92}, {NULL}};
const Segment SGI253260[] = {{&S2163[0], 18}, {NULL}};
const Segment SGI253261[] = {{&S2164[12], 16}, {&S2164[29], 31}, {NULL}};
const Segment SGI253262[] = {{&S2164[0], 60}, {NULL}};
const Segment SGI253263[] = {{&S2165[0], 7}, {NULL}};
const Segment SGI253264[] = {{&S2166[17], 7}, {NULL}};
const Segment SGI253265[] = {{&S2166[0], 115}, {NULL}};
const Segment SGI253266[] = {{&S2167[25], 18}, {&S2167[44], 33}, {NULL}};
const Segment SGI253267[] = {{&S2167[0], 77}, {NULL}};
const Segment SGI253268[] = {{&S2168[25], 14}, {NULL}};
const Segment SGI253269[] = {{&S2168[0], 66}, {NULL}};
const Segment SGI253270[] = {{&S2169[9], 3}, {NULL}};
const Segment SGI253271[] = {{&S2169[0], 12}, {NULL}};
const Segment SGI253272[] = {{&S2170[0], 11}, {&S2170[14], 39}, {NULL}};
const Segment SGI253273[] = {{&S2170[0], 53}, {NULL}};
const Segment SGI253274[] = {{&S2171[7], 21}, {NULL}};
const Segment SGI253275[] = {{&S2171[0], 28}, {NULL}};
const Segment SGI253276[] = {{&S2172[0], 13}, {NULL}};
const Segment SGI253277[] = {{&S2173[0], 10}, {&S2173[11], 35}, {NULL}};
const Segment SGI253278[] = {{&S2173[0], 46}, {NULL}};
const Segment SGI253279[] = {{&S2174[25], 16}, {NULL}};
const Segment SGI253280[] = {{&S2174[0], 76}, {NULL}};
const Segment SGI253281[] = {{&S2175[0], 12}, {&S2175[13], 35}, {&S2175[49], 43}, {NULL}};
const Segment SGI253282[] = {{&S2175[0], 92}, {NULL}};
const Segment SGI253283[] = {{&S2176[0], 15}, {NULL}};
const Segment SGI253284[] = {{&S2176[0], 19}, {NULL}};
const Segment SGI253285[] = {{&S2177[0], 12}, {&S2177[13], 35}, {&S2177[49], 43}, {NULL}};
const Segment SGI253286[] = {{&S2177[0], 92}, {NULL}};
const Segment SGI253287[] = {{&S2178[0], 12}, {&S2178[13], 35}, {&S2178[49], 43}, {NULL}};
const Segment SGI253288[] = {{&S2178[0], 92}, {NULL}};
const Segment SGI253289[] = {{&S2179[25], 14}, {NULL}};
const Segment SGI253290[] = {{&S2179[0], 63}, {NULL}};
const Segment SGI253291[] = {{&S2180[0], 21}, {&S2180[22], 43}, {NULL}};
const Segment SGI253292[] = {{&S2180[0], 65}, {NULL}};
const Segment SGI253293[] = {{&S2181[0], 11}, {NULL}};
const Segment SGI253294[] = {{&S2181[0], 123}, {NULL}};
const Segment SGI253295[] = {{&S2182[15], 7}, {NULL}};
const Segment SGI253296[] = {{&S2182[0], 22}, {NULL}};
const Segment SGI253297[] = {{&S2183[0], 9}, {NULL}};
const Segment SGI253298[] = {{&S2184[25], 15}, {&S2184[41], 31}, {NULL}};
const Segment SGI253299[] = {{&S2184[0], 72}, {NULL}};
const Segment SGI253300[] = {{&S2185[17], 6}, {NULL}};
const Segment SGI253301[] = {{&S2185[0], 34}, {NULL}};
const Segment SGI253302[] = {{&S2186[0], 16}, {NULL}};
const Segment SGI253303[] = {{&S2186[0], 44}, {NULL}};
const Segment SGI253304[] = {{&S2187[11], 11}, {NULL}};
const Segment SGI253305[] = {{&S2187[0], 22}, {NULL}};
const Segment SGI253306[] = {{&S2188[25], 16}, {NULL}};
const Segment SGI253307[] = {{&S2188[0], 41}, {NULL}};
const Segment SGI253308[] = {{&S2189[25], 28}, {&S2189[54], 46}, {NULL}};
const Segment SGI253309[] = {{&S2189[0], 100}, {NULL}};
const Segment SGI253310[] = {{&S2190[0], 6}, {NULL}};
const Segment SGI253311[] = {{&S2190[0], 66}, {NULL}};
const Segment SGI253312[] = {{&S2191[25], 17}, {NULL}};
const Segment SGI253313[] = {{&S2191[0], 67}, {NULL}};
const Segment SGI253314[] = {{&S2192[25], 10}, {&S2192[36], 31}, {NULL}};
const Segment SGI253315[] = {{&S2192[0], 67}, {NULL}};
const Segment SGI253316[] = {{&S2193[0], 23}, {NULL}};
const Segment SGI253317[] = {{&S2194[53], 32}, {NULL}};
const Segment SGI253318[] = {{&S2194[0], 85}, {NULL}};
const Segment SGI253319[] = {{&S2195[25], 12}, {&S2195[38], 33}, {NULL}};
const Segment SGI253320[] = {{&S2195[0], 71}, {NULL}};
const Segment SGI253321[] = {{&S2196[0], 74}, {NULL}};
const Segment SGI253322[] = {{&S2197[25], 16}, {&S2197[42], 26}, {NULL}};
const Segment SGI253323[] = {{&S2197[0], 68}, {NULL}};
const Segment SGI253324[] = {{&S2198[0], 7}, {NULL}};
const Segment SGI253325[] = {{&S2198[0], 11}, {NULL}};
const Segment SGI253326[] = {{&S2199[0], 12}, {&S2199[13], 35}, {&S2199[49], 43}, {NULL}};
const Segment SGI253327[] = {{&S2199[0], 92}, {NULL}};
const Segment SGI253328[] = {{&S2200[29], 28}, {NULL}};
const Segment SGI253329[] = {{&S2200[0], 57}, {NULL}};
const Segment SGI253330[] = {{&S2201[25], 7}, {NULL}};
const Segment SGI253331[] = {{&S2201[0], 46}, {NULL}};
const Segment SGI253332[] = {{&S2202[10], 7}, {NULL}};
const Segment SGI253333[] = {{&S2202[0], 69}, {NULL}};
const Segment SGI253334[] = {{&S2203[0], 11}, {NULL}};
const Segment SGI253335[] = {{&S2204[0], 21}, {NULL}};
const Segment SGI253336[] = {{&S2204[0], 61}, {NULL}};
const Segment SGI253337[] = {{&S2205[0], 12}, {&S2205[13], 35}, {&S2205[49], 43}, {NULL}};
const Segment SGI253338[] = {{&S2205[0], 92}, {NULL}};
const Segment SGI253339[] = {{&S2206[0], 7}, {NULL}};
const Segment SGI253340[] = {{&S2206[0], 45}, {NULL}};
const Segment SGI253341[] = {{&S2207[25], 11}, {&S2207[37], 30}, {NULL}};
const Segment SGI253342[] = {{&S2207[0], 67}, {NULL}};
const Segment SGI253343[] = {{&S2208[0], 10}, {NULL}};
const Segment SGI253344[] = {{&S2208[0], 15}, {NULL}};
const Segment SGI253345[] = {{&S2209[0], 11}, {NULL}};
const Segment SGI253346[] = {{&S2209[0], 110}, {NULL}};
const Segment SGI253347[] = {{&S2210[0], 12}, {&S2210[13], 35}, {&S2210[49], 43}, {NULL}};
const Segment SGI253348[] = {{&S2210[0], 92}, {NULL}};
const Segment SGI253349[] = {{&S2211[0], 12}, {&S2211[13], 35}, {&S2211[49], 43}, {NULL}};
const Segment SGI253350[] = {{&S2211[0], 92}, {NULL}};
const Segment SGI253351[] = {{&S2212[38], 46}, {NULL}};
const Segment SGI253352[] = {{&S2212[0], 110}, {NULL}};
const Segment SGI253353[] = {{&S2213[0], 12}, {&S2213[13], 35}, {&S2213[49], 43}, {NULL}};
const Segment SGI253354[] = {{&S2213[0], 92}, {NULL}};
const Segment SGI253355[] = {{&S2214[0], 12}, {&S2214[13], 35}, {&S2214[49], 43}, {NULL}};
const Segment SGI253356[] = {{&S2214[0], 92}, {NULL}};
const Segment SGI253357[] = {{&S2215[0], 12}, {&S2215[13], 35}, {&S2215[49], 43}, {NULL}};
const Segment SGI253358[] = {{&S2215[0], 92}, {NULL}};
const Segment SGI253359[] = {{&S2216[0], 10}, {NULL}};
const Segment SGI253360[] = {{&S2217[35], 38}, {NULL}};
const Segment SGI253361[] = {{&S2217[0], 98}, {NULL}};
const Segment SGI253362[] = {{&S2218[11], 16}, {NULL}};
const Segment SGI253363[] = {{&S2218[0], 48}, {NULL}};
const Segment SGI253364[] = {{&S2219[25], 11}, {&S2219[37], 33}, {NULL}};
const Segment SGI253365[] = {{&S2219[0], 70}, {NULL}};
const Segment SGI253366[] = {{&S2220[25], 11}, {&S2220[37], 7}, {NULL}};
const Segment SGI253367[] = {{&S2220[0], 57}, {NULL}};
const Segment SGI253368[] = {{&S2221[0], 12}, {&S2221[13], 32}, {&S2221[46], 25}, {NULL}};
const Segment SGI253369[] = {{&S2221[0], 71}, {NULL}};
const Segment SGI253370[] = {{&S2222[0], 12}, {&S2222[13], 43}, {&S2222[57], 31}, {NULL}};
const Segment SGI253371[] = {{&S2222[0], 88}, {NULL}};
const Segment SGI253372[] = {{&S2223[25], 12}, {&S2223[38], 46}, {NULL}};
const Segment SGI253373[] = {{&S2223[0], 110}, {NULL}};
const Segment SGI253374[] = {{&S2224[25], 15}, {&S2224[41], 38}, {NULL}};
const Segment SGI253375[] = {{&S2224[0], 79}, {NULL}};
const Segment SGI253376[] = {{&S2225[0], 15}, {NULL}};
const Segment SGI253377[] = {{&S2226[0], 12}, {&S2226[13], 35}, {&S2226[49], 43}, {NULL}};
const Segment SGI253378[] = {{&S2226[0], 92}, {NULL}};
const Segment SGI253379[] = {{&S2227[16], 7}, {NULL}};
const Segment SGI253380[] = {{&S2227[0], 23}, {NULL}};
const Segment SGI253381[] = {{&S2228[0], 12}, {&S2228[13], 35}, {&S2228[49], 43}, {NULL}};
const Segment SGI253382[] = {{&S2228[0], 92}, {NULL}};
const Segment SGI253383[] = {{&S2229[0], 12}, {&S2229[13], 35}, {&S2229[49], 43}, {NULL}};
const Segment SGI253384[] = {{&S2229[0], 92}, {NULL}};
const Segment SGI253385[] = {{&S2230[0], 12}, {&S2230[13], 35}, {&S2230[49], 43}, {NULL}};
const Segment SGI253386[] = {{&S2230[0], 92}, {NULL}};
const Segment SGI253387[] = {{&S2231[0], 12}, {&S2231[13], 35}, {&S2231[49], 43}, {NULL}};
const Segment SGI253388[] = {{&S2231[0], 92}, {NULL}};
const Segment SGI253389[] = {{&S2232[40], 37}, {&S2232[61], 7}, {NULL}};
const Segment SGI253390[] = {{&S2232[0], 99}, {NULL}};
const Segment SGI253391[] = {{&S2233[25], 15}, {&S2233[41], 55}, {NULL}};
const Segment SGI253392[] = {{&S2233[0], 96}, {NULL}};
const Segment SGI253393[] = {{&S2234[25], 12}, {NULL}};
const Segment SGI253394[] = {{&S2234[0], 76}, {NULL}};
const Segment SGI253395[] = {{&S2235[13], 7}, {NULL}};
const Segment SGI253396[] = {{&S2235[0], 41}, {NULL}};
const Segment SGI253397[] = {{&S2236[0], 10}, {NULL}};
const Segment SGI253398[] = {{&S2236[0], 59}, {NULL}};
const Segment SGI253399[] = {{&S2237[25], 12}, {&S2237[38], 29}, {NULL}};
const Segment SGI253400[] = {{&S2237[0], 67}, {NULL}};
const Segment SGI253401[] = {{&S2238[27], 7}, {&S2238[27], 7}, {NULL}};
const Segment SGI253402[] = {{&S2238[0], 80}, {NULL}};
const Segment SGI253403[] = {{&S2239[0], 12}, {&S2239[13], 35}, {&S2239[49], 43}, {NULL}};
const Segment SGI253404[] = {{&S2239[0], 92}, {NULL}};
const Segment SGI253405[] = {{&S2240[0], 12}, {&S2240[13], 35}, {&S2240[49], 43}, {NULL}};
const Segment SGI253406[] = {{&S2240[0], 92}, {NULL}};
const Segment SGI253407[] = {{&S2241[0], 20}, {NULL}};
const Segment SGI253408[] = {{&S2242[8], 7}, {NULL}};
const Segment SGI253409[] = {{&S2242[0], 36}, {NULL}};
const Segment SGI253410[] = {{&S2243[25], 15}, {&S2243[41], 55}, {NULL}};
const Segment SGI253411[] = {{&S2243[0], 96}, {NULL}};
const Segment SGI253412[] = {{&S2244[25], 15}, {&S2244[41], 55}, {NULL}};
const Segment SGI253413[] = {{&S2244[0], 96}, {NULL}};
const Segment SGI253414[] = {{&S2245[25], 15}, {&S2245[41], 55}, {NULL}};
const Segment SGI253415[] = {{&S2245[0], 96}, {NULL}};
const Segment SGI253416[] = {{&S2246[16], 7}, {NULL}};
const Segment SGI253417[] = {{&S2246[0], 57}, {NULL}};
const Segment SGI253418[] = {{&S2247[25], 15}, {&S2247[41], 55}, {NULL}};
const Segment SGI253419[] = {{&S2247[0], 96}, {NULL}};
const Segment SGI253420[] = {{&S2248[0], 18}, {&S2248[19], 50}, {NULL}};
const Segment SGI253421[] = {{&S2248[0], 69}, {NULL}};
const Segment SGI253422[] = {{&S2249[0], 60}, {NULL}};
const Segment SGI253423[] = {{&S2250[13], 45}, {NULL}};
const Segment SGI253424[] = {{&S2250[0], 58}, {NULL}};
const Segment SGI253425[] = {{&S2251[17], 7}, {NULL}};
const Segment SGI253426[] = {{&S2251[0], 28}, {NULL}};
const Segment SGI253427[] = {{&S2252[25], 15}, {&S2252[41], 40}, {NULL}};
const Segment SGI253428[] = {{&S2252[0], 81}, {NULL}};
const Segment SGI253429[] = {{&S2253[17], 7}, {NULL}};
const Segment SGI253430[] = {{&S2253[0], 130}, {NULL}};
const Segment SGI253431[] = {{&S2254[21], 3}, {&S2254[26], 32}, {&S2254[59], 7}, {NULL}};
const Segment SGI253432[] = {{&S2254[0], 79}, {NULL}};
const Segment SGI253433[] = {{&S2255[8], 7}, {NULL}};
const Segment SGI253434[] = {{&S2255[0], 15}, {NULL}};
const Segment SGI253435[] = {{&S2256[25], 10}, {&S2256[36], 40}, {NULL}};
const Segment SGI253436[] = {{&S2256[0], 76}, {NULL}};
const Segment SGI253437[] = {{&S2257[25], 10}, {&S2257[36], 7}, {&S2257[57], 40}, {NULL}};
const Segment SGI253438[] = {{&S2257[0], 97}, {NULL}};
const Segment SGI253439[] = {{&S2258[0], 12}, {&S2258[13], 43}, {&S2258[57], 40}, {NULL}};
const Segment SGI253440[] = {{&S2258[0], 97}, {NULL}};
const Segment SGI253441[] = {{&S2259[0], 12}, {&S2259[13], 43}, {&S2259[57], 40}, {NULL}};
const Segment SGI253442[] = {{&S2259[0], 97}, {NULL}};
const Segment SGI253443[] = {{&S2260[0], 12}, {&S2260[13], 43}, {&S2260[57], 40}, {NULL}};
const Segment SGI253444[] = {{&S2260[0], 97}, {NULL}};
const Segment SGI253445[] = {{&S2261[0], 12}, {&S2261[13], 43}, {&S2261[57], 40}, {NULL}};
const Segment SGI253446[] = {{&S2261[0], 97}, {NULL}};
const Segment SGI253447[] = {{&S2262[0], 12}, {&S2262[13], 43}, {&S2262[57], 40}, {NULL}};
const Segment SGI253448[] = {{&S2262[0], 97}, {NULL}};
const Segment SGI253449[] = {{&S2263[25], 17}, {NULL}};
const Segment SGI253450[] = {{&S2263[0], 42}, {NULL}};
const Segment SGI253451[] = {{&S2264[0], 11}, {NULL}};
const Segment SGI253452[] = {{&S2264[0], 61}, {NULL}};
const Segment SGI253453[] = {{&S2265[44], 37}, {NULL}};
const Segment SGI253454[] = {{&S2265[0], 81}, {NULL}};
const Segment SGI253455[] = {{&S2266[25], 23}, {NULL}};
const Segment SGI253456[] = {{&S2266[0], 76}, {NULL}};
const Segment SGI253457[] = {{&S2267[0], 18}, {NULL}};
const Segment SGI253458[] = {{&S2268[0], 12}, {&S2268[13], 43}, {&S2268[57], 40}, {NULL}};
const Segment SGI253459[] = {{&S2268[0], 97}, {NULL}};
const Segment SGI253460[] = {{&S2269[0], 12}, {&S2269[13], 43}, {&S2269[57], 40}, {NULL}};
const Segment SGI253461[] = {{&S2269[0], 97}, {NULL}};
const Segment SGI253462[] = {{&S2270[0], 12}, {&S2270[13], 43}, {&S2270[57], 40}, {NULL}};
const Segment SGI253463[] = {{&S2270[0], 97}, {NULL}};
const Segment SGI253464[] = {{&S2271[0], 12}, {&S2271[13], 43}, {&S2271[57], 40}, {NULL}};
const Segment SGI253465[] = {{&S2271[0], 97}, {NULL}};
const Segment SGI253466[] = {{&S2272[0], 7}, {NULL}};
const Segment SGI253467[] = {{&S2272[0], 54}, {NULL}};
const Segment SGI253468[] = {{&S102118[44], 10}, {NULL}};
const Segment SGI253469[] = {{&S102118[44], 21}, {NULL}};
const Segment SGI253470[] = {{&S102118[30], 6}, {NULL}};
const Segment SGI253471[] = {{&S102118[132], 3}, {NULL}};
const Segment SGI253472[] = {{&S102118[97], 6}, {NULL}};
const Segment SGI253473[] = {{&S102118[160], 3}, {NULL}};
const Segment SGI253474[] = {{&S102118[67], 8}, {NULL}};
const Segment SGI253475[] = {{&S102121[44], 10}, {NULL}};
const Segment SGI253476[] = {{&S102121[44], 21}, {NULL}};
const Segment SGI253477[] = {{&S102121[30], 6}, {NULL}};
const Segment SGI253478[] = {{&S102121[132], 3}, {NULL}};
const Segment SGI253479[] = {{&S102121[97], 6}, {NULL}};
const Segment SGI253480[] = {{&S102121[160], 3}, {NULL}};
const Segment SGI253481[] = {{&S102121[67], 8}, {NULL}};
const Segment SGI253482[] = {{&S102523[44], 10}, {NULL}};
const Segment SGI253483[] = {{&S102523[44], 21}, {NULL}};
const Segment SGI253484[] = {{&S102523[30], 6}, {NULL}};
const Segment SGI253485[] = {{&S102523[132], 3}, {NULL}};
const Segment SGI253486[] = {{&S102523[97], 6}, {NULL}};
const Segment SGI253487[] = {{&S102523[160], 3}, {NULL}};
const Segment SGI253488[] = {{&S102523[67], 8}, {NULL}};
const Segment SGI253489[] = {{&S102539[34], 10}, {NULL}};
const Segment SGI253490[] = {{&S102539[27], 6}, {NULL}};
const Segment SGI253491[] = {{&S102539[96], 9}, {NULL}};
const Segment SGI253492[] = {{&S102539[69], 6}, {NULL}};
const Segment SGI253493[] = {{&S102927[44], 10}, {NULL}};
const Segment SGI253494[] = {{&S102927[44], 21}, {NULL}};
const Segment SGI253495[] = {{&S102927[30], 6}, {NULL}};
const Segment SGI253496[] = {{&S102927[132], 3}, {NULL}};
const Segment SGI253497[] = {{&S102927[97], 6}, {NULL}};
const Segment SGI253498[] = {{&S102927[160], 3}, {NULL}};
const Segment SGI253499[] = {{&S102927[67], 8}, {NULL}};
const Segment SGI253500[] = {{&S103179[44], 10}, {NULL}};
const Segment SGI253501[] = {{&S103179[44], 21}, {NULL}};
const Segment SGI253502[] = {{&S103179[30], 6}, {NULL}};
const Segment SGI253503[] = {{&S103179[132], 3}, {NULL}};
const Segment SGI253504[] = {{&S103179[97], 6}, {NULL}};
const Segment SGI253505[] = {{&S103179[67], 8}, {NULL}};
const Segment SGI253506[] = {{&S14512[13], 4}, {NULL}};
const Segment SGI253507[] = {{&S14512[26], 5}, {NULL}};
const Segment SGI253508[] = {{&S14512[66], 6}, {NULL}};
const Segment SGI253509[] = {{&S14512[120], 6}, {NULL}};
const Segment SGI253510[] = {{&S14512[134], 7}, {NULL}};
const Segment SGI253511[] = {{&S14513[13], 4}, {NULL}};
const Segment SGI253512[] = {{&S14513[26], 5}, {NULL}};
const Segment SGI253513[] = {{&S14513[66], 6}, {NULL}};
const Segment SGI253514[] = {{&S14513[120], 7}, {NULL}};
const Segment SGI253515[] = {{&S14513[135], 7}, {NULL}};
const Segment SGI253516[] = {{&S14514[13], 4}, {NULL}};
const Segment SGI253517[] = {{&S14514[26], 3}, {NULL}};
const Segment SGI253518[] = {{&S14514[64], 6}, {NULL}};
const Segment SGI253519[] = {{&S14514[118], 6}, {NULL}};
const Segment SGI253520[] = {{&S14514[132], 7}, {NULL}};
const Segment SGI253521[] = {{&S14515[13], 4}, {NULL}};
const Segment SGI253522[] = {{&S14515[26], 5}, {NULL}};
const Segment SGI253523[] = {{&S14515[66], 6}, {NULL}};
const Segment SGI253524[] = {{&S14515[120], 6}, {NULL}};
const Segment SGI253525[] = {{&S14515[134], 7}, {NULL}};
const Segment SGI253526[] = {{&S14516[13], 4}, {NULL}};
const Segment SGI253527[] = {{&S14516[26], 5}, {NULL}};
const Segment SGI253528[] = {{&S14516[66], 6}, {NULL}};
const Segment SGI253529[] = {{&S14516[120], 7}, {NULL}};
const Segment SGI253530[] = {{&S14516[135], 7}, {NULL}};
const Segment SGI253531[] = {{&S14517[13], 4}, {NULL}};
const Segment SGI253532[] = {{&S14517[26], 3}, {NULL}};
const Segment SGI253533[] = {{&S14517[64], 6}, {NULL}};
const Segment SGI253534[] = {{&S14517[118], 6}, {NULL}};
const Segment SGI253535[] = {{&S14517[132], 7}, {NULL}};
const Segment SGI253536[] = {{&S14518[13], 4}, {NULL}};
const Segment SGI253537[] = {{&S14518[26], 3}, {NULL}};
const Segment SGI253538[] = {{&S14518[64], 6}, {NULL}};
const Segment SGI253539[] = {{&S14518[118], 6}, {NULL}};
const Segment SGI253540[] = {{&S14518[132], 7}, {NULL}};
const Segment SGI253541[] = {{&S103425[44], 10}, {NULL}};
const Segment SGI253542[] = {{&S103425[44], 21}, {NULL}};
const Segment SGI253543[] = {{&S103425[30], 6}, {NULL}};
const Segment SGI253544[] = {{&S103425[114], 3}, {NULL}};
const Segment SGI253545[] = {{&S103425[79], 6}, {NULL}};
const Segment SGI253546[] = {{&S103425[149], 3}, {NULL}};
const Segment SGI253547[] = {{&S103462[34], 10}, {NULL}};
const Segment SGI253548[] = {{&S103462[27], 6}, {NULL}};
const Segment SGI253549[] = {{&S103462[96], 9}, {NULL}};
const Segment SGI253550[] = {{&S103462[69], 6}, {NULL}};
const Segment SGI253551[] = {{&S103462[131], 8}, {NULL}};
const Segment SGI253552[] = {{&S103476[44], 10}, {NULL}};
const Segment SGI253553[] = {{&S103476[44], 21}, {NULL}};
const Segment SGI253554[] = {{&S103476[30], 6}, {NULL}};
const Segment SGI253555[] = {{&S103476[114], 3}, {NULL}};
const Segment SGI253556[] = {{&S103476[79], 6}, {NULL}};
const Segment SGI253557[] = {{&S103476[149], 3}, {NULL}};
const Segment SGI253558[] = {{&S103481[34], 10}, {NULL}};
const Segment SGI253559[] = {{&S103481[27], 6}, {NULL}};
const Segment SGI253560[] = {{&S103481[96], 9}, {NULL}};
const Segment SGI253561[] = {{&S103481[69], 6}, {NULL}};
const Segment SGI253562[] = {{&S103482[54], 10}, {NULL}};
const Segment SGI253563[] = {{&S103482[47], 6}, {NULL}};
const Segment SGI253564[] = {{&S103483[44], 10}, {NULL}};
const Segment SGI253565[] = {{&S103483[44], 21}, {NULL}};
const Segment SGI253566[] = {{&S103483[30], 6}, {NULL}};
const Segment SGI253567[] = {{&S103483[114], 3}, {NULL}};
const Segment SGI253568[] = {{&S103483[79], 6}, {NULL}};
const Segment SGI253569[] = {{&S103782[34], 10}, {NULL}};
const Segment SGI253570[] = {{&S103782[27], 6}, {NULL}};
const Segment SGI253571[] = {{&S103782[96], 9}, {NULL}};
const Segment SGI253572[] = {{&S103782[69], 6}, {NULL}};
const Segment SGI253573[] = {{&S103784[44], 10}, {NULL}};
const Segment SGI253574[] = {{&S103784[44], 21}, {NULL}};
const Segment SGI253575[] = {{&S103784[30], 6}, {NULL}};
const Segment SGI253576[] = {{&S103784[132], 3}, {NULL}};
const Segment SGI253577[] = {{&S103784[97], 6}, {NULL}};
const Segment SGI253578[] = {{&S103784[160], 3}, {NULL}};
const Segment SGI253579[] = {{&S103784[67], 8}, {NULL}};
const Segment SGI253580[] = {{&S103787[34], 10}, {NULL}};
const Segment SGI253581[] = {{&S103787[27], 6}, {NULL}};
const Segment SGI253582[] = {{&S103787[96], 9}, {NULL}};
const Segment SGI253583[] = {{&S103787[69], 6}, {NULL}};
const Segment SGI253584[] = {{&S103791[34], 10}, {NULL}};
const Segment SGI253585[] = {{&S103791[27], 6}, {NULL}};
const Segment SGI253586[] = {{&S103791[96], 9}, {NULL}};
const Segment SGI253587[] = {{&S103791[69], 6}, {NULL}};
const Segment SGI253588[] = {{&S103815[34], 10}, {NULL}};
const Segment SGI253589[] = {{&S103815[27], 6}, {NULL}};
const Segment SGI253590[] = {{&S103815[96], 9}, {NULL}};
const Segment SGI253591[] = {{&S103815[69], 6}, {NULL}};
const Segment SGI253592[] = {{&S104131[34], 17}, {NULL}};
const Segment SGI253593[] = {{&S104131[27], 6}, {NULL}};
const Segment SGI253594[] = {{&S104131[103], 9}, {NULL}};
const Segment SGI253595[] = {{&S104131[76], 6}, {NULL}};
const Segment SGI253596[] = {{&S104131[144], 8}, {NULL}};
const Segment SGI253597[] = {{&S104132[34], 10}, {NULL}};
const Segment SGI253598[] = {{&S104132[27], 6}, {NULL}};
const Segment SGI253599[] = {{&S104132[96], 9}, {NULL}};
const Segment SGI253600[] = {{&S104132[69], 6}, {NULL}};
const Segment SGI253601[] = {{&S104132[137], 8}, {NULL}};
const Segment SGI253602[] = {{&S104133[44], 10}, {NULL}};
const Segment SGI253603[] = {{&S104133[44], 21}, {NULL}};
const Segment SGI253604[] = {{&S104133[30], 6}, {NULL}};
const Segment SGI253605[] = {{&S104133[114], 3}, {NULL}};
const Segment SGI253606[] = {{&S104133[79], 6}, {NULL}};
const Segment SGI253607[] = {{&S104147[34], 10}, {NULL}};
const Segment SGI253608[] = {{&S104147[27], 6}, {NULL}};
const Segment SGI253609[] = {{&S104147[96], 9}, {NULL}};
const Segment SGI253610[] = {{&S104147[69], 6}, {NULL}};
const Segment SGI253611[] = {{&S104147[137], 8}, {NULL}};
const Segment SGI253612[] = {{&S15954[13], 6}, {NULL}};
const Segment SGI253613[] = {{&S15954[35], 5}, {NULL}};
const Segment SGI253614[] = {{&S15954[75], 6}, {NULL}};
const Segment SGI253615[] = {{&S15954[129], 6}, {NULL}};
const Segment SGI253616[] = {{&S15954[143], 7}, {NULL}};
const Segment SGI253617[] = {{&S15955[13], 6}, {NULL}};
const Segment SGI253618[] = {{&S15955[35], 5}, {NULL}};
const Segment SGI253619[] = {{&S15955[75], 6}, {NULL}};
const Segment SGI253620[] = {{&S15955[129], 6}, {NULL}};
const Segment SGI253621[] = {{&S15955[143], 7}, {NULL}};
const Segment SGI253622[] = {{&S15956[13], 6}, {NULL}};
const Segment SGI253623[] = {{&S15956[35], 5}, {NULL}};
const Segment SGI253624[] = {{&S15956[75], 6}, {NULL}};
const Segment SGI253625[] = {{&S15956[129], 6}, {NULL}};
const Segment SGI253626[] = {{&S15956[143], 7}, {NULL}};
const Segment SGI253627[] = {{&S15957[13], 6}, {NULL}};
const Segment SGI253628[] = {{&S15957[35], 3}, {NULL}};
const Segment SGI253629[] = {{&S15957[73], 6}, {NULL}};
const Segment SGI253630[] = {{&S15957[127], 6}, {NULL}};
const Segment SGI253631[] = {{&S15957[141], 7}, {NULL}};
const Segment SGI253632[] = {{&S15958[13], 6}, {NULL}};
const Segment SGI253633[] = {{&S15958[35], 3}, {NULL}};
const Segment SGI253634[] = {{&S15958[73], 6}, {NULL}};
const Segment SGI253635[] = {{&S15958[127], 6}, {NULL}};
const Segment SGI253636[] = {{&S15958[141], 7}, {NULL}};
const Segment SGI253637[] = {{&S104502[44], 10}, {NULL}};
const Segment SGI253638[] = {{&S104502[44], 21}, {NULL}};
const Segment SGI253639[] = {{&S104502[30], 6}, {NULL}};
const Segment SGI253640[] = {{&S104502[132], 3}, {NULL}};
const Segment SGI253641[] = {{&S104502[97], 6}, {NULL}};
const Segment SGI253642[] = {{&S104502[67], 8}, {NULL}};
const Segment SGI253643[] = {{&S104502[149], 10}, {NULL}};
const Segment SGI253644[] = {{&S104510[44], 10}, {NULL}};
const Segment SGI253645[] = {{&S104510[44], 21}, {NULL}};
const Segment SGI253646[] = {{&S104510[30], 6}, {NULL}};
const Segment SGI253647[] = {{&S104510[132], 3}, {NULL}};
const Segment SGI253648[] = {{&S104510[97], 6}, {NULL}};
const Segment SGI253649[] = {{&S104510[160], 3}, {NULL}};
const Segment SGI253650[] = {{&S104510[67], 8}, {NULL}};
const Segment SGI253651[] = {{&S104511[34], 10}, {NULL}};
const Segment SGI253652[] = {{&S104511[27], 6}, {NULL}};
const Segment SGI253653[] = {{&S104511[96], 9}, {NULL}};
const Segment SGI253654[] = {{&S104511[69], 6}, {NULL}};
const Segment SGI253655[] = {{&S104528[44], 10}, {NULL}};
const Segment SGI253656[] = {{&S104528[44], 21}, {NULL}};
const Segment SGI253657[] = {{&S104528[30], 6}, {NULL}};
const Segment SGI253658[] = {{&S104528[114], 3}, {NULL}};
const Segment SGI253659[] = {{&S104528[79], 6}, {NULL}};
const Segment SGI253660[] = {{&S104528[149], 3}, {NULL}};
const Segment SGI253661[] = {{&S104556[34], 9}, {NULL}};
const Segment SGI253662[] = {{&S104556[27], 6}, {NULL}};
const Segment SGI253663[] = {{&S104556[95], 9}, {NULL}};
const Segment SGI253664[] = {{&S104556[68], 6}, {NULL}};
const Segment SGI253665[] = {{&S104556[136], 8}, {NULL}};
const Segment SGI253666[] = {{&S104959[44], 10}, {NULL}};
const Segment SGI253667[] = {{&S104959[44], 21}, {NULL}};
const Segment SGI253668[] = {{&S104959[30], 6}, {NULL}};
const Segment SGI253669[] = {{&S104959[132], 3}, {NULL}};
const Segment SGI253670[] = {{&S104959[97], 6}, {NULL}};
const Segment SGI253671[] = {{&S104959[67], 8}, {NULL}};
const Segment SGI253672[] = {{&S104959[150], 11}, {NULL}};
const Segment SGI253673[] = {{&S104992[44], 10}, {NULL}};
const Segment SGI253674[] = {{&S104992[44], 21}, {NULL}};
const Segment SGI253675[] = {{&S104992[30], 6}, {NULL}};
const Segment SGI253676[] = {{&S104992[132], 3}, {NULL}};
const Segment SGI253677[] = {{&S104992[97], 6}, {NULL}};
const Segment SGI253678[] = {{&S104992[67], 8}, {NULL}};
const Segment SGI253679[] = {{&S104992[150], 11}, {NULL}};
const Segment SGI253680[] = {{&S105002[34], 10}, {NULL}};
const Segment SGI253681[] = {{&S105002[27], 6}, {NULL}};
const Segment SGI253682[] = {{&S105002[96], 9}, {NULL}};
const Segment SGI253683[] = {{&S105002[69], 6}, {NULL}};
const Segment SGI253684[] = {{&S105002[131], 8}, {NULL}};
const Segment SGI253685[] = {{&S105023[44], 10}, {NULL}};
const Segment SGI253686[] = {{&S105023[44], 21}, {NULL}};
const Segment SGI253687[] = {{&S105023[30], 6}, {NULL}};
const Segment SGI253688[] = {{&S105023[151], 3}, {NULL}};
const Segment SGI253689[] = {{&S105023[116], 6}, {NULL}};
const Segment SGI253690[] = {{&S105023[67], 8}, {NULL}};
const Segment SGI253691[] = {{&S105031[34], 7}, {NULL}};
const Segment SGI253692[] = {{&S105031[27], 6}, {NULL}};
const Segment SGI253693[] = {{&S105031[100], 9}, {NULL}};
const Segment SGI253694[] = {{&S105031[73], 6}, {NULL}};
const Segment SGI253695[] = {{&S105069[34], 10}, {NULL}};
const Segment SGI253696[] = {{&S105069[27], 6}, {NULL}};
const Segment SGI253697[] = {{&S105069[96], 9}, {NULL}};
const Segment SGI253698[] = {{&S105069[69], 6}, {NULL}};
const Segment SGI253699[] = {{&S105804[44], 10}, {NULL}};
const Segment SGI253700[] = {{&S105804[44], 21}, {NULL}};
const Segment SGI253701[] = {{&S105804[30], 6}, {NULL}};
const Segment SGI253702[] = {{&S105804[132], 3}, {NULL}};
const Segment SGI253703[] = {{&S105804[97], 6}, {NULL}};
const Segment SGI253704[] = {{&S105804[67], 8}, {NULL}};
const Segment SGI253705[] = {{&S105804[150], 13}, {NULL}};
const Segment SGI253706[] = {{&S105807[34], 10}, {NULL}};
const Segment SGI253707[] = {{&S105807[27], 6}, {NULL}};
const Segment SGI253708[] = {{&S105807[96], 9}, {NULL}};
const Segment SGI253709[] = {{&S105807[69], 6}, {NULL}};
const Segment SGI253710[] = {{&S105807[131], 8}, {NULL}};
const Segment SGI253711[] = {{&S105811[44], 10}, {NULL}};
const Segment SGI253712[] = {{&S105811[44], 21}, {NULL}};
const Segment SGI253713[] = {{&S105811[30], 6}, {NULL}};
const Segment SGI253714[] = {{&S105811[132], 3}, {NULL}};
const Segment SGI253715[] = {{&S105811[97], 6}, {NULL}};
const Segment SGI253716[] = {{&S105811[67], 8}, {NULL}};
const Segment SGI253717[] = {{&S105811[150], 11}, {NULL}};
const Segment SGI253718[] = {{&S105828[34], 10}, {NULL}};
const Segment SGI253719[] = {{&S105828[27], 6}, {NULL}};
const Segment SGI253720[] = {{&S105828[96], 9}, {NULL}};
const Segment SGI253721[] = {{&S105828[69], 6}, {NULL}};
const Segment SGI253722[] = {{&S105828[131], 8}, {NULL}};
const Segment SGI253723[] = {{&S105840[45], 10}, {NULL}};
const Segment SGI253724[] = {{&S105840[30], 6}, {NULL}};
const Segment SGI253725[] = {{&S105840[114], 3}, {NULL}};
const Segment SGI253726[] = {{&S105840[80], 5}, {NULL}};
const Segment SGI253727[] = {{&S105840[138], 13}, {NULL}};
const Segment SGI253728[] = {{&S17332[13], 4}, {NULL}};
const Segment SGI253729[] = {{&S17332[26], 3}, {NULL}};
const Segment SGI253730[] = {{&S17332[57], 6}, {NULL}};
const Segment SGI253731[] = {{&S17332[91], 8}, {NULL}};
const Segment SGI253732[] = {{&S105849[44], 10}, {NULL}};
const Segment SGI253733[] = {{&S105849[44], 21}, {NULL}};
const Segment SGI253734[] = {{&S17333[13], 4}, {&S17333[13], 4}, {&S17333[13], 4}, {NULL}};
const Segment SGI253735[] = {{&S105849[30], 6}, {NULL}};
const Segment SGI253736[] = {{&S17333[26], 3}, {NULL}};
const Segment SGI253737[] = {{&S17333[57], 6}, {NULL}};
const Segment SGI253738[] = {{&S17333[91], 8}, {NULL}};
const Segment SGI253739[] = {{&S105849[151], 3}, {NULL}};
const Segment SGI253740[] = {{&S105849[116], 6}, {NULL}};
const Segment SGI253741[] = {{&S105849[179], 3}, {NULL}};
const Segment SGI253742[] = {{&S105849[67], 8}, {NULL}};
const Segment SGI253743[] = {{&S17334[13], 4}, {NULL}};
const Segment SGI253744[] = {{&S17334[26], 3}, {NULL}};
const Segment SGI253745[] = {{&S17334[57], 6}, {NULL}};
const Segment SGI253746[] = {{&S17334[91], 8}, {NULL}};
const Segment SGI253747[] = {{&S17335[13], 4}, {&S17335[13], 4}, {&S17335[13], 4}, {NULL}};
const Segment SGI253748[] = {{&S17335[26], 3}, {NULL}};
const Segment SGI253749[] = {{&S17335[57], 6}, {NULL}};
const Segment SGI253750[] = {{&S17335[91], 8}, {NULL}};
const Segment SGI253751[] = {{&S17336[13], 4}, {NULL}};
const Segment SGI253752[] = {{&S17336[26], 3}, {NULL}};
const Segment SGI253753[] = {{&S17336[57], 6}, {NULL}};
const Segment SGI253754[] = {{&S17337[13], 4}, {NULL}};
const Segment SGI253755[] = {{&S17337[26], 3}, {NULL}};
const Segment SGI253756[] = {{&S17337[57], 8}, {NULL}};
const Segment SGI253757[] = {{&S17337[93], 8}, {NULL}};
const Segment SGI253758[] = {{&S17338[13], 4}, {NULL}};
const Segment SGI253759[] = {{&S17338[26], 3}, {NULL}};
const Segment SGI253760[] = {{&S17338[57], 8}, {NULL}};
const Segment SGI253761[] = {{&S17339[13], 4}, {&S17339[13], 4}, {&S17339[13], 4}, {NULL}};
const Segment SGI253762[] = {{&S17339[26], 3}, {NULL}};
const Segment SGI253763[] = {{&S17339[57], 8}, {NULL}};
const Segment SGI253764[] = {{&S17339[93], 8}, {NULL}};
const Segment SGI253765[] = {{&S17340[13], 4}, {NULL}};
const Segment SGI253766[] = {{&S17340[26], 3}, {NULL}};
const Segment SGI253767[] = {{&S17340[57], 8}, {NULL}};
const Segment SGI253768[] = {{&S17340[93], 8}, {NULL}};
const Segment SGI253769[] = {{&S17341[13], 4}, {NULL}};
const Segment SGI253770[] = {{&S17341[26], 3}, {NULL}};
const Segment SGI253771[] = {{&S17341[57], 8}, {NULL}};
const Segment SGI253772[] = {{&S17341[93], 8}, {NULL}};
const Segment SGI253773[] = {{&S17342[13], 4}, {&S17342[13], 4}, {&S17342[13], 4}, {NULL}};
const Segment SGI253774[] = {{&S17342[26], 3}, {NULL}};
const Segment SGI253775[] = {{&S17342[57], 8}, {NULL}};
const Segment SGI253776[] = {{&S17342[93], 8}, {NULL}};
const Segment SGI253777[] = {{&S105865[44], 10}, {NULL}};
const Segment SGI253778[] = {{&S105865[44], 21}, {NULL}};
const Segment SGI253779[] = {{&S105865[30], 6}, {NULL}};
const Segment SGI253780[] = {{&S17343[13], 4}, {&S17343[13], 4}, {&S17343[13], 4}, {NULL}};
const Segment SGI253781[] = {{&S105865[112], 3}, {NULL}};
const Segment SGI253782[] = {{&S17343[26], 3}, {NULL}};
const Segment SGI253783[] = {{&S105865[79], 5}, {NULL}};
const Segment SGI253784[] = {{&S17343[57], 8}, {NULL}};
const Segment SGI253785[] = {{&S17343[93], 8}, {NULL}};
const Segment SGI253786[] = {{&S105866[44], 10}, {NULL}};
const Segment SGI253787[] = {{&S17344[13], 4}, {NULL}};
const Segment SGI253788[] = {{&S105866[44], 21}, {NULL}};
const Segment SGI253789[] = {{&S17344[26], 3}, {NULL}};
const Segment SGI253790[] = {{&S105866[30], 6}, {NULL}};
const Segment SGI253791[] = {{&S17344[57], 8}, {NULL}};
const Segment SGI253792[] = {{&S17344[93], 8}, {NULL}};
const Segment SGI253793[] = {{&S105866[112], 3}, {NULL}};
const Segment SGI253794[] = {{&S105866[79], 5}, {NULL}};
const Segment SGI253795[] = {{&S17345[13], 4}, {NULL}};
const Segment SGI253796[] = {{&S17345[26], 3}, {NULL}};
const Segment SGI253797[] = {{&S17345[57], 8}, {NULL}};
const Segment SGI253798[] = {{&S17345[93], 6}, {NULL}};
const Segment SGI253799[] = {{&S17346[13], 4}, {&S17346[13], 4}, {&S17346[13], 4}, {NULL}};
const Segment SGI253800[] = {{&S17346[26], 3}, {NULL}};
const Segment SGI253801[] = {{&S17346[57], 8}, {NULL}};
const Segment SGI253802[] = {{&S17346[93], 6}, {NULL}};
const Segment SGI253803[] = {{&S105869[59], 10}, {NULL}};
const Segment SGI253804[] = {{&S105869[59], 21}, {NULL}};
const Segment SGI253805[] = {{&S105869[45], 6}, {NULL}};
const Segment SGI253806[] = {{&S105869[147], 3}, {NULL}};
const Segment SGI253807[] = {{&S17347[13], 4}, {NULL}};
const Segment SGI253808[] = {{&S105869[112], 6}, {NULL}};
const Segment SGI253809[] = {{&S17347[26], 3}, {NULL}};
const Segment SGI253810[] = {{&S105869[82], 8}, {NULL}};
const Segment SGI253811[] = {{&S17347[57], 8}, {NULL}};
const Segment SGI253812[] = {{&S17347[93], 6}, {NULL}};
const Segment SGI253813[] = {{&S17347[107], 7}, {NULL}};
const Segment SGI253814[] = {{&S17348[13], 4}, {NULL}};
const Segment SGI253815[] = {{&S17348[26], 5}, {NULL}};
const Segment SGI253816[] = {{&S17348[59], 8}, {NULL}};
const Segment SGI253817[] = {{&S17348[95], 6}, {NULL}};
const Segment SGI253818[] = {{&S17349[13], 4}, {&S17349[13], 4}, {NULL}};
const Segment SGI253819[] = {{&S17349[26], 3}, {NULL}};
const Segment SGI253820[] = {{&S17349[57], 8}, {NULL}};
const Segment SGI253821[] = {{&S17349[93], 6}, {NULL}};
const Segment SGI253822[] = {{&S17350[13], 4}, {NULL}};
const Segment SGI253823[] = {{&S17350[26], 3}, {NULL}};
const Segment SGI253824[] = {{&S17350[57], 8}, {NULL}};
const Segment SGI253825[] = {{&S17350[93], 6}, {NULL}};
const Segment SGI253826[] = {{&S17350[107], 9}, {NULL}};
const Segment SGI253827[] = {{&S17351[13], 4}, {&S17351[13], 4}, {NULL}};
const Segment SGI253828[] = {{&S17351[38], 3}, {NULL}};
const Segment SGI253829[] = {{&S17351[119], 4}, {NULL}};
const Segment SGI253830[] = {{&S17352[13], 4}, {&S17352[13], 4}, {&S17352[13], 4}, {NULL}};
const Segment SGI253831[] = {{&S17352[36], 3}, {NULL}};
const Segment SGI253832[] = {{&S105878[38], 10}, {NULL}};
const Segment SGI253833[] = {{&S105878[31], 6}, {NULL}};
const Segment SGI253834[] = {{&S17353[13], 4}, {&S17353[13], 4}, {NULL}};
const Segment SGI253835[] = {{&S17353[38], 3}, {NULL}};
const Segment SGI253836[] = {{&S17353[119], 5}, {NULL}};
const Segment SGI253837[] = {{&S17354[13], 4}, {NULL}};
const Segment SGI253838[] = {{&S17354[26], 5}, {NULL}};
const Segment SGI253839[] = {{&S17354[59], 8}, {NULL}};
const Segment SGI253840[] = {{&S17355[13], 4}, {&S17355[13], 4}, {&S17355[13], 4}, {NULL}};
const Segment SGI253841[] = {{&S17355[26], 5}, {NULL}};
const Segment SGI253842[] = {{&S17355[59], 8}, {NULL}};
const Segment SGI253843[] = {{&S17355[95], 6}, {NULL}};
const Segment SGI253844[] = {{&S17356[13], 4}, {NULL}};
const Segment SGI253845[] = {{&S17356[26], 3}, {NULL}};
const Segment SGI253846[] = {{&S17356[71], 6}, {NULL}};
const Segment SGI253847[] = {{&S17356[105], 5}, {NULL}};
const Segment SGI253848[] = {{&S17357[13], 4}, {&S17357[13], 4}, {NULL}};
const Segment SGI253849[] = {{&S17357[38], 3}, {NULL}};
const Segment SGI253850[] = {{&S17357[119], 6}, {NULL}};
const Segment SGI253851[] = {{&S105886[59], 10}, {NULL}};
const Segment SGI253852[] = {{&S105886[59], 21}, {NULL}};
const Segment SGI253853[] = {{&S105886[45], 6}, {NULL}};
const Segment SGI253854[] = {{&S105886[147], 3}, {NULL}};
const Segment SGI253855[] = {{&S17358[13], 4}, {NULL}};
const Segment SGI253856[] = {{&S105886[112], 6}, {NULL}};
const Segment SGI253857[] = {{&S17358[29], 3}, {NULL}};
const Segment SGI253858[] = {{&S105886[82], 8}, {NULL}};
const Segment SGI253859[] = {{&S17359[13], 4}, {&S17359[13], 4}, {NULL}};
const Segment SGI253860[] = {{&S17359[26], 5}, {NULL}};
const Segment SGI253861[] = {{&S17359[59], 8}, {NULL}};
const Segment SGI253862[] = {{&S17359[95], 6}, {NULL}};
const Segment SGI253863[] = {{&S17360[13], 4}, {NULL}};
const Segment SGI253864[] = {{&S17360[26], 5}, {NULL}};
const Segment SGI253865[] = {{&S17360[59], 8}, {NULL}};
const Segment SGI253866[] = {{&S17360[95], 6}, {NULL}};
const Segment SGI253867[] = {{&S17360[109], 9}, {NULL}};
const Segment SGI253868[] = {{&S17361[13], 4}, {NULL}};
const Segment SGI253869[] = {{&S17361[26], 5}, {NULL}};
const Segment SGI253870[] = {{&S17361[73], 6}, {NULL}};
const Segment SGI253871[] = {{&S17361[107], 5}, {NULL}};
const Segment SGI253872[] = {{&S105907[34], 7}, {NULL}};
const Segment SGI253873[] = {{&S105907[27], 6}, {NULL}};
const Segment SGI253874[] = {{&S105907[100], 9}, {NULL}};
const Segment SGI253875[] = {{&S105907[73], 6}, {NULL}};
const Segment SGI253876[] = {{&S105908[34], 7}, {NULL}};
const Segment SGI253877[] = {{&S105908[27], 6}, {NULL}};
const Segment SGI253878[] = {{&S105908[100], 9}, {NULL}};
const Segment SGI253879[] = {{&S105908[73], 6}, {NULL}};
const Segment SGI253880[] = {{&S105918[34], 7}, {NULL}};
const Segment SGI253881[] = {{&S105918[27], 6}, {NULL}};
const Segment SGI253882[] = {{&S105918[100], 9}, {NULL}};
const Segment SGI253883[] = {{&S105918[73], 6}, {NULL}};
const Segment SGI253884[] = {{&S105922[44], 17}, {NULL}};
const Segment SGI253885[] = {{&S105922[44], 28}, {NULL}};
const Segment SGI253886[] = {{&S105922[30], 6}, {NULL}};
const Segment SGI253887[] = {{&S105922[121], 3}, {NULL}};
const Segment SGI253888[] = {{&S105922[86], 6}, {NULL}};
const Segment SGI253889[] = {{&S105922[145], 10}, {NULL}};
const Segment SGI253890[] = {{&S105923[34], 17}, {NULL}};
const Segment SGI253891[] = {{&S105923[27], 6}, {NULL}};
const Segment SGI253892[] = {{&S105923[103], 9}, {NULL}};
const Segment SGI253893[] = {{&S105923[76], 6}, {NULL}};
const Segment SGI253894[] = {{&S105923[144], 8}, {NULL}};
const Segment SGI253895[] = {{&S105924[44], 17}, {NULL}};
const Segment SGI253896[] = {{&S105924[44], 28}, {NULL}};
const Segment SGI253897[] = {{&S105924[30], 6}, {NULL}};
const Segment SGI253898[] = {{&S105924[121], 3}, {NULL}};
const Segment SGI253899[] = {{&S105924[86], 6}, {NULL}};
const Segment SGI253900[] = {{&S105924[146], 7}, {NULL}};
const Segment SGI253901[] = {{&S105925[34], 10}, {NULL}};
const Segment SGI253902[] = {{&S105925[27], 6}, {NULL}};
const Segment SGI253903[] = {{&S105925[96], 9}, {NULL}};
const Segment SGI253904[] = {{&S105925[69], 6}, {NULL}};
const Segment SGI253905[] = {{&S105926[44], 10}, {NULL}};
const Segment SGI253906[] = {{&S105926[44], 21}, {NULL}};
const Segment SGI253907[] = {{&S105926[30], 6}, {NULL}};
const Segment SGI253908[] = {{&S105926[114], 3}, {NULL}};
const Segment SGI253909[] = {{&S105926[79], 6}, {NULL}};
const Segment SGI253910[] = {{&S105926[149], 3}, {NULL}};
const Segment SGI253911[] = {{&S105930[34], 18}, {NULL}};
const Segment SGI253912[] = {{&S105930[27], 6}, {NULL}};
const Segment SGI253913[] = {{&S105930[104], 9}, {NULL}};
const Segment SGI253914[] = {{&S105930[77], 6}, {NULL}};
const Segment SGI253915[] = {{&S17429[13], 6}, {NULL}};
const Segment SGI253916[] = {{&S17429[101], 3}, {NULL}};
const Segment SGI253917[] = {{&S17429[35], 3}, {NULL}};
const Segment SGI253918[] = {{&S17429[66], 6}, {NULL}};
const Segment SGI253919[] = {{&S17429[112], 7}, {NULL}};
const Segment SGI253920[] = {{&S17429[127], 9}, {NULL}};
const Segment SGI253921[] = {{&S17430[13], 6}, {NULL}};
const Segment SGI253922[] = {{&S17430[101], 3}, {NULL}};
const Segment SGI253923[] = {{&S17430[35], 3}, {NULL}};
const Segment SGI253924[] = {{&S17430[66], 6}, {NULL}};
const Segment SGI253925[] = {{&S17430[112], 5}, {NULL}};
const Segment SGI253926[] = {{&S17430[125], 9}, {NULL}};
const Segment SGI253927[] = {{&S17431[13], 6}, {NULL}};
const Segment SGI253928[] = {{&S17431[101], 3}, {NULL}};
const Segment SGI253929[] = {{&S17431[35], 3}, {NULL}};
const Segment SGI253930[] = {{&S17431[66], 6}, {NULL}};
const Segment SGI253931[] = {{&S17431[112], 5}, {NULL}};
const Segment SGI253932[] = {{&S17431[125], 9}, {NULL}};
const Segment SGI253933[] = {{&S17432[13], 6}, {NULL}};
const Segment SGI253934[] = {{&S17432[101], 3}, {NULL}};
const Segment SGI253935[] = {{&S17432[35], 3}, {NULL}};
const Segment SGI253936[] = {{&S17432[66], 6}, {NULL}};
const Segment SGI253937[] = {{&S17432[112], 5}, {NULL}};
const Segment SGI253938[] = {{&S17432[125], 9}, {NULL}};
const Segment SGI253939[] = {{&S17433[13], 6}, {NULL}};
const Segment SGI253940[] = {{&S17433[101], 3}, {NULL}};
const Segment SGI253941[] = {{&S17433[35], 3}, {NULL}};
const Segment SGI253942[] = {{&S17433[66], 6}, {NULL}};
const Segment SGI253943[] = {{&S17433[112], 5}, {NULL}};
const Segment SGI253944[] = {{&S17433[125], 9}, {NULL}};
const Segment SGI253945[] = {{&S17434[13], 6}, {NULL}};
const Segment SGI253946[] = {{&S17434[101], 3}, {NULL}};
const Segment SGI253947[] = {{&S17434[35], 3}, {NULL}};
const Segment SGI253948[] = {{&S17434[66], 6}, {NULL}};
const Segment SGI253949[] = {{&S17434[112], 5}, {NULL}};
const Segment SGI253950[] = {{&S17434[125], 9}, {NULL}};
const Segment SGI253951[] = {{&S17435[13], 6}, {NULL}};
const Segment SGI253952[] = {{&S17435[101], 3}, {NULL}};
const Segment SGI253953[] = {{&S17435[35], 3}, {NULL}};
const Segment SGI253954[] = {{&S17435[66], 6}, {NULL}};
const Segment SGI253955[] = {{&S17435[112], 5}, {NULL}};
const Segment SGI253956[] = {{&S17435[125], 9}, {NULL}};
const Segment SGI253957[] = {{&S17436[13], 6}, {NULL}};
const Segment SGI253958[] = {{&S17436[101], 3}, {NULL}};
const Segment SGI253959[] = {{&S17436[35], 3}, {NULL}};
const Segment SGI253960[] = {{&S17436[66], 6}, {NULL}};
const Segment SGI253961[] = {{&S17436[112], 5}, {NULL}};
const Segment SGI253962[] = {{&S17436[125], 9}, {NULL}};
const Segment SGI253963[] = {{&S17437[13], 6}, {NULL}};
const Segment SGI253964[] = {{&S17437[101], 3}, {NULL}};
const Segment SGI253965[] = {{&S17437[35], 3}, {NULL}};
const Segment SGI253966[] = {{&S17437[66], 6}, {NULL}};
const Segment SGI253967[] = {{&S17437[112], 5}, {NULL}};
const Segment SGI253968[] = {{&S17437[125], 9}, {NULL}};
const Segment SGI253969[] = {{&S17438[13], 6}, {NULL}};
const Segment SGI253970[] = {{&S17438[101], 3}, {NULL}};
const Segment SGI253971[] = {{&S17438[35], 3}, {NULL}};
const Segment SGI253972[] = {{&S17438[66], 6}, {NULL}};
const Segment SGI253973[] = {{&S17438[112], 5}, {NULL}};
const Segment SGI253974[] = {{&S17438[125], 9}, {NULL}};
const Segment SGI253975[] = {{&S17439[13], 6}, {NULL}};
const Segment SGI253976[] = {{&S17439[101], 3}, {NULL}};
const Segment SGI253977[] = {{&S17439[35], 3}, {NULL}};
const Segment SGI253978[] = {{&S17439[66], 6}, {NULL}};
const Segment SGI253979[] = {{&S17439[112], 5}, {NULL}};
const Segment SGI253980[] = {{&S17439[125], 9}, {NULL}};
const Segment SGI253981[] = {{&S17440[13], 6}, {NULL}};
const Segment SGI253982[] = {{&S17440[103], 3}, {NULL}};
const Segment SGI253983[] = {{&S17440[35], 5}, {NULL}};
const Segment SGI253984[] = {{&S17440[68], 6}, {NULL}};
const Segment SGI253985[] = {{&S17440[114], 5}, {NULL}};
const Segment SGI253986[] = {{&S17440[127], 9}, {NULL}};
const Segment SGI253987[] = {{&S17441[13], 6}, {NULL}};
const Segment SGI253988[] = {{&S17441[101], 3}, {NULL}};
const Segment SGI253989[] = {{&S17441[35], 3}, {NULL}};
const Segment SGI253990[] = {{&S17441[66], 6}, {NULL}};
const Segment SGI253991[] = {{&S17441[112], 5}, {NULL}};
const Segment SGI253992[] = {{&S17441[125], 9}, {NULL}};
const Segment SGI253993[] = {{&S17442[13], 6}, {NULL}};
const Segment SGI253994[] = {{&S17442[103], 3}, {NULL}};
const Segment SGI253995[] = {{&S17442[35], 5}, {NULL}};
const Segment SGI253996[] = {{&S17442[68], 6}, {NULL}};
const Segment SGI253997[] = {{&S17442[114], 5}, {NULL}};
const Segment SGI253998[] = {{&S17442[127], 9}, {NULL}};
const Segment SGI253999[] = {{&S17443[13], 6}, {NULL}};
const Segment SGI254000[] = {{&S17443[103], 3}, {NULL}};
const Segment SGI254001[] = {{&S17443[35], 5}, {NULL}};
const Segment SGI254002[] = {{&S17443[68], 6}, {NULL}};
const Segment SGI254003[] = {{&S17443[114], 5}, {NULL}};
const Segment SGI254004[] = {{&S17443[127], 9}, {NULL}};
const Segment SGI254005[] = {{&S17444[13], 6}, {NULL}};
const Segment SGI254006[] = {{&S17444[103], 3}, {NULL}};
const Segment SGI254007[] = {{&S17444[35], 5}, {NULL}};
const Segment SGI254008[] = {{&S17444[68], 6}, {NULL}};
const Segment SGI254009[] = {{&S17444[114], 5}, {NULL}};
const Segment SGI254010[] = {{&S17444[127], 9}, {NULL}};
const Segment SGI254011[] = {{&S17445[13], 6}, {NULL}};
const Segment SGI254012[] = {{&S17445[96], 3}, {NULL}};
const Segment SGI254013[] = {{&S17445[28], 5}, {NULL}};
const Segment SGI254014[] = {{&S17445[61], 6}, {NULL}};
const Segment SGI254015[] = {{&S17445[107], 5}, {NULL}};
const Segment SGI254016[] = {{&S17445[120], 7}, {NULL}};
const Segment SGI254017[] = {{&S17446[13], 6}, {NULL}};
const Segment SGI254018[] = {{&S17446[103], 3}, {NULL}};
const Segment SGI254019[] = {{&S17446[35], 5}, {NULL}};
const Segment SGI254020[] = {{&S17446[68], 6}, {NULL}};
const Segment SGI254021[] = {{&S17446[114], 5}, {NULL}};
const Segment SGI254022[] = {{&S17446[127], 9}, {NULL}};
const Segment SGI254023[] = {{&S105969[34], 10}, {NULL}};
const Segment SGI254024[] = {{&S105969[27], 6}, {NULL}};
const Segment SGI254025[] = {{&S17448[13], 6}, {NULL}};
const Segment SGI254026[] = {{&S17448[123], 3}, {NULL}};
const Segment SGI254027[] = {{&S105969[96], 9}, {NULL}};
const Segment SGI254028[] = {{&S17448[35], 5}, {NULL}};
const Segment SGI254029[] = {{&S105969[69], 6}, {NULL}};
const Segment SGI254030[] = {{&S17448[68], 6}, {NULL}};
const Segment SGI254031[] = {{&S17448[102], 5}, {NULL}};
const Segment SGI254032[] = {{&S105970[34], 10}, {NULL}};
const Segment SGI254033[] = {{&S17449[13], 6}, {NULL}};
const Segment SGI254034[] = {{&S17449[121], 3}, {NULL}};
const Segment SGI254035[] = {{&S105970[27], 6}, {NULL}};
const Segment SGI254036[] = {{&S17449[35], 3}, {NULL}};
const Segment SGI254037[] = {{&S17449[66], 6}, {NULL}};
const Segment SGI254038[] = {{&S17449[100], 5}, {NULL}};
const Segment SGI254039[] = {{&S105970[96], 9}, {NULL}};
const Segment SGI254040[] = {{&S105970[69], 6}, {NULL}};
const Segment SGI254041[] = {{&S17450[13], 6}, {NULL}};
const Segment SGI254042[] = {{&S17450[121], 3}, {NULL}};
const Segment SGI254043[] = {{&S17450[35], 3}, {NULL}};
const Segment SGI254044[] = {{&S17450[66], 6}, {NULL}};
const Segment SGI254045[] = {{&S17450[100], 5}, {NULL}};
const Segment SGI254046[] = {{&S17451[13], 6}, {NULL}};
const Segment SGI254047[] = {{&S17451[94], 3}, {NULL}};
const Segment SGI254048[] = {{&S17451[28], 3}, {NULL}};
const Segment SGI254049[] = {{&S17451[59], 6}, {NULL}};
const Segment SGI254050[] = {{&S17451[105], 5}, {NULL}};
const Segment SGI254051[] = {{&S17451[118], 7}, {NULL}};
const Segment SGI254052[] = {{&S17452[13], 6}, {NULL}};
const Segment SGI254053[] = {{&S17452[101], 3}, {NULL}};
const Segment SGI254054[] = {{&S17452[35], 3}, {NULL}};
const Segment SGI254055[] = {{&S17452[66], 6}, {NULL}};
const Segment SGI254056[] = {{&S17452[112], 5}, {NULL}};
const Segment SGI254057[] = {{&S17452[125], 9}, {NULL}};
const Segment SGI254058[] = {{&S17453[13], 6}, {NULL}};
const Segment SGI254059[] = {{&S17453[103], 3}, {NULL}};
const Segment SGI254060[] = {{&S17453[35], 5}, {NULL}};
const Segment SGI254061[] = {{&S17453[68], 6}, {NULL}};
const Segment SGI254062[] = {{&S17453[114], 5}, {NULL}};
const Segment SGI254063[] = {{&S17453[127], 9}, {NULL}};
const Segment SGI254064[] = {{&S17454[13], 6}, {NULL}};
const Segment SGI254065[] = {{&S17454[123], 3}, {NULL}};
const Segment SGI254066[] = {{&S17454[35], 5}, {NULL}};
const Segment SGI254067[] = {{&S17454[68], 6}, {NULL}};
const Segment SGI254068[] = {{&S17454[102], 5}, {NULL}};
const Segment SGI254069[] = {{&S17456[13], 6}, {NULL}};
const Segment SGI254070[] = {{&S17456[103], 3}, {NULL}};
const Segment SGI254071[] = {{&S17456[35], 5}, {NULL}};
const Segment SGI254072[] = {{&S17456[68], 6}, {NULL}};
const Segment SGI254073[] = {{&S17456[114], 11}, {NULL}};
const Segment SGI254074[] = {{&S17456[126], 9}, {NULL}};
const Segment SGI254075[] = {{&S105981[34], 7}, {NULL}};
const Segment SGI254076[] = {{&S17457[13], 6}, {NULL}};
const Segment SGI254077[] = {{&S17457[113], 3}, {NULL}};
const Segment SGI254078[] = {{&S105981[27], 6}, {NULL}};
const Segment SGI254079[] = {{&S17457[38], 5}, {NULL}};
const Segment SGI254080[] = {{&S17457[78], 6}, {NULL}};
const Segment SGI254081[] = {{&S17457[124], 5}, {NULL}};
const Segment SGI254082[] = {{&S17457[137], 9}, {NULL}};
const Segment SGI254083[] = {{&S105981[93], 9}, {NULL}};
const Segment SGI254084[] = {{&S105981[66], 6}, {NULL}};
const Segment SGI254085[] = {{&S105981[134], 8}, {NULL}};
const Segment SGI254086[] = {{&S17458[13], 6}, {NULL}};
const Segment SGI254087[] = {{&S17458[103], 5}, {&S17458[130], 3}, {NULL}};
const Segment SGI254088[] = {{&S17458[35], 5}, {NULL}};
const Segment SGI254089[] = {{&S17458[68], 6}, {NULL}};
const Segment SGI254090[] = {{&S17458[116], 5}, {NULL}};
const Segment SGI254091[] = {{&S17458[141], 7}, {&S17458[156], 9}, {NULL}};
const Segment SGI254092[] = {{&S17459[13], 6}, {NULL}};
const Segment SGI254093[] = {{&S17459[103], 3}, {NULL}};
const Segment SGI254094[] = {{&S17459[35], 5}, {NULL}};
const Segment SGI254095[] = {{&S17459[68], 6}, {NULL}};
const Segment SGI254096[] = {{&S17459[114], 6}, {NULL}};
const Segment SGI254097[] = {{&S17459[128], 9}, {NULL}};
const Segment SGI254098[] = {{&S17460[13], 6}, {NULL}};
const Segment SGI254099[] = {{&S17460[102], 3}, {NULL}};
const Segment SGI254100[] = {{&S17460[67], 6}, {NULL}};
const Segment SGI254101[] = {{&S17460[113], 5}, {NULL}};
const Segment SGI254102[] = {{&S17460[126], 9}, {NULL}};
const Segment SGI254103[] = {{&S17461[13], 6}, {NULL}};
const Segment SGI254104[] = {{&S17461[103], 3}, {NULL}};
const Segment SGI254105[] = {{&S17461[35], 5}, {NULL}};
const Segment SGI254106[] = {{&S17461[68], 6}, {NULL}};
const Segment SGI254107[] = {{&S17461[114], 6}, {NULL}};
const Segment SGI254108[] = {{&S17461[128], 9}, {NULL}};
const Segment SGI254109[] = {{&S17462[13], 6}, {NULL}};
const Segment SGI254110[] = {{&S17462[103], 3}, {NULL}};
const Segment SGI254111[] = {{&S17462[35], 5}, {NULL}};
const Segment SGI254112[] = {{&S17462[68], 6}, {NULL}};
const Segment SGI254113[] = {{&S17462[114], 5}, {NULL}};
const Segment SGI254114[] = {{&S17462[127], 9}, {NULL}};
const Segment SGI254115[] = {{&S17463[13], 6}, {NULL}};
const Segment SGI254116[] = {{&S17463[103], 3}, {NULL}};
const Segment SGI254117[] = {{&S17463[35], 5}, {NULL}};
const Segment SGI254118[] = {{&S17463[68], 6}, {NULL}};
const Segment SGI254119[] = {{&S17463[114], 6}, {NULL}};
const Segment SGI254120[] = {{&S17463[128], 9}, {NULL}};
const Segment SGI254121[] = {{&S105991[34], 10}, {NULL}};
const Segment SGI254122[] = {{&S105991[27], 6}, {NULL}};
const Segment SGI254123[] = {{&S105991[97], 9}, {NULL}};
const Segment SGI254124[] = {{&S105991[70], 6}, {NULL}};
const Segment SGI254125[] = {{&S105991[138], 8}, {NULL}};
const Segment SGI254126[] = {{&S18066[13], 4}, {NULL}};
const Segment SGI254127[] = {{&S18066[92], 3}, {NULL}};
const Segment SGI254128[] = {{&S18066[26], 3}, {NULL}};
const Segment SGI254129[] = {{&S18066[57], 6}, {NULL}};
const Segment SGI254130[] = {{&S18066[103], 7}, {NULL}};
const Segment SGI254131[] = {{&S18066[118], 9}, {NULL}};
const Segment SGI254132[] = {{&S18067[13], 4}, {NULL}};
const Segment SGI254133[] = {{&S18067[92], 3}, {NULL}};
const Segment SGI254134[] = {{&S18067[26], 3}, {NULL}};
const Segment SGI254135[] = {{&S18067[57], 6}, {NULL}};
const Segment SGI254136[] = {{&S18067[103], 5}, {NULL}};
const Segment SGI254137[] = {{&S18067[116], 9}, {NULL}};
const Segment SGI254138[] = {{&S18068[13], 4}, {NULL}};
const Segment SGI254139[] = {{&S18068[92], 3}, {NULL}};
const Segment SGI254140[] = {{&S18068[26], 3}, {NULL}};
const Segment SGI254141[] = {{&S18068[57], 6}, {NULL}};
const Segment SGI254142[] = {{&S18068[103], 5}, {NULL}};
const Segment SGI254143[] = {{&S18068[116], 9}, {NULL}};
const Segment SGI254144[] = {{&S18069[13], 4}, {NULL}};
const Segment SGI254145[] = {{&S18069[92], 3}, {NULL}};
const Segment SGI254146[] = {{&S18069[26], 3}, {NULL}};
const Segment SGI254147[] = {{&S18069[57], 6}, {NULL}};
const Segment SGI254148[] = {{&S18069[103], 5}, {NULL}};
const Segment SGI254149[] = {{&S18069[116], 9}, {NULL}};
const Segment SGI254150[] = {{&S18070[13], 4}, {NULL}};
const Segment SGI254151[] = {{&S18070[92], 3}, {NULL}};
const Segment SGI254152[] = {{&S18070[26], 3}, {NULL}};
const Segment SGI254153[] = {{&S18070[57], 6}, {NULL}};
const Segment SGI254154[] = {{&S18070[103], 5}, {NULL}};
const Segment SGI254155[] = {{&S18070[116], 9}, {NULL}};
const Segment SGI254156[] = {{&S18071[13], 4}, {NULL}};
const Segment SGI254157[] = {{&S18071[92], 3}, {NULL}};
const Segment SGI254158[] = {{&S18071[26], 3}, {NULL}};
const Segment SGI254159[] = {{&S18071[57], 6}, {NULL}};
const Segment SGI254160[] = {{&S18071[103], 5}, {NULL}};
const Segment SGI254161[] = {{&S18071[116], 9}, {NULL}};
const Segment SGI254162[] = {{&S18072[13], 4}, {NULL}};
const Segment SGI254163[] = {{&S18072[92], 3}, {NULL}};
const Segment SGI254164[] = {{&S18072[26], 3}, {NULL}};
const Segment SGI254165[] = {{&S18072[57], 6}, {NULL}};
const Segment SGI254166[] = {{&S18072[103], 5}, {NULL}};
const Segment SGI254167[] = {{&S18072[116], 7}, {NULL}};
const Segment SGI254168[] = {{&S18073[13], 4}, {NULL}};
const Segment SGI254169[] = {{&S18073[94], 3}, {NULL}};
const Segment SGI254170[] = {{&S18073[26], 5}, {NULL}};
const Segment SGI254171[] = {{&S18073[59], 6}, {NULL}};
const Segment SGI254172[] = {{&S18073[105], 5}, {NULL}};
const Segment SGI254173[] = {{&S18073[118], 9}, {NULL}};
const Segment SGI254174[] = {{&S18074[13], 4}, {NULL}};
const Segment SGI254175[] = {{&S18074[94], 3}, {NULL}};
const Segment SGI254176[] = {{&S18074[26], 5}, {NULL}};
const Segment SGI254177[] = {{&S18074[59], 6}, {NULL}};
const Segment SGI254178[] = {{&S18074[105], 5}, {NULL}};
const Segment SGI254179[] = {{&S18074[118], 7}, {NULL}};
const Segment SGI254180[] = {{&S18075[13], 4}, {NULL}};
const Segment SGI254181[] = {{&S18075[94], 3}, {NULL}};
const Segment SGI254182[] = {{&S18075[26], 5}, {NULL}};
const Segment SGI254183[] = {{&S18075[59], 6}, {NULL}};
const Segment SGI254184[] = {{&S18075[105], 5}, {NULL}};
const Segment SGI254185[] = {{&S18075[118], 9}, {NULL}};
const Segment SGI254186[] = {{&S18076[13], 4}, {NULL}};
const Segment SGI254187[] = {{&S18076[94], 3}, {NULL}};
const Segment SGI254188[] = {{&S18076[26], 5}, {NULL}};
const Segment SGI254189[] = {{&S18076[59], 6}, {NULL}};
const Segment SGI254190[] = {{&S18076[105], 5}, {NULL}};
const Segment SGI254191[] = {{&S18076[118], 9}, {NULL}};
const Segment SGI254192[] = {{&S18077[13], 4}, {NULL}};
const Segment SGI254193[] = {{&S18077[114], 3}, {NULL}};
const Segment SGI254194[] = {{&S18077[26], 5}, {NULL}};
const Segment SGI254195[] = {{&S18077[59], 6}, {NULL}};
const Segment SGI254196[] = {{&S18077[93], 5}, {NULL}};
const Segment SGI254197[] = {{&S18078[13], 4}, {NULL}};
const Segment SGI254198[] = {{&S18078[112], 3}, {NULL}};
const Segment SGI254199[] = {{&S18078[26], 3}, {NULL}};
const Segment SGI254200[] = {{&S18078[57], 6}, {NULL}};
const Segment SGI254201[] = {{&S18078[91], 5}, {NULL}};
const Segment SGI254202[] = {{&S18079[13], 4}, {NULL}};
const Segment SGI254203[] = {{&S18079[92], 3}, {NULL}};
const Segment SGI254204[] = {{&S18079[26], 3}, {NULL}};
const Segment SGI254205[] = {{&S18079[57], 6}, {NULL}};
const Segment SGI254206[] = {{&S18079[103], 5}, {NULL}};
const Segment SGI254207[] = {{&S18079[116], 9}, {NULL}};
const Segment SGI254208[] = {{&S18080[13], 4}, {NULL}};
const Segment SGI254209[] = {{&S18080[92], 3}, {NULL}};
const Segment SGI254210[] = {{&S18080[26], 3}, {NULL}};
const Segment SGI254211[] = {{&S18080[57], 6}, {NULL}};
const Segment SGI254212[] = {{&S18080[103], 5}, {NULL}};
const Segment SGI254213[] = {{&S18080[116], 9}, {NULL}};
const Segment SGI254214[] = {{&S18081[13], 4}, {NULL}};
const Segment SGI254215[] = {{&S18081[92], 3}, {NULL}};
const Segment SGI254216[] = {{&S18081[26], 3}, {NULL}};
const Segment SGI254217[] = {{&S18081[57], 6}, {NULL}};
const Segment SGI254218[] = {{&S18081[103], 5}, {NULL}};
const Segment SGI254219[] = {{&S18081[116], 9}, {NULL}};
const Segment SGI254220[] = {{&S18082[13], 4}, {NULL}};
const Segment SGI254221[] = {{&S18082[92], 3}, {NULL}};
const Segment SGI254222[] = {{&S18082[26], 3}, {NULL}};
const Segment SGI254223[] = {{&S18082[57], 6}, {NULL}};
const Segment SGI254224[] = {{&S18082[103], 5}, {NULL}};
const Segment SGI254225[] = {{&S18082[116], 9}, {NULL}};
const Segment SGI254226[] = {{&S18083[13], 4}, {NULL}};
const Segment SGI254227[] = {{&S18083[107], 3}, {NULL}};
const Segment SGI254228[] = {{&S18083[26], 5}, {NULL}};
const Segment SGI254229[] = {{&S18083[59], 6}, {NULL}};
const Segment SGI254230[] = {{&S18083[93], 5}, {NULL}};
const Segment SGI254231[] = {{&S18083[118], 7}, {NULL}};
const Segment SGI254232[] = {{&S18084[13], 4}, {NULL}};
const Segment SGI254233[] = {{&S18084[94], 3}, {NULL}};
const Segment SGI254234[] = {{&S18084[26], 5}, {NULL}};
const Segment SGI254235[] = {{&S18084[59], 6}, {NULL}};
const Segment SGI254236[] = {{&S18084[105], 5}, {NULL}};
const Segment SGI254237[] = {{&S18084[118], 9}, {NULL}};
const Segment SGI254238[] = {{&S18085[13], 4}, {NULL}};
const Segment SGI254239[] = {{&S18085[105], 3}, {NULL}};
const Segment SGI254240[] = {{&S18085[26], 3}, {NULL}};
const Segment SGI254241[] = {{&S18085[57], 6}, {NULL}};
const Segment SGI254242[] = {{&S18085[91], 5}, {NULL}};
const Segment SGI254243[] = {{&S18085[116], 9}, {NULL}};
const Segment SGI254244[] = {{&S18086[13], 4}, {&S18086[13], 4}, {NULL}};
const Segment SGI254245[] = {{&S18086[119], 3}, {NULL}};
const Segment SGI254246[] = {{&S18086[29], 3}, {NULL}};
const Segment SGI254247[] = {{&S18086[84], 6}, {NULL}};
const Segment SGI254248[] = {{&S18086[130], 5}, {NULL}};
const Segment SGI254249[] = {{&S18086[143], 9}, {NULL}};
const Segment SGI254250[] = {{&S18087[13], 4}, {NULL}};
const Segment SGI254251[] = {{&S18087[94], 3}, {NULL}};
const Segment SGI254252[] = {{&S18087[26], 5}, {NULL}};
const Segment SGI254253[] = {{&S18087[59], 6}, {NULL}};
const Segment SGI254254[] = {{&S18087[105], 5}, {NULL}};
const Segment SGI254255[] = {{&S18087[118], 9}, {NULL}};
const Segment SGI254256[] = {{&S18088[13], 4}, {NULL}};
const Segment SGI254257[] = {{&S18088[94], 3}, {NULL}};
const Segment SGI254258[] = {{&S18088[26], 5}, {NULL}};
const Segment SGI254259[] = {{&S18088[59], 6}, {NULL}};
const Segment SGI254260[] = {{&S18088[105], 6}, {NULL}};
const Segment SGI254261[] = {{&S18088[119], 9}, {NULL}};
const Segment SGI254262[] = {{&S18089[13], 4}, {NULL}};
const Segment SGI254263[] = {{&S18089[105], 3}, {NULL}};
const Segment SGI254264[] = {{&S18089[26], 3}, {NULL}};
const Segment SGI254265[] = {{&S18089[57], 6}, {NULL}};
const Segment SGI254266[] = {{&S18089[91], 5}, {NULL}};
const Segment SGI254267[] = {{&S18089[116], 7}, {NULL}};
const Segment SGI254268[] = {{&S18090[13], 4}, {NULL}};
const Segment SGI254269[] = {{&S18090[94], 3}, {NULL}};
const Segment SGI254270[] = {{&S18090[26], 5}, {NULL}};
const Segment SGI254271[] = {{&S18090[59], 6}, {NULL}};
const Segment SGI254272[] = {{&S18090[105], 6}, {NULL}};
const Segment SGI254273[] = {{&S18090[119], 9}, {NULL}};
const Segment SGI254274[] = {{&S18091[13], 4}, {NULL}};
const Segment SGI254275[] = {{&S18091[94], 3}, {NULL}};
const Segment SGI254276[] = {{&S18091[26], 5}, {NULL}};
const Segment SGI254277[] = {{&S18091[59], 6}, {NULL}};
const Segment SGI254278[] = {{&S18091[105], 5}, {NULL}};
const Segment SGI254279[] = {{&S18091[118], 9}, {NULL}};
const Segment SGI254280[] = {{&S18092[13], 4}, {NULL}};
const Segment SGI254281[] = {{&S18092[94], 3}, {NULL}};
const Segment SGI254282[] = {{&S18092[26], 5}, {NULL}};
const Segment SGI254283[] = {{&S18092[59], 6}, {NULL}};
const Segment SGI254284[] = {{&S18092[105], 5}, {NULL}};
const Segment SGI254285[] = {{&S18092[118], 9}, {NULL}};
const Segment SGI254286[] = {{&S18093[13], 4}, {NULL}};
const Segment SGI254287[] = {{&S18093[94], 3}, {NULL}};
const Segment SGI254288[] = {{&S18093[26], 5}, {NULL}};
const Segment SGI254289[] = {{&S18093[59], 6}, {NULL}};
const Segment SGI254290[] = {{&S18093[105], 6}, {NULL}};
const Segment SGI254291[] = {{&S18093[119], 9}, {NULL}};
const Segment SGI254292[] = {{&S18106[13], 4}, {NULL}};
const Segment SGI254293[] = {{&S18106[105], 5}, {NULL}};
const Segment SGI254294[] = {{&S18106[29], 3}, {NULL}};
const Segment SGI254295[] = {{&S18106[67], 9}, {NULL}};
const Segment SGI254296[] = {{&S18106[118], 6}, {NULL}};
const Segment SGI254297[] = {{&S18107[13], 4}, {NULL}};
const Segment SGI254298[] = {{&S18107[105], 5}, {NULL}};
const Segment SGI254299[] = {{&S18107[29], 3}, {NULL}};
const Segment SGI254300[] = {{&S18107[67], 9}, {NULL}};
const Segment SGI254301[] = {{&S18107[118], 5}, {NULL}};
const Segment SGI254302[] = {{&S18108[12], 4}, {NULL}};
const Segment SGI254303[] = {{&S18108[111], 5}, {NULL}};
const Segment SGI254304[] = {{&S18108[35], 3}, {NULL}};
const Segment SGI254305[] = {{&S18108[73], 9}, {NULL}};
const Segment SGI254306[] = {{&S18108[124], 5}, {NULL}};
const Segment SGI254307[] = {{&S18109[13], 4}, {NULL}};
const Segment SGI254308[] = {{&S18109[107], 5}, {NULL}};
const Segment SGI254309[] = {{&S18109[29], 5}, {NULL}};
const Segment SGI254310[] = {{&S18109[69], 9}, {NULL}};
const Segment SGI254311[] = {{&S18109[120], 5}, {NULL}};
const Segment SGI254312[] = {{&S18110[13], 4}, {NULL}};
const Segment SGI254313[] = {{&S18110[105], 5}, {NULL}};
const Segment SGI254314[] = {{&S18110[29], 3}, {NULL}};
const Segment SGI254315[] = {{&S18110[67], 9}, {NULL}};
const Segment SGI254316[] = {{&S18110[118], 6}, {NULL}};
const Segment SGI254317[] = {{&S18111[13], 4}, {NULL}};
const Segment SGI254318[] = {{&S18111[107], 5}, {NULL}};
const Segment SGI254319[] = {{&S18111[29], 5}, {NULL}};
const Segment SGI254320[] = {{&S18111[69], 9}, {NULL}};
const Segment SGI254321[] = {{&S18111[120], 5}, {NULL}};
const Segment SGI254322[] = {{&S18112[13], 4}, {NULL}};
const Segment SGI254323[] = {{&S18112[104], 5}, {NULL}};
const Segment SGI254324[] = {{&S18112[29], 3}, {NULL}};
const Segment SGI254325[] = {{&S18112[67], 8}, {NULL}};
const Segment SGI254326[] = {{&S18112[117], 5}, {NULL}};
const Segment SGI254327[] = {{&S18112[130], 9}, {NULL}};
const Segment SGI254328[] = {{&S18113[24], 4}, {NULL}};
const Segment SGI254329[] = {{&S18113[118], 5}, {NULL}};
const Segment SGI254330[] = {{&S18113[40], 5}, {NULL}};
const Segment SGI254331[] = {{&S18113[80], 9}, {NULL}};
const Segment SGI254332[] = {{&S18113[131], 5}, {NULL}};
const Segment SGI254333[] = {{&S18114[13], 4}, {&S18114[13], 4}, {NULL}};
const Segment SGI254334[] = {{&S18114[202], 5}, {NULL}};
const Segment SGI254335[] = {{&S18114[26], 3}, {&S18114[126], 3}, {NULL}};
const Segment SGI254336[] = {{&S18114[57], 6}, {&S18114[164], 9}, {NULL}};
const Segment SGI254337[] = {{&S18114[91], 5}, {&S18114[215], 5}, {NULL}};
const Segment SGI254338[] = {{&S18114[164], 9}, {NULL}};
const Segment SGI254339[] = {{&S18115[13], 4}, {NULL}};
const Segment SGI254340[] = {{&S18115[107], 5}, {NULL}};
const Segment SGI254341[] = {{&S18115[29], 5}, {NULL}};
const Segment SGI254342[] = {{&S18115[69], 9}, {NULL}};
const Segment SGI254343[] = {{&S18115[120], 5}, {NULL}};
const Segment SGI254344[] = {{&S18115[69], 2}, {NULL}};
const Segment SGI254345[] = {{&S18116[13], 4}, {NULL}};
const Segment SGI254346[] = {{&S18116[107], 5}, {NULL}};
const Segment SGI254347[] = {{&S18116[29], 5}, {NULL}};
const Segment SGI254348[] = {{&S18116[69], 9}, {NULL}};
const Segment SGI254349[] = {{&S18116[120], 5}, {NULL}};
const Segment SGI254350[] = {{&S18117[13], 4}, {NULL}};
const Segment SGI254351[] = {{&S18117[105], 5}, {NULL}};
const Segment SGI254352[] = {{&S18117[29], 3}, {NULL}};
const Segment SGI254353[] = {{&S18117[67], 9}, {NULL}};
const Segment SGI254354[] = {{&S18117[118], 6}, {NULL}};
const Segment SGI254355[] = {{&S18117[67], 9}, {&S18117[149], 6}, {NULL}};
const Segment SGI254356[] = {{&S18118[13], 4}, {&S18118[13], 4}, {NULL}};
const Segment SGI254357[] = {{&S18118[204], 5}, {NULL}};
const Segment SGI254358[] = {{&S18118[26], 5}, {&S18118[128], 3}, {NULL}};
const Segment SGI254359[] = {{&S18118[59], 6}, {&S18118[166], 9}, {NULL}};
const Segment SGI254360[] = {{&S18118[93], 5}, {&S18118[217], 5}, {NULL}};
const Segment SGI254361[] = {{&S18118[166], 9}, {NULL}};
const Segment SGI254362[] = {{&S18119[13], 4}, {NULL}};
const Segment SGI254363[] = {{&S18119[105], 5}, {NULL}};
const Segment SGI254364[] = {{&S18119[29], 3}, {NULL}};
const Segment SGI254365[] = {{&S18119[67], 9}, {NULL}};
const Segment SGI254366[] = {{&S18119[118], 5}, {NULL}};
const Segment SGI254367[] = {{&S18120[13], 4}, {NULL}};
const Segment SGI254368[] = {{&S18120[105], 5}, {NULL}};
const Segment SGI254369[] = {{&S18120[29], 3}, {NULL}};
const Segment SGI254370[] = {{&S18120[67], 9}, {NULL}};
const Segment SGI254371[] = {{&S18120[118], 6}, {NULL}};
const Segment SGI254372[] = {{&S18120[67], 7}, {NULL}};
const Segment SGI254373[] = {{&S18121[12], 4}, {NULL}};
const Segment SGI254374[] = {{&S18121[111], 5}, {NULL}};
const Segment SGI254375[] = {{&S18121[35], 3}, {NULL}};
const Segment SGI254376[] = {{&S18121[73], 9}, {NULL}};
const Segment SGI254377[] = {{&S18121[124], 5}, {NULL}};
const Segment SGI254378[] = {{&S18121[137], 3}, {NULL}};
const Segment SGI254379[] = {{&S18122[13], 4}, {NULL}};
const Segment SGI254380[] = {{&S18122[107], 5}, {NULL}};
const Segment SGI254381[] = {{&S18122[29], 5}, {NULL}};
const Segment SGI254382[] = {{&S18122[69], 9}, {NULL}};
const Segment SGI254383[] = {{&S18122[120], 6}, {NULL}};
const Segment SGI254384[] = {{&S18123[13], 4}, {NULL}};
const Segment SGI254385[] = {{&S18123[109], 5}, {NULL}};
const Segment SGI254386[] = {{&S18123[36], 3}, {NULL}};
const Segment SGI254387[] = {{&S18123[74], 6}, {NULL}};
const Segment SGI254388[] = {{&S18123[122], 5}, {NULL}};
const Segment SGI254389[] = {{&S18124[13], 4}, {NULL}};
const Segment SGI254390[] = {{&S18124[104], 5}, {NULL}};
const Segment SGI254391[] = {{&S18124[29], 3}, {NULL}};
const Segment SGI254392[] = {{&S18124[67], 9}, {NULL}};
const Segment SGI254393[] = {{&S18124[117], 5}, {NULL}};
const Segment SGI254394[] = {{&S18125[13], 4}, {NULL}};
const Segment SGI254395[] = {{&S18125[105], 5}, {NULL}};
const Segment SGI254396[] = {{&S18125[29], 3}, {NULL}};
const Segment SGI254397[] = {{&S18125[67], 9}, {NULL}};
const Segment SGI254398[] = {{&S18125[118], 6}, {NULL}};
const Segment SGI254399[] = {{&S18125[132], 14}, {NULL}};
const Segment SGI254400[] = {{&S18126[13], 4}, {NULL}};
const Segment SGI254401[] = {{&S18126[105], 5}, {NULL}};
const Segment SGI254402[] = {{&S18126[29], 3}, {NULL}};
const Segment SGI254403[] = {{&S18126[67], 9}, {NULL}};
const Segment SGI254404[] = {{&S18126[118], 6}, {NULL}};
const Segment SGI254405[] = {{&S18126[132], 14}, {NULL}};
const Segment SGI254406[] = {{&S18127[13], 4}, {NULL}};
const Segment SGI254407[] = {{&S18127[112], 5}, {NULL}};
const Segment SGI254408[] = {{&S18127[36], 3}, {NULL}};
const Segment SGI254409[] = {{&S18127[74], 9}, {NULL}};
const Segment SGI254410[] = {{&S18127[125], 5}, {NULL}};
const Segment SGI254411[] = {{&S18541[13], 6}, {NULL}};
const Segment SGI254412[] = {{&S18541[101], 3}, {NULL}};
const Segment SGI254413[] = {{&S18541[35], 3}, {NULL}};
const Segment SGI254414[] = {{&S18541[66], 6}, {NULL}};
const Segment SGI254415[] = {{&S18541[112], 7}, {NULL}};
const Segment SGI254416[] = {{&S18541[127], 9}, {NULL}};
const Segment SGI254417[] = {{&S18542[13], 6}, {NULL}};
const Segment SGI254418[] = {{&S18542[101], 3}, {NULL}};
const Segment SGI254419[] = {{&S18542[35], 3}, {NULL}};
const Segment SGI254420[] = {{&S18542[66], 6}, {NULL}};
const Segment SGI254421[] = {{&S18542[112], 7}, {NULL}};
const Segment SGI254422[] = {{&S18542[127], 9}, {NULL}};
const Segment SGI254423[] = {{&S18543[13], 6}, {NULL}};
const Segment SGI254424[] = {{&S18543[101], 3}, {NULL}};
const Segment SGI254425[] = {{&S18543[35], 3}, {NULL}};
const Segment SGI254426[] = {{&S18543[66], 6}, {NULL}};
const Segment SGI254427[] = {{&S18543[112], 7}, {NULL}};
const Segment SGI254428[] = {{&S18543[127], 9}, {NULL}};
const Segment SGI254429[] = {{&S18544[13], 6}, {NULL}};
const Segment SGI254430[] = {{&S18544[101], 3}, {NULL}};
const Segment SGI254431[] = {{&S18544[35], 3}, {NULL}};
const Segment SGI254432[] = {{&S18544[66], 6}, {NULL}};
const Segment SGI254433[] = {{&S18544[112], 7}, {NULL}};
const Segment SGI254434[] = {{&S18544[127], 9}, {NULL}};
const Segment SGI254435[] = {{&S18545[13], 6}, {NULL}};
const Segment SGI254436[] = {{&S18545[101], 3}, {NULL}};
const Segment SGI254437[] = {{&S18545[35], 3}, {NULL}};
const Segment SGI254438[] = {{&S18545[66], 6}, {NULL}};
const Segment SGI254439[] = {{&S18545[112], 5}, {NULL}};
const Segment SGI254440[] = {{&S18545[125], 9}, {NULL}};
const Segment SGI254441[] = {{&S18546[13], 6}, {NULL}};
const Segment SGI254442[] = {{&S18546[103], 3}, {NULL}};
const Segment SGI254443[] = {{&S18546[35], 5}, {NULL}};
const Segment SGI254444[] = {{&S18546[68], 6}, {NULL}};
const Segment SGI254445[] = {{&S18546[114], 5}, {NULL}};
const Segment SGI254446[] = {{&S18546[127], 9}, {NULL}};
const Segment SGI254447[] = {{&S18547[13], 6}, {NULL}};
const Segment SGI254448[] = {{&S18547[103], 3}, {NULL}};
const Segment SGI254449[] = {{&S18547[35], 5}, {NULL}};
const Segment SGI254450[] = {{&S18547[68], 6}, {NULL}};
const Segment SGI254451[] = {{&S18547[114], 5}, {NULL}};
const Segment SGI254452[] = {{&S18547[127], 9}, {NULL}};
const Segment SGI254453[] = {{&S18548[13], 6}, {NULL}};
const Segment SGI254454[] = {{&S18548[101], 3}, {NULL}};
const Segment SGI254455[] = {{&S18548[35], 3}, {NULL}};
const Segment SGI254456[] = {{&S18548[66], 6}, {NULL}};
const Segment SGI254457[] = {{&S18548[112], 7}, {NULL}};
const Segment SGI254458[] = {{&S18548[127], 9}, {NULL}};
const Segment SGI254459[] = {{&S18549[13], 6}, {NULL}};
const Segment SGI254460[] = {{&S18549[103], 3}, {NULL}};
const Segment SGI254461[] = {{&S18549[35], 5}, {NULL}};
const Segment SGI254462[] = {{&S18549[68], 6}, {NULL}};
const Segment SGI254463[] = {{&S18549[114], 5}, {NULL}};
const Segment SGI254464[] = {{&S18549[127], 9}, {NULL}};
const Segment SGI254465[] = {{&S18550[13], 6}, {NULL}};
const Segment SGI254466[] = {{&S18550[101], 3}, {NULL}};
const Segment SGI254467[] = {{&S18550[35], 3}, {NULL}};
const Segment SGI254468[] = {{&S18550[66], 6}, {NULL}};
const Segment SGI254469[] = {{&S18550[112], 7}, {NULL}};
const Segment SGI254470[] = {{&S18550[127], 9}, {NULL}};
const Segment SGI254471[] = {{&S18551[13], 6}, {NULL}};
const Segment SGI254472[] = {{&S18551[101], 3}, {NULL}};
const Segment SGI254473[] = {{&S18551[35], 3}, {NULL}};
const Segment SGI254474[] = {{&S18551[66], 6}, {NULL}};
const Segment SGI254475[] = {{&S18551[112], 5}, {NULL}};
const Segment SGI254476[] = {{&S18551[125], 9}, {NULL}};
const Segment SGI254477[] = {{&S18552[13], 6}, {NULL}};
const Segment SGI254478[] = {{&S18552[103], 3}, {NULL}};
const Segment SGI254479[] = {{&S18552[35], 5}, {NULL}};
const Segment SGI254480[] = {{&S18552[68], 6}, {NULL}};
const Segment SGI254481[] = {{&S18552[114], 5}, {NULL}};
const Segment SGI254482[] = {{&S18552[127], 9}, {NULL}};
const Segment SGI254483[] = {{&S18553[13], 6}, {NULL}};
const Segment SGI254484[] = {{&S18553[103], 3}, {NULL}};
const Segment SGI254485[] = {{&S18553[35], 5}, {NULL}};
const Segment SGI254486[] = {{&S18553[68], 6}, {NULL}};
const Segment SGI254487[] = {{&S18553[114], 5}, {NULL}};
const Segment SGI254488[] = {{&S18553[127], 9}, {NULL}};
const Segment SGI254489[] = {{&S18554[13], 6}, {NULL}};
const Segment SGI254490[] = {{&S18554[96], 3}, {NULL}};
const Segment SGI254491[] = {{&S18554[28], 5}, {NULL}};
const Segment SGI254492[] = {{&S18554[61], 6}, {NULL}};
const Segment SGI254493[] = {{&S18554[107], 5}, {NULL}};
const Segment SGI254494[] = {{&S18554[120], 7}, {NULL}};
const Segment SGI254495[] = {{&S18555[13], 6}, {NULL}};
const Segment SGI254496[] = {{&S18555[103], 3}, {NULL}};
const Segment SGI254497[] = {{&S18555[35], 5}, {NULL}};
const Segment SGI254498[] = {{&S18555[68], 6}, {NULL}};
const Segment SGI254499[] = {{&S18555[114], 5}, {NULL}};
const Segment SGI254500[] = {{&S18555[127], 9}, {NULL}};
const Segment SGI254501[] = {{&S18556[13], 6}, {NULL}};
const Segment SGI254502[] = {{&S18556[101], 3}, {NULL}};
const Segment SGI254503[] = {{&S18556[35], 3}, {NULL}};
const Segment SGI254504[] = {{&S18556[66], 6}, {NULL}};
const Segment SGI254505[] = {{&S18556[112], 5}, {NULL}};
const Segment SGI254506[] = {{&S18556[125], 9}, {NULL}};
const Segment SGI254507[] = {{&S18558[13], 6}, {NULL}};
const Segment SGI254508[] = {{&S18558[101], 3}, {NULL}};
const Segment SGI254509[] = {{&S18558[35], 3}, {NULL}};
const Segment SGI254510[] = {{&S18558[66], 6}, {NULL}};
const Segment SGI254511[] = {{&S18558[112], 5}, {NULL}};
const Segment SGI254512[] = {{&S18558[125], 9}, {NULL}};
const Segment SGI254513[] = {{&S18559[13], 6}, {NULL}};
const Segment SGI254514[] = {{&S18559[103], 3}, {NULL}};
const Segment SGI254515[] = {{&S18559[35], 5}, {NULL}};
const Segment SGI254516[] = {{&S18559[68], 6}, {NULL}};
const Segment SGI254517[] = {{&S18559[114], 5}, {NULL}};
const Segment SGI254518[] = {{&S18559[127], 9}, {NULL}};
const Segment SGI254519[] = {{&S18560[13], 6}, {NULL}};
const Segment SGI254520[] = {{&S18560[123], 3}, {NULL}};
const Segment SGI254521[] = {{&S18560[35], 5}, {NULL}};
const Segment SGI254522[] = {{&S18560[68], 6}, {NULL}};
const Segment SGI254523[] = {{&S18560[102], 5}, {NULL}};
const Segment SGI254524[] = {{&S18561[13], 6}, {NULL}};
const Segment SGI254525[] = {{&S18561[101], 3}, {NULL}};
const Segment SGI254526[] = {{&S18561[35], 3}, {NULL}};
const Segment SGI254527[] = {{&S18561[66], 6}, {NULL}};
const Segment SGI254528[] = {{&S18561[112], 5}, {NULL}};
const Segment SGI254529[] = {{&S18561[125], 10}, {NULL}};
const Segment SGI254530[] = {{&S18562[13], 6}, {NULL}};
const Segment SGI254531[] = {{&S18562[121], 3}, {NULL}};
const Segment SGI254532[] = {{&S18562[35], 3}, {NULL}};
const Segment SGI254533[] = {{&S18562[66], 6}, {NULL}};
const Segment SGI254534[] = {{&S18562[100], 5}, {NULL}};
const Segment SGI254535[] = {{&S18563[13], 6}, {NULL}};
const Segment SGI254536[] = {{&S18563[103], 3}, {NULL}};
const Segment SGI254537[] = {{&S18563[35], 5}, {NULL}};
const Segment SGI254538[] = {{&S18563[68], 6}, {NULL}};
const Segment SGI254539[] = {{&S18563[114], 5}, {NULL}};
const Segment SGI254540[] = {{&S18563[127], 9}, {NULL}};
const Segment SGI254541[] = {{&S18564[13], 6}, {NULL}};
const Segment SGI254542[] = {{&S18564[82], 3}, {NULL}};
const Segment SGI254543[] = {{&S18564[31], 4}, {NULL}};
const Segment SGI254544[] = {{&S18564[48], 5}, {NULL}};
const Segment SGI254545[] = {{&S18564[100], 7}, {NULL}};
const Segment SGI254546[] = {{&S18565[13], 6}, {NULL}};
const Segment SGI254547[] = {{&S18565[108], 3}, {NULL}};
const Segment SGI254548[] = {{&S18565[38], 3}, {NULL}};
const Segment SGI254549[] = {{&S18565[73], 6}, {NULL}};
const Segment SGI254550[] = {{&S18565[119], 5}, {NULL}};
const Segment SGI254551[] = {{&S18565[132], 6}, {NULL}};
const Segment SGI254552[] = {{&S18566[13], 6}, {NULL}};
const Segment SGI254553[] = {{&S18566[103], 3}, {NULL}};
const Segment SGI254554[] = {{&S18566[35], 5}, {NULL}};
const Segment SGI254555[] = {{&S18566[68], 6}, {NULL}};
const Segment SGI254556[] = {{&S18566[114], 2}, {NULL}};
const Segment SGI254557[] = {{&S18566[131], 9}, {NULL}};
const Segment SGI254558[] = {{&S18567[13], 6}, {NULL}};
const Segment SGI254559[] = {{&S18567[103], 3}, {NULL}};
const Segment SGI254560[] = {{&S18567[35], 5}, {NULL}};
const Segment SGI254561[] = {{&S18567[68], 6}, {NULL}};
const Segment SGI254562[] = {{&S18567[114], 5}, {NULL}};
const Segment SGI254563[] = {{&S18567[127], 9}, {NULL}};
const Segment SGI254564[] = {{&S18568[13], 6}, {NULL}};
const Segment SGI254565[] = {{&S18568[103], 3}, {NULL}};
const Segment SGI254566[] = {{&S18568[35], 5}, {NULL}};
const Segment SGI254567[] = {{&S18568[68], 6}, {NULL}};
const Segment SGI254568[] = {{&S18568[114], 5}, {NULL}};
const Segment SGI254569[] = {{&S18713[13], 4}, {NULL}};
const Segment SGI254570[] = {{&S18713[113], 5}, {NULL}};
const Segment SGI254571[] = {{&S18713[36], 5}, {NULL}};
const Segment SGI254572[] = {{&S18713[76], 8}, {NULL}};
const Segment SGI254573[] = {{&S18713[126], 3}, {NULL}};
const Segment SGI254574[] = {{&S18713[137], 9}, {NULL}};
const Segment SGI254575[] = {{&S18714[13], 4}, {NULL}};
const Segment SGI254576[] = {{&S18714[111], 5}, {NULL}};
const Segment SGI254577[] = {{&S18714[36], 3}, {NULL}};
const Segment SGI254578[] = {{&S18714[74], 8}, {NULL}};
const Segment SGI254579[] = {{&S18714[124], 5}, {NULL}};
const Segment SGI254580[] = {{&S18714[137], 9}, {NULL}};
const Segment SGI254581[] = {{&S18715[13], 4}, {NULL}};
const Segment SGI254582[] = {{&S18715[113], 5}, {NULL}};
const Segment SGI254583[] = {{&S18715[36], 5}, {NULL}};
const Segment SGI254584[] = {{&S18715[76], 8}, {NULL}};
const Segment SGI254585[] = {{&S18715[126], 3}, {NULL}};
const Segment SGI254586[] = {{&S18715[137], 9}, {NULL}};
const Segment SGI254587[] = {{&S18716[13], 4}, {NULL}};
const Segment SGI254588[] = {{&S18716[113], 5}, {NULL}};
const Segment SGI254589[] = {{&S18716[36], 5}, {NULL}};
const Segment SGI254590[] = {{&S18716[76], 8}, {NULL}};
const Segment SGI254591[] = {{&S18716[126], 3}, {NULL}};
const Segment SGI254592[] = {{&S18716[137], 9}, {NULL}};
const Segment SGI254593[] = {{&S18717[13], 4}, {NULL}};
const Segment SGI254594[] = {{&S18717[113], 5}, {NULL}};
const Segment SGI254595[] = {{&S18717[36], 5}, {NULL}};
const Segment SGI254596[] = {{&S18717[76], 8}, {NULL}};
const Segment SGI254597[] = {{&S18717[126], 3}, {NULL}};
const Segment SGI254598[] = {{&S18717[137], 9}, {NULL}};
const Segment SGI254599[] = {{&S18718[13], 4}, {NULL}};
const Segment SGI254600[] = {{&S18718[113], 5}, {NULL}};
const Segment SGI254601[] = {{&S18718[36], 5}, {NULL}};
const Segment SGI254602[] = {{&S18718[76], 8}, {NULL}};
const Segment SGI254603[] = {{&S18718[126], 3}, {NULL}};
const Segment SGI254604[] = {{&S18718[137], 9}, {NULL}};
const Segment SGI254605[] = {{&S18719[13], 4}, {NULL}};
const Segment SGI254606[] = {{&S18719[113], 5}, {NULL}};
const Segment SGI254607[] = {{&S18719[36], 5}, {NULL}};
const Segment SGI254608[] = {{&S18719[76], 8}, {NULL}};
const Segment SGI254609[] = {{&S18719[126], 5}, {NULL}};
const Segment SGI254610[] = {{&S18719[139], 9}, {NULL}};
const Segment SGI254611[] = {{&S18720[13], 4}, {NULL}};
const Segment SGI254612[] = {{&S18720[113], 5}, {NULL}};
const Segment SGI254613[] = {{&S18720[36], 5}, {NULL}};
const Segment SGI254614[] = {{&S18720[76], 8}, {NULL}};
const Segment SGI254615[] = {{&S18720[126], 5}, {NULL}};
const Segment SGI254616[] = {{&S18720[139], 9}, {NULL}};
const Segment SGI254617[] = {{&S18721[13], 4}, {NULL}};
const Segment SGI254618[] = {{&S18721[111], 5}, {NULL}};
const Segment SGI254619[] = {{&S18721[36], 3}, {NULL}};
const Segment SGI254620[] = {{&S18721[74], 8}, {NULL}};
const Segment SGI254621[] = {{&S18721[124], 5}, {NULL}};
const Segment SGI254622[] = {{&S18721[137], 9}, {NULL}};
const Segment SGI254623[] = {{&S18722[13], 4}, {NULL}};
const Segment SGI254624[] = {{&S18722[110], 5}, {NULL}};
const Segment SGI254625[] = {{&S18722[36], 3}, {NULL}};
const Segment SGI254626[] = {{&S18722[74], 8}, {NULL}};
const Segment SGI254627[] = {{&S18722[123], 3}, {NULL}};
const Segment SGI254628[] = {{&S18722[134], 9}, {NULL}};
const Segment SGI254629[] = {{&S18723[13], 4}, {NULL}};
const Segment SGI254630[] = {{&S18723[110], 5}, {NULL}};
const Segment SGI254631[] = {{&S18723[36], 3}, {NULL}};
const Segment SGI254632[] = {{&S18723[74], 8}, {NULL}};
const Segment SGI254633[] = {{&S18723[123], 3}, {NULL}};
const Segment SGI254634[] = {{&S18723[134], 9}, {NULL}};
const Segment SGI254635[] = {{&S18724[13], 4}, {NULL}};
const Segment SGI254636[] = {{&S18724[109], 5}, {NULL}};
const Segment SGI254637[] = {{&S18724[36], 2}, {NULL}};
const Segment SGI254638[] = {{&S18724[73], 8}, {NULL}};
const Segment SGI254639[] = {{&S18724[122], 5}, {NULL}};
const Segment SGI254640[] = {{&S18724[135], 9}, {NULL}};
const Segment SGI254641[] = {{&S18725[13], 4}, {NULL}};
const Segment SGI254642[] = {{&S18725[110], 5}, {NULL}};
const Segment SGI254643[] = {{&S18725[36], 3}, {NULL}};
const Segment SGI254644[] = {{&S18725[74], 8}, {NULL}};
const Segment SGI254645[] = {{&S18725[123], 3}, {NULL}};
const Segment SGI254646[] = {{&S18725[134], 9}, {NULL}};
const Segment SGI254647[] = {{&S18726[13], 4}, {NULL}};
const Segment SGI254648[] = {{&S18726[111], 5}, {NULL}};
const Segment SGI254649[] = {{&S18726[36], 3}, {NULL}};
const Segment SGI254650[] = {{&S18726[74], 8}, {NULL}};
const Segment SGI254651[] = {{&S18726[124], 3}, {NULL}};
const Segment SGI254652[] = {{&S18726[135], 9}, {NULL}};
const Segment SGI254653[] = {{&S18727[13], 4}, {NULL}};
const Segment SGI254654[] = {{&S18727[110], 5}, {NULL}};
const Segment SGI254655[] = {{&S18727[36], 3}, {NULL}};
const Segment SGI254656[] = {{&S18727[74], 8}, {NULL}};
const Segment SGI254657[] = {{&S18727[123], 3}, {NULL}};
const Segment SGI254658[] = {{&S18727[134], 9}, {NULL}};
const Segment SGI254659[] = {{&S18728[13], 4}, {NULL}};
const Segment SGI254660[] = {{&S18728[110], 5}, {NULL}};
const Segment SGI254661[] = {{&S18728[36], 3}, {NULL}};
const Segment SGI254662[] = {{&S18728[74], 8}, {NULL}};
const Segment SGI254663[] = {{&S18728[123], 3}, {NULL}};
const Segment SGI254664[] = {{&S18728[134], 9}, {NULL}};
const Segment SGI254665[] = {{&S18729[13], 4}, {NULL}};
const Segment SGI254666[] = {{&S18729[113], 5}, {NULL}};
const Segment SGI254667[] = {{&S18729[36], 5}, {NULL}};
const Segment SGI254668[] = {{&S18729[76], 8}, {NULL}};
const Segment SGI254669[] = {{&S18729[126], 5}, {NULL}};
const Segment SGI254670[] = {{&S18729[139], 9}, {NULL}};
const Segment SGI254671[] = {{&S18730[13], 4}, {NULL}};
const Segment SGI254672[] = {{&S18730[113], 5}, {NULL}};
const Segment SGI254673[] = {{&S18730[36], 5}, {NULL}};
const Segment SGI254674[] = {{&S18730[76], 8}, {NULL}};
const Segment SGI254675[] = {{&S18730[126], 5}, {NULL}};
const Segment SGI254676[] = {{&S18730[139], 9}, {NULL}};
const Segment SGI254677[] = {{&S18846[13], 6}, {NULL}};
const Segment SGI254678[] = {{&S18846[38], 3}, {NULL}};
const Segment SGI254679[] = {{&S18846[76], 8}, {NULL}};
const Segment SGI254680[] = {{&S18846[112], 7}, {NULL}};
const Segment SGI254681[] = {{&S18847[13], 6}, {NULL}};
const Segment SGI254682[] = {{&S18847[38], 3}, {NULL}};
const Segment SGI254683[] = {{&S18847[76], 8}, {NULL}};
const Segment SGI254684[] = {{&S18847[112], 5}, {NULL}};
const Segment SGI254685[] = {{&S18848[13], 6}, {NULL}};
const Segment SGI254686[] = {{&S18848[38], 5}, {NULL}};
const Segment SGI254687[] = {{&S18848[121], 4}, {NULL}};
const Segment SGI254688[] = {{&S18849[13], 6}, {NULL}};
const Segment SGI254689[] = {{&S18849[38], 5}, {NULL}};
const Segment SGI254690[] = {{&S18849[78], 8}, {NULL}};
const Segment SGI254691[] = {{&S18849[114], 5}, {NULL}};
const Segment SGI254692[] = {{&S18850[13], 6}, {NULL}};
const Segment SGI254693[] = {{&S18850[38], 5}, {NULL}};
const Segment SGI254694[] = {{&S18850[78], 8}, {NULL}};
const Segment SGI254695[] = {{&S18851[13], 6}, {NULL}};
const Segment SGI254696[] = {{&S18851[38], 5}, {NULL}};
const Segment SGI254697[] = {{&S18851[78], 8}, {NULL}};
const Segment SGI254698[] = {{&S18851[114], 6}, {NULL}};
const Segment SGI254699[] = {{&S18852[13], 6}, {NULL}};
const Segment SGI254700[] = {{&S18852[38], 3}, {NULL}};
const Segment SGI254701[] = {{&S18852[76], 8}, {NULL}};
const Segment SGI254702[] = {{&S18853[13], 6}, {NULL}};
const Segment SGI254703[] = {{&S18853[38], 5}, {NULL}};
const Segment SGI254704[] = {{&S18853[78], 8}, {NULL}};
const Segment SGI254705[] = {{&S18854[13], 6}, {NULL}};
const Segment SGI254706[] = {{&S18854[38], 3}, {NULL}};
const Segment SGI254707[] = {{&S18854[76], 8}, {NULL}};
const Segment SGI254708[] = {{&S18855[13], 6}, {NULL}};
const Segment SGI254709[] = {{&S18855[38], 5}, {NULL}};
const Segment SGI254710[] = {{&S18855[78], 8}, {NULL}};
const Segment SGI254711[] = {{&S18855[114], 5}, {NULL}};
const Segment SGI254712[] = {{&S18856[13], 6}, {NULL}};
const Segment SGI254713[] = {{&S18856[38], 5}, {NULL}};
const Segment SGI254714[] = {{&S18856[78], 8}, {NULL}};
const Segment SGI254715[] = {{&S18856[114], 6}, {NULL}};
const Segment SGI254716[] = {{&S18857[13], 6}, {NULL}};
const Segment SGI254717[] = {{&S18857[38], 5}, {NULL}};
const Segment SGI254718[] = {{&S18857[78], 8}, {NULL}};
const Segment SGI254719[] = {{&S18858[13], 6}, {NULL}};
const Segment SGI254720[] = {{&S18858[38], 5}, {NULL}};
const Segment SGI254721[] = {{&S18858[78], 8}, {NULL}};
const Segment SGI254722[] = {{&S18858[114], 5}, {NULL}};
const Segment SGI254723[] = {{&S18859[13], 6}, {NULL}};
const Segment SGI254724[] = {{&S18859[38], 5}, {NULL}};
const Segment SGI254725[] = {{&S18859[78], 8}, {NULL}};
const Segment SGI254726[] = {{&S18860[13], 6}, {NULL}};
const Segment SGI254727[] = {{&S18860[38], 5}, {NULL}};
const Segment SGI254728[] = {{&S18860[78], 8}, {NULL}};
const Segment SGI254729[] = {{&S18860[114], 5}, {NULL}};
const Segment SGI254730[] = {{&S18861[13], 6}, {NULL}};
const Segment SGI254731[] = {{&S18861[38], 5}, {NULL}};
const Segment SGI254732[] = {{&S18861[78], 8}, {NULL}};
const Segment SGI254733[] = {{&S18861[114], 5}, {NULL}};
const Segment SGI254734[] = {{&S18862[13], 6}, {NULL}};
const Segment SGI254735[] = {{&S18862[38], 5}, {NULL}};
const Segment SGI254736[] = {{&S18862[78], 8}, {NULL}};
const Segment SGI254737[] = {{&S18862[114], 5}, {NULL}};
const Segment SGI254738[] = {{&S18863[13], 6}, {NULL}};
const Segment SGI254739[] = {{&S18863[38], 5}, {NULL}};
const Segment SGI254740[] = {{&S18863[78], 8}, {NULL}};
const Segment SGI254741[] = {{&S18864[13], 6}, {NULL}};
const Segment SGI254742[] = {{&S18864[38], 5}, {NULL}};
const Segment SGI254743[] = {{&S18864[78], 8}, {NULL}};
const Segment SGI254744[] = {{&S18865[13], 6}, {NULL}};
const Segment SGI254745[] = {{&S18865[38], 5}, {NULL}};
const Segment SGI254746[] = {{&S18865[78], 8}, {NULL}};
const Segment SGI254747[] = {{&S18865[114], 5}, {NULL}};
const Segment SGI254748[] = {{&S18866[13], 6}, {NULL}};
const Segment SGI254749[] = {{&S18866[38], 5}, {NULL}};
const Segment SGI254750[] = {{&S18866[78], 8}, {NULL}};
const Segment SGI254751[] = {{&S18866[114], 5}, {NULL}};
const Segment SGI254752[] = {{&S18867[13], 6}, {NULL}};
const Segment SGI254753[] = {{&S18867[38], 6}, {NULL}};
const Segment SGI254754[] = {{&S18867[79], 8}, {NULL}};
const Segment SGI254755[] = {{&S18867[115], 5}, {NULL}};
const Segment SGI254756[] = {{&S18868[13], 6}, {NULL}};
const Segment SGI254757[] = {{&S18868[38], 5}, {NULL}};
const Segment SGI254758[] = {{&S18868[78], 8}, {NULL}};
const Segment SGI254759[] = {{&S18869[13], 6}, {NULL}};
const Segment SGI254760[] = {{&S18869[38], 5}, {NULL}};
const Segment SGI254761[] = {{&S18869[78], 8}, {NULL}};
const Segment SGI254762[] = {{&S18869[114], 5}, {NULL}};
const Segment SGI254763[] = {{&S18870[13], 6}, {NULL}};
const Segment SGI254764[] = {{&S18870[38], 6}, {NULL}};
const Segment SGI254765[] = {{&S18870[79], 8}, {NULL}};
const Segment SGI254766[] = {{&S18870[115], 5}, {NULL}};
const Segment SGI254767[] = {{&S18871[13], 6}, {NULL}};
const Segment SGI254768[] = {{&S18871[38], 6}, {NULL}};
const Segment SGI254769[] = {{&S18871[79], 8}, {NULL}};
const Segment SGI254770[] = {{&S18872[13], 6}, {NULL}};
const Segment SGI254771[] = {{&S18872[38], 5}, {NULL}};
const Segment SGI254772[] = {{&S18872[78], 8}, {NULL}};
const Segment SGI254773[] = {{&S18872[114], 5}, {NULL}};
const Segment SGI254774[] = {{&S18874[15], 6}, {NULL}};
const Segment SGI254775[] = {{&S18874[33], 5}, {NULL}};
const Segment SGI254776[] = {{&S18874[0], 7}, {NULL}};
const Segment SGI254777[] = {{&S18875[13], 6}, {NULL}};
const Segment SGI254778[] = {{&S18875[38], 5}, {NULL}};
const Segment SGI254779[] = {{&S18876[13], 6}, {NULL}};
const Segment SGI254780[] = {{&S18876[38], 5}, {NULL}};
const Segment SGI254781[] = {{&S18876[78], 5}, {NULL}};
const Segment SGI254782[] = {{&S18876[111], 5}, {NULL}};
const Segment SGI254783[] = {{&S18877[13], 6}, {NULL}};
const Segment SGI254784[] = {{&S18877[38], 5}, {NULL}};
const Segment SGI254785[] = {{&S18878[13], 6}, {NULL}};
const Segment SGI254786[] = {{&S18878[38], 6}, {NULL}};
const Segment SGI254787[] = {{&S18879[13], 6}, {NULL}};
const Segment SGI254788[] = {{&S18879[38], 5}, {NULL}};
const Segment SGI254789[] = {{&S18880[13], 6}, {NULL}};
const Segment SGI254790[] = {{&S18880[38], 5}, {NULL}};
const Segment SGI254791[] = {{&S18881[13], 6}, {NULL}};
const Segment SGI254792[] = {{&S18881[38], 5}, {NULL}};
const Segment SGI254793[] = {{&S18882[27], 6}, {NULL}};
const Segment SGI254794[] = {{&S18882[52], 5}, {NULL}};
const Segment SGI254795[] = {{&S18882[92], 6}, {NULL}};
const Segment SGI254796[] = {{&S18884[13], 6}, {NULL}};
const Segment SGI254797[] = {{&S18884[38], 5}, {NULL}};
const Segment SGI254798[] = {{&S18885[13], 6}, {&S18885[120], 4}, {NULL}};
const Segment SGI254799[] = {{&S18885[38], 5}, {NULL}};
const Segment SGI254800[] = {{&S18885[78], 8}, {NULL}};
const Segment SGI254801[] = {{&S18885[114], 5}, {NULL}};
const Segment SGI254802[] = {{&S18886[13], 6}, {NULL}};
const Segment SGI254803[] = {{&S18886[28], 3}, {NULL}};
const Segment SGI254804[] = {{&S18886[59], 6}, {NULL}};
const Segment SGI254805[] = {{&S18886[93], 5}, {NULL}};
const Segment SGI254806[] = {{&S18886[106], 7}, {NULL}};
const Segment SGI254807[] = {{&S18887[13], 6}, {NULL}};
const Segment SGI254808[] = {{&S18887[38], 3}, {NULL}};
const Segment SGI254809[] = {{&S18887[76], 8}, {NULL}};
const Segment SGI254810[] = {{&S18887[112], 6}, {NULL}};
const Segment SGI254811[] = {{&S18888[13], 6}, {NULL}};
const Segment SGI254812[] = {{&S18888[38], 3}, {NULL}};
const Segment SGI254813[] = {{&S18889[13], 6}, {&S18889[13], 6}, {NULL}};
const Segment SGI254814[] = {{&S18889[75], 6}, {NULL}};
const Segment SGI254815[] = {{&S18889[109], 4}, {NULL}};
const Segment SGI254816[] = {{&S18890[13], 6}, {NULL}};
const Segment SGI254817[] = {{&S18890[144], 3}, {NULL}};
const Segment SGI254818[] = {{&S18890[38], 5}, {NULL}};
const Segment SGI254819[] = {{&S18890[78], 8}, {NULL}};
const Segment SGI254820[] = {{&S18890[114], 5}, {NULL}};
const Segment SGI254821[] = {{&S18891[13], 6}, {&S18891[128], 4}, {NULL}};
const Segment SGI254822[] = {{&S18891[75], 6}, {NULL}};
const Segment SGI254823[] = {{&S18891[109], 4}, {NULL}};
const Segment SGI254824[] = {{&S18922[13], 6}, {NULL}};
const Segment SGI254825[] = {{&S18922[35], 3}, {NULL}};
const Segment SGI254826[] = {{&S18922[73], 6}, {NULL}};
const Segment SGI254827[] = {{&S18922[126], 8}, {NULL}};
const Segment SGI254828[] = {{&S18922[142], 7}, {NULL}};
const Segment SGI254829[] = {{&S18923[13], 6}, {NULL}};
const Segment SGI254830[] = {{&S18923[35], 3}, {NULL}};
const Segment SGI254831[] = {{&S18923[73], 6}, {NULL}};
const Segment SGI254832[] = {{&S18923[126], 8}, {NULL}};
const Segment SGI254833[] = {{&S18923[142], 7}, {NULL}};
const Segment SGI254834[] = {{&S18924[13], 6}, {NULL}};
const Segment SGI254835[] = {{&S18924[35], 3}, {NULL}};
const Segment SGI254836[] = {{&S18924[73], 6}, {NULL}};
const Segment SGI254837[] = {{&S18924[127], 8}, {NULL}};
const Segment SGI254838[] = {{&S18924[143], 7}, {NULL}};
const Segment SGI254839[] = {{&S18925[13], 6}, {NULL}};
const Segment SGI254840[] = {{&S18925[35], 3}, {NULL}};
const Segment SGI254841[] = {{&S18925[73], 6}, {NULL}};
const Segment SGI254842[] = {{&S18925[127], 6}, {NULL}};
const Segment SGI254843[] = {{&S18925[141], 7}, {NULL}};
const Segment SGI254844[] = {{&S18926[13], 6}, {NULL}};
const Segment SGI254845[] = {{&S18926[35], 3}, {NULL}};
const Segment SGI254846[] = {{&S18926[73], 6}, {NULL}};
const Segment SGI254847[] = {{&S18926[127], 6}, {NULL}};
const Segment SGI254848[] = {{&S18926[141], 7}, {NULL}};
const Segment SGI254849[] = {{&S18927[13], 6}, {NULL}};
const Segment SGI254850[] = {{&S18927[35], 3}, {NULL}};
const Segment SGI254851[] = {{&S18927[73], 6}, {NULL}};
const Segment SGI254852[] = {{&S18927[127], 6}, {NULL}};
const Segment SGI254853[] = {{&S18927[141], 7}, {NULL}};
const Segment SGI254854[] = {{&S18928[13], 6}, {NULL}};
const Segment SGI254855[] = {{&S18928[35], 3}, {NULL}};
const Segment SGI254856[] = {{&S18928[73], 6}, {NULL}};
const Segment SGI254857[] = {{&S18928[127], 6}, {NULL}};
const Segment SGI254858[] = {{&S18928[141], 7}, {NULL}};
const Segment SGI254859[] = {{&S18929[13], 6}, {NULL}};
const Segment SGI254860[] = {{&S18929[35], 5}, {NULL}};
const Segment SGI254861[] = {{&S18929[75], 6}, {NULL}};
const Segment SGI254862[] = {{&S18929[129], 6}, {NULL}};
const Segment SGI254863[] = {{&S18929[143], 7}, {NULL}};
const Segment SGI254864[] = {{&S18930[13], 6}, {NULL}};
const Segment SGI254865[] = {{&S18930[35], 5}, {NULL}};
const Segment SGI254866[] = {{&S18930[75], 6}, {NULL}};
const Segment SGI254867[] = {{&S18930[129], 6}, {NULL}};
const Segment SGI254868[] = {{&S18930[143], 7}, {NULL}};
const Segment SGI254869[] = {{&S18931[13], 6}, {NULL}};
const Segment SGI254870[] = {{&S18931[35], 5}, {NULL}};
const Segment SGI254871[] = {{&S18931[75], 6}, {NULL}};
const Segment SGI254872[] = {{&S18931[129], 6}, {NULL}};
const Segment SGI254873[] = {{&S18931[143], 7}, {NULL}};
const Segment SGI254874[] = {{&S18932[13], 6}, {NULL}};
const Segment SGI254875[] = {{&S18932[35], 5}, {NULL}};
const Segment SGI254876[] = {{&S18932[75], 6}, {NULL}};
const Segment SGI254877[] = {{&S18932[129], 6}, {NULL}};
const Segment SGI254878[] = {{&S18932[143], 7}, {NULL}};
const Segment SGI254879[] = {{&S8536[0], 22}, {NULL}};
const Segment SGI254880[] = {{&S8536[0], 55}, {NULL}};
const Segment SGI254881[] = {{&S8537[0], 19}, {&S8537[20], 39}, {&S8537[51], 7}, {NULL}};
const Segment SGI254882[] = {{&S8537[0], 82}, {NULL}};
const Segment SGI254883[] = {{&S8539[45], 7}, {NULL}};
const Segment SGI254884[] = {{&S8539[0], 90}, {NULL}};
const Segment SGI254885[] = {{&S8541[9], 6}, {NULL}};
const Segment SGI254886[] = {{&S8541[0], 25}, {NULL}};
const Segment SGI254887[] = {{&S8542[25], 34}, {&S8542[60], 33}, {NULL}};
const Segment SGI254888[] = {{&S8542[0], 118}, {NULL}};
const Segment SGI254889[] = {{&S8544[0], 23}, {NULL}};
const Segment SGI254890[] = {{&S8545[9], 6}, {NULL}};
const Segment SGI254891[] = {{&S8545[0], 25}, {NULL}};
const Segment SGI254892[] = {{&S8546[0], 22}, {NULL}};
const Segment SGI254893[] = {{&S8547[0], 25}, {&S8547[26], 17}, {NULL}};
const Segment SGI254894[] = {{&S8547[0], 75}, {NULL}};
const Segment SGI254895[] = {{&S8549[0], 21}, {NULL}};
const Segment SGI254896[] = {{&S8551[0], 29}, {NULL}};
const Segment SGI254897[] = {{&S8552[23], 32}, {NULL}};
const Segment SGI254898[] = {{&S8552[0], 129}, {NULL}};
const Segment SGI254899[] = {{&S8553[40], 7}, {NULL}};
const Segment SGI254900[] = {{&S8553[0], 106}, {NULL}};
const Segment SGI254901[] = {{&S8554[23], 40}, {NULL}};
const Segment SGI254902[] = {{&S8554[0], 94}, {NULL}};
const Segment SGI254903[] = {{&S8555[0], 19}, {&S8555[20], 23}, {NULL}};
const Segment SGI254904[] = {{&S8555[0], 43}, {NULL}};
const Segment SGI254905[] = {{&S8556[0], 19}, {NULL}};
const Segment SGI254906[] = {{&S8557[20], 8}, {NULL}};
const Segment SGI254907[] = {{&S8557[0], 96}, {NULL}};
const Segment SGI254908[] = {{&S8558[0], 22}, {&S8558[64], 20}, {NULL}};
const Segment SGI254909[] = {{&S8558[0], 84}, {NULL}};
const Segment SGI254910[] = {{&S8559[0], 24}, {NULL}};
const Segment SGI254911[] = {{&S8559[0], 61}, {NULL}};
const Segment SGI254912[] = {{&S8560[0], 18}, {&S8560[60], 20}, {NULL}};
const Segment SGI254913[] = {{&S8560[0], 80}, {NULL}};
const Segment SGI254914[] = {{&S8561[9], 6}, {NULL}};
const Segment SGI254915[] = {{&S8561[0], 25}, {NULL}};
const Segment SGI254916[] = {{&S8562[28], 7}, {NULL}};
const Segment SGI254917[] = {{&S8562[0], 138}, {NULL}};
const Segment SGI254918[] = {{&S8563[10], 7}, {NULL}};
const Segment SGI254919[] = {{&S8563[0], 39}, {NULL}};
const Segment SGI254920[] = {{&S8564[7], 6}, {NULL}};
const Segment SGI254921[] = {{&S8564[0], 23}, {NULL}};
const Segment SGI254922[] = {{&S8565[28], 35}, {NULL}};
const Segment SGI254923[] = {{&S8565[0], 96}, {NULL}};
const Segment SGI254924[] = {{&S8566[30], 7}, {NULL}};
const Segment SGI254925[] = {{&S8566[0], 75}, {NULL}};
const Segment SGI254926[] = {{&S8568[3], 6}, {NULL}};
const Segment SGI254927[] = {{&S8568[0], 19}, {NULL}};
const Segment SGI254928[] = {{&S8569[9], 6}, {NULL}};
const Segment SGI254929[] = {{&S8569[0], 25}, {NULL}};
const Segment SGI254930[] = {{&S8570[37], 7}, {NULL}};
const Segment SGI254931[] = {{&S8570[0], 56}, {NULL}};
const Segment SGI254932[] = {{&S8571[0], 7}, {NULL}};
const Segment SGI254933[] = {{&S8571[0], 17}, {NULL}};
const Segment SGI254934[] = {{&S8572[0], 20}, {&S8572[73], 12}, {NULL}};
const Segment SGI254935[] = {{&S8572[0], 119}, {NULL}};
const Segment SGI254936[] = {{&S8574[35], 7}, {NULL}};
const Segment SGI254937[] = {{&S8574[0], 134}, {NULL}};
const Segment SGI254938[] = {{&S8576[9], 6}, {NULL}};
const Segment SGI254939[] = {{&S8576[0], 27}, {NULL}};
const Segment SGI254940[] = {{&S8577[0], 6}, {&S8577[19], 8}, {NULL}};
const Segment SGI254941[] = {{&S8577[0], 47}, {NULL}};
const Segment SGI254942[] = {{&S8578[0], 22}, {NULL}};
const Segment SGI254943[] = {{&S8579[0], 22}, {&S8579[75], 12}, {NULL}};
const Segment SGI254944[] = {{&S8579[0], 121}, {NULL}};
const Segment SGI254945[] = {{&S8580[23], 7}, {NULL}};
const Segment SGI254946[] = {{&S8580[0], 31}, {NULL}};
const Segment SGI254947[] = {{&S8581[0], 18}, {NULL}};
const Segment SGI254948[] = {{&S8582[12], 7}, {NULL}};
const Segment SGI254949[] = {{&S8582[0], 31}, {NULL}};
const Segment SGI254950[] = {{&S8583[17], 40}, {NULL}};
const Segment SGI254951[] = {{&S8583[0], 88}, {NULL}};
const Segment SGI254952[] = {{&S8584[0], 6}, {NULL}};
const Segment SGI254953[] = {{&S8584[0], 19}, {NULL}};
const Segment SGI254954[] = {{&S8585[0], 19}, {&S8585[20], 11}, {&S8585[48], 17}, {NULL}};
const Segment SGI254955[] = {{&S8585[0], 67}, {NULL}};
const Segment SGI254956[] = {{&S8587[14], 6}, {NULL}};
const Segment SGI254957[] = {{&S8587[0], 30}, {NULL}};
const Segment SGI254958[] = {{&S8588[0], 33}, {NULL}};
const Segment SGI254959[] = {{&S8589[9], 6}, {NULL}};
const Segment SGI254960[] = {{&S8589[0], 27}, {NULL}};
const Segment SGI254961[] = {{&S8590[0], 29}, {NULL}};
const Segment SGI254962[] = {{&S8592[70], 7}, {NULL}};
const Segment SGI254963[] = {{&S8592[0], 98}, {NULL}};
const Segment SGI254964[] = {{&S8593[0], 6}, {NULL}};
const Segment SGI254965[] = {{&S8593[0], 16}, {NULL}};
const Segment SGI254966[] = {{&S8594[0], 24}, {&S8594[47], 16}, {NULL}};
const Segment SGI254967[] = {{&S8594[0], 73}, {NULL}};
const Segment SGI254968[] = {{&S8595[8], 5}, {NULL}};
const Segment SGI254969[] = {{&S8595[0], 28}, {NULL}};
const Segment SGI254970[] = {{&S8596[15], 6}, {NULL}};
const Segment SGI254971[] = {{&S8596[0], 31}, {NULL}};
const Segment SGI254972[] = {{&S8597[0], 20}, {&S8597[73], 12}, {NULL}};
const Segment SGI254973[] = {{&S8597[0], 119}, {NULL}};
const Segment SGI254974[] = {{&S8598[0], 30}, {NULL}};
const Segment SGI254975[] = {{&S8599[10], 7}, {&S8599[10], 7}, {NULL}};
const Segment SGI254976[] = {{&S8599[0], 85}, {NULL}};
const Segment SGI254977[] = {{&S8601[0], 21}, {&S8601[22], 39}, {&S8601[53], 7}, {NULL}};
const Segment SGI254978[] = {{&S8601[0], 84}, {NULL}};
const Segment SGI254979[] = {{&S8602[9], 6}, {NULL}};
const Segment SGI254980[] = {{&S8602[0], 25}, {NULL}};
const Segment SGI254981[] = {{&S8603[0], 20}, {&S8603[76], 12}, {NULL}};
const Segment SGI254982[] = {{&S8603[0], 122}, {NULL}};
const Segment SGI254983[] = {{&S8604[9], 6}, {NULL}};
const Segment SGI254984[] = {{&S8604[0], 25}, {NULL}};
const Segment SGI254985[] = {{&S8605[44], 7}, {NULL}};
const Segment SGI254986[] = {{&S8605[0], 114}, {NULL}};
const Segment SGI254987[] = {{&S8606[38], 3}, {&S8606[59], 11}, {NULL}};
const Segment SGI254988[] = {{&S8606[0], 72}, {NULL}};
const Segment SGI254989[] = {{&S8608[8], 3}, {NULL}};
const Segment SGI254990[] = {{&S8608[0], 21}, {NULL}};
const Segment SGI254991[] = {{&S8609[25], 15}, {NULL}};
const Segment SGI254992[] = {{&S8609[0], 76}, {NULL}};
const Segment SGI254993[] = {{&S8611[39], 38}, {NULL}};
const Segment SGI254994[] = {{&S8611[0], 104}, {NULL}};
const Segment SGI254995[] = {{&S8612[6], 6}, {NULL}};
const Segment SGI254996[] = {{&S8612[0], 22}, {NULL}};
const Segment SGI254997[] = {{&S8613[5], 6}, {NULL}};
const Segment SGI254998[] = {{&S8613[0], 21}, {NULL}};
const Segment SGI254999[] = {{&S8614[6], 6}, {NULL}};
const Segment SGI255000[] = {{&S8614[0], 22}, {NULL}};
const Segment SGI255001[] = {{&S8615[0], 20}, {&S8615[76], 12}, {NULL}};
const Segment SGI255002[] = {{&S8615[0], 122}, {NULL}};
const Segment SGI255003[] = {{&S8616[0], 24}, {NULL}};
const Segment SGI255004[] = {{&S8616[0], 65}, {NULL}};
const Segment SGI255005[] = {{&S8617[8], 5}, {NULL}};
const Segment SGI255006[] = {{&S8617[0], 28}, {NULL}};
const Segment SGI255007[] = {{&S8618[0], 6}, {NULL}};
const Segment SGI255008[] = {{&S8618[0], 16}, {NULL}};
const Segment SGI255009[] = {{&S8619[6], 6}, {NULL}};
const Segment SGI255010[] = {{&S8619[0], 22}, {NULL}};
const Segment SGI255011[] = {{&S8620[12], 6}, {NULL}};
const Segment SGI255012[] = {{&S8620[0], 28}, {NULL}};
const Segment SGI255013[] = {{&S8621[25], 14}, {&S8621[40], 42}, {NULL}};
const Segment SGI255014[] = {{&S8621[0], 82}, {NULL}};
const Segment SGI255015[] = {{&S8622[0], 7}, {NULL}};
const Segment SGI255016[] = {{&S8622[0], 17}, {NULL}};
const Segment SGI255017[] = {{&S8623[0], 22}, {NULL}};
const Segment SGI255018[] = {{&S8624[11], 6}, {NULL}};
const Segment SGI255019[] = {{&S8624[0], 27}, {NULL}};
const Segment SGI255020[] = {{&S8625[40], 7}, {NULL}};
const Segment SGI255021[] = {{&S8625[0], 148}, {NULL}};
const Segment SGI255022[] = {{&S8626[9], 6}, {NULL}};
const Segment SGI255023[] = {{&S8626[0], 34}, {NULL}};
const Segment SGI255024[] = {{&S8627[0], 19}, {&S8627[32], 41}, {NULL}};
const Segment SGI255025[] = {{&S8627[0], 106}, {NULL}};
const Segment SGI255026[] = {{&S8628[0], 22}, {NULL}};
const Segment SGI255027[] = {{&S8629[3], 7}, {NULL}};
const Segment SGI255028[] = {{&S8629[0], 20}, {NULL}};
const Segment SGI255029[] = {{&S8630[8], 5}, {NULL}};
const Segment SGI255030[] = {{&S8630[0], 88}, {NULL}};
const Segment SGI255031[] = {{&S8631[0], 19}, {NULL}};
const Segment SGI255032[] = {{&S8632[23], 40}, {NULL}};
const Segment SGI255033[] = {{&S8632[0], 94}, {NULL}};
const Segment SGI255034[] = {{&S8633[0], 111}, {NULL}};
const Segment SGI255035[] = {{&S8634[0], 23}, {NULL}};
const Segment SGI255036[] = {{&S8635[0], 18}, {NULL}};
const Segment SGI255037[] = {{&S8636[47], 7}, {NULL}};
const Segment SGI255038[] = {{&S8636[0], 155}, {NULL}};
const Segment SGI255039[] = {{&S8638[31], 6}, {NULL}};
const Segment SGI255040[] = {{&S8638[0], 64}, {NULL}};
const Segment SGI255041[] = {{&S8639[9], 6}, {NULL}};
const Segment SGI255042[] = {{&S8639[0], 34}, {NULL}};
const Segment SGI255043[] = {{&S8641[0], 24}, {&S8641[42], 7}, {NULL}};
const Segment SGI255044[] = {{&S8641[0], 50}, {NULL}};
const Segment SGI255045[] = {{&S8642[0], 18}, {NULL}};
const Segment SGI255046[] = {{&S8642[0], 74}, {NULL}};
const Segment SGI255047[] = {{&S8643[30], 6}, {NULL}};
const Segment SGI255048[] = {{&S8643[0], 46}, {NULL}};
const Segment SGI255049[] = {{&S8644[0], 21}, {NULL}};
const Segment SGI255050[] = {{&S8645[0], 19}, {&S8645[25], 7}, {&S8645[51], 30}, {NULL}};
const Segment SGI255051[] = {{&S8645[0], 105}, {NULL}};
const Segment SGI255052[] = {{&S8646[92], 7}, {NULL}};
const Segment SGI255053[] = {{&S8646[0], 265}, {NULL}};
const Segment SGI255054[] = {{&S8647[92], 7}, {NULL}};
const Segment SGI255055[] = {{&S8647[0], 266}, {NULL}};
const Segment SGI255056[] = {{&S8648[4], 7}, {NULL}};
const Segment SGI255057[] = {{&S8648[0], 21}, {NULL}};
const Segment SGI255058[] = {{&S8649[0], 24}, {NULL}};
const Segment SGI255059[] = {{&S19627[13], 6}, {NULL}};
const Segment SGI255060[] = {{&S19627[35], 5}, {NULL}};
const Segment SGI255061[] = {{&S19627[68], 6}, {NULL}};
const Segment SGI255062[] = {{&S19627[121], 6}, {NULL}};
const Segment SGI255063[] = {{&S19627[135], 7}, {NULL}};
const Segment SGI255064[] = {{&S19628[13], 6}, {NULL}};
const Segment SGI255065[] = {{&S19628[35], 5}, {NULL}};
const Segment SGI255066[] = {{&S19628[68], 6}, {NULL}};
const Segment SGI255067[] = {{&S19628[121], 6}, {NULL}};
const Segment SGI255068[] = {{&S19628[135], 7}, {NULL}};
const Segment SGI255069[] = {{&S19629[13], 6}, {NULL}};
const Segment SGI255070[] = {{&S19629[35], 3}, {NULL}};
const Segment SGI255071[] = {{&S19629[66], 6}, {NULL}};
const Segment SGI255072[] = {{&S19629[119], 6}, {NULL}};
const Segment SGI255073[] = {{&S19629[133], 7}, {NULL}};
const Segment SGI255074[] = {{&S19630[13], 6}, {NULL}};
const Segment SGI255075[] = {{&S19630[35], 5}, {NULL}};
const Segment SGI255076[] = {{&S19630[68], 6}, {NULL}};
const Segment SGI255077[] = {{&S19630[121], 6}, {NULL}};
const Segment SGI255078[] = {{&S19630[135], 7}, {NULL}};
const Segment SGI255079[] = {{&S19631[13], 6}, {NULL}};
const Segment SGI255080[] = {{&S19631[35], 3}, {NULL}};
const Segment SGI255081[] = {{&S19631[66], 6}, {NULL}};
const Segment SGI255082[] = {{&S19631[119], 6}, {NULL}};
const Segment SGI255083[] = {{&S19631[133], 7}, {NULL}};
const Segment SGI255084[] = {{&S19632[13], 6}, {NULL}};
const Segment SGI255085[] = {{&S19632[35], 3}, {NULL}};
const Segment SGI255086[] = {{&S19632[66], 6}, {NULL}};
const Segment SGI255087[] = {{&S19632[119], 6}, {NULL}};
const Segment SGI255088[] = {{&S19632[133], 7}, {NULL}};
const Segment SGI255089[] = {{&S19633[13], 6}, {NULL}};
const Segment SGI255090[] = {{&S19633[35], 3}, {NULL}};
const Segment SGI255091[] = {{&S19633[66], 6}, {NULL}};
const Segment SGI255092[] = {{&S19633[119], 6}, {NULL}};
const Segment SGI255093[] = {{&S19633[133], 7}, {NULL}};
const Segment SGI255094[] = {{&S19634[13], 6}, {NULL}};
const Segment SGI255095[] = {{&S19634[35], 5}, {NULL}};
const Segment SGI255096[] = {{&S19634[68], 6}, {NULL}};
const Segment SGI255097[] = {{&S19634[121], 6}, {NULL}};
const Segment SGI255098[] = {{&S19634[135], 7}, {NULL}};
const Segment SGI255099[] = {{&S20135[13], 6}, {NULL}};
const Segment SGI255100[] = {{&S20135[35], 5}, {NULL}};
const Segment SGI255101[] = {{&S20135[68], 6}, {NULL}};
const Segment SGI255102[] = {{&S20135[121], 6}, {NULL}};
const Segment SGI255103[] = {{&S20135[135], 7}, {NULL}};
const Segment SGI255104[] = {{&S20136[13], 6}, {NULL}};
const Segment SGI255105[] = {{&S20136[35], 3}, {NULL}};
const Segment SGI255106[] = {{&S20136[66], 6}, {NULL}};
const Segment SGI255107[] = {{&S20136[119], 6}, {NULL}};
const Segment SGI255108[] = {{&S20136[133], 7}, {NULL}};
const Segment SGI255109[] = {{&S20137[13], 6}, {NULL}};
const Segment SGI255110[] = {{&S20137[35], 5}, {NULL}};
const Segment SGI255111[] = {{&S20137[68], 6}, {NULL}};
const Segment SGI255112[] = {{&S20137[121], 6}, {NULL}};
const Segment SGI255113[] = {{&S20137[135], 7}, {NULL}};
const Segment SGI255114[] = {{&S20138[13], 6}, {NULL}};
const Segment SGI255115[] = {{&S20138[35], 5}, {NULL}};
const Segment SGI255116[] = {{&S20138[68], 6}, {NULL}};
const Segment SGI255117[] = {{&S20138[121], 6}, {NULL}};
const Segment SGI255118[] = {{&S20138[135], 7}, {NULL}};
const Segment SGI255119[] = {{&S20139[13], 6}, {NULL}};
const Segment SGI255120[] = {{&S20139[35], 5}, {NULL}};
const Segment SGI255121[] = {{&S20139[68], 6}, {NULL}};
const Segment SGI255122[] = {{&S20139[121], 6}, {NULL}};
const Segment SGI255123[] = {{&S20139[135], 7}, {NULL}};
const Segment SGI255124[] = {{&S20140[13], 6}, {NULL}};
const Segment SGI255125[] = {{&S20140[35], 3}, {NULL}};
const Segment SGI255126[] = {{&S20140[66], 6}, {NULL}};
const Segment SGI255127[] = {{&S20140[119], 6}, {NULL}};
const Segment SGI255128[] = {{&S20140[133], 7}, {NULL}};
const Segment SGI255129[] = {{&S20141[13], 6}, {NULL}};
const Segment SGI255130[] = {{&S20141[35], 3}, {NULL}};
const Segment SGI255131[] = {{&S20141[66], 6}, {NULL}};
const Segment SGI255132[] = {{&S20141[119], 6}, {NULL}};
const Segment SGI255133[] = {{&S20141[133], 7}, {NULL}};
const Segment SGI255134[] = {{&S20142[13], 6}, {NULL}};
const Segment SGI255135[] = {{&S20142[35], 3}, {NULL}};
const Segment SGI255136[] = {{&S20142[66], 6}, {NULL}};
const Segment SGI255137[] = {{&S20142[119], 6}, {NULL}};
const Segment SGI255138[] = {{&S20142[133], 7}, {NULL}};
const Segment SGI255139[] = {{&S20143[13], 6}, {NULL}};
const Segment SGI255140[] = {{&S20143[35], 5}, {NULL}};
const Segment SGI255141[] = {{&S20143[68], 6}, {NULL}};
const Segment SGI255142[] = {{&S20143[121], 5}, {NULL}};
const Segment SGI255143[] = {{&S20143[134], 7}, {NULL}};
const Segment SGI255144[] = {{&S20663[13], 4}, {NULL}};
const Segment SGI255145[] = {{&S20663[26], 3}, {NULL}};
const Segment SGI255146[] = {{&S20663[64], 8}, {NULL}};
const Segment SGI255147[] = {{&S20663[119], 6}, {NULL}};
const Segment SGI255148[] = {{&S20663[133], 9}, {NULL}};
const Segment SGI255149[] = {{&S20664[13], 4}, {NULL}};
const Segment SGI255150[] = {{&S20664[26], 3}, {NULL}};
const Segment SGI255151[] = {{&S20664[64], 8}, {NULL}};
const Segment SGI255152[] = {{&S20664[119], 6}, {NULL}};
const Segment SGI255153[] = {{&S20664[133], 9}, {NULL}};
const Segment SGI255154[] = {{&S20665[13], 4}, {NULL}};
const Segment SGI255155[] = {{&S20665[26], 3}, {NULL}};
const Segment SGI255156[] = {{&S20665[64], 8}, {NULL}};
const Segment SGI255157[] = {{&S20665[119], 6}, {NULL}};
const Segment SGI255158[] = {{&S20665[133], 9}, {NULL}};
const Segment SGI255159[] = {{&S20666[13], 4}, {NULL}};
const Segment SGI255160[] = {{&S20666[26], 5}, {NULL}};
const Segment SGI255161[] = {{&S20666[66], 8}, {NULL}};
const Segment SGI255162[] = {{&S20666[121], 6}, {NULL}};
const Segment SGI255163[] = {{&S20666[135], 9}, {NULL}};
const Segment SGI255164[] = {{&S20667[13], 4}, {NULL}};
const Segment SGI255165[] = {{&S20667[26], 5}, {NULL}};
const Segment SGI255166[] = {{&S20667[66], 8}, {NULL}};
const Segment SGI255167[] = {{&S20667[121], 7}, {NULL}};
const Segment SGI255168[] = {{&S20667[136], 9}, {NULL}};
const Segment SGI255169[] = {{&S20668[13], 4}, {NULL}};
const Segment SGI255170[] = {{&S20668[26], 5}, {NULL}};
const Segment SGI255171[] = {{&S20668[66], 8}, {NULL}};
const Segment SGI255172[] = {{&S20668[121], 6}, {NULL}};
const Segment SGI255173[] = {{&S20668[135], 9}, {NULL}};
const Segment SGI255174[] = {{&S20669[13], 4}, {NULL}};
const Segment SGI255175[] = {{&S20669[26], 5}, {NULL}};
const Segment SGI255176[] = {{&S20669[66], 8}, {NULL}};
const Segment SGI255177[] = {{&S20669[121], 7}, {NULL}};
const Segment SGI255178[] = {{&S20669[136], 9}, {NULL}};
const Segment SGI255179[] = {{&S21415[13], 6}, {NULL}};
const Segment SGI255180[] = {{&S21415[103], 3}, {NULL}};
const Segment SGI255181[] = {{&S21415[35], 3}, {NULL}};
const Segment SGI255182[] = {{&S21415[66], 8}, {NULL}};
const Segment SGI255183[] = {{&S21415[114], 6}, {NULL}};
const Segment SGI255184[] = {{&S21415[128], 7}, {NULL}};
const Segment SGI255185[] = {{&S21416[13], 6}, {NULL}};
const Segment SGI255186[] = {{&S21416[103], 3}, {NULL}};
const Segment SGI255187[] = {{&S21416[35], 3}, {NULL}};
const Segment SGI255188[] = {{&S21416[66], 8}, {NULL}};
const Segment SGI255189[] = {{&S21416[114], 6}, {NULL}};
const Segment SGI255190[] = {{&S21416[128], 7}, {NULL}};
const Segment SGI255191[] = {{&S21417[13], 6}, {NULL}};
const Segment SGI255192[] = {{&S21417[103], 3}, {NULL}};
const Segment SGI255193[] = {{&S21417[35], 3}, {NULL}};
const Segment SGI255194[] = {{&S21417[66], 8}, {NULL}};
const Segment SGI255195[] = {{&S21417[114], 7}, {NULL}};
const Segment SGI255196[] = {{&S21417[129], 7}, {NULL}};
const Segment SGI255197[] = {{&S21418[13], 6}, {NULL}};
const Segment SGI255198[] = {{&S21418[103], 3}, {NULL}};
const Segment SGI255199[] = {{&S21418[35], 3}, {NULL}};
const Segment SGI255200[] = {{&S21418[66], 8}, {NULL}};
const Segment SGI255201[] = {{&S21418[114], 6}, {NULL}};
const Segment SGI255202[] = {{&S21418[128], 7}, {NULL}};
const Segment SGI255203[] = {{&S21419[13], 6}, {NULL}};
const Segment SGI255204[] = {{&S21419[103], 3}, {NULL}};
const Segment SGI255205[] = {{&S21419[35], 3}, {NULL}};
const Segment SGI255206[] = {{&S21419[66], 8}, {NULL}};
const Segment SGI255207[] = {{&S21419[114], 6}, {NULL}};
const Segment SGI255208[] = {{&S21419[128], 7}, {NULL}};
const Segment SGI255209[] = {{&S21420[13], 6}, {NULL}};
const Segment SGI255210[] = {{&S21420[103], 3}, {NULL}};
const Segment SGI255211[] = {{&S21420[35], 3}, {NULL}};
const Segment SGI255212[] = {{&S21420[66], 8}, {NULL}};
const Segment SGI255213[] = {{&S21420[114], 8}, {NULL}};
const Segment SGI255214[] = {{&S21420[130], 7}, {NULL}};
const Segment SGI255215[] = {{&S21421[13], 6}, {NULL}};
const Segment SGI255216[] = {{&S21421[103], 3}, {NULL}};
const Segment SGI255217[] = {{&S21421[35], 3}, {NULL}};
const Segment SGI255218[] = {{&S21421[66], 8}, {NULL}};
const Segment SGI255219[] = {{&S21421[114], 8}, {NULL}};
const Segment SGI255220[] = {{&S21421[130], 7}, {NULL}};
const Segment SGI255221[] = {{&S21422[13], 6}, {NULL}};
const Segment SGI255222[] = {{&S21422[103], 3}, {NULL}};
const Segment SGI255223[] = {{&S21422[35], 3}, {NULL}};
const Segment SGI255224[] = {{&S21422[66], 8}, {NULL}};
const Segment SGI255225[] = {{&S21422[114], 6}, {NULL}};
const Segment SGI255226[] = {{&S21422[128], 7}, {NULL}};
const Segment SGI255227[] = {{&S21423[13], 6}, {NULL}};
const Segment SGI255228[] = {{&S21423[103], 3}, {NULL}};
const Segment SGI255229[] = {{&S21423[35], 3}, {NULL}};
const Segment SGI255230[] = {{&S21423[66], 8}, {NULL}};
const Segment SGI255231[] = {{&S21423[114], 6}, {NULL}};
const Segment SGI255232[] = {{&S21423[128], 7}, {NULL}};
const Segment SGI255233[] = {{&S21424[13], 6}, {NULL}};
const Segment SGI255234[] = {{&S21424[103], 3}, {NULL}};
const Segment SGI255235[] = {{&S21424[35], 3}, {NULL}};
const Segment SGI255236[] = {{&S21424[66], 8}, {NULL}};
const Segment SGI255237[] = {{&S21424[114], 6}, {NULL}};
const Segment SGI255238[] = {{&S21424[128], 7}, {NULL}};
const Segment SGI255239[] = {{&S21425[13], 6}, {NULL}};
const Segment SGI255240[] = {{&S21425[103], 3}, {NULL}};
const Segment SGI255241[] = {{&S21425[35], 3}, {NULL}};
const Segment SGI255242[] = {{&S21425[66], 8}, {NULL}};
const Segment SGI255243[] = {{&S21425[114], 6}, {NULL}};
const Segment SGI255244[] = {{&S21425[128], 7}, {NULL}};
const Segment SGI255245[] = {{&S21426[13], 6}, {NULL}};
const Segment SGI255246[] = {{&S21426[103], 3}, {NULL}};
const Segment SGI255247[] = {{&S21426[35], 3}, {NULL}};
const Segment SGI255248[] = {{&S21426[66], 8}, {NULL}};
const Segment SGI255249[] = {{&S21426[114], 6}, {NULL}};
const Segment SGI255250[] = {{&S21426[128], 7}, {NULL}};
const Segment SGI255251[] = {{&S21427[13], 6}, {NULL}};
const Segment SGI255252[] = {{&S21427[103], 3}, {NULL}};
const Segment SGI255253[] = {{&S21427[35], 3}, {NULL}};
const Segment SGI255254[] = {{&S21427[66], 8}, {NULL}};
const Segment SGI255255[] = {{&S21427[114], 6}, {NULL}};
const Segment SGI255256[] = {{&S21427[128], 7}, {NULL}};
const Segment SGI255257[] = {{&S21428[13], 6}, {NULL}};
const Segment SGI255258[] = {{&S21428[103], 3}, {NULL}};
const Segment SGI255259[] = {{&S21428[35], 3}, {NULL}};
const Segment SGI255260[] = {{&S21428[66], 8}, {NULL}};
const Segment SGI255261[] = {{&S21428[114], 6}, {NULL}};
const Segment SGI255262[] = {{&S21428[128], 7}, {NULL}};
const Segment SGI255263[] = {{&S21429[13], 6}, {NULL}};
const Segment SGI255264[] = {{&S21429[103], 3}, {NULL}};
const Segment SGI255265[] = {{&S21429[35], 3}, {NULL}};
const Segment SGI255266[] = {{&S21429[66], 8}, {NULL}};
const Segment SGI255267[] = {{&S21429[114], 6}, {NULL}};
const Segment SGI255268[] = {{&S21429[128], 7}, {NULL}};
const Segment SGI255269[] = {{&S21430[13], 6}, {NULL}};
const Segment SGI255270[] = {{&S21430[103], 3}, {NULL}};
const Segment SGI255271[] = {{&S21430[35], 3}, {NULL}};
const Segment SGI255272[] = {{&S21430[66], 8}, {NULL}};
const Segment SGI255273[] = {{&S21430[114], 6}, {NULL}};
const Segment SGI255274[] = {{&S21430[128], 7}, {NULL}};
const Segment SGI255275[] = {{&S21431[13], 6}, {NULL}};
const Segment SGI255276[] = {{&S21431[103], 3}, {NULL}};
const Segment SGI255277[] = {{&S21431[35], 3}, {NULL}};
const Segment SGI255278[] = {{&S21431[66], 8}, {NULL}};
const Segment SGI255279[] = {{&S21431[114], 6}, {NULL}};
const Segment SGI255280[] = {{&S21431[128], 7}, {NULL}};
const Segment SGI255281[] = {{&S21432[13], 6}, {NULL}};
const Segment SGI255282[] = {{&S21432[103], 3}, {NULL}};
const Segment SGI255283[] = {{&S21432[35], 3}, {NULL}};
const Segment SGI255284[] = {{&S21432[66], 8}, {NULL}};
const Segment SGI255285[] = {{&S21432[114], 6}, {NULL}};
const Segment SGI255286[] = {{&S21432[128], 7}, {NULL}};
const Segment SGI255287[] = {{&S21433[13], 6}, {NULL}};
const Segment SGI255288[] = {{&S21433[103], 3}, {NULL}};
const Segment SGI255289[] = {{&S21433[35], 3}, {NULL}};
const Segment SGI255290[] = {{&S21433[66], 8}, {NULL}};
const Segment SGI255291[] = {{&S21433[114], 6}, {NULL}};
const Segment SGI255292[] = {{&S21433[128], 7}, {NULL}};
const Segment SGI255293[] = {{&S21434[13], 6}, {NULL}};
const Segment SGI255294[] = {{&S21434[103], 3}, {NULL}};
const Segment SGI255295[] = {{&S21434[35], 3}, {NULL}};
const Segment SGI255296[] = {{&S21434[66], 8}, {NULL}};
const Segment SGI255297[] = {{&S21434[114], 6}, {NULL}};
const Segment SGI255298[] = {{&S21434[128], 7}, {NULL}};
const Segment SGI255299[] = {{&S21435[13], 6}, {NULL}};
const Segment SGI255300[] = {{&S21435[105], 3}, {NULL}};
const Segment SGI255301[] = {{&S21435[35], 5}, {NULL}};
const Segment SGI255302[] = {{&S21435[68], 8}, {NULL}};
const Segment SGI255303[] = {{&S21435[116], 6}, {NULL}};
const Segment SGI255304[] = {{&S21435[130], 7}, {NULL}};
const Segment SGI255305[] = {{&S21436[13], 6}, {NULL}};
const Segment SGI255306[] = {{&S21436[105], 3}, {NULL}};
const Segment SGI255307[] = {{&S21436[35], 5}, {NULL}};
const Segment SGI255308[] = {{&S21436[68], 8}, {NULL}};
const Segment SGI255309[] = {{&S21436[116], 6}, {NULL}};
const Segment SGI255310[] = {{&S21436[130], 7}, {NULL}};
const Segment SGI255311[] = {{&S21437[13], 6}, {NULL}};
const Segment SGI255312[] = {{&S21437[103], 3}, {NULL}};
const Segment SGI255313[] = {{&S21437[35], 3}, {NULL}};
const Segment SGI255314[] = {{&S21437[66], 8}, {NULL}};
const Segment SGI255315[] = {{&S21437[114], 6}, {NULL}};
const Segment SGI255316[] = {{&S21437[128], 7}, {NULL}};
const Segment SGI255317[] = {{&S21438[13], 6}, {NULL}};
const Segment SGI255318[] = {{&S21438[105], 3}, {NULL}};
const Segment SGI255319[] = {{&S21438[35], 5}, {NULL}};
const Segment SGI255320[] = {{&S21438[68], 8}, {NULL}};
const Segment SGI255321[] = {{&S21438[116], 6}, {NULL}};
const Segment SGI255322[] = {{&S21438[130], 7}, {NULL}};
const Segment SGI255323[] = {{&S21439[13], 6}, {NULL}};
const Segment SGI255324[] = {{&S21439[105], 3}, {NULL}};
const Segment SGI255325[] = {{&S21439[35], 5}, {NULL}};
const Segment SGI255326[] = {{&S21439[68], 8}, {NULL}};
const Segment SGI255327[] = {{&S21439[116], 6}, {NULL}};
const Segment SGI255328[] = {{&S21439[130], 7}, {NULL}};
const Segment SGI255329[] = {{&S21440[13], 6}, {NULL}};
const Segment SGI255330[] = {{&S21440[105], 3}, {NULL}};
const Segment SGI255331[] = {{&S21440[35], 5}, {NULL}};
const Segment SGI255332[] = {{&S21440[68], 8}, {NULL}};
const Segment SGI255333[] = {{&S21440[116], 6}, {NULL}};
const Segment SGI255334[] = {{&S21440[130], 7}, {NULL}};
const Segment SGI255335[] = {{&S21441[13], 6}, {NULL}};
const Segment SGI255336[] = {{&S21441[105], 3}, {NULL}};
const Segment SGI255337[] = {{&S21441[35], 5}, {NULL}};
const Segment SGI255338[] = {{&S21441[68], 8}, {NULL}};
const Segment SGI255339[] = {{&S21441[116], 6}, {NULL}};
const Segment SGI255340[] = {{&S21441[130], 7}, {NULL}};
const Segment SGI255341[] = {{&S21442[13], 6}, {NULL}};
const Segment SGI255342[] = {{&S21442[103], 3}, {NULL}};
const Segment SGI255343[] = {{&S21442[35], 3}, {NULL}};
const Segment SGI255344[] = {{&S21442[66], 8}, {NULL}};
const Segment SGI255345[] = {{&S21442[114], 8}, {NULL}};
const Segment SGI255346[] = {{&S21442[130], 7}, {NULL}};
const Segment SGI255347[] = {{&S21443[13], 6}, {NULL}};
const Segment SGI255348[] = {{&S21443[105], 3}, {NULL}};
const Segment SGI255349[] = {{&S21443[35], 5}, {NULL}};
const Segment SGI255350[] = {{&S21443[68], 8}, {NULL}};
const Segment SGI255351[] = {{&S21443[116], 7}, {NULL}};
const Segment SGI255352[] = {{&S21443[131], 7}, {NULL}};
const Segment SGI255353[] = {{&S21444[13], 6}, {NULL}};
const Segment SGI255354[] = {{&S21444[103], 3}, {NULL}};
const Segment SGI255355[] = {{&S21444[35], 3}, {NULL}};
const Segment SGI255356[] = {{&S21444[66], 8}, {NULL}};
const Segment SGI255357[] = {{&S21444[114], 6}, {NULL}};
const Segment SGI255358[] = {{&S21444[128], 7}, {NULL}};
const Segment SGI255359[] = {{&S21445[13], 6}, {NULL}};
const Segment SGI255360[] = {{&S21445[105], 3}, {NULL}};
const Segment SGI255361[] = {{&S21445[35], 5}, {NULL}};
const Segment SGI255362[] = {{&S21445[68], 8}, {NULL}};
const Segment SGI255363[] = {{&S21445[116], 6}, {NULL}};
const Segment SGI255364[] = {{&S21445[130], 7}, {NULL}};
const Segment SGI255365[] = {{&S21446[13], 6}, {NULL}};
const Segment SGI255366[] = {{&S21446[103], 3}, {NULL}};
const Segment SGI255367[] = {{&S21446[35], 3}, {NULL}};
const Segment SGI255368[] = {{&S21446[66], 8}, {NULL}};
const Segment SGI255369[] = {{&S21446[114], 6}, {NULL}};
const Segment SGI255370[] = {{&S21446[128], 7}, {NULL}};
const Segment SGI255371[] = {{&S21447[13], 6}, {NULL}};
const Segment SGI255372[] = {{&S21447[105], 3}, {NULL}};
const Segment SGI255373[] = {{&S21447[35], 5}, {NULL}};
const Segment SGI255374[] = {{&S21447[68], 8}, {NULL}};
const Segment SGI255375[] = {{&S21447[116], 6}, {NULL}};
const Segment SGI255376[] = {{&S21447[130], 7}, {NULL}};
const Segment SGI255377[] = {{&S21448[13], 6}, {NULL}};
const Segment SGI255378[] = {{&S21448[102], 3}, {NULL}};
const Segment SGI255379[] = {{&S21448[65], 8}, {NULL}};
const Segment SGI255380[] = {{&S21448[113], 6}, {NULL}};
const Segment SGI255381[] = {{&S21448[127], 7}, {NULL}};
const Segment SGI255382[] = {{&S21449[13], 6}, {NULL}};
const Segment SGI255383[] = {{&S21449[103], 3}, {NULL}};
const Segment SGI255384[] = {{&S21449[35], 3}, {NULL}};
const Segment SGI255385[] = {{&S21449[66], 8}, {NULL}};
const Segment SGI255386[] = {{&S21449[114], 12}, {NULL}};
const Segment SGI255387[] = {{&S21449[127], 7}, {NULL}};
const Segment SGI255388[] = {{&S21450[13], 6}, {NULL}};
const Segment SGI255389[] = {{&S21450[105], 3}, {NULL}};
const Segment SGI255390[] = {{&S21450[35], 5}, {NULL}};
const Segment SGI255391[] = {{&S21450[68], 8}, {NULL}};
const Segment SGI255392[] = {{&S21450[116], 6}, {NULL}};
const Segment SGI255393[] = {{&S21450[130], 7}, {NULL}};
const Segment SGI255394[] = {{&S21451[13], 6}, {NULL}};
const Segment SGI255395[] = {{&S21451[105], 3}, {NULL}};
const Segment SGI255396[] = {{&S21451[35], 5}, {NULL}};
const Segment SGI255397[] = {{&S21451[68], 8}, {NULL}};
const Segment SGI255398[] = {{&S21451[116], 6}, {NULL}};
const Segment SGI255399[] = {{&S21451[130], 7}, {NULL}};
const Segment SGI255400[] = {{&S21452[13], 6}, {NULL}};
const Segment SGI255401[] = {{&S21452[105], 3}, {NULL}};
const Segment SGI255402[] = {{&S21452[35], 5}, {NULL}};
const Segment SGI255403[] = {{&S21452[68], 8}, {NULL}};
const Segment SGI255404[] = {{&S21452[116], 6}, {NULL}};
const Segment SGI255405[] = {{&S21452[130], 7}, {NULL}};
const Segment SGI255406[] = {{&S21453[13], 6}, {NULL}};
const Segment SGI255407[] = {{&S21453[105], 3}, {NULL}};
const Segment SGI255408[] = {{&S21453[35], 5}, {NULL}};
const Segment SGI255409[] = {{&S21453[68], 8}, {NULL}};
const Segment SGI255410[] = {{&S21453[116], 6}, {NULL}};
const Segment SGI255411[] = {{&S21453[130], 7}, {NULL}};
const Segment SGI255412[] = {{&S21454[13], 6}, {NULL}};
const Segment SGI255413[] = {{&S21454[105], 3}, {NULL}};
const Segment SGI255414[] = {{&S21454[35], 5}, {NULL}};
const Segment SGI255415[] = {{&S21454[68], 8}, {NULL}};
const Segment SGI255416[] = {{&S21454[116], 6}, {NULL}};
const Segment SGI255417[] = {{&S21454[130], 7}, {NULL}};
const Segment SGI255418[] = {{&S21455[13], 6}, {NULL}};
const Segment SGI255419[] = {{&S21455[105], 3}, {NULL}};
const Segment SGI255420[] = {{&S21455[35], 5}, {NULL}};
const Segment SGI255421[] = {{&S21455[68], 8}, {NULL}};
const Segment SGI255422[] = {{&S21455[116], 6}, {NULL}};
const Segment SGI255423[] = {{&S21455[130], 7}, {NULL}};
const Segment SGI255424[] = {{&S21456[13], 6}, {NULL}};
const Segment SGI255425[] = {{&S21456[105], 3}, {NULL}};
const Segment SGI255426[] = {{&S21456[35], 5}, {NULL}};
const Segment SGI255427[] = {{&S21456[68], 8}, {NULL}};
const Segment SGI255428[] = {{&S21456[116], 6}, {NULL}};
const Segment SGI255429[] = {{&S21456[130], 7}, {NULL}};
const Segment SGI255430[] = {{&S21457[13], 6}, {NULL}};
const Segment SGI255431[] = {{&S21457[105], 3}, {NULL}};
const Segment SGI255432[] = {{&S21457[35], 5}, {NULL}};
const Segment SGI255433[] = {{&S21457[68], 8}, {NULL}};
const Segment SGI255434[] = {{&S21457[116], 12}, {NULL}};
const Segment SGI255435[] = {{&S21457[129], 7}, {NULL}};
const Segment SGI255436[] = {{&S21458[13], 6}, {NULL}};
const Segment SGI255437[] = {{&S21458[105], 3}, {NULL}};
const Segment SGI255438[] = {{&S21458[35], 5}, {NULL}};
const Segment SGI255439[] = {{&S21458[68], 8}, {NULL}};
const Segment SGI255440[] = {{&S21458[116], 7}, {NULL}};
const Segment SGI255441[] = {{&S21458[131], 7}, {NULL}};
const Segment SGI255442[] = {{&S21459[13], 6}, {NULL}};
const Segment SGI255443[] = {{&S21459[105], 3}, {NULL}};
const Segment SGI255444[] = {{&S21459[35], 5}, {NULL}};
const Segment SGI255445[] = {{&S21459[68], 8}, {NULL}};
const Segment SGI255446[] = {{&S21459[116], 6}, {NULL}};
const Segment SGI255447[] = {{&S21459[130], 7}, {NULL}};
const Segment SGI255448[] = {{&S21460[13], 6}, {NULL}};
const Segment SGI255449[] = {{&S21460[105], 3}, {NULL}};
const Segment SGI255450[] = {{&S21460[35], 5}, {NULL}};
const Segment SGI255451[] = {{&S21460[68], 8}, {NULL}};
const Segment SGI255452[] = {{&S21460[116], 6}, {NULL}};
const Segment SGI255453[] = {{&S21460[130], 7}, {NULL}};
const Segment SGI255454[] = {{&S21461[13], 6}, {NULL}};
const Segment SGI255455[] = {{&S21461[104], 3}, {NULL}};
const Segment SGI255456[] = {{&S21461[67], 8}, {NULL}};
const Segment SGI255457[] = {{&S21461[115], 6}, {NULL}};
const Segment SGI255458[] = {{&S21461[129], 7}, {NULL}};
const Segment SGI255459[] = {{&S21634[13], 4}, {NULL}};
const Segment SGI255460[] = {{&S21634[26], 3}, {NULL}};
const Segment SGI255461[] = {{&S21634[64], 6}, {NULL}};
const Segment SGI255462[] = {{&S21634[118], 8}, {NULL}};
const Segment SGI255463[] = {{&S21634[134], 7}, {NULL}};
const Segment SGI255464[] = {{&S21635[13], 4}, {NULL}};
const Segment SGI255465[] = {{&S21635[26], 3}, {NULL}};
const Segment SGI255466[] = {{&S21635[64], 6}, {NULL}};
const Segment SGI255467[] = {{&S21635[118], 6}, {NULL}};
const Segment SGI255468[] = {{&S21635[132], 7}, {NULL}};
const Segment SGI255469[] = {{&S21636[13], 4}, {NULL}};
const Segment SGI255470[] = {{&S21636[26], 5}, {NULL}};
const Segment SGI255471[] = {{&S21636[66], 6}, {NULL}};
const Segment SGI255472[] = {{&S21636[120], 6}, {NULL}};
const Segment SGI255473[] = {{&S21636[134], 7}, {NULL}};
const Segment SGI255474[] = {{&S21637[13], 4}, {NULL}};
const Segment SGI255475[] = {{&S21637[26], 5}, {NULL}};
const Segment SGI255476[] = {{&S21637[66], 6}, {NULL}};
const Segment SGI255477[] = {{&S21637[120], 6}, {NULL}};
const Segment SGI255478[] = {{&S21637[134], 7}, {NULL}};
const Segment SGI255479[] = {{&S21638[13], 4}, {NULL}};
const Segment SGI255480[] = {{&S21638[26], 5}, {NULL}};
const Segment SGI255481[] = {{&S21638[66], 6}, {NULL}};
const Segment SGI255482[] = {{&S21638[120], 6}, {NULL}};
const Segment SGI255483[] = {{&S21638[134], 7}, {NULL}};
const Segment SGI255484[] = {{&S21650[13], 4}, {NULL}};
const Segment SGI255485[] = {{&S21650[92], 3}, {NULL}};
const Segment SGI255486[] = {{&S21650[26], 3}, {NULL}};
const Segment SGI255487[] = {{&S21650[57], 6}, {NULL}};
const Segment SGI255488[] = {{&S21650[103], 7}, {NULL}};
const Segment SGI255489[] = {{&S21650[118], 9}, {NULL}};
const Segment SGI255490[] = {{&S21651[13], 4}, {NULL}};
const Segment SGI255491[] = {{&S21651[92], 3}, {NULL}};
const Segment SGI255492[] = {{&S21651[26], 3}, {NULL}};
const Segment SGI255493[] = {{&S21651[57], 6}, {NULL}};
const Segment SGI255494[] = {{&S21651[103], 7}, {NULL}};
const Segment SGI255495[] = {{&S21651[118], 9}, {NULL}};
const Segment SGI255496[] = {{&S21652[13], 4}, {NULL}};
const Segment SGI255497[] = {{&S21652[92], 3}, {NULL}};
const Segment SGI255498[] = {{&S21652[26], 3}, {NULL}};
const Segment SGI255499[] = {{&S21652[57], 6}, {NULL}};
const Segment SGI255500[] = {{&S21652[103], 7}, {NULL}};
const Segment SGI255501[] = {{&S21652[118], 9}, {NULL}};
const Segment SGI255502[] = {{&S21653[13], 4}, {NULL}};
const Segment SGI255503[] = {{&S21653[92], 3}, {NULL}};
const Segment SGI255504[] = {{&S21653[26], 3}, {NULL}};
const Segment SGI255505[] = {{&S21653[57], 6}, {NULL}};
const Segment SGI255506[] = {{&S21653[103], 5}, {NULL}};
const Segment SGI255507[] = {{&S21653[116], 9}, {NULL}};
const Segment SGI255508[] = {{&S21654[13], 4}, {NULL}};
const Segment SGI255509[] = {{&S21654[92], 3}, {NULL}};
const Segment SGI255510[] = {{&S21654[26], 3}, {NULL}};
const Segment SGI255511[] = {{&S21654[57], 6}, {NULL}};
const Segment SGI255512[] = {{&S21654[103], 5}, {NULL}};
const Segment SGI255513[] = {{&S21654[116], 7}, {NULL}};
const Segment SGI255514[] = {{&S21655[13], 4}, {NULL}};
const Segment SGI255515[] = {{&S21655[94], 3}, {NULL}};
const Segment SGI255516[] = {{&S21655[26], 5}, {NULL}};
const Segment SGI255517[] = {{&S21655[59], 6}, {NULL}};
const Segment SGI255518[] = {{&S21655[105], 5}, {NULL}};
const Segment SGI255519[] = {{&S21655[118], 9}, {NULL}};
const Segment SGI255520[] = {{&S21656[13], 4}, {NULL}};
const Segment SGI255521[] = {{&S21656[94], 3}, {NULL}};
const Segment SGI255522[] = {{&S21656[26], 5}, {NULL}};
const Segment SGI255523[] = {{&S21656[59], 6}, {NULL}};
const Segment SGI255524[] = {{&S21656[105], 5}, {NULL}};
const Segment SGI255525[] = {{&S21656[118], 7}, {NULL}};
const Segment SGI255526[] = {{&S21657[13], 4}, {NULL}};
const Segment SGI255527[] = {{&S21657[92], 3}, {NULL}};
const Segment SGI255528[] = {{&S21657[26], 3}, {NULL}};
const Segment SGI255529[] = {{&S21657[57], 6}, {NULL}};
const Segment SGI255530[] = {{&S21657[103], 5}, {NULL}};
const Segment SGI255531[] = {{&S21658[13], 4}, {NULL}};
const Segment SGI255532[] = {{&S21658[94], 3}, {NULL}};
const Segment SGI255533[] = {{&S21658[26], 5}, {NULL}};
const Segment SGI255534[] = {{&S21658[59], 6}, {NULL}};
const Segment SGI255535[] = {{&S21658[105], 5}, {NULL}};
const Segment SGI255536[] = {{&S21658[118], 9}, {NULL}};
const Segment SGI255537[] = {{&S21659[13], 4}, {NULL}};
const Segment SGI255538[] = {{&S21659[92], 3}, {NULL}};
const Segment SGI255539[] = {{&S21659[26], 3}, {NULL}};
const Segment SGI255540[] = {{&S21659[57], 6}, {NULL}};
const Segment SGI255541[] = {{&S21659[103], 7}, {NULL}};
const Segment SGI255542[] = {{&S21659[118], 9}, {NULL}};
const Segment SGI255543[] = {{&S21660[13], 4}, {NULL}};
const Segment SGI255544[] = {{&S21660[94], 3}, {NULL}};
const Segment SGI255545[] = {{&S21660[26], 5}, {NULL}};
const Segment SGI255546[] = {{&S21660[59], 6}, {NULL}};
const Segment SGI255547[] = {{&S21660[105], 5}, {NULL}};
const Segment SGI255548[] = {{&S21660[118], 9}, {NULL}};
const Segment SGI255549[] = {{&S21661[13], 4}, {NULL}};
const Segment SGI255550[] = {{&S21661[94], 3}, {NULL}};
const Segment SGI255551[] = {{&S21661[26], 5}, {NULL}};
const Segment SGI255552[] = {{&S21661[59], 6}, {NULL}};
const Segment SGI255553[] = {{&S21661[105], 5}, {NULL}};
const Segment SGI255554[] = {{&S21661[118], 9}, {NULL}};
const Segment SGI255555[] = {{&S21662[13], 4}, {NULL}};
const Segment SGI255556[] = {{&S21662[101], 3}, {NULL}};
const Segment SGI255557[] = {{&S21662[33], 5}, {NULL}};
const Segment SGI255558[] = {{&S21662[66], 6}, {NULL}};
const Segment SGI255559[] = {{&S21662[112], 5}, {NULL}};
const Segment SGI255560[] = {{&S21662[125], 9}, {NULL}};
const Segment SGI255561[] = {{&S21663[13], 4}, {NULL}};
const Segment SGI255562[] = {{&S21663[92], 3}, {NULL}};
const Segment SGI255563[] = {{&S21663[26], 3}, {NULL}};
const Segment SGI255564[] = {{&S21663[57], 6}, {NULL}};
const Segment SGI255565[] = {{&S21663[103], 7}, {NULL}};
const Segment SGI255566[] = {{&S21663[118], 9}, {NULL}};
const Segment SGI255567[] = {{&S21664[13], 4}, {NULL}};
const Segment SGI255568[] = {{&S21664[114], 3}, {NULL}};
const Segment SGI255569[] = {{&S21664[26], 5}, {NULL}};
const Segment SGI255570[] = {{&S21664[59], 6}, {NULL}};
const Segment SGI255571[] = {{&S21664[93], 5}, {NULL}};
const Segment SGI255572[] = {{&S21665[13], 4}, {NULL}};
const Segment SGI255573[] = {{&S21665[94], 3}, {NULL}};
const Segment SGI255574[] = {{&S21665[26], 5}, {NULL}};
const Segment SGI255575[] = {{&S21665[59], 6}, {NULL}};
const Segment SGI255576[] = {{&S21665[105], 5}, {NULL}};
const Segment SGI255577[] = {{&S21665[118], 9}, {NULL}};
const Segment SGI255578[] = {{&S21666[13], 4}, {NULL}};
const Segment SGI255579[] = {{&S21666[92], 3}, {NULL}};
const Segment SGI255580[] = {{&S21666[26], 3}, {NULL}};
const Segment SGI255581[] = {{&S21666[57], 6}, {NULL}};
const Segment SGI255582[] = {{&S21666[103], 5}, {NULL}};
const Segment SGI255583[] = {{&S21666[116], 9}, {NULL}};
const Segment SGI255584[] = {{&S21667[13], 4}, {NULL}};
const Segment SGI255585[] = {{&S21667[98], 3}, {NULL}};
const Segment SGI255586[] = {{&S21667[33], 3}, {NULL}};
const Segment SGI255587[] = {{&S21667[64], 6}, {NULL}};
const Segment SGI255588[] = {{&S21667[109], 5}, {NULL}};
const Segment SGI255589[] = {{&S21667[122], 9}, {NULL}};
const Segment SGI255590[] = {{&S21668[13], 4}, {NULL}};
const Segment SGI255591[] = {{&S21668[111], 3}, {NULL}};
const Segment SGI255592[] = {{&S21668[26], 3}, {NULL}};
const Segment SGI255593[] = {{&S21668[57], 5}, {NULL}};
const Segment SGI255594[] = {{&S21668[122], 5}, {NULL}};
const Segment SGI255595[] = {{&S21668[135], 9}, {NULL}};
const Segment SGI255596[] = {{&S21669[13], 4}, {&S21669[13], 4}, {NULL}};
const Segment SGI255597[] = {{&S21669[117], 3}, {NULL}};
const Segment SGI255598[] = {{&S21669[8], 1}, {NULL}};
const Segment SGI255599[] = {{&S21669[82], 6}, {NULL}};
const Segment SGI255600[] = {{&S21669[128], 5}, {NULL}};
const Segment SGI255601[] = {{&S21669[141], 9}, {NULL}};
const Segment SGI255602[] = {{&S21670[13], 4}, {NULL}};
const Segment SGI255603[] = {{&S21670[92], 3}, {NULL}};
const Segment SGI255604[] = {{&S21670[26], 3}, {NULL}};
const Segment SGI255605[] = {{&S21670[57], 6}, {NULL}};
const Segment SGI255606[] = {{&S21670[103], 5}, {NULL}};
const Segment SGI255607[] = {{&S21670[116], 9}, {NULL}};
const Segment SGI255608[] = {{&S21698[13], 6}, {NULL}};
const Segment SGI255609[] = {{&S21698[35], 3}, {NULL}};
const Segment SGI255610[] = {{&S21698[73], 8}, {NULL}};
const Segment SGI255611[] = {{&S21698[128], 6}, {NULL}};
const Segment SGI255612[] = {{&S21698[142], 9}, {NULL}};
const Segment SGI255613[] = {{&S21699[13], 6}, {NULL}};
const Segment SGI255614[] = {{&S21699[35], 3}, {NULL}};
const Segment SGI255615[] = {{&S21699[73], 8}, {NULL}};
const Segment SGI255616[] = {{&S21699[128], 6}, {NULL}};
const Segment SGI255617[] = {{&S21699[142], 9}, {NULL}};
const Segment SGI255618[] = {{&S21700[13], 6}, {NULL}};
const Segment SGI255619[] = {{&S21700[35], 5}, {NULL}};
const Segment SGI255620[] = {{&S21700[75], 8}, {NULL}};
const Segment SGI255621[] = {{&S21700[130], 6}, {NULL}};
const Segment SGI255622[] = {{&S21700[144], 9}, {NULL}};
const Segment SGI255623[] = {{&S21701[13], 6}, {NULL}};
const Segment SGI255624[] = {{&S21701[35], 5}, {NULL}};
const Segment SGI255625[] = {{&S21701[75], 8}, {NULL}};
const Segment SGI255626[] = {{&S21701[130], 6}, {NULL}};
const Segment SGI255627[] = {{&S21701[144], 9}, {NULL}};
const Segment SGI255628[] = {{&S21702[13], 6}, {NULL}};
const Segment SGI255629[] = {{&S21702[35], 5}, {NULL}};
const Segment SGI255630[] = {{&S21702[75], 8}, {NULL}};
const Segment SGI255631[] = {{&S21702[130], 6}, {NULL}};
const Segment SGI255632[] = {{&S21702[144], 9}, {NULL}};
const Segment SGI255633[] = {{&S21755[13], 4}, {&S21755[13], 4}, {&S21755[13], 4}, {NULL}};
const Segment SGI255634[] = {{&S21755[42], 3}, {NULL}};
const Segment SGI255635[] = {{&S21756[13], 4}, {NULL}};
const Segment SGI255636[] = {{&S21756[33], 3}, {NULL}};
const Segment SGI255637[] = {{&S21756[64], 6}, {NULL}};
const Segment SGI255638[] = {{&S21756[98], 8}, {NULL}};
const Segment SGI255639[] = {{&S21757[13], 4}, {NULL}};
const Segment SGI255640[] = {{&S21757[33], 3}, {NULL}};
const Segment SGI255641[] = {{&S21757[64], 6}, {NULL}};
const Segment SGI255642[] = {{&S21757[98], 6}, {NULL}};
const Segment SGI255643[] = {{&S21758[13], 4}, {NULL}};
const Segment SGI255644[] = {{&S21758[33], 3}, {NULL}};
const Segment SGI255645[] = {{&S21758[64], 6}, {NULL}};
const Segment SGI255646[] = {{&S21759[13], 4}, {&S21759[13], 4}, {NULL}};
const Segment SGI255647[] = {{&S21759[33], 3}, {NULL}};
const Segment SGI255648[] = {{&S21759[64], 6}, {NULL}};
const Segment SGI255649[] = {{&S21759[98], 6}, {NULL}};
const Segment SGI255650[] = {{&S21760[13], 4}, {&S21760[13], 4}, {&S21760[13], 4}, {NULL}};
const Segment SGI255651[] = {{&S21760[33], 3}, {NULL}};
const Segment SGI255652[] = {{&S21760[64], 6}, {NULL}};
const Segment SGI255653[] = {{&S21760[98], 6}, {NULL}};
const Segment SGI255654[] = {{&S21761[13], 4}, {NULL}};
const Segment SGI255655[] = {{&S21761[44], 3}, {NULL}};
const Segment SGI255656[] = {{&S21761[125], 5}, {NULL}};
const Segment SGI255657[] = {{&S21762[13], 4}, {NULL}};
const Segment SGI255658[] = {{&S21762[33], 3}, {NULL}};
const Segment SGI255659[] = {{&S21762[64], 6}, {NULL}};
const Segment SGI255660[] = {{&S21762[98], 6}, {NULL}};
const Segment SGI255661[] = {{&S21763[13], 4}, {&S21763[13], 4}, {&S21763[13], 4}, {NULL}};
const Segment SGI255662[] = {{&S21763[42], 5}, {NULL}};
const Segment SGI255663[] = {{&S21764[13], 4}, {NULL}};
const Segment SGI255664[] = {{&S21764[33], 5}, {NULL}};
const Segment SGI255665[] = {{&S21764[66], 6}, {NULL}};
const Segment SGI255666[] = {{&S21764[100], 6}, {NULL}};
const Segment SGI255667[] = {{&S21765[13], 4}, {&S21765[13], 4}, {NULL}};
const Segment SGI255668[] = {{&S21765[33], 5}, {NULL}};
const Segment SGI255669[] = {{&S21765[66], 6}, {NULL}};
const Segment SGI255670[] = {{&S21765[100], 6}, {NULL}};
const Segment SGI255671[] = {{&S21766[13], 4}, {NULL}};
const Segment SGI255672[] = {{&S21766[33], 5}, {NULL}};
const Segment SGI255673[] = {{&S21766[66], 6}, {NULL}};
const Segment SGI255674[] = {{&S21767[13], 6}, {&S21767[109], 4}, {NULL}};
const Segment SGI255675[] = {{&S21767[35], 5}, {NULL}};
const Segment SGI255676[] = {{&S21767[68], 6}, {NULL}};
const Segment SGI255677[] = {{&S21767[102], 6}, {NULL}};
const Segment SGI255678[] = {{&S21768[13], 4}, {&S21768[23], 6}, {&S21768[13], 4}, {NULL}};
const Segment SGI255679[] = {{&S21768[33], 5}, {&S21768[129], 5}, {NULL}};
const Segment SGI255680[] = {{&S21768[66], 6}, {NULL}};
const Segment SGI255681[] = {{&S21768[100], 6}, {NULL}};
const Segment SGI255682[] = {{&S21769[13], 6}, {&S21769[128], 4}, {NULL}};
const Segment SGI255683[] = {{&S21769[75], 6}, {NULL}};
const Segment SGI255684[] = {{&S21769[109], 4}, {NULL}};
const Segment SGI255685[] = {{&S21770[13], 4}, {&S21770[13], 4}, {&S21770[13], 4}, {NULL}};
const Segment SGI255686[] = {{&S21770[33], 5}, {NULL}};
const Segment SGI255687[] = {{&S21770[66], 6}, {NULL}};
const Segment SGI255688[] = {{&S21770[100], 6}, {NULL}};
const Segment SGI255689[] = {{&S21771[13], 4}, {&S21771[13], 4}, {&S21771[13], 4}, {NULL}};
const Segment SGI255690[] = {{&S21771[33], 3}, {NULL}};
const Segment SGI255691[] = {{&S21771[64], 6}, {NULL}};
const Segment SGI255692[] = {{&S21771[98], 6}, {NULL}};
const Segment SGI255693[] = {{&S21772[13], 4}, {NULL}};
const Segment SGI255694[] = {{&S21772[131], 3}, {NULL}};
const Segment SGI255695[] = {{&S21772[33], 5}, {NULL}};
const Segment SGI255696[] = {{&S21772[66], 6}, {NULL}};
const Segment SGI255697[] = {{&S21772[100], 6}, {NULL}};
const Segment SGI255698[] = {{&S21773[13], 4}, {NULL}};
const Segment SGI255699[] = {{&S21773[40], 5}, {NULL}};
const Segment SGI255700[] = {{&S21773[123], 5}, {NULL}};
const Segment SGI255701[] = {{&S10897[121], 11}, {NULL}};
const Segment SGI255702[] = {{&S10897[0], 132}, {NULL}};
const Segment SGI255703[] = {{&S21774[13], 4}, {&S21774[144], 4}, {&S21774[144], 4}, {NULL}};
const Segment SGI255704[] = {{&S21774[33], 5}, {NULL}};
const Segment SGI255705[] = {{&S21774[66], 6}, {NULL}};
const Segment SGI255706[] = {{&S21774[100], 6}, {NULL}};
const Segment SGI255707[] = {{&S21775[13], 4}, {NULL}};
const Segment SGI255708[] = {{&S21775[33], 5}, {NULL}};
const Segment SGI255709[] = {{&S21775[66], 6}, {NULL}};
const Segment SGI255710[] = {{&S21775[100], 6}, {NULL}};
const Segment SGI255711[] = {{&S21775[114], 9}, {NULL}};
const Segment SGI255712[] = {{&S21776[13], 4}, {NULL}};
const Segment SGI255713[] = {{&S21776[129], 3}, {NULL}};
const Segment SGI255714[] = {{&S21776[33], 3}, {NULL}};
const Segment SGI255715[] = {{&S21776[64], 6}, {NULL}};
const Segment SGI255716[] = {{&S21776[98], 6}, {NULL}};
const Segment SGI255717[] = {{&S21777[13], 4}, {NULL}};
const Segment SGI255718[] = {{&S21777[106], 5}, {NULL}};
const Segment SGI255719[] = {{&S21777[29], 5}, {NULL}};
const Segment SGI255720[] = {{&S21777[69], 8}, {NULL}};
const Segment SGI255721[] = {{&S21777[119], 5}, {NULL}};
const Segment SGI255722[] = {{&S21777[132], 9}, {NULL}};
const Segment SGI255723[] = {{&S21778[13], 4}, {NULL}};
const Segment SGI255724[] = {{&S21778[29], 5}, {NULL}};
const Segment SGI255725[] = {{&S21778[69], 8}, {NULL}};
const Segment SGI255726[] = {{&S21778[105], 5}, {NULL}};
const Segment SGI255727[] = {{&S21778[118], 9}, {NULL}};
const Segment SGI255728[] = {{&S21779[13], 4}, {NULL}};
const Segment SGI255729[] = {{&S21779[104], 5}, {NULL}};
const Segment SGI255730[] = {{&S21779[29], 3}, {NULL}};
const Segment SGI255731[] = {{&S21779[67], 8}, {NULL}};
const Segment SGI255732[] = {{&S21779[117], 5}, {NULL}};
const Segment SGI255733[] = {{&S21779[130], 9}, {NULL}};
const Segment SGI255734[] = {{&S21780[13], 4}, {NULL}};
const Segment SGI255735[] = {{&S21780[104], 5}, {NULL}};
const Segment SGI255736[] = {{&S21780[29], 3}, {NULL}};
const Segment SGI255737[] = {{&S21780[67], 8}, {NULL}};
const Segment SGI255738[] = {{&S21780[117], 6}, {NULL}};
const Segment SGI255739[] = {{&S21780[131], 9}, {NULL}};
const Segment SGI255740[] = {{&S21781[13], 4}, {NULL}};
const Segment SGI255741[] = {{&S21781[106], 5}, {NULL}};
const Segment SGI255742[] = {{&S21781[29], 5}, {NULL}};
const Segment SGI255743[] = {{&S21781[69], 8}, {NULL}};
const Segment SGI255744[] = {{&S21781[119], 5}, {NULL}};
const Segment SGI255745[] = {{&S21781[132], 9}, {NULL}};
const Segment SGI255746[] = {{&S21782[13], 4}, {NULL}};
const Segment SGI255747[] = {{&S21782[107], 5}, {NULL}};
const Segment SGI255748[] = {{&S21782[29], 5}, {NULL}};
const Segment SGI255749[] = {{&S21782[69], 9}, {NULL}};
const Segment SGI255750[] = {{&S21782[120], 3}, {NULL}};
const Segment SGI255751[] = {{&S21782[131], 9}, {NULL}};
const Segment SGI255752[] = {{&S21783[13], 4}, {&S21783[13], 4}, {NULL}};
const Segment SGI255753[] = {{&S21783[118], 5}, {NULL}};
const Segment SGI255754[] = {{&S21783[29], 3}, {NULL}};
const Segment SGI255755[] = {{&S21783[81], 8}, {NULL}};
const Segment SGI255756[] = {{&S21783[131], 5}, {NULL}};
const Segment SGI255757[] = {{&S21783[144], 9}, {NULL}};
const Segment SGI255758[] = {{&S21784[13], 4}, {NULL}};
const Segment SGI255759[] = {{&S21784[104], 5}, {NULL}};
const Segment SGI255760[] = {{&S21784[29], 3}, {NULL}};
const Segment SGI255761[] = {{&S21784[67], 8}, {NULL}};
const Segment SGI255762[] = {{&S21784[117], 7}, {NULL}};
const Segment SGI255763[] = {{&S21784[132], 9}, {NULL}};
const Segment SGI255764[] = {{&S21785[13], 4}, {NULL}};
const Segment SGI255765[] = {{&S21785[106], 5}, {NULL}};
const Segment SGI255766[] = {{&S21785[29], 5}, {NULL}};
const Segment SGI255767[] = {{&S21785[69], 8}, {NULL}};
const Segment SGI255768[] = {{&S21785[119], 5}, {NULL}};
const Segment SGI255769[] = {{&S21785[132], 9}, {NULL}};
const Segment SGI255770[] = {{&S21786[13], 4}, {NULL}};
const Segment SGI255771[] = {{&S21786[104], 5}, {NULL}};
const Segment SGI255772[] = {{&S21786[29], 3}, {NULL}};
const Segment SGI255773[] = {{&S21786[67], 8}, {NULL}};
const Segment SGI255774[] = {{&S21786[117], 7}, {NULL}};
const Segment SGI255775[] = {{&S21786[132], 9}, {NULL}};
const Segment SGI255776[] = {{&S21787[13], 4}, {NULL}};
const Segment SGI255777[] = {{&S21787[29], 3}, {NULL}};
const Segment SGI255778[] = {{&S21787[67], 8}, {NULL}};
const Segment SGI255779[] = {{&S21787[103], 5}, {NULL}};
const Segment SGI255780[] = {{&S21787[116], 9}, {NULL}};
const Segment SGI255781[] = {{&S21788[13], 4}, {NULL}};
const Segment SGI255782[] = {{&S21788[106], 5}, {NULL}};
const Segment SGI255783[] = {{&S21788[29], 5}, {NULL}};
const Segment SGI255784[] = {{&S21788[69], 8}, {NULL}};
const Segment SGI255785[] = {{&S21788[119], 5}, {NULL}};
const Segment SGI255786[] = {{&S21788[132], 6}, {NULL}};
const Segment SGI255787[] = {{&S21789[13], 4}, {NULL}};
const Segment SGI255788[] = {{&S21789[104], 5}, {NULL}};
const Segment SGI255789[] = {{&S21789[29], 3}, {NULL}};
const Segment SGI255790[] = {{&S21789[68], 8}, {NULL}};
const Segment SGI255791[] = {{&S21789[117], 5}, {NULL}};
const Segment SGI255792[] = {{&S21789[130], 9}, {NULL}};
const Segment SGI255793[] = {{&S11391[24], 3}, {NULL}};
const Segment SGI255794[] = {{&S11391[0], 158}, {NULL}};
const Segment SGI255795[] = {{&S11392[29], 16}, {NULL}};
const Segment SGI255796[] = {{&S11392[0], 45}, {NULL}};
const Segment SGI255797[] = {{&S11398[11], 3}, {NULL}};
const Segment SGI255798[] = {{&S11398[0], 82}, {NULL}};
const Segment SGI255799[] = {{&S11399[3], 7}, {NULL}};
const Segment SGI255800[] = {{&S11399[0], 63}, {NULL}};
const Segment SGI255801[] = {{&S11400[13], 3}, {NULL}};
const Segment SGI255802[] = {{&S11400[0], 20}, {NULL}};
const Segment SGI255803[] = {{&S11401[0], 7}, {&S11401[13], 33}, {NULL}};
const Segment SGI255804[] = {{&S11401[0], 46}, {NULL}};
const Segment SGI255805[] = {{&S11402[25], 15}, {&S11402[41], 36}, {NULL}};
const Segment SGI255806[] = {{&S11402[0], 77}, {NULL}};
const Segment SGI255807[] = {{&S11403[0], 19}, {NULL}};
const Segment SGI255808[] = {{&S11404[25], 12}, {NULL}};
const Segment SGI255809[] = {{&S11404[0], 64}, {NULL}};
const Segment SGI255810[] = {{&S11405[9], 3}, {NULL}};
const Segment SGI255811[] = {{&S11405[0], 39}, {NULL}};
const Segment SGI255812[] = {{&S11406[10], 6}, {NULL}};
const Segment SGI255813[] = {{&S11406[0], 25}, {NULL}};
const Segment SGI255814[] = {{&S11407[35], 3}, {NULL}};
const Segment SGI255815[] = {{&S11407[0], 63}, {NULL}};
const Segment SGI255816[] = {{&S11408[20], 29}, {NULL}};
const Segment SGI255817[] = {{&S11408[0], 49}, {NULL}};
const Segment SGI255818[] = {{&S11409[0], 17}, {NULL}};
const Segment SGI255819[] = {{&S11409[0], 54}, {NULL}};
const Segment SGI255820[] = {{&S11410[19], 6}, {NULL}};
const Segment SGI255821[] = {{&S11410[0], 30}, {NULL}};
const Segment SGI255822[] = {{&S11411[25], 21}, {NULL}};
const Segment SGI255823[] = {{&S11411[0], 90}, {NULL}};
const Segment SGI255824[] = {{&S11412[19], 6}, {NULL}};
const Segment SGI255825[] = {{&S11412[0], 30}, {NULL}};
const Segment SGI255826[] = {{&S11413[25], 15}, {NULL}};
const Segment SGI255827[] = {{&S11413[0], 67}, {NULL}};
const Segment SGI255828[] = {{&S11414[25], 20}, {NULL}};
const Segment SGI255829[] = {{&S11414[0], 73}, {NULL}};
const Segment SGI255830[] = {{&S11415[34], 3}, {NULL}};
const Segment SGI255831[] = {{&S11415[0], 37}, {NULL}};
const Segment SGI255832[] = {{&S11416[0], 5}, {&S11416[93], 3}, {NULL}};
const Segment SGI255833[] = {{&S11416[0], 110}, {NULL}};
const Segment SGI255834[] = {{&S11417[0], 18}, {NULL}};
const Segment SGI255835[] = {{&S11417[0], 65}, {NULL}};
const Segment SGI255836[] = {{&S11418[0], 13}, {&S11418[14], 30}, {NULL}};
const Segment SGI255837[] = {{&S11418[0], 44}, {NULL}};
const Segment SGI255838[] = {{&S11419[25], 11}, {&S11419[38], 31}, {NULL}};
const Segment SGI255839[] = {{&S11419[0], 69}, {NULL}};
const Segment SGI255840[] = {{&S11420[0], 17}, {&S11420[18], 30}, {NULL}};
const Segment SGI255841[] = {{&S11420[0], 48}, {NULL}};
const Segment SGI255842[] = {{&S11421[40], 30}, {NULL}};
const Segment SGI255843[] = {{&S11421[0], 70}, {NULL}};
const Segment SGI255844[] = {{&S11422[61], 21}, {NULL}};
const Segment SGI255845[] = {{&S11422[0], 82}, {NULL}};
const Segment SGI255846[] = {{&S11423[44], 23}, {NULL}};
const Segment SGI255847[] = {{&S11423[0], 73}, {NULL}};
const Segment SGI255848[] = {{&S11424[44], 23}, {NULL}};
const Segment SGI255849[] = {{&S11424[0], 73}, {NULL}};
const Segment SGI255850[] = {{&S11425[25], 3}, {NULL}};
const Segment SGI255851[] = {{&S11425[0], 28}, {NULL}};
const Segment SGI255852[] = {{&S11426[0], 12}, {&S11426[13], 36}, {NULL}};
const Segment SGI255853[] = {{&S11426[0], 49}, {NULL}};
const Segment SGI255854[] = {{&S11427[25], 21}, {NULL}};
const Segment SGI255855[] = {{&S11427[0], 72}, {NULL}};
const Segment SGI255856[] = {{&S11428[25], 23}, {NULL}};
const Segment SGI255857[] = {{&S11428[0], 79}, {NULL}};
const Segment SGI255858[] = {{&S11429[25], 18}, {NULL}};
const Segment SGI255859[] = {{&S11429[0], 73}, {NULL}};
const Segment SGI255860[] = {{&S11430[85], 8}, {NULL}};
const Segment SGI255861[] = {{&S11430[0], 93}, {NULL}};
const Segment SGI255862[] = {{&S11431[0], 13}, {&S11431[14], 41}, {NULL}};
const Segment SGI255863[] = {{&S11431[0], 55}, {NULL}};
const Segment SGI255864[] = {{&S11432[25], 19}, {NULL}};
const Segment SGI255865[] = {{&S11432[0], 85}, {NULL}};
const Segment SGI255866[] = {{&S11433[0], 13}, {&S11433[14], 41}, {NULL}};
const Segment SGI255867[] = {{&S11433[0], 55}, {NULL}};
const Segment SGI255868[] = {{&S11434[42], 34}, {NULL}};
const Segment SGI255869[] = {{&S11434[0], 114}, {NULL}};
const Segment SGI255870[] = {{&S11435[0], 11}, {NULL}};
const Segment SGI255871[] = {{&S11435[0], 63}, {NULL}};
const Segment SGI255872[] = {{&S11436[0], 21}, {NULL}};
const Segment SGI255873[] = {{&S11436[0], 66}, {NULL}};
const Segment SGI255874[] = {{&S11437[0], 6}, {NULL}};
const Segment SGI255875[] = {{&S11438[25], 21}, {&S11438[47], 38}, {NULL}};
const Segment SGI255876[] = {{&S11438[0], 85}, {NULL}};
const Segment SGI255877[] = {{&S11439[0], 8}, {NULL}};
const Segment SGI255878[] = {{&S11440[25], 11}, {NULL}};
const Segment SGI255879[] = {{&S11440[0], 70}, {NULL}};
const Segment SGI255880[] = {{&S11441[0], 23}, {NULL}};
const Segment SGI255881[] = {{&S11441[0], 69}, {NULL}};
const Segment SGI255882[] = {{&S11442[18], 6}, {&S11442[24], 8}, {NULL}};
const Segment SGI255883[] = {{&S11442[0], 63}, {NULL}};
const Segment SGI255884[] = {{&S11443[0], 5}, {NULL}};
const Segment SGI255885[] = {{&S11443[0], 73}, {NULL}};
const Segment SGI255886[] = {{&S11444[0], 11}, {NULL}};
const Segment SGI255887[] = {{&S18587[53], 5}, {NULL}};
const Segment SGI255888[] = {{&S18591[53], 5}, {NULL}};
const Segment SGI255889[] = {{&S11445[24], 3}, {&S11445[59], 16}, {NULL}};
const Segment SGI255890[] = {{&S11445[0], 75}, {NULL}};
const Segment SGI255891[] = {{&S11446[25], 18}, {&S11446[44], 33}, {NULL}};
const Segment SGI255892[] = {{&S11446[0], 77}, {NULL}};
const Segment SGI255893[] = {{&S11447[0], 5}, {&S11447[100], 3}, {NULL}};
const Segment SGI255894[] = {{&S11447[0], 117}, {NULL}};
const Segment SGI255895[] = {{&S11448[0], 9}, {NULL}};
const Segment SGI255896[] = {{&S11449[0], 12}, {NULL}};
const Segment SGI255897[] = {{&S11450[0], 5}, {NULL}};
const Segment SGI255898[] = {{&S11450[0], 74}, {NULL}};
const Segment SGI255899[] = {{&S11451[10], 3}, {NULL}};
const Segment SGI255900[] = {{&S11451[0], 17}, {NULL}};
const Segment SGI255901[] = {{&S11452[25], 14}, {&S11452[40], 31}, {NULL}};
const Segment SGI255902[] = {{&S11452[0], 71}, {NULL}};
const Segment SGI255903[] = {{&S11453[18], 6}, {&S11453[24], 8}, {NULL}};
const Segment SGI255904[] = {{&S11453[0], 63}, {NULL}};
const Segment SGI255905[] = {{&S11454[5], 6}, {NULL}};
const Segment SGI255906[] = {{&S11454[0], 11}, {NULL}};
const Segment SGI255907[] = {{&S11455[25], 16}, {NULL}};
const Segment SGI255908[] = {{&S11455[0], 41}, {NULL}};
const Segment SGI255909[] = {{&S11456[0], 6}, {&S11456[6], 4}, {NULL}};
const Segment SGI255910[] = {{&S11456[0], 10}, {NULL}};
const Segment SGI255911[] = {{&S11457[10], 6}, {NULL}};
const Segment SGI255912[] = {{&S11457[0], 25}, {NULL}};
const Segment SGI255913[] = {{&S11458[10], 6}, {NULL}};
const Segment SGI255914[] = {{&S11458[0], 25}, {NULL}};
const Segment SGI255915[] = {{&S11459[11], 36}, {NULL}};
const Segment SGI255916[] = {{&S11459[0], 47}, {NULL}};
const Segment SGI255917[] = {{&S11460[25], 19}, {NULL}};
const Segment SGI255918[] = {{&S11460[0], 64}, {NULL}};
const Segment SGI255919[] = {{&S11461[0], 9}, {&S11461[10], 67}, {NULL}};
const Segment SGI255920[] = {{&S11461[0], 77}, {NULL}};
const Segment SGI255921[] = {{&S11462[25], 13}, {&S11462[39], 62}, {NULL}};
const Segment SGI255922[] = {{&S11462[0], 101}, {NULL}};
const Segment SGI255923[] = {{&S11463[0], 59}, {NULL}};
const Segment SGI255924[] = {{&S11464[19], 3}, {NULL}};
const Segment SGI255925[] = {{&S11464[0], 45}, {NULL}};
const Segment SGI255926[] = {{&S11465[25], 13}, {&S11465[39], 58}, {NULL}};
const Segment SGI255927[] = {{&S11465[0], 97}, {NULL}};
const Segment SGI255928[] = {{&S11466[35], 35}, {NULL}};
const Segment SGI255929[] = {{&S11466[0], 95}, {NULL}};
const Segment SGI255930[] = {{&S11467[0], 9}, {&S11467[10], 51}, {NULL}};
const Segment SGI255931[] = {{&S11467[0], 61}, {NULL}};
const Segment SGI255932[] = {{&S11468[0], 9}, {&S11468[10], 55}, {NULL}};
const Segment SGI255933[] = {{&S11468[0], 65}, {NULL}};
const Segment SGI255934[] = {{&S11469[0], 9}, {&S11469[24], 23}, {NULL}};
const Segment SGI255935[] = {{&S11469[0], 82}, {NULL}};
const Segment SGI255936[] = {{&S11470[0], 13}, {&S11470[14], 40}, {NULL}};
const Segment SGI255937[] = {{&S11470[0], 54}, {NULL}};
const Segment SGI255938[] = {{&S11471[0], 16}, {&S11471[17], 37}, {&S11471[47], 5}, {NULL}};
const Segment SGI255939[] = {{&S11471[0], 107}, {NULL}};
const Segment SGI255940[] = {{&S11472[25], 16}, {&S11472[42], 58}, {NULL}};
const Segment SGI255941[] = {{&S11472[0], 100}, {NULL}};
const Segment SGI255942[] = {{&S11473[42], 11}, {NULL}};
const Segment SGI255943[] = {{&S11473[0], 85}, {NULL}};
const Segment SGI255944[] = {{&S11474[0], 8}, {NULL}};
const Segment SGI255945[] = {{&S11474[0], 26}, {NULL}};
const Segment SGI255946[] = {{&S11475[25], 16}, {&S11475[42], 33}, {NULL}};
const Segment SGI255947[] = {{&S11475[0], 75}, {NULL}};
const Segment SGI255948[] = {{&S11476[44], 24}, {NULL}};
const Segment SGI255949[] = {{&S11476[0], 68}, {NULL}};
const Segment SGI255950[] = {{&S11477[0], 22}, {NULL}};
const Segment SGI255951[] = {{&S11478[45], 27}, {NULL}};
const Segment SGI255952[] = {{&S11478[0], 72}, {NULL}};
const Segment SGI255953[] = {{&S11479[25], 14}, {NULL}};
const Segment SGI255954[] = {{&S11479[0], 63}, {NULL}};
const Segment SGI255955[] = {{&S11480[0], 10}, {NULL}};
const Segment SGI255956[] = {{&S11480[0], 101}, {NULL}};
const Segment SGI255957[] = {{&S11481[0], 8}, {&S11481[11], 40}, {NULL}};
const Segment SGI255958[] = {{&S11481[0], 51}, {NULL}};
const Segment SGI255959[] = {{&S11482[0], 16}, {NULL}};
const Segment SGI255960[] = {{&S11482[0], 58}, {NULL}};
const Segment SGI255961[] = {{&S11483[0], 10}, {&S11483[11], 52}, {NULL}};
const Segment SGI255962[] = {{&S11483[0], 63}, {NULL}};
const Segment SGI255963[] = {{&S11484[10], 3}, {NULL}};
const Segment SGI255964[] = {{&S11484[0], 40}, {NULL}};
const Segment SGI255965[] = {{&S11485[25], 12}, {NULL}};
const Segment SGI255966[] = {{&S11485[0], 85}, {NULL}};
const Segment SGI255967[] = {{&S11486[0], 13}, {NULL}};
const Segment SGI255968[] = {{&S11486[0], 40}, {NULL}};
const Segment SGI255969[] = {{&S11487[0], 11}, {&S11487[12], 52}, {NULL}};
const Segment SGI255970[] = {{&S11487[0], 64}, {NULL}};
const Segment SGI255971[] = {{&S11488[40], 48}, {NULL}};
const Segment SGI255972[] = {{&S11488[0], 88}, {NULL}};
const Segment SGI255973[] = {{&S11489[39], 6}, {NULL}};
const Segment SGI255974[] = {{&S11489[0], 101}, {NULL}};
const Segment SGI255975[] = {{&S11490[46], 24}, {NULL}};
const Segment SGI255976[] = {{&S11490[0], 70}, {NULL}};
const Segment SGI255977[] = {{&S11491[39], 34}, {NULL}};
const Segment SGI255978[] = {{&S11491[0], 73}, {NULL}};
const Segment SGI255979[] = {{&S11492[29], 6}, {NULL}};
const Segment SGI255980[] = {{&S11492[0], 91}, {NULL}};
const Segment SGI255981[] = {{&S11493[25], 14}, {NULL}};
const Segment SGI255982[] = {{&S11493[0], 39}, {NULL}};
const Segment SGI255983[] = {{&S11495[6], 3}, {NULL}};
const Segment SGI255984[] = {{&S11495[0], 33}, {NULL}};
const Segment SGI255985[] = {{&S11496[25], 11}, {&S11496[37], 31}, {NULL}};
const Segment SGI255986[] = {{&S11496[0], 70}, {NULL}};
const Segment SGI255987[] = {{&S11497[25], 13}, {NULL}};
const Segment SGI255988[] = {{&S11497[0], 85}, {NULL}};
const Segment SGI255989[] = {{&S11498[11], 3}, {NULL}};
const Segment SGI255990[] = {{&S11498[0], 45}, {NULL}};
const Segment SGI255991[] = {{&S11499[0], 7}, {NULL}};
const Segment SGI255992[] = {{&S11500[0], 11}, {&S11500[12], 26}, {&S11500[39], 36}, {NULL}};
const Segment SGI255993[] = {{&S11500[0], 75}, {NULL}};
const Segment SGI255994[] = {{&S11501[0], 8}, {NULL}};
const Segment SGI255995[] = {{&S11501[0], 26}, {NULL}};
const Segment SGI255996[] = {{&S11502[5], 9}, {NULL}};
const Segment SGI255997[] = {{&S11502[0], 56}, {NULL}};
const Segment SGI255998[] = {{&S11503[0], 12}, {&S11503[32], 20}, {NULL}};
const Segment SGI255999[] = {{&S11503[0], 52}, {NULL}};
const Segment SGI256000[] = {{&S11504[42], 6}, {&S11504[49], 40}, {NULL}};
const Segment SGI256001[] = {{&S11504[0], 89}, {NULL}};
const Segment SGI256002[] = {{&S11505[9], 63}, {NULL}};
const Segment SGI256003[] = {{&S11505[0], 72}, {NULL}};
const Segment SGI256004[] = {{&S11506[11], 3}, {&S11506[21], 54}, {NULL}};
const Segment SGI256005[] = {{&S11506[0], 75}, {NULL}};
const Segment SGI256006[] = {{&S11507[0], 7}, {&S11507[33], 12}, {&S11507[46], 44}, {NULL}};
const Segment SGI256007[] = {{&S11507[0], 90}, {NULL}};
const Segment SGI256008[] = {{&S11508[11], 27}, {NULL}};
const Segment SGI256009[] = {{&S11508[0], 38}, {NULL}};
const Segment SGI256010[] = {{&S11509[25], 14}, {NULL}};
const Segment SGI256011[] = {{&S11509[0], 63}, {NULL}};
const Segment SGI256012[] = {{&S11510[25], 11}, {&S11510[37], 31}, {NULL}};
const Segment SGI256013[] = {{&S11510[0], 70}, {NULL}};
const Segment SGI256014[] = {{&S11610[25], 14}, {&S11610[40], 31}, {NULL}};
const Segment SGI256015[] = {{&S11610[0], 71}, {NULL}};
const Segment SGI256016[] = {{&S11611[25], 14}, {&S11611[40], 32}, {NULL}};
const Segment SGI256017[] = {{&S11611[0], 72}, {NULL}};
const Segment SGI256018[] = {{&S11612[0], 9}, {NULL}};
const Segment SGI256019[] = {{&S11613[25], 14}, {&S11613[41], 31}, {NULL}};
const Segment SGI256020[] = {{&S11613[0], 72}, {NULL}};
const Segment SGI256021[] = {{&S11614[0], 13}, {&S11614[14], 33}, {NULL}};
const Segment SGI256022[] = {{&S11614[0], 47}, {NULL}};
const Segment SGI256023[] = {{&S11615[0], 19}, {NULL}};
const Segment SGI256024[] = {{&S11616[0], 72}, {NULL}};
const Segment SGI256025[] = {{&S11617[0], 13}, {&S11617[14], 36}, {NULL}};
const Segment SGI256026[] = {{&S11617[0], 50}, {NULL}};
const Segment SGI256027[] = {{&S11618[0], 13}, {&S11618[14], 35}, {NULL}};
const Segment SGI256028[] = {{&S11618[0], 49}, {NULL}};
const Segment SGI256029[] = {{&S11619[0], 29}, {NULL}};
const Segment SGI256030[] = {{&S11619[0], 58}, {NULL}};
const Segment SGI256031[] = {{&S11620[43], 10}, {NULL}};
const Segment SGI256032[] = {{&S11620[0], 53}, {NULL}};
const Segment SGI256033[] = {{&S11621[25], 14}, {&S11621[40], 42}, {NULL}};
const Segment SGI256034[] = {{&S11621[0], 111}, {NULL}};
const Segment SGI256035[] = {{&S11622[0], 19}, {NULL}};
const Segment SGI256036[] = {{&S11623[0], 22}, {NULL}};
const Segment SGI256037[] = {{&S11624[25], 14}, {&S11624[40], 32}, {NULL}};
const Segment SGI256038[] = {{&S11624[0], 86}, {NULL}};
const Segment SGI256039[] = {{&S11625[0], 19}, {&S11625[42], 40}, {NULL}};
const Segment SGI256040[] = {{&S11625[0], 111}, {NULL}};
const Segment SGI256041[] = {{&S11626[25], 14}, {&S11626[52], 14}, {&S11626[67], 32}, {NULL}};
const Segment SGI256042[] = {{&S11626[0], 99}, {NULL}};
const Segment SGI256043[] = {{&S11627[0], 13}, {NULL}};
const Segment SGI256044[] = {{&S11628[25], 14}, {&S11628[40], 42}, {NULL}};
const Segment SGI256045[] = {{&S11628[0], 109}, {NULL}};
const Segment SGI256046[] = {{&S11629[25], 14}, {&S11629[40], 32}, {&S11629[25], 14}, {&S11629[146], 42}, {NULL}};
const Segment SGI256047[] = {{&S11629[0], 217}, {NULL}};
const Segment SGI256048[] = {{&S11630[0], 14}, {NULL}};
const Segment SGI256049[] = {{&S11631[26], 14}, {&S11631[41], 32}, {NULL}};
const Segment SGI256050[] = {{&S11631[0], 73}, {NULL}};
const Segment SGI256051[] = {{&S11632[25], 14}, {&S11632[40], 32}, {NULL}};
const Segment SGI256052[] = {{&S11632[0], 86}, {NULL}};
const Segment SGI256053[] = {{&S11633[0], 19}, {&S11633[24], 11}, {NULL}};
const Segment SGI256054[] = {{&S11633[0], 69}, {NULL}};
const Segment SGI256055[] = {{&S11634[0], 13}, {&S11634[16], 34}, {NULL}};
const Segment SGI256056[] = {{&S11634[0], 50}, {NULL}};
const Segment SGI256057[] = {{&S11635[25], 14}, {&S11635[40], 32}, {NULL}};
const Segment SGI256058[] = {{&S11635[0], 96}, {NULL}};
const Segment SGI256059[] = {{&S11636[0], 19}, {&S11636[22], 34}, {NULL}};
const Segment SGI256060[] = {{&S11636[0], 56}, {NULL}};
const Segment SGI256061[] = {{&S11637[25], 14}, {&S11637[40], 31}, {NULL}};
const Segment SGI256062[] = {{&S11637[0], 71}, {NULL}};
const Segment SGI256063[] = {{&S11638[0], 47}, {NULL}};
const Segment SGI256064[] = {{&S11639[0], 13}, {&S11639[16], 31}, {NULL}};
const Segment SGI256065[] = {{&S11639[0], 47}, {NULL}};
const Segment SGI256066[] = {{&S11640[0], 33}, {NULL}};
const Segment SGI256067[] = {{&S11641[47], 32}, {NULL}};
const Segment SGI256068[] = {{&S11641[0], 79}, {NULL}};
const Segment SGI256069[] = {{&S11642[101], 32}, {NULL}};
const Segment SGI256070[] = {{&S11642[0], 133}, {NULL}};
const Segment SGI256071[] = {{&S11643[25], 14}, {&S11643[40], 32}, {NULL}};
const Segment SGI256072[] = {{&S11643[0], 72}, {NULL}};
const Segment SGI256073[] = {{&S11644[25], 14}, {NULL}};
const Segment SGI256074[] = {{&S11644[0], 39}, {NULL}};
const Segment SGI256075[] = {{&S11645[25], 14}, {&S11645[40], 32}, {NULL}};
const Segment SGI256076[] = {{&S11645[0], 86}, {NULL}};
const Segment SGI256077[] = {{&S11646[36], 10}, {NULL}};
const Segment SGI256078[] = {{&S11646[0], 46}, {NULL}};
const Segment SGI256079[] = {{&S11647[0], 13}, {&S11647[14], 24}, {&S11647[39], 22}, {NULL}};
const Segment SGI256080[] = {{&S11647[0], 61}, {NULL}};
const Segment SGI256081[] = {{&S11648[0], 9}, {NULL}};
const Segment SGI256082[] = {{&S11648[0], 13}, {NULL}};
const Segment SGI256083[] = {{&S11649[0], 19}, {NULL}};
const Segment SGI256084[] = {{&S11649[0], 33}, {NULL}};
const Segment SGI256085[] = {{&S11650[0], 7}, {&S11650[21], 10}, {&S11650[32], 7}, {NULL}};
const Segment SGI256086[] = {{&S11650[0], 39}, {NULL}};
const Segment SGI256087[] = {{&S11651[0], 9}, {NULL}};
const Segment SGI256088[] = {{&S11651[0], 13}, {NULL}};
const Segment SGI256089[] = {{&S11652[0], 13}, {&S11652[14], 32}, {NULL}};
const Segment SGI256090[] = {{&S11652[0], 46}, {NULL}};
const Segment SGI256091[] = {{&S11653[25], 45}, {NULL}};
const Segment SGI256092[] = {{&S11653[0], 70}, {NULL}};
const Segment SGI256093[] = {{&S11654[21], 61}, {NULL}};
const Segment SGI256094[] = {{&S11654[0], 82}, {NULL}};
const Segment SGI256095[] = {{&S11655[25], 58}, {&S11655[84], 48}, {NULL}};
const Segment SGI256096[] = {{&S11655[0], 132}, {NULL}};
const Segment SGI256097[] = {{&S11656[25], 46}, {NULL}};
const Segment SGI256098[] = {{&S11656[0], 71}, {NULL}};
const Segment SGI256099[] = {{&S11657[24], 14}, {&S11657[39], 32}, {NULL}};
const Segment SGI256100[] = {{&S11657[0], 71}, {NULL}};
const Segment SGI256101[] = {{&S11658[0], 49}, {NULL}};
const Segment SGI256102[] = {{&S11659[0], 14}, {&S11659[15], 32}, {NULL}};
const Segment SGI256103[] = {{&S11659[0], 47}, {NULL}};
const Segment SGI256104[] = {{&S11660[0], 17}, {NULL}};
const Segment SGI256105[] = {{&S11661[0], 13}, {&S11661[14], 32}, {NULL}};
const Segment SGI256106[] = {{&S11661[0], 46}, {NULL}};
const Segment SGI256107[] = {{&S11662[0], 9}, {NULL}};
const Segment SGI256108[] = {{&S11662[0], 33}, {NULL}};
const Segment SGI256109[] = {{&S11663[25], 14}, {&S11663[40], 36}, {NULL}};
const Segment SGI256110[] = {{&S11663[0], 76}, {NULL}};
const Segment SGI256111[] = {{&S11664[27], 14}, {&S11664[42], 32}, {NULL}};
const Segment SGI256112[] = {{&S11664[0], 74}, {NULL}};
const Segment SGI256113[] = {{&S11665[56], 35}, {NULL}};
const Segment SGI256114[] = {{&S11665[0], 91}, {NULL}};
const Segment SGI256115[] = {{&S11666[25], 14}, {NULL}};
const Segment SGI256116[] = {{&S11666[0], 39}, {NULL}};
const Segment SGI256117[] = {{&S11667[0], 46}, {NULL}};
const Segment SGI256118[] = {{&S11668[55], 35}, {NULL}};
const Segment SGI256119[] = {{&S11668[0], 90}, {NULL}};
const Segment SGI256120[] = {{&S23128[13], 4}, {NULL}};
const Segment SGI256121[] = {{&S23128[33], 3}, {NULL}};
const Segment SGI256122[] = {{&S23128[64], 6}, {NULL}};
const Segment SGI256123[] = {{&S23128[98], 5}, {NULL}};
const Segment SGI256124[] = {{&S23129[13], 4}, {&S23129[13], 4}, {&S23129[13], 4}, {NULL}};
const Segment SGI256125[] = {{&S23129[42], 3}, {NULL}};
const Segment SGI256126[] = {{&S23130[13], 4}, {NULL}};
const Segment SGI256127[] = {{&S23130[33], 5}, {NULL}};
const Segment SGI256128[] = {{&S23130[66], 6}, {NULL}};
const Segment SGI256129[] = {{&S23131[13], 4}, {&S23131[13], 4}, {&S23131[13], 4}, {NULL}};
const Segment SGI256130[] = {{&S23131[42], 5}, {NULL}};
const Segment SGI256131[] = {{&S23132[13], 4}, {NULL}};
const Segment SGI256132[] = {{&S23132[33], 5}, {NULL}};
const Segment SGI256133[] = {{&S23132[66], 6}, {NULL}};
const Segment SGI256134[] = {{&S23132[100], 5}, {NULL}};
const Segment SGI256135[] = {{&S23133[13], 4}, {&S23133[13], 4}, {NULL}};
const Segment SGI256136[] = {{&S23133[33], 3}, {NULL}};
const Segment SGI256137[] = {{&S23133[64], 6}, {NULL}};
const Segment SGI256138[] = {{&S23133[98], 5}, {NULL}};
const Segment SGI256139[] = {{&S23134[13], 4}, {NULL}};
const Segment SGI256140[] = {{&S23134[33], 3}, {NULL}};
const Segment SGI256141[] = {{&S23134[64], 6}, {NULL}};
const Segment SGI256142[] = {{&S23135[13], 4}, {&S23135[13], 4}, {NULL}};
const Segment SGI256143[] = {{&S23135[33], 5}, {NULL}};
const Segment SGI256144[] = {{&S23135[66], 6}, {NULL}};
const Segment SGI256145[] = {{&S23135[100], 5}, {NULL}};
const Segment SGI256146[] = {{&S23136[13], 4}, {NULL}};
const Segment SGI256147[] = {{&S23136[130], 3}, {NULL}};
const Segment SGI256148[] = {{&S23136[33], 5}, {NULL}};
const Segment SGI256149[] = {{&S23136[66], 6}, {NULL}};
const Segment SGI256150[] = {{&S23136[100], 5}, {NULL}};
const Segment SGI256151[] = {{&S23137[13], 4}, {&S23137[13], 4}, {&S23137[13], 4}, {NULL}};
const Segment SGI256152[] = {{&S23137[33], 5}, {NULL}};
const Segment SGI256153[] = {{&S23137[66], 6}, {NULL}};
const Segment SGI256154[] = {{&S23137[100], 5}, {NULL}};
const Segment SGI256155[] = {{&S23138[13], 4}, {NULL}};
const Segment SGI256156[] = {{&S23138[44], 5}, {NULL}};
const Segment SGI256157[] = {{&S23138[126], 5}, {NULL}};
const Segment SGI256158[] = {{&S23139[13], 4}, {&S23139[13], 4}, {&S23139[13], 4}, {NULL}};
const Segment SGI256159[] = {{&S23139[33], 3}, {NULL}};
const Segment SGI256160[] = {{&S23139[64], 6}, {NULL}};
const Segment SGI256161[] = {{&S23139[98], 5}, {NULL}};
const Segment SGI256162[] = {{&S23140[13], 4}, {NULL}};
const Segment SGI256163[] = {{&S23140[33], 3}, {NULL}};
const Segment SGI256164[] = {{&S23140[64], 6}, {NULL}};
const Segment SGI256165[] = {{&S23140[98], 5}, {NULL}};
const Segment SGI256166[] = {{&S23141[13], 4}, {NULL}};
const Segment SGI256167[] = {{&S23141[128], 3}, {NULL}};
const Segment SGI256168[] = {{&S23141[33], 3}, {NULL}};
const Segment SGI256169[] = {{&S23141[64], 6}, {NULL}};
const Segment SGI256170[] = {{&S23141[98], 5}, {NULL}};
const Segment SGI256171[] = {{&S23142[13], 6}, {&S23142[128], 4}, {NULL}};
const Segment SGI256172[] = {{&S23142[75], 6}, {NULL}};
const Segment SGI256173[] = {{&S23142[109], 4}, {NULL}};
const Segment SGI256174[] = {{&S23143[13], 4}, {NULL}};
const Segment SGI256175[] = {{&S23143[33], 5}, {NULL}};
const Segment SGI256176[] = {{&S23143[66], 6}, {NULL}};
const Segment SGI256177[] = {{&S23143[100], 5}, {NULL}};
const Segment SGI256178[] = {{&S23143[113], 9}, {NULL}};
const Segment SGI256179[] = {{&S23144[15], 4}, {NULL}};
const Segment SGI256180[] = {{&S23144[37], 5}, {NULL}};
const Segment SGI256181[] = {{&S5322[22], 12}, {NULL}};
const Segment SGI256182[] = {{&S23300[13], 6}, {NULL}};
const Segment SGI256183[] = {{&S23300[112], 5}, {NULL}};
const Segment SGI256184[] = {{&S23300[38], 5}, {NULL}};
const Segment SGI256185[] = {{&S23300[78], 5}, {NULL}};
const Segment SGI256186[] = {{&S23300[125], 5}, {NULL}};
const Segment SGI256187[] = {{&S23300[138], 9}, {NULL}};
const Segment SGI256188[] = {{&S23301[13], 6}, {NULL}};
const Segment SGI256189[] = {{&S23301[112], 5}, {NULL}};
const Segment SGI256190[] = {{&S23301[38], 5}, {NULL}};
const Segment SGI256191[] = {{&S23301[78], 5}, {NULL}};
const Segment SGI256192[] = {{&S23301[125], 5}, {NULL}};
const Segment SGI256193[] = {{&S23301[138], 9}, {NULL}};
const Segment SGI256194[] = {{&S23302[13], 6}, {NULL}};
const Segment SGI256195[] = {{&S23302[112], 5}, {NULL}};
const Segment SGI256196[] = {{&S23302[38], 5}, {NULL}};
const Segment SGI256197[] = {{&S23302[78], 5}, {NULL}};
const Segment SGI256198[] = {{&S23302[125], 6}, {NULL}};
const Segment SGI256199[] = {{&S23302[139], 9}, {NULL}};
const Segment SGI256200[] = {{&S23303[13], 6}, {NULL}};
const Segment SGI256201[] = {{&S23303[110], 5}, {NULL}};
const Segment SGI256202[] = {{&S23303[38], 3}, {NULL}};
const Segment SGI256203[] = {{&S23303[76], 5}, {NULL}};
const Segment SGI256204[] = {{&S23303[123], 5}, {NULL}};
const Segment SGI256205[] = {{&S23303[136], 9}, {NULL}};
const Segment SGI256206[] = {{&S23304[13], 6}, {NULL}};
const Segment SGI256207[] = {{&S23304[112], 5}, {NULL}};
const Segment SGI256208[] = {{&S23304[38], 5}, {NULL}};
const Segment SGI256209[] = {{&S23304[78], 5}, {NULL}};
const Segment SGI256210[] = {{&S23304[125], 6}, {NULL}};
const Segment SGI256211[] = {{&S23304[139], 9}, {NULL}};
const Segment SGI256212[] = {{&S23305[13], 6}, {NULL}};
const Segment SGI256213[] = {{&S23305[110], 5}, {NULL}};
const Segment SGI256214[] = {{&S23305[38], 3}, {NULL}};
const Segment SGI256215[] = {{&S23305[76], 5}, {NULL}};
const Segment SGI256216[] = {{&S23305[123], 5}, {NULL}};
const Segment SGI256217[] = {{&S23306[13], 6}, {NULL}};
const Segment SGI256218[] = {{&S23306[110], 5}, {NULL}};
const Segment SGI256219[] = {{&S23306[38], 3}, {NULL}};
const Segment SGI256220[] = {{&S23306[76], 5}, {NULL}};
const Segment SGI256221[] = {{&S23306[123], 5}, {NULL}};
const Segment SGI256222[] = {{&S23306[136], 9}, {NULL}};
const Segment SGI256223[] = {{&S23307[13], 6}, {NULL}};
const Segment SGI256224[] = {{&S23307[112], 5}, {NULL}};
const Segment SGI256225[] = {{&S23307[38], 5}, {NULL}};
const Segment SGI256226[] = {{&S23307[78], 5}, {NULL}};
const Segment SGI256227[] = {{&S23307[125], 5}, {NULL}};
const Segment SGI256228[] = {{&S23307[138], 9}, {NULL}};
const Segment SGI256229[] = {{&S23308[13], 6}, {NULL}};
const Segment SGI256230[] = {{&S23308[111], 3}, {NULL}};
const Segment SGI256231[] = {{&S23308[38], 3}, {NULL}};
const Segment SGI256232[] = {{&S23308[76], 6}, {NULL}};
const Segment SGI256233[] = {{&S23308[122], 5}, {NULL}};
const Segment SGI256234[] = {{&S23308[135], 6}, {NULL}};
const Segment SGI256235[] = {{&S23309[13], 6}, {NULL}};
const Segment SGI256236[] = {{&S23309[110], 5}, {NULL}};
const Segment SGI256237[] = {{&S23309[38], 3}, {NULL}};
const Segment SGI256238[] = {{&S23309[76], 5}, {NULL}};
const Segment SGI256239[] = {{&S23309[123], 5}, {NULL}};
const Segment SGI256240[] = {{&S23309[136], 8}, {NULL}};
const Segment SGI256241[] = {{&S23310[13], 6}, {NULL}};
const Segment SGI256242[] = {{&S23310[113], 5}, {NULL}};
const Segment SGI256243[] = {{&S23310[38], 3}, {NULL}};
const Segment SGI256244[] = {{&S23310[76], 8}, {NULL}};
const Segment SGI256245[] = {{&S23310[126], 5}, {NULL}};
const Segment SGI256246[] = {{&S23310[139], 9}, {NULL}};
const Segment SGI256247[] = {{&S23311[13], 6}, {NULL}};
const Segment SGI256248[] = {{&S23311[110], 4}, {NULL}};
const Segment SGI256249[] = {{&S23311[38], 3}, {NULL}};
const Segment SGI256250[] = {{&S23311[76], 5}, {NULL}};
const Segment SGI256251[] = {{&S23311[122], 5}, {NULL}};
const Segment SGI256252[] = {{&S23311[135], 9}, {NULL}};
const Segment SGI256253[] = {{&S23312[13], 6}, {NULL}};
const Segment SGI256254[] = {{&S23312[112], 5}, {NULL}};
const Segment SGI256255[] = {{&S23312[38], 5}, {NULL}};
const Segment SGI256256[] = {{&S23312[78], 5}, {NULL}};
const Segment SGI256257[] = {{&S23312[125], 5}, {NULL}};
const Segment SGI256258[] = {{&S23312[138], 9}, {NULL}};
const Segment SGI256259[] = {{&S23313[13], 6}, {NULL}};
const Segment SGI256260[] = {{&S23313[111], 3}, {NULL}};
const Segment SGI256261[] = {{&S23313[38], 3}, {NULL}};
const Segment SGI256262[] = {{&S23313[76], 6}, {NULL}};
const Segment SGI256263[] = {{&S23313[122], 5}, {NULL}};
const Segment SGI256264[] = {{&S23313[135], 6}, {NULL}};
const Segment SGI256265[] = {{&S23314[13], 6}, {NULL}};
const Segment SGI256266[] = {{&S23314[112], 5}, {NULL}};
const Segment SGI256267[] = {{&S23314[38], 5}, {NULL}};
const Segment SGI256268[] = {{&S23314[78], 5}, {NULL}};
const Segment SGI256269[] = {{&S23314[125], 5}, {NULL}};
const Segment SGI256270[] = {{&S23314[138], 9}, {NULL}};
const Segment SGI256271[] = {{&S23315[13], 6}, {NULL}};
const Segment SGI256272[] = {{&S23315[110], 5}, {NULL}};
const Segment SGI256273[] = {{&S23315[38], 3}, {NULL}};
const Segment SGI256274[] = {{&S23315[76], 5}, {NULL}};
const Segment SGI256275[] = {{&S23315[123], 5}, {NULL}};
const Segment SGI256276[] = {{&S23315[136], 9}, {NULL}};
const Segment SGI256277[] = {{&S23316[13], 6}, {NULL}};
const Segment SGI256278[] = {{&S23316[131], 5}, {NULL}};
const Segment SGI256279[] = {{&S23316[38], 3}, {NULL}};
const Segment SGI256280[] = {{&S23316[76], 6}, {NULL}};
const Segment SGI256281[] = {{&S23317[13], 6}, {NULL}};
const Segment SGI256282[] = {{&S23317[111], 3}, {NULL}};
const Segment SGI256283[] = {{&S23317[38], 3}, {NULL}};
const Segment SGI256284[] = {{&S23317[76], 5}, {NULL}};
const Segment SGI256285[] = {{&S23317[122], 5}, {NULL}};
const Segment SGI256286[] = {{&S23317[135], 9}, {NULL}};
const Segment SGI256287[] = {{&S23318[13], 6}, {NULL}};
const Segment SGI256288[] = {{&S23318[140], 5}, {NULL}};
const Segment SGI256289[] = {{&S23318[38], 5}, {NULL}};
const Segment SGI256290[] = {{&S23318[78], 5}, {NULL}};
const Segment SGI256291[] = {{&S23318[153], 5}, {NULL}};
const Segment SGI256292[] = {{&S23318[166], 9}, {NULL}};
const Segment SGI256293[] = {{&S23319[13], 6}, {NULL}};
const Segment SGI256294[] = {{&S23319[140], 5}, {NULL}};
const Segment SGI256295[] = {{&S23319[38], 5}, {NULL}};
const Segment SGI256296[] = {{&S23319[78], 5}, {NULL}};
const Segment SGI256297[] = {{&S23319[153], 5}, {NULL}};
const Segment SGI256298[] = {{&S23319[166], 9}, {NULL}};
const Segment SGI256299[] = {{&S23320[13], 6}, {NULL}};
const Segment SGI256300[] = {{&S23320[108], 5}, {NULL}};
const Segment SGI256301[] = {{&S23320[38], 2}, {NULL}};
const Segment SGI256302[] = {{&S23320[75], 5}, {NULL}};
const Segment SGI256303[] = {{&S23320[121], 5}, {NULL}};
const Segment SGI256304[] = {{&S23320[134], 9}, {NULL}};
const Segment SGI256305[] = {{&S23321[13], 6}, {NULL}};
const Segment SGI256306[] = {{&S23321[109], 5}, {NULL}};
const Segment SGI256307[] = {{&S23321[38], 3}, {NULL}};
const Segment SGI256308[] = {{&S23321[76], 5}, {NULL}};
const Segment SGI256309[] = {{&S23321[122], 5}, {NULL}};
const Segment SGI256310[] = {{&S23321[135], 9}, {NULL}};
const Segment SGI256311[] = {{&S23335[13], 6}, {NULL}};
const Segment SGI256312[] = {{&S23335[35], 5}, {NULL}};
const Segment SGI256313[] = {{&S23335[68], 6}, {NULL}};
const Segment SGI256314[] = {{&S23335[122], 6}, {NULL}};
const Segment SGI256315[] = {{&S23335[136], 7}, {NULL}};
const Segment SGI256316[] = {{&S23336[13], 6}, {NULL}};
const Segment SGI256317[] = {{&S23336[35], 3}, {NULL}};
const Segment SGI256318[] = {{&S23336[66], 6}, {NULL}};
const Segment SGI256319[] = {{&S23336[120], 6}, {NULL}};
const Segment SGI256320[] = {{&S23336[134], 7}, {NULL}};
const Segment SGI256321[] = {{&S23337[13], 6}, {NULL}};
const Segment SGI256322[] = {{&S23337[35], 3}, {NULL}};
const Segment SGI256323[] = {{&S23337[66], 6}, {NULL}};
const Segment SGI256324[] = {{&S23337[120], 6}, {NULL}};
const Segment SGI256325[] = {{&S23337[134], 7}, {NULL}};
const Segment SGI256326[] = {{&S23338[13], 6}, {NULL}};
const Segment SGI256327[] = {{&S23338[35], 3}, {NULL}};
const Segment SGI256328[] = {{&S23338[66], 6}, {NULL}};
const Segment SGI256329[] = {{&S23338[120], 6}, {NULL}};
const Segment SGI256330[] = {{&S23338[134], 7}, {NULL}};
const Segment SGI256331[] = {{&S23339[13], 6}, {NULL}};
const Segment SGI256332[] = {{&S23339[35], 5}, {NULL}};
const Segment SGI256333[] = {{&S23339[68], 6}, {NULL}};
const Segment SGI256334[] = {{&S23339[122], 6}, {NULL}};
const Segment SGI256335[] = {{&S23339[136], 7}, {NULL}};
const Segment SGI256336[] = {{&S23340[13], 6}, {NULL}};
const Segment SGI256337[] = {{&S23340[35], 3}, {NULL}};
const Segment SGI256338[] = {{&S23340[66], 6}, {NULL}};
const Segment SGI256339[] = {{&S23340[120], 6}, {NULL}};
const Segment SGI256340[] = {{&S23340[134], 7}, {NULL}};
const Segment SGI256341[] = {{&S23341[13], 6}, {NULL}};
const Segment SGI256342[] = {{&S23341[35], 5}, {NULL}};
const Segment SGI256343[] = {{&S23341[68], 6}, {NULL}};
const Segment SGI256344[] = {{&S23341[122], 6}, {NULL}};
const Segment SGI256345[] = {{&S23341[136], 7}, {NULL}};
const Segment SGI256346[] = {{&S23342[13], 6}, {NULL}};
const Segment SGI256347[] = {{&S23342[35], 5}, {NULL}};
const Segment SGI256348[] = {{&S23342[68], 6}, {NULL}};
const Segment SGI256349[] = {{&S23342[122], 6}, {NULL}};
const Segment SGI256350[] = {{&S23342[136], 7}, {NULL}};
const Segment SGI256351[] = {{&S23343[13], 6}, {NULL}};
const Segment SGI256352[] = {{&S23343[35], 5}, {NULL}};
const Segment SGI256353[] = {{&S23343[68], 6}, {NULL}};
const Segment SGI256354[] = {{&S23343[122], 6}, {NULL}};
const Segment SGI256355[] = {{&S23343[136], 7}, {NULL}};
const Segment SGI256356[] = {{&S23344[13], 4}, {NULL}};
const Segment SGI256357[] = {{&S23344[99], 3}, {NULL}};
const Segment SGI256358[] = {{&S23344[33], 3}, {NULL}};
const Segment SGI256359[] = {{&S23344[64], 6}, {NULL}};
const Segment SGI256360[] = {{&S23344[110], 8}, {NULL}};
const Segment SGI256361[] = {{&S23344[126], 7}, {NULL}};
const Segment SGI256362[] = {{&S23345[13], 4}, {NULL}};
const Segment SGI256363[] = {{&S23345[99], 3}, {NULL}};
const Segment SGI256364[] = {{&S23345[33], 3}, {NULL}};
const Segment SGI256365[] = {{&S23345[64], 6}, {NULL}};
const Segment SGI256366[] = {{&S23345[110], 8}, {NULL}};
const Segment SGI256367[] = {{&S23345[126], 7}, {NULL}};
const Segment SGI256368[] = {{&S23346[13], 4}, {NULL}};
const Segment SGI256369[] = {{&S23346[99], 3}, {NULL}};
const Segment SGI256370[] = {{&S23346[33], 3}, {NULL}};
const Segment SGI256371[] = {{&S23346[64], 6}, {NULL}};
const Segment SGI256372[] = {{&S23346[110], 6}, {NULL}};
const Segment SGI256373[] = {{&S23346[124], 7}, {NULL}};
const Segment SGI256374[] = {{&S23347[13], 4}, {NULL}};
const Segment SGI256375[] = {{&S23347[99], 3}, {NULL}};
const Segment SGI256376[] = {{&S23347[33], 3}, {NULL}};
const Segment SGI256377[] = {{&S23347[64], 6}, {NULL}};
const Segment SGI256378[] = {{&S23347[110], 6}, {NULL}};
const Segment SGI256379[] = {{&S23347[124], 7}, {NULL}};
const Segment SGI256380[] = {{&S23348[13], 4}, {NULL}};
const Segment SGI256381[] = {{&S23348[101], 3}, {NULL}};
const Segment SGI256382[] = {{&S23348[33], 5}, {NULL}};
const Segment SGI256383[] = {{&S23348[66], 6}, {NULL}};
const Segment SGI256384[] = {{&S23348[112], 6}, {NULL}};
const Segment SGI256385[] = {{&S23348[126], 7}, {NULL}};
const Segment SGI256386[] = {{&S23349[13], 4}, {NULL}};
const Segment SGI256387[] = {{&S23349[101], 3}, {NULL}};
const Segment SGI256388[] = {{&S23349[33], 5}, {NULL}};
const Segment SGI256389[] = {{&S23349[66], 6}, {NULL}};
const Segment SGI256390[] = {{&S23349[112], 6}, {NULL}};
const Segment SGI256391[] = {{&S23349[126], 7}, {NULL}};
const Segment SGI256392[] = {{&S23350[13], 4}, {NULL}};
const Segment SGI256393[] = {{&S23350[101], 3}, {NULL}};
const Segment SGI256394[] = {{&S23350[33], 5}, {NULL}};
const Segment SGI256395[] = {{&S23350[66], 6}, {NULL}};
const Segment SGI256396[] = {{&S23350[112], 6}, {NULL}};
const Segment SGI256397[] = {{&S23350[126], 7}, {NULL}};
const Segment SGI256398[] = {{&S5804[10], 9}, {NULL}};
const Segment SGI256399[] = {{&S5805[10], 4}, {NULL}};
const Segment SGI256400[] = {{&S5805[15], 5}, {NULL}};
const Segment SGI256401[] = {{&S5806[25], 21}, {NULL}};
const Segment SGI256402[] = {{&S5806[47], 5}, {NULL}};
const Segment SGI256403[] = {{&S23973[13], 4}, {NULL}};
const Segment SGI256404[] = {{&S23973[26], 5}, {NULL}};
const Segment SGI256405[] = {{&S23973[59], 6}, {NULL}};
const Segment SGI256406[] = {{&S23973[112], 6}, {NULL}};
const Segment SGI256407[] = {{&S23973[126], 7}, {NULL}};
const Segment SGI256408[] = {{&S23974[13], 4}, {NULL}};
const Segment SGI256409[] = {{&S23974[26], 3}, {NULL}};
const Segment SGI256410[] = {{&S23974[57], 6}, {NULL}};
const Segment SGI256411[] = {{&S23974[110], 6}, {NULL}};
const Segment SGI256412[] = {{&S23974[124], 7}, {NULL}};
const Segment SGI256413[] = {{&S23975[13], 4}, {NULL}};
const Segment SGI256414[] = {{&S23975[26], 5}, {NULL}};
const Segment SGI256415[] = {{&S23975[59], 6}, {NULL}};
const Segment SGI256416[] = {{&S23975[112], 6}, {NULL}};
const Segment SGI256417[] = {{&S23975[126], 7}, {NULL}};
const Segment SGI256418[] = {{&S23976[13], 4}, {NULL}};
const Segment SGI256419[] = {{&S23976[26], 5}, {NULL}};
const Segment SGI256420[] = {{&S23976[59], 6}, {NULL}};
const Segment SGI256421[] = {{&S23976[112], 6}, {NULL}};
const Segment SGI256422[] = {{&S23976[126], 7}, {NULL}};
const Segment SGI256423[] = {{&S24006[13], 4}, {NULL}};
const Segment SGI256424[] = {{&S24006[26], 5}, {NULL}};
const Segment SGI256425[] = {{&S24006[66], 8}, {NULL}};
const Segment SGI256426[] = {{&S24006[121], 5}, {NULL}};
const Segment SGI256427[] = {{&S24006[134], 9}, {NULL}};
const Segment SGI256428[] = {{&S24007[13], 4}, {NULL}};
const Segment SGI256429[] = {{&S24007[26], 3}, {NULL}};
const Segment SGI256430[] = {{&S24007[64], 8}, {NULL}};
const Segment SGI256431[] = {{&S24007[119], 5}, {NULL}};
const Segment SGI256432[] = {{&S24007[132], 9}, {NULL}};
const Segment SGI256433[] = {{&S24008[13], 4}, {NULL}};
const Segment SGI256434[] = {{&S24008[26], 3}, {NULL}};
const Segment SGI256435[] = {{&S24008[64], 8}, {NULL}};
const Segment SGI256436[] = {{&S24008[119], 5}, {NULL}};
const Segment SGI256437[] = {{&S24008[132], 9}, {NULL}};
const Segment SGI256438[] = {{&S24009[13], 4}, {NULL}};
const Segment SGI256439[] = {{&S24009[26], 5}, {NULL}};
const Segment SGI256440[] = {{&S24009[66], 8}, {NULL}};
const Segment SGI256441[] = {{&S24009[121], 5}, {NULL}};
const Segment SGI256442[] = {{&S24009[134], 9}, {NULL}};
const Segment SGI256443[] = {{&S24112[13], 6}, {NULL}};
const Segment SGI256444[] = {{&S24112[110], 5}, {NULL}};
const Segment SGI256445[] = {{&S24112[38], 3}, {NULL}};
const Segment SGI256446[] = {{&S24112[76], 5}, {NULL}};
const Segment SGI256447[] = {{&S24112[123], 5}, {NULL}};
const Segment SGI256448[] = {{&S24112[136], 9}, {NULL}};
const Segment SGI256449[] = {{&S24113[13], 6}, {NULL}};
const Segment SGI256450[] = {{&S24113[110], 5}, {NULL}};
const Segment SGI256451[] = {{&S24113[38], 3}, {NULL}};
const Segment SGI256452[] = {{&S24113[76], 5}, {NULL}};
const Segment SGI256453[] = {{&S24113[123], 5}, {NULL}};
const Segment SGI256454[] = {{&S24113[136], 9}, {NULL}};
const Segment SGI256455[] = {{&S24114[13], 6}, {NULL}};
const Segment SGI256456[] = {{&S24114[110], 5}, {NULL}};
const Segment SGI256457[] = {{&S24114[38], 3}, {NULL}};
const Segment SGI256458[] = {{&S24114[76], 5}, {NULL}};
const Segment SGI256459[] = {{&S24114[123], 7}, {NULL}};
const Segment SGI256460[] = {{&S24114[138], 9}, {NULL}};
const Segment SGI256461[] = {{&S24115[13], 6}, {NULL}};
const Segment SGI256462[] = {{&S24115[110], 5}, {NULL}};
const Segment SGI256463[] = {{&S24115[38], 3}, {NULL}};
const Segment SGI256464[] = {{&S24115[76], 5}, {NULL}};
const Segment SGI256465[] = {{&S24115[123], 5}, {NULL}};
const Segment SGI256466[] = {{&S24115[136], 9}, {NULL}};
const Segment SGI256467[] = {{&S24116[13], 6}, {NULL}};
const Segment SGI256468[] = {{&S24116[110], 5}, {NULL}};
const Segment SGI256469[] = {{&S24116[38], 3}, {NULL}};
const Segment SGI256470[] = {{&S24116[76], 5}, {NULL}};
const Segment SGI256471[] = {{&S24116[123], 5}, {NULL}};
const Segment SGI256472[] = {{&S24117[13], 6}, {NULL}};
const Segment SGI256473[] = {{&S24117[111], 3}, {NULL}};
const Segment SGI256474[] = {{&S24117[38], 3}, {NULL}};
const Segment SGI256475[] = {{&S24117[76], 6}, {NULL}};
const Segment SGI256476[] = {{&S24117[122], 5}, {NULL}};
const Segment SGI256477[] = {{&S24117[135], 6}, {NULL}};
const Segment SGI256478[] = {{&S24118[13], 6}, {NULL}};
const Segment SGI256479[] = {{&S24118[110], 5}, {NULL}};
const Segment SGI256480[] = {{&S24118[38], 3}, {NULL}};
const Segment SGI256481[] = {{&S24118[76], 5}, {NULL}};
const Segment SGI256482[] = {{&S24118[123], 5}, {NULL}};
const Segment SGI256483[] = {{&S24118[136], 9}, {NULL}};
const Segment SGI256484[] = {{&S24119[13], 6}, {NULL}};
const Segment SGI256485[] = {{&S24119[110], 5}, {NULL}};
const Segment SGI256486[] = {{&S24119[38], 3}, {NULL}};
const Segment SGI256487[] = {{&S24119[76], 5}, {NULL}};
const Segment SGI256488[] = {{&S24119[123], 5}, {NULL}};
const Segment SGI256489[] = {{&S24119[136], 9}, {NULL}};
const Segment SGI256490[] = {{&S24120[13], 6}, {NULL}};
const Segment SGI256491[] = {{&S24120[110], 5}, {NULL}};
const Segment SGI256492[] = {{&S24120[38], 3}, {NULL}};
const Segment SGI256493[] = {{&S24120[76], 5}, {NULL}};
const Segment SGI256494[] = {{&S24120[138], 9}, {NULL}};
const Segment SGI256495[] = {{&S24121[13], 6}, {NULL}};
const Segment SGI256496[] = {{&S24121[108], 5}, {NULL}};
const Segment SGI256497[] = {{&S24121[38], 2}, {NULL}};
const Segment SGI256498[] = {{&S24121[75], 5}, {NULL}};
const Segment SGI256499[] = {{&S24121[121], 5}, {NULL}};
const Segment SGI256500[] = {{&S24121[134], 9}, {NULL}};
const Segment SGI256501[] = {{&S24122[13], 4}, {NULL}};
const Segment SGI256502[] = {{&S24122[36], 5}, {NULL}};
const Segment SGI256503[] = {{&S24122[76], 8}, {NULL}};
const Segment SGI256504[] = {{&S24123[13], 4}, {NULL}};
const Segment SGI256505[] = {{&S24123[36], 5}, {NULL}};
const Segment SGI256506[] = {{&S24123[76], 8}, {NULL}};
const Segment SGI256507[] = {{&S24123[112], 3}, {NULL}};
const Segment SGI256508[] = {{&S24124[13], 4}, {NULL}};
const Segment SGI256509[] = {{&S24124[36], 5}, {NULL}};
const Segment SGI256510[] = {{&S24124[76], 8}, {NULL}};
const Segment SGI256511[] = {{&S24124[112], 3}, {NULL}};
const Segment SGI256512[] = {{&S24125[13], 4}, {NULL}};
const Segment SGI256513[] = {{&S24125[36], 3}, {NULL}};
const Segment SGI256514[] = {{&S24125[74], 8}, {NULL}};
const Segment SGI256515[] = {{&S24126[13], 4}, {NULL}};
const Segment SGI256516[] = {{&S24126[36], 5}, {NULL}};
const Segment SGI256517[] = {{&S24126[76], 8}, {NULL}};
const Segment SGI256518[] = {{&S24127[13], 4}, {NULL}};
const Segment SGI256519[] = {{&S24127[36], 3}, {NULL}};
const Segment SGI256520[] = {{&S24127[74], 8}, {NULL}};
const Segment SGI256521[] = {{&S24128[13], 4}, {NULL}};
const Segment SGI256522[] = {{&S24128[36], 5}, {NULL}};
const Segment SGI256523[] = {{&S24128[76], 8}, {NULL}};
const Segment SGI256524[] = {{&S24129[13], 4}, {NULL}};
const Segment SGI256525[] = {{&S24129[36], 3}, {NULL}};
const Segment SGI256526[] = {{&S24129[74], 8}, {NULL}};
const Segment SGI256527[] = {{&S24129[110], 5}, {NULL}};
const Segment SGI256528[] = {{&S24130[13], 4}, {NULL}};
const Segment SGI256529[] = {{&S24130[36], 5}, {NULL}};
const Segment SGI256530[] = {{&S24130[76], 8}, {NULL}};
const Segment SGI256531[] = {{&S24130[112], 3}, {NULL}};
const Segment SGI256532[] = {{&S24131[13], 4}, {NULL}};
const Segment SGI256533[] = {{&S24131[36], 5}, {NULL}};
const Segment SGI256534[] = {{&S24131[76], 8}, {NULL}};
const Segment SGI256535[] = {{&S24132[13], 4}, {NULL}};
const Segment SGI256536[] = {{&S24132[36], 5}, {NULL}};
const Segment SGI256537[] = {{&S24132[76], 8}, {NULL}};
const Segment SGI256538[] = {{&S24132[112], 3}, {NULL}};
const Segment SGI256539[] = {{&S24133[13], 4}, {NULL}};
const Segment SGI256540[] = {{&S24133[36], 5}, {NULL}};
const Segment SGI256541[] = {{&S24133[76], 8}, {NULL}};
const Segment SGI256542[] = {{&S24134[13], 4}, {NULL}};
const Segment SGI256543[] = {{&S24134[36], 5}, {NULL}};
const Segment SGI256544[] = {{&S24134[76], 8}, {NULL}};
const Segment SGI256545[] = {{&S24134[112], 3}, {NULL}};
const Segment SGI256546[] = {{&S24135[13], 4}, {NULL}};
const Segment SGI256547[] = {{&S24135[36], 5}, {NULL}};
const Segment SGI256548[] = {{&S24135[76], 8}, {NULL}};
const Segment SGI256549[] = {{&S24136[13], 4}, {NULL}};
const Segment SGI256550[] = {{&S24136[36], 5}, {NULL}};
const Segment SGI256551[] = {{&S24136[76], 8}, {NULL}};
const Segment SGI256552[] = {{&S24136[112], 3}, {NULL}};
const Segment SGI256553[] = {{&S24137[13], 4}, {&S24137[13], 4}, {&S24137[13], 4}, {NULL}};
const Segment SGI256554[] = {{&S24137[42], 5}, {NULL}};
const Segment SGI256555[] = {{&S24138[13], 4}, {&S24138[13], 4}, {&S24138[13], 4}, {NULL}};
const Segment SGI256556[] = {{&S24138[42], 3}, {NULL}};
const Segment SGI256557[] = {{&S24139[13], 4}, {&S24139[13], 4}, {&S24139[13], 4}, {NULL}};
const Segment SGI256558[] = {{&S24139[42], 5}, {NULL}};
const Segment SGI256559[] = {{&S24140[13], 4}, {&S24140[13], 4}, {&S24140[13], 4}, {NULL}};
const Segment SGI256560[] = {{&S24140[42], 5}, {NULL}};
const Segment SGI256561[] = {{&S24141[13], 4}, {&S24141[13], 4}, {&S24141[13], 4}, {NULL}};
const Segment SGI256562[] = {{&S24141[42], 5}, {NULL}};
const Segment SGI256563[] = {{&S24142[13], 4}, {&S24142[13], 4}, {NULL}};
const Segment SGI256564[] = {{&S24142[36], 5}, {NULL}};
const Segment SGI256565[] = {{&S24142[76], 8}, {NULL}};
const Segment SGI256566[] = {{&S24142[112], 3}, {NULL}};
const Segment SGI256567[] = {{&S24143[13], 4}, {NULL}};
const Segment SGI256568[] = {{&S24143[36], 5}, {NULL}};
const Segment SGI256569[] = {{&S24143[76], 8}, {NULL}};
const Segment SGI256570[] = {{&S24143[112], 3}, {NULL}};
const Segment SGI256571[] = {{&S24144[13], 4}, {&S24144[13], 4}, {&S24144[13], 4}, {NULL}};
const Segment SGI256572[] = {{&S24144[42], 5}, {NULL}};
const Segment SGI256573[] = {{&S24145[13], 4}, {&S24145[13], 4}, {NULL}};
const Segment SGI256574[] = {{&S24145[36], 5}, {NULL}};
const Segment SGI256575[] = {{&S24145[76], 8}, {NULL}};
const Segment SGI256576[] = {{&S24145[112], 3}, {NULL}};
const Segment SGI256577[] = {{&S24146[13], 4}, {&S24146[13], 4}, {NULL}};
const Segment SGI256578[] = {{&S24146[36], 5}, {NULL}};
const Segment SGI256579[] = {{&S24146[76], 8}, {NULL}};
const Segment SGI256580[] = {{&S24146[112], 3}, {NULL}};
const Segment SGI256581[] = {{&S24147[13], 4}, {&S24147[13], 4}, {NULL}};
const Segment SGI256582[] = {{&S24147[36], 5}, {NULL}};
const Segment SGI256583[] = {{&S24147[76], 8}, {NULL}};
const Segment SGI256584[] = {{&S24147[112], 3}, {NULL}};
const Segment SGI256585[] = {{&S24148[13], 4}, {&S24148[13], 4}, {&S24148[13], 4}, {NULL}};
const Segment SGI256586[] = {{&S24148[36], 5}, {NULL}};
const Segment SGI256587[] = {{&S24148[76], 8}, {NULL}};
const Segment SGI256588[] = {{&S24148[112], 3}, {NULL}};
const Segment SGI256589[] = {{&S24149[13], 4}, {&S24149[13], 4}, {&S24149[13], 4}, {NULL}};
const Segment SGI256590[] = {{&S24149[36], 3}, {NULL}};
const Segment SGI256591[] = {{&S24149[74], 8}, {NULL}};
const Segment SGI256592[] = {{&S24149[110], 5}, {NULL}};
const Segment SGI256593[] = {{&S24150[13], 4}, {&S24150[13], 4}, {&S24150[13], 4}, {NULL}};
const Segment SGI256594[] = {{&S24150[36], 5}, {NULL}};
const Segment SGI256595[] = {{&S24150[76], 8}, {NULL}};
const Segment SGI256596[] = {{&S24150[112], 3}, {NULL}};
const Segment SGI256597[] = {{&S24151[13], 4}, {&S24151[13], 4}, {&S24151[13], 4}, {NULL}};
const Segment SGI256598[] = {{&S24151[36], 5}, {NULL}};
const Segment SGI256599[] = {{&S24151[76], 8}, {NULL}};
const Segment SGI256600[] = {{&S24151[112], 3}, {NULL}};
const Segment SGI256601[] = {{&S24152[13], 4}, {&S24152[13], 4}, {&S24152[13], 4}, {NULL}};
const Segment SGI256602[] = {{&S24152[36], 5}, {NULL}};
const Segment SGI256603[] = {{&S24152[76], 8}, {NULL}};
const Segment SGI256604[] = {{&S24152[112], 3}, {NULL}};
const Segment SGI256605[] = {{&S24153[13], 4}, {&S24153[13], 4}, {NULL}};
const Segment SGI256606[] = {{&S24153[36], 3}, {NULL}};
const Segment SGI256607[] = {{&S24153[74], 8}, {NULL}};
const Segment SGI256608[] = {{&S24153[110], 5}, {NULL}};
const Segment SGI256609[] = {{&S24154[13], 4}, {&S24154[13], 4}, {NULL}};
const Segment SGI256610[] = {{&S24154[36], 5}, {NULL}};
const Segment SGI256611[] = {{&S24154[76], 8}, {NULL}};
const Segment SGI256612[] = {{&S24154[112], 3}, {NULL}};
const Segment SGI256613[] = {{&S24155[13], 4}, {&S24155[13], 4}, {&S24155[13], 4}, {NULL}};
const Segment SGI256614[] = {{&S24155[36], 5}, {NULL}};
const Segment SGI256615[] = {{&S24155[76], 8}, {NULL}};
const Segment SGI256616[] = {{&S24155[112], 3}, {NULL}};
const Segment SGI256617[] = {{&S24156[13], 4}, {NULL}};
const Segment SGI256618[] = {{&S24156[36], 2}, {NULL}};
const Segment SGI256619[] = {{&S24156[73], 8}, {NULL}};
const Segment SGI256620[] = {{&S24156[108], 5}, {NULL}};
const Segment SGI256621[] = {{&S24157[13], 4}, {NULL}};
const Segment SGI256622[] = {{&S24157[36], 3}, {NULL}};
const Segment SGI256623[] = {{&S24157[74], 8}, {NULL}};
const Segment SGI256624[] = {{&S24157[109], 3}, {NULL}};
const Segment SGI256625[] = {{&S24158[13], 4}, {NULL}};
const Segment SGI256626[] = {{&S24158[36], 3}, {NULL}};
const Segment SGI256627[] = {{&S24158[74], 8}, {NULL}};
const Segment SGI256628[] = {{&S24158[109], 3}, {NULL}};
const Segment SGI256629[] = {{&S24159[13], 6}, {&S24159[128], 4}, {NULL}};
const Segment SGI256630[] = {{&S24159[75], 6}, {NULL}};
const Segment SGI256631[] = {{&S24159[109], 4}, {NULL}};
const Segment SGI256632[] = {{&S24421[13], 4}, {NULL}};
const Segment SGI256633[] = {{&S24421[26], 5}, {NULL}};
const Segment SGI256634[] = {{&S24421[59], 6}, {NULL}};
const Segment SGI256635[] = {{&S24421[112], 6}, {NULL}};
const Segment SGI256636[] = {{&S24421[126], 7}, {NULL}};
const Segment SGI256637[] = {{&S24422[13], 4}, {NULL}};
const Segment SGI256638[] = {{&S24422[26], 5}, {NULL}};
const Segment SGI256639[] = {{&S24422[59], 6}, {NULL}};
const Segment SGI256640[] = {{&S24422[112], 6}, {NULL}};
const Segment SGI256641[] = {{&S24422[126], 7}, {NULL}};
const Segment SGI256642[] = {{&S24423[13], 4}, {NULL}};
const Segment SGI256643[] = {{&S24423[26], 3}, {NULL}};
const Segment SGI256644[] = {{&S24423[57], 6}, {NULL}};
const Segment SGI256645[] = {{&S24423[110], 6}, {NULL}};
const Segment SGI256646[] = {{&S24423[124], 7}, {NULL}};
const Segment SGI256647[] = {{&S24424[13], 4}, {NULL}};
const Segment SGI256648[] = {{&S24424[26], 5}, {NULL}};
const Segment SGI256649[] = {{&S24424[59], 6}, {NULL}};
const Segment SGI256650[] = {{&S24424[112], 6}, {NULL}};
const Segment SGI256651[] = {{&S24424[126], 7}, {NULL}};
const Segment SGI256652[] = {{&S24664[13], 4}, {NULL}};
const Segment SGI256653[] = {{&S24664[26], 3}, {NULL}};
const Segment SGI256654[] = {{&S24664[57], 6}, {NULL}};
const Segment SGI256655[] = {{&S24664[91], 7}, {NULL}};
const Segment SGI256656[] = {{&S24665[13], 4}, {NULL}};
const Segment SGI256657[] = {{&S24665[26], 3}, {NULL}};
const Segment SGI256658[] = {{&S24665[57], 6}, {NULL}};
const Segment SGI256659[] = {{&S24665[91], 7}, {NULL}};
const Segment SGI256660[] = {{&S24666[13], 4}, {NULL}};
const Segment SGI256661[] = {{&S24666[26], 3}, {NULL}};
const Segment SGI256662[] = {{&S24666[57], 6}, {NULL}};
const Segment SGI256663[] = {{&S24666[91], 7}, {NULL}};
const Segment SGI256664[] = {{&S24667[13], 4}, {NULL}};
const Segment SGI256665[] = {{&S24667[26], 3}, {NULL}};
const Segment SGI256666[] = {{&S24667[57], 6}, {NULL}};
const Segment SGI256667[] = {{&S24667[91], 5}, {NULL}};
const Segment SGI256668[] = {{&S24668[13], 4}, {&S24668[13], 4}, {NULL}};
const Segment SGI256669[] = {{&S24668[34], 3}, {NULL}};
const Segment SGI256670[] = {{&S24668[65], 6}, {NULL}};
const Segment SGI256671[] = {{&S24668[99], 5}, {NULL}};
const Segment SGI256672[] = {{&S24669[27], 4}, {NULL}};
const Segment SGI256673[] = {{&S24669[40], 3}, {NULL}};
const Segment SGI256674[] = {{&S24669[71], 6}, {NULL}};
const Segment SGI256675[] = {{&S24669[105], 5}, {NULL}};
const Segment SGI256676[] = {{&S24669[118], 9}, {NULL}};
const Segment SGI256677[] = {{&S24669[0], 14}, {NULL}};
const Segment SGI256678[] = {{&S24670[13], 4}, {NULL}};
const Segment SGI256679[] = {{&S24670[26], 5}, {NULL}};
const Segment SGI256680[] = {{&S24670[59], 6}, {NULL}};
const Segment SGI256681[] = {{&S24671[13], 4}, {NULL}};
const Segment SGI256682[] = {{&S24671[26], 5}, {NULL}};
const Segment SGI256683[] = {{&S24671[59], 6}, {NULL}};
const Segment SGI256684[] = {{&S24671[93], 5}, {NULL}};
const Segment SGI256685[] = {{&S24672[13], 4}, {NULL}};
const Segment SGI256686[] = {{&S24672[26], 3}, {NULL}};
const Segment SGI256687[] = {{&S24672[57], 6}, {NULL}};
const Segment SGI256688[] = {{&S24673[30], 4}, {NULL}};
const Segment SGI256689[] = {{&S24673[43], 5}, {NULL}};
const Segment SGI256690[] = {{&S24673[76], 6}, {NULL}};
const Segment SGI256691[] = {{&S24673[110], 5}, {NULL}};
const Segment SGI256692[] = {{&S24673[123], 9}, {NULL}};
const Segment SGI256693[] = {{&S24673[0], 14}, {NULL}};
const Segment SGI256694[] = {{&S24674[13], 4}, {NULL}};
const Segment SGI256695[] = {{&S24674[26], 5}, {NULL}};
const Segment SGI256696[] = {{&S24674[59], 6}, {NULL}};
const Segment SGI256697[] = {{&S24674[93], 5}, {NULL}};
const Segment SGI256698[] = {{&S24674[106], 6}, {NULL}};
const Segment SGI256699[] = {{&S24675[27], 4}, {NULL}};
const Segment SGI256700[] = {{&S24675[40], 5}, {NULL}};
const Segment SGI256701[] = {{&S24675[73], 6}, {NULL}};
const Segment SGI256702[] = {{&S24675[107], 5}, {NULL}};
const Segment SGI256703[] = {{&S24675[120], 9}, {NULL}};
const Segment SGI256704[] = {{&S24675[0], 14}, {NULL}};
const Segment SGI256705[] = {{&S24676[13], 4}, {&S24676[13], 4}, {&S24676[13], 4}, {NULL}};
const Segment SGI256706[] = {{&S24676[36], 5}, {NULL}};
const Segment SGI256707[] = {{&S24677[13], 4}, {&S24677[13], 4}, {&S24677[13], 4}, {NULL}};
const Segment SGI256708[] = {{&S24677[36], 3}, {NULL}};
const Segment SGI256709[] = {{&S24678[13], 4}, {NULL}};
const Segment SGI256710[] = {{&S24678[26], 5}, {NULL}};
const Segment SGI256711[] = {{&S24678[59], 6}, {NULL}};
const Segment SGI256712[] = {{&S24678[93], 4}, {NULL}};
const Segment SGI256713[] = {{&S24679[27], 4}, {NULL}};
const Segment SGI256714[] = {{&S24679[40], 5}, {NULL}};
const Segment SGI256715[] = {{&S24679[73], 6}, {NULL}};
const Segment SGI256716[] = {{&S24679[107], 5}, {NULL}};
const Segment SGI256717[] = {{&S24679[120], 9}, {NULL}};
const Segment SGI256718[] = {{&S24679[0], 14}, {NULL}};
const Segment SGI256719[] = {{&S24680[13], 4}, {NULL}};
const Segment SGI256720[] = {{&S24680[26], 3}, {NULL}};
const Segment SGI256721[] = {{&S24680[57], 6}, {NULL}};
const Segment SGI256722[] = {{&S24680[91], 5}, {NULL}};
const Segment SGI256723[] = {{&S24680[104], 7}, {NULL}};
const Segment SGI256724[] = {{&S24681[13], 4}, {NULL}};
const Segment SGI256725[] = {{&S24681[26], 5}, {NULL}};
const Segment SGI256726[] = {{&S24681[59], 6}, {NULL}};
const Segment SGI256727[] = {{&S24681[93], 5}, {NULL}};
const Segment SGI256728[] = {{&S24682[30], 4}, {NULL}};
const Segment SGI256729[] = {{&S24682[43], 3}, {NULL}};
const Segment SGI256730[] = {{&S24682[74], 6}, {NULL}};
const Segment SGI256731[] = {{&S24682[108], 5}, {NULL}};
const Segment SGI256732[] = {{&S24682[121], 9}, {NULL}};
const Segment SGI256733[] = {{&S24682[0], 14}, {NULL}};
const Segment SGI256734[] = {{&S24683[27], 4}, {NULL}};
const Segment SGI256735[] = {{&S24683[40], 5}, {NULL}};
const Segment SGI256736[] = {{&S24683[73], 6}, {NULL}};
const Segment SGI256737[] = {{&S24683[107], 5}, {NULL}};
const Segment SGI256738[] = {{&S24683[0], 14}, {NULL}};
const Segment SGI256739[] = {{&S24684[13], 4}, {&S24684[13], 4}, {&S24684[13], 4}, {NULL}};
const Segment SGI256740[] = {{&S24684[26], 5}, {NULL}};
const Segment SGI256741[] = {{&S24684[59], 6}, {NULL}};
const Segment SGI256742[] = {{&S24684[93], 5}, {NULL}};
const Segment SGI256743[] = {{&S24685[13], 4}, {&S24685[13], 4}, {&S24685[13], 4}, {NULL}};
const Segment SGI256744[] = {{&S24685[26], 3}, {NULL}};
const Segment SGI256745[] = {{&S24685[57], 6}, {NULL}};
const Segment SGI256746[] = {{&S24685[91], 5}, {NULL}};
const Segment SGI256747[] = {{&S24686[13], 4}, {&S24686[13], 4}, {NULL}};
const Segment SGI256748[] = {{&S24686[26], 5}, {NULL}};
const Segment SGI256749[] = {{&S24686[59], 6}, {NULL}};
const Segment SGI256750[] = {{&S24686[93], 5}, {NULL}};
const Segment SGI256751[] = {{&S24687[13], 4}, {NULL}};
const Segment SGI256752[] = {{&S24687[26], 5}, {NULL}};
const Segment SGI256753[] = {{&S24687[59], 6}, {NULL}};
const Segment SGI256754[] = {{&S24687[93], 5}, {NULL}};
const Segment SGI256755[] = {{&S24687[106], 9}, {NULL}};
const Segment SGI256756[] = {{&S24688[13], 4}, {NULL}};
const Segment SGI256757[] = {{&S24688[26], 5}, {NULL}};
const Segment SGI256758[] = {{&S24688[59], 6}, {NULL}};
const Segment SGI256759[] = {{&S24688[93], 5}, {NULL}};
const Segment SGI256760[] = {{&S24892[13], 6}, {NULL}};
const Segment SGI256761[] = {{&S24892[113], 3}, {NULL}};
const Segment SGI256762[] = {{&S24892[38], 5}, {NULL}};
const Segment SGI256763[] = {{&S24892[78], 6}, {NULL}};
const Segment SGI256764[] = {{&S24892[124], 4}, {NULL}};
const Segment SGI256765[] = {{&S24892[136], 6}, {NULL}};
const Segment SGI256766[] = {{&S24893[13], 6}, {NULL}};
const Segment SGI256767[] = {{&S24893[111], 3}, {NULL}};
const Segment SGI256768[] = {{&S24893[38], 3}, {NULL}};
const Segment SGI256769[] = {{&S24893[76], 6}, {NULL}};
const Segment SGI256770[] = {{&S24893[122], 5}, {NULL}};
const Segment SGI256771[] = {{&S24893[135], 6}, {NULL}};
const Segment SGI256772[] = {{&S24894[13], 6}, {NULL}};
const Segment SGI256773[] = {{&S24894[113], 3}, {NULL}};
const Segment SGI256774[] = {{&S24894[38], 5}, {NULL}};
const Segment SGI256775[] = {{&S24894[78], 6}, {NULL}};
const Segment SGI256776[] = {{&S24894[124], 4}, {NULL}};
const Segment SGI256777[] = {{&S24894[136], 6}, {NULL}};
const Segment SGI256778[] = {{&S24895[13], 6}, {NULL}};
const Segment SGI256779[] = {{&S24895[111], 3}, {NULL}};
const Segment SGI256780[] = {{&S24895[38], 3}, {NULL}};
const Segment SGI256781[] = {{&S24895[76], 6}, {NULL}};
const Segment SGI256782[] = {{&S24895[122], 6}, {NULL}};
const Segment SGI256783[] = {{&S24895[136], 6}, {NULL}};
const Segment SGI256784[] = {{&S24896[13], 6}, {NULL}};
const Segment SGI256785[] = {{&S24896[113], 3}, {NULL}};
const Segment SGI256786[] = {{&S24896[38], 5}, {NULL}};
const Segment SGI256787[] = {{&S24896[78], 6}, {NULL}};
const Segment SGI256788[] = {{&S24896[124], 5}, {NULL}};
const Segment SGI256789[] = {{&S24896[137], 6}, {NULL}};
const Segment SGI256790[] = {{&S24898[13], 6}, {NULL}};
const Segment SGI256791[] = {{&S24898[113], 3}, {NULL}};
const Segment SGI256792[] = {{&S24898[38], 5}, {NULL}};
const Segment SGI256793[] = {{&S24898[78], 6}, {NULL}};
const Segment SGI256794[] = {{&S24898[124], 5}, {NULL}};
const Segment SGI256795[] = {{&S24898[137], 6}, {NULL}};
const Segment SGI256796[] = {{&S24899[13], 6}, {NULL}};
const Segment SGI256797[] = {{&S24899[110], 3}, {NULL}};
const Segment SGI256798[] = {{&S24899[38], 5}, {NULL}};
const Segment SGI256799[] = {{&S24899[75], 6}, {NULL}};
const Segment SGI256800[] = {{&S24899[121], 4}, {NULL}};
const Segment SGI256801[] = {{&S24899[133], 6}, {NULL}};
const Segment SGI256802[] = {{&S24901[13], 6}, {NULL}};
const Segment SGI256803[] = {{&S24901[112], 5}, {NULL}};
const Segment SGI256804[] = {{&S24901[38], 5}, {NULL}};
const Segment SGI256805[] = {{&S24901[78], 5}, {NULL}};
const Segment SGI256806[] = {{&S24901[125], 5}, {NULL}};
const Segment SGI256807[] = {{&S24901[138], 9}, {NULL}};
const Segment SGI256808[] = {{&S24907[13], 6}, {NULL}};
const Segment SGI256809[] = {{&S24907[111], 3}, {NULL}};
const Segment SGI256810[] = {{&S24907[38], 3}, {NULL}};
const Segment SGI256811[] = {{&S24907[76], 6}, {NULL}};
const Segment SGI256812[] = {{&S24907[122], 5}, {NULL}};
const Segment SGI256813[] = {{&S24907[135], 6}, {NULL}};
const Segment SGI256814[] = {{&S24908[13], 6}, {NULL}};
const Segment SGI256815[] = {{&S24908[113], 3}, {NULL}};
const Segment SGI256816[] = {{&S24908[38], 5}, {NULL}};
const Segment SGI256817[] = {{&S24908[78], 6}, {NULL}};
const Segment SGI256818[] = {{&S24908[124], 5}, {NULL}};
const Segment SGI256819[] = {{&S24908[137], 6}, {NULL}};
const Segment SGI256820[] = {{&S24909[13], 6}, {NULL}};
const Segment SGI256821[] = {{&S24909[110], 3}, {NULL}};
const Segment SGI256822[] = {{&S24909[38], 5}, {NULL}};
const Segment SGI256823[] = {{&S24909[75], 6}, {NULL}};
const Segment SGI256824[] = {{&S24909[121], 4}, {NULL}};
const Segment SGI256825[] = {{&S24909[133], 6}, {NULL}};
const Segment SGI256826[] = {{&S24913[13], 6}, {NULL}};
const Segment SGI256827[] = {{&S24913[113], 3}, {NULL}};
const Segment SGI256828[] = {{&S24913[38], 5}, {NULL}};
const Segment SGI256829[] = {{&S24913[78], 6}, {NULL}};
const Segment SGI256830[] = {{&S24913[124], 4}, {NULL}};
const Segment SGI256831[] = {{&S24913[136], 6}, {NULL}};
const Segment SGI256832[] = {{&S24914[13], 6}, {NULL}};
const Segment SGI256833[] = {{&S24914[110], 3}, {NULL}};
const Segment SGI256834[] = {{&S24914[38], 3}, {NULL}};
const Segment SGI256835[] = {{&S24914[76], 6}, {NULL}};
const Segment SGI256836[] = {{&S24914[121], 4}, {NULL}};
const Segment SGI256837[] = {{&S24914[133], 6}, {NULL}};
const Segment SGI256838[] = {{&S24915[13], 6}, {NULL}};
const Segment SGI256839[] = {{&S24915[110], 3}, {NULL}};
const Segment SGI256840[] = {{&S24915[38], 3}, {NULL}};
const Segment SGI256841[] = {{&S24915[76], 6}, {NULL}};
const Segment SGI256842[] = {{&S24915[121], 4}, {NULL}};
const Segment SGI256843[] = {{&S24915[133], 6}, {NULL}};
const Segment SGI256844[] = {{&S24948[13], 6}, {NULL}};
const Segment SGI256845[] = {{&S24948[35], 5}, {NULL}};
const Segment SGI256846[] = {{&S24948[68], 6}, {NULL}};
const Segment SGI256847[] = {{&S24948[121], 6}, {NULL}};
const Segment SGI256848[] = {{&S24948[135], 7}, {NULL}};
const Segment SGI256849[] = {{&S24949[13], 6}, {NULL}};
const Segment SGI256850[] = {{&S24949[35], 5}, {NULL}};
const Segment SGI256851[] = {{&S24949[68], 6}, {NULL}};
const Segment SGI256852[] = {{&S24949[121], 6}, {NULL}};
const Segment SGI256853[] = {{&S24949[135], 7}, {NULL}};
const Segment SGI256854[] = {{&S24950[13], 6}, {NULL}};
const Segment SGI256855[] = {{&S24950[35], 5}, {NULL}};
const Segment SGI256856[] = {{&S24950[68], 6}, {NULL}};
const Segment SGI256857[] = {{&S24950[121], 6}, {NULL}};
const Segment SGI256858[] = {{&S24950[135], 7}, {NULL}};
const Segment SGI256859[] = {{&S24951[13], 6}, {NULL}};
const Segment SGI256860[] = {{&S24951[35], 5}, {NULL}};
const Segment SGI256861[] = {{&S24951[68], 6}, {NULL}};
const Segment SGI256862[] = {{&S24951[121], 6}, {NULL}};
const Segment SGI256863[] = {{&S24951[135], 7}, {NULL}};
const Segment SGI256864[] = {{&S24952[13], 6}, {NULL}};
const Segment SGI256865[] = {{&S24952[35], 3}, {NULL}};
const Segment SGI256866[] = {{&S24952[66], 6}, {NULL}};
const Segment SGI256867[] = {{&S24952[119], 6}, {NULL}};
const Segment SGI256868[] = {{&S24952[133], 7}, {NULL}};
const Segment SGI256869[] = {{&S24953[13], 6}, {NULL}};
const Segment SGI256870[] = {{&S24953[35], 3}, {NULL}};
const Segment SGI256871[] = {{&S24953[66], 6}, {NULL}};
const Segment SGI256872[] = {{&S24953[119], 6}, {NULL}};
const Segment SGI256873[] = {{&S24953[133], 7}, {NULL}};
const Segment SGI256874[] = {{&S24954[13], 6}, {NULL}};
const Segment SGI256875[] = {{&S24954[35], 3}, {NULL}};
const Segment SGI256876[] = {{&S24954[66], 6}, {NULL}};
const Segment SGI256877[] = {{&S24954[119], 6}, {NULL}};
const Segment SGI256878[] = {{&S24954[133], 7}, {NULL}};
const Segment SGI256879[] = {{&S24955[13], 6}, {NULL}};
const Segment SGI256880[] = {{&S24955[35], 3}, {NULL}};
const Segment SGI256881[] = {{&S24955[66], 6}, {NULL}};
const Segment SGI256882[] = {{&S24955[119], 6}, {NULL}};
const Segment SGI256883[] = {{&S24955[133], 7}, {NULL}};
const Segment SGI256884[] = {{&S24956[13], 6}, {NULL}};
const Segment SGI256885[] = {{&S24956[35], 5}, {NULL}};
const Segment SGI256886[] = {{&S24956[68], 6}, {NULL}};
const Segment SGI256887[] = {{&S24956[121], 5}, {NULL}};
const Segment SGI256888[] = {{&S24956[134], 7}, {NULL}};
const Segment SGI256889[] = {{&S25365[13], 4}, {NULL}};
const Segment SGI256890[] = {{&S25365[26], 5}, {NULL}};
const Segment SGI256891[] = {{&S25365[59], 6}, {NULL}};
const Segment SGI256892[] = {{&S25365[113], 6}, {NULL}};
const Segment SGI256893[] = {{&S25365[127], 7}, {NULL}};
const Segment SGI256894[] = {{&S25366[13], 4}, {NULL}};
const Segment SGI256895[] = {{&S25366[26], 5}, {NULL}};
const Segment SGI256896[] = {{&S25366[59], 6}, {NULL}};
const Segment SGI256897[] = {{&S25366[113], 6}, {NULL}};
const Segment SGI256898[] = {{&S25366[127], 7}, {NULL}};
const Segment SGI256899[] = {{&S25367[13], 4}, {NULL}};
const Segment SGI256900[] = {{&S25367[26], 3}, {NULL}};
const Segment SGI256901[] = {{&S25367[57], 6}, {NULL}};
const Segment SGI256902[] = {{&S25367[111], 6}, {NULL}};
const Segment SGI256903[] = {{&S25367[125], 7}, {NULL}};
const Segment SGI256904[] = {{&S25368[13], 4}, {NULL}};
const Segment SGI256905[] = {{&S25368[26], 5}, {NULL}};
const Segment SGI256906[] = {{&S25368[59], 6}, {NULL}};
const Segment SGI256907[] = {{&S25368[113], 6}, {NULL}};
const Segment SGI256908[] = {{&S25368[127], 7}, {NULL}};
const Segment SGI256909[] = {{&S25838[13], 4}, {NULL}};
const Segment SGI256910[] = {{&S25838[94], 3}, {NULL}};
const Segment SGI256911[] = {{&S25838[26], 3}, {NULL}};
const Segment SGI256912[] = {{&S25838[57], 8}, {NULL}};
const Segment SGI256913[] = {{&S25838[105], 8}, {NULL}};
const Segment SGI256914[] = {{&S25838[121], 7}, {NULL}};
const Segment SGI256915[] = {{&S25839[13], 4}, {NULL}};
const Segment SGI256916[] = {{&S25839[94], 3}, {NULL}};
const Segment SGI256917[] = {{&S25839[26], 3}, {NULL}};
const Segment SGI256918[] = {{&S25839[57], 8}, {NULL}};
const Segment SGI256919[] = {{&S25839[105], 8}, {NULL}};
const Segment SGI256920[] = {{&S25839[121], 7}, {NULL}};
const Segment SGI256921[] = {{&S25840[13], 4}, {NULL}};
const Segment SGI256922[] = {{&S25840[94], 3}, {NULL}};
const Segment SGI256923[] = {{&S25840[26], 3}, {NULL}};
const Segment SGI256924[] = {{&S25840[57], 8}, {NULL}};
const Segment SGI256925[] = {{&S25840[105], 6}, {NULL}};
const Segment SGI256926[] = {{&S25840[119], 7}, {NULL}};
const Segment SGI256927[] = {{&S25841[13], 4}, {NULL}};
const Segment SGI256928[] = {{&S25841[94], 3}, {NULL}};
const Segment SGI256929[] = {{&S25841[26], 3}, {NULL}};
const Segment SGI256930[] = {{&S25841[57], 8}, {NULL}};
const Segment SGI256931[] = {{&S25841[105], 6}, {NULL}};
const Segment SGI256932[] = {{&S25841[119], 7}, {NULL}};
const Segment SGI256933[] = {{&S25842[13], 4}, {NULL}};
const Segment SGI256934[] = {{&S25842[94], 3}, {NULL}};
const Segment SGI256935[] = {{&S25842[26], 3}, {NULL}};
const Segment SGI256936[] = {{&S25842[57], 8}, {NULL}};
const Segment SGI256937[] = {{&S25842[105], 6}, {NULL}};
const Segment SGI256938[] = {{&S25842[119], 7}, {NULL}};
const Segment SGI256939[] = {{&S25843[13], 4}, {NULL}};
const Segment SGI256940[] = {{&S25843[96], 3}, {NULL}};
const Segment SGI256941[] = {{&S25843[26], 5}, {NULL}};
const Segment SGI256942[] = {{&S25843[59], 8}, {NULL}};
const Segment SGI256943[] = {{&S25843[107], 6}, {NULL}};
const Segment SGI256944[] = {{&S25843[121], 7}, {NULL}};
const Segment SGI256945[] = {{&S25844[13], 4}, {NULL}};
const Segment SGI256946[] = {{&S25844[96], 3}, {NULL}};
const Segment SGI256947[] = {{&S25844[26], 5}, {NULL}};
const Segment SGI256948[] = {{&S25844[59], 8}, {NULL}};
const Segment SGI256949[] = {{&S25844[107], 6}, {NULL}};
const Segment SGI256950[] = {{&S25844[121], 7}, {NULL}};
const Segment SGI256951[] = {{&S25845[13], 4}, {NULL}};
const Segment SGI256952[] = {{&S25845[94], 3}, {NULL}};
const Segment SGI256953[] = {{&S25845[26], 3}, {NULL}};
const Segment SGI256954[] = {{&S25845[57], 8}, {NULL}};
const Segment SGI256955[] = {{&S25845[105], 8}, {NULL}};
const Segment SGI256956[] = {{&S25845[121], 7}, {NULL}};
const Segment SGI256957[] = {{&S25846[13], 4}, {NULL}};
const Segment SGI256958[] = {{&S25846[94], 3}, {NULL}};
const Segment SGI256959[] = {{&S25846[26], 3}, {NULL}};
const Segment SGI256960[] = {{&S25846[57], 8}, {NULL}};
const Segment SGI256961[] = {{&S25846[105], 6}, {NULL}};
const Segment SGI256962[] = {{&S25847[13], 4}, {NULL}};
const Segment SGI256963[] = {{&S25847[108], 3}, {NULL}};
const Segment SGI256964[] = {{&S25847[26], 3}, {NULL}};
const Segment SGI256965[] = {{&S25847[57], 8}, {NULL}};
const Segment SGI256966[] = {{&S25847[93], 6}, {NULL}};
const Segment SGI256967[] = {{&S25847[119], 7}, {NULL}};
const Segment SGI256968[] = {{&S25848[13], 4}, {NULL}};
const Segment SGI256969[] = {{&S25848[96], 3}, {NULL}};
const Segment SGI256970[] = {{&S25848[26], 5}, {NULL}};
const Segment SGI256971[] = {{&S25848[59], 8}, {NULL}};
const Segment SGI256972[] = {{&S25848[107], 6}, {NULL}};
const Segment SGI256973[] = {{&S25848[121], 7}, {NULL}};
const Segment SGI256974[] = {{&S25849[13], 4}, {NULL}};
const Segment SGI256975[] = {{&S25849[93], 3}, {NULL}};
const Segment SGI256976[] = {{&S25849[56], 8}, {NULL}};
const Segment SGI256977[] = {{&S25849[104], 6}, {NULL}};
const Segment SGI256978[] = {{&S25849[118], 7}, {NULL}};
const Segment SGI256979[] = {{&S25850[13], 4}, {NULL}};
const Segment SGI256980[] = {{&S25850[110], 3}, {NULL}};
const Segment SGI256981[] = {{&S25850[26], 5}, {NULL}};
const Segment SGI256982[] = {{&S25850[59], 8}, {NULL}};
const Segment SGI256983[] = {{&S25850[95], 6}, {NULL}};
const Segment SGI256984[] = {{&S25850[121], 7}, {NULL}};
const Segment SGI256985[] = {{&S25851[13], 4}, {NULL}};
const Segment SGI256986[] = {{&S25851[96], 3}, {NULL}};
const Segment SGI256987[] = {{&S25851[26], 5}, {NULL}};
const Segment SGI256988[] = {{&S25851[59], 8}, {NULL}};
const Segment SGI256989[] = {{&S25851[107], 6}, {NULL}};
const Segment SGI256990[] = {{&S26014[12], 4}, {NULL}};
const Segment SGI256991[] = {{&S26014[74], 5}, {NULL}};
const Segment SGI256992[] = {{&S26505[13], 4}, {NULL}};
const Segment SGI256993[] = {{&S26505[99], 3}, {NULL}};
const Segment SGI256994[] = {{&S26505[33], 3}, {NULL}};
const Segment SGI256995[] = {{&S26505[64], 6}, {NULL}};
const Segment SGI256996[] = {{&S26505[110], 8}, {NULL}};
const Segment SGI256997[] = {{&S26505[126], 7}, {NULL}};
const Segment SGI256998[] = {{&S26506[13], 4}, {NULL}};
const Segment SGI256999[] = {{&S26506[99], 3}, {NULL}};
const Segment SGI257000[] = {{&S26506[33], 3}, {NULL}};
const Segment SGI257001[] = {{&S26506[64], 6}, {NULL}};
const Segment SGI257002[] = {{&S26506[110], 8}, {NULL}};
const Segment SGI257003[] = {{&S26506[126], 7}, {NULL}};
const Segment SGI257004[] = {{&S26507[13], 4}, {NULL}};
const Segment SGI257005[] = {{&S26507[99], 3}, {NULL}};
const Segment SGI257006[] = {{&S26507[33], 3}, {NULL}};
const Segment SGI257007[] = {{&S26507[64], 6}, {NULL}};
const Segment SGI257008[] = {{&S26507[110], 6}, {NULL}};
const Segment SGI257009[] = {{&S26507[124], 7}, {NULL}};
const Segment SGI257010[] = {{&S26508[13], 4}, {NULL}};
const Segment SGI257011[] = {{&S26508[99], 3}, {NULL}};
const Segment SGI257012[] = {{&S26508[33], 3}, {NULL}};
const Segment SGI257013[] = {{&S26508[64], 6}, {NULL}};
const Segment SGI257014[] = {{&S26508[110], 6}, {NULL}};
const Segment SGI257015[] = {{&S26508[124], 7}, {NULL}};
const Segment SGI257016[] = {{&S26509[13], 4}, {NULL}};
const Segment SGI257017[] = {{&S26509[101], 3}, {NULL}};
const Segment SGI257018[] = {{&S26509[33], 5}, {NULL}};
const Segment SGI257019[] = {{&S26509[66], 6}, {NULL}};
const Segment SGI257020[] = {{&S26509[112], 6}, {NULL}};
const Segment SGI257021[] = {{&S26509[126], 7}, {NULL}};
const Segment SGI257022[] = {{&S26510[13], 4}, {NULL}};
const Segment SGI257023[] = {{&S26510[101], 5}, {&S26510[129], 3}, {NULL}};
const Segment SGI257024[] = {{&S26510[33], 5}, {NULL}};
const Segment SGI257025[] = {{&S26510[66], 6}, {NULL}};
const Segment SGI257026[] = {{&S26510[114], 6}, {NULL}};
const Segment SGI257027[] = {{&S26510[140], 7}, {&S26510[155], 9}, {NULL}};
const Segment SGI257028[] = {{&S26511[13], 4}, {NULL}};
const Segment SGI257029[] = {{&S26511[99], 5}, {&S26511[127], 3}, {NULL}};
const Segment SGI257030[] = {{&S26511[33], 3}, {NULL}};
const Segment SGI257031[] = {{&S26511[64], 6}, {NULL}};
const Segment SGI257032[] = {{&S26511[112], 6}, {NULL}};
const Segment SGI257033[] = {{&S26511[138], 7}, {&S26511[153], 9}, {NULL}};
const Segment SGI257034[] = {{&S26512[13], 4}, {NULL}};
const Segment SGI257035[] = {{&S26512[97], 3}, {NULL}};
const Segment SGI257036[] = {{&S26512[33], 2}, {NULL}};
const Segment SGI257037[] = {{&S26512[63], 6}, {NULL}};
const Segment SGI257038[] = {{&S26512[108], 6}, {NULL}};
const Segment SGI257039[] = {{&S26512[122], 7}, {NULL}};
const Segment SGI257040[] = {{&S26513[13], 4}, {NULL}};
const Segment SGI257041[] = {{&S26513[97], 3}, {NULL}};
const Segment SGI257042[] = {{&S26513[33], 2}, {NULL}};
const Segment SGI257043[] = {{&S26513[63], 6}, {NULL}};
const Segment SGI257044[] = {{&S26513[108], 6}, {NULL}};
const Segment SGI257045[] = {{&S26513[122], 7}, {NULL}};
const Segment SGI257046[] = {{&S26514[13], 4}, {NULL}};
const Segment SGI257047[] = {{&S26514[98], 3}, {NULL}};
const Segment SGI257048[] = {{&S26514[33], 3}, {NULL}};
const Segment SGI257049[] = {{&S26514[64], 6}, {NULL}};
const Segment SGI257050[] = {{&S26514[109], 6}, {NULL}};
const Segment SGI257051[] = {{&S26514[123], 7}, {NULL}};
const Segment SGI257052[] = {{&S26515[13], 4}, {NULL}};
const Segment SGI257053[] = {{&S26515[97], 3}, {NULL}};
const Segment SGI257054[] = {{&S26515[33], 2}, {NULL}};
const Segment SGI257055[] = {{&S26515[63], 6}, {NULL}};
const Segment SGI257056[] = {{&S26515[108], 8}, {NULL}};
const Segment SGI257057[] = {{&S26515[124], 7}, {NULL}};
const Segment SGI257058[] = {{&S26575[13], 6}, {NULL}};
const Segment SGI257059[] = {{&S26575[35], 5}, {NULL}};
const Segment SGI257060[] = {{&S26575[75], 8}, {NULL}};
const Segment SGI257061[] = {{&S26575[130], 5}, {NULL}};
const Segment SGI257062[] = {{&S26575[143], 9}, {NULL}};
const Segment SGI257063[] = {{&S26576[13], 6}, {NULL}};
const Segment SGI257064[] = {{&S26576[35], 5}, {NULL}};
const Segment SGI257065[] = {{&S26576[75], 8}, {NULL}};
const Segment SGI257066[] = {{&S26576[130], 5}, {NULL}};
const Segment SGI257067[] = {{&S26576[143], 9}, {NULL}};
const Segment SGI257068[] = {{&S26577[13], 6}, {NULL}};
const Segment SGI257069[] = {{&S26577[35], 3}, {NULL}};
const Segment SGI257070[] = {{&S26577[73], 8}, {NULL}};
const Segment SGI257071[] = {{&S26577[128], 5}, {NULL}};
const Segment SGI257072[] = {{&S26577[141], 9}, {NULL}};
const Segment SGI257073[] = {{&S26578[13], 6}, {NULL}};
const Segment SGI257074[] = {{&S26578[35], 3}, {NULL}};
const Segment SGI257075[] = {{&S26578[73], 8}, {NULL}};
const Segment SGI257076[] = {{&S26578[128], 5}, {NULL}};
const Segment SGI257077[] = {{&S26578[141], 9}, {NULL}};
const Segment SGI257078[] = {{&S26579[13], 6}, {NULL}};
const Segment SGI257079[] = {{&S26579[35], 5}, {NULL}};
const Segment SGI257080[] = {{&S26579[75], 8}, {NULL}};
const Segment SGI257081[] = {{&S26579[130], 5}, {NULL}};
const Segment SGI257082[] = {{&S26579[143], 9}, {NULL}};
const Segment SGI257083[] = {{&S26580[13], 6}, {NULL}};
const Segment SGI257084[] = {{&S26580[35], 3}, {NULL}};
const Segment SGI257085[] = {{&S26580[73], 8}, {NULL}};
const Segment SGI257086[] = {{&S26580[128], 5}, {NULL}};
const Segment SGI257087[] = {{&S26580[141], 9}, {NULL}};
const Segment SGI257088[] = {{&S26945[13], 4}, {NULL}};
const Segment SGI257089[] = {{&S26945[113], 5}, {NULL}};
const Segment SGI257090[] = {{&S26945[36], 5}, {NULL}};
const Segment SGI257091[] = {{&S26945[76], 8}, {NULL}};
const Segment SGI257092[] = {{&S26945[126], 5}, {NULL}};
const Segment SGI257093[] = {{&S26945[139], 9}, {NULL}};
const Segment SGI257094[] = {{&S26946[13], 4}, {NULL}};
const Segment SGI257095[] = {{&S26946[113], 5}, {NULL}};
const Segment SGI257096[] = {{&S26946[36], 5}, {NULL}};
const Segment SGI257097[] = {{&S26946[76], 8}, {NULL}};
const Segment SGI257098[] = {{&S26946[126], 5}, {NULL}};
const Segment SGI257099[] = {{&S26946[139], 9}, {NULL}};
const Segment SGI257100[] = {{&S26947[13], 4}, {NULL}};
const Segment SGI257101[] = {{&S26947[110], 5}, {NULL}};
const Segment SGI257102[] = {{&S26947[36], 3}, {NULL}};
const Segment SGI257103[] = {{&S26947[74], 8}, {NULL}};
const Segment SGI257104[] = {{&S26947[123], 5}, {NULL}};
const Segment SGI257105[] = {{&S26947[136], 9}, {NULL}};
const Segment SGI257106[] = {{&S27031[13], 4}, {NULL}};
const Segment SGI257107[] = {{&S27031[26], 5}, {NULL}};
const Segment SGI257108[] = {{&S27031[66], 6}, {NULL}};
const Segment SGI257109[] = {{&S27031[120], 5}, {NULL}};
const Segment SGI257110[] = {{&S27031[133], 9}, {NULL}};
const Segment SGI257111[] = {{&S27032[13], 4}, {NULL}};
const Segment SGI257112[] = {{&S27032[26], 3}, {NULL}};
const Segment SGI257113[] = {{&S27032[64], 6}, {NULL}};
const Segment SGI257114[] = {{&S27032[118], 5}, {NULL}};
const Segment SGI257115[] = {{&S27032[131], 9}, {NULL}};
const Segment SGI257116[] = {{&S27033[13], 4}, {NULL}};
const Segment SGI257117[] = {{&S27033[26], 3}, {NULL}};
const Segment SGI257118[] = {{&S27033[64], 6}, {NULL}};
const Segment SGI257119[] = {{&S27033[118], 5}, {NULL}};
const Segment SGI257120[] = {{&S27033[131], 9}, {NULL}};
const Segment SGI257121[] = {{&S28083[13], 6}, {NULL}};
const Segment SGI257122[] = {{&S28083[101], 3}, {NULL}};
const Segment SGI257123[] = {{&S28083[35], 3}, {NULL}};
const Segment SGI257124[] = {{&S28083[66], 6}, {NULL}};
const Segment SGI257125[] = {{&S28083[112], 6}, {NULL}};
const Segment SGI257126[] = {{&S28083[126], 7}, {NULL}};
const Segment SGI257127[] = {{&S28084[13], 6}, {NULL}};
const Segment SGI257128[] = {{&S28084[101], 3}, {NULL}};
const Segment SGI257129[] = {{&S28084[35], 3}, {NULL}};
const Segment SGI257130[] = {{&S28084[66], 6}, {NULL}};
const Segment SGI257131[] = {{&S28084[112], 6}, {NULL}};
const Segment SGI257132[] = {{&S28084[126], 7}, {NULL}};
const Segment SGI257133[] = {{&S28085[13], 6}, {NULL}};
const Segment SGI257134[] = {{&S28085[101], 3}, {NULL}};
const Segment SGI257135[] = {{&S28085[35], 3}, {NULL}};
const Segment SGI257136[] = {{&S28085[66], 6}, {NULL}};
const Segment SGI257137[] = {{&S28085[112], 6}, {NULL}};
const Segment SGI257138[] = {{&S28085[126], 7}, {NULL}};
const Segment SGI257139[] = {{&S28086[13], 6}, {NULL}};
const Segment SGI257140[] = {{&S28086[101], 3}, {NULL}};
const Segment SGI257141[] = {{&S28086[35], 3}, {NULL}};
const Segment SGI257142[] = {{&S28086[66], 6}, {NULL}};
const Segment SGI257143[] = {{&S28086[112], 6}, {NULL}};
const Segment SGI257144[] = {{&S28086[126], 7}, {NULL}};
const Segment SGI257145[] = {{&S28087[13], 6}, {NULL}};
const Segment SGI257146[] = {{&S28087[101], 3}, {NULL}};
const Segment SGI257147[] = {{&S28087[35], 3}, {NULL}};
const Segment SGI257148[] = {{&S28087[66], 6}, {NULL}};
const Segment SGI257149[] = {{&S28087[112], 8}, {NULL}};
const Segment SGI257150[] = {{&S28087[128], 7}, {NULL}};
const Segment SGI257151[] = {{&S28088[13], 6}, {NULL}};
const Segment SGI257152[] = {{&S28088[101], 3}, {NULL}};
const Segment SGI257153[] = {{&S28088[35], 3}, {NULL}};
const Segment SGI257154[] = {{&S28088[66], 6}, {NULL}};
const Segment SGI257155[] = {{&S28088[112], 6}, {NULL}};
const Segment SGI257156[] = {{&S28088[126], 7}, {NULL}};
const Segment SGI257157[] = {{&S28089[13], 6}, {NULL}};
const Segment SGI257158[] = {{&S28089[101], 3}, {NULL}};
const Segment SGI257159[] = {{&S28089[35], 3}, {NULL}};
const Segment SGI257160[] = {{&S28089[66], 6}, {NULL}};
const Segment SGI257161[] = {{&S28089[112], 6}, {NULL}};
const Segment SGI257162[] = {{&S28089[126], 7}, {NULL}};
const Segment SGI257163[] = {{&S28090[13], 6}, {NULL}};
const Segment SGI257164[] = {{&S28090[101], 3}, {NULL}};
const Segment SGI257165[] = {{&S28090[35], 3}, {NULL}};
const Segment SGI257166[] = {{&S28090[66], 6}, {NULL}};
const Segment SGI257167[] = {{&S28090[112], 6}, {NULL}};
const Segment SGI257168[] = {{&S28090[126], 7}, {NULL}};
const Segment SGI257169[] = {{&S28091[13], 6}, {NULL}};
const Segment SGI257170[] = {{&S28091[101], 3}, {NULL}};
const Segment SGI257171[] = {{&S28091[35], 3}, {NULL}};
const Segment SGI257172[] = {{&S28091[66], 6}, {NULL}};
const Segment SGI257173[] = {{&S28091[112], 6}, {NULL}};
const Segment SGI257174[] = {{&S28091[126], 7}, {NULL}};
const Segment SGI257175[] = {{&S28092[13], 6}, {NULL}};
const Segment SGI257176[] = {{&S28092[101], 3}, {NULL}};
const Segment SGI257177[] = {{&S28092[35], 3}, {NULL}};
const Segment SGI257178[] = {{&S28092[66], 6}, {NULL}};
const Segment SGI257179[] = {{&S28092[112], 6}, {NULL}};
const Segment SGI257180[] = {{&S28092[126], 7}, {NULL}};
const Segment SGI257181[] = {{&S28093[13], 6}, {NULL}};
const Segment SGI257182[] = {{&S28093[101], 3}, {NULL}};
const Segment SGI257183[] = {{&S28093[35], 3}, {NULL}};
const Segment SGI257184[] = {{&S28093[66], 6}, {NULL}};
const Segment SGI257185[] = {{&S28093[112], 6}, {NULL}};
const Segment SGI257186[] = {{&S28093[126], 7}, {NULL}};
const Segment SGI257187[] = {{&S28094[13], 6}, {NULL}};
const Segment SGI257188[] = {{&S28094[101], 3}, {NULL}};
const Segment SGI257189[] = {{&S28094[35], 3}, {NULL}};
const Segment SGI257190[] = {{&S28094[66], 6}, {NULL}};
const Segment SGI257191[] = {{&S28094[112], 8}, {NULL}};
const Segment SGI257192[] = {{&S28094[128], 7}, {NULL}};
const Segment SGI257193[] = {{&S28095[13], 6}, {NULL}};
const Segment SGI257194[] = {{&S28095[101], 3}, {NULL}};
const Segment SGI257195[] = {{&S28095[35], 3}, {NULL}};
const Segment SGI257196[] = {{&S28095[66], 6}, {NULL}};
const Segment SGI257197[] = {{&S28095[112], 6}, {NULL}};
const Segment SGI257198[] = {{&S28095[126], 7}, {NULL}};
const Segment SGI257199[] = {{&S28096[13], 6}, {NULL}};
const Segment SGI257200[] = {{&S28096[101], 3}, {NULL}};
const Segment SGI257201[] = {{&S28096[35], 3}, {NULL}};
const Segment SGI257202[] = {{&S28096[66], 6}, {NULL}};
const Segment SGI257203[] = {{&S28096[112], 6}, {NULL}};
const Segment SGI257204[] = {{&S28096[126], 7}, {NULL}};
const Segment SGI257205[] = {{&S28097[13], 6}, {NULL}};
const Segment SGI257206[] = {{&S28097[101], 3}, {NULL}};
const Segment SGI257207[] = {{&S28097[35], 3}, {NULL}};
const Segment SGI257208[] = {{&S28097[66], 6}, {NULL}};
const Segment SGI257209[] = {{&S28097[112], 8}, {NULL}};
const Segment SGI257210[] = {{&S28097[128], 7}, {NULL}};
const Segment SGI257211[] = {{&S28098[13], 6}, {NULL}};
const Segment SGI257212[] = {{&S28098[101], 3}, {NULL}};
const Segment SGI257213[] = {{&S28098[35], 3}, {NULL}};
const Segment SGI257214[] = {{&S28098[66], 6}, {NULL}};
const Segment SGI257215[] = {{&S28098[112], 6}, {NULL}};
const Segment SGI257216[] = {{&S28098[126], 7}, {NULL}};
const Segment SGI257217[] = {{&S28099[13], 6}, {NULL}};
const Segment SGI257218[] = {{&S28099[101], 3}, {NULL}};
const Segment SGI257219[] = {{&S28099[35], 3}, {NULL}};
const Segment SGI257220[] = {{&S28099[66], 6}, {NULL}};
const Segment SGI257221[] = {{&S28099[112], 6}, {NULL}};
const Segment SGI257222[] = {{&S28099[126], 7}, {NULL}};
const Segment SGI257223[] = {{&S28100[13], 6}, {NULL}};
const Segment SGI257224[] = {{&S28100[101], 3}, {NULL}};
const Segment SGI257225[] = {{&S28100[35], 3}, {NULL}};
const Segment SGI257226[] = {{&S28100[66], 6}, {NULL}};
const Segment SGI257227[] = {{&S28100[112], 6}, {NULL}};
const Segment SGI257228[] = {{&S28100[126], 7}, {NULL}};
const Segment SGI257229[] = {{&S28101[13], 6}, {NULL}};
const Segment SGI257230[] = {{&S28101[101], 3}, {NULL}};
const Segment SGI257231[] = {{&S28101[35], 3}, {NULL}};
const Segment SGI257232[] = {{&S28101[66], 6}, {NULL}};
const Segment SGI257233[] = {{&S28101[112], 6}, {NULL}};
const Segment SGI257234[] = {{&S28101[126], 7}, {NULL}};
const Segment SGI257235[] = {{&S28102[13], 6}, {NULL}};
const Segment SGI257236[] = {{&S28102[101], 3}, {NULL}};
const Segment SGI257237[] = {{&S28102[35], 3}, {NULL}};
const Segment SGI257238[] = {{&S28102[66], 6}, {NULL}};
const Segment SGI257239[] = {{&S28102[112], 6}, {NULL}};
const Segment SGI257240[] = {{&S28102[126], 7}, {NULL}};
const Segment SGI257241[] = {{&S28103[13], 6}, {NULL}};
const Segment SGI257242[] = {{&S28103[101], 3}, {NULL}};
const Segment SGI257243[] = {{&S28103[35], 3}, {NULL}};
const Segment SGI257244[] = {{&S28103[66], 6}, {NULL}};
const Segment SGI257245[] = {{&S28103[112], 6}, {NULL}};
const Segment SGI257246[] = {{&S28103[126], 7}, {NULL}};
const Segment SGI257247[] = {{&S28104[13], 6}, {NULL}};
const Segment SGI257248[] = {{&S28104[101], 3}, {NULL}};
const Segment SGI257249[] = {{&S28104[35], 3}, {NULL}};
const Segment SGI257250[] = {{&S28104[66], 6}, {NULL}};
const Segment SGI257251[] = {{&S28104[112], 6}, {NULL}};
const Segment SGI257252[] = {{&S28104[126], 7}, {NULL}};
const Segment SGI257253[] = {{&S28105[13], 6}, {NULL}};
const Segment SGI257254[] = {{&S28105[101], 3}, {NULL}};
const Segment SGI257255[] = {{&S28105[35], 3}, {NULL}};
const Segment SGI257256[] = {{&S28105[66], 6}, {NULL}};
const Segment SGI257257[] = {{&S28105[112], 6}, {NULL}};
const Segment SGI257258[] = {{&S28105[126], 7}, {NULL}};
const Segment SGI257259[] = {{&S28106[13], 6}, {NULL}};
const Segment SGI257260[] = {{&S28106[101], 3}, {NULL}};
const Segment SGI257261[] = {{&S28106[35], 3}, {NULL}};
const Segment SGI257262[] = {{&S28106[66], 6}, {NULL}};
const Segment SGI257263[] = {{&S28106[112], 8}, {NULL}};
const Segment SGI257264[] = {{&S28106[128], 7}, {NULL}};
const Segment SGI257265[] = {{&S28107[13], 6}, {NULL}};
const Segment SGI257266[] = {{&S28107[101], 3}, {NULL}};
const Segment SGI257267[] = {{&S28107[35], 3}, {NULL}};
const Segment SGI257268[] = {{&S28107[66], 6}, {NULL}};
const Segment SGI257269[] = {{&S28107[112], 6}, {NULL}};
const Segment SGI257270[] = {{&S28107[126], 7}, {NULL}};
const Segment SGI257271[] = {{&S28108[13], 6}, {NULL}};
const Segment SGI257272[] = {{&S28108[101], 3}, {NULL}};
const Segment SGI257273[] = {{&S28108[35], 3}, {NULL}};
const Segment SGI257274[] = {{&S28108[66], 6}, {NULL}};
const Segment SGI257275[] = {{&S28108[112], 8}, {NULL}};
const Segment SGI257276[] = {{&S28108[128], 7}, {NULL}};
const Segment SGI257277[] = {{&S28109[13], 6}, {NULL}};
const Segment SGI257278[] = {{&S28109[101], 3}, {NULL}};
const Segment SGI257279[] = {{&S28109[35], 3}, {NULL}};
const Segment SGI257280[] = {{&S28109[66], 6}, {NULL}};
const Segment SGI257281[] = {{&S28109[112], 6}, {NULL}};
const Segment SGI257282[] = {{&S28109[126], 7}, {NULL}};
const Segment SGI257283[] = {{&S28110[13], 6}, {NULL}};
const Segment SGI257284[] = {{&S28110[103], 3}, {NULL}};
const Segment SGI257285[] = {{&S28110[35], 3}, {NULL}};
const Segment SGI257286[] = {{&S28110[66], 8}, {NULL}};
const Segment SGI257287[] = {{&S28110[114], 6}, {NULL}};
const Segment SGI257288[] = {{&S28110[128], 7}, {NULL}};
const Segment SGI257289[] = {{&S28111[13], 6}, {NULL}};
const Segment SGI257290[] = {{&S28111[101], 3}, {NULL}};
const Segment SGI257291[] = {{&S28111[35], 3}, {NULL}};
const Segment SGI257292[] = {{&S28111[66], 6}, {NULL}};
const Segment SGI257293[] = {{&S28111[112], 6}, {NULL}};
const Segment SGI257294[] = {{&S28111[126], 7}, {NULL}};
const Segment SGI257295[] = {{&S28112[13], 6}, {NULL}};
const Segment SGI257296[] = {{&S28112[103], 3}, {NULL}};
const Segment SGI257297[] = {{&S28112[35], 3}, {NULL}};
const Segment SGI257298[] = {{&S28112[66], 8}, {NULL}};
const Segment SGI257299[] = {{&S28112[114], 6}, {NULL}};
const Segment SGI257300[] = {{&S28112[128], 7}, {NULL}};
const Segment SGI257301[] = {{&S28113[13], 6}, {NULL}};
const Segment SGI257302[] = {{&S28113[103], 3}, {NULL}};
const Segment SGI257303[] = {{&S28113[35], 3}, {NULL}};
const Segment SGI257304[] = {{&S28113[66], 8}, {NULL}};
const Segment SGI257305[] = {{&S28113[114], 6}, {NULL}};
const Segment SGI257306[] = {{&S28113[128], 7}, {NULL}};
const Segment SGI257307[] = {{&S28114[13], 6}, {NULL}};
const Segment SGI257308[] = {{&S28114[103], 3}, {NULL}};
const Segment SGI257309[] = {{&S28114[35], 3}, {NULL}};
const Segment SGI257310[] = {{&S28114[66], 8}, {NULL}};
const Segment SGI257311[] = {{&S28114[114], 6}, {NULL}};
const Segment SGI257312[] = {{&S28114[128], 7}, {NULL}};
const Segment SGI257313[] = {{&S28115[13], 6}, {NULL}};
const Segment SGI257314[] = {{&S28115[103], 3}, {NULL}};
const Segment SGI257315[] = {{&S28115[35], 3}, {NULL}};
const Segment SGI257316[] = {{&S28115[66], 8}, {NULL}};
const Segment SGI257317[] = {{&S28115[114], 8}, {NULL}};
const Segment SGI257318[] = {{&S28115[130], 7}, {NULL}};
const Segment SGI257319[] = {{&S28116[13], 6}, {NULL}};
const Segment SGI257320[] = {{&S28116[103], 3}, {NULL}};
const Segment SGI257321[] = {{&S28116[35], 3}, {NULL}};
const Segment SGI257322[] = {{&S28116[66], 8}, {NULL}};
const Segment SGI257323[] = {{&S28116[114], 6}, {NULL}};
const Segment SGI257324[] = {{&S28116[128], 7}, {NULL}};
const Segment SGI257325[] = {{&S28117[13], 6}, {NULL}};
const Segment SGI257326[] = {{&S28117[103], 3}, {NULL}};
const Segment SGI257327[] = {{&S28117[35], 3}, {NULL}};
const Segment SGI257328[] = {{&S28117[66], 8}, {NULL}};
const Segment SGI257329[] = {{&S28117[114], 6}, {NULL}};
const Segment SGI257330[] = {{&S28117[128], 7}, {NULL}};
const Segment SGI257331[] = {{&S28118[13], 6}, {NULL}};
const Segment SGI257332[] = {{&S28118[101], 3}, {NULL}};
const Segment SGI257333[] = {{&S28118[35], 3}, {NULL}};
const Segment SGI257334[] = {{&S28118[66], 6}, {NULL}};
const Segment SGI257335[] = {{&S28118[112], 6}, {NULL}};
const Segment SGI257336[] = {{&S28118[126], 7}, {NULL}};
const Segment SGI257337[] = {{&S28119[13], 6}, {NULL}};
const Segment SGI257338[] = {{&S28119[105], 3}, {NULL}};
const Segment SGI257339[] = {{&S28119[35], 5}, {NULL}};
const Segment SGI257340[] = {{&S28119[68], 8}, {NULL}};
const Segment SGI257341[] = {{&S28119[116], 6}, {NULL}};
const Segment SGI257342[] = {{&S28119[130], 7}, {NULL}};
const Segment SGI257343[] = {{&S28120[13], 6}, {NULL}};
const Segment SGI257344[] = {{&S28120[105], 3}, {NULL}};
const Segment SGI257345[] = {{&S28120[35], 5}, {NULL}};
const Segment SGI257346[] = {{&S28120[68], 8}, {NULL}};
const Segment SGI257347[] = {{&S28120[116], 6}, {NULL}};
const Segment SGI257348[] = {{&S28120[130], 7}, {NULL}};
const Segment SGI257349[] = {{&S28121[13], 6}, {NULL}};
const Segment SGI257350[] = {{&S28121[103], 3}, {NULL}};
const Segment SGI257351[] = {{&S28121[35], 5}, {NULL}};
const Segment SGI257352[] = {{&S28121[68], 6}, {NULL}};
const Segment SGI257353[] = {{&S28121[114], 6}, {NULL}};
const Segment SGI257354[] = {{&S28121[128], 7}, {NULL}};
const Segment SGI257355[] = {{&S9635[10], 4}, {NULL}};
const Segment SGI257356[] = {{&S9635[15], 5}, {NULL}};
const Segment SGI257357[] = {{&S9727[10], 4}, {NULL}};
const Segment SGI257358[] = {{&S9727[15], 5}, {NULL}};
const Segment SGI257359[] = {{&S28840[13], 4}, {NULL}};
const Segment SGI257360[] = {{&S28840[111], 3}, {NULL}};
const Segment SGI257361[] = {{&S28840[36], 5}, {NULL}};
const Segment SGI257362[] = {{&S28840[76], 6}, {NULL}};
const Segment SGI257363[] = {{&S28840[122], 4}, {NULL}};
const Segment SGI257364[] = {{&S28840[134], 6}, {NULL}};
const Segment SGI257365[] = {{&S28841[13], 4}, {NULL}};
const Segment SGI257366[] = {{&S28841[111], 3}, {NULL}};
const Segment SGI257367[] = {{&S28841[36], 5}, {NULL}};
const Segment SGI257368[] = {{&S28841[76], 6}, {NULL}};
const Segment SGI257369[] = {{&S28841[122], 5}, {NULL}};
const Segment SGI257370[] = {{&S28841[135], 6}, {NULL}};
const Segment SGI257371[] = {{&S28842[13], 4}, {NULL}};
const Segment SGI257372[] = {{&S28842[111], 3}, {NULL}};
const Segment SGI257373[] = {{&S28842[36], 5}, {NULL}};
const Segment SGI257374[] = {{&S28842[76], 6}, {NULL}};
const Segment SGI257375[] = {{&S28842[122], 4}, {NULL}};
const Segment SGI257376[] = {{&S28842[134], 6}, {NULL}};
const Segment SGI257377[] = {{&S28843[13], 4}, {NULL}};
const Segment SGI257378[] = {{&S28843[109], 3}, {NULL}};
const Segment SGI257379[] = {{&S28843[36], 3}, {NULL}};
const Segment SGI257380[] = {{&S28843[74], 6}, {NULL}};
const Segment SGI257381[] = {{&S28843[120], 5}, {NULL}};
const Segment SGI257382[] = {{&S28843[133], 6}, {NULL}};
const Segment SGI257383[] = {{&S28844[13], 4}, {NULL}};
const Segment SGI257384[] = {{&S28844[111], 3}, {NULL}};
const Segment SGI257385[] = {{&S28844[36], 5}, {NULL}};
const Segment SGI257386[] = {{&S28844[76], 6}, {NULL}};
const Segment SGI257387[] = {{&S28844[122], 5}, {NULL}};
const Segment SGI257388[] = {{&S28844[135], 6}, {NULL}};
const Segment SGI257389[] = {{&S28845[13], 4}, {NULL}};
const Segment SGI257390[] = {{&S28845[107], 5}, {NULL}};
const Segment SGI257391[] = {{&S28845[29], 5}, {NULL}};
const Segment SGI257392[] = {{&S28845[69], 9}, {NULL}};
const Segment SGI257393[] = {{&S28845[120], 5}, {NULL}};
const Segment SGI257394[] = {{&S28849[13], 4}, {NULL}};
const Segment SGI257395[] = {{&S28849[108], 3}, {NULL}};
const Segment SGI257396[] = {{&S28849[36], 3}, {NULL}};
const Segment SGI257397[] = {{&S28849[74], 6}, {NULL}};
const Segment SGI257398[] = {{&S28849[119], 4}, {NULL}};
const Segment SGI257399[] = {{&S28849[131], 6}, {NULL}};
const Segment SGI257400[] = {{&S28850[13], 4}, {NULL}};
const Segment SGI257401[] = {{&S28850[110], 3}, {NULL}};
const Segment SGI257402[] = {{&S28850[36], 1}, {NULL}};
const Segment SGI257403[] = {{&S28850[76], 6}, {NULL}};
const Segment SGI257404[] = {{&S28850[121], 4}, {NULL}};
const Segment SGI257405[] = {{&S28850[133], 6}, {NULL}};
const Segment SGI257406[] = {{&S28851[13], 4}, {NULL}};
const Segment SGI257407[] = {{&S28851[108], 3}, {NULL}};
const Segment SGI257408[] = {{&S28851[36], 3}, {NULL}};
const Segment SGI257409[] = {{&S28851[74], 6}, {NULL}};
const Segment SGI257410[] = {{&S28851[119], 5}, {NULL}};
const Segment SGI257411[] = {{&S28851[132], 6}, {NULL}};
const Segment SGI257412[] = {{&S28852[13], 4}, {NULL}};
const Segment SGI257413[] = {{&S28852[109], 3}, {NULL}};
const Segment SGI257414[] = {{&S28852[36], 3}, {NULL}};
const Segment SGI257415[] = {{&S28852[75], 6}, {NULL}};
const Segment SGI257416[] = {{&S28852[120], 4}, {NULL}};
const Segment SGI257417[] = {{&S28852[132], 6}, {NULL}};
const Segment SGI257418[] = {{&S29012[13], 6}, {NULL}};
const Segment SGI257419[] = {{&S29012[35], 3}, {NULL}};
const Segment SGI257420[] = {{&S29012[66], 6}, {NULL}};
const Segment SGI257421[] = {{&S29012[100], 8}, {NULL}};
const Segment SGI257422[] = {{&S29012[109], 7}, {NULL}};
const Segment SGI257423[] = {{&S29013[13], 6}, {NULL}};
const Segment SGI257424[] = {{&S29013[35], 3}, {NULL}};
const Segment SGI257425[] = {{&S29013[66], 6}, {NULL}};
const Segment SGI257426[] = {{&S29013[100], 8}, {NULL}};
const Segment SGI257427[] = {{&S29013[109], 7}, {NULL}};
const Segment SGI257428[] = {{&S29014[13], 6}, {NULL}};
const Segment SGI257429[] = {{&S29014[35], 3}, {NULL}};
const Segment SGI257430[] = {{&S29014[66], 6}, {NULL}};
const Segment SGI257431[] = {{&S29014[100], 8}, {NULL}};
const Segment SGI257432[] = {{&S29014[109], 7}, {NULL}};
const Segment SGI257433[] = {{&S29015[13], 6}, {NULL}};
const Segment SGI257434[] = {{&S29015[35], 3}, {NULL}};
const Segment SGI257435[] = {{&S29015[66], 6}, {NULL}};
const Segment SGI257436[] = {{&S29015[100], 6}, {NULL}};
const Segment SGI257437[] = {{&S29015[107], 7}, {NULL}};
const Segment SGI257438[] = {{&S29016[13], 6}, {NULL}};
const Segment SGI257439[] = {{&S29016[35], 3}, {NULL}};
const Segment SGI257440[] = {{&S29016[66], 6}, {NULL}};
const Segment SGI257441[] = {{&S29016[100], 6}, {NULL}};
const Segment SGI257442[] = {{&S29016[107], 7}, {NULL}};
const Segment SGI257443[] = {{&S29017[13], 6}, {NULL}};
const Segment SGI257444[] = {{&S29017[35], 3}, {NULL}};
const Segment SGI257445[] = {{&S29017[66], 6}, {NULL}};
const Segment SGI257446[] = {{&S29017[100], 6}, {NULL}};
const Segment SGI257447[] = {{&S29017[107], 7}, {NULL}};
const Segment SGI257448[] = {{&S29018[13], 6}, {NULL}};
const Segment SGI257449[] = {{&S29018[35], 3}, {NULL}};
const Segment SGI257450[] = {{&S29018[66], 6}, {NULL}};
const Segment SGI257451[] = {{&S29018[100], 6}, {NULL}};
const Segment SGI257452[] = {{&S29018[107], 7}, {NULL}};
const Segment SGI257453[] = {{&S29019[13], 6}, {NULL}};
const Segment SGI257454[] = {{&S29019[35], 5}, {NULL}};
const Segment SGI257455[] = {{&S29019[68], 6}, {NULL}};
const Segment SGI257456[] = {{&S29019[102], 6}, {NULL}};
const Segment SGI257457[] = {{&S29019[109], 7}, {NULL}};
const Segment SGI257458[] = {{&S29020[13], 6}, {NULL}};
const Segment SGI257459[] = {{&S29020[35], 5}, {NULL}};
const Segment SGI257460[] = {{&S29020[68], 6}, {NULL}};
const Segment SGI257461[] = {{&S29020[102], 6}, {NULL}};
const Segment SGI257462[] = {{&S29020[109], 7}, {NULL}};
const Segment SGI257463[] = {{&S29021[13], 6}, {NULL}};
const Segment SGI257464[] = {{&S29021[35], 5}, {NULL}};
const Segment SGI257465[] = {{&S29021[68], 6}, {NULL}};
const Segment SGI257466[] = {{&S29021[102], 6}, {NULL}};
const Segment SGI257467[] = {{&S29021[109], 7}, {NULL}};
const Segment SGI257468[] = {{&S29022[13], 6}, {NULL}};
const Segment SGI257469[] = {{&S29022[35], 5}, {NULL}};
const Segment SGI257470[] = {{&S29022[68], 6}, {NULL}};
const Segment SGI257471[] = {{&S29022[102], 6}, {NULL}};
const Segment SGI257472[] = {{&S29022[109], 7}, {NULL}};
const Segment SGI257473[] = {{&S29023[13], 6}, {NULL}};
const Segment SGI257474[] = {{&S29023[35], 5}, {NULL}};
const Segment SGI257475[] = {{&S29023[68], 6}, {NULL}};
const Segment SGI257476[] = {{&S29023[102], 6}, {NULL}};
const Segment SGI257477[] = {{&S29023[109], 7}, {NULL}};
const Segment SGI257478[] = {{&S10321[10], 4}, {NULL}};
const Segment SGI257479[] = {{&S10321[15], 5}, {NULL}};
const Segment SGI257480[] = {{&S10322[10], 4}, {NULL}};
const Segment SGI257481[] = {{&S10322[15], 5}, {NULL}};
const Segment SGI257482[] = {{&S10323[10], 4}, {NULL}};
const Segment SGI257483[] = {{&S10323[15], 5}, {NULL}};
const Segment SGI257484[] = {{&S10454[10], 4}, {NULL}};
const Segment SGI257485[] = {{&S10454[15], 5}, {NULL}};
const Segment SGI257486[] = {{&S29679[13], 4}, {NULL}};
const Segment SGI257487[] = {{&S29679[99], 3}, {NULL}};
const Segment SGI257488[] = {{&S29679[33], 3}, {NULL}};
const Segment SGI257489[] = {{&S29679[64], 6}, {NULL}};
const Segment SGI257490[] = {{&S29679[110], 5}, {NULL}};
const Segment SGI257491[] = {{&S29679[123], 9}, {NULL}};
const Segment SGI257492[] = {{&S29680[13], 4}, {NULL}};
const Segment SGI257493[] = {{&S29680[101], 3}, {NULL}};
const Segment SGI257494[] = {{&S29680[33], 5}, {NULL}};
const Segment SGI257495[] = {{&S29680[66], 6}, {NULL}};
const Segment SGI257496[] = {{&S29680[112], 5}, {NULL}};
const Segment SGI257497[] = {{&S29680[125], 9}, {NULL}};
const Segment SGI257498[] = {{&S29681[13], 4}, {NULL}};
const Segment SGI257499[] = {{&S29681[99], 3}, {NULL}};
const Segment SGI257500[] = {{&S29681[33], 3}, {NULL}};
const Segment SGI257501[] = {{&S29681[64], 6}, {NULL}};
const Segment SGI257502[] = {{&S29681[110], 5}, {NULL}};
const Segment SGI257503[] = {{&S29681[123], 9}, {NULL}};
const Segment SGI257504[] = {{&S29682[13], 4}, {NULL}};
const Segment SGI257505[] = {{&S29682[101], 3}, {NULL}};
const Segment SGI257506[] = {{&S29682[33], 5}, {NULL}};
const Segment SGI257507[] = {{&S29682[66], 6}, {NULL}};
const Segment SGI257508[] = {{&S29682[112], 5}, {NULL}};
const Segment SGI257509[] = {{&S29682[125], 9}, {NULL}};
const Segment SGI257510[] = {{&S29683[13], 4}, {NULL}};
const Segment SGI257511[] = {{&S29683[101], 5}, {&S29683[128], 3}, {NULL}};
const Segment SGI257512[] = {{&S29683[33], 5}, {NULL}};
const Segment SGI257513[] = {{&S29683[66], 6}, {NULL}};
const Segment SGI257514[] = {{&S29683[114], 5}, {NULL}};
const Segment SGI257515[] = {{&S29683[139], 7}, {&S29683[154], 9}, {NULL}};
const Segment SGI257516[] = {{&S29684[13], 4}, {NULL}};
const Segment SGI257517[] = {{&S29684[98], 3}, {NULL}};
const Segment SGI257518[] = {{&S29684[33], 3}, {NULL}};
const Segment SGI257519[] = {{&S29684[64], 6}, {NULL}};
const Segment SGI257520[] = {{&S29684[109], 5}, {NULL}};
const Segment SGI257521[] = {{&S29684[122], 9}, {NULL}};
const Segment SGI257522[] = {{&S29685[13], 4}, {NULL}};
const Segment SGI257523[] = {{&S29685[101], 3}, {NULL}};
const Segment SGI257524[] = {{&S29685[33], 5}, {NULL}};
const Segment SGI257525[] = {{&S29685[66], 6}, {NULL}};
const Segment SGI257526[] = {{&S29685[112], 5}, {NULL}};
const Segment SGI257527[] = {{&S29685[125], 9}, {NULL}};
const Segment SGI257528[] = {{&S10916[10], 4}, {NULL}};
const Segment SGI257529[] = {{&S10916[15], 5}, {NULL}};
const Segment SGI257530[] = {{&S29931[13], 4}, {NULL}};
const Segment SGI257531[] = {{&S29931[26], 5}, {NULL}};
const Segment SGI257532[] = {{&S29931[59], 6}, {NULL}};
const Segment SGI257533[] = {{&S29931[112], 6}, {NULL}};
const Segment SGI257534[] = {{&S29931[126], 7}, {NULL}};
const Segment SGI257535[] = {{&S29932[13], 4}, {NULL}};
const Segment SGI257536[] = {{&S29932[26], 5}, {NULL}};
const Segment SGI257537[] = {{&S29932[59], 6}, {NULL}};
const Segment SGI257538[] = {{&S29932[112], 6}, {NULL}};
const Segment SGI257539[] = {{&S29932[126], 7}, {NULL}};
const Segment SGI257540[] = {{&S29933[13], 4}, {NULL}};
const Segment SGI257541[] = {{&S29933[26], 3}, {NULL}};
const Segment SGI257542[] = {{&S29933[57], 6}, {NULL}};
const Segment SGI257543[] = {{&S29933[110], 6}, {NULL}};
const Segment SGI257544[] = {{&S29933[124], 7}, {NULL}};
const Segment SGI257545[] = {{&S29934[13], 4}, {NULL}};
const Segment SGI257546[] = {{&S29934[26], 5}, {NULL}};
const Segment SGI257547[] = {{&S29934[59], 6}, {NULL}};
const Segment SGI257548[] = {{&S29934[112], 6}, {NULL}};
const Segment SGI257549[] = {{&S29934[126], 7}, {NULL}};
const Segment SGI257550[] = {{&S29977[13], 6}, {NULL}};
const Segment SGI257551[] = {{&S29977[35], 5}, {NULL}};
const Segment SGI257552[] = {{&S29977[75], 6}, {NULL}};
const Segment SGI257553[] = {{&S29977[129], 5}, {NULL}};
const Segment SGI257554[] = {{&S29977[142], 9}, {NULL}};
const Segment SGI257555[] = {{&S29978[13], 6}, {NULL}};
const Segment SGI257556[] = {{&S29978[35], 3}, {NULL}};
const Segment SGI257557[] = {{&S29978[73], 6}, {NULL}};
const Segment SGI257558[] = {{&S29978[127], 5}, {NULL}};
const Segment SGI257559[] = {{&S29978[140], 9}, {NULL}};
const Segment SGI257560[] = {{&S29979[13], 6}, {NULL}};
const Segment SGI257561[] = {{&S29979[35], 5}, {NULL}};
const Segment SGI257562[] = {{&S29979[75], 6}, {NULL}};
const Segment SGI257563[] = {{&S29979[129], 5}, {NULL}};
const Segment SGI257564[] = {{&S29979[142], 9}, {NULL}};
const Segment SGI257565[] = {{&S29980[13], 6}, {NULL}};
const Segment SGI257566[] = {{&S29980[35], 3}, {NULL}};
const Segment SGI257567[] = {{&S29980[73], 6}, {NULL}};
const Segment SGI257568[] = {{&S29980[127], 5}, {NULL}};
const Segment SGI257569[] = {{&S29980[140], 9}, {NULL}};
const Segment SGI257570[] = {{&S29981[13], 6}, {NULL}};
const Segment SGI257571[] = {{&S29981[35], 3}, {NULL}};
const Segment SGI257572[] = {{&S29981[73], 6}, {NULL}};
const Segment SGI257573[] = {{&S29981[127], 5}, {NULL}};
const Segment SGI257574[] = {{&S29981[140], 9}, {NULL}};
const Segment SGI257575[] = {{&S30107[13], 4}, {NULL}};
const Segment SGI257576[] = {{&S30107[26], 3}, {NULL}};
const Segment SGI257577[] = {{&S30107[64], 8}, {NULL}};
const Segment SGI257578[] = {{&S30107[119], 8}, {NULL}};
const Segment SGI257579[] = {{&S30107[135], 9}, {NULL}};
const Segment SGI257580[] = {{&S30108[13], 4}, {NULL}};
const Segment SGI257581[] = {{&S30108[26], 3}, {NULL}};
const Segment SGI257582[] = {{&S30108[64], 8}, {NULL}};
const Segment SGI257583[] = {{&S30108[119], 6}, {NULL}};
const Segment SGI257584[] = {{&S30108[133], 9}, {NULL}};
const Segment SGI257585[] = {{&S30109[13], 4}, {NULL}};
const Segment SGI257586[] = {{&S30109[26], 5}, {NULL}};
const Segment SGI257587[] = {{&S30109[66], 8}, {NULL}};
const Segment SGI257588[] = {{&S30109[121], 6}, {NULL}};
const Segment SGI257589[] = {{&S30109[135], 9}, {NULL}};
const Segment SGI257590[] = {{&S30110[13], 4}, {NULL}};
const Segment SGI257591[] = {{&S30110[26], 5}, {NULL}};
const Segment SGI257592[] = {{&S30110[66], 8}, {NULL}};
const Segment SGI257593[] = {{&S30110[121], 6}, {NULL}};
const Segment SGI257594[] = {{&S30110[135], 9}, {NULL}};
const Segment SGI257595[] = {{&S30111[13], 4}, {NULL}};
const Segment SGI257596[] = {{&S30111[26], 5}, {NULL}};
const Segment SGI257597[] = {{&S30111[66], 8}, {NULL}};
const Segment SGI257598[] = {{&S30111[121], 6}, {NULL}};
const Segment SGI257599[] = {{&S30111[135], 9}, {NULL}};
const Segment SGI257600[] = {{&S30269[13], 6}, {NULL}};
const Segment SGI257601[] = {{&S30269[35], 5}, {NULL}};
const Segment SGI257602[] = {{&S30269[68], 6}, {NULL}};
const Segment SGI257603[] = {{&S30269[121], 6}, {NULL}};
const Segment SGI257604[] = {{&S30269[135], 7}, {NULL}};
const Segment SGI257605[] = {{&S30270[13], 6}, {NULL}};
const Segment SGI257606[] = {{&S30270[35], 3}, {NULL}};
const Segment SGI257607[] = {{&S30270[66], 6}, {NULL}};
const Segment SGI257608[] = {{&S30270[119], 6}, {NULL}};
const Segment SGI257609[] = {{&S30270[133], 7}, {NULL}};
const Segment SGI257610[] = {{&S30271[13], 6}, {NULL}};
const Segment SGI257611[] = {{&S30271[35], 3}, {NULL}};
const Segment SGI257612[] = {{&S30271[66], 6}, {NULL}};
const Segment SGI257613[] = {{&S30271[119], 6}, {NULL}};
const Segment SGI257614[] = {{&S30271[133], 7}, {NULL}};
const Segment SGI257615[] = {{&S30272[13], 6}, {NULL}};
const Segment SGI257616[] = {{&S30272[35], 5}, {NULL}};
const Segment SGI257617[] = {{&S30272[68], 6}, {NULL}};
const Segment SGI257618[] = {{&S30272[121], 6}, {NULL}};
const Segment SGI257619[] = {{&S30272[135], 7}, {NULL}};
const Segment SGI257620[] = {{&S30273[13], 6}, {NULL}};
const Segment SGI257621[] = {{&S30273[35], 5}, {NULL}};
const Segment SGI257622[] = {{&S30273[68], 6}, {NULL}};
const Segment SGI257623[] = {{&S30273[121], 6}, {NULL}};
const Segment SGI257624[] = {{&S30273[135], 7}, {NULL}};
const Segment SGI257625[] = {{&S30274[13], 4}, {NULL}};
const Segment SGI257626[] = {{&S30274[26], 3}, {NULL}};
const Segment SGI257627[] = {{&S30274[57], 6}, {NULL}};
const Segment SGI257628[] = {{&S30274[110], 6}, {NULL}};
const Segment SGI257629[] = {{&S30274[124], 7}, {NULL}};
const Segment SGI257630[] = {{&S30275[13], 4}, {NULL}};
const Segment SGI257631[] = {{&S30275[26], 3}, {NULL}};
const Segment SGI257632[] = {{&S30275[57], 6}, {NULL}};
const Segment SGI257633[] = {{&S30275[110], 6}, {NULL}};
const Segment SGI257634[] = {{&S30275[124], 7}, {NULL}};
const Segment SGI257635[] = {{&S30276[13], 4}, {NULL}};
const Segment SGI257636[] = {{&S30276[26], 5}, {NULL}};
const Segment SGI257637[] = {{&S30276[59], 6}, {NULL}};
const Segment SGI257638[] = {{&S30276[112], 6}, {NULL}};
const Segment SGI257639[] = {{&S30276[126], 7}, {NULL}};
const Segment SGI257640[] = {{&S30277[13], 4}, {NULL}};
const Segment SGI257641[] = {{&S30277[26], 3}, {NULL}};
const Segment SGI257642[] = {{&S30277[57], 6}, {NULL}};
const Segment SGI257643[] = {{&S30277[110], 6}, {NULL}};
const Segment SGI257644[] = {{&S30277[124], 7}, {NULL}};
const Segment SGI257645[] = {{&S30278[13], 4}, {NULL}};
const Segment SGI257646[] = {{&S30278[26], 5}, {NULL}};
const Segment SGI257647[] = {{&S30278[59], 6}, {NULL}};
const Segment SGI257648[] = {{&S30278[112], 7}, {NULL}};
const Segment SGI257649[] = {{&S30278[127], 7}, {NULL}};
const Segment SGI257650[] = {{&S30279[13], 4}, {NULL}};
const Segment SGI257651[] = {{&S30279[26], 5}, {NULL}};
const Segment SGI257652[] = {{&S30279[59], 6}, {NULL}};
const Segment SGI257653[] = {{&S30279[112], 6}, {NULL}};
const Segment SGI257654[] = {{&S30279[126], 7}, {NULL}};
const Segment SGI257655[] = {{&S30280[13], 4}, {NULL}};
const Segment SGI257656[] = {{&S30280[26], 5}, {NULL}};
const Segment SGI257657[] = {{&S30280[59], 6}, {NULL}};
const Segment SGI257658[] = {{&S30280[112], 7}, {NULL}};
const Segment SGI257659[] = {{&S30280[127], 7}, {NULL}};
const Segment SGI257660[] = {{&S30570[13], 6}, {NULL}};
const Segment SGI257661[] = {{&S30570[35], 3}, {NULL}};
const Segment SGI257662[] = {{&S30570[66], 8}, {NULL}};
const Segment SGI257663[] = {{&S30570[121], 8}, {NULL}};
const Segment SGI257664[] = {{&S30570[137], 7}, {NULL}};
const Segment SGI257665[] = {{&S30571[13], 6}, {NULL}};
const Segment SGI257666[] = {{&S30571[35], 3}, {NULL}};
const Segment SGI257667[] = {{&S30571[66], 8}, {NULL}};
const Segment SGI257668[] = {{&S30571[121], 6}, {NULL}};
const Segment SGI257669[] = {{&S30571[135], 7}, {NULL}};
const Segment SGI257670[] = {{&S30572[13], 6}, {NULL}};
const Segment SGI257671[] = {{&S30572[35], 3}, {NULL}};
const Segment SGI257672[] = {{&S30572[66], 8}, {NULL}};
const Segment SGI257673[] = {{&S30572[121], 8}, {NULL}};
const Segment SGI257674[] = {{&S30572[137], 7}, {NULL}};
const Segment SGI257675[] = {{&S30573[13], 6}, {NULL}};
const Segment SGI257676[] = {{&S30573[35], 3}, {NULL}};
const Segment SGI257677[] = {{&S30573[66], 8}, {NULL}};
const Segment SGI257678[] = {{&S30573[121], 6}, {NULL}};
const Segment SGI257679[] = {{&S30573[135], 7}, {NULL}};
const Segment SGI257680[] = {{&S30574[13], 6}, {NULL}};
const Segment SGI257681[] = {{&S30574[35], 5}, {NULL}};
const Segment SGI257682[] = {{&S30574[68], 8}, {NULL}};
const Segment SGI257683[] = {{&S30574[123], 6}, {NULL}};
const Segment SGI257684[] = {{&S30574[137], 7}, {NULL}};
const Segment SGI257685[] = {{&S30575[13], 6}, {NULL}};
const Segment SGI257686[] = {{&S30575[35], 5}, {NULL}};
const Segment SGI257687[] = {{&S30575[68], 8}, {NULL}};
const Segment SGI257688[] = {{&S30575[123], 7}, {NULL}};
const Segment SGI257689[] = {{&S30575[138], 7}, {NULL}};
const Segment SGI257690[] = {{&S31102[13], 6}, {NULL}};
const Segment SGI257691[] = {{&S31102[35], 3}, {NULL}};
const Segment SGI257692[] = {{&S31102[73], 8}, {NULL}};
const Segment SGI257693[] = {{&S31102[128], 8}, {NULL}};
const Segment SGI257694[] = {{&S31102[144], 9}, {NULL}};
const Segment SGI257695[] = {{&S31103[13], 6}, {NULL}};
const Segment SGI257696[] = {{&S31103[35], 3}, {NULL}};
const Segment SGI257697[] = {{&S31103[73], 8}, {NULL}};
const Segment SGI257698[] = {{&S31103[128], 8}, {NULL}};
const Segment SGI257699[] = {{&S31103[144], 9}, {NULL}};
const Segment SGI257700[] = {{&S31104[13], 6}, {NULL}};
const Segment SGI257701[] = {{&S31104[35], 3}, {NULL}};
const Segment SGI257702[] = {{&S31104[73], 8}, {NULL}};
const Segment SGI257703[] = {{&S31104[128], 6}, {NULL}};
const Segment SGI257704[] = {{&S31104[142], 9}, {NULL}};
const Segment SGI257705[] = {{&S31105[13], 6}, {NULL}};
const Segment SGI257706[] = {{&S31105[35], 3}, {NULL}};
const Segment SGI257707[] = {{&S31105[73], 8}, {NULL}};
const Segment SGI257708[] = {{&S31105[128], 6}, {NULL}};
const Segment SGI257709[] = {{&S31105[142], 9}, {NULL}};
const Segment SGI257710[] = {{&S31106[13], 6}, {NULL}};
const Segment SGI257711[] = {{&S31106[35], 3}, {NULL}};
const Segment SGI257712[] = {{&S31106[73], 8}, {NULL}};
const Segment SGI257713[] = {{&S31106[128], 6}, {NULL}};
const Segment SGI257714[] = {{&S31106[142], 9}, {NULL}};
const Segment SGI257715[] = {{&S31107[13], 6}, {NULL}};
const Segment SGI257716[] = {{&S31107[35], 3}, {NULL}};
const Segment SGI257717[] = {{&S31107[73], 8}, {NULL}};
const Segment SGI257718[] = {{&S31107[128], 6}, {NULL}};
const Segment SGI257719[] = {{&S31107[142], 9}, {NULL}};
const Segment SGI257720[] = {{&S31108[13], 6}, {NULL}};
const Segment SGI257721[] = {{&S31108[35], 5}, {NULL}};
const Segment SGI257722[] = {{&S31108[75], 8}, {NULL}};
const Segment SGI257723[] = {{&S31108[130], 6}, {NULL}};
const Segment SGI257724[] = {{&S31108[144], 9}, {NULL}};
const Segment SGI257725[] = {{&S31109[13], 6}, {NULL}};
const Segment SGI257726[] = {{&S31109[35], 5}, {NULL}};
const Segment SGI257727[] = {{&S31109[75], 8}, {NULL}};
const Segment SGI257728[] = {{&S31109[130], 6}, {NULL}};
const Segment SGI257729[] = {{&S31109[144], 9}, {NULL}};
const Segment SGI257730[] = {{&S31110[13], 6}, {NULL}};
const Segment SGI257731[] = {{&S31110[35], 5}, {NULL}};
const Segment SGI257732[] = {{&S31110[75], 8}, {NULL}};
const Segment SGI257733[] = {{&S31110[130], 6}, {NULL}};
const Segment SGI257734[] = {{&S31110[144], 9}, {NULL}};
const Segment SGI257735[] = {{&S31111[13], 6}, {NULL}};
const Segment SGI257736[] = {{&S31111[35], 5}, {NULL}};
const Segment SGI257737[] = {{&S31111[75], 8}, {NULL}};
const Segment SGI257738[] = {{&S31111[130], 6}, {NULL}};
const Segment SGI257739[] = {{&S31111[144], 9}, {NULL}};
const Segment SGI257740[] = {{&S31959[13], 4}, {NULL}};
const Segment SGI257741[] = {{&S31959[26], 5}, {NULL}};
const Segment SGI257742[] = {{&S31959[59], 6}, {NULL}};
const Segment SGI257743[] = {{&S31959[113], 6}, {NULL}};
const Segment SGI257744[] = {{&S31959[127], 7}, {NULL}};
const Segment SGI257745[] = {{&S31960[13], 4}, {NULL}};
const Segment SGI257746[] = {{&S31960[26], 3}, {NULL}};
const Segment SGI257747[] = {{&S31960[57], 6}, {NULL}};
const Segment SGI257748[] = {{&S31960[111], 6}, {NULL}};
const Segment SGI257749[] = {{&S31960[125], 7}, {NULL}};
const Segment SGI257750[] = {{&S31961[13], 4}, {NULL}};
const Segment SGI257751[] = {{&S31961[26], 5}, {NULL}};
const Segment SGI257752[] = {{&S31961[59], 6}, {NULL}};
const Segment SGI257753[] = {{&S31961[113], 7}, {NULL}};
const Segment SGI257754[] = {{&S31961[128], 7}, {NULL}};
const Segment SGI257755[] = {{&S31962[13], 4}, {NULL}};
const Segment SGI257756[] = {{&S31962[26], 5}, {NULL}};
const Segment SGI257757[] = {{&S31962[59], 6}, {NULL}};
const Segment SGI257758[] = {{&S31962[113], 6}, {NULL}};
const Segment SGI257759[] = {{&S31962[127], 7}, {NULL}};
const Segment SGI257760[] = {{&S31963[13], 4}, {NULL}};
const Segment SGI257761[] = {{&S31963[26], 3}, {NULL}};
const Segment SGI257762[] = {{&S31963[57], 6}, {NULL}};
const Segment SGI257763[] = {{&S31963[111], 6}, {NULL}};
const Segment SGI257764[] = {{&S31963[125], 7}, {NULL}};
const Segment SGI257765[] = {{&S31964[13], 4}, {NULL}};
const Segment SGI257766[] = {{&S31964[26], 5}, {NULL}};
const Segment SGI257767[] = {{&S31964[59], 6}, {NULL}};
const Segment SGI257768[] = {{&S31964[113], 7}, {NULL}};
const Segment SGI257769[] = {{&S31964[128], 7}, {NULL}};
const Segment SGI257770[] = {{&S31965[13], 4}, {NULL}};
const Segment SGI257771[] = {{&S31965[26], 3}, {NULL}};
const Segment SGI257772[] = {{&S31965[57], 6}, {NULL}};
const Segment SGI257773[] = {{&S31965[111], 6}, {NULL}};
const Segment SGI257774[] = {{&S31965[125], 7}, {NULL}};
const Segment SGI257775[] = {{&S32020[13], 4}, {NULL}};
const Segment SGI257776[] = {{&S32020[36], 5}, {NULL}};
const Segment SGI257777[] = {{&S32020[76], 8}, {NULL}};
const Segment SGI257778[] = {{&S32021[13], 4}, {NULL}};
const Segment SGI257779[] = {{&S32021[36], 5}, {NULL}};
const Segment SGI257780[] = {{&S32021[76], 8}, {NULL}};
const Segment SGI257781[] = {{&S32022[13], 4}, {NULL}};
const Segment SGI257782[] = {{&S32022[36], 5}, {NULL}};
const Segment SGI257783[] = {{&S32022[76], 8}, {NULL}};
const Segment SGI257784[] = {{&S32022[112], 5}, {NULL}};
const Segment SGI257785[] = {{&S32023[13], 4}, {NULL}};
const Segment SGI257786[] = {{&S32023[36], 5}, {NULL}};
const Segment SGI257787[] = {{&S32023[76], 8}, {NULL}};
const Segment SGI257788[] = {{&S32023[112], 5}, {NULL}};
const Segment SGI257789[] = {{&S32024[13], 4}, {NULL}};
const Segment SGI257790[] = {{&S32024[142], 3}, {NULL}};
const Segment SGI257791[] = {{&S32024[36], 5}, {NULL}};
const Segment SGI257792[] = {{&S32024[76], 8}, {NULL}};
const Segment SGI257793[] = {{&S32024[112], 5}, {NULL}};
const Segment SGI257794[] = {{&S32025[13], 4}, {&S32025[13], 4}, {&S32025[13], 4}, {NULL}};
const Segment SGI257795[] = {{&S32025[42], 5}, {NULL}};
const Segment SGI257796[] = {{&S32026[13], 4}, {&S32026[13], 4}, {NULL}};
const Segment SGI257797[] = {{&S32026[36], 5}, {NULL}};
const Segment SGI257798[] = {{&S32026[76], 8}, {NULL}};
const Segment SGI257799[] = {{&S32026[112], 5}, {NULL}};
const Segment SGI257800[] = {{&S32027[13], 6}, {&S32027[128], 4}, {NULL}};
const Segment SGI257801[] = {{&S32027[75], 6}, {NULL}};
const Segment SGI257802[] = {{&S32027[109], 4}, {NULL}};
const Segment SGI257803[] = {{&S32028[13], 4}, {&S32028[13], 4}, {NULL}};
const Segment SGI257804[] = {{&S32028[36], 3}, {NULL}};
const Segment SGI257805[] = {{&S32028[74], 8}, {NULL}};
const Segment SGI257806[] = {{&S32028[109], 5}, {NULL}};
const Segment SGI257807[] = {{&S32029[17], 4}, {NULL}};
const Segment SGI257808[] = {{&S32029[39], 5}, {NULL}};
const Segment SGI257809[] = {{&S32214[13], 6}, {NULL}};
const Segment SGI257810[] = {{&S32214[35], 5}, {NULL}};
const Segment SGI257811[] = {{&S32214[68], 6}, {NULL}};
const Segment SGI257812[] = {{&S32214[122], 6}, {NULL}};
const Segment SGI257813[] = {{&S32214[136], 7}, {NULL}};
const Segment SGI257814[] = {{&S32215[13], 6}, {NULL}};
const Segment SGI257815[] = {{&S32215[35], 3}, {NULL}};
const Segment SGI257816[] = {{&S32215[66], 6}, {NULL}};
const Segment SGI257817[] = {{&S32215[120], 6}, {NULL}};
const Segment SGI257818[] = {{&S32215[134], 7}, {NULL}};
const Segment SGI257819[] = {{&S32216[13], 6}, {NULL}};
const Segment SGI257820[] = {{&S32216[35], 5}, {NULL}};
const Segment SGI257821[] = {{&S32216[68], 6}, {NULL}};
const Segment SGI257822[] = {{&S32216[122], 6}, {NULL}};
const Segment SGI257823[] = {{&S32216[136], 7}, {NULL}};
const Segment SGI257824[] = {{&S32217[13], 6}, {NULL}};
const Segment SGI257825[] = {{&S32217[35], 5}, {NULL}};
const Segment SGI257826[] = {{&S32217[68], 6}, {NULL}};
const Segment SGI257827[] = {{&S32217[122], 6}, {NULL}};
const Segment SGI257828[] = {{&S32217[136], 7}, {NULL}};
const Segment SGI257829[] = {{&S32218[13], 6}, {NULL}};
const Segment SGI257830[] = {{&S32218[35], 3}, {NULL}};
const Segment SGI257831[] = {{&S32218[66], 6}, {NULL}};
const Segment SGI257832[] = {{&S32218[120], 6}, {NULL}};
const Segment SGI257833[] = {{&S32218[134], 7}, {NULL}};
const Segment SGI257834[] = {{&S28733[53], 5}, {NULL}};
const Segment SGI257835[] = {{&S32485[13], 4}, {NULL}};
const Segment SGI257836[] = {{&S32485[26], 5}, {NULL}};
const Segment SGI257837[] = {{&S32485[59], 6}, {NULL}};
const Segment SGI257838[] = {{&S32485[112], 6}, {NULL}};
const Segment SGI257839[] = {{&S32485[126], 7}, {NULL}};
const Segment SGI257840[] = {{&S32486[13], 4}, {NULL}};
const Segment SGI257841[] = {{&S32486[26], 3}, {NULL}};
const Segment SGI257842[] = {{&S32486[57], 6}, {NULL}};
const Segment SGI257843[] = {{&S32486[110], 6}, {NULL}};
const Segment SGI257844[] = {{&S32486[124], 7}, {NULL}};
const Segment SGI257845[] = {{&S32487[13], 4}, {NULL}};
const Segment SGI257846[] = {{&S32487[26], 3}, {NULL}};
const Segment SGI257847[] = {{&S32487[57], 6}, {NULL}};
const Segment SGI257848[] = {{&S32487[110], 6}, {NULL}};
const Segment SGI257849[] = {{&S32487[124], 7}, {NULL}};
const Segment SGI257850[] = {{&S32488[13], 4}, {NULL}};
const Segment SGI257851[] = {{&S32488[26], 5}, {NULL}};
const Segment SGI257852[] = {{&S32488[59], 6}, {NULL}};
const Segment SGI257853[] = {{&S32488[112], 6}, {NULL}};
const Segment SGI257854[] = {{&S32488[126], 7}, {NULL}};
const Segment SGI257855[] = {{&S32489[13], 4}, {NULL}};
const Segment SGI257856[] = {{&S32489[26], 5}, {NULL}};
const Segment SGI257857[] = {{&S32489[59], 6}, {NULL}};
const Segment SGI257858[] = {{&S32489[112], 7}, {NULL}};
const Segment SGI257859[] = {{&S32489[127], 7}, {NULL}};
const Segment SGI257860[] = {{&S32490[13], 4}, {NULL}};
const Segment SGI257861[] = {{&S32490[26], 3}, {NULL}};
const Segment SGI257862[] = {{&S32490[57], 6}, {NULL}};
const Segment SGI257863[] = {{&S32490[110], 6}, {NULL}};
const Segment SGI257864[] = {{&S32490[124], 7}, {NULL}};
const Segment SGI257865[] = {{&S32491[13], 4}, {NULL}};
const Segment SGI257866[] = {{&S32491[26], 5}, {NULL}};
const Segment SGI257867[] = {{&S32491[59], 6}, {NULL}};
const Segment SGI257868[] = {{&S32491[112], 7}, {NULL}};
const Segment SGI257869[] = {{&S32491[127], 7}, {NULL}};
const Segment SGI257870[] = {{&S32616[13], 4}, {NULL}};
const Segment SGI257871[] = {{&S32616[26], 3}, {NULL}};
const Segment SGI257872[] = {{&S32616[57], 8}, {NULL}};
const Segment SGI257873[] = {{&S32616[111], 5}, {NULL}};
const Segment SGI257874[] = {{&S32616[124], 7}, {NULL}};
const Segment SGI257875[] = {{&S32617[13], 4}, {NULL}};
const Segment SGI257876[] = {{&S32617[26], 3}, {NULL}};
const Segment SGI257877[] = {{&S32617[57], 8}, {NULL}};
const Segment SGI257878[] = {{&S32617[112], 6}, {NULL}};
const Segment SGI257879[] = {{&S32617[126], 7}, {NULL}};
const Segment SGI257880[] = {{&S32618[13], 4}, {NULL}};
const Segment SGI257881[] = {{&S32618[26], 3}, {NULL}};
const Segment SGI257882[] = {{&S32618[57], 8}, {NULL}};
const Segment SGI257883[] = {{&S32618[112], 8}, {NULL}};
const Segment SGI257884[] = {{&S32618[128], 7}, {NULL}};
const Segment SGI257885[] = {{&S32619[13], 4}, {NULL}};
const Segment SGI257886[] = {{&S32619[26], 3}, {NULL}};
const Segment SGI257887[] = {{&S32619[57], 8}, {NULL}};
const Segment SGI257888[] = {{&S32619[112], 8}, {NULL}};
const Segment SGI257889[] = {{&S32619[128], 7}, {NULL}};
const Segment SGI257890[] = {{&S32620[13], 4}, {NULL}};
const Segment SGI257891[] = {{&S32620[26], 5}, {NULL}};
const Segment SGI257892[] = {{&S32620[59], 8}, {NULL}};
const Segment SGI257893[] = {{&S32620[114], 6}, {NULL}};
const Segment SGI257894[] = {{&S32620[128], 7}, {NULL}};
const Segment SGI257895[] = {{&S32652[13], 4}, {NULL}};
const Segment SGI257896[] = {{&S32652[108], 5}, {NULL}};
const Segment SGI257897[] = {{&S32652[36], 3}, {NULL}};
const Segment SGI257898[] = {{&S32652[74], 5}, {NULL}};
const Segment SGI257899[] = {{&S32652[121], 5}, {NULL}};
const Segment SGI257900[] = {{&S32652[134], 9}, {NULL}};
const Segment SGI257901[] = {{&S32653[13], 4}, {NULL}};
const Segment SGI257902[] = {{&S32653[108], 5}, {NULL}};
const Segment SGI257903[] = {{&S32653[36], 3}, {NULL}};
const Segment SGI257904[] = {{&S32653[74], 5}, {NULL}};
const Segment SGI257905[] = {{&S32653[121], 5}, {NULL}};
const Segment SGI257906[] = {{&S32653[134], 9}, {NULL}};
const Segment SGI257907[] = {{&S32654[13], 4}, {NULL}};
const Segment SGI257908[] = {{&S32654[108], 5}, {NULL}};
const Segment SGI257909[] = {{&S32654[36], 3}, {NULL}};
const Segment SGI257910[] = {{&S32654[74], 5}, {NULL}};
const Segment SGI257911[] = {{&S32654[121], 5}, {NULL}};
const Segment SGI257912[] = {{&S32654[134], 9}, {NULL}};
const Segment SGI257913[] = {{&S32655[13], 4}, {NULL}};
const Segment SGI257914[] = {{&S32655[115], 3}, {NULL}};
const Segment SGI257915[] = {{&S32655[42], 3}, {NULL}};
const Segment SGI257916[] = {{&S32655[80], 6}, {NULL}};
const Segment SGI257917[] = {{&S32655[126], 5}, {NULL}};
const Segment SGI257918[] = {{&S32655[139], 6}, {NULL}};
const Segment SGI257919[] = {{&S32656[13], 4}, {NULL}};
const Segment SGI257920[] = {{&S32656[106], 5}, {NULL}};
const Segment SGI257921[] = {{&S32656[36], 2}, {NULL}};
const Segment SGI257922[] = {{&S32656[73], 5}, {NULL}};
const Segment SGI257923[] = {{&S32656[119], 5}, {NULL}};
const Segment SGI257924[] = {{&S32656[132], 9}, {NULL}};
const Segment SGI257925[] = {{&S32657[13], 4}, {NULL}};
const Segment SGI257926[] = {{&S32657[106], 5}, {NULL}};
const Segment SGI257927[] = {{&S32657[36], 2}, {NULL}};
const Segment SGI257928[] = {{&S32657[73], 5}, {NULL}};
const Segment SGI257929[] = {{&S32657[119], 5}, {NULL}};
const Segment SGI257930[] = {{&S32657[132], 9}, {NULL}};
const Segment SGI257931[] = {{&S32856[13], 6}, {NULL}};
const Segment SGI257932[] = {{&S32856[35], 3}, {NULL}};
const Segment SGI257933[] = {{&S32856[66], 6}, {NULL}};
const Segment SGI257934[] = {{&S32856[100], 8}, {NULL}};
const Segment SGI257935[] = {{&S32856[109], 7}, {NULL}};
const Segment SGI257936[] = {{&S32857[13], 6}, {NULL}};
const Segment SGI257937[] = {{&S32857[35], 3}, {NULL}};
const Segment SGI257938[] = {{&S32857[66], 6}, {NULL}};
const Segment SGI257939[] = {{&S32857[100], 6}, {NULL}};
const Segment SGI257940[] = {{&S32857[107], 7}, {NULL}};
const Segment SGI257941[] = {{&S32858[13], 6}, {NULL}};
const Segment SGI257942[] = {{&S32858[35], 3}, {NULL}};
const Segment SGI257943[] = {{&S32858[66], 6}, {NULL}};
const Segment SGI257944[] = {{&S32858[100], 6}, {NULL}};
const Segment SGI257945[] = {{&S32858[107], 7}, {NULL}};
const Segment SGI257946[] = {{&S32859[13], 6}, {NULL}};
const Segment SGI257947[] = {{&S32859[35], 5}, {NULL}};
const Segment SGI257948[] = {{&S32859[68], 6}, {NULL}};
const Segment SGI257949[] = {{&S32859[102], 6}, {NULL}};
const Segment SGI257950[] = {{&S32859[109], 7}, {NULL}};
const Segment SGI257951[] = {{&S32860[13], 6}, {NULL}};
const Segment SGI257952[] = {{&S32860[35], 5}, {NULL}};
const Segment SGI257953[] = {{&S32860[68], 6}, {NULL}};
const Segment SGI257954[] = {{&S32860[102], 6}, {NULL}};
const Segment SGI257955[] = {{&S32860[109], 7}, {NULL}};
const Segment SGI257956[] = {{&S32861[13], 6}, {NULL}};
const Segment SGI257957[] = {{&S32861[35], 5}, {NULL}};
const Segment SGI257958[] = {{&S32861[68], 6}, {NULL}};
const Segment SGI257959[] = {{&S32861[102], 6}, {NULL}};
const Segment SGI257960[] = {{&S32861[109], 7}, {NULL}};
const Segment SGI257961[] = {{&S32952[13], 6}, {NULL}};
const Segment SGI257962[] = {{&S32952[113], 3}, {NULL}};
const Segment SGI257963[] = {{&S32952[38], 5}, {NULL}};
const Segment SGI257964[] = {{&S32952[78], 6}, {NULL}};
const Segment SGI257965[] = {{&S32952[124], 5}, {NULL}};
const Segment SGI257966[] = {{&S32952[137], 6}, {NULL}};
const Segment SGI257967[] = {{&S32953[13], 6}, {NULL}};
const Segment SGI257968[] = {{&S32953[111], 3}, {NULL}};
const Segment SGI257969[] = {{&S32953[38], 3}, {NULL}};
const Segment SGI257970[] = {{&S32953[76], 6}, {NULL}};
const Segment SGI257971[] = {{&S32953[122], 5}, {NULL}};
const Segment SGI257972[] = {{&S32953[135], 6}, {NULL}};
const Segment SGI257973[] = {{&S32954[13], 6}, {NULL}};
const Segment SGI257974[] = {{&S32954[110], 5}, {NULL}};
const Segment SGI257975[] = {{&S32954[38], 3}, {NULL}};
const Segment SGI257976[] = {{&S32954[76], 5}, {NULL}};
const Segment SGI257977[] = {{&S32954[123], 5}, {NULL}};
const Segment SGI257978[] = {{&S32954[136], 9}, {NULL}};
const Segment SGI257979[] = {{&S32955[13], 6}, {NULL}};
const Segment SGI257980[] = {{&S32955[109], 3}, {NULL}};
const Segment SGI257981[] = {{&S32955[38], 2}, {NULL}};
const Segment SGI257982[] = {{&S32955[75], 6}, {NULL}};
const Segment SGI257983[] = {{&S32955[120], 5}, {NULL}};
const Segment SGI257984[] = {{&S32955[133], 6}, {NULL}};
const Segment SGI257985[] = {{&S22275[0], 15}, {&S22275[16], 44}, {NULL}};
const Segment SGI257986[] = {{&S22275[0], 60}, {NULL}};
const Segment SGI257987[] = {{&S22276[0], 15}, {&S22276[16], 37}, {NULL}};
const Segment SGI257988[] = {{&S22276[0], 78}, {NULL}};
const Segment SGI257989[] = {{&S22277[0], 13}, {&S22277[23], 31}, {NULL}};
const Segment SGI257990[] = {{&S22277[0], 54}, {NULL}};
const Segment SGI257991[] = {{&S22278[25], 17}, {&S22278[43], 35}, {NULL}};
const Segment SGI257992[] = {{&S22278[0], 78}, {NULL}};
const Segment SGI257993[] = {{&S22279[25], 11}, {&S22279[37], 36}, {NULL}};
const Segment SGI257994[] = {{&S22279[0], 73}, {NULL}};
const Segment SGI257995[] = {{&S22280[25], 3}, {&S22280[30], 6}, {&S22280[38], 17}, {NULL}};
const Segment SGI257996[] = {{&S22280[0], 55}, {NULL}};
const Segment SGI257997[] = {{&S22281[31], 7}, {&S22281[39], 6}, {NULL}};
const Segment SGI257998[] = {{&S22281[0], 86}, {NULL}};
const Segment SGI257999[] = {{&S22282[0], 9}, {&S22282[37], 29}, {NULL}};
const Segment SGI258000[] = {{&S22282[0], 66}, {NULL}};
const Segment SGI258001[] = {{&S22283[25], 9}, {&S22283[35], 38}, {NULL}};
const Segment SGI258002[] = {{&S22283[0], 73}, {NULL}};
const Segment SGI258003[] = {{&S22284[0], 13}, {&S22284[21], 31}, {NULL}};
const Segment SGI258004[] = {{&S22284[0], 52}, {NULL}};
const Segment SGI258005[] = {{&S22285[62], 25}, {NULL}};
const Segment SGI258006[] = {{&S22285[0], 87}, {NULL}};
const Segment SGI258007[] = {{&S22286[18], 6}, {&S22286[24], 8}, {NULL}};
const Segment SGI258008[] = {{&S22286[0], 63}, {NULL}};
const Segment SGI258009[] = {{&S22287[0], 16}, {NULL}};
const Segment SGI258010[] = {{&S22288[25], 16}, {&S22288[42], 51}, {NULL}};
const Segment SGI258011[] = {{&S22288[0], 93}, {NULL}};
const Segment SGI258012[] = {{&S22289[25], 14}, {&S22289[47], 38}, {NULL}};
const Segment SGI258013[] = {{&S22289[0], 85}, {NULL}};
const Segment SGI258014[] = {{&S22290[0], 33}, {&S22290[34], 13}, {NULL}};
const Segment SGI258015[] = {{&S22290[0], 47}, {NULL}};
const Segment SGI258016[] = {{&S22291[25], 12}, {&S22291[38], 34}, {NULL}};
const Segment SGI258017[] = {{&S22291[0], 72}, {NULL}};
const Segment SGI258018[] = {{&S22292[8], 7}, {&S22292[16], 6}, {NULL}};
const Segment SGI258019[] = {{&S22292[0], 22}, {NULL}};
const Segment SGI258020[] = {{&S22293[25], 12}, {&S22293[38], 41}, {NULL}};
const Segment SGI258021[] = {{&S22293[0], 79}, {NULL}};
const Segment SGI258022[] = {{&S22294[25], 14}, {&S22294[40], 37}, {NULL}};
const Segment SGI258023[] = {{&S22294[0], 77}, {NULL}};
const Segment SGI258024[] = {{&S22295[25], 14}, {&S22295[40], 41}, {NULL}};
const Segment SGI258025[] = {{&S22295[0], 81}, {NULL}};
const Segment SGI258026[] = {{&S22296[25], 16}, {&S22296[42], 33}, {NULL}};
const Segment SGI258027[] = {{&S22296[0], 75}, {NULL}};
const Segment SGI258028[] = {{&S22297[25], 17}, {&S22297[43], 41}, {NULL}};
const Segment SGI258029[] = {{&S22297[0], 84}, {NULL}};
const Segment SGI258030[] = {{&S15505[22], 12}, {NULL}};
const Segment SGI258031[] = {{&S33373[13], 6}, {NULL}};
const Segment SGI258032[] = {{&S33373[35], 5}, {NULL}};
const Segment SGI258033[] = {{&S33373[68], 6}, {NULL}};
const Segment SGI258034[] = {{&S33373[121], 6}, {NULL}};
const Segment SGI258035[] = {{&S33373[135], 7}, {NULL}};
const Segment SGI258036[] = {{&S33374[13], 6}, {NULL}};
const Segment SGI258037[] = {{&S33374[35], 3}, {NULL}};
const Segment SGI258038[] = {{&S33374[66], 6}, {NULL}};
const Segment SGI258039[] = {{&S33374[119], 6}, {NULL}};
const Segment SGI258040[] = {{&S33374[133], 7}, {NULL}};
const Segment SGI258041[] = {{&S33375[13], 6}, {NULL}};
const Segment SGI258042[] = {{&S33375[35], 3}, {NULL}};
const Segment SGI258043[] = {{&S33375[66], 6}, {NULL}};
const Segment SGI258044[] = {{&S33375[119], 6}, {NULL}};
const Segment SGI258045[] = {{&S33375[133], 7}, {NULL}};
const Segment SGI258046[] = {{&S33376[13], 6}, {NULL}};
const Segment SGI258047[] = {{&S33376[35], 5}, {NULL}};
const Segment SGI258048[] = {{&S33376[68], 6}, {NULL}};
const Segment SGI258049[] = {{&S33376[121], 6}, {NULL}};
const Segment SGI258050[] = {{&S33376[135], 7}, {NULL}};
const Segment SGI258051[] = {{&S33377[13], 6}, {NULL}};
const Segment SGI258052[] = {{&S33377[35], 5}, {NULL}};
const Segment SGI258053[] = {{&S33377[68], 6}, {NULL}};
const Segment SGI258054[] = {{&S33377[121], 6}, {NULL}};
const Segment SGI258055[] = {{&S33377[135], 7}, {NULL}};
const Segment SGI258056[] = {{&S33378[13], 4}, {&S33378[13], 4}, {NULL}};
const Segment SGI258057[] = {{&S33378[119], 3}, {NULL}};
const Segment SGI258058[] = {{&S33378[29], 3}, {NULL}};
const Segment SGI258059[] = {{&S33378[84], 6}, {NULL}};
const Segment SGI258060[] = {{&S33378[130], 5}, {NULL}};
const Segment SGI258061[] = {{&S33378[143], 9}, {NULL}};
const Segment SGI258062[] = {{&S33379[13], 4}, {NULL}};
const Segment SGI258063[] = {{&S33379[113], 3}, {NULL}};
const Segment SGI258064[] = {{&S33379[26], 3}, {NULL}};
const Segment SGI258065[] = {{&S33379[57], 6}, {NULL}};
const Segment SGI258066[] = {{&S33379[91], 6}, {NULL}};
const Segment SGI258067[] = {{&S33380[13], 4}, {NULL}};
const Segment SGI258068[] = {{&S33380[92], 3}, {NULL}};
const Segment SGI258069[] = {{&S33380[26], 3}, {NULL}};
const Segment SGI258070[] = {{&S33380[57], 6}, {NULL}};
const Segment SGI258071[] = {{&S33380[103], 6}, {NULL}};
const Segment SGI258072[] = {{&S33380[117], 7}, {NULL}};
const Segment SGI258073[] = {{&S33381[13], 4}, {NULL}};
const Segment SGI258074[] = {{&S33381[105], 3}, {NULL}};
const Segment SGI258075[] = {{&S33381[26], 3}, {NULL}};
const Segment SGI258076[] = {{&S33381[57], 6}, {NULL}};
const Segment SGI258077[] = {{&S33381[91], 5}, {NULL}};
const Segment SGI258078[] = {{&S33381[116], 9}, {NULL}};
const Segment SGI258079[] = {{&S33382[13], 4}, {NULL}};
const Segment SGI258080[] = {{&S33382[106], 3}, {NULL}};
const Segment SGI258081[] = {{&S33382[26], 3}, {NULL}};
const Segment SGI258082[] = {{&S33382[57], 6}, {NULL}};
const Segment SGI258083[] = {{&S33382[91], 6}, {NULL}};
const Segment SGI258084[] = {{&S33382[117], 9}, {NULL}};
const Segment SGI258085[] = {{&S33383[13], 4}, {NULL}};
const Segment SGI258086[] = {{&S33383[94], 3}, {NULL}};
const Segment SGI258087[] = {{&S33383[26], 5}, {NULL}};
const Segment SGI258088[] = {{&S33383[59], 6}, {NULL}};
const Segment SGI258089[] = {{&S33383[105], 5}, {NULL}};
const Segment SGI258090[] = {{&S33383[118], 9}, {NULL}};
const Segment SGI258091[] = {{&S33384[13], 4}, {NULL}};
const Segment SGI258092[] = {{&S33384[108], 3}, {NULL}};
const Segment SGI258093[] = {{&S33384[26], 5}, {NULL}};
const Segment SGI258094[] = {{&S33384[59], 6}, {NULL}};
const Segment SGI258095[] = {{&S33384[93], 6}, {NULL}};
const Segment SGI258096[] = {{&S33384[119], 9}, {NULL}};
const Segment SGI258097[] = {{&S33385[13], 4}, {NULL}};
const Segment SGI258098[] = {{&S33385[94], 3}, {NULL}};
const Segment SGI258099[] = {{&S33385[26], 5}, {NULL}};
const Segment SGI258100[] = {{&S33385[59], 6}, {NULL}};
const Segment SGI258101[] = {{&S33385[105], 6}, {NULL}};
const Segment SGI258102[] = {{&S33385[119], 7}, {NULL}};
const Segment SGI258103[] = {{&S33386[13], 4}, {NULL}};
const Segment SGI258104[] = {{&S33386[115], 3}, {NULL}};
const Segment SGI258105[] = {{&S33386[26], 5}, {NULL}};
const Segment SGI258106[] = {{&S33386[59], 6}, {NULL}};
const Segment SGI258107[] = {{&S33386[93], 6}, {NULL}};
const Segment SGI258108[] = {{&S33387[13], 4}, {NULL}};
const Segment SGI258109[] = {{&S33387[92], 3}, {NULL}};
const Segment SGI258110[] = {{&S33387[26], 3}, {NULL}};
const Segment SGI258111[] = {{&S33387[57], 6}, {NULL}};
const Segment SGI258112[] = {{&S33387[103], 5}, {NULL}};
const Segment SGI258113[] = {{&S33387[116], 9}, {NULL}};
const Segment SGI258114[] = {{&S33388[13], 4}, {NULL}};
const Segment SGI258115[] = {{&S33388[115], 3}, {NULL}};
const Segment SGI258116[] = {{&S33388[26], 5}, {NULL}};
const Segment SGI258117[] = {{&S33388[59], 6}, {NULL}};
const Segment SGI258118[] = {{&S33388[93], 6}, {NULL}};
const Segment SGI258119[] = {{&S33389[13], 4}, {NULL}};
const Segment SGI258120[] = {{&S33389[94], 3}, {NULL}};
const Segment SGI258121[] = {{&S33389[26], 5}, {NULL}};
const Segment SGI258122[] = {{&S33389[59], 6}, {NULL}};
const Segment SGI258123[] = {{&S33389[105], 5}, {NULL}};
const Segment SGI258124[] = {{&S33389[118], 9}, {NULL}};
const Segment SGI258125[] = {{&S33390[13], 4}, {NULL}};
const Segment SGI258126[] = {{&S33390[115], 3}, {NULL}};
const Segment SGI258127[] = {{&S33390[26], 5}, {NULL}};
const Segment SGI258128[] = {{&S33390[59], 6}, {NULL}};
const Segment SGI258129[] = {{&S33390[93], 6}, {NULL}};
const Segment SGI258130[] = {{&S33391[13], 4}, {NULL}};
const Segment SGI258131[] = {{&S33391[94], 3}, {NULL}};
const Segment SGI258132[] = {{&S33391[26], 5}, {NULL}};
const Segment SGI258133[] = {{&S33391[59], 6}, {NULL}};
const Segment SGI258134[] = {{&S33391[105], 6}, {NULL}};
const Segment SGI258135[] = {{&S33391[119], 7}, {NULL}};
const Segment SGI258136[] = {{&S33392[13], 4}, {NULL}};
const Segment SGI258137[] = {{&S33392[108], 3}, {NULL}};
const Segment SGI258138[] = {{&S33392[26], 5}, {NULL}};
const Segment SGI258139[] = {{&S33392[59], 6}, {NULL}};
const Segment SGI258140[] = {{&S33392[93], 6}, {NULL}};
const Segment SGI258141[] = {{&S33392[119], 9}, {NULL}};
const Segment SGI258142[] = {{&S33393[13], 4}, {NULL}};
const Segment SGI258143[] = {{&S33393[94], 3}, {NULL}};
const Segment SGI258144[] = {{&S33393[26], 5}, {NULL}};
const Segment SGI258145[] = {{&S33393[59], 6}, {NULL}};
const Segment SGI258146[] = {{&S33393[105], 6}, {NULL}};
const Segment SGI258147[] = {{&S33393[119], 7}, {NULL}};
const Segment SGI258148[] = {{&S33394[13], 4}, {NULL}};
const Segment SGI258149[] = {{&S33394[108], 3}, {NULL}};
const Segment SGI258150[] = {{&S33394[26], 5}, {NULL}};
const Segment SGI258151[] = {{&S33394[59], 6}, {NULL}};
const Segment SGI258152[] = {{&S33394[93], 6}, {NULL}};
const Segment SGI258153[] = {{&S33394[119], 9}, {NULL}};
const Segment SGI258154[] = {{&S22634[25], 14}, {&S22634[40], 26}, {NULL}};
const Segment SGI258155[] = {{&S22634[0], 66}, {NULL}};
const Segment SGI258156[] = {{&S22635[25], 14}, {&S22635[40], 26}, {NULL}};
const Segment SGI258157[] = {{&S22635[0], 66}, {NULL}};
const Segment SGI258158[] = {{&S22636[25], 14}, {&S22636[40], 36}, {NULL}};
const Segment SGI258159[] = {{&S22636[0], 105}, {NULL}};
const Segment SGI258160[] = {{&S22637[25], 14}, {&S22637[40], 26}, {NULL}};
const Segment SGI258161[] = {{&S22637[0], 66}, {NULL}};
const Segment SGI258162[] = {{&S22638[25], 14}, {&S22638[40], 26}, {NULL}};
const Segment SGI258163[] = {{&S22638[0], 66}, {NULL}};
const Segment SGI258164[] = {{&S22639[0], 15}, {NULL}};
const Segment SGI258165[] = {{&S22639[0], 40}, {NULL}};
const Segment SGI258166[] = {{&S22640[25], 25}, {&S22640[51], 26}, {NULL}};
const Segment SGI258167[] = {{&S22640[0], 77}, {NULL}};
const Segment SGI258168[] = {{&S22641[25], 14}, {&S22641[40], 26}, {NULL}};
const Segment SGI258169[] = {{&S22641[0], 66}, {NULL}};
const Segment SGI258170[] = {{&S15734[10], 4}, {NULL}};
const Segment SGI258171[] = {{&S15734[15], 5}, {NULL}};
const Segment SGI258172[] = {{&S15735[10], 4}, {NULL}};
const Segment SGI258173[] = {{&S22991[25], 11}, {&S22991[37], 33}, {NULL}};
const Segment SGI258174[] = {{&S22991[0], 70}, {NULL}};
const Segment SGI258175[] = {{&S22992[25], 12}, {&S22992[38], 33}, {NULL}};
const Segment SGI258176[] = {{&S22992[0], 71}, {NULL}};
const Segment SGI258177[] = {{&S22993[12], 57}, {NULL}};
const Segment SGI258178[] = {{&S22993[0], 69}, {NULL}};
const Segment SGI258179[] = {{&S22994[25], 12}, {&S22994[38], 33}, {NULL}};
const Segment SGI258180[] = {{&S22994[0], 143}, {NULL}};
const Segment SGI258181[] = {{&S22995[25], 12}, {&S22995[38], 43}, {NULL}};
const Segment SGI258182[] = {{&S22995[0], 110}, {NULL}};
const Segment SGI258183[] = {{&S22996[25], 12}, {&S22996[38], 33}, {&S22996[25], 12}, {&S22996[143], 43}, {NULL}};
const Segment SGI258184[] = {{&S22996[0], 215}, {NULL}};
const Segment SGI258185[] = {{&S22997[25], 12}, {&S22997[38], 33}, {NULL}};
const Segment SGI258186[] = {{&S22997[0], 144}, {NULL}};
const Segment SGI258187[] = {{&S22998[25], 12}, {&S22998[38], 33}, {NULL}};
const Segment SGI258188[] = {{&S22998[0], 143}, {NULL}};
const Segment SGI258189[] = {{&S22999[38], 12}, {&S22999[51], 33}, {NULL}};
const Segment SGI258190[] = {{&S22999[0], 84}, {NULL}};
const Segment SGI258191[] = {{&S23000[25], 12}, {&S23000[38], 33}, {NULL}};
const Segment SGI258192[] = {{&S23000[0], 144}, {NULL}};
const Segment SGI258193[] = {{&S23001[25], 45}, {NULL}};
const Segment SGI258194[] = {{&S23001[0], 70}, {NULL}};
const Segment SGI258195[] = {{&S23002[25], 12}, {&S23002[38], 33}, {NULL}};
const Segment SGI258196[] = {{&S23002[0], 84}, {NULL}};
const Segment SGI258197[] = {{&S23003[25], 11}, {&S23003[38], 32}, {NULL}};
const Segment SGI258198[] = {{&S23003[0], 70}, {NULL}};
const Segment SGI258199[] = {{&S23004[25], 12}, {&S23004[38], 33}, {NULL}};
const Segment SGI258200[] = {{&S23004[0], 145}, {NULL}};
const Segment SGI258201[] = {{&S23010[0], 18}, {NULL}};
const Segment SGI258202[] = {{&S23010[0], 70}, {NULL}};
const Segment SGI258203[] = {{&S23011[0], 18}, {NULL}};
const Segment SGI258204[] = {{&S23011[0], 69}, {NULL}};
const Segment SGI258205[] = {{&S23012[32], 19}, {&S23012[43], 3}, {NULL}};
const Segment SGI258206[] = {{&S23012[0], 74}, {NULL}};
const Segment SGI258207[] = {{&S23013[0], 7}, {&S23013[81], 24}, {NULL}};
const Segment SGI258208[] = {{&S23013[0], 105}, {NULL}};
const Segment SGI258209[] = {{&S23014[0], 7}, {&S23014[82], 24}, {NULL}};
const Segment SGI258210[] = {{&S23014[0], 106}, {NULL}};
const Segment SGI258211[] = {{&S23015[0], 7}, {&S23015[79], 24}, {NULL}};
const Segment SGI258212[] = {{&S23015[0], 103}, {NULL}};
const Segment SGI258213[] = {{&S23016[0], 7}, {&S23016[81], 24}, {NULL}};
const Segment SGI258214[] = {{&S23016[0], 105}, {NULL}};
const Segment SGI258215[] = {{&S23017[71], 12}, {NULL}};
const Segment SGI258216[] = {{&S23017[0], 136}, {NULL}};
const Segment SGI258217[] = {{&S23018[52], 13}, {NULL}};
const Segment SGI258218[] = {{&S23018[0], 82}, {NULL}};
const Segment SGI258219[] = {{&S23019[0], 7}, {&S23019[81], 24}, {NULL}};
const Segment SGI258220[] = {{&S23019[0], 105}, {NULL}};
const Segment SGI258221[] = {{&S23020[0], 7}, {&S23020[81], 24}, {NULL}};
const Segment SGI258222[] = {{&S23020[0], 105}, {NULL}};
const Segment SGI258223[] = {{&S23021[0], 7}, {&S23021[80], 24}, {NULL}};
const Segment SGI258224[] = {{&S23021[0], 104}, {NULL}};
const Segment SGI258225[] = {{&S23022[0], 7}, {&S23022[81], 24}, {NULL}};
const Segment SGI258226[] = {{&S23022[0], 105}, {NULL}};
const Segment SGI258227[] = {{&S23023[0], 7}, {&S23023[78], 24}, {NULL}};
const Segment SGI258228[] = {{&S23023[0], 102}, {NULL}};
const Segment SGI258229[] = {{&S23024[0], 7}, {&S23024[78], 24}, {NULL}};
const Segment SGI258230[] = {{&S23024[0], 102}, {NULL}};
const Segment SGI258231[] = {{&S23025[0], 7}, {&S23025[78], 24}, {NULL}};
const Segment SGI258232[] = {{&S23025[0], 102}, {NULL}};
const Segment SGI258233[] = {{&S23026[0], 7}, {&S23026[81], 24}, {NULL}};
const Segment SGI258234[] = {{&S23026[0], 105}, {NULL}};
const Segment SGI258235[] = {{&S23027[0], 7}, {&S23027[88], 24}, {NULL}};
const Segment SGI258236[] = {{&S23027[0], 112}, {NULL}};
const Segment SGI258237[] = {{&S23028[0], 7}, {&S23028[81], 24}, {NULL}};
const Segment SGI258238[] = {{&S23028[0], 105}, {NULL}};
const Segment SGI258239[] = {{&S23029[0], 7}, {&S23029[83], 24}, {NULL}};
const Segment SGI258240[] = {{&S23029[0], 107}, {NULL}};
const Segment SGI258241[] = {{&S23030[0], 7}, {&S23030[86], 24}, {NULL}};
const Segment SGI258242[] = {{&S23030[0], 110}, {NULL}};
const Segment SGI258243[] = {{&S23031[0], 7}, {&S23031[81], 24}, {NULL}};
const Segment SGI258244[] = {{&S23031[0], 105}, {NULL}};
const Segment SGI258245[] = {{&S23032[0], 7}, {&S23032[83], 24}, {NULL}};
const Segment SGI258246[] = {{&S23032[0], 107}, {NULL}};
const Segment SGI258247[] = {{&S23033[0], 7}, {&S23033[83], 24}, {NULL}};
const Segment SGI258248[] = {{&S23033[0], 107}, {NULL}};
const Segment SGI258249[] = {{&S23034[0], 7}, {&S23034[83], 24}, {NULL}};
const Segment SGI258250[] = {{&S23034[0], 107}, {NULL}};
const Segment SGI258251[] = {{&S23035[0], 7}, {&S23035[79], 24}, {NULL}};
const Segment SGI258252[] = {{&S23035[0], 103}, {NULL}};
const Segment SGI258253[] = {{&S23036[0], 7}, {&S23036[86], 24}, {NULL}};
const Segment SGI258254[] = {{&S23036[0], 110}, {NULL}};
const Segment SGI258255[] = {{&S23037[0], 7}, {&S23037[86], 24}, {NULL}};
const Segment SGI258256[] = {{&S23037[0], 110}, {NULL}};
const Segment SGI258257[] = {{&S23038[0], 7}, {&S23038[80], 24}, {NULL}};
const Segment SGI258258[] = {{&S23038[0], 104}, {NULL}};
const Segment SGI258259[] = {{&S23039[0], 7}, {&S23039[80], 24}, {NULL}};
const Segment SGI258260[] = {{&S23039[0], 104}, {NULL}};
const Segment SGI258261[] = {{&S23040[0], 7}, {&S23040[78], 24}, {NULL}};
const Segment SGI258262[] = {{&S23040[0], 102}, {NULL}};
const Segment SGI258263[] = {{&S23041[0], 7}, {&S23041[80], 24}, {NULL}};
const Segment SGI258264[] = {{&S23041[0], 104}, {NULL}};
const Segment SGI258265[] = {{&S23042[0], 7}, {&S23042[81], 24}, {NULL}};
const Segment SGI258266[] = {{&S23042[0], 105}, {NULL}};
const Segment SGI258267[] = {{&S23043[0], 7}, {&S23043[83], 24}, {NULL}};
const Segment SGI258268[] = {{&S23043[0], 107}, {NULL}};
const Segment SGI258269[] = {{&S23044[0], 7}, {&S23044[80], 24}, {NULL}};
const Segment SGI258270[] = {{&S23044[0], 104}, {NULL}};
const Segment SGI258271[] = {{&S23045[0], 7}, {&S23045[86], 24}, {NULL}};
const Segment SGI258272[] = {{&S23045[0], 110}, {NULL}};
const Segment SGI258273[] = {{&S23046[56], 7}, {NULL}};
const Segment SGI258274[] = {{&S23046[0], 85}, {NULL}};
const Segment SGI258275[] = {{&S23047[71], 19}, {NULL}};
const Segment SGI258276[] = {{&S23047[0], 104}, {NULL}};
const Segment SGI258277[] = {{&S23048[0], 7}, {&S23048[80], 24}, {NULL}};
const Segment SGI258278[] = {{&S23048[0], 104}, {NULL}};
const Segment SGI258279[] = {{&S23049[0], 7}, {&S23049[86], 24}, {NULL}};
const Segment SGI258280[] = {{&S23049[0], 110}, {NULL}};
const Segment SGI258281[] = {{&S23050[0], 7}, {&S23050[80], 24}, {NULL}};
const Segment SGI258282[] = {{&S23050[0], 104}, {NULL}};
const Segment SGI258283[] = {{&S23051[51], 3}, {NULL}};
const Segment SGI258284[] = {{&S23051[0], 85}, {NULL}};
const Segment SGI258285[] = {{&S23052[0], 7}, {&S23052[81], 24}, {NULL}};
const Segment SGI258286[] = {{&S23052[0], 105}, {NULL}};
const Segment SGI258287[] = {{&S23053[0], 7}, {&S23053[83], 24}, {NULL}};
const Segment SGI258288[] = {{&S23053[0], 107}, {NULL}};
const Segment SGI258289[] = {{&S23054[0], 7}, {&S23054[80], 24}, {NULL}};
const Segment SGI258290[] = {{&S23054[0], 104}, {NULL}};
const Segment SGI258291[] = {{&S23055[39], 16}, {&S23055[56], 31}, {NULL}};
const Segment SGI258292[] = {{&S23055[0], 87}, {NULL}};
const Segment SGI258293[] = {{&S23056[0], 7}, {&S23056[86], 24}, {NULL}};
const Segment SGI258294[] = {{&S23056[0], 110}, {NULL}};
const Segment SGI258295[] = {{&S23057[0], 7}, {&S23057[84], 24}, {NULL}};
const Segment SGI258296[] = {{&S23057[0], 108}, {NULL}};
const Segment SGI258297[] = {{&S23058[0], 7}, {&S23058[80], 24}, {NULL}};
const Segment SGI258298[] = {{&S23058[0], 104}, {NULL}};
const Segment SGI258299[] = {{&S23059[0], 7}, {&S23059[80], 24}, {NULL}};
const Segment SGI258300[] = {{&S23059[0], 104}, {NULL}};
const Segment SGI258301[] = {{&S23060[0], 7}, {&S23060[84], 24}, {NULL}};
const Segment SGI258302[] = {{&S23060[0], 108}, {NULL}};
const Segment SGI258303[] = {{&S23061[0], 7}, {&S23061[84], 24}, {NULL}};
const Segment SGI258304[] = {{&S23061[0], 108}, {NULL}};
const Segment SGI258305[] = {{&S23062[0], 7}, {&S23062[79], 24}, {NULL}};
const Segment SGI258306[] = {{&S23062[0], 103}, {NULL}};
const Segment SGI258307[] = {{&S23063[0], 7}, {&S23063[83], 24}, {NULL}};
const Segment SGI258308[] = {{&S23063[0], 107}, {NULL}};
const Segment SGI258309[] = {{&S23064[0], 7}, {&S23064[80], 24}, {NULL}};
const Segment SGI258310[] = {{&S23064[0], 104}, {NULL}};
const Segment SGI258311[] = {{&S23065[0], 7}, {&S23065[86], 24}, {NULL}};
const Segment SGI258312[] = {{&S23065[0], 110}, {NULL}};
const Segment SGI258313[] = {{&S23066[0], 7}, {&S23066[83], 24}, {NULL}};
const Segment SGI258314[] = {{&S23066[0], 107}, {NULL}};
const Segment SGI258315[] = {{&S23067[0], 7}, {&S23067[83], 24}, {NULL}};
const Segment SGI258316[] = {{&S23067[0], 107}, {NULL}};
const Segment SGI258317[] = {{&S23068[44], 24}, {NULL}};
const Segment SGI258318[] = {{&S23068[0], 77}, {NULL}};
const Segment SGI258319[] = {{&S23069[26], 3}, {&S23069[29], 22}, {NULL}};
const Segment SGI258320[] = {{&S23069[0], 51}, {NULL}};
const Segment SGI258321[] = {{&S23070[0], 7}, {&S23070[76], 24}, {NULL}};
const Segment SGI258322[] = {{&S23070[0], 100}, {NULL}};
const Segment SGI258323[] = {{&S23071[0], 7}, {&S23071[83], 24}, {NULL}};
const Segment SGI258324[] = {{&S23071[0], 107}, {NULL}};
const Segment SGI258325[] = {{&S23072[0], 7}, {&S23072[77], 24}, {NULL}};
const Segment SGI258326[] = {{&S23072[0], 101}, {NULL}};
const Segment SGI258327[] = {{&S23073[0], 7}, {&S23073[72], 24}, {NULL}};
const Segment SGI258328[] = {{&S23073[0], 96}, {NULL}};
const Segment SGI258329[] = {{&S23074[0], 7}, {&S23074[76], 24}, {NULL}};
const Segment SGI258330[] = {{&S23074[0], 100}, {NULL}};
const Segment SGI258331[] = {{&S23075[0], 7}, {&S23075[81], 24}, {NULL}};
const Segment SGI258332[] = {{&S23075[0], 105}, {NULL}};
const Segment SGI258333[] = {{&S23076[0], 7}, {&S23076[80], 24}, {NULL}};
const Segment SGI258334[] = {{&S23076[0], 104}, {NULL}};
const Segment SGI258335[] = {{&S23077[0], 7}, {&S23077[81], 24}, {NULL}};
const Segment SGI258336[] = {{&S23077[0], 105}, {NULL}};
const Segment SGI258337[] = {{&S23078[0], 7}, {&S23078[79], 24}, {NULL}};
const Segment SGI258338[] = {{&S23078[0], 103}, {NULL}};
const Segment SGI258339[] = {{&S23079[0], 7}, {&S23079[80], 24}, {NULL}};
const Segment SGI258340[] = {{&S23079[0], 104}, {NULL}};
const Segment SGI258341[] = {{&S23080[0], 7}, {&S23080[83], 24}, {NULL}};
const Segment SGI258342[] = {{&S23080[0], 107}, {NULL}};
const Segment SGI258343[] = {{&S23081[0], 7}, {&S23081[77], 24}, {NULL}};
const Segment SGI258344[] = {{&S23081[0], 101}, {NULL}};
const Segment SGI258345[] = {{&S23082[0], 7}, {&S23082[83], 24}, {NULL}};
const Segment SGI258346[] = {{&S23082[0], 107}, {NULL}};
const Segment SGI258347[] = {{&S23083[0], 7}, {&S23083[81], 24}, {NULL}};
const Segment SGI258348[] = {{&S23083[0], 105}, {NULL}};
const Segment SGI258349[] = {{&S23084[0], 7}, {&S23084[80], 24}, {NULL}};
const Segment SGI258350[] = {{&S23084[0], 104}, {NULL}};
const Segment SGI258351[] = {{&S23085[0], 7}, {&S23085[64], 41}, {NULL}};
const Segment SGI258352[] = {{&S23085[0], 105}, {NULL}};
const Segment SGI258353[] = {{&S23086[0], 7}, {&S23086[61], 41}, {NULL}};
const Segment SGI258354[] = {{&S23086[0], 102}, {NULL}};
const Segment SGI258355[] = {{&S23087[0], 7}, {&S23087[83], 24}, {NULL}};
const Segment SGI258356[] = {{&S23087[0], 107}, {NULL}};
const Segment SGI258357[] = {{&S23088[0], 7}, {&S23088[80], 24}, {NULL}};
const Segment SGI258358[] = {{&S23088[0], 104}, {NULL}};
const Segment SGI258359[] = {{&S23089[0], 7}, {&S23089[81], 24}, {NULL}};
const Segment SGI258360[] = {{&S23089[0], 105}, {NULL}};
const Segment SGI258361[] = {{&S23090[0], 7}, {&S23090[86], 24}, {NULL}};
const Segment SGI258362[] = {{&S23090[0], 110}, {NULL}};
const Segment SGI258363[] = {{&S23091[0], 7}, {&S23091[86], 24}, {NULL}};
const Segment SGI258364[] = {{&S23091[0], 110}, {NULL}};
const Segment SGI258365[] = {{&S23092[0], 7}, {&S23092[63], 41}, {NULL}};
const Segment SGI258366[] = {{&S23092[0], 104}, {NULL}};
const Segment SGI258367[] = {{&S23093[0], 7}, {&S23093[80], 24}, {NULL}};
const Segment SGI258368[] = {{&S23093[0], 104}, {NULL}};
const Segment SGI258369[] = {{&S23094[0], 7}, {&S23094[86], 24}, {NULL}};
const Segment SGI258370[] = {{&S23094[0], 110}, {NULL}};
const Segment SGI258371[] = {{&S23095[0], 7}, {&S23095[81], 24}, {NULL}};
const Segment SGI258372[] = {{&S23095[0], 105}, {NULL}};
const Segment SGI258373[] = {{&S23096[0], 7}, {&S23096[80], 24}, {NULL}};
const Segment SGI258374[] = {{&S23096[0], 104}, {NULL}};
const Segment SGI258375[] = {{&S23097[0], 7}, {&S23097[80], 24}, {NULL}};
const Segment SGI258376[] = {{&S23097[0], 104}, {NULL}};
const Segment SGI258377[] = {{&S23098[0], 7}, {&S23098[77], 24}, {NULL}};
const Segment SGI258378[] = {{&S23098[0], 101}, {NULL}};
const Segment SGI258379[] = {{&S34537[13], 6}, {NULL}};
const Segment SGI258380[] = {{&S34537[35], 3}, {NULL}};
const Segment SGI258381[] = {{&S34537[66], 6}, {NULL}};
const Segment SGI258382[] = {{&S34537[119], 8}, {NULL}};
const Segment SGI258383[] = {{&S34537[135], 7}, {NULL}};
const Segment SGI258384[] = {{&S34538[13], 6}, {NULL}};
const Segment SGI258385[] = {{&S34538[35], 3}, {NULL}};
const Segment SGI258386[] = {{&S34538[66], 6}, {NULL}};
const Segment SGI258387[] = {{&S34538[119], 8}, {NULL}};
const Segment SGI258388[] = {{&S34538[135], 7}, {NULL}};
const Segment SGI258389[] = {{&S34539[13], 6}, {NULL}};
const Segment SGI258390[] = {{&S34539[35], 3}, {NULL}};
const Segment SGI258391[] = {{&S34539[66], 6}, {NULL}};
const Segment SGI258392[] = {{&S34539[119], 8}, {NULL}};
const Segment SGI258393[] = {{&S34539[135], 7}, {NULL}};
const Segment SGI258394[] = {{&S34540[13], 6}, {NULL}};
const Segment SGI258395[] = {{&S34540[35], 3}, {NULL}};
const Segment SGI258396[] = {{&S34540[66], 6}, {NULL}};
const Segment SGI258397[] = {{&S34540[119], 8}, {NULL}};
const Segment SGI258398[] = {{&S34540[135], 7}, {NULL}};
const Segment SGI258399[] = {{&S34541[13], 6}, {NULL}};
const Segment SGI258400[] = {{&S34541[35], 3}, {NULL}};
const Segment SGI258401[] = {{&S34541[66], 6}, {NULL}};
const Segment SGI258402[] = {{&S34541[119], 8}, {NULL}};
const Segment SGI258403[] = {{&S34541[135], 7}, {NULL}};
const Segment SGI258404[] = {{&S34542[13], 6}, {NULL}};
const Segment SGI258405[] = {{&S34542[35], 3}, {NULL}};
const Segment SGI258406[] = {{&S34542[66], 6}, {NULL}};
const Segment SGI258407[] = {{&S34542[119], 8}, {NULL}};
const Segment SGI258408[] = {{&S34542[135], 7}, {NULL}};
const Segment SGI258409[] = {{&S34543[13], 6}, {NULL}};
const Segment SGI258410[] = {{&S34543[35], 3}, {NULL}};
const Segment SGI258411[] = {{&S34543[66], 6}, {NULL}};
const Segment SGI258412[] = {{&S34543[119], 6}, {NULL}};
const Segment SGI258413[] = {{&S34543[133], 7}, {NULL}};
const Segment SGI258414[] = {{&S34544[13], 6}, {NULL}};
const Segment SGI258415[] = {{&S34544[35], 5}, {NULL}};
const Segment SGI258416[] = {{&S34544[68], 6}, {NULL}};
const Segment SGI258417[] = {{&S34544[121], 6}, {NULL}};
const Segment SGI258418[] = {{&S34544[135], 7}, {NULL}};
const Segment SGI258419[] = {{&S34545[13], 6}, {NULL}};
const Segment SGI258420[] = {{&S34545[35], 3}, {NULL}};
const Segment SGI258421[] = {{&S34545[66], 6}, {NULL}};
const Segment SGI258422[] = {{&S34545[119], 6}, {NULL}};
const Segment SGI258423[] = {{&S34545[133], 7}, {NULL}};
const Segment SGI258424[] = {{&S34546[13], 6}, {NULL}};
const Segment SGI258425[] = {{&S34546[35], 5}, {NULL}};
const Segment SGI258426[] = {{&S34546[68], 6}, {NULL}};
const Segment SGI258427[] = {{&S34546[121], 7}, {NULL}};
const Segment SGI258428[] = {{&S34546[136], 7}, {NULL}};
const Segment SGI258429[] = {{&S34794[13], 4}, {NULL}};
const Segment SGI258430[] = {{&S34794[99], 3}, {NULL}};
const Segment SGI258431[] = {{&S34794[33], 3}, {NULL}};
const Segment SGI258432[] = {{&S34794[64], 6}, {NULL}};
const Segment SGI258433[] = {{&S34794[110], 5}, {NULL}};
const Segment SGI258434[] = {{&S34794[123], 9}, {NULL}};
const Segment SGI258435[] = {{&S34795[13], 4}, {NULL}};
const Segment SGI258436[] = {{&S34795[99], 3}, {NULL}};
const Segment SGI258437[] = {{&S34795[33], 3}, {NULL}};
const Segment SGI258438[] = {{&S34795[64], 6}, {NULL}};
const Segment SGI258439[] = {{&S34795[110], 7}, {NULL}};
const Segment SGI258440[] = {{&S34795[125], 9}, {NULL}};
const Segment SGI258441[] = {{&S34796[13], 4}, {NULL}};
const Segment SGI258442[] = {{&S34796[101], 3}, {NULL}};
const Segment SGI258443[] = {{&S34796[33], 5}, {NULL}};
const Segment SGI258444[] = {{&S34796[66], 6}, {NULL}};
const Segment SGI258445[] = {{&S34796[112], 5}, {NULL}};
const Segment SGI258446[] = {{&S34796[125], 9}, {NULL}};
const Segment SGI258447[] = {{&S34797[13], 4}, {NULL}};
const Segment SGI258448[] = {{&S34797[101], 3}, {NULL}};
const Segment SGI258449[] = {{&S34797[33], 5}, {NULL}};
const Segment SGI258450[] = {{&S34797[66], 6}, {NULL}};
const Segment SGI258451[] = {{&S34797[112], 5}, {NULL}};
const Segment SGI258452[] = {{&S34797[125], 9}, {NULL}};
const Segment SGI258453[] = {{&S34798[13], 4}, {NULL}};
const Segment SGI258454[] = {{&S34798[99], 3}, {NULL}};
const Segment SGI258455[] = {{&S34798[33], 3}, {NULL}};
const Segment SGI258456[] = {{&S34798[64], 6}, {NULL}};
const Segment SGI258457[] = {{&S34798[110], 7}, {NULL}};
const Segment SGI258458[] = {{&S34798[125], 9}, {NULL}};
const Segment SGI258459[] = {{&S34799[13], 4}, {NULL}};
const Segment SGI258460[] = {{&S34799[97], 3}, {NULL}};
const Segment SGI258461[] = {{&S34799[33], 2}, {NULL}};
const Segment SGI258462[] = {{&S34799[63], 6}, {NULL}};
const Segment SGI258463[] = {{&S34799[108], 5}, {NULL}};
const Segment SGI258464[] = {{&S34799[121], 9}, {NULL}};
const Segment SGI258465[] = {{&S34800[13], 4}, {NULL}};
const Segment SGI258466[] = {{&S34800[98], 3}, {NULL}};
const Segment SGI258467[] = {{&S34800[33], 3}, {NULL}};
const Segment SGI258468[] = {{&S34800[64], 6}, {NULL}};
const Segment SGI258469[] = {{&S34800[109], 5}, {NULL}};
const Segment SGI258470[] = {{&S34800[122], 9}, {NULL}};
const Segment SGI258471[] = {{&S34801[13], 4}, {NULL}};
const Segment SGI258472[] = {{&S34801[98], 3}, {NULL}};
const Segment SGI258473[] = {{&S34801[33], 3}, {NULL}};
const Segment SGI258474[] = {{&S34801[64], 6}, {NULL}};
const Segment SGI258475[] = {{&S34801[109], 5}, {NULL}};
const Segment SGI258476[] = {{&S34801[122], 9}, {NULL}};
const Segment SGI258477[] = {{&S17394[10], 4}, {NULL}};
const Segment SGI258478[] = {{&S17394[15], 5}, {NULL}};
const Segment SGI258479[] = {{&S24325[9], 6}, {NULL}};
const Segment SGI258480[] = {{&S24325[0], 25}, {NULL}};
const Segment SGI258481[] = {{&S24327[10], 6}, {NULL}};
const Segment SGI258482[] = {{&S24327[0], 26}, {NULL}};
const Segment SGI258483[] = {{&S24329[0], 24}, {NULL}};
const Segment SGI258484[] = {{&S24330[6], 6}, {NULL}};
const Segment SGI258485[] = {{&S24330[0], 22}, {NULL}};
const Segment SGI258486[] = {{&S24332[0], 25}, {&S24332[59], 46}, {NULL}};
const Segment SGI258487[] = {{&S24332[0], 144}, {NULL}};
const Segment SGI258488[] = {{&S24333[0], 21}, {NULL}};
const Segment SGI258489[] = {{&S24336[23], 40}, {NULL}};
const Segment SGI258490[] = {{&S24336[0], 111}, {NULL}};
const Segment SGI258491[] = {{&S24337[9], 6}, {NULL}};
const Segment SGI258492[] = {{&S24337[0], 70}, {NULL}};
const Segment SGI258493[] = {{&S24338[6], 6}, {NULL}};
const Segment SGI258494[] = {{&S24338[0], 22}, {NULL}};
const Segment SGI258495[] = {{&S24340[0], 17}, {NULL}};
const Segment SGI258496[] = {{&S24340[0], 62}, {NULL}};
const Segment SGI258497[] = {{&S24342[0], 27}, {NULL}};
const Segment SGI258498[] = {{&S24343[11], 3}, {NULL}};
const Segment SGI258499[] = {{&S24343[0], 51}, {NULL}};
const Segment SGI258500[] = {{&S24345[0], 22}, {NULL}};
const Segment SGI258501[] = {{&S24345[0], 72}, {NULL}};
const Segment SGI258502[] = {{&S24348[9], 6}, {NULL}};
const Segment SGI258503[] = {{&S24348[0], 27}, {NULL}};
const Segment SGI258504[] = {{&S24352[38], 3}, {&S24352[58], 11}, {NULL}};
const Segment SGI258505[] = {{&S24352[0], 119}, {NULL}};
const Segment SGI258506[] = {{&S35392[13], 4}, {NULL}};
const Segment SGI258507[] = {{&S35392[33], 5}, {NULL}};
const Segment SGI258508[] = {{&S35392[66], 6}, {NULL}};
const Segment SGI258509[] = {{&S35392[100], 5}, {NULL}};
const Segment SGI258510[] = {{&S35393[13], 4}, {NULL}};
const Segment SGI258511[] = {{&S35393[33], 3}, {NULL}};
const Segment SGI258512[] = {{&S35393[64], 6}, {NULL}};
const Segment SGI258513[] = {{&S35393[98], 5}, {NULL}};
const Segment SGI258514[] = {{&S35394[13], 4}, {NULL}};
const Segment SGI258515[] = {{&S35394[33], 5}, {NULL}};
const Segment SGI258516[] = {{&S35394[66], 6}, {NULL}};
const Segment SGI258517[] = {{&S35395[13], 4}, {NULL}};
const Segment SGI258518[] = {{&S35395[33], 3}, {NULL}};
const Segment SGI258519[] = {{&S35395[64], 6}, {NULL}};
const Segment SGI258520[] = {{&S35396[15], 4}, {NULL}};
const Segment SGI258521[] = {{&S35396[37], 5}, {NULL}};
const Segment SGI258522[] = {{&S35396[0], 7}, {NULL}};
const Segment SGI258523[] = {{&S35397[13], 4}, {&S35397[13], 4}, {&S35397[13], 4}, {NULL}};
const Segment SGI258524[] = {{&S35397[42], 3}, {NULL}};
const Segment SGI258525[] = {{&S35398[13], 4}, {&S35398[13], 4}, {&S35398[13], 4}, {NULL}};
const Segment SGI258526[] = {{&S35398[42], 5}, {NULL}};
const Segment SGI258527[] = {{&S35399[13], 4}, {&S35399[13], 4}, {NULL}};
const Segment SGI258528[] = {{&S35399[33], 5}, {NULL}};
const Segment SGI258529[] = {{&S35399[66], 6}, {NULL}};
const Segment SGI258530[] = {{&S35399[100], 5}, {NULL}};
const Segment SGI258531[] = {{&S35400[13], 4}, {&S35400[13], 4}, {NULL}};
const Segment SGI258532[] = {{&S35400[33], 3}, {NULL}};
const Segment SGI258533[] = {{&S35400[64], 6}, {NULL}};
const Segment SGI258534[] = {{&S35400[98], 5}, {NULL}};
const Segment SGI258535[] = {{&S35401[13], 4}, {&S35401[13], 4}, {&S35401[13], 4}, {NULL}};
const Segment SGI258536[] = {{&S35401[33], 5}, {NULL}};
const Segment SGI258537[] = {{&S35401[66], 6}, {NULL}};
const Segment SGI258538[] = {{&S35401[100], 5}, {NULL}};
const Segment SGI258539[] = {{&S35402[13], 4}, {&S35402[13], 4}, {&S35402[13], 4}, {NULL}};
const Segment SGI258540[] = {{&S35402[33], 3}, {NULL}};
const Segment SGI258541[] = {{&S35402[64], 6}, {NULL}};
const Segment SGI258542[] = {{&S35402[98], 5}, {NULL}};
const Segment SGI258543[] = {{&S35403[13], 4}, {NULL}};
const Segment SGI258544[] = {{&S35403[130], 3}, {NULL}};
const Segment SGI258545[] = {{&S35403[33], 5}, {NULL}};
const Segment SGI258546[] = {{&S35403[66], 6}, {NULL}};
const Segment SGI258547[] = {{&S35403[100], 5}, {NULL}};
const Segment SGI258548[] = {{&S35563[13], 6}, {NULL}};
const Segment SGI258549[] = {{&S35563[38], 3}, {NULL}};
const Segment SGI258550[] = {{&S35563[76], 5}, {NULL}};
const Segment SGI258551[] = {{&S35563[109], 5}, {NULL}};
const Segment SGI258552[] = {{&S35564[13], 6}, {NULL}};
const Segment SGI258553[] = {{&S35564[38], 3}, {NULL}};
const Segment SGI258554[] = {{&S35564[76], 5}, {NULL}};
const Segment SGI258555[] = {{&S35565[13], 6}, {NULL}};
const Segment SGI258556[] = {{&S35565[38], 3}, {NULL}};
const Segment SGI258557[] = {{&S35565[76], 5}, {NULL}};
const Segment SGI258558[] = {{&S35565[109], 5}, {NULL}};
const Segment SGI258559[] = {{&S35566[13], 6}, {NULL}};
const Segment SGI258560[] = {{&S35566[38], 3}, {NULL}};
const Segment SGI258561[] = {{&S35566[119], 4}, {NULL}};
const Segment SGI258562[] = {{&S35567[13], 6}, {NULL}};
const Segment SGI258563[] = {{&S35567[38], 3}, {NULL}};
const Segment SGI258564[] = {{&S35567[76], 5}, {NULL}};
const Segment SGI258565[] = {{&S35567[109], 5}, {NULL}};
const Segment SGI258566[] = {{&S35568[13], 6}, {NULL}};
const Segment SGI258567[] = {{&S35568[38], 3}, {NULL}};
const Segment SGI258568[] = {{&S35569[13], 6}, {NULL}};
const Segment SGI258569[] = {{&S35569[38], 3}, {NULL}};
const Segment SGI258570[] = {{&S35569[76], 5}, {NULL}};
const Segment SGI258571[] = {{&S35569[109], 3}, {NULL}};
const Segment SGI258572[] = {{&S35570[13], 6}, {&S35570[13], 6}, {NULL}};
const Segment SGI258573[] = {{&S35570[73], 6}, {NULL}};
const Segment SGI258574[] = {{&S35570[107], 4}, {NULL}};
const Segment SGI258575[] = {{&S35901[13], 6}, {NULL}};
const Segment SGI258576[] = {{&S35901[35], 3}, {NULL}};
const Segment SGI258577[] = {{&S35901[66], 6}, {NULL}};
const Segment SGI258578[] = {{&S35901[119], 6}, {NULL}};
const Segment SGI258579[] = {{&S35901[133], 7}, {NULL}};
const Segment SGI258580[] = {{&S35902[13], 6}, {NULL}};
const Segment SGI258581[] = {{&S35902[35], 5}, {NULL}};
const Segment SGI258582[] = {{&S35902[68], 6}, {NULL}};
const Segment SGI258583[] = {{&S35902[121], 6}, {NULL}};
const Segment SGI258584[] = {{&S35902[135], 7}, {NULL}};
const Segment SGI258585[] = {{&S35903[13], 6}, {NULL}};
const Segment SGI258586[] = {{&S35903[35], 5}, {NULL}};
const Segment SGI258587[] = {{&S35903[68], 6}, {NULL}};
const Segment SGI258588[] = {{&S35903[121], 6}, {NULL}};
const Segment SGI258589[] = {{&S35903[135], 7}, {NULL}};
const Segment SGI258590[] = {{&S35904[13], 6}, {NULL}};
const Segment SGI258591[] = {{&S35904[35], 5}, {NULL}};
const Segment SGI258592[] = {{&S35904[68], 6}, {NULL}};
const Segment SGI258593[] = {{&S35904[121], 6}, {NULL}};
const Segment SGI258594[] = {{&S35904[135], 7}, {NULL}};
const Segment SGI258595[] = {{&S35905[13], 6}, {NULL}};
const Segment SGI258596[] = {{&S35905[35], 3}, {NULL}};
const Segment SGI258597[] = {{&S35905[66], 6}, {NULL}};
const Segment SGI258598[] = {{&S35905[119], 6}, {NULL}};
const Segment SGI258599[] = {{&S35905[133], 7}, {NULL}};
const Segment SGI258600[] = {{&S35906[13], 6}, {NULL}};
const Segment SGI258601[] = {{&S35906[35], 5}, {NULL}};
const Segment SGI258602[] = {{&S35906[68], 6}, {NULL}};
const Segment SGI258603[] = {{&S35906[121], 6}, {NULL}};
const Segment SGI258604[] = {{&S35906[135], 7}, {NULL}};
const Segment SGI258605[] = {{&S35907[13], 6}, {NULL}};
const Segment SGI258606[] = {{&S35907[35], 3}, {NULL}};
const Segment SGI258607[] = {{&S35907[66], 6}, {NULL}};
const Segment SGI258608[] = {{&S35907[119], 6}, {NULL}};
const Segment SGI258609[] = {{&S35907[133], 7}, {NULL}};
const Segment SGI258610[] = {{&S35908[13], 6}, {NULL}};
const Segment SGI258611[] = {{&S35908[35], 3}, {NULL}};
const Segment SGI258612[] = {{&S35908[66], 6}, {NULL}};
const Segment SGI258613[] = {{&S35908[119], 6}, {NULL}};
const Segment SGI258614[] = {{&S35908[133], 7}, {NULL}};
const Segment SGI258615[] = {{&S35997[13], 6}, {NULL}};
const Segment SGI258616[] = {{&S35997[35], 5}, {NULL}};
const Segment SGI258617[] = {{&S35997[68], 6}, {NULL}};
const Segment SGI258618[] = {{&S35997[121], 6}, {NULL}};
const Segment SGI258619[] = {{&S35997[135], 7}, {NULL}};
const Segment SGI258620[] = {{&S35998[13], 6}, {NULL}};
const Segment SGI258621[] = {{&S35998[35], 5}, {NULL}};
const Segment SGI258622[] = {{&S35998[68], 6}, {NULL}};
const Segment SGI258623[] = {{&S35998[121], 6}, {NULL}};
const Segment SGI258624[] = {{&S35998[135], 7}, {NULL}};
const Segment SGI258625[] = {{&S35999[13], 6}, {NULL}};
const Segment SGI258626[] = {{&S35999[35], 5}, {NULL}};
const Segment SGI258627[] = {{&S35999[68], 6}, {NULL}};
const Segment SGI258628[] = {{&S35999[121], 6}, {NULL}};
const Segment SGI258629[] = {{&S35999[135], 7}, {NULL}};
const Segment SGI258630[] = {{&S36000[13], 6}, {NULL}};
const Segment SGI258631[] = {{&S36000[35], 3}, {NULL}};
const Segment SGI258632[] = {{&S36000[66], 6}, {NULL}};
const Segment SGI258633[] = {{&S36000[119], 6}, {NULL}};
const Segment SGI258634[] = {{&S36000[133], 7}, {NULL}};
const Segment SGI258635[] = {{&S36001[13], 6}, {NULL}};
const Segment SGI258636[] = {{&S36001[35], 3}, {NULL}};
const Segment SGI258637[] = {{&S36001[66], 6}, {NULL}};
const Segment SGI258638[] = {{&S36001[119], 6}, {NULL}};
const Segment SGI258639[] = {{&S36001[133], 7}, {NULL}};
const Segment SGI258640[] = {{&S36002[13], 6}, {NULL}};
const Segment SGI258641[] = {{&S36002[35], 5}, {NULL}};
const Segment SGI258642[] = {{&S36002[68], 6}, {NULL}};
const Segment SGI258643[] = {{&S36002[121], 6}, {NULL}};
const Segment SGI258644[] = {{&S36002[135], 7}, {NULL}};
const Segment SGI258645[] = {{&S36003[13], 6}, {NULL}};
const Segment SGI258646[] = {{&S36003[35], 3}, {NULL}};
const Segment SGI258647[] = {{&S36003[66], 6}, {NULL}};
const Segment SGI258648[] = {{&S36003[119], 6}, {NULL}};
const Segment SGI258649[] = {{&S36003[133], 7}, {NULL}};
const Segment SGI258650[] = {{&S36004[13], 6}, {NULL}};
const Segment SGI258651[] = {{&S36004[35], 3}, {NULL}};
const Segment SGI258652[] = {{&S36004[66], 6}, {NULL}};
const Segment SGI258653[] = {{&S36004[119], 6}, {NULL}};
const Segment SGI258654[] = {{&S36004[133], 7}, {NULL}};
const Segment SGI258655[] = {{&S18532[10], 4}, {NULL}};
const Segment SGI258656[] = {{&S18532[15], 5}, {NULL}};
const Segment SGI258657[] = {{&S36336[13], 4}, {NULL}};
const Segment SGI258658[] = {{&S36336[29], 3}, {NULL}};
const Segment SGI258659[] = {{&S36336[67], 8}, {NULL}};
const Segment SGI258660[] = {{&S36337[13], 4}, {NULL}};
const Segment SGI258661[] = {{&S36337[29], 5}, {NULL}};
const Segment SGI258662[] = {{&S36337[69], 8}, {NULL}};
const Segment SGI258663[] = {{&S36337[105], 5}, {NULL}};
const Segment SGI258664[] = {{&S36338[13], 4}, {NULL}};
const Segment SGI258665[] = {{&S36338[29], 5}, {NULL}};
const Segment SGI258666[] = {{&S36338[69], 8}, {NULL}};
const Segment SGI258667[] = {{&S36339[13], 4}, {NULL}};
const Segment SGI258668[] = {{&S36339[29], 3}, {NULL}};
const Segment SGI258669[] = {{&S36339[67], 8}, {NULL}};
const Segment SGI258670[] = {{&S36339[103], 6}, {NULL}};
const Segment SGI258671[] = {{&S36340[13], 4}, {NULL}};
const Segment SGI258672[] = {{&S36340[29], 5}, {NULL}};
const Segment SGI258673[] = {{&S36340[69], 8}, {NULL}};
const Segment SGI258674[] = {{&S36340[105], 5}, {NULL}};
const Segment SGI258675[] = {{&S36341[13], 4}, {&S36341[13], 4}, {&S36341[13], 4}, {NULL}};
const Segment SGI258676[] = {{&S36341[36], 5}, {NULL}};
const Segment SGI258677[] = {{&S36342[13], 4}, {&S36342[13], 4}, {NULL}};
const Segment SGI258678[] = {{&S36342[29], 5}, {NULL}};
const Segment SGI258679[] = {{&S36342[69], 8}, {NULL}};
const Segment SGI258680[] = {{&S36342[105], 5}, {NULL}};
const Segment SGI258681[] = {{&S36343[13], 4}, {NULL}};
const Segment SGI258682[] = {{&S36343[29], 5}, {NULL}};
const Segment SGI258683[] = {{&S36343[69], 8}, {NULL}};
const Segment SGI258684[] = {{&S36343[105], 5}, {NULL}};
const Segment SGI258685[] = {{&S36343[118], 6}, {NULL}};
const Segment SGI258686[] = {{&S36344[13], 4}, {NULL}};
const Segment SGI258687[] = {{&S36344[29], 3}, {NULL}};
const Segment SGI258688[] = {{&S36344[67], 8}, {NULL}};
const Segment SGI258689[] = {{&S36344[103], 5}, {NULL}};
const Segment SGI258690[] = {{&S36345[30], 4}, {NULL}};
const Segment SGI258691[] = {{&S36345[46], 5}, {NULL}};
const Segment SGI258692[] = {{&S36345[86], 8}, {NULL}};
const Segment SGI258693[] = {{&S36345[122], 5}, {NULL}};
const Segment SGI258694[] = {{&S36345[135], 9}, {NULL}};
const Segment SGI258695[] = {{&S36345[0], 14}, {NULL}};
const Segment SGI258696[] = {{&S36346[13], 4}, {NULL}};
const Segment SGI258697[] = {{&S36346[29], 3}, {NULL}};
const Segment SGI258698[] = {{&S36346[67], 8}, {NULL}};
const Segment SGI258699[] = {{&S36346[103], 5}, {NULL}};
const Segment SGI258700[] = {{&S36347[27], 4}, {NULL}};
const Segment SGI258701[] = {{&S36347[43], 5}, {NULL}};
const Segment SGI258702[] = {{&S36347[83], 8}, {NULL}};
const Segment SGI258703[] = {{&S36347[119], 5}, {NULL}};
const Segment SGI258704[] = {{&S36347[132], 9}, {NULL}};
const Segment SGI258705[] = {{&S36347[0], 14}, {NULL}};
const Segment SGI258706[] = {{&S36348[13], 4}, {&S36348[13], 4}, {&S36348[13], 4}, {NULL}};
const Segment SGI258707[] = {{&S36348[36], 3}, {NULL}};
const Segment SGI258708[] = {{&S36519[13], 4}, {NULL}};
const Segment SGI258709[] = {{&S36519[29], 5}, {NULL}};
const Segment SGI258710[] = {{&S36519[69], 8}, {NULL}};
const Segment SGI258711[] = {{&S36519[124], 3}, {NULL}};
const Segment SGI258712[] = {{&S36519[135], 9}, {NULL}};
const Segment SGI258713[] = {{&S36520[13], 4}, {NULL}};
const Segment SGI258714[] = {{&S36520[29], 5}, {NULL}};
const Segment SGI258715[] = {{&S36520[69], 8}, {NULL}};
const Segment SGI258716[] = {{&S36520[124], 3}, {NULL}};
const Segment SGI258717[] = {{&S36520[135], 9}, {NULL}};
const Segment SGI258718[] = {{&S36521[13], 4}, {NULL}};
const Segment SGI258719[] = {{&S36521[29], 5}, {NULL}};
const Segment SGI258720[] = {{&S36521[69], 8}, {NULL}};
const Segment SGI258721[] = {{&S36521[124], 3}, {NULL}};
const Segment SGI258722[] = {{&S36521[135], 9}, {NULL}};
const Segment SGI258723[] = {{&S36522[13], 4}, {NULL}};
const Segment SGI258724[] = {{&S36522[29], 5}, {NULL}};
const Segment SGI258725[] = {{&S36522[69], 8}, {NULL}};
const Segment SGI258726[] = {{&S36522[124], 3}, {NULL}};
const Segment SGI258727[] = {{&S36522[135], 9}, {NULL}};
const Segment SGI258728[] = {{&S36523[13], 4}, {NULL}};
const Segment SGI258729[] = {{&S36523[29], 5}, {NULL}};
const Segment SGI258730[] = {{&S36523[69], 8}, {NULL}};
const Segment SGI258731[] = {{&S36523[124], 3}, {NULL}};
const Segment SGI258732[] = {{&S36523[135], 9}, {NULL}};
const Segment SGI258733[] = {{&S36524[13], 4}, {NULL}};
const Segment SGI258734[] = {{&S36524[29], 3}, {NULL}};
const Segment SGI258735[] = {{&S36524[67], 8}, {NULL}};
const Segment SGI258736[] = {{&S36524[122], 5}, {NULL}};
const Segment SGI258737[] = {{&S36524[135], 9}, {NULL}};
const Segment SGI258738[] = {{&S36525[13], 4}, {NULL}};
const Segment SGI258739[] = {{&S36525[29], 3}, {NULL}};
const Segment SGI258740[] = {{&S36525[67], 8}, {NULL}};
const Segment SGI258741[] = {{&S36525[122], 5}, {NULL}};
const Segment SGI258742[] = {{&S36525[135], 9}, {NULL}};
const Segment SGI258743[] = {{&S36526[13], 4}, {NULL}};
const Segment SGI258744[] = {{&S36526[29], 5}, {NULL}};
const Segment SGI258745[] = {{&S36526[69], 8}, {NULL}};
const Segment SGI258746[] = {{&S36526[124], 3}, {NULL}};
const Segment SGI258747[] = {{&S36526[135], 9}, {NULL}};
const Segment SGI258748[] = {{&S36527[13], 4}, {NULL}};
const Segment SGI258749[] = {{&S36527[29], 5}, {NULL}};
const Segment SGI258750[] = {{&S36527[69], 8}, {NULL}};
const Segment SGI258751[] = {{&S36527[124], 3}, {NULL}};
const Segment SGI258752[] = {{&S36527[135], 9}, {NULL}};
const Segment SGI258753[] = {{&S36553[13], 6}, {NULL}};
const Segment SGI258754[] = {{&S36553[35], 5}, {NULL}};
const Segment SGI258755[] = {{&S36553[68], 6}, {NULL}};
const Segment SGI258756[] = {{&S36553[121], 6}, {NULL}};
const Segment SGI258757[] = {{&S36553[135], 7}, {NULL}};
const Segment SGI258758[] = {{&S36554[13], 6}, {NULL}};
const Segment SGI258759[] = {{&S36554[35], 3}, {NULL}};
const Segment SGI258760[] = {{&S36554[66], 6}, {NULL}};
const Segment SGI258761[] = {{&S36554[119], 6}, {NULL}};
const Segment SGI258762[] = {{&S36554[133], 7}, {NULL}};
const Segment SGI258763[] = {{&S36555[13], 6}, {NULL}};
const Segment SGI258764[] = {{&S36555[35], 5}, {NULL}};
const Segment SGI258765[] = {{&S36555[68], 6}, {NULL}};
const Segment SGI258766[] = {{&S36555[121], 6}, {NULL}};
const Segment SGI258767[] = {{&S36555[135], 7}, {NULL}};
const Segment SGI258768[] = {{&S36556[13], 6}, {NULL}};
const Segment SGI258769[] = {{&S36556[35], 3}, {NULL}};
const Segment SGI258770[] = {{&S36556[66], 6}, {NULL}};
const Segment SGI258771[] = {{&S36556[119], 6}, {NULL}};
const Segment SGI258772[] = {{&S36556[133], 7}, {NULL}};
const Segment SGI258773[] = {{&S36557[13], 6}, {NULL}};
const Segment SGI258774[] = {{&S36557[35], 5}, {NULL}};
const Segment SGI258775[] = {{&S36557[68], 6}, {NULL}};
const Segment SGI258776[] = {{&S36557[121], 6}, {NULL}};
const Segment SGI258777[] = {{&S36557[135], 7}, {NULL}};
const Segment SGI258778[] = {{&S36602[13], 4}, {NULL}};
const Segment SGI258779[] = {{&S36602[26], 3}, {NULL}};
const Segment SGI258780[] = {{&S36602[57], 6}, {NULL}};
const Segment SGI258781[] = {{&S36602[110], 6}, {NULL}};
const Segment SGI258782[] = {{&S36602[124], 7}, {NULL}};
const Segment SGI258783[] = {{&S36603[13], 4}, {NULL}};
const Segment SGI258784[] = {{&S36603[26], 5}, {NULL}};
const Segment SGI258785[] = {{&S36603[59], 6}, {NULL}};
const Segment SGI258786[] = {{&S36603[112], 6}, {NULL}};
const Segment SGI258787[] = {{&S36603[126], 7}, {NULL}};
const Segment SGI258788[] = {{&S36604[13], 4}, {NULL}};
const Segment SGI258789[] = {{&S36604[26], 5}, {NULL}};
const Segment SGI258790[] = {{&S36604[59], 6}, {NULL}};
const Segment SGI258791[] = {{&S36604[112], 6}, {NULL}};
const Segment SGI258792[] = {{&S36604[126], 7}, {NULL}};
const Segment SGI258793[] = {{&S36605[13], 4}, {NULL}};
const Segment SGI258794[] = {{&S36605[26], 5}, {NULL}};
const Segment SGI258795[] = {{&S36605[59], 6}, {NULL}};
const Segment SGI258796[] = {{&S36605[112], 7}, {NULL}};
const Segment SGI258797[] = {{&S36605[127], 7}, {NULL}};
const Segment SGI258798[] = {{&S36606[13], 4}, {NULL}};
const Segment SGI258799[] = {{&S36606[26], 3}, {NULL}};
const Segment SGI258800[] = {{&S36606[57], 6}, {NULL}};
const Segment SGI258801[] = {{&S36606[110], 6}, {NULL}};
const Segment SGI258802[] = {{&S36606[124], 7}, {NULL}};
const Segment SGI258803[] = {{&S36607[13], 4}, {NULL}};
const Segment SGI258804[] = {{&S36607[26], 5}, {NULL}};
const Segment SGI258805[] = {{&S36607[59], 6}, {NULL}};
const Segment SGI258806[] = {{&S36607[112], 7}, {NULL}};
const Segment SGI258807[] = {{&S36607[127], 7}, {NULL}};
const Segment SGI258808[] = {{&S36608[13], 4}, {NULL}};
const Segment SGI258809[] = {{&S36608[26], 3}, {NULL}};
const Segment SGI258810[] = {{&S36608[57], 6}, {NULL}};
const Segment SGI258811[] = {{&S36608[110], 6}, {NULL}};
const Segment SGI258812[] = {{&S36608[124], 7}, {NULL}};
const Segment SGI258813[] = {{&S36756[13], 6}, {NULL}};
const Segment SGI258814[] = {{&S36756[38], 3}, {NULL}};
const Segment SGI258815[] = {{&S36756[76], 5}, {NULL}};
const Segment SGI258816[] = {{&S36757[13], 6}, {NULL}};
const Segment SGI258817[] = {{&S36757[38], 3}, {NULL}};
const Segment SGI258818[] = {{&S36757[76], 5}, {NULL}};
const Segment SGI258819[] = {{&S36757[109], 5}, {NULL}};
const Segment SGI258820[] = {{&S36758[13], 6}, {NULL}};
const Segment SGI258821[] = {{&S36758[38], 5}, {NULL}};
const Segment SGI258822[] = {{&S36758[78], 5}, {NULL}};
const Segment SGI258823[] = {{&S36758[111], 5}, {NULL}};
const Segment SGI258824[] = {{&S36759[13], 6}, {NULL}};
const Segment SGI258825[] = {{&S36759[38], 5}, {NULL}};
const Segment SGI258826[] = {{&S36759[78], 5}, {NULL}};
const Segment SGI258827[] = {{&S36759[111], 5}, {NULL}};
const Segment SGI258828[] = {{&S36760[13], 6}, {NULL}};
const Segment SGI258829[] = {{&S36760[38], 5}, {NULL}};
const Segment SGI258830[] = {{&S36760[78], 5}, {NULL}};
const Segment SGI258831[] = {{&S36761[13], 6}, {NULL}};
const Segment SGI258832[] = {{&S36761[38], 5}, {NULL}};
const Segment SGI258833[] = {{&S36761[78], 5}, {NULL}};
const Segment SGI258834[] = {{&S36762[13], 6}, {NULL}};
const Segment SGI258835[] = {{&S36762[38], 3}, {NULL}};
const Segment SGI258836[] = {{&S36762[76], 5}, {NULL}};
const Segment SGI258837[] = {{&S36762[109], 4}, {NULL}};
const Segment SGI258838[] = {{&S36763[13], 6}, {NULL}};
const Segment SGI258839[] = {{&S36763[38], 5}, {NULL}};
const Segment SGI258840[] = {{&S36763[78], 5}, {NULL}};
const Segment SGI258841[] = {{&S36763[111], 5}, {NULL}};
const Segment SGI258842[] = {{&S36764[13], 6}, {NULL}};
const Segment SGI258843[] = {{&S36764[38], 5}, {NULL}};
const Segment SGI258844[] = {{&S36764[78], 5}, {NULL}};
const Segment SGI258845[] = {{&S36764[111], 5}, {NULL}};
const Segment SGI258846[] = {{&S36765[13], 6}, {NULL}};
const Segment SGI258847[] = {{&S36765[38], 5}, {NULL}};
const Segment SGI258848[] = {{&S36766[13], 6}, {NULL}};
const Segment SGI258849[] = {{&S36766[38], 5}, {NULL}};
const Segment SGI258850[] = {{&S36767[13], 6}, {NULL}};
const Segment SGI258851[] = {{&S36767[38], 3}, {NULL}};
const Segment SGI258852[] = {{&S36767[76], 5}, {NULL}};
const Segment SGI258853[] = {{&S36767[109], 5}, {NULL}};
const Segment SGI258854[] = {{&S36768[13], 6}, {NULL}};
const Segment SGI258855[] = {{&S36768[38], 3}, {NULL}};
const Segment SGI258856[] = {{&S37131[13], 6}, {NULL}};
const Segment SGI258857[] = {{&S37131[38], 5}, {NULL}};
const Segment SGI258858[] = {{&S37131[78], 6}, {NULL}};
const Segment SGI258859[] = {{&S37131[112], 4}, {NULL}};
const Segment SGI258860[] = {{&S37132[13], 6}, {NULL}};
const Segment SGI258861[] = {{&S37132[38], 5}, {NULL}};
const Segment SGI258862[] = {{&S37132[78], 6}, {NULL}};
const Segment SGI258863[] = {{&S37133[13], 6}, {NULL}};
const Segment SGI258864[] = {{&S37133[38], 5}, {NULL}};
const Segment SGI258865[] = {{&S37133[78], 6}, {NULL}};
const Segment SGI258866[] = {{&S37133[112], 4}, {NULL}};
const Segment SGI258867[] = {{&S37134[13], 6}, {NULL}};
const Segment SGI258868[] = {{&S37134[38], 5}, {NULL}};
const Segment SGI258869[] = {{&S37134[78], 6}, {NULL}};
const Segment SGI258870[] = {{&S37135[13], 6}, {NULL}};
const Segment SGI258871[] = {{&S37135[38], 3}, {NULL}};
const Segment SGI258872[] = {{&S37135[76], 6}, {NULL}};
const Segment SGI258873[] = {{&S37135[110], 5}, {NULL}};
const Segment SGI258874[] = {{&S37136[13], 6}, {NULL}};
const Segment SGI258875[] = {{&S37136[38], 3}, {NULL}};
const Segment SGI258876[] = {{&S37136[76], 6}, {NULL}};
const Segment SGI258877[] = {{&S37137[13], 6}, {NULL}};
const Segment SGI258878[] = {{&S37137[38], 5}, {NULL}};
const Segment SGI258879[] = {{&S37137[78], 6}, {NULL}};
const Segment SGI258880[] = {{&S37137[112], 4}, {NULL}};
const Segment SGI258881[] = {{&S37137[124], 6}, {NULL}};
const Segment SGI258882[] = {{&S37138[13], 6}, {NULL}};
const Segment SGI258883[] = {{&S37138[38], 5}, {NULL}};
const Segment SGI258884[] = {{&S37138[78], 8}, {NULL}};
const Segment SGI258885[] = {{&S37139[13], 6}, {NULL}};
const Segment SGI258886[] = {{&S37139[38], 5}, {NULL}};
const Segment SGI258887[] = {{&S37139[78], 6}, {NULL}};
const Segment SGI258888[] = {{&S37139[112], 4}, {NULL}};
const Segment SGI258889[] = {{&S37140[13], 6}, {NULL}};
const Segment SGI258890[] = {{&S37140[38], 5}, {NULL}};
const Segment SGI258891[] = {{&S37140[78], 6}, {NULL}};
const Segment SGI258892[] = {{&S37140[112], 4}, {NULL}};
const Segment SGI258893[] = {{&S37141[13], 6}, {NULL}};
const Segment SGI258894[] = {{&S37141[38], 3}, {NULL}};
const Segment SGI258895[] = {{&S37141[76], 6}, {NULL}};
const Segment SGI258896[] = {{&S37141[110], 5}, {NULL}};
const Segment SGI258897[] = {{&S37145[13], 6}, {NULL}};
const Segment SGI258898[] = {{&S37145[38], 5}, {NULL}};
const Segment SGI258899[] = {{&S37145[78], 6}, {NULL}};
const Segment SGI258900[] = {{&S37145[112], 4}, {NULL}};
const Segment SGI258901[] = {{&S37145[124], 6}, {NULL}};
const Segment SGI258902[] = {{&S37148[13], 6}, {NULL}};
const Segment SGI258903[] = {{&S37148[38], 1}, {NULL}};
const Segment SGI258904[] = {{&S37148[78], 6}, {NULL}};
const Segment SGI258905[] = {{&S37148[112], 4}, {NULL}};
const Segment SGI258906[] = {{&S19337[10], 4}, {NULL}};
const Segment SGI258907[] = {{&S19337[15], 5}, {NULL}};
const Segment SGI258908[] = {{&S19386[10], 4}, {NULL}};
const Segment SGI258909[] = {{&S19386[15], 5}, {NULL}};
const Segment SGI258910[] = {{&S26555[159], 21}, {&S26555[181], 32}, {NULL}};
const Segment SGI258911[] = {{&S26555[0], 213}, {NULL}};
const Segment SGI258912[] = {{&S26556[185], 32}, {NULL}};
const Segment SGI258913[] = {{&S26556[0], 217}, {NULL}};
const Segment SGI258914[] = {{&S37452[13], 4}, {NULL}};
const Segment SGI258915[] = {{&S37452[29], 5}, {NULL}};
const Segment SGI258916[] = {{&S37452[69], 8}, {NULL}};
const Segment SGI258917[] = {{&S37452[125], 3}, {NULL}};
const Segment SGI258918[] = {{&S37452[136], 9}, {NULL}};
const Segment SGI258919[] = {{&S37453[13], 4}, {NULL}};
const Segment SGI258920[] = {{&S37453[29], 5}, {NULL}};
const Segment SGI258921[] = {{&S37453[69], 8}, {NULL}};
const Segment SGI258922[] = {{&S37453[125], 3}, {NULL}};
const Segment SGI258923[] = {{&S37453[136], 9}, {NULL}};
const Segment SGI258924[] = {{&S37454[13], 4}, {NULL}};
const Segment SGI258925[] = {{&S37454[29], 5}, {NULL}};
const Segment SGI258926[] = {{&S37454[69], 8}, {NULL}};
const Segment SGI258927[] = {{&S37454[125], 3}, {NULL}};
const Segment SGI258928[] = {{&S37454[136], 9}, {NULL}};
const Segment SGI258929[] = {{&S37455[13], 4}, {NULL}};
const Segment SGI258930[] = {{&S37455[29], 5}, {NULL}};
const Segment SGI258931[] = {{&S37455[69], 8}, {NULL}};
const Segment SGI258932[] = {{&S37455[125], 3}, {NULL}};
const Segment SGI258933[] = {{&S37455[136], 9}, {NULL}};
const Segment SGI258934[] = {{&S37456[13], 4}, {NULL}};
const Segment SGI258935[] = {{&S37456[29], 5}, {NULL}};
const Segment SGI258936[] = {{&S37456[69], 8}, {NULL}};
const Segment SGI258937[] = {{&S37456[125], 3}, {NULL}};
const Segment SGI258938[] = {{&S37456[136], 9}, {NULL}};
const Segment SGI258939[] = {{&S37457[13], 4}, {NULL}};
const Segment SGI258940[] = {{&S37457[29], 3}, {NULL}};
const Segment SGI258941[] = {{&S37457[67], 8}, {NULL}};
const Segment SGI258942[] = {{&S37457[123], 5}, {NULL}};
const Segment SGI258943[] = {{&S37457[136], 9}, {NULL}};
const Segment SGI258944[] = {{&S37458[13], 4}, {NULL}};
const Segment SGI258945[] = {{&S37458[29], 5}, {NULL}};
const Segment SGI258946[] = {{&S37458[69], 8}, {NULL}};
const Segment SGI258947[] = {{&S37458[125], 3}, {NULL}};
const Segment SGI258948[] = {{&S37458[136], 9}, {NULL}};
const Segment SGI258949[] = {{&S37459[13], 4}, {NULL}};
const Segment SGI258950[] = {{&S37459[29], 3}, {NULL}};
const Segment SGI258951[] = {{&S37459[67], 8}, {NULL}};
const Segment SGI258952[] = {{&S37459[123], 5}, {NULL}};
const Segment SGI258953[] = {{&S37459[136], 9}, {NULL}};
const Segment SGI258954[] = {{&S37460[13], 4}, {NULL}};
const Segment SGI258955[] = {{&S37460[29], 5}, {NULL}};
const Segment SGI258956[] = {{&S37460[69], 8}, {NULL}};
const Segment SGI258957[] = {{&S37460[125], 3}, {NULL}};
const Segment SGI258958[] = {{&S37460[136], 9}, {NULL}};
const Segment SGI258959[] = {{&S37532[13], 4}, {NULL}};
const Segment SGI258960[] = {{&S37532[33], 5}, {NULL}};
const Segment SGI258961[] = {{&S37532[73], 6}, {NULL}};
const Segment SGI258962[] = {{&S37532[127], 6}, {NULL}};
const Segment SGI258963[] = {{&S37532[141], 7}, {NULL}};
const Segment SGI258964[] = {{&S37533[13], 4}, {NULL}};
const Segment SGI258965[] = {{&S37533[33], 3}, {NULL}};
const Segment SGI258966[] = {{&S37533[71], 6}, {NULL}};
const Segment SGI258967[] = {{&S37533[125], 6}, {NULL}};
const Segment SGI258968[] = {{&S37533[139], 7}, {NULL}};
const Segment SGI258969[] = {{&S37534[13], 4}, {NULL}};
const Segment SGI258970[] = {{&S37534[33], 3}, {NULL}};
const Segment SGI258971[] = {{&S37534[71], 6}, {NULL}};
const Segment SGI258972[] = {{&S37534[125], 6}, {NULL}};
const Segment SGI258973[] = {{&S37534[139], 7}, {NULL}};
const Segment SGI258974[] = {{&S37535[13], 4}, {NULL}};
const Segment SGI258975[] = {{&S37535[33], 3}, {NULL}};
const Segment SGI258976[] = {{&S37535[71], 6}, {NULL}};
const Segment SGI258977[] = {{&S37535[123], 6}, {NULL}};
const Segment SGI258978[] = {{&S37535[137], 7}, {NULL}};
const Segment SGI258979[] = {{&S37549[13], 6}, {NULL}};
const Segment SGI258980[] = {{&S37549[35], 5}, {NULL}};
const Segment SGI258981[] = {{&S37549[75], 8}, {NULL}};
const Segment SGI258982[] = {{&S37549[130], 5}, {NULL}};
const Segment SGI258983[] = {{&S37549[143], 9}, {NULL}};
const Segment SGI258984[] = {{&S37550[13], 6}, {NULL}};
const Segment SGI258985[] = {{&S37550[35], 5}, {NULL}};
const Segment SGI258986[] = {{&S37550[75], 8}, {NULL}};
const Segment SGI258987[] = {{&S37550[130], 5}, {NULL}};
const Segment SGI258988[] = {{&S37550[143], 9}, {NULL}};
const Segment SGI258989[] = {{&S37551[13], 6}, {NULL}};
const Segment SGI258990[] = {{&S37551[35], 3}, {NULL}};
const Segment SGI258991[] = {{&S37551[73], 8}, {NULL}};
const Segment SGI258992[] = {{&S37551[128], 5}, {NULL}};
const Segment SGI258993[] = {{&S37551[141], 9}, {NULL}};
const Segment SGI258994[] = {{&S37580[13], 4}, {NULL}};
const Segment SGI258995[] = {{&S37580[26], 3}, {NULL}};
const Segment SGI258996[] = {{&S37580[57], 6}, {NULL}};
const Segment SGI258997[] = {{&S37580[91], 6}, {NULL}};
const Segment SGI258998[] = {{&S37580[98], 7}, {NULL}};
const Segment SGI258999[] = {{&S37581[13], 4}, {NULL}};
const Segment SGI259000[] = {{&S37581[26], 5}, {NULL}};
const Segment SGI259001[] = {{&S37581[59], 6}, {NULL}};
const Segment SGI259002[] = {{&S37581[93], 6}, {NULL}};
const Segment SGI259003[] = {{&S37581[100], 7}, {NULL}};
const Segment SGI259004[] = {{&S37582[13], 4}, {NULL}};
const Segment SGI259005[] = {{&S37582[26], 5}, {NULL}};
const Segment SGI259006[] = {{&S37582[59], 6}, {NULL}};
const Segment SGI259007[] = {{&S37582[93], 6}, {NULL}};
const Segment SGI259008[] = {{&S37582[100], 7}, {NULL}};
const Segment SGI259009[] = {{&S37583[13], 4}, {NULL}};
const Segment SGI259010[] = {{&S37583[26], 5}, {NULL}};
const Segment SGI259011[] = {{&S37583[59], 6}, {NULL}};
const Segment SGI259012[] = {{&S37583[93], 6}, {NULL}};
const Segment SGI259013[] = {{&S37583[100], 7}, {NULL}};
const Segment SGI259014[] = {{&S37638[13], 4}, {NULL}};
const Segment SGI259015[] = {{&S37638[29], 5}, {NULL}};
const Segment SGI259016[] = {{&S37638[69], 8}, {NULL}};
const Segment SGI259017[] = {{&S37638[124], 5}, {NULL}};
const Segment SGI259018[] = {{&S37638[137], 9}, {NULL}};
const Segment SGI259019[] = {{&S37639[13], 4}, {NULL}};
const Segment SGI259020[] = {{&S37639[29], 3}, {NULL}};
const Segment SGI259021[] = {{&S37639[67], 8}, {NULL}};
const Segment SGI259022[] = {{&S37639[122], 5}, {NULL}};
const Segment SGI259023[] = {{&S37639[135], 9}, {NULL}};
const Segment SGI259024[] = {{&S37640[13], 4}, {NULL}};
const Segment SGI259025[] = {{&S37640[29], 3}, {NULL}};
const Segment SGI259026[] = {{&S37640[67], 8}, {NULL}};
const Segment SGI259027[] = {{&S37640[122], 5}, {NULL}};
const Segment SGI259028[] = {{&S37640[135], 9}, {NULL}};
const Segment SGI259029[] = {{&S20182[10], 4}, {NULL}};
const Segment SGI259030[] = {{&S20182[15], 5}, {NULL}};
const Segment SGI259031[] = {{&S37793[13], 4}, {NULL}};
const Segment SGI259032[] = {{&S37793[26], 5}, {NULL}};
const Segment SGI259033[] = {{&S37793[59], 6}, {NULL}};
const Segment SGI259034[] = {{&S37793[112], 6}, {NULL}};
const Segment SGI259035[] = {{&S37793[126], 7}, {NULL}};
const Segment SGI259036[] = {{&S37794[13], 4}, {NULL}};
const Segment SGI259037[] = {{&S37794[26], 5}, {NULL}};
const Segment SGI259038[] = {{&S37794[59], 6}, {NULL}};
const Segment SGI259039[] = {{&S37794[112], 6}, {NULL}};
const Segment SGI259040[] = {{&S37794[126], 7}, {NULL}};
const Segment SGI259041[] = {{&S37795[13], 4}, {NULL}};
const Segment SGI259042[] = {{&S37795[26], 3}, {NULL}};
const Segment SGI259043[] = {{&S37795[57], 6}, {NULL}};
const Segment SGI259044[] = {{&S37795[110], 6}, {NULL}};
const Segment SGI259045[] = {{&S37795[124], 7}, {NULL}};
const Segment SGI259046[] = {{&S37796[13], 4}, {NULL}};
const Segment SGI259047[] = {{&S37796[26], 5}, {NULL}};
const Segment SGI259048[] = {{&S37796[59], 6}, {NULL}};
const Segment SGI259049[] = {{&S37796[112], 6}, {NULL}};
const Segment SGI259050[] = {{&S37796[126], 7}, {NULL}};
const Segment SGI259051[] = {{&S38021[13], 4}, {NULL}};
const Segment SGI259052[] = {{&S38021[26], 3}, {NULL}};
const Segment SGI259053[] = {{&S38021[64], 8}, {NULL}};
const Segment SGI259054[] = {{&S38021[119], 5}, {NULL}};
const Segment SGI259055[] = {{&S38021[132], 9}, {NULL}};
const Segment SGI259056[] = {{&S38022[13], 4}, {NULL}};
const Segment SGI259057[] = {{&S38022[26], 5}, {NULL}};
const Segment SGI259058[] = {{&S38022[66], 8}, {NULL}};
const Segment SGI259059[] = {{&S38022[121], 5}, {NULL}};
const Segment SGI259060[] = {{&S38022[134], 9}, {NULL}};
const Segment SGI259061[] = {{&S38236[13], 4}, {NULL}};
const Segment SGI259062[] = {{&S38236[33], 5}, {NULL}};
const Segment SGI259063[] = {{&S38236[66], 6}, {NULL}};
const Segment SGI259064[] = {{&S38236[119], 6}, {NULL}};
const Segment SGI259065[] = {{&S38236[133], 7}, {NULL}};
const Segment SGI259066[] = {{&S38237[13], 4}, {NULL}};
const Segment SGI259067[] = {{&S38237[33], 5}, {NULL}};
const Segment SGI259068[] = {{&S38237[66], 6}, {NULL}};
const Segment SGI259069[] = {{&S38237[119], 6}, {NULL}};
const Segment SGI259070[] = {{&S38237[133], 7}, {NULL}};
const Segment SGI259071[] = {{&S38238[13], 4}, {NULL}};
const Segment SGI259072[] = {{&S38238[33], 3}, {NULL}};
const Segment SGI259073[] = {{&S38238[64], 6}, {NULL}};
const Segment SGI259074[] = {{&S38238[117], 6}, {NULL}};
const Segment SGI259075[] = {{&S38238[131], 7}, {NULL}};
const Segment SGI259076[] = {{&S38239[13], 4}, {NULL}};
const Segment SGI259077[] = {{&S38239[33], 3}, {NULL}};
const Segment SGI259078[] = {{&S38239[64], 6}, {NULL}};
const Segment SGI259079[] = {{&S38239[117], 6}, {NULL}};
const Segment SGI259080[] = {{&S38239[131], 7}, {NULL}};
const Segment SGI259081[] = {{&S38266[13], 4}, {NULL}};
const Segment SGI259082[] = {{&S38266[26], 5}, {NULL}};
const Segment SGI259083[] = {{&S38266[59], 6}, {NULL}};
const Segment SGI259084[] = {{&S38266[112], 5}, {NULL}};
const Segment SGI259085[] = {{&S38266[125], 9}, {NULL}};
const Segment SGI259086[] = {{&S38267[13], 4}, {NULL}};
const Segment SGI259087[] = {{&S38267[26], 3}, {NULL}};
const Segment SGI259088[] = {{&S38267[57], 6}, {NULL}};
const Segment SGI259089[] = {{&S38267[110], 5}, {NULL}};
const Segment SGI259090[] = {{&S38267[123], 9}, {NULL}};
const Segment SGI259091[] = {{&S38268[13], 4}, {NULL}};
const Segment SGI259092[] = {{&S38268[26], 3}, {NULL}};
const Segment SGI259093[] = {{&S38268[57], 6}, {NULL}};
const Segment SGI259094[] = {{&S38268[110], 5}, {NULL}};
const Segment SGI259095[] = {{&S38268[123], 9}, {NULL}};
const Segment SGI259096[] = {{&S38330[13], 4}, {NULL}};
const Segment SGI259097[] = {{&S38330[33], 3}, {NULL}};
const Segment SGI259098[] = {{&S38330[71], 8}, {NULL}};
const Segment SGI259099[] = {{&S38330[126], 6}, {NULL}};
const Segment SGI259100[] = {{&S38330[140], 9}, {NULL}};
const Segment SGI259101[] = {{&S38331[13], 4}, {NULL}};
const Segment SGI259102[] = {{&S38331[33], 3}, {NULL}};
const Segment SGI259103[] = {{&S38331[71], 8}, {NULL}};
const Segment SGI259104[] = {{&S38331[126], 6}, {NULL}};
const Segment SGI259105[] = {{&S38331[140], 9}, {NULL}};
const Segment SGI259106[] = {{&S38332[13], 4}, {NULL}};
const Segment SGI259107[] = {{&S38332[33], 5}, {NULL}};
const Segment SGI259108[] = {{&S38332[73], 8}, {NULL}};
const Segment SGI259109[] = {{&S38332[128], 6}, {NULL}};
const Segment SGI259110[] = {{&S38332[142], 9}, {NULL}};
const Segment SGI259111[] = {{&S38333[13], 4}, {NULL}};
const Segment SGI259112[] = {{&S38333[33], 3}, {NULL}};
const Segment SGI259113[] = {{&S38333[71], 8}, {NULL}};
const Segment SGI259114[] = {{&S38333[125], 6}, {NULL}};
const Segment SGI259115[] = {{&S38333[139], 9}, {NULL}};
const Segment SGI259116[] = {{&S38334[13], 4}, {NULL}};
const Segment SGI259117[] = {{&S38334[33], 2}, {NULL}};
const Segment SGI259118[] = {{&S38334[70], 8}, {NULL}};
const Segment SGI259119[] = {{&S38334[124], 6}, {NULL}};
const Segment SGI259120[] = {{&S38334[138], 9}, {NULL}};
const Segment SGI259121[] = {{&S21284[10], 4}, {NULL}};
const Segment SGI259122[] = {{&S21284[15], 5}, {NULL}};
const Segment SGI259123[] = {{&S21285[10], 4}, {NULL}};
const Segment SGI259124[] = {{&S21285[15], 5}, {NULL}};
const Segment SGI259125[] = {{&S21287[41], 36}, {NULL}};
const Segment SGI259126[] = {{&S38556[13], 4}, {NULL}};
const Segment SGI259127[] = {{&S38556[36], 3}, {NULL}};
const Segment SGI259128[] = {{&S38556[74], 9}, {NULL}};
const Segment SGI259129[] = {{&S38556[111], 4}, {NULL}};
const Segment SGI259130[] = {{&S38557[13], 4}, {NULL}};
const Segment SGI259131[] = {{&S38557[29], 3}, {NULL}};
const Segment SGI259132[] = {{&S38557[67], 9}, {NULL}};
const Segment SGI259133[] = {{&S38557[104], 5}, {NULL}};
const Segment SGI259134[] = {{&S38558[13], 4}, {NULL}};
const Segment SGI259135[] = {{&S38558[29], 3}, {NULL}};
const Segment SGI259136[] = {{&S38558[67], 9}, {NULL}};
const Segment SGI259137[] = {{&S38559[13], 4}, {NULL}};
const Segment SGI259138[] = {{&S38559[29], 5}, {NULL}};
const Segment SGI259139[] = {{&S38559[69], 9}, {NULL}};
const Segment SGI259140[] = {{&S38559[106], 5}, {NULL}};
const Segment SGI259141[] = {{&S38560[13], 4}, {NULL}};
const Segment SGI259142[] = {{&S38560[29], 5}, {NULL}};
const Segment SGI259143[] = {{&S38560[69], 9}, {NULL}};
const Segment SGI259144[] = {{&S38561[13], 4}, {NULL}};
const Segment SGI259145[] = {{&S38561[29], 5}, {NULL}};
const Segment SGI259146[] = {{&S38561[69], 9}, {NULL}};
const Segment SGI259147[] = {{&S38562[13], 4}, {NULL}};
const Segment SGI259148[] = {{&S38562[29], 5}, {NULL}};
const Segment SGI259149[] = {{&S38562[69], 9}, {NULL}};
const Segment SGI259150[] = {{&S38562[106], 5}, {NULL}};
const Segment SGI259151[] = {{&S38563[13], 4}, {NULL}};
const Segment SGI259152[] = {{&S38563[29], 3}, {NULL}};
const Segment SGI259153[] = {{&S38563[67], 9}, {NULL}};
const Segment SGI259154[] = {{&S38563[104], 5}, {NULL}};
const Segment SGI259155[] = {{&S38563[117], 6}, {NULL}};
const Segment SGI259156[] = {{&S38564[13], 4}, {NULL}};
const Segment SGI259157[] = {{&S38564[29], 3}, {NULL}};
const Segment SGI259158[] = {{&S38564[67], 9}, {NULL}};
const Segment SGI259159[] = {{&S38564[104], 5}, {NULL}};
const Segment SGI259160[] = {{&S38565[13], 4}, {NULL}};
const Segment SGI259161[] = {{&S38565[29], 5}, {NULL}};
const Segment SGI259162[] = {{&S38565[69], 9}, {NULL}};
const Segment SGI259163[] = {{&S38565[106], 5}, {NULL}};
const Segment SGI259164[] = {{&S38566[13], 4}, {NULL}};
const Segment SGI259165[] = {{&S38566[29], 5}, {NULL}};
const Segment SGI259166[] = {{&S38566[69], 9}, {NULL}};
const Segment SGI259167[] = {{&S38566[106], 5}, {NULL}};
const Segment SGI259168[] = {{&S38567[12], 4}, {NULL}};
const Segment SGI259169[] = {{&S38567[31], 3}, {NULL}};
const Segment SGI259170[] = {{&S38567[55], 9}, {NULL}};
const Segment SGI259171[] = {{&S38567[72], 5}, {NULL}};
const Segment SGI259172[] = {{&S38677[13], 4}, {NULL}};
const Segment SGI259173[] = {{&S38677[26], 5}, {NULL}};
const Segment SGI259174[] = {{&S38677[59], 6}, {NULL}};
const Segment SGI259175[] = {{&S38677[112], 6}, {NULL}};
const Segment SGI259176[] = {{&S38677[126], 7}, {NULL}};
const Segment SGI259177[] = {{&S38678[13], 4}, {NULL}};
const Segment SGI259178[] = {{&S38678[26], 5}, {NULL}};
const Segment SGI259179[] = {{&S38678[59], 6}, {NULL}};
const Segment SGI259180[] = {{&S38678[112], 6}, {NULL}};
const Segment SGI259181[] = {{&S38678[126], 7}, {NULL}};
const Segment SGI259182[] = {{&S38679[13], 4}, {NULL}};
const Segment SGI259183[] = {{&S38679[26], 3}, {NULL}};
const Segment SGI259184[] = {{&S38679[57], 6}, {NULL}};
const Segment SGI259185[] = {{&S38679[110], 6}, {NULL}};
const Segment SGI259186[] = {{&S38679[124], 7}, {NULL}};
const Segment SGI259187[] = {{&S38680[13], 4}, {NULL}};
const Segment SGI259188[] = {{&S38680[26], 5}, {NULL}};
const Segment SGI259189[] = {{&S38680[59], 6}, {NULL}};
const Segment SGI259190[] = {{&S38680[112], 6}, {NULL}};
const Segment SGI259191[] = {{&S38680[126], 7}, {NULL}};
const Segment SGI259192[] = {{&S38818[13], 4}, {NULL}};
const Segment SGI259193[] = {{&S38818[26], 3}, {NULL}};
const Segment SGI259194[] = {{&S38818[57], 6}, {NULL}};
const Segment SGI259195[] = {{&S38818[110], 8}, {NULL}};
const Segment SGI259196[] = {{&S38818[126], 7}, {NULL}};
const Segment SGI259197[] = {{&S38819[13], 4}, {NULL}};
const Segment SGI259198[] = {{&S38819[26], 3}, {NULL}};
const Segment SGI259199[] = {{&S38819[57], 6}, {NULL}};
const Segment SGI259200[] = {{&S38819[110], 8}, {NULL}};
const Segment SGI259201[] = {{&S38819[126], 7}, {NULL}};
const Segment SGI259202[] = {{&S38820[13], 4}, {NULL}};
const Segment SGI259203[] = {{&S38820[26], 3}, {NULL}};
const Segment SGI259204[] = {{&S38820[57], 6}, {NULL}};
const Segment SGI259205[] = {{&S38820[110], 8}, {NULL}};
const Segment SGI259206[] = {{&S38820[126], 7}, {NULL}};
const Segment SGI259207[] = {{&S38821[13], 4}, {NULL}};
const Segment SGI259208[] = {{&S38821[26], 3}, {NULL}};
const Segment SGI259209[] = {{&S38821[57], 6}, {NULL}};
const Segment SGI259210[] = {{&S38821[110], 8}, {NULL}};
const Segment SGI259211[] = {{&S38821[126], 7}, {NULL}};
const Segment SGI259212[] = {{&S38822[13], 4}, {NULL}};
const Segment SGI259213[] = {{&S38822[26], 3}, {NULL}};
const Segment SGI259214[] = {{&S38822[57], 6}, {NULL}};
const Segment SGI259215[] = {{&S38822[110], 8}, {NULL}};
const Segment SGI259216[] = {{&S38822[126], 7}, {NULL}};
const Segment SGI259217[] = {{&S38823[13], 4}, {NULL}};
const Segment SGI259218[] = {{&S38823[26], 3}, {NULL}};
const Segment SGI259219[] = {{&S38823[57], 6}, {NULL}};
const Segment SGI259220[] = {{&S38823[110], 6}, {NULL}};
const Segment SGI259221[] = {{&S38823[124], 7}, {NULL}};
const Segment SGI259222[] = {{&S38824[13], 4}, {NULL}};
const Segment SGI259223[] = {{&S38824[26], 5}, {NULL}};
const Segment SGI259224[] = {{&S38824[59], 6}, {NULL}};
const Segment SGI259225[] = {{&S38824[112], 6}, {NULL}};
const Segment SGI259226[] = {{&S38824[126], 7}, {NULL}};
const Segment SGI259227[] = {{&S38887[13], 4}, {NULL}};
const Segment SGI259228[] = {{&S38887[26], 5}, {NULL}};
const Segment SGI259229[] = {{&S38887[59], 6}, {NULL}};
const Segment SGI259230[] = {{&S38887[112], 5}, {NULL}};
const Segment SGI259231[] = {{&S38887[125], 9}, {NULL}};
const Segment SGI259232[] = {{&S38888[13], 4}, {NULL}};
const Segment SGI259233[] = {{&S38888[26], 3}, {NULL}};
const Segment SGI259234[] = {{&S38888[57], 6}, {NULL}};
const Segment SGI259235[] = {{&S38888[110], 5}, {NULL}};
const Segment SGI259236[] = {{&S38888[123], 9}, {NULL}};
const Segment SGI259237[] = {{&S38889[13], 4}, {NULL}};
const Segment SGI259238[] = {{&S38889[26], 3}, {NULL}};
const Segment SGI259239[] = {{&S38889[57], 6}, {NULL}};
const Segment SGI259240[] = {{&S38889[110], 5}, {NULL}};
const Segment SGI259241[] = {{&S38889[123], 9}, {NULL}};
const Segment SGI259242[] = {{&S39649[13], 6}, {NULL}};
const Segment SGI259243[] = {{&S39649[59], 5}, {NULL}};
const Segment SGI259244[] = {{&S39651[13], 6}, {NULL}};
const Segment SGI259245[] = {{&S39651[59], 3}, {NULL}};
const Segment SGI259246[] = {{&S39664[29], 6}, {NULL}};
const Segment SGI259247[] = {{&S39694[13], 4}, {NULL}};
const Segment SGI259248[] = {{&S39694[33], 5}, {NULL}};
const Segment SGI259249[] = {{&S39694[66], 6}, {NULL}};
const Segment SGI259250[] = {{&S39694[119], 6}, {NULL}};
const Segment SGI259251[] = {{&S39694[133], 7}, {NULL}};
const Segment SGI259252[] = {{&S39695[13], 4}, {NULL}};
const Segment SGI259253[] = {{&S39695[33], 5}, {NULL}};
const Segment SGI259254[] = {{&S39695[66], 6}, {NULL}};
const Segment SGI259255[] = {{&S39695[119], 6}, {NULL}};
const Segment SGI259256[] = {{&S39695[133], 7}, {NULL}};
const Segment SGI259257[] = {{&S39696[13], 4}, {NULL}};
const Segment SGI259258[] = {{&S39696[33], 3}, {NULL}};
const Segment SGI259259[] = {{&S39696[64], 6}, {NULL}};
const Segment SGI259260[] = {{&S39696[117], 6}, {NULL}};
const Segment SGI259261[] = {{&S39696[131], 7}, {NULL}};
const Segment SGI259262[] = {{&S39697[13], 4}, {NULL}};
const Segment SGI259263[] = {{&S39697[33], 3}, {NULL}};
const Segment SGI259264[] = {{&S39697[64], 6}, {NULL}};
const Segment SGI259265[] = {{&S39697[117], 6}, {NULL}};
const Segment SGI259266[] = {{&S39697[131], 7}, {NULL}};
const Segment SGI259267[] = {{&S39826[13], 6}, {NULL}};
const Segment SGI259268[] = {{&S39826[92], 3}, {NULL}};
const Segment SGI259269[] = {{&S39826[59], 3}, {NULL}};
const Segment SGI259270[] = {{&S39826[103], 5}, {NULL}};
const Segment SGI259271[] = {{&S39826[116], 5}, {NULL}};
const Segment SGI259272[] = {{&S39827[13], 6}, {NULL}};
const Segment SGI259273[] = {{&S39827[92], 3}, {NULL}};
const Segment SGI259274[] = {{&S39827[59], 3}, {NULL}};
const Segment SGI259275[] = {{&S39827[103], 6}, {NULL}};
const Segment SGI259276[] = {{&S39827[117], 5}, {NULL}};
const Segment SGI259277[] = {{&S39828[13], 6}, {NULL}};
const Segment SGI259278[] = {{&S39828[92], 3}, {NULL}};
const Segment SGI259279[] = {{&S39828[59], 3}, {NULL}};
const Segment SGI259280[] = {{&S39828[103], 4}, {NULL}};
const Segment SGI259281[] = {{&S39828[115], 5}, {NULL}};
const Segment SGI259282[] = {{&S39829[13], 6}, {NULL}};
const Segment SGI259283[] = {{&S39829[92], 3}, {NULL}};
const Segment SGI259284[] = {{&S39829[59], 3}, {NULL}};
const Segment SGI259285[] = {{&S39829[103], 6}, {NULL}};
const Segment SGI259286[] = {{&S39829[117], 5}, {NULL}};
const Segment SGI259287[] = {{&S39830[13], 6}, {NULL}};
const Segment SGI259288[] = {{&S39830[92], 3}, {NULL}};
const Segment SGI259289[] = {{&S39830[59], 3}, {NULL}};
const Segment SGI259290[] = {{&S39830[103], 4}, {NULL}};
const Segment SGI259291[] = {{&S39830[115], 5}, {NULL}};
const Segment SGI259292[] = {{&S39831[13], 6}, {NULL}};
const Segment SGI259293[] = {{&S39831[93], 3}, {NULL}};
const Segment SGI259294[] = {{&S39831[59], 5}, {NULL}};
const Segment SGI259295[] = {{&S39831[104], 5}, {NULL}};
const Segment SGI259296[] = {{&S39831[117], 5}, {NULL}};
const Segment SGI259297[] = {{&S39832[13], 6}, {NULL}};
const Segment SGI259298[] = {{&S39832[93], 3}, {NULL}};
const Segment SGI259299[] = {{&S39832[59], 5}, {NULL}};
const Segment SGI259300[] = {{&S39832[104], 5}, {NULL}};
const Segment SGI259301[] = {{&S39832[117], 5}, {NULL}};
const Segment SGI259302[] = {{&S39833[13], 6}, {NULL}};
const Segment SGI259303[] = {{&S39833[93], 3}, {NULL}};
const Segment SGI259304[] = {{&S39833[59], 5}, {NULL}};
const Segment SGI259305[] = {{&S39833[104], 6}, {NULL}};
const Segment SGI259306[] = {{&S39833[118], 5}, {NULL}};
const Segment SGI259307[] = {{&S39834[13], 6}, {NULL}};
const Segment SGI259308[] = {{&S39834[94], 3}, {NULL}};
const Segment SGI259309[] = {{&S39834[61], 3}, {NULL}};
const Segment SGI259310[] = {{&S39834[105], 3}, {NULL}};
const Segment SGI259311[] = {{&S39834[116], 5}, {NULL}};
const Segment SGI259312[] = {{&S39835[13], 6}, {NULL}};
const Segment SGI259313[] = {{&S39835[93], 3}, {NULL}};
const Segment SGI259314[] = {{&S39835[59], 5}, {NULL}};
const Segment SGI259315[] = {{&S39835[104], 5}, {NULL}};
const Segment SGI259316[] = {{&S39835[117], 5}, {NULL}};
const Segment SGI259317[] = {{&S39836[13], 6}, {NULL}};
const Segment SGI259318[] = {{&S39836[93], 3}, {NULL}};
const Segment SGI259319[] = {{&S39836[59], 5}, {NULL}};
const Segment SGI259320[] = {{&S39836[104], 4}, {NULL}};
const Segment SGI259321[] = {{&S39836[116], 5}, {NULL}};
const Segment SGI259322[] = {{&S39837[13], 6}, {NULL}};
const Segment SGI259323[] = {{&S39837[92], 3}, {NULL}};
const Segment SGI259324[] = {{&S39837[59], 3}, {NULL}};
const Segment SGI259325[] = {{&S39837[103], 6}, {NULL}};
const Segment SGI259326[] = {{&S39837[117], 5}, {NULL}};
const Segment SGI259327[] = {{&S39838[13], 6}, {NULL}};
const Segment SGI259328[] = {{&S39838[93], 3}, {NULL}};
const Segment SGI259329[] = {{&S39838[59], 5}, {NULL}};
const Segment SGI259330[] = {{&S39838[104], 6}, {NULL}};
const Segment SGI259331[] = {{&S39838[118], 5}, {NULL}};
const Segment SGI259332[] = {{&S39839[13], 6}, {NULL}};
const Segment SGI259333[] = {{&S39839[92], 3}, {NULL}};
const Segment SGI259334[] = {{&S39839[59], 3}, {NULL}};
const Segment SGI259335[] = {{&S39839[103], 5}, {NULL}};
const Segment SGI259336[] = {{&S39840[13], 6}, {NULL}};
const Segment SGI259337[] = {{&S39840[92], 3}, {NULL}};
const Segment SGI259338[] = {{&S39840[59], 3}, {NULL}};
const Segment SGI259339[] = {{&S39840[103], 6}, {NULL}};
const Segment SGI259340[] = {{&S39840[117], 5}, {NULL}};
const Segment SGI259341[] = {{&S39841[13], 6}, {NULL}};
const Segment SGI259342[] = {{&S39841[93], 3}, {NULL}};
const Segment SGI259343[] = {{&S39841[59], 5}, {NULL}};
const Segment SGI259344[] = {{&S39841[104], 5}, {NULL}};
const Segment SGI259345[] = {{&S39841[117], 3}, {NULL}};
const Segment SGI259346[] = {{&S39842[13], 6}, {NULL}};
const Segment SGI259347[] = {{&S39842[92], 3}, {NULL}};
const Segment SGI259348[] = {{&S39842[59], 3}, {NULL}};
const Segment SGI259349[] = {{&S39842[103], 4}, {NULL}};
const Segment SGI259350[] = {{&S39842[115], 5}, {&S39842[128], 6}, {NULL}};
const Segment SGI259351[] = {{&S39843[13], 6}, {NULL}};
const Segment SGI259352[] = {{&S39843[92], 3}, {NULL}};
const Segment SGI259353[] = {{&S39843[59], 3}, {NULL}};
const Segment SGI259354[] = {{&S39843[103], 6}, {NULL}};
const Segment SGI259355[] = {{&S39843[117], 5}, {NULL}};
const Segment SGI259356[] = {{&S39844[13], 6}, {NULL}};
const Segment SGI259357[] = {{&S39844[92], 3}, {NULL}};
const Segment SGI259358[] = {{&S39844[59], 3}, {NULL}};
const Segment SGI259359[] = {{&S39844[103], 3}, {NULL}};
const Segment SGI259360[] = {{&S39844[8], 1}, {NULL}};
const Segment SGI259361[] = {{&S39845[13], 6}, {NULL}};
const Segment SGI259362[] = {{&S39845[92], 3}, {NULL}};
const Segment SGI259363[] = {{&S39845[59], 3}, {NULL}};
const Segment SGI259364[] = {{&S39845[103], 6}, {NULL}};
const Segment SGI259365[] = {{&S39845[8], 1}, {NULL}};
const Segment SGI259366[] = {{&S39846[13], 6}, {NULL}};
const Segment SGI259367[] = {{&S39846[92], 3}, {NULL}};
const Segment SGI259368[] = {{&S39846[59], 3}, {NULL}};
const Segment SGI259369[] = {{&S39846[103], 6}, {NULL}};
const Segment SGI259370[] = {{&S39846[117], 4}, {NULL}};
const Segment SGI259371[] = {{&S39877[13], 4}, {NULL}};
const Segment SGI259372[] = {{&S39877[26], 5}, {NULL}};
const Segment SGI259373[] = {{&S39877[59], 6}, {NULL}};
const Segment SGI259374[] = {{&S39877[107], 6}, {NULL}};
const Segment SGI259375[] = {{&S39877[121], 7}, {NULL}};
const Segment SGI259376[] = {{&S39878[13], 4}, {NULL}};
const Segment SGI259377[] = {{&S39878[26], 3}, {NULL}};
const Segment SGI259378[] = {{&S39878[57], 6}, {NULL}};
const Segment SGI259379[] = {{&S39878[105], 6}, {NULL}};
const Segment SGI259380[] = {{&S39878[119], 7}, {NULL}};
const Segment SGI259381[] = {{&S39880[13], 4}, {NULL}};
const Segment SGI259382[] = {{&S39880[26], 3}, {NULL}};
const Segment SGI259383[] = {{&S39880[57], 6}, {NULL}};
const Segment SGI259384[] = {{&S39880[91], 6}, {NULL}};
const Segment SGI259385[] = {{&S39880[98], 7}, {NULL}};
const Segment SGI259386[] = {{&S39881[13], 4}, {NULL}};
const Segment SGI259387[] = {{&S39881[26], 5}, {NULL}};
const Segment SGI259388[] = {{&S39881[59], 6}, {NULL}};
const Segment SGI259389[] = {{&S39881[93], 6}, {NULL}};
const Segment SGI259390[] = {{&S39881[100], 7}, {NULL}};
const Segment SGI259391[] = {{&S39882[13], 4}, {NULL}};
const Segment SGI259392[] = {{&S39882[26], 3}, {NULL}};
const Segment SGI259393[] = {{&S39882[57], 6}, {NULL}};
const Segment SGI259394[] = {{&S39882[91], 6}, {NULL}};
const Segment SGI259395[] = {{&S39882[98], 7}, {NULL}};
const Segment SGI259396[] = {{&S39883[13], 4}, {NULL}};
const Segment SGI259397[] = {{&S39883[26], 3}, {NULL}};
const Segment SGI259398[] = {{&S39883[57], 6}, {NULL}};
const Segment SGI259399[] = {{&S39883[91], 6}, {NULL}};
const Segment SGI259400[] = {{&S39883[98], 7}, {NULL}};
const Segment SGI259401[] = {{&S39884[13], 4}, {NULL}};
const Segment SGI259402[] = {{&S39884[26], 5}, {NULL}};
const Segment SGI259403[] = {{&S39884[59], 6}, {NULL}};
const Segment SGI259404[] = {{&S39884[93], 7}, {NULL}};
const Segment SGI259405[] = {{&S39884[101], 7}, {NULL}};
const Segment SGI259406[] = {{&S39885[13], 4}, {NULL}};
const Segment SGI259407[] = {{&S39885[26], 5}, {NULL}};
const Segment SGI259408[] = {{&S39885[59], 6}, {NULL}};
const Segment SGI259409[] = {{&S39885[93], 7}, {NULL}};
const Segment SGI259410[] = {{&S39885[101], 7}, {NULL}};
const Segment SGI259411[] = {{&S39886[13], 4}, {NULL}};
const Segment SGI259412[] = {{&S39886[26], 5}, {NULL}};
const Segment SGI259413[] = {{&S39886[59], 6}, {NULL}};
const Segment SGI259414[] = {{&S39886[93], 6}, {NULL}};
const Segment SGI259415[] = {{&S39886[100], 7}, {NULL}};
const Segment SGI259416[] = {{&S39897[13], 6}, {NULL}};
const Segment SGI259417[] = {{&S39897[35], 5}, {NULL}};
const Segment SGI259418[] = {{&S39897[68], 6}, {NULL}};
const Segment SGI259419[] = {{&S39897[116], 6}, {NULL}};
const Segment SGI259420[] = {{&S39897[130], 7}, {NULL}};
const Segment SGI259421[] = {{&S39898[13], 6}, {NULL}};
const Segment SGI259422[] = {{&S39898[35], 3}, {NULL}};
const Segment SGI259423[] = {{&S39898[66], 6}, {NULL}};
const Segment SGI259424[] = {{&S39898[114], 6}, {NULL}};
const Segment SGI259425[] = {{&S39898[128], 7}, {NULL}};
const Segment SGI259426[] = {{&S39910[13], 6}, {NULL}};
const Segment SGI259427[] = {{&S39910[35], 3}, {NULL}};
const Segment SGI259428[] = {{&S39910[66], 6}, {NULL}};
const Segment SGI259429[] = {{&S39910[119], 5}, {NULL}};
const Segment SGI259430[] = {{&S39910[132], 9}, {NULL}};
const Segment SGI259431[] = {{&S39911[13], 6}, {NULL}};
const Segment SGI259432[] = {{&S39911[35], 5}, {NULL}};
const Segment SGI259433[] = {{&S39911[68], 6}, {NULL}};
const Segment SGI259434[] = {{&S39911[121], 5}, {NULL}};
const Segment SGI259435[] = {{&S39911[134], 9}, {NULL}};
const Segment SGI259436[] = {{&S39912[13], 6}, {NULL}};
const Segment SGI259437[] = {{&S39912[35], 3}, {NULL}};
const Segment SGI259438[] = {{&S39912[66], 6}, {NULL}};
const Segment SGI259439[] = {{&S39912[119], 5}, {NULL}};
const Segment SGI259440[] = {{&S39912[132], 9}, {NULL}};
const Segment SGI259441[] = {{&S39913[13], 6}, {NULL}};
const Segment SGI259442[] = {{&S39913[35], 5}, {NULL}};
const Segment SGI259443[] = {{&S39913[68], 6}, {NULL}};
const Segment SGI259444[] = {{&S39913[121], 5}, {NULL}};
const Segment SGI259445[] = {{&S39913[134], 9}, {NULL}};
const Segment SGI259446[] = {{&S39914[13], 6}, {NULL}};
const Segment SGI259447[] = {{&S39914[35], 5}, {NULL}};
const Segment SGI259448[] = {{&S39914[68], 6}, {NULL}};
const Segment SGI259449[] = {{&S39914[121], 5}, {NULL}};
const Segment SGI259450[] = {{&S39914[134], 9}, {NULL}};
const Segment SGI259451[] = {{&S39924[13], 6}, {NULL}};
const Segment SGI259452[] = {{&S39924[35], 3}, {NULL}};
const Segment SGI259453[] = {{&S39924[66], 6}, {NULL}};
const Segment SGI259454[] = {{&S39924[100], 5}, {NULL}};
const Segment SGI259455[] = {{&S39924[106], 7}, {NULL}};
const Segment SGI259456[] = {{&S39925[13], 6}, {NULL}};
const Segment SGI259457[] = {{&S39925[35], 3}, {NULL}};
const Segment SGI259458[] = {{&S39925[66], 6}, {NULL}};
const Segment SGI259459[] = {{&S39925[100], 5}, {NULL}};
const Segment SGI259460[] = {{&S39925[106], 7}, {NULL}};
const Segment SGI259461[] = {{&S39926[13], 6}, {NULL}};
const Segment SGI259462[] = {{&S39926[35], 5}, {NULL}};
const Segment SGI259463[] = {{&S39926[68], 6}, {NULL}};
const Segment SGI259464[] = {{&S39926[102], 5}, {NULL}};
const Segment SGI259465[] = {{&S39926[108], 7}, {NULL}};
const Segment SGI259466[] = {{&S39927[13], 6}, {NULL}};
const Segment SGI259467[] = {{&S39927[35], 3}, {NULL}};
const Segment SGI259468[] = {{&S39927[66], 6}, {NULL}};
const Segment SGI259469[] = {{&S39927[100], 5}, {NULL}};
const Segment SGI259470[] = {{&S39927[106], 7}, {NULL}};
const Segment SGI259471[] = {{&S39928[13], 6}, {NULL}};
const Segment SGI259472[] = {{&S39928[35], 5}, {NULL}};
const Segment SGI259473[] = {{&S39928[68], 6}, {NULL}};
const Segment SGI259474[] = {{&S39928[102], 5}, {NULL}};
const Segment SGI259475[] = {{&S39928[108], 7}, {NULL}};
const Segment SGI259476[] = {{&S39929[13], 6}, {NULL}};
const Segment SGI259477[] = {{&S39929[35], 3}, {NULL}};
const Segment SGI259478[] = {{&S39929[66], 6}, {NULL}};
const Segment SGI259479[] = {{&S39929[100], 5}, {NULL}};
const Segment SGI259480[] = {{&S39929[106], 7}, {NULL}};
const Segment SGI259481[] = {{&S39967[13], 4}, {NULL}};
const Segment SGI259482[] = {{&S39967[107], 3}, {NULL}};
const Segment SGI259483[] = {{&S39967[39], 3}, {NULL}};
const Segment SGI259484[] = {{&S39967[70], 8}, {NULL}};
const Segment SGI259485[] = {{&S39967[118], 8}, {NULL}};
const Segment SGI259486[] = {{&S39967[134], 7}, {NULL}};
const Segment SGI259487[] = {{&S39968[13], 4}, {NULL}};
const Segment SGI259488[] = {{&S39968[107], 3}, {NULL}};
const Segment SGI259489[] = {{&S39968[39], 3}, {NULL}};
const Segment SGI259490[] = {{&S39968[70], 8}, {NULL}};
const Segment SGI259491[] = {{&S39968[118], 8}, {NULL}};
const Segment SGI259492[] = {{&S39968[134], 7}, {NULL}};
const Segment SGI259493[] = {{&S39969[13], 4}, {NULL}};
const Segment SGI259494[] = {{&S39969[107], 3}, {NULL}};
const Segment SGI259495[] = {{&S39969[39], 3}, {NULL}};
const Segment SGI259496[] = {{&S39969[70], 8}, {NULL}};
const Segment SGI259497[] = {{&S39969[118], 6}, {NULL}};
const Segment SGI259498[] = {{&S39969[132], 7}, {NULL}};
const Segment SGI259499[] = {{&S39970[13], 4}, {NULL}};
const Segment SGI259500[] = {{&S39970[109], 3}, {NULL}};
const Segment SGI259501[] = {{&S39970[39], 5}, {NULL}};
const Segment SGI259502[] = {{&S39970[72], 8}, {NULL}};
const Segment SGI259503[] = {{&S39970[120], 6}, {NULL}};
const Segment SGI259504[] = {{&S39970[134], 7}, {NULL}};
const Segment SGI259505[] = {{&S40056[13], 4}, {NULL}};
const Segment SGI259506[] = {{&S40056[110], 5}, {NULL}};
const Segment SGI259507[] = {{&S40056[36], 5}, {NULL}};
const Segment SGI259508[] = {{&S40056[76], 5}, {NULL}};
const Segment SGI259509[] = {{&S40056[123], 5}, {NULL}};
const Segment SGI259510[] = {{&S40056[136], 9}, {NULL}};
const Segment SGI259511[] = {{&S40057[13], 4}, {NULL}};
const Segment SGI259512[] = {{&S40057[108], 5}, {NULL}};
const Segment SGI259513[] = {{&S40057[36], 3}, {NULL}};
const Segment SGI259514[] = {{&S40057[74], 5}, {NULL}};
const Segment SGI259515[] = {{&S40057[121], 5}, {NULL}};
const Segment SGI259516[] = {{&S40057[134], 9}, {NULL}};
const Segment SGI259517[] = {{&S40058[13], 4}, {NULL}};
const Segment SGI259518[] = {{&S40058[106], 5}, {NULL}};
const Segment SGI259519[] = {{&S40058[36], 2}, {NULL}};
const Segment SGI259520[] = {{&S40058[73], 5}, {NULL}};
const Segment SGI259521[] = {{&S40058[119], 5}, {NULL}};
const Segment SGI259522[] = {{&S40058[132], 9}, {NULL}};
const Segment SGI259523[] = {{&S40059[13], 4}, {NULL}};
const Segment SGI259524[] = {{&S40059[107], 5}, {NULL}};
const Segment SGI259525[] = {{&S40059[36], 3}, {NULL}};
const Segment SGI259526[] = {{&S40059[74], 5}, {NULL}};
const Segment SGI259527[] = {{&S40059[120], 5}, {NULL}};
const Segment SGI259528[] = {{&S40059[133], 9}, {NULL}};
const Segment SGI259529[] = {{&S40066[13], 4}, {NULL}};
const Segment SGI259530[] = {{&S40066[33], 3}, {NULL}};
const Segment SGI259531[] = {{&S40066[71], 6}, {NULL}};
const Segment SGI259532[] = {{&S40066[125], 6}, {NULL}};
const Segment SGI259533[] = {{&S40066[139], 7}, {NULL}};
const Segment SGI259534[] = {{&S40067[13], 4}, {NULL}};
const Segment SGI259535[] = {{&S40067[33], 3}, {NULL}};
const Segment SGI259536[] = {{&S40067[71], 6}, {NULL}};
const Segment SGI259537[] = {{&S40067[125], 6}, {NULL}};
const Segment SGI259538[] = {{&S40067[139], 7}, {NULL}};
const Segment SGI259539[] = {{&S40068[13], 4}, {NULL}};
const Segment SGI259540[] = {{&S40068[33], 5}, {NULL}};
const Segment SGI259541[] = {{&S40068[73], 6}, {NULL}};
const Segment SGI259542[] = {{&S40068[127], 6}, {NULL}};
const Segment SGI259543[] = {{&S40068[141], 7}, {NULL}};
const Segment SGI259544[] = {{&S40069[13], 4}, {NULL}};
const Segment SGI259545[] = {{&S40069[33], 5}, {NULL}};
const Segment SGI259546[] = {{&S40069[73], 6}, {NULL}};
const Segment SGI259547[] = {{&S40069[127], 6}, {NULL}};
const Segment SGI259548[] = {{&S40069[141], 7}, {NULL}};
const Segment SGI259549[] = {{&S40093[13], 4}, {NULL}};
const Segment SGI259550[] = {{&S40093[39], 3}, {NULL}};
const Segment SGI259551[] = {{&S40093[70], 6}, {NULL}};
const Segment SGI259552[] = {{&S40093[104], 8}, {NULL}};
const Segment SGI259553[] = {{&S40094[13], 4}, {NULL}};
const Segment SGI259554[] = {{&S40094[39], 3}, {NULL}};
const Segment SGI259555[] = {{&S40094[70], 6}, {NULL}};
const Segment SGI259556[] = {{&S40095[13], 4}, {&S40095[13], 4}, {&S40095[13], 4}, {NULL}};
const Segment SGI259557[] = {{&S40095[39], 3}, {NULL}};
const Segment SGI259558[] = {{&S40095[70], 6}, {NULL}};
const Segment SGI259559[] = {{&S40095[104], 8}, {NULL}};
const Segment SGI259560[] = {{&S40096[13], 4}, {NULL}};
const Segment SGI259561[] = {{&S40096[39], 3}, {NULL}};
const Segment SGI259562[] = {{&S40096[70], 6}, {NULL}};
const Segment SGI259563[] = {{&S40096[104], 8}, {NULL}};
const Segment SGI259564[] = {{&S40097[13], 4}, {&S40097[13], 4}, {&S40097[13], 4}, {NULL}};
const Segment SGI259565[] = {{&S40097[39], 3}, {NULL}};
const Segment SGI259566[] = {{&S40097[70], 6}, {NULL}};
const Segment SGI259567[] = {{&S40097[104], 8}, {NULL}};
const Segment SGI259568[] = {{&S40098[13], 4}, {NULL}};
const Segment SGI259569[] = {{&S40098[39], 3}, {NULL}};
const Segment SGI259570[] = {{&S40098[70], 6}, {NULL}};
const Segment SGI259571[] = {{&S40098[104], 8}, {NULL}};
const Segment SGI259572[] = {{&S40099[13], 4}, {&S40099[13], 4}, {&S40099[13], 4}, {NULL}};
const Segment SGI259573[] = {{&S40099[39], 3}, {NULL}};
const Segment SGI259574[] = {{&S40099[70], 6}, {NULL}};
const Segment SGI259575[] = {{&S40099[104], 8}, {NULL}};
const Segment SGI259576[] = {{&S40100[13], 4}, {&S40100[13], 4}, {&S40100[13], 4}, {NULL}};
const Segment SGI259577[] = {{&S40100[39], 3}, {NULL}};
const Segment SGI259578[] = {{&S40100[70], 8}, {NULL}};
const Segment SGI259579[] = {{&S40100[106], 8}, {NULL}};
const Segment SGI259580[] = {{&S40101[13], 4}, {NULL}};
const Segment SGI259581[] = {{&S40101[39], 3}, {NULL}};
const Segment SGI259582[] = {{&S40101[70], 8}, {NULL}};
const Segment SGI259583[] = {{&S40101[106], 8}, {NULL}};
const Segment SGI259584[] = {{&S40102[13], 4}, {&S40102[13], 4}, {&S40102[13], 4}, {NULL}};
const Segment SGI259585[] = {{&S40102[39], 3}, {NULL}};
const Segment SGI259586[] = {{&S40102[70], 8}, {NULL}};
const Segment SGI259587[] = {{&S40102[106], 8}, {NULL}};
const Segment SGI259588[] = {{&S40103[13], 4}, {NULL}};
const Segment SGI259589[] = {{&S40103[39], 3}, {NULL}};
const Segment SGI259590[] = {{&S40103[70], 8}, {NULL}};
const Segment SGI259591[] = {{&S40103[106], 8}, {NULL}};
const Segment SGI259592[] = {{&S40104[13], 4}, {NULL}};
const Segment SGI259593[] = {{&S40104[39], 3}, {NULL}};
const Segment SGI259594[] = {{&S40104[70], 8}, {NULL}};
const Segment SGI259595[] = {{&S40104[106], 8}, {NULL}};
const Segment SGI259596[] = {{&S40105[13], 4}, {&S40105[13], 4}, {&S40105[13], 4}, {NULL}};
const Segment SGI259597[] = {{&S40105[39], 3}, {NULL}};
const Segment SGI259598[] = {{&S40105[70], 8}, {NULL}};
const Segment SGI259599[] = {{&S40105[106], 8}, {NULL}};
const Segment SGI259600[] = {{&S40106[13], 4}, {NULL}};
const Segment SGI259601[] = {{&S40106[39], 3}, {NULL}};
const Segment SGI259602[] = {{&S40106[70], 8}, {NULL}};
const Segment SGI259603[] = {{&S40106[106], 6}, {NULL}};
const Segment SGI259604[] = {{&S40107[13], 4}, {&S40107[13], 4}, {&S40107[13], 4}, {NULL}};
const Segment SGI259605[] = {{&S40107[39], 3}, {NULL}};
const Segment SGI259606[] = {{&S40107[70], 8}, {NULL}};
const Segment SGI259607[] = {{&S40107[106], 6}, {NULL}};
const Segment SGI259608[] = {{&S40108[13], 4}, {NULL}};
const Segment SGI259609[] = {{&S40108[39], 3}, {NULL}};
const Segment SGI259610[] = {{&S40108[70], 8}, {NULL}};
const Segment SGI259611[] = {{&S40109[13], 4}, {&S40109[13], 4}, {NULL}};
const Segment SGI259612[] = {{&S40109[39], 3}, {NULL}};
const Segment SGI259613[] = {{&S40109[70], 8}, {NULL}};
const Segment SGI259614[] = {{&S40109[106], 6}, {NULL}};
const Segment SGI259615[] = {{&S40110[13], 4}, {NULL}};
const Segment SGI259616[] = {{&S40110[39], 5}, {NULL}};
const Segment SGI259617[] = {{&S40110[72], 8}, {NULL}};
const Segment SGI259618[] = {{&S40110[108], 6}, {NULL}};
const Segment SGI259619[] = {{&S40111[13], 4}, {&S40111[13], 4}, {&S40111[13], 4}, {NULL}};
const Segment SGI259620[] = {{&S40111[39], 5}, {NULL}};
const Segment SGI259621[] = {{&S40111[72], 8}, {NULL}};
const Segment SGI259622[] = {{&S40111[108], 6}, {NULL}};
const Segment SGI259623[] = {{&S40249[13], 4}, {NULL}};
const Segment SGI259624[] = {{&S40249[33], 5}, {NULL}};
const Segment SGI259625[] = {{&S40249[66], 6}, {NULL}};
const Segment SGI259626[] = {{&S40249[120], 6}, {NULL}};
const Segment SGI259627[] = {{&S40249[134], 7}, {NULL}};
const Segment SGI259628[] = {{&S40250[13], 4}, {NULL}};
const Segment SGI259629[] = {{&S40250[33], 3}, {NULL}};
const Segment SGI259630[] = {{&S40250[64], 6}, {NULL}};
const Segment SGI259631[] = {{&S40250[118], 6}, {NULL}};
const Segment SGI259632[] = {{&S40250[132], 7}, {NULL}};
const Segment SGI259633[] = {{&S40251[13], 4}, {NULL}};
const Segment SGI259634[] = {{&S40251[33], 3}, {NULL}};
const Segment SGI259635[] = {{&S40251[64], 6}, {NULL}};
const Segment SGI259636[] = {{&S40251[118], 6}, {NULL}};
const Segment SGI259637[] = {{&S40251[132], 7}, {NULL}};
const Segment SGI259638[] = {{&S40252[13], 4}, {NULL}};
const Segment SGI259639[] = {{&S40252[33], 5}, {NULL}};
const Segment SGI259640[] = {{&S40252[66], 6}, {NULL}};
const Segment SGI259641[] = {{&S40252[120], 6}, {NULL}};
const Segment SGI259642[] = {{&S40252[134], 7}, {NULL}};
const Segment SGI259643[] = {{&S40267[13], 4}, {NULL}};
const Segment SGI259644[] = {{&S40267[26], 5}, {NULL}};
const Segment SGI259645[] = {{&S40267[59], 6}, {NULL}};
const Segment SGI259646[] = {{&S40267[113], 5}, {NULL}};
const Segment SGI259647[] = {{&S40267[126], 9}, {NULL}};
const Segment SGI259648[] = {{&S40268[13], 4}, {NULL}};
const Segment SGI259649[] = {{&S40268[26], 3}, {NULL}};
const Segment SGI259650[] = {{&S40268[57], 6}, {NULL}};
const Segment SGI259651[] = {{&S40268[111], 5}, {NULL}};
const Segment SGI259652[] = {{&S40268[124], 9}, {NULL}};
const Segment SGI259653[] = {{&S40358[13], 6}, {NULL}};
const Segment SGI259654[] = {{&S40358[35], 3}, {NULL}};
const Segment SGI259655[] = {{&S40358[66], 6}, {NULL}};
const Segment SGI259656[] = {{&S40358[100], 8}, {NULL}};
const Segment SGI259657[] = {{&S40358[109], 7}, {NULL}};
const Segment SGI259658[] = {{&S40359[13], 6}, {NULL}};
const Segment SGI259659[] = {{&S40359[35], 3}, {NULL}};
const Segment SGI259660[] = {{&S40359[66], 6}, {NULL}};
const Segment SGI259661[] = {{&S40359[100], 8}, {NULL}};
const Segment SGI259662[] = {{&S40359[109], 7}, {NULL}};
const Segment SGI259663[] = {{&S40360[13], 6}, {NULL}};
const Segment SGI259664[] = {{&S40360[35], 3}, {NULL}};
const Segment SGI259665[] = {{&S40360[66], 6}, {NULL}};
const Segment SGI259666[] = {{&S40360[100], 8}, {NULL}};
const Segment SGI259667[] = {{&S40360[109], 7}, {NULL}};
const Segment SGI259668[] = {{&S40361[13], 6}, {NULL}};
const Segment SGI259669[] = {{&S40361[35], 3}, {NULL}};
const Segment SGI259670[] = {{&S40361[66], 8}, {NULL}};
const Segment SGI259671[] = {{&S40361[102], 8}, {NULL}};
const Segment SGI259672[] = {{&S40361[111], 7}, {NULL}};
const Segment SGI259673[] = {{&S40362[13], 6}, {NULL}};
const Segment SGI259674[] = {{&S40362[35], 3}, {NULL}};
const Segment SGI259675[] = {{&S40362[66], 8}, {NULL}};
const Segment SGI259676[] = {{&S40362[102], 8}, {NULL}};
const Segment SGI259677[] = {{&S40362[111], 7}, {NULL}};
const Segment SGI259678[] = {{&S40363[13], 6}, {NULL}};
const Segment SGI259679[] = {{&S40363[35], 3}, {NULL}};
const Segment SGI259680[] = {{&S40363[66], 8}, {NULL}};
const Segment SGI259681[] = {{&S40363[102], 8}, {NULL}};
const Segment SGI259682[] = {{&S40363[111], 7}, {NULL}};
const Segment SGI259683[] = {{&S40364[13], 6}, {NULL}};
const Segment SGI259684[] = {{&S40364[35], 3}, {NULL}};
const Segment SGI259685[] = {{&S40364[66], 8}, {NULL}};
const Segment SGI259686[] = {{&S40364[102], 6}, {NULL}};
const Segment SGI259687[] = {{&S40364[109], 7}, {NULL}};
const Segment SGI259688[] = {{&S40365[13], 6}, {NULL}};
const Segment SGI259689[] = {{&S40365[35], 3}, {NULL}};
const Segment SGI259690[] = {{&S40365[66], 8}, {NULL}};
const Segment SGI259691[] = {{&S40365[102], 6}, {NULL}};
const Segment SGI259692[] = {{&S40365[109], 7}, {NULL}};
const Segment SGI259693[] = {{&S40366[13], 6}, {NULL}};
const Segment SGI259694[] = {{&S40366[35], 5}, {NULL}};
const Segment SGI259695[] = {{&S40366[68], 8}, {NULL}};
const Segment SGI259696[] = {{&S40366[104], 7}, {NULL}};
const Segment SGI259697[] = {{&S40366[112], 7}, {NULL}};
const Segment SGI259698[] = {{&S40367[13], 6}, {NULL}};
const Segment SGI259699[] = {{&S40367[35], 5}, {NULL}};
const Segment SGI259700[] = {{&S40367[68], 8}, {NULL}};
const Segment SGI259701[] = {{&S40367[104], 6}, {NULL}};
const Segment SGI259702[] = {{&S40367[111], 7}, {NULL}};
const Segment SGI259703[] = {{&S40388[13], 4}, {NULL}};
const Segment SGI259704[] = {{&S40388[111], 5}, {NULL}};
const Segment SGI259705[] = {{&S40388[36], 3}, {NULL}};
const Segment SGI259706[] = {{&S40388[74], 8}, {NULL}};
const Segment SGI259707[] = {{&S40388[124], 5}, {NULL}};
const Segment SGI259708[] = {{&S40388[137], 6}, {NULL}};
const Segment SGI259709[] = {{&S40389[13], 4}, {NULL}};
const Segment SGI259710[] = {{&S40389[113], 5}, {NULL}};
const Segment SGI259711[] = {{&S40389[36], 5}, {NULL}};
const Segment SGI259712[] = {{&S40389[76], 8}, {NULL}};
const Segment SGI259713[] = {{&S40389[126], 5}, {NULL}};
const Segment SGI259714[] = {{&S40389[139], 6}, {NULL}};
const Segment SGI259715[] = {{&S40390[13], 4}, {NULL}};
const Segment SGI259716[] = {{&S40390[113], 5}, {NULL}};
const Segment SGI259717[] = {{&S40390[36], 5}, {NULL}};
const Segment SGI259718[] = {{&S40390[76], 8}, {NULL}};
const Segment SGI259719[] = {{&S40390[126], 4}, {NULL}};
const Segment SGI259720[] = {{&S40390[138], 6}, {NULL}};
const Segment SGI259721[] = {{&S40391[13], 4}, {NULL}};
const Segment SGI259722[] = {{&S40391[111], 5}, {NULL}};
const Segment SGI259723[] = {{&S40391[36], 3}, {NULL}};
const Segment SGI259724[] = {{&S40391[74], 8}, {NULL}};
const Segment SGI259725[] = {{&S40391[124], 4}, {NULL}};
const Segment SGI259726[] = {{&S40391[136], 6}, {NULL}};
const Segment SGI259727[] = {{&S40392[13], 4}, {NULL}};
const Segment SGI259728[] = {{&S40392[110], 5}, {NULL}};
const Segment SGI259729[] = {{&S40392[36], 3}, {NULL}};
const Segment SGI259730[] = {{&S40392[74], 8}, {NULL}};
const Segment SGI259731[] = {{&S40392[123], 4}, {NULL}};
const Segment SGI259732[] = {{&S40392[135], 6}, {NULL}};
const Segment SGI259733[] = {{&S40393[13], 4}, {NULL}};
const Segment SGI259734[] = {{&S40393[110], 5}, {NULL}};
const Segment SGI259735[] = {{&S40393[36], 3}, {NULL}};
const Segment SGI259736[] = {{&S40393[74], 8}, {NULL}};
const Segment SGI259737[] = {{&S40393[123], 5}, {NULL}};
const Segment SGI259738[] = {{&S40393[136], 6}, {NULL}};
const Segment SGI259739[] = {{&S30086[0], 15}, {NULL}};
const Segment SGI259740[] = {{&S30087[0], 6}, {NULL}};
const Segment SGI259741[] = {{&S30087[0], 22}, {NULL}};
const Segment SGI259742[] = {{&S30088[0], 10}, {&S30088[26], 8}, {NULL}};
const Segment SGI259743[] = {{&S30088[0], 34}, {NULL}};
const Segment SGI259744[] = {{&S30089[0], 26}, {NULL}};
const Segment SGI259745[] = {{&S30090[7], 9}, {NULL}};
const Segment SGI259746[] = {{&S30090[0], 16}, {NULL}};
const Segment SGI259747[] = {{&S30091[8], 6}, {NULL}};
const Segment SGI259748[] = {{&S30091[0], 14}, {NULL}};
const Segment SGI259749[] = {{&S30092[10], 6}, {NULL}};
const Segment SGI259750[] = {{&S30092[0], 71}, {NULL}};
const Segment SGI259751[] = {{&S30093[8], 6}, {NULL}};
const Segment SGI259752[] = {{&S30093[0], 39}, {NULL}};
const Segment SGI259753[] = {{&S30094[5], 6}, {NULL}};
const Segment SGI259754[] = {{&S30094[0], 15}, {NULL}};
const Segment SGI259755[] = {{&S30095[0], 14}, {NULL}};
const Segment SGI259756[] = {{&S30096[8], 6}, {NULL}};
const Segment SGI259757[] = {{&S30096[0], 14}, {NULL}};
const Segment SGI259758[] = {{&S30098[0], 16}, {NULL}};
const Segment SGI259759[] = {{&S30099[0], 14}, {NULL}};
const Segment SGI259760[] = {{&S30100[4], 6}, {NULL}};
const Segment SGI259761[] = {{&S30100[0], 10}, {NULL}};
const Segment SGI259762[] = {{&S30101[25], 6}, {&S30101[39], 6}, {NULL}};
const Segment SGI259763[] = {{&S30101[0], 109}, {NULL}};
const Segment SGI259764[] = {{&S30102[11], 6}, {NULL}};
const Segment SGI259765[] = {{&S30102[0], 34}, {NULL}};
const Segment SGI259766[] = {{&S30103[11], 6}, {NULL}};
const Segment SGI259767[] = {{&S30103[0], 72}, {NULL}};
const Segment SGI259768[] = {{&S30104[13], 6}, {NULL}};
const Segment SGI259769[] = {{&S30104[0], 45}, {NULL}};
const Segment SGI259770[] = {{&S30105[10], 6}, {NULL}};
const Segment SGI259771[] = {{&S30105[0], 16}, {NULL}};
const Segment SGI259772[] = {{&S30106[0], 11}, {NULL}};
const Segment SGI259773[] = {{&S40491[13], 6}, {NULL}};
const Segment SGI259774[] = {{&S40491[35], 5}, {NULL}};
const Segment SGI259775[] = {{&S40491[75], 6}, {NULL}};
const Segment SGI259776[] = {{&S40491[129], 5}, {NULL}};
const Segment SGI259777[] = {{&S40491[142], 9}, {NULL}};
const Segment SGI259778[] = {{&S30215[115], 21}, {&S30215[137], 32}, {NULL}};
const Segment SGI259779[] = {{&S30215[0], 169}, {NULL}};
const Segment SGI259780[] = {{&S30216[115], 21}, {NULL}};
const Segment SGI259781[] = {{&S30216[0], 167}, {NULL}};
const Segment SGI259782[] = {{&S30217[115], 21}, {&S30217[138], 31}, {NULL}};
const Segment SGI259783[] = {{&S30217[0], 169}, {NULL}};
const Segment SGI259784[] = {{&S30245[32], 5}, {&S30245[43], 44}, {NULL}};
const Segment SGI259785[] = {{&S30245[0], 87}, {NULL}};
const Segment SGI259786[] = {{&S30246[32], 5}, {&S30246[39], 44}, {NULL}};
const Segment SGI259787[] = {{&S30246[0], 83}, {NULL}};
const Segment SGI259788[] = {{&S30247[32], 5}, {&S30247[37], 45}, {NULL}};
const Segment SGI259789[] = {{&S30247[0], 82}, {NULL}};
const Segment SGI259790[] = {{&S30248[32], 5}, {&S30248[44], 44}, {NULL}};
const Segment SGI259791[] = {{&S30248[0], 88}, {NULL}};
const Segment SGI259792[] = {{&S30249[32], 5}, {&S30249[48], 44}, {NULL}};
const Segment SGI259793[] = {{&S30249[0], 92}, {NULL}};
const Segment SGI259794[] = {{&S30250[35], 5}, {&S30250[42], 44}, {NULL}};
const Segment SGI259795[] = {{&S30250[0], 86}, {NULL}};
const Segment SGI259796[] = {{&S30251[32], 5}, {&S30251[44], 44}, {NULL}};
const Segment SGI259797[] = {{&S30251[0], 88}, {NULL}};
const Segment SGI259798[] = {{&S30252[12], 10}, {&S30252[23], 5}, {&S30252[42], 34}, {NULL}};
const Segment SGI259799[] = {{&S30252[0], 76}, {NULL}};
const Segment SGI259800[] = {{&S30253[12], 11}, {&S30253[24], 5}, {&S30253[43], 34}, {NULL}};
const Segment SGI259801[] = {{&S30253[0], 77}, {NULL}};
const Segment SGI259802[] = {{&S30254[38], 44}, {NULL}};
const Segment SGI259803[] = {{&S30254[0], 82}, {NULL}};
const Segment SGI259804[] = {{&S30255[12], 14}, {&S30255[27], 5}, {&S30255[46], 34}, {NULL}};
const Segment SGI259805[] = {{&S30255[0], 80}, {NULL}};
const Segment SGI259806[] = {{&S30256[12], 10}, {&S30256[23], 5}, {&S30256[42], 34}, {NULL}};
const Segment SGI259807[] = {{&S30256[0], 76}, {NULL}};
const Segment SGI259808[] = {{&S30257[12], 11}, {&S30257[24], 5}, {&S30257[43], 34}, {NULL}};
const Segment SGI259809[] = {{&S30257[0], 77}, {NULL}};
const Segment SGI259810[] = {{&S30258[32], 5}, {&S30258[43], 54}, {NULL}};
const Segment SGI259811[] = {{&S30258[0], 126}, {NULL}};
const Segment SGI259812[] = {{&S40492[13], 6}, {NULL}};
const Segment SGI259813[] = {{&S40492[35], 5}, {NULL}};
const Segment SGI259814[] = {{&S40492[75], 6}, {NULL}};
const Segment SGI259815[] = {{&S40492[129], 5}, {NULL}};
const Segment SGI259816[] = {{&S40492[142], 9}, {NULL}};
const Segment SGI259817[] = {{&S40493[13], 6}, {NULL}};
const Segment SGI259818[] = {{&S40493[35], 3}, {NULL}};
const Segment SGI259819[] = {{&S40493[73], 6}, {NULL}};
const Segment SGI259820[] = {{&S40493[127], 5}, {NULL}};
const Segment SGI259821[] = {{&S40493[140], 9}, {NULL}};
const Segment SGI259822[] = {{&S30457[46], 21}, {&S30457[68], 32}, {NULL}};
const Segment SGI259823[] = {{&S30457[0], 100}, {NULL}};
const Segment SGI259824[] = {{&S30458[46], 21}, {&S30458[69], 31}, {NULL}};
const Segment SGI259825[] = {{&S30458[0], 100}, {NULL}};
const Segment SGI259826[] = {{&S40565[13], 4}, {NULL}};
const Segment SGI259827[] = {{&S40565[33], 5}, {NULL}};
const Segment SGI259828[] = {{&S40565[73], 8}, {NULL}};
const Segment SGI259829[] = {{&S40565[128], 5}, {NULL}};
const Segment SGI259830[] = {{&S40565[141], 9}, {NULL}};
const Segment SGI259831[] = {{&S40566[13], 4}, {NULL}};
const Segment SGI259832[] = {{&S40566[33], 3}, {NULL}};
const Segment SGI259833[] = {{&S40566[71], 8}, {NULL}};
const Segment SGI259834[] = {{&S40566[126], 5}, {NULL}};
const Segment SGI259835[] = {{&S40566[139], 9}, {NULL}};
const Segment SGI259836[] = {{&S40567[13], 4}, {NULL}};
const Segment SGI259837[] = {{&S40567[33], 3}, {NULL}};
const Segment SGI259838[] = {{&S40567[71], 8}, {NULL}};
const Segment SGI259839[] = {{&S40567[125], 5}, {NULL}};
const Segment SGI259840[] = {{&S40567[138], 9}, {NULL}};
const Segment SGI259841[] = {{&S40568[13], 4}, {NULL}};
const Segment SGI259842[] = {{&S40568[33], 2}, {NULL}};
const Segment SGI259843[] = {{&S40568[70], 8}, {NULL}};
const Segment SGI259844[] = {{&S40568[124], 5}, {NULL}};
const Segment SGI259845[] = {{&S40568[137], 9}, {NULL}};
const Segment SGI259846[] = {{&S23675[10], 4}, {NULL}};
const Segment SGI259847[] = {{&S23675[15], 5}, {NULL}};
const Segment SGI259848[] = {{&S23972[10], 4}, {NULL}};
const Segment SGI259849[] = {{&S23972[15], 5}, {NULL}};
const Segment SGI259850[] = {{&S40950[13], 6}, {NULL}};
const Segment SGI259851[] = {{&S40950[38], 3}, {NULL}};
const Segment SGI259852[] = {{&S40950[76], 8}, {NULL}};
const Segment SGI259853[] = {{&S40950[112], 5}, {NULL}};
const Segment SGI259854[] = {{&S40950[118], 7}, {NULL}};
const Segment SGI259855[] = {{&S40951[13], 6}, {NULL}};
const Segment SGI259856[] = {{&S40951[38], 5}, {NULL}};
const Segment SGI259857[] = {{&S40951[78], 8}, {NULL}};
const Segment SGI259858[] = {{&S40951[114], 3}, {NULL}};
const Segment SGI259859[] = {{&S40951[118], 7}, {NULL}};
const Segment SGI259860[] = {{&S40952[13], 6}, {NULL}};
const Segment SGI259861[] = {{&S40952[38], 5}, {NULL}};
const Segment SGI259862[] = {{&S40952[78], 8}, {NULL}};
const Segment SGI259863[] = {{&S40952[114], 3}, {NULL}};
const Segment SGI259864[] = {{&S40952[118], 7}, {NULL}};
const Segment SGI259865[] = {{&S40953[13], 6}, {NULL}};
const Segment SGI259866[] = {{&S40953[38], 5}, {NULL}};
const Segment SGI259867[] = {{&S40953[78], 8}, {NULL}};
const Segment SGI259868[] = {{&S40953[114], 3}, {NULL}};
const Segment SGI259869[] = {{&S40953[118], 7}, {NULL}};
const Segment SGI259870[] = {{&S40954[13], 6}, {NULL}};
const Segment SGI259871[] = {{&S40954[38], 5}, {NULL}};
const Segment SGI259872[] = {{&S40954[78], 8}, {NULL}};
const Segment SGI259873[] = {{&S40954[114], 3}, {NULL}};
const Segment SGI259874[] = {{&S40954[118], 7}, {NULL}};
const Segment SGI259875[] = {{&S40955[13], 6}, {NULL}};
const Segment SGI259876[] = {{&S40955[38], 5}, {NULL}};
const Segment SGI259877[] = {{&S40955[78], 8}, {NULL}};
const Segment SGI259878[] = {{&S40955[114], 3}, {NULL}};
const Segment SGI259879[] = {{&S40955[118], 7}, {NULL}};
const Segment SGI259880[] = {{&S40956[13], 6}, {NULL}};
const Segment SGI259881[] = {{&S40956[38], 5}, {NULL}};
const Segment SGI259882[] = {{&S40956[78], 8}, {NULL}};
const Segment SGI259883[] = {{&S40956[114], 5}, {NULL}};
const Segment SGI259884[] = {{&S40956[120], 7}, {NULL}};
const Segment SGI259885[] = {{&S41088[17], 6}, {NULL}};
const Segment SGI259886[] = {{&S41088[38], 3}, {NULL}};
const Segment SGI259887[] = {{&S41088[66], 3}, {NULL}};
const Segment SGI259888[] = {{&S41089[17], 6}, {NULL}};
const Segment SGI259889[] = {{&S41089[38], 5}, {NULL}};
const Segment SGI259890[] = {{&S41089[68], 3}, {NULL}};
const Segment SGI259891[] = {{&S41090[17], 6}, {NULL}};
const Segment SGI259892[] = {{&S41090[38], 5}, {NULL}};
const Segment SGI259893[] = {{&S41090[68], 3}, {NULL}};
const Segment SGI259894[] = {{&S41091[17], 6}, {NULL}};
const Segment SGI259895[] = {{&S41091[38], 5}, {NULL}};
const Segment SGI259896[] = {{&S41091[68], 3}, {NULL}};
const Segment SGI259897[] = {{&S41092[17], 6}, {NULL}};
const Segment SGI259898[] = {{&S41092[38], 5}, {NULL}};
const Segment SGI259899[] = {{&S41092[68], 3}, {NULL}};
const Segment SGI259900[] = {{&S41093[13], 4}, {NULL}};
const Segment SGI259901[] = {{&S41093[113], 3}, {NULL}};
const Segment SGI259902[] = {{&S41093[26], 3}, {NULL}};
const Segment SGI259903[] = {{&S41093[57], 6}, {NULL}};
const Segment SGI259904[] = {{&S41093[91], 6}, {NULL}};
const Segment SGI259905[] = {{&S41094[13], 4}, {NULL}};
const Segment SGI259906[] = {{&S41094[106], 3}, {NULL}};
const Segment SGI259907[] = {{&S41094[26], 3}, {NULL}};
const Segment SGI259908[] = {{&S41094[57], 6}, {NULL}};
const Segment SGI259909[] = {{&S41094[91], 6}, {NULL}};
const Segment SGI259910[] = {{&S41094[117], 9}, {NULL}};
const Segment SGI259911[] = {{&S41095[13], 4}, {NULL}};
const Segment SGI259912[] = {{&S41095[92], 3}, {NULL}};
const Segment SGI259913[] = {{&S41095[26], 3}, {NULL}};
const Segment SGI259914[] = {{&S41095[57], 6}, {NULL}};
const Segment SGI259915[] = {{&S41095[103], 6}, {NULL}};
const Segment SGI259916[] = {{&S41095[117], 7}, {NULL}};
const Segment SGI259917[] = {{&S41096[13], 4}, {NULL}};
const Segment SGI259918[] = {{&S41096[103], 3}, {NULL}};
const Segment SGI259919[] = {{&S41096[26], 1}, {NULL}};
const Segment SGI259920[] = {{&S41096[55], 6}, {NULL}};
const Segment SGI259921[] = {{&S41096[89], 5}, {NULL}};
const Segment SGI259922[] = {{&S41096[114], 9}, {NULL}};
const Segment SGI259923[] = {{&S41097[13], 4}, {NULL}};
const Segment SGI259924[] = {{&S41097[94], 3}, {NULL}};
const Segment SGI259925[] = {{&S41097[26], 5}, {NULL}};
const Segment SGI259926[] = {{&S41097[59], 6}, {NULL}};
const Segment SGI259927[] = {{&S41097[105], 6}, {NULL}};
const Segment SGI259928[] = {{&S41097[119], 7}, {NULL}};
const Segment SGI259929[] = {{&S41098[13], 4}, {NULL}};
const Segment SGI259930[] = {{&S41098[115], 3}, {NULL}};
const Segment SGI259931[] = {{&S41098[26], 5}, {NULL}};
const Segment SGI259932[] = {{&S41098[59], 6}, {NULL}};
const Segment SGI259933[] = {{&S41098[93], 6}, {NULL}};
const Segment SGI259934[] = {{&S41099[13], 4}, {&S41099[13], 4}, {NULL}};
const Segment SGI259935[] = {{&S41099[117], 3}, {NULL}};
const Segment SGI259936[] = {{&S41099[29], 1}, {NULL}};
const Segment SGI259937[] = {{&S41099[82], 6}, {NULL}};
const Segment SGI259938[] = {{&S41099[128], 5}, {NULL}};
const Segment SGI259939[] = {{&S41099[141], 9}, {NULL}};
const Segment SGI259940[] = {{&S41100[13], 4}, {NULL}};
const Segment SGI259941[] = {{&S41100[108], 3}, {NULL}};
const Segment SGI259942[] = {{&S41100[26], 5}, {NULL}};
const Segment SGI259943[] = {{&S41100[59], 6}, {NULL}};
const Segment SGI259944[] = {{&S41100[93], 6}, {NULL}};
const Segment SGI259945[] = {{&S41100[119], 9}, {NULL}};
const Segment SGI259946[] = {{&S41101[13], 4}, {NULL}};
const Segment SGI259947[] = {{&S41101[108], 3}, {NULL}};
const Segment SGI259948[] = {{&S41101[26], 5}, {NULL}};
const Segment SGI259949[] = {{&S41101[59], 6}, {NULL}};
const Segment SGI259950[] = {{&S41101[93], 6}, {NULL}};
const Segment SGI259951[] = {{&S41101[119], 9}, {NULL}};
const Segment SGI259952[] = {{&S41102[13], 4}, {NULL}};
const Segment SGI259953[] = {{&S41102[116], 3}, {NULL}};
const Segment SGI259954[] = {{&S41102[26], 5}, {NULL}};
const Segment SGI259955[] = {{&S41102[59], 6}, {NULL}};
const Segment SGI259956[] = {{&S41102[93], 7}, {NULL}};
const Segment SGI259957[] = {{&S41103[13], 4}, {NULL}};
const Segment SGI259958[] = {{&S41103[94], 3}, {NULL}};
const Segment SGI259959[] = {{&S41103[26], 5}, {NULL}};
const Segment SGI259960[] = {{&S41103[59], 6}, {NULL}};
const Segment SGI259961[] = {{&S41103[105], 5}, {NULL}};
const Segment SGI259962[] = {{&S41103[118], 9}, {NULL}};
const Segment SGI259963[] = {{&S41104[13], 4}, {NULL}};
const Segment SGI259964[] = {{&S41104[92], 3}, {NULL}};
const Segment SGI259965[] = {{&S41104[26], 3}, {NULL}};
const Segment SGI259966[] = {{&S41104[57], 6}, {NULL}};
const Segment SGI259967[] = {{&S41104[103], 5}, {NULL}};
const Segment SGI259968[] = {{&S41104[116], 9}, {NULL}};
const Segment SGI259969[] = {{&S41105[13], 4}, {NULL}};
const Segment SGI259970[] = {{&S41105[94], 3}, {NULL}};
const Segment SGI259971[] = {{&S41105[26], 5}, {NULL}};
const Segment SGI259972[] = {{&S41105[59], 6}, {NULL}};
const Segment SGI259973[] = {{&S41105[105], 5}, {NULL}};
const Segment SGI259974[] = {{&S41105[118], 9}, {NULL}};
const Segment SGI259975[] = {{&S41106[13], 4}, {NULL}};
const Segment SGI259976[] = {{&S41106[116], 3}, {NULL}};
const Segment SGI259977[] = {{&S41106[26], 5}, {NULL}};
const Segment SGI259978[] = {{&S41106[59], 6}, {NULL}};
const Segment SGI259979[] = {{&S41106[93], 7}, {NULL}};
const Segment SGI259980[] = {{&S41107[13], 4}, {NULL}};
const Segment SGI259981[] = {{&S41107[115], 3}, {NULL}};
const Segment SGI259982[] = {{&S41107[26], 5}, {NULL}};
const Segment SGI259983[] = {{&S41107[59], 6}, {NULL}};
const Segment SGI259984[] = {{&S41107[93], 6}, {NULL}};
const Segment SGI259985[] = {{&S41108[13], 4}, {NULL}};
const Segment SGI259986[] = {{&S41108[92], 3}, {NULL}};
const Segment SGI259987[] = {{&S41108[26], 3}, {NULL}};
const Segment SGI259988[] = {{&S41108[57], 6}, {NULL}};
const Segment SGI259989[] = {{&S41108[103], 5}, {NULL}};
const Segment SGI259990[] = {{&S41108[116], 9}, {NULL}};
const Segment SGI259991[] = {{&S41109[13], 4}, {NULL}};
const Segment SGI259992[] = {{&S41109[92], 3}, {NULL}};
const Segment SGI259993[] = {{&S41109[26], 3}, {NULL}};
const Segment SGI259994[] = {{&S41109[57], 6}, {NULL}};
const Segment SGI259995[] = {{&S41109[103], 6}, {NULL}};
const Segment SGI259996[] = {{&S41109[117], 9}, {NULL}};
const Segment SGI259997[] = {{&S41110[13], 4}, {NULL}};
const Segment SGI259998[] = {{&S41110[92], 3}, {NULL}};
const Segment SGI259999[] = {{&S41110[26], 3}, {NULL}};
const Segment SGI260000[] = {{&S41110[57], 6}, {NULL}};
const Segment SGI260001[] = {{&S41110[103], 5}, {NULL}};
const Segment SGI260002[] = {{&S41110[116], 9}, {NULL}};
const Segment SGI260003[] = {{&S41111[13], 4}, {NULL}};
const Segment SGI260004[] = {{&S41111[94], 3}, {NULL}};
const Segment SGI260005[] = {{&S41111[26], 5}, {NULL}};
const Segment SGI260006[] = {{&S41111[59], 6}, {NULL}};
const Segment SGI260007[] = {{&S41111[105], 5}, {NULL}};
const Segment SGI260008[] = {{&S41111[118], 9}, {NULL}};
const Segment SGI260009[] = {{&S41112[13], 4}, {NULL}};
const Segment SGI260010[] = {{&S41112[113], 3}, {NULL}};
const Segment SGI260011[] = {{&S41112[26], 3}, {NULL}};
const Segment SGI260012[] = {{&S41112[57], 6}, {NULL}};
const Segment SGI260013[] = {{&S41112[91], 6}, {NULL}};
const Segment SGI260014[] = {{&S41207[13], 4}, {NULL}};
const Segment SGI260015[] = {{&S41207[92], 3}, {NULL}};
const Segment SGI260016[] = {{&S41207[26], 3}, {NULL}};
const Segment SGI260017[] = {{&S41207[57], 6}, {NULL}};
const Segment SGI260018[] = {{&S41207[103], 8}, {NULL}};
const Segment SGI260019[] = {{&S41207[119], 7}, {NULL}};
const Segment SGI260020[] = {{&S41208[13], 4}, {NULL}};
const Segment SGI260021[] = {{&S41208[92], 3}, {NULL}};
const Segment SGI260022[] = {{&S41208[26], 3}, {NULL}};
const Segment SGI260023[] = {{&S41208[57], 6}, {NULL}};
const Segment SGI260024[] = {{&S41208[103], 8}, {NULL}};
const Segment SGI260025[] = {{&S41208[119], 7}, {NULL}};
const Segment SGI260026[] = {{&S41209[13], 4}, {NULL}};
const Segment SGI260027[] = {{&S41209[92], 3}, {NULL}};
const Segment SGI260028[] = {{&S41209[26], 3}, {NULL}};
const Segment SGI260029[] = {{&S41209[57], 6}, {NULL}};
const Segment SGI260030[] = {{&S41209[103], 8}, {NULL}};
const Segment SGI260031[] = {{&S41209[119], 7}, {NULL}};
const Segment SGI260032[] = {{&S41210[13], 4}, {NULL}};
const Segment SGI260033[] = {{&S41210[94], 3}, {NULL}};
const Segment SGI260034[] = {{&S41210[26], 3}, {NULL}};
const Segment SGI260035[] = {{&S41210[57], 8}, {NULL}};
const Segment SGI260036[] = {{&S41210[105], 8}, {NULL}};
const Segment SGI260037[] = {{&S41210[121], 7}, {NULL}};
const Segment SGI260038[] = {{&S41211[13], 4}, {NULL}};
const Segment SGI260039[] = {{&S41211[94], 3}, {NULL}};
const Segment SGI260040[] = {{&S41211[26], 3}, {NULL}};
const Segment SGI260041[] = {{&S41211[57], 8}, {NULL}};
const Segment SGI260042[] = {{&S41211[105], 6}, {NULL}};
const Segment SGI260043[] = {{&S41211[119], 7}, {NULL}};
const Segment SGI260044[] = {{&S41212[13], 4}, {NULL}};
const Segment SGI260045[] = {{&S41212[92], 3}, {NULL}};
const Segment SGI260046[] = {{&S41212[26], 3}, {NULL}};
const Segment SGI260047[] = {{&S41212[57], 6}, {NULL}};
const Segment SGI260048[] = {{&S41212[103], 6}, {NULL}};
const Segment SGI260049[] = {{&S41212[117], 7}, {NULL}};
const Segment SGI260050[] = {{&S41213[13], 4}, {NULL}};
const Segment SGI260051[] = {{&S41213[92], 3}, {NULL}};
const Segment SGI260052[] = {{&S41213[26], 3}, {NULL}};
const Segment SGI260053[] = {{&S41213[57], 6}, {NULL}};
const Segment SGI260054[] = {{&S41213[103], 6}, {NULL}};
const Segment SGI260055[] = {{&S41213[117], 7}, {NULL}};
const Segment SGI260056[] = {{&S41214[13], 4}, {NULL}};
const Segment SGI260057[] = {{&S41214[96], 3}, {NULL}};
const Segment SGI260058[] = {{&S41214[26], 5}, {NULL}};
const Segment SGI260059[] = {{&S41214[59], 8}, {NULL}};
const Segment SGI260060[] = {{&S41214[107], 6}, {NULL}};
const Segment SGI260061[] = {{&S41214[121], 7}, {NULL}};
const Segment SGI260062[] = {{&S41215[13], 4}, {NULL}};
const Segment SGI260063[] = {{&S41215[94], 3}, {NULL}};
const Segment SGI260064[] = {{&S41215[26], 5}, {NULL}};
const Segment SGI260065[] = {{&S41215[59], 6}, {NULL}};
const Segment SGI260066[] = {{&S41215[105], 6}, {NULL}};
const Segment SGI260067[] = {{&S41215[119], 7}, {NULL}};
const Segment SGI260068[] = {{&S41310[13], 6}, {&S41310[13], 6}, {NULL}};
const Segment SGI260069[] = {{&S41310[35], 3}, {NULL}};
const Segment SGI260070[] = {{&S41310[66], 6}, {NULL}};
const Segment SGI260071[] = {{&S41310[100], 6}, {NULL}};
const Segment SGI260072[] = {{&S41310[114], 9}, {NULL}};
const Segment SGI260073[] = {{&S41311[13], 6}, {&S41311[13], 6}, {NULL}};
const Segment SGI260074[] = {{&S41311[35], 3}, {NULL}};
const Segment SGI260075[] = {{&S41311[66], 6}, {NULL}};
const Segment SGI260076[] = {{&S41311[100], 6}, {NULL}};
const Segment SGI260077[] = {{&S41311[114], 9}, {NULL}};
const Segment SGI260078[] = {{&S41312[13], 6}, {&S41312[13], 6}, {NULL}};
const Segment SGI260079[] = {{&S41312[35], 3}, {NULL}};
const Segment SGI260080[] = {{&S41312[66], 6}, {NULL}};
const Segment SGI260081[] = {{&S41312[100], 6}, {NULL}};
const Segment SGI260082[] = {{&S41312[114], 9}, {NULL}};
const Segment SGI260083[] = {{&S41313[13], 6}, {&S41313[13], 6}, {NULL}};
const Segment SGI260084[] = {{&S41313[35], 5}, {NULL}};
const Segment SGI260085[] = {{&S41313[68], 6}, {NULL}};
const Segment SGI260086[] = {{&S41313[102], 6}, {NULL}};
const Segment SGI260087[] = {{&S41313[116], 9}, {NULL}};
const Segment SGI260088[] = {{&S41314[13], 6}, {&S41314[13], 6}, {NULL}};
const Segment SGI260089[] = {{&S41314[35], 5}, {NULL}};
const Segment SGI260090[] = {{&S41314[68], 6}, {NULL}};
const Segment SGI260091[] = {{&S41314[102], 6}, {NULL}};
const Segment SGI260092[] = {{&S41314[116], 9}, {NULL}};
const Segment SGI260093[] = {{&S41315[13], 6}, {&S41315[13], 6}, {NULL}};
const Segment SGI260094[] = {{&S41315[35], 3}, {NULL}};
const Segment SGI260095[] = {{&S41315[66], 6}, {NULL}};
const Segment SGI260096[] = {{&S41315[100], 6}, {NULL}};
const Segment SGI260097[] = {{&S41315[114], 9}, {NULL}};
const Segment SGI260098[] = {{&S41316[13], 6}, {&S41316[13], 6}, {NULL}};
const Segment SGI260099[] = {{&S41316[35], 5}, {NULL}};
const Segment SGI260100[] = {{&S41316[68], 6}, {NULL}};
const Segment SGI260101[] = {{&S41316[102], 6}, {NULL}};
const Segment SGI260102[] = {{&S41316[116], 9}, {NULL}};
const Segment SGI260103[] = {{&S41317[13], 6}, {&S41317[13], 6}, {NULL}};
const Segment SGI260104[] = {{&S41317[35], 5}, {NULL}};
const Segment SGI260105[] = {{&S41317[68], 6}, {NULL}};
const Segment SGI260106[] = {{&S41317[102], 6}, {NULL}};
const Segment SGI260107[] = {{&S41317[116], 9}, {NULL}};
const Segment SGI260108[] = {{&S41336[13], 4}, {NULL}};
const Segment SGI260109[] = {{&S41336[36], 3}, {NULL}};
const Segment SGI260110[] = {{&S41336[74], 6}, {NULL}};
const Segment SGI260111[] = {{&S41336[108], 6}, {NULL}};
const Segment SGI260112[] = {{&S41337[13], 4}, {NULL}};
const Segment SGI260113[] = {{&S41337[36], 5}, {NULL}};
const Segment SGI260114[] = {{&S41337[76], 6}, {NULL}};
const Segment SGI260115[] = {{&S41338[13], 4}, {NULL}};
const Segment SGI260116[] = {{&S41338[36], 5}, {NULL}};
const Segment SGI260117[] = {{&S41338[76], 6}, {NULL}};
const Segment SGI260118[] = {{&S41339[13], 4}, {NULL}};
const Segment SGI260119[] = {{&S41339[36], 5}, {NULL}};
const Segment SGI260120[] = {{&S41339[76], 6}, {NULL}};
const Segment SGI260121[] = {{&S41340[13], 4}, {NULL}};
const Segment SGI260122[] = {{&S41340[36], 5}, {NULL}};
const Segment SGI260123[] = {{&S41340[76], 6}, {NULL}};
const Segment SGI260124[] = {{&S41340[110], 4}, {NULL}};
const Segment SGI260125[] = {{&S41341[13], 4}, {NULL}};
const Segment SGI260126[] = {{&S41341[36], 5}, {NULL}};
const Segment SGI260127[] = {{&S41341[76], 6}, {NULL}};
const Segment SGI260128[] = {{&S41341[110], 5}, {NULL}};
const Segment SGI260129[] = {{&S41342[13], 4}, {NULL}};
const Segment SGI260130[] = {{&S41342[36], 3}, {NULL}};
const Segment SGI260131[] = {{&S41342[74], 6}, {NULL}};
const Segment SGI260132[] = {{&S41342[108], 5}, {NULL}};
const Segment SGI260133[] = {{&S41343[13], 4}, {NULL}};
const Segment SGI260134[] = {{&S41343[36], 5}, {NULL}};
const Segment SGI260135[] = {{&S41343[76], 6}, {NULL}};
const Segment SGI260136[] = {{&S41343[110], 5}, {NULL}};
const Segment SGI260137[] = {{&S41344[13], 4}, {NULL}};
const Segment SGI260138[] = {{&S41344[36], 5}, {NULL}};
const Segment SGI260139[] = {{&S41344[76], 6}, {NULL}};
const Segment SGI260140[] = {{&S41344[110], 4}, {NULL}};
const Segment SGI260141[] = {{&S41345[13], 4}, {NULL}};
const Segment SGI260142[] = {{&S41345[36], 3}, {NULL}};
const Segment SGI260143[] = {{&S41345[74], 6}, {NULL}};
const Segment SGI260144[] = {{&S41346[13], 4}, {NULL}};
const Segment SGI260145[] = {{&S41346[36], 5}, {NULL}};
const Segment SGI260146[] = {{&S41346[76], 6}, {NULL}};
const Segment SGI260147[] = {{&S41346[110], 4}, {NULL}};
const Segment SGI260148[] = {{&S41347[13], 4}, {NULL}};
const Segment SGI260149[] = {{&S41347[36], 5}, {NULL}};
const Segment SGI260150[] = {{&S41347[76], 6}, {NULL}};
const Segment SGI260151[] = {{&S41347[110], 4}, {NULL}};
const Segment SGI260152[] = {{&S41436[13], 6}, {NULL}};
const Segment SGI260153[] = {{&S41436[35], 3}, {NULL}};
const Segment SGI260154[] = {{&S41436[66], 6}, {NULL}};
const Segment SGI260155[] = {{&S41436[119], 5}, {NULL}};
const Segment SGI260156[] = {{&S41436[132], 9}, {NULL}};
const Segment SGI260157[] = {{&S41437[13], 6}, {NULL}};
const Segment SGI260158[] = {{&S41437[35], 5}, {NULL}};
const Segment SGI260159[] = {{&S41437[68], 6}, {NULL}};
const Segment SGI260160[] = {{&S41437[121], 5}, {NULL}};
const Segment SGI260161[] = {{&S41437[134], 9}, {NULL}};
const Segment SGI260162[] = {{&S41438[13], 6}, {NULL}};
const Segment SGI260163[] = {{&S41438[35], 5}, {NULL}};
const Segment SGI260164[] = {{&S41438[68], 6}, {NULL}};
const Segment SGI260165[] = {{&S41438[121], 5}, {NULL}};
const Segment SGI260166[] = {{&S41438[134], 9}, {NULL}};
const Segment SGI260167[] = {{&S41439[13], 6}, {NULL}};
const Segment SGI260168[] = {{&S41439[35], 3}, {NULL}};
const Segment SGI260169[] = {{&S41439[66], 6}, {NULL}};
const Segment SGI260170[] = {{&S41439[119], 5}, {NULL}};
const Segment SGI260171[] = {{&S41439[132], 9}, {NULL}};
const Segment SGI260172[] = {{&S41623[13], 6}, {NULL}};
const Segment SGI260173[] = {{&S41623[35], 5}, {NULL}};
const Segment SGI260174[] = {{&S41623[68], 6}, {NULL}};
const Segment SGI260175[] = {{&S41623[122], 5}, {NULL}};
const Segment SGI260176[] = {{&S41623[135], 9}, {NULL}};
const Segment SGI260177[] = {{&S41624[13], 6}, {NULL}};
const Segment SGI260178[] = {{&S41624[35], 5}, {NULL}};
const Segment SGI260179[] = {{&S41624[68], 6}, {NULL}};
const Segment SGI260180[] = {{&S41624[122], 5}, {NULL}};
const Segment SGI260181[] = {{&S41624[135], 9}, {NULL}};
const Segment SGI260182[] = {{&S41625[13], 6}, {NULL}};
const Segment SGI260183[] = {{&S41625[35], 3}, {NULL}};
const Segment SGI260184[] = {{&S41625[66], 6}, {NULL}};
const Segment SGI260185[] = {{&S41625[120], 5}, {NULL}};
const Segment SGI260186[] = {{&S41625[133], 9}, {NULL}};
const Segment SGI260187[] = {{&S41626[13], 6}, {NULL}};
const Segment SGI260188[] = {{&S41626[35], 3}, {NULL}};
const Segment SGI260189[] = {{&S41626[66], 6}, {NULL}};
const Segment SGI260190[] = {{&S41626[120], 5}, {NULL}};
const Segment SGI260191[] = {{&S41626[133], 9}, {NULL}};
const Segment SGI260192[] = {{&S41761[13], 4}, {NULL}};
const Segment SGI260193[] = {{&S41761[26], 5}, {NULL}};
const Segment SGI260194[] = {{&S41761[59], 6}, {NULL}};
const Segment SGI260195[] = {{&S41761[105], 6}, {NULL}};
const Segment SGI260196[] = {{&S41761[119], 7}, {NULL}};
const Segment SGI260197[] = {{&S41762[13], 4}, {NULL}};
const Segment SGI260198[] = {{&S41762[26], 3}, {NULL}};
const Segment SGI260199[] = {{&S41762[57], 6}, {NULL}};
const Segment SGI260200[] = {{&S41762[103], 6}, {NULL}};
const Segment SGI260201[] = {{&S41762[117], 7}, {NULL}};
const Segment SGI260202[] = {{&S41839[13], 4}, {NULL}};
const Segment SGI260203[] = {{&S41839[26], 5}, {NULL}};
const Segment SGI260204[] = {{&S41839[66], 6}, {NULL}};
const Segment SGI260205[] = {{&S41839[120], 5}, {NULL}};
const Segment SGI260206[] = {{&S41839[133], 9}, {NULL}};
const Segment SGI260207[] = {{&S41840[13], 4}, {NULL}};
const Segment SGI260208[] = {{&S41840[26], 3}, {NULL}};
const Segment SGI260209[] = {{&S41840[64], 6}, {NULL}};
const Segment SGI260210[] = {{&S41840[118], 5}, {NULL}};
const Segment SGI260211[] = {{&S41840[131], 9}, {NULL}};
const Segment SGI260212[] = {{&S41918[13], 6}, {NULL}};
const Segment SGI260213[] = {{&S41918[35], 3}, {NULL}};
const Segment SGI260214[] = {{&S41918[66], 6}, {NULL}};
const Segment SGI260215[] = {{&S41918[119], 8}, {NULL}};
const Segment SGI260216[] = {{&S41918[135], 7}, {NULL}};
const Segment SGI260217[] = {{&S41919[13], 6}, {NULL}};
const Segment SGI260218[] = {{&S41919[35], 3}, {NULL}};
const Segment SGI260219[] = {{&S41919[66], 6}, {NULL}};
const Segment SGI260220[] = {{&S41919[119], 8}, {NULL}};
const Segment SGI260221[] = {{&S41919[135], 7}, {NULL}};
const Segment SGI260222[] = {{&S41920[13], 6}, {NULL}};
const Segment SGI260223[] = {{&S41920[35], 3}, {NULL}};
const Segment SGI260224[] = {{&S41920[66], 6}, {NULL}};
const Segment SGI260225[] = {{&S41920[119], 8}, {NULL}};
const Segment SGI260226[] = {{&S41920[135], 7}, {NULL}};
const Segment SGI260227[] = {{&S41921[13], 6}, {NULL}};
const Segment SGI260228[] = {{&S41921[35], 3}, {NULL}};
const Segment SGI260229[] = {{&S41921[66], 6}, {NULL}};
const Segment SGI260230[] = {{&S41921[119], 8}, {NULL}};
const Segment SGI260231[] = {{&S41921[135], 7}, {NULL}};
const Segment SGI260232[] = {{&S41922[13], 6}, {NULL}};
const Segment SGI260233[] = {{&S41922[35], 3}, {NULL}};
const Segment SGI260234[] = {{&S41922[66], 6}, {NULL}};
const Segment SGI260235[] = {{&S41922[119], 8}, {NULL}};
const Segment SGI260236[] = {{&S41922[135], 7}, {NULL}};
const Segment SGI260237[] = {{&S41923[13], 6}, {NULL}};
const Segment SGI260238[] = {{&S41923[35], 3}, {NULL}};
const Segment SGI260239[] = {{&S41923[66], 6}, {NULL}};
const Segment SGI260240[] = {{&S41923[119], 8}, {NULL}};
const Segment SGI260241[] = {{&S41923[135], 7}, {NULL}};
const Segment SGI260242[] = {{&S41924[13], 6}, {NULL}};
const Segment SGI260243[] = {{&S41924[35], 3}, {NULL}};
const Segment SGI260244[] = {{&S41924[66], 6}, {NULL}};
const Segment SGI260245[] = {{&S41924[119], 6}, {NULL}};
const Segment SGI260246[] = {{&S41924[133], 7}, {NULL}};
const Segment SGI260247[] = {{&S41925[13], 6}, {NULL}};
const Segment SGI260248[] = {{&S41925[35], 5}, {NULL}};
const Segment SGI260249[] = {{&S41925[68], 6}, {NULL}};
const Segment SGI260250[] = {{&S41925[121], 6}, {NULL}};
const Segment SGI260251[] = {{&S41925[135], 7}, {NULL}};
const Segment SGI260252[] = {{&S42148[13], 6}, {NULL}};
const Segment SGI260253[] = {{&S42148[38], 5}, {NULL}};
const Segment SGI260254[] = {{&S42148[78], 8}, {NULL}};
const Segment SGI260255[] = {{&S42148[133], 5}, {NULL}};
const Segment SGI260256[] = {{&S42148[146], 9}, {NULL}};
const Segment SGI260257[] = {{&S42149[13], 6}, {NULL}};
const Segment SGI260258[] = {{&S42149[38], 3}, {NULL}};
const Segment SGI260259[] = {{&S42149[76], 8}, {NULL}};
const Segment SGI260260[] = {{&S42149[131], 5}, {NULL}};
const Segment SGI260261[] = {{&S42149[144], 9}, {NULL}};
const Segment SGI260262[] = {{&S42150[13], 6}, {NULL}};
const Segment SGI260263[] = {{&S42150[38], 3}, {NULL}};
const Segment SGI260264[] = {{&S42150[76], 8}, {NULL}};
const Segment SGI260265[] = {{&S42150[131], 5}, {NULL}};
const Segment SGI260266[] = {{&S42150[144], 9}, {NULL}};
const Segment SGI260267[] = {{&S42151[13], 6}, {NULL}};
const Segment SGI260268[] = {{&S42151[38], 5}, {NULL}};
const Segment SGI260269[] = {{&S42151[78], 8}, {NULL}};
const Segment SGI260270[] = {{&S42151[133], 5}, {NULL}};
const Segment SGI260271[] = {{&S42151[146], 9}, {NULL}};
const Segment SGI260272[] = {{&S42152[13], 6}, {NULL}};
const Segment SGI260273[] = {{&S42152[38], 3}, {NULL}};
const Segment SGI260274[] = {{&S42152[76], 8}, {NULL}};
const Segment SGI260275[] = {{&S42152[131], 5}, {NULL}};
const Segment SGI260276[] = {{&S42152[144], 9}, {NULL}};
const Segment SGI260277[] = {{&S42199[13], 6}, {NULL}};
const Segment SGI260278[] = {{&S42199[38], 5}, {NULL}};
const Segment SGI260279[] = {{&S42199[78], 8}, {NULL}};
const Segment SGI260280[] = {{&S42199[133], 3}, {NULL}};
const Segment SGI260281[] = {{&S42199[144], 9}, {NULL}};
const Segment SGI260282[] = {{&S42200[13], 6}, {NULL}};
const Segment SGI260283[] = {{&S42200[38], 5}, {NULL}};
const Segment SGI260284[] = {{&S42200[78], 8}, {NULL}};
const Segment SGI260285[] = {{&S42200[133], 3}, {NULL}};
const Segment SGI260286[] = {{&S42200[144], 9}, {NULL}};
const Segment SGI260287[] = {{&S42201[13], 6}, {NULL}};
const Segment SGI260288[] = {{&S42201[38], 5}, {NULL}};
const Segment SGI260289[] = {{&S42201[78], 8}, {NULL}};
const Segment SGI260290[] = {{&S42201[133], 3}, {NULL}};
const Segment SGI260291[] = {{&S42201[144], 9}, {NULL}};
const Segment SGI260292[] = {{&S42202[13], 6}, {NULL}};
const Segment SGI260293[] = {{&S42202[38], 3}, {NULL}};
const Segment SGI260294[] = {{&S42202[76], 8}, {NULL}};
const Segment SGI260295[] = {{&S42202[131], 5}, {NULL}};
const Segment SGI260296[] = {{&S42202[144], 9}, {NULL}};
const Segment SGI260297[] = {{&S42203[13], 6}, {NULL}};
const Segment SGI260298[] = {{&S42203[38], 5}, {NULL}};
const Segment SGI260299[] = {{&S42203[78], 8}, {NULL}};
const Segment SGI260300[] = {{&S42203[133], 3}, {NULL}};
const Segment SGI260301[] = {{&S42203[144], 9}, {NULL}};
const Segment SGI260302[] = {{&S42204[13], 6}, {NULL}};
const Segment SGI260303[] = {{&S42204[38], 5}, {NULL}};
const Segment SGI260304[] = {{&S42204[78], 8}, {NULL}};
const Segment SGI260305[] = {{&S42204[133], 3}, {NULL}};
const Segment SGI260306[] = {{&S42204[144], 9}, {NULL}};
const Segment SGI260307[] = {{&S42205[13], 6}, {NULL}};
const Segment SGI260308[] = {{&S42205[38], 3}, {NULL}};
const Segment SGI260309[] = {{&S42205[76], 8}, {NULL}};
const Segment SGI260310[] = {{&S42205[130], 3}, {NULL}};
const Segment SGI260311[] = {{&S42205[141], 9}, {NULL}};
const Segment SGI260312[] = {{&S42296[13], 4}, {NULL}};
const Segment SGI260313[] = {{&S42296[33], 3}, {NULL}};
const Segment SGI260314[] = {{&S42296[64], 6}, {NULL}};
const Segment SGI260315[] = {{&S42296[117], 6}, {NULL}};
const Segment SGI260316[] = {{&S42296[131], 7}, {NULL}};
const Segment SGI260317[] = {{&S42297[13], 4}, {NULL}};
const Segment SGI260318[] = {{&S42297[33], 5}, {NULL}};
const Segment SGI260319[] = {{&S42297[66], 6}, {NULL}};
const Segment SGI260320[] = {{&S42297[119], 6}, {NULL}};
const Segment SGI260321[] = {{&S42297[133], 7}, {NULL}};
const Segment SGI260322[] = {{&S42298[13], 4}, {NULL}};
const Segment SGI260323[] = {{&S42298[33], 5}, {NULL}};
const Segment SGI260324[] = {{&S42298[66], 6}, {NULL}};
const Segment SGI260325[] = {{&S42298[119], 6}, {NULL}};
const Segment SGI260326[] = {{&S42298[133], 7}, {NULL}};
const Segment SGI260327[] = {{&S42299[13], 4}, {NULL}};
const Segment SGI260328[] = {{&S42299[33], 3}, {NULL}};
const Segment SGI260329[] = {{&S42299[64], 6}, {NULL}};
const Segment SGI260330[] = {{&S42299[117], 6}, {NULL}};
const Segment SGI260331[] = {{&S42299[131], 7}, {NULL}};
const Segment SGI260332[] = {{&S25352[10], 4}, {NULL}};
const Segment SGI260333[] = {{&S25352[15], 5}, {NULL}};
const Segment SGI260334[] = {{&S42542[13], 4}, {&S42542[13], 4}, {NULL}};
const Segment SGI260335[] = {{&S42542[29], 5}, {NULL}};
const Segment SGI260336[] = {{&S42542[69], 8}, {NULL}};
const Segment SGI260337[] = {{&S42542[105], 3}, {NULL}};
const Segment SGI260338[] = {{&S42542[109], 7}, {NULL}};
const Segment SGI260339[] = {{&S42543[13], 4}, {&S42543[13], 4}, {NULL}};
const Segment SGI260340[] = {{&S42543[29], 3}, {NULL}};
const Segment SGI260341[] = {{&S42543[67], 8}, {NULL}};
const Segment SGI260342[] = {{&S42543[103], 5}, {NULL}};
const Segment SGI260343[] = {{&S42543[109], 7}, {NULL}};
const Segment SGI260344[] = {{&S42544[13], 4}, {&S42544[13], 4}, {NULL}};
const Segment SGI260345[] = {{&S42544[29], 5}, {NULL}};
const Segment SGI260346[] = {{&S42544[69], 8}, {NULL}};
const Segment SGI260347[] = {{&S42544[105], 3}, {NULL}};
const Segment SGI260348[] = {{&S42544[109], 7}, {NULL}};
const Segment SGI260349[] = {{&S42545[13], 4}, {&S42545[13], 4}, {NULL}};
const Segment SGI260350[] = {{&S42545[29], 5}, {NULL}};
const Segment SGI260351[] = {{&S42545[69], 8}, {NULL}};
const Segment SGI260352[] = {{&S42545[105], 3}, {NULL}};
const Segment SGI260353[] = {{&S42545[109], 7}, {NULL}};
const Segment SGI260354[] = {{&S42546[13], 4}, {&S42546[13], 4}, {NULL}};
const Segment SGI260355[] = {{&S42546[29], 5}, {NULL}};
const Segment SGI260356[] = {{&S42546[69], 8}, {NULL}};
const Segment SGI260357[] = {{&S42546[105], 3}, {NULL}};
const Segment SGI260358[] = {{&S42546[109], 7}, {NULL}};
const Segment SGI260359[] = {{&S42547[13], 4}, {&S42547[13], 4}, {NULL}};
const Segment SGI260360[] = {{&S42547[29], 5}, {NULL}};
const Segment SGI260361[] = {{&S42547[69], 8}, {NULL}};
const Segment SGI260362[] = {{&S42547[105], 3}, {NULL}};
const Segment SGI260363[] = {{&S42547[109], 7}, {NULL}};
const Segment SGI260364[] = {{&S42548[13], 4}, {&S42548[13], 4}, {NULL}};
const Segment SGI260365[] = {{&S42548[29], 5}, {NULL}};
const Segment SGI260366[] = {{&S42548[69], 8}, {NULL}};
const Segment SGI260367[] = {{&S42548[105], 3}, {NULL}};
const Segment SGI260368[] = {{&S42548[109], 7}, {NULL}};
const Segment SGI260369[] = {{&S42549[13], 4}, {&S42549[13], 4}, {NULL}};
const Segment SGI260370[] = {{&S42549[29], 3}, {NULL}};
const Segment SGI260371[] = {{&S42549[67], 8}, {NULL}};
const Segment SGI260372[] = {{&S42549[103], 5}, {NULL}};
const Segment SGI260373[] = {{&S42549[109], 7}, {NULL}};
const Segment SGI260374[] = {{&S42550[13], 4}, {&S42550[13], 4}, {NULL}};
const Segment SGI260375[] = {{&S42550[29], 5}, {NULL}};
const Segment SGI260376[] = {{&S42550[69], 8}, {NULL}};
const Segment SGI260377[] = {{&S42550[105], 3}, {NULL}};
const Segment SGI260378[] = {{&S42550[109], 7}, {NULL}};
const Segment SGI260379[] = {{&S42629[13], 4}, {NULL}};
const Segment SGI260380[] = {{&S42629[26], 3}, {NULL}};
const Segment SGI260381[] = {{&S42629[57], 6}, {NULL}};
const Segment SGI260382[] = {{&S42629[110], 5}, {NULL}};
const Segment SGI260383[] = {{&S42629[123], 9}, {NULL}};
const Segment SGI260384[] = {{&S42630[13], 4}, {NULL}};
const Segment SGI260385[] = {{&S42630[26], 5}, {NULL}};
const Segment SGI260386[] = {{&S42630[59], 6}, {NULL}};
const Segment SGI260387[] = {{&S42630[112], 5}, {NULL}};
const Segment SGI260388[] = {{&S42630[125], 9}, {NULL}};
const Segment SGI260389[] = {{&S42631[13], 4}, {NULL}};
const Segment SGI260390[] = {{&S42631[26], 3}, {NULL}};
const Segment SGI260391[] = {{&S42631[57], 6}, {NULL}};
const Segment SGI260392[] = {{&S42631[110], 5}, {NULL}};
const Segment SGI260393[] = {{&S42631[123], 9}, {NULL}};
const Segment SGI260394[] = {{&S42674[13], 4}, {NULL}};
const Segment SGI260395[] = {{&S42674[36], 3}, {NULL}};
const Segment SGI260396[] = {{&S42674[74], 5}, {NULL}};
const Segment SGI260397[] = {{&S42675[13], 4}, {NULL}};
const Segment SGI260398[] = {{&S42675[36], 3}, {NULL}};
const Segment SGI260399[] = {{&S42675[74], 8}, {NULL}};
const Segment SGI260400[] = {{&S42676[13], 4}, {NULL}};
const Segment SGI260401[] = {{&S42676[36], 3}, {NULL}};
const Segment SGI260402[] = {{&S42676[74], 5}, {NULL}};
const Segment SGI260403[] = {{&S42676[107], 5}, {NULL}};
const Segment SGI260404[] = {{&S42677[13], 4}, {NULL}};
const Segment SGI260405[] = {{&S42677[36], 3}, {NULL}};
const Segment SGI260406[] = {{&S42677[74], 5}, {NULL}};
const Segment SGI260407[] = {{&S42677[107], 5}, {NULL}};
const Segment SGI260408[] = {{&S42678[13], 4}, {NULL}};
const Segment SGI260409[] = {{&S42678[36], 3}, {NULL}};
const Segment SGI260410[] = {{&S42678[74], 5}, {NULL}};
const Segment SGI260411[] = {{&S42678[107], 5}, {NULL}};
const Segment SGI260412[] = {{&S42679[13], 4}, {&S42679[13], 4}, {&S42679[13], 4}, {NULL}};
const Segment SGI260413[] = {{&S42679[42], 3}, {NULL}};
const Segment SGI260414[] = {{&S42680[13], 4}, {&S42680[13], 4}, {NULL}};
const Segment SGI260415[] = {{&S42680[36], 3}, {NULL}};
const Segment SGI260416[] = {{&S42680[74], 5}, {NULL}};
const Segment SGI260417[] = {{&S42680[107], 5}, {NULL}};
const Segment SGI260418[] = {{&S42681[13], 4}, {NULL}};
const Segment SGI260419[] = {{&S42681[36], 3}, {NULL}};
const Segment SGI260420[] = {{&S42681[74], 5}, {NULL}};
const Segment SGI260421[] = {{&S42681[107], 5}, {NULL}};
const Segment SGI260422[] = {{&S42682[13], 4}, {&S42682[13], 4}, {NULL}};
const Segment SGI260423[] = {{&S42682[36], 3}, {NULL}};
const Segment SGI260424[] = {{&S42682[74], 5}, {NULL}};
const Segment SGI260425[] = {{&S42682[107], 5}, {NULL}};
const Segment SGI260426[] = {{&S42683[13], 4}, {NULL}};
const Segment SGI260427[] = {{&S42683[36], 2}, {NULL}};
const Segment SGI260428[] = {{&S42683[73], 5}, {NULL}};
const Segment SGI260429[] = {{&S42683[105], 5}, {NULL}};
const Segment SGI260430[] = {{&S26013[10], 4}, {NULL}};
const Segment SGI260431[] = {{&S26013[15], 5}, {NULL}};
const Segment SGI260432[] = {{&S42836[13], 6}, {NULL}};
const Segment SGI260433[] = {{&S42836[38], 5}, {NULL}};
const Segment SGI260434[] = {{&S42836[78], 8}, {NULL}};
const Segment SGI260435[] = {{&S42836[134], 3}, {NULL}};
const Segment SGI260436[] = {{&S42836[145], 9}, {NULL}};
const Segment SGI260437[] = {{&S42837[13], 6}, {NULL}};
const Segment SGI260438[] = {{&S42837[38], 5}, {NULL}};
const Segment SGI260439[] = {{&S42837[78], 8}, {NULL}};
const Segment SGI260440[] = {{&S42837[134], 3}, {NULL}};
const Segment SGI260441[] = {{&S42837[145], 9}, {NULL}};
const Segment SGI260442[] = {{&S42838[13], 6}, {NULL}};
const Segment SGI260443[] = {{&S42838[38], 5}, {NULL}};
const Segment SGI260444[] = {{&S42838[78], 8}, {NULL}};
const Segment SGI260445[] = {{&S42838[134], 3}, {NULL}};
const Segment SGI260446[] = {{&S42838[145], 9}, {NULL}};
const Segment SGI260447[] = {{&S42839[13], 6}, {NULL}};
const Segment SGI260448[] = {{&S42839[38], 3}, {NULL}};
const Segment SGI260449[] = {{&S42839[76], 8}, {NULL}};
const Segment SGI260450[] = {{&S42839[132], 5}, {NULL}};
const Segment SGI260451[] = {{&S42839[145], 9}, {NULL}};
const Segment SGI260452[] = {{&S42840[13], 6}, {NULL}};
const Segment SGI260453[] = {{&S42840[38], 5}, {NULL}};
const Segment SGI260454[] = {{&S42840[78], 8}, {NULL}};
const Segment SGI260455[] = {{&S42840[134], 3}, {NULL}};
const Segment SGI260456[] = {{&S42840[145], 9}, {NULL}};
const Segment SGI260457[] = {{&S42841[13], 6}, {NULL}};
const Segment SGI260458[] = {{&S42841[38], 5}, {NULL}};
const Segment SGI260459[] = {{&S42841[78], 8}, {NULL}};
const Segment SGI260460[] = {{&S42841[134], 3}, {NULL}};
const Segment SGI260461[] = {{&S42841[145], 9}, {NULL}};
const Segment SGI260462[] = {{&S42842[13], 6}, {NULL}};
const Segment SGI260463[] = {{&S42842[38], 3}, {NULL}};
const Segment SGI260464[] = {{&S42842[76], 8}, {NULL}};
const Segment SGI260465[] = {{&S42842[130], 3}, {NULL}};
const Segment SGI260466[] = {{&S42842[141], 9}, {NULL}};
const Segment SGI260467[] = {{&S26207[10], 4}, {NULL}};
const Segment SGI260468[] = {{&S26207[15], 5}, {NULL}};
const Segment SGI260469[] = {{&S42990[13], 6}, {NULL}};
const Segment SGI260470[] = {{&S42990[35], 5}, {NULL}};
const Segment SGI260471[] = {{&S42990[68], 6}, {NULL}};
const Segment SGI260472[] = {{&S42990[114], 6}, {NULL}};
const Segment SGI260473[] = {{&S42990[128], 7}, {NULL}};
const Segment SGI260474[] = {{&S42991[13], 6}, {NULL}};
const Segment SGI260475[] = {{&S42991[35], 3}, {NULL}};
const Segment SGI260476[] = {{&S42991[66], 6}, {NULL}};
const Segment SGI260477[] = {{&S42991[112], 6}, {NULL}};
const Segment SGI260478[] = {{&S42991[126], 7}, {NULL}};
const Segment SGI260479[] = {{&S26649[89], 23}, {NULL}};
const Segment SGI260480[] = {{&S26650[89], 23}, {NULL}};
const Segment SGI260481[] = {{&S26651[89], 23}, {NULL}};
const Segment SGI260482[] = {{&S26654[89], 23}, {NULL}};
const Segment SGI260483[] = {{&S33883[32], 5}, {&S33883[43], 44}, {NULL}};
const Segment SGI260484[] = {{&S33883[0], 103}, {NULL}};
const Segment SGI260485[] = {{&S33884[32], 5}, {&S33884[43], 44}, {NULL}};
const Segment SGI260486[] = {{&S33884[0], 142}, {NULL}};
const Segment SGI260487[] = {{&S43651[12], 4}, {&S43651[12], 4}, {NULL}};
const Segment SGI260488[] = {{&S43651[27], 3}, {NULL}};
const Segment SGI260489[] = {{&S43651[74], 6}, {NULL}};
const Segment SGI260490[] = {{&S43651[137], 9}, {NULL}};
const Segment SGI260491[] = {{&S43651[101], 13}, {NULL}};
const Segment SGI260492[] = {{&S43741[13], 4}, {NULL}};
const Segment SGI260493[] = {{&S43741[29], 3}, {NULL}};
const Segment SGI260494[] = {{&S43741[67], 8}, {NULL}};
const Segment SGI260495[] = {{&S43741[122], 8}, {NULL}};
const Segment SGI260496[] = {{&S43741[138], 9}, {NULL}};
const Segment SGI260497[] = {{&S43742[13], 4}, {NULL}};
const Segment SGI260498[] = {{&S43742[29], 3}, {NULL}};
const Segment SGI260499[] = {{&S43742[67], 8}, {NULL}};
const Segment SGI260500[] = {{&S43742[122], 6}, {NULL}};
const Segment SGI260501[] = {{&S43742[136], 9}, {NULL}};
const Segment SGI260502[] = {{&S43743[13], 4}, {NULL}};
const Segment SGI260503[] = {{&S43743[29], 5}, {NULL}};
const Segment SGI260504[] = {{&S43743[69], 8}, {NULL}};
const Segment SGI260505[] = {{&S43743[124], 6}, {NULL}};
const Segment SGI260506[] = {{&S43743[138], 9}, {NULL}};
const Segment SGI260507[] = {{&S43744[13], 4}, {NULL}};
const Segment SGI260508[] = {{&S43744[111], 3}, {NULL}};
const Segment SGI260509[] = {{&S43744[26], 3}, {NULL}};
const Segment SGI260510[] = {{&S43744[57], 6}, {NULL}};
const Segment SGI260511[] = {{&S43744[122], 6}, {NULL}};
const Segment SGI260512[] = {{&S43744[136], 7}, {NULL}};
const Segment SGI260513[] = {{&S43745[13], 4}, {NULL}};
const Segment SGI260514[] = {{&S43745[29], 5}, {NULL}};
const Segment SGI260515[] = {{&S43745[69], 8}, {NULL}};
const Segment SGI260516[] = {{&S43745[124], 7}, {NULL}};
const Segment SGI260517[] = {{&S43745[139], 9}, {NULL}};
const Segment SGI260518[] = {{&S43746[13], 4}, {NULL}};
const Segment SGI260519[] = {{&S43746[29], 5}, {NULL}};
const Segment SGI260520[] = {{&S43746[69], 8}, {NULL}};
const Segment SGI260521[] = {{&S43746[124], 6}, {NULL}};
const Segment SGI260522[] = {{&S43746[138], 9}, {NULL}};
const Segment SGI260523[] = {{&S27133[89], 23}, {NULL}};
const Segment SGI260524[] = {{&S27134[89], 23}, {NULL}};
const Segment SGI260525[] = {{&S27135[89], 23}, {NULL}};
const Segment SGI260526[] = {{&S27137[89], 23}, {NULL}};
const Segment SGI260527[] = {{&S27160[10], 4}, {NULL}};
const Segment SGI260528[] = {{&S27160[15], 5}, {NULL}};
const Segment SGI260529[] = {{&S44269[13], 6}, {NULL}};
const Segment SGI260530[] = {{&S44269[35], 5}, {NULL}};
const Segment SGI260531[] = {{&S44269[68], 6}, {NULL}};
const Segment SGI260532[] = {{&S44269[121], 5}, {NULL}};
const Segment SGI260533[] = {{&S44269[134], 9}, {NULL}};
const Segment SGI260534[] = {{&S44270[13], 6}, {NULL}};
const Segment SGI260535[] = {{&S44270[35], 5}, {NULL}};
const Segment SGI260536[] = {{&S44270[68], 6}, {NULL}};
const Segment SGI260537[] = {{&S44270[121], 5}, {NULL}};
const Segment SGI260538[] = {{&S44270[134], 9}, {NULL}};
const Segment SGI260539[] = {{&S44271[13], 6}, {NULL}};
const Segment SGI260540[] = {{&S44271[35], 3}, {NULL}};
const Segment SGI260541[] = {{&S44271[66], 6}, {NULL}};
const Segment SGI260542[] = {{&S44271[119], 5}, {NULL}};
const Segment SGI260543[] = {{&S44271[132], 9}, {NULL}};
const Segment SGI260544[] = {{&S44272[13], 6}, {NULL}};
const Segment SGI260545[] = {{&S44272[35], 3}, {NULL}};
const Segment SGI260546[] = {{&S44272[66], 6}, {NULL}};
const Segment SGI260547[] = {{&S44272[119], 5}, {NULL}};
const Segment SGI260548[] = {{&S44272[132], 9}, {NULL}};
const Segment SGI260549[] = {{&S44375[13], 4}, {NULL}};
const Segment SGI260550[] = {{&S44375[33], 3}, {NULL}};
const Segment SGI260551[] = {{&S44375[64], 6}, {NULL}};
const Segment SGI260552[] = {{&S44375[98], 6}, {NULL}};
const Segment SGI260553[] = {{&S44375[105], 7}, {NULL}};
const Segment SGI260554[] = {{&S44376[13], 4}, {NULL}};
const Segment SGI260555[] = {{&S44376[33], 3}, {NULL}};
const Segment SGI260556[] = {{&S44376[64], 6}, {NULL}};
const Segment SGI260557[] = {{&S44376[98], 6}, {NULL}};
const Segment SGI260558[] = {{&S44376[105], 7}, {NULL}};
const Segment SGI260559[] = {{&S44377[13], 4}, {NULL}};
const Segment SGI260560[] = {{&S44377[33], 5}, {NULL}};
const Segment SGI260561[] = {{&S44377[66], 6}, {NULL}};
const Segment SGI260562[] = {{&S44377[100], 6}, {NULL}};
const Segment SGI260563[] = {{&S44377[107], 7}, {NULL}};
const Segment SGI260564[] = {{&S44378[13], 4}, {NULL}};
const Segment SGI260565[] = {{&S44378[33], 5}, {NULL}};
const Segment SGI260566[] = {{&S44378[66], 6}, {NULL}};
const Segment SGI260567[] = {{&S44378[100], 6}, {NULL}};
const Segment SGI260568[] = {{&S44378[107], 7}, {NULL}};
const Segment SGI260569[] = {{&S34574[21], 7}, {&S34574[34], 29}, {NULL}};
const Segment SGI260570[] = {{&S34574[0], 63}, {NULL}};
const Segment SGI260571[] = {{&S27801[10], 4}, {NULL}};
const Segment SGI260572[] = {{&S27801[15], 5}, {NULL}};
const Segment SGI260573[] = {{&S34606[0], 16}, {&S34606[22], 33}, {NULL}};
const Segment SGI260574[] = {{&S34606[0], 55}, {NULL}};
const Segment SGI260575[] = {{&S44432[13], 6}, {NULL}};
const Segment SGI260576[] = {{&S44432[35], 3}, {NULL}};
const Segment SGI260577[] = {{&S44432[66], 6}, {NULL}};
const Segment SGI260578[] = {{&S44432[118], 8}, {NULL}};
const Segment SGI260579[] = {{&S44432[134], 7}, {NULL}};
const Segment SGI260580[] = {{&S44433[13], 6}, {NULL}};
const Segment SGI260581[] = {{&S44433[35], 3}, {NULL}};
const Segment SGI260582[] = {{&S44433[66], 6}, {NULL}};
const Segment SGI260583[] = {{&S44433[118], 8}, {NULL}};
const Segment SGI260584[] = {{&S44433[134], 7}, {NULL}};
const Segment SGI260585[] = {{&S44434[13], 6}, {NULL}};
const Segment SGI260586[] = {{&S44434[35], 3}, {NULL}};
const Segment SGI260587[] = {{&S44434[66], 6}, {NULL}};
const Segment SGI260588[] = {{&S44434[119], 6}, {NULL}};
const Segment SGI260589[] = {{&S44434[133], 7}, {NULL}};
const Segment SGI260590[] = {{&S44435[13], 6}, {NULL}};
const Segment SGI260591[] = {{&S44435[35], 3}, {NULL}};
const Segment SGI260592[] = {{&S44435[66], 6}, {NULL}};
const Segment SGI260593[] = {{&S44435[119], 8}, {NULL}};
const Segment SGI260594[] = {{&S44435[135], 7}, {NULL}};
const Segment SGI260595[] = {{&S44436[13], 6}, {NULL}};
const Segment SGI260596[] = {{&S44436[35], 3}, {NULL}};
const Segment SGI260597[] = {{&S44436[66], 6}, {NULL}};
const Segment SGI260598[] = {{&S44436[119], 8}, {NULL}};
const Segment SGI260599[] = {{&S44436[135], 7}, {NULL}};
const Segment SGI260600[] = {{&S44437[13], 6}, {NULL}};
const Segment SGI260601[] = {{&S44437[35], 3}, {NULL}};
const Segment SGI260602[] = {{&S44437[66], 6}, {NULL}};
const Segment SGI260603[] = {{&S44437[119], 6}, {NULL}};
const Segment SGI260604[] = {{&S44437[133], 7}, {NULL}};
const Segment SGI260605[] = {{&S44438[13], 6}, {NULL}};
const Segment SGI260606[] = {{&S44438[35], 5}, {NULL}};
const Segment SGI260607[] = {{&S44438[68], 6}, {NULL}};
const Segment SGI260608[] = {{&S44438[121], 6}, {NULL}};
const Segment SGI260609[] = {{&S44438[135], 7}, {NULL}};
const Segment SGI260610[] = {{&S44439[13], 6}, {NULL}};
const Segment SGI260611[] = {{&S44439[35], 5}, {NULL}};
const Segment SGI260612[] = {{&S44439[68], 6}, {NULL}};
const Segment SGI260613[] = {{&S44439[121], 7}, {NULL}};
const Segment SGI260614[] = {{&S44439[136], 7}, {NULL}};
const Segment SGI260615[] = {{&S34644[15], 31}, {NULL}};
const Segment SGI260616[] = {{&S34644[0], 46}, {NULL}};
const Segment SGI260617[] = {{&S44636[20], 6}, {NULL}};
const Segment SGI260618[] = {{&S44636[38], 3}, {NULL}};
const Segment SGI260619[] = {{&S44637[20], 6}, {NULL}};
const Segment SGI260620[] = {{&S44637[38], 5}, {NULL}};
const Segment SGI260621[] = {{&S44638[29], 6}, {NULL}};
const Segment SGI260622[] = {{&S44639[20], 6}, {NULL}};
const Segment SGI260623[] = {{&S44639[38], 5}, {NULL}};
const Segment SGI260624[] = {{&S44640[20], 6}, {NULL}};
const Segment SGI260625[] = {{&S44640[38], 5}, {NULL}};
const Segment SGI260626[] = {{&S44641[15], 6}, {NULL}};
const Segment SGI260627[] = {{&S44641[33], 5}, {NULL}};
const Segment SGI260628[] = {{&S44641[0], 7}, {NULL}};
const Segment SGI260629[] = {{&S44646[12], 6}, {NULL}};
const Segment SGI260630[] = {{&S44646[82], 5}, {NULL}};
const Segment SGI260631[] = {{&S44647[12], 6}, {NULL}};
const Segment SGI260632[] = {{&S44647[76], 5}, {NULL}};
const Segment SGI260633[] = {{&S28311[89], 23}, {NULL}};
const Segment SGI260634[] = {{&S28340[89], 23}, {NULL}};
const Segment SGI260635[] = {{&S28344[89], 23}, {NULL}};
const Segment SGI260636[] = {{&S28345[89], 23}, {NULL}};
const Segment SGI260637[] = {{&S28347[10], 4}, {NULL}};
const Segment SGI260638[] = {{&S28347[15], 5}, {NULL}};
const Segment SGI260639[] = {{&S45152[13], 4}, {NULL}};
const Segment SGI260640[] = {{&S45152[33], 5}, {NULL}};
const Segment SGI260641[] = {{&S45152[73], 6}, {NULL}};
const Segment SGI260642[] = {{&S45152[127], 5}, {NULL}};
const Segment SGI260643[] = {{&S45152[140], 9}, {NULL}};
const Segment SGI260644[] = {{&S45153[13], 4}, {NULL}};
const Segment SGI260645[] = {{&S45153[33], 3}, {NULL}};
const Segment SGI260646[] = {{&S45153[71], 6}, {NULL}};
const Segment SGI260647[] = {{&S45153[125], 5}, {NULL}};
const Segment SGI260648[] = {{&S45153[138], 9}, {NULL}};
const Segment SGI260649[] = {{&S45154[13], 4}, {NULL}};
const Segment SGI260650[] = {{&S45154[33], 3}, {NULL}};
const Segment SGI260651[] = {{&S45154[71], 6}, {NULL}};
const Segment SGI260652[] = {{&S45154[123], 5}, {NULL}};
const Segment SGI260653[] = {{&S45154[136], 9}, {NULL}};
const Segment SGI260654[] = {{&S45168[13], 6}, {NULL}};
const Segment SGI260655[] = {{&S45168[115], 5}, {NULL}};
const Segment SGI260656[] = {{&S45168[38], 5}, {NULL}};
const Segment SGI260657[] = {{&S45168[78], 8}, {NULL}};
const Segment SGI260658[] = {{&S45168[128], 4}, {NULL}};
const Segment SGI260659[] = {{&S45168[140], 6}, {NULL}};
const Segment SGI260660[] = {{&S45169[13], 6}, {NULL}};
const Segment SGI260661[] = {{&S45169[113], 5}, {NULL}};
const Segment SGI260662[] = {{&S45169[38], 3}, {NULL}};
const Segment SGI260663[] = {{&S45169[76], 8}, {NULL}};
const Segment SGI260664[] = {{&S45169[126], 4}, {NULL}};
const Segment SGI260665[] = {{&S45169[138], 6}, {NULL}};
const Segment SGI260666[] = {{&S45170[13], 6}, {NULL}};
const Segment SGI260667[] = {{&S45170[112], 5}, {NULL}};
const Segment SGI260668[] = {{&S45170[38], 3}, {NULL}};
const Segment SGI260669[] = {{&S45170[76], 7}, {NULL}};
const Segment SGI260670[] = {{&S45170[125], 4}, {NULL}};
const Segment SGI260671[] = {{&S45170[137], 6}, {NULL}};
const Segment SGI260672[] = {{&S45407[29], 6}, {NULL}};
const Segment SGI260673[] = {{&S45407[47], 3}, {NULL}};
const Segment SGI260674[] = {{&S45408[29], 6}, {NULL}};
const Segment SGI260675[] = {{&S45408[47], 5}, {NULL}};
const Segment SGI260676[] = {{&S45409[29], 6}, {NULL}};
const Segment SGI260677[] = {{&S45409[47], 5}, {NULL}};
const Segment SGI260678[] = {{&S45410[29], 6}, {NULL}};
const Segment SGI260679[] = {{&S45410[47], 5}, {NULL}};
const Segment SGI260680[] = {{&S45411[29], 6}, {NULL}};
const Segment SGI260681[] = {{&S45411[47], 5}, {NULL}};
const Segment SGI260682[] = {{&S35615[25], 11}, {NULL}};
const Segment SGI260683[] = {{&S35615[0], 96}, {NULL}};
const Segment SGI260684[] = {{&S35616[0], 10}, {NULL}};
const Segment SGI260685[] = {{&S35616[0], 89}, {NULL}};
const Segment SGI260686[] = {{&S35617[0], 19}, {NULL}};
const Segment SGI260687[] = {{&S35617[0], 82}, {NULL}};
const Segment SGI260688[] = {{&S35618[41], 26}, {NULL}};
const Segment SGI260689[] = {{&S35618[0], 67}, {NULL}};
const Segment SGI260690[] = {{&S35619[44], 27}, {NULL}};
const Segment SGI260691[] = {{&S35619[0], 71}, {NULL}};
const Segment SGI260692[] = {{&S35620[41], 34}, {NULL}};
const Segment SGI260693[] = {{&S35620[0], 75}, {NULL}};
const Segment SGI260694[] = {{&S35621[12], 44}, {NULL}};
const Segment SGI260695[] = {{&S35621[0], 95}, {NULL}};
const Segment SGI260696[] = {{&S35622[40], 26}, {NULL}};
const Segment SGI260697[] = {{&S35622[0], 66}, {NULL}};
const Segment SGI260698[] = {{&S35623[31], 3}, {NULL}};
const Segment SGI260699[] = {{&S35623[0], 69}, {NULL}};
const Segment SGI260700[] = {{&S35624[40], 27}, {NULL}};
const Segment SGI260701[] = {{&S35624[0], 67}, {NULL}};
const Segment SGI260702[] = {{&S35625[53], 44}, {NULL}};
const Segment SGI260703[] = {{&S35625[0], 97}, {NULL}};
const Segment SGI260704[] = {{&S35626[0], 13}, {NULL}};
const Segment SGI260705[] = {{&S35626[0], 65}, {NULL}};
const Segment SGI260706[] = {{&S35627[53], 48}, {NULL}};
const Segment SGI260707[] = {{&S35627[0], 101}, {NULL}};
const Segment SGI260708[] = {{&S35629[0], 12}, {NULL}};
const Segment SGI260709[] = {{&S35629[0], 72}, {NULL}};
const Segment SGI260710[] = {{&S45532[13], 4}, {NULL}};
const Segment SGI260711[] = {{&S45532[26], 3}, {NULL}};
const Segment SGI260712[] = {{&S45532[57], 6}, {NULL}};
const Segment SGI260713[] = {{&S45532[109], 8}, {NULL}};
const Segment SGI260714[] = {{&S45532[125], 7}, {NULL}};
const Segment SGI260715[] = {{&S45533[13], 4}, {NULL}};
const Segment SGI260716[] = {{&S45533[26], 3}, {NULL}};
const Segment SGI260717[] = {{&S45533[57], 6}, {NULL}};
const Segment SGI260718[] = {{&S45533[110], 6}, {NULL}};
const Segment SGI260719[] = {{&S45533[124], 7}, {NULL}};
const Segment SGI260720[] = {{&S45534[13], 4}, {NULL}};
const Segment SGI260721[] = {{&S45534[26], 3}, {NULL}};
const Segment SGI260722[] = {{&S45534[57], 6}, {NULL}};
const Segment SGI260723[] = {{&S45534[110], 8}, {NULL}};
const Segment SGI260724[] = {{&S45534[126], 7}, {NULL}};
const Segment SGI260725[] = {{&S45535[13], 4}, {NULL}};
const Segment SGI260726[] = {{&S45535[26], 3}, {NULL}};
const Segment SGI260727[] = {{&S45535[57], 6}, {NULL}};
const Segment SGI260728[] = {{&S45535[110], 8}, {NULL}};
const Segment SGI260729[] = {{&S45535[126], 7}, {NULL}};
const Segment SGI260730[] = {{&S45536[13], 4}, {NULL}};
const Segment SGI260731[] = {{&S45536[26], 5}, {NULL}};
const Segment SGI260732[] = {{&S45536[59], 6}, {NULL}};
const Segment SGI260733[] = {{&S45536[112], 6}, {NULL}};
const Segment SGI260734[] = {{&S45536[126], 7}, {NULL}};
const Segment SGI260735[] = {{&S45695[13], 6}, {NULL}};
const Segment SGI260736[] = {{&S45695[110], 5}, {NULL}};
const Segment SGI260737[] = {{&S45695[38], 3}, {NULL}};
const Segment SGI260738[] = {{&S45695[76], 5}, {NULL}};
const Segment SGI260739[] = {{&S45695[123], 5}, {NULL}};
const Segment SGI260740[] = {{&S45696[13], 6}, {NULL}};
const Segment SGI260741[] = {{&S45696[111], 3}, {NULL}};
const Segment SGI260742[] = {{&S45696[38], 3}, {NULL}};
const Segment SGI260743[] = {{&S45696[76], 6}, {NULL}};
const Segment SGI260744[] = {{&S45696[122], 5}, {NULL}};
const Segment SGI260745[] = {{&S45696[135], 6}, {NULL}};
const Segment SGI260746[] = {{&S45697[13], 6}, {NULL}};
const Segment SGI260747[] = {{&S45697[110], 5}, {NULL}};
const Segment SGI260748[] = {{&S45697[38], 3}, {NULL}};
const Segment SGI260749[] = {{&S45697[76], 5}, {NULL}};
const Segment SGI260750[] = {{&S45697[123], 5}, {NULL}};
const Segment SGI260751[] = {{&S45697[136], 9}, {NULL}};
const Segment SGI260752[] = {{&S45698[13], 6}, {NULL}};
const Segment SGI260753[] = {{&S45698[112], 5}, {NULL}};
const Segment SGI260754[] = {{&S45698[38], 5}, {NULL}};
const Segment SGI260755[] = {{&S45698[78], 5}, {NULL}};
const Segment SGI260756[] = {{&S45698[125], 5}, {NULL}};
const Segment SGI260757[] = {{&S45699[13], 6}, {NULL}};
const Segment SGI260758[] = {{&S45699[113], 3}, {NULL}};
const Segment SGI260759[] = {{&S45699[38], 5}, {NULL}};
const Segment SGI260760[] = {{&S45699[78], 6}, {NULL}};
const Segment SGI260761[] = {{&S45699[124], 5}, {NULL}};
const Segment SGI260762[] = {{&S45699[137], 6}, {NULL}};
const Segment SGI260763[] = {{&S45700[13], 6}, {NULL}};
const Segment SGI260764[] = {{&S45700[112], 5}, {NULL}};
const Segment SGI260765[] = {{&S45700[38], 5}, {NULL}};
const Segment SGI260766[] = {{&S45700[78], 5}, {NULL}};
const Segment SGI260767[] = {{&S45700[125], 5}, {NULL}};
const Segment SGI260768[] = {{&S45700[138], 9}, {NULL}};
const Segment SGI260769[] = {{&S45701[13], 6}, {NULL}};
const Segment SGI260770[] = {{&S45701[35], 5}, {NULL}};
const Segment SGI260771[] = {{&S45701[68], 6}, {NULL}};
const Segment SGI260772[] = {{&S45701[102], 6}, {NULL}};
const Segment SGI260773[] = {{&S45701[116], 9}, {NULL}};
const Segment SGI260774[] = {{&S45702[13], 6}, {NULL}};
const Segment SGI260775[] = {{&S45702[112], 5}, {NULL}};
const Segment SGI260776[] = {{&S45702[38], 5}, {NULL}};
const Segment SGI260777[] = {{&S45702[78], 5}, {NULL}};
const Segment SGI260778[] = {{&S45702[125], 5}, {NULL}};
const Segment SGI260779[] = {{&S45703[13], 6}, {NULL}};
const Segment SGI260780[] = {{&S45703[112], 5}, {NULL}};
const Segment SGI260781[] = {{&S45703[38], 5}, {NULL}};
const Segment SGI260782[] = {{&S45703[78], 5}, {NULL}};
const Segment SGI260783[] = {{&S45703[125], 5}, {NULL}};
const Segment SGI260784[] = {{&S45704[13], 6}, {NULL}};
const Segment SGI260785[] = {{&S45704[113], 3}, {NULL}};
const Segment SGI260786[] = {{&S45704[38], 5}, {NULL}};
const Segment SGI260787[] = {{&S45704[78], 6}, {NULL}};
const Segment SGI260788[] = {{&S45704[124], 5}, {NULL}};
const Segment SGI260789[] = {{&S45704[137], 6}, {NULL}};
const Segment SGI260790[] = {{&S45705[13], 6}, {NULL}};
const Segment SGI260791[] = {{&S45705[112], 5}, {NULL}};
const Segment SGI260792[] = {{&S45705[38], 5}, {NULL}};
const Segment SGI260793[] = {{&S45705[78], 5}, {NULL}};
const Segment SGI260794[] = {{&S45705[125], 5}, {NULL}};
const Segment SGI260795[] = {{&S45705[138], 9}, {NULL}};
const Segment SGI260796[] = {{&S45706[13], 6}, {NULL}};
const Segment SGI260797[] = {{&S45706[35], 5}, {NULL}};
const Segment SGI260798[] = {{&S45706[68], 6}, {NULL}};
const Segment SGI260799[] = {{&S45706[102], 6}, {NULL}};
const Segment SGI260800[] = {{&S45706[116], 9}, {NULL}};
const Segment SGI260801[] = {{&S45707[13], 6}, {NULL}};
const Segment SGI260802[] = {{&S45707[113], 3}, {NULL}};
const Segment SGI260803[] = {{&S45707[38], 5}, {NULL}};
const Segment SGI260804[] = {{&S45707[78], 6}, {NULL}};
const Segment SGI260805[] = {{&S45707[124], 5}, {NULL}};
const Segment SGI260806[] = {{&S45707[137], 6}, {NULL}};
const Segment SGI260807[] = {{&S45708[13], 6}, {NULL}};
const Segment SGI260808[] = {{&S45708[38], 5}, {NULL}};
const Segment SGI260809[] = {{&S45708[86], 5}, {NULL}};
const Segment SGI260810[] = {{&S45708[119], 5}, {NULL}};
const Segment SGI260811[] = {{&S45708[132], 9}, {NULL}};
const Segment SGI260812[] = {{&S45709[13], 6}, {NULL}};
const Segment SGI260813[] = {{&S45709[113], 3}, {NULL}};
const Segment SGI260814[] = {{&S45709[38], 5}, {NULL}};
const Segment SGI260815[] = {{&S45709[78], 6}, {NULL}};
const Segment SGI260816[] = {{&S45709[124], 5}, {NULL}};
const Segment SGI260817[] = {{&S45709[137], 6}, {NULL}};
const Segment SGI260818[] = {{&S45710[13], 6}, {NULL}};
const Segment SGI260819[] = {{&S45710[112], 5}, {NULL}};
const Segment SGI260820[] = {{&S45710[38], 5}, {NULL}};
const Segment SGI260821[] = {{&S45710[78], 5}, {NULL}};
const Segment SGI260822[] = {{&S45710[125], 5}, {NULL}};
const Segment SGI260823[] = {{&S45711[13], 6}, {NULL}};
const Segment SGI260824[] = {{&S45711[38], 5}, {NULL}};
const Segment SGI260825[] = {{&S45711[86], 5}, {NULL}};
const Segment SGI260826[] = {{&S45711[119], 5}, {NULL}};
const Segment SGI260827[] = {{&S45711[132], 9}, {NULL}};
const Segment SGI260828[] = {{&S45712[13], 6}, {NULL}};
const Segment SGI260829[] = {{&S45712[112], 5}, {NULL}};
const Segment SGI260830[] = {{&S45712[38], 5}, {NULL}};
const Segment SGI260831[] = {{&S45712[78], 5}, {NULL}};
const Segment SGI260832[] = {{&S45712[125], 5}, {NULL}};
const Segment SGI260833[] = {{&S45712[138], 9}, {NULL}};
const Segment SGI260834[] = {{&S45713[13], 6}, {NULL}};
const Segment SGI260835[] = {{&S45713[112], 5}, {NULL}};
const Segment SGI260836[] = {{&S45713[38], 5}, {NULL}};
const Segment SGI260837[] = {{&S45713[78], 5}, {NULL}};
const Segment SGI260838[] = {{&S45713[125], 6}, {NULL}};
const Segment SGI260839[] = {{&S45713[139], 9}, {NULL}};
const Segment SGI260840[] = {{&S45714[13], 6}, {NULL}};
const Segment SGI260841[] = {{&S45714[110], 5}, {NULL}};
const Segment SGI260842[] = {{&S45714[38], 3}, {NULL}};
const Segment SGI260843[] = {{&S45714[76], 5}, {NULL}};
const Segment SGI260844[] = {{&S45714[123], 6}, {NULL}};
const Segment SGI260845[] = {{&S45714[137], 9}, {NULL}};
const Segment SGI260846[] = {{&S45715[13], 6}, {NULL}};
const Segment SGI260847[] = {{&S45715[35], 5}, {NULL}};
const Segment SGI260848[] = {{&S45715[68], 6}, {NULL}};
const Segment SGI260849[] = {{&S45715[102], 6}, {NULL}};
const Segment SGI260850[] = {{&S45715[116], 9}, {NULL}};
const Segment SGI260851[] = {{&S45716[13], 6}, {NULL}};
const Segment SGI260852[] = {{&S45716[38], 5}, {NULL}};
const Segment SGI260853[] = {{&S45716[86], 5}, {NULL}};
const Segment SGI260854[] = {{&S45716[119], 5}, {NULL}};
const Segment SGI260855[] = {{&S45716[132], 9}, {NULL}};
const Segment SGI260856[] = {{&S45767[13], 6}, {NULL}};
const Segment SGI260857[] = {{&S45767[38], 3}, {NULL}};
const Segment SGI260858[] = {{&S45767[76], 8}, {NULL}};
const Segment SGI260859[] = {{&S45767[131], 8}, {NULL}};
const Segment SGI260860[] = {{&S45767[147], 9}, {NULL}};
const Segment SGI260861[] = {{&S45768[13], 6}, {NULL}};
const Segment SGI260862[] = {{&S45768[38], 3}, {NULL}};
const Segment SGI260863[] = {{&S45768[76], 8}, {NULL}};
const Segment SGI260864[] = {{&S45768[131], 6}, {NULL}};
const Segment SGI260865[] = {{&S45768[145], 9}, {NULL}};
const Segment SGI260866[] = {{&S45769[13], 6}, {NULL}};
const Segment SGI260867[] = {{&S45769[38], 3}, {NULL}};
const Segment SGI260868[] = {{&S45769[76], 8}, {NULL}};
const Segment SGI260869[] = {{&S45769[131], 6}, {NULL}};
const Segment SGI260870[] = {{&S45769[145], 9}, {NULL}};
const Segment SGI260871[] = {{&S45770[13], 6}, {NULL}};
const Segment SGI260872[] = {{&S45770[38], 5}, {NULL}};
const Segment SGI260873[] = {{&S45770[78], 8}, {NULL}};
const Segment SGI260874[] = {{&S45770[133], 6}, {NULL}};
const Segment SGI260875[] = {{&S45770[147], 9}, {NULL}};
const Segment SGI260876[] = {{&S45771[13], 6}, {NULL}};
const Segment SGI260877[] = {{&S45771[38], 5}, {NULL}};
const Segment SGI260878[] = {{&S45771[78], 8}, {NULL}};
const Segment SGI260879[] = {{&S45771[133], 6}, {NULL}};
const Segment SGI260880[] = {{&S45771[147], 9}, {NULL}};
const Segment SGI260881[] = {{&S45772[13], 6}, {NULL}};
const Segment SGI260882[] = {{&S45772[38], 5}, {NULL}};
const Segment SGI260883[] = {{&S45772[78], 8}, {NULL}};
const Segment SGI260884[] = {{&S45772[133], 6}, {NULL}};
const Segment SGI260885[] = {{&S45772[147], 9}, {NULL}};
const Segment SGI260886[] = {{&S45774[13], 4}, {NULL}};
const Segment SGI260887[] = {{&S45774[91], 3}, {NULL}};
const Segment SGI260888[] = {{&S45774[57], 5}, {NULL}};
const Segment SGI260889[] = {{&S45774[102], 6}, {NULL}};
const Segment SGI260890[] = {{&S45774[116], 5}, {NULL}};
const Segment SGI260891[] = {{&S45775[13], 4}, {NULL}};
const Segment SGI260892[] = {{&S45775[91], 3}, {NULL}};
const Segment SGI260893[] = {{&S45775[57], 5}, {NULL}};
const Segment SGI260894[] = {{&S45775[102], 5}, {NULL}};
const Segment SGI260895[] = {{&S45775[115], 5}, {NULL}};
const Segment SGI260896[] = {{&S45776[13], 4}, {NULL}};
const Segment SGI260897[] = {{&S45776[91], 3}, {NULL}};
const Segment SGI260898[] = {{&S45776[57], 5}, {NULL}};
const Segment SGI260899[] = {{&S45776[102], 3}, {NULL}};
const Segment SGI260900[] = {{&S45776[113], 5}, {NULL}};
const Segment SGI260901[] = {{&S45777[13], 4}, {NULL}};
const Segment SGI260902[] = {{&S45777[91], 3}, {NULL}};
const Segment SGI260903[] = {{&S45777[57], 5}, {NULL}};
const Segment SGI260904[] = {{&S45777[102], 4}, {NULL}};
const Segment SGI260905[] = {{&S45777[114], 5}, {NULL}};
const Segment SGI260906[] = {{&S45778[13], 4}, {NULL}};
const Segment SGI260907[] = {{&S45778[91], 3}, {NULL}};
const Segment SGI260908[] = {{&S45778[57], 5}, {NULL}};
const Segment SGI260909[] = {{&S45778[102], 5}, {NULL}};
const Segment SGI260910[] = {{&S45778[115], 5}, {NULL}};
const Segment SGI260911[] = {{&S45779[13], 4}, {NULL}};
const Segment SGI260912[] = {{&S45779[91], 3}, {NULL}};
const Segment SGI260913[] = {{&S45779[57], 5}, {NULL}};
const Segment SGI260914[] = {{&S45779[102], 6}, {NULL}};
const Segment SGI260915[] = {{&S45779[116], 5}, {NULL}};
const Segment SGI260916[] = {{&S45780[13], 4}, {NULL}};
const Segment SGI260917[] = {{&S45780[90], 3}, {NULL}};
const Segment SGI260918[] = {{&S45780[57], 5}, {NULL}};
const Segment SGI260919[] = {{&S45780[101], 6}, {NULL}};
const Segment SGI260920[] = {{&S45780[115], 5}, {NULL}};
const Segment SGI260921[] = {{&S45792[13], 6}, {NULL}};
const Segment SGI260922[] = {{&S45792[35], 5}, {NULL}};
const Segment SGI260923[] = {{&S45792[75], 6}, {NULL}};
const Segment SGI260924[] = {{&S45792[109], 6}, {NULL}};
const Segment SGI260925[] = {{&S45792[116], 13}, {NULL}};
const Segment SGI260926[] = {{&S45793[13], 6}, {NULL}};
const Segment SGI260927[] = {{&S45793[35], 5}, {NULL}};
const Segment SGI260928[] = {{&S45793[75], 6}, {NULL}};
const Segment SGI260929[] = {{&S45793[109], 6}, {NULL}};
const Segment SGI260930[] = {{&S45793[116], 13}, {NULL}};
const Segment SGI260931[] = {{&S45794[13], 6}, {NULL}};
const Segment SGI260932[] = {{&S45794[35], 5}, {NULL}};
const Segment SGI260933[] = {{&S45794[75], 6}, {NULL}};
const Segment SGI260934[] = {{&S45794[109], 6}, {NULL}};
const Segment SGI260935[] = {{&S45794[116], 13}, {NULL}};
const Segment SGI260936[] = {{&S45795[13], 6}, {NULL}};
const Segment SGI260937[] = {{&S45795[35], 3}, {NULL}};
const Segment SGI260938[] = {{&S45795[73], 6}, {NULL}};
const Segment SGI260939[] = {{&S45795[107], 6}, {NULL}};
const Segment SGI260940[] = {{&S45795[114], 13}, {NULL}};
const Segment SGI260941[] = {{&S45796[13], 6}, {NULL}};
const Segment SGI260942[] = {{&S45796[35], 3}, {NULL}};
const Segment SGI260943[] = {{&S45796[73], 6}, {NULL}};
const Segment SGI260944[] = {{&S45796[107], 6}, {NULL}};
const Segment SGI260945[] = {{&S45796[114], 13}, {NULL}};
const Segment SGI260946[] = {{&S45797[13], 6}, {NULL}};
const Segment SGI260947[] = {{&S45797[35], 3}, {NULL}};
const Segment SGI260948[] = {{&S45797[73], 6}, {NULL}};
const Segment SGI260949[] = {{&S45797[107], 6}, {NULL}};
const Segment SGI260950[] = {{&S45797[114], 13}, {NULL}};
const Segment SGI260951[] = {{&S45798[13], 6}, {NULL}};
const Segment SGI260952[] = {{&S45798[35], 5}, {NULL}};
const Segment SGI260953[] = {{&S45798[75], 6}, {NULL}};
const Segment SGI260954[] = {{&S45798[109], 6}, {NULL}};
const Segment SGI260955[] = {{&S45798[116], 13}, {NULL}};
const Segment SGI260956[] = {{&S45799[13], 6}, {NULL}};
const Segment SGI260957[] = {{&S45799[35], 3}, {NULL}};
const Segment SGI260958[] = {{&S45799[73], 6}, {NULL}};
const Segment SGI260959[] = {{&S45799[107], 6}, {NULL}};
const Segment SGI260960[] = {{&S45799[114], 13}, {NULL}};
const Segment SGI260961[] = {{&S45800[13], 6}, {NULL}};
const Segment SGI260962[] = {{&S45800[35], 5}, {NULL}};
const Segment SGI260963[] = {{&S45800[75], 6}, {NULL}};
const Segment SGI260964[] = {{&S45800[129], 6}, {NULL}};
const Segment SGI260965[] = {{&S45800[143], 9}, {NULL}};
const Segment SGI260966[] = {{&S45800[153], 13}, {NULL}};
const Segment SGI260967[] = {{&S45831[13], 4}, {NULL}};
const Segment SGI260968[] = {{&S45831[26], 3}, {NULL}};
const Segment SGI260969[] = {{&S45831[57], 6}, {NULL}};
const Segment SGI260970[] = {{&S45831[91], 5}, {NULL}};
const Segment SGI260971[] = {{&S45831[97], 7}, {NULL}};
const Segment SGI260972[] = {{&S45832[13], 4}, {NULL}};
const Segment SGI260973[] = {{&S45832[26], 5}, {NULL}};
const Segment SGI260974[] = {{&S45832[59], 6}, {NULL}};
const Segment SGI260975[] = {{&S45832[93], 5}, {NULL}};
const Segment SGI260976[] = {{&S45832[99], 7}, {NULL}};
const Segment SGI260977[] = {{&S45839[17], 6}, {NULL}};
const Segment SGI260978[] = {{&S45839[38], 5}, {NULL}};
const Segment SGI260979[] = {{&S45839[68], 3}, {NULL}};
const Segment SGI260980[] = {{&S45840[17], 6}, {NULL}};
const Segment SGI260981[] = {{&S45840[38], 5}, {NULL}};
const Segment SGI260982[] = {{&S45840[68], 3}, {NULL}};
const Segment SGI260983[] = {{&S45841[17], 6}, {NULL}};
const Segment SGI260984[] = {{&S45841[38], 3}, {NULL}};
const Segment SGI260985[] = {{&S45841[66], 3}, {NULL}};
const Segment SGI260986[] = {{&S36169[11], 41}, {NULL}};
const Segment SGI260987[] = {{&S36169[0], 52}, {NULL}};
const Segment SGI260988[] = {{&S45976[29], 4}, {NULL}};
const Segment SGI260989[] = {{&S45976[45], 3}, {NULL}};
const Segment SGI260990[] = {{&S45977[29], 4}, {NULL}};
const Segment SGI260991[] = {{&S45977[45], 5}, {NULL}};
const Segment SGI260992[] = {{&S45978[29], 4}, {NULL}};
const Segment SGI260993[] = {{&S45978[45], 5}, {NULL}};
const Segment SGI260994[] = {{&S46056[13], 4}, {NULL}};
const Segment SGI260995[] = {{&S46056[98], 5}, {NULL}};
const Segment SGI260996[] = {{&S46056[29], 3}, {NULL}};
const Segment SGI260997[] = {{&S46056[60], 9}, {NULL}};
const Segment SGI260998[] = {{&S46056[111], 5}, {NULL}};
const Segment SGI260999[] = {{&S46056[134], 13}, {NULL}};
const Segment SGI261000[] = {{&S46085[13], 4}, {NULL}};
const Segment SGI261001[] = {{&S46085[29], 3}, {NULL}};
const Segment SGI261002[] = {{&S46085[67], 8}, {NULL}};
const Segment SGI261003[] = {{&S46085[122], 6}, {NULL}};
const Segment SGI261004[] = {{&S46085[136], 9}, {NULL}};
const Segment SGI261005[] = {{&S46086[13], 4}, {NULL}};
const Segment SGI261006[] = {{&S46086[29], 5}, {NULL}};
const Segment SGI261007[] = {{&S46086[69], 8}, {NULL}};
const Segment SGI261008[] = {{&S46086[124], 6}, {NULL}};
const Segment SGI261009[] = {{&S46086[138], 9}, {NULL}};
const Segment SGI261010[] = {{&S46087[13], 4}, {NULL}};
const Segment SGI261011[] = {{&S46087[29], 5}, {NULL}};
const Segment SGI261012[] = {{&S46087[69], 8}, {NULL}};
const Segment SGI261013[] = {{&S46087[124], 6}, {NULL}};
const Segment SGI261014[] = {{&S46087[138], 9}, {NULL}};
const Segment SGI261015[] = {{&S46088[13], 4}, {NULL}};
const Segment SGI261016[] = {{&S46088[29], 5}, {NULL}};
const Segment SGI261017[] = {{&S46088[69], 8}, {NULL}};
const Segment SGI261018[] = {{&S46088[124], 6}, {NULL}};
const Segment SGI261019[] = {{&S46088[138], 9}, {NULL}};
const Segment SGI261020[] = {{&S46092[13], 6}, {NULL}};
const Segment SGI261021[] = {{&S46092[35], 3}, {NULL}};
const Segment SGI261022[] = {{&S46092[66], 6}, {NULL}};
const Segment SGI261023[] = {{&S46092[100], 5}, {NULL}};
const Segment SGI261024[] = {{&S46092[106], 7}, {NULL}};
const Segment SGI261025[] = {{&S46093[13], 6}, {NULL}};
const Segment SGI261026[] = {{&S46093[35], 5}, {NULL}};
const Segment SGI261027[] = {{&S46093[68], 6}, {NULL}};
const Segment SGI261028[] = {{&S46093[102], 5}, {NULL}};
const Segment SGI261029[] = {{&S46093[108], 7}, {NULL}};
const Segment SGI261030[] = {{&S46094[13], 6}, {NULL}};
const Segment SGI261031[] = {{&S46094[35], 5}, {NULL}};
const Segment SGI261032[] = {{&S46094[68], 6}, {NULL}};
const Segment SGI261033[] = {{&S46094[102], 5}, {NULL}};
const Segment SGI261034[] = {{&S46094[108], 7}, {NULL}};
const Segment SGI261035[] = {{&S46095[13], 6}, {NULL}};
const Segment SGI261036[] = {{&S46095[35], 5}, {NULL}};
const Segment SGI261037[] = {{&S46095[68], 6}, {NULL}};
const Segment SGI261038[] = {{&S46095[102], 5}, {NULL}};
const Segment SGI261039[] = {{&S46095[108], 7}, {NULL}};
const Segment SGI261040[] = {{&S36367[21], 30}, {NULL}};
const Segment SGI261041[] = {{&S36367[0], 51}, {NULL}};
const Segment SGI261042[] = {{&S36368[21], 40}, {NULL}};
const Segment SGI261043[] = {{&S36368[0], 88}, {NULL}};
const Segment SGI261044[] = {{&S36369[21], 30}, {NULL}};
const Segment SGI261045[] = {{&S36369[0], 51}, {NULL}};
const Segment SGI261046[] = {{&S46572[13], 6}, {NULL}};
const Segment SGI261047[] = {{&S46572[38], 5}, {NULL}};
const Segment SGI261048[] = {{&S46572[71], 8}, {NULL}};
const Segment SGI261049[] = {{&S46572[119], 3}, {NULL}};
const Segment SGI261050[] = {{&S46572[130], 9}, {NULL}};
const Segment SGI261051[] = {{&S46573[13], 6}, {NULL}};
const Segment SGI261052[] = {{&S46573[38], 5}, {NULL}};
const Segment SGI261053[] = {{&S46573[71], 8}, {NULL}};
const Segment SGI261054[] = {{&S46573[119], 3}, {NULL}};
const Segment SGI261055[] = {{&S46573[130], 9}, {NULL}};
const Segment SGI261056[] = {{&S46574[13], 6}, {NULL}};
const Segment SGI261057[] = {{&S46574[38], 5}, {NULL}};
const Segment SGI261058[] = {{&S46574[71], 8}, {NULL}};
const Segment SGI261059[] = {{&S46574[119], 3}, {NULL}};
const Segment SGI261060[] = {{&S46574[130], 9}, {NULL}};
const Segment SGI261061[] = {{&S30360[10], 4}, {NULL}};
const Segment SGI261062[] = {{&S30360[15], 5}, {NULL}};
const Segment SGI261063[] = {{&S46646[28], 4}, {NULL}};
const Segment SGI261064[] = {{&S46754[29], 4}, {NULL}};
const Segment SGI261065[] = {{&S46915[13], 4}, {&S46915[13], 4}, {NULL}};
const Segment SGI261066[] = {{&S46915[26], 3}, {NULL}};
const Segment SGI261067[] = {{&S46915[57], 6}, {NULL}};
const Segment SGI261068[] = {{&S46915[91], 6}, {NULL}};
const Segment SGI261069[] = {{&S46915[105], 9}, {NULL}};
const Segment SGI261070[] = {{&S46916[13], 4}, {&S46916[13], 4}, {NULL}};
const Segment SGI261071[] = {{&S46916[26], 5}, {NULL}};
const Segment SGI261072[] = {{&S46916[59], 6}, {NULL}};
const Segment SGI261073[] = {{&S46916[93], 6}, {NULL}};
const Segment SGI261074[] = {{&S46916[107], 9}, {NULL}};
const Segment SGI261075[] = {{&S46917[13], 4}, {&S46917[13], 4}, {NULL}};
const Segment SGI261076[] = {{&S46917[26], 5}, {NULL}};
const Segment SGI261077[] = {{&S46917[59], 6}, {NULL}};
const Segment SGI261078[] = {{&S46917[93], 6}, {NULL}};
const Segment SGI261079[] = {{&S46917[107], 9}, {NULL}};
const Segment SGI261080[] = {{&S47189[13], 6}, {NULL}};
const Segment SGI261081[] = {{&S47189[38], 5}, {NULL}};
const Segment SGI261082[] = {{&S47189[78], 8}, {NULL}};
const Segment SGI261083[] = {{&S47189[114], 5}, {NULL}};
const Segment SGI261084[] = {{&S47189[120], 7}, {NULL}};
const Segment SGI261085[] = {{&S47190[13], 6}, {NULL}};
const Segment SGI261086[] = {{&S47190[38], 5}, {NULL}};
const Segment SGI261087[] = {{&S47190[78], 8}, {NULL}};
const Segment SGI261088[] = {{&S47190[114], 6}, {NULL}};
const Segment SGI261089[] = {{&S47190[121], 7}, {NULL}};
const Segment SGI261090[] = {{&S47191[13], 6}, {NULL}};
const Segment SGI261091[] = {{&S47191[38], 5}, {NULL}};
const Segment SGI261092[] = {{&S47191[78], 8}, {NULL}};
const Segment SGI261093[] = {{&S47191[114], 5}, {NULL}};
const Segment SGI261094[] = {{&S47191[120], 7}, {NULL}};
const Segment SGI261095[] = {{&S47192[13], 6}, {NULL}};
const Segment SGI261096[] = {{&S47192[38], 5}, {NULL}};
const Segment SGI261097[] = {{&S47192[78], 8}, {NULL}};
const Segment SGI261098[] = {{&S47192[114], 5}, {NULL}};
const Segment SGI261099[] = {{&S47192[120], 7}, {NULL}};
const Segment SGI261100[] = {{&S47193[13], 6}, {NULL}};
const Segment SGI261101[] = {{&S47193[38], 5}, {NULL}};
const Segment SGI261102[] = {{&S47193[78], 8}, {NULL}};
const Segment SGI261103[] = {{&S47193[114], 5}, {NULL}};
const Segment SGI261104[] = {{&S47193[120], 7}, {NULL}};
const Segment SGI261105[] = {{&S47194[13], 6}, {NULL}};
const Segment SGI261106[] = {{&S47194[38], 6}, {NULL}};
const Segment SGI261107[] = {{&S47194[79], 8}, {NULL}};
const Segment SGI261108[] = {{&S47194[115], 5}, {NULL}};
const Segment SGI261109[] = {{&S47194[121], 7}, {NULL}};
const Segment SGI261110[] = {{&S47195[13], 6}, {NULL}};
const Segment SGI261111[] = {{&S47195[38], 5}, {NULL}};
const Segment SGI261112[] = {{&S47195[78], 8}, {NULL}};
const Segment SGI261113[] = {{&S47195[114], 5}, {NULL}};
const Segment SGI261114[] = {{&S47195[120], 7}, {NULL}};
const Segment SGI261115[] = {{&S47196[13], 6}, {NULL}};
const Segment SGI261116[] = {{&S47196[38], 5}, {NULL}};
const Segment SGI261117[] = {{&S47196[78], 8}, {NULL}};
const Segment SGI261118[] = {{&S47196[114], 5}, {NULL}};
const Segment SGI261119[] = {{&S47196[120], 7}, {NULL}};
const Segment SGI261120[] = {{&S47197[13], 6}, {NULL}};
const Segment SGI261121[] = {{&S47197[38], 3}, {NULL}};
const Segment SGI261122[] = {{&S47197[76], 8}, {NULL}};
const Segment SGI261123[] = {{&S47197[112], 5}, {NULL}};
const Segment SGI261124[] = {{&S47197[118], 7}, {NULL}};
const Segment SGI261125[] = {{&S47242[13], 4}, {NULL}};
const Segment SGI261126[] = {{&S47242[29], 5}, {NULL}};
const Segment SGI261127[] = {{&S47242[62], 8}, {NULL}};
const Segment SGI261128[] = {{&S47242[110], 3}, {NULL}};
const Segment SGI261129[] = {{&S47242[121], 9}, {NULL}};
const Segment SGI261130[] = {{&S47243[13], 4}, {NULL}};
const Segment SGI261131[] = {{&S47243[29], 5}, {NULL}};
const Segment SGI261132[] = {{&S47243[62], 8}, {NULL}};
const Segment SGI261133[] = {{&S47243[110], 3}, {NULL}};
const Segment SGI261134[] = {{&S47243[121], 9}, {NULL}};
const Segment SGI261135[] = {{&S47244[13], 4}, {NULL}};
const Segment SGI261136[] = {{&S47244[29], 5}, {NULL}};
const Segment SGI261137[] = {{&S47244[62], 8}, {NULL}};
const Segment SGI261138[] = {{&S47244[110], 3}, {NULL}};
const Segment SGI261139[] = {{&S47244[121], 9}, {NULL}};
const Segment SGI261140[] = {{&S47483[13], 4}, {NULL}};
const Segment SGI261141[] = {{&S47483[92], 3}, {NULL}};
const Segment SGI261142[] = {{&S47483[26], 3}, {NULL}};
const Segment SGI261143[] = {{&S47483[57], 6}, {NULL}};
const Segment SGI261144[] = {{&S47483[103], 6}, {NULL}};
const Segment SGI261145[] = {{&S47483[117], 7}, {NULL}};
const Segment SGI261146[] = {{&S47484[13], 4}, {NULL}};
const Segment SGI261147[] = {{&S47484[94], 3}, {NULL}};
const Segment SGI261148[] = {{&S47484[26], 5}, {NULL}};
const Segment SGI261149[] = {{&S47484[59], 6}, {NULL}};
const Segment SGI261150[] = {{&S47484[105], 6}, {NULL}};
const Segment SGI261151[] = {{&S47484[119], 7}, {NULL}};
const Segment SGI261152[] = {{&S47485[13], 4}, {NULL}};
const Segment SGI261153[] = {{&S47485[92], 3}, {NULL}};
const Segment SGI261154[] = {{&S47485[26], 3}, {NULL}};
const Segment SGI261155[] = {{&S47485[57], 6}, {NULL}};
const Segment SGI261156[] = {{&S47485[103], 5}, {NULL}};
const Segment SGI261157[] = {{&S47485[116], 9}, {NULL}};
const Segment SGI261158[] = {{&S47486[13], 4}, {NULL}};
const Segment SGI261159[] = {{&S47486[94], 3}, {NULL}};
const Segment SGI261160[] = {{&S47486[26], 5}, {NULL}};
const Segment SGI261161[] = {{&S47486[59], 6}, {NULL}};
const Segment SGI261162[] = {{&S47486[105], 6}, {NULL}};
const Segment SGI261163[] = {{&S47486[119], 7}, {NULL}};
const Segment SGI261164[] = {{&S47487[13], 4}, {NULL}};
const Segment SGI261165[] = {{&S47487[94], 3}, {NULL}};
const Segment SGI261166[] = {{&S47487[26], 5}, {NULL}};
const Segment SGI261167[] = {{&S47487[59], 6}, {NULL}};
const Segment SGI261168[] = {{&S47487[105], 6}, {NULL}};
const Segment SGI261169[] = {{&S47487[119], 7}, {NULL}};
const Segment SGI261170[] = {{&S47488[13], 4}, {NULL}};
const Segment SGI261171[] = {{&S47488[94], 5}, {NULL}};
const Segment SGI261172[] = {{&S47488[26], 5}, {NULL}};
const Segment SGI261173[] = {{&S47488[59], 6}, {NULL}};
const Segment SGI261174[] = {{&S47488[107], 6}, {NULL}};
const Segment SGI261175[] = {{&S47488[121], 7}, {NULL}};
const Segment SGI261176[] = {{&S47556[13], 4}, {NULL}};
const Segment SGI261177[] = {{&S47556[26], 5}, {NULL}};
const Segment SGI261178[] = {{&S47556[59], 6}, {NULL}};
const Segment SGI261179[] = {{&S47556[105], 6}, {NULL}};
const Segment SGI261180[] = {{&S47556[119], 7}, {NULL}};
const Segment SGI261181[] = {{&S47557[13], 4}, {NULL}};
const Segment SGI261182[] = {{&S47557[26], 3}, {NULL}};
const Segment SGI261183[] = {{&S47557[57], 6}, {NULL}};
const Segment SGI261184[] = {{&S47557[103], 6}, {NULL}};
const Segment SGI261185[] = {{&S47557[117], 7}, {NULL}};
const Segment SGI261186[] = {{&S47725[13], 6}, {NULL}};
const Segment SGI261187[] = {{&S47725[35], 5}, {NULL}};
const Segment SGI261188[] = {{&S47725[68], 6}, {NULL}};
const Segment SGI261189[] = {{&S47725[114], 6}, {NULL}};
const Segment SGI261190[] = {{&S47725[128], 7}, {NULL}};
const Segment SGI261191[] = {{&S47726[13], 6}, {NULL}};
const Segment SGI261192[] = {{&S47726[35], 3}, {NULL}};
const Segment SGI261193[] = {{&S47726[66], 6}, {NULL}};
const Segment SGI261194[] = {{&S47726[112], 6}, {NULL}};
const Segment SGI261195[] = {{&S47726[126], 7}, {NULL}};
const Segment SGI261196[] = {{&S38142[125], 41}, {NULL}};
const Segment SGI261197[] = {{&S38142[0], 166}, {NULL}};
const Segment SGI261198[] = {{&S47900[12], 6}, {NULL}};
const Segment SGI261199[] = {{&S47900[76], 5}, {NULL}};
const Segment SGI261200[] = {{&S47963[13], 4}, {NULL}};
const Segment SGI261201[] = {{&S47963[26], 3}, {NULL}};
const Segment SGI261202[] = {{&S47963[57], 6}, {NULL}};
const Segment SGI261203[] = {{&S47963[91], 8}, {NULL}};
const Segment SGI261204[] = {{&S47963[100], 7}, {NULL}};
const Segment SGI261205[] = {{&S47964[13], 4}, {NULL}};
const Segment SGI261206[] = {{&S47964[26], 3}, {NULL}};
const Segment SGI261207[] = {{&S47964[57], 6}, {NULL}};
const Segment SGI261208[] = {{&S47964[91], 8}, {NULL}};
const Segment SGI261209[] = {{&S47964[100], 7}, {NULL}};
const Segment SGI261210[] = {{&S47965[13], 4}, {NULL}};
const Segment SGI261211[] = {{&S47965[26], 3}, {NULL}};
const Segment SGI261212[] = {{&S47965[57], 8}, {NULL}};
const Segment SGI261213[] = {{&S47965[93], 8}, {NULL}};
const Segment SGI261214[] = {{&S47965[102], 7}, {NULL}};
const Segment SGI261215[] = {{&S47966[13], 4}, {NULL}};
const Segment SGI261216[] = {{&S47966[26], 3}, {NULL}};
const Segment SGI261217[] = {{&S47966[57], 8}, {NULL}};
const Segment SGI261218[] = {{&S47966[93], 8}, {NULL}};
const Segment SGI261219[] = {{&S47966[102], 7}, {NULL}};
const Segment SGI261220[] = {{&S47967[13], 4}, {NULL}};
const Segment SGI261221[] = {{&S47967[26], 3}, {NULL}};
const Segment SGI261222[] = {{&S47967[57], 8}, {NULL}};
const Segment SGI261223[] = {{&S47967[93], 8}, {NULL}};
const Segment SGI261224[] = {{&S47967[102], 7}, {NULL}};
const Segment SGI261225[] = {{&S47968[13], 4}, {NULL}};
const Segment SGI261226[] = {{&S47968[26], 3}, {NULL}};
const Segment SGI261227[] = {{&S47968[57], 8}, {NULL}};
const Segment SGI261228[] = {{&S47968[93], 6}, {NULL}};
const Segment SGI261229[] = {{&S47968[100], 7}, {NULL}};
const Segment SGI261230[] = {{&S47969[13], 4}, {NULL}};
const Segment SGI261231[] = {{&S47969[26], 5}, {NULL}};
const Segment SGI261232[] = {{&S47969[59], 8}, {NULL}};
const Segment SGI261233[] = {{&S47969[95], 6}, {NULL}};
const Segment SGI261234[] = {{&S47969[102], 7}, {NULL}};
const Segment SGI261235[] = {{&S38408[25], 16}, {&S38408[42], 31}, {NULL}};
const Segment SGI261236[] = {{&S38408[0], 73}, {NULL}};
const Segment SGI261237[] = {{&S38409[25], 16}, {&S38409[42], 31}, {NULL}};
const Segment SGI261238[] = {{&S38409[0], 73}, {NULL}};
const Segment SGI261239[] = {{&S48130[13], 4}, {&S48130[13], 4}, {NULL}};
const Segment SGI261240[] = {{&S48130[204], 5}, {NULL}};
const Segment SGI261241[] = {{&S48130[26], 5}, {&S48130[128], 3}, {NULL}};
const Segment SGI261242[] = {{&S48130[59], 6}, {&S48130[166], 9}, {NULL}};
const Segment SGI261243[] = {{&S48130[93], 5}, {&S48130[217], 5}, {NULL}};
const Segment SGI261244[] = {{&S48130[166], 9}, {NULL}};
const Segment SGI261245[] = {{&S48131[13], 4}, {NULL}};
const Segment SGI261246[] = {{&S48131[111], 3}, {NULL}};
const Segment SGI261247[] = {{&S48131[36], 5}, {NULL}};
const Segment SGI261248[] = {{&S48131[76], 6}, {NULL}};
const Segment SGI261249[] = {{&S48131[122], 5}, {NULL}};
const Segment SGI261250[] = {{&S48131[135], 6}, {NULL}};
const Segment SGI261251[] = {{&S48132[13], 4}, {NULL}};
const Segment SGI261252[] = {{&S48132[109], 3}, {NULL}};
const Segment SGI261253[] = {{&S48132[36], 3}, {NULL}};
const Segment SGI261254[] = {{&S48132[74], 6}, {NULL}};
const Segment SGI261255[] = {{&S48132[120], 5}, {NULL}};
const Segment SGI261256[] = {{&S48132[133], 6}, {NULL}};
const Segment SGI261257[] = {{&S48133[13], 4}, {NULL}};
const Segment SGI261258[] = {{&S48133[107], 5}, {NULL}};
const Segment SGI261259[] = {{&S48133[29], 5}, {NULL}};
const Segment SGI261260[] = {{&S48133[69], 9}, {NULL}};
const Segment SGI261261[] = {{&S48133[120], 6}, {NULL}};
const Segment SGI261262[] = {{&S48134[13], 4}, {NULL}};
const Segment SGI261263[] = {{&S48134[107], 5}, {NULL}};
const Segment SGI261264[] = {{&S48134[29], 5}, {NULL}};
const Segment SGI261265[] = {{&S48134[69], 9}, {NULL}};
const Segment SGI261266[] = {{&S48134[120], 5}, {NULL}};
const Segment SGI261267[] = {{&S38491[57], 39}, {NULL}};
const Segment SGI261268[] = {{&S38491[0], 98}, {NULL}};
const Segment SGI261269[] = {{&S38492[64], 19}, {NULL}};
const Segment SGI261270[] = {{&S38492[0], 83}, {NULL}};
const Segment SGI261271[] = {{&S38493[56], 19}, {NULL}};
const Segment SGI261272[] = {{&S38493[0], 146}, {NULL}};
const Segment SGI261273[] = {{&S38494[51], 8}, {&S38494[64], 19}, {NULL}};
const Segment SGI261274[] = {{&S38494[0], 83}, {NULL}};
const Segment SGI261275[] = {{&S38495[54], 6}, {NULL}};
const Segment SGI261276[] = {{&S38495[0], 60}, {NULL}};
const Segment SGI261277[] = {{&S38496[55], 6}, {NULL}};
const Segment SGI261278[] = {{&S38496[0], 61}, {NULL}};
const Segment SGI261279[] = {{&S38497[51], 19}, {NULL}};
const Segment SGI261280[] = {{&S38497[0], 70}, {NULL}};
const Segment SGI261281[] = {{&S38498[78], 3}, {NULL}};
const Segment SGI261282[] = {{&S38498[0], 82}, {NULL}};
const Segment SGI261283[] = {{&S38499[90], 9}, {NULL}};
const Segment SGI261284[] = {{&S38499[0], 99}, {NULL}};
const Segment SGI261285[] = {{&S48157[13], 6}, {NULL}};
const Segment SGI261286[] = {{&S48157[35], 3}, {NULL}};
const Segment SGI261287[] = {{&S48157[66], 6}, {NULL}};
const Segment SGI261288[] = {{&S48157[119], 5}, {NULL}};
const Segment SGI261289[] = {{&S48157[132], 9}, {NULL}};
const Segment SGI261290[] = {{&S38500[56], 39}, {NULL}};
const Segment SGI261291[] = {{&S38500[0], 97}, {NULL}};
const Segment SGI261292[] = {{&S48158[13], 6}, {NULL}};
const Segment SGI261293[] = {{&S48158[35], 5}, {NULL}};
const Segment SGI261294[] = {{&S48158[68], 6}, {NULL}};
const Segment SGI261295[] = {{&S48158[121], 5}, {NULL}};
const Segment SGI261296[] = {{&S48158[134], 9}, {NULL}};
const Segment SGI261297[] = {{&S38501[51], 8}, {&S38501[65], 19}, {NULL}};
const Segment SGI261298[] = {{&S38501[0], 84}, {NULL}};
const Segment SGI261299[] = {{&S48159[13], 6}, {NULL}};
const Segment SGI261300[] = {{&S48159[35], 5}, {NULL}};
const Segment SGI261301[] = {{&S48159[68], 6}, {NULL}};
const Segment SGI261302[] = {{&S48159[121], 5}, {NULL}};
const Segment SGI261303[] = {{&S48159[134], 9}, {NULL}};
const Segment SGI261304[] = {{&S38502[56], 19}, {NULL}};
const Segment SGI261305[] = {{&S38502[0], 120}, {NULL}};
const Segment SGI261306[] = {{&S38536[51], 14}, {NULL}};
const Segment SGI261307[] = {{&S38536[0], 65}, {NULL}};
const Segment SGI261308[] = {{&S38537[70], 6}, {&S38537[70], 6}, {NULL}};
const Segment SGI261309[] = {{&S38537[0], 117}, {NULL}};
const Segment SGI261310[] = {{&S38538[146], 6}, {NULL}};
const Segment SGI261311[] = {{&S38538[0], 160}, {NULL}};
const Segment SGI261312[] = {{&S38539[102], 6}, {NULL}};
const Segment SGI261313[] = {{&S38539[0], 116}, {NULL}};
const Segment SGI261314[] = {{&S38540[70], 6}, {NULL}};
const Segment SGI261315[] = {{&S38540[0], 96}, {NULL}};
const Segment SGI261316[] = {{&S38541[64], 3}, {&S38541[64], 3}, {NULL}};
const Segment SGI261317[] = {{&S38541[0], 170}, {NULL}};
const Segment SGI261318[] = {{&S48227[13], 4}, {NULL}};
const Segment SGI261319[] = {{&S48227[92], 5}, {NULL}};
const Segment SGI261320[] = {{&S48227[26], 3}, {NULL}};
const Segment SGI261321[] = {{&S48227[57], 6}, {NULL}};
const Segment SGI261322[] = {{&S48227[105], 5}, {NULL}};
const Segment SGI261323[] = {{&S48227[118], 9}, {NULL}};
const Segment SGI261324[] = {{&S48228[13], 4}, {&S48228[13], 4}, {NULL}};
const Segment SGI261325[] = {{&S48228[118], 5}, {NULL}};
const Segment SGI261326[] = {{&S48228[29], 3}, {NULL}};
const Segment SGI261327[] = {{&S48228[81], 8}, {NULL}};
const Segment SGI261328[] = {{&S48228[131], 5}, {NULL}};
const Segment SGI261329[] = {{&S48228[144], 9}, {NULL}};
const Segment SGI261330[] = {{&S48229[13], 4}, {NULL}};
const Segment SGI261331[] = {{&S48229[29], 3}, {NULL}};
const Segment SGI261332[] = {{&S48229[102], 5}, {NULL}};
const Segment SGI261333[] = {{&S48229[115], 9}, {NULL}};
const Segment SGI261334[] = {{&S48230[13], 4}, {NULL}};
const Segment SGI261335[] = {{&S48230[105], 5}, {NULL}};
const Segment SGI261336[] = {{&S48230[29], 5}, {NULL}};
const Segment SGI261337[] = {{&S48230[69], 7}, {NULL}};
const Segment SGI261338[] = {{&S48230[118], 3}, {NULL}};
const Segment SGI261339[] = {{&S48230[129], 9}, {NULL}};
const Segment SGI261340[] = {{&S48231[13], 4}, {NULL}};
const Segment SGI261341[] = {{&S48231[106], 5}, {NULL}};
const Segment SGI261342[] = {{&S48231[29], 5}, {NULL}};
const Segment SGI261343[] = {{&S48231[69], 8}, {NULL}};
const Segment SGI261344[] = {{&S48231[119], 5}, {NULL}};
const Segment SGI261345[] = {{&S48231[132], 9}, {NULL}};
const Segment SGI261346[] = {{&S48232[13], 4}, {NULL}};
const Segment SGI261347[] = {{&S48232[103], 5}, {NULL}};
const Segment SGI261348[] = {{&S48232[29], 3}, {NULL}};
const Segment SGI261349[] = {{&S48232[67], 7}, {NULL}};
const Segment SGI261350[] = {{&S48232[116], 3}, {NULL}};
const Segment SGI261351[] = {{&S48232[127], 9}, {NULL}};
const Segment SGI261352[] = {{&S48233[13], 4}, {NULL}};
const Segment SGI261353[] = {{&S48233[104], 5}, {NULL}};
const Segment SGI261354[] = {{&S48233[29], 3}, {NULL}};
const Segment SGI261355[] = {{&S48233[67], 8}, {NULL}};
const Segment SGI261356[] = {{&S48233[117], 5}, {NULL}};
const Segment SGI261357[] = {{&S48233[130], 9}, {NULL}};
const Segment SGI261358[] = {{&S48234[13], 4}, {NULL}};
const Segment SGI261359[] = {{&S48234[94], 5}, {NULL}};
const Segment SGI261360[] = {{&S48234[26], 5}, {NULL}};
const Segment SGI261361[] = {{&S48234[59], 6}, {NULL}};
const Segment SGI261362[] = {{&S48234[107], 5}, {NULL}};
const Segment SGI261363[] = {{&S48234[120], 9}, {NULL}};
const Segment SGI261364[] = {{&S48235[13], 4}, {NULL}};
const Segment SGI261365[] = {{&S48235[29], 5}, {NULL}};
const Segment SGI261366[] = {{&S48235[104], 5}, {NULL}};
const Segment SGI261367[] = {{&S48235[117], 9}, {NULL}};
const Segment SGI261368[] = {{&S48236[13], 4}, {NULL}};
const Segment SGI261369[] = {{&S48236[26], 5}, {NULL}};
const Segment SGI261370[] = {{&S48236[59], 6}, {NULL}};
const Segment SGI261371[] = {{&S48236[93], 5}, {NULL}};
const Segment SGI261372[] = {{&S48236[106], 9}, {NULL}};
const Segment SGI261373[] = {{&S48237[13], 4}, {NULL}};
const Segment SGI261374[] = {{&S48237[26], 5}, {NULL}};
const Segment SGI261375[] = {{&S48237[59], 6}, {NULL}};
const Segment SGI261376[] = {{&S48237[93], 5}, {NULL}};
const Segment SGI261377[] = {{&S48237[106], 9}, {&S48237[106], 9}, {NULL}};
const Segment SGI261378[] = {{&S48238[13], 4}, {NULL}};
const Segment SGI261379[] = {{&S48238[26], 3}, {NULL}};
const Segment SGI261380[] = {{&S48238[57], 6}, {NULL}};
const Segment SGI261381[] = {{&S48238[91], 5}, {NULL}};
const Segment SGI261382[] = {{&S48238[104], 9}, {NULL}};
const Segment SGI261383[] = {{&S48239[13], 4}, {NULL}};
const Segment SGI261384[] = {{&S48239[26], 5}, {NULL}};
const Segment SGI261385[] = {{&S48239[59], 6}, {NULL}};
const Segment SGI261386[] = {{&S48239[93], 5}, {NULL}};
const Segment SGI261387[] = {{&S48239[106], 9}, {&S48239[106], 9}, {&S48239[106], 9}, {NULL}};
const Segment SGI261388[] = {{&S48240[13], 4}, {NULL}};
const Segment SGI261389[] = {{&S48240[106], 5}, {NULL}};
const Segment SGI261390[] = {{&S48240[29], 5}, {NULL}};
const Segment SGI261391[] = {{&S48240[69], 8}, {NULL}};
const Segment SGI261392[] = {{&S48240[119], 5}, {&S48240[31], 1}, {NULL}};
const Segment SGI261393[] = {{&S48240[152], 9}, {NULL}};
const Segment SGI261394[] = {{&S38757[55], 8}, {&S38757[73], 37}, {NULL}};
const Segment SGI261395[] = {{&S38757[0], 110}, {NULL}};
const Segment SGI261396[] = {{&S38758[53], 17}, {NULL}};
const Segment SGI261397[] = {{&S38758[0], 104}, {NULL}};
const Segment SGI261398[] = {{&S48268[17], 6}, {NULL}};
const Segment SGI261399[] = {{&S48268[35], 3}, {NULL}};
const Segment SGI261400[] = {{&S48269[29], 6}, {NULL}};
const Segment SGI261401[] = {{&S48270[20], 6}, {NULL}};
const Segment SGI261402[] = {{&S48270[38], 5}, {NULL}};
const Segment SGI261403[] = {{&S48271[13], 6}, {NULL}};
const Segment SGI261404[] = {{&S48271[31], 5}, {NULL}};
const Segment SGI261405[] = {{&S48288[13], 4}, {NULL}};
const Segment SGI261406[] = {{&S48288[26], 5}, {NULL}};
const Segment SGI261407[] = {{&S48288[59], 6}, {NULL}};
const Segment SGI261408[] = {{&S48288[112], 6}, {NULL}};
const Segment SGI261409[] = {{&S48288[126], 7}, {NULL}};
const Segment SGI261410[] = {{&S48289[13], 4}, {NULL}};
const Segment SGI261411[] = {{&S48289[26], 3}, {NULL}};
const Segment SGI261412[] = {{&S48289[57], 6}, {NULL}};
const Segment SGI261413[] = {{&S48289[110], 6}, {NULL}};
const Segment SGI261414[] = {{&S48289[124], 7}, {NULL}};
const Segment SGI261415[] = {{&S48290[13], 4}, {NULL}};
const Segment SGI261416[] = {{&S48290[26], 5}, {NULL}};
const Segment SGI261417[] = {{&S48290[59], 6}, {NULL}};
const Segment SGI261418[] = {{&S48290[112], 6}, {NULL}};
const Segment SGI261419[] = {{&S48290[126], 7}, {NULL}};
const Segment SGI261420[] = {{&S48291[13], 4}, {NULL}};
const Segment SGI261421[] = {{&S48291[26], 5}, {NULL}};
const Segment SGI261422[] = {{&S48291[59], 6}, {NULL}};
const Segment SGI261423[] = {{&S48291[112], 7}, {NULL}};
const Segment SGI261424[] = {{&S48291[127], 7}, {NULL}};
const Segment SGI261425[] = {{&S48292[13], 4}, {NULL}};
const Segment SGI261426[] = {{&S48292[26], 3}, {NULL}};
const Segment SGI261427[] = {{&S48292[57], 6}, {NULL}};
const Segment SGI261428[] = {{&S48292[110], 6}, {NULL}};
const Segment SGI261429[] = {{&S48292[124], 7}, {NULL}};
const Segment SGI261430[] = {{&S48293[13], 4}, {NULL}};
const Segment SGI261431[] = {{&S48293[26], 5}, {NULL}};
const Segment SGI261432[] = {{&S48293[59], 6}, {NULL}};
const Segment SGI261433[] = {{&S48293[112], 7}, {NULL}};
const Segment SGI261434[] = {{&S48293[127], 7}, {NULL}};
const Segment SGI261435[] = {{&S48294[13], 4}, {NULL}};
const Segment SGI261436[] = {{&S48294[26], 3}, {NULL}};
const Segment SGI261437[] = {{&S48294[57], 6}, {NULL}};
const Segment SGI261438[] = {{&S48294[110], 6}, {NULL}};
const Segment SGI261439[] = {{&S48294[124], 7}, {NULL}};
const Segment SGI261440[] = {{&S48298[29], 6}, {NULL}};
const Segment SGI261441[] = {{&S48298[47], 3}, {NULL}};
const Segment SGI261442[] = {{&S48299[29], 6}, {NULL}};
const Segment SGI261443[] = {{&S48299[47], 5}, {NULL}};
const Segment SGI261444[] = {{&S48300[29], 6}, {NULL}};
const Segment SGI261445[] = {{&S48300[47], 5}, {NULL}};
const Segment SGI261446[] = {{&S48308[13], 4}, {NULL}};
const Segment SGI261447[] = {{&S48308[26], 3}, {NULL}};
const Segment SGI261448[] = {{&S48308[64], 6}, {NULL}};
const Segment SGI261449[] = {{&S48308[110], 6}, {NULL}};
const Segment SGI261450[] = {{&S48308[124], 7}, {NULL}};
const Segment SGI261451[] = {{&S38869[25], 10}, {&S38869[46], 35}, {NULL}};
const Segment SGI261452[] = {{&S38869[0], 81}, {NULL}};
const Segment SGI261453[] = {{&S38958[25], 17}, {&S38958[43], 46}, {NULL}};
const Segment SGI261454[] = {{&S38958[0], 89}, {NULL}};
const Segment SGI261455[] = {{&S38959[25], 17}, {&S38959[43], 56}, {NULL}};
const Segment SGI261456[] = {{&S38959[0], 128}, {NULL}};
const Segment SGI261457[] = {{&S48489[13], 4}, {NULL}};
const Segment SGI261458[] = {{&S48489[92], 5}, {NULL}};
const Segment SGI261459[] = {{&S48489[26], 3}, {NULL}};
const Segment SGI261460[] = {{&S48489[57], 6}, {NULL}};
const Segment SGI261461[] = {{&S48489[105], 5}, {NULL}};
const Segment SGI261462[] = {{&S48489[118], 9}, {NULL}};
const Segment SGI261463[] = {{&S48490[13], 4}, {&S48490[13], 4}, {NULL}};
const Segment SGI261464[] = {{&S48490[116], 5}, {NULL}};
const Segment SGI261465[] = {{&S48490[8], 1}, {NULL}};
const Segment SGI261466[] = {{&S48490[79], 8}, {NULL}};
const Segment SGI261467[] = {{&S48490[129], 5}, {NULL}};
const Segment SGI261468[] = {{&S48490[142], 9}, {NULL}};
const Segment SGI261469[] = {{&S48491[13], 4}, {NULL}};
const Segment SGI261470[] = {{&S48491[105], 5}, {NULL}};
const Segment SGI261471[] = {{&S48491[29], 5}, {NULL}};
const Segment SGI261472[] = {{&S48491[69], 7}, {NULL}};
const Segment SGI261473[] = {{&S48491[118], 3}, {NULL}};
const Segment SGI261474[] = {{&S48491[129], 9}, {NULL}};
const Segment SGI261475[] = {{&S48492[13], 4}, {NULL}};
const Segment SGI261476[] = {{&S48492[104], 5}, {NULL}};
const Segment SGI261477[] = {{&S48492[29], 3}, {NULL}};
const Segment SGI261478[] = {{&S48492[67], 8}, {NULL}};
const Segment SGI261479[] = {{&S48492[117], 3}, {NULL}};
const Segment SGI261480[] = {{&S48492[128], 9}, {NULL}};
const Segment SGI261481[] = {{&S48493[13], 4}, {NULL}};
const Segment SGI261482[] = {{&S48493[94], 5}, {NULL}};
const Segment SGI261483[] = {{&S48493[26], 5}, {NULL}};
const Segment SGI261484[] = {{&S48493[59], 6}, {NULL}};
const Segment SGI261485[] = {{&S48493[107], 5}, {NULL}};
const Segment SGI261486[] = {{&S48493[120], 9}, {NULL}};
const Segment SGI261487[] = {{&S48494[13], 4}, {NULL}};
const Segment SGI261488[] = {{&S48494[104], 5}, {NULL}};
const Segment SGI261489[] = {{&S48494[29], 3}, {NULL}};
const Segment SGI261490[] = {{&S48494[67], 8}, {NULL}};
const Segment SGI261491[] = {{&S48494[117], 5}, {NULL}};
const Segment SGI261492[] = {{&S48494[130], 9}, {NULL}};
const Segment SGI261493[] = {{&S48495[13], 4}, {NULL}};
const Segment SGI261494[] = {{&S48495[106], 5}, {NULL}};
const Segment SGI261495[] = {{&S48495[29], 5}, {NULL}};
const Segment SGI261496[] = {{&S48495[69], 8}, {NULL}};
const Segment SGI261497[] = {{&S48495[119], 5}, {NULL}};
const Segment SGI261498[] = {{&S48495[132], 9}, {NULL}};
const Segment SGI261499[] = {{&S48496[13], 4}, {NULL}};
const Segment SGI261500[] = {{&S48496[29], 5}, {NULL}};
const Segment SGI261501[] = {{&S48496[104], 5}, {NULL}};
const Segment SGI261502[] = {{&S48496[117], 9}, {NULL}};
const Segment SGI261503[] = {{&S48497[13], 4}, {NULL}};
const Segment SGI261504[] = {{&S48497[103], 5}, {NULL}};
const Segment SGI261505[] = {{&S48497[29], 3}, {NULL}};
const Segment SGI261506[] = {{&S48497[67], 7}, {NULL}};
const Segment SGI261507[] = {{&S48497[116], 3}, {NULL}};
const Segment SGI261508[] = {{&S48497[127], 9}, {NULL}};
const Segment SGI261509[] = {{&S48498[13], 4}, {NULL}};
const Segment SGI261510[] = {{&S48498[92], 5}, {NULL}};
const Segment SGI261511[] = {{&S48498[26], 3}, {NULL}};
const Segment SGI261512[] = {{&S48498[57], 6}, {NULL}};
const Segment SGI261513[] = {{&S48498[105], 7}, {NULL}};
const Segment SGI261514[] = {{&S48498[120], 9}, {NULL}};
const Segment SGI261515[] = {{&S48499[13], 4}, {NULL}};
const Segment SGI261516[] = {{&S48499[26], 5}, {NULL}};
const Segment SGI261517[] = {{&S48499[59], 6}, {NULL}};
const Segment SGI261518[] = {{&S48499[93], 5}, {NULL}};
const Segment SGI261519[] = {{&S48499[106], 9}, {NULL}};
const Segment SGI261520[] = {{&S48500[13], 4}, {NULL}};
const Segment SGI261521[] = {{&S48500[104], 5}, {NULL}};
const Segment SGI261522[] = {{&S48500[29], 3}, {NULL}};
const Segment SGI261523[] = {{&S48500[67], 8}, {NULL}};
const Segment SGI261524[] = {{&S48500[117], 3}, {NULL}};
const Segment SGI261525[] = {{&S48500[128], 9}, {NULL}};
const Segment SGI261526[] = {{&S48501[12], 4}, {NULL}};
const Segment SGI261527[] = {{&S48501[103], 5}, {NULL}};
const Segment SGI261528[] = {{&S48501[28], 3}, {NULL}};
const Segment SGI261529[] = {{&S48501[66], 8}, {NULL}};
const Segment SGI261530[] = {{&S48501[116], 5}, {NULL}};
const Segment SGI261531[] = {{&S48501[129], 9}, {NULL}};
const Segment SGI261532[] = {{&S48502[13], 4}, {NULL}};
const Segment SGI261533[] = {{&S48502[8], 3}, {NULL}};
const Segment SGI261534[] = {{&S48502[26], 5}, {NULL}};
const Segment SGI261535[] = {{&S48502[59], 6}, {NULL}};
const Segment SGI261536[] = {{&S48502[105], 5}, {NULL}};
const Segment SGI261537[] = {{&S48502[118], 9}, {NULL}};
const Segment SGI261538[] = {{&S48503[13], 4}, {NULL}};
const Segment SGI261539[] = {{&S48503[102], 5}, {NULL}};
const Segment SGI261540[] = {{&S48503[29], 3}, {NULL}};
const Segment SGI261541[] = {{&S48503[67], 6}, {NULL}};
const Segment SGI261542[] = {{&S48503[115], 7}, {NULL}};
const Segment SGI261543[] = {{&S48503[130], 9}, {NULL}};
const Segment SGI261544[] = {{&S48517[13], 6}, {NULL}};
const Segment SGI261545[] = {{&S48517[38], 3}, {NULL}};
const Segment SGI261546[] = {{&S48517[76], 8}, {NULL}};
const Segment SGI261547[] = {{&S48517[131], 6}, {NULL}};
const Segment SGI261548[] = {{&S48517[145], 9}, {NULL}};
const Segment SGI261549[] = {{&S48518[13], 6}, {NULL}};
const Segment SGI261550[] = {{&S48518[38], 3}, {NULL}};
const Segment SGI261551[] = {{&S48518[76], 8}, {NULL}};
const Segment SGI261552[] = {{&S48518[131], 6}, {NULL}};
const Segment SGI261553[] = {{&S48518[145], 9}, {NULL}};
const Segment SGI261554[] = {{&S48519[13], 6}, {NULL}};
const Segment SGI261555[] = {{&S48519[38], 3}, {NULL}};
const Segment SGI261556[] = {{&S48519[76], 8}, {NULL}};
const Segment SGI261557[] = {{&S48519[131], 6}, {NULL}};
const Segment SGI261558[] = {{&S48519[145], 9}, {NULL}};
const Segment SGI261559[] = {{&S48520[13], 6}, {NULL}};
const Segment SGI261560[] = {{&S48520[38], 3}, {NULL}};
const Segment SGI261561[] = {{&S48520[76], 8}, {NULL}};
const Segment SGI261562[] = {{&S48520[131], 6}, {NULL}};
const Segment SGI261563[] = {{&S48520[145], 9}, {NULL}};
const Segment SGI261564[] = {{&S48521[13], 6}, {NULL}};
const Segment SGI261565[] = {{&S48521[38], 5}, {NULL}};
const Segment SGI261566[] = {{&S48521[78], 8}, {NULL}};
const Segment SGI261567[] = {{&S48521[133], 6}, {NULL}};
const Segment SGI261568[] = {{&S48521[147], 9}, {NULL}};
const Segment SGI261569[] = {{&S48522[13], 6}, {NULL}};
const Segment SGI261570[] = {{&S48522[38], 5}, {NULL}};
const Segment SGI261571[] = {{&S48522[78], 8}, {NULL}};
const Segment SGI261572[] = {{&S48522[133], 6}, {NULL}};
const Segment SGI261573[] = {{&S48522[147], 9}, {NULL}};
const Segment SGI261574[] = {{&S48523[13], 6}, {NULL}};
const Segment SGI261575[] = {{&S48523[38], 5}, {NULL}};
const Segment SGI261576[] = {{&S48523[78], 8}, {NULL}};
const Segment SGI261577[] = {{&S48523[133], 6}, {NULL}};
const Segment SGI261578[] = {{&S48523[147], 9}, {NULL}};
const Segment SGI261579[] = {{&S48524[13], 6}, {NULL}};
const Segment SGI261580[] = {{&S48524[38], 5}, {NULL}};
const Segment SGI261581[] = {{&S48524[78], 8}, {NULL}};
const Segment SGI261582[] = {{&S48524[133], 6}, {NULL}};
const Segment SGI261583[] = {{&S48524[147], 9}, {NULL}};
const Segment SGI261584[] = {{&S48531[13], 6}, {NULL}};
const Segment SGI261585[] = {{&S48531[111], 3}, {&S48531[191], 5}, {NULL}};
const Segment SGI261586[] = {{&S48531[38], 3}, {NULL}};
const Segment SGI261587[] = {{&S48531[76], 6}, {&S48531[154], 8}, {NULL}};
const Segment SGI261588[] = {{&S48531[122], 5}, {&S48531[204], 3}, {NULL}};
const Segment SGI261589[] = {{&S48531[135], 6}, {NULL}};
const Segment SGI261590[] = {{&S32530[22], 12}, {NULL}};
const Segment SGI261591[] = {{&S48733[29], 4}, {NULL}};
const Segment SGI261592[] = {{&S48733[45], 3}, {NULL}};
const Segment SGI261593[] = {{&S48734[29], 4}, {NULL}};
const Segment SGI261594[] = {{&S48734[45], 5}, {NULL}};
const Segment SGI261595[] = {{&S48735[29], 4}, {NULL}};
const Segment SGI261596[] = {{&S48735[45], 5}, {NULL}};
const Segment SGI261597[] = {{&S48862[13], 4}, {NULL}};
const Segment SGI261598[] = {{&S48862[36], 3}, {NULL}};
const Segment SGI261599[] = {{&S48862[74], 8}, {NULL}};
const Segment SGI261600[] = {{&S48862[129], 5}, {NULL}};
const Segment SGI261601[] = {{&S48862[142], 9}, {NULL}};
const Segment SGI261602[] = {{&S48863[13], 4}, {NULL}};
const Segment SGI261603[] = {{&S48863[36], 5}, {NULL}};
const Segment SGI261604[] = {{&S48863[76], 8}, {NULL}};
const Segment SGI261605[] = {{&S48863[131], 5}, {NULL}};
const Segment SGI261606[] = {{&S48863[144], 9}, {NULL}};
const Segment SGI261607[] = {{&S48864[13], 4}, {NULL}};
const Segment SGI261608[] = {{&S48864[36], 3}, {NULL}};
const Segment SGI261609[] = {{&S48864[74], 8}, {NULL}};
const Segment SGI261610[] = {{&S48864[128], 5}, {NULL}};
const Segment SGI261611[] = {{&S48864[141], 9}, {NULL}};
const Segment SGI261612[] = {{&S48865[13], 4}, {NULL}};
const Segment SGI261613[] = {{&S48865[36], 2}, {NULL}};
const Segment SGI261614[] = {{&S48865[73], 8}, {NULL}};
const Segment SGI261615[] = {{&S48865[127], 5}, {NULL}};
const Segment SGI261616[] = {{&S48865[140], 9}, {NULL}};
const Segment SGI261617[] = {{&S48891[13], 4}, {NULL}};
const Segment SGI261618[] = {{&S48891[33], 3}, {NULL}};
const Segment SGI261619[] = {{&S48891[64], 6}, {NULL}};
const Segment SGI261620[] = {{&S48891[98], 6}, {NULL}};
const Segment SGI261621[] = {{&S48891[105], 7}, {NULL}};
const Segment SGI261622[] = {{&S48892[13], 4}, {NULL}};
const Segment SGI261623[] = {{&S48892[33], 3}, {NULL}};
const Segment SGI261624[] = {{&S48892[64], 6}, {NULL}};
const Segment SGI261625[] = {{&S48892[98], 6}, {NULL}};
const Segment SGI261626[] = {{&S48892[105], 7}, {NULL}};
const Segment SGI261627[] = {{&S48893[13], 4}, {NULL}};
const Segment SGI261628[] = {{&S48893[33], 5}, {NULL}};
const Segment SGI261629[] = {{&S48893[66], 6}, {NULL}};
const Segment SGI261630[] = {{&S48893[100], 6}, {NULL}};
const Segment SGI261631[] = {{&S48893[107], 7}, {NULL}};
const Segment SGI261632[] = {{&S39524[38], 33}, {NULL}};
const Segment SGI261633[] = {{&S39524[0], 71}, {NULL}};
const Segment SGI261634[] = {{&S39542[0], 20}, {&S39542[21], 43}, {NULL}};
const Segment SGI261635[] = {{&S39542[0], 163}, {NULL}};
const Segment SGI261636[] = {{&S39543[64], 12}, {&S39543[77], 41}, {NULL}};
const Segment SGI261637[] = {{&S39543[0], 118}, {NULL}};
const Segment SGI261638[] = {{&S49239[13], 4}, {NULL}};
const Segment SGI261639[] = {{&S49239[26], 5}, {NULL}};
const Segment SGI261640[] = {{&S49239[59], 6}, {NULL}};
const Segment SGI261641[] = {{&S49239[112], 6}, {NULL}};
const Segment SGI261642[] = {{&S49239[126], 7}, {NULL}};
const Segment SGI261643[] = {{&S49240[13], 4}, {NULL}};
const Segment SGI261644[] = {{&S49240[26], 3}, {NULL}};
const Segment SGI261645[] = {{&S49240[57], 6}, {NULL}};
const Segment SGI261646[] = {{&S49240[110], 6}, {NULL}};
const Segment SGI261647[] = {{&S49240[124], 7}, {NULL}};
const Segment SGI261648[] = {{&S49241[13], 4}, {NULL}};
const Segment SGI261649[] = {{&S49241[26], 5}, {NULL}};
const Segment SGI261650[] = {{&S49241[59], 6}, {NULL}};
const Segment SGI261651[] = {{&S49241[112], 6}, {NULL}};
const Segment SGI261652[] = {{&S49241[126], 7}, {NULL}};
const Segment SGI261653[] = {{&S49242[13], 4}, {NULL}};
const Segment SGI261654[] = {{&S49242[26], 3}, {NULL}};
const Segment SGI261655[] = {{&S49242[57], 6}, {NULL}};
const Segment SGI261656[] = {{&S49242[110], 6}, {NULL}};
const Segment SGI261657[] = {{&S49242[124], 7}, {NULL}};
const Segment SGI261658[] = {{&S49243[13], 4}, {NULL}};
const Segment SGI261659[] = {{&S49243[26], 5}, {NULL}};
const Segment SGI261660[] = {{&S49243[59], 6}, {NULL}};
const Segment SGI261661[] = {{&S49243[112], 7}, {NULL}};
const Segment SGI261662[] = {{&S49243[127], 7}, {NULL}};
const Segment SGI261663[] = {{&S49244[13], 4}, {NULL}};
const Segment SGI261664[] = {{&S49244[26], 5}, {NULL}};
const Segment SGI261665[] = {{&S49244[59], 6}, {NULL}};
const Segment SGI261666[] = {{&S49244[112], 7}, {NULL}};
const Segment SGI261667[] = {{&S49244[127], 7}, {NULL}};
const Segment SGI261668[] = {{&S49245[13], 4}, {NULL}};
const Segment SGI261669[] = {{&S49245[26], 3}, {NULL}};
const Segment SGI261670[] = {{&S49245[57], 6}, {NULL}};
const Segment SGI261671[] = {{&S49245[110], 6}, {NULL}};
const Segment SGI261672[] = {{&S49245[124], 7}, {NULL}};
const Segment SGI261673[] = {{&S39921[48], 12}, {NULL}};
const Segment SGI261674[] = {{&S39921[0], 60}, {NULL}};
const Segment SGI261675[] = {{&S39922[47], 9}, {NULL}};
const Segment SGI261676[] = {{&S39922[0], 60}, {NULL}};
const Segment SGI261677[] = {{&S39923[47], 8}, {NULL}};
const Segment SGI261678[] = {{&S39923[0], 88}, {NULL}};
const Segment SGI261679[] = {{&S33156[10], 4}, {NULL}};
const Segment SGI261680[] = {{&S33156[15], 5}, {NULL}};
const Segment SGI261681[] = {{&S49376[13], 4}, {NULL}};
const Segment SGI261682[] = {{&S49376[29], 5}, {NULL}};
const Segment SGI261683[] = {{&S49376[69], 8}, {NULL}};
const Segment SGI261684[] = {{&S49376[124], 3}, {NULL}};
const Segment SGI261685[] = {{&S49376[135], 9}, {NULL}};
const Segment SGI261686[] = {{&S49377[13], 4}, {NULL}};
const Segment SGI261687[] = {{&S49377[29], 3}, {NULL}};
const Segment SGI261688[] = {{&S49377[67], 8}, {NULL}};
const Segment SGI261689[] = {{&S49377[122], 5}, {NULL}};
const Segment SGI261690[] = {{&S49377[135], 9}, {NULL}};
const Segment SGI261691[] = {{&S49378[13], 4}, {NULL}};
const Segment SGI261692[] = {{&S49378[29], 5}, {NULL}};
const Segment SGI261693[] = {{&S49378[69], 8}, {NULL}};
const Segment SGI261694[] = {{&S49378[124], 3}, {NULL}};
const Segment SGI261695[] = {{&S49378[135], 9}, {NULL}};
const Segment SGI261696[] = {{&S49379[13], 4}, {NULL}};
const Segment SGI261697[] = {{&S49379[29], 5}, {NULL}};
const Segment SGI261698[] = {{&S49379[69], 8}, {NULL}};
const Segment SGI261699[] = {{&S49379[124], 3}, {NULL}};
const Segment SGI261700[] = {{&S49379[135], 9}, {NULL}};
const Segment SGI261701[] = {{&S49380[13], 4}, {NULL}};
const Segment SGI261702[] = {{&S49380[29], 3}, {NULL}};
const Segment SGI261703[] = {{&S49380[67], 8}, {NULL}};
const Segment SGI261704[] = {{&S49380[122], 5}, {NULL}};
const Segment SGI261705[] = {{&S49380[135], 9}, {NULL}};
const Segment SGI261706[] = {{&S49381[13], 4}, {NULL}};
const Segment SGI261707[] = {{&S49381[29], 5}, {NULL}};
const Segment SGI261708[] = {{&S49381[69], 8}, {NULL}};
const Segment SGI261709[] = {{&S49381[124], 3}, {NULL}};
const Segment SGI261710[] = {{&S49381[135], 9}, {NULL}};
const Segment SGI261711[] = {{&S49382[13], 4}, {NULL}};
const Segment SGI261712[] = {{&S49382[29], 5}, {NULL}};
const Segment SGI261713[] = {{&S49382[69], 8}, {NULL}};
const Segment SGI261714[] = {{&S49382[124], 3}, {NULL}};
const Segment SGI261715[] = {{&S49382[135], 9}, {NULL}};
const Segment SGI261716[] = {{&S49383[13], 4}, {NULL}};
const Segment SGI261717[] = {{&S49383[29], 5}, {NULL}};
const Segment SGI261718[] = {{&S49383[69], 8}, {NULL}};
const Segment SGI261719[] = {{&S49383[124], 3}, {NULL}};
const Segment SGI261720[] = {{&S49383[135], 9}, {NULL}};
const Segment SGI261721[] = {{&S49384[13], 4}, {NULL}};
const Segment SGI261722[] = {{&S49384[29], 5}, {NULL}};
const Segment SGI261723[] = {{&S49384[69], 8}, {NULL}};
const Segment SGI261724[] = {{&S49384[124], 3}, {NULL}};
const Segment SGI261725[] = {{&S49384[135], 9}, {NULL}};
const Segment SGI261726[] = {{&S49418[13], 4}, {NULL}};
const Segment SGI261727[] = {{&S49418[26], 3}, {NULL}};
const Segment SGI261728[] = {{&S49418[57], 6}, {NULL}};
const Segment SGI261729[] = {{&S49418[91], 6}, {NULL}};
const Segment SGI261730[] = {{&S49418[105], 9}, {NULL}};
const Segment SGI261731[] = {{&S49419[13], 4}, {NULL}};
const Segment SGI261732[] = {{&S49419[104], 5}, {NULL}};
const Segment SGI261733[] = {{&S49419[29], 3}, {NULL}};
const Segment SGI261734[] = {{&S49419[67], 8}, {NULL}};
const Segment SGI261735[] = {{&S49419[117], 5}, {NULL}};
const Segment SGI261736[] = {{&S49419[130], 9}, {NULL}};
const Segment SGI261737[] = {{&S49420[13], 4}, {NULL}};
const Segment SGI261738[] = {{&S49420[106], 5}, {NULL}};
const Segment SGI261739[] = {{&S49420[29], 5}, {NULL}};
const Segment SGI261740[] = {{&S49420[69], 8}, {NULL}};
const Segment SGI261741[] = {{&S49420[119], 5}, {NULL}};
const Segment SGI261742[] = {{&S49420[132], 9}, {NULL}};
const Segment SGI261743[] = {{&S49421[13], 4}, {NULL}};
const Segment SGI261744[] = {{&S49421[103], 5}, {NULL}};
const Segment SGI261745[] = {{&S49421[29], 3}, {NULL}};
const Segment SGI261746[] = {{&S49421[67], 7}, {NULL}};
const Segment SGI261747[] = {{&S49421[116], 3}, {NULL}};
const Segment SGI261748[] = {{&S49421[127], 9}, {NULL}};
const Segment SGI261749[] = {{&S49422[13], 4}, {NULL}};
const Segment SGI261750[] = {{&S49422[26], 5}, {NULL}};
const Segment SGI261751[] = {{&S49422[59], 6}, {NULL}};
const Segment SGI261752[] = {{&S49422[93], 6}, {NULL}};
const Segment SGI261753[] = {{&S49422[107], 9}, {NULL}};
const Segment SGI261754[] = {{&S49423[13], 4}, {NULL}};
const Segment SGI261755[] = {{&S49423[105], 5}, {NULL}};
const Segment SGI261756[] = {{&S49423[29], 5}, {NULL}};
const Segment SGI261757[] = {{&S49423[69], 7}, {NULL}};
const Segment SGI261758[] = {{&S49423[118], 3}, {NULL}};
const Segment SGI261759[] = {{&S49423[129], 9}, {NULL}};
const Segment SGI261760[] = {{&S49424[25], 4}, {NULL}};
const Segment SGI261761[] = {{&S49424[118], 5}, {NULL}};
const Segment SGI261762[] = {{&S49424[41], 5}, {NULL}};
const Segment SGI261763[] = {{&S49424[81], 8}, {NULL}};
const Segment SGI261764[] = {{&S49424[131], 5}, {&S49424[43], 1}, {NULL}};
const Segment SGI261765[] = {{&S49424[164], 9}, {NULL}};
const Segment SGI261766[] = {{&S49425[13], 4}, {NULL}};
const Segment SGI261767[] = {{&S49425[106], 5}, {NULL}};
const Segment SGI261768[] = {{&S49425[29], 5}, {NULL}};
const Segment SGI261769[] = {{&S49425[69], 8}, {NULL}};
const Segment SGI261770[] = {{&S49425[119], 5}, {NULL}};
const Segment SGI261771[] = {{&S49425[132], 9}, {NULL}};
const Segment SGI261772[] = {{&S49426[13], 4}, {NULL}};
const Segment SGI261773[] = {{&S49426[105], 5}, {NULL}};
const Segment SGI261774[] = {{&S49426[29], 5}, {NULL}};
const Segment SGI261775[] = {{&S49426[69], 7}, {NULL}};
const Segment SGI261776[] = {{&S49426[118], 3}, {NULL}};
const Segment SGI261777[] = {{&S49426[129], 9}, {NULL}};
const Segment SGI261778[] = {{&S49427[13], 4}, {NULL}};
const Segment SGI261779[] = {{&S49427[106], 5}, {NULL}};
const Segment SGI261780[] = {{&S49427[29], 5}, {NULL}};
const Segment SGI261781[] = {{&S49427[69], 8}, {NULL}};
const Segment SGI261782[] = {{&S49427[119], 5}, {&S49427[31], 1}, {NULL}};
const Segment SGI261783[] = {{&S49427[152], 9}, {NULL}};
const Segment SGI261784[] = {{&S49428[13], 4}, {&S49428[13], 4}, {NULL}};
const Segment SGI261785[] = {{&S49428[118], 5}, {NULL}};
const Segment SGI261786[] = {{&S49428[29], 3}, {NULL}};
const Segment SGI261787[] = {{&S49428[81], 8}, {NULL}};
const Segment SGI261788[] = {{&S49428[131], 5}, {NULL}};
const Segment SGI261789[] = {{&S49428[144], 9}, {NULL}};
const Segment SGI261790[] = {{&S49429[13], 4}, {NULL}};
const Segment SGI261791[] = {{&S49429[26], 5}, {NULL}};
const Segment SGI261792[] = {{&S49429[59], 6}, {NULL}};
const Segment SGI261793[] = {{&S49429[93], 6}, {NULL}};
const Segment SGI261794[] = {{&S49429[107], 9}, {NULL}};
const Segment SGI261795[] = {{&S49558[13], 4}, {NULL}};
const Segment SGI261796[] = {{&S49558[26], 5}, {NULL}};
const Segment SGI261797[] = {{&S49558[59], 6}, {NULL}};
const Segment SGI261798[] = {{&S49558[112], 5}, {NULL}};
const Segment SGI261799[] = {{&S49558[125], 9}, {NULL}};
const Segment SGI261800[] = {{&S49559[13], 4}, {NULL}};
const Segment SGI261801[] = {{&S49559[26], 3}, {NULL}};
const Segment SGI261802[] = {{&S49559[57], 6}, {NULL}};
const Segment SGI261803[] = {{&S49559[110], 5}, {NULL}};
const Segment SGI261804[] = {{&S49559[123], 9}, {NULL}};
const Segment SGI261805[] = {{&S40403[79], 3}, {NULL}};
const Segment SGI261806[] = {{&S40403[0], 87}, {NULL}};
const Segment SGI261807[] = {{&S40745[7], 5}, {NULL}};
const Segment SGI261808[] = {{&S40745[0], 18}, {NULL}};
const Segment SGI261809[] = {{&S49604[13], 6}, {NULL}};
const Segment SGI261810[] = {{&S49604[101], 5}, {&S49604[129], 3}, {NULL}};
const Segment SGI261811[] = {{&S49604[35], 3}, {NULL}};
const Segment SGI261812[] = {{&S49604[66], 6}, {NULL}};
const Segment SGI261813[] = {{&S49604[114], 6}, {NULL}};
const Segment SGI261814[] = {{&S49604[140], 7}, {&S49604[155], 9}, {NULL}};
const Segment SGI261815[] = {{&S49605[13], 6}, {NULL}};
const Segment SGI261816[] = {{&S49605[101], 5}, {&S49605[129], 3}, {NULL}};
const Segment SGI261817[] = {{&S49605[35], 3}, {NULL}};
const Segment SGI261818[] = {{&S49605[66], 6}, {NULL}};
const Segment SGI261819[] = {{&S49605[114], 6}, {NULL}};
const Segment SGI261820[] = {{&S49605[140], 7}, {&S49605[155], 9}, {NULL}};
const Segment SGI261821[] = {{&S49606[13], 6}, {NULL}};
const Segment SGI261822[] = {{&S49606[103], 5}, {&S49606[131], 3}, {NULL}};
const Segment SGI261823[] = {{&S49606[35], 5}, {NULL}};
const Segment SGI261824[] = {{&S49606[68], 6}, {NULL}};
const Segment SGI261825[] = {{&S49606[116], 6}, {NULL}};
const Segment SGI261826[] = {{&S49606[142], 7}, {&S49606[157], 9}, {NULL}};
const Segment SGI261827[] = {{&S49607[13], 6}, {NULL}};
const Segment SGI261828[] = {{&S49607[112], 5}, {NULL}};
const Segment SGI261829[] = {{&S49607[38], 3}, {NULL}};
const Segment SGI261830[] = {{&S49607[76], 7}, {NULL}};
const Segment SGI261831[] = {{&S49607[125], 6}, {NULL}};
const Segment SGI261832[] = {{&S49607[139], 9}, {NULL}};
const Segment SGI261833[] = {{&S49608[25], 6}, {NULL}};
const Segment SGI261834[] = {{&S49608[127], 5}, {NULL}};
const Segment SGI261835[] = {{&S49608[50], 5}, {NULL}};
const Segment SGI261836[] = {{&S49608[90], 8}, {NULL}};
const Segment SGI261837[] = {{&S49608[140], 5}, {&S49608[52], 1}, {NULL}};
const Segment SGI261838[] = {{&S49608[173], 9}, {NULL}};
const Segment SGI261839[] = {{&S49609[13], 6}, {NULL}};
const Segment SGI261840[] = {{&S49609[114], 5}, {NULL}};
const Segment SGI261841[] = {{&S49609[38], 5}, {NULL}};
const Segment SGI261842[] = {{&S49609[78], 7}, {NULL}};
const Segment SGI261843[] = {{&S49609[127], 6}, {NULL}};
const Segment SGI261844[] = {{&S49609[141], 9}, {NULL}};
const Segment SGI261845[] = {{&S49610[13], 6}, {NULL}};
const Segment SGI261846[] = {{&S49610[101], 5}, {&S49610[129], 3}, {NULL}};
const Segment SGI261847[] = {{&S49610[35], 3}, {NULL}};
const Segment SGI261848[] = {{&S49610[66], 6}, {NULL}};
const Segment SGI261849[] = {{&S49610[114], 6}, {NULL}};
const Segment SGI261850[] = {{&S49610[140], 7}, {&S49610[155], 9}, {NULL}};
const Segment SGI261851[] = {{&S49611[13], 6}, {NULL}};
const Segment SGI261852[] = {{&S49611[103], 5}, {&S49611[131], 3}, {NULL}};
const Segment SGI261853[] = {{&S49611[35], 5}, {NULL}};
const Segment SGI261854[] = {{&S49611[68], 6}, {NULL}};
const Segment SGI261855[] = {{&S49611[116], 6}, {NULL}};
const Segment SGI261856[] = {{&S49611[142], 7}, {&S49611[157], 9}, {NULL}};
const Segment SGI261857[] = {{&S49612[13], 6}, {NULL}};
const Segment SGI261858[] = {{&S49612[101], 5}, {&S49612[129], 3}, {NULL}};
const Segment SGI261859[] = {{&S49612[35], 3}, {NULL}};
const Segment SGI261860[] = {{&S49612[66], 6}, {NULL}};
const Segment SGI261861[] = {{&S49612[114], 6}, {NULL}};
const Segment SGI261862[] = {{&S49612[140], 7}, {&S49612[155], 9}, {NULL}};
const Segment SGI261863[] = {{&S49613[13], 6}, {NULL}};
const Segment SGI261864[] = {{&S49613[114], 5}, {NULL}};
const Segment SGI261865[] = {{&S49613[38], 5}, {NULL}};
const Segment SGI261866[] = {{&S49613[78], 7}, {NULL}};
const Segment SGI261867[] = {{&S49613[127], 6}, {NULL}};
const Segment SGI261868[] = {{&S49613[141], 9}, {NULL}};
const Segment SGI261869[] = {{&S49614[13], 6}, {NULL}};
const Segment SGI261870[] = {{&S49614[115], 5}, {NULL}};
const Segment SGI261871[] = {{&S49614[38], 5}, {NULL}};
const Segment SGI261872[] = {{&S49614[78], 8}, {NULL}};
const Segment SGI261873[] = {{&S49614[128], 5}, {&S49614[40], 1}, {NULL}};
const Segment SGI261874[] = {{&S49614[161], 9}, {NULL}};
const Segment SGI261875[] = {{&S49615[13], 6}, {NULL}};
const Segment SGI261876[] = {{&S49615[115], 5}, {NULL}};
const Segment SGI261877[] = {{&S49615[38], 5}, {NULL}};
const Segment SGI261878[] = {{&S49615[78], 8}, {NULL}};
const Segment SGI261879[] = {{&S49615[128], 5}, {&S49615[40], 1}, {NULL}};
const Segment SGI261880[] = {{&S49615[161], 9}, {NULL}};
const Segment SGI261881[] = {{&S49616[13], 6}, {NULL}};
const Segment SGI261882[] = {{&S49616[103], 5}, {&S49616[131], 3}, {NULL}};
const Segment SGI261883[] = {{&S49616[35], 5}, {NULL}};
const Segment SGI261884[] = {{&S49616[68], 6}, {NULL}};
const Segment SGI261885[] = {{&S49616[116], 6}, {NULL}};
const Segment SGI261886[] = {{&S49616[142], 7}, {&S49616[157], 9}, {NULL}};
const Segment SGI261887[] = {{&S49617[13], 6}, {NULL}};
const Segment SGI261888[] = {{&S49617[103], 5}, {&S49617[131], 3}, {NULL}};
const Segment SGI261889[] = {{&S49617[35], 5}, {NULL}};
const Segment SGI261890[] = {{&S49617[68], 6}, {NULL}};
const Segment SGI261891[] = {{&S49617[116], 6}, {NULL}};
const Segment SGI261892[] = {{&S49617[142], 7}, {&S49617[157], 9}, {NULL}};
const Segment SGI261893[] = {{&S49618[13], 6}, {NULL}};
const Segment SGI261894[] = {{&S49618[114], 5}, {NULL}};
const Segment SGI261895[] = {{&S49618[38], 5}, {NULL}};
const Segment SGI261896[] = {{&S49618[78], 7}, {NULL}};
const Segment SGI261897[] = {{&S49618[127], 6}, {NULL}};
const Segment SGI261898[] = {{&S49618[141], 9}, {NULL}};
const Segment SGI261899[] = {{&S49619[13], 6}, {NULL}};
const Segment SGI261900[] = {{&S49619[103], 5}, {NULL}};
const Segment SGI261901[] = {{&S49619[35], 5}, {NULL}};
const Segment SGI261902[] = {{&S49619[68], 6}, {NULL}};
const Segment SGI261903[] = {{&S49619[116], 6}, {NULL}};
const Segment SGI261904[] = {{&S49619[130], 9}, {NULL}};
const Segment SGI261905[] = {{&S49620[13], 6}, {NULL}};
const Segment SGI261906[] = {{&S49620[38], 5}, {NULL}};
const Segment SGI261907[] = {{&S49620[113], 5}, {NULL}};
const Segment SGI261908[] = {{&S49620[126], 9}, {NULL}};
const Segment SGI261909[] = {{&S40886[80], 9}, {NULL}};
const Segment SGI261910[] = {{&S40886[0], 89}, {NULL}};
const Segment SGI261911[] = {{&S40887[78], 10}, {NULL}};
const Segment SGI261912[] = {{&S40887[0], 88}, {NULL}};
const Segment SGI261913[] = {{&S49745[13], 4}, {NULL}};
const Segment SGI261914[] = {{&S49745[92], 3}, {NULL}};
const Segment SGI261915[] = {{&S49745[26], 3}, {NULL}};
const Segment SGI261916[] = {{&S49745[57], 6}, {NULL}};
const Segment SGI261917[] = {{&S49745[103], 5}, {NULL}};
const Segment SGI261918[] = {{&S49745[116], 9}, {NULL}};
const Segment SGI261919[] = {{&S49746[13], 4}, {NULL}};
const Segment SGI261920[] = {{&S49746[94], 3}, {NULL}};
const Segment SGI261921[] = {{&S49746[26], 3}, {NULL}};
const Segment SGI261922[] = {{&S49746[57], 8}, {NULL}};
const Segment SGI261923[] = {{&S49746[105], 5}, {NULL}};
const Segment SGI261924[] = {{&S49746[118], 9}, {NULL}};
const Segment SGI261925[] = {{&S49747[13], 4}, {NULL}};
const Segment SGI261926[] = {{&S49747[115], 3}, {NULL}};
const Segment SGI261927[] = {{&S49747[26], 3}, {NULL}};
const Segment SGI261928[] = {{&S49747[57], 8}, {NULL}};
const Segment SGI261929[] = {{&S49747[93], 6}, {NULL}};
const Segment SGI261930[] = {{&S49748[13], 4}, {NULL}};
const Segment SGI261931[] = {{&S49748[94], 3}, {NULL}};
const Segment SGI261932[] = {{&S49748[26], 3}, {NULL}};
const Segment SGI261933[] = {{&S49748[57], 8}, {NULL}};
const Segment SGI261934[] = {{&S49748[105], 6}, {NULL}};
const Segment SGI261935[] = {{&S49748[119], 9}, {NULL}};
const Segment SGI261936[] = {{&S49749[13], 4}, {NULL}};
const Segment SGI261937[] = {{&S49749[108], 3}, {NULL}};
const Segment SGI261938[] = {{&S49749[26], 3}, {NULL}};
const Segment SGI261939[] = {{&S49749[57], 8}, {NULL}};
const Segment SGI261940[] = {{&S49749[93], 6}, {NULL}};
const Segment SGI261941[] = {{&S49749[119], 9}, {NULL}};
const Segment SGI261942[] = {{&S49750[13], 4}, {NULL}};
const Segment SGI261943[] = {{&S49750[96], 3}, {NULL}};
const Segment SGI261944[] = {{&S49750[26], 5}, {NULL}};
const Segment SGI261945[] = {{&S49750[59], 8}, {NULL}};
const Segment SGI261946[] = {{&S49750[107], 5}, {NULL}};
const Segment SGI261947[] = {{&S49750[120], 9}, {NULL}};
const Segment SGI261948[] = {{&S49751[13], 4}, {NULL}};
const Segment SGI261949[] = {{&S49751[117], 3}, {NULL}};
const Segment SGI261950[] = {{&S49751[26], 5}, {NULL}};
const Segment SGI261951[] = {{&S49751[59], 8}, {NULL}};
const Segment SGI261952[] = {{&S49751[95], 6}, {NULL}};
const Segment SGI261953[] = {{&S49752[13], 4}, {NULL}};
const Segment SGI261954[] = {{&S49752[26], 3}, {NULL}};
const Segment SGI261955[] = {{&S49752[57], 8}, {NULL}};
const Segment SGI261956[] = {{&S49752[93], 6}, {NULL}};
const Segment SGI261957[] = {{&S49752[107], 9}, {NULL}};
const Segment SGI261958[] = {{&S49753[13], 4}, {NULL}};
const Segment SGI261959[] = {{&S49753[110], 3}, {NULL}};
const Segment SGI261960[] = {{&S49753[26], 5}, {NULL}};
const Segment SGI261961[] = {{&S49753[59], 8}, {NULL}};
const Segment SGI261962[] = {{&S49753[95], 6}, {NULL}};
const Segment SGI261963[] = {{&S49753[121], 9}, {NULL}};
const Segment SGI261964[] = {{&S49754[13], 4}, {NULL}};
const Segment SGI261965[] = {{&S49754[103], 3}, {NULL}};
const Segment SGI261966[] = {{&S49754[26], 1}, {NULL}};
const Segment SGI261967[] = {{&S49754[55], 6}, {NULL}};
const Segment SGI261968[] = {{&S49754[89], 5}, {NULL}};
const Segment SGI261969[] = {{&S49754[114], 9}, {NULL}};
const Segment SGI261970[] = {{&S49765[13], 4}, {NULL}};
const Segment SGI261971[] = {{&S49765[26], 5}, {NULL}};
const Segment SGI261972[] = {{&S49765[59], 6}, {NULL}};
const Segment SGI261973[] = {{&S49765[105], 5}, {NULL}};
const Segment SGI261974[] = {{&S49765[118], 9}, {NULL}};
const Segment SGI261975[] = {{&S49766[13], 4}, {NULL}};
const Segment SGI261976[] = {{&S49766[26], 3}, {NULL}};
const Segment SGI261977[] = {{&S49766[57], 6}, {NULL}};
const Segment SGI261978[] = {{&S49766[103], 5}, {NULL}};
const Segment SGI261979[] = {{&S49766[116], 9}, {NULL}};
const Segment SGI261980[] = {{&S49800[13], 4}, {NULL}};
const Segment SGI261981[] = {{&S49800[26], 3}, {NULL}};
const Segment SGI261982[] = {{&S49800[57], 6}, {NULL}};
const Segment SGI261983[] = {{&S49800[110], 8}, {NULL}};
const Segment SGI261984[] = {{&S49800[126], 7}, {NULL}};
const Segment SGI261985[] = {{&S49801[13], 4}, {NULL}};
const Segment SGI261986[] = {{&S49801[26], 3}, {NULL}};
const Segment SGI261987[] = {{&S49801[57], 6}, {NULL}};
const Segment SGI261988[] = {{&S49801[110], 8}, {NULL}};
const Segment SGI261989[] = {{&S49801[126], 7}, {NULL}};
const Segment SGI261990[] = {{&S49802[13], 4}, {NULL}};
const Segment SGI261991[] = {{&S49802[26], 3}, {NULL}};
const Segment SGI261992[] = {{&S49802[57], 6}, {NULL}};
const Segment SGI261993[] = {{&S49802[110], 8}, {NULL}};
const Segment SGI261994[] = {{&S49802[126], 7}, {NULL}};
const Segment SGI261995[] = {{&S49803[13], 4}, {NULL}};
const Segment SGI261996[] = {{&S49803[26], 3}, {NULL}};
const Segment SGI261997[] = {{&S49803[57], 6}, {NULL}};
const Segment SGI261998[] = {{&S49803[110], 6}, {NULL}};
const Segment SGI261999[] = {{&S49803[124], 7}, {NULL}};
const Segment SGI262000[] = {{&S49804[13], 4}, {NULL}};
const Segment SGI262001[] = {{&S49804[26], 5}, {NULL}};
const Segment SGI262002[] = {{&S49804[59], 6}, {NULL}};
const Segment SGI262003[] = {{&S49804[112], 6}, {NULL}};
const Segment SGI262004[] = {{&S49804[126], 7}, {NULL}};
const Segment SGI262005[] = {{&S49805[13], 6}, {NULL}};
const Segment SGI262006[] = {{&S49805[113], 5}, {NULL}};
const Segment SGI262007[] = {{&S49805[38], 3}, {NULL}};
const Segment SGI262008[] = {{&S49805[76], 8}, {NULL}};
const Segment SGI262009[] = {{&S49805[126], 5}, {NULL}};
const Segment SGI262010[] = {{&S49805[139], 6}, {NULL}};
const Segment SGI262011[] = {{&S49806[13], 6}, {NULL}};
const Segment SGI262012[] = {{&S49806[113], 5}, {NULL}};
const Segment SGI262013[] = {{&S49806[38], 3}, {NULL}};
const Segment SGI262014[] = {{&S49806[76], 8}, {NULL}};
const Segment SGI262015[] = {{&S49806[126], 5}, {NULL}};
const Segment SGI262016[] = {{&S49806[139], 6}, {NULL}};
const Segment SGI262017[] = {{&S49807[13], 6}, {NULL}};
const Segment SGI262018[] = {{&S49807[113], 5}, {NULL}};
const Segment SGI262019[] = {{&S49807[38], 3}, {NULL}};
const Segment SGI262020[] = {{&S49807[76], 8}, {NULL}};
const Segment SGI262021[] = {{&S49807[126], 5}, {NULL}};
const Segment SGI262022[] = {{&S49807[139], 7}, {NULL}};
const Segment SGI262023[] = {{&S49808[13], 6}, {NULL}};
const Segment SGI262024[] = {{&S49808[115], 5}, {NULL}};
const Segment SGI262025[] = {{&S49808[38], 5}, {NULL}};
const Segment SGI262026[] = {{&S49808[78], 8}, {NULL}};
const Segment SGI262027[] = {{&S49808[128], 3}, {NULL}};
const Segment SGI262028[] = {{&S49808[139], 6}, {NULL}};
const Segment SGI262029[] = {{&S49809[13], 6}, {NULL}};
const Segment SGI262030[] = {{&S49809[115], 5}, {NULL}};
const Segment SGI262031[] = {{&S49809[38], 5}, {NULL}};
const Segment SGI262032[] = {{&S49809[78], 8}, {NULL}};
const Segment SGI262033[] = {{&S49809[128], 5}, {NULL}};
const Segment SGI262034[] = {{&S49809[141], 6}, {NULL}};
const Segment SGI262035[] = {{&S49810[13], 6}, {NULL}};
const Segment SGI262036[] = {{&S49810[112], 5}, {NULL}};
const Segment SGI262037[] = {{&S49810[38], 3}, {NULL}};
const Segment SGI262038[] = {{&S49810[76], 7}, {NULL}};
const Segment SGI262039[] = {{&S49810[125], 5}, {NULL}};
const Segment SGI262040[] = {{&S49810[138], 6}, {NULL}};
const Segment SGI262041[] = {{&S49811[26], 6}, {NULL}};
const Segment SGI262042[] = {{&S49811[129], 5}, {NULL}};
const Segment SGI262043[] = {{&S49811[51], 5}, {NULL}};
const Segment SGI262044[] = {{&S49811[92], 8}, {NULL}};
const Segment SGI262045[] = {{&S49811[142], 4}, {NULL}};
const Segment SGI262046[] = {{&S49811[154], 6}, {NULL}};
const Segment SGI262047[] = {{&S49812[14], 6}, {NULL}};
const Segment SGI262048[] = {{&S49812[108], 5}, {NULL}};
const Segment SGI262049[] = {{&S49812[33], 3}, {NULL}};
const Segment SGI262050[] = {{&S49812[71], 8}, {NULL}};
const Segment SGI262051[] = {{&S49812[121], 5}, {NULL}};
const Segment SGI262052[] = {{&S49812[134], 6}, {NULL}};
const Segment SGI262053[] = {{&S41068[69], 12}, {NULL}};
const Segment SGI262054[] = {{&S41068[0], 102}, {NULL}};
const Segment SGI262055[] = {{&S41069[64], 12}, {NULL}};
const Segment SGI262056[] = {{&S41069[0], 76}, {NULL}};
const Segment SGI262057[] = {{&S41070[69], 4}, {&S41070[83], 30}, {NULL}};
const Segment SGI262058[] = {{&S41070[0], 113}, {NULL}};
const Segment SGI262059[] = {{&S41071[51], 4}, {&S41071[65], 30}, {NULL}};
const Segment SGI262060[] = {{&S41071[0], 95}, {NULL}};
const Segment SGI262061[] = {{&S49883[13], 4}, {NULL}};
const Segment SGI262062[] = {{&S49883[33], 5}, {NULL}};
const Segment SGI262063[] = {{&S49883[66], 6}, {NULL}};
const Segment SGI262064[] = {{&S49883[119], 6}, {NULL}};
const Segment SGI262065[] = {{&S49883[133], 7}, {NULL}};
const Segment SGI262066[] = {{&S49884[13], 4}, {NULL}};
const Segment SGI262067[] = {{&S49884[33], 3}, {NULL}};
const Segment SGI262068[] = {{&S49884[64], 6}, {NULL}};
const Segment SGI262069[] = {{&S49884[117], 6}, {NULL}};
const Segment SGI262070[] = {{&S49884[131], 7}, {NULL}};
const Segment SGI262071[] = {{&S49885[13], 4}, {NULL}};
const Segment SGI262072[] = {{&S49885[33], 3}, {NULL}};
const Segment SGI262073[] = {{&S49885[64], 6}, {NULL}};
const Segment SGI262074[] = {{&S49885[117], 6}, {NULL}};
const Segment SGI262075[] = {{&S49885[131], 7}, {NULL}};
const Segment SGI262076[] = {{&S41668[0], 5}, {NULL}};
const Segment SGI262077[] = {{&S41669[33], 5}, {NULL}};
const Segment SGI262078[] = {{&S41669[0], 80}, {NULL}};
const Segment SGI262079[] = {{&S50302[13], 6}, {NULL}};
const Segment SGI262080[] = {{&S50302[35], 5}, {NULL}};
const Segment SGI262081[] = {{&S50302[68], 6}, {NULL}};
const Segment SGI262082[] = {{&S50302[121], 6}, {NULL}};
const Segment SGI262083[] = {{&S50302[135], 7}, {NULL}};
const Segment SGI262084[] = {{&S50303[13], 6}, {NULL}};
const Segment SGI262085[] = {{&S50303[35], 3}, {NULL}};
const Segment SGI262086[] = {{&S50303[66], 6}, {NULL}};
const Segment SGI262087[] = {{&S50303[119], 6}, {NULL}};
const Segment SGI262088[] = {{&S50303[133], 7}, {NULL}};
const Segment SGI262089[] = {{&S50304[13], 6}, {NULL}};
const Segment SGI262090[] = {{&S50304[35], 5}, {NULL}};
const Segment SGI262091[] = {{&S50304[68], 6}, {NULL}};
const Segment SGI262092[] = {{&S50304[121], 6}, {NULL}};
const Segment SGI262093[] = {{&S50304[135], 7}, {NULL}};
const Segment SGI262094[] = {{&S50305[13], 6}, {NULL}};
const Segment SGI262095[] = {{&S50305[35], 5}, {NULL}};
const Segment SGI262096[] = {{&S50305[68], 6}, {NULL}};
const Segment SGI262097[] = {{&S50305[121], 6}, {NULL}};
const Segment SGI262098[] = {{&S50305[135], 7}, {NULL}};
const Segment SGI262099[] = {{&S50306[13], 6}, {NULL}};
const Segment SGI262100[] = {{&S50306[35], 3}, {NULL}};
const Segment SGI262101[] = {{&S50306[66], 6}, {NULL}};
const Segment SGI262102[] = {{&S50306[119], 6}, {NULL}};
const Segment SGI262103[] = {{&S50306[133], 7}, {NULL}};
const Segment SGI262104[] = {{&S50359[13], 6}, {NULL}};
const Segment SGI262105[] = {{&S50359[35], 5}, {NULL}};
const Segment SGI262106[] = {{&S50359[68], 6}, {NULL}};
const Segment SGI262107[] = {{&S50359[121], 5}, {NULL}};
const Segment SGI262108[] = {{&S50359[134], 9}, {NULL}};
const Segment SGI262109[] = {{&S50360[13], 6}, {NULL}};
const Segment SGI262110[] = {{&S50360[35], 5}, {NULL}};
const Segment SGI262111[] = {{&S50360[68], 6}, {NULL}};
const Segment SGI262112[] = {{&S50360[121], 5}, {NULL}};
const Segment SGI262113[] = {{&S50360[134], 9}, {NULL}};
const Segment SGI262114[] = {{&S50361[13], 6}, {NULL}};
const Segment SGI262115[] = {{&S50361[35], 3}, {NULL}};
const Segment SGI262116[] = {{&S50361[66], 6}, {NULL}};
const Segment SGI262117[] = {{&S50361[119], 5}, {NULL}};
const Segment SGI262118[] = {{&S50361[132], 9}, {NULL}};
const Segment SGI262119[] = {{&S50363[13], 6}, {NULL}};
const Segment SGI262120[] = {{&S50363[35], 3}, {NULL}};
const Segment SGI262121[] = {{&S50363[66], 6}, {NULL}};
const Segment SGI262122[] = {{&S50363[112], 6}, {NULL}};
const Segment SGI262123[] = {{&S50363[126], 7}, {NULL}};
const Segment SGI262124[] = {{&S50437[13], 6}, {NULL}};
const Segment SGI262125[] = {{&S50437[122], 3}, {NULL}};
const Segment SGI262126[] = {{&S50437[35], 3}, {NULL}};
const Segment SGI262127[] = {{&S50437[66], 6}, {NULL}};
const Segment SGI262128[] = {{&S50437[100], 6}, {NULL}};
const Segment SGI262129[] = {{&S50438[13], 6}, {NULL}};
const Segment SGI262130[] = {{&S50438[122], 3}, {NULL}};
const Segment SGI262131[] = {{&S50438[35], 3}, {NULL}};
const Segment SGI262132[] = {{&S50438[66], 6}, {NULL}};
const Segment SGI262133[] = {{&S50438[100], 6}, {NULL}};
const Segment SGI262134[] = {{&S50439[13], 6}, {NULL}};
const Segment SGI262135[] = {{&S50439[122], 3}, {NULL}};
const Segment SGI262136[] = {{&S50439[35], 3}, {NULL}};
const Segment SGI262137[] = {{&S50439[66], 6}, {NULL}};
const Segment SGI262138[] = {{&S50439[100], 6}, {NULL}};
const Segment SGI262139[] = {{&S50440[13], 6}, {NULL}};
const Segment SGI262140[] = {{&S50440[122], 3}, {NULL}};
const Segment SGI262141[] = {{&S50440[35], 3}, {NULL}};
const Segment SGI262142[] = {{&S50440[66], 6}, {NULL}};
const Segment SGI262143[] = {{&S50440[100], 6}, {NULL}};
const Segment SGI262144[] = {{&S50441[13], 6}, {NULL}};
const Segment SGI262145[] = {{&S50441[124], 3}, {NULL}};
const Segment SGI262146[] = {{&S50441[35], 5}, {NULL}};
const Segment SGI262147[] = {{&S50441[68], 6}, {NULL}};
const Segment SGI262148[] = {{&S50441[102], 6}, {NULL}};
const Segment SGI262149[] = {{&S50442[13], 6}, {NULL}};
const Segment SGI262150[] = {{&S50442[124], 3}, {NULL}};
const Segment SGI262151[] = {{&S50442[35], 5}, {NULL}};
const Segment SGI262152[] = {{&S50442[68], 6}, {NULL}};
const Segment SGI262153[] = {{&S50442[102], 6}, {NULL}};
const Segment SGI262154[] = {{&S50443[13], 6}, {NULL}};
const Segment SGI262155[] = {{&S50443[96], 3}, {NULL}};
const Segment SGI262156[] = {{&S50443[28], 5}, {NULL}};
const Segment SGI262157[] = {{&S50443[61], 6}, {NULL}};
const Segment SGI262158[] = {{&S50443[107], 6}, {NULL}};
const Segment SGI262159[] = {{&S50443[121], 7}, {NULL}};
const Segment SGI262160[] = {{&S50444[13], 6}, {NULL}};
const Segment SGI262161[] = {{&S50444[124], 3}, {NULL}};
const Segment SGI262162[] = {{&S50444[35], 5}, {NULL}};
const Segment SGI262163[] = {{&S50444[68], 6}, {NULL}};
const Segment SGI262164[] = {{&S50444[102], 6}, {NULL}};
const Segment SGI262165[] = {{&S50445[13], 6}, {NULL}};
const Segment SGI262166[] = {{&S50445[96], 3}, {NULL}};
const Segment SGI262167[] = {{&S50445[28], 5}, {NULL}};
const Segment SGI262168[] = {{&S50445[61], 6}, {NULL}};
const Segment SGI262169[] = {{&S50445[107], 6}, {NULL}};
const Segment SGI262170[] = {{&S50445[121], 7}, {NULL}};
const Segment SGI262171[] = {{&S50446[13], 6}, {NULL}};
const Segment SGI262172[] = {{&S50446[124], 3}, {NULL}};
const Segment SGI262173[] = {{&S50446[35], 5}, {NULL}};
const Segment SGI262174[] = {{&S50446[68], 6}, {NULL}};
const Segment SGI262175[] = {{&S50446[102], 6}, {NULL}};
const Segment SGI262176[] = {{&S50447[13], 6}, {NULL}};
const Segment SGI262177[] = {{&S50447[103], 3}, {NULL}};
const Segment SGI262178[] = {{&S50447[35], 5}, {NULL}};
const Segment SGI262179[] = {{&S50447[68], 6}, {NULL}};
const Segment SGI262180[] = {{&S50447[114], 6}, {NULL}};
const Segment SGI262181[] = {{&S50447[128], 9}, {NULL}};
const Segment SGI262182[] = {{&S50448[13], 6}, {NULL}};
const Segment SGI262183[] = {{&S50448[96], 3}, {NULL}};
const Segment SGI262184[] = {{&S50448[28], 5}, {NULL}};
const Segment SGI262185[] = {{&S50448[61], 6}, {NULL}};
const Segment SGI262186[] = {{&S50448[107], 6}, {NULL}};
const Segment SGI262187[] = {{&S50448[121], 7}, {NULL}};
const Segment SGI262188[] = {{&S50449[13], 6}, {NULL}};
const Segment SGI262189[] = {{&S50449[94], 3}, {NULL}};
const Segment SGI262190[] = {{&S50449[28], 3}, {NULL}};
const Segment SGI262191[] = {{&S50449[59], 6}, {NULL}};
const Segment SGI262192[] = {{&S50449[105], 6}, {NULL}};
const Segment SGI262193[] = {{&S50449[119], 7}, {NULL}};
const Segment SGI262194[] = {{&S50477[26], 4}, {NULL}};
const Segment SGI262195[] = {{&S50477[42], 5}, {NULL}};
const Segment SGI262196[] = {{&S50478[26], 4}, {NULL}};
const Segment SGI262197[] = {{&S50478[42], 5}, {NULL}};
const Segment SGI262198[] = {{&S50479[26], 4}, {NULL}};
const Segment SGI262199[] = {{&S50479[42], 5}, {NULL}};
const Segment SGI262200[] = {{&S50480[26], 4}, {NULL}};
const Segment SGI262201[] = {{&S50480[42], 5}, {NULL}};
const Segment SGI262202[] = {{&S50481[26], 4}, {NULL}};
const Segment SGI262203[] = {{&S50481[42], 5}, {NULL}};
const Segment SGI262204[] = {{&S50482[28], 4}, {NULL}};
const Segment SGI262205[] = {{&S50482[44], 3}, {NULL}};
const Segment SGI262206[] = {{&S50483[38], 4}, {NULL}};
const Segment SGI262207[] = {{&S50483[129], 5}, {NULL}};
const Segment SGI262208[] = {{&S50483[54], 3}, {NULL}};
const Segment SGI262209[] = {{&S50483[92], 8}, {NULL}};
const Segment SGI262210[] = {{&S50483[142], 5}, {NULL}};
const Segment SGI262211[] = {{&S50483[155], 9}, {NULL}};
const Segment SGI262212[] = {{&S50484[38], 4}, {NULL}};
const Segment SGI262213[] = {{&S50484[128], 5}, {NULL}};
const Segment SGI262214[] = {{&S50484[54], 3}, {NULL}};
const Segment SGI262215[] = {{&S50484[91], 8}, {NULL}};
const Segment SGI262216[] = {{&S50484[141], 5}, {NULL}};
const Segment SGI262217[] = {{&S50484[154], 9}, {NULL}};
const Segment SGI262218[] = {{&S50643[13], 4}, {NULL}};
const Segment SGI262219[] = {{&S50643[33], 3}, {NULL}};
const Segment SGI262220[] = {{&S50643[64], 8}, {NULL}};
const Segment SGI262221[] = {{&S50643[119], 6}, {NULL}};
const Segment SGI262222[] = {{&S50643[133], 7}, {NULL}};
const Segment SGI262223[] = {{&S50644[13], 4}, {NULL}};
const Segment SGI262224[] = {{&S50644[33], 3}, {NULL}};
const Segment SGI262225[] = {{&S50644[64], 8}, {NULL}};
const Segment SGI262226[] = {{&S50644[119], 8}, {NULL}};
const Segment SGI262227[] = {{&S50644[135], 7}, {NULL}};
const Segment SGI262228[] = {{&S50645[13], 4}, {NULL}};
const Segment SGI262229[] = {{&S50645[33], 5}, {NULL}};
const Segment SGI262230[] = {{&S50645[66], 8}, {NULL}};
const Segment SGI262231[] = {{&S50645[121], 6}, {NULL}};
const Segment SGI262232[] = {{&S50645[135], 7}, {NULL}};
const Segment SGI262233[] = {{&S50652[13], 4}, {NULL}};
const Segment SGI262234[] = {{&S50652[26], 3}, {NULL}};
const Segment SGI262235[] = {{&S50652[57], 6}, {NULL}};
const Segment SGI262236[] = {{&S50652[110], 5}, {NULL}};
const Segment SGI262237[] = {{&S50652[123], 9}, {NULL}};
const Segment SGI262238[] = {{&S50653[13], 4}, {NULL}};
const Segment SGI262239[] = {{&S50653[26], 5}, {NULL}};
const Segment SGI262240[] = {{&S50653[59], 6}, {NULL}};
const Segment SGI262241[] = {{&S50653[112], 5}, {NULL}};
const Segment SGI262242[] = {{&S50653[125], 9}, {NULL}};
const Segment SGI262243[] = {{&S42153[213], 9}, {NULL}};
const Segment SGI262244[] = {{&S42153[0], 222}, {NULL}};
const Segment SGI262245[] = {{&S50797[13], 6}, {NULL}};
const Segment SGI262246[] = {{&S50797[35], 5}, {NULL}};
const Segment SGI262247[] = {{&S50797[68], 6}, {NULL}};
const Segment SGI262248[] = {{&S50797[121], 6}, {NULL}};
const Segment SGI262249[] = {{&S50797[135], 7}, {NULL}};
const Segment SGI262250[] = {{&S50798[13], 6}, {NULL}};
const Segment SGI262251[] = {{&S50798[35], 5}, {NULL}};
const Segment SGI262252[] = {{&S50798[68], 6}, {NULL}};
const Segment SGI262253[] = {{&S50798[121], 6}, {NULL}};
const Segment SGI262254[] = {{&S50798[135], 7}, {NULL}};
const Segment SGI262255[] = {{&S50799[13], 6}, {NULL}};
const Segment SGI262256[] = {{&S50799[35], 3}, {NULL}};
const Segment SGI262257[] = {{&S50799[66], 6}, {NULL}};
const Segment SGI262258[] = {{&S50799[119], 6}, {NULL}};
const Segment SGI262259[] = {{&S50799[133], 7}, {NULL}};
const Segment SGI262260[] = {{&S50800[13], 6}, {NULL}};
const Segment SGI262261[] = {{&S50800[35], 3}, {NULL}};
const Segment SGI262262[] = {{&S50800[66], 6}, {NULL}};
const Segment SGI262263[] = {{&S50800[119], 6}, {NULL}};
const Segment SGI262264[] = {{&S50800[133], 7}, {NULL}};
const Segment SGI262265[] = {{&S50801[13], 6}, {NULL}};
const Segment SGI262266[] = {{&S50801[35], 5}, {NULL}};
const Segment SGI262267[] = {{&S50801[68], 6}, {NULL}};
const Segment SGI262268[] = {{&S50801[121], 6}, {NULL}};
const Segment SGI262269[] = {{&S50801[135], 7}, {NULL}};
const Segment SGI262270[] = {{&S50833[13], 6}, {NULL}};
const Segment SGI262271[] = {{&S50833[113], 5}, {NULL}};
const Segment SGI262272[] = {{&S50833[38], 3}, {NULL}};
const Segment SGI262273[] = {{&S50833[76], 8}, {NULL}};
const Segment SGI262274[] = {{&S50833[126], 5}, {NULL}};
const Segment SGI262275[] = {{&S50833[139], 6}, {NULL}};
const Segment SGI262276[] = {{&S50834[13], 6}, {NULL}};
const Segment SGI262277[] = {{&S50834[105], 5}, {NULL}};
const Segment SGI262278[] = {{&S50834[70], 5}, {NULL}};
const Segment SGI262279[] = {{&S50834[118], 5}, {NULL}};
const Segment SGI262280[] = {{&S50834[131], 8}, {NULL}};
const Segment SGI262281[] = {{&S50835[13], 6}, {NULL}};
const Segment SGI262282[] = {{&S50835[123], 5}, {NULL}};
const Segment SGI262283[] = {{&S50835[48], 3}, {NULL}};
const Segment SGI262284[] = {{&S50835[86], 8}, {NULL}};
const Segment SGI262285[] = {{&S50835[136], 5}, {NULL}};
const Segment SGI262286[] = {{&S50835[149], 6}, {NULL}};
const Segment SGI262287[] = {{&S50836[13], 6}, {NULL}};
const Segment SGI262288[] = {{&S50836[113], 5}, {NULL}};
const Segment SGI262289[] = {{&S50836[38], 3}, {NULL}};
const Segment SGI262290[] = {{&S50836[76], 8}, {NULL}};
const Segment SGI262291[] = {{&S50836[126], 5}, {NULL}};
const Segment SGI262292[] = {{&S50837[13], 6}, {NULL}};
const Segment SGI262293[] = {{&S50837[122], 5}, {NULL}};
const Segment SGI262294[] = {{&S50837[38], 2}, {NULL}};
const Segment SGI262295[] = {{&S50837[75], 8}, {NULL}};
const Segment SGI262296[] = {{&S50837[135], 5}, {NULL}};
const Segment SGI262297[] = {{&S50837[148], 6}, {NULL}};
const Segment SGI262298[] = {{&S50838[13], 6}, {NULL}};
const Segment SGI262299[] = {{&S50838[113], 5}, {NULL}};
const Segment SGI262300[] = {{&S50838[38], 3}, {NULL}};
const Segment SGI262301[] = {{&S50838[76], 8}, {NULL}};
const Segment SGI262302[] = {{&S50838[126], 4}, {NULL}};
const Segment SGI262303[] = {{&S50838[138], 6}, {NULL}};
const Segment SGI262304[] = {{&S50886[13], 4}, {NULL}};
const Segment SGI262305[] = {{&S50886[29], 5}, {NULL}};
const Segment SGI262306[] = {{&S50886[69], 8}, {NULL}};
const Segment SGI262307[] = {{&S50886[124], 5}, {NULL}};
const Segment SGI262308[] = {{&S50886[137], 9}, {NULL}};
const Segment SGI262309[] = {{&S50887[13], 4}, {NULL}};
const Segment SGI262310[] = {{&S50887[29], 3}, {NULL}};
const Segment SGI262311[] = {{&S50887[67], 8}, {NULL}};
const Segment SGI262312[] = {{&S50887[122], 5}, {NULL}};
const Segment SGI262313[] = {{&S50887[135], 9}, {NULL}};
const Segment SGI262314[] = {{&S50894[13], 6}, {&S50894[13], 6}, {NULL}};
const Segment SGI262315[] = {{&S50894[35], 3}, {NULL}};
const Segment SGI262316[] = {{&S50894[66], 6}, {NULL}};
const Segment SGI262317[] = {{&S50894[100], 6}, {NULL}};
const Segment SGI262318[] = {{&S50894[114], 9}, {NULL}};
const Segment SGI262319[] = {{&S50895[13], 6}, {&S50895[13], 6}, {NULL}};
const Segment SGI262320[] = {{&S50895[35], 5}, {NULL}};
const Segment SGI262321[] = {{&S50895[68], 6}, {NULL}};
const Segment SGI262322[] = {{&S50895[102], 6}, {NULL}};
const Segment SGI262323[] = {{&S50895[116], 9}, {NULL}};
const Segment SGI262324[] = {{&S50896[13], 6}, {&S50896[13], 6}, {NULL}};
const Segment SGI262325[] = {{&S50896[35], 5}, {NULL}};
const Segment SGI262326[] = {{&S50896[68], 6}, {NULL}};
const Segment SGI262327[] = {{&S50896[102], 6}, {NULL}};
const Segment SGI262328[] = {{&S50896[116], 9}, {NULL}};
const Segment SGI262329[] = {{&S50897[13], 6}, {&S50897[13], 6}, {NULL}};
const Segment SGI262330[] = {{&S50897[35], 3}, {NULL}};
const Segment SGI262331[] = {{&S50897[66], 6}, {NULL}};
const Segment SGI262332[] = {{&S50897[100], 6}, {NULL}};
const Segment SGI262333[] = {{&S50897[114], 9}, {NULL}};
const Segment SGI262334[] = {{&S50898[13], 6}, {&S50898[13], 6}, {NULL}};
const Segment SGI262335[] = {{&S50898[35], 5}, {NULL}};
const Segment SGI262336[] = {{&S50898[68], 6}, {NULL}};
const Segment SGI262337[] = {{&S50898[102], 6}, {NULL}};
const Segment SGI262338[] = {{&S50898[116], 9}, {NULL}};
const Segment SGI262339[] = {{&S50918[13], 6}, {NULL}};
const Segment SGI262340[] = {{&S50918[35], 5}, {NULL}};
const Segment SGI262341[] = {{&S50918[68], 6}, {NULL}};
const Segment SGI262342[] = {{&S50918[122], 5}, {NULL}};
const Segment SGI262343[] = {{&S50918[135], 9}, {NULL}};
const Segment SGI262344[] = {{&S50919[13], 6}, {NULL}};
const Segment SGI262345[] = {{&S50919[35], 3}, {NULL}};
const Segment SGI262346[] = {{&S50919[66], 6}, {NULL}};
const Segment SGI262347[] = {{&S50919[120], 5}, {NULL}};
const Segment SGI262348[] = {{&S50919[133], 9}, {NULL}};
const Segment SGI262349[] = {{&S50920[13], 6}, {NULL}};
const Segment SGI262350[] = {{&S50920[35], 5}, {NULL}};
const Segment SGI262351[] = {{&S50920[68], 6}, {NULL}};
const Segment SGI262352[] = {{&S50920[122], 5}, {NULL}};
const Segment SGI262353[] = {{&S50920[135], 9}, {NULL}};
const Segment SGI262354[] = {{&S50939[27], 4}, {NULL}};
const Segment SGI262355[] = {{&S50939[43], 5}, {NULL}};
const Segment SGI262356[] = {{&S50940[13], 4}, {NULL}};
const Segment SGI262357[] = {{&S50940[26], 5}, {NULL}};
const Segment SGI262358[] = {{&S50940[59], 6}, {NULL}};
const Segment SGI262359[] = {{&S50940[93], 6}, {NULL}};
const Segment SGI262360[] = {{&S50940[100], 15}, {NULL}};
const Segment SGI262361[] = {{&S50941[13], 4}, {NULL}};
const Segment SGI262362[] = {{&S50941[26], 3}, {NULL}};
const Segment SGI262363[] = {{&S50941[57], 6}, {NULL}};
const Segment SGI262364[] = {{&S50941[91], 6}, {NULL}};
const Segment SGI262365[] = {{&S50941[98], 15}, {NULL}};
const Segment SGI262366[] = {{&S50942[13], 4}, {NULL}};
const Segment SGI262367[] = {{&S50942[26], 5}, {NULL}};
const Segment SGI262368[] = {{&S50942[59], 6}, {NULL}};
const Segment SGI262369[] = {{&S50942[93], 6}, {NULL}};
const Segment SGI262370[] = {{&S50942[100], 15}, {NULL}};
const Segment SGI262371[] = {{&S35470[10], 4}, {NULL}};
const Segment SGI262372[] = {{&S35470[15], 5}, {NULL}};
const Segment SGI262373[] = {{&S42710[72], 19}, {NULL}};
const Segment SGI262374[] = {{&S42710[0], 91}, {NULL}};
const Segment SGI262375[] = {{&S43175[0], 17}, {NULL}};
const Segment SGI262376[] = {{&S43175[0], 29}, {NULL}};
const Segment SGI262377[] = {{&S43176[0], 7}, {&S43176[74], 24}, {NULL}};
const Segment SGI262378[] = {{&S43176[0], 98}, {NULL}};
const Segment SGI262379[] = {{&S43177[0], 7}, {&S43177[74], 24}, {NULL}};
const Segment SGI262380[] = {{&S43177[0], 98}, {NULL}};
const Segment SGI262381[] = {{&S43178[0], 7}, {&S43178[72], 24}, {NULL}};
const Segment SGI262382[] = {{&S43178[0], 96}, {NULL}};
const Segment SGI262383[] = {{&S43179[0], 7}, {&S43179[74], 24}, {NULL}};
const Segment SGI262384[] = {{&S43179[0], 98}, {NULL}};
const Segment SGI262385[] = {{&S43180[0], 7}, {&S43180[71], 24}, {NULL}};
const Segment SGI262386[] = {{&S43180[0], 95}, {NULL}};
const Segment SGI262387[] = {{&S43181[0], 7}, {&S43181[75], 24}, {NULL}};
const Segment SGI262388[] = {{&S43181[0], 99}, {NULL}};
const Segment SGI262389[] = {{&S43182[89], 21}, {NULL}};
const Segment SGI262390[] = {{&S43182[0], 140}, {NULL}};
const Segment SGI262391[] = {{&S43183[0], 7}, {&S43183[74], 24}, {NULL}};
const Segment SGI262392[] = {{&S43183[0], 98}, {NULL}};
const Segment SGI262393[] = {{&S43184[0], 7}, {&S43184[74], 24}, {NULL}};
const Segment SGI262394[] = {{&S43184[0], 98}, {NULL}};
const Segment SGI262395[] = {{&S43185[0], 7}, {&S43185[74], 24}, {NULL}};
const Segment SGI262396[] = {{&S43185[0], 98}, {NULL}};
const Segment SGI262397[] = {{&S43186[0], 7}, {&S43186[74], 24}, {NULL}};
const Segment SGI262398[] = {{&S43186[0], 98}, {NULL}};
const Segment SGI262399[] = {{&S43187[0], 7}, {&S43187[74], 24}, {NULL}};
const Segment SGI262400[] = {{&S43187[0], 98}, {NULL}};
const Segment SGI262401[] = {{&S43188[0], 7}, {&S43188[74], 24}, {NULL}};
const Segment SGI262402[] = {{&S43188[0], 98}, {NULL}};
const Segment SGI262403[] = {{&S43189[0], 7}, {&S43189[74], 24}, {NULL}};
const Segment SGI262404[] = {{&S43189[0], 98}, {NULL}};
const Segment SGI262405[] = {{&S43190[0], 7}, {&S43190[74], 24}, {NULL}};
const Segment SGI262406[] = {{&S43190[0], 98}, {NULL}};
const Segment SGI262407[] = {{&S43191[0], 7}, {&S43191[74], 24}, {NULL}};
const Segment SGI262408[] = {{&S43191[0], 98}, {NULL}};
const Segment SGI262409[] = {{&S43192[0], 7}, {&S43192[74], 24}, {NULL}};
const Segment SGI262410[] = {{&S43192[0], 98}, {NULL}};
const Segment SGI262411[] = {{&S43193[0], 7}, {&S43193[74], 24}, {NULL}};
const Segment SGI262412[] = {{&S43193[0], 98}, {NULL}};
const Segment SGI262413[] = {{&S43194[0], 7}, {&S43194[74], 24}, {NULL}};
const Segment SGI262414[] = {{&S43194[0], 98}, {NULL}};
const Segment SGI262415[] = {{&S43195[0], 7}, {&S43195[74], 24}, {NULL}};
const Segment SGI262416[] = {{&S43195[0], 98}, {NULL}};
const Segment SGI262417[] = {{&S43196[0], 7}, {&S43196[75], 24}, {NULL}};
const Segment SGI262418[] = {{&S43196[0], 99}, {NULL}};
const Segment SGI262419[] = {{&S43197[59], 8}, {NULL}};
const Segment SGI262420[] = {{&S43197[0], 105}, {NULL}};
const Segment SGI262421[] = {{&S43198[0], 7}, {&S43198[74], 24}, {NULL}};
const Segment SGI262422[] = {{&S43198[0], 98}, {NULL}};
const Segment SGI262423[] = {{&S43199[0], 7}, {&S43199[74], 24}, {NULL}};
const Segment SGI262424[] = {{&S43199[0], 98}, {NULL}};
const Segment SGI262425[] = {{&S43200[0], 7}, {&S43200[75], 24}, {NULL}};
const Segment SGI262426[] = {{&S43200[0], 99}, {NULL}};
const Segment SGI262427[] = {{&S43201[0], 7}, {&S43201[74], 24}, {NULL}};
const Segment SGI262428[] = {{&S43201[0], 98}, {NULL}};
const Segment SGI262429[] = {{&S43202[0], 7}, {&S43202[74], 24}, {NULL}};
const Segment SGI262430[] = {{&S43202[0], 98}, {NULL}};
const Segment SGI262431[] = {{&S43203[0], 7}, {&S43203[74], 24}, {NULL}};
const Segment SGI262432[] = {{&S43203[0], 98}, {NULL}};
const Segment SGI262433[] = {{&S43204[0], 7}, {&S43204[74], 24}, {NULL}};
const Segment SGI262434[] = {{&S43204[0], 98}, {NULL}};
const Segment SGI262435[] = {{&S43205[0], 7}, {&S43205[72], 24}, {NULL}};
const Segment SGI262436[] = {{&S43205[0], 96}, {NULL}};
const Segment SGI262437[] = {{&S43206[0], 7}, {&S43206[74], 24}, {NULL}};
const Segment SGI262438[] = {{&S43206[0], 98}, {NULL}};
const Segment SGI262439[] = {{&S43207[0], 7}, {&S43207[75], 24}, {NULL}};
const Segment SGI262440[] = {{&S43207[0], 99}, {NULL}};
const Segment SGI262441[] = {{&S43208[0], 7}, {&S43208[56], 41}, {NULL}};
const Segment SGI262442[] = {{&S43208[0], 97}, {NULL}};
const Segment SGI262443[] = {{&S43209[0], 7}, {&S43209[54], 41}, {NULL}};
const Segment SGI262444[] = {{&S43209[0], 95}, {NULL}};
const Segment SGI262445[] = {{&S43210[0], 7}, {&S43210[74], 24}, {NULL}};
const Segment SGI262446[] = {{&S43210[0], 98}, {NULL}};
const Segment SGI262447[] = {{&S43211[0], 7}, {&S43211[74], 24}, {NULL}};
const Segment SGI262448[] = {{&S43211[0], 98}, {NULL}};
const Segment SGI262449[] = {{&S43212[0], 7}, {&S43212[56], 24}, {NULL}};
const Segment SGI262450[] = {{&S43212[0], 80}, {NULL}};
const Segment SGI262451[] = {{&S51283[13], 6}, {NULL}};
const Segment SGI262452[] = {{&S51283[38], 5}, {NULL}};
const Segment SGI262453[] = {{&S51283[78], 8}, {NULL}};
const Segment SGI262454[] = {{&S51283[133], 5}, {NULL}};
const Segment SGI262455[] = {{&S51283[146], 9}, {NULL}};
const Segment SGI262456[] = {{&S51284[13], 6}, {NULL}};
const Segment SGI262457[] = {{&S51284[38], 5}, {NULL}};
const Segment SGI262458[] = {{&S51284[78], 8}, {NULL}};
const Segment SGI262459[] = {{&S51284[133], 5}, {NULL}};
const Segment SGI262460[] = {{&S51284[146], 9}, {NULL}};
const Segment SGI262461[] = {{&S51285[13], 6}, {NULL}};
const Segment SGI262462[] = {{&S51285[38], 3}, {NULL}};
const Segment SGI262463[] = {{&S51285[76], 8}, {NULL}};
const Segment SGI262464[] = {{&S51285[131], 5}, {NULL}};
const Segment SGI262465[] = {{&S51285[144], 9}, {NULL}};
const Segment SGI262466[] = {{&S51289[13], 4}, {NULL}};
const Segment SGI262467[] = {{&S51289[116], 2}, {NULL}};
const Segment SGI262468[] = {{&S51289[26], 5}, {NULL}};
const Segment SGI262469[] = {{&S51289[59], 6}, {NULL}};
const Segment SGI262470[] = {{&S51289[26], 1}, {NULL}};
const Segment SGI262471[] = {{&S51289[143], 7}, {NULL}};
const Segment SGI262472[] = {{&S51290[13], 4}, {NULL}};
const Segment SGI262473[] = {{&S51290[114], 2}, {NULL}};
const Segment SGI262474[] = {{&S51290[26], 3}, {NULL}};
const Segment SGI262475[] = {{&S51290[57], 6}, {NULL}};
const Segment SGI262476[] = {{&S51290[26], 1}, {NULL}};
const Segment SGI262477[] = {{&S51290[139], 7}, {NULL}};
const Segment SGI262478[] = {{&S51291[13], 4}, {NULL}};
const Segment SGI262479[] = {{&S51291[116], 2}, {NULL}};
const Segment SGI262480[] = {{&S51291[26], 5}, {NULL}};
const Segment SGI262481[] = {{&S51291[59], 6}, {NULL}};
const Segment SGI262482[] = {{&S51291[26], 1}, {NULL}};
const Segment SGI262483[] = {{&S51291[143], 7}, {NULL}};
const Segment SGI262484[] = {{&S43650[25], 16}, {&S43650[42], 31}, {NULL}};
const Segment SGI262485[] = {{&S43650[0], 73}, {NULL}};
const Segment SGI262486[] = {{&S51346[13], 4}, {NULL}};
const Segment SGI262487[] = {{&S51346[33], 5}, {NULL}};
const Segment SGI262488[] = {{&S51346[66], 6}, {NULL}};
const Segment SGI262489[] = {{&S51346[120], 6}, {NULL}};
const Segment SGI262490[] = {{&S51346[134], 7}, {NULL}};
const Segment SGI262491[] = {{&S51347[13], 4}, {NULL}};
const Segment SGI262492[] = {{&S51347[33], 3}, {NULL}};
const Segment SGI262493[] = {{&S51347[64], 6}, {NULL}};
const Segment SGI262494[] = {{&S51347[118], 6}, {NULL}};
const Segment SGI262495[] = {{&S51347[132], 7}, {NULL}};
const Segment SGI262496[] = {{&S51348[13], 4}, {NULL}};
const Segment SGI262497[] = {{&S51348[33], 3}, {NULL}};
const Segment SGI262498[] = {{&S51348[64], 6}, {NULL}};
const Segment SGI262499[] = {{&S51348[118], 6}, {NULL}};
const Segment SGI262500[] = {{&S51348[132], 7}, {NULL}};
const Segment SGI262501[] = {{&S36278[10], 4}, {NULL}};
const Segment SGI262502[] = {{&S36278[15], 5}, {NULL}};
const Segment SGI262503[] = {{&S36279[10], 5}, {NULL}};
const Segment SGI262504[] = {{&S36279[16], 5}, {NULL}};
const Segment SGI262505[] = {{&S51469[13], 4}, {NULL}};
const Segment SGI262506[] = {{&S51469[105], 3}, {NULL}};
const Segment SGI262507[] = {{&S51469[39], 3}, {NULL}};
const Segment SGI262508[] = {{&S51469[70], 6}, {NULL}};
const Segment SGI262509[] = {{&S51469[116], 8}, {NULL}};
const Segment SGI262510[] = {{&S51469[132], 7}, {NULL}};
const Segment SGI262511[] = {{&S51470[13], 4}, {NULL}};
const Segment SGI262512[] = {{&S51470[105], 3}, {NULL}};
const Segment SGI262513[] = {{&S51470[39], 3}, {NULL}};
const Segment SGI262514[] = {{&S51470[70], 6}, {NULL}};
const Segment SGI262515[] = {{&S51470[116], 8}, {NULL}};
const Segment SGI262516[] = {{&S51470[132], 7}, {NULL}};
const Segment SGI262517[] = {{&S51471[13], 4}, {NULL}};
const Segment SGI262518[] = {{&S51471[105], 3}, {NULL}};
const Segment SGI262519[] = {{&S51471[39], 3}, {NULL}};
const Segment SGI262520[] = {{&S51471[70], 6}, {NULL}};
const Segment SGI262521[] = {{&S51471[116], 8}, {NULL}};
const Segment SGI262522[] = {{&S51471[132], 7}, {NULL}};
const Segment SGI262523[] = {{&S51472[13], 4}, {NULL}};
const Segment SGI262524[] = {{&S51472[107], 3}, {NULL}};
const Segment SGI262525[] = {{&S51472[39], 3}, {NULL}};
const Segment SGI262526[] = {{&S51472[70], 8}, {NULL}};
const Segment SGI262527[] = {{&S51472[118], 8}, {NULL}};
const Segment SGI262528[] = {{&S51472[134], 7}, {NULL}};
const Segment SGI262529[] = {{&S51485[13], 4}, {NULL}};
const Segment SGI262530[] = {{&S51485[33], 5}, {NULL}};
const Segment SGI262531[] = {{&S51485[66], 6}, {NULL}};
const Segment SGI262532[] = {{&S51485[119], 6}, {NULL}};
const Segment SGI262533[] = {{&S51485[133], 7}, {NULL}};
const Segment SGI262534[] = {{&S51486[13], 4}, {NULL}};
const Segment SGI262535[] = {{&S51486[33], 3}, {NULL}};
const Segment SGI262536[] = {{&S51486[64], 6}, {NULL}};
const Segment SGI262537[] = {{&S51486[117], 6}, {NULL}};
const Segment SGI262538[] = {{&S51486[131], 7}, {NULL}};
const Segment SGI262539[] = {{&S51487[13], 4}, {NULL}};
const Segment SGI262540[] = {{&S51487[33], 5}, {NULL}};
const Segment SGI262541[] = {{&S51487[66], 6}, {NULL}};
const Segment SGI262542[] = {{&S51487[119], 6}, {NULL}};
const Segment SGI262543[] = {{&S51487[133], 7}, {NULL}};
const Segment SGI262544[] = {{&S51488[13], 4}, {NULL}};
const Segment SGI262545[] = {{&S51488[33], 3}, {NULL}};
const Segment SGI262546[] = {{&S51488[64], 6}, {NULL}};
const Segment SGI262547[] = {{&S51488[117], 6}, {NULL}};
const Segment SGI262548[] = {{&S51488[131], 7}, {NULL}};
const Segment SGI262549[] = {{&S51706[13], 4}, {NULL}};
const Segment SGI262550[] = {{&S51706[104], 5}, {NULL}};
const Segment SGI262551[] = {{&S51706[29], 3}, {NULL}};
const Segment SGI262552[] = {{&S51706[67], 8}, {NULL}};
const Segment SGI262553[] = {{&S51706[117], 5}, {NULL}};
const Segment SGI262554[] = {{&S51706[130], 9}, {NULL}};
const Segment SGI262555[] = {{&S51707[13], 4}, {NULL}};
const Segment SGI262556[] = {{&S51707[26], 3}, {NULL}};
const Segment SGI262557[] = {{&S51707[57], 6}, {NULL}};
const Segment SGI262558[] = {{&S51707[91], 6}, {NULL}};
const Segment SGI262559[] = {{&S51707[105], 9}, {NULL}};
const Segment SGI262560[] = {{&S51708[13], 4}, {NULL}};
const Segment SGI262561[] = {{&S51708[26], 3}, {NULL}};
const Segment SGI262562[] = {{&S51708[57], 6}, {NULL}};
const Segment SGI262563[] = {{&S51708[91], 6}, {NULL}};
const Segment SGI262564[] = {{&S51708[105], 9}, {&S51708[105], 9}, {NULL}};
const Segment SGI262565[] = {{&S51709[13], 4}, {NULL}};
const Segment SGI262566[] = {{&S51709[26], 3}, {NULL}};
const Segment SGI262567[] = {{&S51709[57], 6}, {NULL}};
const Segment SGI262568[] = {{&S51709[91], 6}, {NULL}};
const Segment SGI262569[] = {{&S51709[105], 9}, {&S51709[105], 9}, {&S51709[105], 9}, {NULL}};
const Segment SGI262570[] = {{&S51710[13], 4}, {NULL}};
const Segment SGI262571[] = {{&S51710[103], 5}, {NULL}};
const Segment SGI262572[] = {{&S51710[29], 3}, {NULL}};
const Segment SGI262573[] = {{&S51710[67], 7}, {NULL}};
const Segment SGI262574[] = {{&S51710[116], 3}, {NULL}};
const Segment SGI262575[] = {{&S51710[127], 9}, {NULL}};
const Segment SGI262576[] = {{&S51711[13], 4}, {NULL}};
const Segment SGI262577[] = {{&S51711[92], 5}, {NULL}};
const Segment SGI262578[] = {{&S51711[26], 3}, {NULL}};
const Segment SGI262579[] = {{&S51711[57], 6}, {NULL}};
const Segment SGI262580[] = {{&S51711[105], 6}, {NULL}};
const Segment SGI262581[] = {{&S51711[119], 9}, {NULL}};
const Segment SGI262582[] = {{&S51712[13], 4}, {NULL}};
const Segment SGI262583[] = {{&S51712[26], 3}, {NULL}};
const Segment SGI262584[] = {{&S51712[57], 6}, {NULL}};
const Segment SGI262585[] = {{&S51712[91], 6}, {NULL}};
const Segment SGI262586[] = {{&S51712[105], 9}, {&S51712[105], 9}, {&S51712[105], 9}, {&S51712[105], 9}, {NULL}};
const Segment SGI262587[] = {{&S51713[13], 4}, {NULL}};
const Segment SGI262588[] = {{&S51713[106], 5}, {NULL}};
const Segment SGI262589[] = {{&S51713[29], 5}, {NULL}};
const Segment SGI262590[] = {{&S51713[69], 8}, {NULL}};
const Segment SGI262591[] = {{&S51713[119], 5}, {NULL}};
const Segment SGI262592[] = {{&S51713[132], 9}, {NULL}};
const Segment SGI262593[] = {{&S51714[13], 4}, {&S51714[13], 4}, {NULL}};
const Segment SGI262594[] = {{&S51714[116], 5}, {NULL}};
const Segment SGI262595[] = {{&S51714[29], 1}, {NULL}};
const Segment SGI262596[] = {{&S51714[79], 8}, {NULL}};
const Segment SGI262597[] = {{&S51714[129], 5}, {NULL}};
const Segment SGI262598[] = {{&S51714[142], 9}, {NULL}};
const Segment SGI262599[] = {{&S51715[13], 4}, {NULL}};
const Segment SGI262600[] = {{&S51715[105], 5}, {NULL}};
const Segment SGI262601[] = {{&S51715[29], 5}, {NULL}};
const Segment SGI262602[] = {{&S51715[69], 7}, {NULL}};
const Segment SGI262603[] = {{&S51715[118], 3}, {NULL}};
const Segment SGI262604[] = {{&S51715[129], 9}, {NULL}};
const Segment SGI262605[] = {{&S51716[13], 4}, {NULL}};
const Segment SGI262606[] = {{&S51716[26], 5}, {NULL}};
const Segment SGI262607[] = {{&S51716[59], 6}, {NULL}};
const Segment SGI262608[] = {{&S51716[93], 6}, {NULL}};
const Segment SGI262609[] = {{&S51716[107], 9}, {NULL}};
const Segment SGI262610[] = {{&S51717[13], 4}, {NULL}};
const Segment SGI262611[] = {{&S51717[94], 5}, {NULL}};
const Segment SGI262612[] = {{&S51717[26], 5}, {NULL}};
const Segment SGI262613[] = {{&S51717[59], 6}, {NULL}};
const Segment SGI262614[] = {{&S51717[107], 6}, {NULL}};
const Segment SGI262615[] = {{&S51717[121], 9}, {NULL}};
const Segment SGI262616[] = {{&S51718[13], 4}, {NULL}};
const Segment SGI262617[] = {{&S51718[106], 5}, {NULL}};
const Segment SGI262618[] = {{&S51718[29], 5}, {NULL}};
const Segment SGI262619[] = {{&S51718[69], 8}, {NULL}};
const Segment SGI262620[] = {{&S51718[119], 5}, {NULL}};
const Segment SGI262621[] = {{&S51718[132], 9}, {NULL}};
const Segment SGI262622[] = {{&S51719[13], 4}, {NULL}};
const Segment SGI262623[] = {{&S51719[29], 5}, {NULL}};
const Segment SGI262624[] = {{&S51719[104], 5}, {NULL}};
const Segment SGI262625[] = {{&S51719[117], 9}, {NULL}};
const Segment SGI262626[] = {{&S51720[13], 4}, {NULL}};
const Segment SGI262627[] = {{&S51720[106], 5}, {NULL}};
const Segment SGI262628[] = {{&S51720[29], 5}, {NULL}};
const Segment SGI262629[] = {{&S51720[69], 8}, {NULL}};
const Segment SGI262630[] = {{&S51720[119], 5}, {&S51720[33], 1}, {NULL}};
const Segment SGI262631[] = {{&S51720[152], 9}, {NULL}};
const Segment SGI262632[] = {{&S51746[13], 4}, {NULL}};
const Segment SGI262633[] = {{&S51746[26], 5}, {NULL}};
const Segment SGI262634[] = {{&S51746[59], 6}, {NULL}};
const Segment SGI262635[] = {{&S51746[107], 5}, {NULL}};
const Segment SGI262636[] = {{&S51746[120], 9}, {NULL}};
const Segment SGI262637[] = {{&S51747[13], 4}, {NULL}};
const Segment SGI262638[] = {{&S51747[26], 3}, {NULL}};
const Segment SGI262639[] = {{&S51747[57], 6}, {NULL}};
const Segment SGI262640[] = {{&S51747[105], 5}, {NULL}};
const Segment SGI262641[] = {{&S51747[118], 9}, {NULL}};
const Segment SGI262642[] = {{&S44211[62], 6}, {NULL}};
const Segment SGI262643[] = {{&S44211[0], 68}, {NULL}};
const Segment SGI262644[] = {{&S44212[62], 6}, {NULL}};
const Segment SGI262645[] = {{&S44212[0], 68}, {NULL}};
const Segment SGI262646[] = {{&S51786[13], 4}, {&S51786[13], 4}, {NULL}};
const Segment SGI262647[] = {{&S51786[26], 3}, {NULL}};
const Segment SGI262648[] = {{&S51786[57], 6}, {NULL}};
const Segment SGI262649[] = {{&S51786[91], 5}, {NULL}};
const Segment SGI262650[] = {{&S51786[97], 7}, {NULL}};
const Segment SGI262651[] = {{&S51787[13], 4}, {&S51787[13], 4}, {NULL}};
const Segment SGI262652[] = {{&S51787[26], 5}, {NULL}};
const Segment SGI262653[] = {{&S51787[59], 6}, {NULL}};
const Segment SGI262654[] = {{&S51787[93], 5}, {NULL}};
const Segment SGI262655[] = {{&S51787[99], 7}, {NULL}};
const Segment SGI262656[] = {{&S51788[13], 4}, {&S51788[13], 4}, {NULL}};
const Segment SGI262657[] = {{&S51788[26], 3}, {NULL}};
const Segment SGI262658[] = {{&S51788[57], 6}, {NULL}};
const Segment SGI262659[] = {{&S51788[91], 5}, {NULL}};
const Segment SGI262660[] = {{&S51788[97], 7}, {NULL}};
const Segment SGI262661[] = {{&S51864[13], 4}, {NULL}};
const Segment SGI262662[] = {{&S51864[33], 3}, {NULL}};
const Segment SGI262663[] = {{&S51864[64], 6}, {NULL}};
const Segment SGI262664[] = {{&S51864[117], 5}, {NULL}};
const Segment SGI262665[] = {{&S51864[130], 9}, {NULL}};
const Segment SGI262666[] = {{&S51865[13], 4}, {NULL}};
const Segment SGI262667[] = {{&S51865[33], 5}, {NULL}};
const Segment SGI262668[] = {{&S51865[66], 6}, {NULL}};
const Segment SGI262669[] = {{&S51865[119], 5}, {NULL}};
const Segment SGI262670[] = {{&S51865[132], 9}, {NULL}};
const Segment SGI262671[] = {{&S51879[13], 6}, {NULL}};
const Segment SGI262672[] = {{&S51879[111], 3}, {NULL}};
const Segment SGI262673[] = {{&S51879[38], 3}, {NULL}};
const Segment SGI262674[] = {{&S51879[76], 6}, {NULL}};
const Segment SGI262675[] = {{&S51879[122], 5}, {NULL}};
const Segment SGI262676[] = {{&S51879[135], 6}, {NULL}};
const Segment SGI262677[] = {{&S51880[13], 6}, {NULL}};
const Segment SGI262678[] = {{&S51880[110], 5}, {NULL}};
const Segment SGI262679[] = {{&S51880[38], 3}, {NULL}};
const Segment SGI262680[] = {{&S51880[76], 5}, {NULL}};
const Segment SGI262681[] = {{&S51880[123], 5}, {NULL}};
const Segment SGI262682[] = {{&S51881[13], 6}, {NULL}};
const Segment SGI262683[] = {{&S51881[110], 5}, {NULL}};
const Segment SGI262684[] = {{&S51881[38], 3}, {NULL}};
const Segment SGI262685[] = {{&S51881[76], 5}, {NULL}};
const Segment SGI262686[] = {{&S51881[123], 5}, {NULL}};
const Segment SGI262687[] = {{&S51881[136], 9}, {NULL}};
const Segment SGI262688[] = {{&S51882[13], 6}, {NULL}};
const Segment SGI262689[] = {{&S51882[35], 3}, {NULL}};
const Segment SGI262690[] = {{&S51882[66], 6}, {NULL}};
const Segment SGI262691[] = {{&S51882[100], 6}, {NULL}};
const Segment SGI262692[] = {{&S51882[114], 9}, {NULL}};
const Segment SGI262693[] = {{&S44312[52], 11}, {NULL}};
const Segment SGI262694[] = {{&S44312[0], 63}, {NULL}};
const Segment SGI262695[] = {{&S51883[13], 6}, {NULL}};
const Segment SGI262696[] = {{&S51883[112], 5}, {NULL}};
const Segment SGI262697[] = {{&S51883[38], 5}, {NULL}};
const Segment SGI262698[] = {{&S51883[78], 5}, {NULL}};
const Segment SGI262699[] = {{&S51883[125], 5}, {NULL}};
const Segment SGI262700[] = {{&S51884[13], 6}, {NULL}};
const Segment SGI262701[] = {{&S51884[113], 3}, {NULL}};
const Segment SGI262702[] = {{&S51884[38], 5}, {NULL}};
const Segment SGI262703[] = {{&S51884[78], 6}, {NULL}};
const Segment SGI262704[] = {{&S51884[124], 5}, {NULL}};
const Segment SGI262705[] = {{&S51884[137], 6}, {NULL}};
const Segment SGI262706[] = {{&S51885[13], 6}, {NULL}};
const Segment SGI262707[] = {{&S51885[112], 5}, {NULL}};
const Segment SGI262708[] = {{&S51885[38], 5}, {NULL}};
const Segment SGI262709[] = {{&S51885[78], 5}, {NULL}};
const Segment SGI262710[] = {{&S51885[125], 5}, {NULL}};
const Segment SGI262711[] = {{&S51885[138], 9}, {NULL}};
const Segment SGI262712[] = {{&S51886[13], 6}, {NULL}};
const Segment SGI262713[] = {{&S51886[35], 5}, {NULL}};
const Segment SGI262714[] = {{&S51886[68], 6}, {NULL}};
const Segment SGI262715[] = {{&S51886[102], 6}, {NULL}};
const Segment SGI262716[] = {{&S51886[116], 9}, {NULL}};
const Segment SGI262717[] = {{&S51887[13], 6}, {NULL}};
const Segment SGI262718[] = {{&S51887[35], 5}, {NULL}};
const Segment SGI262719[] = {{&S51887[68], 6}, {NULL}};
const Segment SGI262720[] = {{&S51887[102], 6}, {NULL}};
const Segment SGI262721[] = {{&S51887[116], 9}, {NULL}};
const Segment SGI262722[] = {{&S51888[13], 6}, {NULL}};
const Segment SGI262723[] = {{&S51888[112], 5}, {NULL}};
const Segment SGI262724[] = {{&S51888[38], 5}, {NULL}};
const Segment SGI262725[] = {{&S51888[78], 5}, {NULL}};
const Segment SGI262726[] = {{&S51888[125], 5}, {NULL}};
const Segment SGI262727[] = {{&S51889[13], 6}, {NULL}};
const Segment SGI262728[] = {{&S51889[113], 3}, {NULL}};
const Segment SGI262729[] = {{&S51889[38], 5}, {NULL}};
const Segment SGI262730[] = {{&S51889[78], 6}, {NULL}};
const Segment SGI262731[] = {{&S51889[124], 5}, {NULL}};
const Segment SGI262732[] = {{&S51889[137], 6}, {NULL}};
const Segment SGI262733[] = {{&S51890[13], 6}, {NULL}};
const Segment SGI262734[] = {{&S51890[35], 5}, {NULL}};
const Segment SGI262735[] = {{&S51890[68], 6}, {NULL}};
const Segment SGI262736[] = {{&S51890[102], 6}, {NULL}};
const Segment SGI262737[] = {{&S51890[116], 9}, {NULL}};
const Segment SGI262738[] = {{&S51891[13], 6}, {NULL}};
const Segment SGI262739[] = {{&S51891[112], 5}, {NULL}};
const Segment SGI262740[] = {{&S51891[38], 5}, {NULL}};
const Segment SGI262741[] = {{&S51891[78], 5}, {NULL}};
const Segment SGI262742[] = {{&S51891[125], 5}, {NULL}};
const Segment SGI262743[] = {{&S51891[138], 9}, {NULL}};
const Segment SGI262744[] = {{&S51892[13], 6}, {NULL}};
const Segment SGI262745[] = {{&S51892[38], 5}, {NULL}};
const Segment SGI262746[] = {{&S51892[86], 5}, {NULL}};
const Segment SGI262747[] = {{&S51892[119], 5}, {NULL}};
const Segment SGI262748[] = {{&S51892[132], 9}, {NULL}};
const Segment SGI262749[] = {{&S44351[95], 6}, {NULL}};
const Segment SGI262750[] = {{&S44351[0], 105}, {NULL}};
const Segment SGI262751[] = {{&S51911[13], 4}, {NULL}};
const Segment SGI262752[] = {{&S51911[33], 3}, {NULL}};
const Segment SGI262753[] = {{&S51911[64], 6}, {NULL}};
const Segment SGI262754[] = {{&S51911[98], 5}, {NULL}};
const Segment SGI262755[] = {{&S51911[104], 7}, {NULL}};
const Segment SGI262756[] = {{&S51912[13], 4}, {NULL}};
const Segment SGI262757[] = {{&S51912[33], 5}, {NULL}};
const Segment SGI262758[] = {{&S51912[66], 6}, {NULL}};
const Segment SGI262759[] = {{&S51912[100], 5}, {NULL}};
const Segment SGI262760[] = {{&S51912[106], 7}, {NULL}};
const Segment SGI262761[] = {{&S51936[12], 6}, {NULL}};
const Segment SGI262762[] = {{&S51936[74], 5}, {NULL}};
const Segment SGI262763[] = {{&S51938[12], 6}, {NULL}};
const Segment SGI262764[] = {{&S51939[12], 6}, {NULL}};
const Segment SGI262765[] = {{&S51939[74], 5}, {NULL}};
const Segment SGI262766[] = {{&S52192[13], 4}, {NULL}};
const Segment SGI262767[] = {{&S52192[26], 5}, {NULL}};
const Segment SGI262768[] = {{&S52192[59], 6}, {NULL}};
const Segment SGI262769[] = {{&S52192[112], 5}, {NULL}};
const Segment SGI262770[] = {{&S52192[125], 9}, {NULL}};
const Segment SGI262771[] = {{&S52193[13], 4}, {NULL}};
const Segment SGI262772[] = {{&S52193[26], 3}, {NULL}};
const Segment SGI262773[] = {{&S52193[57], 6}, {NULL}};
const Segment SGI262774[] = {{&S52193[110], 5}, {NULL}};
const Segment SGI262775[] = {{&S52193[123], 9}, {NULL}};
const Segment SGI262776[] = {{&S44540[0], 20}, {NULL}};
const Segment SGI262777[] = {{&S44540[0], 55}, {NULL}};
const Segment SGI262778[] = {{&S44541[0], 21}, {NULL}};
const Segment SGI262779[] = {{&S44541[0], 56}, {NULL}};
const Segment SGI262780[] = {{&S52330[28], 4}, {NULL}};
const Segment SGI262781[] = {{&S52330[44], 3}, {NULL}};
const Segment SGI262782[] = {{&S52331[28], 4}, {NULL}};
const Segment SGI262783[] = {{&S52331[44], 5}, {NULL}};
const Segment SGI262784[] = {{&S52332[13], 4}, {NULL}};
const Segment SGI262785[] = {{&S52332[109], 3}, {NULL}};
const Segment SGI262786[] = {{&S52332[36], 3}, {NULL}};
const Segment SGI262787[] = {{&S52332[74], 6}, {NULL}};
const Segment SGI262788[] = {{&S52332[120], 5}, {NULL}};
const Segment SGI262789[] = {{&S52332[133], 6}, {NULL}};
const Segment SGI262790[] = {{&S52333[13], 4}, {NULL}};
const Segment SGI262791[] = {{&S52333[107], 3}, {NULL}};
const Segment SGI262792[] = {{&S52333[36], 2}, {NULL}};
const Segment SGI262793[] = {{&S52333[73], 6}, {NULL}};
const Segment SGI262794[] = {{&S52333[118], 5}, {NULL}};
const Segment SGI262795[] = {{&S52333[131], 6}, {NULL}};
const Segment SGI262796[] = {{&S52372[13], 6}, {NULL}};
const Segment SGI262797[] = {{&S52372[35], 5}, {NULL}};
const Segment SGI262798[] = {{&S52372[68], 6}, {NULL}};
const Segment SGI262799[] = {{&S52372[121], 5}, {NULL}};
const Segment SGI262800[] = {{&S52372[134], 9}, {NULL}};
const Segment SGI262801[] = {{&S52373[13], 6}, {NULL}};
const Segment SGI262802[] = {{&S52373[35], 5}, {NULL}};
const Segment SGI262803[] = {{&S52373[68], 6}, {NULL}};
const Segment SGI262804[] = {{&S52373[121], 5}, {NULL}};
const Segment SGI262805[] = {{&S52373[134], 9}, {NULL}};
const Segment SGI262806[] = {{&S52374[13], 6}, {NULL}};
const Segment SGI262807[] = {{&S52374[35], 3}, {NULL}};
const Segment SGI262808[] = {{&S52374[66], 6}, {NULL}};
const Segment SGI262809[] = {{&S52374[119], 5}, {NULL}};
const Segment SGI262810[] = {{&S52374[132], 9}, {NULL}};
const Segment SGI262811[] = {{&S37544[10], 9}, {NULL}};
const Segment SGI262812[] = {{&S37545[10], 4}, {NULL}};
const Segment SGI262813[] = {{&S37545[15], 5}, {NULL}};
const Segment SGI262814[] = {{&S44937[87], 9}, {NULL}};
const Segment SGI262815[] = {{&S44937[0], 96}, {NULL}};
const Segment SGI262816[] = {{&S52715[13], 6}, {NULL}};
const Segment SGI262817[] = {{&S52715[101], 3}, {NULL}};
const Segment SGI262818[] = {{&S52715[35], 3}, {NULL}};
const Segment SGI262819[] = {{&S52715[66], 6}, {NULL}};
const Segment SGI262820[] = {{&S52715[112], 5}, {NULL}};
const Segment SGI262821[] = {{&S52715[125], 9}, {NULL}};
const Segment SGI262822[] = {{&S52716[13], 6}, {NULL}};
const Segment SGI262823[] = {{&S52716[101], 3}, {NULL}};
const Segment SGI262824[] = {{&S52716[35], 3}, {NULL}};
const Segment SGI262825[] = {{&S52716[66], 6}, {NULL}};
const Segment SGI262826[] = {{&S52716[112], 8}, {NULL}};
const Segment SGI262827[] = {{&S52716[128], 9}, {NULL}};
const Segment SGI262828[] = {{&S52717[13], 6}, {NULL}};
const Segment SGI262829[] = {{&S52717[122], 3}, {NULL}};
const Segment SGI262830[] = {{&S52717[35], 3}, {NULL}};
const Segment SGI262831[] = {{&S52717[66], 6}, {NULL}};
const Segment SGI262832[] = {{&S52717[100], 6}, {NULL}};
const Segment SGI262833[] = {{&S52718[13], 6}, {NULL}};
const Segment SGI262834[] = {{&S52718[122], 3}, {NULL}};
const Segment SGI262835[] = {{&S52718[35], 3}, {NULL}};
const Segment SGI262836[] = {{&S52718[66], 6}, {NULL}};
const Segment SGI262837[] = {{&S52718[100], 6}, {NULL}};
const Segment SGI262838[] = {{&S52719[13], 6}, {NULL}};
const Segment SGI262839[] = {{&S52719[124], 3}, {NULL}};
const Segment SGI262840[] = {{&S52719[35], 5}, {NULL}};
const Segment SGI262841[] = {{&S52719[68], 6}, {NULL}};
const Segment SGI262842[] = {{&S52719[102], 6}, {NULL}};
const Segment SGI262843[] = {{&S52720[13], 6}, {NULL}};
const Segment SGI262844[] = {{&S52720[94], 3}, {NULL}};
const Segment SGI262845[] = {{&S52720[28], 3}, {NULL}};
const Segment SGI262846[] = {{&S52720[59], 6}, {NULL}};
const Segment SGI262847[] = {{&S52720[105], 6}, {NULL}};
const Segment SGI262848[] = {{&S52720[119], 7}, {NULL}};
const Segment SGI262849[] = {{&S52721[13], 6}, {NULL}};
const Segment SGI262850[] = {{&S52721[115], 3}, {NULL}};
const Segment SGI262851[] = {{&S52721[35], 3}, {NULL}};
const Segment SGI262852[] = {{&S52721[66], 6}, {NULL}};
const Segment SGI262853[] = {{&S52721[100], 6}, {NULL}};
const Segment SGI262854[] = {{&S52721[126], 9}, {NULL}};
const Segment SGI262855[] = {{&S52722[13], 6}, {NULL}};
const Segment SGI262856[] = {{&S52722[124], 3}, {NULL}};
const Segment SGI262857[] = {{&S52722[35], 5}, {NULL}};
const Segment SGI262858[] = {{&S52722[68], 6}, {NULL}};
const Segment SGI262859[] = {{&S52722[102], 6}, {NULL}};
const Segment SGI262860[] = {{&S52723[13], 6}, {NULL}};
const Segment SGI262861[] = {{&S52723[96], 3}, {NULL}};
const Segment SGI262862[] = {{&S52723[28], 5}, {NULL}};
const Segment SGI262863[] = {{&S52723[61], 6}, {NULL}};
const Segment SGI262864[] = {{&S52723[107], 6}, {NULL}};
const Segment SGI262865[] = {{&S52723[121], 7}, {NULL}};
const Segment SGI262866[] = {{&S52724[13], 6}, {NULL}};
const Segment SGI262867[] = {{&S52724[101], 3}, {NULL}};
const Segment SGI262868[] = {{&S52724[35], 3}, {NULL}};
const Segment SGI262869[] = {{&S52724[66], 6}, {NULL}};
const Segment SGI262870[] = {{&S52724[112], 6}, {NULL}};
const Segment SGI262871[] = {{&S52724[126], 9}, {NULL}};
const Segment SGI262872[] = {{&S52725[13], 6}, {NULL}};
const Segment SGI262873[] = {{&S52725[117], 3}, {NULL}};
const Segment SGI262874[] = {{&S52725[35], 5}, {NULL}};
const Segment SGI262875[] = {{&S52725[68], 6}, {NULL}};
const Segment SGI262876[] = {{&S52725[102], 6}, {NULL}};
const Segment SGI262877[] = {{&S52725[128], 9}, {NULL}};
const Segment SGI262878[] = {{&S52726[13], 6}, {NULL}};
const Segment SGI262879[] = {{&S52726[124], 3}, {NULL}};
const Segment SGI262880[] = {{&S52726[35], 5}, {NULL}};
const Segment SGI262881[] = {{&S52726[68], 6}, {NULL}};
const Segment SGI262882[] = {{&S52726[102], 6}, {NULL}};
const Segment SGI262883[] = {{&S52727[13], 6}, {NULL}};
const Segment SGI262884[] = {{&S52727[103], 3}, {NULL}};
const Segment SGI262885[] = {{&S52727[35], 5}, {NULL}};
const Segment SGI262886[] = {{&S52727[68], 6}, {NULL}};
const Segment SGI262887[] = {{&S52727[114], 6}, {NULL}};
const Segment SGI262888[] = {{&S52727[128], 9}, {NULL}};
const Segment SGI262889[] = {{&S52728[13], 6}, {NULL}};
const Segment SGI262890[] = {{&S52728[99], 3}, {NULL}};
const Segment SGI262891[] = {{&S52728[35], 2}, {NULL}};
const Segment SGI262892[] = {{&S52728[65], 6}, {NULL}};
const Segment SGI262893[] = {{&S52728[110], 8}, {NULL}};
const Segment SGI262894[] = {{&S52728[126], 9}, {NULL}};
const Segment SGI262895[] = {{&S52729[13], 6}, {NULL}};
const Segment SGI262896[] = {{&S52729[101], 3}, {NULL}};
const Segment SGI262897[] = {{&S52729[35], 3}, {NULL}};
const Segment SGI262898[] = {{&S52729[66], 6}, {NULL}};
const Segment SGI262899[] = {{&S52729[112], 5}, {NULL}};
const Segment SGI262900[] = {{&S52729[125], 9}, {NULL}};
const Segment SGI262901[] = {{&S52730[13], 6}, {NULL}};
const Segment SGI262902[] = {{&S52730[121], 3}, {NULL}};
const Segment SGI262903[] = {{&S52730[35], 3}, {NULL}};
const Segment SGI262904[] = {{&S52730[66], 6}, {NULL}};
const Segment SGI262905[] = {{&S52730[99], 6}, {NULL}};
const Segment SGI262906[] = {{&S52731[13], 6}, {NULL}};
const Segment SGI262907[] = {{&S52731[103], 3}, {NULL}};
const Segment SGI262908[] = {{&S52731[35], 5}, {NULL}};
const Segment SGI262909[] = {{&S52731[68], 6}, {NULL}};
const Segment SGI262910[] = {{&S52731[114], 5}, {NULL}};
const Segment SGI262911[] = {{&S52731[127], 9}, {NULL}};
const Segment SGI262912[] = {{&S52860[13], 4}, {NULL}};
const Segment SGI262913[] = {{&S52860[26], 5}, {NULL}};
const Segment SGI262914[] = {{&S52860[59], 6}, {NULL}};
const Segment SGI262915[] = {{&S52860[113], 5}, {NULL}};
const Segment SGI262916[] = {{&S52860[126], 9}, {NULL}};
const Segment SGI262917[] = {{&S52861[13], 4}, {NULL}};
const Segment SGI262918[] = {{&S52861[26], 3}, {NULL}};
const Segment SGI262919[] = {{&S52861[57], 6}, {NULL}};
const Segment SGI262920[] = {{&S52861[111], 5}, {NULL}};
const Segment SGI262921[] = {{&S52861[124], 9}, {NULL}};
const Segment SGI262922[] = {{&S52907[13], 4}, {NULL}};
const Segment SGI262923[] = {{&S52907[36], 5}, {NULL}};
const Segment SGI262924[] = {{&S52907[76], 8}, {NULL}};
const Segment SGI262925[] = {{&S52907[131], 3}, {NULL}};
const Segment SGI262926[] = {{&S52907[142], 9}, {NULL}};
const Segment SGI262927[] = {{&S52908[13], 4}, {NULL}};
const Segment SGI262928[] = {{&S52908[36], 5}, {NULL}};
const Segment SGI262929[] = {{&S52908[76], 8}, {NULL}};
const Segment SGI262930[] = {{&S52908[131], 3}, {NULL}};
const Segment SGI262931[] = {{&S52908[142], 9}, {NULL}};
const Segment SGI262932[] = {{&S52909[13], 4}, {NULL}};
const Segment SGI262933[] = {{&S52909[36], 3}, {NULL}};
const Segment SGI262934[] = {{&S52909[74], 8}, {NULL}};
const Segment SGI262935[] = {{&S52909[129], 5}, {NULL}};
const Segment SGI262936[] = {{&S52909[142], 9}, {NULL}};
const Segment SGI262937[] = {{&S52910[13], 4}, {NULL}};
const Segment SGI262938[] = {{&S52910[36], 5}, {NULL}};
const Segment SGI262939[] = {{&S52910[76], 8}, {NULL}};
const Segment SGI262940[] = {{&S52910[131], 3}, {NULL}};
const Segment SGI262941[] = {{&S52910[142], 9}, {NULL}};
const Segment SGI262942[] = {{&S52911[13], 4}, {NULL}};
const Segment SGI262943[] = {{&S52911[36], 5}, {NULL}};
const Segment SGI262944[] = {{&S52911[76], 8}, {NULL}};
const Segment SGI262945[] = {{&S52911[131], 3}, {NULL}};
const Segment SGI262946[] = {{&S52911[142], 9}, {NULL}};
const Segment SGI262947[] = {{&S52912[13], 4}, {NULL}};
const Segment SGI262948[] = {{&S52912[36], 5}, {NULL}};
const Segment SGI262949[] = {{&S52912[76], 8}, {NULL}};
const Segment SGI262950[] = {{&S52912[131], 3}, {NULL}};
const Segment SGI262951[] = {{&S52912[142], 9}, {NULL}};
const Segment SGI262952[] = {{&S53080[13], 6}, {NULL}};
const Segment SGI262953[] = {{&S53080[112], 5}, {NULL}};
const Segment SGI262954[] = {{&S53080[38], 5}, {NULL}};
const Segment SGI262955[] = {{&S53080[78], 5}, {NULL}};
const Segment SGI262956[] = {{&S53080[125], 5}, {NULL}};
const Segment SGI262957[] = {{&S53080[138], 9}, {NULL}};
const Segment SGI262958[] = {{&S53081[13], 6}, {NULL}};
const Segment SGI262959[] = {{&S53081[112], 5}, {NULL}};
const Segment SGI262960[] = {{&S53081[38], 5}, {NULL}};
const Segment SGI262961[] = {{&S53081[78], 5}, {NULL}};
const Segment SGI262962[] = {{&S53081[125], 5}, {NULL}};
const Segment SGI262963[] = {{&S53081[138], 9}, {NULL}};
const Segment SGI262964[] = {{&S53082[13], 6}, {NULL}};
const Segment SGI262965[] = {{&S53082[112], 5}, {NULL}};
const Segment SGI262966[] = {{&S53082[38], 5}, {NULL}};
const Segment SGI262967[] = {{&S53082[78], 5}, {NULL}};
const Segment SGI262968[] = {{&S53082[125], 5}, {NULL}};
const Segment SGI262969[] = {{&S53082[138], 9}, {NULL}};
const Segment SGI262970[] = {{&S53083[13], 6}, {NULL}};
const Segment SGI262971[] = {{&S53083[112], 5}, {NULL}};
const Segment SGI262972[] = {{&S53083[38], 5}, {NULL}};
const Segment SGI262973[] = {{&S53083[78], 5}, {NULL}};
const Segment SGI262974[] = {{&S53083[125], 5}, {NULL}};
const Segment SGI262975[] = {{&S53083[138], 9}, {NULL}};
const Segment SGI262976[] = {{&S53084[13], 6}, {NULL}};
const Segment SGI262977[] = {{&S53084[110], 5}, {NULL}};
const Segment SGI262978[] = {{&S53084[38], 3}, {NULL}};
const Segment SGI262979[] = {{&S53084[76], 5}, {NULL}};
const Segment SGI262980[] = {{&S53084[123], 5}, {NULL}};
const Segment SGI262981[] = {{&S53084[136], 9}, {NULL}};
const Segment SGI262982[] = {{&S53085[13], 6}, {NULL}};
const Segment SGI262983[] = {{&S53085[112], 5}, {NULL}};
const Segment SGI262984[] = {{&S53085[38], 5}, {NULL}};
const Segment SGI262985[] = {{&S53085[78], 5}, {NULL}};
const Segment SGI262986[] = {{&S53085[125], 3}, {NULL}};
const Segment SGI262987[] = {{&S53085[136], 9}, {NULL}};
const Segment SGI262988[] = {{&S53086[13], 6}, {NULL}};
const Segment SGI262989[] = {{&S53086[115], 5}, {NULL}};
const Segment SGI262990[] = {{&S53086[38], 5}, {NULL}};
const Segment SGI262991[] = {{&S53086[78], 8}, {NULL}};
const Segment SGI262992[] = {{&S53086[128], 3}, {NULL}};
const Segment SGI262993[] = {{&S53086[139], 9}, {NULL}};
const Segment SGI262994[] = {{&S53087[13], 6}, {NULL}};
const Segment SGI262995[] = {{&S53087[112], 5}, {NULL}};
const Segment SGI262996[] = {{&S53087[38], 5}, {NULL}};
const Segment SGI262997[] = {{&S53087[78], 5}, {NULL}};
const Segment SGI262998[] = {{&S53087[125], 3}, {NULL}};
const Segment SGI262999[] = {{&S53087[136], 9}, {NULL}};
const Segment SGI263000[] = {{&S53088[13], 6}, {NULL}};
const Segment SGI263001[] = {{&S53088[112], 5}, {NULL}};
const Segment SGI263002[] = {{&S53088[38], 5}, {NULL}};
const Segment SGI263003[] = {{&S53088[78], 5}, {NULL}};
const Segment SGI263004[] = {{&S53088[125], 5}, {NULL}};
const Segment SGI263005[] = {{&S53089[13], 6}, {NULL}};
const Segment SGI263006[] = {{&S53089[112], 5}, {NULL}};
const Segment SGI263007[] = {{&S53089[38], 5}, {NULL}};
const Segment SGI263008[] = {{&S53089[78], 5}, {NULL}};
const Segment SGI263009[] = {{&S53089[125], 5}, {NULL}};
const Segment SGI263010[] = {{&S53090[13], 6}, {NULL}};
const Segment SGI263011[] = {{&S53090[112], 5}, {NULL}};
const Segment SGI263012[] = {{&S53090[38], 5}, {NULL}};
const Segment SGI263013[] = {{&S53090[78], 5}, {NULL}};
const Segment SGI263014[] = {{&S53090[125], 3}, {NULL}};
const Segment SGI263015[] = {{&S53090[136], 9}, {NULL}};
const Segment SGI263016[] = {{&S53091[13], 6}, {NULL}};
const Segment SGI263017[] = {{&S53091[113], 3}, {NULL}};
const Segment SGI263018[] = {{&S53091[38], 5}, {NULL}};
const Segment SGI263019[] = {{&S53091[78], 6}, {NULL}};
const Segment SGI263020[] = {{&S53091[124], 5}, {NULL}};
const Segment SGI263021[] = {{&S53091[137], 6}, {NULL}};
const Segment SGI263022[] = {{&S53092[13], 6}, {NULL}};
const Segment SGI263023[] = {{&S53092[113], 3}, {NULL}};
const Segment SGI263024[] = {{&S53092[38], 5}, {NULL}};
const Segment SGI263025[] = {{&S53092[78], 6}, {NULL}};
const Segment SGI263026[] = {{&S53092[124], 5}, {NULL}};
const Segment SGI263027[] = {{&S53092[137], 6}, {NULL}};
const Segment SGI263028[] = {{&S53093[13], 6}, {NULL}};
const Segment SGI263029[] = {{&S53093[113], 3}, {NULL}};
const Segment SGI263030[] = {{&S53093[38], 5}, {NULL}};
const Segment SGI263031[] = {{&S53093[78], 6}, {NULL}};
const Segment SGI263032[] = {{&S53093[124], 5}, {NULL}};
const Segment SGI263033[] = {{&S53093[137], 6}, {NULL}};
const Segment SGI263034[] = {{&S53094[13], 6}, {NULL}};
const Segment SGI263035[] = {{&S53094[112], 5}, {NULL}};
const Segment SGI263036[] = {{&S53094[38], 5}, {NULL}};
const Segment SGI263037[] = {{&S53094[78], 5}, {NULL}};
const Segment SGI263038[] = {{&S53094[125], 5}, {NULL}};
const Segment SGI263039[] = {{&S53095[13], 6}, {NULL}};
const Segment SGI263040[] = {{&S53095[113], 3}, {NULL}};
const Segment SGI263041[] = {{&S53095[38], 5}, {NULL}};
const Segment SGI263042[] = {{&S53095[78], 6}, {NULL}};
const Segment SGI263043[] = {{&S53095[124], 5}, {NULL}};
const Segment SGI263044[] = {{&S53095[137], 6}, {NULL}};
const Segment SGI263045[] = {{&S53096[13], 6}, {NULL}};
const Segment SGI263046[] = {{&S53096[111], 3}, {NULL}};
const Segment SGI263047[] = {{&S53096[38], 3}, {NULL}};
const Segment SGI263048[] = {{&S53096[76], 6}, {NULL}};
const Segment SGI263049[] = {{&S53096[122], 5}, {NULL}};
const Segment SGI263050[] = {{&S53096[135], 6}, {NULL}};
const Segment SGI263051[] = {{&S53097[13], 6}, {NULL}};
const Segment SGI263052[] = {{&S53097[110], 5}, {NULL}};
const Segment SGI263053[] = {{&S53097[38], 3}, {NULL}};
const Segment SGI263054[] = {{&S53097[76], 5}, {NULL}};
const Segment SGI263055[] = {{&S53097[123], 3}, {NULL}};
const Segment SGI263056[] = {{&S53097[134], 9}, {NULL}};
const Segment SGI263057[] = {{&S53098[13], 6}, {NULL}};
const Segment SGI263058[] = {{&S53098[110], 5}, {NULL}};
const Segment SGI263059[] = {{&S53098[38], 3}, {NULL}};
const Segment SGI263060[] = {{&S53098[76], 5}, {NULL}};
const Segment SGI263061[] = {{&S53098[123], 5}, {NULL}};
const Segment SGI263062[] = {{&S53099[13], 6}, {NULL}};
const Segment SGI263063[] = {{&S53099[112], 5}, {NULL}};
const Segment SGI263064[] = {{&S53099[38], 5}, {NULL}};
const Segment SGI263065[] = {{&S53099[78], 5}, {NULL}};
const Segment SGI263066[] = {{&S53099[125], 3}, {NULL}};
const Segment SGI263067[] = {{&S53099[136], 9}, {NULL}};
const Segment SGI263068[] = {{&S53100[13], 6}, {NULL}};
const Segment SGI263069[] = {{&S53100[112], 5}, {NULL}};
const Segment SGI263070[] = {{&S53100[38], 5}, {NULL}};
const Segment SGI263071[] = {{&S53100[78], 5}, {NULL}};
const Segment SGI263072[] = {{&S53100[125], 3}, {NULL}};
const Segment SGI263073[] = {{&S53100[136], 9}, {NULL}};
const Segment SGI263074[] = {{&S53101[13], 6}, {NULL}};
const Segment SGI263075[] = {{&S53101[110], 5}, {NULL}};
const Segment SGI263076[] = {{&S53101[38], 3}, {NULL}};
const Segment SGI263077[] = {{&S53101[76], 5}, {NULL}};
const Segment SGI263078[] = {{&S53101[123], 5}, {NULL}};
const Segment SGI263079[] = {{&S53101[136], 9}, {NULL}};
const Segment SGI263080[] = {{&S53102[13], 6}, {NULL}};
const Segment SGI263081[] = {{&S53102[112], 5}, {NULL}};
const Segment SGI263082[] = {{&S53102[38], 5}, {NULL}};
const Segment SGI263083[] = {{&S53102[78], 5}, {NULL}};
const Segment SGI263084[] = {{&S53102[125], 5}, {NULL}};
const Segment SGI263085[] = {{&S53103[13], 6}, {NULL}};
const Segment SGI263086[] = {{&S53103[112], 5}, {NULL}};
const Segment SGI263087[] = {{&S53103[38], 5}, {NULL}};
const Segment SGI263088[] = {{&S53103[78], 5}, {NULL}};
const Segment SGI263089[] = {{&S53103[125], 3}, {NULL}};
const Segment SGI263090[] = {{&S53103[136], 9}, {NULL}};
const Segment SGI263091[] = {{&S53104[13], 6}, {NULL}};
const Segment SGI263092[] = {{&S53104[112], 5}, {NULL}};
const Segment SGI263093[] = {{&S53104[38], 5}, {NULL}};
const Segment SGI263094[] = {{&S53104[78], 5}, {NULL}};
const Segment SGI263095[] = {{&S53104[125], 5}, {NULL}};
const Segment SGI263096[] = {{&S53104[138], 9}, {NULL}};
const Segment SGI263097[] = {{&S53105[13], 6}, {NULL}};
const Segment SGI263098[] = {{&S53105[112], 5}, {NULL}};
const Segment SGI263099[] = {{&S53105[38], 5}, {NULL}};
const Segment SGI263100[] = {{&S53105[78], 5}, {NULL}};
const Segment SGI263101[] = {{&S53105[125], 5}, {NULL}};
const Segment SGI263102[] = {{&S53105[138], 9}, {NULL}};
const Segment SGI263103[] = {{&S53153[13], 4}, {NULL}};
const Segment SGI263104[] = {{&S53153[33], 3}, {NULL}};
const Segment SGI263105[] = {{&S53153[71], 8}, {NULL}};
const Segment SGI263106[] = {{&S53153[126], 6}, {NULL}};
const Segment SGI263107[] = {{&S53153[140], 9}, {NULL}};
const Segment SGI263108[] = {{&S53154[13], 4}, {NULL}};
const Segment SGI263109[] = {{&S53154[33], 3}, {NULL}};
const Segment SGI263110[] = {{&S53154[71], 8}, {NULL}};
const Segment SGI263111[] = {{&S53154[126], 6}, {NULL}};
const Segment SGI263112[] = {{&S53154[140], 9}, {NULL}};
const Segment SGI263113[] = {{&S53155[13], 4}, {NULL}};
const Segment SGI263114[] = {{&S53155[33], 5}, {NULL}};
const Segment SGI263115[] = {{&S53155[73], 8}, {NULL}};
const Segment SGI263116[] = {{&S53155[128], 6}, {NULL}};
const Segment SGI263117[] = {{&S53155[142], 9}, {NULL}};
const Segment SGI263118[] = {{&S53156[13], 4}, {NULL}};
const Segment SGI263119[] = {{&S53156[33], 5}, {NULL}};
const Segment SGI263120[] = {{&S53156[73], 8}, {NULL}};
const Segment SGI263121[] = {{&S53156[128], 6}, {NULL}};
const Segment SGI263122[] = {{&S53156[142], 9}, {NULL}};
const Segment SGI263123[] = {{&S45633[0], 11}, {NULL}};
const Segment SGI263124[] = {{&S45633[0], 51}, {NULL}};
const Segment SGI263125[] = {{&S38637[10], 4}, {NULL}};
const Segment SGI263126[] = {{&S38637[15], 5}, {NULL}};
const Segment SGI263127[] = {{&S53508[82], 4}, {NULL}};
const Segment SGI263128[] = {{&S53580[13], 4}, {NULL}};
const Segment SGI263129[] = {{&S53580[36], 3}, {NULL}};
const Segment SGI263130[] = {{&S53580[74], 5}, {NULL}};
const Segment SGI263131[] = {{&S53581[13], 4}, {NULL}};
const Segment SGI263132[] = {{&S53581[36], 3}, {NULL}};
const Segment SGI263133[] = {{&S53581[74], 5}, {NULL}};
const Segment SGI263134[] = {{&S53581[107], 5}, {NULL}};
const Segment SGI263135[] = {{&S53582[13], 4}, {NULL}};
const Segment SGI263136[] = {{&S53582[36], 5}, {NULL}};
const Segment SGI263137[] = {{&S53582[76], 5}, {NULL}};
const Segment SGI263138[] = {{&S53582[109], 5}, {NULL}};
const Segment SGI263139[] = {{&S53583[13], 4}, {NULL}};
const Segment SGI263140[] = {{&S53583[36], 5}, {NULL}};
const Segment SGI263141[] = {{&S53583[76], 5}, {NULL}};
const Segment SGI263142[] = {{&S53584[13], 4}, {&S53584[13], 4}, {&S53584[13], 4}, {NULL}};
const Segment SGI263143[] = {{&S53584[42], 3}, {NULL}};
const Segment SGI263144[] = {{&S53585[13], 4}, {&S53585[13], 4}, {&S53585[13], 4}, {NULL}};
const Segment SGI263145[] = {{&S53585[42], 5}, {NULL}};
const Segment SGI263146[] = {{&S53586[13], 4}, {&S53586[13], 4}, {NULL}};
const Segment SGI263147[] = {{&S53586[36], 3}, {NULL}};
const Segment SGI263148[] = {{&S53586[74], 5}, {NULL}};
const Segment SGI263149[] = {{&S53586[107], 5}, {NULL}};
const Segment SGI263150[] = {{&S38930[10], 4}, {NULL}};
const Segment SGI263151[] = {{&S38930[15], 5}, {NULL}};
const Segment SGI263152[] = {{&S38931[10], 4}, {NULL}};
const Segment SGI263153[] = {{&S38931[15], 5}, {NULL}};
const Segment SGI263154[] = {{&S53659[13], 4}, {NULL}};
const Segment SGI263155[] = {{&S53659[33], 5}, {NULL}};
const Segment SGI263156[] = {{&S53659[73], 8}, {NULL}};
const Segment SGI263157[] = {{&S53659[128], 5}, {NULL}};
const Segment SGI263158[] = {{&S53659[141], 9}, {NULL}};
const Segment SGI263159[] = {{&S53660[13], 4}, {NULL}};
const Segment SGI263160[] = {{&S53660[33], 3}, {NULL}};
const Segment SGI263161[] = {{&S53660[71], 8}, {NULL}};
const Segment SGI263162[] = {{&S53660[126], 5}, {NULL}};
const Segment SGI263163[] = {{&S53660[139], 9}, {NULL}};
const Segment SGI263164[] = {{&S53661[13], 4}, {NULL}};
const Segment SGI263165[] = {{&S53661[33], 3}, {NULL}};
const Segment SGI263166[] = {{&S53661[71], 8}, {NULL}};
const Segment SGI263167[] = {{&S53661[125], 5}, {NULL}};
const Segment SGI263168[] = {{&S53661[138], 9}, {NULL}};
const Segment SGI263169[] = {{&S53678[13], 4}, {NULL}};
const Segment SGI263170[] = {{&S53678[33], 5}, {NULL}};
const Segment SGI263171[] = {{&S53678[66], 6}, {NULL}};
const Segment SGI263172[] = {{&S53678[119], 6}, {NULL}};
const Segment SGI263173[] = {{&S53678[133], 7}, {NULL}};
const Segment SGI263174[] = {{&S53679[13], 4}, {NULL}};
const Segment SGI263175[] = {{&S53679[33], 3}, {NULL}};
const Segment SGI263176[] = {{&S53679[64], 6}, {NULL}};
const Segment SGI263177[] = {{&S53679[117], 6}, {NULL}};
const Segment SGI263178[] = {{&S53679[131], 7}, {NULL}};
const Segment SGI263179[] = {{&S53680[13], 4}, {NULL}};
const Segment SGI263180[] = {{&S53680[33], 3}, {NULL}};
const Segment SGI263181[] = {{&S53680[64], 6}, {NULL}};
const Segment SGI263182[] = {{&S53680[117], 6}, {NULL}};
const Segment SGI263183[] = {{&S53680[131], 7}, {NULL}};
const Segment SGI263184[] = {{&S53727[13], 6}, {NULL}};
const Segment SGI263185[] = {{&S53727[35], 5}, {NULL}};
const Segment SGI263186[] = {{&S53727[68], 6}, {NULL}};
const Segment SGI263187[] = {{&S53727[114], 5}, {NULL}};
const Segment SGI263188[] = {{&S53727[127], 9}, {NULL}};
const Segment SGI263189[] = {{&S53728[13], 6}, {NULL}};
const Segment SGI263190[] = {{&S53728[35], 3}, {NULL}};
const Segment SGI263191[] = {{&S53728[66], 6}, {NULL}};
const Segment SGI263192[] = {{&S53728[112], 5}, {NULL}};
const Segment SGI263193[] = {{&S53728[125], 9}, {NULL}};
const Segment SGI263194[] = {{&S53731[13], 4}, {NULL}};
const Segment SGI263195[] = {{&S53731[33], 3}, {NULL}};
const Segment SGI263196[] = {{&S53731[64], 6}, {NULL}};
const Segment SGI263197[] = {{&S53731[117], 6}, {NULL}};
const Segment SGI263198[] = {{&S53731[131], 7}, {NULL}};
const Segment SGI263199[] = {{&S53732[13], 4}, {NULL}};
const Segment SGI263200[] = {{&S53732[33], 5}, {NULL}};
const Segment SGI263201[] = {{&S53732[66], 6}, {NULL}};
const Segment SGI263202[] = {{&S53732[119], 6}, {NULL}};
const Segment SGI263203[] = {{&S53732[133], 7}, {NULL}};
const Segment SGI263204[] = {{&S53733[13], 4}, {NULL}};
const Segment SGI263205[] = {{&S53733[33], 3}, {NULL}};
const Segment SGI263206[] = {{&S53733[64], 6}, {NULL}};
const Segment SGI263207[] = {{&S53733[117], 6}, {NULL}};
const Segment SGI263208[] = {{&S53733[131], 7}, {NULL}};
const Segment SGI263209[] = {{&S53824[13], 6}, {NULL}};
const Segment SGI263210[] = {{&S53824[125], 2}, {NULL}};
const Segment SGI263211[] = {{&S53824[35], 5}, {NULL}};
const Segment SGI263212[] = {{&S53824[68], 6}, {NULL}};
const Segment SGI263213[] = {{&S53824[35], 1}, {NULL}};
const Segment SGI263214[] = {{&S53824[152], 7}, {NULL}};
const Segment SGI263215[] = {{&S53825[13], 6}, {NULL}};
const Segment SGI263216[] = {{&S53825[125], 2}, {NULL}};
const Segment SGI263217[] = {{&S53825[35], 5}, {NULL}};
const Segment SGI263218[] = {{&S53825[68], 6}, {NULL}};
const Segment SGI263219[] = {{&S53825[35], 1}, {NULL}};
const Segment SGI263220[] = {{&S53825[152], 7}, {NULL}};
const Segment SGI263221[] = {{&S53826[13], 6}, {NULL}};
const Segment SGI263222[] = {{&S53826[125], 2}, {NULL}};
const Segment SGI263223[] = {{&S53826[35], 5}, {NULL}};
const Segment SGI263224[] = {{&S53826[68], 6}, {NULL}};
const Segment SGI263225[] = {{&S53826[35], 1}, {NULL}};
const Segment SGI263226[] = {{&S53826[152], 7}, {NULL}};
const Segment SGI263227[] = {{&S53833[13], 4}, {NULL}};
const Segment SGI263228[] = {{&S53833[33], 3}, {NULL}};
const Segment SGI263229[] = {{&S53833[64], 6}, {NULL}};
const Segment SGI263230[] = {{&S53833[117], 5}, {NULL}};
const Segment SGI263231[] = {{&S53833[130], 9}, {NULL}};
const Segment SGI263232[] = {{&S53834[13], 4}, {NULL}};
const Segment SGI263233[] = {{&S53834[33], 5}, {NULL}};
const Segment SGI263234[] = {{&S53834[66], 6}, {NULL}};
const Segment SGI263235[] = {{&S53834[119], 5}, {NULL}};
const Segment SGI263236[] = {{&S53834[132], 9}, {NULL}};
const Segment SGI263237[] = {{&S53885[13], 6}, {NULL}};
const Segment SGI263238[] = {{&S53885[110], 5}, {NULL}};
const Segment SGI263239[] = {{&S53885[38], 3}, {NULL}};
const Segment SGI263240[] = {{&S53885[76], 5}, {NULL}};
const Segment SGI263241[] = {{&S53885[123], 5}, {NULL}};
const Segment SGI263242[] = {{&S53885[136], 9}, {NULL}};
const Segment SGI263243[] = {{&S53891[13], 4}, {NULL}};
const Segment SGI263244[] = {{&S53891[36], 3}, {NULL}};
const Segment SGI263245[] = {{&S53891[74], 8}, {NULL}};
const Segment SGI263246[] = {{&S53891[110], 5}, {NULL}};
const Segment SGI263247[] = {{&S53891[116], 7}, {NULL}};
const Segment SGI263248[] = {{&S53892[13], 4}, {NULL}};
const Segment SGI263249[] = {{&S53892[36], 5}, {NULL}};
const Segment SGI263250[] = {{&S53892[76], 8}, {NULL}};
const Segment SGI263251[] = {{&S53892[112], 3}, {NULL}};
const Segment SGI263252[] = {{&S53892[116], 7}, {NULL}};
const Segment SGI263253[] = {{&S53893[13], 4}, {NULL}};
const Segment SGI263254[] = {{&S53893[36], 5}, {NULL}};
const Segment SGI263255[] = {{&S53893[76], 8}, {NULL}};
const Segment SGI263256[] = {{&S53893[112], 3}, {NULL}};
const Segment SGI263257[] = {{&S53893[116], 7}, {NULL}};
const Segment SGI263258[] = {{&S53894[13], 4}, {NULL}};
const Segment SGI263259[] = {{&S53894[36], 5}, {NULL}};
const Segment SGI263260[] = {{&S53894[76], 8}, {NULL}};
const Segment SGI263261[] = {{&S53894[112], 3}, {NULL}};
const Segment SGI263262[] = {{&S53894[116], 7}, {NULL}};
const Segment SGI263263[] = {{&S53895[13], 4}, {NULL}};
const Segment SGI263264[] = {{&S53895[36], 5}, {NULL}};
const Segment SGI263265[] = {{&S53895[76], 8}, {NULL}};
const Segment SGI263266[] = {{&S53895[112], 3}, {NULL}};
const Segment SGI263267[] = {{&S53895[116], 7}, {NULL}};
const Segment SGI263268[] = {{&S53896[13], 4}, {NULL}};
const Segment SGI263269[] = {{&S53896[36], 5}, {NULL}};
const Segment SGI263270[] = {{&S53896[76], 8}, {NULL}};
const Segment SGI263271[] = {{&S53896[112], 3}, {NULL}};
const Segment SGI263272[] = {{&S53896[116], 7}, {NULL}};
const Segment SGI263273[] = {{&S53940[13], 4}, {NULL}};
const Segment SGI263274[] = {{&S53940[33], 5}, {NULL}};
const Segment SGI263275[] = {{&S53940[66], 6}, {NULL}};
const Segment SGI263276[] = {{&S53940[119], 6}, {NULL}};
const Segment SGI263277[] = {{&S53940[133], 7}, {NULL}};
const Segment SGI263278[] = {{&S53941[13], 4}, {NULL}};
const Segment SGI263279[] = {{&S53941[33], 5}, {NULL}};
const Segment SGI263280[] = {{&S53941[66], 6}, {NULL}};
const Segment SGI263281[] = {{&S53941[119], 6}, {NULL}};
const Segment SGI263282[] = {{&S53941[133], 7}, {NULL}};
const Segment SGI263283[] = {{&S53942[13], 4}, {NULL}};
const Segment SGI263284[] = {{&S53942[33], 3}, {NULL}};
const Segment SGI263285[] = {{&S53942[64], 6}, {NULL}};
const Segment SGI263286[] = {{&S53942[117], 6}, {NULL}};
const Segment SGI263287[] = {{&S53942[131], 7}, {NULL}};
const Segment SGI263288[] = {{&S53943[13], 4}, {NULL}};
const Segment SGI263289[] = {{&S53943[33], 3}, {NULL}};
const Segment SGI263290[] = {{&S53943[64], 6}, {NULL}};
const Segment SGI263291[] = {{&S53943[117], 6}, {NULL}};
const Segment SGI263292[] = {{&S53943[131], 7}, {NULL}};
const Segment SGI263293[] = {{&S54049[13], 4}, {&S54049[13], 4}, {NULL}};
const Segment SGI263294[] = {{&S54049[26], 3}, {NULL}};
const Segment SGI263295[] = {{&S54049[57], 6}, {NULL}};
const Segment SGI263296[] = {{&S54049[91], 6}, {NULL}};
const Segment SGI263297[] = {{&S54049[98], 7}, {NULL}};
const Segment SGI263298[] = {{&S54050[13], 4}, {&S54050[13], 4}, {NULL}};
const Segment SGI263299[] = {{&S54050[26], 5}, {NULL}};
const Segment SGI263300[] = {{&S54050[59], 6}, {NULL}};
const Segment SGI263301[] = {{&S54050[93], 6}, {NULL}};
const Segment SGI263302[] = {{&S54050[100], 7}, {NULL}};
const Segment SGI263303[] = {{&S54051[13], 4}, {&S54051[13], 4}, {NULL}};
const Segment SGI263304[] = {{&S54051[26], 5}, {NULL}};
const Segment SGI263305[] = {{&S54051[59], 6}, {NULL}};
const Segment SGI263306[] = {{&S54051[93], 6}, {NULL}};
const Segment SGI263307[] = {{&S54051[100], 7}, {NULL}};
const Segment SGI263308[] = {{&S54052[13], 4}, {&S54052[13], 4}, {NULL}};
const Segment SGI263309[] = {{&S54052[26], 5}, {NULL}};
const Segment SGI263310[] = {{&S54052[59], 6}, {NULL}};
const Segment SGI263311[] = {{&S54052[93], 7}, {NULL}};
const Segment SGI263312[] = {{&S54052[101], 7}, {NULL}};
const Segment SGI263313[] = {{&S54053[13], 4}, {&S54053[13], 4}, {NULL}};
const Segment SGI263314[] = {{&S54053[26], 3}, {NULL}};
const Segment SGI263315[] = {{&S54053[57], 6}, {NULL}};
const Segment SGI263316[] = {{&S54053[91], 6}, {NULL}};
const Segment SGI263317[] = {{&S54053[98], 7}, {NULL}};
const Segment SGI263318[] = {{&S54057[13], 4}, {NULL}};
const Segment SGI263319[] = {{&S54057[33], 5}, {NULL}};
const Segment SGI263320[] = {{&S54057[66], 6}, {NULL}};
const Segment SGI263321[] = {{&S54057[120], 5}, {NULL}};
const Segment SGI263322[] = {{&S54057[133], 9}, {NULL}};
const Segment SGI263323[] = {{&S54058[13], 4}, {NULL}};
const Segment SGI263324[] = {{&S54058[33], 3}, {NULL}};
const Segment SGI263325[] = {{&S54058[64], 6}, {NULL}};
const Segment SGI263326[] = {{&S54058[118], 5}, {NULL}};
const Segment SGI263327[] = {{&S54058[131], 9}, {NULL}};
const Segment SGI263328[] = {{&S54059[8], 6}, {NULL}};
const Segment SGI263329[] = {{&S54061[10], 6}, {NULL}};
const Segment SGI263330[] = {{&S54061[57], 5}, {NULL}};
const Segment SGI263331[] = {{&S39533[10], 4}, {NULL}};
const Segment SGI263332[] = {{&S39533[15], 5}, {NULL}};
const Segment SGI263333[] = {{&S54198[28], 6}, {NULL}};
const Segment SGI263334[] = {{&S54198[46], 3}, {NULL}};
const Segment SGI263335[] = {{&S54199[28], 6}, {NULL}};
const Segment SGI263336[] = {{&S54199[46], 5}, {NULL}};
const Segment SGI263337[] = {{&S54464[13], 4}, {NULL}};
const Segment SGI263338[] = {{&S54464[36], 5}, {NULL}};
const Segment SGI263339[] = {{&S54464[76], 8}, {NULL}};
const Segment SGI263340[] = {{&S54464[132], 3}, {NULL}};
const Segment SGI263341[] = {{&S54464[143], 9}, {NULL}};
const Segment SGI263342[] = {{&S54465[13], 4}, {NULL}};
const Segment SGI263343[] = {{&S54465[36], 5}, {NULL}};
const Segment SGI263344[] = {{&S54465[76], 8}, {NULL}};
const Segment SGI263345[] = {{&S54465[132], 3}, {NULL}};
const Segment SGI263346[] = {{&S54465[143], 9}, {NULL}};
const Segment SGI263347[] = {{&S54466[13], 4}, {NULL}};
const Segment SGI263348[] = {{&S54466[36], 5}, {NULL}};
const Segment SGI263349[] = {{&S54466[76], 8}, {NULL}};
const Segment SGI263350[] = {{&S54466[132], 3}, {NULL}};
const Segment SGI263351[] = {{&S54466[143], 9}, {NULL}};
const Segment SGI263352[] = {{&S54467[13], 4}, {NULL}};
const Segment SGI263353[] = {{&S54467[36], 3}, {NULL}};
const Segment SGI263354[] = {{&S54467[74], 8}, {NULL}};
const Segment SGI263355[] = {{&S54467[130], 5}, {NULL}};
const Segment SGI263356[] = {{&S54467[143], 9}, {NULL}};
const Segment SGI263357[] = {{&S54468[13], 4}, {NULL}};
const Segment SGI263358[] = {{&S54468[36], 5}, {NULL}};
const Segment SGI263359[] = {{&S54468[76], 8}, {NULL}};
const Segment SGI263360[] = {{&S54468[132], 3}, {NULL}};
const Segment SGI263361[] = {{&S54468[143], 9}, {NULL}};
const Segment SGI263362[] = {{&S54469[13], 4}, {NULL}};
const Segment SGI263363[] = {{&S54469[36], 5}, {NULL}};
const Segment SGI263364[] = {{&S54469[76], 8}, {NULL}};
const Segment SGI263365[] = {{&S54469[132], 3}, {NULL}};
const Segment SGI263366[] = {{&S54469[143], 9}, {NULL}};
const Segment SGI263367[] = {{&S54502[12], 4}, {&S54502[12], 4}, {NULL}};
const Segment SGI263368[] = {{&S54502[27], 3}, {NULL}};
const Segment SGI263369[] = {{&S54502[74], 6}, {NULL}};
const Segment SGI263370[] = {{&S54502[137], 9}, {NULL}};
const Segment SGI263371[] = {{&S54502[101], 13}, {NULL}};
const Segment SGI263372[] = {{&S54630[13], 4}, {NULL}};
const Segment SGI263373[] = {{&S54630[8], 3}, {NULL}};
const Segment SGI263374[] = {{&S54630[64], 6}, {NULL}};
const Segment SGI263375[] = {{&S54631[13], 4}, {NULL}};
const Segment SGI263376[] = {{&S54631[92], 5}, {NULL}};
const Segment SGI263377[] = {{&S54631[57], 6}, {NULL}};
const Segment SGI263378[] = {{&S54632[13], 4}, {NULL}};
const Segment SGI263379[] = {{&S54632[92], 5}, {NULL}};
const Segment SGI263380[] = {{&S54632[57], 6}, {NULL}};
const Segment SGI263381[] = {{&S54636[13], 6}, {NULL}};
const Segment SGI263382[] = {{&S54636[35], 5}, {NULL}};
const Segment SGI263383[] = {{&S54636[68], 6}, {NULL}};
const Segment SGI263384[] = {{&S54636[121], 5}, {NULL}};
const Segment SGI263385[] = {{&S54636[134], 9}, {NULL}};
const Segment SGI263386[] = {{&S54637[13], 6}, {NULL}};
const Segment SGI263387[] = {{&S54637[35], 5}, {NULL}};
const Segment SGI263388[] = {{&S54637[68], 6}, {NULL}};
const Segment SGI263389[] = {{&S54637[121], 5}, {NULL}};
const Segment SGI263390[] = {{&S54637[134], 9}, {NULL}};
const Segment SGI263391[] = {{&S54638[13], 6}, {NULL}};
const Segment SGI263392[] = {{&S54638[35], 3}, {NULL}};
const Segment SGI263393[] = {{&S54638[66], 6}, {NULL}};
const Segment SGI263394[] = {{&S54638[119], 5}, {NULL}};
const Segment SGI263395[] = {{&S54638[132], 9}, {NULL}};
const Segment SGI263396[] = {{&S54639[13], 6}, {NULL}};
const Segment SGI263397[] = {{&S54639[35], 3}, {NULL}};
const Segment SGI263398[] = {{&S54639[66], 6}, {NULL}};
const Segment SGI263399[] = {{&S54639[119], 5}, {NULL}};
const Segment SGI263400[] = {{&S54639[132], 9}, {NULL}};
const Segment SGI263401[] = {{&S54643[13], 4}, {NULL}};
const Segment SGI263402[] = {{&S54643[26], 3}, {NULL}};
const Segment SGI263403[] = {{&S54643[57], 6}, {NULL}};
const Segment SGI263404[] = {{&S54643[110], 6}, {NULL}};
const Segment SGI263405[] = {{&S54643[124], 7}, {NULL}};
const Segment SGI263406[] = {{&S54644[13], 4}, {NULL}};
const Segment SGI263407[] = {{&S54644[26], 5}, {NULL}};
const Segment SGI263408[] = {{&S54644[59], 6}, {NULL}};
const Segment SGI263409[] = {{&S54644[112], 6}, {NULL}};
const Segment SGI263410[] = {{&S54644[126], 7}, {NULL}};
const Segment SGI263411[] = {{&S47127[22], 7}, {NULL}};
const Segment SGI263412[] = {{&S47127[0], 111}, {NULL}};
const Segment SGI263413[] = {{&S54738[13], 4}, {NULL}};
const Segment SGI263414[] = {{&S54738[26], 5}, {NULL}};
const Segment SGI263415[] = {{&S54738[59], 6}, {NULL}};
const Segment SGI263416[] = {{&S54738[112], 5}, {NULL}};
const Segment SGI263417[] = {{&S54738[125], 9}, {NULL}};
const Segment SGI263418[] = {{&S54739[13], 4}, {NULL}};
const Segment SGI263419[] = {{&S54739[26], 3}, {NULL}};
const Segment SGI263420[] = {{&S54739[57], 6}, {NULL}};
const Segment SGI263421[] = {{&S54739[110], 5}, {NULL}};
const Segment SGI263422[] = {{&S54739[123], 9}, {NULL}};
const Segment SGI263423[] = {{&S54773[20], 4}, {NULL}};
const Segment SGI263424[] = {{&S54773[36], 3}, {NULL}};
const Segment SGI263425[] = {{&S54774[20], 4}, {NULL}};
const Segment SGI263426[] = {{&S54774[36], 5}, {NULL}};
const Segment SGI263427[] = {{&S54775[29], 4}, {NULL}};
const Segment SGI263428[] = {{&S54776[20], 4}, {NULL}};
const Segment SGI263429[] = {{&S54776[36], 5}, {NULL}};
const Segment SGI263430[] = {{&S47341[90], 9}, {NULL}};
const Segment SGI263431[] = {{&S47341[0], 99}, {NULL}};
const Segment SGI263432[] = {{&S47342[15], 3}, {NULL}};
const Segment SGI263433[] = {{&S47342[0], 106}, {NULL}};
const Segment SGI263434[] = {{&S47343[18], 3}, {NULL}};
const Segment SGI263435[] = {{&S47343[0], 106}, {NULL}};
const Segment SGI263436[] = {{&S40300[22], 12}, {NULL}};
const Segment SGI263437[] = {{&S47703[0], 53}, {NULL}};
const Segment SGI263438[] = {{&S47704[0], 52}, {NULL}};
const Segment SGI263439[] = {{&S55356[13], 6}, {NULL}};
const Segment SGI263440[] = {{&S55356[103], 3}, {NULL}};
const Segment SGI263441[] = {{&S55356[35], 5}, {NULL}};
const Segment SGI263442[] = {{&S55356[68], 6}, {NULL}};
const Segment SGI263443[] = {{&S55356[114], 6}, {NULL}};
const Segment SGI263444[] = {{&S55356[128], 7}, {NULL}};
const Segment SGI263445[] = {{&S55357[13], 6}, {NULL}};
const Segment SGI263446[] = {{&S55357[101], 3}, {NULL}};
const Segment SGI263447[] = {{&S55357[35], 3}, {NULL}};
const Segment SGI263448[] = {{&S55357[66], 6}, {NULL}};
const Segment SGI263449[] = {{&S55357[112], 6}, {NULL}};
const Segment SGI263450[] = {{&S55357[126], 7}, {NULL}};
const Segment SGI263451[] = {{&S55358[13], 6}, {NULL}};
const Segment SGI263452[] = {{&S55358[103], 3}, {NULL}};
const Segment SGI263453[] = {{&S55358[35], 5}, {NULL}};
const Segment SGI263454[] = {{&S55358[68], 6}, {NULL}};
const Segment SGI263455[] = {{&S55358[114], 6}, {NULL}};
const Segment SGI263456[] = {{&S55358[128], 7}, {NULL}};
const Segment SGI263457[] = {{&S55359[13], 6}, {NULL}};
const Segment SGI263458[] = {{&S55359[103], 3}, {NULL}};
const Segment SGI263459[] = {{&S55359[35], 5}, {NULL}};
const Segment SGI263460[] = {{&S55359[68], 6}, {NULL}};
const Segment SGI263461[] = {{&S55359[114], 6}, {NULL}};
const Segment SGI263462[] = {{&S55359[128], 7}, {NULL}};
const Segment SGI263463[] = {{&S55395[27], 6}, {NULL}};
const Segment SGI263464[] = {{&S55395[51], 5}, {NULL}};
const Segment SGI263465[] = {{&S55395[84], 6}, {NULL}};
const Segment SGI263466[] = {{&S55395[118], 6}, {NULL}};
const Segment SGI263467[] = {{&S55395[132], 9}, {NULL}};
const Segment SGI263468[] = {{&S55395[0], 14}, {NULL}};
const Segment SGI263469[] = {{&S55396[27], 6}, {NULL}};
const Segment SGI263470[] = {{&S55396[52], 5}, {NULL}};
const Segment SGI263471[] = {{&S55396[85], 6}, {NULL}};
const Segment SGI263472[] = {{&S55396[119], 6}, {NULL}};
const Segment SGI263473[] = {{&S55396[133], 9}, {NULL}};
const Segment SGI263474[] = {{&S55396[0], 14}, {NULL}};
const Segment SGI263475[] = {{&S55397[27], 6}, {NULL}};
const Segment SGI263476[] = {{&S55397[52], 5}, {NULL}};
const Segment SGI263477[] = {{&S55397[85], 6}, {NULL}};
const Segment SGI263478[] = {{&S55397[119], 6}, {NULL}};
const Segment SGI263479[] = {{&S55397[133], 9}, {NULL}};
const Segment SGI263480[] = {{&S55397[0], 14}, {NULL}};
const Segment SGI263481[] = {{&S55398[27], 6}, {NULL}};
const Segment SGI263482[] = {{&S55398[51], 3}, {NULL}};
const Segment SGI263483[] = {{&S55398[82], 6}, {NULL}};
const Segment SGI263484[] = {{&S55398[116], 6}, {NULL}};
const Segment SGI263485[] = {{&S55398[130], 9}, {NULL}};
const Segment SGI263486[] = {{&S55398[0], 14}, {NULL}};
const Segment SGI263487[] = {{&S55399[27], 6}, {NULL}};
const Segment SGI263488[] = {{&S55399[51], 3}, {NULL}};
const Segment SGI263489[] = {{&S55399[82], 6}, {NULL}};
const Segment SGI263490[] = {{&S55399[116], 6}, {NULL}};
const Segment SGI263491[] = {{&S55399[130], 9}, {NULL}};
const Segment SGI263492[] = {{&S55399[0], 14}, {NULL}};
const Segment SGI263493[] = {{&S55400[27], 6}, {NULL}};
const Segment SGI263494[] = {{&S55400[52], 3}, {NULL}};
const Segment SGI263495[] = {{&S55400[83], 6}, {NULL}};
const Segment SGI263496[] = {{&S55400[117], 6}, {NULL}};
const Segment SGI263497[] = {{&S55400[131], 9}, {NULL}};
const Segment SGI263498[] = {{&S55400[0], 14}, {NULL}};
const Segment SGI263499[] = {{&S55401[27], 6}, {NULL}};
const Segment SGI263500[] = {{&S55401[52], 5}, {NULL}};
const Segment SGI263501[] = {{&S55401[85], 6}, {NULL}};
const Segment SGI263502[] = {{&S55401[119], 6}, {NULL}};
const Segment SGI263503[] = {{&S55401[133], 9}, {NULL}};
const Segment SGI263504[] = {{&S55401[0], 14}, {NULL}};
const Segment SGI263505[] = {{&S55419[27], 6}, {NULL}};
const Segment SGI263506[] = {{&S55419[51], 5}, {NULL}};
const Segment SGI263507[] = {{&S55419[84], 6}, {NULL}};
const Segment SGI263508[] = {{&S55419[118], 6}, {NULL}};
const Segment SGI263509[] = {{&S55419[132], 9}, {NULL}};
const Segment SGI263510[] = {{&S55419[0], 14}, {NULL}};
const Segment SGI263511[] = {{&S55420[27], 6}, {NULL}};
const Segment SGI263512[] = {{&S55420[52], 5}, {NULL}};
const Segment SGI263513[] = {{&S55420[85], 6}, {NULL}};
const Segment SGI263514[] = {{&S55420[119], 6}, {NULL}};
const Segment SGI263515[] = {{&S55420[133], 9}, {NULL}};
const Segment SGI263516[] = {{&S55420[0], 14}, {NULL}};
const Segment SGI263517[] = {{&S55421[27], 6}, {NULL}};
const Segment SGI263518[] = {{&S55421[51], 3}, {NULL}};
const Segment SGI263519[] = {{&S55421[82], 6}, {NULL}};
const Segment SGI263520[] = {{&S55421[116], 6}, {NULL}};
const Segment SGI263521[] = {{&S55421[130], 9}, {NULL}};
const Segment SGI263522[] = {{&S55421[0], 14}, {NULL}};
const Segment SGI263523[] = {{&S55422[27], 6}, {NULL}};
const Segment SGI263524[] = {{&S55422[52], 3}, {NULL}};
const Segment SGI263525[] = {{&S55422[83], 6}, {NULL}};
const Segment SGI263526[] = {{&S55422[117], 6}, {NULL}};
const Segment SGI263527[] = {{&S55422[131], 9}, {NULL}};
const Segment SGI263528[] = {{&S55422[0], 14}, {NULL}};
const Segment SGI263529[] = {{&S55423[28], 6}, {NULL}};
const Segment SGI263530[] = {{&S55423[52], 5}, {NULL}};
const Segment SGI263531[] = {{&S55423[85], 6}, {NULL}};
const Segment SGI263532[] = {{&S55423[119], 6}, {NULL}};
const Segment SGI263533[] = {{&S55423[133], 9}, {NULL}};
const Segment SGI263534[] = {{&S55423[0], 15}, {NULL}};
const Segment SGI263535[] = {{&S55424[27], 6}, {NULL}};
const Segment SGI263536[] = {{&S55424[52], 5}, {NULL}};
const Segment SGI263537[] = {{&S55424[85], 6}, {NULL}};
const Segment SGI263538[] = {{&S55424[119], 6}, {NULL}};
const Segment SGI263539[] = {{&S55424[133], 9}, {NULL}};
const Segment SGI263540[] = {{&S55424[0], 14}, {NULL}};
const Segment SGI263541[] = {{&S55425[28], 6}, {NULL}};
const Segment SGI263542[] = {{&S55425[53], 5}, {NULL}};
const Segment SGI263543[] = {{&S55425[86], 6}, {NULL}};
const Segment SGI263544[] = {{&S55425[120], 6}, {NULL}};
const Segment SGI263545[] = {{&S55425[134], 9}, {NULL}};
const Segment SGI263546[] = {{&S55425[0], 15}, {NULL}};
const Segment SGI263547[] = {{&S55426[27], 6}, {NULL}};
const Segment SGI263548[] = {{&S55426[51], 3}, {NULL}};
const Segment SGI263549[] = {{&S55426[82], 6}, {NULL}};
const Segment SGI263550[] = {{&S55426[116], 6}, {NULL}};
const Segment SGI263551[] = {{&S55426[130], 9}, {NULL}};
const Segment SGI263552[] = {{&S55426[0], 14}, {NULL}};
const Segment SGI263553[] = {{&S55427[28], 6}, {NULL}};
const Segment SGI263554[] = {{&S55427[53], 5}, {NULL}};
const Segment SGI263555[] = {{&S55427[86], 6}, {NULL}};
const Segment SGI263556[] = {{&S55427[120], 6}, {NULL}};
const Segment SGI263557[] = {{&S55427[134], 9}, {NULL}};
const Segment SGI263558[] = {{&S55427[0], 15}, {NULL}};
const Segment SGI263559[] = {{&S55428[28], 6}, {NULL}};
const Segment SGI263560[] = {{&S55428[53], 3}, {NULL}};
const Segment SGI263561[] = {{&S55428[84], 6}, {NULL}};
const Segment SGI263562[] = {{&S55428[118], 6}, {NULL}};
const Segment SGI263563[] = {{&S55428[132], 9}, {NULL}};
const Segment SGI263564[] = {{&S55428[0], 15}, {NULL}};
const Segment SGI263565[] = {{&S55429[28], 6}, {NULL}};
const Segment SGI263566[] = {{&S55429[52], 3}, {NULL}};
const Segment SGI263567[] = {{&S55429[83], 6}, {NULL}};
const Segment SGI263568[] = {{&S55429[117], 6}, {NULL}};
const Segment SGI263569[] = {{&S55429[131], 9}, {NULL}};
const Segment SGI263570[] = {{&S55429[0], 15}, {NULL}};
const Segment SGI263571[] = {{&S55430[28], 6}, {NULL}};
const Segment SGI263572[] = {{&S55430[52], 3}, {NULL}};
const Segment SGI263573[] = {{&S55430[83], 6}, {NULL}};
const Segment SGI263574[] = {{&S55430[117], 6}, {NULL}};
const Segment SGI263575[] = {{&S55430[131], 9}, {NULL}};
const Segment SGI263576[] = {{&S55430[0], 15}, {NULL}};
const Segment SGI263577[] = {{&S55431[28], 6}, {NULL}};
const Segment SGI263578[] = {{&S55431[53], 5}, {NULL}};
const Segment SGI263579[] = {{&S55431[86], 6}, {NULL}};
const Segment SGI263580[] = {{&S55431[120], 6}, {NULL}};
const Segment SGI263581[] = {{&S55431[134], 9}, {NULL}};
const Segment SGI263582[] = {{&S55431[0], 15}, {NULL}};
const Segment SGI263583[] = {{&S55432[27], 6}, {NULL}};
const Segment SGI263584[] = {{&S55432[51], 5}, {NULL}};
const Segment SGI263585[] = {{&S55432[84], 6}, {NULL}};
const Segment SGI263586[] = {{&S55432[118], 6}, {NULL}};
const Segment SGI263587[] = {{&S55432[132], 9}, {NULL}};
const Segment SGI263588[] = {{&S55432[0], 14}, {NULL}};
const Segment SGI263589[] = {{&S55485[13], 6}, {NULL}};
const Segment SGI263590[] = {{&S55485[110], 5}, {NULL}};
const Segment SGI263591[] = {{&S55485[38], 3}, {NULL}};
const Segment SGI263592[] = {{&S55485[76], 5}, {NULL}};
const Segment SGI263593[] = {{&S55485[123], 5}, {NULL}};
const Segment SGI263594[] = {{&S55486[13], 6}, {NULL}};
const Segment SGI263595[] = {{&S55486[110], 5}, {NULL}};
const Segment SGI263596[] = {{&S55486[38], 3}, {NULL}};
const Segment SGI263597[] = {{&S55486[76], 5}, {NULL}};
const Segment SGI263598[] = {{&S55486[123], 5}, {NULL}};
const Segment SGI263599[] = {{&S55486[136], 9}, {NULL}};
const Segment SGI263600[] = {{&S55487[13], 6}, {NULL}};
const Segment SGI263601[] = {{&S55487[113], 3}, {NULL}};
const Segment SGI263602[] = {{&S55487[38], 5}, {NULL}};
const Segment SGI263603[] = {{&S55487[78], 6}, {NULL}};
const Segment SGI263604[] = {{&S55487[124], 5}, {NULL}};
const Segment SGI263605[] = {{&S55487[137], 6}, {NULL}};
const Segment SGI263606[] = {{&S55488[13], 6}, {NULL}};
const Segment SGI263607[] = {{&S55488[112], 5}, {NULL}};
const Segment SGI263608[] = {{&S55488[38], 5}, {NULL}};
const Segment SGI263609[] = {{&S55488[78], 5}, {NULL}};
const Segment SGI263610[] = {{&S55488[125], 5}, {NULL}};
const Segment SGI263611[] = {{&S55489[13], 6}, {NULL}};
const Segment SGI263612[] = {{&S55489[111], 3}, {NULL}};
const Segment SGI263613[] = {{&S55489[38], 3}, {NULL}};
const Segment SGI263614[] = {{&S55489[76], 6}, {NULL}};
const Segment SGI263615[] = {{&S55489[122], 5}, {NULL}};
const Segment SGI263616[] = {{&S55489[135], 6}, {NULL}};
const Segment SGI263617[] = {{&S55490[13], 6}, {NULL}};
const Segment SGI263618[] = {{&S55490[112], 5}, {NULL}};
const Segment SGI263619[] = {{&S55490[38], 5}, {NULL}};
const Segment SGI263620[] = {{&S55490[78], 5}, {NULL}};
const Segment SGI263621[] = {{&S55490[125], 5}, {NULL}};
const Segment SGI263622[] = {{&S55490[138], 9}, {NULL}};
const Segment SGI263623[] = {{&S55491[13], 6}, {NULL}};
const Segment SGI263624[] = {{&S55491[35], 3}, {NULL}};
const Segment SGI263625[] = {{&S55491[66], 6}, {NULL}};
const Segment SGI263626[] = {{&S55491[100], 5}, {NULL}};
const Segment SGI263627[] = {{&S55491[113], 9}, {NULL}};
const Segment SGI263628[] = {{&S55492[13], 6}, {NULL}};
const Segment SGI263629[] = {{&S55492[112], 5}, {NULL}};
const Segment SGI263630[] = {{&S55492[38], 5}, {NULL}};
const Segment SGI263631[] = {{&S55492[78], 5}, {NULL}};
const Segment SGI263632[] = {{&S55492[125], 5}, {NULL}};
const Segment SGI263633[] = {{&S55492[138], 9}, {NULL}};
const Segment SGI263634[] = {{&S55493[13], 6}, {NULL}};
const Segment SGI263635[] = {{&S55493[35], 5}, {NULL}};
const Segment SGI263636[] = {{&S55493[68], 6}, {NULL}};
const Segment SGI263637[] = {{&S55493[102], 5}, {NULL}};
const Segment SGI263638[] = {{&S55493[115], 9}, {NULL}};
const Segment SGI263639[] = {{&S55494[13], 6}, {NULL}};
const Segment SGI263640[] = {{&S55494[35], 3}, {NULL}};
const Segment SGI263641[] = {{&S55494[66], 6}, {NULL}};
const Segment SGI263642[] = {{&S55494[100], 5}, {NULL}};
const Segment SGI263643[] = {{&S55494[113], 9}, {NULL}};
const Segment SGI263644[] = {{&S55495[13], 6}, {NULL}};
const Segment SGI263645[] = {{&S55495[110], 5}, {NULL}};
const Segment SGI263646[] = {{&S55495[38], 3}, {NULL}};
const Segment SGI263647[] = {{&S55495[76], 5}, {NULL}};
const Segment SGI263648[] = {{&S55495[123], 5}, {NULL}};
const Segment SGI263649[] = {{&S55495[136], 9}, {NULL}};
const Segment SGI263650[] = {{&S48177[0], 17}, {NULL}};
const Segment SGI263651[] = {{&S48177[0], 178}, {NULL}};
const Segment SGI263652[] = {{&S48178[0], 17}, {NULL}};
const Segment SGI263653[] = {{&S48178[0], 166}, {NULL}};
const Segment SGI263654[] = {{&S55744[13], 6}, {NULL}};
const Segment SGI263655[] = {{&S55744[38], 5}, {NULL}};
const Segment SGI263656[] = {{&S55744[78], 6}, {NULL}};
const Segment SGI263657[] = {{&S55744[112], 5}, {NULL}};
const Segment SGI263658[] = {{&S55745[13], 6}, {NULL}};
const Segment SGI263659[] = {{&S55745[38], 3}, {NULL}};
const Segment SGI263660[] = {{&S55745[76], 6}, {NULL}};
const Segment SGI263661[] = {{&S55746[13], 6}, {NULL}};
const Segment SGI263662[] = {{&S55746[38], 5}, {NULL}};
const Segment SGI263663[] = {{&S55746[78], 6}, {NULL}};
const Segment SGI263664[] = {{&S55747[13], 6}, {NULL}};
const Segment SGI263665[] = {{&S55747[38], 3}, {NULL}};
const Segment SGI263666[] = {{&S55747[76], 6}, {NULL}};
const Segment SGI263667[] = {{&S55747[110], 5}, {NULL}};
const Segment SGI263668[] = {{&S55793[17], 4}, {NULL}};
const Segment SGI263669[] = {{&S55793[36], 3}, {NULL}};
const Segment SGI263670[] = {{&S55793[64], 3}, {NULL}};
const Segment SGI263671[] = {{&S55794[17], 4}, {NULL}};
const Segment SGI263672[] = {{&S55794[36], 5}, {NULL}};
const Segment SGI263673[] = {{&S55794[66], 3}, {NULL}};
const Segment SGI263674[] = {{&S55795[17], 4}, {NULL}};
const Segment SGI263675[] = {{&S55795[36], 5}, {NULL}};
const Segment SGI263676[] = {{&S55795[66], 3}, {NULL}};
const Segment SGI263677[] = {{&S56012[13], 6}, {&S56012[13], 6}, {NULL}};
const Segment SGI263678[] = {{&S56012[199], 3}, {NULL}};
const Segment SGI263679[] = {{&S56012[35], 3}, {NULL}};
const Segment SGI263680[] = {{&S56012[66], 6}, {&S56012[166], 3}, {NULL}};
const Segment SGI263681[] = {{&S56012[100], 6}, {&S56012[210], 3}, {NULL}};
const Segment SGI263682[] = {{&S56012[221], 5}, {NULL}};
const Segment SGI263683[] = {{&S56013[13], 6}, {&S56013[13], 6}, {NULL}};
const Segment SGI263684[] = {{&S56013[199], 3}, {NULL}};
const Segment SGI263685[] = {{&S56013[35], 3}, {NULL}};
const Segment SGI263686[] = {{&S56013[66], 6}, {&S56013[166], 3}, {NULL}};
const Segment SGI263687[] = {{&S56013[100], 6}, {&S56013[210], 3}, {NULL}};
const Segment SGI263688[] = {{&S56013[221], 5}, {NULL}};
const Segment SGI263689[] = {{&S56014[13], 6}, {&S56014[13], 6}, {NULL}};
const Segment SGI263690[] = {{&S56014[201], 3}, {NULL}};
const Segment SGI263691[] = {{&S56014[35], 5}, {NULL}};
const Segment SGI263692[] = {{&S56014[68], 6}, {&S56014[168], 3}, {NULL}};
const Segment SGI263693[] = {{&S56014[102], 6}, {&S56014[212], 3}, {NULL}};
const Segment SGI263694[] = {{&S56014[223], 5}, {NULL}};
const Segment SGI263695[] = {{&S56015[13], 6}, {&S56015[13], 6}, {NULL}};
const Segment SGI263696[] = {{&S56015[201], 3}, {NULL}};
const Segment SGI263697[] = {{&S56015[35], 5}, {NULL}};
const Segment SGI263698[] = {{&S56015[68], 6}, {&S56015[168], 3}, {NULL}};
const Segment SGI263699[] = {{&S56015[102], 6}, {&S56015[212], 3}, {NULL}};
const Segment SGI263700[] = {{&S56015[223], 5}, {NULL}};
const Segment SGI263701[] = {{&S56016[13], 6}, {&S56016[13], 6}, {NULL}};
const Segment SGI263702[] = {{&S56016[199], 3}, {NULL}};
const Segment SGI263703[] = {{&S56016[35], 3}, {NULL}};
const Segment SGI263704[] = {{&S56016[66], 6}, {&S56016[166], 3}, {NULL}};
const Segment SGI263705[] = {{&S56016[100], 6}, {&S56016[210], 3}, {NULL}};
const Segment SGI263706[] = {{&S56016[221], 5}, {NULL}};
const Segment SGI263707[] = {{&S56017[13], 6}, {&S56017[13], 6}, {NULL}};
const Segment SGI263708[] = {{&S56017[199], 3}, {NULL}};
const Segment SGI263709[] = {{&S56017[35], 3}, {NULL}};
const Segment SGI263710[] = {{&S56017[66], 6}, {&S56017[166], 3}, {NULL}};
const Segment SGI263711[] = {{&S56017[100], 6}, {&S56017[210], 3}, {NULL}};
const Segment SGI263712[] = {{&S56017[221], 5}, {NULL}};
const Segment SGI263713[] = {{&S56018[13], 6}, {&S56018[13], 6}, {NULL}};
const Segment SGI263714[] = {{&S56018[201], 3}, {NULL}};
const Segment SGI263715[] = {{&S56018[35], 5}, {NULL}};
const Segment SGI263716[] = {{&S56018[68], 6}, {&S56018[168], 3}, {NULL}};
const Segment SGI263717[] = {{&S56018[102], 6}, {&S56018[212], 3}, {NULL}};
const Segment SGI263718[] = {{&S56018[223], 5}, {NULL}};
const Segment SGI263719[] = {{&S56019[13], 6}, {&S56019[13], 6}, {NULL}};
const Segment SGI263720[] = {{&S56019[201], 3}, {NULL}};
const Segment SGI263721[] = {{&S56019[35], 5}, {NULL}};
const Segment SGI263722[] = {{&S56019[68], 6}, {&S56019[168], 3}, {NULL}};
const Segment SGI263723[] = {{&S56019[102], 6}, {&S56019[212], 3}, {NULL}};
const Segment SGI263724[] = {{&S56019[223], 5}, {NULL}};
const Segment SGI263725[] = {{&S48786[59], 3}, {NULL}};
const Segment SGI263726[] = {{&S48786[0], 62}, {NULL}};
const Segment SGI263727[] = {{&S56319[13], 4}, {NULL}};
const Segment SGI263728[] = {{&S56319[33], 5}, {NULL}};
const Segment SGI263729[] = {{&S56319[66], 6}, {NULL}};
const Segment SGI263730[] = {{&S56319[119], 5}, {NULL}};
const Segment SGI263731[] = {{&S56319[132], 9}, {NULL}};
const Segment SGI263732[] = {{&S56320[13], 4}, {NULL}};
const Segment SGI263733[] = {{&S56320[33], 3}, {NULL}};
const Segment SGI263734[] = {{&S56320[64], 6}, {NULL}};
const Segment SGI263735[] = {{&S56320[117], 5}, {NULL}};
const Segment SGI263736[] = {{&S56320[130], 9}, {NULL}};
const Segment SGI263737[] = {{&S49335[125], 31}, {NULL}};
const Segment SGI263738[] = {{&S49335[0], 156}, {NULL}};
const Segment SGI263739[] = {{&S56746[27], 6}, {NULL}};
const Segment SGI263740[] = {{&S56746[52], 3}, {NULL}};
const Segment SGI263741[] = {{&S56746[83], 6}, {NULL}};
const Segment SGI263742[] = {{&S56746[117], 6}, {NULL}};
const Segment SGI263743[] = {{&S56746[131], 9}, {NULL}};
const Segment SGI263744[] = {{&S56746[0], 14}, {NULL}};
const Segment SGI263745[] = {{&S56747[27], 6}, {NULL}};
const Segment SGI263746[] = {{&S56747[51], 3}, {NULL}};
const Segment SGI263747[] = {{&S56747[82], 6}, {NULL}};
const Segment SGI263748[] = {{&S56747[116], 6}, {NULL}};
const Segment SGI263749[] = {{&S56747[130], 9}, {NULL}};
const Segment SGI263750[] = {{&S56747[0], 14}, {NULL}};
const Segment SGI263751[] = {{&S56748[27], 6}, {NULL}};
const Segment SGI263752[] = {{&S56748[51], 3}, {NULL}};
const Segment SGI263753[] = {{&S56748[82], 6}, {NULL}};
const Segment SGI263754[] = {{&S56748[116], 6}, {NULL}};
const Segment SGI263755[] = {{&S56748[130], 9}, {NULL}};
const Segment SGI263756[] = {{&S56748[0], 14}, {NULL}};
const Segment SGI263757[] = {{&S56749[27], 6}, {NULL}};
const Segment SGI263758[] = {{&S56749[52], 5}, {NULL}};
const Segment SGI263759[] = {{&S56749[85], 6}, {NULL}};
const Segment SGI263760[] = {{&S56749[119], 6}, {NULL}};
const Segment SGI263761[] = {{&S56749[133], 9}, {NULL}};
const Segment SGI263762[] = {{&S56749[0], 14}, {NULL}};
const Segment SGI263763[] = {{&S56750[27], 6}, {NULL}};
const Segment SGI263764[] = {{&S56750[52], 5}, {NULL}};
const Segment SGI263765[] = {{&S56750[85], 6}, {NULL}};
const Segment SGI263766[] = {{&S56750[119], 6}, {NULL}};
const Segment SGI263767[] = {{&S56750[133], 9}, {NULL}};
const Segment SGI263768[] = {{&S56750[0], 14}, {NULL}};
const Segment SGI263769[] = {{&S56751[27], 6}, {NULL}};
const Segment SGI263770[] = {{&S56751[52], 5}, {NULL}};
const Segment SGI263771[] = {{&S56751[85], 6}, {NULL}};
const Segment SGI263772[] = {{&S56751[119], 6}, {NULL}};
const Segment SGI263773[] = {{&S56751[133], 9}, {NULL}};
const Segment SGI263774[] = {{&S56751[0], 14}, {NULL}};
const Segment SGI263775[] = {{&S56752[27], 6}, {NULL}};
const Segment SGI263776[] = {{&S56752[51], 5}, {NULL}};
const Segment SGI263777[] = {{&S56752[84], 6}, {NULL}};
const Segment SGI263778[] = {{&S56752[118], 6}, {NULL}};
const Segment SGI263779[] = {{&S56752[132], 9}, {NULL}};
const Segment SGI263780[] = {{&S56752[0], 14}, {NULL}};
const Segment SGI263781[] = {{&S56794[13], 6}, {NULL}};
const Segment SGI263782[] = {{&S56794[38], 3}, {NULL}};
const Segment SGI263783[] = {{&S56794[76], 8}, {NULL}};
const Segment SGI263784[] = {{&S56794[131], 5}, {NULL}};
const Segment SGI263785[] = {{&S56794[144], 9}, {NULL}};
const Segment SGI263786[] = {{&S56795[13], 6}, {NULL}};
const Segment SGI263787[] = {{&S56795[38], 5}, {NULL}};
const Segment SGI263788[] = {{&S56795[78], 8}, {NULL}};
const Segment SGI263789[] = {{&S56795[133], 3}, {NULL}};
const Segment SGI263790[] = {{&S56795[144], 9}, {NULL}};
const Segment SGI263791[] = {{&S56796[13], 6}, {NULL}};
const Segment SGI263792[] = {{&S56796[38], 5}, {NULL}};
const Segment SGI263793[] = {{&S56796[78], 8}, {NULL}};
const Segment SGI263794[] = {{&S56796[133], 3}, {NULL}};
const Segment SGI263795[] = {{&S56796[144], 9}, {NULL}};
const Segment SGI263796[] = {{&S56797[13], 6}, {NULL}};
const Segment SGI263797[] = {{&S56797[38], 5}, {NULL}};
const Segment SGI263798[] = {{&S56797[78], 8}, {NULL}};
const Segment SGI263799[] = {{&S56797[133], 3}, {NULL}};
const Segment SGI263800[] = {{&S56797[144], 9}, {NULL}};
const Segment SGI263801[] = {{&S56798[13], 6}, {NULL}};
const Segment SGI263802[] = {{&S56798[38], 5}, {NULL}};
const Segment SGI263803[] = {{&S56798[78], 8}, {NULL}};
const Segment SGI263804[] = {{&S56798[133], 3}, {NULL}};
const Segment SGI263805[] = {{&S56798[144], 9}, {NULL}};
const Segment SGI263806[] = {{&S56799[13], 6}, {NULL}};
const Segment SGI263807[] = {{&S56799[38], 5}, {NULL}};
const Segment SGI263808[] = {{&S56799[78], 8}, {NULL}};
const Segment SGI263809[] = {{&S56799[133], 3}, {NULL}};
const Segment SGI263810[] = {{&S56799[144], 9}, {NULL}};
const Segment SGI263811[] = {{&S49430[70], 12}, {NULL}};
const Segment SGI263812[] = {{&S49430[0], 82}, {NULL}};
const Segment SGI263813[] = {{&S49431[76], 12}, {NULL}};
const Segment SGI263814[] = {{&S49431[0], 88}, {NULL}};
const Segment SGI263815[] = {{&S49432[75], 12}, {NULL}};
const Segment SGI263816[] = {{&S49432[0], 87}, {NULL}};
const Segment SGI263817[] = {{&S49433[146], 12}, {NULL}};
const Segment SGI263818[] = {{&S49433[0], 158}, {NULL}};
const Segment SGI263819[] = {{&S49434[56], 12}, {NULL}};
const Segment SGI263820[] = {{&S49434[0], 68}, {NULL}};
const Segment SGI263821[] = {{&S49435[107], 12}, {NULL}};
const Segment SGI263822[] = {{&S49435[0], 119}, {NULL}};
const Segment SGI263823[] = {{&S49436[120], 12}, {NULL}};
const Segment SGI263824[] = {{&S49436[0], 132}, {NULL}};
const Segment SGI263825[] = {{&S49437[51], 12}, {NULL}};
const Segment SGI263826[] = {{&S49437[0], 63}, {NULL}};
const Segment SGI263827[] = {{&S49438[68], 12}, {NULL}};
const Segment SGI263828[] = {{&S49438[0], 80}, {NULL}};
const Segment SGI263829[] = {{&S49439[66], 12}, {NULL}};
const Segment SGI263830[] = {{&S49439[0], 78}, {NULL}};
const Segment SGI263831[] = {{&S49440[95], 12}, {NULL}};
const Segment SGI263832[] = {{&S49440[0], 107}, {NULL}};
const Segment SGI263833[] = {{&S49441[64], 12}, {NULL}};
const Segment SGI263834[] = {{&S49441[0], 76}, {NULL}};
const Segment SGI263835[] = {{&S49442[139], 12}, {NULL}};
const Segment SGI263836[] = {{&S49442[0], 151}, {NULL}};
const Segment SGI263837[] = {{&S49443[132], 12}, {NULL}};
const Segment SGI263838[] = {{&S49443[0], 144}, {NULL}};
const Segment SGI263839[] = {{&S49444[132], 12}, {NULL}};
const Segment SGI263840[] = {{&S49444[0], 144}, {NULL}};
const Segment SGI263841[] = {{&S49445[87], 12}, {NULL}};
const Segment SGI263842[] = {{&S49445[0], 99}, {NULL}};
const Segment SGI263843[] = {{&S49446[68], 12}, {NULL}};
const Segment SGI263844[] = {{&S49446[0], 80}, {NULL}};
const Segment SGI263845[] = {{&S49447[88], 12}, {NULL}};
const Segment SGI263846[] = {{&S49447[0], 100}, {NULL}};
const Segment SGI263847[] = {{&S49448[70], 12}, {NULL}};
const Segment SGI263848[] = {{&S49448[0], 82}, {NULL}};
const Segment SGI263849[] = {{&S49449[75], 12}, {NULL}};
const Segment SGI263850[] = {{&S49449[0], 87}, {NULL}};
const Segment SGI263851[] = {{&S49450[174], 12}, {NULL}};
const Segment SGI263852[] = {{&S49450[0], 186}, {NULL}};
const Segment SGI263853[] = {{&S49451[151], 12}, {NULL}};
const Segment SGI263854[] = {{&S49451[0], 163}, {NULL}};
const Segment SGI263855[] = {{&S49452[115], 12}, {NULL}};
const Segment SGI263856[] = {{&S49452[0], 127}, {NULL}};
const Segment SGI263857[] = {{&S49453[64], 12}, {NULL}};
const Segment SGI263858[] = {{&S49453[0], 76}, {NULL}};
const Segment SGI263859[] = {{&S49454[71], 12}, {NULL}};
const Segment SGI263860[] = {{&S49454[0], 83}, {NULL}};
const Segment SGI263861[] = {{&S49455[164], 12}, {NULL}};
const Segment SGI263862[] = {{&S49455[0], 176}, {NULL}};
const Segment SGI263863[] = {{&S49456[88], 12}, {NULL}};
const Segment SGI263864[] = {{&S49456[0], 100}, {NULL}};
const Segment SGI263865[] = {{&S49457[83], 12}, {NULL}};
const Segment SGI263866[] = {{&S49457[0], 95}, {NULL}};
const Segment SGI263867[] = {{&S49458[67], 12}, {NULL}};
const Segment SGI263868[] = {{&S49458[0], 79}, {NULL}};
const Segment SGI263869[] = {{&S49459[99], 12}, {NULL}};
const Segment SGI263870[] = {{&S49459[0], 111}, {NULL}};
const Segment SGI263871[] = {{&S49460[64], 12}, {NULL}};
const Segment SGI263872[] = {{&S49460[0], 76}, {NULL}};
const Segment SGI263873[] = {{&S49461[72], 12}, {NULL}};
const Segment SGI263874[] = {{&S49461[0], 84}, {NULL}};
const Segment SGI263875[] = {{&S49462[103], 12}, {NULL}};
const Segment SGI263876[] = {{&S49462[0], 115}, {NULL}};
const Segment SGI263877[] = {{&S49463[58], 7}, {NULL}};
const Segment SGI263878[] = {{&S49463[0], 87}, {NULL}};
const Segment SGI263879[] = {{&S49464[64], 12}, {NULL}};
const Segment SGI263880[] = {{&S49464[0], 76}, {NULL}};
const Segment SGI263881[] = {{&S49465[146], 12}, {NULL}};
const Segment SGI263882[] = {{&S49465[0], 158}, {NULL}};
const Segment SGI263883[] = {{&S49466[95], 12}, {NULL}};
const Segment SGI263884[] = {{&S49466[0], 107}, {NULL}};
const Segment SGI263885[] = {{&S49467[83], 12}, {NULL}};
const Segment SGI263886[] = {{&S49467[0], 95}, {NULL}};
const Segment SGI263887[] = {{&S49468[139], 12}, {NULL}};
const Segment SGI263888[] = {{&S49468[0], 151}, {NULL}};
const Segment SGI263889[] = {{&S49469[139], 12}, {NULL}};
const Segment SGI263890[] = {{&S49469[0], 151}, {NULL}};
const Segment SGI263891[] = {{&S49470[107], 12}, {NULL}};
const Segment SGI263892[] = {{&S49470[0], 119}, {NULL}};
const Segment SGI263893[] = {{&S49471[72], 12}, {NULL}};
const Segment SGI263894[] = {{&S49471[0], 84}, {NULL}};
const Segment SGI263895[] = {{&S49472[76], 12}, {NULL}};
const Segment SGI263896[] = {{&S49472[0], 88}, {NULL}};
const Segment SGI263897[] = {{&S49473[223], 12}, {NULL}};
const Segment SGI263898[] = {{&S49473[0], 235}, {NULL}};
const Segment SGI263899[] = {{&S49474[233], 12}, {NULL}};
const Segment SGI263900[] = {{&S49474[0], 245}, {NULL}};
const Segment SGI263901[] = {{&S49475[233], 12}, {NULL}};
const Segment SGI263902[] = {{&S49475[0], 245}, {NULL}};
const Segment SGI263903[] = {{&S49476[237], 12}, {NULL}};
const Segment SGI263904[] = {{&S49476[0], 249}, {NULL}};
const Segment SGI263905[] = {{&S49477[285], 12}, {NULL}};
const Segment SGI263906[] = {{&S49477[0], 297}, {NULL}};
const Segment SGI263907[] = {{&S49478[64], 12}, {NULL}};
const Segment SGI263908[] = {{&S49478[0], 76}, {NULL}};
const Segment SGI263909[] = {{&S49479[56], 29}, {NULL}};
const Segment SGI263910[] = {{&S49479[0], 85}, {NULL}};
const Segment SGI263911[] = {{&S42265[10], 4}, {NULL}};
const Segment SGI263912[] = {{&S42265[15], 5}, {NULL}};
const Segment SGI263913[] = {{&S57118[13], 4}, {NULL}};
const Segment SGI263914[] = {{&S57118[105], 5}, {NULL}};
const Segment SGI263915[] = {{&S57118[29], 3}, {NULL}};
const Segment SGI263916[] = {{&S57118[67], 9}, {NULL}};
const Segment SGI263917[] = {{&S57118[118], 5}, {NULL}};
const Segment SGI263918[] = {{&S57119[13], 4}, {&S57119[13], 4}, {NULL}};
const Segment SGI263919[] = {{&S57119[106], 5}, {NULL}};
const Segment SGI263920[] = {{&S57119[34], 3}, {NULL}};
const Segment SGI263921[] = {{&S57119[72], 5}, {NULL}};
const Segment SGI263922[] = {{&S57119[119], 5}, {NULL}};
const Segment SGI263923[] = {{&S57119[132], 9}, {NULL}};
const Segment SGI263924[] = {{&S57120[13], 4}, {NULL}};
const Segment SGI263925[] = {{&S57120[105], 5}, {NULL}};
const Segment SGI263926[] = {{&S57120[29], 3}, {NULL}};
const Segment SGI263927[] = {{&S57120[67], 9}, {NULL}};
const Segment SGI263928[] = {{&S57120[118], 5}, {NULL}};
const Segment SGI263929[] = {{&S49758[0], 16}, {NULL}};
const Segment SGI263930[] = {{&S57315[13], 4}, {NULL}};
const Segment SGI263931[] = {{&S57315[26], 3}, {NULL}};
const Segment SGI263932[] = {{&S57315[64], 8}, {NULL}};
const Segment SGI263933[] = {{&S57315[119], 6}, {NULL}};
const Segment SGI263934[] = {{&S57315[133], 9}, {NULL}};
const Segment SGI263935[] = {{&S57316[13], 4}, {NULL}};
const Segment SGI263936[] = {{&S57316[26], 5}, {NULL}};
const Segment SGI263937[] = {{&S57316[66], 8}, {NULL}};
const Segment SGI263938[] = {{&S57316[121], 6}, {NULL}};
const Segment SGI263939[] = {{&S57316[135], 9}, {NULL}};
const Segment SGI263940[] = {{&S57409[13], 4}, {NULL}};
const Segment SGI263941[] = {{&S57409[98], 5}, {NULL}};
const Segment SGI263942[] = {{&S57409[29], 3}, {NULL}};
const Segment SGI263943[] = {{&S57409[60], 9}, {NULL}};
const Segment SGI263944[] = {{&S57409[111], 5}, {NULL}};
const Segment SGI263945[] = {{&S57409[134], 13}, {NULL}};
const Segment SGI263946[] = {{&S57458[13], 6}, {NULL}};
const Segment SGI263947[] = {{&S57458[107], 5}, {NULL}};
const Segment SGI263948[] = {{&S57458[31], 3}, {NULL}};
const Segment SGI263949[] = {{&S57458[69], 9}, {NULL}};
const Segment SGI263950[] = {{&S57458[120], 6}, {NULL}};
const Segment SGI263951[] = {{&S57459[12], 6}, {NULL}};
const Segment SGI263952[] = {{&S57459[113], 5}, {NULL}};
const Segment SGI263953[] = {{&S57459[37], 3}, {NULL}};
const Segment SGI263954[] = {{&S57459[75], 9}, {NULL}};
const Segment SGI263955[] = {{&S57459[126], 5}, {NULL}};
const Segment SGI263956[] = {{&S57460[13], 6}, {NULL}};
const Segment SGI263957[] = {{&S57460[124], 5}, {NULL}};
const Segment SGI263958[] = {{&S57460[48], 3}, {NULL}};
const Segment SGI263959[] = {{&S57460[86], 9}, {NULL}};
const Segment SGI263960[] = {{&S57460[137], 4}, {NULL}};
const Segment SGI263961[] = {{&S57525[13], 4}, {NULL}};
const Segment SGI263962[] = {{&S57525[36], 5}, {NULL}};
const Segment SGI263963[] = {{&S57525[76], 8}, {NULL}};
const Segment SGI263964[] = {{&S57525[112], 5}, {NULL}};
const Segment SGI263965[] = {{&S57525[118], 7}, {NULL}};
const Segment SGI263966[] = {{&S50407[0], 17}, {NULL}};
const Segment SGI263967[] = {{&S50407[0], 30}, {NULL}};
const Segment SGI263968[] = {{&S50408[42], 23}, {NULL}};
const Segment SGI263969[] = {{&S50408[0], 65}, {NULL}};
const Segment SGI263970[] = {{&S50409[0], 17}, {NULL}};
const Segment SGI263971[] = {{&S50409[0], 30}, {NULL}};
const Segment SGI263972[] = {{&S50410[8], 7}, {&S50410[23], 32}, {NULL}};
const Segment SGI263973[] = {{&S50410[0], 91}, {NULL}};
const Segment SGI263974[] = {{&S50411[87], 10}, {&S50411[98], 32}, {NULL}};
const Segment SGI263975[] = {{&S50411[0], 130}, {NULL}};
const Segment SGI263976[] = {{&S50412[0], 7}, {&S50412[55], 41}, {NULL}};
const Segment SGI263977[] = {{&S50412[0], 96}, {NULL}};
const Segment SGI263978[] = {{&S50413[8], 7}, {&S50413[23], 32}, {NULL}};
const Segment SGI263979[] = {{&S50413[0], 91}, {NULL}};
const Segment SGI263980[] = {{&S50414[0], 7}, {&S50414[56], 41}, {NULL}};
const Segment SGI263981[] = {{&S50414[0], 97}, {NULL}};
const Segment SGI263982[] = {{&S50415[0], 7}, {&S50415[74], 24}, {NULL}};
const Segment SGI263983[] = {{&S50415[0], 98}, {NULL}};
const Segment SGI263984[] = {{&S43307[10], 4}, {NULL}};
const Segment SGI263985[] = {{&S43307[15], 5}, {NULL}};
const Segment SGI263986[] = {{&S57728[20], 4}, {NULL}};
const Segment SGI263987[] = {{&S57728[36], 3}, {NULL}};
const Segment SGI263988[] = {{&S57729[29], 4}, {NULL}};
const Segment SGI263989[] = {{&S57730[20], 4}, {NULL}};
const Segment SGI263990[] = {{&S57730[36], 5}, {NULL}};
const Segment SGI263991[] = {{&S57733[13], 4}, {NULL}};
const Segment SGI263992[] = {{&S57733[26], 3}, {NULL}};
const Segment SGI263993[] = {{&S57733[57], 6}, {NULL}};
const Segment SGI263994[] = {{&S57733[108], 6}, {NULL}};
const Segment SGI263995[] = {{&S57733[122], 9}, {NULL}};
const Segment SGI263996[] = {{&S57733[84], 14}, {NULL}};
const Segment SGI263997[] = {{&S57734[13], 4}, {NULL}};
const Segment SGI263998[] = {{&S57734[26], 5}, {NULL}};
const Segment SGI263999[] = {{&S57734[59], 6}, {NULL}};
const Segment SGI264000[] = {{&S57734[110], 6}, {NULL}};
const Segment SGI264001[] = {{&S57734[124], 9}, {NULL}};
const Segment SGI264002[] = {{&S57734[86], 14}, {NULL}};
const Segment SGI264003[] = {{&S57735[13], 4}, {NULL}};
const Segment SGI264004[] = {{&S57735[26], 5}, {NULL}};
const Segment SGI264005[] = {{&S57735[59], 6}, {NULL}};
const Segment SGI264006[] = {{&S57735[110], 6}, {NULL}};
const Segment SGI264007[] = {{&S57735[124], 9}, {NULL}};
const Segment SGI264008[] = {{&S57735[86], 14}, {NULL}};
const Segment SGI264009[] = {{&S57736[13], 4}, {NULL}};
const Segment SGI264010[] = {{&S57736[26], 5}, {NULL}};
const Segment SGI264011[] = {{&S57736[59], 6}, {NULL}};
const Segment SGI264012[] = {{&S57736[110], 6}, {NULL}};
const Segment SGI264013[] = {{&S57736[124], 9}, {NULL}};
const Segment SGI264014[] = {{&S57736[86], 14}, {NULL}};
const Segment SGI264015[] = {{&S57840[13], 4}, {NULL}};
const Segment SGI264016[] = {{&S57840[26], 5}, {NULL}};
const Segment SGI264017[] = {{&S57840[59], 6}, {NULL}};
const Segment SGI264018[] = {{&S57840[111], 6}, {NULL}};
const Segment SGI264019[] = {{&S57840[125], 9}, {NULL}};
const Segment SGI264020[] = {{&S57841[13], 4}, {NULL}};
const Segment SGI264021[] = {{&S57841[26], 5}, {NULL}};
const Segment SGI264022[] = {{&S57841[59], 6}, {NULL}};
const Segment SGI264023[] = {{&S57841[111], 6}, {NULL}};
const Segment SGI264024[] = {{&S57841[125], 9}, {NULL}};
const Segment SGI264025[] = {{&S57842[13], 4}, {NULL}};
const Segment SGI264026[] = {{&S57842[26], 5}, {NULL}};
const Segment SGI264027[] = {{&S57842[59], 6}, {NULL}};
const Segment SGI264028[] = {{&S57842[111], 6}, {NULL}};
const Segment SGI264029[] = {{&S57842[125], 9}, {NULL}};
const Segment SGI264030[] = {{&S57843[13], 4}, {NULL}};
const Segment SGI264031[] = {{&S57843[26], 3}, {NULL}};
const Segment SGI264032[] = {{&S57843[57], 6}, {NULL}};
const Segment SGI264033[] = {{&S57843[109], 6}, {NULL}};
const Segment SGI264034[] = {{&S57843[123], 9}, {NULL}};
const Segment SGI264035[] = {{&S43573[10], 4}, {NULL}};
const Segment SGI264036[] = {{&S43573[15], 5}, {NULL}};
const Segment SGI264037[] = {{&S58047[13], 6}, {NULL}};
const Segment SGI264038[] = {{&S58047[35], 3}, {NULL}};
const Segment SGI264039[] = {{&S58047[66], 6}, {NULL}};
const Segment SGI264040[] = {{&S58047[119], 8}, {NULL}};
const Segment SGI264041[] = {{&S58047[135], 7}, {NULL}};
const Segment SGI264042[] = {{&S58048[13], 6}, {NULL}};
const Segment SGI264043[] = {{&S58048[35], 3}, {NULL}};
const Segment SGI264044[] = {{&S58048[66], 6}, {NULL}};
const Segment SGI264045[] = {{&S58048[119], 6}, {NULL}};
const Segment SGI264046[] = {{&S58048[133], 7}, {NULL}};
const Segment SGI264047[] = {{&S58049[13], 6}, {NULL}};
const Segment SGI264048[] = {{&S58049[35], 5}, {NULL}};
const Segment SGI264049[] = {{&S58049[68], 6}, {NULL}};
const Segment SGI264050[] = {{&S58049[121], 6}, {NULL}};
const Segment SGI264051[] = {{&S58049[135], 7}, {NULL}};
const Segment SGI264052[] = {{&S58095[26], 6}, {NULL}};
const Segment SGI264053[] = {{&S58095[44], 5}, {NULL}};
const Segment SGI264054[] = {{&S58096[28], 6}, {NULL}};
const Segment SGI264055[] = {{&S58096[46], 3}, {NULL}};
const Segment SGI264056[] = {{&S58097[26], 6}, {NULL}};
const Segment SGI264057[] = {{&S58097[44], 5}, {NULL}};
const Segment SGI264058[] = {{&S58098[26], 6}, {NULL}};
const Segment SGI264059[] = {{&S58098[44], 5}, {NULL}};
const Segment SGI264060[] = {{&S58099[26], 6}, {NULL}};
const Segment SGI264061[] = {{&S58099[44], 5}, {NULL}};
const Segment SGI264062[] = {{&S58100[26], 6}, {NULL}};
const Segment SGI264063[] = {{&S58100[44], 5}, {NULL}};
const Segment SGI264064[] = {{&S58297[13], 4}, {NULL}};
const Segment SGI264065[] = {{&S58297[26], 5}, {NULL}};
const Segment SGI264066[] = {{&S58297[59], 6}, {NULL}};
const Segment SGI264067[] = {{&S58297[108], 6}, {NULL}};
const Segment SGI264068[] = {{&S58297[122], 9}, {NULL}};
const Segment SGI264069[] = {{&S58297[86], 14}, {NULL}};
const Segment SGI264070[] = {{&S58298[13], 4}, {NULL}};
const Segment SGI264071[] = {{&S58298[26], 3}, {NULL}};
const Segment SGI264072[] = {{&S58298[57], 6}, {NULL}};
const Segment SGI264073[] = {{&S58298[106], 6}, {NULL}};
const Segment SGI264074[] = {{&S58298[120], 9}, {NULL}};
const Segment SGI264075[] = {{&S58298[84], 14}, {NULL}};
const Segment SGI264076[] = {{&S58299[13], 4}, {NULL}};
const Segment SGI264077[] = {{&S58299[26], 5}, {NULL}};
const Segment SGI264078[] = {{&S58299[59], 6}, {NULL}};
const Segment SGI264079[] = {{&S58299[108], 6}, {NULL}};
const Segment SGI264080[] = {{&S58299[122], 9}, {NULL}};
const Segment SGI264081[] = {{&S58299[86], 14}, {NULL}};
const Segment SGI264082[] = {{&S58300[13], 4}, {NULL}};
const Segment SGI264083[] = {{&S58300[26], 5}, {NULL}};
const Segment SGI264084[] = {{&S58300[59], 6}, {NULL}};
const Segment SGI264085[] = {{&S58300[108], 6}, {NULL}};
const Segment SGI264086[] = {{&S58300[122], 9}, {NULL}};
const Segment SGI264087[] = {{&S58300[86], 14}, {NULL}};
const Segment SGI264088[] = {{&S43945[20], 26}, {NULL}};
const Segment SGI264089[] = {{&S43946[20], 26}, {NULL}};
const Segment SGI264090[] = {{&S43947[20], 27}, {NULL}};
const Segment SGI264091[] = {{&S43948[20], 27}, {NULL}};
const Segment SGI264092[] = {{&S43949[20], 27}, {NULL}};
const Segment SGI264093[] = {{&S43950[20], 26}, {NULL}};
const Segment SGI264094[] = {{&S43951[20], 26}, {NULL}};
const Segment SGI264095[] = {{&S43952[20], 27}, {NULL}};
const Segment SGI264096[] = {{&S43953[20], 26}, {NULL}};
const Segment SGI264097[] = {{&S43954[20], 27}, {NULL}};
const Segment SGI264098[] = {{&S43955[20], 27}, {NULL}};
const Segment SGI264099[] = {{&S43956[20], 26}, {NULL}};
const Segment SGI264100[] = {{&S43957[20], 26}, {NULL}};
const Segment SGI264101[] = {{&S43958[20], 27}, {NULL}};
const Segment SGI264102[] = {{&S43959[20], 26}, {NULL}};
const Segment SGI264103[] = {{&S43960[20], 26}, {NULL}};
const Segment SGI264104[] = {{&S43961[20], 27}, {NULL}};
const Segment SGI264105[] = {{&S43962[20], 27}, {NULL}};
const Segment SGI264106[] = {{&S43963[20], 27}, {NULL}};
const Segment SGI264107[] = {{&S43985[10], 4}, {NULL}};
const Segment SGI264108[] = {{&S43985[15], 5}, {NULL}};
const Segment SGI264109[] = {{&S58429[13], 6}, {NULL}};
const Segment SGI264110[] = {{&S58429[38], 3}, {NULL}};
const Segment SGI264111[] = {{&S58429[76], 5}, {NULL}};
const Segment SGI264112[] = {{&S58429[109], 5}, {NULL}};
const Segment SGI264113[] = {{&S58429[115], 7}, {NULL}};
const Segment SGI264114[] = {{&S58549[12], 4}, {&S58549[12], 4}, {NULL}};
const Segment SGI264115[] = {{&S58549[27], 1}, {NULL}};
const Segment SGI264116[] = {{&S58549[72], 6}, {NULL}};
const Segment SGI264117[] = {{&S58549[135], 9}, {NULL}};
const Segment SGI264118[] = {{&S58549[99], 13}, {NULL}};
const Segment SGI264119[] = {{&S58607[13], 4}, {NULL}};
const Segment SGI264120[] = {{&S58607[101], 5}, {NULL}};
const Segment SGI264121[] = {{&S58607[29], 3}, {NULL}};
const Segment SGI264122[] = {{&S58607[67], 5}, {NULL}};
const Segment SGI264123[] = {{&S58607[114], 5}, {NULL}};
const Segment SGI264124[] = {{&S58607[127], 9}, {NULL}};
const Segment SGI264125[] = {{&S58620[17], 4}, {NULL}};
const Segment SGI264126[] = {{&S58620[36], 3}, {NULL}};
const Segment SGI264127[] = {{&S58620[64], 3}, {NULL}};
const Segment SGI264128[] = {{&S58621[17], 4}, {NULL}};
const Segment SGI264129[] = {{&S58621[36], 5}, {NULL}};
const Segment SGI264130[] = {{&S58621[66], 3}, {NULL}};
const Segment SGI264131[] = {{&S58622[17], 4}, {NULL}};
const Segment SGI264132[] = {{&S58622[36], 5}, {NULL}};
const Segment SGI264133[] = {{&S58622[66], 3}, {NULL}};
const Segment SGI264134[] = {{&S51242[12], 10}, {&S51242[35], 15}, {NULL}};
const Segment SGI264135[] = {{&S51242[0], 75}, {NULL}};
const Segment SGI264136[] = {{&S51243[25], 14}, {&S51243[40], 35}, {NULL}};
const Segment SGI264137[] = {{&S51243[0], 75}, {NULL}};
const Segment SGI264138[] = {{&S51244[0], 13}, {&S51244[14], 52}, {NULL}};
const Segment SGI264139[] = {{&S51244[0], 66}, {NULL}};
const Segment SGI264140[] = {{&S51245[44], 35}, {NULL}};
const Segment SGI264141[] = {{&S51245[0], 79}, {NULL}};
const Segment SGI264142[] = {{&S51246[25], 14}, {&S51246[40], 32}, {NULL}};
const Segment SGI264143[] = {{&S51246[0], 93}, {NULL}};
const Segment SGI264144[] = {{&S51247[25], 14}, {&S51247[40], 32}, {NULL}};
const Segment SGI264145[] = {{&S51247[0], 91}, {NULL}};
const Segment SGI264146[] = {{&S51248[2], 13}, {&S51248[16], 36}, {NULL}};
const Segment SGI264147[] = {{&S51248[0], 52}, {NULL}};
const Segment SGI264148[] = {{&S51249[25], 13}, {NULL}};
const Segment SGI264149[] = {{&S51249[0], 38}, {NULL}};
const Segment SGI264150[] = {{&S51250[0], 14}, {&S51250[15], 31}, {NULL}};
const Segment SGI264151[] = {{&S51250[0], 46}, {NULL}};
const Segment SGI264152[] = {{&S51251[25], 14}, {NULL}};
const Segment SGI264153[] = {{&S51251[0], 58}, {NULL}};
const Segment SGI264154[] = {{&S58701[13], 6}, {NULL}};
const Segment SGI264155[] = {{&S58701[35], 5}, {NULL}};
const Segment SGI264156[] = {{&S51252[0], 14}, {NULL}};
const Segment SGI264157[] = {{&S51252[0], 47}, {NULL}};
const Segment SGI264158[] = {{&S58701[68], 6}, {NULL}};
const Segment SGI264159[] = {{&S58701[116], 5}, {NULL}};
const Segment SGI264160[] = {{&S58701[129], 9}, {NULL}};
const Segment SGI264161[] = {{&S58702[13], 6}, {NULL}};
const Segment SGI264162[] = {{&S58702[35], 3}, {NULL}};
const Segment SGI264163[] = {{&S58702[66], 6}, {NULL}};
const Segment SGI264164[] = {{&S58702[114], 5}, {NULL}};
const Segment SGI264165[] = {{&S58702[127], 9}, {NULL}};
const Segment SGI264166[] = {{&S58708[13], 4}, {NULL}};
const Segment SGI264167[] = {{&S58708[26], 5}, {NULL}};
const Segment SGI264168[] = {{&S58708[59], 6}, {NULL}};
const Segment SGI264169[] = {{&S58708[108], 6}, {NULL}};
const Segment SGI264170[] = {{&S58708[122], 9}, {NULL}};
const Segment SGI264171[] = {{&S58708[86], 14}, {NULL}};
const Segment SGI264172[] = {{&S58709[13], 4}, {NULL}};
const Segment SGI264173[] = {{&S58709[26], 3}, {NULL}};
const Segment SGI264174[] = {{&S58709[57], 6}, {NULL}};
const Segment SGI264175[] = {{&S58709[106], 6}, {NULL}};
const Segment SGI264176[] = {{&S58709[120], 9}, {NULL}};
const Segment SGI264177[] = {{&S58709[84], 14}, {NULL}};
const Segment SGI264178[] = {{&S58710[13], 4}, {NULL}};
const Segment SGI264179[] = {{&S58710[26], 3}, {NULL}};
const Segment SGI264180[] = {{&S58710[57], 6}, {NULL}};
const Segment SGI264181[] = {{&S58710[106], 6}, {NULL}};
const Segment SGI264182[] = {{&S58710[120], 9}, {NULL}};
const Segment SGI264183[] = {{&S58710[84], 14}, {NULL}};
const Segment SGI264184[] = {{&S58711[13], 4}, {NULL}};
const Segment SGI264185[] = {{&S58711[26], 5}, {NULL}};
const Segment SGI264186[] = {{&S58711[59], 6}, {NULL}};
const Segment SGI264187[] = {{&S58711[108], 7}, {NULL}};
const Segment SGI264188[] = {{&S58711[123], 9}, {NULL}};
const Segment SGI264189[] = {{&S58711[86], 14}, {NULL}};
const Segment SGI264190[] = {{&S58712[13], 4}, {NULL}};
const Segment SGI264191[] = {{&S58712[26], 5}, {NULL}};
const Segment SGI264192[] = {{&S58712[59], 6}, {NULL}};
const Segment SGI264193[] = {{&S58712[110], 6}, {NULL}};
const Segment SGI264194[] = {{&S58712[124], 9}, {NULL}};
const Segment SGI264195[] = {{&S58712[86], 14}, {NULL}};
const Segment SGI264196[] = {{&S58713[13], 4}, {NULL}};
const Segment SGI264197[] = {{&S58713[26], 5}, {NULL}};
const Segment SGI264198[] = {{&S58713[59], 6}, {NULL}};
const Segment SGI264199[] = {{&S58713[110], 7}, {NULL}};
const Segment SGI264200[] = {{&S58713[125], 9}, {NULL}};
const Segment SGI264201[] = {{&S58713[86], 14}, {NULL}};
const Segment SGI264202[] = {{&S58742[13], 4}, {NULL}};
const Segment SGI264203[] = {{&S58742[33], 3}, {NULL}};
const Segment SGI264204[] = {{&S58742[64], 6}, {NULL}};
const Segment SGI264205[] = {{&S58742[117], 8}, {NULL}};
const Segment SGI264206[] = {{&S58742[133], 7}, {NULL}};
const Segment SGI264207[] = {{&S58743[13], 4}, {NULL}};
const Segment SGI264208[] = {{&S58743[33], 3}, {NULL}};
const Segment SGI264209[] = {{&S58743[64], 6}, {NULL}};
const Segment SGI264210[] = {{&S58743[117], 8}, {NULL}};
const Segment SGI264211[] = {{&S58743[133], 7}, {NULL}};
const Segment SGI264212[] = {{&S58744[13], 4}, {NULL}};
const Segment SGI264213[] = {{&S58744[33], 3}, {NULL}};
const Segment SGI264214[] = {{&S58744[64], 6}, {NULL}};
const Segment SGI264215[] = {{&S58744[117], 8}, {NULL}};
const Segment SGI264216[] = {{&S58744[133], 7}, {NULL}};
const Segment SGI264217[] = {{&S58745[13], 4}, {NULL}};
const Segment SGI264218[] = {{&S58745[33], 3}, {NULL}};
const Segment SGI264219[] = {{&S58745[64], 6}, {NULL}};
const Segment SGI264220[] = {{&S58745[117], 8}, {NULL}};
const Segment SGI264221[] = {{&S58745[133], 7}, {NULL}};
const Segment SGI264222[] = {{&S58746[13], 4}, {NULL}};
const Segment SGI264223[] = {{&S58746[33], 3}, {NULL}};
const Segment SGI264224[] = {{&S58746[64], 6}, {NULL}};
const Segment SGI264225[] = {{&S58746[117], 6}, {NULL}};
const Segment SGI264226[] = {{&S58746[131], 7}, {NULL}};
const Segment SGI264227[] = {{&S58747[13], 4}, {NULL}};
const Segment SGI264228[] = {{&S58747[33], 5}, {NULL}};
const Segment SGI264229[] = {{&S58747[66], 6}, {NULL}};
const Segment SGI264230[] = {{&S58747[119], 6}, {NULL}};
const Segment SGI264231[] = {{&S58747[133], 7}, {NULL}};
const Segment SGI264232[] = {{&S58841[13], 4}, {NULL}};
const Segment SGI264233[] = {{&S58841[26], 5}, {NULL}};
const Segment SGI264234[] = {{&S58841[59], 6}, {NULL}};
const Segment SGI264235[] = {{&S58841[105], 5}, {NULL}};
const Segment SGI264236[] = {{&S58841[118], 9}, {NULL}};
const Segment SGI264237[] = {{&S58842[13], 4}, {NULL}};
const Segment SGI264238[] = {{&S58842[26], 3}, {NULL}};
const Segment SGI264239[] = {{&S58842[57], 6}, {NULL}};
const Segment SGI264240[] = {{&S58842[103], 5}, {NULL}};
const Segment SGI264241[] = {{&S58842[116], 9}, {NULL}};
const Segment SGI264242[] = {{&S44445[10], 4}, {NULL}};
const Segment SGI264243[] = {{&S44445[15], 5}, {NULL}};
const Segment SGI264244[] = {{&S51528[34], 32}, {&S51528[67], 7}, {NULL}};
const Segment SGI264245[] = {{&S51528[0], 86}, {NULL}};
const Segment SGI264246[] = {{&S51529[34], 33}, {&S51529[68], 20}, {NULL}};
const Segment SGI264247[] = {{&S51529[0], 88}, {NULL}};
const Segment SGI264248[] = {{&S51530[18], 33}, {&S51530[52], 7}, {NULL}};
const Segment SGI264249[] = {{&S51530[0], 71}, {NULL}};
const Segment SGI264250[] = {{&S51531[18], 33}, {&S51531[52], 20}, {NULL}};
const Segment SGI264251[] = {{&S51531[0], 72}, {NULL}};
const Segment SGI264252[] = {{&S59034[17], 6}, {NULL}};
const Segment SGI264253[] = {{&S59034[35], 5}, {NULL}};
const Segment SGI264254[] = {{&S59035[17], 6}, {NULL}};
const Segment SGI264255[] = {{&S59035[35], 3}, {NULL}};
const Segment SGI264256[] = {{&S59036[17], 6}, {NULL}};
const Segment SGI264257[] = {{&S59036[35], 5}, {NULL}};
const Segment SGI264258[] = {{&S59036[61], 3}, {NULL}};
const Segment SGI264259[] = {{&S44648[22], 12}, {NULL}};
const Segment SGI264260[] = {{&S59319[13], 4}, {NULL}};
const Segment SGI264261[] = {{&S59319[26], 5}, {NULL}};
const Segment SGI264262[] = {{&S59319[59], 6}, {NULL}};
const Segment SGI264263[] = {{&S59319[93], 5}, {NULL}};
const Segment SGI264264[] = {{&S59319[99], 7}, {NULL}};
const Segment SGI264265[] = {{&S59320[13], 4}, {NULL}};
const Segment SGI264266[] = {{&S59320[26], 3}, {NULL}};
const Segment SGI264267[] = {{&S59320[57], 6}, {NULL}};
const Segment SGI264268[] = {{&S59320[91], 5}, {NULL}};
const Segment SGI264269[] = {{&S59320[97], 7}, {NULL}};
const Segment SGI264270[] = {{&S59390[13], 4}, {NULL}};
const Segment SGI264271[] = {{&S59390[124], 3}, {NULL}};
const Segment SGI264272[] = {{&S59390[29], 5}, {NULL}};
const Segment SGI264273[] = {{&S59390[69], 8}, {NULL}};
const Segment SGI264274[] = {{&S52025[25], 18}, {NULL}};
const Segment SGI264275[] = {{&S59390[105], 3}, {NULL}};
const Segment SGI264276[] = {{&S52025[0], 67}, {NULL}};
const Segment SGI264277[] = {{&S59391[13], 4}, {NULL}};
const Segment SGI264278[] = {{&S59391[124], 3}, {NULL}};
const Segment SGI264279[] = {{&S59391[29], 5}, {NULL}};
const Segment SGI264280[] = {{&S59391[69], 8}, {NULL}};
const Segment SGI264281[] = {{&S59391[105], 3}, {NULL}};
const Segment SGI264282[] = {{&S59392[13], 4}, {&S59392[13], 4}, {NULL}};
const Segment SGI264283[] = {{&S59392[119], 3}, {NULL}};
const Segment SGI264284[] = {{&S59392[29], 3}, {NULL}};
const Segment SGI264285[] = {{&S59392[84], 6}, {NULL}};
const Segment SGI264286[] = {{&S59392[130], 5}, {NULL}};
const Segment SGI264287[] = {{&S59392[143], 9}, {NULL}};
const Segment SGI264288[] = {{&S59393[13], 4}, {NULL}};
const Segment SGI264289[] = {{&S59393[96], 3}, {NULL}};
const Segment SGI264290[] = {{&S59393[26], 5}, {NULL}};
const Segment SGI264291[] = {{&S59393[59], 8}, {NULL}};
const Segment SGI264292[] = {{&S59393[107], 5}, {NULL}};
const Segment SGI264293[] = {{&S59393[120], 9}, {NULL}};
const Segment SGI264294[] = {{&S59394[13], 4}, {NULL}};
const Segment SGI264295[] = {{&S59394[96], 3}, {NULL}};
const Segment SGI264296[] = {{&S59394[26], 5}, {NULL}};
const Segment SGI264297[] = {{&S59394[59], 8}, {NULL}};
const Segment SGI264298[] = {{&S59394[107], 5}, {NULL}};
const Segment SGI264299[] = {{&S59394[120], 9}, {NULL}};
const Segment SGI264300[] = {{&S59395[13], 4}, {NULL}};
const Segment SGI264301[] = {{&S59395[124], 3}, {NULL}};
const Segment SGI264302[] = {{&S59395[29], 5}, {NULL}};
const Segment SGI264303[] = {{&S59395[69], 8}, {NULL}};
const Segment SGI264304[] = {{&S59395[105], 3}, {NULL}};
const Segment SGI264305[] = {{&S59396[13], 4}, {NULL}};
const Segment SGI264306[] = {{&S59396[124], 3}, {NULL}};
const Segment SGI264307[] = {{&S59396[29], 5}, {NULL}};
const Segment SGI264308[] = {{&S59396[69], 8}, {NULL}};
const Segment SGI264309[] = {{&S59396[105], 3}, {NULL}};
const Segment SGI264310[] = {{&S59397[13], 4}, {NULL}};
const Segment SGI264311[] = {{&S59397[96], 3}, {NULL}};
const Segment SGI264312[] = {{&S59397[26], 5}, {NULL}};
const Segment SGI264313[] = {{&S59397[59], 8}, {NULL}};
const Segment SGI264314[] = {{&S59397[107], 5}, {NULL}};
const Segment SGI264315[] = {{&S59397[120], 9}, {NULL}};
const Segment SGI264316[] = {{&S59398[13], 4}, {NULL}};
const Segment SGI264317[] = {{&S59398[94], 3}, {NULL}};
const Segment SGI264318[] = {{&S59398[26], 3}, {NULL}};
const Segment SGI264319[] = {{&S59398[57], 8}, {NULL}};
const Segment SGI264320[] = {{&S59398[105], 5}, {NULL}};
const Segment SGI264321[] = {{&S59398[118], 9}, {NULL}};
const Segment SGI264322[] = {{&S59399[13], 4}, {NULL}};
const Segment SGI264323[] = {{&S59399[96], 3}, {NULL}};
const Segment SGI264324[] = {{&S59399[26], 5}, {NULL}};
const Segment SGI264325[] = {{&S59399[59], 8}, {NULL}};
const Segment SGI264326[] = {{&S59399[107], 5}, {NULL}};
const Segment SGI264327[] = {{&S59399[120], 9}, {NULL}};
const Segment SGI264328[] = {{&S59400[13], 4}, {NULL}};
const Segment SGI264329[] = {{&S59400[96], 3}, {NULL}};
const Segment SGI264330[] = {{&S59400[26], 5}, {NULL}};
const Segment SGI264331[] = {{&S59400[59], 8}, {NULL}};
const Segment SGI264332[] = {{&S59400[107], 5}, {NULL}};
const Segment SGI264333[] = {{&S59400[120], 9}, {NULL}};
const Segment SGI264334[] = {{&S59626[13], 4}, {&S59626[13], 4}, {NULL}};
const Segment SGI264335[] = {{&S59626[26], 3}, {NULL}};
const Segment SGI264336[] = {{&S59626[57], 6}, {NULL}};
const Segment SGI264337[] = {{&S59626[91], 5}, {NULL}};
const Segment SGI264338[] = {{&S59626[97], 7}, {NULL}};
const Segment SGI264339[] = {{&S59627[13], 4}, {&S59627[13], 4}, {NULL}};
const Segment SGI264340[] = {{&S59627[26], 5}, {NULL}};
const Segment SGI264341[] = {{&S59627[59], 6}, {NULL}};
const Segment SGI264342[] = {{&S59627[93], 5}, {NULL}};
const Segment SGI264343[] = {{&S59627[99], 7}, {NULL}};
const Segment SGI264344[] = {{&S59896[13], 4}, {&S59896[13], 4}, {NULL}};
const Segment SGI264345[] = {{&S59896[26], 3}, {NULL}};
const Segment SGI264346[] = {{&S59896[57], 6}, {NULL}};
const Segment SGI264347[] = {{&S59896[91], 6}, {NULL}};
const Segment SGI264348[] = {{&S59896[98], 7}, {NULL}};
const Segment SGI264349[] = {{&S59897[13], 4}, {&S59897[13], 4}, {NULL}};
const Segment SGI264350[] = {{&S59897[26], 5}, {NULL}};
const Segment SGI264351[] = {{&S59897[59], 6}, {NULL}};
const Segment SGI264352[] = {{&S59897[93], 6}, {NULL}};
const Segment SGI264353[] = {{&S59897[100], 7}, {NULL}};
const Segment SGI264354[] = {{&S59898[13], 4}, {&S59898[13], 4}, {NULL}};
const Segment SGI264355[] = {{&S59898[26], 5}, {NULL}};
const Segment SGI264356[] = {{&S59898[59], 6}, {NULL}};
const Segment SGI264357[] = {{&S59898[93], 6}, {NULL}};
const Segment SGI264358[] = {{&S59898[100], 7}, {NULL}};
const Segment SGI264359[] = {{&S59995[13], 4}, {NULL}};
const Segment SGI264360[] = {{&S59995[33], 5}, {NULL}};
const Segment SGI264361[] = {{&S59995[73], 6}, {NULL}};
const Segment SGI264362[] = {{&S59995[127], 5}, {NULL}};
const Segment SGI264363[] = {{&S59995[140], 9}, {NULL}};
const Segment SGI264364[] = {{&S59996[13], 4}, {NULL}};
const Segment SGI264365[] = {{&S59996[33], 3}, {NULL}};
const Segment SGI264366[] = {{&S59996[71], 6}, {NULL}};
const Segment SGI264367[] = {{&S59996[125], 5}, {NULL}};
const Segment SGI264368[] = {{&S59996[138], 9}, {NULL}};
const Segment SGI264369[] = {{&S60003[13], 4}, {&S60003[13], 4}, {NULL}};
const Segment SGI264370[] = {{&S60003[26], 5}, {NULL}};
const Segment SGI264371[] = {{&S60003[59], 6}, {NULL}};
const Segment SGI264372[] = {{&S60003[93], 6}, {NULL}};
const Segment SGI264373[] = {{&S60003[107], 9}, {NULL}};
const Segment SGI264374[] = {{&S60004[13], 4}, {&S60004[13], 4}, {NULL}};
const Segment SGI264375[] = {{&S60004[26], 3}, {NULL}};
const Segment SGI264376[] = {{&S60004[57], 6}, {NULL}};
const Segment SGI264377[] = {{&S60004[91], 6}, {NULL}};
const Segment SGI264378[] = {{&S60004[105], 9}, {NULL}};
const Segment SGI264379[] = {{&S60005[13], 4}, {&S60005[13], 4}, {NULL}};
const Segment SGI264380[] = {{&S60005[26], 5}, {NULL}};
const Segment SGI264381[] = {{&S60005[59], 6}, {NULL}};
const Segment SGI264382[] = {{&S60005[93], 6}, {NULL}};
const Segment SGI264383[] = {{&S60005[107], 9}, {NULL}};
const Segment SGI264384[] = {{&S60006[13], 4}, {&S60006[13], 4}, {NULL}};
const Segment SGI264385[] = {{&S60006[26], 5}, {NULL}};
const Segment SGI264386[] = {{&S60006[59], 6}, {NULL}};
const Segment SGI264387[] = {{&S60006[93], 7}, {NULL}};
const Segment SGI264388[] = {{&S60006[108], 9}, {NULL}};
const Segment SGI264389[] = {{&S53007[52], 11}, {NULL}};
const Segment SGI264390[] = {{&S53007[0], 63}, {NULL}};
const Segment SGI264391[] = {{&S53008[52], 14}, {NULL}};
const Segment SGI264392[] = {{&S53008[0], 66}, {NULL}};
const Segment SGI264393[] = {{&S53009[68], 25}, {NULL}};
const Segment SGI264394[] = {{&S53009[0], 93}, {NULL}};
const Segment SGI264395[] = {{&S53010[52], 19}, {&S53010[72], 25}, {NULL}};
const Segment SGI264396[] = {{&S53010[0], 97}, {NULL}};
const Segment SGI264397[] = {{&S53011[52], 12}, {NULL}};
const Segment SGI264398[] = {{&S53011[0], 64}, {NULL}};
const Segment SGI264399[] = {{&S53012[67], 12}, {NULL}};
const Segment SGI264400[] = {{&S53012[0], 79}, {NULL}};
const Segment SGI264401[] = {{&S53013[69], 25}, {NULL}};
const Segment SGI264402[] = {{&S53013[0], 94}, {NULL}};
const Segment SGI264403[] = {{&S60118[13], 4}, {NULL}};
const Segment SGI264404[] = {{&S60118[26], 3}, {NULL}};
const Segment SGI264405[] = {{&S60118[57], 6}, {NULL}};
const Segment SGI264406[] = {{&S60118[115], 5}, {NULL}};
const Segment SGI264407[] = {{&S60118[128], 9}, {NULL}};
const Segment SGI264408[] = {{&S60119[13], 4}, {NULL}};
const Segment SGI264409[] = {{&S60119[26], 5}, {NULL}};
const Segment SGI264410[] = {{&S60119[59], 6}, {NULL}};
const Segment SGI264411[] = {{&S60119[117], 5}, {NULL}};
const Segment SGI264412[] = {{&S60119[130], 9}, {NULL}};
const Segment SGI264413[] = {{&S60120[13], 4}, {NULL}};
const Segment SGI264414[] = {{&S60120[26], 5}, {NULL}};
const Segment SGI264415[] = {{&S60120[59], 6}, {NULL}};
const Segment SGI264416[] = {{&S60120[117], 5}, {NULL}};
const Segment SGI264417[] = {{&S60120[130], 9}, {NULL}};
const Segment SGI264418[] = {{&S60138[13], 6}, {NULL}};
const Segment SGI264419[] = {{&S60138[35], 5}, {NULL}};
const Segment SGI264420[] = {{&S60138[135], 3}, {NULL}};
const Segment SGI264421[] = {{&S60138[68], 6}, {NULL}};
const Segment SGI264422[] = {{&S60138[102], 6}, {NULL}};
const Segment SGI264423[] = {{&S60138[116], 9}, {NULL}};
const Segment SGI264424[] = {{&S60139[13], 6}, {NULL}};
const Segment SGI264425[] = {{&S60139[35], 5}, {NULL}};
const Segment SGI264426[] = {{&S60139[135], 5}, {NULL}};
const Segment SGI264427[] = {{&S60139[68], 6}, {NULL}};
const Segment SGI264428[] = {{&S60139[102], 6}, {NULL}};
const Segment SGI264429[] = {{&S60139[116], 9}, {NULL}};
const Segment SGI264430[] = {{&S60140[13], 6}, {NULL}};
const Segment SGI264431[] = {{&S60140[35], 5}, {NULL}};
const Segment SGI264432[] = {{&S60140[135], 5}, {NULL}};
const Segment SGI264433[] = {{&S60140[68], 6}, {NULL}};
const Segment SGI264434[] = {{&S60140[102], 6}, {NULL}};
const Segment SGI264435[] = {{&S60140[116], 9}, {NULL}};
const Segment SGI264436[] = {{&S60141[13], 6}, {NULL}};
const Segment SGI264437[] = {{&S60141[35], 5}, {NULL}};
const Segment SGI264438[] = {{&S60141[135], 5}, {NULL}};
const Segment SGI264439[] = {{&S60141[68], 6}, {NULL}};
const Segment SGI264440[] = {{&S60141[102], 6}, {NULL}};
const Segment SGI264441[] = {{&S60141[116], 9}, {NULL}};
const Segment SGI264442[] = {{&S60142[13], 6}, {NULL}};
const Segment SGI264443[] = {{&S60142[35], 5}, {NULL}};
const Segment SGI264444[] = {{&S60142[135], 5}, {NULL}};
const Segment SGI264445[] = {{&S60142[68], 6}, {NULL}};
const Segment SGI264446[] = {{&S60142[102], 6}, {NULL}};
const Segment SGI264447[] = {{&S60142[116], 9}, {NULL}};
const Segment SGI264448[] = {{&S60143[13], 6}, {NULL}};
const Segment SGI264449[] = {{&S60143[35], 5}, {NULL}};
const Segment SGI264450[] = {{&S60143[135], 5}, {NULL}};
const Segment SGI264451[] = {{&S60143[68], 6}, {NULL}};
const Segment SGI264452[] = {{&S60143[102], 6}, {NULL}};
const Segment SGI264453[] = {{&S60143[116], 9}, {NULL}};
const Segment SGI264454[] = {{&S60144[13], 6}, {NULL}};
const Segment SGI264455[] = {{&S60144[35], 5}, {NULL}};
const Segment SGI264456[] = {{&S60144[135], 3}, {NULL}};
const Segment SGI264457[] = {{&S46096[10], 4}, {NULL}};
const Segment SGI264458[] = {{&S60144[68], 6}, {NULL}};
const Segment SGI264459[] = {{&S46096[15], 5}, {NULL}};
const Segment SGI264460[] = {{&S60144[102], 6}, {NULL}};
const Segment SGI264461[] = {{&S60144[116], 9}, {NULL}};
const Segment SGI264462[] = {{&S46097[10], 7}, {NULL}};
const Segment SGI264463[] = {{&S60145[13], 6}, {NULL}};
const Segment SGI264464[] = {{&S60145[35], 5}, {NULL}};
const Segment SGI264465[] = {{&S60145[135], 5}, {NULL}};
const Segment SGI264466[] = {{&S60145[68], 6}, {NULL}};
const Segment SGI264467[] = {{&S60145[102], 6}, {NULL}};
const Segment SGI264468[] = {{&S60145[116], 9}, {NULL}};
const Segment SGI264469[] = {{&S46098[10], 4}, {NULL}};
const Segment SGI264470[] = {{&S46098[15], 5}, {NULL}};
const Segment SGI264471[] = {{&S60146[13], 6}, {NULL}};
const Segment SGI264472[] = {{&S60146[35], 5}, {NULL}};
const Segment SGI264473[] = {{&S60146[135], 5}, {NULL}};
const Segment SGI264474[] = {{&S60146[68], 6}, {NULL}};
const Segment SGI264475[] = {{&S60146[102], 6}, {NULL}};
const Segment SGI264476[] = {{&S60146[116], 9}, {NULL}};
const Segment SGI264477[] = {{&S60147[13], 6}, {NULL}};
const Segment SGI264478[] = {{&S60147[35], 5}, {NULL}};
const Segment SGI264479[] = {{&S60147[135], 5}, {NULL}};
const Segment SGI264480[] = {{&S60147[68], 6}, {NULL}};
const Segment SGI264481[] = {{&S60147[102], 6}, {NULL}};
const Segment SGI264482[] = {{&S60147[116], 9}, {NULL}};
const Segment SGI264483[] = {{&S60148[13], 6}, {NULL}};
const Segment SGI264484[] = {{&S60148[35], 5}, {NULL}};
const Segment SGI264485[] = {{&S60148[135], 5}, {NULL}};
const Segment SGI264486[] = {{&S60148[68], 6}, {NULL}};
const Segment SGI264487[] = {{&S60148[102], 6}, {NULL}};
const Segment SGI264488[] = {{&S60148[116], 9}, {NULL}};
const Segment SGI264489[] = {{&S60149[13], 6}, {NULL}};
const Segment SGI264490[] = {{&S60149[35], 5}, {NULL}};
const Segment SGI264491[] = {{&S60149[135], 5}, {NULL}};
const Segment SGI264492[] = {{&S60149[68], 6}, {NULL}};
const Segment SGI264493[] = {{&S60149[102], 6}, {NULL}};
const Segment SGI264494[] = {{&S60149[116], 9}, {NULL}};
const Segment SGI264495[] = {{&S60150[13], 6}, {NULL}};
const Segment SGI264496[] = {{&S60150[35], 5}, {NULL}};
const Segment SGI264497[] = {{&S60150[135], 5}, {NULL}};
const Segment SGI264498[] = {{&S60150[68], 6}, {NULL}};
const Segment SGI264499[] = {{&S60150[102], 6}, {NULL}};
const Segment SGI264500[] = {{&S60150[116], 9}, {NULL}};
const Segment SGI264501[] = {{&S60151[13], 6}, {NULL}};
const Segment SGI264502[] = {{&S60151[35], 5}, {NULL}};
const Segment SGI264503[] = {{&S60151[135], 5}, {NULL}};
const Segment SGI264504[] = {{&S60151[68], 6}, {NULL}};
const Segment SGI264505[] = {{&S60151[102], 6}, {NULL}};
const Segment SGI264506[] = {{&S60151[116], 9}, {NULL}};
const Segment SGI264507[] = {{&S60152[13], 6}, {NULL}};
const Segment SGI264508[] = {{&S60152[35], 5}, {NULL}};
const Segment SGI264509[] = {{&S60152[135], 5}, {NULL}};
const Segment SGI264510[] = {{&S60152[68], 6}, {NULL}};
const Segment SGI264511[] = {{&S60152[102], 6}, {NULL}};
const Segment SGI264512[] = {{&S60152[116], 9}, {NULL}};
const Segment SGI264513[] = {{&S60186[13], 6}, {NULL}};
const Segment SGI264514[] = {{&S60186[35], 5}, {NULL}};
const Segment SGI264515[] = {{&S60186[75], 6}, {NULL}};
const Segment SGI264516[] = {{&S60186[109], 6}, {NULL}};
const Segment SGI264517[] = {{&S60186[116], 13}, {NULL}};
const Segment SGI264518[] = {{&S60187[13], 6}, {NULL}};
const Segment SGI264519[] = {{&S60187[35], 5}, {NULL}};
const Segment SGI264520[] = {{&S60187[75], 6}, {NULL}};
const Segment SGI264521[] = {{&S60187[109], 6}, {NULL}};
const Segment SGI264522[] = {{&S60187[116], 13}, {NULL}};
const Segment SGI264523[] = {{&S60188[13], 6}, {NULL}};
const Segment SGI264524[] = {{&S60188[35], 5}, {NULL}};
const Segment SGI264525[] = {{&S60188[75], 6}, {NULL}};
const Segment SGI264526[] = {{&S60188[109], 6}, {NULL}};
const Segment SGI264527[] = {{&S60188[116], 13}, {NULL}};
const Segment SGI264528[] = {{&S60189[13], 6}, {NULL}};
const Segment SGI264529[] = {{&S60189[35], 3}, {NULL}};
const Segment SGI264530[] = {{&S60189[73], 6}, {NULL}};
const Segment SGI264531[] = {{&S60189[107], 6}, {NULL}};
const Segment SGI264532[] = {{&S60189[114], 13}, {NULL}};
const Segment SGI264533[] = {{&S60190[13], 6}, {NULL}};
const Segment SGI264534[] = {{&S60190[35], 3}, {NULL}};
const Segment SGI264535[] = {{&S60190[73], 6}, {NULL}};
const Segment SGI264536[] = {{&S60190[107], 6}, {NULL}};
const Segment SGI264537[] = {{&S60190[114], 13}, {NULL}};
const Segment SGI264538[] = {{&S53334[25], 7}, {NULL}};
const Segment SGI264539[] = {{&S53334[0], 84}, {NULL}};
const Segment SGI264540[] = {{&S53335[128], 12}, {NULL}};
const Segment SGI264541[] = {{&S53335[0], 140}, {NULL}};
const Segment SGI264542[] = {{&S53336[123], 7}, {NULL}};
const Segment SGI264543[] = {{&S53336[0], 150}, {NULL}};
const Segment SGI264544[] = {{&S53337[64], 12}, {NULL}};
const Segment SGI264545[] = {{&S53337[0], 76}, {NULL}};
const Segment SGI264546[] = {{&S53338[51], 12}, {NULL}};
const Segment SGI264547[] = {{&S53338[0], 63}, {NULL}};
const Segment SGI264548[] = {{&S53339[147], 12}, {NULL}};
const Segment SGI264549[] = {{&S53339[0], 159}, {NULL}};
const Segment SGI264550[] = {{&S60246[13], 4}, {NULL}};
const Segment SGI264551[] = {{&S60246[26], 5}, {NULL}};
const Segment SGI264552[] = {{&S60246[59], 6}, {NULL}};
const Segment SGI264553[] = {{&S60246[93], 6}, {NULL}};
const Segment SGI264554[] = {{&S60246[100], 15}, {NULL}};
const Segment SGI264555[] = {{&S53340[58], 7}, {NULL}};
const Segment SGI264556[] = {{&S53340[0], 102}, {NULL}};
const Segment SGI264557[] = {{&S60247[13], 4}, {NULL}};
const Segment SGI264558[] = {{&S60247[26], 3}, {NULL}};
const Segment SGI264559[] = {{&S60247[57], 6}, {NULL}};
const Segment SGI264560[] = {{&S60247[91], 6}, {NULL}};
const Segment SGI264561[] = {{&S60247[98], 15}, {NULL}};
const Segment SGI264562[] = {{&S60248[13], 4}, {NULL}};
const Segment SGI264563[] = {{&S60248[26], 5}, {NULL}};
const Segment SGI264564[] = {{&S60248[59], 6}, {NULL}};
const Segment SGI264565[] = {{&S60248[93], 6}, {NULL}};
const Segment SGI264566[] = {{&S53341[75], 12}, {NULL}};
const Segment SGI264567[] = {{&S60248[100], 15}, {NULL}};
const Segment SGI264568[] = {{&S53341[0], 162}, {NULL}};
const Segment SGI264569[] = {{&S60249[27], 4}, {NULL}};
const Segment SGI264570[] = {{&S60249[43], 5}, {NULL}};
const Segment SGI264571[] = {{&S60250[13], 4}, {NULL}};
const Segment SGI264572[] = {{&S60250[26], 5}, {NULL}};
const Segment SGI264573[] = {{&S60250[59], 6}, {NULL}};
const Segment SGI264574[] = {{&S60250[93], 6}, {NULL}};
const Segment SGI264575[] = {{&S60250[100], 15}, {NULL}};
const Segment SGI264576[] = {{&S60251[27], 4}, {NULL}};
const Segment SGI264577[] = {{&S60251[43], 5}, {NULL}};
const Segment SGI264578[] = {{&S60267[13], 4}, {NULL}};
const Segment SGI264579[] = {{&S60267[114], 2}, {NULL}};
const Segment SGI264580[] = {{&S60267[26], 3}, {NULL}};
const Segment SGI264581[] = {{&S60267[57], 6}, {NULL}};
const Segment SGI264582[] = {{&S60267[26], 1}, {NULL}};
const Segment SGI264583[] = {{&S60267[139], 7}, {NULL}};
const Segment SGI264584[] = {{&S60268[13], 4}, {NULL}};
const Segment SGI264585[] = {{&S60268[116], 2}, {NULL}};
const Segment SGI264586[] = {{&S60268[26], 5}, {NULL}};
const Segment SGI264587[] = {{&S60268[59], 6}, {NULL}};
const Segment SGI264588[] = {{&S60268[26], 1}, {NULL}};
const Segment SGI264589[] = {{&S60268[143], 7}, {NULL}};
const Segment SGI264590[] = {{&S60269[13], 4}, {NULL}};
const Segment SGI264591[] = {{&S60269[116], 2}, {NULL}};
const Segment SGI264592[] = {{&S60269[26], 5}, {NULL}};
const Segment SGI264593[] = {{&S60269[59], 6}, {NULL}};
const Segment SGI264594[] = {{&S60269[26], 1}, {NULL}};
const Segment SGI264595[] = {{&S60269[143], 7}, {NULL}};
const Segment SGI264596[] = {{&S60278[13], 6}, {NULL}};
const Segment SGI264597[] = {{&S60278[110], 5}, {NULL}};
const Segment SGI264598[] = {{&S60278[38], 3}, {NULL}};
const Segment SGI264599[] = {{&S60278[76], 5}, {NULL}};
const Segment SGI264600[] = {{&S60278[123], 5}, {NULL}};
const Segment SGI264601[] = {{&S60278[136], 9}, {NULL}};
const Segment SGI264602[] = {{&S60279[13], 6}, {NULL}};
const Segment SGI264603[] = {{&S60279[113], 3}, {NULL}};
const Segment SGI264604[] = {{&S60279[38], 5}, {NULL}};
const Segment SGI264605[] = {{&S60279[78], 6}, {NULL}};
const Segment SGI264606[] = {{&S60279[124], 5}, {NULL}};
const Segment SGI264607[] = {{&S60279[137], 6}, {NULL}};
const Segment SGI264608[] = {{&S60280[13], 6}, {NULL}};
const Segment SGI264609[] = {{&S60280[112], 5}, {NULL}};
const Segment SGI264610[] = {{&S60280[38], 5}, {NULL}};
const Segment SGI264611[] = {{&S60280[78], 5}, {NULL}};
const Segment SGI264612[] = {{&S60280[125], 5}, {NULL}};
const Segment SGI264613[] = {{&S60281[13], 6}, {NULL}};
const Segment SGI264614[] = {{&S60281[112], 5}, {NULL}};
const Segment SGI264615[] = {{&S60281[38], 5}, {NULL}};
const Segment SGI264616[] = {{&S60281[78], 5}, {NULL}};
const Segment SGI264617[] = {{&S60281[125], 5}, {NULL}};
const Segment SGI264618[] = {{&S60281[138], 9}, {NULL}};
const Segment SGI264619[] = {{&S60282[13], 6}, {NULL}};
const Segment SGI264620[] = {{&S60282[35], 5}, {NULL}};
const Segment SGI264621[] = {{&S60282[68], 6}, {NULL}};
const Segment SGI264622[] = {{&S60282[102], 5}, {NULL}};
const Segment SGI264623[] = {{&S60282[115], 9}, {NULL}};
const Segment SGI264624[] = {{&S60283[13], 6}, {NULL}};
const Segment SGI264625[] = {{&S60283[111], 3}, {NULL}};
const Segment SGI264626[] = {{&S60283[38], 3}, {NULL}};
const Segment SGI264627[] = {{&S60283[76], 6}, {NULL}};
const Segment SGI264628[] = {{&S60283[122], 5}, {NULL}};
const Segment SGI264629[] = {{&S60283[135], 6}, {NULL}};
const Segment SGI264630[] = {{&S60284[13], 6}, {NULL}};
const Segment SGI264631[] = {{&S60284[110], 5}, {NULL}};
const Segment SGI264632[] = {{&S60284[38], 3}, {NULL}};
const Segment SGI264633[] = {{&S60284[76], 5}, {NULL}};
const Segment SGI264634[] = {{&S60284[123], 5}, {NULL}};
const Segment SGI264635[] = {{&S60285[13], 6}, {NULL}};
const Segment SGI264636[] = {{&S60285[107], 5}, {NULL}};
const Segment SGI264637[] = {{&S60285[36], 3}, {NULL}};
const Segment SGI264638[] = {{&S60285[73], 5}, {NULL}};
const Segment SGI264639[] = {{&S60285[120], 5}, {NULL}};
const Segment SGI264640[] = {{&S60285[133], 9}, {NULL}};
const Segment SGI264641[] = {{&S60286[13], 6}, {NULL}};
const Segment SGI264642[] = {{&S60286[112], 5}, {NULL}};
const Segment SGI264643[] = {{&S60286[38], 5}, {NULL}};
const Segment SGI264644[] = {{&S60286[78], 5}, {NULL}};
const Segment SGI264645[] = {{&S60286[125], 5}, {NULL}};
const Segment SGI264646[] = {{&S60286[138], 9}, {NULL}};
const Segment SGI264647[] = {{&S60287[13], 6}, {NULL}};
const Segment SGI264648[] = {{&S60287[106], 3}, {NULL}};
const Segment SGI264649[] = {{&S60287[31], 3}, {NULL}};
const Segment SGI264650[] = {{&S60287[69], 8}, {NULL}};
const Segment SGI264651[] = {{&S60287[117], 5}, {NULL}};
const Segment SGI264652[] = {{&S60287[130], 6}, {NULL}};
const Segment SGI264653[] = {{&S60288[13], 4}, {NULL}};
const Segment SGI264654[] = {{&S60288[26], 3}, {NULL}};
const Segment SGI264655[] = {{&S60288[57], 8}, {NULL}};
const Segment SGI264656[] = {{&S60288[111], 8}, {NULL}};
const Segment SGI264657[] = {{&S60288[127], 9}, {NULL}};
const Segment SGI264658[] = {{&S60289[13], 4}, {NULL}};
const Segment SGI264659[] = {{&S60289[26], 5}, {NULL}};
const Segment SGI264660[] = {{&S60289[59], 8}, {NULL}};
const Segment SGI264661[] = {{&S60289[113], 6}, {NULL}};
const Segment SGI264662[] = {{&S60289[127], 9}, {NULL}};
const Segment SGI264663[] = {{&S60290[13], 4}, {NULL}};
const Segment SGI264664[] = {{&S60290[26], 3}, {NULL}};
const Segment SGI264665[] = {{&S60290[57], 8}, {NULL}};
const Segment SGI264666[] = {{&S60290[111], 6}, {NULL}};
const Segment SGI264667[] = {{&S60290[125], 9}, {NULL}};
const Segment SGI264668[] = {{&S60291[13], 4}, {NULL}};
const Segment SGI264669[] = {{&S60291[26], 3}, {NULL}};
const Segment SGI264670[] = {{&S60291[57], 8}, {NULL}};
const Segment SGI264671[] = {{&S60291[111], 8}, {NULL}};
const Segment SGI264672[] = {{&S60291[127], 9}, {NULL}};
const Segment SGI264673[] = {{&S46513[10], 4}, {NULL}};
const Segment SGI264674[] = {{&S46513[15], 5}, {NULL}};
const Segment SGI264675[] = {{&S53551[58], 13}, {&S53551[58], 13}, {&S53551[129], 7}, {NULL}};
const Segment SGI264676[] = {{&S53551[0], 155}, {NULL}};
const Segment SGI264677[] = {{&S53552[85], 46}, {NULL}};
const Segment SGI264678[] = {{&S53552[0], 157}, {NULL}};
const Segment SGI264679[] = {{&S60546[13], 4}, {NULL}};
const Segment SGI264680[] = {{&S60546[113], 5}, {NULL}};
const Segment SGI264681[] = {{&S60546[36], 5}, {NULL}};
const Segment SGI264682[] = {{&S60546[76], 8}, {NULL}};
const Segment SGI264683[] = {{&S60546[126], 5}, {NULL}};
const Segment SGI264684[] = {{&S60546[139], 6}, {NULL}};
const Segment SGI264685[] = {{&S46529[20], 27}, {NULL}};
const Segment SGI264686[] = {{&S60547[13], 4}, {NULL}};
const Segment SGI264687[] = {{&S60547[111], 5}, {NULL}};
const Segment SGI264688[] = {{&S60547[36], 3}, {NULL}};
const Segment SGI264689[] = {{&S60547[74], 8}, {NULL}};
const Segment SGI264690[] = {{&S60547[124], 5}, {NULL}};
const Segment SGI264691[] = {{&S60547[137], 6}, {NULL}};
const Segment SGI264692[] = {{&S46530[20], 27}, {NULL}};
const Segment SGI264693[] = {{&S46531[20], 27}, {NULL}};
const Segment SGI264694[] = {{&S46532[20], 27}, {NULL}};
const Segment SGI264695[] = {{&S46533[20], 27}, {NULL}};
const Segment SGI264696[] = {{&S46534[20], 27}, {NULL}};
const Segment SGI264697[] = {{&S46535[20], 27}, {NULL}};
const Segment SGI264698[] = {{&S46536[20], 26}, {NULL}};
const Segment SGI264699[] = {{&S46537[20], 27}, {NULL}};
const Segment SGI264700[] = {{&S46538[20], 26}, {NULL}};
const Segment SGI264701[] = {{&S46539[20], 27}, {NULL}};
const Segment SGI264702[] = {{&S46540[20], 27}, {NULL}};
const Segment SGI264703[] = {{&S46541[20], 27}, {NULL}};
const Segment SGI264704[] = {{&S46542[20], 27}, {NULL}};
const Segment SGI264705[] = {{&S46543[20], 27}, {NULL}};
const Segment SGI264706[] = {{&S46544[20], 27}, {NULL}};
const Segment SGI264707[] = {{&S46545[20], 27}, {NULL}};
const Segment SGI264708[] = {{&S46546[20], 27}, {NULL}};
const Segment SGI264709[] = {{&S46547[20], 26}, {NULL}};
const Segment SGI264710[] = {{&S46548[20], 27}, {NULL}};
const Segment SGI264711[] = {{&S46549[20], 27}, {NULL}};
const Segment SGI264712[] = {{&S46550[20], 27}, {NULL}};
const Segment SGI264713[] = {{&S46551[20], 27}, {NULL}};
const Segment SGI264714[] = {{&S46552[20], 26}, {NULL}};
const Segment SGI264715[] = {{&S46553[20], 27}, {NULL}};
const Segment SGI264716[] = {{&S46554[20], 27}, {NULL}};
const Segment SGI264717[] = {{&S46555[20], 27}, {NULL}};
const Segment SGI264718[] = {{&S46556[20], 27}, {NULL}};
const Segment SGI264719[] = {{&S46557[20], 26}, {NULL}};
const Segment SGI264720[] = {{&S46558[20], 27}, {NULL}};
const Segment SGI264721[] = {{&S46559[20], 27}, {NULL}};
const Segment SGI264722[] = {{&S46560[20], 27}, {NULL}};
const Segment SGI264723[] = {{&S46561[20], 26}, {NULL}};
const Segment SGI264724[] = {{&S60628[13], 4}, {&S60628[13], 4}, {NULL}};
const Segment SGI264725[] = {{&S60628[29], 5}, {NULL}};
const Segment SGI264726[] = {{&S60628[69], 8}, {NULL}};
const Segment SGI264727[] = {{&S60628[105], 5}, {NULL}};
const Segment SGI264728[] = {{&S60628[111], 7}, {NULL}};
const Segment SGI264729[] = {{&S60629[13], 4}, {&S60629[13], 4}, {NULL}};
const Segment SGI264730[] = {{&S60629[29], 3}, {NULL}};
const Segment SGI264731[] = {{&S60629[67], 8}, {NULL}};
const Segment SGI264732[] = {{&S60629[103], 6}, {NULL}};
const Segment SGI264733[] = {{&S60629[110], 7}, {NULL}};
const Segment SGI264734[] = {{&S60630[13], 4}, {&S60630[13], 4}, {NULL}};
const Segment SGI264735[] = {{&S60630[29], 3}, {NULL}};
const Segment SGI264736[] = {{&S60630[67], 8}, {NULL}};
const Segment SGI264737[] = {{&S60630[103], 5}, {NULL}};
const Segment SGI264738[] = {{&S60630[109], 7}, {NULL}};
const Segment SGI264739[] = {{&S61124[13], 6}, {NULL}};
const Segment SGI264740[] = {{&S61124[38], 3}, {NULL}};
const Segment SGI264741[] = {{&S61124[76], 9}, {NULL}};
const Segment SGI264742[] = {{&S61124[113], 5}, {NULL}};
const Segment SGI264743[] = {{&S61125[13], 6}, {NULL}};
const Segment SGI264744[] = {{&S61125[38], 5}, {NULL}};
const Segment SGI264745[] = {{&S61125[78], 9}, {NULL}};
const Segment SGI264746[] = {{&S61125[115], 5}, {NULL}};
const Segment SGI264747[] = {{&S61126[13], 6}, {NULL}};
const Segment SGI264748[] = {{&S61126[38], 5}, {NULL}};
const Segment SGI264749[] = {{&S61126[78], 9}, {NULL}};
const Segment SGI264750[] = {{&S61126[115], 5}, {NULL}};
const Segment SGI264751[] = {{&S61127[13], 6}, {NULL}};
const Segment SGI264752[] = {{&S61127[38], 3}, {NULL}};
const Segment SGI264753[] = {{&S61127[76], 9}, {NULL}};
const Segment SGI264754[] = {{&S61127[113], 6}, {NULL}};
const Segment SGI264755[] = {{&S61128[13], 6}, {NULL}};
const Segment SGI264756[] = {{&S61128[38], 3}, {NULL}};
const Segment SGI264757[] = {{&S61128[76], 9}, {NULL}};
const Segment SGI264758[] = {{&S61128[113], 6}, {NULL}};
const Segment SGI264759[] = {{&S61159[17], 6}, {NULL}};
const Segment SGI264760[] = {{&S61159[38], 5}, {NULL}};
const Segment SGI264761[] = {{&S61159[64], 3}, {NULL}};
const Segment SGI264762[] = {{&S47318[20], 27}, {NULL}};
const Segment SGI264763[] = {{&S47319[20], 27}, {NULL}};
const Segment SGI264764[] = {{&S47320[20], 27}, {NULL}};
const Segment SGI264765[] = {{&S47321[20], 27}, {NULL}};
const Segment SGI264766[] = {{&S47322[20], 27}, {NULL}};
const Segment SGI264767[] = {{&S47323[20], 26}, {NULL}};
const Segment SGI264768[] = {{&S47324[20], 26}, {NULL}};
const Segment SGI264769[] = {{&S47325[20], 27}, {NULL}};
const Segment SGI264770[] = {{&S47326[20], 27}, {NULL}};
const Segment SGI264771[] = {{&S47327[20], 27}, {NULL}};
const Segment SGI264772[] = {{&S47328[20], 27}, {NULL}};
const Segment SGI264773[] = {{&S47329[20], 26}, {NULL}};
const Segment SGI264774[] = {{&S47330[20], 27}, {NULL}};
const Segment SGI264775[] = {{&S47331[20], 26}, {NULL}};
const Segment SGI264776[] = {{&S47332[20], 27}, {NULL}};
const Segment SGI264777[] = {{&S47333[20], 27}, {NULL}};
const Segment SGI264778[] = {{&S47334[20], 26}, {NULL}};
const Segment SGI264779[] = {{&S47335[20], 26}, {NULL}};
const Segment SGI264780[] = {{&S47336[20], 27}, {NULL}};
const Segment SGI264781[] = {{&S47337[20], 27}, {NULL}};
const Segment SGI264782[] = {{&S47338[20], 26}, {NULL}};
const Segment SGI264783[] = {{&S47339[20], 27}, {NULL}};
const Segment SGI264784[] = {{&S47340[20], 26}, {NULL}};
const Segment SGI264785[] = {{&S61160[17], 6}, {NULL}};
const Segment SGI264786[] = {{&S61160[38], 3}, {NULL}};
const Segment SGI264787[] = {{&S61160[62], 3}, {NULL}};
const Segment SGI264788[] = {{&S61161[17], 6}, {NULL}};
const Segment SGI264789[] = {{&S61161[38], 5}, {NULL}};
const Segment SGI264790[] = {{&S61161[64], 3}, {NULL}};
const Segment SGI264791[] = {{&S61406[13], 6}, {NULL}};
const Segment SGI264792[] = {{&S61406[115], 5}, {NULL}};
const Segment SGI264793[] = {{&S61406[38], 5}, {NULL}};
const Segment SGI264794[] = {{&S61406[78], 8}, {NULL}};
const Segment SGI264795[] = {{&S61406[128], 3}, {NULL}};
const Segment SGI264796[] = {{&S61406[139], 9}, {NULL}};
const Segment SGI264797[] = {{&S61454[13], 4}, {NULL}};
const Segment SGI264798[] = {{&S61454[36], 3}, {NULL}};
const Segment SGI264799[] = {{&S61454[74], 8}, {NULL}};
const Segment SGI264800[] = {{&S61454[129], 6}, {NULL}};
const Segment SGI264801[] = {{&S61454[143], 9}, {NULL}};
const Segment SGI264802[] = {{&S61455[13], 4}, {NULL}};
const Segment SGI264803[] = {{&S61455[36], 5}, {NULL}};
const Segment SGI264804[] = {{&S61455[76], 8}, {NULL}};
const Segment SGI264805[] = {{&S61455[131], 6}, {NULL}};
const Segment SGI264806[] = {{&S61455[145], 9}, {NULL}};
const Segment SGI264807[] = {{&S61456[13], 4}, {NULL}};
const Segment SGI264808[] = {{&S61456[36], 2}, {NULL}};
const Segment SGI264809[] = {{&S61456[73], 8}, {NULL}};
const Segment SGI264810[] = {{&S61456[127], 6}, {NULL}};
const Segment SGI264811[] = {{&S61456[141], 9}, {NULL}};
const Segment SGI264812[] = {{&S61483[13], 4}, {NULL}};
const Segment SGI264813[] = {{&S61483[36], 5}, {NULL}};
const Segment SGI264814[] = {{&S61483[76], 8}, {NULL}};
const Segment SGI264815[] = {{&S61483[131], 5}, {NULL}};
const Segment SGI264816[] = {{&S61483[144], 9}, {NULL}};
const Segment SGI264817[] = {{&S61484[13], 4}, {NULL}};
const Segment SGI264818[] = {{&S61484[36], 3}, {NULL}};
const Segment SGI264819[] = {{&S61484[74], 8}, {NULL}};
const Segment SGI264820[] = {{&S61484[129], 5}, {NULL}};
const Segment SGI264821[] = {{&S61484[142], 9}, {NULL}};
const Segment SGI264822[] = {{&S61485[13], 4}, {NULL}};
const Segment SGI264823[] = {{&S61485[36], 3}, {NULL}};
const Segment SGI264824[] = {{&S61485[74], 8}, {NULL}};
const Segment SGI264825[] = {{&S61485[128], 5}, {NULL}};
const Segment SGI264826[] = {{&S61485[141], 9}, {NULL}};
const Segment SGI264827[] = {{&S61486[13], 4}, {NULL}};
const Segment SGI264828[] = {{&S61486[36], 2}, {NULL}};
const Segment SGI264829[] = {{&S61486[73], 8}, {NULL}};
const Segment SGI264830[] = {{&S61486[127], 5}, {NULL}};
const Segment SGI264831[] = {{&S61486[140], 9}, {NULL}};
const Segment SGI264832[] = {{&S61521[13], 4}, {NULL}};
const Segment SGI264833[] = {{&S61521[26], 3}, {NULL}};
const Segment SGI264834[] = {{&S61521[57], 6}, {NULL}};
const Segment SGI264835[] = {{&S61521[106], 6}, {NULL}};
const Segment SGI264836[] = {{&S61521[120], 9}, {NULL}};
const Segment SGI264837[] = {{&S61521[84], 14}, {NULL}};
const Segment SGI264838[] = {{&S61522[13], 4}, {NULL}};
const Segment SGI264839[] = {{&S61522[26], 5}, {NULL}};
const Segment SGI264840[] = {{&S61522[59], 6}, {NULL}};
const Segment SGI264841[] = {{&S61522[108], 6}, {NULL}};
const Segment SGI264842[] = {{&S61522[122], 9}, {NULL}};
const Segment SGI264843[] = {{&S61522[86], 14}, {NULL}};
const Segment SGI264844[] = {{&S61523[13], 4}, {NULL}};
const Segment SGI264845[] = {{&S61523[26], 5}, {NULL}};
const Segment SGI264846[] = {{&S61523[59], 6}, {NULL}};
const Segment SGI264847[] = {{&S61523[108], 6}, {NULL}};
const Segment SGI264848[] = {{&S61523[122], 9}, {NULL}};
const Segment SGI264849[] = {{&S61523[86], 14}, {NULL}};
const Segment SGI264850[] = {{&S61524[13], 4}, {NULL}};
const Segment SGI264851[] = {{&S61524[26], 5}, {NULL}};
const Segment SGI264852[] = {{&S61524[59], 6}, {NULL}};
const Segment SGI264853[] = {{&S61524[108], 6}, {NULL}};
const Segment SGI264854[] = {{&S61524[122], 9}, {NULL}};
const Segment SGI264855[] = {{&S61524[86], 14}, {NULL}};
const Segment SGI264856[] = {{&S61671[17], 4}, {NULL}};
const Segment SGI264857[] = {{&S61671[36], 5}, {NULL}};
const Segment SGI264858[] = {{&S61671[66], 3}, {NULL}};
const Segment SGI264859[] = {{&S61672[17], 4}, {NULL}};
const Segment SGI264860[] = {{&S61672[36], 5}, {NULL}};
const Segment SGI264861[] = {{&S61672[66], 3}, {NULL}};
const Segment SGI264862[] = {{&S61673[17], 4}, {NULL}};
const Segment SGI264863[] = {{&S61673[36], 3}, {NULL}};
const Segment SGI264864[] = {{&S61673[64], 3}, {NULL}};
const Segment SGI264865[] = {{&S61740[13], 6}, {NULL}};
const Segment SGI264866[] = {{&S61740[101], 3}, {NULL}};
const Segment SGI264867[] = {{&S61740[35], 3}, {NULL}};
const Segment SGI264868[] = {{&S61740[66], 6}, {NULL}};
const Segment SGI264869[] = {{&S61740[112], 6}, {NULL}};
const Segment SGI264870[] = {{&S61740[126], 7}, {NULL}};
const Segment SGI264871[] = {{&S61741[13], 6}, {NULL}};
const Segment SGI264872[] = {{&S61741[101], 3}, {NULL}};
const Segment SGI264873[] = {{&S61741[35], 3}, {NULL}};
const Segment SGI264874[] = {{&S61741[66], 6}, {NULL}};
const Segment SGI264875[] = {{&S61741[112], 6}, {NULL}};
const Segment SGI264876[] = {{&S61741[126], 7}, {NULL}};
const Segment SGI264877[] = {{&S61742[13], 6}, {NULL}};
const Segment SGI264878[] = {{&S61742[103], 3}, {NULL}};
const Segment SGI264879[] = {{&S61742[35], 5}, {NULL}};
const Segment SGI264880[] = {{&S61742[68], 6}, {NULL}};
const Segment SGI264881[] = {{&S61742[114], 6}, {NULL}};
const Segment SGI264882[] = {{&S61742[128], 7}, {NULL}};
const Segment SGI264883[] = {{&S61743[13], 6}, {NULL}};
const Segment SGI264884[] = {{&S61743[103], 3}, {NULL}};
const Segment SGI264885[] = {{&S61743[35], 5}, {NULL}};
const Segment SGI264886[] = {{&S61743[68], 6}, {NULL}};
const Segment SGI264887[] = {{&S61743[114], 6}, {NULL}};
const Segment SGI264888[] = {{&S61743[128], 7}, {NULL}};
const Segment SGI264889[] = {{&S61744[13], 6}, {NULL}};
const Segment SGI264890[] = {{&S61744[101], 3}, {NULL}};
const Segment SGI264891[] = {{&S61744[35], 3}, {NULL}};
const Segment SGI264892[] = {{&S61744[66], 6}, {NULL}};
const Segment SGI264893[] = {{&S61744[112], 6}, {NULL}};
const Segment SGI264894[] = {{&S61744[126], 7}, {NULL}};
const Segment SGI264895[] = {{&S61745[13], 6}, {NULL}};
const Segment SGI264896[] = {{&S61745[101], 3}, {NULL}};
const Segment SGI264897[] = {{&S61745[35], 3}, {NULL}};
const Segment SGI264898[] = {{&S61745[66], 6}, {NULL}};
const Segment SGI264899[] = {{&S61745[112], 6}, {NULL}};
const Segment SGI264900[] = {{&S61745[126], 7}, {NULL}};
const Segment SGI264901[] = {{&S61746[13], 6}, {NULL}};
const Segment SGI264902[] = {{&S61746[103], 3}, {NULL}};
const Segment SGI264903[] = {{&S61746[35], 5}, {NULL}};
const Segment SGI264904[] = {{&S61746[68], 6}, {NULL}};
const Segment SGI264905[] = {{&S61746[114], 6}, {NULL}};
const Segment SGI264906[] = {{&S61746[128], 7}, {NULL}};
const Segment SGI264907[] = {{&S55099[0], 68}, {&S55099[163], 44}, {NULL}};
const Segment SGI264908[] = {{&S55099[0], 207}, {NULL}};
const Segment SGI264909[] = {{&S55110[45], 11}, {NULL}};
const Segment SGI264910[] = {{&S55110[0], 56}, {NULL}};
const Segment SGI264911[] = {{&S61981[13], 4}, {NULL}};
const Segment SGI264912[] = {{&S61981[36], 5}, {NULL}};
const Segment SGI264913[] = {{&S61981[76], 8}, {NULL}};
const Segment SGI264914[] = {{&S61981[131], 3}, {NULL}};
const Segment SGI264915[] = {{&S61981[142], 9}, {NULL}};
const Segment SGI264916[] = {{&S61982[13], 4}, {NULL}};
const Segment SGI264917[] = {{&S61982[36], 5}, {NULL}};
const Segment SGI264918[] = {{&S61982[76], 8}, {NULL}};
const Segment SGI264919[] = {{&S61982[131], 3}, {NULL}};
const Segment SGI264920[] = {{&S61982[142], 9}, {NULL}};
const Segment SGI264921[] = {{&S61983[13], 4}, {NULL}};
const Segment SGI264922[] = {{&S61983[36], 3}, {NULL}};
const Segment SGI264923[] = {{&S61983[74], 8}, {NULL}};
const Segment SGI264924[] = {{&S61983[129], 5}, {NULL}};
const Segment SGI264925[] = {{&S61983[142], 9}, {NULL}};
const Segment SGI264926[] = {{&S61984[13], 4}, {NULL}};
const Segment SGI264927[] = {{&S61984[36], 5}, {NULL}};
const Segment SGI264928[] = {{&S61984[76], 8}, {NULL}};
const Segment SGI264929[] = {{&S61984[131], 3}, {NULL}};
const Segment SGI264930[] = {{&S61984[142], 9}, {NULL}};
const Segment SGI264931[] = {{&S61985[13], 4}, {NULL}};
const Segment SGI264932[] = {{&S61985[36], 5}, {NULL}};
const Segment SGI264933[] = {{&S61985[76], 8}, {NULL}};
const Segment SGI264934[] = {{&S61985[131], 3}, {NULL}};
const Segment SGI264935[] = {{&S61985[142], 9}, {NULL}};
const Segment SGI264936[] = {{&S61986[13], 4}, {NULL}};
const Segment SGI264937[] = {{&S61986[36], 5}, {NULL}};
const Segment SGI264938[] = {{&S61986[76], 8}, {NULL}};
const Segment SGI264939[] = {{&S61986[131], 3}, {NULL}};
const Segment SGI264940[] = {{&S61986[142], 9}, {NULL}};
const Segment SGI264941[] = {{&S61987[13], 4}, {NULL}};
const Segment SGI264942[] = {{&S61987[36], 3}, {NULL}};
const Segment SGI264943[] = {{&S61987[74], 8}, {NULL}};
const Segment SGI264944[] = {{&S61987[128], 3}, {NULL}};
const Segment SGI264945[] = {{&S61987[139], 9}, {NULL}};
const Segment SGI264946[] = {{&S61988[13], 4}, {NULL}};
const Segment SGI264947[] = {{&S61988[36], 3}, {NULL}};
const Segment SGI264948[] = {{&S61988[74], 8}, {NULL}};
const Segment SGI264949[] = {{&S61988[128], 3}, {NULL}};
const Segment SGI264950[] = {{&S61988[139], 9}, {NULL}};
const Segment SGI264951[] = {{&S62001[28], 6}, {NULL}};
const Segment SGI264952[] = {{&S62001[46], 3}, {NULL}};
const Segment SGI264953[] = {{&S62002[28], 6}, {NULL}};
const Segment SGI264954[] = {{&S62002[46], 5}, {NULL}};
const Segment SGI264955[] = {{&S48226[22], 12}, {NULL}};
const Segment SGI264956[] = {{&S62082[13], 6}, {&S62082[13], 6}, {NULL}};
const Segment SGI264957[] = {{&S62082[35], 3}, {NULL}};
const Segment SGI264958[] = {{&S62082[66], 6}, {NULL}};
const Segment SGI264959[] = {{&S62082[100], 5}, {NULL}};
const Segment SGI264960[] = {{&S62083[13], 6}, {&S62083[13], 6}, {NULL}};
const Segment SGI264961[] = {{&S62083[35], 3}, {NULL}};
const Segment SGI264962[] = {{&S62083[66], 6}, {NULL}};
const Segment SGI264963[] = {{&S62083[100], 5}, {NULL}};
const Segment SGI264964[] = {{&S62084[13], 6}, {&S62084[13], 6}, {NULL}};
const Segment SGI264965[] = {{&S62084[35], 3}, {NULL}};
const Segment SGI264966[] = {{&S62084[66], 6}, {NULL}};
const Segment SGI264967[] = {{&S62084[100], 5}, {NULL}};
const Segment SGI264968[] = {{&S62084[113], 9}, {NULL}};
const Segment SGI264969[] = {{&S62085[13], 6}, {&S62085[13], 6}, {NULL}};
const Segment SGI264970[] = {{&S62085[35], 3}, {NULL}};
const Segment SGI264971[] = {{&S62085[66], 6}, {NULL}};
const Segment SGI264972[] = {{&S62085[100], 5}, {NULL}};
const Segment SGI264973[] = {{&S62085[113], 9}, {NULL}};
const Segment SGI264974[] = {{&S62086[13], 6}, {&S62086[13], 6}, {NULL}};
const Segment SGI264975[] = {{&S62086[35], 5}, {NULL}};
const Segment SGI264976[] = {{&S62086[68], 6}, {NULL}};
const Segment SGI264977[] = {{&S62086[102], 5}, {NULL}};
const Segment SGI264978[] = {{&S62086[115], 9}, {NULL}};
const Segment SGI264979[] = {{&S62087[13], 6}, {&S62087[13], 6}, {NULL}};
const Segment SGI264980[] = {{&S62087[35], 5}, {NULL}};
const Segment SGI264981[] = {{&S62087[68], 6}, {NULL}};
const Segment SGI264982[] = {{&S62087[102], 5}, {NULL}};
const Segment SGI264983[] = {{&S62088[13], 6}, {&S62088[13], 6}, {NULL}};
const Segment SGI264984[] = {{&S62088[35], 5}, {NULL}};
const Segment SGI264985[] = {{&S62088[68], 6}, {NULL}};
const Segment SGI264986[] = {{&S62088[102], 5}, {NULL}};
const Segment SGI264987[] = {{&S62088[115], 9}, {NULL}};
const Segment SGI264988[] = {{&S62147[13], 4}, {NULL}};
const Segment SGI264989[] = {{&S62147[33], 5}, {NULL}};
const Segment SGI264990[] = {{&S62147[66], 6}, {NULL}};
const Segment SGI264991[] = {{&S62147[100], 5}, {NULL}};
const Segment SGI264992[] = {{&S62147[106], 7}, {NULL}};
const Segment SGI264993[] = {{&S62148[13], 4}, {NULL}};
const Segment SGI264994[] = {{&S62148[33], 3}, {NULL}};
const Segment SGI264995[] = {{&S62148[64], 6}, {NULL}};
const Segment SGI264996[] = {{&S62148[98], 5}, {NULL}};
const Segment SGI264997[] = {{&S62148[104], 7}, {NULL}};
const Segment SGI264998[] = {{&S62221[13], 6}, {NULL}};
const Segment SGI264999[] = {{&S62221[38], 5}, {NULL}};
const Segment SGI265000[] = {{&S62221[78], 5}, {NULL}};
const Segment SGI265001[] = {{&S62221[111], 5}, {NULL}};
const Segment SGI265002[] = {{&S62221[117], 7}, {NULL}};
const Segment SGI265003[] = {{&S62222[13], 6}, {NULL}};
const Segment SGI265004[] = {{&S62222[38], 3}, {NULL}};
const Segment SGI265005[] = {{&S62222[76], 5}, {NULL}};
const Segment SGI265006[] = {{&S62222[109], 5}, {NULL}};
const Segment SGI265007[] = {{&S62222[115], 7}, {NULL}};
const Segment SGI265008[] = {{&S62223[13], 6}, {NULL}};
const Segment SGI265009[] = {{&S62223[38], 5}, {NULL}};
const Segment SGI265010[] = {{&S62223[78], 5}, {NULL}};
const Segment SGI265011[] = {{&S62223[111], 5}, {NULL}};
const Segment SGI265012[] = {{&S62223[117], 7}, {NULL}};
const Segment SGI265013[] = {{&S62285[13], 4}, {NULL}};
const Segment SGI265014[] = {{&S62285[111], 3}, {NULL}};
const Segment SGI265015[] = {{&S62285[36], 5}, {NULL}};
const Segment SGI265016[] = {{&S62285[76], 6}, {NULL}};
const Segment SGI265017[] = {{&S62285[122], 5}, {NULL}};
const Segment SGI265018[] = {{&S62285[135], 6}, {NULL}};
const Segment SGI265019[] = {{&S62286[13], 4}, {NULL}};
const Segment SGI265020[] = {{&S62286[109], 3}, {NULL}};
const Segment SGI265021[] = {{&S62286[36], 3}, {NULL}};
const Segment SGI265022[] = {{&S62286[74], 6}, {NULL}};
const Segment SGI265023[] = {{&S62286[120], 5}, {NULL}};
const Segment SGI265024[] = {{&S62286[133], 6}, {NULL}};
const Segment SGI265025[] = {{&S62287[13], 4}, {NULL}};
const Segment SGI265026[] = {{&S62287[105], 5}, {NULL}};
const Segment SGI265027[] = {{&S62287[29], 3}, {NULL}};
const Segment SGI265028[] = {{&S62287[67], 9}, {NULL}};
const Segment SGI265029[] = {{&S62287[118], 5}, {NULL}};
const Segment SGI265030[] = {{&S55441[149], 21}, {&S55441[171], 32}, {NULL}};
const Segment SGI265031[] = {{&S55441[0], 203}, {NULL}};
const Segment SGI265032[] = {{&S55442[163], 13}, {NULL}};
const Segment SGI265033[] = {{&S55442[0], 176}, {NULL}};
const Segment SGI265034[] = {{&S62341[27], 6}, {NULL}};
const Segment SGI265035[] = {{&S62341[52], 5}, {NULL}};
const Segment SGI265036[] = {{&S62341[85], 6}, {NULL}};
const Segment SGI265037[] = {{&S62341[119], 6}, {NULL}};
const Segment SGI265038[] = {{&S62341[133], 9}, {NULL}};
const Segment SGI265039[] = {{&S62341[0], 14}, {NULL}};
const Segment SGI265040[] = {{&S62342[27], 6}, {NULL}};
const Segment SGI265041[] = {{&S62342[52], 5}, {NULL}};
const Segment SGI265042[] = {{&S62342[85], 6}, {NULL}};
const Segment SGI265043[] = {{&S62342[119], 6}, {NULL}};
const Segment SGI265044[] = {{&S62342[133], 9}, {NULL}};
const Segment SGI265045[] = {{&S62342[0], 14}, {NULL}};
const Segment SGI265046[] = {{&S62343[27], 6}, {NULL}};
const Segment SGI265047[] = {{&S62343[51], 5}, {NULL}};
const Segment SGI265048[] = {{&S62343[84], 6}, {NULL}};
const Segment SGI265049[] = {{&S62343[118], 6}, {NULL}};
const Segment SGI265050[] = {{&S62343[132], 9}, {NULL}};
const Segment SGI265051[] = {{&S62343[0], 14}, {NULL}};
const Segment SGI265052[] = {{&S62344[27], 6}, {NULL}};
const Segment SGI265053[] = {{&S62344[52], 3}, {NULL}};
const Segment SGI265054[] = {{&S62344[83], 6}, {NULL}};
const Segment SGI265055[] = {{&S62344[117], 6}, {NULL}};
const Segment SGI265056[] = {{&S62344[131], 9}, {NULL}};
const Segment SGI265057[] = {{&S62344[0], 14}, {NULL}};
const Segment SGI265058[] = {{&S62345[27], 6}, {NULL}};
const Segment SGI265059[] = {{&S62345[51], 3}, {NULL}};
const Segment SGI265060[] = {{&S62345[82], 6}, {NULL}};
const Segment SGI265061[] = {{&S62345[116], 6}, {NULL}};
const Segment SGI265062[] = {{&S62345[130], 9}, {NULL}};
const Segment SGI265063[] = {{&S62345[0], 14}, {NULL}};
const Segment SGI265064[] = {{&S62346[27], 6}, {NULL}};
const Segment SGI265065[] = {{&S62346[51], 3}, {NULL}};
const Segment SGI265066[] = {{&S62346[82], 6}, {NULL}};
const Segment SGI265067[] = {{&S62346[116], 6}, {NULL}};
const Segment SGI265068[] = {{&S62346[130], 9}, {NULL}};
const Segment SGI265069[] = {{&S62346[0], 14}, {NULL}};
const Segment SGI265070[] = {{&S62357[13], 4}, {NULL}};
const Segment SGI265071[] = {{&S62357[26], 3}, {NULL}};
const Segment SGI265072[] = {{&S62357[64], 6}, {NULL}};
const Segment SGI265073[] = {{&S62357[118], 6}, {NULL}};
const Segment SGI265074[] = {{&S62357[132], 7}, {NULL}};
const Segment SGI265075[] = {{&S62358[13], 4}, {NULL}};
const Segment SGI265076[] = {{&S62358[26], 5}, {NULL}};
const Segment SGI265077[] = {{&S62358[66], 6}, {NULL}};
const Segment SGI265078[] = {{&S62358[120], 6}, {NULL}};
const Segment SGI265079[] = {{&S62358[134], 7}, {NULL}};
const Segment SGI265080[] = {{&S62379[17], 6}, {NULL}};
const Segment SGI265081[] = {{&S62379[38], 5}, {NULL}};
const Segment SGI265082[] = {{&S62379[68], 3}, {NULL}};
const Segment SGI265083[] = {{&S62380[17], 6}, {NULL}};
const Segment SGI265084[] = {{&S62380[38], 3}, {NULL}};
const Segment SGI265085[] = {{&S62380[66], 3}, {NULL}};
const Segment SGI265086[] = {{&S62410[13], 6}, {&S62410[13], 6}, {NULL}};
const Segment SGI265087[] = {{&S62410[199], 3}, {NULL}};
const Segment SGI265088[] = {{&S62410[35], 3}, {NULL}};
const Segment SGI265089[] = {{&S62410[66], 6}, {&S62410[166], 3}, {NULL}};
const Segment SGI265090[] = {{&S62410[100], 6}, {&S62410[210], 3}, {NULL}};
const Segment SGI265091[] = {{&S62410[221], 5}, {NULL}};
const Segment SGI265092[] = {{&S62411[13], 6}, {&S62411[13], 6}, {NULL}};
const Segment SGI265093[] = {{&S62411[199], 3}, {NULL}};
const Segment SGI265094[] = {{&S62411[35], 3}, {NULL}};
const Segment SGI265095[] = {{&S62411[66], 6}, {&S62411[166], 3}, {NULL}};
const Segment SGI265096[] = {{&S62411[100], 6}, {&S62411[210], 3}, {NULL}};
const Segment SGI265097[] = {{&S62411[221], 5}, {NULL}};
const Segment SGI265098[] = {{&S62412[13], 6}, {&S62412[13], 6}, {NULL}};
const Segment SGI265099[] = {{&S62412[201], 3}, {NULL}};
const Segment SGI265100[] = {{&S62412[35], 5}, {NULL}};
const Segment SGI265101[] = {{&S62412[68], 6}, {&S62412[168], 3}, {NULL}};
const Segment SGI265102[] = {{&S62412[102], 6}, {&S62412[212], 3}, {NULL}};
const Segment SGI265103[] = {{&S62412[223], 5}, {NULL}};
const Segment SGI265104[] = {{&S62413[13], 6}, {&S62413[13], 6}, {NULL}};
const Segment SGI265105[] = {{&S62413[201], 3}, {NULL}};
const Segment SGI265106[] = {{&S62413[35], 5}, {NULL}};
const Segment SGI265107[] = {{&S62413[68], 6}, {&S62413[168], 3}, {NULL}};
const Segment SGI265108[] = {{&S62413[102], 6}, {&S62413[212], 3}, {NULL}};
const Segment SGI265109[] = {{&S62413[223], 5}, {NULL}};
const Segment SGI265110[] = {{&S62414[13], 6}, {&S62414[13], 6}, {NULL}};
const Segment SGI265111[] = {{&S62414[201], 3}, {NULL}};
const Segment SGI265112[] = {{&S62414[35], 5}, {NULL}};
const Segment SGI265113[] = {{&S62414[68], 6}, {&S62414[168], 3}, {NULL}};
const Segment SGI265114[] = {{&S62414[102], 6}, {&S62414[212], 3}, {NULL}};
const Segment SGI265115[] = {{&S62414[223], 5}, {NULL}};
const Segment SGI265116[] = {{&S62497[13], 4}, {NULL}};
const Segment SGI265117[] = {{&S62497[26], 3}, {NULL}};
const Segment SGI265118[] = {{&S62497[57], 6}, {NULL}};
const Segment SGI265119[] = {{&S62497[113], 5}, {NULL}};
const Segment SGI265120[] = {{&S62497[126], 9}, {NULL}};
const Segment SGI265121[] = {{&S62498[13], 4}, {NULL}};
const Segment SGI265122[] = {{&S62498[26], 5}, {NULL}};
const Segment SGI265123[] = {{&S62498[59], 6}, {NULL}};
const Segment SGI265124[] = {{&S62498[117], 5}, {NULL}};
const Segment SGI265125[] = {{&S62498[130], 9}, {NULL}};
const Segment SGI265126[] = {{&S62499[13], 4}, {NULL}};
const Segment SGI265127[] = {{&S62499[26], 5}, {NULL}};
const Segment SGI265128[] = {{&S62499[59], 6}, {NULL}};
const Segment SGI265129[] = {{&S62499[117], 5}, {NULL}};
const Segment SGI265130[] = {{&S62499[130], 9}, {NULL}};
const Segment SGI265131[] = {{&S62546[13], 4}, {NULL}};
const Segment SGI265132[] = {{&S62546[26], 3}, {NULL}};
const Segment SGI265133[] = {{&S62546[57], 6}, {NULL}};
const Segment SGI265134[] = {{&S62546[111], 6}, {NULL}};
const Segment SGI265135[] = {{&S62546[125], 7}, {NULL}};
const Segment SGI265136[] = {{&S62547[13], 4}, {NULL}};
const Segment SGI265137[] = {{&S62547[26], 5}, {NULL}};
const Segment SGI265138[] = {{&S62547[59], 6}, {NULL}};
const Segment SGI265139[] = {{&S62547[113], 6}, {NULL}};
const Segment SGI265140[] = {{&S62547[127], 7}, {NULL}};
const Segment SGI265141[] = {{&S62872[13], 6}, {NULL}};
const Segment SGI265142[] = {{&S62872[35], 5}, {NULL}};
const Segment SGI265143[] = {{&S62872[75], 6}, {NULL}};
const Segment SGI265144[] = {{&S62872[109], 5}, {NULL}};
const Segment SGI265145[] = {{&S62872[115], 13}, {NULL}};
const Segment SGI265146[] = {{&S62873[13], 6}, {NULL}};
const Segment SGI265147[] = {{&S62873[35], 5}, {NULL}};
const Segment SGI265148[] = {{&S62873[75], 6}, {NULL}};
const Segment SGI265149[] = {{&S62873[109], 5}, {NULL}};
const Segment SGI265150[] = {{&S62873[115], 13}, {NULL}};
const Segment SGI265151[] = {{&S62874[13], 6}, {NULL}};
const Segment SGI265152[] = {{&S62874[35], 3}, {NULL}};
const Segment SGI265153[] = {{&S62874[73], 6}, {NULL}};
const Segment SGI265154[] = {{&S62874[107], 5}, {NULL}};
const Segment SGI265155[] = {{&S62874[113], 13}, {NULL}};
const Segment SGI265156[] = {{&S62875[13], 6}, {NULL}};
const Segment SGI265157[] = {{&S62875[35], 3}, {NULL}};
const Segment SGI265158[] = {{&S62875[73], 6}, {NULL}};
const Segment SGI265159[] = {{&S62875[107], 5}, {NULL}};
const Segment SGI265160[] = {{&S62875[113], 13}, {NULL}};
const Segment SGI265161[] = {{&S62983[28], 4}, {NULL}};
const Segment SGI265162[] = {{&S62983[44], 3}, {NULL}};
const Segment SGI265163[] = {{&S62984[28], 4}, {NULL}};
const Segment SGI265164[] = {{&S62984[44], 5}, {NULL}};
const Segment SGI265165[] = {{&S63048[13], 4}, {NULL}};
const Segment SGI265166[] = {{&S63048[39], 3}, {NULL}};
const Segment SGI265167[] = {{&S63048[70], 6}, {NULL}};
const Segment SGI265168[] = {{&S63048[104], 8}, {NULL}};
const Segment SGI265169[] = {{&S63048[113], 7}, {NULL}};
const Segment SGI265170[] = {{&S63049[13], 4}, {NULL}};
const Segment SGI265171[] = {{&S63049[39], 3}, {NULL}};
const Segment SGI265172[] = {{&S63049[70], 8}, {NULL}};
const Segment SGI265173[] = {{&S63049[106], 8}, {NULL}};
const Segment SGI265174[] = {{&S63049[115], 7}, {NULL}};
const Segment SGI265175[] = {{&S63050[13], 4}, {NULL}};
const Segment SGI265176[] = {{&S63050[39], 3}, {NULL}};
const Segment SGI265177[] = {{&S63050[70], 8}, {NULL}};
const Segment SGI265178[] = {{&S63050[106], 8}, {NULL}};
const Segment SGI265179[] = {{&S63050[115], 7}, {NULL}};
const Segment SGI265180[] = {{&S63051[13], 4}, {NULL}};
const Segment SGI265181[] = {{&S63051[39], 3}, {NULL}};
const Segment SGI265182[] = {{&S63051[70], 8}, {NULL}};
const Segment SGI265183[] = {{&S63051[106], 8}, {NULL}};
const Segment SGI265184[] = {{&S63051[115], 7}, {NULL}};
const Segment SGI265185[] = {{&S63052[13], 4}, {NULL}};
const Segment SGI265186[] = {{&S63052[39], 3}, {NULL}};
const Segment SGI265187[] = {{&S63052[70], 8}, {NULL}};
const Segment SGI265188[] = {{&S63052[106], 6}, {NULL}};
const Segment SGI265189[] = {{&S63052[113], 7}, {NULL}};
const Segment SGI265190[] = {{&S63053[13], 4}, {NULL}};
const Segment SGI265191[] = {{&S63053[39], 5}, {NULL}};
const Segment SGI265192[] = {{&S63053[72], 8}, {NULL}};
const Segment SGI265193[] = {{&S63053[108], 6}, {NULL}};
const Segment SGI265194[] = {{&S63053[115], 7}, {NULL}};
const Segment SGI265195[] = {{&S63253[13], 4}, {NULL}};
const Segment SGI265196[] = {{&S63253[26], 5}, {NULL}};
const Segment SGI265197[] = {{&S63253[59], 6}, {NULL}};
const Segment SGI265198[] = {{&S63253[108], 5}, {NULL}};
const Segment SGI265199[] = {{&S63253[121], 9}, {NULL}};
const Segment SGI265200[] = {{&S63253[86], 14}, {NULL}};
const Segment SGI265201[] = {{&S63254[13], 4}, {NULL}};
const Segment SGI265202[] = {{&S63254[26], 3}, {NULL}};
const Segment SGI265203[] = {{&S63254[57], 6}, {NULL}};
const Segment SGI265204[] = {{&S63254[106], 5}, {NULL}};
const Segment SGI265205[] = {{&S63254[119], 9}, {NULL}};
const Segment SGI265206[] = {{&S63254[84], 14}, {NULL}};
const Segment SGI265207[] = {{&S63352[13], 6}, {NULL}};
const Segment SGI265208[] = {{&S63352[123], 2}, {NULL}};
const Segment SGI265209[] = {{&S63352[35], 3}, {NULL}};
const Segment SGI265210[] = {{&S63352[66], 6}, {NULL}};
const Segment SGI265211[] = {{&S63352[35], 1}, {NULL}};
const Segment SGI265212[] = {{&S63352[148], 7}, {NULL}};
const Segment SGI265213[] = {{&S63353[13], 6}, {NULL}};
const Segment SGI265214[] = {{&S63353[125], 2}, {NULL}};
const Segment SGI265215[] = {{&S63353[35], 5}, {NULL}};
const Segment SGI265216[] = {{&S63353[68], 6}, {NULL}};
const Segment SGI265217[] = {{&S63353[35], 1}, {NULL}};
const Segment SGI265218[] = {{&S63353[152], 7}, {NULL}};
const Segment SGI265219[] = {{&S63354[13], 6}, {NULL}};
const Segment SGI265220[] = {{&S63354[125], 2}, {NULL}};
const Segment SGI265221[] = {{&S63354[35], 5}, {NULL}};
const Segment SGI265222[] = {{&S63354[68], 6}, {NULL}};
const Segment SGI265223[] = {{&S63354[35], 1}, {NULL}};
const Segment SGI265224[] = {{&S63354[152], 7}, {NULL}};
const Segment SGI265225[] = {{&S63355[13], 6}, {NULL}};
const Segment SGI265226[] = {{&S63355[125], 2}, {NULL}};
const Segment SGI265227[] = {{&S63355[35], 5}, {NULL}};
const Segment SGI265228[] = {{&S63355[68], 6}, {NULL}};
const Segment SGI265229[] = {{&S63355[35], 1}, {NULL}};
const Segment SGI265230[] = {{&S63355[152], 7}, {NULL}};
const Segment SGI265231[] = {{&S63356[13], 6}, {NULL}};
const Segment SGI265232[] = {{&S63356[125], 2}, {NULL}};
const Segment SGI265233[] = {{&S63356[35], 5}, {NULL}};
const Segment SGI265234[] = {{&S63356[68], 6}, {NULL}};
const Segment SGI265235[] = {{&S63356[35], 1}, {NULL}};
const Segment SGI265236[] = {{&S63356[152], 7}, {NULL}};
const Segment SGI265237[] = {{&S63667[13], 4}, {&S63667[13], 4}, {NULL}};
const Segment SGI265238[] = {{&S63667[203], 5}, {NULL}};
const Segment SGI265239[] = {{&S63667[26], 3}, {&S63667[127], 3}, {NULL}};
const Segment SGI265240[] = {{&S63667[57], 6}, {&S63667[165], 9}, {NULL}};
const Segment SGI265241[] = {{&S63667[91], 6}, {&S63667[216], 5}, {NULL}};
const Segment SGI265242[] = {{&S63667[165], 9}, {NULL}};
const Segment SGI265243[] = {{&S63668[13], 4}, {NULL}};
const Segment SGI265244[] = {{&S63668[109], 3}, {NULL}};
const Segment SGI265245[] = {{&S63668[36], 3}, {NULL}};
const Segment SGI265246[] = {{&S63668[74], 6}, {NULL}};
const Segment SGI265247[] = {{&S63668[120], 5}, {NULL}};
const Segment SGI265248[] = {{&S63668[133], 6}, {NULL}};
const Segment SGI265249[] = {{&S63669[13], 4}, {NULL}};
const Segment SGI265250[] = {{&S63669[111], 3}, {NULL}};
const Segment SGI265251[] = {{&S63669[36], 5}, {NULL}};
const Segment SGI265252[] = {{&S63669[76], 6}, {NULL}};
const Segment SGI265253[] = {{&S63669[122], 5}, {NULL}};
const Segment SGI265254[] = {{&S63669[135], 6}, {NULL}};
const Segment SGI265255[] = {{&S63670[13], 4}, {&S63670[13], 4}, {NULL}};
const Segment SGI265256[] = {{&S63670[205], 5}, {NULL}};
const Segment SGI265257[] = {{&S63670[26], 5}, {&S63670[129], 3}, {NULL}};
const Segment SGI265258[] = {{&S63670[59], 6}, {&S63670[167], 9}, {NULL}};
const Segment SGI265259[] = {{&S63670[93], 6}, {&S63670[218], 5}, {NULL}};
const Segment SGI265260[] = {{&S63670[167], 9}, {NULL}};
const Segment SGI265261[] = {{&S63671[13], 4}, {NULL}};
const Segment SGI265262[] = {{&S63671[111], 3}, {NULL}};
const Segment SGI265263[] = {{&S63671[36], 5}, {NULL}};
const Segment SGI265264[] = {{&S63671[76], 6}, {NULL}};
const Segment SGI265265[] = {{&S63671[122], 5}, {NULL}};
const Segment SGI265266[] = {{&S63671[135], 6}, {NULL}};
const Segment SGI265267[] = {{&S63672[13], 4}, {NULL}};
const Segment SGI265268[] = {{&S63672[105], 5}, {NULL}};
const Segment SGI265269[] = {{&S63672[29], 3}, {NULL}};
const Segment SGI265270[] = {{&S63672[67], 9}, {NULL}};
const Segment SGI265271[] = {{&S63672[118], 5}, {NULL}};
const Segment SGI265272[] = {{&S63676[13], 4}, {&S63676[13], 4}, {NULL}};
const Segment SGI265273[] = {{&S63676[26], 3}, {NULL}};
const Segment SGI265274[] = {{&S63676[57], 6}, {NULL}};
const Segment SGI265275[] = {{&S63676[91], 5}, {NULL}};
const Segment SGI265276[] = {{&S63676[104], 9}, {NULL}};
const Segment SGI265277[] = {{&S63677[13], 4}, {&S63677[13], 4}, {NULL}};
const Segment SGI265278[] = {{&S63677[26], 5}, {NULL}};
const Segment SGI265279[] = {{&S63677[59], 6}, {NULL}};
const Segment SGI265280[] = {{&S63677[93], 5}, {NULL}};
const Segment SGI265281[] = {{&S63677[106], 9}, {NULL}};
const Segment SGI265282[] = {{&S63767[27], 6}, {NULL}};
const Segment SGI265283[] = {{&S63767[52], 5}, {NULL}};
const Segment SGI265284[] = {{&S63767[85], 6}, {NULL}};
const Segment SGI265285[] = {{&S63767[119], 6}, {NULL}};
const Segment SGI265286[] = {{&S63767[133], 9}, {NULL}};
const Segment SGI265287[] = {{&S63767[0], 14}, {NULL}};
const Segment SGI265288[] = {{&S63768[27], 6}, {NULL}};
const Segment SGI265289[] = {{&S63768[51], 5}, {NULL}};
const Segment SGI265290[] = {{&S63768[84], 6}, {NULL}};
const Segment SGI265291[] = {{&S63768[118], 6}, {NULL}};
const Segment SGI265292[] = {{&S63768[132], 9}, {NULL}};
const Segment SGI265293[] = {{&S63768[0], 14}, {NULL}};
const Segment SGI265294[] = {{&S63769[27], 6}, {NULL}};
const Segment SGI265295[] = {{&S63769[52], 3}, {NULL}};
const Segment SGI265296[] = {{&S63769[83], 6}, {NULL}};
const Segment SGI265297[] = {{&S63769[117], 6}, {NULL}};
const Segment SGI265298[] = {{&S63769[131], 9}, {NULL}};
const Segment SGI265299[] = {{&S63769[0], 14}, {NULL}};
const Segment SGI265300[] = {{&S63770[28], 6}, {NULL}};
const Segment SGI265301[] = {{&S63770[53], 5}, {NULL}};
const Segment SGI265302[] = {{&S63770[86], 6}, {NULL}};
const Segment SGI265303[] = {{&S63770[120], 6}, {NULL}};
const Segment SGI265304[] = {{&S63770[134], 9}, {NULL}};
const Segment SGI265305[] = {{&S63770[0], 15}, {NULL}};
const Segment SGI265306[] = {{&S63771[28], 6}, {NULL}};
const Segment SGI265307[] = {{&S63771[52], 5}, {NULL}};
const Segment SGI265308[] = {{&S63771[85], 6}, {NULL}};
const Segment SGI265309[] = {{&S63771[119], 6}, {NULL}};
const Segment SGI265310[] = {{&S63771[133], 9}, {NULL}};
const Segment SGI265311[] = {{&S63771[0], 15}, {NULL}};
const Segment SGI265312[] = {{&S63772[27], 6}, {NULL}};
const Segment SGI265313[] = {{&S63772[51], 5}, {NULL}};
const Segment SGI265314[] = {{&S63772[84], 6}, {NULL}};
const Segment SGI265315[] = {{&S63772[118], 6}, {NULL}};
const Segment SGI265316[] = {{&S63772[132], 9}, {NULL}};
const Segment SGI265317[] = {{&S63772[0], 14}, {NULL}};
const Segment SGI265318[] = {{&S63773[28], 6}, {NULL}};
const Segment SGI265319[] = {{&S63773[53], 3}, {NULL}};
const Segment SGI265320[] = {{&S63773[84], 6}, {NULL}};
const Segment SGI265321[] = {{&S63773[118], 6}, {NULL}};
const Segment SGI265322[] = {{&S63773[132], 9}, {NULL}};
const Segment SGI265323[] = {{&S63773[0], 15}, {NULL}};
const Segment SGI265324[] = {{&S63774[28], 6}, {NULL}};
const Segment SGI265325[] = {{&S63774[52], 5}, {NULL}};
const Segment SGI265326[] = {{&S63774[85], 6}, {NULL}};
const Segment SGI265327[] = {{&S63774[119], 6}, {NULL}};
const Segment SGI265328[] = {{&S63774[133], 9}, {NULL}};
const Segment SGI265329[] = {{&S63774[0], 15}, {NULL}};
const Segment SGI265330[] = {{&S63947[13], 4}, {NULL}};
const Segment SGI265331[] = {{&S63947[26], 5}, {NULL}};
const Segment SGI265332[] = {{&S63947[59], 6}, {NULL}};
const Segment SGI265333[] = {{&S63947[108], 6}, {NULL}};
const Segment SGI265334[] = {{&S63947[122], 9}, {NULL}};
const Segment SGI265335[] = {{&S63947[86], 14}, {NULL}};
const Segment SGI265336[] = {{&S63948[13], 4}, {NULL}};
const Segment SGI265337[] = {{&S63948[26], 5}, {NULL}};
const Segment SGI265338[] = {{&S63948[59], 6}, {NULL}};
const Segment SGI265339[] = {{&S63948[108], 6}, {NULL}};
const Segment SGI265340[] = {{&S63948[122], 9}, {NULL}};
const Segment SGI265341[] = {{&S63948[86], 14}, {NULL}};
const Segment SGI265342[] = {{&S63949[13], 4}, {NULL}};
const Segment SGI265343[] = {{&S63949[26], 3}, {NULL}};
const Segment SGI265344[] = {{&S63949[57], 6}, {NULL}};
const Segment SGI265345[] = {{&S63949[106], 6}, {NULL}};
const Segment SGI265346[] = {{&S63949[120], 9}, {NULL}};
const Segment SGI265347[] = {{&S63949[84], 14}, {NULL}};
const Segment SGI265348[] = {{&S63950[13], 4}, {NULL}};
const Segment SGI265349[] = {{&S63950[26], 5}, {NULL}};
const Segment SGI265350[] = {{&S63950[59], 6}, {NULL}};
const Segment SGI265351[] = {{&S63950[108], 6}, {NULL}};
const Segment SGI265352[] = {{&S63950[122], 9}, {NULL}};
const Segment SGI265353[] = {{&S63950[86], 14}, {NULL}};
const Segment SGI265354[] = {{&S63953[13], 6}, {NULL}};
const Segment SGI265355[] = {{&S63953[38], 3}, {NULL}};
const Segment SGI265356[] = {{&S63953[111], 5}, {NULL}};
const Segment SGI265357[] = {{&S63953[124], 9}, {NULL}};
const Segment SGI265358[] = {{&S63954[13], 6}, {NULL}};
const Segment SGI265359[] = {{&S63954[101], 5}, {NULL}};
const Segment SGI265360[] = {{&S63954[35], 3}, {NULL}};
const Segment SGI265361[] = {{&S63954[66], 6}, {NULL}};
const Segment SGI265362[] = {{&S63954[114], 5}, {NULL}};
const Segment SGI265363[] = {{&S63954[127], 9}, {NULL}};
const Segment SGI265364[] = {{&S63955[13], 6}, {NULL}};
const Segment SGI265365[] = {{&S63955[101], 5}, {NULL}};
const Segment SGI265366[] = {{&S63955[35], 3}, {NULL}};
const Segment SGI265367[] = {{&S63955[66], 6}, {NULL}};
const Segment SGI265368[] = {{&S63955[114], 5}, {NULL}};
const Segment SGI265369[] = {{&S63955[127], 9}, {NULL}};
const Segment SGI265370[] = {{&S63956[13], 6}, {NULL}};
const Segment SGI265371[] = {{&S63956[114], 5}, {NULL}};
const Segment SGI265372[] = {{&S63956[38], 5}, {NULL}};
const Segment SGI265373[] = {{&S63956[78], 7}, {NULL}};
const Segment SGI265374[] = {{&S63956[127], 6}, {NULL}};
const Segment SGI265375[] = {{&S63956[141], 9}, {NULL}};
const Segment SGI265376[] = {{&S63957[13], 6}, {NULL}};
const Segment SGI265377[] = {{&S63957[112], 5}, {NULL}};
const Segment SGI265378[] = {{&S63957[38], 3}, {NULL}};
const Segment SGI265379[] = {{&S63957[76], 7}, {NULL}};
const Segment SGI265380[] = {{&S63957[125], 6}, {NULL}};
const Segment SGI265381[] = {{&S63957[139], 9}, {NULL}};
const Segment SGI265382[] = {{&S63958[13], 6}, {NULL}};
const Segment SGI265383[] = {{&S63958[103], 5}, {NULL}};
const Segment SGI265384[] = {{&S63958[35], 5}, {NULL}};
const Segment SGI265385[] = {{&S63958[68], 6}, {NULL}};
const Segment SGI265386[] = {{&S63958[116], 5}, {NULL}};
const Segment SGI265387[] = {{&S63958[129], 9}, {NULL}};
const Segment SGI265388[] = {{&S63959[13], 6}, {NULL}};
const Segment SGI265389[] = {{&S63959[38], 5}, {NULL}};
const Segment SGI265390[] = {{&S63959[113], 5}, {NULL}};
const Segment SGI265391[] = {{&S63959[126], 9}, {NULL}};
const Segment SGI265392[] = {{&S63960[13], 6}, {NULL}};
const Segment SGI265393[] = {{&S63960[35], 5}, {NULL}};
const Segment SGI265394[] = {{&S63960[68], 6}, {NULL}};
const Segment SGI265395[] = {{&S63960[102], 5}, {NULL}};
const Segment SGI265396[] = {{&S63960[115], 9}, {NULL}};
const Segment SGI265397[] = {{&S63961[13], 6}, {NULL}};
const Segment SGI265398[] = {{&S63961[115], 5}, {NULL}};
const Segment SGI265399[] = {{&S63961[38], 5}, {NULL}};
const Segment SGI265400[] = {{&S63961[78], 8}, {NULL}};
const Segment SGI265401[] = {{&S63961[128], 5}, {NULL}};
const Segment SGI265402[] = {{&S63961[141], 9}, {NULL}};
const Segment SGI265403[] = {{&S63962[13], 6}, {NULL}};
const Segment SGI265404[] = {{&S63962[115], 5}, {NULL}};
const Segment SGI265405[] = {{&S63962[38], 5}, {NULL}};
const Segment SGI265406[] = {{&S63962[78], 8}, {NULL}};
const Segment SGI265407[] = {{&S63962[128], 5}, {NULL}};
const Segment SGI265408[] = {{&S63962[141], 9}, {NULL}};
const Segment SGI265409[] = {{&S63963[13], 6}, {NULL}};
const Segment SGI265410[] = {{&S63963[113], 5}, {NULL}};
const Segment SGI265411[] = {{&S63963[38], 3}, {NULL}};
const Segment SGI265412[] = {{&S63963[76], 8}, {NULL}};
const Segment SGI265413[] = {{&S63963[126], 5}, {NULL}};
const Segment SGI265414[] = {{&S63963[139], 9}, {NULL}};
const Segment SGI265415[] = {{&S63970[29], 6}, {NULL}};
const Segment SGI265416[] = {{&S63970[47], 3}, {NULL}};
const Segment SGI265417[] = {{&S63971[30], 6}, {NULL}};
const Segment SGI265418[] = {{&S63971[48], 5}, {NULL}};
const Segment SGI265419[] = {{&S63972[29], 6}, {NULL}};
const Segment SGI265420[] = {{&S63972[47], 5}, {NULL}};
const Segment SGI265421[] = {{&S64194[28], 6}, {NULL}};
const Segment SGI265422[] = {{&S64194[46], 5}, {NULL}};
const Segment SGI265423[] = {{&S64195[28], 6}, {NULL}};
const Segment SGI265424[] = {{&S64195[46], 5}, {NULL}};
const Segment SGI265425[] = {{&S64196[28], 6}, {NULL}};
const Segment SGI265426[] = {{&S64196[46], 5}, {NULL}};
const Segment SGI265427[] = {{&S64197[28], 6}, {NULL}};
const Segment SGI265428[] = {{&S64197[46], 6}, {NULL}};
const Segment SGI265429[] = {{&S64198[28], 6}, {NULL}};
const Segment SGI265430[] = {{&S64198[46], 5}, {NULL}};
const Segment SGI265431[] = {{&S64199[28], 6}, {NULL}};
const Segment SGI265432[] = {{&S64199[46], 5}, {NULL}};
const Segment SGI265433[] = {{&S64200[28], 6}, {NULL}};
const Segment SGI265434[] = {{&S64200[46], 5}, {NULL}};
const Segment SGI265435[] = {{&S64240[13], 6}, {NULL}};
const Segment SGI265436[] = {{&S64240[35], 5}, {NULL}};
const Segment SGI265437[] = {{&S64240[68], 6}, {NULL}};
const Segment SGI265438[] = {{&S64240[114], 5}, {NULL}};
const Segment SGI265439[] = {{&S64240[127], 9}, {NULL}};
const Segment SGI265440[] = {{&S64241[13], 6}, {NULL}};
const Segment SGI265441[] = {{&S64241[35], 3}, {NULL}};
const Segment SGI265442[] = {{&S64241[66], 6}, {NULL}};
const Segment SGI265443[] = {{&S64241[112], 5}, {NULL}};
const Segment SGI265444[] = {{&S64241[125], 9}, {NULL}};
const Segment SGI265445[] = {{&S64242[13], 4}, {NULL}};
const Segment SGI265446[] = {{&S64242[26], 3}, {NULL}};
const Segment SGI265447[] = {{&S64242[57], 6}, {NULL}};
const Segment SGI265448[] = {{&S64242[106], 6}, {NULL}};
const Segment SGI265449[] = {{&S64242[120], 9}, {NULL}};
const Segment SGI265450[] = {{&S64242[84], 14}, {NULL}};
const Segment SGI265451[] = {{&S64243[13], 4}, {NULL}};
const Segment SGI265452[] = {{&S64243[26], 5}, {NULL}};
const Segment SGI265453[] = {{&S64243[59], 6}, {NULL}};
const Segment SGI265454[] = {{&S64243[108], 6}, {NULL}};
const Segment SGI265455[] = {{&S64243[122], 9}, {NULL}};
const Segment SGI265456[] = {{&S64243[86], 14}, {NULL}};
const Segment SGI265457[] = {{&S64244[13], 4}, {NULL}};
const Segment SGI265458[] = {{&S64244[26], 5}, {NULL}};
const Segment SGI265459[] = {{&S64244[59], 6}, {NULL}};
const Segment SGI265460[] = {{&S64244[108], 6}, {NULL}};
const Segment SGI265461[] = {{&S64244[122], 9}, {NULL}};
const Segment SGI265462[] = {{&S64244[86], 14}, {NULL}};
const Segment SGI265463[] = {{&S50626[10], 4}, {NULL}};
const Segment SGI265464[] = {{&S50626[15], 5}, {NULL}};
const Segment SGI265465[] = {{&S64527[13], 4}, {NULL}};
const Segment SGI265466[] = {{&S64527[109], 3}, {NULL}};
const Segment SGI265467[] = {{&S64527[36], 3}, {NULL}};
const Segment SGI265468[] = {{&S64527[74], 6}, {NULL}};
const Segment SGI265469[] = {{&S64527[120], 5}, {NULL}};
const Segment SGI265470[] = {{&S64527[133], 6}, {NULL}};
const Segment SGI265471[] = {{&S64528[13], 4}, {&S64528[13], 4}, {NULL}};
const Segment SGI265472[] = {{&S64528[203], 5}, {NULL}};
const Segment SGI265473[] = {{&S64528[26], 3}, {&S64528[127], 3}, {NULL}};
const Segment SGI265474[] = {{&S64528[57], 6}, {&S64528[165], 9}, {NULL}};
const Segment SGI265475[] = {{&S64528[91], 6}, {&S64528[216], 5}, {NULL}};
const Segment SGI265476[] = {{&S64528[165], 9}, {NULL}};
const Segment SGI265477[] = {{&S64529[13], 4}, {&S64529[13], 4}, {NULL}};
const Segment SGI265478[] = {{&S64529[205], 5}, {NULL}};
const Segment SGI265479[] = {{&S64529[26], 5}, {&S64529[129], 3}, {NULL}};
const Segment SGI265480[] = {{&S64529[59], 6}, {&S64529[167], 9}, {NULL}};
const Segment SGI265481[] = {{&S64529[93], 6}, {&S64529[218], 5}, {NULL}};
const Segment SGI265482[] = {{&S64529[167], 9}, {NULL}};
const Segment SGI265483[] = {{&S64530[13], 4}, {NULL}};
const Segment SGI265484[] = {{&S64530[111], 3}, {NULL}};
const Segment SGI265485[] = {{&S64530[36], 5}, {NULL}};
const Segment SGI265486[] = {{&S64530[76], 6}, {NULL}};
const Segment SGI265487[] = {{&S64530[122], 5}, {NULL}};
const Segment SGI265488[] = {{&S64530[135], 6}, {NULL}};
const Segment SGI265489[] = {{&S64531[13], 4}, {&S64531[13], 4}, {NULL}};
const Segment SGI265490[] = {{&S64531[205], 5}, {NULL}};
const Segment SGI265491[] = {{&S64531[26], 5}, {&S64531[129], 3}, {NULL}};
const Segment SGI265492[] = {{&S64531[59], 6}, {&S64531[167], 9}, {NULL}};
const Segment SGI265493[] = {{&S64531[93], 6}, {&S64531[218], 5}, {NULL}};
const Segment SGI265494[] = {{&S64531[167], 9}, {NULL}};
const Segment SGI265495[] = {{&S64532[13], 4}, {NULL}};
const Segment SGI265496[] = {{&S64532[111], 3}, {NULL}};
const Segment SGI265497[] = {{&S64532[36], 5}, {NULL}};
const Segment SGI265498[] = {{&S64532[76], 6}, {NULL}};
const Segment SGI265499[] = {{&S64532[122], 5}, {NULL}};
const Segment SGI265500[] = {{&S64532[135], 6}, {NULL}};
const Segment SGI265501[] = {{&S50885[10], 4}, {NULL}};
const Segment SGI265502[] = {{&S50885[15], 5}, {NULL}};
const Segment SGI265503[] = {{&S64628[13], 4}, {NULL}};
const Segment SGI265504[] = {{&S64628[26], 3}, {NULL}};
const Segment SGI265505[] = {{&S64628[57], 6}, {NULL}};
const Segment SGI265506[] = {{&S64628[109], 8}, {NULL}};
const Segment SGI265507[] = {{&S64628[125], 9}, {NULL}};
const Segment SGI265508[] = {{&S64629[13], 4}, {NULL}};
const Segment SGI265509[] = {{&S64629[26], 3}, {NULL}};
const Segment SGI265510[] = {{&S64629[57], 8}, {NULL}};
const Segment SGI265511[] = {{&S64629[111], 6}, {NULL}};
const Segment SGI265512[] = {{&S64629[125], 9}, {NULL}};
const Segment SGI265513[] = {{&S64630[13], 4}, {NULL}};
const Segment SGI265514[] = {{&S64630[26], 3}, {NULL}};
const Segment SGI265515[] = {{&S64630[57], 8}, {NULL}};
const Segment SGI265516[] = {{&S64630[111], 8}, {NULL}};
const Segment SGI265517[] = {{&S64630[127], 9}, {NULL}};
const Segment SGI265518[] = {{&S64631[13], 4}, {NULL}};
const Segment SGI265519[] = {{&S64631[26], 3}, {NULL}};
const Segment SGI265520[] = {{&S64631[57], 8}, {NULL}};
const Segment SGI265521[] = {{&S64631[111], 8}, {NULL}};
const Segment SGI265522[] = {{&S64631[127], 9}, {NULL}};
const Segment SGI265523[] = {{&S64632[13], 4}, {NULL}};
const Segment SGI265524[] = {{&S64632[26], 3}, {NULL}};
const Segment SGI265525[] = {{&S64632[57], 8}, {NULL}};
const Segment SGI265526[] = {{&S64632[111], 8}, {NULL}};
const Segment SGI265527[] = {{&S64632[127], 9}, {NULL}};
const Segment SGI265528[] = {{&S64633[13], 4}, {NULL}};
const Segment SGI265529[] = {{&S64633[26], 5}, {NULL}};
const Segment SGI265530[] = {{&S64633[59], 8}, {NULL}};
const Segment SGI265531[] = {{&S64633[113], 6}, {NULL}};
const Segment SGI265532[] = {{&S64633[127], 9}, {NULL}};
const Segment SGI265533[] = {{&S64634[13], 4}, {NULL}};
const Segment SGI265534[] = {{&S64634[36], 5}, {NULL}};
const Segment SGI265535[] = {{&S64634[76], 8}, {NULL}};
const Segment SGI265536[] = {{&S64634[112], 4}, {NULL}};
const Segment SGI265537[] = {{&S64635[13], 4}, {NULL}};
const Segment SGI265538[] = {{&S64635[36], 5}, {NULL}};
const Segment SGI265539[] = {{&S64635[76], 8}, {NULL}};
const Segment SGI265540[] = {{&S64636[13], 4}, {NULL}};
const Segment SGI265541[] = {{&S64636[36], 3}, {NULL}};
const Segment SGI265542[] = {{&S64636[74], 8}, {NULL}};
const Segment SGI265543[] = {{&S64637[13], 4}, {NULL}};
const Segment SGI265544[] = {{&S64637[36], 5}, {NULL}};
const Segment SGI265545[] = {{&S64637[76], 8}, {NULL}};
const Segment SGI265546[] = {{&S64637[112], 5}, {NULL}};
const Segment SGI265547[] = {{&S64638[13], 4}, {NULL}};
const Segment SGI265548[] = {{&S64638[36], 3}, {NULL}};
const Segment SGI265549[] = {{&S64638[74], 8}, {NULL}};
const Segment SGI265550[] = {{&S64638[110], 5}, {NULL}};
const Segment SGI265551[] = {{&S58125[61], 9}, {NULL}};
const Segment SGI265552[] = {{&S58125[0], 70}, {NULL}};
const Segment SGI265553[] = {{&S58126[74], 16}, {NULL}};
const Segment SGI265554[] = {{&S58126[0], 129}, {NULL}};
const Segment SGI265555[] = {{&S58127[74], 16}, {NULL}};
const Segment SGI265556[] = {{&S58127[0], 129}, {NULL}};
const Segment SGI265557[] = {{&S58128[74], 16}, {NULL}};
const Segment SGI265558[] = {{&S58128[0], 129}, {NULL}};
const Segment SGI265559[] = {{&S58129[74], 16}, {NULL}};
const Segment SGI265560[] = {{&S58129[0], 129}, {NULL}};
const Segment SGI265561[] = {{&S58130[74], 15}, {NULL}};
const Segment SGI265562[] = {{&S58130[0], 128}, {NULL}};
const Segment SGI265563[] = {{&S51138[22], 12}, {NULL}};
const Segment SGI265564[] = {{&S51202[10], 5}, {NULL}};
const Segment SGI265565[] = {{&S51202[16], 5}, {NULL}};
const Segment SGI265566[] = {{&S64942[13], 4}, {NULL}};
const Segment SGI265567[] = {{&S64942[26], 5}, {NULL}};
const Segment SGI265568[] = {{&S64942[126], 5}, {NULL}};
const Segment SGI265569[] = {{&S64942[59], 6}, {NULL}};
const Segment SGI265570[] = {{&S64942[93], 6}, {NULL}};
const Segment SGI265571[] = {{&S64942[107], 9}, {NULL}};
const Segment SGI265572[] = {{&S64943[13], 4}, {NULL}};
const Segment SGI265573[] = {{&S64943[26], 5}, {NULL}};
const Segment SGI265574[] = {{&S64943[126], 5}, {NULL}};
const Segment SGI265575[] = {{&S64943[59], 6}, {NULL}};
const Segment SGI265576[] = {{&S64943[93], 6}, {NULL}};
const Segment SGI265577[] = {{&S64943[107], 9}, {NULL}};
const Segment SGI265578[] = {{&S64944[13], 4}, {NULL}};
const Segment SGI265579[] = {{&S64944[26], 5}, {NULL}};
const Segment SGI265580[] = {{&S64944[126], 5}, {NULL}};
const Segment SGI265581[] = {{&S64944[59], 6}, {NULL}};
const Segment SGI265582[] = {{&S64944[93], 6}, {NULL}};
const Segment SGI265583[] = {{&S64944[107], 9}, {NULL}};
const Segment SGI265584[] = {{&S64945[13], 4}, {NULL}};
const Segment SGI265585[] = {{&S64945[26], 5}, {NULL}};
const Segment SGI265586[] = {{&S64945[126], 5}, {NULL}};
const Segment SGI265587[] = {{&S64945[59], 6}, {NULL}};
const Segment SGI265588[] = {{&S64945[93], 6}, {NULL}};
const Segment SGI265589[] = {{&S64945[107], 9}, {NULL}};
const Segment SGI265590[] = {{&S64946[13], 4}, {NULL}};
const Segment SGI265591[] = {{&S64946[26], 5}, {NULL}};
const Segment SGI265592[] = {{&S64946[126], 5}, {NULL}};
const Segment SGI265593[] = {{&S64946[59], 6}, {NULL}};
const Segment SGI265594[] = {{&S64946[93], 6}, {NULL}};
const Segment SGI265595[] = {{&S64946[107], 9}, {NULL}};
const Segment SGI265596[] = {{&S64947[13], 4}, {NULL}};
const Segment SGI265597[] = {{&S64947[26], 3}, {NULL}};
const Segment SGI265598[] = {{&S64947[124], 5}, {NULL}};
const Segment SGI265599[] = {{&S64947[57], 6}, {NULL}};
const Segment SGI265600[] = {{&S64947[91], 6}, {NULL}};
const Segment SGI265601[] = {{&S64947[105], 9}, {NULL}};
const Segment SGI265602[] = {{&S64948[13], 4}, {NULL}};
const Segment SGI265603[] = {{&S64948[26], 5}, {NULL}};
const Segment SGI265604[] = {{&S64948[126], 5}, {NULL}};
const Segment SGI265605[] = {{&S64948[59], 6}, {NULL}};
const Segment SGI265606[] = {{&S64948[93], 6}, {NULL}};
const Segment SGI265607[] = {{&S64948[107], 9}, {NULL}};
const Segment SGI265608[] = {{&S64949[13], 4}, {NULL}};
const Segment SGI265609[] = {{&S64949[26], 5}, {NULL}};
const Segment SGI265610[] = {{&S64949[126], 5}, {NULL}};
const Segment SGI265611[] = {{&S64949[59], 6}, {NULL}};
const Segment SGI265612[] = {{&S64949[93], 6}, {NULL}};
const Segment SGI265613[] = {{&S64949[107], 9}, {NULL}};
const Segment SGI265614[] = {{&S65029[17], 4}, {NULL}};
const Segment SGI265615[] = {{&S65029[33], 3}, {NULL}};
const Segment SGI265616[] = {{&S65030[20], 4}, {NULL}};
const Segment SGI265617[] = {{&S65030[36], 5}, {NULL}};
const Segment SGI265618[] = {{&S51542[10], 4}, {NULL}};
const Segment SGI265619[] = {{&S51542[15], 5}, {NULL}};
const Segment SGI265620[] = {{&S65248[13], 4}, {NULL}};
const Segment SGI265621[] = {{&S65248[101], 5}, {&S65248[129], 3}, {NULL}};
const Segment SGI265622[] = {{&S65248[33], 5}, {NULL}};
const Segment SGI265623[] = {{&S65248[66], 6}, {NULL}};
const Segment SGI265624[] = {{&S65248[114], 6}, {NULL}};
const Segment SGI265625[] = {{&S65248[140], 7}, {&S65248[155], 9}, {NULL}};
const Segment SGI265626[] = {{&S65249[13], 4}, {NULL}};
const Segment SGI265627[] = {{&S65249[99], 5}, {&S65249[127], 3}, {NULL}};
const Segment SGI265628[] = {{&S65249[33], 3}, {NULL}};
const Segment SGI265629[] = {{&S65249[64], 6}, {NULL}};
const Segment SGI265630[] = {{&S65249[112], 6}, {NULL}};
const Segment SGI265631[] = {{&S65249[138], 7}, {&S65249[153], 9}, {NULL}};
const Segment SGI265632[] = {{&S65250[13], 4}, {NULL}};
const Segment SGI265633[] = {{&S65250[101], 5}, {&S65250[129], 3}, {NULL}};
const Segment SGI265634[] = {{&S65250[33], 5}, {NULL}};
const Segment SGI265635[] = {{&S65250[66], 6}, {NULL}};
const Segment SGI265636[] = {{&S65250[114], 6}, {NULL}};
const Segment SGI265637[] = {{&S65250[140], 7}, {&S65250[155], 9}, {NULL}};
const Segment SGI265638[] = {{&S65251[13], 4}, {NULL}};
const Segment SGI265639[] = {{&S65251[113], 5}, {NULL}};
const Segment SGI265640[] = {{&S65251[36], 5}, {NULL}};
const Segment SGI265641[] = {{&S65251[76], 8}, {NULL}};
const Segment SGI265642[] = {{&S65251[126], 5}, {&S65251[38], 1}, {NULL}};
const Segment SGI265643[] = {{&S65251[159], 9}, {NULL}};
const Segment SGI265644[] = {{&S65252[13], 4}, {NULL}};
const Segment SGI265645[] = {{&S65252[113], 5}, {NULL}};
const Segment SGI265646[] = {{&S65252[36], 5}, {NULL}};
const Segment SGI265647[] = {{&S65252[76], 8}, {NULL}};
const Segment SGI265648[] = {{&S65252[126], 6}, {NULL}};
const Segment SGI265649[] = {{&S65252[140], 9}, {NULL}};
const Segment SGI265650[] = {{&S65268[13], 6}, {NULL}};
const Segment SGI265651[] = {{&S65268[92], 3}, {NULL}};
const Segment SGI265652[] = {{&S65268[59], 3}, {NULL}};
const Segment SGI265653[] = {{&S65268[103], 6}, {NULL}};
const Segment SGI265654[] = {{&S65268[117], 5}, {NULL}};
const Segment SGI265655[] = {{&S65272[13], 6}, {NULL}};
const Segment SGI265656[] = {{&S65272[92], 3}, {NULL}};
const Segment SGI265657[] = {{&S65272[59], 3}, {NULL}};
const Segment SGI265658[] = {{&S65272[103], 6}, {NULL}};
const Segment SGI265659[] = {{&S65272[117], 5}, {NULL}};
const Segment SGI265660[] = {{&S65315[17], 4}, {NULL}};
const Segment SGI265661[] = {{&S65315[36], 5}, {NULL}};
const Segment SGI265662[] = {{&S65315[66], 3}, {NULL}};
const Segment SGI265663[] = {{&S65316[17], 4}, {NULL}};
const Segment SGI265664[] = {{&S65316[36], 3}, {NULL}};
const Segment SGI265665[] = {{&S65316[64], 3}, {NULL}};
const Segment SGI265666[] = {{&S65333[13], 6}, {NULL}};
const Segment SGI265667[] = {{&S65333[35], 3}, {NULL}};
const Segment SGI265668[] = {{&S65333[73], 8}, {NULL}};
const Segment SGI265669[] = {{&S65333[128], 6}, {NULL}};
const Segment SGI265670[] = {{&S65333[142], 9}, {NULL}};
const Segment SGI265671[] = {{&S65334[13], 6}, {NULL}};
const Segment SGI265672[] = {{&S65334[35], 5}, {NULL}};
const Segment SGI265673[] = {{&S65334[75], 8}, {NULL}};
const Segment SGI265674[] = {{&S65334[130], 6}, {NULL}};
const Segment SGI265675[] = {{&S65334[144], 9}, {NULL}};
const Segment SGI265676[] = {{&S65361[27], 6}, {NULL}};
const Segment SGI265677[] = {{&S65361[52], 3}, {NULL}};
const Segment SGI265678[] = {{&S65361[83], 6}, {NULL}};
const Segment SGI265679[] = {{&S65361[117], 6}, {NULL}};
const Segment SGI265680[] = {{&S65361[131], 9}, {NULL}};
const Segment SGI265681[] = {{&S65361[0], 14}, {NULL}};
const Segment SGI265682[] = {{&S65362[28], 6}, {NULL}};
const Segment SGI265683[] = {{&S65362[53], 3}, {NULL}};
const Segment SGI265684[] = {{&S65362[84], 6}, {NULL}};
const Segment SGI265685[] = {{&S65362[118], 6}, {NULL}};
const Segment SGI265686[] = {{&S65362[132], 9}, {NULL}};
const Segment SGI265687[] = {{&S65362[0], 15}, {NULL}};
const Segment SGI265688[] = {{&S65363[28], 6}, {NULL}};
const Segment SGI265689[] = {{&S65363[52], 3}, {NULL}};
const Segment SGI265690[] = {{&S65363[83], 6}, {NULL}};
const Segment SGI265691[] = {{&S65363[117], 6}, {NULL}};
const Segment SGI265692[] = {{&S65363[131], 9}, {NULL}};
const Segment SGI265693[] = {{&S65363[0], 15}, {NULL}};
const Segment SGI265694[] = {{&S65364[28], 6}, {NULL}};
const Segment SGI265695[] = {{&S65364[53], 5}, {NULL}};
const Segment SGI265696[] = {{&S65364[86], 6}, {NULL}};
const Segment SGI265697[] = {{&S65364[120], 6}, {NULL}};
const Segment SGI265698[] = {{&S65364[134], 9}, {NULL}};
const Segment SGI265699[] = {{&S65364[0], 15}, {NULL}};
const Segment SGI265700[] = {{&S65365[28], 6}, {NULL}};
const Segment SGI265701[] = {{&S65365[52], 5}, {NULL}};
const Segment SGI265702[] = {{&S65365[85], 6}, {NULL}};
const Segment SGI265703[] = {{&S65365[119], 6}, {NULL}};
const Segment SGI265704[] = {{&S65365[133], 9}, {NULL}};
const Segment SGI265705[] = {{&S65365[0], 15}, {NULL}};
const Segment SGI265706[] = {{&S65366[27], 6}, {NULL}};
const Segment SGI265707[] = {{&S65366[52], 5}, {NULL}};
const Segment SGI265708[] = {{&S65366[85], 6}, {NULL}};
const Segment SGI265709[] = {{&S65366[119], 6}, {NULL}};
const Segment SGI265710[] = {{&S65366[133], 9}, {NULL}};
const Segment SGI265711[] = {{&S65366[0], 14}, {NULL}};
const Segment SGI265712[] = {{&S51974[20], 27}, {NULL}};
const Segment SGI265713[] = {{&S51975[20], 27}, {NULL}};
const Segment SGI265714[] = {{&S51976[20], 27}, {NULL}};
const Segment SGI265715[] = {{&S51977[20], 27}, {NULL}};
const Segment SGI265716[] = {{&S51978[20], 27}, {NULL}};
const Segment SGI265717[] = {{&S51979[20], 27}, {NULL}};
const Segment SGI265718[] = {{&S51980[20], 27}, {NULL}};
const Segment SGI265719[] = {{&S51981[20], 27}, {NULL}};
const Segment SGI265720[] = {{&S51982[20], 27}, {NULL}};
const Segment SGI265721[] = {{&S51983[20], 27}, {NULL}};
const Segment SGI265722[] = {{&S51984[20], 27}, {NULL}};
const Segment SGI265723[] = {{&S51985[20], 27}, {NULL}};
const Segment SGI265724[] = {{&S51986[20], 27}, {NULL}};
const Segment SGI265725[] = {{&S51987[20], 26}, {NULL}};
const Segment SGI265726[] = {{&S51988[20], 27}, {NULL}};
const Segment SGI265727[] = {{&S51989[20], 27}, {NULL}};
const Segment SGI265728[] = {{&S51990[20], 27}, {NULL}};
const Segment SGI265729[] = {{&S51991[20], 26}, {NULL}};
const Segment SGI265730[] = {{&S52154[10], 4}, {NULL}};
const Segment SGI265731[] = {{&S52154[15], 5}, {NULL}};
const Segment SGI265732[] = {{&S65950[13], 6}, {NULL}};
const Segment SGI265733[] = {{&S65950[115], 5}, {NULL}};
const Segment SGI265734[] = {{&S65950[38], 5}, {NULL}};
const Segment SGI265735[] = {{&S65950[78], 8}, {NULL}};
const Segment SGI265736[] = {{&S65950[128], 3}, {NULL}};
const Segment SGI265737[] = {{&S65950[139], 9}, {NULL}};
const Segment SGI265738[] = {{&S65950[149], 13}, {NULL}};
const Segment SGI265739[] = {{&S65951[13], 6}, {NULL}};
const Segment SGI265740[] = {{&S65951[35], 5}, {NULL}};
const Segment SGI265741[] = {{&S65951[75], 8}, {NULL}};
const Segment SGI265742[] = {{&S65951[111], 3}, {NULL}};
const Segment SGI265743[] = {{&S65951[115], 13}, {NULL}};
const Segment SGI265744[] = {{&S65952[13], 6}, {NULL}};
const Segment SGI265745[] = {{&S65952[35], 5}, {NULL}};
const Segment SGI265746[] = {{&S65952[75], 8}, {NULL}};
const Segment SGI265747[] = {{&S65952[111], 3}, {NULL}};
const Segment SGI265748[] = {{&S65952[115], 13}, {NULL}};
const Segment SGI265749[] = {{&S65985[29], 4}, {NULL}};
const Segment SGI265750[] = {{&S65985[45], 3}, {NULL}};
const Segment SGI265751[] = {{&S65986[29], 4}, {NULL}};
const Segment SGI265752[] = {{&S65986[45], 5}, {NULL}};
const Segment SGI265753[] = {{&S66012[28], 4}, {NULL}};
const Segment SGI265754[] = {{&S66012[44], 3}, {NULL}};
const Segment SGI265755[] = {{&S66013[28], 4}, {NULL}};
const Segment SGI265756[] = {{&S66013[44], 5}, {NULL}};
const Segment SGI265757[] = {{&S66023[13], 6}, {NULL}};
const Segment SGI265758[] = {{&S66023[35], 5}, {NULL}};
const Segment SGI265759[] = {{&S66023[68], 6}, {NULL}};
const Segment SGI265760[] = {{&S66023[121], 5}, {NULL}};
const Segment SGI265761[] = {{&S66023[134], 9}, {NULL}};
const Segment SGI265762[] = {{&S66024[13], 6}, {NULL}};
const Segment SGI265763[] = {{&S66024[35], 5}, {NULL}};
const Segment SGI265764[] = {{&S66024[68], 6}, {NULL}};
const Segment SGI265765[] = {{&S66024[121], 5}, {NULL}};
const Segment SGI265766[] = {{&S66024[134], 9}, {NULL}};
const Segment SGI265767[] = {{&S66025[13], 6}, {NULL}};
const Segment SGI265768[] = {{&S66025[35], 3}, {NULL}};
const Segment SGI265769[] = {{&S66025[66], 6}, {NULL}};
const Segment SGI265770[] = {{&S66025[119], 5}, {NULL}};
const Segment SGI265771[] = {{&S66025[132], 9}, {NULL}};
const Segment SGI265772[] = {{&S66326[13], 4}, {NULL}};
const Segment SGI265773[] = {{&S66326[26], 5}, {NULL}};
const Segment SGI265774[] = {{&S66326[59], 6}, {NULL}};
const Segment SGI265775[] = {{&S66326[105], 5}, {NULL}};
const Segment SGI265776[] = {{&S66326[118], 9}, {NULL}};
const Segment SGI265777[] = {{&S66327[13], 4}, {NULL}};
const Segment SGI265778[] = {{&S66327[26], 3}, {NULL}};
const Segment SGI265779[] = {{&S66327[57], 6}, {NULL}};
const Segment SGI265780[] = {{&S66327[103], 5}, {NULL}};
const Segment SGI265781[] = {{&S66327[116], 9}, {NULL}};
const Segment SGI265782[] = {{&S66405[13], 4}, {NULL}};
const Segment SGI265783[] = {{&S66405[107], 5}, {NULL}};
const Segment SGI265784[] = {{&S66405[29], 5}, {NULL}};
const Segment SGI265785[] = {{&S66405[69], 9}, {NULL}};
const Segment SGI265786[] = {{&S66405[120], 6}, {NULL}};
const Segment SGI265787[] = {{&S66406[13], 4}, {NULL}};
const Segment SGI265788[] = {{&S66406[99], 5}, {NULL}};
const Segment SGI265789[] = {{&S66406[29], 5}, {NULL}};
const Segment SGI265790[] = {{&S66406[62], 8}, {NULL}};
const Segment SGI265791[] = {{&S66406[112], 5}, {NULL}};
const Segment SGI265792[] = {{&S66406[125], 9}, {NULL}};
const Segment SGI265793[] = {{&S66407[13], 4}, {NULL}};
const Segment SGI265794[] = {{&S66407[106], 5}, {NULL}};
const Segment SGI265795[] = {{&S66407[29], 5}, {NULL}};
const Segment SGI265796[] = {{&S66407[69], 8}, {NULL}};
const Segment SGI265797[] = {{&S66407[119], 5}, {NULL}};
const Segment SGI265798[] = {{&S66407[132], 9}, {NULL}};
const Segment SGI265799[] = {{&S66408[13], 4}, {&S66408[13], 4}, {NULL}};
const Segment SGI265800[] = {{&S66408[216], 5}, {NULL}};
const Segment SGI265801[] = {{&S66408[29], 5}, {&S66408[140], 3}, {NULL}};
const Segment SGI265802[] = {{&S66408[69], 8}, {&S66408[178], 9}, {NULL}};
const Segment SGI265803[] = {{&S66408[105], 5}, {&S66408[229], 5}, {NULL}};
const Segment SGI265804[] = {{&S66408[178], 9}, {NULL}};
const Segment SGI265805[] = {{&S66409[13], 4}, {NULL}};
const Segment SGI265806[] = {{&S66409[111], 3}, {NULL}};
const Segment SGI265807[] = {{&S66409[36], 5}, {NULL}};
const Segment SGI265808[] = {{&S66409[76], 6}, {NULL}};
const Segment SGI265809[] = {{&S66409[122], 5}, {NULL}};
const Segment SGI265810[] = {{&S66409[135], 6}, {NULL}};
const Segment SGI265811[] = {{&S66410[13], 4}, {NULL}};
const Segment SGI265812[] = {{&S66410[105], 5}, {NULL}};
const Segment SGI265813[] = {{&S66410[29], 3}, {NULL}};
const Segment SGI265814[] = {{&S66410[67], 9}, {NULL}};
const Segment SGI265815[] = {{&S66410[118], 5}, {NULL}};
const Segment SGI265816[] = {{&S66497[13], 6}, {NULL}};
const Segment SGI265817[] = {{&S66497[35], 3}, {NULL}};
const Segment SGI265818[] = {{&S66497[66], 6}, {NULL}};
const Segment SGI265819[] = {{&S66497[120], 6}, {NULL}};
const Segment SGI265820[] = {{&S66497[134], 7}, {NULL}};
const Segment SGI265821[] = {{&S66498[13], 6}, {NULL}};
const Segment SGI265822[] = {{&S66498[35], 5}, {NULL}};
const Segment SGI265823[] = {{&S66498[68], 6}, {NULL}};
const Segment SGI265824[] = {{&S66498[122], 6}, {NULL}};
const Segment SGI265825[] = {{&S66498[136], 7}, {NULL}};
const Segment SGI265826[] = {{&S66624[13], 6}, {NULL}};
const Segment SGI265827[] = {{&S66624[35], 3}, {NULL}};
const Segment SGI265828[] = {{&S66624[73], 6}, {NULL}};
const Segment SGI265829[] = {{&S66624[127], 6}, {NULL}};
const Segment SGI265830[] = {{&S66624[141], 7}, {NULL}};
const Segment SGI265831[] = {{&S66625[13], 6}, {NULL}};
const Segment SGI265832[] = {{&S66625[35], 5}, {NULL}};
const Segment SGI265833[] = {{&S66625[75], 6}, {NULL}};
const Segment SGI265834[] = {{&S66625[129], 6}, {NULL}};
const Segment SGI265835[] = {{&S66625[143], 7}, {NULL}};
const Segment SGI265836[] = {{&S66756[13], 4}, {NULL}};
const Segment SGI265837[] = {{&S66756[98], 5}, {NULL}};
const Segment SGI265838[] = {{&S66756[29], 3}, {NULL}};
const Segment SGI265839[] = {{&S66756[60], 9}, {NULL}};
const Segment SGI265840[] = {{&S66756[111], 5}, {NULL}};
const Segment SGI265841[] = {{&S66756[134], 13}, {NULL}};
const Segment SGI265842[] = {{&S66764[13], 6}, {NULL}};
const Segment SGI265843[] = {{&S66764[38], 5}, {NULL}};
const Segment SGI265844[] = {{&S66764[78], 8}, {NULL}};
const Segment SGI265845[] = {{&S66765[13], 6}, {NULL}};
const Segment SGI265846[] = {{&S66765[38], 3}, {NULL}};
const Segment SGI265847[] = {{&S66765[76], 8}, {NULL}};
const Segment SGI265848[] = {{&S66766[13], 6}, {NULL}};
const Segment SGI265849[] = {{&S66766[38], 3}, {NULL}};
const Segment SGI265850[] = {{&S66766[76], 8}, {NULL}};
const Segment SGI265851[] = {{&S66766[112], 4}, {NULL}};
const Segment SGI265852[] = {{&S66767[13], 6}, {NULL}};
const Segment SGI265853[] = {{&S66767[38], 5}, {NULL}};
const Segment SGI265854[] = {{&S66768[13], 6}, {NULL}};
const Segment SGI265855[] = {{&S66768[38], 5}, {NULL}};
const Segment SGI265856[] = {{&S66768[78], 8}, {NULL}};
const Segment SGI265857[] = {{&S66768[114], 4}, {NULL}};
const Segment SGI265858[] = {{&S66843[13], 6}, {NULL}};
const Segment SGI265859[] = {{&S66843[35], 5}, {NULL}};
const Segment SGI265860[] = {{&S66843[68], 6}, {NULL}};
const Segment SGI265861[] = {{&S66843[114], 5}, {NULL}};
const Segment SGI265862[] = {{&S66843[127], 9}, {NULL}};
const Segment SGI265863[] = {{&S66844[13], 6}, {NULL}};
const Segment SGI265864[] = {{&S66844[35], 3}, {NULL}};
const Segment SGI265865[] = {{&S66844[66], 6}, {NULL}};
const Segment SGI265866[] = {{&S66844[112], 5}, {NULL}};
const Segment SGI265867[] = {{&S66844[125], 9}, {NULL}};
const Segment SGI265868[] = {{&S66968[13], 6}, {NULL}};
const Segment SGI265869[] = {{&S66968[35], 3}, {NULL}};
const Segment SGI265870[] = {{&S66968[66], 6}, {NULL}};
const Segment SGI265871[] = {{&S66968[124], 5}, {NULL}};
const Segment SGI265872[] = {{&S66968[137], 9}, {NULL}};
const Segment SGI265873[] = {{&S66969[13], 6}, {NULL}};
const Segment SGI265874[] = {{&S66969[35], 5}, {NULL}};
const Segment SGI265875[] = {{&S66969[68], 6}, {NULL}};
const Segment SGI265876[] = {{&S66969[126], 5}, {NULL}};
const Segment SGI265877[] = {{&S66969[139], 9}, {NULL}};
const Segment SGI265878[] = {{&S66970[13], 6}, {NULL}};
const Segment SGI265879[] = {{&S66970[35], 5}, {NULL}};
const Segment SGI265880[] = {{&S66970[68], 6}, {NULL}};
const Segment SGI265881[] = {{&S66970[126], 5}, {NULL}};
const Segment SGI265882[] = {{&S66970[139], 9}, {NULL}};
const Segment SGI265883[] = {{&S66971[13], 6}, {NULL}};
const Segment SGI265884[] = {{&S66971[35], 5}, {NULL}};
const Segment SGI265885[] = {{&S66971[68], 6}, {NULL}};
const Segment SGI265886[] = {{&S66971[126], 5}, {NULL}};
const Segment SGI265887[] = {{&S66971[139], 9}, {NULL}};
const Segment SGI265888[] = {{&S66972[13], 6}, {NULL}};
const Segment SGI265889[] = {{&S66972[35], 5}, {NULL}};
const Segment SGI265890[] = {{&S66972[68], 6}, {NULL}};
const Segment SGI265891[] = {{&S66972[126], 5}, {NULL}};
const Segment SGI265892[] = {{&S66972[139], 9}, {NULL}};
const Segment SGI265893[] = {{&S67051[13], 6}, {NULL}};
const Segment SGI265894[] = {{&S67051[35], 3}, {NULL}};
const Segment SGI265895[] = {{&S67051[66], 6}, {NULL}};
const Segment SGI265896[] = {{&S67051[122], 5}, {NULL}};
const Segment SGI265897[] = {{&S67051[135], 9}, {NULL}};
const Segment SGI265898[] = {{&S67052[13], 6}, {NULL}};
const Segment SGI265899[] = {{&S67052[35], 5}, {NULL}};
const Segment SGI265900[] = {{&S67052[68], 6}, {NULL}};
const Segment SGI265901[] = {{&S67052[126], 5}, {NULL}};
const Segment SGI265902[] = {{&S67052[139], 9}, {NULL}};
const Segment SGI265903[] = {{&S67053[13], 6}, {NULL}};
const Segment SGI265904[] = {{&S67053[35], 5}, {NULL}};
const Segment SGI265905[] = {{&S67053[68], 6}, {NULL}};
const Segment SGI265906[] = {{&S67053[126], 5}, {NULL}};
const Segment SGI265907[] = {{&S67053[139], 9}, {NULL}};
const Segment SGI265908[] = {{&S67107[12], 4}, {&S67107[12], 4}, {NULL}};
const Segment SGI265909[] = {{&S67107[27], 1}, {NULL}};
const Segment SGI265910[] = {{&S67107[72], 6}, {NULL}};
const Segment SGI265911[] = {{&S67107[135], 9}, {NULL}};
const Segment SGI265912[] = {{&S67107[99], 13}, {NULL}};
const Segment SGI265913[] = {{&S53664[10], 4}, {NULL}};
const Segment SGI265914[] = {{&S53664[15], 5}, {NULL}};
const Segment SGI265915[] = {{&S67196[13], 4}, {&S67196[13], 4}, {NULL}};
const Segment SGI265916[] = {{&S67196[33], 5}, {NULL}};
const Segment SGI265917[] = {{&S67196[66], 6}, {NULL}};
const Segment SGI265918[] = {{&S67196[100], 6}, {NULL}};
const Segment SGI265919[] = {{&S67196[114], 9}, {NULL}};
const Segment SGI265920[] = {{&S67197[13], 4}, {&S67197[13], 4}, {NULL}};
const Segment SGI265921[] = {{&S67197[33], 3}, {NULL}};
const Segment SGI265922[] = {{&S67197[64], 6}, {NULL}};
const Segment SGI265923[] = {{&S67197[98], 6}, {NULL}};
const Segment SGI265924[] = {{&S67197[112], 9}, {NULL}};
const Segment SGI265925[] = {{&S67198[13], 4}, {&S67198[13], 4}, {NULL}};
const Segment SGI265926[] = {{&S67198[33], 5}, {NULL}};
const Segment SGI265927[] = {{&S67198[66], 6}, {NULL}};
const Segment SGI265928[] = {{&S67198[100], 6}, {NULL}};
const Segment SGI265929[] = {{&S67198[114], 9}, {NULL}};
const Segment SGI265930[] = {{&S67381[13], 6}, {NULL}};
const Segment SGI265931[] = {{&S67381[123], 2}, {NULL}};
const Segment SGI265932[] = {{&S67381[35], 3}, {NULL}};
const Segment SGI265933[] = {{&S67381[66], 6}, {NULL}};
const Segment SGI265934[] = {{&S67381[35], 1}, {NULL}};
const Segment SGI265935[] = {{&S67381[150], 7}, {NULL}};
const Segment SGI265936[] = {{&S67382[13], 6}, {NULL}};
const Segment SGI265937[] = {{&S67382[123], 2}, {NULL}};
const Segment SGI265938[] = {{&S67382[35], 3}, {NULL}};
const Segment SGI265939[] = {{&S67382[66], 6}, {NULL}};
const Segment SGI265940[] = {{&S67382[35], 1}, {NULL}};
const Segment SGI265941[] = {{&S67382[150], 7}, {NULL}};
const Segment SGI265942[] = {{&S67383[13], 6}, {NULL}};
const Segment SGI265943[] = {{&S67383[125], 2}, {NULL}};
const Segment SGI265944[] = {{&S67383[35], 3}, {NULL}};
const Segment SGI265945[] = {{&S67383[66], 8}, {NULL}};
const Segment SGI265946[] = {{&S67383[35], 1}, {NULL}};
const Segment SGI265947[] = {{&S67383[150], 7}, {NULL}};
const Segment SGI265948[] = {{&S67384[13], 6}, {NULL}};
const Segment SGI265949[] = {{&S67384[127], 2}, {NULL}};
const Segment SGI265950[] = {{&S67384[35], 5}, {NULL}};
const Segment SGI265951[] = {{&S67384[68], 8}, {NULL}};
const Segment SGI265952[] = {{&S67384[35], 1}, {NULL}};
const Segment SGI265953[] = {{&S67384[154], 7}, {NULL}};
const Segment SGI265954[] = {{&S67385[13], 6}, {NULL}};
const Segment SGI265955[] = {{&S67385[127], 2}, {NULL}};
const Segment SGI265956[] = {{&S67385[35], 5}, {NULL}};
const Segment SGI265957[] = {{&S67385[68], 8}, {NULL}};
const Segment SGI265958[] = {{&S67385[35], 1}, {NULL}};
const Segment SGI265959[] = {{&S67385[155], 7}, {NULL}};
const Segment SGI265960[] = {{&S67453[13], 4}, {NULL}};
const Segment SGI265961[] = {{&S67453[33], 5}, {NULL}};
const Segment SGI265962[] = {{&S67453[66], 6}, {NULL}};
const Segment SGI265963[] = {{&S67453[119], 5}, {NULL}};
const Segment SGI265964[] = {{&S67453[132], 9}, {NULL}};
const Segment SGI265965[] = {{&S67454[13], 4}, {NULL}};
const Segment SGI265966[] = {{&S67454[33], 3}, {NULL}};
const Segment SGI265967[] = {{&S67454[64], 6}, {NULL}};
const Segment SGI265968[] = {{&S67454[117], 5}, {NULL}};
const Segment SGI265969[] = {{&S67454[130], 9}, {NULL}};
const Segment SGI265970[] = {{&S60854[128], 13}, {NULL}};
const Segment SGI265971[] = {{&S60854[0], 141}, {NULL}};
const Segment SGI265972[] = {{&S67643[13], 4}, {NULL}};
const Segment SGI265973[] = {{&S67643[26], 5}, {NULL}};
const Segment SGI265974[] = {{&S67643[59], 6}, {NULL}};
const Segment SGI265975[] = {{&S67643[112], 5}, {NULL}};
const Segment SGI265976[] = {{&S67643[125], 9}, {NULL}};
const Segment SGI265977[] = {{&S67644[13], 4}, {NULL}};
const Segment SGI265978[] = {{&S67644[26], 3}, {NULL}};
const Segment SGI265979[] = {{&S67644[57], 6}, {NULL}};
const Segment SGI265980[] = {{&S67644[110], 5}, {NULL}};
const Segment SGI265981[] = {{&S67644[123], 9}, {NULL}};
const Segment SGI265982[] = {{&S53985[10], 4}, {NULL}};
const Segment SGI265983[] = {{&S53985[15], 5}, {NULL}};
const Segment SGI265984[] = {{&S61040[59], 8}, {NULL}};
const Segment SGI265985[] = {{&S61040[0], 105}, {NULL}};
const Segment SGI265986[] = {{&S61041[54], 8}, {NULL}};
const Segment SGI265987[] = {{&S61041[0], 62}, {NULL}};
const Segment SGI265988[] = {{&S67664[13], 6}, {NULL}};
const Segment SGI265989[] = {{&S67664[38], 5}, {NULL}};
const Segment SGI265990[] = {{&S67664[78], 6}, {NULL}};
const Segment SGI265991[] = {{&S67664[112], 4}, {NULL}};
const Segment SGI265992[] = {{&S67664[117], 7}, {NULL}};
const Segment SGI265993[] = {{&S67665[13], 6}, {NULL}};
const Segment SGI265994[] = {{&S67665[38], 5}, {NULL}};
const Segment SGI265995[] = {{&S67665[78], 6}, {NULL}};
const Segment SGI265996[] = {{&S67665[112], 4}, {NULL}};
const Segment SGI265997[] = {{&S67665[117], 7}, {NULL}};
const Segment SGI265998[] = {{&S67666[13], 6}, {NULL}};
const Segment SGI265999[] = {{&S67666[38], 3}, {NULL}};
const Segment SGI266000[] = {{&S67666[76], 6}, {NULL}};
const Segment SGI266001[] = {{&S67666[110], 5}, {NULL}};
const Segment SGI266002[] = {{&S67666[116], 7}, {NULL}};
const Segment SGI266003[] = {{&S54027[20], 26}, {NULL}};
const Segment SGI266004[] = {{&S54028[20], 27}, {NULL}};
const Segment SGI266005[] = {{&S54029[20], 27}, {NULL}};
const Segment SGI266006[] = {{&S54030[20], 27}, {NULL}};
const Segment SGI266007[] = {{&S54031[20], 27}, {NULL}};
const Segment SGI266008[] = {{&S54032[20], 27}, {NULL}};
const Segment SGI266009[] = {{&S54033[20], 27}, {NULL}};
const Segment SGI266010[] = {{&S54034[20], 27}, {NULL}};
const Segment SGI266011[] = {{&S54035[20], 27}, {NULL}};
const Segment SGI266012[] = {{&S54036[20], 27}, {NULL}};
const Segment SGI266013[] = {{&S54037[20], 26}, {NULL}};
const Segment SGI266014[] = {{&S54038[20], 27}, {NULL}};
const Segment SGI266015[] = {{&S54039[20], 27}, {NULL}};
const Segment SGI266016[] = {{&S54040[20], 27}, {NULL}};
const Segment SGI266017[] = {{&S54041[20], 27}, {NULL}};
const Segment SGI266018[] = {{&S54113[22], 12}, {NULL}};
const Segment SGI266019[] = {{&S54150[10], 4}, {NULL}};
const Segment SGI266020[] = {{&S54150[15], 5}, {NULL}};
const Segment SGI266021[] = {{&S54151[72], 4}, {NULL}};
const Segment SGI266022[] = {{&S54151[77], 5}, {NULL}};
const Segment SGI266023[] = {{&S54152[72], 4}, {NULL}};
const Segment SGI266024[] = {{&S54152[77], 5}, {NULL}};
const Segment SGI266025[] = {{&S54153[72], 4}, {NULL}};
const Segment SGI266026[] = {{&S54153[77], 5}, {NULL}};
const Segment SGI266027[] = {{&S54214[10], 4}, {NULL}};
const Segment SGI266028[] = {{&S54214[15], 5}, {NULL}};
const Segment SGI266029[] = {{&S67895[29], 4}, {NULL}};
const Segment SGI266030[] = {{&S67895[45], 3}, {NULL}};
const Segment SGI266031[] = {{&S67896[29], 4}, {NULL}};
const Segment SGI266032[] = {{&S67896[45], 5}, {NULL}};
const Segment SGI266033[] = {{&S67897[29], 4}, {NULL}};
const Segment SGI266034[] = {{&S67897[45], 5}, {NULL}};
const Segment SGI266035[] = {{&S68003[27], 6}, {NULL}};
const Segment SGI266036[] = {{&S68003[52], 5}, {NULL}};
const Segment SGI266037[] = {{&S68003[85], 6}, {NULL}};
const Segment SGI266038[] = {{&S68003[119], 6}, {NULL}};
const Segment SGI266039[] = {{&S68003[133], 9}, {NULL}};
const Segment SGI266040[] = {{&S68003[0], 14}, {NULL}};
const Segment SGI266041[] = {{&S68004[27], 6}, {NULL}};
const Segment SGI266042[] = {{&S68004[51], 5}, {NULL}};
const Segment SGI266043[] = {{&S68004[84], 6}, {NULL}};
const Segment SGI266044[] = {{&S68004[118], 6}, {NULL}};
const Segment SGI266045[] = {{&S68004[132], 9}, {NULL}};
const Segment SGI266046[] = {{&S68004[0], 14}, {NULL}};
const Segment SGI266047[] = {{&S68005[28], 6}, {NULL}};
const Segment SGI266048[] = {{&S68005[53], 5}, {NULL}};
const Segment SGI266049[] = {{&S68005[86], 6}, {NULL}};
const Segment SGI266050[] = {{&S68005[120], 6}, {NULL}};
const Segment SGI266051[] = {{&S68005[134], 9}, {NULL}};
const Segment SGI266052[] = {{&S68005[0], 15}, {NULL}};
const Segment SGI266053[] = {{&S68006[28], 6}, {NULL}};
const Segment SGI266054[] = {{&S68006[53], 3}, {NULL}};
const Segment SGI266055[] = {{&S68006[84], 6}, {NULL}};
const Segment SGI266056[] = {{&S68006[118], 6}, {NULL}};
const Segment SGI266057[] = {{&S68006[132], 9}, {NULL}};
const Segment SGI266058[] = {{&S68006[0], 15}, {NULL}};
const Segment SGI266059[] = {{&S68007[28], 6}, {NULL}};
const Segment SGI266060[] = {{&S68007[52], 5}, {NULL}};
const Segment SGI266061[] = {{&S68007[85], 6}, {NULL}};
const Segment SGI266062[] = {{&S68007[119], 6}, {NULL}};
const Segment SGI266063[] = {{&S68007[133], 9}, {NULL}};
const Segment SGI266064[] = {{&S68007[0], 15}, {NULL}};
const Segment SGI266065[] = {{&S68008[27], 6}, {NULL}};
const Segment SGI266066[] = {{&S68008[51], 3}, {NULL}};
const Segment SGI266067[] = {{&S68008[82], 6}, {NULL}};
const Segment SGI266068[] = {{&S68008[116], 6}, {NULL}};
const Segment SGI266069[] = {{&S68008[130], 9}, {NULL}};
const Segment SGI266070[] = {{&S68008[0], 14}, {NULL}};
const Segment SGI266071[] = {{&S68014[13], 4}, {NULL}};
const Segment SGI266072[] = {{&S68014[33], 5}, {NULL}};
const Segment SGI266073[] = {{&S68014[66], 6}, {NULL}};
const Segment SGI266074[] = {{&S68014[119], 5}, {NULL}};
const Segment SGI266075[] = {{&S68014[132], 9}, {NULL}};
const Segment SGI266076[] = {{&S68015[13], 4}, {NULL}};
const Segment SGI266077[] = {{&S68015[33], 3}, {NULL}};
const Segment SGI266078[] = {{&S68015[64], 6}, {NULL}};
const Segment SGI266079[] = {{&S68015[117], 5}, {NULL}};
const Segment SGI266080[] = {{&S68015[130], 9}, {NULL}};
const Segment SGI266081[] = {{&S68028[13], 6}, {NULL}};
const Segment SGI266082[] = {{&S68028[35], 3}, {NULL}};
const Segment SGI266083[] = {{&S68028[66], 6}, {NULL}};
const Segment SGI266084[] = {{&S68028[100], 6}, {NULL}};
const Segment SGI266085[] = {{&S68028[114], 9}, {NULL}};
const Segment SGI266086[] = {{&S68029[13], 6}, {NULL}};
const Segment SGI266087[] = {{&S68029[112], 5}, {NULL}};
const Segment SGI266088[] = {{&S68029[38], 3}, {NULL}};
const Segment SGI266089[] = {{&S68029[76], 7}, {NULL}};
const Segment SGI266090[] = {{&S68029[125], 6}, {NULL}};
const Segment SGI266091[] = {{&S68029[139], 9}, {NULL}};
const Segment SGI266092[] = {{&S68030[13], 6}, {NULL}};
const Segment SGI266093[] = {{&S68030[35], 3}, {NULL}};
const Segment SGI266094[] = {{&S68030[66], 6}, {NULL}};
const Segment SGI266095[] = {{&S68030[100], 6}, {NULL}};
const Segment SGI266096[] = {{&S68030[114], 9}, {&S68030[114], 9}, {NULL}};
const Segment SGI266097[] = {{&S68031[13], 6}, {NULL}};
const Segment SGI266098[] = {{&S68031[101], 5}, {NULL}};
const Segment SGI266099[] = {{&S68031[35], 3}, {NULL}};
const Segment SGI266100[] = {{&S68031[66], 6}, {NULL}};
const Segment SGI266101[] = {{&S68031[114], 6}, {NULL}};
const Segment SGI266102[] = {{&S68031[128], 9}, {NULL}};
const Segment SGI266103[] = {{&S68032[13], 6}, {NULL}};
const Segment SGI266104[] = {{&S68032[114], 5}, {NULL}};
const Segment SGI266105[] = {{&S68032[38], 5}, {NULL}};
const Segment SGI266106[] = {{&S68032[78], 7}, {NULL}};
const Segment SGI266107[] = {{&S68032[127], 6}, {NULL}};
const Segment SGI266108[] = {{&S68032[141], 9}, {NULL}};
const Segment SGI266109[] = {{&S68033[13], 6}, {NULL}};
const Segment SGI266110[] = {{&S68033[38], 3}, {NULL}};
const Segment SGI266111[] = {{&S68033[111], 5}, {NULL}};
const Segment SGI266112[] = {{&S68033[124], 9}, {NULL}};
const Segment SGI266113[] = {{&S68034[13], 6}, {NULL}};
const Segment SGI266114[] = {{&S68034[103], 5}, {NULL}};
const Segment SGI266115[] = {{&S68034[35], 5}, {NULL}};
const Segment SGI266116[] = {{&S68034[68], 6}, {NULL}};
const Segment SGI266117[] = {{&S68034[116], 6}, {NULL}};
const Segment SGI266118[] = {{&S68034[130], 9}, {NULL}};
const Segment SGI266119[] = {{&S68035[13], 6}, {NULL}};
const Segment SGI266120[] = {{&S68035[38], 5}, {NULL}};
const Segment SGI266121[] = {{&S68035[113], 5}, {NULL}};
const Segment SGI266122[] = {{&S68035[126], 9}, {NULL}};
const Segment SGI266123[] = {{&S68036[13], 6}, {NULL}};
const Segment SGI266124[] = {{&S68036[115], 5}, {NULL}};
const Segment SGI266125[] = {{&S68036[38], 5}, {NULL}};
const Segment SGI266126[] = {{&S68036[78], 8}, {NULL}};
const Segment SGI266127[] = {{&S68036[128], 5}, {&S68036[42], 1}, {NULL}};
const Segment SGI266128[] = {{&S68036[161], 9}, {NULL}};
const Segment SGI266129[] = {{&S68086[28], 4}, {NULL}};
const Segment SGI266130[] = {{&S68086[44], 5}, {NULL}};
const Segment SGI266131[] = {{&S68087[28], 4}, {NULL}};
const Segment SGI266132[] = {{&S68087[44], 3}, {NULL}};
const Segment SGI266133[] = {{&S68088[38], 4}, {NULL}};
const Segment SGI266134[] = {{&S68088[130], 5}, {NULL}};
const Segment SGI266135[] = {{&S68088[54], 3}, {NULL}};
const Segment SGI266136[] = {{&S68088[92], 9}, {NULL}};
const Segment SGI266137[] = {{&S68088[143], 5}, {NULL}};
const Segment SGI266138[] = {{&S68192[13], 4}, {NULL}};
const Segment SGI266139[] = {{&S68192[137], 5}, {NULL}};
const Segment SGI266140[] = {{&S68192[29], 3}, {NULL}};
const Segment SGI266141[] = {{&S68192[99], 9}, {NULL}};
const Segment SGI266142[] = {{&S68192[150], 5}, {NULL}};
const Segment SGI266143[] = {{&S54420[10], 4}, {NULL}};
const Segment SGI266144[] = {{&S54420[15], 5}, {NULL}};
const Segment SGI266145[] = {{&S68313[13], 6}, {NULL}};
const Segment SGI266146[] = {{&S68313[115], 5}, {NULL}};
const Segment SGI266147[] = {{&S68313[38], 5}, {NULL}};
const Segment SGI266148[] = {{&S68313[78], 8}, {NULL}};
const Segment SGI266149[] = {{&S68313[128], 3}, {NULL}};
const Segment SGI266150[] = {{&S68313[139], 9}, {NULL}};
const Segment SGI266151[] = {{&S68313[149], 13}, {NULL}};
const Segment SGI266152[] = {{&S68314[13], 6}, {NULL}};
const Segment SGI266153[] = {{&S68314[35], 5}, {NULL}};
const Segment SGI266154[] = {{&S68314[75], 8}, {NULL}};
const Segment SGI266155[] = {{&S68314[111], 3}, {NULL}};
const Segment SGI266156[] = {{&S68314[115], 13}, {NULL}};
const Segment SGI266157[] = {{&S68343[13], 4}, {NULL}};
const Segment SGI266158[] = {{&S68343[36], 3}, {NULL}};
const Segment SGI266159[] = {{&S68343[74], 5}, {NULL}};
const Segment SGI266160[] = {{&S68343[107], 5}, {NULL}};
const Segment SGI266161[] = {{&S68343[113], 7}, {NULL}};
const Segment SGI266162[] = {{&S68344[13], 4}, {NULL}};
const Segment SGI266163[] = {{&S68344[36], 3}, {NULL}};
const Segment SGI266164[] = {{&S68344[74], 5}, {NULL}};
const Segment SGI266165[] = {{&S68344[107], 5}, {NULL}};
const Segment SGI266166[] = {{&S68344[113], 7}, {NULL}};
const Segment SGI266167[] = {{&S68417[13], 6}, {NULL}};
const Segment SGI266168[] = {{&S68417[120], 5}, {NULL}};
const Segment SGI266169[] = {{&S68417[38], 3}, {NULL}};
const Segment SGI266170[] = {{&S68417[83], 8}, {NULL}};
const Segment SGI266171[] = {{&S68417[133], 4}, {NULL}};
const Segment SGI266172[] = {{&S68417[145], 6}, {NULL}};
const Segment SGI266173[] = {{&S68457[27], 6}, {NULL}};
const Segment SGI266174[] = {{&S68457[52], 5}, {NULL}};
const Segment SGI266175[] = {{&S68457[85], 6}, {NULL}};
const Segment SGI266176[] = {{&S68457[119], 6}, {NULL}};
const Segment SGI266177[] = {{&S68457[133], 9}, {NULL}};
const Segment SGI266178[] = {{&S68457[0], 14}, {NULL}};
const Segment SGI266179[] = {{&S68458[27], 6}, {NULL}};
const Segment SGI266180[] = {{&S68458[51], 5}, {NULL}};
const Segment SGI266181[] = {{&S68458[84], 6}, {NULL}};
const Segment SGI266182[] = {{&S68458[118], 6}, {NULL}};
const Segment SGI266183[] = {{&S68458[132], 9}, {NULL}};
const Segment SGI266184[] = {{&S68458[0], 14}, {NULL}};
const Segment SGI266185[] = {{&S68459[27], 6}, {NULL}};
const Segment SGI266186[] = {{&S68459[51], 5}, {NULL}};
const Segment SGI266187[] = {{&S68459[84], 6}, {NULL}};
const Segment SGI266188[] = {{&S68459[118], 6}, {NULL}};
const Segment SGI266189[] = {{&S68459[132], 9}, {NULL}};
const Segment SGI266190[] = {{&S68459[0], 14}, {NULL}};
const Segment SGI266191[] = {{&S68460[27], 6}, {NULL}};
const Segment SGI266192[] = {{&S68460[52], 3}, {NULL}};
const Segment SGI266193[] = {{&S68460[83], 6}, {NULL}};
const Segment SGI266194[] = {{&S68460[117], 6}, {NULL}};
const Segment SGI266195[] = {{&S68460[131], 9}, {NULL}};
const Segment SGI266196[] = {{&S68460[0], 14}, {NULL}};
const Segment SGI266197[] = {{&S68461[27], 6}, {NULL}};
const Segment SGI266198[] = {{&S68461[51], 3}, {NULL}};
const Segment SGI266199[] = {{&S68461[82], 6}, {NULL}};
const Segment SGI266200[] = {{&S68461[116], 6}, {NULL}};
const Segment SGI266201[] = {{&S68461[130], 9}, {NULL}};
const Segment SGI266202[] = {{&S68461[0], 14}, {NULL}};
const Segment SGI266203[] = {{&S68472[13], 6}, {NULL}};
const Segment SGI266204[] = {{&S68472[112], 5}, {NULL}};
const Segment SGI266205[] = {{&S68472[38], 5}, {NULL}};
const Segment SGI266206[] = {{&S68472[78], 5}, {NULL}};
const Segment SGI266207[] = {{&S68472[125], 5}, {NULL}};
const Segment SGI266208[] = {{&S68472[138], 9}, {NULL}};
const Segment SGI266209[] = {{&S68473[13], 6}, {NULL}};
const Segment SGI266210[] = {{&S68473[112], 5}, {NULL}};
const Segment SGI266211[] = {{&S68473[38], 5}, {NULL}};
const Segment SGI266212[] = {{&S68473[78], 5}, {NULL}};
const Segment SGI266213[] = {{&S68473[125], 5}, {NULL}};
const Segment SGI266214[] = {{&S68473[138], 9}, {NULL}};
const Segment SGI266215[] = {{&S68474[13], 6}, {NULL}};
const Segment SGI266216[] = {{&S68474[112], 5}, {NULL}};
const Segment SGI266217[] = {{&S68474[38], 5}, {NULL}};
const Segment SGI266218[] = {{&S68474[78], 5}, {NULL}};
const Segment SGI266219[] = {{&S68474[125], 5}, {NULL}};
const Segment SGI266220[] = {{&S68474[138], 9}, {NULL}};
const Segment SGI266221[] = {{&S68475[13], 6}, {NULL}};
const Segment SGI266222[] = {{&S68475[115], 5}, {NULL}};
const Segment SGI266223[] = {{&S68475[38], 5}, {NULL}};
const Segment SGI266224[] = {{&S68475[78], 8}, {NULL}};
const Segment SGI266225[] = {{&S68475[128], 5}, {NULL}};
const Segment SGI266226[] = {{&S68475[141], 9}, {NULL}};
const Segment SGI266227[] = {{&S68476[13], 6}, {NULL}};
const Segment SGI266228[] = {{&S68476[115], 5}, {NULL}};
const Segment SGI266229[] = {{&S68476[38], 5}, {NULL}};
const Segment SGI266230[] = {{&S68476[78], 8}, {NULL}};
const Segment SGI266231[] = {{&S68476[128], 5}, {NULL}};
const Segment SGI266232[] = {{&S68476[141], 9}, {NULL}};
const Segment SGI266233[] = {{&S68477[13], 6}, {NULL}};
const Segment SGI266234[] = {{&S68477[113], 3}, {NULL}};
const Segment SGI266235[] = {{&S68477[38], 5}, {NULL}};
const Segment SGI266236[] = {{&S68477[78], 6}, {NULL}};
const Segment SGI266237[] = {{&S68477[124], 5}, {NULL}};
const Segment SGI266238[] = {{&S68477[137], 6}, {NULL}};
const Segment SGI266239[] = {{&S68478[13], 6}, {NULL}};
const Segment SGI266240[] = {{&S68478[112], 5}, {NULL}};
const Segment SGI266241[] = {{&S68478[38], 5}, {NULL}};
const Segment SGI266242[] = {{&S68478[78], 5}, {NULL}};
const Segment SGI266243[] = {{&S68478[125], 5}, {NULL}};
const Segment SGI266244[] = {{&S68479[13], 6}, {NULL}};
const Segment SGI266245[] = {{&S68479[115], 5}, {NULL}};
const Segment SGI266246[] = {{&S68479[38], 5}, {NULL}};
const Segment SGI266247[] = {{&S68479[78], 8}, {NULL}};
const Segment SGI266248[] = {{&S68479[128], 5}, {NULL}};
const Segment SGI266249[] = {{&S68479[141], 9}, {NULL}};
const Segment SGI266250[] = {{&S68480[13], 6}, {NULL}};
const Segment SGI266251[] = {{&S68480[112], 5}, {NULL}};
const Segment SGI266252[] = {{&S68480[38], 5}, {NULL}};
const Segment SGI266253[] = {{&S68480[78], 5}, {NULL}};
const Segment SGI266254[] = {{&S68480[125], 6}, {NULL}};
const Segment SGI266255[] = {{&S68480[139], 9}, {NULL}};
const Segment SGI266256[] = {{&S68481[13], 6}, {NULL}};
const Segment SGI266257[] = {{&S68481[108], 3}, {NULL}};
const Segment SGI266258[] = {{&S68481[31], 5}, {NULL}};
const Segment SGI266259[] = {{&S68481[71], 8}, {NULL}};
const Segment SGI266260[] = {{&S68481[119], 5}, {NULL}};
const Segment SGI266261[] = {{&S68481[132], 6}, {NULL}};
const Segment SGI266262[] = {{&S68482[13], 6}, {NULL}};
const Segment SGI266263[] = {{&S68482[113], 3}, {NULL}};
const Segment SGI266264[] = {{&S68482[38], 5}, {NULL}};
const Segment SGI266265[] = {{&S68482[78], 6}, {NULL}};
const Segment SGI266266[] = {{&S68482[124], 5}, {NULL}};
const Segment SGI266267[] = {{&S68482[137], 6}, {NULL}};
const Segment SGI266268[] = {{&S68483[13], 6}, {NULL}};
const Segment SGI266269[] = {{&S68483[114], 3}, {NULL}};
const Segment SGI266270[] = {{&S68483[38], 6}, {NULL}};
const Segment SGI266271[] = {{&S68483[79], 6}, {NULL}};
const Segment SGI266272[] = {{&S68483[125], 5}, {NULL}};
const Segment SGI266273[] = {{&S68483[138], 6}, {NULL}};
const Segment SGI266274[] = {{&S68484[13], 6}, {NULL}};
const Segment SGI266275[] = {{&S68484[113], 3}, {NULL}};
const Segment SGI266276[] = {{&S68484[38], 5}, {NULL}};
const Segment SGI266277[] = {{&S68484[78], 6}, {NULL}};
const Segment SGI266278[] = {{&S68484[124], 5}, {NULL}};
const Segment SGI266279[] = {{&S68484[137], 6}, {NULL}};
const Segment SGI266280[] = {{&S68526[13], 4}, {NULL}};
const Segment SGI266281[] = {{&S68526[114], 2}, {NULL}};
const Segment SGI266282[] = {{&S68526[26], 3}, {NULL}};
const Segment SGI266283[] = {{&S68526[57], 6}, {NULL}};
const Segment SGI266284[] = {{&S68526[26], 1}, {NULL}};
const Segment SGI266285[] = {{&S68526[141], 7}, {NULL}};
const Segment SGI266286[] = {{&S68527[13], 4}, {NULL}};
const Segment SGI266287[] = {{&S68527[116], 2}, {NULL}};
const Segment SGI266288[] = {{&S68527[26], 3}, {NULL}};
const Segment SGI266289[] = {{&S68527[57], 8}, {NULL}};
const Segment SGI266290[] = {{&S68527[26], 1}, {NULL}};
const Segment SGI266291[] = {{&S68527[141], 7}, {NULL}};
const Segment SGI266292[] = {{&S68528[13], 4}, {NULL}};
const Segment SGI266293[] = {{&S68528[118], 2}, {NULL}};
const Segment SGI266294[] = {{&S68528[26], 5}, {NULL}};
const Segment SGI266295[] = {{&S68528[59], 8}, {NULL}};
const Segment SGI266296[] = {{&S68528[26], 1}, {NULL}};
const Segment SGI266297[] = {{&S68528[145], 7}, {NULL}};
const Segment SGI266298[] = {{&S68668[13], 6}, {&S68668[13], 6}, {NULL}};
const Segment SGI266299[] = {{&S68668[35], 3}, {NULL}};
const Segment SGI266300[] = {{&S68668[66], 8}, {NULL}};
const Segment SGI266301[] = {{&S68668[102], 8}, {NULL}};
const Segment SGI266302[] = {{&S68668[118], 9}, {NULL}};
const Segment SGI266303[] = {{&S68669[13], 6}, {&S68669[13], 6}, {NULL}};
const Segment SGI266304[] = {{&S68669[35], 3}, {NULL}};
const Segment SGI266305[] = {{&S68669[66], 8}, {NULL}};
const Segment SGI266306[] = {{&S68669[102], 8}, {NULL}};
const Segment SGI266307[] = {{&S68669[118], 9}, {NULL}};
const Segment SGI266308[] = {{&S68670[13], 6}, {&S68670[13], 6}, {NULL}};
const Segment SGI266309[] = {{&S68670[35], 3}, {NULL}};
const Segment SGI266310[] = {{&S68670[66], 8}, {NULL}};
const Segment SGI266311[] = {{&S68670[102], 6}, {NULL}};
const Segment SGI266312[] = {{&S68670[116], 9}, {NULL}};
const Segment SGI266313[] = {{&S68671[13], 6}, {&S68671[13], 6}, {NULL}};
const Segment SGI266314[] = {{&S68671[35], 5}, {NULL}};
const Segment SGI266315[] = {{&S68671[68], 8}, {NULL}};
const Segment SGI266316[] = {{&S68671[104], 6}, {NULL}};
const Segment SGI266317[] = {{&S68671[118], 9}, {NULL}};
const Segment SGI266318[] = {{&S68672[13], 4}, {NULL}};
const Segment SGI266319[] = {{&S68672[26], 3}, {NULL}};
const Segment SGI266320[] = {{&S68672[57], 8}, {NULL}};
const Segment SGI266321[] = {{&S68672[93], 6}, {NULL}};
const Segment SGI266322[] = {{&S68672[100], 15}, {NULL}};
const Segment SGI266323[] = {{&S68673[13], 4}, {NULL}};
const Segment SGI266324[] = {{&S68673[26], 5}, {NULL}};
const Segment SGI266325[] = {{&S68673[59], 8}, {NULL}};
const Segment SGI266326[] = {{&S68673[95], 6}, {NULL}};
const Segment SGI266327[] = {{&S68673[102], 15}, {NULL}};
const Segment SGI266328[] = {{&S62265[13], 7}, {NULL}};
const Segment SGI266329[] = {{&S62265[0], 145}, {NULL}};
const Segment SGI266330[] = {{&S62266[13], 7}, {NULL}};
const Segment SGI266331[] = {{&S62266[0], 164}, {NULL}};
const Segment SGI266332[] = {{&S54796[10], 4}, {NULL}};
const Segment SGI266333[] = {{&S54796[15], 5}, {NULL}};
const Segment SGI266334[] = {{&S54796[64], 13}, {NULL}};
const Segment SGI266335[] = {{&S69038[13], 4}, {NULL}};
const Segment SGI266336[] = {{&S69038[57], 5}, {NULL}};
const Segment SGI266337[] = {{&S69044[28], 4}, {NULL}};
const Segment SGI266338[] = {{&S69052[13], 4}, {NULL}};
const Segment SGI266339[] = {{&S69052[113], 5}, {NULL}};
const Segment SGI266340[] = {{&S69052[29], 3}, {NULL}};
const Segment SGI266341[] = {{&S69052[75], 9}, {NULL}};
const Segment SGI266342[] = {{&S69052[126], 6}, {NULL}};
const Segment SGI266343[] = {{&S69060[13], 4}, {NULL}};
const Segment SGI266344[] = {{&S69060[33], 3}, {NULL}};
const Segment SGI266345[] = {{&S69060[64], 6}, {NULL}};
const Segment SGI266346[] = {{&S69060[116], 8}, {NULL}};
const Segment SGI266347[] = {{&S69060[132], 7}, {NULL}};
const Segment SGI266348[] = {{&S69061[13], 4}, {NULL}};
const Segment SGI266349[] = {{&S69061[33], 3}, {NULL}};
const Segment SGI266350[] = {{&S69061[64], 6}, {NULL}};
const Segment SGI266351[] = {{&S69061[117], 8}, {NULL}};
const Segment SGI266352[] = {{&S69061[133], 7}, {NULL}};
const Segment SGI266353[] = {{&S69062[13], 4}, {NULL}};
const Segment SGI266354[] = {{&S69062[33], 3}, {NULL}};
const Segment SGI266355[] = {{&S69062[64], 6}, {NULL}};
const Segment SGI266356[] = {{&S69062[117], 6}, {NULL}};
const Segment SGI266357[] = {{&S69062[131], 7}, {NULL}};
const Segment SGI266358[] = {{&S69063[13], 4}, {NULL}};
const Segment SGI266359[] = {{&S69063[33], 5}, {NULL}};
const Segment SGI266360[] = {{&S69063[66], 6}, {NULL}};
const Segment SGI266361[] = {{&S69063[119], 6}, {NULL}};
const Segment SGI266362[] = {{&S69063[133], 7}, {NULL}};
const Segment SGI266363[] = {{&S69110[29], 4}, {NULL}};
const Segment SGI266364[] = {{&S69110[45], 3}, {NULL}};
const Segment SGI266365[] = {{&S69111[29], 4}, {NULL}};
const Segment SGI266366[] = {{&S69111[45], 5}, {NULL}};
const Segment SGI266367[] = {{&S62608[0], 15}, {NULL}};
const Segment SGI266368[] = {{&S62608[0], 78}, {NULL}};
const Segment SGI266369[] = {{&S62609[12], 15}, {NULL}};
const Segment SGI266370[] = {{&S62609[0], 78}, {NULL}};
const Segment SGI266371[] = {{&S62627[0], 7}, {&S62627[74], 24}, {NULL}};
const Segment SGI266372[] = {{&S62627[0], 98}, {NULL}};
const Segment SGI266373[] = {{&S62628[0], 7}, {&S62628[74], 24}, {NULL}};
const Segment SGI266374[] = {{&S62628[0], 98}, {NULL}};
const Segment SGI266375[] = {{&S62629[0], 7}, {&S62629[75], 24}, {NULL}};
const Segment SGI266376[] = {{&S62629[0], 99}, {NULL}};
const Segment SGI266377[] = {{&S62645[0], 21}, {&S62645[45], 11}, {&S62645[57], 32}, {&S62645[3], 6}, {NULL}};
const Segment SGI266378[] = {{&S62645[0], 111}, {NULL}};
const Segment SGI266379[] = {{&S55456[72], 4}, {NULL}};
const Segment SGI266380[] = {{&S55456[77], 5}, {NULL}};
const Segment SGI266381[] = {{&S55457[10], 4}, {NULL}};
const Segment SGI266382[] = {{&S55457[15], 5}, {NULL}};
const Segment SGI266383[] = {{&S55458[72], 4}, {NULL}};
const Segment SGI266384[] = {{&S55458[77], 5}, {NULL}};
const Segment SGI266385[] = {{&S69434[13], 4}, {&S69434[13], 4}, {NULL}};
const Segment SGI266386[] = {{&S69434[29], 5}, {NULL}};
const Segment SGI266387[] = {{&S69434[69], 9}, {NULL}};
const Segment SGI266388[] = {{&S69434[106], 5}, {NULL}};
const Segment SGI266389[] = {{&S69434[112], 7}, {NULL}};
const Segment SGI266390[] = {{&S69435[13], 4}, {&S69435[13], 4}, {NULL}};
const Segment SGI266391[] = {{&S69435[29], 5}, {NULL}};
const Segment SGI266392[] = {{&S69435[69], 9}, {NULL}};
const Segment SGI266393[] = {{&S69435[106], 5}, {NULL}};
const Segment SGI266394[] = {{&S69435[112], 7}, {NULL}};
const Segment SGI266395[] = {{&S69436[13], 4}, {&S69436[13], 4}, {NULL}};
const Segment SGI266396[] = {{&S69436[29], 3}, {NULL}};
const Segment SGI266397[] = {{&S69436[67], 9}, {NULL}};
const Segment SGI266398[] = {{&S69436[104], 5}, {NULL}};
const Segment SGI266399[] = {{&S69436[110], 7}, {NULL}};
const Segment SGI266400[] = {{&S62895[51], 11}, {NULL}};
const Segment SGI266401[] = {{&S62895[0], 62}, {NULL}};
const Segment SGI266402[] = {{&S62896[51], 12}, {NULL}};
const Segment SGI266403[] = {{&S62896[0], 63}, {NULL}};
const Segment SGI266404[] = {{&S62897[70], 12}, {NULL}};
const Segment SGI266405[] = {{&S62897[0], 82}, {NULL}};
const Segment SGI266406[] = {{&S62898[90], 12}, {NULL}};
const Segment SGI266407[] = {{&S62898[0], 102}, {NULL}};
const Segment SGI266408[] = {{&S62899[71], 12}, {NULL}};
const Segment SGI266409[] = {{&S62899[0], 83}, {NULL}};
const Segment SGI266410[] = {{&S62900[51], 28}, {NULL}};
const Segment SGI266411[] = {{&S62900[0], 79}, {NULL}};
const Segment SGI266412[] = {{&S62901[85], 29}, {NULL}};
const Segment SGI266413[] = {{&S62901[0], 114}, {NULL}};
const Segment SGI266414[] = {{&S69596[12], 4}, {&S69596[12], 4}, {NULL}};
const Segment SGI266415[] = {{&S69596[8], 1}, {NULL}};
const Segment SGI266416[] = {{&S69596[72], 6}, {NULL}};
const Segment SGI266417[] = {{&S69596[135], 9}, {NULL}};
const Segment SGI266418[] = {{&S69596[99], 13}, {NULL}};
const Segment SGI266419[] = {{&S55834[20], 26}, {NULL}};
const Segment SGI266420[] = {{&S55835[20], 27}, {NULL}};
const Segment SGI266421[] = {{&S55836[20], 27}, {NULL}};
const Segment SGI266422[] = {{&S55837[20], 27}, {NULL}};
const Segment SGI266423[] = {{&S55838[20], 27}, {NULL}};
const Segment SGI266424[] = {{&S55839[20], 26}, {NULL}};
const Segment SGI266425[] = {{&S55840[20], 27}, {NULL}};
const Segment SGI266426[] = {{&S55841[20], 27}, {NULL}};
const Segment SGI266427[] = {{&S55842[20], 26}, {NULL}};
const Segment SGI266428[] = {{&S55843[20], 26}, {NULL}};
const Segment SGI266429[] = {{&S55844[20], 26}, {NULL}};
const Segment SGI266430[] = {{&S55845[20], 27}, {NULL}};
const Segment SGI266431[] = {{&S55846[20], 27}, {NULL}};
const Segment SGI266432[] = {{&S55847[20], 27}, {NULL}};
const Segment SGI266433[] = {{&S55848[20], 27}, {NULL}};
const Segment SGI266434[] = {{&S55849[20], 26}, {NULL}};
const Segment SGI266435[] = {{&S55850[20], 26}, {NULL}};
const Segment SGI266436[] = {{&S69863[13], 6}, {NULL}};
const Segment SGI266437[] = {{&S69863[124], 3}, {NULL}};
const Segment SGI266438[] = {{&S69863[35], 3}, {NULL}};
const Segment SGI266439[] = {{&S69863[66], 8}, {NULL}};
const Segment SGI266440[] = {{&S69863[102], 6}, {NULL}};
const Segment SGI266441[] = {{&S69864[13], 6}, {NULL}};
const Segment SGI266442[] = {{&S69864[127], 3}, {NULL}};
const Segment SGI266443[] = {{&S69864[35], 5}, {NULL}};
const Segment SGI266444[] = {{&S69864[68], 8}, {NULL}};
const Segment SGI266445[] = {{&S69864[104], 7}, {NULL}};
const Segment SGI266446[] = {{&S69865[13], 6}, {NULL}};
const Segment SGI266447[] = {{&S69865[126], 3}, {NULL}};
const Segment SGI266448[] = {{&S69865[35], 5}, {NULL}};
const Segment SGI266449[] = {{&S69865[68], 8}, {NULL}};
const Segment SGI266450[] = {{&S69865[104], 6}, {NULL}};
const Segment SGI266451[] = {{&S69866[13], 6}, {NULL}};
const Segment SGI266452[] = {{&S69866[126], 3}, {NULL}};
const Segment SGI266453[] = {{&S69866[35], 3}, {NULL}};
const Segment SGI266454[] = {{&S69866[66], 8}, {NULL}};
const Segment SGI266455[] = {{&S69866[102], 8}, {NULL}};
const Segment SGI266456[] = {{&S69867[13], 6}, {NULL}};
const Segment SGI266457[] = {{&S69867[124], 3}, {NULL}};
const Segment SGI266458[] = {{&S69867[35], 3}, {NULL}};
const Segment SGI266459[] = {{&S69867[66], 8}, {NULL}};
const Segment SGI266460[] = {{&S69867[102], 6}, {NULL}};
const Segment SGI266461[] = {{&S63201[24], 3}, {&S63201[27], 22}, {NULL}};
const Segment SGI266462[] = {{&S63201[0], 49}, {NULL}};
const Segment SGI266463[] = {{&S63202[26], 3}, {&S63202[29], 22}, {NULL}};
const Segment SGI266464[] = {{&S63202[0], 51}, {NULL}};
const Segment SGI266465[] = {{&S69902[27], 6}, {NULL}};
const Segment SGI266466[] = {{&S69902[52], 5}, {NULL}};
const Segment SGI266467[] = {{&S69902[85], 6}, {NULL}};
const Segment SGI266468[] = {{&S69902[119], 5}, {NULL}};
const Segment SGI266469[] = {{&S69902[132], 9}, {NULL}};
const Segment SGI266470[] = {{&S69902[0], 14}, {NULL}};
const Segment SGI266471[] = {{&S69903[28], 6}, {NULL}};
const Segment SGI266472[] = {{&S69903[53], 5}, {NULL}};
const Segment SGI266473[] = {{&S69903[86], 6}, {NULL}};
const Segment SGI266474[] = {{&S69903[120], 5}, {NULL}};
const Segment SGI266475[] = {{&S69903[133], 9}, {NULL}};
const Segment SGI266476[] = {{&S69903[0], 15}, {NULL}};
const Segment SGI266477[] = {{&S69904[27], 6}, {NULL}};
const Segment SGI266478[] = {{&S69904[51], 5}, {NULL}};
const Segment SGI266479[] = {{&S69904[84], 6}, {NULL}};
const Segment SGI266480[] = {{&S69904[118], 5}, {NULL}};
const Segment SGI266481[] = {{&S69904[131], 9}, {NULL}};
const Segment SGI266482[] = {{&S69904[0], 14}, {NULL}};
const Segment SGI266483[] = {{&S69905[28], 6}, {NULL}};
const Segment SGI266484[] = {{&S69905[52], 5}, {NULL}};
const Segment SGI266485[] = {{&S69905[85], 6}, {NULL}};
const Segment SGI266486[] = {{&S69905[119], 5}, {NULL}};
const Segment SGI266487[] = {{&S69905[132], 9}, {NULL}};
const Segment SGI266488[] = {{&S69905[0], 15}, {NULL}};
const Segment SGI266489[] = {{&S69906[28], 6}, {NULL}};
const Segment SGI266490[] = {{&S69906[53], 3}, {NULL}};
const Segment SGI266491[] = {{&S69906[84], 6}, {NULL}};
const Segment SGI266492[] = {{&S69906[118], 5}, {NULL}};
const Segment SGI266493[] = {{&S69906[131], 9}, {NULL}};
const Segment SGI266494[] = {{&S69906[0], 15}, {NULL}};
const Segment SGI266495[] = {{&S69907[28], 6}, {NULL}};
const Segment SGI266496[] = {{&S69907[52], 3}, {NULL}};
const Segment SGI266497[] = {{&S69907[83], 6}, {NULL}};
const Segment SGI266498[] = {{&S69907[117], 5}, {NULL}};
const Segment SGI266499[] = {{&S69907[130], 9}, {NULL}};
const Segment SGI266500[] = {{&S69907[0], 15}, {NULL}};
const Segment SGI266501[] = {{&S63283[36], 12}, {NULL}};
const Segment SGI266502[] = {{&S63283[0], 60}, {NULL}};
const Segment SGI266503[] = {{&S63284[35], 12}, {NULL}};
const Segment SGI266504[] = {{&S63284[0], 59}, {NULL}};
const Segment SGI266505[] = {{&S69962[13], 4}, {NULL}};
const Segment SGI266506[] = {{&S69962[33], 3}, {NULL}};
const Segment SGI266507[] = {{&S69962[64], 6}, {NULL}};
const Segment SGI266508[] = {{&S69962[117], 8}, {NULL}};
const Segment SGI266509[] = {{&S69962[133], 7}, {NULL}};
const Segment SGI266510[] = {{&S69963[13], 4}, {NULL}};
const Segment SGI266511[] = {{&S69963[33], 3}, {NULL}};
const Segment SGI266512[] = {{&S69963[64], 6}, {NULL}};
const Segment SGI266513[] = {{&S69963[117], 8}, {NULL}};
const Segment SGI266514[] = {{&S69963[133], 7}, {NULL}};
const Segment SGI266515[] = {{&S69964[13], 4}, {NULL}};
const Segment SGI266516[] = {{&S69964[33], 3}, {NULL}};
const Segment SGI266517[] = {{&S69964[64], 6}, {NULL}};
const Segment SGI266518[] = {{&S69964[117], 8}, {NULL}};
const Segment SGI266519[] = {{&S69964[133], 7}, {NULL}};
const Segment SGI266520[] = {{&S69965[13], 4}, {NULL}};
const Segment SGI266521[] = {{&S69965[33], 3}, {NULL}};
const Segment SGI266522[] = {{&S69965[64], 6}, {NULL}};
const Segment SGI266523[] = {{&S69965[117], 6}, {NULL}};
const Segment SGI266524[] = {{&S69965[131], 7}, {NULL}};
const Segment SGI266525[] = {{&S70057[13], 4}, {NULL}};
const Segment SGI266526[] = {{&S70057[33], 5}, {NULL}};
const Segment SGI266527[] = {{&S70057[66], 6}, {NULL}};
const Segment SGI266528[] = {{&S70057[120], 5}, {NULL}};
const Segment SGI266529[] = {{&S70057[133], 9}, {NULL}};
const Segment SGI266530[] = {{&S70058[13], 4}, {NULL}};
const Segment SGI266531[] = {{&S70058[33], 3}, {NULL}};
const Segment SGI266532[] = {{&S70058[64], 6}, {NULL}};
const Segment SGI266533[] = {{&S70058[118], 5}, {NULL}};
const Segment SGI266534[] = {{&S70058[131], 9}, {NULL}};
const Segment SGI266535[] = {{&S63534[40], 26}, {NULL}};
const Segment SGI266536[] = {{&S63534[0], 76}, {NULL}};
const Segment SGI266537[] = {{&S70107[27], 6}, {NULL}};
const Segment SGI266538[] = {{&S70107[52], 5}, {NULL}};
const Segment SGI266539[] = {{&S70107[85], 6}, {NULL}};
const Segment SGI266540[] = {{&S70107[119], 5}, {NULL}};
const Segment SGI266541[] = {{&S70107[132], 9}, {NULL}};
const Segment SGI266542[] = {{&S70107[0], 14}, {NULL}};
const Segment SGI266543[] = {{&S70108[27], 6}, {NULL}};
const Segment SGI266544[] = {{&S70108[51], 5}, {NULL}};
const Segment SGI266545[] = {{&S70108[84], 6}, {NULL}};
const Segment SGI266546[] = {{&S70108[118], 5}, {NULL}};
const Segment SGI266547[] = {{&S70108[131], 9}, {NULL}};
const Segment SGI266548[] = {{&S70108[0], 14}, {NULL}};
const Segment SGI266549[] = {{&S70109[27], 6}, {NULL}};
const Segment SGI266550[] = {{&S70109[52], 3}, {NULL}};
const Segment SGI266551[] = {{&S70109[83], 6}, {NULL}};
const Segment SGI266552[] = {{&S70109[117], 5}, {NULL}};
const Segment SGI266553[] = {{&S70109[130], 9}, {NULL}};
const Segment SGI266554[] = {{&S70109[0], 14}, {NULL}};
const Segment SGI266555[] = {{&S70110[27], 6}, {NULL}};
const Segment SGI266556[] = {{&S70110[51], 3}, {NULL}};
const Segment SGI266557[] = {{&S70110[82], 6}, {NULL}};
const Segment SGI266558[] = {{&S70110[116], 5}, {NULL}};
const Segment SGI266559[] = {{&S70110[129], 9}, {NULL}};
const Segment SGI266560[] = {{&S70110[0], 14}, {NULL}};
const Segment SGI266561[] = {{&S70133[13], 4}, {NULL}};
const Segment SGI266562[] = {{&S70133[115], 3}, {NULL}};
const Segment SGI266563[] = {{&S70133[42], 3}, {NULL}};
const Segment SGI266564[] = {{&S70133[80], 6}, {NULL}};
const Segment SGI266565[] = {{&S70133[126], 5}, {NULL}};
const Segment SGI266566[] = {{&S70133[139], 6}, {NULL}};
const Segment SGI266567[] = {{&S70134[13], 4}, {NULL}};
const Segment SGI266568[] = {{&S70134[117], 3}, {NULL}};
const Segment SGI266569[] = {{&S70134[42], 5}, {NULL}};
const Segment SGI266570[] = {{&S70134[82], 6}, {NULL}};
const Segment SGI266571[] = {{&S70134[128], 5}, {NULL}};
const Segment SGI266572[] = {{&S70134[141], 6}, {NULL}};
const Segment SGI266573[] = {{&S70135[13], 4}, {NULL}};
const Segment SGI266574[] = {{&S70135[117], 3}, {NULL}};
const Segment SGI266575[] = {{&S70135[42], 5}, {NULL}};
const Segment SGI266576[] = {{&S70135[82], 6}, {NULL}};
const Segment SGI266577[] = {{&S70135[128], 5}, {NULL}};
const Segment SGI266578[] = {{&S70135[141], 6}, {NULL}};
const Segment SGI266579[] = {{&S70136[13], 4}, {NULL}};
const Segment SGI266580[] = {{&S70136[33], 5}, {NULL}};
const Segment SGI266581[] = {{&S70136[66], 6}, {NULL}};
const Segment SGI266582[] = {{&S70136[100], 6}, {NULL}};
const Segment SGI266583[] = {{&S70136[114], 9}, {NULL}};
const Segment SGI266584[] = {{&S70171[13], 6}, {NULL}};
const Segment SGI266585[] = {{&S70171[35], 5}, {NULL}};
const Segment SGI266586[] = {{&S70171[135], 3}, {NULL}};
const Segment SGI266587[] = {{&S70171[68], 6}, {NULL}};
const Segment SGI266588[] = {{&S70171[102], 6}, {NULL}};
const Segment SGI266589[] = {{&S70171[116], 9}, {NULL}};
const Segment SGI266590[] = {{&S70172[13], 6}, {NULL}};
const Segment SGI266591[] = {{&S70172[35], 5}, {NULL}};
const Segment SGI266592[] = {{&S70172[135], 3}, {NULL}};
const Segment SGI266593[] = {{&S70172[68], 6}, {NULL}};
const Segment SGI266594[] = {{&S70172[102], 6}, {NULL}};
const Segment SGI266595[] = {{&S70172[116], 9}, {NULL}};
const Segment SGI266596[] = {{&S70173[13], 6}, {NULL}};
const Segment SGI266597[] = {{&S70173[35], 5}, {NULL}};
const Segment SGI266598[] = {{&S70173[135], 5}, {NULL}};
const Segment SGI266599[] = {{&S70173[68], 6}, {NULL}};
const Segment SGI266600[] = {{&S70173[102], 6}, {NULL}};
const Segment SGI266601[] = {{&S70173[116], 9}, {NULL}};
const Segment SGI266602[] = {{&S70174[13], 6}, {NULL}};
const Segment SGI266603[] = {{&S70174[35], 5}, {NULL}};
const Segment SGI266604[] = {{&S70174[135], 5}, {NULL}};
const Segment SGI266605[] = {{&S70174[68], 6}, {NULL}};
const Segment SGI266606[] = {{&S70174[102], 6}, {NULL}};
const Segment SGI266607[] = {{&S70174[116], 9}, {NULL}};
const Segment SGI266608[] = {{&S70175[13], 6}, {NULL}};
const Segment SGI266609[] = {{&S70175[35], 5}, {NULL}};
const Segment SGI266610[] = {{&S70175[135], 5}, {NULL}};
const Segment SGI266611[] = {{&S70175[68], 6}, {NULL}};
const Segment SGI266612[] = {{&S70175[102], 6}, {NULL}};
const Segment SGI266613[] = {{&S70175[116], 9}, {NULL}};
const Segment SGI266614[] = {{&S70176[13], 6}, {NULL}};
const Segment SGI266615[] = {{&S70176[35], 5}, {NULL}};
const Segment SGI266616[] = {{&S70176[135], 5}, {NULL}};
const Segment SGI266617[] = {{&S70176[68], 6}, {NULL}};
const Segment SGI266618[] = {{&S70176[102], 6}, {NULL}};
const Segment SGI266619[] = {{&S70176[116], 9}, {NULL}};
const Segment SGI266620[] = {{&S70177[13], 6}, {NULL}};
const Segment SGI266621[] = {{&S70177[35], 5}, {NULL}};
const Segment SGI266622[] = {{&S70177[135], 5}, {NULL}};
const Segment SGI266623[] = {{&S70177[68], 6}, {NULL}};
const Segment SGI266624[] = {{&S70177[102], 6}, {NULL}};
const Segment SGI266625[] = {{&S70177[116], 9}, {NULL}};
const Segment SGI266626[] = {{&S70420[12], 4}, {NULL}};
const Segment SGI266627[] = {{&S70438[13], 6}, {NULL}};
const Segment SGI266628[] = {{&S70438[101], 3}, {NULL}};
const Segment SGI266629[] = {{&S70438[35], 3}, {NULL}};
const Segment SGI266630[] = {{&S70438[66], 6}, {NULL}};
const Segment SGI266631[] = {{&S70438[112], 5}, {NULL}};
const Segment SGI266632[] = {{&S70438[144], 5}, {NULL}};
const Segment SGI266633[] = {{&S70467[13], 4}, {NULL}};
const Segment SGI266634[] = {{&S70467[26], 5}, {NULL}};
const Segment SGI266635[] = {{&S70467[59], 6}, {NULL}};
const Segment SGI266636[] = {{&S70467[107], 5}, {NULL}};
const Segment SGI266637[] = {{&S70467[120], 9}, {NULL}};
const Segment SGI266638[] = {{&S70468[13], 4}, {NULL}};
const Segment SGI266639[] = {{&S70468[26], 3}, {NULL}};
const Segment SGI266640[] = {{&S70468[57], 6}, {NULL}};
const Segment SGI266641[] = {{&S70468[105], 5}, {NULL}};
const Segment SGI266642[] = {{&S70468[118], 9}, {NULL}};
const Segment SGI266643[] = {{&S56922[10], 4}, {NULL}};
const Segment SGI266644[] = {{&S56922[15], 5}, {NULL}};
const Segment SGI266645[] = {{&S70701[13], 4}, {NULL}};
const Segment SGI266646[] = {{&S70701[36], 3}, {NULL}};
const Segment SGI266647[] = {{&S70701[74], 8}, {NULL}};
const Segment SGI266648[] = {{&S70701[129], 6}, {NULL}};
const Segment SGI266649[] = {{&S70701[143], 9}, {NULL}};
const Segment SGI266650[] = {{&S70702[13], 4}, {NULL}};
const Segment SGI266651[] = {{&S70702[36], 5}, {NULL}};
const Segment SGI266652[] = {{&S70702[76], 8}, {NULL}};
const Segment SGI266653[] = {{&S70702[131], 6}, {NULL}};
const Segment SGI266654[] = {{&S70702[145], 9}, {NULL}};
const Segment SGI266655[] = {{&S70703[13], 4}, {NULL}};
const Segment SGI266656[] = {{&S70703[36], 5}, {NULL}};
const Segment SGI266657[] = {{&S70703[76], 8}, {NULL}};
const Segment SGI266658[] = {{&S70703[131], 6}, {NULL}};
const Segment SGI266659[] = {{&S70703[145], 9}, {NULL}};
const Segment SGI266660[] = {{&S70977[13], 4}, {NULL}};
const Segment SGI266661[] = {{&S70977[26], 5}, {NULL}};
const Segment SGI266662[] = {{&S70977[59], 6}, {NULL}};
const Segment SGI266663[] = {{&S70977[117], 6}, {NULL}};
const Segment SGI266664[] = {{&S70977[131], 7}, {NULL}};
const Segment SGI266665[] = {{&S70978[13], 4}, {NULL}};
const Segment SGI266666[] = {{&S70978[26], 5}, {NULL}};
const Segment SGI266667[] = {{&S70978[59], 6}, {NULL}};
const Segment SGI266668[] = {{&S70978[117], 6}, {NULL}};
const Segment SGI266669[] = {{&S70978[131], 7}, {NULL}};
const Segment SGI266670[] = {{&S70994[13], 4}, {NULL}};
const Segment SGI266671[] = {{&S70994[33], 5}, {NULL}};
const Segment SGI266672[] = {{&S70994[66], 6}, {NULL}};
const Segment SGI266673[] = {{&S70994[119], 5}, {NULL}};
const Segment SGI266674[] = {{&S70994[132], 9}, {NULL}};
const Segment SGI266675[] = {{&S70995[13], 4}, {NULL}};
const Segment SGI266676[] = {{&S70995[33], 3}, {NULL}};
const Segment SGI266677[] = {{&S70995[64], 6}, {NULL}};
const Segment SGI266678[] = {{&S70995[117], 5}, {NULL}};
const Segment SGI266679[] = {{&S70995[130], 9}, {NULL}};
const Segment SGI266680[] = {{&S71077[13], 4}, {NULL}};
const Segment SGI266681[] = {{&S71077[26], 5}, {NULL}};
const Segment SGI266682[] = {{&S71077[59], 6}, {NULL}};
const Segment SGI266683[] = {{&S71077[108], 6}, {NULL}};
const Segment SGI266684[] = {{&S71077[122], 9}, {NULL}};
const Segment SGI266685[] = {{&S71077[86], 14}, {NULL}};
const Segment SGI266686[] = {{&S71078[13], 4}, {NULL}};
const Segment SGI266687[] = {{&S71078[26], 3}, {NULL}};
const Segment SGI266688[] = {{&S71078[57], 6}, {NULL}};
const Segment SGI266689[] = {{&S71078[106], 6}, {NULL}};
const Segment SGI266690[] = {{&S71078[120], 9}, {NULL}};
const Segment SGI266691[] = {{&S71078[84], 14}, {NULL}};
const Segment SGI266692[] = {{&S71079[13], 4}, {NULL}};
const Segment SGI266693[] = {{&S71079[26], 5}, {NULL}};
const Segment SGI266694[] = {{&S71079[59], 6}, {NULL}};
const Segment SGI266695[] = {{&S71079[108], 6}, {NULL}};
const Segment SGI266696[] = {{&S71079[122], 9}, {NULL}};
const Segment SGI266697[] = {{&S71079[86], 14}, {NULL}};
const Segment SGI266698[] = {{&S71080[13], 4}, {NULL}};
const Segment SGI266699[] = {{&S71080[26], 3}, {NULL}};
const Segment SGI266700[] = {{&S71080[57], 6}, {NULL}};
const Segment SGI266701[] = {{&S71080[106], 6}, {NULL}};
const Segment SGI266702[] = {{&S71080[120], 9}, {NULL}};
const Segment SGI266703[] = {{&S71080[84], 14}, {NULL}};
const Segment SGI266704[] = {{&S71081[13], 4}, {NULL}};
const Segment SGI266705[] = {{&S71081[26], 5}, {NULL}};
const Segment SGI266706[] = {{&S71081[59], 6}, {NULL}};
const Segment SGI266707[] = {{&S71081[108], 7}, {NULL}};
const Segment SGI266708[] = {{&S71081[123], 9}, {NULL}};
const Segment SGI266709[] = {{&S71081[86], 14}, {NULL}};
const Segment SGI266710[] = {{&S71082[13], 4}, {NULL}};
const Segment SGI266711[] = {{&S71082[26], 5}, {NULL}};
const Segment SGI266712[] = {{&S71082[59], 6}, {NULL}};
const Segment SGI266713[] = {{&S71082[108], 7}, {NULL}};
const Segment SGI266714[] = {{&S71082[123], 9}, {NULL}};
const Segment SGI266715[] = {{&S71082[86], 14}, {NULL}};
const Segment SGI266716[] = {{&S71123[13], 6}, {NULL}};
const Segment SGI266717[] = {{&S71123[35], 3}, {NULL}};
const Segment SGI266718[] = {{&S71123[73], 8}, {NULL}};
const Segment SGI266719[] = {{&S71123[109], 8}, {NULL}};
const Segment SGI266720[] = {{&S71123[118], 13}, {NULL}};
const Segment SGI266721[] = {{&S71124[13], 6}, {NULL}};
const Segment SGI266722[] = {{&S71124[35], 3}, {NULL}};
const Segment SGI266723[] = {{&S71124[73], 8}, {NULL}};
const Segment SGI266724[] = {{&S71124[109], 6}, {NULL}};
const Segment SGI266725[] = {{&S71124[116], 13}, {NULL}};
const Segment SGI266726[] = {{&S71125[13], 6}, {NULL}};
const Segment SGI266727[] = {{&S71125[35], 5}, {NULL}};
const Segment SGI266728[] = {{&S71125[75], 8}, {NULL}};
const Segment SGI266729[] = {{&S71125[111], 6}, {NULL}};
const Segment SGI266730[] = {{&S71125[118], 13}, {NULL}};
const Segment SGI266731[] = {{&S71128[13], 4}, {NULL}};
const Segment SGI266732[] = {{&S71128[26], 5}, {NULL}};
const Segment SGI266733[] = {{&S71128[59], 6}, {NULL}};
const Segment SGI266734[] = {{&S71128[117], 5}, {NULL}};
const Segment SGI266735[] = {{&S71128[130], 9}, {NULL}};
const Segment SGI266736[] = {{&S71225[26], 4}, {NULL}};
const Segment SGI266737[] = {{&S71225[42], 5}, {NULL}};
const Segment SGI266738[] = {{&S71226[26], 4}, {NULL}};
const Segment SGI266739[] = {{&S71226[42], 5}, {NULL}};
const Segment SGI266740[] = {{&S71227[26], 4}, {NULL}};
const Segment SGI266741[] = {{&S71227[42], 5}, {NULL}};
const Segment SGI266742[] = {{&S71228[26], 4}, {NULL}};
const Segment SGI266743[] = {{&S71228[42], 5}, {NULL}};
const Segment SGI266744[] = {{&S71229[26], 4}, {NULL}};
const Segment SGI266745[] = {{&S71229[42], 5}, {NULL}};
const Segment SGI266746[] = {{&S71230[28], 4}, {NULL}};
const Segment SGI266747[] = {{&S71230[44], 3}, {NULL}};
const Segment SGI266748[] = {{&S71266[13], 6}, {NULL}};
const Segment SGI266749[] = {{&S71266[111], 5}, {NULL}};
const Segment SGI266750[] = {{&S71266[38], 3}, {NULL}};
const Segment SGI266751[] = {{&S71266[76], 6}, {NULL}};
const Segment SGI266752[] = {{&S71266[124], 6}, {NULL}};
const Segment SGI266753[] = {{&S71266[138], 6}, {NULL}};
const Segment SGI266754[] = {{&S71267[13], 6}, {NULL}};
const Segment SGI266755[] = {{&S71267[112], 5}, {NULL}};
const Segment SGI266756[] = {{&S71267[38], 3}, {NULL}};
const Segment SGI266757[] = {{&S71267[76], 7}, {NULL}};
const Segment SGI266758[] = {{&S71267[125], 4}, {NULL}};
const Segment SGI266759[] = {{&S71267[137], 6}, {NULL}};
const Segment SGI266760[] = {{&S64650[74], 12}, {NULL}};
const Segment SGI266761[] = {{&S64650[0], 86}, {NULL}};
const Segment SGI266762[] = {{&S71355[13], 6}, {NULL}};
const Segment SGI266763[] = {{&S71355[35], 5}, {NULL}};
const Segment SGI266764[] = {{&S71355[75], 6}, {NULL}};
const Segment SGI266765[] = {{&S71355[109], 6}, {NULL}};
const Segment SGI266766[] = {{&S71355[116], 13}, {NULL}};
const Segment SGI266767[] = {{&S71356[13], 6}, {NULL}};
const Segment SGI266768[] = {{&S71356[35], 3}, {NULL}};
const Segment SGI266769[] = {{&S71356[73], 6}, {NULL}};
const Segment SGI266770[] = {{&S71356[107], 6}, {NULL}};
const Segment SGI266771[] = {{&S71356[114], 13}, {NULL}};
const Segment SGI266772[] = {{&S71357[13], 6}, {NULL}};
const Segment SGI266773[] = {{&S71357[35], 3}, {NULL}};
const Segment SGI266774[] = {{&S71357[73], 6}, {NULL}};
const Segment SGI266775[] = {{&S71357[107], 6}, {NULL}};
const Segment SGI266776[] = {{&S71357[114], 13}, {NULL}};
const Segment SGI266777[] = {{&S71358[13], 6}, {NULL}};
const Segment SGI266778[] = {{&S71358[35], 3}, {NULL}};
const Segment SGI266779[] = {{&S71358[73], 6}, {NULL}};
const Segment SGI266780[] = {{&S71358[107], 6}, {NULL}};
const Segment SGI266781[] = {{&S71358[114], 13}, {NULL}};
const Segment SGI266782[] = {{&S71359[13], 6}, {NULL}};
const Segment SGI266783[] = {{&S71359[35], 5}, {NULL}};
const Segment SGI266784[] = {{&S71359[75], 6}, {NULL}};
const Segment SGI266785[] = {{&S71359[109], 6}, {NULL}};
const Segment SGI266786[] = {{&S71359[116], 13}, {NULL}};
const Segment SGI266787[] = {{&S71360[13], 6}, {NULL}};
const Segment SGI266788[] = {{&S71360[35], 5}, {NULL}};
const Segment SGI266789[] = {{&S71360[75], 6}, {NULL}};
const Segment SGI266790[] = {{&S71360[109], 6}, {NULL}};
const Segment SGI266791[] = {{&S71360[116], 13}, {NULL}};
const Segment SGI266792[] = {{&S71361[13], 6}, {NULL}};
const Segment SGI266793[] = {{&S71361[35], 5}, {NULL}};
const Segment SGI266794[] = {{&S71361[75], 6}, {NULL}};
const Segment SGI266795[] = {{&S71361[109], 6}, {NULL}};
const Segment SGI266796[] = {{&S71361[116], 13}, {NULL}};
const Segment SGI266797[] = {{&S57539[10], 4}, {NULL}};
const Segment SGI266798[] = {{&S57539[15], 5}, {NULL}};
const Segment SGI266799[] = {{&S71428[13], 6}, {NULL}};
const Segment SGI266800[] = {{&S71428[101], 5}, {NULL}};
const Segment SGI266801[] = {{&S71428[35], 3}, {NULL}};
const Segment SGI266802[] = {{&S71428[66], 6}, {NULL}};
const Segment SGI266803[] = {{&S71428[114], 7}, {NULL}};
const Segment SGI266804[] = {{&S71428[129], 9}, {NULL}};
const Segment SGI266805[] = {{&S71429[13], 6}, {NULL}};
const Segment SGI266806[] = {{&S71429[103], 5}, {NULL}};
const Segment SGI266807[] = {{&S71429[35], 5}, {NULL}};
const Segment SGI266808[] = {{&S71429[68], 6}, {NULL}};
const Segment SGI266809[] = {{&S71429[116], 5}, {NULL}};
const Segment SGI266810[] = {{&S71429[129], 9}, {NULL}};
const Segment SGI266811[] = {{&S71430[13], 6}, {NULL}};
const Segment SGI266812[] = {{&S71430[114], 5}, {NULL}};
const Segment SGI266813[] = {{&S71430[38], 5}, {NULL}};
const Segment SGI266814[] = {{&S71430[78], 7}, {NULL}};
const Segment SGI266815[] = {{&S71430[127], 6}, {NULL}};
const Segment SGI266816[] = {{&S71430[141], 9}, {NULL}};
const Segment SGI266817[] = {{&S71431[13], 6}, {NULL}};
const Segment SGI266818[] = {{&S71431[38], 5}, {NULL}};
const Segment SGI266819[] = {{&S71431[113], 5}, {NULL}};
const Segment SGI266820[] = {{&S71431[126], 9}, {NULL}};
const Segment SGI266821[] = {{&S71432[13], 6}, {NULL}};
const Segment SGI266822[] = {{&S71432[113], 5}, {NULL}};
const Segment SGI266823[] = {{&S71432[38], 3}, {NULL}};
const Segment SGI266824[] = {{&S71432[76], 8}, {NULL}};
const Segment SGI266825[] = {{&S71432[126], 3}, {NULL}};
const Segment SGI266826[] = {{&S71432[137], 9}, {NULL}};
const Segment SGI266827[] = {{&S71433[13], 6}, {NULL}};
const Segment SGI266828[] = {{&S71433[112], 5}, {NULL}};
const Segment SGI266829[] = {{&S71433[38], 3}, {NULL}};
const Segment SGI266830[] = {{&S71433[76], 7}, {NULL}};
const Segment SGI266831[] = {{&S71433[125], 6}, {NULL}};
const Segment SGI266832[] = {{&S71433[139], 9}, {NULL}};
const Segment SGI266833[] = {{&S71434[13], 6}, {NULL}};
const Segment SGI266834[] = {{&S71434[101], 5}, {NULL}};
const Segment SGI266835[] = {{&S71434[35], 3}, {NULL}};
const Segment SGI266836[] = {{&S71434[66], 6}, {NULL}};
const Segment SGI266837[] = {{&S71434[114], 5}, {NULL}};
const Segment SGI266838[] = {{&S71434[127], 9}, {NULL}};
const Segment SGI266839[] = {{&S71435[13], 6}, {NULL}};
const Segment SGI266840[] = {{&S71435[103], 5}, {NULL}};
const Segment SGI266841[] = {{&S71435[35], 5}, {NULL}};
const Segment SGI266842[] = {{&S71435[68], 6}, {NULL}};
const Segment SGI266843[] = {{&S71435[116], 5}, {NULL}};
const Segment SGI266844[] = {{&S71435[129], 9}, {NULL}};
const Segment SGI266845[] = {{&S71436[13], 6}, {NULL}};
const Segment SGI266846[] = {{&S71436[8], 3}, {NULL}};
const Segment SGI266847[] = {{&S71436[35], 3}, {NULL}};
const Segment SGI266848[] = {{&S71436[66], 6}, {NULL}};
const Segment SGI266849[] = {{&S71436[112], 5}, {NULL}};
const Segment SGI266850[] = {{&S71436[125], 9}, {NULL}};
const Segment SGI266851[] = {{&S71437[13], 6}, {NULL}};
const Segment SGI266852[] = {{&S71437[115], 5}, {NULL}};
const Segment SGI266853[] = {{&S71437[38], 5}, {NULL}};
const Segment SGI266854[] = {{&S71437[78], 8}, {NULL}};
const Segment SGI266855[] = {{&S71437[128], 3}, {NULL}};
const Segment SGI266856[] = {{&S71437[139], 9}, {NULL}};
const Segment SGI266857[] = {{&S71438[13], 6}, {NULL}};
const Segment SGI266858[] = {{&S71438[113], 5}, {NULL}};
const Segment SGI266859[] = {{&S71438[38], 3}, {NULL}};
const Segment SGI266860[] = {{&S71438[76], 8}, {NULL}};
const Segment SGI266861[] = {{&S71438[126], 5}, {NULL}};
const Segment SGI266862[] = {{&S71438[139], 9}, {NULL}};
const Segment SGI266863[] = {{&S71495[13], 4}, {NULL}};
const Segment SGI266864[] = {{&S71495[29], 5}, {NULL}};
const Segment SGI266865[] = {{&S71495[69], 8}, {NULL}};
const Segment SGI266866[] = {{&S71495[120], 3}, {NULL}};
const Segment SGI266867[] = {{&S71495[131], 9}, {NULL}};
const Segment SGI266868[] = {{&S71495[98], 14}, {NULL}};
const Segment SGI266869[] = {{&S71496[13], 4}, {NULL}};
const Segment SGI266870[] = {{&S71496[29], 5}, {NULL}};
const Segment SGI266871[] = {{&S71496[69], 8}, {NULL}};
const Segment SGI266872[] = {{&S71496[120], 3}, {NULL}};
const Segment SGI266873[] = {{&S71496[131], 9}, {NULL}};
const Segment SGI266874[] = {{&S71496[98], 14}, {NULL}};
const Segment SGI266875[] = {{&S57738[20], 27}, {NULL}};
const Segment SGI266876[] = {{&S57739[20], 27}, {NULL}};
const Segment SGI266877[] = {{&S57740[20], 27}, {NULL}};
const Segment SGI266878[] = {{&S57741[20], 27}, {NULL}};
const Segment SGI266879[] = {{&S57742[20], 27}, {NULL}};
const Segment SGI266880[] = {{&S57743[20], 27}, {NULL}};
const Segment SGI266881[] = {{&S71651[30], 4}, {NULL}};
const Segment SGI266882[] = {{&S71651[43], 5}, {NULL}};
const Segment SGI266883[] = {{&S71651[76], 6}, {NULL}};
const Segment SGI266884[] = {{&S71651[110], 5}, {NULL}};
const Segment SGI266885[] = {{&S71651[123], 9}, {NULL}};
const Segment SGI266886[] = {{&S71651[0], 14}, {NULL}};
const Segment SGI266887[] = {{&S71652[30], 4}, {NULL}};
const Segment SGI266888[] = {{&S71652[43], 3}, {NULL}};
const Segment SGI266889[] = {{&S71652[74], 6}, {NULL}};
const Segment SGI266890[] = {{&S71652[108], 5}, {NULL}};
const Segment SGI266891[] = {{&S71652[121], 9}, {NULL}};
const Segment SGI266892[] = {{&S71652[0], 14}, {NULL}};
const Segment SGI266893[] = {{&S71664[13], 6}, {NULL}};
const Segment SGI266894[] = {{&S71664[35], 5}, {NULL}};
const Segment SGI266895[] = {{&S71664[68], 6}, {NULL}};
const Segment SGI266896[] = {{&S71664[116], 5}, {NULL}};
const Segment SGI266897[] = {{&S71664[129], 9}, {NULL}};
const Segment SGI266898[] = {{&S71665[13], 6}, {NULL}};
const Segment SGI266899[] = {{&S71665[35], 3}, {NULL}};
const Segment SGI266900[] = {{&S71665[66], 6}, {NULL}};
const Segment SGI266901[] = {{&S71665[114], 5}, {NULL}};
const Segment SGI266902[] = {{&S71665[127], 9}, {NULL}};
const Segment SGI266903[] = {{&S71777[13], 6}, {NULL}};
const Segment SGI266904[] = {{&S71777[35], 5}, {NULL}};
const Segment SGI266905[] = {{&S71777[75], 6}, {NULL}};
const Segment SGI266906[] = {{&S71777[109], 5}, {NULL}};
const Segment SGI266907[] = {{&S71777[115], 13}, {NULL}};
const Segment SGI266908[] = {{&S71778[13], 6}, {NULL}};
const Segment SGI266909[] = {{&S71778[35], 5}, {NULL}};
const Segment SGI266910[] = {{&S71778[75], 6}, {NULL}};
const Segment SGI266911[] = {{&S71778[109], 5}, {NULL}};
const Segment SGI266912[] = {{&S71778[115], 13}, {NULL}};
const Segment SGI266913[] = {{&S71783[17], 6}, {NULL}};
const Segment SGI266914[] = {{&S71783[35], 3}, {NULL}};
const Segment SGI266915[] = {{&S71784[17], 6}, {NULL}};
const Segment SGI266916[] = {{&S71784[35], 5}, {NULL}};
const Segment SGI266917[] = {{&S71785[17], 6}, {NULL}};
const Segment SGI266918[] = {{&S71785[35], 3}, {NULL}};
const Segment SGI266919[] = {{&S71785[59], 3}, {NULL}};
const Segment SGI266920[] = {{&S71812[13], 6}, {NULL}};
const Segment SGI266921[] = {{&S71812[108], 3}, {NULL}};
const Segment SGI266922[] = {{&S71812[35], 3}, {NULL}};
const Segment SGI266923[] = {{&S71812[73], 6}, {NULL}};
const Segment SGI266924[] = {{&S71812[119], 5}, {NULL}};
const Segment SGI266925[] = {{&S71812[132], 5}, {NULL}};
const Segment SGI266926[] = {{&S57952[10], 5}, {NULL}};
const Segment SGI266927[] = {{&S57952[16], 5}, {NULL}};
const Segment SGI266928[] = {{&S71882[13], 4}, {NULL}};
const Segment SGI266929[] = {{&S71882[94], 3}, {NULL}};
const Segment SGI266930[] = {{&S71882[26], 5}, {NULL}};
const Segment SGI266931[] = {{&S71882[59], 6}, {NULL}};
const Segment SGI266932[] = {{&S71882[105], 5}, {NULL}};
const Segment SGI266933[] = {{&S71882[118], 9}, {NULL}};
const Segment SGI266934[] = {{&S71883[13], 4}, {NULL}};
const Segment SGI266935[] = {{&S71883[94], 3}, {NULL}};
const Segment SGI266936[] = {{&S71883[26], 5}, {NULL}};
const Segment SGI266937[] = {{&S71883[59], 6}, {NULL}};
const Segment SGI266938[] = {{&S71883[105], 6}, {NULL}};
const Segment SGI266939[] = {{&S71883[119], 7}, {NULL}};
const Segment SGI266940[] = {{&S71884[13], 4}, {NULL}};
const Segment SGI266941[] = {{&S71884[107], 3}, {NULL}};
const Segment SGI266942[] = {{&S71884[26], 5}, {NULL}};
const Segment SGI266943[] = {{&S71884[59], 6}, {NULL}};
const Segment SGI266944[] = {{&S71884[93], 5}, {NULL}};
const Segment SGI266945[] = {{&S71884[118], 7}, {NULL}};
const Segment SGI266946[] = {{&S71885[13], 4}, {NULL}};
const Segment SGI266947[] = {{&S71885[94], 3}, {NULL}};
const Segment SGI266948[] = {{&S71885[26], 5}, {NULL}};
const Segment SGI266949[] = {{&S71885[59], 6}, {NULL}};
const Segment SGI266950[] = {{&S71885[105], 6}, {NULL}};
const Segment SGI266951[] = {{&S71885[119], 7}, {NULL}};
const Segment SGI266952[] = {{&S71886[13], 4}, {NULL}};
const Segment SGI266953[] = {{&S71886[92], 3}, {NULL}};
const Segment SGI266954[] = {{&S71886[26], 3}, {NULL}};
const Segment SGI266955[] = {{&S71886[57], 6}, {NULL}};
const Segment SGI266956[] = {{&S71886[103], 6}, {NULL}};
const Segment SGI266957[] = {{&S71886[117], 7}, {NULL}};
const Segment SGI266958[] = {{&S71887[13], 4}, {NULL}};
const Segment SGI266959[] = {{&S71887[92], 3}, {NULL}};
const Segment SGI266960[] = {{&S71887[26], 3}, {NULL}};
const Segment SGI266961[] = {{&S71887[57], 6}, {NULL}};
const Segment SGI266962[] = {{&S71887[103], 5}, {NULL}};
const Segment SGI266963[] = {{&S71887[116], 9}, {NULL}};
const Segment SGI266964[] = {{&S71910[28], 4}, {NULL}};
const Segment SGI266965[] = {{&S71910[44], 3}, {NULL}};
const Segment SGI266966[] = {{&S71911[28], 4}, {NULL}};
const Segment SGI266967[] = {{&S71911[44], 5}, {NULL}};
const Segment SGI266968[] = {{&S71912[28], 4}, {NULL}};
const Segment SGI266969[] = {{&S71912[44], 5}, {NULL}};
const Segment SGI266970[] = {{&S71913[38], 4}, {NULL}};
const Segment SGI266971[] = {{&S71913[130], 5}, {NULL}};
const Segment SGI266972[] = {{&S71913[54], 3}, {NULL}};
const Segment SGI266973[] = {{&S71913[92], 9}, {NULL}};
const Segment SGI266974[] = {{&S71913[143], 5}, {NULL}};
const Segment SGI266975[] = {{&S71918[28], 4}, {NULL}};
const Segment SGI266976[] = {{&S71918[41], 5}, {NULL}};
const Segment SGI266977[] = {{&S71918[74], 6}, {NULL}};
const Segment SGI266978[] = {{&S71918[108], 5}, {NULL}};
const Segment SGI266979[] = {{&S71918[121], 9}, {NULL}};
const Segment SGI266980[] = {{&S71918[0], 14}, {NULL}};
const Segment SGI266981[] = {{&S71919[28], 4}, {NULL}};
const Segment SGI266982[] = {{&S71919[41], 3}, {NULL}};
const Segment SGI266983[] = {{&S71919[72], 6}, {NULL}};
const Segment SGI266984[] = {{&S71919[106], 5}, {NULL}};
const Segment SGI266985[] = {{&S71919[119], 9}, {NULL}};
const Segment SGI266986[] = {{&S71919[0], 14}, {NULL}};
const Segment SGI266987[] = {{&S71980[13], 4}, {NULL}};
const Segment SGI266988[] = {{&S71980[26], 5}, {NULL}};
const Segment SGI266989[] = {{&S71980[59], 6}, {NULL}};
const Segment SGI266990[] = {{&S71980[108], 5}, {NULL}};
const Segment SGI266991[] = {{&S71980[121], 9}, {NULL}};
const Segment SGI266992[] = {{&S71980[86], 14}, {NULL}};
const Segment SGI266993[] = {{&S71981[13], 4}, {NULL}};
const Segment SGI266994[] = {{&S71981[26], 3}, {NULL}};
const Segment SGI266995[] = {{&S71981[57], 6}, {NULL}};
const Segment SGI266996[] = {{&S71981[108], 5}, {NULL}};
const Segment SGI266997[] = {{&S71981[121], 9}, {NULL}};
const Segment SGI266998[] = {{&S71981[84], 14}, {NULL}};
const Segment SGI266999[] = {{&S72132[17], 4}, {NULL}};
const Segment SGI267000[] = {{&S72132[36], 5}, {NULL}};
const Segment SGI267001[] = {{&S72132[66], 3}, {NULL}};
const Segment SGI267002[] = {{&S65317[0], 7}, {&S65317[77], 24}, {NULL}};
const Segment SGI267003[] = {{&S65317[0], 101}, {NULL}};
const Segment SGI267004[] = {{&S65318[0], 7}, {&S65318[77], 24}, {NULL}};
const Segment SGI267005[] = {{&S65318[0], 101}, {NULL}};
const Segment SGI267006[] = {{&S65319[0], 7}, {&S65319[77], 24}, {NULL}};
const Segment SGI267007[] = {{&S65319[0], 101}, {NULL}};
const Segment SGI267008[] = {{&S65320[4], 3}, {NULL}};
const Segment SGI267009[] = {{&S65320[0], 51}, {NULL}};
const Segment SGI267010[] = {{&S65321[4], 3}, {NULL}};
const Segment SGI267011[] = {{&S65321[0], 51}, {NULL}};
const Segment SGI267012[] = {{&S72291[13], 4}, {NULL}};
const Segment SGI267013[] = {{&S72291[33], 5}, {NULL}};
const Segment SGI267014[] = {{&S72291[66], 6}, {NULL}};
const Segment SGI267015[] = {{&S72291[119], 5}, {NULL}};
const Segment SGI267016[] = {{&S72291[132], 9}, {NULL}};
const Segment SGI267017[] = {{&S72292[13], 4}, {NULL}};
const Segment SGI267018[] = {{&S72292[33], 3}, {NULL}};
const Segment SGI267019[] = {{&S72292[64], 6}, {NULL}};
const Segment SGI267020[] = {{&S72292[117], 5}, {NULL}};
const Segment SGI267021[] = {{&S72292[130], 9}, {NULL}};
const Segment SGI267022[] = {{&S58539[10], 4}, {NULL}};
const Segment SGI267023[] = {{&S58539[15], 5}, {NULL}};
const Segment SGI267024[] = {{&S58555[10], 4}, {NULL}};
const Segment SGI267025[] = {{&S58555[15], 5}, {NULL}};
const Segment SGI267026[] = {{&S72522[13], 4}, {NULL}};
const Segment SGI267027[] = {{&S72522[26], 5}, {NULL}};
const Segment SGI267028[] = {{&S72522[59], 6}, {NULL}};
const Segment SGI267029[] = {{&S72522[108], 5}, {NULL}};
const Segment SGI267030[] = {{&S72522[121], 9}, {NULL}};
const Segment SGI267031[] = {{&S72522[86], 14}, {NULL}};
const Segment SGI267032[] = {{&S72523[13], 4}, {NULL}};
const Segment SGI267033[] = {{&S72523[26], 3}, {NULL}};
const Segment SGI267034[] = {{&S72523[57], 6}, {NULL}};
const Segment SGI267035[] = {{&S72523[106], 5}, {NULL}};
const Segment SGI267036[] = {{&S72523[119], 9}, {NULL}};
const Segment SGI267037[] = {{&S72523[84], 14}, {NULL}};
const Segment SGI267038[] = {{&S72584[26], 6}, {NULL}};
const Segment SGI267039[] = {{&S72585[17], 6}, {NULL}};
const Segment SGI267040[] = {{&S72585[12], 1}, {NULL}};
const Segment SGI267041[] = {{&S72619[27], 6}, {NULL}};
const Segment SGI267042[] = {{&S72619[52], 5}, {NULL}};
const Segment SGI267043[] = {{&S72619[85], 6}, {NULL}};
const Segment SGI267044[] = {{&S72619[119], 5}, {NULL}};
const Segment SGI267045[] = {{&S72619[132], 9}, {NULL}};
const Segment SGI267046[] = {{&S72619[0], 14}, {NULL}};
const Segment SGI267047[] = {{&S72620[27], 6}, {NULL}};
const Segment SGI267048[] = {{&S72620[51], 5}, {NULL}};
const Segment SGI267049[] = {{&S72620[84], 6}, {NULL}};
const Segment SGI267050[] = {{&S72620[118], 5}, {NULL}};
const Segment SGI267051[] = {{&S72620[131], 9}, {NULL}};
const Segment SGI267052[] = {{&S72620[0], 14}, {NULL}};
const Segment SGI267053[] = {{&S72621[28], 6}, {NULL}};
const Segment SGI267054[] = {{&S72621[53], 5}, {NULL}};
const Segment SGI267055[] = {{&S72621[86], 6}, {NULL}};
const Segment SGI267056[] = {{&S72621[120], 5}, {NULL}};
const Segment SGI267057[] = {{&S72621[133], 9}, {NULL}};
const Segment SGI267058[] = {{&S72621[0], 15}, {NULL}};
const Segment SGI267059[] = {{&S72622[28], 6}, {NULL}};
const Segment SGI267060[] = {{&S72622[52], 5}, {NULL}};
const Segment SGI267061[] = {{&S72622[85], 6}, {NULL}};
const Segment SGI267062[] = {{&S72622[119], 5}, {NULL}};
const Segment SGI267063[] = {{&S72622[132], 9}, {NULL}};
const Segment SGI267064[] = {{&S72622[0], 15}, {NULL}};
const Segment SGI267065[] = {{&S72746[13], 4}, {NULL}};
const Segment SGI267066[] = {{&S72746[92], 3}, {NULL}};
const Segment SGI267067[] = {{&S72746[26], 3}, {NULL}};
const Segment SGI267068[] = {{&S72746[57], 6}, {NULL}};
const Segment SGI267069[] = {{&S72746[103], 5}, {NULL}};
const Segment SGI267070[] = {{&S72746[137], 5}, {NULL}};
const Segment SGI267071[] = {{&S72762[27], 6}, {NULL}};
const Segment SGI267072[] = {{&S72762[51], 3}, {NULL}};
const Segment SGI267073[] = {{&S72762[82], 8}, {NULL}};
const Segment SGI267074[] = {{&S72762[118], 8}, {NULL}};
const Segment SGI267075[] = {{&S72762[134], 9}, {NULL}};
const Segment SGI267076[] = {{&S72762[0], 14}, {NULL}};
const Segment SGI267077[] = {{&S72763[27], 6}, {NULL}};
const Segment SGI267078[] = {{&S72763[51], 3}, {NULL}};
const Segment SGI267079[] = {{&S72763[82], 8}, {NULL}};
const Segment SGI267080[] = {{&S72763[118], 6}, {NULL}};
const Segment SGI267081[] = {{&S72763[132], 9}, {NULL}};
const Segment SGI267082[] = {{&S72763[0], 14}, {NULL}};
const Segment SGI267083[] = {{&S72764[27], 6}, {NULL}};
const Segment SGI267084[] = {{&S72764[51], 5}, {NULL}};
const Segment SGI267085[] = {{&S72764[84], 8}, {NULL}};
const Segment SGI267086[] = {{&S72764[120], 6}, {NULL}};
const Segment SGI267087[] = {{&S72764[134], 9}, {NULL}};
const Segment SGI267088[] = {{&S72764[0], 14}, {NULL}};
const Segment SGI267089[] = {{&S58952[72], 4}, {NULL}};
const Segment SGI267090[] = {{&S58952[77], 5}, {NULL}};
const Segment SGI267091[] = {{&S58953[10], 4}, {NULL}};
const Segment SGI267092[] = {{&S58953[15], 5}, {NULL}};
const Segment SGI267093[] = {{&S58975[20], 27}, {NULL}};
const Segment SGI267094[] = {{&S58976[20], 27}, {NULL}};
const Segment SGI267095[] = {{&S58977[20], 27}, {NULL}};
const Segment SGI267096[] = {{&S58978[20], 27}, {NULL}};
const Segment SGI267097[] = {{&S58979[20], 27}, {NULL}};
const Segment SGI267098[] = {{&S58980[20], 27}, {NULL}};
const Segment SGI267099[] = {{&S58981[20], 27}, {NULL}};
const Segment SGI267100[] = {{&S58982[20], 27}, {NULL}};
const Segment SGI267101[] = {{&S58983[20], 27}, {NULL}};
const Segment SGI267102[] = {{&S58984[20], 27}, {NULL}};
const Segment SGI267103[] = {{&S58985[20], 27}, {NULL}};
const Segment SGI267104[] = {{&S58986[20], 26}, {NULL}};
const Segment SGI267105[] = {{&S72880[13], 4}, {NULL}};
const Segment SGI267106[] = {{&S72880[26], 3}, {NULL}};
const Segment SGI267107[] = {{&S72880[57], 8}, {NULL}};
const Segment SGI267108[] = {{&S72880[108], 6}, {NULL}};
const Segment SGI267109[] = {{&S72880[122], 9}, {NULL}};
const Segment SGI267110[] = {{&S72880[86], 14}, {NULL}};
const Segment SGI267111[] = {{&S72881[13], 4}, {NULL}};
const Segment SGI267112[] = {{&S72881[26], 5}, {NULL}};
const Segment SGI267113[] = {{&S72881[59], 8}, {NULL}};
const Segment SGI267114[] = {{&S72881[110], 6}, {NULL}};
const Segment SGI267115[] = {{&S72881[124], 9}, {NULL}};
const Segment SGI267116[] = {{&S72881[88], 14}, {NULL}};
const Segment SGI267117[] = {{&S65840[60], 12}, {NULL}};
const Segment SGI267118[] = {{&S65840[0], 72}, {NULL}};
const Segment SGI267119[] = {{&S72953[13], 6}, {NULL}};
const Segment SGI267120[] = {{&S72953[35], 3}, {NULL}};
const Segment SGI267121[] = {{&S72953[133], 5}, {NULL}};
const Segment SGI267122[] = {{&S72953[66], 6}, {NULL}};
const Segment SGI267123[] = {{&S72953[100], 6}, {NULL}};
const Segment SGI267124[] = {{&S72953[114], 9}, {NULL}};
const Segment SGI267125[] = {{&S72954[13], 6}, {NULL}};
const Segment SGI267126[] = {{&S72954[35], 3}, {NULL}};
const Segment SGI267127[] = {{&S72954[133], 5}, {NULL}};
const Segment SGI267128[] = {{&S72954[66], 6}, {NULL}};
const Segment SGI267129[] = {{&S72954[100], 6}, {NULL}};
const Segment SGI267130[] = {{&S72954[114], 9}, {NULL}};
const Segment SGI267131[] = {{&S72955[13], 6}, {NULL}};
const Segment SGI267132[] = {{&S72955[35], 3}, {NULL}};
const Segment SGI267133[] = {{&S72955[133], 5}, {NULL}};
const Segment SGI267134[] = {{&S72955[66], 6}, {NULL}};
const Segment SGI267135[] = {{&S72955[100], 6}, {NULL}};
const Segment SGI267136[] = {{&S72955[114], 9}, {NULL}};
const Segment SGI267137[] = {{&S72956[13], 6}, {NULL}};
const Segment SGI267138[] = {{&S72956[35], 5}, {NULL}};
const Segment SGI267139[] = {{&S72956[135], 5}, {NULL}};
const Segment SGI267140[] = {{&S72956[68], 6}, {NULL}};
const Segment SGI267141[] = {{&S72956[102], 6}, {NULL}};
const Segment SGI267142[] = {{&S72956[116], 9}, {NULL}};
const Segment SGI267143[] = {{&S72957[13], 6}, {NULL}};
const Segment SGI267144[] = {{&S72957[35], 5}, {NULL}};
const Segment SGI267145[] = {{&S72957[135], 5}, {NULL}};
const Segment SGI267146[] = {{&S72957[68], 6}, {NULL}};
const Segment SGI267147[] = {{&S72957[102], 6}, {NULL}};
const Segment SGI267148[] = {{&S72957[116], 9}, {NULL}};
const Segment SGI267149[] = {{&S72958[13], 6}, {NULL}};
const Segment SGI267150[] = {{&S72958[35], 5}, {NULL}};
const Segment SGI267151[] = {{&S72958[135], 5}, {NULL}};
const Segment SGI267152[] = {{&S72958[68], 6}, {NULL}};
const Segment SGI267153[] = {{&S72958[102], 6}, {NULL}};
const Segment SGI267154[] = {{&S72958[116], 9}, {NULL}};
const Segment SGI267155[] = {{&S72959[13], 6}, {NULL}};
const Segment SGI267156[] = {{&S72959[35], 5}, {NULL}};
const Segment SGI267157[] = {{&S72959[135], 5}, {NULL}};
const Segment SGI267158[] = {{&S72959[68], 6}, {NULL}};
const Segment SGI267159[] = {{&S72959[102], 6}, {NULL}};
const Segment SGI267160[] = {{&S72959[116], 9}, {NULL}};
const Segment SGI267161[] = {{&S72960[13], 6}, {NULL}};
const Segment SGI267162[] = {{&S72960[35], 5}, {NULL}};
const Segment SGI267163[] = {{&S72960[135], 5}, {NULL}};
const Segment SGI267164[] = {{&S72960[68], 6}, {NULL}};
const Segment SGI267165[] = {{&S72960[102], 6}, {NULL}};
const Segment SGI267166[] = {{&S72960[116], 9}, {NULL}};
const Segment SGI267167[] = {{&S72961[13], 6}, {NULL}};
const Segment SGI267168[] = {{&S72961[35], 5}, {NULL}};
const Segment SGI267169[] = {{&S72961[135], 5}, {NULL}};
const Segment SGI267170[] = {{&S72961[68], 6}, {NULL}};
const Segment SGI267171[] = {{&S72961[102], 6}, {NULL}};
const Segment SGI267172[] = {{&S72961[116], 9}, {NULL}};
const Segment SGI267173[] = {{&S73005[13], 4}, {NULL}};
const Segment SGI267174[] = {{&S73005[114], 2}, {NULL}};
const Segment SGI267175[] = {{&S73005[26], 5}, {NULL}};
const Segment SGI267176[] = {{&S73005[59], 6}, {NULL}};
const Segment SGI267177[] = {{&S73005[26], 1}, {NULL}};
const Segment SGI267178[] = {{&S73005[141], 7}, {NULL}};
const Segment SGI267179[] = {{&S73006[13], 4}, {NULL}};
const Segment SGI267180[] = {{&S73006[114], 2}, {NULL}};
const Segment SGI267181[] = {{&S73006[26], 5}, {NULL}};
const Segment SGI267182[] = {{&S73006[59], 6}, {NULL}};
const Segment SGI267183[] = {{&S73006[26], 1}, {NULL}};
const Segment SGI267184[] = {{&S73006[141], 7}, {NULL}};
const Segment SGI267185[] = {{&S73018[27], 6}, {NULL}};
const Segment SGI267186[] = {{&S73018[51], 5}, {NULL}};
const Segment SGI267187[] = {{&S73018[84], 6}, {NULL}};
const Segment SGI267188[] = {{&S73018[118], 6}, {NULL}};
const Segment SGI267189[] = {{&S73018[132], 9}, {NULL}};
const Segment SGI267190[] = {{&S73018[0], 14}, {NULL}};
const Segment SGI267191[] = {{&S73019[27], 6}, {NULL}};
const Segment SGI267192[] = {{&S73019[52], 5}, {NULL}};
const Segment SGI267193[] = {{&S73019[85], 6}, {NULL}};
const Segment SGI267194[] = {{&S73019[119], 6}, {NULL}};
const Segment SGI267195[] = {{&S73019[133], 9}, {NULL}};
const Segment SGI267196[] = {{&S73019[0], 14}, {NULL}};
const Segment SGI267197[] = {{&S73020[27], 6}, {NULL}};
const Segment SGI267198[] = {{&S73020[51], 5}, {NULL}};
const Segment SGI267199[] = {{&S73020[84], 6}, {NULL}};
const Segment SGI267200[] = {{&S73020[118], 6}, {NULL}};
const Segment SGI267201[] = {{&S73020[132], 9}, {NULL}};
const Segment SGI267202[] = {{&S73020[0], 14}, {NULL}};
const Segment SGI267203[] = {{&S73021[27], 6}, {NULL}};
const Segment SGI267204[] = {{&S73021[52], 3}, {NULL}};
const Segment SGI267205[] = {{&S73021[83], 6}, {NULL}};
const Segment SGI267206[] = {{&S73021[117], 6}, {NULL}};
const Segment SGI267207[] = {{&S73021[131], 9}, {NULL}};
const Segment SGI267208[] = {{&S73021[0], 14}, {NULL}};
const Segment SGI267209[] = {{&S73022[27], 6}, {NULL}};
const Segment SGI267210[] = {{&S73022[51], 3}, {NULL}};
const Segment SGI267211[] = {{&S73022[82], 6}, {NULL}};
const Segment SGI267212[] = {{&S73022[116], 6}, {NULL}};
const Segment SGI267213[] = {{&S73022[130], 9}, {NULL}};
const Segment SGI267214[] = {{&S73022[0], 14}, {NULL}};
const Segment SGI267215[] = {{&S65971[56], 9}, {NULL}};
const Segment SGI267216[] = {{&S65971[0], 65}, {NULL}};
const Segment SGI267217[] = {{&S73028[28], 4}, {NULL}};
const Segment SGI267218[] = {{&S73028[41], 3}, {NULL}};
const Segment SGI267219[] = {{&S73028[72], 6}, {NULL}};
const Segment SGI267220[] = {{&S73028[106], 6}, {NULL}};
const Segment SGI267221[] = {{&S73028[120], 9}, {NULL}};
const Segment SGI267222[] = {{&S73028[0], 14}, {NULL}};
const Segment SGI267223[] = {{&S73029[28], 4}, {NULL}};
const Segment SGI267224[] = {{&S73029[41], 5}, {NULL}};
const Segment SGI267225[] = {{&S73029[74], 6}, {NULL}};
const Segment SGI267226[] = {{&S73029[108], 6}, {NULL}};
const Segment SGI267227[] = {{&S73029[122], 9}, {NULL}};
const Segment SGI267228[] = {{&S73029[0], 14}, {NULL}};
const Segment SGI267229[] = {{&S73030[28], 4}, {NULL}};
const Segment SGI267230[] = {{&S73030[41], 3}, {NULL}};
const Segment SGI267231[] = {{&S73030[72], 6}, {NULL}};
const Segment SGI267232[] = {{&S73030[106], 6}, {NULL}};
const Segment SGI267233[] = {{&S73030[120], 9}, {NULL}};
const Segment SGI267234[] = {{&S73030[0], 14}, {NULL}};
const Segment SGI267235[] = {{&S73165[27], 6}, {NULL}};
const Segment SGI267236[] = {{&S73165[52], 5}, {NULL}};
const Segment SGI267237[] = {{&S73165[85], 6}, {NULL}};
const Segment SGI267238[] = {{&S73165[119], 6}, {NULL}};
const Segment SGI267239[] = {{&S73165[133], 9}, {NULL}};
const Segment SGI267240[] = {{&S73165[0], 14}, {NULL}};
const Segment SGI267241[] = {{&S73166[28], 6}, {NULL}};
const Segment SGI267242[] = {{&S73166[53], 5}, {NULL}};
const Segment SGI267243[] = {{&S73166[86], 6}, {NULL}};
const Segment SGI267244[] = {{&S73166[120], 6}, {NULL}};
const Segment SGI267245[] = {{&S73166[134], 9}, {NULL}};
const Segment SGI267246[] = {{&S73166[0], 15}, {NULL}};
const Segment SGI267247[] = {{&S73167[28], 6}, {NULL}};
const Segment SGI267248[] = {{&S73167[52], 5}, {NULL}};
const Segment SGI267249[] = {{&S73167[85], 6}, {NULL}};
const Segment SGI267250[] = {{&S73167[119], 6}, {NULL}};
const Segment SGI267251[] = {{&S73167[133], 9}, {NULL}};
const Segment SGI267252[] = {{&S73167[0], 15}, {NULL}};
const Segment SGI267253[] = {{&S73168[28], 6}, {NULL}};
const Segment SGI267254[] = {{&S73168[53], 3}, {NULL}};
const Segment SGI267255[] = {{&S73168[84], 6}, {NULL}};
const Segment SGI267256[] = {{&S73168[118], 6}, {NULL}};
const Segment SGI267257[] = {{&S73168[132], 9}, {NULL}};
const Segment SGI267258[] = {{&S73168[0], 15}, {NULL}};
const Segment SGI267259[] = {{&S73169[28], 6}, {NULL}};
const Segment SGI267260[] = {{&S73169[52], 5}, {NULL}};
const Segment SGI267261[] = {{&S73169[85], 6}, {NULL}};
const Segment SGI267262[] = {{&S73169[119], 6}, {NULL}};
const Segment SGI267263[] = {{&S73169[133], 9}, {NULL}};
const Segment SGI267264[] = {{&S73169[0], 15}, {NULL}};
const Segment SGI267265[] = {{&S73170[27], 6}, {NULL}};
const Segment SGI267266[] = {{&S73170[51], 5}, {NULL}};
const Segment SGI267267[] = {{&S73170[84], 6}, {NULL}};
const Segment SGI267268[] = {{&S73170[118], 6}, {NULL}};
const Segment SGI267269[] = {{&S73170[132], 9}, {NULL}};
const Segment SGI267270[] = {{&S73170[0], 14}, {NULL}};
const Segment SGI267271[] = {{&S73171[27], 6}, {NULL}};
const Segment SGI267272[] = {{&S73171[51], 5}, {NULL}};
const Segment SGI267273[] = {{&S73171[84], 6}, {NULL}};
const Segment SGI267274[] = {{&S73171[118], 6}, {NULL}};
const Segment SGI267275[] = {{&S73171[132], 9}, {NULL}};
const Segment SGI267276[] = {{&S73171[0], 14}, {NULL}};
const Segment SGI267277[] = {{&S73353[13], 4}, {NULL}};
const Segment SGI267278[] = {{&S73353[29], 5}, {NULL}};
const Segment SGI267279[] = {{&S73353[62], 8}, {NULL}};
const Segment SGI267280[] = {{&S73353[110], 3}, {NULL}};
const Segment SGI267281[] = {{&S73353[121], 9}, {NULL}};
const Segment SGI267282[] = {{&S73354[13], 4}, {NULL}};
const Segment SGI267283[] = {{&S73354[29], 5}, {NULL}};
const Segment SGI267284[] = {{&S73354[62], 8}, {NULL}};
const Segment SGI267285[] = {{&S73354[110], 3}, {NULL}};
const Segment SGI267286[] = {{&S73354[121], 9}, {NULL}};
const Segment SGI267287[] = {{&S73355[13], 4}, {NULL}};
const Segment SGI267288[] = {{&S73355[29], 5}, {NULL}};
const Segment SGI267289[] = {{&S73355[62], 8}, {NULL}};
const Segment SGI267290[] = {{&S73355[110], 3}, {NULL}};
const Segment SGI267291[] = {{&S73355[121], 9}, {NULL}};
const Segment SGI267292[] = {{&S73402[13], 6}, {NULL}};
const Segment SGI267293[] = {{&S73402[136], 5}, {NULL}};
const Segment SGI267294[] = {{&S73402[35], 3}, {NULL}};
const Segment SGI267295[] = {{&S73402[66], 6}, {NULL}};
const Segment SGI267296[] = {{&S73402[35], 1}, {NULL}};
const Segment SGI267297[] = {{&S73403[13], 6}, {NULL}};
const Segment SGI267298[] = {{&S73403[138], 5}, {NULL}};
const Segment SGI267299[] = {{&S73403[35], 5}, {NULL}};
const Segment SGI267300[] = {{&S73403[68], 6}, {NULL}};
const Segment SGI267301[] = {{&S73403[35], 1}, {NULL}};
const Segment SGI267302[] = {{&S73404[13], 6}, {NULL}};
const Segment SGI267303[] = {{&S73404[138], 5}, {NULL}};
const Segment SGI267304[] = {{&S73404[35], 5}, {NULL}};
const Segment SGI267305[] = {{&S73404[68], 6}, {NULL}};
const Segment SGI267306[] = {{&S73404[35], 1}, {NULL}};
const Segment SGI267307[] = {{&S73405[13], 6}, {NULL}};
const Segment SGI267308[] = {{&S73405[138], 5}, {NULL}};
const Segment SGI267309[] = {{&S73405[35], 5}, {NULL}};
const Segment SGI267310[] = {{&S73405[68], 6}, {NULL}};
const Segment SGI267311[] = {{&S73405[35], 1}, {NULL}};
const Segment SGI267312[] = {{&S73406[13], 6}, {NULL}};
const Segment SGI267313[] = {{&S73406[136], 5}, {NULL}};
const Segment SGI267314[] = {{&S73406[35], 3}, {NULL}};
const Segment SGI267315[] = {{&S73406[66], 6}, {NULL}};
const Segment SGI267316[] = {{&S73406[35], 1}, {NULL}};
const Segment SGI267317[] = {{&S73407[13], 6}, {NULL}};
const Segment SGI267318[] = {{&S73407[138], 5}, {NULL}};
const Segment SGI267319[] = {{&S73407[35], 5}, {NULL}};
const Segment SGI267320[] = {{&S73407[68], 6}, {NULL}};
const Segment SGI267321[] = {{&S73407[35], 1}, {NULL}};
const Segment SGI267322[] = {{&S73443[13], 6}, {NULL}};
const Segment SGI267323[] = {{&S73443[35], 3}, {NULL}};
const Segment SGI267324[] = {{&S73443[133], 3}, {NULL}};
const Segment SGI267325[] = {{&S73443[66], 6}, {NULL}};
const Segment SGI267326[] = {{&S73443[100], 6}, {NULL}};
const Segment SGI267327[] = {{&S73443[114], 9}, {NULL}};
const Segment SGI267328[] = {{&S73444[13], 6}, {NULL}};
const Segment SGI267329[] = {{&S73444[35], 3}, {NULL}};
const Segment SGI267330[] = {{&S73444[133], 3}, {NULL}};
const Segment SGI267331[] = {{&S73444[66], 6}, {NULL}};
const Segment SGI267332[] = {{&S73444[100], 6}, {NULL}};
const Segment SGI267333[] = {{&S73444[114], 9}, {NULL}};
const Segment SGI267334[] = {{&S73445[13], 6}, {NULL}};
const Segment SGI267335[] = {{&S73445[35], 5}, {NULL}};
const Segment SGI267336[] = {{&S73445[135], 3}, {NULL}};
const Segment SGI267337[] = {{&S73445[68], 6}, {NULL}};
const Segment SGI267338[] = {{&S73445[102], 6}, {NULL}};
const Segment SGI267339[] = {{&S73445[116], 9}, {NULL}};
const Segment SGI267340[] = {{&S73446[13], 6}, {NULL}};
const Segment SGI267341[] = {{&S73446[35], 5}, {NULL}};
const Segment SGI267342[] = {{&S73446[135], 3}, {NULL}};
const Segment SGI267343[] = {{&S73446[68], 6}, {NULL}};
const Segment SGI267344[] = {{&S73446[102], 6}, {NULL}};
const Segment SGI267345[] = {{&S73446[116], 9}, {NULL}};
const Segment SGI267346[] = {{&S73447[13], 6}, {NULL}};
const Segment SGI267347[] = {{&S73447[35], 3}, {NULL}};
const Segment SGI267348[] = {{&S73447[133], 3}, {NULL}};
const Segment SGI267349[] = {{&S73447[66], 6}, {NULL}};
const Segment SGI267350[] = {{&S73447[100], 6}, {NULL}};
const Segment SGI267351[] = {{&S73447[114], 9}, {NULL}};
const Segment SGI267352[] = {{&S73448[13], 6}, {NULL}};
const Segment SGI267353[] = {{&S73448[35], 5}, {NULL}};
const Segment SGI267354[] = {{&S73448[135], 3}, {NULL}};
const Segment SGI267355[] = {{&S73448[68], 6}, {NULL}};
const Segment SGI267356[] = {{&S73448[102], 6}, {NULL}};
const Segment SGI267357[] = {{&S73448[116], 9}, {NULL}};
const Segment SGI267358[] = {{&S73450[13], 6}, {NULL}};
const Segment SGI267359[] = {{&S73450[105], 3}, {NULL}};
const Segment SGI267360[] = {{&S73450[35], 5}, {NULL}};
const Segment SGI267361[] = {{&S73450[68], 8}, {NULL}};
const Segment SGI267362[] = {{&S73450[116], 6}, {NULL}};
const Segment SGI267363[] = {{&S73450[130], 7}, {NULL}};
const Segment SGI267364[] = {{&S73451[13], 6}, {NULL}};
const Segment SGI267365[] = {{&S73451[105], 3}, {NULL}};
const Segment SGI267366[] = {{&S73451[35], 5}, {NULL}};
const Segment SGI267367[] = {{&S73451[68], 8}, {NULL}};
const Segment SGI267368[] = {{&S73451[116], 6}, {NULL}};
const Segment SGI267369[] = {{&S73451[130], 7}, {NULL}};
const Segment SGI267370[] = {{&S73452[13], 6}, {NULL}};
const Segment SGI267371[] = {{&S73452[105], 3}, {NULL}};
const Segment SGI267372[] = {{&S73452[35], 5}, {NULL}};
const Segment SGI267373[] = {{&S73452[68], 8}, {NULL}};
const Segment SGI267374[] = {{&S73452[116], 6}, {NULL}};
const Segment SGI267375[] = {{&S73452[130], 7}, {NULL}};
const Segment SGI267376[] = {{&S73453[13], 6}, {NULL}};
const Segment SGI267377[] = {{&S73453[103], 3}, {NULL}};
const Segment SGI267378[] = {{&S73453[35], 3}, {NULL}};
const Segment SGI267379[] = {{&S73453[66], 8}, {NULL}};
const Segment SGI267380[] = {{&S73453[114], 6}, {NULL}};
const Segment SGI267381[] = {{&S73453[128], 7}, {NULL}};
const Segment SGI267382[] = {{&S73581[17], 4}, {NULL}};
const Segment SGI267383[] = {{&S73581[36], 5}, {NULL}};
const Segment SGI267384[] = {{&S73581[66], 3}, {NULL}};
const Segment SGI267385[] = {{&S73582[17], 4}, {NULL}};
const Segment SGI267386[] = {{&S73582[36], 3}, {NULL}};
const Segment SGI267387[] = {{&S73582[64], 3}, {NULL}};
const Segment SGI267388[] = {{&S73589[13], 4}, {NULL}};
const Segment SGI267389[] = {{&S73589[26], 5}, {NULL}};
const Segment SGI267390[] = {{&S73589[126], 3}, {NULL}};
const Segment SGI267391[] = {{&S73589[59], 6}, {NULL}};
const Segment SGI267392[] = {{&S73589[93], 6}, {NULL}};
const Segment SGI267393[] = {{&S73589[107], 9}, {NULL}};
const Segment SGI267394[] = {{&S73590[13], 4}, {NULL}};
const Segment SGI267395[] = {{&S73590[26], 5}, {NULL}};
const Segment SGI267396[] = {{&S73590[126], 5}, {NULL}};
const Segment SGI267397[] = {{&S73590[59], 6}, {NULL}};
const Segment SGI267398[] = {{&S73590[93], 6}, {NULL}};
const Segment SGI267399[] = {{&S73590[107], 9}, {NULL}};
const Segment SGI267400[] = {{&S73591[13], 4}, {NULL}};
const Segment SGI267401[] = {{&S73591[26], 5}, {NULL}};
const Segment SGI267402[] = {{&S73591[126], 5}, {NULL}};
const Segment SGI267403[] = {{&S73591[59], 6}, {NULL}};
const Segment SGI267404[] = {{&S73591[93], 6}, {NULL}};
const Segment SGI267405[] = {{&S73591[107], 9}, {NULL}};
const Segment SGI267406[] = {{&S73592[13], 4}, {NULL}};
const Segment SGI267407[] = {{&S73592[26], 5}, {NULL}};
const Segment SGI267408[] = {{&S73592[126], 3}, {NULL}};
const Segment SGI267409[] = {{&S73592[59], 6}, {NULL}};
const Segment SGI267410[] = {{&S73592[93], 6}, {NULL}};
const Segment SGI267411[] = {{&S73592[107], 9}, {NULL}};
const Segment SGI267412[] = {{&S73593[13], 4}, {NULL}};
const Segment SGI267413[] = {{&S73593[26], 5}, {NULL}};
const Segment SGI267414[] = {{&S73593[126], 5}, {NULL}};
const Segment SGI267415[] = {{&S73593[59], 6}, {NULL}};
const Segment SGI267416[] = {{&S73593[93], 6}, {NULL}};
const Segment SGI267417[] = {{&S73593[107], 9}, {NULL}};
const Segment SGI267418[] = {{&S73638[27], 6}, {NULL}};
const Segment SGI267419[] = {{&S73638[52], 5}, {NULL}};
const Segment SGI267420[] = {{&S73638[85], 6}, {NULL}};
const Segment SGI267421[] = {{&S73638[119], 5}, {NULL}};
const Segment SGI267422[] = {{&S73638[132], 9}, {NULL}};
const Segment SGI267423[] = {{&S73638[0], 14}, {NULL}};
const Segment SGI267424[] = {{&S73639[27], 6}, {NULL}};
const Segment SGI267425[] = {{&S73639[51], 5}, {NULL}};
const Segment SGI267426[] = {{&S73639[84], 6}, {NULL}};
const Segment SGI267427[] = {{&S73639[118], 5}, {NULL}};
const Segment SGI267428[] = {{&S73639[131], 9}, {NULL}};
const Segment SGI267429[] = {{&S73639[0], 14}, {NULL}};
const Segment SGI267430[] = {{&S73640[27], 6}, {NULL}};
const Segment SGI267431[] = {{&S73640[52], 3}, {NULL}};
const Segment SGI267432[] = {{&S73640[83], 6}, {NULL}};
const Segment SGI267433[] = {{&S73640[117], 5}, {NULL}};
const Segment SGI267434[] = {{&S73640[130], 9}, {NULL}};
const Segment SGI267435[] = {{&S73640[0], 14}, {NULL}};
const Segment SGI267436[] = {{&S73642[27], 6}, {NULL}};
const Segment SGI267437[] = {{&S73642[51], 3}, {NULL}};
const Segment SGI267438[] = {{&S73642[82], 6}, {NULL}};
const Segment SGI267439[] = {{&S73642[116], 5}, {NULL}};
const Segment SGI267440[] = {{&S73642[129], 9}, {NULL}};
const Segment SGI267441[] = {{&S73642[0], 14}, {NULL}};
const Segment SGI267442[] = {{&S59864[20], 27}, {NULL}};
const Segment SGI267443[] = {{&S59865[20], 27}, {NULL}};
const Segment SGI267444[] = {{&S59866[20], 27}, {NULL}};
const Segment SGI267445[] = {{&S59867[20], 27}, {NULL}};
const Segment SGI267446[] = {{&S59868[20], 27}, {NULL}};
const Segment SGI267447[] = {{&S59869[20], 27}, {NULL}};
const Segment SGI267448[] = {{&S59870[20], 27}, {NULL}};
const Segment SGI267449[] = {{&S59871[20], 27}, {NULL}};
const Segment SGI267450[] = {{&S59872[20], 27}, {NULL}};
const Segment SGI267451[] = {{&S59873[20], 27}, {NULL}};
const Segment SGI267452[] = {{&S59874[20], 27}, {NULL}};
const Segment SGI267453[] = {{&S59875[20], 27}, {NULL}};
const Segment SGI267454[] = {{&S59876[20], 27}, {NULL}};
const Segment SGI267455[] = {{&S59877[20], 27}, {NULL}};
const Segment SGI267456[] = {{&S59878[20], 27}, {NULL}};
const Segment SGI267457[] = {{&S73693[13], 4}, {&S73693[13], 4}, {NULL}};
const Segment SGI267458[] = {{&S73693[116], 3}, {NULL}};
const Segment SGI267459[] = {{&S73693[29], 3}, {NULL}};
const Segment SGI267460[] = {{&S73693[81], 6}, {NULL}};
const Segment SGI267461[] = {{&S73693[127], 5}, {NULL}};
const Segment SGI267462[] = {{&S73693[140], 9}, {NULL}};
const Segment SGI267463[] = {{&S73693[150], 13}, {NULL}};
const Segment SGI267464[] = {{&S60198[10], 4}, {NULL}};
const Segment SGI267465[] = {{&S60198[15], 5}, {NULL}};
const Segment SGI267466[] = {{&S73975[13], 4}, {NULL}};
const Segment SGI267467[] = {{&S73975[106], 5}, {NULL}};
const Segment SGI267468[] = {{&S73975[29], 5}, {NULL}};
const Segment SGI267469[] = {{&S73975[69], 8}, {NULL}};
const Segment SGI267470[] = {{&S73975[119], 3}, {NULL}};
const Segment SGI267471[] = {{&S73975[130], 9}, {NULL}};
const Segment SGI267472[] = {{&S73976[13], 4}, {NULL}};
const Segment SGI267473[] = {{&S73976[106], 5}, {NULL}};
const Segment SGI267474[] = {{&S73976[29], 5}, {NULL}};
const Segment SGI267475[] = {{&S73976[69], 8}, {NULL}};
const Segment SGI267476[] = {{&S73976[119], 3}, {NULL}};
const Segment SGI267477[] = {{&S73976[130], 9}, {NULL}};
const Segment SGI267478[] = {{&S73977[13], 4}, {NULL}};
const Segment SGI267479[] = {{&S73977[106], 5}, {NULL}};
const Segment SGI267480[] = {{&S73977[29], 5}, {NULL}};
const Segment SGI267481[] = {{&S73977[69], 8}, {NULL}};
const Segment SGI267482[] = {{&S73977[119], 3}, {NULL}};
const Segment SGI267483[] = {{&S73977[130], 9}, {NULL}};
const Segment SGI267484[] = {{&S73978[13], 4}, {NULL}};
const Segment SGI267485[] = {{&S73978[111], 3}, {NULL}};
const Segment SGI267486[] = {{&S73978[36], 5}, {NULL}};
const Segment SGI267487[] = {{&S73978[76], 6}, {NULL}};
const Segment SGI267488[] = {{&S73978[122], 5}, {NULL}};
const Segment SGI267489[] = {{&S73978[135], 6}, {NULL}};
const Segment SGI267490[] = {{&S73979[13], 4}, {NULL}};
const Segment SGI267491[] = {{&S73979[111], 3}, {NULL}};
const Segment SGI267492[] = {{&S73979[36], 5}, {NULL}};
const Segment SGI267493[] = {{&S73979[76], 6}, {NULL}};
const Segment SGI267494[] = {{&S73979[122], 5}, {NULL}};
const Segment SGI267495[] = {{&S73979[135], 6}, {NULL}};
const Segment SGI267496[] = {{&S73980[13], 4}, {&S73980[13], 4}, {NULL}};
const Segment SGI267497[] = {{&S73980[214], 5}, {NULL}};
const Segment SGI267498[] = {{&S73980[29], 5}, {&S73980[138], 3}, {NULL}};
const Segment SGI267499[] = {{&S73980[69], 8}, {&S73980[176], 9}, {NULL}};
const Segment SGI267500[] = {{&S73980[105], 3}, {&S73980[227], 5}, {NULL}};
const Segment SGI267501[] = {{&S73980[176], 9}, {NULL}};
const Segment SGI267502[] = {{&S73981[13], 4}, {NULL}};
const Segment SGI267503[] = {{&S73981[109], 3}, {NULL}};
const Segment SGI267504[] = {{&S73981[36], 3}, {NULL}};
const Segment SGI267505[] = {{&S73981[74], 6}, {NULL}};
const Segment SGI267506[] = {{&S73981[120], 5}, {NULL}};
const Segment SGI267507[] = {{&S73981[133], 6}, {NULL}};
const Segment SGI267508[] = {{&S73982[13], 4}, {NULL}};
const Segment SGI267509[] = {{&S73982[111], 3}, {NULL}};
const Segment SGI267510[] = {{&S73982[36], 5}, {NULL}};
const Segment SGI267511[] = {{&S73982[76], 6}, {NULL}};
const Segment SGI267512[] = {{&S73982[122], 5}, {NULL}};
const Segment SGI267513[] = {{&S73982[135], 6}, {NULL}};
const Segment SGI267514[] = {{&S73983[13], 4}, {NULL}};
const Segment SGI267515[] = {{&S73983[111], 3}, {NULL}};
const Segment SGI267516[] = {{&S73983[36], 5}, {NULL}};
const Segment SGI267517[] = {{&S73983[76], 6}, {NULL}};
const Segment SGI267518[] = {{&S73983[122], 5}, {NULL}};
const Segment SGI267519[] = {{&S73983[135], 6}, {NULL}};
const Segment SGI267520[] = {{&S66889[47], 10}, {&S66889[58], 9}, {NULL}};
const Segment SGI267521[] = {{&S66889[0], 109}, {NULL}};
const Segment SGI267522[] = {{&S74217[13], 4}, {NULL}};
const Segment SGI267523[] = {{&S74217[26], 3}, {NULL}};
const Segment SGI267524[] = {{&S74217[57], 6}, {NULL}};
const Segment SGI267525[] = {{&S74217[109], 6}, {NULL}};
const Segment SGI267526[] = {{&S74217[123], 9}, {NULL}};
const Segment SGI267527[] = {{&S74218[13], 4}, {NULL}};
const Segment SGI267528[] = {{&S74218[26], 5}, {NULL}};
const Segment SGI267529[] = {{&S74218[59], 6}, {NULL}};
const Segment SGI267530[] = {{&S74218[111], 6}, {NULL}};
const Segment SGI267531[] = {{&S74218[125], 9}, {NULL}};
const Segment SGI267532[] = {{&S74219[13], 4}, {NULL}};
const Segment SGI267533[] = {{&S74219[26], 5}, {NULL}};
const Segment SGI267534[] = {{&S74219[59], 6}, {NULL}};
const Segment SGI267535[] = {{&S74219[111], 6}, {NULL}};
const Segment SGI267536[] = {{&S74219[125], 9}, {NULL}};
const Segment SGI267537[] = {{&S74220[13], 4}, {NULL}};
const Segment SGI267538[] = {{&S74220[26], 5}, {NULL}};
const Segment SGI267539[] = {{&S74220[59], 6}, {NULL}};
const Segment SGI267540[] = {{&S74220[111], 6}, {NULL}};
const Segment SGI267541[] = {{&S74220[125], 9}, {NULL}};
const Segment SGI267542[] = {{&S74360[27], 6}, {NULL}};
const Segment SGI267543[] = {{&S74360[52], 5}, {NULL}};
const Segment SGI267544[] = {{&S74360[85], 6}, {NULL}};
const Segment SGI267545[] = {{&S74360[119], 5}, {NULL}};
const Segment SGI267546[] = {{&S74360[132], 9}, {NULL}};
const Segment SGI267547[] = {{&S74360[0], 14}, {NULL}};
const Segment SGI267548[] = {{&S74361[28], 6}, {NULL}};
const Segment SGI267549[] = {{&S74361[53], 5}, {NULL}};
const Segment SGI267550[] = {{&S74361[86], 6}, {NULL}};
const Segment SGI267551[] = {{&S74361[120], 5}, {NULL}};
const Segment SGI267552[] = {{&S74361[133], 9}, {NULL}};
const Segment SGI267553[] = {{&S74361[0], 15}, {NULL}};
const Segment SGI267554[] = {{&S74362[27], 6}, {NULL}};
const Segment SGI267555[] = {{&S74362[51], 5}, {NULL}};
const Segment SGI267556[] = {{&S74362[84], 6}, {NULL}};
const Segment SGI267557[] = {{&S74362[118], 5}, {NULL}};
const Segment SGI267558[] = {{&S74362[131], 9}, {NULL}};
const Segment SGI267559[] = {{&S74362[0], 14}, {NULL}};
const Segment SGI267560[] = {{&S74363[28], 6}, {NULL}};
const Segment SGI267561[] = {{&S74363[53], 3}, {NULL}};
const Segment SGI267562[] = {{&S74363[84], 6}, {NULL}};
const Segment SGI267563[] = {{&S74363[118], 5}, {NULL}};
const Segment SGI267564[] = {{&S74363[131], 9}, {NULL}};
const Segment SGI267565[] = {{&S74363[0], 15}, {NULL}};
const Segment SGI267566[] = {{&S60503[72], 4}, {NULL}};
const Segment SGI267567[] = {{&S60503[77], 5}, {NULL}};
const Segment SGI267568[] = {{&S60504[10], 4}, {NULL}};
const Segment SGI267569[] = {{&S60504[15], 5}, {NULL}};
const Segment SGI267570[] = {{&S60505[72], 4}, {NULL}};
const Segment SGI267571[] = {{&S60505[77], 5}, {NULL}};
const Segment SGI267572[] = {{&S74410[28], 4}, {NULL}};
const Segment SGI267573[] = {{&S74410[41], 5}, {NULL}};
const Segment SGI267574[] = {{&S74410[74], 6}, {NULL}};
const Segment SGI267575[] = {{&S74410[108], 5}, {NULL}};
const Segment SGI267576[] = {{&S74410[121], 9}, {NULL}};
const Segment SGI267577[] = {{&S74410[0], 14}, {NULL}};
const Segment SGI267578[] = {{&S74411[28], 4}, {NULL}};
const Segment SGI267579[] = {{&S74411[41], 3}, {NULL}};
const Segment SGI267580[] = {{&S74411[72], 6}, {NULL}};
const Segment SGI267581[] = {{&S74411[106], 5}, {NULL}};
const Segment SGI267582[] = {{&S74411[119], 9}, {NULL}};
const Segment SGI267583[] = {{&S74411[0], 14}, {NULL}};
const Segment SGI267584[] = {{&S67345[41], 11}, {NULL}};
const Segment SGI267585[] = {{&S67345[0], 170}, {NULL}};
const Segment SGI267586[] = {{&S67346[41], 11}, {&S67346[53], 50}, {NULL}};
const Segment SGI267587[] = {{&S67346[0], 174}, {NULL}};
const Segment SGI267588[] = {{&S60673[20], 26}, {NULL}};
const Segment SGI267589[] = {{&S60674[20], 27}, {NULL}};
const Segment SGI267590[] = {{&S60675[20], 26}, {NULL}};
const Segment SGI267591[] = {{&S60676[20], 26}, {NULL}};
const Segment SGI267592[] = {{&S60677[20], 27}, {NULL}};
const Segment SGI267593[] = {{&S60678[20], 26}, {NULL}};
const Segment SGI267594[] = {{&S60679[20], 27}, {NULL}};
const Segment SGI267595[] = {{&S60680[20], 26}, {NULL}};
const Segment SGI267596[] = {{&S60681[20], 26}, {NULL}};
const Segment SGI267597[] = {{&S60682[20], 26}, {NULL}};
const Segment SGI267598[] = {{&S60683[20], 27}, {NULL}};
const Segment SGI267599[] = {{&S60684[20], 27}, {NULL}};
const Segment SGI267600[] = {{&S60685[20], 27}, {NULL}};
const Segment SGI267601[] = {{&S60686[20], 27}, {NULL}};
const Segment SGI267602[] = {{&S74633[13], 4}, {&S74633[13], 4}, {NULL}};
const Segment SGI267603[] = {{&S74633[114], 3}, {NULL}};
const Segment SGI267604[] = {{&S74633[29], 1}, {NULL}};
const Segment SGI267605[] = {{&S74633[79], 6}, {NULL}};
const Segment SGI267606[] = {{&S74633[125], 5}, {NULL}};
const Segment SGI267607[] = {{&S74633[138], 9}, {NULL}};
const Segment SGI267608[] = {{&S74633[148], 13}, {NULL}};
const Segment SGI267609[] = {{&S67604[66], 5}, {NULL}};
const Segment SGI267610[] = {{&S67604[0], 71}, {NULL}};
const Segment SGI267611[] = {{&S67605[59], 5}, {NULL}};
const Segment SGI267612[] = {{&S67605[0], 64}, {NULL}};
const Segment SGI267613[] = {{&S67606[66], 5}, {&S67606[66], 5}, {NULL}};
const Segment SGI267614[] = {{&S67606[0], 77}, {NULL}};
const Segment SGI267615[] = {{&S74740[13], 6}, {NULL}};
const Segment SGI267616[] = {{&S74740[35], 5}, {NULL}};
const Segment SGI267617[] = {{&S74740[68], 6}, {NULL}};
const Segment SGI267618[] = {{&S74740[126], 6}, {NULL}};
const Segment SGI267619[] = {{&S74740[140], 7}, {NULL}};
const Segment SGI267620[] = {{&S74741[13], 6}, {NULL}};
const Segment SGI267621[] = {{&S74741[35], 5}, {NULL}};
const Segment SGI267622[] = {{&S74741[68], 6}, {NULL}};
const Segment SGI267623[] = {{&S74741[126], 6}, {NULL}};
const Segment SGI267624[] = {{&S74741[140], 7}, {NULL}};
const Segment SGI267625[] = {{&S74834[13], 6}, {NULL}};
const Segment SGI267626[] = {{&S74834[133], 3}, {NULL}};
const Segment SGI267627[] = {{&S74834[38], 5}, {NULL}};
const Segment SGI267628[] = {{&S74834[78], 8}, {NULL}};
const Segment SGI267629[] = {{&S74834[114], 3}, {NULL}};
const Segment SGI267630[] = {{&S74835[13], 6}, {NULL}};
const Segment SGI267631[] = {{&S74835[133], 3}, {NULL}};
const Segment SGI267632[] = {{&S74835[38], 5}, {NULL}};
const Segment SGI267633[] = {{&S74835[78], 8}, {NULL}};
const Segment SGI267634[] = {{&S74835[114], 3}, {NULL}};
const Segment SGI267635[] = {{&S61209[20], 27}, {NULL}};
const Segment SGI267636[] = {{&S61210[20], 27}, {NULL}};
const Segment SGI267637[] = {{&S61211[20], 27}, {NULL}};
const Segment SGI267638[] = {{&S61212[20], 27}, {NULL}};
const Segment SGI267639[] = {{&S61213[20], 27}, {NULL}};
const Segment SGI267640[] = {{&S61214[20], 27}, {NULL}};
const Segment SGI267641[] = {{&S67899[66], 5}, {NULL}};
const Segment SGI267642[] = {{&S67899[0], 71}, {NULL}};
const Segment SGI267643[] = {{&S61215[20], 27}, {NULL}};
const Segment SGI267644[] = {{&S67900[59], 5}, {NULL}};
const Segment SGI267645[] = {{&S67900[0], 64}, {NULL}};
const Segment SGI267646[] = {{&S61216[20], 27}, {NULL}};
const Segment SGI267647[] = {{&S61217[20], 27}, {NULL}};
const Segment SGI267648[] = {{&S67901[66], 5}, {&S67901[66], 5}, {NULL}};
const Segment SGI267649[] = {{&S67901[0], 77}, {NULL}};
const Segment SGI267650[] = {{&S61218[20], 27}, {NULL}};
const Segment SGI267651[] = {{&S61219[20], 27}, {NULL}};
const Segment SGI267652[] = {{&S61220[20], 27}, {NULL}};
const Segment SGI267653[] = {{&S61221[20], 27}, {NULL}};
const Segment SGI267654[] = {{&S61222[20], 27}, {NULL}};
const Segment SGI267655[] = {{&S61223[20], 26}, {NULL}};
const Segment SGI267656[] = {{&S61224[20], 27}, {NULL}};
const Segment SGI267657[] = {{&S74836[13], 6}, {NULL}};
const Segment SGI267658[] = {{&S74836[133], 3}, {NULL}};
const Segment SGI267659[] = {{&S74836[38], 5}, {NULL}};
const Segment SGI267660[] = {{&S74836[78], 8}, {NULL}};
const Segment SGI267661[] = {{&S74836[114], 3}, {NULL}};
const Segment SGI267662[] = {{&S74837[13], 6}, {NULL}};
const Segment SGI267663[] = {{&S74837[133], 3}, {NULL}};
const Segment SGI267664[] = {{&S74837[38], 3}, {NULL}};
const Segment SGI267665[] = {{&S74837[76], 8}, {NULL}};
const Segment SGI267666[] = {{&S74837[112], 5}, {NULL}};
const Segment SGI267667[] = {{&S74838[13], 6}, {NULL}};
const Segment SGI267668[] = {{&S74838[111], 3}, {NULL}};
const Segment SGI267669[] = {{&S74838[38], 3}, {NULL}};
const Segment SGI267670[] = {{&S74838[76], 6}, {NULL}};
const Segment SGI267671[] = {{&S74838[122], 5}, {NULL}};
const Segment SGI267672[] = {{&S74838[135], 9}, {NULL}};
const Segment SGI267673[] = {{&S74839[13], 6}, {NULL}};
const Segment SGI267674[] = {{&S74839[133], 3}, {NULL}};
const Segment SGI267675[] = {{&S74839[38], 5}, {NULL}};
const Segment SGI267676[] = {{&S74839[78], 8}, {NULL}};
const Segment SGI267677[] = {{&S74839[114], 3}, {NULL}};
const Segment SGI267678[] = {{&S74847[27], 6}, {NULL}};
const Segment SGI267679[] = {{&S74847[52], 5}, {NULL}};
const Segment SGI267680[] = {{&S74847[85], 6}, {NULL}};
const Segment SGI267681[] = {{&S74847[119], 5}, {NULL}};
const Segment SGI267682[] = {{&S74847[132], 9}, {NULL}};
const Segment SGI267683[] = {{&S74847[0], 14}, {NULL}};
const Segment SGI267684[] = {{&S74848[27], 6}, {NULL}};
const Segment SGI267685[] = {{&S74848[51], 5}, {NULL}};
const Segment SGI267686[] = {{&S74848[84], 6}, {NULL}};
const Segment SGI267687[] = {{&S74848[118], 5}, {NULL}};
const Segment SGI267688[] = {{&S74848[131], 9}, {NULL}};
const Segment SGI267689[] = {{&S74848[0], 14}, {NULL}};
const Segment SGI267690[] = {{&S74849[27], 6}, {NULL}};
const Segment SGI267691[] = {{&S74849[51], 3}, {NULL}};
const Segment SGI267692[] = {{&S74849[82], 6}, {NULL}};
const Segment SGI267693[] = {{&S74849[116], 5}, {NULL}};
const Segment SGI267694[] = {{&S74849[129], 9}, {NULL}};
const Segment SGI267695[] = {{&S74849[0], 14}, {NULL}};
const Segment SGI267696[] = {{&S74850[27], 6}, {NULL}};
const Segment SGI267697[] = {{&S74850[52], 3}, {NULL}};
const Segment SGI267698[] = {{&S74850[83], 6}, {NULL}};
const Segment SGI267699[] = {{&S74850[117], 5}, {NULL}};
const Segment SGI267700[] = {{&S74850[130], 9}, {NULL}};
const Segment SGI267701[] = {{&S74850[0], 14}, {NULL}};
const Segment SGI267702[] = {{&S74960[13], 6}, {NULL}};
const Segment SGI267703[] = {{&S74960[35], 5}, {NULL}};
const Segment SGI267704[] = {{&S74960[75], 6}, {NULL}};
const Segment SGI267705[] = {{&S74960[109], 6}, {NULL}};
const Segment SGI267706[] = {{&S74960[116], 13}, {NULL}};
const Segment SGI267707[] = {{&S74961[13], 6}, {NULL}};
const Segment SGI267708[] = {{&S74961[35], 3}, {NULL}};
const Segment SGI267709[] = {{&S74961[73], 6}, {NULL}};
const Segment SGI267710[] = {{&S74961[107], 6}, {NULL}};
const Segment SGI267711[] = {{&S74961[114], 13}, {NULL}};
const Segment SGI267712[] = {{&S74962[13], 6}, {NULL}};
const Segment SGI267713[] = {{&S74962[35], 3}, {NULL}};
const Segment SGI267714[] = {{&S74962[73], 6}, {NULL}};
const Segment SGI267715[] = {{&S74962[107], 6}, {NULL}};
const Segment SGI267716[] = {{&S74962[114], 13}, {NULL}};
const Segment SGI267717[] = {{&S74963[13], 6}, {NULL}};
const Segment SGI267718[] = {{&S74963[35], 3}, {NULL}};
const Segment SGI267719[] = {{&S74963[73], 6}, {NULL}};
const Segment SGI267720[] = {{&S74963[107], 6}, {NULL}};
const Segment SGI267721[] = {{&S74963[114], 13}, {NULL}};
const Segment SGI267722[] = {{&S74964[13], 6}, {NULL}};
const Segment SGI267723[] = {{&S74964[35], 5}, {NULL}};
const Segment SGI267724[] = {{&S74964[75], 6}, {NULL}};
const Segment SGI267725[] = {{&S74964[109], 6}, {NULL}};
const Segment SGI267726[] = {{&S74964[116], 13}, {NULL}};
const Segment SGI267727[] = {{&S74965[13], 6}, {NULL}};
const Segment SGI267728[] = {{&S74965[35], 5}, {NULL}};
const Segment SGI267729[] = {{&S74965[75], 6}, {NULL}};
const Segment SGI267730[] = {{&S74965[109], 6}, {NULL}};
const Segment SGI267731[] = {{&S74965[116], 13}, {NULL}};
const Segment SGI267732[] = {{&S74979[13], 6}, {&S74979[13], 6}, {NULL}};
const Segment SGI267733[] = {{&S74979[198], 3}, {NULL}};
const Segment SGI267734[] = {{&S74979[35], 3}, {NULL}};
const Segment SGI267735[] = {{&S74979[66], 6}, {&S74979[165], 3}, {NULL}};
const Segment SGI267736[] = {{&S74979[100], 5}, {&S74979[209], 3}, {NULL}};
const Segment SGI267737[] = {{&S74979[220], 5}, {NULL}};
const Segment SGI267738[] = {{&S74980[13], 6}, {&S74980[13], 6}, {NULL}};
const Segment SGI267739[] = {{&S74980[200], 3}, {NULL}};
const Segment SGI267740[] = {{&S74980[35], 5}, {NULL}};
const Segment SGI267741[] = {{&S74980[68], 6}, {&S74980[167], 3}, {NULL}};
const Segment SGI267742[] = {{&S74980[102], 5}, {&S74980[211], 3}, {NULL}};
const Segment SGI267743[] = {{&S74980[222], 5}, {NULL}};
const Segment SGI267744[] = {{&S74981[13], 6}, {&S74981[13], 6}, {NULL}};
const Segment SGI267745[] = {{&S74981[198], 3}, {NULL}};
const Segment SGI267746[] = {{&S74981[35], 3}, {NULL}};
const Segment SGI267747[] = {{&S74981[66], 6}, {&S74981[165], 3}, {NULL}};
const Segment SGI267748[] = {{&S74981[100], 5}, {&S74981[209], 3}, {NULL}};
const Segment SGI267749[] = {{&S74981[220], 5}, {NULL}};
const Segment SGI267750[] = {{&S74982[13], 6}, {&S74982[13], 6}, {NULL}};
const Segment SGI267751[] = {{&S74982[200], 3}, {NULL}};
const Segment SGI267752[] = {{&S74982[35], 5}, {NULL}};
const Segment SGI267753[] = {{&S74982[68], 6}, {&S74982[167], 3}, {NULL}};
const Segment SGI267754[] = {{&S74982[102], 5}, {&S74982[211], 3}, {NULL}};
const Segment SGI267755[] = {{&S74982[222], 5}, {NULL}};
const Segment SGI267756[] = {{&S75132[28], 4}, {NULL}};
const Segment SGI267757[] = {{&S75132[44], 5}, {NULL}};
const Segment SGI267758[] = {{&S75148[13], 6}, {NULL}};
const Segment SGI267759[] = {{&S75148[35], 5}, {NULL}};
const Segment SGI267760[] = {{&S75148[68], 6}, {NULL}};
const Segment SGI267761[] = {{&S75148[114], 5}, {NULL}};
const Segment SGI267762[] = {{&S75148[127], 9}, {NULL}};
const Segment SGI267763[] = {{&S75149[13], 6}, {NULL}};
const Segment SGI267764[] = {{&S75149[35], 3}, {NULL}};
const Segment SGI267765[] = {{&S75149[66], 6}, {NULL}};
const Segment SGI267766[] = {{&S75149[112], 5}, {NULL}};
const Segment SGI267767[] = {{&S75149[125], 9}, {NULL}};
const Segment SGI267768[] = {{&S75158[13], 6}, {NULL}};
const Segment SGI267769[] = {{&S75158[115], 5}, {NULL}};
const Segment SGI267770[] = {{&S75158[38], 5}, {NULL}};
const Segment SGI267771[] = {{&S75158[78], 8}, {NULL}};
const Segment SGI267772[] = {{&S75158[128], 3}, {NULL}};
const Segment SGI267773[] = {{&S75158[139], 9}, {NULL}};
const Segment SGI267774[] = {{&S75158[149], 13}, {NULL}};
const Segment SGI267775[] = {{&S75319[13], 4}, {NULL}};
const Segment SGI267776[] = {{&S75319[36], 5}, {NULL}};
const Segment SGI267777[] = {{&S75319[76], 6}, {NULL}};
const Segment SGI267778[] = {{&S75319[110], 4}, {NULL}};
const Segment SGI267779[] = {{&S75319[115], 7}, {NULL}};
const Segment SGI267780[] = {{&S75320[13], 4}, {NULL}};
const Segment SGI267781[] = {{&S75320[36], 5}, {NULL}};
const Segment SGI267782[] = {{&S75320[76], 6}, {NULL}};
const Segment SGI267783[] = {{&S75320[110], 4}, {NULL}};
const Segment SGI267784[] = {{&S75320[115], 7}, {NULL}};
const Segment SGI267785[] = {{&S75321[13], 4}, {NULL}};
const Segment SGI267786[] = {{&S75321[36], 5}, {NULL}};
const Segment SGI267787[] = {{&S75321[76], 6}, {NULL}};
const Segment SGI267788[] = {{&S75321[110], 5}, {NULL}};
const Segment SGI267789[] = {{&S75321[116], 7}, {NULL}};
const Segment SGI267790[] = {{&S75368[13], 4}, {NULL}};
const Segment SGI267791[] = {{&S75368[26], 5}, {NULL}};
const Segment SGI267792[] = {{&S75368[59], 6}, {NULL}};
const Segment SGI267793[] = {{&S75368[110], 6}, {NULL}};
const Segment SGI267794[] = {{&S75368[124], 7}, {NULL}};
const Segment SGI267795[] = {{&S75486[13], 4}, {NULL}};
const Segment SGI267796[] = {{&S75486[26], 3}, {NULL}};
const Segment SGI267797[] = {{&S75486[124], 5}, {NULL}};
const Segment SGI267798[] = {{&S75486[57], 6}, {NULL}};
const Segment SGI267799[] = {{&S75486[91], 6}, {NULL}};
const Segment SGI267800[] = {{&S75486[105], 9}, {NULL}};
const Segment SGI267801[] = {{&S75487[13], 4}, {NULL}};
const Segment SGI267802[] = {{&S75487[26], 3}, {NULL}};
const Segment SGI267803[] = {{&S75487[124], 5}, {NULL}};
const Segment SGI267804[] = {{&S75487[57], 6}, {NULL}};
const Segment SGI267805[] = {{&S75487[91], 6}, {NULL}};
const Segment SGI267806[] = {{&S75487[105], 9}, {NULL}};
const Segment SGI267807[] = {{&S75488[13], 4}, {NULL}};
const Segment SGI267808[] = {{&S75488[26], 5}, {NULL}};
const Segment SGI267809[] = {{&S75488[126], 5}, {NULL}};
const Segment SGI267810[] = {{&S75488[59], 6}, {NULL}};
const Segment SGI267811[] = {{&S75488[93], 6}, {NULL}};
const Segment SGI267812[] = {{&S75488[107], 9}, {NULL}};
const Segment SGI267813[] = {{&S75489[13], 4}, {NULL}};
const Segment SGI267814[] = {{&S75489[26], 3}, {NULL}};
const Segment SGI267815[] = {{&S75489[124], 5}, {NULL}};
const Segment SGI267816[] = {{&S75489[57], 6}, {NULL}};
const Segment SGI267817[] = {{&S75489[91], 6}, {NULL}};
const Segment SGI267818[] = {{&S75489[105], 9}, {NULL}};
const Segment SGI267819[] = {{&S75490[13], 4}, {NULL}};
const Segment SGI267820[] = {{&S75490[26], 5}, {NULL}};
const Segment SGI267821[] = {{&S75490[126], 5}, {NULL}};
const Segment SGI267822[] = {{&S75490[59], 6}, {NULL}};
const Segment SGI267823[] = {{&S75490[93], 6}, {NULL}};
const Segment SGI267824[] = {{&S75490[107], 9}, {NULL}};
const Segment SGI267825[] = {{&S75491[13], 4}, {NULL}};
const Segment SGI267826[] = {{&S75491[26], 5}, {NULL}};
const Segment SGI267827[] = {{&S75491[126], 5}, {NULL}};
const Segment SGI267828[] = {{&S75491[59], 6}, {NULL}};
const Segment SGI267829[] = {{&S75491[93], 6}, {NULL}};
const Segment SGI267830[] = {{&S75491[107], 9}, {NULL}};
const Segment SGI267831[] = {{&S61664[22], 12}, {NULL}};
const Segment SGI267832[] = {{&S75666[13], 6}, {NULL}};
const Segment SGI267833[] = {{&S75666[35], 5}, {NULL}};
const Segment SGI267834[] = {{&S75666[135], 3}, {NULL}};
const Segment SGI267835[] = {{&S75666[68], 6}, {NULL}};
const Segment SGI267836[] = {{&S75666[102], 6}, {NULL}};
const Segment SGI267837[] = {{&S75666[116], 9}, {NULL}};
const Segment SGI267838[] = {{&S75667[13], 6}, {NULL}};
const Segment SGI267839[] = {{&S75667[35], 5}, {NULL}};
const Segment SGI267840[] = {{&S75667[135], 3}, {NULL}};
const Segment SGI267841[] = {{&S75667[68], 6}, {NULL}};
const Segment SGI267842[] = {{&S75667[102], 6}, {NULL}};
const Segment SGI267843[] = {{&S75667[116], 9}, {NULL}};
const Segment SGI267844[] = {{&S75668[13], 6}, {NULL}};
const Segment SGI267845[] = {{&S75668[35], 5}, {NULL}};
const Segment SGI267846[] = {{&S75668[135], 3}, {NULL}};
const Segment SGI267847[] = {{&S75668[68], 6}, {NULL}};
const Segment SGI267848[] = {{&S75668[102], 6}, {NULL}};
const Segment SGI267849[] = {{&S75668[116], 9}, {NULL}};
const Segment SGI267850[] = {{&S75669[13], 6}, {NULL}};
const Segment SGI267851[] = {{&S75669[35], 3}, {NULL}};
const Segment SGI267852[] = {{&S75669[133], 3}, {NULL}};
const Segment SGI267853[] = {{&S75669[66], 6}, {NULL}};
const Segment SGI267854[] = {{&S75669[100], 6}, {NULL}};
const Segment SGI267855[] = {{&S75669[114], 9}, {NULL}};
const Segment SGI267856[] = {{&S75672[13], 4}, {NULL}};
const Segment SGI267857[] = {{&S75672[26], 5}, {NULL}};
const Segment SGI267858[] = {{&S75672[59], 6}, {NULL}};
const Segment SGI267859[] = {{&S75672[108], 6}, {NULL}};
const Segment SGI267860[] = {{&S75672[122], 9}, {NULL}};
const Segment SGI267861[] = {{&S75672[86], 14}, {NULL}};
const Segment SGI267862[] = {{&S75673[13], 4}, {NULL}};
const Segment SGI267863[] = {{&S75673[26], 3}, {NULL}};
const Segment SGI267864[] = {{&S75673[57], 6}, {NULL}};
const Segment SGI267865[] = {{&S75673[106], 6}, {NULL}};
const Segment SGI267866[] = {{&S75673[120], 9}, {NULL}};
const Segment SGI267867[] = {{&S75673[84], 14}, {NULL}};
const Segment SGI267868[] = {{&S75674[13], 4}, {NULL}};
const Segment SGI267869[] = {{&S75674[26], 5}, {NULL}};
const Segment SGI267870[] = {{&S75674[59], 6}, {NULL}};
const Segment SGI267871[] = {{&S75674[108], 6}, {NULL}};
const Segment SGI267872[] = {{&S75674[122], 9}, {NULL}};
const Segment SGI267873[] = {{&S75674[86], 14}, {NULL}};
const Segment SGI267874[] = {{&S75675[13], 4}, {NULL}};
const Segment SGI267875[] = {{&S75675[26], 5}, {NULL}};
const Segment SGI267876[] = {{&S75675[59], 6}, {NULL}};
const Segment SGI267877[] = {{&S75675[108], 7}, {NULL}};
const Segment SGI267878[] = {{&S75675[123], 9}, {NULL}};
const Segment SGI267879[] = {{&S75675[86], 14}, {NULL}};
const Segment SGI267880[] = {{&S75676[13], 4}, {NULL}};
const Segment SGI267881[] = {{&S75676[26], 3}, {NULL}};
const Segment SGI267882[] = {{&S75676[57], 6}, {NULL}};
const Segment SGI267883[] = {{&S75676[106], 6}, {NULL}};
const Segment SGI267884[] = {{&S75676[120], 9}, {NULL}};
const Segment SGI267885[] = {{&S75676[84], 14}, {NULL}};
const Segment SGI267886[] = {{&S61865[10], 4}, {NULL}};
const Segment SGI267887[] = {{&S61865[15], 5}, {NULL}};
const Segment SGI267888[] = {{&S75696[17], 6}, {NULL}};
const Segment SGI267889[] = {{&S75696[35], 3}, {NULL}};
const Segment SGI267890[] = {{&S75696[59], 3}, {NULL}};
const Segment SGI267891[] = {{&S75697[17], 6}, {NULL}};
const Segment SGI267892[] = {{&S75697[35], 5}, {NULL}};
const Segment SGI267893[] = {{&S75697[61], 3}, {NULL}};
const Segment SGI267894[] = {{&S75833[28], 4}, {NULL}};
const Segment SGI267895[] = {{&S75833[41], 3}, {NULL}};
const Segment SGI267896[] = {{&S75833[72], 6}, {NULL}};
const Segment SGI267897[] = {{&S75833[106], 6}, {NULL}};
const Segment SGI267898[] = {{&S75833[120], 9}, {NULL}};
const Segment SGI267899[] = {{&S75833[0], 14}, {NULL}};
const Segment SGI267900[] = {{&S75834[28], 4}, {NULL}};
const Segment SGI267901[] = {{&S75834[41], 5}, {NULL}};
const Segment SGI267902[] = {{&S75834[74], 6}, {NULL}};
const Segment SGI267903[] = {{&S75834[108], 6}, {NULL}};
const Segment SGI267904[] = {{&S75834[122], 9}, {NULL}};
const Segment SGI267905[] = {{&S75834[0], 14}, {NULL}};
const Segment SGI267906[] = {{&S68726[56], 34}, {NULL}};
const Segment SGI267907[] = {{&S68726[0], 106}, {NULL}};
const Segment SGI267908[] = {{&S68727[58], 3}, {NULL}};
const Segment SGI267909[] = {{&S68727[0], 113}, {NULL}};
const Segment SGI267910[] = {{&S75871[13], 6}, {NULL}};
const Segment SGI267911[] = {{&S75871[135], 3}, {NULL}};
const Segment SGI267912[] = {{&S75871[38], 5}, {NULL}};
const Segment SGI267913[] = {{&S75871[78], 8}, {NULL}};
const Segment SGI267914[] = {{&S75871[114], 5}, {NULL}};
const Segment SGI267915[] = {{&S75872[13], 6}, {NULL}};
const Segment SGI267916[] = {{&S75872[135], 3}, {NULL}};
const Segment SGI267917[] = {{&S75872[38], 5}, {NULL}};
const Segment SGI267918[] = {{&S75872[78], 8}, {NULL}};
const Segment SGI267919[] = {{&S75872[114], 5}, {NULL}};
const Segment SGI267920[] = {{&S75873[13], 6}, {NULL}};
const Segment SGI267921[] = {{&S75873[136], 3}, {NULL}};
const Segment SGI267922[] = {{&S75873[38], 6}, {NULL}};
const Segment SGI267923[] = {{&S75873[79], 8}, {NULL}};
const Segment SGI267924[] = {{&S75873[115], 5}, {NULL}};
const Segment SGI267925[] = {{&S75874[13], 6}, {NULL}};
const Segment SGI267926[] = {{&S75874[135], 3}, {NULL}};
const Segment SGI267927[] = {{&S75874[38], 5}, {NULL}};
const Segment SGI267928[] = {{&S75874[78], 8}, {NULL}};
const Segment SGI267929[] = {{&S75874[114], 5}, {NULL}};
const Segment SGI267930[] = {{&S75875[13], 6}, {NULL}};
const Segment SGI267931[] = {{&S75875[136], 3}, {NULL}};
const Segment SGI267932[] = {{&S75875[38], 5}, {NULL}};
const Segment SGI267933[] = {{&S75875[78], 8}, {NULL}};
const Segment SGI267934[] = {{&S75875[114], 6}, {NULL}};
const Segment SGI267935[] = {{&S68816[66], 5}, {NULL}};
const Segment SGI267936[] = {{&S68816[0], 71}, {NULL}};
const Segment SGI267937[] = {{&S68817[59], 5}, {NULL}};
const Segment SGI267938[] = {{&S68817[0], 64}, {NULL}};
const Segment SGI267939[] = {{&S68818[66], 5}, {&S68818[66], 5}, {NULL}};
const Segment SGI267940[] = {{&S68818[0], 77}, {NULL}};
const Segment SGI267941[] = {{&S76027[13], 4}, {NULL}};
const Segment SGI267942[] = {{&S76027[33], 5}, {NULL}};
const Segment SGI267943[] = {{&S76027[66], 6}, {NULL}};
const Segment SGI267944[] = {{&S76027[119], 6}, {NULL}};
const Segment SGI267945[] = {{&S76027[133], 7}, {NULL}};
const Segment SGI267946[] = {{&S76028[13], 4}, {NULL}};
const Segment SGI267947[] = {{&S76028[33], 3}, {NULL}};
const Segment SGI267948[] = {{&S76028[64], 6}, {NULL}};
const Segment SGI267949[] = {{&S76028[117], 6}, {NULL}};
const Segment SGI267950[] = {{&S76028[131], 7}, {NULL}};
const Segment SGI267951[] = {{&S76029[13], 4}, {NULL}};
const Segment SGI267952[] = {{&S76029[33], 3}, {NULL}};
const Segment SGI267953[] = {{&S76029[64], 6}, {NULL}};
const Segment SGI267954[] = {{&S76029[117], 6}, {NULL}};
const Segment SGI267955[] = {{&S76029[131], 7}, {NULL}};
const Segment SGI267956[] = {{&S76222[13], 6}, {NULL}};
const Segment SGI267957[] = {{&S76222[115], 5}, {NULL}};
const Segment SGI267958[] = {{&S76222[38], 5}, {NULL}};
const Segment SGI267959[] = {{&S76222[78], 8}, {NULL}};
const Segment SGI267960[] = {{&S76222[128], 3}, {NULL}};
const Segment SGI267961[] = {{&S76222[139], 9}, {NULL}};
const Segment SGI267962[] = {{&S76222[149], 13}, {NULL}};
const Segment SGI267963[] = {{&S76362[13], 4}, {NULL}};
const Segment SGI267964[] = {{&S76362[26], 5}, {NULL}};
const Segment SGI267965[] = {{&S76362[59], 6}, {NULL}};
const Segment SGI267966[] = {{&S76362[111], 6}, {NULL}};
const Segment SGI267967[] = {{&S76362[125], 9}, {NULL}};
const Segment SGI267968[] = {{&S76363[13], 4}, {NULL}};
const Segment SGI267969[] = {{&S76363[26], 5}, {NULL}};
const Segment SGI267970[] = {{&S76363[59], 6}, {NULL}};
const Segment SGI267971[] = {{&S76363[111], 7}, {NULL}};
const Segment SGI267972[] = {{&S76363[126], 9}, {NULL}};
const Segment SGI267973[] = {{&S76364[13], 4}, {NULL}};
const Segment SGI267974[] = {{&S76364[26], 3}, {NULL}};
const Segment SGI267975[] = {{&S76364[57], 6}, {NULL}};
const Segment SGI267976[] = {{&S76364[109], 6}, {NULL}};
const Segment SGI267977[] = {{&S76364[123], 9}, {NULL}};
const Segment SGI267978[] = {{&S76365[13], 4}, {NULL}};
const Segment SGI267979[] = {{&S76365[26], 5}, {NULL}};
const Segment SGI267980[] = {{&S76365[59], 6}, {NULL}};
const Segment SGI267981[] = {{&S76365[111], 6}, {NULL}};
const Segment SGI267982[] = {{&S76365[125], 9}, {NULL}};
const Segment SGI267983[] = {{&S76366[13], 4}, {NULL}};
const Segment SGI267984[] = {{&S76366[26], 3}, {NULL}};
const Segment SGI267985[] = {{&S76366[57], 6}, {NULL}};
const Segment SGI267986[] = {{&S76366[109], 6}, {NULL}};
const Segment SGI267987[] = {{&S76366[123], 9}, {NULL}};
const Segment SGI267988[] = {{&S76367[13], 4}, {NULL}};
const Segment SGI267989[] = {{&S76367[26], 3}, {NULL}};
const Segment SGI267990[] = {{&S76367[57], 6}, {NULL}};
const Segment SGI267991[] = {{&S76367[109], 6}, {NULL}};
const Segment SGI267992[] = {{&S76367[123], 9}, {NULL}};
const Segment SGI267993[] = {{&S76368[13], 4}, {NULL}};
const Segment SGI267994[] = {{&S76368[26], 5}, {NULL}};
const Segment SGI267995[] = {{&S76368[59], 6}, {NULL}};
const Segment SGI267996[] = {{&S76368[111], 7}, {NULL}};
const Segment SGI267997[] = {{&S76368[126], 9}, {NULL}};
const Segment SGI267998[] = {{&S76369[28], 6}, {NULL}};
const Segment SGI267999[] = {{&S76369[46], 3}, {NULL}};
const Segment SGI268000[] = {{&S76401[13], 4}, {NULL}};
const Segment SGI268001[] = {{&S76401[26], 3}, {NULL}};
const Segment SGI268002[] = {{&S76401[57], 6}, {NULL}};
const Segment SGI268003[] = {{&S76401[106], 6}, {NULL}};
const Segment SGI268004[] = {{&S76401[120], 9}, {NULL}};
const Segment SGI268005[] = {{&S76401[84], 14}, {NULL}};
const Segment SGI268006[] = {{&S76402[13], 4}, {NULL}};
const Segment SGI268007[] = {{&S76402[26], 5}, {NULL}};
const Segment SGI268008[] = {{&S76402[59], 6}, {NULL}};
const Segment SGI268009[] = {{&S76402[108], 6}, {NULL}};
const Segment SGI268010[] = {{&S76402[122], 9}, {NULL}};
const Segment SGI268011[] = {{&S76402[86], 14}, {NULL}};
const Segment SGI268012[] = {{&S76403[13], 4}, {NULL}};
const Segment SGI268013[] = {{&S76403[26], 5}, {NULL}};
const Segment SGI268014[] = {{&S76403[59], 6}, {NULL}};
const Segment SGI268015[] = {{&S76403[108], 6}, {NULL}};
const Segment SGI268016[] = {{&S76403[122], 9}, {NULL}};
const Segment SGI268017[] = {{&S76403[86], 14}, {NULL}};
const Segment SGI268018[] = {{&S76404[13], 4}, {NULL}};
const Segment SGI268019[] = {{&S76404[26], 3}, {NULL}};
const Segment SGI268020[] = {{&S76404[57], 6}, {NULL}};
const Segment SGI268021[] = {{&S76404[106], 6}, {NULL}};
const Segment SGI268022[] = {{&S76404[120], 9}, {NULL}};
const Segment SGI268023[] = {{&S76404[84], 14}, {NULL}};
const Segment SGI268024[] = {{&S76405[13], 4}, {NULL}};
const Segment SGI268025[] = {{&S76405[26], 5}, {NULL}};
const Segment SGI268026[] = {{&S76405[59], 6}, {NULL}};
const Segment SGI268027[] = {{&S76405[108], 7}, {NULL}};
const Segment SGI268028[] = {{&S76405[123], 9}, {NULL}};
const Segment SGI268029[] = {{&S76405[86], 14}, {NULL}};
const Segment SGI268030[] = {{&S76591[13], 4}, {&S76591[13], 4}, {NULL}};
const Segment SGI268031[] = {{&S76591[26], 3}, {NULL}};
const Segment SGI268032[] = {{&S76591[57], 8}, {NULL}};
const Segment SGI268033[] = {{&S76591[93], 6}, {NULL}};
const Segment SGI268034[] = {{&S76591[107], 9}, {NULL}};
const Segment SGI268035[] = {{&S76592[13], 4}, {&S76592[13], 4}, {NULL}};
const Segment SGI268036[] = {{&S76592[26], 5}, {NULL}};
const Segment SGI268037[] = {{&S76592[59], 8}, {NULL}};
const Segment SGI268038[] = {{&S76592[95], 6}, {NULL}};
const Segment SGI268039[] = {{&S76592[109], 9}, {NULL}};
const Segment SGI268040[] = {{&S76633[13], 4}, {NULL}};
const Segment SGI268041[] = {{&S76633[33], 5}, {NULL}};
const Segment SGI268042[] = {{&S76633[66], 6}, {NULL}};
const Segment SGI268043[] = {{&S76633[119], 6}, {NULL}};
const Segment SGI268044[] = {{&S76633[133], 7}, {NULL}};
const Segment SGI268045[] = {{&S76634[13], 4}, {NULL}};
const Segment SGI268046[] = {{&S76634[33], 3}, {NULL}};
const Segment SGI268047[] = {{&S76634[64], 6}, {NULL}};
const Segment SGI268048[] = {{&S76634[117], 6}, {NULL}};
const Segment SGI268049[] = {{&S76634[131], 7}, {NULL}};
const Segment SGI268050[] = {{&S69547[59], 5}, {NULL}};
const Segment SGI268051[] = {{&S69547[0], 64}, {NULL}};
const Segment SGI268052[] = {{&S69548[66], 5}, {NULL}};
const Segment SGI268053[] = {{&S69548[0], 71}, {NULL}};
const Segment SGI268054[] = {{&S69549[66], 5}, {&S69549[66], 5}, {NULL}};
const Segment SGI268055[] = {{&S69549[0], 77}, {NULL}};
const Segment SGI268056[] = {{&S76782[27], 6}, {NULL}};
const Segment SGI268057[] = {{&S76782[52], 5}, {NULL}};
const Segment SGI268058[] = {{&S76782[85], 6}, {NULL}};
const Segment SGI268059[] = {{&S76782[119], 5}, {NULL}};
const Segment SGI268060[] = {{&S76782[132], 9}, {NULL}};
const Segment SGI268061[] = {{&S76782[0], 14}, {NULL}};
const Segment SGI268062[] = {{&S76783[28], 6}, {NULL}};
const Segment SGI268063[] = {{&S76783[53], 5}, {NULL}};
const Segment SGI268064[] = {{&S76783[86], 6}, {NULL}};
const Segment SGI268065[] = {{&S76783[120], 5}, {NULL}};
const Segment SGI268066[] = {{&S76783[133], 9}, {NULL}};
const Segment SGI268067[] = {{&S76783[0], 15}, {NULL}};
const Segment SGI268068[] = {{&S76784[27], 6}, {NULL}};
const Segment SGI268069[] = {{&S76784[51], 5}, {NULL}};
const Segment SGI268070[] = {{&S76784[84], 6}, {NULL}};
const Segment SGI268071[] = {{&S76784[118], 5}, {NULL}};
const Segment SGI268072[] = {{&S76784[131], 9}, {NULL}};
const Segment SGI268073[] = {{&S76784[0], 14}, {NULL}};
const Segment SGI268074[] = {{&S76833[13], 6}, {NULL}};
const Segment SGI268075[] = {{&S76833[113], 3}, {NULL}};
const Segment SGI268076[] = {{&S76833[38], 5}, {NULL}};
const Segment SGI268077[] = {{&S76833[78], 6}, {NULL}};
const Segment SGI268078[] = {{&S76833[124], 6}, {NULL}};
const Segment SGI268079[] = {{&S76833[138], 9}, {NULL}};
const Segment SGI268080[] = {{&S76834[13], 6}, {NULL}};
const Segment SGI268081[] = {{&S76834[111], 3}, {NULL}};
const Segment SGI268082[] = {{&S76834[38], 3}, {NULL}};
const Segment SGI268083[] = {{&S76834[76], 6}, {NULL}};
const Segment SGI268084[] = {{&S76834[122], 6}, {NULL}};
const Segment SGI268085[] = {{&S76834[136], 9}, {NULL}};
const Segment SGI268086[] = {{&S76835[13], 6}, {NULL}};
const Segment SGI268087[] = {{&S76835[113], 3}, {NULL}};
const Segment SGI268088[] = {{&S76835[38], 5}, {NULL}};
const Segment SGI268089[] = {{&S76835[78], 6}, {NULL}};
const Segment SGI268090[] = {{&S76835[124], 6}, {NULL}};
const Segment SGI268091[] = {{&S76835[138], 9}, {NULL}};
const Segment SGI268092[] = {{&S76836[13], 6}, {NULL}};
const Segment SGI268093[] = {{&S76836[111], 3}, {NULL}};
const Segment SGI268094[] = {{&S76836[38], 3}, {NULL}};
const Segment SGI268095[] = {{&S76836[76], 6}, {NULL}};
const Segment SGI268096[] = {{&S76836[122], 6}, {NULL}};
const Segment SGI268097[] = {{&S76836[136], 9}, {NULL}};
const Segment SGI268098[] = {{&S76837[13], 6}, {NULL}};
const Segment SGI268099[] = {{&S76837[111], 3}, {NULL}};
const Segment SGI268100[] = {{&S76837[38], 3}, {NULL}};
const Segment SGI268101[] = {{&S76837[76], 6}, {NULL}};
const Segment SGI268102[] = {{&S76837[122], 6}, {NULL}};
const Segment SGI268103[] = {{&S76837[136], 9}, {NULL}};
const Segment SGI268104[] = {{&S76841[13], 6}, {NULL}};
const Segment SGI268105[] = {{&S76841[101], 3}, {NULL}};
const Segment SGI268106[] = {{&S76841[35], 3}, {NULL}};
const Segment SGI268107[] = {{&S76841[66], 6}, {NULL}};
const Segment SGI268108[] = {{&S76841[112], 5}, {NULL}};
const Segment SGI268109[] = {{&S76841[125], 9}, {NULL}};
const Segment SGI268110[] = {{&S76842[13], 6}, {NULL}};
const Segment SGI268111[] = {{&S76842[103], 3}, {NULL}};
const Segment SGI268112[] = {{&S76842[35], 5}, {NULL}};
const Segment SGI268113[] = {{&S76842[68], 6}, {NULL}};
const Segment SGI268114[] = {{&S76842[114], 6}, {NULL}};
const Segment SGI268115[] = {{&S76842[128], 7}, {NULL}};
const Segment SGI268116[] = {{&S76843[13], 6}, {NULL}};
const Segment SGI268117[] = {{&S76843[103], 3}, {NULL}};
const Segment SGI268118[] = {{&S76843[35], 5}, {NULL}};
const Segment SGI268119[] = {{&S76843[68], 6}, {NULL}};
const Segment SGI268120[] = {{&S76843[114], 6}, {NULL}};
const Segment SGI268121[] = {{&S76843[128], 7}, {NULL}};
const Segment SGI268122[] = {{&S76882[13], 4}, {NULL}};
const Segment SGI268123[] = {{&S76882[26], 5}, {NULL}};
const Segment SGI268124[] = {{&S76882[126], 3}, {NULL}};
const Segment SGI268125[] = {{&S76882[59], 6}, {NULL}};
const Segment SGI268126[] = {{&S76882[93], 6}, {NULL}};
const Segment SGI268127[] = {{&S76882[107], 9}, {NULL}};
const Segment SGI268128[] = {{&S76883[13], 4}, {NULL}};
const Segment SGI268129[] = {{&S76883[26], 3}, {NULL}};
const Segment SGI268130[] = {{&S76883[124], 3}, {NULL}};
const Segment SGI268131[] = {{&S76883[57], 6}, {NULL}};
const Segment SGI268132[] = {{&S76883[91], 6}, {NULL}};
const Segment SGI268133[] = {{&S76883[105], 9}, {NULL}};
const Segment SGI268134[] = {{&S76884[13], 4}, {NULL}};
const Segment SGI268135[] = {{&S76884[26], 5}, {NULL}};
const Segment SGI268136[] = {{&S76884[126], 3}, {NULL}};
const Segment SGI268137[] = {{&S76884[59], 6}, {NULL}};
const Segment SGI268138[] = {{&S76884[93], 6}, {NULL}};
const Segment SGI268139[] = {{&S76884[107], 9}, {NULL}};
const Segment SGI268140[] = {{&S69694[0], 15}, {&S69694[4], 7}, {&S69694[58], 27}, {NULL}};
const Segment SGI268141[] = {{&S69694[0], 87}, {NULL}};
const Segment SGI268142[] = {{&S69695[59], 3}, {NULL}};
const Segment SGI268143[] = {{&S69695[0], 63}, {NULL}};
const Segment SGI268144[] = {{&S69696[35], 10}, {NULL}};
const Segment SGI268145[] = {{&S69696[0], 66}, {NULL}};
const Segment SGI268146[] = {{&S69697[25], 14}, {NULL}};
const Segment SGI268147[] = {{&S69697[0], 100}, {NULL}};
const Segment SGI268148[] = {{&S69698[52], 7}, {NULL}};
const Segment SGI268149[] = {{&S69698[0], 78}, {NULL}};
const Segment SGI268150[] = {{&S69699[25], 16}, {NULL}};
const Segment SGI268151[] = {{&S69699[0], 105}, {NULL}};
const Segment SGI268152[] = {{&S69700[25], 15}, {NULL}};
const Segment SGI268153[] = {{&S69700[0], 100}, {NULL}};
const Segment SGI268154[] = {{&S69701[25], 15}, {NULL}};
const Segment SGI268155[] = {{&S69701[0], 50}, {NULL}};
const Segment SGI268156[] = {{&S69702[0], 10}, {&S69702[34], 31}, {NULL}};
const Segment SGI268157[] = {{&S69702[0], 65}, {NULL}};
const Segment SGI268158[] = {{&S63152[10], 4}, {NULL}};
const Segment SGI268159[] = {{&S63152[15], 5}, {NULL}};
const Segment SGI268160[] = {{&S63153[72], 4}, {NULL}};
const Segment SGI268161[] = {{&S63153[77], 5}, {NULL}};
const Segment SGI268162[] = {{&S63213[20], 27}, {NULL}};
const Segment SGI268163[] = {{&S63214[20], 27}, {NULL}};
const Segment SGI268164[] = {{&S63215[20], 27}, {NULL}};
const Segment SGI268165[] = {{&S63216[20], 27}, {NULL}};
const Segment SGI268166[] = {{&S63217[20], 27}, {NULL}};
const Segment SGI268167[] = {{&S63218[20], 27}, {NULL}};
const Segment SGI268168[] = {{&S69777[66], 5}, {NULL}};
const Segment SGI268169[] = {{&S69777[0], 71}, {NULL}};
const Segment SGI268170[] = {{&S69778[59], 5}, {NULL}};
const Segment SGI268171[] = {{&S69778[0], 64}, {NULL}};
const Segment SGI268172[] = {{&S69779[66], 5}, {&S69779[66], 5}, {NULL}};
const Segment SGI268173[] = {{&S69779[0], 77}, {NULL}};
const Segment SGI268174[] = {{&S76966[13], 4}, {NULL}};
const Segment SGI268175[] = {{&S76966[26], 5}, {NULL}};
const Segment SGI268176[] = {{&S76966[59], 6}, {NULL}};
const Segment SGI268177[] = {{&S76966[117], 5}, {NULL}};
const Segment SGI268178[] = {{&S76966[130], 9}, {NULL}};
const Segment SGI268179[] = {{&S76967[13], 4}, {NULL}};
const Segment SGI268180[] = {{&S76967[26], 3}, {NULL}};
const Segment SGI268181[] = {{&S76967[57], 6}, {NULL}};
const Segment SGI268182[] = {{&S76967[115], 5}, {NULL}};
const Segment SGI268183[] = {{&S76967[128], 9}, {NULL}};
const Segment SGI268184[] = {{&S77029[13], 4}, {NULL}};
const Segment SGI268185[] = {{&S77029[26], 3}, {NULL}};
const Segment SGI268186[] = {{&S77029[124], 3}, {NULL}};
const Segment SGI268187[] = {{&S77029[57], 6}, {NULL}};
const Segment SGI268188[] = {{&S77029[91], 6}, {NULL}};
const Segment SGI268189[] = {{&S77029[105], 9}, {NULL}};
const Segment SGI268190[] = {{&S77030[13], 4}, {NULL}};
const Segment SGI268191[] = {{&S77030[26], 5}, {NULL}};
const Segment SGI268192[] = {{&S77030[126], 3}, {NULL}};
const Segment SGI268193[] = {{&S77030[59], 6}, {NULL}};
const Segment SGI268194[] = {{&S77030[93], 6}, {NULL}};
const Segment SGI268195[] = {{&S77030[107], 9}, {NULL}};
const Segment SGI268196[] = {{&S77031[13], 4}, {NULL}};
const Segment SGI268197[] = {{&S77031[26], 5}, {NULL}};
const Segment SGI268198[] = {{&S77031[126], 3}, {NULL}};
const Segment SGI268199[] = {{&S77031[59], 6}, {NULL}};
const Segment SGI268200[] = {{&S77031[93], 6}, {NULL}};
const Segment SGI268201[] = {{&S77031[107], 9}, {NULL}};
const Segment SGI268202[] = {{&S77055[13], 4}, {NULL}};
const Segment SGI268203[] = {{&S77055[29], 3}, {NULL}};
const Segment SGI268204[] = {{&S77055[67], 8}, {NULL}};
const Segment SGI268205[] = {{&S77055[122], 6}, {NULL}};
const Segment SGI268206[] = {{&S77055[136], 9}, {NULL}};
const Segment SGI268207[] = {{&S77056[13], 4}, {NULL}};
const Segment SGI268208[] = {{&S77056[29], 5}, {NULL}};
const Segment SGI268209[] = {{&S77056[69], 8}, {NULL}};
const Segment SGI268210[] = {{&S77056[124], 6}, {NULL}};
const Segment SGI268211[] = {{&S77056[138], 9}, {NULL}};
const Segment SGI268212[] = {{&S77058[17], 6}, {NULL}};
const Segment SGI268213[] = {{&S77058[35], 5}, {NULL}};
const Segment SGI268214[] = {{&S77059[17], 6}, {NULL}};
const Segment SGI268215[] = {{&S77059[35], 5}, {NULL}};
const Segment SGI268216[] = {{&S77060[17], 6}, {NULL}};
const Segment SGI268217[] = {{&S77060[35], 6}, {NULL}};
const Segment SGI268218[] = {{&S77061[17], 6}, {NULL}};
const Segment SGI268219[] = {{&S77061[35], 5}, {NULL}};
const Segment SGI268220[] = {{&S77083[13], 4}, {NULL}};
const Segment SGI268221[] = {{&S77083[26], 5}, {NULL}};
const Segment SGI268222[] = {{&S77083[59], 6}, {NULL}};
const Segment SGI268223[] = {{&S77083[108], 5}, {NULL}};
const Segment SGI268224[] = {{&S77083[121], 9}, {NULL}};
const Segment SGI268225[] = {{&S77083[86], 14}, {NULL}};
const Segment SGI268226[] = {{&S77084[13], 4}, {NULL}};
const Segment SGI268227[] = {{&S77084[26], 3}, {NULL}};
const Segment SGI268228[] = {{&S77084[57], 6}, {NULL}};
const Segment SGI268229[] = {{&S77084[106], 5}, {NULL}};
const Segment SGI268230[] = {{&S77084[119], 9}, {NULL}};
const Segment SGI268231[] = {{&S77084[84], 14}, {NULL}};
const Segment SGI268232[] = {{&S77091[13], 6}, {NULL}};
const Segment SGI268233[] = {{&S77091[67], 5}, {NULL}};
const Segment SGI268234[] = {{&S77091[33], 5}, {NULL}};
const Segment SGI268235[] = {{&S77091[80], 5}, {NULL}};
const Segment SGI268236[] = {{&S77091[93], 9}, {NULL}};
const Segment SGI268237[] = {{&S70206[66], 5}, {NULL}};
const Segment SGI268238[] = {{&S70206[0], 71}, {NULL}};
const Segment SGI268239[] = {{&S70207[59], 5}, {NULL}};
const Segment SGI268240[] = {{&S70207[0], 64}, {NULL}};
const Segment SGI268241[] = {{&S70208[66], 5}, {&S70208[66], 5}, {NULL}};
const Segment SGI268242[] = {{&S70208[0], 77}, {NULL}};
const Segment SGI268243[] = {{&S70209[59], 5}, {&S70209[59], 5}, {NULL}};
const Segment SGI268244[] = {{&S70209[0], 70}, {NULL}};
const Segment SGI268245[] = {{&S64271[10], 4}, {NULL}};
const Segment SGI268246[] = {{&S64271[15], 5}, {NULL}};
const Segment SGI268247[] = {{&S77654[13], 6}, {NULL}};
const Segment SGI268248[] = {{&S77654[28], 5}, {NULL}};
const Segment SGI268249[] = {{&S77654[61], 6}, {NULL}};
const Segment SGI268250[] = {{&S77654[112], 6}, {NULL}};
const Segment SGI268251[] = {{&S77654[126], 7}, {NULL}};
const Segment SGI268252[] = {{&S77691[12], 4}, {NULL}};
const Segment SGI268253[] = {{&S77691[74], 5}, {NULL}};
const Segment SGI268254[] = {{&S77883[13], 4}, {NULL}};
const Segment SGI268255[] = {{&S77883[29], 3}, {NULL}};
const Segment SGI268256[] = {{&S77883[67], 9}, {NULL}};
const Segment SGI268257[] = {{&S64689[20], 26}, {NULL}};
const Segment SGI268258[] = {{&S64690[20], 26}, {NULL}};
const Segment SGI268259[] = {{&S64691[20], 26}, {NULL}};
const Segment SGI268260[] = {{&S64692[20], 26}, {NULL}};
const Segment SGI268261[] = {{&S64693[20], 26}, {NULL}};
const Segment SGI268262[] = {{&S64694[20], 26}, {NULL}};
const Segment SGI268263[] = {{&S64695[20], 27}, {NULL}};
const Segment SGI268264[] = {{&S64696[20], 27}, {NULL}};
const Segment SGI268265[] = {{&S64697[20], 26}, {NULL}};
const Segment SGI268266[] = {{&S64698[20], 27}, {NULL}};
const Segment SGI268267[] = {{&S64699[20], 27}, {NULL}};
const Segment SGI268268[] = {{&S64700[20], 26}, {NULL}};
const Segment SGI268269[] = {{&S71279[69], 5}, {NULL}};
const Segment SGI268270[] = {{&S71279[0], 74}, {NULL}};
const Segment SGI268271[] = {{&S71280[66], 5}, {NULL}};
const Segment SGI268272[] = {{&S71280[0], 71}, {NULL}};
const Segment SGI268273[] = {{&S71281[66], 5}, {&S71281[66], 5}, {NULL}};
const Segment SGI268274[] = {{&S71281[0], 77}, {NULL}};
const Segment SGI268275[] = {{&S71282[59], 5}, {NULL}};
const Segment SGI268276[] = {{&S71282[0], 64}, {NULL}};
const Segment SGI268277[] = {{&S71370[66], 5}, {NULL}};
const Segment SGI268278[] = {{&S71370[0], 71}, {NULL}};
const Segment SGI268279[] = {{&S71371[59], 5}, {NULL}};
const Segment SGI268280[] = {{&S71371[0], 64}, {NULL}};
const Segment SGI268281[] = {{&S71372[66], 5}, {&S71372[66], 5}, {NULL}};
const Segment SGI268282[] = {{&S71372[0], 77}, {NULL}};
const Segment SGI268283[] = {{&S78251[13], 4}, {NULL}};
const Segment SGI268284[] = {{&S78251[33], 2}, {NULL}};
const Segment SGI268285[] = {{&S78251[129], 5}, {NULL}};
const Segment SGI268286[] = {{&S78251[63], 6}, {NULL}};
const Segment SGI268287[] = {{&S78251[96], 6}, {NULL}};
const Segment SGI268288[] = {{&S78251[110], 9}, {NULL}};
const Segment SGI268289[] = {{&S78317[12], 4}, {&S78317[12], 4}, {NULL}};
const Segment SGI268290[] = {{&S78317[27], 3}, {NULL}};
const Segment SGI268291[] = {{&S78317[74], 6}, {NULL}};
const Segment SGI268292[] = {{&S78317[137], 9}, {NULL}};
const Segment SGI268293[] = {{&S78317[101], 13}, {NULL}};
const Segment SGI268294[] = {{&S78366[13], 4}, {NULL}};
const Segment SGI268295[] = {{&S78366[26], 5}, {NULL}};
const Segment SGI268296[] = {{&S78366[59], 6}, {NULL}};
const Segment SGI268297[] = {{&S78366[108], 5}, {NULL}};
const Segment SGI268298[] = {{&S78366[121], 9}, {NULL}};
const Segment SGI268299[] = {{&S78366[86], 14}, {NULL}};
const Segment SGI268300[] = {{&S78367[13], 4}, {NULL}};
const Segment SGI268301[] = {{&S78367[26], 3}, {NULL}};
const Segment SGI268302[] = {{&S78367[57], 6}, {NULL}};
const Segment SGI268303[] = {{&S78367[106], 5}, {NULL}};
const Segment SGI268304[] = {{&S78367[119], 9}, {NULL}};
const Segment SGI268305[] = {{&S78367[84], 14}, {NULL}};
const Segment SGI268306[] = {{&S78460[17], 6}, {NULL}};
const Segment SGI268307[] = {{&S78460[38], 5}, {NULL}};
const Segment SGI268308[] = {{&S78460[64], 3}, {NULL}};
const Segment SGI268309[] = {{&S78461[17], 6}, {NULL}};
const Segment SGI268310[] = {{&S78461[38], 3}, {NULL}};
const Segment SGI268311[] = {{&S78461[62], 3}, {NULL}};
const Segment SGI268312[] = {{&S71531[25], 12}, {&S71531[38], 33}, {NULL}};
const Segment SGI268313[] = {{&S71531[0], 143}, {NULL}};
const Segment SGI268314[] = {{&S71532[25], 12}, {&S71532[38], 33}, {NULL}};
const Segment SGI268315[] = {{&S71532[0], 142}, {NULL}};
const Segment SGI268316[] = {{&S71533[25], 12}, {&S71533[38], 33}, {NULL}};
const Segment SGI268317[] = {{&S71533[0], 151}, {NULL}};
const Segment SGI268318[] = {{&S71534[25], 12}, {&S71534[38], 33}, {NULL}};
const Segment SGI268319[] = {{&S71534[0], 148}, {NULL}};
const Segment SGI268320[] = {{&S71535[25], 12}, {&S71535[38], 33}, {NULL}};
const Segment SGI268321[] = {{&S71535[0], 149}, {NULL}};
const Segment SGI268322[] = {{&S71536[25], 12}, {&S71536[38], 33}, {NULL}};
const Segment SGI268323[] = {{&S71536[0], 148}, {NULL}};
const Segment SGI268324[] = {{&S71537[25], 12}, {&S71537[38], 33}, {NULL}};
const Segment SGI268325[] = {{&S71537[0], 150}, {NULL}};
const Segment SGI268326[] = {{&S78571[13], 4}, {NULL}};
const Segment SGI268327[] = {{&S78571[36], 3}, {NULL}};
const Segment SGI268328[] = {{&S78571[74], 6}, {NULL}};
const Segment SGI268329[] = {{&S78571[108], 5}, {NULL}};
const Segment SGI268330[] = {{&S78572[13], 4}, {NULL}};
const Segment SGI268331[] = {{&S78572[36], 3}, {NULL}};
const Segment SGI268332[] = {{&S78572[74], 6}, {NULL}};
const Segment SGI268333[] = {{&S71538[25], 12}, {&S71538[38], 33}, {NULL}};
const Segment SGI268334[] = {{&S71538[0], 150}, {NULL}};
const Segment SGI268335[] = {{&S71539[25], 11}, {&S71539[37], 50}, {NULL}};
const Segment SGI268336[] = {{&S71539[0], 87}, {NULL}};
const Segment SGI268337[] = {{&S71540[25], 12}, {&S71540[38], 33}, {NULL}};
const Segment SGI268338[] = {{&S71540[0], 147}, {NULL}};
const Segment SGI268339[] = {{&S71541[25], 12}, {&S71541[38], 33}, {NULL}};
const Segment SGI268340[] = {{&S71541[0], 153}, {NULL}};
const Segment SGI268341[] = {{&S71542[25], 12}, {&S71542[38], 33}, {NULL}};
const Segment SGI268342[] = {{&S71542[0], 150}, {NULL}};
const Segment SGI268343[] = {{&S71543[25], 12}, {&S71543[38], 33}, {NULL}};
const Segment SGI268344[] = {{&S71543[0], 147}, {NULL}};
const Segment SGI268345[] = {{&S78621[13], 4}, {NULL}};
const Segment SGI268346[] = {{&S78621[94], 3}, {NULL}};
const Segment SGI268347[] = {{&S78621[105], 5}, {NULL}};
const Segment SGI268348[] = {{&S78621[118], 9}, {NULL}};
const Segment SGI268349[] = {{&S78622[13], 4}, {NULL}};
const Segment SGI268350[] = {{&S78622[91], 3}, {NULL}};
const Segment SGI268351[] = {{&S78622[56], 6}, {NULL}};
const Segment SGI268352[] = {{&S78622[102], 5}, {NULL}};
const Segment SGI268353[] = {{&S78622[115], 9}, {NULL}};
const Segment SGI268354[] = {{&S78648[13], 4}, {NULL}};
const Segment SGI268355[] = {{&S78648[26], 5}, {NULL}};
const Segment SGI268356[] = {{&S78648[59], 6}, {NULL}};
const Segment SGI268357[] = {{&S78648[108], 5}, {NULL}};
const Segment SGI268358[] = {{&S78648[121], 9}, {NULL}};
const Segment SGI268359[] = {{&S78648[86], 14}, {NULL}};
const Segment SGI268360[] = {{&S78649[13], 4}, {NULL}};
const Segment SGI268361[] = {{&S78649[26], 3}, {NULL}};
const Segment SGI268362[] = {{&S78649[57], 6}, {NULL}};
const Segment SGI268363[] = {{&S78649[106], 5}, {NULL}};
const Segment SGI268364[] = {{&S78649[119], 9}, {NULL}};
const Segment SGI268365[] = {{&S78649[84], 14}, {NULL}};
const Segment SGI268366[] = {{&S71677[66], 5}, {NULL}};
const Segment SGI268367[] = {{&S71677[0], 71}, {NULL}};
const Segment SGI268368[] = {{&S71678[68], 5}, {NULL}};
const Segment SGI268369[] = {{&S71678[0], 73}, {NULL}};
const Segment SGI268370[] = {{&S71679[61], 5}, {NULL}};
const Segment SGI268371[] = {{&S71679[0], 66}, {NULL}};
const Segment SGI268372[] = {{&S71680[68], 5}, {&S71680[68], 5}, {NULL}};
const Segment SGI268373[] = {{&S71680[0], 79}, {NULL}};
const Segment SGI268374[] = {{&S71681[59], 5}, {NULL}};
const Segment SGI268375[] = {{&S71681[0], 64}, {NULL}};
const Segment SGI268376[] = {{&S78740[13], 4}, {NULL}};
const Segment SGI268377[] = {{&S78740[26], 5}, {NULL}};
const Segment SGI268378[] = {{&S78740[59], 6}, {NULL}};
const Segment SGI268379[] = {{&S78740[105], 5}, {NULL}};
const Segment SGI268380[] = {{&S78740[118], 9}, {NULL}};
const Segment SGI268381[] = {{&S78741[13], 4}, {NULL}};
const Segment SGI268382[] = {{&S78741[26], 3}, {NULL}};
const Segment SGI268383[] = {{&S78741[57], 6}, {NULL}};
const Segment SGI268384[] = {{&S78741[103], 5}, {NULL}};
const Segment SGI268385[] = {{&S78741[116], 9}, {NULL}};
const Segment SGI268386[] = {{&S78918[13], 6}, {NULL}};
const Segment SGI268387[] = {{&S78918[38], 3}, {NULL}};
const Segment SGI268388[] = {{&S78918[76], 8}, {NULL}};
const Segment SGI268389[] = {{&S78918[131], 6}, {NULL}};
const Segment SGI268390[] = {{&S78918[145], 9}, {NULL}};
const Segment SGI268391[] = {{&S78919[13], 6}, {NULL}};
const Segment SGI268392[] = {{&S78919[38], 5}, {NULL}};
const Segment SGI268393[] = {{&S78919[78], 8}, {NULL}};
const Segment SGI268394[] = {{&S78919[133], 6}, {NULL}};
const Segment SGI268395[] = {{&S78919[147], 9}, {NULL}};
const Segment SGI268396[] = {{&S65549[10], 4}, {NULL}};
const Segment SGI268397[] = {{&S65549[15], 5}, {NULL}};
const Segment SGI268398[] = {{&S65550[72], 4}, {NULL}};
const Segment SGI268399[] = {{&S65550[77], 5}, {NULL}};
const Segment SGI268400[] = {{&S78957[17], 6}, {NULL}};
const Segment SGI268401[] = {{&S78957[38], 5}, {NULL}};
const Segment SGI268402[] = {{&S78957[68], 3}, {NULL}};
const Segment SGI268403[] = {{&S78958[17], 6}, {NULL}};
const Segment SGI268404[] = {{&S78958[38], 5}, {NULL}};
const Segment SGI268405[] = {{&S78958[68], 3}, {NULL}};
const Segment SGI268406[] = {{&S79316[17], 6}, {NULL}};
const Segment SGI268407[] = {{&S79316[38], 5}, {NULL}};
const Segment SGI268408[] = {{&S79316[68], 3}, {NULL}};
const Segment SGI268409[] = {{&S79317[27], 6}, {NULL}};
const Segment SGI268410[] = {{&S79317[52], 5}, {NULL}};
const Segment SGI268411[] = {{&S79317[85], 6}, {NULL}};
const Segment SGI268412[] = {{&S79317[119], 5}, {NULL}};
const Segment SGI268413[] = {{&S79317[132], 9}, {NULL}};
const Segment SGI268414[] = {{&S79317[0], 14}, {NULL}};
const Segment SGI268415[] = {{&S79318[27], 6}, {NULL}};
const Segment SGI268416[] = {{&S79318[51], 5}, {NULL}};
const Segment SGI268417[] = {{&S79318[84], 6}, {NULL}};
const Segment SGI268418[] = {{&S79318[118], 5}, {NULL}};
const Segment SGI268419[] = {{&S79318[131], 9}, {NULL}};
const Segment SGI268420[] = {{&S79318[0], 14}, {NULL}};
const Segment SGI268421[] = {{&S66097[20], 27}, {NULL}};
const Segment SGI268422[] = {{&S66098[20], 27}, {NULL}};
const Segment SGI268423[] = {{&S66099[20], 27}, {NULL}};
const Segment SGI268424[] = {{&S66100[20], 27}, {NULL}};
const Segment SGI268425[] = {{&S66101[20], 27}, {NULL}};
const Segment SGI268426[] = {{&S66102[20], 27}, {NULL}};
const Segment SGI268427[] = {{&S66103[20], 27}, {NULL}};
const Segment SGI268428[] = {{&S66104[20], 27}, {NULL}};
const Segment SGI268429[] = {{&S66105[20], 27}, {NULL}};
const Segment SGI268430[] = {{&S66106[20], 27}, {NULL}};
const Segment SGI268431[] = {{&S66107[20], 27}, {NULL}};
const Segment SGI268432[] = {{&S66108[20], 27}, {NULL}};
const Segment SGI268433[] = {{&S79517[13], 4}, {NULL}};
const Segment SGI268434[] = {{&S79517[26], 5}, {NULL}};
const Segment SGI268435[] = {{&S79517[59], 6}, {NULL}};
const Segment SGI268436[] = {{&S79517[108], 6}, {NULL}};
const Segment SGI268437[] = {{&S79517[122], 9}, {NULL}};
const Segment SGI268438[] = {{&S79517[86], 14}, {NULL}};
const Segment SGI268439[] = {{&S79518[13], 4}, {NULL}};
const Segment SGI268440[] = {{&S79518[26], 5}, {NULL}};
const Segment SGI268441[] = {{&S79518[59], 6}, {NULL}};
const Segment SGI268442[] = {{&S79518[108], 6}, {NULL}};
const Segment SGI268443[] = {{&S79518[122], 9}, {NULL}};
const Segment SGI268444[] = {{&S79518[86], 14}, {NULL}};
const Segment SGI268445[] = {{&S79519[13], 4}, {NULL}};
const Segment SGI268446[] = {{&S79519[26], 3}, {NULL}};
const Segment SGI268447[] = {{&S79519[57], 6}, {NULL}};
const Segment SGI268448[] = {{&S79519[106], 6}, {NULL}};
const Segment SGI268449[] = {{&S79519[120], 9}, {NULL}};
const Segment SGI268450[] = {{&S79519[84], 14}, {NULL}};
const Segment SGI268451[] = {{&S66361[20], 28}, {NULL}};
const Segment SGI268452[] = {{&S66362[20], 27}, {NULL}};
const Segment SGI268453[] = {{&S66363[20], 27}, {NULL}};
const Segment SGI268454[] = {{&S66364[20], 28}, {NULL}};
const Segment SGI268455[] = {{&S66365[20], 28}, {NULL}};
const Segment SGI268456[] = {{&S66366[20], 27}, {NULL}};
const Segment SGI268457[] = {{&S66367[20], 28}, {NULL}};
const Segment SGI268458[] = {{&S66368[20], 28}, {NULL}};
const Segment SGI268459[] = {{&S72872[109], 5}, {NULL}};
const Segment SGI268460[] = {{&S72872[0], 114}, {NULL}};
const Segment SGI268461[] = {{&S72873[110], 5}, {NULL}};
const Segment SGI268462[] = {{&S72873[0], 115}, {NULL}};
const Segment SGI268463[] = {{&S72874[103], 5}, {NULL}};
const Segment SGI268464[] = {{&S72874[0], 108}, {NULL}};
const Segment SGI268465[] = {{&S72875[110], 5}, {NULL}};
const Segment SGI268466[] = {{&S72875[0], 115}, {NULL}};
const Segment SGI268467[] = {{&S72876[103], 5}, {NULL}};
const Segment SGI268468[] = {{&S72876[0], 108}, {NULL}};
const Segment SGI268469[] = {{&S73208[109], 5}, {NULL}};
const Segment SGI268470[] = {{&S73208[0], 114}, {NULL}};
const Segment SGI268471[] = {{&S73209[102], 5}, {NULL}};
const Segment SGI268472[] = {{&S73209[0], 107}, {NULL}};
const Segment SGI268473[] = {{&S73210[109], 5}, {NULL}};
const Segment SGI268474[] = {{&S73210[0], 114}, {NULL}};
const Segment SGI268475[] = {{&S73211[102], 5}, {NULL}};
const Segment SGI268476[] = {{&S73211[0], 107}, {NULL}};
const Segment SGI268477[] = {{&S79897[13], 4}, {NULL}};
const Segment SGI268478[] = {{&S79897[115], 3}, {NULL}};
const Segment SGI268479[] = {{&S79897[42], 3}, {NULL}};
const Segment SGI268480[] = {{&S79897[80], 6}, {NULL}};
const Segment SGI268481[] = {{&S79897[126], 5}, {NULL}};
const Segment SGI268482[] = {{&S79897[139], 6}, {NULL}};
const Segment SGI268483[] = {{&S79898[13], 4}, {NULL}};
const Segment SGI268484[] = {{&S79898[117], 3}, {NULL}};
const Segment SGI268485[] = {{&S79898[42], 5}, {NULL}};
const Segment SGI268486[] = {{&S79898[82], 6}, {NULL}};
const Segment SGI268487[] = {{&S79898[128], 5}, {NULL}};
const Segment SGI268488[] = {{&S79898[141], 6}, {NULL}};
const Segment SGI268489[] = {{&S79950[17], 4}, {NULL}};
const Segment SGI268490[] = {{&S79950[36], 5}, {NULL}};
const Segment SGI268491[] = {{&S79950[66], 3}, {NULL}};
const Segment SGI268492[] = {{&S79951[17], 4}, {NULL}};
const Segment SGI268493[] = {{&S79951[36], 5}, {NULL}};
const Segment SGI268494[] = {{&S79951[66], 3}, {NULL}};
const Segment SGI268495[] = {{&S80170[13], 6}, {NULL}};
const Segment SGI268496[] = {{&S80170[113], 3}, {NULL}};
const Segment SGI268497[] = {{&S80170[38], 5}, {NULL}};
const Segment SGI268498[] = {{&S80170[78], 6}, {NULL}};
const Segment SGI268499[] = {{&S80170[124], 5}, {NULL}};
const Segment SGI268500[] = {{&S80170[137], 9}, {NULL}};
const Segment SGI268501[] = {{&S73465[0], 20}, {NULL}};
const Segment SGI268502[] = {{&S73465[0], 75}, {NULL}};
const Segment SGI268503[] = {{&S80197[28], 6}, {NULL}};
const Segment SGI268504[] = {{&S80197[46], 5}, {NULL}};
const Segment SGI268505[] = {{&S80198[28], 6}, {NULL}};
const Segment SGI268506[] = {{&S80198[46], 3}, {NULL}};
const Segment SGI268507[] = {{&S80199[28], 6}, {NULL}};
const Segment SGI268508[] = {{&S80199[46], 5}, {NULL}};
const Segment SGI268509[] = {{&S80220[13], 4}, {NULL}};
const Segment SGI268510[] = {{&S80220[26], 5}, {NULL}};
const Segment SGI268511[] = {{&S80220[125], 5}, {NULL}};
const Segment SGI268512[] = {{&S80220[59], 6}, {NULL}};
const Segment SGI268513[] = {{&S80220[93], 5}, {NULL}};
const Segment SGI268514[] = {{&S80220[106], 9}, {NULL}};
const Segment SGI268515[] = {{&S80221[13], 4}, {NULL}};
const Segment SGI268516[] = {{&S80221[26], 5}, {NULL}};
const Segment SGI268517[] = {{&S80221[125], 5}, {NULL}};
const Segment SGI268518[] = {{&S80221[59], 6}, {NULL}};
const Segment SGI268519[] = {{&S80221[93], 5}, {NULL}};
const Segment SGI268520[] = {{&S80221[106], 9}, {NULL}};
const Segment SGI268521[] = {{&S80222[13], 4}, {NULL}};
const Segment SGI268522[] = {{&S80222[26], 5}, {NULL}};
const Segment SGI268523[] = {{&S80222[125], 5}, {NULL}};
const Segment SGI268524[] = {{&S80222[59], 6}, {NULL}};
const Segment SGI268525[] = {{&S80222[93], 5}, {NULL}};
const Segment SGI268526[] = {{&S80222[106], 9}, {NULL}};
const Segment SGI268527[] = {{&S80223[13], 4}, {NULL}};
const Segment SGI268528[] = {{&S80223[26], 5}, {NULL}};
const Segment SGI268529[] = {{&S80223[125], 5}, {NULL}};
const Segment SGI268530[] = {{&S80223[59], 6}, {NULL}};
const Segment SGI268531[] = {{&S80223[93], 5}, {NULL}};
const Segment SGI268532[] = {{&S80223[106], 9}, {NULL}};
const Segment SGI268533[] = {{&S73512[51], 5}, {NULL}};
const Segment SGI268534[] = {{&S73512[0], 56}, {NULL}};
const Segment SGI268535[] = {{&S80231[13], 6}, {NULL}};
const Segment SGI268536[] = {{&S80231[123], 2}, {NULL}};
const Segment SGI268537[] = {{&S80231[35], 5}, {NULL}};
const Segment SGI268538[] = {{&S80231[68], 6}, {NULL}};
const Segment SGI268539[] = {{&S80231[35], 1}, {NULL}};
const Segment SGI268540[] = {{&S80231[150], 7}, {NULL}};
const Segment SGI268541[] = {{&S80232[13], 6}, {NULL}};
const Segment SGI268542[] = {{&S80232[123], 2}, {NULL}};
const Segment SGI268543[] = {{&S80232[35], 5}, {NULL}};
const Segment SGI268544[] = {{&S80232[68], 6}, {NULL}};
const Segment SGI268545[] = {{&S80232[35], 1}, {NULL}};
const Segment SGI268546[] = {{&S80232[150], 7}, {NULL}};
const Segment SGI268547[] = {{&S80284[27], 6}, {NULL}};
const Segment SGI268548[] = {{&S80284[51], 5}, {NULL}};
const Segment SGI268549[] = {{&S80284[84], 6}, {NULL}};
const Segment SGI268550[] = {{&S80284[118], 5}, {NULL}};
const Segment SGI268551[] = {{&S80284[131], 9}, {NULL}};
const Segment SGI268552[] = {{&S80284[0], 14}, {NULL}};
const Segment SGI268553[] = {{&S80285[27], 6}, {NULL}};
const Segment SGI268554[] = {{&S80285[52], 5}, {NULL}};
const Segment SGI268555[] = {{&S80285[85], 6}, {NULL}};
const Segment SGI268556[] = {{&S80285[119], 5}, {NULL}};
const Segment SGI268557[] = {{&S80285[132], 9}, {NULL}};
const Segment SGI268558[] = {{&S80285[0], 14}, {NULL}};
const Segment SGI268559[] = {{&S80286[28], 6}, {NULL}};
const Segment SGI268560[] = {{&S80286[52], 5}, {NULL}};
const Segment SGI268561[] = {{&S80286[85], 6}, {NULL}};
const Segment SGI268562[] = {{&S80286[119], 5}, {NULL}};
const Segment SGI268563[] = {{&S80286[132], 9}, {NULL}};
const Segment SGI268564[] = {{&S80286[0], 15}, {NULL}};
const Segment SGI268565[] = {{&S80287[28], 6}, {NULL}};
const Segment SGI268566[] = {{&S80287[53], 5}, {NULL}};
const Segment SGI268567[] = {{&S80287[86], 6}, {NULL}};
const Segment SGI268568[] = {{&S80287[120], 5}, {NULL}};
const Segment SGI268569[] = {{&S80287[133], 9}, {NULL}};
const Segment SGI268570[] = {{&S80287[0], 15}, {NULL}};
const Segment SGI268571[] = {{&S73756[63], 31}, {NULL}};
const Segment SGI268572[] = {{&S73756[0], 94}, {NULL}};
const Segment SGI268573[] = {{&S67505[10], 4}, {NULL}};
const Segment SGI268574[] = {{&S67505[15], 5}, {NULL}};
const Segment SGI268575[] = {{&S80534[13], 4}, {&S80534[13], 4}, {NULL}};
const Segment SGI268576[] = {{&S80534[116], 3}, {NULL}};
const Segment SGI268577[] = {{&S80534[29], 3}, {NULL}};
const Segment SGI268578[] = {{&S80534[81], 6}, {NULL}};
const Segment SGI268579[] = {{&S80534[127], 5}, {NULL}};
const Segment SGI268580[] = {{&S80534[140], 9}, {NULL}};
const Segment SGI268581[] = {{&S80534[150], 13}, {NULL}};
const Segment SGI268582[] = {{&S67663[10], 4}, {NULL}};
const Segment SGI268583[] = {{&S67663[15], 5}, {NULL}};
const Segment SGI268584[] = {{&S80625[13], 6}, {NULL}};
const Segment SGI268585[] = {{&S80625[35], 5}, {NULL}};
const Segment SGI268586[] = {{&S80625[68], 6}, {NULL}};
const Segment SGI268587[] = {{&S80625[102], 6}, {NULL}};
const Segment SGI268588[] = {{&S80625[116], 9}, {NULL}};
const Segment SGI268589[] = {{&S80626[13], 6}, {NULL}};
const Segment SGI268590[] = {{&S80626[35], 5}, {NULL}};
const Segment SGI268591[] = {{&S80626[68], 6}, {NULL}};
const Segment SGI268592[] = {{&S80626[102], 6}, {NULL}};
const Segment SGI268593[] = {{&S80626[116], 9}, {NULL}};
const Segment SGI268594[] = {{&S80663[30], 4}, {NULL}};
const Segment SGI268595[] = {{&S80663[46], 5}, {NULL}};
const Segment SGI268596[] = {{&S80663[86], 8}, {NULL}};
const Segment SGI268597[] = {{&S80663[122], 3}, {NULL}};
const Segment SGI268598[] = {{&S80663[133], 9}, {NULL}};
const Segment SGI268599[] = {{&S80663[0], 14}, {NULL}};
const Segment SGI268600[] = {{&S80664[30], 4}, {NULL}};
const Segment SGI268601[] = {{&S80664[46], 5}, {NULL}};
const Segment SGI268602[] = {{&S80664[86], 8}, {NULL}};
const Segment SGI268603[] = {{&S80664[122], 3}, {NULL}};
const Segment SGI268604[] = {{&S80664[133], 9}, {NULL}};
const Segment SGI268605[] = {{&S80664[0], 14}, {NULL}};
const Segment SGI268606[] = {{&S80665[28], 4}, {NULL}};
const Segment SGI268607[] = {{&S80665[44], 5}, {NULL}};
const Segment SGI268608[] = {{&S80665[84], 8}, {NULL}};
const Segment SGI268609[] = {{&S80665[120], 3}, {NULL}};
const Segment SGI268610[] = {{&S80665[131], 9}, {NULL}};
const Segment SGI268611[] = {{&S80665[0], 14}, {NULL}};
const Segment SGI268612[] = {{&S80743[13], 4}, {NULL}};
const Segment SGI268613[] = {{&S80743[113], 5}, {NULL}};
const Segment SGI268614[] = {{&S80743[36], 5}, {NULL}};
const Segment SGI268615[] = {{&S80743[76], 8}, {NULL}};
const Segment SGI268616[] = {{&S80743[126], 3}, {NULL}};
const Segment SGI268617[] = {{&S80743[137], 9}, {NULL}};
const Segment SGI268618[] = {{&S80744[13], 4}, {NULL}};
const Segment SGI268619[] = {{&S80744[113], 5}, {NULL}};
const Segment SGI268620[] = {{&S80744[36], 5}, {NULL}};
const Segment SGI268621[] = {{&S80744[76], 8}, {NULL}};
const Segment SGI268622[] = {{&S80744[126], 3}, {NULL}};
const Segment SGI268623[] = {{&S80744[137], 9}, {NULL}};
const Segment SGI268624[] = {{&S80745[13], 4}, {NULL}};
const Segment SGI268625[] = {{&S80745[117], 3}, {NULL}};
const Segment SGI268626[] = {{&S80745[42], 5}, {NULL}};
const Segment SGI268627[] = {{&S80745[82], 6}, {NULL}};
const Segment SGI268628[] = {{&S80745[128], 5}, {NULL}};
const Segment SGI268629[] = {{&S80745[141], 6}, {NULL}};
const Segment SGI268630[] = {{&S80746[13], 4}, {NULL}};
const Segment SGI268631[] = {{&S80746[117], 3}, {NULL}};
const Segment SGI268632[] = {{&S80746[42], 5}, {NULL}};
const Segment SGI268633[] = {{&S80746[82], 6}, {NULL}};
const Segment SGI268634[] = {{&S80746[128], 5}, {NULL}};
const Segment SGI268635[] = {{&S80746[141], 6}, {NULL}};
const Segment SGI268636[] = {{&S80747[13], 4}, {NULL}};
const Segment SGI268637[] = {{&S80747[117], 3}, {NULL}};
const Segment SGI268638[] = {{&S80747[42], 5}, {NULL}};
const Segment SGI268639[] = {{&S80747[82], 6}, {NULL}};
const Segment SGI268640[] = {{&S80747[128], 5}, {NULL}};
const Segment SGI268641[] = {{&S80747[141], 6}, {NULL}};
const Segment SGI268642[] = {{&S80748[13], 4}, {NULL}};
const Segment SGI268643[] = {{&S80748[117], 3}, {NULL}};
const Segment SGI268644[] = {{&S80748[42], 5}, {NULL}};
const Segment SGI268645[] = {{&S80748[82], 6}, {NULL}};
const Segment SGI268646[] = {{&S80748[128], 5}, {NULL}};
const Segment SGI268647[] = {{&S80748[141], 6}, {NULL}};
const Segment SGI268648[] = {{&S68125[10], 4}, {NULL}};
const Segment SGI268649[] = {{&S68125[15], 5}, {NULL}};
const Segment SGI268650[] = {{&S80922[13], 4}, {NULL}};
const Segment SGI268651[] = {{&S80922[29], 5}, {NULL}};
const Segment SGI268652[] = {{&S80922[69], 8}, {NULL}};
const Segment SGI268653[] = {{&S80922[120], 3}, {NULL}};
const Segment SGI268654[] = {{&S80922[131], 9}, {NULL}};
const Segment SGI268655[] = {{&S80922[98], 14}, {NULL}};
const Segment SGI268656[] = {{&S80923[13], 4}, {NULL}};
const Segment SGI268657[] = {{&S80923[29], 5}, {NULL}};
const Segment SGI268658[] = {{&S80923[69], 8}, {NULL}};
const Segment SGI268659[] = {{&S80923[120], 3}, {NULL}};
const Segment SGI268660[] = {{&S80923[131], 9}, {NULL}};
const Segment SGI268661[] = {{&S80923[98], 14}, {NULL}};
const Segment SGI268662[] = {{&S80927[13], 6}, {NULL}};
const Segment SGI268663[] = {{&S80927[35], 5}, {NULL}};
const Segment SGI268664[] = {{&S80927[75], 6}, {NULL}};
const Segment SGI268665[] = {{&S80927[109], 5}, {NULL}};
const Segment SGI268666[] = {{&S80927[115], 13}, {NULL}};
const Segment SGI268667[] = {{&S80928[13], 6}, {NULL}};
const Segment SGI268668[] = {{&S80928[35], 5}, {NULL}};
const Segment SGI268669[] = {{&S80928[75], 6}, {NULL}};
const Segment SGI268670[] = {{&S80928[109], 5}, {NULL}};
const Segment SGI268671[] = {{&S80928[115], 13}, {NULL}};
const Segment SGI268672[] = {{&S80999[13], 4}, {NULL}};
const Segment SGI268673[] = {{&S80999[117], 3}, {NULL}};
const Segment SGI268674[] = {{&S80999[42], 5}, {NULL}};
const Segment SGI268675[] = {{&S80999[82], 6}, {NULL}};
const Segment SGI268676[] = {{&S80999[128], 5}, {NULL}};
const Segment SGI268677[] = {{&S80999[141], 6}, {NULL}};
const Segment SGI268678[] = {{&S81000[13], 4}, {NULL}};
const Segment SGI268679[] = {{&S81000[115], 3}, {NULL}};
const Segment SGI268680[] = {{&S81000[42], 3}, {NULL}};
const Segment SGI268681[] = {{&S81000[80], 6}, {NULL}};
const Segment SGI268682[] = {{&S81000[126], 5}, {NULL}};
const Segment SGI268683[] = {{&S81000[139], 6}, {NULL}};
const Segment SGI268684[] = {{&S81001[27], 6}, {NULL}};
const Segment SGI268685[] = {{&S81001[52], 5}, {NULL}};
const Segment SGI268686[] = {{&S81001[85], 6}, {NULL}};
const Segment SGI268687[] = {{&S81001[119], 5}, {NULL}};
const Segment SGI268688[] = {{&S81001[132], 9}, {NULL}};
const Segment SGI268689[] = {{&S81001[0], 14}, {NULL}};
const Segment SGI268690[] = {{&S81002[27], 6}, {NULL}};
const Segment SGI268691[] = {{&S81002[51], 5}, {NULL}};
const Segment SGI268692[] = {{&S81002[84], 6}, {NULL}};
const Segment SGI268693[] = {{&S81002[118], 5}, {NULL}};
const Segment SGI268694[] = {{&S81002[131], 9}, {NULL}};
const Segment SGI268695[] = {{&S81002[0], 14}, {NULL}};
const Segment SGI268696[] = {{&S81003[27], 6}, {NULL}};
const Segment SGI268697[] = {{&S81003[52], 3}, {NULL}};
const Segment SGI268698[] = {{&S81003[83], 6}, {NULL}};
const Segment SGI268699[] = {{&S81003[117], 5}, {NULL}};
const Segment SGI268700[] = {{&S81003[130], 9}, {NULL}};
const Segment SGI268701[] = {{&S81003[0], 14}, {NULL}};
const Segment SGI268702[] = {{&S81022[13], 4}, {NULL}};
const Segment SGI268703[] = {{&S81022[33], 3}, {NULL}};
const Segment SGI268704[] = {{&S81022[129], 5}, {NULL}};
const Segment SGI268705[] = {{&S81022[64], 6}, {NULL}};
const Segment SGI268706[] = {{&S81022[97], 5}, {NULL}};
const Segment SGI268707[] = {{&S81022[110], 9}, {NULL}};
const Segment SGI268708[] = {{&S81052[28], 4}, {NULL}};
const Segment SGI268709[] = {{&S81052[44], 3}, {NULL}};
const Segment SGI268710[] = {{&S81053[28], 4}, {NULL}};
const Segment SGI268711[] = {{&S81053[44], 5}, {NULL}};
const Segment SGI268712[] = {{&S68316[10], 4}, {NULL}};
const Segment SGI268713[] = {{&S68316[15], 5}, {NULL}};
const Segment SGI268714[] = {{&S68317[10], 4}, {NULL}};
const Segment SGI268715[] = {{&S68317[15], 5}, {NULL}};
const Segment SGI268716[] = {{&S81124[13], 6}, {NULL}};
const Segment SGI268717[] = {{&S81124[38], 3}, {NULL}};
const Segment SGI268718[] = {{&S81124[76], 8}, {NULL}};
const Segment SGI268719[] = {{&S81124[112], 5}, {NULL}};
const Segment SGI268720[] = {{&S81125[13], 6}, {NULL}};
const Segment SGI268721[] = {{&S81125[38], 3}, {NULL}};
const Segment SGI268722[] = {{&S81125[76], 8}, {NULL}};
const Segment SGI268723[] = {{&S81251[13], 4}, {NULL}};
const Segment SGI268724[] = {{&S81251[101], 5}, {NULL}};
const Segment SGI268725[] = {{&S81251[33], 5}, {NULL}};
const Segment SGI268726[] = {{&S81251[66], 6}, {NULL}};
const Segment SGI268727[] = {{&S81251[114], 5}, {NULL}};
const Segment SGI268728[] = {{&S81251[127], 9}, {NULL}};
const Segment SGI268729[] = {{&S81252[13], 4}, {NULL}};
const Segment SGI268730[] = {{&S81252[99], 5}, {NULL}};
const Segment SGI268731[] = {{&S81252[33], 3}, {NULL}};
const Segment SGI268732[] = {{&S81252[64], 6}, {NULL}};
const Segment SGI268733[] = {{&S81252[112], 5}, {NULL}};
const Segment SGI268734[] = {{&S81252[125], 9}, {NULL}};
const Segment SGI268735[] = {{&S81253[13], 4}, {NULL}};
const Segment SGI268736[] = {{&S81253[113], 5}, {NULL}};
const Segment SGI268737[] = {{&S81253[36], 5}, {NULL}};
const Segment SGI268738[] = {{&S81253[76], 8}, {NULL}};
const Segment SGI268739[] = {{&S81253[126], 5}, {NULL}};
const Segment SGI268740[] = {{&S81253[139], 9}, {NULL}};
const Segment SGI268741[] = {{&S81254[13], 4}, {NULL}};
const Segment SGI268742[] = {{&S81254[113], 5}, {NULL}};
const Segment SGI268743[] = {{&S81254[36], 5}, {NULL}};
const Segment SGI268744[] = {{&S81254[76], 8}, {NULL}};
const Segment SGI268745[] = {{&S81254[126], 5}, {NULL}};
const Segment SGI268746[] = {{&S81254[139], 9}, {NULL}};
const Segment SGI268747[] = {{&S81269[17], 4}, {NULL}};
const Segment SGI268748[] = {{&S81269[36], 5}, {NULL}};
const Segment SGI268749[] = {{&S81269[62], 3}, {NULL}};
const Segment SGI268750[] = {{&S81270[17], 4}, {NULL}};
const Segment SGI268751[] = {{&S81270[36], 3}, {NULL}};
const Segment SGI268752[] = {{&S81270[60], 3}, {NULL}};
const Segment SGI268753[] = {{&S68587[10], 4}, {NULL}};
const Segment SGI268754[] = {{&S68587[15], 5}, {NULL}};
const Segment SGI268755[] = {{&S68587[17], 3}, {NULL}};
const Segment SGI268756[] = {{&S68588[10], 5}, {NULL}};
const Segment SGI268757[] = {{&S68588[16], 5}, {NULL}};
const Segment SGI268758[] = {{&S68588[18], 3}, {NULL}};
const Segment SGI268759[] = {{&S81432[27], 6}, {NULL}};
const Segment SGI268760[] = {{&S81432[52], 5}, {NULL}};
const Segment SGI268761[] = {{&S81432[85], 6}, {NULL}};
const Segment SGI268762[] = {{&S81432[119], 5}, {NULL}};
const Segment SGI268763[] = {{&S81432[132], 9}, {NULL}};
const Segment SGI268764[] = {{&S81432[0], 14}, {NULL}};
const Segment SGI268765[] = {{&S81433[27], 6}, {NULL}};
const Segment SGI268766[] = {{&S81433[51], 5}, {NULL}};
const Segment SGI268767[] = {{&S81433[84], 6}, {NULL}};
const Segment SGI268768[] = {{&S81433[118], 5}, {NULL}};
const Segment SGI268769[] = {{&S81433[131], 9}, {NULL}};
const Segment SGI268770[] = {{&S81433[0], 14}, {NULL}};
const Segment SGI268771[] = {{&S81470[17], 6}, {NULL}};
const Segment SGI268772[] = {{&S81470[35], 5}, {NULL}};
const Segment SGI268773[] = {{&S81470[61], 3}, {NULL}};
const Segment SGI268774[] = {{&S81471[17], 6}, {NULL}};
const Segment SGI268775[] = {{&S81471[35], 3}, {NULL}};
const Segment SGI268776[] = {{&S81471[59], 3}, {NULL}};
const Segment SGI268777[] = {{&S68916[10], 4}, {NULL}};
const Segment SGI268778[] = {{&S68916[15], 5}, {NULL}};
const Segment SGI268779[] = {{&S74863[0], 17}, {NULL}};
const Segment SGI268780[] = {{&S74863[0], 33}, {NULL}};
const Segment SGI268781[] = {{&S74864[85], 7}, {NULL}};
const Segment SGI268782[] = {{&S74864[0], 130}, {NULL}};
const Segment SGI268783[] = {{&S74865[0], 7}, {&S74865[76], 24}, {NULL}};
const Segment SGI268784[] = {{&S74865[0], 100}, {NULL}};
const Segment SGI268785[] = {{&S81712[27], 6}, {NULL}};
const Segment SGI268786[] = {{&S81712[51], 5}, {NULL}};
const Segment SGI268787[] = {{&S81712[84], 6}, {NULL}};
const Segment SGI268788[] = {{&S81712[118], 5}, {NULL}};
const Segment SGI268789[] = {{&S81712[131], 9}, {NULL}};
const Segment SGI268790[] = {{&S81712[0], 14}, {NULL}};
const Segment SGI268791[] = {{&S81713[27], 6}, {NULL}};
const Segment SGI268792[] = {{&S81713[52], 5}, {NULL}};
const Segment SGI268793[] = {{&S81713[85], 6}, {NULL}};
const Segment SGI268794[] = {{&S81713[119], 5}, {NULL}};
const Segment SGI268795[] = {{&S81713[132], 9}, {NULL}};
const Segment SGI268796[] = {{&S81713[0], 14}, {NULL}};
const Segment SGI268797[] = {{&S81714[28], 6}, {NULL}};
const Segment SGI268798[] = {{&S81714[52], 5}, {NULL}};
const Segment SGI268799[] = {{&S81714[85], 6}, {NULL}};
const Segment SGI268800[] = {{&S81714[119], 5}, {NULL}};
const Segment SGI268801[] = {{&S81714[132], 9}, {NULL}};
const Segment SGI268802[] = {{&S81714[0], 15}, {NULL}};
const Segment SGI268803[] = {{&S81715[28], 6}, {NULL}};
const Segment SGI268804[] = {{&S81715[53], 5}, {NULL}};
const Segment SGI268805[] = {{&S81715[86], 6}, {NULL}};
const Segment SGI268806[] = {{&S81715[120], 5}, {NULL}};
const Segment SGI268807[] = {{&S81715[133], 9}, {NULL}};
const Segment SGI268808[] = {{&S81715[0], 15}, {NULL}};
const Segment SGI268809[] = {{&S81802[27], 6}, {NULL}};
const Segment SGI268810[] = {{&S81802[51], 5}, {NULL}};
const Segment SGI268811[] = {{&S81802[84], 6}, {NULL}};
const Segment SGI268812[] = {{&S81802[118], 5}, {NULL}};
const Segment SGI268813[] = {{&S81802[131], 9}, {NULL}};
const Segment SGI268814[] = {{&S81802[0], 14}, {NULL}};
const Segment SGI268815[] = {{&S81803[27], 6}, {NULL}};
const Segment SGI268816[] = {{&S81803[52], 5}, {NULL}};
const Segment SGI268817[] = {{&S81803[85], 6}, {NULL}};
const Segment SGI268818[] = {{&S81803[119], 5}, {NULL}};
const Segment SGI268819[] = {{&S81803[132], 9}, {NULL}};
const Segment SGI268820[] = {{&S81803[0], 14}, {NULL}};
const Segment SGI268821[] = {{&S81815[13], 4}, {NULL}};
const Segment SGI268822[] = {{&S75072[59], 7}, {NULL}};
const Segment SGI268823[] = {{&S75072[0], 79}, {NULL}};
const Segment SGI268824[] = {{&S81815[94], 3}, {NULL}};
const Segment SGI268825[] = {{&S81815[26], 5}, {NULL}};
const Segment SGI268826[] = {{&S81815[59], 6}, {NULL}};
const Segment SGI268827[] = {{&S81815[105], 6}, {NULL}};
const Segment SGI268828[] = {{&S81913[13], 6}, {NULL}};
const Segment SGI268829[] = {{&S81913[38], 5}, {NULL}};
const Segment SGI268830[] = {{&S81913[71], 8}, {NULL}};
const Segment SGI268831[] = {{&S81913[119], 3}, {NULL}};
const Segment SGI268832[] = {{&S81913[130], 9}, {NULL}};
const Segment SGI268833[] = {{&S81914[13], 6}, {NULL}};
const Segment SGI268834[] = {{&S81914[38], 5}, {NULL}};
const Segment SGI268835[] = {{&S81914[71], 8}, {NULL}};
const Segment SGI268836[] = {{&S81914[119], 3}, {NULL}};
const Segment SGI268837[] = {{&S81914[130], 9}, {NULL}};
const Segment SGI268838[] = {{&S81915[13], 6}, {NULL}};
const Segment SGI268839[] = {{&S81915[38], 5}, {NULL}};
const Segment SGI268840[] = {{&S81915[71], 8}, {NULL}};
const Segment SGI268841[] = {{&S81915[119], 3}, {NULL}};
const Segment SGI268842[] = {{&S81915[130], 9}, {NULL}};
const Segment SGI268843[] = {{&S69325[10], 4}, {NULL}};
const Segment SGI268844[] = {{&S69325[15], 5}, {NULL}};
const Segment SGI268845[] = {{&S75286[48], 19}, {NULL}};
const Segment SGI268846[] = {{&S75286[0], 137}, {NULL}};
const Segment SGI268847[] = {{&S82009[27], 6}, {NULL}};
const Segment SGI268848[] = {{&S82009[52], 5}, {NULL}};
const Segment SGI268849[] = {{&S82009[85], 6}, {NULL}};
const Segment SGI268850[] = {{&S82009[119], 6}, {NULL}};
const Segment SGI268851[] = {{&S82009[133], 9}, {NULL}};
const Segment SGI268852[] = {{&S82009[0], 14}, {NULL}};
const Segment SGI268853[] = {{&S82010[27], 6}, {NULL}};
const Segment SGI268854[] = {{&S82010[52], 3}, {NULL}};
const Segment SGI268855[] = {{&S82010[83], 6}, {NULL}};
const Segment SGI268856[] = {{&S82010[117], 6}, {NULL}};
const Segment SGI268857[] = {{&S82010[131], 9}, {NULL}};
const Segment SGI268858[] = {{&S82010[0], 14}, {NULL}};
const Segment SGI268859[] = {{&S82011[27], 6}, {NULL}};
const Segment SGI268860[] = {{&S82011[51], 5}, {NULL}};
const Segment SGI268861[] = {{&S82011[84], 6}, {NULL}};
const Segment SGI268862[] = {{&S82011[118], 6}, {NULL}};
const Segment SGI268863[] = {{&S82011[132], 9}, {NULL}};
const Segment SGI268864[] = {{&S82011[0], 14}, {NULL}};
const Segment SGI268865[] = {{&S82012[27], 6}, {NULL}};
const Segment SGI268866[] = {{&S82012[51], 5}, {NULL}};
const Segment SGI268867[] = {{&S82012[84], 6}, {NULL}};
const Segment SGI268868[] = {{&S82012[118], 6}, {NULL}};
const Segment SGI268869[] = {{&S82012[132], 9}, {NULL}};
const Segment SGI268870[] = {{&S82012[0], 14}, {NULL}};
const Segment SGI268871[] = {{&S75599[93], 6}, {NULL}};
const Segment SGI268872[] = {{&S75599[0], 104}, {NULL}};
const Segment SGI268873[] = {{&S69680[20], 27}, {NULL}};
const Segment SGI268874[] = {{&S69681[20], 27}, {NULL}};
const Segment SGI268875[] = {{&S69682[20], 27}, {NULL}};
const Segment SGI268876[] = {{&S69683[20], 27}, {NULL}};
const Segment SGI268877[] = {{&S69684[20], 28}, {NULL}};
const Segment SGI268878[] = {{&S69685[20], 28}, {NULL}};
const Segment SGI268879[] = {{&S69686[20], 28}, {NULL}};
const Segment SGI268880[] = {{&S82107[27], 6}, {NULL}};
const Segment SGI268881[] = {{&S82107[49], 3}, {NULL}};
const Segment SGI268882[] = {{&S82107[80], 6}, {NULL}};
const Segment SGI268883[] = {{&S82107[114], 5}, {NULL}};
const Segment SGI268884[] = {{&S82107[127], 9}, {NULL}};
const Segment SGI268885[] = {{&S82107[0], 14}, {NULL}};
const Segment SGI268886[] = {{&S82108[27], 6}, {NULL}};
const Segment SGI268887[] = {{&S82108[49], 3}, {NULL}};
const Segment SGI268888[] = {{&S82108[80], 6}, {NULL}};
const Segment SGI268889[] = {{&S82108[114], 5}, {NULL}};
const Segment SGI268890[] = {{&S82108[127], 9}, {NULL}};
const Segment SGI268891[] = {{&S82108[0], 14}, {NULL}};
const Segment SGI268892[] = {{&S82109[27], 6}, {NULL}};
const Segment SGI268893[] = {{&S82109[49], 5}, {NULL}};
const Segment SGI268894[] = {{&S82109[82], 6}, {NULL}};
const Segment SGI268895[] = {{&S82109[116], 5}, {NULL}};
const Segment SGI268896[] = {{&S82109[129], 9}, {NULL}};
const Segment SGI268897[] = {{&S82109[0], 14}, {NULL}};
const Segment SGI268898[] = {{&S82207[17], 6}, {NULL}};
const Segment SGI268899[] = {{&S82207[35], 5}, {NULL}};
const Segment SGI268900[] = {{&S82207[61], 3}, {NULL}};
const Segment SGI268901[] = {{&S82208[17], 6}, {NULL}};
const Segment SGI268902[] = {{&S82208[35], 3}, {NULL}};
const Segment SGI268903[] = {{&S82208[59], 3}, {NULL}};
const Segment SGI268904[] = {{&S82296[27], 6}, {NULL}};
const Segment SGI268905[] = {{&S82296[51], 5}, {NULL}};
const Segment SGI268906[] = {{&S82296[84], 6}, {NULL}};
const Segment SGI268907[] = {{&S82296[118], 5}, {NULL}};
const Segment SGI268908[] = {{&S82296[131], 9}, {NULL}};
const Segment SGI268909[] = {{&S82296[0], 14}, {NULL}};
const Segment SGI268910[] = {{&S82297[27], 6}, {NULL}};
const Segment SGI268911[] = {{&S82297[52], 5}, {NULL}};
const Segment SGI268912[] = {{&S82297[85], 6}, {NULL}};
const Segment SGI268913[] = {{&S82297[119], 5}, {NULL}};
const Segment SGI268914[] = {{&S82297[132], 9}, {NULL}};
const Segment SGI268915[] = {{&S82297[0], 14}, {NULL}};
const Segment SGI268916[] = {{&S82298[28], 6}, {NULL}};
const Segment SGI268917[] = {{&S82298[52], 5}, {NULL}};
const Segment SGI268918[] = {{&S82298[85], 6}, {NULL}};
const Segment SGI268919[] = {{&S82298[119], 5}, {NULL}};
const Segment SGI268920[] = {{&S82298[132], 9}, {NULL}};
const Segment SGI268921[] = {{&S82298[0], 15}, {NULL}};
const Segment SGI268922[] = {{&S82299[28], 6}, {NULL}};
const Segment SGI268923[] = {{&S82299[53], 5}, {NULL}};
const Segment SGI268924[] = {{&S82299[86], 6}, {NULL}};
const Segment SGI268925[] = {{&S82299[120], 5}, {NULL}};
const Segment SGI268926[] = {{&S82299[133], 9}, {NULL}};
const Segment SGI268927[] = {{&S82299[0], 15}, {NULL}};
const Segment SGI268928[] = {{&S82377[13], 6}, {NULL}};
const Segment SGI268929[] = {{&S82377[35], 5}, {NULL}};
const Segment SGI268930[] = {{&S82377[68], 6}, {NULL}};
const Segment SGI268931[] = {{&S82377[126], 5}, {NULL}};
const Segment SGI268932[] = {{&S82377[139], 9}, {NULL}};
const Segment SGI268933[] = {{&S82378[13], 6}, {NULL}};
const Segment SGI268934[] = {{&S82378[35], 3}, {NULL}};
const Segment SGI268935[] = {{&S82378[66], 6}, {NULL}};
const Segment SGI268936[] = {{&S82378[124], 5}, {NULL}};
const Segment SGI268937[] = {{&S82378[137], 9}, {NULL}};
const Segment SGI268938[] = {{&S82389[28], 4}, {NULL}};
const Segment SGI268939[] = {{&S82389[41], 5}, {NULL}};
const Segment SGI268940[] = {{&S82389[74], 6}, {NULL}};
const Segment SGI268941[] = {{&S82389[108], 5}, {NULL}};
const Segment SGI268942[] = {{&S82389[121], 9}, {NULL}};
const Segment SGI268943[] = {{&S82389[0], 14}, {NULL}};
const Segment SGI268944[] = {{&S82506[13], 4}, {NULL}};
const Segment SGI268945[] = {{&S82506[94], 3}, {NULL}};
const Segment SGI268946[] = {{&S82506[26], 5}, {NULL}};
const Segment SGI268947[] = {{&S82506[59], 6}, {NULL}};
const Segment SGI268948[] = {{&S82506[105], 6}, {NULL}};
const Segment SGI268949[] = {{&S82588[13], 4}, {NULL}};
const Segment SGI268950[] = {{&S82588[26], 5}, {NULL}};
const Segment SGI268951[] = {{&S82588[59], 6}, {NULL}};
const Segment SGI268952[] = {{&S82588[108], 5}, {NULL}};
const Segment SGI268953[] = {{&S82588[121], 9}, {NULL}};
const Segment SGI268954[] = {{&S82588[86], 14}, {NULL}};
const Segment SGI268955[] = {{&S82617[28], 4}, {NULL}};
const Segment SGI268956[] = {{&S82617[44], 5}, {NULL}};
const Segment SGI268957[] = {{&S82617[84], 8}, {NULL}};
const Segment SGI268958[] = {{&S82617[120], 3}, {NULL}};
const Segment SGI268959[] = {{&S82617[131], 9}, {NULL}};
const Segment SGI268960[] = {{&S82617[0], 14}, {NULL}};
const Segment SGI268961[] = {{&S82618[28], 4}, {NULL}};
const Segment SGI268962[] = {{&S82618[44], 5}, {NULL}};
const Segment SGI268963[] = {{&S82618[84], 8}, {NULL}};
const Segment SGI268964[] = {{&S82618[120], 3}, {NULL}};
const Segment SGI268965[] = {{&S82618[131], 9}, {NULL}};
const Segment SGI268966[] = {{&S82618[0], 14}, {NULL}};
const Segment SGI268967[] = {{&S82664[13], 6}, {NULL}};
const Segment SGI268968[] = {{&S82664[110], 5}, {NULL}};
const Segment SGI268969[] = {{&S82664[38], 3}, {NULL}};
const Segment SGI268970[] = {{&S82664[76], 5}, {NULL}};
const Segment SGI268971[] = {{&S82664[123], 5}, {NULL}};
const Segment SGI268972[] = {{&S82665[13], 6}, {NULL}};
const Segment SGI268973[] = {{&S82665[111], 3}, {NULL}};
const Segment SGI268974[] = {{&S82665[38], 3}, {NULL}};
const Segment SGI268975[] = {{&S82665[76], 6}, {NULL}};
const Segment SGI268976[] = {{&S82665[122], 5}, {NULL}};
const Segment SGI268977[] = {{&S82665[135], 6}, {NULL}};
const Segment SGI268978[] = {{&S82666[13], 6}, {NULL}};
const Segment SGI268979[] = {{&S82666[110], 5}, {NULL}};
const Segment SGI268980[] = {{&S82666[38], 3}, {NULL}};
const Segment SGI268981[] = {{&S82666[76], 5}, {NULL}};
const Segment SGI268982[] = {{&S82666[123], 5}, {NULL}};
const Segment SGI268983[] = {{&S82666[136], 9}, {NULL}};
const Segment SGI268984[] = {{&S82757[13], 6}, {NULL}};
const Segment SGI268985[] = {{&S82757[35], 5}, {NULL}};
const Segment SGI268986[] = {{&S82757[134], 5}, {NULL}};
const Segment SGI268987[] = {{&S82757[68], 6}, {NULL}};
const Segment SGI268988[] = {{&S82757[102], 5}, {NULL}};
const Segment SGI268989[] = {{&S82757[115], 9}, {NULL}};
const Segment SGI268990[] = {{&S82758[13], 6}, {NULL}};
const Segment SGI268991[] = {{&S82758[35], 5}, {NULL}};
const Segment SGI268992[] = {{&S82758[134], 5}, {NULL}};
const Segment SGI268993[] = {{&S82758[68], 6}, {NULL}};
const Segment SGI268994[] = {{&S82758[102], 5}, {NULL}};
const Segment SGI268995[] = {{&S82758[115], 9}, {NULL}};
const Segment SGI268996[] = {{&S82759[13], 6}, {NULL}};
const Segment SGI268997[] = {{&S82759[35], 5}, {NULL}};
const Segment SGI268998[] = {{&S82759[134], 5}, {NULL}};
const Segment SGI268999[] = {{&S82759[68], 6}, {NULL}};
const Segment SGI269000[] = {{&S82759[102], 5}, {NULL}};
const Segment SGI269001[] = {{&S82759[115], 9}, {NULL}};
const Segment SGI269002[] = {{&S82948[13], 4}, {NULL}};
const Segment SGI269003[] = {{&S82948[104], 5}, {NULL}};
const Segment SGI269004[] = {{&S82948[29], 3}, {NULL}};
const Segment SGI269005[] = {{&S82948[67], 8}, {NULL}};
const Segment SGI269006[] = {{&S82948[117], 5}, {NULL}};
const Segment SGI269007[] = {{&S82948[130], 9}, {NULL}};
const Segment SGI269008[] = {{&S82949[13], 4}, {NULL}};
const Segment SGI269009[] = {{&S82949[104], 5}, {NULL}};
const Segment SGI269010[] = {{&S82949[29], 3}, {NULL}};
const Segment SGI269011[] = {{&S82949[67], 8}, {NULL}};
const Segment SGI269012[] = {{&S82949[117], 5}, {&S82949[71], 1}, {NULL}};
const Segment SGI269013[] = {{&S82949[150], 9}, {NULL}};
const Segment SGI269014[] = {{&S82950[13], 4}, {NULL}};
const Segment SGI269015[] = {{&S82950[26], 3}, {NULL}};
const Segment SGI269016[] = {{&S82950[57], 8}, {NULL}};
const Segment SGI269017[] = {{&S82950[93], 6}, {NULL}};
const Segment SGI269018[] = {{&S82950[107], 9}, {NULL}};
const Segment SGI269019[] = {{&S82951[13], 4}, {NULL}};
const Segment SGI269020[] = {{&S82951[26], 5}, {NULL}};
const Segment SGI269021[] = {{&S82951[59], 8}, {NULL}};
const Segment SGI269022[] = {{&S82951[95], 6}, {NULL}};
const Segment SGI269023[] = {{&S82951[109], 9}, {NULL}};
const Segment SGI269024[] = {{&S83012[28], 4}, {NULL}};
const Segment SGI269025[] = {{&S83012[41], 3}, {NULL}};
const Segment SGI269026[] = {{&S83012[72], 6}, {NULL}};
const Segment SGI269027[] = {{&S83012[106], 6}, {NULL}};
const Segment SGI269028[] = {{&S83012[120], 9}, {NULL}};
const Segment SGI269029[] = {{&S83012[0], 14}, {NULL}};
const Segment SGI269030[] = {{&S83013[28], 4}, {NULL}};
const Segment SGI269031[] = {{&S83013[41], 5}, {NULL}};
const Segment SGI269032[] = {{&S83013[74], 6}, {NULL}};
const Segment SGI269033[] = {{&S83013[108], 6}, {NULL}};
const Segment SGI269034[] = {{&S83013[122], 9}, {NULL}};
const Segment SGI269035[] = {{&S83013[0], 14}, {NULL}};
const Segment SGI269036[] = {{&S83014[28], 4}, {NULL}};
const Segment SGI269037[] = {{&S83014[41], 5}, {NULL}};
const Segment SGI269038[] = {{&S83014[74], 6}, {NULL}};
const Segment SGI269039[] = {{&S83014[108], 6}, {NULL}};
const Segment SGI269040[] = {{&S83014[122], 9}, {NULL}};
const Segment SGI269041[] = {{&S83014[0], 14}, {NULL}};
const Segment SGI269042[] = {{&S83051[28], 4}, {NULL}};
const Segment SGI269043[] = {{&S83051[44], 5}, {NULL}};
const Segment SGI269044[] = {{&S83051[84], 8}, {NULL}};
const Segment SGI269045[] = {{&S83051[120], 3}, {NULL}};
const Segment SGI269046[] = {{&S83051[131], 9}, {NULL}};
const Segment SGI269047[] = {{&S83051[0], 14}, {NULL}};
const Segment SGI269048[] = {{&S83052[28], 4}, {NULL}};
const Segment SGI269049[] = {{&S83052[44], 5}, {NULL}};
const Segment SGI269050[] = {{&S83052[84], 8}, {NULL}};
const Segment SGI269051[] = {{&S83052[120], 3}, {NULL}};
const Segment SGI269052[] = {{&S83052[131], 9}, {NULL}};
const Segment SGI269053[] = {{&S83052[0], 14}, {NULL}};
const Segment SGI269054[] = {{&S83104[13], 4}, {NULL}};
const Segment SGI269055[] = {{&S83104[99], 3}, {NULL}};
const Segment SGI269056[] = {{&S83104[26], 3}, {NULL}};
const Segment SGI269057[] = {{&S83104[64], 6}, {NULL}};
const Segment SGI269058[] = {{&S83104[110], 5}, {NULL}};
const Segment SGI269059[] = {{&S83104[123], 5}, {NULL}};
const Segment SGI269060[] = {{&S83184[27], 6}, {NULL}};
const Segment SGI269061[] = {{&S83184[52], 5}, {NULL}};
const Segment SGI269062[] = {{&S83184[85], 6}, {NULL}};
const Segment SGI269063[] = {{&S83184[119], 5}, {NULL}};
const Segment SGI269064[] = {{&S83184[132], 9}, {NULL}};
const Segment SGI269065[] = {{&S83184[0], 14}, {NULL}};
const Segment SGI269066[] = {{&S83185[27], 6}, {NULL}};
const Segment SGI269067[] = {{&S83185[51], 5}, {NULL}};
const Segment SGI269068[] = {{&S83185[84], 6}, {NULL}};
const Segment SGI269069[] = {{&S83185[118], 5}, {NULL}};
const Segment SGI269070[] = {{&S83185[131], 9}, {NULL}};
const Segment SGI269071[] = {{&S83185[0], 14}, {NULL}};
const Segment SGI269072[] = {{&S83192[13], 6}, {NULL}};
const Segment SGI269073[] = {{&S83192[35], 5}, {NULL}};
const Segment SGI269074[] = {{&S83192[75], 6}, {NULL}};
const Segment SGI269075[] = {{&S83192[109], 5}, {NULL}};
const Segment SGI269076[] = {{&S83192[115], 13}, {NULL}};
const Segment SGI269077[] = {{&S83193[13], 6}, {NULL}};
const Segment SGI269078[] = {{&S83193[35], 5}, {NULL}};
const Segment SGI269079[] = {{&S83193[75], 6}, {NULL}};
const Segment SGI269080[] = {{&S83193[109], 5}, {NULL}};
const Segment SGI269081[] = {{&S83193[115], 13}, {NULL}};
const Segment SGI269082[] = {{&S83405[13], 6}, {NULL}};
const Segment SGI269083[] = {{&S83405[113], 3}, {NULL}};
const Segment SGI269084[] = {{&S83405[38], 5}, {NULL}};
const Segment SGI269085[] = {{&S83405[78], 6}, {NULL}};
const Segment SGI269086[] = {{&S83405[124], 5}, {NULL}};
const Segment SGI269087[] = {{&S83405[137], 9}, {NULL}};
const Segment SGI269088[] = {{&S83406[13], 6}, {NULL}};
const Segment SGI269089[] = {{&S83406[111], 3}, {NULL}};
const Segment SGI269090[] = {{&S83406[38], 3}, {NULL}};
const Segment SGI269091[] = {{&S83406[76], 6}, {NULL}};
const Segment SGI269092[] = {{&S83406[122], 5}, {NULL}};
const Segment SGI269093[] = {{&S83406[135], 9}, {NULL}};
const Segment SGI269094[] = {{&S76372[105], 36}, {NULL}};
const Segment SGI269095[] = {{&S76372[0], 143}, {NULL}};
const Segment SGI269096[] = {{&S83416[13], 6}, {NULL}};
const Segment SGI269097[] = {{&S83416[111], 3}, {NULL}};
const Segment SGI269098[] = {{&S83416[38], 3}, {NULL}};
const Segment SGI269099[] = {{&S83416[76], 6}, {NULL}};
const Segment SGI269100[] = {{&S83416[122], 6}, {NULL}};
const Segment SGI269101[] = {{&S83416[136], 9}, {NULL}};
const Segment SGI269102[] = {{&S83417[13], 6}, {NULL}};
const Segment SGI269103[] = {{&S83417[113], 3}, {NULL}};
const Segment SGI269104[] = {{&S83417[38], 5}, {NULL}};
const Segment SGI269105[] = {{&S83417[78], 6}, {NULL}};
const Segment SGI269106[] = {{&S83417[124], 6}, {NULL}};
const Segment SGI269107[] = {{&S83417[138], 9}, {NULL}};
const Segment SGI269108[] = {{&S83445[13], 4}, {&S83445[23], 6}, {NULL}};
const Segment SGI269109[] = {{&S83445[199], 3}, {NULL}};
const Segment SGI269110[] = {{&S83445[33], 5}, {NULL}};
const Segment SGI269111[] = {{&S83445[66], 6}, {&S83445[166], 3}, {NULL}};
const Segment SGI269112[] = {{&S83445[100], 6}, {&S83445[210], 3}, {NULL}};
const Segment SGI269113[] = {{&S83445[221], 5}, {NULL}};
const Segment SGI269114[] = {{&S83446[13], 4}, {&S83446[23], 6}, {NULL}};
const Segment SGI269115[] = {{&S83446[199], 3}, {NULL}};
const Segment SGI269116[] = {{&S83446[33], 5}, {NULL}};
const Segment SGI269117[] = {{&S83446[66], 6}, {&S83446[166], 3}, {NULL}};
const Segment SGI269118[] = {{&S83446[100], 6}, {&S83446[210], 3}, {NULL}};
const Segment SGI269119[] = {{&S83446[221], 5}, {NULL}};
const Segment SGI269120[] = {{&S83447[13], 4}, {&S83447[23], 6}, {NULL}};
const Segment SGI269121[] = {{&S83447[197], 3}, {NULL}};
const Segment SGI269122[] = {{&S83447[33], 3}, {NULL}};
const Segment SGI269123[] = {{&S83447[64], 6}, {&S83447[164], 3}, {NULL}};
const Segment SGI269124[] = {{&S83447[98], 6}, {&S83447[208], 3}, {NULL}};
const Segment SGI269125[] = {{&S83447[219], 5}, {NULL}};
const Segment SGI269126[] = {{&S83562[13], 6}, {NULL}};
const Segment SGI269127[] = {{&S83562[35], 3}, {NULL}};
const Segment SGI269128[] = {{&S83562[73], 8}, {NULL}};
const Segment SGI269129[] = {{&S83562[109], 6}, {NULL}};
const Segment SGI269130[] = {{&S83562[116], 13}, {NULL}};
const Segment SGI269131[] = {{&S83563[13], 6}, {NULL}};
const Segment SGI269132[] = {{&S83563[35], 5}, {NULL}};
const Segment SGI269133[] = {{&S83563[75], 8}, {NULL}};
const Segment SGI269134[] = {{&S83563[111], 6}, {NULL}};
const Segment SGI269135[] = {{&S83563[118], 13}, {NULL}};
const Segment SGI269136[] = {{&S83587[30], 4}, {NULL}};
const Segment SGI269137[] = {{&S83587[43], 5}, {NULL}};
const Segment SGI269138[] = {{&S83587[76], 6}, {NULL}};
const Segment SGI269139[] = {{&S83587[110], 5}, {NULL}};
const Segment SGI269140[] = {{&S83587[123], 9}, {NULL}};
const Segment SGI269141[] = {{&S83587[0], 14}, {NULL}};
const Segment SGI269142[] = {{&S83821[13], 4}, {NULL}};
const Segment SGI269143[] = {{&S83821[29], 5}, {NULL}};
const Segment SGI269144[] = {{&S83821[69], 8}, {NULL}};
const Segment SGI269145[] = {{&S83821[129], 5}, {NULL}};
const Segment SGI269146[] = {{&S83821[142], 9}, {NULL}};
const Segment SGI269147[] = {{&S83822[13], 4}, {NULL}};
const Segment SGI269148[] = {{&S83822[29], 5}, {NULL}};
const Segment SGI269149[] = {{&S83822[69], 8}, {NULL}};
const Segment SGI269150[] = {{&S83822[129], 5}, {NULL}};
const Segment SGI269151[] = {{&S83822[142], 9}, {NULL}};
const Segment SGI269152[] = {{&S83839[13], 6}, {NULL}};
const Segment SGI269153[] = {{&S83839[93], 3}, {NULL}};
const Segment SGI269154[] = {{&S83839[58], 6}, {NULL}};
const Segment SGI269155[] = {{&S83839[104], 5}, {NULL}};
const Segment SGI269156[] = {{&S83839[117], 9}, {NULL}};
const Segment SGI269157[] = {{&S76790[59], 5}, {NULL}};
const Segment SGI269158[] = {{&S76790[0], 64}, {NULL}};
const Segment SGI269159[] = {{&S76791[59], 5}, {&S76791[59], 5}, {NULL}};
const Segment SGI269160[] = {{&S76791[0], 70}, {NULL}};
const Segment SGI269161[] = {{&S84005[17], 4}, {NULL}};
const Segment SGI269162[] = {{&S84005[39], 3}, {NULL}};
const Segment SGI269163[] = {{&S84005[63], 3}, {NULL}};
const Segment SGI269164[] = {{&S84006[17], 4}, {NULL}};
const Segment SGI269165[] = {{&S84006[39], 5}, {NULL}};
const Segment SGI269166[] = {{&S84006[65], 3}, {NULL}};
const Segment SGI269167[] = {{&S76992[109], 5}, {NULL}};
const Segment SGI269168[] = {{&S76992[0], 114}, {NULL}};
const Segment SGI269169[] = {{&S76993[109], 5}, {NULL}};
const Segment SGI269170[] = {{&S76993[0], 114}, {NULL}};
const Segment SGI269171[] = {{&S84053[13], 4}, {NULL}};
const Segment SGI269172[] = {{&S84053[26], 3}, {NULL}};
const Segment SGI269173[] = {{&S84053[57], 8}, {NULL}};
const Segment SGI269174[] = {{&S84053[117], 5}, {NULL}};
const Segment SGI269175[] = {{&S84053[130], 9}, {NULL}};
const Segment SGI269176[] = {{&S76994[109], 5}, {NULL}};
const Segment SGI269177[] = {{&S76994[0], 114}, {NULL}};
const Segment SGI269178[] = {{&S84054[13], 4}, {NULL}};
const Segment SGI269179[] = {{&S84054[26], 5}, {NULL}};
const Segment SGI269180[] = {{&S84054[59], 8}, {NULL}};
const Segment SGI269181[] = {{&S84054[119], 5}, {NULL}};
const Segment SGI269182[] = {{&S84054[132], 9}, {NULL}};
const Segment SGI269183[] = {{&S76995[109], 5}, {NULL}};
const Segment SGI269184[] = {{&S76995[0], 114}, {NULL}};
const Segment SGI269185[] = {{&S84079[27], 6}, {NULL}};
const Segment SGI269186[] = {{&S84079[49], 5}, {NULL}};
const Segment SGI269187[] = {{&S84079[82], 6}, {NULL}};
const Segment SGI269188[] = {{&S84079[116], 5}, {NULL}};
const Segment SGI269189[] = {{&S84079[129], 9}, {NULL}};
const Segment SGI269190[] = {{&S84079[0], 14}, {NULL}};
const Segment SGI269191[] = {{&S84080[27], 6}, {NULL}};
const Segment SGI269192[] = {{&S84080[49], 3}, {NULL}};
const Segment SGI269193[] = {{&S84080[80], 6}, {NULL}};
const Segment SGI269194[] = {{&S84080[114], 5}, {NULL}};
const Segment SGI269195[] = {{&S84080[127], 9}, {NULL}};
const Segment SGI269196[] = {{&S84080[0], 14}, {NULL}};
const Segment SGI269197[] = {{&S84081[27], 6}, {NULL}};
const Segment SGI269198[] = {{&S84081[49], 5}, {NULL}};
const Segment SGI269199[] = {{&S84081[82], 6}, {NULL}};
const Segment SGI269200[] = {{&S84081[116], 5}, {NULL}};
const Segment SGI269201[] = {{&S84081[129], 9}, {NULL}};
const Segment SGI269202[] = {{&S84081[0], 14}, {NULL}};
const Segment SGI269203[] = {{&S84082[29], 6}, {NULL}};
const Segment SGI269204[] = {{&S84082[51], 5}, {NULL}};
const Segment SGI269205[] = {{&S84082[84], 6}, {NULL}};
const Segment SGI269206[] = {{&S84082[118], 5}, {NULL}};
const Segment SGI269207[] = {{&S84082[131], 9}, {NULL}};
const Segment SGI269208[] = {{&S84082[0], 16}, {NULL}};
const Segment SGI269209[] = {{&S84118[13], 4}, {NULL}};
const Segment SGI269210[] = {{&S84118[26], 5}, {NULL}};
const Segment SGI269211[] = {{&S84118[59], 6}, {NULL}};
const Segment SGI269212[] = {{&S84118[113], 6}, {NULL}};
const Segment SGI269213[] = {{&S84118[127], 7}, {NULL}};
const Segment SGI269214[] = {{&S84119[13], 6}, {NULL}};
const Segment SGI269215[] = {{&S84119[28], 5}, {NULL}};
const Segment SGI269216[] = {{&S84119[61], 6}, {NULL}};
const Segment SGI269217[] = {{&S84119[115], 6}, {NULL}};
const Segment SGI269218[] = {{&S84119[129], 7}, {NULL}};
const Segment SGI269219[] = {{&S84121[28], 6}, {NULL}};
const Segment SGI269220[] = {{&S84121[53], 3}, {NULL}};
const Segment SGI269221[] = {{&S84121[84], 6}, {NULL}};
const Segment SGI269222[] = {{&S84121[118], 6}, {NULL}};
const Segment SGI269223[] = {{&S84121[132], 9}, {NULL}};
const Segment SGI269224[] = {{&S84121[0], 15}, {NULL}};
const Segment SGI269225[] = {{&S84122[28], 6}, {NULL}};
const Segment SGI269226[] = {{&S84122[52], 3}, {NULL}};
const Segment SGI269227[] = {{&S84122[83], 6}, {NULL}};
const Segment SGI269228[] = {{&S84122[117], 6}, {NULL}};
const Segment SGI269229[] = {{&S84122[131], 9}, {NULL}};
const Segment SGI269230[] = {{&S84122[0], 15}, {NULL}};
const Segment SGI269231[] = {{&S84123[28], 6}, {NULL}};
const Segment SGI269232[] = {{&S84123[53], 5}, {NULL}};
const Segment SGI269233[] = {{&S84123[86], 6}, {NULL}};
const Segment SGI269234[] = {{&S84123[120], 6}, {NULL}};
const Segment SGI269235[] = {{&S84123[134], 9}, {NULL}};
const Segment SGI269236[] = {{&S84123[0], 15}, {NULL}};
const Segment SGI269237[] = {{&S77068[84], 13}, {&S77068[98], 22}, {NULL}};
const Segment SGI269238[] = {{&S77068[0], 120}, {NULL}};
const Segment SGI269239[] = {{&S71336[61], 4}, {NULL}};
const Segment SGI269240[] = {{&S71336[66], 5}, {NULL}};
const Segment SGI269241[] = {{&S84153[13], 6}, {NULL}};
const Segment SGI269242[] = {{&S84153[103], 3}, {NULL}};
const Segment SGI269243[] = {{&S84153[35], 3}, {NULL}};
const Segment SGI269244[] = {{&S84153[66], 8}, {NULL}};
const Segment SGI269245[] = {{&S84153[114], 5}, {NULL}};
const Segment SGI269246[] = {{&S84153[127], 7}, {NULL}};
const Segment SGI269247[] = {{&S84154[13], 6}, {NULL}};
const Segment SGI269248[] = {{&S84154[103], 3}, {NULL}};
const Segment SGI269249[] = {{&S84154[35], 3}, {NULL}};
const Segment SGI269250[] = {{&S84154[66], 8}, {NULL}};
const Segment SGI269251[] = {{&S84154[114], 5}, {NULL}};
const Segment SGI269252[] = {{&S84154[127], 7}, {NULL}};
const Segment SGI269253[] = {{&S84155[13], 6}, {NULL}};
const Segment SGI269254[] = {{&S84155[103], 3}, {NULL}};
const Segment SGI269255[] = {{&S84155[35], 3}, {NULL}};
const Segment SGI269256[] = {{&S84155[66], 8}, {NULL}};
const Segment SGI269257[] = {{&S84155[114], 5}, {NULL}};
const Segment SGI269258[] = {{&S84155[127], 7}, {NULL}};
const Segment SGI269259[] = {{&S84156[13], 6}, {NULL}};
const Segment SGI269260[] = {{&S84156[103], 3}, {NULL}};
const Segment SGI269261[] = {{&S84156[35], 3}, {NULL}};
const Segment SGI269262[] = {{&S84156[66], 8}, {NULL}};
const Segment SGI269263[] = {{&S84156[114], 5}, {NULL}};
const Segment SGI269264[] = {{&S84156[127], 7}, {NULL}};
const Segment SGI269265[] = {{&S84157[13], 6}, {NULL}};
const Segment SGI269266[] = {{&S84157[103], 3}, {NULL}};
const Segment SGI269267[] = {{&S84157[35], 3}, {NULL}};
const Segment SGI269268[] = {{&S84157[66], 8}, {NULL}};
const Segment SGI269269[] = {{&S84157[114], 5}, {NULL}};
const Segment SGI269270[] = {{&S84157[127], 7}, {NULL}};
const Segment SGI269271[] = {{&S84158[13], 6}, {NULL}};
const Segment SGI269272[] = {{&S84158[103], 3}, {NULL}};
const Segment SGI269273[] = {{&S84158[35], 3}, {NULL}};
const Segment SGI269274[] = {{&S84158[66], 8}, {NULL}};
const Segment SGI269275[] = {{&S84158[114], 5}, {NULL}};
const Segment SGI269276[] = {{&S84158[127], 7}, {NULL}};
const Segment SGI269277[] = {{&S84159[13], 6}, {NULL}};
const Segment SGI269278[] = {{&S84159[103], 3}, {NULL}};
const Segment SGI269279[] = {{&S84159[35], 3}, {NULL}};
const Segment SGI269280[] = {{&S84159[66], 8}, {NULL}};
const Segment SGI269281[] = {{&S84159[114], 5}, {NULL}};
const Segment SGI269282[] = {{&S84159[127], 7}, {NULL}};
const Segment SGI269283[] = {{&S84160[13], 6}, {NULL}};
const Segment SGI269284[] = {{&S84160[103], 3}, {NULL}};
const Segment SGI269285[] = {{&S84160[35], 3}, {NULL}};
const Segment SGI269286[] = {{&S84160[66], 8}, {NULL}};
const Segment SGI269287[] = {{&S84160[114], 5}, {NULL}};
const Segment SGI269288[] = {{&S84160[127], 7}, {NULL}};
const Segment SGI269289[] = {{&S84194[13], 4}, {&S84194[13], 4}, {NULL}};
const Segment SGI269290[] = {{&S84194[29], 5}, {NULL}};
const Segment SGI269291[] = {{&S84194[69], 8}, {NULL}};
const Segment SGI269292[] = {{&S84194[105], 3}, {NULL}};
const Segment SGI269293[] = {{&S84194[116], 9}, {NULL}};
const Segment SGI269294[] = {{&S84195[13], 4}, {&S84195[13], 4}, {NULL}};
const Segment SGI269295[] = {{&S84195[29], 5}, {NULL}};
const Segment SGI269296[] = {{&S84195[69], 8}, {NULL}};
const Segment SGI269297[] = {{&S84195[105], 3}, {NULL}};
const Segment SGI269298[] = {{&S84195[116], 9}, {NULL}};
const Segment SGI269299[] = {{&S84268[17], 4}, {NULL}};
const Segment SGI269300[] = {{&S84268[33], 5}, {NULL}};
const Segment SGI269301[] = {{&S84269[17], 4}, {NULL}};
const Segment SGI269302[] = {{&S84269[33], 3}, {NULL}};
const Segment SGI269303[] = {{&S84329[13], 6}, {NULL}};
const Segment SGI269304[] = {{&S84329[35], 5}, {NULL}};
const Segment SGI269305[] = {{&S84329[75], 6}, {NULL}};
const Segment SGI269306[] = {{&S84329[109], 5}, {NULL}};
const Segment SGI269307[] = {{&S84329[115], 13}, {NULL}};
const Segment SGI269308[] = {{&S84330[13], 6}, {NULL}};
const Segment SGI269309[] = {{&S84330[35], 5}, {NULL}};
const Segment SGI269310[] = {{&S84330[75], 6}, {NULL}};
const Segment SGI269311[] = {{&S84330[109], 5}, {NULL}};
const Segment SGI269312[] = {{&S84330[115], 13}, {NULL}};
const Segment SGI269313[] = {{&S71551[10], 4}, {NULL}};
const Segment SGI269314[] = {{&S71551[15], 5}, {NULL}};
const Segment SGI269315[] = {{&S84413[13], 4}, {&S84413[13], 4}, {NULL}};
const Segment SGI269316[] = {{&S84413[33], 5}, {NULL}};
const Segment SGI269317[] = {{&S84413[66], 6}, {NULL}};
const Segment SGI269318[] = {{&S84413[100], 6}, {NULL}};
const Segment SGI269319[] = {{&S84413[114], 9}, {NULL}};
const Segment SGI269320[] = {{&S84414[13], 4}, {&S84414[13], 4}, {NULL}};
const Segment SGI269321[] = {{&S84414[33], 3}, {NULL}};
const Segment SGI269322[] = {{&S84414[64], 6}, {NULL}};
const Segment SGI269323[] = {{&S84414[98], 6}, {NULL}};
const Segment SGI269324[] = {{&S84414[112], 9}, {NULL}};
const Segment SGI269325[] = {{&S84506[13], 6}, {NULL}};
const Segment SGI269326[] = {{&S84506[110], 3}, {NULL}};
const Segment SGI269327[] = {{&S84506[38], 3}, {NULL}};
const Segment SGI269328[] = {{&S84506[76], 5}, {NULL}};
const Segment SGI269329[] = {{&S84506[121], 5}, {NULL}};
const Segment SGI269330[] = {{&S84506[134], 9}, {NULL}};
const Segment SGI269331[] = {{&S77494[59], 5}, {NULL}};
const Segment SGI269332[] = {{&S77494[0], 64}, {NULL}};
const Segment SGI269333[] = {{&S77495[59], 5}, {&S77495[59], 5}, {NULL}};
const Segment SGI269334[] = {{&S77495[0], 70}, {NULL}};
const Segment SGI269335[] = {{&S84687[13], 4}, {NULL}};
const Segment SGI269336[] = {{&S84687[26], 5}, {NULL}};
const Segment SGI269337[] = {{&S84687[59], 6}, {NULL}};
const Segment SGI269338[] = {{&S84687[117], 5}, {NULL}};
const Segment SGI269339[] = {{&S84687[130], 9}, {NULL}};
const Segment SGI269340[] = {{&S84729[13], 6}, {NULL}};
const Segment SGI269341[] = {{&S84729[96], 3}, {NULL}};
const Segment SGI269342[] = {{&S84729[28], 5}, {NULL}};
const Segment SGI269343[] = {{&S84729[61], 6}, {NULL}};
const Segment SGI269344[] = {{&S84729[107], 6}, {NULL}};
const Segment SGI269345[] = {{&S84793[13], 4}, {NULL}};
const Segment SGI269346[] = {{&S84793[26], 5}, {NULL}};
const Segment SGI269347[] = {{&S84793[125], 3}, {NULL}};
const Segment SGI269348[] = {{&S84793[59], 6}, {NULL}};
const Segment SGI269349[] = {{&S84793[93], 5}, {NULL}};
const Segment SGI269350[] = {{&S84793[106], 9}, {NULL}};
const Segment SGI269351[] = {{&S84805[13], 4}, {NULL}};
const Segment SGI269352[] = {{&S84805[26], 5}, {NULL}};
const Segment SGI269353[] = {{&S84805[59], 6}, {NULL}};
const Segment SGI269354[] = {{&S84805[93], 5}, {NULL}};
const Segment SGI269355[] = {{&S84805[99], 15}, {NULL}};
const Segment SGI269356[] = {{&S84861[28], 4}, {NULL}};
const Segment SGI269357[] = {{&S84861[41], 3}, {NULL}};
const Segment SGI269358[] = {{&S84861[72], 6}, {NULL}};
const Segment SGI269359[] = {{&S84861[106], 5}, {NULL}};
const Segment SGI269360[] = {{&S84861[119], 9}, {NULL}};
const Segment SGI269361[] = {{&S84861[0], 14}, {NULL}};
const Segment SGI269362[] = {{&S84862[28], 4}, {NULL}};
const Segment SGI269363[] = {{&S84862[41], 5}, {NULL}};
const Segment SGI269364[] = {{&S84862[74], 6}, {NULL}};
const Segment SGI269365[] = {{&S84862[108], 5}, {NULL}};
const Segment SGI269366[] = {{&S84862[121], 9}, {NULL}};
const Segment SGI269367[] = {{&S84862[0], 14}, {NULL}};
const Segment SGI269368[] = {{&S84864[27], 6}, {NULL}};
const Segment SGI269369[] = {{&S84864[52], 5}, {NULL}};
const Segment SGI269370[] = {{&S84864[85], 6}, {NULL}};
const Segment SGI269371[] = {{&S84864[119], 5}, {NULL}};
const Segment SGI269372[] = {{&S84864[132], 9}, {NULL}};
const Segment SGI269373[] = {{&S84864[0], 14}, {NULL}};
const Segment SGI269374[] = {{&S84865[27], 6}, {NULL}};
const Segment SGI269375[] = {{&S84865[51], 3}, {NULL}};
const Segment SGI269376[] = {{&S84865[82], 6}, {NULL}};
const Segment SGI269377[] = {{&S84865[116], 5}, {NULL}};
const Segment SGI269378[] = {{&S84865[129], 9}, {NULL}};
const Segment SGI269379[] = {{&S84865[0], 14}, {NULL}};
const Segment SGI269380[] = {{&S84866[27], 6}, {NULL}};
const Segment SGI269381[] = {{&S84866[52], 3}, {NULL}};
const Segment SGI269382[] = {{&S84866[83], 6}, {NULL}};
const Segment SGI269383[] = {{&S84866[117], 5}, {NULL}};
const Segment SGI269384[] = {{&S84866[130], 9}, {NULL}};
const Segment SGI269385[] = {{&S84866[0], 14}, {NULL}};
const Segment SGI269386[] = {{&S84867[27], 6}, {NULL}};
const Segment SGI269387[] = {{&S84867[51], 5}, {NULL}};
const Segment SGI269388[] = {{&S84867[84], 6}, {NULL}};
const Segment SGI269389[] = {{&S84867[118], 5}, {NULL}};
const Segment SGI269390[] = {{&S84867[131], 9}, {NULL}};
const Segment SGI269391[] = {{&S84867[0], 14}, {NULL}};
const Segment SGI269392[] = {{&S84872[13], 6}, {NULL}};
const Segment SGI269393[] = {{&S84872[35], 3}, {NULL}};
const Segment SGI269394[] = {{&S84872[66], 6}, {NULL}};
const Segment SGI269395[] = {{&S84872[124], 5}, {NULL}};
const Segment SGI269396[] = {{&S84872[137], 9}, {NULL}};
const Segment SGI269397[] = {{&S77984[109], 5}, {NULL}};
const Segment SGI269398[] = {{&S77984[0], 114}, {NULL}};
const Segment SGI269399[] = {{&S77985[109], 5}, {NULL}};
const Segment SGI269400[] = {{&S77985[0], 114}, {NULL}};
const Segment SGI269401[] = {{&S85004[13], 4}, {NULL}};
const Segment SGI269402[] = {{&S85004[29], 5}, {NULL}};
const Segment SGI269403[] = {{&S85004[69], 8}, {NULL}};
const Segment SGI269404[] = {{&S85004[120], 3}, {NULL}};
const Segment SGI269405[] = {{&S85004[131], 9}, {NULL}};
const Segment SGI269406[] = {{&S85004[98], 14}, {NULL}};
const Segment SGI269407[] = {{&S85005[13], 4}, {NULL}};
const Segment SGI269408[] = {{&S85005[29], 5}, {NULL}};
const Segment SGI269409[] = {{&S85005[69], 8}, {NULL}};
const Segment SGI269410[] = {{&S85005[120], 3}, {NULL}};
const Segment SGI269411[] = {{&S85005[131], 9}, {NULL}};
const Segment SGI269412[] = {{&S85005[98], 14}, {NULL}};
const Segment SGI269413[] = {{&S77999[59], 5}, {NULL}};
const Segment SGI269414[] = {{&S77999[0], 64}, {NULL}};
const Segment SGI269415[] = {{&S78000[59], 5}, {&S78000[59], 5}, {NULL}};
const Segment SGI269416[] = {{&S78000[0], 70}, {NULL}};
const Segment SGI269417[] = {{&S72225[20], 26}, {NULL}};
const Segment SGI269418[] = {{&S72226[20], 27}, {NULL}};
const Segment SGI269419[] = {{&S72227[20], 27}, {NULL}};
const Segment SGI269420[] = {{&S85015[13], 4}, {NULL}};
const Segment SGI269421[] = {{&S85015[26], 5}, {NULL}};
const Segment SGI269422[] = {{&S85015[125], 5}, {NULL}};
const Segment SGI269423[] = {{&S85015[59], 6}, {NULL}};
const Segment SGI269424[] = {{&S85015[93], 5}, {NULL}};
const Segment SGI269425[] = {{&S85015[106], 9}, {NULL}};
const Segment SGI269426[] = {{&S85016[13], 4}, {NULL}};
const Segment SGI269427[] = {{&S85016[26], 5}, {NULL}};
const Segment SGI269428[] = {{&S85016[125], 5}, {NULL}};
const Segment SGI269429[] = {{&S85016[59], 6}, {NULL}};
const Segment SGI269430[] = {{&S85016[93], 5}, {NULL}};
const Segment SGI269431[] = {{&S85016[106], 9}, {NULL}};
const Segment SGI269432[] = {{&S85017[13], 4}, {NULL}};
const Segment SGI269433[] = {{&S85017[26], 5}, {NULL}};
const Segment SGI269434[] = {{&S85017[125], 5}, {NULL}};
const Segment SGI269435[] = {{&S85017[59], 6}, {NULL}};
const Segment SGI269436[] = {{&S85017[93], 5}, {NULL}};
const Segment SGI269437[] = {{&S85017[106], 9}, {NULL}};
const Segment SGI269438[] = {{&S78134[151], 21}, {&S78134[173], 32}, {NULL}};
const Segment SGI269439[] = {{&S78134[0], 205}, {NULL}};
const Segment SGI269440[] = {{&S85057[13], 4}, {NULL}};
const Segment SGI269441[] = {{&S85057[117], 3}, {NULL}};
const Segment SGI269442[] = {{&S85057[42], 5}, {NULL}};
const Segment SGI269443[] = {{&S85057[82], 6}, {NULL}};
const Segment SGI269444[] = {{&S85057[128], 5}, {NULL}};
const Segment SGI269445[] = {{&S85057[141], 6}, {NULL}};
const Segment SGI269446[] = {{&S85058[13], 4}, {NULL}};
const Segment SGI269447[] = {{&S85058[113], 5}, {NULL}};
const Segment SGI269448[] = {{&S85058[36], 5}, {NULL}};
const Segment SGI269449[] = {{&S85058[76], 8}, {NULL}};
const Segment SGI269450[] = {{&S85058[126], 5}, {NULL}};
const Segment SGI269451[] = {{&S85058[139], 9}, {NULL}};
const Segment SGI269452[] = {{&S78166[61], 6}, {NULL}};
const Segment SGI269453[] = {{&S78166[0], 67}, {NULL}};
const Segment SGI269454[] = {{&S78194[62], 5}, {NULL}};
const Segment SGI269455[] = {{&S78194[0], 67}, {NULL}};
const Segment SGI269456[] = {{&S78195[59], 5}, {NULL}};
const Segment SGI269457[] = {{&S78195[0], 64}, {NULL}};
const Segment SGI269458[] = {{&S78196[59], 5}, {&S78196[59], 5}, {NULL}};
const Segment SGI269459[] = {{&S78196[0], 70}, {NULL}};
const Segment SGI269460[] = {{&S85136[17], 6}, {NULL}};
const Segment SGI269461[] = {{&S85136[35], 5}, {NULL}};
const Segment SGI269462[] = {{&S85137[17], 6}, {NULL}};
const Segment SGI269463[] = {{&S85137[35], 5}, {NULL}};
const Segment SGI269464[] = {{&S85138[17], 6}, {NULL}};
const Segment SGI269465[] = {{&S85138[35], 5}, {NULL}};
const Segment SGI269466[] = {{&S85139[17], 6}, {NULL}};
const Segment SGI269467[] = {{&S85139[35], 3}, {NULL}};
const Segment SGI269468[] = {{&S85140[17], 6}, {NULL}};
const Segment SGI269469[] = {{&S85140[35], 5}, {NULL}};
const Segment SGI269470[] = {{&S78264[59], 5}, {NULL}};
const Segment SGI269471[] = {{&S78264[0], 64}, {NULL}};
const Segment SGI269472[] = {{&S78265[59], 5}, {&S78265[59], 5}, {NULL}};
const Segment SGI269473[] = {{&S78265[0], 70}, {NULL}};
const Segment SGI269474[] = {{&S72228[20], 26}, {NULL}};
const Segment SGI269475[] = {{&S85169[13], 6}, {NULL}};
const Segment SGI269476[] = {{&S85169[113], 3}, {NULL}};
const Segment SGI269477[] = {{&S85169[38], 5}, {NULL}};
const Segment SGI269478[] = {{&S85169[78], 6}, {NULL}};
const Segment SGI269479[] = {{&S85169[124], 5}, {NULL}};
const Segment SGI269480[] = {{&S85169[137], 9}, {NULL}};
const Segment SGI269481[] = {{&S72229[20], 27}, {NULL}};
const Segment SGI269482[] = {{&S72230[20], 26}, {NULL}};
const Segment SGI269483[] = {{&S78382[59], 5}, {NULL}};
const Segment SGI269484[] = {{&S78382[0], 64}, {NULL}};
const Segment SGI269485[] = {{&S78383[66], 5}, {NULL}};
const Segment SGI269486[] = {{&S78383[0], 71}, {NULL}};
const Segment SGI269487[] = {{&S78384[66], 5}, {&S78384[66], 5}, {NULL}};
const Segment SGI269488[] = {{&S78384[0], 77}, {NULL}};
const Segment SGI269489[] = {{&S85292[17], 6}, {NULL}};
const Segment SGI269490[] = {{&S85292[38], 5}, {NULL}};
const Segment SGI269491[] = {{&S85292[68], 3}, {NULL}};
const Segment SGI269492[] = {{&S85293[17], 6}, {NULL}};
const Segment SGI269493[] = {{&S85293[38], 5}, {NULL}};
const Segment SGI269494[] = {{&S85293[68], 3}, {NULL}};
const Segment SGI269495[] = {{&S85329[13], 6}, {NULL}};
const Segment SGI269496[] = {{&S85329[35], 3}, {NULL}};
const Segment SGI269497[] = {{&S85329[135], 5}, {NULL}};
const Segment SGI269498[] = {{&S85329[66], 8}, {NULL}};
const Segment SGI269499[] = {{&S85329[102], 6}, {NULL}};
const Segment SGI269500[] = {{&S85329[116], 9}, {NULL}};
const Segment SGI269501[] = {{&S85330[13], 6}, {NULL}};
const Segment SGI269502[] = {{&S85330[35], 5}, {NULL}};
const Segment SGI269503[] = {{&S85330[137], 5}, {NULL}};
const Segment SGI269504[] = {{&S85330[68], 8}, {NULL}};
const Segment SGI269505[] = {{&S85330[104], 6}, {NULL}};
const Segment SGI269506[] = {{&S85330[118], 9}, {NULL}};
const Segment SGI269507[] = {{&S78466[59], 5}, {NULL}};
const Segment SGI269508[] = {{&S78466[0], 64}, {NULL}};
const Segment SGI269509[] = {{&S78467[59], 5}, {&S78467[59], 5}, {NULL}};
const Segment SGI269510[] = {{&S78467[0], 70}, {NULL}};
const Segment SGI269511[] = {{&S85583[17], 6}, {NULL}};
const Segment SGI269512[] = {{&S85583[38], 3}, {NULL}};
const Segment SGI269513[] = {{&S85583[62], 3}, {NULL}};
const Segment SGI269514[] = {{&S85584[17], 6}, {NULL}};
const Segment SGI269515[] = {{&S85584[38], 5}, {NULL}};
const Segment SGI269516[] = {{&S85584[64], 3}, {NULL}};
const Segment SGI269517[] = {{&S85585[17], 6}, {NULL}};
const Segment SGI269518[] = {{&S85585[38], 5}, {NULL}};
const Segment SGI269519[] = {{&S85585[64], 3}, {NULL}};
const Segment SGI269520[] = {{&S78864[0], 6}, {NULL}};
const Segment SGI269521[] = {{&S78864[0], 56}, {NULL}};
const Segment SGI269522[] = {{&S78865[0], 6}, {NULL}};
const Segment SGI269523[] = {{&S78865[0], 57}, {NULL}};
const Segment SGI269524[] = {{&S85653[28], 4}, {NULL}};
const Segment SGI269525[] = {{&S85653[44], 3}, {NULL}};
const Segment SGI269526[] = {{&S85688[17], 4}, {NULL}};
const Segment SGI269527[] = {{&S85688[36], 5}, {NULL}};
const Segment SGI269528[] = {{&S85688[62], 3}, {NULL}};
const Segment SGI269529[] = {{&S78952[59], 5}, {NULL}};
const Segment SGI269530[] = {{&S78952[0], 64}, {NULL}};
const Segment SGI269531[] = {{&S78953[59], 5}, {&S78953[59], 5}, {NULL}};
const Segment SGI269532[] = {{&S78953[0], 70}, {NULL}};
const Segment SGI269533[] = {{&S72815[10], 5}, {NULL}};
const Segment SGI269534[] = {{&S72815[16], 5}, {NULL}};
const Segment SGI269535[] = {{&S79087[59], 5}, {NULL}};
const Segment SGI269536[] = {{&S79087[0], 64}, {NULL}};
const Segment SGI269537[] = {{&S79088[59], 5}, {&S79088[59], 5}, {NULL}};
const Segment SGI269538[] = {{&S79088[0], 70}, {NULL}};
const Segment SGI269539[] = {{&S85904[13], 4}, {NULL}};
const Segment SGI269540[] = {{&S85904[36], 3}, {NULL}};
const Segment SGI269541[] = {{&S85904[74], 8}, {NULL}};
const Segment SGI269542[] = {{&S85904[110], 5}, {NULL}};
const Segment SGI269543[] = {{&S85905[13], 4}, {NULL}};
const Segment SGI269544[] = {{&S85905[36], 5}, {NULL}};
const Segment SGI269545[] = {{&S85905[76], 8}, {NULL}};
const Segment SGI269546[] = {{&S85905[112], 5}, {NULL}};
const Segment SGI269547[] = {{&S85906[13], 4}, {NULL}};
const Segment SGI269548[] = {{&S85906[36], 3}, {NULL}};
const Segment SGI269549[] = {{&S85906[74], 8}, {NULL}};
const Segment SGI269550[] = {{&S85907[13], 4}, {NULL}};
const Segment SGI269551[] = {{&S85907[36], 5}, {NULL}};
const Segment SGI269552[] = {{&S85907[76], 8}, {NULL}};
const Segment SGI269553[] = {{&S85973[13], 4}, {NULL}};
const Segment SGI269554[] = {{&S85973[26], 5}, {NULL}};
const Segment SGI269555[] = {{&S85973[59], 6}, {NULL}};
const Segment SGI269556[] = {{&S85973[108], 5}, {NULL}};
const Segment SGI269557[] = {{&S85973[121], 9}, {NULL}};
const Segment SGI269558[] = {{&S85973[86], 14}, {NULL}};
const Segment SGI269559[] = {{&S86027[13], 4}, {NULL}};
const Segment SGI269560[] = {{&S86027[111], 5}, {NULL}};
const Segment SGI269561[] = {{&S86027[36], 3}, {NULL}};
const Segment SGI269562[] = {{&S86027[74], 8}, {NULL}};
const Segment SGI269563[] = {{&S86027[124], 5}, {NULL}};
const Segment SGI269564[] = {{&S86027[137], 6}, {NULL}};
const Segment SGI269565[] = {{&S86028[13], 4}, {NULL}};
const Segment SGI269566[] = {{&S86028[111], 5}, {NULL}};
const Segment SGI269567[] = {{&S86028[36], 3}, {NULL}};
const Segment SGI269568[] = {{&S86028[74], 8}, {NULL}};
const Segment SGI269569[] = {{&S86028[124], 5}, {NULL}};
const Segment SGI269570[] = {{&S86028[137], 6}, {NULL}};
const Segment SGI269571[] = {{&S79202[109], 5}, {NULL}};
const Segment SGI269572[] = {{&S79202[0], 114}, {NULL}};
const Segment SGI269573[] = {{&S79203[109], 5}, {NULL}};
const Segment SGI269574[] = {{&S79203[0], 114}, {NULL}};
const Segment SGI269575[] = {{&S79204[109], 5}, {NULL}};
const Segment SGI269576[] = {{&S79204[0], 114}, {NULL}};
const Segment SGI269577[] = {{&S86097[13], 6}, {NULL}};
const Segment SGI269578[] = {{&S86097[35], 5}, {NULL}};
const Segment SGI269579[] = {{&S86097[75], 6}, {NULL}};
const Segment SGI269580[] = {{&S86097[109], 6}, {NULL}};
const Segment SGI269581[] = {{&S86097[116], 13}, {NULL}};
const Segment SGI269582[] = {{&S86098[13], 6}, {NULL}};
const Segment SGI269583[] = {{&S86098[35], 5}, {NULL}};
const Segment SGI269584[] = {{&S86098[75], 6}, {NULL}};
const Segment SGI269585[] = {{&S86098[109], 6}, {NULL}};
const Segment SGI269586[] = {{&S86098[116], 13}, {NULL}};
const Segment SGI269587[] = {{&S86099[13], 6}, {NULL}};
const Segment SGI269588[] = {{&S86099[35], 5}, {NULL}};
const Segment SGI269589[] = {{&S86099[75], 6}, {NULL}};
const Segment SGI269590[] = {{&S86099[109], 6}, {NULL}};
const Segment SGI269591[] = {{&S86099[116], 13}, {NULL}};
const Segment SGI269592[] = {{&S86148[17], 4}, {NULL}};
const Segment SGI269593[] = {{&S86148[36], 5}, {NULL}};
const Segment SGI269594[] = {{&S86148[66], 3}, {NULL}};
const Segment SGI269595[] = {{&S73295[20], 26}, {NULL}};
const Segment SGI269596[] = {{&S73296[20], 27}, {NULL}};
const Segment SGI269597[] = {{&S73297[20], 27}, {NULL}};
const Segment SGI269598[] = {{&S73298[20], 26}, {NULL}};
const Segment SGI269599[] = {{&S73299[20], 26}, {NULL}};
const Segment SGI269600[] = {{&S73300[20], 27}, {NULL}};
const Segment SGI269601[] = {{&S73301[20], 27}, {NULL}};
const Segment SGI269602[] = {{&S73302[20], 27}, {NULL}};
const Segment SGI269603[] = {{&S73303[20], 27}, {NULL}};
const Segment SGI269604[] = {{&S73304[20], 27}, {NULL}};
const Segment SGI269605[] = {{&S73305[20], 26}, {NULL}};
const Segment SGI269606[] = {{&S73306[20], 27}, {NULL}};
const Segment SGI269607[] = {{&S73307[20], 26}, {NULL}};
const Segment SGI269608[] = {{&S86204[13], 6}, {NULL}};
const Segment SGI269609[] = {{&S86204[113], 5}, {NULL}};
const Segment SGI269610[] = {{&S86204[38], 3}, {NULL}};
const Segment SGI269611[] = {{&S86204[76], 8}, {NULL}};
const Segment SGI269612[] = {{&S86204[126], 5}, {&S86204[80], 1}, {NULL}};
const Segment SGI269613[] = {{&S86204[159], 9}, {NULL}};
const Segment SGI269614[] = {{&S86205[13], 6}, {NULL}};
const Segment SGI269615[] = {{&S86205[115], 5}, {NULL}};
const Segment SGI269616[] = {{&S86205[38], 5}, {NULL}};
const Segment SGI269617[] = {{&S86205[78], 8}, {NULL}};
const Segment SGI269618[] = {{&S86205[128], 5}, {&S86205[82], 1}, {NULL}};
const Segment SGI269619[] = {{&S86205[161], 9}, {NULL}};
const Segment SGI269620[] = {{&S86340[13], 4}, {NULL}};
const Segment SGI269621[] = {{&S86340[126], 3}, {NULL}};
const Segment SGI269622[] = {{&S86340[29], 5}, {NULL}};
const Segment SGI269623[] = {{&S86340[69], 8}, {NULL}};
const Segment SGI269624[] = {{&S86340[105], 5}, {NULL}};
const Segment SGI269625[] = {{&S86341[13], 4}, {&S86341[13], 4}, {NULL}};
const Segment SGI269626[] = {{&S86341[116], 3}, {NULL}};
const Segment SGI269627[] = {{&S86341[29], 3}, {NULL}};
const Segment SGI269628[] = {{&S86341[81], 6}, {NULL}};
const Segment SGI269629[] = {{&S86341[127], 5}, {NULL}};
const Segment SGI269630[] = {{&S86341[140], 9}, {NULL}};
const Segment SGI269631[] = {{&S86386[13], 6}, {NULL}};
const Segment SGI269632[] = {{&S86386[110], 5}, {NULL}};
const Segment SGI269633[] = {{&S86386[38], 3}, {NULL}};
const Segment SGI269634[] = {{&S86386[76], 5}, {NULL}};
const Segment SGI269635[] = {{&S86386[123], 5}, {NULL}};
const Segment SGI269636[] = {{&S86386[136], 9}, {NULL}};
const Segment SGI269637[] = {{&S86386[159], 13}, {NULL}};
const Segment SGI269638[] = {{&S86427[12], 6}, {NULL}};
const Segment SGI269639[] = {{&S86427[80], 5}, {NULL}};
const Segment SGI269640[] = {{&S86625[13], 6}, {NULL}};
const Segment SGI269641[] = {{&S86625[35], 5}, {NULL}};
const Segment SGI269642[] = {{&S86625[134], 5}, {NULL}};
const Segment SGI269643[] = {{&S86625[68], 6}, {NULL}};
const Segment SGI269644[] = {{&S86625[102], 5}, {NULL}};
const Segment SGI269645[] = {{&S86625[115], 9}, {NULL}};
const Segment SGI269646[] = {{&S86626[13], 6}, {NULL}};
const Segment SGI269647[] = {{&S86626[35], 5}, {NULL}};
const Segment SGI269648[] = {{&S86626[134], 5}, {NULL}};
const Segment SGI269649[] = {{&S86626[68], 6}, {NULL}};
const Segment SGI269650[] = {{&S86626[102], 5}, {NULL}};
const Segment SGI269651[] = {{&S86626[115], 9}, {NULL}};
const Segment SGI269652[] = {{&S86627[13], 6}, {NULL}};
const Segment SGI269653[] = {{&S86627[35], 5}, {NULL}};
const Segment SGI269654[] = {{&S86627[134], 5}, {NULL}};
const Segment SGI269655[] = {{&S86627[68], 6}, {NULL}};
const Segment SGI269656[] = {{&S86627[102], 5}, {NULL}};
const Segment SGI269657[] = {{&S86627[115], 9}, {NULL}};
const Segment SGI269658[] = {{&S86676[27], 6}, {NULL}};
const Segment SGI269659[] = {{&S86676[49], 5}, {NULL}};
const Segment SGI269660[] = {{&S86676[82], 6}, {NULL}};
const Segment SGI269661[] = {{&S86676[116], 5}, {NULL}};
const Segment SGI269662[] = {{&S86676[129], 9}, {NULL}};
const Segment SGI269663[] = {{&S86676[0], 14}, {NULL}};
const Segment SGI269664[] = {{&S86677[27], 6}, {NULL}};
const Segment SGI269665[] = {{&S86677[49], 5}, {NULL}};
const Segment SGI269666[] = {{&S86677[82], 6}, {NULL}};
const Segment SGI269667[] = {{&S86677[116], 5}, {NULL}};
const Segment SGI269668[] = {{&S86677[129], 9}, {NULL}};
const Segment SGI269669[] = {{&S86677[0], 14}, {NULL}};
const Segment SGI269670[] = {{&S86684[28], 4}, {NULL}};
const Segment SGI269671[] = {{&S86684[41], 5}, {NULL}};
const Segment SGI269672[] = {{&S86684[74], 6}, {NULL}};
const Segment SGI269673[] = {{&S86684[108], 5}, {NULL}};
const Segment SGI269674[] = {{&S86684[121], 9}, {NULL}};
const Segment SGI269675[] = {{&S86684[0], 14}, {NULL}};
const Segment SGI269676[] = {{&S86686[17], 6}, {NULL}};
const Segment SGI269677[] = {{&S86686[35], 3}, {NULL}};
const Segment SGI269678[] = {{&S86686[59], 3}, {NULL}};
const Segment SGI269679[] = {{&S86799[13], 6}, {NULL}};
const Segment SGI269680[] = {{&S86799[35], 5}, {NULL}};
const Segment SGI269681[] = {{&S86799[68], 6}, {NULL}};
const Segment SGI269682[] = {{&S86799[124], 5}, {NULL}};
const Segment SGI269683[] = {{&S86799[137], 9}, {NULL}};
const Segment SGI269684[] = {{&S86872[13], 6}, {NULL}};
const Segment SGI269685[] = {{&S86872[96], 3}, {NULL}};
const Segment SGI269686[] = {{&S86872[28], 5}, {NULL}};
const Segment SGI269687[] = {{&S86872[61], 6}, {NULL}};
const Segment SGI269688[] = {{&S86872[107], 6}, {NULL}};
const Segment SGI269689[] = {{&S79841[0], 14}, {NULL}};
const Segment SGI269690[] = {{&S79841[0], 42}, {NULL}};
const Segment SGI269691[] = {{&S79842[0], 14}, {NULL}};
const Segment SGI269692[] = {{&S79842[0], 42}, {NULL}};
const Segment SGI269693[] = {{&S79843[0], 14}, {NULL}};
const Segment SGI269694[] = {{&S79843[0], 42}, {NULL}};
const Segment SGI269695[] = {{&S79844[0], 14}, {NULL}};
const Segment SGI269696[] = {{&S79844[0], 46}, {NULL}};
const Segment SGI269697[] = {{&S79845[0], 14}, {NULL}};
const Segment SGI269698[] = {{&S79845[0], 46}, {NULL}};
const Segment SGI269699[] = {{&S79846[0], 14}, {NULL}};
const Segment SGI269700[] = {{&S79846[0], 46}, {NULL}};
const Segment SGI269701[] = {{&S86986[13], 6}, {NULL}};
const Segment SGI269702[] = {{&S86986[113], 3}, {NULL}};
const Segment SGI269703[] = {{&S86986[38], 5}, {NULL}};
const Segment SGI269704[] = {{&S86986[78], 6}, {NULL}};
const Segment SGI269705[] = {{&S86986[124], 6}, {NULL}};
const Segment SGI269706[] = {{&S86986[138], 9}, {NULL}};
const Segment SGI269707[] = {{&S86987[13], 6}, {NULL}};
const Segment SGI269708[] = {{&S86987[113], 3}, {NULL}};
const Segment SGI269709[] = {{&S86987[38], 5}, {NULL}};
const Segment SGI269710[] = {{&S86987[78], 6}, {NULL}};
const Segment SGI269711[] = {{&S86987[124], 6}, {NULL}};
const Segment SGI269712[] = {{&S86987[138], 9}, {NULL}};
const Segment SGI269713[] = {{&S86994[29], 6}, {NULL}};
const Segment SGI269714[] = {{&S80071[61], 12}, {NULL}};
const Segment SGI269715[] = {{&S80071[0], 88}, {NULL}};
const Segment SGI269716[] = {{&S87070[13], 4}, {NULL}};
const Segment SGI269717[] = {{&S87070[117], 3}, {NULL}};
const Segment SGI269718[] = {{&S87070[42], 5}, {NULL}};
const Segment SGI269719[] = {{&S87070[82], 6}, {NULL}};
const Segment SGI269720[] = {{&S87070[128], 5}, {NULL}};
const Segment SGI269721[] = {{&S87070[141], 6}, {NULL}};
const Segment SGI269722[] = {{&S87071[13], 4}, {NULL}};
const Segment SGI269723[] = {{&S87071[115], 3}, {NULL}};
const Segment SGI269724[] = {{&S87071[42], 3}, {NULL}};
const Segment SGI269725[] = {{&S87071[80], 6}, {NULL}};
const Segment SGI269726[] = {{&S87071[126], 5}, {NULL}};
const Segment SGI269727[] = {{&S87071[139], 6}, {NULL}};
const Segment SGI269728[] = {{&S80281[59], 5}, {NULL}};
const Segment SGI269729[] = {{&S80281[0], 64}, {NULL}};
const Segment SGI269730[] = {{&S80282[61], 5}, {NULL}};
const Segment SGI269731[] = {{&S80282[0], 66}, {NULL}};
const Segment SGI269732[] = {{&S80283[61], 5}, {&S80283[61], 5}, {NULL}};
const Segment SGI269733[] = {{&S80283[0], 72}, {NULL}};
const Segment SGI269734[] = {{&S87278[13], 4}, {NULL}};
const Segment SGI269735[] = {{&S87278[29], 5}, {NULL}};
const Segment SGI269736[] = {{&S87278[69], 8}, {NULL}};
const Segment SGI269737[] = {{&S87278[120], 3}, {NULL}};
const Segment SGI269738[] = {{&S87278[131], 9}, {NULL}};
const Segment SGI269739[] = {{&S87278[98], 14}, {NULL}};
const Segment SGI269740[] = {{&S87279[13], 4}, {NULL}};
const Segment SGI269741[] = {{&S87279[29], 5}, {NULL}};
const Segment SGI269742[] = {{&S87279[69], 8}, {NULL}};
const Segment SGI269743[] = {{&S87279[120], 3}, {NULL}};
const Segment SGI269744[] = {{&S87279[131], 9}, {NULL}};
const Segment SGI269745[] = {{&S87279[98], 14}, {NULL}};
const Segment SGI269746[] = {{&S87301[28], 4}, {NULL}};
const Segment SGI269747[] = {{&S87301[41], 3}, {NULL}};
const Segment SGI269748[] = {{&S87301[72], 6}, {NULL}};
const Segment SGI269749[] = {{&S87301[106], 5}, {NULL}};
const Segment SGI269750[] = {{&S87301[119], 9}, {NULL}};
const Segment SGI269751[] = {{&S87301[0], 14}, {NULL}};
const Segment SGI269752[] = {{&S87302[28], 4}, {NULL}};
const Segment SGI269753[] = {{&S87302[41], 5}, {NULL}};
const Segment SGI269754[] = {{&S87302[74], 6}, {NULL}};
const Segment SGI269755[] = {{&S87302[108], 5}, {NULL}};
const Segment SGI269756[] = {{&S87302[121], 9}, {NULL}};
const Segment SGI269757[] = {{&S87302[0], 14}, {NULL}};
const Segment SGI269758[] = {{&S87341[17], 4}, {NULL}};
const Segment SGI269759[] = {{&S87341[36], 5}, {NULL}};
const Segment SGI269760[] = {{&S87341[66], 3}, {NULL}};
const Segment SGI269761[] = {{&S87357[17], 6}, {NULL}};
const Segment SGI269762[] = {{&S87357[38], 5}, {NULL}};
const Segment SGI269763[] = {{&S87357[68], 3}, {NULL}};
const Segment SGI269764[] = {{&S87441[27], 6}, {NULL}};
const Segment SGI269765[] = {{&S87441[51], 5}, {NULL}};
const Segment SGI269766[] = {{&S87441[84], 6}, {NULL}};
const Segment SGI269767[] = {{&S87441[118], 5}, {NULL}};
const Segment SGI269768[] = {{&S87441[131], 9}, {NULL}};
const Segment SGI269769[] = {{&S87441[0], 14}, {NULL}};
const Segment SGI269770[] = {{&S87442[27], 6}, {NULL}};
const Segment SGI269771[] = {{&S87442[52], 5}, {NULL}};
const Segment SGI269772[] = {{&S87442[85], 6}, {NULL}};
const Segment SGI269773[] = {{&S87442[119], 5}, {NULL}};
const Segment SGI269774[] = {{&S87442[132], 9}, {NULL}};
const Segment SGI269775[] = {{&S87442[0], 14}, {NULL}};
const Segment SGI269776[] = {{&S87443[28], 6}, {NULL}};
const Segment SGI269777[] = {{&S87443[52], 5}, {NULL}};
const Segment SGI269778[] = {{&S87443[85], 6}, {NULL}};
const Segment SGI269779[] = {{&S87443[119], 5}, {NULL}};
const Segment SGI269780[] = {{&S87443[132], 9}, {NULL}};
const Segment SGI269781[] = {{&S87443[0], 15}, {NULL}};
const Segment SGI269782[] = {{&S87465[13], 4}, {NULL}};
const Segment SGI269783[] = {{&S87465[26], 5}, {NULL}};
const Segment SGI269784[] = {{&S87465[59], 6}, {NULL}};
const Segment SGI269785[] = {{&S87465[93], 6}, {NULL}};
const Segment SGI269786[] = {{&S87465[100], 15}, {NULL}};
const Segment SGI269787[] = {{&S87583[13], 6}, {NULL}};
const Segment SGI269788[] = {{&S87583[35], 5}, {NULL}};
const Segment SGI269789[] = {{&S87583[134], 3}, {NULL}};
const Segment SGI269790[] = {{&S87583[68], 6}, {NULL}};
const Segment SGI269791[] = {{&S87583[102], 5}, {NULL}};
const Segment SGI269792[] = {{&S87583[115], 9}, {NULL}};
const Segment SGI269793[] = {{&S74773[20], 27}, {NULL}};
const Segment SGI269794[] = {{&S74774[20], 27}, {NULL}};
const Segment SGI269795[] = {{&S74775[20], 27}, {NULL}};
const Segment SGI269796[] = {{&S74776[20], 27}, {NULL}};
const Segment SGI269797[] = {{&S74777[20], 27}, {NULL}};
const Segment SGI269798[] = {{&S74778[20], 27}, {NULL}};
const Segment SGI269799[] = {{&S87695[17], 4}, {NULL}};
const Segment SGI269800[] = {{&S87695[33], 5}, {NULL}};
const Segment SGI269801[] = {{&S80828[63], 12}, {NULL}};
const Segment SGI269802[] = {{&S80828[0], 90}, {NULL}};
const Segment SGI269803[] = {{&S87853[13], 4}, {NULL}};
const Segment SGI269804[] = {{&S87853[31], 3}, {NULL}};
const Segment SGI269805[] = {{&S87853[69], 6}, {NULL}};
const Segment SGI269806[] = {{&S80879[57], 5}, {NULL}};
const Segment SGI269807[] = {{&S80879[0], 62}, {NULL}};
const Segment SGI269808[] = {{&S80880[64], 5}, {NULL}};
const Segment SGI269809[] = {{&S80880[0], 69}, {NULL}};
const Segment SGI269810[] = {{&S80881[68], 5}, {NULL}};
const Segment SGI269811[] = {{&S80881[0], 73}, {NULL}};
const Segment SGI269812[] = {{&S80882[64], 5}, {&S80882[64], 5}, {NULL}};
const Segment SGI269813[] = {{&S80882[0], 75}, {NULL}};
const Segment SGI269814[] = {{&S80883[61], 5}, {NULL}};
const Segment SGI269815[] = {{&S80883[0], 66}, {NULL}};
const Segment SGI269816[] = {{&S88060[27], 6}, {NULL}};
const Segment SGI269817[] = {{&S88060[49], 3}, {NULL}};
const Segment SGI269818[] = {{&S88060[80], 6}, {NULL}};
const Segment SGI269819[] = {{&S88060[114], 5}, {NULL}};
const Segment SGI269820[] = {{&S88060[127], 9}, {NULL}};
const Segment SGI269821[] = {{&S88060[0], 14}, {NULL}};
const Segment SGI269822[] = {{&S88061[27], 6}, {NULL}};
const Segment SGI269823[] = {{&S88061[49], 3}, {NULL}};
const Segment SGI269824[] = {{&S88061[80], 6}, {NULL}};
const Segment SGI269825[] = {{&S88061[114], 5}, {NULL}};
const Segment SGI269826[] = {{&S88061[127], 9}, {NULL}};
const Segment SGI269827[] = {{&S88061[0], 14}, {NULL}};
const Segment SGI269828[] = {{&S88062[27], 6}, {NULL}};
const Segment SGI269829[] = {{&S88062[49], 5}, {NULL}};
const Segment SGI269830[] = {{&S88062[82], 6}, {NULL}};
const Segment SGI269831[] = {{&S88062[116], 5}, {NULL}};
const Segment SGI269832[] = {{&S88062[129], 9}, {NULL}};
const Segment SGI269833[] = {{&S88062[0], 14}, {NULL}};
const Segment SGI269834[] = {{&S88091[13], 4}, {NULL}};
const Segment SGI269835[] = {{&S88091[29], 5}, {NULL}};
const Segment SGI269836[] = {{&S88091[69], 8}, {NULL}};
const Segment SGI269837[] = {{&S88091[120], 3}, {NULL}};
const Segment SGI269838[] = {{&S88091[131], 9}, {NULL}};
const Segment SGI269839[] = {{&S88091[98], 14}, {NULL}};
const Segment SGI269840[] = {{&S88092[13], 4}, {NULL}};
const Segment SGI269841[] = {{&S88092[29], 5}, {NULL}};
const Segment SGI269842[] = {{&S88092[69], 8}, {NULL}};
const Segment SGI269843[] = {{&S88092[120], 3}, {NULL}};
const Segment SGI269844[] = {{&S88092[131], 9}, {NULL}};
const Segment SGI269845[] = {{&S88092[98], 14}, {NULL}};
const Segment SGI269846[] = {{&S75284[10], 4}, {NULL}};
const Segment SGI269847[] = {{&S75284[15], 5}, {NULL}};
const Segment SGI269848[] = {{&S75284[64], 13}, {NULL}};
const Segment SGI269849[] = {{&S75285[10], 4}, {NULL}};
const Segment SGI269850[] = {{&S75285[15], 5}, {NULL}};
const Segment SGI269851[] = {{&S75285[65], 13}, {NULL}};
const Segment SGI269852[] = {{&S88148[13], 4}, {NULL}};
const Segment SGI269853[] = {{&S88148[94], 3}, {NULL}};
const Segment SGI269854[] = {{&S88148[26], 3}, {NULL}};
const Segment SGI269855[] = {{&S88148[57], 8}, {NULL}};
const Segment SGI269856[] = {{&S88148[105], 5}, {NULL}};
const Segment SGI269857[] = {{&S88148[118], 7}, {NULL}};
const Segment SGI269858[] = {{&S88149[13], 4}, {NULL}};
const Segment SGI269859[] = {{&S88149[94], 3}, {NULL}};
const Segment SGI269860[] = {{&S88149[26], 3}, {NULL}};
const Segment SGI269861[] = {{&S88149[57], 8}, {NULL}};
const Segment SGI269862[] = {{&S88149[105], 5}, {NULL}};
const Segment SGI269863[] = {{&S88149[118], 7}, {NULL}};
const Segment SGI269864[] = {{&S88150[13], 4}, {NULL}};
const Segment SGI269865[] = {{&S88150[94], 3}, {NULL}};
const Segment SGI269866[] = {{&S88150[26], 3}, {NULL}};
const Segment SGI269867[] = {{&S88150[57], 8}, {NULL}};
const Segment SGI269868[] = {{&S88150[105], 5}, {NULL}};
const Segment SGI269869[] = {{&S88150[118], 7}, {NULL}};
const Segment SGI269870[] = {{&S88151[13], 4}, {NULL}};
const Segment SGI269871[] = {{&S88151[94], 3}, {NULL}};
const Segment SGI269872[] = {{&S88151[26], 3}, {NULL}};
const Segment SGI269873[] = {{&S88151[57], 8}, {NULL}};
const Segment SGI269874[] = {{&S88151[105], 5}, {NULL}};
const Segment SGI269875[] = {{&S88151[118], 7}, {NULL}};
const Segment SGI269876[] = {{&S88152[13], 4}, {NULL}};
const Segment SGI269877[] = {{&S88152[94], 3}, {NULL}};
const Segment SGI269878[] = {{&S88152[26], 3}, {NULL}};
const Segment SGI269879[] = {{&S88152[57], 8}, {NULL}};
const Segment SGI269880[] = {{&S88152[105], 5}, {NULL}};
const Segment SGI269881[] = {{&S88152[118], 7}, {NULL}};
const Segment SGI269882[] = {{&S88153[13], 4}, {NULL}};
const Segment SGI269883[] = {{&S88153[94], 3}, {NULL}};
const Segment SGI269884[] = {{&S88153[26], 3}, {NULL}};
const Segment SGI269885[] = {{&S88153[57], 8}, {NULL}};
const Segment SGI269886[] = {{&S88153[105], 5}, {NULL}};
const Segment SGI269887[] = {{&S88153[118], 7}, {NULL}};
const Segment SGI269888[] = {{&S88436[13], 6}, {NULL}};
const Segment SGI269889[] = {{&S88436[113], 3}, {NULL}};
const Segment SGI269890[] = {{&S88436[38], 5}, {NULL}};
const Segment SGI269891[] = {{&S88436[78], 6}, {NULL}};
const Segment SGI269892[] = {{&S88436[124], 5}, {NULL}};
const Segment SGI269893[] = {{&S88436[137], 9}, {NULL}};
const Segment SGI269894[] = {{&S88437[13], 6}, {NULL}};
const Segment SGI269895[] = {{&S88437[113], 3}, {NULL}};
const Segment SGI269896[] = {{&S88437[38], 5}, {NULL}};
const Segment SGI269897[] = {{&S88437[78], 6}, {NULL}};
const Segment SGI269898[] = {{&S88437[124], 5}, {NULL}};
const Segment SGI269899[] = {{&S88437[137], 9}, {NULL}};
const Segment SGI269900[] = {{&S88445[13], 4}, {NULL}};
const Segment SGI269901[] = {{&S88445[26], 3}, {NULL}};
const Segment SGI269902[] = {{&S88445[57], 6}, {NULL}};
const Segment SGI269903[] = {{&S88445[110], 6}, {NULL}};
const Segment SGI269904[] = {{&S88445[124], 6}, {NULL}};
const Segment SGI269905[] = {{&S81460[66], 5}, {NULL}};
const Segment SGI269906[] = {{&S81460[0], 71}, {NULL}};
const Segment SGI269907[] = {{&S81461[68], 5}, {&S81461[68], 5}, {NULL}};
const Segment SGI269908[] = {{&S81461[0], 79}, {NULL}};
const Segment SGI269909[] = {{&S81462[68], 5}, {NULL}};
const Segment SGI269910[] = {{&S81462[0], 73}, {NULL}};
const Segment SGI269911[] = {{&S81463[61], 5}, {NULL}};
const Segment SGI269912[] = {{&S81463[0], 66}, {NULL}};
const Segment SGI269913[] = {{&S81464[59], 5}, {NULL}};
const Segment SGI269914[] = {{&S81464[0], 64}, {NULL}};
const Segment SGI269915[] = {{&S88547[27], 6}, {NULL}};
const Segment SGI269916[] = {{&S88547[51], 5}, {NULL}};
const Segment SGI269917[] = {{&S88547[84], 6}, {NULL}};
const Segment SGI269918[] = {{&S88547[118], 5}, {NULL}};
const Segment SGI269919[] = {{&S88547[131], 9}, {NULL}};
const Segment SGI269920[] = {{&S88547[0], 14}, {NULL}};
const Segment SGI269921[] = {{&S88548[27], 6}, {NULL}};
const Segment SGI269922[] = {{&S88548[52], 5}, {NULL}};
const Segment SGI269923[] = {{&S88548[85], 6}, {NULL}};
const Segment SGI269924[] = {{&S88548[119], 5}, {NULL}};
const Segment SGI269925[] = {{&S88548[132], 9}, {NULL}};
const Segment SGI269926[] = {{&S88548[0], 14}, {NULL}};
const Segment SGI269927[] = {{&S88557[13], 4}, {NULL}};
const Segment SGI269928[] = {{&S88557[26], 5}, {NULL}};
const Segment SGI269929[] = {{&S88557[59], 6}, {NULL}};
const Segment SGI269930[] = {{&S88557[108], 5}, {NULL}};
const Segment SGI269931[] = {{&S88557[121], 9}, {NULL}};
const Segment SGI269932[] = {{&S88557[86], 14}, {NULL}};
const Segment SGI269933[] = {{&S88568[13], 6}, {NULL}};
const Segment SGI269934[] = {{&S88568[35], 5}, {NULL}};
const Segment SGI269935[] = {{&S88568[75], 6}, {NULL}};
const Segment SGI269936[] = {{&S88568[109], 6}, {NULL}};
const Segment SGI269937[] = {{&S88568[116], 13}, {NULL}};
const Segment SGI269938[] = {{&S88569[13], 6}, {NULL}};
const Segment SGI269939[] = {{&S88569[35], 5}, {NULL}};
const Segment SGI269940[] = {{&S88569[75], 6}, {NULL}};
const Segment SGI269941[] = {{&S88569[109], 6}, {NULL}};
const Segment SGI269942[] = {{&S88569[116], 13}, {NULL}};
const Segment SGI269943[] = {{&S88570[13], 6}, {NULL}};
const Segment SGI269944[] = {{&S88570[35], 5}, {NULL}};
const Segment SGI269945[] = {{&S88570[75], 6}, {NULL}};
const Segment SGI269946[] = {{&S88570[109], 6}, {NULL}};
const Segment SGI269947[] = {{&S88570[116], 13}, {NULL}};
const Segment SGI269948[] = {{&S88595[28], 4}, {NULL}};
const Segment SGI269949[] = {{&S88595[41], 5}, {NULL}};
const Segment SGI269950[] = {{&S88595[74], 6}, {NULL}};
const Segment SGI269951[] = {{&S88595[108], 5}, {NULL}};
const Segment SGI269952[] = {{&S88595[121], 9}, {NULL}};
const Segment SGI269953[] = {{&S88595[0], 14}, {NULL}};
const Segment SGI269954[] = {{&S88623[13], 4}, {NULL}};
const Segment SGI269955[] = {{&S88623[26], 5}, {NULL}};
const Segment SGI269956[] = {{&S88623[59], 6}, {NULL}};
const Segment SGI269957[] = {{&S88623[112], 6}, {NULL}};
const Segment SGI269958[] = {{&S88623[126], 7}, {NULL}};
const Segment SGI269959[] = {{&S88664[28], 4}, {NULL}};
const Segment SGI269960[] = {{&S88664[41], 5}, {NULL}};
const Segment SGI269961[] = {{&S88664[74], 6}, {NULL}};
const Segment SGI269962[] = {{&S88664[108], 6}, {NULL}};
const Segment SGI269963[] = {{&S88664[122], 9}, {NULL}};
const Segment SGI269964[] = {{&S88664[0], 14}, {NULL}};
const Segment SGI269965[] = {{&S88665[28], 4}, {NULL}};
const Segment SGI269966[] = {{&S88665[41], 5}, {NULL}};
const Segment SGI269967[] = {{&S88665[74], 6}, {NULL}};
const Segment SGI269968[] = {{&S88665[108], 6}, {NULL}};
const Segment SGI269969[] = {{&S88665[122], 9}, {NULL}};
const Segment SGI269970[] = {{&S88665[0], 14}, {NULL}};
const Segment SGI269971[] = {{&S88666[28], 4}, {NULL}};
const Segment SGI269972[] = {{&S88666[41], 3}, {NULL}};
const Segment SGI269973[] = {{&S88666[72], 6}, {NULL}};
const Segment SGI269974[] = {{&S88666[106], 6}, {NULL}};
const Segment SGI269975[] = {{&S88666[120], 9}, {NULL}};
const Segment SGI269976[] = {{&S88666[0], 14}, {NULL}};
const Segment SGI269977[] = {{&S88680[13], 4}, {NULL}};
const Segment SGI269978[] = {{&S88680[26], 5}, {NULL}};
const Segment SGI269979[] = {{&S88680[59], 6}, {NULL}};
const Segment SGI269980[] = {{&S88680[111], 5}, {NULL}};
const Segment SGI269981[] = {{&S88680[124], 9}, {NULL}};
const Segment SGI269982[] = {{&S88681[13], 4}, {NULL}};
const Segment SGI269983[] = {{&S88681[26], 3}, {NULL}};
const Segment SGI269984[] = {{&S88681[57], 6}, {NULL}};
const Segment SGI269985[] = {{&S88681[109], 5}, {NULL}};
const Segment SGI269986[] = {{&S88681[122], 9}, {NULL}};
const Segment SGI269987[] = {{&S88704[28], 4}, {NULL}};
const Segment SGI269988[] = {{&S88704[41], 5}, {NULL}};
const Segment SGI269989[] = {{&S88704[74], 6}, {NULL}};
const Segment SGI269990[] = {{&S88704[108], 5}, {NULL}};
const Segment SGI269991[] = {{&S88704[121], 9}, {NULL}};
const Segment SGI269992[] = {{&S88704[0], 14}, {NULL}};
const Segment SGI269993[] = {{&S88734[13], 4}, {&S88734[23], 6}, {NULL}};
const Segment SGI269994[] = {{&S88734[197], 3}, {NULL}};
const Segment SGI269995[] = {{&S88734[33], 3}, {NULL}};
const Segment SGI269996[] = {{&S88734[64], 6}, {&S88734[164], 3}, {NULL}};
const Segment SGI269997[] = {{&S88734[98], 6}, {&S88734[208], 3}, {NULL}};
const Segment SGI269998[] = {{&S88734[219], 5}, {NULL}};
const Segment SGI269999[] = {{&S88735[13], 4}, {&S88735[23], 6}, {NULL}};
const Segment SGI270000[] = {{&S88735[199], 3}, {NULL}};
const Segment SGI270001[] = {{&S88735[33], 5}, {NULL}};
const Segment SGI270002[] = {{&S88735[66], 6}, {&S88735[166], 3}, {NULL}};
const Segment SGI270003[] = {{&S88735[100], 6}, {&S88735[210], 3}, {NULL}};
const Segment SGI270004[] = {{&S88735[221], 5}, {NULL}};
const Segment SGI270005[] = {{&S81756[109], 5}, {NULL}};
const Segment SGI270006[] = {{&S81756[0], 114}, {NULL}};
const Segment SGI270007[] = {{&S81757[110], 5}, {NULL}};
const Segment SGI270008[] = {{&S81757[0], 115}, {NULL}};
const Segment SGI270009[] = {{&S81758[110], 5}, {NULL}};
const Segment SGI270010[] = {{&S81758[0], 115}, {NULL}};
const Segment SGI270011[] = {{&S81788[38], 20}, {NULL}};
const Segment SGI270012[] = {{&S81788[0], 58}, {NULL}};
const Segment SGI270013[] = {{&S81789[41], 18}, {NULL}};
const Segment SGI270014[] = {{&S81789[0], 99}, {NULL}};
const Segment SGI270015[] = {{&S88806[13], 6}, {NULL}};
const Segment SGI270016[] = {{&S88806[35], 5}, {NULL}};
const Segment SGI270017[] = {{&S88806[75], 6}, {NULL}};
const Segment SGI270018[] = {{&S88806[109], 5}, {NULL}};
const Segment SGI270019[] = {{&S88806[115], 13}, {NULL}};
const Segment SGI270020[] = {{&S88807[13], 6}, {NULL}};
const Segment SGI270021[] = {{&S88807[35], 5}, {NULL}};
const Segment SGI270022[] = {{&S88807[75], 6}, {NULL}};
const Segment SGI270023[] = {{&S88807[109], 5}, {NULL}};
const Segment SGI270024[] = {{&S88807[115], 13}, {NULL}};
const Segment SGI270025[] = {{&S88856[30], 4}, {NULL}};
const Segment SGI270026[] = {{&S88856[43], 3}, {NULL}};
const Segment SGI270027[] = {{&S88856[74], 6}, {NULL}};
const Segment SGI270028[] = {{&S88856[108], 6}, {NULL}};
const Segment SGI270029[] = {{&S88856[122], 9}, {NULL}};
const Segment SGI270030[] = {{&S88856[0], 14}, {NULL}};
const Segment SGI270031[] = {{&S88857[30], 4}, {NULL}};
const Segment SGI270032[] = {{&S88857[43], 5}, {NULL}};
const Segment SGI270033[] = {{&S88857[76], 6}, {NULL}};
const Segment SGI270034[] = {{&S88857[110], 6}, {NULL}};
const Segment SGI270035[] = {{&S88857[124], 9}, {NULL}};
const Segment SGI270036[] = {{&S88857[0], 14}, {NULL}};
const Segment SGI270037[] = {{&S89029[13], 4}, {NULL}};
const Segment SGI270038[] = {{&S89029[26], 5}, {NULL}};
const Segment SGI270039[] = {{&S89029[59], 6}, {NULL}};
const Segment SGI270040[] = {{&S89029[108], 5}, {NULL}};
const Segment SGI270041[] = {{&S89029[121], 9}, {NULL}};
const Segment SGI270042[] = {{&S89029[86], 14}, {NULL}};
const Segment SGI270043[] = {{&S82114[68], 5}, {NULL}};
const Segment SGI270044[] = {{&S82114[0], 73}, {NULL}};
const Segment SGI270045[] = {{&S82115[64], 5}, {NULL}};
const Segment SGI270046[] = {{&S82115[0], 69}, {NULL}};
const Segment SGI270047[] = {{&S82116[68], 5}, {&S82116[68], 5}, {NULL}};
const Segment SGI270048[] = {{&S82116[0], 79}, {NULL}};
const Segment SGI270049[] = {{&S82117[61], 5}, {NULL}};
const Segment SGI270050[] = {{&S82117[0], 66}, {NULL}};
const Segment SGI270051[] = {{&S89168[13], 6}, {NULL}};
const Segment SGI270052[] = {{&S89168[35], 5}, {NULL}};
const Segment SGI270053[] = {{&S89168[134], 3}, {NULL}};
const Segment SGI270054[] = {{&S89168[68], 6}, {NULL}};
const Segment SGI270055[] = {{&S89168[102], 5}, {NULL}};
const Segment SGI270056[] = {{&S89168[115], 9}, {NULL}};
const Segment SGI270057[] = {{&S89169[13], 6}, {NULL}};
const Segment SGI270058[] = {{&S89169[35], 5}, {NULL}};
const Segment SGI270059[] = {{&S89169[134], 5}, {NULL}};
const Segment SGI270060[] = {{&S89169[68], 6}, {NULL}};
const Segment SGI270061[] = {{&S89169[102], 5}, {NULL}};
const Segment SGI270062[] = {{&S89169[115], 9}, {NULL}};
const Segment SGI270063[] = {{&S76513[20], 27}, {NULL}};
const Segment SGI270064[] = {{&S76514[20], 27}, {NULL}};
const Segment SGI270065[] = {{&S76515[20], 27}, {NULL}};
const Segment SGI270066[] = {{&S76516[20], 27}, {NULL}};
const Segment SGI270067[] = {{&S89242[27], 6}, {NULL}};
const Segment SGI270068[] = {{&S89242[49], 3}, {NULL}};
const Segment SGI270069[] = {{&S89242[80], 6}, {NULL}};
const Segment SGI270070[] = {{&S89242[114], 5}, {NULL}};
const Segment SGI270071[] = {{&S89242[127], 9}, {NULL}};
const Segment SGI270072[] = {{&S89242[0], 14}, {NULL}};
const Segment SGI270073[] = {{&S89243[27], 6}, {NULL}};
const Segment SGI270074[] = {{&S89243[49], 3}, {NULL}};
const Segment SGI270075[] = {{&S89243[80], 6}, {NULL}};
const Segment SGI270076[] = {{&S89243[114], 5}, {NULL}};
const Segment SGI270077[] = {{&S89243[127], 9}, {NULL}};
const Segment SGI270078[] = {{&S89243[0], 14}, {NULL}};
const Segment SGI270079[] = {{&S89244[27], 6}, {NULL}};
const Segment SGI270080[] = {{&S89244[49], 5}, {NULL}};
const Segment SGI270081[] = {{&S89244[82], 6}, {NULL}};
const Segment SGI270082[] = {{&S89244[116], 5}, {NULL}};
const Segment SGI270083[] = {{&S89244[129], 9}, {NULL}};
const Segment SGI270084[] = {{&S89244[0], 14}, {NULL}};
const Segment SGI270085[] = {{&S89276[17], 4}, {NULL}};
const Segment SGI270086[] = {{&S89276[36], 5}, {NULL}};
const Segment SGI270087[] = {{&S89276[62], 3}, {NULL}};
const Segment SGI270088[] = {{&S89425[13], 6}, {NULL}};
const Segment SGI270089[] = {{&S89425[38], 5}, {NULL}};
const Segment SGI270090[] = {{&S89425[78], 8}, {NULL}};
const Segment SGI270091[] = {{&S89425[138], 5}, {NULL}};
const Segment SGI270092[] = {{&S89425[151], 9}, {NULL}};
const Segment SGI270093[] = {{&S89426[13], 6}, {NULL}};
const Segment SGI270094[] = {{&S89426[38], 5}, {NULL}};
const Segment SGI270095[] = {{&S89426[78], 8}, {NULL}};
const Segment SGI270096[] = {{&S89426[138], 5}, {NULL}};
const Segment SGI270097[] = {{&S89426[151], 9}, {NULL}};
const Segment SGI270098[] = {{&S89443[13], 4}, {NULL}};
const Segment SGI270099[] = {{&S89443[26], 3}, {NULL}};
const Segment SGI270100[] = {{&S89443[57], 6}, {NULL}};
const Segment SGI270101[] = {{&S89443[115], 5}, {NULL}};
const Segment SGI270102[] = {{&S89443[128], 9}, {NULL}};
const Segment SGI270103[] = {{&S89444[13], 4}, {NULL}};
const Segment SGI270104[] = {{&S89444[26], 5}, {NULL}};
const Segment SGI270105[] = {{&S89444[59], 6}, {NULL}};
const Segment SGI270106[] = {{&S89444[117], 5}, {NULL}};
const Segment SGI270107[] = {{&S89444[130], 9}, {NULL}};
const Segment SGI270108[] = {{&S82309[109], 5}, {NULL}};
const Segment SGI270109[] = {{&S82309[0], 114}, {NULL}};
const Segment SGI270110[] = {{&S82310[109], 5}, {NULL}};
const Segment SGI270111[] = {{&S82310[0], 114}, {NULL}};
const Segment SGI270112[] = {{&S82311[109], 5}, {NULL}};
const Segment SGI270113[] = {{&S82311[0], 114}, {NULL}};
const Segment SGI270114[] = {{&S82523[79], 12}, {NULL}};
const Segment SGI270115[] = {{&S82523[0], 91}, {NULL}};
const Segment SGI270116[] = {{&S89677[13], 6}, {NULL}};
const Segment SGI270117[] = {{&S89677[35], 5}, {NULL}};
const Segment SGI270118[] = {{&S89677[75], 6}, {NULL}};
const Segment SGI270119[] = {{&S89677[109], 6}, {NULL}};
const Segment SGI270120[] = {{&S82524[66], 12}, {NULL}};
const Segment SGI270121[] = {{&S89677[116], 13}, {NULL}};
const Segment SGI270122[] = {{&S82524[0], 78}, {NULL}};
const Segment SGI270123[] = {{&S89678[13], 6}, {NULL}};
const Segment SGI270124[] = {{&S89678[35], 5}, {NULL}};
const Segment SGI270125[] = {{&S82525[47], 12}, {NULL}};
const Segment SGI270126[] = {{&S82525[0], 59}, {NULL}};
const Segment SGI270127[] = {{&S89678[75], 6}, {NULL}};
const Segment SGI270128[] = {{&S89678[109], 6}, {NULL}};
const Segment SGI270129[] = {{&S89678[116], 13}, {NULL}};
const Segment SGI270130[] = {{&S82526[60], 12}, {NULL}};
const Segment SGI270131[] = {{&S82526[0], 72}, {NULL}};
const Segment SGI270132[] = {{&S82527[47], 29}, {NULL}};
const Segment SGI270133[] = {{&S82527[0], 76}, {NULL}};
const Segment SGI270134[] = {{&S89710[17], 4}, {NULL}};
const Segment SGI270135[] = {{&S89710[33], 5}, {NULL}};
const Segment SGI270136[] = {{&S89711[17], 4}, {NULL}};
const Segment SGI270137[] = {{&S89711[33], 5}, {NULL}};
const Segment SGI270138[] = {{&S89721[27], 6}, {NULL}};
const Segment SGI270139[] = {{&S89721[49], 5}, {NULL}};
const Segment SGI270140[] = {{&S89721[82], 6}, {NULL}};
const Segment SGI270141[] = {{&S89721[116], 5}, {NULL}};
const Segment SGI270142[] = {{&S89721[129], 9}, {NULL}};
const Segment SGI270143[] = {{&S89721[0], 14}, {NULL}};
const Segment SGI270144[] = {{&S89722[27], 6}, {NULL}};
const Segment SGI270145[] = {{&S89722[49], 5}, {NULL}};
const Segment SGI270146[] = {{&S89722[82], 6}, {NULL}};
const Segment SGI270147[] = {{&S89722[116], 5}, {NULL}};
const Segment SGI270148[] = {{&S89722[129], 9}, {NULL}};
const Segment SGI270149[] = {{&S89722[0], 14}, {NULL}};
const Segment SGI270150[] = {{&S77294[20], 27}, {NULL}};
const Segment SGI270151[] = {{&S77295[20], 27}, {NULL}};
const Segment SGI270152[] = {{&S77296[20], 27}, {NULL}};
const Segment SGI270153[] = {{&S77370[10], 5}, {NULL}};
const Segment SGI270154[] = {{&S77370[16], 5}, {NULL}};
const Segment SGI270155[] = {{&S77540[25], 21}, {NULL}};
const Segment SGI270156[] = {{&S77540[47], 5}, {NULL}};
const Segment SGI270157[] = {{&S90223[17], 4}, {NULL}};
const Segment SGI270158[] = {{&S90223[39], 5}, {NULL}};
const Segment SGI270159[] = {{&S90223[65], 3}, {NULL}};
const Segment SGI270160[] = {{&S90288[13], 4}, {NULL}};
const Segment SGI270161[] = {{&S90288[26], 5}, {NULL}};
const Segment SGI270162[] = {{&S90288[125], 5}, {NULL}};
const Segment SGI270163[] = {{&S90288[59], 6}, {NULL}};
const Segment SGI270164[] = {{&S90288[93], 5}, {NULL}};
const Segment SGI270165[] = {{&S90288[106], 9}, {NULL}};
const Segment SGI270166[] = {{&S90289[13], 4}, {NULL}};
const Segment SGI270167[] = {{&S90289[26], 5}, {NULL}};
const Segment SGI270168[] = {{&S90289[125], 3}, {NULL}};
const Segment SGI270169[] = {{&S90289[59], 6}, {NULL}};
const Segment SGI270170[] = {{&S90289[93], 5}, {NULL}};
const Segment SGI270171[] = {{&S90289[106], 9}, {NULL}};
const Segment SGI270172[] = {{&S77677[10], 4}, {NULL}};
const Segment SGI270173[] = {{&S77677[15], 5}, {NULL}};
const Segment SGI270174[] = {{&S77677[65], 13}, {NULL}};
const Segment SGI270175[] = {{&S90348[29], 4}, {NULL}};
const Segment SGI270176[] = {{&S90348[51], 3}, {NULL}};
const Segment SGI270177[] = {{&S90349[29], 4}, {NULL}};
const Segment SGI270178[] = {{&S90349[51], 5}, {NULL}};
const Segment SGI270179[] = {{&S83045[81], 14}, {&S83045[96], 31}, {NULL}};
const Segment SGI270180[] = {{&S83045[0], 127}, {NULL}};
const Segment SGI270181[] = {{&S83046[76], 14}, {&S83046[91], 31}, {NULL}};
const Segment SGI270182[] = {{&S83046[0], 122}, {NULL}};
const Segment SGI270183[] = {{&S77783[20], 27}, {NULL}};
const Segment SGI270184[] = {{&S77784[20], 27}, {NULL}};
const Segment SGI270185[] = {{&S77785[20], 27}, {NULL}};
const Segment SGI270186[] = {{&S77786[20], 27}, {NULL}};
const Segment SGI270187[] = {{&S77787[20], 27}, {NULL}};
const Segment SGI270188[] = {{&S83136[51], 12}, {NULL}};
const Segment SGI270189[] = {{&S83136[0], 68}, {NULL}};
const Segment SGI270190[] = {{&S83137[51], 14}, {&S83137[66], 3}, {NULL}};
const Segment SGI270191[] = {{&S83137[0], 89}, {NULL}};
const Segment SGI270192[] = {{&S83210[0], 7}, {&S83210[87], 24}, {NULL}};
const Segment SGI270193[] = {{&S83210[0], 111}, {NULL}};
const Segment SGI270194[] = {{&S90606[13], 4}, {NULL}};
const Segment SGI270195[] = {{&S90606[92], 3}, {NULL}};
const Segment SGI270196[] = {{&S90606[26], 3}, {NULL}};
const Segment SGI270197[] = {{&S90606[57], 6}, {NULL}};
const Segment SGI270198[] = {{&S90606[103], 6}, {NULL}};
const Segment SGI270199[] = {{&S90606[117], 7}, {NULL}};
const Segment SGI270200[] = {{&S90607[13], 4}, {NULL}};
const Segment SGI270201[] = {{&S90607[94], 3}, {NULL}};
const Segment SGI270202[] = {{&S90607[26], 5}, {NULL}};
const Segment SGI270203[] = {{&S90607[59], 6}, {NULL}};
const Segment SGI270204[] = {{&S90607[105], 5}, {NULL}};
const Segment SGI270205[] = {{&S90607[118], 9}, {NULL}};
const Segment SGI270206[] = {{&S90729[13], 4}, {NULL}};
const Segment SGI270207[] = {{&S90729[26], 3}, {NULL}};
const Segment SGI270208[] = {{&S90729[126], 5}, {NULL}};
const Segment SGI270209[] = {{&S90729[57], 8}, {NULL}};
const Segment SGI270210[] = {{&S90729[93], 6}, {NULL}};
const Segment SGI270211[] = {{&S90729[107], 9}, {NULL}};
const Segment SGI270212[] = {{&S83499[70], 5}, {NULL}};
const Segment SGI270213[] = {{&S83499[0], 75}, {NULL}};
const Segment SGI270214[] = {{&S83500[63], 5}, {NULL}};
const Segment SGI270215[] = {{&S83500[0], 68}, {NULL}};
const Segment SGI270216[] = {{&S83501[70], 5}, {&S83501[70], 5}, {NULL}};
const Segment SGI270217[] = {{&S83501[0], 81}, {NULL}};
const Segment SGI270218[] = {{&S83502[66], 5}, {NULL}};
const Segment SGI270219[] = {{&S83502[0], 71}, {NULL}};
const Segment SGI270220[] = {{&S90830[13], 6}, {NULL}};
const Segment SGI270221[] = {{&S90830[113], 3}, {NULL}};
const Segment SGI270222[] = {{&S90830[38], 5}, {NULL}};
const Segment SGI270223[] = {{&S90830[78], 6}, {NULL}};
const Segment SGI270224[] = {{&S90830[124], 5}, {NULL}};
const Segment SGI270225[] = {{&S90830[137], 9}, {NULL}};
const Segment SGI270226[] = {{&S90988[13], 6}, {NULL}};
const Segment SGI270227[] = {{&S90988[28], 5}, {NULL}};
const Segment SGI270228[] = {{&S90988[61], 6}, {NULL}};
const Segment SGI270229[] = {{&S90988[114], 6}, {NULL}};
const Segment SGI270230[] = {{&S90988[128], 7}, {NULL}};
const Segment SGI270231[] = {{&S91174[13], 6}, {NULL}};
const Segment SGI270232[] = {{&S91174[113], 3}, {NULL}};
const Segment SGI270233[] = {{&S91174[38], 5}, {NULL}};
const Segment SGI270234[] = {{&S91174[78], 6}, {NULL}};
const Segment SGI270235[] = {{&S91174[124], 6}, {NULL}};
const Segment SGI270236[] = {{&S91174[138], 9}, {NULL}};
const Segment SGI270237[] = {{&S91298[27], 6}, {NULL}};
const Segment SGI270238[] = {{&S91298[49], 5}, {NULL}};
const Segment SGI270239[] = {{&S91298[82], 6}, {NULL}};
const Segment SGI270240[] = {{&S91298[116], 5}, {NULL}};
const Segment SGI270241[] = {{&S91298[129], 9}, {NULL}};
const Segment SGI270242[] = {{&S91298[0], 14}, {NULL}};
const Segment SGI270243[] = {{&S91299[27], 6}, {NULL}};
const Segment SGI270244[] = {{&S91299[49], 3}, {NULL}};
const Segment SGI270245[] = {{&S91299[80], 6}, {NULL}};
const Segment SGI270246[] = {{&S91299[114], 5}, {NULL}};
const Segment SGI270247[] = {{&S91299[127], 9}, {NULL}};
const Segment SGI270248[] = {{&S91299[0], 14}, {NULL}};
const Segment SGI270249[] = {{&S91300[27], 6}, {NULL}};
const Segment SGI270250[] = {{&S91300[49], 3}, {NULL}};
const Segment SGI270251[] = {{&S91300[80], 6}, {NULL}};
const Segment SGI270252[] = {{&S91300[114], 5}, {NULL}};
const Segment SGI270253[] = {{&S91300[127], 9}, {NULL}};
const Segment SGI270254[] = {{&S91300[0], 14}, {NULL}};
const Segment SGI270255[] = {{&S91324[28], 6}, {NULL}};
const Segment SGI270256[] = {{&S91324[55], 5}, {NULL}};
const Segment SGI270257[] = {{&S91324[95], 8}, {NULL}};
const Segment SGI270258[] = {{&S91324[131], 3}, {NULL}};
const Segment SGI270259[] = {{&S91324[142], 9}, {NULL}};
const Segment SGI270260[] = {{&S91324[0], 15}, {NULL}};
const Segment SGI270261[] = {{&S91325[27], 6}, {NULL}};
const Segment SGI270262[] = {{&S91325[54], 5}, {NULL}};
const Segment SGI270263[] = {{&S91325[94], 8}, {NULL}};
const Segment SGI270264[] = {{&S91325[130], 3}, {NULL}};
const Segment SGI270265[] = {{&S91325[141], 9}, {NULL}};
const Segment SGI270266[] = {{&S91325[0], 14}, {NULL}};
const Segment SGI270267[] = {{&S78596[20], 27}, {NULL}};
const Segment SGI270268[] = {{&S78597[20], 27}, {NULL}};
const Segment SGI270269[] = {{&S78598[20], 27}, {NULL}};
const Segment SGI270270[] = {{&S78599[20], 27}, {NULL}};
const Segment SGI270271[] = {{&S78600[20], 27}, {NULL}};
const Segment SGI270272[] = {{&S78601[20], 27}, {NULL}};
const Segment SGI270273[] = {{&S78602[20], 27}, {NULL}};
const Segment SGI270274[] = {{&S78603[20], 27}, {NULL}};
const Segment SGI270275[] = {{&S78604[20], 27}, {NULL}};
const Segment SGI270276[] = {{&S91506[13], 4}, {&S91506[23], 6}, {NULL}};
const Segment SGI270277[] = {{&S91506[198], 3}, {NULL}};
const Segment SGI270278[] = {{&S91506[33], 5}, {NULL}};
const Segment SGI270279[] = {{&S91506[66], 6}, {&S91506[165], 3}, {NULL}};
const Segment SGI270280[] = {{&S91506[100], 5}, {&S91506[209], 3}, {NULL}};
const Segment SGI270281[] = {{&S91506[220], 5}, {NULL}};
const Segment SGI270282[] = {{&S91507[13], 4}, {&S91507[23], 6}, {NULL}};
const Segment SGI270283[] = {{&S91507[196], 3}, {NULL}};
const Segment SGI270284[] = {{&S91507[33], 3}, {NULL}};
const Segment SGI270285[] = {{&S91507[64], 6}, {&S91507[163], 3}, {NULL}};
const Segment SGI270286[] = {{&S91507[98], 5}, {&S91507[207], 3}, {NULL}};
const Segment SGI270287[] = {{&S91507[218], 5}, {NULL}};
const Segment SGI270288[] = {{&S91520[27], 6}, {NULL}};
const Segment SGI270289[] = {{&S91520[51], 5}, {NULL}};
const Segment SGI270290[] = {{&S91520[84], 6}, {NULL}};
const Segment SGI270291[] = {{&S91520[118], 5}, {NULL}};
const Segment SGI270292[] = {{&S91520[131], 9}, {NULL}};
const Segment SGI270293[] = {{&S84084[63], 5}, {NULL}};
const Segment SGI270294[] = {{&S84084[0], 68}, {NULL}};
const Segment SGI270295[] = {{&S91520[0], 14}, {NULL}};
const Segment SGI270296[] = {{&S91521[27], 6}, {NULL}};
const Segment SGI270297[] = {{&S91521[52], 5}, {NULL}};
const Segment SGI270298[] = {{&S91521[85], 6}, {NULL}};
const Segment SGI270299[] = {{&S84085[59], 5}, {NULL}};
const Segment SGI270300[] = {{&S91521[119], 5}, {NULL}};
const Segment SGI270301[] = {{&S84085[0], 64}, {NULL}};
const Segment SGI270302[] = {{&S91521[132], 9}, {NULL}};
const Segment SGI270303[] = {{&S91521[0], 14}, {NULL}};
const Segment SGI270304[] = {{&S84086[63], 5}, {NULL}};
const Segment SGI270305[] = {{&S84086[0], 68}, {NULL}};
const Segment SGI270306[] = {{&S84087[63], 5}, {&S84087[63], 5}, {NULL}};
const Segment SGI270307[] = {{&S84087[0], 74}, {NULL}};
const Segment SGI270308[] = {{&S91609[27], 4}, {NULL}};
const Segment SGI270309[] = {{&S91609[47], 5}, {NULL}};
const Segment SGI270310[] = {{&S91609[80], 6}, {NULL}};
const Segment SGI270311[] = {{&S91609[114], 6}, {NULL}};
const Segment SGI270312[] = {{&S91609[128], 9}, {NULL}};
const Segment SGI270313[] = {{&S91609[0], 14}, {NULL}};
const Segment SGI270314[] = {{&S91708[28], 4}, {NULL}};
const Segment SGI270315[] = {{&S91708[44], 5}, {NULL}};
const Segment SGI270316[] = {{&S91708[84], 8}, {NULL}};
const Segment SGI270317[] = {{&S91708[120], 3}, {NULL}};
const Segment SGI270318[] = {{&S91708[131], 9}, {NULL}};
const Segment SGI270319[] = {{&S91708[0], 14}, {NULL}};
const Segment SGI270320[] = {{&S91709[28], 4}, {NULL}};
const Segment SGI270321[] = {{&S91709[44], 5}, {NULL}};
const Segment SGI270322[] = {{&S91709[84], 8}, {NULL}};
const Segment SGI270323[] = {{&S91709[120], 3}, {NULL}};
const Segment SGI270324[] = {{&S91709[131], 9}, {NULL}};
const Segment SGI270325[] = {{&S91709[0], 14}, {NULL}};
const Segment SGI270326[] = {{&S91710[28], 4}, {NULL}};
const Segment SGI270327[] = {{&S91710[44], 5}, {NULL}};
const Segment SGI270328[] = {{&S91710[84], 8}, {NULL}};
const Segment SGI270329[] = {{&S91710[120], 3}, {NULL}};
const Segment SGI270330[] = {{&S91710[131], 9}, {NULL}};
const Segment SGI270331[] = {{&S91710[0], 14}, {NULL}};
const Segment SGI270332[] = {{&S91827[38], 4}, {NULL}};
const Segment SGI270333[] = {{&S91827[129], 5}, {NULL}};
const Segment SGI270334[] = {{&S91827[54], 3}, {NULL}};
const Segment SGI270335[] = {{&S91827[92], 9}, {NULL}};
const Segment SGI270336[] = {{&S91827[142], 5}, {NULL}};
const Segment SGI270337[] = {{&S91888[28], 4}, {NULL}};
const Segment SGI270338[] = {{&S91888[41], 3}, {NULL}};
const Segment SGI270339[] = {{&S91888[72], 6}, {NULL}};
const Segment SGI270340[] = {{&S91888[106], 5}, {NULL}};
const Segment SGI270341[] = {{&S91888[119], 9}, {NULL}};
const Segment SGI270342[] = {{&S91888[0], 14}, {NULL}};
const Segment SGI270343[] = {{&S91889[28], 4}, {NULL}};
const Segment SGI270344[] = {{&S91889[41], 5}, {NULL}};
const Segment SGI270345[] = {{&S91889[74], 6}, {NULL}};
const Segment SGI270346[] = {{&S91889[108], 5}, {NULL}};
const Segment SGI270347[] = {{&S91889[121], 9}, {NULL}};
const Segment SGI270348[] = {{&S91889[0], 14}, {NULL}};
const Segment SGI270349[] = {{&S91892[13], 6}, {&S91892[13], 6}, {NULL}};
const Segment SGI270350[] = {{&S91892[212], 3}, {NULL}};
const Segment SGI270351[] = {{&S91892[38], 5}, {NULL}};
const Segment SGI270352[] = {{&S91892[78], 8}, {&S91892[179], 3}, {NULL}};
const Segment SGI270353[] = {{&S91892[114], 5}, {&S91892[223], 3}, {NULL}};
const Segment SGI270354[] = {{&S91892[234], 5}, {NULL}};
const Segment SGI270355[] = {{&S91893[13], 6}, {&S91893[13], 6}, {NULL}};
const Segment SGI270356[] = {{&S91893[212], 3}, {NULL}};
const Segment SGI270357[] = {{&S91893[38], 5}, {NULL}};
const Segment SGI270358[] = {{&S91893[78], 8}, {&S91893[179], 3}, {NULL}};
const Segment SGI270359[] = {{&S91893[114], 5}, {&S91893[223], 3}, {NULL}};
const Segment SGI270360[] = {{&S91893[234], 5}, {NULL}};
const Segment SGI270361[] = {{&S91894[13], 6}, {&S91894[13], 6}, {NULL}};
const Segment SGI270362[] = {{&S91894[213], 3}, {NULL}};
const Segment SGI270363[] = {{&S91894[38], 6}, {NULL}};
const Segment SGI270364[] = {{&S91894[79], 8}, {&S91894[180], 3}, {NULL}};
const Segment SGI270365[] = {{&S91894[115], 5}, {&S91894[224], 3}, {NULL}};
const Segment SGI270366[] = {{&S91894[235], 5}, {NULL}};
const Segment SGI270367[] = {{&S84475[64], 29}, {NULL}};
const Segment SGI270368[] = {{&S84475[0], 93}, {NULL}};
const Segment SGI270369[] = {{&S91895[13], 6}, {&S91895[13], 6}, {NULL}};
const Segment SGI270370[] = {{&S91895[212], 3}, {NULL}};
const Segment SGI270371[] = {{&S91895[38], 5}, {NULL}};
const Segment SGI270372[] = {{&S91895[78], 8}, {&S91895[179], 3}, {NULL}};
const Segment SGI270373[] = {{&S91895[114], 5}, {&S91895[223], 3}, {NULL}};
const Segment SGI270374[] = {{&S91895[234], 5}, {NULL}};
const Segment SGI270375[] = {{&S91896[13], 6}, {&S91896[13], 6}, {NULL}};
const Segment SGI270376[] = {{&S91896[213], 3}, {NULL}};
const Segment SGI270377[] = {{&S91896[38], 5}, {NULL}};
const Segment SGI270378[] = {{&S91896[78], 8}, {&S91896[180], 3}, {NULL}};
const Segment SGI270379[] = {{&S91896[114], 6}, {&S91896[224], 3}, {NULL}};
const Segment SGI270380[] = {{&S91896[235], 5}, {NULL}};
const Segment SGI270381[] = {{&S91900[28], 4}, {NULL}};
const Segment SGI270382[] = {{&S91900[44], 5}, {NULL}};
const Segment SGI270383[] = {{&S91900[84], 8}, {NULL}};
const Segment SGI270384[] = {{&S91900[120], 3}, {NULL}};
const Segment SGI270385[] = {{&S91900[131], 9}, {NULL}};
const Segment SGI270386[] = {{&S91900[0], 14}, {NULL}};
const Segment SGI270387[] = {{&S91901[28], 4}, {NULL}};
const Segment SGI270388[] = {{&S91901[44], 5}, {NULL}};
const Segment SGI270389[] = {{&S91901[84], 8}, {NULL}};
const Segment SGI270390[] = {{&S91901[120], 3}, {NULL}};
const Segment SGI270391[] = {{&S91901[131], 9}, {NULL}};
const Segment SGI270392[] = {{&S91901[0], 14}, {NULL}};
const Segment SGI270393[] = {{&S84503[59], 5}, {NULL}};
const Segment SGI270394[] = {{&S84503[0], 64}, {NULL}};
const Segment SGI270395[] = {{&S84504[66], 5}, {NULL}};
const Segment SGI270396[] = {{&S84504[0], 71}, {NULL}};
const Segment SGI270397[] = {{&S91945[13], 4}, {NULL}};
const Segment SGI270398[] = {{&S91945[29], 5}, {NULL}};
const Segment SGI270399[] = {{&S91945[69], 8}, {NULL}};
const Segment SGI270400[] = {{&S91945[120], 5}, {NULL}};
const Segment SGI270401[] = {{&S91945[133], 9}, {NULL}};
const Segment SGI270402[] = {{&S91945[98], 14}, {NULL}};
const Segment SGI270403[] = {{&S92053[13], 4}, {NULL}};
const Segment SGI270404[] = {{&S92053[33], 5}, {NULL}};
const Segment SGI270405[] = {{&S92053[66], 6}, {NULL}};
const Segment SGI270406[] = {{&S92053[119], 5}, {NULL}};
const Segment SGI270407[] = {{&S92053[132], 9}, {NULL}};
const Segment SGI270408[] = {{&S92096[28], 4}, {NULL}};
const Segment SGI270409[] = {{&S92096[41], 5}, {NULL}};
const Segment SGI270410[] = {{&S92096[74], 6}, {NULL}};
const Segment SGI270411[] = {{&S92096[108], 5}, {NULL}};
const Segment SGI270412[] = {{&S92096[121], 9}, {NULL}};
const Segment SGI270413[] = {{&S92096[0], 14}, {NULL}};
const Segment SGI270414[] = {{&S92097[28], 4}, {NULL}};
const Segment SGI270415[] = {{&S92097[41], 3}, {NULL}};
const Segment SGI270416[] = {{&S92097[72], 6}, {NULL}};
const Segment SGI270417[] = {{&S92097[106], 5}, {NULL}};
const Segment SGI270418[] = {{&S92097[119], 9}, {NULL}};
const Segment SGI270419[] = {{&S92097[0], 14}, {NULL}};
const Segment SGI270420[] = {{&S84688[68], 5}, {NULL}};
const Segment SGI270421[] = {{&S84688[0], 73}, {NULL}};
const Segment SGI270422[] = {{&S84689[61], 5}, {NULL}};
const Segment SGI270423[] = {{&S84689[0], 66}, {NULL}};
const Segment SGI270424[] = {{&S84690[68], 5}, {&S84690[68], 5}, {NULL}};
const Segment SGI270425[] = {{&S84690[0], 79}, {NULL}};
const Segment SGI270426[] = {{&S84895[59], 5}, {NULL}};
const Segment SGI270427[] = {{&S84895[0], 64}, {NULL}};
const Segment SGI270428[] = {{&S84896[66], 5}, {NULL}};
const Segment SGI270429[] = {{&S84896[0], 71}, {NULL}};
const Segment SGI270430[] = {{&S92427[13], 4}, {NULL}};
const Segment SGI270431[] = {{&S92427[33], 5}, {NULL}};
const Segment SGI270432[] = {{&S92427[133], 5}, {NULL}};
const Segment SGI270433[] = {{&S92427[66], 6}, {NULL}};
const Segment SGI270434[] = {{&S92427[100], 6}, {NULL}};
const Segment SGI270435[] = {{&S92427[114], 9}, {NULL}};
const Segment SGI270436[] = {{&S92428[13], 4}, {NULL}};
const Segment SGI270437[] = {{&S92428[33], 5}, {NULL}};
const Segment SGI270438[] = {{&S92428[133], 5}, {NULL}};
const Segment SGI270439[] = {{&S92428[66], 6}, {NULL}};
const Segment SGI270440[] = {{&S92428[100], 6}, {NULL}};
const Segment SGI270441[] = {{&S92428[114], 9}, {NULL}};
const Segment SGI270442[] = {{&S84957[59], 5}, {NULL}};
const Segment SGI270443[] = {{&S84957[0], 64}, {NULL}};
const Segment SGI270444[] = {{&S92486[13], 4}, {NULL}};
const Segment SGI270445[] = {{&S92486[36], 3}, {NULL}};
const Segment SGI270446[] = {{&S92486[74], 5}, {NULL}};
const Segment SGI270447[] = {{&S92486[107], 5}, {NULL}};
const Segment SGI270448[] = {{&S84958[66], 5}, {NULL}};
const Segment SGI270449[] = {{&S84958[0], 71}, {NULL}};
const Segment SGI270450[] = {{&S92486[113], 7}, {NULL}};
const Segment SGI270451[] = {{&S92487[13], 4}, {NULL}};
const Segment SGI270452[] = {{&S92487[36], 5}, {NULL}};
const Segment SGI270453[] = {{&S92487[76], 5}, {NULL}};
const Segment SGI270454[] = {{&S92487[109], 5}, {NULL}};
const Segment SGI270455[] = {{&S92487[115], 7}, {NULL}};
const Segment SGI270456[] = {{&S84959[66], 5}, {&S84959[66], 5}, {NULL}};
const Segment SGI270457[] = {{&S84959[0], 77}, {NULL}};
const Segment SGI270458[] = {{&S84966[59], 5}, {NULL}};
const Segment SGI270459[] = {{&S84966[0], 64}, {NULL}};
const Segment SGI270460[] = {{&S84967[61], 5}, {NULL}};
const Segment SGI270461[] = {{&S84967[0], 66}, {NULL}};
const Segment SGI270462[] = {{&S84968[61], 5}, {&S84968[61], 5}, {NULL}};
const Segment SGI270463[] = {{&S84968[0], 72}, {NULL}};
const Segment SGI270464[] = {{&S84969[59], 5}, {&S84969[59], 5}, {NULL}};
const Segment SGI270465[] = {{&S84969[0], 70}, {NULL}};
const Segment SGI270466[] = {{&S85060[57], 5}, {NULL}};
const Segment SGI270467[] = {{&S85060[0], 62}, {NULL}};
const Segment SGI270468[] = {{&S85061[61], 5}, {NULL}};
const Segment SGI270469[] = {{&S85061[0], 66}, {NULL}};
const Segment SGI270470[] = {{&S85062[57], 5}, {&S85062[57], 5}, {NULL}};
const Segment SGI270471[] = {{&S85062[0], 68}, {NULL}};
const Segment SGI270472[] = {{&S92603[17], 4}, {NULL}};
const Segment SGI270473[] = {{&S92603[36], 5}, {NULL}};
const Segment SGI270474[] = {{&S92603[66], 3}, {NULL}};
const Segment SGI270475[] = {{&S92682[17], 4}, {NULL}};
const Segment SGI270476[] = {{&S92682[39], 3}, {NULL}};
const Segment SGI270477[] = {{&S92683[17], 4}, {NULL}};
const Segment SGI270478[] = {{&S92683[39], 5}, {NULL}};
const Segment SGI270479[] = {{&S92684[17], 4}, {NULL}};
const Segment SGI270480[] = {{&S92684[39], 5}, {NULL}};
const Segment SGI270481[] = {{&S92684[65], 3}, {NULL}};
const Segment SGI270482[] = {{&S92685[13], 6}, {NULL}};
const Segment SGI270483[] = {{&S92685[35], 5}, {NULL}};
const Segment SGI270484[] = {{&S92685[68], 6}, {NULL}};
const Segment SGI270485[] = {{&S92685[126], 5}, {NULL}};
const Segment SGI270486[] = {{&S92685[139], 9}, {NULL}};
const Segment SGI270487[] = {{&S85325[66], 5}, {NULL}};
const Segment SGI270488[] = {{&S85325[0], 71}, {NULL}};
const Segment SGI270489[] = {{&S92847[13], 6}, {&S92847[13], 6}, {NULL}};
const Segment SGI270490[] = {{&S92847[198], 3}, {NULL}};
const Segment SGI270491[] = {{&S92847[35], 3}, {NULL}};
const Segment SGI270492[] = {{&S92847[66], 6}, {&S92847[165], 3}, {NULL}};
const Segment SGI270493[] = {{&S92847[100], 5}, {&S92847[209], 3}, {NULL}};
const Segment SGI270494[] = {{&S92847[220], 5}, {NULL}};
const Segment SGI270495[] = {{&S92848[13], 6}, {&S92848[13], 6}, {NULL}};
const Segment SGI270496[] = {{&S92848[200], 3}, {NULL}};
const Segment SGI270497[] = {{&S92848[35], 5}, {NULL}};
const Segment SGI270498[] = {{&S92848[68], 6}, {&S92848[167], 3}, {NULL}};
const Segment SGI270499[] = {{&S92848[102], 5}, {&S92848[211], 3}, {NULL}};
const Segment SGI270500[] = {{&S92848[222], 5}, {NULL}};
const Segment SGI270501[] = {{&S92849[13], 6}, {&S92849[13], 6}, {NULL}};
const Segment SGI270502[] = {{&S92849[200], 3}, {NULL}};
const Segment SGI270503[] = {{&S92849[35], 5}, {NULL}};
const Segment SGI270504[] = {{&S92849[68], 6}, {&S92849[167], 3}, {NULL}};
const Segment SGI270505[] = {{&S92849[102], 5}, {&S92849[211], 3}, {NULL}};
const Segment SGI270506[] = {{&S92849[222], 5}, {NULL}};
const Segment SGI270507[] = {{&S79989[10], 4}, {NULL}};
const Segment SGI270508[] = {{&S79989[15], 5}, {NULL}};
const Segment SGI270509[] = {{&S79989[65], 13}, {NULL}};
const Segment SGI270510[] = {{&S92913[13], 4}, {NULL}};
const Segment SGI270511[] = {{&S92913[99], 3}, {NULL}};
const Segment SGI270512[] = {{&S92913[33], 3}, {NULL}};
const Segment SGI270513[] = {{&S92913[64], 6}, {NULL}};
const Segment SGI270514[] = {{&S92913[110], 6}, {NULL}};
const Segment SGI270515[] = {{&S92913[124], 7}, {NULL}};
const Segment SGI270516[] = {{&S92914[13], 4}, {NULL}};
const Segment SGI270517[] = {{&S92914[99], 3}, {NULL}};
const Segment SGI270518[] = {{&S92914[33], 3}, {NULL}};
const Segment SGI270519[] = {{&S92914[64], 6}, {NULL}};
const Segment SGI270520[] = {{&S92914[110], 6}, {NULL}};
const Segment SGI270521[] = {{&S92914[124], 7}, {NULL}};
const Segment SGI270522[] = {{&S92949[13], 4}, {NULL}};
const Segment SGI270523[] = {{&S92949[97], 3}, {NULL}};
const Segment SGI270524[] = {{&S92949[29], 5}, {NULL}};
const Segment SGI270525[] = {{&S92949[62], 6}, {NULL}};
const Segment SGI270526[] = {{&S92949[108], 6}, {NULL}};
const Segment SGI270527[] = {{&S92949[122], 7}, {NULL}};
const Segment SGI270528[] = {{&S92954[13], 6}, {NULL}};
const Segment SGI270529[] = {{&S92954[35], 5}, {NULL}};
const Segment SGI270530[] = {{&S92954[68], 6}, {NULL}};
const Segment SGI270531[] = {{&S92954[102], 6}, {NULL}};
const Segment SGI270532[] = {{&S92954[116], 9}, {NULL}};
const Segment SGI270533[] = {{&S92955[13], 6}, {NULL}};
const Segment SGI270534[] = {{&S92955[35], 3}, {NULL}};
const Segment SGI270535[] = {{&S92955[66], 6}, {NULL}};
const Segment SGI270536[] = {{&S92955[100], 6}, {NULL}};
const Segment SGI270537[] = {{&S92955[114], 9}, {NULL}};
const Segment SGI270538[] = {{&S92970[13], 6}, {NULL}};
const Segment SGI270539[] = {{&S92970[138], 5}, {NULL}};
const Segment SGI270540[] = {{&S92970[35], 5}, {NULL}};
const Segment SGI270541[] = {{&S92970[68], 6}, {NULL}};
const Segment SGI270542[] = {{&S92970[8], 1}, {NULL}};
const Segment SGI270543[] = {{&S92971[13], 6}, {NULL}};
const Segment SGI270544[] = {{&S92971[138], 5}, {NULL}};
const Segment SGI270545[] = {{&S92971[35], 5}, {NULL}};
const Segment SGI270546[] = {{&S92971[68], 6}, {NULL}};
const Segment SGI270547[] = {{&S92971[8], 1}, {NULL}};
const Segment SGI270548[] = {{&S85659[57], 5}, {NULL}};
const Segment SGI270549[] = {{&S85659[0], 62}, {NULL}};
const Segment SGI270550[] = {{&S85660[64], 5}, {NULL}};
const Segment SGI270551[] = {{&S85660[0], 69}, {NULL}};
const Segment SGI270552[] = {{&S80200[10], 9}, {NULL}};
const Segment SGI270553[] = {{&S80201[10], 4}, {NULL}};
const Segment SGI270554[] = {{&S80201[15], 5}, {NULL}};
const Segment SGI270555[] = {{&S93107[17], 4}, {NULL}};
const Segment SGI270556[] = {{&S93107[33], 5}, {NULL}};
const Segment SGI270557[] = {{&S93107[59], 3}, {NULL}};
const Segment SGI270558[] = {{&S93108[17], 4}, {NULL}};
const Segment SGI270559[] = {{&S93108[33], 3}, {NULL}};
const Segment SGI270560[] = {{&S93108[57], 3}, {NULL}};
const Segment SGI270561[] = {{&S93131[13], 6}, {NULL}};
const Segment SGI270562[] = {{&S93131[130], 3}, {NULL}};
const Segment SGI270563[] = {{&S93131[38], 3}, {NULL}};
const Segment SGI270564[] = {{&S93131[76], 5}, {NULL}};
const Segment SGI270565[] = {{&S93131[109], 5}, {NULL}};
const Segment SGI270566[] = {{&S93132[13], 6}, {NULL}};
const Segment SGI270567[] = {{&S93132[110], 3}, {NULL}};
const Segment SGI270568[] = {{&S93132[38], 3}, {NULL}};
const Segment SGI270569[] = {{&S93132[76], 5}, {NULL}};
const Segment SGI270570[] = {{&S93132[121], 5}, {NULL}};
const Segment SGI270571[] = {{&S93132[134], 9}, {NULL}};
const Segment SGI270572[] = {{&S93204[27], 6}, {NULL}};
const Segment SGI270573[] = {{&S93204[52], 5}, {NULL}};
const Segment SGI270574[] = {{&S93204[85], 6}, {NULL}};
const Segment SGI270575[] = {{&S93204[119], 6}, {NULL}};
const Segment SGI270576[] = {{&S93204[133], 9}, {NULL}};
const Segment SGI270577[] = {{&S93204[0], 14}, {NULL}};
const Segment SGI270578[] = {{&S93205[28], 6}, {NULL}};
const Segment SGI270579[] = {{&S93205[53], 5}, {NULL}};
const Segment SGI270580[] = {{&S93205[86], 6}, {NULL}};
const Segment SGI270581[] = {{&S93205[120], 6}, {NULL}};
const Segment SGI270582[] = {{&S93205[134], 9}, {NULL}};
const Segment SGI270583[] = {{&S93205[0], 15}, {NULL}};
const Segment SGI270584[] = {{&S93300[28], 4}, {NULL}};
const Segment SGI270585[] = {{&S93300[44], 5}, {NULL}};
const Segment SGI270586[] = {{&S93300[84], 8}, {NULL}};
const Segment SGI270587[] = {{&S93300[120], 3}, {NULL}};
const Segment SGI270588[] = {{&S93300[131], 9}, {NULL}};
const Segment SGI270589[] = {{&S93300[0], 14}, {NULL}};
const Segment SGI270590[] = {{&S93301[28], 4}, {NULL}};
const Segment SGI270591[] = {{&S93301[44], 5}, {NULL}};
const Segment SGI270592[] = {{&S93301[84], 8}, {NULL}};
const Segment SGI270593[] = {{&S93301[120], 3}, {NULL}};
const Segment SGI270594[] = {{&S93301[131], 9}, {NULL}};
const Segment SGI270595[] = {{&S93301[0], 14}, {NULL}};
const Segment SGI270596[] = {{&S85993[12], 13}, {NULL}};
const Segment SGI270597[] = {{&S85993[0], 119}, {NULL}};
const Segment SGI270598[] = {{&S93344[12], 4}, {NULL}};
const Segment SGI270599[] = {{&S93344[74], 5}, {NULL}};
const Segment SGI270600[] = {{&S80452[10], 4}, {NULL}};
const Segment SGI270601[] = {{&S80452[15], 5}, {NULL}};
const Segment SGI270602[] = {{&S86135[107], 5}, {NULL}};
const Segment SGI270603[] = {{&S86135[0], 112}, {NULL}};
const Segment SGI270604[] = {{&S86136[107], 5}, {NULL}};
const Segment SGI270605[] = {{&S86136[0], 112}, {NULL}};
const Segment SGI270606[] = {{&S86137[100], 5}, {NULL}};
const Segment SGI270607[] = {{&S86137[0], 105}, {NULL}};
const Segment SGI270608[] = {{&S80676[20], 26}, {NULL}};
const Segment SGI270609[] = {{&S80677[20], 27}, {NULL}};
const Segment SGI270610[] = {{&S80678[20], 27}, {NULL}};
const Segment SGI270611[] = {{&S80679[20], 27}, {NULL}};
const Segment SGI270612[] = {{&S80680[20], 26}, {NULL}};
const Segment SGI270613[] = {{&S86324[100], 32}, {NULL}};
const Segment SGI270614[] = {{&S86324[0], 132}, {NULL}};
const Segment SGI270615[] = {{&S86378[59], 5}, {NULL}};
const Segment SGI270616[] = {{&S86378[0], 64}, {NULL}};
const Segment SGI270617[] = {{&S86404[59], 5}, {NULL}};
const Segment SGI270618[] = {{&S86404[0], 64}, {NULL}};
const Segment SGI270619[] = {{&S93758[13], 6}, {&S93758[13], 6}, {NULL}};
const Segment SGI270620[] = {{&S93758[210], 3}, {NULL}};
const Segment SGI270621[] = {{&S93758[38], 5}, {NULL}};
const Segment SGI270622[] = {{&S93758[78], 8}, {&S93758[177], 3}, {NULL}};
const Segment SGI270623[] = {{&S93758[114], 3}, {&S93758[221], 3}, {NULL}};
const Segment SGI270624[] = {{&S93758[232], 5}, {NULL}};
const Segment SGI270625[] = {{&S93759[13], 6}, {&S93759[13], 6}, {NULL}};
const Segment SGI270626[] = {{&S93759[210], 3}, {NULL}};
const Segment SGI270627[] = {{&S93759[38], 5}, {NULL}};
const Segment SGI270628[] = {{&S93759[78], 8}, {&S93759[177], 3}, {NULL}};
const Segment SGI270629[] = {{&S93759[114], 3}, {&S93759[221], 3}, {NULL}};
const Segment SGI270630[] = {{&S93759[232], 5}, {NULL}};
const Segment SGI270631[] = {{&S93776[28], 4}, {NULL}};
const Segment SGI270632[] = {{&S93776[44], 5}, {NULL}};
const Segment SGI270633[] = {{&S93776[84], 8}, {NULL}};
const Segment SGI270634[] = {{&S93776[120], 3}, {NULL}};
const Segment SGI270635[] = {{&S93776[131], 9}, {NULL}};
const Segment SGI270636[] = {{&S93776[0], 14}, {NULL}};
const Segment SGI270637[] = {{&S93777[28], 4}, {NULL}};
const Segment SGI270638[] = {{&S93777[44], 5}, {NULL}};
const Segment SGI270639[] = {{&S93777[84], 8}, {NULL}};
const Segment SGI270640[] = {{&S93777[120], 3}, {NULL}};
const Segment SGI270641[] = {{&S93777[131], 9}, {NULL}};
const Segment SGI270642[] = {{&S93777[0], 14}, {NULL}};
const Segment SGI270643[] = {{&S93778[28], 4}, {NULL}};
const Segment SGI270644[] = {{&S93778[44], 5}, {NULL}};
const Segment SGI270645[] = {{&S93778[84], 8}, {NULL}};
const Segment SGI270646[] = {{&S93778[120], 3}, {NULL}};
const Segment SGI270647[] = {{&S93778[131], 9}, {NULL}};
const Segment SGI270648[] = {{&S93778[0], 14}, {NULL}};
const Segment SGI270649[] = {{&S93780[13], 4}, {NULL}};
const Segment SGI270650[] = {{&S93780[101], 5}, {NULL}};
const Segment SGI270651[] = {{&S93780[33], 5}, {NULL}};
const Segment SGI270652[] = {{&S93780[66], 6}, {NULL}};
const Segment SGI270653[] = {{&S93780[114], 5}, {NULL}};
const Segment SGI270654[] = {{&S93780[127], 9}, {NULL}};
const Segment SGI270655[] = {{&S93781[13], 4}, {NULL}};
const Segment SGI270656[] = {{&S93781[113], 5}, {NULL}};
const Segment SGI270657[] = {{&S93781[36], 5}, {NULL}};
const Segment SGI270658[] = {{&S93781[76], 8}, {NULL}};
const Segment SGI270659[] = {{&S93781[126], 5}, {NULL}};
const Segment SGI270660[] = {{&S93781[139], 9}, {NULL}};
const Segment SGI270661[] = {{&S86603[59], 5}, {NULL}};
const Segment SGI270662[] = {{&S86603[0], 64}, {NULL}};
const Segment SGI270663[] = {{&S86754[7], 26}, {NULL}};
const Segment SGI270664[] = {{&S86754[0], 33}, {NULL}};
const Segment SGI270665[] = {{&S86755[5], 6}, {NULL}};
const Segment SGI270666[] = {{&S86755[0], 32}, {NULL}};
const Segment SGI270667[] = {{&S86756[12], 6}, {NULL}};
const Segment SGI270668[] = {{&S86756[0], 45}, {NULL}};
const Segment SGI270669[] = {{&S94218[27], 6}, {NULL}};
const Segment SGI270670[] = {{&S94218[49], 5}, {NULL}};
const Segment SGI270671[] = {{&S94218[82], 6}, {NULL}};
const Segment SGI270672[] = {{&S94218[116], 5}, {NULL}};
const Segment SGI270673[] = {{&S94218[129], 9}, {NULL}};
const Segment SGI270674[] = {{&S94218[0], 14}, {NULL}};
const Segment SGI270675[] = {{&S94219[27], 6}, {NULL}};
const Segment SGI270676[] = {{&S94219[49], 5}, {NULL}};
const Segment SGI270677[] = {{&S94219[82], 6}, {NULL}};
const Segment SGI270678[] = {{&S94219[116], 5}, {NULL}};
const Segment SGI270679[] = {{&S94219[129], 9}, {NULL}};
const Segment SGI270680[] = {{&S94219[0], 14}, {NULL}};
const Segment SGI270681[] = {{&S94223[17], 4}, {NULL}};
const Segment SGI270682[] = {{&S94223[39], 5}, {NULL}};
const Segment SGI270683[] = {{&S94267[27], 6}, {NULL}};
const Segment SGI270684[] = {{&S94267[54], 5}, {NULL}};
const Segment SGI270685[] = {{&S94267[94], 8}, {NULL}};
const Segment SGI270686[] = {{&S94267[130], 3}, {NULL}};
const Segment SGI270687[] = {{&S94267[141], 9}, {NULL}};
const Segment SGI270688[] = {{&S94267[0], 14}, {NULL}};
const Segment SGI270689[] = {{&S81308[10], 5}, {NULL}};
const Segment SGI270690[] = {{&S81308[16], 5}, {NULL}};
const Segment SGI270691[] = {{&S86989[59], 5}, {NULL}};
const Segment SGI270692[] = {{&S86989[0], 64}, {NULL}};
const Segment SGI270693[] = {{&S94316[13], 4}, {NULL}};
const Segment SGI270694[] = {{&S94316[26], 5}, {NULL}};
const Segment SGI270695[] = {{&S94316[59], 6}, {NULL}};
const Segment SGI270696[] = {{&S94316[117], 5}, {NULL}};
const Segment SGI270697[] = {{&S94316[130], 9}, {NULL}};
const Segment SGI270698[] = {{&S87194[70], 8}, {&S87194[79], 22}, {NULL}};
const Segment SGI270699[] = {{&S87194[0], 101}, {NULL}};
const Segment SGI270700[] = {{&S87195[61], 5}, {NULL}};
const Segment SGI270701[] = {{&S87195[0], 66}, {NULL}};
const Segment SGI270702[] = {{&S87196[57], 5}, {NULL}};
const Segment SGI270703[] = {{&S87196[0], 62}, {NULL}};
const Segment SGI270704[] = {{&S87275[0], 13}, {&S87275[14], 36}, {NULL}};
const Segment SGI270705[] = {{&S87275[0], 167}, {NULL}};
const Segment SGI270706[] = {{&S87276[0], 13}, {&S87276[14], 36}, {NULL}};
const Segment SGI270707[] = {{&S87276[0], 167}, {NULL}};
const Segment SGI270708[] = {{&S87277[0], 13}, {&S87277[14], 36}, {NULL}};
const Segment SGI270709[] = {{&S87277[0], 167}, {NULL}};
const Segment SGI270710[] = {{&S94470[17], 4}, {NULL}};
const Segment SGI270711[] = {{&S94470[39], 5}, {NULL}};
const Segment SGI270712[] = {{&S94470[65], 3}, {NULL}};
const Segment SGI270713[] = {{&S94683[13], 4}, {NULL}};
const Segment SGI270714[] = {{&S94683[26], 5}, {NULL}};
const Segment SGI270715[] = {{&S94683[125], 3}, {NULL}};
const Segment SGI270716[] = {{&S94683[59], 6}, {NULL}};
const Segment SGI270717[] = {{&S94683[93], 5}, {NULL}};
const Segment SGI270718[] = {{&S94683[106], 9}, {NULL}};
const Segment SGI270719[] = {{&S94684[13], 4}, {NULL}};
const Segment SGI270720[] = {{&S94684[26], 3}, {NULL}};
const Segment SGI270721[] = {{&S94684[123], 3}, {NULL}};
const Segment SGI270722[] = {{&S94684[57], 6}, {NULL}};
const Segment SGI270723[] = {{&S94684[91], 5}, {NULL}};
const Segment SGI270724[] = {{&S94684[104], 9}, {NULL}};
const Segment SGI270725[] = {{&S94700[28], 4}, {NULL}};
const Segment SGI270726[] = {{&S94700[41], 3}, {NULL}};
const Segment SGI270727[] = {{&S94700[72], 6}, {NULL}};
const Segment SGI270728[] = {{&S94700[106], 5}, {NULL}};
const Segment SGI270729[] = {{&S94700[119], 9}, {NULL}};
const Segment SGI270730[] = {{&S94700[0], 14}, {NULL}};
const Segment SGI270731[] = {{&S94701[28], 4}, {NULL}};
const Segment SGI270732[] = {{&S94701[41], 5}, {NULL}};
const Segment SGI270733[] = {{&S94701[74], 6}, {NULL}};
const Segment SGI270734[] = {{&S94701[108], 5}, {NULL}};
const Segment SGI270735[] = {{&S94701[121], 9}, {NULL}};
const Segment SGI270736[] = {{&S94701[0], 14}, {NULL}};
const Segment SGI270737[] = {{&S94890[13], 6}, {NULL}};
const Segment SGI270738[] = {{&S94890[38], 3}, {NULL}};
const Segment SGI270739[] = {{&S94890[76], 8}, {NULL}};
const Segment SGI270740[] = {{&S94906[27], 6}, {NULL}};
const Segment SGI270741[] = {{&S94906[49], 5}, {NULL}};
const Segment SGI270742[] = {{&S94906[82], 6}, {NULL}};
const Segment SGI270743[] = {{&S94906[116], 5}, {NULL}};
const Segment SGI270744[] = {{&S94906[129], 9}, {NULL}};
const Segment SGI270745[] = {{&S94906[0], 14}, {NULL}};
const Segment SGI270746[] = {{&S94907[27], 6}, {NULL}};
const Segment SGI270747[] = {{&S94907[49], 5}, {NULL}};
const Segment SGI270748[] = {{&S94907[82], 6}, {NULL}};
const Segment SGI270749[] = {{&S94907[116], 5}, {NULL}};
const Segment SGI270750[] = {{&S94907[129], 9}, {NULL}};
const Segment SGI270751[] = {{&S94907[0], 14}, {NULL}};
const Segment SGI270752[] = {{&S87678[57], 5}, {NULL}};
const Segment SGI270753[] = {{&S87678[0], 62}, {NULL}};
const Segment SGI270754[] = {{&S87932[68], 5}, {NULL}};
const Segment SGI270755[] = {{&S87932[0], 73}, {NULL}};
const Segment SGI270756[] = {{&S87933[64], 5}, {NULL}};
const Segment SGI270757[] = {{&S87933[0], 69}, {NULL}};
const Segment SGI270758[] = {{&S87956[59], 5}, {NULL}};
const Segment SGI270759[] = {{&S87956[0], 64}, {NULL}};
const Segment SGI270760[] = {{&S95288[13], 4}, {NULL}};
const Segment SGI270761[] = {{&S95288[26], 5}, {NULL}};
const Segment SGI270762[] = {{&S95288[59], 6}, {NULL}};
const Segment SGI270763[] = {{&S95288[93], 6}, {NULL}};
const Segment SGI270764[] = {{&S95288[100], 15}, {NULL}};
const Segment SGI270765[] = {{&S95426[17], 4}, {NULL}};
const Segment SGI270766[] = {{&S95426[39], 5}, {NULL}};
const Segment SGI270767[] = {{&S95426[65], 3}, {NULL}};
const Segment SGI270768[] = {{&S95460[13], 4}, {NULL}};
const Segment SGI270769[] = {{&S95460[115], 3}, {NULL}};
const Segment SGI270770[] = {{&S95460[33], 5}, {NULL}};
const Segment SGI270771[] = {{&S95460[66], 6}, {NULL}};
const Segment SGI270772[] = {{&S95460[100], 6}, {NULL}};
const Segment SGI270773[] = {{&S95460[126], 9}, {NULL}};
const Segment SGI270774[] = {{&S88317[61], 5}, {NULL}};
const Segment SGI270775[] = {{&S88317[0], 66}, {NULL}};
const Segment SGI270776[] = {{&S88318[68], 5}, {NULL}};
const Segment SGI270777[] = {{&S88318[0], 73}, {NULL}};
const Segment SGI270778[] = {{&S95713[13], 6}, {NULL}};
const Segment SGI270779[] = {{&S95713[35], 5}, {NULL}};
const Segment SGI270780[] = {{&S95713[134], 3}, {NULL}};
const Segment SGI270781[] = {{&S95713[68], 6}, {NULL}};
const Segment SGI270782[] = {{&S95713[102], 5}, {NULL}};
const Segment SGI270783[] = {{&S95713[115], 9}, {NULL}};
const Segment SGI270784[] = {{&S95721[27], 6}, {NULL}};
const Segment SGI270785[] = {{&S95721[54], 5}, {NULL}};
const Segment SGI270786[] = {{&S95721[94], 8}, {NULL}};
const Segment SGI270787[] = {{&S95721[130], 3}, {NULL}};
const Segment SGI270788[] = {{&S95721[141], 9}, {NULL}};
const Segment SGI270789[] = {{&S95721[0], 14}, {NULL}};
const Segment SGI270790[] = {{&S88556[66], 5}, {NULL}};
const Segment SGI270791[] = {{&S88556[0], 71}, {NULL}};
const Segment SGI270792[] = {{&S88840[59], 5}, {NULL}};
const Segment SGI270793[] = {{&S88840[0], 64}, {NULL}};
const Segment SGI270794[] = {{&S88841[59], 5}, {&S88841[59], 5}, {NULL}};
const Segment SGI270795[] = {{&S88841[0], 70}, {NULL}};
const Segment SGI270796[] = {{&S88847[59], 5}, {NULL}};
const Segment SGI270797[] = {{&S88847[0], 64}, {NULL}};
const Segment SGI270798[] = {{&S88848[66], 5}, {NULL}};
const Segment SGI270799[] = {{&S88848[0], 71}, {NULL}};
const Segment SGI270800[] = {{&S83376[10], 4}, {NULL}};
const Segment SGI270801[] = {{&S83376[15], 5}, {NULL}};
const Segment SGI270802[] = {{&S83376[17], 3}, {NULL}};
const Segment SGI270803[] = {{&S96363[27], 6}, {NULL}};
const Segment SGI270804[] = {{&S96363[52], 3}, {NULL}};
const Segment SGI270805[] = {{&S96363[83], 6}, {NULL}};
const Segment SGI270806[] = {{&S96363[117], 6}, {NULL}};
const Segment SGI270807[] = {{&S96363[131], 9}, {NULL}};
const Segment SGI270808[] = {{&S96363[0], 14}, {NULL}};
const Segment SGI270809[] = {{&S96367[13], 4}, {&S96367[13], 4}, {NULL}};
const Segment SGI270810[] = {{&S96367[29], 5}, {NULL}};
const Segment SGI270811[] = {{&S96367[69], 8}, {NULL}};
const Segment SGI270812[] = {{&S96367[105], 5}, {NULL}};
const Segment SGI270813[] = {{&S96367[118], 9}, {NULL}};
const Segment SGI270814[] = {{&S88908[102], 5}, {NULL}};
const Segment SGI270815[] = {{&S88908[0], 107}, {NULL}};
const Segment SGI270816[] = {{&S88979[25], 15}, {&S88979[41], 36}, {NULL}};
const Segment SGI270817[] = {{&S88979[0], 77}, {NULL}};
const Segment SGI270818[] = {{&S88980[0], 9}, {NULL}};
const Segment SGI270819[] = {{&S88985[76], 14}, {NULL}};
const Segment SGI270820[] = {{&S88985[0], 90}, {NULL}};
const Segment SGI270821[] = {{&S96630[28], 4}, {NULL}};
const Segment SGI270822[] = {{&S96630[41], 5}, {NULL}};
const Segment SGI270823[] = {{&S96630[74], 6}, {NULL}};
const Segment SGI270824[] = {{&S96630[108], 5}, {NULL}};
const Segment SGI270825[] = {{&S96630[121], 9}, {NULL}};
const Segment SGI270826[] = {{&S96630[0], 14}, {NULL}};
const Segment SGI270827[] = {{&S96631[28], 4}, {NULL}};
const Segment SGI270828[] = {{&S96631[41], 3}, {NULL}};
const Segment SGI270829[] = {{&S96631[72], 6}, {NULL}};
const Segment SGI270830[] = {{&S96631[106], 5}, {NULL}};
const Segment SGI270831[] = {{&S96631[119], 9}, {NULL}};
const Segment SGI270832[] = {{&S96631[0], 14}, {NULL}};
const Segment SGI270833[] = {{&S83668[10], 4}, {NULL}};
const Segment SGI270834[] = {{&S83668[15], 5}, {NULL}};
const Segment SGI270835[] = {{&S96891[13], 4}, {NULL}};
const Segment SGI270836[] = {{&S96891[26], 5}, {NULL}};
const Segment SGI270837[] = {{&S96891[59], 6}, {NULL}};
const Segment SGI270838[] = {{&S96891[111], 6}, {NULL}};
const Segment SGI270839[] = {{&S96891[125], 9}, {NULL}};
const Segment SGI270840[] = {{&S96892[13], 4}, {NULL}};
const Segment SGI270841[] = {{&S96892[26], 5}, {NULL}};
const Segment SGI270842[] = {{&S96892[59], 6}, {NULL}};
const Segment SGI270843[] = {{&S96892[111], 7}, {NULL}};
const Segment SGI270844[] = {{&S96892[126], 9}, {NULL}};
const Segment SGI270845[] = {{&S96893[13], 4}, {NULL}};
const Segment SGI270846[] = {{&S96893[26], 5}, {NULL}};
const Segment SGI270847[] = {{&S96893[59], 6}, {NULL}};
const Segment SGI270848[] = {{&S96893[111], 7}, {NULL}};
const Segment SGI270849[] = {{&S96893[126], 9}, {NULL}};
const Segment SGI270850[] = {{&S96894[13], 4}, {NULL}};
const Segment SGI270851[] = {{&S96894[26], 5}, {NULL}};
const Segment SGI270852[] = {{&S96894[59], 6}, {NULL}};
const Segment SGI270853[] = {{&S96894[111], 6}, {NULL}};
const Segment SGI270854[] = {{&S96894[125], 9}, {NULL}};
const Segment SGI270855[] = {{&S96895[13], 4}, {NULL}};
const Segment SGI270856[] = {{&S96895[26], 3}, {NULL}};
const Segment SGI270857[] = {{&S96895[57], 6}, {NULL}};
const Segment SGI270858[] = {{&S96895[109], 6}, {NULL}};
const Segment SGI270859[] = {{&S96895[123], 9}, {NULL}};
const Segment SGI270860[] = {{&S89405[89], 31}, {NULL}};
const Segment SGI270861[] = {{&S89405[0], 120}, {NULL}};
const Segment SGI270862[] = {{&S89429[92], 16}, {NULL}};
const Segment SGI270863[] = {{&S89429[0], 157}, {NULL}};
const Segment SGI270864[] = {{&S89433[35], 15}, {NULL}};
const Segment SGI270865[] = {{&S89433[0], 104}, {NULL}};
const Segment SGI270866[] = {{&S83991[10], 4}, {NULL}};
const Segment SGI270867[] = {{&S83991[15], 5}, {NULL}};
const Segment SGI270868[] = {{&S83991[65], 13}, {NULL}};
const Segment SGI270869[] = {{&S89725[68], 5}, {&S89725[68], 5}, {NULL}};
const Segment SGI270870[] = {{&S89725[0], 79}, {NULL}};
const Segment SGI270871[] = {{&S89726[61], 5}, {NULL}};
const Segment SGI270872[] = {{&S89726[0], 66}, {NULL}};
const Segment SGI270873[] = {{&S97292[27], 6}, {NULL}};
const Segment SGI270874[] = {{&S97292[51], 5}, {NULL}};
const Segment SGI270875[] = {{&S97292[84], 6}, {NULL}};
const Segment SGI270876[] = {{&S97292[118], 5}, {NULL}};
const Segment SGI270877[] = {{&S97292[131], 9}, {NULL}};
const Segment SGI270878[] = {{&S97292[0], 14}, {NULL}};
const Segment SGI270879[] = {{&S97293[27], 6}, {NULL}};
const Segment SGI270880[] = {{&S97293[52], 5}, {NULL}};
const Segment SGI270881[] = {{&S97293[85], 6}, {NULL}};
const Segment SGI270882[] = {{&S97293[119], 5}, {NULL}};
const Segment SGI270883[] = {{&S97293[132], 9}, {NULL}};
const Segment SGI270884[] = {{&S97293[0], 14}, {NULL}};
const Segment SGI270885[] = {{&S97327[17], 4}, {NULL}};
const Segment SGI270886[] = {{&S97327[33], 5}, {NULL}};
const Segment SGI270887[] = {{&S97327[59], 3}, {NULL}};
const Segment SGI270888[] = {{&S84055[10], 4}, {NULL}};
const Segment SGI270889[] = {{&S84055[15], 5}, {NULL}};
const Segment SGI270890[] = {{&S84055[17], 3}, {NULL}};
const Segment SGI270891[] = {{&S97332[13], 6}, {NULL}};
const Segment SGI270892[] = {{&S97332[115], 3}, {NULL}};
const Segment SGI270893[] = {{&S97332[38], 5}, {NULL}};
const Segment SGI270894[] = {{&S97332[78], 8}, {NULL}};
const Segment SGI270895[] = {{&S97332[126], 5}, {NULL}};
const Segment SGI270896[] = {{&S97332[139], 9}, {NULL}};
const Segment SGI270897[] = {{&S84224[20], 27}, {NULL}};
const Segment SGI270898[] = {{&S84225[20], 27}, {NULL}};
const Segment SGI270899[] = {{&S84226[20], 27}, {NULL}};
const Segment SGI270900[] = {{&S84227[20], 27}, {NULL}};
const Segment SGI270901[] = {{&S97566[28], 4}, {NULL}};
const Segment SGI270902[] = {{&S97566[41], 5}, {NULL}};
const Segment SGI270903[] = {{&S97566[74], 6}, {NULL}};
const Segment SGI270904[] = {{&S97566[108], 6}, {NULL}};
const Segment SGI270905[] = {{&S97566[122], 9}, {NULL}};
const Segment SGI270906[] = {{&S97566[0], 14}, {NULL}};
const Segment SGI270907[] = {{&S89882[61], 5}, {NULL}};
const Segment SGI270908[] = {{&S89882[0], 66}, {NULL}};
const Segment SGI270909[] = {{&S89883[61], 5}, {&S89883[61], 5}, {NULL}};
const Segment SGI270910[] = {{&S89883[0], 72}, {NULL}};
const Segment SGI270911[] = {{&S97615[13], 4}, {NULL}};
const Segment SGI270912[] = {{&S97615[33], 3}, {NULL}};
const Segment SGI270913[] = {{&S97615[64], 6}, {NULL}};
const Segment SGI270914[] = {{&S97615[117], 6}, {NULL}};
const Segment SGI270915[] = {{&S97615[131], 7}, {NULL}};
const Segment SGI270916[] = {{&S97722[17], 6}, {NULL}};
const Segment SGI270917[] = {{&S97722[35], 5}, {NULL}};
const Segment SGI270918[] = {{&S97722[61], 3}, {NULL}};
const Segment SGI270919[] = {{&S90047[59], 5}, {NULL}};
const Segment SGI270920[] = {{&S90047[0], 64}, {NULL}};
const Segment SGI270921[] = {{&S90048[59], 5}, {&S90048[59], 5}, {NULL}};
const Segment SGI270922[] = {{&S90048[0], 70}, {NULL}};
const Segment SGI270923[] = {{&S97776[13], 4}, {&S97776[13], 4}, {NULL}};
const Segment SGI270924[] = {{&S97776[36], 5}, {NULL}};
const Segment SGI270925[] = {{&S97776[76], 8}, {NULL}};
const Segment SGI270926[] = {{&S97776[112], 5}, {NULL}};
const Segment SGI270927[] = {{&S97776[125], 9}, {NULL}};
const Segment SGI270928[] = {{&S97813[28], 4}, {NULL}};
const Segment SGI270929[] = {{&S97813[48], 5}, {NULL}};
const Segment SGI270930[] = {{&S97813[81], 6}, {NULL}};
const Segment SGI270931[] = {{&S97813[115], 6}, {NULL}};
const Segment SGI270932[] = {{&S97813[129], 9}, {NULL}};
const Segment SGI270933[] = {{&S97813[0], 15}, {NULL}};
const Segment SGI270934[] = {{&S90132[0], 14}, {NULL}};
const Segment SGI270935[] = {{&S90132[0], 46}, {NULL}};
const Segment SGI270936[] = {{&S90133[0], 14}, {NULL}};
const Segment SGI270937[] = {{&S90133[0], 46}, {NULL}};
const Segment SGI270938[] = {{&S97855[13], 6}, {&S97855[13], 6}, {NULL}};
const Segment SGI270939[] = {{&S97855[38], 5}, {NULL}};
const Segment SGI270940[] = {{&S97855[78], 8}, {NULL}};
const Segment SGI270941[] = {{&S97855[114], 3}, {NULL}};
const Segment SGI270942[] = {{&S97855[125], 9}, {NULL}};
const Segment SGI270943[] = {{&S90202[61], 5}, {NULL}};
const Segment SGI270944[] = {{&S90202[0], 66}, {NULL}};
const Segment SGI270945[] = {{&S90203[61], 5}, {&S90203[61], 5}, {NULL}};
const Segment SGI270946[] = {{&S90203[0], 72}, {NULL}};
const Segment SGI270947[] = {{&S98026[13], 6}, {NULL}};
const Segment SGI270948[] = {{&S98026[35], 5}, {NULL}};
const Segment SGI270949[] = {{&S98026[75], 6}, {NULL}};
const Segment SGI270950[] = {{&S98026[109], 5}, {NULL}};
const Segment SGI270951[] = {{&S98026[115], 13}, {NULL}};
const Segment SGI270952[] = {{&S98161[13], 6}, {NULL}};
const Segment SGI270953[] = {{&S98161[138], 5}, {NULL}};
const Segment SGI270954[] = {{&S98161[35], 5}, {NULL}};
const Segment SGI270955[] = {{&S98161[68], 6}, {NULL}};
const Segment SGI270956[] = {{&S98161[35], 1}, {NULL}};
const Segment SGI270957[] = {{&S98166[27], 4}, {NULL}};
const Segment SGI270958[] = {{&S98166[47], 5}, {NULL}};
const Segment SGI270959[] = {{&S98166[80], 6}, {NULL}};
const Segment SGI270960[] = {{&S98166[114], 6}, {NULL}};
const Segment SGI270961[] = {{&S98166[128], 9}, {NULL}};
const Segment SGI270962[] = {{&S98166[0], 14}, {NULL}};
const Segment SGI270963[] = {{&S90559[51], 7}, {NULL}};
const Segment SGI270964[] = {{&S90559[0], 60}, {NULL}};
const Segment SGI270965[] = {{&S98207[13], 6}, {NULL}};
const Segment SGI270966[] = {{&S98207[113], 3}, {NULL}};
const Segment SGI270967[] = {{&S98207[38], 5}, {NULL}};
const Segment SGI270968[] = {{&S98207[78], 6}, {NULL}};
const Segment SGI270969[] = {{&S98207[124], 5}, {NULL}};
const Segment SGI270970[] = {{&S98207[137], 9}, {NULL}};
const Segment SGI270971[] = {{&S98274[17], 6}, {NULL}};
const Segment SGI270972[] = {{&S98274[35], 5}, {NULL}};
const Segment SGI270973[] = {{&S98274[61], 3}, {NULL}};
const Segment SGI270974[] = {{&S90769[30], 25}, {NULL}};
const Segment SGI270975[] = {{&S90769[0], 72}, {NULL}};
const Segment SGI270976[] = {{&S90793[61], 5}, {NULL}};
const Segment SGI270977[] = {{&S90793[0], 66}, {NULL}};
const Segment SGI270978[] = {{&S90794[68], 5}, {NULL}};
const Segment SGI270979[] = {{&S90794[0], 73}, {NULL}};
const Segment SGI270980[] = {{&S98503[13], 4}, {NULL}};
const Segment SGI270981[] = {{&S98503[26], 3}, {NULL}};
const Segment SGI270982[] = {{&S98503[57], 8}, {NULL}};
const Segment SGI270983[] = {{&S98503[108], 6}, {NULL}};
const Segment SGI270984[] = {{&S98503[122], 9}, {NULL}};
const Segment SGI270985[] = {{&S98503[86], 14}, {NULL}};
const Segment SGI270986[] = {{&S98552[13], 6}, {NULL}};
const Segment SGI270987[] = {{&S98552[110], 3}, {NULL}};
const Segment SGI270988[] = {{&S98552[38], 3}, {NULL}};
const Segment SGI270989[] = {{&S98552[76], 5}, {NULL}};
const Segment SGI270990[] = {{&S98552[121], 6}, {NULL}};
const Segment SGI270991[] = {{&S98552[135], 5}, {NULL}};
const Segment SGI270992[] = {{&S98568[13], 4}, {NULL}};
const Segment SGI270993[] = {{&S98568[26], 3}, {NULL}};
const Segment SGI270994[] = {{&S98568[114], 5}, {NULL}};
const Segment SGI270995[] = {{&S98568[57], 6}, {NULL}};
const Segment SGI270996[] = {{&S98568[91], 5}, {NULL}};
const Segment SGI270997[] = {{&S98569[13], 4}, {NULL}};
const Segment SGI270998[] = {{&S98569[26], 3}, {NULL}};
const Segment SGI270999[] = {{&S98569[123], 5}, {NULL}};
const Segment SGI271000[] = {{&S98569[57], 6}, {NULL}};
const Segment SGI271001[] = {{&S98569[91], 5}, {NULL}};
const Segment SGI271002[] = {{&S98569[104], 9}, {NULL}};
const Segment SGI271003[] = {{&S98570[13], 4}, {NULL}};
const Segment SGI271004[] = {{&S98570[26], 5}, {NULL}};
const Segment SGI271005[] = {{&S98570[125], 5}, {NULL}};
const Segment SGI271006[] = {{&S98570[59], 6}, {NULL}};
const Segment SGI271007[] = {{&S98570[93], 5}, {NULL}};
const Segment SGI271008[] = {{&S98570[106], 9}, {NULL}};
const Segment SGI271009[] = {{&S98629[13], 4}, {&S98629[13], 4}, {NULL}};
const Segment SGI271010[] = {{&S98629[26], 5}, {NULL}};
const Segment SGI271011[] = {{&S98629[59], 6}, {NULL}};
const Segment SGI271012[] = {{&S98629[93], 5}, {NULL}};
const Segment SGI271013[] = {{&S98629[106], 9}, {NULL}};
const Segment SGI271014[] = {{&S91391[66], 5}, {NULL}};
const Segment SGI271015[] = {{&S91391[0], 71}, {NULL}};
const Segment SGI271016[] = {{&S91415[93], 7}, {NULL}};
const Segment SGI271017[] = {{&S91415[0], 120}, {NULL}};
const Segment SGI271018[] = {{&S98966[13], 6}, {NULL}};
const Segment SGI271019[] = {{&S98966[136], 5}, {NULL}};
const Segment SGI271020[] = {{&S98966[35], 3}, {NULL}};
const Segment SGI271021[] = {{&S98966[66], 6}, {NULL}};
const Segment SGI271022[] = {{&S98966[35], 1}, {NULL}};
const Segment SGI271023[] = {{&S91490[179], 7}, {NULL}};
const Segment SGI271024[] = {{&S91490[0], 206}, {NULL}};
const Segment SGI271025[] = {{&S99112[27], 6}, {NULL}};
const Segment SGI271026[] = {{&S99112[56], 3}, {NULL}};
const Segment SGI271027[] = {{&S99112[94], 5}, {NULL}};
const Segment SGI271028[] = {{&S99112[127], 5}, {NULL}};
const Segment SGI271029[] = {{&S99112[140], 9}, {NULL}};
const Segment SGI271030[] = {{&S99112[0], 14}, {NULL}};
const Segment SGI271031[] = {{&S99121[13], 6}, {NULL}};
const Segment SGI271032[] = {{&S99121[138], 5}, {NULL}};
const Segment SGI271033[] = {{&S99121[35], 5}, {NULL}};
const Segment SGI271034[] = {{&S99121[68], 6}, {NULL}};
const Segment SGI271035[] = {{&S99121[35], 1}, {NULL}};
const Segment SGI271036[] = {{&S99239[13], 6}, {NULL}};
const Segment SGI271037[] = {{&S99239[35], 5}, {NULL}};
const Segment SGI271038[] = {{&S99239[68], 6}, {NULL}};
const Segment SGI271039[] = {{&S99239[102], 5}, {NULL}};
const Segment SGI271040[] = {{&S99239[115], 9}, {NULL}};
const Segment SGI271041[] = {{&S99287[27], 6}, {NULL}};
const Segment SGI271042[] = {{&S99287[49], 5}, {NULL}};
const Segment SGI271043[] = {{&S99287[82], 6}, {NULL}};
const Segment SGI271044[] = {{&S99287[116], 6}, {NULL}};
const Segment SGI271045[] = {{&S99287[130], 9}, {NULL}};
const Segment SGI271046[] = {{&S99287[0], 14}, {NULL}};
const Segment SGI271047[] = {{&S99306[13], 6}, {NULL}};
const Segment SGI271048[] = {{&S99306[35], 5}, {NULL}};
const Segment SGI271049[] = {{&S99306[68], 6}, {NULL}};
const Segment SGI271050[] = {{&S99306[126], 5}, {NULL}};
const Segment SGI271051[] = {{&S99306[139], 9}, {NULL}};
const Segment SGI271052[] = {{&S99309[27], 6}, {NULL}};
const Segment SGI271053[] = {{&S99309[49], 5}, {NULL}};
const Segment SGI271054[] = {{&S99309[82], 6}, {NULL}};
const Segment SGI271055[] = {{&S99309[116], 6}, {NULL}};
const Segment SGI271056[] = {{&S99309[130], 9}, {NULL}};
const Segment SGI271057[] = {{&S99309[0], 14}, {NULL}};
const Segment SGI271058[] = {{&S91908[55], 7}, {&S91908[67], 45}, {&S91908[141], 6}, {NULL}};
const Segment SGI271059[] = {{&S91908[0], 168}, {NULL}};
const Segment SGI271060[] = {{&S99421[13], 6}, {NULL}};
const Segment SGI271061[] = {{&S99421[35], 3}, {NULL}};
const Segment SGI271062[] = {{&S99421[66], 8}, {NULL}};
const Segment SGI271063[] = {{&S99421[126], 5}, {NULL}};
const Segment SGI271064[] = {{&S99421[139], 9}, {NULL}};
const Segment SGI271065[] = {{&S99465[17], 6}, {NULL}};
const Segment SGI271066[] = {{&S99465[38], 5}, {NULL}};
const Segment SGI271067[] = {{&S99465[68], 3}, {NULL}};
const Segment SGI271068[] = {{&S92001[0], 6}, {NULL}};
const Segment SGI271069[] = {{&S92001[0], 44}, {NULL}};
const Segment SGI271070[] = {{&S92002[0], 6}, {NULL}};
const Segment SGI271071[] = {{&S92002[0], 44}, {NULL}};
const Segment SGI271072[] = {{&S99601[27], 6}, {NULL}};
const Segment SGI271073[] = {{&S99601[52], 5}, {NULL}};
const Segment SGI271074[] = {{&S99601[92], 8}, {NULL}};
const Segment SGI271075[] = {{&S99601[128], 3}, {NULL}};
const Segment SGI271076[] = {{&S99601[139], 9}, {NULL}};
const Segment SGI271077[] = {{&S99601[0], 14}, {NULL}};
const Segment SGI271078[] = {{&S99602[27], 6}, {NULL}};
const Segment SGI271079[] = {{&S99602[52], 5}, {NULL}};
const Segment SGI271080[] = {{&S99602[92], 8}, {NULL}};
const Segment SGI271081[] = {{&S99602[128], 3}, {NULL}};
const Segment SGI271082[] = {{&S99602[139], 9}, {NULL}};
const Segment SGI271083[] = {{&S99602[0], 14}, {NULL}};
const Segment SGI271084[] = {{&S92106[116], 5}, {NULL}};
const Segment SGI271085[] = {{&S92106[0], 121}, {NULL}};
const Segment SGI271086[] = {{&S92107[84], 5}, {NULL}};
const Segment SGI271087[] = {{&S92107[0], 89}, {NULL}};
const Segment SGI271088[] = {{&S92108[84], 5}, {NULL}};
const Segment SGI271089[] = {{&S92108[0], 89}, {NULL}};
const Segment SGI271090[] = {{&S92235[66], 5}, {NULL}};
const Segment SGI271091[] = {{&S92235[0], 71}, {NULL}};
const Segment SGI271092[] = {{&S99771[28], 4}, {NULL}};
const Segment SGI271093[] = {{&S99771[41], 5}, {NULL}};
const Segment SGI271094[] = {{&S99771[74], 6}, {NULL}};
const Segment SGI271095[] = {{&S99771[108], 6}, {NULL}};
const Segment SGI271096[] = {{&S99771[122], 9}, {NULL}};
const Segment SGI271097[] = {{&S99771[0], 14}, {NULL}};
const Segment SGI271098[] = {{&S99803[27], 6}, {NULL}};
const Segment SGI271099[] = {{&S99803[54], 5}, {NULL}};
const Segment SGI271100[] = {{&S99803[94], 8}, {NULL}};
const Segment SGI271101[] = {{&S99803[130], 3}, {NULL}};
const Segment SGI271102[] = {{&S99803[141], 9}, {NULL}};
const Segment SGI271103[] = {{&S99803[0], 14}, {NULL}};
const Segment SGI271104[] = {{&S99966[27], 6}, {NULL}};
const Segment SGI271105[] = {{&S99966[49], 5}, {NULL}};
const Segment SGI271106[] = {{&S99966[82], 6}, {NULL}};
const Segment SGI271107[] = {{&S99966[116], 6}, {NULL}};
const Segment SGI271108[] = {{&S99966[130], 9}, {NULL}};
const Segment SGI271109[] = {{&S99966[0], 14}, {NULL}};
const Segment SGI271110[] = {{&S99982[27], 4}, {NULL}};
const Segment SGI271111[] = {{&S99982[47], 5}, {NULL}};
const Segment SGI271112[] = {{&S99982[80], 6}, {NULL}};
const Segment SGI271113[] = {{&S99982[114], 6}, {NULL}};
const Segment SGI271114[] = {{&S99982[128], 9}, {NULL}};
const Segment SGI271115[] = {{&S99982[0], 14}, {NULL}};
const Segment SGI271116[] = {{&S99988[21], 4}, {NULL}};
const Segment SGI271117[] = {{&S99988[102], 3}, {NULL}};
const Segment SGI271118[] = {{&S99988[34], 5}, {NULL}};
const Segment SGI271119[] = {{&S99988[67], 6}, {NULL}};
const Segment SGI271120[] = {{&S99988[113], 5}, {NULL}};
const Segment SGI271121[] = {{&S99988[126], 9}, {NULL}};
const Segment SGI271122[] = {{&S92490[45], 3}, {NULL}};
const Segment SGI271123[] = {{&S92490[0], 138}, {NULL}};
const Segment SGI271124[] = {{&S86310[10], 4}, {NULL}};
const Segment SGI271125[] = {{&S86310[15], 5}, {NULL}};
const Segment SGI271126[] = {{&S100097[13], 6}, {NULL}};
const Segment SGI271127[] = {{&S100097[35], 3}, {NULL}};
const Segment SGI271128[] = {{&S100097[73], 8}, {NULL}};
const Segment SGI271129[] = {{&S100097[109], 6}, {NULL}};
const Segment SGI271130[] = {{&S100097[116], 13}, {NULL}};
const Segment SGI271131[] = {{&S100179[28], 4}, {NULL}};
const Segment SGI271132[] = {{&S100179[44], 3}, {NULL}};
const Segment SGI271133[] = {{&S100180[28], 4}, {NULL}};
const Segment SGI271134[] = {{&S100180[44], 5}, {NULL}};
const Segment SGI271135[] = {{&S100187[27], 6}, {NULL}};
const Segment SGI271136[] = {{&S100187[52], 5}, {NULL}};
const Segment SGI271137[] = {{&S100187[92], 8}, {NULL}};
const Segment SGI271138[] = {{&S100187[128], 3}, {NULL}};
const Segment SGI271139[] = {{&S100187[139], 9}, {NULL}};
const Segment SGI271140[] = {{&S100187[0], 14}, {NULL}};
const Segment SGI271141[] = {{&S100188[27], 6}, {NULL}};
const Segment SGI271142[] = {{&S100188[52], 5}, {NULL}};
const Segment SGI271143[] = {{&S100188[92], 8}, {NULL}};
const Segment SGI271144[] = {{&S100188[128], 3}, {NULL}};
const Segment SGI271145[] = {{&S100188[139], 9}, {NULL}};
const Segment SGI271146[] = {{&S100188[0], 14}, {NULL}};
const Segment SGI271147[] = {{&S100248[28], 4}, {NULL}};
const Segment SGI271148[] = {{&S100248[41], 5}, {NULL}};
const Segment SGI271149[] = {{&S100248[74], 6}, {NULL}};
const Segment SGI271150[] = {{&S100248[108], 5}, {NULL}};
const Segment SGI271151[] = {{&S100248[121], 9}, {NULL}};
const Segment SGI271152[] = {{&S100248[0], 14}, {NULL}};
const Segment SGI271153[] = {{&S100297[13], 6}, {&S100297[13], 6}, {NULL}};
const Segment SGI271154[] = {{&S100297[38], 5}, {NULL}};
const Segment SGI271155[] = {{&S100297[78], 8}, {NULL}};
const Segment SGI271156[] = {{&S100297[114], 5}, {NULL}};
const Segment SGI271157[] = {{&S100297[127], 9}, {NULL}};
const Segment SGI271158[] = {{&S100379[17], 4}, {NULL}};
const Segment SGI271159[] = {{&S100379[36], 5}, {NULL}};
const Segment SGI271160[] = {{&S100379[66], 3}, {NULL}};
const Segment SGI271161[] = {{&S92631[109], 5}, {NULL}};
const Segment SGI271162[] = {{&S92631[0], 114}, {NULL}};
const Segment SGI271163[] = {{&S100481[13], 4}, {NULL}};
const Segment SGI271164[] = {{&S100481[116], 2}, {NULL}};
const Segment SGI271165[] = {{&S100481[26], 5}, {NULL}};
const Segment SGI271166[] = {{&S100481[59], 6}, {NULL}};
const Segment SGI271167[] = {{&S100481[26], 1}, {NULL}};
const Segment SGI271168[] = {{&S100481[143], 7}, {NULL}};
const Segment SGI271169[] = {{&S100517[17], 4}, {NULL}};
const Segment SGI271170[] = {{&S100517[36], 5}, {NULL}};
const Segment SGI271171[] = {{&S100517[66], 3}, {NULL}};
const Segment SGI271172[] = {{&S86891[10], 5}, {NULL}};
const Segment SGI271173[] = {{&S86891[16], 5}, {NULL}};
const Segment SGI271174[] = {{&S92901[68], 11}, {NULL}};
const Segment SGI271175[] = {{&S92901[0], 79}, {NULL}};
const Segment SGI271176[] = {{&S92937[109], 5}, {NULL}};
const Segment SGI271177[] = {{&S92937[0], 114}, {NULL}};
const Segment SGI271178[] = {{&S92953[74], 23}, {NULL}};
const Segment SGI271179[] = {{&S92953[0], 169}, {NULL}};
const Segment SGI271180[] = {{&S100716[13], 4}, {NULL}};
const Segment SGI271181[] = {{&S100716[26], 5}, {NULL}};
const Segment SGI271182[] = {{&S100716[128], 3}, {NULL}};
const Segment SGI271183[] = {{&S100716[59], 8}, {NULL}};
const Segment SGI271184[] = {{&S100716[95], 6}, {NULL}};
const Segment SGI271185[] = {{&S100716[109], 9}, {NULL}};
const Segment SGI271186[] = {{&S86961[20], 27}, {NULL}};
const Segment SGI271187[] = {{&S86962[20], 27}, {NULL}};
const Segment SGI271188[] = {{&S87030[10], 4}, {NULL}};
const Segment SGI271189[] = {{&S87030[15], 5}, {NULL}};
const Segment SGI271190[] = {{&S87031[10], 9}, {NULL}};
const Segment SGI271191[] = {{&S93106[59], 5}, {NULL}};
const Segment SGI271192[] = {{&S93106[0], 64}, {NULL}};
const Segment SGI271193[] = {{&S100891[27], 6}, {NULL}};
const Segment SGI271194[] = {{&S100891[49], 3}, {NULL}};
const Segment SGI271195[] = {{&S100891[80], 6}, {NULL}};
const Segment SGI271196[] = {{&S100891[114], 6}, {NULL}};
const Segment SGI271197[] = {{&S100891[128], 9}, {NULL}};
const Segment SGI271198[] = {{&S100891[0], 14}, {NULL}};
const Segment SGI271199[] = {{&S100906[17], 6}, {NULL}};
const Segment SGI271200[] = {{&S100906[38], 5}, {NULL}};
const Segment SGI271201[] = {{&S100906[64], 3}, {NULL}};
const Segment SGI271202[] = {{&S87190[10], 4}, {NULL}};
const Segment SGI271203[] = {{&S87190[15], 5}, {NULL}};
const Segment SGI271204[] = {{&S101055[13], 6}, {NULL}};
const Segment SGI271205[] = {{&S101055[113], 3}, {NULL}};
const Segment SGI271206[] = {{&S101055[38], 5}, {NULL}};
const Segment SGI271207[] = {{&S101055[78], 6}, {NULL}};
const Segment SGI271208[] = {{&S101055[124], 5}, {NULL}};
const Segment SGI271209[] = {{&S101055[137], 9}, {NULL}};
const Segment SGI271210[] = {{&S93322[57], 5}, {NULL}};
const Segment SGI271211[] = {{&S93322[0], 62}, {NULL}};
const Segment SGI271212[] = {{&S93338[51], 10}, {NULL}};
const Segment SGI271213[] = {{&S93338[0], 61}, {NULL}};
const Segment SGI271214[] = {{&S101127[13], 4}, {NULL}};
const Segment SGI271215[] = {{&S101127[33], 5}, {NULL}};
const Segment SGI271216[] = {{&S101127[133], 5}, {NULL}};
const Segment SGI271217[] = {{&S101127[66], 6}, {NULL}};
const Segment SGI271218[] = {{&S101127[100], 6}, {NULL}};
const Segment SGI271219[] = {{&S101127[114], 9}, {NULL}};
const Segment SGI271220[] = {{&S101200[28], 4}, {NULL}};
const Segment SGI271221[] = {{&S101200[41], 5}, {NULL}};
const Segment SGI271222[] = {{&S101200[74], 6}, {NULL}};
const Segment SGI271223[] = {{&S101200[108], 5}, {NULL}};
const Segment SGI271224[] = {{&S101200[121], 9}, {NULL}};
const Segment SGI271225[] = {{&S101200[0], 14}, {NULL}};
const Segment SGI271226[] = {{&S101210[27], 6}, {NULL}};
const Segment SGI271227[] = {{&S101210[52], 5}, {NULL}};
const Segment SGI271228[] = {{&S101210[92], 8}, {NULL}};
const Segment SGI271229[] = {{&S101210[128], 3}, {NULL}};
const Segment SGI271230[] = {{&S101210[139], 9}, {NULL}};
const Segment SGI271231[] = {{&S101210[0], 14}, {NULL}};
const Segment SGI271232[] = {{&S101211[27], 6}, {NULL}};
const Segment SGI271233[] = {{&S101211[52], 5}, {NULL}};
const Segment SGI271234[] = {{&S101211[92], 8}, {NULL}};
const Segment SGI271235[] = {{&S101211[128], 3}, {NULL}};
const Segment SGI271236[] = {{&S101211[139], 9}, {NULL}};
const Segment SGI271237[] = {{&S101211[0], 14}, {NULL}};
const Segment SGI271238[] = {{&S101220[30], 4}, {NULL}};
const Segment SGI271239[] = {{&S101220[46], 5}, {NULL}};
const Segment SGI271240[] = {{&S101220[86], 8}, {NULL}};
const Segment SGI271241[] = {{&S101220[122], 5}, {NULL}};
const Segment SGI271242[] = {{&S101220[135], 9}, {NULL}};
const Segment SGI271243[] = {{&S101220[0], 14}, {NULL}};
const Segment SGI271244[] = {{&S101225[27], 6}, {NULL}};
const Segment SGI271245[] = {{&S101225[52], 5}, {NULL}};
const Segment SGI271246[] = {{&S101225[92], 8}, {NULL}};
const Segment SGI271247[] = {{&S101225[128], 3}, {NULL}};
const Segment SGI271248[] = {{&S101225[139], 9}, {NULL}};
const Segment SGI271249[] = {{&S101225[0], 14}, {NULL}};
const Segment SGI271250[] = {{&S101274[17], 6}, {NULL}};
const Segment SGI271251[] = {{&S101274[35], 5}, {NULL}};
const Segment SGI271252[] = {{&S101274[61], 3}, {NULL}};
const Segment SGI271253[] = {{&S101347[13], 4}, {&S101347[26], 6}, {NULL}};
const Segment SGI271254[] = {{&S101347[210], 3}, {NULL}};
const Segment SGI271255[] = {{&S101347[36], 5}, {NULL}};
const Segment SGI271256[] = {{&S101347[76], 8}, {&S101347[177], 3}, {NULL}};
const Segment SGI271257[] = {{&S101347[112], 5}, {&S101347[221], 3}, {NULL}};
const Segment SGI271258[] = {{&S101347[232], 5}, {NULL}};
const Segment SGI271259[] = {{&S101352[28], 6}, {NULL}};
const Segment SGI271260[] = {{&S101352[50], 5}, {NULL}};
const Segment SGI271261[] = {{&S101352[83], 6}, {NULL}};
const Segment SGI271262[] = {{&S101352[117], 5}, {NULL}};
const Segment SGI271263[] = {{&S101352[0], 15}, {NULL}};
const Segment SGI271264[] = {{&S101380[27], 6}, {NULL}};
const Segment SGI271265[] = {{&S101380[49], 5}, {NULL}};
const Segment SGI271266[] = {{&S101380[82], 6}, {NULL}};
const Segment SGI271267[] = {{&S101380[116], 5}, {NULL}};
const Segment SGI271268[] = {{&S101380[129], 9}, {NULL}};
const Segment SGI271269[] = {{&S101380[0], 14}, {NULL}};
const Segment SGI271270[] = {{&S101419[17], 4}, {NULL}};
const Segment SGI271271[] = {{&S101419[39], 3}, {NULL}};
const Segment SGI271272[] = {{&S101419[63], 3}, {NULL}};
const Segment SGI271273[] = {{&S93550[28], 23}, {NULL}};
const Segment SGI271274[] = {{&S93550[0], 70}, {NULL}};
const Segment SGI271275[] = {{&S93551[28], 23}, {NULL}};
const Segment SGI271276[] = {{&S93551[0], 70}, {NULL}};
const Segment SGI271277[] = {{&S101551[13], 6}, {NULL}};
const Segment SGI271278[] = {{&S101551[110], 3}, {NULL}};
const Segment SGI271279[] = {{&S101551[38], 3}, {NULL}};
const Segment SGI271280[] = {{&S101551[76], 5}, {NULL}};
const Segment SGI271281[] = {{&S101551[121], 5}, {NULL}};
const Segment SGI271282[] = {{&S101551[134], 9}, {NULL}};
const Segment SGI271283[] = {{&S101595[27], 6}, {NULL}};
const Segment SGI271284[] = {{&S101595[52], 3}, {NULL}};
const Segment SGI271285[] = {{&S101595[83], 6}, {NULL}};
const Segment SGI271286[] = {{&S101595[117], 6}, {NULL}};
const Segment SGI271287[] = {{&S101595[131], 9}, {NULL}};
const Segment SGI271288[] = {{&S101595[0], 14}, {NULL}};
const Segment SGI271289[] = {{&S93788[59], 5}, {NULL}};
const Segment SGI271290[] = {{&S93788[0], 64}, {NULL}};
const Segment SGI271291[] = {{&S93789[66], 5}, {NULL}};
const Segment SGI271292[] = {{&S93789[0], 71}, {NULL}};
const Segment SGI271293[] = {{&S101785[17], 6}, {NULL}};
const Segment SGI271294[] = {{&S101785[35], 5}, {NULL}};
const Segment SGI271295[] = {{&S101785[61], 3}, {NULL}};
const Segment SGI271296[] = {{&S101788[13], 4}, {NULL}};
const Segment SGI271297[] = {{&S101788[92], 3}, {NULL}};
const Segment SGI271298[] = {{&S101788[26], 3}, {NULL}};
const Segment SGI271299[] = {{&S101788[57], 6}, {NULL}};
const Segment SGI271300[] = {{&S101788[103], 7}, {NULL}};
const Segment SGI271301[] = {{&S101788[118], 7}, {NULL}};
const Segment SGI271302[] = {{&S101872[13], 4}, {&S101872[13], 4}, {NULL}};
const Segment SGI271303[] = {{&S101872[33], 5}, {NULL}};
const Segment SGI271304[] = {{&S101872[66], 6}, {NULL}};
const Segment SGI271305[] = {{&S101872[100], 5}, {NULL}};
const Segment SGI271306[] = {{&S101872[113], 9}, {NULL}};
const Segment SGI271307[] = {{&S101884[13], 6}, {NULL}};
const Segment SGI271308[] = {{&S101884[111], 3}, {NULL}};
const Segment SGI271309[] = {{&S101884[38], 3}, {NULL}};
const Segment SGI271310[] = {{&S101884[76], 6}, {NULL}};
const Segment SGI271311[] = {{&S101884[122], 6}, {NULL}};
const Segment SGI271312[] = {{&S101884[136], 9}, {NULL}};
const Segment SGI271313[] = {{&S101885[17], 4}, {NULL}};
const Segment SGI271314[] = {{&S101885[33], 5}, {NULL}};
const Segment SGI271315[] = {{&S101885[59], 3}, {NULL}};
const Segment SGI271316[] = {{&S87986[20], 27}, {NULL}};
const Segment SGI271317[] = {{&S87987[20], 27}, {NULL}};
const Segment SGI271318[] = {{&S87988[20], 27}, {NULL}};
const Segment SGI271319[] = {{&S88044[10], 5}, {NULL}};
const Segment SGI271320[] = {{&S88044[16], 5}, {NULL}};
const Segment SGI271321[] = {{&S102043[13], 6}, {NULL}};
const Segment SGI271322[] = {{&S102043[35], 3}, {NULL}};
const Segment SGI271323[] = {{&S102043[132], 5}, {NULL}};
const Segment SGI271324[] = {{&S102043[66], 6}, {NULL}};
const Segment SGI271325[] = {{&S102043[100], 5}, {NULL}};
const Segment SGI271326[] = {{&S102043[113], 9}, {NULL}};
const Segment SGI271327[] = {{&S102044[13], 6}, {NULL}};
const Segment SGI271328[] = {{&S102044[35], 5}, {NULL}};
const Segment SGI271329[] = {{&S102044[134], 5}, {NULL}};
const Segment SGI271330[] = {{&S102044[68], 6}, {NULL}};
const Segment SGI271331[] = {{&S102044[102], 5}, {NULL}};
const Segment SGI271332[] = {{&S102044[115], 9}, {NULL}};
const Segment SGI271333[] = {{&S102045[12], 6}, {NULL}};
const Segment SGI271334[] = {{&S102045[76], 5}, {NULL}};
const Segment SGI271335[] = {{&S102055[28], 4}, {NULL}};
const Segment SGI271336[] = {{&S102055[44], 5}, {NULL}};
const Segment SGI271337[] = {{&S102055[84], 8}, {NULL}};
const Segment SGI271338[] = {{&S102055[120], 5}, {NULL}};
const Segment SGI271339[] = {{&S102055[133], 9}, {NULL}};
const Segment SGI271340[] = {{&S102055[0], 14}, {NULL}};
const Segment SGI271341[] = {{&S102082[13], 6}, {NULL}};
const Segment SGI271342[] = {{&S102082[35], 5}, {NULL}};
const Segment SGI271343[] = {{&S102082[134], 3}, {NULL}};
const Segment SGI271344[] = {{&S102082[68], 6}, {NULL}};
const Segment SGI271345[] = {{&S102082[102], 5}, {NULL}};
const Segment SGI271346[] = {{&S102082[115], 9}, {NULL}};
const Segment SGI271347[] = {{&S94277[75], 5}, {NULL}};
const Segment SGI271348[] = {{&S94277[0], 80}, {NULL}};
const Segment SGI271349[] = {{&S94369[0], 13}, {&S94369[14], 36}, {NULL}};
const Segment SGI271350[] = {{&S94369[0], 167}, {NULL}};
const Segment SGI271351[] = {{&S102230[13], 6}, {NULL}};
const Segment SGI271352[] = {{&S102230[38], 5}, {NULL}};
const Segment SGI271353[] = {{&S102243[12], 6}, {NULL}};
const Segment SGI271354[] = {{&S102243[76], 5}, {NULL}};
const Segment SGI271355[] = {{&S102424[13], 4}, {NULL}};
const Segment SGI271356[] = {{&S102424[26], 5}, {NULL}};
const Segment SGI271357[] = {{&S102424[116], 5}, {NULL}};
const Segment SGI271358[] = {{&S102424[59], 6}, {NULL}};
const Segment SGI271359[] = {{&S102424[93], 5}, {NULL}};
const Segment SGI271360[] = {{&S102425[13], 4}, {NULL}};
const Segment SGI271361[] = {{&S102425[26], 5}, {NULL}};
const Segment SGI271362[] = {{&S102425[116], 3}, {NULL}};
const Segment SGI271363[] = {{&S102425[59], 6}, {NULL}};
const Segment SGI271364[] = {{&S102425[93], 5}, {NULL}};
const Segment SGI271365[] = {{&S102426[13], 4}, {NULL}};
const Segment SGI271366[] = {{&S102426[26], 5}, {NULL}};
const Segment SGI271367[] = {{&S102426[116], 5}, {NULL}};
const Segment SGI271368[] = {{&S102426[59], 6}, {NULL}};
const Segment SGI271369[] = {{&S102426[93], 5}, {NULL}};
const Segment SGI271370[] = {{&S102453[13], 6}, {NULL}};
const Segment SGI271371[] = {{&S102453[38], 5}, {NULL}};
const Segment SGI271372[] = {{&S102454[13], 6}, {NULL}};
const Segment SGI271373[] = {{&S102454[38], 3}, {NULL}};
const Segment SGI271374[] = {{&S102487[13], 6}, {NULL}};
const Segment SGI271375[] = {{&S102487[35], 3}, {NULL}};
const Segment SGI271376[] = {{&S102487[133], 3}, {NULL}};
const Segment SGI271377[] = {{&S102487[66], 6}, {NULL}};
const Segment SGI271378[] = {{&S102487[100], 6}, {NULL}};
const Segment SGI271379[] = {{&S102487[114], 9}, {NULL}};
const Segment SGI271380[] = {{&S88485[20], 26}, {NULL}};
const Segment SGI271381[] = {{&S88486[20], 26}, {NULL}};
const Segment SGI271382[] = {{&S94519[75], 5}, {NULL}};
const Segment SGI271383[] = {{&S94519[0], 80}, {NULL}};
const Segment SGI271384[] = {{&S94598[68], 13}, {&S94598[82], 24}, {NULL}};
const Segment SGI271385[] = {{&S94598[0], 158}, {NULL}};
const Segment SGI271386[] = {{&S102630[27], 6}, {NULL}};
const Segment SGI271387[] = {{&S102630[49], 5}, {NULL}};
const Segment SGI271388[] = {{&S102630[82], 6}, {NULL}};
const Segment SGI271389[] = {{&S102630[116], 5}, {NULL}};
const Segment SGI271390[] = {{&S102630[0], 14}, {NULL}};
const Segment SGI271391[] = {{&S102679[28], 6}, {NULL}};
const Segment SGI271392[] = {{&S102679[53], 5}, {NULL}};
const Segment SGI271393[] = {{&S102679[93], 8}, {NULL}};
const Segment SGI271394[] = {{&S102679[129], 3}, {NULL}};
const Segment SGI271395[] = {{&S102679[0], 15}, {NULL}};
const Segment SGI271396[] = {{&S102680[28], 6}, {NULL}};
const Segment SGI271397[] = {{&S102680[53], 5}, {NULL}};
const Segment SGI271398[] = {{&S102680[93], 8}, {NULL}};
const Segment SGI271399[] = {{&S102680[129], 3}, {NULL}};
const Segment SGI271400[] = {{&S102680[0], 15}, {NULL}};
const Segment SGI271401[] = {{&S102682[13], 6}, {NULL}};
const Segment SGI271402[] = {{&S102682[35], 5}, {NULL}};
const Segment SGI271403[] = {{&S102682[134], 5}, {NULL}};
const Segment SGI271404[] = {{&S102682[68], 6}, {NULL}};
const Segment SGI271405[] = {{&S102682[102], 5}, {NULL}};
const Segment SGI271406[] = {{&S102682[115], 9}, {NULL}};
const Segment SGI271407[] = {{&S102830[28], 4}, {NULL}};
const Segment SGI271408[] = {{&S102830[44], 5}, {NULL}};
const Segment SGI271409[] = {{&S102830[84], 8}, {NULL}};
const Segment SGI271410[] = {{&S102830[120], 3}, {NULL}};
const Segment SGI271411[] = {{&S102830[131], 9}, {NULL}};
const Segment SGI271412[] = {{&S102830[0], 14}, {NULL}};
const Segment SGI271413[] = {{&S102831[28], 4}, {NULL}};
const Segment SGI271414[] = {{&S102831[44], 5}, {NULL}};
const Segment SGI271415[] = {{&S102831[84], 8}, {NULL}};
const Segment SGI271416[] = {{&S102831[120], 3}, {NULL}};
const Segment SGI271417[] = {{&S102831[131], 9}, {NULL}};
const Segment SGI271418[] = {{&S102831[0], 14}, {NULL}};
const Segment SGI271419[] = {{&S102844[13], 6}, {NULL}};
const Segment SGI271420[] = {{&S102844[38], 5}, {NULL}};
const Segment SGI271421[] = {{&S102845[13], 6}, {NULL}};
const Segment SGI271422[] = {{&S102845[38], 3}, {NULL}};
const Segment SGI271423[] = {{&S102856[13], 6}, {NULL}};
const Segment SGI271424[] = {{&S102856[38], 5}, {NULL}};
const Segment SGI271425[] = {{&S102861[13], 4}, {&S102861[23], 6}, {NULL}};
const Segment SGI271426[] = {{&S102861[198], 3}, {NULL}};
const Segment SGI271427[] = {{&S102861[33], 5}, {NULL}};
const Segment SGI271428[] = {{&S102861[66], 6}, {&S102861[165], 3}, {NULL}};
const Segment SGI271429[] = {{&S102861[100], 5}, {&S102861[209], 3}, {NULL}};
const Segment SGI271430[] = {{&S102861[220], 5}, {NULL}};
const Segment SGI271431[] = {{&S102880[27], 4}, {NULL}};
const Segment SGI271432[] = {{&S102880[47], 5}, {NULL}};
const Segment SGI271433[] = {{&S102880[80], 6}, {NULL}};
const Segment SGI271434[] = {{&S102880[114], 5}, {NULL}};
const Segment SGI271435[] = {{&S102880[127], 9}, {NULL}};
const Segment SGI271436[] = {{&S102880[0], 14}, {NULL}};
const Segment SGI271437[] = {{&S102884[13], 6}, {&S102884[13], 6}, {NULL}};
const Segment SGI271438[] = {{&S102884[207], 3}, {NULL}};
const Segment SGI271439[] = {{&S102884[38], 3}, {NULL}};
const Segment SGI271440[] = {{&S102884[76], 5}, {&S102884[174], 3}, {NULL}};
const Segment SGI271441[] = {{&S102884[109], 5}, {&S102884[218], 3}, {NULL}};
const Segment SGI271442[] = {{&S102884[229], 5}, {NULL}};
const Segment SGI271443[] = {{&S102931[13], 6}, {NULL}};
const Segment SGI271444[] = {{&S102931[111], 3}, {NULL}};
const Segment SGI271445[] = {{&S102931[38], 3}, {NULL}};
const Segment SGI271446[] = {{&S102931[76], 6}, {NULL}};
const Segment SGI271447[] = {{&S102931[122], 5}, {NULL}};
const Segment SGI271448[] = {{&S102931[135], 9}, {NULL}};
const Segment SGI271449[] = {{&S102944[13], 6}, {NULL}};
const Segment SGI271450[] = {{&S102944[103], 3}, {NULL}};
const Segment SGI271451[] = {{&S102944[35], 5}, {NULL}};
const Segment SGI271452[] = {{&S102944[68], 6}, {NULL}};
const Segment SGI271453[] = {{&S102944[114], 6}, {NULL}};
const Segment SGI271454[] = {{&S102944[128], 7}, {NULL}};
const Segment SGI271455[] = {{&S102945[13], 6}, {NULL}};
const Segment SGI271456[] = {{&S102945[103], 3}, {NULL}};
const Segment SGI271457[] = {{&S102945[35], 5}, {NULL}};
const Segment SGI271458[] = {{&S102945[68], 6}, {NULL}};
const Segment SGI271459[] = {{&S102945[114], 6}, {NULL}};
const Segment SGI271460[] = {{&S102945[128], 7}, {NULL}};
const Segment SGI271461[] = {{&S102946[13], 6}, {NULL}};
const Segment SGI271462[] = {{&S102946[101], 3}, {NULL}};
const Segment SGI271463[] = {{&S102946[35], 3}, {NULL}};
const Segment SGI271464[] = {{&S102946[66], 6}, {NULL}};
const Segment SGI271465[] = {{&S102946[112], 4}, {NULL}};
const Segment SGI271466[] = {{&S102946[124], 7}, {NULL}};
const Segment SGI271467[] = {{&S102947[13], 6}, {NULL}};
const Segment SGI271468[] = {{&S102947[101], 3}, {NULL}};
const Segment SGI271469[] = {{&S102947[35], 3}, {NULL}};
const Segment SGI271470[] = {{&S102947[66], 6}, {NULL}};
const Segment SGI271471[] = {{&S102947[112], 6}, {NULL}};
const Segment SGI271472[] = {{&S102947[126], 7}, {NULL}};
const Segment SGI271473[] = {{&S88953[10], 4}, {NULL}};
const Segment SGI271474[] = {{&S88953[15], 5}, {NULL}};
const Segment SGI271475[] = {{&S88953[17], 3}, {NULL}};
const Segment SGI271476[] = {{&S95040[110], 5}, {NULL}};
const Segment SGI271477[] = {{&S95040[0], 115}, {NULL}};
const Segment SGI271478[] = {{&S103073[13], 6}, {NULL}};
const Segment SGI271479[] = {{&S103073[38], 5}, {NULL}};
const Segment SGI271480[] = {{&S103074[13], 6}, {NULL}};
const Segment SGI271481[] = {{&S103074[38], 3}, {NULL}};
const Segment SGI271482[] = {{&S103075[13], 6}, {NULL}};
const Segment SGI271483[] = {{&S103075[38], 5}, {NULL}};
const Segment SGI271484[] = {{&S103076[13], 6}, {NULL}};
const Segment SGI271485[] = {{&S103076[38], 5}, {NULL}};
const Segment SGI271486[] = {{&S103083[28], 4}, {NULL}};
const Segment SGI271487[] = {{&S103083[44], 5}, {NULL}};
const Segment SGI271488[] = {{&S103083[84], 8}, {NULL}};
const Segment SGI271489[] = {{&S103083[120], 5}, {NULL}};
const Segment SGI271490[] = {{&S103083[133], 9}, {NULL}};
const Segment SGI271491[] = {{&S103083[0], 14}, {NULL}};
const Segment SGI271492[] = {{&S95071[75], 5}, {NULL}};
const Segment SGI271493[] = {{&S95071[0], 80}, {NULL}};
const Segment SGI271494[] = {{&S103120[13], 4}, {NULL}};
const Segment SGI271495[] = {{&S103120[26], 3}, {NULL}};
const Segment SGI271496[] = {{&S103120[124], 3}, {NULL}};
const Segment SGI271497[] = {{&S103120[57], 6}, {NULL}};
const Segment SGI271498[] = {{&S103120[91], 6}, {NULL}};
const Segment SGI271499[] = {{&S103120[105], 9}, {NULL}};
const Segment SGI271500[] = {{&S103126[27], 6}, {NULL}};
const Segment SGI271501[] = {{&S103126[52], 5}, {NULL}};
const Segment SGI271502[] = {{&S103126[92], 8}, {NULL}};
const Segment SGI271503[] = {{&S103126[128], 5}, {NULL}};
const Segment SGI271504[] = {{&S103126[141], 9}, {NULL}};
const Segment SGI271505[] = {{&S103126[0], 14}, {NULL}};
const Segment SGI271506[] = {{&S89106[10], 5}, {NULL}};
const Segment SGI271507[] = {{&S89106[16], 5}, {NULL}};
const Segment SGI271508[] = {{&S95257[102], 5}, {NULL}};
const Segment SGI271509[] = {{&S95257[0], 107}, {NULL}};
const Segment SGI271510[] = {{&S103343[12], 4}, {NULL}};
const Segment SGI271511[] = {{&S103343[78], 5}, {NULL}};
const Segment SGI271512[] = {{&S103352[13], 6}, {NULL}};
const Segment SGI271513[] = {{&S103352[115], 5}, {NULL}};
const Segment SGI271514[] = {{&S103352[38], 5}, {NULL}};
const Segment SGI271515[] = {{&S103352[78], 8}, {NULL}};
const Segment SGI271516[] = {{&S103352[128], 5}, {NULL}};
const Segment SGI271517[] = {{&S103352[141], 9}, {NULL}};
const Segment SGI271518[] = {{&S103368[26], 6}, {&S103368[26], 6}, {NULL}};
const Segment SGI271519[] = {{&S103368[25], 1}, {NULL}};
const Segment SGI271520[] = {{&S103369[26], 6}, {&S103369[36], 4}, {NULL}};
const Segment SGI271521[] = {{&S103369[25], 1}, {NULL}};
const Segment SGI271522[] = {{&S89385[10], 4}, {NULL}};
const Segment SGI271523[] = {{&S89385[15], 5}, {NULL}};
const Segment SGI271524[] = {{&S89385[85], 3}, {NULL}};
const Segment SGI271525[] = {{&S103405[27], 6}, {NULL}};
const Segment SGI271526[] = {{&S103405[52], 5}, {NULL}};
const Segment SGI271527[] = {{&S103405[92], 8}, {NULL}};
const Segment SGI271528[] = {{&S103405[128], 3}, {NULL}};
const Segment SGI271529[] = {{&S103405[0], 14}, {NULL}};
const Segment SGI271530[] = {{&S103406[17], 4}, {NULL}};
const Segment SGI271531[] = {{&S103406[39], 5}, {NULL}};
const Segment SGI271532[] = {{&S95412[61], 5}, {NULL}};
const Segment SGI271533[] = {{&S95412[0], 66}, {NULL}};
const Segment SGI271534[] = {{&S103509[13], 4}, {NULL}};
const Segment SGI271535[] = {{&S103509[26], 5}, {NULL}};
const Segment SGI271536[] = {{&S103509[59], 6}, {NULL}};
const Segment SGI271537[] = {{&S103509[111], 5}, {NULL}};
const Segment SGI271538[] = {{&S103509[124], 9}, {NULL}};
const Segment SGI271539[] = {{&S95603[25], 14}, {NULL}};
const Segment SGI271540[] = {{&S95603[0], 118}, {NULL}};
const Segment SGI271541[] = {{&S103636[13], 4}, {NULL}};
const Segment SGI271542[] = {{&S103636[36], 3}, {NULL}};
const Segment SGI271543[] = {{&S103636[74], 8}, {NULL}};
const Segment SGI271544[] = {{&S103677[13], 6}, {NULL}};
const Segment SGI271545[] = {{&S103677[38], 3}, {NULL}};
const Segment SGI271546[] = {{&S103679[17], 4}, {NULL}};
const Segment SGI271547[] = {{&S103679[39], 5}, {NULL}};
const Segment SGI271548[] = {{&S103685[13], 4}, {NULL}};
const Segment SGI271549[] = {{&S103685[26], 5}, {NULL}};
const Segment SGI271550[] = {{&S103685[125], 5}, {NULL}};
const Segment SGI271551[] = {{&S103685[59], 6}, {NULL}};
const Segment SGI271552[] = {{&S103685[93], 5}, {NULL}};
const Segment SGI271553[] = {{&S103685[106], 9}, {NULL}};
const Segment SGI271554[] = {{&S103687[28], 4}, {NULL}};
const Segment SGI271555[] = {{&S103687[44], 5}, {NULL}};
const Segment SGI271556[] = {{&S103687[84], 8}, {NULL}};
const Segment SGI271557[] = {{&S103687[120], 5}, {NULL}};
const Segment SGI271558[] = {{&S103687[133], 9}, {NULL}};
const Segment SGI271559[] = {{&S103687[0], 14}, {NULL}};
const Segment SGI271560[] = {{&S103697[13], 4}, {NULL}};
const Segment SGI271561[] = {{&S103697[29], 5}, {NULL}};
const Segment SGI271562[] = {{&S103697[69], 6}, {NULL}};
const Segment SGI271563[] = {{&S95792[75], 5}, {NULL}};
const Segment SGI271564[] = {{&S95792[0], 80}, {NULL}};
const Segment SGI271565[] = {{&S103825[13], 6}, {NULL}};
const Segment SGI271566[] = {{&S103825[35], 5}, {NULL}};
const Segment SGI271567[] = {{&S103825[68], 6}, {NULL}};
const Segment SGI271568[] = {{&S103825[102], 6}, {NULL}};
const Segment SGI271569[] = {{&S103826[13], 6}, {NULL}};
const Segment SGI271570[] = {{&S103826[35], 5}, {NULL}};
const Segment SGI271571[] = {{&S103826[68], 6}, {NULL}};
const Segment SGI271572[] = {{&S103826[102], 6}, {NULL}};
const Segment SGI271573[] = {{&S103827[13], 6}, {NULL}};
const Segment SGI271574[] = {{&S103827[35], 3}, {NULL}};
const Segment SGI271575[] = {{&S103827[66], 8}, {NULL}};
const Segment SGI271576[] = {{&S103827[102], 5}, {NULL}};
const Segment SGI271577[] = {{&S103963[16], 4}, {NULL}};
const Segment SGI271578[] = {{&S103963[32], 3}, {NULL}};
const Segment SGI271579[] = {{&S103963[11], 3}, {NULL}};
const Segment SGI271580[] = {{&S103964[16], 4}, {NULL}};
const Segment SGI271581[] = {{&S103964[32], 5}, {NULL}};
const Segment SGI271582[] = {{&S103964[11], 3}, {NULL}};
const Segment SGI271583[] = {{&S103973[13], 4}, {NULL}};
const Segment SGI271584[] = {{&S103973[42], 5}, {NULL}};
const Segment SGI271585[] = {{&S103974[13], 4}, {NULL}};
const Segment SGI271586[] = {{&S103974[42], 3}, {NULL}};
const Segment SGI271587[] = {{&S103980[17], 6}, {NULL}};
const Segment SGI271588[] = {{&S103980[35], 3}, {NULL}};
const Segment SGI271589[] = {{&S96007[66], 16}, {NULL}};
const Segment SGI271590[] = {{&S96007[0], 196}, {NULL}};
const Segment SGI271591[] = {{&S96031[48], 11}, {NULL}};
const Segment SGI271592[] = {{&S96031[0], 59}, {NULL}};
const Segment SGI271593[] = {{&S104013[27], 4}, {NULL}};
const Segment SGI271594[] = {{&S104013[47], 5}, {NULL}};
const Segment SGI271595[] = {{&S104013[80], 6}, {NULL}};
const Segment SGI271596[] = {{&S104013[114], 5}, {NULL}};
const Segment SGI271597[] = {{&S104013[127], 9}, {NULL}};
const Segment SGI271598[] = {{&S104013[0], 14}, {NULL}};
const Segment SGI271599[] = {{&S90173[10], 4}, {NULL}};
const Segment SGI271600[] = {{&S90173[15], 5}, {NULL}};
const Segment SGI271601[] = {{&S104142[13], 4}, {NULL}};
const Segment SGI271602[] = {{&S104142[92], 3}, {NULL}};
const Segment SGI271603[] = {{&S104142[26], 3}, {NULL}};
const Segment SGI271604[] = {{&S104142[57], 6}, {NULL}};
const Segment SGI271605[] = {{&S104142[103], 4}, {NULL}};
const Segment SGI271606[] = {{&S104142[115], 7}, {NULL}};
const Segment SGI271607[] = {{&S104143[13], 4}, {NULL}};
const Segment SGI271608[] = {{&S104143[92], 3}, {NULL}};
const Segment SGI271609[] = {{&S104143[26], 3}, {NULL}};
const Segment SGI271610[] = {{&S104143[57], 6}, {NULL}};
const Segment SGI271611[] = {{&S104143[103], 5}, {NULL}};
const Segment SGI271612[] = {{&S104143[116], 7}, {NULL}};
const Segment SGI271613[] = {{&S104144[13], 4}, {NULL}};
const Segment SGI271614[] = {{&S104144[94], 3}, {NULL}};
const Segment SGI271615[] = {{&S104144[26], 3}, {NULL}};
const Segment SGI271616[] = {{&S104144[57], 8}, {NULL}};
const Segment SGI271617[] = {{&S104144[105], 5}, {NULL}};
const Segment SGI271618[] = {{&S104144[118], 7}, {NULL}};
const Segment SGI271619[] = {{&S104145[13], 4}, {NULL}};
const Segment SGI271620[] = {{&S104145[92], 3}, {NULL}};
const Segment SGI271621[] = {{&S104145[26], 3}, {NULL}};
const Segment SGI271622[] = {{&S104145[57], 6}, {NULL}};
const Segment SGI271623[] = {{&S104145[103], 5}, {NULL}};
const Segment SGI271624[] = {{&S104145[116], 7}, {NULL}};
const Segment SGI271625[] = {{&S104146[13], 6}, {NULL}};
const Segment SGI271626[] = {{&S104146[94], 3}, {NULL}};
const Segment SGI271627[] = {{&S104146[28], 3}, {NULL}};
const Segment SGI271628[] = {{&S104146[59], 6}, {NULL}};
const Segment SGI271629[] = {{&S104146[105], 7}, {NULL}};
const Segment SGI271630[] = {{&S104146[120], 7}, {NULL}};
const Segment SGI271631[] = {{&S96339[63], 5}, {NULL}};
const Segment SGI271632[] = {{&S96339[0], 68}, {NULL}};
const Segment SGI271633[] = {{&S96385[66], 5}, {NULL}};
const Segment SGI271634[] = {{&S96385[0], 71}, {NULL}};
const Segment SGI271635[] = {{&S96441[102], 5}, {NULL}};
const Segment SGI271636[] = {{&S96441[0], 107}, {NULL}};
const Segment SGI271637[] = {{&S104332[13], 4}, {NULL}};
const Segment SGI271638[] = {{&S104332[26], 3}, {NULL}};
const Segment SGI271639[] = {{&S104332[114], 5}, {NULL}};
const Segment SGI271640[] = {{&S104332[57], 6}, {NULL}};
const Segment SGI271641[] = {{&S104332[91], 5}, {NULL}};
const Segment SGI271642[] = {{&S96456[75], 5}, {NULL}};
const Segment SGI271643[] = {{&S96456[0], 80}, {NULL}};
const Segment SGI271644[] = {{&S104347[13], 6}, {NULL}};
const Segment SGI271645[] = {{&S104347[38], 5}, {NULL}};
const Segment SGI271646[] = {{&S104348[13], 4}, {NULL}};
const Segment SGI271647[] = {{&S104348[36], 5}, {NULL}};
const Segment SGI271648[] = {{&S104366[28], 4}, {NULL}};
const Segment SGI271649[] = {{&S104366[44], 5}, {NULL}};
const Segment SGI271650[] = {{&S104366[84], 8}, {NULL}};
const Segment SGI271651[] = {{&S104366[120], 5}, {NULL}};
const Segment SGI271652[] = {{&S104366[133], 9}, {NULL}};
const Segment SGI271653[] = {{&S104366[0], 14}, {NULL}};
const Segment SGI271654[] = {{&S104397[13], 6}, {NULL}};
const Segment SGI271655[] = {{&S104397[38], 3}, {NULL}};
const Segment SGI271656[] = {{&S104397[110], 5}, {NULL}};
const Segment SGI271657[] = {{&S104397[123], 9}, {NULL}};
const Segment SGI271658[] = {{&S96553[0], 13}, {&S96553[14], 36}, {NULL}};
const Segment SGI271659[] = {{&S96553[0], 167}, {NULL}};
const Segment SGI271660[] = {{&S96721[66], 5}, {NULL}};
const Segment SGI271661[] = {{&S96721[0], 71}, {NULL}};
const Segment SGI271662[] = {{&S96862[0], 13}, {&S96862[14], 36}, {NULL}};
const Segment SGI271663[] = {{&S96862[0], 167}, {NULL}};
const Segment SGI271664[] = {{&S96871[75], 5}, {NULL}};
const Segment SGI271665[] = {{&S96871[0], 80}, {NULL}};
const Segment SGI271666[] = {{&S104789[16], 4}, {NULL}};
const Segment SGI271667[] = {{&S104789[32], 5}, {NULL}};
const Segment SGI271668[] = {{&S104789[11], 3}, {NULL}};
const Segment SGI271669[] = {{&S104790[13], 4}, {NULL}};
const Segment SGI271670[] = {{&S104790[42], 5}, {NULL}};
const Segment SGI271671[] = {{&S104791[13], 4}, {NULL}};
const Segment SGI271672[] = {{&S104791[42], 5}, {NULL}};
const Segment SGI271673[] = {{&S104792[13], 6}, {NULL}};
const Segment SGI271674[] = {{&S104794[16], 4}, {NULL}};
const Segment SGI271675[] = {{&S104794[32], 5}, {NULL}};
const Segment SGI271676[] = {{&S104794[11], 3}, {NULL}};
const Segment SGI271677[] = {{&S104813[13], 4}, {NULL}};
const Segment SGI271678[] = {{&S104813[33], 5}, {NULL}};
const Segment SGI271679[] = {{&S104813[132], 3}, {NULL}};
const Segment SGI271680[] = {{&S104813[66], 6}, {NULL}};
const Segment SGI271681[] = {{&S104813[100], 5}, {NULL}};
const Segment SGI271682[] = {{&S104813[113], 9}, {NULL}};
const Segment SGI271683[] = {{&S90935[10], 4}, {NULL}};
const Segment SGI271684[] = {{&S90935[15], 5}, {NULL}};
const Segment SGI271685[] = {{&S90941[10], 4}, {NULL}};
const Segment SGI271686[] = {{&S90941[15], 5}, {NULL}};
const Segment SGI271687[] = {{&S90941[17], 3}, {NULL}};
const Segment SGI271688[] = {{&S97203[75], 5}, {NULL}};
const Segment SGI271689[] = {{&S97203[0], 80}, {NULL}};
const Segment SGI271690[] = {{&S105085[13], 6}, {NULL}};
const Segment SGI271691[] = {{&S105085[35], 3}, {NULL}};
const Segment SGI271692[] = {{&S105085[66], 8}, {NULL}};
const Segment SGI271693[] = {{&S105085[120], 8}, {NULL}};
const Segment SGI271694[] = {{&S105085[136], 9}, {NULL}};
const Segment SGI271695[] = {{&S105086[13], 6}, {NULL}};
const Segment SGI271696[] = {{&S105086[35], 3}, {NULL}};
const Segment SGI271697[] = {{&S105086[66], 8}, {NULL}};
const Segment SGI271698[] = {{&S105086[120], 6}, {NULL}};
const Segment SGI271699[] = {{&S105086[134], 9}, {NULL}};
const Segment SGI271700[] = {{&S97403[68], 5}, {NULL}};
const Segment SGI271701[] = {{&S97403[0], 73}, {NULL}};
const Segment SGI271702[] = {{&S91286[10], 4}, {NULL}};
const Segment SGI271703[] = {{&S91286[15], 5}, {NULL}};
const Segment SGI271704[] = {{&S97499[75], 5}, {NULL}};
const Segment SGI271705[] = {{&S97499[0], 80}, {NULL}};
const Segment SGI271706[] = {{&S97546[76], 18}, {NULL}};
const Segment SGI271707[] = {{&S97546[0], 132}, {NULL}};
const Segment SGI271708[] = {{&S105484[15], 6}, {NULL}};
const Segment SGI271709[] = {{&S105484[33], 3}, {NULL}};
const Segment SGI271710[] = {{&S105484[8], 5}, {NULL}};
const Segment SGI271711[] = {{&S105492[24], 6}, {&S105492[31], 4}, {NULL}};
const Segment SGI271712[] = {{&S105493[13], 4}, {NULL}};
const Segment SGI271713[] = {{&S105493[36], 5}, {NULL}};
const Segment SGI271714[] = {{&S105494[12], 6}, {NULL}};
const Segment SGI271715[] = {{&S105494[80], 5}, {NULL}};
const Segment SGI271716[] = {{&S105497[15], 4}, {NULL}};
const Segment SGI271717[] = {{&S105497[37], 5}, {NULL}};
const Segment SGI271718[] = {{&S105497[8], 5}, {NULL}};
const Segment SGI271719[] = {{&S105498[15], 6}, {NULL}};
const Segment SGI271720[] = {{&S105498[33], 5}, {NULL}};
const Segment SGI271721[] = {{&S105498[8], 5}, {NULL}};
const Segment SGI271722[] = {{&S105500[16], 4}, {NULL}};
const Segment SGI271723[] = {{&S105500[32], 5}, {NULL}};
const Segment SGI271724[] = {{&S105500[11], 3}, {NULL}};
const Segment SGI271725[] = {{&S105502[15], 6}, {NULL}};
const Segment SGI271726[] = {{&S105502[33], 5}, {NULL}};
const Segment SGI271727[] = {{&S105502[8], 5}, {NULL}};
const Segment SGI271728[] = {{&S105503[16], 4}, {NULL}};
const Segment SGI271729[] = {{&S105503[32], 5}, {NULL}};
const Segment SGI271730[] = {{&S105503[11], 3}, {NULL}};
const Segment SGI271731[] = {{&S105504[15], 6}, {NULL}};
const Segment SGI271732[] = {{&S105504[33], 5}, {NULL}};
const Segment SGI271733[] = {{&S105504[8], 5}, {NULL}};
const Segment SGI271734[] = {{&S105505[16], 4}, {NULL}};
const Segment SGI271735[] = {{&S105505[32], 5}, {NULL}};
const Segment SGI271736[] = {{&S105505[11], 3}, {NULL}};
const Segment SGI271737[] = {{&S105506[15], 6}, {NULL}};
const Segment SGI271738[] = {{&S105506[33], 5}, {NULL}};
const Segment SGI271739[] = {{&S105506[8], 5}, {NULL}};
const Segment SGI271740[] = {{&S105510[19], 4}, {NULL}};
const Segment SGI271741[] = {{&S105510[41], 3}, {NULL}};
const Segment SGI271742[] = {{&S105523[13], 6}, {NULL}};
const Segment SGI271743[] = {{&S105523[35], 5}, {NULL}};
const Segment SGI271744[] = {{&S105523[125], 5}, {NULL}};
const Segment SGI271745[] = {{&S105523[68], 6}, {NULL}};
const Segment SGI271746[] = {{&S105523[102], 5}, {NULL}};
const Segment SGI271747[] = {{&S91481[10], 5}, {NULL}};
const Segment SGI271748[] = {{&S91481[16], 5}, {NULL}};
const Segment SGI271749[] = {{&S97836[75], 5}, {NULL}};
const Segment SGI271750[] = {{&S97836[0], 80}, {NULL}};
const Segment SGI271751[] = {{&S97931[102], 5}, {NULL}};
const Segment SGI271752[] = {{&S97931[0], 107}, {NULL}};
const Segment SGI271753[] = {{&S105965[13], 6}, {NULL}};
const Segment SGI271754[] = {{&S105965[112], 5}, {NULL}};
const Segment SGI271755[] = {{&S105965[38], 5}, {NULL}};
const Segment SGI271756[] = {{&S105965[78], 5}, {NULL}};
const Segment SGI271757[] = {{&S105965[125], 5}, {NULL}};
const Segment SGI271758[] = {{&S105965[138], 9}, {NULL}};
const Segment SGI271759[] = {{&S105966[13], 4}, {NULL}};
const Segment SGI271760[] = {{&S105966[26], 3}, {NULL}};
const Segment SGI271761[] = {{&S105966[57], 6}, {NULL}};
const Segment SGI271762[] = {{&S105966[91], 5}, {NULL}};
const Segment SGI271763[] = {{&S105966[97], 7}, {NULL}};
const Segment SGI271764[] = {{&S105967[13], 4}, {NULL}};
const Segment SGI271765[] = {{&S105967[26], 3}, {NULL}};
const Segment SGI271766[] = {{&S105967[57], 8}, {NULL}};
const Segment SGI271767[] = {{&S105967[93], 5}, {NULL}};
const Segment SGI271768[] = {{&S105972[13], 4}, {NULL}};
const Segment SGI271769[] = {{&S105972[26], 3}, {NULL}};
const Segment SGI271770[] = {{&S105972[57], 6}, {NULL}};
const Segment SGI271771[] = {{&S105972[109], 5}, {NULL}};
const Segment SGI271772[] = {{&S105972[122], 7}, {NULL}};
const Segment SGI271773[] = {{&S105973[13], 4}, {NULL}};
const Segment SGI271774[] = {{&S105973[26], 3}, {NULL}};
const Segment SGI271775[] = {{&S105973[57], 6}, {NULL}};
const Segment SGI271776[] = {{&S105973[109], 5}, {NULL}};
const Segment SGI271777[] = {{&S105973[122], 7}, {NULL}};
const Segment SGI271778[] = {{&S105974[13], 4}, {NULL}};
const Segment SGI271779[] = {{&S105974[26], 3}, {NULL}};
const Segment SGI271780[] = {{&S105974[57], 8}, {NULL}};
const Segment SGI271781[] = {{&S105974[111], 5}, {NULL}};
const Segment SGI271782[] = {{&S105974[124], 7}, {NULL}};
const Segment SGI271783[] = {{&S105975[13], 6}, {NULL}};
const Segment SGI271784[] = {{&S105975[35], 3}, {NULL}};
const Segment SGI271785[] = {{&S105975[66], 8}, {NULL}};
const Segment SGI271786[] = {{&S105975[122], 8}, {NULL}};
const Segment SGI271787[] = {{&S105975[138], 7}, {NULL}};
const Segment SGI271788[] = {{&S106033[15], 6}, {NULL}};
const Segment SGI271789[] = {{&S106033[8], 5}, {NULL}};
const Segment SGI271790[] = {{&S9636[38], 5}, {NULL}};
const Segment SGI271791[] = {{&S9636[99], 5}, {NULL}};
const Segment SGI271792[] = {{&S9726[38], 5}, {NULL}};
const Segment SGI271793[] = {{&S9726[97], 5}, {NULL}};
const Segment SGI271794[] = {{&S91968[25], 21}, {NULL}};
const Segment SGI271795[] = {{&S91968[47], 5}, {NULL}};
const Segment SGI271796[] = {{&S98246[0], 17}, {NULL}};
const Segment SGI271797[] = {{&S98246[0], 25}, {NULL}};
const Segment SGI271798[] = {{&S92063[10], 4}, {NULL}};
const Segment SGI271799[] = {{&S92063[15], 5}, {NULL}};
const Segment SGI271800[] = {{&S98573[57], 5}, {NULL}};
const Segment SGI271801[] = {{&S98573[0], 62}, {NULL}};
const Segment SGI271802[] = {{&S98584[61], 5}, {NULL}};
const Segment SGI271803[] = {{&S98584[0], 66}, {NULL}};
const Segment SGI271804[] = {{&S98587[61], 5}, {NULL}};
const Segment SGI271805[] = {{&S98587[0], 66}, {NULL}};
const Segment SGI271806[] = {{&S98664[75], 5}, {NULL}};
const Segment SGI271807[] = {{&S98664[0], 80}, {NULL}};
const Segment SGI271808[] = {{&S98747[75], 5}, {NULL}};
const Segment SGI271809[] = {{&S98747[0], 80}, {NULL}};
const Segment SGI271810[] = {{&S10320[38], 5}, {NULL}};
const Segment SGI271811[] = {{&S10320[96], 5}, {NULL}};
const Segment SGI271812[] = {{&S98831[75], 5}, {NULL}};
const Segment SGI271813[] = {{&S98831[0], 80}, {NULL}};
const Segment SGI271814[] = {{&S10455[38], 5}, {NULL}};
const Segment SGI271815[] = {{&S10455[96], 5}, {NULL}};
const Segment SGI271816[] = {{&S92593[25], 21}, {NULL}};
const Segment SGI271817[] = {{&S92593[47], 5}, {NULL}};
const Segment SGI271818[] = {{&S99207[75], 5}, {NULL}};
const Segment SGI271819[] = {{&S99207[0], 80}, {NULL}};
const Segment SGI271820[] = {{&S99489[125], 70}, {NULL}};
const Segment SGI271821[] = {{&S99489[0], 195}, {NULL}};
const Segment SGI271822[] = {{&S99657[77], 5}, {NULL}};
const Segment SGI271823[] = {{&S99657[0], 82}, {NULL}};
const Segment SGI271824[] = {{&S99782[102], 5}, {NULL}};
const Segment SGI271825[] = {{&S99782[0], 107}, {NULL}};
const Segment SGI271826[] = {{&S99824[75], 5}, {NULL}};
const Segment SGI271827[] = {{&S99824[0], 80}, {NULL}};
const Segment SGI271828[] = {{&S100011[51], 18}, {NULL}};
const Segment SGI271829[] = {{&S100011[0], 107}, {NULL}};
const Segment SGI271830[] = {{&S100012[34], 7}, {&S100012[68], 7}, {NULL}};
const Segment SGI271831[] = {{&S100012[0], 107}, {NULL}};
const Segment SGI271832[] = {{&S100013[0], 13}, {&S100013[14], 35}, {NULL}};
const Segment SGI271833[] = {{&S100013[0], 167}, {NULL}};
const Segment SGI271834[] = {{&S100014[86], 7}, {&S100014[96], 25}, {NULL}};
const Segment SGI271835[] = {{&S100014[0], 146}, {NULL}};
const Segment SGI271836[] = {{&S100067[59], 5}, {NULL}};
const Segment SGI271837[] = {{&S100067[0], 64}, {NULL}};
const Segment SGI271838[] = {{&S100219[61], 5}, {NULL}};
const Segment SGI271839[] = {{&S100219[0], 66}, {NULL}};
const Segment SGI271840[] = {{&S100340[75], 5}, {NULL}};
const Segment SGI271841[] = {{&S100340[0], 80}, {NULL}};
const Segment SGI271842[] = {{&S100348[102], 5}, {NULL}};
const Segment SGI271843[] = {{&S100348[0], 107}, {NULL}};
const Segment SGI271844[] = {{&S100712[0], 14}, {NULL}};
const Segment SGI271845[] = {{&S100712[0], 46}, {NULL}};
const Segment SGI271846[] = {{&S93985[10], 4}, {NULL}};
const Segment SGI271847[] = {{&S93985[15], 5}, {NULL}};
const Segment SGI271848[] = {{&S101110[66], 5}, {NULL}};
const Segment SGI271849[] = {{&S101110[0], 71}, {NULL}};
const Segment SGI271850[] = {{&S101197[89], 12}, {&S101197[102], 41}, {NULL}};
const Segment SGI271851[] = {{&S101197[0], 143}, {NULL}};
const Segment SGI271852[] = {{&S101369[35], 14}, {&S101369[50], 32}, {NULL}};
const Segment SGI271853[] = {{&S101369[0], 82}, {NULL}};
const Segment SGI271854[] = {{&S101711[0], 14}, {NULL}};
const Segment SGI271855[] = {{&S101711[0], 46}, {NULL}};
const Segment SGI271856[] = {{&S101712[0], 14}, {NULL}};
const Segment SGI271857[] = {{&S101712[0], 46}, {NULL}};
const Segment SGI271858[] = {{&S101713[0], 14}, {NULL}};
const Segment SGI271859[] = {{&S101713[0], 46}, {NULL}};
const Segment SGI271860[] = {{&S96371[10], 4}, {NULL}};
const Segment SGI271861[] = {{&S96371[15], 5}, {NULL}};
const Segment SGI271862[] = {{&S103059[51], 17}, {NULL}};
const Segment SGI271863[] = {{&S103059[0], 90}, {NULL}};
const Segment SGI271864[] = {{&S96593[10], 4}, {NULL}};
const Segment SGI271865[] = {{&S96593[15], 5}, {NULL}};
const Segment SGI271866[] = {{&S96593[17], 3}, {NULL}};
const Segment SGI271867[] = {{&S96594[10], 4}, {NULL}};
const Segment SGI271868[] = {{&S96594[15], 5}, {NULL}};
const Segment SGI271869[] = {{&S96594[85], 3}, {NULL}};
const Segment SGI271870[] = {{&S96659[23], 21}, {NULL}};
const Segment SGI271871[] = {{&S96659[45], 5}, {NULL}};
const Segment SGI271872[] = {{&S103359[12], 36}, {NULL}};
const Segment SGI271873[] = {{&S103359[0], 48}, {NULL}};
const Segment SGI271874[] = {{&S96962[10], 4}, {NULL}};
const Segment SGI271875[] = {{&S96962[15], 5}, {NULL}};
const Segment SGI271876[] = {{&S96962[65], 13}, {NULL}};
const Segment SGI271877[] = {{&S97240[10], 4}, {NULL}};
const Segment SGI271878[] = {{&S97240[15], 5}, {NULL}};
const Segment SGI271879[] = {{&S97240[85], 3}, {NULL}};
const Segment SGI271880[] = {{&S103952[55], 6}, {&S103952[62], 59}, {NULL}};
const Segment SGI271881[] = {{&S103952[0], 121}, {NULL}};
const Segment SGI271882[] = {{&S97645[10], 4}, {NULL}};
const Segment SGI271883[] = {{&S97645[15], 5}, {NULL}};
const Segment SGI271884[] = {{&S97667[10], 4}, {NULL}};
const Segment SGI271885[] = {{&S97667[15], 5}, {NULL}};
const Segment SGI271886[] = {{&S97667[65], 13}, {NULL}};
const Segment SGI271887[] = {{&S104350[59], 45}, {NULL}};
const Segment SGI271888[] = {{&S104350[0], 104}, {NULL}};
const Segment SGI271889[] = {{&S15736[38], 5}, {NULL}};
const Segment SGI271890[] = {{&S15736[99], 5}, {NULL}};
const Segment SGI271891[] = {{&S104536[71], 5}, {NULL}};
const Segment SGI271892[] = {{&S104536[0], 76}, {NULL}};
const Segment SGI271893[] = {{&S99030[10], 5}, {NULL}};
const Segment SGI271894[] = {{&S99030[16], 5}, {NULL}};
const Segment SGI271895[] = {{&S105547[107], 5}, {NULL}};
const Segment SGI271896[] = {{&S105547[0], 112}, {NULL}};
const Segment SGI271897[] = {{&S99057[10], 4}, {NULL}};
const Segment SGI271898[] = {{&S99057[15], 5}, {NULL}};
const Segment SGI271899[] = {{&S99280[10], 4}, {NULL}};
const Segment SGI271900[] = {{&S99280[15], 5}, {NULL}};
const Segment SGI271901[] = {{&S105927[71], 5}, {NULL}};
const Segment SGI271902[] = {{&S105927[0], 76}, {NULL}};
const Segment SGI271903[] = {{&S106026[74], 21}, {&S106026[96], 32}, {NULL}};
const Segment SGI271904[] = {{&S106026[0], 128}, {NULL}};
const Segment SGI271905[] = {{&S99425[25], 21}, {NULL}};
const Segment SGI271906[] = {{&S99425[47], 5}, {NULL}};
const Segment SGI271907[] = {{&S99526[10], 4}, {NULL}};
const Segment SGI271908[] = {{&S99526[15], 5}, {NULL}};
const Segment SGI271909[] = {{&S99526[17], 3}, {NULL}};
const Segment SGI271910[] = {{&S17393[38], 5}, {NULL}};
const Segment SGI271911[] = {{&S17393[100], 5}, {NULL}};
const Segment SGI271912[] = {{&S100054[88], 22}, {NULL}};
const Segment SGI271913[] = {{&S100141[10], 4}, {NULL}};
const Segment SGI271914[] = {{&S100141[15], 5}, {NULL}};
const Segment SGI271915[] = {{&S100155[10], 4}, {NULL}};
const Segment SGI271916[] = {{&S100155[15], 5}, {NULL}};
const Segment SGI271917[] = {{&S100163[10], 4}, {NULL}};
const Segment SGI271918[] = {{&S100163[15], 5}, {NULL}};
const Segment SGI271919[] = {{&S100163[65], 13}, {NULL}};
const Segment SGI271920[] = {{&S100513[25], 21}, {NULL}};
const Segment SGI271921[] = {{&S100513[47], 5}, {NULL}};
const Segment SGI271922[] = {{&S100743[10], 4}, {NULL}};
const Segment SGI271923[] = {{&S100743[15], 5}, {NULL}};
const Segment SGI271924[] = {{&S100751[10], 4}, {NULL}};
const Segment SGI271925[] = {{&S100751[15], 5}, {NULL}};
const Segment SGI271926[] = {{&S100752[10], 4}, {NULL}};
const Segment SGI271927[] = {{&S100752[15], 5}, {NULL}};
const Segment SGI271928[] = {{&S100944[10], 4}, {NULL}};
const Segment SGI271929[] = {{&S100944[15], 5}, {NULL}};
const Segment SGI271930[] = {{&S100944[17], 3}, {NULL}};
const Segment SGI271931[] = {{&S100970[10], 4}, {NULL}};
const Segment SGI271932[] = {{&S100970[15], 5}, {NULL}};
const Segment SGI271933[] = {{&S101134[10], 4}, {NULL}};
const Segment SGI271934[] = {{&S101134[15], 5}, {NULL}};
const Segment SGI271935[] = {{&S101134[85], 3}, {NULL}};
const Segment SGI271936[] = {{&S101424[10], 4}, {NULL}};
const Segment SGI271937[] = {{&S101424[15], 5}, {NULL}};
const Segment SGI271938[] = {{&S101424[85], 3}, {NULL}};
const Segment SGI271939[] = {{&S101604[10], 4}, {NULL}};
const Segment SGI271940[] = {{&S101604[15], 5}, {NULL}};
const Segment SGI271941[] = {{&S101604[85], 3}, {NULL}};
const Segment SGI271942[] = {{&S101635[10], 4}, {NULL}};
const Segment SGI271943[] = {{&S101635[15], 5}, {NULL}};
const Segment SGI271944[] = {{&S101635[85], 3}, {NULL}};
const Segment SGI271945[] = {{&S19336[38], 5}, {NULL}};
const Segment SGI271946[] = {{&S19336[100], 5}, {NULL}};
const Segment SGI271947[] = {{&S19385[38], 5}, {NULL}};
const Segment SGI271948[] = {{&S19385[100], 5}, {NULL}};
const Segment SGI271949[] = {{&S19479[38], 5}, {NULL}};
const Segment SGI271950[] = {{&S19479[97], 5}, {NULL}};
const Segment SGI271951[] = {{&S102178[10], 4}, {NULL}};
const Segment SGI271952[] = {{&S102178[15], 5}, {NULL}};
const Segment SGI271953[] = {{&S102178[85], 3}, {NULL}};
const Segment SGI271954[] = {{&S102754[10], 4}, {NULL}};
const Segment SGI271955[] = {{&S102754[15], 5}, {NULL}};
const Segment SGI271956[] = {{&S102754[85], 3}, {NULL}};
const Segment SGI271957[] = {{&S20181[38], 5}, {NULL}};
const Segment SGI271958[] = {{&S20181[97], 5}, {NULL}};
const Segment SGI271959[] = {{&S102822[10], 4}, {NULL}};
const Segment SGI271960[] = {{&S102822[15], 5}, {NULL}};
const Segment SGI271961[] = {{&S102961[10], 4}, {NULL}};
const Segment SGI271962[] = {{&S102961[15], 5}, {NULL}};
const Segment SGI271963[] = {{&S102961[85], 3}, {NULL}};
const Segment SGI271964[] = {{&S103050[10], 4}, {NULL}};
const Segment SGI271965[] = {{&S103050[15], 5}, {NULL}};
const Segment SGI271966[] = {{&S103240[10], 4}, {NULL}};
const Segment SGI271967[] = {{&S103240[15], 5}, {NULL}};
const Segment SGI271968[] = {{&S103240[17], 3}, {NULL}};
const Segment SGI271969[] = {{&S103242[10], 4}, {NULL}};
const Segment SGI271970[] = {{&S103242[15], 5}, {NULL}};
const Segment SGI271971[] = {{&S103242[85], 3}, {NULL}};
const Segment SGI271972[] = {{&S103548[10], 4}, {NULL}};
const Segment SGI271973[] = {{&S103548[15], 5}, {NULL}};
const Segment SGI271974[] = {{&S103548[85], 3}, {NULL}};
const Segment SGI271975[] = {{&S103676[22], 12}, {NULL}};
const Segment SGI271976[] = {{&S103891[10], 4}, {NULL}};
const Segment SGI271977[] = {{&S103891[15], 5}, {NULL}};
const Segment SGI271978[] = {{&S103987[10], 4}, {NULL}};
const Segment SGI271979[] = {{&S103987[15], 5}, {NULL}};
const Segment SGI271980[] = {{&S103987[65], 13}, {NULL}};
const Segment SGI271981[] = {{&S21286[38], 5}, {NULL}};
const Segment SGI271982[] = {{&S21286[100], 5}, {NULL}};
const Segment SGI271983[] = {{&S3090[41], 4}, {NULL}};
const Segment SGI271984[] = {{&S3090[13], 14}, {&S3090[28], 25}, {&S3090[75], 17}, {NULL}};
const Segment SGI271985[] = {{&S3091[41], 4}, {NULL}};
const Segment SGI271986[] = {{&S3091[13], 14}, {&S3091[28], 25}, {&S3091[75], 17}, {NULL}};
const Segment SGI271987[] = {{&S104289[10], 4}, {NULL}};
const Segment SGI271988[] = {{&S104289[15], 5}, {NULL}};
const Segment SGI271989[] = {{&S104289[17], 3}, {NULL}};
const Segment SGI271990[] = {{&S3092[41], 4}, {NULL}};
const Segment SGI271991[] = {{&S104290[10], 4}, {NULL}};
const Segment SGI271992[] = {{&S104290[15], 5}, {NULL}};
const Segment SGI271993[] = {{&S3092[13], 14}, {&S3092[28], 25}, {&S3092[75], 17}, {NULL}};
const Segment SGI271994[] = {{&S3093[41], 4}, {NULL}};
const Segment SGI271995[] = {{&S3093[13], 14}, {&S3093[28], 25}, {&S3093[75], 17}, {NULL}};
const Segment SGI271996[] = {{&S3094[41], 4}, {NULL}};
const Segment SGI271997[] = {{&S3094[13], 14}, {&S3094[28], 24}, {&S3094[74], 17}, {NULL}};
const Segment SGI271998[] = {{&S3095[41], 4}, {NULL}};
const Segment SGI271999[] = {{&S3095[13], 14}, {&S3095[28], 25}, {&S3095[75], 17}, {NULL}};
const Segment SGI272000[] = {{&S3096[41], 4}, {NULL}};
const Segment SGI272001[] = {{&S3096[13], 14}, {&S3096[28], 25}, {&S3096[75], 17}, {NULL}};
const Segment SGI272002[] = {{&S3097[41], 4}, {NULL}};
const Segment SGI272003[] = {{&S3097[13], 14}, {&S3097[28], 25}, {&S3097[75], 17}, {NULL}};
const Segment SGI272004[] = {{&S3098[48], 4}, {NULL}};
const Segment SGI272005[] = {{&S3098[13], 14}, {&S3098[28], 6}, {&S3098[35], 25}, {&S3098[82], 17}, {NULL}};
const Segment SGI272006[] = {{&S3100[41], 4}, {NULL}};
const Segment SGI272007[] = {{&S3100[13], 14}, {&S3100[28], 25}, {&S3100[75], 17}, {NULL}};
const Segment SGI272008[] = {{&S3104[48], 4}, {NULL}};
const Segment SGI272009[] = {{&S3104[13], 14}, {&S3104[28], 6}, {&S3104[35], 25}, {&S3104[82], 17}, {NULL}};
const Segment SGI272010[] = {{&S3109[41], 4}, {NULL}};
const Segment SGI272011[] = {{&S3109[13], 14}, {&S3109[28], 25}, {&S3109[75], 17}, {NULL}};
const Segment SGI272012[] = {{&S104329[10], 4}, {NULL}};
const Segment SGI272013[] = {{&S104329[15], 5}, {NULL}};
const Segment SGI272014[] = {{&S104329[65], 13}, {NULL}};
const Segment SGI272015[] = {{&S104330[10], 4}, {NULL}};
const Segment SGI272016[] = {{&S104330[15], 5}, {NULL}};
const Segment SGI272017[] = {{&S104330[64], 13}, {NULL}};
const Segment SGI272018[] = {{&S3122[41], 4}, {NULL}};
const Segment SGI272019[] = {{&S3122[13], 14}, {&S3122[28], 25}, {&S3122[75], 17}, {NULL}};
const Segment SGI272020[] = {{&S3145[41], 4}, {NULL}};
const Segment SGI272021[] = {{&S3145[13], 14}, {&S3145[28], 25}, {&S3145[75], 17}, {NULL}};
const Segment SGI272022[] = {{&S3146[41], 4}, {NULL}};
const Segment SGI272023[] = {{&S3146[13], 14}, {&S3146[28], 25}, {&S3146[75], 17}, {NULL}};
const Segment SGI272024[] = {{&S3147[41], 4}, {NULL}};
const Segment SGI272025[] = {{&S3147[13], 14}, {&S3147[28], 25}, {&S3147[75], 17}, {NULL}};
const Segment SGI272026[] = {{&S3148[41], 4}, {NULL}};
const Segment SGI272027[] = {{&S3148[13], 14}, {&S3148[28], 25}, {&S3148[75], 17}, {NULL}};
const Segment SGI272028[] = {{&S3149[41], 4}, {NULL}};
const Segment SGI272029[] = {{&S3149[13], 14}, {&S3149[28], 25}, {&S3149[75], 17}, {NULL}};
const Segment SGI272030[] = {{&S3150[41], 4}, {NULL}};
const Segment SGI272031[] = {{&S3150[13], 14}, {&S3150[28], 25}, {&S3150[75], 17}, {NULL}};
const Segment SGI272032[] = {{&S3151[41], 4}, {NULL}};
const Segment SGI272033[] = {{&S3151[13], 14}, {&S3151[28], 25}, {&S3151[75], 17}, {NULL}};
const Segment SGI272034[] = {{&S3152[48], 4}, {NULL}};
const Segment SGI272035[] = {{&S3152[13], 14}, {&S3152[28], 6}, {&S3152[35], 25}, {&S3152[82], 17}, {NULL}};
const Segment SGI272036[] = {{&S3153[41], 4}, {NULL}};
const Segment SGI272037[] = {{&S3153[13], 14}, {&S3153[28], 25}, {&S3153[75], 17}, {NULL}};
const Segment SGI272038[] = {{&S3154[41], 4}, {NULL}};
const Segment SGI272039[] = {{&S3154[13], 14}, {&S3154[28], 25}, {&S3154[75], 17}, {NULL}};
const Segment SGI272040[] = {{&S3156[41], 4}, {NULL}};
const Segment SGI272041[] = {{&S3156[13], 14}, {&S3156[28], 26}, {&S3156[76], 17}, {NULL}};
const Segment SGI272042[] = {{&S3157[41], 4}, {NULL}};
const Segment SGI272043[] = {{&S3157[13], 14}, {&S3157[28], 26}, {&S3157[76], 17}, {NULL}};
const Segment SGI272044[] = {{&S3159[48], 4}, {NULL}};
const Segment SGI272045[] = {{&S3159[13], 14}, {&S3159[28], 6}, {&S3159[35], 26}, {&S3159[83], 17}, {NULL}};
const Segment SGI272046[] = {{&S3162[41], 4}, {NULL}};
const Segment SGI272047[] = {{&S3162[13], 14}, {&S3162[28], 26}, {&S3162[76], 17}, {NULL}};
const Segment SGI272048[] = {{&S3166[41], 4}, {NULL}};
const Segment SGI272049[] = {{&S3166[13], 14}, {&S3166[28], 26}, {&S3166[76], 17}, {NULL}};
const Segment SGI272050[] = {{&S3169[48], 4}, {NULL}};
const Segment SGI272051[] = {{&S3169[13], 14}, {&S3169[28], 6}, {&S3169[35], 26}, {&S3169[83], 17}, {NULL}};
const Segment SGI272052[] = {{&S8834[81], 4}, {NULL}};
const Segment SGI272053[] = {{&S8834[13], 14}, {&S8834[28], 6}, {&S8834[55], 17}, {NULL}};
const Segment SGI272054[] = {{&S8835[82], 4}, {NULL}};
const Segment SGI272055[] = {{&S8835[13], 14}, {&S8835[28], 6}, {&S8835[56], 17}, {&S8835[110], 8}, {&S8835[121], 26}, {NULL}};
const Segment SGI272056[] = {{&S8836[75], 4}, {NULL}};
const Segment SGI272057[] = {{&S8836[13], 14}, {&S8836[49], 17}, {&S8836[103], 8}, {&S8836[114], 26}, {NULL}};
const Segment SGI272058[] = {{&S8837[126], 4}, {NULL}};
const Segment SGI272059[] = {{&S8837[13], 7}, {&S8837[21], 2}, {&S8837[24], 15}, {&S8837[40], 6}, {&S8837[47], 29}, {&S8837[11], 1}, {&S8837[100], 17}, {NULL}};
const Segment SGI272060[] = {{&S8838[75], 4}, {NULL}};
const Segment SGI272061[] = {{&S8838[13], 14}, {&S8838[49], 17}, {&S8838[103], 8}, {&S8838[114], 24}, {NULL}};
const Segment SGI272062[] = {{&S8839[82], 4}, {NULL}};
const Segment SGI272063[] = {{&S8839[13], 14}, {&S8839[28], 6}, {&S8839[56], 17}, {&S8839[110], 8}, {&S8839[121], 24}, {NULL}};
const Segment SGI272064[] = {{&S8840[93], 4}, {NULL}};
const Segment SGI272065[] = {{&S8840[13], 7}, {&S8840[21], 2}, {&S8840[24], 15}, {&S8840[40], 6}, {&S8840[67], 17}, {NULL}};
const Segment SGI272066[] = {{&S8841[82], 4}, {NULL}};
const Segment SGI272067[] = {{&S8841[13], 14}, {&S8841[28], 6}, {&S8841[56], 17}, {NULL}};
const Segment SGI272068[] = {{&S8842[74], 4}, {NULL}};
const Segment SGI272069[] = {{&S8842[13], 14}, {&S8842[48], 17}, {NULL}};
const Segment SGI272070[] = {{&S8843[82], 5}, {NULL}};
const Segment SGI272071[] = {{&S8843[13], 14}, {&S8843[28], 6}, {&S8843[56], 17}, {NULL}};
const Segment SGI272072[] = {{&S8844[126], 4}, {NULL}};
const Segment SGI272073[] = {{&S8844[13], 7}, {&S8844[21], 2}, {&S8844[24], 15}, {&S8844[40], 6}, {&S8844[47], 29}, {&S8844[11], 1}, {&S8844[100], 17}, {NULL}};
const Segment SGI272074[] = {{&S8845[126], 4}, {NULL}};
const Segment SGI272075[] = {{&S8845[13], 7}, {&S8845[21], 2}, {&S8845[24], 15}, {&S8845[40], 6}, {&S8845[47], 29}, {&S8845[11], 1}, {&S8845[100], 17}, {NULL}};
const Segment SGI272076[] = {{&S8846[121], 4}, {NULL}};
const Segment SGI272077[] = {{&S8846[13], 7}, {&S8846[21], 2}, {&S8846[24], 15}, {&S8846[40], 6}, {&S8846[47], 24}, {&S8846[11], 1}, {&S8846[95], 17}, {NULL}};
const Segment SGI272078[] = {{&S8847[112], 4}, {NULL}};
const Segment SGI272079[] = {{&S8847[13], 7}, {&S8847[21], 2}, {&S8847[24], 15}, {&S8847[40], 6}, {&S8847[47], 15}, {&S8847[11], 1}, {&S8847[86], 17}, {NULL}};
const Segment SGI272080[] = {{&S8848[75], 4}, {NULL}};
const Segment SGI272081[] = {{&S8848[13], 14}, {&S8848[49], 17}, {NULL}};
const Segment SGI272082[] = {{&S8849[75], 4}, {NULL}};
const Segment SGI272083[] = {{&S8849[13], 14}, {&S8849[49], 17}, {&S8849[120], 35}, {&S8849[156], 13}, {NULL}};
const Segment SGI272084[] = {{&S8850[121], 4}, {NULL}};
const Segment SGI272085[] = {{&S8850[13], 7}, {&S8850[21], 2}, {&S8850[24], 15}, {&S8850[40], 6}, {&S8850[47], 24}, {&S8850[11], 1}, {&S8850[95], 17}, {NULL}};
const Segment SGI272086[] = {{&S8851[126], 4}, {NULL}};
const Segment SGI272087[] = {{&S8851[13], 7}, {&S8851[21], 2}, {&S8851[24], 15}, {&S8851[40], 6}, {&S8851[47], 29}, {&S8851[11], 1}, {&S8851[100], 17}, {NULL}};
const Segment SGI272088[] = {{&S8852[126], 4}, {NULL}};
const Segment SGI272089[] = {{&S8852[13], 7}, {&S8852[21], 2}, {&S8852[24], 15}, {&S8852[40], 6}, {&S8852[47], 29}, {&S8852[11], 1}, {&S8852[100], 17}, {NULL}};
const Segment SGI272090[] = {{&S8853[126], 4}, {NULL}};
const Segment SGI272091[] = {{&S8853[13], 7}, {&S8853[21], 2}, {&S8853[24], 15}, {&S8853[40], 6}, {&S8853[47], 29}, {&S8853[11], 1}, {&S8853[100], 17}, {NULL}};
const Segment SGI272092[] = {{&S8854[126], 4}, {NULL}};
const Segment SGI272093[] = {{&S8854[13], 7}, {&S8854[21], 2}, {&S8854[24], 15}, {&S8854[40], 6}, {&S8854[47], 29}, {&S8854[11], 1}, {&S8854[100], 17}, {NULL}};
const Segment SGI272094[] = {{&S8855[75], 4}, {NULL}};
const Segment SGI272095[] = {{&S8855[13], 14}, {&S8855[49], 17}, {&S8855[120], 35}, {&S8855[156], 20}, {NULL}};
const Segment SGI272096[] = {{&S8856[126], 4}, {NULL}};
const Segment SGI272097[] = {{&S8856[13], 7}, {&S8856[21], 2}, {&S8856[24], 15}, {&S8856[40], 6}, {&S8856[47], 29}, {&S8856[11], 1}, {&S8856[100], 17}, {NULL}};
const Segment SGI272098[] = {{&S8857[121], 4}, {NULL}};
const Segment SGI272099[] = {{&S8857[13], 7}, {&S8857[21], 2}, {&S8857[24], 15}, {&S8857[40], 6}, {&S8857[47], 24}, {&S8857[11], 1}, {&S8857[95], 17}, {NULL}};
const Segment SGI272100[] = {{&S8858[126], 4}, {NULL}};
const Segment SGI272101[] = {{&S8858[13], 7}, {&S8858[21], 2}, {&S8858[24], 15}, {&S8858[40], 6}, {&S8858[47], 29}, {&S8858[11], 1}, {&S8858[100], 17}, {NULL}};
const Segment SGI272102[] = {{&S8859[126], 4}, {NULL}};
const Segment SGI272103[] = {{&S8859[13], 7}, {&S8859[21], 2}, {&S8859[24], 15}, {&S8859[40], 6}, {&S8859[47], 29}, {&S8859[11], 1}, {&S8859[100], 17}, {NULL}};
const Segment SGI272104[] = {{&S8860[126], 4}, {NULL}};
const Segment SGI272105[] = {{&S8860[13], 7}, {&S8860[21], 2}, {&S8860[24], 15}, {&S8860[40], 6}, {&S8860[47], 29}, {&S8860[11], 1}, {&S8860[100], 17}, {NULL}};
const Segment SGI272106[] = {{&S8861[126], 4}, {NULL}};
const Segment SGI272107[] = {{&S8861[13], 7}, {&S8861[21], 2}, {&S8861[24], 15}, {&S8861[40], 6}, {&S8861[47], 29}, {&S8861[11], 1}, {&S8861[100], 17}, {NULL}};
const Segment SGI272108[] = {{&S8862[88], 4}, {NULL}};
const Segment SGI272109[] = {{&S8862[13], 7}, {&S8862[21], 2}, {&S8862[24], 15}, {&S8862[62], 17}, {NULL}};
const Segment SGI272110[] = {{&S8863[91], 4}, {NULL}};
const Segment SGI272111[] = {{&S8863[13], 7}, {&S8863[21], 2}, {&S8863[24], 15}, {&S8863[40], 3}, {&S8863[65], 17}, {NULL}};
const Segment SGI272112[] = {{&S8864[75], 4}, {NULL}};
const Segment SGI272113[] = {{&S8864[13], 14}, {&S8864[49], 17}, {&S8864[120], 35}, {&S8864[156], 18}, {NULL}};
const Segment SGI272114[] = {{&S8865[126], 4}, {NULL}};
const Segment SGI272115[] = {{&S8865[13], 7}, {&S8865[21], 2}, {&S8865[24], 15}, {&S8865[40], 6}, {&S8865[47], 29}, {&S8865[11], 1}, {&S8865[100], 17}, {NULL}};
const Segment SGI272116[] = {{&S8866[126], 4}, {NULL}};
const Segment SGI272117[] = {{&S8866[13], 7}, {&S8866[21], 2}, {&S8866[24], 15}, {&S8866[40], 6}, {&S8866[47], 29}, {&S8866[11], 1}, {&S8866[100], 17}, {NULL}};
const Segment SGI272118[] = {{&S8867[126], 4}, {NULL}};
const Segment SGI272119[] = {{&S8867[13], 7}, {&S8867[21], 2}, {&S8867[24], 15}, {&S8867[40], 6}, {&S8867[47], 29}, {&S8867[11], 1}, {&S8867[100], 17}, {NULL}};
const Segment SGI272120[] = {{&S8868[126], 4}, {NULL}};
const Segment SGI272121[] = {{&S8868[13], 7}, {&S8868[21], 2}, {&S8868[24], 15}, {&S8868[40], 6}, {&S8868[47], 29}, {&S8868[11], 1}, {&S8868[100], 17}, {NULL}};
const Segment SGI272122[] = {{&S8869[126], 4}, {NULL}};
const Segment SGI272123[] = {{&S8869[13], 7}, {&S8869[21], 2}, {&S8869[24], 15}, {&S8869[40], 6}, {&S8869[47], 29}, {&S8869[11], 1}, {&S8869[100], 17}, {NULL}};
const Segment SGI272124[] = {{&S8870[126], 4}, {NULL}};
const Segment SGI272125[] = {{&S8870[13], 7}, {&S8870[21], 2}, {&S8870[24], 15}, {&S8870[40], 6}, {&S8870[47], 29}, {&S8870[11], 1}, {&S8870[100], 17}, {NULL}};
const Segment SGI272126[] = {{&S8871[126], 4}, {NULL}};
const Segment SGI272127[] = {{&S8871[13], 7}, {&S8871[21], 2}, {&S8871[24], 15}, {&S8871[40], 6}, {&S8871[47], 29}, {&S8871[11], 1}, {&S8871[100], 17}, {NULL}};
const Segment SGI272128[] = {{&S8872[121], 4}, {NULL}};
const Segment SGI272129[] = {{&S8872[13], 7}, {&S8872[21], 2}, {&S8872[24], 15}, {&S8872[40], 6}, {&S8872[47], 24}, {&S8872[11], 1}, {&S8872[95], 17}, {NULL}};
const Segment SGI272130[] = {{&S8873[121], 4}, {NULL}};
const Segment SGI272131[] = {{&S8873[13], 7}, {&S8873[21], 2}, {&S8873[24], 15}, {&S8873[40], 6}, {&S8873[47], 24}, {&S8873[11], 1}, {&S8873[95], 17}, {NULL}};
const Segment SGI272132[] = {{&S8874[126], 4}, {NULL}};
const Segment SGI272133[] = {{&S8874[13], 7}, {&S8874[21], 2}, {&S8874[24], 15}, {&S8874[40], 6}, {&S8874[47], 29}, {&S8874[11], 1}, {&S8874[100], 17}, {NULL}};
const Segment SGI272134[] = {{&S8875[126], 4}, {NULL}};
const Segment SGI272135[] = {{&S8875[13], 7}, {&S8875[21], 2}, {&S8875[24], 15}, {&S8875[40], 6}, {&S8875[47], 29}, {&S8875[11], 1}, {&S8875[100], 17}, {NULL}};
const Segment SGI272136[] = {{&S8876[126], 4}, {NULL}};
const Segment SGI272137[] = {{&S8876[13], 7}, {&S8876[21], 2}, {&S8876[24], 15}, {&S8876[40], 6}, {&S8876[47], 29}, {&S8876[11], 1}, {&S8876[100], 17}, {NULL}};
const Segment SGI272138[] = {{&S8877[126], 4}, {NULL}};
const Segment SGI272139[] = {{&S8877[13], 7}, {&S8877[21], 2}, {&S8877[24], 15}, {&S8877[40], 6}, {&S8877[47], 29}, {&S8877[11], 1}, {&S8877[100], 17}, {NULL}};
const Segment SGI272140[] = {{&S8878[126], 4}, {NULL}};
const Segment SGI272141[] = {{&S8878[13], 7}, {&S8878[21], 2}, {&S8878[24], 15}, {&S8878[40], 6}, {&S8878[47], 29}, {&S8878[11], 1}, {&S8878[100], 17}, {NULL}};
const Segment SGI272142[] = {{&S8879[126], 4}, {NULL}};
const Segment SGI272143[] = {{&S8879[13], 7}, {&S8879[21], 2}, {&S8879[24], 15}, {&S8879[40], 6}, {&S8879[47], 29}, {&S8879[11], 1}, {&S8879[100], 17}, {NULL}};
const Segment SGI272144[] = {{&S8880[126], 4}, {NULL}};
const Segment SGI272145[] = {{&S8880[13], 7}, {&S8880[21], 2}, {&S8880[24], 15}, {&S8880[40], 6}, {&S8880[47], 29}, {&S8880[11], 1}, {&S8880[100], 17}, {NULL}};
const Segment SGI272146[] = {{&S8881[122], 4}, {NULL}};
const Segment SGI272147[] = {{&S8881[13], 7}, {&S8881[21], 2}, {&S8881[24], 15}, {&S8881[40], 6}, {&S8881[47], 25}, {&S8881[11], 1}, {&S8881[96], 17}, {NULL}};
const Segment SGI272148[] = {{&S8882[126], 4}, {NULL}};
const Segment SGI272149[] = {{&S8882[13], 7}, {&S8882[21], 2}, {&S8882[24], 15}, {&S8882[40], 6}, {&S8882[47], 29}, {&S8882[11], 1}, {&S8882[100], 17}, {NULL}};
const Segment SGI272150[] = {{&S8883[126], 4}, {NULL}};
const Segment SGI272151[] = {{&S8883[13], 7}, {&S8883[21], 2}, {&S8883[24], 15}, {&S8883[40], 6}, {&S8883[47], 29}, {&S8883[11], 1}, {&S8883[100], 17}, {NULL}};
const Segment SGI272152[] = {{&S8884[126], 4}, {NULL}};
const Segment SGI272153[] = {{&S8884[13], 7}, {&S8884[21], 2}, {&S8884[24], 15}, {&S8884[40], 6}, {&S8884[47], 29}, {&S8884[11], 1}, {&S8884[100], 17}, {NULL}};
const Segment SGI272154[] = {{&S8885[93], 4}, {NULL}};
const Segment SGI272155[] = {{&S8885[13], 7}, {&S8885[21], 2}, {&S8885[24], 15}, {&S8885[40], 6}, {&S8885[67], 17}, {NULL}};
const Segment SGI272156[] = {{&S8886[126], 4}, {NULL}};
const Segment SGI272157[] = {{&S8886[13], 7}, {&S8886[21], 2}, {&S8886[24], 15}, {&S8886[40], 6}, {&S8886[47], 29}, {&S8886[11], 1}, {&S8886[100], 17}, {NULL}};
const Segment SGI272158[] = {{&S8887[122], 4}, {NULL}};
const Segment SGI272159[] = {{&S8887[13], 7}, {&S8887[21], 2}, {&S8887[24], 15}, {&S8887[40], 6}, {&S8887[47], 25}, {&S8887[11], 1}, {&S8887[96], 17}, {NULL}};
const Segment SGI272160[] = {{&S8888[121], 4}, {NULL}};
const Segment SGI272161[] = {{&S8888[13], 7}, {&S8888[21], 2}, {&S8888[24], 15}, {&S8888[40], 6}, {&S8888[47], 24}, {&S8888[11], 1}, {&S8888[95], 17}, {NULL}};
const Segment SGI272162[] = {{&S8889[122], 4}, {NULL}};
const Segment SGI272163[] = {{&S8889[13], 7}, {&S8889[21], 2}, {&S8889[24], 15}, {&S8889[40], 6}, {&S8889[47], 25}, {&S8889[11], 1}, {&S8889[96], 17}, {NULL}};
const Segment SGI272164[] = {{&S8890[126], 4}, {NULL}};
const Segment SGI272165[] = {{&S8890[13], 7}, {&S8890[21], 2}, {&S8890[24], 15}, {&S8890[40], 6}, {&S8890[47], 29}, {&S8890[11], 1}, {&S8890[100], 17}, {NULL}};
const Segment SGI272166[] = {{&S8891[75], 4}, {NULL}};
const Segment SGI272167[] = {{&S8891[13], 14}, {&S8891[49], 17}, {&S8891[103], 8}, {NULL}};
const Segment SGI272168[] = {{&S8892[126], 4}, {NULL}};
const Segment SGI272169[] = {{&S8892[13], 7}, {&S8892[21], 2}, {&S8892[24], 15}, {&S8892[40], 6}, {&S8892[47], 29}, {&S8892[11], 1}, {&S8892[100], 17}, {NULL}};
const Segment SGI272170[] = {{&S8893[126], 4}, {NULL}};
const Segment SGI272171[] = {{&S8893[13], 7}, {&S8893[21], 2}, {&S8893[24], 15}, {&S8893[40], 6}, {&S8893[47], 29}, {&S8893[11], 1}, {&S8893[100], 17}, {NULL}};
const Segment SGI272172[] = {{&S8894[126], 4}, {NULL}};
const Segment SGI272173[] = {{&S8894[13], 7}, {&S8894[21], 2}, {&S8894[24], 15}, {&S8894[40], 6}, {&S8894[47], 29}, {&S8894[11], 1}, {&S8894[100], 17}, {NULL}};
const Segment SGI272174[] = {{&S8895[122], 4}, {NULL}};
const Segment SGI272175[] = {{&S8895[13], 7}, {&S8895[21], 2}, {&S8895[24], 15}, {&S8895[40], 6}, {&S8895[47], 25}, {&S8895[11], 1}, {&S8895[96], 17}, {NULL}};
const Segment SGI272176[] = {{&S8896[121], 4}, {NULL}};
const Segment SGI272177[] = {{&S8896[13], 7}, {&S8896[21], 2}, {&S8896[24], 15}, {&S8896[40], 6}, {&S8896[47], 24}, {&S8896[11], 1}, {&S8896[95], 17}, {NULL}};
const Segment SGI272178[] = {{&S8897[119], 4}, {NULL}};
const Segment SGI272179[] = {{&S8897[13], 7}, {&S8897[21], 2}, {&S8897[24], 15}, {&S8897[40], 6}, {&S8897[47], 22}, {&S8897[11], 1}, {&S8897[93], 17}, {NULL}};
const Segment SGI272180[] = {{&S8898[82], 4}, {NULL}};
const Segment SGI272181[] = {{&S8898[13], 14}, {&S8898[28], 6}, {&S8898[56], 17}, {&S8898[110], 8}, {NULL}};
const Segment SGI272182[] = {{&S8899[118], 4}, {NULL}};
const Segment SGI272183[] = {{&S8899[13], 7}, {&S8899[21], 2}, {&S8899[24], 15}, {&S8899[40], 6}, {&S8899[47], 21}, {&S8899[11], 1}, {&S8899[92], 17}, {NULL}};
const Segment SGI272184[] = {{&S8900[81], 4}, {NULL}};
const Segment SGI272185[] = {{&S8900[13], 14}, {&S8900[28], 6}, {&S8900[55], 17}, {NULL}};
const Segment SGI272186[] = {{&S8901[75], 4}, {NULL}};
const Segment SGI272187[] = {{&S8901[13], 14}, {&S8901[49], 17}, {&S8901[103], 14}, {NULL}};
const Segment SGI272188[] = {{&S104584[10], 5}, {NULL}};
const Segment SGI272189[] = {{&S104584[16], 5}, {NULL}};
const Segment SGI272190[] = {{&S8902[75], 5}, {NULL}};
const Segment SGI272191[] = {{&S8902[13], 14}, {&S8902[49], 17}, {NULL}};
const Segment SGI272192[] = {{&S8903[123], 4}, {NULL}};
const Segment SGI272193[] = {{&S8903[13], 7}, {&S8903[21], 2}, {&S8903[24], 15}, {&S8903[40], 6}, {&S8903[47], 26}, {&S8903[11], 1}, {&S8903[97], 17}, {NULL}};
const Segment SGI272194[] = {{&S8904[113], 4}, {NULL}};
const Segment SGI272195[] = {{&S8904[13], 7}, {&S8904[21], 2}, {&S8904[24], 15}, {&S8904[40], 6}, {&S8904[47], 16}, {&S8904[11], 1}, {&S8904[87], 17}, {NULL}};
const Segment SGI272196[] = {{&S8905[115], 4}, {NULL}};
const Segment SGI272197[] = {{&S8905[13], 7}, {&S8905[21], 2}, {&S8905[24], 15}, {&S8905[40], 6}, {&S8905[47], 18}, {&S8905[11], 1}, {&S8905[89], 17}, {NULL}};
const Segment SGI272198[] = {{&S8920[75], 4}, {NULL}};
const Segment SGI272199[] = {{&S8920[13], 14}, {&S8920[49], 17}, {NULL}};
const Segment SGI272200[] = {{&S8921[82], 4}, {NULL}};
const Segment SGI272201[] = {{&S8921[13], 14}, {&S8921[28], 6}, {&S8921[56], 17}, {NULL}};
const Segment SGI272202[] = {{&S8922[82], 4}, {NULL}};
const Segment SGI272203[] = {{&S8922[13], 14}, {&S8922[28], 6}, {&S8922[56], 17}, {&S8922[108], 8}, {&S8922[119], 26}, {NULL}};
const Segment SGI272204[] = {{&S8923[86], 4}, {NULL}};
const Segment SGI272205[] = {{&S8923[24], 14}, {&S8923[60], 17}, {NULL}};
const Segment SGI272206[] = {{&S8923[0], 11}, {NULL}};
const Segment SGI272207[] = {{&S8924[93], 4}, {NULL}};
const Segment SGI272208[] = {{&S8924[13], 7}, {&S8924[21], 2}, {&S8924[24], 15}, {&S8924[40], 6}, {&S8924[67], 17}, {NULL}};
const Segment SGI272209[] = {{&S8925[75], 4}, {NULL}};
const Segment SGI272210[] = {{&S8925[13], 14}, {&S8925[49], 17}, {&S8925[101], 8}, {&S8925[112], 26}, {NULL}};
const Segment SGI272211[] = {{&S8926[82], 4}, {NULL}};
const Segment SGI272212[] = {{&S8926[13], 14}, {&S8926[28], 6}, {&S8926[56], 17}, {&S8926[126], 8}, {&S8926[137], 24}, {NULL}};
const Segment SGI272213[] = {{&S8927[75], 4}, {NULL}};
const Segment SGI272214[] = {{&S8927[13], 14}, {&S8927[49], 17}, {&S8927[119], 8}, {&S8927[130], 24}, {NULL}};
const Segment SGI272215[] = {{&S8928[75], 4}, {NULL}};
const Segment SGI272216[] = {{&S8928[13], 14}, {&S8928[49], 17}, {&S8928[119], 8}, {NULL}};
const Segment SGI272217[] = {{&S8929[82], 4}, {NULL}};
const Segment SGI272218[] = {{&S8929[13], 14}, {&S8929[28], 6}, {&S8929[56], 17}, {&S8929[126], 8}, {NULL}};
const Segment SGI272219[] = {{&S8930[82], 4}, {NULL}};
const Segment SGI272220[] = {{&S8930[13], 14}, {&S8930[28], 6}, {&S8930[56], 17}, {&S8930[125], 35}, {&S8930[161], 22}, {NULL}};
const Segment SGI272221[] = {{&S8931[82], 4}, {NULL}};
const Segment SGI272222[] = {{&S8931[13], 14}, {&S8931[28], 6}, {&S8931[56], 17}, {NULL}};
const Segment SGI272223[] = {{&S8932[81], 4}, {NULL}};
const Segment SGI272224[] = {{&S8932[13], 14}, {&S8932[28], 6}, {&S8932[55], 17}, {&S8932[133], 3}, {&S8932[137], 11}, {&S8932[149], 10}, {NULL}};
const Segment SGI272225[] = {{&S8933[74], 4}, {NULL}};
const Segment SGI272226[] = {{&S8933[13], 14}, {&S8933[48], 17}, {&S8933[126], 3}, {&S8933[130], 11}, {&S8933[142], 10}, {NULL}};
const Segment SGI272227[] = {{&S8934[74], 4}, {NULL}};
const Segment SGI272228[] = {{&S8934[13], 14}, {&S8934[49], 16}, {NULL}};
const Segment SGI272229[] = {{&S8935[75], 4}, {NULL}};
const Segment SGI272230[] = {{&S8935[13], 14}, {&S8935[49], 17}, {NULL}};
const Segment SGI272231[] = {{&S8936[82], 4}, {NULL}};
const Segment SGI272232[] = {{&S8936[13], 14}, {&S8936[28], 6}, {&S8936[56], 17}, {NULL}};
const Segment SGI272233[] = {{&S104639[10], 5}, {NULL}};
const Segment SGI272234[] = {{&S104639[16], 5}, {NULL}};
const Segment SGI272235[] = {{&S104703[10], 4}, {NULL}};
const Segment SGI272236[] = {{&S104703[15], 5}, {NULL}};
const Segment SGI272237[] = {{&S105097[10], 5}, {NULL}};
const Segment SGI272238[] = {{&S105097[16], 5}, {NULL}};
const Segment SGI272239[] = {{&S105097[18], 3}, {NULL}};
const Segment SGI272240[] = {{&S105159[10], 5}, {NULL}};
const Segment SGI272241[] = {{&S105159[16], 5}, {NULL}};
const Segment SGI272242[] = {{&S105159[86], 3}, {NULL}};
const Segment SGI272243[] = {{&S105173[10], 4}, {NULL}};
const Segment SGI272244[] = {{&S105173[15], 5}, {NULL}};
const Segment SGI272245[] = {{&S105173[85], 3}, {NULL}};
const Segment SGI272246[] = {{&S105196[10], 4}, {NULL}};
const Segment SGI272247[] = {{&S105196[15], 5}, {NULL}};
const Segment SGI272248[] = {{&S105196[85], 3}, {NULL}};
const Segment SGI272249[] = {{&S105211[10], 4}, {NULL}};
const Segment SGI272250[] = {{&S105211[15], 5}, {NULL}};
const Segment SGI272251[] = {{&S105211[17], 3}, {NULL}};
const Segment SGI272252[] = {{&S105235[10], 7}, {NULL}};
const Segment SGI272253[] = {{&S4127[80], 4}, {NULL}};
const Segment SGI272254[] = {{&S4127[13], 14}, {&S4127[28], 6}, {&S4127[56], 17}, {NULL}};
const Segment SGI272255[] = {{&S4127[75], 9}, {NULL}};
const Segment SGI272256[] = {{&S4128[73], 4}, {NULL}};
const Segment SGI272257[] = {{&S4128[13], 14}, {&S4128[49], 17}, {NULL}};
const Segment SGI272258[] = {{&S4128[68], 9}, {NULL}};
const Segment SGI272259[] = {{&S4129[82], 4}, {NULL}};
const Segment SGI272260[] = {{&S4129[13], 14}, {&S4129[28], 6}, {&S4129[56], 17}, {NULL}};
const Segment SGI272261[] = {{&S4130[75], 4}, {NULL}};
const Segment SGI272262[] = {{&S4130[13], 14}, {&S4130[49], 17}, {NULL}};
const Segment SGI272263[] = {{&S4131[82], 4}, {NULL}};
const Segment SGI272264[] = {{&S4131[13], 14}, {&S4131[28], 6}, {&S4131[56], 17}, {NULL}};
const Segment SGI272265[] = {{&S4132[75], 4}, {NULL}};
const Segment SGI272266[] = {{&S4132[13], 14}, {&S4132[49], 17}, {NULL}};
const Segment SGI272267[] = {{&S4133[82], 4}, {NULL}};
const Segment SGI272268[] = {{&S4133[13], 14}, {&S4133[28], 6}, {&S4133[56], 17}, {&S4133[110], 8}, {NULL}};
const Segment SGI272269[] = {{&S4134[75], 4}, {NULL}};
const Segment SGI272270[] = {{&S4134[13], 14}, {&S4134[49], 17}, {&S4134[103], 8}, {NULL}};
const Segment SGI272271[] = {{&S4135[82], 4}, {NULL}};
const Segment SGI272272[] = {{&S4135[13], 14}, {&S4135[28], 6}, {&S4135[56], 17}, {&S4135[110], 8}, {&S4135[121], 26}, {NULL}};
const Segment SGI272273[] = {{&S4136[75], 4}, {NULL}};
const Segment SGI272274[] = {{&S4136[13], 14}, {&S4136[49], 17}, {&S4136[103], 8}, {&S4136[114], 26}, {NULL}};
const Segment SGI272275[] = {{&S4137[82], 4}, {NULL}};
const Segment SGI272276[] = {{&S4137[13], 14}, {&S4137[28], 6}, {&S4137[56], 17}, {NULL}};
const Segment SGI272277[] = {{&S4138[75], 4}, {NULL}};
const Segment SGI272278[] = {{&S4138[13], 14}, {&S4138[49], 17}, {NULL}};
const Segment SGI272279[] = {{&S4139[93], 4}, {NULL}};
const Segment SGI272280[] = {{&S4139[13], 7}, {&S4139[21], 2}, {&S4139[24], 15}, {&S4139[40], 6}, {&S4139[67], 17}, {NULL}};
const Segment SGI272281[] = {{&S4140[82], 4}, {NULL}};
const Segment SGI272282[] = {{&S4140[13], 14}, {&S4140[28], 6}, {&S4140[56], 17}, {&S4140[127], 35}, {&S4140[163], 28}, {NULL}};
const Segment SGI272283[] = {{&S4141[82], 4}, {NULL}};
const Segment SGI272284[] = {{&S4141[13], 14}, {&S4141[28], 6}, {&S4141[56], 17}, {&S4141[127], 35}, {&S4141[163], 15}, {NULL}};
const Segment SGI272285[] = {{&S4142[75], 4}, {NULL}};
const Segment SGI272286[] = {{&S4142[13], 14}, {&S4142[49], 17}, {&S4142[120], 35}, {&S4142[156], 20}, {NULL}};
const Segment SGI272287[] = {{&S4143[82], 5}, {NULL}};
const Segment SGI272288[] = {{&S4143[13], 14}, {&S4143[28], 6}, {&S4143[56], 17}, {NULL}};
const Segment SGI272289[] = {{&S4144[75], 5}, {NULL}};
const Segment SGI272290[] = {{&S4144[13], 14}, {&S4144[49], 17}, {NULL}};
const Segment SGI272291[] = {{&S4145[41], 4}, {NULL}};
const Segment SGI272292[] = {{&S4145[13], 14}, {&S4145[28], 25}, {&S4145[75], 17}, {NULL}};
const Segment SGI272293[] = {{&S4146[41], 4}, {NULL}};
const Segment SGI272294[] = {{&S4146[13], 14}, {&S4146[28], 25}, {&S4146[75], 17}, {NULL}};
const Segment SGI272295[] = {{&S4147[41], 4}, {NULL}};
const Segment SGI272296[] = {{&S4147[13], 14}, {&S4147[28], 25}, {&S4147[75], 17}, {NULL}};
const Segment SGI272297[] = {{&S4148[41], 4}, {NULL}};
const Segment SGI272298[] = {{&S4148[13], 14}, {&S4148[28], 25}, {&S4148[75], 17}, {NULL}};
const Segment SGI272299[] = {{&S4149[41], 4}, {NULL}};
const Segment SGI272300[] = {{&S4149[13], 14}, {&S4149[28], 25}, {&S4149[75], 17}, {NULL}};
const Segment SGI272301[] = {{&S4150[41], 4}, {NULL}};
const Segment SGI272302[] = {{&S4150[13], 14}, {&S4150[28], 25}, {&S4150[75], 17}, {NULL}};
const Segment SGI272303[] = {{&S4151[41], 4}, {NULL}};
const Segment SGI272304[] = {{&S4151[13], 14}, {&S4151[28], 25}, {&S4151[75], 17}, {NULL}};
const Segment SGI272305[] = {{&S4152[41], 4}, {NULL}};
const Segment SGI272306[] = {{&S4152[13], 14}, {&S4152[28], 25}, {&S4152[75], 17}, {NULL}};
const Segment SGI272307[] = {{&S4153[41], 4}, {NULL}};
const Segment SGI272308[] = {{&S4153[13], 14}, {&S4153[28], 25}, {&S4153[75], 17}, {NULL}};
const Segment SGI272309[] = {{&S4154[41], 4}, {NULL}};
const Segment SGI272310[] = {{&S4154[13], 14}, {&S4154[28], 25}, {&S4154[75], 17}, {NULL}};
const Segment SGI272311[] = {{&S4155[41], 4}, {NULL}};
const Segment SGI272312[] = {{&S4155[13], 14}, {&S4155[28], 25}, {&S4155[75], 17}, {NULL}};
const Segment SGI272313[] = {{&S4156[48], 4}, {NULL}};
const Segment SGI272314[] = {{&S4156[13], 14}, {&S4156[28], 6}, {&S4156[35], 25}, {&S4156[82], 17}, {NULL}};
const Segment SGI272315[] = {{&S4158[41], 4}, {NULL}};
const Segment SGI272316[] = {{&S4158[13], 14}, {&S4158[28], 25}, {&S4158[75], 17}, {NULL}};
const Segment SGI272317[] = {{&S4159[48], 4}, {NULL}};
const Segment SGI272318[] = {{&S4159[13], 14}, {&S4159[28], 6}, {&S4159[35], 25}, {&S4159[82], 17}, {NULL}};
const Segment SGI272319[] = {{&S4160[41], 4}, {NULL}};
const Segment SGI272320[] = {{&S4160[13], 14}, {&S4160[28], 25}, {&S4160[75], 17}, {NULL}};
const Segment SGI272321[] = {{&S4164[41], 4}, {NULL}};
const Segment SGI272322[] = {{&S4164[13], 14}, {&S4164[28], 25}, {&S4164[75], 17}, {NULL}};
const Segment SGI272323[] = {{&S105358[10], 4}, {NULL}};
const Segment SGI272324[] = {{&S105358[15], 5}, {NULL}};
const Segment SGI272325[] = {{&S105359[10], 4}, {NULL}};
const Segment SGI272326[] = {{&S105359[15], 5}, {NULL}};
const Segment SGI272327[] = {{&S4174[48], 4}, {NULL}};
const Segment SGI272328[] = {{&S4174[13], 14}, {&S4174[28], 6}, {&S4174[35], 25}, {&S4174[82], 17}, {NULL}};
const Segment SGI272329[] = {{&S4184[41], 4}, {NULL}};
const Segment SGI272330[] = {{&S4184[13], 14}, {&S4184[28], 25}, {&S4184[75], 17}, {NULL}};
const Segment SGI272331[] = {{&S4185[41], 4}, {NULL}};
const Segment SGI272332[] = {{&S4185[13], 14}, {&S4185[28], 25}, {&S4185[75], 17}, {NULL}};
const Segment SGI272333[] = {{&S4186[41], 4}, {NULL}};
const Segment SGI272334[] = {{&S4186[13], 14}, {&S4186[28], 25}, {&S4186[75], 17}, {NULL}};
const Segment SGI272335[] = {{&S4187[41], 4}, {NULL}};
const Segment SGI272336[] = {{&S4187[13], 14}, {&S4187[28], 25}, {&S4187[75], 17}, {NULL}};
const Segment SGI272337[] = {{&S4188[41], 4}, {NULL}};
const Segment SGI272338[] = {{&S4188[13], 14}, {&S4188[28], 25}, {&S4188[75], 17}, {NULL}};
const Segment SGI272339[] = {{&S4189[41], 4}, {NULL}};
const Segment SGI272340[] = {{&S4189[13], 14}, {&S4189[28], 25}, {&S4189[75], 17}, {NULL}};
const Segment SGI272341[] = {{&S4190[41], 4}, {NULL}};
const Segment SGI272342[] = {{&S4190[13], 14}, {&S4190[28], 25}, {&S4190[75], 17}, {NULL}};
const Segment SGI272343[] = {{&S105411[10], 4}, {NULL}};
const Segment SGI272344[] = {{&S105411[15], 5}, {NULL}};
const Segment SGI272345[] = {{&S4191[41], 4}, {NULL}};
const Segment SGI272346[] = {{&S4191[13], 14}, {&S4191[28], 25}, {&S4191[75], 17}, {NULL}};
const Segment SGI272347[] = {{&S4192[48], 4}, {NULL}};
const Segment SGI272348[] = {{&S4192[13], 14}, {&S4192[28], 6}, {&S4192[35], 25}, {&S4192[82], 17}, {NULL}};
const Segment SGI272349[] = {{&S4193[41], 4}, {NULL}};
const Segment SGI272350[] = {{&S4193[13], 14}, {&S4193[28], 25}, {&S4193[75], 17}, {NULL}};
const Segment SGI272351[] = {{&S4194[41], 4}, {NULL}};
const Segment SGI272352[] = {{&S4194[13], 14}, {&S4194[28], 25}, {&S4194[75], 17}, {NULL}};
const Segment SGI272353[] = {{&S4195[41], 4}, {NULL}};
const Segment SGI272354[] = {{&S4195[13], 14}, {&S4195[28], 25}, {&S4195[75], 17}, {NULL}};
const Segment SGI272355[] = {{&S4196[41], 4}, {NULL}};
const Segment SGI272356[] = {{&S4196[13], 14}, {&S4196[28], 25}, {&S4196[75], 17}, {NULL}};
const Segment SGI272357[] = {{&S4197[41], 4}, {NULL}};
const Segment SGI272358[] = {{&S4197[13], 14}, {&S4197[28], 25}, {&S4197[75], 17}, {NULL}};
const Segment SGI272359[] = {{&S4208[48], 4}, {NULL}};
const Segment SGI272360[] = {{&S4208[13], 14}, {&S4208[28], 6}, {&S4208[35], 25}, {&S4208[82], 17}, {NULL}};
const Segment SGI272361[] = {{&S4210[41], 4}, {NULL}};
const Segment SGI272362[] = {{&S4210[13], 14}, {&S4210[28], 25}, {&S4210[75], 17}, {NULL}};
const Segment SGI272363[] = {{&S4213[48], 4}, {NULL}};
const Segment SGI272364[] = {{&S4213[13], 14}, {&S4213[28], 6}, {&S4213[35], 25}, {&S4213[82], 17}, {NULL}};
const Segment SGI272365[] = {{&S4218[81], 4}, {NULL}};
const Segment SGI272366[] = {{&S4218[13], 14}, {&S4218[28], 6}, {&S4218[56], 16}, {NULL}};
const Segment SGI272367[] = {{&S4219[82], 4}, {NULL}};
const Segment SGI272368[] = {{&S4219[13], 14}, {&S4219[28], 6}, {&S4219[56], 17}, {&S4219[126], 35}, {&S4219[162], 28}, {NULL}};
const Segment SGI272369[] = {{&S4220[82], 4}, {NULL}};
const Segment SGI272370[] = {{&S4220[13], 14}, {&S4220[28], 6}, {&S4220[56], 17}, {NULL}};
const Segment SGI272371[] = {{&S4221[75], 4}, {NULL}};
const Segment SGI272372[] = {{&S4221[13], 14}, {&S4221[49], 17}, {NULL}};
const Segment SGI272373[] = {{&S4222[75], 4}, {NULL}};
const Segment SGI272374[] = {{&S4222[13], 14}, {&S4222[49], 17}, {&S4222[102], 8}, {NULL}};
const Segment SGI272375[] = {{&S4223[82], 4}, {NULL}};
const Segment SGI272376[] = {{&S4223[13], 14}, {&S4223[28], 6}, {&S4223[56], 17}, {&S4223[109], 8}, {&S4223[120], 26}, {NULL}};
const Segment SGI272377[] = {{&S4224[82], 4}, {NULL}};
const Segment SGI272378[] = {{&S4224[13], 14}, {&S4224[28], 6}, {&S4224[56], 17}, {&S4224[109], 8}, {NULL}};
const Segment SGI272379[] = {{&S4225[82], 4}, {NULL}};
const Segment SGI272380[] = {{&S4225[13], 14}, {&S4225[28], 6}, {&S4225[56], 17}, {&S4225[109], 10}, {NULL}};
const Segment SGI272381[] = {{&S4226[82], 4}, {NULL}};
const Segment SGI272382[] = {{&S4226[13], 14}, {&S4226[28], 6}, {&S4226[56], 17}, {NULL}};
const Segment SGI272383[] = {{&S4227[75], 4}, {NULL}};
const Segment SGI272384[] = {{&S4227[13], 14}, {&S4227[49], 17}, {NULL}};
const Segment SGI272385[] = {{&S4228[93], 4}, {NULL}};
const Segment SGI272386[] = {{&S4228[13], 7}, {&S4228[21], 2}, {&S4228[24], 15}, {&S4228[40], 6}, {&S4228[67], 17}, {NULL}};
const Segment SGI272387[] = {{&S4229[75], 4}, {NULL}};
const Segment SGI272388[] = {{&S4229[13], 14}, {&S4229[49], 17}, {&S4229[102], 8}, {&S4229[113], 26}, {NULL}};
const Segment SGI272389[] = {{&S4230[75], 4}, {NULL}};
const Segment SGI272390[] = {{&S4230[13], 14}, {&S4230[49], 17}, {&S4230[102], 8}, {&S4230[102], 8}, {NULL}};
const Segment SGI272391[] = {{&S4231[82], 4}, {NULL}};
const Segment SGI272392[] = {{&S4231[13], 14}, {&S4231[28], 6}, {&S4231[56], 17}, {&S4231[109], 8}, {&S4231[109], 8}, {NULL}};
const Segment SGI272393[] = {{&S4232[90], 4}, {NULL}};
const Segment SGI272394[] = {{&S4232[13], 14}, {&S4232[28], 6}, {&S4232[35], 6}, {&S4232[64], 17}, {NULL}};
const Segment SGI272395[] = {{&S4233[82], 4}, {NULL}};
const Segment SGI272396[] = {{&S4233[13], 14}, {&S4233[28], 6}, {&S4233[56], 17}, {NULL}};
const Segment SGI272397[] = {{&S4234[82], 4}, {NULL}};
const Segment SGI272398[] = {{&S4234[13], 14}, {&S4234[28], 6}, {&S4234[56], 17}, {&S4234[126], 35}, {&S4234[162], 19}, {NULL}};
const Segment SGI272399[] = {{&S4235[82], 4}, {NULL}};
const Segment SGI272400[] = {{&S4235[13], 14}, {&S4235[28], 6}, {&S4235[56], 17}, {NULL}};
const Segment SGI272401[] = {{&S4236[75], 4}, {NULL}};
const Segment SGI272402[] = {{&S4236[13], 14}, {&S4236[49], 17}, {NULL}};
const Segment SGI272403[] = {{&S4237[82], 4}, {NULL}};
const Segment SGI272404[] = {{&S4237[13], 14}, {&S4237[28], 6}, {&S4237[56], 17}, {NULL}};
const Segment SGI272405[] = {{&S4238[75], 4}, {NULL}};
const Segment SGI272406[] = {{&S4238[13], 14}, {&S4238[49], 17}, {NULL}};
const Segment SGI272407[] = {{&S4239[75], 4}, {NULL}};
const Segment SGI272408[] = {{&S4239[13], 14}, {&S4239[49], 17}, {&S4239[103], 8}, {NULL}};
const Segment SGI272409[] = {{&S4240[82], 4}, {NULL}};
const Segment SGI272410[] = {{&S4240[13], 14}, {&S4240[28], 6}, {&S4240[56], 17}, {&S4240[110], 8}, {NULL}};
const Segment SGI272411[] = {{&S4241[82], 4}, {NULL}};
const Segment SGI272412[] = {{&S4241[13], 14}, {&S4241[28], 6}, {&S4241[56], 17}, {&S4241[110], 8}, {&S4241[121], 26}, {NULL}};
const Segment SGI272413[] = {{&S4242[75], 4}, {NULL}};
const Segment SGI272414[] = {{&S4242[13], 14}, {&S4242[49], 17}, {&S4242[103], 8}, {&S4242[114], 26}, {NULL}};
const Segment SGI272415[] = {{&S4243[82], 4}, {NULL}};
const Segment SGI272416[] = {{&S4243[13], 14}, {&S4243[28], 6}, {&S4243[56], 17}, {NULL}};
const Segment SGI272417[] = {{&S4244[75], 4}, {NULL}};
const Segment SGI272418[] = {{&S4244[13], 14}, {&S4244[49], 17}, {NULL}};
const Segment SGI272419[] = {{&S4245[117], 4}, {NULL}};
const Segment SGI272420[] = {{&S4245[13], 10}, {&S4245[24], 9}, {&S4245[34], 15}, {&S4245[50], 6}, {&S4245[57], 12}, {&S4245[91], 17}, {NULL}};
const Segment SGI272421[] = {{&S4246[93], 4}, {NULL}};
const Segment SGI272422[] = {{&S4246[13], 7}, {&S4246[21], 2}, {&S4246[24], 15}, {&S4246[40], 6}, {&S4246[67], 17}, {NULL}};
const Segment SGI272423[] = {{&S4247[81], 4}, {NULL}};
const Segment SGI272424[] = {{&S4247[13], 14}, {&S4247[28], 6}, {&S4247[56], 16}, {NULL}};
const Segment SGI272425[] = {{&S4248[74], 4}, {NULL}};
const Segment SGI272426[] = {{&S4248[13], 14}, {&S4248[49], 16}, {NULL}};
const Segment SGI272427[] = {{&S4249[82], 4}, {NULL}};
const Segment SGI272428[] = {{&S4249[13], 14}, {&S4249[28], 6}, {&S4249[56], 17}, {&S4249[127], 35}, {&S4249[163], 22}, {NULL}};
const Segment SGI272429[] = {{&S4250[75], 4}, {NULL}};
const Segment SGI272430[] = {{&S4250[13], 14}, {&S4250[49], 17}, {&S4250[115], 9}, {NULL}};
const Segment SGI272431[] = {{&S4251[75], 4}, {NULL}};
const Segment SGI272432[] = {{&S4251[13], 14}, {&S4251[49], 17}, {&S4251[128], 8}, {&S4251[128], 8}, {NULL}};
const Segment SGI272433[] = {{&S4252[82], 4}, {NULL}};
const Segment SGI272434[] = {{&S4252[13], 14}, {&S4252[28], 6}, {&S4252[56], 17}, {&S4252[140], 3}, {&S4252[144], 14}, {&S4252[159], 11}, {NULL}};
const Segment SGI272435[] = {{&S4253[75], 4}, {NULL}};
const Segment SGI272436[] = {{&S4253[13], 14}, {&S4253[49], 17}, {&S4253[133], 3}, {&S4253[137], 14}, {&S4253[152], 11}, {NULL}};
const Segment SGI272437[] = {{&S4254[82], 4}, {NULL}};
const Segment SGI272438[] = {{&S4254[13], 14}, {&S4254[28], 6}, {&S4254[56], 17}, {&S4254[122], 9}, {NULL}};
const Segment SGI272439[] = {{&S4465[73], 4}, {NULL}};
const Segment SGI272440[] = {{&S4465[13], 14}, {&S4465[48], 16}, {NULL}};
const Segment SGI272441[] = {{&S4466[75], 4}, {NULL}};
const Segment SGI272442[] = {{&S4466[13], 14}, {&S4466[49], 17}, {&S4466[118], 35}, {&S4466[154], 19}, {NULL}};
const Segment SGI272443[] = {{&S4467[75], 4}, {NULL}};
const Segment SGI272444[] = {{&S4467[13], 14}, {&S4467[49], 17}, {NULL}};
const Segment SGI272445[] = {{&S4468[82], 4}, {NULL}};
const Segment SGI272446[] = {{&S4468[13], 14}, {&S4468[28], 6}, {&S4468[56], 17}, {NULL}};
const Segment SGI272447[] = {{&S4469[93], 4}, {NULL}};
const Segment SGI272448[] = {{&S4469[13], 7}, {&S4469[21], 2}, {&S4469[24], 15}, {&S4469[40], 6}, {&S4469[67], 17}, {NULL}};
const Segment SGI272449[] = {{&S4470[82], 4}, {NULL}};
const Segment SGI272450[] = {{&S4470[13], 14}, {&S4470[28], 6}, {&S4470[56], 17}, {NULL}};
const Segment SGI272451[] = {{&S4471[75], 4}, {NULL}};
const Segment SGI272452[] = {{&S4471[13], 14}, {&S4471[49], 17}, {NULL}};
const Segment SGI272453[] = {{&S4472[75], 4}, {NULL}};
const Segment SGI272454[] = {{&S4472[13], 14}, {&S4472[49], 17}, {&S4472[102], 8}, {NULL}};
const Segment SGI272455[] = {{&S4473[82], 4}, {NULL}};
const Segment SGI272456[] = {{&S4473[13], 14}, {&S4473[28], 6}, {&S4473[56], 17}, {&S4473[109], 8}, {NULL}};
const Segment SGI272457[] = {{&S4474[82], 4}, {NULL}};
const Segment SGI272458[] = {{&S4474[13], 14}, {&S4474[28], 6}, {&S4474[56], 17}, {NULL}};
const Segment SGI272459[] = {{&S4475[81], 4}, {NULL}};
const Segment SGI272460[] = {{&S4475[13], 14}, {&S4475[28], 6}, {&S4475[56], 16}, {NULL}};
const Segment SGI272461[] = {{&S4476[82], 4}, {NULL}};
const Segment SGI272462[] = {{&S4476[13], 14}, {&S4476[28], 6}, {&S4476[56], 17}, {NULL}};
const Segment SGI272463[] = {{&S4477[75], 4}, {NULL}};
const Segment SGI272464[] = {{&S4477[13], 14}, {&S4477[49], 17}, {NULL}};
const Segment SGI272465[] = {{&S4478[82], 4}, {NULL}};
const Segment SGI272466[] = {{&S4478[13], 14}, {&S4478[28], 6}, {&S4478[56], 17}, {&S4478[109], 8}, {&S4478[120], 26}, {NULL}};
const Segment SGI272467[] = {{&S4479[75], 4}, {NULL}};
const Segment SGI272468[] = {{&S4479[13], 14}, {&S4479[49], 17}, {&S4479[102], 8}, {&S4479[113], 26}, {NULL}};
const Segment SGI272469[] = {{&S4480[75], 4}, {NULL}};
const Segment SGI272470[] = {{&S4480[13], 14}, {&S4480[49], 17}, {NULL}};
const Segment SGI272471[] = {{&S4481[82], 4}, {NULL}};
const Segment SGI272472[] = {{&S4481[13], 14}, {&S4481[28], 6}, {&S4481[56], 17}, {NULL}};
const Segment SGI272473[] = {{&S4482[75], 4}, {NULL}};
const Segment SGI272474[] = {{&S4482[13], 14}, {&S4482[49], 17}, {NULL}};
const Segment SGI272475[] = {{&S4483[75], 4}, {NULL}};
const Segment SGI272476[] = {{&S4483[13], 14}, {&S4483[49], 17}, {&S4483[119], 35}, {&S4483[155], 21}, {NULL}};
const Segment SGI272477[] = {{&S4484[82], 4}, {NULL}};
const Segment SGI272478[] = {{&S4484[13], 14}, {&S4484[28], 6}, {&S4484[56], 17}, {&S4484[126], 35}, {&S4484[162], 22}, {NULL}};
const Segment SGI272479[] = {{&S4485[101], 4}, {NULL}};
const Segment SGI272480[] = {{&S4485[13], 14}, {&S4485[28], 6}, {&S4485[11], 1}, {&S4485[37], 16}, {&S4485[75], 17}, {NULL}};
const Segment SGI272481[] = {{&S4486[110], 4}, {NULL}};
const Segment SGI272482[] = {{&S4486[13], 14}, {&S4486[28], 6}, {&S4486[11], 1}, {&S4486[37], 25}, {&S4486[84], 17}, {NULL}};
const Segment SGI272483[] = {{&S4487[43], 4}, {NULL}};
const Segment SGI272484[] = {{&S4487[13], 14}, {&S4487[28], 6}, {NULL}};
const Segment SGI272485[] = {{&S4488[97], 4}, {NULL}};
const Segment SGI272486[] = {{&S4488[13], 14}, {&S4488[28], 6}, {&S4488[56], 32}, {NULL}};
const Segment SGI272487[] = {{&S4489[41], 4}, {NULL}};
const Segment SGI272488[] = {{&S4489[13], 14}, {&S4489[28], 24}, {&S4489[74], 17}, {NULL}};
const Segment SGI272489[] = {{&S4490[41], 4}, {NULL}};
const Segment SGI272490[] = {{&S4490[13], 14}, {&S4490[28], 24}, {&S4490[74], 17}, {NULL}};
const Segment SGI272491[] = {{&S4491[41], 4}, {NULL}};
const Segment SGI272492[] = {{&S4491[13], 14}, {&S4491[28], 25}, {&S4491[75], 17}, {NULL}};
const Segment SGI272493[] = {{&S4492[41], 4}, {NULL}};
const Segment SGI272494[] = {{&S4492[13], 14}, {&S4492[28], 25}, {&S4492[75], 17}, {NULL}};
const Segment SGI272495[] = {{&S4493[41], 4}, {NULL}};
const Segment SGI272496[] = {{&S4493[13], 14}, {&S4493[28], 25}, {&S4493[75], 17}, {NULL}};
const Segment SGI272497[] = {{&S4494[41], 4}, {NULL}};
const Segment SGI272498[] = {{&S4494[13], 14}, {&S4494[28], 25}, {&S4494[75], 17}, {NULL}};
const Segment SGI272499[] = {{&S4495[41], 4}, {NULL}};
const Segment SGI272500[] = {{&S4495[13], 14}, {&S4495[28], 25}, {&S4495[75], 17}, {NULL}};
const Segment SGI272501[] = {{&S4496[41], 4}, {NULL}};
const Segment SGI272502[] = {{&S4496[13], 14}, {&S4496[28], 25}, {&S4496[75], 17}, {NULL}};
const Segment SGI272503[] = {{&S4497[41], 4}, {NULL}};
const Segment SGI272504[] = {{&S4497[13], 14}, {&S4497[28], 25}, {&S4497[75], 17}, {NULL}};
const Segment SGI272505[] = {{&S4498[41], 4}, {NULL}};
const Segment SGI272506[] = {{&S4498[13], 14}, {&S4498[28], 25}, {&S4498[75], 17}, {NULL}};
const Segment SGI272507[] = {{&S4499[41], 4}, {NULL}};
const Segment SGI272508[] = {{&S4499[13], 14}, {&S4499[28], 25}, {&S4499[75], 17}, {NULL}};
const Segment SGI272509[] = {{&S4500[41], 4}, {NULL}};
const Segment SGI272510[] = {{&S4500[13], 14}, {&S4500[28], 25}, {&S4500[75], 17}, {NULL}};
const Segment SGI272511[] = {{&S4503[41], 4}, {NULL}};
const Segment SGI272512[] = {{&S4503[13], 14}, {&S4503[28], 25}, {&S4503[75], 17}, {NULL}};
const Segment SGI272513[] = {{&S4504[41], 4}, {NULL}};
const Segment SGI272514[] = {{&S4504[13], 14}, {&S4504[28], 25}, {&S4504[75], 17}, {NULL}};
const Segment SGI272515[] = {{&S4507[41], 4}, {NULL}};
const Segment SGI272516[] = {{&S4507[13], 14}, {&S4507[28], 25}, {&S4507[75], 17}, {NULL}};
const Segment SGI272517[] = {{&S4510[41], 4}, {NULL}};
const Segment SGI272518[] = {{&S4510[13], 14}, {&S4510[28], 25}, {&S4510[75], 17}, {NULL}};
const Segment SGI272519[] = {{&S4517[48], 4}, {NULL}};
const Segment SGI272520[] = {{&S4517[13], 14}, {&S4517[28], 6}, {&S4517[35], 25}, {&S4517[82], 17}, {NULL}};
const Segment SGI272521[] = {{&S4520[41], 4}, {NULL}};
const Segment SGI272522[] = {{&S4520[13], 14}, {&S4520[28], 25}, {&S4520[75], 17}, {NULL}};
const Segment SGI272523[] = {{&S4522[48], 4}, {NULL}};
const Segment SGI272524[] = {{&S4522[13], 14}, {&S4522[28], 6}, {&S4522[35], 25}, {&S4522[82], 17}, {NULL}};
const Segment SGI272525[] = {{&S4526[82], 4}, {NULL}};
const Segment SGI272526[] = {{&S4526[13], 14}, {&S4526[28], 6}, {&S4526[56], 17}, {&S4526[125], 35}, {&S4526[161], 20}, {NULL}};
const Segment SGI272527[] = {{&S4527[82], 4}, {NULL}};
const Segment SGI272528[] = {{&S4527[13], 14}, {&S4527[28], 6}, {&S4527[56], 17}, {NULL}};
const Segment SGI272529[] = {{&S4528[75], 4}, {NULL}};
const Segment SGI272530[] = {{&S4528[13], 14}, {&S4528[49], 17}, {NULL}};
const Segment SGI272531[] = {{&S4529[82], 4}, {NULL}};
const Segment SGI272532[] = {{&S4529[13], 14}, {&S4529[28], 6}, {&S4529[56], 17}, {NULL}};
const Segment SGI272533[] = {{&S4530[75], 4}, {NULL}};
const Segment SGI272534[] = {{&S4530[13], 14}, {&S4530[49], 17}, {&S4530[102], 8}, {&S4530[113], 26}, {NULL}};
const Segment SGI272535[] = {{&S4531[82], 4}, {NULL}};
const Segment SGI272536[] = {{&S4531[13], 14}, {&S4531[28], 6}, {&S4531[56], 17}, {NULL}};
const Segment SGI272537[] = {{&S4532[75], 4}, {NULL}};
const Segment SGI272538[] = {{&S4532[13], 14}, {&S4532[49], 17}, {NULL}};
const Segment SGI272539[] = {{&S4533[75], 4}, {NULL}};
const Segment SGI272540[] = {{&S4533[13], 14}, {&S4533[49], 17}, {NULL}};
const Segment SGI272541[] = {{&S4534[82], 4}, {NULL}};
const Segment SGI272542[] = {{&S4534[13], 14}, {&S4534[28], 6}, {&S4534[56], 17}, {&S4534[109], 8}, {&S4534[120], 26}, {NULL}};
const Segment SGI272543[] = {{&S4535[82], 4}, {NULL}};
const Segment SGI272544[] = {{&S4535[13], 14}, {&S4535[28], 6}, {&S4535[56], 17}, {&S4535[109], 8}, {NULL}};
const Segment SGI272545[] = {{&S4536[75], 4}, {NULL}};
const Segment SGI272546[] = {{&S4536[13], 14}, {&S4536[49], 17}, {&S4536[102], 8}, {NULL}};
const Segment SGI272547[] = {{&S4537[93], 4}, {NULL}};
const Segment SGI272548[] = {{&S4537[13], 7}, {&S4537[21], 2}, {&S4537[24], 15}, {&S4537[40], 6}, {&S4537[67], 17}, {NULL}};
const Segment SGI272549[] = {{&S4538[81], 4}, {NULL}};
const Segment SGI272550[] = {{&S4538[13], 14}, {&S4538[28], 6}, {&S4538[56], 16}, {NULL}};
const Segment SGI272551[] = {{&S4539[103], 4}, {NULL}};
const Segment SGI272552[] = {{&S4539[13], 14}, {&S4539[11], 1}, {&S4539[30], 25}, {&S4539[77], 17}, {NULL}};
const Segment SGI272553[] = {{&S4540[101], 4}, {NULL}};
const Segment SGI272554[] = {{&S4540[13], 14}, {&S4540[28], 6}, {&S4540[11], 1}, {&S4540[37], 16}, {&S4540[75], 17}, {NULL}};
const Segment SGI272555[] = {{&S4541[74], 4}, {NULL}};
const Segment SGI272556[] = {{&S4541[13], 14}, {&S4541[49], 16}, {NULL}};
const Segment SGI272557[] = {{&S4542[75], 4}, {NULL}};
const Segment SGI272558[] = {{&S4542[13], 14}, {&S4542[49], 17}, {&S4542[119], 35}, {&S4542[155], 17}, {NULL}};
const Segment SGI272559[] = {{&S4543[82], 5}, {NULL}};
const Segment SGI272560[] = {{&S4543[13], 14}, {&S4543[28], 6}, {&S4543[56], 17}, {NULL}};
const Segment SGI272561[] = {{&S4544[75], 5}, {NULL}};
const Segment SGI272562[] = {{&S4544[13], 14}, {&S4544[49], 17}, {NULL}};
const Segment SGI272563[] = {{&S4545[75], 4}, {NULL}};
const Segment SGI272564[] = {{&S4545[13], 14}, {&S4545[49], 17}, {&S4545[102], 8}, {&S4545[113], 26}, {NULL}};
const Segment SGI272565[] = {{&S4546[82], 4}, {NULL}};
const Segment SGI272566[] = {{&S4546[13], 14}, {&S4546[28], 6}, {&S4546[56], 17}, {&S4546[109], 8}, {&S4546[120], 26}, {NULL}};
const Segment SGI272567[] = {{&S4547[82], 4}, {NULL}};
const Segment SGI272568[] = {{&S4547[13], 14}, {&S4547[28], 6}, {&S4547[56], 17}, {NULL}};
const Segment SGI272569[] = {{&S4548[75], 4}, {NULL}};
const Segment SGI272570[] = {{&S4548[13], 14}, {&S4548[49], 17}, {NULL}};
const Segment SGI272571[] = {{&S4549[75], 4}, {NULL}};
const Segment SGI272572[] = {{&S4549[13], 14}, {&S4549[49], 17}, {&S4549[102], 8}, {NULL}};
const Segment SGI272573[] = {{&S4550[82], 4}, {NULL}};
const Segment SGI272574[] = {{&S4550[13], 14}, {&S4550[28], 6}, {&S4550[56], 17}, {&S4550[109], 8}, {NULL}};
const Segment SGI272575[] = {{&S4551[75], 4}, {NULL}};
const Segment SGI272576[] = {{&S4551[13], 14}, {&S4551[49], 17}, {&S4551[102], 8}, {&S4551[102], 8}, {NULL}};
const Segment SGI272577[] = {{&S4552[82], 4}, {NULL}};
const Segment SGI272578[] = {{&S4552[13], 14}, {&S4552[28], 6}, {&S4552[56], 17}, {&S4552[109], 8}, {&S4552[109], 8}, {NULL}};
const Segment SGI272579[] = {{&S4553[80], 4}, {NULL}};
const Segment SGI272580[] = {{&S4553[13], 14}, {&S4553[28], 6}, {&S4553[56], 17}, {NULL}};
const Segment SGI272581[] = {{&S4553[75], 9}, {NULL}};
const Segment SGI272582[] = {{&S4554[73], 4}, {NULL}};
const Segment SGI272583[] = {{&S4554[13], 14}, {&S4554[49], 17}, {NULL}};
const Segment SGI272584[] = {{&S4554[68], 9}, {NULL}};
const Segment SGI272585[] = {{&S4555[93], 4}, {NULL}};
const Segment SGI272586[] = {{&S4555[13], 7}, {&S4555[21], 2}, {&S4555[24], 15}, {&S4555[40], 6}, {&S4555[67], 17}, {NULL}};
const Segment SGI272587[] = {{&S4556[82], 4}, {NULL}};
const Segment SGI272588[] = {{&S4556[13], 14}, {&S4556[28], 6}, {&S4556[56], 17}, {&S4556[130], 8}, {NULL}};
const Segment SGI272589[] = {{&S4557[82], 4}, {NULL}};
const Segment SGI272590[] = {{&S4557[13], 14}, {&S4557[28], 6}, {&S4557[56], 17}, {&S4557[126], 35}, {&S4557[162], 28}, {NULL}};
const Segment SGI272591[] = {{&S4910[82], 4}, {NULL}};
const Segment SGI272592[] = {{&S4910[13], 14}, {&S4910[28], 6}, {&S4910[56], 17}, {&S4910[109], 8}, {&S4910[120], 26}, {NULL}};
const Segment SGI272593[] = {{&S4911[93], 4}, {NULL}};
const Segment SGI272594[] = {{&S4911[13], 7}, {&S4911[21], 2}, {&S4911[24], 15}, {&S4911[40], 6}, {&S4911[67], 17}, {NULL}};
const Segment SGI272595[] = {{&S4912[75], 4}, {NULL}};
const Segment SGI272596[] = {{&S4912[13], 14}, {&S4912[49], 17}, {&S4912[102], 8}, {&S4912[113], 26}, {NULL}};
const Segment SGI272597[] = {{&S4913[82], 4}, {NULL}};
const Segment SGI272598[] = {{&S4913[13], 14}, {&S4913[28], 6}, {&S4913[56], 17}, {NULL}};
const Segment SGI272599[] = {{&S4914[75], 4}, {NULL}};
const Segment SGI272600[] = {{&S4914[13], 14}, {&S4914[49], 17}, {NULL}};
const Segment SGI272601[] = {{&S4915[82], 4}, {NULL}};
const Segment SGI272602[] = {{&S4915[13], 14}, {&S4915[28], 6}, {&S4915[56], 17}, {NULL}};
const Segment SGI272603[] = {{&S4916[75], 4}, {NULL}};
const Segment SGI272604[] = {{&S4916[13], 14}, {&S4916[49], 17}, {NULL}};
const Segment SGI272605[] = {{&S4917[75], 4}, {NULL}};
const Segment SGI272606[] = {{&S4917[13], 14}, {&S4917[49], 17}, {&S4917[102], 8}, {NULL}};
const Segment SGI272607[] = {{&S4918[82], 4}, {NULL}};
const Segment SGI272608[] = {{&S4918[13], 14}, {&S4918[28], 6}, {&S4918[56], 17}, {&S4918[109], 8}, {NULL}};
const Segment SGI272609[] = {{&S4919[82], 4}, {NULL}};
const Segment SGI272610[] = {{&S4919[13], 14}, {&S4919[28], 6}, {&S4919[56], 17}, {NULL}};
const Segment SGI272611[] = {{&S4920[75], 4}, {NULL}};
const Segment SGI272612[] = {{&S4920[13], 14}, {&S4920[49], 17}, {NULL}};
const Segment SGI272613[] = {{&S4921[82], 4}, {NULL}};
const Segment SGI272614[] = {{&S4921[13], 14}, {&S4921[28], 6}, {&S4921[56], 17}, {&S4921[126], 35}, {&S4921[162], 22}, {NULL}};
const Segment SGI272615[] = {{&S4922[82], 4}, {NULL}};
const Segment SGI272616[] = {{&S4922[13], 14}, {&S4922[28], 6}, {&S4922[56], 17}, {&S4922[148], 35}, {&S4922[184], 14}, {NULL}};
const Segment SGI272617[] = {{&S4923[81], 4}, {NULL}};
const Segment SGI272618[] = {{&S4923[13], 14}, {&S4923[28], 6}, {&S4923[56], 16}, {NULL}};
const Segment SGI272619[] = {{&S4924[75], 4}, {NULL}};
const Segment SGI272620[] = {{&S4924[13], 14}, {&S4924[49], 17}, {&S4924[119], 35}, {&S4924[155], 15}, {NULL}};
const Segment SGI272621[] = {{&S4925[82], 4}, {NULL}};
const Segment SGI272622[] = {{&S4925[13], 14}, {&S4925[28], 6}, {&S4925[56], 17}, {&S4925[95], 4}, {NULL}};
const Segment SGI272623[] = {{&S4926[75], 4}, {NULL}};
const Segment SGI272624[] = {{&S4926[13], 14}, {&S4926[49], 17}, {&S4926[119], 35}, {&S4926[155], 21}, {NULL}};
const Segment SGI272625[] = {{&S4928[97], 4}, {NULL}};
const Segment SGI272626[] = {{&S4928[13], 14}, {&S4928[28], 6}, {&S4928[55], 17}, {NULL}};
const Segment SGI272627[] = {{&S4928[74], 11}, {NULL}};
const Segment SGI272628[] = {{&S4930[41], 4}, {NULL}};
const Segment SGI272629[] = {{&S4930[13], 14}, {&S4930[28], 24}, {&S4930[74], 17}, {NULL}};
const Segment SGI272630[] = {{&S4931[41], 4}, {NULL}};
const Segment SGI272631[] = {{&S4931[13], 14}, {&S4931[28], 24}, {&S4931[74], 17}, {NULL}};
const Segment SGI272632[] = {{&S4932[41], 4}, {NULL}};
const Segment SGI272633[] = {{&S4932[13], 14}, {&S4932[28], 25}, {&S4932[75], 17}, {NULL}};
const Segment SGI272634[] = {{&S4933[41], 4}, {NULL}};
const Segment SGI272635[] = {{&S4933[13], 14}, {&S4933[28], 25}, {&S4933[75], 17}, {NULL}};
const Segment SGI272636[] = {{&S4934[41], 4}, {NULL}};
const Segment SGI272637[] = {{&S4934[13], 14}, {&S4934[28], 25}, {&S4934[75], 17}, {NULL}};
const Segment SGI272638[] = {{&S4935[41], 4}, {NULL}};
const Segment SGI272639[] = {{&S4935[13], 14}, {&S4935[28], 25}, {&S4935[75], 17}, {NULL}};
const Segment SGI272640[] = {{&S4936[41], 4}, {NULL}};
const Segment SGI272641[] = {{&S4936[13], 14}, {&S4936[28], 25}, {&S4936[75], 17}, {NULL}};
const Segment SGI272642[] = {{&S4937[41], 4}, {NULL}};
const Segment SGI272643[] = {{&S4937[13], 14}, {&S4937[28], 25}, {&S4937[75], 17}, {NULL}};
const Segment SGI272644[] = {{&S4938[41], 4}, {NULL}};
const Segment SGI272645[] = {{&S4938[13], 14}, {&S4938[28], 25}, {&S4938[75], 17}, {NULL}};
const Segment SGI272646[] = {{&S4939[41], 4}, {NULL}};
const Segment SGI272647[] = {{&S4939[13], 14}, {&S4939[28], 25}, {&S4939[75], 17}, {NULL}};
const Segment SGI272648[] = {{&S4940[41], 4}, {NULL}};
const Segment SGI272649[] = {{&S4940[13], 14}, {&S4940[28], 26}, {&S4940[76], 17}, {NULL}};
const Segment SGI272650[] = {{&S4944[41], 4}, {NULL}};
const Segment SGI272651[] = {{&S4944[13], 14}, {&S4944[28], 26}, {&S4944[76], 17}, {NULL}};
const Segment SGI272652[] = {{&S4946[41], 4}, {NULL}};
const Segment SGI272653[] = {{&S4946[13], 14}, {&S4946[28], 26}, {&S4946[76], 17}, {NULL}};
const Segment SGI272654[] = {{&S4948[41], 4}, {NULL}};
const Segment SGI272655[] = {{&S4948[13], 14}, {&S4948[28], 26}, {&S4948[76], 17}, {NULL}};
const Segment SGI272656[] = {{&S4949[41], 4}, {NULL}};
const Segment SGI272657[] = {{&S4949[13], 14}, {&S4949[28], 26}, {&S4949[76], 17}, {NULL}};
const Segment SGI272658[] = {{&S4950[41], 4}, {NULL}};
const Segment SGI272659[] = {{&S4950[13], 14}, {&S4950[28], 26}, {&S4950[76], 17}, {NULL}};
const Segment SGI272660[] = {{&S4956[48], 4}, {NULL}};
const Segment SGI272661[] = {{&S4956[13], 14}, {&S4956[28], 6}, {&S4956[35], 26}, {&S4956[83], 17}, {NULL}};
const Segment SGI272662[] = {{&S4960[48], 4}, {NULL}};
const Segment SGI272663[] = {{&S4960[13], 14}, {&S4960[28], 6}, {&S4960[35], 25}, {&S4960[82], 17}, {NULL}};
const Segment SGI272664[] = {{&S4961[74], 4}, {NULL}};
const Segment SGI272665[] = {{&S4961[13], 14}, {&S4961[48], 17}, {NULL}};
const Segment SGI272666[] = {{&S4962[81], 4}, {NULL}};
const Segment SGI272667[] = {{&S4962[13], 14}, {&S4962[28], 6}, {&S4962[55], 17}, {NULL}};
const Segment SGI272668[] = {{&S4963[81], 4}, {NULL}};
const Segment SGI272669[] = {{&S4963[13], 14}, {&S4963[28], 6}, {&S4963[55], 17}, {&S4963[107], 8}, {&S4963[118], 24}, {NULL}};
const Segment SGI272670[] = {{&S4964[74], 4}, {NULL}};
const Segment SGI272671[] = {{&S4964[13], 14}, {&S4964[48], 17}, {&S4964[100], 8}, {&S4964[111], 24}, {NULL}};
const Segment SGI272672[] = {{&S4965[81], 4}, {NULL}};
const Segment SGI272673[] = {{&S4965[13], 14}, {&S4965[28], 6}, {&S4965[55], 17}, {&S4965[107], 8}, {&S4965[118], 26}, {NULL}};
const Segment SGI272674[] = {{&S4966[93], 4}, {NULL}};
const Segment SGI272675[] = {{&S4966[13], 7}, {&S4966[21], 2}, {&S4966[24], 15}, {&S4966[40], 6}, {&S4966[67], 17}, {NULL}};
const Segment SGI272676[] = {{&S4967[81], 4}, {NULL}};
const Segment SGI272677[] = {{&S4967[13], 14}, {&S4967[28], 6}, {&S4967[55], 17}, {&S4967[107], 8}, {NULL}};
const Segment SGI272678[] = {{&S4968[74], 4}, {NULL}};
const Segment SGI272679[] = {{&S4968[13], 14}, {&S4968[48], 17}, {&S4968[100], 8}, {NULL}};
const Segment SGI272680[] = {{&S4969[74], 4}, {NULL}};
const Segment SGI272681[] = {{&S4969[13], 14}, {&S4969[48], 17}, {&S4969[100], 8}, {&S4969[111], 26}, {NULL}};
const Segment SGI272682[] = {{&S4970[81], 4}, {NULL}};
const Segment SGI272683[] = {{&S4970[13], 14}, {&S4970[28], 6}, {&S4970[55], 17}, {&S4970[124], 35}, {&S4970[160], 22}, {NULL}};
const Segment SGI272684[] = {{&S4971[80], 4}, {NULL}};
const Segment SGI272685[] = {{&S4971[13], 14}, {&S4971[28], 6}, {&S4971[55], 16}, {NULL}};
const Segment SGI272686[] = {{&S4972[73], 4}, {NULL}};
const Segment SGI272687[] = {{&S4972[13], 14}, {&S4972[48], 16}, {NULL}};
const Segment SGI272688[] = {{&S4973[74], 4}, {NULL}};
const Segment SGI272689[] = {{&S4973[13], 14}, {&S4973[48], 17}, {&S4973[112], 4}, {NULL}};
const Segment SGI272690[] = {{&S4974[81], 4}, {NULL}};
const Segment SGI272691[] = {{&S4974[13], 14}, {&S4974[28], 6}, {&S4974[55], 17}, {&S4974[119], 4}, {NULL}};
const Segment SGI272692[] = {{&S4975[74], 4}, {NULL}};
const Segment SGI272693[] = {{&S4975[13], 14}, {&S4975[48], 17}, {&S4975[112], 4}, {NULL}};
const Segment SGI272694[] = {{&S4976[81], 4}, {NULL}};
const Segment SGI272695[] = {{&S4976[13], 14}, {&S4976[28], 6}, {&S4976[55], 17}, {&S4976[119], 4}, {NULL}};
const Segment SGI272696[] = {{&S4977[81], 4}, {NULL}};
const Segment SGI272697[] = {{&S4977[13], 14}, {&S4977[28], 6}, {&S4977[55], 17}, {&S4977[119], 4}, {NULL}};
const Segment SGI272698[] = {{&S4978[74], 4}, {NULL}};
const Segment SGI272699[] = {{&S4978[13], 14}, {&S4978[48], 17}, {&S4978[112], 4}, {NULL}};
const Segment SGI272700[] = {{&S4979[74], 4}, {NULL}};
const Segment SGI272701[] = {{&S4979[13], 14}, {&S4979[48], 17}, {NULL}};
const Segment SGI272702[] = {{&S4980[81], 4}, {NULL}};
const Segment SGI272703[] = {{&S4980[13], 14}, {&S4980[28], 6}, {&S4980[55], 17}, {NULL}};
const Segment SGI272704[] = {{&S4981[48], 4}, {NULL}};
const Segment SGI272705[] = {{&S4981[13], 14}, {&S4981[28], 6}, {&S4981[35], 25}, {&S4981[82], 17}, {NULL}};
const Segment SGI272706[] = {{&S4982[41], 4}, {NULL}};
const Segment SGI272707[] = {{&S4982[13], 14}, {&S4982[28], 25}, {&S4982[75], 17}, {NULL}};
const Segment SGI272708[] = {{&S4983[41], 4}, {NULL}};
const Segment SGI272709[] = {{&S4983[13], 14}, {&S4983[28], 25}, {&S4983[75], 17}, {NULL}};
const Segment SGI272710[] = {{&S4984[48], 4}, {NULL}};
const Segment SGI272711[] = {{&S4984[13], 14}, {&S4984[28], 6}, {&S4984[35], 25}, {&S4984[82], 17}, {NULL}};
const Segment SGI272712[] = {{&S4985[48], 4}, {NULL}};
const Segment SGI272713[] = {{&S4985[13], 14}, {&S4985[28], 6}, {&S4985[35], 25}, {&S4985[82], 17}, {NULL}};
const Segment SGI272714[] = {{&S4986[41], 4}, {NULL}};
const Segment SGI272715[] = {{&S4986[13], 14}, {&S4986[28], 25}, {&S4986[75], 17}, {NULL}};
const Segment SGI272716[] = {{&S4987[48], 4}, {NULL}};
const Segment SGI272717[] = {{&S4987[13], 14}, {&S4987[28], 6}, {&S4987[35], 25}, {&S4987[82], 17}, {NULL}};
const Segment SGI272718[] = {{&S4988[41], 4}, {NULL}};
const Segment SGI272719[] = {{&S4988[13], 14}, {&S4988[28], 25}, {&S4988[75], 17}, {NULL}};
const Segment SGI272720[] = {{&S4989[41], 4}, {NULL}};
const Segment SGI272721[] = {{&S4989[13], 14}, {&S4989[28], 25}, {&S4989[75], 17}, {NULL}};
const Segment SGI272722[] = {{&S4990[48], 4}, {NULL}};
const Segment SGI272723[] = {{&S4990[13], 14}, {&S4990[28], 6}, {&S4990[35], 25}, {&S4990[82], 17}, {NULL}};
const Segment SGI272724[] = {{&S4991[41], 4}, {NULL}};
const Segment SGI272725[] = {{&S4991[13], 14}, {&S4991[28], 25}, {&S4991[75], 17}, {NULL}};
const Segment SGI272726[] = {{&S4992[48], 4}, {NULL}};
const Segment SGI272727[] = {{&S4992[13], 14}, {&S4992[28], 6}, {&S4992[35], 25}, {&S4992[82], 17}, {NULL}};
const Segment SGI272728[] = {{&S4997[41], 4}, {NULL}};
const Segment SGI272729[] = {{&S4997[13], 14}, {&S4997[28], 25}, {&S4997[75], 17}, {NULL}};
const Segment SGI272730[] = {{&S4999[48], 4}, {NULL}};
const Segment SGI272731[] = {{&S4999[13], 14}, {&S4999[28], 6}, {&S4999[35], 25}, {&S4999[82], 17}, {NULL}};
const Segment SGI272732[] = {{&S5012[48], 4}, {NULL}};
const Segment SGI272733[] = {{&S5012[13], 14}, {&S5012[28], 6}, {&S5012[35], 25}, {&S5012[82], 17}, {NULL}};
const Segment SGI272734[] = {{&S5013[41], 4}, {NULL}};
const Segment SGI272735[] = {{&S5013[13], 14}, {&S5013[28], 25}, {&S5013[75], 17}, {NULL}};
const Segment SGI272736[] = {{&S23674[38], 5}, {NULL}};
const Segment SGI272737[] = {{&S23674[100], 5}, {NULL}};
const Segment SGI272738[] = {{&S5250[48], 4}, {NULL}};
const Segment SGI272739[] = {{&S5250[13], 14}, {&S5250[28], 6}, {&S5250[35], 25}, {&S5250[82], 17}, {NULL}};
const Segment SGI272740[] = {{&S5251[41], 4}, {NULL}};
const Segment SGI272741[] = {{&S5251[13], 14}, {&S5251[28], 25}, {&S5251[75], 17}, {NULL}};
const Segment SGI272742[] = {{&S5252[48], 4}, {NULL}};
const Segment SGI272743[] = {{&S5252[13], 14}, {&S5252[28], 6}, {&S5252[35], 25}, {&S5252[82], 17}, {NULL}};
const Segment SGI272744[] = {{&S5253[41], 4}, {NULL}};
const Segment SGI272745[] = {{&S5253[13], 14}, {&S5253[28], 25}, {&S5253[75], 17}, {NULL}};
const Segment SGI272746[] = {{&S5254[48], 4}, {NULL}};
const Segment SGI272747[] = {{&S5254[13], 14}, {&S5254[28], 6}, {&S5254[35], 25}, {&S5254[82], 17}, {NULL}};
const Segment SGI272748[] = {{&S5255[41], 4}, {NULL}};
const Segment SGI272749[] = {{&S5255[13], 14}, {&S5255[28], 25}, {&S5255[75], 17}, {NULL}};
const Segment SGI272750[] = {{&S5256[48], 4}, {NULL}};
const Segment SGI272751[] = {{&S5256[13], 14}, {&S5256[28], 6}, {&S5256[35], 25}, {&S5256[82], 17}, {NULL}};
const Segment SGI272752[] = {{&S5257[41], 4}, {NULL}};
const Segment SGI272753[] = {{&S5257[13], 14}, {&S5257[28], 25}, {&S5257[75], 17}, {NULL}};
const Segment SGI272754[] = {{&S5258[48], 4}, {NULL}};
const Segment SGI272755[] = {{&S5258[13], 14}, {&S5258[28], 6}, {&S5258[35], 25}, {&S5258[82], 17}, {NULL}};
const Segment SGI272756[] = {{&S5259[41], 4}, {NULL}};
const Segment SGI272757[] = {{&S5259[13], 14}, {&S5259[28], 25}, {&S5259[75], 17}, {NULL}};
const Segment SGI272758[] = {{&S5260[48], 4}, {NULL}};
const Segment SGI272759[] = {{&S5260[13], 14}, {&S5260[28], 6}, {&S5260[35], 25}, {&S5260[82], 17}, {NULL}};
const Segment SGI272760[] = {{&S5261[41], 4}, {NULL}};
const Segment SGI272761[] = {{&S5261[13], 14}, {&S5261[28], 25}, {&S5261[75], 17}, {NULL}};
const Segment SGI272762[] = {{&S5262[48], 4}, {NULL}};
const Segment SGI272763[] = {{&S5262[13], 14}, {&S5262[28], 6}, {&S5262[35], 25}, {&S5262[82], 17}, {NULL}};
const Segment SGI272764[] = {{&S5263[41], 4}, {NULL}};
const Segment SGI272765[] = {{&S5263[13], 14}, {&S5263[28], 25}, {&S5263[75], 17}, {NULL}};
const Segment SGI272766[] = {{&S5265[41], 4}, {NULL}};
const Segment SGI272767[] = {{&S5265[13], 14}, {&S5265[28], 25}, {&S5265[75], 17}, {NULL}};
const Segment SGI272768[] = {{&S5266[48], 4}, {NULL}};
const Segment SGI272769[] = {{&S5266[13], 14}, {&S5266[28], 6}, {&S5266[35], 25}, {&S5266[82], 17}, {NULL}};
const Segment SGI272770[] = {{&S5279[48], 4}, {NULL}};
const Segment SGI272771[] = {{&S5279[13], 14}, {&S5279[28], 6}, {&S5279[35], 25}, {&S5279[82], 17}, {NULL}};
const Segment SGI272772[] = {{&S5281[41], 4}, {NULL}};
const Segment SGI272773[] = {{&S5281[13], 14}, {&S5281[28], 25}, {&S5281[75], 17}, {NULL}};
const Segment SGI272774[] = {{&S10871[74], 4}, {NULL}};
const Segment SGI272775[] = {{&S10871[13], 14}, {&S10871[48], 17}, {NULL}};
const Segment SGI272776[] = {{&S10872[74], 4}, {NULL}};
const Segment SGI272777[] = {{&S10872[13], 14}, {&S10872[48], 17}, {&S10872[100], 8}, {&S10872[111], 24}, {NULL}};
const Segment SGI272778[] = {{&S10873[81], 4}, {NULL}};
const Segment SGI272779[] = {{&S10873[13], 14}, {&S10873[28], 6}, {&S10873[55], 17}, {NULL}};
const Segment SGI272780[] = {{&S10874[81], 4}, {NULL}};
const Segment SGI272781[] = {{&S10874[13], 14}, {&S10874[28], 6}, {&S10874[55], 17}, {&S10874[107], 8}, {&S10874[118], 24}, {NULL}};
const Segment SGI272782[] = {{&S10875[81], 4}, {NULL}};
const Segment SGI272783[] = {{&S10875[13], 14}, {&S10875[28], 6}, {&S10875[55], 17}, {&S10875[107], 8}, {&S10875[118], 26}, {NULL}};
const Segment SGI272784[] = {{&S10876[74], 4}, {NULL}};
const Segment SGI272785[] = {{&S10876[13], 14}, {&S10876[48], 17}, {&S10876[100], 8}, {&S10876[111], 26}, {NULL}};
const Segment SGI272786[] = {{&S10877[93], 4}, {NULL}};
const Segment SGI272787[] = {{&S10877[13], 7}, {&S10877[21], 2}, {&S10877[24], 15}, {&S10877[40], 6}, {&S10877[67], 17}, {NULL}};
const Segment SGI272788[] = {{&S10878[81], 4}, {NULL}};
const Segment SGI272789[] = {{&S10878[13], 14}, {&S10878[28], 6}, {&S10878[55], 17}, {&S10878[107], 8}, {NULL}};
const Segment SGI272790[] = {{&S10879[74], 4}, {NULL}};
const Segment SGI272791[] = {{&S10879[13], 14}, {&S10879[48], 17}, {&S10879[100], 8}, {NULL}};
const Segment SGI272792[] = {{&S10880[98], 4}, {NULL}};
const Segment SGI272793[] = {{&S10880[13], 14}, {&S10880[48], 17}, {NULL}};
const Segment SGI272794[] = {{&S10881[105], 4}, {NULL}};
const Segment SGI272795[] = {{&S10881[13], 14}, {&S10881[28], 6}, {&S10881[55], 17}, {NULL}};
const Segment SGI272796[] = {{&S10882[98], 4}, {NULL}};
const Segment SGI272797[] = {{&S10882[13], 14}, {&S10882[48], 17}, {&S10882[126], 8}, {NULL}};
const Segment SGI272798[] = {{&S10883[83], 4}, {NULL}};
const Segment SGI272799[] = {{&S10883[13], 7}, {&S10883[21], 15}, {&S10883[57], 17}, {NULL}};
const Segment SGI272800[] = {{&S10884[105], 4}, {NULL}};
const Segment SGI272801[] = {{&S10884[13], 14}, {&S10884[28], 6}, {&S10884[55], 17}, {&S10884[133], 8}, {NULL}};
const Segment SGI272802[] = {{&S10885[93], 4}, {NULL}};
const Segment SGI272803[] = {{&S10885[25], 14}, {&S10885[40], 6}, {&S10885[67], 17}, {NULL}};
const Segment SGI272804[] = {{&S10885[0], 12}, {NULL}};
const Segment SGI272805[] = {{&S10886[74], 5}, {NULL}};
const Segment SGI272806[] = {{&S10886[13], 14}, {&S10886[48], 17}, {NULL}};
const Segment SGI272807[] = {{&S5624[48], 4}, {NULL}};
const Segment SGI272808[] = {{&S5624[13], 14}, {&S5624[28], 6}, {&S5624[35], 25}, {&S5624[82], 17}, {NULL}};
const Segment SGI272809[] = {{&S5625[41], 4}, {NULL}};
const Segment SGI272810[] = {{&S5625[13], 14}, {&S5625[28], 25}, {&S5625[75], 17}, {NULL}};
const Segment SGI272811[] = {{&S5626[48], 4}, {NULL}};
const Segment SGI272812[] = {{&S5626[13], 14}, {&S5626[28], 6}, {&S5626[35], 25}, {&S5626[82], 17}, {NULL}};
const Segment SGI272813[] = {{&S5627[48], 4}, {NULL}};
const Segment SGI272814[] = {{&S5627[13], 14}, {&S5627[28], 6}, {&S5627[35], 25}, {&S5627[82], 17}, {NULL}};
const Segment SGI272815[] = {{&S5628[41], 4}, {NULL}};
const Segment SGI272816[] = {{&S5628[13], 14}, {&S5628[28], 25}, {&S5628[75], 17}, {NULL}};
const Segment SGI272817[] = {{&S5629[41], 4}, {NULL}};
const Segment SGI272818[] = {{&S5629[13], 14}, {&S5629[28], 25}, {&S5629[75], 17}, {NULL}};
const Segment SGI272819[] = {{&S5630[48], 4}, {NULL}};
const Segment SGI272820[] = {{&S5630[13], 14}, {&S5630[28], 6}, {&S5630[35], 25}, {&S5630[82], 17}, {NULL}};
const Segment SGI272821[] = {{&S5631[41], 4}, {NULL}};
const Segment SGI272822[] = {{&S5631[13], 14}, {&S5631[28], 25}, {&S5631[75], 17}, {NULL}};
const Segment SGI272823[] = {{&S5632[48], 4}, {NULL}};
const Segment SGI272824[] = {{&S5632[13], 14}, {&S5632[28], 6}, {&S5632[35], 25}, {&S5632[82], 17}, {NULL}};
const Segment SGI272825[] = {{&S5633[48], 4}, {NULL}};
const Segment SGI272826[] = {{&S5633[13], 14}, {&S5633[28], 6}, {&S5633[35], 25}, {&S5633[82], 17}, {NULL}};
const Segment SGI272827[] = {{&S5634[41], 4}, {NULL}};
const Segment SGI272828[] = {{&S5634[13], 14}, {&S5634[28], 25}, {&S5634[75], 17}, {NULL}};
const Segment SGI272829[] = {{&S5635[41], 4}, {NULL}};
const Segment SGI272830[] = {{&S5635[13], 14}, {&S5635[28], 25}, {&S5635[75], 17}, {NULL}};
const Segment SGI272831[] = {{&S5636[48], 4}, {NULL}};
const Segment SGI272832[] = {{&S5636[13], 14}, {&S5636[28], 6}, {&S5636[35], 25}, {&S5636[82], 17}, {NULL}};
const Segment SGI272833[] = {{&S5638[48], 4}, {NULL}};
const Segment SGI272834[] = {{&S5638[13], 14}, {&S5638[28], 6}, {&S5638[35], 25}, {&S5638[82], 17}, {NULL}};
const Segment SGI272835[] = {{&S5640[41], 4}, {NULL}};
const Segment SGI272836[] = {{&S5640[13], 14}, {&S5640[28], 25}, {&S5640[75], 17}, {NULL}};
const Segment SGI272837[] = {{&S5642[41], 4}, {NULL}};
const Segment SGI272838[] = {{&S5642[13], 14}, {&S5642[28], 26}, {&S5642[76], 17}, {NULL}};
const Segment SGI272839[] = {{&S5643[48], 4}, {NULL}};
const Segment SGI272840[] = {{&S5643[13], 14}, {&S5643[28], 6}, {&S5643[35], 26}, {&S5643[83], 17}, {NULL}};
const Segment SGI272841[] = {{&S5644[41], 4}, {NULL}};
const Segment SGI272842[] = {{&S5644[13], 14}, {&S5644[28], 25}, {&S5644[75], 17}, {NULL}};
const Segment SGI272843[] = {{&S5648[48], 4}, {NULL}};
const Segment SGI272844[] = {{&S5648[13], 14}, {&S5648[28], 6}, {&S5648[35], 26}, {&S5648[83], 17}, {NULL}};
const Segment SGI272845[] = {{&S5650[41], 4}, {NULL}};
const Segment SGI272846[] = {{&S5650[13], 14}, {&S5650[28], 26}, {&S5650[76], 17}, {NULL}};
const Segment SGI272847[] = {{&S5750[48], 4}, {NULL}};
const Segment SGI272848[] = {{&S5750[13], 14}, {&S5750[28], 6}, {&S5750[35], 24}, {&S5750[81], 17}, {NULL}};
const Segment SGI272849[] = {{&S5751[48], 4}, {NULL}};
const Segment SGI272850[] = {{&S5751[13], 14}, {&S5751[28], 6}, {&S5751[35], 25}, {&S5751[82], 17}, {NULL}};
const Segment SGI272851[] = {{&S5752[48], 4}, {NULL}};
const Segment SGI272852[] = {{&S5752[13], 14}, {&S5752[28], 6}, {&S5752[35], 25}, {&S5752[82], 17}, {NULL}};
const Segment SGI272853[] = {{&S5753[48], 4}, {NULL}};
const Segment SGI272854[] = {{&S5753[13], 14}, {&S5753[28], 6}, {&S5753[35], 25}, {&S5753[82], 17}, {NULL}};
const Segment SGI272855[] = {{&S5754[48], 4}, {NULL}};
const Segment SGI272856[] = {{&S5754[13], 14}, {&S5754[28], 6}, {&S5754[35], 25}, {&S5754[82], 17}, {NULL}};
const Segment SGI272857[] = {{&S5755[41], 4}, {NULL}};
const Segment SGI272858[] = {{&S5755[13], 14}, {&S5755[28], 25}, {&S5755[75], 17}, {NULL}};
const Segment SGI272859[] = {{&S5756[48], 4}, {NULL}};
const Segment SGI272860[] = {{&S5756[13], 14}, {&S5756[28], 6}, {&S5756[35], 25}, {&S5756[82], 17}, {NULL}};
const Segment SGI272861[] = {{&S5765[48], 4}, {NULL}};
const Segment SGI272862[] = {{&S5765[13], 14}, {&S5765[28], 6}, {&S5765[35], 25}, {&S5765[82], 17}, {NULL}};
const Segment SGI272863[] = {{&S5767[41], 4}, {NULL}};
const Segment SGI272864[] = {{&S5767[13], 14}, {&S5767[28], 25}, {&S5767[75], 17}, {NULL}};
const Segment SGI272865[] = {{&S5780[48], 4}, {NULL}};
const Segment SGI272866[] = {{&S5780[13], 14}, {&S5780[28], 6}, {&S5780[35], 25}, {&S5780[82], 17}, {NULL}};
const Segment SGI272867[] = {{&S11541[74], 4}, {NULL}};
const Segment SGI272868[] = {{&S11541[13], 14}, {&S11541[48], 17}, {NULL}};
const Segment SGI272869[] = {{&S11542[81], 4}, {NULL}};
const Segment SGI272870[] = {{&S11542[13], 14}, {&S11542[28], 6}, {&S11542[55], 17}, {NULL}};
const Segment SGI272871[] = {{&S11543[81], 4}, {NULL}};
const Segment SGI272872[] = {{&S11543[13], 14}, {&S11543[28], 6}, {&S11543[55], 17}, {&S11543[107], 8}, {&S11543[118], 26}, {NULL}};
const Segment SGI272873[] = {{&S11544[74], 4}, {NULL}};
const Segment SGI272874[] = {{&S11544[13], 14}, {&S11544[48], 17}, {&S11544[100], 8}, {&S11544[111], 26}, {NULL}};
const Segment SGI272875[] = {{&S11545[81], 4}, {NULL}};
const Segment SGI272876[] = {{&S11545[13], 14}, {&S11545[28], 6}, {&S11545[55], 17}, {&S11545[124], 8}, {&S11545[135], 24}, {NULL}};
const Segment SGI272877[] = {{&S11546[74], 4}, {NULL}};
const Segment SGI272878[] = {{&S11546[13], 14}, {&S11546[48], 17}, {&S11546[117], 8}, {&S11546[128], 24}, {NULL}};
const Segment SGI272879[] = {{&S11547[86], 4}, {NULL}};
const Segment SGI272880[] = {{&S11547[13], 7}, {&S11547[21], 2}, {&S11547[24], 15}, {&S11547[60], 17}, {NULL}};
const Segment SGI272881[] = {{&S11548[13], 14}, {&S11548[46], 17}, {NULL}};
const Segment SGI272882[] = {{&S11549[74], 4}, {NULL}};
const Segment SGI272883[] = {{&S11549[13], 14}, {&S11549[48], 17}, {&S11549[117], 8}, {NULL}};
const Segment SGI272884[] = {{&S11550[81], 4}, {NULL}};
const Segment SGI272885[] = {{&S11550[13], 14}, {&S11550[28], 6}, {&S11550[55], 17}, {&S11550[124], 8}, {NULL}};
const Segment SGI272886[] = {{&S11551[93], 4}, {NULL}};
const Segment SGI272887[] = {{&S11551[13], 7}, {&S11551[21], 2}, {&S11551[24], 15}, {&S11551[40], 6}, {&S11551[67], 17}, {NULL}};
const Segment SGI272888[] = {{&S11552[81], 4}, {NULL}};
const Segment SGI272889[] = {{&S11552[13], 14}, {&S11552[28], 6}, {&S11552[55], 17}, {NULL}};
const Segment SGI272890[] = {{&S11553[80], 4}, {NULL}};
const Segment SGI272891[] = {{&S11553[13], 14}, {&S11553[28], 6}, {&S11553[55], 16}, {NULL}};
const Segment SGI272892[] = {{&S11554[73], 4}, {NULL}};
const Segment SGI272893[] = {{&S11554[13], 14}, {&S11554[48], 16}, {NULL}};
const Segment SGI272894[] = {{&S24783[38], 5}, {NULL}};
const Segment SGI272895[] = {{&S24783[100], 5}, {NULL}};
const Segment SGI272896[] = {{&S12152[82], 4}, {NULL}};
const Segment SGI272897[] = {{&S12152[13], 14}, {&S12152[28], 6}, {&S12152[56], 17}, {NULL}};
const Segment SGI272898[] = {{&S12153[75], 4}, {NULL}};
const Segment SGI272899[] = {{&S12153[13], 14}, {&S12153[49], 17}, {NULL}};
const Segment SGI272900[] = {{&S12154[82], 4}, {NULL}};
const Segment SGI272901[] = {{&S12154[13], 14}, {&S12154[28], 6}, {&S12154[56], 17}, {&S12154[109], 8}, {&S12154[120], 24}, {NULL}};
const Segment SGI272902[] = {{&S12155[75], 4}, {NULL}};
const Segment SGI272903[] = {{&S12155[13], 14}, {&S12155[49], 17}, {&S12155[101], 8}, {&S12155[112], 24}, {NULL}};
const Segment SGI272904[] = {{&S12156[82], 4}, {NULL}};
const Segment SGI272905[] = {{&S12156[13], 14}, {&S12156[28], 6}, {&S12156[56], 17}, {NULL}};
const Segment SGI272906[] = {{&S12157[93], 4}, {NULL}};
const Segment SGI272907[] = {{&S12157[13], 7}, {&S12157[21], 2}, {&S12157[24], 15}, {&S12157[40], 6}, {&S12157[67], 17}, {NULL}};
const Segment SGI272908[] = {{&S12158[82], 4}, {NULL}};
const Segment SGI272909[] = {{&S12158[13], 14}, {&S12158[28], 6}, {&S12158[35], 19}, {&S12158[55], 6}, {&S12158[62], 11}, {&S12158[74], 32}, {NULL}};
const Segment SGI272910[] = {{&S12159[87], 4}, {NULL}};
const Segment SGI272911[] = {{&S12159[13], 10}, {&S12159[24], 15}, {&S12159[40], 19}, {&S12159[60], 18}, {&S12159[79], 32}, {NULL}};
const Segment SGI272912[] = {{&S12160[82], 4}, {NULL}};
const Segment SGI272913[] = {{&S12160[13], 14}, {&S12160[28], 6}, {&S12160[56], 17}, {&S12160[109], 8}, {NULL}};
const Segment SGI272914[] = {{&S12161[75], 4}, {NULL}};
const Segment SGI272915[] = {{&S12161[13], 14}, {&S12161[49], 17}, {NULL}};
const Segment SGI272916[] = {{&S12162[75], 4}, {NULL}};
const Segment SGI272917[] = {{&S12162[13], 14}, {&S12162[49], 17}, {&S12162[102], 8}, {NULL}};
const Segment SGI272918[] = {{&S12163[82], 4}, {NULL}};
const Segment SGI272919[] = {{&S12163[13], 14}, {&S12163[28], 6}, {&S12163[56], 17}, {NULL}};
const Segment SGI272920[] = {{&S12164[82], 4}, {NULL}};
const Segment SGI272921[] = {{&S12164[13], 14}, {&S12164[28], 6}, {&S12164[56], 17}, {NULL}};
const Segment SGI272922[] = {{&S12165[82], 4}, {NULL}};
const Segment SGI272923[] = {{&S12165[13], 14}, {&S12165[28], 6}, {&S12165[56], 17}, {NULL}};
const Segment SGI272924[] = {{&S12166[83], 4}, {NULL}};
const Segment SGI272925[] = {{&S12166[13], 14}, {&S12166[28], 7}, {&S12166[57], 17}, {NULL}};
const Segment SGI272926[] = {{&S12167[82], 4}, {NULL}};
const Segment SGI272927[] = {{&S12167[13], 14}, {&S12167[28], 6}, {&S12167[56], 17}, {NULL}};
const Segment SGI272928[] = {{&S12168[82], 4}, {NULL}};
const Segment SGI272929[] = {{&S12168[13], 14}, {&S12168[28], 6}, {&S12168[56], 17}, {&S12168[109], 8}, {&S12168[120], 26}, {NULL}};
const Segment SGI272930[] = {{&S12169[82], 4}, {NULL}};
const Segment SGI272931[] = {{&S12169[13], 14}, {&S12169[28], 6}, {&S12169[56], 17}, {NULL}};
const Segment SGI272932[] = {{&S12170[81], 4}, {NULL}};
const Segment SGI272933[] = {{&S12170[13], 14}, {&S12170[28], 6}, {&S12170[56], 16}, {NULL}};
const Segment SGI272934[] = {{&S12171[81], 4}, {NULL}};
const Segment SGI272935[] = {{&S12171[13], 14}, {&S12171[28], 6}, {&S12171[56], 16}, {NULL}};
const Segment SGI272936[] = {{&S12172[94], 4}, {NULL}};
const Segment SGI272937[] = {{&S12172[13], 10}, {&S12172[24], 15}, {&S12172[40], 6}, {&S12172[47], 19}, {&S12172[67], 18}, {&S12172[86], 32}, {NULL}};
const Segment SGI272938[] = {{&S12173[100], 4}, {NULL}};
const Segment SGI272939[] = {{&S12173[13], 10}, {&S12173[24], 15}, {&S12173[40], 19}, {&S12173[60], 12}, {&S12173[73], 18}, {&S12173[92], 32}, {NULL}};
const Segment SGI272940[] = {{&S25535[38], 5}, {NULL}};
const Segment SGI272941[] = {{&S25535[97], 5}, {NULL}};
const Segment SGI272942[] = {{&S15488[74], 4}, {&S15488[97], 3}, {NULL}};
const Segment SGI272943[] = {{&S15488[13], 14}, {&S15488[48], 17}, {NULL}};
const Segment SGI272944[] = {{&S15489[81], 4}, {&S15489[104], 3}, {NULL}};
const Segment SGI272945[] = {{&S15489[13], 14}, {&S15489[28], 6}, {&S15489[55], 17}, {NULL}};
const Segment SGI272946[] = {{&S8906[74], 4}, {NULL}};
const Segment SGI272947[] = {{&S8906[13], 14}, {&S8906[48], 17}, {NULL}};
const Segment SGI272948[] = {{&S8907[81], 4}, {NULL}};
const Segment SGI272949[] = {{&S8907[13], 14}, {&S8907[28], 6}, {&S8907[55], 17}, {NULL}};
const Segment SGI272950[] = {{&S8908[74], 4}, {NULL}};
const Segment SGI272951[] = {{&S8908[13], 14}, {&S8908[48], 17}, {&S8908[100], 8}, {&S8908[111], 24}, {NULL}};
const Segment SGI272952[] = {{&S8909[81], 4}, {NULL}};
const Segment SGI272953[] = {{&S8909[13], 14}, {&S8909[28], 6}, {&S8909[55], 17}, {&S8909[107], 8}, {&S8909[118], 26}, {NULL}};
const Segment SGI272954[] = {{&S8910[74], 4}, {NULL}};
const Segment SGI272955[] = {{&S8910[13], 14}, {&S8910[48], 17}, {&S8910[100], 8}, {&S8910[111], 26}, {NULL}};
const Segment SGI272956[] = {{&S8911[81], 4}, {NULL}};
const Segment SGI272957[] = {{&S8911[13], 14}, {&S8911[28], 6}, {&S8911[55], 17}, {&S8911[107], 8}, {&S8911[118], 24}, {NULL}};
const Segment SGI272958[] = {{&S8912[74], 4}, {NULL}};
const Segment SGI272959[] = {{&S8912[13], 14}, {&S8912[48], 17}, {&S8912[117], 35}, {&S8912[153], 22}, {NULL}};
const Segment SGI272960[] = {{&S8913[74], 4}, {NULL}};
const Segment SGI272961[] = {{&S8913[13], 14}, {&S8913[48], 17}, {&S8913[100], 8}, {NULL}};
const Segment SGI272962[] = {{&S8914[81], 4}, {NULL}};
const Segment SGI272963[] = {{&S8914[13], 14}, {&S8914[28], 6}, {&S8914[55], 17}, {&S8914[107], 8}, {NULL}};
const Segment SGI272964[] = {{&S8915[74], 4}, {NULL}};
const Segment SGI272965[] = {{&S8915[13], 14}, {&S8915[48], 17}, {&S8915[117], 35}, {&S8915[153], 23}, {NULL}};
const Segment SGI272966[] = {{&S8916[86], 4}, {NULL}};
const Segment SGI272967[] = {{&S8916[13], 7}, {&S8916[21], 2}, {&S8916[24], 15}, {&S8916[60], 17}, {NULL}};
const Segment SGI272968[] = {{&S8917[74], 4}, {NULL}};
const Segment SGI272969[] = {{&S8917[13], 14}, {&S8917[48], 17}, {&S8917[117], 35}, {&S8917[153], 20}, {NULL}};
const Segment SGI272970[] = {{&S8918[80], 4}, {NULL}};
const Segment SGI272971[] = {{&S8918[13], 14}, {&S8918[28], 6}, {&S8918[55], 16}, {NULL}};
const Segment SGI272972[] = {{&S8919[94], 4}, {NULL}};
const Segment SGI272973[] = {{&S8919[13], 7}, {&S8919[21], 2}, {&S8919[24], 15}, {&S8919[40], 6}, {&S8919[68], 17}, {NULL}};
const Segment SGI272974[] = {{&S8943[73], 4}, {NULL}};
const Segment SGI272975[] = {{&S8943[13], 14}, {&S8943[49], 17}, {NULL}};
const Segment SGI272976[] = {{&S8943[68], 9}, {NULL}};
const Segment SGI272977[] = {{&S8944[75], 4}, {NULL}};
const Segment SGI272978[] = {{&S8944[13], 14}, {&S8944[49], 17}, {NULL}};
const Segment SGI272979[] = {{&S8945[82], 4}, {NULL}};
const Segment SGI272980[] = {{&S8945[13], 14}, {&S8945[28], 6}, {&S8945[56], 17}, {NULL}};
const Segment SGI272981[] = {{&S8946[75], 4}, {NULL}};
const Segment SGI272982[] = {{&S8946[13], 14}, {&S8946[49], 17}, {NULL}};
const Segment SGI272983[] = {{&S8947[82], 4}, {NULL}};
const Segment SGI272984[] = {{&S8947[13], 14}, {&S8947[28], 6}, {&S8947[56], 17}, {NULL}};
const Segment SGI272985[] = {{&S8948[75], 4}, {NULL}};
const Segment SGI272986[] = {{&S8948[13], 14}, {&S8948[49], 17}, {&S8948[103], 8}, {NULL}};
const Segment SGI272987[] = {{&S8949[75], 4}, {NULL}};
const Segment SGI272988[] = {{&S8949[13], 14}, {&S8949[49], 17}, {NULL}};
const Segment SGI272989[] = {{&S8950[75], 4}, {NULL}};
const Segment SGI272990[] = {{&S8950[13], 14}, {&S8950[49], 17}, {&S8950[103], 8}, {&S8950[114], 26}, {NULL}};
const Segment SGI272991[] = {{&S8951[82], 4}, {NULL}};
const Segment SGI272992[] = {{&S8951[13], 14}, {&S8951[28], 6}, {&S8951[56], 17}, {NULL}};
const Segment SGI272993[] = {{&S8952[75], 4}, {NULL}};
const Segment SGI272994[] = {{&S8952[13], 14}, {&S8952[49], 17}, {NULL}};
const Segment SGI272995[] = {{&S8953[80], 4}, {NULL}};
const Segment SGI272996[] = {{&S8953[13], 14}, {&S8953[28], 6}, {&S8953[56], 17}, {NULL}};
const Segment SGI272997[] = {{&S8953[75], 9}, {NULL}};
const Segment SGI272998[] = {{&S8954[75], 4}, {NULL}};
const Segment SGI272999[] = {{&S8954[13], 14}, {&S8954[49], 17}, {NULL}};
const Segment SGI273000[] = {{&S8955[73], 4}, {NULL}};
const Segment SGI273001[] = {{&S8955[13], 14}, {&S8955[49], 17}, {NULL}};
const Segment SGI273002[] = {{&S8955[68], 9}, {NULL}};
const Segment SGI273003[] = {{&S8956[94], 4}, {NULL}};
const Segment SGI273004[] = {{&S8956[13], 7}, {&S8956[21], 2}, {&S8956[24], 15}, {&S8956[40], 6}, {&S8956[68], 17}, {NULL}};
const Segment SGI273005[] = {{&S8957[82], 4}, {NULL}};
const Segment SGI273006[] = {{&S8957[13], 14}, {&S8957[28], 6}, {&S8957[56], 17}, {NULL}};
const Segment SGI273007[] = {{&S8958[75], 4}, {NULL}};
const Segment SGI273008[] = {{&S8958[13], 14}, {&S8958[49], 17}, {&S8958[119], 35}, {&S8958[155], 20}, {NULL}};
const Segment SGI273009[] = {{&S8959[75], 5}, {NULL}};
const Segment SGI273010[] = {{&S8959[13], 14}, {&S8959[49], 17}, {NULL}};
const Segment SGI273011[] = {{&S27159[38], 5}, {NULL}};
const Segment SGI273012[] = {{&S27159[100], 5}, {NULL}};
const Segment SGI273013[] = {{&S9013[93], 4}, {NULL}};
const Segment SGI273014[] = {{&S9013[13], 9}, {&S9013[23], 22}, {&S9013[67], 17}, {NULL}};
const Segment SGI273015[] = {{&S9014[93], 4}, {NULL}};
const Segment SGI273016[] = {{&S9014[13], 9}, {&S9014[23], 22}, {&S9014[67], 17}, {NULL}};
const Segment SGI273017[] = {{&S9015[93], 4}, {NULL}};
const Segment SGI273018[] = {{&S9015[13], 9}, {&S9015[23], 22}, {&S9015[67], 17}, {NULL}};
const Segment SGI273019[] = {{&S9016[93], 4}, {NULL}};
const Segment SGI273020[] = {{&S9016[13], 9}, {&S9016[23], 22}, {&S9016[67], 17}, {NULL}};
const Segment SGI273021[] = {{&S9017[93], 4}, {NULL}};
const Segment SGI273022[] = {{&S9017[13], 9}, {&S9017[23], 22}, {&S9017[67], 17}, {NULL}};
const Segment SGI273023[] = {{&S9018[93], 4}, {NULL}};
const Segment SGI273024[] = {{&S9018[13], 9}, {&S9018[23], 22}, {&S9018[67], 17}, {&S9018[120], 8}, {NULL}};
const Segment SGI273025[] = {{&S9019[93], 4}, {NULL}};
const Segment SGI273026[] = {{&S9019[13], 9}, {&S9019[23], 22}, {&S9019[67], 17}, {&S9019[120], 8}, {&S9019[131], 26}, {NULL}};
const Segment SGI273027[] = {{&S9020[93], 4}, {NULL}};
const Segment SGI273028[] = {{&S9020[13], 9}, {&S9020[23], 22}, {&S9020[67], 17}, {&S9020[120], 8}, {NULL}};
const Segment SGI273029[] = {{&S9021[93], 4}, {NULL}};
const Segment SGI273030[] = {{&S9021[13], 9}, {&S9021[23], 22}, {&S9021[67], 17}, {NULL}};
const Segment SGI273031[] = {{&S9022[91], 4}, {NULL}};
const Segment SGI273032[] = {{&S9022[13], 9}, {&S9022[23], 22}, {&S9022[67], 17}, {NULL}};
const Segment SGI273033[] = {{&S9022[86], 9}, {NULL}};
const Segment SGI273034[] = {{&S9023[93], 4}, {NULL}};
const Segment SGI273035[] = {{&S9023[13], 9}, {&S9023[23], 22}, {&S9023[67], 17}, {&S9023[120], 8}, {NULL}};
const Segment SGI273036[] = {{&S9024[93], 4}, {NULL}};
const Segment SGI273037[] = {{&S9024[13], 9}, {&S9024[23], 22}, {&S9024[67], 17}, {&S9024[120], 8}, {&S9024[131], 26}, {NULL}};
const Segment SGI273038[] = {{&S9146[48], 4}, {NULL}};
const Segment SGI273039[] = {{&S9146[13], 14}, {&S9146[28], 6}, {&S9146[35], 25}, {&S9146[82], 17}, {NULL}};
const Segment SGI273040[] = {{&S9147[48], 4}, {NULL}};
const Segment SGI273041[] = {{&S9147[13], 14}, {&S9147[28], 6}, {&S9147[35], 25}, {&S9147[82], 17}, {NULL}};
const Segment SGI273042[] = {{&S9148[41], 4}, {NULL}};
const Segment SGI273043[] = {{&S9148[13], 14}, {&S9148[28], 25}, {&S9148[75], 17}, {NULL}};
const Segment SGI273044[] = {{&S9149[48], 4}, {NULL}};
const Segment SGI273045[] = {{&S9149[13], 14}, {&S9149[28], 6}, {&S9149[35], 25}, {&S9149[82], 17}, {NULL}};
const Segment SGI273046[] = {{&S9150[48], 4}, {NULL}};
const Segment SGI273047[] = {{&S9150[13], 14}, {&S9150[28], 6}, {&S9150[35], 25}, {&S9150[82], 17}, {NULL}};
const Segment SGI273048[] = {{&S9151[48], 4}, {NULL}};
const Segment SGI273049[] = {{&S9151[13], 14}, {&S9151[28], 6}, {&S9151[35], 25}, {&S9151[82], 17}, {NULL}};
const Segment SGI273050[] = {{&S9152[48], 4}, {NULL}};
const Segment SGI273051[] = {{&S9152[13], 14}, {&S9152[28], 6}, {&S9152[35], 25}, {&S9152[82], 17}, {NULL}};
const Segment SGI273052[] = {{&S9153[48], 4}, {NULL}};
const Segment SGI273053[] = {{&S9153[13], 14}, {&S9153[28], 6}, {&S9153[35], 25}, {&S9153[82], 17}, {NULL}};
const Segment SGI273054[] = {{&S9154[41], 4}, {NULL}};
const Segment SGI273055[] = {{&S9154[13], 14}, {&S9154[28], 25}, {&S9154[75], 17}, {NULL}};
const Segment SGI273056[] = {{&S9156[41], 4}, {NULL}};
const Segment SGI273057[] = {{&S9156[13], 14}, {&S9156[28], 25}, {&S9156[75], 17}, {NULL}};
const Segment SGI273058[] = {{&S9157[48], 4}, {NULL}};
const Segment SGI273059[] = {{&S9157[13], 14}, {&S9157[28], 6}, {&S9157[35], 25}, {&S9157[82], 17}, {NULL}};
const Segment SGI273060[] = {{&S9159[48], 4}, {NULL}};
const Segment SGI273061[] = {{&S9159[13], 14}, {&S9159[28], 6}, {&S9159[35], 25}, {&S9159[82], 17}, {NULL}};
const Segment SGI273062[] = {{&S9160[41], 4}, {NULL}};
const Segment SGI273063[] = {{&S9160[13], 14}, {&S9160[28], 25}, {&S9160[75], 17}, {NULL}};
const Segment SGI273064[] = {{&S9165[48], 4}, {NULL}};
const Segment SGI273065[] = {{&S9165[13], 14}, {&S9165[28], 6}, {&S9165[35], 25}, {&S9165[82], 17}, {NULL}};
const Segment SGI273066[] = {{&S9166[41], 4}, {NULL}};
const Segment SGI273067[] = {{&S9166[13], 14}, {&S9166[28], 25}, {&S9166[75], 17}, {NULL}};
const Segment SGI273068[] = {{&S9207[80], 4}, {NULL}};
const Segment SGI273069[] = {{&S9207[13], 14}, {&S9207[28], 6}, {&S9207[56], 17}, {NULL}};
const Segment SGI273070[] = {{&S9207[75], 9}, {NULL}};
const Segment SGI273071[] = {{&S9208[73], 4}, {NULL}};
const Segment SGI273072[] = {{&S9208[13], 14}, {&S9208[49], 17}, {NULL}};
const Segment SGI273073[] = {{&S9208[68], 9}, {NULL}};
const Segment SGI273074[] = {{&S9209[82], 4}, {NULL}};
const Segment SGI273075[] = {{&S9209[13], 14}, {&S9209[28], 6}, {&S9209[56], 17}, {NULL}};
const Segment SGI273076[] = {{&S9210[75], 4}, {NULL}};
const Segment SGI273077[] = {{&S9210[13], 14}, {&S9210[49], 17}, {NULL}};
const Segment SGI273078[] = {{&S9211[82], 4}, {NULL}};
const Segment SGI273079[] = {{&S9211[13], 14}, {&S9211[28], 6}, {&S9211[56], 17}, {NULL}};
const Segment SGI273080[] = {{&S9212[75], 4}, {NULL}};
const Segment SGI273081[] = {{&S9212[13], 14}, {&S9212[49], 17}, {NULL}};
const Segment SGI273082[] = {{&S9213[82], 4}, {NULL}};
const Segment SGI273083[] = {{&S9213[13], 14}, {&S9213[28], 6}, {&S9213[56], 17}, {&S9213[110], 8}, {NULL}};
const Segment SGI273084[] = {{&S9214[82], 4}, {NULL}};
const Segment SGI273085[] = {{&S9214[13], 14}, {&S9214[28], 6}, {&S9214[56], 17}, {&S9214[110], 8}, {&S9214[121], 26}, {NULL}};
const Segment SGI273086[] = {{&S9215[82], 4}, {NULL}};
const Segment SGI273087[] = {{&S9215[13], 14}, {&S9215[28], 6}, {&S9215[56], 17}, {NULL}};
const Segment SGI273088[] = {{&S9216[75], 4}, {NULL}};
const Segment SGI273089[] = {{&S9216[13], 14}, {&S9216[49], 17}, {NULL}};
const Segment SGI273090[] = {{&S9217[75], 4}, {NULL}};
const Segment SGI273091[] = {{&S9217[13], 14}, {&S9217[49], 17}, {&S9217[103], 8}, {NULL}};
const Segment SGI273092[] = {{&S9218[75], 4}, {NULL}};
const Segment SGI273093[] = {{&S9218[13], 14}, {&S9218[49], 17}, {&S9218[103], 8}, {&S9218[114], 26}, {NULL}};
const Segment SGI273094[] = {{&S14989[81], 4}, {NULL}};
const Segment SGI273095[] = {{&S14989[13], 14}, {&S14989[28], 6}, {&S14989[55], 17}, {NULL}};
const Segment SGI273096[] = {{&S14990[74], 4}, {NULL}};
const Segment SGI273097[] = {{&S14990[13], 14}, {&S14990[48], 17}, {NULL}};
const Segment SGI273098[] = {{&S14991[81], 4}, {NULL}};
const Segment SGI273099[] = {{&S14991[13], 14}, {&S14991[28], 6}, {&S14991[55], 17}, {&S14991[107], 8}, {&S14991[118], 26}, {NULL}};
const Segment SGI273100[] = {{&S14992[81], 4}, {NULL}};
const Segment SGI273101[] = {{&S14992[13], 14}, {&S14992[28], 6}, {&S14992[55], 17}, {&S14992[107], 8}, {&S14992[118], 24}, {NULL}};
const Segment SGI273102[] = {{&S14993[74], 4}, {NULL}};
const Segment SGI273103[] = {{&S14993[13], 14}, {&S14993[48], 17}, {&S14993[100], 8}, {&S14993[111], 24}, {NULL}};
const Segment SGI273104[] = {{&S14994[93], 4}, {NULL}};
const Segment SGI273105[] = {{&S14994[13], 7}, {&S14994[21], 2}, {&S14994[24], 15}, {&S14994[40], 6}, {&S14994[67], 17}, {NULL}};
const Segment SGI273106[] = {{&S14995[74], 4}, {NULL}};
const Segment SGI273107[] = {{&S14995[13], 14}, {&S14995[48], 17}, {&S14995[100], 8}, {&S14995[111], 26}, {NULL}};
const Segment SGI273108[] = {{&S14996[74], 4}, {NULL}};
const Segment SGI273109[] = {{&S14996[13], 14}, {&S14996[48], 17}, {&S14996[100], 8}, {NULL}};
const Segment SGI273110[] = {{&S14997[81], 4}, {NULL}};
const Segment SGI273111[] = {{&S14997[13], 14}, {&S14997[28], 6}, {&S14997[55], 17}, {&S14997[107], 8}, {NULL}};
const Segment SGI273112[] = {{&S9625[81], 4}, {NULL}};
const Segment SGI273113[] = {{&S9625[13], 14}, {&S9625[28], 6}, {&S9625[56], 16}, {NULL}};
const Segment SGI273114[] = {{&S9626[75], 4}, {NULL}};
const Segment SGI273115[] = {{&S9626[13], 14}, {&S9626[49], 17}, {&S9626[102], 8}, {NULL}};
const Segment SGI273116[] = {{&S9627[82], 4}, {NULL}};
const Segment SGI273117[] = {{&S9627[13], 14}, {&S9627[28], 6}, {&S9627[56], 17}, {&S9627[109], 8}, {&S9627[120], 26}, {NULL}};
const Segment SGI273118[] = {{&S9628[82], 4}, {NULL}};
const Segment SGI273119[] = {{&S9628[13], 14}, {&S9628[28], 6}, {&S9628[56], 17}, {NULL}};
const Segment SGI273120[] = {{&S9629[75], 4}, {NULL}};
const Segment SGI273121[] = {{&S9629[13], 14}, {&S9629[49], 17}, {NULL}};
const Segment SGI273122[] = {{&S9630[82], 4}, {NULL}};
const Segment SGI273123[] = {{&S9630[13], 14}, {&S9630[28], 6}, {&S9630[56], 17}, {&S9630[109], 8}, {NULL}};
const Segment SGI273124[] = {{&S9631[80], 4}, {NULL}};
const Segment SGI273125[] = {{&S9631[13], 14}, {&S9631[28], 6}, {&S9631[56], 17}, {NULL}};
const Segment SGI273126[] = {{&S9631[75], 9}, {NULL}};
const Segment SGI273127[] = {{&S9632[82], 4}, {NULL}};
const Segment SGI273128[] = {{&S9632[13], 14}, {&S9632[28], 6}, {&S9632[56], 17}, {&S9632[109], 8}, {&S9632[109], 8}, {NULL}};
const Segment SGI273129[] = {{&S9633[75], 4}, {NULL}};
const Segment SGI273130[] = {{&S9633[13], 14}, {&S9633[49], 17}, {&S9633[102], 8}, {&S9633[113], 26}, {NULL}};
const Segment SGI273131[] = {{&S9634[73], 4}, {NULL}};
const Segment SGI273132[] = {{&S9634[13], 14}, {&S9634[49], 17}, {NULL}};
const Segment SGI273133[] = {{&S9634[68], 9}, {NULL}};
const Segment SGI273134[] = {{&S9637[81], 4}, {NULL}};
const Segment SGI273135[] = {{&S9637[13], 14}, {&S9637[28], 6}, {&S9637[56], 16}, {NULL}};
const Segment SGI273136[] = {{&S9638[82], 4}, {NULL}};
const Segment SGI273137[] = {{&S9638[13], 14}, {&S9638[28], 6}, {&S9638[56], 17}, {&S9638[109], 8}, {&S9638[109], 8}, {NULL}};
const Segment SGI273138[] = {{&S9639[82], 4}, {NULL}};
const Segment SGI273139[] = {{&S9639[13], 14}, {&S9639[28], 6}, {&S9639[56], 17}, {NULL}};
const Segment SGI273140[] = {{&S9640[75], 4}, {NULL}};
const Segment SGI273141[] = {{&S9640[13], 14}, {&S9640[49], 17}, {NULL}};
const Segment SGI273142[] = {{&S9641[75], 4}, {NULL}};
const Segment SGI273143[] = {{&S9641[13], 14}, {&S9641[49], 17}, {&S9641[102], 8}, {&S9641[113], 26}, {NULL}};
const Segment SGI273144[] = {{&S9642[82], 4}, {NULL}};
const Segment SGI273145[] = {{&S9642[13], 14}, {&S9642[28], 6}, {&S9642[56], 17}, {&S9642[109], 8}, {NULL}};
const Segment SGI273146[] = {{&S9643[82], 4}, {NULL}};
const Segment SGI273147[] = {{&S9643[13], 14}, {&S9643[28], 6}, {&S9643[56], 17}, {NULL}};
const Segment SGI273148[] = {{&S9644[75], 4}, {NULL}};
const Segment SGI273149[] = {{&S9644[13], 14}, {&S9644[49], 17}, {NULL}};
const Segment SGI273150[] = {{&S9645[82], 4}, {NULL}};
const Segment SGI273151[] = {{&S9645[13], 14}, {&S9645[28], 6}, {&S9645[56], 17}, {&S9645[109], 8}, {&S9645[120], 26}, {NULL}};
const Segment SGI273152[] = {{&S9646[75], 4}, {NULL}};
const Segment SGI273153[] = {{&S9646[13], 14}, {&S9646[49], 17}, {&S9646[102], 8}, {NULL}};
const Segment SGI273154[] = {{&S9647[93], 4}, {NULL}};
const Segment SGI273155[] = {{&S9647[13], 9}, {&S9647[23], 22}, {&S9647[67], 17}, {NULL}};
const Segment SGI273156[] = {{&S9648[93], 4}, {NULL}};
const Segment SGI273157[] = {{&S9648[13], 9}, {&S9648[23], 22}, {&S9648[67], 17}, {&S9648[120], 8}, {&S9648[131], 26}, {NULL}};
const Segment SGI273158[] = {{&S9649[93], 4}, {NULL}};
const Segment SGI273159[] = {{&S9649[13], 9}, {&S9649[23], 22}, {&S9649[67], 17}, {&S9649[120], 8}, {NULL}};
const Segment SGI273160[] = {{&S9650[93], 4}, {NULL}};
const Segment SGI273161[] = {{&S9650[13], 9}, {&S9650[23], 22}, {&S9650[67], 17}, {&S9650[120], 8}, {NULL}};
const Segment SGI273162[] = {{&S9651[91], 4}, {NULL}};
const Segment SGI273163[] = {{&S9651[13], 9}, {&S9651[23], 22}, {&S9651[67], 17}, {NULL}};
const Segment SGI273164[] = {{&S9651[86], 9}, {NULL}};
const Segment SGI273165[] = {{&S9652[93], 4}, {NULL}};
const Segment SGI273166[] = {{&S9652[13], 9}, {&S9652[23], 22}, {&S9652[67], 17}, {NULL}};
const Segment SGI273167[] = {{&S9653[93], 4}, {NULL}};
const Segment SGI273168[] = {{&S9653[13], 9}, {&S9653[23], 22}, {&S9653[67], 17}, {NULL}};
const Segment SGI273169[] = {{&S9654[93], 4}, {NULL}};
const Segment SGI273170[] = {{&S9654[13], 9}, {&S9654[23], 22}, {&S9654[67], 17}, {NULL}};
const Segment SGI273171[] = {{&S9655[93], 4}, {NULL}};
const Segment SGI273172[] = {{&S9655[13], 9}, {&S9655[23], 22}, {&S9655[67], 17}, {NULL}};
const Segment SGI273173[] = {{&S9656[93], 4}, {NULL}};
const Segment SGI273174[] = {{&S9656[13], 9}, {&S9656[23], 22}, {&S9656[67], 17}, {&S9656[121], 8}, {NULL}};
const Segment SGI273175[] = {{&S9657[93], 4}, {NULL}};
const Segment SGI273176[] = {{&S9657[13], 9}, {&S9657[23], 22}, {&S9657[67], 17}, {NULL}};
const Segment SGI273177[] = {{&S9658[93], 4}, {NULL}};
const Segment SGI273178[] = {{&S9658[13], 9}, {&S9658[23], 22}, {&S9658[67], 17}, {&S9658[121], 8}, {&S9658[132], 26}, {NULL}};
const Segment SGI273179[] = {{&S9659[91], 4}, {NULL}};
const Segment SGI273180[] = {{&S9659[13], 9}, {&S9659[23], 22}, {&S9659[67], 17}, {NULL}};
const Segment SGI273181[] = {{&S9659[86], 9}, {NULL}};
const Segment SGI273182[] = {{&S9660[91], 4}, {NULL}};
const Segment SGI273183[] = {{&S9660[13], 9}, {&S9660[23], 22}, {&S9660[67], 17}, {NULL}};
const Segment SGI273184[] = {{&S9660[86], 9}, {NULL}};
const Segment SGI273185[] = {{&S9661[91], 4}, {NULL}};
const Segment SGI273186[] = {{&S9661[13], 9}, {&S9661[23], 22}, {&S9661[67], 17}, {NULL}};
const Segment SGI273187[] = {{&S9661[86], 9}, {NULL}};
const Segment SGI273188[] = {{&S9662[75], 4}, {NULL}};
const Segment SGI273189[] = {{&S9662[13], 14}, {&S9662[49], 17}, {&S9662[119], 35}, {&S9662[155], 23}, {NULL}};
const Segment SGI273190[] = {{&S9663[75], 4}, {NULL}};
const Segment SGI273191[] = {{&S9663[13], 14}, {&S9663[49], 17}, {&S9663[102], 8}, {&S9663[113], 26}, {NULL}};
const Segment SGI273192[] = {{&S9664[75], 4}, {NULL}};
const Segment SGI273193[] = {{&S9664[13], 14}, {&S9664[49], 17}, {NULL}};
const Segment SGI273194[] = {{&S9665[82], 4}, {NULL}};
const Segment SGI273195[] = {{&S9665[13], 14}, {&S9665[28], 6}, {&S9665[56], 17}, {NULL}};
const Segment SGI273196[] = {{&S9666[75], 4}, {NULL}};
const Segment SGI273197[] = {{&S9666[13], 14}, {&S9666[49], 17}, {NULL}};
const Segment SGI273198[] = {{&S9667[75], 4}, {NULL}};
const Segment SGI273199[] = {{&S9667[13], 14}, {&S9667[49], 17}, {&S9667[102], 8}, {NULL}};
const Segment SGI273200[] = {{&S9668[75], 4}, {NULL}};
const Segment SGI273201[] = {{&S9668[13], 14}, {&S9668[49], 17}, {&S9668[102], 8}, {&S9668[102], 8}, {NULL}};
const Segment SGI273202[] = {{&S9669[75], 4}, {NULL}};
const Segment SGI273203[] = {{&S9669[13], 14}, {&S9669[49], 17}, {NULL}};
const Segment SGI273204[] = {{&S27802[38], 5}, {NULL}};
const Segment SGI273205[] = {{&S27802[99], 5}, {NULL}};
const Segment SGI273206[] = {{&S9670[82], 4}, {NULL}};
const Segment SGI273207[] = {{&S9670[13], 14}, {&S9670[28], 6}, {&S9670[56], 17}, {NULL}};
const Segment SGI273208[] = {{&S9671[82], 4}, {NULL}};
const Segment SGI273209[] = {{&S9671[13], 14}, {&S9671[28], 6}, {&S9671[56], 17}, {NULL}};
const Segment SGI273210[] = {{&S9672[75], 4}, {NULL}};
const Segment SGI273211[] = {{&S9672[13], 14}, {&S9672[49], 17}, {NULL}};
const Segment SGI273212[] = {{&S9673[75], 4}, {NULL}};
const Segment SGI273213[] = {{&S9673[13], 14}, {&S9673[49], 17}, {NULL}};
const Segment SGI273214[] = {{&S9674[75], 4}, {NULL}};
const Segment SGI273215[] = {{&S9674[13], 14}, {&S9674[49], 17}, {NULL}};
const Segment SGI273216[] = {{&S9675[75], 4}, {NULL}};
const Segment SGI273217[] = {{&S9675[13], 14}, {&S9675[49], 17}, {NULL}};
const Segment SGI273218[] = {{&S9676[82], 4}, {NULL}};
const Segment SGI273219[] = {{&S9676[13], 14}, {&S9676[28], 6}, {&S9676[56], 17}, {NULL}};
const Segment SGI273220[] = {{&S9677[75], 4}, {NULL}};
const Segment SGI273221[] = {{&S9677[13], 14}, {&S9677[49], 17}, {&S9677[102], 8}, {NULL}};
const Segment SGI273222[] = {{&S9678[74], 4}, {NULL}};
const Segment SGI273223[] = {{&S9678[13], 14}, {&S9678[49], 16}, {NULL}};
const Segment SGI273224[] = {{&S9679[75], 4}, {NULL}};
const Segment SGI273225[] = {{&S9679[13], 14}, {&S9679[49], 17}, {NULL}};
const Segment SGI273226[] = {{&S9680[82], 4}, {NULL}};
const Segment SGI273227[] = {{&S9680[13], 14}, {&S9680[28], 6}, {&S9680[56], 17}, {NULL}};
const Segment SGI273228[] = {{&S9681[75], 4}, {NULL}};
const Segment SGI273229[] = {{&S9681[13], 14}, {&S9681[49], 17}, {&S9681[102], 8}, {&S9681[113], 26}, {NULL}};
const Segment SGI273230[] = {{&S9682[75], 4}, {NULL}};
const Segment SGI273231[] = {{&S9682[13], 14}, {&S9682[49], 17}, {NULL}};
const Segment SGI273232[] = {{&S9683[75], 4}, {NULL}};
const Segment SGI273233[] = {{&S9683[13], 14}, {&S9683[49], 17}, {NULL}};
const Segment SGI273234[] = {{&S9684[82], 4}, {NULL}};
const Segment SGI273235[] = {{&S9684[13], 14}, {&S9684[28], 6}, {&S9684[56], 17}, {NULL}};
const Segment SGI273236[] = {{&S9685[75], 4}, {NULL}};
const Segment SGI273237[] = {{&S9685[13], 14}, {&S9685[49], 17}, {&S9685[119], 35}, {&S9685[155], 20}, {&S9685[119], 35}, {&S9685[230], 20}, {NULL}};
const Segment SGI273238[] = {{&S9686[75], 4}, {NULL}};
const Segment SGI273239[] = {{&S9686[13], 14}, {&S9686[49], 17}, {&S9686[119], 35}, {&S9686[155], 20}, {&S9686[119], 35}, {&S9686[230], 20}, {NULL}};
const Segment SGI273240[] = {{&S9687[75], 4}, {NULL}};
const Segment SGI273241[] = {{&S9687[13], 14}, {&S9687[49], 17}, {&S9687[119], 35}, {&S9687[155], 20}, {&S9687[119], 35}, {&S9687[155], 20}, {NULL}};
const Segment SGI273242[] = {{&S9688[103], 4}, {NULL}};
const Segment SGI273243[] = {{&S9688[13], 14}, {&S9688[11], 1}, {&S9688[30], 25}, {&S9688[77], 17}, {NULL}};
const Segment SGI273244[] = {{&S9689[75], 4}, {NULL}};
const Segment SGI273245[] = {{&S9689[13], 14}, {&S9689[49], 17}, {&S9689[119], 35}, {&S9689[155], 20}, {&S9689[119], 35}, {&S9689[230], 20}, {NULL}};
const Segment SGI273246[] = {{&S9690[75], 4}, {NULL}};
const Segment SGI273247[] = {{&S9690[13], 14}, {&S9690[49], 17}, {NULL}};
const Segment SGI273248[] = {{&S9691[97], 4}, {NULL}};
const Segment SGI273249[] = {{&S9691[13], 14}, {&S9691[28], 6}, {&S9691[56], 32}, {NULL}};
const Segment SGI273250[] = {{&S9692[75], 4}, {NULL}};
const Segment SGI273251[] = {{&S9692[13], 14}, {&S9692[49], 17}, {&S9692[119], 35}, {&S9692[155], 20}, {&S9692[119], 35}, {&S9692[230], 20}, {NULL}};
const Segment SGI273252[] = {{&S9693[41], 4}, {NULL}};
const Segment SGI273253[] = {{&S9693[13], 14}, {&S9693[28], 24}, {&S9693[73], 17}, {NULL}};
const Segment SGI273254[] = {{&S9694[41], 4}, {NULL}};
const Segment SGI273255[] = {{&S9694[13], 14}, {&S9694[28], 24}, {&S9694[73], 17}, {NULL}};
const Segment SGI273256[] = {{&S9695[41], 4}, {NULL}};
const Segment SGI273257[] = {{&S9695[13], 14}, {&S9695[28], 25}, {&S9695[74], 17}, {NULL}};
const Segment SGI273258[] = {{&S9696[41], 4}, {NULL}};
const Segment SGI273259[] = {{&S9696[13], 14}, {&S9696[28], 25}, {&S9696[74], 17}, {NULL}};
const Segment SGI273260[] = {{&S9697[41], 4}, {NULL}};
const Segment SGI273261[] = {{&S9697[13], 14}, {&S9697[28], 25}, {&S9697[74], 17}, {NULL}};
const Segment SGI273262[] = {{&S9698[41], 4}, {NULL}};
const Segment SGI273263[] = {{&S9698[13], 14}, {&S9698[28], 25}, {&S9698[74], 17}, {NULL}};
const Segment SGI273264[] = {{&S9699[41], 4}, {NULL}};
const Segment SGI273265[] = {{&S9699[13], 14}, {&S9699[28], 25}, {&S9699[74], 17}, {NULL}};
const Segment SGI273266[] = {{&S9700[41], 4}, {NULL}};
const Segment SGI273267[] = {{&S9700[13], 14}, {&S9700[28], 25}, {&S9700[74], 17}, {NULL}};
const Segment SGI273268[] = {{&S9701[41], 4}, {NULL}};
const Segment SGI273269[] = {{&S9701[13], 14}, {&S9701[28], 25}, {&S9701[74], 17}, {NULL}};
const Segment SGI273270[] = {{&S9702[41], 4}, {NULL}};
const Segment SGI273271[] = {{&S9702[13], 14}, {&S9702[28], 25}, {&S9702[74], 17}, {NULL}};
const Segment SGI273272[] = {{&S9704[41], 4}, {NULL}};
const Segment SGI273273[] = {{&S9704[13], 14}, {&S9704[28], 25}, {&S9704[74], 17}, {NULL}};
const Segment SGI273274[] = {{&S9708[41], 4}, {NULL}};
const Segment SGI273275[] = {{&S9708[13], 14}, {&S9708[28], 25}, {&S9708[74], 17}, {NULL}};
const Segment SGI273276[] = {{&S9709[41], 4}, {NULL}};
const Segment SGI273277[] = {{&S9709[13], 14}, {&S9709[28], 25}, {&S9709[74], 17}, {NULL}};
const Segment SGI273278[] = {{&S9710[48], 4}, {NULL}};
const Segment SGI273279[] = {{&S9710[13], 14}, {&S9710[28], 6}, {&S9710[35], 25}, {&S9710[81], 17}, {NULL}};
const Segment SGI273280[] = {{&S9719[48], 4}, {NULL}};
const Segment SGI273281[] = {{&S9719[13], 14}, {&S9719[28], 6}, {&S9719[35], 25}, {&S9719[81], 17}, {NULL}};
const Segment SGI273282[] = {{&S9722[41], 4}, {NULL}};
const Segment SGI273283[] = {{&S9722[13], 14}, {&S9722[28], 25}, {&S9722[74], 17}, {NULL}};
const Segment SGI273284[] = {{&S9724[41], 4}, {NULL}};
const Segment SGI273285[] = {{&S9724[13], 14}, {&S9724[28], 25}, {&S9724[74], 17}, {NULL}};
const Segment SGI273286[] = {{&S9761[48], 4}, {NULL}};
const Segment SGI273287[] = {{&S9761[13], 14}, {&S9761[28], 6}, {&S9761[35], 24}, {&S9761[81], 17}, {NULL}};
const Segment SGI273288[] = {{&S9762[48], 4}, {NULL}};
const Segment SGI273289[] = {{&S9762[13], 14}, {&S9762[28], 6}, {&S9762[35], 24}, {&S9762[81], 17}, {NULL}};
const Segment SGI273290[] = {{&S9763[48], 4}, {NULL}};
const Segment SGI273291[] = {{&S9763[13], 14}, {&S9763[28], 6}, {&S9763[35], 24}, {&S9763[81], 17}, {NULL}};
const Segment SGI273292[] = {{&S9764[41], 4}, {NULL}};
const Segment SGI273293[] = {{&S9764[13], 14}, {&S9764[28], 24}, {&S9764[74], 17}, {NULL}};
const Segment SGI273294[] = {{&S9765[48], 4}, {NULL}};
const Segment SGI273295[] = {{&S9765[13], 14}, {&S9765[28], 6}, {&S9765[35], 25}, {&S9765[82], 17}, {NULL}};
const Segment SGI273296[] = {{&S9766[41], 4}, {NULL}};
const Segment SGI273297[] = {{&S9766[13], 14}, {&S9766[28], 25}, {&S9766[75], 17}, {NULL}};
const Segment SGI273298[] = {{&S9767[48], 4}, {NULL}};
const Segment SGI273299[] = {{&S9767[13], 14}, {&S9767[28], 6}, {&S9767[35], 25}, {&S9767[82], 17}, {NULL}};
const Segment SGI273300[] = {{&S9768[48], 4}, {NULL}};
const Segment SGI273301[] = {{&S9768[13], 14}, {&S9768[28], 6}, {&S9768[35], 25}, {&S9768[82], 17}, {NULL}};
const Segment SGI273302[] = {{&S9769[48], 4}, {NULL}};
const Segment SGI273303[] = {{&S9769[13], 14}, {&S9769[28], 6}, {&S9769[35], 25}, {&S9769[82], 17}, {NULL}};
const Segment SGI273304[] = {{&S9770[48], 4}, {NULL}};
const Segment SGI273305[] = {{&S9770[13], 14}, {&S9770[28], 6}, {&S9770[35], 25}, {&S9770[82], 17}, {NULL}};
const Segment SGI273306[] = {{&S9771[41], 4}, {NULL}};
const Segment SGI273307[] = {{&S9771[13], 14}, {&S9771[28], 25}, {&S9771[75], 17}, {NULL}};
const Segment SGI273308[] = {{&S9772[41], 4}, {NULL}};
const Segment SGI273309[] = {{&S9772[13], 14}, {&S9772[28], 25}, {&S9772[75], 17}, {NULL}};
const Segment SGI273310[] = {{&S9773[48], 4}, {NULL}};
const Segment SGI273311[] = {{&S9773[13], 14}, {&S9773[28], 6}, {&S9773[35], 25}, {&S9773[82], 17}, {NULL}};
const Segment SGI273312[] = {{&S9774[41], 4}, {NULL}};
const Segment SGI273313[] = {{&S9774[13], 14}, {&S9774[28], 25}, {&S9774[75], 17}, {NULL}};
const Segment SGI273314[] = {{&S9776[48], 4}, {NULL}};
const Segment SGI273315[] = {{&S9776[13], 14}, {&S9776[28], 6}, {&S9776[35], 25}, {&S9776[82], 17}, {NULL}};
const Segment SGI273316[] = {{&S9777[41], 4}, {NULL}};
const Segment SGI273317[] = {{&S9777[13], 14}, {&S9777[28], 25}, {&S9777[75], 17}, {NULL}};
const Segment SGI273318[] = {{&S9786[41], 4}, {NULL}};
const Segment SGI273319[] = {{&S9786[13], 14}, {&S9786[28], 25}, {&S9786[75], 17}, {NULL}};
const Segment SGI273320[] = {{&S9787[48], 4}, {NULL}};
const Segment SGI273321[] = {{&S9787[13], 14}, {&S9787[28], 6}, {&S9787[35], 25}, {&S9787[82], 17}, {NULL}};
const Segment SGI273322[] = {{&S9799[75], 4}, {NULL}};
const Segment SGI273323[] = {{&S9799[13], 14}, {&S9799[49], 17}, {NULL}};
const Segment SGI273324[] = {{&S9800[75], 4}, {NULL}};
const Segment SGI273325[] = {{&S9800[13], 14}, {&S9800[49], 17}, {NULL}};
const Segment SGI273326[] = {{&S9801[75], 4}, {NULL}};
const Segment SGI273327[] = {{&S9801[13], 14}, {&S9801[49], 17}, {&S9801[103], 8}, {&S9801[114], 26}, {NULL}};
const Segment SGI273328[] = {{&S9802[82], 4}, {NULL}};
const Segment SGI273329[] = {{&S9802[13], 14}, {&S9802[28], 6}, {&S9802[56], 17}, {NULL}};
const Segment SGI273330[] = {{&S9803[75], 4}, {NULL}};
const Segment SGI273331[] = {{&S9803[13], 14}, {&S9803[49], 17}, {NULL}};
const Segment SGI273332[] = {{&S9804[75], 4}, {NULL}};
const Segment SGI273333[] = {{&S9804[13], 14}, {&S9804[49], 17}, {&S9804[103], 8}, {NULL}};
const Segment SGI273334[] = {{&S9805[75], 4}, {NULL}};
const Segment SGI273335[] = {{&S9805[13], 14}, {&S9805[49], 17}, {NULL}};
const Segment SGI273336[] = {{&S9806[82], 4}, {NULL}};
const Segment SGI273337[] = {{&S9806[13], 14}, {&S9806[28], 6}, {&S9806[56], 17}, {NULL}};
const Segment SGI273338[] = {{&S9807[82], 4}, {NULL}};
const Segment SGI273339[] = {{&S9807[13], 14}, {&S9807[28], 6}, {&S9807[56], 17}, {NULL}};
const Segment SGI273340[] = {{&S9808[74], 4}, {NULL}};
const Segment SGI273341[] = {{&S9808[13], 14}, {&S9808[49], 16}, {NULL}};
const Segment SGI273342[] = {{&S9809[75], 4}, {NULL}};
const Segment SGI273343[] = {{&S9809[13], 14}, {&S9809[49], 17}, {&S9809[128], 8}, {&S9809[128], 8}, {NULL}};
const Segment SGI273344[] = {{&S9810[75], 4}, {NULL}};
const Segment SGI273345[] = {{&S9810[13], 14}, {&S9810[49], 17}, {&S9810[115], 9}, {NULL}};
const Segment SGI273346[] = {{&S9811[75], 4}, {NULL}};
const Segment SGI273347[] = {{&S9811[13], 14}, {&S9811[49], 17}, {&S9811[120], 35}, {&S9811[156], 22}, {NULL}};
const Segment SGI273348[] = {{&S9812[75], 4}, {NULL}};
const Segment SGI273349[] = {{&S9812[13], 14}, {&S9812[49], 17}, {&S9812[133], 3}, {&S9812[137], 14}, {&S9812[152], 11}, {NULL}};
const Segment SGI273350[] = {{&S15479[81], 4}, {NULL}};
const Segment SGI273351[] = {{&S15479[13], 14}, {&S15479[28], 6}, {&S15479[55], 17}, {&S15479[123], 35}, {&S15479[159], 17}, {NULL}};
const Segment SGI273352[] = {{&S15480[81], 4}, {NULL}};
const Segment SGI273353[] = {{&S15480[13], 14}, {&S15480[28], 6}, {&S15480[55], 17}, {&S15480[106], 8}, {&S15480[117], 24}, {NULL}};
const Segment SGI273354[] = {{&S15481[74], 4}, {NULL}};
const Segment SGI273355[] = {{&S15481[13], 14}, {&S15481[48], 17}, {&S15481[99], 8}, {&S15481[110], 24}, {NULL}};
const Segment SGI273356[] = {{&S15482[81], 4}, {NULL}};
const Segment SGI273357[] = {{&S15482[13], 14}, {&S15482[28], 6}, {&S15482[55], 17}, {&S15482[106], 8}, {&S15482[117], 26}, {NULL}};
const Segment SGI273358[] = {{&S15483[81], 4}, {NULL}};
const Segment SGI273359[] = {{&S15483[13], 14}, {&S15483[28], 6}, {&S15483[55], 17}, {NULL}};
const Segment SGI273360[] = {{&S15484[74], 4}, {NULL}};
const Segment SGI273361[] = {{&S15484[13], 14}, {&S15484[48], 17}, {NULL}};
const Segment SGI273362[] = {{&S15485[93], 4}, {NULL}};
const Segment SGI273363[] = {{&S15485[13], 7}, {&S15485[21], 2}, {&S15485[24], 15}, {&S15485[40], 6}, {&S15485[67], 17}, {NULL}};
const Segment SGI273364[] = {{&S15486[74], 4}, {NULL}};
const Segment SGI273365[] = {{&S15486[13], 14}, {&S15486[48], 17}, {&S15486[99], 8}, {NULL}};
const Segment SGI273366[] = {{&S15487[81], 4}, {NULL}};
const Segment SGI273367[] = {{&S15487[13], 14}, {&S15487[28], 6}, {&S15487[55], 17}, {&S15487[106], 8}, {NULL}};
const Segment SGI273368[] = {{&S15490[73], 4}, {NULL}};
const Segment SGI273369[] = {{&S15490[13], 14}, {&S15490[48], 16}, {NULL}};
const Segment SGI273370[] = {{&S15491[92], 4}, {NULL}};
const Segment SGI273371[] = {{&S15491[13], 14}, {&S15491[28], 6}, {&S15491[35], 10}, {&S15491[66], 17}, {NULL}};
const Segment SGI273372[] = {{&S15492[74], 4}, {NULL}};
const Segment SGI273373[] = {{&S15492[13], 14}, {&S15492[48], 17}, {&S15492[98], 8}, {&S15492[109], 26}, {NULL}};
const Segment SGI273374[] = {{&S15493[69], 4}, {NULL}};
const Segment SGI273375[] = {{&S15493[13], 9}, {&S15493[43], 17}, {NULL}};
const Segment SGI273376[] = {{&S9952[75], 4}, {NULL}};
const Segment SGI273377[] = {{&S9952[13], 14}, {&S9952[49], 17}, {NULL}};
const Segment SGI273378[] = {{&S9953[75], 4}, {NULL}};
const Segment SGI273379[] = {{&S9953[13], 14}, {&S9953[49], 17}, {&S9953[102], 8}, {&S9953[113], 26}, {NULL}};
const Segment SGI273380[] = {{&S9954[75], 4}, {NULL}};
const Segment SGI273381[] = {{&S9954[13], 14}, {&S9954[49], 17}, {NULL}};
const Segment SGI273382[] = {{&S9955[82], 4}, {NULL}};
const Segment SGI273383[] = {{&S9955[13], 14}, {&S9955[28], 6}, {&S9955[56], 17}, {NULL}};
const Segment SGI273384[] = {{&S9956[75], 4}, {NULL}};
const Segment SGI273385[] = {{&S9956[13], 14}, {&S9956[49], 17}, {NULL}};
const Segment SGI273386[] = {{&S9957[75], 4}, {NULL}};
const Segment SGI273387[] = {{&S9957[13], 14}, {&S9957[49], 17}, {NULL}};
const Segment SGI273388[] = {{&S9958[75], 4}, {NULL}};
const Segment SGI273389[] = {{&S9958[13], 14}, {&S9958[49], 17}, {&S9958[102], 8}, {NULL}};
const Segment SGI273390[] = {{&S9959[82], 4}, {NULL}};
const Segment SGI273391[] = {{&S9959[13], 14}, {&S9959[28], 6}, {&S9959[56], 17}, {NULL}};
const Segment SGI273392[] = {{&S9960[82], 4}, {NULL}};
const Segment SGI273393[] = {{&S9960[13], 14}, {&S9960[28], 6}, {&S9960[56], 17}, {NULL}};
const Segment SGI273394[] = {{&S9961[103], 4}, {NULL}};
const Segment SGI273395[] = {{&S9961[13], 14}, {&S9961[11], 1}, {&S9961[30], 25}, {&S9961[77], 17}, {NULL}};
const Segment SGI273396[] = {{&S9962[74], 4}, {NULL}};
const Segment SGI273397[] = {{&S9962[13], 14}, {&S9962[49], 16}, {NULL}};
const Segment SGI273398[] = {{&S9963[75], 5}, {NULL}};
const Segment SGI273399[] = {{&S9963[13], 14}, {&S9963[49], 17}, {NULL}};
const Segment SGI273400[] = {{&S15510[75], 4}, {NULL}};
const Segment SGI273401[] = {{&S15510[13], 14}, {&S15510[49], 17}, {NULL}};
const Segment SGI273402[] = {{&S9964[75], 4}, {NULL}};
const Segment SGI273403[] = {{&S9964[13], 14}, {&S9964[49], 17}, {&S9964[98], 8}, {NULL}};
const Segment SGI273404[] = {{&S15511[75], 4}, {NULL}};
const Segment SGI273405[] = {{&S15511[13], 14}, {&S15511[49], 17}, {NULL}};
const Segment SGI273406[] = {{&S9965[75], 4}, {NULL}};
const Segment SGI273407[] = {{&S9965[13], 14}, {&S9965[49], 17}, {&S9965[102], 8}, {NULL}};
const Segment SGI273408[] = {{&S15512[82], 4}, {NULL}};
const Segment SGI273409[] = {{&S15512[13], 14}, {&S15512[28], 6}, {&S15512[56], 17}, {NULL}};
const Segment SGI273410[] = {{&S15513[75], 4}, {NULL}};
const Segment SGI273411[] = {{&S15513[13], 14}, {&S15513[49], 17}, {&S15513[101], 8}, {&S15513[112], 26}, {NULL}};
const Segment SGI273412[] = {{&S15514[93], 4}, {NULL}};
const Segment SGI273413[] = {{&S15514[13], 7}, {&S15514[21], 2}, {&S15514[24], 15}, {&S15514[40], 6}, {&S15514[67], 17}, {NULL}};
const Segment SGI273414[] = {{&S15515[94], 4}, {NULL}};
const Segment SGI273415[] = {{&S15515[13], 7}, {&S15515[21], 2}, {&S15515[24], 15}, {&S15515[40], 6}, {&S15515[68], 17}, {NULL}};
const Segment SGI273416[] = {{&S15516[75], 4}, {NULL}};
const Segment SGI273417[] = {{&S15516[13], 14}, {&S15516[49], 17}, {&S15516[118], 35}, {&S15516[154], 22}, {NULL}};
const Segment SGI273418[] = {{&S15517[75], 4}, {NULL}};
const Segment SGI273419[] = {{&S15517[13], 14}, {&S15517[49], 17}, {&S15517[119], 8}, {&S15517[130], 24}, {NULL}};
const Segment SGI273420[] = {{&S15518[75], 4}, {NULL}};
const Segment SGI273421[] = {{&S15518[13], 14}, {&S15518[49], 17}, {&S15518[119], 8}, {NULL}};
const Segment SGI273422[] = {{&S15519[74], 4}, {NULL}};
const Segment SGI273423[] = {{&S15519[13], 14}, {&S15519[48], 17}, {&S15519[126], 3}, {&S15519[130], 11}, {&S15519[142], 10}, {NULL}};
const Segment SGI273424[] = {{&S15520[82], 4}, {NULL}};
const Segment SGI273425[] = {{&S15520[13], 14}, {&S15520[28], 6}, {&S15520[56], 17}, {NULL}};
const Segment SGI273426[] = {{&S15521[74], 4}, {NULL}};
const Segment SGI273427[] = {{&S15521[13], 14}, {&S15521[49], 16}, {NULL}};
const Segment SGI273428[] = {{&S15522[75], 4}, {NULL}};
const Segment SGI273429[] = {{&S15522[13], 14}, {&S15522[49], 17}, {NULL}};
const Segment SGI273430[] = {{&S15523[91], 4}, {NULL}};
const Segment SGI273431[] = {{&S15523[13], 14}, {&S15523[28], 15}, {&S15523[65], 17}, {NULL}};
const Segment SGI273432[] = {{&S10007[75], 4}, {NULL}};
const Segment SGI273433[] = {{&S10007[13], 14}, {&S10007[49], 17}, {&S10007[102], 8}, {NULL}};
const Segment SGI273434[] = {{&S10008[82], 4}, {NULL}};
const Segment SGI273435[] = {{&S10008[13], 14}, {&S10008[28], 6}, {&S10008[56], 17}, {NULL}};
const Segment SGI273436[] = {{&S15547[81], 4}, {NULL}};
const Segment SGI273437[] = {{&S15547[13], 14}, {&S15547[28], 6}, {&S15547[55], 17}, {NULL}};
const Segment SGI273438[] = {{&S10009[75], 4}, {NULL}};
const Segment SGI273439[] = {{&S10009[13], 14}, {&S10009[49], 17}, {&S10009[102], 8}, {&S10009[113], 26}, {NULL}};
const Segment SGI273440[] = {{&S15548[74], 4}, {NULL}};
const Segment SGI273441[] = {{&S15548[13], 14}, {&S15548[48], 17}, {NULL}};
const Segment SGI273442[] = {{&S10010[75], 4}, {NULL}};
const Segment SGI273443[] = {{&S10010[13], 14}, {&S10010[49], 17}, {NULL}};
const Segment SGI273444[] = {{&S15549[93], 4}, {NULL}};
const Segment SGI273445[] = {{&S15549[13], 7}, {&S15549[21], 2}, {&S15549[24], 15}, {&S15549[40], 6}, {&S15549[67], 17}, {NULL}};
const Segment SGI273446[] = {{&S10011[75], 4}, {NULL}};
const Segment SGI273447[] = {{&S10011[13], 14}, {&S10011[49], 17}, {NULL}};
const Segment SGI273448[] = {{&S10012[82], 4}, {NULL}};
const Segment SGI273449[] = {{&S15550[81], 4}, {NULL}};
const Segment SGI273450[] = {{&S10012[13], 14}, {&S10012[28], 6}, {&S10012[56], 17}, {NULL}};
const Segment SGI273451[] = {{&S15550[13], 14}, {&S15550[28], 6}, {&S15550[55], 17}, {&S15550[107], 8}, {&S15550[118], 24}, {NULL}};
const Segment SGI273452[] = {{&S10013[75], 4}, {NULL}};
const Segment SGI273453[] = {{&S10013[13], 14}, {&S10013[49], 17}, {&S10013[102], 8}, {&S10013[102], 8}, {NULL}};
const Segment SGI273454[] = {{&S15551[74], 4}, {NULL}};
const Segment SGI273455[] = {{&S15551[13], 14}, {&S15551[48], 17}, {&S15551[100], 8}, {&S15551[111], 24}, {NULL}};
const Segment SGI273456[] = {{&S10014[73], 4}, {NULL}};
const Segment SGI273457[] = {{&S10014[13], 14}, {&S10014[49], 17}, {NULL}};
const Segment SGI273458[] = {{&S10014[68], 9}, {NULL}};
const Segment SGI273459[] = {{&S15552[93], 4}, {NULL}};
const Segment SGI273460[] = {{&S15552[13], 7}, {&S15552[21], 2}, {&S15552[24], 15}, {&S15552[40], 6}, {&S15552[67], 17}, {NULL}};
const Segment SGI273461[] = {{&S15553[74], 4}, {NULL}};
const Segment SGI273462[] = {{&S15553[13], 14}, {&S15553[48], 17}, {&S15553[117], 35}, {&S15553[153], 20}, {NULL}};
const Segment SGI273463[] = {{&S10015[48], 4}, {NULL}};
const Segment SGI273464[] = {{&S10015[13], 14}, {&S10015[28], 6}, {&S10015[35], 24}, {&S10015[81], 17}, {NULL}};
const Segment SGI273465[] = {{&S15554[74], 4}, {NULL}};
const Segment SGI273466[] = {{&S15554[13], 14}, {&S15554[48], 17}, {&S15554[100], 8}, {NULL}};
const Segment SGI273467[] = {{&S10016[48], 4}, {NULL}};
const Segment SGI273468[] = {{&S10016[13], 14}, {&S10016[28], 6}, {&S10016[35], 24}, {&S10016[81], 17}, {NULL}};
const Segment SGI273469[] = {{&S15555[81], 4}, {NULL}};
const Segment SGI273470[] = {{&S15555[13], 14}, {&S15555[28], 6}, {&S15555[55], 17}, {&S15555[107], 8}, {NULL}};
const Segment SGI273471[] = {{&S10017[48], 4}, {NULL}};
const Segment SGI273472[] = {{&S15556[80], 4}, {NULL}};
const Segment SGI273473[] = {{&S10017[13], 14}, {&S10017[28], 6}, {&S10017[35], 25}, {&S10017[82], 17}, {NULL}};
const Segment SGI273474[] = {{&S15556[13], 14}, {&S15556[28], 6}, {&S15556[55], 16}, {NULL}};
const Segment SGI273475[] = {{&S10018[48], 4}, {NULL}};
const Segment SGI273476[] = {{&S10018[13], 14}, {&S10018[28], 6}, {&S10018[35], 25}, {&S10018[82], 17}, {NULL}};
const Segment SGI273477[] = {{&S15557[86], 4}, {NULL}};
const Segment SGI273478[] = {{&S15557[13], 14}, {&S15557[28], 11}, {&S15557[40], 37}, {NULL}};
const Segment SGI273479[] = {{&S10019[41], 4}, {NULL}};
const Segment SGI273480[] = {{&S10019[13], 14}, {&S10019[28], 25}, {&S10019[75], 17}, {NULL}};
const Segment SGI273481[] = {{&S10020[48], 4}, {NULL}};
const Segment SGI273482[] = {{&S10020[13], 14}, {&S10020[28], 6}, {&S10020[35], 25}, {&S10020[82], 17}, {NULL}};
const Segment SGI273483[] = {{&S10021[48], 4}, {NULL}};
const Segment SGI273484[] = {{&S10021[13], 14}, {&S10021[28], 6}, {&S10021[35], 25}, {&S10021[82], 17}, {NULL}};
const Segment SGI273485[] = {{&S10022[41], 4}, {NULL}};
const Segment SGI273486[] = {{&S10022[13], 14}, {&S10022[28], 25}, {&S10022[75], 17}, {NULL}};
const Segment SGI273487[] = {{&S10023[48], 4}, {NULL}};
const Segment SGI273488[] = {{&S10023[13], 14}, {&S10023[28], 6}, {&S10023[35], 25}, {&S10023[82], 17}, {NULL}};
const Segment SGI273489[] = {{&S10024[41], 4}, {NULL}};
const Segment SGI273490[] = {{&S10024[13], 14}, {&S10024[28], 25}, {&S10024[75], 17}, {NULL}};
const Segment SGI273491[] = {{&S10025[48], 4}, {NULL}};
const Segment SGI273492[] = {{&S10025[13], 14}, {&S10025[28], 6}, {&S10025[35], 25}, {&S10025[82], 17}, {NULL}};
const Segment SGI273493[] = {{&S10027[48], 4}, {NULL}};
const Segment SGI273494[] = {{&S10027[13], 14}, {&S10027[28], 6}, {&S10027[35], 25}, {&S10027[82], 17}, {NULL}};
const Segment SGI273495[] = {{&S10029[41], 4}, {NULL}};
const Segment SGI273496[] = {{&S10029[13], 14}, {&S10029[28], 25}, {&S10029[75], 17}, {NULL}};
const Segment SGI273497[] = {{&S10034[48], 4}, {NULL}};
const Segment SGI273498[] = {{&S10034[13], 14}, {&S10034[28], 6}, {&S10034[35], 26}, {&S10034[83], 17}, {NULL}};
const Segment SGI273499[] = {{&S10035[41], 4}, {NULL}};
const Segment SGI273500[] = {{&S10035[13], 14}, {&S10035[28], 26}, {&S10035[76], 17}, {NULL}};
const Segment SGI273501[] = {{&S10040[48], 4}, {NULL}};
const Segment SGI273502[] = {{&S10040[13], 14}, {&S10040[28], 6}, {&S10040[35], 26}, {&S10040[83], 17}, {NULL}};
const Segment SGI273503[] = {{&S10041[41], 4}, {NULL}};
const Segment SGI273504[] = {{&S10041[13], 14}, {&S10041[28], 26}, {&S10041[76], 17}, {NULL}};
const Segment SGI273505[] = {{&S10042[48], 4}, {NULL}};
const Segment SGI273506[] = {{&S10042[13], 14}, {&S10042[28], 6}, {&S10042[35], 26}, {&S10042[83], 17}, {NULL}};
const Segment SGI273507[] = {{&S10043[41], 4}, {NULL}};
const Segment SGI273508[] = {{&S10043[13], 14}, {&S10043[28], 26}, {&S10043[76], 17}, {NULL}};
const Segment SGI273509[] = {{&S28346[38], 5}, {NULL}};
const Segment SGI273510[] = {{&S28346[100], 5}, {NULL}};
const Segment SGI273511[] = {{&S28467[38], 5}, {NULL}};
const Segment SGI273512[] = {{&S28467[97], 5}, {NULL}};
const Segment SGI273513[] = {{&S10433[82], 4}, {NULL}};
const Segment SGI273514[] = {{&S10433[13], 14}, {&S10433[28], 6}, {&S10433[56], 17}, {NULL}};
const Segment SGI273515[] = {{&S10434[75], 4}, {NULL}};
const Segment SGI273516[] = {{&S10434[13], 14}, {&S10434[49], 17}, {NULL}};
const Segment SGI273517[] = {{&S10435[82], 4}, {NULL}};
const Segment SGI273518[] = {{&S10435[13], 14}, {&S10435[28], 6}, {&S10435[56], 17}, {NULL}};
const Segment SGI273519[] = {{&S10436[75], 4}, {NULL}};
const Segment SGI273520[] = {{&S10436[13], 14}, {&S10436[49], 17}, {NULL}};
const Segment SGI273521[] = {{&S10437[82], 4}, {NULL}};
const Segment SGI273522[] = {{&S10437[13], 14}, {&S10437[28], 6}, {&S10437[56], 17}, {&S10437[110], 8}, {NULL}};
const Segment SGI273523[] = {{&S10438[75], 4}, {NULL}};
const Segment SGI273524[] = {{&S10438[13], 14}, {&S10438[49], 17}, {&S10438[103], 8}, {NULL}};
const Segment SGI273525[] = {{&S10439[82], 4}, {NULL}};
const Segment SGI273526[] = {{&S10439[13], 14}, {&S10439[28], 6}, {&S10439[56], 17}, {NULL}};
const Segment SGI273527[] = {{&S10440[75], 4}, {NULL}};
const Segment SGI273528[] = {{&S10440[13], 14}, {&S10440[49], 17}, {NULL}};
const Segment SGI273529[] = {{&S10441[82], 4}, {NULL}};
const Segment SGI273530[] = {{&S10441[13], 14}, {&S10441[28], 6}, {&S10441[56], 17}, {&S10441[110], 8}, {&S10441[121], 26}, {NULL}};
const Segment SGI273531[] = {{&S10442[82], 4}, {NULL}};
const Segment SGI273532[] = {{&S10442[13], 14}, {&S10442[28], 6}, {&S10442[56], 17}, {&S10442[140], 3}, {&S10442[144], 14}, {&S10442[159], 11}, {NULL}};
const Segment SGI273533[] = {{&S10443[81], 4}, {NULL}};
const Segment SGI273534[] = {{&S10443[13], 14}, {&S10443[28], 6}, {&S10443[56], 16}, {NULL}};
const Segment SGI273535[] = {{&S10444[90], 4}, {NULL}};
const Segment SGI273536[] = {{&S10444[13], 9}, {&S10444[23], 20}, {&S10444[65], 16}, {NULL}};
const Segment SGI273537[] = {{&S10445[93], 4}, {NULL}};
const Segment SGI273538[] = {{&S10445[13], 9}, {&S10445[23], 22}, {&S10445[67], 17}, {NULL}};
const Segment SGI273539[] = {{&S10446[93], 4}, {NULL}};
const Segment SGI273540[] = {{&S10446[13], 9}, {&S10446[23], 22}, {&S10446[67], 17}, {NULL}};
const Segment SGI273541[] = {{&S10447[93], 4}, {NULL}};
const Segment SGI273542[] = {{&S10447[13], 9}, {&S10447[23], 22}, {&S10447[67], 17}, {&S10447[120], 8}, {NULL}};
const Segment SGI273543[] = {{&S10448[93], 4}, {NULL}};
const Segment SGI273544[] = {{&S10448[13], 9}, {&S10448[23], 22}, {&S10448[67], 17}, {NULL}};
const Segment SGI273545[] = {{&S10449[93], 4}, {NULL}};
const Segment SGI273546[] = {{&S10449[13], 9}, {&S10449[23], 22}, {&S10449[67], 17}, {NULL}};
const Segment SGI273547[] = {{&S10450[93], 4}, {NULL}};
const Segment SGI273548[] = {{&S10450[13], 9}, {&S10450[23], 22}, {&S10450[67], 17}, {NULL}};
const Segment SGI273549[] = {{&S10451[93], 4}, {NULL}};
const Segment SGI273550[] = {{&S10451[13], 9}, {&S10451[23], 22}, {&S10451[67], 17}, {&S10451[120], 8}, {NULL}};
const Segment SGI273551[] = {{&S10452[93], 4}, {NULL}};
const Segment SGI273552[] = {{&S10452[13], 9}, {&S10452[23], 22}, {&S10452[67], 17}, {NULL}};
const Segment SGI273553[] = {{&S10453[93], 4}, {NULL}};
const Segment SGI273554[] = {{&S10453[13], 9}, {&S10453[23], 22}, {&S10453[67], 17}, {&S10453[120], 8}, {&S10453[131], 26}, {NULL}};
const Segment SGI273555[] = {{&S10809[93], 4}, {NULL}};
const Segment SGI273556[] = {{&S10809[13], 9}, {&S10809[23], 22}, {&S10809[67], 17}, {NULL}};
const Segment SGI273557[] = {{&S10810[93], 4}, {NULL}};
const Segment SGI273558[] = {{&S10810[13], 9}, {&S10810[23], 22}, {&S10810[67], 17}, {NULL}};
const Segment SGI273559[] = {{&S10811[93], 4}, {NULL}};
const Segment SGI273560[] = {{&S10811[13], 9}, {&S10811[23], 22}, {&S10811[67], 17}, {NULL}};
const Segment SGI273561[] = {{&S10812[93], 4}, {NULL}};
const Segment SGI273562[] = {{&S10812[13], 9}, {&S10812[23], 22}, {&S10812[67], 17}, {NULL}};
const Segment SGI273563[] = {{&S10813[93], 4}, {NULL}};
const Segment SGI273564[] = {{&S10813[13], 9}, {&S10813[23], 22}, {&S10813[67], 17}, {&S10813[121], 8}, {NULL}};
const Segment SGI273565[] = {{&S10814[93], 4}, {NULL}};
const Segment SGI273566[] = {{&S10814[13], 9}, {&S10814[23], 22}, {&S10814[67], 17}, {&S10814[121], 8}, {NULL}};
const Segment SGI273567[] = {{&S10815[93], 4}, {NULL}};
const Segment SGI273568[] = {{&S10815[13], 9}, {&S10815[23], 22}, {&S10815[67], 17}, {&S10815[121], 8}, {&S10815[132], 26}, {NULL}};
const Segment SGI273569[] = {{&S10816[93], 4}, {NULL}};
const Segment SGI273570[] = {{&S10816[13], 9}, {&S10816[23], 22}, {&S10816[67], 17}, {&S10816[121], 8}, {&S10816[132], 26}, {NULL}};
const Segment SGI273571[] = {{&S10817[93], 4}, {NULL}};
const Segment SGI273572[] = {{&S10817[13], 9}, {&S10817[23], 22}, {&S10817[67], 17}, {NULL}};
const Segment SGI273573[] = {{&S10818[93], 4}, {NULL}};
const Segment SGI273574[] = {{&S10818[13], 9}, {&S10818[23], 22}, {&S10818[67], 17}, {NULL}};
const Segment SGI273575[] = {{&S10887[93], 4}, {NULL}};
const Segment SGI273576[] = {{&S10887[13], 9}, {&S10887[23], 22}, {&S10887[67], 17}, {NULL}};
const Segment SGI273577[] = {{&S10888[93], 4}, {NULL}};
const Segment SGI273578[] = {{&S10888[13], 9}, {&S10888[23], 22}, {&S10888[67], 17}, {NULL}};
const Segment SGI273579[] = {{&S10889[93], 4}, {NULL}};
const Segment SGI273580[] = {{&S10889[13], 9}, {&S10889[23], 22}, {&S10889[67], 17}, {NULL}};
const Segment SGI273581[] = {{&S10890[93], 4}, {NULL}};
const Segment SGI273582[] = {{&S10890[13], 9}, {&S10890[23], 22}, {&S10890[67], 17}, {&S10890[120], 8}, {NULL}};
const Segment SGI273583[] = {{&S10891[93], 4}, {NULL}};
const Segment SGI273584[] = {{&S10891[13], 9}, {&S10891[23], 22}, {&S10891[67], 17}, {NULL}};
const Segment SGI273585[] = {{&S10892[92], 4}, {NULL}};
const Segment SGI273586[] = {{&S10892[13], 9}, {&S10892[23], 22}, {&S10892[66], 17}, {&S10892[134], 35}, {&S10892[170], 22}, {NULL}};
const Segment SGI273587[] = {{&S10893[93], 4}, {NULL}};
const Segment SGI273588[] = {{&S10893[13], 9}, {&S10893[23], 22}, {&S10893[67], 17}, {NULL}};
const Segment SGI273589[] = {{&S10894[93], 4}, {NULL}};
const Segment SGI273590[] = {{&S10894[13], 9}, {&S10894[23], 22}, {&S10894[67], 17}, {NULL}};
const Segment SGI273591[] = {{&S10895[93], 4}, {NULL}};
const Segment SGI273592[] = {{&S10895[13], 9}, {&S10895[23], 22}, {&S10895[67], 17}, {&S10895[121], 8}, {NULL}};
const Segment SGI273593[] = {{&S10896[93], 4}, {NULL}};
const Segment SGI273594[] = {{&S10896[13], 9}, {&S10896[23], 22}, {&S10896[67], 17}, {&S10896[121], 8}, {&S10896[132], 26}, {NULL}};
const Segment SGI273595[] = {{&S10898[92], 4}, {NULL}};
const Segment SGI273596[] = {{&S10898[13], 9}, {&S10898[23], 22}, {&S10898[66], 17}, {&S10898[134], 35}, {&S10898[170], 24}, {NULL}};
const Segment SGI273597[] = {{&S10899[74], 4}, {NULL}};
const Segment SGI273598[] = {{&S10899[13], 14}, {&S10899[48], 17}, {NULL}};
const Segment SGI273599[] = {{&S10900[74], 4}, {NULL}};
const Segment SGI273600[] = {{&S10900[13], 14}, {&S10900[48], 17}, {NULL}};
const Segment SGI273601[] = {{&S10901[81], 4}, {NULL}};
const Segment SGI273602[] = {{&S10901[13], 14}, {&S10901[28], 6}, {&S10901[55], 17}, {NULL}};
const Segment SGI273603[] = {{&S10902[74], 4}, {NULL}};
const Segment SGI273604[] = {{&S10902[13], 14}, {&S10902[48], 17}, {&S10902[100], 8}, {&S10902[111], 24}, {NULL}};
const Segment SGI273605[] = {{&S10903[81], 4}, {NULL}};
const Segment SGI273606[] = {{&S10903[13], 14}, {&S10903[28], 6}, {&S10903[55], 17}, {NULL}};
const Segment SGI273607[] = {{&S10904[74], 4}, {NULL}};
const Segment SGI273608[] = {{&S10904[13], 14}, {&S10904[48], 17}, {&S10904[85], 9}, {NULL}};
const Segment SGI273609[] = {{&S10905[74], 4}, {NULL}};
const Segment SGI273610[] = {{&S10905[13], 14}, {&S10905[48], 17}, {&S10905[100], 8}, {NULL}};
const Segment SGI273611[] = {{&S10906[74], 4}, {NULL}};
const Segment SGI273612[] = {{&S10906[13], 14}, {&S10906[48], 17}, {&S10906[100], 8}, {&S10906[111], 26}, {NULL}};
const Segment SGI273613[] = {{&S10907[74], 4}, {NULL}};
const Segment SGI273614[] = {{&S10907[13], 14}, {&S10907[48], 17}, {&S10907[117], 35}, {&S10907[153], 20}, {NULL}};
const Segment SGI273615[] = {{&S10908[82], 4}, {NULL}};
const Segment SGI273616[] = {{&S10908[13], 14}, {&S10908[28], 6}, {&S10908[56], 17}, {NULL}};
const Segment SGI273617[] = {{&S10909[73], 4}, {NULL}};
const Segment SGI273618[] = {{&S10909[13], 14}, {&S10909[48], 16}, {NULL}};
const Segment SGI273619[] = {{&S10910[74], 4}, {NULL}};
const Segment SGI273620[] = {{&S10910[13], 14}, {&S10910[48], 17}, {&S10910[112], 4}, {NULL}};
const Segment SGI273621[] = {{&S10911[74], 4}, {NULL}};
const Segment SGI273622[] = {{&S10911[13], 14}, {&S10911[48], 17}, {&S10911[112], 4}, {NULL}};
const Segment SGI273623[] = {{&S10912[102], 4}, {NULL}};
const Segment SGI273624[] = {{&S10912[13], 14}, {&S10912[11], 1}, {&S10912[30], 25}, {&S10912[76], 17}, {NULL}};
const Segment SGI273625[] = {{&S10913[74], 4}, {NULL}};
const Segment SGI273626[] = {{&S10913[13], 14}, {&S10913[48], 17}, {&S10913[112], 4}, {NULL}};
const Segment SGI273627[] = {{&S10914[82], 4}, {NULL}};
const Segment SGI273628[] = {{&S10914[13], 14}, {&S10914[28], 6}, {&S10914[56], 17}, {NULL}};
const Segment SGI273629[] = {{&S10915[74], 4}, {NULL}};
const Segment SGI273630[] = {{&S10915[13], 14}, {&S10915[48], 17}, {NULL}};
const Segment SGI273631[] = {{&S18655[74], 4}, {&S18655[97], 3}, {NULL}};
const Segment SGI273632[] = {{&S18655[13], 14}, {&S18655[48], 17}, {NULL}};
const Segment SGI273633[] = {{&S11347[75], 4}, {NULL}};
const Segment SGI273634[] = {{&S11347[13], 14}, {&S11347[49], 17}, {&S11347[102], 8}, {&S11347[113], 26}, {NULL}};
const Segment SGI273635[] = {{&S11348[82], 4}, {NULL}};
const Segment SGI273636[] = {{&S11348[13], 14}, {&S11348[28], 6}, {&S11348[56], 17}, {NULL}};
const Segment SGI273637[] = {{&S11349[75], 4}, {NULL}};
const Segment SGI273638[] = {{&S11349[13], 14}, {&S11349[49], 17}, {NULL}};
const Segment SGI273639[] = {{&S11350[75], 4}, {NULL}};
const Segment SGI273640[] = {{&S11350[13], 14}, {&S11350[49], 17}, {NULL}};
const Segment SGI273641[] = {{&S11351[75], 4}, {NULL}};
const Segment SGI273642[] = {{&S11351[13], 14}, {&S11351[49], 17}, {&S11351[102], 8}, {NULL}};
const Segment SGI273643[] = {{&S11352[82], 4}, {NULL}};
const Segment SGI273644[] = {{&S11352[13], 14}, {&S11352[28], 6}, {&S11352[56], 17}, {NULL}};
const Segment SGI273645[] = {{&S11353[75], 4}, {NULL}};
const Segment SGI273646[] = {{&S11353[13], 14}, {&S11353[49], 17}, {NULL}};
const Segment SGI273647[] = {{&S11354[75], 4}, {NULL}};
const Segment SGI273648[] = {{&S11354[13], 14}, {&S11354[49], 17}, {NULL}};
const Segment SGI273649[] = {{&S11355[82], 4}, {NULL}};
const Segment SGI273650[] = {{&S11355[13], 14}, {&S11355[28], 6}, {&S11355[56], 17}, {NULL}};
const Segment SGI273651[] = {{&S11356[82], 4}, {NULL}};
const Segment SGI273652[] = {{&S11356[13], 14}, {&S11356[28], 6}, {&S11356[56], 17}, {NULL}};
const Segment SGI273653[] = {{&S11357[75], 4}, {NULL}};
const Segment SGI273654[] = {{&S11357[13], 14}, {&S11357[49], 17}, {&S11357[102], 8}, {NULL}};
const Segment SGI273655[] = {{&S11358[75], 4}, {NULL}};
const Segment SGI273656[] = {{&S11358[13], 14}, {&S11358[49], 17}, {NULL}};
const Segment SGI273657[] = {{&S11359[82], 4}, {NULL}};
const Segment SGI273658[] = {{&S11359[13], 14}, {&S11359[28], 6}, {&S11359[56], 17}, {NULL}};
const Segment SGI273659[] = {{&S11360[93], 4}, {NULL}};
const Segment SGI273660[] = {{&S11360[13], 9}, {&S11360[23], 22}, {&S11360[67], 17}, {&S11360[120], 8}, {&S11360[131], 26}, {NULL}};
const Segment SGI273661[] = {{&S11361[93], 4}, {NULL}};
const Segment SGI273662[] = {{&S11361[13], 9}, {&S11361[23], 22}, {&S11361[67], 17}, {NULL}};
const Segment SGI273663[] = {{&S11362[93], 4}, {NULL}};
const Segment SGI273664[] = {{&S11362[13], 9}, {&S11362[23], 22}, {&S11362[67], 17}, {NULL}};
const Segment SGI273665[] = {{&S11363[93], 4}, {NULL}};
const Segment SGI273666[] = {{&S11363[13], 9}, {&S11363[23], 22}, {&S11363[67], 17}, {NULL}};
const Segment SGI273667[] = {{&S11364[93], 4}, {NULL}};
const Segment SGI273668[] = {{&S11364[13], 9}, {&S11364[23], 22}, {&S11364[67], 17}, {NULL}};
const Segment SGI273669[] = {{&S11365[93], 4}, {NULL}};
const Segment SGI273670[] = {{&S11365[13], 9}, {&S11365[23], 22}, {&S11365[67], 17}, {NULL}};
const Segment SGI273671[] = {{&S11366[93], 4}, {NULL}};
const Segment SGI273672[] = {{&S11366[13], 9}, {&S11366[23], 22}, {&S11366[67], 17}, {&S11366[120], 8}, {NULL}};
const Segment SGI273673[] = {{&S11367[93], 4}, {NULL}};
const Segment SGI273674[] = {{&S11367[13], 9}, {&S11367[23], 22}, {&S11367[67], 17}, {&S11367[120], 8}, {NULL}};
const Segment SGI273675[] = {{&S11368[93], 4}, {NULL}};
const Segment SGI273676[] = {{&S11368[13], 9}, {&S11368[23], 22}, {&S11368[67], 17}, {&S11368[120], 8}, {&S11368[131], 26}, {NULL}};
const Segment SGI273677[] = {{&S11369[92], 4}, {NULL}};
const Segment SGI273678[] = {{&S11369[13], 9}, {&S11369[23], 22}, {&S11369[67], 16}, {NULL}};
const Segment SGI273679[] = {{&S11370[93], 4}, {NULL}};
const Segment SGI273680[] = {{&S11370[13], 9}, {&S11370[23], 22}, {&S11370[67], 17}, {NULL}};
const Segment SGI273681[] = {{&S11698[93], 4}, {NULL}};
const Segment SGI273682[] = {{&S11698[13], 9}, {&S11698[23], 22}, {&S11698[67], 17}, {NULL}};
const Segment SGI273683[] = {{&S11699[93], 4}, {NULL}};
const Segment SGI273684[] = {{&S11699[13], 9}, {&S11699[23], 22}, {&S11699[67], 17}, {NULL}};
const Segment SGI273685[] = {{&S11700[93], 4}, {NULL}};
const Segment SGI273686[] = {{&S11700[13], 9}, {&S11700[23], 22}, {&S11700[67], 17}, {NULL}};
const Segment SGI273687[] = {{&S11701[93], 4}, {NULL}};
const Segment SGI273688[] = {{&S11701[13], 9}, {&S11701[23], 22}, {&S11701[67], 17}, {NULL}};
const Segment SGI273689[] = {{&S11702[93], 4}, {NULL}};
const Segment SGI273690[] = {{&S11702[13], 9}, {&S11702[23], 22}, {&S11702[67], 17}, {&S11702[121], 8}, {NULL}};
const Segment SGI273691[] = {{&S11703[93], 4}, {NULL}};
const Segment SGI273692[] = {{&S11703[13], 9}, {&S11703[23], 22}, {&S11703[67], 17}, {&S11703[121], 8}, {&S11703[132], 26}, {NULL}};
const Segment SGI273693[] = {{&S11704[90], 4}, {NULL}};
const Segment SGI273694[] = {{&S11704[13], 9}, {&S11704[23], 20}, {&S11704[65], 16}, {NULL}};
const Segment SGI273695[] = {{&S11705[93], 4}, {NULL}};
const Segment SGI273696[] = {{&S11705[13], 9}, {&S11705[23], 22}, {&S11705[67], 17}, {NULL}};
const Segment SGI273697[] = {{&S11706[93], 4}, {NULL}};
const Segment SGI273698[] = {{&S11706[13], 9}, {&S11706[23], 22}, {&S11706[67], 17}, {NULL}};
const Segment SGI273699[] = {{&S11707[82], 4}, {NULL}};
const Segment SGI273700[] = {{&S11707[13], 14}, {&S11707[28], 6}, {&S11707[56], 17}, {&S11707[109], 8}, {&S11707[120], 26}, {NULL}};
const Segment SGI273701[] = {{&S11708[75], 4}, {NULL}};
const Segment SGI273702[] = {{&S11708[13], 14}, {&S11708[49], 17}, {NULL}};
const Segment SGI273703[] = {{&S11709[82], 4}, {NULL}};
const Segment SGI273704[] = {{&S11709[13], 14}, {&S11709[28], 6}, {&S11709[56], 17}, {NULL}};
const Segment SGI273705[] = {{&S11710[75], 4}, {NULL}};
const Segment SGI273706[] = {{&S11710[13], 14}, {&S11710[49], 17}, {&S11710[102], 8}, {&S11710[113], 24}, {NULL}};
const Segment SGI273707[] = {{&S11711[82], 4}, {NULL}};
const Segment SGI273708[] = {{&S11711[13], 14}, {&S11711[28], 6}, {&S11711[56], 17}, {&S11711[109], 8}, {&S11711[120], 24}, {NULL}};
const Segment SGI273709[] = {{&S11712[75], 4}, {NULL}};
const Segment SGI273710[] = {{&S11712[13], 14}, {&S11712[49], 17}, {&S11712[102], 8}, {&S11712[113], 26}, {NULL}};
const Segment SGI273711[] = {{&S11713[93], 4}, {NULL}};
const Segment SGI273712[] = {{&S11713[13], 7}, {&S11713[21], 2}, {&S11713[24], 15}, {&S11713[40], 6}, {&S11713[67], 17}, {NULL}};
const Segment SGI273713[] = {{&S11714[82], 4}, {NULL}};
const Segment SGI273714[] = {{&S11714[13], 14}, {&S11714[28], 6}, {&S11714[56], 17}, {&S11714[109], 8}, {NULL}};
const Segment SGI273715[] = {{&S11715[75], 4}, {NULL}};
const Segment SGI273716[] = {{&S11715[13], 14}, {&S11715[49], 17}, {&S11715[102], 8}, {NULL}};
const Segment SGI273717[] = {{&S11716[180], 4}, {NULL}};
const Segment SGI273718[] = {{&S11716[13], 10}, {&S11716[24], 9}, {&S11716[34], 15}, {&S11716[50], 6}, {&S11716[57], 12}, {&S11716[70], 6}, {&S11716[77], 20}, {&S11716[98], 11}, {&S11716[110], 12}, {&S11716[123], 9}, {&S11716[154], 17}, {NULL}};
const Segment SGI273719[] = {{&S11717[81], 4}, {NULL}};
const Segment SGI273720[] = {{&S11717[13], 14}, {&S11717[28], 6}, {&S11717[56], 16}, {NULL}};
const Segment SGI273721[] = {{&S11718[82], 4}, {NULL}};
const Segment SGI273722[] = {{&S11718[13], 14}, {&S11718[28], 6}, {&S11718[56], 17}, {&S11718[126], 35}, {&S11718[162], 20}, {NULL}};
const Segment SGI273723[] = {{&S9928[100], 4}, {NULL}};
const Segment SGI273724[] = {{&S9928[13], 14}, {&S9928[28], 6}, {&S9928[56], 17}, {NULL}};
const Segment SGI273725[] = {{&S9928[75], 11}, {NULL}};
const Segment SGI273726[] = {{&S9929[82], 4}, {NULL}};
const Segment SGI273727[] = {{&S9929[13], 14}, {&S9929[28], 6}, {&S9929[56], 17}, {NULL}};
const Segment SGI273728[] = {{&S9930[75], 4}, {NULL}};
const Segment SGI273729[] = {{&S9930[13], 14}, {&S9930[49], 17}, {NULL}};
const Segment SGI273730[] = {{&S9931[82], 4}, {NULL}};
const Segment SGI273731[] = {{&S9931[13], 14}, {&S9931[28], 6}, {&S9931[56], 17}, {&S9931[109], 8}, {&S9931[120], 26}, {NULL}};
const Segment SGI273732[] = {{&S9932[93], 4}, {NULL}};
const Segment SGI273733[] = {{&S9932[13], 14}, {&S9932[49], 17}, {NULL}};
const Segment SGI273734[] = {{&S9932[68], 11}, {NULL}};
const Segment SGI273735[] = {{&S9933[82], 4}, {NULL}};
const Segment SGI273736[] = {{&S9933[13], 14}, {&S9933[28], 6}, {&S9933[56], 17}, {&S9933[109], 8}, {NULL}};
const Segment SGI273737[] = {{&S9934[75], 4}, {NULL}};
const Segment SGI273738[] = {{&S9934[13], 14}, {&S9934[49], 17}, {&S9934[102], 8}, {NULL}};
const Segment SGI273739[] = {{&S12144[93], 4}, {NULL}};
const Segment SGI273740[] = {{&S12144[13], 9}, {&S12144[23], 22}, {&S12144[67], 17}, {NULL}};
const Segment SGI273741[] = {{&S12145[93], 4}, {NULL}};
const Segment SGI273742[] = {{&S12145[13], 9}, {&S12145[23], 22}, {&S12145[67], 17}, {NULL}};
const Segment SGI273743[] = {{&S12146[93], 4}, {NULL}};
const Segment SGI273744[] = {{&S12146[13], 9}, {&S12146[23], 22}, {&S12146[67], 17}, {NULL}};
const Segment SGI273745[] = {{&S12147[93], 4}, {NULL}};
const Segment SGI273746[] = {{&S12147[13], 9}, {&S12147[23], 22}, {&S12147[67], 17}, {NULL}};
const Segment SGI273747[] = {{&S12148[93], 4}, {NULL}};
const Segment SGI273748[] = {{&S12148[13], 9}, {&S12148[23], 22}, {&S12148[67], 17}, {NULL}};
const Segment SGI273749[] = {{&S12149[93], 4}, {NULL}};
const Segment SGI273750[] = {{&S12149[13], 9}, {&S12149[23], 22}, {&S12149[67], 17}, {NULL}};
const Segment SGI273751[] = {{&S12150[93], 4}, {NULL}};
const Segment SGI273752[] = {{&S12150[13], 9}, {&S12150[23], 22}, {&S12150[67], 17}, {&S12150[121], 8}, {NULL}};
const Segment SGI273753[] = {{&S12151[93], 4}, {NULL}};
const Segment SGI273754[] = {{&S12151[13], 9}, {&S12151[23], 22}, {&S12151[67], 17}, {&S12151[121], 8}, {&S12151[132], 26}, {NULL}};
const Segment SGI273755[] = {{&S12174[73], 4}, {NULL}};
const Segment SGI273756[] = {{&S12174[13], 14}, {&S12174[49], 17}, {NULL}};
const Segment SGI273757[] = {{&S12174[68], 9}, {NULL}};
const Segment SGI273758[] = {{&S12175[75], 4}, {NULL}};
const Segment SGI273759[] = {{&S12175[13], 14}, {&S12175[49], 17}, {NULL}};
const Segment SGI273760[] = {{&S12176[82], 4}, {NULL}};
const Segment SGI273761[] = {{&S12176[13], 14}, {&S12176[28], 6}, {&S12176[56], 17}, {NULL}};
const Segment SGI273762[] = {{&S12177[80], 4}, {NULL}};
const Segment SGI273763[] = {{&S12177[13], 14}, {&S12177[28], 6}, {&S12177[56], 17}, {NULL}};
const Segment SGI273764[] = {{&S12177[75], 9}, {NULL}};
const Segment SGI273765[] = {{&S12178[75], 4}, {NULL}};
const Segment SGI273766[] = {{&S12178[13], 14}, {&S12178[49], 17}, {NULL}};
const Segment SGI273767[] = {{&S12179[82], 4}, {NULL}};
const Segment SGI273768[] = {{&S12179[13], 14}, {&S12179[28], 6}, {&S12179[56], 17}, {NULL}};
const Segment SGI273769[] = {{&S12180[75], 4}, {NULL}};
const Segment SGI273770[] = {{&S12180[13], 14}, {&S12180[49], 17}, {&S12180[103], 8}, {NULL}};
const Segment SGI273771[] = {{&S12181[75], 4}, {NULL}};
const Segment SGI273772[] = {{&S12181[13], 14}, {&S12181[49], 17}, {&S12181[103], 8}, {&S12181[114], 26}, {NULL}};
const Segment SGI273773[] = {{&S12182[82], 4}, {NULL}};
const Segment SGI273774[] = {{&S12182[13], 14}, {&S12182[28], 6}, {&S12182[56], 17}, {NULL}};
const Segment SGI273775[] = {{&S12183[75], 4}, {NULL}};
const Segment SGI273776[] = {{&S12183[13], 14}, {&S12183[49], 17}, {NULL}};
const Segment SGI273777[] = {{&S12184[82], 4}, {NULL}};
const Segment SGI273778[] = {{&S12184[13], 14}, {&S12184[28], 6}, {&S12184[56], 17}, {&S12184[110], 8}, {NULL}};
const Segment SGI273779[] = {{&S12185[82], 4}, {NULL}};
const Segment SGI273780[] = {{&S12185[13], 14}, {&S12185[28], 6}, {&S12185[56], 17}, {&S12185[110], 8}, {&S12185[121], 26}, {NULL}};
const Segment SGI273781[] = {{&S12186[75], 4}, {NULL}};
const Segment SGI273782[] = {{&S12186[13], 14}, {&S12186[49], 17}, {NULL}};
const Segment SGI273783[] = {{&S12187[82], 4}, {NULL}};
const Segment SGI273784[] = {{&S12187[13], 14}, {&S12187[28], 6}, {&S12187[56], 17}, {NULL}};
const Segment SGI273785[] = {{&S12188[82], 4}, {NULL}};
const Segment SGI273786[] = {{&S12188[13], 14}, {&S12188[28], 6}, {&S12188[56], 17}, {&S12188[109], 8}, {NULL}};
const Segment SGI273787[] = {{&S12189[75], 4}, {NULL}};
const Segment SGI273788[] = {{&S12189[13], 14}, {&S12189[49], 17}, {&S12189[102], 8}, {NULL}};
const Segment SGI273789[] = {{&S12190[82], 4}, {NULL}};
const Segment SGI273790[] = {{&S12190[13], 14}, {&S12190[28], 6}, {&S12190[56], 17}, {NULL}};
const Segment SGI273791[] = {{&S12191[75], 4}, {NULL}};
const Segment SGI273792[] = {{&S12191[13], 14}, {&S12191[49], 17}, {NULL}};
const Segment SGI273793[] = {{&S12192[82], 4}, {NULL}};
const Segment SGI273794[] = {{&S12192[13], 14}, {&S12192[28], 6}, {&S12192[56], 17}, {NULL}};
const Segment SGI273795[] = {{&S12193[75], 4}, {NULL}};
const Segment SGI273796[] = {{&S12193[13], 14}, {&S12193[49], 17}, {NULL}};
const Segment SGI273797[] = {{&S12194[89], 4}, {NULL}};
const Segment SGI273798[] = {{&S12194[13], 14}, {&S12194[28], 6}, {&S12194[35], 6}, {&S12194[63], 17}, {NULL}};
const Segment SGI273799[] = {{&S12195[82], 4}, {NULL}};
const Segment SGI273800[] = {{&S12195[13], 14}, {&S12195[28], 6}, {&S12195[56], 17}, {&S12195[109], 8}, {&S12195[120], 26}, {NULL}};
const Segment SGI273801[] = {{&S12196[75], 4}, {NULL}};
const Segment SGI273802[] = {{&S12196[13], 14}, {&S12196[49], 17}, {&S12196[102], 8}, {&S12196[113], 26}, {NULL}};
const Segment SGI273803[] = {{&S12197[82], 5}, {NULL}};
const Segment SGI273804[] = {{&S12197[13], 14}, {&S12197[28], 6}, {&S12197[56], 17}, {NULL}};
const Segment SGI273805[] = {{&S12386[75], 4}, {NULL}};
const Segment SGI273806[] = {{&S12386[13], 14}, {&S12386[49], 17}, {&S12386[102], 8}, {NULL}};
const Segment SGI273807[] = {{&S12387[82], 4}, {NULL}};
const Segment SGI273808[] = {{&S12387[13], 14}, {&S12387[28], 6}, {&S12387[56], 17}, {&S12387[109], 8}, {&S12387[120], 26}, {NULL}};
const Segment SGI273809[] = {{&S12388[82], 4}, {NULL}};
const Segment SGI273810[] = {{&S12388[13], 14}, {&S12388[28], 6}, {&S12388[56], 17}, {NULL}};
const Segment SGI273811[] = {{&S12389[82], 4}, {NULL}};
const Segment SGI273812[] = {{&S12389[13], 14}, {&S12389[28], 6}, {&S12389[56], 17}, {NULL}};
const Segment SGI273813[] = {{&S12390[75], 4}, {NULL}};
const Segment SGI273814[] = {{&S12390[13], 14}, {&S12390[49], 17}, {NULL}};
const Segment SGI273815[] = {{&S12391[75], 4}, {NULL}};
const Segment SGI273816[] = {{&S12391[13], 14}, {&S12391[49], 17}, {NULL}};
const Segment SGI273817[] = {{&S12392[82], 4}, {NULL}};
const Segment SGI273818[] = {{&S12392[13], 14}, {&S12392[28], 6}, {&S12392[56], 17}, {&S12392[109], 8}, {NULL}};
const Segment SGI273819[] = {{&S12393[82], 4}, {NULL}};
const Segment SGI273820[] = {{&S12393[13], 14}, {&S12393[28], 6}, {&S12393[56], 17}, {NULL}};
const Segment SGI273821[] = {{&S12394[75], 4}, {NULL}};
const Segment SGI273822[] = {{&S12394[13], 14}, {&S12394[49], 17}, {NULL}};
const Segment SGI273823[] = {{&S12395[75], 4}, {NULL}};
const Segment SGI273824[] = {{&S12395[13], 14}, {&S12395[49], 17}, {&S12395[102], 8}, {&S12395[113], 26}, {NULL}};
const Segment SGI273825[] = {{&S12396[82], 4}, {NULL}};
const Segment SGI273826[] = {{&S12396[13], 14}, {&S12396[28], 6}, {&S12396[56], 17}, {NULL}};
const Segment SGI273827[] = {{&S12397[75], 4}, {NULL}};
const Segment SGI273828[] = {{&S12397[13], 14}, {&S12397[49], 17}, {&S12397[102], 8}, {&S12397[113], 26}, {NULL}};
const Segment SGI273829[] = {{&S12398[82], 4}, {NULL}};
const Segment SGI273830[] = {{&S12398[13], 14}, {&S12398[28], 6}, {&S12398[56], 17}, {&S12398[109], 8}, {&S12398[120], 26}, {NULL}};
const Segment SGI273831[] = {{&S12399[75], 4}, {NULL}};
const Segment SGI273832[] = {{&S12399[13], 14}, {&S12399[49], 17}, {NULL}};
const Segment SGI273833[] = {{&S12400[82], 4}, {NULL}};
const Segment SGI273834[] = {{&S12400[13], 14}, {&S12400[28], 6}, {&S12400[56], 17}, {NULL}};
const Segment SGI273835[] = {{&S12401[75], 4}, {NULL}};
const Segment SGI273836[] = {{&S12401[13], 14}, {&S12401[49], 17}, {&S12401[102], 8}, {&S12401[102], 8}, {NULL}};
const Segment SGI273837[] = {{&S12402[82], 4}, {NULL}};
const Segment SGI273838[] = {{&S12402[13], 14}, {&S12402[28], 6}, {&S12402[56], 17}, {&S12402[109], 8}, {NULL}};
const Segment SGI273839[] = {{&S12403[75], 4}, {NULL}};
const Segment SGI273840[] = {{&S12403[13], 14}, {&S12403[49], 17}, {NULL}};
const Segment SGI273841[] = {{&S12404[82], 4}, {NULL}};
const Segment SGI273842[] = {{&S12404[13], 14}, {&S12404[28], 6}, {&S12404[56], 17}, {NULL}};
const Segment SGI273843[] = {{&S12405[75], 4}, {NULL}};
const Segment SGI273844[] = {{&S12405[13], 14}, {&S12405[49], 17}, {&S12405[102], 8}, {NULL}};
const Segment SGI273845[] = {{&S12454[93], 4}, {NULL}};
const Segment SGI273846[] = {{&S12454[13], 9}, {&S12454[23], 22}, {&S12454[67], 17}, {NULL}};
const Segment SGI273847[] = {{&S12455[93], 4}, {NULL}};
const Segment SGI273848[] = {{&S12455[13], 9}, {&S12455[23], 22}, {&S12455[67], 17}, {NULL}};
const Segment SGI273849[] = {{&S12456[93], 4}, {NULL}};
const Segment SGI273850[] = {{&S12456[13], 9}, {&S12456[23], 22}, {&S12456[67], 17}, {NULL}};
const Segment SGI273851[] = {{&S12457[93], 4}, {NULL}};
const Segment SGI273852[] = {{&S12457[13], 9}, {&S12457[23], 22}, {&S12457[67], 17}, {NULL}};
const Segment SGI273853[] = {{&S12458[93], 4}, {NULL}};
const Segment SGI273854[] = {{&S12458[13], 9}, {&S12458[23], 22}, {&S12458[67], 17}, {&S12458[120], 8}, {&S12458[131], 26}, {NULL}};
const Segment SGI273855[] = {{&S12459[93], 4}, {NULL}};
const Segment SGI273856[] = {{&S12459[13], 9}, {&S12459[23], 22}, {&S12459[67], 17}, {&S12459[120], 8}, {NULL}};
const Segment SGI273857[] = {{&S12460[100], 4}, {NULL}};
const Segment SGI273858[] = {{&S12460[13], 9}, {&S12460[23], 22}, {&S12460[46], 6}, {&S12460[74], 17}, {NULL}};
const Segment SGI273859[] = {{&S12461[93], 4}, {NULL}};
const Segment SGI273860[] = {{&S12461[13], 9}, {&S12461[23], 22}, {&S12461[67], 17}, {NULL}};
const Segment SGI273861[] = {{&S12462[93], 4}, {NULL}};
const Segment SGI273862[] = {{&S12462[13], 9}, {&S12462[23], 22}, {&S12462[67], 17}, {NULL}};
const Segment SGI273863[] = {{&S12610[75], 4}, {NULL}};
const Segment SGI273864[] = {{&S12610[13], 14}, {&S12610[49], 17}, {NULL}};
const Segment SGI273865[] = {{&S12611[82], 4}, {NULL}};
const Segment SGI273866[] = {{&S12611[13], 14}, {&S12611[28], 6}, {&S12611[56], 17}, {NULL}};
const Segment SGI273867[] = {{&S12612[75], 4}, {NULL}};
const Segment SGI273868[] = {{&S12612[13], 14}, {&S12612[49], 17}, {NULL}};
const Segment SGI273869[] = {{&S12613[82], 4}, {NULL}};
const Segment SGI273870[] = {{&S12613[13], 14}, {&S12613[28], 6}, {&S12613[56], 17}, {NULL}};
const Segment SGI273871[] = {{&S12614[75], 4}, {NULL}};
const Segment SGI273872[] = {{&S12614[13], 14}, {&S12614[49], 17}, {&S12614[102], 8}, {NULL}};
const Segment SGI273873[] = {{&S12615[82], 4}, {NULL}};
const Segment SGI273874[] = {{&S12615[13], 14}, {&S12615[28], 6}, {&S12615[56], 17}, {&S12615[109], 8}, {&S12615[120], 26}, {NULL}};
const Segment SGI273875[] = {{&S12616[82], 4}, {NULL}};
const Segment SGI273876[] = {{&S12616[13], 14}, {&S12616[28], 6}, {&S12616[56], 17}, {NULL}};
const Segment SGI273877[] = {{&S12617[75], 4}, {NULL}};
const Segment SGI273878[] = {{&S12617[13], 14}, {&S12617[49], 17}, {NULL}};
const Segment SGI273879[] = {{&S12618[82], 4}, {NULL}};
const Segment SGI273880[] = {{&S12618[13], 14}, {&S12618[28], 6}, {&S12618[56], 17}, {NULL}};
const Segment SGI273881[] = {{&S12619[82], 4}, {NULL}};
const Segment SGI273882[] = {{&S12619[13], 14}, {&S12619[28], 6}, {&S12619[56], 17}, {&S12619[109], 8}, {NULL}};
const Segment SGI273883[] = {{&S12620[75], 4}, {NULL}};
const Segment SGI273884[] = {{&S12620[13], 14}, {&S12620[49], 17}, {&S12620[102], 8}, {&S12620[113], 26}, {NULL}};
const Segment SGI273885[] = {{&S12621[75], 4}, {NULL}};
const Segment SGI273886[] = {{&S12621[13], 14}, {&S12621[49], 17}, {NULL}};
const Segment SGI273887[] = {{&S12622[75], 4}, {NULL}};
const Segment SGI273888[] = {{&S12622[13], 14}, {&S12622[49], 17}, {NULL}};
const Segment SGI273889[] = {{&S12623[82], 4}, {NULL}};
const Segment SGI273890[] = {{&S12623[13], 14}, {&S12623[28], 6}, {&S12623[56], 17}, {NULL}};
const Segment SGI273891[] = {{&S12624[74], 4}, {NULL}};
const Segment SGI273892[] = {{&S12624[13], 14}, {&S12624[49], 16}, {NULL}};
const Segment SGI273893[] = {{&S12625[80], 4}, {NULL}};
const Segment SGI273894[] = {{&S12625[13], 14}, {&S12625[32], 39}, {NULL}};
const Segment SGI273895[] = {{&S12626[93], 4}, {NULL}};
const Segment SGI273896[] = {{&S12626[13], 9}, {&S12626[23], 22}, {&S12626[67], 17}, {NULL}};
const Segment SGI273897[] = {{&S12627[93], 4}, {NULL}};
const Segment SGI273898[] = {{&S12627[13], 9}, {&S12627[23], 22}, {&S12627[67], 17}, {NULL}};
const Segment SGI273899[] = {{&S12628[93], 4}, {NULL}};
const Segment SGI273900[] = {{&S12628[13], 9}, {&S12628[23], 22}, {&S12628[67], 17}, {NULL}};
const Segment SGI273901[] = {{&S12629[93], 4}, {NULL}};
const Segment SGI273902[] = {{&S12629[13], 9}, {&S12629[23], 22}, {&S12629[67], 17}, {NULL}};
const Segment SGI273903[] = {{&S12630[93], 4}, {NULL}};
const Segment SGI273904[] = {{&S12630[13], 9}, {&S12630[23], 22}, {&S12630[67], 17}, {NULL}};
const Segment SGI273905[] = {{&S12631[93], 4}, {NULL}};
const Segment SGI273906[] = {{&S12631[13], 9}, {&S12631[23], 22}, {&S12631[67], 17}, {NULL}};
const Segment SGI273907[] = {{&S12632[93], 4}, {NULL}};
const Segment SGI273908[] = {{&S12632[13], 9}, {&S12632[23], 22}, {&S12632[67], 17}, {&S12632[121], 8}, {NULL}};
const Segment SGI273909[] = {{&S12633[91], 4}, {NULL}};
const Segment SGI273910[] = {{&S12633[13], 9}, {&S12633[23], 22}, {&S12633[67], 17}, {NULL}};
const Segment SGI273911[] = {{&S12633[86], 9}, {NULL}};
const Segment SGI273912[] = {{&S12634[93], 4}, {NULL}};
const Segment SGI273913[] = {{&S12634[13], 9}, {&S12634[23], 22}, {&S12634[67], 17}, {&S12634[121], 8}, {&S12634[132], 26}, {NULL}};
const Segment SGI273914[] = {{&S12635[75], 4}, {NULL}};
const Segment SGI273915[] = {{&S12635[13], 14}, {&S12635[49], 17}, {NULL}};
const Segment SGI273916[] = {{&S12636[75], 4}, {NULL}};
const Segment SGI273917[] = {{&S12636[13], 14}, {&S12636[49], 17}, {NULL}};
const Segment SGI273918[] = {{&S12637[82], 4}, {NULL}};
const Segment SGI273919[] = {{&S12637[13], 14}, {&S12637[28], 6}, {&S12637[56], 17}, {NULL}};
const Segment SGI273920[] = {{&S12638[82], 4}, {NULL}};
const Segment SGI273921[] = {{&S12638[13], 14}, {&S12638[28], 6}, {&S12638[56], 17}, {NULL}};
const Segment SGI273922[] = {{&S12639[82], 4}, {NULL}};
const Segment SGI273923[] = {{&S12639[13], 14}, {&S12639[28], 6}, {&S12639[56], 17}, {NULL}};
const Segment SGI273924[] = {{&S12640[75], 4}, {NULL}};
const Segment SGI273925[] = {{&S12640[13], 14}, {&S12640[49], 17}, {NULL}};
const Segment SGI273926[] = {{&S12641[75], 4}, {NULL}};
const Segment SGI273927[] = {{&S12641[13], 14}, {&S12641[49], 17}, {NULL}};
const Segment SGI273928[] = {{&S12642[82], 4}, {NULL}};
const Segment SGI273929[] = {{&S12642[13], 14}, {&S12642[28], 6}, {&S12642[56], 17}, {&S12642[109], 8}, {&S12642[120], 26}, {NULL}};
const Segment SGI273930[] = {{&S12643[82], 4}, {NULL}};
const Segment SGI273931[] = {{&S12643[13], 14}, {&S12643[28], 6}, {&S12643[56], 17}, {&S12643[109], 8}, {NULL}};
const Segment SGI273932[] = {{&S12644[75], 4}, {NULL}};
const Segment SGI273933[] = {{&S12644[13], 14}, {&S12644[49], 17}, {&S12644[102], 8}, {NULL}};
const Segment SGI273934[] = {{&S12645[82], 4}, {NULL}};
const Segment SGI273935[] = {{&S12645[13], 14}, {&S12645[28], 6}, {&S12645[56], 17}, {NULL}};
const Segment SGI273936[] = {{&S12646[82], 4}, {NULL}};
const Segment SGI273937[] = {{&S12646[13], 14}, {&S12646[28], 6}, {&S12646[56], 17}, {NULL}};
const Segment SGI273938[] = {{&S12647[75], 4}, {NULL}};
const Segment SGI273939[] = {{&S12647[13], 14}, {&S12647[49], 17}, {NULL}};
const Segment SGI273940[] = {{&S12648[75], 4}, {NULL}};
const Segment SGI273941[] = {{&S12648[13], 14}, {&S12648[49], 17}, {&S12648[102], 8}, {&S12648[113], 26}, {NULL}};
const Segment SGI273942[] = {{&S12649[81], 4}, {NULL}};
const Segment SGI273943[] = {{&S12649[13], 14}, {&S12649[28], 6}, {&S12649[56], 16}, {NULL}};
const Segment SGI273944[] = {{&S19956[94], 3}, {NULL}};
const Segment SGI273945[] = {{&S19956[13], 7}, {&S19956[21], 2}, {&S19956[24], 15}, {&S19956[40], 6}, {&S19956[68], 17}, {NULL}};
const Segment SGI273946[] = {{&S19957[94], 3}, {NULL}};
const Segment SGI273947[] = {{&S19957[13], 7}, {&S19957[21], 2}, {&S19957[24], 15}, {&S19957[40], 6}, {&S19957[68], 17}, {&S19957[119], 8}, {&S19957[130], 24}, {NULL}};
const Segment SGI273948[] = {{&S17321[74], 4}, {NULL}};
const Segment SGI273949[] = {{&S17321[13], 14}, {&S17321[48], 17}, {NULL}};
const Segment SGI273950[] = {{&S19958[94], 3}, {&S19958[94], 3}, {NULL}};
const Segment SGI273951[] = {{&S19958[13], 7}, {&S19958[21], 2}, {&S19958[24], 15}, {&S19958[40], 6}, {&S19958[68], 17}, {NULL}};
const Segment SGI273952[] = {{&S19959[94], 3}, {NULL}};
const Segment SGI273953[] = {{&S17322[74], 4}, {NULL}};
const Segment SGI273954[] = {{&S19959[13], 7}, {&S19959[21], 2}, {&S19959[24], 15}, {&S19959[40], 6}, {&S19959[68], 17}, {&S19959[119], 12}, {NULL}};
const Segment SGI273955[] = {{&S17322[13], 14}, {&S17322[48], 17}, {NULL}};
const Segment SGI273956[] = {{&S19960[75], 3}, {NULL}};
const Segment SGI273957[] = {{&S19960[13], 14}, {&S19960[49], 17}, {NULL}};
const Segment SGI273958[] = {{&S17323[81], 4}, {NULL}};
const Segment SGI273959[] = {{&S17323[13], 14}, {&S17323[28], 6}, {&S17323[55], 17}, {NULL}};
const Segment SGI273960[] = {{&S19961[94], 3}, {NULL}};
const Segment SGI273961[] = {{&S19961[13], 7}, {&S19961[21], 2}, {&S19961[24], 15}, {&S19961[40], 6}, {&S19961[68], 17}, {&S19961[119], 8}, {NULL}};
const Segment SGI273962[] = {{&S17324[94], 4}, {NULL}};
const Segment SGI273963[] = {{&S17324[13], 7}, {&S17324[21], 2}, {&S17324[24], 15}, {&S17324[40], 6}, {&S17324[68], 17}, {NULL}};
const Segment SGI273964[] = {{&S19962[93], 3}, {NULL}};
const Segment SGI273965[] = {{&S19962[13], 7}, {&S19962[21], 2}, {&S19962[24], 15}, {&S19962[40], 6}, {&S19962[68], 16}, {NULL}};
const Segment SGI273966[] = {{&S17325[74], 4}, {NULL}};
const Segment SGI273967[] = {{&S17325[13], 14}, {&S17325[48], 17}, {&S17325[117], 8}, {&S17325[128], 24}, {NULL}};
const Segment SGI273968[] = {{&S19963[93], 3}, {NULL}};
const Segment SGI273969[] = {{&S19963[13], 7}, {&S19963[21], 2}, {&S19963[24], 15}, {&S19963[40], 6}, {&S19963[68], 16}, {NULL}};
const Segment SGI273970[] = {{&S17326[74], 4}, {NULL}};
const Segment SGI273971[] = {{&S17326[13], 14}, {&S17326[48], 17}, {&S17326[100], 8}, {&S17326[111], 26}, {NULL}};
const Segment SGI273972[] = {{&S19964[93], 3}, {NULL}};
const Segment SGI273973[] = {{&S19964[13], 7}, {&S19964[21], 2}, {&S19964[24], 15}, {&S19964[40], 6}, {&S19964[68], 16}, {NULL}};
const Segment SGI273974[] = {{&S17327[74], 4}, {NULL}};
const Segment SGI273975[] = {{&S19965[93], 3}, {NULL}};
const Segment SGI273976[] = {{&S17327[13], 14}, {&S17327[48], 17}, {&S17327[117], 8}, {NULL}};
const Segment SGI273977[] = {{&S19965[13], 7}, {&S19965[21], 2}, {&S19965[24], 15}, {&S19965[40], 6}, {&S19965[68], 16}, {NULL}};
const Segment SGI273978[] = {{&S17328[81], 4}, {NULL}};
const Segment SGI273979[] = {{&S17328[13], 14}, {&S17328[28], 6}, {&S17328[55], 17}, {NULL}};
const Segment SGI273980[] = {{&S19966[93], 3}, {NULL}};
const Segment SGI273981[] = {{&S19966[13], 7}, {&S19966[21], 2}, {&S19966[24], 15}, {&S19966[40], 6}, {&S19966[68], 16}, {NULL}};
const Segment SGI273982[] = {{&S17329[73], 4}, {NULL}};
const Segment SGI273983[] = {{&S17329[13], 14}, {&S17329[48], 16}, {NULL}};
const Segment SGI273984[] = {{&S19967[93], 3}, {NULL}};
const Segment SGI273985[] = {{&S19967[13], 7}, {&S19967[21], 2}, {&S19967[24], 15}, {&S19967[40], 6}, {&S19967[68], 16}, {NULL}};
const Segment SGI273986[] = {{&S17330[73], 4}, {NULL}};
const Segment SGI273987[] = {{&S17330[13], 14}, {&S17330[48], 16}, {NULL}};
const Segment SGI273988[] = {{&S19968[94], 3}, {NULL}};
const Segment SGI273989[] = {{&S19968[13], 7}, {&S19968[21], 2}, {&S19968[24], 15}, {&S19968[40], 7}, {&S19968[69], 16}, {NULL}};
const Segment SGI273990[] = {{&S17331[86], 4}, {NULL}};
const Segment SGI273991[] = {{&S19969[93], 3}, {NULL}};
const Segment SGI273992[] = {{&S17331[13], 7}, {&S17331[21], 2}, {&S17331[24], 15}, {&S17331[60], 17}, {NULL}};
const Segment SGI273993[] = {{&S19969[13], 7}, {&S19969[21], 2}, {&S19969[24], 15}, {&S19969[40], 6}, {&S19969[68], 16}, {NULL}};
const Segment SGI273994[] = {{&S17380[94], 4}, {NULL}};
const Segment SGI273995[] = {{&S17380[13], 7}, {&S17380[21], 2}, {&S17380[24], 15}, {&S17380[40], 6}, {&S17380[68], 17}, {NULL}};
const Segment SGI273996[] = {{&S17381[94], 4}, {NULL}};
const Segment SGI273997[] = {{&S17381[13], 7}, {&S17381[21], 2}, {&S17381[24], 15}, {&S17381[40], 6}, {&S17381[68], 17}, {&S17381[121], 8}, {&S17381[132], 24}, {NULL}};
const Segment SGI273998[] = {{&S17382[94], 4}, {NULL}};
const Segment SGI273999[] = {{&S17382[13], 7}, {&S17382[21], 2}, {&S17382[24], 15}, {&S17382[40], 6}, {&S17382[68], 17}, {NULL}};
const Segment SGI274000[] = {{&S17383[94], 4}, {NULL}};
const Segment SGI274001[] = {{&S17383[13], 7}, {&S17383[21], 2}, {&S17383[24], 15}, {&S17383[40], 6}, {&S17383[68], 17}, {NULL}};
const Segment SGI274002[] = {{&S17384[94], 4}, {NULL}};
const Segment SGI274003[] = {{&S17384[13], 7}, {&S17384[21], 2}, {&S17384[24], 15}, {&S17384[40], 6}, {&S17384[68], 17}, {&S17384[121], 8}, {&S17384[132], 26}, {NULL}};
const Segment SGI274004[] = {{&S17385[93], 4}, {NULL}};
const Segment SGI274005[] = {{&S17385[13], 7}, {&S17385[21], 2}, {&S17385[24], 15}, {&S17385[40], 6}, {&S17385[67], 17}, {NULL}};
const Segment SGI274006[] = {{&S17386[94], 4}, {NULL}};
const Segment SGI274007[] = {{&S17386[13], 7}, {&S17386[21], 2}, {&S17386[24], 15}, {&S17386[40], 6}, {&S17386[47], 19}, {&S17386[67], 6}, {&S17386[74], 11}, {&S17386[86], 19}, {&S17386[106], 13}, {NULL}};
const Segment SGI274008[] = {{&S17387[94], 4}, {NULL}};
const Segment SGI274009[] = {{&S17387[13], 7}, {&S17387[21], 2}, {&S17387[24], 15}, {&S17387[40], 6}, {&S17387[68], 17}, {&S17387[121], 8}, {NULL}};
const Segment SGI274010[] = {{&S17388[74], 4}, {NULL}};
const Segment SGI274011[] = {{&S17388[13], 14}, {&S17388[48], 17}, {NULL}};
const Segment SGI274012[] = {{&S17389[82], 4}, {NULL}};
const Segment SGI274013[] = {{&S17389[13], 14}, {&S17389[28], 6}, {&S17389[56], 17}, {NULL}};
const Segment SGI274014[] = {{&S17390[93], 4}, {NULL}};
const Segment SGI274015[] = {{&S17390[13], 7}, {&S17390[21], 2}, {&S17390[24], 15}, {&S17390[40], 6}, {&S17390[68], 16}, {NULL}};
const Segment SGI274016[] = {{&S17391[94], 4}, {NULL}};
const Segment SGI274017[] = {{&S17391[13], 7}, {&S17391[21], 2}, {&S17391[24], 15}, {&S17391[40], 6}, {&S17391[68], 17}, {NULL}};
const Segment SGI274018[] = {{&S17392[95], 4}, {NULL}};
const Segment SGI274019[] = {{&S17392[14], 7}, {&S17392[22], 2}, {&S17392[25], 15}, {&S17392[41], 6}, {&S17392[69], 17}, {NULL}};
const Segment SGI274020[] = {{&S17392[0], 1}, {NULL}};
const Segment SGI274021[] = {{&S12758[75], 4}, {NULL}};
const Segment SGI274022[] = {{&S12758[13], 14}, {&S12758[49], 17}, {NULL}};
const Segment SGI274023[] = {{&S17395[74], 4}, {NULL}};
const Segment SGI274024[] = {{&S17395[13], 14}, {&S17395[48], 17}, {NULL}};
const Segment SGI274025[] = {{&S12759[82], 4}, {NULL}};
const Segment SGI274026[] = {{&S12759[13], 14}, {&S12759[28], 6}, {&S12759[56], 17}, {NULL}};
const Segment SGI274027[] = {{&S17396[75], 4}, {NULL}};
const Segment SGI274028[] = {{&S17396[13], 14}, {&S17396[49], 17}, {&S17396[103], 8}, {&S17396[114], 26}, {NULL}};
const Segment SGI274029[] = {{&S12760[75], 4}, {NULL}};
const Segment SGI274030[] = {{&S12760[13], 14}, {&S12760[49], 17}, {&S12760[102], 8}, {NULL}};
const Segment SGI274031[] = {{&S17397[94], 4}, {NULL}};
const Segment SGI274032[] = {{&S17397[13], 7}, {&S17397[21], 2}, {&S17397[24], 15}, {&S17397[40], 6}, {&S17397[68], 17}, {&S17397[119], 8}, {&S17397[130], 24}, {NULL}};
const Segment SGI274033[] = {{&S12761[82], 4}, {NULL}};
const Segment SGI274034[] = {{&S12761[13], 14}, {&S12761[28], 6}, {&S12761[56], 17}, {&S12761[109], 8}, {NULL}};
const Segment SGI274035[] = {{&S17398[82], 4}, {NULL}};
const Segment SGI274036[] = {{&S17398[13], 14}, {&S17398[28], 6}, {&S17398[56], 17}, {NULL}};
const Segment SGI274037[] = {{&S12762[75], 4}, {NULL}};
const Segment SGI274038[] = {{&S12762[13], 14}, {&S12762[49], 17}, {&S12762[102], 8}, {&S12762[102], 8}, {NULL}};
const Segment SGI274039[] = {{&S17399[75], 4}, {NULL}};
const Segment SGI274040[] = {{&S17399[13], 14}, {&S17399[49], 17}, {&S17399[103], 8}, {&S17399[114], 24}, {NULL}};
const Segment SGI274041[] = {{&S12763[73], 4}, {NULL}};
const Segment SGI274042[] = {{&S12763[13], 14}, {&S12763[49], 17}, {NULL}};
const Segment SGI274043[] = {{&S12763[68], 9}, {NULL}};
const Segment SGI274044[] = {{&S17400[75], 4}, {NULL}};
const Segment SGI274045[] = {{&S17400[13], 14}, {&S17400[49], 17}, {NULL}};
const Segment SGI274046[] = {{&S12764[80], 4}, {NULL}};
const Segment SGI274047[] = {{&S12764[13], 14}, {&S12764[28], 6}, {&S12764[56], 17}, {NULL}};
const Segment SGI274048[] = {{&S12764[75], 9}, {NULL}};
const Segment SGI274049[] = {{&S17401[82], 4}, {NULL}};
const Segment SGI274050[] = {{&S17401[13], 14}, {&S17401[28], 6}, {&S17401[56], 17}, {NULL}};
const Segment SGI274051[] = {{&S12765[75], 4}, {NULL}};
const Segment SGI274052[] = {{&S12765[13], 14}, {&S12765[49], 17}, {&S12765[102], 8}, {&S12765[113], 26}, {NULL}};
const Segment SGI274053[] = {{&S17402[94], 4}, {NULL}};
const Segment SGI274054[] = {{&S17402[13], 7}, {&S17402[21], 2}, {&S17402[24], 15}, {&S17402[40], 6}, {&S17402[68], 17}, {NULL}};
const Segment SGI274055[] = {{&S12766[82], 4}, {NULL}};
const Segment SGI274056[] = {{&S12766[13], 14}, {&S12766[28], 6}, {&S12766[56], 17}, {&S12766[109], 8}, {&S12766[120], 26}, {NULL}};
const Segment SGI274057[] = {{&S17403[74], 4}, {NULL}};
const Segment SGI274058[] = {{&S17403[13], 14}, {&S17403[48], 17}, {NULL}};
const Segment SGI274059[] = {{&S17404[81], 4}, {NULL}};
const Segment SGI274060[] = {{&S17404[13], 14}, {&S17404[28], 6}, {&S17404[55], 17}, {NULL}};
const Segment SGI274061[] = {{&S17405[126], 4}, {NULL}};
const Segment SGI274062[] = {{&S17405[13], 7}, {&S17405[21], 2}, {&S17405[24], 15}, {&S17405[40], 6}, {&S17405[47], 29}, {&S17405[11], 1}, {&S17405[100], 17}, {NULL}};
const Segment SGI274063[] = {{&S17406[75], 4}, {NULL}};
const Segment SGI274064[] = {{&S17406[13], 14}, {&S17406[49], 17}, {NULL}};
const Segment SGI274065[] = {{&S17407[112], 4}, {NULL}};
const Segment SGI274066[] = {{&S17407[13], 7}, {&S17407[21], 2}, {&S17407[24], 15}, {&S17407[40], 6}, {&S17407[47], 15}, {&S17407[11], 1}, {&S17407[86], 17}, {NULL}};
const Segment SGI274067[] = {{&S17408[94], 4}, {NULL}};
const Segment SGI274068[] = {{&S17408[13], 7}, {&S17408[21], 2}, {&S17408[24], 15}, {&S17408[40], 6}, {&S17408[68], 17}, {NULL}};
const Segment SGI274069[] = {{&S17409[126], 4}, {NULL}};
const Segment SGI274070[] = {{&S17409[13], 7}, {&S17409[21], 2}, {&S17409[24], 15}, {&S17409[40], 6}, {&S17409[47], 29}, {&S17409[11], 1}, {&S17409[100], 17}, {NULL}};
const Segment SGI274071[] = {{&S17410[126], 4}, {NULL}};
const Segment SGI274072[] = {{&S17410[13], 7}, {&S17410[21], 2}, {&S17410[24], 15}, {&S17410[40], 6}, {&S17410[47], 29}, {&S17410[11], 1}, {&S17410[100], 17}, {NULL}};
const Segment SGI274073[] = {{&S17411[126], 4}, {NULL}};
const Segment SGI274074[] = {{&S17411[13], 7}, {&S17411[21], 2}, {&S17411[24], 15}, {&S17411[40], 6}, {&S17411[47], 29}, {&S17411[11], 1}, {&S17411[100], 17}, {NULL}};
const Segment SGI274075[] = {{&S17412[82], 4}, {NULL}};
const Segment SGI274076[] = {{&S17412[13], 14}, {&S17412[28], 6}, {&S17412[56], 17}, {NULL}};
const Segment SGI274077[] = {{&S17413[126], 4}, {NULL}};
const Segment SGI274078[] = {{&S17413[13], 7}, {&S17413[21], 2}, {&S17413[24], 15}, {&S17413[40], 6}, {&S17413[47], 29}, {&S17413[11], 1}, {&S17413[100], 17}, {NULL}};
const Segment SGI274079[] = {{&S17414[126], 4}, {NULL}};
const Segment SGI274080[] = {{&S17414[13], 7}, {&S17414[21], 2}, {&S17414[24], 15}, {&S17414[40], 6}, {&S17414[47], 29}, {&S17414[11], 1}, {&S17414[100], 17}, {NULL}};
const Segment SGI274081[] = {{&S17415[75], 4}, {NULL}};
const Segment SGI274082[] = {{&S17415[13], 14}, {&S17415[49], 17}, {&S17415[103], 8}, {NULL}};
const Segment SGI274083[] = {{&S17416[94], 4}, {NULL}};
const Segment SGI274084[] = {{&S17416[13], 7}, {&S17416[21], 2}, {&S17416[24], 15}, {&S17416[40], 6}, {&S17416[68], 17}, {&S17416[119], 8}, {NULL}};
const Segment SGI274085[] = {{&S17417[88], 4}, {NULL}};
const Segment SGI274086[] = {{&S17417[13], 7}, {&S17417[21], 2}, {&S17417[24], 15}, {&S17417[62], 17}, {NULL}};
const Segment SGI274087[] = {{&S17418[88], 4}, {NULL}};
const Segment SGI274088[] = {{&S17418[13], 7}, {&S17418[21], 2}, {&S17418[24], 15}, {&S17418[62], 17}, {NULL}};
const Segment SGI274089[] = {{&S17419[126], 4}, {NULL}};
const Segment SGI274090[] = {{&S17419[13], 7}, {&S17419[21], 2}, {&S17419[24], 15}, {&S17419[40], 6}, {&S17419[47], 29}, {&S17419[11], 1}, {&S17419[100], 17}, {NULL}};
const Segment SGI274091[] = {{&S17420[91], 4}, {NULL}};
const Segment SGI274092[] = {{&S17420[13], 7}, {&S17420[21], 2}, {&S17420[24], 15}, {&S17420[40], 3}, {&S17420[65], 17}, {NULL}};
const Segment SGI274093[] = {{&S17421[82], 4}, {NULL}};
const Segment SGI274094[] = {{&S17421[13], 14}, {&S17421[28], 6}, {&S17421[56], 17}, {NULL}};
const Segment SGI274095[] = {{&S17422[93], 4}, {NULL}};
const Segment SGI274096[] = {{&S17422[13], 7}, {&S17422[21], 2}, {&S17422[24], 15}, {&S17422[40], 6}, {&S17422[67], 17}, {NULL}};
const Segment SGI274097[] = {{&S17423[126], 4}, {NULL}};
const Segment SGI274098[] = {{&S17423[13], 7}, {&S17423[21], 2}, {&S17423[24], 15}, {&S17423[40], 6}, {&S17423[47], 29}, {&S17423[11], 1}, {&S17423[100], 17}, {NULL}};
const Segment SGI274099[] = {{&S12799[93], 4}, {NULL}};
const Segment SGI274100[] = {{&S12799[13], 9}, {&S12799[23], 22}, {&S12799[67], 17}, {NULL}};
const Segment SGI274101[] = {{&S17424[126], 4}, {NULL}};
const Segment SGI274102[] = {{&S17424[13], 7}, {&S17424[21], 2}, {&S17424[24], 15}, {&S17424[40], 6}, {&S17424[47], 29}, {&S17424[11], 1}, {&S17424[100], 17}, {NULL}};
const Segment SGI274103[] = {{&S12800[93], 4}, {NULL}};
const Segment SGI274104[] = {{&S12800[13], 9}, {&S12800[23], 22}, {&S12800[67], 17}, {NULL}};
const Segment SGI274105[] = {{&S17425[126], 4}, {NULL}};
const Segment SGI274106[] = {{&S12801[93], 4}, {NULL}};
const Segment SGI274107[] = {{&S12801[13], 9}, {&S12801[23], 22}, {&S12801[67], 17}, {NULL}};
const Segment SGI274108[] = {{&S17425[13], 7}, {&S17425[21], 2}, {&S17425[24], 15}, {&S17425[40], 6}, {&S17425[47], 29}, {&S17425[11], 1}, {&S17425[100], 17}, {NULL}};
const Segment SGI274109[] = {{&S12802[93], 4}, {NULL}};
const Segment SGI274110[] = {{&S12802[13], 9}, {&S12802[23], 22}, {&S12802[67], 17}, {NULL}};
const Segment SGI274111[] = {{&S17426[123], 4}, {NULL}};
const Segment SGI274112[] = {{&S17426[13], 7}, {&S17426[21], 2}, {&S17426[24], 15}, {&S17426[40], 6}, {&S17426[47], 26}, {&S17426[11], 1}, {&S17426[97], 17}, {NULL}};
const Segment SGI274113[] = {{&S12803[93], 4}, {NULL}};
const Segment SGI274114[] = {{&S12803[13], 9}, {&S12803[23], 22}, {&S12803[67], 17}, {NULL}};
const Segment SGI274115[] = {{&S17427[81], 4}, {NULL}};
const Segment SGI274116[] = {{&S17427[13], 14}, {&S17427[28], 6}, {&S17427[55], 17}, {NULL}};
const Segment SGI274117[] = {{&S12804[93], 4}, {NULL}};
const Segment SGI274118[] = {{&S12804[13], 9}, {&S12804[23], 22}, {&S12804[67], 17}, {NULL}};
const Segment SGI274119[] = {{&S17428[89], 4}, {NULL}};
const Segment SGI274120[] = {{&S17428[13], 14}, {&S17428[28], 6}, {&S17428[35], 6}, {&S17428[63], 17}, {NULL}};
const Segment SGI274121[] = {{&S12805[93], 4}, {NULL}};
const Segment SGI274122[] = {{&S12805[13], 9}, {&S12805[23], 22}, {&S12805[67], 17}, {&S12805[120], 8}, {NULL}};
const Segment SGI274123[] = {{&S12806[93], 4}, {NULL}};
const Segment SGI274124[] = {{&S12806[13], 9}, {&S12806[23], 22}, {&S12806[67], 17}, {&S12806[120], 8}, {&S12806[131], 26}, {NULL}};
const Segment SGI274125[] = {{&S12807[91], 4}, {NULL}};
const Segment SGI274126[] = {{&S12807[13], 9}, {&S12807[23], 22}, {&S12807[67], 17}, {NULL}};
const Segment SGI274127[] = {{&S12807[86], 9}, {NULL}};
const Segment SGI274128[] = {{&S20200[125], 3}, {NULL}};
const Segment SGI274129[] = {{&S20200[13], 7}, {&S20200[21], 2}, {&S20200[24], 15}, {&S20200[40], 6}, {&S20200[47], 29}, {&S20200[11], 1}, {&S20200[99], 17}, {NULL}};
const Segment SGI274130[] = {{&S20201[93], 3}, {NULL}};
const Segment SGI274131[] = {{&S20201[13], 7}, {&S20201[21], 2}, {&S20201[24], 15}, {&S20201[40], 6}, {&S20201[67], 17}, {NULL}};
const Segment SGI274132[] = {{&S20202[93], 3}, {NULL}};
const Segment SGI274133[] = {{&S20202[13], 7}, {&S20202[21], 2}, {&S20202[24], 15}, {&S20202[40], 6}, {&S20202[67], 17}, {&S20202[135], 8}, {&S20202[146], 24}, {NULL}};
const Segment SGI274134[] = {{&S20203[125], 3}, {NULL}};
const Segment SGI274135[] = {{&S20203[13], 7}, {&S20203[21], 2}, {&S20203[24], 15}, {&S20203[40], 6}, {&S20203[47], 29}, {&S20203[11], 1}, {&S20203[99], 17}, {NULL}};
const Segment SGI274136[] = {{&S20204[118], 3}, {NULL}};
const Segment SGI274137[] = {{&S20204[13], 7}, {&S20204[21], 2}, {&S20204[24], 15}, {&S20204[40], 6}, {&S20204[47], 22}, {&S20204[11], 1}, {&S20204[92], 17}, {NULL}};
const Segment SGI274138[] = {{&S20205[120], 3}, {NULL}};
const Segment SGI274139[] = {{&S20205[13], 7}, {&S20205[21], 2}, {&S20205[24], 15}, {&S20205[40], 6}, {&S20205[47], 24}, {&S20205[11], 1}, {&S20205[94], 17}, {NULL}};
const Segment SGI274140[] = {{&S20206[93], 3}, {NULL}};
const Segment SGI274141[] = {{&S20206[13], 7}, {&S20206[21], 2}, {&S20206[24], 15}, {&S20206[40], 6}, {&S20206[67], 17}, {NULL}};
const Segment SGI274142[] = {{&S20207[93], 3}, {NULL}};
const Segment SGI274143[] = {{&S20207[13], 7}, {&S20207[21], 2}, {&S20207[24], 15}, {&S20207[40], 6}, {&S20207[67], 17}, {NULL}};
const Segment SGI274144[] = {{&S20208[93], 3}, {NULL}};
const Segment SGI274145[] = {{&S20208[13], 7}, {&S20208[21], 2}, {&S20208[24], 15}, {&S20208[40], 6}, {&S20208[67], 17}, {&S20208[116], 8}, {NULL}};
const Segment SGI274146[] = {{&S31169[38], 5}, {NULL}};
const Segment SGI274147[] = {{&S31169[99], 5}, {NULL}};
const Segment SGI274148[] = {{&S17691[94], 4}, {NULL}};
const Segment SGI274149[] = {{&S17691[13], 7}, {&S17691[21], 2}, {&S17691[24], 15}, {&S17691[40], 6}, {&S17691[68], 17}, {NULL}};
const Segment SGI274150[] = {{&S17692[84], 4}, {NULL}};
const Segment SGI274151[] = {{&S17692[13], 7}, {&S17692[21], 15}, {&S17692[58], 17}, {NULL}};
const Segment SGI274152[] = {{&S17693[75], 4}, {NULL}};
const Segment SGI274153[] = {{&S17693[13], 14}, {&S17693[49], 17}, {NULL}};
const Segment SGI274154[] = {{&S17694[82], 4}, {NULL}};
const Segment SGI274155[] = {{&S17694[13], 14}, {&S17694[28], 6}, {&S17694[56], 17}, {NULL}};
const Segment SGI274156[] = {{&S17695[75], 4}, {NULL}};
const Segment SGI274157[] = {{&S17695[13], 14}, {&S17695[49], 17}, {&S17695[101], 8}, {&S17695[112], 24}, {NULL}};
const Segment SGI274158[] = {{&S17696[82], 4}, {NULL}};
const Segment SGI274159[] = {{&S17696[13], 14}, {&S17696[28], 6}, {&S17696[56], 17}, {&S17696[108], 8}, {&S17696[119], 24}, {NULL}};
const Segment SGI274160[] = {{&S17697[94], 4}, {NULL}};
const Segment SGI274161[] = {{&S17697[13], 7}, {&S17697[21], 2}, {&S17697[24], 15}, {&S17697[40], 6}, {&S17697[68], 17}, {&S17697[119], 8}, {&S17697[130], 24}, {NULL}};
const Segment SGI274162[] = {{&S17698[87], 4}, {NULL}};
const Segment SGI274163[] = {{&S17698[13], 7}, {&S17698[21], 2}, {&S17698[24], 15}, {&S17698[61], 17}, {NULL}};
const Segment SGI274164[] = {{&S17699[75], 4}, {&S17699[75], 4}, {NULL}};
const Segment SGI274165[] = {{&S17699[13], 14}, {&S17699[49], 17}, {&S17699[13], 14}, {&S17699[49], 17}, {NULL}};
const Segment SGI274166[] = {{&S17699[0], 99}, {NULL}};
const Segment SGI274167[] = {{&S17700[94], 4}, {NULL}};
const Segment SGI274168[] = {{&S17700[13], 7}, {&S17700[21], 2}, {&S17700[24], 15}, {&S17700[40], 6}, {&S17700[68], 17}, {NULL}};
const Segment SGI274169[] = {{&S17701[75], 4}, {NULL}};
const Segment SGI274170[] = {{&S17701[13], 14}, {&S17701[49], 17}, {&S17701[101], 8}, {NULL}};
const Segment SGI274171[] = {{&S17702[82], 4}, {NULL}};
const Segment SGI274172[] = {{&S17702[13], 14}, {&S17702[28], 6}, {&S17702[56], 17}, {&S17702[108], 8}, {NULL}};
const Segment SGI274173[] = {{&S17703[74], 4}, {NULL}};
const Segment SGI274174[] = {{&S17703[13], 14}, {&S17703[49], 16}, {NULL}};
const Segment SGI274175[] = {{&S17729[87], 4}, {NULL}};
const Segment SGI274176[] = {{&S17729[13], 3}, {&S17729[17], 2}, {&S17729[20], 11}, {&S17729[32], 6}, {&S17729[60], 17}, {NULL}};
const Segment SGI274177[] = {{&S17729[113], 10}, {NULL}};
const Segment SGI274178[] = {{&S17730[87], 4}, {NULL}};
const Segment SGI274179[] = {{&S17730[13], 3}, {&S17730[17], 2}, {&S17730[20], 11}, {&S17730[32], 6}, {&S17730[60], 17}, {NULL}};
const Segment SGI274180[] = {{&S17730[113], 10}, {NULL}};
const Segment SGI274181[] = {{&S17731[87], 4}, {NULL}};
const Segment SGI274182[] = {{&S17731[13], 3}, {&S17731[17], 2}, {&S17731[20], 11}, {&S17731[32], 6}, {&S17731[60], 17}, {NULL}};
const Segment SGI274183[] = {{&S17731[113], 10}, {NULL}};
const Segment SGI274184[] = {{&S17732[87], 4}, {NULL}};
const Segment SGI274185[] = {{&S17732[13], 3}, {&S17732[17], 2}, {&S17732[20], 11}, {&S17732[32], 6}, {&S17732[60], 17}, {NULL}};
const Segment SGI274186[] = {{&S17732[113], 10}, {NULL}};
const Segment SGI274187[] = {{&S17733[87], 4}, {NULL}};
const Segment SGI274188[] = {{&S17733[13], 3}, {&S17733[17], 2}, {&S17733[20], 11}, {&S17733[32], 6}, {&S17733[60], 17}, {NULL}};
const Segment SGI274189[] = {{&S17733[113], 10}, {NULL}};
const Segment SGI274190[] = {{&S17734[87], 4}, {NULL}};
const Segment SGI274191[] = {{&S17734[13], 3}, {&S17734[17], 2}, {&S17734[20], 11}, {&S17734[32], 6}, {&S17734[60], 17}, {NULL}};
const Segment SGI274192[] = {{&S17734[113], 10}, {NULL}};
const Segment SGI274193[] = {{&S17735[87], 4}, {NULL}};
const Segment SGI274194[] = {{&S17735[13], 3}, {&S17735[17], 2}, {&S17735[20], 11}, {&S17735[32], 6}, {&S17735[60], 17}, {NULL}};
const Segment SGI274195[] = {{&S17735[113], 10}, {NULL}};
const Segment SGI274196[] = {{&S17736[87], 4}, {NULL}};
const Segment SGI274197[] = {{&S17736[13], 3}, {&S17736[17], 2}, {&S17736[20], 11}, {&S17736[32], 6}, {&S17736[60], 17}, {NULL}};
const Segment SGI274198[] = {{&S17736[113], 10}, {NULL}};
const Segment SGI274199[] = {{&S17737[87], 4}, {NULL}};
const Segment SGI274200[] = {{&S17737[13], 3}, {&S17737[17], 2}, {&S17737[20], 11}, {&S17737[32], 6}, {&S17737[60], 17}, {NULL}};
const Segment SGI274201[] = {{&S17737[113], 10}, {NULL}};
const Segment SGI274202[] = {{&S17738[87], 4}, {NULL}};
const Segment SGI274203[] = {{&S17738[13], 3}, {&S17738[17], 2}, {&S17738[20], 11}, {&S17738[32], 6}, {&S17738[60], 17}, {NULL}};
const Segment SGI274204[] = {{&S17738[113], 10}, {NULL}};
const Segment SGI274205[] = {{&S17739[87], 4}, {NULL}};
const Segment SGI274206[] = {{&S17739[13], 3}, {&S17739[17], 2}, {&S17739[20], 11}, {&S17739[32], 6}, {&S17739[60], 17}, {NULL}};
const Segment SGI274207[] = {{&S17739[113], 10}, {NULL}};
const Segment SGI274208[] = {{&S17740[87], 4}, {NULL}};
const Segment SGI274209[] = {{&S17740[13], 3}, {&S17740[17], 2}, {&S17740[20], 11}, {&S17740[32], 6}, {&S17740[60], 17}, {NULL}};
const Segment SGI274210[] = {{&S17740[113], 10}, {NULL}};
const Segment SGI274211[] = {{&S17741[87], 4}, {NULL}};
const Segment SGI274212[] = {{&S17741[13], 3}, {&S17741[17], 2}, {&S17741[20], 11}, {&S17741[32], 6}, {&S17741[60], 17}, {NULL}};
const Segment SGI274213[] = {{&S17741[113], 10}, {NULL}};
const Segment SGI274214[] = {{&S17742[87], 4}, {NULL}};
const Segment SGI274215[] = {{&S17742[13], 3}, {&S17742[17], 2}, {&S17742[20], 11}, {&S17742[32], 6}, {&S17742[60], 17}, {NULL}};
const Segment SGI274216[] = {{&S17742[113], 10}, {NULL}};
const Segment SGI274217[] = {{&S17743[87], 4}, {NULL}};
const Segment SGI274218[] = {{&S17743[13], 3}, {&S17743[17], 2}, {&S17743[20], 11}, {&S17743[32], 6}, {&S17743[60], 17}, {NULL}};
const Segment SGI274219[] = {{&S17743[113], 10}, {NULL}};
const Segment SGI274220[] = {{&S17744[87], 4}, {NULL}};
const Segment SGI274221[] = {{&S17744[13], 3}, {&S17744[17], 2}, {&S17744[20], 11}, {&S17744[32], 6}, {&S17744[60], 17}, {NULL}};
const Segment SGI274222[] = {{&S17744[113], 10}, {NULL}};
const Segment SGI274223[] = {{&S17745[87], 4}, {NULL}};
const Segment SGI274224[] = {{&S17745[13], 3}, {&S17745[17], 2}, {&S17745[20], 11}, {&S17745[32], 6}, {&S17745[60], 17}, {NULL}};
const Segment SGI274225[] = {{&S17745[113], 10}, {NULL}};
const Segment SGI274226[] = {{&S17746[87], 4}, {NULL}};
const Segment SGI274227[] = {{&S17746[13], 3}, {&S17746[17], 2}, {&S17746[20], 11}, {&S17746[32], 6}, {&S17746[60], 17}, {NULL}};
const Segment SGI274228[] = {{&S17746[113], 10}, {NULL}};
const Segment SGI274229[] = {{&S17747[87], 4}, {NULL}};
const Segment SGI274230[] = {{&S17747[13], 3}, {&S17747[17], 2}, {&S17747[20], 11}, {&S17747[32], 6}, {&S17747[60], 17}, {NULL}};
const Segment SGI274231[] = {{&S17747[113], 10}, {NULL}};
const Segment SGI274232[] = {{&S17748[87], 4}, {NULL}};
const Segment SGI274233[] = {{&S17748[13], 3}, {&S17748[17], 2}, {&S17748[20], 11}, {&S17748[32], 6}, {&S17748[60], 17}, {NULL}};
const Segment SGI274234[] = {{&S17748[113], 10}, {NULL}};
const Segment SGI274235[] = {{&S17749[87], 4}, {NULL}};
const Segment SGI274236[] = {{&S17749[13], 3}, {&S17749[17], 2}, {&S17749[20], 11}, {&S17749[32], 6}, {&S17749[60], 17}, {NULL}};
const Segment SGI274237[] = {{&S17749[113], 10}, {NULL}};
const Segment SGI274238[] = {{&S17750[87], 4}, {NULL}};
const Segment SGI274239[] = {{&S17750[13], 3}, {&S17750[17], 2}, {&S17750[20], 11}, {&S17750[32], 6}, {&S17750[60], 17}, {NULL}};
const Segment SGI274240[] = {{&S17750[113], 10}, {NULL}};
const Segment SGI274241[] = {{&S17751[87], 4}, {NULL}};
const Segment SGI274242[] = {{&S17751[13], 3}, {&S17751[17], 2}, {&S17751[20], 11}, {&S17751[32], 6}, {&S17751[60], 17}, {NULL}};
const Segment SGI274243[] = {{&S17751[113], 10}, {NULL}};
const Segment SGI274244[] = {{&S17752[87], 4}, {NULL}};
const Segment SGI274245[] = {{&S17752[13], 3}, {&S17752[17], 2}, {&S17752[20], 11}, {&S17752[32], 6}, {&S17752[60], 17}, {NULL}};
const Segment SGI274246[] = {{&S17752[113], 10}, {NULL}};
const Segment SGI274247[] = {{&S17753[87], 4}, {NULL}};
const Segment SGI274248[] = {{&S17753[13], 3}, {&S17753[17], 2}, {&S17753[20], 11}, {&S17753[32], 6}, {&S17753[60], 17}, {NULL}};
const Segment SGI274249[] = {{&S17753[113], 10}, {NULL}};
const Segment SGI274250[] = {{&S17754[87], 4}, {NULL}};
const Segment SGI274251[] = {{&S17754[13], 3}, {&S17754[17], 2}, {&S17754[20], 11}, {&S17754[32], 6}, {&S17754[60], 17}, {NULL}};
const Segment SGI274252[] = {{&S17754[113], 10}, {NULL}};
const Segment SGI274253[] = {{&S17755[87], 4}, {NULL}};
const Segment SGI274254[] = {{&S17755[13], 3}, {&S17755[17], 2}, {&S17755[20], 11}, {&S17755[32], 6}, {&S17755[60], 17}, {NULL}};
const Segment SGI274255[] = {{&S17755[113], 10}, {NULL}};
const Segment SGI274256[] = {{&S17756[87], 4}, {NULL}};
const Segment SGI274257[] = {{&S17756[13], 3}, {&S17756[17], 2}, {&S17756[20], 11}, {&S17756[32], 6}, {&S17756[60], 17}, {NULL}};
const Segment SGI274258[] = {{&S17756[113], 10}, {NULL}};
const Segment SGI274259[] = {{&S17757[87], 4}, {NULL}};
const Segment SGI274260[] = {{&S17757[13], 3}, {&S17757[17], 2}, {&S17757[20], 11}, {&S17757[32], 6}, {&S17757[60], 17}, {NULL}};
const Segment SGI274261[] = {{&S17757[113], 10}, {NULL}};
const Segment SGI274262[] = {{&S17758[87], 4}, {NULL}};
const Segment SGI274263[] = {{&S17758[13], 3}, {&S17758[17], 2}, {&S17758[20], 11}, {&S17758[32], 6}, {&S17758[60], 17}, {NULL}};
const Segment SGI274264[] = {{&S17758[113], 10}, {NULL}};
const Segment SGI274265[] = {{&S17759[87], 4}, {NULL}};
const Segment SGI274266[] = {{&S17759[13], 3}, {&S17759[17], 2}, {&S17759[20], 11}, {&S17759[32], 6}, {&S17759[60], 17}, {NULL}};
const Segment SGI274267[] = {{&S17759[113], 10}, {NULL}};
const Segment SGI274268[] = {{&S17760[87], 4}, {NULL}};
const Segment SGI274269[] = {{&S17760[13], 3}, {&S17760[17], 2}, {&S17760[20], 11}, {&S17760[32], 6}, {&S17760[60], 17}, {NULL}};
const Segment SGI274270[] = {{&S17760[113], 10}, {NULL}};
const Segment SGI274271[] = {{&S17761[87], 4}, {NULL}};
const Segment SGI274272[] = {{&S17761[13], 3}, {&S17761[17], 2}, {&S17761[20], 11}, {&S17761[32], 6}, {&S17761[60], 17}, {NULL}};
const Segment SGI274273[] = {{&S17761[113], 10}, {NULL}};
const Segment SGI274274[] = {{&S17762[87], 4}, {NULL}};
const Segment SGI274275[] = {{&S17762[13], 3}, {&S17762[17], 2}, {&S17762[20], 11}, {&S17762[32], 6}, {&S17762[60], 17}, {NULL}};
const Segment SGI274276[] = {{&S17762[113], 10}, {NULL}};
const Segment SGI274277[] = {{&S17763[87], 4}, {NULL}};
const Segment SGI274278[] = {{&S17763[13], 3}, {&S17763[17], 2}, {&S17763[20], 11}, {&S17763[32], 6}, {&S17763[60], 17}, {NULL}};
const Segment SGI274279[] = {{&S17763[113], 10}, {NULL}};
const Segment SGI274280[] = {{&S17764[87], 4}, {NULL}};
const Segment SGI274281[] = {{&S17764[13], 3}, {&S17764[17], 2}, {&S17764[20], 11}, {&S17764[32], 6}, {&S17764[60], 17}, {NULL}};
const Segment SGI274282[] = {{&S17764[113], 10}, {NULL}};
const Segment SGI274283[] = {{&S17765[87], 4}, {NULL}};
const Segment SGI274284[] = {{&S17765[13], 3}, {&S17765[17], 2}, {&S17765[20], 11}, {&S17765[32], 6}, {&S17765[60], 17}, {NULL}};
const Segment SGI274285[] = {{&S17765[113], 10}, {NULL}};
const Segment SGI274286[] = {{&S17766[87], 4}, {NULL}};
const Segment SGI274287[] = {{&S17766[13], 3}, {&S17766[17], 2}, {&S17766[20], 11}, {&S17766[32], 6}, {&S17766[60], 17}, {NULL}};
const Segment SGI274288[] = {{&S17766[113], 10}, {NULL}};
const Segment SGI274289[] = {{&S17767[87], 4}, {NULL}};
const Segment SGI274290[] = {{&S17767[13], 3}, {&S17767[17], 2}, {&S17767[20], 11}, {&S17767[32], 6}, {&S17767[60], 17}, {NULL}};
const Segment SGI274291[] = {{&S17767[113], 10}, {NULL}};
const Segment SGI274292[] = {{&S17768[87], 4}, {NULL}};
const Segment SGI274293[] = {{&S17768[13], 3}, {&S17768[17], 2}, {&S17768[20], 11}, {&S17768[32], 6}, {&S17768[60], 17}, {NULL}};
const Segment SGI274294[] = {{&S17768[113], 10}, {NULL}};
const Segment SGI274295[] = {{&S17769[87], 4}, {NULL}};
const Segment SGI274296[] = {{&S17769[13], 3}, {&S17769[17], 2}, {&S17769[20], 11}, {&S17769[32], 6}, {&S17769[60], 17}, {NULL}};
const Segment SGI274297[] = {{&S17769[113], 10}, {NULL}};
const Segment SGI274298[] = {{&S17770[87], 4}, {NULL}};
const Segment SGI274299[] = {{&S17770[13], 3}, {&S17770[17], 2}, {&S17770[20], 11}, {&S17770[32], 6}, {&S17770[60], 17}, {NULL}};
const Segment SGI274300[] = {{&S17770[113], 10}, {NULL}};
const Segment SGI274301[] = {{&S17771[87], 4}, {NULL}};
const Segment SGI274302[] = {{&S17771[13], 3}, {&S17771[17], 2}, {&S17771[20], 11}, {&S17771[32], 6}, {&S17771[60], 17}, {NULL}};
const Segment SGI274303[] = {{&S17771[113], 10}, {NULL}};
const Segment SGI274304[] = {{&S17772[87], 4}, {NULL}};
const Segment SGI274305[] = {{&S17772[13], 3}, {&S17772[17], 2}, {&S17772[20], 11}, {&S17772[32], 6}, {&S17772[60], 17}, {NULL}};
const Segment SGI274306[] = {{&S17772[113], 10}, {NULL}};
const Segment SGI274307[] = {{&S17773[87], 4}, {NULL}};
const Segment SGI274308[] = {{&S17773[13], 3}, {&S17773[17], 2}, {&S17773[20], 11}, {&S17773[32], 6}, {&S17773[60], 17}, {NULL}};
const Segment SGI274309[] = {{&S17773[113], 10}, {NULL}};
const Segment SGI274310[] = {{&S17774[87], 4}, {NULL}};
const Segment SGI274311[] = {{&S17774[13], 3}, {&S17774[17], 2}, {&S17774[20], 11}, {&S17774[32], 6}, {&S17774[60], 17}, {NULL}};
const Segment SGI274312[] = {{&S17774[113], 10}, {NULL}};
const Segment SGI274313[] = {{&S17775[87], 4}, {NULL}};
const Segment SGI274314[] = {{&S17775[13], 3}, {&S17775[17], 2}, {&S17775[20], 11}, {&S17775[32], 6}, {&S17775[60], 17}, {NULL}};
const Segment SGI274315[] = {{&S17775[113], 10}, {NULL}};
const Segment SGI274316[] = {{&S17776[87], 4}, {NULL}};
const Segment SGI274317[] = {{&S17776[13], 3}, {&S17776[17], 2}, {&S17776[20], 11}, {&S17776[32], 6}, {&S17776[60], 17}, {NULL}};
const Segment SGI274318[] = {{&S17776[113], 10}, {NULL}};
const Segment SGI274319[] = {{&S17777[87], 4}, {NULL}};
const Segment SGI274320[] = {{&S17777[13], 3}, {&S17777[17], 2}, {&S17777[20], 11}, {&S17777[32], 6}, {&S17777[60], 17}, {NULL}};
const Segment SGI274321[] = {{&S17777[113], 10}, {NULL}};
const Segment SGI274322[] = {{&S17778[87], 4}, {NULL}};
const Segment SGI274323[] = {{&S17778[13], 3}, {&S17778[17], 2}, {&S17778[20], 11}, {&S17778[32], 6}, {&S17778[60], 17}, {NULL}};
const Segment SGI274324[] = {{&S17778[113], 10}, {NULL}};
const Segment SGI274325[] = {{&S17779[87], 4}, {NULL}};
const Segment SGI274326[] = {{&S17779[13], 3}, {&S17779[17], 2}, {&S17779[20], 11}, {&S17779[32], 6}, {&S17779[60], 17}, {NULL}};
const Segment SGI274327[] = {{&S17779[113], 10}, {NULL}};
const Segment SGI274328[] = {{&S17780[87], 4}, {NULL}};
const Segment SGI274329[] = {{&S17780[13], 3}, {&S17780[17], 2}, {&S17780[20], 11}, {&S17780[32], 6}, {&S17780[60], 17}, {NULL}};
const Segment SGI274330[] = {{&S17780[113], 10}, {NULL}};
const Segment SGI274331[] = {{&S17781[87], 4}, {NULL}};
const Segment SGI274332[] = {{&S17781[13], 3}, {&S17781[17], 2}, {&S17781[20], 11}, {&S17781[32], 6}, {&S17781[60], 17}, {NULL}};
const Segment SGI274333[] = {{&S17781[113], 10}, {NULL}};
const Segment SGI274334[] = {{&S17782[87], 4}, {NULL}};
const Segment SGI274335[] = {{&S17782[13], 3}, {&S17782[17], 2}, {&S17782[20], 11}, {&S17782[32], 6}, {&S17782[60], 17}, {NULL}};
const Segment SGI274336[] = {{&S17782[113], 10}, {NULL}};
const Segment SGI274337[] = {{&S17783[87], 4}, {NULL}};
const Segment SGI274338[] = {{&S17783[13], 3}, {&S17783[17], 2}, {&S17783[20], 11}, {&S17783[32], 6}, {&S17783[60], 17}, {NULL}};
const Segment SGI274339[] = {{&S17783[113], 10}, {NULL}};
const Segment SGI274340[] = {{&S17784[87], 4}, {NULL}};
const Segment SGI274341[] = {{&S17784[13], 3}, {&S17784[17], 2}, {&S17784[20], 11}, {&S17784[32], 6}, {&S17784[60], 17}, {NULL}};
const Segment SGI274342[] = {{&S17784[113], 10}, {NULL}};
const Segment SGI274343[] = {{&S17785[87], 4}, {NULL}};
const Segment SGI274344[] = {{&S17785[13], 3}, {&S17785[17], 2}, {&S17785[20], 11}, {&S17785[32], 6}, {&S17785[60], 17}, {NULL}};
const Segment SGI274345[] = {{&S17785[113], 10}, {NULL}};
const Segment SGI274346[] = {{&S17786[87], 4}, {NULL}};
const Segment SGI274347[] = {{&S17786[13], 3}, {&S17786[17], 2}, {&S17786[20], 11}, {&S17786[32], 6}, {&S17786[60], 17}, {NULL}};
const Segment SGI274348[] = {{&S17786[113], 10}, {NULL}};
const Segment SGI274349[] = {{&S17787[87], 4}, {NULL}};
const Segment SGI274350[] = {{&S17787[13], 3}, {&S17787[17], 2}, {&S17787[20], 11}, {&S17787[32], 6}, {&S17787[60], 17}, {NULL}};
const Segment SGI274351[] = {{&S17787[113], 10}, {NULL}};
const Segment SGI274352[] = {{&S17788[87], 4}, {NULL}};
const Segment SGI274353[] = {{&S17788[13], 3}, {&S17788[17], 2}, {&S17788[20], 11}, {&S17788[32], 6}, {&S17788[60], 17}, {NULL}};
const Segment SGI274354[] = {{&S17788[113], 10}, {NULL}};
const Segment SGI274355[] = {{&S17789[87], 4}, {NULL}};
const Segment SGI274356[] = {{&S17789[13], 3}, {&S17789[17], 2}, {&S17789[20], 11}, {&S17789[32], 6}, {&S17789[60], 17}, {NULL}};
const Segment SGI274357[] = {{&S17789[113], 10}, {NULL}};
const Segment SGI274358[] = {{&S17790[87], 4}, {NULL}};
const Segment SGI274359[] = {{&S17790[13], 3}, {&S17790[17], 2}, {&S17790[20], 11}, {&S17790[32], 6}, {&S17790[60], 17}, {NULL}};
const Segment SGI274360[] = {{&S17790[113], 10}, {NULL}};
const Segment SGI274361[] = {{&S17791[87], 4}, {NULL}};
const Segment SGI274362[] = {{&S17791[13], 3}, {&S17791[17], 2}, {&S17791[20], 11}, {&S17791[32], 6}, {&S17791[60], 17}, {NULL}};
const Segment SGI274363[] = {{&S17791[113], 10}, {NULL}};
const Segment SGI274364[] = {{&S17792[87], 4}, {NULL}};
const Segment SGI274365[] = {{&S17792[13], 3}, {&S17792[17], 2}, {&S17792[20], 11}, {&S17792[32], 6}, {&S17792[60], 17}, {NULL}};
const Segment SGI274366[] = {{&S17792[113], 10}, {NULL}};
const Segment SGI274367[] = {{&S17793[87], 4}, {NULL}};
const Segment SGI274368[] = {{&S17793[13], 3}, {&S17793[17], 2}, {&S17793[20], 11}, {&S17793[32], 6}, {&S17793[60], 17}, {NULL}};
const Segment SGI274369[] = {{&S17793[113], 10}, {NULL}};
const Segment SGI274370[] = {{&S17794[87], 4}, {NULL}};
const Segment SGI274371[] = {{&S17794[13], 3}, {&S17794[17], 2}, {&S17794[20], 11}, {&S17794[32], 6}, {&S17794[60], 17}, {NULL}};
const Segment SGI274372[] = {{&S17794[113], 10}, {NULL}};
const Segment SGI274373[] = {{&S17795[87], 4}, {NULL}};
const Segment SGI274374[] = {{&S17795[13], 3}, {&S17795[17], 2}, {&S17795[20], 11}, {&S17795[32], 6}, {&S17795[60], 17}, {NULL}};
const Segment SGI274375[] = {{&S17795[113], 10}, {NULL}};
const Segment SGI274376[] = {{&S17796[87], 4}, {NULL}};
const Segment SGI274377[] = {{&S17796[13], 3}, {&S17796[17], 2}, {&S17796[20], 11}, {&S17796[32], 6}, {&S17796[60], 17}, {NULL}};
const Segment SGI274378[] = {{&S17796[113], 10}, {NULL}};
const Segment SGI274379[] = {{&S17797[87], 4}, {NULL}};
const Segment SGI274380[] = {{&S17797[13], 3}, {&S17797[17], 2}, {&S17797[20], 11}, {&S17797[32], 6}, {&S17797[60], 17}, {NULL}};
const Segment SGI274381[] = {{&S17797[113], 10}, {NULL}};
const Segment SGI274382[] = {{&S17798[87], 4}, {NULL}};
const Segment SGI274383[] = {{&S17798[13], 3}, {&S17798[17], 2}, {&S17798[20], 11}, {&S17798[32], 6}, {&S17798[60], 17}, {NULL}};
const Segment SGI274384[] = {{&S17798[113], 10}, {NULL}};
const Segment SGI274385[] = {{&S17799[87], 4}, {NULL}};
const Segment SGI274386[] = {{&S17799[13], 3}, {&S17799[17], 2}, {&S17799[20], 11}, {&S17799[32], 6}, {&S17799[60], 17}, {NULL}};
const Segment SGI274387[] = {{&S17799[113], 10}, {NULL}};
const Segment SGI274388[] = {{&S17800[87], 4}, {NULL}};
const Segment SGI274389[] = {{&S17800[13], 3}, {&S17800[17], 2}, {&S17800[20], 11}, {&S17800[32], 6}, {&S17800[60], 17}, {NULL}};
const Segment SGI274390[] = {{&S17800[113], 10}, {NULL}};
const Segment SGI274391[] = {{&S17801[87], 4}, {NULL}};
const Segment SGI274392[] = {{&S17801[13], 3}, {&S17801[17], 2}, {&S17801[20], 11}, {&S17801[32], 6}, {&S17801[60], 17}, {NULL}};
const Segment SGI274393[] = {{&S17801[113], 10}, {NULL}};
const Segment SGI274394[] = {{&S17802[87], 4}, {NULL}};
const Segment SGI274395[] = {{&S17802[13], 3}, {&S17802[17], 2}, {&S17802[20], 11}, {&S17802[32], 6}, {&S17802[60], 17}, {NULL}};
const Segment SGI274396[] = {{&S17802[113], 10}, {NULL}};
const Segment SGI274397[] = {{&S17803[87], 4}, {NULL}};
const Segment SGI274398[] = {{&S17803[13], 3}, {&S17803[17], 2}, {&S17803[20], 11}, {&S17803[32], 6}, {&S17803[60], 17}, {NULL}};
const Segment SGI274399[] = {{&S17803[113], 10}, {NULL}};
const Segment SGI274400[] = {{&S17804[87], 4}, {NULL}};
const Segment SGI274401[] = {{&S17804[13], 3}, {&S17804[17], 2}, {&S17804[20], 11}, {&S17804[32], 6}, {&S17804[60], 17}, {NULL}};
const Segment SGI274402[] = {{&S17804[113], 10}, {NULL}};
const Segment SGI274403[] = {{&S17805[87], 4}, {NULL}};
const Segment SGI274404[] = {{&S17805[13], 3}, {&S17805[17], 2}, {&S17805[20], 11}, {&S17805[32], 6}, {&S17805[60], 17}, {NULL}};
const Segment SGI274405[] = {{&S17805[113], 10}, {NULL}};
const Segment SGI274406[] = {{&S17806[87], 4}, {NULL}};
const Segment SGI274407[] = {{&S17806[13], 3}, {&S17806[17], 2}, {&S17806[20], 11}, {&S17806[32], 6}, {&S17806[60], 17}, {NULL}};
const Segment SGI274408[] = {{&S17806[113], 10}, {NULL}};
const Segment SGI274409[] = {{&S17807[87], 4}, {NULL}};
const Segment SGI274410[] = {{&S17807[13], 3}, {&S17807[17], 2}, {&S17807[20], 11}, {&S17807[32], 6}, {&S17807[60], 17}, {NULL}};
const Segment SGI274411[] = {{&S17807[113], 10}, {NULL}};
const Segment SGI274412[] = {{&S17808[87], 4}, {NULL}};
const Segment SGI274413[] = {{&S17808[13], 3}, {&S17808[17], 2}, {&S17808[20], 11}, {&S17808[32], 6}, {&S17808[60], 17}, {NULL}};
const Segment SGI274414[] = {{&S17808[113], 10}, {NULL}};
const Segment SGI274415[] = {{&S17809[87], 4}, {NULL}};
const Segment SGI274416[] = {{&S17809[13], 3}, {&S17809[17], 2}, {&S17809[20], 11}, {&S17809[32], 6}, {&S17809[60], 17}, {NULL}};
const Segment SGI274417[] = {{&S17809[113], 10}, {NULL}};
const Segment SGI274418[] = {{&S17810[87], 4}, {NULL}};
const Segment SGI274419[] = {{&S17810[13], 3}, {&S17810[17], 2}, {&S17810[20], 11}, {&S17810[32], 6}, {&S17810[60], 17}, {NULL}};
const Segment SGI274420[] = {{&S17810[113], 10}, {NULL}};
const Segment SGI274421[] = {{&S17811[87], 4}, {NULL}};
const Segment SGI274422[] = {{&S17811[13], 3}, {&S17811[17], 2}, {&S17811[20], 11}, {&S17811[32], 6}, {&S17811[60], 17}, {NULL}};
const Segment SGI274423[] = {{&S17811[113], 10}, {NULL}};
const Segment SGI274424[] = {{&S17812[87], 4}, {NULL}};
const Segment SGI274425[] = {{&S17812[13], 3}, {&S17812[17], 2}, {&S17812[20], 11}, {&S17812[32], 6}, {&S17812[60], 17}, {NULL}};
const Segment SGI274426[] = {{&S17812[113], 10}, {NULL}};
const Segment SGI274427[] = {{&S17813[87], 4}, {NULL}};
const Segment SGI274428[] = {{&S17813[13], 3}, {&S17813[17], 2}, {&S17813[20], 11}, {&S17813[32], 6}, {&S17813[60], 17}, {NULL}};
const Segment SGI274429[] = {{&S17813[113], 10}, {NULL}};
const Segment SGI274430[] = {{&S17814[87], 4}, {NULL}};
const Segment SGI274431[] = {{&S17814[13], 3}, {&S17814[17], 2}, {&S17814[20], 11}, {&S17814[32], 6}, {&S17814[60], 17}, {NULL}};
const Segment SGI274432[] = {{&S17814[113], 10}, {NULL}};
const Segment SGI274433[] = {{&S17815[87], 4}, {NULL}};
const Segment SGI274434[] = {{&S17815[13], 3}, {&S17815[17], 2}, {&S17815[20], 11}, {&S17815[32], 6}, {&S17815[60], 17}, {NULL}};
const Segment SGI274435[] = {{&S17815[113], 10}, {NULL}};
const Segment SGI274436[] = {{&S17816[87], 4}, {NULL}};
const Segment SGI274437[] = {{&S17816[13], 3}, {&S17816[17], 2}, {&S17816[20], 11}, {&S17816[32], 6}, {&S17816[60], 17}, {NULL}};
const Segment SGI274438[] = {{&S17816[113], 10}, {NULL}};
const Segment SGI274439[] = {{&S17817[87], 4}, {NULL}};
const Segment SGI274440[] = {{&S17817[13], 3}, {&S17817[17], 2}, {&S17817[20], 11}, {&S17817[32], 6}, {&S17817[60], 17}, {NULL}};
const Segment SGI274441[] = {{&S17817[113], 10}, {NULL}};
const Segment SGI274442[] = {{&S17818[87], 4}, {NULL}};
const Segment SGI274443[] = {{&S17818[13], 3}, {&S17818[17], 2}, {&S17818[20], 11}, {&S17818[32], 6}, {&S17818[60], 17}, {NULL}};
const Segment SGI274444[] = {{&S17818[113], 10}, {NULL}};
const Segment SGI274445[] = {{&S17819[87], 4}, {NULL}};
const Segment SGI274446[] = {{&S17819[13], 3}, {&S17819[17], 2}, {&S17819[20], 11}, {&S17819[32], 6}, {&S17819[60], 17}, {NULL}};
const Segment SGI274447[] = {{&S17819[113], 10}, {NULL}};
const Segment SGI274448[] = {{&S17820[87], 4}, {NULL}};
const Segment SGI274449[] = {{&S17820[13], 3}, {&S17820[17], 2}, {&S17820[20], 11}, {&S17820[32], 6}, {&S17820[60], 17}, {NULL}};
const Segment SGI274450[] = {{&S17820[113], 10}, {NULL}};
const Segment SGI274451[] = {{&S17821[87], 4}, {NULL}};
const Segment SGI274452[] = {{&S17821[13], 3}, {&S17821[17], 2}, {&S17821[20], 11}, {&S17821[32], 6}, {&S17821[60], 17}, {NULL}};
const Segment SGI274453[] = {{&S17821[113], 10}, {NULL}};
const Segment SGI274454[] = {{&S17822[87], 4}, {NULL}};
const Segment SGI274455[] = {{&S17822[13], 3}, {&S17822[17], 2}, {&S17822[20], 11}, {&S17822[32], 6}, {&S17822[60], 17}, {NULL}};
const Segment SGI274456[] = {{&S17822[113], 10}, {NULL}};
const Segment SGI274457[] = {{&S17823[87], 4}, {NULL}};
const Segment SGI274458[] = {{&S17823[13], 3}, {&S17823[17], 2}, {&S17823[20], 11}, {&S17823[32], 6}, {&S17823[60], 17}, {NULL}};
const Segment SGI274459[] = {{&S17823[113], 10}, {NULL}};
const Segment SGI274460[] = {{&S17824[87], 4}, {NULL}};
const Segment SGI274461[] = {{&S17824[13], 3}, {&S17824[17], 2}, {&S17824[20], 11}, {&S17824[32], 6}, {&S17824[60], 17}, {NULL}};
const Segment SGI274462[] = {{&S17824[113], 10}, {NULL}};
const Segment SGI274463[] = {{&S17825[87], 4}, {NULL}};
const Segment SGI274464[] = {{&S17825[13], 3}, {&S17825[17], 2}, {&S17825[20], 11}, {&S17825[32], 6}, {&S17825[60], 17}, {NULL}};
const Segment SGI274465[] = {{&S17825[113], 10}, {NULL}};
const Segment SGI274466[] = {{&S17826[87], 4}, {NULL}};
const Segment SGI274467[] = {{&S17826[13], 3}, {&S17826[17], 2}, {&S17826[20], 11}, {&S17826[32], 6}, {&S17826[60], 17}, {NULL}};
const Segment SGI274468[] = {{&S17826[113], 10}, {NULL}};
const Segment SGI274469[] = {{&S17827[89], 4}, {NULL}};
const Segment SGI274470[] = {{&S17827[13], 3}, {&S17827[17], 2}, {&S17827[20], 13}, {&S17827[34], 6}, {&S17827[62], 17}, {NULL}};
const Segment SGI274471[] = {{&S17827[115], 10}, {NULL}};
const Segment SGI274472[] = {{&S17828[89], 4}, {NULL}};
const Segment SGI274473[] = {{&S17828[13], 3}, {&S17828[17], 2}, {&S17828[20], 13}, {&S17828[34], 6}, {&S17828[62], 17}, {NULL}};
const Segment SGI274474[] = {{&S17828[115], 10}, {NULL}};
const Segment SGI274475[] = {{&S17829[89], 4}, {NULL}};
const Segment SGI274476[] = {{&S17829[13], 3}, {&S17829[17], 2}, {&S17829[20], 13}, {&S17829[34], 6}, {&S17829[62], 17}, {NULL}};
const Segment SGI274477[] = {{&S17829[115], 10}, {NULL}};
const Segment SGI274478[] = {{&S17830[89], 4}, {NULL}};
const Segment SGI274479[] = {{&S17830[13], 3}, {&S17830[17], 2}, {&S17830[20], 13}, {&S17830[34], 6}, {&S17830[62], 17}, {NULL}};
const Segment SGI274480[] = {{&S17830[115], 10}, {NULL}};
const Segment SGI274481[] = {{&S17831[89], 4}, {NULL}};
const Segment SGI274482[] = {{&S17831[13], 3}, {&S17831[17], 2}, {&S17831[20], 13}, {&S17831[34], 6}, {&S17831[62], 17}, {NULL}};
const Segment SGI274483[] = {{&S17831[115], 10}, {NULL}};
const Segment SGI274484[] = {{&S17832[89], 4}, {NULL}};
const Segment SGI274485[] = {{&S17832[13], 3}, {&S17832[17], 2}, {&S17832[20], 13}, {&S17832[34], 6}, {&S17832[62], 17}, {NULL}};
const Segment SGI274486[] = {{&S17832[115], 10}, {NULL}};
const Segment SGI274487[] = {{&S17833[89], 4}, {NULL}};
const Segment SGI274488[] = {{&S17833[13], 3}, {&S17833[17], 2}, {&S17833[20], 13}, {&S17833[34], 6}, {&S17833[62], 17}, {NULL}};
const Segment SGI274489[] = {{&S17833[115], 10}, {NULL}};
const Segment SGI274490[] = {{&S17834[89], 4}, {NULL}};
const Segment SGI274491[] = {{&S17834[13], 3}, {&S17834[17], 2}, {&S17834[20], 13}, {&S17834[34], 6}, {&S17834[62], 17}, {NULL}};
const Segment SGI274492[] = {{&S17834[115], 10}, {NULL}};
const Segment SGI274493[] = {{&S17835[89], 4}, {NULL}};
const Segment SGI274494[] = {{&S17835[13], 3}, {&S17835[17], 2}, {&S17835[20], 13}, {&S17835[34], 6}, {&S17835[62], 17}, {NULL}};
const Segment SGI274495[] = {{&S17835[115], 10}, {NULL}};
const Segment SGI274496[] = {{&S17836[89], 4}, {NULL}};
const Segment SGI274497[] = {{&S17836[13], 3}, {&S17836[17], 2}, {&S17836[20], 13}, {&S17836[34], 6}, {&S17836[62], 17}, {NULL}};
const Segment SGI274498[] = {{&S17836[115], 10}, {NULL}};
const Segment SGI274499[] = {{&S17837[89], 4}, {NULL}};
const Segment SGI274500[] = {{&S17837[13], 3}, {&S17837[17], 2}, {&S17837[20], 13}, {&S17837[34], 6}, {&S17837[62], 17}, {NULL}};
const Segment SGI274501[] = {{&S17837[115], 10}, {NULL}};
const Segment SGI274502[] = {{&S17838[89], 4}, {NULL}};
const Segment SGI274503[] = {{&S17838[13], 3}, {&S17838[17], 2}, {&S17838[20], 13}, {&S17838[34], 6}, {&S17838[62], 17}, {NULL}};
const Segment SGI274504[] = {{&S17838[115], 10}, {NULL}};
const Segment SGI274505[] = {{&S17839[89], 4}, {NULL}};
const Segment SGI274506[] = {{&S17839[13], 3}, {&S17839[17], 2}, {&S17839[20], 13}, {&S17839[34], 6}, {&S17839[62], 17}, {NULL}};
const Segment SGI274507[] = {{&S17839[115], 10}, {NULL}};
const Segment SGI274508[] = {{&S17840[89], 4}, {NULL}};
const Segment SGI274509[] = {{&S17840[13], 3}, {&S17840[17], 2}, {&S17840[20], 13}, {&S17840[34], 6}, {&S17840[62], 17}, {NULL}};
const Segment SGI274510[] = {{&S17840[115], 10}, {NULL}};
const Segment SGI274511[] = {{&S17841[89], 4}, {NULL}};
const Segment SGI274512[] = {{&S17841[13], 3}, {&S17841[17], 2}, {&S17841[20], 13}, {&S17841[34], 6}, {&S17841[62], 17}, {NULL}};
const Segment SGI274513[] = {{&S17841[115], 10}, {NULL}};
const Segment SGI274514[] = {{&S17842[89], 4}, {NULL}};
const Segment SGI274515[] = {{&S17842[13], 3}, {&S17842[17], 2}, {&S17842[20], 13}, {&S17842[34], 6}, {&S17842[62], 17}, {NULL}};
const Segment SGI274516[] = {{&S17842[115], 10}, {NULL}};
const Segment SGI274517[] = {{&S17843[89], 4}, {NULL}};
const Segment SGI274518[] = {{&S17843[13], 3}, {&S17843[17], 2}, {&S17843[20], 13}, {&S17843[34], 6}, {&S17843[62], 17}, {NULL}};
const Segment SGI274519[] = {{&S17843[115], 10}, {NULL}};
const Segment SGI274520[] = {{&S17844[89], 4}, {NULL}};
const Segment SGI274521[] = {{&S17844[13], 3}, {&S17844[17], 2}, {&S17844[20], 13}, {&S17844[34], 6}, {&S17844[62], 17}, {NULL}};
const Segment SGI274522[] = {{&S17844[115], 10}, {NULL}};
const Segment SGI274523[] = {{&S17845[89], 4}, {NULL}};
const Segment SGI274524[] = {{&S17845[13], 3}, {&S17845[17], 2}, {&S17845[20], 13}, {&S17845[34], 6}, {&S17845[62], 17}, {NULL}};
const Segment SGI274525[] = {{&S17845[115], 10}, {NULL}};
const Segment SGI274526[] = {{&S17846[89], 4}, {NULL}};
const Segment SGI274527[] = {{&S17846[13], 3}, {&S17846[17], 2}, {&S17846[20], 13}, {&S17846[34], 6}, {&S17846[62], 17}, {NULL}};
const Segment SGI274528[] = {{&S17846[115], 10}, {NULL}};
const Segment SGI274529[] = {{&S17847[89], 4}, {NULL}};
const Segment SGI274530[] = {{&S17847[13], 3}, {&S17847[17], 2}, {&S17847[20], 13}, {&S17847[34], 6}, {&S17847[62], 17}, {NULL}};
const Segment SGI274531[] = {{&S17847[115], 10}, {NULL}};
const Segment SGI274532[] = {{&S17848[89], 4}, {NULL}};
const Segment SGI274533[] = {{&S17848[13], 3}, {&S17848[17], 2}, {&S17848[20], 13}, {&S17848[34], 6}, {&S17848[62], 17}, {NULL}};
const Segment SGI274534[] = {{&S17848[115], 10}, {NULL}};
const Segment SGI274535[] = {{&S17849[89], 4}, {NULL}};
const Segment SGI274536[] = {{&S17849[13], 3}, {&S17849[17], 2}, {&S17849[20], 13}, {&S17849[34], 6}, {&S17849[62], 17}, {NULL}};
const Segment SGI274537[] = {{&S17849[115], 10}, {NULL}};
const Segment SGI274538[] = {{&S17850[89], 4}, {NULL}};
const Segment SGI274539[] = {{&S17850[13], 3}, {&S17850[17], 2}, {&S17850[20], 13}, {&S17850[34], 6}, {&S17850[62], 17}, {NULL}};
const Segment SGI274540[] = {{&S17850[115], 10}, {NULL}};
const Segment SGI274541[] = {{&S17851[89], 4}, {NULL}};
const Segment SGI274542[] = {{&S17851[13], 3}, {&S17851[17], 2}, {&S17851[20], 13}, {&S17851[34], 6}, {&S17851[62], 17}, {NULL}};
const Segment SGI274543[] = {{&S17851[115], 10}, {NULL}};
const Segment SGI274544[] = {{&S17852[89], 4}, {NULL}};
const Segment SGI274545[] = {{&S17852[13], 3}, {&S17852[17], 2}, {&S17852[20], 13}, {&S17852[34], 6}, {&S17852[62], 17}, {NULL}};
const Segment SGI274546[] = {{&S17852[115], 10}, {NULL}};
const Segment SGI274547[] = {{&S17853[89], 4}, {NULL}};
const Segment SGI274548[] = {{&S17853[13], 3}, {&S17853[17], 2}, {&S17853[20], 13}, {&S17853[34], 6}, {&S17853[62], 17}, {NULL}};
const Segment SGI274549[] = {{&S17853[115], 10}, {NULL}};
const Segment SGI274550[] = {{&S17854[89], 4}, {NULL}};
const Segment SGI274551[] = {{&S17854[13], 3}, {&S17854[17], 2}, {&S17854[20], 13}, {&S17854[34], 6}, {&S17854[62], 17}, {NULL}};
const Segment SGI274552[] = {{&S17854[115], 10}, {NULL}};
const Segment SGI274553[] = {{&S17855[89], 4}, {NULL}};
const Segment SGI274554[] = {{&S17855[13], 3}, {&S17855[17], 2}, {&S17855[20], 13}, {&S17855[34], 6}, {&S17855[62], 17}, {NULL}};
const Segment SGI274555[] = {{&S17855[115], 10}, {NULL}};
const Segment SGI274556[] = {{&S17856[89], 4}, {NULL}};
const Segment SGI274557[] = {{&S17856[13], 3}, {&S17856[17], 2}, {&S17856[20], 13}, {&S17856[34], 6}, {&S17856[62], 17}, {NULL}};
const Segment SGI274558[] = {{&S17856[115], 10}, {NULL}};
const Segment SGI274559[] = {{&S17857[89], 4}, {NULL}};
const Segment SGI274560[] = {{&S17857[13], 3}, {&S17857[17], 2}, {&S17857[20], 13}, {&S17857[34], 6}, {&S17857[62], 17}, {NULL}};
const Segment SGI274561[] = {{&S17857[115], 10}, {NULL}};
const Segment SGI274562[] = {{&S17858[89], 4}, {NULL}};
const Segment SGI274563[] = {{&S17858[13], 3}, {&S17858[17], 2}, {&S17858[20], 13}, {&S17858[34], 6}, {&S17858[62], 17}, {NULL}};
const Segment SGI274564[] = {{&S17858[115], 10}, {NULL}};
const Segment SGI274565[] = {{&S17859[89], 4}, {NULL}};
const Segment SGI274566[] = {{&S17859[13], 3}, {&S17859[17], 2}, {&S17859[20], 13}, {&S17859[34], 6}, {&S17859[62], 17}, {NULL}};
const Segment SGI274567[] = {{&S17859[115], 10}, {NULL}};
const Segment SGI274568[] = {{&S17860[89], 4}, {NULL}};
const Segment SGI274569[] = {{&S17860[13], 3}, {&S17860[17], 2}, {&S17860[20], 13}, {&S17860[34], 6}, {&S17860[62], 17}, {NULL}};
const Segment SGI274570[] = {{&S17860[115], 10}, {NULL}};
const Segment SGI274571[] = {{&S17861[89], 4}, {NULL}};
const Segment SGI274572[] = {{&S17861[13], 3}, {&S17861[17], 2}, {&S17861[20], 13}, {&S17861[34], 6}, {&S17861[62], 17}, {NULL}};
const Segment SGI274573[] = {{&S17861[115], 10}, {NULL}};
const Segment SGI274574[] = {{&S17862[89], 4}, {NULL}};
const Segment SGI274575[] = {{&S17862[13], 3}, {&S17862[17], 2}, {&S17862[20], 13}, {&S17862[34], 6}, {&S17862[62], 17}, {NULL}};
const Segment SGI274576[] = {{&S17862[115], 10}, {NULL}};
const Segment SGI274577[] = {{&S17863[89], 4}, {NULL}};
const Segment SGI274578[] = {{&S17863[13], 3}, {&S17863[17], 2}, {&S17863[20], 13}, {&S17863[34], 6}, {&S17863[62], 17}, {NULL}};
const Segment SGI274579[] = {{&S17863[115], 10}, {NULL}};
const Segment SGI274580[] = {{&S17864[89], 4}, {NULL}};
const Segment SGI274581[] = {{&S17864[13], 3}, {&S17864[17], 2}, {&S17864[20], 13}, {&S17864[34], 6}, {&S17864[62], 17}, {NULL}};
const Segment SGI274582[] = {{&S17864[115], 10}, {NULL}};
const Segment SGI274583[] = {{&S17865[89], 4}, {NULL}};
const Segment SGI274584[] = {{&S17865[13], 3}, {&S17865[17], 2}, {&S17865[20], 13}, {&S17865[34], 6}, {&S17865[62], 17}, {NULL}};
const Segment SGI274585[] = {{&S17865[115], 10}, {NULL}};
const Segment SGI274586[] = {{&S17866[89], 4}, {NULL}};
const Segment SGI274587[] = {{&S17866[13], 3}, {&S17866[17], 2}, {&S17866[20], 13}, {&S17866[34], 6}, {&S17866[62], 17}, {NULL}};
const Segment SGI274588[] = {{&S17866[115], 10}, {NULL}};
const Segment SGI274589[] = {{&S17867[89], 4}, {NULL}};
const Segment SGI274590[] = {{&S17867[13], 3}, {&S17867[17], 2}, {&S17867[20], 13}, {&S17867[34], 6}, {&S17867[62], 17}, {NULL}};
const Segment SGI274591[] = {{&S17867[115], 10}, {NULL}};
const Segment SGI274592[] = {{&S17868[89], 4}, {NULL}};
const Segment SGI274593[] = {{&S17868[13], 3}, {&S17868[17], 2}, {&S17868[20], 13}, {&S17868[34], 6}, {&S17868[62], 17}, {NULL}};
const Segment SGI274594[] = {{&S17868[115], 10}, {NULL}};
const Segment SGI274595[] = {{&S17869[89], 4}, {NULL}};
const Segment SGI274596[] = {{&S17869[13], 3}, {&S17869[17], 2}, {&S17869[20], 13}, {&S17869[34], 6}, {&S17869[62], 17}, {NULL}};
const Segment SGI274597[] = {{&S17869[115], 10}, {NULL}};
const Segment SGI274598[] = {{&S17870[89], 4}, {NULL}};
const Segment SGI274599[] = {{&S17870[13], 3}, {&S17870[17], 2}, {&S17870[20], 13}, {&S17870[34], 6}, {&S17870[62], 17}, {NULL}};
const Segment SGI274600[] = {{&S17870[115], 10}, {NULL}};
const Segment SGI274601[] = {{&S17871[89], 4}, {NULL}};
const Segment SGI274602[] = {{&S17871[13], 3}, {&S17871[17], 2}, {&S17871[20], 13}, {&S17871[34], 6}, {&S17871[62], 17}, {NULL}};
const Segment SGI274603[] = {{&S17871[115], 10}, {NULL}};
const Segment SGI274604[] = {{&S17872[89], 4}, {NULL}};
const Segment SGI274605[] = {{&S17872[13], 3}, {&S17872[17], 2}, {&S17872[20], 13}, {&S17872[34], 6}, {&S17872[62], 17}, {NULL}};
const Segment SGI274606[] = {{&S17872[115], 10}, {NULL}};
const Segment SGI274607[] = {{&S17873[89], 4}, {NULL}};
const Segment SGI274608[] = {{&S17873[13], 3}, {&S17873[17], 2}, {&S17873[20], 13}, {&S17873[34], 6}, {&S17873[62], 17}, {NULL}};
const Segment SGI274609[] = {{&S17873[115], 10}, {NULL}};
const Segment SGI274610[] = {{&S17874[89], 4}, {NULL}};
const Segment SGI274611[] = {{&S17874[13], 3}, {&S17874[17], 2}, {&S17874[20], 13}, {&S17874[34], 6}, {&S17874[62], 17}, {NULL}};
const Segment SGI274612[] = {{&S17874[115], 10}, {NULL}};
const Segment SGI274613[] = {{&S17875[89], 4}, {NULL}};
const Segment SGI274614[] = {{&S17875[13], 3}, {&S17875[17], 2}, {&S17875[20], 13}, {&S17875[34], 6}, {&S17875[62], 17}, {NULL}};
const Segment SGI274615[] = {{&S17875[115], 10}, {NULL}};
const Segment SGI274616[] = {{&S17876[89], 4}, {NULL}};
const Segment SGI274617[] = {{&S17876[13], 3}, {&S17876[17], 2}, {&S17876[20], 13}, {&S17876[34], 6}, {&S17876[62], 17}, {NULL}};
const Segment SGI274618[] = {{&S17876[115], 10}, {NULL}};
const Segment SGI274619[] = {{&S17877[89], 4}, {NULL}};
const Segment SGI274620[] = {{&S17877[13], 3}, {&S17877[17], 2}, {&S17877[20], 13}, {&S17877[34], 6}, {&S17877[62], 17}, {NULL}};
const Segment SGI274621[] = {{&S17877[115], 10}, {NULL}};
const Segment SGI274622[] = {{&S17878[89], 4}, {NULL}};
const Segment SGI274623[] = {{&S17878[13], 3}, {&S17878[17], 2}, {&S17878[20], 13}, {&S17878[34], 6}, {&S17878[62], 17}, {NULL}};
const Segment SGI274624[] = {{&S17878[115], 10}, {NULL}};
const Segment SGI274625[] = {{&S17879[89], 4}, {NULL}};
const Segment SGI274626[] = {{&S17879[13], 3}, {&S17879[17], 2}, {&S17879[20], 13}, {&S17879[34], 6}, {&S17879[62], 17}, {NULL}};
const Segment SGI274627[] = {{&S17879[115], 10}, {NULL}};
const Segment SGI274628[] = {{&S17880[89], 4}, {NULL}};
const Segment SGI274629[] = {{&S17880[13], 3}, {&S17880[17], 2}, {&S17880[20], 13}, {&S17880[34], 6}, {&S17880[62], 17}, {NULL}};
const Segment SGI274630[] = {{&S17880[115], 10}, {NULL}};
const Segment SGI274631[] = {{&S17881[89], 4}, {NULL}};
const Segment SGI274632[] = {{&S17881[13], 3}, {&S17881[17], 2}, {&S17881[20], 13}, {&S17881[34], 6}, {&S17881[62], 17}, {NULL}};
const Segment SGI274633[] = {{&S17881[115], 10}, {NULL}};
const Segment SGI274634[] = {{&S17882[89], 4}, {NULL}};
const Segment SGI274635[] = {{&S17882[13], 3}, {&S17882[17], 2}, {&S17882[20], 13}, {&S17882[34], 6}, {&S17882[62], 17}, {NULL}};
const Segment SGI274636[] = {{&S17882[115], 10}, {NULL}};
const Segment SGI274637[] = {{&S17883[89], 4}, {NULL}};
const Segment SGI274638[] = {{&S17883[13], 3}, {&S17883[17], 2}, {&S17883[20], 13}, {&S17883[34], 6}, {&S17883[62], 17}, {NULL}};
const Segment SGI274639[] = {{&S17883[115], 10}, {NULL}};
const Segment SGI274640[] = {{&S17884[89], 4}, {NULL}};
const Segment SGI274641[] = {{&S17884[13], 3}, {&S17884[17], 2}, {&S17884[20], 13}, {&S17884[34], 6}, {&S17884[62], 17}, {NULL}};
const Segment SGI274642[] = {{&S17884[115], 10}, {NULL}};
const Segment SGI274643[] = {{&S17885[89], 4}, {NULL}};
const Segment SGI274644[] = {{&S17885[13], 3}, {&S17885[17], 2}, {&S17885[20], 13}, {&S17885[34], 6}, {&S17885[62], 17}, {NULL}};
const Segment SGI274645[] = {{&S17885[115], 10}, {NULL}};
const Segment SGI274646[] = {{&S17886[89], 4}, {NULL}};
const Segment SGI274647[] = {{&S17886[13], 3}, {&S17886[17], 2}, {&S17886[20], 13}, {&S17886[34], 6}, {&S17886[62], 17}, {NULL}};
const Segment SGI274648[] = {{&S17886[115], 10}, {NULL}};
const Segment SGI274649[] = {{&S17887[86], 4}, {NULL}};
const Segment SGI274650[] = {{&S17887[13], 3}, {&S17887[17], 2}, {&S17887[20], 13}, {&S17887[34], 3}, {&S17887[59], 17}, {NULL}};
const Segment SGI274651[] = {{&S17887[112], 10}, {NULL}};
const Segment SGI274652[] = {{&S17888[89], 4}, {NULL}};
const Segment SGI274653[] = {{&S17888[13], 3}, {&S17888[17], 2}, {&S17888[20], 13}, {&S17888[34], 6}, {&S17888[62], 17}, {NULL}};
const Segment SGI274654[] = {{&S17888[115], 10}, {NULL}};
const Segment SGI274655[] = {{&S17889[89], 4}, {NULL}};
const Segment SGI274656[] = {{&S17889[13], 3}, {&S17889[17], 2}, {&S17889[20], 13}, {&S17889[34], 6}, {&S17889[62], 17}, {NULL}};
const Segment SGI274657[] = {{&S17889[115], 10}, {NULL}};
const Segment SGI274658[] = {{&S17890[89], 4}, {NULL}};
const Segment SGI274659[] = {{&S17890[13], 3}, {&S17890[17], 2}, {&S17890[20], 13}, {&S17890[34], 6}, {&S17890[62], 17}, {NULL}};
const Segment SGI274660[] = {{&S17890[115], 10}, {NULL}};
const Segment SGI274661[] = {{&S17891[89], 4}, {NULL}};
const Segment SGI274662[] = {{&S17891[13], 3}, {&S17891[17], 2}, {&S17891[20], 13}, {&S17891[34], 6}, {&S17891[62], 17}, {NULL}};
const Segment SGI274663[] = {{&S17891[115], 10}, {NULL}};
const Segment SGI274664[] = {{&S17892[89], 4}, {NULL}};
const Segment SGI274665[] = {{&S17892[13], 3}, {&S17892[17], 2}, {&S17892[20], 13}, {&S17892[34], 6}, {&S17892[62], 17}, {NULL}};
const Segment SGI274666[] = {{&S17892[115], 10}, {NULL}};
const Segment SGI274667[] = {{&S17893[89], 4}, {NULL}};
const Segment SGI274668[] = {{&S17893[13], 3}, {&S17893[17], 2}, {&S17893[20], 13}, {&S17893[34], 6}, {&S17893[62], 17}, {NULL}};
const Segment SGI274669[] = {{&S17893[115], 10}, {NULL}};
const Segment SGI274670[] = {{&S17894[89], 4}, {NULL}};
const Segment SGI274671[] = {{&S17894[13], 3}, {&S17894[17], 2}, {&S17894[20], 13}, {&S17894[34], 6}, {&S17894[62], 17}, {NULL}};
const Segment SGI274672[] = {{&S17894[115], 10}, {NULL}};
const Segment SGI274673[] = {{&S17895[89], 4}, {NULL}};
const Segment SGI274674[] = {{&S17895[13], 3}, {&S17895[17], 2}, {&S17895[20], 13}, {&S17895[34], 6}, {&S17895[62], 17}, {NULL}};
const Segment SGI274675[] = {{&S17895[115], 10}, {NULL}};
const Segment SGI274676[] = {{&S17896[89], 4}, {NULL}};
const Segment SGI274677[] = {{&S17896[13], 3}, {&S17896[17], 2}, {&S17896[20], 13}, {&S17896[34], 6}, {&S17896[62], 17}, {NULL}};
const Segment SGI274678[] = {{&S17896[115], 10}, {NULL}};
const Segment SGI274679[] = {{&S17897[89], 4}, {NULL}};
const Segment SGI274680[] = {{&S17897[13], 3}, {&S17897[17], 2}, {&S17897[20], 13}, {&S17897[34], 6}, {&S17897[62], 17}, {NULL}};
const Segment SGI274681[] = {{&S17897[115], 10}, {NULL}};
const Segment SGI274682[] = {{&S17898[89], 4}, {NULL}};
const Segment SGI274683[] = {{&S17898[13], 3}, {&S17898[17], 2}, {&S17898[20], 13}, {&S17898[34], 6}, {&S17898[62], 17}, {NULL}};
const Segment SGI274684[] = {{&S17898[115], 10}, {NULL}};
const Segment SGI274685[] = {{&S17899[89], 4}, {NULL}};
const Segment SGI274686[] = {{&S17899[13], 3}, {&S17899[17], 2}, {&S17899[20], 13}, {&S17899[34], 6}, {&S17899[62], 17}, {NULL}};
const Segment SGI274687[] = {{&S17899[115], 10}, {NULL}};
const Segment SGI274688[] = {{&S17900[89], 4}, {NULL}};
const Segment SGI274689[] = {{&S17900[13], 3}, {&S17900[17], 2}, {&S17900[20], 13}, {&S17900[34], 6}, {&S17900[62], 17}, {NULL}};
const Segment SGI274690[] = {{&S17900[115], 10}, {NULL}};
const Segment SGI274691[] = {{&S17901[89], 4}, {NULL}};
const Segment SGI274692[] = {{&S17901[13], 3}, {&S17901[17], 2}, {&S17901[20], 13}, {&S17901[34], 6}, {&S17901[62], 17}, {NULL}};
const Segment SGI274693[] = {{&S17901[115], 10}, {NULL}};
const Segment SGI274694[] = {{&S17902[89], 4}, {NULL}};
const Segment SGI274695[] = {{&S17902[13], 3}, {&S17902[17], 2}, {&S17902[20], 13}, {&S17902[34], 6}, {&S17902[62], 17}, {NULL}};
const Segment SGI274696[] = {{&S17902[115], 10}, {NULL}};
const Segment SGI274697[] = {{&S17903[89], 4}, {NULL}};
const Segment SGI274698[] = {{&S17903[13], 3}, {&S17903[17], 2}, {&S17903[20], 13}, {&S17903[34], 6}, {&S17903[62], 17}, {NULL}};
const Segment SGI274699[] = {{&S17903[115], 10}, {NULL}};
const Segment SGI274700[] = {{&S17904[89], 4}, {NULL}};
const Segment SGI274701[] = {{&S17904[13], 3}, {&S17904[17], 2}, {&S17904[20], 13}, {&S17904[34], 6}, {&S17904[62], 17}, {NULL}};
const Segment SGI274702[] = {{&S17904[115], 10}, {NULL}};
const Segment SGI274703[] = {{&S17905[89], 4}, {NULL}};
const Segment SGI274704[] = {{&S17905[13], 3}, {&S17905[17], 2}, {&S17905[20], 13}, {&S17905[34], 6}, {&S17905[62], 17}, {NULL}};
const Segment SGI274705[] = {{&S17905[115], 10}, {NULL}};
const Segment SGI274706[] = {{&S17906[89], 4}, {NULL}};
const Segment SGI274707[] = {{&S17906[13], 3}, {&S17906[17], 2}, {&S17906[20], 13}, {&S17906[34], 6}, {&S17906[62], 17}, {NULL}};
const Segment SGI274708[] = {{&S17906[115], 10}, {NULL}};
const Segment SGI274709[] = {{&S17907[89], 4}, {NULL}};
const Segment SGI274710[] = {{&S17907[13], 3}, {&S17907[17], 2}, {&S17907[20], 13}, {&S17907[34], 6}, {&S17907[62], 17}, {NULL}};
const Segment SGI274711[] = {{&S17907[115], 10}, {NULL}};
const Segment SGI274712[] = {{&S17908[89], 4}, {NULL}};
const Segment SGI274713[] = {{&S17908[13], 3}, {&S17908[17], 2}, {&S17908[20], 13}, {&S17908[34], 6}, {&S17908[62], 17}, {NULL}};
const Segment SGI274714[] = {{&S17908[115], 10}, {NULL}};
const Segment SGI274715[] = {{&S17909[89], 4}, {NULL}};
const Segment SGI274716[] = {{&S17909[13], 3}, {&S17909[17], 2}, {&S17909[20], 13}, {&S17909[34], 6}, {&S17909[62], 17}, {NULL}};
const Segment SGI274717[] = {{&S17909[115], 10}, {NULL}};
const Segment SGI274718[] = {{&S17910[89], 4}, {NULL}};
const Segment SGI274719[] = {{&S17910[13], 3}, {&S17910[17], 2}, {&S17910[20], 13}, {&S17910[34], 6}, {&S17910[62], 17}, {NULL}};
const Segment SGI274720[] = {{&S17910[115], 10}, {NULL}};
const Segment SGI274721[] = {{&S17911[89], 4}, {NULL}};
const Segment SGI274722[] = {{&S17911[13], 3}, {&S17911[17], 2}, {&S17911[20], 13}, {&S17911[34], 6}, {&S17911[62], 17}, {NULL}};
const Segment SGI274723[] = {{&S17911[115], 10}, {NULL}};
const Segment SGI274724[] = {{&S17912[89], 4}, {NULL}};
const Segment SGI274725[] = {{&S17912[13], 3}, {&S17912[17], 2}, {&S17912[20], 13}, {&S17912[34], 6}, {&S17912[62], 17}, {NULL}};
const Segment SGI274726[] = {{&S17912[115], 10}, {NULL}};
const Segment SGI274727[] = {{&S17913[89], 4}, {NULL}};
const Segment SGI274728[] = {{&S17913[13], 3}, {&S17913[17], 2}, {&S17913[20], 13}, {&S17913[34], 6}, {&S17913[62], 17}, {NULL}};
const Segment SGI274729[] = {{&S17913[115], 10}, {NULL}};
const Segment SGI274730[] = {{&S17914[89], 4}, {NULL}};
const Segment SGI274731[] = {{&S17914[13], 3}, {&S17914[17], 2}, {&S17914[20], 13}, {&S17914[34], 6}, {&S17914[62], 17}, {NULL}};
const Segment SGI274732[] = {{&S17914[115], 10}, {NULL}};
const Segment SGI274733[] = {{&S17915[89], 4}, {NULL}};
const Segment SGI274734[] = {{&S17915[13], 3}, {&S17915[17], 2}, {&S17915[20], 13}, {&S17915[34], 6}, {&S17915[62], 17}, {NULL}};
const Segment SGI274735[] = {{&S17915[115], 10}, {NULL}};
const Segment SGI274736[] = {{&S17916[89], 4}, {NULL}};
const Segment SGI274737[] = {{&S17916[13], 3}, {&S17916[17], 2}, {&S17916[20], 13}, {&S17916[34], 6}, {&S17916[62], 17}, {NULL}};
const Segment SGI274738[] = {{&S17916[115], 10}, {NULL}};
const Segment SGI274739[] = {{&S17917[89], 4}, {NULL}};
const Segment SGI274740[] = {{&S17917[13], 3}, {&S17917[17], 2}, {&S17917[20], 13}, {&S17917[34], 6}, {&S17917[62], 17}, {NULL}};
const Segment SGI274741[] = {{&S17917[115], 10}, {NULL}};
const Segment SGI274742[] = {{&S17918[89], 4}, {NULL}};
const Segment SGI274743[] = {{&S17918[13], 3}, {&S17918[17], 2}, {&S17918[20], 13}, {&S17918[34], 6}, {&S17918[62], 17}, {NULL}};
const Segment SGI274744[] = {{&S17918[115], 10}, {NULL}};
const Segment SGI274745[] = {{&S17919[89], 4}, {NULL}};
const Segment SGI274746[] = {{&S17919[13], 3}, {&S17919[17], 2}, {&S17919[20], 13}, {&S17919[34], 6}, {&S17919[62], 17}, {NULL}};
const Segment SGI274747[] = {{&S17919[115], 10}, {NULL}};
const Segment SGI274748[] = {{&S17920[87], 4}, {NULL}};
const Segment SGI274749[] = {{&S17920[13], 3}, {&S17920[17], 2}, {&S17920[20], 13}, {&S17920[34], 4}, {&S17920[60], 17}, {NULL}};
const Segment SGI274750[] = {{&S17920[113], 10}, {NULL}};
const Segment SGI274751[] = {{&S17921[89], 4}, {NULL}};
const Segment SGI274752[] = {{&S17921[13], 3}, {&S17921[17], 2}, {&S17921[20], 13}, {&S17921[34], 6}, {&S17921[62], 17}, {NULL}};
const Segment SGI274753[] = {{&S17921[115], 10}, {NULL}};
const Segment SGI274754[] = {{&S17922[89], 4}, {NULL}};
const Segment SGI274755[] = {{&S17922[13], 3}, {&S17922[17], 2}, {&S17922[20], 13}, {&S17922[34], 6}, {&S17922[62], 17}, {NULL}};
const Segment SGI274756[] = {{&S17922[115], 10}, {NULL}};
const Segment SGI274757[] = {{&S17923[89], 4}, {NULL}};
const Segment SGI274758[] = {{&S17923[13], 3}, {&S17923[17], 2}, {&S17923[20], 13}, {&S17923[34], 6}, {&S17923[62], 17}, {NULL}};
const Segment SGI274759[] = {{&S17923[115], 10}, {NULL}};
const Segment SGI274760[] = {{&S17924[89], 4}, {NULL}};
const Segment SGI274761[] = {{&S17924[13], 3}, {&S17924[17], 2}, {&S17924[20], 13}, {&S17924[34], 6}, {&S17924[62], 17}, {NULL}};
const Segment SGI274762[] = {{&S17924[115], 10}, {NULL}};
const Segment SGI274763[] = {{&S17925[89], 4}, {NULL}};
const Segment SGI274764[] = {{&S17925[13], 3}, {&S17925[17], 2}, {&S17925[20], 13}, {&S17925[34], 6}, {&S17925[62], 17}, {NULL}};
const Segment SGI274765[] = {{&S17925[115], 10}, {NULL}};
const Segment SGI274766[] = {{&S17926[89], 4}, {NULL}};
const Segment SGI274767[] = {{&S17926[13], 3}, {&S17926[17], 2}, {&S17926[20], 13}, {&S17926[34], 6}, {&S17926[62], 17}, {NULL}};
const Segment SGI274768[] = {{&S17926[115], 10}, {NULL}};
const Segment SGI274769[] = {{&S17927[89], 4}, {NULL}};
const Segment SGI274770[] = {{&S17927[13], 3}, {&S17927[17], 2}, {&S17927[20], 13}, {&S17927[34], 6}, {&S17927[62], 17}, {NULL}};
const Segment SGI274771[] = {{&S17927[115], 10}, {NULL}};
const Segment SGI274772[] = {{&S17928[89], 4}, {NULL}};
const Segment SGI274773[] = {{&S17928[13], 3}, {&S17928[17], 2}, {&S17928[20], 13}, {&S17928[34], 6}, {&S17928[62], 17}, {NULL}};
const Segment SGI274774[] = {{&S17928[115], 10}, {NULL}};
const Segment SGI274775[] = {{&S17929[89], 4}, {NULL}};
const Segment SGI274776[] = {{&S17929[13], 3}, {&S17929[17], 2}, {&S17929[20], 13}, {&S17929[34], 6}, {&S17929[62], 17}, {NULL}};
const Segment SGI274777[] = {{&S17929[115], 10}, {NULL}};
const Segment SGI274778[] = {{&S17930[89], 4}, {NULL}};
const Segment SGI274779[] = {{&S17930[13], 3}, {&S17930[17], 2}, {&S17930[20], 13}, {&S17930[34], 6}, {&S17930[62], 17}, {NULL}};
const Segment SGI274780[] = {{&S17930[115], 10}, {NULL}};
const Segment SGI274781[] = {{&S17931[89], 4}, {NULL}};
const Segment SGI274782[] = {{&S17931[13], 3}, {&S17931[17], 2}, {&S17931[20], 13}, {&S17931[34], 6}, {&S17931[62], 17}, {NULL}};
const Segment SGI274783[] = {{&S17931[115], 10}, {NULL}};
const Segment SGI274784[] = {{&S17932[89], 4}, {NULL}};
const Segment SGI274785[] = {{&S17932[13], 3}, {&S17932[17], 2}, {&S17932[20], 13}, {&S17932[34], 6}, {&S17932[62], 17}, {NULL}};
const Segment SGI274786[] = {{&S17932[115], 10}, {NULL}};
const Segment SGI274787[] = {{&S17933[89], 4}, {NULL}};
const Segment SGI274788[] = {{&S17933[13], 3}, {&S17933[17], 2}, {&S17933[20], 13}, {&S17933[34], 6}, {&S17933[62], 17}, {NULL}};
const Segment SGI274789[] = {{&S17933[115], 10}, {NULL}};
const Segment SGI274790[] = {{&S17934[89], 4}, {NULL}};
const Segment SGI274791[] = {{&S17934[13], 3}, {&S17934[17], 2}, {&S17934[20], 13}, {&S17934[34], 6}, {&S17934[62], 17}, {NULL}};
const Segment SGI274792[] = {{&S17934[115], 10}, {NULL}};
const Segment SGI274793[] = {{&S17935[89], 4}, {NULL}};
const Segment SGI274794[] = {{&S17935[13], 3}, {&S17935[17], 2}, {&S17935[20], 13}, {&S17935[34], 6}, {&S17935[62], 17}, {NULL}};
const Segment SGI274795[] = {{&S17935[115], 10}, {NULL}};
const Segment SGI274796[] = {{&S17936[89], 4}, {NULL}};
const Segment SGI274797[] = {{&S17936[13], 3}, {&S17936[17], 2}, {&S17936[20], 13}, {&S17936[34], 6}, {&S17936[62], 17}, {NULL}};
const Segment SGI274798[] = {{&S17936[115], 10}, {NULL}};
const Segment SGI274799[] = {{&S17937[89], 4}, {NULL}};
const Segment SGI274800[] = {{&S17937[13], 3}, {&S17937[17], 2}, {&S17937[20], 13}, {&S17937[34], 6}, {&S17937[62], 17}, {NULL}};
const Segment SGI274801[] = {{&S17937[115], 10}, {NULL}};
const Segment SGI274802[] = {{&S17938[89], 4}, {NULL}};
const Segment SGI274803[] = {{&S17938[13], 3}, {&S17938[17], 2}, {&S17938[20], 13}, {&S17938[34], 6}, {&S17938[62], 17}, {NULL}};
const Segment SGI274804[] = {{&S17938[115], 10}, {NULL}};
const Segment SGI274805[] = {{&S17939[89], 4}, {NULL}};
const Segment SGI274806[] = {{&S17939[13], 3}, {&S17939[17], 2}, {&S17939[20], 13}, {&S17939[34], 6}, {&S17939[62], 17}, {NULL}};
const Segment SGI274807[] = {{&S17939[115], 10}, {NULL}};
const Segment SGI274808[] = {{&S17940[95], 4}, {NULL}};
const Segment SGI274809[] = {{&S17940[13], 3}, {&S17940[17], 2}, {&S17940[20], 13}, {&S17940[34], 12}, {&S17940[68], 17}, {NULL}};
const Segment SGI274810[] = {{&S17940[121], 10}, {NULL}};
const Segment SGI274811[] = {{&S17941[107], 4}, {NULL}};
const Segment SGI274812[] = {{&S17941[13], 3}, {&S17941[17], 2}, {&S17941[20], 13}, {&S17941[34], 24}, {&S17941[80], 17}, {NULL}};
const Segment SGI274813[] = {{&S17941[133], 10}, {NULL}};
const Segment SGI274814[] = {{&S17942[107], 4}, {NULL}};
const Segment SGI274815[] = {{&S17942[13], 3}, {&S17942[17], 2}, {&S17942[20], 13}, {&S17942[34], 24}, {&S17942[80], 17}, {NULL}};
const Segment SGI274816[] = {{&S17942[133], 10}, {NULL}};
const Segment SGI274817[] = {{&S17943[89], 4}, {NULL}};
const Segment SGI274818[] = {{&S17943[13], 3}, {&S17943[17], 2}, {&S17943[20], 13}, {&S17943[34], 6}, {&S17943[62], 17}, {NULL}};
const Segment SGI274819[] = {{&S17943[115], 10}, {NULL}};
const Segment SGI274820[] = {{&S17944[89], 4}, {NULL}};
const Segment SGI274821[] = {{&S17944[13], 3}, {&S17944[17], 2}, {&S17944[20], 13}, {&S17944[34], 6}, {&S17944[62], 17}, {NULL}};
const Segment SGI274822[] = {{&S17944[115], 10}, {NULL}};
const Segment SGI274823[] = {{&S17945[89], 4}, {NULL}};
const Segment SGI274824[] = {{&S17945[13], 3}, {&S17945[17], 2}, {&S17945[20], 13}, {&S17945[34], 6}, {&S17945[62], 17}, {NULL}};
const Segment SGI274825[] = {{&S17945[115], 10}, {NULL}};
const Segment SGI274826[] = {{&S17946[89], 4}, {NULL}};
const Segment SGI274827[] = {{&S17946[13], 3}, {&S17946[17], 2}, {&S17946[20], 13}, {&S17946[34], 6}, {&S17946[62], 17}, {NULL}};
const Segment SGI274828[] = {{&S17946[115], 10}, {NULL}};
const Segment SGI274829[] = {{&S17947[89], 4}, {NULL}};
const Segment SGI274830[] = {{&S17947[13], 3}, {&S17947[17], 2}, {&S17947[20], 13}, {&S17947[34], 6}, {&S17947[62], 17}, {NULL}};
const Segment SGI274831[] = {{&S17947[115], 10}, {NULL}};
const Segment SGI274832[] = {{&S17948[89], 4}, {NULL}};
const Segment SGI274833[] = {{&S17948[13], 3}, {&S17948[17], 2}, {&S17948[20], 13}, {&S17948[34], 6}, {&S17948[62], 17}, {NULL}};
const Segment SGI274834[] = {{&S17948[115], 10}, {NULL}};
const Segment SGI274835[] = {{&S17949[89], 4}, {NULL}};
const Segment SGI274836[] = {{&S17949[13], 3}, {&S17949[17], 2}, {&S17949[20], 13}, {&S17949[34], 6}, {&S17949[62], 17}, {NULL}};
const Segment SGI274837[] = {{&S17949[115], 10}, {NULL}};
const Segment SGI274838[] = {{&S17950[106], 4}, {NULL}};
const Segment SGI274839[] = {{&S17950[13], 3}, {&S17950[17], 2}, {&S17950[20], 13}, {&S17950[34], 23}, {&S17950[79], 17}, {NULL}};
const Segment SGI274840[] = {{&S17950[132], 10}, {NULL}};
const Segment SGI274841[] = {{&S17951[89], 4}, {NULL}};
const Segment SGI274842[] = {{&S17951[13], 3}, {&S17951[17], 2}, {&S17951[20], 13}, {&S17951[34], 6}, {&S17951[62], 17}, {NULL}};
const Segment SGI274843[] = {{&S17951[115], 10}, {NULL}};
const Segment SGI274844[] = {{&S17952[89], 4}, {NULL}};
const Segment SGI274845[] = {{&S17952[13], 3}, {&S17952[17], 2}, {&S17952[20], 13}, {&S17952[34], 6}, {&S17952[62], 17}, {NULL}};
const Segment SGI274846[] = {{&S17952[115], 10}, {NULL}};
const Segment SGI274847[] = {{&S17953[89], 4}, {NULL}};
const Segment SGI274848[] = {{&S17953[13], 3}, {&S17953[17], 2}, {&S17953[20], 13}, {&S17953[34], 6}, {&S17953[62], 17}, {NULL}};
const Segment SGI274849[] = {{&S17953[115], 10}, {NULL}};
const Segment SGI274850[] = {{&S17954[89], 4}, {NULL}};
const Segment SGI274851[] = {{&S17954[13], 3}, {&S17954[17], 2}, {&S17954[20], 13}, {&S17954[34], 6}, {&S17954[62], 17}, {NULL}};
const Segment SGI274852[] = {{&S17954[115], 10}, {NULL}};
const Segment SGI274853[] = {{&S17955[89], 4}, {NULL}};
const Segment SGI274854[] = {{&S17955[13], 3}, {&S17955[17], 2}, {&S17955[20], 13}, {&S17955[34], 6}, {&S17955[62], 17}, {NULL}};
const Segment SGI274855[] = {{&S17955[115], 10}, {NULL}};
const Segment SGI274856[] = {{&S17956[89], 4}, {NULL}};
const Segment SGI274857[] = {{&S17956[13], 3}, {&S17956[17], 2}, {&S17956[20], 13}, {&S17956[34], 6}, {&S17956[62], 17}, {NULL}};
const Segment SGI274858[] = {{&S17956[115], 10}, {NULL}};
const Segment SGI274859[] = {{&S17957[89], 4}, {NULL}};
const Segment SGI274860[] = {{&S17957[13], 3}, {&S17957[17], 2}, {&S17957[20], 13}, {&S17957[34], 6}, {&S17957[62], 17}, {NULL}};
const Segment SGI274861[] = {{&S17957[115], 10}, {NULL}};
const Segment SGI274862[] = {{&S17958[89], 4}, {NULL}};
const Segment SGI274863[] = {{&S17958[13], 3}, {&S17958[17], 2}, {&S17958[20], 13}, {&S17958[34], 6}, {&S17958[62], 17}, {NULL}};
const Segment SGI274864[] = {{&S17958[115], 10}, {NULL}};
const Segment SGI274865[] = {{&S17959[89], 4}, {NULL}};
const Segment SGI274866[] = {{&S17959[13], 3}, {&S17959[17], 2}, {&S17959[20], 13}, {&S17959[34], 6}, {&S17959[62], 17}, {NULL}};
const Segment SGI274867[] = {{&S17959[115], 10}, {NULL}};
const Segment SGI274868[] = {{&S17960[87], 4}, {NULL}};
const Segment SGI274869[] = {{&S17960[13], 3}, {&S17960[17], 2}, {&S17960[20], 13}, {&S17960[34], 4}, {&S17960[60], 17}, {NULL}};
const Segment SGI274870[] = {{&S17960[113], 10}, {NULL}};
const Segment SGI274871[] = {{&S17961[89], 4}, {NULL}};
const Segment SGI274872[] = {{&S17961[13], 3}, {&S17961[17], 2}, {&S17961[20], 13}, {&S17961[34], 6}, {&S17961[62], 17}, {&S17961[131], 8}, {NULL}};
const Segment SGI274873[] = {{&S17961[115], 10}, {NULL}};
const Segment SGI274874[] = {{&S17962[89], 4}, {NULL}};
const Segment SGI274875[] = {{&S17962[13], 3}, {&S17962[17], 2}, {&S17962[20], 13}, {&S17962[34], 6}, {&S17962[62], 17}, {NULL}};
const Segment SGI274876[] = {{&S17962[115], 10}, {NULL}};
const Segment SGI274877[] = {{&S17963[89], 4}, {NULL}};
const Segment SGI274878[] = {{&S17963[13], 3}, {&S17963[17], 2}, {&S17963[20], 13}, {&S17963[34], 6}, {&S17963[62], 17}, {NULL}};
const Segment SGI274879[] = {{&S17963[115], 10}, {NULL}};
const Segment SGI274880[] = {{&S17964[89], 4}, {NULL}};
const Segment SGI274881[] = {{&S17964[13], 3}, {&S17964[17], 2}, {&S17964[20], 13}, {&S17964[34], 6}, {&S17964[62], 17}, {NULL}};
const Segment SGI274882[] = {{&S17964[115], 10}, {NULL}};
const Segment SGI274883[] = {{&S18094[74], 4}, {NULL}};
const Segment SGI274884[] = {{&S18094[13], 14}, {&S18094[48], 17}, {NULL}};
const Segment SGI274885[] = {{&S18095[81], 4}, {NULL}};
const Segment SGI274886[] = {{&S18095[13], 14}, {&S18095[28], 6}, {&S18095[55], 17}, {NULL}};
const Segment SGI274887[] = {{&S18096[74], 4}, {NULL}};
const Segment SGI274888[] = {{&S18096[13], 14}, {&S18096[48], 17}, {&S18096[100], 8}, {&S18096[111], 26}, {NULL}};
const Segment SGI274889[] = {{&S18097[74], 4}, {NULL}};
const Segment SGI274890[] = {{&S18097[13], 14}, {&S18097[48], 17}, {NULL}};
const Segment SGI274891[] = {{&S18098[74], 4}, {NULL}};
const Segment SGI274892[] = {{&S18098[13], 14}, {&S18098[48], 17}, {&S18098[100], 8}, {&S18098[111], 24}, {NULL}};
const Segment SGI274893[] = {{&S18099[81], 4}, {NULL}};
const Segment SGI274894[] = {{&S18099[13], 14}, {&S18099[28], 6}, {&S18099[55], 17}, {NULL}};
const Segment SGI274895[] = {{&S18100[74], 4}, {NULL}};
const Segment SGI274896[] = {{&S18100[13], 14}, {&S18100[48], 17}, {&S18100[100], 8}, {NULL}};
const Segment SGI274897[] = {{&S18101[98], 4}, {NULL}};
const Segment SGI274898[] = {{&S18101[13], 14}, {&S18101[48], 17}, {NULL}};
const Segment SGI274899[] = {{&S18102[105], 4}, {NULL}};
const Segment SGI274900[] = {{&S18102[13], 14}, {&S18102[28], 6}, {&S18102[55], 17}, {NULL}};
const Segment SGI274901[] = {{&S18103[94], 4}, {NULL}};
const Segment SGI274902[] = {{&S18103[13], 10}, {&S18103[24], 15}, {&S18103[40], 6}, {&S18103[68], 17}, {NULL}};
const Segment SGI274903[] = {{&S18104[98], 4}, {NULL}};
const Segment SGI274904[] = {{&S18104[13], 14}, {&S18104[48], 17}, {NULL}};
const Segment SGI274905[] = {{&S18105[98], 4}, {NULL}};
const Segment SGI274906[] = {{&S18105[13], 14}, {&S18105[48], 17}, {&S18105[126], 8}, {NULL}};
const Segment SGI274907[] = {{&S18231[75], 4}, {NULL}};
const Segment SGI274908[] = {{&S18231[13], 14}, {&S18231[49], 17}, {NULL}};
const Segment SGI274909[] = {{&S18232[75], 4}, {NULL}};
const Segment SGI274910[] = {{&S18232[13], 14}, {&S18232[49], 17}, {&S18232[102], 8}, {&S18232[113], 24}, {NULL}};
const Segment SGI274911[] = {{&S18233[75], 4}, {NULL}};
const Segment SGI274912[] = {{&S18233[13], 14}, {&S18233[49], 17}, {NULL}};
const Segment SGI274913[] = {{&S18234[82], 4}, {NULL}};
const Segment SGI274914[] = {{&S18234[13], 14}, {&S18234[28], 6}, {&S18234[56], 17}, {NULL}};
const Segment SGI274915[] = {{&S18235[75], 4}, {NULL}};
const Segment SGI274916[] = {{&S18235[13], 14}, {&S18235[49], 17}, {&S18235[102], 8}, {&S18235[113], 26}, {NULL}};
const Segment SGI274917[] = {{&S31999[38], 5}, {NULL}};
const Segment SGI274918[] = {{&S31999[97], 5}, {NULL}};
const Segment SGI274919[] = {{&S18236[94], 4}, {NULL}};
const Segment SGI274920[] = {{&S18236[13], 7}, {&S18236[21], 2}, {&S18236[24], 15}, {&S18236[40], 6}, {&S18236[68], 17}, {NULL}};
const Segment SGI274921[] = {{&S18237[75], 4}, {NULL}};
const Segment SGI274922[] = {{&S18237[13], 14}, {&S18237[49], 17}, {&S18237[87], 21}, {NULL}};
const Segment SGI274923[] = {{&S18238[93], 4}, {NULL}};
const Segment SGI274924[] = {{&S18238[13], 7}, {&S18238[21], 2}, {&S18238[24], 15}, {&S18238[40], 6}, {&S18238[67], 17}, {NULL}};
const Segment SGI274925[] = {{&S18239[75], 4}, {NULL}};
const Segment SGI274926[] = {{&S18239[13], 14}, {&S18239[49], 17}, {&S18239[102], 12}, {NULL}};
const Segment SGI274927[] = {{&S18240[82], 4}, {NULL}};
const Segment SGI274928[] = {{&S18240[13], 14}, {&S18240[28], 6}, {&S18240[56], 17}, {NULL}};
const Segment SGI274929[] = {{&S18241[75], 4}, {NULL}};
const Segment SGI274930[] = {{&S18241[13], 14}, {&S18241[49], 17}, {&S18241[102], 8}, {NULL}};
const Segment SGI274931[] = {{&S18242[91], 4}, {NULL}};
const Segment SGI274932[] = {{&S18242[13], 7}, {&S18242[21], 2}, {&S18242[24], 15}, {&S18242[40], 3}, {&S18242[65], 17}, {NULL}};
const Segment SGI274933[] = {{&S18243[74], 4}, {NULL}};
const Segment SGI274934[] = {{&S18243[13], 14}, {&S18243[49], 16}, {NULL}};
const Segment SGI274935[] = {{&S18244[94], 4}, {NULL}};
const Segment SGI274936[] = {{&S18244[13], 7}, {&S18244[21], 2}, {&S18244[24], 15}, {&S18244[40], 6}, {&S18244[68], 17}, {NULL}};
const Segment SGI274937[] = {{&S18245[94], 4}, {NULL}};
const Segment SGI274938[] = {{&S18245[13], 7}, {&S18245[21], 2}, {&S18245[24], 15}, {&S18245[40], 6}, {&S18245[68], 17}, {NULL}};
const Segment SGI274939[] = {{&S18246[94], 4}, {NULL}};
const Segment SGI274940[] = {{&S18246[13], 7}, {&S18246[21], 2}, {&S18246[24], 15}, {&S18246[40], 6}, {&S18246[68], 17}, {NULL}};
const Segment SGI274941[] = {{&S18247[95], 4}, {NULL}};
const Segment SGI274942[] = {{&S18247[13], 7}, {&S18247[21], 2}, {&S18247[24], 15}, {&S18247[40], 7}, {&S18247[69], 17}, {NULL}};
const Segment SGI274943[] = {{&S18248[94], 4}, {NULL}};
const Segment SGI274944[] = {{&S18248[13], 7}, {&S18248[21], 2}, {&S18248[24], 15}, {&S18248[40], 6}, {&S18248[68], 17}, {NULL}};
const Segment SGI274945[] = {{&S18249[94], 4}, {NULL}};
const Segment SGI274946[] = {{&S18249[13], 7}, {&S18249[21], 2}, {&S18249[24], 15}, {&S18249[40], 6}, {&S18249[68], 17}, {NULL}};
const Segment SGI274947[] = {{&S18250[91], 4}, {NULL}};
const Segment SGI274948[] = {{&S18250[13], 14}, {&S18250[28], 15}, {&S18250[65], 17}, {NULL}};
const Segment SGI274949[] = {{&S18251[75], 4}, {NULL}};
const Segment SGI274950[] = {{&S18251[13], 14}, {&S18251[49], 17}, {&S18251[87], 21}, {NULL}};
const Segment SGI274951[] = {{&S18252[93], 4}, {NULL}};
const Segment SGI274952[] = {{&S18252[13], 7}, {&S18252[21], 2}, {&S18252[24], 15}, {&S18252[40], 6}, {&S18252[68], 16}, {NULL}};
const Segment SGI274953[] = {{&S21790[92], 3}, {NULL}};
const Segment SGI274954[] = {{&S21790[13], 7}, {&S21790[21], 2}, {&S21790[24], 15}, {&S21790[40], 6}, {&S21790[67], 16}, {NULL}};
const Segment SGI274955[] = {{&S21791[93], 3}, {NULL}};
const Segment SGI274956[] = {{&S21791[13], 7}, {&S21791[21], 2}, {&S21791[24], 15}, {&S21791[40], 6}, {&S21791[67], 17}, {NULL}};
const Segment SGI274957[] = {{&S21792[93], 3}, {NULL}};
const Segment SGI274958[] = {{&S21792[13], 7}, {&S21792[21], 2}, {&S21792[24], 15}, {&S21792[40], 6}, {&S21792[67], 17}, {NULL}};
const Segment SGI274959[] = {{&S21793[90], 3}, {NULL}};
const Segment SGI274960[] = {{&S21793[13], 7}, {&S21793[21], 2}, {&S21793[24], 15}, {&S21793[40], 37}, {&S21793[60], 17}, {NULL}};
const Segment SGI274961[] = {{&S21794[121], 3}, {NULL}};
const Segment SGI274962[] = {{&S21794[13], 7}, {&S21794[21], 2}, {&S21794[24], 15}, {&S21794[40], 20}, {&S21794[81], 31}, {NULL}};
const Segment SGI274963[] = {{&S21795[94], 3}, {NULL}};
const Segment SGI274964[] = {{&S21795[13], 7}, {&S21795[21], 2}, {&S21795[24], 15}, {&S21795[40], 6}, {&S21795[47], 18}, {&S21795[66], 19}, {&S21795[86], 31}, {NULL}};
const Segment SGI274965[] = {{&S21796[93], 3}, {NULL}};
const Segment SGI274966[] = {{&S21796[13], 7}, {&S21796[21], 2}, {&S21796[24], 15}, {&S21796[40], 6}, {&S21796[67], 17}, {NULL}};
const Segment SGI274967[] = {{&S18645[74], 4}, {NULL}};
const Segment SGI274968[] = {{&S18645[13], 14}, {&S18645[48], 17}, {NULL}};
const Segment SGI274969[] = {{&S18646[81], 4}, {NULL}};
const Segment SGI274970[] = {{&S18646[13], 14}, {&S18646[28], 6}, {&S18646[55], 17}, {NULL}};
const Segment SGI274971[] = {{&S14497[93], 4}, {NULL}};
const Segment SGI274972[] = {{&S18647[74], 4}, {NULL}};
const Segment SGI274973[] = {{&S14497[13], 9}, {&S14497[23], 22}, {&S14497[67], 17}, {NULL}};
const Segment SGI274974[] = {{&S18647[13], 14}, {&S18647[48], 17}, {NULL}};
const Segment SGI274975[] = {{&S14498[93], 4}, {NULL}};
const Segment SGI274976[] = {{&S18648[93], 4}, {NULL}};
const Segment SGI274977[] = {{&S14498[13], 9}, {&S14498[23], 22}, {&S14498[67], 17}, {NULL}};
const Segment SGI274978[] = {{&S18648[13], 7}, {&S18648[21], 2}, {&S18648[24], 15}, {&S18648[40], 6}, {&S18648[67], 17}, {NULL}};
const Segment SGI274979[] = {{&S14499[93], 4}, {NULL}};
const Segment SGI274980[] = {{&S14499[13], 9}, {&S14499[23], 22}, {&S14499[67], 17}, {NULL}};
const Segment SGI274981[] = {{&S18649[74], 4}, {NULL}};
const Segment SGI274982[] = {{&S18649[13], 14}, {&S18649[48], 17}, {&S18649[99], 8}, {&S18649[110], 24}, {NULL}};
const Segment SGI274983[] = {{&S18650[81], 4}, {NULL}};
const Segment SGI274984[] = {{&S18650[13], 14}, {&S18650[28], 6}, {&S18650[55], 17}, {NULL}};
const Segment SGI274985[] = {{&S18651[70], 4}, {NULL}};
const Segment SGI274986[] = {{&S18651[51], 42}, {NULL}};
const Segment SGI274987[] = {{&S18652[93], 4}, {NULL}};
const Segment SGI274988[] = {{&S18652[13], 7}, {&S18652[21], 2}, {&S18652[24], 15}, {&S18652[40], 6}, {&S18652[67], 17}, {&S18652[117], 8}, {&S18652[128], 24}, {NULL}};
const Segment SGI274989[] = {{&S18653[74], 4}, {NULL}};
const Segment SGI274990[] = {{&S18653[13], 14}, {&S18653[48], 17}, {&S18653[115], 35}, {&S18653[151], 19}, {NULL}};
const Segment SGI274991[] = {{&S18654[93], 4}, {NULL}};
const Segment SGI274992[] = {{&S18654[13], 7}, {&S18654[21], 2}, {&S18654[24], 15}, {&S18654[40], 6}, {&S18654[67], 17}, {NULL}};
const Segment SGI274993[] = {{&S18656[74], 4}, {NULL}};
const Segment SGI274994[] = {{&S18656[13], 14}, {&S18656[48], 17}, {&S18656[99], 8}, {NULL}};
const Segment SGI274995[] = {{&S18657[74], 4}, {NULL}};
const Segment SGI274996[] = {{&S18657[13], 14}, {&S18657[48], 17}, {NULL}};
const Segment SGI274997[] = {{&S18658[74], 4}, {NULL}};
const Segment SGI274998[] = {{&S18658[13], 14}, {&S18658[48], 17}, {NULL}};
const Segment SGI274999[] = {{&S18659[73], 4}, {NULL}};
const Segment SGI275000[] = {{&S18659[13], 14}, {&S18659[48], 16}, {NULL}};
const Segment SGI275001[] = {{&S18660[74], 4}, {NULL}};
const Segment SGI275002[] = {{&S18660[13], 14}, {&S18660[48], 17}, {&S18660[98], 8}, {&S18660[109], 26}, {NULL}};
const Segment SGI275003[] = {{&S18661[93], 4}, {NULL}};
const Segment SGI275004[] = {{&S18661[13], 7}, {&S18661[21], 2}, {&S18661[24], 15}, {&S18661[40], 6}, {&S18661[67], 17}, {&S18661[117], 8}, {NULL}};
const Segment SGI275005[] = {{&S18686[94], 4}, {NULL}};
const Segment SGI275006[] = {{&S18686[13], 7}, {&S18686[21], 2}, {&S18686[24], 15}, {&S18686[40], 6}, {&S18686[68], 17}, {NULL}};
const Segment SGI275007[] = {{&S18687[84], 4}, {NULL}};
const Segment SGI275008[] = {{&S18687[13], 7}, {&S18687[21], 15}, {&S18687[58], 17}, {NULL}};
const Segment SGI275009[] = {{&S18688[75], 4}, {NULL}};
const Segment SGI275010[] = {{&S18688[13], 14}, {&S18688[49], 17}, {&S18688[101], 8}, {&S18688[112], 24}, {NULL}};
const Segment SGI275011[] = {{&S18689[82], 4}, {NULL}};
const Segment SGI275012[] = {{&S18689[13], 14}, {&S18689[28], 6}, {&S18689[56], 17}, {NULL}};
const Segment SGI275013[] = {{&S18690[75], 4}, {NULL}};
const Segment SGI275014[] = {{&S18690[13], 14}, {&S18690[49], 17}, {NULL}};
const Segment SGI275015[] = {{&S18691[75], 4}, {NULL}};
const Segment SGI275016[] = {{&S18691[13], 14}, {&S18691[49], 17}, {NULL}};
const Segment SGI275017[] = {{&S18692[84], 4}, {NULL}};
const Segment SGI275018[] = {{&S18692[13], 7}, {&S18692[21], 15}, {&S18692[58], 17}, {NULL}};
const Segment SGI275019[] = {{&S18693[87], 4}, {NULL}};
const Segment SGI275020[] = {{&S18693[13], 14}, {&S18693[49], 17}, {&S18693[67], 11}, {NULL}};
const Segment SGI275021[] = {{&S18694[94], 4}, {NULL}};
const Segment SGI275022[] = {{&S18694[13], 7}, {&S18694[21], 2}, {&S18694[24], 15}, {&S18694[40], 6}, {&S18694[68], 17}, {NULL}};
const Segment SGI275023[] = {{&S14500[93], 4}, {NULL}};
const Segment SGI275024[] = {{&S14500[13], 9}, {&S14500[23], 22}, {&S14500[67], 17}, {NULL}};
const Segment SGI275025[] = {{&S18695[84], 4}, {NULL}};
const Segment SGI275026[] = {{&S18695[13], 7}, {&S18695[21], 15}, {&S18695[58], 17}, {&S18695[109], 8}, {&S18695[120], 24}, {NULL}};
const Segment SGI275027[] = {{&S14501[93], 4}, {NULL}};
const Segment SGI275028[] = {{&S14501[13], 9}, {&S14501[23], 22}, {&S14501[67], 17}, {NULL}};
const Segment SGI275029[] = {{&S18696[75], 4}, {NULL}};
const Segment SGI275030[] = {{&S18696[13], 14}, {&S18696[49], 17}, {&S18696[101], 8}, {&S18696[112], 26}, {NULL}};
const Segment SGI275031[] = {{&S14502[93], 4}, {NULL}};
const Segment SGI275032[] = {{&S14502[13], 9}, {&S14502[23], 22}, {&S14502[67], 17}, {NULL}};
const Segment SGI275033[] = {{&S18697[75], 4}, {NULL}};
const Segment SGI275034[] = {{&S18697[13], 14}, {&S18697[49], 17}, {&S18697[101], 12}, {NULL}};
const Segment SGI275035[] = {{&S14503[93], 4}, {NULL}};
const Segment SGI275036[] = {{&S14503[13], 9}, {&S14503[23], 22}, {&S14503[67], 17}, {NULL}};
const Segment SGI275037[] = {{&S18698[75], 4}, {NULL}};
const Segment SGI275038[] = {{&S18698[13], 14}, {&S18698[49], 17}, {&S18698[88], 21}, {NULL}};
const Segment SGI275039[] = {{&S14504[93], 4}, {NULL}};
const Segment SGI275040[] = {{&S14504[13], 9}, {&S14504[23], 22}, {&S14504[67], 17}, {NULL}};
const Segment SGI275041[] = {{&S18699[75], 4}, {NULL}};
const Segment SGI275042[] = {{&S18699[13], 14}, {&S18699[49], 17}, {NULL}};
const Segment SGI275043[] = {{&S14505[93], 4}, {NULL}};
const Segment SGI275044[] = {{&S14505[13], 9}, {&S14505[23], 22}, {&S14505[67], 17}, {NULL}};
const Segment SGI275045[] = {{&S18700[75], 4}, {NULL}};
const Segment SGI275046[] = {{&S18700[13], 14}, {&S18700[49], 17}, {&S18700[101], 8}, {NULL}};
const Segment SGI275047[] = {{&S14506[93], 4}, {NULL}};
const Segment SGI275048[] = {{&S14506[13], 9}, {&S14506[23], 22}, {&S14506[67], 17}, {&S14506[121], 8}, {NULL}};
const Segment SGI275049[] = {{&S18701[94], 4}, {NULL}};
const Segment SGI275050[] = {{&S18701[13], 7}, {&S18701[21], 2}, {&S18701[24], 15}, {&S18701[40], 6}, {&S18701[68], 17}, {NULL}};
const Segment SGI275051[] = {{&S14507[93], 4}, {NULL}};
const Segment SGI275052[] = {{&S14507[13], 9}, {&S14507[23], 22}, {&S14507[67], 17}, {&S14507[121], 8}, {&S14507[132], 26}, {NULL}};
const Segment SGI275053[] = {{&S18702[74], 4}, {NULL}};
const Segment SGI275054[] = {{&S18702[13], 14}, {&S18702[49], 16}, {NULL}};
const Segment SGI275055[] = {{&S14508[90], 4}, {NULL}};
const Segment SGI275056[] = {{&S14508[13], 9}, {&S14508[23], 20}, {&S14508[65], 16}, {NULL}};
const Segment SGI275057[] = {{&S18703[87], 4}, {NULL}};
const Segment SGI275058[] = {{&S18703[13], 14}, {&S18703[49], 17}, {&S18703[67], 11}, {NULL}};
const Segment SGI275059[] = {{&S14509[103], 4}, {NULL}};
const Segment SGI275060[] = {{&S14509[13], 9}, {&S14509[23], 2}, {&S14509[26], 22}, {&S14509[49], 6}, {&S14509[77], 17}, {NULL}};
const Segment SGI275061[] = {{&S18704[93], 4}, {NULL}};
const Segment SGI275062[] = {{&S18704[13], 7}, {&S18704[21], 2}, {&S18704[24], 15}, {&S18704[40], 6}, {&S18704[68], 16}, {NULL}};
const Segment SGI275063[] = {{&S14510[103], 4}, {NULL}};
const Segment SGI275064[] = {{&S14510[13], 9}, {&S14510[23], 2}, {&S14510[26], 22}, {&S14510[49], 6}, {&S14510[77], 17}, {NULL}};
const Segment SGI275065[] = {{&S14511[103], 4}, {NULL}};
const Segment SGI275066[] = {{&S14511[13], 9}, {&S14511[23], 2}, {&S14511[26], 22}, {&S14511[49], 6}, {&S14511[77], 17}, {NULL}};
const Segment SGI275067[] = {{&S14519[93], 4}, {NULL}};
const Segment SGI275068[] = {{&S14519[13], 9}, {&S14519[23], 22}, {&S14519[67], 17}, {NULL}};
const Segment SGI275069[] = {{&S14520[93], 4}, {NULL}};
const Segment SGI275070[] = {{&S14520[13], 9}, {&S14520[23], 22}, {&S14520[67], 17}, {NULL}};
const Segment SGI275071[] = {{&S14521[93], 4}, {NULL}};
const Segment SGI275072[] = {{&S14521[13], 9}, {&S14521[23], 22}, {&S14521[67], 17}, {NULL}};
const Segment SGI275073[] = {{&S14522[93], 4}, {NULL}};
const Segment SGI275074[] = {{&S14522[13], 9}, {&S14522[23], 22}, {&S14522[67], 17}, {NULL}};
const Segment SGI275075[] = {{&S14523[93], 4}, {NULL}};
const Segment SGI275076[] = {{&S14523[13], 9}, {&S14523[23], 22}, {&S14523[67], 17}, {NULL}};
const Segment SGI275077[] = {{&S14524[93], 4}, {NULL}};
const Segment SGI275078[] = {{&S14524[13], 9}, {&S14524[23], 22}, {&S14524[67], 17}, {NULL}};
const Segment SGI275079[] = {{&S14525[93], 4}, {NULL}};
const Segment SGI275080[] = {{&S14525[13], 9}, {&S14525[23], 22}, {&S14525[67], 17}, {&S14525[120], 8}, {NULL}};
const Segment SGI275081[] = {{&S14526[93], 4}, {NULL}};
const Segment SGI275082[] = {{&S14526[13], 9}, {&S14526[23], 22}, {&S14526[67], 17}, {&S14526[120], 8}, {&S14526[131], 26}, {NULL}};
const Segment SGI275083[] = {{&S14527[93], 4}, {NULL}};
const Segment SGI275084[] = {{&S14527[13], 9}, {&S14527[23], 22}, {&S14527[67], 17}, {&S14527[120], 8}, {NULL}};
const Segment SGI275085[] = {{&S14528[100], 4}, {NULL}};
const Segment SGI275086[] = {{&S14528[13], 9}, {&S14528[23], 22}, {&S14528[46], 6}, {&S14528[74], 17}, {NULL}};
const Segment SGI275087[] = {{&S14575[82], 4}, {NULL}};
const Segment SGI275088[] = {{&S14575[13], 14}, {&S14575[28], 6}, {&S14575[56], 17}, {NULL}};
const Segment SGI275089[] = {{&S14576[75], 4}, {NULL}};
const Segment SGI275090[] = {{&S14576[13], 14}, {&S14576[49], 17}, {NULL}};
const Segment SGI275091[] = {{&S14577[75], 4}, {NULL}};
const Segment SGI275092[] = {{&S14577[13], 14}, {&S14577[49], 17}, {NULL}};
const Segment SGI275093[] = {{&S14578[82], 4}, {NULL}};
const Segment SGI275094[] = {{&S14578[13], 14}, {&S14578[28], 6}, {&S14578[56], 17}, {NULL}};
const Segment SGI275095[] = {{&S14579[75], 4}, {NULL}};
const Segment SGI275096[] = {{&S14579[13], 14}, {&S14579[49], 17}, {&S14579[103], 8}, {NULL}};
const Segment SGI275097[] = {{&S14580[82], 4}, {NULL}};
const Segment SGI275098[] = {{&S14580[13], 14}, {&S14580[28], 6}, {&S14580[56], 17}, {NULL}};
const Segment SGI275099[] = {{&S14581[75], 4}, {NULL}};
const Segment SGI275100[] = {{&S14581[13], 14}, {&S14581[49], 17}, {NULL}};
const Segment SGI275101[] = {{&S14582[82], 4}, {NULL}};
const Segment SGI275102[] = {{&S14582[13], 14}, {&S14582[28], 6}, {&S14582[56], 17}, {&S14582[110], 8}, {NULL}};
const Segment SGI275103[] = {{&S14583[75], 4}, {NULL}};
const Segment SGI275104[] = {{&S14583[13], 14}, {&S14583[49], 17}, {&S14583[103], 8}, {&S14583[114], 26}, {NULL}};
const Segment SGI275105[] = {{&S14584[74], 4}, {NULL}};
const Segment SGI275106[] = {{&S14584[13], 14}, {&S14584[49], 16}, {NULL}};
const Segment SGI275107[] = {{&S14585[75], 4}, {NULL}};
const Segment SGI275108[] = {{&S14585[13], 14}, {&S14585[49], 17}, {&S14585[133], 3}, {&S14585[137], 14}, {&S14585[152], 11}, {NULL}};
const Segment SGI275109[] = {{&S19090[74], 4}, {NULL}};
const Segment SGI275110[] = {{&S19090[13], 14}, {&S19090[48], 17}, {NULL}};
const Segment SGI275111[] = {{&S19091[74], 4}, {NULL}};
const Segment SGI275112[] = {{&S19091[13], 14}, {&S19091[48], 17}, {&S19091[100], 8}, {&S19091[111], 24}, {NULL}};
const Segment SGI275113[] = {{&S19092[74], 4}, {NULL}};
const Segment SGI275114[] = {{&S19092[13], 14}, {&S19092[48], 17}, {&S19092[100], 8}, {&S19092[111], 26}, {NULL}};
const Segment SGI275115[] = {{&S19093[36], 4}, {NULL}};
const Segment SGI275116[] = {{&S19093[13], 14}, {NULL}};
const Segment SGI275117[] = {{&S19094[74], 4}, {NULL}};
const Segment SGI275118[] = {{&S19094[13], 14}, {&S19094[48], 17}, {NULL}};
const Segment SGI275119[] = {{&S19095[81], 4}, {NULL}};
const Segment SGI275120[] = {{&S19095[13], 14}, {&S19095[28], 6}, {&S19095[55], 17}, {NULL}};
const Segment SGI275121[] = {{&S19096[93], 4}, {NULL}};
const Segment SGI275122[] = {{&S19096[13], 7}, {&S19096[21], 2}, {&S19096[24], 15}, {&S19096[40], 6}, {&S19096[67], 17}, {NULL}};
const Segment SGI275123[] = {{&S19097[81], 4}, {NULL}};
const Segment SGI275124[] = {{&S19097[13], 14}, {&S19097[28], 6}, {&S19097[55], 17}, {NULL}};
const Segment SGI275125[] = {{&S19098[74], 4}, {NULL}};
const Segment SGI275126[] = {{&S19098[13], 14}, {&S19098[48], 17}, {&S19098[117], 35}, {&S19098[153], 20}, {NULL}};
const Segment SGI275127[] = {{&S33368[38], 5}, {NULL}};
const Segment SGI275128[] = {{&S33368[100], 5}, {NULL}};
const Segment SGI275129[] = {{&S19099[74], 4}, {NULL}};
const Segment SGI275130[] = {{&S19099[13], 14}, {&S19099[48], 17}, {&S19099[100], 8}, {NULL}};
const Segment SGI275131[] = {{&S19100[74], 5}, {NULL}};
const Segment SGI275132[] = {{&S19100[13], 14}, {&S19100[48], 17}, {NULL}};
const Segment SGI275133[] = {{&S19101[73], 4}, {NULL}};
const Segment SGI275134[] = {{&S19101[13], 14}, {&S19101[48], 16}, {NULL}};
const Segment SGI275135[] = {{&S19102[94], 4}, {NULL}};
const Segment SGI275136[] = {{&S19102[13], 7}, {&S19102[21], 2}, {&S19102[24], 15}, {&S19102[40], 6}, {&S19102[68], 17}, {NULL}};
const Segment SGI275137[] = {{&S19103[94], 4}, {NULL}};
const Segment SGI275138[] = {{&S14978[75], 4}, {NULL}};
const Segment SGI275139[] = {{&S14978[13], 14}, {&S14978[49], 17}, {NULL}};
const Segment SGI275140[] = {{&S19103[13], 7}, {&S19103[21], 2}, {&S19103[24], 15}, {&S19103[40], 6}, {&S19103[68], 17}, {NULL}};
const Segment SGI275141[] = {{&S14979[82], 4}, {NULL}};
const Segment SGI275142[] = {{&S14979[13], 14}, {&S14979[28], 6}, {&S14979[56], 17}, {NULL}};
const Segment SGI275143[] = {{&S19104[94], 4}, {NULL}};
const Segment SGI275144[] = {{&S19104[13], 7}, {&S19104[21], 2}, {&S19104[24], 15}, {&S19104[40], 6}, {&S19104[68], 17}, {&S19104[121], 8}, {&S19104[132], 24}, {NULL}};
const Segment SGI275145[] = {{&S14980[75], 4}, {NULL}};
const Segment SGI275146[] = {{&S14980[13], 14}, {&S14980[49], 17}, {NULL}};
const Segment SGI275147[] = {{&S19105[93], 4}, {NULL}};
const Segment SGI275148[] = {{&S19105[13], 7}, {&S19105[21], 2}, {&S19105[24], 15}, {&S19105[40], 6}, {&S19105[67], 17}, {NULL}};
const Segment SGI275149[] = {{&S14981[82], 4}, {NULL}};
const Segment SGI275150[] = {{&S14981[13], 14}, {&S14981[28], 6}, {&S14981[56], 17}, {NULL}};
const Segment SGI275151[] = {{&S19106[94], 4}, {NULL}};
const Segment SGI275152[] = {{&S19106[13], 7}, {&S19106[21], 2}, {&S19106[24], 15}, {&S19106[40], 6}, {&S19106[68], 17}, {&S19106[121], 8}, {&S19106[132], 26}, {NULL}};
const Segment SGI275153[] = {{&S14982[82], 4}, {NULL}};
const Segment SGI275154[] = {{&S14982[13], 14}, {&S14982[28], 6}, {&S14982[56], 17}, {NULL}};
const Segment SGI275155[] = {{&S14983[75], 4}, {NULL}};
const Segment SGI275156[] = {{&S14983[13], 14}, {&S14983[49], 17}, {NULL}};
const Segment SGI275157[] = {{&S19107[94], 4}, {NULL}};
const Segment SGI275158[] = {{&S19107[13], 7}, {&S19107[21], 2}, {&S19107[24], 15}, {&S19107[40], 6}, {&S19107[68], 17}, {&S19107[121], 8}, {NULL}};
const Segment SGI275159[] = {{&S14984[75], 4}, {NULL}};
const Segment SGI275160[] = {{&S14984[13], 14}, {&S14984[49], 17}, {&S14984[102], 8}, {NULL}};
const Segment SGI275161[] = {{&S19108[93], 4}, {NULL}};
const Segment SGI275162[] = {{&S19108[13], 7}, {&S19108[21], 2}, {&S19108[24], 15}, {&S19108[40], 6}, {&S19108[68], 16}, {NULL}};
const Segment SGI275163[] = {{&S14985[75], 4}, {NULL}};
const Segment SGI275164[] = {{&S14985[13], 14}, {&S14985[49], 17}, {&S14985[102], 8}, {&S14985[113], 26}, {NULL}};
const Segment SGI275165[] = {{&S19109[75], 4}, {NULL}};
const Segment SGI275166[] = {{&S19109[13], 14}, {&S19109[49], 17}, {NULL}};
const Segment SGI275167[] = {{&S14986[82], 4}, {NULL}};
const Segment SGI275168[] = {{&S14986[13], 14}, {&S14986[28], 6}, {&S14986[56], 17}, {&S14986[109], 8}, {NULL}};
const Segment SGI275169[] = {{&S19110[94], 4}, {NULL}};
const Segment SGI275170[] = {{&S14987[82], 4}, {NULL}};
const Segment SGI275171[] = {{&S14987[13], 14}, {&S14987[28], 6}, {&S14987[56], 17}, {&S14987[109], 8}, {&S14987[120], 26}, {NULL}};
const Segment SGI275172[] = {{&S14988[74], 4}, {NULL}};
const Segment SGI275173[] = {{&S14988[13], 14}, {&S14988[49], 16}, {NULL}};
const Segment SGI275174[] = {{&S19110[13], 7}, {&S19110[21], 2}, {&S19110[24], 15}, {&S19110[40], 7}, {&S19110[69], 16}, {NULL}};
const Segment SGI275175[] = {{&S19111[93], 4}, {NULL}};
const Segment SGI275176[] = {{&S19111[13], 7}, {&S19111[21], 2}, {&S19111[24], 15}, {&S19111[40], 6}, {&S19111[68], 16}, {NULL}};
const Segment SGI275177[] = {{&S19112[93], 4}, {NULL}};
const Segment SGI275178[] = {{&S19112[13], 7}, {&S19112[21], 2}, {&S19112[24], 15}, {&S19112[40], 6}, {&S19112[68], 16}, {NULL}};
const Segment SGI275179[] = {{&S19113[93], 4}, {NULL}};
const Segment SGI275180[] = {{&S19113[13], 7}, {&S19113[21], 2}, {&S19113[24], 15}, {&S19113[40], 6}, {&S19113[68], 16}, {NULL}};
const Segment SGI275181[] = {{&S19114[75], 4}, {NULL}};
const Segment SGI275182[] = {{&S19114[13], 14}, {&S19114[49], 17}, {NULL}};
const Segment SGI275183[] = {{&S19115[93], 4}, {NULL}};
const Segment SGI275184[] = {{&S19115[13], 7}, {&S19115[21], 2}, {&S19115[24], 15}, {&S19115[40], 6}, {&S19115[68], 16}, {NULL}};
const Segment SGI275185[] = {{&S19116[93], 4}, {NULL}};
const Segment SGI275186[] = {{&S19116[13], 7}, {&S19116[21], 2}, {&S19116[24], 15}, {&S19116[40], 6}, {&S19116[68], 16}, {NULL}};
const Segment SGI275187[] = {{&S19117[94], 4}, {NULL}};
const Segment SGI275188[] = {{&S19117[13], 7}, {&S19117[21], 2}, {&S19117[24], 15}, {&S19117[40], 6}, {&S19117[68], 17}, {&S19117[122], 23}, {NULL}};
const Segment SGI275189[] = {{&S22564[94], 3}, {NULL}};
const Segment SGI275190[] = {{&S22564[13], 7}, {&S22564[21], 2}, {&S22564[24], 15}, {&S22564[40], 6}, {&S22564[68], 17}, {NULL}};
const Segment SGI275191[] = {{&S22565[94], 3}, {NULL}};
const Segment SGI275192[] = {{&S22565[13], 7}, {&S22565[21], 2}, {&S22565[24], 15}, {&S22565[40], 6}, {&S22565[68], 17}, {&S22565[120], 8}, {&S22565[131], 24}, {NULL}};
const Segment SGI275193[] = {{&S22566[94], 3}, {NULL}};
const Segment SGI275194[] = {{&S22566[13], 7}, {&S22566[21], 2}, {&S22566[24], 15}, {&S22566[40], 6}, {&S22566[68], 17}, {NULL}};
const Segment SGI275195[] = {{&S22567[94], 3}, {NULL}};
const Segment SGI275196[] = {{&S22567[13], 7}, {&S22567[21], 2}, {&S22567[24], 15}, {&S22567[40], 6}, {&S22567[68], 17}, {NULL}};
const Segment SGI275197[] = {{&S22568[94], 3}, {NULL}};
const Segment SGI275198[] = {{&S22568[13], 7}, {&S22568[21], 2}, {&S22568[24], 15}, {&S22568[40], 6}, {&S22568[68], 17}, {&S22568[120], 8}, {NULL}};
const Segment SGI275199[] = {{&S22569[94], 3}, {NULL}};
const Segment SGI275200[] = {{&S22569[13], 7}, {&S22569[21], 2}, {&S22569[24], 15}, {&S22569[40], 6}, {&S22569[68], 17}, {NULL}};
const Segment SGI275201[] = {{&S22570[94], 3}, {NULL}};
const Segment SGI275202[] = {{&S22570[13], 7}, {&S22570[21], 2}, {&S22570[24], 15}, {&S22570[40], 6}, {&S22570[68], 17}, {NULL}};
const Segment SGI275203[] = {{&S22571[94], 3}, {NULL}};
const Segment SGI275204[] = {{&S22571[13], 7}, {&S22571[21], 2}, {&S22571[24], 15}, {&S22571[40], 6}, {&S22571[68], 17}, {&S22571[118], 8}, {&S22571[129], 24}, {NULL}};
const Segment SGI275205[] = {{&S22572[94], 3}, {&S22572[94], 3}, {NULL}};
const Segment SGI275206[] = {{&S22572[13], 7}, {&S22572[21], 2}, {&S22572[24], 15}, {&S22572[40], 6}, {&S22572[68], 17}, {NULL}};
const Segment SGI275207[] = {{&S22573[94], 3}, {NULL}};
const Segment SGI275208[] = {{&S22573[13], 7}, {&S22573[21], 2}, {&S22573[24], 15}, {&S22573[40], 6}, {&S22573[68], 17}, {NULL}};
const Segment SGI275209[] = {{&S22574[94], 3}, {NULL}};
const Segment SGI275210[] = {{&S22574[13], 7}, {&S22574[21], 2}, {&S22574[24], 15}, {&S22574[40], 6}, {&S22574[68], 17}, {&S22574[119], 8}, {NULL}};
const Segment SGI275211[] = {{&S22575[93], 3}, {NULL}};
const Segment SGI275212[] = {{&S22575[13], 7}, {&S22575[21], 2}, {&S22575[24], 15}, {&S22575[40], 6}, {&S22575[68], 16}, {NULL}};
const Segment SGI275213[] = {{&S22576[93], 3}, {NULL}};
const Segment SGI275214[] = {{&S22576[13], 7}, {&S22576[21], 2}, {&S22576[24], 15}, {&S22576[40], 6}, {&S22576[68], 16}, {NULL}};
const Segment SGI275215[] = {{&S22577[93], 3}, {NULL}};
const Segment SGI275216[] = {{&S22577[13], 7}, {&S22577[21], 2}, {&S22577[24], 15}, {&S22577[40], 6}, {&S22577[68], 16}, {NULL}};
const Segment SGI275217[] = {{&S22578[93], 3}, {NULL}};
const Segment SGI275218[] = {{&S22578[13], 7}, {&S22578[21], 2}, {&S22578[24], 15}, {&S22578[40], 6}, {&S22578[68], 16}, {NULL}};
const Segment SGI275219[] = {{&S22579[94], 3}, {NULL}};
const Segment SGI275220[] = {{&S22579[13], 7}, {&S22579[21], 2}, {&S22579[24], 15}, {&S22579[40], 7}, {&S22579[69], 16}, {NULL}};
const Segment SGI275221[] = {{&S22580[93], 3}, {NULL}};
const Segment SGI275222[] = {{&S22580[13], 7}, {&S22580[21], 2}, {&S22580[24], 15}, {&S22580[40], 6}, {&S22580[68], 16}, {NULL}};
const Segment SGI275223[] = {{&S22581[93], 3}, {NULL}};
const Segment SGI275224[] = {{&S22581[13], 7}, {&S22581[21], 2}, {&S22581[24], 15}, {&S22581[40], 6}, {&S22581[68], 16}, {NULL}};
const Segment SGI275225[] = {{&S22582[93], 3}, {NULL}};
const Segment SGI275226[] = {{&S22582[13], 7}, {&S22582[21], 2}, {&S22582[24], 15}, {&S22582[40], 6}, {&S22582[68], 16}, {NULL}};
const Segment SGI275227[] = {{&S22583[93], 3}, {NULL}};
const Segment SGI275228[] = {{&S22583[13], 7}, {&S22583[21], 2}, {&S22583[24], 15}, {&S22583[40], 6}, {&S22583[68], 16}, {NULL}};
const Segment SGI275229[] = {{&S22584[93], 3}, {NULL}};
const Segment SGI275230[] = {{&S22584[13], 7}, {&S22584[21], 2}, {&S22584[24], 15}, {&S22584[40], 6}, {&S22584[68], 16}, {NULL}};
const Segment SGI275231[] = {{&S22585[94], 3}, {NULL}};
const Segment SGI275232[] = {{&S22585[13], 7}, {&S22585[21], 2}, {&S22585[24], 15}, {&S22585[40], 6}, {&S22585[68], 17}, {NULL}};
const Segment SGI275233[] = {{&S22586[93], 3}, {NULL}};
const Segment SGI275234[] = {{&S22586[13], 7}, {&S22586[21], 2}, {&S22586[24], 15}, {&S22586[40], 6}, {&S22586[68], 16}, {NULL}};
const Segment SGI275235[] = {{&S22587[93], 3}, {NULL}};
const Segment SGI275236[] = {{&S22587[13], 7}, {&S22587[21], 2}, {&S22587[24], 15}, {&S22587[40], 6}, {&S22587[68], 16}, {NULL}};
const Segment SGI275237[] = {{&S22588[93], 3}, {NULL}};
const Segment SGI275238[] = {{&S22588[13], 7}, {&S22588[21], 2}, {&S22588[24], 15}, {&S22588[40], 6}, {&S22588[68], 16}, {NULL}};
const Segment SGI275239[] = {{&S22589[93], 3}, {NULL}};
const Segment SGI275240[] = {{&S22589[13], 7}, {&S22589[21], 2}, {&S22589[24], 15}, {&S22589[40], 6}, {&S22589[68], 16}, {NULL}};
const Segment SGI275241[] = {{&S22590[91], 3}, {NULL}};
const Segment SGI275242[] = {{&S22590[13], 30}, {&S22590[65], 17}, {NULL}};
const Segment SGI275243[] = {{&S33696[38], 5}, {NULL}};
const Segment SGI275244[] = {{&S33696[100], 5}, {NULL}};
const Segment SGI275245[] = {{&S19526[41], 4}, {NULL}};
const Segment SGI275246[] = {{&S19526[13], 14}, {&S19526[28], 25}, {&S19526[74], 17}, {NULL}};
const Segment SGI275247[] = {{&S19526[29], 16}, {NULL}};
const Segment SGI275248[] = {{&S19528[41], 4}, {NULL}};
const Segment SGI275249[] = {{&S19528[13], 14}, {&S19528[28], 25}, {&S19528[74], 17}, {NULL}};
const Segment SGI275250[] = {{&S19528[29], 16}, {NULL}};
const Segment SGI275251[] = {{&S19530[41], 4}, {NULL}};
const Segment SGI275252[] = {{&S19530[13], 14}, {&S19530[28], 25}, {&S19530[74], 17}, {NULL}};
const Segment SGI275253[] = {{&S19530[29], 16}, {NULL}};
const Segment SGI275254[] = {{&S19533[41], 4}, {NULL}};
const Segment SGI275255[] = {{&S19533[13], 14}, {&S19533[28], 25}, {&S19533[74], 17}, {NULL}};
const Segment SGI275256[] = {{&S19533[29], 16}, {NULL}};
const Segment SGI275257[] = {{&S19538[41], 4}, {NULL}};
const Segment SGI275258[] = {{&S19538[13], 14}, {&S19538[28], 23}, {&S19538[72], 17}, {NULL}};
const Segment SGI275259[] = {{&S19538[29], 16}, {NULL}};
const Segment SGI275260[] = {{&S19539[41], 4}, {NULL}};
const Segment SGI275261[] = {{&S19539[13], 14}, {&S19539[28], 25}, {&S19539[74], 17}, {NULL}};
const Segment SGI275262[] = {{&S19539[29], 16}, {NULL}};
const Segment SGI275263[] = {{&S19541[41], 4}, {NULL}};
const Segment SGI275264[] = {{&S19541[13], 14}, {&S19541[28], 23}, {&S19541[72], 17}, {NULL}};
const Segment SGI275265[] = {{&S19541[29], 16}, {NULL}};
const Segment SGI275266[] = {{&S15494[74], 4}, {NULL}};
const Segment SGI275267[] = {{&S15494[13], 14}, {&S15494[48], 17}, {NULL}};
const Segment SGI275268[] = {{&S15495[74], 4}, {NULL}};
const Segment SGI275269[] = {{&S15495[13], 14}, {&S15495[48], 17}, {NULL}};
const Segment SGI275270[] = {{&S15496[81], 4}, {NULL}};
const Segment SGI275271[] = {{&S15496[13], 14}, {&S15496[28], 6}, {&S15496[55], 17}, {NULL}};
const Segment SGI275272[] = {{&S15497[74], 4}, {NULL}};
const Segment SGI275273[] = {{&S15497[13], 14}, {&S15497[48], 17}, {&S15497[100], 8}, {&S15497[111], 24}, {NULL}};
const Segment SGI275274[] = {{&S15498[74], 4}, {NULL}};
const Segment SGI275275[] = {{&S15498[13], 14}, {&S15498[48], 17}, {&S15498[100], 8}, {&S15498[111], 26}, {NULL}};
const Segment SGI275276[] = {{&S15499[74], 4}, {NULL}};
const Segment SGI275277[] = {{&S15499[13], 14}, {&S15499[48], 17}, {&S15499[100], 8}, {NULL}};
const Segment SGI275278[] = {{&S15500[96], 4}, {NULL}};
const Segment SGI275279[] = {{&S15500[13], 14}, {&S15500[11], 1}, {&S15500[30], 19}, {&S15500[70], 17}, {NULL}};
const Segment SGI275280[] = {{&S15501[81], 4}, {NULL}};
const Segment SGI275281[] = {{&S15501[13], 14}, {&S15501[28], 6}, {&S15501[55], 17}, {NULL}};
const Segment SGI275282[] = {{&S15524[93], 4}, {NULL}};
const Segment SGI275283[] = {{&S15524[13], 9}, {&S15524[23], 22}, {&S15524[67], 17}, {&S15524[120], 8}, {NULL}};
const Segment SGI275284[] = {{&S15525[93], 4}, {NULL}};
const Segment SGI275285[] = {{&S15525[13], 9}, {&S15525[23], 22}, {&S15525[67], 17}, {NULL}};
const Segment SGI275286[] = {{&S15526[93], 4}, {NULL}};
const Segment SGI275287[] = {{&S15526[13], 9}, {&S15526[23], 22}, {&S15526[67], 17}, {NULL}};
const Segment SGI275288[] = {{&S15527[93], 4}, {NULL}};
const Segment SGI275289[] = {{&S15527[13], 9}, {&S15527[23], 22}, {&S15527[67], 17}, {NULL}};
const Segment SGI275290[] = {{&S15528[93], 4}, {NULL}};
const Segment SGI275291[] = {{&S15528[13], 9}, {&S15528[23], 22}, {&S15528[67], 17}, {NULL}};
const Segment SGI275292[] = {{&S15529[93], 4}, {NULL}};
const Segment SGI275293[] = {{&S15529[13], 9}, {&S15529[23], 22}, {&S15529[67], 17}, {NULL}};
const Segment SGI275294[] = {{&S15530[93], 4}, {NULL}};
const Segment SGI275295[] = {{&S15530[13], 9}, {&S15530[23], 22}, {&S15530[67], 17}, {NULL}};
const Segment SGI275296[] = {{&S15531[93], 4}, {NULL}};
const Segment SGI275297[] = {{&S15531[13], 9}, {&S15531[23], 22}, {&S15531[67], 17}, {&S15531[120], 8}, {&S15531[131], 26}, {NULL}};
const Segment SGI275298[] = {{&S15532[93], 4}, {NULL}};
const Segment SGI275299[] = {{&S15532[13], 9}, {&S15532[23], 22}, {&S15532[67], 17}, {&S15532[120], 8}, {NULL}};
const Segment SGI275300[] = {{&S15533[93], 4}, {NULL}};
const Segment SGI275301[] = {{&S15533[13], 9}, {&S15533[23], 22}, {&S15533[67], 17}, {NULL}};
const Segment SGI275302[] = {{&S15534[93], 4}, {NULL}};
const Segment SGI275303[] = {{&S15534[13], 9}, {&S15534[23], 22}, {&S15534[67], 17}, {NULL}};
const Segment SGI275304[] = {{&S15535[93], 4}, {NULL}};
const Segment SGI275305[] = {{&S15535[13], 9}, {&S15535[23], 22}, {&S15535[67], 17}, {NULL}};
const Segment SGI275306[] = {{&S15536[93], 4}, {NULL}};
const Segment SGI275307[] = {{&S15536[13], 9}, {&S15536[23], 22}, {&S15536[67], 17}, {NULL}};
const Segment SGI275308[] = {{&S15537[93], 4}, {NULL}};
const Segment SGI275309[] = {{&S15537[13], 9}, {&S15537[23], 22}, {&S15537[67], 17}, {NULL}};
const Segment SGI275310[] = {{&S15538[93], 4}, {NULL}};
const Segment SGI275311[] = {{&S15538[13], 9}, {&S15538[23], 22}, {&S15538[67], 17}, {NULL}};
const Segment SGI275312[] = {{&S15539[93], 4}, {NULL}};
const Segment SGI275313[] = {{&S15539[13], 9}, {&S15539[23], 22}, {&S15539[67], 17}, {NULL}};
const Segment SGI275314[] = {{&S15540[93], 4}, {NULL}};
const Segment SGI275315[] = {{&S15540[13], 9}, {&S15540[23], 22}, {&S15540[67], 17}, {NULL}};
const Segment SGI275316[] = {{&S15541[93], 4}, {NULL}};
const Segment SGI275317[] = {{&S15541[13], 9}, {&S15541[23], 22}, {&S15541[67], 17}, {NULL}};
const Segment SGI275318[] = {{&S15542[93], 4}, {NULL}};
const Segment SGI275319[] = {{&S15542[13], 9}, {&S15542[23], 22}, {&S15542[67], 17}, {&S15542[121], 8}, {NULL}};
const Segment SGI275320[] = {{&S15543[93], 4}, {NULL}};
const Segment SGI275321[] = {{&S15543[13], 9}, {&S15543[23], 22}, {&S15543[67], 17}, {&S15543[121], 8}, {&S15543[132], 26}, {NULL}};
const Segment SGI275322[] = {{&S15544[103], 4}, {NULL}};
const Segment SGI275323[] = {{&S15544[13], 9}, {&S15544[23], 2}, {&S15544[26], 22}, {&S15544[49], 6}, {&S15544[77], 17}, {NULL}};
const Segment SGI275324[] = {{&S15545[103], 4}, {NULL}};
const Segment SGI275325[] = {{&S15545[13], 9}, {&S15545[23], 2}, {&S15545[26], 22}, {&S15545[49], 6}, {&S15545[77], 17}, {NULL}};
const Segment SGI275326[] = {{&S15546[103], 4}, {NULL}};
const Segment SGI275327[] = {{&S15546[13], 9}, {&S15546[23], 2}, {&S15546[26], 22}, {&S15546[49], 6}, {&S15546[77], 17}, {NULL}};
const Segment SGI275328[] = {{&S15737[41], 4}, {NULL}};
const Segment SGI275329[] = {{&S15737[13], 14}, {&S15737[28], 25}, {&S15737[74], 17}, {NULL}};
const Segment SGI275330[] = {{&S15738[48], 4}, {NULL}};
const Segment SGI275331[] = {{&S15738[13], 14}, {&S15738[28], 6}, {&S15738[35], 25}, {&S15738[81], 17}, {NULL}};
const Segment SGI275332[] = {{&S15739[48], 4}, {NULL}};
const Segment SGI275333[] = {{&S15739[13], 14}, {&S15739[28], 6}, {&S15739[35], 25}, {&S15739[81], 17}, {NULL}};
const Segment SGI275334[] = {{&S15740[41], 4}, {NULL}};
const Segment SGI275335[] = {{&S15740[13], 14}, {&S15740[28], 25}, {&S15740[74], 17}, {NULL}};
const Segment SGI275336[] = {{&S15741[48], 4}, {NULL}};
const Segment SGI275337[] = {{&S15741[13], 14}, {&S15741[28], 6}, {&S15741[35], 25}, {&S15741[81], 17}, {NULL}};
const Segment SGI275338[] = {{&S15742[41], 4}, {NULL}};
const Segment SGI275339[] = {{&S15742[13], 14}, {&S15742[28], 25}, {&S15742[74], 17}, {NULL}};
const Segment SGI275340[] = {{&S15746[41], 4}, {NULL}};
const Segment SGI275341[] = {{&S15746[13], 14}, {&S15746[28], 25}, {&S15746[74], 17}, {NULL}};
const Segment SGI275342[] = {{&S15747[48], 4}, {NULL}};
const Segment SGI275343[] = {{&S15747[13], 14}, {&S15747[28], 6}, {&S15747[35], 25}, {&S15747[81], 17}, {NULL}};
const Segment SGI275344[] = {{&S15752[41], 4}, {NULL}};
const Segment SGI275345[] = {{&S15752[13], 14}, {&S15752[28], 25}, {&S15752[74], 17}, {NULL}};
const Segment SGI275346[] = {{&S15754[48], 4}, {NULL}};
const Segment SGI275347[] = {{&S15754[13], 14}, {&S15754[28], 6}, {&S15754[35], 25}, {&S15754[81], 17}, {NULL}};
const Segment SGI275348[] = {{&S15760[41], 4}, {NULL}};
const Segment SGI275349[] = {{&S15760[13], 14}, {&S15760[28], 25}, {&S15760[74], 17}, {NULL}};
const Segment SGI275350[] = {{&S15762[41], 4}, {NULL}};
const Segment SGI275351[] = {{&S15762[13], 14}, {&S15762[28], 25}, {&S15762[74], 17}, {NULL}};
const Segment SGI275352[] = {{&S15763[48], 4}, {NULL}};
const Segment SGI275353[] = {{&S15763[13], 14}, {&S15763[28], 6}, {&S15763[35], 25}, {&S15763[81], 17}, {NULL}};
const Segment SGI275354[] = {{&S15770[48], 4}, {NULL}};
const Segment SGI275355[] = {{&S15770[13], 14}, {&S15770[28], 6}, {&S15770[35], 25}, {&S15770[81], 17}, {NULL}};
const Segment SGI275356[] = {{&S15771[93], 4}, {NULL}};
const Segment SGI275357[] = {{&S15771[13], 9}, {&S15771[23], 22}, {&S15771[67], 17}, {NULL}};
const Segment SGI275358[] = {{&S15772[93], 4}, {NULL}};
const Segment SGI275359[] = {{&S15772[13], 9}, {&S15772[23], 22}, {&S15772[67], 17}, {NULL}};
const Segment SGI275360[] = {{&S15773[93], 4}, {NULL}};
const Segment SGI275361[] = {{&S15773[13], 9}, {&S15773[23], 22}, {&S15773[67], 17}, {NULL}};
const Segment SGI275362[] = {{&S15774[93], 4}, {NULL}};
const Segment SGI275363[] = {{&S15774[13], 9}, {&S15774[23], 22}, {&S15774[67], 17}, {NULL}};
const Segment SGI275364[] = {{&S15775[93], 4}, {NULL}};
const Segment SGI275365[] = {{&S15775[13], 9}, {&S15775[23], 22}, {&S15775[67], 17}, {NULL}};
const Segment SGI275366[] = {{&S15776[93], 4}, {NULL}};
const Segment SGI275367[] = {{&S15776[13], 9}, {&S15776[23], 22}, {&S15776[67], 17}, {NULL}};
const Segment SGI275368[] = {{&S15777[93], 4}, {NULL}};
const Segment SGI275369[] = {{&S15777[13], 9}, {&S15777[23], 22}, {&S15777[67], 17}, {NULL}};
const Segment SGI275370[] = {{&S23351[8], 3}, {NULL}};
const Segment SGI275371[] = {{&S23351[13], 7}, {&S23351[21], 2}, {&S23351[24], 15}, {&S23351[40], 6}, {&S23351[67], 17}, {NULL}};
const Segment SGI275372[] = {{&S15778[93], 4}, {NULL}};
const Segment SGI275373[] = {{&S15778[13], 9}, {&S15778[23], 22}, {&S15778[67], 17}, {NULL}};
const Segment SGI275374[] = {{&S23352[8], 3}, {NULL}};
const Segment SGI275375[] = {{&S23352[13], 7}, {&S23352[21], 2}, {&S23352[24], 15}, {&S23352[40], 6}, {&S23352[67], 17}, {&S23352[118], 8}, {&S23352[129], 24}, {NULL}};
const Segment SGI275376[] = {{&S15779[93], 4}, {NULL}};
const Segment SGI275377[] = {{&S15779[13], 9}, {&S15779[23], 22}, {&S15779[67], 17}, {&S15779[121], 8}, {NULL}};
const Segment SGI275378[] = {{&S23353[8], 3}, {NULL}};
const Segment SGI275379[] = {{&S23353[13], 7}, {&S23353[21], 2}, {&S23353[24], 15}, {&S23353[40], 6}, {&S23353[67], 17}, {NULL}};
const Segment SGI275380[] = {{&S15780[93], 4}, {NULL}};
const Segment SGI275381[] = {{&S15780[13], 9}, {&S15780[23], 22}, {&S15780[67], 17}, {NULL}};
const Segment SGI275382[] = {{&S23354[8], 3}, {NULL}};
const Segment SGI275383[] = {{&S23354[13], 14}, {&S23354[46], 17}, {NULL}};
const Segment SGI275384[] = {{&S15781[93], 4}, {NULL}};
const Segment SGI275385[] = {{&S15781[13], 9}, {&S15781[23], 22}, {&S15781[67], 17}, {&S15781[121], 8}, {&S15781[132], 26}, {NULL}};
const Segment SGI275386[] = {{&S23355[8], 3}, {NULL}};
const Segment SGI275387[] = {{&S23355[13], 7}, {&S23355[21], 2}, {&S23355[24], 15}, {&S23355[40], 6}, {&S23355[67], 17}, {&S23355[117], 8}, {NULL}};
const Segment SGI275388[] = {{&S23356[8], 3}, {NULL}};
const Segment SGI275389[] = {{&S23356[13], 7}, {&S23356[21], 2}, {&S23356[24], 15}, {&S23356[40], 6}, {&S23356[47], 18}, {&S23356[66], 19}, {&S23356[86], 30}, {NULL}};
const Segment SGI275390[] = {{&S15822[93], 4}, {NULL}};
const Segment SGI275391[] = {{&S15822[13], 9}, {&S15822[23], 22}, {&S15822[67], 17}, {NULL}};
const Segment SGI275392[] = {{&S15823[93], 4}, {NULL}};
const Segment SGI275393[] = {{&S15823[13], 9}, {&S15823[23], 22}, {&S15823[67], 17}, {NULL}};
const Segment SGI275394[] = {{&S15824[93], 4}, {NULL}};
const Segment SGI275395[] = {{&S15824[13], 9}, {&S15824[23], 22}, {&S15824[67], 17}, {NULL}};
const Segment SGI275396[] = {{&S15825[93], 4}, {NULL}};
const Segment SGI275397[] = {{&S15825[13], 9}, {&S15825[23], 22}, {&S15825[67], 17}, {NULL}};
const Segment SGI275398[] = {{&S15826[93], 4}, {NULL}};
const Segment SGI275399[] = {{&S15826[13], 9}, {&S15826[23], 22}, {&S15826[67], 17}, {NULL}};
const Segment SGI275400[] = {{&S15827[93], 4}, {NULL}};
const Segment SGI275401[] = {{&S15827[13], 9}, {&S15827[23], 22}, {&S15827[67], 17}, {NULL}};
const Segment SGI275402[] = {{&S15828[93], 4}, {NULL}};
const Segment SGI275403[] = {{&S15828[13], 9}, {&S15828[23], 22}, {&S15828[67], 17}, {NULL}};
const Segment SGI275404[] = {{&S15829[93], 4}, {NULL}};
const Segment SGI275405[] = {{&S15829[13], 9}, {&S15829[23], 22}, {&S15829[67], 17}, {NULL}};
const Segment SGI275406[] = {{&S15830[93], 4}, {NULL}};
const Segment SGI275407[] = {{&S15830[13], 9}, {&S15830[23], 22}, {&S15830[67], 17}, {NULL}};
const Segment SGI275408[] = {{&S15831[93], 4}, {NULL}};
const Segment SGI275409[] = {{&S15831[13], 9}, {&S15831[23], 22}, {&S15831[67], 17}, {&S15831[120], 8}, {NULL}};
const Segment SGI275410[] = {{&S15832[93], 4}, {NULL}};
const Segment SGI275411[] = {{&S15832[13], 9}, {&S15832[23], 22}, {&S15832[67], 17}, {&S15832[120], 8}, {&S15832[131], 26}, {NULL}};
const Segment SGI275412[] = {{&S23595[93], 3}, {NULL}};
const Segment SGI275413[] = {{&S23595[13], 7}, {&S23595[21], 2}, {&S23595[24], 15}, {&S23595[40], 6}, {&S23595[67], 17}, {NULL}};
const Segment SGI275414[] = {{&S23596[93], 3}, {NULL}};
const Segment SGI275415[] = {{&S23596[13], 7}, {&S23596[21], 2}, {&S23596[24], 15}, {&S23596[40], 6}, {&S23596[67], 17}, {NULL}};
const Segment SGI275416[] = {{&S23597[93], 3}, {NULL}};
const Segment SGI275417[] = {{&S23597[13], 7}, {&S23597[21], 2}, {&S23597[24], 15}, {&S23597[40], 6}, {&S23597[67], 17}, {NULL}};
const Segment SGI275418[] = {{&S23598[93], 3}, {NULL}};
const Segment SGI275419[] = {{&S23598[13], 7}, {&S23598[21], 2}, {&S23598[24], 15}, {&S23598[40], 6}, {&S23598[67], 17}, {NULL}};
const Segment SGI275420[] = {{&S23599[55], 3}, {NULL}};
const Segment SGI275421[] = {{&S23599[13], 7}, {&S23599[21], 2}, {&S23599[24], 15}, {&S23599[40], 6}, {NULL}};
const Segment SGI275422[] = {{&S23600[86], 3}, {NULL}};
const Segment SGI275423[] = {{&S23600[13], 7}, {&S23600[21], 2}, {&S23600[24], 15}, {&S23600[60], 17}, {NULL}};
const Segment SGI275424[] = {{&S23617[8], 3}, {NULL}};
const Segment SGI275425[] = {{&S23617[13], 3}, {&S23617[17], 2}, {&S23617[20], 13}, {&S23617[34], 6}, {&S23617[61], 17}, {NULL}};
const Segment SGI275426[] = {{&S23618[8], 3}, {NULL}};
const Segment SGI275427[] = {{&S23618[13], 3}, {&S23618[17], 2}, {&S23618[20], 11}, {&S23618[32], 6}, {&S23618[59], 17}, {NULL}};
const Segment SGI275428[] = {{&S23619[8], 3}, {NULL}};
const Segment SGI275429[] = {{&S23619[13], 3}, {&S23619[17], 2}, {&S23619[20], 21}, {&S23619[42], 6}, {&S23619[69], 17}, {NULL}};
const Segment SGI275430[] = {{&S23620[8], 3}, {NULL}};
const Segment SGI275431[] = {{&S23620[13], 3}, {&S23620[17], 2}, {&S23620[20], 11}, {&S23620[32], 6}, {&S23620[59], 17}, {NULL}};
const Segment SGI275432[] = {{&S23621[8], 3}, {NULL}};
const Segment SGI275433[] = {{&S23621[13], 3}, {&S23621[17], 2}, {&S23621[20], 13}, {&S23621[34], 13}, {&S23621[68], 17}, {NULL}};
const Segment SGI275434[] = {{&S23622[8], 3}, {NULL}};
const Segment SGI275435[] = {{&S23622[13], 3}, {&S23622[17], 2}, {&S23622[20], 13}, {&S23622[34], 6}, {&S23622[61], 17}, {NULL}};
const Segment SGI275436[] = {{&S23960[8], 3}, {NULL}};
const Segment SGI275437[] = {{&S23960[13], 7}, {&S23960[21], 2}, {&S23960[24], 15}, {&S23960[40], 6}, {&S23960[67], 17}, {NULL}};
const Segment SGI275438[] = {{&S23961[8], 3}, {NULL}};
const Segment SGI275439[] = {{&S23961[13], 7}, {&S23961[21], 2}, {&S23961[24], 15}, {&S23961[40], 6}, {&S23961[67], 17}, {NULL}};
const Segment SGI275440[] = {{&S23962[8], 3}, {NULL}};
const Segment SGI275441[] = {{&S23962[13], 7}, {&S23962[21], 2}, {&S23962[24], 15}, {&S23962[40], 6}, {&S23962[67], 17}, {&S23962[118], 8}, {&S23962[129], 24}, {NULL}};
const Segment SGI275442[] = {{&S23963[8], 3}, {NULL}};
const Segment SGI275443[] = {{&S23963[13], 7}, {&S23963[21], 2}, {&S23963[24], 15}, {&S23963[40], 6}, {&S23963[67], 17}, {NULL}};
const Segment SGI275444[] = {{&S23964[8], 3}, {NULL}};
const Segment SGI275445[] = {{&S23964[13], 7}, {&S23964[21], 2}, {&S23964[24], 15}, {&S23964[40], 6}, {&S23964[67], 17}, {&S23964[117], 8}, {NULL}};
const Segment SGI275446[] = {{&S23965[8], 3}, {NULL}};
const Segment SGI275447[] = {{&S23965[13], 7}, {&S23965[21], 2}, {&S23965[24], 15}, {&S23965[40], 6}, {&S23965[67], 17}, {&S23965[134], 35}, {&S23965[170], 20}, {NULL}};
const Segment SGI275448[] = {{&S23994[93], 3}, {NULL}};
const Segment SGI275449[] = {{&S23994[13], 7}, {&S23994[21], 2}, {&S23994[24], 15}, {&S23994[40], 6}, {&S23994[67], 17}, {NULL}};
const Segment SGI275450[] = {{&S23995[82], 3}, {NULL}};
const Segment SGI275451[] = {{&S23995[13], 14}, {&S23995[28], 6}, {&S23995[56], 17}, {NULL}};
const Segment SGI275452[] = {{&S23996[75], 3}, {NULL}};
const Segment SGI275453[] = {{&S23996[13], 14}, {&S23996[49], 17}, {NULL}};
const Segment SGI275454[] = {{&S23997[93], 3}, {&S23997[93], 3}, {NULL}};
const Segment SGI275455[] = {{&S23997[13], 7}, {&S23997[21], 2}, {&S23997[24], 15}, {&S23997[40], 6}, {&S23997[67], 17}, {NULL}};
const Segment SGI275456[] = {{&S23998[94], 3}, {NULL}};
const Segment SGI275457[] = {{&S23998[13], 7}, {&S23998[21], 2}, {&S23998[24], 15}, {&S23998[40], 6}, {&S23998[68], 17}, {&S23998[118], 31}, {NULL}};
const Segment SGI275458[] = {{&S23999[94], 3}, {NULL}};
const Segment SGI275459[] = {{&S23999[13], 7}, {&S23999[21], 2}, {&S23999[24], 15}, {&S23999[40], 6}, {&S23999[68], 17}, {NULL}};
const Segment SGI275460[] = {{&S24000[94], 3}, {NULL}};
const Segment SGI275461[] = {{&S24000[13], 7}, {&S24000[21], 2}, {&S24000[24], 15}, {&S24000[40], 6}, {&S24000[68], 17}, {&S24000[118], 31}, {NULL}};
const Segment SGI275462[] = {{&S24001[93], 3}, {NULL}};
const Segment SGI275463[] = {{&S24001[13], 7}, {&S24001[21], 2}, {&S24001[24], 15}, {&S24001[40], 6}, {&S24001[67], 17}, {NULL}};
const Segment SGI275464[] = {{&S24002[94], 3}, {NULL}};
const Segment SGI275465[] = {{&S24002[13], 7}, {&S24002[21], 2}, {&S24002[24], 15}, {&S24002[40], 6}, {&S24002[68], 17}, {&S24002[118], 8}, {&S24002[129], 24}, {NULL}};
const Segment SGI275466[] = {{&S24003[94], 3}, {NULL}};
const Segment SGI275467[] = {{&S24003[13], 7}, {&S24003[21], 2}, {&S24003[24], 15}, {&S24003[40], 6}, {&S24003[68], 17}, {&S24003[118], 31}, {NULL}};
const Segment SGI275468[] = {{&S24004[93], 3}, {NULL}};
const Segment SGI275469[] = {{&S24004[13], 7}, {&S24004[21], 2}, {&S24004[24], 15}, {&S24004[40], 6}, {&S24004[67], 17}, {&S24004[117], 8}, {NULL}};
const Segment SGI275470[] = {{&S21161[78], 4}, {NULL}};
const Segment SGI275471[] = {{&S21161[13], 3}, {&S21161[17], 13}, {&S21161[52], 17}, {NULL}};
const Segment SGI275472[] = {{&S21162[86], 4}, {NULL}};
const Segment SGI275473[] = {{&S21162[13], 3}, {&S21162[17], 2}, {&S21162[20], 11}, {&S21162[32], 6}, {&S21162[60], 17}, {NULL}};
const Segment SGI275474[] = {{&S21163[100], 4}, {NULL}};
const Segment SGI275475[] = {{&S21163[13], 3}, {&S21163[17], 21}, {&S21163[60], 17}, {NULL}};
const Segment SGI275476[] = {{&S21164[76], 4}, {NULL}};
const Segment SGI275477[] = {{&S21164[13], 3}, {&S21164[17], 11}, {&S21164[50], 17}, {NULL}};
const Segment SGI275478[] = {{&S21165[85], 4}, {NULL}};
const Segment SGI275479[] = {{&S21165[13], 3}, {&S21165[17], 19}, {&S21165[59], 17}, {NULL}};
const Segment SGI275480[] = {{&S21166[98], 4}, {NULL}};
const Segment SGI275481[] = {{&S21166[13], 3}, {&S21166[17], 13}, {&S21166[52], 17}, {&S21166[70], 19}, {NULL}};
const Segment SGI275482[] = {{&S21167[81], 4}, {NULL}};
const Segment SGI275483[] = {{&S21167[13], 3}, {&S21167[17], 2}, {&S21167[20], 6}, {&S21167[27], 6}, {&S21167[55], 17}, {NULL}};
const Segment SGI275484[] = {{&S21168[85], 4}, {NULL}};
const Segment SGI275485[] = {{&S24317[93], 3}, {NULL}};
const Segment SGI275486[] = {{&S24317[13], 7}, {&S24317[21], 2}, {&S24317[24], 15}, {&S24317[40], 6}, {&S24317[67], 17}, {NULL}};
const Segment SGI275487[] = {{&S24318[93], 3}, {NULL}};
const Segment SGI275488[] = {{&S24318[13], 7}, {&S24318[21], 2}, {&S24318[24], 15}, {&S24318[40], 6}, {&S24318[67], 17}, {&S24318[117], 8}, {&S24318[128], 24}, {NULL}};
const Segment SGI275489[] = {{&S24319[93], 3}, {NULL}};
const Segment SGI275490[] = {{&S24319[13], 7}, {&S24319[21], 2}, {&S24319[24], 15}, {&S24319[40], 6}, {&S24319[67], 17}, {NULL}};
const Segment SGI275491[] = {{&S24320[118], 3}, {NULL}};
const Segment SGI275492[] = {{&S24320[13], 7}, {&S24320[21], 2}, {&S24320[24], 15}, {&S24320[40], 6}, {&S24320[47], 22}, {&S24320[11], 1}, {&S24320[92], 17}, {NULL}};
const Segment SGI275493[] = {{&S21168[13], 3}, {&S21168[17], 13}, {&S21168[31], 6}, {&S21168[59], 17}, {NULL}};
const Segment SGI275494[] = {{&S24321[120], 3}, {NULL}};
const Segment SGI275495[] = {{&S24321[13], 7}, {&S24321[21], 2}, {&S24321[24], 15}, {&S24321[40], 6}, {&S24321[47], 24}, {&S24321[11], 1}, {&S24321[94], 17}, {NULL}};
const Segment SGI275496[] = {{&S21169[104], 4}, {NULL}};
const Segment SGI275497[] = {{&S21169[29], 3}, {&S21169[33], 2}, {&S21169[36], 13}, {&S21169[50], 6}, {&S21169[78], 17}, {NULL}};
const Segment SGI275498[] = {{&S24322[93], 3}, {NULL}};
const Segment SGI275499[] = {{&S24322[13], 7}, {&S24322[21], 2}, {&S24322[24], 15}, {&S24322[40], 6}, {&S24322[67], 17}, {NULL}};
const Segment SGI275500[] = {{&S24323[93], 3}, {NULL}};
const Segment SGI275501[] = {{&S24323[13], 7}, {&S24323[21], 2}, {&S24323[24], 15}, {&S24323[40], 6}, {&S24323[67], 17}, {&S24323[117], 8}, {NULL}};
const Segment SGI275502[] = {{&S21170[95], 4}, {NULL}};
const Segment SGI275503[] = {{&S21170[13], 3}, {&S21170[17], 2}, {&S21170[20], 19}, {&S21170[41], 6}, {&S21170[69], 17}, {NULL}};
const Segment SGI275504[] = {{&S21171[105], 4}, {NULL}};
const Segment SGI275505[] = {{&S21171[13], 5}, {&S21171[19], 1}, {&S21171[21], 12}, {&S21171[34], 5}, {&S21171[40], 17}, {&S21171[79], 17}, {NULL}};
const Segment SGI275506[] = {{&S21172[109], 4}, {NULL}};
const Segment SGI275507[] = {{&S21172[13], 3}, {&S21172[17], 13}, {&S21172[31], 6}, {&S21172[38], 23}, {&S21172[83], 17}, {NULL}};
const Segment SGI275508[] = {{&S21173[78], 4}, {NULL}};
const Segment SGI275509[] = {{&S21173[13], 3}, {&S21173[17], 13}, {&S21173[52], 17}, {&S21173[104], 8}, {&S21173[115], 24}, {NULL}};
const Segment SGI275510[] = {{&S21174[92], 4}, {NULL}};
const Segment SGI275511[] = {{&S21174[13], 3}, {&S21174[17], 13}, {&S21174[31], 6}, {&S21174[38], 6}, {&S21174[66], 17}, {NULL}};
const Segment SGI275512[] = {{&S21175[92], 4}, {NULL}};
const Segment SGI275513[] = {{&S21175[13], 3}, {&S21175[17], 13}, {&S21175[31], 6}, {&S21175[38], 6}, {&S21175[66], 17}, {NULL}};
const Segment SGI275514[] = {{&S21176[92], 4}, {NULL}};
const Segment SGI275515[] = {{&S21176[13], 3}, {&S21176[17], 13}, {&S21176[31], 6}, {&S21176[38], 6}, {&S21176[66], 17}, {NULL}};
const Segment SGI275516[] = {{&S21177[92], 4}, {NULL}};
const Segment SGI275517[] = {{&S21177[13], 3}, {&S21177[17], 13}, {&S21177[31], 6}, {&S21177[38], 6}, {&S21177[66], 17}, {NULL}};
const Segment SGI275518[] = {{&S21178[92], 4}, {NULL}};
const Segment SGI275519[] = {{&S21178[13], 3}, {&S21178[17], 13}, {&S21178[31], 6}, {&S21178[38], 6}, {&S21178[66], 17}, {NULL}};
const Segment SGI275520[] = {{&S21179[92], 4}, {NULL}};
const Segment SGI275521[] = {{&S21179[13], 3}, {&S21179[17], 13}, {&S21179[31], 6}, {&S21179[38], 6}, {&S21179[66], 17}, {NULL}};
const Segment SGI275522[] = {{&S21180[85], 4}, {NULL}};
const Segment SGI275523[] = {{&S21180[13], 3}, {&S21180[17], 13}, {&S21180[31], 6}, {&S21180[59], 17}, {NULL}};
const Segment SGI275524[] = {{&S21181[105], 4}, {NULL}};
const Segment SGI275525[] = {{&S21181[13], 5}, {&S21181[19], 1}, {&S21181[21], 12}, {&S21181[34], 5}, {&S21181[40], 17}, {&S21181[79], 17}, {NULL}};
const Segment SGI275526[] = {{&S21182[107], 4}, {NULL}};
const Segment SGI275527[] = {{&S21182[13], 5}, {&S21182[19], 1}, {&S21182[21], 12}, {&S21182[34], 5}, {&S21182[40], 19}, {&S21182[81], 17}, {NULL}};
const Segment SGI275528[] = {{&S21183[104], 4}, {NULL}};
const Segment SGI275529[] = {{&S21183[13], 5}, {&S21183[19], 1}, {&S21183[21], 12}, {&S21183[34], 5}, {&S21183[40], 16}, {&S21183[78], 17}, {NULL}};
const Segment SGI275530[] = {{&S21184[109], 4}, {NULL}};
const Segment SGI275531[] = {{&S21184[13], 3}, {&S21184[17], 13}, {&S21184[31], 6}, {&S21184[38], 23}, {&S21184[83], 17}, {NULL}};
const Segment SGI275532[] = {{&S21185[78], 4}, {NULL}};
const Segment SGI275533[] = {{&S21185[13], 3}, {&S21185[17], 13}, {&S21185[52], 17}, {&S21185[104], 8}, {NULL}};
const Segment SGI275534[] = {{&S21186[104], 4}, {NULL}};
const Segment SGI275535[] = {{&S21186[13], 5}, {&S21186[19], 1}, {&S21186[21], 12}, {&S21186[34], 5}, {&S21186[40], 16}, {&S21186[78], 17}, {NULL}};
const Segment SGI275536[] = {{&S21187[78], 4}, {NULL}};
const Segment SGI275537[] = {{&S21187[13], 3}, {&S21187[17], 13}, {&S21187[52], 17}, {NULL}};
const Segment SGI275538[] = {{&S21188[94], 4}, {NULL}};
const Segment SGI275539[] = {{&S21188[13], 3}, {&S21188[17], 13}, {&S21188[31], 6}, {&S21188[38], 8}, {&S21188[68], 17}, {NULL}};
const Segment SGI275540[] = {{&S21189[79], 4}, {NULL}};
const Segment SGI275541[] = {{&S21189[13], 3}, {&S21189[17], 2}, {&S21189[20], 11}, {&S21189[53], 17}, {NULL}};
const Segment SGI275542[] = {{&S21190[78], 4}, {NULL}};
const Segment SGI275543[] = {{&S21190[13], 3}, {&S21190[17], 13}, {&S21190[52], 17}, {NULL}};
const Segment SGI275544[] = {{&S21191[87], 4}, {NULL}};
const Segment SGI275545[] = {{&S21191[13], 3}, {&S21191[17], 13}, {&S21191[31], 8}, {&S21191[61], 17}, {NULL}};
const Segment SGI275546[] = {{&S21192[66], 4}, {NULL}};
const Segment SGI275547[] = {{&S21192[13], 3}, {&S21192[11], 1}, {&S21192[40], 17}, {NULL}};
const Segment SGI275548[] = {{&S21193[92], 4}, {NULL}};
const Segment SGI275549[] = {{&S21193[13], 3}, {&S21193[17], 13}, {&S21193[31], 6}, {&S21193[38], 6}, {&S21193[66], 17}, {NULL}};
const Segment SGI275550[] = {{&S21194[92], 4}, {NULL}};
const Segment SGI275551[] = {{&S21194[13], 3}, {&S21194[17], 13}, {&S21194[31], 6}, {&S21194[38], 6}, {&S21194[66], 17}, {NULL}};
const Segment SGI275552[] = {{&S21195[92], 4}, {NULL}};
const Segment SGI275553[] = {{&S21195[13], 3}, {&S21195[17], 13}, {&S21195[31], 6}, {&S21195[38], 6}, {&S21195[66], 17}, {NULL}};
const Segment SGI275554[] = {{&S21196[90], 4}, {NULL}};
const Segment SGI275555[] = {{&S21196[25], 3}, {&S21196[29], 13}, {&S21196[64], 17}, {NULL}};
const Segment SGI275556[] = {{&S21196[0], 12}, {NULL}};
const Segment SGI275557[] = {{&S21197[92], 4}, {NULL}};
const Segment SGI275558[] = {{&S21197[13], 3}, {&S21197[17], 13}, {&S21197[31], 6}, {&S21197[38], 6}, {&S21197[66], 17}, {NULL}};
const Segment SGI275559[] = {{&S21198[78], 4}, {NULL}};
const Segment SGI275560[] = {{&S21198[13], 3}, {&S21198[17], 13}, {&S21198[52], 17}, {&S21198[104], 41}, {NULL}};
const Segment SGI275561[] = {{&S21199[89], 4}, {NULL}};
const Segment SGI275562[] = {{&S21199[24], 3}, {&S21199[28], 13}, {&S21199[63], 17}, {NULL}};
const Segment SGI275563[] = {{&S21199[0], 11}, {NULL}};
const Segment SGI275564[] = {{&S21200[90], 4}, {NULL}};
const Segment SGI275565[] = {{&S21200[25], 3}, {&S21200[29], 13}, {&S21200[64], 17}, {NULL}};
const Segment SGI275566[] = {{&S21200[0], 12}, {NULL}};
const Segment SGI275567[] = {{&S21201[92], 4}, {NULL}};
const Segment SGI275568[] = {{&S21201[13], 3}, {&S21201[17], 13}, {&S21201[31], 6}, {&S21201[38], 6}, {&S21201[66], 17}, {NULL}};
const Segment SGI275569[] = {{&S21202[94], 4}, {NULL}};
const Segment SGI275570[] = {{&S21202[13], 1}, {&S21202[29], 3}, {&S21202[33], 13}, {&S21202[68], 17}, {NULL}};
const Segment SGI275571[] = {{&S21202[0], 16}, {NULL}};
const Segment SGI275572[] = {{&S21203[88], 4}, {NULL}};
const Segment SGI275573[] = {{&S21203[23], 3}, {&S21203[27], 13}, {&S21203[62], 17}, {NULL}};
const Segment SGI275574[] = {{&S21203[0], 10}, {NULL}};
const Segment SGI275575[] = {{&S21204[85], 4}, {NULL}};
const Segment SGI275576[] = {{&S21204[13], 3}, {&S21204[17], 13}, {&S21204[31], 6}, {&S21204[59], 17}, {NULL}};
const Segment SGI275577[] = {{&S21205[78], 4}, {NULL}};
const Segment SGI275578[] = {{&S21205[13], 3}, {&S21205[17], 13}, {&S21205[52], 17}, {&S21205[104], 8}, {&S21205[115], 26}, {NULL}};
const Segment SGI275579[] = {{&S21206[77], 4}, {NULL}};
const Segment SGI275580[] = {{&S21206[13], 3}, {&S21206[17], 12}, {&S21206[51], 17}, {NULL}};
const Segment SGI275581[] = {{&S21207[111], 4}, {NULL}};
const Segment SGI275582[] = {{&S21207[13], 3}, {&S21207[17], 13}, {&S21207[31], 25}, {&S21207[57], 6}, {&S21207[85], 17}, {NULL}};
const Segment SGI275583[] = {{&S21208[79], 4}, {NULL}};
const Segment SGI275584[] = {{&S21208[13], 4}, {&S21208[18], 13}, {&S21208[53], 17}, {NULL}};
const Segment SGI275585[] = {{&S21209[103], 4}, {NULL}};
const Segment SGI275586[] = {{&S21209[38], 3}, {&S21209[42], 13}, {&S21209[77], 17}, {NULL}};
const Segment SGI275587[] = {{&S21209[0], 25}, {NULL}};
const Segment SGI275588[] = {{&S21210[87], 4}, {NULL}};
const Segment SGI275589[] = {{&S21210[22], 3}, {&S21210[26], 13}, {&S21210[61], 17}, {NULL}};
const Segment SGI275590[] = {{&S21210[0], 9}, {NULL}};
const Segment SGI275591[] = {{&S21211[86], 4}, {NULL}};
const Segment SGI275592[] = {{&S21211[21], 3}, {&S21211[25], 13}, {&S21211[60], 17}, {NULL}};
const Segment SGI275593[] = {{&S21211[0], 8}, {NULL}};
const Segment SGI275594[] = {{&S21212[88], 4}, {NULL}};
const Segment SGI275595[] = {{&S21212[23], 3}, {&S21212[27], 13}, {&S21212[62], 17}, {NULL}};
const Segment SGI275596[] = {{&S21212[0], 10}, {NULL}};
const Segment SGI275597[] = {{&S21213[85], 4}, {NULL}};
const Segment SGI275598[] = {{&S21213[13], 3}, {&S21213[17], 13}, {&S21213[31], 6}, {&S21213[59], 17}, {NULL}};
const Segment SGI275599[] = {{&S21214[83], 4}, {NULL}};
const Segment SGI275600[] = {{&S21214[13], 3}, {&S21214[17], 13}, {&S21214[31], 4}, {&S21214[57], 17}, {NULL}};
const Segment SGI275601[] = {{&S21215[80], 4}, {NULL}};
const Segment SGI275602[] = {{&S21215[13], 3}, {&S21215[17], 10}, {&S21215[28], 4}, {&S21215[54], 17}, {NULL}};
const Segment SGI275603[] = {{&S21216[80], 4}, {NULL}};
const Segment SGI275604[] = {{&S21216[13], 3}, {&S21216[17], 10}, {&S21216[28], 4}, {&S21216[54], 17}, {NULL}};
const Segment SGI275605[] = {{&S21217[80], 4}, {NULL}};
const Segment SGI275606[] = {{&S21217[13], 3}, {&S21217[17], 10}, {&S21217[28], 4}, {&S21217[54], 17}, {NULL}};
const Segment SGI275607[] = {{&S21218[83], 4}, {NULL}};
const Segment SGI275608[] = {{&S21218[13], 3}, {&S21218[17], 13}, {&S21218[31], 4}, {&S21218[57], 17}, {NULL}};
const Segment SGI275609[] = {{&S21219[85], 4}, {NULL}};
const Segment SGI275610[] = {{&S21219[13], 3}, {&S21219[17], 13}, {&S21219[31], 6}, {&S21219[59], 17}, {NULL}};
const Segment SGI275611[] = {{&S21220[85], 4}, {NULL}};
const Segment SGI275612[] = {{&S21220[20], 3}, {&S21220[24], 13}, {&S21220[59], 17}, {NULL}};
const Segment SGI275613[] = {{&S21220[0], 7}, {NULL}};
const Segment SGI275614[] = {{&S21221[80], 4}, {NULL}};
const Segment SGI275615[] = {{&S21221[13], 3}, {&S21221[17], 10}, {&S21221[28], 4}, {&S21221[54], 17}, {NULL}};
const Segment SGI275616[] = {{&S21222[86], 4}, {NULL}};
const Segment SGI275617[] = {{&S21222[21], 3}, {&S21222[25], 13}, {&S21222[60], 17}, {NULL}};
const Segment SGI275618[] = {{&S21222[0], 8}, {NULL}};
const Segment SGI275619[] = {{&S21223[83], 4}, {NULL}};
const Segment SGI275620[] = {{&S21223[13], 3}, {&S21223[17], 13}, {&S21223[31], 4}, {&S21223[57], 17}, {NULL}};
const Segment SGI275621[] = {{&S21224[86], 4}, {NULL}};
const Segment SGI275622[] = {{&S21224[21], 3}, {&S21224[25], 13}, {&S21224[60], 17}, {NULL}};
const Segment SGI275623[] = {{&S21224[0], 8}, {NULL}};
const Segment SGI275624[] = {{&S21225[85], 4}, {NULL}};
const Segment SGI275625[] = {{&S21225[13], 3}, {&S21225[17], 13}, {&S21225[31], 6}, {&S21225[59], 17}, {NULL}};
const Segment SGI275626[] = {{&S21226[92], 4}, {NULL}};
const Segment SGI275627[] = {{&S21226[13], 3}, {&S21226[17], 13}, {&S21226[31], 6}, {&S21226[38], 6}, {&S21226[66], 17}, {NULL}};
const Segment SGI275628[] = {{&S21227[85], 4}, {NULL}};
const Segment SGI275629[] = {{&S21227[13], 3}, {&S21227[17], 13}, {&S21227[31], 6}, {&S21227[59], 17}, {NULL}};
const Segment SGI275630[] = {{&S21228[108], 4}, {NULL}};
const Segment SGI275631[] = {{&S21228[13], 5}, {&S21228[19], 2}, {&S21228[22], 13}, {&S21228[36], 6}, {&S21228[43], 17}, {&S21228[82], 17}, {NULL}};
const Segment SGI275632[] = {{&S21229[85], 4}, {NULL}};
const Segment SGI275633[] = {{&S21229[13], 3}, {&S21229[17], 13}, {&S21229[31], 6}, {&S21229[59], 17}, {NULL}};
const Segment SGI275634[] = {{&S21230[66], 4}, {NULL}};
const Segment SGI275635[] = {{&S21230[13], 3}, {&S21230[11], 1}, {&S21230[40], 17}, {&S21230[92], 8}, {NULL}};
const Segment SGI275636[] = {{&S21231[85], 4}, {NULL}};
const Segment SGI275637[] = {{&S21231[20], 3}, {&S21231[24], 13}, {&S21231[59], 17}, {NULL}};
const Segment SGI275638[] = {{&S21231[0], 7}, {NULL}};
const Segment SGI275639[] = {{&S21232[86], 4}, {NULL}};
const Segment SGI275640[] = {{&S21232[21], 3}, {&S21232[25], 13}, {&S21232[60], 17}, {NULL}};
const Segment SGI275641[] = {{&S21232[0], 8}, {NULL}};
const Segment SGI275642[] = {{&S21233[92], 4}, {NULL}};
const Segment SGI275643[] = {{&S21233[13], 3}, {&S21233[17], 13}, {&S21233[31], 6}, {&S21233[38], 6}, {&S21233[66], 17}, {NULL}};
const Segment SGI275644[] = {{&S21234[95], 4}, {NULL}};
const Segment SGI275645[] = {{&S21234[30], 3}, {&S21234[34], 13}, {&S21234[69], 17}, {NULL}};
const Segment SGI275646[] = {{&S21234[0], 17}, {NULL}};
const Segment SGI275647[] = {{&S21235[86], 4}, {NULL}};
const Segment SGI275648[] = {{&S21235[21], 3}, {&S21235[25], 13}, {&S21235[60], 17}, {NULL}};
const Segment SGI275649[] = {{&S21235[0], 8}, {NULL}};
const Segment SGI275650[] = {{&S21236[90], 4}, {NULL}};
const Segment SGI275651[] = {{&S21236[25], 3}, {&S21236[29], 13}, {&S21236[64], 17}, {NULL}};
const Segment SGI275652[] = {{&S21236[0], 12}, {NULL}};
const Segment SGI275653[] = {{&S21237[83], 4}, {NULL}};
const Segment SGI275654[] = {{&S21237[18], 3}, {&S21237[22], 13}, {&S21237[57], 17}, {NULL}};
const Segment SGI275655[] = {{&S21237[0], 5}, {NULL}};
const Segment SGI275656[] = {{&S21238[86], 4}, {NULL}};
const Segment SGI275657[] = {{&S21238[21], 3}, {&S21238[25], 13}, {&S21238[60], 17}, {NULL}};
const Segment SGI275658[] = {{&S21238[0], 8}, {NULL}};
const Segment SGI275659[] = {{&S21239[78], 4}, {NULL}};
const Segment SGI275660[] = {{&S21239[13], 3}, {&S21239[17], 13}, {&S21239[52], 17}, {&S21239[104], 8}, {&S21239[104], 8}, {NULL}};
const Segment SGI275661[] = {{&S21240[83], 4}, {NULL}};
const Segment SGI275662[] = {{&S21240[18], 3}, {&S21240[22], 13}, {&S21240[57], 17}, {NULL}};
const Segment SGI275663[] = {{&S21240[0], 5}, {NULL}};
const Segment SGI275664[] = {{&S21241[84], 4}, {NULL}};
const Segment SGI275665[] = {{&S21241[19], 3}, {&S21241[23], 13}, {&S21241[58], 17}, {NULL}};
const Segment SGI275666[] = {{&S21241[0], 6}, {NULL}};
const Segment SGI275667[] = {{&S21242[86], 4}, {NULL}};
const Segment SGI275668[] = {{&S21242[13], 3}, {&S21242[17], 13}, {&S21242[31], 6}, {&S21242[60], 17}, {NULL}};
const Segment SGI275669[] = {{&S21243[89], 4}, {NULL}};
const Segment SGI275670[] = {{&S21243[24], 3}, {&S21243[28], 13}, {&S21243[63], 17}, {NULL}};
const Segment SGI275671[] = {{&S21243[0], 11}, {NULL}};
const Segment SGI275672[] = {{&S21244[86], 4}, {NULL}};
const Segment SGI275673[] = {{&S21244[21], 3}, {&S21244[25], 13}, {&S21244[60], 17}, {NULL}};
const Segment SGI275674[] = {{&S21244[0], 8}, {NULL}};
const Segment SGI275675[] = {{&S21245[85], 4}, {NULL}};
const Segment SGI275676[] = {{&S21245[13], 3}, {&S21245[17], 13}, {&S21245[31], 6}, {&S21245[59], 17}, {NULL}};
const Segment SGI275677[] = {{&S21639[93], 4}, {NULL}};
const Segment SGI275678[] = {{&S21639[13], 7}, {&S21639[21], 2}, {&S21639[24], 15}, {&S21639[40], 6}, {&S21639[67], 17}, {NULL}};
const Segment SGI275679[] = {{&S21640[74], 4}, {NULL}};
const Segment SGI275680[] = {{&S21640[13], 14}, {&S21640[48], 17}, {NULL}};
const Segment SGI275681[] = {{&S21641[74], 4}, {NULL}};
const Segment SGI275682[] = {{&S21641[13], 14}, {&S21641[48], 17}, {&S21641[100], 8}, {&S21641[111], 24}, {NULL}};
const Segment SGI275683[] = {{&S21642[74], 4}, {NULL}};
const Segment SGI275684[] = {{&S21642[13], 14}, {&S21642[48], 17}, {NULL}};
const Segment SGI275685[] = {{&S21643[81], 4}, {NULL}};
const Segment SGI275686[] = {{&S21643[13], 14}, {&S21643[28], 6}, {&S21643[55], 17}, {NULL}};
const Segment SGI275687[] = {{&S21644[81], 4}, {NULL}};
const Segment SGI275688[] = {{&S21644[13], 14}, {&S21644[28], 6}, {&S21644[55], 17}, {NULL}};
const Segment SGI275689[] = {{&S21645[13], 7}, {&S21645[21], 2}, {&S21645[24], 15}, {&S21645[40], 6}, {&S21645[68], 17}, {NULL}};
const Segment SGI275690[] = {{&S21646[74], 4}, {NULL}};
const Segment SGI275691[] = {{&S21646[13], 14}, {&S21646[48], 17}, {&S21646[100], 12}, {NULL}};
const Segment SGI275692[] = {{&S21647[74], 4}, {NULL}};
const Segment SGI275693[] = {{&S21647[13], 14}, {&S21647[48], 17}, {&S21647[100], 8}, {&S21647[111], 26}, {NULL}};
const Segment SGI275694[] = {{&S21648[74], 4}, {NULL}};
const Segment SGI275695[] = {{&S21648[13], 14}, {&S21648[48], 17}, {&S21648[100], 8}, {NULL}};
const Segment SGI275696[] = {{&S21649[80], 4}, {NULL}};
const Segment SGI275697[] = {{&S21649[13], 14}, {&S21649[28], 6}, {&S21649[55], 16}, {NULL}};
const Segment SGI275698[] = {{&S17965[85], 4}, {NULL}};
const Segment SGI275699[] = {{&S17965[13], 3}, {&S17965[17], 19}, {&S17965[59], 17}, {NULL}};
const Segment SGI275700[] = {{&S17966[76], 4}, {NULL}};
const Segment SGI275701[] = {{&S17966[13], 3}, {&S17966[17], 11}, {&S17966[50], 17}, {NULL}};
const Segment SGI275702[] = {{&S17967[78], 4}, {NULL}};
const Segment SGI275703[] = {{&S17967[13], 3}, {&S17967[17], 13}, {&S17967[52], 17}, {&S17967[105], 8}, {NULL}};
const Segment SGI275704[] = {{&S17968[78], 4}, {NULL}};
const Segment SGI275705[] = {{&S17968[13], 3}, {&S17968[17], 13}, {&S17968[52], 17}, {NULL}};
const Segment SGI275706[] = {{&S17969[76], 4}, {NULL}};
const Segment SGI275707[] = {{&S17969[13], 3}, {&S17969[17], 11}, {&S17969[50], 17}, {&S17969[103], 8}, {NULL}};
const Segment SGI275708[] = {{&S17970[80], 4}, {NULL}};
const Segment SGI275709[] = {{&S17970[13], 3}, {&S17970[17], 2}, {&S17970[20], 5}, {&S17970[26], 6}, {&S17970[54], 17}, {NULL}};
const Segment SGI275710[] = {{&S17971[80], 4}, {NULL}};
const Segment SGI275711[] = {{&S17971[13], 3}, {&S17971[17], 10}, {&S17971[28], 4}, {&S17971[54], 17}, {NULL}};
const Segment SGI275712[] = {{&S17972[83], 4}, {NULL}};
const Segment SGI275713[] = {{&S17972[13], 3}, {&S17972[17], 13}, {&S17972[31], 4}, {&S17972[57], 17}, {NULL}};
const Segment SGI275714[] = {{&S17973[80], 4}, {NULL}};
const Segment SGI275715[] = {{&S17973[13], 3}, {&S17973[17], 10}, {&S17973[28], 4}, {&S17973[54], 17}, {NULL}};
const Segment SGI275716[] = {{&S17974[80], 4}, {NULL}};
const Segment SGI275717[] = {{&S17974[13], 3}, {&S17974[17], 10}, {&S17974[28], 4}, {&S17974[54], 17}, {NULL}};
const Segment SGI275718[] = {{&S17975[83], 4}, {NULL}};
const Segment SGI275719[] = {{&S17975[13], 3}, {&S17975[17], 13}, {&S17975[31], 4}, {&S17975[57], 17}, {NULL}};
const Segment SGI275720[] = {{&S17976[83], 4}, {NULL}};
const Segment SGI275721[] = {{&S17976[13], 3}, {&S17976[17], 13}, {&S17976[31], 4}, {&S17976[57], 17}, {NULL}};
const Segment SGI275722[] = {{&S17977[80], 4}, {NULL}};
const Segment SGI275723[] = {{&S17977[13], 3}, {&S17977[17], 10}, {&S17977[28], 4}, {&S17977[54], 17}, {NULL}};
const Segment SGI275724[] = {{&S18043[88], 4}, {NULL}};
const Segment SGI275725[] = {{&S18043[69], 44}, {NULL}};
const Segment SGI275726[] = {{&S18044[93], 4}, {NULL}};
const Segment SGI275727[] = {{&S18044[13], 9}, {&S18044[23], 22}, {&S18044[67], 17}, {NULL}};
const Segment SGI275728[] = {{&S18045[93], 4}, {NULL}};
const Segment SGI275729[] = {{&S18045[13], 9}, {&S18045[23], 22}, {&S18045[67], 17}, {NULL}};
const Segment SGI275730[] = {{&S18046[93], 4}, {NULL}};
const Segment SGI275731[] = {{&S25515[94], 3}, {NULL}};
const Segment SGI275732[] = {{&S25515[13], 7}, {&S25515[21], 2}, {&S25515[24], 15}, {&S25515[40], 6}, {&S25515[68], 17}, {NULL}};
const Segment SGI275733[] = {{&S25516[94], 3}, {NULL}};
const Segment SGI275734[] = {{&S25516[13], 7}, {&S25516[21], 2}, {&S25516[24], 15}, {&S25516[40], 6}, {&S25516[68], 17}, {NULL}};
const Segment SGI275735[] = {{&S25517[94], 3}, {NULL}};
const Segment SGI275736[] = {{&S25517[13], 7}, {&S25517[21], 2}, {&S25517[24], 15}, {&S25517[40], 6}, {&S25517[68], 17}, {&S25517[120], 8}, {&S25517[131], 24}, {NULL}};
const Segment SGI275737[] = {{&S25518[94], 3}, {NULL}};
const Segment SGI275738[] = {{&S25518[13], 7}, {&S25518[21], 2}, {&S25518[24], 15}, {&S25518[40], 6}, {&S25518[68], 17}, {&S25518[120], 8}, {&S25518[131], 26}, {NULL}};
const Segment SGI275739[] = {{&S25519[91], 3}, {NULL}};
const Segment SGI275740[] = {{&S25519[13], 7}, {&S25519[21], 2}, {&S25519[24], 15}, {&S25519[40], 3}, {&S25519[65], 17}, {NULL}};
const Segment SGI275741[] = {{&S25520[94], 3}, {NULL}};
const Segment SGI275742[] = {{&S25520[13], 7}, {&S25520[21], 2}, {&S25520[24], 15}, {&S25520[40], 6}, {&S25520[68], 17}, {&S25520[120], 8}, {NULL}};
const Segment SGI275743[] = {{&S25521[91], 3}, {NULL}};
const Segment SGI275744[] = {{&S25521[13], 7}, {&S25521[21], 2}, {&S25521[24], 15}, {&S25521[40], 3}, {&S25521[65], 17}, {NULL}};
const Segment SGI275745[] = {{&S25522[94], 3}, {NULL}};
const Segment SGI275746[] = {{&S25522[13], 7}, {&S25522[21], 2}, {&S25522[24], 15}, {&S25522[40], 6}, {&S25522[68], 17}, {NULL}};
const Segment SGI275747[] = {{&S25523[94], 3}, {NULL}};
const Segment SGI275748[] = {{&S25523[13], 7}, {&S25523[21], 2}, {&S25523[24], 15}, {&S25523[40], 6}, {&S25523[68], 17}, {NULL}};
const Segment SGI275749[] = {{&S25524[94], 3}, {NULL}};
const Segment SGI275750[] = {{&S25524[13], 7}, {&S25524[21], 2}, {&S25524[24], 15}, {&S25524[40], 6}, {&S25524[68], 17}, {NULL}};
const Segment SGI275751[] = {{&S25525[94], 3}, {NULL}};
const Segment SGI275752[] = {{&S25525[13], 7}, {&S25525[21], 2}, {&S25525[24], 15}, {&S25525[40], 7}, {&S25525[69], 16}, {NULL}};
const Segment SGI275753[] = {{&S25526[93], 3}, {NULL}};
const Segment SGI275754[] = {{&S25526[13], 7}, {&S25526[21], 2}, {&S25526[24], 15}, {&S25526[40], 6}, {&S25526[68], 16}, {NULL}};
const Segment SGI275755[] = {{&S25527[93], 3}, {NULL}};
const Segment SGI275756[] = {{&S25527[13], 7}, {&S25527[21], 2}, {&S25527[24], 15}, {&S25527[40], 6}, {&S25527[68], 16}, {NULL}};
const Segment SGI275757[] = {{&S25528[93], 3}, {NULL}};
const Segment SGI275758[] = {{&S25528[13], 7}, {&S25528[21], 2}, {&S25528[24], 15}, {&S25528[40], 6}, {&S25528[68], 16}, {NULL}};
const Segment SGI275759[] = {{&S25529[93], 3}, {NULL}};
const Segment SGI275760[] = {{&S25529[13], 7}, {&S25529[21], 2}, {&S25529[24], 15}, {&S25529[40], 6}, {&S25529[68], 16}, {NULL}};
const Segment SGI275761[] = {{&S25530[93], 3}, {NULL}};
const Segment SGI275762[] = {{&S25530[13], 7}, {&S25530[21], 2}, {&S25530[24], 15}, {&S25530[40], 6}, {&S25530[68], 16}, {NULL}};
const Segment SGI275763[] = {{&S25531[93], 3}, {NULL}};
const Segment SGI275764[] = {{&S25531[13], 7}, {&S25531[21], 2}, {&S25531[24], 15}, {&S25531[40], 6}, {&S25531[68], 16}, {NULL}};
const Segment SGI275765[] = {{&S25532[93], 3}, {NULL}};
const Segment SGI275766[] = {{&S25532[13], 7}, {&S25532[21], 2}, {&S25532[24], 15}, {&S25532[40], 6}, {&S25532[68], 16}, {NULL}};
const Segment SGI275767[] = {{&S25533[93], 3}, {NULL}};
const Segment SGI275768[] = {{&S25533[13], 7}, {&S25533[21], 2}, {&S25533[24], 15}, {&S25533[40], 6}, {&S25533[68], 16}, {NULL}};
const Segment SGI275769[] = {{&S25534[93], 3}, {NULL}};
const Segment SGI275770[] = {{&S25534[13], 7}, {&S25534[21], 2}, {&S25534[24], 15}, {&S25534[40], 6}, {&S25534[68], 16}, {NULL}};
const Segment SGI275771[] = {{&S18046[13], 9}, {&S18046[23], 22}, {&S18046[67], 17}, {NULL}};
const Segment SGI275772[] = {{&S18047[93], 4}, {NULL}};
const Segment SGI275773[] = {{&S18047[13], 9}, {&S18047[23], 22}, {&S18047[67], 17}, {NULL}};
const Segment SGI275774[] = {{&S18048[93], 4}, {NULL}};
const Segment SGI275775[] = {{&S18048[13], 9}, {&S18048[23], 22}, {&S18048[67], 17}, {NULL}};
const Segment SGI275776[] = {{&S18049[93], 4}, {NULL}};
const Segment SGI275777[] = {{&S18049[13], 9}, {&S18049[23], 22}, {&S18049[67], 17}, {NULL}};
const Segment SGI275778[] = {{&S18050[93], 4}, {NULL}};
const Segment SGI275779[] = {{&S18050[13], 9}, {&S18050[23], 22}, {&S18050[67], 17}, {NULL}};
const Segment SGI275780[] = {{&S18051[93], 4}, {NULL}};
const Segment SGI275781[] = {{&S18051[13], 9}, {&S18051[23], 22}, {&S18051[67], 17}, {NULL}};
const Segment SGI275782[] = {{&S18052[93], 4}, {NULL}};
const Segment SGI275783[] = {{&S18052[13], 9}, {&S18052[23], 22}, {&S18052[67], 17}, {NULL}};
const Segment SGI275784[] = {{&S18053[93], 4}, {NULL}};
const Segment SGI275785[] = {{&S18053[13], 9}, {&S18053[23], 22}, {&S18053[67], 17}, {&S18053[120], 8}, {&S18053[131], 26}, {NULL}};
const Segment SGI275786[] = {{&S18054[93], 4}, {NULL}};
const Segment SGI275787[] = {{&S18054[13], 9}, {&S18054[23], 22}, {&S18054[67], 17}, {&S18054[120], 8}, {NULL}};
const Segment SGI275788[] = {{&S18055[103], 4}, {NULL}};
const Segment SGI275789[] = {{&S18055[13], 9}, {&S18055[23], 2}, {&S18055[26], 22}, {&S18055[49], 6}, {&S18055[77], 17}, {NULL}};
const Segment SGI275790[] = {{&S18056[103], 4}, {NULL}};
const Segment SGI275791[] = {{&S18056[13], 9}, {&S18056[23], 2}, {&S18056[26], 22}, {&S18056[49], 6}, {&S18056[77], 17}, {NULL}};
const Segment SGI275792[] = {{&S18057[103], 4}, {NULL}};
const Segment SGI275793[] = {{&S18057[13], 9}, {&S18057[23], 2}, {&S18057[26], 22}, {&S18057[49], 6}, {&S18057[77], 17}, {NULL}};
const Segment SGI275794[] = {{&S18058[93], 4}, {NULL}};
const Segment SGI275795[] = {{&S18058[13], 9}, {&S18058[23], 22}, {&S18058[67], 17}, {NULL}};
const Segment SGI275796[] = {{&S18059[93], 4}, {NULL}};
const Segment SGI275797[] = {{&S18059[13], 9}, {&S18059[23], 22}, {&S18059[67], 17}, {NULL}};
const Segment SGI275798[] = {{&S18060[93], 4}, {NULL}};
const Segment SGI275799[] = {{&S18060[13], 9}, {&S18060[23], 22}, {&S18060[67], 17}, {NULL}};
const Segment SGI275800[] = {{&S18061[93], 4}, {NULL}};
const Segment SGI275801[] = {{&S18061[13], 9}, {&S18061[23], 22}, {&S18061[67], 17}, {NULL}};
const Segment SGI275802[] = {{&S18062[93], 4}, {NULL}};
const Segment SGI275803[] = {{&S18062[13], 9}, {&S18062[23], 22}, {&S18062[67], 17}, {NULL}};
const Segment SGI275804[] = {{&S18063[93], 4}, {NULL}};
const Segment SGI275805[] = {{&S18063[13], 9}, {&S18063[23], 22}, {&S18063[67], 17}, {NULL}};
const Segment SGI275806[] = {{&S18064[93], 4}, {NULL}};
const Segment SGI275807[] = {{&S18064[13], 9}, {&S18064[23], 22}, {&S18064[67], 17}, {&S18064[120], 8}, {&S18064[131], 26}, {NULL}};
const Segment SGI275808[] = {{&S18065[93], 4}, {NULL}};
const Segment SGI275809[] = {{&S18065[13], 9}, {&S18065[23], 22}, {&S18065[67], 17}, {&S18065[120], 8}, {NULL}};
const Segment SGI275810[] = {{&S25652[93], 3}, {NULL}};
const Segment SGI275811[] = {{&S25652[13], 7}, {&S25652[21], 2}, {&S25652[24], 15}, {&S25652[40], 6}, {&S25652[67], 17}, {NULL}};
const Segment SGI275812[] = {{&S25653[75], 3}, {NULL}};
const Segment SGI275813[] = {{&S25653[13], 14}, {&S25653[49], 17}, {NULL}};
const Segment SGI275814[] = {{&S25654[93], 3}, {NULL}};
const Segment SGI275815[] = {{&S25654[13], 7}, {&S25654[21], 2}, {&S25654[24], 15}, {&S25654[40], 6}, {&S25654[67], 17}, {NULL}};
const Segment SGI275816[] = {{&S25655[93], 3}, {NULL}};
const Segment SGI275817[] = {{&S25655[13], 7}, {&S25655[21], 2}, {&S25655[24], 15}, {&S25655[40], 6}, {&S25655[67], 17}, {&S25655[117], 8}, {&S25655[128], 24}, {NULL}};
const Segment SGI275818[] = {{&S25656[93], 3}, {&S25656[93], 3}, {NULL}};
const Segment SGI275819[] = {{&S25656[13], 7}, {&S25656[21], 2}, {&S25656[24], 15}, {&S25656[40], 6}, {&S25656[67], 17}, {NULL}};
const Segment SGI275820[] = {{&S25657[93], 3}, {NULL}};
const Segment SGI275821[] = {{&S25657[13], 7}, {&S25657[21], 2}, {&S25657[24], 15}, {&S25657[40], 6}, {&S25657[67], 17}, {NULL}};
const Segment SGI275822[] = {{&S25658[94], 3}, {NULL}};
const Segment SGI275823[] = {{&S25658[13], 7}, {&S25658[21], 2}, {&S25658[24], 15}, {&S25658[40], 6}, {&S25658[68], 17}, {&S25658[118], 31}, {NULL}};
const Segment SGI275824[] = {{&S25659[94], 3}, {NULL}};
const Segment SGI275825[] = {{&S25659[13], 7}, {&S25659[21], 2}, {&S25659[24], 15}, {&S25659[40], 6}, {&S25659[68], 17}, {&S25659[118], 31}, {NULL}};
const Segment SGI275826[] = {{&S25660[93], 3}, {NULL}};
const Segment SGI275827[] = {{&S25660[13], 7}, {&S25660[21], 2}, {&S25660[24], 15}, {&S25660[40], 6}, {&S25660[67], 17}, {&S25660[117], 8}, {NULL}};
const Segment SGI275828[] = {{&S25661[92], 3}, {NULL}};
const Segment SGI275829[] = {{&S25661[13], 7}, {&S25661[21], 2}, {&S25661[24], 15}, {&S25661[40], 6}, {&S25661[67], 16}, {NULL}};
const Segment SGI275830[] = {{&S25662[92], 3}, {NULL}};
const Segment SGI275831[] = {{&S25662[13], 7}, {&S25662[21], 2}, {&S25662[24], 15}, {&S25662[40], 6}, {&S25662[67], 16}, {NULL}};
const Segment SGI275832[] = {{&S25663[92], 3}, {NULL}};
const Segment SGI275833[] = {{&S25663[13], 7}, {&S25663[21], 2}, {&S25663[24], 15}, {&S25663[40], 6}, {&S25663[67], 16}, {NULL}};
const Segment SGI275834[] = {{&S25664[92], 3}, {NULL}};
const Segment SGI275835[] = {{&S25664[13], 7}, {&S25664[21], 2}, {&S25664[24], 15}, {&S25664[40], 6}, {&S25664[67], 16}, {NULL}};
const Segment SGI275836[] = {{&S25665[92], 3}, {NULL}};
const Segment SGI275837[] = {{&S25665[13], 7}, {&S25665[21], 2}, {&S25665[24], 15}, {&S25665[40], 6}, {&S25665[67], 16}, {NULL}};
const Segment SGI275838[] = {{&S25666[93], 3}, {NULL}};
const Segment SGI275839[] = {{&S25666[13], 7}, {&S25666[21], 2}, {&S25666[24], 15}, {&S25666[40], 7}, {&S25666[68], 16}, {NULL}};
const Segment SGI275840[] = {{&S25667[92], 3}, {NULL}};
const Segment SGI275841[] = {{&S25667[13], 7}, {&S25667[21], 2}, {&S25667[24], 15}, {&S25667[40], 6}, {&S25667[67], 16}, {NULL}};
const Segment SGI275842[] = {{&S25668[92], 3}, {NULL}};
const Segment SGI275843[] = {{&S25668[13], 7}, {&S25668[21], 2}, {&S25668[24], 15}, {&S25668[40], 6}, {&S25668[67], 16}, {NULL}};
const Segment SGI275844[] = {{&S18171[75], 4}, {NULL}};
const Segment SGI275845[] = {{&S18171[13], 14}, {&S18171[49], 17}, {&S18171[102], 8}, {NULL}};
const Segment SGI275846[] = {{&S25669[94], 3}, {NULL}};
const Segment SGI275847[] = {{&S25669[13], 7}, {&S25669[21], 2}, {&S25669[24], 15}, {&S25669[40], 6}, {&S25669[47], 18}, {&S25669[66], 19}, {&S25669[86], 56}, {NULL}};
const Segment SGI275848[] = {{&S18172[75], 4}, {NULL}};
const Segment SGI275849[] = {{&S18172[13], 14}, {&S18172[49], 17}, {&S18172[102], 8}, {&S18172[113], 26}, {NULL}};
const Segment SGI275850[] = {{&S18173[82], 4}, {NULL}};
const Segment SGI275851[] = {{&S18173[13], 14}, {&S18173[28], 6}, {&S18173[56], 17}, {&S18173[109], 8}, {NULL}};
const Segment SGI275852[] = {{&S18174[75], 4}, {NULL}};
const Segment SGI275853[] = {{&S18174[13], 14}, {&S18174[49], 17}, {NULL}};
const Segment SGI275854[] = {{&S18175[82], 4}, {NULL}};
const Segment SGI275855[] = {{&S18175[13], 14}, {&S18175[28], 6}, {&S18175[56], 17}, {NULL}};
const Segment SGI275856[] = {{&S18176[75], 4}, {NULL}};
const Segment SGI275857[] = {{&S18176[13], 14}, {&S18176[49], 17}, {NULL}};
const Segment SGI275858[] = {{&S18177[82], 4}, {NULL}};
const Segment SGI275859[] = {{&S18177[13], 14}, {&S18177[28], 6}, {&S18177[56], 17}, {NULL}};
const Segment SGI275860[] = {{&S18178[82], 4}, {NULL}};
const Segment SGI275861[] = {{&S18178[13], 14}, {&S18178[28], 6}, {&S18178[56], 17}, {&S18178[109], 8}, {&S18178[120], 26}, {NULL}};
const Segment SGI275862[] = {{&S18179[82], 4}, {NULL}};
const Segment SGI275863[] = {{&S18179[13], 14}, {&S18179[28], 6}, {&S18179[56], 17}, {NULL}};
const Segment SGI275864[] = {{&S18180[75], 4}, {NULL}};
const Segment SGI275865[] = {{&S18180[13], 14}, {&S18180[49], 17}, {NULL}};
const Segment SGI275866[] = {{&S18192[76], 4}, {NULL}};
const Segment SGI275867[] = {{&S18192[13], 3}, {&S18192[17], 11}, {&S18192[50], 17}, {NULL}};
const Segment SGI275868[] = {{&S18193[78], 4}, {NULL}};
const Segment SGI275869[] = {{&S18193[13], 3}, {&S18193[17], 13}, {&S18193[52], 17}, {NULL}};
const Segment SGI275870[] = {{&S18194[85], 4}, {NULL}};
const Segment SGI275871[] = {{&S18194[13], 3}, {&S18194[17], 19}, {&S18194[59], 17}, {NULL}};
const Segment SGI275872[] = {{&S18195[78], 4}, {NULL}};
const Segment SGI275873[] = {{&S18195[13], 3}, {&S18195[17], 13}, {&S18195[52], 17}, {&S18195[105], 8}, {NULL}};
const Segment SGI275874[] = {{&S18196[76], 4}, {NULL}};
const Segment SGI275875[] = {{&S18196[13], 3}, {&S18196[17], 11}, {&S18196[50], 17}, {&S18196[103], 8}, {NULL}};
const Segment SGI275876[] = {{&S18197[78], 4}, {NULL}};
const Segment SGI275877[] = {{&S18197[13], 3}, {&S18197[17], 13}, {&S18197[52], 17}, {&S18197[105], 8}, {&S18197[116], 26}, {NULL}};
const Segment SGI275878[] = {{&S18217[103], 4}, {NULL}};
const Segment SGI275879[] = {{&S18217[13], 9}, {&S18217[23], 2}, {&S18217[26], 22}, {&S18217[49], 6}, {&S18217[77], 17}, {NULL}};
const Segment SGI275880[] = {{&S18218[103], 4}, {NULL}};
const Segment SGI275881[] = {{&S18218[13], 9}, {&S18218[23], 2}, {&S18218[26], 22}, {&S18218[49], 6}, {&S18218[77], 17}, {NULL}};
const Segment SGI275882[] = {{&S18219[93], 4}, {NULL}};
const Segment SGI275883[] = {{&S18219[13], 9}, {&S18219[23], 22}, {&S18219[67], 17}, {&S18219[120], 8}, {NULL}};
const Segment SGI275884[] = {{&S18220[93], 4}, {NULL}};
const Segment SGI275885[] = {{&S18220[13], 9}, {&S18220[23], 22}, {&S18220[67], 17}, {&S18220[120], 8}, {&S18220[131], 26}, {NULL}};
const Segment SGI275886[] = {{&S18221[93], 4}, {NULL}};
const Segment SGI275887[] = {{&S18221[13], 9}, {&S18221[23], 22}, {&S18221[67], 17}, {NULL}};
const Segment SGI275888[] = {{&S18222[93], 4}, {NULL}};
const Segment SGI275889[] = {{&S18222[13], 9}, {&S18222[23], 22}, {&S18222[67], 17}, {NULL}};
const Segment SGI275890[] = {{&S18223[93], 4}, {NULL}};
const Segment SGI275891[] = {{&S18223[13], 9}, {&S18223[23], 22}, {&S18223[67], 17}, {NULL}};
const Segment SGI275892[] = {{&S18224[93], 4}, {NULL}};
const Segment SGI275893[] = {{&S18224[13], 9}, {&S18224[23], 22}, {&S18224[67], 17}, {NULL}};
const Segment SGI275894[] = {{&S18225[93], 4}, {NULL}};
const Segment SGI275895[] = {{&S18225[13], 9}, {&S18225[23], 22}, {&S18225[67], 17}, {NULL}};
const Segment SGI275896[] = {{&S18226[93], 4}, {NULL}};
const Segment SGI275897[] = {{&S18226[13], 9}, {&S18226[23], 22}, {&S18226[67], 17}, {NULL}};
const Segment SGI275898[] = {{&S18227[93], 4}, {NULL}};
const Segment SGI275899[] = {{&S18227[13], 9}, {&S18227[23], 22}, {&S18227[67], 17}, {NULL}};
const Segment SGI275900[] = {{&S18228[93], 4}, {NULL}};
const Segment SGI275901[] = {{&S18228[13], 9}, {&S18228[23], 22}, {&S18228[67], 17}, {NULL}};
const Segment SGI275902[] = {{&S18229[93], 4}, {NULL}};
const Segment SGI275903[] = {{&S18229[13], 9}, {&S18229[23], 22}, {&S18229[67], 17}, {NULL}};
const Segment SGI275904[] = {{&S18230[103], 4}, {NULL}};
const Segment SGI275905[] = {{&S18230[13], 9}, {&S18230[23], 2}, {&S18230[26], 22}, {&S18230[49], 6}, {&S18230[77], 17}, {NULL}};
const Segment SGI275906[] = {{&S18253[77], 4}, {NULL}};
const Segment SGI275907[] = {{&S18253[13], 3}, {&S18253[17], 13}, {&S18253[51], 17}, {NULL}};
const Segment SGI275908[] = {{&S18254[75], 4}, {NULL}};
const Segment SGI275909[] = {{&S18254[13], 3}, {&S18254[17], 11}, {&S18254[49], 17}, {NULL}};
const Segment SGI275910[] = {{&S18255[97], 4}, {NULL}};
const Segment SGI275911[] = {{&S18255[13], 3}, {&S18255[17], 13}, {&S18255[51], 17}, {&S18255[69], 19}, {NULL}};
const Segment SGI275912[] = {{&S18256[87], 4}, {NULL}};
const Segment SGI275913[] = {{&S18256[13], 3}, {&S18256[17], 2}, {&S18256[20], 13}, {&S18256[34], 6}, {&S18256[61], 17}, {NULL}};
const Segment SGI275914[] = {{&S18257[84], 4}, {NULL}};
const Segment SGI275915[] = {{&S18257[13], 3}, {&S18257[17], 19}, {&S18257[58], 17}, {NULL}};
const Segment SGI275916[] = {{&S18258[105], 4}, {NULL}};
const Segment SGI275917[] = {{&S18258[13], 3}, {&S18258[17], 13}, {&S18258[51], 17}, {&S18258[69], 27}, {NULL}};
const Segment SGI275918[] = {{&S18259[97], 4}, {NULL}};
const Segment SGI275919[] = {{&S18259[13], 3}, {&S18259[17], 13}, {&S18259[51], 17}, {&S18259[69], 19}, {&S18259[120], 8}, {&S18259[131], 26}, {NULL}};
const Segment SGI275920[] = {{&S18260[75], 4}, {NULL}};
const Segment SGI275921[] = {{&S18260[13], 3}, {&S18260[17], 11}, {&S18260[49], 17}, {&S18260[101], 8}, {NULL}};
const Segment SGI275922[] = {{&S18261[75], 4}, {NULL}};
const Segment SGI275923[] = {{&S18261[13], 3}, {&S18261[17], 11}, {&S18261[49], 17}, {&S18261[101], 8}, {&S18261[112], 24}, {NULL}};
const Segment SGI275924[] = {{&S18262[77], 4}, {NULL}};
const Segment SGI275925[] = {{&S18262[13], 3}, {&S18262[17], 13}, {&S18262[51], 17}, {&S18262[103], 8}, {NULL}};
const Segment SGI275926[] = {{&S18263[76], 4}, {NULL}};
const Segment SGI275927[] = {{&S18263[13], 3}, {&S18263[17], 13}, {&S18263[51], 16}, {NULL}};
const Segment SGI275928[] = {{&S18264[77], 4}, {NULL}};
const Segment SGI275929[] = {{&S18264[13], 3}, {&S18264[17], 13}, {&S18264[51], 17}, {NULL}};
const Segment SGI275930[] = {{&S18265[68], 4}, {NULL}};
const Segment SGI275931[] = {{&S18265[13], 5}, {&S18265[19], 2}, {&S18265[42], 17}, {NULL}};
const Segment SGI275932[] = {{&S18266[83], 4}, {NULL}};
const Segment SGI275933[] = {{&S18266[13], 3}, {&S18266[17], 13}, {&S18266[31], 4}, {&S18266[57], 17}, {NULL}};
const Segment SGI275934[] = {{&S18267[80], 4}, {NULL}};
const Segment SGI275935[] = {{&S18267[13], 3}, {&S18267[17], 10}, {&S18267[28], 4}, {&S18267[54], 17}, {NULL}};
const Segment SGI275936[] = {{&S18268[80], 4}, {NULL}};
const Segment SGI275937[] = {{&S18268[13], 3}, {&S18268[17], 10}, {&S18268[28], 4}, {&S18268[54], 17}, {NULL}};
const Segment SGI275938[] = {{&S18269[83], 4}, {NULL}};
const Segment SGI275939[] = {{&S18269[13], 3}, {&S18269[17], 13}, {&S18269[31], 4}, {&S18269[57], 17}, {NULL}};
const Segment SGI275940[] = {{&S18270[80], 4}, {NULL}};
const Segment SGI275941[] = {{&S18270[13], 3}, {&S18270[17], 10}, {&S18270[28], 4}, {&S18270[54], 17}, {NULL}};
const Segment SGI275942[] = {{&S18271[83], 4}, {NULL}};
const Segment SGI275943[] = {{&S18271[13], 3}, {&S18271[17], 13}, {&S18271[31], 4}, {&S18271[57], 17}, {NULL}};
const Segment SGI275944[] = {{&S18272[80], 4}, {NULL}};
const Segment SGI275945[] = {{&S18272[13], 3}, {&S18272[17], 10}, {&S18272[28], 4}, {&S18272[54], 17}, {NULL}};
const Segment SGI275946[] = {{&S18273[97], 4}, {NULL}};
const Segment SGI275947[] = {{&S18273[13], 3}, {&S18273[17], 13}, {&S18273[51], 17}, {&S18273[69], 19}, {&S18273[120], 8}, {NULL}};
const Segment SGI275948[] = {{&S18274[78], 4}, {NULL}};
const Segment SGI275949[] = {{&S18274[13], 3}, {&S18274[17], 2}, {&S18274[20], 11}, {&S18274[52], 17}, {NULL}};
const Segment SGI275950[] = {{&S18275[68], 4}, {NULL}};
const Segment SGI275951[] = {{&S18275[13], 5}, {&S18275[19], 2}, {&S18275[42], 17}, {&S18275[94], 8}, {NULL}};
const Segment SGI275952[] = {{&S18533[76], 4}, {NULL}};
const Segment SGI275953[] = {{&S18533[13], 3}, {&S18533[17], 11}, {&S18533[50], 17}, {NULL}};
const Segment SGI275954[] = {{&S18534[78], 4}, {NULL}};
const Segment SGI275955[] = {{&S18534[13], 3}, {&S18534[17], 13}, {&S18534[52], 17}, {NULL}};
const Segment SGI275956[] = {{&S18535[85], 4}, {NULL}};
const Segment SGI275957[] = {{&S18535[13], 3}, {&S18535[17], 19}, {&S18535[59], 17}, {NULL}};
const Segment SGI275958[] = {{&S18536[76], 4}, {NULL}};
const Segment SGI275959[] = {{&S18536[13], 3}, {&S18536[17], 11}, {&S18536[50], 17}, {&S18536[103], 8}, {NULL}};
const Segment SGI275960[] = {{&S18537[78], 4}, {NULL}};
const Segment SGI275961[] = {{&S18537[13], 3}, {&S18537[17], 13}, {&S18537[52], 17}, {&S18537[105], 8}, {NULL}};
const Segment SGI275962[] = {{&S18538[76], 4}, {NULL}};
const Segment SGI275963[] = {{&S18538[13], 3}, {&S18538[17], 13}, {&S18538[52], 17}, {NULL}};
const Segment SGI275964[] = {{&S18538[71], 9}, {NULL}};
const Segment SGI275965[] = {{&S18539[78], 4}, {NULL}};
const Segment SGI275966[] = {{&S18539[13], 3}, {&S18539[17], 13}, {&S18539[52], 17}, {NULL}};
const Segment SGI275967[] = {{&S18540[74], 4}, {NULL}};
const Segment SGI275968[] = {{&S18540[13], 3}, {&S18540[17], 11}, {&S18540[50], 17}, {NULL}};
const Segment SGI275969[] = {{&S18540[69], 9}, {NULL}};
const Segment SGI275970[] = {{&S22662[78], 4}, {NULL}};
const Segment SGI275971[] = {{&S22662[13], 3}, {&S22662[17], 13}, {&S22662[52], 17}, {NULL}};
const Segment SGI275972[] = {{&S22663[76], 4}, {NULL}};
const Segment SGI275973[] = {{&S22663[13], 3}, {&S22663[17], 11}, {&S22663[50], 17}, {NULL}};
const Segment SGI275974[] = {{&S22664[85], 4}, {NULL}};
const Segment SGI275975[] = {{&S22664[13], 3}, {&S22664[17], 19}, {&S22664[59], 17}, {NULL}};
const Segment SGI275976[] = {{&S22665[100], 4}, {NULL}};
const Segment SGI275977[] = {{&S22665[13], 3}, {&S22665[17], 21}, {&S22665[60], 17}, {NULL}};
const Segment SGI275978[] = {{&S22666[81], 4}, {NULL}};
const Segment SGI275979[] = {{&S22666[13], 3}, {&S22666[17], 13}, {&S22666[53], 17}, {&S22666[144], 7}, {NULL}};
const Segment SGI275980[] = {{&S22667[79], 4}, {NULL}};
const Segment SGI275981[] = {{&S22667[13], 3}, {&S22667[17], 11}, {&S22667[51], 17}, {&S22667[142], 7}, {NULL}};
const Segment SGI275982[] = {{&S22668[81], 4}, {NULL}};
const Segment SGI275983[] = {{&S22668[13], 3}, {&S22668[17], 13}, {&S22668[53], 17}, {&S22668[144], 7}, {NULL}};
const Segment SGI275984[] = {{&S22669[76], 4}, {NULL}};
const Segment SGI275985[] = {{&S22669[13], 3}, {&S22669[17], 11}, {&S22669[50], 17}, {&S22669[102], 8}, {&S22669[113], 24}, {NULL}};
const Segment SGI275986[] = {{&S22670[81], 4}, {NULL}};
const Segment SGI275987[] = {{&S22670[13], 3}, {&S22670[17], 13}, {&S22670[53], 17}, {&S22670[152], 7}, {NULL}};
const Segment SGI275988[] = {{&S22671[79], 4}, {NULL}};
const Segment SGI275989[] = {{&S22671[13], 3}, {&S22671[17], 11}, {&S22671[51], 17}, {&S22671[142], 14}, {NULL}};
const Segment SGI275990[] = {{&S22672[81], 4}, {NULL}};
const Segment SGI275991[] = {{&S22672[13], 3}, {&S22672[17], 13}, {&S22672[53], 17}, {&S22672[144], 14}, {NULL}};
const Segment SGI275992[] = {{&S22673[81], 4}, {NULL}};
const Segment SGI275993[] = {{&S22673[13], 3}, {&S22673[17], 13}, {&S22673[53], 17}, {&S22673[127], 45}, {&S22673[173], 16}, {NULL}};
const Segment SGI275994[] = {{&S18569[93], 4}, {NULL}};
const Segment SGI275995[] = {{&S18569[13], 9}, {&S18569[23], 22}, {&S18569[67], 17}, {&S18569[120], 8}, {&S18569[131], 26}, {NULL}};
const Segment SGI275996[] = {{&S22674[81], 4}, {NULL}};
const Segment SGI275997[] = {{&S22674[13], 3}, {&S22674[17], 13}, {&S22674[53], 17}, {&S22674[144], 7}, {NULL}};
const Segment SGI275998[] = {{&S18570[93], 4}, {NULL}};
const Segment SGI275999[] = {{&S18570[13], 9}, {&S18570[23], 22}, {&S18570[67], 17}, {NULL}};
const Segment SGI276000[] = {{&S22675[79], 4}, {NULL}};
const Segment SGI276001[] = {{&S22675[13], 3}, {&S22675[17], 11}, {&S22675[51], 17}, {&S22675[142], 7}, {NULL}};
const Segment SGI276002[] = {{&S18571[93], 4}, {NULL}};
const Segment SGI276003[] = {{&S18571[13], 9}, {&S18571[23], 22}, {&S18571[67], 17}, {NULL}};
const Segment SGI276004[] = {{&S22676[74], 4}, {NULL}};
const Segment SGI276005[] = {{&S22676[13], 3}, {&S22676[17], 6}, {&S22676[46], 17}, {NULL}};
const Segment SGI276006[] = {{&S22677[79], 4}, {NULL}};
const Segment SGI276007[] = {{&S22677[13], 3}, {&S22677[17], 11}, {&S22677[51], 17}, {&S22677[142], 7}, {NULL}};
const Segment SGI276008[] = {{&S22678[79], 4}, {NULL}};
const Segment SGI276009[] = {{&S22678[13], 3}, {&S22678[17], 11}, {&S22678[51], 17}, {&S22678[150], 7}, {NULL}};
const Segment SGI276010[] = {{&S22679[81], 4}, {NULL}};
const Segment SGI276011[] = {{&S22679[13], 3}, {&S22679[17], 13}, {&S22679[53], 17}, {NULL}};
const Segment SGI276012[] = {{&S22680[81], 4}, {NULL}};
const Segment SGI276013[] = {{&S22680[13], 3}, {&S22680[17], 13}, {&S22680[53], 17}, {&S22680[139], 9}, {NULL}};
const Segment SGI276014[] = {{&S22681[79], 4}, {NULL}};
const Segment SGI276015[] = {{&S22681[13], 3}, {&S22681[17], 11}, {&S22681[51], 17}, {&S22681[137], 9}, {NULL}};
const Segment SGI276016[] = {{&S22682[80], 4}, {NULL}};
const Segment SGI276017[] = {{&S22682[13], 3}, {&S22682[17], 10}, {&S22682[28], 4}, {&S22682[54], 17}, {NULL}};
const Segment SGI276018[] = {{&S22683[83], 4}, {NULL}};
const Segment SGI276019[] = {{&S22683[13], 3}, {&S22683[17], 13}, {&S22683[31], 4}, {&S22683[57], 17}, {NULL}};
const Segment SGI276020[] = {{&S22684[80], 4}, {NULL}};
const Segment SGI276021[] = {{&S22684[13], 3}, {&S22684[17], 10}, {&S22684[28], 4}, {&S22684[54], 17}, {NULL}};
const Segment SGI276022[] = {{&S22685[80], 4}, {NULL}};
const Segment SGI276023[] = {{&S22685[13], 3}, {&S22685[17], 10}, {&S22685[28], 4}, {&S22685[54], 17}, {NULL}};
const Segment SGI276024[] = {{&S22686[83], 4}, {NULL}};
const Segment SGI276025[] = {{&S22686[13], 3}, {&S22686[17], 13}, {&S22686[31], 4}, {&S22686[57], 17}, {NULL}};
const Segment SGI276026[] = {{&S18572[93], 4}, {NULL}};
const Segment SGI276027[] = {{&S18572[13], 9}, {&S18572[23], 22}, {&S18572[67], 17}, {NULL}};
const Segment SGI276028[] = {{&S18573[93], 4}, {NULL}};
const Segment SGI276029[] = {{&S18573[13], 9}, {&S18573[23], 22}, {&S18573[67], 17}, {NULL}};
const Segment SGI276030[] = {{&S22687[80], 4}, {NULL}};
const Segment SGI276031[] = {{&S22687[13], 3}, {&S22687[17], 10}, {&S22687[28], 4}, {&S22687[54], 17}, {NULL}};
const Segment SGI276032[] = {{&S22688[83], 4}, {NULL}};
const Segment SGI276033[] = {{&S22688[13], 3}, {&S22688[17], 13}, {&S22688[31], 4}, {&S22688[57], 17}, {NULL}};
const Segment SGI276034[] = {{&S18574[93], 4}, {NULL}};
const Segment SGI276035[] = {{&S18574[13], 9}, {&S18574[23], 22}, {&S18574[67], 17}, {NULL}};
const Segment SGI276036[] = {{&S37041[38], 5}, {NULL}};
const Segment SGI276037[] = {{&S37041[100], 5}, {NULL}};
const Segment SGI276038[] = {{&S22689[80], 4}, {NULL}};
const Segment SGI276039[] = {{&S22689[13], 3}, {&S22689[17], 13}, {&S22689[52], 17}, {NULL}};
const Segment SGI276040[] = {{&S18575[93], 4}, {NULL}};
const Segment SGI276041[] = {{&S18575[13], 9}, {&S18575[23], 22}, {&S18575[67], 17}, {NULL}};
const Segment SGI276042[] = {{&S22690[79], 4}, {NULL}};
const Segment SGI276043[] = {{&S22690[13], 3}, {&S22690[17], 11}, {&S22690[51], 17}, {&S22690[136], 5}, {NULL}};
const Segment SGI276044[] = {{&S18576[93], 4}, {NULL}};
const Segment SGI276045[] = {{&S18576[13], 9}, {&S18576[23], 22}, {&S18576[67], 17}, {NULL}};
const Segment SGI276046[] = {{&S18577[93], 4}, {NULL}};
const Segment SGI276047[] = {{&S18577[13], 9}, {&S18577[23], 22}, {&S18577[67], 17}, {NULL}};
const Segment SGI276048[] = {{&S18578[93], 4}, {NULL}};
const Segment SGI276049[] = {{&S18578[13], 9}, {&S18578[23], 22}, {&S18578[67], 17}, {&S18578[120], 8}, {NULL}};
const Segment SGI276050[] = {{&S18579[93], 4}, {NULL}};
const Segment SGI276051[] = {{&S18579[13], 9}, {&S18579[23], 22}, {&S18579[67], 17}, {NULL}};
const Segment SGI276052[] = {{&S37168[38], 5}, {NULL}};
const Segment SGI276053[] = {{&S37168[99], 5}, {NULL}};
const Segment SGI276054[] = {{&S18705[75], 4}, {NULL}};
const Segment SGI276055[] = {{&S18705[13], 14}, {&S18705[49], 17}, {NULL}};
const Segment SGI276056[] = {{&S18706[75], 4}, {NULL}};
const Segment SGI276057[] = {{&S18706[13], 14}, {&S18706[49], 17}, {NULL}};
const Segment SGI276058[] = {{&S18707[82], 4}, {NULL}};
const Segment SGI276059[] = {{&S18707[13], 14}, {&S18707[28], 6}, {&S18707[56], 17}, {NULL}};
const Segment SGI276060[] = {{&S18708[75], 4}, {NULL}};
const Segment SGI276061[] = {{&S18708[13], 14}, {&S18708[49], 17}, {&S18708[102], 8}, {&S18708[113], 24}, {NULL}};
const Segment SGI276062[] = {{&S18709[75], 4}, {NULL}};
const Segment SGI276063[] = {{&S18709[13], 14}, {&S18709[49], 17}, {&S18709[102], 8}, {&S18709[113], 26}, {NULL}};
const Segment SGI276064[] = {{&S18710[82], 4}, {NULL}};
const Segment SGI276065[] = {{&S18710[13], 14}, {&S18710[28], 6}, {&S18710[56], 17}, {NULL}};
const Segment SGI276066[] = {{&S18711[75], 4}, {NULL}};
const Segment SGI276067[] = {{&S18711[13], 14}, {&S18711[49], 17}, {&S18711[102], 8}, {NULL}};
const Segment SGI276068[] = {{&S18712[74], 4}, {NULL}};
const Segment SGI276069[] = {{&S18712[13], 14}, {&S18712[49], 16}, {NULL}};
const Segment SGI276070[] = {{&S19118[78], 4}, {NULL}};
const Segment SGI276071[] = {{&S19118[13], 3}, {&S19118[17], 13}, {&S19118[52], 17}, {NULL}};
const Segment SGI276072[] = {{&S19119[76], 4}, {NULL}};
const Segment SGI276073[] = {{&S19119[13], 3}, {&S19119[17], 11}, {&S19119[50], 17}, {NULL}};
const Segment SGI276074[] = {{&S19120[85], 4}, {NULL}};
const Segment SGI276075[] = {{&S19120[13], 3}, {&S19120[17], 19}, {&S19120[59], 17}, {NULL}};
const Segment SGI276076[] = {{&S19121[78], 4}, {NULL}};
const Segment SGI276077[] = {{&S19121[13], 3}, {&S19121[17], 13}, {&S19121[52], 17}, {&S19121[122], 35}, {&S19121[158], 27}, {NULL}};
const Segment SGI276078[] = {{&S19122[83], 4}, {NULL}};
const Segment SGI276079[] = {{&S19122[13], 3}, {&S19122[17], 13}, {&S19122[31], 4}, {&S19122[57], 17}, {NULL}};
const Segment SGI276080[] = {{&S19123[80], 4}, {NULL}};
const Segment SGI276081[] = {{&S19123[13], 3}, {&S19123[17], 10}, {&S19123[28], 4}, {&S19123[54], 17}, {NULL}};
const Segment SGI276082[] = {{&S19124[80], 4}, {NULL}};
const Segment SGI276083[] = {{&S19124[13], 3}, {&S19124[17], 10}, {&S19124[28], 4}, {&S19124[54], 17}, {NULL}};
const Segment SGI276084[] = {{&S19125[80], 4}, {NULL}};
const Segment SGI276085[] = {{&S19125[13], 3}, {&S19125[17], 10}, {&S19125[28], 4}, {&S19125[54], 17}, {NULL}};
const Segment SGI276086[] = {{&S19126[83], 4}, {NULL}};
const Segment SGI276087[] = {{&S19126[13], 3}, {&S19126[17], 13}, {&S19126[31], 4}, {&S19126[57], 17}, {NULL}};
const Segment SGI276088[] = {{&S19127[83], 4}, {NULL}};
const Segment SGI276089[] = {{&S19127[13], 3}, {&S19127[17], 13}, {&S19127[31], 4}, {&S19127[57], 17}, {NULL}};
const Segment SGI276090[] = {{&S19128[72], 4}, {NULL}};
const Segment SGI276091[] = {{&S19128[13], 5}, {&S19128[19], 6}, {&S19128[46], 17}, {NULL}};
const Segment SGI276092[] = {{&S19129[80], 4}, {NULL}};
const Segment SGI276093[] = {{&S19129[13], 3}, {&S19129[17], 10}, {&S19129[28], 4}, {&S19129[54], 17}, {NULL}};
const Segment SGI276094[] = {{&S19130[80], 4}, {NULL}};
const Segment SGI276095[] = {{&S19130[13], 3}, {&S19130[17], 13}, {&S19130[52], 17}, {&S19130[137], 8}, {NULL}};
const Segment SGI276096[] = {{&S19131[78], 4}, {NULL}};
const Segment SGI276097[] = {{&S19131[13], 3}, {&S19131[17], 13}, {&S19131[52], 17}, {&S19131[106], 8}, {NULL}};
const Segment SGI276098[] = {{&S19132[76], 4}, {NULL}};
const Segment SGI276099[] = {{&S19132[13], 3}, {&S19132[17], 11}, {&S19132[50], 17}, {&S19132[103], 8}, {NULL}};
const Segment SGI276100[] = {{&S19133[80], 4}, {NULL}};
const Segment SGI276101[] = {{&S19133[13], 3}, {&S19133[17], 13}, {&S19133[52], 17}, {&S19133[142], 7}, {NULL}};
const Segment SGI276102[] = {{&S19134[88], 4}, {NULL}};
const Segment SGI276103[] = {{&S19134[13], 3}, {&S19134[17], 21}, {&S19134[60], 17}, {NULL}};
const Segment SGI276104[] = {{&S19135[80], 4}, {NULL}};
const Segment SGI276105[] = {{&S19135[13], 3}, {&S19135[17], 13}, {&S19135[52], 17}, {&S19135[137], 9}, {NULL}};
const Segment SGI276106[] = {{&S19136[90], 4}, {NULL}};
const Segment SGI276107[] = {{&S19136[13], 3}, {&S19136[17], 23}, {&S19136[62], 17}, {NULL}};
const Segment SGI276108[] = {{&S19137[80], 4}, {NULL}};
const Segment SGI276109[] = {{&S19137[13], 3}, {&S19137[17], 13}, {&S19137[52], 17}, {&S19137[142], 7}, {NULL}};
const Segment SGI276110[] = {{&S19174[76], 4}, {NULL}};
const Segment SGI276111[] = {{&S19174[13], 3}, {&S19174[17], 11}, {&S19174[50], 17}, {NULL}};
const Segment SGI276112[] = {{&S19175[99], 4}, {NULL}};
const Segment SGI276113[] = {{&S19175[13], 3}, {&S19175[17], 21}, {&S19175[59], 17}, {NULL}};
const Segment SGI276114[] = {{&S19176[78], 4}, {NULL}};
const Segment SGI276115[] = {{&S19176[13], 3}, {&S19176[17], 13}, {&S19176[52], 17}, {NULL}};
const Segment SGI276116[] = {{&S19177[85], 4}, {NULL}};
const Segment SGI276117[] = {{&S19177[13], 3}, {&S19177[17], 19}, {&S19177[59], 17}, {NULL}};
const Segment SGI276118[] = {{&S19178[78], 4}, {NULL}};
const Segment SGI276119[] = {{&S19178[13], 3}, {&S19178[17], 13}, {&S19178[52], 17}, {&S19178[105], 8}, {NULL}};
const Segment SGI276120[] = {{&S19179[76], 4}, {NULL}};
const Segment SGI276121[] = {{&S19179[13], 3}, {&S19179[17], 11}, {&S19179[50], 17}, {&S19179[103], 8}, {NULL}};
const Segment SGI276122[] = {{&S19180[83], 4}, {NULL}};
const Segment SGI276123[] = {{&S19180[13], 3}, {&S19180[17], 13}, {&S19180[31], 4}, {&S19180[57], 17}, {NULL}};
const Segment SGI276124[] = {{&S19181[80], 4}, {NULL}};
const Segment SGI276125[] = {{&S19181[13], 3}, {&S19181[17], 10}, {&S19181[28], 4}, {&S19181[54], 17}, {NULL}};
const Segment SGI276126[] = {{&S19182[83], 4}, {NULL}};
const Segment SGI276127[] = {{&S19182[13], 3}, {&S19182[17], 13}, {&S19182[31], 4}, {&S19182[57], 17}, {NULL}};
const Segment SGI276128[] = {{&S19183[80], 4}, {NULL}};
const Segment SGI276129[] = {{&S19183[13], 3}, {&S19183[17], 10}, {&S19183[28], 4}, {&S19183[54], 17}, {NULL}};
const Segment SGI276130[] = {{&S19184[80], 4}, {NULL}};
const Segment SGI276131[] = {{&S19184[13], 3}, {&S19184[17], 10}, {&S19184[28], 4}, {&S19184[54], 17}, {NULL}};
const Segment SGI276132[] = {{&S19185[80], 4}, {NULL}};
const Segment SGI276133[] = {{&S19185[13], 3}, {&S19185[17], 10}, {&S19185[28], 4}, {&S19185[54], 17}, {NULL}};
const Segment SGI276134[] = {{&S19186[83], 4}, {NULL}};
const Segment SGI276135[] = {{&S19186[13], 3}, {&S19186[17], 13}, {&S19186[31], 4}, {&S19186[57], 17}, {NULL}};
const Segment SGI276136[] = {{&S19187[76], 4}, {NULL}};
const Segment SGI276137[] = {{&S19187[13], 3}, {&S19187[17], 11}, {&S19187[50], 17}, {&S19187[103], 8}, {&S19187[114], 26}, {NULL}};
const Segment SGI276138[] = {{&S19188[80], 4}, {NULL}};
const Segment SGI276139[] = {{&S19188[13], 3}, {&S19188[17], 13}, {&S19188[52], 17}, {&S19188[138], 5}, {NULL}};
const Segment SGI276140[] = {{&S19189[80], 4}, {NULL}};
const Segment SGI276141[] = {{&S19189[13], 3}, {&S19189[17], 13}, {&S19189[52], 17}, {NULL}};
const Segment SGI276142[] = {{&S19304[92], 4}, {NULL}};
const Segment SGI276143[] = {{&S19304[13], 9}, {&S19304[23], 22}, {&S19304[66], 17}, {NULL}};
const Segment SGI276144[] = {{&S19305[92], 4}, {NULL}};
const Segment SGI276145[] = {{&S19305[13], 9}, {&S19305[23], 22}, {&S19305[66], 17}, {NULL}};
const Segment SGI276146[] = {{&S19306[92], 4}, {NULL}};
const Segment SGI276147[] = {{&S19306[13], 9}, {&S19306[23], 22}, {&S19306[66], 17}, {NULL}};
const Segment SGI276148[] = {{&S19307[92], 4}, {NULL}};
const Segment SGI276149[] = {{&S19307[13], 9}, {&S19307[23], 22}, {&S19307[66], 17}, {NULL}};
const Segment SGI276150[] = {{&S19308[92], 4}, {NULL}};
const Segment SGI276151[] = {{&S19308[13], 9}, {&S19308[23], 22}, {&S19308[66], 17}, {NULL}};
const Segment SGI276152[] = {{&S19309[92], 4}, {NULL}};
const Segment SGI276153[] = {{&S19309[13], 9}, {&S19309[23], 22}, {&S19309[66], 17}, {NULL}};
const Segment SGI276154[] = {{&S19310[92], 4}, {NULL}};
const Segment SGI276155[] = {{&S19310[13], 9}, {&S19310[23], 22}, {&S19310[66], 17}, {&S19310[118], 8}, {NULL}};
const Segment SGI276156[] = {{&S19311[92], 4}, {NULL}};
const Segment SGI276157[] = {{&S19311[13], 9}, {&S19311[23], 22}, {&S19311[66], 17}, {&S19311[118], 8}, {&S19311[129], 24}, {NULL}};
const Segment SGI276158[] = {{&S19312[92], 4}, {NULL}};
const Segment SGI276159[] = {{&S19312[13], 9}, {&S19312[23], 22}, {&S19312[66], 17}, {&S19312[118], 8}, {&S19312[129], 24}, {NULL}};
const Segment SGI276160[] = {{&S19313[92], 4}, {NULL}};
const Segment SGI276161[] = {{&S19313[13], 9}, {&S19313[23], 22}, {&S19313[66], 17}, {&S19313[118], 8}, {NULL}};
const Segment SGI276162[] = {{&S19314[92], 4}, {NULL}};
const Segment SGI276163[] = {{&S19314[13], 9}, {&S19314[23], 22}, {&S19314[66], 17}, {&S19314[118], 8}, {&S19314[129], 26}, {NULL}};
const Segment SGI276164[] = {{&S19315[92], 4}, {NULL}};
const Segment SGI276165[] = {{&S19315[13], 9}, {&S19315[23], 22}, {&S19315[66], 17}, {&S19315[118], 8}, {&S19315[129], 26}, {NULL}};
const Segment SGI276166[] = {{&S19316[92], 4}, {NULL}};
const Segment SGI276167[] = {{&S19316[13], 9}, {&S19316[23], 22}, {&S19316[66], 17}, {&S19316[135], 35}, {&S19316[171], 21}, {NULL}};
const Segment SGI276168[] = {{&S19317[92], 4}, {NULL}};
const Segment SGI276169[] = {{&S19317[13], 9}, {&S19317[23], 22}, {&S19317[66], 17}, {&S19317[135], 35}, {&S19317[171], 16}, {NULL}};
const Segment SGI276170[] = {{&S19318[92], 4}, {NULL}};
const Segment SGI276171[] = {{&S19318[13], 9}, {&S19318[23], 22}, {&S19318[66], 17}, {&S19318[135], 35}, {&S19318[171], 16}, {NULL}};
const Segment SGI276172[] = {{&S19319[89], 4}, {NULL}};
const Segment SGI276173[] = {{&S19319[13], 9}, {&S19319[23], 20}, {&S19319[64], 16}, {NULL}};
const Segment SGI276174[] = {{&S19320[92], 4}, {NULL}};
const Segment SGI276175[] = {{&S19320[13], 9}, {&S19320[23], 22}, {&S19320[66], 17}, {&S19320[130], 4}, {NULL}};
const Segment SGI276176[] = {{&S19321[92], 4}, {NULL}};
const Segment SGI276177[] = {{&S19321[13], 9}, {&S19321[23], 22}, {&S19321[66], 17}, {&S19321[130], 4}, {NULL}};
const Segment SGI276178[] = {{&S19322[92], 4}, {NULL}};
const Segment SGI276179[] = {{&S19322[13], 9}, {&S19322[23], 22}, {&S19322[66], 17}, {&S19322[130], 4}, {NULL}};
const Segment SGI276180[] = {{&S19323[74], 4}, {NULL}};
const Segment SGI276181[] = {{&S19323[13], 14}, {&S19323[48], 17}, {NULL}};
const Segment SGI276182[] = {{&S19324[81], 4}, {NULL}};
const Segment SGI276183[] = {{&S19324[13], 14}, {&S19324[28], 6}, {&S19324[55], 17}, {NULL}};
const Segment SGI276184[] = {{&S19325[74], 4}, {NULL}};
const Segment SGI276185[] = {{&S19325[13], 14}, {&S19325[48], 17}, {&S19325[100], 8}, {&S19325[111], 24}, {NULL}};
const Segment SGI276186[] = {{&S19326[81], 4}, {NULL}};
const Segment SGI276187[] = {{&S19326[13], 14}, {&S19326[28], 6}, {&S19326[55], 17}, {&S19326[107], 8}, {&S19326[118], 24}, {NULL}};
const Segment SGI276188[] = {{&S19327[81], 4}, {NULL}};
const Segment SGI276189[] = {{&S19327[13], 14}, {&S19327[28], 6}, {&S19327[55], 17}, {&S19327[107], 8}, {NULL}};
const Segment SGI276190[] = {{&S19328[74], 4}, {NULL}};
const Segment SGI276191[] = {{&S19328[13], 14}, {&S19328[48], 17}, {&S19328[100], 8}, {NULL}};
const Segment SGI276192[] = {{&S19329[74], 4}, {NULL}};
const Segment SGI276193[] = {{&S19329[13], 14}, {&S19329[48], 17}, {&S19329[100], 8}, {&S19329[111], 26}, {NULL}};
const Segment SGI276194[] = {{&S19330[73], 4}, {NULL}};
const Segment SGI276195[] = {{&S19330[13], 14}, {&S19330[48], 16}, {NULL}};
const Segment SGI276196[] = {{&S19331[74], 4}, {NULL}};
const Segment SGI276197[] = {{&S19331[13], 14}, {&S19331[48], 17}, {&S19331[112], 4}, {NULL}};
const Segment SGI276198[] = {{&S19332[74], 4}, {NULL}};
const Segment SGI276199[] = {{&S19332[13], 14}, {&S19332[48], 17}, {&S19332[112], 4}, {NULL}};
const Segment SGI276200[] = {{&S19333[74], 4}, {NULL}};
const Segment SGI276201[] = {{&S19333[13], 14}, {&S19333[48], 17}, {&S19333[112], 4}, {NULL}};
const Segment SGI276202[] = {{&S19334[74], 4}, {NULL}};
const Segment SGI276203[] = {{&S19334[13], 14}, {&S19334[48], 17}, {NULL}};
const Segment SGI276204[] = {{&S19335[81], 4}, {NULL}};
const Segment SGI276205[] = {{&S19335[13], 14}, {&S19335[28], 6}, {&S19335[55], 17}, {&S19335[119], 4}, {NULL}};
const Segment SGI276206[] = {{&S19338[41], 4}, {NULL}};
const Segment SGI276207[] = {{&S19338[13], 14}, {&S19338[28], 25}, {&S19338[74], 17}, {NULL}};
const Segment SGI276208[] = {{&S19339[41], 4}, {NULL}};
const Segment SGI276209[] = {{&S19339[13], 14}, {&S19339[28], 25}, {&S19339[74], 17}, {NULL}};
const Segment SGI276210[] = {{&S19340[41], 4}, {NULL}};
const Segment SGI276211[] = {{&S19340[13], 14}, {&S19340[28], 25}, {&S19340[74], 17}, {NULL}};
const Segment SGI276212[] = {{&S19341[41], 4}, {NULL}};
const Segment SGI276213[] = {{&S19341[13], 14}, {&S19341[28], 25}, {&S19341[74], 17}, {NULL}};
const Segment SGI276214[] = {{&S19342[41], 4}, {NULL}};
const Segment SGI276215[] = {{&S19342[13], 14}, {&S19342[28], 25}, {&S19342[74], 17}, {NULL}};
const Segment SGI276216[] = {{&S19343[48], 4}, {NULL}};
const Segment SGI276217[] = {{&S19343[13], 14}, {&S19343[28], 6}, {&S19343[35], 25}, {&S19343[81], 17}, {NULL}};
const Segment SGI276218[] = {{&S19345[41], 4}, {NULL}};
const Segment SGI276219[] = {{&S19345[13], 14}, {&S19345[28], 25}, {&S19345[74], 17}, {NULL}};
const Segment SGI276220[] = {{&S19347[41], 4}, {NULL}};
const Segment SGI276221[] = {{&S19347[13], 14}, {&S19347[28], 25}, {&S19347[74], 17}, {NULL}};
const Segment SGI276222[] = {{&S19348[41], 4}, {NULL}};
const Segment SGI276223[] = {{&S19348[13], 14}, {&S19348[28], 25}, {&S19348[74], 17}, {NULL}};
const Segment SGI276224[] = {{&S19349[41], 4}, {NULL}};
const Segment SGI276225[] = {{&S19349[13], 14}, {&S19349[28], 25}, {&S19349[74], 17}, {NULL}};
const Segment SGI276226[] = {{&S19351[41], 4}, {NULL}};
const Segment SGI276227[] = {{&S19351[13], 14}, {&S19351[28], 25}, {&S19351[74], 17}, {NULL}};
const Segment SGI276228[] = {{&S19352[41], 4}, {NULL}};
const Segment SGI276229[] = {{&S19352[13], 14}, {&S19352[28], 25}, {&S19352[74], 17}, {NULL}};
const Segment SGI276230[] = {{&S19353[41], 4}, {NULL}};
const Segment SGI276231[] = {{&S19353[13], 14}, {&S19353[28], 25}, {&S19353[74], 17}, {NULL}};
const Segment SGI276232[] = {{&S19354[41], 4}, {NULL}};
const Segment SGI276233[] = {{&S19354[13], 14}, {&S19354[28], 25}, {&S19354[74], 17}, {NULL}};
const Segment SGI276234[] = {{&S19357[41], 4}, {NULL}};
const Segment SGI276235[] = {{&S19357[13], 14}, {&S19357[28], 25}, {&S19357[74], 17}, {NULL}};
const Segment SGI276236[] = {{&S19360[48], 4}, {NULL}};
const Segment SGI276237[] = {{&S19360[13], 14}, {&S19360[28], 6}, {&S19360[35], 25}, {&S19360[81], 17}, {NULL}};
const Segment SGI276238[] = {{&S19365[48], 4}, {NULL}};
const Segment SGI276239[] = {{&S19365[13], 14}, {&S19365[28], 6}, {&S19365[35], 25}, {&S19365[81], 17}, {NULL}};
const Segment SGI276240[] = {{&S38281[38], 5}, {NULL}};
const Segment SGI276241[] = {{&S38281[100], 5}, {NULL}};
const Segment SGI276242[] = {{&S23659[13], 7}, {&S23659[21], 2}, {&S23659[24], 15}, {&S23659[54], 17}, {NULL}};
const Segment SGI276243[] = {{&S23660[74], 4}, {NULL}};
const Segment SGI276244[] = {{&S23660[13], 14}, {&S23660[48], 17}, {NULL}};
const Segment SGI276245[] = {{&S23661[81], 4}, {NULL}};
const Segment SGI276246[] = {{&S23661[13], 14}, {&S23661[28], 6}, {&S23661[55], 17}, {NULL}};
const Segment SGI276247[] = {{&S23662[93], 4}, {NULL}};
const Segment SGI276248[] = {{&S23662[13], 7}, {&S23662[21], 2}, {&S23662[24], 15}, {&S23662[40], 6}, {&S23662[67], 17}, {NULL}};
const Segment SGI276249[] = {{&S23663[74], 4}, {NULL}};
const Segment SGI276250[] = {{&S23663[13], 14}, {&S23663[48], 17}, {NULL}};
const Segment SGI276251[] = {{&S23664[81], 4}, {NULL}};
const Segment SGI276252[] = {{&S23664[13], 14}, {&S23664[28], 6}, {&S23664[55], 17}, {&S23664[107], 8}, {NULL}};
const Segment SGI276253[] = {{&S23665[93], 4}, {NULL}};
const Segment SGI276254[] = {{&S23665[13], 7}, {&S23665[21], 2}, {&S23665[24], 15}, {&S23665[40], 6}, {&S23665[67], 17}, {NULL}};
const Segment SGI276255[] = {{&S23666[93], 4}, {NULL}};
const Segment SGI276256[] = {{&S23666[13], 7}, {&S23666[21], 2}, {&S23666[24], 15}, {&S23666[40], 6}, {&S23666[67], 17}, {NULL}};
const Segment SGI276257[] = {{&S23667[93], 4}, {NULL}};
const Segment SGI276258[] = {{&S23667[13], 7}, {&S23667[21], 2}, {&S23667[24], 15}, {&S23667[40], 6}, {&S23667[67], 17}, {NULL}};
const Segment SGI276259[] = {{&S23668[94], 4}, {NULL}};
const Segment SGI276260[] = {{&S23668[13], 7}, {&S23668[21], 2}, {&S23668[24], 15}, {&S23668[40], 7}, {&S23668[68], 17}, {NULL}};
const Segment SGI276261[] = {{&S23669[93], 4}, {NULL}};
const Segment SGI276262[] = {{&S23669[13], 7}, {&S23669[21], 2}, {&S23669[24], 15}, {&S23669[40], 6}, {&S23669[67], 17}, {NULL}};
const Segment SGI276263[] = {{&S23670[93], 4}, {NULL}};
const Segment SGI276264[] = {{&S23670[13], 7}, {&S23670[21], 2}, {&S23670[24], 15}, {&S23670[40], 6}, {&S23670[67], 17}, {NULL}};
const Segment SGI276265[] = {{&S23671[93], 4}, {NULL}};
const Segment SGI276266[] = {{&S23671[13], 7}, {&S23671[21], 2}, {&S23671[24], 15}, {&S23671[40], 6}, {&S23671[67], 17}, {NULL}};
const Segment SGI276267[] = {{&S23672[80], 4}, {NULL}};
const Segment SGI276268[] = {{&S23672[13], 14}, {&S23672[28], 6}, {&S23672[55], 16}, {NULL}};
const Segment SGI276269[] = {{&S23673[93], 4}, {NULL}};
const Segment SGI276270[] = {{&S23673[13], 7}, {&S23673[21], 2}, {&S23673[24], 15}, {&S23673[40], 6}, {&S23673[67], 17}, {NULL}};
const Segment SGI276271[] = {{&S18351[93], 4}, {NULL}};
const Segment SGI276272[] = {{&S18351[13], 9}, {&S18351[23], 22}, {&S18351[67], 17}, {NULL}};
const Segment SGI276273[] = {{&S18352[93], 4}, {NULL}};
const Segment SGI276274[] = {{&S18352[13], 9}, {&S18352[23], 22}, {&S18352[67], 17}, {NULL}};
const Segment SGI276275[] = {{&S18353[93], 4}, {NULL}};
const Segment SGI276276[] = {{&S18353[13], 9}, {&S18353[23], 22}, {&S18353[67], 17}, {NULL}};
const Segment SGI276277[] = {{&S18354[93], 4}, {NULL}};
const Segment SGI276278[] = {{&S18354[13], 9}, {&S18354[23], 22}, {&S18354[67], 17}, {NULL}};
const Segment SGI276279[] = {{&S18355[93], 4}, {NULL}};
const Segment SGI276280[] = {{&S18355[13], 9}, {&S18355[23], 22}, {&S18355[67], 17}, {NULL}};
const Segment SGI276281[] = {{&S18356[93], 4}, {NULL}};
const Segment SGI276282[] = {{&S18356[13], 9}, {&S18356[23], 22}, {&S18356[67], 17}, {NULL}};
const Segment SGI276283[] = {{&S18357[93], 4}, {NULL}};
const Segment SGI276284[] = {{&S18357[13], 9}, {&S18357[23], 22}, {&S18357[67], 17}, {&S18357[120], 8}, {NULL}};
const Segment SGI276285[] = {{&S19861[80], 4}, {NULL}};
const Segment SGI276286[] = {{&S19861[13], 3}, {&S19861[17], 10}, {&S19861[28], 4}, {&S19861[54], 17}, {NULL}};
const Segment SGI276287[] = {{&S19862[83], 4}, {NULL}};
const Segment SGI276288[] = {{&S19862[13], 3}, {&S19862[17], 13}, {&S19862[31], 4}, {&S19862[57], 17}, {NULL}};
const Segment SGI276289[] = {{&S19863[80], 4}, {NULL}};
const Segment SGI276290[] = {{&S19863[13], 3}, {&S19863[17], 10}, {&S19863[28], 4}, {&S19863[54], 17}, {NULL}};
const Segment SGI276291[] = {{&S19864[80], 4}, {NULL}};
const Segment SGI276292[] = {{&S19864[13], 3}, {&S19864[17], 10}, {&S19864[28], 4}, {&S19864[54], 17}, {NULL}};
const Segment SGI276293[] = {{&S19865[83], 4}, {NULL}};
const Segment SGI276294[] = {{&S19865[13], 3}, {&S19865[17], 13}, {&S19865[31], 4}, {&S19865[57], 17}, {NULL}};
const Segment SGI276295[] = {{&S19866[85], 4}, {NULL}};
const Segment SGI276296[] = {{&S19866[13], 3}, {&S19866[17], 19}, {&S19866[59], 17}, {NULL}};
const Segment SGI276297[] = {{&S19867[78], 4}, {NULL}};
const Segment SGI276298[] = {{&S19867[13], 3}, {&S19867[17], 13}, {&S19867[52], 17}, {NULL}};
const Segment SGI276299[] = {{&S19868[76], 4}, {NULL}};
const Segment SGI276300[] = {{&S19868[13], 3}, {&S19868[17], 11}, {&S19868[50], 17}, {NULL}};
const Segment SGI276301[] = {{&S19869[83], 4}, {NULL}};
const Segment SGI276302[] = {{&S19869[13], 3}, {&S19869[17], 13}, {&S19869[31], 4}, {&S19869[57], 17}, {NULL}};
const Segment SGI276303[] = {{&S19870[78], 4}, {NULL}};
const Segment SGI276304[] = {{&S19870[13], 3}, {&S19870[17], 13}, {&S19870[52], 17}, {&S19870[105], 8}, {NULL}};
const Segment SGI276305[] = {{&S19871[76], 4}, {NULL}};
const Segment SGI276306[] = {{&S19871[13], 3}, {&S19871[17], 11}, {&S19871[50], 17}, {&S19871[103], 8}, {NULL}};
const Segment SGI276307[] = {{&S19872[80], 4}, {NULL}};
const Segment SGI276308[] = {{&S19872[13], 3}, {&S19872[17], 10}, {&S19872[28], 4}, {&S19872[54], 17}, {NULL}};
const Segment SGI276309[] = {{&S19873[78], 4}, {NULL}};
const Segment SGI276310[] = {{&S19873[13], 3}, {&S19873[17], 13}, {&S19873[52], 17}, {&S19873[105], 8}, {&S19873[116], 26}, {NULL}};
const Segment SGI276311[] = {{&S19874[80], 4}, {NULL}};
const Segment SGI276312[] = {{&S19874[13], 3}, {&S19874[17], 10}, {&S19874[28], 4}, {&S19874[54], 17}, {NULL}};
const Segment SGI276313[] = {{&S20185[92], 4}, {NULL}};
const Segment SGI276314[] = {{&S20185[13], 9}, {&S20185[23], 22}, {&S20185[66], 17}, {NULL}};
const Segment SGI276315[] = {{&S20186[92], 4}, {NULL}};
const Segment SGI276316[] = {{&S20186[13], 9}, {&S20186[23], 22}, {&S20186[66], 17}, {NULL}};
const Segment SGI276317[] = {{&S20187[92], 4}, {NULL}};
const Segment SGI276318[] = {{&S20187[13], 9}, {&S20187[23], 22}, {&S20187[66], 17}, {NULL}};
const Segment SGI276319[] = {{&S20188[92], 4}, {NULL}};
const Segment SGI276320[] = {{&S20188[13], 9}, {&S20188[23], 22}, {&S20188[66], 17}, {&S20188[118], 8}, {&S20188[129], 24}, {NULL}};
const Segment SGI276321[] = {{&S20189[92], 4}, {NULL}};
const Segment SGI276322[] = {{&S20189[13], 9}, {&S20189[23], 22}, {&S20189[66], 17}, {&S20189[118], 8}, {&S20189[129], 26}, {NULL}};
const Segment SGI276323[] = {{&S20190[92], 4}, {NULL}};
const Segment SGI276324[] = {{&S20190[13], 9}, {&S20190[23], 22}, {&S20190[66], 17}, {&S20190[118], 8}, {NULL}};
const Segment SGI276325[] = {{&S20191[91], 4}, {NULL}};
const Segment SGI276326[] = {{&S20191[13], 9}, {&S20191[23], 22}, {&S20191[66], 16}, {NULL}};
const Segment SGI276327[] = {{&S20192[92], 4}, {NULL}};
const Segment SGI276328[] = {{&S20192[13], 9}, {&S20192[23], 22}, {&S20192[66], 17}, {NULL}};
const Segment SGI276329[] = {{&S20193[89], 4}, {NULL}};
const Segment SGI276330[] = {{&S20193[13], 9}, {&S20193[23], 20}, {&S20193[64], 16}, {NULL}};
const Segment SGI276331[] = {{&S20194[89], 4}, {NULL}};
const Segment SGI276332[] = {{&S20194[13], 9}, {&S20194[23], 20}, {&S20194[64], 16}, {NULL}};
const Segment SGI276333[] = {{&S20195[92], 4}, {NULL}};
const Segment SGI276334[] = {{&S20195[13], 9}, {&S20195[23], 22}, {&S20195[66], 17}, {&S20195[130], 4}, {NULL}};
const Segment SGI276335[] = {{&S20196[92], 4}, {NULL}};
const Segment SGI276336[] = {{&S20196[13], 9}, {&S20196[23], 22}, {&S20196[66], 17}, {&S20196[130], 4}, {NULL}};
const Segment SGI276337[] = {{&S20197[92], 4}, {NULL}};
const Segment SGI276338[] = {{&S20197[13], 9}, {&S20197[23], 22}, {&S20197[66], 17}, {&S20197[130], 4}, {NULL}};
const Segment SGI276339[] = {{&S20198[92], 4}, {NULL}};
const Segment SGI276340[] = {{&S20198[13], 9}, {&S20198[23], 22}, {&S20198[66], 17}, {NULL}};
const Segment SGI276341[] = {{&S20199[92], 4}, {NULL}};
const Segment SGI276342[] = {{&S20199[13], 9}, {&S20199[23], 22}, {&S20199[66], 17}, {NULL}};
const Segment SGI276343[] = {{&S24236[77], 4}, {NULL}};
const Segment SGI276344[] = {{&S24236[13], 3}, {&S24236[17], 13}, {&S24236[51], 17}, {NULL}};
const Segment SGI276345[] = {{&S24237[39], 4}, {NULL}};
const Segment SGI276346[] = {{&S24237[13], 3}, {&S24237[17], 13}, {NULL}};
const Segment SGI276347[] = {{&S24238[97], 4}, {NULL}};
const Segment SGI276348[] = {{&S24238[13], 3}, {&S24238[17], 13}, {&S24238[51], 17}, {&S24238[69], 19}, {NULL}};
const Segment SGI276349[] = {{&S24239[75], 4}, {NULL}};
const Segment SGI276350[] = {{&S24239[13], 3}, {&S24239[17], 11}, {&S24239[49], 17}, {&S24239[101], 8}, {&S24239[112], 24}, {NULL}};
const Segment SGI276351[] = {{&S24240[87], 4}, {NULL}};
const Segment SGI276352[] = {{&S24240[13], 3}, {&S24240[17], 2}, {&S24240[20], 13}, {&S24240[34], 6}, {&S24240[61], 17}, {NULL}};
const Segment SGI276353[] = {{&S24241[75], 4}, {NULL}};
const Segment SGI276354[] = {{&S24241[13], 3}, {&S24241[17], 11}, {&S24241[49], 17}, {NULL}};
const Segment SGI276355[] = {{&S24242[97], 4}, {NULL}};
const Segment SGI276356[] = {{&S24242[13], 3}, {&S24242[17], 13}, {&S24242[51], 17}, {&S24242[69], 19}, {&S24242[123], 8}, {&S24242[134], 26}, {NULL}};
const Segment SGI276357[] = {{&S24243[84], 4}, {NULL}};
const Segment SGI276358[] = {{&S24243[13], 3}, {&S24243[17], 19}, {&S24243[58], 17}, {NULL}};
const Segment SGI276359[] = {{&S24244[105], 4}, {NULL}};
const Segment SGI276360[] = {{&S24244[13], 3}, {&S24244[17], 13}, {&S24244[51], 17}, {&S24244[69], 27}, {NULL}};
const Segment SGI276361[] = {{&S24245[81], 4}, {NULL}};
const Segment SGI276362[] = {{&S24245[14], 3}, {&S24245[18], 16}, {&S24245[55], 17}, {NULL}};
const Segment SGI276363[] = {{&S24245[0], 1}, {NULL}};
const Segment SGI276364[] = {{&S24246[78], 4}, {NULL}};
const Segment SGI276365[] = {{&S24246[13], 3}, {&S24246[17], 13}, {&S24246[52], 17}, {NULL}};
const Segment SGI276366[] = {{&S24247[76], 4}, {NULL}};
const Segment SGI276367[] = {{&S24247[13], 3}, {&S24247[17], 11}, {&S24247[50], 17}, {NULL}};
const Segment SGI276368[] = {{&S24248[77], 4}, {NULL}};
const Segment SGI276369[] = {{&S24248[13], 3}, {&S24248[17], 13}, {&S24248[51], 17}, {NULL}};
const Segment SGI276370[] = {{&S24249[80], 4}, {NULL}};
const Segment SGI276371[] = {{&S24249[13], 3}, {&S24249[17], 10}, {&S24249[28], 4}, {&S24249[54], 17}, {NULL}};
const Segment SGI276372[] = {{&S24250[80], 4}, {NULL}};
const Segment SGI276373[] = {{&S24250[13], 3}, {&S24250[17], 10}, {&S24250[28], 4}, {&S24250[54], 17}, {NULL}};
const Segment SGI276374[] = {{&S24251[83], 4}, {NULL}};
const Segment SGI276375[] = {{&S24251[13], 3}, {&S24251[17], 13}, {&S24251[31], 4}, {&S24251[57], 17}, {NULL}};
const Segment SGI276376[] = {{&S24252[80], 4}, {NULL}};
const Segment SGI276377[] = {{&S24252[13], 3}, {&S24252[17], 10}, {&S24252[28], 4}, {&S24252[54], 17}, {NULL}};
const Segment SGI276378[] = {{&S24253[83], 4}, {NULL}};
const Segment SGI276379[] = {{&S24253[13], 3}, {&S24253[17], 13}, {&S24253[31], 4}, {&S24253[57], 17}, {NULL}};
const Segment SGI276380[] = {{&S24254[83], 4}, {NULL}};
const Segment SGI276381[] = {{&S24254[13], 3}, {&S24254[17], 13}, {&S24254[31], 4}, {&S24254[57], 17}, {NULL}};
const Segment SGI276382[] = {{&S24255[80], 4}, {NULL}};
const Segment SGI276383[] = {{&S24255[13], 3}, {&S24255[17], 10}, {&S24255[28], 4}, {&S24255[54], 17}, {NULL}};
const Segment SGI276384[] = {{&S24256[65], 4}, {NULL}};
const Segment SGI276385[] = {{&S24256[13], 5}, {&S24256[39], 17}, {NULL}};
const Segment SGI276386[] = {{&S24257[75], 5}, {NULL}};
const Segment SGI276387[] = {{&S24257[13], 3}, {&S24257[17], 11}, {&S24257[49], 17}, {NULL}};
const Segment SGI276388[] = {{&S24258[77], 5}, {NULL}};
const Segment SGI276389[] = {{&S24258[13], 3}, {&S24258[17], 13}, {&S24258[51], 17}, {NULL}};
const Segment SGI276390[] = {{&S24259[78], 5}, {NULL}};
const Segment SGI276391[] = {{&S24259[13], 3}, {&S24259[17], 2}, {&S24259[20], 11}, {&S24259[52], 17}, {NULL}};
const Segment SGI276392[] = {{&S24744[103], 4}, {NULL}};
const Segment SGI276393[] = {{&S24744[13], 9}, {&S24744[23], 2}, {&S24744[26], 22}, {&S24744[49], 6}, {&S24744[77], 17}, {NULL}};
const Segment SGI276394[] = {{&S24745[103], 4}, {NULL}};
const Segment SGI276395[] = {{&S24745[13], 9}, {&S24745[23], 2}, {&S24745[26], 22}, {&S24745[49], 6}, {&S24745[77], 17}, {NULL}};
const Segment SGI276396[] = {{&S24746[103], 4}, {NULL}};
const Segment SGI276397[] = {{&S24746[13], 9}, {&S24746[23], 2}, {&S24746[26], 22}, {&S24746[49], 6}, {&S24746[77], 17}, {NULL}};
const Segment SGI276398[] = {{&S24747[103], 4}, {NULL}};
const Segment SGI276399[] = {{&S24747[13], 9}, {&S24747[23], 2}, {&S24747[26], 22}, {&S24747[49], 6}, {&S24747[77], 17}, {NULL}};
const Segment SGI276400[] = {{&S24748[103], 4}, {NULL}};
const Segment SGI276401[] = {{&S24748[13], 9}, {&S24748[23], 2}, {&S24748[26], 22}, {&S24748[49], 6}, {&S24748[77], 17}, {NULL}};
const Segment SGI276402[] = {{&S24749[103], 4}, {NULL}};
const Segment SGI276403[] = {{&S24749[13], 9}, {&S24749[23], 2}, {&S24749[26], 22}, {&S24749[49], 6}, {&S24749[77], 17}, {NULL}};
const Segment SGI276404[] = {{&S24750[103], 4}, {NULL}};
const Segment SGI276405[] = {{&S24750[13], 9}, {&S24750[23], 2}, {&S24750[26], 22}, {&S24750[49], 6}, {&S24750[77], 17}, {NULL}};
const Segment SGI276406[] = {{&S24751[103], 4}, {NULL}};
const Segment SGI276407[] = {{&S24751[13], 9}, {&S24751[23], 2}, {&S24751[26], 22}, {&S24751[49], 6}, {&S24751[77], 17}, {NULL}};
const Segment SGI276408[] = {{&S24752[103], 4}, {NULL}};
const Segment SGI276409[] = {{&S24752[13], 9}, {&S24752[23], 2}, {&S24752[26], 22}, {&S24752[49], 6}, {&S24752[77], 17}, {NULL}};
const Segment SGI276410[] = {{&S24753[104], 4}, {NULL}};
const Segment SGI276411[] = {{&S24753[13], 9}, {&S24753[23], 2}, {&S24753[26], 23}, {&S24753[50], 7}, {&S24753[78], 17}, {NULL}};
const Segment SGI276412[] = {{&S24754[104], 4}, {NULL}};
const Segment SGI276413[] = {{&S24754[13], 9}, {&S24754[23], 2}, {&S24754[26], 23}, {&S24754[50], 7}, {&S24754[78], 17}, {NULL}};
const Segment SGI276414[] = {{&S24755[103], 4}, {NULL}};
const Segment SGI276415[] = {{&S24755[13], 9}, {&S24755[23], 2}, {&S24755[26], 22}, {&S24755[49], 6}, {&S24755[77], 17}, {NULL}};
const Segment SGI276416[] = {{&S24756[166], 4}, {NULL}};
const Segment SGI276417[] = {{&S24756[12], 3}, {&S24756[16], 11}, {&S24756[28], 2}, {&S24756[31], 3}, {&S24756[76], 9}, {&S24756[28], 2}, {&S24756[89], 22}, {&S24756[112], 6}, {&S24756[140], 17}, {NULL}};
const Segment SGI276418[] = {{&S24756[0], 63}, {NULL}};
const Segment SGI276419[] = {{&S24784[89], 4}, {NULL}};
const Segment SGI276420[] = {{&S24784[13], 3}, {&S24784[17], 2}, {&S24784[20], 13}, {&S24784[34], 6}, {&S24784[62], 17}, {NULL}};
const Segment SGI276421[] = {{&S24784[115], 10}, {NULL}};
const Segment SGI276422[] = {{&S24785[89], 4}, {NULL}};
const Segment SGI276423[] = {{&S24785[13], 3}, {&S24785[17], 2}, {&S24785[20], 13}, {&S24785[34], 6}, {&S24785[62], 17}, {NULL}};
const Segment SGI276424[] = {{&S24785[115], 10}, {NULL}};
const Segment SGI276425[] = {{&S24786[89], 4}, {NULL}};
const Segment SGI276426[] = {{&S24786[13], 3}, {&S24786[17], 2}, {&S24786[20], 13}, {&S24786[34], 6}, {&S24786[62], 17}, {NULL}};
const Segment SGI276427[] = {{&S24786[115], 10}, {NULL}};
const Segment SGI276428[] = {{&S24787[89], 4}, {NULL}};
const Segment SGI276429[] = {{&S24787[13], 3}, {&S24787[17], 2}, {&S24787[20], 13}, {&S24787[34], 6}, {&S24787[62], 17}, {NULL}};
const Segment SGI276430[] = {{&S24787[115], 10}, {NULL}};
const Segment SGI276431[] = {{&S24788[89], 4}, {NULL}};
const Segment SGI276432[] = {{&S24788[13], 3}, {&S24788[17], 2}, {&S24788[20], 13}, {&S24788[34], 6}, {&S24788[62], 17}, {NULL}};
const Segment SGI276433[] = {{&S24788[115], 10}, {NULL}};
const Segment SGI276434[] = {{&S24789[89], 4}, {NULL}};
const Segment SGI276435[] = {{&S24789[13], 3}, {&S24789[17], 2}, {&S24789[20], 13}, {&S24789[34], 6}, {&S24789[62], 17}, {NULL}};
const Segment SGI276436[] = {{&S24789[115], 10}, {NULL}};
const Segment SGI276437[] = {{&S24790[89], 4}, {NULL}};
const Segment SGI276438[] = {{&S24790[13], 3}, {&S24790[17], 2}, {&S24790[20], 13}, {&S24790[34], 6}, {&S24790[62], 17}, {NULL}};
const Segment SGI276439[] = {{&S24790[115], 10}, {NULL}};
const Segment SGI276440[] = {{&S39532[38], 5}, {NULL}};
const Segment SGI276441[] = {{&S39532[100], 5}, {NULL}};
const Segment SGI276442[] = {{&S24791[89], 4}, {NULL}};
const Segment SGI276443[] = {{&S24791[13], 3}, {&S24791[17], 2}, {&S24791[20], 13}, {&S24791[34], 6}, {&S24791[62], 17}, {NULL}};
const Segment SGI276444[] = {{&S24791[115], 10}, {NULL}};
const Segment SGI276445[] = {{&S24792[89], 4}, {NULL}};
const Segment SGI276446[] = {{&S24792[13], 3}, {&S24792[17], 2}, {&S24792[20], 13}, {&S24792[34], 6}, {&S24792[62], 17}, {NULL}};
const Segment SGI276447[] = {{&S24792[115], 10}, {NULL}};
const Segment SGI276448[] = {{&S24793[89], 4}, {NULL}};
const Segment SGI276449[] = {{&S24793[13], 3}, {&S24793[17], 2}, {&S24793[20], 13}, {&S24793[34], 6}, {&S24793[62], 17}, {NULL}};
const Segment SGI276450[] = {{&S24793[115], 10}, {NULL}};
const Segment SGI276451[] = {{&S24794[89], 4}, {NULL}};
const Segment SGI276452[] = {{&S24794[13], 3}, {&S24794[17], 2}, {&S24794[20], 13}, {&S24794[34], 6}, {&S24794[62], 17}, {NULL}};
const Segment SGI276453[] = {{&S24794[115], 10}, {NULL}};
const Segment SGI276454[] = {{&S24795[89], 4}, {NULL}};
const Segment SGI276455[] = {{&S24795[13], 3}, {&S24795[17], 2}, {&S24795[20], 13}, {&S24795[34], 6}, {&S24795[62], 17}, {NULL}};
const Segment SGI276456[] = {{&S24795[115], 10}, {NULL}};
const Segment SGI276457[] = {{&S24796[89], 4}, {NULL}};
const Segment SGI276458[] = {{&S24796[13], 3}, {&S24796[17], 2}, {&S24796[20], 13}, {&S24796[34], 6}, {&S24796[62], 17}, {NULL}};
const Segment SGI276459[] = {{&S24796[115], 10}, {NULL}};
const Segment SGI276460[] = {{&S24797[89], 4}, {NULL}};
const Segment SGI276461[] = {{&S24797[13], 3}, {&S24797[17], 2}, {&S24797[20], 13}, {&S24797[34], 6}, {&S24797[62], 17}, {NULL}};
const Segment SGI276462[] = {{&S24797[115], 10}, {NULL}};
const Segment SGI276463[] = {{&S24798[89], 4}, {NULL}};
const Segment SGI276464[] = {{&S24798[13], 3}, {&S24798[17], 2}, {&S24798[20], 13}, {&S24798[34], 6}, {&S24798[62], 17}, {NULL}};
const Segment SGI276465[] = {{&S24798[115], 10}, {NULL}};
const Segment SGI276466[] = {{&S24799[89], 4}, {NULL}};
const Segment SGI276467[] = {{&S24799[13], 3}, {&S24799[17], 2}, {&S24799[20], 13}, {&S24799[34], 6}, {&S24799[62], 17}, {NULL}};
const Segment SGI276468[] = {{&S24799[115], 10}, {NULL}};
const Segment SGI276469[] = {{&S24800[89], 4}, {NULL}};
const Segment SGI276470[] = {{&S24800[13], 3}, {&S24800[17], 2}, {&S24800[20], 13}, {&S24800[34], 6}, {&S24800[62], 17}, {NULL}};
const Segment SGI276471[] = {{&S24800[115], 10}, {NULL}};
const Segment SGI276472[] = {{&S24801[89], 4}, {NULL}};
const Segment SGI276473[] = {{&S24801[13], 3}, {&S24801[17], 2}, {&S24801[20], 13}, {&S24801[34], 6}, {&S24801[62], 17}, {NULL}};
const Segment SGI276474[] = {{&S24801[115], 10}, {NULL}};
const Segment SGI276475[] = {{&S24802[89], 4}, {NULL}};
const Segment SGI276476[] = {{&S24802[13], 3}, {&S24802[17], 2}, {&S24802[20], 13}, {&S24802[34], 6}, {&S24802[62], 17}, {NULL}};
const Segment SGI276477[] = {{&S24802[115], 10}, {NULL}};
const Segment SGI276478[] = {{&S24803[89], 4}, {NULL}};
const Segment SGI276479[] = {{&S24803[13], 3}, {&S24803[17], 2}, {&S24803[20], 13}, {&S24803[34], 6}, {&S24803[62], 17}, {NULL}};
const Segment SGI276480[] = {{&S24803[115], 10}, {NULL}};
const Segment SGI276481[] = {{&S24804[89], 4}, {NULL}};
const Segment SGI276482[] = {{&S24804[13], 3}, {&S24804[17], 2}, {&S24804[20], 13}, {&S24804[34], 6}, {&S24804[62], 17}, {NULL}};
const Segment SGI276483[] = {{&S24804[115], 10}, {NULL}};
const Segment SGI276484[] = {{&S24805[89], 4}, {NULL}};
const Segment SGI276485[] = {{&S24805[13], 3}, {&S24805[17], 2}, {&S24805[20], 13}, {&S24805[34], 6}, {&S24805[62], 17}, {NULL}};
const Segment SGI276486[] = {{&S24805[115], 10}, {NULL}};
const Segment SGI276487[] = {{&S24806[89], 4}, {NULL}};
const Segment SGI276488[] = {{&S24806[13], 3}, {&S24806[17], 2}, {&S24806[20], 13}, {&S24806[34], 6}, {&S24806[62], 17}, {NULL}};
const Segment SGI276489[] = {{&S24806[115], 10}, {NULL}};
const Segment SGI276490[] = {{&S24807[89], 4}, {NULL}};
const Segment SGI276491[] = {{&S24807[13], 3}, {&S24807[17], 2}, {&S24807[20], 13}, {&S24807[34], 6}, {&S24807[62], 17}, {NULL}};
const Segment SGI276492[] = {{&S24807[115], 10}, {NULL}};
const Segment SGI276493[] = {{&S24808[89], 4}, {NULL}};
const Segment SGI276494[] = {{&S24808[13], 3}, {&S24808[17], 2}, {&S24808[20], 13}, {&S24808[34], 6}, {&S24808[62], 17}, {NULL}};
const Segment SGI276495[] = {{&S24808[115], 10}, {NULL}};
const Segment SGI276496[] = {{&S24809[89], 4}, {NULL}};
const Segment SGI276497[] = {{&S24809[13], 3}, {&S24809[17], 2}, {&S24809[20], 13}, {&S24809[34], 6}, {&S24809[62], 17}, {NULL}};
const Segment SGI276498[] = {{&S24809[115], 10}, {NULL}};
const Segment SGI276499[] = {{&S24810[89], 4}, {NULL}};
const Segment SGI276500[] = {{&S24810[13], 3}, {&S24810[17], 2}, {&S24810[20], 13}, {&S24810[34], 6}, {&S24810[62], 17}, {NULL}};
const Segment SGI276501[] = {{&S24810[115], 10}, {NULL}};
const Segment SGI276502[] = {{&S24811[89], 4}, {NULL}};
const Segment SGI276503[] = {{&S24811[13], 3}, {&S24811[17], 2}, {&S24811[20], 13}, {&S24811[34], 6}, {&S24811[62], 17}, {NULL}};
const Segment SGI276504[] = {{&S24811[115], 10}, {NULL}};
const Segment SGI276505[] = {{&S24812[89], 4}, {NULL}};
const Segment SGI276506[] = {{&S24812[13], 3}, {&S24812[17], 2}, {&S24812[20], 13}, {&S24812[34], 6}, {&S24812[62], 17}, {NULL}};
const Segment SGI276507[] = {{&S24812[115], 10}, {NULL}};
const Segment SGI276508[] = {{&S24813[89], 4}, {NULL}};
const Segment SGI276509[] = {{&S24813[13], 3}, {&S24813[17], 2}, {&S24813[20], 13}, {&S24813[34], 6}, {&S24813[62], 17}, {NULL}};
const Segment SGI276510[] = {{&S24813[115], 10}, {NULL}};
const Segment SGI276511[] = {{&S24814[89], 4}, {NULL}};
const Segment SGI276512[] = {{&S24814[13], 3}, {&S24814[17], 2}, {&S24814[20], 13}, {&S24814[34], 6}, {&S24814[62], 17}, {NULL}};
const Segment SGI276513[] = {{&S24814[115], 10}, {NULL}};
const Segment SGI276514[] = {{&S24815[89], 4}, {NULL}};
const Segment SGI276515[] = {{&S24815[13], 3}, {&S24815[17], 2}, {&S24815[20], 13}, {&S24815[34], 6}, {&S24815[62], 17}, {NULL}};
const Segment SGI276516[] = {{&S24815[115], 10}, {NULL}};
const Segment SGI276517[] = {{&S24816[89], 4}, {NULL}};
const Segment SGI276518[] = {{&S24816[13], 3}, {&S24816[17], 2}, {&S24816[20], 13}, {&S24816[34], 6}, {&S24816[62], 17}, {NULL}};
const Segment SGI276519[] = {{&S24816[115], 10}, {NULL}};
const Segment SGI276520[] = {{&S24817[89], 4}, {NULL}};
const Segment SGI276521[] = {{&S24817[13], 3}, {&S24817[17], 2}, {&S24817[20], 13}, {&S24817[34], 6}, {&S24817[62], 17}, {NULL}};
const Segment SGI276522[] = {{&S24817[115], 10}, {NULL}};
const Segment SGI276523[] = {{&S24818[89], 4}, {NULL}};
const Segment SGI276524[] = {{&S24818[13], 3}, {&S24818[17], 2}, {&S24818[20], 13}, {&S24818[34], 6}, {&S24818[62], 17}, {NULL}};
const Segment SGI276525[] = {{&S24818[115], 10}, {NULL}};
const Segment SGI276526[] = {{&S24819[89], 4}, {NULL}};
const Segment SGI276527[] = {{&S24819[13], 3}, {&S24819[17], 2}, {&S24819[20], 13}, {&S24819[34], 6}, {&S24819[62], 17}, {NULL}};
const Segment SGI276528[] = {{&S24819[115], 10}, {NULL}};
const Segment SGI276529[] = {{&S24820[89], 4}, {NULL}};
const Segment SGI276530[] = {{&S24820[13], 3}, {&S24820[17], 2}, {&S24820[20], 13}, {&S24820[34], 6}, {&S24820[62], 17}, {NULL}};
const Segment SGI276531[] = {{&S24820[115], 10}, {NULL}};
const Segment SGI276532[] = {{&S24821[89], 4}, {NULL}};
const Segment SGI276533[] = {{&S24821[13], 3}, {&S24821[17], 2}, {&S24821[20], 13}, {&S24821[34], 6}, {&S24821[62], 17}, {NULL}};
const Segment SGI276534[] = {{&S24821[115], 10}, {NULL}};
const Segment SGI276535[] = {{&S24822[89], 4}, {NULL}};
const Segment SGI276536[] = {{&S24822[13], 3}, {&S24822[17], 2}, {&S24822[20], 13}, {&S24822[34], 6}, {&S24822[62], 17}, {NULL}};
const Segment SGI276537[] = {{&S24822[115], 10}, {NULL}};
const Segment SGI276538[] = {{&S24823[89], 4}, {NULL}};
const Segment SGI276539[] = {{&S24823[13], 3}, {&S24823[17], 2}, {&S24823[20], 13}, {&S24823[34], 6}, {&S24823[62], 17}, {NULL}};
const Segment SGI276540[] = {{&S24823[115], 10}, {NULL}};
const Segment SGI276541[] = {{&S24824[89], 4}, {NULL}};
const Segment SGI276542[] = {{&S24824[13], 3}, {&S24824[17], 2}, {&S24824[20], 13}, {&S24824[34], 6}, {&S24824[62], 17}, {NULL}};
const Segment SGI276543[] = {{&S24824[115], 10}, {NULL}};
const Segment SGI276544[] = {{&S24825[89], 4}, {NULL}};
const Segment SGI276545[] = {{&S24825[13], 3}, {&S24825[17], 2}, {&S24825[20], 13}, {&S24825[34], 6}, {&S24825[62], 17}, {NULL}};
const Segment SGI276546[] = {{&S24825[115], 10}, {NULL}};
const Segment SGI276547[] = {{&S24826[89], 4}, {NULL}};
const Segment SGI276548[] = {{&S24826[13], 3}, {&S24826[17], 2}, {&S24826[20], 13}, {&S24826[34], 6}, {&S24826[62], 17}, {NULL}};
const Segment SGI276549[] = {{&S24826[115], 10}, {NULL}};
const Segment SGI276550[] = {{&S24827[89], 4}, {NULL}};
const Segment SGI276551[] = {{&S24827[13], 3}, {&S24827[17], 2}, {&S24827[20], 13}, {&S24827[34], 6}, {&S24827[62], 17}, {NULL}};
const Segment SGI276552[] = {{&S24827[115], 10}, {NULL}};
const Segment SGI276553[] = {{&S24828[89], 4}, {NULL}};
const Segment SGI276554[] = {{&S24828[13], 3}, {&S24828[17], 2}, {&S24828[20], 13}, {&S24828[34], 6}, {&S24828[62], 17}, {NULL}};
const Segment SGI276555[] = {{&S24828[115], 10}, {NULL}};
const Segment SGI276556[] = {{&S24829[89], 4}, {NULL}};
const Segment SGI276557[] = {{&S24829[13], 3}, {&S24829[17], 2}, {&S24829[20], 13}, {&S24829[34], 6}, {&S24829[62], 17}, {NULL}};
const Segment SGI276558[] = {{&S24829[115], 10}, {NULL}};
const Segment SGI276559[] = {{&S24830[89], 4}, {NULL}};
const Segment SGI276560[] = {{&S24830[13], 3}, {&S24830[17], 2}, {&S24830[20], 13}, {&S24830[34], 6}, {&S24830[62], 17}, {NULL}};
const Segment SGI276561[] = {{&S24830[115], 10}, {NULL}};
const Segment SGI276562[] = {{&S24831[89], 4}, {NULL}};
const Segment SGI276563[] = {{&S24831[13], 3}, {&S24831[17], 2}, {&S24831[20], 13}, {&S24831[34], 6}, {&S24831[62], 17}, {NULL}};
const Segment SGI276564[] = {{&S24831[115], 10}, {NULL}};
const Segment SGI276565[] = {{&S24832[89], 4}, {NULL}};
const Segment SGI276566[] = {{&S24832[13], 3}, {&S24832[17], 2}, {&S24832[20], 13}, {&S24832[34], 6}, {&S24832[62], 17}, {NULL}};
const Segment SGI276567[] = {{&S24832[115], 10}, {NULL}};
const Segment SGI276568[] = {{&S24833[89], 4}, {NULL}};
const Segment SGI276569[] = {{&S24833[13], 3}, {&S24833[17], 2}, {&S24833[20], 13}, {&S24833[34], 6}, {&S24833[62], 17}, {NULL}};
const Segment SGI276570[] = {{&S24833[115], 10}, {NULL}};
const Segment SGI276571[] = {{&S24834[89], 4}, {NULL}};
const Segment SGI276572[] = {{&S24834[13], 3}, {&S24834[17], 2}, {&S24834[20], 13}, {&S24834[34], 6}, {&S24834[62], 17}, {NULL}};
const Segment SGI276573[] = {{&S24834[115], 10}, {NULL}};
const Segment SGI276574[] = {{&S24835[89], 4}, {NULL}};
const Segment SGI276575[] = {{&S24835[13], 3}, {&S24835[17], 2}, {&S24835[20], 13}, {&S24835[34], 6}, {&S24835[62], 17}, {NULL}};
const Segment SGI276576[] = {{&S24835[115], 10}, {NULL}};
const Segment SGI276577[] = {{&S24836[89], 4}, {NULL}};
const Segment SGI276578[] = {{&S24836[13], 3}, {&S24836[17], 2}, {&S24836[20], 13}, {&S24836[34], 6}, {&S24836[62], 17}, {NULL}};
const Segment SGI276579[] = {{&S24836[115], 10}, {NULL}};
const Segment SGI276580[] = {{&S24837[89], 4}, {NULL}};
const Segment SGI276581[] = {{&S24837[13], 3}, {&S24837[17], 2}, {&S24837[20], 13}, {&S24837[34], 6}, {&S24837[62], 17}, {NULL}};
const Segment SGI276582[] = {{&S24837[115], 10}, {NULL}};
const Segment SGI276583[] = {{&S24838[89], 4}, {NULL}};
const Segment SGI276584[] = {{&S24838[13], 3}, {&S24838[17], 2}, {&S24838[20], 13}, {&S24838[34], 6}, {&S24838[62], 17}, {NULL}};
const Segment SGI276585[] = {{&S24838[115], 10}, {NULL}};
const Segment SGI276586[] = {{&S24839[89], 4}, {NULL}};
const Segment SGI276587[] = {{&S24839[13], 3}, {&S24839[17], 2}, {&S24839[20], 13}, {&S24839[34], 6}, {&S24839[62], 17}, {NULL}};
const Segment SGI276588[] = {{&S24839[115], 10}, {NULL}};
const Segment SGI276589[] = {{&S24840[89], 4}, {NULL}};
const Segment SGI276590[] = {{&S24840[13], 3}, {&S24840[17], 2}, {&S24840[20], 13}, {&S24840[34], 6}, {&S24840[62], 17}, {NULL}};
const Segment SGI276591[] = {{&S24840[115], 10}, {NULL}};
const Segment SGI276592[] = {{&S24841[89], 4}, {NULL}};
const Segment SGI276593[] = {{&S24841[13], 3}, {&S24841[17], 2}, {&S24841[20], 13}, {&S24841[34], 6}, {&S24841[62], 17}, {NULL}};
const Segment SGI276594[] = {{&S24841[115], 10}, {NULL}};
const Segment SGI276595[] = {{&S24842[89], 4}, {NULL}};
const Segment SGI276596[] = {{&S24842[13], 3}, {&S24842[17], 2}, {&S24842[20], 13}, {&S24842[34], 6}, {&S24842[62], 17}, {NULL}};
const Segment SGI276597[] = {{&S24842[115], 10}, {NULL}};
const Segment SGI276598[] = {{&S24843[89], 4}, {NULL}};
const Segment SGI276599[] = {{&S24843[13], 3}, {&S24843[17], 2}, {&S24843[20], 13}, {&S24843[34], 6}, {&S24843[62], 17}, {NULL}};
const Segment SGI276600[] = {{&S24843[115], 10}, {NULL}};
const Segment SGI276601[] = {{&S24844[89], 4}, {NULL}};
const Segment SGI276602[] = {{&S24844[13], 3}, {&S24844[17], 2}, {&S24844[20], 13}, {&S24844[34], 6}, {&S24844[62], 17}, {NULL}};
const Segment SGI276603[] = {{&S24844[115], 10}, {NULL}};
const Segment SGI276604[] = {{&S24845[86], 4}, {NULL}};
const Segment SGI276605[] = {{&S24845[13], 3}, {&S24845[17], 2}, {&S24845[20], 13}, {&S24845[34], 3}, {&S24845[59], 17}, {NULL}};
const Segment SGI276606[] = {{&S24845[112], 10}, {NULL}};
const Segment SGI276607[] = {{&S24846[89], 4}, {NULL}};
const Segment SGI276608[] = {{&S24846[13], 3}, {&S24846[17], 2}, {&S24846[20], 13}, {&S24846[34], 6}, {&S24846[62], 17}, {NULL}};
const Segment SGI276609[] = {{&S24846[115], 10}, {NULL}};
const Segment SGI276610[] = {{&S24847[89], 4}, {NULL}};
const Segment SGI276611[] = {{&S24847[13], 3}, {&S24847[17], 2}, {&S24847[20], 13}, {&S24847[34], 6}, {&S24847[62], 17}, {NULL}};
const Segment SGI276612[] = {{&S24847[115], 10}, {NULL}};
const Segment SGI276613[] = {{&S21151[100], 4}, {NULL}};
const Segment SGI276614[] = {{&S21151[13], 14}, {&S21151[28], 6}, {&S21151[55], 17}, {NULL}};
const Segment SGI276615[] = {{&S21151[74], 11}, {NULL}};
const Segment SGI276616[] = {{&S21152[93], 4}, {NULL}};
const Segment SGI276617[] = {{&S21152[13], 14}, {&S21152[48], 17}, {NULL}};
const Segment SGI276618[] = {{&S21152[67], 11}, {NULL}};
const Segment SGI276619[] = {{&S21153[93], 4}, {NULL}};
const Segment SGI276620[] = {{&S21153[13], 14}, {&S21153[48], 17}, {&S21153[119], 8}, {NULL}};
const Segment SGI276621[] = {{&S21153[67], 11}, {NULL}};
const Segment SGI276622[] = {{&S21154[100], 4}, {NULL}};
const Segment SGI276623[] = {{&S21154[13], 14}, {&S21154[28], 6}, {&S21154[55], 17}, {&S21154[126], 8}, {NULL}};
const Segment SGI276624[] = {{&S21154[74], 11}, {NULL}};
const Segment SGI276625[] = {{&S21156[93], 4}, {NULL}};
const Segment SGI276626[] = {{&S21156[13], 14}, {&S21156[48], 17}, {NULL}};
const Segment SGI276627[] = {{&S21156[67], 11}, {NULL}};
const Segment SGI276628[] = {{&S21157[100], 4}, {NULL}};
const Segment SGI276629[] = {{&S21157[13], 14}, {&S21157[28], 6}, {&S21157[55], 17}, {NULL}};
const Segment SGI276630[] = {{&S21157[74], 11}, {NULL}};
const Segment SGI276631[] = {{&S21158[93], 4}, {NULL}};
const Segment SGI276632[] = {{&S21158[13], 14}, {&S21158[48], 17}, {&S21158[119], 8}, {NULL}};
const Segment SGI276633[] = {{&S21158[67], 11}, {NULL}};
const Segment SGI276634[] = {{&S21159[100], 4}, {NULL}};
const Segment SGI276635[] = {{&S21159[13], 14}, {&S21159[28], 6}, {&S21159[55], 17}, {&S21159[126], 8}, {NULL}};
const Segment SGI276636[] = {{&S21159[74], 11}, {NULL}};
const Segment SGI276637[] = {{&S21160[99], 4}, {NULL}};
const Segment SGI276638[] = {{&S21160[13], 14}, {&S21160[28], 6}, {&S21160[55], 16}, {NULL}};
const Segment SGI276639[] = {{&S21160[73], 11}, {NULL}};
const Segment SGI276640[] = {{&S21277[78], 4}, {NULL}};
const Segment SGI276641[] = {{&S21277[13], 3}, {&S21277[17], 13}, {&S21277[52], 17}, {&S21277[105], 8}, {NULL}};
const Segment SGI276642[] = {{&S21278[78], 4}, {NULL}};
const Segment SGI276643[] = {{&S21278[13], 3}, {&S21278[17], 13}, {&S21278[52], 17}, {NULL}};
const Segment SGI276644[] = {{&S21279[76], 4}, {NULL}};
const Segment SGI276645[] = {{&S21279[13], 3}, {&S21279[17], 11}, {&S21279[50], 17}, {NULL}};
const Segment SGI276646[] = {{&S21280[85], 4}, {NULL}};
const Segment SGI276647[] = {{&S21280[13], 3}, {&S21280[17], 19}, {&S21280[59], 17}, {NULL}};
const Segment SGI276648[] = {{&S21281[76], 4}, {NULL}};
const Segment SGI276649[] = {{&S21281[13], 3}, {&S21281[17], 13}, {&S21281[52], 17}, {NULL}};
const Segment SGI276650[] = {{&S21281[71], 9}, {NULL}};
const Segment SGI276651[] = {{&S21282[74], 4}, {NULL}};
const Segment SGI276652[] = {{&S21282[13], 3}, {&S21282[17], 11}, {&S21282[50], 17}, {NULL}};
const Segment SGI276653[] = {{&S21282[69], 9}, {NULL}};
const Segment SGI276654[] = {{&S21283[76], 4}, {NULL}};
const Segment SGI276655[] = {{&S21283[13], 3}, {&S21283[17], 11}, {&S21283[50], 17}, {&S21283[104], 8}, {NULL}};
const Segment SGI276656[] = {{&S21403[92], 4}, {NULL}};
const Segment SGI276657[] = {{&S21403[13], 9}, {&S21403[23], 22}, {&S21403[66], 17}, {NULL}};
const Segment SGI276658[] = {{&S21404[92], 4}, {NULL}};
const Segment SGI276659[] = {{&S21404[13], 9}, {&S21404[23], 22}, {&S21404[66], 17}, {NULL}};
const Segment SGI276660[] = {{&S21405[92], 4}, {NULL}};
const Segment SGI276661[] = {{&S21405[13], 9}, {&S21405[23], 22}, {&S21405[66], 17}, {NULL}};
const Segment SGI276662[] = {{&S21406[92], 4}, {NULL}};
const Segment SGI276663[] = {{&S21406[13], 9}, {&S21406[23], 22}, {&S21406[66], 17}, {&S21406[118], 8}, {&S21406[129], 24}, {NULL}};
const Segment SGI276664[] = {{&S21407[92], 4}, {NULL}};
const Segment SGI276665[] = {{&S21407[13], 9}, {&S21407[23], 22}, {&S21407[66], 17}, {&S21407[118], 8}, {&S21407[129], 24}, {NULL}};
const Segment SGI276666[] = {{&S21408[92], 4}, {NULL}};
const Segment SGI276667[] = {{&S21408[13], 9}, {&S21408[23], 22}, {&S21408[66], 17}, {&S21408[118], 8}, {&S21408[129], 24}, {NULL}};
const Segment SGI276668[] = {{&S21409[92], 4}, {NULL}};
const Segment SGI276669[] = {{&S21409[13], 9}, {&S21409[23], 22}, {&S21409[66], 17}, {NULL}};
const Segment SGI276670[] = {{&S21410[89], 4}, {NULL}};
const Segment SGI276671[] = {{&S21410[13], 9}, {&S21410[23], 20}, {&S21410[64], 16}, {NULL}};
const Segment SGI276672[] = {{&S21411[89], 4}, {NULL}};
const Segment SGI276673[] = {{&S21411[13], 9}, {&S21411[23], 20}, {&S21411[64], 16}, {NULL}};
const Segment SGI276674[] = {{&S21412[89], 4}, {NULL}};
const Segment SGI276675[] = {{&S21412[13], 9}, {&S21412[23], 20}, {&S21412[64], 16}, {NULL}};
const Segment SGI276676[] = {{&S21413[92], 4}, {NULL}};
const Segment SGI276677[] = {{&S21413[13], 9}, {&S21413[23], 22}, {&S21413[66], 17}, {NULL}};
const Segment SGI276678[] = {{&S21414[92], 4}, {NULL}};
const Segment SGI276679[] = {{&S21414[13], 9}, {&S21414[23], 22}, {&S21414[66], 17}, {NULL}};
const Segment SGI276680[] = {{&S29982[94], 3}, {NULL}};
const Segment SGI276681[] = {{&S29982[13], 7}, {&S29982[21], 2}, {&S29982[24], 15}, {&S29982[40], 6}, {&S29982[68], 17}, {NULL}};
const Segment SGI276682[] = {{&S29983[94], 3}, {NULL}};
const Segment SGI276683[] = {{&S29983[13], 7}, {&S29983[21], 2}, {&S29983[24], 15}, {&S29983[40], 6}, {&S29983[68], 17}, {NULL}};
const Segment SGI276684[] = {{&S29984[13], 7}, {&S29984[21], 2}, {&S29984[24], 15}, {&S29984[40], 6}, {&S29984[68], 17}, {NULL}};
const Segment SGI276685[] = {{&S29985[94], 3}, {NULL}};
const Segment SGI276686[] = {{&S29985[13], 7}, {&S29985[21], 2}, {&S29985[24], 15}, {&S29985[40], 6}, {&S29985[68], 17}, {NULL}};
const Segment SGI276687[] = {{&S29986[94], 3}, {NULL}};
const Segment SGI276688[] = {{&S29986[13], 7}, {&S29986[21], 2}, {&S29986[24], 15}, {&S29986[40], 6}, {&S29986[68], 17}, {NULL}};
const Segment SGI276689[] = {{&S29987[87], 3}, {NULL}};
const Segment SGI276690[] = {{&S29987[13], 7}, {&S29987[21], 2}, {&S29987[24], 15}, {&S29987[61], 17}, {NULL}};
const Segment SGI276691[] = {{&S29988[94], 3}, {NULL}};
const Segment SGI276692[] = {{&S29988[13], 7}, {&S29988[21], 2}, {&S29988[24], 15}, {&S29988[40], 6}, {&S29988[68], 17}, {NULL}};
const Segment SGI276693[] = {{&S29989[94], 3}, {NULL}};
const Segment SGI276694[] = {{&S29989[13], 7}, {&S29989[21], 2}, {&S29989[24], 15}, {&S29989[40], 6}, {&S29989[68], 17}, {NULL}};
const Segment SGI276695[] = {{&S29990[75], 3}, {NULL}};
const Segment SGI276696[] = {{&S29990[13], 14}, {&S29990[49], 17}, {NULL}};
const Segment SGI276697[] = {{&S29991[13], 7}, {&S29991[21], 2}, {&S29991[24], 15}, {&S29991[40], 6}, {&S29991[68], 17}, {NULL}};
const Segment SGI276698[] = {{&S29992[91], 3}, {NULL}};
const Segment SGI276699[] = {{&S29992[13], 7}, {&S29992[21], 2}, {&S29992[24], 15}, {&S29992[40], 3}, {&S29992[65], 17}, {NULL}};
const Segment SGI276700[] = {{&S29993[91], 3}, {NULL}};
const Segment SGI276701[] = {{&S29993[13], 7}, {&S29993[21], 2}, {&S29993[24], 15}, {&S29993[40], 3}, {&S29993[65], 17}, {NULL}};
const Segment SGI276702[] = {{&S29994[94], 3}, {NULL}};
const Segment SGI276703[] = {{&S29994[13], 7}, {&S29994[21], 2}, {&S29994[24], 15}, {&S29994[40], 6}, {&S29994[68], 17}, {NULL}};
const Segment SGI276704[] = {{&S29995[13], 7}, {&S29995[21], 2}, {&S29995[24], 15}, {&S29995[40], 6}, {&S29995[68], 17}, {NULL}};
const Segment SGI276705[] = {{&S29996[13], 7}, {&S29996[21], 2}, {&S29996[24], 15}, {&S29996[40], 6}, {&S29996[68], 17}, {NULL}};
const Segment SGI276706[] = {{&S29997[13], 7}, {&S29997[21], 2}, {&S29997[24], 15}, {&S29997[40], 6}, {&S29997[68], 17}, {NULL}};
const Segment SGI276707[] = {{&S29998[13], 7}, {&S29998[21], 2}, {&S29998[24], 15}, {&S29998[40], 6}, {&S29998[68], 17}, {&S29998[133], 35}, {&S29998[169], 21}, {NULL}};
const Segment SGI276708[] = {{&S29999[95], 3}, {NULL}};
const Segment SGI276709[] = {{&S29999[13], 7}, {&S29999[21], 2}, {&S29999[24], 15}, {&S29999[40], 6}, {&S29999[47], 19}, {&S29999[67], 19}, {&S29999[87], 32}, {NULL}};
const Segment SGI276710[] = {{&S30000[13], 7}, {&S30000[21], 2}, {&S30000[24], 15}, {&S30000[62], 17}, {NULL}};
const Segment SGI276711[] = {{&S21703[92], 4}, {NULL}};
const Segment SGI276712[] = {{&S21703[13], 9}, {&S21703[23], 22}, {&S21703[66], 17}, {NULL}};
const Segment SGI276713[] = {{&S21704[92], 4}, {NULL}};
const Segment SGI276714[] = {{&S21704[13], 9}, {&S21704[23], 22}, {&S21704[66], 17}, {NULL}};
const Segment SGI276715[] = {{&S21705[92], 4}, {NULL}};
const Segment SGI276716[] = {{&S21705[13], 9}, {&S21705[23], 22}, {&S21705[66], 17}, {NULL}};
const Segment SGI276717[] = {{&S21706[92], 4}, {NULL}};
const Segment SGI276718[] = {{&S21706[13], 9}, {&S21706[23], 22}, {&S21706[66], 17}, {NULL}};
const Segment SGI276719[] = {{&S21707[92], 4}, {NULL}};
const Segment SGI276720[] = {{&S21707[13], 9}, {&S21707[23], 22}, {&S21707[66], 17}, {NULL}};
const Segment SGI276721[] = {{&S21708[92], 4}, {NULL}};
const Segment SGI276722[] = {{&S21708[13], 9}, {&S21708[23], 22}, {&S21708[66], 17}, {NULL}};
const Segment SGI276723[] = {{&S21709[92], 4}, {NULL}};
const Segment SGI276724[] = {{&S21709[13], 9}, {&S21709[23], 22}, {&S21709[66], 17}, {NULL}};
const Segment SGI276725[] = {{&S21710[92], 4}, {NULL}};
const Segment SGI276726[] = {{&S21710[13], 9}, {&S21710[23], 22}, {&S21710[66], 17}, {NULL}};
const Segment SGI276727[] = {{&S21711[92], 4}, {NULL}};
const Segment SGI276728[] = {{&S21711[13], 9}, {&S21711[23], 22}, {&S21711[66], 17}, {NULL}};
const Segment SGI276729[] = {{&S21712[92], 4}, {NULL}};
const Segment SGI276730[] = {{&S21712[13], 9}, {&S21712[23], 22}, {&S21712[66], 17}, {&S21712[118], 8}, {&S21712[129], 24}, {NULL}};
const Segment SGI276731[] = {{&S21713[92], 4}, {NULL}};
const Segment SGI276732[] = {{&S21713[13], 9}, {&S21713[23], 22}, {&S21713[66], 17}, {&S21713[118], 8}, {NULL}};
const Segment SGI276733[] = {{&S21714[92], 4}, {NULL}};
const Segment SGI276734[] = {{&S21714[13], 9}, {&S21714[23], 22}, {&S21714[66], 17}, {&S21714[118], 8}, {&S21714[129], 26}, {NULL}};
const Segment SGI276735[] = {{&S21715[89], 4}, {NULL}};
const Segment SGI276736[] = {{&S21715[13], 9}, {&S21715[23], 20}, {&S21715[64], 16}, {NULL}};
const Segment SGI276737[] = {{&S21716[103], 4}, {NULL}};
const Segment SGI276738[] = {{&S21716[13], 9}, {&S21716[23], 2}, {&S21716[26], 22}, {&S21716[49], 6}, {&S21716[77], 17}, {NULL}};
const Segment SGI276739[] = {{&S21717[103], 4}, {NULL}};
const Segment SGI276740[] = {{&S21717[13], 9}, {&S21717[23], 2}, {&S21717[26], 22}, {&S21717[49], 6}, {&S21717[77], 17}, {NULL}};
const Segment SGI276741[] = {{&S21718[103], 4}, {NULL}};
const Segment SGI276742[] = {{&S21718[13], 9}, {&S21718[23], 2}, {&S21718[26], 22}, {&S21718[49], 6}, {&S21718[77], 17}, {NULL}};
const Segment SGI276743[] = {{&S21719[92], 4}, {NULL}};
const Segment SGI276744[] = {{&S21719[13], 9}, {&S21719[23], 22}, {&S21719[66], 17}, {&S21719[130], 4}, {NULL}};
const Segment SGI276745[] = {{&S21720[92], 4}, {NULL}};
const Segment SGI276746[] = {{&S21720[13], 9}, {&S21720[23], 22}, {&S21720[66], 17}, {&S21720[130], 4}, {NULL}};
const Segment SGI276747[] = {{&S30189[93], 3}, {NULL}};
const Segment SGI276748[] = {{&S30189[13], 7}, {&S30189[21], 2}, {&S30189[24], 15}, {&S30189[40], 6}, {&S30189[67], 17}, {NULL}};
const Segment SGI276749[] = {{&S30190[93], 3}, {NULL}};
const Segment SGI276750[] = {{&S30190[13], 7}, {&S30190[21], 2}, {&S30190[24], 15}, {&S30190[40], 6}, {&S30190[67], 17}, {NULL}};
const Segment SGI276751[] = {{&S30191[93], 3}, {NULL}};
const Segment SGI276752[] = {{&S30191[13], 7}, {&S30191[21], 2}, {&S30191[24], 15}, {&S30191[40], 6}, {&S30191[67], 17}, {NULL}};
const Segment SGI276753[] = {{&S30192[92], 3}, {NULL}};
const Segment SGI276754[] = {{&S30192[13], 7}, {&S30192[21], 2}, {&S30192[24], 15}, {&S30192[40], 6}, {&S30192[67], 16}, {NULL}};
const Segment SGI276755[] = {{&S30193[116], 3}, {NULL}};
const Segment SGI276756[] = {{&S30193[13], 7}, {&S30193[21], 2}, {&S30193[24], 15}, {&S30193[40], 6}, {&S30193[47], 20}, {&S30193[11], 1}, {&S30193[90], 17}, {NULL}};
const Segment SGI276757[] = {{&S25501[94], 4}, {NULL}};
const Segment SGI276758[] = {{&S25501[13], 14}, {&S25501[28], 6}, {&S25501[56], 17}, {NULL}};
const Segment SGI276759[] = {{&S25501[75], 11}, {NULL}};
const Segment SGI276760[] = {{&S25502[87], 4}, {NULL}};
const Segment SGI276761[] = {{&S25502[13], 14}, {&S25502[49], 17}, {NULL}};
const Segment SGI276762[] = {{&S25502[68], 11}, {NULL}};
const Segment SGI276763[] = {{&S25504[87], 4}, {NULL}};
const Segment SGI276764[] = {{&S25504[13], 14}, {&S25504[49], 17}, {&S25504[112], 8}, {&S25504[123], 24}, {NULL}};
const Segment SGI276765[] = {{&S25504[68], 11}, {NULL}};
const Segment SGI276766[] = {{&S25507[94], 4}, {NULL}};
const Segment SGI276767[] = {{&S25507[13], 14}, {&S25507[28], 6}, {&S25507[56], 17}, {&S25507[119], 8}, {&S25507[130], 24}, {NULL}};
const Segment SGI276768[] = {{&S25507[75], 11}, {NULL}};
const Segment SGI276769[] = {{&S25508[106], 4}, {NULL}};
const Segment SGI276770[] = {{&S25508[13], 7}, {&S25508[21], 2}, {&S25508[24], 15}, {&S25508[40], 6}, {&S25508[68], 17}, {NULL}};
const Segment SGI276771[] = {{&S25508[87], 11}, {NULL}};
const Segment SGI276772[] = {{&S25509[94], 4}, {NULL}};
const Segment SGI276773[] = {{&S25509[13], 14}, {&S25509[28], 6}, {&S25509[56], 17}, {&S25509[119], 8}, {NULL}};
const Segment SGI276774[] = {{&S25509[75], 11}, {NULL}};
const Segment SGI276775[] = {{&S25510[87], 4}, {NULL}};
const Segment SGI276776[] = {{&S25510[13], 14}, {&S25510[49], 17}, {&S25510[112], 8}, {NULL}};
const Segment SGI276777[] = {{&S25510[68], 11}, {NULL}};
const Segment SGI276778[] = {{&S25513[94], 4}, {NULL}};
const Segment SGI276779[] = {{&S25513[13], 14}, {&S25513[28], 6}, {&S25513[56], 17}, {&S25513[119], 8}, {&S25513[130], 26}, {NULL}};
const Segment SGI276780[] = {{&S25513[75], 11}, {NULL}};
const Segment SGI276781[] = {{&S25514[87], 4}, {NULL}};
const Segment SGI276782[] = {{&S25514[13], 14}, {&S25514[49], 17}, {&S25514[112], 8}, {&S25514[123], 26}, {NULL}};
const Segment SGI276783[] = {{&S25514[68], 11}, {NULL}};
const Segment SGI276784[] = {{&S25780[93], 4}, {NULL}};
const Segment SGI276785[] = {{&S25780[13], 9}, {&S25780[23], 22}, {&S25780[67], 17}, {NULL}};
const Segment SGI276786[] = {{&S25781[93], 4}, {NULL}};
const Segment SGI276787[] = {{&S25781[13], 9}, {&S25781[23], 22}, {&S25781[67], 17}, {NULL}};
const Segment SGI276788[] = {{&S25782[93], 4}, {NULL}};
const Segment SGI276789[] = {{&S25782[13], 9}, {&S25782[23], 22}, {&S25782[67], 17}, {NULL}};
const Segment SGI276790[] = {{&S25783[93], 4}, {NULL}};
const Segment SGI276791[] = {{&S25783[13], 9}, {&S25783[23], 22}, {&S25783[67], 17}, {NULL}};
const Segment SGI276792[] = {{&S25784[93], 4}, {NULL}};
const Segment SGI276793[] = {{&S25784[13], 9}, {&S25784[23], 22}, {&S25784[67], 17}, {NULL}};
const Segment SGI276794[] = {{&S25785[93], 4}, {NULL}};
const Segment SGI276795[] = {{&S25785[13], 9}, {&S25785[23], 22}, {&S25785[67], 17}, {NULL}};
const Segment SGI276796[] = {{&S25786[93], 4}, {NULL}};
const Segment SGI276797[] = {{&S25786[13], 9}, {&S25786[23], 22}, {&S25786[67], 17}, {NULL}};
const Segment SGI276798[] = {{&S25787[93], 4}, {NULL}};
const Segment SGI276799[] = {{&S25787[13], 9}, {&S25787[23], 22}, {&S25787[67], 17}, {NULL}};
const Segment SGI276800[] = {{&S25788[93], 4}, {NULL}};
const Segment SGI276801[] = {{&S25788[13], 9}, {&S25788[23], 22}, {&S25788[67], 17}, {NULL}};
const Segment SGI276802[] = {{&S25789[93], 4}, {NULL}};
const Segment SGI276803[] = {{&S25789[13], 9}, {&S25789[23], 22}, {&S25789[67], 17}, {&S25789[120], 8}, {&S25789[131], 24}, {NULL}};
const Segment SGI276804[] = {{&S25790[103], 4}, {NULL}};
const Segment SGI276805[] = {{&S25790[13], 9}, {&S25790[23], 2}, {&S25790[26], 22}, {&S25790[49], 6}, {&S25790[77], 17}, {NULL}};
const Segment SGI276806[] = {{&S25791[103], 4}, {NULL}};
const Segment SGI276807[] = {{&S25791[13], 9}, {&S25791[23], 2}, {&S25791[26], 22}, {&S25791[49], 6}, {&S25791[77], 17}, {NULL}};
const Segment SGI276808[] = {{&S25792[103], 4}, {NULL}};
const Segment SGI276809[] = {{&S25792[13], 9}, {&S25792[23], 2}, {&S25792[26], 22}, {&S25792[49], 6}, {&S25792[77], 17}, {NULL}};
const Segment SGI276810[] = {{&S25793[103], 4}, {NULL}};
const Segment SGI276811[] = {{&S25793[13], 9}, {&S25793[23], 2}, {&S25793[26], 22}, {&S25793[49], 6}, {&S25793[77], 17}, {NULL}};
const Segment SGI276812[] = {{&S25794[81], 4}, {NULL}};
const Segment SGI276813[] = {{&S25794[13], 14}, {&S25794[28], 6}, {&S25794[55], 17}, {&S25794[106], 8}, {&S25794[117], 24}, {NULL}};
const Segment SGI276814[] = {{&S25795[74], 4}, {NULL}};
const Segment SGI276815[] = {{&S25795[13], 14}, {&S25795[48], 17}, {&S25795[99], 8}, {&S25795[110], 24}, {NULL}};
const Segment SGI276816[] = {{&S25796[74], 4}, {NULL}};
const Segment SGI276817[] = {{&S25796[13], 14}, {&S25796[48], 17}, {NULL}};
const Segment SGI276818[] = {{&S25797[81], 4}, {NULL}};
const Segment SGI276819[] = {{&S25797[13], 14}, {&S25797[28], 6}, {&S25797[55], 17}, {NULL}};
const Segment SGI276820[] = {{&S25798[93], 4}, {NULL}};
const Segment SGI276821[] = {{&S25798[13], 7}, {&S25798[21], 2}, {&S25798[24], 15}, {&S25798[40], 6}, {&S25798[67], 17}, {NULL}};
const Segment SGI276822[] = {{&S25799[81], 4}, {NULL}};
const Segment SGI276823[] = {{&S25799[13], 14}, {&S25799[28], 6}, {&S25799[55], 17}, {NULL}};
const Segment SGI276824[] = {{&S25800[74], 4}, {NULL}};
const Segment SGI276825[] = {{&S25800[13], 14}, {&S25800[48], 17}, {NULL}};
const Segment SGI276826[] = {{&S22262[81], 4}, {NULL}};
const Segment SGI276827[] = {{&S22262[13], 14}, {&S22262[28], 6}, {&S22262[55], 17}, {NULL}};
const Segment SGI276828[] = {{&S22263[74], 4}, {NULL}};
const Segment SGI276829[] = {{&S22263[13], 14}, {&S22263[48], 17}, {NULL}};
const Segment SGI276830[] = {{&S22264[81], 4}, {NULL}};
const Segment SGI276831[] = {{&S22264[13], 14}, {&S22264[28], 6}, {&S22264[55], 17}, {&S22264[107], 8}, {&S22264[118], 24}, {NULL}};
const Segment SGI276832[] = {{&S22265[74], 4}, {NULL}};
const Segment SGI276833[] = {{&S22265[13], 14}, {&S22265[48], 17}, {&S22265[100], 8}, {NULL}};
const Segment SGI276834[] = {{&S22266[81], 4}, {NULL}};
const Segment SGI276835[] = {{&S22266[13], 14}, {&S22266[28], 6}, {&S22266[55], 17}, {&S22266[107], 8}, {NULL}};
const Segment SGI276836[] = {{&S22267[74], 4}, {NULL}};
const Segment SGI276837[] = {{&S22267[13], 14}, {&S22267[48], 17}, {&S22267[112], 4}, {NULL}};
const Segment SGI276838[] = {{&S22268[81], 4}, {NULL}};
const Segment SGI276839[] = {{&S22268[13], 14}, {&S22268[28], 6}, {&S22268[55], 17}, {&S22268[119], 4}, {NULL}};
const Segment SGI276840[] = {{&S22269[81], 4}, {NULL}};
const Segment SGI276841[] = {{&S22269[13], 14}, {&S22269[28], 6}, {&S22269[55], 17}, {&S22269[119], 4}, {NULL}};
const Segment SGI276842[] = {{&S22270[74], 4}, {NULL}};
const Segment SGI276843[] = {{&S22270[13], 14}, {&S22270[48], 17}, {&S22270[112], 4}, {NULL}};
const Segment SGI276844[] = {{&S22271[81], 4}, {NULL}};
const Segment SGI276845[] = {{&S22271[13], 14}, {&S22271[28], 6}, {&S22271[55], 17}, {&S22271[119], 4}, {NULL}};
const Segment SGI276846[] = {{&S22272[74], 4}, {NULL}};
const Segment SGI276847[] = {{&S22272[13], 14}, {&S22272[48], 17}, {&S22272[112], 4}, {NULL}};
const Segment SGI276848[] = {{&S22273[81], 4}, {NULL}};
const Segment SGI276849[] = {{&S22273[13], 14}, {&S22273[28], 6}, {&S22273[55], 17}, {NULL}};
const Segment SGI276850[] = {{&S22274[74], 4}, {NULL}};
const Segment SGI276851[] = {{&S22274[13], 14}, {&S22274[48], 17}, {NULL}};
const Segment SGI276852[] = {{&S26015[78], 4}, {NULL}};
const Segment SGI276853[] = {{&S26015[13], 3}, {&S26015[17], 13}, {&S26015[52], 17}, {NULL}};
const Segment SGI276854[] = {{&S26016[85], 4}, {NULL}};
const Segment SGI276855[] = {{&S26016[13], 3}, {&S26016[17], 19}, {&S26016[59], 17}, {NULL}};
const Segment SGI276856[] = {{&S26017[76], 4}, {NULL}};
const Segment SGI276857[] = {{&S26017[13], 3}, {&S26017[17], 11}, {&S26017[50], 17}, {NULL}};
const Segment SGI276858[] = {{&S26018[76], 4}, {NULL}};
const Segment SGI276859[] = {{&S26018[13], 3}, {&S26018[17], 11}, {&S26018[50], 17}, {&S26018[104], 8}, {&S26018[115], 24}, {NULL}};
const Segment SGI276860[] = {{&S26019[100], 4}, {NULL}};
const Segment SGI276861[] = {{&S26019[13], 3}, {&S26019[17], 21}, {&S26019[60], 17}, {NULL}};
const Segment SGI276862[] = {{&S26020[88], 4}, {NULL}};
const Segment SGI276863[] = {{&S26020[25], 3}, {&S26020[29], 11}, {&S26020[62], 17}, {NULL}};
const Segment SGI276864[] = {{&S26020[0], 12}, {NULL}};
const Segment SGI276865[] = {{&S26021[78], 4}, {NULL}};
const Segment SGI276866[] = {{&S26021[13], 3}, {&S26021[17], 13}, {&S26021[52], 17}, {&S26021[105], 8}, {&S26021[116], 24}, {NULL}};
const Segment SGI276867[] = {{&S26022[78], 4}, {NULL}};
const Segment SGI276868[] = {{&S26022[13], 3}, {&S26022[17], 13}, {&S26022[52], 17}, {&S26022[105], 8}, {NULL}};
const Segment SGI276869[] = {{&S26023[83], 4}, {NULL}};
const Segment SGI276870[] = {{&S26023[13], 3}, {&S26023[17], 13}, {&S26023[31], 4}, {&S26023[57], 17}, {NULL}};
const Segment SGI276871[] = {{&S26024[80], 4}, {NULL}};
const Segment SGI276872[] = {{&S26024[13], 3}, {&S26024[17], 10}, {&S26024[28], 4}, {&S26024[54], 17}, {NULL}};
const Segment SGI276873[] = {{&S26025[80], 4}, {NULL}};
const Segment SGI276874[] = {{&S26025[13], 3}, {&S26025[17], 10}, {&S26025[28], 4}, {&S26025[54], 17}, {NULL}};
const Segment SGI276875[] = {{&S26026[83], 4}, {NULL}};
const Segment SGI276876[] = {{&S26026[13], 3}, {&S26026[17], 13}, {&S26026[31], 4}, {&S26026[57], 17}, {NULL}};
const Segment SGI276877[] = {{&S26027[80], 4}, {NULL}};
const Segment SGI276878[] = {{&S26027[13], 3}, {&S26027[17], 10}, {&S26027[28], 4}, {&S26027[54], 17}, {NULL}};
const Segment SGI276879[] = {{&S26028[83], 4}, {NULL}};
const Segment SGI276880[] = {{&S26028[13], 3}, {&S26028[17], 13}, {&S26028[31], 4}, {&S26028[57], 17}, {NULL}};
const Segment SGI276881[] = {{&S26029[80], 4}, {NULL}};
const Segment SGI276882[] = {{&S26029[13], 3}, {&S26029[17], 10}, {&S26029[28], 4}, {&S26029[54], 17}, {NULL}};
const Segment SGI276883[] = {{&S26030[85], 4}, {NULL}};
const Segment SGI276884[] = {{&S26030[13], 3}, {&S26030[17], 13}, {&S26030[31], 6}, {&S26030[59], 17}, {NULL}};
const Segment SGI276885[] = {{&S26031[99], 4}, {NULL}};
const Segment SGI276886[] = {{&S26031[13], 5}, {&S26031[19], 2}, {&S26031[22], 4}, {&S26031[27], 6}, {&S26031[34], 15}, {&S26031[11], 1}, {&S26031[73], 17}, {NULL}};
const Segment SGI276887[] = {{&S26084[82], 4}, {NULL}};
const Segment SGI276888[] = {{&S26084[13], 14}, {&S26084[28], 6}, {&S26084[56], 17}, {NULL}};
const Segment SGI276889[] = {{&S26085[75], 4}, {NULL}};
const Segment SGI276890[] = {{&S26085[13], 14}, {&S26085[49], 17}, {&S26085[101], 8}, {&S26085[112], 24}, {NULL}};
const Segment SGI276891[] = {{&S26086[75], 4}, {NULL}};
const Segment SGI276892[] = {{&S26086[13], 14}, {&S26086[49], 17}, {&S26086[101], 8}, {&S26086[112], 26}, {NULL}};
const Segment SGI276893[] = {{&S26087[75], 4}, {NULL}};
const Segment SGI276894[] = {{&S26087[13], 14}, {&S26087[49], 17}, {NULL}};
const Segment SGI276895[] = {{&S26088[74], 4}, {NULL}};
const Segment SGI276896[] = {{&S26088[13], 14}, {&S26088[48], 17}, {NULL}};
const Segment SGI276897[] = {{&S26089[74], 4}, {NULL}};
const Segment SGI276898[] = {{&S26089[13], 14}, {&S26089[48], 17}, {&S26089[126], 3}, {&S26089[130], 11}, {&S26089[142], 10}, {NULL}};
const Segment SGI276899[] = {{&S26090[110], 4}, {NULL}};
const Segment SGI276900[] = {{&S26090[13], 10}, {&S26090[24], 9}, {&S26090[34], 15}, {&S26090[50], 12}, {&S26090[84], 17}, {NULL}};
const Segment SGI276901[] = {{&S26091[81], 4}, {NULL}};
const Segment SGI276902[] = {{&S26091[13], 14}, {&S26091[28], 6}, {&S26091[55], 17}, {&S26091[133], 3}, {&S26091[137], 11}, {&S26091[149], 10}, {NULL}};
const Segment SGI276903[] = {{&S26092[75], 4}, {NULL}};
const Segment SGI276904[] = {{&S26092[13], 14}, {&S26092[49], 17}, {&S26092[119], 8}, {NULL}};
const Segment SGI276905[] = {{&S26093[93], 4}, {NULL}};
const Segment SGI276906[] = {{&S26093[13], 9}, {&S26093[23], 22}, {&S26093[67], 17}, {NULL}};
const Segment SGI276907[] = {{&S26094[93], 4}, {NULL}};
const Segment SGI276908[] = {{&S26094[13], 9}, {&S26094[23], 22}, {&S26094[67], 17}, {NULL}};
const Segment SGI276909[] = {{&S26095[93], 5}, {NULL}};
const Segment SGI276910[] = {{&S26095[13], 9}, {&S26095[23], 22}, {&S26095[67], 17}, {NULL}};
const Segment SGI276911[] = {{&S26096[93], 4}, {NULL}};
const Segment SGI276912[] = {{&S26096[13], 9}, {&S26096[23], 22}, {&S26096[67], 17}, {NULL}};
const Segment SGI276913[] = {{&S26097[93], 4}, {NULL}};
const Segment SGI276914[] = {{&S26097[13], 9}, {&S26097[23], 22}, {&S26097[67], 17}, {NULL}};
const Segment SGI276915[] = {{&S26098[93], 4}, {NULL}};
const Segment SGI276916[] = {{&S26098[13], 9}, {&S26098[23], 22}, {&S26098[67], 17}, {NULL}};
const Segment SGI276917[] = {{&S26099[93], 5}, {NULL}};
const Segment SGI276918[] = {{&S26099[13], 9}, {&S26099[23], 22}, {&S26099[67], 17}, {NULL}};
const Segment SGI276919[] = {{&S26100[93], 4}, {NULL}};
const Segment SGI276920[] = {{&S26100[13], 9}, {&S26100[23], 22}, {&S26100[67], 17}, {NULL}};
const Segment SGI276921[] = {{&S26101[93], 5}, {NULL}};
const Segment SGI276922[] = {{&S26101[13], 9}, {&S26101[23], 22}, {&S26101[67], 17}, {NULL}};
const Segment SGI276923[] = {{&S26102[93], 4}, {NULL}};
const Segment SGI276924[] = {{&S26102[13], 9}, {&S26102[23], 22}, {&S26102[67], 17}, {NULL}};
const Segment SGI276925[] = {{&S26103[93], 4}, {NULL}};
const Segment SGI276926[] = {{&S26103[13], 9}, {&S26103[23], 22}, {&S26103[67], 17}, {NULL}};
const Segment SGI276927[] = {{&S26104[92], 4}, {NULL}};
const Segment SGI276928[] = {{&S26104[13], 9}, {&S26104[23], 22}, {&S26104[66], 17}, {NULL}};
const Segment SGI276929[] = {{&S26105[92], 4}, {NULL}};
const Segment SGI276930[] = {{&S26105[13], 9}, {&S26105[23], 22}, {&S26105[66], 17}, {NULL}};
const Segment SGI276931[] = {{&S26106[92], 4}, {NULL}};
const Segment SGI276932[] = {{&S26106[13], 9}, {&S26106[23], 22}, {&S26106[66], 17}, {NULL}};
const Segment SGI276933[] = {{&S26107[92], 4}, {NULL}};
const Segment SGI276934[] = {{&S26107[13], 9}, {&S26107[23], 22}, {&S26107[66], 17}, {&S26107[118], 8}, {&S26107[129], 26}, {NULL}};
const Segment SGI276935[] = {{&S26108[92], 4}, {NULL}};
const Segment SGI276936[] = {{&S26108[13], 9}, {&S26108[23], 22}, {&S26108[66], 17}, {NULL}};
const Segment SGI276937[] = {{&S26109[116], 4}, {NULL}};
const Segment SGI276938[] = {{&S26109[13], 9}, {&S26109[23], 22}, {&S26109[66], 17}, {NULL}};
const Segment SGI276939[] = {{&S26110[116], 4}, {NULL}};
const Segment SGI276940[] = {{&S26110[13], 9}, {&S26110[23], 22}, {&S26110[66], 17}, {NULL}};
const Segment SGI276941[] = {{&S26111[116], 4}, {NULL}};
const Segment SGI276942[] = {{&S26111[13], 9}, {&S26111[23], 22}, {&S26111[66], 17}, {NULL}};
const Segment SGI276943[] = {{&S20967[100], 4}, {NULL}};
const Segment SGI276944[] = {{&S20967[13], 14}, {&S20967[28], 6}, {&S20967[56], 17}, {NULL}};
const Segment SGI276945[] = {{&S20967[75], 11}, {NULL}};
const Segment SGI276946[] = {{&S20968[75], 4}, {NULL}};
const Segment SGI276947[] = {{&S20968[13], 14}, {&S20968[49], 17}, {NULL}};
const Segment SGI276948[] = {{&S20969[82], 4}, {NULL}};
const Segment SGI276949[] = {{&S20969[13], 14}, {&S20969[28], 6}, {&S20969[56], 17}, {NULL}};
const Segment SGI276950[] = {{&S20970[82], 4}, {NULL}};
const Segment SGI276951[] = {{&S20970[13], 14}, {&S20970[28], 6}, {&S20970[56], 17}, {&S20970[109], 8}, {NULL}};
const Segment SGI276952[] = {{&S20971[93], 4}, {NULL}};
const Segment SGI276953[] = {{&S20971[13], 14}, {&S20971[49], 17}, {NULL}};
const Segment SGI276954[] = {{&S20971[68], 11}, {NULL}};
const Segment SGI276955[] = {{&S22617[41], 4}, {NULL}};
const Segment SGI276956[] = {{&S22617[13], 14}, {&S22617[28], 25}, {&S22617[75], 17}, {NULL}};
const Segment SGI276957[] = {{&S22618[48], 4}, {NULL}};
const Segment SGI276958[] = {{&S22618[13], 14}, {&S22618[28], 6}, {&S22618[35], 25}, {&S22618[82], 17}, {NULL}};
const Segment SGI276959[] = {{&S22619[48], 4}, {NULL}};
const Segment SGI276960[] = {{&S22619[13], 14}, {&S22619[28], 6}, {&S22619[35], 25}, {&S22619[82], 17}, {NULL}};
const Segment SGI276961[] = {{&S22620[41], 4}, {NULL}};
const Segment SGI276962[] = {{&S22620[13], 14}, {&S22620[28], 25}, {&S22620[75], 17}, {NULL}};
const Segment SGI276963[] = {{&S22621[41], 4}, {NULL}};
const Segment SGI276964[] = {{&S22621[13], 14}, {&S22621[28], 25}, {&S22621[75], 17}, {NULL}};
const Segment SGI276965[] = {{&S22622[48], 4}, {NULL}};
const Segment SGI276966[] = {{&S22622[13], 14}, {&S22622[28], 6}, {&S22622[35], 25}, {&S22622[82], 17}, {NULL}};
const Segment SGI276967[] = {{&S22626[41], 4}, {NULL}};
const Segment SGI276968[] = {{&S22626[13], 14}, {&S22626[28], 25}, {&S22626[75], 17}, {NULL}};
const Segment SGI276969[] = {{&S22628[48], 4}, {NULL}};
const Segment SGI276970[] = {{&S22628[13], 14}, {&S22628[28], 6}, {&S22628[35], 25}, {&S22628[82], 17}, {NULL}};
const Segment SGI276971[] = {{&S22647[77], 4}, {NULL}};
const Segment SGI276972[] = {{&S22647[13], 3}, {&S22647[17], 13}, {&S22647[51], 17}, {NULL}};
const Segment SGI276973[] = {{&S22648[75], 4}, {NULL}};
const Segment SGI276974[] = {{&S22648[13], 3}, {&S22648[17], 11}, {&S22648[49], 17}, {NULL}};
const Segment SGI276975[] = {{&S22649[84], 4}, {NULL}};
const Segment SGI276976[] = {{&S22649[13], 3}, {&S22649[17], 19}, {&S22649[58], 17}, {NULL}};
const Segment SGI276977[] = {{&S22650[99], 4}, {NULL}};
const Segment SGI276978[] = {{&S22650[13], 3}, {&S22650[17], 21}, {&S22650[59], 17}, {NULL}};
const Segment SGI276979[] = {{&S22651[77], 4}, {NULL}};
const Segment SGI276980[] = {{&S22651[13], 3}, {&S22651[17], 13}, {&S22651[51], 17}, {&S22651[103], 8}, {&S22651[114], 24}, {NULL}};
const Segment SGI276981[] = {{&S22652[75], 4}, {NULL}};
const Segment SGI276982[] = {{&S22652[13], 3}, {&S22652[17], 11}, {&S22652[49], 17}, {&S22652[101], 8}, {&S22652[112], 24}, {NULL}};
const Segment SGI276983[] = {{&S22653[79], 4}, {NULL}};
const Segment SGI276984[] = {{&S22653[14], 3}, {&S22653[18], 13}, {&S22653[53], 17}, {NULL}};
const Segment SGI276985[] = {{&S22654[75], 4}, {NULL}};
const Segment SGI276986[] = {{&S22654[13], 3}, {&S22654[17], 11}, {&S22654[49], 17}, {&S22654[101], 8}, {NULL}};
const Segment SGI276987[] = {{&S22655[80], 4}, {NULL}};
const Segment SGI276988[] = {{&S22655[13], 3}, {&S22655[17], 10}, {&S22655[28], 4}, {&S22655[54], 17}, {NULL}};
const Segment SGI276989[] = {{&S22656[83], 4}, {NULL}};
const Segment SGI276990[] = {{&S22656[13], 3}, {&S22656[17], 13}, {&S22656[31], 4}, {&S22656[57], 17}, {NULL}};
const Segment SGI276991[] = {{&S22657[80], 4}, {NULL}};
const Segment SGI276992[] = {{&S22657[13], 3}, {&S22657[17], 10}, {&S22657[28], 4}, {&S22657[54], 17}, {NULL}};
const Segment SGI276993[] = {{&S22658[83], 4}, {NULL}};
const Segment SGI276994[] = {{&S22658[13], 3}, {&S22658[17], 13}, {&S22658[31], 4}, {&S22658[57], 17}, {NULL}};
const Segment SGI276995[] = {{&S22659[80], 4}, {NULL}};
const Segment SGI276996[] = {{&S22659[13], 3}, {&S22659[17], 10}, {&S22659[28], 4}, {&S22659[54], 17}, {NULL}};
const Segment SGI276997[] = {{&S22660[83], 4}, {NULL}};
const Segment SGI276998[] = {{&S22660[13], 3}, {&S22660[17], 13}, {&S22660[31], 4}, {&S22660[57], 17}, {NULL}};
const Segment SGI276999[] = {{&S22661[80], 4}, {NULL}};
const Segment SGI277000[] = {{&S22661[13], 3}, {&S22661[17], 10}, {&S22661[28], 4}, {&S22661[54], 17}, {NULL}};
const Segment SGI277001[] = {{&S26478[13], 7}, {&S26478[21], 2}, {&S26478[24], 15}, {&S26478[54], 17}, {NULL}};
const Segment SGI277002[] = {{&S26479[82], 4}, {NULL}};
const Segment SGI277003[] = {{&S26479[13], 14}, {&S26479[28], 6}, {&S26479[56], 17}, {NULL}};
const Segment SGI277004[] = {{&S26480[75], 4}, {NULL}};
const Segment SGI277005[] = {{&S26480[13], 14}, {&S26480[49], 17}, {NULL}};
const Segment SGI277006[] = {{&S26481[75], 4}, {NULL}};
const Segment SGI277007[] = {{&S26481[13], 14}, {&S26481[49], 17}, {&S26481[102], 8}, {&S26481[113], 24}, {NULL}};
const Segment SGI277008[] = {{&S26482[76], 4}, {NULL}};
const Segment SGI277009[] = {{&S26482[14], 14}, {&S26482[50], 17}, {NULL}};
const Segment SGI277010[] = {{&S26482[0], 1}, {NULL}};
const Segment SGI277011[] = {{&S26483[75], 4}, {NULL}};
const Segment SGI277012[] = {{&S26483[13], 14}, {&S26483[49], 17}, {&S26483[87], 17}, {NULL}};
const Segment SGI277013[] = {{&S26484[93], 4}, {NULL}};
const Segment SGI277014[] = {{&S26484[13], 7}, {&S26484[21], 2}, {&S26484[24], 15}, {&S26484[40], 6}, {&S26484[67], 17}, {NULL}};
const Segment SGI277015[] = {{&S26485[82], 4}, {NULL}};
const Segment SGI277016[] = {{&S26485[13], 14}, {&S26485[28], 6}, {&S26485[56], 17}, {NULL}};
const Segment SGI277017[] = {{&S26486[75], 4}, {NULL}};
const Segment SGI277018[] = {{&S26486[13], 14}, {&S26486[28], 38}, {&S26486[102], 15}, {&S26486[118], 12}, {NULL}};
const Segment SGI277019[] = {{&S26487[91], 4}, {NULL}};
const Segment SGI277020[] = {{&S26487[13], 7}, {&S26487[21], 2}, {&S26487[24], 15}, {&S26487[40], 3}, {&S26487[65], 17}, {NULL}};
const Segment SGI277021[] = {{&S26488[82], 4}, {NULL}};
const Segment SGI277022[] = {{&S26488[13], 14}, {&S26488[28], 6}, {&S26488[56], 17}, {&S26488[108], 8}, {&S26488[119], 26}, {NULL}};
const Segment SGI277023[] = {{&S26489[74], 4}, {NULL}};
const Segment SGI277024[] = {{&S26489[13], 14}, {&S26489[49], 16}, {NULL}};
const Segment SGI277025[] = {{&S26490[75], 4}, {NULL}};
const Segment SGI277026[] = {{&S26490[13], 14}, {&S26490[28], 38}, {&S26490[102], 15}, {&S26490[118], 12}, {NULL}};
const Segment SGI277027[] = {{&S22720[84], 4}, {NULL}};
const Segment SGI277028[] = {{&S22720[13], 3}, {&S22720[17], 19}, {&S22720[58], 17}, {NULL}};
const Segment SGI277029[] = {{&S22721[86], 4}, {NULL}};
const Segment SGI277030[] = {{&S22721[13], 3}, {&S22721[17], 21}, {&S22721[60], 17}, {NULL}};
const Segment SGI277031[] = {{&S22722[84], 4}, {NULL}};
const Segment SGI277032[] = {{&S22722[13], 3}, {&S22722[17], 19}, {&S22722[58], 17}, {NULL}};
const Segment SGI277033[] = {{&S22723[86], 4}, {NULL}};
const Segment SGI277034[] = {{&S22723[13], 3}, {&S22723[17], 21}, {&S22723[60], 17}, {NULL}};
const Segment SGI277035[] = {{&S22724[84], 4}, {NULL}};
const Segment SGI277036[] = {{&S22724[13], 3}, {&S22724[17], 19}, {&S22724[58], 17}, {NULL}};
const Segment SGI277037[] = {{&S22725[86], 4}, {NULL}};
const Segment SGI277038[] = {{&S22725[13], 3}, {&S22725[17], 21}, {&S22725[60], 17}, {NULL}};
const Segment SGI277039[] = {{&S22726[30], 4}, {NULL}};
const Segment SGI277040[] = {{&S22726[13], 3}, {&S22726[17], 22}, {&S22726[61], 17}, {NULL}};
const Segment SGI277041[] = {{&S22727[86], 4}, {NULL}};
const Segment SGI277042[] = {{&S22727[13], 3}, {&S22727[17], 21}, {&S22727[60], 17}, {NULL}};
const Segment SGI277043[] = {{&S22728[84], 4}, {NULL}};
const Segment SGI277044[] = {{&S22728[13], 3}, {&S22728[17], 19}, {&S22728[58], 17}, {NULL}};
const Segment SGI277045[] = {{&S22729[86], 4}, {NULL}};
const Segment SGI277046[] = {{&S22729[13], 3}, {&S22729[17], 21}, {&S22729[60], 17}, {NULL}};
const Segment SGI277047[] = {{&S22730[86], 4}, {NULL}};
const Segment SGI277048[] = {{&S22730[13], 3}, {&S22730[17], 21}, {&S22730[60], 17}, {NULL}};
const Segment SGI277049[] = {{&S22731[86], 4}, {NULL}};
const Segment SGI277050[] = {{&S22731[13], 3}, {&S22731[17], 21}, {&S22731[60], 17}, {NULL}};
const Segment SGI277051[] = {{&S22732[86], 4}, {NULL}};
const Segment SGI277052[] = {{&S22732[13], 3}, {&S22732[17], 21}, {&S22732[60], 17}, {NULL}};
const Segment SGI277053[] = {{&S22733[84], 4}, {NULL}};
const Segment SGI277054[] = {{&S22733[13], 3}, {&S22733[17], 19}, {&S22733[58], 17}, {NULL}};
const Segment SGI277055[] = {{&S22734[86], 4}, {NULL}};
const Segment SGI277056[] = {{&S22734[13], 3}, {&S22734[17], 21}, {&S22734[60], 17}, {NULL}};
const Segment SGI277057[] = {{&S22735[86], 4}, {NULL}};
const Segment SGI277058[] = {{&S22735[13], 3}, {&S22735[17], 21}, {&S22735[60], 17}, {NULL}};
const Segment SGI277059[] = {{&S22736[84], 4}, {NULL}};
const Segment SGI277060[] = {{&S22736[13], 3}, {&S22736[17], 19}, {&S22736[58], 17}, {NULL}};
const Segment SGI277061[] = {{&S22737[84], 4}, {NULL}};
const Segment SGI277062[] = {{&S22737[13], 3}, {&S22737[17], 19}, {&S22737[58], 17}, {NULL}};
const Segment SGI277063[] = {{&S22738[87], 4}, {NULL}};
const Segment SGI277064[] = {{&S22738[13], 3}, {&S22738[17], 22}, {&S22738[61], 17}, {NULL}};
const Segment SGI277065[] = {{&S22739[85], 4}, {NULL}};
const Segment SGI277066[] = {{&S22739[13], 3}, {&S22739[17], 20}, {&S22739[59], 17}, {NULL}};
const Segment SGI277067[] = {{&S22740[87], 4}, {NULL}};
const Segment SGI277068[] = {{&S22740[13], 3}, {&S22740[17], 22}, {&S22740[61], 17}, {NULL}};
const Segment SGI277069[] = {{&S22741[87], 4}, {NULL}};
const Segment SGI277070[] = {{&S22741[13], 3}, {&S22741[17], 22}, {&S22741[61], 17}, {NULL}};
const Segment SGI277071[] = {{&S22742[85], 4}, {NULL}};
const Segment SGI277072[] = {{&S22742[13], 3}, {&S22742[17], 20}, {&S22742[59], 17}, {NULL}};
const Segment SGI277073[] = {{&S22743[87], 4}, {NULL}};
const Segment SGI277074[] = {{&S22743[13], 3}, {&S22743[17], 22}, {&S22743[61], 17}, {NULL}};
const Segment SGI277075[] = {{&S22744[85], 4}, {NULL}};
const Segment SGI277076[] = {{&S22744[13], 3}, {&S22744[17], 20}, {&S22744[59], 17}, {NULL}};
const Segment SGI277077[] = {{&S22745[85], 4}, {NULL}};
const Segment SGI277078[] = {{&S22745[13], 3}, {&S22745[17], 20}, {&S22745[59], 17}, {NULL}};
const Segment SGI277079[] = {{&S22746[87], 4}, {NULL}};
const Segment SGI277080[] = {{&S22746[13], 3}, {&S22746[17], 22}, {&S22746[61], 17}, {NULL}};
const Segment SGI277081[] = {{&S22747[87], 4}, {NULL}};
const Segment SGI277082[] = {{&S22747[13], 3}, {&S22747[17], 22}, {&S22747[61], 17}, {NULL}};
const Segment SGI277083[] = {{&S22748[85], 4}, {NULL}};
const Segment SGI277084[] = {{&S22748[13], 3}, {&S22748[17], 20}, {&S22748[59], 17}, {NULL}};
const Segment SGI277085[] = {{&S22749[87], 4}, {NULL}};
const Segment SGI277086[] = {{&S22749[13], 3}, {&S22749[17], 22}, {&S22749[61], 17}, {NULL}};
const Segment SGI277087[] = {{&S22750[87], 4}, {NULL}};
const Segment SGI277088[] = {{&S22750[13], 3}, {&S22750[17], 22}, {&S22750[61], 17}, {NULL}};
const Segment SGI277089[] = {{&S22751[85], 4}, {NULL}};
const Segment SGI277090[] = {{&S22751[13], 3}, {&S22751[17], 20}, {&S22751[59], 17}, {NULL}};
const Segment SGI277091[] = {{&S22752[87], 4}, {NULL}};
const Segment SGI277092[] = {{&S22752[13], 3}, {&S22752[17], 22}, {&S22752[61], 17}, {NULL}};
const Segment SGI277093[] = {{&S22753[87], 4}, {NULL}};
const Segment SGI277094[] = {{&S22753[13], 3}, {&S22753[17], 22}, {&S22753[61], 17}, {NULL}};
const Segment SGI277095[] = {{&S22754[85], 4}, {NULL}};
const Segment SGI277096[] = {{&S22754[13], 3}, {&S22754[17], 20}, {&S22754[59], 17}, {NULL}};
const Segment SGI277097[] = {{&S22755[87], 4}, {NULL}};
const Segment SGI277098[] = {{&S22755[13], 3}, {&S22755[17], 22}, {&S22755[61], 17}, {NULL}};
const Segment SGI277099[] = {{&S22756[87], 4}, {NULL}};
const Segment SGI277100[] = {{&S22756[13], 3}, {&S22756[17], 22}, {&S22756[61], 17}, {NULL}};
const Segment SGI277101[] = {{&S22757[85], 4}, {NULL}};
const Segment SGI277102[] = {{&S22757[13], 3}, {&S22757[17], 20}, {&S22757[59], 17}, {NULL}};
const Segment SGI277103[] = {{&S22758[87], 4}, {NULL}};
const Segment SGI277104[] = {{&S22758[13], 3}, {&S22758[17], 22}, {&S22758[61], 17}, {NULL}};
const Segment SGI277105[] = {{&S22759[87], 4}, {NULL}};
const Segment SGI277106[] = {{&S22759[13], 3}, {&S22759[17], 22}, {&S22759[61], 17}, {NULL}};
const Segment SGI277107[] = {{&S22760[85], 4}, {NULL}};
const Segment SGI277108[] = {{&S22760[13], 3}, {&S22760[17], 20}, {&S22760[59], 17}, {NULL}};
const Segment SGI277109[] = {{&S22761[85], 4}, {NULL}};
const Segment SGI277110[] = {{&S22761[13], 3}, {&S22761[17], 20}, {&S22761[59], 17}, {NULL}};
const Segment SGI277111[] = {{&S22762[87], 4}, {NULL}};
const Segment SGI277112[] = {{&S22762[13], 3}, {&S22762[17], 22}, {&S22762[61], 17}, {NULL}};
const Segment SGI277113[] = {{&S22763[85], 4}, {NULL}};
const Segment SGI277114[] = {{&S22763[13], 3}, {&S22763[17], 20}, {&S22763[59], 17}, {NULL}};
const Segment SGI277115[] = {{&S22764[87], 4}, {NULL}};
const Segment SGI277116[] = {{&S22764[13], 3}, {&S22764[17], 22}, {&S22764[61], 17}, {NULL}};
const Segment SGI277117[] = {{&S22765[87], 4}, {NULL}};
const Segment SGI277118[] = {{&S22765[13], 3}, {&S22765[17], 22}, {&S22765[61], 17}, {NULL}};
const Segment SGI277119[] = {{&S22766[87], 4}, {NULL}};
const Segment SGI277120[] = {{&S22766[13], 3}, {&S22766[17], 22}, {&S22766[61], 17}, {NULL}};
const Segment SGI277121[] = {{&S22767[87], 4}, {NULL}};
const Segment SGI277122[] = {{&S22767[13], 3}, {&S22767[17], 22}, {&S22767[61], 17}, {NULL}};
const Segment SGI277123[] = {{&S22768[85], 4}, {NULL}};
const Segment SGI277124[] = {{&S22768[13], 3}, {&S22768[17], 20}, {&S22768[59], 17}, {NULL}};
const Segment SGI277125[] = {{&S22769[88], 4}, {NULL}};
const Segment SGI277126[] = {{&S22769[13], 3}, {&S22769[17], 23}, {&S22769[62], 17}, {NULL}};
const Segment SGI277127[] = {{&S22770[92], 4}, {NULL}};
const Segment SGI277128[] = {{&S22770[13], 9}, {&S22770[23], 22}, {&S22770[66], 17}, {NULL}};
const Segment SGI277129[] = {{&S22771[92], 4}, {NULL}};
const Segment SGI277130[] = {{&S22771[13], 9}, {&S22771[23], 22}, {&S22771[66], 17}, {NULL}};
const Segment SGI277131[] = {{&S22772[92], 4}, {NULL}};
const Segment SGI277132[] = {{&S22772[13], 9}, {&S22772[23], 22}, {&S22772[66], 17}, {NULL}};
const Segment SGI277133[] = {{&S22773[92], 4}, {NULL}};
const Segment SGI277134[] = {{&S22773[13], 9}, {&S22773[23], 22}, {&S22773[66], 17}, {NULL}};
const Segment SGI277135[] = {{&S22774[92], 4}, {NULL}};
const Segment SGI277136[] = {{&S22774[13], 9}, {&S22774[23], 22}, {&S22774[66], 17}, {NULL}};
const Segment SGI277137[] = {{&S22775[92], 4}, {NULL}};
const Segment SGI277138[] = {{&S22775[13], 9}, {&S22775[23], 22}, {&S22775[66], 17}, {&S22775[118], 8}, {&S22775[129], 24}, {NULL}};
const Segment SGI277139[] = {{&S22776[92], 4}, {NULL}};
const Segment SGI277140[] = {{&S22776[13], 9}, {&S22776[23], 22}, {&S22776[66], 17}, {NULL}};
const Segment SGI277141[] = {{&S22777[92], 4}, {NULL}};
const Segment SGI277142[] = {{&S22777[13], 9}, {&S22777[23], 22}, {&S22777[66], 17}, {&S22777[118], 8}, {&S22777[129], 26}, {NULL}};
const Segment SGI277143[] = {{&S41462[38], 5}, {NULL}};
const Segment SGI277144[] = {{&S41462[100], 5}, {NULL}};
const Segment SGI277145[] = {{&S26859[86], 4}, {NULL}};
const Segment SGI277146[] = {{&S26859[13], 3}, {&S26859[17], 2}, {&S26859[20], 11}, {&S26859[32], 6}, {&S26859[60], 17}, {NULL}};
const Segment SGI277147[] = {{&S26860[110], 4}, {NULL}};
const Segment SGI277148[] = {{&S26860[13], 3}, {&S26860[17], 2}, {&S26860[20], 21}, {&S26860[42], 6}, {&S26860[70], 17}, {NULL}};
const Segment SGI277149[] = {{&S26861[115], 4}, {NULL}};
const Segment SGI277150[] = {{&S26861[13], 3}, {&S26861[17], 2}, {&S26861[20], 13}, {&S26861[34], 6}, {&S26861[62], 44}, {NULL}};
const Segment SGI277151[] = {{&S26862[66], 4}, {NULL}};
const Segment SGI277152[] = {{&S26862[13], 3}, {&S26862[17], 2}, {&S26862[20], 13}, {&S26862[34], 6}, {&S26862[78], 17}, {NULL}};
const Segment SGI277153[] = {{&S26863[88], 4}, {NULL}};
const Segment SGI277154[] = {{&S26863[13], 3}, {&S26863[17], 2}, {&S26863[20], 13}, {&S26863[34], 6}, {&S26863[62], 17}, {NULL}};
const Segment SGI277155[] = {{&S26864[95], 4}, {NULL}};
const Segment SGI277156[] = {{&S26864[13], 3}, {&S26864[17], 2}, {&S26864[20], 19}, {&S26864[41], 6}, {&S26864[69], 17}, {NULL}};
const Segment SGI277157[] = {{&S26865[88], 4}, {NULL}};
const Segment SGI277158[] = {{&S26865[13], 1}, {&S26865[15], 11}, {&S26865[27], 6}, {&S26865[34], 6}, {&S26865[62], 17}, {NULL}};
const Segment SGI277159[] = {{&S26866[80], 4}, {NULL}};
const Segment SGI277160[] = {{&S26866[13], 3}, {&S26866[17], 10}, {&S26866[28], 4}, {&S26866[54], 17}, {NULL}};
const Segment SGI277161[] = {{&S26867[83], 4}, {NULL}};
const Segment SGI277162[] = {{&S26867[13], 3}, {&S26867[17], 13}, {&S26867[31], 4}, {&S26867[57], 17}, {NULL}};
const Segment SGI277163[] = {{&S26868[80], 4}, {NULL}};
const Segment SGI277164[] = {{&S26868[13], 3}, {&S26868[17], 10}, {&S26868[28], 4}, {&S26868[54], 17}, {NULL}};
const Segment SGI277165[] = {{&S26869[83], 4}, {NULL}};
const Segment SGI277166[] = {{&S26869[13], 3}, {&S26869[17], 13}, {&S26869[31], 4}, {&S26869[57], 17}, {NULL}};
const Segment SGI277167[] = {{&S26870[80], 4}, {NULL}};
const Segment SGI277168[] = {{&S26870[13], 3}, {&S26870[17], 10}, {&S26870[28], 4}, {&S26870[54], 17}, {NULL}};
const Segment SGI277169[] = {{&S26871[83], 4}, {NULL}};
const Segment SGI277170[] = {{&S26871[13], 3}, {&S26871[17], 13}, {&S26871[31], 4}, {&S26871[57], 17}, {NULL}};
const Segment SGI277171[] = {{&S26872[80], 4}, {NULL}};
const Segment SGI277172[] = {{&S26872[13], 3}, {&S26872[17], 10}, {&S26872[28], 4}, {&S26872[54], 17}, {NULL}};
const Segment SGI277173[] = {{&S23379[86], 4}, {NULL}};
const Segment SGI277174[] = {{&S23379[13], 3}, {&S23379[17], 21}, {&S23379[60], 17}, {NULL}};
const Segment SGI277175[] = {{&S23380[84], 4}, {NULL}};
const Segment SGI277176[] = {{&S23380[13], 3}, {&S23380[17], 19}, {&S23380[58], 17}, {NULL}};
const Segment SGI277177[] = {{&S23381[86], 4}, {NULL}};
const Segment SGI277178[] = {{&S23381[13], 3}, {&S23381[17], 21}, {&S23381[60], 17}, {NULL}};
const Segment SGI277179[] = {{&S23382[84], 4}, {NULL}};
const Segment SGI277180[] = {{&S23382[13], 3}, {&S23382[17], 19}, {&S23382[58], 17}, {NULL}};
const Segment SGI277181[] = {{&S32013[93], 3}, {NULL}};
const Segment SGI277182[] = {{&S32013[13], 7}, {&S32013[21], 2}, {&S32013[24], 15}, {&S32013[40], 6}, {&S32013[67], 17}, {NULL}};
const Segment SGI277183[] = {{&S23383[86], 4}, {NULL}};
const Segment SGI277184[] = {{&S23383[13], 3}, {&S23383[17], 21}, {&S23383[60], 17}, {NULL}};
const Segment SGI277185[] = {{&S32014[93], 3}, {NULL}};
const Segment SGI277186[] = {{&S23384[84], 4}, {NULL}};
const Segment SGI277187[] = {{&S32014[13], 7}, {&S32014[21], 2}, {&S32014[24], 15}, {&S32014[40], 6}, {&S32014[67], 17}, {NULL}};
const Segment SGI277188[] = {{&S23384[13], 3}, {&S23384[17], 19}, {&S23384[58], 17}, {NULL}};
const Segment SGI277189[] = {{&S23385[86], 4}, {NULL}};
const Segment SGI277190[] = {{&S23385[13], 3}, {&S23385[17], 21}, {&S23385[60], 17}, {NULL}};
const Segment SGI277191[] = {{&S32015[87], 3}, {NULL}};
const Segment SGI277192[] = {{&S32015[13], 7}, {&S32015[21], 2}, {&S32015[24], 15}, {&S32015[61], 17}, {NULL}};
const Segment SGI277193[] = {{&S23386[84], 4}, {NULL}};
const Segment SGI277194[] = {{&S23386[13], 3}, {&S23386[17], 19}, {&S23386[58], 17}, {NULL}};
const Segment SGI277195[] = {{&S32016[90], 3}, {NULL}};
const Segment SGI277196[] = {{&S32016[13], 7}, {&S32016[21], 2}, {&S32016[24], 15}, {&S32016[40], 3}, {&S32016[64], 17}, {NULL}};
const Segment SGI277197[] = {{&S23387[86], 4}, {NULL}};
const Segment SGI277198[] = {{&S23387[13], 3}, {&S23387[17], 21}, {&S23387[60], 17}, {NULL}};
const Segment SGI277199[] = {{&S32017[93], 3}, {NULL}};
const Segment SGI277200[] = {{&S32017[13], 7}, {&S32017[21], 2}, {&S32017[24], 15}, {&S32017[40], 6}, {&S32017[67], 17}, {&S32017[134], 35}, {&S32017[170], 18}, {NULL}};
const Segment SGI277201[] = {{&S23388[86], 4}, {NULL}};
const Segment SGI277202[] = {{&S23388[13], 3}, {&S23388[17], 21}, {&S23388[60], 17}, {NULL}};
const Segment SGI277203[] = {{&S32018[75], 3}, {NULL}};
const Segment SGI277204[] = {{&S32018[13], 14}, {&S32018[49], 17}, {NULL}};
const Segment SGI277205[] = {{&S23389[86], 4}, {NULL}};
const Segment SGI277206[] = {{&S23389[13], 3}, {&S23389[17], 21}, {&S23389[60], 17}, {NULL}};
const Segment SGI277207[] = {{&S32019[75], 3}, {NULL}};
const Segment SGI277208[] = {{&S32019[13], 14}, {&S32019[49], 17}, {NULL}};
const Segment SGI277209[] = {{&S23390[84], 4}, {NULL}};
const Segment SGI277210[] = {{&S32030[93], 3}, {NULL}};
const Segment SGI277211[] = {{&S32030[13], 7}, {&S32030[21], 2}, {&S32030[24], 15}, {&S32030[40], 6}, {&S32030[67], 17}, {NULL}};
const Segment SGI277212[] = {{&S32031[93], 3}, {NULL}};
const Segment SGI277213[] = {{&S32031[13], 7}, {&S32031[21], 2}, {&S32031[24], 15}, {&S32031[40], 6}, {&S32031[67], 17}, {NULL}};
const Segment SGI277214[] = {{&S32032[120], 3}, {NULL}};
const Segment SGI277215[] = {{&S32032[13], 7}, {&S32032[21], 2}, {&S32032[24], 15}, {&S32032[40], 6}, {&S32032[47], 24}, {&S32032[11], 1}, {&S32032[94], 17}, {NULL}};
const Segment SGI277216[] = {{&S32033[116], 3}, {NULL}};
const Segment SGI277217[] = {{&S32033[13], 7}, {&S32033[21], 2}, {&S32033[24], 15}, {&S32033[40], 6}, {&S32033[47], 20}, {&S32033[11], 1}, {&S32033[90], 17}, {NULL}};
const Segment SGI277218[] = {{&S32034[93], 3}, {NULL}};
const Segment SGI277219[] = {{&S32034[13], 7}, {&S32034[21], 2}, {&S32034[24], 15}, {&S32034[40], 6}, {&S32034[67], 17}, {NULL}};
const Segment SGI277220[] = {{&S22072[75], 4}, {NULL}};
const Segment SGI277221[] = {{&S22072[13], 14}, {&S22072[49], 17}, {&S22072[118], 35}, {&S22072[154], 19}, {NULL}};
const Segment SGI277222[] = {{&S22073[75], 4}, {NULL}};
const Segment SGI277223[] = {{&S22073[13], 14}, {&S22073[49], 17}, {NULL}};
const Segment SGI277224[] = {{&S22074[75], 4}, {NULL}};
const Segment SGI277225[] = {{&S22074[13], 14}, {&S22074[49], 17}, {&S22074[102], 8}, {NULL}};
const Segment SGI277226[] = {{&S22075[93], 4}, {NULL}};
const Segment SGI277227[] = {{&S22075[13], 14}, {&S22075[49], 17}, {NULL}};
const Segment SGI277228[] = {{&S22075[68], 11}, {NULL}};
const Segment SGI277229[] = {{&S22076[82], 4}, {NULL}};
const Segment SGI277230[] = {{&S22076[13], 14}, {&S22076[28], 6}, {&S22076[56], 17}, {NULL}};
const Segment SGI277231[] = {{&S22077[75], 4}, {NULL}};
const Segment SGI277232[] = {{&S22077[13], 14}, {&S22077[49], 17}, {NULL}};
const Segment SGI277233[] = {{&S22078[75], 4}, {NULL}};
const Segment SGI277234[] = {{&S22078[13], 14}, {&S22078[49], 17}, {&S22078[102], 8}, {&S22078[113], 26}, {NULL}};
const Segment SGI277235[] = {{&S27236[93], 4}, {NULL}};
const Segment SGI277236[] = {{&S27236[13], 9}, {&S27236[23], 22}, {&S27236[67], 17}, {NULL}};
const Segment SGI277237[] = {{&S27237[93], 4}, {NULL}};
const Segment SGI277238[] = {{&S27237[13], 9}, {&S27237[23], 22}, {&S27237[67], 17}, {NULL}};
const Segment SGI277239[] = {{&S27238[93], 4}, {NULL}};
const Segment SGI277240[] = {{&S27238[13], 9}, {&S27238[23], 22}, {&S27238[67], 17}, {NULL}};
const Segment SGI277241[] = {{&S27239[93], 4}, {NULL}};
const Segment SGI277242[] = {{&S27239[13], 9}, {&S27239[23], 22}, {&S27239[67], 17}, {NULL}};
const Segment SGI277243[] = {{&S27240[104], 4}, {NULL}};
const Segment SGI277244[] = {{&S27240[24], 9}, {&S27240[34], 22}, {&S27240[78], 17}, {NULL}};
const Segment SGI277245[] = {{&S27240[0], 11}, {NULL}};
const Segment SGI277246[] = {{&S27241[93], 4}, {NULL}};
const Segment SGI277247[] = {{&S27241[13], 9}, {&S27241[23], 22}, {&S27241[67], 17}, {NULL}};
const Segment SGI277248[] = {{&S27242[93], 4}, {NULL}};
const Segment SGI277249[] = {{&S27242[13], 9}, {&S27242[23], 22}, {&S27242[67], 17}, {NULL}};
const Segment SGI277250[] = {{&S27242[114], 3}, {NULL}};
const Segment SGI277251[] = {{&S27243[93], 4}, {NULL}};
const Segment SGI277252[] = {{&S27243[13], 9}, {&S27243[23], 22}, {&S27243[67], 17}, {NULL}};
const Segment SGI277253[] = {{&S27243[115], 3}, {NULL}};
const Segment SGI277254[] = {{&S27244[93], 4}, {NULL}};
const Segment SGI277255[] = {{&S27244[13], 9}, {&S27244[23], 22}, {&S27244[67], 17}, {NULL}};
const Segment SGI277256[] = {{&S27244[115], 3}, {NULL}};
const Segment SGI277257[] = {{&S27245[93], 4}, {NULL}};
const Segment SGI277258[] = {{&S27245[13], 9}, {&S27245[23], 22}, {&S27245[67], 17}, {NULL}};
const Segment SGI277259[] = {{&S27245[114], 3}, {NULL}};
const Segment SGI277260[] = {{&S27246[93], 4}, {NULL}};
const Segment SGI277261[] = {{&S27246[13], 9}, {&S27246[23], 22}, {&S27246[67], 17}, {NULL}};
const Segment SGI277262[] = {{&S27246[114], 3}, {NULL}};
const Segment SGI277263[] = {{&S27247[93], 4}, {NULL}};
const Segment SGI277264[] = {{&S27247[13], 9}, {&S27247[23], 22}, {&S27247[67], 17}, {NULL}};
const Segment SGI277265[] = {{&S27248[93], 4}, {NULL}};
const Segment SGI277266[] = {{&S27248[13], 9}, {&S27248[23], 22}, {&S27248[67], 17}, {NULL}};
const Segment SGI277267[] = {{&S27248[115], 3}, {NULL}};
const Segment SGI277268[] = {{&S27249[93], 4}, {NULL}};
const Segment SGI277269[] = {{&S27249[13], 9}, {&S27249[23], 22}, {&S27249[67], 17}, {NULL}};
const Segment SGI277270[] = {{&S27249[115], 3}, {NULL}};
const Segment SGI277271[] = {{&S23390[13], 3}, {&S23390[17], 19}, {&S23390[58], 17}, {NULL}};
const Segment SGI277272[] = {{&S23391[85], 4}, {NULL}};
const Segment SGI277273[] = {{&S23391[13], 3}, {&S23391[17], 20}, {&S23391[59], 17}, {NULL}};
const Segment SGI277274[] = {{&S23392[87], 4}, {NULL}};
const Segment SGI277275[] = {{&S23392[13], 3}, {&S23392[17], 22}, {&S23392[61], 17}, {NULL}};
const Segment SGI277276[] = {{&S23393[87], 4}, {NULL}};
const Segment SGI277277[] = {{&S23393[13], 3}, {&S23393[17], 22}, {&S23393[61], 17}, {NULL}};
const Segment SGI277278[] = {{&S23394[87], 4}, {NULL}};
const Segment SGI277279[] = {{&S23394[13], 3}, {&S23394[17], 22}, {&S23394[61], 17}, {NULL}};
const Segment SGI277280[] = {{&S23395[85], 4}, {NULL}};
const Segment SGI277281[] = {{&S23395[13], 3}, {&S23395[17], 20}, {&S23395[59], 17}, {NULL}};
const Segment SGI277282[] = {{&S23396[87], 4}, {NULL}};
const Segment SGI277283[] = {{&S23396[13], 3}, {&S23396[17], 22}, {&S23396[61], 17}, {NULL}};
const Segment SGI277284[] = {{&S23397[85], 4}, {NULL}};
const Segment SGI277285[] = {{&S23397[13], 3}, {&S23397[17], 20}, {&S23397[59], 17}, {NULL}};
const Segment SGI277286[] = {{&S23398[87], 4}, {NULL}};
const Segment SGI277287[] = {{&S23398[13], 3}, {&S23398[17], 22}, {&S23398[61], 17}, {NULL}};
const Segment SGI277288[] = {{&S23399[87], 4}, {NULL}};
const Segment SGI277289[] = {{&S23399[13], 3}, {&S23399[17], 22}, {&S23399[61], 17}, {NULL}};
const Segment SGI277290[] = {{&S23400[87], 4}, {NULL}};
const Segment SGI277291[] = {{&S23400[13], 3}, {&S23400[17], 22}, {&S23400[61], 17}, {NULL}};
const Segment SGI277292[] = {{&S23401[87], 4}, {NULL}};
const Segment SGI277293[] = {{&S23401[13], 3}, {&S23401[17], 22}, {&S23401[61], 17}, {NULL}};
const Segment SGI277294[] = {{&S23402[85], 4}, {NULL}};
const Segment SGI277295[] = {{&S23402[13], 3}, {&S23402[17], 20}, {&S23402[59], 17}, {NULL}};
const Segment SGI277296[] = {{&S23403[87], 4}, {NULL}};
const Segment SGI277297[] = {{&S23403[13], 3}, {&S23403[17], 22}, {&S23403[61], 17}, {NULL}};
const Segment SGI277298[] = {{&S23404[85], 4}, {NULL}};
const Segment SGI277299[] = {{&S23404[13], 3}, {&S23404[17], 20}, {&S23404[59], 17}, {NULL}};
const Segment SGI277300[] = {{&S23405[87], 4}, {NULL}};
const Segment SGI277301[] = {{&S23405[13], 3}, {&S23405[17], 22}, {&S23405[61], 17}, {NULL}};
const Segment SGI277302[] = {{&S23406[87], 4}, {NULL}};
const Segment SGI277303[] = {{&S23406[13], 3}, {&S23406[17], 22}, {&S23406[61], 17}, {NULL}};
const Segment SGI277304[] = {{&S23407[87], 4}, {NULL}};
const Segment SGI277305[] = {{&S23407[13], 3}, {&S23407[17], 22}, {&S23407[61], 17}, {NULL}};
const Segment SGI277306[] = {{&S23408[85], 4}, {NULL}};
const Segment SGI277307[] = {{&S23408[13], 3}, {&S23408[17], 20}, {&S23408[59], 17}, {NULL}};
const Segment SGI277308[] = {{&S23409[87], 4}, {NULL}};
const Segment SGI277309[] = {{&S23409[13], 3}, {&S23409[17], 22}, {&S23409[61], 17}, {NULL}};
const Segment SGI277310[] = {{&S23410[87], 4}, {NULL}};
const Segment SGI277311[] = {{&S23410[13], 3}, {&S23410[17], 22}, {&S23410[61], 17}, {NULL}};
const Segment SGI277312[] = {{&S23411[85], 4}, {NULL}};
const Segment SGI277313[] = {{&S23411[13], 3}, {&S23411[17], 20}, {&S23411[59], 17}, {NULL}};
const Segment SGI277314[] = {{&S23412[87], 4}, {NULL}};
const Segment SGI277315[] = {{&S23412[13], 3}, {&S23412[17], 22}, {&S23412[61], 17}, {NULL}};
const Segment SGI277316[] = {{&S23413[87], 4}, {NULL}};
const Segment SGI277317[] = {{&S23413[13], 3}, {&S23413[17], 22}, {&S23413[61], 17}, {NULL}};
const Segment SGI277318[] = {{&S23414[85], 4}, {NULL}};
const Segment SGI277319[] = {{&S23414[13], 3}, {&S23414[17], 20}, {&S23414[59], 17}, {NULL}};
const Segment SGI277320[] = {{&S23415[87], 4}, {NULL}};
const Segment SGI277321[] = {{&S23415[13], 3}, {&S23415[17], 22}, {&S23415[61], 17}, {NULL}};
const Segment SGI277322[] = {{&S23416[85], 4}, {NULL}};
const Segment SGI277323[] = {{&S23416[13], 3}, {&S23416[17], 20}, {&S23416[59], 17}, {NULL}};
const Segment SGI277324[] = {{&S23417[87], 4}, {NULL}};
const Segment SGI277325[] = {{&S23417[13], 3}, {&S23417[17], 22}, {&S23417[61], 17}, {NULL}};
const Segment SGI277326[] = {{&S23418[87], 4}, {NULL}};
const Segment SGI277327[] = {{&S23418[13], 3}, {&S23418[17], 22}, {&S23418[61], 17}, {NULL}};
const Segment SGI277328[] = {{&S23419[85], 4}, {NULL}};
const Segment SGI277329[] = {{&S23419[13], 3}, {&S23419[17], 20}, {&S23419[59], 17}, {NULL}};
const Segment SGI277330[] = {{&S23420[87], 4}, {NULL}};
const Segment SGI277331[] = {{&S23420[13], 3}, {&S23420[17], 22}, {&S23420[61], 17}, {NULL}};
const Segment SGI277332[] = {{&S23601[41], 4}, {NULL}};
const Segment SGI277333[] = {{&S23601[13], 14}, {&S23601[28], 25}, {&S23601[75], 17}, {NULL}};
const Segment SGI277334[] = {{&S23602[48], 4}, {NULL}};
const Segment SGI277335[] = {{&S23602[13], 14}, {&S23602[28], 6}, {&S23602[35], 25}, {&S23602[82], 17}, {NULL}};
const Segment SGI277336[] = {{&S23607[48], 4}, {NULL}};
const Segment SGI277337[] = {{&S23607[13], 14}, {&S23607[28], 6}, {&S23607[35], 25}, {&S23607[82], 17}, {NULL}};
const Segment SGI277338[] = {{&S23612[41], 4}, {NULL}};
const Segment SGI277339[] = {{&S23612[13], 14}, {&S23612[28], 25}, {&S23612[75], 17}, {NULL}};
const Segment SGI277340[] = {{&S27541[76], 4}, {NULL}};
const Segment SGI277341[] = {{&S27541[13], 3}, {&S27541[17], 11}, {&S27541[50], 17}, {NULL}};
const Segment SGI277342[] = {{&S27542[78], 4}, {NULL}};
const Segment SGI277343[] = {{&S27542[13], 3}, {&S27542[17], 13}, {&S27542[52], 17}, {NULL}};
const Segment SGI277344[] = {{&S27543[98], 4}, {NULL}};
const Segment SGI277345[] = {{&S27543[13], 3}, {&S27543[17], 13}, {&S27543[52], 17}, {&S27543[70], 19}, {NULL}};
const Segment SGI277346[] = {{&S27544[85], 4}, {NULL}};
const Segment SGI277347[] = {{&S27544[13], 3}, {&S27544[17], 19}, {&S27544[59], 17}, {NULL}};
const Segment SGI277348[] = {{&S27545[98], 4}, {NULL}};
const Segment SGI277349[] = {{&S27545[23], 3}, {&S27545[27], 2}, {&S27545[30], 13}, {&S27545[44], 6}, {&S27545[72], 17}, {NULL}};
const Segment SGI277350[] = {{&S27546[77], 4}, {NULL}};
const Segment SGI277351[] = {{&S27546[14], 3}, {&S27546[18], 11}, {&S27546[51], 17}, {NULL}};
const Segment SGI277352[] = {{&S27546[0], 1}, {NULL}};
const Segment SGI277353[] = {{&S27547[83], 4}, {NULL}};
const Segment SGI277354[] = {{&S27547[13], 3}, {&S27547[17], 13}, {&S27547[31], 4}, {&S27547[57], 17}, {NULL}};
const Segment SGI277355[] = {{&S27548[80], 4}, {NULL}};
const Segment SGI277356[] = {{&S27548[13], 3}, {&S27548[17], 10}, {&S27548[28], 4}, {&S27548[54], 17}, {NULL}};
const Segment SGI277357[] = {{&S27549[80], 4}, {NULL}};
const Segment SGI277358[] = {{&S27549[13], 3}, {&S27549[17], 10}, {&S27549[28], 4}, {&S27549[54], 17}, {NULL}};
const Segment SGI277359[] = {{&S27550[83], 4}, {NULL}};
const Segment SGI277360[] = {{&S27550[13], 3}, {&S27550[17], 13}, {&S27550[31], 4}, {&S27550[57], 17}, {NULL}};
const Segment SGI277361[] = {{&S27551[80], 4}, {NULL}};
const Segment SGI277362[] = {{&S27551[13], 3}, {&S27551[17], 10}, {&S27551[28], 4}, {&S27551[54], 17}, {NULL}};
const Segment SGI277363[] = {{&S27552[83], 4}, {NULL}};
const Segment SGI277364[] = {{&S27552[13], 3}, {&S27552[17], 13}, {&S27552[31], 4}, {&S27552[57], 17}, {NULL}};
const Segment SGI277365[] = {{&S27553[80], 4}, {NULL}};
const Segment SGI277366[] = {{&S27553[13], 3}, {&S27553[17], 10}, {&S27553[28], 4}, {&S27553[54], 17}, {NULL}};
const Segment SGI277367[] = {{&S27554[76], 4}, {NULL}};
const Segment SGI277368[] = {{&S27554[13], 3}, {&S27554[17], 11}, {&S27554[50], 17}, {&S27554[120], 35}, {&S27554[156], 17}, {NULL}};
const Segment SGI277369[] = {{&S27680[93], 4}, {NULL}};
const Segment SGI277370[] = {{&S27680[13], 9}, {&S27680[23], 22}, {&S27680[67], 17}, {NULL}};
const Segment SGI277371[] = {{&S27681[93], 4}, {NULL}};
const Segment SGI277372[] = {{&S27681[13], 9}, {&S27681[23], 22}, {&S27681[67], 17}, {NULL}};
const Segment SGI277373[] = {{&S27682[93], 4}, {NULL}};
const Segment SGI277374[] = {{&S27682[13], 9}, {&S27682[23], 22}, {&S27682[67], 17}, {NULL}};
const Segment SGI277375[] = {{&S27683[93], 4}, {NULL}};
const Segment SGI277376[] = {{&S27683[13], 9}, {&S27683[23], 22}, {&S27683[67], 17}, {NULL}};
const Segment SGI277377[] = {{&S27684[93], 4}, {NULL}};
const Segment SGI277378[] = {{&S27684[13], 9}, {&S27684[23], 22}, {&S27684[67], 17}, {NULL}};
const Segment SGI277379[] = {{&S27685[93], 4}, {NULL}};
const Segment SGI277380[] = {{&S27685[13], 9}, {&S27685[23], 22}, {&S27685[67], 17}, {NULL}};
const Segment SGI277381[] = {{&S27686[103], 4}, {NULL}};
const Segment SGI277382[] = {{&S27686[13], 9}, {&S27686[23], 2}, {&S27686[26], 22}, {&S27686[49], 6}, {&S27686[77], 17}, {NULL}};
const Segment SGI277383[] = {{&S27687[93], 4}, {NULL}};
const Segment SGI277384[] = {{&S27687[13], 9}, {&S27687[23], 22}, {&S27687[67], 17}, {NULL}};
const Segment SGI277385[] = {{&S27688[93], 4}, {NULL}};
const Segment SGI277386[] = {{&S27688[13], 9}, {&S27688[23], 22}, {&S27688[67], 17}, {NULL}};
const Segment SGI277387[] = {{&S27689[103], 4}, {NULL}};
const Segment SGI277388[] = {{&S27689[13], 9}, {&S27689[23], 2}, {&S27689[26], 22}, {&S27689[49], 6}, {&S27689[77], 17}, {NULL}};
const Segment SGI277389[] = {{&S27690[93], 4}, {NULL}};
const Segment SGI277390[] = {{&S27690[13], 9}, {&S27690[23], 22}, {&S27690[67], 17}, {NULL}};
const Segment SGI277391[] = {{&S27691[103], 4}, {NULL}};
const Segment SGI277392[] = {{&S27691[13], 9}, {&S27691[23], 2}, {&S27691[26], 22}, {&S27691[49], 6}, {&S27691[77], 17}, {NULL}};
const Segment SGI277393[] = {{&S27692[103], 4}, {NULL}};
const Segment SGI277394[] = {{&S27692[13], 9}, {&S27692[23], 2}, {&S27692[26], 22}, {&S27692[49], 6}, {&S27692[77], 17}, {NULL}};
const Segment SGI277395[] = {{&S27693[103], 4}, {NULL}};
const Segment SGI277396[] = {{&S27693[13], 9}, {&S27693[23], 2}, {&S27693[26], 22}, {&S27693[49], 6}, {&S27693[77], 17}, {NULL}};
const Segment SGI277397[] = {{&S27694[93], 4}, {NULL}};
const Segment SGI277398[] = {{&S27694[13], 9}, {&S27694[23], 22}, {&S27694[67], 17}, {NULL}};
const Segment SGI277399[] = {{&S27695[103], 4}, {NULL}};
const Segment SGI277400[] = {{&S27695[13], 9}, {&S27695[23], 2}, {&S27695[26], 22}, {&S27695[49], 6}, {&S27695[77], 17}, {NULL}};
const Segment SGI277401[] = {{&S27696[103], 4}, {NULL}};
const Segment SGI277402[] = {{&S27696[13], 9}, {&S27696[23], 2}, {&S27696[26], 22}, {&S27696[49], 6}, {&S27696[77], 17}, {NULL}};
const Segment SGI277403[] = {{&S23977[82], 4}, {NULL}};
const Segment SGI277404[] = {{&S23977[13], 14}, {&S23977[28], 6}, {&S23977[56], 17}, {&S23977[128], 12}, {NULL}};
const Segment SGI277405[] = {{&S23977[109], 8}, {NULL}};
const Segment SGI277406[] = {{&S27697[103], 4}, {NULL}};
const Segment SGI277407[] = {{&S27697[13], 9}, {&S27697[23], 2}, {&S27697[26], 22}, {&S27697[49], 6}, {&S27697[77], 17}, {NULL}};
const Segment SGI277408[] = {{&S23978[75], 4}, {NULL}};
const Segment SGI277409[] = {{&S23978[13], 14}, {&S23978[49], 17}, {&S23978[121], 12}, {NULL}};
const Segment SGI277410[] = {{&S23978[102], 8}, {NULL}};
const Segment SGI277411[] = {{&S23979[75], 4}, {NULL}};
const Segment SGI277412[] = {{&S23979[13], 14}, {&S23979[49], 17}, {&S23979[121], 15}, {NULL}};
const Segment SGI277413[] = {{&S23979[102], 8}, {NULL}};
const Segment SGI277414[] = {{&S23980[75], 4}, {NULL}};
const Segment SGI277415[] = {{&S23980[13], 14}, {&S23980[49], 17}, {&S23980[121], 19}, {NULL}};
const Segment SGI277416[] = {{&S23980[102], 8}, {NULL}};
const Segment SGI277417[] = {{&S23981[82], 4}, {NULL}};
const Segment SGI277418[] = {{&S23981[13], 14}, {&S23981[28], 6}, {&S23981[56], 17}, {&S23981[128], 19}, {NULL}};
const Segment SGI277419[] = {{&S23981[109], 8}, {NULL}};
const Segment SGI277420[] = {{&S23982[75], 4}, {NULL}};
const Segment SGI277421[] = {{&S23982[13], 14}, {&S23982[49], 17}, {&S23982[121], 14}, {NULL}};
const Segment SGI277422[] = {{&S23982[102], 8}, {NULL}};
const Segment SGI277423[] = {{&S23983[82], 4}, {NULL}};
const Segment SGI277424[] = {{&S23983[13], 14}, {&S23983[28], 6}, {&S23983[56], 17}, {&S23983[128], 10}, {NULL}};
const Segment SGI277425[] = {{&S23983[109], 8}, {NULL}};
const Segment SGI277426[] = {{&S23984[82], 4}, {NULL}};
const Segment SGI277427[] = {{&S23984[13], 14}, {&S23984[28], 6}, {&S23984[56], 17}, {NULL}};
const Segment SGI277428[] = {{&S23984[109], 8}, {NULL}};
const Segment SGI277429[] = {{&S23985[75], 4}, {NULL}};
const Segment SGI277430[] = {{&S23985[13], 14}, {&S23985[49], 17}, {NULL}};
const Segment SGI277431[] = {{&S23985[102], 8}, {NULL}};
const Segment SGI277432[] = {{&S23986[82], 4}, {NULL}};
const Segment SGI277433[] = {{&S23986[13], 14}, {&S23986[28], 6}, {&S23986[56], 17}, {&S23986[127], 8}, {NULL}};
const Segment SGI277434[] = {{&S23986[109], 8}, {NULL}};
const Segment SGI277435[] = {{&S23987[82], 4}, {NULL}};
const Segment SGI277436[] = {{&S23987[13], 14}, {&S23987[28], 6}, {&S23987[56], 17}, {&S23987[128], 14}, {NULL}};
const Segment SGI277437[] = {{&S23987[109], 8}, {NULL}};
const Segment SGI277438[] = {{&S23988[82], 4}, {NULL}};
const Segment SGI277439[] = {{&S23988[13], 14}, {&S23988[28], 6}, {&S23988[56], 17}, {&S23988[128], 19}, {NULL}};
const Segment SGI277440[] = {{&S23988[109], 8}, {NULL}};
const Segment SGI277441[] = {{&S23989[82], 4}, {NULL}};
const Segment SGI277442[] = {{&S23989[13], 14}, {&S23989[28], 6}, {&S23989[56], 17}, {&S23989[128], 15}, {NULL}};
const Segment SGI277443[] = {{&S23989[109], 8}, {NULL}};
const Segment SGI277444[] = {{&S23990[75], 4}, {NULL}};
const Segment SGI277445[] = {{&S23990[13], 14}, {&S23990[49], 17}, {&S23990[120], 8}, {NULL}};
const Segment SGI277446[] = {{&S23990[102], 8}, {NULL}};
const Segment SGI277447[] = {{&S23991[75], 4}, {NULL}};
const Segment SGI277448[] = {{&S23991[13], 14}, {&S23991[49], 17}, {&S23991[121], 10}, {NULL}};
const Segment SGI277449[] = {{&S23991[102], 8}, {NULL}};
const Segment SGI277450[] = {{&S23992[75], 4}, {NULL}};
const Segment SGI277451[] = {{&S23992[13], 14}, {&S23992[49], 17}, {&S23992[121], 19}, {NULL}};
const Segment SGI277452[] = {{&S23992[102], 8}, {NULL}};
const Segment SGI277453[] = {{&S23993[82], 4}, {NULL}};
const Segment SGI277454[] = {{&S23993[13], 14}, {&S23993[28], 6}, {&S23993[56], 17}, {&S23993[127], 8}, {&S23993[127], 8}, {NULL}};
const Segment SGI277455[] = {{&S23993[109], 8}, {NULL}};
const Segment SGI277456[] = {{&S24092[41], 4}, {NULL}};
const Segment SGI277457[] = {{&S24092[13], 14}, {&S24092[28], 25}, {&S24092[75], 17}, {NULL}};
const Segment SGI277458[] = {{&S24093[41], 4}, {NULL}};
const Segment SGI277459[] = {{&S24093[13], 14}, {&S24093[28], 25}, {&S24093[75], 17}, {NULL}};
const Segment SGI277460[] = {{&S24094[41], 4}, {NULL}};
const Segment SGI277461[] = {{&S24094[13], 14}, {&S24094[28], 25}, {&S24094[75], 17}, {NULL}};
const Segment SGI277462[] = {{&S24096[48], 4}, {NULL}};
const Segment SGI277463[] = {{&S24096[13], 14}, {&S24096[28], 6}, {&S24096[35], 25}, {&S24096[82], 17}, {NULL}};
const Segment SGI277464[] = {{&S24097[48], 4}, {NULL}};
const Segment SGI277465[] = {{&S24097[13], 14}, {&S24097[28], 6}, {&S24097[35], 25}, {&S24097[82], 17}, {NULL}};
const Segment SGI277466[] = {{&S24098[41], 4}, {NULL}};
const Segment SGI277467[] = {{&S24098[13], 14}, {&S24098[28], 25}, {&S24098[75], 17}, {NULL}};
const Segment SGI277468[] = {{&S24099[41], 4}, {NULL}};
const Segment SGI277469[] = {{&S24099[13], 14}, {&S24099[28], 26}, {&S24099[76], 17}, {NULL}};
const Segment SGI277470[] = {{&S24101[48], 4}, {NULL}};
const Segment SGI277471[] = {{&S24101[13], 14}, {&S24101[28], 6}, {&S24101[35], 26}, {&S24101[83], 17}, {NULL}};
const Segment SGI277472[] = {{&S24108[41], 4}, {NULL}};
const Segment SGI277473[] = {{&S24108[13], 14}, {&S24108[28], 26}, {&S24108[76], 17}, {NULL}};
const Segment SGI277474[] = {{&S24109[48], 4}, {NULL}};
const Segment SGI277475[] = {{&S24109[13], 14}, {&S24109[28], 6}, {&S24109[35], 26}, {&S24109[83], 17}, {NULL}};
const Segment SGI277476[] = {{&S27885[92], 4}, {NULL}};
const Segment SGI277477[] = {{&S27885[13], 9}, {&S27885[23], 22}, {&S27885[66], 17}, {NULL}};
const Segment SGI277478[] = {{&S27886[92], 4}, {NULL}};
const Segment SGI277479[] = {{&S27886[13], 9}, {&S27886[23], 22}, {&S27886[66], 17}, {NULL}};
const Segment SGI277480[] = {{&S27887[92], 4}, {NULL}};
const Segment SGI277481[] = {{&S27887[13], 9}, {&S27887[23], 22}, {&S27887[66], 17}, {NULL}};
const Segment SGI277482[] = {{&S27888[92], 4}, {NULL}};
const Segment SGI277483[] = {{&S27888[13], 9}, {&S27888[23], 22}, {&S27888[66], 17}, {NULL}};
const Segment SGI277484[] = {{&S27889[92], 4}, {NULL}};
const Segment SGI277485[] = {{&S27889[13], 9}, {&S27889[23], 22}, {&S27889[66], 17}, {NULL}};
const Segment SGI277486[] = {{&S27890[92], 4}, {NULL}};
const Segment SGI277487[] = {{&S27890[13], 9}, {&S27890[23], 22}, {&S27890[66], 17}, {NULL}};
const Segment SGI277488[] = {{&S27891[92], 4}, {NULL}};
const Segment SGI277489[] = {{&S27891[13], 9}, {&S27891[23], 22}, {&S27891[66], 17}, {NULL}};
const Segment SGI277490[] = {{&S27892[92], 4}, {NULL}};
const Segment SGI277491[] = {{&S27892[13], 9}, {&S27892[23], 22}, {&S27892[66], 17}, {NULL}};
const Segment SGI277492[] = {{&S27893[90], 4}, {NULL}};
const Segment SGI277493[] = {{&S27893[13], 9}, {&S27893[23], 20}, {&S27893[64], 17}, {NULL}};
const Segment SGI277494[] = {{&S27894[92], 4}, {NULL}};
const Segment SGI277495[] = {{&S27894[13], 9}, {&S27894[23], 22}, {&S27894[66], 17}, {NULL}};
const Segment SGI277496[] = {{&S27895[98], 4}, {NULL}};
const Segment SGI277497[] = {{&S27895[5], 23}, {&S27895[29], 22}, {&S27895[72], 17}, {NULL}};
const Segment SGI277498[] = {{&S27895[0], 6}, {NULL}};
const Segment SGI277499[] = {{&S27896[92], 4}, {NULL}};
const Segment SGI277500[] = {{&S27896[13], 9}, {&S27896[23], 22}, {&S27896[66], 17}, {NULL}};
const Segment SGI277501[] = {{&S27897[92], 4}, {NULL}};
const Segment SGI277502[] = {{&S27897[13], 9}, {&S27897[23], 22}, {&S27897[66], 17}, {NULL}};
const Segment SGI277503[] = {{&S27898[92], 4}, {NULL}};
const Segment SGI277504[] = {{&S27898[13], 9}, {&S27898[23], 22}, {&S27898[66], 17}, {NULL}};
const Segment SGI277505[] = {{&S27899[103], 4}, {NULL}};
const Segment SGI277506[] = {{&S27899[13], 9}, {&S27899[23], 2}, {&S27899[26], 22}, {&S27899[49], 6}, {&S27899[77], 17}, {NULL}};
const Segment SGI277507[] = {{&S27900[103], 4}, {NULL}};
const Segment SGI277508[] = {{&S27900[13], 9}, {&S27900[23], 2}, {&S27900[26], 22}, {&S27900[49], 6}, {&S27900[77], 17}, {NULL}};
const Segment SGI277509[] = {{&S27901[103], 4}, {NULL}};
const Segment SGI277510[] = {{&S27901[13], 9}, {&S27901[23], 2}, {&S27901[26], 22}, {&S27901[49], 6}, {&S27901[77], 17}, {NULL}};
const Segment SGI277511[] = {{&S27902[99], 4}, {NULL}};
const Segment SGI277512[] = {{&S27902[13], 3}, {&S27902[17], 21}, {&S27902[59], 17}, {NULL}};
const Segment SGI277513[] = {{&S27903[77], 4}, {NULL}};
const Segment SGI277514[] = {{&S27903[13], 3}, {&S27903[17], 13}, {&S27903[51], 17}, {NULL}};
const Segment SGI277515[] = {{&S27904[75], 4}, {NULL}};
const Segment SGI277516[] = {{&S27904[13], 3}, {&S27904[17], 11}, {&S27904[49], 17}, {NULL}};
const Segment SGI277517[] = {{&S27905[103], 4}, {NULL}};
const Segment SGI277518[] = {{&S27905[29], 3}, {&S27905[33], 2}, {&S27905[36], 13}, {&S27905[50], 6}, {&S27905[77], 17}, {NULL}};
const Segment SGI277519[] = {{&S27906[77], 4}, {NULL}};
const Segment SGI277520[] = {{&S27906[13], 3}, {&S27906[17], 12}, {&S27906[51], 17}, {NULL}};
const Segment SGI277521[] = {{&S27907[85], 4}, {NULL}};
const Segment SGI277522[] = {{&S27907[13], 3}, {&S27907[17], 19}, {&S27907[59], 17}, {NULL}};
const Segment SGI277523[] = {{&S27908[83], 4}, {NULL}};
const Segment SGI277524[] = {{&S27908[13], 3}, {&S27908[17], 13}, {&S27908[31], 4}, {&S27908[57], 17}, {NULL}};
const Segment SGI277525[] = {{&S27909[83], 4}, {NULL}};
const Segment SGI277526[] = {{&S27909[13], 3}, {&S27909[17], 13}, {&S27909[31], 4}, {&S27909[57], 17}, {NULL}};
const Segment SGI277527[] = {{&S42642[38], 5}, {NULL}};
const Segment SGI277528[] = {{&S27910[80], 4}, {NULL}};
const Segment SGI277529[] = {{&S42642[100], 5}, {NULL}};
const Segment SGI277530[] = {{&S27910[13], 3}, {&S27910[17], 10}, {&S27910[28], 4}, {&S27910[54], 17}, {NULL}};
const Segment SGI277531[] = {{&S27911[80], 4}, {NULL}};
const Segment SGI277532[] = {{&S27911[13], 3}, {&S27911[17], 10}, {&S27911[28], 4}, {&S27911[54], 17}, {NULL}};
const Segment SGI277533[] = {{&S27912[80], 4}, {NULL}};
const Segment SGI277534[] = {{&S27912[13], 3}, {&S27912[17], 10}, {&S27912[28], 4}, {&S27912[54], 17}, {NULL}};
const Segment SGI277535[] = {{&S27913[83], 4}, {NULL}};
const Segment SGI277536[] = {{&S27913[13], 3}, {&S27913[17], 13}, {&S27913[31], 4}, {&S27913[57], 17}, {NULL}};
const Segment SGI277537[] = {{&S27914[80], 4}, {NULL}};
const Segment SGI277538[] = {{&S27914[13], 3}, {&S27914[17], 10}, {&S27914[28], 4}, {&S27914[54], 17}, {NULL}};
const Segment SGI277539[] = {{&S28001[106], 4}, {NULL}};
const Segment SGI277540[] = {{&S28001[13], 7}, {&S28001[21], 2}, {&S28001[24], 15}, {&S28001[40], 6}, {&S28001[68], 17}, {NULL}};
const Segment SGI277541[] = {{&S28001[87], 11}, {NULL}};
const Segment SGI277542[] = {{&S28003[106], 4}, {NULL}};
const Segment SGI277543[] = {{&S28003[13], 7}, {&S28003[21], 2}, {&S28003[24], 15}, {&S28003[40], 6}, {&S28003[68], 17}, {NULL}};
const Segment SGI277544[] = {{&S28003[87], 11}, {NULL}};
const Segment SGI277545[] = {{&S28006[87], 4}, {NULL}};
const Segment SGI277546[] = {{&S28006[13], 14}, {&S28006[49], 17}, {NULL}};
const Segment SGI277547[] = {{&S28006[68], 11}, {NULL}};
const Segment SGI277548[] = {{&S28007[106], 4}, {NULL}};
const Segment SGI277549[] = {{&S28007[13], 7}, {&S28007[21], 2}, {&S28007[24], 15}, {&S28007[40], 6}, {&S28007[68], 17}, {NULL}};
const Segment SGI277550[] = {{&S28007[87], 11}, {NULL}};
const Segment SGI277551[] = {{&S28008[94], 4}, {NULL}};
const Segment SGI277552[] = {{&S28008[13], 14}, {&S28008[28], 6}, {&S28008[56], 17}, {NULL}};
const Segment SGI277553[] = {{&S28008[75], 11}, {NULL}};
const Segment SGI277554[] = {{&S28009[94], 4}, {NULL}};
const Segment SGI277555[] = {{&S28009[13], 14}, {&S28009[28], 6}, {&S28009[56], 17}, {NULL}};
const Segment SGI277556[] = {{&S28009[75], 11}, {NULL}};
const Segment SGI277557[] = {{&S28011[87], 4}, {NULL}};
const Segment SGI277558[] = {{&S28011[13], 14}, {&S28011[49], 17}, {NULL}};
const Segment SGI277559[] = {{&S28011[68], 11}, {NULL}};
const Segment SGI277560[] = {{&S28012[87], 4}, {NULL}};
const Segment SGI277561[] = {{&S28012[13], 14}, {&S28012[49], 17}, {&S28012[112], 8}, {&S28012[123], 24}, {NULL}};
const Segment SGI277562[] = {{&S28012[68], 11}, {NULL}};
const Segment SGI277563[] = {{&S28013[87], 4}, {NULL}};
const Segment SGI277564[] = {{&S28013[13], 14}, {&S28013[49], 17}, {&S28013[112], 8}, {NULL}};
const Segment SGI277565[] = {{&S28013[68], 11}, {NULL}};
const Segment SGI277566[] = {{&S28235[92], 4}, {NULL}};
const Segment SGI277567[] = {{&S28235[13], 9}, {&S28235[23], 22}, {&S28235[66], 17}, {NULL}};
const Segment SGI277568[] = {{&S28236[92], 4}, {NULL}};
const Segment SGI277569[] = {{&S28236[13], 9}, {&S28236[23], 22}, {&S28236[66], 17}, {NULL}};
const Segment SGI277570[] = {{&S28237[92], 4}, {NULL}};
const Segment SGI277571[] = {{&S28237[13], 9}, {&S28237[23], 22}, {&S28237[66], 17}, {NULL}};
const Segment SGI277572[] = {{&S28238[92], 4}, {NULL}};
const Segment SGI277573[] = {{&S28238[13], 9}, {&S28238[23], 22}, {&S28238[66], 17}, {NULL}};
const Segment SGI277574[] = {{&S28239[92], 4}, {NULL}};
const Segment SGI277575[] = {{&S28239[13], 9}, {&S28239[23], 22}, {&S28239[66], 17}, {&S28239[134], 35}, {&S28239[170], 23}, {NULL}};
const Segment SGI277576[] = {{&S28240[92], 4}, {NULL}};
const Segment SGI277577[] = {{&S28240[13], 9}, {&S28240[23], 22}, {&S28240[66], 17}, {NULL}};
const Segment SGI277578[] = {{&S28241[92], 4}, {NULL}};
const Segment SGI277579[] = {{&S28241[13], 9}, {&S28241[23], 22}, {&S28241[66], 17}, {&S28241[134], 35}, {&S28241[170], 24}, {NULL}};
const Segment SGI277580[] = {{&S28242[92], 4}, {NULL}};
const Segment SGI277581[] = {{&S28242[13], 9}, {&S28242[23], 22}, {&S28242[66], 17}, {NULL}};
const Segment SGI277582[] = {{&S28243[92], 4}, {NULL}};
const Segment SGI277583[] = {{&S28243[13], 9}, {&S28243[23], 22}, {&S28243[66], 17}, {&S28243[134], 35}, {&S28243[170], 23}, {NULL}};
const Segment SGI277584[] = {{&S28244[94], 4}, {NULL}};
const Segment SGI277585[] = {{&S28244[13], 9}, {&S28244[23], 24}, {&S28244[68], 17}, {NULL}};
const Segment SGI277586[] = {{&S24729[92], 4}, {NULL}};
const Segment SGI277587[] = {{&S24729[13], 9}, {&S24729[23], 22}, {&S24729[66], 17}, {NULL}};
const Segment SGI277588[] = {{&S24730[92], 4}, {NULL}};
const Segment SGI277589[] = {{&S24730[13], 9}, {&S24730[23], 22}, {&S24730[66], 17}, {NULL}};
const Segment SGI277590[] = {{&S24731[92], 4}, {NULL}};
const Segment SGI277591[] = {{&S24731[13], 9}, {&S24731[23], 22}, {&S24731[66], 17}, {NULL}};
const Segment SGI277592[] = {{&S24732[92], 4}, {NULL}};
const Segment SGI277593[] = {{&S24732[13], 9}, {&S24732[23], 22}, {&S24732[66], 17}, {NULL}};
const Segment SGI277594[] = {{&S24733[92], 4}, {NULL}};
const Segment SGI277595[] = {{&S24733[13], 9}, {&S24733[23], 22}, {&S24733[66], 17}, {NULL}};
const Segment SGI277596[] = {{&S24734[92], 4}, {NULL}};
const Segment SGI277597[] = {{&S24734[13], 9}, {&S24734[23], 22}, {&S24734[66], 17}, {NULL}};
const Segment SGI277598[] = {{&S24735[92], 4}, {NULL}};
const Segment SGI277599[] = {{&S24735[13], 9}, {&S24735[23], 22}, {&S24735[66], 17}, {NULL}};
const Segment SGI277600[] = {{&S24736[92], 4}, {NULL}};
const Segment SGI277601[] = {{&S24736[13], 9}, {&S24736[23], 22}, {&S24736[66], 17}, {NULL}};
const Segment SGI277602[] = {{&S24737[92], 4}, {NULL}};
const Segment SGI277603[] = {{&S24737[13], 9}, {&S24737[23], 22}, {&S24737[66], 17}, {&S24737[118], 8}, {&S24737[129], 24}, {NULL}};
const Segment SGI277604[] = {{&S24738[92], 4}, {NULL}};
const Segment SGI277605[] = {{&S24738[13], 9}, {&S24738[23], 22}, {&S24738[66], 17}, {NULL}};
const Segment SGI277606[] = {{&S24739[92], 4}, {NULL}};
const Segment SGI277607[] = {{&S24739[13], 9}, {&S24739[23], 22}, {&S24739[66], 17}, {&S24739[118], 8}, {&S24739[129], 26}, {NULL}};
const Segment SGI277608[] = {{&S24740[89], 4}, {NULL}};
const Segment SGI277609[] = {{&S24740[13], 9}, {&S24740[23], 20}, {&S24740[64], 16}, {NULL}};
const Segment SGI277610[] = {{&S24741[103], 4}, {NULL}};
const Segment SGI277611[] = {{&S24741[13], 9}, {&S24741[23], 2}, {&S24741[26], 22}, {&S24741[49], 6}, {&S24741[77], 17}, {NULL}};
const Segment SGI277612[] = {{&S24742[103], 4}, {NULL}};
const Segment SGI277613[] = {{&S24742[13], 9}, {&S24742[23], 2}, {&S24742[26], 22}, {&S24742[49], 6}, {&S24742[77], 17}, {NULL}};
const Segment SGI277614[] = {{&S24743[103], 4}, {NULL}};
const Segment SGI277615[] = {{&S24743[13], 9}, {&S24743[23], 2}, {&S24743[26], 22}, {&S24743[49], 6}, {&S24743[77], 17}, {NULL}};
const Segment SGI277616[] = {{&S28735[74], 4}, {NULL}};
const Segment SGI277617[] = {{&S28735[13], 14}, {&S28735[48], 17}, {NULL}};
const Segment SGI277618[] = {{&S28736[81], 4}, {NULL}};
const Segment SGI277619[] = {{&S28736[13], 14}, {&S28736[28], 6}, {&S28736[55], 17}, {NULL}};
const Segment SGI277620[] = {{&S43306[38], 5}, {NULL}};
const Segment SGI277621[] = {{&S43306[99], 5}, {NULL}};
const Segment SGI277622[] = {{&S28737[81], 5}, {NULL}};
const Segment SGI277623[] = {{&S28737[13], 14}, {&S28737[28], 6}, {&S28737[55], 17}, {NULL}};
const Segment SGI277624[] = {{&S28738[98], 4}, {NULL}};
const Segment SGI277625[] = {{&S28738[13], 14}, {&S28738[48], 17}, {NULL}};
const Segment SGI277626[] = {{&S28739[105], 4}, {NULL}};
const Segment SGI277627[] = {{&S28739[13], 14}, {&S28739[28], 6}, {&S28739[55], 17}, {NULL}};
const Segment SGI277628[] = {{&S28757[13], 7}, {&S28757[21], 2}, {&S28757[24], 15}, {&S28757[54], 17}, {NULL}};
const Segment SGI277629[] = {{&S28758[94], 4}, {NULL}};
const Segment SGI277630[] = {{&S28758[13], 7}, {&S28758[21], 2}, {&S28758[24], 15}, {&S28758[40], 6}, {&S28758[68], 17}, {NULL}};
const Segment SGI277631[] = {{&S28759[84], 4}, {NULL}};
const Segment SGI277632[] = {{&S28759[13], 7}, {&S28759[21], 15}, {&S28759[58], 17}, {NULL}};
const Segment SGI277633[] = {{&S28760[75], 4}, {NULL}};
const Segment SGI277634[] = {{&S28760[13], 14}, {&S28760[49], 17}, {NULL}};
const Segment SGI277635[] = {{&S28761[82], 4}, {NULL}};
const Segment SGI277636[] = {{&S28761[13], 14}, {&S28761[28], 6}, {&S28761[56], 17}, {NULL}};
const Segment SGI277637[] = {{&S28762[75], 4}, {NULL}};
const Segment SGI277638[] = {{&S28762[13], 14}, {&S28762[49], 17}, {&S28762[101], 8}, {&S28762[112], 24}, {NULL}};
const Segment SGI277639[] = {{&S28763[75], 4}, {NULL}};
const Segment SGI277640[] = {{&S28763[13], 14}, {&S28763[49], 17}, {NULL}};
const Segment SGI277641[] = {{&S28764[82], 4}, {NULL}};
const Segment SGI277642[] = {{&S28764[13], 14}, {&S28764[28], 6}, {&S28764[56], 17}, {NULL}};
const Segment SGI277643[] = {{&S28765[93], 4}, {NULL}};
const Segment SGI277644[] = {{&S28765[13], 7}, {&S28765[21], 2}, {&S28765[24], 15}, {&S28765[40], 6}, {&S28765[68], 16}, {NULL}};
const Segment SGI277645[] = {{&S28766[74], 4}, {NULL}};
const Segment SGI277646[] = {{&S28766[13], 14}, {&S28766[49], 16}, {NULL}};
const Segment SGI277647[] = {{&S28767[81], 4}, {NULL}};
const Segment SGI277648[] = {{&S28767[13], 14}, {&S28767[28], 6}, {&S28767[56], 16}, {NULL}};
const Segment SGI277649[] = {{&S33720[94], 3}, {NULL}};
const Segment SGI277650[] = {{&S33720[13], 7}, {&S33720[21], 2}, {&S33720[24], 15}, {&S33720[40], 6}, {&S33720[68], 17}, {NULL}};
const Segment SGI277651[] = {{&S33720[120], 15}, {NULL}};
const Segment SGI277652[] = {{&S33722[94], 3}, {NULL}};
const Segment SGI277653[] = {{&S33722[13], 7}, {&S33722[21], 2}, {&S33722[24], 15}, {&S33722[40], 6}, {&S33722[68], 17}, {NULL}};
const Segment SGI277654[] = {{&S33722[120], 15}, {NULL}};
const Segment SGI277655[] = {{&S28792[93], 4}, {NULL}};
const Segment SGI277656[] = {{&S28792[13], 9}, {&S28792[23], 22}, {&S28792[67], 17}, {NULL}};
const Segment SGI277657[] = {{&S28793[93], 4}, {NULL}};
const Segment SGI277658[] = {{&S28793[13], 9}, {&S28793[23], 22}, {&S28793[67], 17}, {NULL}};
const Segment SGI277659[] = {{&S28794[93], 4}, {NULL}};
const Segment SGI277660[] = {{&S28794[13], 9}, {&S28794[23], 22}, {&S28794[67], 17}, {NULL}};
const Segment SGI277661[] = {{&S28795[93], 4}, {NULL}};
const Segment SGI277662[] = {{&S28795[13], 9}, {&S28795[23], 22}, {&S28795[67], 17}, {NULL}};
const Segment SGI277663[] = {{&S28796[93], 4}, {NULL}};
const Segment SGI277664[] = {{&S28796[13], 9}, {&S28796[23], 22}, {&S28796[67], 17}, {NULL}};
const Segment SGI277665[] = {{&S28797[93], 4}, {NULL}};
const Segment SGI277666[] = {{&S28797[13], 9}, {&S28797[23], 22}, {&S28797[67], 17}, {NULL}};
const Segment SGI277667[] = {{&S28798[93], 4}, {NULL}};
const Segment SGI277668[] = {{&S28798[13], 9}, {&S28798[23], 22}, {&S28798[67], 17}, {NULL}};
const Segment SGI277669[] = {{&S28799[93], 4}, {NULL}};
const Segment SGI277670[] = {{&S28799[13], 9}, {&S28799[23], 22}, {&S28799[67], 17}, {NULL}};
const Segment SGI277671[] = {{&S28800[93], 4}, {NULL}};
const Segment SGI277672[] = {{&S28800[13], 9}, {&S28800[23], 22}, {&S28800[67], 17}, {NULL}};
const Segment SGI277673[] = {{&S28801[93], 5}, {NULL}};
const Segment SGI277674[] = {{&S28801[13], 9}, {&S28801[23], 22}, {&S28801[67], 17}, {NULL}};
const Segment SGI277675[] = {{&S28802[90], 4}, {NULL}};
const Segment SGI277676[] = {{&S28802[13], 9}, {&S28802[23], 20}, {&S28802[65], 16}, {NULL}};
const Segment SGI277677[] = {{&S28803[103], 4}, {NULL}};
const Segment SGI277678[] = {{&S28803[13], 9}, {&S28803[23], 2}, {&S28803[26], 22}, {&S28803[49], 6}, {&S28803[77], 17}, {NULL}};
const Segment SGI277679[] = {{&S28804[103], 4}, {NULL}};
const Segment SGI277680[] = {{&S28804[13], 9}, {&S28804[23], 2}, {&S28804[26], 22}, {&S28804[49], 6}, {&S28804[77], 17}, {NULL}};
const Segment SGI277681[] = {{&S28805[103], 4}, {NULL}};
const Segment SGI277682[] = {{&S28805[13], 9}, {&S28805[23], 2}, {&S28805[26], 22}, {&S28805[49], 6}, {&S28805[77], 17}, {NULL}};
const Segment SGI277683[] = {{&S28806[102], 4}, {NULL}};
const Segment SGI277684[] = {{&S28806[13], 9}, {&S28806[23], 2}, {&S28806[26], 22}, {&S28806[49], 6}, {&S28806[76], 17}, {NULL}};
const Segment SGI277685[] = {{&S25178[41], 4}, {NULL}};
const Segment SGI277686[] = {{&S25178[13], 14}, {&S25178[28], 25}, {&S25178[75], 17}, {NULL}};
const Segment SGI277687[] = {{&S25179[41], 4}, {NULL}};
const Segment SGI277688[] = {{&S25179[13], 14}, {&S25179[28], 25}, {&S25179[75], 17}, {NULL}};
const Segment SGI277689[] = {{&S25180[41], 4}, {NULL}};
const Segment SGI277690[] = {{&S25180[13], 14}, {&S25180[28], 25}, {&S25180[75], 17}, {NULL}};
const Segment SGI277691[] = {{&S25181[41], 4}, {NULL}};
const Segment SGI277692[] = {{&S25181[13], 14}, {&S25181[28], 25}, {&S25181[75], 17}, {NULL}};
const Segment SGI277693[] = {{&S25182[48], 4}, {NULL}};
const Segment SGI277694[] = {{&S25182[13], 14}, {&S25182[28], 6}, {&S25182[35], 25}, {&S25182[82], 17}, {NULL}};
const Segment SGI277695[] = {{&S25183[48], 4}, {NULL}};
const Segment SGI277696[] = {{&S25183[13], 14}, {&S25183[28], 6}, {&S25183[35], 25}, {&S25183[82], 17}, {NULL}};
const Segment SGI277697[] = {{&S25184[41], 4}, {NULL}};
const Segment SGI277698[] = {{&S25184[13], 14}, {&S25184[28], 25}, {&S25184[75], 17}, {NULL}};
const Segment SGI277699[] = {{&S25187[48], 4}, {NULL}};
const Segment SGI277700[] = {{&S25187[13], 14}, {&S25187[28], 6}, {&S25187[35], 25}, {&S25187[82], 17}, {NULL}};
const Segment SGI277701[] = {{&S25189[41], 4}, {NULL}};
const Segment SGI277702[] = {{&S25189[13], 14}, {&S25189[28], 25}, {&S25189[75], 17}, {NULL}};
const Segment SGI277703[] = {{&S25284[41], 4}, {NULL}};
const Segment SGI277704[] = {{&S25284[13], 14}, {&S25284[28], 25}, {&S25284[75], 17}, {NULL}};
const Segment SGI277705[] = {{&S25285[41], 4}, {NULL}};
const Segment SGI277706[] = {{&S25285[13], 14}, {&S25285[28], 25}, {&S25285[75], 17}, {NULL}};
const Segment SGI277707[] = {{&S25286[41], 4}, {NULL}};
const Segment SGI277708[] = {{&S25286[13], 14}, {&S25286[28], 25}, {&S25286[75], 17}, {NULL}};
const Segment SGI277709[] = {{&S25287[48], 4}, {NULL}};
const Segment SGI277710[] = {{&S25287[13], 14}, {&S25287[28], 6}, {&S25287[35], 25}, {&S25287[82], 17}, {NULL}};
const Segment SGI277711[] = {{&S25288[48], 4}, {NULL}};
const Segment SGI277712[] = {{&S25288[13], 14}, {&S25288[28], 6}, {&S25288[35], 25}, {&S25288[82], 17}, {NULL}};
const Segment SGI277713[] = {{&S25290[41], 4}, {NULL}};
const Segment SGI277714[] = {{&S25290[13], 14}, {&S25290[28], 25}, {&S25290[75], 17}, {NULL}};
const Segment SGI277715[] = {{&S25291[48], 4}, {NULL}};
const Segment SGI277716[] = {{&S25291[13], 14}, {&S25291[28], 6}, {&S25291[35], 25}, {&S25291[82], 17}, {NULL}};
const Segment SGI277717[] = {{&S25295[41], 4}, {NULL}};
const Segment SGI277718[] = {{&S25295[13], 14}, {&S25295[28], 25}, {&S25295[75], 17}, {NULL}};
const Segment SGI277719[] = {{&S34423[93], 3}, {NULL}};
const Segment SGI277720[] = {{&S34423[13], 7}, {&S34423[21], 2}, {&S34423[24], 15}, {&S34423[40], 6}, {&S34423[67], 17}, {NULL}};
const Segment SGI277721[] = {{&S34424[82], 3}, {NULL}};
const Segment SGI277722[] = {{&S34424[13], 14}, {&S34424[28], 6}, {&S34424[56], 17}, {NULL}};
const Segment SGI277723[] = {{&S34425[75], 3}, {NULL}};
const Segment SGI277724[] = {{&S34425[13], 14}, {&S34425[49], 17}, {NULL}};
const Segment SGI277725[] = {{&S34426[81], 3}, {NULL}};
const Segment SGI277726[] = {{&S34426[13], 14}, {&S34426[28], 6}, {&S34426[55], 17}, {NULL}};
const Segment SGI277727[] = {{&S25377[84], 4}, {NULL}};
const Segment SGI277728[] = {{&S25377[13], 3}, {&S25377[17], 19}, {&S25377[58], 17}, {NULL}};
const Segment SGI277729[] = {{&S25378[86], 4}, {NULL}};
const Segment SGI277730[] = {{&S25378[13], 3}, {&S25378[17], 21}, {&S25378[60], 17}, {NULL}};
const Segment SGI277731[] = {{&S25379[87], 4}, {NULL}};
const Segment SGI277732[] = {{&S25379[13], 3}, {&S25379[17], 22}, {&S25379[61], 17}, {NULL}};
const Segment SGI277733[] = {{&S25380[85], 4}, {NULL}};
const Segment SGI277734[] = {{&S25380[13], 3}, {&S25380[17], 20}, {&S25380[59], 17}, {NULL}};
const Segment SGI277735[] = {{&S25381[85], 4}, {NULL}};
const Segment SGI277736[] = {{&S25381[13], 3}, {&S25381[17], 20}, {&S25381[59], 17}, {NULL}};
const Segment SGI277737[] = {{&S25382[85], 4}, {NULL}};
const Segment SGI277738[] = {{&S25382[13], 3}, {&S25382[17], 20}, {&S25382[59], 17}, {NULL}};
const Segment SGI277739[] = {{&S25383[85], 4}, {NULL}};
const Segment SGI277740[] = {{&S25383[13], 3}, {&S25383[17], 20}, {&S25383[59], 17}, {NULL}};
const Segment SGI277741[] = {{&S25384[87], 4}, {NULL}};
const Segment SGI277742[] = {{&S25384[13], 3}, {&S25384[17], 22}, {&S25384[61], 17}, {NULL}};
const Segment SGI277743[] = {{&S25385[86], 4}, {NULL}};
const Segment SGI277744[] = {{&S25385[13], 3}, {&S25385[17], 21}, {&S25385[60], 17}, {NULL}};
const Segment SGI277745[] = {{&S25386[84], 4}, {NULL}};
const Segment SGI277746[] = {{&S25386[13], 3}, {&S25386[17], 19}, {&S25386[58], 17}, {NULL}};
const Segment SGI277747[] = {{&S25387[85], 4}, {NULL}};
const Segment SGI277748[] = {{&S25387[13], 3}, {&S25387[17], 20}, {&S25387[59], 17}, {NULL}};
const Segment SGI277749[] = {{&S25388[85], 4}, {NULL}};
const Segment SGI277750[] = {{&S25388[13], 3}, {&S25388[17], 20}, {&S25388[59], 17}, {NULL}};
const Segment SGI277751[] = {{&S25389[87], 4}, {NULL}};
const Segment SGI277752[] = {{&S25389[13], 3}, {&S25389[17], 22}, {&S25389[61], 17}, {NULL}};
const Segment SGI277753[] = {{&S25390[88], 4}, {NULL}};
const Segment SGI277754[] = {{&S25390[13], 3}, {&S25390[17], 23}, {&S25390[62], 17}, {NULL}};
const Segment SGI277755[] = {{&S25391[86], 4}, {NULL}};
const Segment SGI277756[] = {{&S25391[13], 3}, {&S25391[17], 21}, {&S25391[60], 17}, {NULL}};
const Segment SGI277757[] = {{&S25392[88], 4}, {NULL}};
const Segment SGI277758[] = {{&S25392[13], 3}, {&S25392[17], 23}, {&S25392[62], 17}, {NULL}};
const Segment SGI277759[] = {{&S25393[88], 4}, {NULL}};
const Segment SGI277760[] = {{&S25393[13], 3}, {&S25393[17], 23}, {&S25393[62], 17}, {NULL}};
const Segment SGI277761[] = {{&S25394[86], 4}, {NULL}};
const Segment SGI277762[] = {{&S25394[13], 3}, {&S25394[17], 21}, {&S25394[60], 17}, {NULL}};
const Segment SGI277763[] = {{&S25395[88], 4}, {NULL}};
const Segment SGI277764[] = {{&S25395[13], 3}, {&S25395[17], 23}, {&S25395[62], 17}, {NULL}};
const Segment SGI277765[] = {{&S25396[86], 4}, {NULL}};
const Segment SGI277766[] = {{&S25396[13], 3}, {&S25396[17], 21}, {&S25396[60], 17}, {NULL}};
const Segment SGI277767[] = {{&S25397[88], 4}, {NULL}};
const Segment SGI277768[] = {{&S25397[13], 3}, {&S25397[17], 23}, {&S25397[62], 17}, {NULL}};
const Segment SGI277769[] = {{&S25398[87], 4}, {NULL}};
const Segment SGI277770[] = {{&S25398[13], 3}, {&S25398[17], 22}, {&S25398[61], 17}, {NULL}};
const Segment SGI277771[] = {{&S25399[88], 4}, {NULL}};
const Segment SGI277772[] = {{&S25399[13], 3}, {&S25399[17], 23}, {&S25399[62], 17}, {NULL}};
const Segment SGI277773[] = {{&S25400[88], 4}, {NULL}};
const Segment SGI277774[] = {{&S25400[13], 3}, {&S25400[17], 23}, {&S25400[62], 17}, {NULL}};
const Segment SGI277775[] = {{&S25401[86], 4}, {NULL}};
const Segment SGI277776[] = {{&S25401[13], 3}, {&S25401[17], 21}, {&S25401[60], 17}, {NULL}};
const Segment SGI277777[] = {{&S25402[88], 4}, {NULL}};
const Segment SGI277778[] = {{&S25402[13], 3}, {&S25402[17], 23}, {&S25402[62], 17}, {NULL}};
const Segment SGI277779[] = {{&S25403[88], 4}, {NULL}};
const Segment SGI277780[] = {{&S25403[13], 3}, {&S25403[17], 23}, {&S25403[62], 17}, {NULL}};
const Segment SGI277781[] = {{&S25404[86], 4}, {NULL}};
const Segment SGI277782[] = {{&S25404[13], 3}, {&S25404[17], 21}, {&S25404[60], 17}, {NULL}};
const Segment SGI277783[] = {{&S25405[86], 4}, {NULL}};
const Segment SGI277784[] = {{&S25405[13], 3}, {&S25405[17], 21}, {&S25405[60], 17}, {NULL}};
const Segment SGI277785[] = {{&S25406[88], 4}, {NULL}};
const Segment SGI277786[] = {{&S25406[13], 3}, {&S25406[17], 23}, {&S25406[62], 17}, {NULL}};
const Segment SGI277787[] = {{&S25407[88], 4}, {NULL}};
const Segment SGI277788[] = {{&S25407[13], 3}, {&S25407[17], 23}, {&S25407[62], 17}, {NULL}};
const Segment SGI277789[] = {{&S25408[87], 4}, {NULL}};
const Segment SGI277790[] = {{&S25408[13], 3}, {&S25408[17], 22}, {&S25408[61], 17}, {NULL}};
const Segment SGI277791[] = {{&S25409[88], 4}, {NULL}};
const Segment SGI277792[] = {{&S25409[13], 3}, {&S25409[17], 23}, {&S25409[62], 17}, {NULL}};
const Segment SGI277793[] = {{&S25410[88], 4}, {NULL}};
const Segment SGI277794[] = {{&S25410[13], 3}, {&S25410[17], 23}, {&S25410[62], 17}, {NULL}};
const Segment SGI277795[] = {{&S25411[86], 4}, {NULL}};
const Segment SGI277796[] = {{&S25411[13], 3}, {&S25411[17], 21}, {&S25411[60], 17}, {NULL}};
const Segment SGI277797[] = {{&S25412[86], 4}, {NULL}};
const Segment SGI277798[] = {{&S25412[13], 3}, {&S25412[17], 21}, {&S25412[60], 17}, {NULL}};
const Segment SGI277799[] = {{&S25413[86], 4}, {NULL}};
const Segment SGI277800[] = {{&S29476[74], 4}, {NULL}};
const Segment SGI277801[] = {{&S25413[13], 3}, {&S25413[17], 21}, {&S25413[60], 17}, {NULL}};
const Segment SGI277802[] = {{&S29476[13], 14}, {&S29476[48], 17}, {NULL}};
const Segment SGI277803[] = {{&S25414[88], 4}, {NULL}};
const Segment SGI277804[] = {{&S25414[13], 3}, {&S25414[17], 23}, {&S25414[62], 17}, {NULL}};
const Segment SGI277805[] = {{&S29477[81], 4}, {NULL}};
const Segment SGI277806[] = {{&S29477[13], 14}, {&S29477[28], 6}, {&S29477[55], 17}, {NULL}};
const Segment SGI277807[] = {{&S25415[88], 4}, {NULL}};
const Segment SGI277808[] = {{&S25415[13], 3}, {&S25415[17], 23}, {&S25415[62], 17}, {NULL}};
const Segment SGI277809[] = {{&S29478[82], 4}, {NULL}};
const Segment SGI277810[] = {{&S29478[13], 14}, {&S29478[28], 6}, {&S29478[56], 17}, {NULL}};
const Segment SGI277811[] = {{&S25416[86], 4}, {NULL}};
const Segment SGI277812[] = {{&S25416[13], 3}, {&S25416[17], 21}, {&S25416[60], 17}, {NULL}};
const Segment SGI277813[] = {{&S29479[75], 4}, {NULL}};
const Segment SGI277814[] = {{&S29479[13], 14}, {&S29479[49], 17}, {&S29479[103], 8}, {&S29479[114], 24}, {NULL}};
const Segment SGI277815[] = {{&S25417[86], 4}, {NULL}};
const Segment SGI277816[] = {{&S25417[13], 3}, {&S25417[17], 21}, {&S25417[60], 17}, {NULL}};
const Segment SGI277817[] = {{&S29480[75], 4}, {NULL}};
const Segment SGI277818[] = {{&S29480[13], 14}, {&S29480[49], 17}, {NULL}};
const Segment SGI277819[] = {{&S25418[88], 4}, {NULL}};
const Segment SGI277820[] = {{&S25418[13], 3}, {&S25418[17], 23}, {&S25418[62], 17}, {NULL}};
const Segment SGI277821[] = {{&S29481[82], 4}, {NULL}};
const Segment SGI277822[] = {{&S29481[13], 14}, {&S29481[28], 6}, {&S29481[56], 17}, {&S29481[110], 8}, {&S29481[121], 24}, {NULL}};
const Segment SGI277823[] = {{&S25419[88], 4}, {NULL}};
const Segment SGI277824[] = {{&S25419[13], 3}, {&S25419[17], 23}, {&S25419[62], 17}, {NULL}};
const Segment SGI277825[] = {{&S29482[112], 4}, {NULL}};
const Segment SGI277826[] = {{&S25420[88], 4}, {NULL}};
const Segment SGI277827[] = {{&S25420[13], 3}, {&S25420[17], 23}, {&S25420[62], 17}, {NULL}};
const Segment SGI277828[] = {{&S29482[13], 7}, {&S29482[21], 2}, {&S29482[24], 15}, {&S29482[40], 6}, {&S29482[47], 15}, {&S29482[11], 1}, {&S29482[86], 17}, {NULL}};
const Segment SGI277829[] = {{&S25421[88], 4}, {NULL}};
const Segment SGI277830[] = {{&S25421[13], 3}, {&S25421[17], 23}, {&S25421[62], 17}, {NULL}};
const Segment SGI277831[] = {{&S29483[94], 4}, {NULL}};
const Segment SGI277832[] = {{&S29483[13], 14}, {&S29483[28], 6}, {&S29483[56], 29}, {NULL}};
const Segment SGI277833[] = {{&S25422[88], 4}, {NULL}};
const Segment SGI277834[] = {{&S25422[13], 3}, {&S25422[17], 23}, {&S25422[62], 17}, {NULL}};
const Segment SGI277835[] = {{&S29484[126], 4}, {NULL}};
const Segment SGI277836[] = {{&S29484[13], 7}, {&S29484[21], 2}, {&S29484[24], 15}, {&S29484[40], 6}, {&S29484[47], 29}, {&S29484[11], 1}, {&S29484[100], 17}, {NULL}};
const Segment SGI277837[] = {{&S25423[88], 4}, {NULL}};
const Segment SGI277838[] = {{&S25423[13], 3}, {&S25423[17], 23}, {&S25423[62], 17}, {NULL}};
const Segment SGI277839[] = {{&S29485[126], 4}, {NULL}};
const Segment SGI277840[] = {{&S25424[86], 4}, {NULL}};
const Segment SGI277841[] = {{&S25424[13], 3}, {&S25424[17], 21}, {&S25424[60], 17}, {NULL}};
const Segment SGI277842[] = {{&S29485[13], 7}, {&S29485[21], 2}, {&S29485[24], 15}, {&S29485[40], 6}, {&S29485[47], 29}, {&S29485[11], 1}, {&S29485[100], 17}, {NULL}};
const Segment SGI277843[] = {{&S25425[88], 4}, {NULL}};
const Segment SGI277844[] = {{&S25425[13], 3}, {&S25425[17], 23}, {&S25425[62], 17}, {NULL}};
const Segment SGI277845[] = {{&S29486[126], 4}, {NULL}};
const Segment SGI277846[] = {{&S29486[13], 7}, {&S29486[21], 2}, {&S29486[24], 15}, {&S29486[40], 6}, {&S29486[47], 29}, {&S29486[11], 1}, {&S29486[100], 17}, {NULL}};
const Segment SGI277847[] = {{&S25426[86], 4}, {NULL}};
const Segment SGI277848[] = {{&S25426[13], 3}, {&S25426[17], 21}, {&S25426[60], 17}, {NULL}};
const Segment SGI277849[] = {{&S29487[126], 4}, {NULL}};
const Segment SGI277850[] = {{&S29487[13], 7}, {&S29487[21], 2}, {&S29487[24], 15}, {&S29487[40], 6}, {&S29487[47], 29}, {&S29487[11], 1}, {&S29487[100], 17}, {NULL}};
const Segment SGI277851[] = {{&S25427[88], 4}, {NULL}};
const Segment SGI277852[] = {{&S25427[13], 3}, {&S25427[17], 23}, {&S25427[62], 17}, {NULL}};
const Segment SGI277853[] = {{&S29488[91], 4}, {NULL}};
const Segment SGI277854[] = {{&S25428[88], 4}, {NULL}};
const Segment SGI277855[] = {{&S25428[13], 3}, {&S25428[17], 23}, {&S25428[62], 17}, {NULL}};
const Segment SGI277856[] = {{&S29488[13], 7}, {&S29488[21], 2}, {&S29488[24], 15}, {&S29488[40], 3}, {&S29488[65], 17}, {NULL}};
const Segment SGI277857[] = {{&S25429[88], 4}, {NULL}};
const Segment SGI277858[] = {{&S25429[13], 3}, {&S25429[17], 23}, {&S25429[62], 17}, {NULL}};
const Segment SGI277859[] = {{&S29489[126], 4}, {NULL}};
const Segment SGI277860[] = {{&S29489[13], 7}, {&S29489[21], 2}, {&S29489[24], 15}, {&S29489[40], 6}, {&S29489[47], 29}, {&S29489[11], 1}, {&S29489[100], 17}, {NULL}};
const Segment SGI277861[] = {{&S29490[126], 4}, {NULL}};
const Segment SGI277862[] = {{&S29490[13], 7}, {&S29490[21], 2}, {&S29490[24], 15}, {&S29490[40], 6}, {&S29490[47], 29}, {&S29490[11], 1}, {&S29490[100], 17}, {NULL}};
const Segment SGI277863[] = {{&S29491[126], 4}, {NULL}};
const Segment SGI277864[] = {{&S29491[13], 7}, {&S29491[21], 2}, {&S29491[24], 15}, {&S29491[40], 6}, {&S29491[47], 29}, {&S29491[11], 1}, {&S29491[100], 17}, {NULL}};
const Segment SGI277865[] = {{&S29492[126], 4}, {NULL}};
const Segment SGI277866[] = {{&S29492[13], 7}, {&S29492[21], 2}, {&S29492[24], 15}, {&S29492[40], 6}, {&S29492[47], 29}, {&S29492[11], 1}, {&S29492[100], 17}, {NULL}};
const Segment SGI277867[] = {{&S29493[123], 4}, {NULL}};
const Segment SGI277868[] = {{&S29493[13], 7}, {&S29493[21], 2}, {&S29493[24], 15}, {&S29493[40], 6}, {&S29493[47], 26}, {&S29493[11], 1}, {&S29493[97], 17}, {NULL}};
const Segment SGI277869[] = {{&S29494[93], 4}, {NULL}};
const Segment SGI277870[] = {{&S29494[13], 7}, {&S29494[21], 2}, {&S29494[24], 15}, {&S29494[40], 6}, {&S29494[67], 17}, {NULL}};
const Segment SGI277871[] = {{&S29495[81], 4}, {NULL}};
const Segment SGI277872[] = {{&S29495[13], 14}, {&S29495[28], 6}, {&S29495[55], 17}, {NULL}};
const Segment SGI277873[] = {{&S29496[90], 4}, {NULL}};
const Segment SGI277874[] = {{&S29496[13], 14}, {&S29496[28], 6}, {&S29496[56], 25}, {NULL}};
const Segment SGI277875[] = {{&S29497[118], 4}, {NULL}};
const Segment SGI277876[] = {{&S29497[13], 7}, {&S29497[21], 2}, {&S29497[24], 15}, {&S29497[40], 6}, {&S29497[47], 21}, {&S29497[11], 1}, {&S29497[92], 17}, {NULL}};
const Segment SGI277877[] = {{&S25505[86], 4}, {NULL}};
const Segment SGI277878[] = {{&S25505[13], 14}, {&S25505[48], 17}, {NULL}};
const Segment SGI277879[] = {{&S25505[67], 11}, {NULL}};
const Segment SGI277880[] = {{&S25506[93], 4}, {NULL}};
const Segment SGI277881[] = {{&S25506[13], 14}, {&S25506[28], 6}, {&S25506[55], 17}, {NULL}};
const Segment SGI277882[] = {{&S25506[74], 11}, {NULL}};
const Segment SGI277883[] = {{&S25511[93], 4}, {NULL}};
const Segment SGI277884[] = {{&S25511[13], 14}, {&S25511[28], 6}, {&S25511[55], 17}, {&S25511[119], 8}, {NULL}};
const Segment SGI277885[] = {{&S25511[74], 11}, {NULL}};
const Segment SGI277886[] = {{&S25512[86], 4}, {NULL}};
const Segment SGI277887[] = {{&S25512[13], 14}, {&S25512[48], 17}, {&S25512[112], 8}, {NULL}};
const Segment SGI277888[] = {{&S25512[67], 11}, {NULL}};
const Segment SGI277889[] = {{&S25556[92], 4}, {NULL}};
const Segment SGI277890[] = {{&S25556[13], 9}, {&S25556[23], 22}, {&S25556[66], 17}, {NULL}};
const Segment SGI277891[] = {{&S25557[92], 4}, {NULL}};
const Segment SGI277892[] = {{&S25557[13], 9}, {&S25557[23], 22}, {&S25557[66], 17}, {NULL}};
const Segment SGI277893[] = {{&S25558[92], 4}, {NULL}};
const Segment SGI277894[] = {{&S25558[13], 9}, {&S25558[23], 22}, {&S25558[66], 17}, {NULL}};
const Segment SGI277895[] = {{&S25559[92], 4}, {NULL}};
const Segment SGI277896[] = {{&S25559[13], 9}, {&S25559[23], 22}, {&S25559[66], 17}, {NULL}};
const Segment SGI277897[] = {{&S25560[92], 4}, {NULL}};
const Segment SGI277898[] = {{&S25560[13], 9}, {&S25560[23], 22}, {&S25560[66], 17}, {NULL}};
const Segment SGI277899[] = {{&S25561[92], 4}, {NULL}};
const Segment SGI277900[] = {{&S25561[13], 9}, {&S25561[23], 22}, {&S25561[66], 17}, {NULL}};
const Segment SGI277901[] = {{&S25562[92], 4}, {NULL}};
const Segment SGI277902[] = {{&S25562[13], 9}, {&S25562[23], 22}, {&S25562[66], 17}, {&S25562[118], 8}, {&S25562[129], 24}, {NULL}};
const Segment SGI277903[] = {{&S25563[92], 4}, {NULL}};
const Segment SGI277904[] = {{&S25563[13], 9}, {&S25563[23], 22}, {&S25563[66], 17}, {NULL}};
const Segment SGI277905[] = {{&S25564[92], 4}, {NULL}};
const Segment SGI277906[] = {{&S25564[13], 9}, {&S25564[23], 22}, {&S25564[66], 17}, {&S25564[118], 8}, {NULL}};
const Segment SGI277907[] = {{&S25565[92], 4}, {NULL}};
const Segment SGI277908[] = {{&S25565[13], 9}, {&S25565[23], 22}, {&S25565[66], 17}, {&S25565[118], 8}, {&S25565[129], 26}, {NULL}};
const Segment SGI277909[] = {{&S25566[92], 4}, {NULL}};
const Segment SGI277910[] = {{&S25566[13], 9}, {&S25566[23], 22}, {&S25566[66], 17}, {NULL}};
const Segment SGI277911[] = {{&S25567[103], 4}, {NULL}};
const Segment SGI277912[] = {{&S25567[13], 9}, {&S25567[23], 2}, {&S25567[26], 22}, {&S25567[49], 6}, {&S25567[77], 17}, {NULL}};
const Segment SGI277913[] = {{&S25568[103], 4}, {NULL}};
const Segment SGI277914[] = {{&S25568[13], 9}, {&S25568[23], 2}, {&S25568[26], 22}, {&S25568[49], 6}, {&S25568[77], 17}, {NULL}};
const Segment SGI277915[] = {{&S34685[94], 3}, {NULL}};
const Segment SGI277916[] = {{&S34685[13], 5}, {&S34685[40], 17}, {&S34685[58], 27}, {NULL}};
const Segment SGI277917[] = {{&S34687[94], 3}, {NULL}};
const Segment SGI277918[] = {{&S34687[13], 5}, {&S34687[40], 17}, {&S34687[58], 27}, {&S34687[116], 8}, {&S34687[127], 24}, {NULL}};
const Segment SGI277919[] = {{&S29760[81], 4}, {NULL}};
const Segment SGI277920[] = {{&S29760[13], 14}, {&S29760[28], 6}, {&S29760[55], 17}, {NULL}};
const Segment SGI277921[] = {{&S29761[74], 4}, {NULL}};
const Segment SGI277922[] = {{&S29761[13], 14}, {&S29761[48], 17}, {&S29761[100], 8}, {&S29761[111], 24}, {NULL}};
const Segment SGI277923[] = {{&S29762[74], 4}, {NULL}};
const Segment SGI277924[] = {{&S29762[13], 14}, {&S29762[48], 17}, {NULL}};
const Segment SGI277925[] = {{&S29763[98], 4}, {NULL}};
const Segment SGI277926[] = {{&S29763[13], 14}, {&S29763[48], 17}, {NULL}};
const Segment SGI277927[] = {{&S29764[105], 4}, {NULL}};
const Segment SGI277928[] = {{&S29764[13], 14}, {&S29764[28], 6}, {&S29764[55], 17}, {NULL}};
const Segment SGI277929[] = {{&S29809[93], 4}, {NULL}};
const Segment SGI277930[] = {{&S29809[13], 9}, {&S29809[23], 22}, {&S29809[67], 17}, {NULL}};
const Segment SGI277931[] = {{&S29810[93], 4}, {NULL}};
const Segment SGI277932[] = {{&S29810[13], 9}, {&S29810[23], 22}, {&S29810[67], 17}, {NULL}};
const Segment SGI277933[] = {{&S29811[93], 4}, {NULL}};
const Segment SGI277934[] = {{&S29811[13], 9}, {&S29811[23], 22}, {&S29811[67], 17}, {NULL}};
const Segment SGI277935[] = {{&S29812[93], 4}, {NULL}};
const Segment SGI277936[] = {{&S29812[13], 9}, {&S29812[23], 22}, {&S29812[67], 17}, {NULL}};
const Segment SGI277937[] = {{&S29813[93], 4}, {NULL}};
const Segment SGI277938[] = {{&S29813[13], 9}, {&S29813[23], 22}, {&S29813[67], 17}, {NULL}};
const Segment SGI277939[] = {{&S29814[93], 4}, {NULL}};
const Segment SGI277940[] = {{&S29814[13], 9}, {&S29814[23], 22}, {&S29814[67], 17}, {NULL}};
const Segment SGI277941[] = {{&S29815[93], 4}, {NULL}};
const Segment SGI277942[] = {{&S29815[13], 9}, {&S29815[23], 22}, {&S29815[67], 17}, {NULL}};
const Segment SGI277943[] = {{&S29816[93], 4}, {NULL}};
const Segment SGI277944[] = {{&S29816[13], 9}, {&S29816[23], 22}, {&S29816[67], 17}, {NULL}};
const Segment SGI277945[] = {{&S29817[93], 4}, {NULL}};
const Segment SGI277946[] = {{&S29817[13], 9}, {&S29817[23], 22}, {&S29817[67], 17}, {NULL}};
const Segment SGI277947[] = {{&S29818[93], 4}, {NULL}};
const Segment SGI277948[] = {{&S29818[13], 9}, {&S29818[23], 22}, {&S29818[67], 17}, {NULL}};
const Segment SGI277949[] = {{&S29818[114], 3}, {NULL}};
const Segment SGI277950[] = {{&S29819[106], 4}, {NULL}};
const Segment SGI277951[] = {{&S29819[13], 9}, {&S29819[23], 2}, {&S29819[26], 22}, {&S29819[49], 6}, {&S29819[80], 17}, {NULL}};
const Segment SGI277952[] = {{&S29820[93], 4}, {NULL}};
const Segment SGI277953[] = {{&S29820[13], 9}, {&S29820[23], 22}, {&S29820[67], 17}, {NULL}};
const Segment SGI277954[] = {{&S29821[93], 4}, {NULL}};
const Segment SGI277955[] = {{&S29821[13], 9}, {&S29821[23], 22}, {&S29821[67], 17}, {NULL}};
const Segment SGI277956[] = {{&S29821[115], 3}, {NULL}};
const Segment SGI277957[] = {{&S29822[90], 4}, {NULL}};
const Segment SGI277958[] = {{&S29822[13], 9}, {&S29822[23], 20}, {&S29822[65], 16}, {NULL}};
const Segment SGI277959[] = {{&S29823[90], 4}, {NULL}};
const Segment SGI277960[] = {{&S29823[13], 9}, {&S29823[23], 20}, {&S29823[65], 16}, {NULL}};
const Segment SGI277961[] = {{&S29824[103], 4}, {NULL}};
const Segment SGI277962[] = {{&S29824[13], 9}, {&S29824[23], 2}, {&S29824[26], 22}, {&S29824[49], 6}, {&S29824[77], 17}, {NULL}};
const Segment SGI277963[] = {{&S29825[103], 4}, {NULL}};
const Segment SGI277964[] = {{&S29825[13], 9}, {&S29825[23], 2}, {&S29825[26], 22}, {&S29825[49], 6}, {&S29825[77], 17}, {NULL}};
const Segment SGI277965[] = {{&S29826[103], 4}, {NULL}};
const Segment SGI277966[] = {{&S29826[13], 9}, {&S29826[23], 2}, {&S29826[26], 22}, {&S29826[49], 6}, {&S29826[77], 17}, {NULL}};
const Segment SGI277967[] = {{&S24945[76], 4}, {NULL}};
const Segment SGI277968[] = {{&S24945[13], 3}, {&S24945[17], 11}, {&S24945[50], 17}, {NULL}};
const Segment SGI277969[] = {{&S34906[93], 3}, {NULL}};
const Segment SGI277970[] = {{&S34906[13], 7}, {&S34906[21], 2}, {&S34906[24], 15}, {&S34906[40], 6}, {&S34906[67], 17}, {NULL}};
const Segment SGI277971[] = {{&S24946[78], 4}, {NULL}};
const Segment SGI277972[] = {{&S24946[13], 3}, {&S24946[17], 13}, {&S24946[52], 17}, {NULL}};
const Segment SGI277973[] = {{&S34907[93], 3}, {NULL}};
const Segment SGI277974[] = {{&S34907[13], 7}, {&S34907[21], 2}, {&S34907[24], 15}, {&S34907[40], 6}, {&S34907[67], 17}, {NULL}};
const Segment SGI277975[] = {{&S24947[85], 4}, {NULL}};
const Segment SGI277976[] = {{&S24947[13], 3}, {&S24947[17], 19}, {&S24947[59], 17}, {NULL}};
const Segment SGI277977[] = {{&S34908[102], 3}, {NULL}};
const Segment SGI277978[] = {{&S34908[13], 7}, {&S34908[21], 2}, {&S34908[24], 23}, {&S34908[49], 6}, {&S34908[76], 17}, {NULL}};
const Segment SGI277979[] = {{&S34985[93], 3}, {NULL}};
const Segment SGI277980[] = {{&S34985[13], 7}, {&S34985[21], 2}, {&S34985[24], 15}, {&S34985[40], 6}, {&S34985[67], 17}, {NULL}};
const Segment SGI277981[] = {{&S30030[92], 4}, {NULL}};
const Segment SGI277982[] = {{&S30030[13], 9}, {&S30030[23], 22}, {&S30030[66], 17}, {NULL}};
const Segment SGI277983[] = {{&S30031[92], 4}, {NULL}};
const Segment SGI277984[] = {{&S30031[13], 9}, {&S30031[23], 22}, {&S30031[66], 17}, {NULL}};
const Segment SGI277985[] = {{&S30032[92], 4}, {NULL}};
const Segment SGI277986[] = {{&S30032[13], 9}, {&S30032[23], 22}, {&S30032[66], 17}, {NULL}};
const Segment SGI277987[] = {{&S30033[92], 4}, {NULL}};
const Segment SGI277988[] = {{&S30033[13], 9}, {&S30033[23], 22}, {&S30033[66], 17}, {NULL}};
const Segment SGI277989[] = {{&S30034[92], 4}, {NULL}};
const Segment SGI277990[] = {{&S30034[13], 9}, {&S30034[23], 22}, {&S30034[66], 17}, {NULL}};
const Segment SGI277991[] = {{&S30035[92], 4}, {NULL}};
const Segment SGI277992[] = {{&S30035[13], 9}, {&S30035[23], 22}, {&S30035[66], 17}, {NULL}};
const Segment SGI277993[] = {{&S30036[92], 4}, {NULL}};
const Segment SGI277994[] = {{&S30036[13], 9}, {&S30036[23], 22}, {&S30036[66], 17}, {NULL}};
const Segment SGI277995[] = {{&S30037[92], 4}, {NULL}};
const Segment SGI277996[] = {{&S30037[13], 9}, {&S30037[23], 22}, {&S30037[66], 17}, {NULL}};
const Segment SGI277997[] = {{&S30038[90], 4}, {NULL}};
const Segment SGI277998[] = {{&S30038[13], 9}, {&S30038[23], 20}, {&S30038[64], 17}, {NULL}};
const Segment SGI277999[] = {{&S30039[92], 4}, {NULL}};
const Segment SGI278000[] = {{&S30039[13], 9}, {&S30039[23], 22}, {&S30039[66], 17}, {NULL}};
const Segment SGI278001[] = {{&S30040[92], 4}, {NULL}};
const Segment SGI278002[] = {{&S30040[13], 9}, {&S30040[23], 22}, {&S30040[66], 17}, {NULL}};
const Segment SGI278003[] = {{&S30041[103], 4}, {NULL}};
const Segment SGI278004[] = {{&S30041[13], 9}, {&S30041[23], 2}, {&S30041[26], 22}, {&S30041[49], 6}, {&S30041[77], 17}, {NULL}};
const Segment SGI278005[] = {{&S30042[103], 4}, {NULL}};
const Segment SGI278006[] = {{&S30042[13], 9}, {&S30042[23], 2}, {&S30042[26], 22}, {&S30042[49], 6}, {&S30042[77], 17}, {NULL}};
const Segment SGI278007[] = {{&S30043[103], 4}, {NULL}};
const Segment SGI278008[] = {{&S30043[13], 9}, {&S30043[23], 2}, {&S30043[26], 22}, {&S30043[49], 6}, {&S30043[77], 17}, {NULL}};
const Segment SGI278009[] = {{&S30080[92], 4}, {NULL}};
const Segment SGI278010[] = {{&S30080[13], 9}, {&S30080[23], 22}, {&S30080[66], 17}, {NULL}};
const Segment SGI278011[] = {{&S30081[92], 4}, {NULL}};
const Segment SGI278012[] = {{&S30081[13], 9}, {&S30081[23], 22}, {&S30081[66], 17}, {NULL}};
const Segment SGI278013[] = {{&S30082[92], 4}, {NULL}};
const Segment SGI278014[] = {{&S30082[13], 9}, {&S30082[23], 22}, {&S30082[66], 17}, {NULL}};
const Segment SGI278015[] = {{&S30083[92], 4}, {NULL}};
const Segment SGI278016[] = {{&S30083[13], 9}, {&S30083[23], 22}, {&S30083[66], 17}, {NULL}};
const Segment SGI278017[] = {{&S30084[92], 4}, {NULL}};
const Segment SGI278018[] = {{&S30084[13], 9}, {&S30084[23], 22}, {&S30084[66], 17}, {NULL}};
const Segment SGI278019[] = {{&S30085[92], 4}, {NULL}};
const Segment SGI278020[] = {{&S30085[13], 9}, {&S30085[23], 22}, {&S30085[66], 17}, {NULL}};
const Segment SGI278021[] = {{&S30218[13], 7}, {&S30218[21], 2}, {&S30218[24], 15}, {&S30218[54], 17}, {NULL}};
const Segment SGI278022[] = {{&S30219[94], 4}, {NULL}};
const Segment SGI278023[] = {{&S30219[13], 7}, {&S30219[21], 2}, {&S30219[24], 15}, {&S30219[40], 6}, {&S30219[68], 17}, {NULL}};
const Segment SGI278024[] = {{&S30220[94], 4}, {NULL}};
const Segment SGI278025[] = {{&S30220[13], 7}, {&S30220[21], 2}, {&S30220[24], 15}, {&S30220[40], 6}, {&S30220[68], 17}, {&S30220[121], 8}, {&S30220[132], 24}, {NULL}};
const Segment SGI278026[] = {{&S30221[82], 4}, {NULL}};
const Segment SGI278027[] = {{&S30221[13], 14}, {&S30221[28], 6}, {&S30221[56], 17}, {NULL}};
const Segment SGI278028[] = {{&S30222[75], 4}, {NULL}};
const Segment SGI278029[] = {{&S30222[13], 14}, {&S30222[49], 17}, {NULL}};
const Segment SGI278030[] = {{&S30459[82], 4}, {NULL}};
const Segment SGI278031[] = {{&S30459[13], 14}, {&S30459[28], 6}, {&S30459[56], 17}, {NULL}};
const Segment SGI278032[] = {{&S30459[110], 21}, {NULL}};
const Segment SGI278033[] = {{&S30460[75], 4}, {NULL}};
const Segment SGI278034[] = {{&S30460[13], 14}, {&S30460[49], 17}, {NULL}};
const Segment SGI278035[] = {{&S30460[103], 21}, {NULL}};
const Segment SGI278036[] = {{&S30461[81], 4}, {NULL}};
const Segment SGI278037[] = {{&S30461[13], 14}, {&S30461[28], 6}, {&S30461[55], 17}, {NULL}};
const Segment SGI278038[] = {{&S30461[107], 21}, {NULL}};
const Segment SGI278039[] = {{&S30462[74], 4}, {NULL}};
const Segment SGI278040[] = {{&S30462[13], 14}, {&S30462[48], 17}, {NULL}};
const Segment SGI278041[] = {{&S30462[100], 21}, {NULL}};
const Segment SGI278042[] = {{&S35447[8], 3}, {NULL}};
const Segment SGI278043[] = {{&S35447[13], 7}, {&S35447[21], 2}, {&S35447[24], 15}, {&S35447[40], 6}, {&S35447[67], 17}, {NULL}};
const Segment SGI278044[] = {{&S26187[74], 4}, {NULL}};
const Segment SGI278045[] = {{&S26187[13], 14}, {&S26187[48], 17}, {NULL}};
const Segment SGI278046[] = {{&S26188[81], 4}, {NULL}};
const Segment SGI278047[] = {{&S26188[13], 14}, {&S26188[28], 6}, {&S26188[55], 17}, {NULL}};
const Segment SGI278048[] = {{&S26189[74], 4}, {NULL}};
const Segment SGI278049[] = {{&S26189[13], 14}, {&S26189[48], 17}, {&S26189[100], 8}, {&S26189[111], 24}, {NULL}};
const Segment SGI278050[] = {{&S26190[74], 4}, {NULL}};
const Segment SGI278051[] = {{&S26190[13], 14}, {&S26190[48], 17}, {&S26190[100], 8}, {&S26190[111], 26}, {NULL}};
const Segment SGI278052[] = {{&S26191[81], 4}, {NULL}};
const Segment SGI278053[] = {{&S26191[13], 14}, {&S26191[28], 6}, {&S26191[55], 17}, {&S26191[107], 8}, {&S26191[118], 26}, {NULL}};
const Segment SGI278054[] = {{&S26192[74], 4}, {NULL}};
const Segment SGI278055[] = {{&S26192[13], 14}, {&S26192[48], 17}, {&S26192[100], 8}, {NULL}};
const Segment SGI278056[] = {{&S26193[87], 4}, {NULL}};
const Segment SGI278057[] = {{&S26193[13], 14}, {&S26193[28], 12}, {&S26193[61], 17}, {NULL}};
const Segment SGI278058[] = {{&S26194[94], 4}, {NULL}};
const Segment SGI278059[] = {{&S26194[13], 7}, {&S26194[21], 2}, {&S26194[24], 15}, {&S26194[40], 6}, {&S26194[68], 17}, {NULL}};
const Segment SGI278060[] = {{&S45500[38], 5}, {NULL}};
const Segment SGI278061[] = {{&S45500[100], 5}, {NULL}};
const Segment SGI278062[] = {{&S35749[8], 3}, {NULL}};
const Segment SGI278063[] = {{&S35749[13], 9}, {&S35749[23], 2}, {&S35749[26], 22}, {&S35749[49], 6}, {&S35749[76], 17}, {NULL}};
const Segment SGI278064[] = {{&S35750[8], 3}, {NULL}};
const Segment SGI278065[] = {{&S35750[13], 9}, {&S35750[23], 2}, {&S35750[26], 22}, {&S35750[49], 6}, {&S35750[76], 17}, {NULL}};
const Segment SGI278066[] = {{&S35751[8], 3}, {NULL}};
const Segment SGI278067[] = {{&S35751[13], 9}, {&S35751[23], 2}, {&S35751[26], 22}, {&S35751[49], 6}, {&S35751[76], 17}, {NULL}};
const Segment SGI278068[] = {{&S35752[8], 3}, {NULL}};
const Segment SGI278069[] = {{&S35752[13], 9}, {&S35752[23], 2}, {&S35752[26], 22}, {&S35752[49], 6}, {&S35752[76], 17}, {NULL}};
const Segment SGI278070[] = {{&S35753[8], 3}, {NULL}};
const Segment SGI278071[] = {{&S35753[13], 9}, {&S35753[23], 2}, {&S35753[26], 22}, {&S35753[49], 6}, {&S35753[76], 17}, {NULL}};
const Segment SGI278072[] = {{&S35754[8], 3}, {NULL}};
const Segment SGI278073[] = {{&S35754[13], 9}, {&S35754[23], 2}, {&S35754[26], 22}, {&S35754[49], 6}, {&S35754[76], 17}, {NULL}};
const Segment SGI278074[] = {{&S35755[8], 3}, {NULL}};
const Segment SGI278075[] = {{&S35755[13], 9}, {&S35755[23], 2}, {&S35755[26], 22}, {&S35755[49], 6}, {&S35755[76], 17}, {NULL}};
const Segment SGI278076[] = {{&S35756[8], 3}, {NULL}};
const Segment SGI278077[] = {{&S35756[13], 9}, {&S35756[23], 2}, {&S35756[26], 22}, {&S35756[49], 6}, {&S35756[76], 17}, {NULL}};
const Segment SGI278078[] = {{&S35757[8], 3}, {NULL}};
const Segment SGI278079[] = {{&S35757[13], 9}, {&S35757[23], 2}, {&S35757[26], 22}, {&S35757[49], 6}, {&S35757[76], 17}, {NULL}};
const Segment SGI278080[] = {{&S35758[8], 3}, {NULL}};
const Segment SGI278081[] = {{&S35758[13], 9}, {&S35758[23], 2}, {&S35758[26], 22}, {&S35758[49], 6}, {&S35758[76], 17}, {NULL}};
const Segment SGI278082[] = {{&S26536[41], 4}, {NULL}};
const Segment SGI278083[] = {{&S26536[13], 14}, {&S26536[28], 25}, {&S26536[75], 17}, {NULL}};
const Segment SGI278084[] = {{&S26537[48], 4}, {NULL}};
const Segment SGI278085[] = {{&S26537[13], 14}, {&S26537[28], 6}, {&S26537[35], 25}, {&S26537[82], 17}, {NULL}};
const Segment SGI278086[] = {{&S26538[48], 4}, {NULL}};
const Segment SGI278087[] = {{&S26538[13], 14}, {&S26538[28], 6}, {&S26538[35], 26}, {&S26538[83], 17}, {NULL}};
const Segment SGI278088[] = {{&S26542[41], 4}, {NULL}};
const Segment SGI278089[] = {{&S26542[13], 14}, {&S26542[28], 26}, {&S26542[76], 17}, {NULL}};
const Segment SGI278090[] = {{&S26545[48], 4}, {NULL}};
const Segment SGI278091[] = {{&S26545[13], 14}, {&S26545[28], 6}, {&S26545[35], 26}, {&S26545[83], 17}, {NULL}};
const Segment SGI278092[] = {{&S26546[41], 4}, {NULL}};
const Segment SGI278093[] = {{&S26546[13], 14}, {&S26546[28], 26}, {&S26546[76], 17}, {NULL}};
const Segment SGI278094[] = {{&S26547[48], 4}, {NULL}};
const Segment SGI278095[] = {{&S26547[13], 14}, {&S26547[28], 6}, {&S26547[35], 26}, {&S26547[83], 17}, {NULL}};
const Segment SGI278096[] = {{&S26548[41], 4}, {NULL}};
const Segment SGI278097[] = {{&S26548[13], 14}, {&S26548[28], 26}, {&S26548[76], 17}, {NULL}};
const Segment SGI278098[] = {{&S26557[41], 4}, {NULL}};
const Segment SGI278099[] = {{&S26557[13], 14}, {&S26557[28], 25}, {&S26557[75], 17}, {NULL}};
const Segment SGI278100[] = {{&S26558[41], 4}, {NULL}};
const Segment SGI278101[] = {{&S26558[13], 14}, {&S26558[28], 25}, {&S26558[75], 17}, {NULL}};
const Segment SGI278102[] = {{&S26559[41], 4}, {NULL}};
const Segment SGI278103[] = {{&S26559[13], 14}, {&S26559[28], 25}, {&S26559[75], 17}, {NULL}};
const Segment SGI278104[] = {{&S26561[48], 4}, {NULL}};
const Segment SGI278105[] = {{&S26561[13], 14}, {&S26561[28], 6}, {&S26561[35], 25}, {&S26561[82], 17}, {NULL}};
const Segment SGI278106[] = {{&S26562[48], 4}, {NULL}};
const Segment SGI278107[] = {{&S26562[13], 14}, {&S26562[28], 6}, {&S26562[35], 25}, {&S26562[82], 17}, {NULL}};
const Segment SGI278108[] = {{&S26563[41], 4}, {NULL}};
const Segment SGI278109[] = {{&S26563[13], 14}, {&S26563[28], 25}, {&S26563[75], 17}, {NULL}};
const Segment SGI278110[] = {{&S26565[48], 4}, {NULL}};
const Segment SGI278111[] = {{&S26565[13], 14}, {&S26565[28], 6}, {&S26565[35], 25}, {&S26565[82], 17}, {NULL}};
const Segment SGI278112[] = {{&S26566[41], 4}, {NULL}};
const Segment SGI278113[] = {{&S26566[13], 14}, {&S26566[28], 25}, {&S26566[75], 17}, {NULL}};
const Segment SGI278114[] = {{&S26573[41], 4}, {NULL}};
const Segment SGI278115[] = {{&S26573[13], 14}, {&S26573[28], 25}, {&S26573[75], 17}, {NULL}};
const Segment SGI278116[] = {{&S26574[48], 4}, {NULL}};
const Segment SGI278117[] = {{&S26574[13], 14}, {&S26574[28], 6}, {&S26574[35], 25}, {&S26574[82], 17}, {NULL}};
const Segment SGI278118[] = {{&S31424[13], 7}, {&S31424[21], 2}, {&S31424[24], 15}, {&S31424[54], 17}, {NULL}};
const Segment SGI278119[] = {{&S31425[81], 4}, {NULL}};
const Segment SGI278120[] = {{&S31425[13], 14}, {&S31425[28], 6}, {&S31425[55], 17}, {NULL}};
const Segment SGI278121[] = {{&S31426[74], 4}, {NULL}};
const Segment SGI278122[] = {{&S31426[13], 14}, {&S31426[48], 17}, {NULL}};
const Segment SGI278123[] = {{&S31427[81], 4}, {NULL}};
const Segment SGI278124[] = {{&S31427[13], 14}, {&S31427[28], 6}, {&S31427[55], 17}, {&S31427[107], 8}, {&S31427[118], 24}, {NULL}};
const Segment SGI278125[] = {{&S31428[74], 4}, {NULL}};
const Segment SGI278126[] = {{&S31428[13], 14}, {&S31428[48], 17}, {&S31428[100], 8}, {&S31428[111], 24}, {NULL}};
const Segment SGI278127[] = {{&S31429[7], 4}, {NULL}};
const Segment SGI278128[] = {{&S31429[19], 7}, {&S31429[27], 2}, {&S31429[30], 15}, {&S31429[46], 6}, {&S31429[77], 17}, {NULL}};
const Segment SGI278129[] = {{&S31430[7], 4}, {NULL}};
const Segment SGI278130[] = {{&S31430[19], 7}, {&S31430[27], 2}, {&S31430[30], 15}, {&S31430[46], 6}, {&S31430[77], 17}, {NULL}};
const Segment SGI278131[] = {{&S31431[7], 4}, {NULL}};
const Segment SGI278132[] = {{&S31431[19], 7}, {&S31431[27], 2}, {&S31431[30], 15}, {&S31431[46], 6}, {&S31431[77], 17}, {NULL}};
const Segment SGI278133[] = {{&S31432[7], 4}, {NULL}};
const Segment SGI278134[] = {{&S31432[19], 7}, {&S31432[27], 2}, {&S31432[30], 15}, {&S31432[46], 6}, {&S31432[77], 17}, {NULL}};
const Segment SGI278135[] = {{&S31433[7], 4}, {NULL}};
const Segment SGI278136[] = {{&S31433[19], 7}, {&S31433[27], 2}, {&S31433[30], 15}, {&S31433[46], 7}, {&S31433[78], 17}, {NULL}};
const Segment SGI278137[] = {{&S31434[7], 4}, {NULL}};
const Segment SGI278138[] = {{&S31434[19], 7}, {&S31434[27], 2}, {&S31434[30], 15}, {&S31434[46], 6}, {&S31434[77], 17}, {NULL}};
const Segment SGI278139[] = {{&S31435[7], 4}, {NULL}};
const Segment SGI278140[] = {{&S31435[19], 7}, {&S31435[27], 2}, {&S31435[30], 15}, {&S31435[46], 6}, {&S31435[77], 17}, {NULL}};
const Segment SGI278141[] = {{&S31436[73], 4}, {&S31436[73], 4}, {NULL}};
const Segment SGI278142[] = {{&S31436[13], 14}, {&S31436[48], 16}, {&S31436[13], 7}, {&S31436[123], 2}, {&S31436[126], 15}, {&S31436[142], 6}, {&S31436[48], 16}, {NULL}};
const Segment SGI278143[] = {{&S31437[7], 4}, {NULL}};
const Segment SGI278144[] = {{&S31437[19], 7}, {&S31437[27], 2}, {&S31437[30], 15}, {&S31437[46], 6}, {&S31437[77], 17}, {NULL}};
const Segment SGI278145[] = {{&S26974[84], 4}, {NULL}};
const Segment SGI278146[] = {{&S26974[13], 3}, {&S26974[17], 19}, {&S26974[58], 17}, {NULL}};
const Segment SGI278147[] = {{&S26975[86], 4}, {NULL}};
const Segment SGI278148[] = {{&S26975[13], 3}, {&S26975[17], 21}, {&S26975[60], 17}, {NULL}};
const Segment SGI278149[] = {{&S26976[86], 4}, {NULL}};
const Segment SGI278150[] = {{&S26976[13], 3}, {&S26976[17], 21}, {&S26976[60], 17}, {NULL}};
const Segment SGI278151[] = {{&S26977[86], 4}, {NULL}};
const Segment SGI278152[] = {{&S26977[13], 3}, {&S26977[17], 21}, {&S26977[60], 17}, {NULL}};
const Segment SGI278153[] = {{&S26978[86], 4}, {NULL}};
const Segment SGI278154[] = {{&S26978[13], 3}, {&S26978[17], 21}, {&S26978[60], 17}, {NULL}};
const Segment SGI278155[] = {{&S26979[84], 4}, {NULL}};
const Segment SGI278156[] = {{&S26979[13], 3}, {&S26979[17], 19}, {&S26979[58], 17}, {NULL}};
const Segment SGI278157[] = {{&S26980[86], 4}, {NULL}};
const Segment SGI278158[] = {{&S26980[13], 3}, {&S26980[17], 21}, {&S26980[60], 17}, {NULL}};
const Segment SGI278159[] = {{&S26981[86], 4}, {NULL}};
const Segment SGI278160[] = {{&S26981[13], 3}, {&S26981[17], 21}, {&S26981[60], 17}, {NULL}};
const Segment SGI278161[] = {{&S26982[86], 4}, {NULL}};
const Segment SGI278162[] = {{&S26982[13], 3}, {&S26982[17], 21}, {&S26982[60], 17}, {NULL}};
const Segment SGI278163[] = {{&S26983[86], 4}, {NULL}};
const Segment SGI278164[] = {{&S26983[13], 3}, {&S26983[17], 21}, {&S26983[60], 17}, {NULL}};
const Segment SGI278165[] = {{&S26984[84], 4}, {NULL}};
const Segment SGI278166[] = {{&S26984[13], 3}, {&S26984[17], 19}, {&S26984[58], 17}, {NULL}};
const Segment SGI278167[] = {{&S26985[86], 4}, {NULL}};
const Segment SGI278168[] = {{&S26985[13], 3}, {&S26985[17], 21}, {&S26985[60], 17}, {NULL}};
const Segment SGI278169[] = {{&S26986[86], 4}, {NULL}};
const Segment SGI278170[] = {{&S26986[13], 3}, {&S26986[17], 21}, {&S26986[60], 17}, {NULL}};
const Segment SGI278171[] = {{&S26987[84], 4}, {NULL}};
const Segment SGI278172[] = {{&S26987[13], 3}, {&S26987[17], 19}, {&S26987[58], 17}, {NULL}};
const Segment SGI278173[] = {{&S26988[87], 4}, {NULL}};
const Segment SGI278174[] = {{&S26988[13], 3}, {&S26988[17], 22}, {&S26988[61], 17}, {NULL}};
const Segment SGI278175[] = {{&S26989[87], 4}, {NULL}};
const Segment SGI278176[] = {{&S26989[13], 3}, {&S26989[17], 22}, {&S26989[61], 17}, {NULL}};
const Segment SGI278177[] = {{&S26990[85], 4}, {NULL}};
const Segment SGI278178[] = {{&S26990[13], 3}, {&S26990[17], 20}, {&S26990[59], 17}, {NULL}};
const Segment SGI278179[] = {{&S26991[87], 4}, {NULL}};
const Segment SGI278180[] = {{&S26991[13], 3}, {&S26991[17], 22}, {&S26991[61], 17}, {NULL}};
const Segment SGI278181[] = {{&S26992[85], 4}, {NULL}};
const Segment SGI278182[] = {{&S26992[13], 3}, {&S26992[17], 20}, {&S26992[59], 17}, {NULL}};
const Segment SGI278183[] = {{&S26993[87], 4}, {NULL}};
const Segment SGI278184[] = {{&S26993[13], 3}, {&S26993[17], 22}, {&S26993[61], 17}, {NULL}};
const Segment SGI278185[] = {{&S26994[87], 4}, {NULL}};
const Segment SGI278186[] = {{&S26994[13], 3}, {&S26994[17], 22}, {&S26994[61], 17}, {NULL}};
const Segment SGI278187[] = {{&S26995[85], 4}, {NULL}};
const Segment SGI278188[] = {{&S26995[13], 3}, {&S26995[17], 20}, {&S26995[59], 17}, {NULL}};
const Segment SGI278189[] = {{&S26996[87], 4}, {NULL}};
const Segment SGI278190[] = {{&S26996[13], 3}, {&S26996[17], 22}, {&S26996[61], 17}, {NULL}};
const Segment SGI278191[] = {{&S26997[87], 4}, {NULL}};
const Segment SGI278192[] = {{&S26997[13], 3}, {&S26997[17], 22}, {&S26997[61], 17}, {NULL}};
const Segment SGI278193[] = {{&S26998[87], 4}, {NULL}};
const Segment SGI278194[] = {{&S26998[13], 3}, {&S26998[17], 22}, {&S26998[61], 17}, {NULL}};
const Segment SGI278195[] = {{&S26999[88], 4}, {NULL}};
const Segment SGI278196[] = {{&S26999[13], 3}, {&S26999[17], 23}, {&S26999[62], 17}, {NULL}};
const Segment SGI278197[] = {{&S27000[88], 4}, {NULL}};
const Segment SGI278198[] = {{&S27000[13], 3}, {&S27000[17], 23}, {&S27000[62], 17}, {NULL}};
const Segment SGI278199[] = {{&S27001[88], 4}, {NULL}};
const Segment SGI278200[] = {{&S27001[13], 3}, {&S27001[17], 23}, {&S27001[62], 17}, {NULL}};
const Segment SGI278201[] = {{&S27002[86], 4}, {NULL}};
const Segment SGI278202[] = {{&S27002[13], 3}, {&S27002[17], 21}, {&S27002[60], 17}, {NULL}};
const Segment SGI278203[] = {{&S27003[88], 4}, {NULL}};
const Segment SGI278204[] = {{&S27003[13], 3}, {&S27003[17], 23}, {&S27003[62], 17}, {NULL}};
const Segment SGI278205[] = {{&S27004[88], 4}, {NULL}};
const Segment SGI278206[] = {{&S27004[13], 3}, {&S27004[17], 23}, {&S27004[62], 17}, {NULL}};
const Segment SGI278207[] = {{&S27005[88], 4}, {NULL}};
const Segment SGI278208[] = {{&S27005[13], 3}, {&S27005[17], 23}, {&S27005[62], 17}, {NULL}};
const Segment SGI278209[] = {{&S27006[86], 4}, {NULL}};
const Segment SGI278210[] = {{&S27006[13], 3}, {&S27006[17], 21}, {&S27006[60], 17}, {NULL}};
const Segment SGI278211[] = {{&S27007[88], 4}, {NULL}};
const Segment SGI278212[] = {{&S27007[13], 3}, {&S27007[17], 23}, {&S27007[62], 17}, {NULL}};
const Segment SGI278213[] = {{&S27008[88], 4}, {NULL}};
const Segment SGI278214[] = {{&S27008[13], 3}, {&S27008[17], 23}, {&S27008[62], 17}, {NULL}};
const Segment SGI278215[] = {{&S27009[88], 4}, {NULL}};
const Segment SGI278216[] = {{&S27009[13], 3}, {&S27009[17], 23}, {&S27009[62], 17}, {NULL}};
const Segment SGI278217[] = {{&S27010[86], 4}, {NULL}};
const Segment SGI278218[] = {{&S27010[13], 3}, {&S27010[17], 21}, {&S27010[60], 17}, {NULL}};
const Segment SGI278219[] = {{&S27011[88], 4}, {NULL}};
const Segment SGI278220[] = {{&S27011[13], 3}, {&S27011[17], 23}, {&S27011[62], 17}, {NULL}};
const Segment SGI278221[] = {{&S27012[88], 4}, {NULL}};
const Segment SGI278222[] = {{&S27012[13], 3}, {&S27012[17], 23}, {&S27012[62], 17}, {NULL}};
const Segment SGI278223[] = {{&S27013[88], 4}, {NULL}};
const Segment SGI278224[] = {{&S27013[13], 3}, {&S27013[17], 23}, {&S27013[62], 17}, {NULL}};
const Segment SGI278225[] = {{&S27014[86], 4}, {NULL}};
const Segment SGI278226[] = {{&S27014[13], 3}, {&S27014[17], 21}, {&S27014[60], 17}, {NULL}};
const Segment SGI278227[] = {{&S27015[88], 4}, {NULL}};
const Segment SGI278228[] = {{&S27015[13], 3}, {&S27015[17], 23}, {&S27015[62], 17}, {NULL}};
const Segment SGI278229[] = {{&S27016[88], 4}, {NULL}};
const Segment SGI278230[] = {{&S27016[13], 3}, {&S27016[17], 23}, {&S27016[62], 17}, {NULL}};
const Segment SGI278231[] = {{&S27034[84], 4}, {NULL}};
const Segment SGI278232[] = {{&S27034[13], 3}, {&S27034[17], 19}, {&S27034[58], 17}, {NULL}};
const Segment SGI278233[] = {{&S27035[86], 4}, {NULL}};
const Segment SGI278234[] = {{&S27035[13], 3}, {&S27035[17], 21}, {&S27035[60], 17}, {NULL}};
const Segment SGI278235[] = {{&S27036[86], 4}, {NULL}};
const Segment SGI278236[] = {{&S27036[13], 3}, {&S27036[17], 21}, {&S27036[60], 17}, {NULL}};
const Segment SGI278237[] = {{&S27037[86], 4}, {NULL}};
const Segment SGI278238[] = {{&S27037[13], 3}, {&S27037[17], 21}, {&S27037[60], 17}, {NULL}};
const Segment SGI278239[] = {{&S27038[84], 4}, {NULL}};
const Segment SGI278240[] = {{&S27038[13], 3}, {&S27038[17], 19}, {&S27038[58], 17}, {NULL}};
const Segment SGI278241[] = {{&S27039[86], 4}, {NULL}};
const Segment SGI278242[] = {{&S27039[13], 3}, {&S27039[17], 21}, {&S27039[60], 17}, {NULL}};
const Segment SGI278243[] = {{&S27040[84], 4}, {NULL}};
const Segment SGI278244[] = {{&S27040[13], 3}, {&S27040[17], 19}, {&S27040[58], 17}, {NULL}};
const Segment SGI278245[] = {{&S27041[86], 4}, {NULL}};
const Segment SGI278246[] = {{&S27041[13], 3}, {&S27041[17], 21}, {&S27041[60], 17}, {NULL}};
const Segment SGI278247[] = {{&S27042[86], 4}, {NULL}};
const Segment SGI278248[] = {{&S27042[13], 3}, {&S27042[17], 21}, {&S27042[60], 17}, {NULL}};
const Segment SGI278249[] = {{&S27043[85], 4}, {NULL}};
const Segment SGI278250[] = {{&S27043[13], 3}, {&S27043[17], 20}, {&S27043[59], 17}, {NULL}};
const Segment SGI278251[] = {{&S27044[87], 4}, {NULL}};
const Segment SGI278252[] = {{&S27044[13], 3}, {&S27044[17], 22}, {&S27044[61], 17}, {NULL}};
const Segment SGI278253[] = {{&S27045[87], 4}, {NULL}};
const Segment SGI278254[] = {{&S27045[13], 3}, {&S27045[17], 22}, {&S27045[61], 17}, {NULL}};
const Segment SGI278255[] = {{&S27046[87], 4}, {NULL}};
const Segment SGI278256[] = {{&S27046[13], 3}, {&S27046[17], 22}, {&S27046[61], 17}, {NULL}};
const Segment SGI278257[] = {{&S27047[87], 4}, {NULL}};
const Segment SGI278258[] = {{&S27047[13], 3}, {&S27047[17], 22}, {&S27047[61], 17}, {NULL}};
const Segment SGI278259[] = {{&S27048[85], 4}, {NULL}};
const Segment SGI278260[] = {{&S27048[13], 3}, {&S27048[17], 20}, {&S27048[59], 17}, {NULL}};
const Segment SGI278261[] = {{&S27049[85], 4}, {NULL}};
const Segment SGI278262[] = {{&S27049[13], 3}, {&S27049[17], 20}, {&S27049[59], 17}, {NULL}};
const Segment SGI278263[] = {{&S27050[87], 4}, {NULL}};
const Segment SGI278264[] = {{&S27050[13], 3}, {&S27050[17], 22}, {&S27050[61], 17}, {NULL}};
const Segment SGI278265[] = {{&S27051[87], 4}, {NULL}};
const Segment SGI278266[] = {{&S27051[13], 3}, {&S27051[17], 22}, {&S27051[61], 17}, {NULL}};
const Segment SGI278267[] = {{&S27052[87], 4}, {NULL}};
const Segment SGI278268[] = {{&S27052[13], 3}, {&S27052[17], 22}, {&S27052[61], 17}, {NULL}};
const Segment SGI278269[] = {{&S27053[85], 4}, {NULL}};
const Segment SGI278270[] = {{&S27053[13], 3}, {&S27053[17], 20}, {&S27053[59], 17}, {NULL}};
const Segment SGI278271[] = {{&S27054[87], 4}, {NULL}};
const Segment SGI278272[] = {{&S27054[13], 3}, {&S27054[17], 22}, {&S27054[61], 17}, {NULL}};
const Segment SGI278273[] = {{&S27055[87], 4}, {NULL}};
const Segment SGI278274[] = {{&S27055[13], 3}, {&S27055[17], 22}, {&S27055[61], 17}, {NULL}};
const Segment SGI278275[] = {{&S27056[85], 4}, {NULL}};
const Segment SGI278276[] = {{&S27056[13], 3}, {&S27056[17], 20}, {&S27056[59], 17}, {NULL}};
const Segment SGI278277[] = {{&S27057[87], 4}, {NULL}};
const Segment SGI278278[] = {{&S27057[13], 3}, {&S27057[17], 22}, {&S27057[61], 17}, {NULL}};
const Segment SGI278279[] = {{&S27058[87], 4}, {NULL}};
const Segment SGI278280[] = {{&S27058[13], 3}, {&S27058[17], 22}, {&S27058[61], 17}, {NULL}};
const Segment SGI278281[] = {{&S27059[85], 4}, {NULL}};
const Segment SGI278282[] = {{&S27059[13], 3}, {&S27059[17], 20}, {&S27059[59], 17}, {NULL}};
const Segment SGI278283[] = {{&S27060[87], 4}, {NULL}};
const Segment SGI278284[] = {{&S27060[13], 3}, {&S27060[17], 22}, {&S27060[61], 17}, {NULL}};
const Segment SGI278285[] = {{&S27061[87], 4}, {NULL}};
const Segment SGI278286[] = {{&S27061[13], 3}, {&S27061[17], 22}, {&S27061[61], 17}, {NULL}};
const Segment SGI278287[] = {{&S27062[87], 4}, {NULL}};
const Segment SGI278288[] = {{&S27062[13], 3}, {&S27062[17], 22}, {&S27062[61], 17}, {NULL}};
const Segment SGI278289[] = {{&S27063[85], 4}, {NULL}};
const Segment SGI278290[] = {{&S27063[13], 3}, {&S27063[17], 20}, {&S27063[59], 17}, {NULL}};
const Segment SGI278291[] = {{&S27064[85], 4}, {NULL}};
const Segment SGI278292[] = {{&S27064[13], 3}, {&S27064[17], 20}, {&S27064[59], 17}, {NULL}};
const Segment SGI278293[] = {{&S27065[87], 4}, {NULL}};
const Segment SGI278294[] = {{&S27065[13], 3}, {&S27065[17], 22}, {&S27065[61], 17}, {NULL}};
const Segment SGI278295[] = {{&S27066[87], 4}, {NULL}};
const Segment SGI278296[] = {{&S27066[13], 3}, {&S27066[17], 22}, {&S27066[61], 17}, {NULL}};
const Segment SGI278297[] = {{&S27067[87], 4}, {NULL}};
const Segment SGI278298[] = {{&S27067[13], 3}, {&S27067[17], 22}, {&S27067[61], 17}, {NULL}};
const Segment SGI278299[] = {{&S27068[87], 4}, {NULL}};
const Segment SGI278300[] = {{&S27068[13], 3}, {&S27068[17], 22}, {&S27068[61], 17}, {NULL}};
const Segment SGI278301[] = {{&S27069[88], 4}, {NULL}};
const Segment SGI278302[] = {{&S27069[13], 3}, {&S27069[17], 23}, {&S27069[62], 17}, {NULL}};
const Segment SGI278303[] = {{&S27070[88], 4}, {NULL}};
const Segment SGI278304[] = {{&S27070[13], 3}, {&S27070[17], 23}, {&S27070[62], 17}, {NULL}};
const Segment SGI278305[] = {{&S27071[88], 4}, {NULL}};
const Segment SGI278306[] = {{&S27071[13], 3}, {&S27071[17], 23}, {&S27071[62], 17}, {NULL}};
const Segment SGI278307[] = {{&S27072[85], 4}, {NULL}};
const Segment SGI278308[] = {{&S27072[13], 3}, {&S27072[17], 20}, {&S27072[59], 17}, {&S27072[112], 8}, {NULL}};
const Segment SGI278309[] = {{&S27073[87], 4}, {NULL}};
const Segment SGI278310[] = {{&S27073[13], 3}, {&S27073[17], 22}, {&S27073[61], 17}, {&S27073[114], 8}, {&S27073[125], 26}, {NULL}};
const Segment SGI278311[] = {{&S27074[87], 4}, {NULL}};
const Segment SGI278312[] = {{&S27074[13], 3}, {&S27074[17], 22}, {&S27074[61], 17}, {&S27074[114], 8}, {NULL}};
const Segment SGI278313[] = {{&S27075[87], 4}, {NULL}};
const Segment SGI278314[] = {{&S27075[13], 3}, {&S27075[17], 22}, {&S27075[61], 17}, {NULL}};
const Segment SGI278315[] = {{&S27170[86], 4}, {NULL}};
const Segment SGI278316[] = {{&S27170[13], 3}, {&S27170[17], 21}, {&S27170[60], 17}, {NULL}};
const Segment SGI278317[] = {{&S27171[84], 4}, {NULL}};
const Segment SGI278318[] = {{&S27171[13], 3}, {&S27171[17], 19}, {&S27171[58], 17}, {NULL}};
const Segment SGI278319[] = {{&S27172[86], 4}, {NULL}};
const Segment SGI278320[] = {{&S27172[13], 3}, {&S27172[17], 21}, {&S27172[60], 17}, {NULL}};
const Segment SGI278321[] = {{&S27173[86], 4}, {NULL}};
const Segment SGI278322[] = {{&S27173[13], 3}, {&S27173[17], 21}, {&S27173[60], 17}, {NULL}};
const Segment SGI278323[] = {{&S27174[86], 4}, {NULL}};
const Segment SGI278324[] = {{&S27174[13], 3}, {&S27174[17], 21}, {&S27174[60], 17}, {NULL}};
const Segment SGI278325[] = {{&S27175[84], 4}, {NULL}};
const Segment SGI278326[] = {{&S27175[13], 3}, {&S27175[17], 19}, {&S27175[58], 17}, {NULL}};
const Segment SGI278327[] = {{&S27176[86], 4}, {NULL}};
const Segment SGI278328[] = {{&S27176[13], 3}, {&S27176[17], 21}, {&S27176[60], 17}, {NULL}};
const Segment SGI278329[] = {{&S27177[84], 4}, {NULL}};
const Segment SGI278330[] = {{&S27177[13], 3}, {&S27177[17], 19}, {&S27177[58], 17}, {NULL}};
const Segment SGI278331[] = {{&S27178[86], 4}, {NULL}};
const Segment SGI278332[] = {{&S27178[13], 3}, {&S27178[17], 21}, {&S27178[60], 17}, {NULL}};
const Segment SGI278333[] = {{&S26581[93], 4}, {NULL}};
const Segment SGI278334[] = {{&S26581[13], 9}, {&S26581[23], 22}, {&S26581[67], 17}, {NULL}};
const Segment SGI278335[] = {{&S27179[84], 4}, {NULL}};
const Segment SGI278336[] = {{&S27179[13], 3}, {&S27179[17], 19}, {&S27179[58], 17}, {NULL}};
const Segment SGI278337[] = {{&S26582[93], 4}, {NULL}};
const Segment SGI278338[] = {{&S27180[86], 4}, {NULL}};
const Segment SGI278339[] = {{&S26582[13], 9}, {&S26582[23], 22}, {&S26582[67], 17}, {NULL}};
const Segment SGI278340[] = {{&S27180[13], 3}, {&S27180[17], 21}, {&S27180[60], 17}, {NULL}};
const Segment SGI278341[] = {{&S26583[93], 4}, {NULL}};
const Segment SGI278342[] = {{&S26583[13], 9}, {&S26583[23], 22}, {&S26583[67], 17}, {NULL}};
const Segment SGI278343[] = {{&S27181[86], 4}, {NULL}};
const Segment SGI278344[] = {{&S27181[13], 3}, {&S27181[17], 21}, {&S27181[60], 17}, {NULL}};
const Segment SGI278345[] = {{&S26584[93], 4}, {NULL}};
const Segment SGI278346[] = {{&S26584[13], 9}, {&S26584[23], 22}, {&S26584[67], 17}, {NULL}};
const Segment SGI278347[] = {{&S27182[87], 4}, {NULL}};
const Segment SGI278348[] = {{&S27182[13], 3}, {&S27182[17], 22}, {&S27182[61], 17}, {NULL}};
const Segment SGI278349[] = {{&S26585[93], 4}, {NULL}};
const Segment SGI278350[] = {{&S27183[87], 4}, {NULL}};
const Segment SGI278351[] = {{&S26585[13], 9}, {&S26585[23], 22}, {&S26585[67], 17}, {NULL}};
const Segment SGI278352[] = {{&S27183[13], 3}, {&S27183[17], 22}, {&S27183[61], 17}, {NULL}};
const Segment SGI278353[] = {{&S26586[93], 4}, {NULL}};
const Segment SGI278354[] = {{&S27184[87], 4}, {NULL}};
const Segment SGI278355[] = {{&S26586[13], 9}, {&S26586[23], 22}, {&S26586[67], 17}, {NULL}};
const Segment SGI278356[] = {{&S27184[13], 3}, {&S27184[17], 22}, {&S27184[61], 17}, {NULL}};
const Segment SGI278357[] = {{&S27185[85], 4}, {NULL}};
const Segment SGI278358[] = {{&S27185[13], 3}, {&S27185[17], 20}, {&S27185[59], 17}, {NULL}};
const Segment SGI278359[] = {{&S27186[87], 4}, {NULL}};
const Segment SGI278360[] = {{&S27186[13], 3}, {&S27186[17], 22}, {&S27186[61], 17}, {NULL}};
const Segment SGI278361[] = {{&S27187[85], 4}, {NULL}};
const Segment SGI278362[] = {{&S27187[13], 3}, {&S27187[17], 20}, {&S27187[59], 17}, {NULL}};
const Segment SGI278363[] = {{&S27188[87], 4}, {NULL}};
const Segment SGI278364[] = {{&S27188[13], 3}, {&S27188[17], 22}, {&S27188[61], 17}, {NULL}};
const Segment SGI278365[] = {{&S27189[87], 4}, {NULL}};
const Segment SGI278366[] = {{&S27189[13], 3}, {&S27189[17], 22}, {&S27189[61], 17}, {NULL}};
const Segment SGI278367[] = {{&S27190[85], 4}, {NULL}};
const Segment SGI278368[] = {{&S27190[13], 3}, {&S27190[17], 20}, {&S27190[59], 17}, {NULL}};
const Segment SGI278369[] = {{&S27191[87], 4}, {NULL}};
const Segment SGI278370[] = {{&S27191[13], 3}, {&S27191[17], 22}, {&S27191[61], 17}, {NULL}};
const Segment SGI278371[] = {{&S27192[87], 4}, {NULL}};
const Segment SGI278372[] = {{&S27192[13], 3}, {&S27192[17], 22}, {&S27192[61], 17}, {NULL}};
const Segment SGI278373[] = {{&S27193[87], 4}, {NULL}};
const Segment SGI278374[] = {{&S27193[13], 3}, {&S27193[17], 22}, {&S27193[61], 17}, {NULL}};
const Segment SGI278375[] = {{&S27194[85], 4}, {NULL}};
const Segment SGI278376[] = {{&S27194[13], 3}, {&S27194[17], 20}, {&S27194[59], 17}, {NULL}};
const Segment SGI278377[] = {{&S27195[87], 4}, {NULL}};
const Segment SGI278378[] = {{&S27195[13], 3}, {&S27195[17], 22}, {&S27195[61], 17}, {NULL}};
const Segment SGI278379[] = {{&S27196[85], 4}, {NULL}};
const Segment SGI278380[] = {{&S27196[13], 3}, {&S27196[17], 20}, {&S27196[59], 17}, {NULL}};
const Segment SGI278381[] = {{&S27197[87], 4}, {NULL}};
const Segment SGI278382[] = {{&S27197[13], 3}, {&S27197[17], 22}, {&S27197[61], 17}, {NULL}};
const Segment SGI278383[] = {{&S27198[87], 4}, {NULL}};
const Segment SGI278384[] = {{&S27198[13], 3}, {&S27198[17], 22}, {&S27198[61], 17}, {NULL}};
const Segment SGI278385[] = {{&S27199[85], 4}, {NULL}};
const Segment SGI278386[] = {{&S27199[13], 3}, {&S27199[17], 20}, {&S27199[59], 17}, {NULL}};
const Segment SGI278387[] = {{&S27200[87], 4}, {NULL}};
const Segment SGI278388[] = {{&S27200[13], 3}, {&S27200[17], 22}, {&S27200[61], 17}, {NULL}};
const Segment SGI278389[] = {{&S27201[85], 4}, {NULL}};
const Segment SGI278390[] = {{&S27201[13], 3}, {&S27201[17], 20}, {&S27201[59], 17}, {NULL}};
const Segment SGI278391[] = {{&S27202[87], 4}, {NULL}};
const Segment SGI278392[] = {{&S27202[13], 3}, {&S27202[17], 22}, {&S27202[61], 17}, {NULL}};
const Segment SGI278393[] = {{&S27203[87], 4}, {NULL}};
const Segment SGI278394[] = {{&S27203[13], 3}, {&S27203[17], 22}, {&S27203[61], 17}, {NULL}};
const Segment SGI278395[] = {{&S27204[87], 4}, {NULL}};
const Segment SGI278396[] = {{&S27204[13], 3}, {&S27204[17], 22}, {&S27204[61], 17}, {NULL}};
const Segment SGI278397[] = {{&S27205[87], 4}, {NULL}};
const Segment SGI278398[] = {{&S27205[13], 3}, {&S27205[17], 22}, {&S27205[61], 17}, {NULL}};
const Segment SGI278399[] = {{&S27206[88], 4}, {NULL}};
const Segment SGI278400[] = {{&S27206[13], 3}, {&S27206[17], 23}, {&S27206[62], 17}, {NULL}};
const Segment SGI278401[] = {{&S27207[86], 4}, {NULL}};
const Segment SGI278402[] = {{&S27207[13], 3}, {&S27207[17], 21}, {&S27207[60], 17}, {NULL}};
const Segment SGI278403[] = {{&S27208[88], 4}, {NULL}};
const Segment SGI278404[] = {{&S27208[13], 3}, {&S27208[17], 23}, {&S27208[62], 17}, {NULL}};
const Segment SGI278405[] = {{&S31899[92], 4}, {NULL}};
const Segment SGI278406[] = {{&S31899[13], 9}, {&S31899[23], 22}, {&S31899[66], 17}, {NULL}};
const Segment SGI278407[] = {{&S31900[92], 4}, {NULL}};
const Segment SGI278408[] = {{&S31900[13], 9}, {&S31900[23], 22}, {&S31900[66], 17}, {NULL}};
const Segment SGI278409[] = {{&S31901[92], 4}, {NULL}};
const Segment SGI278410[] = {{&S31901[13], 9}, {&S31901[23], 22}, {&S31901[66], 17}, {NULL}};
const Segment SGI278411[] = {{&S31902[92], 4}, {NULL}};
const Segment SGI278412[] = {{&S31902[13], 9}, {&S31902[23], 22}, {&S31902[66], 17}, {NULL}};
const Segment SGI278413[] = {{&S31903[92], 4}, {NULL}};
const Segment SGI278414[] = {{&S31903[13], 9}, {&S31903[23], 22}, {&S31903[66], 17}, {&S31903[118], 8}, {&S31903[129], 24}, {NULL}};
const Segment SGI278415[] = {{&S31904[92], 4}, {NULL}};
const Segment SGI278416[] = {{&S31904[13], 9}, {&S31904[23], 22}, {&S31904[66], 17}, {NULL}};
const Segment SGI278417[] = {{&S31905[92], 4}, {NULL}};
const Segment SGI278418[] = {{&S31905[13], 9}, {&S31905[23], 22}, {&S31905[66], 17}, {NULL}};
const Segment SGI278419[] = {{&S31906[92], 4}, {NULL}};
const Segment SGI278420[] = {{&S31906[13], 9}, {&S31906[23], 22}, {&S31906[66], 17}, {NULL}};
const Segment SGI278421[] = {{&S31907[92], 4}, {NULL}};
const Segment SGI278422[] = {{&S31907[13], 9}, {&S31907[23], 22}, {&S31907[66], 17}, {NULL}};
const Segment SGI278423[] = {{&S31908[92], 4}, {NULL}};
const Segment SGI278424[] = {{&S31908[13], 9}, {&S31908[23], 22}, {&S31908[66], 17}, {NULL}};
const Segment SGI278425[] = {{&S31909[116], 4}, {NULL}};
const Segment SGI278426[] = {{&S31909[13], 9}, {&S31909[23], 22}, {&S31909[66], 17}, {NULL}};
const Segment SGI278427[] = {{&S31910[89], 4}, {NULL}};
const Segment SGI278428[] = {{&S31910[13], 9}, {&S31910[23], 20}, {&S31910[64], 16}, {NULL}};
const Segment SGI278429[] = {{&S31911[103], 4}, {NULL}};
const Segment SGI278430[] = {{&S31911[13], 9}, {&S31911[23], 2}, {&S31911[26], 22}, {&S31911[49], 6}, {&S31911[77], 17}, {NULL}};
const Segment SGI278431[] = {{&S31912[103], 4}, {NULL}};
const Segment SGI278432[] = {{&S31912[13], 9}, {&S31912[23], 2}, {&S31912[26], 22}, {&S31912[49], 6}, {&S31912[77], 17}, {NULL}};
const Segment SGI278433[] = {{&S31913[103], 4}, {NULL}};
const Segment SGI278434[] = {{&S31913[13], 9}, {&S31913[23], 2}, {&S31913[26], 22}, {&S31913[49], 6}, {&S31913[77], 17}, {NULL}};
const Segment SGI278435[] = {{&S27294[41], 4}, {NULL}};
const Segment SGI278436[] = {{&S27294[13], 14}, {&S27294[28], 25}, {&S27294[74], 17}, {NULL}};
const Segment SGI278437[] = {{&S27295[48], 4}, {NULL}};
const Segment SGI278438[] = {{&S27295[13], 14}, {&S27295[28], 6}, {&S27295[35], 25}, {&S27295[81], 17}, {NULL}};
const Segment SGI278439[] = {{&S27296[48], 4}, {NULL}};
const Segment SGI278440[] = {{&S27296[13], 14}, {&S27296[28], 6}, {&S27296[35], 25}, {&S27296[81], 17}, {NULL}};
const Segment SGI278441[] = {{&S27297[48], 4}, {NULL}};
const Segment SGI278442[] = {{&S27297[13], 14}, {&S27297[28], 6}, {&S27297[35], 25}, {&S27297[81], 17}, {NULL}};
const Segment SGI278443[] = {{&S27298[41], 4}, {NULL}};
const Segment SGI278444[] = {{&S27298[13], 14}, {&S27298[28], 25}, {&S27298[74], 17}, {NULL}};
const Segment SGI278445[] = {{&S27299[48], 4}, {NULL}};
const Segment SGI278446[] = {{&S27299[13], 14}, {&S27299[28], 6}, {&S27299[35], 25}, {&S27299[81], 17}, {NULL}};
const Segment SGI278447[] = {{&S27300[41], 4}, {NULL}};
const Segment SGI278448[] = {{&S27300[13], 14}, {&S27300[28], 25}, {&S27300[74], 17}, {NULL}};
const Segment SGI278449[] = {{&S27301[48], 4}, {NULL}};
const Segment SGI278450[] = {{&S27301[13], 14}, {&S27301[28], 6}, {&S27301[35], 25}, {&S27301[81], 17}, {NULL}};
const Segment SGI278451[] = {{&S27302[41], 4}, {NULL}};
const Segment SGI278452[] = {{&S27302[13], 14}, {&S27302[28], 25}, {&S27302[74], 17}, {NULL}};
const Segment SGI278453[] = {{&S27303[48], 4}, {NULL}};
const Segment SGI278454[] = {{&S27303[13], 14}, {&S27303[28], 6}, {&S27303[35], 25}, {&S27303[81], 17}, {NULL}};
const Segment SGI278455[] = {{&S27304[41], 4}, {NULL}};
const Segment SGI278456[] = {{&S27304[13], 14}, {&S27304[28], 25}, {&S27304[74], 17}, {NULL}};
const Segment SGI278457[] = {{&S27310[48], 4}, {NULL}};
const Segment SGI278458[] = {{&S27310[13], 14}, {&S27310[28], 6}, {&S27310[35], 25}, {&S27310[81], 17}, {NULL}};
const Segment SGI278459[] = {{&S27311[41], 4}, {NULL}};
const Segment SGI278460[] = {{&S27311[13], 14}, {&S27311[28], 25}, {&S27311[74], 17}, {NULL}};
const Segment SGI278461[] = {{&S46512[38], 5}, {NULL}};
const Segment SGI278462[] = {{&S46512[96], 5}, {NULL}};
const Segment SGI278463[] = {{&S26789[75], 4}, {NULL}};
const Segment SGI278464[] = {{&S26789[13], 14}, {&S26789[49], 17}, {NULL}};
const Segment SGI278465[] = {{&S26790[82], 4}, {NULL}};
const Segment SGI278466[] = {{&S26790[13], 14}, {&S26790[28], 6}, {&S26790[56], 17}, {NULL}};
const Segment SGI278467[] = {{&S26791[75], 4}, {NULL}};
const Segment SGI278468[] = {{&S26791[13], 14}, {&S26791[49], 17}, {&S26791[102], 8}, {NULL}};
const Segment SGI278469[] = {{&S32306[75], 4}, {NULL}};
const Segment SGI278470[] = {{&S32306[13], 14}, {&S32306[49], 17}, {NULL}};
const Segment SGI278471[] = {{&S32307[82], 4}, {NULL}};
const Segment SGI278472[] = {{&S32307[13], 14}, {&S32307[28], 6}, {&S32307[56], 17}, {NULL}};
const Segment SGI278473[] = {{&S32308[81], 4}, {NULL}};
const Segment SGI278474[] = {{&S32308[13], 14}, {&S32308[28], 6}, {&S32308[55], 17}, {&S32308[133], 3}, {&S32308[137], 11}, {&S32308[149], 10}, {NULL}};
const Segment SGI278475[] = {{&S32309[74], 4}, {NULL}};
const Segment SGI278476[] = {{&S32309[13], 14}, {&S32309[48], 17}, {&S32309[126], 3}, {&S32309[130], 11}, {&S32309[142], 10}, {NULL}};
const Segment SGI278477[] = {{&S32310[82], 4}, {NULL}};
const Segment SGI278478[] = {{&S32310[13], 14}, {&S32310[28], 6}, {&S32310[56], 17}, {&S32310[126], 8}, {NULL}};
const Segment SGI278479[] = {{&S32311[82], 4}, {NULL}};
const Segment SGI278480[] = {{&S32311[13], 14}, {&S32311[28], 6}, {&S32311[56], 17}, {NULL}};
const Segment SGI278481[] = {{&S27610[93], 4}, {NULL}};
const Segment SGI278482[] = {{&S27610[13], 9}, {&S27610[23], 22}, {&S27610[67], 17}, {&S27610[120], 8}, {&S27610[131], 24}, {NULL}};
const Segment SGI278483[] = {{&S27611[93], 4}, {NULL}};
const Segment SGI278484[] = {{&S27611[13], 9}, {&S27611[23], 22}, {&S27611[67], 17}, {NULL}};
const Segment SGI278485[] = {{&S27612[93], 4}, {NULL}};
const Segment SGI278486[] = {{&S27612[13], 9}, {&S27612[23], 22}, {&S27612[67], 17}, {NULL}};
const Segment SGI278487[] = {{&S27613[93], 4}, {NULL}};
const Segment SGI278488[] = {{&S27613[13], 9}, {&S27613[23], 22}, {&S27613[67], 17}, {NULL}};
const Segment SGI278489[] = {{&S27614[93], 4}, {NULL}};
const Segment SGI278490[] = {{&S27614[13], 9}, {&S27614[23], 22}, {&S27614[67], 17}, {NULL}};
const Segment SGI278491[] = {{&S27615[93], 4}, {NULL}};
const Segment SGI278492[] = {{&S27615[13], 9}, {&S27615[23], 22}, {&S27615[67], 17}, {NULL}};
const Segment SGI278493[] = {{&S27616[93], 4}, {NULL}};
const Segment SGI278494[] = {{&S27616[13], 9}, {&S27616[23], 22}, {&S27616[67], 17}, {NULL}};
const Segment SGI278495[] = {{&S27617[93], 4}, {NULL}};
const Segment SGI278496[] = {{&S27617[13], 9}, {&S27617[23], 22}, {&S27617[67], 17}, {&S27617[120], 8}, {&S27617[131], 26}, {NULL}};
const Segment SGI278497[] = {{&S27618[88], 4}, {NULL}};
const Segment SGI278498[] = {{&S27618[69], 45}, {NULL}};
const Segment SGI278499[] = {{&S27619[90], 4}, {NULL}};
const Segment SGI278500[] = {{&S27619[13], 9}, {&S27619[23], 20}, {&S27619[65], 16}, {NULL}};
const Segment SGI278501[] = {{&S27663[100], 4}, {NULL}};
const Segment SGI278502[] = {{&S27663[13], 14}, {&S27663[28], 6}, {&S27663[55], 17}, {NULL}};
const Segment SGI278503[] = {{&S27663[74], 11}, {NULL}};
const Segment SGI278504[] = {{&S27664[93], 4}, {NULL}};
const Segment SGI278505[] = {{&S27664[13], 14}, {&S27664[48], 17}, {NULL}};
const Segment SGI278506[] = {{&S27664[67], 11}, {NULL}};
const Segment SGI278507[] = {{&S27665[100], 4}, {NULL}};
const Segment SGI278508[] = {{&S27665[13], 14}, {&S27665[28], 6}, {&S27665[55], 17}, {&S27665[126], 8}, {NULL}};
const Segment SGI278509[] = {{&S27665[74], 11}, {NULL}};
const Segment SGI278510[] = {{&S27666[93], 4}, {NULL}};
const Segment SGI278511[] = {{&S27666[13], 14}, {&S27666[48], 17}, {&S27666[119], 8}, {NULL}};
const Segment SGI278512[] = {{&S27666[67], 11}, {NULL}};
const Segment SGI278513[] = {{&S37281[103], 3}, {NULL}};
const Segment SGI278514[] = {{&S37281[13], 9}, {&S37281[23], 2}, {&S37281[26], 22}, {&S37281[49], 6}, {&S37281[77], 17}, {NULL}};
const Segment SGI278515[] = {{&S37282[103], 3}, {NULL}};
const Segment SGI278516[] = {{&S37282[13], 9}, {&S37282[23], 2}, {&S37282[26], 22}, {&S37282[49], 6}, {&S37282[77], 17}, {NULL}};
const Segment SGI278517[] = {{&S37283[103], 3}, {NULL}};
const Segment SGI278518[] = {{&S37283[13], 9}, {&S37283[23], 2}, {&S37283[26], 22}, {&S37283[49], 6}, {&S37283[77], 17}, {NULL}};
const Segment SGI278519[] = {{&S37284[103], 3}, {NULL}};
const Segment SGI278520[] = {{&S37284[13], 9}, {&S37284[23], 2}, {&S37284[26], 22}, {&S37284[49], 6}, {&S37284[77], 17}, {NULL}};
const Segment SGI278521[] = {{&S37285[122], 3}, {NULL}};
const Segment SGI278522[] = {{&S37285[13], 9}, {&S37285[23], 2}, {&S37285[26], 22}, {&S37285[49], 6}, {&S37285[77], 17}, {&S37285[95], 18}, {NULL}};
const Segment SGI278523[] = {{&S37286[103], 3}, {NULL}};
const Segment SGI278524[] = {{&S37286[13], 9}, {&S37286[23], 2}, {&S37286[26], 22}, {&S37286[49], 6}, {&S37286[77], 17}, {NULL}};
const Segment SGI278525[] = {{&S37287[200], 3}, {NULL}};
const Segment SGI278526[] = {{&S37287[13], 9}, {&S37287[23], 2}, {&S37287[26], 22}, {&S37287[49], 6}, {&S37287[79], 17}, {&S37287[13], 9}, {&S37287[23], 2}, {&S37287[26], 22}, {&S37287[49], 3}, {&S37287[79], 17}, {NULL}};
const Segment SGI278527[] = {{&S37288[103], 3}, {NULL}};
const Segment SGI278528[] = {{&S37288[13], 9}, {&S37288[23], 2}, {&S37288[26], 22}, {&S37288[49], 6}, {&S37288[77], 17}, {NULL}};
const Segment SGI278529[] = {{&S37289[103], 3}, {NULL}};
const Segment SGI278530[] = {{&S37289[13], 9}, {&S37289[23], 2}, {&S37289[26], 22}, {&S37289[49], 6}, {&S37289[77], 17}, {NULL}};
const Segment SGI278531[] = {{&S37290[103], 3}, {NULL}};
const Segment SGI278532[] = {{&S37290[13], 9}, {&S37290[23], 2}, {&S37290[26], 22}, {&S37290[49], 6}, {&S37290[77], 17}, {NULL}};
const Segment SGI278533[] = {{&S37291[103], 3}, {NULL}};
const Segment SGI278534[] = {{&S37291[13], 9}, {&S37291[23], 2}, {&S37291[26], 22}, {&S37291[49], 6}, {&S37291[77], 17}, {NULL}};
const Segment SGI278535[] = {{&S37292[119], 3}, {NULL}};
const Segment SGI278536[] = {{&S37292[13], 9}, {&S37292[23], 2}, {&S37292[26], 20}, {&S37292[47], 6}, {&S37292[75], 16}, {&S37292[92], 18}, {NULL}};
const Segment SGI278537[] = {{&S37293[119], 3}, {NULL}};
const Segment SGI278538[] = {{&S37293[13], 9}, {&S37293[23], 2}, {&S37293[26], 20}, {&S37293[47], 6}, {&S37293[75], 16}, {&S37293[92], 18}, {NULL}};
const Segment SGI278539[] = {{&S37294[119], 3}, {NULL}};
const Segment SGI278540[] = {{&S37294[13], 9}, {&S37294[23], 2}, {&S37294[26], 20}, {&S37294[47], 6}, {&S37294[75], 16}, {&S37294[92], 18}, {NULL}};
const Segment SGI278541[] = {{&S37295[120], 3}, {NULL}};
const Segment SGI278542[] = {{&S37295[13], 9}, {&S37295[23], 2}, {&S37295[26], 20}, {&S37295[47], 7}, {&S37295[76], 16}, {&S37295[93], 18}, {NULL}};
const Segment SGI278543[] = {{&S37296[119], 3}, {NULL}};
const Segment SGI278544[] = {{&S37296[13], 9}, {&S37296[23], 2}, {&S37296[26], 20}, {&S37296[47], 6}, {&S37296[75], 16}, {&S37296[92], 18}, {NULL}};
const Segment SGI278545[] = {{&S37297[119], 3}, {NULL}};
const Segment SGI278546[] = {{&S37297[13], 9}, {&S37297[23], 2}, {&S37297[26], 20}, {&S37297[47], 6}, {&S37297[75], 16}, {&S37297[92], 18}, {NULL}};
const Segment SGI278547[] = {{&S37298[119], 3}, {NULL}};
const Segment SGI278548[] = {{&S37298[13], 9}, {&S37298[23], 2}, {&S37298[26], 20}, {&S37298[47], 6}, {&S37298[75], 16}, {&S37298[92], 18}, {NULL}};
const Segment SGI278549[] = {{&S32621[92], 4}, {NULL}};
const Segment SGI278550[] = {{&S32621[13], 9}, {&S32621[23], 22}, {&S32621[66], 17}, {NULL}};
const Segment SGI278551[] = {{&S32622[92], 4}, {NULL}};
const Segment SGI278552[] = {{&S32622[13], 9}, {&S32622[23], 22}, {&S32622[66], 17}, {NULL}};
const Segment SGI278553[] = {{&S32623[92], 4}, {NULL}};
const Segment SGI278554[] = {{&S32623[13], 9}, {&S32623[23], 22}, {&S32623[66], 17}, {NULL}};
const Segment SGI278555[] = {{&S32624[92], 4}, {NULL}};
const Segment SGI278556[] = {{&S32624[13], 9}, {&S32624[23], 22}, {&S32624[66], 17}, {NULL}};
const Segment SGI278557[] = {{&S32625[92], 4}, {NULL}};
const Segment SGI278558[] = {{&S32625[13], 9}, {&S32625[23], 22}, {&S32625[66], 17}, {NULL}};
const Segment SGI278559[] = {{&S32626[92], 4}, {NULL}};
const Segment SGI278560[] = {{&S32626[13], 9}, {&S32626[23], 22}, {&S32626[66], 17}, {NULL}};
const Segment SGI278561[] = {{&S28010[86], 4}, {NULL}};
const Segment SGI278562[] = {{&S28010[13], 14}, {&S28010[48], 17}, {NULL}};
const Segment SGI278563[] = {{&S28010[67], 11}, {NULL}};
const Segment SGI278564[] = {{&S28014[86], 4}, {NULL}};
const Segment SGI278565[] = {{&S28014[13], 14}, {&S28014[48], 17}, {&S28014[112], 8}, {NULL}};
const Segment SGI278566[] = {{&S28014[67], 11}, {NULL}};
const Segment SGI278567[] = {{&S28015[93], 4}, {NULL}};
const Segment SGI278568[] = {{&S28015[13], 14}, {&S28015[28], 6}, {&S28015[55], 17}, {NULL}};
const Segment SGI278569[] = {{&S28015[74], 11}, {NULL}};
const Segment SGI278570[] = {{&S28016[86], 4}, {NULL}};
const Segment SGI278571[] = {{&S28016[13], 14}, {&S28016[48], 17}, {NULL}};
const Segment SGI278572[] = {{&S28016[67], 11}, {NULL}};
const Segment SGI278573[] = {{&S28017[93], 4}, {NULL}};
const Segment SGI278574[] = {{&S28017[13], 14}, {&S28017[28], 6}, {&S28017[55], 17}, {NULL}};
const Segment SGI278575[] = {{&S28017[74], 11}, {NULL}};
const Segment SGI278576[] = {{&S28018[87], 4}, {NULL}};
const Segment SGI278577[] = {{&S28018[13], 14}, {&S28018[49], 17}, {NULL}};
const Segment SGI278578[] = {{&S28018[68], 11}, {NULL}};
const Segment SGI278579[] = {{&S28019[87], 4}, {NULL}};
const Segment SGI278580[] = {{&S28019[13], 14}, {&S28019[49], 17}, {&S28019[114], 8}, {NULL}};
const Segment SGI278581[] = {{&S28019[68], 11}, {NULL}};
const Segment SGI278582[] = {{&S28034[76], 4}, {NULL}};
const Segment SGI278583[] = {{&S28034[13], 3}, {&S28034[17], 11}, {&S28034[50], 17}, {NULL}};
const Segment SGI278584[] = {{&S28035[78], 4}, {NULL}};
const Segment SGI278585[] = {{&S28035[13], 3}, {&S28035[17], 13}, {&S28035[52], 17}, {NULL}};
const Segment SGI278586[] = {{&S28036[85], 4}, {NULL}};
const Segment SGI278587[] = {{&S28036[13], 3}, {&S28036[17], 19}, {&S28036[59], 17}, {NULL}};
const Segment SGI278588[] = {{&S28037[76], 4}, {NULL}};
const Segment SGI278589[] = {{&S28037[13], 3}, {&S28037[17], 11}, {&S28037[50], 17}, {&S28037[103], 8}, {&S28037[114], 24}, {NULL}};
const Segment SGI278590[] = {{&S28038[78], 4}, {NULL}};
const Segment SGI278591[] = {{&S28038[13], 3}, {&S28038[17], 13}, {&S28038[52], 17}, {NULL}};
const Segment SGI278592[] = {{&S28039[83], 4}, {NULL}};
const Segment SGI278593[] = {{&S28039[13], 3}, {&S28039[17], 13}, {&S28039[31], 4}, {&S28039[57], 17}, {NULL}};
const Segment SGI278594[] = {{&S28040[80], 4}, {NULL}};
const Segment SGI278595[] = {{&S28040[13], 3}, {&S28040[17], 10}, {&S28040[28], 4}, {&S28040[54], 17}, {NULL}};
const Segment SGI278596[] = {{&S28041[83], 4}, {NULL}};
const Segment SGI278597[] = {{&S28041[13], 3}, {&S28041[17], 13}, {&S28041[31], 4}, {&S28041[57], 17}, {NULL}};
const Segment SGI278598[] = {{&S28042[80], 4}, {NULL}};
const Segment SGI278599[] = {{&S28042[13], 3}, {&S28042[17], 10}, {&S28042[28], 4}, {&S28042[54], 17}, {NULL}};
const Segment SGI278600[] = {{&S28043[80], 4}, {NULL}};
const Segment SGI278601[] = {{&S28043[13], 3}, {&S28043[17], 10}, {&S28043[28], 4}, {&S28043[54], 17}, {NULL}};
const Segment SGI278602[] = {{&S28044[83], 4}, {NULL}};
const Segment SGI278603[] = {{&S28044[13], 3}, {&S28044[17], 13}, {&S28044[31], 4}, {&S28044[57], 17}, {NULL}};
const Segment SGI278604[] = {{&S28045[80], 4}, {NULL}};
const Segment SGI278605[] = {{&S28045[13], 3}, {&S28045[17], 10}, {&S28045[28], 4}, {&S28045[54], 17}, {NULL}};
const Segment SGI278606[] = {{&S28046[76], 4}, {NULL}};
const Segment SGI278607[] = {{&S28046[13], 3}, {&S28046[17], 11}, {&S28046[50], 17}, {&S28046[104], 28}, {NULL}};
const Segment SGI278608[] = {{&S28047[76], 4}, {NULL}};
const Segment SGI278609[] = {{&S28047[13], 3}, {&S28047[17], 11}, {&S28047[50], 17}, {&S28047[103], 8}, {NULL}};
const Segment SGI278610[] = {{&S28048[78], 4}, {NULL}};
const Segment SGI278611[] = {{&S28048[13], 3}, {&S28048[17], 2}, {&S28048[20], 11}, {&S28048[52], 17}, {NULL}};
const Segment SGI278612[] = {{&S27565[93], 4}, {NULL}};
const Segment SGI278613[] = {{&S32862[13], 7}, {&S32862[21], 2}, {&S32862[24], 15}, {&S32862[54], 17}, {NULL}};
const Segment SGI278614[] = {{&S27565[13], 9}, {&S27565[23], 22}, {&S27565[67], 17}, {NULL}};
const Segment SGI278615[] = {{&S27566[93], 4}, {NULL}};
const Segment SGI278616[] = {{&S27566[13], 9}, {&S27566[23], 22}, {&S27566[67], 17}, {NULL}};
const Segment SGI278617[] = {{&S32863[81], 4}, {NULL}};
const Segment SGI278618[] = {{&S32863[13], 14}, {&S32863[28], 6}, {&S32863[55], 17}, {NULL}};
const Segment SGI278619[] = {{&S27567[93], 4}, {NULL}};
const Segment SGI278620[] = {{&S27567[13], 9}, {&S27567[23], 22}, {&S27567[67], 17}, {&S27567[120], 8}, {NULL}};
const Segment SGI278621[] = {{&S32864[74], 4}, {NULL}};
const Segment SGI278622[] = {{&S32864[13], 14}, {&S32864[48], 17}, {NULL}};
const Segment SGI278623[] = {{&S27568[93], 4}, {NULL}};
const Segment SGI278624[] = {{&S27568[13], 9}, {&S27568[23], 22}, {&S27568[67], 17}, {NULL}};
const Segment SGI278625[] = {{&S32865[74], 4}, {NULL}};
const Segment SGI278626[] = {{&S32865[13], 14}, {&S32865[48], 17}, {&S32865[100], 8}, {&S32865[111], 24}, {NULL}};
const Segment SGI278627[] = {{&S27569[93], 4}, {NULL}};
const Segment SGI278628[] = {{&S27569[13], 9}, {&S27569[23], 22}, {&S27569[67], 17}, {NULL}};
const Segment SGI278629[] = {{&S27570[93], 4}, {NULL}};
const Segment SGI278630[] = {{&S27570[13], 9}, {&S27570[23], 22}, {&S27570[67], 17}, {NULL}};
const Segment SGI278631[] = {{&S27571[93], 4}, {NULL}};
const Segment SGI278632[] = {{&S27571[13], 9}, {&S27571[23], 22}, {&S27571[67], 17}, {NULL}};
const Segment SGI278633[] = {{&S27572[93], 4}, {NULL}};
const Segment SGI278634[] = {{&S27572[13], 9}, {&S27572[23], 22}, {&S27572[67], 17}, {NULL}};
const Segment SGI278635[] = {{&S27573[93], 4}, {NULL}};
const Segment SGI278636[] = {{&S27573[13], 9}, {&S27573[23], 22}, {&S27573[67], 17}, {NULL}};
const Segment SGI278637[] = {{&S27574[93], 4}, {NULL}};
const Segment SGI278638[] = {{&S27574[13], 9}, {&S27574[23], 22}, {&S27574[67], 17}, {NULL}};
const Segment SGI278639[] = {{&S33143[92], 4}, {NULL}};
const Segment SGI278640[] = {{&S33143[13], 9}, {&S33143[23], 22}, {&S33143[66], 17}, {NULL}};
const Segment SGI278641[] = {{&S33144[92], 4}, {NULL}};
const Segment SGI278642[] = {{&S33144[13], 9}, {&S33144[23], 22}, {&S33144[66], 17}, {NULL}};
const Segment SGI278643[] = {{&S33145[92], 4}, {NULL}};
const Segment SGI278644[] = {{&S33145[13], 9}, {&S33145[23], 22}, {&S33145[66], 17}, {NULL}};
const Segment SGI278645[] = {{&S33146[92], 4}, {NULL}};
const Segment SGI278646[] = {{&S33146[13], 9}, {&S33146[23], 22}, {&S33146[66], 17}, {NULL}};
const Segment SGI278647[] = {{&S33147[92], 4}, {NULL}};
const Segment SGI278648[] = {{&S33147[13], 9}, {&S33147[23], 22}, {&S33147[66], 17}, {NULL}};
const Segment SGI278649[] = {{&S33148[92], 4}, {NULL}};
const Segment SGI278650[] = {{&S33148[13], 9}, {&S33148[23], 22}, {&S33148[66], 17}, {NULL}};
const Segment SGI278651[] = {{&S33149[92], 4}, {NULL}};
const Segment SGI278652[] = {{&S33149[13], 9}, {&S33149[23], 22}, {&S33149[66], 17}, {NULL}};
const Segment SGI278653[] = {{&S33150[92], 4}, {NULL}};
const Segment SGI278654[] = {{&S33150[13], 9}, {&S33150[23], 22}, {&S33150[66], 17}, {NULL}};
const Segment SGI278655[] = {{&S33151[92], 4}, {NULL}};
const Segment SGI278656[] = {{&S33151[13], 9}, {&S33151[23], 22}, {&S33151[66], 17}, {NULL}};
const Segment SGI278657[] = {{&S33152[94], 4}, {NULL}};
const Segment SGI278658[] = {{&S33152[13], 9}, {&S33152[23], 24}, {&S33152[68], 17}, {NULL}};
const Segment SGI278659[] = {{&S33153[103], 4}, {NULL}};
const Segment SGI278660[] = {{&S33153[13], 9}, {&S33153[23], 2}, {&S33153[26], 22}, {&S33153[49], 6}, {&S33153[77], 17}, {NULL}};
const Segment SGI278661[] = {{&S33154[103], 4}, {NULL}};
const Segment SGI278662[] = {{&S33154[13], 9}, {&S33154[23], 2}, {&S33154[26], 22}, {&S33154[49], 6}, {&S33154[77], 17}, {NULL}};
const Segment SGI278663[] = {{&S33155[103], 4}, {NULL}};
const Segment SGI278664[] = {{&S33155[13], 9}, {&S33155[23], 2}, {&S33155[26], 22}, {&S33155[49], 6}, {&S33155[77], 17}, {NULL}};
const Segment SGI278665[] = {{&S28443[85], 4}, {NULL}};
const Segment SGI278666[] = {{&S28443[13], 3}, {&S28443[17], 11}, {&S28443[50], 17}, {NULL}};
const Segment SGI278667[] = {{&S28443[69], 15}, {NULL}};
const Segment SGI278668[] = {{&S28444[87], 4}, {NULL}};
const Segment SGI278669[] = {{&S28444[13], 3}, {&S28444[17], 13}, {&S28444[52], 17}, {NULL}};
const Segment SGI278670[] = {{&S28444[71], 15}, {NULL}};
const Segment SGI278671[] = {{&S28445[87], 4}, {NULL}};
const Segment SGI278672[] = {{&S28445[13], 3}, {&S28445[17], 13}, {&S28445[52], 17}, {NULL}};
const Segment SGI278673[] = {{&S28445[71], 15}, {NULL}};
const Segment SGI278674[] = {{&S28446[85], 4}, {NULL}};
const Segment SGI278675[] = {{&S28446[13], 3}, {&S28446[17], 11}, {&S28446[50], 17}, {&S28446[132], 8}, {NULL}};
const Segment SGI278676[] = {{&S28446[69], 15}, {NULL}};
const Segment SGI278677[] = {{&S28447[87], 4}, {NULL}};
const Segment SGI278678[] = {{&S28447[13], 3}, {&S28447[17], 13}, {&S28447[52], 17}, {&S28447[134], 8}, {&S28447[145], 26}, {NULL}};
const Segment SGI278679[] = {{&S28447[71], 15}, {NULL}};
const Segment SGI278680[] = {{&S28448[87], 4}, {NULL}};
const Segment SGI278681[] = {{&S28448[13], 3}, {&S28448[17], 13}, {&S28448[52], 17}, {&S28448[134], 8}, {NULL}};
const Segment SGI278682[] = {{&S28448[71], 15}, {NULL}};
const Segment SGI278683[] = {{&S28449[85], 4}, {NULL}};
const Segment SGI278684[] = {{&S28449[13], 3}, {&S28449[17], 11}, {&S28449[50], 17}, {&S28449[132], 8}, {&S28449[143], 26}, {NULL}};
const Segment SGI278685[] = {{&S28449[69], 15}, {NULL}};
const Segment SGI278686[] = {{&S37839[94], 3}, {NULL}};
const Segment SGI278687[] = {{&S37839[13], 7}, {&S37839[21], 2}, {&S37839[24], 15}, {&S37839[40], 6}, {&S37839[68], 17}, {NULL}};
const Segment SGI278688[] = {{&S37840[13], 7}, {&S37840[21], 2}, {&S37840[24], 15}, {&S37840[40], 6}, {&S37840[67], 17}, {&S37840[95], 21}, {NULL}};
const Segment SGI278689[] = {{&S37841[13], 7}, {&S37841[21], 2}, {&S37841[24], 15}, {&S37841[40], 6}, {&S37841[67], 17}, {&S37841[95], 21}, {NULL}};
const Segment SGI278690[] = {{&S37842[94], 3}, {NULL}};
const Segment SGI278691[] = {{&S37842[13], 7}, {&S37842[21], 2}, {&S37842[24], 15}, {&S37842[40], 6}, {&S37842[68], 17}, {NULL}};
const Segment SGI278692[] = {{&S37843[94], 3}, {NULL}};
const Segment SGI278693[] = {{&S37843[13], 7}, {&S37843[21], 2}, {&S37843[24], 15}, {&S37843[40], 6}, {&S37843[68], 17}, {NULL}};
const Segment SGI278694[] = {{&S37844[13], 7}, {&S37844[21], 2}, {&S37844[24], 15}, {&S37844[40], 6}, {&S37844[67], 17}, {&S37844[95], 21}, {NULL}};
const Segment SGI278695[] = {{&S37845[94], 3}, {NULL}};
const Segment SGI278696[] = {{&S37845[13], 7}, {&S37845[21], 2}, {&S37845[24], 15}, {&S37845[40], 6}, {&S37845[68], 17}, {NULL}};
const Segment SGI278697[] = {{&S37846[13], 14}, {&S37846[48], 17}, {&S37846[76], 21}, {NULL}};
const Segment SGI278698[] = {{&S37847[74], 3}, {NULL}};
const Segment SGI278699[] = {{&S37847[13], 14}, {&S37847[48], 17}, {NULL}};
const Segment SGI278700[] = {{&S48090[38], 5}, {NULL}};
const Segment SGI278701[] = {{&S48090[100], 5}, {NULL}};
const Segment SGI278702[] = {{&S38083[101], 3}, {NULL}};
const Segment SGI278703[] = {{&S38083[13], 3}, {&S38083[17], 2}, {&S38083[20], 11}, {&S38083[32], 6}, {&S38083[60], 17}, {NULL}};
const Segment SGI278704[] = {{&S38083[79], 12}, {NULL}};
const Segment SGI278705[] = {{&S38084[100], 3}, {NULL}};
const Segment SGI278706[] = {{&S38084[13], 3}, {&S38084[17], 2}, {&S38084[20], 13}, {&S38084[34], 6}, {&S38084[61], 17}, {NULL}};
const Segment SGI278707[] = {{&S38084[80], 12}, {NULL}};
const Segment SGI278708[] = {{&S38085[103], 3}, {NULL}};
const Segment SGI278709[] = {{&S38085[13], 3}, {&S38085[17], 2}, {&S38085[20], 13}, {&S38085[34], 6}, {&S38085[62], 17}, {NULL}};
const Segment SGI278710[] = {{&S38085[81], 12}, {NULL}};
const Segment SGI278711[] = {{&S33759[89], 4}, {NULL}};
const Segment SGI278712[] = {{&S33759[13], 3}, {&S33759[17], 2}, {&S33759[20], 13}, {&S33759[34], 6}, {&S33759[62], 17}, {NULL}};
const Segment SGI278713[] = {{&S33759[115], 10}, {NULL}};
const Segment SGI278714[] = {{&S33760[89], 4}, {NULL}};
const Segment SGI278715[] = {{&S33760[13], 3}, {&S33760[17], 2}, {&S33760[20], 13}, {&S33760[34], 6}, {&S33760[62], 17}, {NULL}};
const Segment SGI278716[] = {{&S33760[115], 10}, {NULL}};
const Segment SGI278717[] = {{&S33761[89], 4}, {NULL}};
const Segment SGI278718[] = {{&S33761[13], 3}, {&S33761[17], 2}, {&S33761[20], 13}, {&S33761[34], 6}, {&S33761[62], 17}, {NULL}};
const Segment SGI278719[] = {{&S33761[115], 10}, {NULL}};
const Segment SGI278720[] = {{&S33762[89], 4}, {NULL}};
const Segment SGI278721[] = {{&S33762[13], 3}, {&S33762[17], 2}, {&S33762[20], 13}, {&S33762[34], 6}, {&S33762[62], 17}, {NULL}};
const Segment SGI278722[] = {{&S33762[115], 10}, {NULL}};
const Segment SGI278723[] = {{&S33763[89], 4}, {NULL}};
const Segment SGI278724[] = {{&S33763[13], 3}, {&S33763[17], 2}, {&S33763[20], 13}, {&S33763[34], 6}, {&S33763[62], 17}, {NULL}};
const Segment SGI278725[] = {{&S33763[115], 10}, {NULL}};
const Segment SGI278726[] = {{&S33764[89], 4}, {NULL}};
const Segment SGI278727[] = {{&S33764[13], 3}, {&S33764[17], 2}, {&S33764[20], 13}, {&S33764[34], 6}, {&S33764[62], 17}, {NULL}};
const Segment SGI278728[] = {{&S33764[115], 10}, {NULL}};
const Segment SGI278729[] = {{&S33765[89], 4}, {NULL}};
const Segment SGI278730[] = {{&S33765[13], 3}, {&S33765[17], 2}, {&S33765[20], 13}, {&S33765[34], 6}, {&S33765[62], 17}, {NULL}};
const Segment SGI278731[] = {{&S33765[115], 10}, {NULL}};
const Segment SGI278732[] = {{&S33766[89], 4}, {NULL}};
const Segment SGI278733[] = {{&S33766[13], 3}, {&S33766[17], 2}, {&S33766[20], 13}, {&S33766[34], 6}, {&S33766[62], 17}, {NULL}};
const Segment SGI278734[] = {{&S33766[115], 10}, {NULL}};
const Segment SGI278735[] = {{&S33767[89], 4}, {NULL}};
const Segment SGI278736[] = {{&S33767[13], 3}, {&S33767[17], 2}, {&S33767[20], 13}, {&S33767[34], 6}, {&S33767[62], 17}, {NULL}};
const Segment SGI278737[] = {{&S33767[115], 10}, {NULL}};
const Segment SGI278738[] = {{&S33768[89], 4}, {NULL}};
const Segment SGI278739[] = {{&S33768[13], 3}, {&S33768[17], 2}, {&S33768[20], 13}, {&S33768[34], 6}, {&S33768[62], 17}, {NULL}};
const Segment SGI278740[] = {{&S33768[115], 10}, {NULL}};
const Segment SGI278741[] = {{&S33769[89], 4}, {NULL}};
const Segment SGI278742[] = {{&S33769[13], 3}, {&S33769[17], 2}, {&S33769[20], 13}, {&S33769[34], 6}, {&S33769[62], 17}, {NULL}};
const Segment SGI278743[] = {{&S33769[115], 10}, {NULL}};
const Segment SGI278744[] = {{&S33770[89], 4}, {NULL}};
const Segment SGI278745[] = {{&S33770[13], 3}, {&S33770[17], 2}, {&S33770[20], 13}, {&S33770[34], 6}, {&S33770[62], 17}, {NULL}};
const Segment SGI278746[] = {{&S33770[115], 10}, {NULL}};
const Segment SGI278747[] = {{&S33771[89], 4}, {NULL}};
const Segment SGI278748[] = {{&S33771[13], 3}, {&S33771[17], 2}, {&S33771[20], 13}, {&S33771[34], 6}, {&S33771[62], 17}, {NULL}};
const Segment SGI278749[] = {{&S33771[115], 10}, {NULL}};
const Segment SGI278750[] = {{&S33772[89], 4}, {NULL}};
const Segment SGI278751[] = {{&S33772[13], 3}, {&S33772[17], 2}, {&S33772[20], 13}, {&S33772[34], 6}, {&S33772[62], 17}, {NULL}};
const Segment SGI278752[] = {{&S33772[115], 10}, {NULL}};
const Segment SGI278753[] = {{&S33773[89], 4}, {NULL}};
const Segment SGI278754[] = {{&S33773[13], 3}, {&S33773[17], 2}, {&S33773[20], 13}, {&S33773[34], 6}, {&S33773[62], 17}, {NULL}};
const Segment SGI278755[] = {{&S33773[115], 10}, {NULL}};
const Segment SGI278756[] = {{&S33774[89], 4}, {NULL}};
const Segment SGI278757[] = {{&S33774[13], 3}, {&S33774[17], 2}, {&S33774[20], 13}, {&S33774[34], 6}, {&S33774[62], 17}, {NULL}};
const Segment SGI278758[] = {{&S33774[115], 10}, {NULL}};
const Segment SGI278759[] = {{&S33775[89], 4}, {NULL}};
const Segment SGI278760[] = {{&S33775[13], 3}, {&S33775[17], 2}, {&S33775[20], 13}, {&S33775[34], 6}, {&S33775[62], 17}, {NULL}};
const Segment SGI278761[] = {{&S33775[115], 10}, {NULL}};
const Segment SGI278762[] = {{&S33776[89], 4}, {NULL}};
const Segment SGI278763[] = {{&S33776[13], 3}, {&S33776[17], 2}, {&S33776[20], 13}, {&S33776[34], 6}, {&S33776[62], 17}, {NULL}};
const Segment SGI278764[] = {{&S33776[115], 10}, {NULL}};
const Segment SGI278765[] = {{&S33777[89], 4}, {NULL}};
const Segment SGI278766[] = {{&S33777[13], 3}, {&S33777[17], 2}, {&S33777[20], 13}, {&S33777[34], 6}, {&S33777[62], 17}, {NULL}};
const Segment SGI278767[] = {{&S33777[115], 10}, {NULL}};
const Segment SGI278768[] = {{&S33778[89], 4}, {NULL}};
const Segment SGI278769[] = {{&S33778[13], 3}, {&S33778[17], 2}, {&S33778[20], 13}, {&S33778[34], 6}, {&S33778[62], 17}, {NULL}};
const Segment SGI278770[] = {{&S33778[115], 10}, {NULL}};
const Segment SGI278771[] = {{&S33779[89], 4}, {NULL}};
const Segment SGI278772[] = {{&S33779[13], 3}, {&S33779[17], 2}, {&S33779[20], 13}, {&S33779[34], 6}, {&S33779[62], 17}, {NULL}};
const Segment SGI278773[] = {{&S33779[115], 10}, {NULL}};
const Segment SGI278774[] = {{&S33780[89], 4}, {NULL}};
const Segment SGI278775[] = {{&S33780[13], 3}, {&S33780[17], 2}, {&S33780[20], 13}, {&S33780[34], 6}, {&S33780[62], 17}, {NULL}};
const Segment SGI278776[] = {{&S33780[115], 10}, {NULL}};
const Segment SGI278777[] = {{&S33781[89], 4}, {NULL}};
const Segment SGI278778[] = {{&S33781[13], 3}, {&S33781[17], 2}, {&S33781[20], 13}, {&S33781[34], 6}, {&S33781[62], 17}, {NULL}};
const Segment SGI278779[] = {{&S33781[115], 10}, {NULL}};
const Segment SGI278780[] = {{&S33782[89], 4}, {NULL}};
const Segment SGI278781[] = {{&S33782[13], 3}, {&S33782[17], 2}, {&S33782[20], 13}, {&S33782[34], 6}, {&S33782[62], 17}, {NULL}};
const Segment SGI278782[] = {{&S33782[115], 10}, {NULL}};
const Segment SGI278783[] = {{&S33783[89], 4}, {NULL}};
const Segment SGI278784[] = {{&S33783[13], 3}, {&S33783[17], 2}, {&S33783[20], 13}, {&S33783[34], 6}, {&S33783[62], 17}, {NULL}};
const Segment SGI278785[] = {{&S33783[115], 10}, {NULL}};
const Segment SGI278786[] = {{&S33784[89], 4}, {NULL}};
const Segment SGI278787[] = {{&S33784[13], 3}, {&S33784[17], 2}, {&S33784[20], 13}, {&S33784[34], 6}, {&S33784[62], 17}, {NULL}};
const Segment SGI278788[] = {{&S33784[115], 10}, {NULL}};
const Segment SGI278789[] = {{&S33785[89], 4}, {NULL}};
const Segment SGI278790[] = {{&S33785[13], 3}, {&S33785[17], 2}, {&S33785[20], 13}, {&S33785[34], 6}, {&S33785[62], 17}, {NULL}};
const Segment SGI278791[] = {{&S33785[115], 10}, {NULL}};
const Segment SGI278792[] = {{&S33786[89], 4}, {NULL}};
const Segment SGI278793[] = {{&S33786[13], 3}, {&S33786[17], 2}, {&S33786[20], 13}, {&S33786[34], 6}, {&S33786[62], 17}, {NULL}};
const Segment SGI278794[] = {{&S33786[115], 10}, {NULL}};
const Segment SGI278795[] = {{&S33787[87], 4}, {NULL}};
const Segment SGI278796[] = {{&S33787[13], 3}, {&S33787[17], 2}, {&S33787[20], 13}, {&S33787[34], 4}, {&S33787[60], 17}, {NULL}};
const Segment SGI278797[] = {{&S33787[113], 10}, {NULL}};
const Segment SGI278798[] = {{&S33788[89], 4}, {NULL}};
const Segment SGI278799[] = {{&S33788[13], 3}, {&S33788[17], 2}, {&S33788[20], 13}, {&S33788[34], 6}, {&S33788[62], 17}, {NULL}};
const Segment SGI278800[] = {{&S33788[115], 10}, {NULL}};
const Segment SGI278801[] = {{&S33789[89], 4}, {NULL}};
const Segment SGI278802[] = {{&S33789[13], 3}, {&S33789[17], 2}, {&S33789[20], 13}, {&S33789[34], 6}, {&S33789[62], 17}, {NULL}};
const Segment SGI278803[] = {{&S33789[115], 10}, {NULL}};
const Segment SGI278804[] = {{&S33790[89], 4}, {NULL}};
const Segment SGI278805[] = {{&S33790[13], 3}, {&S33790[17], 2}, {&S33790[20], 13}, {&S33790[34], 6}, {&S33790[62], 17}, {NULL}};
const Segment SGI278806[] = {{&S33790[115], 10}, {NULL}};
const Segment SGI278807[] = {{&S33791[89], 4}, {NULL}};
const Segment SGI278808[] = {{&S33791[13], 3}, {&S33791[17], 2}, {&S33791[20], 13}, {&S33791[34], 6}, {&S33791[62], 17}, {NULL}};
const Segment SGI278809[] = {{&S33791[115], 10}, {NULL}};
const Segment SGI278810[] = {{&S48861[38], 5}, {NULL}};
const Segment SGI278811[] = {{&S48861[100], 5}, {NULL}};
const Segment SGI278812[] = {{&S29795[74], 4}, {NULL}};
const Segment SGI278813[] = {{&S29795[13], 14}, {&S29795[48], 17}, {NULL}};
const Segment SGI278814[] = {{&S29796[81], 4}, {NULL}};
const Segment SGI278815[] = {{&S29796[13], 14}, {&S29796[28], 6}, {&S29796[55], 17}, {NULL}};
const Segment SGI278816[] = {{&S29797[74], 4}, {NULL}};
const Segment SGI278817[] = {{&S29797[13], 14}, {&S29797[48], 17}, {&S29797[100], 8}, {&S29797[111], 24}, {NULL}};
const Segment SGI278818[] = {{&S29798[81], 4}, {NULL}};
const Segment SGI278819[] = {{&S29798[13], 14}, {&S29798[28], 6}, {&S29798[55], 17}, {&S29798[107], 8}, {NULL}};
const Segment SGI278820[] = {{&S29799[74], 4}, {NULL}};
const Segment SGI278821[] = {{&S29799[13], 14}, {&S29799[48], 17}, {&S29799[100], 8}, {NULL}};
const Segment SGI278822[] = {{&S38881[111], 3}, {NULL}};
const Segment SGI278823[] = {{&S38881[13], 3}, {&S38881[17], 2}, {&S38881[20], 21}, {&S38881[42], 6}, {&S38881[69], 17}, {NULL}};
const Segment SGI278824[] = {{&S38882[87], 3}, {NULL}};
const Segment SGI278825[] = {{&S38882[13], 3}, {&S38882[17], 2}, {&S38882[20], 13}, {&S38882[34], 6}, {&S38882[61], 17}, {NULL}};
const Segment SGI278826[] = {{&S38883[85], 3}, {NULL}};
const Segment SGI278827[] = {{&S38883[13], 3}, {&S38883[17], 2}, {&S38883[20], 11}, {&S38883[32], 6}, {&S38883[59], 17}, {NULL}};
const Segment SGI278828[] = {{&S38884[94], 3}, {NULL}};
const Segment SGI278829[] = {{&S38884[13], 3}, {&S38884[17], 2}, {&S38884[20], 19}, {&S38884[41], 6}, {&S38884[68], 17}, {NULL}};
const Segment SGI278830[] = {{&S38885[89], 3}, {NULL}};
const Segment SGI278831[] = {{&S38885[13], 5}, {&S38885[19], 2}, {&S38885[22], 13}, {&S38885[36], 6}, {&S38885[63], 17}, {NULL}};
const Segment SGI278832[] = {{&S38886[87], 3}, {NULL}};
const Segment SGI278833[] = {{&S38886[13], 3}, {&S38886[17], 2}, {&S38886[20], 13}, {&S38886[34], 6}, {&S38886[61], 17}, {NULL}};
const Segment SGI278834[] = {{&S30172[41], 4}, {NULL}};
const Segment SGI278835[] = {{&S30172[13], 14}, {&S30172[28], 25}, {&S30172[74], 17}, {NULL}};
const Segment SGI278836[] = {{&S30173[48], 4}, {NULL}};
const Segment SGI278837[] = {{&S30173[13], 14}, {&S30173[28], 6}, {&S30173[35], 25}, {&S30173[81], 17}, {NULL}};
const Segment SGI278838[] = {{&S30174[41], 4}, {NULL}};
const Segment SGI278839[] = {{&S30174[13], 14}, {&S30174[28], 25}, {&S30174[74], 17}, {NULL}};
const Segment SGI278840[] = {{&S34763[92], 4}, {NULL}};
const Segment SGI278841[] = {{&S34763[13], 9}, {&S34763[23], 22}, {&S34763[66], 17}, {NULL}};
const Segment SGI278842[] = {{&S30175[48], 4}, {NULL}};
const Segment SGI278843[] = {{&S30175[13], 14}, {&S30175[28], 6}, {&S30175[35], 25}, {&S30175[81], 17}, {NULL}};
const Segment SGI278844[] = {{&S34764[92], 4}, {NULL}};
const Segment SGI278845[] = {{&S34764[13], 9}, {&S34764[23], 22}, {&S34764[66], 17}, {NULL}};
const Segment SGI278846[] = {{&S30176[41], 4}, {NULL}};
const Segment SGI278847[] = {{&S30176[13], 14}, {&S30176[28], 25}, {&S30176[74], 17}, {NULL}};
const Segment SGI278848[] = {{&S34765[92], 4}, {NULL}};
const Segment SGI278849[] = {{&S34765[13], 9}, {&S34765[23], 22}, {&S34765[66], 17}, {NULL}};
const Segment SGI278850[] = {{&S34766[92], 4}, {NULL}};
const Segment SGI278851[] = {{&S34766[13], 9}, {&S34766[23], 22}, {&S34766[66], 17}, {NULL}};
const Segment SGI278852[] = {{&S34767[92], 4}, {NULL}};
const Segment SGI278853[] = {{&S34767[13], 9}, {&S34767[23], 22}, {&S34767[66], 17}, {NULL}};
const Segment SGI278854[] = {{&S30181[48], 4}, {NULL}};
const Segment SGI278855[] = {{&S30181[13], 14}, {&S30181[28], 6}, {&S30181[35], 25}, {&S30181[81], 17}, {NULL}};
const Segment SGI278856[] = {{&S34768[92], 4}, {NULL}};
const Segment SGI278857[] = {{&S34768[13], 9}, {&S34768[23], 22}, {&S34768[66], 17}, {NULL}};
const Segment SGI278858[] = {{&S34769[92], 4}, {NULL}};
const Segment SGI278859[] = {{&S34769[13], 9}, {&S34769[23], 22}, {&S34769[66], 17}, {NULL}};
const Segment SGI278860[] = {{&S34770[92], 4}, {NULL}};
const Segment SGI278861[] = {{&S34770[13], 9}, {&S34770[23], 22}, {&S34770[66], 17}, {NULL}};
const Segment SGI278862[] = {{&S34771[92], 4}, {NULL}};
const Segment SGI278863[] = {{&S34771[13], 9}, {&S34771[23], 22}, {&S34771[66], 17}, {NULL}};
const Segment SGI278864[] = {{&S34772[92], 4}, {NULL}};
const Segment SGI278865[] = {{&S34772[13], 9}, {&S34772[23], 22}, {&S34772[66], 17}, {&S34772[117], 8}, {&S34772[128], 24}, {NULL}};
const Segment SGI278866[] = {{&S30187[41], 4}, {NULL}};
const Segment SGI278867[] = {{&S30187[13], 14}, {&S30187[28], 25}, {&S30187[74], 17}, {NULL}};
const Segment SGI278868[] = {{&S34773[92], 4}, {NULL}};
const Segment SGI278869[] = {{&S34773[13], 9}, {&S34773[23], 22}, {&S34773[66], 17}, {NULL}};
const Segment SGI278870[] = {{&S34774[92], 4}, {NULL}};
const Segment SGI278871[] = {{&S34774[13], 9}, {&S34774[23], 22}, {&S34774[66], 17}, {NULL}};
const Segment SGI278872[] = {{&S34775[92], 4}, {NULL}};
const Segment SGI278873[] = {{&S34775[13], 9}, {&S34775[23], 22}, {&S34775[66], 17}, {NULL}};
const Segment SGI278874[] = {{&S34776[103], 4}, {NULL}};
const Segment SGI278875[] = {{&S34776[13], 9}, {&S34776[23], 2}, {&S34776[26], 22}, {&S34776[49], 6}, {&S34776[77], 17}, {NULL}};
const Segment SGI278876[] = {{&S34777[103], 4}, {NULL}};
const Segment SGI278877[] = {{&S34777[13], 9}, {&S34777[23], 2}, {&S34777[26], 22}, {&S34777[49], 6}, {&S34777[77], 17}, {NULL}};
const Segment SGI278878[] = {{&S34778[103], 4}, {NULL}};
const Segment SGI278879[] = {{&S34778[13], 9}, {&S34778[23], 2}, {&S34778[26], 22}, {&S34778[49], 6}, {&S34778[77], 17}, {NULL}};
const Segment SGI278880[] = {{&S34826[95], 4}, {NULL}};
const Segment SGI278881[] = {{&S34826[13], 14}, {&S34826[48], 17}, {NULL}};
const Segment SGI278882[] = {{&S34826[76], 4}, {NULL}};
const Segment SGI278883[] = {{&S34827[102], 4}, {NULL}};
const Segment SGI278884[] = {{&S34827[13], 14}, {&S34827[28], 6}, {&S34827[55], 17}, {NULL}};
const Segment SGI278885[] = {{&S34827[83], 4}, {NULL}};
const Segment SGI278886[] = {{&S34828[95], 4}, {NULL}};
const Segment SGI278887[] = {{&S34828[13], 14}, {&S34828[48], 17}, {&S34828[120], 8}, {NULL}};
const Segment SGI278888[] = {{&S34828[76], 4}, {NULL}};
const Segment SGI278889[] = {{&S34829[102], 4}, {NULL}};
const Segment SGI278890[] = {{&S34829[13], 14}, {&S34829[28], 6}, {&S34829[55], 17}, {&S34829[127], 8}, {NULL}};
const Segment SGI278891[] = {{&S34829[83], 4}, {NULL}};
const Segment SGI278892[] = {{&S34909[92], 4}, {NULL}};
const Segment SGI278893[] = {{&S34909[13], 9}, {&S34909[23], 22}, {&S34909[66], 17}, {NULL}};
const Segment SGI278894[] = {{&S34910[92], 4}, {NULL}};
const Segment SGI278895[] = {{&S34910[13], 9}, {&S34910[23], 22}, {&S34910[66], 17}, {NULL}};
const Segment SGI278896[] = {{&S34911[92], 4}, {NULL}};
const Segment SGI278897[] = {{&S34911[13], 9}, {&S34911[23], 22}, {&S34911[66], 17}, {NULL}};
const Segment SGI278898[] = {{&S34912[92], 4}, {NULL}};
const Segment SGI278899[] = {{&S34912[13], 9}, {&S34912[23], 22}, {&S34912[66], 17}, {&S34912[118], 31}, {NULL}};
const Segment SGI278900[] = {{&S34913[116], 4}, {NULL}};
const Segment SGI278901[] = {{&S34913[13], 9}, {&S34913[23], 22}, {&S34913[66], 17}, {NULL}};
const Segment SGI278902[] = {{&S34914[92], 4}, {NULL}};
const Segment SGI278903[] = {{&S34914[13], 9}, {&S34914[23], 22}, {&S34914[66], 17}, {NULL}};
const Segment SGI278904[] = {{&S34915[92], 4}, {NULL}};
const Segment SGI278905[] = {{&S34915[13], 9}, {&S34915[23], 22}, {&S34915[66], 17}, {NULL}};
const Segment SGI278906[] = {{&S34916[92], 4}, {NULL}};
const Segment SGI278907[] = {{&S34916[13], 9}, {&S34916[23], 22}, {&S34916[66], 17}, {&S34916[118], 29}, {NULL}};
const Segment SGI278908[] = {{&S34917[92], 4}, {NULL}};
const Segment SGI278909[] = {{&S34917[13], 9}, {&S34917[23], 22}, {&S34917[66], 17}, {NULL}};
const Segment SGI278910[] = {{&S34918[92], 4}, {NULL}};
const Segment SGI278911[] = {{&S34918[13], 9}, {&S34918[23], 22}, {&S34918[66], 17}, {&S34918[118], 31}, {NULL}};
const Segment SGI278912[] = {{&S34960[84], 4}, {NULL}};
const Segment SGI278913[] = {{&S34960[13], 3}, {&S34960[17], 19}, {&S34960[58], 17}, {NULL}};
const Segment SGI278914[] = {{&S34961[75], 4}, {NULL}};
const Segment SGI278915[] = {{&S34961[13], 3}, {&S34961[17], 11}, {&S34961[49], 17}, {NULL}};
const Segment SGI278916[] = {{&S34962[99], 4}, {NULL}};
const Segment SGI278917[] = {{&S34962[13], 3}, {&S34962[17], 21}, {&S34962[59], 17}, {NULL}};
const Segment SGI278918[] = {{&S34963[77], 4}, {NULL}};
const Segment SGI278919[] = {{&S34963[13], 3}, {&S34963[17], 13}, {&S34963[51], 17}, {NULL}};
const Segment SGI278920[] = {{&S34964[75], 4}, {NULL}};
const Segment SGI278921[] = {{&S34964[13], 3}, {&S34964[17], 11}, {&S34964[49], 17}, {&S34964[102], 8}, {&S34964[111], 11}, {&S34964[123], 8}, {&S34964[132], 5}, {NULL}};
const Segment SGI278922[] = {{&S34965[80], 4}, {NULL}};
const Segment SGI278923[] = {{&S34965[13], 3}, {&S34965[17], 2}, {&S34965[20], 13}, {&S34965[54], 17}, {NULL}};
const Segment SGI278924[] = {{&S34966[75], 4}, {NULL}};
const Segment SGI278925[] = {{&S34966[13], 3}, {&S34966[17], 11}, {&S34966[49], 17}, {&S34966[101], 8}, {&S34966[112], 24}, {NULL}};
const Segment SGI278926[] = {{&S34967[75], 4}, {NULL}};
const Segment SGI278927[] = {{&S34967[13], 3}, {&S34967[17], 11}, {&S34967[49], 17}, {NULL}};
const Segment SGI278928[] = {{&S34968[83], 4}, {NULL}};
const Segment SGI278929[] = {{&S34968[13], 3}, {&S34968[17], 13}, {&S34968[31], 4}, {&S34968[57], 17}, {NULL}};
const Segment SGI278930[] = {{&S34969[80], 4}, {NULL}};
const Segment SGI278931[] = {{&S34969[13], 3}, {&S34969[17], 10}, {&S34969[28], 4}, {&S34969[54], 17}, {NULL}};
const Segment SGI278932[] = {{&S34970[80], 4}, {NULL}};
const Segment SGI278933[] = {{&S34970[13], 3}, {&S34970[17], 10}, {&S34970[28], 4}, {&S34970[54], 17}, {NULL}};
const Segment SGI278934[] = {{&S34971[80], 4}, {NULL}};
const Segment SGI278935[] = {{&S34971[13], 3}, {&S34971[17], 10}, {&S34971[28], 4}, {&S34971[54], 17}, {NULL}};
const Segment SGI278936[] = {{&S34972[83], 4}, {NULL}};
const Segment SGI278937[] = {{&S34972[13], 3}, {&S34972[17], 13}, {&S34972[31], 4}, {&S34972[57], 17}, {NULL}};
const Segment SGI278938[] = {{&S34973[83], 4}, {NULL}};
const Segment SGI278939[] = {{&S34973[13], 3}, {&S34973[17], 13}, {&S34973[31], 4}, {&S34973[57], 17}, {NULL}};
const Segment SGI278940[] = {{&S34974[80], 4}, {NULL}};
const Segment SGI278941[] = {{&S34974[13], 3}, {&S34974[17], 10}, {&S34974[28], 4}, {&S34974[54], 17}, {NULL}};
const Segment SGI278942[] = {{&S34988[41], 4}, {NULL}};
const Segment SGI278943[] = {{&S34988[13], 14}, {&S34988[28], 23}, {&S34988[72], 17}, {NULL}};
const Segment SGI278944[] = {{&S34988[29], 16}, {NULL}};
const Segment SGI278945[] = {{&S34989[41], 4}, {NULL}};
const Segment SGI278946[] = {{&S34989[13], 14}, {&S34989[28], 24}, {&S34989[74], 17}, {NULL}};
const Segment SGI278947[] = {{&S34989[29], 16}, {NULL}};
const Segment SGI278948[] = {{&S34990[41], 4}, {NULL}};
const Segment SGI278949[] = {{&S34990[13], 14}, {&S34990[28], 24}, {&S34990[74], 17}, {NULL}};
const Segment SGI278950[] = {{&S34990[29], 16}, {NULL}};
const Segment SGI278951[] = {{&S34991[41], 4}, {NULL}};
const Segment SGI278952[] = {{&S34991[13], 14}, {&S34991[28], 24}, {&S34991[74], 17}, {NULL}};
const Segment SGI278953[] = {{&S34991[29], 16}, {NULL}};
const Segment SGI278954[] = {{&S34994[41], 4}, {NULL}};
const Segment SGI278955[] = {{&S34994[13], 14}, {&S34994[28], 24}, {&S34994[74], 17}, {NULL}};
const Segment SGI278956[] = {{&S34994[29], 16}, {NULL}};
const Segment SGI278957[] = {{&S34995[41], 4}, {NULL}};
const Segment SGI278958[] = {{&S34995[13], 14}, {&S34995[28], 24}, {&S34995[74], 17}, {NULL}};
const Segment SGI278959[] = {{&S34995[29], 16}, {NULL}};
const Segment SGI278960[] = {{&S34996[41], 4}, {NULL}};
const Segment SGI278961[] = {{&S34996[13], 14}, {&S34996[28], 24}, {&S34996[74], 17}, {NULL}};
const Segment SGI278962[] = {{&S34996[29], 16}, {NULL}};
const Segment SGI278963[] = {{&S34997[41], 4}, {NULL}};
const Segment SGI278964[] = {{&S34997[13], 14}, {&S34997[28], 24}, {&S34997[74], 17}, {NULL}};
const Segment SGI278965[] = {{&S34997[29], 16}, {NULL}};
const Segment SGI278966[] = {{&S34999[41], 4}, {NULL}};
const Segment SGI278967[] = {{&S34999[13], 14}, {&S34999[28], 24}, {&S34999[74], 17}, {NULL}};
const Segment SGI278968[] = {{&S34999[29], 16}, {NULL}};
const Segment SGI278969[] = {{&S35001[41], 4}, {NULL}};
const Segment SGI278970[] = {{&S35001[13], 14}, {&S35001[28], 24}, {&S35001[74], 17}, {NULL}};
const Segment SGI278971[] = {{&S35001[29], 16}, {NULL}};
const Segment SGI278972[] = {{&S35002[41], 4}, {NULL}};
const Segment SGI278973[] = {{&S35002[13], 14}, {&S35002[28], 24}, {&S35002[74], 17}, {NULL}};
const Segment SGI278974[] = {{&S35002[29], 16}, {NULL}};
const Segment SGI278975[] = {{&S35004[41], 4}, {NULL}};
const Segment SGI278976[] = {{&S35004[13], 14}, {&S35004[28], 24}, {&S35004[74], 17}, {NULL}};
const Segment SGI278977[] = {{&S35004[29], 16}, {NULL}};
const Segment SGI278978[] = {{&S30463[82], 4}, {NULL}};
const Segment SGI278979[] = {{&S30463[13], 14}, {&S30463[28], 6}, {&S30463[56], 17}, {NULL}};
const Segment SGI278980[] = {{&S30463[109], 21}, {NULL}};
const Segment SGI278981[] = {{&S30464[75], 4}, {NULL}};
const Segment SGI278982[] = {{&S30464[13], 14}, {&S30464[49], 17}, {NULL}};
const Segment SGI278983[] = {{&S30464[102], 21}, {NULL}};
const Segment SGI278984[] = {{&S30465[81], 4}, {NULL}};
const Segment SGI278985[] = {{&S30465[13], 14}, {&S30465[28], 6}, {&S30465[55], 17}, {NULL}};
const Segment SGI278986[] = {{&S30465[107], 21}, {NULL}};
const Segment SGI278987[] = {{&S30466[74], 4}, {NULL}};
const Segment SGI278988[] = {{&S30466[13], 14}, {&S30466[48], 17}, {NULL}};
const Segment SGI278989[] = {{&S30466[100], 21}, {NULL}};
const Segment SGI278990[] = {{&S30467[89], 4}, {NULL}};
const Segment SGI278991[] = {{&S30467[13], 14}, {&S30467[28], 14}, {&S30467[63], 17}, {NULL}};
const Segment SGI278992[] = {{&S30468[96], 4}, {NULL}};
const Segment SGI278993[] = {{&S30468[13], 14}, {&S30468[28], 6}, {&S30468[35], 14}, {&S30468[70], 17}, {NULL}};
const Segment SGI278994[] = {{&S30469[96], 4}, {NULL}};
const Segment SGI278995[] = {{&S30469[13], 14}, {&S30469[28], 6}, {&S30469[35], 14}, {&S30469[70], 17}, {NULL}};
const Segment SGI278996[] = {{&S30470[89], 4}, {NULL}};
const Segment SGI278997[] = {{&S30470[13], 14}, {&S30470[28], 14}, {&S30470[63], 17}, {NULL}};
const Segment SGI278998[] = {{&S30471[97], 4}, {NULL}};
const Segment SGI278999[] = {{&S30471[13], 14}, {&S30471[28], 6}, {&S30471[35], 14}, {&S30471[71], 17}, {NULL}};
const Segment SGI279000[] = {{&S30472[90], 4}, {NULL}};
const Segment SGI279001[] = {{&S30472[13], 14}, {&S30472[28], 14}, {&S30472[64], 17}, {NULL}};
const Segment SGI279002[] = {{&S30473[97], 4}, {NULL}};
const Segment SGI279003[] = {{&S30473[13], 14}, {&S30473[28], 6}, {&S30473[35], 14}, {&S30473[71], 17}, {NULL}};
const Segment SGI279004[] = {{&S30474[90], 4}, {NULL}};
const Segment SGI279005[] = {{&S30474[13], 14}, {&S30474[28], 14}, {&S30474[64], 17}, {NULL}};
const Segment SGI279006[] = {{&S30475[90], 4}, {NULL}};
const Segment SGI279007[] = {{&S30475[13], 14}, {&S30475[28], 14}, {&S30475[64], 17}, {NULL}};
const Segment SGI279008[] = {{&S30476[97], 4}, {NULL}};
const Segment SGI279009[] = {{&S30476[13], 14}, {&S30476[28], 6}, {&S30476[35], 14}, {&S30476[71], 17}, {NULL}};
const Segment SGI279010[] = {{&S30477[97], 4}, {NULL}};
const Segment SGI279011[] = {{&S30477[13], 14}, {&S30477[28], 6}, {&S30477[35], 14}, {&S30477[71], 17}, {NULL}};
const Segment SGI279012[] = {{&S30478[90], 4}, {NULL}};
const Segment SGI279013[] = {{&S30478[13], 14}, {&S30478[28], 14}, {&S30478[64], 17}, {NULL}};
const Segment SGI279014[] = {{&S30479[90], 4}, {NULL}};
const Segment SGI279015[] = {{&S30479[13], 14}, {&S30479[28], 14}, {&S30479[64], 17}, {NULL}};
const Segment SGI279016[] = {{&S30480[97], 4}, {NULL}};
const Segment SGI279017[] = {{&S30480[13], 14}, {&S30480[28], 6}, {&S30480[35], 14}, {&S30480[71], 17}, {NULL}};
const Segment SGI279018[] = {{&S30481[90], 4}, {NULL}};
const Segment SGI279019[] = {{&S30481[13], 14}, {&S30481[28], 14}, {&S30481[64], 17}, {NULL}};
const Segment SGI279020[] = {{&S30482[97], 4}, {NULL}};
const Segment SGI279021[] = {{&S30482[13], 14}, {&S30482[28], 6}, {&S30482[35], 14}, {&S30482[71], 17}, {NULL}};
const Segment SGI279022[] = {{&S30483[97], 4}, {NULL}};
const Segment SGI279023[] = {{&S30483[13], 14}, {&S30483[28], 6}, {&S30483[35], 14}, {&S30483[71], 17}, {NULL}};
const Segment SGI279024[] = {{&S30484[90], 4}, {NULL}};
const Segment SGI279025[] = {{&S30484[13], 14}, {&S30484[28], 14}, {&S30484[64], 17}, {NULL}};
const Segment SGI279026[] = {{&S30580[93], 4}, {NULL}};
const Segment SGI279027[] = {{&S30580[13], 14}, {&S30580[48], 17}, {NULL}};
const Segment SGI279028[] = {{&S30580[67], 11}, {NULL}};
const Segment SGI279029[] = {{&S30581[100], 4}, {NULL}};
const Segment SGI279030[] = {{&S30581[13], 14}, {&S30581[28], 6}, {&S30581[55], 17}, {NULL}};
const Segment SGI279031[] = {{&S30581[74], 11}, {NULL}};
const Segment SGI279032[] = {{&S30582[100], 4}, {NULL}};
const Segment SGI279033[] = {{&S30582[13], 14}, {&S30582[28], 6}, {&S30582[55], 17}, {NULL}};
const Segment SGI279034[] = {{&S30582[74], 11}, {NULL}};
const Segment SGI279035[] = {{&S30583[93], 4}, {NULL}};
const Segment SGI279036[] = {{&S30583[13], 14}, {&S30583[48], 17}, {NULL}};
const Segment SGI279037[] = {{&S30583[67], 11}, {NULL}};
const Segment SGI279038[] = {{&S30584[100], 4}, {NULL}};
const Segment SGI279039[] = {{&S30584[13], 14}, {&S30584[28], 6}, {&S30584[55], 17}, {&S30584[126], 8}, {NULL}};
const Segment SGI279040[] = {{&S30584[74], 11}, {NULL}};
const Segment SGI279041[] = {{&S30585[100], 4}, {NULL}};
const Segment SGI279042[] = {{&S30585[13], 14}, {&S30585[28], 6}, {&S30585[55], 17}, {&S30585[126], 8}, {NULL}};
const Segment SGI279043[] = {{&S30585[74], 11}, {NULL}};
const Segment SGI279044[] = {{&S35207[41], 4}, {NULL}};
const Segment SGI279045[] = {{&S35207[13], 14}, {&S35207[28], 25}, {&S35207[75], 17}, {NULL}};
const Segment SGI279046[] = {{&S35207[29], 16}, {NULL}};
const Segment SGI279047[] = {{&S35208[41], 4}, {NULL}};
const Segment SGI279048[] = {{&S35208[13], 14}, {&S35208[28], 26}, {&S35208[76], 17}, {NULL}};
const Segment SGI279049[] = {{&S35208[29], 16}, {NULL}};
const Segment SGI279050[] = {{&S35209[41], 4}, {NULL}};
const Segment SGI279051[] = {{&S35209[13], 14}, {&S35209[28], 26}, {&S35209[76], 17}, {NULL}};
const Segment SGI279052[] = {{&S35209[29], 16}, {NULL}};
const Segment SGI279053[] = {{&S35211[41], 4}, {NULL}};
const Segment SGI279054[] = {{&S35211[13], 14}, {&S35211[28], 26}, {&S35211[76], 17}, {NULL}};
const Segment SGI279055[] = {{&S35211[29], 16}, {NULL}};
const Segment SGI279056[] = {{&S35212[41], 4}, {NULL}};
const Segment SGI279057[] = {{&S35212[13], 14}, {&S35212[28], 24}, {&S35212[74], 17}, {NULL}};
const Segment SGI279058[] = {{&S35212[29], 16}, {NULL}};
const Segment SGI279059[] = {{&S35213[41], 4}, {NULL}};
const Segment SGI279060[] = {{&S35213[13], 14}, {&S35213[28], 26}, {&S35213[76], 17}, {NULL}};
const Segment SGI279061[] = {{&S35213[29], 16}, {NULL}};
const Segment SGI279062[] = {{&S35214[41], 4}, {NULL}};
const Segment SGI279063[] = {{&S35214[13], 14}, {&S35214[28], 26}, {&S35214[76], 17}, {NULL}};
const Segment SGI279064[] = {{&S35214[29], 16}, {NULL}};
const Segment SGI279065[] = {{&S35215[41], 4}, {NULL}};
const Segment SGI279066[] = {{&S35215[13], 14}, {&S35215[28], 26}, {&S35215[76], 17}, {NULL}};
const Segment SGI279067[] = {{&S35215[29], 16}, {NULL}};
const Segment SGI279068[] = {{&S35219[48], 4}, {NULL}};
const Segment SGI279069[] = {{&S35219[13], 14}, {&S35219[28], 6}, {&S35219[35], 26}, {&S35219[83], 17}, {NULL}};
const Segment SGI279070[] = {{&S35219[36], 16}, {NULL}};
const Segment SGI279071[] = {{&S35221[41], 4}, {NULL}};
const Segment SGI279072[] = {{&S35221[13], 14}, {&S35221[28], 26}, {&S35221[76], 17}, {NULL}};
const Segment SGI279073[] = {{&S35221[29], 16}, {NULL}};
const Segment SGI279074[] = {{&S35222[41], 4}, {NULL}};
const Segment SGI279075[] = {{&S35222[13], 14}, {&S35222[28], 26}, {&S35222[76], 17}, {NULL}};
const Segment SGI279076[] = {{&S35222[29], 16}, {NULL}};
const Segment SGI279077[] = {{&S35224[48], 4}, {NULL}};
const Segment SGI279078[] = {{&S35224[13], 14}, {&S35224[28], 6}, {&S35224[35], 26}, {&S35224[83], 17}, {NULL}};
const Segment SGI279079[] = {{&S35224[36], 16}, {NULL}};
const Segment SGI279080[] = {{&S35231[41], 4}, {NULL}};
const Segment SGI279081[] = {{&S35231[13], 14}, {&S35231[28], 25}, {&S35231[74], 17}, {NULL}};
const Segment SGI279082[] = {{&S35231[29], 16}, {NULL}};
const Segment SGI279083[] = {{&S35232[41], 4}, {NULL}};
const Segment SGI279084[] = {{&S35232[13], 14}, {&S35232[28], 25}, {&S35232[74], 17}, {NULL}};
const Segment SGI279085[] = {{&S35232[29], 16}, {NULL}};
const Segment SGI279086[] = {{&S35233[41], 4}, {NULL}};
const Segment SGI279087[] = {{&S35233[13], 14}, {&S35233[28], 25}, {&S35233[74], 17}, {NULL}};
const Segment SGI279088[] = {{&S35233[29], 16}, {NULL}};
const Segment SGI279089[] = {{&S35234[41], 4}, {NULL}};
const Segment SGI279090[] = {{&S35234[13], 14}, {&S35234[28], 25}, {&S35234[74], 17}, {NULL}};
const Segment SGI279091[] = {{&S35234[29], 16}, {NULL}};
const Segment SGI279092[] = {{&S35235[41], 4}, {NULL}};
const Segment SGI279093[] = {{&S35235[13], 14}, {&S35235[28], 25}, {&S35235[74], 17}, {NULL}};
const Segment SGI279094[] = {{&S35235[29], 16}, {NULL}};
const Segment SGI279095[] = {{&S35236[41], 4}, {NULL}};
const Segment SGI279096[] = {{&S35236[13], 14}, {&S35236[28], 25}, {&S35236[74], 17}, {NULL}};
const Segment SGI279097[] = {{&S35236[29], 16}, {NULL}};
const Segment SGI279098[] = {{&S35240[41], 4}, {NULL}};
const Segment SGI279099[] = {{&S35240[13], 14}, {&S35240[28], 25}, {&S35240[74], 17}, {NULL}};
const Segment SGI279100[] = {{&S35240[29], 16}, {NULL}};
const Segment SGI279101[] = {{&S35245[41], 4}, {NULL}};
const Segment SGI279102[] = {{&S35245[13], 14}, {&S35245[28], 25}, {&S35245[74], 17}, {NULL}};
const Segment SGI279103[] = {{&S35245[29], 16}, {NULL}};
const Segment SGI279104[] = {{&S35246[41], 4}, {NULL}};
const Segment SGI279105[] = {{&S35246[13], 14}, {&S35246[28], 25}, {&S35246[74], 17}, {NULL}};
const Segment SGI279106[] = {{&S35246[29], 16}, {NULL}};
const Segment SGI279107[] = {{&S35247[48], 4}, {NULL}};
const Segment SGI279108[] = {{&S35247[13], 14}, {&S35247[28], 6}, {&S35247[35], 25}, {&S35247[81], 17}, {NULL}};
const Segment SGI279109[] = {{&S35247[36], 16}, {NULL}};
const Segment SGI279110[] = {{&S35375[77], 4}, {NULL}};
const Segment SGI279111[] = {{&S35375[13], 3}, {&S35375[17], 13}, {&S35375[51], 17}, {NULL}};
const Segment SGI279112[] = {{&S35376[75], 4}, {NULL}};
const Segment SGI279113[] = {{&S35376[13], 3}, {&S35376[17], 11}, {&S35376[49], 17}, {&S35376[100], 8}, {&S35376[111], 24}, {NULL}};
const Segment SGI279114[] = {{&S35377[75], 4}, {NULL}};
const Segment SGI279115[] = {{&S35377[13], 3}, {&S35377[17], 11}, {&S35377[49], 17}, {NULL}};
const Segment SGI279116[] = {{&S35378[99], 4}, {NULL}};
const Segment SGI279117[] = {{&S35378[13], 3}, {&S35378[17], 21}, {&S35378[59], 17}, {NULL}};
const Segment SGI279118[] = {{&S35379[84], 4}, {NULL}};
const Segment SGI279119[] = {{&S35379[13], 3}, {&S35379[17], 19}, {&S35379[58], 17}, {NULL}};
const Segment SGI279120[] = {{&S35380[147], 4}, {NULL}};
const Segment SGI279121[] = {{&S35380[13], 3}, {&S35380[17], 2}, {&S35380[20], 6}, {&S35380[27], 6}, {&S35380[54], 17}, {&S35380[86], 14}, {&S35380[54], 17}, {NULL}};
const Segment SGI279122[] = {{&S35380[0], 73}, {NULL}};
const Segment SGI279123[] = {{&S35381[75], 4}, {NULL}};
const Segment SGI279124[] = {{&S35381[13], 3}, {&S35381[17], 11}, {&S35381[49], 17}, {NULL}};
const Segment SGI279125[] = {{&S35382[99], 4}, {NULL}};
const Segment SGI279126[] = {{&S35382[13], 3}, {&S35382[17], 21}, {&S35382[59], 17}, {NULL}};
const Segment SGI279127[] = {{&S35383[70], 4}, {NULL}};
const Segment SGI279128[] = {{&S35383[13], 3}, {&S35383[17], 11}, {&S35383[47], 15}, {NULL}};
const Segment SGI279129[] = {{&S35383[0], 1}, {NULL}};
const Segment SGI279130[] = {{&S35384[65], 4}, {NULL}};
const Segment SGI279131[] = {{&S35384[13], 5}, {&S35384[39], 17}, {NULL}};
const Segment SGI279132[] = {{&S35385[80], 4}, {NULL}};
const Segment SGI279133[] = {{&S35385[13], 3}, {&S35385[17], 10}, {&S35385[28], 4}, {&S35385[54], 17}, {NULL}};
const Segment SGI279134[] = {{&S35386[83], 4}, {NULL}};
const Segment SGI279135[] = {{&S35386[13], 3}, {&S35386[17], 13}, {&S35386[31], 4}, {&S35386[57], 17}, {NULL}};
const Segment SGI279136[] = {{&S35387[80], 4}, {NULL}};
const Segment SGI279137[] = {{&S35387[13], 3}, {&S35387[17], 10}, {&S35387[28], 4}, {&S35387[54], 17}, {NULL}};
const Segment SGI279138[] = {{&S35388[83], 4}, {NULL}};
const Segment SGI279139[] = {{&S35388[13], 3}, {&S35388[17], 13}, {&S35388[31], 4}, {&S35388[57], 17}, {NULL}};
const Segment SGI279140[] = {{&S35389[80], 4}, {NULL}};
const Segment SGI279141[] = {{&S35389[13], 3}, {&S35389[17], 10}, {&S35389[28], 4}, {&S35389[54], 17}, {NULL}};
const Segment SGI279142[] = {{&S35390[83], 4}, {NULL}};
const Segment SGI279143[] = {{&S35390[13], 3}, {&S35390[17], 13}, {&S35390[31], 4}, {&S35390[57], 17}, {NULL}};
const Segment SGI279144[] = {{&S35391[80], 4}, {NULL}};
const Segment SGI279145[] = {{&S35391[13], 3}, {&S35391[17], 10}, {&S35391[28], 4}, {&S35391[54], 17}, {NULL}};
const Segment SGI279146[] = {{&S31218[93], 4}, {NULL}};
const Segment SGI279147[] = {{&S31218[13], 14}, {&S31218[48], 17}, {NULL}};
const Segment SGI279148[] = {{&S31218[67], 11}, {NULL}};
const Segment SGI279149[] = {{&S31219[93], 4}, {NULL}};
const Segment SGI279150[] = {{&S31219[13], 14}, {&S31219[48], 17}, {&S31219[119], 8}, {NULL}};
const Segment SGI279151[] = {{&S31219[67], 11}, {NULL}};
const Segment SGI279152[] = {{&S31220[100], 4}, {NULL}};
const Segment SGI279153[] = {{&S31220[13], 14}, {&S31220[28], 6}, {&S31220[55], 17}, {NULL}};
const Segment SGI279154[] = {{&S31220[74], 11}, {NULL}};
const Segment SGI279155[] = {{&S31221[93], 4}, {NULL}};
const Segment SGI279156[] = {{&S31221[13], 14}, {&S31221[48], 17}, {NULL}};
const Segment SGI279157[] = {{&S31221[67], 11}, {NULL}};
const Segment SGI279158[] = {{&S31222[100], 4}, {NULL}};
const Segment SGI279159[] = {{&S31222[13], 14}, {&S31222[28], 6}, {&S31222[55], 17}, {NULL}};
const Segment SGI279160[] = {{&S31222[74], 11}, {NULL}};
const Segment SGI279161[] = {{&S31223[93], 4}, {NULL}};
const Segment SGI279162[] = {{&S31223[13], 14}, {&S31223[48], 17}, {NULL}};
const Segment SGI279163[] = {{&S31223[67], 11}, {NULL}};
const Segment SGI279164[] = {{&S31224[93], 4}, {NULL}};
const Segment SGI279165[] = {{&S31224[13], 14}, {&S31224[48], 17}, {&S31224[119], 8}, {NULL}};
const Segment SGI279166[] = {{&S31224[67], 11}, {NULL}};
const Segment SGI279167[] = {{&S31225[100], 4}, {NULL}};
const Segment SGI279168[] = {{&S31225[13], 14}, {&S31225[28], 6}, {&S31225[55], 17}, {NULL}};
const Segment SGI279169[] = {{&S31225[74], 11}, {NULL}};
const Segment SGI279170[] = {{&S31226[100], 4}, {NULL}};
const Segment SGI279171[] = {{&S31226[13], 14}, {&S31226[28], 6}, {&S31226[55], 17}, {NULL}};
const Segment SGI279172[] = {{&S31226[74], 11}, {NULL}};
const Segment SGI279173[] = {{&S31227[93], 4}, {NULL}};
const Segment SGI279174[] = {{&S31227[13], 14}, {&S31227[48], 17}, {NULL}};
const Segment SGI279175[] = {{&S31227[67], 11}, {NULL}};
const Segment SGI279176[] = {{&S35893[94], 4}, {NULL}};
const Segment SGI279177[] = {{&S35893[13], 7}, {&S35893[21], 2}, {&S35893[24], 15}, {&S35893[40], 6}, {&S35893[68], 17}, {NULL}};
const Segment SGI279178[] = {{&S35893[121], 16}, {NULL}};
const Segment SGI279179[] = {{&S35898[94], 4}, {NULL}};
const Segment SGI279180[] = {{&S35898[13], 7}, {&S35898[21], 2}, {&S35898[24], 15}, {&S35898[40], 6}, {&S35898[68], 17}, {NULL}};
const Segment SGI279181[] = {{&S35898[121], 16}, {NULL}};
const Segment SGI279182[] = {{&S36102[92], 4}, {NULL}};
const Segment SGI279183[] = {{&S36102[13], 9}, {&S36102[23], 22}, {&S36102[66], 17}, {NULL}};
const Segment SGI279184[] = {{&S36103[92], 4}, {NULL}};
const Segment SGI279185[] = {{&S36103[13], 9}, {&S36103[23], 22}, {&S36103[66], 17}, {NULL}};
const Segment SGI279186[] = {{&S36104[92], 4}, {NULL}};
const Segment SGI279187[] = {{&S36104[13], 9}, {&S36104[23], 22}, {&S36104[66], 17}, {NULL}};
const Segment SGI279188[] = {{&S36105[92], 4}, {NULL}};
const Segment SGI279189[] = {{&S36105[13], 9}, {&S36105[23], 22}, {&S36105[66], 17}, {NULL}};
const Segment SGI279190[] = {{&S36106[92], 4}, {NULL}};
const Segment SGI279191[] = {{&S36106[13], 9}, {&S36106[23], 22}, {&S36106[66], 17}, {NULL}};
const Segment SGI279192[] = {{&S36107[92], 4}, {NULL}};
const Segment SGI279193[] = {{&S36107[13], 9}, {&S36107[23], 22}, {&S36107[66], 17}, {NULL}};
const Segment SGI279194[] = {{&S31228[87], 4}, {NULL}};
const Segment SGI279195[] = {{&S31228[13], 14}, {&S31228[49], 17}, {NULL}};
const Segment SGI279196[] = {{&S31228[68], 11}, {NULL}};
const Segment SGI279197[] = {{&S40848[86], 3}, {NULL}};
const Segment SGI279198[] = {{&S40848[13], 3}, {&S40848[17], 2}, {&S40848[20], 13}, {&S40848[34], 6}, {&S40848[61], 16}, {NULL}};
const Segment SGI279199[] = {{&S40849[86], 3}, {NULL}};
const Segment SGI279200[] = {{&S40849[13], 3}, {&S40849[17], 2}, {&S40849[20], 11}, {&S40849[32], 6}, {&S40849[60], 17}, {NULL}};
const Segment SGI279201[] = {{&S40850[98], 3}, {NULL}};
const Segment SGI279202[] = {{&S40850[13], 3}, {&S40850[17], 13}, {&S40850[52], 17}, {&S40850[70], 19}, {NULL}};
const Segment SGI279203[] = {{&S40851[88], 3}, {NULL}};
const Segment SGI279204[] = {{&S40851[13], 3}, {&S40851[17], 2}, {&S40851[20], 13}, {&S40851[34], 6}, {&S40851[62], 17}, {NULL}};
const Segment SGI279205[] = {{&S40852[88], 3}, {NULL}};
const Segment SGI279206[] = {{&S40852[13], 3}, {&S40852[17], 2}, {&S40852[20], 13}, {&S40852[34], 6}, {&S40852[62], 17}, {NULL}};
const Segment SGI279207[] = {{&S40853[95], 3}, {NULL}};
const Segment SGI279208[] = {{&S40853[13], 3}, {&S40853[17], 2}, {&S40853[20], 19}, {&S40853[41], 6}, {&S40853[69], 17}, {NULL}};
const Segment SGI279209[] = {{&S51652[38], 5}, {NULL}};
const Segment SGI279210[] = {{&S51652[100], 5}, {NULL}};
const Segment SGI279211[] = {{&S41000[81], 4}, {&S41000[104], 3}, {NULL}};
const Segment SGI279212[] = {{&S41000[13], 14}, {&S41000[28], 6}, {&S41000[55], 17}, {NULL}};
const Segment SGI279213[] = {{&S41145[94], 3}, {NULL}};
const Segment SGI279214[] = {{&S41145[13], 7}, {&S41145[21], 2}, {&S41145[24], 15}, {&S41145[40], 6}, {&S41145[68], 17}, {NULL}};
const Segment SGI279215[] = {{&S41146[178], 3}, {NULL}};
const Segment SGI279216[] = {{&S41146[13], 7}, {&S41146[21], 2}, {&S41146[24], 43}, {&S41146[68], 6}, {&S41146[124], 17}, {NULL}};
const Segment SGI279217[] = {{&S41192[93], 3}, {NULL}};
const Segment SGI279218[] = {{&S41192[13], 7}, {&S41192[21], 2}, {&S41192[24], 15}, {&S41192[40], 6}, {&S41192[67], 17}, {NULL}};
const Segment SGI279219[] = {{&S41193[75], 3}, {NULL}};
const Segment SGI279220[] = {{&S41193[13], 14}, {&S41193[49], 17}, {NULL}};
const Segment SGI279221[] = {{&S41194[82], 3}, {NULL}};
const Segment SGI279222[] = {{&S41194[13], 14}, {&S41194[28], 6}, {&S41194[56], 17}, {NULL}};
const Segment SGI279223[] = {{&S36532[48], 4}, {NULL}};
const Segment SGI279224[] = {{&S36532[13], 14}, {&S36532[28], 6}, {&S36532[35], 24}, {&S36532[80], 17}, {NULL}};
const Segment SGI279225[] = {{&S36532[36], 16}, {NULL}};
const Segment SGI279226[] = {{&S36533[48], 4}, {NULL}};
const Segment SGI279227[] = {{&S36533[13], 14}, {&S36533[28], 6}, {&S36533[35], 25}, {&S36533[81], 17}, {NULL}};
const Segment SGI279228[] = {{&S36533[36], 16}, {NULL}};
const Segment SGI279229[] = {{&S36534[48], 4}, {NULL}};
const Segment SGI279230[] = {{&S36534[13], 14}, {&S36534[28], 6}, {&S36534[35], 25}, {&S36534[81], 17}, {NULL}};
const Segment SGI279231[] = {{&S36534[36], 16}, {NULL}};
const Segment SGI279232[] = {{&S36535[41], 4}, {NULL}};
const Segment SGI279233[] = {{&S36535[13], 14}, {&S36535[28], 25}, {&S36535[74], 17}, {NULL}};
const Segment SGI279234[] = {{&S36535[29], 16}, {NULL}};
const Segment SGI279235[] = {{&S36536[41], 4}, {NULL}};
const Segment SGI279236[] = {{&S36536[13], 14}, {&S36536[28], 25}, {&S36536[74], 17}, {NULL}};
const Segment SGI279237[] = {{&S36536[29], 16}, {NULL}};
const Segment SGI279238[] = {{&S36537[48], 4}, {NULL}};
const Segment SGI279239[] = {{&S36537[13], 14}, {&S36537[28], 6}, {&S36537[35], 25}, {&S36537[81], 17}, {NULL}};
const Segment SGI279240[] = {{&S36537[36], 16}, {NULL}};
const Segment SGI279241[] = {{&S36544[48], 4}, {NULL}};
const Segment SGI279242[] = {{&S36544[13], 14}, {&S36544[28], 6}, {&S36544[35], 25}, {&S36544[81], 17}, {NULL}};
const Segment SGI279243[] = {{&S36544[36], 16}, {NULL}};
const Segment SGI279244[] = {{&S36545[41], 4}, {NULL}};
const Segment SGI279245[] = {{&S36545[13], 14}, {&S36545[28], 25}, {&S36545[74], 17}, {NULL}};
const Segment SGI279246[] = {{&S36545[29], 16}, {NULL}};
const Segment SGI279247[] = {{&S32644[85], 4}, {NULL}};
const Segment SGI279248[] = {{&S32644[13], 3}, {&S32644[17], 11}, {&S32644[50], 17}, {NULL}};
const Segment SGI279249[] = {{&S32644[69], 15}, {NULL}};
const Segment SGI279250[] = {{&S32645[87], 4}, {NULL}};
const Segment SGI279251[] = {{&S32645[13], 3}, {&S32645[17], 13}, {&S32645[52], 17}, {NULL}};
const Segment SGI279252[] = {{&S32645[71], 15}, {NULL}};
const Segment SGI279253[] = {{&S32646[87], 4}, {NULL}};
const Segment SGI279254[] = {{&S32646[13], 3}, {&S32646[17], 13}, {&S32646[52], 17}, {&S32646[136], 8}, {NULL}};
const Segment SGI279255[] = {{&S32646[71], 15}, {NULL}};
const Segment SGI279256[] = {{&S41524[88], 3}, {NULL}};
const Segment SGI279257[] = {{&S41524[13], 3}, {&S41524[17], 2}, {&S41524[20], 13}, {&S41524[34], 6}, {&S41524[62], 17}, {NULL}};
const Segment SGI279258[] = {{&S32647[85], 4}, {NULL}};
const Segment SGI279259[] = {{&S32647[13], 3}, {&S32647[17], 11}, {&S32647[50], 17}, {&S32647[134], 8}, {NULL}};
const Segment SGI279260[] = {{&S32647[69], 15}, {NULL}};
const Segment SGI279261[] = {{&S41525[86], 3}, {NULL}};
const Segment SGI279262[] = {{&S41525[13], 3}, {&S41525[17], 2}, {&S41525[20], 11}, {&S41525[32], 6}, {&S41525[60], 17}, {NULL}};
const Segment SGI279263[] = {{&S32648[87], 4}, {NULL}};
const Segment SGI279264[] = {{&S32648[13], 3}, {&S32648[17], 13}, {&S32648[52], 17}, {NULL}};
const Segment SGI279265[] = {{&S32648[71], 15}, {NULL}};
const Segment SGI279266[] = {{&S41526[90], 3}, {NULL}};
const Segment SGI279267[] = {{&S41526[13], 5}, {&S41526[19], 2}, {&S41526[22], 14}, {&S41526[37], 6}, {&S41526[64], 17}, {NULL}};
const Segment SGI279268[] = {{&S32649[87], 4}, {NULL}};
const Segment SGI279269[] = {{&S32649[13], 3}, {&S32649[17], 13}, {&S32649[52], 17}, {&S32649[136], 8}, {&S32649[147], 26}, {NULL}};
const Segment SGI279270[] = {{&S32649[71], 15}, {NULL}};
const Segment SGI279271[] = {{&S41527[112], 3}, {NULL}};
const Segment SGI279272[] = {{&S41527[13], 3}, {&S41527[17], 2}, {&S41527[20], 21}, {&S41527[42], 6}, {&S41527[70], 17}, {NULL}};
const Segment SGI279273[] = {{&S32650[85], 4}, {NULL}};
const Segment SGI279274[] = {{&S32650[13], 3}, {&S32650[17], 11}, {&S32650[50], 17}, {&S32650[134], 8}, {&S32650[145], 26}, {NULL}};
const Segment SGI279275[] = {{&S32650[69], 15}, {NULL}};
const Segment SGI279276[] = {{&S41528[109], 3}, {NULL}};
const Segment SGI279277[] = {{&S41528[13], 5}, {&S41528[19], 2}, {&S41528[22], 14}, {&S41528[37], 6}, {&S41528[44], 17}, {&S41528[83], 17}, {NULL}};
const Segment SGI279278[] = {{&S32651[84], 4}, {NULL}};
const Segment SGI279279[] = {{&S32651[13], 3}, {&S32651[17], 11}, {&S32651[50], 16}, {NULL}};
const Segment SGI279280[] = {{&S32651[68], 15}, {NULL}};
const Segment SGI279281[] = {{&S41529[95], 3}, {NULL}};
const Segment SGI279282[] = {{&S41529[13], 3}, {&S41529[17], 2}, {&S41529[20], 19}, {&S41529[41], 6}, {&S41529[69], 17}, {NULL}};
const Segment SGI279283[] = {{&S41530[109], 3}, {NULL}};
const Segment SGI279284[] = {{&S41530[13], 5}, {&S41530[19], 2}, {&S41530[22], 14}, {&S41530[37], 6}, {&S41530[44], 17}, {&S41530[83], 17}, {NULL}};
const Segment SGI279285[] = {{&S41531[86], 3}, {NULL}};
const Segment SGI279286[] = {{&S41531[13], 3}, {&S41531[17], 2}, {&S41531[20], 11}, {&S41531[32], 6}, {&S41531[60], 17}, {NULL}};
const Segment SGI279287[] = {{&S41532[109], 3}, {NULL}};
const Segment SGI279288[] = {{&S41532[13], 5}, {&S41532[19], 2}, {&S41532[22], 14}, {&S41532[37], 6}, {&S41532[44], 17}, {&S41532[83], 17}, {NULL}};
const Segment SGI279289[] = {{&S41533[113], 3}, {NULL}};
const Segment SGI279290[] = {{&S41533[13], 5}, {&S41533[19], 2}, {&S41533[22], 14}, {&S41533[37], 6}, {&S41533[44], 21}, {&S41533[87], 17}, {NULL}};
const Segment SGI279291[] = {{&S36932[93], 4}, {NULL}};
const Segment SGI279292[] = {{&S36932[13], 9}, {&S36932[23], 22}, {&S36932[67], 17}, {NULL}};
const Segment SGI279293[] = {{&S36933[93], 4}, {NULL}};
const Segment SGI279294[] = {{&S36933[13], 9}, {&S36933[23], 22}, {&S36933[67], 17}, {NULL}};
const Segment SGI279295[] = {{&S36934[93], 4}, {NULL}};
const Segment SGI279296[] = {{&S36934[13], 9}, {&S36934[23], 22}, {&S36934[67], 17}, {NULL}};
const Segment SGI279297[] = {{&S36935[93], 4}, {NULL}};
const Segment SGI279298[] = {{&S36935[13], 9}, {&S36935[23], 22}, {&S36935[67], 17}, {NULL}};
const Segment SGI279299[] = {{&S36936[93], 4}, {NULL}};
const Segment SGI279300[] = {{&S36936[13], 9}, {&S36936[23], 22}, {&S36936[67], 17}, {NULL}};
const Segment SGI279301[] = {{&S36937[93], 4}, {NULL}};
const Segment SGI279302[] = {{&S36937[13], 9}, {&S36937[23], 22}, {&S36937[67], 17}, {NULL}};
const Segment SGI279303[] = {{&S36938[93], 4}, {NULL}};
const Segment SGI279304[] = {{&S36938[13], 9}, {&S36938[23], 22}, {&S36938[67], 17}, {NULL}};
const Segment SGI279305[] = {{&S36939[93], 4}, {NULL}};
const Segment SGI279306[] = {{&S36939[13], 9}, {&S36939[23], 22}, {&S36939[67], 17}, {NULL}};
const Segment SGI279307[] = {{&S36940[103], 4}, {NULL}};
const Segment SGI279308[] = {{&S36940[13], 9}, {&S36940[23], 2}, {&S36940[26], 22}, {&S36940[49], 6}, {&S36940[77], 17}, {NULL}};
const Segment SGI279309[] = {{&S36941[103], 4}, {NULL}};
const Segment SGI279310[] = {{&S36941[13], 9}, {&S36941[23], 2}, {&S36941[26], 22}, {&S36941[49], 6}, {&S36941[77], 17}, {NULL}};
const Segment SGI279311[] = {{&S32866[82], 4}, {NULL}};
const Segment SGI279312[] = {{&S32866[13], 14}, {&S32866[28], 6}, {&S32866[56], 17}, {&S32866[128], 12}, {NULL}};
const Segment SGI279313[] = {{&S32866[109], 8}, {NULL}};
const Segment SGI279314[] = {{&S32867[75], 4}, {NULL}};
const Segment SGI279315[] = {{&S32867[13], 14}, {&S32867[49], 17}, {&S32867[121], 12}, {NULL}};
const Segment SGI279316[] = {{&S32867[102], 8}, {NULL}};
const Segment SGI279317[] = {{&S32868[82], 4}, {NULL}};
const Segment SGI279318[] = {{&S32868[13], 14}, {&S32868[28], 6}, {&S32868[56], 17}, {NULL}};
const Segment SGI279319[] = {{&S32868[109], 8}, {NULL}};
const Segment SGI279320[] = {{&S32869[75], 4}, {NULL}};
const Segment SGI279321[] = {{&S32869[13], 14}, {&S32869[49], 17}, {NULL}};
const Segment SGI279322[] = {{&S32869[102], 8}, {NULL}};
const Segment SGI279323[] = {{&S32870[82], 4}, {NULL}};
const Segment SGI279324[] = {{&S32870[13], 14}, {&S32870[28], 6}, {&S32870[56], 17}, {&S32870[128], 19}, {NULL}};
const Segment SGI279325[] = {{&S32870[109], 8}, {NULL}};
const Segment SGI279326[] = {{&S32871[82], 4}, {NULL}};
const Segment SGI279327[] = {{&S32871[13], 14}, {&S32871[28], 6}, {&S32871[56], 17}, {&S32871[128], 15}, {NULL}};
const Segment SGI279328[] = {{&S32871[109], 8}, {NULL}};
const Segment SGI279329[] = {{&S32872[82], 4}, {NULL}};
const Segment SGI279330[] = {{&S32872[13], 14}, {&S32872[28], 6}, {&S32872[56], 17}, {&S32872[127], 8}, {NULL}};
const Segment SGI279331[] = {{&S32872[109], 8}, {NULL}};
const Segment SGI279332[] = {{&S32873[82], 4}, {NULL}};
const Segment SGI279333[] = {{&S32873[13], 14}, {&S32873[28], 6}, {&S32873[56], 17}, {&S32873[128], 10}, {NULL}};
const Segment SGI279334[] = {{&S32873[109], 8}, {NULL}};
const Segment SGI279335[] = {{&S32874[75], 4}, {NULL}};
const Segment SGI279336[] = {{&S32874[13], 14}, {&S32874[49], 17}, {&S32874[121], 19}, {NULL}};
const Segment SGI279337[] = {{&S32874[102], 8}, {NULL}};
const Segment SGI279338[] = {{&S32875[82], 4}, {NULL}};
const Segment SGI279339[] = {{&S32875[13], 14}, {&S32875[28], 6}, {&S32875[56], 17}, {&S32875[128], 19}, {NULL}};
const Segment SGI279340[] = {{&S32875[109], 8}, {NULL}};
const Segment SGI279341[] = {{&S37125[100], 4}, {NULL}};
const Segment SGI279342[] = {{&S37125[13], 14}, {&S37125[28], 6}, {&S37125[55], 17}, {NULL}};
const Segment SGI279343[] = {{&S37125[74], 11}, {NULL}};
const Segment SGI279344[] = {{&S37126[93], 4}, {NULL}};
const Segment SGI279345[] = {{&S37126[13], 14}, {&S37126[48], 17}, {NULL}};
const Segment SGI279346[] = {{&S37126[67], 11}, {NULL}};
const Segment SGI279347[] = {{&S37127[101], 4}, {NULL}};
const Segment SGI279348[] = {{&S37127[13], 14}, {&S37127[28], 6}, {&S37127[56], 17}, {NULL}};
const Segment SGI279349[] = {{&S37127[75], 11}, {NULL}};
const Segment SGI279350[] = {{&S37128[94], 4}, {NULL}};
const Segment SGI279351[] = {{&S37128[13], 14}, {&S37128[49], 17}, {NULL}};
const Segment SGI279352[] = {{&S37128[68], 11}, {NULL}};
const Segment SGI279353[] = {{&S37129[93], 4}, {NULL}};
const Segment SGI279354[] = {{&S37129[13], 14}, {&S37129[49], 16}, {NULL}};
const Segment SGI279355[] = {{&S37129[67], 11}, {NULL}};
const Segment SGI279356[] = {{&S32926[75], 4}, {NULL}};
const Segment SGI279357[] = {{&S32926[13], 14}, {&S32926[49], 17}, {&S32926[102], 8}, {&S32926[113], 26}, {NULL}};
const Segment SGI279358[] = {{&S32927[75], 4}, {NULL}};
const Segment SGI279359[] = {{&S32927[13], 14}, {&S32927[49], 17}, {NULL}};
const Segment SGI279360[] = {{&S32928[82], 4}, {NULL}};
const Segment SGI279361[] = {{&S32928[13], 14}, {&S32928[28], 6}, {&S32928[56], 17}, {NULL}};
const Segment SGI279362[] = {{&S32929[75], 4}, {NULL}};
const Segment SGI279363[] = {{&S32929[13], 14}, {&S32929[49], 17}, {&S32929[102], 8}, {&S32929[113], 24}, {NULL}};
const Segment SGI279364[] = {{&S41822[36], 3}, {NULL}};
const Segment SGI279365[] = {{&S41822[13], 7}, {&S41822[21], 2}, {&S41822[24], 15}, {&S41822[40], 6}, {&S41822[67], 17}, {NULL}};
const Segment SGI279366[] = {{&S41823[36], 3}, {NULL}};
const Segment SGI279367[] = {{&S41823[13], 7}, {&S41823[21], 2}, {&S41823[24], 15}, {&S41823[40], 6}, {&S41823[47], 29}, {&S41823[11], 1}, {&S41823[99], 17}, {NULL}};
const Segment SGI279368[] = {{&S41824[36], 3}, {NULL}};
const Segment SGI279369[] = {{&S41824[13], 7}, {&S41824[21], 2}, {&S41824[24], 15}, {&S41824[40], 6}, {&S41824[47], 22}, {&S41824[11], 1}, {&S41824[92], 17}, {NULL}};
const Segment SGI279370[] = {{&S41825[36], 3}, {NULL}};
const Segment SGI279371[] = {{&S41825[13], 7}, {&S41825[21], 2}, {&S41825[24], 15}, {&S41825[40], 6}, {&S41825[67], 17}, {&S41825[117], 8}, {&S41825[128], 24}, {NULL}};
const Segment SGI279372[] = {{&S41826[36], 3}, {NULL}};
const Segment SGI279373[] = {{&S41826[13], 7}, {&S41826[21], 2}, {&S41826[24], 15}, {&S41826[40], 6}, {&S41826[47], 24}, {&S41826[11], 1}, {&S41826[94], 17}, {NULL}};
const Segment SGI279374[] = {{&S41830[103], 3}, {NULL}};
const Segment SGI279375[] = {{&S41830[13], 9}, {&S41830[23], 2}, {&S41830[26], 22}, {&S41830[49], 6}, {&S41830[77], 17}, {NULL}};
const Segment SGI279376[] = {{&S41831[103], 3}, {NULL}};
const Segment SGI279377[] = {{&S41831[13], 9}, {&S41831[23], 2}, {&S41831[26], 22}, {&S41831[49], 6}, {&S41831[77], 17}, {NULL}};
const Segment SGI279378[] = {{&S41832[103], 3}, {NULL}};
const Segment SGI279379[] = {{&S41832[13], 9}, {&S41832[23], 2}, {&S41832[26], 22}, {&S41832[49], 6}, {&S41832[77], 17}, {NULL}};
const Segment SGI279380[] = {{&S41833[103], 3}, {NULL}};
const Segment SGI279381[] = {{&S41833[13], 9}, {&S41833[23], 2}, {&S41833[26], 22}, {&S41833[49], 6}, {&S41833[77], 17}, {NULL}};
const Segment SGI279382[] = {{&S41834[103], 3}, {NULL}};
const Segment SGI279383[] = {{&S41834[13], 9}, {&S41834[23], 2}, {&S41834[26], 22}, {&S41834[49], 6}, {&S41834[77], 17}, {NULL}};
const Segment SGI279384[] = {{&S41835[103], 3}, {NULL}};
const Segment SGI279385[] = {{&S41835[13], 9}, {&S41835[23], 2}, {&S41835[26], 22}, {&S41835[49], 6}, {&S41835[77], 17}, {NULL}};
const Segment SGI279386[] = {{&S41836[103], 3}, {NULL}};
const Segment SGI279387[] = {{&S41836[13], 9}, {&S41836[23], 2}, {&S41836[26], 22}, {&S41836[49], 6}, {&S41836[77], 17}, {NULL}};
const Segment SGI279388[] = {{&S41837[103], 3}, {NULL}};
const Segment SGI279389[] = {{&S41837[13], 9}, {&S41837[23], 2}, {&S41837[26], 22}, {&S41837[49], 6}, {&S41837[77], 17}, {NULL}};
const Segment SGI279390[] = {{&S41838[103], 3}, {NULL}};
const Segment SGI279391[] = {{&S41838[13], 9}, {&S41838[23], 2}, {&S41838[26], 22}, {&S41838[49], 6}, {&S41838[77], 17}, {NULL}};
const Segment SGI279392[] = {{&S37415[93], 4}, {NULL}};
const Segment SGI279393[] = {{&S37415[13], 3}, {&S37415[17], 13}, {&S37415[52], 17}, {NULL}};
const Segment SGI279394[] = {{&S37415[71], 12}, {NULL}};
const Segment SGI279395[] = {{&S37416[91], 4}, {NULL}};
const Segment SGI279396[] = {{&S37416[13], 3}, {&S37416[17], 11}, {&S37416[50], 17}, {NULL}};
const Segment SGI279397[] = {{&S37416[69], 12}, {NULL}};
const Segment SGI279398[] = {{&S37417[93], 4}, {NULL}};
const Segment SGI279399[] = {{&S37417[13], 3}, {&S37417[17], 13}, {&S37417[52], 17}, {NULL}};
const Segment SGI279400[] = {{&S37417[71], 12}, {NULL}};
const Segment SGI279401[] = {{&S42046[94], 3}, {NULL}};
const Segment SGI279402[] = {{&S42046[13], 7}, {&S42046[21], 2}, {&S42046[24], 15}, {&S42046[40], 6}, {&S42046[68], 17}, {NULL}};
const Segment SGI279403[] = {{&S37418[91], 4}, {NULL}};
const Segment SGI279404[] = {{&S37418[13], 3}, {&S37418[17], 11}, {&S37418[50], 17}, {&S37418[140], 8}, {&S37418[151], 24}, {NULL}};
const Segment SGI279405[] = {{&S37418[69], 12}, {NULL}};
const Segment SGI279406[] = {{&S42047[94], 3}, {NULL}};
const Segment SGI279407[] = {{&S42047[13], 7}, {&S42047[21], 2}, {&S42047[24], 15}, {&S42047[40], 6}, {&S42047[68], 17}, {&S42047[120], 8}, {&S42047[131], 24}, {NULL}};
const Segment SGI279408[] = {{&S37419[93], 4}, {NULL}};
const Segment SGI279409[] = {{&S37419[13], 3}, {&S37419[17], 13}, {&S37419[52], 17}, {&S37419[142], 8}, {&S37419[153], 24}, {NULL}};
const Segment SGI279410[] = {{&S37419[71], 12}, {NULL}};
const Segment SGI279411[] = {{&S37420[91], 4}, {NULL}};
const Segment SGI279412[] = {{&S37420[13], 3}, {&S37420[17], 11}, {&S37420[50], 17}, {&S37420[140], 8}, {NULL}};
const Segment SGI279413[] = {{&S37420[69], 12}, {NULL}};
const Segment SGI279414[] = {{&S42178[8], 3}, {NULL}};
const Segment SGI279415[] = {{&S42178[13], 7}, {&S42178[21], 2}, {&S42178[24], 15}, {&S42178[40], 6}, {&S42178[67], 17}, {NULL}};
const Segment SGI279416[] = {{&S42178[86], 8}, {NULL}};
const Segment SGI279417[] = {{&S37596[93], 4}, {NULL}};
const Segment SGI279418[] = {{&S37596[13], 3}, {&S37596[17], 13}, {&S37596[52], 17}, {NULL}};
const Segment SGI279419[] = {{&S37596[71], 12}, {NULL}};
const Segment SGI279420[] = {{&S37597[91], 4}, {NULL}};
const Segment SGI279421[] = {{&S37597[13], 3}, {&S37597[17], 11}, {&S37597[50], 17}, {NULL}};
const Segment SGI279422[] = {{&S37597[69], 12}, {NULL}};
const Segment SGI279423[] = {{&S37741[41], 4}, {NULL}};
const Segment SGI279424[] = {{&S37741[13], 14}, {&S37741[28], 24}, {&S37741[73], 17}, {NULL}};
const Segment SGI279425[] = {{&S37741[29], 16}, {NULL}};
const Segment SGI279426[] = {{&S37744[41], 4}, {NULL}};
const Segment SGI279427[] = {{&S37744[13], 14}, {&S37744[28], 24}, {&S37744[73], 17}, {NULL}};
const Segment SGI279428[] = {{&S37744[29], 16}, {NULL}};
const Segment SGI279429[] = {{&S37746[41], 4}, {NULL}};
const Segment SGI279430[] = {{&S37746[13], 14}, {&S37746[28], 24}, {&S37746[73], 17}, {NULL}};
const Segment SGI279431[] = {{&S37746[29], 16}, {NULL}};
const Segment SGI279432[] = {{&S37750[41], 4}, {NULL}};
const Segment SGI279433[] = {{&S37750[13], 14}, {&S37750[28], 24}, {&S37750[73], 17}, {NULL}};
const Segment SGI279434[] = {{&S37750[29], 16}, {NULL}};
const Segment SGI279435[] = {{&S37753[41], 4}, {NULL}};
const Segment SGI279436[] = {{&S37753[13], 14}, {&S37753[28], 24}, {&S37753[73], 17}, {NULL}};
const Segment SGI279437[] = {{&S37753[29], 16}, {NULL}};
const Segment SGI279438[] = {{&S37778[48], 4}, {NULL}};
const Segment SGI279439[] = {{&S37778[13], 14}, {&S37778[28], 6}, {&S37778[35], 25}, {&S37778[82], 17}, {NULL}};
const Segment SGI279440[] = {{&S37778[36], 16}, {NULL}};
const Segment SGI279441[] = {{&S37782[41], 4}, {NULL}};
const Segment SGI279442[] = {{&S37782[13], 14}, {&S37782[28], 25}, {&S37782[75], 17}, {NULL}};
const Segment SGI279443[] = {{&S37782[29], 16}, {NULL}};
const Segment SGI279444[] = {{&S37783[48], 4}, {NULL}};
const Segment SGI279445[] = {{&S37783[13], 14}, {&S37783[28], 6}, {&S37783[35], 25}, {&S37783[82], 17}, {NULL}};
const Segment SGI279446[] = {{&S37783[36], 16}, {NULL}};
const Segment SGI279447[] = {{&S37786[48], 4}, {NULL}};
const Segment SGI279448[] = {{&S37786[13], 14}, {&S37786[28], 6}, {&S37786[35], 25}, {&S37786[82], 17}, {NULL}};
const Segment SGI279449[] = {{&S37786[36], 16}, {NULL}};
const Segment SGI279450[] = {{&S37787[48], 4}, {NULL}};
const Segment SGI279451[] = {{&S37787[13], 14}, {&S37787[28], 6}, {&S37787[35], 24}, {&S37787[81], 17}, {NULL}};
const Segment SGI279452[] = {{&S37787[36], 16}, {NULL}};
const Segment SGI279453[] = {{&S37788[41], 4}, {NULL}};
const Segment SGI279454[] = {{&S37788[13], 14}, {&S37788[28], 25}, {&S37788[75], 17}, {NULL}};
const Segment SGI279455[] = {{&S37788[29], 16}, {NULL}};
const Segment SGI279456[] = {{&S37790[41], 4}, {NULL}};
const Segment SGI279457[] = {{&S37790[13], 14}, {&S37790[28], 25}, {&S37790[75], 17}, {NULL}};
const Segment SGI279458[] = {{&S37790[29], 16}, {NULL}};
const Segment SGI279459[] = {{&S33637[93], 4}, {NULL}};
const Segment SGI279460[] = {{&S33637[13], 9}, {&S33637[23], 22}, {&S33637[67], 17}, {NULL}};
const Segment SGI279461[] = {{&S33638[93], 4}, {NULL}};
const Segment SGI279462[] = {{&S33638[13], 9}, {&S33638[23], 22}, {&S33638[67], 17}, {NULL}};
const Segment SGI279463[] = {{&S33639[93], 4}, {NULL}};
const Segment SGI279464[] = {{&S33639[13], 9}, {&S33639[23], 22}, {&S33639[67], 17}, {NULL}};
const Segment SGI279465[] = {{&S33640[93], 4}, {NULL}};
const Segment SGI279466[] = {{&S33640[13], 9}, {&S33640[23], 22}, {&S33640[67], 17}, {NULL}};
const Segment SGI279467[] = {{&S33641[93], 4}, {NULL}};
const Segment SGI279468[] = {{&S33641[13], 9}, {&S33641[23], 22}, {&S33641[67], 17}, {NULL}};
const Segment SGI279469[] = {{&S33642[93], 4}, {NULL}};
const Segment SGI279470[] = {{&S33642[13], 9}, {&S33642[23], 22}, {&S33642[67], 17}, {NULL}};
const Segment SGI279471[] = {{&S33643[93], 4}, {NULL}};
const Segment SGI279472[] = {{&S33643[13], 9}, {&S33643[23], 22}, {&S33643[67], 17}, {NULL}};
const Segment SGI279473[] = {{&S33644[93], 4}, {NULL}};
const Segment SGI279474[] = {{&S33644[13], 9}, {&S33644[23], 22}, {&S33644[67], 17}, {NULL}};
const Segment SGI279475[] = {{&S33645[93], 4}, {NULL}};
const Segment SGI279476[] = {{&S33645[13], 9}, {&S33645[23], 22}, {&S33645[67], 17}, {NULL}};
const Segment SGI279477[] = {{&S33646[93], 4}, {NULL}};
const Segment SGI279478[] = {{&S33646[13], 9}, {&S33646[23], 22}, {&S33646[67], 17}, {&S33646[120], 8}, {&S33646[131], 26}, {NULL}};
const Segment SGI279479[] = {{&S33647[90], 4}, {NULL}};
const Segment SGI279480[] = {{&S33647[13], 9}, {&S33647[23], 20}, {&S33647[65], 16}, {NULL}};
const Segment SGI279481[] = {{&S33648[103], 4}, {NULL}};
const Segment SGI279482[] = {{&S33648[13], 9}, {&S33648[23], 2}, {&S33648[26], 22}, {&S33648[49], 6}, {&S33648[77], 17}, {NULL}};
const Segment SGI279483[] = {{&S33649[103], 4}, {NULL}};
const Segment SGI279484[] = {{&S33649[13], 9}, {&S33649[23], 2}, {&S33649[26], 22}, {&S33649[49], 6}, {&S33649[77], 17}, {NULL}};
const Segment SGI279485[] = {{&S33650[103], 4}, {NULL}};
const Segment SGI279486[] = {{&S33650[13], 9}, {&S33650[23], 2}, {&S33650[26], 22}, {&S33650[49], 6}, {&S33650[77], 17}, {NULL}};
const Segment SGI279487[] = {{&S42712[94], 3}, {NULL}};
const Segment SGI279488[] = {{&S42712[13], 7}, {&S42712[21], 2}, {&S42712[24], 15}, {&S42712[40], 6}, {&S42712[68], 17}, {NULL}};
const Segment SGI279489[] = {{&S42712[120], 15}, {NULL}};
const Segment SGI279490[] = {{&S42860[87], 3}, {NULL}};
const Segment SGI279491[] = {{&S42860[13], 3}, {&S42860[17], 2}, {&S42860[20], 13}, {&S42860[34], 6}, {&S42860[61], 17}, {NULL}};
const Segment SGI279492[] = {{&S42861[85], 3}, {NULL}};
const Segment SGI279493[] = {{&S42861[13], 3}, {&S42861[17], 2}, {&S42861[20], 11}, {&S42861[32], 6}, {&S42861[59], 17}, {NULL}};
const Segment SGI279494[] = {{&S42862[87], 3}, {NULL}};
const Segment SGI279495[] = {{&S42862[13], 3}, {&S42862[17], 2}, {&S42862[20], 13}, {&S42862[34], 6}, {&S42862[61], 17}, {NULL}};
const Segment SGI279496[] = {{&S42863[72], 3}, {NULL}};
const Segment SGI279497[] = {{&S42863[13], 5}, {&S42863[19], 6}, {&S42863[46], 17}, {NULL}};
const Segment SGI279498[] = {{&S42864[94], 3}, {NULL}};
const Segment SGI279499[] = {{&S42864[13], 3}, {&S42864[17], 2}, {&S42864[20], 19}, {&S42864[41], 6}, {&S42864[68], 17}, {NULL}};
const Segment SGI279500[] = {{&S42865[85], 3}, {NULL}};
const Segment SGI279501[] = {{&S42865[13], 3}, {&S42865[17], 2}, {&S42865[20], 11}, {&S42865[32], 6}, {&S42865[59], 17}, {NULL}};
const Segment SGI279502[] = {{&S42866[157], 3}, {NULL}};
const Segment SGI279503[] = {{&S42866[13], 3}, {&S42866[17], 2}, {&S42866[20], 13}, {&S42866[34], 6}, {&S42866[41], 12}, {&S42866[76], 7}, {&S42866[17], 2}, {&S42866[87], 15}, {&S42866[34], 6}, {&S42866[131], 17}, {NULL}};
const Segment SGI279504[] = {{&S37996[91], 4}, {NULL}};
const Segment SGI279505[] = {{&S37996[13], 3}, {&S37996[17], 11}, {&S37996[50], 17}, {NULL}};
const Segment SGI279506[] = {{&S37996[69], 12}, {NULL}};
const Segment SGI279507[] = {{&S37997[93], 4}, {NULL}};
const Segment SGI279508[] = {{&S37997[13], 3}, {&S37997[17], 13}, {&S37997[52], 17}, {NULL}};
const Segment SGI279509[] = {{&S37997[71], 12}, {NULL}};
const Segment SGI279510[] = {{&S37998[91], 4}, {NULL}};
const Segment SGI279511[] = {{&S37998[13], 3}, {&S37998[17], 11}, {&S37998[50], 17}, {&S37998[140], 8}, {&S37998[151], 24}, {NULL}};
const Segment SGI279512[] = {{&S37998[69], 12}, {NULL}};
const Segment SGI279513[] = {{&S37999[91], 4}, {NULL}};
const Segment SGI279514[] = {{&S37999[13], 3}, {&S37999[17], 11}, {&S37999[50], 17}, {&S37999[140], 8}, {NULL}};
const Segment SGI279515[] = {{&S37999[69], 12}, {NULL}};
const Segment SGI279516[] = {{&S38188[82], 4}, {NULL}};
const Segment SGI279517[] = {{&S38188[13], 14}, {&S38188[28], 6}, {&S38188[56], 17}, {NULL}};
const Segment SGI279518[] = {{&S38189[70], 4}, {NULL}};
const Segment SGI279519[] = {{&S38189[13], 9}, {&S38189[44], 17}, {NULL}};
const Segment SGI279520[] = {{&S38190[75], 4}, {NULL}};
const Segment SGI279521[] = {{&S38190[13], 14}, {&S38190[49], 17}, {NULL}};
const Segment SGI279522[] = {{&S38191[93], 4}, {NULL}};
const Segment SGI279523[] = {{&S38191[13], 7}, {&S38191[21], 2}, {&S38191[24], 15}, {&S38191[40], 6}, {&S38191[67], 17}, {NULL}};
const Segment SGI279524[] = {{&S38192[69], 4}, {NULL}};
const Segment SGI279525[] = {{&S38192[13], 9}, {&S38192[44], 16}, {NULL}};
const Segment SGI279526[] = {{&S38257[93], 4}, {NULL}};
const Segment SGI279527[] = {{&S38257[13], 9}, {&S38257[23], 22}, {&S38257[67], 17}, {NULL}};
const Segment SGI279528[] = {{&S38258[93], 4}, {NULL}};
const Segment SGI279529[] = {{&S38258[13], 9}, {&S38258[23], 22}, {&S38258[67], 17}, {NULL}};
const Segment SGI279530[] = {{&S38259[93], 4}, {NULL}};
const Segment SGI279531[] = {{&S38259[13], 9}, {&S38259[23], 22}, {&S38259[67], 17}, {NULL}};
const Segment SGI279532[] = {{&S38260[93], 5}, {NULL}};
const Segment SGI279533[] = {{&S38260[13], 9}, {&S38260[23], 22}, {&S38260[67], 17}, {NULL}};
const Segment SGI279534[] = {{&S38261[93], 4}, {NULL}};
const Segment SGI279535[] = {{&S38261[13], 9}, {&S38261[23], 22}, {&S38261[67], 17}, {NULL}};
const Segment SGI279536[] = {{&S38262[91], 4}, {NULL}};
const Segment SGI279537[] = {{&S38262[13], 9}, {&S38262[23], 20}, {&S38262[65], 17}, {NULL}};
const Segment SGI279538[] = {{&S38263[93], 5}, {NULL}};
const Segment SGI279539[] = {{&S38263[13], 9}, {&S38263[23], 22}, {&S38263[67], 17}, {NULL}};
const Segment SGI279540[] = {{&S38264[93], 4}, {NULL}};
const Segment SGI279541[] = {{&S38264[13], 9}, {&S38264[23], 22}, {&S38264[67], 17}, {NULL}};
const Segment SGI279542[] = {{&S38265[93], 4}, {NULL}};
const Segment SGI279543[] = {{&S38265[13], 9}, {&S38265[23], 22}, {&S38265[67], 17}, {NULL}};
const Segment SGI279544[] = {{&S34416[93], 4}, {NULL}};
const Segment SGI279545[] = {{&S34416[13], 9}, {&S34416[23], 22}, {&S34416[67], 17}, {NULL}};
const Segment SGI279546[] = {{&S34417[93], 4}, {NULL}};
const Segment SGI279547[] = {{&S34417[13], 9}, {&S34417[23], 22}, {&S34417[67], 17}, {NULL}};
const Segment SGI279548[] = {{&S34418[93], 4}, {NULL}};
const Segment SGI279549[] = {{&S34418[13], 9}, {&S34418[23], 22}, {&S34418[67], 17}, {NULL}};
const Segment SGI279550[] = {{&S34419[93], 4}, {NULL}};
const Segment SGI279551[] = {{&S34419[13], 9}, {&S34419[23], 22}, {&S34419[67], 17}, {NULL}};
const Segment SGI279552[] = {{&S34420[90], 4}, {NULL}};
const Segment SGI279553[] = {{&S34420[13], 9}, {&S34420[23], 20}, {&S34420[65], 16}, {NULL}};
const Segment SGI279554[] = {{&S34421[93], 4}, {NULL}};
const Segment SGI279555[] = {{&S34421[13], 9}, {&S34421[23], 22}, {&S34421[67], 17}, {NULL}};
const Segment SGI279556[] = {{&S34422[93], 4}, {NULL}};
const Segment SGI279557[] = {{&S34422[13], 9}, {&S34422[23], 22}, {&S34422[67], 17}, {NULL}};
const Segment SGI279558[] = {{&S38410[101], 4}, {NULL}};
const Segment SGI279559[] = {{&S38410[13], 3}, {&S38410[17], 2}, {&S38410[20], 11}, {&S38410[32], 6}, {&S38410[60], 17}, {NULL}};
const Segment SGI279560[] = {{&S38410[79], 12}, {NULL}};
const Segment SGI279561[] = {{&S38411[103], 4}, {NULL}};
const Segment SGI279562[] = {{&S38411[13], 3}, {&S38411[17], 2}, {&S38411[20], 13}, {&S38411[34], 6}, {&S38411[62], 17}, {NULL}};
const Segment SGI279563[] = {{&S38411[81], 12}, {NULL}};
const Segment SGI279564[] = {{&S38412[103], 4}, {NULL}};
const Segment SGI279565[] = {{&S38412[13], 3}, {&S38412[17], 2}, {&S38412[20], 13}, {&S38412[34], 6}, {&S38412[62], 17}, {NULL}};
const Segment SGI279566[] = {{&S38412[81], 12}, {NULL}};
const Segment SGI279567[] = {{&S38413[77], 4}, {NULL}};
const Segment SGI279568[] = {{&S38413[13], 3}, {&S38413[17], 13}, {&S38413[51], 17}, {NULL}};
const Segment SGI279569[] = {{&S38414[84], 4}, {NULL}};
const Segment SGI279570[] = {{&S38414[13], 3}, {&S38414[17], 19}, {&S38414[58], 17}, {NULL}};
const Segment SGI279571[] = {{&S38415[99], 4}, {NULL}};
const Segment SGI279572[] = {{&S38415[13], 3}, {&S38415[17], 21}, {&S38415[59], 17}, {NULL}};
const Segment SGI279573[] = {{&S38416[75], 4}, {NULL}};
const Segment SGI279574[] = {{&S38416[13], 3}, {&S38416[17], 11}, {&S38416[49], 17}, {NULL}};
const Segment SGI279575[] = {{&S38417[80], 4}, {NULL}};
const Segment SGI279576[] = {{&S38417[13], 3}, {&S38417[17], 10}, {&S38417[28], 4}, {&S38417[54], 17}, {NULL}};
const Segment SGI279577[] = {{&S38418[80], 4}, {NULL}};
const Segment SGI279578[] = {{&S38418[13], 3}, {&S38418[17], 10}, {&S38418[28], 4}, {&S38418[54], 17}, {NULL}};
const Segment SGI279579[] = {{&S38419[83], 4}, {NULL}};
const Segment SGI279580[] = {{&S38419[13], 3}, {&S38419[17], 13}, {&S38419[31], 4}, {&S38419[57], 17}, {NULL}};
const Segment SGI279581[] = {{&S38420[83], 4}, {NULL}};
const Segment SGI279582[] = {{&S38420[13], 3}, {&S38420[17], 13}, {&S38420[31], 4}, {&S38420[57], 17}, {NULL}};
const Segment SGI279583[] = {{&S38421[80], 4}, {NULL}};
const Segment SGI279584[] = {{&S38421[13], 3}, {&S38421[17], 10}, {&S38421[28], 4}, {&S38421[54], 17}, {NULL}};
const Segment SGI279585[] = {{&S38422[83], 4}, {NULL}};
const Segment SGI279586[] = {{&S38422[13], 3}, {&S38422[17], 13}, {&S38422[31], 4}, {&S38422[57], 17}, {NULL}};
const Segment SGI279587[] = {{&S38423[80], 4}, {NULL}};
const Segment SGI279588[] = {{&S38423[13], 3}, {&S38423[17], 10}, {&S38423[28], 4}, {&S38423[54], 17}, {NULL}};
const Segment SGI279589[] = {{&S38461[92], 4}, {NULL}};
const Segment SGI279590[] = {{&S38461[13], 9}, {&S38461[23], 22}, {&S38461[66], 17}, {NULL}};
const Segment SGI279591[] = {{&S38462[92], 4}, {NULL}};
const Segment SGI279592[] = {{&S38462[13], 9}, {&S38462[23], 22}, {&S38462[66], 17}, {NULL}};
const Segment SGI279593[] = {{&S38463[92], 4}, {NULL}};
const Segment SGI279594[] = {{&S38463[13], 9}, {&S38463[23], 22}, {&S38463[66], 17}, {NULL}};
const Segment SGI279595[] = {{&S38464[92], 4}, {NULL}};
const Segment SGI279596[] = {{&S38464[13], 9}, {&S38464[23], 22}, {&S38464[66], 17}, {NULL}};
const Segment SGI279597[] = {{&S38465[92], 4}, {NULL}};
const Segment SGI279598[] = {{&S38465[13], 9}, {&S38465[23], 22}, {&S38465[66], 17}, {NULL}};
const Segment SGI279599[] = {{&S38466[92], 4}, {NULL}};
const Segment SGI279600[] = {{&S38466[13], 9}, {&S38466[23], 22}, {&S38466[66], 17}, {NULL}};
const Segment SGI279601[] = {{&S38467[92], 4}, {NULL}};
const Segment SGI279602[] = {{&S38467[13], 9}, {&S38467[23], 22}, {&S38467[66], 17}, {NULL}};
const Segment SGI279603[] = {{&S38468[92], 4}, {NULL}};
const Segment SGI279604[] = {{&S38468[13], 9}, {&S38468[23], 22}, {&S38468[66], 17}, {NULL}};
const Segment SGI279605[] = {{&S38469[103], 4}, {NULL}};
const Segment SGI279606[] = {{&S38469[13], 9}, {&S38469[23], 2}, {&S38469[26], 22}, {&S38469[49], 6}, {&S38469[77], 17}, {NULL}};
const Segment SGI279607[] = {{&S38470[103], 4}, {NULL}};
const Segment SGI279608[] = {{&S38470[13], 9}, {&S38470[23], 2}, {&S38470[26], 22}, {&S38470[49], 6}, {&S38470[77], 17}, {NULL}};
const Segment SGI279609[] = {{&S38489[81], 4}, {NULL}};
const Segment SGI279610[] = {{&S38489[13], 14}, {&S38489[28], 6}, {&S38489[55], 17}, {NULL}};
const Segment SGI279611[] = {{&S38490[74], 4}, {NULL}};
const Segment SGI279612[] = {{&S38490[13], 14}, {&S38490[48], 17}, {NULL}};
const Segment SGI279613[] = {{&S38638[82], 4}, {NULL}};
const Segment SGI279614[] = {{&S38638[13], 14}, {&S38638[28], 6}, {&S38638[56], 17}, {NULL}};
const Segment SGI279615[] = {{&S38639[75], 4}, {NULL}};
const Segment SGI279616[] = {{&S38639[13], 14}, {&S38639[49], 17}, {NULL}};
const Segment SGI279617[] = {{&S38640[126], 4}, {NULL}};
const Segment SGI279618[] = {{&S38640[13], 7}, {&S38640[21], 2}, {&S38640[24], 15}, {&S38640[40], 6}, {&S38640[47], 29}, {&S38640[11], 1}, {&S38640[100], 17}, {NULL}};
const Segment SGI279619[] = {{&S38641[126], 4}, {NULL}};
const Segment SGI279620[] = {{&S38641[13], 7}, {&S38641[21], 2}, {&S38641[24], 15}, {&S38641[40], 6}, {&S38641[47], 29}, {&S38641[11], 1}, {&S38641[100], 17}, {NULL}};
const Segment SGI279621[] = {{&S38642[112], 4}, {NULL}};
const Segment SGI279622[] = {{&S38642[13], 7}, {&S38642[21], 2}, {&S38642[24], 15}, {&S38642[40], 6}, {&S38642[47], 15}, {&S38642[11], 1}, {&S38642[86], 17}, {NULL}};
const Segment SGI279623[] = {{&S38643[126], 4}, {NULL}};
const Segment SGI279624[] = {{&S38643[13], 7}, {&S38643[21], 2}, {&S38643[24], 15}, {&S38643[40], 6}, {&S38643[47], 29}, {&S38643[11], 1}, {&S38643[100], 17}, {NULL}};
const Segment SGI279625[] = {{&S38644[126], 4}, {NULL}};
const Segment SGI279626[] = {{&S38644[13], 7}, {&S38644[21], 2}, {&S38644[24], 15}, {&S38644[40], 6}, {&S38644[47], 29}, {&S38644[11], 1}, {&S38644[100], 17}, {NULL}};
const Segment SGI279627[] = {{&S38645[126], 4}, {NULL}};
const Segment SGI279628[] = {{&S38645[13], 7}, {&S38645[21], 2}, {&S38645[24], 15}, {&S38645[40], 6}, {&S38645[47], 29}, {&S38645[11], 1}, {&S38645[100], 17}, {NULL}};
const Segment SGI279629[] = {{&S38646[126], 4}, {NULL}};
const Segment SGI279630[] = {{&S38646[13], 7}, {&S38646[21], 2}, {&S38646[24], 15}, {&S38646[40], 6}, {&S38646[47], 29}, {&S38646[11], 1}, {&S38646[100], 17}, {NULL}};
const Segment SGI279631[] = {{&S38647[93], 4}, {NULL}};
const Segment SGI279632[] = {{&S38647[13], 7}, {&S38647[21], 2}, {&S38647[24], 15}, {&S38647[40], 6}, {&S38647[67], 17}, {NULL}};
const Segment SGI279633[] = {{&S38648[81], 4}, {NULL}};
const Segment SGI279634[] = {{&S38648[13], 14}, {&S38648[28], 6}, {&S38648[55], 17}, {NULL}};
const Segment SGI279635[] = {{&S38649[119], 4}, {NULL}};
const Segment SGI279636[] = {{&S38649[13], 7}, {&S38649[21], 2}, {&S38649[24], 15}, {&S38649[40], 6}, {&S38649[47], 22}, {&S38649[11], 1}, {&S38649[93], 17}, {NULL}};
const Segment SGI279637[] = {{&S38650[123], 4}, {NULL}};
const Segment SGI279638[] = {{&S38650[13], 7}, {&S38650[21], 2}, {&S38650[24], 15}, {&S38650[40], 6}, {&S38650[47], 26}, {&S38650[11], 1}, {&S38650[97], 17}, {NULL}};
const Segment SGI279639[] = {{&S38651[118], 4}, {NULL}};
const Segment SGI279640[] = {{&S38651[13], 7}, {&S38651[21], 2}, {&S38651[24], 15}, {&S38651[40], 6}, {&S38651[47], 21}, {&S38651[11], 1}, {&S38651[92], 17}, {NULL}};
const Segment SGI279641[] = {{&S43702[8], 3}, {NULL}};
const Segment SGI279642[] = {{&S43702[13], 9}, {&S43702[23], 2}, {&S43702[26], 22}, {&S43702[49], 6}, {&S43702[76], 17}, {NULL}};
const Segment SGI279643[] = {{&S43703[8], 3}, {NULL}};
const Segment SGI279644[] = {{&S43703[13], 9}, {&S43703[23], 2}, {&S43703[26], 22}, {&S43703[49], 6}, {&S43703[76], 17}, {NULL}};
const Segment SGI279645[] = {{&S43704[8], 3}, {NULL}};
const Segment SGI279646[] = {{&S43704[13], 9}, {&S43704[23], 2}, {&S43704[26], 22}, {&S43704[49], 6}, {&S43704[76], 17}, {NULL}};
const Segment SGI279647[] = {{&S43705[8], 3}, {NULL}};
const Segment SGI279648[] = {{&S43705[13], 9}, {&S43705[23], 2}, {&S43705[26], 22}, {&S43705[49], 6}, {&S43705[76], 17}, {NULL}};
const Segment SGI279649[] = {{&S43706[8], 3}, {NULL}};
const Segment SGI279650[] = {{&S43706[13], 9}, {&S43706[23], 2}, {&S43706[26], 22}, {&S43706[49], 6}, {&S43706[76], 17}, {NULL}};
const Segment SGI279651[] = {{&S43707[8], 3}, {NULL}};
const Segment SGI279652[] = {{&S43707[13], 9}, {&S43707[23], 2}, {&S43707[26], 22}, {&S43707[49], 6}, {&S43707[76], 17}, {NULL}};
const Segment SGI279653[] = {{&S43708[8], 3}, {NULL}};
const Segment SGI279654[] = {{&S43708[13], 9}, {&S43708[23], 2}, {&S43708[26], 22}, {&S43708[49], 6}, {&S43708[76], 17}, {NULL}};
const Segment SGI279655[] = {{&S43709[8], 3}, {NULL}};
const Segment SGI279656[] = {{&S43709[13], 9}, {&S43709[23], 2}, {&S43709[26], 22}, {&S43709[49], 6}, {&S43709[76], 17}, {NULL}};
const Segment SGI279657[] = {{&S43710[8], 3}, {NULL}};
const Segment SGI279658[] = {{&S43710[13], 9}, {&S43710[23], 2}, {&S43710[26], 22}, {&S43710[49], 6}, {&S43710[76], 17}, {NULL}};
const Segment SGI279659[] = {{&S35041[74], 4}, {NULL}};
const Segment SGI279660[] = {{&S35041[13], 14}, {&S35041[48], 17}, {NULL}};
const Segment SGI279661[] = {{&S35042[82], 4}, {NULL}};
const Segment SGI279662[] = {{&S35042[13], 14}, {&S35042[28], 6}, {&S35042[56], 17}, {NULL}};
const Segment SGI279663[] = {{&S35043[75], 4}, {NULL}};
const Segment SGI279664[] = {{&S35043[13], 14}, {&S35043[49], 17}, {&S35043[102], 8}, {&S35043[113], 24}, {NULL}};
const Segment SGI279665[] = {{&S35044[13], 14}, {&S35044[28], 6}, {&S35044[56], 17}, {NULL}};
const Segment SGI279666[] = {{&S44035[102], 3}, {NULL}};
const Segment SGI279667[] = {{&S44035[13], 9}, {&S44035[23], 2}, {&S44035[26], 22}, {&S44035[49], 6}, {&S44035[76], 17}, {NULL}};
const Segment SGI279668[] = {{&S44036[102], 3}, {NULL}};
const Segment SGI279669[] = {{&S44036[13], 9}, {&S44036[23], 2}, {&S44036[26], 22}, {&S44036[49], 6}, {&S44036[76], 17}, {NULL}};
const Segment SGI279670[] = {{&S44037[102], 3}, {NULL}};
const Segment SGI279671[] = {{&S44037[13], 9}, {&S44037[23], 2}, {&S44037[26], 22}, {&S44037[49], 6}, {&S44037[76], 17}, {NULL}};
const Segment SGI279672[] = {{&S44038[102], 3}, {NULL}};
const Segment SGI279673[] = {{&S44038[13], 9}, {&S44038[23], 2}, {&S44038[26], 22}, {&S44038[49], 6}, {&S44038[76], 17}, {NULL}};
const Segment SGI279674[] = {{&S44039[102], 3}, {NULL}};
const Segment SGI279675[] = {{&S44039[13], 9}, {&S44039[23], 2}, {&S44039[26], 22}, {&S44039[49], 6}, {&S44039[76], 17}, {NULL}};
const Segment SGI279676[] = {{&S44040[102], 3}, {NULL}};
const Segment SGI279677[] = {{&S44040[13], 9}, {&S44040[23], 2}, {&S44040[26], 22}, {&S44040[49], 6}, {&S44040[76], 17}, {NULL}};
const Segment SGI279678[] = {{&S44041[102], 3}, {NULL}};
const Segment SGI279679[] = {{&S44041[13], 9}, {&S44041[23], 2}, {&S44041[26], 22}, {&S44041[49], 6}, {&S44041[76], 17}, {NULL}};
const Segment SGI279680[] = {{&S44042[102], 3}, {NULL}};
const Segment SGI279681[] = {{&S44042[13], 9}, {&S44042[23], 2}, {&S44042[26], 22}, {&S44042[49], 6}, {&S44042[76], 17}, {NULL}};
const Segment SGI279682[] = {{&S44043[102], 3}, {NULL}};
const Segment SGI279683[] = {{&S44043[13], 9}, {&S44043[23], 2}, {&S44043[26], 22}, {&S44043[49], 6}, {&S44043[76], 17}, {NULL}};
const Segment SGI279684[] = {{&S44046[101], 3}, {NULL}};
const Segment SGI279685[] = {{&S44046[13], 3}, {&S44046[17], 2}, {&S44046[20], 11}, {&S44046[32], 6}, {&S44046[60], 17}, {NULL}};
const Segment SGI279686[] = {{&S44046[79], 12}, {NULL}};
const Segment SGI279687[] = {{&S44047[103], 3}, {NULL}};
const Segment SGI279688[] = {{&S44047[13], 3}, {&S44047[17], 2}, {&S44047[20], 13}, {&S44047[34], 6}, {&S44047[62], 17}, {NULL}};
const Segment SGI279689[] = {{&S44047[81], 12}, {NULL}};
const Segment SGI279690[] = {{&S44048[100], 3}, {NULL}};
const Segment SGI279691[] = {{&S44048[13], 3}, {&S44048[17], 2}, {&S44048[20], 13}, {&S44048[34], 6}, {&S44048[61], 17}, {NULL}};
const Segment SGI279692[] = {{&S44048[80], 12}, {NULL}};
const Segment SGI279693[] = {{&S44049[103], 3}, {NULL}};
const Segment SGI279694[] = {{&S44049[13], 3}, {&S44049[17], 2}, {&S44049[20], 13}, {&S44049[34], 6}, {&S44049[62], 17}, {NULL}};
const Segment SGI279695[] = {{&S44049[81], 12}, {NULL}};
const Segment SGI279696[] = {{&S35277[41], 4}, {NULL}};
const Segment SGI279697[] = {{&S35277[13], 14}, {&S35277[28], 25}, {&S35277[75], 17}, {NULL}};
const Segment SGI279698[] = {{&S35278[41], 4}, {NULL}};
const Segment SGI279699[] = {{&S35278[13], 14}, {&S35278[28], 25}, {&S35278[75], 17}, {NULL}};
const Segment SGI279700[] = {{&S35279[41], 4}, {NULL}};
const Segment SGI279701[] = {{&S35279[13], 14}, {&S35279[28], 25}, {&S35279[75], 17}, {NULL}};
const Segment SGI279702[] = {{&S35280[41], 4}, {NULL}};
const Segment SGI279703[] = {{&S35280[13], 14}, {&S35280[28], 25}, {&S35280[75], 17}, {NULL}};
const Segment SGI279704[] = {{&S35281[41], 4}, {NULL}};
const Segment SGI279705[] = {{&S35281[13], 14}, {&S35281[28], 25}, {&S35281[75], 17}, {NULL}};
const Segment SGI279706[] = {{&S35282[41], 4}, {NULL}};
const Segment SGI279707[] = {{&S35282[13], 14}, {&S35282[28], 25}, {&S35282[75], 17}, {NULL}};
const Segment SGI279708[] = {{&S35285[41], 4}, {NULL}};
const Segment SGI279709[] = {{&S35285[13], 14}, {&S35285[28], 25}, {&S35285[75], 17}, {NULL}};
const Segment SGI279710[] = {{&S35287[41], 4}, {NULL}};
const Segment SGI279711[] = {{&S35287[13], 14}, {&S35287[28], 25}, {&S35287[75], 17}, {NULL}};
const Segment SGI279712[] = {{&S35290[48], 4}, {NULL}};
const Segment SGI279713[] = {{&S35290[13], 14}, {&S35290[28], 6}, {&S35290[35], 25}, {&S35290[82], 17}, {NULL}};
const Segment SGI279714[] = {{&S38891[81], 4}, {NULL}};
const Segment SGI279715[] = {{&S38891[13], 14}, {&S38891[28], 6}, {&S38891[55], 17}, {NULL}};
const Segment SGI279716[] = {{&S38892[74], 4}, {NULL}};
const Segment SGI279717[] = {{&S38892[13], 14}, {&S38892[48], 17}, {NULL}};
const Segment SGI279718[] = {{&S38893[93], 4}, {NULL}};
const Segment SGI279719[] = {{&S38893[13], 9}, {&S38893[23], 22}, {&S38893[67], 17}, {NULL}};
const Segment SGI279720[] = {{&S38894[93], 4}, {NULL}};
const Segment SGI279721[] = {{&S38894[13], 9}, {&S38894[23], 22}, {&S38894[67], 17}, {NULL}};
const Segment SGI279722[] = {{&S38895[93], 4}, {NULL}};
const Segment SGI279723[] = {{&S38895[13], 9}, {&S38895[23], 22}, {&S38895[67], 17}, {NULL}};
const Segment SGI279724[] = {{&S38896[93], 4}, {NULL}};
const Segment SGI279725[] = {{&S38896[13], 9}, {&S38896[23], 22}, {&S38896[67], 17}, {NULL}};
const Segment SGI279726[] = {{&S38897[93], 4}, {NULL}};
const Segment SGI279727[] = {{&S38897[13], 9}, {&S38897[23], 22}, {&S38897[67], 17}, {NULL}};
const Segment SGI279728[] = {{&S38898[93], 4}, {NULL}};
const Segment SGI279729[] = {{&S38898[13], 9}, {&S38898[23], 22}, {&S38898[67], 17}, {NULL}};
const Segment SGI279730[] = {{&S38899[93], 4}, {NULL}};
const Segment SGI279731[] = {{&S38899[13], 9}, {&S38899[23], 22}, {&S38899[67], 17}, {NULL}};
const Segment SGI279732[] = {{&S38900[93], 4}, {NULL}};
const Segment SGI279733[] = {{&S38900[13], 9}, {&S38900[23], 22}, {&S38900[67], 17}, {NULL}};
const Segment SGI279734[] = {{&S38901[103], 4}, {NULL}};
const Segment SGI279735[] = {{&S38901[13], 9}, {&S38901[23], 2}, {&S38901[26], 22}, {&S38901[49], 6}, {&S38901[77], 17}, {NULL}};
const Segment SGI279736[] = {{&S38902[103], 4}, {NULL}};
const Segment SGI279737[] = {{&S38902[13], 9}, {&S38902[23], 2}, {&S38902[26], 22}, {&S38902[49], 6}, {&S38902[77], 17}, {NULL}};
const Segment SGI279738[] = {{&S39208[94], 4}, {NULL}};
const Segment SGI279739[] = {{&S39208[13], 14}, {&S39208[28], 6}, {&S39208[56], 17}, {NULL}};
const Segment SGI279740[] = {{&S39208[75], 11}, {NULL}};
const Segment SGI279741[] = {{&S39209[87], 4}, {NULL}};
const Segment SGI279742[] = {{&S39209[13], 14}, {&S39209[49], 17}, {NULL}};
const Segment SGI279743[] = {{&S39209[68], 11}, {NULL}};
const Segment SGI279744[] = {{&S39212[94], 4}, {NULL}};
const Segment SGI279745[] = {{&S39212[13], 14}, {&S39212[28], 6}, {&S39212[56], 17}, {&S39212[119], 8}, {NULL}};
const Segment SGI279746[] = {{&S39212[75], 11}, {NULL}};
const Segment SGI279747[] = {{&S39214[87], 4}, {NULL}};
const Segment SGI279748[] = {{&S39214[13], 14}, {&S39214[49], 17}, {&S39214[112], 8}, {NULL}};
const Segment SGI279749[] = {{&S39214[68], 11}, {NULL}};
const Segment SGI279750[] = {{&S39398[101], 4}, {NULL}};
const Segment SGI279751[] = {{&S39398[13], 14}, {&S39398[28], 6}, {&S39398[56], 17}, {NULL}};
const Segment SGI279752[] = {{&S39398[75], 11}, {NULL}};
const Segment SGI279753[] = {{&S39399[94], 4}, {NULL}};
const Segment SGI279754[] = {{&S39399[13], 14}, {&S39399[49], 17}, {NULL}};
const Segment SGI279755[] = {{&S39399[68], 11}, {NULL}};
const Segment SGI279756[] = {{&S39402[48], 4}, {NULL}};
const Segment SGI279757[] = {{&S39402[13], 14}, {&S39402[28], 6}, {&S39402[35], 24}, {&S39402[80], 17}, {NULL}};
const Segment SGI279758[] = {{&S39402[36], 16}, {NULL}};
const Segment SGI279759[] = {{&S39405[48], 4}, {NULL}};
const Segment SGI279760[] = {{&S39405[13], 14}, {&S39405[28], 6}, {&S39405[35], 24}, {&S39405[80], 17}, {NULL}};
const Segment SGI279761[] = {{&S39405[36], 16}, {NULL}};
const Segment SGI279762[] = {{&S44484[94], 3}, {NULL}};
const Segment SGI279763[] = {{&S39406[41], 4}, {NULL}};
const Segment SGI279764[] = {{&S44484[13], 7}, {&S44484[21], 2}, {&S44484[24], 15}, {&S44484[40], 6}, {&S44484[68], 17}, {NULL}};
const Segment SGI279765[] = {{&S39406[13], 14}, {&S39406[28], 24}, {&S39406[73], 17}, {NULL}};
const Segment SGI279766[] = {{&S39406[29], 16}, {NULL}};
const Segment SGI279767[] = {{&S44485[94], 3}, {NULL}};
const Segment SGI279768[] = {{&S44485[13], 7}, {&S44485[21], 2}, {&S44485[24], 15}, {&S44485[40], 6}, {&S44485[68], 17}, {NULL}};
const Segment SGI279769[] = {{&S39408[48], 4}, {NULL}};
const Segment SGI279770[] = {{&S39408[13], 14}, {&S39408[28], 6}, {&S39408[35], 24}, {&S39408[80], 17}, {NULL}};
const Segment SGI279771[] = {{&S39408[36], 16}, {NULL}};
const Segment SGI279772[] = {{&S44486[94], 3}, {NULL}};
const Segment SGI279773[] = {{&S44486[13], 7}, {&S44486[21], 2}, {&S44486[24], 15}, {&S44486[40], 6}, {&S44486[68], 17}, {NULL}};
const Segment SGI279774[] = {{&S39409[41], 4}, {NULL}};
const Segment SGI279775[] = {{&S39409[13], 14}, {&S39409[28], 24}, {&S39409[73], 17}, {NULL}};
const Segment SGI279776[] = {{&S44487[75], 3}, {NULL}};
const Segment SGI279777[] = {{&S39409[29], 16}, {NULL}};
const Segment SGI279778[] = {{&S44487[13], 14}, {&S44487[49], 17}, {NULL}};
const Segment SGI279779[] = {{&S44488[82], 3}, {NULL}};
const Segment SGI279780[] = {{&S44488[13], 14}, {&S44488[28], 6}, {&S44488[56], 17}, {NULL}};
const Segment SGI279781[] = {{&S44489[91], 3}, {NULL}};
const Segment SGI279782[] = {{&S44489[13], 7}, {&S44489[21], 2}, {&S44489[24], 15}, {&S44489[40], 3}, {&S44489[65], 17}, {NULL}};
const Segment SGI279783[] = {{&S44490[93], 3}, {NULL}};
const Segment SGI279784[] = {{&S44490[13], 7}, {&S44490[21], 2}, {&S44490[24], 15}, {&S44490[40], 6}, {&S44490[67], 17}, {NULL}};
const Segment SGI279785[] = {{&S44491[91], 3}, {NULL}};
const Segment SGI279786[] = {{&S44491[13], 7}, {&S44491[21], 2}, {&S44491[24], 15}, {&S44491[40], 3}, {&S44491[65], 17}, {NULL}};
const Segment SGI279787[] = {{&S44492[81], 3}, {NULL}};
const Segment SGI279788[] = {{&S44492[13], 14}, {&S44492[28], 6}, {&S44492[55], 17}, {NULL}};
const Segment SGI279789[] = {{&S39415[41], 4}, {NULL}};
const Segment SGI279790[] = {{&S39415[13], 14}, {&S39415[28], 24}, {&S39415[73], 17}, {NULL}};
const Segment SGI279791[] = {{&S39415[29], 16}, {NULL}};
const Segment SGI279792[] = {{&S39496[41], 4}, {NULL}};
const Segment SGI279793[] = {{&S39496[13], 14}, {&S39496[28], 25}, {&S39496[74], 17}, {NULL}};
const Segment SGI279794[] = {{&S39496[29], 16}, {NULL}};
const Segment SGI279795[] = {{&S39497[41], 4}, {NULL}};
const Segment SGI279796[] = {{&S39497[13], 14}, {&S39497[28], 25}, {&S39497[74], 17}, {NULL}};
const Segment SGI279797[] = {{&S39497[29], 16}, {NULL}};
const Segment SGI279798[] = {{&S39498[41], 4}, {NULL}};
const Segment SGI279799[] = {{&S39498[13], 14}, {&S39498[28], 25}, {&S39498[74], 17}, {NULL}};
const Segment SGI279800[] = {{&S39498[29], 16}, {NULL}};
const Segment SGI279801[] = {{&S39499[41], 4}, {NULL}};
const Segment SGI279802[] = {{&S39499[13], 14}, {&S39499[28], 25}, {&S39499[74], 17}, {NULL}};
const Segment SGI279803[] = {{&S39499[29], 16}, {NULL}};
const Segment SGI279804[] = {{&S39504[41], 4}, {NULL}};
const Segment SGI279805[] = {{&S39504[13], 14}, {&S39504[28], 25}, {&S39504[74], 17}, {NULL}};
const Segment SGI279806[] = {{&S39504[29], 16}, {NULL}};
const Segment SGI279807[] = {{&S39507[41], 4}, {NULL}};
const Segment SGI279808[] = {{&S39507[13], 14}, {&S39507[28], 23}, {&S39507[73], 17}, {NULL}};
const Segment SGI279809[] = {{&S39507[29], 16}, {NULL}};
const Segment SGI279810[] = {{&S39508[41], 4}, {NULL}};
const Segment SGI279811[] = {{&S39508[13], 14}, {&S39508[28], 24}, {&S39508[73], 17}, {NULL}};
const Segment SGI279812[] = {{&S39508[29], 16}, {NULL}};
const Segment SGI279813[] = {{&S39511[48], 4}, {NULL}};
const Segment SGI279814[] = {{&S39511[13], 14}, {&S39511[28], 6}, {&S39511[35], 25}, {&S39511[81], 17}, {NULL}};
const Segment SGI279815[] = {{&S39511[36], 16}, {NULL}};
const Segment SGI279816[] = {{&S39512[41], 4}, {NULL}};
const Segment SGI279817[] = {{&S39512[13], 14}, {&S39512[28], 24}, {&S39512[73], 17}, {NULL}};
const Segment SGI279818[] = {{&S39512[29], 16}, {NULL}};
const Segment SGI279819[] = {{&S39513[78], 4}, {NULL}};
const Segment SGI279820[] = {{&S39513[13], 3}, {&S39513[17], 13}, {&S39513[52], 17}, {NULL}};
const Segment SGI279821[] = {{&S39513[117], 8}, {NULL}};
const Segment SGI279822[] = {{&S39514[86], 4}, {NULL}};
const Segment SGI279823[] = {{&S39514[13], 3}, {&S39514[17], 2}, {&S39514[20], 11}, {&S39514[32], 6}, {&S39514[60], 17}, {NULL}};
const Segment SGI279824[] = {{&S39514[124], 8}, {NULL}};
const Segment SGI279825[] = {{&S39515[76], 4}, {NULL}};
const Segment SGI279826[] = {{&S39515[13], 3}, {&S39515[17], 11}, {&S39515[50], 17}, {NULL}};
const Segment SGI279827[] = {{&S39515[115], 8}, {NULL}};
const Segment SGI279828[] = {{&S39516[107], 4}, {NULL}};
const Segment SGI279829[] = {{&S39516[13], 5}, {&S39516[19], 13}, {&S39516[33], 26}, {&S39516[81], 17}, {NULL}};
const Segment SGI279830[] = {{&S39516[20], 8}, {NULL}};
const Segment SGI279831[] = {{&S39517[101], 4}, {NULL}};
const Segment SGI279832[] = {{&S39517[13], 5}, {&S39517[19], 13}, {&S39517[33], 20}, {&S39517[75], 17}, {NULL}};
const Segment SGI279833[] = {{&S39517[20], 8}, {NULL}};
const Segment SGI279834[] = {{&S39518[107], 4}, {NULL}};
const Segment SGI279835[] = {{&S39518[13], 5}, {&S39518[19], 13}, {&S39518[33], 26}, {&S39518[81], 17}, {NULL}};
const Segment SGI279836[] = {{&S39518[20], 8}, {NULL}};
const Segment SGI279837[] = {{&S39519[110], 4}, {NULL}};
const Segment SGI279838[] = {{&S39519[13], 5}, {&S39519[19], 13}, {&S39519[33], 29}, {&S39519[84], 17}, {NULL}};
const Segment SGI279839[] = {{&S39519[20], 8}, {NULL}};
const Segment SGI279840[] = {{&S39520[101], 4}, {NULL}};
const Segment SGI279841[] = {{&S39520[13], 5}, {&S39520[19], 13}, {&S39520[33], 20}, {&S39520[75], 17}, {NULL}};
const Segment SGI279842[] = {{&S39520[20], 8}, {NULL}};
const Segment SGI279843[] = {{&S39521[110], 4}, {NULL}};
const Segment SGI279844[] = {{&S39521[13], 5}, {&S39521[19], 13}, {&S39521[33], 29}, {&S39521[84], 17}, {NULL}};
const Segment SGI279845[] = {{&S39521[20], 8}, {NULL}};
const Segment SGI279846[] = {{&S39522[111], 4}, {NULL}};
const Segment SGI279847[] = {{&S39522[13], 5}, {&S39522[19], 13}, {&S39522[33], 30}, {&S39522[85], 17}, {NULL}};
const Segment SGI279848[] = {{&S39522[20], 8}, {NULL}};
const Segment SGI279849[] = {{&S39523[103], 4}, {NULL}};
const Segment SGI279850[] = {{&S39523[13], 5}, {&S39523[19], 13}, {&S39523[33], 22}, {&S39523[77], 17}, {NULL}};
const Segment SGI279851[] = {{&S39523[20], 8}, {NULL}};
const Segment SGI279852[] = {{&S39642[87], 4}, {NULL}};
const Segment SGI279853[] = {{&S39642[13], 14}, {&S39642[49], 17}, {NULL}};
const Segment SGI279854[] = {{&S39642[68], 11}, {NULL}};
const Segment SGI279855[] = {{&S39643[94], 4}, {NULL}};
const Segment SGI279856[] = {{&S39643[13], 14}, {&S39643[28], 6}, {&S39643[56], 17}, {NULL}};
const Segment SGI279857[] = {{&S39643[75], 11}, {NULL}};
const Segment SGI279858[] = {{&S39646[87], 4}, {NULL}};
const Segment SGI279859[] = {{&S39646[13], 14}, {&S39646[49], 17}, {&S39646[112], 8}, {NULL}};
const Segment SGI279860[] = {{&S39646[68], 11}, {NULL}};
const Segment SGI279861[] = {{&S44743[93], 3}, {NULL}};
const Segment SGI279862[] = {{&S44743[13], 7}, {&S44743[21], 2}, {&S44743[24], 15}, {&S44743[40], 6}, {&S44743[67], 17}, {NULL}};
const Segment SGI279863[] = {{&S36131[91], 4}, {NULL}};
const Segment SGI279864[] = {{&S36131[13], 3}, {&S36131[17], 11}, {&S36131[50], 17}, {NULL}};
const Segment SGI279865[] = {{&S36131[69], 12}, {NULL}};
const Segment SGI279866[] = {{&S36132[93], 4}, {NULL}};
const Segment SGI279867[] = {{&S36132[13], 3}, {&S36132[17], 13}, {&S36132[52], 17}, {NULL}};
const Segment SGI279868[] = {{&S36132[71], 12}, {NULL}};
const Segment SGI279869[] = {{&S36133[91], 4}, {NULL}};
const Segment SGI279870[] = {{&S36133[13], 3}, {&S36133[17], 11}, {&S36133[50], 17}, {&S36133[138], 8}, {NULL}};
const Segment SGI279871[] = {{&S36133[69], 12}, {NULL}};
const Segment SGI279872[] = {{&S39677[74], 4}, {NULL}};
const Segment SGI279873[] = {{&S39677[13], 14}, {&S39677[48], 17}, {NULL}};
const Segment SGI279874[] = {{&S36134[93], 4}, {NULL}};
const Segment SGI279875[] = {{&S36134[13], 3}, {&S36134[17], 13}, {&S36134[52], 17}, {&S36134[140], 8}, {&S36134[151], 24}, {NULL}};
const Segment SGI279876[] = {{&S36134[71], 12}, {NULL}};
const Segment SGI279877[] = {{&S39678[81], 4}, {NULL}};
const Segment SGI279878[] = {{&S39678[13], 14}, {&S39678[28], 6}, {&S39678[55], 17}, {NULL}};
const Segment SGI279879[] = {{&S36135[93], 4}, {NULL}};
const Segment SGI279880[] = {{&S36135[13], 3}, {&S36135[17], 13}, {&S36135[52], 17}, {&S36135[140], 8}, {NULL}};
const Segment SGI279881[] = {{&S36135[71], 12}, {NULL}};
const Segment SGI279882[] = {{&S36136[93], 4}, {NULL}};
const Segment SGI279883[] = {{&S36136[13], 3}, {&S36136[17], 13}, {&S36136[52], 17}, {NULL}};
const Segment SGI279884[] = {{&S36136[71], 12}, {NULL}};
const Segment SGI279885[] = {{&S36137[91], 4}, {NULL}};
const Segment SGI279886[] = {{&S36137[13], 3}, {&S36137[17], 11}, {&S36137[50], 17}, {&S36137[138], 8}, {&S36137[149], 26}, {NULL}};
const Segment SGI279887[] = {{&S36137[69], 12}, {NULL}};
const Segment SGI279888[] = {{&S39686[91], 4}, {NULL}};
const Segment SGI279889[] = {{&S39686[13], 3}, {&S39686[17], 11}, {&S39686[50], 17}, {NULL}};
const Segment SGI279890[] = {{&S39686[69], 12}, {NULL}};
const Segment SGI279891[] = {{&S39687[93], 4}, {NULL}};
const Segment SGI279892[] = {{&S39687[13], 3}, {&S39687[17], 13}, {&S39687[52], 17}, {NULL}};
const Segment SGI279893[] = {{&S39687[71], 12}, {NULL}};
const Segment SGI279894[] = {{&S39688[91], 4}, {NULL}};
const Segment SGI279895[] = {{&S39688[13], 3}, {&S39688[17], 11}, {&S39688[50], 17}, {&S39688[140], 8}, {&S39688[151], 24}, {NULL}};
const Segment SGI279896[] = {{&S39688[69], 12}, {NULL}};
const Segment SGI279897[] = {{&S39689[91], 4}, {NULL}};
const Segment SGI279898[] = {{&S39689[13], 3}, {&S39689[17], 11}, {&S39689[50], 17}, {&S39689[140], 8}, {NULL}};
const Segment SGI279899[] = {{&S39689[69], 12}, {NULL}};
const Segment SGI279900[] = {{&S44861[93], 3}, {NULL}};
const Segment SGI279901[] = {{&S44861[13], 7}, {&S44861[21], 2}, {&S44861[24], 15}, {&S44861[40], 6}, {&S44861[67], 17}, {NULL}};
const Segment SGI279902[] = {{&S44862[92], 3}, {NULL}};
const Segment SGI279903[] = {{&S44862[13], 7}, {&S44862[21], 2}, {&S44862[24], 15}, {&S44862[40], 6}, {&S44862[67], 16}, {NULL}};
const Segment SGI279904[] = {{&S44863[63], 3}, {NULL}};
const Segment SGI279905[] = {{&S44863[13], 7}, {&S44863[21], 2}, {&S44863[24], 15}, {&S44863[40], 6}, {NULL}};
const Segment SGI279906[] = {{&S44864[75], 3}, {NULL}};
const Segment SGI279907[] = {{&S44864[13], 14}, {&S44864[49], 17}, {NULL}};
const Segment SGI279908[] = {{&S44865[94], 3}, {NULL}};
const Segment SGI279909[] = {{&S44865[13], 7}, {&S44865[21], 2}, {&S44865[24], 15}, {&S44865[40], 6}, {&S44865[47], 18}, {&S44865[66], 19}, {&S44865[86], 30}, {NULL}};
const Segment SGI279910[] = {{&S44879[102], 3}, {NULL}};
const Segment SGI279911[] = {{&S44879[13], 9}, {&S44879[23], 2}, {&S44879[26], 22}, {&S44879[49], 6}, {&S44879[76], 17}, {NULL}};
const Segment SGI279912[] = {{&S44880[102], 3}, {NULL}};
const Segment SGI279913[] = {{&S44880[13], 9}, {&S44880[23], 2}, {&S44880[26], 22}, {&S44880[49], 6}, {&S44880[76], 17}, {NULL}};
const Segment SGI279914[] = {{&S44881[102], 3}, {NULL}};
const Segment SGI279915[] = {{&S44881[13], 9}, {&S44881[23], 2}, {&S44881[26], 22}, {&S44881[49], 6}, {&S44881[76], 17}, {NULL}};
const Segment SGI279916[] = {{&S44882[102], 3}, {NULL}};
const Segment SGI279917[] = {{&S44882[13], 9}, {&S44882[23], 2}, {&S44882[26], 22}, {&S44882[49], 6}, {&S44882[76], 17}, {NULL}};
const Segment SGI279918[] = {{&S44883[102], 3}, {NULL}};
const Segment SGI279919[] = {{&S44883[13], 9}, {&S44883[23], 2}, {&S44883[26], 22}, {&S44883[49], 6}, {&S44883[76], 17}, {NULL}};
const Segment SGI279920[] = {{&S44884[102], 3}, {NULL}};
const Segment SGI279921[] = {{&S44884[13], 9}, {&S44884[23], 2}, {&S44884[26], 22}, {&S44884[49], 6}, {&S44884[76], 17}, {NULL}};
const Segment SGI279922[] = {{&S44885[102], 3}, {NULL}};
const Segment SGI279923[] = {{&S44885[13], 9}, {&S44885[23], 2}, {&S44885[26], 22}, {&S44885[49], 6}, {&S44885[76], 17}, {NULL}};
const Segment SGI279924[] = {{&S44886[103], 3}, {NULL}};
const Segment SGI279925[] = {{&S44886[13], 9}, {&S44886[23], 2}, {&S44886[26], 22}, {&S44886[49], 6}, {&S44886[76], 18}, {NULL}};
const Segment SGI279926[] = {{&S44887[102], 3}, {NULL}};
const Segment SGI279927[] = {{&S44887[13], 9}, {&S44887[23], 2}, {&S44887[26], 22}, {&S44887[49], 6}, {&S44887[76], 17}, {NULL}};
const Segment SGI279928[] = {{&S44888[102], 3}, {NULL}};
const Segment SGI279929[] = {{&S44888[13], 9}, {&S44888[23], 2}, {&S44888[26], 22}, {&S44888[49], 6}, {&S44888[76], 17}, {NULL}};
const Segment SGI279930[] = {{&S44889[102], 3}, {NULL}};
const Segment SGI279931[] = {{&S44889[13], 9}, {&S44889[23], 2}, {&S44889[26], 22}, {&S44889[49], 6}, {&S44889[76], 17}, {NULL}};
const Segment SGI279932[] = {{&S44985[93], 3}, {NULL}};
const Segment SGI279933[] = {{&S44985[13], 7}, {&S44985[21], 2}, {&S44985[24], 15}, {&S44985[40], 6}, {&S44985[67], 17}, {NULL}};
const Segment SGI279934[] = {{&S44986[93], 3}, {NULL}};
const Segment SGI279935[] = {{&S44986[13], 7}, {&S44986[21], 2}, {&S44986[24], 15}, {&S44986[40], 6}, {&S44986[67], 17}, {NULL}};
const Segment SGI279936[] = {{&S36562[93], 4}, {NULL}};
const Segment SGI279937[] = {{&S36562[13], 14}, {&S36562[48], 17}, {NULL}};
const Segment SGI279938[] = {{&S36562[67], 11}, {NULL}};
const Segment SGI279939[] = {{&S36563[100], 4}, {NULL}};
const Segment SGI279940[] = {{&S36563[13], 14}, {&S36563[28], 6}, {&S36563[55], 17}, {NULL}};
const Segment SGI279941[] = {{&S36563[74], 11}, {NULL}};
const Segment SGI279942[] = {{&S36564[100], 4}, {NULL}};
const Segment SGI279943[] = {{&S36564[13], 14}, {&S36564[28], 6}, {&S36564[55], 17}, {&S36564[126], 8}, {NULL}};
const Segment SGI279944[] = {{&S36564[74], 11}, {NULL}};
const Segment SGI279945[] = {{&S40164[75], 4}, {NULL}};
const Segment SGI279946[] = {{&S40164[13], 14}, {&S40164[49], 17}, {NULL}};
const Segment SGI279947[] = {{&S40164[103], 21}, {NULL}};
const Segment SGI279948[] = {{&S40165[74], 4}, {NULL}};
const Segment SGI279949[] = {{&S40165[13], 14}, {&S40165[48], 17}, {NULL}};
const Segment SGI279950[] = {{&S40165[100], 21}, {NULL}};
const Segment SGI279951[] = {{&S40214[92], 4}, {NULL}};
const Segment SGI279952[] = {{&S40214[13], 9}, {&S40214[23], 22}, {&S40214[66], 17}, {NULL}};
const Segment SGI279953[] = {{&S40215[92], 4}, {NULL}};
const Segment SGI279954[] = {{&S40215[13], 9}, {&S40215[23], 22}, {&S40215[66], 17}, {NULL}};
const Segment SGI279955[] = {{&S40216[92], 4}, {NULL}};
const Segment SGI279956[] = {{&S40216[13], 9}, {&S40216[23], 22}, {&S40216[66], 17}, {NULL}};
const Segment SGI279957[] = {{&S40217[92], 4}, {NULL}};
const Segment SGI279958[] = {{&S40217[13], 9}, {&S40217[23], 22}, {&S40217[66], 17}, {NULL}};
const Segment SGI279959[] = {{&S40218[92], 4}, {NULL}};
const Segment SGI279960[] = {{&S40218[13], 9}, {&S40218[23], 22}, {&S40218[66], 17}, {NULL}};
const Segment SGI279961[] = {{&S40219[92], 4}, {NULL}};
const Segment SGI279962[] = {{&S40219[13], 9}, {&S40219[23], 22}, {&S40219[66], 17}, {NULL}};
const Segment SGI279963[] = {{&S40220[92], 4}, {NULL}};
const Segment SGI279964[] = {{&S40220[13], 9}, {&S40220[23], 22}, {&S40220[66], 17}, {NULL}};
const Segment SGI279965[] = {{&S40221[92], 4}, {NULL}};
const Segment SGI279966[] = {{&S40221[13], 9}, {&S40221[23], 22}, {&S40221[66], 17}, {NULL}};
const Segment SGI279967[] = {{&S40222[92], 4}, {NULL}};
const Segment SGI279968[] = {{&S40222[13], 9}, {&S40222[23], 22}, {&S40222[66], 17}, {NULL}};
const Segment SGI279969[] = {{&S40223[103], 4}, {NULL}};
const Segment SGI279970[] = {{&S40223[13], 9}, {&S40223[23], 2}, {&S40223[26], 22}, {&S40223[49], 6}, {&S40223[77], 17}, {NULL}};
const Segment SGI279971[] = {{&S40224[103], 4}, {NULL}};
const Segment SGI279972[] = {{&S40224[13], 9}, {&S40224[23], 2}, {&S40224[26], 22}, {&S40224[49], 6}, {&S40224[77], 17}, {NULL}};
const Segment SGI279973[] = {{&S40257[92], 4}, {NULL}};
const Segment SGI279974[] = {{&S40257[13], 9}, {&S40257[23], 22}, {&S40257[66], 17}, {NULL}};
const Segment SGI279975[] = {{&S40258[92], 4}, {NULL}};
const Segment SGI279976[] = {{&S40258[13], 9}, {&S40258[23], 22}, {&S40258[66], 17}, {NULL}};
const Segment SGI279977[] = {{&S40259[92], 4}, {NULL}};
const Segment SGI279978[] = {{&S40259[13], 9}, {&S40259[23], 22}, {&S40259[66], 17}, {NULL}};
const Segment SGI279979[] = {{&S40260[92], 4}, {NULL}};
const Segment SGI279980[] = {{&S40260[13], 9}, {&S40260[23], 22}, {&S40260[66], 17}, {NULL}};
const Segment SGI279981[] = {{&S40261[93], 4}, {NULL}};
const Segment SGI279982[] = {{&S40261[13], 9}, {&S40261[23], 22}, {&S40261[67], 17}, {NULL}};
const Segment SGI279983[] = {{&S40262[92], 4}, {NULL}};
const Segment SGI279984[] = {{&S40262[13], 9}, {&S40262[23], 22}, {&S40262[66], 17}, {NULL}};
const Segment SGI279985[] = {{&S40263[92], 4}, {NULL}};
const Segment SGI279986[] = {{&S40263[13], 9}, {&S40263[23], 22}, {&S40263[66], 17}, {NULL}};
const Segment SGI279987[] = {{&S40264[92], 4}, {NULL}};
const Segment SGI279988[] = {{&S40264[13], 9}, {&S40264[23], 22}, {&S40264[66], 17}, {NULL}};
const Segment SGI279989[] = {{&S40265[103], 4}, {NULL}};
const Segment SGI279990[] = {{&S40265[13], 9}, {&S40265[23], 2}, {&S40265[26], 22}, {&S40265[49], 6}, {&S40265[77], 17}, {NULL}};
const Segment SGI279991[] = {{&S40266[103], 4}, {NULL}};
const Segment SGI279992[] = {{&S40266[13], 9}, {&S40266[23], 2}, {&S40266[26], 22}, {&S40266[49], 6}, {&S40266[77], 17}, {NULL}};
const Segment SGI279993[] = {{&S40330[41], 4}, {NULL}};
const Segment SGI279994[] = {{&S40330[13], 14}, {&S40330[28], 24}, {&S40330[74], 17}, {NULL}};
const Segment SGI279995[] = {{&S40330[29], 16}, {NULL}};
const Segment SGI279996[] = {{&S40331[41], 4}, {NULL}};
const Segment SGI279997[] = {{&S40331[13], 14}, {&S40331[28], 25}, {&S40331[75], 17}, {NULL}};
const Segment SGI279998[] = {{&S40331[29], 16}, {NULL}};
const Segment SGI279999[] = {{&S40333[41], 4}, {NULL}};
const Segment SGI280000[] = {{&S40333[13], 14}, {&S40333[28], 25}, {&S40333[75], 17}, {NULL}};
const Segment SGI280001[] = {{&S40333[29], 16}, {NULL}};
const Segment SGI280002[] = {{&S40335[41], 4}, {NULL}};
const Segment SGI280003[] = {{&S40335[13], 14}, {&S40335[28], 25}, {&S40335[75], 17}, {NULL}};
const Segment SGI280004[] = {{&S40335[29], 16}, {NULL}};
const Segment SGI280005[] = {{&S40337[41], 4}, {NULL}};
const Segment SGI280006[] = {{&S40337[13], 14}, {&S40337[28], 25}, {&S40337[75], 17}, {NULL}};
const Segment SGI280007[] = {{&S40337[29], 16}, {NULL}};
const Segment SGI280008[] = {{&S40339[41], 4}, {NULL}};
const Segment SGI280009[] = {{&S40339[13], 14}, {&S40339[28], 24}, {&S40339[74], 17}, {NULL}};
const Segment SGI280010[] = {{&S40339[29], 16}, {NULL}};
const Segment SGI280011[] = {{&S45564[85], 3}, {NULL}};
const Segment SGI280012[] = {{&S45564[13], 3}, {&S45564[17], 2}, {&S45564[20], 11}, {&S45564[32], 6}, {&S45564[59], 17}, {NULL}};
const Segment SGI280013[] = {{&S45565[87], 3}, {NULL}};
const Segment SGI280014[] = {{&S45565[13], 3}, {&S45565[17], 2}, {&S45565[20], 13}, {&S45565[34], 6}, {&S45565[61], 17}, {NULL}};
const Segment SGI280015[] = {{&S45566[94], 3}, {NULL}};
const Segment SGI280016[] = {{&S45566[13], 3}, {&S45566[17], 2}, {&S45566[20], 19}, {&S45566[41], 6}, {&S45566[68], 17}, {NULL}};
const Segment SGI280017[] = {{&S45567[87], 3}, {NULL}};
const Segment SGI280018[] = {{&S45567[13], 3}, {&S45567[17], 2}, {&S45567[20], 13}, {&S45567[34], 6}, {&S45567[61], 17}, {NULL}};
const Segment SGI280019[] = {{&S45568[97], 3}, {NULL}};
const Segment SGI280020[] = {{&S45568[13], 3}, {&S45568[17], 2}, {&S45568[20], 23}, {&S45568[44], 6}, {&S45568[71], 17}, {NULL}};
const Segment SGI280021[] = {{&S45638[94], 3}, {NULL}};
const Segment SGI280022[] = {{&S45638[13], 7}, {&S45638[21], 2}, {&S45638[24], 15}, {&S45638[40], 6}, {&S45638[68], 17}, {NULL}};
const Segment SGI280023[] = {{&S45639[13], 7}, {&S45639[21], 2}, {&S45639[24], 15}, {&S45639[40], 6}, {&S45639[67], 17}, {&S45639[95], 21}, {NULL}};
const Segment SGI280024[] = {{&S45640[13], 7}, {&S45640[21], 2}, {&S45640[24], 15}, {&S45640[40], 6}, {&S45640[67], 17}, {&S45640[95], 21}, {NULL}};
const Segment SGI280025[] = {{&S45641[13], 14}, {&S45641[28], 6}, {&S45641[55], 17}, {&S45641[83], 21}, {NULL}};
const Segment SGI280026[] = {{&S45642[13], 7}, {&S45642[21], 2}, {&S45642[24], 15}, {&S45642[40], 6}, {&S45642[67], 17}, {&S45642[95], 21}, {NULL}};
const Segment SGI280027[] = {{&S45643[13], 7}, {&S45643[21], 2}, {&S45643[24], 15}, {&S45643[40], 6}, {&S45643[67], 17}, {&S45643[95], 21}, {NULL}};
const Segment SGI280028[] = {{&S45644[13], 14}, {&S45644[48], 17}, {&S45644[76], 21}, {NULL}};
const Segment SGI280029[] = {{&S45645[13], 7}, {&S45645[21], 2}, {&S45645[24], 15}, {&S45645[40], 6}, {&S45645[67], 17}, {&S45645[95], 21}, {NULL}};
const Segment SGI280030[] = {{&S40569[93], 4}, {NULL}};
const Segment SGI280031[] = {{&S40569[13], 3}, {&S40569[17], 13}, {&S40569[52], 17}, {NULL}};
const Segment SGI280032[] = {{&S40569[71], 12}, {NULL}};
const Segment SGI280033[] = {{&S40570[91], 4}, {NULL}};
const Segment SGI280034[] = {{&S40570[13], 3}, {&S40570[17], 11}, {&S40570[50], 17}, {NULL}};
const Segment SGI280035[] = {{&S40570[69], 12}, {NULL}};
const Segment SGI280036[] = {{&S40571[91], 4}, {NULL}};
const Segment SGI280037[] = {{&S40571[13], 3}, {&S40571[17], 11}, {&S40571[50], 17}, {&S40571[140], 8}, {&S40571[151], 24}, {NULL}};
const Segment SGI280038[] = {{&S40571[69], 12}, {NULL}};
const Segment SGI280039[] = {{&S40572[91], 4}, {NULL}};
const Segment SGI280040[] = {{&S40572[13], 3}, {&S40572[17], 11}, {&S40572[50], 17}, {&S40572[140], 8}, {NULL}};
const Segment SGI280041[] = {{&S40572[69], 12}, {NULL}};
const Segment SGI280042[] = {{&S36975[75], 4}, {NULL}};
const Segment SGI280043[] = {{&S36975[13], 14}, {&S36975[49], 17}, {&S36975[121], 12}, {NULL}};
const Segment SGI280044[] = {{&S36975[102], 8}, {NULL}};
const Segment SGI280045[] = {{&S36976[75], 4}, {NULL}};
const Segment SGI280046[] = {{&S36976[13], 14}, {&S36976[49], 17}, {NULL}};
const Segment SGI280047[] = {{&S36976[102], 8}, {NULL}};
const Segment SGI280048[] = {{&S36977[75], 4}, {NULL}};
const Segment SGI280049[] = {{&S36977[13], 14}, {&S36977[49], 17}, {&S36977[121], 15}, {NULL}};
const Segment SGI280050[] = {{&S36977[102], 8}, {NULL}};
const Segment SGI280051[] = {{&S36978[82], 4}, {NULL}};
const Segment SGI280052[] = {{&S36978[13], 14}, {&S36978[28], 6}, {&S36978[56], 17}, {NULL}};
const Segment SGI280053[] = {{&S36978[109], 8}, {NULL}};
const Segment SGI280054[] = {{&S36979[75], 4}, {NULL}};
const Segment SGI280055[] = {{&S36979[13], 14}, {&S36979[49], 17}, {&S36979[121], 19}, {NULL}};
const Segment SGI280056[] = {{&S36979[102], 8}, {NULL}};
const Segment SGI280057[] = {{&S36980[75], 4}, {NULL}};
const Segment SGI280058[] = {{&S36980[13], 14}, {&S36980[49], 17}, {NULL}};
const Segment SGI280059[] = {{&S36980[102], 8}, {NULL}};
const Segment SGI280060[] = {{&S36981[75], 4}, {NULL}};
const Segment SGI280061[] = {{&S36981[13], 14}, {&S36981[49], 17}, {&S36981[120], 8}, {NULL}};
const Segment SGI280062[] = {{&S36981[102], 8}, {NULL}};
const Segment SGI280063[] = {{&S36982[75], 4}, {NULL}};
const Segment SGI280064[] = {{&S36982[13], 14}, {&S36982[49], 17}, {&S36982[121], 10}, {NULL}};
const Segment SGI280065[] = {{&S36982[102], 8}, {NULL}};
const Segment SGI280066[] = {{&S36983[75], 4}, {NULL}};
const Segment SGI280067[] = {{&S36983[13], 14}, {&S36983[49], 17}, {&S36983[121], 14}, {NULL}};
const Segment SGI280068[] = {{&S36983[102], 8}, {NULL}};
const Segment SGI280069[] = {{&S36984[75], 4}, {NULL}};
const Segment SGI280070[] = {{&S36984[13], 14}, {&S36984[49], 17}, {&S36984[121], 19}, {NULL}};
const Segment SGI280071[] = {{&S36984[102], 8}, {NULL}};
const Segment SGI280072[] = {{&S40780[75], 4}, {NULL}};
const Segment SGI280073[] = {{&S40780[13], 14}, {&S40780[49], 17}, {NULL}};
const Segment SGI280074[] = {{&S40780[111], 4}, {NULL}};
const Segment SGI280075[] = {{&S40793[75], 4}, {NULL}};
const Segment SGI280076[] = {{&S40793[13], 14}, {&S40793[49], 17}, {NULL}};
const Segment SGI280077[] = {{&S40793[111], 4}, {NULL}};
const Segment SGI280078[] = {{&S40890[91], 4}, {NULL}};
const Segment SGI280079[] = {{&S40890[13], 3}, {&S40890[17], 11}, {&S40890[50], 17}, {NULL}};
const Segment SGI280080[] = {{&S40890[69], 12}, {NULL}};
const Segment SGI280081[] = {{&S40891[93], 4}, {NULL}};
const Segment SGI280082[] = {{&S40891[13], 3}, {&S40891[17], 13}, {&S40891[52], 17}, {NULL}};
const Segment SGI280083[] = {{&S40891[71], 12}, {NULL}};
const Segment SGI280084[] = {{&S40892[100], 4}, {NULL}};
const Segment SGI280085[] = {{&S40892[13], 3}, {&S40892[17], 19}, {&S40892[59], 17}, {NULL}};
const Segment SGI280086[] = {{&S40892[78], 12}, {NULL}};
const Segment SGI280087[] = {{&S40998[81], 4}, {NULL}};
const Segment SGI280088[] = {{&S40998[13], 14}, {&S40998[28], 6}, {&S40998[55], 17}, {NULL}};
const Segment SGI280089[] = {{&S40999[74], 4}, {NULL}};
const Segment SGI280090[] = {{&S40999[13], 14}, {&S40999[48], 17}, {NULL}};
const Segment SGI280091[] = {{&S46073[8], 3}, {NULL}};
const Segment SGI280092[] = {{&S46073[13], 7}, {&S46073[21], 2}, {&S46073[24], 15}, {&S46073[40], 6}, {&S46073[67], 17}, {NULL}};
const Segment SGI280093[] = {{&S32219[7], 3}, {NULL}};
const Segment SGI280094[] = {{&S32219[11], 5}, {NULL}};
const Segment SGI280095[] = {{&S41638[90], 4}, {NULL}};
const Segment SGI280096[] = {{&S41638[13], 3}, {&S41638[17], 11}, {&S41638[49], 17}, {NULL}};
const Segment SGI280097[] = {{&S41638[68], 12}, {NULL}};
const Segment SGI280098[] = {{&S41639[92], 4}, {NULL}};
const Segment SGI280099[] = {{&S41639[13], 3}, {&S41639[17], 13}, {&S41639[51], 17}, {NULL}};
const Segment SGI280100[] = {{&S41639[70], 12}, {NULL}};
const Segment SGI280101[] = {{&S41640[90], 4}, {NULL}};
const Segment SGI280102[] = {{&S41640[13], 3}, {&S41640[17], 11}, {&S41640[49], 17}, {&S41640[136], 8}, {&S41640[147], 24}, {NULL}};
const Segment SGI280103[] = {{&S41640[68], 12}, {NULL}};
const Segment SGI280104[] = {{&S41641[90], 4}, {NULL}};
const Segment SGI280105[] = {{&S41641[13], 3}, {&S41641[17], 11}, {&S41641[49], 17}, {NULL}};
const Segment SGI280106[] = {{&S41641[68], 12}, {NULL}};
const Segment SGI280107[] = {{&S41763[93], 4}, {NULL}};
const Segment SGI280108[] = {{&S41763[13], 9}, {&S41763[23], 22}, {&S41763[67], 17}, {NULL}};
const Segment SGI280109[] = {{&S41764[93], 4}, {NULL}};
const Segment SGI280110[] = {{&S41764[13], 9}, {&S41764[23], 22}, {&S41764[67], 17}, {NULL}};
const Segment SGI280111[] = {{&S41765[93], 4}, {NULL}};
const Segment SGI280112[] = {{&S41765[13], 9}, {&S41765[23], 22}, {&S41765[67], 17}, {NULL}};
const Segment SGI280113[] = {{&S41766[93], 4}, {NULL}};
const Segment SGI280114[] = {{&S41766[13], 9}, {&S41766[23], 22}, {&S41766[67], 17}, {NULL}};
const Segment SGI280115[] = {{&S41767[93], 4}, {NULL}};
const Segment SGI280116[] = {{&S41767[13], 9}, {&S41767[23], 22}, {&S41767[67], 17}, {NULL}};
const Segment SGI280117[] = {{&S41768[93], 4}, {NULL}};
const Segment SGI280118[] = {{&S41768[13], 9}, {&S41768[23], 22}, {&S41768[67], 17}, {NULL}};
const Segment SGI280119[] = {{&S41955[92], 4}, {NULL}};
const Segment SGI280120[] = {{&S41955[13], 9}, {&S41955[23], 22}, {&S41955[66], 17}, {NULL}};
const Segment SGI280121[] = {{&S41956[92], 4}, {NULL}};
const Segment SGI280122[] = {{&S41956[13], 9}, {&S41956[23], 22}, {&S41956[66], 17}, {NULL}};
const Segment SGI280123[] = {{&S41957[92], 4}, {NULL}};
const Segment SGI280124[] = {{&S41957[13], 9}, {&S41957[23], 22}, {&S41957[66], 17}, {NULL}};
const Segment SGI280125[] = {{&S41958[92], 4}, {NULL}};
const Segment SGI280126[] = {{&S41958[13], 9}, {&S41958[23], 22}, {&S41958[66], 17}, {NULL}};
const Segment SGI280127[] = {{&S41959[92], 4}, {NULL}};
const Segment SGI280128[] = {{&S41959[13], 9}, {&S41959[23], 22}, {&S41959[66], 17}, {NULL}};
const Segment SGI280129[] = {{&S41960[92], 4}, {NULL}};
const Segment SGI280130[] = {{&S41960[13], 9}, {&S41960[23], 22}, {&S41960[66], 17}, {NULL}};
const Segment SGI280131[] = {{&S41961[92], 4}, {NULL}};
const Segment SGI280132[] = {{&S41961[13], 9}, {&S41961[23], 22}, {&S41961[66], 17}, {NULL}};
const Segment SGI280133[] = {{&S41962[92], 4}, {NULL}};
const Segment SGI280134[] = {{&S41962[13], 9}, {&S41962[23], 22}, {&S41962[66], 17}, {NULL}};
const Segment SGI280135[] = {{&S41963[103], 4}, {NULL}};
const Segment SGI280136[] = {{&S41963[13], 9}, {&S41963[23], 2}, {&S41963[26], 22}, {&S41963[49], 6}, {&S41963[77], 17}, {NULL}};
const Segment SGI280137[] = {{&S41964[103], 4}, {NULL}};
const Segment SGI280138[] = {{&S41964[13], 9}, {&S41964[23], 2}, {&S41964[26], 22}, {&S41964[49], 6}, {&S41964[77], 17}, {NULL}};
const Segment SGI280139[] = {{&S42048[92], 4}, {NULL}};
const Segment SGI280140[] = {{&S42048[13], 9}, {&S42048[23], 22}, {&S42048[66], 17}, {NULL}};
const Segment SGI280141[] = {{&S42049[92], 4}, {NULL}};
const Segment SGI280142[] = {{&S42049[13], 9}, {&S42049[23], 22}, {&S42049[66], 17}, {NULL}};
const Segment SGI280143[] = {{&S42050[92], 4}, {NULL}};
const Segment SGI280144[] = {{&S42050[13], 9}, {&S42050[23], 22}, {&S42050[66], 17}, {NULL}};
const Segment SGI280145[] = {{&S42051[92], 4}, {NULL}};
const Segment SGI280146[] = {{&S42051[13], 9}, {&S42051[23], 22}, {&S42051[66], 17}, {NULL}};
const Segment SGI280147[] = {{&S42052[92], 4}, {NULL}};
const Segment SGI280148[] = {{&S42052[13], 9}, {&S42052[23], 22}, {&S42052[66], 17}, {NULL}};
const Segment SGI280149[] = {{&S42053[92], 4}, {NULL}};
const Segment SGI280150[] = {{&S42053[13], 9}, {&S42053[23], 22}, {&S42053[66], 17}, {NULL}};
const Segment SGI280151[] = {{&S42054[92], 4}, {NULL}};
const Segment SGI280152[] = {{&S42054[13], 9}, {&S42054[23], 22}, {&S42054[66], 17}, {NULL}};
const Segment SGI280153[] = {{&S42055[92], 4}, {NULL}};
const Segment SGI280154[] = {{&S42055[13], 9}, {&S42055[23], 22}, {&S42055[66], 17}, {NULL}};
const Segment SGI280155[] = {{&S42056[103], 4}, {NULL}};
const Segment SGI280156[] = {{&S42056[13], 9}, {&S42056[23], 2}, {&S42056[26], 22}, {&S42056[49], 6}, {&S42056[77], 17}, {NULL}};
const Segment SGI280157[] = {{&S42057[103], 4}, {NULL}};
const Segment SGI280158[] = {{&S42057[13], 9}, {&S42057[23], 2}, {&S42057[26], 22}, {&S42057[49], 6}, {&S42057[77], 17}, {NULL}};
const Segment SGI280159[] = {{&S47155[93], 3}, {NULL}};
const Segment SGI280160[] = {{&S47155[13], 7}, {&S47155[21], 2}, {&S47155[24], 15}, {&S47155[40], 6}, {&S47155[67], 17}, {NULL}};
const Segment SGI280161[] = {{&S47156[82], 3}, {NULL}};
const Segment SGI280162[] = {{&S47156[13], 14}, {&S47156[28], 6}, {&S47156[56], 17}, {NULL}};
const Segment SGI280163[] = {{&S38484[93], 4}, {NULL}};
const Segment SGI280164[] = {{&S38484[13], 14}, {&S38484[48], 17}, {NULL}};
const Segment SGI280165[] = {{&S38484[67], 11}, {NULL}};
const Segment SGI280166[] = {{&S38485[93], 4}, {NULL}};
const Segment SGI280167[] = {{&S38485[13], 14}, {&S38485[48], 17}, {NULL}};
const Segment SGI280168[] = {{&S38485[67], 11}, {NULL}};
const Segment SGI280169[] = {{&S42364[93], 4}, {NULL}};
const Segment SGI280170[] = {{&S42364[13], 9}, {&S42364[23], 22}, {&S42364[67], 17}, {NULL}};
const Segment SGI280171[] = {{&S42365[93], 4}, {NULL}};
const Segment SGI280172[] = {{&S42365[13], 9}, {&S42365[23], 22}, {&S42365[67], 17}, {NULL}};
const Segment SGI280173[] = {{&S42366[93], 4}, {NULL}};
const Segment SGI280174[] = {{&S42366[13], 9}, {&S42366[23], 22}, {&S42366[67], 17}, {NULL}};
const Segment SGI280175[] = {{&S42367[93], 4}, {NULL}};
const Segment SGI280176[] = {{&S42367[13], 9}, {&S42367[23], 22}, {&S42367[67], 17}, {NULL}};
const Segment SGI280177[] = {{&S42368[93], 4}, {NULL}};
const Segment SGI280178[] = {{&S42368[13], 9}, {&S42368[23], 22}, {&S42368[67], 17}, {NULL}};
const Segment SGI280179[] = {{&S42369[93], 4}, {NULL}};
const Segment SGI280180[] = {{&S42369[13], 9}, {&S42369[23], 22}, {&S42369[67], 17}, {NULL}};
const Segment SGI280181[] = {{&S42370[93], 4}, {NULL}};
const Segment SGI280182[] = {{&S42370[13], 9}, {&S42370[23], 22}, {&S42370[67], 17}, {NULL}};
const Segment SGI280183[] = {{&S42371[93], 4}, {NULL}};
const Segment SGI280184[] = {{&S42371[13], 9}, {&S42371[23], 22}, {&S42371[67], 17}, {NULL}};
const Segment SGI280185[] = {{&S42372[103], 4}, {NULL}};
const Segment SGI280186[] = {{&S42372[13], 9}, {&S42372[23], 2}, {&S42372[26], 22}, {&S42372[49], 6}, {&S42372[77], 17}, {NULL}};
const Segment SGI280187[] = {{&S42373[103], 4}, {NULL}};
const Segment SGI280188[] = {{&S42373[13], 9}, {&S42373[23], 2}, {&S42373[26], 22}, {&S42373[49], 6}, {&S42373[77], 17}, {NULL}};
const Segment SGI280189[] = {{&S47304[94], 3}, {NULL}};
const Segment SGI280190[] = {{&S47304[13], 7}, {&S47304[21], 2}, {&S47304[24], 15}, {&S47304[40], 6}, {&S47304[68], 17}, {NULL}};
const Segment SGI280191[] = {{&S38486[93], 4}, {NULL}};
const Segment SGI280192[] = {{&S38486[13], 14}, {&S38486[48], 17}, {&S38486[119], 8}, {NULL}};
const Segment SGI280193[] = {{&S38486[67], 11}, {NULL}};
const Segment SGI280194[] = {{&S38487[100], 4}, {NULL}};
const Segment SGI280195[] = {{&S38487[13], 14}, {&S38487[28], 6}, {&S38487[55], 17}, {NULL}};
const Segment SGI280196[] = {{&S38487[74], 11}, {NULL}};
const Segment SGI280197[] = {{&S38488[100], 4}, {NULL}};
const Segment SGI280198[] = {{&S38488[13], 14}, {&S38488[28], 6}, {&S38488[55], 17}, {NULL}};
const Segment SGI280199[] = {{&S38488[74], 11}, {NULL}};
const Segment SGI280200[] = {{&S47390[94], 3}, {NULL}};
const Segment SGI280201[] = {{&S47390[13], 7}, {&S47390[21], 2}, {&S47390[24], 15}, {&S47390[40], 6}, {&S47390[68], 17}, {NULL}};
const Segment SGI280202[] = {{&S42495[48], 4}, {NULL}};
const Segment SGI280203[] = {{&S42495[13], 14}, {&S42495[28], 6}, {&S42495[35], 24}, {&S42495[81], 17}, {NULL}};
const Segment SGI280204[] = {{&S42495[36], 16}, {NULL}};
const Segment SGI280205[] = {{&S42496[48], 4}, {NULL}};
const Segment SGI280206[] = {{&S42496[13], 14}, {&S42496[28], 6}, {&S42496[35], 24}, {&S42496[81], 17}, {NULL}};
const Segment SGI280207[] = {{&S42496[36], 16}, {NULL}};
const Segment SGI280208[] = {{&S42497[48], 4}, {NULL}};
const Segment SGI280209[] = {{&S42497[13], 14}, {&S42497[28], 6}, {&S42497[35], 24}, {&S42497[81], 17}, {NULL}};
const Segment SGI280210[] = {{&S42497[36], 16}, {NULL}};
const Segment SGI280211[] = {{&S42500[41], 4}, {NULL}};
const Segment SGI280212[] = {{&S42500[13], 14}, {&S42500[28], 24}, {&S42500[74], 17}, {NULL}};
const Segment SGI280213[] = {{&S42500[29], 16}, {NULL}};
const Segment SGI280214[] = {{&S42501[48], 4}, {NULL}};
const Segment SGI280215[] = {{&S42501[13], 14}, {&S42501[28], 6}, {&S42501[35], 24}, {&S42501[81], 17}, {NULL}};
const Segment SGI280216[] = {{&S42501[36], 16}, {NULL}};
const Segment SGI280217[] = {{&S42502[41], 4}, {NULL}};
const Segment SGI280218[] = {{&S42502[13], 14}, {&S42502[28], 24}, {&S42502[74], 17}, {NULL}};
const Segment SGI280219[] = {{&S42502[29], 16}, {NULL}};
const Segment SGI280220[] = {{&S42503[48], 4}, {NULL}};
const Segment SGI280221[] = {{&S42503[13], 14}, {&S42503[28], 6}, {&S42503[35], 24}, {&S42503[81], 17}, {NULL}};
const Segment SGI280222[] = {{&S42503[36], 16}, {NULL}};
const Segment SGI280223[] = {{&S42504[48], 4}, {NULL}};
const Segment SGI280224[] = {{&S42504[13], 14}, {&S42504[28], 6}, {&S42504[35], 24}, {&S42504[81], 17}, {NULL}};
const Segment SGI280225[] = {{&S42504[36], 16}, {NULL}};
const Segment SGI280226[] = {{&S42505[41], 4}, {NULL}};
const Segment SGI280227[] = {{&S42505[13], 14}, {&S42505[28], 24}, {&S42505[74], 17}, {NULL}};
const Segment SGI280228[] = {{&S42505[29], 16}, {NULL}};
const Segment SGI280229[] = {{&S42506[41], 4}, {NULL}};
const Segment SGI280230[] = {{&S42506[13], 14}, {&S42506[28], 24}, {&S42506[74], 17}, {NULL}};
const Segment SGI280231[] = {{&S42506[29], 16}, {NULL}};
const Segment SGI280232[] = {{&S42507[48], 4}, {NULL}};
const Segment SGI280233[] = {{&S42507[13], 14}, {&S42507[28], 6}, {&S42507[35], 24}, {&S42507[81], 17}, {NULL}};
const Segment SGI280234[] = {{&S42507[36], 16}, {NULL}};
const Segment SGI280235[] = {{&S42509[48], 4}, {NULL}};
const Segment SGI280236[] = {{&S42509[13], 14}, {&S42509[28], 6}, {&S42509[35], 24}, {&S42509[81], 17}, {NULL}};
const Segment SGI280237[] = {{&S42509[36], 16}, {NULL}};
const Segment SGI280238[] = {{&S42512[41], 4}, {NULL}};
const Segment SGI280239[] = {{&S42512[13], 14}, {&S42512[28], 24}, {&S42512[74], 17}, {NULL}};
const Segment SGI280240[] = {{&S42512[29], 16}, {NULL}};
const Segment SGI280241[] = {{&S42514[41], 4}, {NULL}};
const Segment SGI280242[] = {{&S42514[13], 14}, {&S42514[28], 24}, {&S42514[74], 17}, {NULL}};
const Segment SGI280243[] = {{&S42514[29], 16}, {NULL}};
const Segment SGI280244[] = {{&S42530[93], 4}, {NULL}};
const Segment SGI280245[] = {{&S42530[13], 9}, {&S42530[23], 22}, {&S42530[67], 17}, {NULL}};
const Segment SGI280246[] = {{&S42531[93], 4}, {NULL}};
const Segment SGI280247[] = {{&S42531[13], 9}, {&S42531[23], 22}, {&S42531[67], 17}, {NULL}};
const Segment SGI280248[] = {{&S42532[93], 4}, {NULL}};
const Segment SGI280249[] = {{&S42532[13], 9}, {&S42532[23], 22}, {&S42532[67], 17}, {NULL}};
const Segment SGI280250[] = {{&S42533[93], 4}, {NULL}};
const Segment SGI280251[] = {{&S42533[13], 9}, {&S42533[23], 22}, {&S42533[67], 17}, {NULL}};
const Segment SGI280252[] = {{&S42534[93], 4}, {NULL}};
const Segment SGI280253[] = {{&S42534[13], 9}, {&S42534[23], 22}, {&S42534[67], 17}, {NULL}};
const Segment SGI280254[] = {{&S42535[93], 4}, {NULL}};
const Segment SGI280255[] = {{&S42535[13], 9}, {&S42535[23], 22}, {&S42535[67], 17}, {NULL}};
const Segment SGI280256[] = {{&S42536[93], 4}, {NULL}};
const Segment SGI280257[] = {{&S42536[13], 9}, {&S42536[23], 22}, {&S42536[67], 17}, {NULL}};
const Segment SGI280258[] = {{&S42537[93], 4}, {NULL}};
const Segment SGI280259[] = {{&S42537[13], 9}, {&S42537[23], 22}, {&S42537[67], 17}, {NULL}};
const Segment SGI280260[] = {{&S42538[103], 4}, {NULL}};
const Segment SGI280261[] = {{&S42538[13], 9}, {&S42538[23], 2}, {&S42538[26], 22}, {&S42538[49], 6}, {&S42538[77], 17}, {NULL}};
const Segment SGI280262[] = {{&S42539[93], 4}, {NULL}};
const Segment SGI280263[] = {{&S42539[13], 9}, {&S42539[23], 22}, {&S42539[67], 17}, {NULL}};
const Segment SGI280264[] = {{&S42540[103], 4}, {NULL}};
const Segment SGI280265[] = {{&S42540[13], 9}, {&S42540[23], 2}, {&S42540[26], 22}, {&S42540[49], 6}, {&S42540[77], 17}, {NULL}};
const Segment SGI280266[] = {{&S42541[103], 4}, {NULL}};
const Segment SGI280267[] = {{&S42541[13], 9}, {&S42541[23], 2}, {&S42541[26], 22}, {&S42541[49], 6}, {&S42541[77], 17}, {NULL}};
const Segment SGI280268[] = {{&S42586[48], 4}, {NULL}};
const Segment SGI280269[] = {{&S42586[13], 14}, {&S42586[28], 6}, {&S42586[35], 25}, {&S42586[81], 17}, {NULL}};
const Segment SGI280270[] = {{&S42586[36], 16}, {NULL}};
const Segment SGI280271[] = {{&S42587[41], 4}, {NULL}};
const Segment SGI280272[] = {{&S42587[13], 14}, {&S42587[28], 25}, {&S42587[74], 17}, {NULL}};
const Segment SGI280273[] = {{&S42587[29], 16}, {NULL}};
const Segment SGI280274[] = {{&S42590[41], 4}, {NULL}};
const Segment SGI280275[] = {{&S42590[13], 14}, {&S42590[28], 25}, {&S42590[74], 17}, {NULL}};
const Segment SGI280276[] = {{&S42590[29], 16}, {NULL}};
const Segment SGI280277[] = {{&S42591[48], 4}, {NULL}};
const Segment SGI280278[] = {{&S42591[13], 14}, {&S42591[28], 6}, {&S42591[35], 25}, {&S42591[81], 17}, {NULL}};
const Segment SGI280279[] = {{&S42591[36], 16}, {NULL}};
const Segment SGI280280[] = {{&S42593[48], 4}, {NULL}};
const Segment SGI280281[] = {{&S42593[13], 14}, {&S42593[28], 6}, {&S42593[35], 25}, {&S42593[81], 17}, {NULL}};
const Segment SGI280282[] = {{&S42593[36], 16}, {NULL}};
const Segment SGI280283[] = {{&S42594[41], 4}, {NULL}};
const Segment SGI280284[] = {{&S42594[13], 14}, {&S42594[28], 25}, {&S42594[74], 17}, {NULL}};
const Segment SGI280285[] = {{&S42594[29], 16}, {NULL}};
const Segment SGI280286[] = {{&S42598[48], 4}, {NULL}};
const Segment SGI280287[] = {{&S42598[13], 14}, {&S42598[28], 6}, {&S42598[35], 23}, {&S42598[79], 17}, {NULL}};
const Segment SGI280288[] = {{&S42598[36], 16}, {NULL}};
const Segment SGI280289[] = {{&S42599[41], 4}, {NULL}};
const Segment SGI280290[] = {{&S42599[13], 14}, {&S42599[28], 23}, {&S42599[72], 17}, {NULL}};
const Segment SGI280291[] = {{&S42599[29], 16}, {NULL}};
const Segment SGI280292[] = {{&S42604[75], 4}, {NULL}};
const Segment SGI280293[] = {{&S42604[13], 14}, {&S42604[49], 17}, {NULL}};
const Segment SGI280294[] = {{&S42605[82], 4}, {NULL}};
const Segment SGI280295[] = {{&S42605[13], 14}, {&S42605[28], 6}, {&S42605[56], 17}, {NULL}};
const Segment SGI280296[] = {{&S42606[84], 4}, {NULL}};
const Segment SGI280297[] = {{&S42606[13], 7}, {&S42606[21], 15}, {&S42606[58], 17}, {NULL}};
const Segment SGI280298[] = {{&S42607[94], 4}, {NULL}};
const Segment SGI280299[] = {{&S42607[13], 7}, {&S42607[21], 2}, {&S42607[24], 15}, {&S42607[40], 6}, {&S42607[68], 17}, {NULL}};
const Segment SGI280300[] = {{&S42616[113], 4}, {NULL}};
const Segment SGI280301[] = {{&S42616[13], 9}, {&S42616[23], 22}, {&S42616[66], 17}, {NULL}};
const Segment SGI280302[] = {{&S42616[94], 4}, {NULL}};
const Segment SGI280303[] = {{&S42617[113], 4}, {NULL}};
const Segment SGI280304[] = {{&S42617[13], 9}, {&S42617[23], 22}, {&S42617[66], 17}, {NULL}};
const Segment SGI280305[] = {{&S42617[94], 4}, {NULL}};
const Segment SGI280306[] = {{&S42618[113], 4}, {NULL}};
const Segment SGI280307[] = {{&S42618[13], 9}, {&S42618[23], 22}, {&S42618[66], 17}, {NULL}};
const Segment SGI280308[] = {{&S42618[94], 4}, {NULL}};
const Segment SGI280309[] = {{&S42619[113], 4}, {NULL}};
const Segment SGI280310[] = {{&S42619[13], 9}, {&S42619[23], 22}, {&S42619[66], 17}, {NULL}};
const Segment SGI280311[] = {{&S42619[94], 4}, {NULL}};
const Segment SGI280312[] = {{&S42620[113], 4}, {NULL}};
const Segment SGI280313[] = {{&S42620[13], 9}, {&S42620[23], 22}, {&S42620[66], 17}, {NULL}};
const Segment SGI280314[] = {{&S42620[94], 4}, {NULL}};
const Segment SGI280315[] = {{&S42621[113], 4}, {NULL}};
const Segment SGI280316[] = {{&S42621[13], 9}, {&S42621[23], 22}, {&S42621[66], 17}, {NULL}};
const Segment SGI280317[] = {{&S42621[94], 4}, {NULL}};
const Segment SGI280318[] = {{&S38666[93], 4}, {NULL}};
const Segment SGI280319[] = {{&S38666[13], 9}, {&S38666[23], 22}, {&S38666[67], 17}, {NULL}};
const Segment SGI280320[] = {{&S38667[93], 4}, {NULL}};
const Segment SGI280321[] = {{&S38667[13], 9}, {&S38667[23], 22}, {&S38667[67], 17}, {NULL}};
const Segment SGI280322[] = {{&S38668[93], 4}, {NULL}};
const Segment SGI280323[] = {{&S38668[13], 9}, {&S38668[23], 22}, {&S38668[67], 17}, {NULL}};
const Segment SGI280324[] = {{&S38669[93], 4}, {NULL}};
const Segment SGI280325[] = {{&S38669[13], 9}, {&S38669[23], 22}, {&S38669[67], 17}, {NULL}};
const Segment SGI280326[] = {{&S38670[93], 4}, {NULL}};
const Segment SGI280327[] = {{&S38670[13], 9}, {&S38670[23], 22}, {&S38670[67], 17}, {NULL}};
const Segment SGI280328[] = {{&S38671[93], 4}, {NULL}};
const Segment SGI280329[] = {{&S38671[13], 9}, {&S38671[23], 22}, {&S38671[67], 17}, {NULL}};
const Segment SGI280330[] = {{&S38672[93], 4}, {NULL}};
const Segment SGI280331[] = {{&S38672[13], 9}, {&S38672[23], 22}, {&S38672[67], 17}, {&S38672[120], 8}, {&S38672[131], 24}, {NULL}};
const Segment SGI280332[] = {{&S38673[93], 4}, {NULL}};
const Segment SGI280333[] = {{&S38673[13], 9}, {&S38673[23], 22}, {&S38673[67], 17}, {NULL}};
const Segment SGI280334[] = {{&S38674[93], 4}, {NULL}};
const Segment SGI280335[] = {{&S38674[13], 9}, {&S38674[23], 22}, {&S38674[67], 17}, {NULL}};
const Segment SGI280336[] = {{&S38675[103], 4}, {NULL}};
const Segment SGI280337[] = {{&S38675[13], 9}, {&S38675[23], 2}, {&S38675[26], 22}, {&S38675[49], 6}, {&S38675[77], 17}, {NULL}};
const Segment SGI280338[] = {{&S38676[103], 4}, {NULL}};
const Segment SGI280339[] = {{&S38676[13], 9}, {&S38676[23], 2}, {&S38676[26], 22}, {&S38676[49], 6}, {&S38676[77], 17}, {NULL}};
const Segment SGI280340[] = {{&S42972[94], 4}, {NULL}};
const Segment SGI280341[] = {{&S42972[13], 7}, {&S42972[21], 2}, {&S42972[24], 15}, {&S42972[40], 6}, {&S42972[68], 17}, {NULL}};
const Segment SGI280342[] = {{&S42973[82], 4}, {NULL}};
const Segment SGI280343[] = {{&S42973[13], 14}, {&S42973[28], 6}, {&S42973[56], 17}, {NULL}};
const Segment SGI280344[] = {{&S47910[102], 3}, {NULL}};
const Segment SGI280345[] = {{&S47910[13], 9}, {&S47910[23], 2}, {&S47910[26], 22}, {&S47910[49], 6}, {&S47910[76], 17}, {NULL}};
const Segment SGI280346[] = {{&S47911[102], 3}, {NULL}};
const Segment SGI280347[] = {{&S47911[13], 9}, {&S47911[23], 2}, {&S47911[26], 22}, {&S47911[49], 6}, {&S47911[76], 17}, {NULL}};
const Segment SGI280348[] = {{&S47912[102], 3}, {NULL}};
const Segment SGI280349[] = {{&S47912[13], 9}, {&S47912[23], 2}, {&S47912[26], 22}, {&S47912[49], 6}, {&S47912[76], 17}, {NULL}};
const Segment SGI280350[] = {{&S47913[102], 3}, {NULL}};
const Segment SGI280351[] = {{&S47913[13], 9}, {&S47913[23], 2}, {&S47913[26], 22}, {&S47913[49], 6}, {&S47913[76], 17}, {NULL}};
const Segment SGI280352[] = {{&S47914[102], 3}, {NULL}};
const Segment SGI280353[] = {{&S47914[13], 9}, {&S47914[23], 2}, {&S47914[26], 22}, {&S47914[49], 6}, {&S47914[76], 17}, {NULL}};
const Segment SGI280354[] = {{&S47915[102], 3}, {NULL}};
const Segment SGI280355[] = {{&S47915[13], 9}, {&S47915[23], 2}, {&S47915[26], 22}, {&S47915[49], 6}, {&S47915[76], 17}, {NULL}};
const Segment SGI280356[] = {{&S47916[102], 3}, {NULL}};
const Segment SGI280357[] = {{&S47916[13], 9}, {&S47916[23], 2}, {&S47916[26], 22}, {&S47916[49], 6}, {&S47916[76], 17}, {NULL}};
const Segment SGI280358[] = {{&S47917[102], 3}, {NULL}};
const Segment SGI280359[] = {{&S47917[13], 9}, {&S47917[23], 2}, {&S47917[26], 22}, {&S47917[49], 6}, {&S47917[76], 17}, {NULL}};
const Segment SGI280360[] = {{&S47918[102], 3}, {NULL}};
const Segment SGI280361[] = {{&S47918[13], 9}, {&S47918[23], 2}, {&S47918[26], 22}, {&S47918[49], 6}, {&S47918[76], 17}, {NULL}};
const Segment SGI280362[] = {{&S47921[103], 3}, {NULL}};
const Segment SGI280363[] = {{&S47921[13], 9}, {&S47921[23], 2}, {&S47921[26], 22}, {&S47921[49], 6}, {&S47921[77], 17}, {NULL}};
const Segment SGI280364[] = {{&S47922[103], 3}, {NULL}};
const Segment SGI280365[] = {{&S47922[13], 9}, {&S47922[23], 2}, {&S47922[26], 22}, {&S47922[49], 6}, {&S47922[77], 17}, {NULL}};
const Segment SGI280366[] = {{&S47923[103], 3}, {NULL}};
const Segment SGI280367[] = {{&S47923[13], 9}, {&S47923[23], 2}, {&S47923[26], 22}, {&S47923[49], 6}, {&S47923[77], 17}, {NULL}};
const Segment SGI280368[] = {{&S47924[103], 3}, {NULL}};
const Segment SGI280369[] = {{&S47924[13], 9}, {&S47924[23], 2}, {&S47924[26], 22}, {&S47924[49], 6}, {&S47924[77], 17}, {NULL}};
const Segment SGI280370[] = {{&S47925[103], 3}, {NULL}};
const Segment SGI280371[] = {{&S47925[13], 9}, {&S47925[23], 2}, {&S47925[26], 22}, {&S47925[49], 6}, {&S47925[77], 17}, {NULL}};
const Segment SGI280372[] = {{&S47926[103], 3}, {NULL}};
const Segment SGI280373[] = {{&S47926[13], 9}, {&S47926[23], 2}, {&S47926[26], 22}, {&S47926[49], 6}, {&S47926[77], 17}, {NULL}};
const Segment SGI280374[] = {{&S47927[103], 3}, {NULL}};
const Segment SGI280375[] = {{&S47927[13], 9}, {&S47927[23], 2}, {&S47927[26], 22}, {&S47927[49], 6}, {&S47927[77], 17}, {NULL}};
const Segment SGI280376[] = {{&S47928[103], 3}, {NULL}};
const Segment SGI280377[] = {{&S47928[13], 9}, {&S47928[23], 2}, {&S47928[26], 22}, {&S47928[49], 6}, {&S47928[77], 17}, {NULL}};
const Segment SGI280378[] = {{&S47929[103], 3}, {NULL}};
const Segment SGI280379[] = {{&S47929[13], 9}, {&S47929[23], 2}, {&S47929[26], 22}, {&S47929[49], 6}, {&S47929[77], 17}, {NULL}};
const Segment SGI280380[] = {{&S48034[100], 3}, {NULL}};
const Segment SGI280381[] = {{&S48034[13], 3}, {&S48034[17], 2}, {&S48034[20], 11}, {&S48034[32], 6}, {&S48034[59], 17}, {NULL}};
const Segment SGI280382[] = {{&S48034[78], 12}, {NULL}};
const Segment SGI280383[] = {{&S48035[102], 3}, {NULL}};
const Segment SGI280384[] = {{&S48035[13], 3}, {&S48035[17], 2}, {&S48035[20], 13}, {&S48035[34], 6}, {&S48035[61], 17}, {NULL}};
const Segment SGI280385[] = {{&S48035[80], 12}, {NULL}};
const Segment SGI280386[] = {{&S39210[93], 4}, {NULL}};
const Segment SGI280387[] = {{&S39210[13], 14}, {&S39210[28], 6}, {&S39210[55], 17}, {NULL}};
const Segment SGI280388[] = {{&S39210[74], 11}, {NULL}};
const Segment SGI280389[] = {{&S39211[86], 4}, {NULL}};
const Segment SGI280390[] = {{&S39211[13], 14}, {&S39211[48], 17}, {NULL}};
const Segment SGI280391[] = {{&S39211[67], 11}, {NULL}};
const Segment SGI280392[] = {{&S39215[93], 4}, {NULL}};
const Segment SGI280393[] = {{&S39215[13], 14}, {&S39215[28], 6}, {&S39215[55], 17}, {&S39215[119], 8}, {NULL}};
const Segment SGI280394[] = {{&S39215[74], 11}, {NULL}};
const Segment SGI280395[] = {{&S39400[100], 4}, {NULL}};
const Segment SGI280396[] = {{&S39400[13], 14}, {&S39400[28], 6}, {&S39400[55], 17}, {NULL}};
const Segment SGI280397[] = {{&S39400[74], 11}, {NULL}};
const Segment SGI280398[] = {{&S39401[93], 4}, {NULL}};
const Segment SGI280399[] = {{&S39401[13], 14}, {&S39401[48], 17}, {NULL}};
const Segment SGI280400[] = {{&S39401[67], 11}, {NULL}};
const Segment SGI280401[] = {{&S39461[93], 4}, {NULL}};
const Segment SGI280402[] = {{&S39461[13], 14}, {&S39461[48], 17}, {NULL}};
const Segment SGI280403[] = {{&S39461[67], 11}, {NULL}};
const Segment SGI280404[] = {{&S39462[100], 4}, {NULL}};
const Segment SGI280405[] = {{&S39462[13], 14}, {&S39462[28], 6}, {&S39462[55], 17}, {NULL}};
const Segment SGI280406[] = {{&S39462[74], 11}, {NULL}};
const Segment SGI280407[] = {{&S39463[93], 4}, {NULL}};
const Segment SGI280408[] = {{&S39463[13], 14}, {&S39463[48], 17}, {NULL}};
const Segment SGI280409[] = {{&S39463[67], 11}, {NULL}};
const Segment SGI280410[] = {{&S39464[100], 4}, {NULL}};
const Segment SGI280411[] = {{&S39464[13], 14}, {&S39464[28], 6}, {&S39464[55], 17}, {NULL}};
const Segment SGI280412[] = {{&S39464[74], 11}, {NULL}};
const Segment SGI280413[] = {{&S39644[86], 4}, {NULL}};
const Segment SGI280414[] = {{&S39644[13], 14}, {&S39644[48], 17}, {NULL}};
const Segment SGI280415[] = {{&S39644[67], 11}, {NULL}};
const Segment SGI280416[] = {{&S39645[93], 4}, {NULL}};
const Segment SGI280417[] = {{&S39645[13], 14}, {&S39645[28], 6}, {&S39645[55], 17}, {NULL}};
const Segment SGI280418[] = {{&S39645[74], 11}, {NULL}};
const Segment SGI280419[] = {{&S43136[94], 4}, {NULL}};
const Segment SGI280420[] = {{&S43136[13], 7}, {&S43136[21], 2}, {&S43136[24], 15}, {&S43136[40], 6}, {&S43136[68], 17}, {NULL}};
const Segment SGI280421[] = {{&S43136[121], 16}, {NULL}};
const Segment SGI280422[] = {{&S43141[92], 4}, {NULL}};
const Segment SGI280423[] = {{&S43141[13], 9}, {&S43141[23], 22}, {&S43141[66], 17}, {NULL}};
const Segment SGI280424[] = {{&S43142[92], 4}, {NULL}};
const Segment SGI280425[] = {{&S43142[13], 9}, {&S43142[23], 22}, {&S43142[66], 17}, {NULL}};
const Segment SGI280426[] = {{&S43143[92], 4}, {NULL}};
const Segment SGI280427[] = {{&S43143[13], 9}, {&S43143[23], 22}, {&S43143[66], 17}, {NULL}};
const Segment SGI280428[] = {{&S43144[92], 4}, {NULL}};
const Segment SGI280429[] = {{&S43144[13], 9}, {&S43144[23], 22}, {&S43144[66], 17}, {NULL}};
const Segment SGI280430[] = {{&S43145[92], 4}, {NULL}};
const Segment SGI280431[] = {{&S43145[13], 9}, {&S43145[23], 22}, {&S43145[66], 17}, {NULL}};
const Segment SGI280432[] = {{&S43146[92], 4}, {NULL}};
const Segment SGI280433[] = {{&S43146[13], 9}, {&S43146[23], 22}, {&S43146[66], 17}, {NULL}};
const Segment SGI280434[] = {{&S40060[93], 4}, {NULL}};
const Segment SGI280435[] = {{&S40060[13], 9}, {&S40060[23], 22}, {&S40060[67], 17}, {NULL}};
const Segment SGI280436[] = {{&S43393[95], 4}, {NULL}};
const Segment SGI280437[] = {{&S43393[13], 14}, {&S43393[48], 17}, {NULL}};
const Segment SGI280438[] = {{&S43393[76], 4}, {NULL}};
const Segment SGI280439[] = {{&S43394[102], 4}, {NULL}};
const Segment SGI280440[] = {{&S43394[13], 14}, {&S43394[28], 6}, {&S43394[55], 17}, {NULL}};
const Segment SGI280441[] = {{&S43394[83], 4}, {NULL}};
const Segment SGI280442[] = {{&S43395[95], 4}, {NULL}};
const Segment SGI280443[] = {{&S43395[13], 14}, {&S43395[48], 17}, {NULL}};
const Segment SGI280444[] = {{&S43395[76], 4}, {NULL}};
const Segment SGI280445[] = {{&S43396[95], 4}, {NULL}};
const Segment SGI280446[] = {{&S43396[13], 14}, {&S43396[48], 17}, {&S43396[120], 8}, {NULL}};
const Segment SGI280447[] = {{&S43396[76], 4}, {NULL}};
const Segment SGI280448[] = {{&S43415[93], 4}, {NULL}};
const Segment SGI280449[] = {{&S43415[13], 3}, {&S43415[17], 13}, {&S43415[52], 17}, {NULL}};
const Segment SGI280450[] = {{&S43415[71], 12}, {NULL}};
const Segment SGI280451[] = {{&S43416[91], 4}, {NULL}};
const Segment SGI280452[] = {{&S43416[13], 3}, {&S43416[17], 11}, {&S43416[50], 17}, {NULL}};
const Segment SGI280453[] = {{&S43416[69], 12}, {NULL}};
const Segment SGI280454[] = {{&S43417[91], 4}, {NULL}};
const Segment SGI280455[] = {{&S43417[13], 3}, {&S43417[17], 11}, {&S43417[50], 17}, {&S43417[140], 8}, {NULL}};
const Segment SGI280456[] = {{&S43417[69], 12}, {NULL}};
const Segment SGI280457[] = {{&S40121[83], 4}, {NULL}};
const Segment SGI280458[] = {{&S40121[13], 3}, {&S40121[17], 19}, {&S40121[57], 17}, {NULL}};
const Segment SGI280459[] = {{&S40122[84], 4}, {NULL}};
const Segment SGI280460[] = {{&S40122[13], 3}, {&S40122[17], 20}, {&S40122[58], 17}, {NULL}};
const Segment SGI280461[] = {{&S40123[84], 4}, {NULL}};
const Segment SGI280462[] = {{&S40123[13], 3}, {&S40123[17], 20}, {&S40123[58], 17}, {NULL}};
const Segment SGI280463[] = {{&S40124[86], 4}, {NULL}};
const Segment SGI280464[] = {{&S40124[13], 3}, {&S40124[17], 22}, {&S40124[60], 17}, {NULL}};
const Segment SGI280465[] = {{&S40125[84], 4}, {NULL}};
const Segment SGI280466[] = {{&S40125[13], 3}, {&S40125[17], 20}, {&S40125[58], 17}, {NULL}};
const Segment SGI280467[] = {{&S40126[84], 4}, {NULL}};
const Segment SGI280468[] = {{&S40126[13], 3}, {&S40126[17], 20}, {&S40126[58], 17}, {NULL}};
const Segment SGI280469[] = {{&S40127[85], 4}, {NULL}};
const Segment SGI280470[] = {{&S40127[13], 3}, {&S40127[17], 21}, {&S40127[59], 17}, {NULL}};
const Segment SGI280471[] = {{&S40128[87], 4}, {NULL}};
const Segment SGI280472[] = {{&S40128[13], 3}, {&S40128[17], 23}, {&S40128[61], 17}, {NULL}};
const Segment SGI280473[] = {{&S40129[87], 4}, {NULL}};
const Segment SGI280474[] = {{&S40129[13], 3}, {&S40129[17], 23}, {&S40129[61], 17}, {NULL}};
const Segment SGI280475[] = {{&S40130[87], 4}, {NULL}};
const Segment SGI280476[] = {{&S40130[13], 3}, {&S40130[17], 23}, {&S40130[61], 17}, {NULL}};
const Segment SGI280477[] = {{&S40131[85], 4}, {NULL}};
const Segment SGI280478[] = {{&S40131[13], 3}, {&S40131[17], 21}, {&S40131[59], 17}, {NULL}};
const Segment SGI280479[] = {{&S40132[34], 4}, {NULL}};
const Segment SGI280480[] = {{&S40132[13], 3}, {&S40132[17], 21}, {&S40132[59], 17}, {NULL}};
const Segment SGI280481[] = {{&S40133[36], 4}, {NULL}};
const Segment SGI280482[] = {{&S40133[13], 3}, {&S40133[17], 23}, {&S40133[61], 17}, {NULL}};
const Segment SGI280483[] = {{&S40134[85], 4}, {NULL}};
const Segment SGI280484[] = {{&S40134[13], 3}, {&S40134[17], 21}, {&S40134[59], 17}, {NULL}};
const Segment SGI280485[] = {{&S40135[85], 4}, {NULL}};
const Segment SGI280486[] = {{&S40135[13], 3}, {&S40135[17], 21}, {&S40135[59], 17}, {NULL}};
const Segment SGI280487[] = {{&S40136[87], 4}, {NULL}};
const Segment SGI280488[] = {{&S40136[13], 3}, {&S40136[17], 23}, {&S40136[61], 17}, {NULL}};
const Segment SGI280489[] = {{&S40137[87], 4}, {NULL}};
const Segment SGI280490[] = {{&S40137[13], 3}, {&S40137[17], 23}, {&S40137[61], 17}, {NULL}};
const Segment SGI280491[] = {{&S40138[85], 4}, {NULL}};
const Segment SGI280492[] = {{&S40138[13], 3}, {&S40138[17], 21}, {&S40138[59], 17}, {NULL}};
const Segment SGI280493[] = {{&S40139[87], 4}, {NULL}};
const Segment SGI280494[] = {{&S40139[13], 3}, {&S40139[17], 23}, {&S40139[61], 17}, {NULL}};
const Segment SGI280495[] = {{&S40140[87], 4}, {NULL}};
const Segment SGI280496[] = {{&S40140[13], 3}, {&S40140[17], 23}, {&S40140[61], 17}, {NULL}};
const Segment SGI280497[] = {{&S40141[85], 4}, {NULL}};
const Segment SGI280498[] = {{&S40141[13], 3}, {&S40141[17], 21}, {&S40141[59], 17}, {NULL}};
const Segment SGI280499[] = {{&S43595[113], 4}, {NULL}};
const Segment SGI280500[] = {{&S43595[13], 9}, {&S43595[23], 22}, {&S43595[66], 17}, {NULL}};
const Segment SGI280501[] = {{&S43595[94], 4}, {NULL}};
const Segment SGI280502[] = {{&S40142[87], 4}, {NULL}};
const Segment SGI280503[] = {{&S40142[13], 3}, {&S40142[17], 23}, {&S40142[61], 17}, {NULL}};
const Segment SGI280504[] = {{&S43596[113], 4}, {NULL}};
const Segment SGI280505[] = {{&S43596[13], 9}, {&S43596[23], 22}, {&S43596[66], 17}, {NULL}};
const Segment SGI280506[] = {{&S43596[94], 4}, {NULL}};
const Segment SGI280507[] = {{&S40143[85], 4}, {NULL}};
const Segment SGI280508[] = {{&S40143[13], 3}, {&S40143[17], 21}, {&S40143[59], 17}, {NULL}};
const Segment SGI280509[] = {{&S43597[113], 4}, {NULL}};
const Segment SGI280510[] = {{&S43597[13], 9}, {&S43597[23], 22}, {&S43597[66], 17}, {NULL}};
const Segment SGI280511[] = {{&S43597[94], 4}, {NULL}};
const Segment SGI280512[] = {{&S40144[87], 4}, {NULL}};
const Segment SGI280513[] = {{&S40144[13], 3}, {&S40144[17], 23}, {&S40144[61], 17}, {NULL}};
const Segment SGI280514[] = {{&S43598[113], 4}, {NULL}};
const Segment SGI280515[] = {{&S43598[13], 9}, {&S43598[23], 22}, {&S43598[66], 17}, {NULL}};
const Segment SGI280516[] = {{&S43598[94], 4}, {NULL}};
const Segment SGI280517[] = {{&S40145[87], 4}, {NULL}};
const Segment SGI280518[] = {{&S40145[13], 3}, {&S40145[17], 23}, {&S40145[61], 17}, {NULL}};
const Segment SGI280519[] = {{&S43599[113], 4}, {NULL}};
const Segment SGI280520[] = {{&S43599[13], 9}, {&S43599[23], 22}, {&S43599[66], 17}, {NULL}};
const Segment SGI280521[] = {{&S43599[94], 4}, {NULL}};
const Segment SGI280522[] = {{&S40146[87], 4}, {NULL}};
const Segment SGI280523[] = {{&S40146[13], 3}, {&S40146[17], 23}, {&S40146[61], 17}, {NULL}};
const Segment SGI280524[] = {{&S43600[113], 4}, {NULL}};
const Segment SGI280525[] = {{&S43600[13], 9}, {&S43600[23], 22}, {&S43600[66], 17}, {NULL}};
const Segment SGI280526[] = {{&S43600[94], 4}, {NULL}};
const Segment SGI280527[] = {{&S40166[75], 4}, {NULL}};
const Segment SGI280528[] = {{&S40166[13], 14}, {&S40166[49], 17}, {NULL}};
const Segment SGI280529[] = {{&S40166[102], 21}, {NULL}};
const Segment SGI280530[] = {{&S40167[74], 4}, {NULL}};
const Segment SGI280531[] = {{&S40167[13], 14}, {&S40167[48], 17}, {NULL}};
const Segment SGI280532[] = {{&S40167[100], 21}, {NULL}};
const Segment SGI280533[] = {{&S40168[89], 4}, {NULL}};
const Segment SGI280534[] = {{&S40168[13], 14}, {&S40168[28], 14}, {&S40168[63], 17}, {NULL}};
const Segment SGI280535[] = {{&S40169[89], 4}, {NULL}};
const Segment SGI280536[] = {{&S40169[13], 14}, {&S40169[28], 14}, {&S40169[63], 17}, {NULL}};
const Segment SGI280537[] = {{&S40170[90], 4}, {NULL}};
const Segment SGI280538[] = {{&S40170[13], 14}, {&S40170[28], 14}, {&S40170[64], 17}, {NULL}};
const Segment SGI280539[] = {{&S40171[90], 4}, {NULL}};
const Segment SGI280540[] = {{&S40171[13], 14}, {&S40171[28], 14}, {&S40171[64], 17}, {NULL}};
const Segment SGI280541[] = {{&S40172[90], 4}, {NULL}};
const Segment SGI280542[] = {{&S40172[13], 14}, {&S40172[28], 14}, {&S40172[64], 17}, {NULL}};
const Segment SGI280543[] = {{&S40173[90], 4}, {NULL}};
const Segment SGI280544[] = {{&S40173[13], 14}, {&S40173[28], 14}, {&S40173[64], 17}, {NULL}};
const Segment SGI280545[] = {{&S40174[90], 4}, {NULL}};
const Segment SGI280546[] = {{&S40174[13], 14}, {&S40174[28], 14}, {&S40174[64], 17}, {NULL}};
const Segment SGI280547[] = {{&S40175[90], 4}, {NULL}};
const Segment SGI280548[] = {{&S40175[13], 14}, {&S40175[28], 14}, {&S40175[64], 17}, {NULL}};
const Segment SGI280549[] = {{&S40176[90], 4}, {NULL}};
const Segment SGI280550[] = {{&S40176[13], 14}, {&S40176[28], 14}, {&S40176[64], 17}, {NULL}};
const Segment SGI280551[] = {{&S43660[93], 4}, {NULL}};
const Segment SGI280552[] = {{&S43660[13], 14}, {&S43660[48], 17}, {NULL}};
const Segment SGI280553[] = {{&S43660[67], 11}, {NULL}};
const Segment SGI280554[] = {{&S43661[101], 4}, {NULL}};
const Segment SGI280555[] = {{&S43661[13], 14}, {&S43661[28], 6}, {&S43661[56], 17}, {NULL}};
const Segment SGI280556[] = {{&S43661[75], 11}, {NULL}};
const Segment SGI280557[] = {{&S43662[94], 4}, {NULL}};
const Segment SGI280558[] = {{&S43662[13], 14}, {&S43662[49], 17}, {NULL}};
const Segment SGI280559[] = {{&S43662[68], 11}, {NULL}};
const Segment SGI280560[] = {{&S43663[101], 4}, {NULL}};
const Segment SGI280561[] = {{&S43663[13], 14}, {&S43663[28], 6}, {&S43663[56], 17}, {NULL}};
const Segment SGI280562[] = {{&S43663[75], 11}, {NULL}};
const Segment SGI280563[] = {{&S43664[94], 4}, {NULL}};
const Segment SGI280564[] = {{&S43664[13], 14}, {&S43664[49], 17}, {NULL}};
const Segment SGI280565[] = {{&S43664[68], 11}, {NULL}};
const Segment SGI280566[] = {{&S43694[93], 4}, {NULL}};
const Segment SGI280567[] = {{&S43694[13], 3}, {&S43694[17], 13}, {&S43694[52], 17}, {NULL}};
const Segment SGI280568[] = {{&S43694[71], 12}, {NULL}};
const Segment SGI280569[] = {{&S40244[90], 4}, {NULL}};
const Segment SGI280570[] = {{&S40244[13], 3}, {&S40244[17], 11}, {&S40244[49], 17}, {NULL}};
const Segment SGI280571[] = {{&S40244[68], 12}, {NULL}};
const Segment SGI280572[] = {{&S40245[92], 4}, {NULL}};
const Segment SGI280573[] = {{&S40245[13], 3}, {&S40245[17], 13}, {&S40245[51], 17}, {NULL}};
const Segment SGI280574[] = {{&S40245[70], 12}, {NULL}};
const Segment SGI280575[] = {{&S40246[92], 4}, {NULL}};
const Segment SGI280576[] = {{&S40246[13], 3}, {&S40246[17], 13}, {&S40246[51], 17}, {NULL}};
const Segment SGI280577[] = {{&S40246[70], 12}, {NULL}};
const Segment SGI280578[] = {{&S40247[90], 4}, {NULL}};
const Segment SGI280579[] = {{&S40247[13], 3}, {&S40247[17], 11}, {&S40247[49], 17}, {&S40247[136], 8}, {NULL}};
const Segment SGI280580[] = {{&S40247[68], 12}, {NULL}};
const Segment SGI280581[] = {{&S43695[91], 4}, {NULL}};
const Segment SGI280582[] = {{&S43695[13], 3}, {&S43695[17], 11}, {&S43695[50], 17}, {NULL}};
const Segment SGI280583[] = {{&S43695[69], 12}, {NULL}};
const Segment SGI280584[] = {{&S60697[38], 5}, {NULL}};
const Segment SGI280585[] = {{&S60697[100], 5}, {NULL}};
const Segment SGI280586[] = {{&S43814[41], 4}, {NULL}};
const Segment SGI280587[] = {{&S43814[13], 14}, {&S43814[28], 26}, {&S43814[76], 17}, {NULL}};
const Segment SGI280588[] = {{&S43814[29], 16}, {NULL}};
const Segment SGI280589[] = {{&S43815[48], 4}, {NULL}};
const Segment SGI280590[] = {{&S43815[13], 14}, {&S43815[28], 6}, {&S43815[35], 26}, {&S43815[83], 17}, {NULL}};
const Segment SGI280591[] = {{&S43815[36], 16}, {NULL}};
const Segment SGI280592[] = {{&S43818[48], 4}, {NULL}};
const Segment SGI280593[] = {{&S43818[13], 14}, {&S43818[28], 6}, {&S43818[35], 26}, {&S43818[83], 17}, {NULL}};
const Segment SGI280594[] = {{&S43818[36], 16}, {NULL}};
const Segment SGI280595[] = {{&S43819[41], 4}, {NULL}};
const Segment SGI280596[] = {{&S43819[13], 14}, {&S43819[28], 26}, {&S43819[76], 17}, {NULL}};
const Segment SGI280597[] = {{&S43819[29], 16}, {NULL}};
const Segment SGI280598[] = {{&S43820[48], 4}, {NULL}};
const Segment SGI280599[] = {{&S43820[13], 14}, {&S43820[28], 6}, {&S43820[35], 26}, {&S43820[83], 17}, {NULL}};
const Segment SGI280600[] = {{&S43820[36], 16}, {NULL}};
const Segment SGI280601[] = {{&S43821[41], 4}, {NULL}};
const Segment SGI280602[] = {{&S43821[13], 14}, {&S43821[28], 26}, {&S43821[76], 17}, {NULL}};
const Segment SGI280603[] = {{&S43821[29], 16}, {NULL}};
const Segment SGI280604[] = {{&S43823[41], 4}, {NULL}};
const Segment SGI280605[] = {{&S43823[13], 14}, {&S43823[28], 26}, {&S43823[76], 17}, {NULL}};
const Segment SGI280606[] = {{&S43823[29], 16}, {NULL}};
const Segment SGI280607[] = {{&S43825[48], 4}, {NULL}};
const Segment SGI280608[] = {{&S43825[13], 14}, {&S43825[28], 6}, {&S43825[35], 26}, {&S43825[83], 17}, {NULL}};
const Segment SGI280609[] = {{&S43825[36], 16}, {NULL}};
const Segment SGI280610[] = {{&S43828[41], 4}, {NULL}};
const Segment SGI280611[] = {{&S43828[13], 14}, {&S43828[28], 26}, {&S43828[76], 17}, {NULL}};
const Segment SGI280612[] = {{&S43828[29], 16}, {NULL}};
const Segment SGI280613[] = {{&S43829[48], 4}, {NULL}};
const Segment SGI280614[] = {{&S43829[13], 14}, {&S43829[28], 6}, {&S43829[35], 26}, {&S43829[83], 17}, {NULL}};
const Segment SGI280615[] = {{&S43829[36], 16}, {NULL}};
const Segment SGI280616[] = {{&S43991[93], 4}, {NULL}};
const Segment SGI280617[] = {{&S43991[13], 9}, {&S43991[23], 22}, {&S43991[67], 17}, {NULL}};
const Segment SGI280618[] = {{&S43992[93], 4}, {NULL}};
const Segment SGI280619[] = {{&S43992[13], 9}, {&S43992[23], 22}, {&S43992[67], 17}, {NULL}};
const Segment SGI280620[] = {{&S43993[93], 4}, {NULL}};
const Segment SGI280621[] = {{&S43993[13], 9}, {&S43993[23], 22}, {&S43993[67], 17}, {NULL}};
const Segment SGI280622[] = {{&S43994[93], 4}, {NULL}};
const Segment SGI280623[] = {{&S43994[13], 9}, {&S43994[23], 22}, {&S43994[67], 17}, {NULL}};
const Segment SGI280624[] = {{&S43995[93], 4}, {NULL}};
const Segment SGI280625[] = {{&S43995[13], 9}, {&S43995[23], 22}, {&S43995[67], 17}, {NULL}};
const Segment SGI280626[] = {{&S43996[93], 4}, {NULL}};
const Segment SGI280627[] = {{&S43996[13], 9}, {&S43996[23], 22}, {&S43996[67], 17}, {NULL}};
const Segment SGI280628[] = {{&S44068[92], 4}, {NULL}};
const Segment SGI280629[] = {{&S44068[13], 9}, {&S44068[23], 22}, {&S44068[66], 17}, {NULL}};
const Segment SGI280630[] = {{&S44069[92], 4}, {NULL}};
const Segment SGI280631[] = {{&S44069[13], 9}, {&S44069[23], 22}, {&S44069[66], 17}, {NULL}};
const Segment SGI280632[] = {{&S44070[92], 4}, {NULL}};
const Segment SGI280633[] = {{&S44070[13], 9}, {&S44070[23], 22}, {&S44070[66], 17}, {NULL}};
const Segment SGI280634[] = {{&S44071[92], 4}, {NULL}};
const Segment SGI280635[] = {{&S44071[13], 9}, {&S44071[23], 22}, {&S44071[66], 17}, {NULL}};
const Segment SGI280636[] = {{&S44072[92], 4}, {NULL}};
const Segment SGI280637[] = {{&S44072[13], 9}, {&S44072[23], 22}, {&S44072[66], 17}, {NULL}};
const Segment SGI280638[] = {{&S44073[92], 4}, {NULL}};
const Segment SGI280639[] = {{&S44073[13], 9}, {&S44073[23], 22}, {&S44073[66], 17}, {NULL}};
const Segment SGI280640[] = {{&S40781[74], 4}, {NULL}};
const Segment SGI280641[] = {{&S40781[13], 14}, {&S40781[48], 17}, {NULL}};
const Segment SGI280642[] = {{&S40781[109], 4}, {NULL}};
const Segment SGI280643[] = {{&S44184[81], 4}, {NULL}};
const Segment SGI280644[] = {{&S44184[13], 14}, {&S44184[28], 6}, {&S44184[56], 17}, {NULL}};
const Segment SGI280645[] = {{&S44184[76], 9}, {NULL}};
const Segment SGI280646[] = {{&S44185[74], 4}, {NULL}};
const Segment SGI280647[] = {{&S44185[13], 14}, {&S44185[49], 17}, {NULL}};
const Segment SGI280648[] = {{&S44185[69], 9}, {NULL}};
const Segment SGI280649[] = {{&S40835[74], 4}, {NULL}};
const Segment SGI280650[] = {{&S40835[13], 14}, {&S40835[48], 17}, {NULL}};
const Segment SGI280651[] = {{&S40835[109], 4}, {NULL}};
const Segment SGI280652[] = {{&S44205[95], 4}, {NULL}};
const Segment SGI280653[] = {{&S44205[13], 14}, {&S44205[48], 17}, {NULL}};
const Segment SGI280654[] = {{&S44205[76], 4}, {NULL}};
const Segment SGI280655[] = {{&S44206[102], 4}, {NULL}};
const Segment SGI280656[] = {{&S44206[13], 14}, {&S44206[28], 6}, {&S44206[55], 17}, {NULL}};
const Segment SGI280657[] = {{&S44206[83], 4}, {NULL}};
const Segment SGI280658[] = {{&S40843[74], 4}, {NULL}};
const Segment SGI280659[] = {{&S40843[13], 14}, {&S40843[48], 17}, {&S40843[124], 8}, {NULL}};
const Segment SGI280660[] = {{&S40843[109], 4}, {NULL}};
const Segment SGI280661[] = {{&S49898[94], 3}, {NULL}};
const Segment SGI280662[] = {{&S49898[13], 7}, {&S49898[21], 2}, {&S49898[24], 15}, {&S49898[40], 6}, {&S49898[68], 17}, {NULL}};
const Segment SGI280663[] = {{&S44299[41], 4}, {NULL}};
const Segment SGI280664[] = {{&S44299[13], 14}, {&S44299[28], 25}, {&S44299[74], 17}, {NULL}};
const Segment SGI280665[] = {{&S44299[29], 16}, {NULL}};
const Segment SGI280666[] = {{&S49899[94], 3}, {NULL}};
const Segment SGI280667[] = {{&S49899[13], 7}, {&S49899[21], 2}, {&S49899[24], 15}, {&S49899[40], 6}, {&S49899[68], 17}, {&S49899[128], 10}, {NULL}};
const Segment SGI280668[] = {{&S44300[48], 4}, {NULL}};
const Segment SGI280669[] = {{&S44300[13], 14}, {&S44300[28], 6}, {&S44300[35], 25}, {&S44300[81], 17}, {NULL}};
const Segment SGI280670[] = {{&S44300[36], 16}, {NULL}};
const Segment SGI280671[] = {{&S49900[13], 7}, {&S49900[21], 2}, {&S49900[24], 15}, {&S49900[40], 6}, {&S49900[67], 17}, {&S49900[95], 21}, {NULL}};
const Segment SGI280672[] = {{&S44301[41], 4}, {NULL}};
const Segment SGI280673[] = {{&S44301[13], 14}, {&S44301[28], 25}, {&S44301[74], 17}, {NULL}};
const Segment SGI280674[] = {{&S44301[29], 16}, {NULL}};
const Segment SGI280675[] = {{&S44302[48], 4}, {NULL}};
const Segment SGI280676[] = {{&S44302[13], 14}, {&S44302[28], 6}, {&S44302[35], 25}, {&S44302[81], 17}, {NULL}};
const Segment SGI280677[] = {{&S44302[36], 16}, {NULL}};
const Segment SGI280678[] = {{&S44303[48], 4}, {NULL}};
const Segment SGI280679[] = {{&S44303[13], 14}, {&S44303[28], 6}, {&S44303[35], 25}, {&S44303[81], 17}, {NULL}};
const Segment SGI280680[] = {{&S44303[36], 16}, {NULL}};
const Segment SGI280681[] = {{&S44304[48], 4}, {NULL}};
const Segment SGI280682[] = {{&S44304[13], 14}, {&S44304[28], 6}, {&S44304[35], 25}, {&S44304[81], 17}, {NULL}};
const Segment SGI280683[] = {{&S44304[36], 16}, {NULL}};
const Segment SGI280684[] = {{&S49901[13], 14}, {&S49901[28], 6}, {&S49901[55], 17}, {&S49901[83], 21}, {NULL}};
const Segment SGI280685[] = {{&S44305[41], 4}, {NULL}};
const Segment SGI280686[] = {{&S44305[13], 14}, {&S44305[28], 25}, {&S44305[74], 17}, {NULL}};
const Segment SGI280687[] = {{&S44305[29], 16}, {NULL}};
const Segment SGI280688[] = {{&S44307[41], 4}, {NULL}};
const Segment SGI280689[] = {{&S44307[13], 14}, {&S44307[28], 25}, {&S44307[74], 17}, {NULL}};
const Segment SGI280690[] = {{&S44307[29], 16}, {NULL}};
const Segment SGI280691[] = {{&S44457[113], 4}, {NULL}};
const Segment SGI280692[] = {{&S44457[13], 9}, {&S44457[23], 22}, {&S44457[66], 17}, {NULL}};
const Segment SGI280693[] = {{&S44457[94], 4}, {NULL}};
const Segment SGI280694[] = {{&S44458[113], 4}, {NULL}};
const Segment SGI280695[] = {{&S44458[13], 9}, {&S44458[23], 22}, {&S44458[66], 17}, {NULL}};
const Segment SGI280696[] = {{&S44458[94], 4}, {NULL}};
const Segment SGI280697[] = {{&S44459[113], 4}, {NULL}};
const Segment SGI280698[] = {{&S44459[13], 9}, {&S44459[23], 22}, {&S44459[66], 17}, {NULL}};
const Segment SGI280699[] = {{&S44459[94], 4}, {NULL}};
const Segment SGI280700[] = {{&S44460[113], 4}, {NULL}};
const Segment SGI280701[] = {{&S44460[13], 9}, {&S44460[23], 22}, {&S44460[66], 17}, {NULL}};
const Segment SGI280702[] = {{&S44460[94], 4}, {NULL}};
const Segment SGI280703[] = {{&S44461[113], 4}, {NULL}};
const Segment SGI280704[] = {{&S44461[13], 9}, {&S44461[23], 22}, {&S44461[66], 17}, {NULL}};
const Segment SGI280705[] = {{&S44461[94], 4}, {NULL}};
const Segment SGI280706[] = {{&S44462[113], 4}, {NULL}};
const Segment SGI280707[] = {{&S44462[13], 9}, {&S44462[23], 22}, {&S44462[66], 17}, {NULL}};
const Segment SGI280708[] = {{&S44462[94], 4}, {NULL}};
const Segment SGI280709[] = {{&S44463[113], 4}, {NULL}};
const Segment SGI280710[] = {{&S44463[13], 9}, {&S44463[23], 22}, {&S44463[66], 17}, {NULL}};
const Segment SGI280711[] = {{&S44463[94], 4}, {NULL}};
const Segment SGI280712[] = {{&S44464[113], 4}, {NULL}};
const Segment SGI280713[] = {{&S44464[13], 9}, {&S44464[23], 22}, {&S44464[66], 17}, {NULL}};
const Segment SGI280714[] = {{&S44464[94], 4}, {NULL}};
const Segment SGI280715[] = {{&S44465[113], 4}, {NULL}};
const Segment SGI280716[] = {{&S44465[13], 9}, {&S44465[23], 22}, {&S44465[66], 17}, {NULL}};
const Segment SGI280717[] = {{&S44465[94], 4}, {NULL}};
const Segment SGI280718[] = {{&S44466[110], 4}, {NULL}};
const Segment SGI280719[] = {{&S44466[13], 9}, {&S44466[23], 20}, {&S44466[64], 16}, {NULL}};
const Segment SGI280720[] = {{&S44466[91], 4}, {NULL}};
const Segment SGI280721[] = {{&S41183[87], 4}, {NULL}};
const Segment SGI280722[] = {{&S41183[13], 3}, {&S41183[17], 13}, {&S41183[52], 17}, {NULL}};
const Segment SGI280723[] = {{&S41183[71], 15}, {NULL}};
const Segment SGI280724[] = {{&S41184[85], 4}, {NULL}};
const Segment SGI280725[] = {{&S41184[13], 3}, {&S41184[17], 11}, {&S41184[50], 17}, {NULL}};
const Segment SGI280726[] = {{&S41184[69], 15}, {NULL}};
const Segment SGI280727[] = {{&S41185[85], 4}, {NULL}};
const Segment SGI280728[] = {{&S41185[13], 3}, {&S41185[17], 11}, {&S41185[50], 17}, {&S41185[132], 8}, {NULL}};
const Segment SGI280729[] = {{&S41185[69], 15}, {NULL}};
const Segment SGI280730[] = {{&S41186[87], 4}, {NULL}};
const Segment SGI280731[] = {{&S41186[13], 3}, {&S41186[17], 13}, {&S41186[52], 17}, {&S41186[134], 8}, {NULL}};
const Segment SGI280732[] = {{&S41186[71], 15}, {NULL}};
const Segment SGI280733[] = {{&S44599[103], 4}, {NULL}};
const Segment SGI280734[] = {{&S44599[13], 9}, {&S44599[23], 2}, {&S44599[26], 22}, {&S44599[49], 6}, {&S44599[77], 17}, {NULL}};
const Segment SGI280735[] = {{&S44600[103], 4}, {NULL}};
const Segment SGI280736[] = {{&S44600[13], 9}, {&S44600[23], 2}, {&S44600[26], 22}, {&S44600[49], 6}, {&S44600[77], 17}, {NULL}};
const Segment SGI280737[] = {{&S44601[103], 4}, {NULL}};
const Segment SGI280738[] = {{&S44601[13], 9}, {&S44601[23], 2}, {&S44601[26], 22}, {&S44601[49], 6}, {&S44601[77], 17}, {NULL}};
const Segment SGI280739[] = {{&S44602[103], 4}, {NULL}};
const Segment SGI280740[] = {{&S44602[13], 9}, {&S44602[23], 2}, {&S44602[26], 22}, {&S44602[49], 6}, {&S44602[77], 17}, {NULL}};
const Segment SGI280741[] = {{&S44603[103], 4}, {NULL}};
const Segment SGI280742[] = {{&S44603[13], 9}, {&S44603[23], 2}, {&S44603[26], 22}, {&S44603[49], 6}, {&S44603[77], 17}, {NULL}};
const Segment SGI280743[] = {{&S44604[103], 4}, {NULL}};
const Segment SGI280744[] = {{&S44604[13], 9}, {&S44604[23], 2}, {&S44604[26], 22}, {&S44604[49], 6}, {&S44604[77], 17}, {NULL}};
const Segment SGI280745[] = {{&S44605[103], 4}, {NULL}};
const Segment SGI280746[] = {{&S44605[13], 9}, {&S44605[23], 2}, {&S44605[26], 22}, {&S44605[49], 6}, {&S44605[77], 17}, {NULL}};
const Segment SGI280747[] = {{&S44606[103], 4}, {NULL}};
const Segment SGI280748[] = {{&S44606[13], 9}, {&S44606[23], 2}, {&S44606[26], 22}, {&S44606[49], 6}, {&S44606[77], 17}, {NULL}};
const Segment SGI280749[] = {{&S50295[94], 3}, {NULL}};
const Segment SGI280750[] = {{&S50295[13], 7}, {&S50295[21], 2}, {&S50295[24], 15}, {&S50295[40], 6}, {&S50295[68], 17}, {NULL}};
const Segment SGI280751[] = {{&S50296[94], 3}, {NULL}};
const Segment SGI280752[] = {{&S50296[13], 7}, {&S50296[21], 2}, {&S50296[24], 15}, {&S50296[40], 6}, {&S50296[68], 17}, {NULL}};
const Segment SGI280753[] = {{&S50297[94], 3}, {NULL}};
const Segment SGI280754[] = {{&S50297[13], 7}, {&S50297[21], 2}, {&S50297[24], 15}, {&S50297[40], 6}, {&S50297[68], 17}, {NULL}};
const Segment SGI280755[] = {{&S50298[91], 3}, {NULL}};
const Segment SGI280756[] = {{&S41327[74], 4}, {NULL}};
const Segment SGI280757[] = {{&S41327[13], 14}, {&S41327[48], 17}, {NULL}};
const Segment SGI280758[] = {{&S41327[109], 4}, {NULL}};
const Segment SGI280759[] = {{&S41330[74], 4}, {NULL}};
const Segment SGI280760[] = {{&S41330[13], 14}, {&S41330[48], 17}, {NULL}};
const Segment SGI280761[] = {{&S41330[109], 4}, {NULL}};
const Segment SGI280762[] = {{&S41332[74], 4}, {NULL}};
const Segment SGI280763[] = {{&S41332[13], 14}, {&S41332[48], 17}, {&S41332[124], 8}, {NULL}};
const Segment SGI280764[] = {{&S41332[109], 4}, {NULL}};
const Segment SGI280765[] = {{&S44749[93], 4}, {NULL}};
const Segment SGI280766[] = {{&S44749[13], 9}, {&S44749[23], 22}, {&S44749[67], 17}, {NULL}};
const Segment SGI280767[] = {{&S44750[93], 4}, {NULL}};
const Segment SGI280768[] = {{&S44750[13], 9}, {&S44750[23], 22}, {&S44750[67], 17}, {NULL}};
const Segment SGI280769[] = {{&S44751[93], 4}, {NULL}};
const Segment SGI280770[] = {{&S44751[13], 9}, {&S44751[23], 22}, {&S44751[67], 17}, {NULL}};
const Segment SGI280771[] = {{&S44752[103], 4}, {NULL}};
const Segment SGI280772[] = {{&S44752[13], 9}, {&S44752[23], 2}, {&S44752[26], 22}, {&S44752[49], 6}, {&S44752[77], 17}, {NULL}};
const Segment SGI280773[] = {{&S44753[93], 4}, {NULL}};
const Segment SGI280774[] = {{&S44753[13], 9}, {&S44753[23], 22}, {&S44753[67], 17}, {NULL}};
const Segment SGI280775[] = {{&S44754[93], 4}, {NULL}};
const Segment SGI280776[] = {{&S44754[13], 9}, {&S44754[23], 22}, {&S44754[67], 17}, {NULL}};
const Segment SGI280777[] = {{&S44755[93], 4}, {NULL}};
const Segment SGI280778[] = {{&S44755[13], 9}, {&S44755[23], 22}, {&S44755[67], 17}, {NULL}};
const Segment SGI280779[] = {{&S44756[103], 4}, {NULL}};
const Segment SGI280780[] = {{&S44756[13], 9}, {&S44756[23], 2}, {&S44756[26], 20}, {&S44756[47], 6}, {&S44756[77], 17}, {NULL}};
const Segment SGI280781[] = {{&S50298[13], 7}, {&S50298[21], 2}, {&S50298[24], 15}, {&S50298[40], 3}, {&S50298[65], 17}, {NULL}};
const Segment SGI280782[] = {{&S50299[75], 3}, {NULL}};
const Segment SGI280783[] = {{&S50299[13], 14}, {&S50299[49], 17}, {NULL}};
const Segment SGI280784[] = {{&S50300[91], 3}, {NULL}};
const Segment SGI280785[] = {{&S50300[13], 7}, {&S50300[21], 2}, {&S50300[24], 15}, {&S50300[40], 3}, {&S50300[65], 17}, {NULL}};
const Segment SGI280786[] = {{&S50301[93], 3}, {NULL}};
const Segment SGI280787[] = {{&S50301[13], 7}, {&S50301[21], 2}, {&S50301[24], 15}, {&S50301[40], 6}, {&S50301[67], 17}, {NULL}};
const Segment SGI280788[] = {{&S44983[80], 4}, {NULL}};
const Segment SGI280789[] = {{&S44983[13], 14}, {&S44983[28], 6}, {&S44983[56], 17}, {NULL}};
const Segment SGI280790[] = {{&S44983[75], 9}, {NULL}};
const Segment SGI280791[] = {{&S44984[73], 4}, {NULL}};
const Segment SGI280792[] = {{&S44984[13], 14}, {&S44984[49], 17}, {NULL}};
const Segment SGI280793[] = {{&S44984[68], 9}, {NULL}};
const Segment SGI280794[] = {{&S45045[48], 4}, {NULL}};
const Segment SGI280795[] = {{&S45045[13], 14}, {&S45045[28], 6}, {&S45045[35], 24}, {&S45045[81], 17}, {NULL}};
const Segment SGI280796[] = {{&S45045[36], 16}, {NULL}};
const Segment SGI280797[] = {{&S45046[41], 4}, {NULL}};
const Segment SGI280798[] = {{&S45046[13], 14}, {&S45046[28], 24}, {&S45046[74], 17}, {NULL}};
const Segment SGI280799[] = {{&S45046[29], 16}, {NULL}};
const Segment SGI280800[] = {{&S45047[48], 4}, {NULL}};
const Segment SGI280801[] = {{&S45047[13], 14}, {&S45047[28], 6}, {&S45047[35], 24}, {&S45047[81], 17}, {NULL}};
const Segment SGI280802[] = {{&S45047[36], 16}, {NULL}};
const Segment SGI280803[] = {{&S45048[48], 4}, {NULL}};
const Segment SGI280804[] = {{&S45048[13], 14}, {&S45048[28], 6}, {&S45048[35], 24}, {&S45048[81], 17}, {NULL}};
const Segment SGI280805[] = {{&S45048[36], 16}, {NULL}};
const Segment SGI280806[] = {{&S45049[41], 4}, {NULL}};
const Segment SGI280807[] = {{&S45049[13], 14}, {&S45049[28], 24}, {&S45049[74], 17}, {NULL}};
const Segment SGI280808[] = {{&S45049[29], 16}, {NULL}};
const Segment SGI280809[] = {{&S45050[48], 4}, {NULL}};
const Segment SGI280810[] = {{&S45050[13], 14}, {&S45050[28], 6}, {&S45050[35], 24}, {&S45050[81], 17}, {NULL}};
const Segment SGI280811[] = {{&S45050[36], 16}, {NULL}};
const Segment SGI280812[] = {{&S45051[41], 4}, {NULL}};
const Segment SGI280813[] = {{&S45051[13], 14}, {&S45051[28], 24}, {&S45051[74], 17}, {NULL}};
const Segment SGI280814[] = {{&S45051[29], 16}, {NULL}};
const Segment SGI280815[] = {{&S45123[47], 4}, {NULL}};
const Segment SGI280816[] = {{&S45123[13], 14}, {&S45123[28], 10}, {&S45123[85], 17}, {NULL}};
const Segment SGI280817[] = {{&S41934[103], 4}, {NULL}};
const Segment SGI280818[] = {{&S41934[13], 9}, {&S41934[23], 2}, {&S41934[26], 22}, {&S41934[49], 6}, {&S41934[77], 17}, {NULL}};
const Segment SGI280819[] = {{&S41935[92], 4}, {NULL}};
const Segment SGI280820[] = {{&S41935[13], 9}, {&S41935[23], 22}, {&S41935[66], 17}, {NULL}};
const Segment SGI280821[] = {{&S41936[93], 4}, {NULL}};
const Segment SGI280822[] = {{&S41936[13], 9}, {&S41936[23], 22}, {&S41936[67], 17}, {NULL}};
const Segment SGI280823[] = {{&S41937[93], 4}, {NULL}};
const Segment SGI280824[] = {{&S41937[13], 9}, {&S41937[23], 22}, {&S41937[67], 17}, {NULL}};
const Segment SGI280825[] = {{&S41938[103], 4}, {NULL}};
const Segment SGI280826[] = {{&S41938[13], 9}, {&S41938[23], 2}, {&S41938[26], 22}, {&S41938[49], 6}, {&S41938[77], 17}, {NULL}};
const Segment SGI280827[] = {{&S42014[75], 4}, {NULL}};
const Segment SGI280828[] = {{&S42014[13], 14}, {&S42014[49], 17}, {&S42014[121], 12}, {NULL}};
const Segment SGI280829[] = {{&S42014[102], 8}, {NULL}};
const Segment SGI280830[] = {{&S42015[82], 4}, {NULL}};
const Segment SGI280831[] = {{&S42015[13], 14}, {&S42015[28], 6}, {&S42015[56], 17}, {&S42015[128], 12}, {NULL}};
const Segment SGI280832[] = {{&S42015[109], 8}, {NULL}};
const Segment SGI280833[] = {{&S42016[75], 4}, {NULL}};
const Segment SGI280834[] = {{&S42016[13], 14}, {&S42016[49], 17}, {NULL}};
const Segment SGI280835[] = {{&S42016[102], 8}, {NULL}};
const Segment SGI280836[] = {{&S42017[82], 4}, {NULL}};
const Segment SGI280837[] = {{&S42017[13], 14}, {&S42017[28], 6}, {&S42017[56], 17}, {NULL}};
const Segment SGI280838[] = {{&S42017[109], 8}, {NULL}};
const Segment SGI280839[] = {{&S42018[75], 4}, {NULL}};
const Segment SGI280840[] = {{&S42018[13], 14}, {&S42018[49], 17}, {&S42018[121], 19}, {NULL}};
const Segment SGI280841[] = {{&S42018[102], 8}, {NULL}};
const Segment SGI280842[] = {{&S42019[75], 4}, {NULL}};
const Segment SGI280843[] = {{&S42019[13], 14}, {&S42019[49], 17}, {&S42019[121], 15}, {NULL}};
const Segment SGI280844[] = {{&S42019[102], 8}, {NULL}};
const Segment SGI280845[] = {{&S42020[82], 4}, {NULL}};
const Segment SGI280846[] = {{&S42020[13], 14}, {&S42020[28], 6}, {&S42020[56], 17}, {&S42020[128], 19}, {NULL}};
const Segment SGI280847[] = {{&S42020[109], 8}, {NULL}};
const Segment SGI280848[] = {{&S45425[41], 4}, {NULL}};
const Segment SGI280849[] = {{&S45425[13], 14}, {&S45425[28], 24}, {&S45425[74], 17}, {NULL}};
const Segment SGI280850[] = {{&S45425[29], 16}, {NULL}};
const Segment SGI280851[] = {{&S45428[41], 4}, {NULL}};
const Segment SGI280852[] = {{&S45428[13], 14}, {&S45428[28], 24}, {&S45428[74], 17}, {NULL}};
const Segment SGI280853[] = {{&S45428[29], 16}, {NULL}};
const Segment SGI280854[] = {{&S45545[90], 4}, {NULL}};
const Segment SGI280855[] = {{&S45545[13], 3}, {&S45545[17], 11}, {&S45545[49], 17}, {NULL}};
const Segment SGI280856[] = {{&S45545[68], 12}, {NULL}};
const Segment SGI280857[] = {{&S45546[92], 4}, {NULL}};
const Segment SGI280858[] = {{&S45546[13], 3}, {&S45546[17], 13}, {&S45546[51], 17}, {NULL}};
const Segment SGI280859[] = {{&S45546[70], 12}, {NULL}};
const Segment SGI280860[] = {{&S45547[90], 4}, {NULL}};
const Segment SGI280861[] = {{&S45547[13], 3}, {&S45547[17], 11}, {&S45547[49], 17}, {&S45547[136], 8}, {&S45547[147], 24}, {NULL}};
const Segment SGI280862[] = {{&S45547[68], 12}, {NULL}};
const Segment SGI280863[] = {{&S62631[38], 5}, {NULL}};
const Segment SGI280864[] = {{&S62631[97], 5}, {NULL}};
const Segment SGI280865[] = {{&S45596[92], 4}, {NULL}};
const Segment SGI280866[] = {{&S45596[13], 3}, {&S45596[17], 13}, {&S45596[51], 17}, {NULL}};
const Segment SGI280867[] = {{&S45596[70], 12}, {NULL}};
const Segment SGI280868[] = {{&S45597[90], 4}, {NULL}};
const Segment SGI280869[] = {{&S45597[13], 3}, {&S45597[17], 11}, {&S45597[49], 17}, {NULL}};
const Segment SGI280870[] = {{&S45597[68], 12}, {NULL}};
const Segment SGI280871[] = {{&S42602[82], 4}, {NULL}};
const Segment SGI280872[] = {{&S42602[13], 14}, {&S42602[28], 6}, {&S42602[56], 17}, {NULL}};
const Segment SGI280873[] = {{&S42602[94], 9}, {NULL}};
const Segment SGI280874[] = {{&S42603[75], 4}, {NULL}};
const Segment SGI280875[] = {{&S42603[13], 14}, {&S42603[49], 17}, {NULL}};
const Segment SGI280876[] = {{&S42603[87], 9}, {NULL}};
const Segment SGI280877[] = {{&S51653[103], 3}, {NULL}};
const Segment SGI280878[] = {{&S51653[13], 9}, {&S51653[23], 2}, {&S51653[26], 22}, {&S51653[49], 6}, {&S51653[77], 17}, {NULL}};
const Segment SGI280879[] = {{&S51654[103], 3}, {NULL}};
const Segment SGI280880[] = {{&S51654[13], 9}, {&S51654[23], 2}, {&S51654[26], 22}, {&S51654[49], 6}, {&S51654[77], 17}, {NULL}};
const Segment SGI280881[] = {{&S51655[103], 3}, {NULL}};
const Segment SGI280882[] = {{&S51655[13], 9}, {&S51655[23], 2}, {&S51655[26], 22}, {&S51655[49], 6}, {&S51655[77], 17}, {NULL}};
const Segment SGI280883[] = {{&S51656[103], 3}, {NULL}};
const Segment SGI280884[] = {{&S51656[13], 9}, {&S51656[23], 2}, {&S51656[26], 22}, {&S51656[49], 6}, {&S51656[77], 17}, {NULL}};
const Segment SGI280885[] = {{&S51657[103], 3}, {NULL}};
const Segment SGI280886[] = {{&S51657[13], 9}, {&S51657[23], 2}, {&S51657[26], 22}, {&S51657[49], 6}, {&S51657[77], 17}, {NULL}};
const Segment SGI280887[] = {{&S51658[103], 3}, {NULL}};
const Segment SGI280888[] = {{&S51658[13], 9}, {&S51658[23], 2}, {&S51658[26], 22}, {&S51658[49], 6}, {&S51658[77], 17}, {NULL}};
const Segment SGI280889[] = {{&S51659[103], 3}, {NULL}};
const Segment SGI280890[] = {{&S51659[13], 9}, {&S51659[23], 2}, {&S51659[26], 22}, {&S51659[49], 6}, {&S51659[77], 17}, {NULL}};
const Segment SGI280891[] = {{&S51660[103], 3}, {NULL}};
const Segment SGI280892[] = {{&S51660[13], 9}, {&S51660[23], 2}, {&S51660[26], 22}, {&S51660[49], 6}, {&S51660[77], 17}, {NULL}};
const Segment SGI280893[] = {{&S45846[102], 4}, {NULL}};
const Segment SGI280894[] = {{&S45846[13], 14}, {&S45846[28], 6}, {&S45846[55], 17}, {NULL}};
const Segment SGI280895[] = {{&S45846[83], 4}, {NULL}};
const Segment SGI280896[] = {{&S45847[95], 4}, {NULL}};
const Segment SGI280897[] = {{&S45847[13], 14}, {&S45847[48], 17}, {NULL}};
const Segment SGI280898[] = {{&S45847[76], 4}, {NULL}};
const Segment SGI280899[] = {{&S46132[91], 4}, {NULL}};
const Segment SGI280900[] = {{&S46132[13], 3}, {&S46132[17], 11}, {&S46132[50], 17}, {NULL}};
const Segment SGI280901[] = {{&S46132[69], 12}, {NULL}};
const Segment SGI280902[] = {{&S46133[93], 4}, {NULL}};
const Segment SGI280903[] = {{&S46133[13], 3}, {&S46133[17], 13}, {&S46133[52], 17}, {NULL}};
const Segment SGI280904[] = {{&S46133[71], 12}, {NULL}};
const Segment SGI280905[] = {{&S43498[74], 4}, {NULL}};
const Segment SGI280906[] = {{&S43498[13], 14}, {&S43498[48], 17}, {NULL}};
const Segment SGI280907[] = {{&S43498[86], 9}, {NULL}};
const Segment SGI280908[] = {{&S43499[81], 4}, {NULL}};
const Segment SGI280909[] = {{&S43499[13], 14}, {&S43499[28], 6}, {&S43499[55], 17}, {NULL}};
const Segment SGI280910[] = {{&S43499[93], 9}, {NULL}};
const Segment SGI280911[] = {{&S46415[78], 4}, {NULL}};
const Segment SGI280912[] = {{&S46415[13], 3}, {&S46415[17], 14}, {&S46415[52], 17}, {NULL}};
const Segment SGI280913[] = {{&S46416[96], 4}, {NULL}};
const Segment SGI280914[] = {{&S46416[13], 7}, {&S46416[21], 2}, {&S46416[24], 6}, {&S46416[31], 6}, {&S46416[58], 17}, {NULL}};
const Segment SGI280915[] = {{&S46417[96], 4}, {NULL}};
const Segment SGI280916[] = {{&S46417[13], 7}, {&S46417[21], 2}, {&S46417[24], 6}, {&S46417[31], 6}, {&S46417[58], 17}, {NULL}};
const Segment SGI280917[] = {{&S46418[96], 4}, {NULL}};
const Segment SGI280918[] = {{&S46418[13], 7}, {&S46418[21], 2}, {&S46418[24], 6}, {&S46418[31], 6}, {&S46418[58], 17}, {NULL}};
const Segment SGI280919[] = {{&S46419[96], 4}, {NULL}};
const Segment SGI280920[] = {{&S46419[13], 7}, {&S46419[21], 2}, {&S46419[24], 6}, {&S46419[31], 6}, {&S46419[58], 17}, {NULL}};
const Segment SGI280921[] = {{&S46420[77], 4}, {NULL}};
const Segment SGI280922[] = {{&S46420[13], 3}, {&S46420[17], 13}, {&S46420[51], 17}, {NULL}};
const Segment SGI280923[] = {{&S46421[96], 4}, {NULL}};
const Segment SGI280924[] = {{&S46421[13], 7}, {&S46421[21], 2}, {&S46421[24], 6}, {&S46421[31], 6}, {&S46421[58], 17}, {NULL}};
const Segment SGI280925[] = {{&S46422[96], 4}, {NULL}};
const Segment SGI280926[] = {{&S46422[13], 7}, {&S46422[21], 2}, {&S46422[24], 6}, {&S46422[31], 6}, {&S46422[58], 17}, {NULL}};
const Segment SGI280927[] = {{&S46423[96], 4}, {NULL}};
const Segment SGI280928[] = {{&S46423[13], 7}, {&S46423[21], 2}, {&S46423[24], 6}, {&S46423[31], 6}, {&S46423[58], 17}, {NULL}};
const Segment SGI280929[] = {{&S46424[96], 4}, {NULL}};
const Segment SGI280930[] = {{&S46424[13], 7}, {&S46424[21], 2}, {&S46424[24], 6}, {&S46424[31], 6}, {&S46424[58], 17}, {NULL}};
const Segment SGI280931[] = {{&S46425[96], 4}, {NULL}};
const Segment SGI280932[] = {{&S46425[13], 7}, {&S46425[21], 2}, {&S46425[24], 6}, {&S46425[31], 6}, {&S46425[58], 17}, {NULL}};
const Segment SGI280933[] = {{&S46426[96], 4}, {NULL}};
const Segment SGI280934[] = {{&S46426[13], 7}, {&S46426[21], 2}, {&S46426[24], 6}, {&S46426[31], 6}, {&S46426[58], 17}, {NULL}};
const Segment SGI280935[] = {{&S46427[96], 4}, {NULL}};
const Segment SGI280936[] = {{&S46427[13], 7}, {&S46427[21], 2}, {&S46427[24], 6}, {&S46427[31], 6}, {&S46427[58], 17}, {NULL}};
const Segment SGI280937[] = {{&S46428[96], 4}, {NULL}};
const Segment SGI280938[] = {{&S46428[13], 7}, {&S46428[21], 2}, {&S46428[24], 6}, {&S46428[31], 6}, {&S46428[58], 17}, {NULL}};
const Segment SGI280939[] = {{&S46429[96], 4}, {NULL}};
const Segment SGI280940[] = {{&S46429[13], 7}, {&S46429[21], 2}, {&S46429[24], 6}, {&S46429[31], 6}, {&S46429[58], 17}, {NULL}};
const Segment SGI280941[] = {{&S46430[96], 4}, {NULL}};
const Segment SGI280942[] = {{&S46430[13], 7}, {&S46430[21], 2}, {&S46430[24], 6}, {&S46430[31], 6}, {&S46430[58], 17}, {NULL}};
const Segment SGI280943[] = {{&S46431[96], 4}, {NULL}};
const Segment SGI280944[] = {{&S46431[13], 7}, {&S46431[21], 2}, {&S46431[24], 6}, {&S46431[31], 6}, {&S46431[58], 17}, {NULL}};
const Segment SGI280945[] = {{&S46432[96], 4}, {NULL}};
const Segment SGI280946[] = {{&S46432[13], 7}, {&S46432[21], 2}, {&S46432[24], 6}, {&S46432[31], 6}, {&S46432[58], 17}, {NULL}};
const Segment SGI280947[] = {{&S46433[96], 4}, {NULL}};
const Segment SGI280948[] = {{&S46433[13], 7}, {&S46433[21], 2}, {&S46433[24], 6}, {&S46433[31], 6}, {&S46433[58], 17}, {NULL}};
const Segment SGI280949[] = {{&S46434[96], 4}, {NULL}};
const Segment SGI280950[] = {{&S46434[13], 7}, {&S46434[21], 2}, {&S46434[24], 6}, {&S46434[31], 6}, {&S46434[58], 17}, {NULL}};
const Segment SGI280951[] = {{&S46435[96], 4}, {NULL}};
const Segment SGI280952[] = {{&S46435[13], 7}, {&S46435[21], 2}, {&S46435[24], 6}, {&S46435[31], 6}, {&S46435[58], 17}, {NULL}};
const Segment SGI280953[] = {{&S46436[96], 4}, {NULL}};
const Segment SGI280954[] = {{&S46436[13], 7}, {&S46436[21], 2}, {&S46436[24], 6}, {&S46436[31], 6}, {&S46436[58], 17}, {NULL}};
const Segment SGI280955[] = {{&S46437[96], 4}, {NULL}};
const Segment SGI280956[] = {{&S46437[13], 7}, {&S46437[21], 2}, {&S46437[24], 6}, {&S46437[31], 6}, {&S46437[58], 17}, {NULL}};
const Segment SGI280957[] = {{&S46438[96], 4}, {NULL}};
const Segment SGI280958[] = {{&S46438[13], 7}, {&S46438[21], 2}, {&S46438[24], 6}, {&S46438[31], 6}, {&S46438[58], 17}, {NULL}};
const Segment SGI280959[] = {{&S46439[96], 4}, {NULL}};
const Segment SGI280960[] = {{&S46439[13], 7}, {&S46439[21], 2}, {&S46439[24], 6}, {&S46439[31], 6}, {&S46439[58], 17}, {NULL}};
const Segment SGI280961[] = {{&S46440[96], 4}, {NULL}};
const Segment SGI280962[] = {{&S46440[13], 7}, {&S46440[21], 2}, {&S46440[24], 6}, {&S46440[31], 6}, {&S46440[58], 17}, {NULL}};
const Segment SGI280963[] = {{&S46441[96], 4}, {NULL}};
const Segment SGI280964[] = {{&S46441[13], 7}, {&S46441[21], 2}, {&S46441[24], 6}, {&S46441[31], 6}, {&S46441[58], 17}, {NULL}};
const Segment SGI280965[] = {{&S46442[96], 4}, {NULL}};
const Segment SGI280966[] = {{&S46442[13], 7}, {&S46442[21], 2}, {&S46442[24], 6}, {&S46442[31], 6}, {&S46442[58], 17}, {NULL}};
const Segment SGI280967[] = {{&S46443[96], 4}, {NULL}};
const Segment SGI280968[] = {{&S46443[13], 7}, {&S46443[21], 2}, {&S46443[24], 6}, {&S46443[31], 6}, {&S46443[58], 17}, {NULL}};
const Segment SGI280969[] = {{&S46444[96], 4}, {NULL}};
const Segment SGI280970[] = {{&S46444[13], 7}, {&S46444[21], 2}, {&S46444[24], 6}, {&S46444[31], 6}, {&S46444[58], 17}, {NULL}};
const Segment SGI280971[] = {{&S46445[96], 4}, {NULL}};
const Segment SGI280972[] = {{&S46445[13], 7}, {&S46445[21], 2}, {&S46445[24], 6}, {&S46445[31], 6}, {&S46445[58], 17}, {NULL}};
const Segment SGI280973[] = {{&S46446[96], 4}, {NULL}};
const Segment SGI280974[] = {{&S46446[13], 7}, {&S46446[21], 2}, {&S46446[24], 6}, {&S46446[31], 6}, {&S46446[58], 17}, {NULL}};
const Segment SGI280975[] = {{&S46447[96], 4}, {NULL}};
const Segment SGI280976[] = {{&S46447[13], 7}, {&S46447[21], 2}, {&S46447[24], 6}, {&S46447[31], 6}, {&S46447[58], 17}, {NULL}};
const Segment SGI280977[] = {{&S46448[96], 4}, {NULL}};
const Segment SGI280978[] = {{&S46448[13], 7}, {&S46448[21], 2}, {&S46448[24], 6}, {&S46448[31], 6}, {&S46448[58], 17}, {NULL}};
const Segment SGI280979[] = {{&S46449[96], 4}, {NULL}};
const Segment SGI280980[] = {{&S46449[13], 7}, {&S46449[21], 2}, {&S46449[24], 6}, {&S46449[31], 6}, {&S46449[58], 17}, {NULL}};
const Segment SGI280981[] = {{&S46450[101], 4}, {NULL}};
const Segment SGI280982[] = {{&S46450[13], 3}, {&S46450[17], 8}, {&S46450[26], 2}, {&S46450[29], 6}, {&S46450[36], 6}, {&S46450[63], 17}, {NULL}};
const Segment SGI280983[] = {{&S46451[101], 4}, {NULL}};
const Segment SGI280984[] = {{&S46451[13], 3}, {&S46451[17], 8}, {&S46451[26], 2}, {&S46451[29], 6}, {&S46451[36], 6}, {&S46451[63], 17}, {NULL}};
const Segment SGI280985[] = {{&S46452[101], 4}, {NULL}};
const Segment SGI280986[] = {{&S46452[13], 3}, {&S46452[17], 8}, {&S46452[26], 2}, {&S46452[29], 6}, {&S46452[36], 6}, {&S46452[63], 17}, {NULL}};
const Segment SGI280987[] = {{&S46453[101], 4}, {NULL}};
const Segment SGI280988[] = {{&S46453[13], 3}, {&S46453[17], 8}, {&S46453[26], 2}, {&S46453[29], 6}, {&S46453[36], 6}, {&S46453[63], 17}, {NULL}};
const Segment SGI280989[] = {{&S46454[101], 4}, {NULL}};
const Segment SGI280990[] = {{&S46454[13], 3}, {&S46454[17], 8}, {&S46454[26], 2}, {&S46454[29], 6}, {&S46454[36], 6}, {&S46454[63], 17}, {NULL}};
const Segment SGI280991[] = {{&S46455[101], 4}, {NULL}};
const Segment SGI280992[] = {{&S46455[13], 3}, {&S46455[17], 8}, {&S46455[26], 2}, {&S46455[29], 6}, {&S46455[36], 6}, {&S46455[63], 17}, {NULL}};
const Segment SGI280993[] = {{&S46456[101], 4}, {NULL}};
const Segment SGI280994[] = {{&S46456[13], 3}, {&S46456[17], 8}, {&S46456[26], 2}, {&S46456[29], 6}, {&S46456[36], 6}, {&S46456[63], 17}, {NULL}};
const Segment SGI280995[] = {{&S46457[101], 4}, {NULL}};
const Segment SGI280996[] = {{&S46457[13], 3}, {&S46457[17], 8}, {&S46457[26], 2}, {&S46457[29], 6}, {&S46457[36], 6}, {&S46457[63], 17}, {NULL}};
const Segment SGI280997[] = {{&S46458[101], 4}, {NULL}};
const Segment SGI280998[] = {{&S46458[13], 3}, {&S46458[17], 8}, {&S46458[26], 2}, {&S46458[29], 6}, {&S46458[36], 6}, {&S46458[63], 17}, {NULL}};
const Segment SGI280999[] = {{&S46459[101], 4}, {NULL}};
const Segment SGI281000[] = {{&S46459[13], 3}, {&S46459[17], 8}, {&S46459[26], 2}, {&S46459[29], 6}, {&S46459[36], 6}, {&S46459[63], 17}, {NULL}};
const Segment SGI281001[] = {{&S46460[101], 4}, {NULL}};
const Segment SGI281002[] = {{&S46460[13], 3}, {&S46460[17], 8}, {&S46460[26], 2}, {&S46460[29], 6}, {&S46460[36], 6}, {&S46460[63], 17}, {NULL}};
const Segment SGI281003[] = {{&S46461[101], 4}, {NULL}};
const Segment SGI281004[] = {{&S46461[13], 3}, {&S46461[17], 8}, {&S46461[26], 2}, {&S46461[29], 6}, {&S46461[36], 6}, {&S46461[63], 17}, {NULL}};
const Segment SGI281005[] = {{&S46462[101], 4}, {NULL}};
const Segment SGI281006[] = {{&S46462[13], 3}, {&S46462[17], 8}, {&S46462[26], 2}, {&S46462[29], 6}, {&S46462[36], 6}, {&S46462[63], 17}, {NULL}};
const Segment SGI281007[] = {{&S46463[101], 4}, {NULL}};
const Segment SGI281008[] = {{&S46463[13], 3}, {&S46463[17], 8}, {&S46463[26], 2}, {&S46463[29], 6}, {&S46463[36], 6}, {&S46463[63], 17}, {NULL}};
const Segment SGI281009[] = {{&S46464[101], 4}, {NULL}};
const Segment SGI281010[] = {{&S46464[13], 3}, {&S46464[17], 8}, {&S46464[26], 2}, {&S46464[29], 6}, {&S46464[36], 6}, {&S46464[63], 17}, {NULL}};
const Segment SGI281011[] = {{&S46465[101], 4}, {NULL}};
const Segment SGI281012[] = {{&S46465[13], 3}, {&S46465[17], 8}, {&S46465[26], 2}, {&S46465[29], 6}, {&S46465[36], 6}, {&S46465[63], 17}, {NULL}};
const Segment SGI281013[] = {{&S46466[101], 4}, {NULL}};
const Segment SGI281014[] = {{&S46466[13], 3}, {&S46466[17], 8}, {&S46466[26], 2}, {&S46466[29], 6}, {&S46466[36], 6}, {&S46466[63], 17}, {NULL}};
const Segment SGI281015[] = {{&S46467[101], 4}, {NULL}};
const Segment SGI281016[] = {{&S46467[13], 3}, {&S46467[17], 8}, {&S46467[26], 2}, {&S46467[29], 6}, {&S46467[36], 6}, {&S46467[63], 17}, {NULL}};
const Segment SGI281017[] = {{&S52217[102], 3}, {NULL}};
const Segment SGI281018[] = {{&S52217[13], 9}, {&S52217[23], 2}, {&S52217[26], 22}, {&S52217[49], 6}, {&S52217[76], 17}, {NULL}};
const Segment SGI281019[] = {{&S52218[102], 3}, {NULL}};
const Segment SGI281020[] = {{&S52218[13], 9}, {&S52218[23], 2}, {&S52218[26], 22}, {&S52218[49], 6}, {&S52218[76], 17}, {NULL}};
const Segment SGI281021[] = {{&S52219[102], 3}, {NULL}};
const Segment SGI281022[] = {{&S52219[13], 9}, {&S52219[23], 2}, {&S52219[26], 22}, {&S52219[49], 6}, {&S52219[76], 17}, {NULL}};
const Segment SGI281023[] = {{&S52220[102], 3}, {NULL}};
const Segment SGI281024[] = {{&S52220[13], 9}, {&S52220[23], 2}, {&S52220[26], 22}, {&S52220[49], 6}, {&S52220[76], 17}, {NULL}};
const Segment SGI281025[] = {{&S52221[102], 3}, {NULL}};
const Segment SGI281026[] = {{&S52221[13], 9}, {&S52221[23], 2}, {&S52221[26], 22}, {&S52221[49], 6}, {&S52221[76], 17}, {NULL}};
const Segment SGI281027[] = {{&S52222[102], 3}, {NULL}};
const Segment SGI281028[] = {{&S52222[13], 9}, {&S52222[23], 2}, {&S52222[26], 22}, {&S52222[49], 6}, {&S52222[76], 17}, {NULL}};
const Segment SGI281029[] = {{&S52223[102], 3}, {NULL}};
const Segment SGI281030[] = {{&S52223[13], 9}, {&S52223[23], 2}, {&S52223[26], 22}, {&S52223[49], 6}, {&S52223[76], 17}, {NULL}};
const Segment SGI281031[] = {{&S52224[102], 3}, {NULL}};
const Segment SGI281032[] = {{&S52224[13], 9}, {&S52224[23], 2}, {&S52224[26], 22}, {&S52224[49], 6}, {&S52224[76], 17}, {NULL}};
const Segment SGI281033[] = {{&S52225[102], 3}, {NULL}};
const Segment SGI281034[] = {{&S52225[13], 9}, {&S52225[23], 2}, {&S52225[26], 22}, {&S52225[49], 6}, {&S52225[76], 17}, {NULL}};
const Segment SGI281035[] = {{&S46668[92], 4}, {NULL}};
const Segment SGI281036[] = {{&S46668[13], 3}, {&S46668[17], 13}, {&S46668[51], 17}, {NULL}};
const Segment SGI281037[] = {{&S46668[70], 12}, {NULL}};
const Segment SGI281038[] = {{&S46669[90], 4}, {NULL}};
const Segment SGI281039[] = {{&S46669[13], 3}, {&S46669[17], 11}, {&S46669[49], 17}, {NULL}};
const Segment SGI281040[] = {{&S46669[68], 12}, {NULL}};
const Segment SGI281041[] = {{&S46670[90], 4}, {NULL}};
const Segment SGI281042[] = {{&S46670[13], 3}, {&S46670[17], 11}, {&S46670[49], 17}, {NULL}};
const Segment SGI281043[] = {{&S46670[68], 12}, {NULL}};
const Segment SGI281044[] = {{&S46706[75], 4}, {NULL}};
const Segment SGI281045[] = {{&S46706[13], 14}, {&S46706[49], 17}, {NULL}};
const Segment SGI281046[] = {{&S46706[103], 21}, {NULL}};
const Segment SGI281047[] = {{&S46712[74], 4}, {NULL}};
const Segment SGI281048[] = {{&S46712[13], 14}, {&S46712[48], 17}, {NULL}};
const Segment SGI281049[] = {{&S46712[100], 21}, {NULL}};
const Segment SGI281050[] = {{&S43850[81], 4}, {NULL}};
const Segment SGI281051[] = {{&S43850[13], 14}, {&S43850[28], 6}, {&S43850[55], 17}, {NULL}};
const Segment SGI281052[] = {{&S43850[93], 9}, {NULL}};
const Segment SGI281053[] = {{&S43851[74], 4}, {NULL}};
const Segment SGI281054[] = {{&S43851[13], 14}, {&S43851[48], 17}, {NULL}};
const Segment SGI281055[] = {{&S43851[86], 9}, {NULL}};
const Segment SGI281056[] = {{&S46913[72], 4}, {NULL}};
const Segment SGI281057[] = {{&S46913[13], 14}, {&S46913[48], 17}, {NULL}};
const Segment SGI281058[] = {{&S46913[67], 9}, {NULL}};
const Segment SGI281059[] = {{&S46914[79], 4}, {NULL}};
const Segment SGI281060[] = {{&S46914[13], 14}, {&S46914[28], 6}, {&S46914[55], 17}, {NULL}};
const Segment SGI281061[] = {{&S46914[74], 9}, {NULL}};
const Segment SGI281062[] = {{&S44186[87], 4}, {NULL}};
const Segment SGI281063[] = {{&S44186[13], 3}, {&S44186[17], 13}, {&S44186[52], 17}, {NULL}};
const Segment SGI281064[] = {{&S44186[71], 15}, {NULL}};
const Segment SGI281065[] = {{&S44187[85], 4}, {NULL}};
const Segment SGI281066[] = {{&S44187[13], 3}, {&S44187[17], 11}, {&S44187[50], 17}, {NULL}};
const Segment SGI281067[] = {{&S44187[69], 15}, {NULL}};
const Segment SGI281068[] = {{&S44188[85], 4}, {NULL}};
const Segment SGI281069[] = {{&S44188[13], 3}, {&S44188[17], 11}, {&S44188[50], 17}, {&S44188[132], 8}, {NULL}};
const Segment SGI281070[] = {{&S44188[69], 15}, {NULL}};
const Segment SGI281071[] = {{&S47213[75], 4}, {NULL}};
const Segment SGI281072[] = {{&S47213[13], 14}, {&S47213[49], 17}, {NULL}};
const Segment SGI281073[] = {{&S47213[111], 4}, {NULL}};
const Segment SGI281074[] = {{&S47219[75], 4}, {NULL}};
const Segment SGI281075[] = {{&S47219[13], 14}, {&S47219[49], 17}, {&S47219[126], 8}, {&S47219[137], 24}, {NULL}};
const Segment SGI281076[] = {{&S47219[111], 4}, {NULL}};
const Segment SGI281077[] = {{&S44255[111], 4}, {NULL}};
const Segment SGI281078[] = {{&S44255[13], 9}, {&S44255[23], 22}, {&S44255[66], 17}, {NULL}};
const Segment SGI281079[] = {{&S44255[85], 11}, {NULL}};
const Segment SGI281080[] = {{&S44256[111], 4}, {NULL}};
const Segment SGI281081[] = {{&S44256[13], 9}, {&S44256[23], 22}, {&S44256[66], 17}, {NULL}};
const Segment SGI281082[] = {{&S44256[85], 11}, {NULL}};
const Segment SGI281083[] = {{&S44257[112], 4}, {NULL}};
const Segment SGI281084[] = {{&S44257[13], 9}, {&S44257[23], 22}, {&S44257[67], 17}, {NULL}};
const Segment SGI281085[] = {{&S44257[86], 11}, {NULL}};
const Segment SGI281086[] = {{&S44258[112], 4}, {NULL}};
const Segment SGI281087[] = {{&S44258[13], 9}, {&S44258[23], 22}, {&S44258[67], 17}, {NULL}};
const Segment SGI281088[] = {{&S44258[86], 11}, {NULL}};
const Segment SGI281089[] = {{&S44259[112], 4}, {NULL}};
const Segment SGI281090[] = {{&S44259[13], 9}, {&S44259[23], 22}, {&S44259[67], 17}, {NULL}};
const Segment SGI281091[] = {{&S44259[86], 11}, {NULL}};
const Segment SGI281092[] = {{&S44260[112], 4}, {NULL}};
const Segment SGI281093[] = {{&S44260[13], 9}, {&S44260[23], 22}, {&S44260[67], 17}, {NULL}};
const Segment SGI281094[] = {{&S44260[86], 11}, {NULL}};
const Segment SGI281095[] = {{&S44261[112], 4}, {NULL}};
const Segment SGI281096[] = {{&S44261[13], 9}, {&S44261[23], 22}, {&S44261[67], 17}, {NULL}};
const Segment SGI281097[] = {{&S44261[86], 11}, {NULL}};
const Segment SGI281098[] = {{&S44493[48], 4}, {NULL}};
const Segment SGI281099[] = {{&S44493[13], 14}, {&S44493[28], 6}, {&S44493[35], 25}, {&S44493[82], 17}, {NULL}};
const Segment SGI281100[] = {{&S44494[48], 4}, {NULL}};
const Segment SGI281101[] = {{&S44494[13], 14}, {&S44494[28], 6}, {&S44494[35], 25}, {&S44494[82], 17}, {NULL}};
const Segment SGI281102[] = {{&S44498[48], 4}, {NULL}};
const Segment SGI281103[] = {{&S44498[13], 14}, {&S44498[28], 6}, {&S44498[35], 25}, {&S44498[82], 17}, {NULL}};
const Segment SGI281104[] = {{&S44512[48], 4}, {NULL}};
const Segment SGI281105[] = {{&S44512[13], 14}, {&S44512[28], 6}, {&S44512[35], 25}, {&S44512[82], 17}, {NULL}};
const Segment SGI281106[] = {{&S44513[48], 4}, {NULL}};
const Segment SGI281107[] = {{&S44513[13], 14}, {&S44513[28], 6}, {&S44513[35], 25}, {&S44513[82], 17}, {NULL}};
const Segment SGI281108[] = {{&S44514[41], 4}, {NULL}};
const Segment SGI281109[] = {{&S44514[13], 14}, {&S44514[28], 25}, {&S44514[75], 17}, {NULL}};
const Segment SGI281110[] = {{&S44515[48], 4}, {NULL}};
const Segment SGI281111[] = {{&S44515[13], 14}, {&S44515[28], 6}, {&S44515[35], 25}, {&S44515[82], 17}, {NULL}};
const Segment SGI281112[] = {{&S44516[41], 4}, {NULL}};
const Segment SGI281113[] = {{&S44516[13], 14}, {&S44516[28], 25}, {&S44516[75], 17}, {NULL}};
const Segment SGI281114[] = {{&S44520[48], 4}, {NULL}};
const Segment SGI281115[] = {{&S44520[13], 14}, {&S44520[28], 6}, {&S44520[35], 25}, {&S44520[82], 17}, {NULL}};
const Segment SGI281116[] = {{&S44521[41], 4}, {NULL}};
const Segment SGI281117[] = {{&S44521[13], 14}, {&S44521[28], 25}, {&S44521[75], 17}, {NULL}};
const Segment SGI281118[] = {{&S53049[93], 3}, {NULL}};
const Segment SGI281119[] = {{&S53049[13], 7}, {&S53049[21], 2}, {&S53049[24], 15}, {&S53049[40], 6}, {&S53049[67], 17}, {NULL}};
const Segment SGI281120[] = {{&S44642[73], 4}, {NULL}};
const Segment SGI281121[] = {{&S44642[13], 14}, {&S44642[49], 17}, {NULL}};
const Segment SGI281122[] = {{&S44642[68], 9}, {NULL}};
const Segment SGI281123[] = {{&S44643[80], 4}, {NULL}};
const Segment SGI281124[] = {{&S44643[13], 14}, {&S44643[28], 6}, {&S44643[56], 17}, {NULL}};
const Segment SGI281125[] = {{&S44643[75], 9}, {NULL}};
const Segment SGI281126[] = {{&S47414[41], 4}, {NULL}};
const Segment SGI281127[] = {{&S47414[13], 14}, {&S47414[28], 24}, {&S47414[74], 17}, {NULL}};
const Segment SGI281128[] = {{&S47414[29], 16}, {NULL}};
const Segment SGI281129[] = {{&S47415[41], 4}, {NULL}};
const Segment SGI281130[] = {{&S47415[13], 14}, {&S47415[28], 24}, {&S47415[74], 17}, {NULL}};
const Segment SGI281131[] = {{&S47415[29], 16}, {NULL}};
const Segment SGI281132[] = {{&S47417[41], 4}, {NULL}};
const Segment SGI281133[] = {{&S47417[13], 14}, {&S47417[28], 24}, {&S47417[74], 17}, {NULL}};
const Segment SGI281134[] = {{&S47417[29], 16}, {NULL}};
const Segment SGI281135[] = {{&S47419[48], 4}, {NULL}};
const Segment SGI281136[] = {{&S47419[13], 14}, {&S47419[28], 6}, {&S47419[35], 24}, {&S47419[81], 17}, {NULL}};
const Segment SGI281137[] = {{&S47419[36], 16}, {NULL}};
const Segment SGI281138[] = {{&S47422[41], 4}, {NULL}};
const Segment SGI281139[] = {{&S47422[13], 14}, {&S47422[28], 24}, {&S47422[74], 17}, {NULL}};
const Segment SGI281140[] = {{&S47422[29], 16}, {NULL}};
const Segment SGI281141[] = {{&S47423[41], 4}, {NULL}};
const Segment SGI281142[] = {{&S47423[13], 14}, {&S47423[28], 24}, {&S47423[74], 17}, {NULL}};
const Segment SGI281143[] = {{&S47423[29], 16}, {NULL}};
const Segment SGI281144[] = {{&S47424[41], 4}, {NULL}};
const Segment SGI281145[] = {{&S47424[13], 14}, {&S47424[28], 24}, {&S47424[74], 17}, {NULL}};
const Segment SGI281146[] = {{&S47424[29], 16}, {NULL}};
const Segment SGI281147[] = {{&S44748[93], 4}, {NULL}};
const Segment SGI281148[] = {{&S44748[13], 9}, {&S44748[23], 22}, {&S44748[67], 17}, {NULL}};
const Segment SGI281149[] = {{&S44846[48], 4}, {NULL}};
const Segment SGI281150[] = {{&S44846[13], 14}, {&S44846[28], 6}, {&S44846[35], 25}, {&S44846[82], 17}, {NULL}};
const Segment SGI281151[] = {{&S44847[48], 4}, {NULL}};
const Segment SGI281152[] = {{&S44847[13], 14}, {&S44847[28], 6}, {&S44847[35], 26}, {&S44847[83], 17}, {NULL}};
const Segment SGI281153[] = {{&S47602[84], 4}, {NULL}};
const Segment SGI281154[] = {{&S47602[13], 3}, {&S47602[17], 19}, {&S47602[58], 17}, {NULL}};
const Segment SGI281155[] = {{&S44849[48], 4}, {NULL}};
const Segment SGI281156[] = {{&S47603[84], 4}, {NULL}};
const Segment SGI281157[] = {{&S47603[13], 3}, {&S47603[17], 19}, {&S47603[58], 17}, {NULL}};
const Segment SGI281158[] = {{&S44849[13], 14}, {&S44849[28], 6}, {&S44849[35], 26}, {&S44849[83], 17}, {NULL}};
const Segment SGI281159[] = {{&S47604[85], 4}, {NULL}};
const Segment SGI281160[] = {{&S47604[13], 3}, {&S47604[17], 21}, {&S47604[59], 17}, {NULL}};
const Segment SGI281161[] = {{&S47605[85], 4}, {NULL}};
const Segment SGI281162[] = {{&S47605[13], 3}, {&S47605[17], 21}, {&S47605[59], 17}, {NULL}};
const Segment SGI281163[] = {{&S47606[83], 4}, {NULL}};
const Segment SGI281164[] = {{&S47606[13], 3}, {&S47606[17], 19}, {&S47606[57], 17}, {NULL}};
const Segment SGI281165[] = {{&S47607[85], 4}, {NULL}};
const Segment SGI281166[] = {{&S47607[13], 3}, {&S47607[17], 21}, {&S47607[59], 17}, {NULL}};
const Segment SGI281167[] = {{&S47608[84], 4}, {NULL}};
const Segment SGI281168[] = {{&S47608[13], 3}, {&S47608[17], 20}, {&S47608[58], 17}, {NULL}};
const Segment SGI281169[] = {{&S47609[85], 4}, {NULL}};
const Segment SGI281170[] = {{&S47609[13], 3}, {&S47609[17], 21}, {&S47609[59], 17}, {NULL}};
const Segment SGI281171[] = {{&S47610[84], 4}, {NULL}};
const Segment SGI281172[] = {{&S47610[13], 3}, {&S47610[17], 19}, {&S47610[58], 17}, {NULL}};
const Segment SGI281173[] = {{&S47611[85], 4}, {NULL}};
const Segment SGI281174[] = {{&S47611[13], 3}, {&S47611[17], 21}, {&S47611[59], 17}, {NULL}};
const Segment SGI281175[] = {{&S47612[84], 4}, {NULL}};
const Segment SGI281176[] = {{&S47612[13], 3}, {&S47612[17], 20}, {&S47612[58], 17}, {NULL}};
const Segment SGI281177[] = {{&S47613[84], 4}, {NULL}};
const Segment SGI281178[] = {{&S47613[13], 3}, {&S47613[17], 20}, {&S47613[58], 17}, {NULL}};
const Segment SGI281179[] = {{&S47614[84], 4}, {NULL}};
const Segment SGI281180[] = {{&S47614[13], 3}, {&S47614[17], 20}, {&S47614[58], 17}, {NULL}};
const Segment SGI281181[] = {{&S47615[84], 4}, {NULL}};
const Segment SGI281182[] = {{&S47615[13], 3}, {&S47615[17], 20}, {&S47615[58], 17}, {NULL}};
const Segment SGI281183[] = {{&S47616[84], 4}, {NULL}};
const Segment SGI281184[] = {{&S47616[13], 3}, {&S47616[17], 20}, {&S47616[58], 17}, {NULL}};
const Segment SGI281185[] = {{&S47617[83], 4}, {NULL}};
const Segment SGI281186[] = {{&S47617[13], 3}, {&S47617[17], 19}, {&S47617[57], 17}, {NULL}};
const Segment SGI281187[] = {{&S47618[84], 4}, {NULL}};
const Segment SGI281188[] = {{&S47618[13], 3}, {&S47618[17], 20}, {&S47618[58], 17}, {NULL}};
const Segment SGI281189[] = {{&S47619[84], 4}, {NULL}};
const Segment SGI281190[] = {{&S47619[13], 3}, {&S47619[17], 20}, {&S47619[58], 17}, {NULL}};
const Segment SGI281191[] = {{&S47810[92], 4}, {NULL}};
const Segment SGI281192[] = {{&S47810[13], 9}, {&S47810[23], 22}, {&S47810[66], 17}, {NULL}};
const Segment SGI281193[] = {{&S47811[92], 4}, {NULL}};
const Segment SGI281194[] = {{&S47811[13], 9}, {&S47811[23], 22}, {&S47811[66], 17}, {NULL}};
const Segment SGI281195[] = {{&S47812[92], 4}, {NULL}};
const Segment SGI281196[] = {{&S47812[13], 9}, {&S47812[23], 22}, {&S47812[66], 17}, {NULL}};
const Segment SGI281197[] = {{&S47813[92], 4}, {NULL}};
const Segment SGI281198[] = {{&S47813[13], 9}, {&S47813[23], 22}, {&S47813[66], 17}, {NULL}};
const Segment SGI281199[] = {{&S47814[92], 4}, {NULL}};
const Segment SGI281200[] = {{&S47814[13], 9}, {&S47814[23], 22}, {&S47814[66], 17}, {NULL}};
const Segment SGI281201[] = {{&S47815[92], 4}, {NULL}};
const Segment SGI281202[] = {{&S47815[13], 9}, {&S47815[23], 22}, {&S47815[66], 17}, {NULL}};
const Segment SGI281203[] = {{&S45073[75], 4}, {NULL}};
const Segment SGI281204[] = {{&S45073[13], 14}, {&S45073[49], 17}, {NULL}};
const Segment SGI281205[] = {{&S45073[87], 9}, {NULL}};
const Segment SGI281206[] = {{&S45074[82], 4}, {NULL}};
const Segment SGI281207[] = {{&S45074[13], 14}, {&S45074[28], 6}, {&S45074[56], 17}, {NULL}};
const Segment SGI281208[] = {{&S45074[94], 9}, {NULL}};
const Segment SGI281209[] = {{&S45111[93], 4}, {NULL}};
const Segment SGI281210[] = {{&S45111[13], 9}, {&S45111[23], 22}, {&S45111[67], 17}, {&S45111[138], 12}, {NULL}};
const Segment SGI281211[] = {{&S45111[120], 8}, {NULL}};
const Segment SGI281212[] = {{&S45112[93], 4}, {NULL}};
const Segment SGI281213[] = {{&S45112[13], 9}, {&S45112[23], 22}, {&S45112[67], 17}, {&S45112[139], 12}, {NULL}};
const Segment SGI281214[] = {{&S45112[120], 8}, {NULL}};
const Segment SGI281215[] = {{&S45113[93], 4}, {NULL}};
const Segment SGI281216[] = {{&S45113[13], 9}, {&S45113[23], 22}, {&S45113[67], 17}, {&S45113[139], 12}, {NULL}};
const Segment SGI281217[] = {{&S45113[120], 8}, {NULL}};
const Segment SGI281218[] = {{&S45114[93], 4}, {NULL}};
const Segment SGI281219[] = {{&S45114[13], 9}, {&S45114[23], 22}, {&S45114[67], 17}, {NULL}};
const Segment SGI281220[] = {{&S45114[120], 8}, {NULL}};
const Segment SGI281221[] = {{&S45115[93], 4}, {NULL}};
const Segment SGI281222[] = {{&S45115[13], 9}, {&S45115[23], 22}, {&S45115[67], 17}, {NULL}};
const Segment SGI281223[] = {{&S45115[120], 8}, {NULL}};
const Segment SGI281224[] = {{&S45116[93], 4}, {NULL}};
const Segment SGI281225[] = {{&S45116[13], 9}, {&S45116[23], 22}, {&S45116[67], 17}, {NULL}};
const Segment SGI281226[] = {{&S45116[120], 8}, {NULL}};
const Segment SGI281227[] = {{&S45117[93], 4}, {NULL}};
const Segment SGI281228[] = {{&S45117[13], 9}, {&S45117[23], 22}, {&S45117[67], 17}, {NULL}};
const Segment SGI281229[] = {{&S45117[120], 8}, {NULL}};
const Segment SGI281230[] = {{&S45118[93], 4}, {NULL}};
const Segment SGI281231[] = {{&S45118[13], 9}, {&S45118[23], 22}, {&S45118[67], 17}, {NULL}};
const Segment SGI281232[] = {{&S45118[120], 8}, {NULL}};
const Segment SGI281233[] = {{&S45487[82], 4}, {NULL}};
const Segment SGI281234[] = {{&S45487[13], 14}, {&S45487[28], 6}, {&S45487[56], 17}, {NULL}};
const Segment SGI281235[] = {{&S45487[95], 9}, {NULL}};
const Segment SGI281236[] = {{&S45488[75], 4}, {NULL}};
const Segment SGI281237[] = {{&S45488[13], 14}, {&S45488[49], 17}, {NULL}};
const Segment SGI281238[] = {{&S45488[88], 9}, {NULL}};
const Segment SGI281239[] = {{&S53966[110], 3}, {NULL}};
const Segment SGI281240[] = {{&S53966[13], 7}, {&S53966[21], 2}, {&S53966[24], 15}, {&S53966[40], 6}, {&S53966[67], 17}, {NULL}};
const Segment SGI281241[] = {{&S53966[86], 9}, {NULL}};
const Segment SGI281242[] = {{&S48112[41], 4}, {NULL}};
const Segment SGI281243[] = {{&S48112[13], 14}, {&S48112[28], 25}, {&S48112[74], 17}, {NULL}};
const Segment SGI281244[] = {{&S48112[29], 16}, {NULL}};
const Segment SGI281245[] = {{&S48115[41], 4}, {NULL}};
const Segment SGI281246[] = {{&S48115[13], 14}, {&S48115[28], 25}, {&S48115[74], 17}, {NULL}};
const Segment SGI281247[] = {{&S48115[29], 16}, {NULL}};
const Segment SGI281248[] = {{&S48117[41], 4}, {NULL}};
const Segment SGI281249[] = {{&S48117[13], 14}, {&S48117[28], 25}, {&S48117[74], 17}, {NULL}};
const Segment SGI281250[] = {{&S48117[29], 16}, {NULL}};
const Segment SGI281251[] = {{&S48119[41], 4}, {NULL}};
const Segment SGI281252[] = {{&S48119[13], 14}, {&S48119[28], 25}, {&S48119[74], 17}, {NULL}};
const Segment SGI281253[] = {{&S48119[29], 16}, {NULL}};
const Segment SGI281254[] = {{&S48120[41], 4}, {NULL}};
const Segment SGI281255[] = {{&S48120[13], 14}, {&S48120[28], 25}, {&S48120[74], 17}, {NULL}};
const Segment SGI281256[] = {{&S48120[29], 16}, {NULL}};
const Segment SGI281257[] = {{&S48121[41], 4}, {NULL}};
const Segment SGI281258[] = {{&S48121[13], 14}, {&S48121[28], 24}, {&S48121[73], 17}, {NULL}};
const Segment SGI281259[] = {{&S48121[29], 16}, {NULL}};
const Segment SGI281260[] = {{&S48123[41], 4}, {NULL}};
const Segment SGI281261[] = {{&S48123[13], 14}, {&S48123[28], 24}, {&S48123[73], 17}, {NULL}};
const Segment SGI281262[] = {{&S48123[29], 16}, {NULL}};
const Segment SGI281263[] = {{&S48125[41], 4}, {NULL}};
const Segment SGI281264[] = {{&S48125[13], 14}, {&S48125[28], 23}, {&S48125[72], 17}, {NULL}};
const Segment SGI281265[] = {{&S48125[29], 16}, {NULL}};
const Segment SGI281266[] = {{&S48168[86], 4}, {NULL}};
const Segment SGI281267[] = {{&S48168[13], 3}, {&S48168[17], 21}, {&S48168[60], 17}, {NULL}};
const Segment SGI281268[] = {{&S48169[86], 4}, {NULL}};
const Segment SGI281269[] = {{&S48169[13], 3}, {&S48169[17], 21}, {&S48169[60], 17}, {NULL}};
const Segment SGI281270[] = {{&S48170[86], 4}, {NULL}};
const Segment SGI281271[] = {{&S48170[13], 3}, {&S48170[17], 21}, {&S48170[60], 17}, {NULL}};
const Segment SGI281272[] = {{&S48171[86], 4}, {NULL}};
const Segment SGI281273[] = {{&S48171[13], 3}, {&S48171[17], 21}, {&S48171[60], 17}, {NULL}};
const Segment SGI281274[] = {{&S48172[28], 4}, {NULL}};
const Segment SGI281275[] = {{&S48172[13], 3}, {&S48172[17], 22}, {&S48172[61], 17}, {NULL}};
const Segment SGI281276[] = {{&S48173[86], 4}, {NULL}};
const Segment SGI281277[] = {{&S48173[13], 3}, {&S48173[17], 21}, {&S48173[60], 17}, {NULL}};
const Segment SGI281278[] = {{&S48174[84], 4}, {NULL}};
const Segment SGI281279[] = {{&S48174[13], 3}, {&S48174[17], 19}, {&S48174[58], 17}, {NULL}};
const Segment SGI281280[] = {{&S48175[81], 4}, {NULL}};
const Segment SGI281281[] = {{&S48175[13], 3}, {&S48175[17], 16}, {&S48175[55], 17}, {NULL}};
const Segment SGI281282[] = {{&S45744[74], 4}, {NULL}};
const Segment SGI281283[] = {{&S45744[13], 14}, {&S45744[48], 17}, {NULL}};
const Segment SGI281284[] = {{&S45744[100], 21}, {NULL}};
const Segment SGI281285[] = {{&S45745[81], 4}, {NULL}};
const Segment SGI281286[] = {{&S45745[13], 14}, {&S45745[28], 6}, {&S45745[55], 17}, {NULL}};
const Segment SGI281287[] = {{&S45745[107], 21}, {NULL}};
const Segment SGI281288[] = {{&S45746[96], 4}, {NULL}};
const Segment SGI281289[] = {{&S45746[13], 14}, {&S45746[28], 6}, {&S45746[35], 14}, {&S45746[70], 17}, {NULL}};
const Segment SGI281290[] = {{&S45747[89], 4}, {NULL}};
const Segment SGI281291[] = {{&S45747[13], 14}, {&S45747[28], 14}, {&S45747[63], 17}, {NULL}};
const Segment SGI281292[] = {{&S45748[97], 4}, {NULL}};
const Segment SGI281293[] = {{&S45748[13], 14}, {&S45748[28], 6}, {&S45748[35], 14}, {&S45748[71], 17}, {NULL}};
const Segment SGI281294[] = {{&S45749[90], 4}, {NULL}};
const Segment SGI281295[] = {{&S45749[13], 14}, {&S45749[28], 14}, {&S45749[64], 17}, {NULL}};
const Segment SGI281296[] = {{&S45750[96], 4}, {NULL}};
const Segment SGI281297[] = {{&S45750[13], 14}, {&S45750[28], 6}, {&S45750[35], 14}, {&S45750[70], 17}, {NULL}};
const Segment SGI281298[] = {{&S45751[97], 4}, {NULL}};
const Segment SGI281299[] = {{&S45751[13], 14}, {&S45751[28], 6}, {&S45751[35], 14}, {&S45751[71], 17}, {NULL}};
const Segment SGI281300[] = {{&S45752[97], 4}, {NULL}};
const Segment SGI281301[] = {{&S45752[13], 14}, {&S45752[28], 6}, {&S45752[35], 14}, {&S45752[71], 17}, {NULL}};
const Segment SGI281302[] = {{&S45753[90], 4}, {NULL}};
const Segment SGI281303[] = {{&S45753[13], 14}, {&S45753[28], 14}, {&S45753[64], 17}, {NULL}};
const Segment SGI281304[] = {{&S45754[90], 4}, {NULL}};
const Segment SGI281305[] = {{&S45754[13], 14}, {&S45754[28], 14}, {&S45754[64], 17}, {NULL}};
const Segment SGI281306[] = {{&S45755[97], 4}, {NULL}};
const Segment SGI281307[] = {{&S45755[13], 14}, {&S45755[28], 6}, {&S45755[35], 14}, {&S45755[71], 17}, {NULL}};
const Segment SGI281308[] = {{&S45756[97], 4}, {NULL}};
const Segment SGI281309[] = {{&S45756[13], 14}, {&S45756[28], 6}, {&S45756[35], 14}, {&S45756[71], 17}, {NULL}};
const Segment SGI281310[] = {{&S45757[90], 4}, {NULL}};
const Segment SGI281311[] = {{&S45757[13], 14}, {&S45757[28], 14}, {&S45757[64], 17}, {NULL}};
const Segment SGI281312[] = {{&S45758[97], 4}, {NULL}};
const Segment SGI281313[] = {{&S45758[13], 14}, {&S45758[28], 6}, {&S45758[35], 14}, {&S45758[71], 17}, {NULL}};
const Segment SGI281314[] = {{&S45759[90], 4}, {NULL}};
const Segment SGI281315[] = {{&S45759[13], 14}, {&S45759[28], 14}, {&S45759[64], 17}, {NULL}};
const Segment SGI281316[] = {{&S45760[97], 4}, {NULL}};
const Segment SGI281317[] = {{&S45760[13], 14}, {&S45760[28], 6}, {&S45760[35], 14}, {&S45760[71], 17}, {NULL}};
const Segment SGI281318[] = {{&S45761[90], 4}, {NULL}};
const Segment SGI281319[] = {{&S45761[13], 14}, {&S45761[28], 14}, {&S45761[64], 17}, {NULL}};
const Segment SGI281320[] = {{&S45848[80], 4}, {NULL}};
const Segment SGI281321[] = {{&S45848[13], 14}, {&S45848[28], 6}, {&S45848[56], 17}, {NULL}};
const Segment SGI281322[] = {{&S45848[75], 9}, {NULL}};
const Segment SGI281323[] = {{&S45849[73], 4}, {NULL}};
const Segment SGI281324[] = {{&S45849[13], 14}, {&S45849[49], 17}, {NULL}};
const Segment SGI281325[] = {{&S45849[68], 9}, {NULL}};
const Segment SGI281326[] = {{&S54223[7], 3}, {NULL}};
const Segment SGI281327[] = {{&S48472[74], 4}, {NULL}};
const Segment SGI281328[] = {{&S48472[13], 14}, {&S48472[48], 17}, {NULL}};
const Segment SGI281329[] = {{&S48472[100], 18}, {NULL}};
const Segment SGI281330[] = {{&S48473[81], 4}, {NULL}};
const Segment SGI281331[] = {{&S48473[13], 14}, {&S48473[28], 6}, {&S48473[55], 17}, {NULL}};
const Segment SGI281332[] = {{&S48473[107], 18}, {NULL}};
const Segment SGI281333[] = {{&S48708[91], 4}, {NULL}};
const Segment SGI281334[] = {{&S48708[13], 3}, {&S48708[17], 11}, {&S48708[50], 17}, {NULL}};
const Segment SGI281335[] = {{&S48708[69], 12}, {NULL}};
const Segment SGI281336[] = {{&S48709[93], 4}, {NULL}};
const Segment SGI281337[] = {{&S48709[13], 3}, {&S48709[17], 13}, {&S48709[52], 17}, {NULL}};
const Segment SGI281338[] = {{&S48709[71], 12}, {NULL}};
const Segment SGI281339[] = {{&S48710[101], 4}, {NULL}};
const Segment SGI281340[] = {{&S48710[13], 3}, {&S48710[17], 2}, {&S48710[20], 11}, {&S48710[32], 6}, {&S48710[60], 17}, {NULL}};
const Segment SGI281341[] = {{&S48710[79], 12}, {NULL}};
const Segment SGI281342[] = {{&S48711[103], 4}, {NULL}};
const Segment SGI281343[] = {{&S48711[13], 3}, {&S48711[17], 2}, {&S48711[20], 13}, {&S48711[34], 6}, {&S48711[62], 17}, {NULL}};
const Segment SGI281344[] = {{&S48711[81], 12}, {NULL}};
const Segment SGI281345[] = {{&S54658[102], 3}, {NULL}};
const Segment SGI281346[] = {{&S54658[13], 9}, {&S54658[23], 2}, {&S54658[26], 22}, {&S54658[49], 6}, {&S54658[76], 17}, {NULL}};
const Segment SGI281347[] = {{&S54659[102], 3}, {NULL}};
const Segment SGI281348[] = {{&S54659[13], 9}, {&S54659[23], 2}, {&S54659[26], 22}, {&S54659[49], 6}, {&S54659[76], 17}, {NULL}};
const Segment SGI281349[] = {{&S54660[102], 3}, {NULL}};
const Segment SGI281350[] = {{&S54660[13], 9}, {&S54660[23], 2}, {&S54660[26], 22}, {&S54660[49], 6}, {&S54660[76], 17}, {NULL}};
const Segment SGI281351[] = {{&S54661[102], 3}, {NULL}};
const Segment SGI281352[] = {{&S54661[13], 9}, {&S54661[23], 2}, {&S54661[26], 22}, {&S54661[49], 6}, {&S54661[76], 17}, {NULL}};
const Segment SGI281353[] = {{&S54662[102], 3}, {NULL}};
const Segment SGI281354[] = {{&S54662[13], 9}, {&S54662[23], 2}, {&S54662[26], 22}, {&S54662[49], 6}, {&S54662[76], 17}, {NULL}};
const Segment SGI281355[] = {{&S54663[102], 3}, {NULL}};
const Segment SGI281356[] = {{&S54663[13], 9}, {&S54663[23], 2}, {&S54663[26], 22}, {&S54663[49], 6}, {&S54663[76], 17}, {NULL}};
const Segment SGI281357[] = {{&S48835[74], 4}, {NULL}};
const Segment SGI281358[] = {{&S48835[13], 14}, {&S48835[48], 17}, {NULL}};
const Segment SGI281359[] = {{&S48835[100], 18}, {NULL}};
const Segment SGI281360[] = {{&S48836[81], 4}, {NULL}};
const Segment SGI281361[] = {{&S48836[13], 14}, {&S48836[28], 6}, {&S48836[55], 17}, {NULL}};
const Segment SGI281362[] = {{&S48836[107], 18}, {NULL}};
const Segment SGI281363[] = {{&S48846[93], 4}, {NULL}};
const Segment SGI281364[] = {{&S48846[13], 3}, {&S48846[17], 13}, {&S48846[52], 17}, {NULL}};
const Segment SGI281365[] = {{&S48846[71], 12}, {NULL}};
const Segment SGI281366[] = {{&S48847[91], 4}, {NULL}};
const Segment SGI281367[] = {{&S48847[13], 3}, {&S48847[17], 11}, {&S48847[50], 17}, {NULL}};
const Segment SGI281368[] = {{&S48847[69], 12}, {NULL}};
const Segment SGI281369[] = {{&S46707[75], 4}, {NULL}};
const Segment SGI281370[] = {{&S46707[13], 14}, {&S46707[49], 17}, {NULL}};
const Segment SGI281371[] = {{&S46707[102], 21}, {NULL}};
const Segment SGI281372[] = {{&S46708[74], 4}, {NULL}};
const Segment SGI281373[] = {{&S46708[13], 14}, {&S46708[48], 17}, {NULL}};
const Segment SGI281374[] = {{&S46708[100], 21}, {NULL}};
const Segment SGI281375[] = {{&S46709[89], 4}, {NULL}};
const Segment SGI281376[] = {{&S46709[13], 14}, {&S46709[28], 14}, {&S46709[63], 17}, {NULL}};
const Segment SGI281377[] = {{&S46710[89], 4}, {NULL}};
const Segment SGI281378[] = {{&S46710[13], 14}, {&S46710[28], 14}, {&S46710[63], 17}, {NULL}};
const Segment SGI281379[] = {{&S46711[90], 4}, {NULL}};
const Segment SGI281380[] = {{&S46711[13], 14}, {&S46711[28], 14}, {&S46711[64], 17}, {NULL}};
const Segment SGI281381[] = {{&S46713[90], 4}, {NULL}};
const Segment SGI281382[] = {{&S46713[13], 14}, {&S46713[28], 14}, {&S46713[64], 17}, {NULL}};
const Segment SGI281383[] = {{&S46714[90], 4}, {NULL}};
const Segment SGI281384[] = {{&S46714[13], 14}, {&S46714[28], 14}, {&S46714[64], 17}, {NULL}};
const Segment SGI281385[] = {{&S46715[90], 4}, {NULL}};
const Segment SGI281386[] = {{&S46715[13], 14}, {&S46715[28], 14}, {&S46715[64], 17}, {NULL}};
const Segment SGI281387[] = {{&S46716[90], 4}, {NULL}};
const Segment SGI281388[] = {{&S46716[13], 14}, {&S46716[28], 14}, {&S46716[64], 17}, {NULL}};
const Segment SGI281389[] = {{&S46717[90], 4}, {NULL}};
const Segment SGI281390[] = {{&S46717[13], 14}, {&S46717[28], 14}, {&S46717[64], 17}, {NULL}};
const Segment SGI281391[] = {{&S46718[90], 4}, {NULL}};
const Segment SGI281392[] = {{&S46718[13], 14}, {&S46718[28], 14}, {&S46718[64], 17}, {NULL}};
const Segment SGI281393[] = {{&S46755[75], 4}, {NULL}};
const Segment SGI281394[] = {{&S46755[13], 14}, {&S46755[49], 17}, {NULL}};
const Segment SGI281395[] = {{&S46834[75], 4}, {NULL}};
const Segment SGI281396[] = {{&S46834[13], 14}, {&S46834[49], 17}, {NULL}};
const Segment SGI281397[] = {{&S49374[79], 4}, {NULL}};
const Segment SGI281398[] = {{&S49374[13], 14}, {&S49374[28], 6}, {&S49374[55], 17}, {NULL}};
const Segment SGI281399[] = {{&S49374[74], 9}, {NULL}};
const Segment SGI281400[] = {{&S49375[72], 4}, {NULL}};
const Segment SGI281401[] = {{&S49375[13], 14}, {&S49375[48], 17}, {NULL}};
const Segment SGI281402[] = {{&S49375[67], 9}, {NULL}};
const Segment SGI281403[] = {{&S55245[102], 3}, {NULL}};
const Segment SGI281404[] = {{&S55245[13], 9}, {&S55245[23], 2}, {&S55245[26], 22}, {&S55245[49], 6}, {&S55245[76], 17}, {NULL}};
const Segment SGI281405[] = {{&S55246[102], 3}, {NULL}};
const Segment SGI281406[] = {{&S55246[13], 9}, {&S55246[23], 2}, {&S55246[26], 22}, {&S55246[49], 6}, {&S55246[76], 17}, {NULL}};
const Segment SGI281407[] = {{&S55247[102], 3}, {NULL}};
const Segment SGI281408[] = {{&S55247[13], 9}, {&S55247[23], 2}, {&S55247[26], 22}, {&S55247[49], 6}, {&S55247[76], 17}, {NULL}};
const Segment SGI281409[] = {{&S55248[102], 3}, {NULL}};
const Segment SGI281410[] = {{&S55248[13], 9}, {&S55248[23], 2}, {&S55248[26], 22}, {&S55248[49], 6}, {&S55248[76], 17}, {NULL}};
const Segment SGI281411[] = {{&S55249[102], 3}, {NULL}};
const Segment SGI281412[] = {{&S55249[13], 9}, {&S55249[23], 2}, {&S55249[26], 22}, {&S55249[49], 6}, {&S55249[76], 17}, {NULL}};
const Segment SGI281413[] = {{&S55250[102], 3}, {NULL}};
const Segment SGI281414[] = {{&S55250[13], 9}, {&S55250[23], 2}, {&S55250[26], 22}, {&S55250[49], 6}, {&S55250[76], 17}, {NULL}};
const Segment SGI281415[] = {{&S55251[102], 3}, {NULL}};
const Segment SGI281416[] = {{&S55251[13], 9}, {&S55251[23], 2}, {&S55251[26], 22}, {&S55251[49], 6}, {&S55251[76], 17}, {NULL}};
const Segment SGI281417[] = {{&S55252[102], 3}, {NULL}};
const Segment SGI281418[] = {{&S55252[13], 9}, {&S55252[23], 2}, {&S55252[26], 22}, {&S55252[49], 6}, {&S55252[76], 17}, {NULL}};
const Segment SGI281419[] = {{&S49655[92], 4}, {NULL}};
const Segment SGI281420[] = {{&S49655[13], 9}, {&S49655[23], 22}, {&S49655[66], 17}, {NULL}};
const Segment SGI281421[] = {{&S49656[92], 4}, {NULL}};
const Segment SGI281422[] = {{&S49656[13], 9}, {&S49656[23], 22}, {&S49656[66], 17}, {NULL}};
const Segment SGI281423[] = {{&S49657[92], 4}, {NULL}};
const Segment SGI281424[] = {{&S49657[13], 9}, {&S49657[23], 22}, {&S49657[66], 17}, {NULL}};
const Segment SGI281425[] = {{&S49658[92], 4}, {NULL}};
const Segment SGI281426[] = {{&S49658[13], 9}, {&S49658[23], 22}, {&S49658[66], 17}, {NULL}};
const Segment SGI281427[] = {{&S49659[92], 4}, {NULL}};
const Segment SGI281428[] = {{&S49659[13], 9}, {&S49659[23], 22}, {&S49659[66], 17}, {NULL}};
const Segment SGI281429[] = {{&S49660[92], 4}, {NULL}};
const Segment SGI281430[] = {{&S49660[13], 9}, {&S49660[23], 22}, {&S49660[66], 17}, {NULL}};
const Segment SGI281431[] = {{&S49722[94], 4}, {NULL}};
const Segment SGI281432[] = {{&S49722[13], 14}, {&S49722[49], 17}, {NULL}};
const Segment SGI281433[] = {{&S49722[68], 11}, {NULL}};
const Segment SGI281434[] = {{&S49723[101], 4}, {NULL}};
const Segment SGI281435[] = {{&S49723[13], 14}, {&S49723[28], 6}, {&S49723[56], 17}, {NULL}};
const Segment SGI281436[] = {{&S49723[75], 11}, {NULL}};
const Segment SGI281437[] = {{&S49724[94], 4}, {NULL}};
const Segment SGI281438[] = {{&S49724[13], 14}, {&S49724[49], 17}, {NULL}};
const Segment SGI281439[] = {{&S49724[68], 11}, {NULL}};
const Segment SGI281440[] = {{&S49725[101], 4}, {NULL}};
const Segment SGI281441[] = {{&S49725[13], 14}, {&S49725[28], 6}, {&S49725[56], 17}, {NULL}};
const Segment SGI281442[] = {{&S49725[75], 11}, {NULL}};
const Segment SGI281443[] = {{&S46341[93], 4}, {NULL}};
const Segment SGI281444[] = {{&S46341[13], 9}, {&S46341[23], 22}, {&S46341[67], 17}, {NULL}};
const Segment SGI281445[] = {{&S47216[74], 4}, {NULL}};
const Segment SGI281446[] = {{&S47216[13], 14}, {&S47216[48], 17}, {NULL}};
const Segment SGI281447[] = {{&S47216[109], 4}, {NULL}};
const Segment SGI281448[] = {{&S47306[48], 4}, {NULL}};
const Segment SGI281449[] = {{&S47306[13], 14}, {&S47306[28], 6}, {&S47306[35], 25}, {&S47306[82], 17}, {NULL}};
const Segment SGI281450[] = {{&S47308[48], 4}, {NULL}};
const Segment SGI281451[] = {{&S47308[13], 14}, {&S47308[28], 6}, {&S47308[35], 25}, {&S47308[82], 17}, {NULL}};
const Segment SGI281452[] = {{&S47312[41], 4}, {NULL}};
const Segment SGI281453[] = {{&S47312[13], 14}, {&S47312[28], 25}, {&S47312[75], 17}, {NULL}};
const Segment SGI281454[] = {{&S47313[93], 4}, {NULL}};
const Segment SGI281455[] = {{&S47313[13], 14}, {&S47313[48], 17}, {NULL}};
const Segment SGI281456[] = {{&S47313[67], 11}, {NULL}};
const Segment SGI281457[] = {{&S47314[100], 4}, {NULL}};
const Segment SGI281458[] = {{&S47314[13], 14}, {&S47314[28], 6}, {&S47314[55], 17}, {NULL}};
const Segment SGI281459[] = {{&S47314[74], 11}, {NULL}};
const Segment SGI281460[] = {{&S50036[72], 4}, {NULL}};
const Segment SGI281461[] = {{&S50036[13], 14}, {&S50036[48], 17}, {NULL}};
const Segment SGI281462[] = {{&S50036[67], 9}, {NULL}};
const Segment SGI281463[] = {{&S50037[79], 4}, {NULL}};
const Segment SGI281464[] = {{&S50037[13], 14}, {&S50037[28], 6}, {&S50037[55], 17}, {NULL}};
const Segment SGI281465[] = {{&S50037[74], 9}, {NULL}};
const Segment SGI281466[] = {{&S50252[84], 4}, {NULL}};
const Segment SGI281467[] = {{&S50252[13], 3}, {&S50252[17], 19}, {&S50252[58], 17}, {NULL}};
const Segment SGI281468[] = {{&S50253[85], 4}, {NULL}};
const Segment SGI281469[] = {{&S50253[13], 3}, {&S50253[17], 20}, {&S50253[59], 17}, {NULL}};
const Segment SGI281470[] = {{&S50254[85], 4}, {NULL}};
const Segment SGI281471[] = {{&S50254[13], 3}, {&S50254[17], 20}, {&S50254[59], 17}, {NULL}};
const Segment SGI281472[] = {{&S50255[85], 4}, {NULL}};
const Segment SGI281473[] = {{&S50255[13], 3}, {&S50255[17], 20}, {&S50255[59], 17}, {NULL}};
const Segment SGI281474[] = {{&S50256[85], 4}, {NULL}};
const Segment SGI281475[] = {{&S50256[13], 3}, {&S50256[17], 20}, {&S50256[59], 17}, {NULL}};
const Segment SGI281476[] = {{&S50257[85], 4}, {NULL}};
const Segment SGI281477[] = {{&S50257[13], 3}, {&S50257[17], 20}, {&S50257[59], 17}, {NULL}};
const Segment SGI281478[] = {{&S50258[86], 4}, {NULL}};
const Segment SGI281479[] = {{&S50258[13], 3}, {&S50258[17], 21}, {&S50258[60], 17}, {NULL}};
const Segment SGI281480[] = {{&S50259[87], 4}, {NULL}};
const Segment SGI281481[] = {{&S50259[13], 3}, {&S50259[17], 22}, {&S50259[61], 17}, {NULL}};
const Segment SGI281482[] = {{&S50260[85], 4}, {NULL}};
const Segment SGI281483[] = {{&S50260[13], 3}, {&S50260[17], 20}, {&S50260[59], 17}, {NULL}};
const Segment SGI281484[] = {{&S50261[85], 4}, {NULL}};
const Segment SGI281485[] = {{&S50261[13], 3}, {&S50261[17], 20}, {&S50261[59], 17}, {NULL}};
const Segment SGI281486[] = {{&S50262[86], 4}, {NULL}};
const Segment SGI281487[] = {{&S50262[13], 3}, {&S50262[17], 21}, {&S50262[60], 17}, {NULL}};
const Segment SGI281488[] = {{&S50263[86], 4}, {NULL}};
const Segment SGI281489[] = {{&S50263[13], 3}, {&S50263[17], 21}, {&S50263[60], 17}, {NULL}};
const Segment SGI281490[] = {{&S50264[87], 4}, {NULL}};
const Segment SGI281491[] = {{&S50264[13], 3}, {&S50264[17], 22}, {&S50264[61], 17}, {NULL}};
const Segment SGI281492[] = {{&S50265[87], 4}, {NULL}};
const Segment SGI281493[] = {{&S50265[13], 3}, {&S50265[17], 22}, {&S50265[61], 17}, {NULL}};
const Segment SGI281494[] = {{&S50266[87], 4}, {NULL}};
const Segment SGI281495[] = {{&S50266[13], 3}, {&S50266[17], 22}, {&S50266[61], 17}, {NULL}};
const Segment SGI281496[] = {{&S50267[87], 4}, {NULL}};
const Segment SGI281497[] = {{&S50267[13], 3}, {&S50267[17], 22}, {&S50267[61], 17}, {NULL}};
const Segment SGI281498[] = {{&S50268[87], 4}, {NULL}};
const Segment SGI281499[] = {{&S50268[13], 3}, {&S50268[17], 22}, {&S50268[61], 17}, {NULL}};
const Segment SGI281500[] = {{&S50269[85], 4}, {NULL}};
const Segment SGI281501[] = {{&S50269[13], 3}, {&S50269[17], 20}, {&S50269[59], 17}, {NULL}};
const Segment SGI281502[] = {{&S50270[87], 4}, {NULL}};
const Segment SGI281503[] = {{&S50270[13], 3}, {&S50270[17], 22}, {&S50270[61], 17}, {NULL}};
const Segment SGI281504[] = {{&S50271[88], 4}, {NULL}};
const Segment SGI281505[] = {{&S50271[13], 3}, {&S50271[17], 23}, {&S50271[62], 17}, {NULL}};
const Segment SGI281506[] = {{&S56209[92], 3}, {NULL}};
const Segment SGI281507[] = {{&S56209[13], 7}, {&S56209[21], 2}, {&S56209[24], 15}, {&S56209[40], 6}, {&S56209[68], 17}, {NULL}};
const Segment SGI281508[] = {{&S56209[87], 8}, {NULL}};
const Segment SGI281509[] = {{&S47789[75], 4}, {NULL}};
const Segment SGI281510[] = {{&S47789[13], 14}, {&S47789[49], 17}, {NULL}};
const Segment SGI281511[] = {{&S48140[78], 4}, {NULL}};
const Segment SGI281512[] = {{&S48140[13], 3}, {&S48140[17], 13}, {&S48140[52], 17}, {NULL}};
const Segment SGI281513[] = {{&S48140[106], 8}, {NULL}};
const Segment SGI281514[] = {{&S48141[78], 4}, {NULL}};
const Segment SGI281515[] = {{&S48141[13], 3}, {&S48141[17], 13}, {&S48141[52], 17}, {NULL}};
const Segment SGI281516[] = {{&S48141[105], 8}, {NULL}};
const Segment SGI281517[] = {{&S50630[90], 4}, {NULL}};
const Segment SGI281518[] = {{&S50630[13], 3}, {&S50630[17], 11}, {&S50630[49], 17}, {NULL}};
const Segment SGI281519[] = {{&S50630[68], 12}, {NULL}};
const Segment SGI281520[] = {{&S50631[92], 4}, {NULL}};
const Segment SGI281521[] = {{&S50631[13], 3}, {&S50631[17], 13}, {&S50631[51], 17}, {NULL}};
const Segment SGI281522[] = {{&S50631[70], 12}, {NULL}};
const Segment SGI281523[] = {{&S48539[111], 4}, {NULL}};
const Segment SGI281524[] = {{&S48539[13], 9}, {&S48539[23], 22}, {&S48539[66], 17}, {NULL}};
const Segment SGI281525[] = {{&S48539[85], 11}, {NULL}};
const Segment SGI281526[] = {{&S48541[112], 4}, {NULL}};
const Segment SGI281527[] = {{&S48541[13], 9}, {&S48541[23], 22}, {&S48541[67], 17}, {NULL}};
const Segment SGI281528[] = {{&S48541[86], 11}, {NULL}};
const Segment SGI281529[] = {{&S57010[101], 3}, {NULL}};
const Segment SGI281530[] = {{&S57010[13], 3}, {&S57010[17], 2}, {&S57010[20], 11}, {&S57010[32], 6}, {&S57010[60], 17}, {NULL}};
const Segment SGI281531[] = {{&S57010[79], 12}, {NULL}};
const Segment SGI281532[] = {{&S57011[103], 3}, {NULL}};
const Segment SGI281533[] = {{&S57011[13], 3}, {&S57011[17], 2}, {&S57011[20], 13}, {&S57011[34], 6}, {&S57011[62], 17}, {NULL}};
const Segment SGI281534[] = {{&S57011[81], 12}, {NULL}};
const Segment SGI281535[] = {{&S57012[103], 3}, {NULL}};
const Segment SGI281536[] = {{&S57012[13], 3}, {&S57012[17], 2}, {&S57012[20], 13}, {&S57012[34], 6}, {&S57012[62], 17}, {NULL}};
const Segment SGI281537[] = {{&S57012[81], 12}, {NULL}};
const Segment SGI281538[] = {{&S48689[75], 4}, {NULL}};
const Segment SGI281539[] = {{&S48689[13], 14}, {&S48689[49], 17}, {NULL}};
const Segment SGI281540[] = {{&S48707[78], 4}, {NULL}};
const Segment SGI281541[] = {{&S48707[13], 3}, {&S48707[17], 13}, {&S48707[52], 17}, {NULL}};
const Segment SGI281542[] = {{&S48707[105], 8}, {NULL}};
const Segment SGI281543[] = {{&S49184[86], 4}, {NULL}};
const Segment SGI281544[] = {{&S49184[13], 3}, {&S49184[17], 21}, {&S49184[60], 17}, {NULL}};
const Segment SGI281545[] = {{&S49185[87], 4}, {NULL}};
const Segment SGI281546[] = {{&S49185[13], 3}, {&S49185[17], 22}, {&S49185[61], 17}, {NULL}};
const Segment SGI281547[] = {{&S49186[85], 4}, {NULL}};
const Segment SGI281548[] = {{&S49186[13], 3}, {&S49186[17], 20}, {&S49186[59], 17}, {NULL}};
const Segment SGI281549[] = {{&S49187[85], 4}, {NULL}};
const Segment SGI281550[] = {{&S49187[13], 3}, {&S49187[17], 20}, {&S49187[59], 17}, {NULL}};
const Segment SGI281551[] = {{&S49188[85], 4}, {NULL}};
const Segment SGI281552[] = {{&S49188[13], 3}, {&S49188[17], 20}, {&S49188[59], 17}, {NULL}};
const Segment SGI281553[] = {{&S49189[87], 4}, {NULL}};
const Segment SGI281554[] = {{&S49189[13], 3}, {&S49189[17], 22}, {&S49189[61], 17}, {NULL}};
const Segment SGI281555[] = {{&S49190[85], 4}, {NULL}};
const Segment SGI281556[] = {{&S49190[13], 3}, {&S49190[17], 20}, {&S49190[59], 17}, {NULL}};
const Segment SGI281557[] = {{&S49191[85], 4}, {NULL}};
const Segment SGI281558[] = {{&S49191[13], 3}, {&S49191[17], 20}, {&S49191[59], 17}, {NULL}};
const Segment SGI281559[] = {{&S49192[87], 4}, {NULL}};
const Segment SGI281560[] = {{&S49192[13], 3}, {&S49192[17], 22}, {&S49192[61], 17}, {NULL}};
const Segment SGI281561[] = {{&S49193[86], 4}, {NULL}};
const Segment SGI281562[] = {{&S49193[13], 3}, {&S49193[17], 21}, {&S49193[60], 17}, {NULL}};
const Segment SGI281563[] = {{&S49194[88], 4}, {NULL}};
const Segment SGI281564[] = {{&S49194[13], 3}, {&S49194[17], 23}, {&S49194[62], 17}, {NULL}};
const Segment SGI281565[] = {{&S49195[86], 4}, {NULL}};
const Segment SGI281566[] = {{&S49195[13], 3}, {&S49195[17], 21}, {&S49195[60], 17}, {NULL}};
const Segment SGI281567[] = {{&S49196[88], 4}, {NULL}};
const Segment SGI281568[] = {{&S49196[13], 3}, {&S49196[17], 23}, {&S49196[62], 17}, {NULL}};
const Segment SGI281569[] = {{&S49197[86], 4}, {NULL}};
const Segment SGI281570[] = {{&S49197[13], 3}, {&S49197[17], 21}, {&S49197[60], 17}, {NULL}};
const Segment SGI281571[] = {{&S49198[88], 4}, {NULL}};
const Segment SGI281572[] = {{&S49198[13], 3}, {&S49198[17], 23}, {&S49198[62], 17}, {NULL}};
const Segment SGI281573[] = {{&S49199[86], 4}, {NULL}};
const Segment SGI281574[] = {{&S49199[13], 3}, {&S49199[17], 22}, {&S49199[60], 17}, {NULL}};
const Segment SGI281575[] = {{&S49246[93], 4}, {NULL}};
const Segment SGI281576[] = {{&S49246[13], 9}, {&S49246[23], 22}, {&S49246[67], 17}, {NULL}};
const Segment SGI281577[] = {{&S49247[91], 4}, {NULL}};
const Segment SGI281578[] = {{&S49247[13], 9}, {&S49247[23], 22}, {&S49247[67], 17}, {NULL}};
const Segment SGI281579[] = {{&S49247[86], 9}, {NULL}};
const Segment SGI281580[] = {{&S51789[96], 4}, {NULL}};
const Segment SGI281581[] = {{&S51789[13], 9}, {&S51789[23], 2}, {&S51789[26], 22}, {&S51789[70], 17}, {NULL}};
const Segment SGI281582[] = {{&S57888[91], 3}, {NULL}};
const Segment SGI281583[] = {{&S57888[13], 7}, {&S57888[21], 2}, {&S57888[24], 15}, {&S57888[40], 6}, {&S57888[67], 17}, {NULL}};
const Segment SGI281584[] = {{&S57888[86], 8}, {NULL}};
const Segment SGI281585[] = {{&S49480[75], 4}, {NULL}};
const Segment SGI281586[] = {{&S49480[13], 14}, {&S49480[49], 17}, {NULL}};
const Segment SGI281587[] = {{&S51862[74], 4}, {NULL}};
const Segment SGI281588[] = {{&S51862[13], 14}, {&S51862[49], 17}, {NULL}};
const Segment SGI281589[] = {{&S51862[69], 9}, {NULL}};
const Segment SGI281590[] = {{&S51863[81], 4}, {NULL}};
const Segment SGI281591[] = {{&S51863[13], 14}, {&S51863[28], 6}, {&S51863[56], 17}, {NULL}};
const Segment SGI281592[] = {{&S51863[76], 9}, {NULL}};
const Segment SGI281593[] = {{&S51874[88], 4}, {NULL}};
const Segment SGI281594[] = {{&S51874[13], 14}, {&S51874[28], 6}, {&S51874[55], 17}, {NULL}};
const Segment SGI281595[] = {{&S51874[74], 18}, {NULL}};
const Segment SGI281596[] = {{&S51875[81], 4}, {NULL}};
const Segment SGI281597[] = {{&S51875[13], 14}, {&S51875[48], 17}, {NULL}};
const Segment SGI281598[] = {{&S51875[67], 18}, {NULL}};
const Segment SGI281599[] = {{&S51893[91], 4}, {NULL}};
const Segment SGI281600[] = {{&S51893[13], 3}, {&S51893[17], 11}, {&S51893[50], 17}, {NULL}};
const Segment SGI281601[] = {{&S51893[69], 12}, {NULL}};
const Segment SGI281602[] = {{&S51894[93], 4}, {NULL}};
const Segment SGI281603[] = {{&S51894[13], 3}, {&S51894[17], 13}, {&S51894[52], 17}, {NULL}};
const Segment SGI281604[] = {{&S51894[71], 12}, {NULL}};
const Segment SGI281605[] = {{&S49644[75], 4}, {NULL}};
const Segment SGI281606[] = {{&S49644[13], 14}, {&S49644[49], 17}, {NULL}};
const Segment SGI281607[] = {{&S51944[80], 4}, {NULL}};
const Segment SGI281608[] = {{&S51944[13], 14}, {&S51944[28], 6}, {&S51944[55], 17}, {NULL}};
const Segment SGI281609[] = {{&S51944[75], 9}, {NULL}};
const Segment SGI281610[] = {{&S51945[73], 4}, {NULL}};
const Segment SGI281611[] = {{&S51945[13], 14}, {&S51945[48], 17}, {NULL}};
const Segment SGI281612[] = {{&S51945[68], 9}, {NULL}};
const Segment SGI281613[] = {{&S50028[48], 4}, {NULL}};
const Segment SGI281614[] = {{&S50028[13], 14}, {&S50028[28], 6}, {&S50028[35], 25}, {&S50028[82], 17}, {NULL}};
const Segment SGI281615[] = {{&S50031[48], 4}, {NULL}};
const Segment SGI281616[] = {{&S50031[13], 14}, {&S50031[28], 6}, {&S50031[35], 26}, {&S50031[83], 17}, {NULL}};
const Segment SGI281617[] = {{&S50032[48], 4}, {NULL}};
const Segment SGI281618[] = {{&S50032[13], 14}, {&S50032[28], 6}, {&S50032[35], 26}, {&S50032[83], 17}, {NULL}};
const Segment SGI281619[] = {{&S50035[41], 4}, {NULL}};
const Segment SGI281620[] = {{&S50035[13], 14}, {&S50035[28], 26}, {&S50035[76], 17}, {NULL}};
const Segment SGI281621[] = {{&S52254[88], 4}, {NULL}};
const Segment SGI281622[] = {{&S52254[13], 3}, {&S52254[17], 2}, {&S52254[20], 13}, {&S52254[34], 6}, {&S52254[62], 17}, {NULL}};
const Segment SGI281623[] = {{&S52255[90], 4}, {NULL}};
const Segment SGI281624[] = {{&S52255[13], 3}, {&S52255[17], 2}, {&S52255[20], 15}, {&S52255[36], 6}, {&S52255[64], 17}, {NULL}};
const Segment SGI281625[] = {{&S58649[92], 3}, {NULL}};
const Segment SGI281626[] = {{&S58649[13], 7}, {&S58649[21], 2}, {&S58649[24], 15}, {&S58649[40], 6}, {&S58649[68], 17}, {NULL}};
const Segment SGI281627[] = {{&S58649[87], 8}, {NULL}};
const Segment SGI281628[] = {{&S50218[75], 4}, {NULL}};
const Segment SGI281629[] = {{&S50218[13], 14}, {&S50218[49], 17}, {NULL}};
const Segment SGI281630[] = {{&S52498[90], 4}, {NULL}};
const Segment SGI281631[] = {{&S52498[13], 3}, {&S52498[17], 11}, {&S52498[49], 17}, {NULL}};
const Segment SGI281632[] = {{&S52498[68], 12}, {NULL}};
const Segment SGI281633[] = {{&S52499[92], 4}, {NULL}};
const Segment SGI281634[] = {{&S52499[13], 3}, {&S52499[17], 13}, {&S52499[51], 17}, {NULL}};
const Segment SGI281635[] = {{&S58760[94], 3}, {NULL}};
const Segment SGI281636[] = {{&S52499[70], 12}, {NULL}};
const Segment SGI281637[] = {{&S58760[13], 7}, {&S58760[21], 2}, {&S58760[24], 15}, {&S58760[40], 6}, {&S58760[68], 17}, {&S58760[118], 31}, {NULL}};
const Segment SGI281638[] = {{&S52527[105], 4}, {NULL}};
const Segment SGI281639[] = {{&S52527[13], 3}, {&S52527[17], 13}, {&S52527[31], 26}, {&S52527[79], 17}, {NULL}};
const Segment SGI281640[] = {{&S50507[113], 4}, {NULL}};
const Segment SGI281641[] = {{&S50507[13], 9}, {&S50507[23], 22}, {&S50507[46], 20}, {&S50507[87], 17}, {NULL}};
const Segment SGI281642[] = {{&S50581[75], 4}, {NULL}};
const Segment SGI281643[] = {{&S50581[13], 14}, {&S50581[49], 17}, {NULL}};
const Segment SGI281644[] = {{&S52814[93], 4}, {NULL}};
const Segment SGI281645[] = {{&S52814[13], 9}, {&S52814[23], 22}, {&S52814[67], 17}, {NULL}};
const Segment SGI281646[] = {{&S52815[93], 4}, {NULL}};
const Segment SGI281647[] = {{&S52815[13], 9}, {&S52815[23], 22}, {&S52815[67], 17}, {NULL}};
const Segment SGI281648[] = {{&S52816[93], 4}, {NULL}};
const Segment SGI281649[] = {{&S52816[13], 9}, {&S52816[23], 22}, {&S52816[67], 17}, {NULL}};
const Segment SGI281650[] = {{&S52817[93], 4}, {NULL}};
const Segment SGI281651[] = {{&S52817[13], 9}, {&S52817[23], 22}, {&S52817[67], 17}, {NULL}};
const Segment SGI281652[] = {{&S52818[93], 4}, {NULL}};
const Segment SGI281653[] = {{&S52818[13], 9}, {&S52818[23], 22}, {&S52818[67], 17}, {NULL}};
const Segment SGI281654[] = {{&S52819[93], 4}, {NULL}};
const Segment SGI281655[] = {{&S52819[13], 9}, {&S52819[23], 22}, {&S52819[67], 17}, {NULL}};
const Segment SGI281656[] = {{&S52872[71], 4}, {NULL}};
const Segment SGI281657[] = {{&S52872[13], 3}, {&S52872[17], 6}, {&S52872[45], 17}, {NULL}};
const Segment SGI281658[] = {{&S52872[106], 3}, {NULL}};
const Segment SGI281659[] = {{&S52873[78], 4}, {NULL}};
const Segment SGI281660[] = {{&S52873[13], 3}, {&S52873[17], 13}, {&S52873[52], 17}, {NULL}};
const Segment SGI281661[] = {{&S52873[113], 3}, {NULL}};
const Segment SGI281662[] = {{&S52874[71], 4}, {NULL}};
const Segment SGI281663[] = {{&S52874[13], 3}, {&S52874[17], 6}, {&S52874[45], 17}, {NULL}};
const Segment SGI281664[] = {{&S52875[76], 4}, {NULL}};
const Segment SGI281665[] = {{&S52875[13], 3}, {&S52875[17], 11}, {&S52875[50], 17}, {NULL}};
const Segment SGI281666[] = {{&S52875[111], 3}, {NULL}};
const Segment SGI281667[] = {{&S50741[48], 4}, {NULL}};
const Segment SGI281668[] = {{&S50741[13], 14}, {&S50741[28], 6}, {&S50741[35], 25}, {&S50741[82], 17}, {NULL}};
const Segment SGI281669[] = {{&S50743[48], 4}, {NULL}};
const Segment SGI281670[] = {{&S50743[13], 14}, {&S50743[28], 6}, {&S50743[35], 25}, {&S50743[82], 17}, {NULL}};
const Segment SGI281671[] = {{&S50746[48], 4}, {NULL}};
const Segment SGI281672[] = {{&S50746[13], 14}, {&S50746[28], 6}, {&S50746[35], 25}, {&S50746[82], 17}, {NULL}};
const Segment SGI281673[] = {{&S50777[75], 4}, {NULL}};
const Segment SGI281674[] = {{&S50777[13], 14}, {&S50777[49], 17}, {NULL}};
const Segment SGI281675[] = {{&S52979[88], 4}, {NULL}};
const Segment SGI281676[] = {{&S52979[13], 3}, {&S52979[17], 2}, {&S52979[20], 13}, {&S52979[34], 6}, {&S52979[62], 17}, {NULL}};
const Segment SGI281677[] = {{&S52979[81], 12}, {NULL}};
const Segment SGI281678[] = {{&S52980[88], 4}, {NULL}};
const Segment SGI281679[] = {{&S52980[13], 3}, {&S52980[17], 2}, {&S52980[20], 13}, {&S52980[34], 6}, {&S52980[62], 17}, {NULL}};
const Segment SGI281680[] = {{&S52980[81], 12}, {NULL}};
const Segment SGI281681[] = {{&S52981[86], 4}, {NULL}};
const Segment SGI281682[] = {{&S52981[13], 3}, {&S52981[17], 2}, {&S52981[20], 11}, {&S52981[32], 6}, {&S52981[60], 17}, {NULL}};
const Segment SGI281683[] = {{&S52981[79], 12}, {NULL}};
const Segment SGI281684[] = {{&S51261[75], 4}, {NULL}};
const Segment SGI281685[] = {{&S51261[13], 14}, {&S51261[49], 17}, {NULL}};
const Segment SGI281686[] = {{&S51261[87], 9}, {NULL}};
const Segment SGI281687[] = {{&S51262[82], 4}, {NULL}};
const Segment SGI281688[] = {{&S51262[13], 14}, {&S51262[28], 6}, {&S51262[56], 17}, {NULL}};
const Segment SGI281689[] = {{&S51262[94], 9}, {NULL}};
const Segment SGI281690[] = {{&S51263[75], 4}, {NULL}};
const Segment SGI281691[] = {{&S51263[13], 14}, {&S51263[49], 17}, {NULL}};
const Segment SGI281692[] = {{&S51263[87], 9}, {NULL}};
const Segment SGI281693[] = {{&S53307[73], 4}, {NULL}};
const Segment SGI281694[] = {{&S53307[13], 14}, {&S53307[49], 17}, {NULL}};
const Segment SGI281695[] = {{&S53307[68], 9}, {NULL}};
const Segment SGI281696[] = {{&S53308[80], 4}, {NULL}};
const Segment SGI281697[] = {{&S53308[13], 14}, {&S53308[28], 6}, {&S53308[56], 17}, {NULL}};
const Segment SGI281698[] = {{&S53308[75], 9}, {NULL}};
const Segment SGI281699[] = {{&S53309[73], 4}, {NULL}};
const Segment SGI281700[] = {{&S53309[13], 14}, {&S53309[49], 17}, {NULL}};
const Segment SGI281701[] = {{&S53309[68], 9}, {NULL}};
const Segment SGI281702[] = {{&S59628[110], 3}, {NULL}};
const Segment SGI281703[] = {{&S59628[13], 7}, {&S59628[21], 2}, {&S59628[24], 15}, {&S59628[40], 6}, {&S59628[67], 17}, {NULL}};
const Segment SGI281704[] = {{&S59628[86], 9}, {NULL}};
const Segment SGI281705[] = {{&S59629[110], 3}, {NULL}};
const Segment SGI281706[] = {{&S59629[13], 7}, {&S59629[21], 2}, {&S59629[24], 15}, {&S59629[40], 6}, {&S59629[67], 17}, {NULL}};
const Segment SGI281707[] = {{&S59629[86], 9}, {NULL}};
const Segment SGI281708[] = {{&S59656[122], 3}, {NULL}};
const Segment SGI281709[] = {{&S59656[13], 5}, {&S59656[19], 2}, {&S59656[22], 14}, {&S59656[37], 6}, {&S59656[44], 30}, {&S59656[96], 17}, {NULL}};
const Segment SGI281710[] = {{&S59657[122], 3}, {NULL}};
const Segment SGI281711[] = {{&S59657[13], 5}, {&S59657[19], 2}, {&S59657[22], 14}, {&S59657[37], 6}, {&S59657[44], 30}, {&S59657[96], 17}, {NULL}};
const Segment SGI281712[] = {{&S59658[122], 3}, {NULL}};
const Segment SGI281713[] = {{&S51382[48], 4}, {NULL}};
const Segment SGI281714[] = {{&S59658[13], 5}, {&S59658[19], 2}, {&S59658[22], 14}, {&S59658[37], 6}, {&S59658[44], 30}, {&S59658[96], 17}, {NULL}};
const Segment SGI281715[] = {{&S51382[13], 14}, {&S51382[28], 6}, {&S51382[35], 25}, {&S51382[82], 17}, {NULL}};
const Segment SGI281716[] = {{&S59659[122], 3}, {NULL}};
const Segment SGI281717[] = {{&S51383[41], 4}, {NULL}};
const Segment SGI281718[] = {{&S51383[13], 14}, {&S51383[28], 25}, {&S51383[75], 17}, {NULL}};
const Segment SGI281719[] = {{&S59659[13], 5}, {&S59659[19], 2}, {&S59659[22], 14}, {&S59659[37], 6}, {&S59659[44], 30}, {&S59659[96], 17}, {NULL}};
const Segment SGI281720[] = {{&S59660[122], 3}, {NULL}};
const Segment SGI281721[] = {{&S51384[48], 4}, {NULL}};
const Segment SGI281722[] = {{&S51384[13], 14}, {&S51384[28], 6}, {&S51384[35], 25}, {&S51384[82], 17}, {NULL}};
const Segment SGI281723[] = {{&S59660[13], 5}, {&S59660[19], 2}, {&S59660[22], 14}, {&S59660[37], 6}, {&S59660[44], 30}, {&S59660[96], 17}, {NULL}};
const Segment SGI281724[] = {{&S59661[122], 3}, {NULL}};
const Segment SGI281725[] = {{&S59661[13], 5}, {&S59661[19], 2}, {&S59661[22], 14}, {&S59661[37], 6}, {&S59661[44], 30}, {&S59661[96], 17}, {NULL}};
const Segment SGI281726[] = {{&S59662[122], 3}, {NULL}};
const Segment SGI281727[] = {{&S59662[13], 5}, {&S59662[19], 2}, {&S59662[22], 14}, {&S59662[37], 6}, {&S59662[44], 30}, {&S59662[96], 17}, {NULL}};
const Segment SGI281728[] = {{&S51388[48], 4}, {NULL}};
const Segment SGI281729[] = {{&S51388[13], 14}, {&S51388[28], 6}, {&S51388[35], 25}, {&S51388[82], 17}, {NULL}};
const Segment SGI281730[] = {{&S59663[122], 3}, {NULL}};
const Segment SGI281731[] = {{&S59663[13], 5}, {&S59663[19], 2}, {&S59663[22], 14}, {&S59663[37], 6}, {&S59663[44], 30}, {&S59663[96], 17}, {NULL}};
const Segment SGI281732[] = {{&S59664[122], 3}, {NULL}};
const Segment SGI281733[] = {{&S59664[13], 5}, {&S59664[19], 2}, {&S59664[22], 14}, {&S59664[37], 6}, {&S59664[44], 30}, {&S59664[96], 17}, {NULL}};
const Segment SGI281734[] = {{&S51390[41], 4}, {NULL}};
const Segment SGI281735[] = {{&S51390[13], 14}, {&S51390[28], 25}, {&S51390[75], 17}, {NULL}};
const Segment SGI281736[] = {{&S59665[122], 3}, {NULL}};
const Segment SGI281737[] = {{&S59665[13], 5}, {&S59665[19], 2}, {&S59665[22], 14}, {&S59665[37], 6}, {&S59665[44], 30}, {&S59665[96], 17}, {NULL}};
const Segment SGI281738[] = {{&S51391[79], 4}, {NULL}};
const Segment SGI281739[] = {{&S51391[13], 3}, {&S51391[17], 14}, {&S51391[53], 17}, {NULL}};
const Segment SGI281740[] = {{&S59666[122], 3}, {NULL}};
const Segment SGI281741[] = {{&S59666[13], 5}, {&S59666[19], 2}, {&S59666[22], 14}, {&S59666[37], 6}, {&S59666[44], 30}, {&S59666[96], 17}, {NULL}};
const Segment SGI281742[] = {{&S51392[78], 4}, {NULL}};
const Segment SGI281743[] = {{&S51392[13], 3}, {&S51392[17], 13}, {&S51392[52], 17}, {NULL}};
const Segment SGI281744[] = {{&S59667[122], 3}, {NULL}};
const Segment SGI281745[] = {{&S59667[13], 5}, {&S59667[19], 2}, {&S59667[22], 14}, {&S59667[37], 6}, {&S59667[44], 30}, {&S59667[96], 17}, {NULL}};
const Segment SGI281746[] = {{&S51393[97], 4}, {NULL}};
const Segment SGI281747[] = {{&S51393[13], 7}, {&S51393[21], 2}, {&S51393[24], 6}, {&S51393[31], 6}, {&S51393[59], 17}, {NULL}};
const Segment SGI281748[] = {{&S59668[122], 3}, {NULL}};
const Segment SGI281749[] = {{&S59668[13], 5}, {&S59668[19], 2}, {&S59668[22], 14}, {&S59668[37], 6}, {&S59668[44], 30}, {&S59668[96], 17}, {NULL}};
const Segment SGI281750[] = {{&S51394[97], 4}, {NULL}};
const Segment SGI281751[] = {{&S51394[13], 7}, {&S51394[21], 2}, {&S51394[24], 6}, {&S51394[31], 6}, {&S51394[59], 17}, {NULL}};
const Segment SGI281752[] = {{&S59669[122], 3}, {NULL}};
const Segment SGI281753[] = {{&S59669[13], 5}, {&S59669[19], 2}, {&S59669[22], 14}, {&S59669[37], 6}, {&S59669[44], 30}, {&S59669[96], 17}, {NULL}};
const Segment SGI281754[] = {{&S51395[97], 4}, {NULL}};
const Segment SGI281755[] = {{&S51395[13], 7}, {&S51395[21], 2}, {&S51395[24], 6}, {&S51395[31], 6}, {&S51395[59], 17}, {NULL}};
const Segment SGI281756[] = {{&S59670[122], 3}, {NULL}};
const Segment SGI281757[] = {{&S59670[13], 5}, {&S59670[19], 2}, {&S59670[22], 14}, {&S59670[37], 6}, {&S59670[44], 30}, {&S59670[96], 17}, {NULL}};
const Segment SGI281758[] = {{&S51396[97], 4}, {NULL}};
const Segment SGI281759[] = {{&S51396[13], 7}, {&S51396[21], 2}, {&S51396[24], 6}, {&S51396[31], 6}, {&S51396[59], 17}, {NULL}};
const Segment SGI281760[] = {{&S59671[122], 3}, {NULL}};
const Segment SGI281761[] = {{&S51397[97], 4}, {NULL}};
const Segment SGI281762[] = {{&S59671[13], 5}, {&S59671[19], 2}, {&S59671[22], 14}, {&S59671[37], 6}, {&S59671[44], 30}, {&S59671[96], 17}, {NULL}};
const Segment SGI281763[] = {{&S51397[13], 7}, {&S51397[21], 2}, {&S51397[24], 6}, {&S51397[31], 6}, {&S51397[59], 17}, {NULL}};
const Segment SGI281764[] = {{&S51398[97], 4}, {NULL}};
const Segment SGI281765[] = {{&S59672[122], 3}, {NULL}};
const Segment SGI281766[] = {{&S51398[13], 7}, {&S51398[21], 2}, {&S51398[24], 6}, {&S51398[31], 6}, {&S51398[59], 17}, {NULL}};
const Segment SGI281767[] = {{&S59672[13], 5}, {&S59672[19], 2}, {&S59672[22], 14}, {&S59672[37], 6}, {&S59672[44], 30}, {&S59672[96], 17}, {NULL}};
const Segment SGI281768[] = {{&S59673[122], 3}, {NULL}};
const Segment SGI281769[] = {{&S51399[97], 4}, {NULL}};
const Segment SGI281770[] = {{&S59673[13], 5}, {&S59673[19], 2}, {&S59673[22], 14}, {&S59673[37], 6}, {&S59673[44], 30}, {&S59673[96], 17}, {NULL}};
const Segment SGI281771[] = {{&S51399[13], 7}, {&S51399[21], 2}, {&S51399[24], 6}, {&S51399[31], 6}, {&S51399[59], 17}, {NULL}};
const Segment SGI281772[] = {{&S59674[123], 3}, {NULL}};
const Segment SGI281773[] = {{&S51400[97], 4}, {NULL}};
const Segment SGI281774[] = {{&S51400[13], 7}, {&S51400[21], 2}, {&S51400[24], 6}, {&S51400[31], 6}, {&S51400[59], 17}, {NULL}};
const Segment SGI281775[] = {{&S59674[13], 5}, {&S59674[19], 2}, {&S59674[22], 14}, {&S59674[37], 6}, {&S59674[44], 31}, {&S59674[97], 17}, {NULL}};
const Segment SGI281776[] = {{&S59675[122], 3}, {NULL}};
const Segment SGI281777[] = {{&S51401[97], 4}, {NULL}};
const Segment SGI281778[] = {{&S59675[13], 5}, {&S59675[19], 2}, {&S59675[22], 14}, {&S59675[37], 6}, {&S59675[44], 30}, {&S59675[96], 17}, {NULL}};
const Segment SGI281779[] = {{&S51401[13], 7}, {&S51401[21], 2}, {&S51401[24], 6}, {&S51401[31], 6}, {&S51401[59], 17}, {NULL}};
const Segment SGI281780[] = {{&S59676[122], 3}, {NULL}};
const Segment SGI281781[] = {{&S51402[102], 4}, {NULL}};
const Segment SGI281782[] = {{&S59676[13], 5}, {&S59676[19], 2}, {&S59676[22], 14}, {&S59676[37], 6}, {&S59676[44], 30}, {&S59676[96], 17}, {NULL}};
const Segment SGI281783[] = {{&S51402[13], 3}, {&S51402[17], 8}, {&S51402[26], 2}, {&S51402[29], 6}, {&S51402[36], 6}, {&S51402[64], 17}, {NULL}};
const Segment SGI281784[] = {{&S59677[122], 3}, {NULL}};
const Segment SGI281785[] = {{&S51403[102], 4}, {NULL}};
const Segment SGI281786[] = {{&S59677[13], 5}, {&S59677[19], 2}, {&S59677[22], 14}, {&S59677[37], 6}, {&S59677[44], 30}, {&S59677[96], 17}, {NULL}};
const Segment SGI281787[] = {{&S51403[13], 3}, {&S51403[17], 8}, {&S51403[26], 2}, {&S51403[29], 6}, {&S51403[36], 6}, {&S51403[64], 17}, {NULL}};
const Segment SGI281788[] = {{&S59678[122], 3}, {NULL}};
const Segment SGI281789[] = {{&S51404[102], 4}, {NULL}};
const Segment SGI281790[] = {{&S59678[13], 5}, {&S59678[19], 2}, {&S59678[22], 14}, {&S59678[37], 6}, {&S59678[44], 30}, {&S59678[96], 17}, {NULL}};
const Segment SGI281791[] = {{&S51404[13], 3}, {&S51404[17], 8}, {&S51404[26], 2}, {&S51404[29], 6}, {&S51404[36], 6}, {&S51404[64], 17}, {NULL}};
const Segment SGI281792[] = {{&S59679[123], 3}, {NULL}};
const Segment SGI281793[] = {{&S51405[102], 4}, {NULL}};
const Segment SGI281794[] = {{&S59679[13], 5}, {&S59679[19], 2}, {&S59679[22], 14}, {&S59679[37], 6}, {&S59679[44], 31}, {&S59679[97], 17}, {NULL}};
const Segment SGI281795[] = {{&S51405[13], 3}, {&S51405[17], 8}, {&S51405[26], 2}, {&S51405[29], 6}, {&S51405[36], 6}, {&S51405[64], 17}, {NULL}};
const Segment SGI281796[] = {{&S59680[123], 3}, {NULL}};
const Segment SGI281797[] = {{&S51406[102], 4}, {NULL}};
const Segment SGI281798[] = {{&S59680[13], 5}, {&S59680[19], 2}, {&S59680[22], 14}, {&S59680[37], 6}, {&S59680[44], 31}, {&S59680[97], 17}, {NULL}};
const Segment SGI281799[] = {{&S51406[13], 3}, {&S51406[17], 8}, {&S51406[26], 2}, {&S51406[29], 6}, {&S51406[36], 6}, {&S51406[64], 17}, {NULL}};
const Segment SGI281800[] = {{&S59681[123], 3}, {NULL}};
const Segment SGI281801[] = {{&S51407[102], 4}, {NULL}};
const Segment SGI281802[] = {{&S59681[13], 5}, {&S59681[19], 2}, {&S59681[22], 14}, {&S59681[37], 6}, {&S59681[44], 31}, {&S59681[97], 17}, {NULL}};
const Segment SGI281803[] = {{&S51407[13], 3}, {&S51407[17], 8}, {&S51407[26], 2}, {&S51407[29], 6}, {&S51407[36], 6}, {&S51407[64], 17}, {NULL}};
const Segment SGI281804[] = {{&S59682[123], 3}, {NULL}};
const Segment SGI281805[] = {{&S51408[102], 4}, {NULL}};
const Segment SGI281806[] = {{&S59682[13], 5}, {&S59682[19], 2}, {&S59682[22], 14}, {&S59682[37], 6}, {&S59682[44], 31}, {&S59682[97], 17}, {NULL}};
const Segment SGI281807[] = {{&S51408[13], 3}, {&S51408[17], 8}, {&S51408[26], 2}, {&S51408[29], 6}, {&S51408[36], 6}, {&S51408[64], 17}, {NULL}};
const Segment SGI281808[] = {{&S59683[122], 3}, {NULL}};
const Segment SGI281809[] = {{&S51409[102], 4}, {NULL}};
const Segment SGI281810[] = {{&S59683[13], 5}, {&S59683[19], 2}, {&S59683[22], 14}, {&S59683[37], 6}, {&S59683[44], 30}, {&S59683[96], 17}, {NULL}};
const Segment SGI281811[] = {{&S51409[13], 3}, {&S51409[17], 8}, {&S51409[26], 2}, {&S51409[29], 6}, {&S51409[36], 6}, {&S51409[64], 17}, {NULL}};
const Segment SGI281812[] = {{&S59684[123], 3}, {NULL}};
const Segment SGI281813[] = {{&S51410[102], 4}, {NULL}};
const Segment SGI281814[] = {{&S59684[13], 5}, {&S59684[19], 2}, {&S59684[22], 14}, {&S59684[37], 6}, {&S59684[44], 31}, {&S59684[97], 17}, {NULL}};
const Segment SGI281815[] = {{&S51410[13], 3}, {&S51410[17], 8}, {&S51410[26], 2}, {&S51410[29], 6}, {&S51410[36], 6}, {&S51410[64], 17}, {NULL}};
const Segment SGI281816[] = {{&S51411[102], 4}, {NULL}};
const Segment SGI281817[] = {{&S59685[122], 3}, {NULL}};
const Segment SGI281818[] = {{&S51411[13], 3}, {&S51411[17], 8}, {&S51411[26], 2}, {&S51411[29], 6}, {&S51411[36], 6}, {&S51411[64], 17}, {NULL}};
const Segment SGI281819[] = {{&S59685[13], 5}, {&S59685[19], 2}, {&S59685[22], 14}, {&S59685[37], 6}, {&S59685[44], 30}, {&S59685[96], 17}, {NULL}};
const Segment SGI281820[] = {{&S51412[102], 4}, {NULL}};
const Segment SGI281821[] = {{&S59686[122], 3}, {NULL}};
const Segment SGI281822[] = {{&S51412[13], 3}, {&S51412[17], 8}, {&S51412[26], 2}, {&S51412[29], 6}, {&S51412[36], 6}, {&S51412[64], 17}, {NULL}};
const Segment SGI281823[] = {{&S59686[13], 5}, {&S59686[19], 2}, {&S59686[22], 14}, {&S59686[37], 6}, {&S59686[44], 30}, {&S59686[96], 17}, {NULL}};
const Segment SGI281824[] = {{&S51413[102], 4}, {NULL}};
const Segment SGI281825[] = {{&S59687[122], 3}, {NULL}};
const Segment SGI281826[] = {{&S51413[13], 3}, {&S51413[17], 8}, {&S51413[26], 2}, {&S51413[29], 6}, {&S51413[36], 6}, {&S51413[64], 17}, {NULL}};
const Segment SGI281827[] = {{&S59687[13], 5}, {&S59687[19], 2}, {&S59687[22], 14}, {&S59687[37], 6}, {&S59687[44], 30}, {&S59687[96], 17}, {NULL}};
const Segment SGI281828[] = {{&S51414[102], 4}, {NULL}};
const Segment SGI281829[] = {{&S59688[122], 3}, {NULL}};
const Segment SGI281830[] = {{&S51414[13], 3}, {&S51414[17], 8}, {&S51414[26], 2}, {&S51414[29], 6}, {&S51414[36], 6}, {&S51414[64], 17}, {NULL}};
const Segment SGI281831[] = {{&S59688[13], 5}, {&S59688[19], 2}, {&S59688[22], 14}, {&S59688[37], 6}, {&S59688[44], 30}, {&S59688[96], 17}, {NULL}};
const Segment SGI281832[] = {{&S51415[102], 4}, {NULL}};
const Segment SGI281833[] = {{&S51415[13], 3}, {&S51415[17], 8}, {&S51415[26], 2}, {&S51415[29], 6}, {&S51415[36], 6}, {&S51415[64], 17}, {NULL}};
const Segment SGI281834[] = {{&S59689[123], 3}, {NULL}};
const Segment SGI281835[] = {{&S59689[13], 5}, {&S59689[19], 2}, {&S59689[22], 14}, {&S59689[37], 6}, {&S59689[44], 31}, {&S59689[97], 17}, {NULL}};
const Segment SGI281836[] = {{&S59690[122], 3}, {NULL}};
const Segment SGI281837[] = {{&S59690[13], 5}, {&S59690[19], 2}, {&S59690[22], 14}, {&S59690[37], 6}, {&S59690[44], 30}, {&S59690[96], 17}, {NULL}};
const Segment SGI281838[] = {{&S59691[123], 3}, {NULL}};
const Segment SGI281839[] = {{&S59691[13], 5}, {&S59691[19], 2}, {&S59691[22], 14}, {&S59691[37], 6}, {&S59691[44], 31}, {&S59691[97], 17}, {NULL}};
const Segment SGI281840[] = {{&S59692[123], 3}, {NULL}};
const Segment SGI281841[] = {{&S59692[13], 5}, {&S59692[19], 2}, {&S59692[22], 14}, {&S59692[37], 6}, {&S59692[44], 31}, {&S59692[97], 17}, {NULL}};
const Segment SGI281842[] = {{&S59693[123], 3}, {NULL}};
const Segment SGI281843[] = {{&S59693[13], 5}, {&S59693[19], 2}, {&S59693[22], 14}, {&S59693[37], 6}, {&S59693[44], 31}, {&S59693[97], 17}, {NULL}};
const Segment SGI281844[] = {{&S59694[122], 3}, {NULL}};
const Segment SGI281845[] = {{&S59694[13], 5}, {&S59694[19], 2}, {&S59694[22], 14}, {&S59694[37], 6}, {&S59694[44], 30}, {&S59694[96], 17}, {NULL}};
const Segment SGI281846[] = {{&S59695[122], 3}, {NULL}};
const Segment SGI281847[] = {{&S59695[13], 5}, {&S59695[19], 2}, {&S59695[22], 14}, {&S59695[37], 6}, {&S59695[44], 30}, {&S59695[96], 17}, {NULL}};
const Segment SGI281848[] = {{&S59696[123], 3}, {NULL}};
const Segment SGI281849[] = {{&S59696[13], 5}, {&S59696[19], 2}, {&S59696[22], 14}, {&S59696[37], 6}, {&S59696[44], 31}, {&S59696[97], 17}, {NULL}};
const Segment SGI281850[] = {{&S51455[75], 4}, {NULL}};
const Segment SGI281851[] = {{&S51455[13], 14}, {&S51455[49], 17}, {NULL}};
const Segment SGI281852[] = {{&S51455[102], 12}, {NULL}};
const Segment SGI281853[] = {{&S51456[82], 4}, {NULL}};
const Segment SGI281854[] = {{&S51456[13], 14}, {&S51456[28], 6}, {&S51456[56], 17}, {NULL}};
const Segment SGI281855[] = {{&S51456[109], 12}, {NULL}};
const Segment SGI281856[] = {{&S51459[82], 4}, {NULL}};
const Segment SGI281857[] = {{&S51459[13], 14}, {&S51459[28], 6}, {&S51459[56], 17}, {NULL}};
const Segment SGI281858[] = {{&S51459[110], 12}, {NULL}};
const Segment SGI281859[] = {{&S51460[75], 4}, {NULL}};
const Segment SGI281860[] = {{&S51460[13], 14}, {&S51460[49], 17}, {NULL}};
const Segment SGI281861[] = {{&S51460[103], 12}, {NULL}};
const Segment SGI281862[] = {{&S59775[91], 3}, {NULL}};
const Segment SGI281863[] = {{&S59775[13], 7}, {&S59775[21], 2}, {&S59775[24], 15}, {&S59775[40], 6}, {&S59775[67], 17}, {NULL}};
const Segment SGI281864[] = {{&S59775[86], 8}, {NULL}};
const Segment SGI281865[] = {{&S53567[41], 4}, {NULL}};
const Segment SGI281866[] = {{&S53567[13], 14}, {&S53567[28], 25}, {&S53567[74], 17}, {NULL}};
const Segment SGI281867[] = {{&S53567[29], 16}, {NULL}};
const Segment SGI281868[] = {{&S51519[74], 4}, {NULL}};
const Segment SGI281869[] = {{&S51519[13], 14}, {&S51519[48], 17}, {NULL}};
const Segment SGI281870[] = {{&S53570[41], 4}, {NULL}};
const Segment SGI281871[] = {{&S53570[13], 14}, {&S53570[28], 25}, {&S53570[74], 17}, {NULL}};
const Segment SGI281872[] = {{&S53570[29], 16}, {NULL}};
const Segment SGI281873[] = {{&S53571[48], 4}, {NULL}};
const Segment SGI281874[] = {{&S53571[13], 14}, {&S53571[28], 6}, {&S53571[35], 25}, {&S53571[81], 17}, {NULL}};
const Segment SGI281875[] = {{&S53571[36], 16}, {NULL}};
const Segment SGI281876[] = {{&S53574[41], 4}, {NULL}};
const Segment SGI281877[] = {{&S53574[13], 14}, {&S53574[28], 25}, {&S53574[74], 17}, {NULL}};
const Segment SGI281878[] = {{&S53574[29], 16}, {NULL}};
const Segment SGI281879[] = {{&S51562[74], 4}, {NULL}};
const Segment SGI281880[] = {{&S51562[13], 14}, {&S51562[48], 17}, {NULL}};
const Segment SGI281881[] = {{&S51563[106], 4}, {NULL}};
const Segment SGI281882[] = {{&S51563[13], 46}, {&S51563[60], 11}, {&S51563[72], 9}, {&S51563[82], 15}, {NULL}};
const Segment SGI281883[] = {{&S51564[106], 4}, {NULL}};
const Segment SGI281884[] = {{&S51564[13], 46}, {&S51564[60], 11}, {&S51564[72], 9}, {&S51564[82], 15}, {NULL}};
const Segment SGI281885[] = {{&S51565[111], 4}, {NULL}};
const Segment SGI281886[] = {{&S51565[13], 46}, {&S51565[60], 11}, {&S51565[72], 9}, {&S51565[82], 20}, {NULL}};
const Segment SGI281887[] = {{&S51568[75], 4}, {NULL}};
const Segment SGI281888[] = {{&S51568[13], 14}, {&S51568[49], 17}, {NULL}};
const Segment SGI281889[] = {{&S51733[74], 4}, {NULL}};
const Segment SGI281890[] = {{&S51733[13], 14}, {&S51733[48], 17}, {NULL}};
const Segment SGI281891[] = {{&S51733[86], 9}, {NULL}};
const Segment SGI281892[] = {{&S51734[81], 4}, {NULL}};
const Segment SGI281893[] = {{&S51734[13], 14}, {&S51734[28], 6}, {&S51734[55], 17}, {NULL}};
const Segment SGI281894[] = {{&S51734[93], 9}, {NULL}};
const Segment SGI281895[] = {{&S51735[74], 4}, {NULL}};
const Segment SGI281896[] = {{&S51735[13], 14}, {&S51735[48], 17}, {NULL}};
const Segment SGI281897[] = {{&S51735[86], 9}, {NULL}};
const Segment SGI281898[] = {{&S51968[74], 4}, {NULL}};
const Segment SGI281899[] = {{&S51968[13], 14}, {&S51968[48], 17}, {NULL}};
const Segment SGI281900[] = {{&S51968[86], 9}, {NULL}};
const Segment SGI281901[] = {{&S51969[74], 4}, {NULL}};
const Segment SGI281902[] = {{&S51969[13], 14}, {&S51969[48], 17}, {NULL}};
const Segment SGI281903[] = {{&S51969[86], 9}, {NULL}};
const Segment SGI281904[] = {{&S51970[81], 4}, {NULL}};
const Segment SGI281905[] = {{&S51970[13], 14}, {&S51970[28], 6}, {&S51970[55], 17}, {NULL}};
const Segment SGI281906[] = {{&S51970[93], 9}, {NULL}};
const Segment SGI281907[] = {{&S53758[80], 4}, {NULL}};
const Segment SGI281908[] = {{&S53758[13], 14}, {&S53758[28], 6}, {&S53758[56], 17}, {NULL}};
const Segment SGI281909[] = {{&S53758[75], 9}, {NULL}};
const Segment SGI281910[] = {{&S53759[73], 4}, {NULL}};
const Segment SGI281911[] = {{&S53759[13], 14}, {&S53759[49], 17}, {NULL}};
const Segment SGI281912[] = {{&S53759[68], 9}, {NULL}};
const Segment SGI281913[] = {{&S52024[75], 4}, {NULL}};
const Segment SGI281914[] = {{&S52024[13], 14}, {&S52024[49], 17}, {NULL}};
const Segment SGI281915[] = {{&S53799[94], 4}, {NULL}};
const Segment SGI281916[] = {{&S53799[13], 14}, {&S53799[49], 17}, {NULL}};
const Segment SGI281917[] = {{&S53799[68], 11}, {NULL}};
const Segment SGI281918[] = {{&S53800[101], 4}, {NULL}};
const Segment SGI281919[] = {{&S53800[13], 14}, {&S53800[28], 6}, {&S53800[56], 17}, {NULL}};
const Segment SGI281920[] = {{&S53800[75], 11}, {NULL}};
const Segment SGI281921[] = {{&S53861[93], 4}, {NULL}};
const Segment SGI281922[] = {{&S53861[13], 3}, {&S53861[17], 13}, {&S53861[52], 17}, {NULL}};
const Segment SGI281923[] = {{&S53861[71], 12}, {NULL}};
const Segment SGI281924[] = {{&S53862[91], 4}, {NULL}};
const Segment SGI281925[] = {{&S53862[13], 3}, {&S53862[17], 11}, {&S53862[50], 17}, {NULL}};
const Segment SGI281926[] = {{&S53862[69], 12}, {NULL}};
const Segment SGI281927[] = {{&S53904[79], 4}, {NULL}};
const Segment SGI281928[] = {{&S53904[13], 14}, {&S53904[28], 6}, {&S53904[55], 17}, {NULL}};
const Segment SGI281929[] = {{&S53904[74], 9}, {NULL}};
const Segment SGI281930[] = {{&S53905[72], 4}, {NULL}};
const Segment SGI281931[] = {{&S53905[13], 14}, {&S53905[48], 17}, {NULL}};
const Segment SGI281932[] = {{&S53905[67], 9}, {NULL}};
const Segment SGI281933[] = {{&S54045[101], 4}, {NULL}};
const Segment SGI281934[] = {{&S54045[13], 14}, {&S54045[28], 6}, {&S54045[56], 17}, {NULL}};
const Segment SGI281935[] = {{&S54045[75], 11}, {NULL}};
const Segment SGI281936[] = {{&S54046[94], 4}, {NULL}};
const Segment SGI281937[] = {{&S54046[13], 14}, {&S54046[49], 17}, {NULL}};
const Segment SGI281938[] = {{&S54046[68], 11}, {NULL}};
const Segment SGI281939[] = {{&S54047[93], 4}, {NULL}};
const Segment SGI281940[] = {{&S54047[13], 14}, {&S54047[48], 17}, {NULL}};
const Segment SGI281941[] = {{&S54047[67], 11}, {NULL}};
const Segment SGI281942[] = {{&S54048[100], 4}, {NULL}};
const Segment SGI281943[] = {{&S54048[13], 14}, {&S54048[28], 6}, {&S54048[55], 17}, {NULL}};
const Segment SGI281944[] = {{&S54048[74], 11}, {NULL}};
const Segment SGI281945[] = {{&S52208[111], 4}, {NULL}};
const Segment SGI281946[] = {{&S52208[13], 9}, {&S52208[23], 22}, {&S52208[66], 17}, {NULL}};
const Segment SGI281947[] = {{&S52208[85], 11}, {NULL}};
const Segment SGI281948[] = {{&S52209[111], 4}, {NULL}};
const Segment SGI281949[] = {{&S52209[13], 9}, {&S52209[23], 22}, {&S52209[66], 17}, {NULL}};
const Segment SGI281950[] = {{&S52209[85], 11}, {NULL}};
const Segment SGI281951[] = {{&S52210[112], 4}, {NULL}};
const Segment SGI281952[] = {{&S52210[13], 9}, {&S52210[23], 22}, {&S52210[67], 17}, {NULL}};
const Segment SGI281953[] = {{&S52210[86], 11}, {NULL}};
const Segment SGI281954[] = {{&S52211[112], 4}, {NULL}};
const Segment SGI281955[] = {{&S52211[13], 9}, {&S52211[23], 22}, {&S52211[67], 17}, {NULL}};
const Segment SGI281956[] = {{&S52211[86], 11}, {NULL}};
const Segment SGI281957[] = {{&S52236[41], 4}, {NULL}};
const Segment SGI281958[] = {{&S52236[13], 14}, {&S52236[28], 25}, {&S52236[74], 17}, {NULL}};
const Segment SGI281959[] = {{&S52237[48], 4}, {NULL}};
const Segment SGI281960[] = {{&S52237[13], 14}, {&S52237[28], 6}, {&S52237[35], 25}, {&S52237[81], 17}, {NULL}};
const Segment SGI281961[] = {{&S52238[41], 4}, {NULL}};
const Segment SGI281962[] = {{&S52238[13], 14}, {&S52238[28], 25}, {&S52238[74], 17}, {NULL}};
const Segment SGI281963[] = {{&S52239[41], 4}, {NULL}};
const Segment SGI281964[] = {{&S52239[13], 14}, {&S52239[28], 25}, {&S52239[74], 17}, {NULL}};
const Segment SGI281965[] = {{&S52240[48], 4}, {NULL}};
const Segment SGI281966[] = {{&S52240[13], 14}, {&S52240[28], 6}, {&S52240[35], 25}, {&S52240[81], 17}, {NULL}};
const Segment SGI281967[] = {{&S52241[41], 4}, {NULL}};
const Segment SGI281968[] = {{&S52241[13], 14}, {&S52241[28], 25}, {&S52241[74], 17}, {NULL}};
const Segment SGI281969[] = {{&S52244[48], 4}, {NULL}};
const Segment SGI281970[] = {{&S52244[13], 14}, {&S52244[28], 6}, {&S52244[35], 25}, {&S52244[81], 17}, {NULL}};
const Segment SGI281971[] = {{&S52247[41], 4}, {NULL}};
const Segment SGI281972[] = {{&S52247[13], 14}, {&S52247[28], 25}, {&S52247[74], 17}, {NULL}};
const Segment SGI281973[] = {{&S54184[92], 4}, {NULL}};
const Segment SGI281974[] = {{&S54184[13], 3}, {&S54184[17], 13}, {&S54184[51], 17}, {NULL}};
const Segment SGI281975[] = {{&S54184[70], 12}, {NULL}};
const Segment SGI281976[] = {{&S54185[90], 4}, {NULL}};
const Segment SGI281977[] = {{&S54185[13], 3}, {&S54185[17], 11}, {&S54185[49], 17}, {NULL}};
const Segment SGI281978[] = {{&S54185[68], 12}, {NULL}};
const Segment SGI281979[] = {{&S54369[91], 4}, {NULL}};
const Segment SGI281980[] = {{&S54369[13], 3}, {&S54369[17], 11}, {&S54369[50], 17}, {NULL}};
const Segment SGI281981[] = {{&S54369[69], 12}, {NULL}};
const Segment SGI281982[] = {{&S54370[92], 4}, {NULL}};
const Segment SGI281983[] = {{&S54370[13], 3}, {&S54370[17], 13}, {&S54370[51], 17}, {NULL}};
const Segment SGI281984[] = {{&S54370[70], 12}, {NULL}};
const Segment SGI281985[] = {{&S52397[82], 4}, {NULL}};
const Segment SGI281986[] = {{&S52397[13], 14}, {&S52397[28], 6}, {&S52397[56], 17}, {NULL}};
const Segment SGI281987[] = {{&S52397[94], 9}, {NULL}};
const Segment SGI281988[] = {{&S52398[75], 4}, {NULL}};
const Segment SGI281989[] = {{&S52398[13], 14}, {&S52398[49], 17}, {NULL}};
const Segment SGI281990[] = {{&S52398[87], 9}, {NULL}};
const Segment SGI281991[] = {{&S60922[94], 3}, {NULL}};
const Segment SGI281992[] = {{&S60922[13], 7}, {&S60922[21], 2}, {&S60922[24], 15}, {&S60922[40], 6}, {&S60922[68], 17}, {&S60922[118], 31}, {NULL}};
const Segment SGI281993[] = {{&S54425[75], 4}, {NULL}};
const Segment SGI281994[] = {{&S54425[13], 14}, {&S54425[49], 17}, {NULL}};
const Segment SGI281995[] = {{&S54425[111], 4}, {NULL}};
const Segment SGI281996[] = {{&S60979[110], 3}, {NULL}};
const Segment SGI281997[] = {{&S60979[13], 7}, {&S60979[21], 2}, {&S60979[24], 15}, {&S60979[40], 6}, {&S60979[67], 17}, {NULL}};
const Segment SGI281998[] = {{&S60979[86], 9}, {NULL}};
const Segment SGI281999[] = {{&S52648[75], 4}, {NULL}};
const Segment SGI282000[] = {{&S52648[13], 14}, {&S52648[49], 17}, {NULL}};
const Segment SGI282001[] = {{&S54689[112], 4}, {NULL}};
const Segment SGI282002[] = {{&S54689[13], 3}, {&S54689[17], 13}, {&S54689[31], 26}, {&S54689[58], 6}, {&S54689[86], 17}, {NULL}};
const Segment SGI282003[] = {{&S54690[112], 4}, {NULL}};
const Segment SGI282004[] = {{&S54690[13], 3}, {&S54690[17], 13}, {&S54690[31], 26}, {&S54690[58], 6}, {&S54690[86], 17}, {NULL}};
const Segment SGI282005[] = {{&S54691[112], 4}, {NULL}};
const Segment SGI282006[] = {{&S54691[13], 3}, {&S54691[17], 13}, {&S54691[31], 26}, {&S54691[58], 6}, {&S54691[86], 17}, {NULL}};
const Segment SGI282007[] = {{&S54692[112], 4}, {NULL}};
const Segment SGI282008[] = {{&S54692[13], 3}, {&S54692[17], 13}, {&S54692[31], 26}, {&S54692[58], 6}, {&S54692[86], 17}, {NULL}};
const Segment SGI282009[] = {{&S54693[112], 4}, {NULL}};
const Segment SGI282010[] = {{&S54693[13], 3}, {&S54693[17], 13}, {&S54693[31], 26}, {&S54693[58], 6}, {&S54693[86], 17}, {NULL}};
const Segment SGI282011[] = {{&S54694[106], 4}, {NULL}};
const Segment SGI282012[] = {{&S54694[13], 3}, {&S54694[17], 13}, {&S54694[31], 20}, {&S54694[52], 6}, {&S54694[80], 17}, {NULL}};
const Segment SGI282013[] = {{&S54695[96], 4}, {NULL}};
const Segment SGI282014[] = {{&S54695[13], 3}, {&S54695[17], 13}, {&S54695[31], 10}, {&S54695[42], 6}, {&S54695[70], 17}, {NULL}};
const Segment SGI282015[] = {{&S54696[96], 4}, {NULL}};
const Segment SGI282016[] = {{&S54696[13], 3}, {&S54696[17], 13}, {&S54696[31], 10}, {&S54696[42], 6}, {&S54696[70], 17}, {NULL}};
const Segment SGI282017[] = {{&S54697[112], 4}, {NULL}};
const Segment SGI282018[] = {{&S54697[13], 3}, {&S54697[17], 13}, {&S54697[31], 26}, {&S54697[58], 6}, {&S54697[86], 17}, {NULL}};
const Segment SGI282019[] = {{&S54698[112], 4}, {NULL}};
const Segment SGI282020[] = {{&S54698[13], 3}, {&S54698[17], 13}, {&S54698[31], 26}, {&S54698[58], 6}, {&S54698[86], 17}, {NULL}};
const Segment SGI282021[] = {{&S54699[112], 4}, {NULL}};
const Segment SGI282022[] = {{&S54699[13], 3}, {&S54699[17], 13}, {&S54699[31], 26}, {&S54699[58], 6}, {&S54699[86], 17}, {NULL}};
const Segment SGI282023[] = {{&S54700[112], 4}, {NULL}};
const Segment SGI282024[] = {{&S54700[13], 3}, {&S54700[17], 13}, {&S54700[31], 26}, {&S54700[58], 6}, {&S54700[86], 17}, {NULL}};
const Segment SGI282025[] = {{&S54701[112], 4}, {NULL}};
const Segment SGI282026[] = {{&S54701[13], 3}, {&S54701[17], 13}, {&S54701[31], 26}, {&S54701[58], 6}, {&S54701[86], 17}, {NULL}};
const Segment SGI282027[] = {{&S54702[112], 4}, {NULL}};
const Segment SGI282028[] = {{&S54702[13], 3}, {&S54702[17], 13}, {&S54702[31], 26}, {&S54702[58], 6}, {&S54702[86], 17}, {NULL}};
const Segment SGI282029[] = {{&S54703[112], 4}, {NULL}};
const Segment SGI282030[] = {{&S54703[13], 3}, {&S54703[17], 13}, {&S54703[31], 26}, {&S54703[58], 6}, {&S54703[86], 17}, {NULL}};
const Segment SGI282031[] = {{&S54704[112], 4}, {NULL}};
const Segment SGI282032[] = {{&S54704[13], 3}, {&S54704[17], 13}, {&S54704[31], 26}, {&S54704[58], 6}, {&S54704[86], 17}, {NULL}};
const Segment SGI282033[] = {{&S54705[112], 4}, {NULL}};
const Segment SGI282034[] = {{&S54705[13], 3}, {&S54705[17], 13}, {&S54705[31], 26}, {&S54705[58], 6}, {&S54705[86], 17}, {NULL}};
const Segment SGI282035[] = {{&S54706[112], 4}, {NULL}};
const Segment SGI282036[] = {{&S54706[13], 3}, {&S54706[17], 13}, {&S54706[31], 26}, {&S54706[58], 6}, {&S54706[86], 17}, {NULL}};
const Segment SGI282037[] = {{&S54707[112], 4}, {NULL}};
const Segment SGI282038[] = {{&S54707[13], 3}, {&S54707[17], 13}, {&S54707[31], 26}, {&S54707[58], 6}, {&S54707[86], 17}, {NULL}};
const Segment SGI282039[] = {{&S54708[112], 4}, {NULL}};
const Segment SGI282040[] = {{&S54708[13], 3}, {&S54708[17], 13}, {&S54708[31], 26}, {&S54708[58], 6}, {&S54708[86], 17}, {NULL}};
const Segment SGI282041[] = {{&S54709[112], 4}, {NULL}};
const Segment SGI282042[] = {{&S54709[13], 3}, {&S54709[17], 13}, {&S54709[31], 26}, {&S54709[58], 6}, {&S54709[86], 17}, {NULL}};
const Segment SGI282043[] = {{&S54710[112], 4}, {NULL}};
const Segment SGI282044[] = {{&S54710[13], 3}, {&S54710[17], 13}, {&S54710[31], 26}, {&S54710[58], 6}, {&S54710[86], 17}, {NULL}};
const Segment SGI282045[] = {{&S54711[112], 4}, {NULL}};
const Segment SGI282046[] = {{&S54711[13], 3}, {&S54711[17], 13}, {&S54711[31], 26}, {&S54711[58], 6}, {&S54711[86], 17}, {NULL}};
const Segment SGI282047[] = {{&S61471[8], 3}, {NULL}};
const Segment SGI282048[] = {{&S61471[13], 9}, {&S61471[23], 2}, {&S61471[26], 20}, {&S61471[47], 6}, {&S61471[74], 17}, {NULL}};
const Segment SGI282049[] = {{&S61527[91], 3}, {NULL}};
const Segment SGI282050[] = {{&S61527[13], 7}, {&S61527[21], 2}, {&S61527[24], 15}, {&S61527[40], 6}, {&S61527[67], 17}, {NULL}};
const Segment SGI282051[] = {{&S61527[86], 8}, {NULL}};
const Segment SGI282052[] = {{&S61528[91], 3}, {NULL}};
const Segment SGI282053[] = {{&S61528[13], 7}, {&S61528[21], 2}, {&S61528[24], 15}, {&S61528[40], 6}, {&S61528[67], 17}, {NULL}};
const Segment SGI282054[] = {{&S61528[86], 8}, {NULL}};
const Segment SGI282055[] = {{&S53322[81], 4}, {NULL}};
const Segment SGI282056[] = {{&S53322[13], 14}, {&S53322[28], 6}, {&S53322[55], 17}, {NULL}};
const Segment SGI282057[] = {{&S53322[107], 12}, {NULL}};
const Segment SGI282058[] = {{&S53323[74], 4}, {NULL}};
const Segment SGI282059[] = {{&S53323[13], 14}, {&S53323[48], 17}, {NULL}};
const Segment SGI282060[] = {{&S53323[100], 12}, {NULL}};
const Segment SGI282061[] = {{&S53357[74], 4}, {NULL}};
const Segment SGI282062[] = {{&S53357[13], 14}, {&S53357[48], 17}, {NULL}};
const Segment SGI282063[] = {{&S53358[106], 4}, {NULL}};
const Segment SGI282064[] = {{&S53358[13], 46}, {&S53358[60], 11}, {&S53358[72], 9}, {&S53358[82], 15}, {NULL}};
const Segment SGI282065[] = {{&S53359[106], 4}, {NULL}};
const Segment SGI282066[] = {{&S53359[13], 46}, {&S53359[60], 11}, {&S53359[72], 9}, {&S53359[82], 15}, {NULL}};
const Segment SGI282067[] = {{&S53360[111], 4}, {NULL}};
const Segment SGI282068[] = {{&S53360[13], 46}, {&S53360[60], 11}, {&S53360[72], 9}, {&S53360[82], 20}, {NULL}};
const Segment SGI282069[] = {{&S55113[113], 4}, {NULL}};
const Segment SGI282070[] = {{&S55113[13], 9}, {&S55113[23], 22}, {&S55113[66], 17}, {NULL}};
const Segment SGI282071[] = {{&S55113[94], 4}, {NULL}};
const Segment SGI282072[] = {{&S55114[113], 4}, {NULL}};
const Segment SGI282073[] = {{&S55114[13], 9}, {&S55114[23], 22}, {&S55114[66], 17}, {NULL}};
const Segment SGI282074[] = {{&S55114[94], 4}, {NULL}};
const Segment SGI282075[] = {{&S61936[85], 3}, {NULL}};
const Segment SGI282076[] = {{&S61936[13], 3}, {&S61936[17], 2}, {&S61936[20], 11}, {&S61936[32], 6}, {&S61936[59], 17}, {NULL}};
const Segment SGI282077[] = {{&S61937[94], 3}, {NULL}};
const Segment SGI282078[] = {{&S61937[13], 3}, {&S61937[17], 2}, {&S61937[20], 19}, {&S61937[41], 6}, {&S61937[68], 17}, {NULL}};
const Segment SGI282079[] = {{&S61938[119], 3}, {NULL}};
const Segment SGI282080[] = {{&S61938[13], 5}, {&S61938[19], 2}, {&S61938[22], 8}, {&S61938[31], 6}, {&S61938[38], 34}, {&S61938[93], 17}, {NULL}};
const Segment SGI282081[] = {{&S61939[119], 3}, {NULL}};
const Segment SGI282082[] = {{&S61939[13], 5}, {&S61939[19], 2}, {&S61939[22], 8}, {&S61939[31], 6}, {&S61939[38], 34}, {&S61939[93], 17}, {NULL}};
const Segment SGI282083[] = {{&S61940[119], 3}, {NULL}};
const Segment SGI282084[] = {{&S61940[13], 5}, {&S61940[19], 2}, {&S61940[22], 8}, {&S61940[31], 6}, {&S61940[38], 34}, {&S61940[93], 17}, {NULL}};
const Segment SGI282085[] = {{&S55115[113], 4}, {NULL}};
const Segment SGI282086[] = {{&S55115[13], 9}, {&S55115[23], 22}, {&S55115[66], 17}, {NULL}};
const Segment SGI282087[] = {{&S55115[94], 4}, {NULL}};
const Segment SGI282088[] = {{&S62003[92], 3}, {NULL}};
const Segment SGI282089[] = {{&S62003[13], 7}, {&S62003[21], 2}, {&S62003[24], 15}, {&S62003[40], 6}, {&S62003[68], 17}, {NULL}};
const Segment SGI282090[] = {{&S62003[87], 8}, {NULL}};
const Segment SGI282091[] = {{&S62004[92], 3}, {NULL}};
const Segment SGI282092[] = {{&S62004[13], 7}, {&S62004[21], 2}, {&S62004[24], 15}, {&S62004[40], 6}, {&S62004[68], 17}, {NULL}};
const Segment SGI282093[] = {{&S62004[87], 8}, {NULL}};
const Segment SGI282094[] = {{&S62069[91], 3}, {NULL}};
const Segment SGI282095[] = {{&S62069[13], 7}, {&S62069[21], 2}, {&S62069[24], 15}, {&S62069[40], 6}, {&S62069[67], 17}, {NULL}};
const Segment SGI282096[] = {{&S62069[86], 8}, {NULL}};
const Segment SGI282097[] = {{&S62070[91], 3}, {NULL}};
const Segment SGI282098[] = {{&S62070[13], 7}, {&S62070[21], 2}, {&S62070[24], 15}, {&S62070[40], 6}, {&S62070[67], 17}, {NULL}};
const Segment SGI282099[] = {{&S62070[86], 8}, {NULL}};
const Segment SGI282100[] = {{&S53809[75], 4}, {NULL}};
const Segment SGI282101[] = {{&S53809[13], 14}, {&S53809[49], 17}, {NULL}};
const Segment SGI282102[] = {{&S53809[87], 9}, {NULL}};
const Segment SGI282103[] = {{&S53810[75], 4}, {NULL}};
const Segment SGI282104[] = {{&S53810[13], 14}, {&S53810[49], 17}, {NULL}};
const Segment SGI282105[] = {{&S53810[87], 9}, {NULL}};
const Segment SGI282106[] = {{&S53811[82], 4}, {NULL}};
const Segment SGI282107[] = {{&S53811[13], 14}, {&S53811[28], 6}, {&S53811[56], 17}, {NULL}};
const Segment SGI282108[] = {{&S53811[94], 9}, {NULL}};
const Segment SGI282109[] = {{&S53886[75], 4}, {NULL}};
const Segment SGI282110[] = {{&S53886[13], 14}, {&S53886[49], 17}, {NULL}};
const Segment SGI282111[] = {{&S53886[88], 9}, {NULL}};
const Segment SGI282112[] = {{&S53887[82], 4}, {NULL}};
const Segment SGI282113[] = {{&S53887[13], 14}, {&S53887[28], 6}, {&S53887[56], 17}, {NULL}};
const Segment SGI282114[] = {{&S53887[95], 9}, {NULL}};
const Segment SGI282115[] = {{&S53888[75], 4}, {NULL}};
const Segment SGI282116[] = {{&S53888[13], 14}, {&S53888[49], 17}, {NULL}};
const Segment SGI282117[] = {{&S53888[88], 9}, {NULL}};
const Segment SGI282118[] = {{&S55587[72], 4}, {NULL}};
const Segment SGI282119[] = {{&S55587[13], 14}, {&S55587[48], 17}, {NULL}};
const Segment SGI282120[] = {{&S55587[67], 9}, {NULL}};
const Segment SGI282121[] = {{&S55588[72], 4}, {NULL}};
const Segment SGI282122[] = {{&S55588[13], 14}, {&S55588[48], 17}, {NULL}};
const Segment SGI282123[] = {{&S55588[67], 9}, {NULL}};
const Segment SGI282124[] = {{&S55589[79], 4}, {NULL}};
const Segment SGI282125[] = {{&S55589[13], 14}, {&S55589[28], 6}, {&S55589[55], 17}, {NULL}};
const Segment SGI282126[] = {{&S55589[74], 9}, {NULL}};
const Segment SGI282127[] = {{&S62429[93], 3}, {NULL}};
const Segment SGI282128[] = {{&S62429[13], 7}, {&S62429[21], 2}, {&S62429[24], 15}, {&S62429[40], 6}, {&S62429[67], 17}, {NULL}};
const Segment SGI282129[] = {{&S62429[112], 8}, {NULL}};
const Segment SGI282130[] = {{&S54005[73], 4}, {NULL}};
const Segment SGI282131[] = {{&S54005[13], 14}, {&S54005[49], 17}, {NULL}};
const Segment SGI282132[] = {{&S54005[68], 9}, {NULL}};
const Segment SGI282133[] = {{&S54006[73], 4}, {NULL}};
const Segment SGI282134[] = {{&S54006[13], 14}, {&S54006[49], 17}, {NULL}};
const Segment SGI282135[] = {{&S54006[68], 9}, {NULL}};
const Segment SGI282136[] = {{&S54007[80], 4}, {NULL}};
const Segment SGI282137[] = {{&S54007[13], 14}, {&S54007[28], 6}, {&S54007[56], 17}, {NULL}};
const Segment SGI282138[] = {{&S54007[75], 9}, {NULL}};
const Segment SGI282139[] = {{&S55764[84], 4}, {NULL}};
const Segment SGI282140[] = {{&S55764[13], 3}, {&S55764[17], 20}, {&S55764[58], 17}, {NULL}};
const Segment SGI282141[] = {{&S55765[84], 4}, {NULL}};
const Segment SGI282142[] = {{&S55765[13], 3}, {&S55765[17], 20}, {&S55765[58], 17}, {NULL}};
const Segment SGI282143[] = {{&S55766[85], 4}, {NULL}};
const Segment SGI282144[] = {{&S55766[13], 3}, {&S55766[17], 21}, {&S55766[59], 17}, {NULL}};
const Segment SGI282145[] = {{&S55767[85], 4}, {NULL}};
const Segment SGI282146[] = {{&S55767[13], 3}, {&S55767[17], 21}, {&S55767[59], 17}, {NULL}};
const Segment SGI282147[] = {{&S55768[85], 4}, {NULL}};
const Segment SGI282148[] = {{&S55768[13], 3}, {&S55768[17], 21}, {&S55768[59], 17}, {NULL}};
const Segment SGI282149[] = {{&S55769[85], 4}, {NULL}};
const Segment SGI282150[] = {{&S55769[13], 3}, {&S55769[17], 21}, {&S55769[59], 17}, {NULL}};
const Segment SGI282151[] = {{&S55770[84], 4}, {NULL}};
const Segment SGI282152[] = {{&S55770[13], 3}, {&S55770[17], 20}, {&S55770[58], 17}, {NULL}};
const Segment SGI282153[] = {{&S55771[86], 4}, {NULL}};
const Segment SGI282154[] = {{&S55771[13], 3}, {&S55771[17], 22}, {&S55771[60], 17}, {NULL}};
const Segment SGI282155[] = {{&S55801[104], 4}, {NULL}};
const Segment SGI282156[] = {{&S55801[13], 14}, {&S55801[28], 6}, {&S55801[55], 17}, {NULL}};
const Segment SGI282157[] = {{&S55801[74], 18}, {NULL}};
const Segment SGI282158[] = {{&S55802[97], 4}, {NULL}};
const Segment SGI282159[] = {{&S55802[13], 14}, {&S55802[48], 17}, {NULL}};
const Segment SGI282160[] = {{&S55802[67], 18}, {NULL}};
const Segment SGI282161[] = {{&S55871[72], 4}, {NULL}};
const Segment SGI282162[] = {{&S55871[13], 14}, {&S55871[48], 17}, {NULL}};
const Segment SGI282163[] = {{&S55871[67], 9}, {NULL}};
const Segment SGI282164[] = {{&S55872[72], 4}, {NULL}};
const Segment SGI282165[] = {{&S55872[13], 14}, {&S55872[48], 17}, {NULL}};
const Segment SGI282166[] = {{&S55872[67], 9}, {NULL}};
const Segment SGI282167[] = {{&S55873[79], 4}, {NULL}};
const Segment SGI282168[] = {{&S55873[13], 14}, {&S55873[28], 6}, {&S55873[55], 17}, {NULL}};
const Segment SGI282169[] = {{&S55873[74], 9}, {NULL}};
const Segment SGI282170[] = {{&S56026[92], 4}, {NULL}};
const Segment SGI282171[] = {{&S56026[13], 7}, {&S56026[21], 2}, {&S56026[24], 15}, {&S56026[40], 6}, {&S56026[68], 17}, {NULL}};
const Segment SGI282172[] = {{&S56026[87], 9}, {NULL}};
const Segment SGI282173[] = {{&S56067[79], 4}, {NULL}};
const Segment SGI282174[] = {{&S56067[13], 3}, {&S56067[17], 14}, {&S56067[53], 17}, {NULL}};
const Segment SGI282175[] = {{&S56068[78], 4}, {NULL}};
const Segment SGI282176[] = {{&S56068[13], 3}, {&S56068[17], 13}, {&S56068[52], 17}, {NULL}};
const Segment SGI282177[] = {{&S63009[92], 3}, {NULL}};
const Segment SGI282178[] = {{&S63009[13], 7}, {&S63009[21], 2}, {&S63009[24], 15}, {&S63009[40], 6}, {&S63009[68], 17}, {NULL}};
const Segment SGI282179[] = {{&S63009[87], 8}, {NULL}};
const Segment SGI282180[] = {{&S56227[90], 4}, {NULL}};
const Segment SGI282181[] = {{&S56227[13], 3}, {&S56227[17], 11}, {&S56227[49], 17}, {NULL}};
const Segment SGI282182[] = {{&S56227[68], 12}, {NULL}};
const Segment SGI282183[] = {{&S56228[92], 4}, {NULL}};
const Segment SGI282184[] = {{&S56228[13], 3}, {&S56228[17], 13}, {&S56228[51], 17}, {NULL}};
const Segment SGI282185[] = {{&S56228[70], 12}, {NULL}};
const Segment SGI282186[] = {{&S56268[73], 4}, {NULL}};
const Segment SGI282187[] = {{&S56268[13], 14}, {&S56268[49], 17}, {NULL}};
const Segment SGI282188[] = {{&S56268[68], 9}, {NULL}};
const Segment SGI282189[] = {{&S56269[80], 4}, {NULL}};
const Segment SGI282190[] = {{&S56269[13], 14}, {&S56269[28], 6}, {&S56269[56], 17}, {NULL}};
const Segment SGI282191[] = {{&S56269[75], 9}, {NULL}};
const Segment SGI282192[] = {{&S49106[7], 3}, {NULL}};
const Segment SGI282193[] = {{&S49106[11], 5}, {NULL}};
const Segment SGI282194[] = {{&S49107[7], 3}, {NULL}};
const Segment SGI282195[] = {{&S49107[11], 5}, {NULL}};
const Segment SGI282196[] = {{&S49108[7], 3}, {NULL}};
const Segment SGI282197[] = {{&S49108[11], 6}, {NULL}};
const Segment SGI282198[] = {{&S49109[7], 3}, {NULL}};
const Segment SGI282199[] = {{&S49109[11], 5}, {NULL}};
const Segment SGI282200[] = {{&S49110[7], 3}, {NULL}};
const Segment SGI282201[] = {{&S49110[11], 4}, {NULL}};
const Segment SGI282202[] = {{&S49111[7], 3}, {NULL}};
const Segment SGI282203[] = {{&S49111[11], 4}, {NULL}};
const Segment SGI282204[] = {{&S49112[7], 3}, {NULL}};
const Segment SGI282205[] = {{&S49112[11], 4}, {NULL}};
const Segment SGI282206[] = {{&S49113[7], 3}, {NULL}};
const Segment SGI282207[] = {{&S49113[11], 5}, {NULL}};
const Segment SGI282208[] = {{&S49114[7], 3}, {NULL}};
const Segment SGI282209[] = {{&S49114[11], 5}, {NULL}};
const Segment SGI282210[] = {{&S49115[7], 3}, {NULL}};
const Segment SGI282211[] = {{&S49115[11], 4}, {NULL}};
const Segment SGI282212[] = {{&S49116[7], 3}, {NULL}};
const Segment SGI282213[] = {{&S49116[11], 4}, {NULL}};
const Segment SGI282214[] = {{&S49117[7], 3}, {NULL}};
const Segment SGI282215[] = {{&S49117[11], 6}, {NULL}};
const Segment SGI282216[] = {{&S49118[7], 3}, {NULL}};
const Segment SGI282217[] = {{&S49118[11], 4}, {NULL}};
const Segment SGI282218[] = {{&S49119[7], 3}, {NULL}};
const Segment SGI282219[] = {{&S49119[11], 5}, {NULL}};
const Segment SGI282220[] = {{&S49120[7], 3}, {NULL}};
const Segment SGI282221[] = {{&S49120[11], 6}, {NULL}};
const Segment SGI282222[] = {{&S49121[7], 3}, {NULL}};
const Segment SGI282223[] = {{&S49121[11], 5}, {NULL}};
const Segment SGI282224[] = {{&S49122[7], 3}, {NULL}};
const Segment SGI282225[] = {{&S49122[11], 5}, {NULL}};
const Segment SGI282226[] = {{&S49123[7], 3}, {NULL}};
const Segment SGI282227[] = {{&S49123[11], 5}, {NULL}};
const Segment SGI282228[] = {{&S49124[7], 3}, {NULL}};
const Segment SGI282229[] = {{&S49124[11], 5}, {NULL}};
const Segment SGI282230[] = {{&S49125[7], 3}, {NULL}};
const Segment SGI282231[] = {{&S49125[11], 8}, {NULL}};
const Segment SGI282232[] = {{&S49126[7], 3}, {NULL}};
const Segment SGI282233[] = {{&S49126[11], 4}, {NULL}};
const Segment SGI282234[] = {{&S49127[7], 3}, {NULL}};
const Segment SGI282235[] = {{&S49127[11], 5}, {NULL}};
const Segment SGI282236[] = {{&S49128[7], 3}, {NULL}};
const Segment SGI282237[] = {{&S49128[11], 4}, {NULL}};
const Segment SGI282238[] = {{&S49129[7], 3}, {NULL}};
const Segment SGI282239[] = {{&S49129[11], 6}, {NULL}};
const Segment SGI282240[] = {{&S49130[7], 3}, {NULL}};
const Segment SGI282241[] = {{&S49130[11], 4}, {NULL}};
const Segment SGI282242[] = {{&S49131[7], 3}, {NULL}};
const Segment SGI282243[] = {{&S49131[11], 5}, {NULL}};
const Segment SGI282244[] = {{&S49132[7], 3}, {NULL}};
const Segment SGI282245[] = {{&S49132[11], 6}, {NULL}};
const Segment SGI282246[] = {{&S49133[7], 3}, {NULL}};
const Segment SGI282247[] = {{&S49133[11], 4}, {NULL}};
const Segment SGI282248[] = {{&S49134[7], 3}, {NULL}};
const Segment SGI282249[] = {{&S49134[11], 5}, {NULL}};
const Segment SGI282250[] = {{&S49135[7], 3}, {NULL}};
const Segment SGI282251[] = {{&S49135[11], 6}, {NULL}};
const Segment SGI282252[] = {{&S49136[7], 3}, {NULL}};
const Segment SGI282253[] = {{&S49136[11], 6}, {NULL}};
const Segment SGI282254[] = {{&S49137[7], 3}, {NULL}};
const Segment SGI282255[] = {{&S49137[11], 5}, {NULL}};
const Segment SGI282256[] = {{&S49138[7], 3}, {NULL}};
const Segment SGI282257[] = {{&S49138[11], 4}, {NULL}};
const Segment SGI282258[] = {{&S49139[7], 3}, {NULL}};
const Segment SGI282259[] = {{&S49139[11], 4}, {NULL}};
const Segment SGI282260[] = {{&S49140[7], 3}, {NULL}};
const Segment SGI282261[] = {{&S49140[11], 4}, {NULL}};
const Segment SGI282262[] = {{&S49141[7], 3}, {NULL}};
const Segment SGI282263[] = {{&S49141[11], 4}, {NULL}};
const Segment SGI282264[] = {{&S49142[7], 3}, {NULL}};
const Segment SGI282265[] = {{&S49142[11], 5}, {NULL}};
const Segment SGI282266[] = {{&S49143[7], 3}, {NULL}};
const Segment SGI282267[] = {{&S49143[11], 5}, {NULL}};
const Segment SGI282268[] = {{&S49144[7], 3}, {NULL}};
const Segment SGI282269[] = {{&S49144[11], 4}, {NULL}};
const Segment SGI282270[] = {{&S49145[7], 3}, {NULL}};
const Segment SGI282271[] = {{&S49145[11], 6}, {NULL}};
const Segment SGI282272[] = {{&S49146[7], 3}, {NULL}};
const Segment SGI282273[] = {{&S49146[11], 5}, {NULL}};
const Segment SGI282274[] = {{&S49147[7], 3}, {NULL}};
const Segment SGI282275[] = {{&S49147[11], 4}, {NULL}};
const Segment SGI282276[] = {{&S49148[7], 3}, {NULL}};
const Segment SGI282277[] = {{&S49148[11], 4}, {NULL}};
const Segment SGI282278[] = {{&S49149[7], 3}, {NULL}};
const Segment SGI282279[] = {{&S49149[11], 5}, {NULL}};
const Segment SGI282280[] = {{&S49150[7], 3}, {NULL}};
const Segment SGI282281[] = {{&S49150[11], 5}, {NULL}};
const Segment SGI282282[] = {{&S49151[7], 3}, {NULL}};
const Segment SGI282283[] = {{&S49151[11], 4}, {NULL}};
const Segment SGI282284[] = {{&S49152[7], 3}, {NULL}};
const Segment SGI282285[] = {{&S49152[11], 5}, {NULL}};
const Segment SGI282286[] = {{&S49153[7], 3}, {NULL}};
const Segment SGI282287[] = {{&S49153[11], 5}, {NULL}};
const Segment SGI282288[] = {{&S49154[7], 3}, {NULL}};
const Segment SGI282289[] = {{&S49154[11], 5}, {NULL}};
const Segment SGI282290[] = {{&S49155[7], 3}, {NULL}};
const Segment SGI282291[] = {{&S49155[11], 4}, {NULL}};
const Segment SGI282292[] = {{&S49156[7], 3}, {NULL}};
const Segment SGI282293[] = {{&S49156[11], 4}, {NULL}};
const Segment SGI282294[] = {{&S49157[7], 3}, {NULL}};
const Segment SGI282295[] = {{&S49157[11], 4}, {NULL}};
const Segment SGI282296[] = {{&S49158[7], 3}, {NULL}};
const Segment SGI282297[] = {{&S49158[11], 8}, {NULL}};
const Segment SGI282298[] = {{&S49159[7], 3}, {NULL}};
const Segment SGI282299[] = {{&S49159[11], 4}, {NULL}};
const Segment SGI282300[] = {{&S49160[7], 3}, {NULL}};
const Segment SGI282301[] = {{&S49160[11], 6}, {NULL}};
const Segment SGI282302[] = {{&S49161[7], 3}, {NULL}};
const Segment SGI282303[] = {{&S49161[11], 5}, {NULL}};
const Segment SGI282304[] = {{&S49162[7], 3}, {NULL}};
const Segment SGI282305[] = {{&S49162[11], 4}, {NULL}};
const Segment SGI282306[] = {{&S54646[73], 4}, {NULL}};
const Segment SGI282307[] = {{&S54646[13], 14}, {&S54646[49], 17}, {NULL}};
const Segment SGI282308[] = {{&S54646[68], 9}, {NULL}};
const Segment SGI282309[] = {{&S49163[7], 3}, {NULL}};
const Segment SGI282310[] = {{&S49163[11], 4}, {NULL}};
const Segment SGI282311[] = {{&S54647[80], 4}, {NULL}};
const Segment SGI282312[] = {{&S54647[13], 14}, {&S54647[28], 6}, {&S54647[56], 17}, {NULL}};
const Segment SGI282313[] = {{&S49164[7], 3}, {NULL}};
const Segment SGI282314[] = {{&S49164[11], 5}, {NULL}};
const Segment SGI282315[] = {{&S54647[75], 9}, {NULL}};
const Segment SGI282316[] = {{&S49165[7], 3}, {NULL}};
const Segment SGI282317[] = {{&S49165[11], 6}, {NULL}};
const Segment SGI282318[] = {{&S54648[73], 4}, {NULL}};
const Segment SGI282319[] = {{&S54648[13], 14}, {&S54648[49], 17}, {NULL}};
const Segment SGI282320[] = {{&S54648[68], 9}, {NULL}};
const Segment SGI282321[] = {{&S49166[7], 3}, {NULL}};
const Segment SGI282322[] = {{&S49166[11], 6}, {NULL}};
const Segment SGI282323[] = {{&S49167[7], 3}, {NULL}};
const Segment SGI282324[] = {{&S49167[11], 6}, {NULL}};
const Segment SGI282325[] = {{&S49168[7], 3}, {NULL}};
const Segment SGI282326[] = {{&S49168[11], 6}, {NULL}};
const Segment SGI282327[] = {{&S49169[7], 3}, {NULL}};
const Segment SGI282328[] = {{&S49169[11], 4}, {NULL}};
const Segment SGI282329[] = {{&S49170[7], 3}, {NULL}};
const Segment SGI282330[] = {{&S49170[11], 4}, {NULL}};
const Segment SGI282331[] = {{&S49171[7], 3}, {NULL}};
const Segment SGI282332[] = {{&S49171[11], 5}, {NULL}};
const Segment SGI282333[] = {{&S49172[7], 3}, {NULL}};
const Segment SGI282334[] = {{&S49172[11], 4}, {NULL}};
const Segment SGI282335[] = {{&S49173[7], 3}, {NULL}};
const Segment SGI282336[] = {{&S49173[11], 4}, {NULL}};
const Segment SGI282337[] = {{&S49174[7], 3}, {NULL}};
const Segment SGI282338[] = {{&S49174[11], 4}, {NULL}};
const Segment SGI282339[] = {{&S49175[7], 3}, {NULL}};
const Segment SGI282340[] = {{&S49175[11], 4}, {NULL}};
const Segment SGI282341[] = {{&S54845[93], 4}, {NULL}};
const Segment SGI282342[] = {{&S54845[13], 9}, {&S54845[23], 22}, {&S54845[67], 17}, {&S54845[138], 12}, {NULL}};
const Segment SGI282343[] = {{&S54845[120], 8}, {NULL}};
const Segment SGI282344[] = {{&S54846[93], 4}, {NULL}};
const Segment SGI282345[] = {{&S54846[13], 9}, {&S54846[23], 22}, {&S54846[67], 17}, {NULL}};
const Segment SGI282346[] = {{&S54846[120], 8}, {NULL}};
const Segment SGI282347[] = {{&S54847[93], 4}, {NULL}};
const Segment SGI282348[] = {{&S54847[13], 9}, {&S54847[23], 22}, {&S54847[67], 17}, {NULL}};
const Segment SGI282349[] = {{&S54847[121], 8}, {NULL}};
const Segment SGI282350[] = {{&S54848[93], 4}, {NULL}};
const Segment SGI282351[] = {{&S54848[13], 9}, {&S54848[23], 22}, {&S54848[67], 17}, {&S54848[138], 12}, {&S54848[152], 8}, {NULL}};
const Segment SGI282352[] = {{&S54848[120], 8}, {NULL}};
const Segment SGI282353[] = {{&S54849[93], 4}, {NULL}};
const Segment SGI282354[] = {{&S54849[13], 9}, {&S54849[23], 22}, {&S54849[67], 17}, {&S54849[138], 12}, {NULL}};
const Segment SGI282355[] = {{&S54849[120], 8}, {NULL}};
const Segment SGI282356[] = {{&S54850[93], 4}, {NULL}};
const Segment SGI282357[] = {{&S54850[13], 9}, {&S54850[23], 22}, {&S54850[67], 17}, {&S54850[138], 12}, {NULL}};
const Segment SGI282358[] = {{&S54850[120], 8}, {NULL}};
const Segment SGI282359[] = {{&S54851[93], 4}, {NULL}};
const Segment SGI282360[] = {{&S54851[13], 9}, {&S54851[23], 22}, {&S54851[67], 17}, {&S54851[137], 8}, {NULL}};
const Segment SGI282361[] = {{&S54851[120], 8}, {NULL}};
const Segment SGI282362[] = {{&S54852[93], 4}, {NULL}};
const Segment SGI282363[] = {{&S54852[13], 9}, {&S54852[23], 22}, {&S54852[67], 17}, {NULL}};
const Segment SGI282364[] = {{&S54852[120], 8}, {NULL}};
const Segment SGI282365[] = {{&S54853[93], 4}, {NULL}};
const Segment SGI282366[] = {{&S54853[13], 9}, {&S54853[23], 22}, {&S54853[67], 17}, {&S54853[138], 16}, {NULL}};
const Segment SGI282367[] = {{&S54853[120], 8}, {NULL}};
const Segment SGI282368[] = {{&S54854[93], 4}, {NULL}};
const Segment SGI282369[] = {{&S54854[13], 9}, {&S54854[23], 22}, {&S54854[67], 17}, {NULL}};
const Segment SGI282370[] = {{&S54854[120], 8}, {NULL}};
const Segment SGI282371[] = {{&S54855[93], 4}, {NULL}};
const Segment SGI282372[] = {{&S54855[13], 9}, {&S54855[23], 22}, {&S54855[67], 17}, {&S54855[138], 15}, {NULL}};
const Segment SGI282373[] = {{&S54855[120], 8}, {NULL}};
const Segment SGI282374[] = {{&S63564[102], 3}, {NULL}};
const Segment SGI282375[] = {{&S63564[13], 9}, {&S63564[23], 2}, {&S63564[26], 22}, {&S63564[49], 6}, {&S63564[76], 17}, {NULL}};
const Segment SGI282376[] = {{&S63565[102], 3}, {NULL}};
const Segment SGI282377[] = {{&S63565[13], 9}, {&S63565[23], 2}, {&S63565[26], 22}, {&S63565[49], 6}, {&S63565[76], 17}, {NULL}};
const Segment SGI282378[] = {{&S63566[102], 3}, {NULL}};
const Segment SGI282379[] = {{&S63566[13], 9}, {&S63566[23], 2}, {&S63566[26], 22}, {&S63566[49], 6}, {&S63566[76], 17}, {NULL}};
const Segment SGI282380[] = {{&S56725[71], 4}, {NULL}};
const Segment SGI282381[] = {{&S56725[13], 3}, {&S56725[17], 6}, {&S56725[45], 17}, {NULL}};
const Segment SGI282382[] = {{&S56725[106], 3}, {NULL}};
const Segment SGI282383[] = {{&S55106[76], 4}, {NULL}};
const Segment SGI282384[] = {{&S55106[13], 15}, {&S55106[50], 17}, {NULL}};
const Segment SGI282385[] = {{&S55107[74], 4}, {NULL}};
const Segment SGI282386[] = {{&S55107[13], 14}, {&S55107[48], 17}, {NULL}};
const Segment SGI282387[] = {{&S55187[76], 4}, {NULL}};
const Segment SGI282388[] = {{&S55187[13], 15}, {&S55187[50], 17}, {NULL}};
const Segment SGI282389[] = {{&S55378[93], 4}, {NULL}};
const Segment SGI282390[] = {{&S55378[13], 9}, {&S55378[23], 22}, {&S55378[67], 17}, {&S55378[138], 12}, {NULL}};
const Segment SGI282391[] = {{&S55378[120], 8}, {NULL}};
const Segment SGI282392[] = {{&S55379[93], 4}, {NULL}};
const Segment SGI282393[] = {{&S55379[13], 9}, {&S55379[23], 22}, {&S55379[67], 17}, {NULL}};
const Segment SGI282394[] = {{&S55379[120], 8}, {NULL}};
const Segment SGI282395[] = {{&S55380[93], 4}, {NULL}};
const Segment SGI282396[] = {{&S55380[13], 9}, {&S55380[23], 22}, {&S55380[67], 17}, {NULL}};
const Segment SGI282397[] = {{&S55380[120], 8}, {NULL}};
const Segment SGI282398[] = {{&S64020[103], 3}, {NULL}};
const Segment SGI282399[] = {{&S64020[13], 9}, {&S64020[23], 2}, {&S64020[26], 22}, {&S64020[49], 6}, {&S64020[77], 17}, {NULL}};
const Segment SGI282400[] = {{&S64021[103], 3}, {NULL}};
const Segment SGI282401[] = {{&S64021[13], 9}, {&S64021[23], 2}, {&S64021[26], 22}, {&S64021[49], 6}, {&S64021[77], 17}, {NULL}};
const Segment SGI282402[] = {{&S64022[103], 3}, {NULL}};
const Segment SGI282403[] = {{&S64022[13], 9}, {&S64022[23], 2}, {&S64022[26], 22}, {&S64022[49], 6}, {&S64022[77], 17}, {NULL}};
const Segment SGI282404[] = {{&S64023[103], 3}, {NULL}};
const Segment SGI282405[] = {{&S64023[13], 9}, {&S64023[23], 2}, {&S64023[26], 22}, {&S64023[49], 6}, {&S64023[77], 17}, {NULL}};
const Segment SGI282406[] = {{&S64024[103], 3}, {NULL}};
const Segment SGI282407[] = {{&S64024[13], 9}, {&S64024[23], 2}, {&S64024[26], 22}, {&S64024[49], 6}, {&S64024[77], 17}, {NULL}};
const Segment SGI282408[] = {{&S64025[103], 3}, {NULL}};
const Segment SGI282409[] = {{&S64025[13], 9}, {&S64025[23], 2}, {&S64025[26], 22}, {&S64025[49], 6}, {&S64025[77], 17}, {NULL}};
const Segment SGI282410[] = {{&S57117[102], 4}, {NULL}};
const Segment SGI282411[] = {{&S57117[13], 3}, {&S57117[17], 13}, {&S57117[31], 23}, {&S57117[76], 17}, {NULL}};
const Segment SGI282412[] = {{&S57177[93], 4}, {NULL}};
const Segment SGI282413[] = {{&S57177[13], 14}, {&S57177[48], 17}, {NULL}};
const Segment SGI282414[] = {{&S57177[67], 11}, {NULL}};
const Segment SGI282415[] = {{&S57178[100], 4}, {NULL}};
const Segment SGI282416[] = {{&S57178[13], 14}, {&S57178[28], 6}, {&S57178[55], 17}, {NULL}};
const Segment SGI282417[] = {{&S57178[74], 11}, {NULL}};
const Segment SGI282418[] = {{&S57179[94], 4}, {NULL}};
const Segment SGI282419[] = {{&S57179[13], 14}, {&S57179[49], 17}, {NULL}};
const Segment SGI282420[] = {{&S57179[68], 11}, {NULL}};
const Segment SGI282421[] = {{&S57180[101], 4}, {NULL}};
const Segment SGI282422[] = {{&S57180[13], 14}, {&S57180[28], 6}, {&S57180[56], 17}, {NULL}};
const Segment SGI282423[] = {{&S57180[75], 11}, {NULL}};
const Segment SGI282424[] = {{&S55564[93], 4}, {NULL}};
const Segment SGI282425[] = {{&S55564[13], 9}, {&S55564[23], 22}, {&S55564[67], 17}, {&S55564[139], 12}, {NULL}};
const Segment SGI282426[] = {{&S55564[120], 8}, {NULL}};
const Segment SGI282427[] = {{&S55565[93], 4}, {NULL}};
const Segment SGI282428[] = {{&S55565[13], 9}, {&S55565[23], 22}, {&S55565[67], 17}, {NULL}};
const Segment SGI282429[] = {{&S55565[120], 8}, {NULL}};
const Segment SGI282430[] = {{&S55566[93], 4}, {NULL}};
const Segment SGI282431[] = {{&S55566[13], 9}, {&S55566[23], 22}, {&S55566[67], 17}, {NULL}};
const Segment SGI282432[] = {{&S55566[120], 8}, {NULL}};
const Segment SGI282433[] = {{&S57272[90], 4}, {NULL}};
const Segment SGI282434[] = {{&S57272[13], 3}, {&S57272[17], 11}, {&S57272[49], 17}, {NULL}};
const Segment SGI282435[] = {{&S57272[68], 12}, {NULL}};
const Segment SGI282436[] = {{&S57273[92], 4}, {NULL}};
const Segment SGI282437[] = {{&S57273[13], 3}, {&S57273[17], 13}, {&S57273[51], 17}, {NULL}};
const Segment SGI282438[] = {{&S57273[70], 12}, {NULL}};
const Segment SGI282439[] = {{&S57274[13], 3}, {&S57274[17], 11}, {&S57274[49], 17}, {NULL}};
const Segment SGI282440[] = {{&S57274[68], 12}, {NULL}};
const Segment SGI282441[] = {{&S55623[75], 4}, {NULL}};
const Segment SGI282442[] = {{&S55623[13], 14}, {&S55623[49], 17}, {NULL}};
const Segment SGI282443[] = {{&S55623[87], 9}, {NULL}};
const Segment SGI282444[] = {{&S55624[82], 4}, {NULL}};
const Segment SGI282445[] = {{&S55624[13], 14}, {&S55624[28], 6}, {&S55624[56], 17}, {NULL}};
const Segment SGI282446[] = {{&S55624[94], 9}, {NULL}};
const Segment SGI282447[] = {{&S55772[73], 4}, {NULL}};
const Segment SGI282448[] = {{&S55772[13], 14}, {&S55772[49], 17}, {NULL}};
const Segment SGI282449[] = {{&S55772[68], 9}, {NULL}};
const Segment SGI282450[] = {{&S55773[80], 4}, {NULL}};
const Segment SGI282451[] = {{&S55773[13], 14}, {&S55773[28], 6}, {&S55773[56], 17}, {NULL}};
const Segment SGI282452[] = {{&S55773[75], 9}, {NULL}};
const Segment SGI282453[] = {{&S57343[72], 4}, {NULL}};
const Segment SGI282454[] = {{&S57343[13], 14}, {&S57343[48], 17}, {NULL}};
const Segment SGI282455[] = {{&S57343[67], 9}, {NULL}};
const Segment SGI282456[] = {{&S57344[79], 4}, {NULL}};
const Segment SGI282457[] = {{&S57344[13], 14}, {&S57344[28], 6}, {&S57344[55], 17}, {NULL}};
const Segment SGI282458[] = {{&S57344[74], 9}, {NULL}};
const Segment SGI282459[] = {{&S57545[102], 4}, {NULL}};
const Segment SGI282460[] = {{&S57545[13], 3}, {&S57545[17], 13}, {&S57545[31], 16}, {&S57545[48], 6}, {&S57545[76], 17}, {NULL}};
const Segment SGI282461[] = {{&S57546[102], 4}, {NULL}};
const Segment SGI282462[] = {{&S57546[13], 3}, {&S57546[17], 13}, {&S57546[31], 16}, {&S57546[48], 6}, {&S57546[76], 17}, {NULL}};
const Segment SGI282463[] = {{&S57547[102], 4}, {NULL}};
const Segment SGI282464[] = {{&S57547[13], 3}, {&S57547[17], 13}, {&S57547[31], 16}, {&S57547[48], 6}, {&S57547[76], 17}, {NULL}};
const Segment SGI282465[] = {{&S57548[102], 4}, {NULL}};
const Segment SGI282466[] = {{&S57548[13], 3}, {&S57548[17], 13}, {&S57548[31], 16}, {&S57548[48], 6}, {&S57548[76], 17}, {NULL}};
const Segment SGI282467[] = {{&S57549[102], 4}, {NULL}};
const Segment SGI282468[] = {{&S57549[13], 3}, {&S57549[17], 13}, {&S57549[31], 16}, {&S57549[48], 6}, {&S57549[76], 17}, {NULL}};
const Segment SGI282469[] = {{&S57550[102], 4}, {NULL}};
const Segment SGI282470[] = {{&S57550[13], 3}, {&S57550[17], 13}, {&S57550[31], 16}, {&S57550[48], 6}, {&S57550[76], 17}, {NULL}};
const Segment SGI282471[] = {{&S57551[108], 4}, {NULL}};
const Segment SGI282472[] = {{&S57551[13], 3}, {&S57551[17], 13}, {&S57551[31], 22}, {&S57551[54], 6}, {&S57551[82], 17}, {NULL}};
const Segment SGI282473[] = {{&S57552[102], 4}, {NULL}};
const Segment SGI282474[] = {{&S57552[13], 3}, {&S57552[17], 13}, {&S57552[31], 16}, {&S57552[48], 6}, {&S57552[76], 17}, {NULL}};
const Segment SGI282475[] = {{&S57553[102], 4}, {NULL}};
const Segment SGI282476[] = {{&S57553[13], 3}, {&S57553[17], 13}, {&S57553[31], 16}, {&S57553[48], 6}, {&S57553[76], 17}, {NULL}};
const Segment SGI282477[] = {{&S57554[102], 4}, {NULL}};
const Segment SGI282478[] = {{&S57554[13], 3}, {&S57554[17], 13}, {&S57554[31], 16}, {&S57554[48], 6}, {&S57554[76], 17}, {NULL}};
const Segment SGI282479[] = {{&S57555[102], 4}, {NULL}};
const Segment SGI282480[] = {{&S57555[13], 3}, {&S57555[17], 13}, {&S57555[31], 16}, {&S57555[48], 6}, {&S57555[76], 17}, {NULL}};
const Segment SGI282481[] = {{&S57556[108], 4}, {NULL}};
const Segment SGI282482[] = {{&S57556[13], 3}, {&S57556[17], 13}, {&S57556[31], 22}, {&S57556[54], 6}, {&S57556[82], 17}, {NULL}};
const Segment SGI282483[] = {{&S56117[75], 4}, {NULL}};
const Segment SGI282484[] = {{&S56117[13], 14}, {&S56117[49], 17}, {NULL}};
const Segment SGI282485[] = {{&S56117[87], 9}, {NULL}};
const Segment SGI282486[] = {{&S56118[82], 4}, {NULL}};
const Segment SGI282487[] = {{&S56118[13], 14}, {&S56118[28], 6}, {&S56118[56], 17}, {NULL}};
const Segment SGI282488[] = {{&S56118[94], 9}, {NULL}};
const Segment SGI282489[] = {{&S57614[71], 4}, {NULL}};
const Segment SGI282490[] = {{&S57614[13], 10}, {&S57614[45], 17}, {NULL}};
const Segment SGI282491[] = {{&S57615[70], 4}, {NULL}};
const Segment SGI282492[] = {{&S57615[13], 10}, {&S57615[45], 16}, {NULL}};
const Segment SGI282493[] = {{&S56323[136], 4}, {NULL}};
const Segment SGI282494[] = {{&S56323[13], 3}, {&S56323[17], 13}, {&S56323[51], 17}, {&S56323[81], 44}, {NULL}};
const Segment SGI282495[] = {{&S56324[136], 4}, {NULL}};
const Segment SGI282496[] = {{&S56324[13], 3}, {&S56324[17], 13}, {&S56324[51], 17}, {&S56324[81], 44}, {NULL}};
const Segment SGI282497[] = {{&S56325[136], 4}, {NULL}};
const Segment SGI282498[] = {{&S56325[13], 3}, {&S56325[17], 13}, {&S56325[51], 17}, {&S56325[81], 44}, {NULL}};
const Segment SGI282499[] = {{&S56326[136], 4}, {NULL}};
const Segment SGI282500[] = {{&S56326[13], 3}, {&S56326[17], 13}, {&S56326[51], 17}, {&S56326[81], 44}, {NULL}};
const Segment SGI282501[] = {{&S56327[136], 4}, {NULL}};
const Segment SGI282502[] = {{&S56327[13], 3}, {&S56327[17], 13}, {&S56327[51], 17}, {&S56327[81], 44}, {NULL}};
const Segment SGI282503[] = {{&S56328[136], 4}, {NULL}};
const Segment SGI282504[] = {{&S56328[13], 3}, {&S56328[17], 13}, {&S56328[51], 17}, {&S56328[81], 44}, {NULL}};
const Segment SGI282505[] = {{&S56329[136], 4}, {NULL}};
const Segment SGI282506[] = {{&S56329[13], 3}, {&S56329[17], 13}, {&S56329[51], 17}, {&S56329[81], 44}, {NULL}};
const Segment SGI282507[] = {{&S56330[136], 4}, {NULL}};
const Segment SGI282508[] = {{&S56330[13], 3}, {&S56330[17], 13}, {&S56330[51], 17}, {&S56330[81], 44}, {NULL}};
const Segment SGI282509[] = {{&S56331[136], 4}, {NULL}};
const Segment SGI282510[] = {{&S56331[13], 3}, {&S56331[17], 13}, {&S56331[51], 17}, {&S56331[81], 44}, {NULL}};
const Segment SGI282511[] = {{&S56332[136], 4}, {NULL}};
const Segment SGI282512[] = {{&S56332[13], 3}, {&S56332[17], 13}, {&S56332[51], 17}, {&S56332[81], 44}, {NULL}};
const Segment SGI282513[] = {{&S56333[136], 4}, {NULL}};
const Segment SGI282514[] = {{&S56333[13], 3}, {&S56333[17], 13}, {&S56333[51], 17}, {&S56333[81], 44}, {NULL}};
const Segment SGI282515[] = {{&S56334[136], 4}, {NULL}};
const Segment SGI282516[] = {{&S56334[13], 3}, {&S56334[17], 13}, {&S56334[51], 17}, {&S56334[81], 44}, {NULL}};
const Segment SGI282517[] = {{&S56335[136], 4}, {NULL}};
const Segment SGI282518[] = {{&S56335[13], 3}, {&S56335[17], 13}, {&S56335[51], 17}, {&S56335[81], 44}, {NULL}};
const Segment SGI282519[] = {{&S56336[136], 4}, {NULL}};
const Segment SGI282520[] = {{&S56336[13], 3}, {&S56336[17], 13}, {&S56336[51], 17}, {&S56336[81], 44}, {NULL}};
const Segment SGI282521[] = {{&S56337[136], 4}, {NULL}};
const Segment SGI282522[] = {{&S56337[13], 3}, {&S56337[17], 13}, {&S56337[51], 17}, {&S56337[81], 44}, {NULL}};
const Segment SGI282523[] = {{&S56338[136], 4}, {NULL}};
const Segment SGI282524[] = {{&S56338[13], 3}, {&S56338[17], 13}, {&S56338[51], 17}, {&S56338[81], 44}, {NULL}};
const Segment SGI282525[] = {{&S56339[136], 4}, {NULL}};
const Segment SGI282526[] = {{&S56339[13], 3}, {&S56339[17], 13}, {&S56339[51], 17}, {&S56339[81], 44}, {NULL}};
const Segment SGI282527[] = {{&S56340[136], 4}, {NULL}};
const Segment SGI282528[] = {{&S56340[13], 3}, {&S56340[17], 13}, {&S56340[51], 17}, {&S56340[81], 44}, {NULL}};
const Segment SGI282529[] = {{&S56341[136], 4}, {NULL}};
const Segment SGI282530[] = {{&S56341[13], 3}, {&S56341[17], 13}, {&S56341[51], 17}, {&S56341[81], 44}, {NULL}};
const Segment SGI282531[] = {{&S56342[136], 4}, {NULL}};
const Segment SGI282532[] = {{&S56342[13], 3}, {&S56342[17], 13}, {&S56342[51], 17}, {&S56342[81], 44}, {NULL}};
const Segment SGI282533[] = {{&S56343[136], 4}, {NULL}};
const Segment SGI282534[] = {{&S56343[13], 3}, {&S56343[17], 13}, {&S56343[51], 17}, {&S56343[81], 44}, {NULL}};
const Segment SGI282535[] = {{&S56344[136], 4}, {NULL}};
const Segment SGI282536[] = {{&S56344[13], 3}, {&S56344[17], 13}, {&S56344[51], 17}, {&S56344[81], 44}, {NULL}};
const Segment SGI282537[] = {{&S56345[136], 4}, {NULL}};
const Segment SGI282538[] = {{&S56345[13], 3}, {&S56345[17], 13}, {&S56345[51], 17}, {&S56345[81], 44}, {NULL}};
const Segment SGI282539[] = {{&S56346[136], 4}, {NULL}};
const Segment SGI282540[] = {{&S56346[13], 3}, {&S56346[17], 13}, {&S56346[51], 17}, {&S56346[81], 44}, {NULL}};
const Segment SGI282541[] = {{&S56347[136], 4}, {NULL}};
const Segment SGI282542[] = {{&S56347[13], 3}, {&S56347[17], 13}, {&S56347[51], 17}, {&S56347[81], 44}, {NULL}};
const Segment SGI282543[] = {{&S56348[136], 4}, {NULL}};
const Segment SGI282544[] = {{&S56348[13], 3}, {&S56348[17], 13}, {&S56348[51], 17}, {&S56348[81], 44}, {NULL}};
const Segment SGI282545[] = {{&S56349[136], 4}, {NULL}};
const Segment SGI282546[] = {{&S56349[13], 3}, {&S56349[17], 13}, {&S56349[51], 17}, {&S56349[81], 44}, {NULL}};
const Segment SGI282547[] = {{&S56350[136], 4}, {NULL}};
const Segment SGI282548[] = {{&S56350[13], 3}, {&S56350[17], 13}, {&S56350[51], 17}, {&S56350[81], 44}, {NULL}};
const Segment SGI282549[] = {{&S56351[136], 4}, {NULL}};
const Segment SGI282550[] = {{&S56351[13], 3}, {&S56351[17], 13}, {&S56351[51], 17}, {&S56351[81], 44}, {NULL}};
const Segment SGI282551[] = {{&S56352[136], 4}, {NULL}};
const Segment SGI282552[] = {{&S56352[13], 3}, {&S56352[17], 13}, {&S56352[51], 17}, {&S56352[81], 44}, {NULL}};
const Segment SGI282553[] = {{&S56353[136], 4}, {NULL}};
const Segment SGI282554[] = {{&S56353[13], 3}, {&S56353[17], 13}, {&S56353[51], 17}, {&S56353[81], 44}, {NULL}};
const Segment SGI282555[] = {{&S56354[136], 4}, {NULL}};
const Segment SGI282556[] = {{&S56354[13], 3}, {&S56354[17], 13}, {&S56354[51], 17}, {&S56354[81], 44}, {NULL}};
const Segment SGI282557[] = {{&S56355[136], 4}, {NULL}};
const Segment SGI282558[] = {{&S56355[13], 3}, {&S56355[17], 13}, {&S56355[51], 17}, {&S56355[81], 44}, {NULL}};
const Segment SGI282559[] = {{&S56356[136], 4}, {NULL}};
const Segment SGI282560[] = {{&S56356[13], 3}, {&S56356[17], 13}, {&S56356[51], 17}, {&S56356[81], 44}, {NULL}};
const Segment SGI282561[] = {{&S56357[136], 4}, {NULL}};
const Segment SGI282562[] = {{&S56357[13], 3}, {&S56357[17], 13}, {&S56357[51], 17}, {&S56357[81], 44}, {NULL}};
const Segment SGI282563[] = {{&S56358[136], 4}, {NULL}};
const Segment SGI282564[] = {{&S56358[13], 3}, {&S56358[17], 13}, {&S56358[51], 17}, {&S56358[81], 44}, {NULL}};
const Segment SGI282565[] = {{&S56359[136], 4}, {NULL}};
const Segment SGI282566[] = {{&S56359[13], 3}, {&S56359[17], 13}, {&S56359[51], 17}, {&S56359[81], 44}, {NULL}};
const Segment SGI282567[] = {{&S56360[136], 4}, {NULL}};
const Segment SGI282568[] = {{&S56360[13], 3}, {&S56360[17], 13}, {&S56360[51], 17}, {&S56360[81], 44}, {NULL}};
const Segment SGI282569[] = {{&S56361[136], 4}, {NULL}};
const Segment SGI282570[] = {{&S56361[13], 3}, {&S56361[17], 13}, {&S56361[51], 17}, {&S56361[81], 44}, {NULL}};
const Segment SGI282571[] = {{&S56362[136], 4}, {NULL}};
const Segment SGI282572[] = {{&S56362[13], 3}, {&S56362[17], 13}, {&S56362[51], 17}, {&S56362[81], 44}, {NULL}};
const Segment SGI282573[] = {{&S64895[102], 3}, {NULL}};
const Segment SGI282574[] = {{&S64895[13], 3}, {&S64895[17], 2}, {&S64895[20], 13}, {&S64895[34], 6}, {&S64895[61], 17}, {NULL}};
const Segment SGI282575[] = {{&S64895[80], 12}, {NULL}};
const Segment SGI282576[] = {{&S64896[100], 3}, {NULL}};
const Segment SGI282577[] = {{&S64896[13], 3}, {&S64896[17], 2}, {&S64896[20], 11}, {&S64896[32], 6}, {&S64896[59], 17}, {NULL}};
const Segment SGI282578[] = {{&S64896[78], 12}, {NULL}};
const Segment SGI282579[] = {{&S57837[86], 4}, {NULL}};
const Segment SGI282580[] = {{&S57837[21], 3}, {&S57837[25], 13}, {&S57837[60], 17}, {NULL}};
const Segment SGI282581[] = {{&S57837[0], 8}, {NULL}};
const Segment SGI282582[] = {{&S56363[136], 4}, {NULL}};
const Segment SGI282583[] = {{&S56363[13], 3}, {&S56363[17], 13}, {&S56363[51], 17}, {&S56363[81], 44}, {NULL}};
const Segment SGI282584[] = {{&S56364[136], 4}, {NULL}};
const Segment SGI282585[] = {{&S56364[13], 3}, {&S56364[17], 13}, {&S56364[51], 17}, {&S56364[81], 44}, {NULL}};
const Segment SGI282586[] = {{&S56365[136], 4}, {NULL}};
const Segment SGI282587[] = {{&S56365[13], 3}, {&S56365[17], 13}, {&S56365[51], 17}, {&S56365[81], 44}, {NULL}};
const Segment SGI282588[] = {{&S56366[136], 4}, {NULL}};
const Segment SGI282589[] = {{&S56366[13], 3}, {&S56366[17], 13}, {&S56366[51], 17}, {&S56366[81], 44}, {NULL}};
const Segment SGI282590[] = {{&S56367[136], 4}, {NULL}};
const Segment SGI282591[] = {{&S56367[13], 3}, {&S56367[17], 13}, {&S56367[51], 17}, {&S56367[81], 44}, {NULL}};
const Segment SGI282592[] = {{&S56368[136], 4}, {NULL}};
const Segment SGI282593[] = {{&S56368[13], 3}, {&S56368[17], 13}, {&S56368[51], 17}, {&S56368[81], 44}, {NULL}};
const Segment SGI282594[] = {{&S56369[136], 4}, {NULL}};
const Segment SGI282595[] = {{&S56369[13], 3}, {&S56369[17], 13}, {&S56369[51], 17}, {&S56369[81], 44}, {NULL}};
const Segment SGI282596[] = {{&S56370[136], 4}, {NULL}};
const Segment SGI282597[] = {{&S56370[13], 3}, {&S56370[17], 13}, {&S56370[51], 17}, {&S56370[81], 44}, {NULL}};
const Segment SGI282598[] = {{&S56371[136], 4}, {NULL}};
const Segment SGI282599[] = {{&S56371[13], 3}, {&S56371[17], 13}, {&S56371[51], 17}, {&S56371[81], 44}, {NULL}};
const Segment SGI282600[] = {{&S56372[136], 4}, {NULL}};
const Segment SGI282601[] = {{&S56372[13], 3}, {&S56372[17], 13}, {&S56372[51], 17}, {&S56372[81], 44}, {NULL}};
const Segment SGI282602[] = {{&S56373[136], 4}, {NULL}};
const Segment SGI282603[] = {{&S56373[13], 3}, {&S56373[17], 13}, {&S56373[51], 17}, {&S56373[81], 44}, {NULL}};
const Segment SGI282604[] = {{&S56374[136], 4}, {NULL}};
const Segment SGI282605[] = {{&S56374[13], 3}, {&S56374[17], 13}, {&S56374[51], 17}, {&S56374[81], 44}, {NULL}};
const Segment SGI282606[] = {{&S56375[136], 4}, {NULL}};
const Segment SGI282607[] = {{&S56375[13], 3}, {&S56375[17], 13}, {&S56375[51], 17}, {&S56375[81], 44}, {NULL}};
const Segment SGI282608[] = {{&S56376[136], 4}, {NULL}};
const Segment SGI282609[] = {{&S56376[13], 3}, {&S56376[17], 13}, {&S56376[51], 17}, {&S56376[81], 44}, {NULL}};
const Segment SGI282610[] = {{&S56377[136], 4}, {NULL}};
const Segment SGI282611[] = {{&S56377[13], 3}, {&S56377[17], 13}, {&S56377[51], 17}, {&S56377[81], 44}, {NULL}};
const Segment SGI282612[] = {{&S56378[136], 4}, {NULL}};
const Segment SGI282613[] = {{&S56378[13], 3}, {&S56378[17], 13}, {&S56378[51], 17}, {&S56378[81], 44}, {NULL}};
const Segment SGI282614[] = {{&S56379[136], 4}, {NULL}};
const Segment SGI282615[] = {{&S56379[13], 3}, {&S56379[17], 13}, {&S56379[51], 17}, {&S56379[81], 44}, {NULL}};
const Segment SGI282616[] = {{&S56380[136], 4}, {NULL}};
const Segment SGI282617[] = {{&S56380[13], 3}, {&S56380[17], 13}, {&S56380[51], 17}, {&S56380[81], 44}, {NULL}};
const Segment SGI282618[] = {{&S56381[136], 4}, {NULL}};
const Segment SGI282619[] = {{&S56381[13], 3}, {&S56381[17], 13}, {&S56381[51], 17}, {&S56381[81], 44}, {NULL}};
const Segment SGI282620[] = {{&S56382[136], 4}, {NULL}};
const Segment SGI282621[] = {{&S56382[13], 3}, {&S56382[17], 13}, {&S56382[51], 17}, {&S56382[81], 44}, {NULL}};
const Segment SGI282622[] = {{&S56383[136], 4}, {NULL}};
const Segment SGI282623[] = {{&S56383[13], 3}, {&S56383[17], 13}, {&S56383[51], 17}, {&S56383[81], 44}, {NULL}};
const Segment SGI282624[] = {{&S56384[136], 4}, {NULL}};
const Segment SGI282625[] = {{&S56384[13], 3}, {&S56384[17], 13}, {&S56384[51], 17}, {&S56384[81], 44}, {NULL}};
const Segment SGI282626[] = {{&S56385[136], 4}, {NULL}};
const Segment SGI282627[] = {{&S56385[13], 3}, {&S56385[17], 13}, {&S56385[51], 17}, {&S56385[81], 44}, {NULL}};
const Segment SGI282628[] = {{&S56386[136], 4}, {NULL}};
const Segment SGI282629[] = {{&S56386[13], 3}, {&S56386[17], 13}, {&S56386[51], 17}, {&S56386[81], 44}, {NULL}};
const Segment SGI282630[] = {{&S56387[136], 4}, {NULL}};
const Segment SGI282631[] = {{&S56387[13], 3}, {&S56387[17], 13}, {&S56387[51], 17}, {&S56387[81], 44}, {NULL}};
const Segment SGI282632[] = {{&S65010[101], 3}, {NULL}};
const Segment SGI282633[] = {{&S65010[13], 3}, {&S65010[17], 2}, {&S65010[20], 11}, {&S65010[32], 6}, {&S65010[60], 17}, {NULL}};
const Segment SGI282634[] = {{&S65010[79], 12}, {NULL}};
const Segment SGI282635[] = {{&S56388[136], 4}, {NULL}};
const Segment SGI282636[] = {{&S56388[13], 3}, {&S56388[17], 13}, {&S56388[51], 17}, {&S56388[81], 44}, {NULL}};
const Segment SGI282637[] = {{&S65011[103], 3}, {NULL}};
const Segment SGI282638[] = {{&S65011[13], 3}, {&S65011[17], 2}, {&S65011[20], 13}, {&S65011[34], 6}, {&S65011[62], 17}, {NULL}};
const Segment SGI282639[] = {{&S56389[136], 4}, {NULL}};
const Segment SGI282640[] = {{&S65011[81], 12}, {NULL}};
const Segment SGI282641[] = {{&S56389[13], 3}, {&S56389[17], 13}, {&S56389[51], 17}, {&S56389[81], 44}, {NULL}};
const Segment SGI282642[] = {{&S56390[136], 4}, {NULL}};
const Segment SGI282643[] = {{&S56390[13], 3}, {&S56390[17], 13}, {&S56390[51], 17}, {&S56390[81], 44}, {NULL}};
const Segment SGI282644[] = {{&S56391[136], 4}, {NULL}};
const Segment SGI282645[] = {{&S56391[13], 3}, {&S56391[17], 13}, {&S56391[51], 17}, {&S56391[81], 44}, {NULL}};
const Segment SGI282646[] = {{&S56392[136], 4}, {NULL}};
const Segment SGI282647[] = {{&S56392[13], 3}, {&S56392[17], 13}, {&S56392[51], 17}, {&S56392[81], 44}, {NULL}};
const Segment SGI282648[] = {{&S56393[136], 4}, {NULL}};
const Segment SGI282649[] = {{&S56393[13], 3}, {&S56393[17], 13}, {&S56393[51], 17}, {&S56393[81], 44}, {NULL}};
const Segment SGI282650[] = {{&S56394[136], 4}, {NULL}};
const Segment SGI282651[] = {{&S56394[13], 3}, {&S56394[17], 13}, {&S56394[51], 17}, {&S56394[81], 44}, {NULL}};
const Segment SGI282652[] = {{&S56395[136], 4}, {NULL}};
const Segment SGI282653[] = {{&S56395[13], 3}, {&S56395[17], 13}, {&S56395[51], 17}, {&S56395[81], 44}, {NULL}};
const Segment SGI282654[] = {{&S56396[136], 4}, {NULL}};
const Segment SGI282655[] = {{&S56396[13], 3}, {&S56396[17], 13}, {&S56396[51], 17}, {&S56396[81], 44}, {NULL}};
const Segment SGI282656[] = {{&S56397[136], 4}, {NULL}};
const Segment SGI282657[] = {{&S56397[13], 3}, {&S56397[17], 13}, {&S56397[51], 17}, {&S56397[81], 44}, {NULL}};
const Segment SGI282658[] = {{&S56398[136], 4}, {NULL}};
const Segment SGI282659[] = {{&S56398[13], 3}, {&S56398[17], 13}, {&S56398[51], 17}, {&S56398[81], 44}, {NULL}};
const Segment SGI282660[] = {{&S56399[136], 4}, {NULL}};
const Segment SGI282661[] = {{&S56399[13], 3}, {&S56399[17], 13}, {&S56399[51], 17}, {&S56399[81], 44}, {NULL}};
const Segment SGI282662[] = {{&S56400[136], 4}, {NULL}};
const Segment SGI282663[] = {{&S56400[13], 3}, {&S56400[17], 13}, {&S56400[51], 17}, {&S56400[81], 44}, {NULL}};
const Segment SGI282664[] = {{&S56401[136], 4}, {NULL}};
const Segment SGI282665[] = {{&S56401[13], 3}, {&S56401[17], 13}, {&S56401[51], 17}, {&S56401[81], 44}, {NULL}};
const Segment SGI282666[] = {{&S56402[136], 4}, {NULL}};
const Segment SGI282667[] = {{&S56402[13], 3}, {&S56402[17], 13}, {&S56402[51], 17}, {&S56402[81], 44}, {NULL}};
const Segment SGI282668[] = {{&S56403[136], 4}, {NULL}};
const Segment SGI282669[] = {{&S56403[13], 3}, {&S56403[17], 13}, {&S56403[51], 17}, {&S56403[81], 44}, {NULL}};
const Segment SGI282670[] = {{&S56404[136], 4}, {NULL}};
const Segment SGI282671[] = {{&S65039[100], 3}, {NULL}};
const Segment SGI282672[] = {{&S56404[13], 3}, {&S56404[17], 13}, {&S56404[51], 17}, {&S56404[81], 44}, {NULL}};
const Segment SGI282673[] = {{&S65039[13], 7}, {&S65039[21], 2}, {&S65039[24], 15}, {&S65039[40], 6}, {&S65039[67], 17}, {NULL}};
const Segment SGI282674[] = {{&S65039[86], 17}, {NULL}};
const Segment SGI282675[] = {{&S65040[100], 3}, {NULL}};
const Segment SGI282676[] = {{&S65040[13], 7}, {&S65040[21], 2}, {&S65040[24], 15}, {&S65040[40], 6}, {&S65040[67], 17}, {NULL}};
const Segment SGI282677[] = {{&S65040[86], 17}, {NULL}};
const Segment SGI282678[] = {{&S56405[136], 4}, {NULL}};
const Segment SGI282679[] = {{&S56405[13], 3}, {&S56405[17], 13}, {&S56405[51], 17}, {&S56405[81], 44}, {NULL}};
const Segment SGI282680[] = {{&S56406[136], 4}, {NULL}};
const Segment SGI282681[] = {{&S56406[13], 3}, {&S56406[17], 13}, {&S56406[51], 17}, {&S56406[81], 44}, {NULL}};
const Segment SGI282682[] = {{&S56407[136], 4}, {NULL}};
const Segment SGI282683[] = {{&S56407[13], 3}, {&S56407[17], 13}, {&S56407[51], 17}, {&S56407[81], 44}, {NULL}};
const Segment SGI282684[] = {{&S56408[136], 4}, {NULL}};
const Segment SGI282685[] = {{&S56408[13], 3}, {&S56408[17], 13}, {&S56408[51], 17}, {&S56408[81], 44}, {NULL}};
const Segment SGI282686[] = {{&S56409[136], 4}, {NULL}};
const Segment SGI282687[] = {{&S56409[13], 3}, {&S56409[17], 13}, {&S56409[51], 17}, {&S56409[81], 44}, {NULL}};
const Segment SGI282688[] = {{&S56410[136], 4}, {NULL}};
const Segment SGI282689[] = {{&S56410[13], 3}, {&S56410[17], 13}, {&S56410[51], 17}, {&S56410[81], 44}, {NULL}};
const Segment SGI282690[] = {{&S56411[136], 4}, {NULL}};
const Segment SGI282691[] = {{&S56411[13], 3}, {&S56411[17], 13}, {&S56411[51], 17}, {&S56411[81], 44}, {NULL}};
const Segment SGI282692[] = {{&S56412[136], 4}, {NULL}};
const Segment SGI282693[] = {{&S56412[13], 3}, {&S56412[17], 13}, {&S56412[51], 17}, {&S56412[81], 44}, {NULL}};
const Segment SGI282694[] = {{&S56413[136], 4}, {NULL}};
const Segment SGI282695[] = {{&S56413[13], 3}, {&S56413[17], 13}, {&S56413[51], 17}, {&S56413[81], 44}, {NULL}};
const Segment SGI282696[] = {{&S56414[136], 4}, {NULL}};
const Segment SGI282697[] = {{&S56414[13], 3}, {&S56414[17], 13}, {&S56414[51], 17}, {&S56414[81], 44}, {NULL}};
const Segment SGI282698[] = {{&S56415[136], 4}, {NULL}};
const Segment SGI282699[] = {{&S56415[13], 3}, {&S56415[17], 13}, {&S56415[51], 17}, {&S56415[81], 44}, {NULL}};
const Segment SGI282700[] = {{&S56416[136], 4}, {NULL}};
const Segment SGI282701[] = {{&S56416[13], 3}, {&S56416[17], 13}, {&S56416[51], 17}, {&S56416[81], 44}, {NULL}};
const Segment SGI282702[] = {{&S56417[136], 4}, {NULL}};
const Segment SGI282703[] = {{&S56417[13], 3}, {&S56417[17], 13}, {&S56417[51], 17}, {&S56417[81], 44}, {NULL}};
const Segment SGI282704[] = {{&S56418[92], 4}, {NULL}};
const Segment SGI282705[] = {{&S56418[13], 3}, {&S56418[17], 13}, {&S56418[51], 17}, {NULL}};
const Segment SGI282706[] = {{&S56419[136], 4}, {NULL}};
const Segment SGI282707[] = {{&S56419[13], 3}, {&S56419[17], 13}, {&S56419[51], 17}, {&S56419[81], 44}, {NULL}};
const Segment SGI282708[] = {{&S56420[136], 4}, {NULL}};
const Segment SGI282709[] = {{&S56420[13], 3}, {&S56420[17], 13}, {&S56420[51], 17}, {&S56420[81], 44}, {NULL}};
const Segment SGI282710[] = {{&S56421[136], 4}, {NULL}};
const Segment SGI282711[] = {{&S56421[13], 3}, {&S56421[17], 13}, {&S56421[51], 17}, {&S56421[81], 44}, {NULL}};
const Segment SGI282712[] = {{&S56422[136], 4}, {NULL}};
const Segment SGI282713[] = {{&S56422[13], 3}, {&S56422[17], 13}, {&S56422[51], 17}, {&S56422[81], 44}, {NULL}};
const Segment SGI282714[] = {{&S56423[136], 4}, {NULL}};
const Segment SGI282715[] = {{&S56423[13], 3}, {&S56423[17], 13}, {&S56423[51], 17}, {&S56423[81], 44}, {NULL}};
const Segment SGI282716[] = {{&S56424[136], 4}, {NULL}};
const Segment SGI282717[] = {{&S56424[13], 3}, {&S56424[17], 13}, {&S56424[51], 17}, {&S56424[81], 44}, {NULL}};
const Segment SGI282718[] = {{&S56425[136], 4}, {NULL}};
const Segment SGI282719[] = {{&S56425[13], 3}, {&S56425[17], 13}, {&S56425[51], 17}, {&S56425[81], 44}, {NULL}};
const Segment SGI282720[] = {{&S56426[136], 4}, {NULL}};
const Segment SGI282721[] = {{&S56426[13], 3}, {&S56426[17], 13}, {&S56426[51], 17}, {&S56426[81], 44}, {NULL}};
const Segment SGI282722[] = {{&S56427[136], 4}, {NULL}};
const Segment SGI282723[] = {{&S56427[13], 3}, {&S56427[17], 13}, {&S56427[51], 17}, {&S56427[81], 44}, {NULL}};
const Segment SGI282724[] = {{&S56428[136], 4}, {NULL}};
const Segment SGI282725[] = {{&S56428[13], 3}, {&S56428[17], 13}, {&S56428[51], 17}, {&S56428[81], 44}, {NULL}};
const Segment SGI282726[] = {{&S56429[136], 4}, {NULL}};
const Segment SGI282727[] = {{&S56429[13], 3}, {&S56429[17], 13}, {&S56429[51], 17}, {&S56429[81], 44}, {NULL}};
const Segment SGI282728[] = {{&S56430[136], 4}, {NULL}};
const Segment SGI282729[] = {{&S56430[13], 3}, {&S56430[17], 13}, {&S56430[51], 17}, {&S56430[81], 44}, {NULL}};
const Segment SGI282730[] = {{&S56431[136], 4}, {NULL}};
const Segment SGI282731[] = {{&S56431[13], 3}, {&S56431[17], 13}, {&S56431[51], 17}, {&S56431[81], 44}, {NULL}};
const Segment SGI282732[] = {{&S56432[136], 4}, {NULL}};
const Segment SGI282733[] = {{&S56432[13], 3}, {&S56432[17], 13}, {&S56432[51], 17}, {&S56432[81], 44}, {NULL}};
const Segment SGI282734[] = {{&S56433[136], 4}, {NULL}};
const Segment SGI282735[] = {{&S56433[13], 3}, {&S56433[17], 13}, {&S56433[51], 17}, {&S56433[81], 44}, {NULL}};
const Segment SGI282736[] = {{&S56434[136], 4}, {NULL}};
const Segment SGI282737[] = {{&S56434[13], 3}, {&S56434[17], 13}, {&S56434[51], 17}, {&S56434[81], 44}, {NULL}};
const Segment SGI282738[] = {{&S56435[136], 4}, {NULL}};
const Segment SGI282739[] = {{&S56435[13], 3}, {&S56435[17], 13}, {&S56435[51], 17}, {&S56435[81], 44}, {NULL}};
const Segment SGI282740[] = {{&S56436[136], 4}, {NULL}};
const Segment SGI282741[] = {{&S56436[13], 3}, {&S56436[17], 13}, {&S56436[51], 17}, {&S56436[81], 44}, {NULL}};
const Segment SGI282742[] = {{&S56437[136], 4}, {NULL}};
const Segment SGI282743[] = {{&S56437[13], 3}, {&S56437[17], 13}, {&S56437[51], 17}, {&S56437[81], 44}, {NULL}};
const Segment SGI282744[] = {{&S56438[136], 4}, {NULL}};
const Segment SGI282745[] = {{&S56438[13], 3}, {&S56438[17], 13}, {&S56438[51], 17}, {&S56438[81], 44}, {NULL}};
const Segment SGI282746[] = {{&S56439[136], 4}, {NULL}};
const Segment SGI282747[] = {{&S56439[13], 3}, {&S56439[17], 13}, {&S56439[51], 17}, {&S56439[81], 44}, {NULL}};
const Segment SGI282748[] = {{&S56440[136], 4}, {NULL}};
const Segment SGI282749[] = {{&S56440[13], 3}, {&S56440[17], 13}, {&S56440[51], 17}, {&S56440[81], 44}, {NULL}};
const Segment SGI282750[] = {{&S56441[136], 4}, {NULL}};
const Segment SGI282751[] = {{&S56441[13], 3}, {&S56441[17], 13}, {&S56441[51], 17}, {&S56441[81], 44}, {NULL}};
const Segment SGI282752[] = {{&S56442[136], 4}, {NULL}};
const Segment SGI282753[] = {{&S56442[13], 3}, {&S56442[17], 13}, {&S56442[51], 17}, {&S56442[81], 44}, {NULL}};
const Segment SGI282754[] = {{&S56443[136], 4}, {NULL}};
const Segment SGI282755[] = {{&S56443[13], 3}, {&S56443[17], 13}, {&S56443[51], 17}, {&S56443[81], 44}, {NULL}};
const Segment SGI282756[] = {{&S56444[136], 4}, {NULL}};
const Segment SGI282757[] = {{&S56444[13], 3}, {&S56444[17], 13}, {&S56444[51], 17}, {&S56444[81], 44}, {NULL}};
const Segment SGI282758[] = {{&S56445[136], 4}, {NULL}};
const Segment SGI282759[] = {{&S56445[13], 3}, {&S56445[17], 13}, {&S56445[51], 17}, {&S56445[81], 44}, {NULL}};
const Segment SGI282760[] = {{&S56446[136], 4}, {NULL}};
const Segment SGI282761[] = {{&S56446[13], 3}, {&S56446[17], 13}, {&S56446[51], 17}, {&S56446[81], 44}, {NULL}};
const Segment SGI282762[] = {{&S56447[136], 4}, {NULL}};
const Segment SGI282763[] = {{&S56447[13], 3}, {&S56447[17], 13}, {&S56447[51], 17}, {&S56447[81], 44}, {NULL}};
const Segment SGI282764[] = {{&S56448[136], 4}, {NULL}};
const Segment SGI282765[] = {{&S56448[13], 3}, {&S56448[17], 13}, {&S56448[51], 17}, {&S56448[81], 44}, {NULL}};
const Segment SGI282766[] = {{&S56449[136], 4}, {NULL}};
const Segment SGI282767[] = {{&S56449[13], 3}, {&S56449[17], 13}, {&S56449[51], 17}, {&S56449[81], 44}, {NULL}};
const Segment SGI282768[] = {{&S56450[136], 4}, {NULL}};
const Segment SGI282769[] = {{&S56450[13], 3}, {&S56450[17], 13}, {&S56450[51], 17}, {&S56450[81], 44}, {NULL}};
const Segment SGI282770[] = {{&S56451[136], 4}, {NULL}};
const Segment SGI282771[] = {{&S56451[13], 3}, {&S56451[17], 13}, {&S56451[51], 17}, {&S56451[81], 44}, {NULL}};
const Segment SGI282772[] = {{&S56452[136], 4}, {NULL}};
const Segment SGI282773[] = {{&S56452[13], 3}, {&S56452[17], 13}, {&S56452[51], 17}, {&S56452[81], 44}, {NULL}};
const Segment SGI282774[] = {{&S56453[136], 4}, {NULL}};
const Segment SGI282775[] = {{&S56453[13], 3}, {&S56453[17], 13}, {&S56453[51], 17}, {&S56453[81], 44}, {NULL}};
const Segment SGI282776[] = {{&S56454[136], 4}, {NULL}};
const Segment SGI282777[] = {{&S56454[13], 3}, {&S56454[17], 13}, {&S56454[51], 17}, {&S56454[81], 44}, {NULL}};
const Segment SGI282778[] = {{&S56455[136], 4}, {NULL}};
const Segment SGI282779[] = {{&S56455[13], 3}, {&S56455[17], 13}, {&S56455[51], 17}, {&S56455[81], 44}, {NULL}};
const Segment SGI282780[] = {{&S56456[136], 4}, {NULL}};
const Segment SGI282781[] = {{&S56456[13], 3}, {&S56456[17], 13}, {&S56456[51], 17}, {&S56456[81], 44}, {NULL}};
const Segment SGI282782[] = {{&S56457[136], 4}, {NULL}};
const Segment SGI282783[] = {{&S56457[13], 3}, {&S56457[17], 13}, {&S56457[51], 17}, {&S56457[81], 44}, {NULL}};
const Segment SGI282784[] = {{&S56458[136], 4}, {NULL}};
const Segment SGI282785[] = {{&S56458[13], 3}, {&S56458[17], 13}, {&S56458[51], 17}, {&S56458[81], 44}, {NULL}};
const Segment SGI282786[] = {{&S56459[136], 4}, {NULL}};
const Segment SGI282787[] = {{&S56459[13], 3}, {&S56459[17], 13}, {&S56459[51], 17}, {&S56459[81], 44}, {NULL}};
const Segment SGI282788[] = {{&S56460[136], 4}, {NULL}};
const Segment SGI282789[] = {{&S56460[13], 3}, {&S56460[17], 13}, {&S56460[51], 17}, {&S56460[81], 44}, {NULL}};
const Segment SGI282790[] = {{&S56461[136], 4}, {NULL}};
const Segment SGI282791[] = {{&S56461[13], 3}, {&S56461[17], 13}, {&S56461[51], 17}, {&S56461[81], 44}, {NULL}};
const Segment SGI282792[] = {{&S56462[136], 4}, {NULL}};
const Segment SGI282793[] = {{&S56462[13], 3}, {&S56462[17], 13}, {&S56462[51], 17}, {&S56462[81], 44}, {NULL}};
const Segment SGI282794[] = {{&S56463[136], 4}, {NULL}};
const Segment SGI282795[] = {{&S56463[13], 3}, {&S56463[17], 13}, {&S56463[51], 17}, {&S56463[81], 44}, {NULL}};
const Segment SGI282796[] = {{&S56464[136], 4}, {NULL}};
const Segment SGI282797[] = {{&S56464[13], 3}, {&S56464[17], 13}, {&S56464[51], 17}, {&S56464[81], 44}, {NULL}};
const Segment SGI282798[] = {{&S56465[136], 4}, {NULL}};
const Segment SGI282799[] = {{&S56465[13], 3}, {&S56465[17], 13}, {&S56465[51], 17}, {&S56465[81], 44}, {NULL}};
const Segment SGI282800[] = {{&S56466[136], 4}, {NULL}};
const Segment SGI282801[] = {{&S56466[13], 3}, {&S56466[17], 13}, {&S56466[51], 17}, {&S56466[81], 44}, {NULL}};
const Segment SGI282802[] = {{&S56467[136], 4}, {NULL}};
const Segment SGI282803[] = {{&S56467[13], 3}, {&S56467[17], 13}, {&S56467[51], 17}, {&S56467[81], 44}, {NULL}};
const Segment SGI282804[] = {{&S56468[136], 4}, {NULL}};
const Segment SGI282805[] = {{&S56468[13], 3}, {&S56468[17], 13}, {&S56468[51], 17}, {&S56468[81], 44}, {NULL}};
const Segment SGI282806[] = {{&S56469[136], 4}, {NULL}};
const Segment SGI282807[] = {{&S56469[13], 3}, {&S56469[17], 13}, {&S56469[51], 17}, {&S56469[81], 44}, {NULL}};
const Segment SGI282808[] = {{&S56470[119], 4}, {NULL}};
const Segment SGI282809[] = {{&S56470[13], 3}, {&S56470[17], 13}, {&S56470[51], 17}, {&S56470[81], 27}, {NULL}};
const Segment SGI282810[] = {{&S56471[136], 4}, {NULL}};
const Segment SGI282811[] = {{&S56471[13], 3}, {&S56471[17], 13}, {&S56471[51], 17}, {&S56471[81], 44}, {NULL}};
const Segment SGI282812[] = {{&S56472[136], 4}, {NULL}};
const Segment SGI282813[] = {{&S56472[13], 3}, {&S56472[17], 13}, {&S56472[51], 17}, {&S56472[81], 44}, {NULL}};
const Segment SGI282814[] = {{&S56473[136], 4}, {NULL}};
const Segment SGI282815[] = {{&S56473[13], 3}, {&S56473[17], 13}, {&S56473[51], 17}, {&S56473[81], 44}, {NULL}};
const Segment SGI282816[] = {{&S56474[136], 4}, {NULL}};
const Segment SGI282817[] = {{&S56474[13], 3}, {&S56474[17], 13}, {&S56474[51], 17}, {&S56474[81], 44}, {NULL}};
const Segment SGI282818[] = {{&S56475[136], 4}, {NULL}};
const Segment SGI282819[] = {{&S56475[13], 3}, {&S56475[17], 13}, {&S56475[51], 17}, {&S56475[81], 44}, {NULL}};
const Segment SGI282820[] = {{&S56476[136], 4}, {NULL}};
const Segment SGI282821[] = {{&S56476[13], 3}, {&S56476[17], 13}, {&S56476[51], 17}, {&S56476[81], 44}, {NULL}};
const Segment SGI282822[] = {{&S56477[136], 4}, {NULL}};
const Segment SGI282823[] = {{&S56477[13], 3}, {&S56477[17], 13}, {&S56477[51], 17}, {&S56477[81], 44}, {NULL}};
const Segment SGI282824[] = {{&S56478[136], 4}, {NULL}};
const Segment SGI282825[] = {{&S56478[13], 3}, {&S56478[17], 13}, {&S56478[51], 17}, {&S56478[81], 44}, {NULL}};
const Segment SGI282826[] = {{&S56479[136], 4}, {NULL}};
const Segment SGI282827[] = {{&S56479[13], 3}, {&S56479[17], 13}, {&S56479[51], 17}, {&S56479[81], 44}, {NULL}};
const Segment SGI282828[] = {{&S56480[136], 4}, {NULL}};
const Segment SGI282829[] = {{&S56480[13], 3}, {&S56480[17], 13}, {&S56480[51], 17}, {&S56480[81], 44}, {NULL}};
const Segment SGI282830[] = {{&S56481[136], 4}, {NULL}};
const Segment SGI282831[] = {{&S56481[13], 3}, {&S56481[17], 13}, {&S56481[51], 17}, {&S56481[81], 44}, {NULL}};
const Segment SGI282832[] = {{&S56482[136], 4}, {NULL}};
const Segment SGI282833[] = {{&S56482[13], 3}, {&S56482[17], 13}, {&S56482[51], 17}, {&S56482[81], 44}, {NULL}};
const Segment SGI282834[] = {{&S56483[136], 4}, {NULL}};
const Segment SGI282835[] = {{&S56483[13], 3}, {&S56483[17], 13}, {&S56483[51], 17}, {&S56483[81], 44}, {NULL}};
const Segment SGI282836[] = {{&S56484[136], 4}, {NULL}};
const Segment SGI282837[] = {{&S56484[13], 3}, {&S56484[17], 13}, {&S56484[51], 17}, {&S56484[81], 44}, {NULL}};
const Segment SGI282838[] = {{&S56485[136], 4}, {NULL}};
const Segment SGI282839[] = {{&S56485[13], 3}, {&S56485[17], 13}, {&S56485[51], 17}, {&S56485[81], 44}, {NULL}};
const Segment SGI282840[] = {{&S56486[136], 4}, {NULL}};
const Segment SGI282841[] = {{&S56486[13], 3}, {&S56486[17], 13}, {&S56486[51], 17}, {&S56486[81], 44}, {NULL}};
const Segment SGI282842[] = {{&S56487[136], 4}, {NULL}};
const Segment SGI282843[] = {{&S56487[13], 3}, {&S56487[17], 13}, {&S56487[51], 17}, {&S56487[81], 44}, {NULL}};
const Segment SGI282844[] = {{&S56488[136], 4}, {NULL}};
const Segment SGI282845[] = {{&S56488[13], 3}, {&S56488[17], 13}, {&S56488[51], 17}, {&S56488[81], 44}, {NULL}};
const Segment SGI282846[] = {{&S56489[136], 4}, {NULL}};
const Segment SGI282847[] = {{&S56489[13], 3}, {&S56489[17], 13}, {&S56489[51], 17}, {&S56489[81], 44}, {NULL}};
const Segment SGI282848[] = {{&S56490[136], 4}, {NULL}};
const Segment SGI282849[] = {{&S56490[13], 3}, {&S56490[17], 13}, {&S56490[51], 17}, {&S56490[81], 44}, {NULL}};
const Segment SGI282850[] = {{&S56491[136], 4}, {NULL}};
const Segment SGI282851[] = {{&S56491[13], 3}, {&S56491[17], 13}, {&S56491[51], 17}, {&S56491[81], 44}, {NULL}};
const Segment SGI282852[] = {{&S56492[136], 4}, {NULL}};
const Segment SGI282853[] = {{&S56492[13], 3}, {&S56492[17], 13}, {&S56492[51], 17}, {&S56492[81], 44}, {NULL}};
const Segment SGI282854[] = {{&S56493[136], 4}, {NULL}};
const Segment SGI282855[] = {{&S56493[13], 3}, {&S56493[17], 13}, {&S56493[51], 17}, {&S56493[81], 44}, {NULL}};
const Segment SGI282856[] = {{&S56494[136], 4}, {NULL}};
const Segment SGI282857[] = {{&S56494[13], 3}, {&S56494[17], 13}, {&S56494[51], 17}, {&S56494[81], 44}, {NULL}};
const Segment SGI282858[] = {{&S56495[136], 4}, {NULL}};
const Segment SGI282859[] = {{&S56495[13], 3}, {&S56495[17], 13}, {&S56495[51], 17}, {&S56495[81], 44}, {NULL}};
const Segment SGI282860[] = {{&S56496[136], 4}, {NULL}};
const Segment SGI282861[] = {{&S56496[13], 3}, {&S56496[17], 13}, {&S56496[51], 17}, {&S56496[81], 44}, {NULL}};
const Segment SGI282862[] = {{&S56497[136], 4}, {NULL}};
const Segment SGI282863[] = {{&S56497[13], 3}, {&S56497[17], 13}, {&S56497[51], 17}, {&S56497[81], 44}, {NULL}};
const Segment SGI282864[] = {{&S56498[136], 4}, {NULL}};
const Segment SGI282865[] = {{&S56498[13], 3}, {&S56498[17], 13}, {&S56498[51], 17}, {&S56498[81], 44}, {NULL}};
const Segment SGI282866[] = {{&S56499[136], 4}, {NULL}};
const Segment SGI282867[] = {{&S56499[13], 3}, {&S56499[17], 13}, {&S56499[51], 17}, {&S56499[81], 44}, {NULL}};
const Segment SGI282868[] = {{&S56500[136], 4}, {NULL}};
const Segment SGI282869[] = {{&S56500[13], 3}, {&S56500[17], 13}, {&S56500[51], 17}, {&S56500[81], 44}, {NULL}};
const Segment SGI282870[] = {{&S56501[136], 4}, {NULL}};
const Segment SGI282871[] = {{&S56501[13], 3}, {&S56501[17], 13}, {&S56501[51], 17}, {&S56501[81], 44}, {NULL}};
const Segment SGI282872[] = {{&S56502[136], 4}, {NULL}};
const Segment SGI282873[] = {{&S56502[13], 3}, {&S56502[17], 13}, {&S56502[51], 17}, {&S56502[81], 44}, {NULL}};
const Segment SGI282874[] = {{&S56503[136], 4}, {NULL}};
const Segment SGI282875[] = {{&S56503[13], 3}, {&S56503[17], 13}, {&S56503[51], 17}, {&S56503[81], 44}, {NULL}};
const Segment SGI282876[] = {{&S56504[136], 4}, {NULL}};
const Segment SGI282877[] = {{&S56504[13], 3}, {&S56504[17], 13}, {&S56504[51], 17}, {&S56504[81], 44}, {NULL}};
const Segment SGI282878[] = {{&S56505[136], 4}, {NULL}};
const Segment SGI282879[] = {{&S56505[13], 3}, {&S56505[17], 13}, {&S56505[51], 17}, {&S56505[81], 44}, {NULL}};
const Segment SGI282880[] = {{&S56506[136], 4}, {NULL}};
const Segment SGI282881[] = {{&S56506[13], 3}, {&S56506[17], 13}, {&S56506[51], 17}, {&S56506[81], 44}, {NULL}};
const Segment SGI282882[] = {{&S56507[136], 4}, {NULL}};
const Segment SGI282883[] = {{&S56507[13], 3}, {&S56507[17], 13}, {&S56507[51], 17}, {&S56507[81], 44}, {NULL}};
const Segment SGI282884[] = {{&S56508[136], 4}, {NULL}};
const Segment SGI282885[] = {{&S56508[13], 3}, {&S56508[17], 13}, {&S56508[51], 17}, {&S56508[81], 44}, {NULL}};
const Segment SGI282886[] = {{&S56509[136], 4}, {NULL}};
const Segment SGI282887[] = {{&S56509[13], 3}, {&S56509[17], 13}, {&S56509[51], 17}, {&S56509[81], 44}, {NULL}};
const Segment SGI282888[] = {{&S56510[136], 4}, {NULL}};
const Segment SGI282889[] = {{&S56510[13], 3}, {&S56510[17], 13}, {&S56510[51], 17}, {&S56510[81], 44}, {NULL}};
const Segment SGI282890[] = {{&S56511[136], 4}, {NULL}};
const Segment SGI282891[] = {{&S56511[13], 3}, {&S56511[17], 13}, {&S56511[51], 17}, {&S56511[81], 44}, {NULL}};
const Segment SGI282892[] = {{&S56512[136], 4}, {NULL}};
const Segment SGI282893[] = {{&S56512[13], 3}, {&S56512[17], 13}, {&S56512[51], 17}, {&S56512[81], 44}, {NULL}};
const Segment SGI282894[] = {{&S56513[136], 4}, {NULL}};
const Segment SGI282895[] = {{&S56513[13], 3}, {&S56513[17], 13}, {&S56513[51], 17}, {&S56513[81], 44}, {NULL}};
const Segment SGI282896[] = {{&S56514[136], 4}, {NULL}};
const Segment SGI282897[] = {{&S56514[13], 3}, {&S56514[17], 13}, {&S56514[51], 17}, {&S56514[81], 44}, {NULL}};
const Segment SGI282898[] = {{&S56515[136], 4}, {NULL}};
const Segment SGI282899[] = {{&S56515[13], 3}, {&S56515[17], 13}, {&S56515[51], 17}, {&S56515[81], 44}, {NULL}};
const Segment SGI282900[] = {{&S56516[136], 4}, {NULL}};
const Segment SGI282901[] = {{&S56516[13], 3}, {&S56516[17], 13}, {&S56516[51], 17}, {&S56516[81], 44}, {NULL}};
const Segment SGI282902[] = {{&S56517[136], 4}, {NULL}};
const Segment SGI282903[] = {{&S56517[13], 3}, {&S56517[17], 13}, {&S56517[51], 17}, {&S56517[81], 44}, {NULL}};
const Segment SGI282904[] = {{&S56518[136], 4}, {NULL}};
const Segment SGI282905[] = {{&S56518[13], 3}, {&S56518[17], 13}, {&S56518[51], 17}, {&S56518[81], 44}, {NULL}};
const Segment SGI282906[] = {{&S56519[136], 4}, {NULL}};
const Segment SGI282907[] = {{&S56519[13], 3}, {&S56519[17], 13}, {&S56519[51], 17}, {&S56519[81], 44}, {NULL}};
const Segment SGI282908[] = {{&S56520[136], 4}, {NULL}};
const Segment SGI282909[] = {{&S56520[13], 3}, {&S56520[17], 13}, {&S56520[51], 17}, {&S56520[81], 44}, {NULL}};
const Segment SGI282910[] = {{&S56521[136], 4}, {NULL}};
const Segment SGI282911[] = {{&S56521[13], 3}, {&S56521[17], 13}, {&S56521[51], 17}, {&S56521[81], 44}, {NULL}};
const Segment SGI282912[] = {{&S56522[136], 4}, {NULL}};
const Segment SGI282913[] = {{&S56522[13], 3}, {&S56522[17], 13}, {&S56522[51], 17}, {&S56522[81], 44}, {NULL}};
const Segment SGI282914[] = {{&S56523[136], 4}, {NULL}};
const Segment SGI282915[] = {{&S56523[13], 3}, {&S56523[17], 13}, {&S56523[51], 17}, {&S56523[81], 44}, {NULL}};
const Segment SGI282916[] = {{&S56524[136], 4}, {NULL}};
const Segment SGI282917[] = {{&S56524[13], 3}, {&S56524[17], 13}, {&S56524[51], 17}, {&S56524[81], 44}, {NULL}};
const Segment SGI282918[] = {{&S56525[136], 4}, {NULL}};
const Segment SGI282919[] = {{&S56525[13], 3}, {&S56525[17], 13}, {&S56525[51], 17}, {&S56525[81], 44}, {NULL}};
const Segment SGI282920[] = {{&S56526[136], 4}, {NULL}};
const Segment SGI282921[] = {{&S56526[13], 3}, {&S56526[17], 13}, {&S56526[51], 17}, {&S56526[81], 44}, {NULL}};
const Segment SGI282922[] = {{&S56527[136], 4}, {NULL}};
const Segment SGI282923[] = {{&S56527[13], 3}, {&S56527[17], 13}, {&S56527[51], 17}, {&S56527[81], 44}, {NULL}};
const Segment SGI282924[] = {{&S56528[136], 4}, {NULL}};
const Segment SGI282925[] = {{&S56528[13], 3}, {&S56528[17], 13}, {&S56528[51], 17}, {&S56528[81], 44}, {NULL}};
const Segment SGI282926[] = {{&S56529[136], 4}, {NULL}};
const Segment SGI282927[] = {{&S56529[13], 3}, {&S56529[17], 13}, {&S56529[51], 17}, {&S56529[81], 44}, {NULL}};
const Segment SGI282928[] = {{&S56530[136], 4}, {NULL}};
const Segment SGI282929[] = {{&S56530[13], 3}, {&S56530[17], 13}, {&S56530[51], 17}, {&S56530[81], 44}, {NULL}};
const Segment SGI282930[] = {{&S56531[136], 4}, {NULL}};
const Segment SGI282931[] = {{&S56531[13], 3}, {&S56531[17], 13}, {&S56531[51], 17}, {&S56531[81], 44}, {NULL}};
const Segment SGI282932[] = {{&S56532[136], 4}, {NULL}};
const Segment SGI282933[] = {{&S56532[13], 3}, {&S56532[17], 13}, {&S56532[51], 17}, {&S56532[81], 44}, {NULL}};
const Segment SGI282934[] = {{&S56533[136], 4}, {NULL}};
const Segment SGI282935[] = {{&S56533[13], 3}, {&S56533[17], 13}, {&S56533[51], 17}, {&S56533[81], 44}, {NULL}};
const Segment SGI282936[] = {{&S56534[136], 4}, {NULL}};
const Segment SGI282937[] = {{&S56534[13], 3}, {&S56534[17], 13}, {&S56534[51], 17}, {&S56534[81], 44}, {NULL}};
const Segment SGI282938[] = {{&S56535[136], 4}, {NULL}};
const Segment SGI282939[] = {{&S56535[13], 3}, {&S56535[17], 13}, {&S56535[51], 17}, {&S56535[81], 44}, {NULL}};
const Segment SGI282940[] = {{&S56536[136], 4}, {NULL}};
const Segment SGI282941[] = {{&S56536[13], 3}, {&S56536[17], 13}, {&S56536[51], 17}, {&S56536[81], 44}, {NULL}};
const Segment SGI282942[] = {{&S56537[136], 4}, {NULL}};
const Segment SGI282943[] = {{&S56537[13], 3}, {&S56537[17], 13}, {&S56537[51], 17}, {&S56537[81], 44}, {NULL}};
const Segment SGI282944[] = {{&S56538[136], 4}, {NULL}};
const Segment SGI282945[] = {{&S56538[13], 3}, {&S56538[17], 13}, {&S56538[51], 17}, {&S56538[81], 44}, {NULL}};
const Segment SGI282946[] = {{&S56539[136], 4}, {NULL}};
const Segment SGI282947[] = {{&S56539[13], 3}, {&S56539[17], 13}, {&S56539[51], 17}, {&S56539[81], 44}, {NULL}};
const Segment SGI282948[] = {{&S56540[136], 4}, {NULL}};
const Segment SGI282949[] = {{&S56540[13], 3}, {&S56540[17], 13}, {&S56540[51], 17}, {&S56540[81], 44}, {NULL}};
const Segment SGI282950[] = {{&S56541[136], 4}, {NULL}};
const Segment SGI282951[] = {{&S56541[13], 3}, {&S56541[17], 13}, {&S56541[51], 17}, {&S56541[81], 44}, {NULL}};
const Segment SGI282952[] = {{&S56542[136], 4}, {NULL}};
const Segment SGI282953[] = {{&S56542[13], 3}, {&S56542[17], 13}, {&S56542[51], 17}, {&S56542[81], 44}, {NULL}};
const Segment SGI282954[] = {{&S56543[136], 4}, {NULL}};
const Segment SGI282955[] = {{&S56543[13], 3}, {&S56543[17], 13}, {&S56543[51], 17}, {&S56543[81], 44}, {NULL}};
const Segment SGI282956[] = {{&S56544[136], 4}, {NULL}};
const Segment SGI282957[] = {{&S56544[13], 3}, {&S56544[17], 13}, {&S56544[51], 17}, {&S56544[81], 44}, {NULL}};
const Segment SGI282958[] = {{&S56545[136], 4}, {NULL}};
const Segment SGI282959[] = {{&S56545[13], 3}, {&S56545[17], 13}, {&S56545[51], 17}, {&S56545[81], 44}, {NULL}};
const Segment SGI282960[] = {{&S56546[136], 4}, {NULL}};
const Segment SGI282961[] = {{&S56546[13], 3}, {&S56546[17], 13}, {&S56546[51], 17}, {&S56546[81], 44}, {NULL}};
const Segment SGI282962[] = {{&S56547[136], 4}, {NULL}};
const Segment SGI282963[] = {{&S56547[13], 3}, {&S56547[17], 13}, {&S56547[51], 17}, {&S56547[81], 44}, {NULL}};
const Segment SGI282964[] = {{&S56548[136], 4}, {NULL}};
const Segment SGI282965[] = {{&S56548[13], 3}, {&S56548[17], 13}, {&S56548[51], 17}, {&S56548[81], 44}, {NULL}};
const Segment SGI282966[] = {{&S56549[136], 4}, {NULL}};
const Segment SGI282967[] = {{&S56549[13], 3}, {&S56549[17], 13}, {&S56549[51], 17}, {&S56549[81], 44}, {NULL}};
const Segment SGI282968[] = {{&S56550[136], 4}, {NULL}};
const Segment SGI282969[] = {{&S56550[13], 3}, {&S56550[17], 13}, {&S56550[51], 17}, {&S56550[81], 44}, {NULL}};
const Segment SGI282970[] = {{&S56551[136], 4}, {NULL}};
const Segment SGI282971[] = {{&S56551[13], 3}, {&S56551[17], 13}, {&S56551[51], 17}, {&S56551[81], 44}, {NULL}};
const Segment SGI282972[] = {{&S56552[136], 4}, {NULL}};
const Segment SGI282973[] = {{&S56552[13], 3}, {&S56552[17], 13}, {&S56552[51], 17}, {&S56552[81], 44}, {NULL}};
const Segment SGI282974[] = {{&S56553[136], 4}, {NULL}};
const Segment SGI282975[] = {{&S56553[13], 3}, {&S56553[17], 13}, {&S56553[51], 17}, {&S56553[81], 44}, {NULL}};
const Segment SGI282976[] = {{&S56554[136], 4}, {NULL}};
const Segment SGI282977[] = {{&S56554[13], 3}, {&S56554[17], 13}, {&S56554[51], 17}, {&S56554[81], 44}, {NULL}};
const Segment SGI282978[] = {{&S56555[136], 4}, {NULL}};
const Segment SGI282979[] = {{&S56555[13], 3}, {&S56555[17], 13}, {&S56555[51], 17}, {&S56555[81], 44}, {NULL}};
const Segment SGI282980[] = {{&S56556[136], 4}, {NULL}};
const Segment SGI282981[] = {{&S56556[13], 3}, {&S56556[17], 13}, {&S56556[51], 17}, {&S56556[81], 44}, {NULL}};
const Segment SGI282982[] = {{&S56557[136], 4}, {NULL}};
const Segment SGI282983[] = {{&S56557[13], 3}, {&S56557[17], 13}, {&S56557[51], 17}, {&S56557[81], 44}, {NULL}};
const Segment SGI282984[] = {{&S56558[136], 4}, {NULL}};
const Segment SGI282985[] = {{&S56558[13], 3}, {&S56558[17], 13}, {&S56558[51], 17}, {&S56558[81], 44}, {NULL}};
const Segment SGI282986[] = {{&S56559[136], 4}, {NULL}};
const Segment SGI282987[] = {{&S56559[13], 3}, {&S56559[17], 13}, {&S56559[51], 17}, {&S56559[81], 44}, {NULL}};
const Segment SGI282988[] = {{&S56560[136], 4}, {NULL}};
const Segment SGI282989[] = {{&S56560[13], 3}, {&S56560[17], 13}, {&S56560[51], 17}, {&S56560[81], 44}, {NULL}};
const Segment SGI282990[] = {{&S56561[136], 4}, {NULL}};
const Segment SGI282991[] = {{&S56561[13], 3}, {&S56561[17], 13}, {&S56561[51], 17}, {&S56561[81], 44}, {NULL}};
const Segment SGI282992[] = {{&S56562[136], 4}, {NULL}};
const Segment SGI282993[] = {{&S56562[13], 3}, {&S56562[17], 13}, {&S56562[51], 17}, {&S56562[81], 44}, {NULL}};
const Segment SGI282994[] = {{&S56563[136], 4}, {NULL}};
const Segment SGI282995[] = {{&S56563[13], 3}, {&S56563[17], 13}, {&S56563[51], 17}, {&S56563[81], 44}, {NULL}};
const Segment SGI282996[] = {{&S56564[136], 4}, {NULL}};
const Segment SGI282997[] = {{&S56564[13], 3}, {&S56564[17], 13}, {&S56564[51], 17}, {&S56564[81], 44}, {NULL}};
const Segment SGI282998[] = {{&S56565[136], 4}, {NULL}};
const Segment SGI282999[] = {{&S56565[13], 3}, {&S56565[17], 13}, {&S56565[51], 17}, {&S56565[81], 44}, {NULL}};
const Segment SGI283000[] = {{&S56566[136], 4}, {NULL}};
const Segment SGI283001[] = {{&S56566[13], 3}, {&S56566[17], 13}, {&S56566[51], 17}, {&S56566[81], 44}, {NULL}};
const Segment SGI283002[] = {{&S56567[136], 4}, {NULL}};
const Segment SGI283003[] = {{&S56567[13], 3}, {&S56567[17], 13}, {&S56567[51], 17}, {&S56567[81], 44}, {NULL}};
const Segment SGI283004[] = {{&S56568[136], 4}, {NULL}};
const Segment SGI283005[] = {{&S56568[13], 3}, {&S56568[17], 13}, {&S56568[51], 17}, {&S56568[81], 44}, {NULL}};
const Segment SGI283006[] = {{&S56569[136], 4}, {NULL}};
const Segment SGI283007[] = {{&S56569[13], 3}, {&S56569[17], 13}, {&S56569[51], 17}, {&S56569[81], 44}, {NULL}};
const Segment SGI283008[] = {{&S56570[136], 4}, {NULL}};
const Segment SGI283009[] = {{&S56570[13], 3}, {&S56570[17], 13}, {&S56570[51], 17}, {&S56570[81], 44}, {NULL}};
const Segment SGI283010[] = {{&S56571[136], 4}, {NULL}};
const Segment SGI283011[] = {{&S56571[13], 3}, {&S56571[17], 13}, {&S56571[51], 17}, {&S56571[81], 44}, {NULL}};
const Segment SGI283012[] = {{&S56572[136], 4}, {NULL}};
const Segment SGI283013[] = {{&S56572[13], 3}, {&S56572[17], 13}, {&S56572[51], 17}, {&S56572[81], 44}, {NULL}};
const Segment SGI283014[] = {{&S56573[136], 4}, {NULL}};
const Segment SGI283015[] = {{&S56573[13], 3}, {&S56573[17], 13}, {&S56573[51], 17}, {&S56573[81], 44}, {NULL}};
const Segment SGI283016[] = {{&S56574[136], 4}, {NULL}};
const Segment SGI283017[] = {{&S56574[13], 3}, {&S56574[17], 13}, {&S56574[51], 17}, {&S56574[81], 44}, {NULL}};
const Segment SGI283018[] = {{&S56575[136], 4}, {NULL}};
const Segment SGI283019[] = {{&S56575[13], 3}, {&S56575[17], 13}, {&S56575[51], 17}, {&S56575[81], 44}, {NULL}};
const Segment SGI283020[] = {{&S56576[136], 4}, {NULL}};
const Segment SGI283021[] = {{&S56576[13], 3}, {&S56576[17], 13}, {&S56576[51], 17}, {&S56576[81], 44}, {NULL}};
const Segment SGI283022[] = {{&S56577[136], 4}, {NULL}};
const Segment SGI283023[] = {{&S56577[13], 3}, {&S56577[17], 13}, {&S56577[51], 17}, {&S56577[81], 44}, {NULL}};
const Segment SGI283024[] = {{&S56578[136], 4}, {NULL}};
const Segment SGI283025[] = {{&S56578[13], 3}, {&S56578[17], 13}, {&S56578[51], 17}, {&S56578[81], 44}, {NULL}};
const Segment SGI283026[] = {{&S56579[136], 4}, {NULL}};
const Segment SGI283027[] = {{&S56579[13], 3}, {&S56579[17], 13}, {&S56579[51], 17}, {&S56579[81], 44}, {NULL}};
const Segment SGI283028[] = {{&S56580[136], 4}, {NULL}};
const Segment SGI283029[] = {{&S56580[13], 3}, {&S56580[17], 13}, {&S56580[51], 17}, {&S56580[81], 44}, {NULL}};
const Segment SGI283030[] = {{&S56581[136], 4}, {NULL}};
const Segment SGI283031[] = {{&S56581[13], 3}, {&S56581[17], 13}, {&S56581[51], 17}, {&S56581[81], 44}, {NULL}};
const Segment SGI283032[] = {{&S56582[136], 4}, {NULL}};
const Segment SGI283033[] = {{&S56582[13], 3}, {&S56582[17], 13}, {&S56582[51], 17}, {&S56582[81], 44}, {NULL}};
const Segment SGI283034[] = {{&S56583[136], 4}, {NULL}};
const Segment SGI283035[] = {{&S56583[13], 3}, {&S56583[17], 13}, {&S56583[51], 17}, {&S56583[81], 44}, {NULL}};
const Segment SGI283036[] = {{&S56584[136], 4}, {NULL}};
const Segment SGI283037[] = {{&S56584[13], 3}, {&S56584[17], 13}, {&S56584[51], 17}, {&S56584[81], 44}, {NULL}};
const Segment SGI283038[] = {{&S56585[136], 4}, {NULL}};
const Segment SGI283039[] = {{&S56585[13], 3}, {&S56585[17], 13}, {&S56585[51], 17}, {&S56585[81], 44}, {NULL}};
const Segment SGI283040[] = {{&S56586[136], 4}, {NULL}};
const Segment SGI283041[] = {{&S56586[13], 3}, {&S56586[17], 13}, {&S56586[51], 17}, {&S56586[81], 44}, {NULL}};
const Segment SGI283042[] = {{&S56587[136], 4}, {NULL}};
const Segment SGI283043[] = {{&S56587[13], 3}, {&S56587[17], 13}, {&S56587[51], 17}, {&S56587[81], 44}, {NULL}};
const Segment SGI283044[] = {{&S56588[136], 4}, {NULL}};
const Segment SGI283045[] = {{&S56588[13], 3}, {&S56588[17], 13}, {&S56588[51], 17}, {&S56588[81], 44}, {NULL}};
const Segment SGI283046[] = {{&S56589[136], 4}, {NULL}};
const Segment SGI283047[] = {{&S56589[13], 3}, {&S56589[17], 13}, {&S56589[51], 17}, {&S56589[81], 44}, {NULL}};
const Segment SGI283048[] = {{&S56590[136], 4}, {NULL}};
const Segment SGI283049[] = {{&S56590[13], 3}, {&S56590[17], 13}, {&S56590[51], 17}, {&S56590[81], 44}, {NULL}};
const Segment SGI283050[] = {{&S56591[136], 4}, {NULL}};
const Segment SGI283051[] = {{&S56591[13], 3}, {&S56591[17], 13}, {&S56591[51], 17}, {&S56591[81], 44}, {NULL}};
const Segment SGI283052[] = {{&S56592[136], 4}, {NULL}};
const Segment SGI283053[] = {{&S56592[13], 3}, {&S56592[17], 13}, {&S56592[51], 17}, {&S56592[81], 44}, {NULL}};
const Segment SGI283054[] = {{&S56593[136], 4}, {NULL}};
const Segment SGI283055[] = {{&S56593[13], 3}, {&S56593[17], 13}, {&S56593[51], 17}, {&S56593[81], 44}, {NULL}};
const Segment SGI283056[] = {{&S56594[136], 4}, {NULL}};
const Segment SGI283057[] = {{&S56594[13], 3}, {&S56594[17], 13}, {&S56594[51], 17}, {&S56594[81], 44}, {NULL}};
const Segment SGI283058[] = {{&S56595[136], 4}, {NULL}};
const Segment SGI283059[] = {{&S56595[13], 3}, {&S56595[17], 13}, {&S56595[51], 17}, {&S56595[81], 44}, {NULL}};
const Segment SGI283060[] = {{&S56596[136], 4}, {NULL}};
const Segment SGI283061[] = {{&S56596[13], 3}, {&S56596[17], 13}, {&S56596[51], 17}, {&S56596[81], 44}, {NULL}};
const Segment SGI283062[] = {{&S56597[136], 4}, {NULL}};
const Segment SGI283063[] = {{&S56597[13], 3}, {&S56597[17], 13}, {&S56597[51], 17}, {&S56597[81], 44}, {NULL}};
const Segment SGI283064[] = {{&S56598[136], 4}, {NULL}};
const Segment SGI283065[] = {{&S56598[13], 3}, {&S56598[17], 13}, {&S56598[51], 17}, {&S56598[81], 44}, {NULL}};
const Segment SGI283066[] = {{&S56599[136], 4}, {NULL}};
const Segment SGI283067[] = {{&S56599[13], 3}, {&S56599[17], 13}, {&S56599[51], 17}, {&S56599[81], 44}, {NULL}};
const Segment SGI283068[] = {{&S56600[136], 4}, {NULL}};
const Segment SGI283069[] = {{&S56600[13], 3}, {&S56600[17], 13}, {&S56600[51], 17}, {&S56600[81], 44}, {NULL}};
const Segment SGI283070[] = {{&S56601[136], 4}, {NULL}};
const Segment SGI283071[] = {{&S56601[13], 3}, {&S56601[17], 13}, {&S56601[51], 17}, {&S56601[81], 44}, {NULL}};
const Segment SGI283072[] = {{&S56602[136], 4}, {NULL}};
const Segment SGI283073[] = {{&S56602[13], 3}, {&S56602[17], 13}, {&S56602[51], 17}, {&S56602[81], 44}, {NULL}};
const Segment SGI283074[] = {{&S56603[136], 4}, {NULL}};
const Segment SGI283075[] = {{&S56603[13], 3}, {&S56603[17], 13}, {&S56603[51], 17}, {&S56603[81], 44}, {NULL}};
const Segment SGI283076[] = {{&S56604[136], 4}, {NULL}};
const Segment SGI283077[] = {{&S56604[13], 3}, {&S56604[17], 13}, {&S56604[51], 17}, {&S56604[81], 44}, {NULL}};
const Segment SGI283078[] = {{&S56605[136], 4}, {NULL}};
const Segment SGI283079[] = {{&S56605[13], 3}, {&S56605[17], 13}, {&S56605[51], 17}, {&S56605[81], 44}, {NULL}};
const Segment SGI283080[] = {{&S56606[136], 4}, {NULL}};
const Segment SGI283081[] = {{&S56606[13], 3}, {&S56606[17], 13}, {&S56606[51], 17}, {&S56606[81], 44}, {NULL}};
const Segment SGI283082[] = {{&S56607[136], 4}, {NULL}};
const Segment SGI283083[] = {{&S56607[13], 3}, {&S56607[17], 13}, {&S56607[51], 17}, {&S56607[81], 44}, {NULL}};
const Segment SGI283084[] = {{&S56608[136], 4}, {NULL}};
const Segment SGI283085[] = {{&S56608[13], 3}, {&S56608[17], 13}, {&S56608[51], 17}, {&S56608[81], 44}, {NULL}};
const Segment SGI283086[] = {{&S56609[136], 4}, {NULL}};
const Segment SGI283087[] = {{&S56609[13], 3}, {&S56609[17], 13}, {&S56609[51], 17}, {&S56609[81], 44}, {NULL}};
const Segment SGI283088[] = {{&S56610[136], 4}, {NULL}};
const Segment SGI283089[] = {{&S56610[13], 3}, {&S56610[17], 13}, {&S56610[51], 17}, {&S56610[81], 44}, {NULL}};
const Segment SGI283090[] = {{&S56611[136], 4}, {NULL}};
const Segment SGI283091[] = {{&S56611[13], 3}, {&S56611[17], 13}, {&S56611[51], 17}, {&S56611[81], 44}, {NULL}};
const Segment SGI283092[] = {{&S56612[136], 4}, {NULL}};
const Segment SGI283093[] = {{&S56612[13], 3}, {&S56612[17], 13}, {&S56612[51], 17}, {&S56612[81], 44}, {NULL}};
const Segment SGI283094[] = {{&S56613[136], 4}, {NULL}};
const Segment SGI283095[] = {{&S56613[13], 3}, {&S56613[17], 13}, {&S56613[51], 17}, {&S56613[81], 44}, {NULL}};
const Segment SGI283096[] = {{&S56614[136], 4}, {NULL}};
const Segment SGI283097[] = {{&S56614[13], 3}, {&S56614[17], 13}, {&S56614[51], 17}, {&S56614[81], 44}, {NULL}};
const Segment SGI283098[] = {{&S56615[136], 4}, {NULL}};
const Segment SGI283099[] = {{&S56615[13], 3}, {&S56615[17], 13}, {&S56615[51], 17}, {&S56615[81], 44}, {NULL}};
const Segment SGI283100[] = {{&S56616[136], 4}, {NULL}};
const Segment SGI283101[] = {{&S56616[13], 3}, {&S56616[17], 13}, {&S56616[51], 17}, {&S56616[81], 44}, {NULL}};
const Segment SGI283102[] = {{&S56617[136], 4}, {NULL}};
const Segment SGI283103[] = {{&S56617[13], 3}, {&S56617[17], 13}, {&S56617[51], 17}, {&S56617[81], 44}, {NULL}};
const Segment SGI283104[] = {{&S56618[136], 4}, {NULL}};
const Segment SGI283105[] = {{&S56618[13], 3}, {&S56618[17], 13}, {&S56618[51], 17}, {&S56618[81], 44}, {NULL}};
const Segment SGI283106[] = {{&S56619[136], 4}, {NULL}};
const Segment SGI283107[] = {{&S56619[13], 3}, {&S56619[17], 13}, {&S56619[51], 17}, {&S56619[81], 44}, {NULL}};
const Segment SGI283108[] = {{&S56620[136], 4}, {NULL}};
const Segment SGI283109[] = {{&S56620[13], 3}, {&S56620[17], 13}, {&S56620[51], 17}, {&S56620[81], 44}, {NULL}};
const Segment SGI283110[] = {{&S56621[136], 4}, {NULL}};
const Segment SGI283111[] = {{&S56621[13], 3}, {&S56621[17], 13}, {&S56621[51], 17}, {&S56621[81], 44}, {NULL}};
const Segment SGI283112[] = {{&S56622[136], 4}, {NULL}};
const Segment SGI283113[] = {{&S56622[13], 3}, {&S56622[17], 13}, {&S56622[51], 17}, {&S56622[81], 44}, {NULL}};
const Segment SGI283114[] = {{&S56623[136], 4}, {NULL}};
const Segment SGI283115[] = {{&S56623[13], 3}, {&S56623[17], 13}, {&S56623[51], 17}, {&S56623[81], 44}, {NULL}};
const Segment SGI283116[] = {{&S56624[136], 4}, {NULL}};
const Segment SGI283117[] = {{&S56624[13], 3}, {&S56624[17], 13}, {&S56624[51], 17}, {&S56624[81], 44}, {NULL}};
const Segment SGI283118[] = {{&S56625[136], 4}, {NULL}};
const Segment SGI283119[] = {{&S56625[13], 3}, {&S56625[17], 13}, {&S56625[51], 17}, {&S56625[81], 44}, {NULL}};
const Segment SGI283120[] = {{&S56626[136], 4}, {NULL}};
const Segment SGI283121[] = {{&S56626[13], 3}, {&S56626[17], 13}, {&S56626[51], 17}, {&S56626[81], 44}, {NULL}};
const Segment SGI283122[] = {{&S56627[136], 4}, {NULL}};
const Segment SGI283123[] = {{&S56627[13], 3}, {&S56627[17], 13}, {&S56627[51], 17}, {&S56627[81], 44}, {NULL}};
const Segment SGI283124[] = {{&S56628[136], 4}, {NULL}};
const Segment SGI283125[] = {{&S56628[13], 3}, {&S56628[17], 13}, {&S56628[51], 17}, {&S56628[81], 44}, {NULL}};
const Segment SGI283126[] = {{&S56629[136], 4}, {NULL}};
const Segment SGI283127[] = {{&S56629[13], 3}, {&S56629[17], 13}, {&S56629[51], 17}, {&S56629[81], 44}, {NULL}};
const Segment SGI283128[] = {{&S56630[136], 4}, {NULL}};
const Segment SGI283129[] = {{&S56630[13], 3}, {&S56630[17], 13}, {&S56630[51], 17}, {&S56630[81], 44}, {NULL}};
const Segment SGI283130[] = {{&S56631[136], 4}, {NULL}};
const Segment SGI283131[] = {{&S56631[13], 3}, {&S56631[17], 13}, {&S56631[51], 17}, {&S56631[81], 44}, {NULL}};
const Segment SGI283132[] = {{&S56632[136], 4}, {NULL}};
const Segment SGI283133[] = {{&S56632[13], 3}, {&S56632[17], 13}, {&S56632[51], 17}, {&S56632[81], 44}, {NULL}};
const Segment SGI283134[] = {{&S56633[136], 4}, {NULL}};
const Segment SGI283135[] = {{&S56633[13], 3}, {&S56633[17], 13}, {&S56633[51], 17}, {&S56633[81], 44}, {NULL}};
const Segment SGI283136[] = {{&S56634[136], 4}, {NULL}};
const Segment SGI283137[] = {{&S56634[13], 3}, {&S56634[17], 13}, {&S56634[51], 17}, {&S56634[81], 44}, {NULL}};
const Segment SGI283138[] = {{&S56635[136], 4}, {NULL}};
const Segment SGI283139[] = {{&S56635[13], 3}, {&S56635[17], 13}, {&S56635[51], 17}, {&S56635[81], 44}, {NULL}};
const Segment SGI283140[] = {{&S56636[136], 4}, {NULL}};
const Segment SGI283141[] = {{&S56636[13], 3}, {&S56636[17], 13}, {&S56636[51], 17}, {&S56636[81], 44}, {NULL}};
const Segment SGI283142[] = {{&S56637[136], 4}, {NULL}};
const Segment SGI283143[] = {{&S56637[13], 3}, {&S56637[17], 13}, {&S56637[51], 17}, {&S56637[81], 44}, {NULL}};
const Segment SGI283144[] = {{&S56638[136], 4}, {NULL}};
const Segment SGI283145[] = {{&S56638[13], 3}, {&S56638[17], 13}, {&S56638[51], 17}, {&S56638[81], 44}, {NULL}};
const Segment SGI283146[] = {{&S56639[136], 4}, {NULL}};
const Segment SGI283147[] = {{&S56639[13], 3}, {&S56639[17], 13}, {&S56639[51], 17}, {&S56639[81], 44}, {NULL}};
const Segment SGI283148[] = {{&S56640[136], 4}, {NULL}};
const Segment SGI283149[] = {{&S56640[13], 3}, {&S56640[17], 13}, {&S56640[51], 17}, {&S56640[81], 44}, {NULL}};
const Segment SGI283150[] = {{&S65537[113], 3}, {NULL}};
const Segment SGI283151[] = {{&S65537[13], 7}, {&S65537[21], 2}, {&S65537[24], 15}, {&S65537[40], 6}, {&S65537[68], 17}, {NULL}};
const Segment SGI283152[] = {{&S65537[96], 3}, {NULL}};
const Segment SGI283153[] = {{&S65648[38], 3}, {NULL}};
const Segment SGI283154[] = {{&S56641[136], 4}, {NULL}};
const Segment SGI283155[] = {{&S56641[13], 3}, {&S56641[17], 13}, {&S56641[51], 17}, {&S56641[81], 44}, {NULL}};
const Segment SGI283156[] = {{&S56642[136], 4}, {NULL}};
const Segment SGI283157[] = {{&S56642[13], 3}, {&S56642[17], 13}, {&S56642[51], 17}, {&S56642[81], 44}, {NULL}};
const Segment SGI283158[] = {{&S56643[136], 4}, {NULL}};
const Segment SGI283159[] = {{&S56643[13], 3}, {&S56643[17], 13}, {&S56643[51], 17}, {&S56643[81], 44}, {NULL}};
const Segment SGI283160[] = {{&S56644[136], 4}, {NULL}};
const Segment SGI283161[] = {{&S56644[13], 3}, {&S56644[17], 13}, {&S56644[51], 17}, {&S56644[81], 44}, {NULL}};
const Segment SGI283162[] = {{&S56645[136], 4}, {NULL}};
const Segment SGI283163[] = {{&S56645[13], 3}, {&S56645[17], 13}, {&S56645[51], 17}, {&S56645[81], 44}, {NULL}};
const Segment SGI283164[] = {{&S65819[8], 3}, {NULL}};
const Segment SGI283165[] = {{&S56646[136], 4}, {NULL}};
const Segment SGI283166[] = {{&S65819[13], 7}, {&S65819[21], 2}, {&S65819[24], 15}, {&S65819[40], 6}, {&S65819[67], 17}, {NULL}};
const Segment SGI283167[] = {{&S56646[13], 3}, {&S56646[17], 13}, {&S56646[51], 17}, {&S56646[81], 44}, {NULL}};
const Segment SGI283168[] = {{&S65819[86], 8}, {NULL}};
const Segment SGI283169[] = {{&S56647[136], 4}, {NULL}};
const Segment SGI283170[] = {{&S56647[13], 3}, {&S56647[17], 13}, {&S56647[51], 17}, {&S56647[81], 44}, {NULL}};
const Segment SGI283171[] = {{&S56648[136], 4}, {NULL}};
const Segment SGI283172[] = {{&S56648[13], 3}, {&S56648[17], 13}, {&S56648[51], 17}, {&S56648[81], 44}, {NULL}};
const Segment SGI283173[] = {{&S56649[136], 4}, {NULL}};
const Segment SGI283174[] = {{&S56649[13], 3}, {&S56649[17], 13}, {&S56649[51], 17}, {&S56649[81], 44}, {NULL}};
const Segment SGI283175[] = {{&S56650[136], 4}, {NULL}};
const Segment SGI283176[] = {{&S56650[13], 3}, {&S56650[17], 13}, {&S56650[51], 17}, {&S56650[81], 44}, {NULL}};
const Segment SGI283177[] = {{&S56651[136], 4}, {NULL}};
const Segment SGI283178[] = {{&S58611[90], 4}, {NULL}};
const Segment SGI283179[] = {{&S56651[13], 3}, {&S56651[17], 13}, {&S56651[51], 17}, {&S56651[81], 44}, {NULL}};
const Segment SGI283180[] = {{&S58611[13], 3}, {&S58611[17], 11}, {&S58611[50], 17}, {NULL}};
const Segment SGI283181[] = {{&S58611[69], 11}, {NULL}};
const Segment SGI283182[] = {{&S56652[136], 4}, {NULL}};
const Segment SGI283183[] = {{&S56652[13], 3}, {&S56652[17], 13}, {&S56652[51], 17}, {&S56652[81], 44}, {NULL}};
const Segment SGI283184[] = {{&S58612[92], 4}, {NULL}};
const Segment SGI283185[] = {{&S58612[13], 3}, {&S58612[17], 13}, {&S58612[52], 17}, {NULL}};
const Segment SGI283186[] = {{&S58612[71], 11}, {NULL}};
const Segment SGI283187[] = {{&S56653[136], 4}, {NULL}};
const Segment SGI283188[] = {{&S56653[13], 3}, {&S56653[17], 13}, {&S56653[51], 17}, {&S56653[81], 44}, {NULL}};
const Segment SGI283189[] = {{&S56654[136], 4}, {NULL}};
const Segment SGI283190[] = {{&S56654[13], 3}, {&S56654[17], 13}, {&S56654[51], 17}, {&S56654[81], 44}, {NULL}};
const Segment SGI283191[] = {{&S56655[136], 4}, {NULL}};
const Segment SGI283192[] = {{&S56655[13], 3}, {&S56655[17], 13}, {&S56655[51], 17}, {&S56655[81], 44}, {NULL}};
const Segment SGI283193[] = {{&S56656[136], 4}, {NULL}};
const Segment SGI283194[] = {{&S56656[13], 3}, {&S56656[17], 13}, {&S56656[51], 17}, {&S56656[81], 44}, {NULL}};
const Segment SGI283195[] = {{&S56657[136], 4}, {NULL}};
const Segment SGI283196[] = {{&S56657[13], 3}, {&S56657[17], 13}, {&S56657[51], 17}, {&S56657[81], 44}, {NULL}};
const Segment SGI283197[] = {{&S56658[136], 4}, {NULL}};
const Segment SGI283198[] = {{&S56658[13], 3}, {&S56658[17], 13}, {&S56658[51], 17}, {&S56658[81], 44}, {NULL}};
const Segment SGI283199[] = {{&S56659[136], 4}, {NULL}};
const Segment SGI283200[] = {{&S56659[13], 3}, {&S56659[17], 13}, {&S56659[51], 17}, {&S56659[81], 44}, {NULL}};
const Segment SGI283201[] = {{&S56660[136], 4}, {NULL}};
const Segment SGI283202[] = {{&S56660[13], 3}, {&S56660[17], 13}, {&S56660[51], 17}, {&S56660[81], 44}, {NULL}};
const Segment SGI283203[] = {{&S56661[136], 4}, {NULL}};
const Segment SGI283204[] = {{&S56661[13], 3}, {&S56661[17], 13}, {&S56661[51], 17}, {&S56661[81], 44}, {NULL}};
const Segment SGI283205[] = {{&S56662[136], 4}, {NULL}};
const Segment SGI283206[] = {{&S56662[13], 3}, {&S56662[17], 13}, {&S56662[51], 17}, {&S56662[81], 44}, {NULL}};
const Segment SGI283207[] = {{&S56663[136], 4}, {NULL}};
const Segment SGI283208[] = {{&S56663[13], 3}, {&S56663[17], 13}, {&S56663[51], 17}, {&S56663[81], 44}, {NULL}};
const Segment SGI283209[] = {{&S56664[136], 4}, {NULL}};
const Segment SGI283210[] = {{&S56664[13], 3}, {&S56664[17], 13}, {&S56664[51], 17}, {&S56664[81], 44}, {NULL}};
const Segment SGI283211[] = {{&S56665[136], 4}, {NULL}};
const Segment SGI283212[] = {{&S56665[13], 3}, {&S56665[17], 13}, {&S56665[51], 17}, {&S56665[81], 44}, {NULL}};
const Segment SGI283213[] = {{&S56666[136], 4}, {NULL}};
const Segment SGI283214[] = {{&S56666[13], 3}, {&S56666[17], 13}, {&S56666[51], 17}, {&S56666[81], 44}, {NULL}};
const Segment SGI283215[] = {{&S56667[136], 4}, {NULL}};
const Segment SGI283216[] = {{&S56667[13], 3}, {&S56667[17], 13}, {&S56667[51], 17}, {&S56667[81], 44}, {NULL}};
const Segment SGI283217[] = {{&S56668[136], 4}, {NULL}};
const Segment SGI283218[] = {{&S56668[13], 3}, {&S56668[17], 13}, {&S56668[51], 17}, {&S56668[81], 44}, {NULL}};
const Segment SGI283219[] = {{&S56669[136], 4}, {NULL}};
const Segment SGI283220[] = {{&S56669[13], 3}, {&S56669[17], 13}, {&S56669[51], 17}, {&S56669[81], 44}, {NULL}};
const Segment SGI283221[] = {{&S56670[136], 4}, {NULL}};
const Segment SGI283222[] = {{&S56670[13], 3}, {&S56670[17], 13}, {&S56670[51], 17}, {&S56670[81], 44}, {NULL}};
const Segment SGI283223[] = {{&S56671[136], 4}, {NULL}};
const Segment SGI283224[] = {{&S56671[13], 3}, {&S56671[17], 13}, {&S56671[51], 17}, {&S56671[81], 44}, {NULL}};
const Segment SGI283225[] = {{&S56672[136], 4}, {NULL}};
const Segment SGI283226[] = {{&S56672[13], 3}, {&S56672[17], 13}, {&S56672[51], 17}, {&S56672[81], 44}, {NULL}};
const Segment SGI283227[] = {{&S56673[136], 4}, {NULL}};
const Segment SGI283228[] = {{&S56673[13], 3}, {&S56673[17], 13}, {&S56673[51], 17}, {&S56673[81], 44}, {NULL}};
const Segment SGI283229[] = {{&S56674[136], 4}, {NULL}};
const Segment SGI283230[] = {{&S56674[13], 3}, {&S56674[17], 13}, {&S56674[51], 17}, {&S56674[81], 44}, {NULL}};
const Segment SGI283231[] = {{&S56675[136], 4}, {NULL}};
const Segment SGI283232[] = {{&S56675[13], 3}, {&S56675[17], 13}, {&S56675[51], 17}, {&S56675[81], 44}, {NULL}};
const Segment SGI283233[] = {{&S56676[136], 4}, {NULL}};
const Segment SGI283234[] = {{&S56676[13], 3}, {&S56676[17], 13}, {&S56676[51], 17}, {&S56676[81], 44}, {NULL}};
const Segment SGI283235[] = {{&S56677[136], 4}, {NULL}};
const Segment SGI283236[] = {{&S56677[13], 3}, {&S56677[17], 13}, {&S56677[51], 17}, {&S56677[81], 44}, {NULL}};
const Segment SGI283237[] = {{&S56678[136], 4}, {NULL}};
const Segment SGI283238[] = {{&S56678[13], 3}, {&S56678[17], 13}, {&S56678[51], 17}, {&S56678[81], 44}, {NULL}};
const Segment SGI283239[] = {{&S56679[136], 4}, {NULL}};
const Segment SGI283240[] = {{&S56679[13], 3}, {&S56679[17], 13}, {&S56679[51], 17}, {&S56679[81], 44}, {NULL}};
const Segment SGI283241[] = {{&S56680[136], 4}, {NULL}};
const Segment SGI283242[] = {{&S56680[13], 3}, {&S56680[17], 13}, {&S56680[51], 17}, {&S56680[81], 44}, {NULL}};
const Segment SGI283243[] = {{&S56681[136], 4}, {NULL}};
const Segment SGI283244[] = {{&S56681[13], 3}, {&S56681[17], 13}, {&S56681[51], 17}, {&S56681[81], 44}, {NULL}};
const Segment SGI283245[] = {{&S56682[136], 4}, {NULL}};
const Segment SGI283246[] = {{&S56682[13], 3}, {&S56682[17], 13}, {&S56682[51], 17}, {&S56682[81], 44}, {NULL}};
const Segment SGI283247[] = {{&S56683[136], 4}, {NULL}};
const Segment SGI283248[] = {{&S56683[13], 3}, {&S56683[17], 13}, {&S56683[51], 17}, {&S56683[81], 44}, {NULL}};
const Segment SGI283249[] = {{&S56684[136], 4}, {NULL}};
const Segment SGI283250[] = {{&S56684[13], 3}, {&S56684[17], 13}, {&S56684[51], 17}, {&S56684[81], 44}, {NULL}};
const Segment SGI283251[] = {{&S56685[136], 4}, {NULL}};
const Segment SGI283252[] = {{&S56685[13], 3}, {&S56685[17], 13}, {&S56685[51], 17}, {&S56685[81], 44}, {NULL}};
const Segment SGI283253[] = {{&S56686[136], 4}, {NULL}};
const Segment SGI283254[] = {{&S56686[13], 3}, {&S56686[17], 13}, {&S56686[51], 17}, {&S56686[81], 44}, {NULL}};
const Segment SGI283255[] = {{&S56687[136], 4}, {NULL}};
const Segment SGI283256[] = {{&S56687[13], 3}, {&S56687[17], 13}, {&S56687[51], 17}, {&S56687[81], 44}, {NULL}};
const Segment SGI283257[] = {{&S56688[136], 4}, {NULL}};
const Segment SGI283258[] = {{&S56688[13], 3}, {&S56688[17], 13}, {&S56688[51], 17}, {&S56688[81], 44}, {NULL}};
const Segment SGI283259[] = {{&S56689[136], 4}, {NULL}};
const Segment SGI283260[] = {{&S56689[13], 3}, {&S56689[17], 13}, {&S56689[51], 17}, {&S56689[81], 44}, {NULL}};
const Segment SGI283261[] = {{&S56690[136], 4}, {NULL}};
const Segment SGI283262[] = {{&S56690[13], 3}, {&S56690[17], 13}, {&S56690[51], 17}, {&S56690[81], 44}, {NULL}};
const Segment SGI283263[] = {{&S56691[136], 4}, {NULL}};
const Segment SGI283264[] = {{&S56691[13], 3}, {&S56691[17], 13}, {&S56691[51], 17}, {&S56691[81], 44}, {NULL}};
const Segment SGI283265[] = {{&S56692[136], 4}, {NULL}};
const Segment SGI283266[] = {{&S56692[13], 3}, {&S56692[17], 13}, {&S56692[51], 17}, {&S56692[81], 44}, {NULL}};
const Segment SGI283267[] = {{&S56693[136], 4}, {NULL}};
const Segment SGI283268[] = {{&S56693[13], 3}, {&S56693[17], 13}, {&S56693[51], 17}, {&S56693[81], 44}, {NULL}};
const Segment SGI283269[] = {{&S56694[136], 4}, {NULL}};
const Segment SGI283270[] = {{&S56694[13], 3}, {&S56694[17], 13}, {&S56694[51], 17}, {&S56694[81], 44}, {NULL}};
const Segment SGI283271[] = {{&S56695[136], 4}, {NULL}};
const Segment SGI283272[] = {{&S56695[13], 3}, {&S56695[17], 13}, {&S56695[51], 17}, {&S56695[81], 44}, {NULL}};
const Segment SGI283273[] = {{&S56696[136], 4}, {NULL}};
const Segment SGI283274[] = {{&S56696[13], 3}, {&S56696[17], 13}, {&S56696[51], 17}, {&S56696[81], 44}, {NULL}};
const Segment SGI283275[] = {{&S56697[136], 4}, {NULL}};
const Segment SGI283276[] = {{&S56697[13], 3}, {&S56697[17], 13}, {&S56697[51], 17}, {&S56697[81], 44}, {NULL}};
const Segment SGI283277[] = {{&S56698[136], 4}, {NULL}};
const Segment SGI283278[] = {{&S56698[13], 3}, {&S56698[17], 13}, {&S56698[51], 17}, {&S56698[81], 44}, {NULL}};
const Segment SGI283279[] = {{&S56699[136], 4}, {NULL}};
const Segment SGI283280[] = {{&S56699[13], 3}, {&S56699[17], 13}, {&S56699[51], 17}, {&S56699[81], 44}, {NULL}};
const Segment SGI283281[] = {{&S58759[75], 4}, {NULL}};
const Segment SGI283282[] = {{&S58759[13], 14}, {&S58759[49], 17}, {NULL}};
const Segment SGI283283[] = {{&S58759[111], 3}, {NULL}};
const Segment SGI283284[] = {{&S58762[75], 4}, {NULL}};
const Segment SGI283285[] = {{&S58762[13], 14}, {&S58762[49], 17}, {NULL}};
const Segment SGI283286[] = {{&S58762[111], 3}, {NULL}};
const Segment SGI283287[] = {{&S58776[75], 4}, {NULL}};
const Segment SGI283288[] = {{&S58776[13], 14}, {&S58776[49], 17}, {NULL}};
const Segment SGI283289[] = {{&S58777[82], 4}, {NULL}};
const Segment SGI283290[] = {{&S58777[13], 14}, {&S58777[28], 6}, {&S58777[56], 17}, {NULL}};
const Segment SGI283291[] = {{&S58778[82], 4}, {NULL}};
const Segment SGI283292[] = {{&S58778[13], 14}, {&S58778[28], 6}, {&S58778[56], 17}, {NULL}};
const Segment SGI283293[] = {{&S56874[76], 4}, {NULL}};
const Segment SGI283294[] = {{&S56874[13], 15}, {&S56874[50], 17}, {NULL}};
const Segment SGI283295[] = {{&S59025[74], 4}, {NULL}};
const Segment SGI283296[] = {{&S59025[13], 14}, {&S59025[49], 17}, {NULL}};
const Segment SGI283297[] = {{&S59025[69], 9}, {NULL}};
const Segment SGI283298[] = {{&S59026[81], 4}, {NULL}};
const Segment SGI283299[] = {{&S59026[13], 14}, {&S59026[28], 6}, {&S59026[56], 17}, {NULL}};
const Segment SGI283300[] = {{&S59026[76], 9}, {NULL}};
const Segment SGI283301[] = {{&S59038[114], 4}, {NULL}};
const Segment SGI283302[] = {{&S59038[13], 3}, {&S59038[17], 13}, {&S59038[31], 28}, {&S59038[60], 6}, {&S59038[88], 17}, {NULL}};
const Segment SGI283303[] = {{&S59039[114], 4}, {NULL}};
const Segment SGI283304[] = {{&S59039[13], 3}, {&S59039[17], 13}, {&S59039[31], 28}, {&S59039[60], 6}, {&S59039[88], 17}, {NULL}};
const Segment SGI283305[] = {{&S59040[114], 4}, {NULL}};
const Segment SGI283306[] = {{&S59040[13], 3}, {&S59040[17], 13}, {&S59040[31], 28}, {&S59040[60], 6}, {&S59040[88], 17}, {NULL}};
const Segment SGI283307[] = {{&S59041[114], 4}, {NULL}};
const Segment SGI283308[] = {{&S59041[13], 3}, {&S59041[17], 13}, {&S59041[31], 28}, {&S59041[60], 6}, {&S59041[88], 17}, {NULL}};
const Segment SGI283309[] = {{&S59042[114], 4}, {NULL}};
const Segment SGI283310[] = {{&S59042[13], 3}, {&S59042[17], 13}, {&S59042[31], 28}, {&S59042[60], 6}, {&S59042[88], 17}, {NULL}};
const Segment SGI283311[] = {{&S59043[114], 4}, {NULL}};
const Segment SGI283312[] = {{&S59043[13], 3}, {&S59043[17], 13}, {&S59043[31], 28}, {&S59043[60], 6}, {&S59043[88], 17}, {NULL}};
const Segment SGI283313[] = {{&S59044[119], 4}, {NULL}};
const Segment SGI283314[] = {{&S59044[13], 3}, {&S59044[17], 13}, {&S59044[31], 33}, {&S59044[65], 6}, {&S59044[93], 17}, {NULL}};
const Segment SGI283315[] = {{&S59045[114], 4}, {NULL}};
const Segment SGI283316[] = {{&S57128[74], 4}, {NULL}};
const Segment SGI283317[] = {{&S57128[13], 14}, {&S57128[48], 17}, {NULL}};
const Segment SGI283318[] = {{&S59045[13], 3}, {&S59045[17], 13}, {&S59045[31], 28}, {&S59045[60], 6}, {&S59045[88], 17}, {NULL}};
const Segment SGI283319[] = {{&S57128[86], 9}, {NULL}};
const Segment SGI283320[] = {{&S57129[81], 4}, {NULL}};
const Segment SGI283321[] = {{&S57129[13], 14}, {&S57129[28], 6}, {&S57129[55], 17}, {NULL}};
const Segment SGI283322[] = {{&S57129[93], 9}, {NULL}};
const Segment SGI283323[] = {{&S59046[119], 4}, {NULL}};
const Segment SGI283324[] = {{&S59046[13], 3}, {&S59046[17], 13}, {&S59046[31], 33}, {&S59046[65], 6}, {&S59046[93], 17}, {NULL}};
const Segment SGI283325[] = {{&S59047[119], 4}, {NULL}};
const Segment SGI283326[] = {{&S59047[13], 3}, {&S59047[17], 13}, {&S59047[31], 33}, {&S59047[65], 6}, {&S59047[93], 17}, {NULL}};
const Segment SGI283327[] = {{&S59048[114], 4}, {NULL}};
const Segment SGI283328[] = {{&S59048[13], 3}, {&S59048[17], 13}, {&S59048[31], 28}, {&S59048[60], 6}, {&S59048[88], 17}, {NULL}};
const Segment SGI283329[] = {{&S59049[114], 4}, {NULL}};
const Segment SGI283330[] = {{&S59049[13], 3}, {&S59049[17], 13}, {&S59049[31], 28}, {&S59049[60], 6}, {&S59049[88], 17}, {NULL}};
const Segment SGI283331[] = {{&S59050[119], 4}, {NULL}};
const Segment SGI283332[] = {{&S59050[13], 3}, {&S59050[17], 13}, {&S59050[31], 33}, {&S59050[65], 6}, {&S59050[93], 17}, {NULL}};
const Segment SGI283333[] = {{&S59051[114], 4}, {NULL}};
const Segment SGI283334[] = {{&S59051[13], 3}, {&S59051[17], 13}, {&S59051[31], 28}, {&S59051[60], 6}, {&S59051[88], 17}, {NULL}};
const Segment SGI283335[] = {{&S59079[76], 4}, {NULL}};
const Segment SGI283336[] = {{&S59079[13], 3}, {&S59079[17], 11}, {&S59079[50], 17}, {NULL}};
const Segment SGI283337[] = {{&S59079[69], 12}, {NULL}};
const Segment SGI283338[] = {{&S59080[78], 4}, {NULL}};
const Segment SGI283339[] = {{&S59080[13], 3}, {&S59080[17], 13}, {&S59080[52], 17}, {NULL}};
const Segment SGI283340[] = {{&S59080[71], 12}, {NULL}};
const Segment SGI283341[] = {{&S59081[86], 4}, {NULL}};
const Segment SGI283342[] = {{&S59081[13], 3}, {&S59081[17], 2}, {&S59081[20], 11}, {&S59081[32], 6}, {&S59081[60], 17}, {NULL}};
const Segment SGI283343[] = {{&S59081[79], 12}, {NULL}};
const Segment SGI283344[] = {{&S59134[72], 4}, {NULL}};
const Segment SGI283345[] = {{&S59134[13], 14}, {&S59134[48], 17}, {NULL}};
const Segment SGI283346[] = {{&S59134[67], 9}, {NULL}};
const Segment SGI283347[] = {{&S59135[79], 4}, {NULL}};
const Segment SGI283348[] = {{&S59135[13], 14}, {&S59135[28], 6}, {&S59135[55], 17}, {NULL}};
const Segment SGI283349[] = {{&S59135[74], 9}, {NULL}};
const Segment SGI283350[] = {{&S57339[76], 4}, {NULL}};
const Segment SGI283351[] = {{&S57339[13], 15}, {&S57339[50], 17}, {NULL}};
const Segment SGI283352[] = {{&S57697[65], 4}, {NULL}};
const Segment SGI283353[] = {{&S57697[13], 5}, {&S57697[39], 17}, {NULL}};
const Segment SGI283354[] = {{&S59370[75], 4}, {NULL}};
const Segment SGI283355[] = {{&S59370[13], 14}, {&S59370[49], 17}, {NULL}};
const Segment SGI283356[] = {{&S59370[111], 4}, {NULL}};
const Segment SGI283357[] = {{&S57934[80], 4}, {NULL}};
const Segment SGI283358[] = {{&S57934[13], 14}, {&S57934[28], 6}, {&S57934[56], 17}, {NULL}};
const Segment SGI283359[] = {{&S57934[75], 9}, {NULL}};
const Segment SGI283360[] = {{&S57935[73], 4}, {NULL}};
const Segment SGI283361[] = {{&S57935[13], 14}, {&S57935[49], 17}, {NULL}};
const Segment SGI283362[] = {{&S57935[68], 9}, {NULL}};
const Segment SGI283363[] = {{&S59607[81], 4}, {NULL}};
const Segment SGI283364[] = {{&S59607[13], 14}, {&S59607[48], 17}, {NULL}};
const Segment SGI283365[] = {{&S59607[67], 18}, {NULL}};
const Segment SGI283366[] = {{&S59608[88], 4}, {NULL}};
const Segment SGI283367[] = {{&S59608[13], 14}, {&S59608[28], 6}, {&S59608[55], 17}, {NULL}};
const Segment SGI283368[] = {{&S59608[74], 18}, {NULL}};
const Segment SGI283369[] = {{&S59621[80], 4}, {NULL}};
const Segment SGI283370[] = {{&S59621[13], 3}, {&S59621[17], 13}, {&S59621[52], 17}, {&S59621[139], 14}, {NULL}};
const Segment SGI283371[] = {{&S59621[125], 12}, {NULL}};
const Segment SGI283372[] = {{&S59622[78], 4}, {NULL}};
const Segment SGI283373[] = {{&S59622[13], 3}, {&S59622[17], 11}, {&S59622[50], 17}, {&S59622[137], 14}, {NULL}};
const Segment SGI283374[] = {{&S59622[123], 12}, {NULL}};
const Segment SGI283375[] = {{&S59623[80], 4}, {NULL}};
const Segment SGI283376[] = {{&S59623[13], 3}, {&S59623[17], 13}, {&S59623[52], 17}, {&S59623[139], 14}, {NULL}};
const Segment SGI283377[] = {{&S59623[125], 12}, {NULL}};
const Segment SGI283378[] = {{&S59624[78], 4}, {NULL}};
const Segment SGI283379[] = {{&S59624[13], 3}, {&S59624[17], 11}, {&S59624[50], 17}, {&S59624[137], 14}, {NULL}};
const Segment SGI283380[] = {{&S59624[123], 12}, {NULL}};
const Segment SGI283381[] = {{&S58247[74], 4}, {NULL}};
const Segment SGI283382[] = {{&S58247[13], 14}, {&S58247[48], 17}, {NULL}};
const Segment SGI283383[] = {{&S58247[86], 9}, {NULL}};
const Segment SGI283384[] = {{&S58248[81], 4}, {NULL}};
const Segment SGI283385[] = {{&S58248[13], 14}, {&S58248[28], 6}, {&S58248[55], 17}, {NULL}};
const Segment SGI283386[] = {{&S58248[93], 9}, {NULL}};
const Segment SGI283387[] = {{&S59940[112], 4}, {NULL}};
const Segment SGI283388[] = {{&S59940[13], 9}, {&S59940[23], 22}, {&S59940[67], 17}, {NULL}};
const Segment SGI283389[] = {{&S59940[86], 11}, {NULL}};
const Segment SGI283390[] = {{&S59941[112], 4}, {NULL}};
const Segment SGI283391[] = {{&S59941[13], 9}, {&S59941[23], 22}, {&S59941[67], 17}, {NULL}};
const Segment SGI283392[] = {{&S59941[86], 11}, {NULL}};
const Segment SGI283393[] = {{&S59945[112], 4}, {NULL}};
const Segment SGI283394[] = {{&S59945[13], 9}, {&S59945[23], 22}, {&S59945[67], 17}, {NULL}};
const Segment SGI283395[] = {{&S59945[86], 11}, {NULL}};
const Segment SGI283396[] = {{&S60017[101], 4}, {NULL}};
const Segment SGI283397[] = {{&S60017[13], 14}, {&S60017[28], 6}, {&S60017[56], 17}, {NULL}};
const Segment SGI283398[] = {{&S60017[75], 11}, {NULL}};
const Segment SGI283399[] = {{&S60018[94], 4}, {NULL}};
const Segment SGI283400[] = {{&S60018[13], 14}, {&S60018[49], 17}, {NULL}};
const Segment SGI283401[] = {{&S60018[68], 11}, {NULL}};
const Segment SGI283402[] = {{&S58410[76], 4}, {NULL}};
const Segment SGI283403[] = {{&S58410[13], 15}, {&S58410[50], 17}, {NULL}};
const Segment SGI283404[] = {{&S58479[93], 4}, {NULL}};
const Segment SGI283405[] = {{&S58479[13], 9}, {&S58479[23], 22}, {&S58479[67], 17}, {NULL}};
const Segment SGI283406[] = {{&S58479[105], 9}, {NULL}};
const Segment SGI283407[] = {{&S58480[93], 4}, {NULL}};
const Segment SGI283408[] = {{&S58480[13], 9}, {&S58480[23], 22}, {&S58480[67], 17}, {NULL}};
const Segment SGI283409[] = {{&S58480[105], 9}, {NULL}};
const Segment SGI283410[] = {{&S58481[93], 4}, {NULL}};
const Segment SGI283411[] = {{&S58481[13], 9}, {&S58481[23], 22}, {&S58481[67], 17}, {NULL}};
const Segment SGI283412[] = {{&S58481[105], 9}, {NULL}};
const Segment SGI283413[] = {{&S58482[93], 4}, {NULL}};
const Segment SGI283414[] = {{&S58482[13], 9}, {&S58482[23], 22}, {&S58482[67], 17}, {NULL}};
const Segment SGI283415[] = {{&S58482[105], 9}, {NULL}};
const Segment SGI283416[] = {{&S60135[73], 4}, {NULL}};
const Segment SGI283417[] = {{&S60135[13], 14}, {&S60135[48], 17}, {NULL}};
const Segment SGI283418[] = {{&S60135[68], 9}, {NULL}};
const Segment SGI283419[] = {{&S60136[80], 4}, {NULL}};
const Segment SGI283420[] = {{&S60136[13], 14}, {&S60136[28], 6}, {&S60136[55], 17}, {NULL}};
const Segment SGI283421[] = {{&S60136[75], 9}, {NULL}};
const Segment SGI283422[] = {{&S60177[103], 4}, {NULL}};
const Segment SGI283423[] = {{&S60177[13], 9}, {&S60177[23], 2}, {&S60177[26], 22}, {&S60177[49], 6}, {&S60177[77], 17}, {NULL}};
const Segment SGI283424[] = {{&S60178[103], 4}, {NULL}};
const Segment SGI283425[] = {{&S60178[13], 9}, {&S60178[23], 2}, {&S60178[26], 22}, {&S60178[49], 6}, {&S60178[77], 17}, {NULL}};
const Segment SGI283426[] = {{&S60179[103], 4}, {NULL}};
const Segment SGI283427[] = {{&S60179[13], 9}, {&S60179[23], 2}, {&S60179[26], 22}, {&S60179[49], 6}, {&S60179[77], 17}, {NULL}};
const Segment SGI283428[] = {{&S60180[103], 4}, {NULL}};
const Segment SGI283429[] = {{&S60180[13], 9}, {&S60180[23], 2}, {&S60180[26], 22}, {&S60180[49], 6}, {&S60180[77], 17}, {NULL}};
const Segment SGI283430[] = {{&S60181[103], 4}, {NULL}};
const Segment SGI283431[] = {{&S60181[13], 9}, {&S60181[23], 2}, {&S60181[26], 22}, {&S60181[49], 6}, {&S60181[77], 17}, {NULL}};
const Segment SGI283432[] = {{&S60182[75], 4}, {NULL}};
const Segment SGI283433[] = {{&S60182[13], 14}, {&S60182[49], 17}, {NULL}};
const Segment SGI283434[] = {{&S60183[106], 4}, {NULL}};
const Segment SGI283435[] = {{&S60183[13], 46}, {&S60183[60], 11}, {&S60183[72], 9}, {&S60183[82], 15}, {NULL}};
const Segment SGI283436[] = {{&S60184[106], 4}, {NULL}};
const Segment SGI283437[] = {{&S60184[13], 46}, {&S60184[60], 11}, {&S60184[72], 9}, {&S60184[82], 15}, {NULL}};
const Segment SGI283438[] = {{&S60185[111], 4}, {NULL}};
const Segment SGI283439[] = {{&S60185[13], 46}, {&S60185[60], 11}, {&S60185[72], 9}, {&S60185[82], 20}, {NULL}};
const Segment SGI283440[] = {{&S67631[91], 3}, {NULL}};
const Segment SGI283441[] = {{&S67631[13], 7}, {&S67631[21], 2}, {&S67631[24], 15}, {&S67631[40], 6}, {&S67631[67], 17}, {NULL}};
const Segment SGI283442[] = {{&S67631[86], 8}, {NULL}};
const Segment SGI283443[] = {{&S67647[101], 3}, {NULL}};
const Segment SGI283444[] = {{&S67647[13], 7}, {&S67647[21], 2}, {&S67647[24], 15}, {&S67647[40], 6}, {&S67647[68], 17}, {NULL}};
const Segment SGI283445[] = {{&S67647[87], 17}, {NULL}};
const Segment SGI283446[] = {{&S60252[93], 4}, {NULL}};
const Segment SGI283447[] = {{&S60252[13], 3}, {&S60252[17], 13}, {&S60252[52], 17}, {NULL}};
const Segment SGI283448[] = {{&S60252[71], 12}, {NULL}};
const Segment SGI283449[] = {{&S60253[91], 4}, {NULL}};
const Segment SGI283450[] = {{&S60253[13], 3}, {&S60253[17], 11}, {&S60253[50], 17}, {NULL}};
const Segment SGI283451[] = {{&S60253[69], 12}, {NULL}};
const Segment SGI283452[] = {{&S60254[79], 4}, {NULL}};
const Segment SGI283453[] = {{&S60254[13], 3}, {&S60254[17], 11}, {&S60254[51], 17}, {&S60254[139], 14}, {NULL}};
const Segment SGI283454[] = {{&S60254[125], 12}, {NULL}};
const Segment SGI283455[] = {{&S60255[81], 4}, {NULL}};
const Segment SGI283456[] = {{&S60255[13], 3}, {&S60255[17], 13}, {&S60255[53], 17}, {&S60255[141], 14}, {NULL}};
const Segment SGI283457[] = {{&S60255[127], 12}, {NULL}};
const Segment SGI283458[] = {{&S60256[81], 4}, {NULL}};
const Segment SGI283459[] = {{&S60256[13], 3}, {&S60256[17], 13}, {&S60256[53], 17}, {&S60256[141], 16}, {NULL}};
const Segment SGI283460[] = {{&S60256[127], 12}, {NULL}};
const Segment SGI283461[] = {{&S60257[79], 4}, {NULL}};
const Segment SGI283462[] = {{&S60257[13], 3}, {&S60257[17], 11}, {&S60257[51], 17}, {&S60257[139], 16}, {NULL}};
const Segment SGI283463[] = {{&S60257[125], 12}, {NULL}};
const Segment SGI283464[] = {{&S58677[93], 4}, {NULL}};
const Segment SGI283465[] = {{&S58677[13], 9}, {&S58677[23], 22}, {&S58677[67], 17}, {NULL}};
const Segment SGI283466[] = {{&S58677[105], 9}, {NULL}};
const Segment SGI283467[] = {{&S58678[93], 4}, {NULL}};
const Segment SGI283468[] = {{&S58678[13], 9}, {&S58678[23], 22}, {&S58678[67], 17}, {NULL}};
const Segment SGI283469[] = {{&S58678[105], 9}, {NULL}};
const Segment SGI283470[] = {{&S58679[93], 4}, {NULL}};
const Segment SGI283471[] = {{&S58679[13], 9}, {&S58679[23], 22}, {&S58679[67], 17}, {NULL}};
const Segment SGI283472[] = {{&S58679[105], 9}, {NULL}};
const Segment SGI283473[] = {{&S58680[93], 4}, {NULL}};
const Segment SGI283474[] = {{&S58680[13], 9}, {&S58680[23], 22}, {&S58680[67], 17}, {NULL}};
const Segment SGI283475[] = {{&S58680[105], 9}, {NULL}};
const Segment SGI283476[] = {{&S58681[93], 4}, {NULL}};
const Segment SGI283477[] = {{&S58681[13], 9}, {&S58681[23], 22}, {&S58681[67], 17}, {NULL}};
const Segment SGI283478[] = {{&S58681[105], 9}, {NULL}};
const Segment SGI283479[] = {{&S60320[74], 4}, {NULL}};
const Segment SGI283480[] = {{&S60320[13], 14}, {&S60320[48], 17}, {NULL}};
const Segment SGI283481[] = {{&S60321[106], 4}, {NULL}};
const Segment SGI283482[] = {{&S60321[13], 46}, {&S60321[60], 11}, {&S60321[72], 9}, {&S60321[82], 15}, {NULL}};
const Segment SGI283483[] = {{&S60322[106], 4}, {NULL}};
const Segment SGI283484[] = {{&S60322[13], 46}, {&S60322[60], 11}, {&S60322[72], 9}, {&S60322[82], 15}, {NULL}};
const Segment SGI283485[] = {{&S60323[111], 4}, {NULL}};
const Segment SGI283486[] = {{&S60323[13], 46}, {&S60323[60], 11}, {&S60323[72], 9}, {&S60323[82], 20}, {NULL}};
const Segment SGI283487[] = {{&S58876[81], 4}, {NULL}};
const Segment SGI283488[] = {{&S58876[13], 14}, {&S58876[28], 6}, {&S58876[55], 17}, {NULL}};
const Segment SGI283489[] = {{&S58876[93], 9}, {NULL}};
const Segment SGI283490[] = {{&S58877[74], 4}, {NULL}};
const Segment SGI283491[] = {{&S58877[13], 14}, {&S58877[48], 17}, {NULL}};
const Segment SGI283492[] = {{&S58877[86], 9}, {NULL}};
const Segment SGI283493[] = {{&S58878[91], 4}, {NULL}};
const Segment SGI283494[] = {{&S58878[13], 3}, {&S58878[17], 11}, {&S58878[50], 17}, {NULL}};
const Segment SGI283495[] = {{&S58878[69], 12}, {NULL}};
const Segment SGI283496[] = {{&S58879[93], 4}, {NULL}};
const Segment SGI283497[] = {{&S58879[13], 3}, {&S58879[17], 13}, {&S58879[52], 17}, {NULL}};
const Segment SGI283498[] = {{&S58879[71], 12}, {NULL}};
const Segment SGI283499[] = {{&S58888[92], 4}, {NULL}};
const Segment SGI283500[] = {{&S58888[13], 9}, {&S58888[23], 22}, {&S58888[66], 17}, {NULL}};
const Segment SGI283501[] = {{&S58888[104], 9}, {NULL}};
const Segment SGI283502[] = {{&S58889[92], 4}, {NULL}};
const Segment SGI283503[] = {{&S58889[13], 9}, {&S58889[23], 22}, {&S58889[66], 17}, {NULL}};
const Segment SGI283504[] = {{&S58889[104], 9}, {NULL}};
const Segment SGI283505[] = {{&S58890[92], 4}, {NULL}};
const Segment SGI283506[] = {{&S58890[13], 9}, {&S58890[23], 22}, {&S58890[66], 17}, {NULL}};
const Segment SGI283507[] = {{&S58890[104], 9}, {NULL}};
const Segment SGI283508[] = {{&S58891[92], 4}, {NULL}};
const Segment SGI283509[] = {{&S58891[13], 9}, {&S58891[23], 22}, {&S58891[66], 17}, {NULL}};
const Segment SGI283510[] = {{&S58891[104], 9}, {NULL}};
const Segment SGI283511[] = {{&S58892[92], 4}, {NULL}};
const Segment SGI283512[] = {{&S58892[13], 9}, {&S58892[23], 22}, {&S58892[66], 17}, {NULL}};
const Segment SGI283513[] = {{&S58892[104], 9}, {NULL}};
const Segment SGI283514[] = {{&S58893[92], 4}, {NULL}};
const Segment SGI283515[] = {{&S58893[13], 9}, {&S58893[23], 22}, {&S58893[66], 17}, {NULL}};
const Segment SGI283516[] = {{&S58893[104], 9}, {NULL}};
const Segment SGI283517[] = {{&S58070[82], 4}, {NULL}};
const Segment SGI283518[] = {{&S58070[14], 14}, {&S58070[29], 6}, {&S58070[56], 17}, {NULL}};
const Segment SGI283519[] = {{&S58071[82], 4}, {NULL}};
const Segment SGI283520[] = {{&S58071[13], 14}, {&S58071[28], 6}, {&S58071[56], 17}, {NULL}};
const Segment SGI283521[] = {{&S58072[75], 4}, {NULL}};
const Segment SGI283522[] = {{&S58072[13], 14}, {&S58072[49], 17}, {NULL}};
const Segment SGI283523[] = {{&S60646[87], 4}, {NULL}};
const Segment SGI283524[] = {{&S60646[22], 3}, {&S60646[26], 13}, {&S60646[61], 17}, {NULL}};
const Segment SGI283525[] = {{&S60646[0], 9}, {NULL}};
const Segment SGI283526[] = {{&S60688[93], 4}, {NULL}};
const Segment SGI283527[] = {{&S60688[13], 9}, {&S60688[23], 22}, {&S60688[67], 17}, {NULL}};
const Segment SGI283528[] = {{&S60688[115], 3}, {NULL}};
const Segment SGI283529[] = {{&S60689[93], 4}, {NULL}};
const Segment SGI283530[] = {{&S60689[13], 9}, {&S60689[23], 22}, {&S60689[67], 17}, {NULL}};
const Segment SGI283531[] = {{&S60689[115], 3}, {NULL}};
const Segment SGI283532[] = {{&S60690[93], 4}, {NULL}};
const Segment SGI283533[] = {{&S60690[13], 9}, {&S60690[23], 22}, {&S60690[67], 17}, {NULL}};
const Segment SGI283534[] = {{&S60690[115], 3}, {NULL}};
const Segment SGI283535[] = {{&S60691[93], 4}, {NULL}};
const Segment SGI283536[] = {{&S60691[13], 9}, {&S60691[23], 22}, {&S60691[67], 17}, {NULL}};
const Segment SGI283537[] = {{&S60691[115], 3}, {NULL}};
const Segment SGI283538[] = {{&S60692[93], 4}, {NULL}};
const Segment SGI283539[] = {{&S60692[13], 9}, {&S60692[23], 22}, {&S60692[67], 17}, {NULL}};
const Segment SGI283540[] = {{&S60692[115], 3}, {NULL}};
const Segment SGI283541[] = {{&S60693[93], 4}, {NULL}};
const Segment SGI283542[] = {{&S60693[13], 9}, {&S60693[23], 22}, {&S60693[67], 17}, {NULL}};
const Segment SGI283543[] = {{&S60693[115], 3}, {NULL}};
const Segment SGI283544[] = {{&S59368[93], 4}, {NULL}};
const Segment SGI283545[] = {{&S59368[13], 3}, {&S59368[17], 13}, {&S59368[52], 17}, {NULL}};
const Segment SGI283546[] = {{&S59368[71], 12}, {NULL}};
const Segment SGI283547[] = {{&S59369[91], 4}, {NULL}};
const Segment SGI283548[] = {{&S59369[13], 3}, {&S59369[17], 11}, {&S59369[50], 17}, {NULL}};
const Segment SGI283549[] = {{&S59369[69], 12}, {NULL}};
const Segment SGI283550[] = {{&S59538[103], 4}, {NULL}};
const Segment SGI283551[] = {{&S59538[13], 9}, {&S59538[23], 2}, {&S59538[26], 22}, {&S59538[49], 6}, {&S59538[77], 17}, {NULL}};
const Segment SGI283552[] = {{&S59539[93], 4}, {NULL}};
const Segment SGI283553[] = {{&S59539[13], 9}, {&S59539[23], 22}, {&S59539[67], 17}, {NULL}};
const Segment SGI283554[] = {{&S59540[103], 4}, {NULL}};
const Segment SGI283555[] = {{&S59540[13], 9}, {&S59540[23], 2}, {&S59540[26], 22}, {&S59540[49], 6}, {&S59540[77], 17}, {NULL}};
const Segment SGI283556[] = {{&S60920[75], 4}, {NULL}};
const Segment SGI283557[] = {{&S60920[13], 14}, {&S60920[49], 17}, {NULL}};
const Segment SGI283558[] = {{&S60920[111], 3}, {NULL}};
const Segment SGI283559[] = {{&S60921[75], 4}, {NULL}};
const Segment SGI283560[] = {{&S60921[13], 14}, {&S60921[49], 17}, {NULL}};
const Segment SGI283561[] = {{&S60921[111], 3}, {NULL}};
const Segment SGI283562[] = {{&S59573[75], 4}, {NULL}};
const Segment SGI283563[] = {{&S59573[13], 14}, {&S59573[49], 17}, {NULL}};
const Segment SGI283564[] = {{&S59574[106], 4}, {NULL}};
const Segment SGI283565[] = {{&S59574[13], 46}, {&S59574[60], 11}, {&S59574[72], 9}, {&S59574[82], 15}, {NULL}};
const Segment SGI283566[] = {{&S59575[106], 4}, {NULL}};
const Segment SGI283567[] = {{&S59575[13], 46}, {&S59575[60], 11}, {&S59575[72], 9}, {&S59575[82], 15}, {NULL}};
const Segment SGI283568[] = {{&S59576[111], 4}, {NULL}};
const Segment SGI283569[] = {{&S59576[13], 46}, {&S59576[60], 11}, {&S59576[72], 9}, {&S59576[82], 20}, {NULL}};
const Segment SGI283570[] = {{&S59618[103], 4}, {NULL}};
const Segment SGI283571[] = {{&S59618[13], 9}, {&S59618[23], 2}, {&S59618[26], 22}, {&S59618[49], 6}, {&S59618[77], 17}, {NULL}};
const Segment SGI283572[] = {{&S59619[93], 4}, {NULL}};
const Segment SGI283573[] = {{&S59619[13], 9}, {&S59619[23], 22}, {&S59619[67], 17}, {NULL}};
const Segment SGI283574[] = {{&S59620[103], 4}, {NULL}};
const Segment SGI283575[] = {{&S59620[13], 9}, {&S59620[23], 2}, {&S59620[26], 22}, {&S59620[49], 6}, {&S59620[77], 17}, {NULL}};
const Segment SGI283576[] = {{&S59654[76], 4}, {NULL}};
const Segment SGI283577[] = {{&S59654[13], 15}, {&S59654[50], 17}, {NULL}};
const Segment SGI283578[] = {{&S59776[92], 4}, {NULL}};
const Segment SGI283579[] = {{&S59776[13], 9}, {&S59776[23], 22}, {&S59776[66], 17}, {NULL}};
const Segment SGI283580[] = {{&S59776[104], 9}, {NULL}};
const Segment SGI283581[] = {{&S59777[92], 4}, {NULL}};
const Segment SGI283582[] = {{&S59777[13], 9}, {&S59777[23], 22}, {&S59777[66], 17}, {NULL}};
const Segment SGI283583[] = {{&S59777[104], 9}, {NULL}};
const Segment SGI283584[] = {{&S61183[92], 4}, {NULL}};
const Segment SGI283585[] = {{&S61183[13], 7}, {&S61183[21], 2}, {&S61183[24], 15}, {&S61183[40], 6}, {&S61183[68], 17}, {NULL}};
const Segment SGI283586[] = {{&S61183[87], 9}, {NULL}};
const Segment SGI283587[] = {{&S59942[111], 4}, {NULL}};
const Segment SGI283588[] = {{&S59942[13], 9}, {&S59942[23], 22}, {&S59942[66], 17}, {NULL}};
const Segment SGI283589[] = {{&S59942[85], 11}, {NULL}};
const Segment SGI283590[] = {{&S59943[111], 4}, {NULL}};
const Segment SGI283591[] = {{&S59943[13], 9}, {&S59943[23], 22}, {&S59943[66], 17}, {NULL}};
const Segment SGI283592[] = {{&S59943[85], 11}, {NULL}};
const Segment SGI283593[] = {{&S59944[111], 4}, {NULL}};
const Segment SGI283594[] = {{&S59944[13], 9}, {&S59944[23], 22}, {&S59944[66], 17}, {NULL}};
const Segment SGI283595[] = {{&S59944[85], 11}, {NULL}};
const Segment SGI283596[] = {{&S59946[111], 4}, {NULL}};
const Segment SGI283597[] = {{&S59946[13], 9}, {&S59946[23], 22}, {&S59946[66], 17}, {NULL}};
const Segment SGI283598[] = {{&S59946[85], 11}, {NULL}};
const Segment SGI283599[] = {{&S60077[75], 4}, {NULL}};
const Segment SGI283600[] = {{&S60077[13], 14}, {&S60077[49], 17}, {NULL}};
const Segment SGI283601[] = {{&S60077[88], 9}, {NULL}};
const Segment SGI283602[] = {{&S60078[82], 4}, {NULL}};
const Segment SGI283603[] = {{&S60078[13], 14}, {&S60078[28], 6}, {&S60078[56], 17}, {NULL}};
const Segment SGI283604[] = {{&S60078[95], 9}, {NULL}};
const Segment SGI283605[] = {{&S60347[83], 4}, {NULL}};
const Segment SGI283606[] = {{&S60347[13], 3}, {&S60347[17], 19}, {&S60347[57], 17}, {NULL}};
const Segment SGI283607[] = {{&S60348[83], 4}, {NULL}};
const Segment SGI283608[] = {{&S60348[13], 3}, {&S60348[17], 19}, {&S60348[57], 17}, {NULL}};
const Segment SGI283609[] = {{&S60349[85], 4}, {NULL}};
const Segment SGI283610[] = {{&S60349[13], 3}, {&S60349[17], 21}, {&S60349[59], 17}, {NULL}};
const Segment SGI283611[] = {{&S60350[83], 4}, {NULL}};
const Segment SGI283612[] = {{&S60350[13], 3}, {&S60350[17], 19}, {&S60350[57], 17}, {NULL}};
const Segment SGI283613[] = {{&S60351[86], 4}, {NULL}};
const Segment SGI283614[] = {{&S60351[13], 3}, {&S60351[17], 22}, {&S60351[60], 17}, {NULL}};
const Segment SGI283615[] = {{&S60352[84], 4}, {NULL}};
const Segment SGI283616[] = {{&S60352[13], 3}, {&S60352[17], 20}, {&S60352[58], 17}, {NULL}};
const Segment SGI283617[] = {{&S60353[84], 4}, {NULL}};
const Segment SGI283618[] = {{&S60353[13], 3}, {&S60353[17], 20}, {&S60353[58], 17}, {NULL}};
const Segment SGI283619[] = {{&S61412[104], 4}, {NULL}};
const Segment SGI283620[] = {{&S61412[13], 14}, {&S61412[28], 6}, {&S61412[55], 17}, {NULL}};
const Segment SGI283621[] = {{&S61412[74], 18}, {NULL}};
const Segment SGI283622[] = {{&S60354[85], 4}, {NULL}};
const Segment SGI283623[] = {{&S60354[13], 3}, {&S60354[17], 21}, {&S60354[59], 17}, {NULL}};
const Segment SGI283624[] = {{&S61413[97], 4}, {NULL}};
const Segment SGI283625[] = {{&S61413[13], 14}, {&S61413[48], 17}, {NULL}};
const Segment SGI283626[] = {{&S61413[67], 18}, {NULL}};
const Segment SGI283627[] = {{&S60355[83], 4}, {NULL}};
const Segment SGI283628[] = {{&S60355[13], 3}, {&S60355[17], 19}, {&S60355[57], 17}, {NULL}};
const Segment SGI283629[] = {{&S60356[86], 4}, {NULL}};
const Segment SGI283630[] = {{&S60356[13], 3}, {&S60356[17], 22}, {&S60356[60], 17}, {NULL}};
const Segment SGI283631[] = {{&S60357[84], 4}, {NULL}};
const Segment SGI283632[] = {{&S60357[13], 3}, {&S60357[17], 20}, {&S60357[58], 17}, {NULL}};
const Segment SGI283633[] = {{&S60358[84], 4}, {NULL}};
const Segment SGI283634[] = {{&S60358[13], 3}, {&S60358[17], 20}, {&S60358[58], 17}, {NULL}};
const Segment SGI283635[] = {{&S60359[84], 4}, {NULL}};
const Segment SGI283636[] = {{&S60359[13], 3}, {&S60359[17], 20}, {&S60359[58], 17}, {NULL}};
const Segment SGI283637[] = {{&S60360[86], 4}, {NULL}};
const Segment SGI283638[] = {{&S60360[13], 3}, {&S60360[17], 22}, {&S60360[60], 17}, {NULL}};
const Segment SGI283639[] = {{&S60361[86], 4}, {NULL}};
const Segment SGI283640[] = {{&S60361[13], 3}, {&S60361[17], 22}, {&S60361[60], 17}, {NULL}};
const Segment SGI283641[] = {{&S60362[84], 4}, {NULL}};
const Segment SGI283642[] = {{&S60362[13], 3}, {&S60362[17], 20}, {&S60362[58], 17}, {NULL}};
const Segment SGI283643[] = {{&S60363[86], 4}, {NULL}};
const Segment SGI283644[] = {{&S60363[13], 3}, {&S60363[17], 22}, {&S60363[60], 17}, {NULL}};
const Segment SGI283645[] = {{&S60364[84], 4}, {NULL}};
const Segment SGI283646[] = {{&S60364[13], 3}, {&S60364[17], 20}, {&S60364[58], 17}, {NULL}};
const Segment SGI283647[] = {{&S60365[86], 4}, {NULL}};
const Segment SGI283648[] = {{&S60365[13], 3}, {&S60365[17], 22}, {&S60365[60], 17}, {NULL}};
const Segment SGI283649[] = {{&S68917[100], 3}, {NULL}};
const Segment SGI283650[] = {{&S68917[13], 3}, {&S68917[17], 2}, {&S68917[20], 11}, {&S68917[32], 6}, {&S68917[59], 17}, {NULL}};
const Segment SGI283651[] = {{&S68917[78], 12}, {NULL}};
const Segment SGI283652[] = {{&S68918[102], 3}, {NULL}};
const Segment SGI283653[] = {{&S68918[13], 3}, {&S68918[17], 2}, {&S68918[20], 13}, {&S68918[34], 6}, {&S68918[61], 17}, {NULL}};
const Segment SGI283654[] = {{&S68918[80], 12}, {NULL}};
const Segment SGI283655[] = {{&S60512[93], 4}, {NULL}};
const Segment SGI283656[] = {{&S60512[13], 3}, {&S60512[17], 13}, {&S60512[52], 17}, {NULL}};
const Segment SGI283657[] = {{&S60512[71], 12}, {NULL}};
const Segment SGI283658[] = {{&S60513[91], 4}, {NULL}};
const Segment SGI283659[] = {{&S60513[13], 3}, {&S60513[17], 11}, {&S60513[50], 17}, {NULL}};
const Segment SGI283660[] = {{&S60513[69], 12}, {NULL}};
const Segment SGI283661[] = {{&S69022[103], 3}, {NULL}};
const Segment SGI283662[] = {{&S69022[13], 9}, {&S69022[23], 2}, {&S69022[26], 22}, {&S69022[49], 6}, {&S69022[77], 17}, {NULL}};
const Segment SGI283663[] = {{&S69023[103], 3}, {NULL}};
const Segment SGI283664[] = {{&S69023[13], 9}, {&S69023[23], 2}, {&S69023[26], 22}, {&S69023[49], 6}, {&S69023[77], 17}, {NULL}};
const Segment SGI283665[] = {{&S69024[103], 3}, {NULL}};
const Segment SGI283666[] = {{&S69024[13], 9}, {&S69024[23], 2}, {&S69024[26], 22}, {&S69024[49], 6}, {&S69024[77], 17}, {NULL}};
const Segment SGI283667[] = {{&S69025[103], 3}, {NULL}};
const Segment SGI283668[] = {{&S69025[13], 9}, {&S69025[23], 2}, {&S69025[26], 22}, {&S69025[49], 6}, {&S69025[77], 17}, {NULL}};
const Segment SGI283669[] = {{&S69026[103], 3}, {NULL}};
const Segment SGI283670[] = {{&S69026[13], 9}, {&S69026[23], 2}, {&S69026[26], 22}, {&S69026[49], 6}, {&S69026[77], 17}, {NULL}};
const Segment SGI283671[] = {{&S69027[103], 3}, {NULL}};
const Segment SGI283672[] = {{&S69027[13], 9}, {&S69027[23], 2}, {&S69027[26], 22}, {&S69027[49], 6}, {&S69027[77], 17}, {NULL}};
const Segment SGI283673[] = {{&S61753[73], 4}, {NULL}};
const Segment SGI283674[] = {{&S61753[13], 14}, {&S61753[49], 17}, {NULL}};
const Segment SGI283675[] = {{&S61753[68], 9}, {NULL}};
const Segment SGI283676[] = {{&S61754[80], 4}, {NULL}};
const Segment SGI283677[] = {{&S61754[13], 14}, {&S61754[28], 6}, {&S61754[56], 17}, {NULL}};
const Segment SGI283678[] = {{&S61754[75], 9}, {NULL}};
const Segment SGI283679[] = {{&S61755[73], 4}, {NULL}};
const Segment SGI283680[] = {{&S61755[13], 14}, {&S61755[49], 17}, {NULL}};
const Segment SGI283681[] = {{&S61755[68], 9}, {NULL}};
const Segment SGI283682[] = {{&S69553[119], 3}, {NULL}};
const Segment SGI283683[] = {{&S69553[13], 9}, {&S69553[23], 2}, {&S69553[26], 22}, {&S69553[49], 6}, {&S69553[76], 17}, {NULL}};
const Segment SGI283684[] = {{&S69553[95], 9}, {NULL}};
const Segment SGI283685[] = {{&S69554[119], 3}, {NULL}};
const Segment SGI283686[] = {{&S69554[13], 9}, {&S69554[23], 2}, {&S69554[26], 22}, {&S69554[49], 6}, {&S69554[76], 17}, {NULL}};
const Segment SGI283687[] = {{&S69554[95], 9}, {NULL}};
const Segment SGI283688[] = {{&S69555[119], 3}, {NULL}};
const Segment SGI283689[] = {{&S69555[13], 9}, {&S69555[23], 2}, {&S69555[26], 22}, {&S69555[49], 6}, {&S69555[76], 17}, {NULL}};
const Segment SGI283690[] = {{&S69555[95], 9}, {NULL}};
const Segment SGI283691[] = {{&S69556[119], 3}, {NULL}};
const Segment SGI283692[] = {{&S69556[13], 9}, {&S69556[23], 2}, {&S69556[26], 22}, {&S69556[49], 6}, {&S69556[76], 17}, {NULL}};
const Segment SGI283693[] = {{&S69556[95], 9}, {NULL}};
const Segment SGI283694[] = {{&S60906[78], 4}, {NULL}};
const Segment SGI283695[] = {{&S60906[13], 3}, {&S60906[17], 13}, {&S60906[52], 17}, {NULL}};
const Segment SGI283696[] = {{&S60906[90], 9}, {NULL}};
const Segment SGI283697[] = {{&S60907[76], 4}, {NULL}};
const Segment SGI283698[] = {{&S60907[13], 3}, {&S60907[17], 11}, {&S60907[50], 17}, {NULL}};
const Segment SGI283699[] = {{&S60907[88], 9}, {NULL}};
const Segment SGI283700[] = {{&S62089[73], 4}, {NULL}};
const Segment SGI283701[] = {{&S62089[13], 14}, {&S62089[49], 17}, {NULL}};
const Segment SGI283702[] = {{&S62089[68], 9}, {NULL}};
const Segment SGI283703[] = {{&S62090[80], 4}, {NULL}};
const Segment SGI283704[] = {{&S62090[13], 14}, {&S62090[28], 6}, {&S62090[56], 17}, {NULL}};
const Segment SGI283705[] = {{&S62090[75], 9}, {NULL}};
const Segment SGI283706[] = {{&S61188[92], 4}, {NULL}};
const Segment SGI283707[] = {{&S61188[13], 9}, {&S61188[23], 22}, {&S61188[66], 17}, {NULL}};
const Segment SGI283708[] = {{&S61189[103], 4}, {NULL}};
const Segment SGI283709[] = {{&S61189[13], 9}, {&S61189[23], 2}, {&S61189[26], 22}, {&S61189[49], 6}, {&S61189[77], 17}, {NULL}};
const Segment SGI283710[] = {{&S61190[103], 4}, {NULL}};
const Segment SGI283711[] = {{&S61190[13], 9}, {&S61190[23], 2}, {&S61190[26], 22}, {&S61190[49], 6}, {&S61190[77], 17}, {NULL}};
const Segment SGI283712[] = {{&S62176[92], 4}, {NULL}};
const Segment SGI283713[] = {{&S62176[13], 3}, {&S62176[17], 13}, {&S62176[31], 6}, {&S62176[38], 6}, {&S62176[66], 17}, {NULL}};
const Segment SGI283714[] = {{&S62177[92], 4}, {NULL}};
const Segment SGI283715[] = {{&S62177[13], 3}, {&S62177[17], 13}, {&S62177[31], 6}, {&S62177[38], 6}, {&S62177[66], 17}, {NULL}};
const Segment SGI283716[] = {{&S62178[92], 4}, {NULL}};
const Segment SGI283717[] = {{&S62178[13], 3}, {&S62178[17], 13}, {&S62178[31], 6}, {&S62178[38], 6}, {&S62178[66], 17}, {NULL}};
const Segment SGI283718[] = {{&S70120[92], 3}, {NULL}};
const Segment SGI283719[] = {{&S70120[13], 7}, {&S70120[21], 2}, {&S70120[24], 15}, {&S70120[40], 6}, {&S70120[68], 17}, {NULL}};
const Segment SGI283720[] = {{&S70120[87], 8}, {NULL}};
const Segment SGI283721[] = {{&S61630[75], 4}, {NULL}};
const Segment SGI283722[] = {{&S61630[13], 14}, {&S61630[49], 17}, {NULL}};
const Segment SGI283723[] = {{&S61630[103], 12}, {NULL}};
const Segment SGI283724[] = {{&S61633[82], 4}, {NULL}};
const Segment SGI283725[] = {{&S61633[13], 14}, {&S61633[28], 6}, {&S61633[56], 17}, {NULL}};
const Segment SGI283726[] = {{&S61633[110], 12}, {NULL}};
const Segment SGI283727[] = {{&S62545[89], 4}, {NULL}};
const Segment SGI283728[] = {{&S62545[13], 3}, {&S62545[17], 13}, {&S62545[31], 10}, {&S62545[63], 17}, {NULL}};
const Segment SGI283729[] = {{&S61676[91], 4}, {NULL}};
const Segment SGI283730[] = {{&S61676[13], 9}, {&S61676[23], 22}, {&S61676[67], 17}, {NULL}};
const Segment SGI283731[] = {{&S61676[86], 9}, {NULL}};
const Segment SGI283732[] = {{&S61677[91], 4}, {NULL}};
const Segment SGI283733[] = {{&S61677[13], 9}, {&S61677[23], 22}, {&S61677[67], 17}, {NULL}};
const Segment SGI283734[] = {{&S61677[86], 9}, {NULL}};
const Segment SGI283735[] = {{&S61678[91], 4}, {NULL}};
const Segment SGI283736[] = {{&S61678[13], 9}, {&S61678[23], 22}, {&S61678[67], 17}, {NULL}};
const Segment SGI283737[] = {{&S61678[86], 9}, {NULL}};
const Segment SGI283738[] = {{&S61679[91], 4}, {NULL}};
const Segment SGI283739[] = {{&S61679[13], 9}, {&S61679[23], 22}, {&S61679[67], 17}, {NULL}};
const Segment SGI283740[] = {{&S61679[86], 9}, {NULL}};
const Segment SGI283741[] = {{&S62590[75], 4}, {NULL}};
const Segment SGI283742[] = {{&S62590[13], 14}, {&S62590[49], 17}, {NULL}};
const Segment SGI283743[] = {{&S61756[75], 4}, {NULL}};
const Segment SGI283744[] = {{&S61756[13], 15}, {&S61756[49], 17}, {NULL}};
const Segment SGI283745[] = {{&S61907[75], 4}, {NULL}};
const Segment SGI283746[] = {{&S61907[13], 14}, {&S61907[49], 17}, {NULL}};
const Segment SGI283747[] = {{&S61907[87], 9}, {NULL}};
const Segment SGI283748[] = {{&S61908[82], 4}, {NULL}};
const Segment SGI283749[] = {{&S61908[13], 14}, {&S61908[28], 6}, {&S61908[56], 17}, {NULL}};
const Segment SGI283750[] = {{&S61908[94], 9}, {NULL}};
const Segment SGI283751[] = {{&S62227[73], 4}, {NULL}};
const Segment SGI283752[] = {{&S62227[13], 14}, {&S62227[49], 17}, {NULL}};
const Segment SGI283753[] = {{&S62227[68], 9}, {NULL}};
const Segment SGI283754[] = {{&S62228[80], 4}, {NULL}};
const Segment SGI283755[] = {{&S62228[13], 14}, {&S62228[28], 6}, {&S62228[56], 17}, {NULL}};
const Segment SGI283756[] = {{&S62228[75], 9}, {NULL}};
const Segment SGI283757[] = {{&S62329[73], 4}, {NULL}};
const Segment SGI283758[] = {{&S62329[13], 14}, {&S62329[49], 17}, {NULL}};
const Segment SGI283759[] = {{&S62329[68], 9}, {NULL}};
const Segment SGI283760[] = {{&S62330[80], 4}, {NULL}};
const Segment SGI283761[] = {{&S62330[13], 14}, {&S62330[28], 6}, {&S62330[56], 17}, {NULL}};
const Segment SGI283762[] = {{&S62330[75], 9}, {NULL}};
const Segment SGI283763[] = {{&S62802[106], 4}, {NULL}};
const Segment SGI283764[] = {{&S62802[13], 3}, {&S62802[17], 13}, {&S62802[31], 20}, {&S62802[52], 6}, {&S62802[80], 17}, {NULL}};
const Segment SGI283765[] = {{&S62803[106], 4}, {NULL}};
const Segment SGI283766[] = {{&S62803[13], 3}, {&S62803[17], 13}, {&S62803[31], 20}, {&S62803[52], 6}, {&S62803[80], 17}, {NULL}};
const Segment SGI283767[] = {{&S62804[103], 4}, {NULL}};
const Segment SGI283768[] = {{&S62804[13], 3}, {&S62804[17], 13}, {&S62804[31], 17}, {&S62804[49], 6}, {&S62804[77], 17}, {NULL}};
const Segment SGI283769[] = {{&S62805[106], 4}, {NULL}};
const Segment SGI283770[] = {{&S62805[13], 3}, {&S62805[17], 13}, {&S62805[31], 20}, {&S62805[52], 6}, {&S62805[80], 17}, {NULL}};
const Segment SGI283771[] = {{&S62806[106], 4}, {NULL}};
const Segment SGI283772[] = {{&S62806[13], 3}, {&S62806[17], 13}, {&S62806[31], 20}, {&S62806[52], 6}, {&S62806[80], 17}, {NULL}};
const Segment SGI283773[] = {{&S62807[106], 4}, {NULL}};
const Segment SGI283774[] = {{&S62807[13], 3}, {&S62807[17], 13}, {&S62807[31], 20}, {&S62807[52], 6}, {&S62807[80], 17}, {NULL}};
const Segment SGI283775[] = {{&S62808[106], 4}, {NULL}};
const Segment SGI283776[] = {{&S62808[13], 3}, {&S62808[17], 13}, {&S62808[31], 20}, {&S62808[52], 6}, {&S62808[80], 17}, {NULL}};
const Segment SGI283777[] = {{&S62809[106], 4}, {NULL}};
const Segment SGI283778[] = {{&S62809[13], 3}, {&S62809[17], 13}, {&S62809[31], 20}, {&S62809[52], 6}, {&S62809[80], 17}, {NULL}};
const Segment SGI283779[] = {{&S62810[106], 4}, {NULL}};
const Segment SGI283780[] = {{&S62810[13], 3}, {&S62810[17], 13}, {&S62810[31], 20}, {&S62810[52], 6}, {&S62810[80], 17}, {NULL}};
const Segment SGI283781[] = {{&S62811[106], 4}, {NULL}};
const Segment SGI283782[] = {{&S62811[13], 3}, {&S62811[17], 13}, {&S62811[31], 20}, {&S62811[52], 6}, {&S62811[80], 17}, {NULL}};
const Segment SGI283783[] = {{&S62812[106], 4}, {NULL}};
const Segment SGI283784[] = {{&S62812[13], 3}, {&S62812[17], 13}, {&S62812[31], 20}, {&S62812[52], 6}, {&S62812[80], 17}, {NULL}};
const Segment SGI283785[] = {{&S62813[106], 4}, {NULL}};
const Segment SGI283786[] = {{&S62813[13], 3}, {&S62813[17], 13}, {&S62813[31], 20}, {&S62813[52], 6}, {&S62813[80], 17}, {NULL}};
const Segment SGI283787[] = {{&S62856[41], 4}, {NULL}};
const Segment SGI283788[] = {{&S62856[13], 14}, {&S62856[28], 24}, {&S62856[74], 17}, {NULL}};
const Segment SGI283789[] = {{&S62856[29], 16}, {NULL}};
const Segment SGI283790[] = {{&S62857[48], 4}, {NULL}};
const Segment SGI283791[] = {{&S62857[13], 14}, {&S62857[28], 6}, {&S62857[35], 24}, {&S62857[81], 17}, {NULL}};
const Segment SGI283792[] = {{&S62857[36], 16}, {NULL}};
const Segment SGI283793[] = {{&S62527[90], 4}, {NULL}};
const Segment SGI283794[] = {{&S62527[13], 3}, {&S62527[17], 11}, {&S62527[49], 17}, {NULL}};
const Segment SGI283795[] = {{&S62527[68], 12}, {NULL}};
const Segment SGI283796[] = {{&S62859[41], 4}, {NULL}};
const Segment SGI283797[] = {{&S62859[13], 14}, {&S62859[28], 24}, {&S62859[74], 17}, {NULL}};
const Segment SGI283798[] = {{&S62859[29], 16}, {NULL}};
const Segment SGI283799[] = {{&S62528[92], 4}, {NULL}};
const Segment SGI283800[] = {{&S62528[13], 3}, {&S62528[17], 13}, {&S62528[51], 17}, {NULL}};
const Segment SGI283801[] = {{&S62528[70], 12}, {NULL}};
const Segment SGI283802[] = {{&S62860[48], 4}, {NULL}};
const Segment SGI283803[] = {{&S62860[13], 14}, {&S62860[28], 6}, {&S62860[35], 24}, {&S62860[81], 17}, {NULL}};
const Segment SGI283804[] = {{&S62860[36], 16}, {NULL}};
const Segment SGI283805[] = {{&S62862[41], 4}, {NULL}};
const Segment SGI283806[] = {{&S62862[13], 14}, {&S62862[28], 24}, {&S62862[74], 17}, {NULL}};
const Segment SGI283807[] = {{&S62862[29], 16}, {NULL}};
const Segment SGI283808[] = {{&S62680[77], 4}, {NULL}};
const Segment SGI283809[] = {{&S62680[13], 3}, {&S62680[17], 13}, {&S62680[51], 17}, {NULL}};
const Segment SGI283810[] = {{&S62680[89], 9}, {NULL}};
const Segment SGI283811[] = {{&S62681[75], 4}, {NULL}};
const Segment SGI283812[] = {{&S62681[13], 3}, {&S62681[17], 11}, {&S62681[49], 17}, {NULL}};
const Segment SGI283813[] = {{&S62681[87], 9}, {NULL}};
const Segment SGI283814[] = {{&S62682[39], 4}, {NULL}};
const Segment SGI283815[] = {{&S62682[13], 3}, {&S62682[17], 2}, {&S62682[20], 12}, {NULL}};
const Segment SGI283816[] = {{&S62682[34], 9}, {NULL}};
const Segment SGI283817[] = {{&S63027[93], 4}, {NULL}};
const Segment SGI283818[] = {{&S63027[13], 3}, {&S63027[17], 13}, {&S63027[52], 17}, {NULL}};
const Segment SGI283819[] = {{&S63027[71], 12}, {NULL}};
const Segment SGI283820[] = {{&S63028[91], 4}, {NULL}};
const Segment SGI283821[] = {{&S63028[13], 3}, {&S63028[17], 11}, {&S63028[50], 17}, {NULL}};
const Segment SGI283822[] = {{&S63028[69], 12}, {NULL}};
const Segment SGI283823[] = {{&S63029[103], 4}, {NULL}};
const Segment SGI283824[] = {{&S63029[13], 3}, {&S63029[17], 2}, {&S63029[20], 13}, {&S63029[34], 6}, {&S63029[62], 17}, {NULL}};
const Segment SGI283825[] = {{&S63029[81], 12}, {NULL}};
const Segment SGI283826[] = {{&S63030[101], 4}, {NULL}};
const Segment SGI283827[] = {{&S63030[13], 3}, {&S63030[17], 2}, {&S63030[20], 11}, {&S63030[32], 6}, {&S63030[60], 17}, {NULL}};
const Segment SGI283828[] = {{&S63030[79], 12}, {NULL}};
const Segment SGI283829[] = {{&S63031[94], 4}, {NULL}};
const Segment SGI283830[] = {{&S63031[13], 3}, {&S63031[17], 2}, {&S63031[20], 11}, {&S63031[53], 17}, {NULL}};
const Segment SGI283831[] = {{&S63031[72], 12}, {NULL}};
const Segment SGI283832[] = {{&S63187[96], 4}, {NULL}};
const Segment SGI283833[] = {{&S63187[13], 9}, {&S63187[23], 2}, {&S63187[26], 22}, {&S63187[70], 17}, {NULL}};
const Segment SGI283834[] = {{&S63203[112], 4}, {NULL}};
const Segment SGI283835[] = {{&S63203[13], 9}, {&S63203[23], 22}, {&S63203[67], 17}, {NULL}};
const Segment SGI283836[] = {{&S63203[86], 11}, {NULL}};
const Segment SGI283837[] = {{&S63220[72], 4}, {NULL}};
const Segment SGI283838[] = {{&S63220[13], 14}, {&S63220[48], 17}, {NULL}};
const Segment SGI283839[] = {{&S63220[67], 9}, {NULL}};
const Segment SGI283840[] = {{&S63221[79], 4}, {NULL}};
const Segment SGI283841[] = {{&S63221[13], 14}, {&S63221[28], 6}, {&S63221[55], 17}, {NULL}};
const Segment SGI283842[] = {{&S63221[74], 9}, {NULL}};
const Segment SGI283843[] = {{&S63090[81], 4}, {NULL}};
const Segment SGI283844[] = {{&S63090[13], 14}, {&S63090[28], 6}, {&S63090[55], 17}, {NULL}};
const Segment SGI283845[] = {{&S63090[93], 9}, {NULL}};
const Segment SGI283846[] = {{&S63091[74], 4}, {NULL}};
const Segment SGI283847[] = {{&S63091[13], 14}, {&S63091[48], 17}, {NULL}};
const Segment SGI283848[] = {{&S63091[86], 9}, {NULL}};
const Segment SGI283849[] = {{&S63204[111], 4}, {NULL}};
const Segment SGI283850[] = {{&S63204[13], 9}, {&S63204[23], 22}, {&S63204[66], 17}, {NULL}};
const Segment SGI283851[] = {{&S63204[85], 11}, {NULL}};
const Segment SGI283852[] = {{&S63205[111], 4}, {NULL}};
const Segment SGI283853[] = {{&S63205[13], 9}, {&S63205[23], 22}, {&S63205[66], 17}, {NULL}};
const Segment SGI283854[] = {{&S63205[85], 11}, {NULL}};
const Segment SGI283855[] = {{&S63206[111], 4}, {NULL}};
const Segment SGI283856[] = {{&S63206[13], 9}, {&S63206[23], 22}, {&S63206[66], 17}, {NULL}};
const Segment SGI283857[] = {{&S63206[85], 11}, {NULL}};
const Segment SGI283858[] = {{&S63455[72], 4}, {NULL}};
const Segment SGI283859[] = {{&S63455[13], 14}, {&S63455[48], 17}, {NULL}};
const Segment SGI283860[] = {{&S63455[67], 9}, {NULL}};
const Segment SGI283861[] = {{&S63456[79], 4}, {NULL}};
const Segment SGI283862[] = {{&S63456[13], 14}, {&S63456[28], 6}, {&S63456[55], 17}, {NULL}};
const Segment SGI283863[] = {{&S63456[74], 9}, {NULL}};
const Segment SGI283864[] = {{&S63595[94], 4}, {NULL}};
const Segment SGI283865[] = {{&S63595[13], 7}, {&S63595[21], 2}, {&S63595[24], 15}, {&S63595[40], 6}, {&S63595[68], 17}, {NULL}};
const Segment SGI283866[] = {{&S63690[73], 4}, {NULL}};
const Segment SGI283867[] = {{&S63690[13], 14}, {&S63690[49], 17}, {NULL}};
const Segment SGI283868[] = {{&S63690[68], 9}, {NULL}};
const Segment SGI283869[] = {{&S63764[78], 4}, {NULL}};
const Segment SGI283870[] = {{&S63764[13], 3}, {&S63764[17], 13}, {&S63764[52], 17}, {NULL}};
const Segment SGI283871[] = {{&S63766[71], 4}, {NULL}};
const Segment SGI283872[] = {{&S63766[13], 10}, {&S63766[45], 17}, {NULL}};
const Segment SGI283873[] = {{&S63826[90], 4}, {NULL}};
const Segment SGI283874[] = {{&S63826[13], 3}, {&S63826[17], 11}, {&S63826[49], 17}, {NULL}};
const Segment SGI283875[] = {{&S63826[68], 12}, {NULL}};
const Segment SGI283876[] = {{&S63827[92], 4}, {NULL}};
const Segment SGI283877[] = {{&S63827[13], 3}, {&S63827[17], 13}, {&S63827[51], 17}, {NULL}};
const Segment SGI283878[] = {{&S63827[70], 12}, {NULL}};
const Segment SGI283879[] = {{&S63871[114], 4}, {NULL}};
const Segment SGI283880[] = {{&S63871[13], 3}, {&S63871[17], 13}, {&S63871[31], 28}, {&S63871[60], 6}, {&S63871[88], 17}, {NULL}};
const Segment SGI283881[] = {{&S63872[114], 4}, {NULL}};
const Segment SGI283882[] = {{&S63872[13], 3}, {&S63872[17], 13}, {&S63872[31], 28}, {&S63872[60], 6}, {&S63872[88], 17}, {NULL}};
const Segment SGI283883[] = {{&S63873[114], 4}, {NULL}};
const Segment SGI283884[] = {{&S63873[13], 3}, {&S63873[17], 13}, {&S63873[31], 28}, {&S63873[60], 6}, {&S63873[88], 17}, {NULL}};
const Segment SGI283885[] = {{&S63874[114], 4}, {NULL}};
const Segment SGI283886[] = {{&S63874[13], 3}, {&S63874[17], 13}, {&S63874[31], 28}, {&S63874[60], 6}, {&S63874[88], 17}, {NULL}};
const Segment SGI283887[] = {{&S63875[114], 4}, {NULL}};
const Segment SGI283888[] = {{&S63875[13], 3}, {&S63875[17], 13}, {&S63875[31], 28}, {&S63875[60], 6}, {&S63875[88], 17}, {NULL}};
const Segment SGI283889[] = {{&S63876[114], 4}, {NULL}};
const Segment SGI283890[] = {{&S63876[13], 3}, {&S63876[17], 13}, {&S63876[31], 28}, {&S63876[60], 6}, {&S63876[88], 17}, {NULL}};
const Segment SGI283891[] = {{&S63877[114], 4}, {NULL}};
const Segment SGI283892[] = {{&S63877[13], 3}, {&S63877[17], 13}, {&S63877[31], 28}, {&S63877[60], 6}, {&S63877[88], 17}, {NULL}};
const Segment SGI283893[] = {{&S63742[79], 4}, {NULL}};
const Segment SGI283894[] = {{&S63742[13], 3}, {&S63742[17], 14}, {&S63742[53], 17}, {NULL}};
const Segment SGI283895[] = {{&S63743[78], 4}, {NULL}};
const Segment SGI283896[] = {{&S63743[13], 3}, {&S63743[17], 13}, {&S63743[52], 17}, {NULL}};
const Segment SGI283897[] = {{&S63979[83], 4}, {NULL}};
const Segment SGI283898[] = {{&S63979[13], 3}, {&S63979[17], 19}, {&S63979[57], 17}, {NULL}};
const Segment SGI283899[] = {{&S63980[87], 4}, {NULL}};
const Segment SGI283900[] = {{&S63980[13], 3}, {&S63980[17], 22}, {&S63980[61], 17}, {NULL}};
const Segment SGI283901[] = {{&S63981[87], 4}, {NULL}};
const Segment SGI283902[] = {{&S63981[13], 3}, {&S63981[17], 22}, {&S63981[61], 17}, {NULL}};
const Segment SGI283903[] = {{&S63982[85], 4}, {NULL}};
const Segment SGI283904[] = {{&S63982[13], 3}, {&S63982[17], 20}, {&S63982[59], 17}, {NULL}};
const Segment SGI283905[] = {{&S63983[85], 4}, {NULL}};
const Segment SGI283906[] = {{&S63983[13], 3}, {&S63983[17], 20}, {&S63983[59], 17}, {NULL}};
const Segment SGI283907[] = {{&S63984[86], 4}, {NULL}};
const Segment SGI283908[] = {{&S63984[13], 3}, {&S63984[17], 21}, {&S63984[60], 17}, {NULL}};
const Segment SGI283909[] = {{&S63985[86], 4}, {NULL}};
const Segment SGI283910[] = {{&S63985[13], 3}, {&S63985[17], 21}, {&S63985[60], 17}, {NULL}};
const Segment SGI283911[] = {{&S63986[86], 4}, {NULL}};
const Segment SGI283912[] = {{&S63986[13], 3}, {&S63986[17], 21}, {&S63986[60], 17}, {NULL}};
const Segment SGI283913[] = {{&S63987[86], 4}, {NULL}};
const Segment SGI283914[] = {{&S63987[13], 3}, {&S63987[17], 21}, {&S63987[60], 17}, {NULL}};
const Segment SGI283915[] = {{&S63988[86], 4}, {NULL}};
const Segment SGI283916[] = {{&S63988[13], 3}, {&S63988[17], 21}, {&S63988[60], 17}, {NULL}};
const Segment SGI283917[] = {{&S63989[86], 4}, {NULL}};
const Segment SGI283918[] = {{&S63989[13], 3}, {&S63989[17], 21}, {&S63989[60], 17}, {NULL}};
const Segment SGI283919[] = {{&S63990[86], 4}, {NULL}};
const Segment SGI283920[] = {{&S63990[13], 3}, {&S63990[17], 21}, {&S63990[60], 17}, {NULL}};
const Segment SGI283921[] = {{&S63991[86], 4}, {NULL}};
const Segment SGI283922[] = {{&S63991[13], 3}, {&S63991[17], 21}, {&S63991[60], 17}, {NULL}};
const Segment SGI283923[] = {{&S63992[86], 4}, {NULL}};
const Segment SGI283924[] = {{&S63992[13], 3}, {&S63992[17], 21}, {&S63992[60], 17}, {NULL}};
const Segment SGI283925[] = {{&S63993[86], 4}, {NULL}};
const Segment SGI283926[] = {{&S63993[13], 3}, {&S63993[17], 21}, {&S63993[60], 17}, {NULL}};
const Segment SGI283927[] = {{&S63994[86], 4}, {NULL}};
const Segment SGI283928[] = {{&S63994[13], 3}, {&S63994[17], 22}, {&S63994[60], 17}, {NULL}};
const Segment SGI283929[] = {{&S63995[87], 4}, {NULL}};
const Segment SGI283930[] = {{&S63995[13], 3}, {&S63995[17], 22}, {&S63995[61], 17}, {NULL}};
const Segment SGI283931[] = {{&S63996[86], 4}, {NULL}};
const Segment SGI283932[] = {{&S63996[13], 3}, {&S63996[17], 22}, {&S63996[60], 17}, {NULL}};
const Segment SGI283933[] = {{&S63997[86], 4}, {NULL}};
const Segment SGI283934[] = {{&S63997[13], 3}, {&S63997[17], 21}, {&S63997[60], 17}, {NULL}};
const Segment SGI283935[] = {{&S64026[84], 4}, {NULL}};
const Segment SGI283936[] = {{&S64026[13], 3}, {&S64026[17], 20}, {&S64026[58], 17}, {NULL}};
const Segment SGI283937[] = {{&S64027[85], 4}, {NULL}};
const Segment SGI283938[] = {{&S64027[13], 3}, {&S64027[17], 21}, {&S64027[59], 17}, {NULL}};
const Segment SGI283939[] = {{&S64028[85], 4}, {NULL}};
const Segment SGI283940[] = {{&S64028[13], 3}, {&S64028[17], 21}, {&S64028[59], 17}, {NULL}};
const Segment SGI283941[] = {{&S64029[85], 4}, {NULL}};
const Segment SGI283942[] = {{&S64029[13], 3}, {&S64029[17], 21}, {&S64029[59], 17}, {NULL}};
const Segment SGI283943[] = {{&S64030[86], 4}, {NULL}};
const Segment SGI283944[] = {{&S64030[13], 3}, {&S64030[17], 22}, {&S64030[60], 17}, {NULL}};
const Segment SGI283945[] = {{&S64031[84], 4}, {NULL}};
const Segment SGI283946[] = {{&S64031[13], 3}, {&S64031[17], 20}, {&S64031[58], 17}, {NULL}};
const Segment SGI283947[] = {{&S64032[84], 4}, {NULL}};
const Segment SGI283948[] = {{&S64032[13], 3}, {&S64032[17], 20}, {&S64032[58], 17}, {NULL}};
const Segment SGI283949[] = {{&S64033[85], 4}, {NULL}};
const Segment SGI283950[] = {{&S64033[13], 3}, {&S64033[17], 21}, {&S64033[59], 17}, {NULL}};
const Segment SGI283951[] = {{&S64034[83], 4}, {NULL}};
const Segment SGI283952[] = {{&S64034[13], 3}, {&S64034[17], 19}, {&S64034[57], 17}, {NULL}};
const Segment SGI283953[] = {{&S64035[84], 4}, {NULL}};
const Segment SGI283954[] = {{&S64035[13], 3}, {&S64035[17], 20}, {&S64035[58], 17}, {NULL}};
const Segment SGI283955[] = {{&S64036[86], 4}, {NULL}};
const Segment SGI283956[] = {{&S64036[13], 3}, {&S64036[17], 22}, {&S64036[60], 17}, {NULL}};
const Segment SGI283957[] = {{&S64037[86], 4}, {NULL}};
const Segment SGI283958[] = {{&S64037[13], 3}, {&S64037[17], 22}, {&S64037[60], 17}, {NULL}};
const Segment SGI283959[] = {{&S64038[84], 4}, {NULL}};
const Segment SGI283960[] = {{&S64038[13], 3}, {&S64038[17], 20}, {&S64038[58], 17}, {NULL}};
const Segment SGI283961[] = {{&S64039[84], 4}, {NULL}};
const Segment SGI283962[] = {{&S64039[13], 3}, {&S64039[17], 20}, {&S64039[58], 17}, {NULL}};
const Segment SGI283963[] = {{&S64040[84], 4}, {NULL}};
const Segment SGI283964[] = {{&S64040[13], 3}, {&S64040[17], 20}, {&S64040[58], 17}, {NULL}};
const Segment SGI283965[] = {{&S64041[83], 4}, {NULL}};
const Segment SGI283966[] = {{&S64041[13], 3}, {&S64041[17], 19}, {&S64041[57], 17}, {NULL}};
const Segment SGI283967[] = {{&S64042[84], 4}, {NULL}};
const Segment SGI283968[] = {{&S64042[13], 3}, {&S64042[17], 20}, {&S64042[58], 17}, {NULL}};
const Segment SGI283969[] = {{&S64043[86], 4}, {NULL}};
const Segment SGI283970[] = {{&S64043[13], 3}, {&S64043[17], 22}, {&S64043[60], 17}, {NULL}};
const Segment SGI283971[] = {{&S64044[86], 4}, {NULL}};
const Segment SGI283972[] = {{&S64044[13], 3}, {&S64044[17], 22}, {&S64044[60], 17}, {NULL}};
const Segment SGI283973[] = {{&S64045[86], 4}, {NULL}};
const Segment SGI283974[] = {{&S64045[13], 3}, {&S64045[17], 22}, {&S64045[60], 17}, {NULL}};
const Segment SGI283975[] = {{&S64046[86], 4}, {NULL}};
const Segment SGI283976[] = {{&S64046[13], 3}, {&S64046[17], 22}, {&S64046[60], 17}, {NULL}};
const Segment SGI283977[] = {{&S64047[84], 4}, {NULL}};
const Segment SGI283978[] = {{&S64047[13], 3}, {&S64047[17], 20}, {&S64047[58], 17}, {NULL}};
const Segment SGI283979[] = {{&S71978[100], 3}, {NULL}};
const Segment SGI283980[] = {{&S71978[13], 3}, {&S71978[17], 2}, {&S71978[20], 11}, {&S71978[32], 6}, {&S71978[59], 17}, {NULL}};
const Segment SGI283981[] = {{&S71978[78], 12}, {NULL}};
const Segment SGI283982[] = {{&S64048[86], 4}, {NULL}};
const Segment SGI283983[] = {{&S64048[13], 3}, {&S64048[17], 22}, {&S64048[60], 17}, {NULL}};
const Segment SGI283984[] = {{&S64049[86], 4}, {NULL}};
const Segment SGI283985[] = {{&S64049[13], 3}, {&S64049[17], 22}, {&S64049[60], 17}, {NULL}};
const Segment SGI283986[] = {{&S64050[86], 4}, {NULL}};
const Segment SGI283987[] = {{&S64050[13], 3}, {&S64050[17], 22}, {&S64050[60], 17}, {NULL}};
const Segment SGI283988[] = {{&S64051[86], 4}, {NULL}};
const Segment SGI283989[] = {{&S64051[13], 3}, {&S64051[17], 22}, {&S64051[60], 17}, {NULL}};
const Segment SGI283990[] = {{&S64052[84], 4}, {NULL}};
const Segment SGI283991[] = {{&S64052[13], 3}, {&S64052[17], 20}, {&S64052[58], 17}, {NULL}};
const Segment SGI283992[] = {{&S71979[102], 3}, {NULL}};
const Segment SGI283993[] = {{&S64053[86], 4}, {NULL}};
const Segment SGI283994[] = {{&S71979[13], 3}, {&S71979[17], 2}, {&S71979[20], 13}, {&S71979[34], 6}, {&S71979[61], 17}, {NULL}};
const Segment SGI283995[] = {{&S64053[13], 3}, {&S64053[17], 22}, {&S64053[60], 17}, {NULL}};
const Segment SGI283996[] = {{&S71979[80], 12}, {NULL}};
const Segment SGI283997[] = {{&S64054[86], 4}, {NULL}};
const Segment SGI283998[] = {{&S64054[13], 3}, {&S64054[17], 22}, {&S64054[60], 17}, {NULL}};
const Segment SGI283999[] = {{&S64055[86], 4}, {NULL}};
const Segment SGI284000[] = {{&S64055[13], 3}, {&S64055[17], 2}, {&S64055[20], 19}, {&S64055[60], 17}, {NULL}};
const Segment SGI284001[] = {{&S72029[101], 3}, {NULL}};
const Segment SGI284002[] = {{&S72029[13], 7}, {&S72029[21], 2}, {&S72029[24], 15}, {&S72029[40], 6}, {&S72029[68], 17}, {NULL}};
const Segment SGI284003[] = {{&S72029[87], 17}, {NULL}};
const Segment SGI284004[] = {{&S72121[8], 3}, {NULL}};
const Segment SGI284005[] = {{&S72121[13], 9}, {&S72121[23], 2}, {&S72121[26], 15}, {&S72121[42], 6}, {&S72121[69], 17}, {NULL}};
const Segment SGI284006[] = {{&S63968[74], 4}, {NULL}};
const Segment SGI284007[] = {{&S63968[13], 3}, {&S63968[17], 11}, {&S63968[50], 17}, {NULL}};
const Segment SGI284008[] = {{&S63968[69], 9}, {NULL}};
const Segment SGI284009[] = {{&S63969[76], 4}, {NULL}};
const Segment SGI284010[] = {{&S63969[13], 3}, {&S63969[17], 13}, {&S63969[52], 17}, {NULL}};
const Segment SGI284011[] = {{&S63969[71], 9}, {NULL}};
const Segment SGI284012[] = {{&S72213[101], 3}, {NULL}};
const Segment SGI284013[] = {{&S72213[13], 7}, {&S72213[21], 2}, {&S72213[24], 15}, {&S72213[40], 6}, {&S72213[68], 17}, {NULL}};
const Segment SGI284014[] = {{&S72213[87], 17}, {NULL}};
const Segment SGI284015[] = {{&S64358[101], 4}, {NULL}};
const Segment SGI284016[] = {{&S64358[13], 3}, {&S64358[17], 2}, {&S64358[20], 11}, {&S64358[32], 6}, {&S64358[60], 17}, {NULL}};
const Segment SGI284017[] = {{&S64358[79], 12}, {NULL}};
const Segment SGI284018[] = {{&S64359[103], 4}, {NULL}};
const Segment SGI284019[] = {{&S64359[13], 3}, {&S64359[17], 2}, {&S64359[20], 13}, {&S64359[34], 6}, {&S64359[62], 17}, {NULL}};
const Segment SGI284020[] = {{&S64359[81], 12}, {NULL}};
const Segment SGI284021[] = {{&S72269[102], 3}, {NULL}};
const Segment SGI284022[] = {{&S72269[13], 9}, {&S72269[23], 2}, {&S72269[26], 22}, {&S72269[49], 6}, {&S72269[76], 17}, {NULL}};
const Segment SGI284023[] = {{&S72270[102], 3}, {NULL}};
const Segment SGI284024[] = {{&S72270[13], 9}, {&S72270[23], 2}, {&S72270[26], 22}, {&S72270[49], 6}, {&S72270[76], 17}, {NULL}};
const Segment SGI284025[] = {{&S72271[102], 3}, {NULL}};
const Segment SGI284026[] = {{&S72271[13], 9}, {&S72271[23], 2}, {&S72271[26], 22}, {&S72271[49], 6}, {&S72271[76], 17}, {NULL}};
const Segment SGI284027[] = {{&S72272[102], 3}, {NULL}};
const Segment SGI284028[] = {{&S72272[13], 9}, {&S72272[23], 2}, {&S72272[26], 22}, {&S72272[49], 6}, {&S72272[76], 17}, {NULL}};
const Segment SGI284029[] = {{&S72273[102], 3}, {NULL}};
const Segment SGI284030[] = {{&S72273[13], 9}, {&S72273[23], 2}, {&S72273[26], 22}, {&S72273[49], 6}, {&S72273[76], 17}, {NULL}};
const Segment SGI284031[] = {{&S64108[77], 4}, {NULL}};
const Segment SGI284032[] = {{&S64108[13], 3}, {&S64108[17], 13}, {&S64108[51], 17}, {NULL}};
const Segment SGI284033[] = {{&S64108[89], 9}, {NULL}};
const Segment SGI284034[] = {{&S64109[75], 4}, {NULL}};
const Segment SGI284035[] = {{&S64109[13], 3}, {&S64109[17], 11}, {&S64109[49], 17}, {NULL}};
const Segment SGI284036[] = {{&S64109[87], 9}, {NULL}};
const Segment SGI284037[] = {{&S72376[94], 3}, {NULL}};
const Segment SGI284038[] = {{&S72376[13], 7}, {&S72376[21], 2}, {&S72376[24], 15}, {&S72376[40], 6}, {&S72376[68], 17}, {NULL}};
const Segment SGI284039[] = {{&S72376[119], 15}, {NULL}};
const Segment SGI284040[] = {{&S64800[97], 4}, {NULL}};
const Segment SGI284041[] = {{&S64800[13], 14}, {&S64800[48], 17}, {NULL}};
const Segment SGI284042[] = {{&S64800[67], 18}, {NULL}};
const Segment SGI284043[] = {{&S64320[76], 4}, {NULL}};
const Segment SGI284044[] = {{&S64320[13], 3}, {&S64320[17], 11}, {&S64320[50], 17}, {NULL}};
const Segment SGI284045[] = {{&S64320[88], 9}, {NULL}};
const Segment SGI284046[] = {{&S64321[78], 4}, {NULL}};
const Segment SGI284047[] = {{&S64321[13], 3}, {&S64321[17], 13}, {&S64321[52], 17}, {NULL}};
const Segment SGI284048[] = {{&S64321[90], 9}, {NULL}};
const Segment SGI284049[] = {{&S64347[92], 4}, {NULL}};
const Segment SGI284050[] = {{&S64347[13], 3}, {&S64347[17], 13}, {&S64347[51], 17}, {NULL}};
const Segment SGI284051[] = {{&S64347[70], 12}, {NULL}};
const Segment SGI284052[] = {{&S64348[90], 4}, {NULL}};
const Segment SGI284053[] = {{&S64348[13], 3}, {&S64348[17], 11}, {&S64348[49], 17}, {NULL}};
const Segment SGI284054[] = {{&S64348[68], 12}, {NULL}};
const Segment SGI284055[] = {{&S64432[75], 4}, {NULL}};
const Segment SGI284056[] = {{&S64432[13], 14}, {&S64432[49], 17}, {NULL}};
const Segment SGI284057[] = {{&S64432[102], 12}, {NULL}};
const Segment SGI284058[] = {{&S64433[75], 4}, {NULL}};
const Segment SGI284059[] = {{&S64433[13], 14}, {&S64433[49], 17}, {NULL}};
const Segment SGI284060[] = {{&S64433[103], 12}, {NULL}};
const Segment SGI284061[] = {{&S64471[48], 4}, {NULL}};
const Segment SGI284062[] = {{&S64471[13], 14}, {&S64471[28], 6}, {&S64471[35], 25}, {&S64471[81], 17}, {NULL}};
const Segment SGI284063[] = {{&S64541[86], 4}, {NULL}};
const Segment SGI284064[] = {{&S64541[13], 3}, {&S64541[17], 21}, {&S64541[60], 17}, {NULL}};
const Segment SGI284065[] = {{&S64542[84], 4}, {NULL}};
const Segment SGI284066[] = {{&S64542[13], 3}, {&S64542[17], 19}, {&S64542[58], 17}, {NULL}};
const Segment SGI284067[] = {{&S64543[87], 4}, {NULL}};
const Segment SGI284068[] = {{&S64543[13], 3}, {&S64543[17], 22}, {&S64543[61], 17}, {NULL}};
const Segment SGI284069[] = {{&S65022[112], 4}, {NULL}};
const Segment SGI284070[] = {{&S64544[84], 4}, {NULL}};
const Segment SGI284071[] = {{&S65022[13], 9}, {&S65022[23], 22}, {&S65022[67], 17}, {NULL}};
const Segment SGI284072[] = {{&S64544[13], 3}, {&S64544[17], 19}, {&S64544[58], 17}, {NULL}};
const Segment SGI284073[] = {{&S65022[86], 11}, {NULL}};
const Segment SGI284074[] = {{&S64545[86], 4}, {NULL}};
const Segment SGI284075[] = {{&S64545[13], 3}, {&S64545[17], 21}, {&S64545[60], 17}, {NULL}};
const Segment SGI284076[] = {{&S72926[76], 3}, {NULL}};
const Segment SGI284077[] = {{&S72926[13], 5}, {&S72926[19], 2}, {&S72926[22], 6}, {&S72926[50], 17}, {NULL}};
const Segment SGI284078[] = {{&S65146[98], 4}, {NULL}};
const Segment SGI284079[] = {{&S65146[13], 14}, {&S65146[49], 17}, {NULL}};
const Segment SGI284080[] = {{&S65146[68], 18}, {NULL}};
const Segment SGI284081[] = {{&S65147[105], 4}, {NULL}};
const Segment SGI284082[] = {{&S65147[13], 14}, {&S65147[28], 6}, {&S65147[56], 17}, {NULL}};
const Segment SGI284083[] = {{&S65147[75], 18}, {NULL}};
const Segment SGI284084[] = {{&S64830[93], 4}, {NULL}};
const Segment SGI284085[] = {{&S64830[13], 3}, {&S64830[17], 13}, {&S64830[52], 17}, {NULL}};
const Segment SGI284086[] = {{&S64830[71], 12}, {NULL}};
const Segment SGI284087[] = {{&S64831[91], 4}, {NULL}};
const Segment SGI284088[] = {{&S64831[13], 3}, {&S64831[17], 11}, {&S64831[50], 17}, {NULL}};
const Segment SGI284089[] = {{&S64831[69], 12}, {NULL}};
const Segment SGI284090[] = {{&S65423[96], 4}, {NULL}};
const Segment SGI284091[] = {{&S65423[13], 3}, {&S65423[17], 13}, {&S65423[31], 10}, {&S65423[42], 6}, {&S65423[70], 17}, {NULL}};
const Segment SGI284092[] = {{&S65424[96], 4}, {NULL}};
const Segment SGI284093[] = {{&S65424[13], 3}, {&S65424[17], 13}, {&S65424[31], 10}, {&S65424[42], 6}, {&S65424[70], 17}, {NULL}};
const Segment SGI284094[] = {{&S65425[96], 4}, {NULL}};
const Segment SGI284095[] = {{&S65425[13], 3}, {&S65425[17], 13}, {&S65425[31], 10}, {&S65425[42], 6}, {&S65425[70], 17}, {NULL}};
const Segment SGI284096[] = {{&S65426[93], 4}, {NULL}};
const Segment SGI284097[] = {{&S65426[13], 9}, {&S65426[23], 22}, {&S65426[67], 17}, {NULL}};
const Segment SGI284098[] = {{&S65426[115], 3}, {NULL}};
const Segment SGI284099[] = {{&S65427[93], 4}, {NULL}};
const Segment SGI284100[] = {{&S65427[13], 9}, {&S65427[23], 22}, {&S65427[67], 17}, {NULL}};
const Segment SGI284101[] = {{&S65427[115], 3}, {NULL}};
const Segment SGI284102[] = {{&S65428[93], 4}, {NULL}};
const Segment SGI284103[] = {{&S65428[13], 9}, {&S65428[23], 22}, {&S65428[67], 17}, {NULL}};
const Segment SGI284104[] = {{&S65428[115], 3}, {NULL}};
const Segment SGI284105[] = {{&S65429[93], 4}, {NULL}};
const Segment SGI284106[] = {{&S65429[13], 9}, {&S65429[23], 22}, {&S65429[67], 17}, {NULL}};
const Segment SGI284107[] = {{&S65429[115], 3}, {NULL}};
const Segment SGI284108[] = {{&S65023[111], 4}, {NULL}};
const Segment SGI284109[] = {{&S65023[13], 9}, {&S65023[23], 22}, {&S65023[66], 17}, {NULL}};
const Segment SGI284110[] = {{&S65023[85], 11}, {NULL}};
const Segment SGI284111[] = {{&S65530[101], 4}, {NULL}};
const Segment SGI284112[] = {{&S65530[13], 14}, {&S65530[28], 6}, {&S65530[55], 17}, {NULL}};
const Segment SGI284113[] = {{&S65530[83], 3}, {NULL}};
const Segment SGI284114[] = {{&S65531[94], 4}, {NULL}};
const Segment SGI284115[] = {{&S65531[13], 14}, {&S65531[48], 17}, {NULL}};
const Segment SGI284116[] = {{&S65531[76], 3}, {NULL}};
const Segment SGI284117[] = {{&S65532[101], 4}, {NULL}};
const Segment SGI284118[] = {{&S65532[13], 14}, {&S65532[28], 6}, {&S65532[55], 17}, {NULL}};
const Segment SGI284119[] = {{&S65532[83], 3}, {NULL}};
const Segment SGI284120[] = {{&S65533[95], 4}, {NULL}};
const Segment SGI284121[] = {{&S65533[13], 14}, {&S65533[49], 17}, {NULL}};
const Segment SGI284122[] = {{&S65533[77], 3}, {NULL}};
const Segment SGI284123[] = {{&S65534[94], 4}, {NULL}};
const Segment SGI284124[] = {{&S65534[13], 14}, {&S65534[48], 17}, {NULL}};
const Segment SGI284125[] = {{&S65534[76], 3}, {NULL}};
const Segment SGI284126[] = {{&S65535[114], 4}, {NULL}};
const Segment SGI284127[] = {{&S65535[13], 7}, {&S65535[21], 2}, {&S65535[24], 15}, {&S65535[40], 6}, {&S65535[68], 17}, {NULL}};
const Segment SGI284128[] = {{&S65535[96], 3}, {NULL}};
const Segment SGI284129[] = {{&S65536[102], 4}, {NULL}};
const Segment SGI284130[] = {{&S65536[13], 14}, {&S65536[28], 6}, {&S65536[56], 17}, {NULL}};
const Segment SGI284131[] = {{&S65536[84], 3}, {NULL}};
const Segment SGI284132[] = {{&S65538[95], 4}, {NULL}};
const Segment SGI284133[] = {{&S65538[13], 14}, {&S65538[49], 17}, {NULL}};
const Segment SGI284134[] = {{&S65538[77], 3}, {NULL}};
const Segment SGI284135[] = {{&S65539[101], 4}, {NULL}};
const Segment SGI284136[] = {{&S65539[13], 14}, {&S65539[28], 6}, {&S65539[55], 17}, {NULL}};
const Segment SGI284137[] = {{&S65539[83], 3}, {NULL}};
const Segment SGI284138[] = {{&S65598[79], 4}, {NULL}};
const Segment SGI284139[] = {{&S65598[13], 3}, {&S65598[17], 7}, {&S65598[25], 13}, {&S65598[39], 8}, {NULL}};
const Segment SGI284140[] = {{&S73570[91], 3}, {NULL}};
const Segment SGI284141[] = {{&S73570[13], 7}, {&S73570[21], 2}, {&S73570[24], 15}, {&S73570[40], 6}, {&S73570[67], 17}, {NULL}};
const Segment SGI284142[] = {{&S73570[86], 8}, {NULL}};
const Segment SGI284143[] = {{&S65083[75], 4}, {NULL}};
const Segment SGI284144[] = {{&S65083[13], 14}, {&S65083[49], 17}, {NULL}};
const Segment SGI284145[] = {{&S65083[102], 18}, {NULL}};
const Segment SGI284146[] = {{&S65084[82], 4}, {NULL}};
const Segment SGI284147[] = {{&S65084[13], 14}, {&S65084[28], 6}, {&S65084[56], 17}, {NULL}};
const Segment SGI284148[] = {{&S65084[109], 18}, {NULL}};
const Segment SGI284149[] = {{&S65104[92], 4}, {NULL}};
const Segment SGI284150[] = {{&S65104[13], 9}, {&S65104[23], 22}, {&S65104[66], 17}, {NULL}};
const Segment SGI284151[] = {{&S65104[104], 9}, {NULL}};
const Segment SGI284152[] = {{&S65105[92], 4}, {NULL}};
const Segment SGI284153[] = {{&S65105[13], 9}, {&S65105[23], 22}, {&S65105[66], 17}, {NULL}};
const Segment SGI284154[] = {{&S65105[104], 9}, {NULL}};
const Segment SGI284155[] = {{&S73854[113], 3}, {NULL}};
const Segment SGI284156[] = {{&S73854[13], 7}, {&S73854[21], 2}, {&S73854[24], 15}, {&S73854[40], 6}, {&S73854[68], 17}, {NULL}};
const Segment SGI284157[] = {{&S73854[96], 3}, {NULL}};
const Segment SGI284158[] = {{&S65902[74], 4}, {NULL}};
const Segment SGI284159[] = {{&S65902[13], 14}, {&S65902[48], 17}, {NULL}};
const Segment SGI284160[] = {{&S65903[98], 4}, {NULL}};
const Segment SGI284161[] = {{&S65903[13], 14}, {&S65903[48], 17}, {NULL}};
const Segment SGI284162[] = {{&S65930[87], 4}, {NULL}};
const Segment SGI284163[] = {{&S65930[13], 3}, {&S65930[17], 13}, {&S65930[31], 8}, {&S65930[61], 17}, {NULL}};
const Segment SGI284164[] = {{&S65438[82], 4}, {NULL}};
const Segment SGI284165[] = {{&S65438[13], 14}, {&S65438[28], 6}, {&S65438[56], 17}, {NULL}};
const Segment SGI284166[] = {{&S65438[109], 12}, {NULL}};
const Segment SGI284167[] = {{&S65439[75], 4}, {NULL}};
const Segment SGI284168[] = {{&S65439[13], 14}, {&S65439[49], 17}, {NULL}};
const Segment SGI284169[] = {{&S65439[102], 12}, {NULL}};
const Segment SGI284170[] = {{&S66134[81], 4}, {NULL}};
const Segment SGI284171[] = {{&S66134[13], 14}, {&S66134[48], 17}, {NULL}};
const Segment SGI284172[] = {{&S66134[67], 18}, {NULL}};
const Segment SGI284173[] = {{&S66135[88], 4}, {NULL}};
const Segment SGI284174[] = {{&S66135[13], 14}, {&S66135[28], 6}, {&S66135[55], 17}, {NULL}};
const Segment SGI284175[] = {{&S66135[74], 18}, {NULL}};
const Segment SGI284176[] = {{&S66152[81], 4}, {NULL}};
const Segment SGI284177[] = {{&S66152[13], 14}, {&S66152[48], 17}, {NULL}};
const Segment SGI284178[] = {{&S66152[67], 18}, {NULL}};
const Segment SGI284179[] = {{&S66153[88], 4}, {NULL}};
const Segment SGI284180[] = {{&S66153[13], 14}, {&S66153[28], 6}, {&S66153[55], 17}, {NULL}};
const Segment SGI284181[] = {{&S66153[74], 18}, {NULL}};
const Segment SGI284182[] = {{&S65552[103], 4}, {NULL}};
const Segment SGI284183[] = {{&S65552[13], 9}, {&S65552[23], 2}, {&S65552[26], 22}, {&S65552[49], 6}, {&S65552[77], 17}, {NULL}};
const Segment SGI284184[] = {{&S65553[103], 4}, {NULL}};
const Segment SGI284185[] = {{&S65553[13], 9}, {&S65553[23], 2}, {&S65553[26], 22}, {&S65553[49], 6}, {&S65553[77], 17}, {NULL}};
const Segment SGI284186[] = {{&S66203[88], 4}, {NULL}};
const Segment SGI284187[] = {{&S66203[13], 14}, {&S66203[28], 6}, {&S66203[55], 17}, {NULL}};
const Segment SGI284188[] = {{&S66203[74], 18}, {NULL}};
const Segment SGI284189[] = {{&S66204[81], 4}, {NULL}};
const Segment SGI284190[] = {{&S66204[13], 14}, {&S66204[48], 17}, {NULL}};
const Segment SGI284191[] = {{&S66204[67], 18}, {NULL}};
const Segment SGI284192[] = {{&S66325[74], 4}, {NULL}};
const Segment SGI284193[] = {{&S66325[13], 14}, {&S66325[48], 17}, {NULL}};
const Segment SGI284194[] = {{&S65742[81], 4}, {NULL}};
const Segment SGI284195[] = {{&S65742[13], 14}, {&S65742[28], 6}, {&S65742[55], 17}, {NULL}};
const Segment SGI284196[] = {{&S65742[107], 18}, {NULL}};
const Segment SGI284197[] = {{&S65743[74], 4}, {NULL}};
const Segment SGI284198[] = {{&S65743[13], 14}, {&S65743[48], 17}, {NULL}};
const Segment SGI284199[] = {{&S65743[100], 18}, {NULL}};
const Segment SGI284200[] = {{&S65812[75], 4}, {NULL}};
const Segment SGI284201[] = {{&S65812[13], 14}, {&S65812[49], 17}, {NULL}};
const Segment SGI284202[] = {{&S65812[102], 12}, {NULL}};
const Segment SGI284203[] = {{&S65813[82], 4}, {NULL}};
const Segment SGI284204[] = {{&S65813[13], 14}, {&S65813[28], 6}, {&S65813[56], 17}, {NULL}};
const Segment SGI284205[] = {{&S65813[109], 12}, {NULL}};
const Segment SGI284206[] = {{&S65814[82], 4}, {NULL}};
const Segment SGI284207[] = {{&S65814[13], 14}, {&S65814[28], 6}, {&S65814[56], 17}, {NULL}};
const Segment SGI284208[] = {{&S65814[110], 12}, {NULL}};
const Segment SGI284209[] = {{&S65815[75], 4}, {NULL}};
const Segment SGI284210[] = {{&S65815[13], 14}, {&S65815[49], 17}, {NULL}};
const Segment SGI284211[] = {{&S65815[103], 12}, {NULL}};
const Segment SGI284212[] = {{&S65887[78], 4}, {NULL}};
const Segment SGI284213[] = {{&S65887[13], 3}, {&S65887[17], 13}, {&S65887[52], 17}, {NULL}};
const Segment SGI284214[] = {{&S65888[79], 4}, {NULL}};
const Segment SGI284215[] = {{&S65888[13], 3}, {&S65888[17], 14}, {&S65888[53], 17}, {NULL}};
const Segment SGI284216[] = {{&S74466[122], 3}, {NULL}};
const Segment SGI284217[] = {{&S74466[13], 9}, {&S74466[23], 2}, {&S74466[26], 22}, {&S74466[49], 6}, {&S74466[77], 17}, {NULL}};
const Segment SGI284218[] = {{&S74466[105], 3}, {NULL}};
const Segment SGI284219[] = {{&S74467[122], 3}, {NULL}};
const Segment SGI284220[] = {{&S74467[13], 9}, {&S74467[23], 2}, {&S74467[26], 22}, {&S74467[49], 6}, {&S74467[77], 17}, {NULL}};
const Segment SGI284221[] = {{&S74467[105], 3}, {NULL}};
const Segment SGI284222[] = {{&S74528[100], 3}, {NULL}};
const Segment SGI284223[] = {{&S74528[13], 3}, {&S74528[17], 2}, {&S74528[20], 11}, {&S74528[32], 6}, {&S74528[60], 17}, {NULL}};
const Segment SGI284224[] = {{&S74528[79], 11}, {NULL}};
const Segment SGI284225[] = {{&S66734[90], 4}, {NULL}};
const Segment SGI284226[] = {{&S66734[13], 3}, {&S66734[17], 11}, {&S66734[49], 17}, {NULL}};
const Segment SGI284227[] = {{&S66734[68], 12}, {NULL}};
const Segment SGI284228[] = {{&S66735[92], 4}, {NULL}};
const Segment SGI284229[] = {{&S66735[13], 3}, {&S66735[17], 13}, {&S66735[51], 17}, {NULL}};
const Segment SGI284230[] = {{&S66735[70], 12}, {NULL}};
const Segment SGI284231[] = {{&S66789[98], 4}, {NULL}};
const Segment SGI284232[] = {{&S66789[13], 14}, {&S66789[49], 17}, {NULL}};
const Segment SGI284233[] = {{&S66789[68], 18}, {NULL}};
const Segment SGI284234[] = {{&S66790[105], 4}, {NULL}};
const Segment SGI284235[] = {{&S66790[13], 14}, {&S66790[28], 6}, {&S66790[56], 17}, {NULL}};
const Segment SGI284236[] = {{&S66790[75], 18}, {NULL}};
const Segment SGI284237[] = {{&S74658[8], 3}, {NULL}};
const Segment SGI284238[] = {{&S74658[13], 9}, {&S74658[23], 2}, {&S74658[26], 22}, {&S74658[49], 6}, {&S74658[76], 17}, {NULL}};
const Segment SGI284239[] = {{&S74659[8], 3}, {NULL}};
const Segment SGI284240[] = {{&S74659[13], 9}, {&S74659[23], 2}, {&S74659[26], 22}, {&S74659[49], 6}, {&S74659[76], 17}, {NULL}};
const Segment SGI284241[] = {{&S74660[8], 3}, {NULL}};
const Segment SGI284242[] = {{&S74660[13], 9}, {&S74660[23], 2}, {&S74660[26], 22}, {&S74660[49], 6}, {&S74660[76], 17}, {NULL}};
const Segment SGI284243[] = {{&S74661[8], 3}, {NULL}};
const Segment SGI284244[] = {{&S74661[13], 9}, {&S74661[23], 2}, {&S74661[26], 22}, {&S74661[49], 6}, {&S74661[76], 17}, {NULL}};
const Segment SGI284245[] = {{&S74662[8], 3}, {NULL}};
const Segment SGI284246[] = {{&S74662[13], 9}, {&S74662[23], 2}, {&S74662[26], 22}, {&S74662[49], 6}, {&S74662[76], 17}, {NULL}};
const Segment SGI284247[] = {{&S66856[48], 4}, {NULL}};
const Segment SGI284248[] = {{&S66856[13], 14}, {&S66856[28], 6}, {&S66856[35], 25}, {&S66856[81], 17}, {NULL}};
const Segment SGI284249[] = {{&S66856[36], 16}, {NULL}};
const Segment SGI284250[] = {{&S66857[41], 4}, {NULL}};
const Segment SGI284251[] = {{&S66857[13], 14}, {&S66857[28], 25}, {&S66857[74], 17}, {NULL}};
const Segment SGI284252[] = {{&S66857[29], 16}, {NULL}};
const Segment SGI284253[] = {{&S66858[41], 4}, {NULL}};
const Segment SGI284254[] = {{&S66858[13], 14}, {&S66858[28], 25}, {&S66858[74], 17}, {NULL}};
const Segment SGI284255[] = {{&S66858[29], 16}, {NULL}};
const Segment SGI284256[] = {{&S66862[41], 4}, {NULL}};
const Segment SGI284257[] = {{&S66862[13], 14}, {&S66862[28], 25}, {&S66862[74], 17}, {NULL}};
const Segment SGI284258[] = {{&S66862[29], 16}, {NULL}};
const Segment SGI284259[] = {{&S66863[48], 4}, {NULL}};
const Segment SGI284260[] = {{&S66863[13], 14}, {&S66863[28], 6}, {&S66863[35], 25}, {&S66863[81], 17}, {NULL}};
const Segment SGI284261[] = {{&S66863[36], 16}, {NULL}};
const Segment SGI284262[] = {{&S66865[48], 4}, {NULL}};
const Segment SGI284263[] = {{&S66865[13], 14}, {&S66865[28], 6}, {&S66865[35], 25}, {&S66865[81], 17}, {NULL}};
const Segment SGI284264[] = {{&S66865[36], 16}, {NULL}};
const Segment SGI284265[] = {{&S66867[41], 4}, {NULL}};
const Segment SGI284266[] = {{&S66867[13], 14}, {&S66867[28], 24}, {&S66867[73], 17}, {NULL}};
const Segment SGI284267[] = {{&S66867[29], 16}, {NULL}};
const Segment SGI284268[] = {{&S66890[80], 4}, {NULL}};
const Segment SGI284269[] = {{&S66890[13], 14}, {&S66890[28], 6}, {&S66890[55], 17}, {NULL}};
const Segment SGI284270[] = {{&S66890[75], 9}, {NULL}};
const Segment SGI284271[] = {{&S66891[73], 4}, {NULL}};
const Segment SGI284272[] = {{&S66891[13], 14}, {&S66891[48], 17}, {NULL}};
const Segment SGI284273[] = {{&S66891[68], 9}, {NULL}};
const Segment SGI284274[] = {{&S66912[75], 4}, {NULL}};
const Segment SGI284275[] = {{&S66912[13], 14}, {&S66912[49], 17}, {NULL}};
const Segment SGI284276[] = {{&S66524[93], 4}, {NULL}};
const Segment SGI284277[] = {{&S66524[13], 9}, {&S66524[23], 22}, {&S66524[67], 17}, {NULL}};
const Segment SGI284278[] = {{&S66524[105], 9}, {NULL}};
const Segment SGI284279[] = {{&S66525[93], 4}, {NULL}};
const Segment SGI284280[] = {{&S66525[13], 9}, {&S66525[23], 22}, {&S66525[67], 17}, {NULL}};
const Segment SGI284281[] = {{&S66525[105], 9}, {NULL}};
const Segment SGI284282[] = {{&S75172[93], 3}, {NULL}};
const Segment SGI284283[] = {{&S75172[13], 3}, {&S75172[17], 2}, {&S75172[20], 18}, {&S75172[39], 6}, {&S75172[67], 17}, {NULL}};
const Segment SGI284284[] = {{&S75173[93], 3}, {NULL}};
const Segment SGI284285[] = {{&S75173[13], 3}, {&S75173[17], 2}, {&S75173[20], 18}, {&S75173[39], 6}, {&S75173[67], 17}, {NULL}};
const Segment SGI284286[] = {{&S66678[75], 4}, {NULL}};
const Segment SGI284287[] = {{&S66678[13], 14}, {&S66678[49], 17}, {NULL}};
const Segment SGI284288[] = {{&S66779[74], 4}, {NULL}};
const Segment SGI284289[] = {{&S66779[13], 14}, {&S66779[48], 17}, {NULL}};
const Segment SGI284290[] = {{&S66779[100], 12}, {NULL}};
const Segment SGI284291[] = {{&S66801[81], 4}, {NULL}};
const Segment SGI284292[] = {{&S66801[13], 14}, {&S66801[28], 6}, {&S66801[55], 17}, {NULL}};
const Segment SGI284293[] = {{&S66801[107], 18}, {NULL}};
const Segment SGI284294[] = {{&S66802[74], 4}, {NULL}};
const Segment SGI284295[] = {{&S66802[13], 14}, {&S66802[48], 17}, {NULL}};
const Segment SGI284296[] = {{&S66802[100], 18}, {NULL}};
const Segment SGI284297[] = {{&S67673[101], 4}, {NULL}};
const Segment SGI284298[] = {{&S67673[13], 7}, {&S67673[21], 2}, {&S67673[24], 15}, {&S67673[40], 6}, {&S67673[68], 17}, {NULL}};
const Segment SGI284299[] = {{&S67673[87], 18}, {NULL}};
const Segment SGI284300[] = {{&S75481[92], 3}, {NULL}};
const Segment SGI284301[] = {{&S75481[13], 7}, {&S75481[21], 2}, {&S75481[24], 15}, {&S75481[40], 6}, {&S75481[68], 17}, {NULL}};
const Segment SGI284302[] = {{&S75481[87], 8}, {NULL}};
const Segment SGI284303[] = {{&S67714[80], 4}, {NULL}};
const Segment SGI284304[] = {{&S67714[13], 14}, {&S67714[28], 6}, {&S67714[56], 17}, {NULL}};
const Segment SGI284305[] = {{&S67714[75], 9}, {NULL}};
const Segment SGI284306[] = {{&S67715[73], 4}, {NULL}};
const Segment SGI284307[] = {{&S67715[13], 14}, {&S67715[49], 17}, {NULL}};
const Segment SGI284308[] = {{&S67715[68], 9}, {NULL}};
const Segment SGI284309[] = {{&S67060[92], 4}, {NULL}};
const Segment SGI284310[] = {{&S67060[13], 9}, {&S67060[23], 22}, {&S67060[66], 17}, {NULL}};
const Segment SGI284311[] = {{&S67060[104], 9}, {NULL}};
const Segment SGI284312[] = {{&S67061[92], 4}, {NULL}};
const Segment SGI284313[] = {{&S67061[13], 9}, {&S67061[23], 22}, {&S67061[66], 17}, {NULL}};
const Segment SGI284314[] = {{&S67061[104], 9}, {NULL}};
const Segment SGI284315[] = {{&S67062[92], 4}, {NULL}};
const Segment SGI284316[] = {{&S67062[13], 9}, {&S67062[23], 22}, {&S67062[66], 17}, {NULL}};
const Segment SGI284317[] = {{&S67062[104], 9}, {NULL}};
const Segment SGI284318[] = {{&S75726[123], 3}, {NULL}};
const Segment SGI284319[] = {{&S75726[13], 5}, {&S75726[19], 2}, {&S75726[22], 14}, {&S75726[37], 6}, {&S75726[44], 31}, {&S75726[97], 17}, {NULL}};
const Segment SGI284320[] = {{&S75727[123], 3}, {NULL}};
const Segment SGI284321[] = {{&S75727[13], 5}, {&S75727[19], 2}, {&S75727[22], 14}, {&S75727[37], 6}, {&S75727[44], 31}, {&S75727[97], 17}, {NULL}};
const Segment SGI284322[] = {{&S75728[124], 3}, {NULL}};
const Segment SGI284323[] = {{&S75728[13], 5}, {&S75728[19], 2}, {&S75728[22], 14}, {&S75728[37], 6}, {&S75728[44], 32}, {&S75728[98], 17}, {NULL}};
const Segment SGI284324[] = {{&S75729[123], 3}, {NULL}};
const Segment SGI284325[] = {{&S75729[13], 5}, {&S75729[19], 2}, {&S75729[22], 14}, {&S75729[37], 6}, {&S75729[44], 31}, {&S75729[97], 17}, {NULL}};
const Segment SGI284326[] = {{&S75730[123], 3}, {NULL}};
const Segment SGI284327[] = {{&S75730[13], 5}, {&S75730[19], 2}, {&S75730[22], 14}, {&S75730[37], 6}, {&S75730[44], 31}, {&S75730[97], 17}, {NULL}};
const Segment SGI284328[] = {{&S75731[123], 3}, {NULL}};
const Segment SGI284329[] = {{&S75731[13], 5}, {&S75731[19], 2}, {&S75731[22], 14}, {&S75731[37], 6}, {&S75731[44], 31}, {&S75731[97], 17}, {NULL}};
const Segment SGI284330[] = {{&S75732[123], 3}, {NULL}};
const Segment SGI284331[] = {{&S75732[13], 5}, {&S75732[19], 2}, {&S75732[22], 14}, {&S75732[37], 6}, {&S75732[44], 31}, {&S75732[97], 17}, {NULL}};
const Segment SGI284332[] = {{&S75733[123], 3}, {NULL}};
const Segment SGI284333[] = {{&S75733[13], 5}, {&S75733[19], 2}, {&S75733[22], 14}, {&S75733[37], 6}, {&S75733[44], 31}, {&S75733[97], 17}, {NULL}};
const Segment SGI284334[] = {{&S75734[124], 3}, {NULL}};
const Segment SGI284335[] = {{&S75734[13], 5}, {&S75734[19], 2}, {&S75734[22], 14}, {&S75734[37], 6}, {&S75734[44], 32}, {&S75734[98], 17}, {NULL}};
const Segment SGI284336[] = {{&S67810[92], 4}, {NULL}};
const Segment SGI284337[] = {{&S67810[13], 7}, {&S67810[21], 2}, {&S67810[24], 15}, {&S67810[40], 6}, {&S67810[68], 17}, {NULL}};
const Segment SGI284338[] = {{&S67810[87], 9}, {NULL}};
const Segment SGI284339[] = {{&S67154[82], 4}, {NULL}};
const Segment SGI284340[] = {{&S67154[13], 14}, {&S67154[28], 6}, {&S67154[56], 17}, {NULL}};
const Segment SGI284341[] = {{&S67154[109], 18}, {NULL}};
const Segment SGI284342[] = {{&S67155[75], 4}, {NULL}};
const Segment SGI284343[] = {{&S67155[13], 14}, {&S67155[49], 17}, {NULL}};
const Segment SGI284344[] = {{&S67155[102], 18}, {NULL}};
const Segment SGI284345[] = {{&S67193[78], 4}, {NULL}};
const Segment SGI284346[] = {{&S67193[13], 3}, {&S67193[17], 14}, {&S67193[52], 17}, {NULL}};
const Segment SGI284347[] = {{&S67194[77], 4}, {NULL}};
const Segment SGI284348[] = {{&S67194[13], 3}, {&S67194[17], 13}, {&S67194[51], 17}, {NULL}};
const Segment SGI284349[] = {{&S67195[77], 4}, {NULL}};
const Segment SGI284350[] = {{&S67195[13], 3}, {&S67195[17], 14}, {&S67195[52], 16}, {NULL}};
const Segment SGI284351[] = {{&S75849[93], 3}, {NULL}};
const Segment SGI284352[] = {{&S75849[13], 7}, {&S75849[21], 2}, {&S75849[24], 15}, {&S75849[40], 6}, {&S75849[67], 17}, {NULL}};
const Segment SGI284353[] = {{&S67973[72], 4}, {NULL}};
const Segment SGI284354[] = {{&S67973[13], 14}, {&S67973[48], 17}, {NULL}};
const Segment SGI284355[] = {{&S67973[67], 9}, {NULL}};
const Segment SGI284356[] = {{&S67974[79], 4}, {NULL}};
const Segment SGI284357[] = {{&S67974[13], 14}, {&S67974[28], 6}, {&S67974[55], 17}, {NULL}};
const Segment SGI284358[] = {{&S67974[74], 9}, {NULL}};
const Segment SGI284359[] = {{&S68133[86], 4}, {NULL}};
const Segment SGI284360[] = {{&S68133[13], 3}, {&S68133[17], 22}, {&S68133[60], 17}, {NULL}};
const Segment SGI284361[] = {{&S68134[86], 4}, {NULL}};
const Segment SGI284362[] = {{&S68134[13], 3}, {&S68134[17], 22}, {&S68134[60], 17}, {NULL}};
const Segment SGI284363[] = {{&S68135[86], 4}, {NULL}};
const Segment SGI284364[] = {{&S68135[13], 3}, {&S68135[17], 22}, {&S68135[60], 17}, {NULL}};
const Segment SGI284365[] = {{&S68136[86], 4}, {NULL}};
const Segment SGI284366[] = {{&S68136[13], 3}, {&S68136[17], 22}, {&S68136[60], 17}, {NULL}};
const Segment SGI284367[] = {{&S68137[85], 4}, {NULL}};
const Segment SGI284368[] = {{&S68137[13], 3}, {&S68137[17], 21}, {&S68137[59], 17}, {NULL}};
const Segment SGI284369[] = {{&S68138[85], 4}, {NULL}};
const Segment SGI284370[] = {{&S68138[13], 3}, {&S68138[17], 21}, {&S68138[59], 17}, {NULL}};
const Segment SGI284371[] = {{&S68139[85], 4}, {NULL}};
const Segment SGI284372[] = {{&S68139[13], 3}, {&S68139[17], 21}, {&S68139[59], 17}, {NULL}};
const Segment SGI284373[] = {{&S68140[87], 4}, {NULL}};
const Segment SGI284374[] = {{&S68140[13], 3}, {&S68140[17], 23}, {&S68140[61], 17}, {NULL}};
const Segment SGI284375[] = {{&S76285[13], 14}, {&S76285[28], 6}, {&S76285[55], 17}, {&S76285[83], 21}, {NULL}};
const Segment SGI284376[] = {{&S76286[94], 3}, {NULL}};
const Segment SGI284377[] = {{&S76286[13], 7}, {&S76286[21], 2}, {&S76286[24], 15}, {&S76286[40], 6}, {&S76286[68], 17}, {NULL}};
const Segment SGI284378[] = {{&S67627[85], 4}, {NULL}};
const Segment SGI284379[] = {{&S67627[13], 3}, {&S67627[17], 11}, {&S67627[50], 17}, {NULL}};
const Segment SGI284380[] = {{&S67627[69], 15}, {NULL}};
const Segment SGI284381[] = {{&S67628[87], 4}, {NULL}};
const Segment SGI284382[] = {{&S67628[13], 3}, {&S67628[17], 13}, {&S67628[52], 17}, {NULL}};
const Segment SGI284383[] = {{&S67628[71], 15}, {NULL}};
const Segment SGI284384[] = {{&S67632[91], 4}, {NULL}};
const Segment SGI284385[] = {{&S67632[13], 3}, {&S67632[17], 11}, {&S67632[50], 17}, {NULL}};
const Segment SGI284386[] = {{&S67632[69], 12}, {NULL}};
const Segment SGI284387[] = {{&S67633[93], 4}, {NULL}};
const Segment SGI284388[] = {{&S67633[13], 3}, {&S67633[17], 13}, {&S67633[52], 17}, {NULL}};
const Segment SGI284389[] = {{&S67633[71], 12}, {NULL}};
const Segment SGI284390[] = {{&S67634[80], 4}, {NULL}};
const Segment SGI284391[] = {{&S67634[13], 3}, {&S67634[17], 13}, {&S67634[52], 17}, {&S67634[141], 14}, {NULL}};
const Segment SGI284392[] = {{&S67634[127], 12}, {NULL}};
const Segment SGI284393[] = {{&S68600[74], 4}, {NULL}};
const Segment SGI284394[] = {{&S68600[13], 3}, {&S68600[17], 11}, {&S68600[50], 17}, {NULL}};
const Segment SGI284395[] = {{&S68600[69], 9}, {NULL}};
const Segment SGI284396[] = {{&S68601[76], 4}, {NULL}};
const Segment SGI284397[] = {{&S68601[13], 3}, {&S68601[17], 13}, {&S68601[52], 17}, {NULL}};
const Segment SGI284398[] = {{&S68601[71], 9}, {NULL}};
const Segment SGI284399[] = {{&S67977[75], 4}, {NULL}};
const Segment SGI284400[] = {{&S67977[13], 15}, {&S67977[49], 17}, {NULL}};
const Segment SGI284401[] = {{&S68750[78], 4}, {NULL}};
const Segment SGI284402[] = {{&S68750[13], 3}, {&S68750[17], 13}, {&S68750[52], 17}, {NULL}};
const Segment SGI284403[] = {{&S68102[93], 4}, {NULL}};
const Segment SGI284404[] = {{&S68102[13], 3}, {&S68102[17], 13}, {&S68102[52], 17}, {NULL}};
const Segment SGI284405[] = {{&S68102[71], 12}, {NULL}};
const Segment SGI284406[] = {{&S68103[91], 4}, {NULL}};
const Segment SGI284407[] = {{&S68103[13], 3}, {&S68103[17], 11}, {&S68103[50], 17}, {NULL}};
const Segment SGI284408[] = {{&S68103[69], 12}, {NULL}};
const Segment SGI284409[] = {{&S68868[97], 4}, {NULL}};
const Segment SGI284410[] = {{&S68868[13], 14}, {&S68868[48], 17}, {NULL}};
const Segment SGI284411[] = {{&S68868[67], 18}, {NULL}};
const Segment SGI284412[] = {{&S77005[101], 3}, {NULL}};
const Segment SGI284413[] = {{&S77005[13], 9}, {&S77005[23], 2}, {&S77005[26], 22}, {&S77005[49], 6}, {&S77005[77], 17}, {NULL}};
const Segment SGI284414[] = {{&S77005[96], 8}, {NULL}};
const Segment SGI284415[] = {{&S77006[101], 3}, {NULL}};
const Segment SGI284416[] = {{&S77006[13], 9}, {&S77006[23], 2}, {&S77006[26], 22}, {&S77006[49], 6}, {&S77006[77], 17}, {NULL}};
const Segment SGI284417[] = {{&S77006[96], 8}, {NULL}};
const Segment SGI284418[] = {{&S77007[101], 3}, {NULL}};
const Segment SGI284419[] = {{&S77007[13], 9}, {&S77007[23], 2}, {&S77007[26], 22}, {&S77007[49], 6}, {&S77007[77], 17}, {NULL}};
const Segment SGI284420[] = {{&S77007[96], 8}, {NULL}};
const Segment SGI284421[] = {{&S77067[101], 3}, {NULL}};
const Segment SGI284422[] = {{&S77067[13], 7}, {&S77067[21], 2}, {&S77067[24], 15}, {&S77067[40], 6}, {&S77067[68], 17}, {NULL}};
const Segment SGI284423[] = {{&S77067[87], 17}, {NULL}};
const Segment SGI284424[] = {{&S77090[8], 3}, {NULL}};
const Segment SGI284425[] = {{&S77090[13], 7}, {&S77090[21], 2}, {&S77090[24], 15}, {&S77090[40], 6}, {&S77090[67], 17}, {NULL}};
const Segment SGI284426[] = {{&S77090[86], 8}, {NULL}};
const Segment SGI284427[] = {{&S77092[123], 3}, {NULL}};
const Segment SGI284428[] = {{&S77092[13], 9}, {&S77092[23], 2}, {&S77092[26], 22}, {&S77092[49], 6}, {&S77092[77], 17}, {NULL}};
const Segment SGI284429[] = {{&S77092[105], 3}, {NULL}};
const Segment SGI284430[] = {{&S77093[122], 3}, {NULL}};
const Segment SGI284431[] = {{&S77093[13], 9}, {&S77093[23], 2}, {&S77093[26], 22}, {&S77093[49], 6}, {&S77093[76], 17}, {NULL}};
const Segment SGI284432[] = {{&S77093[104], 3}, {NULL}};
const Segment SGI284433[] = {{&S77094[122], 3}, {NULL}};
const Segment SGI284434[] = {{&S77094[13], 9}, {&S77094[23], 2}, {&S77094[26], 22}, {&S77094[49], 6}, {&S77094[76], 17}, {NULL}};
const Segment SGI284435[] = {{&S77094[104], 3}, {NULL}};
const Segment SGI284436[] = {{&S77095[123], 3}, {NULL}};
const Segment SGI284437[] = {{&S77095[13], 9}, {&S77095[23], 2}, {&S77095[26], 22}, {&S77095[49], 6}, {&S77095[77], 17}, {NULL}};
const Segment SGI284438[] = {{&S77095[105], 3}, {NULL}};
const Segment SGI284439[] = {{&S77096[122], 3}, {NULL}};
const Segment SGI284440[] = {{&S77096[13], 9}, {&S77096[23], 2}, {&S77096[26], 22}, {&S77096[49], 6}, {&S77096[76], 17}, {NULL}};
const Segment SGI284441[] = {{&S77096[104], 3}, {NULL}};
const Segment SGI284442[] = {{&S68940[98], 4}, {NULL}};
const Segment SGI284443[] = {{&S68940[13], 14}, {&S68940[49], 17}, {NULL}};
const Segment SGI284444[] = {{&S68940[68], 18}, {NULL}};
const Segment SGI284445[] = {{&S68941[98], 4}, {NULL}};
const Segment SGI284446[] = {{&S68941[13], 14}, {&S68941[49], 17}, {NULL}};
const Segment SGI284447[] = {{&S68941[68], 18}, {NULL}};
const Segment SGI284448[] = {{&S77465[113], 3}, {NULL}};
const Segment SGI284449[] = {{&S77465[13], 7}, {&S77465[21], 2}, {&S77465[24], 15}, {&S77465[40], 6}, {&S77465[67], 17}, {NULL}};
const Segment SGI284450[] = {{&S77465[95], 3}, {NULL}};
const Segment SGI284451[] = {{&S77466[113], 3}, {NULL}};
const Segment SGI284452[] = {{&S77466[13], 7}, {&S77466[21], 2}, {&S77466[24], 15}, {&S77466[40], 6}, {&S77466[67], 17}, {NULL}};
const Segment SGI284453[] = {{&S77466[95], 3}, {NULL}};
const Segment SGI284454[] = {{&S77467[114], 3}, {NULL}};
const Segment SGI284455[] = {{&S77467[13], 7}, {&S77467[21], 2}, {&S77467[24], 15}, {&S77467[40], 6}, {&S77467[68], 17}, {NULL}};
const Segment SGI284456[] = {{&S77467[96], 3}, {NULL}};
const Segment SGI284457[] = {{&S68820[81], 4}, {NULL}};
const Segment SGI284458[] = {{&S68820[13], 14}, {&S68820[28], 6}, {&S68820[55], 17}, {NULL}};
const Segment SGI284459[] = {{&S68820[107], 12}, {NULL}};
const Segment SGI284460[] = {{&S68821[74], 4}, {NULL}};
const Segment SGI284461[] = {{&S68821[13], 14}, {&S68821[48], 17}, {NULL}};
const Segment SGI284462[] = {{&S68821[100], 12}, {NULL}};
const Segment SGI284463[] = {{&S69312[74], 4}, {NULL}};
const Segment SGI284464[] = {{&S68824[81], 4}, {NULL}};
const Segment SGI284465[] = {{&S69312[13], 14}, {&S69312[48], 17}, {NULL}};
const Segment SGI284466[] = {{&S68824[13], 14}, {&S68824[28], 6}, {&S68824[55], 17}, {NULL}};
const Segment SGI284467[] = {{&S68824[107], 12}, {NULL}};
const Segment SGI284468[] = {{&S68825[74], 4}, {NULL}};
const Segment SGI284469[] = {{&S68825[13], 14}, {&S68825[48], 17}, {NULL}};
const Segment SGI284470[] = {{&S68825[100], 12}, {NULL}};
const Segment SGI284471[] = {{&S69317[90], 4}, {NULL}};
const Segment SGI284472[] = {{&S69317[13], 9}, {&S69317[23], 22}, {&S69317[66], 17}, {NULL}};
const Segment SGI284473[] = {{&S69317[85], 9}, {NULL}};
const Segment SGI284474[] = {{&S69459[74], 4}, {NULL}};
const Segment SGI284475[] = {{&S69459[13], 14}, {&S69459[48], 17}, {NULL}};
const Segment SGI284476[] = {{&S69484[101], 4}, {NULL}};
const Segment SGI284477[] = {{&S69484[13], 7}, {&S69484[21], 2}, {&S69484[24], 15}, {&S69484[40], 6}, {&S69484[68], 17}, {NULL}};
const Segment SGI284478[] = {{&S69484[87], 18}, {NULL}};
const Segment SGI284479[] = {{&S69138[74], 4}, {NULL}};
const Segment SGI284480[] = {{&S69138[13], 14}, {&S69138[48], 17}, {NULL}};
const Segment SGI284481[] = {{&S69138[109], 4}, {NULL}};
const Segment SGI284482[] = {{&S69156[93], 4}, {NULL}};
const Segment SGI284483[] = {{&S69156[13], 3}, {&S69156[17], 13}, {&S69156[52], 17}, {NULL}};
const Segment SGI284484[] = {{&S69156[71], 12}, {NULL}};
const Segment SGI284485[] = {{&S69157[91], 4}, {NULL}};
const Segment SGI284486[] = {{&S69157[13], 3}, {&S69157[17], 11}, {&S69157[50], 17}, {NULL}};
const Segment SGI284487[] = {{&S69157[69], 12}, {NULL}};
const Segment SGI284488[] = {{&S69194[75], 4}, {NULL}};
const Segment SGI284489[] = {{&S69194[13], 14}, {&S69194[49], 17}, {NULL}};
const Segment SGI284490[] = {{&S69194[102], 18}, {NULL}};
const Segment SGI284491[] = {{&S63761[7], 3}, {NULL}};
const Segment SGI284492[] = {{&S63761[11], 6}, {NULL}};
const Segment SGI284493[] = {{&S69802[72], 4}, {NULL}};
const Segment SGI284494[] = {{&S69802[13], 14}, {&S69802[48], 17}, {NULL}};
const Segment SGI284495[] = {{&S69802[67], 9}, {NULL}};
const Segment SGI284496[] = {{&S69803[79], 4}, {NULL}};
const Segment SGI284497[] = {{&S69803[13], 14}, {&S69803[28], 6}, {&S69803[55], 17}, {NULL}};
const Segment SGI284498[] = {{&S69803[74], 9}, {NULL}};
const Segment SGI284499[] = {{&S78120[101], 3}, {NULL}};
const Segment SGI284500[] = {{&S78120[13], 7}, {&S78120[21], 2}, {&S78120[24], 15}, {&S78120[40], 6}, {&S78120[68], 17}, {NULL}};
const Segment SGI284501[] = {{&S78120[87], 17}, {NULL}};
const Segment SGI284502[] = {{&S69523[82], 4}, {NULL}};
const Segment SGI284503[] = {{&S69523[13], 14}, {&S69523[28], 6}, {&S69523[56], 17}, {NULL}};
const Segment SGI284504[] = {{&S69523[109], 12}, {NULL}};
const Segment SGI284505[] = {{&S69524[75], 4}, {NULL}};
const Segment SGI284506[] = {{&S69524[13], 14}, {&S69524[49], 17}, {NULL}};
const Segment SGI284507[] = {{&S69524[102], 12}, {NULL}};
const Segment SGI284508[] = {{&S69525[75], 4}, {NULL}};
const Segment SGI284509[] = {{&S69525[13], 14}, {&S69525[49], 17}, {NULL}};
const Segment SGI284510[] = {{&S69525[103], 12}, {NULL}};
const Segment SGI284511[] = {{&S69526[82], 4}, {NULL}};
const Segment SGI284512[] = {{&S69526[13], 14}, {&S69526[28], 6}, {&S69526[56], 17}, {NULL}};
const Segment SGI284513[] = {{&S69526[110], 12}, {NULL}};
const Segment SGI284514[] = {{&S78279[119], 3}, {NULL}};
const Segment SGI284515[] = {{&S78279[13], 9}, {&S78279[23], 2}, {&S78279[26], 22}, {&S78279[49], 6}, {&S78279[76], 17}, {NULL}};
const Segment SGI284516[] = {{&S78279[95], 9}, {NULL}};
const Segment SGI284517[] = {{&S70070[90], 4}, {NULL}};
const Segment SGI284518[] = {{&S70070[13], 3}, {&S70070[17], 11}, {&S70070[49], 17}, {NULL}};
const Segment SGI284519[] = {{&S70070[68], 12}, {NULL}};
const Segment SGI284520[] = {{&S70071[92], 4}, {NULL}};
const Segment SGI284521[] = {{&S70071[13], 3}, {&S70071[17], 13}, {&S70071[51], 17}, {NULL}};
const Segment SGI284522[] = {{&S70071[70], 12}, {NULL}};
const Segment SGI284523[] = {{&S70095[92], 4}, {NULL}};
const Segment SGI284524[] = {{&S70095[13], 3}, {&S70095[17], 13}, {&S70095[51], 17}, {NULL}};
const Segment SGI284525[] = {{&S70095[70], 12}, {NULL}};
const Segment SGI284526[] = {{&S70096[91], 4}, {NULL}};
const Segment SGI284527[] = {{&S70096[13], 3}, {&S70096[17], 11}, {&S70096[50], 17}, {NULL}};
const Segment SGI284528[] = {{&S70096[69], 12}, {NULL}};
const Segment SGI284529[] = {{&S69819[79], 4}, {NULL}};
const Segment SGI284530[] = {{&S69819[13], 3}, {&S69819[17], 14}, {&S69819[53], 17}, {NULL}};
const Segment SGI284531[] = {{&S69820[78], 4}, {NULL}};
const Segment SGI284532[] = {{&S69820[13], 3}, {&S69820[17], 13}, {&S69820[52], 17}, {NULL}};
const Segment SGI284533[] = {{&S78423[100], 3}, {NULL}};
const Segment SGI284534[] = {{&S78423[13], 7}, {&S78423[21], 2}, {&S78423[24], 15}, {&S78423[40], 6}, {&S78423[67], 17}, {NULL}};
const Segment SGI284535[] = {{&S78423[86], 17}, {NULL}};
const Segment SGI284536[] = {{&S70147[105], 4}, {NULL}};
const Segment SGI284537[] = {{&S70147[13], 14}, {&S70147[28], 6}, {&S70147[56], 17}, {NULL}};
const Segment SGI284538[] = {{&S70147[75], 18}, {NULL}};
const Segment SGI284539[] = {{&S70148[98], 4}, {NULL}};
const Segment SGI284540[] = {{&S70148[13], 14}, {&S70148[49], 17}, {NULL}};
const Segment SGI284541[] = {{&S70148[68], 18}, {NULL}};
const Segment SGI284542[] = {{&S70363[105], 4}, {NULL}};
const Segment SGI284543[] = {{&S70363[13], 14}, {&S70363[28], 6}, {&S70363[56], 17}, {NULL}};
const Segment SGI284544[] = {{&S70363[75], 18}, {NULL}};
const Segment SGI284545[] = {{&S70364[98], 4}, {NULL}};
const Segment SGI284546[] = {{&S70364[13], 14}, {&S70364[49], 17}, {NULL}};
const Segment SGI284547[] = {{&S70364[68], 18}, {NULL}};
const Segment SGI284548[] = {{&S78667[114], 3}, {NULL}};
const Segment SGI284549[] = {{&S78667[13], 7}, {&S78667[21], 2}, {&S78667[24], 15}, {&S78667[40], 6}, {&S78667[68], 17}, {NULL}};
const Segment SGI284550[] = {{&S78667[96], 3}, {NULL}};
const Segment SGI284551[] = {{&S78668[113], 3}, {NULL}};
const Segment SGI284552[] = {{&S78668[13], 7}, {&S78668[21], 2}, {&S78668[24], 15}, {&S78668[40], 6}, {&S78668[67], 17}, {NULL}};
const Segment SGI284553[] = {{&S78668[95], 3}, {NULL}};
const Segment SGI284554[] = {{&S70475[75], 4}, {NULL}};
const Segment SGI284555[] = {{&S70475[13], 14}, {&S70475[49], 17}, {NULL}};
const Segment SGI284556[] = {{&S70475[103], 18}, {NULL}};
const Segment SGI284557[] = {{&S70476[82], 4}, {NULL}};
const Segment SGI284558[] = {{&S70476[13], 14}, {&S70476[28], 6}, {&S70476[56], 17}, {NULL}};
const Segment SGI284559[] = {{&S70476[110], 18}, {NULL}};
const Segment SGI284560[] = {{&S70556[104], 4}, {NULL}};
const Segment SGI284561[] = {{&S70556[13], 14}, {&S70556[28], 6}, {&S70556[55], 17}, {NULL}};
const Segment SGI284562[] = {{&S70556[74], 18}, {NULL}};
const Segment SGI284563[] = {{&S70557[97], 4}, {NULL}};
const Segment SGI284564[] = {{&S70557[13], 14}, {&S70557[48], 17}, {NULL}};
const Segment SGI284565[] = {{&S70557[67], 18}, {NULL}};
const Segment SGI284566[] = {{&S70610[97], 4}, {NULL}};
const Segment SGI284567[] = {{&S70610[13], 14}, {&S70610[48], 17}, {NULL}};
const Segment SGI284568[] = {{&S70610[67], 18}, {NULL}};
const Segment SGI284569[] = {{&S70611[104], 4}, {NULL}};
const Segment SGI284570[] = {{&S70611[13], 14}, {&S70611[28], 6}, {&S70611[55], 17}, {NULL}};
const Segment SGI284571[] = {{&S70611[74], 18}, {NULL}};
const Segment SGI284572[] = {{&S70433[93], 4}, {NULL}};
const Segment SGI284573[] = {{&S70433[13], 9}, {&S70433[23], 22}, {&S70433[67], 17}, {NULL}};
const Segment SGI284574[] = {{&S70433[106], 9}, {NULL}};
const Segment SGI284575[] = {{&S70434[93], 4}, {NULL}};
const Segment SGI284576[] = {{&S70434[13], 9}, {&S70434[23], 22}, {&S70434[67], 17}, {NULL}};
const Segment SGI284577[] = {{&S70434[106], 9}, {NULL}};
const Segment SGI284578[] = {{&S78898[94], 3}, {NULL}};
const Segment SGI284579[] = {{&S78898[13], 7}, {&S78898[21], 2}, {&S78898[24], 15}, {&S78898[40], 6}, {&S78898[68], 17}, {NULL}};
const Segment SGI284580[] = {{&S78899[94], 3}, {NULL}};
const Segment SGI284581[] = {{&S78899[13], 7}, {&S78899[21], 2}, {&S78899[24], 15}, {&S78899[40], 6}, {&S78899[68], 17}, {NULL}};
const Segment SGI284582[] = {{&S78900[82], 3}, {NULL}};
const Segment SGI284583[] = {{&S78900[13], 14}, {&S78900[28], 6}, {&S78900[56], 17}, {NULL}};
const Segment SGI284584[] = {{&S70694[75], 4}, {NULL}};
const Segment SGI284585[] = {{&S70694[13], 14}, {&S70694[49], 17}, {NULL}};
const Segment SGI284586[] = {{&S70694[101], 18}, {NULL}};
const Segment SGI284587[] = {{&S70695[82], 4}, {NULL}};
const Segment SGI284588[] = {{&S70695[13], 14}, {&S70695[28], 6}, {&S70695[56], 17}, {NULL}};
const Segment SGI284589[] = {{&S70695[108], 18}, {NULL}};
const Segment SGI284590[] = {{&S70648[93], 4}, {NULL}};
const Segment SGI284591[] = {{&S70648[13], 9}, {&S70648[23], 22}, {&S70648[67], 17}, {NULL}};
const Segment SGI284592[] = {{&S70648[105], 9}, {NULL}};
const Segment SGI284593[] = {{&S79305[94], 3}, {NULL}};
const Segment SGI284594[] = {{&S79305[13], 7}, {&S79305[21], 2}, {&S79305[24], 15}, {&S79305[40], 6}, {&S79305[68], 17}, {NULL}};
const Segment SGI284595[] = {{&S71083[94], 4}, {NULL}};
const Segment SGI284596[] = {{&S71083[13], 7}, {&S71083[21], 2}, {&S71083[24], 15}, {&S71083[40], 6}, {&S71083[68], 17}, {NULL}};
const Segment SGI284597[] = {{&S71093[90], 4}, {NULL}};
const Segment SGI284598[] = {{&S71093[13], 9}, {&S71093[23], 22}, {&S71093[66], 17}, {NULL}};
const Segment SGI284599[] = {{&S71093[85], 9}, {NULL}};
const Segment SGI284600[] = {{&S71094[90], 4}, {NULL}};
const Segment SGI284601[] = {{&S71094[13], 9}, {&S71094[23], 22}, {&S71094[66], 17}, {NULL}};
const Segment SGI284602[] = {{&S71094[85], 9}, {NULL}};
const Segment SGI284603[] = {{&S71095[90], 4}, {NULL}};
const Segment SGI284604[] = {{&S71095[13], 9}, {&S71095[23], 22}, {&S71095[66], 17}, {NULL}};
const Segment SGI284605[] = {{&S71095[85], 9}, {NULL}};
const Segment SGI284606[] = {{&S71096[90], 4}, {NULL}};
const Segment SGI284607[] = {{&S71096[13], 9}, {&S71096[23], 22}, {&S71096[66], 17}, {NULL}};
const Segment SGI284608[] = {{&S71096[85], 9}, {NULL}};
const Segment SGI284609[] = {{&S70853[93], 4}, {NULL}};
const Segment SGI284610[] = {{&S70853[13], 9}, {&S70853[23], 22}, {&S70853[67], 17}, {NULL}};
const Segment SGI284611[] = {{&S70853[106], 9}, {NULL}};
const Segment SGI284612[] = {{&S70854[93], 4}, {NULL}};
const Segment SGI284613[] = {{&S70854[13], 9}, {&S70854[23], 22}, {&S70854[67], 17}, {NULL}};
const Segment SGI284614[] = {{&S70854[106], 9}, {NULL}};
const Segment SGI284615[] = {{&S70855[93], 4}, {NULL}};
const Segment SGI284616[] = {{&S70855[13], 9}, {&S70855[23], 22}, {&S70855[67], 17}, {NULL}};
const Segment SGI284617[] = {{&S70855[106], 9}, {NULL}};
const Segment SGI284618[] = {{&S70967[82], 4}, {NULL}};
const Segment SGI284619[] = {{&S70967[13], 14}, {&S70967[28], 6}, {&S70967[56], 17}, {NULL}};
const Segment SGI284620[] = {{&S70967[95], 9}, {NULL}};
const Segment SGI284621[] = {{&S70968[75], 4}, {NULL}};
const Segment SGI284622[] = {{&S70968[13], 14}, {&S70968[49], 17}, {NULL}};
const Segment SGI284623[] = {{&S70968[88], 9}, {NULL}};
const Segment SGI284624[] = {{&S71104[90], 4}, {NULL}};
const Segment SGI284625[] = {{&S71104[13], 9}, {&S71104[23], 22}, {&S71104[66], 17}, {NULL}};
const Segment SGI284626[] = {{&S71104[85], 9}, {NULL}};
const Segment SGI284627[] = {{&S71105[90], 4}, {NULL}};
const Segment SGI284628[] = {{&S71105[13], 9}, {&S71105[23], 22}, {&S71105[66], 17}, {NULL}};
const Segment SGI284629[] = {{&S71105[85], 9}, {NULL}};
const Segment SGI284630[] = {{&S71106[90], 4}, {NULL}};
const Segment SGI284631[] = {{&S71106[13], 9}, {&S71106[23], 22}, {&S71106[66], 17}, {NULL}};
const Segment SGI284632[] = {{&S71106[85], 9}, {NULL}};
const Segment SGI284633[] = {{&S71175[75], 4}, {NULL}};
const Segment SGI284634[] = {{&S71175[13], 14}, {&S71175[49], 17}, {NULL}};
const Segment SGI284635[] = {{&S71071[93], 4}, {NULL}};
const Segment SGI284636[] = {{&S71071[13], 3}, {&S71071[17], 13}, {&S71071[52], 17}, {NULL}};
const Segment SGI284637[] = {{&S71071[71], 12}, {NULL}};
const Segment SGI284638[] = {{&S71072[91], 4}, {NULL}};
const Segment SGI284639[] = {{&S71072[13], 3}, {&S71072[17], 11}, {&S71072[50], 17}, {NULL}};
const Segment SGI284640[] = {{&S71072[69], 12}, {NULL}};
const Segment SGI284641[] = {{&S71231[73], 4}, {NULL}};
const Segment SGI284642[] = {{&S71231[13], 14}, {&S71231[49], 17}, {NULL}};
const Segment SGI284643[] = {{&S71231[68], 9}, {NULL}};
const Segment SGI284644[] = {{&S71232[80], 4}, {NULL}};
const Segment SGI284645[] = {{&S71232[13], 14}, {&S71232[28], 6}, {&S71232[56], 17}, {NULL}};
const Segment SGI284646[] = {{&S71232[75], 9}, {NULL}};
const Segment SGI284647[] = {{&S71264[97], 4}, {NULL}};
const Segment SGI284648[] = {{&S71264[13], 14}, {&S71264[48], 17}, {NULL}};
const Segment SGI284649[] = {{&S71264[67], 18}, {NULL}};
const Segment SGI284650[] = {{&S79685[100], 3}, {NULL}};
const Segment SGI284651[] = {{&S79685[13], 7}, {&S79685[21], 2}, {&S79685[24], 15}, {&S79685[40], 6}, {&S79685[67], 17}, {NULL}};
const Segment SGI284652[] = {{&S79685[86], 17}, {NULL}};
const Segment SGI284653[] = {{&S71265[104], 4}, {NULL}};
const Segment SGI284654[] = {{&S71265[13], 14}, {&S71265[28], 6}, {&S71265[55], 17}, {NULL}};
const Segment SGI284655[] = {{&S71265[74], 18}, {NULL}};
const Segment SGI284656[] = {{&S71275[78], 4}, {NULL}};
const Segment SGI284657[] = {{&S71275[13], 3}, {&S71275[17], 14}, {&S71275[52], 17}, {NULL}};
const Segment SGI284658[] = {{&S71276[77], 4}, {NULL}};
const Segment SGI284659[] = {{&S71276[13], 3}, {&S71276[17], 13}, {&S71276[51], 17}, {NULL}};
const Segment SGI284660[] = {{&S71277[77], 4}, {NULL}};
const Segment SGI284661[] = {{&S71277[13], 3}, {&S71277[17], 14}, {&S71277[52], 16}, {NULL}};
const Segment SGI284662[] = {{&S79757[100], 3}, {NULL}};
const Segment SGI284663[] = {{&S79757[13], 9}, {&S79757[23], 2}, {&S79757[26], 22}, {&S79757[49], 6}, {&S79757[76], 17}, {NULL}};
const Segment SGI284664[] = {{&S79757[95], 8}, {NULL}};
const Segment SGI284665[] = {{&S79758[100], 3}, {NULL}};
const Segment SGI284666[] = {{&S79758[13], 9}, {&S79758[23], 2}, {&S79758[26], 22}, {&S79758[49], 6}, {&S79758[76], 17}, {NULL}};
const Segment SGI284667[] = {{&S79758[95], 8}, {NULL}};
const Segment SGI284668[] = {{&S71237[92], 4}, {NULL}};
const Segment SGI284669[] = {{&S71237[13], 9}, {&S71237[23], 22}, {&S71237[66], 17}, {NULL}};
const Segment SGI284670[] = {{&S71237[104], 9}, {NULL}};
const Segment SGI284671[] = {{&S71238[92], 4}, {NULL}};
const Segment SGI284672[] = {{&S71238[13], 9}, {&S71238[23], 22}, {&S71238[66], 17}, {NULL}};
const Segment SGI284673[] = {{&S71238[104], 9}, {NULL}};
const Segment SGI284674[] = {{&S71381[91], 4}, {NULL}};
const Segment SGI284675[] = {{&S71381[13], 9}, {&S71381[23], 22}, {&S71381[67], 17}, {NULL}};
const Segment SGI284676[] = {{&S71381[86], 9}, {NULL}};
const Segment SGI284677[] = {{&S71382[91], 4}, {NULL}};
const Segment SGI284678[] = {{&S71382[13], 9}, {&S71382[23], 22}, {&S71382[67], 17}, {NULL}};
const Segment SGI284679[] = {{&S71382[86], 9}, {NULL}};
const Segment SGI284680[] = {{&S71383[91], 4}, {NULL}};
const Segment SGI284681[] = {{&S71383[13], 9}, {&S71383[23], 22}, {&S71383[67], 17}, {NULL}};
const Segment SGI284682[] = {{&S71383[86], 9}, {NULL}};
const Segment SGI284683[] = {{&S71384[74], 4}, {NULL}};
const Segment SGI284684[] = {{&S71384[13], 14}, {&S71384[48], 17}, {NULL}};
const Segment SGI284685[] = {{&S71384[100], 12}, {NULL}};
const Segment SGI284686[] = {{&S71385[81], 4}, {NULL}};
const Segment SGI284687[] = {{&S71385[13], 14}, {&S71385[28], 6}, {&S71385[55], 17}, {NULL}};
const Segment SGI284688[] = {{&S71385[107], 12}, {NULL}};
const Segment SGI284689[] = {{&S80044[100], 3}, {NULL}};
const Segment SGI284690[] = {{&S80044[13], 7}, {&S80044[21], 2}, {&S80044[24], 15}, {&S80044[40], 6}, {&S80044[67], 17}, {NULL}};
const Segment SGI284691[] = {{&S80044[86], 17}, {NULL}};
const Segment SGI284692[] = {{&S71708[92], 4}, {NULL}};
const Segment SGI284693[] = {{&S71708[13], 9}, {&S71708[23], 22}, {&S71708[67], 17}, {NULL}};
const Segment SGI284694[] = {{&S71708[87], 9}, {NULL}};
const Segment SGI284695[] = {{&S71709[92], 4}, {NULL}};
const Segment SGI284696[] = {{&S71709[13], 9}, {&S71709[23], 22}, {&S71709[67], 17}, {NULL}};
const Segment SGI284697[] = {{&S71709[87], 9}, {NULL}};
const Segment SGI284698[] = {{&S71710[92], 4}, {NULL}};
const Segment SGI284699[] = {{&S71710[13], 9}, {&S71710[23], 22}, {&S71710[67], 17}, {NULL}};
const Segment SGI284700[] = {{&S71710[87], 9}, {NULL}};
const Segment SGI284701[] = {{&S71711[92], 4}, {NULL}};
const Segment SGI284702[] = {{&S71711[13], 9}, {&S71711[23], 22}, {&S71711[67], 17}, {NULL}};
const Segment SGI284703[] = {{&S71711[87], 9}, {NULL}};
const Segment SGI284704[] = {{&S71582[93], 4}, {NULL}};
const Segment SGI284705[] = {{&S71582[13], 9}, {&S71582[23], 22}, {&S71582[67], 17}, {NULL}};
const Segment SGI284706[] = {{&S71582[106], 9}, {NULL}};
const Segment SGI284707[] = {{&S71583[93], 4}, {NULL}};
const Segment SGI284708[] = {{&S71583[13], 9}, {&S71583[23], 22}, {&S71583[67], 17}, {NULL}};
const Segment SGI284709[] = {{&S71583[106], 9}, {NULL}};
const Segment SGI284710[] = {{&S71584[93], 4}, {NULL}};
const Segment SGI284711[] = {{&S71584[13], 9}, {&S71584[23], 22}, {&S71584[67], 17}, {NULL}};
const Segment SGI284712[] = {{&S71584[106], 9}, {NULL}};
const Segment SGI284713[] = {{&S71585[93], 4}, {NULL}};
const Segment SGI284714[] = {{&S71585[13], 9}, {&S71585[23], 22}, {&S71585[67], 17}, {NULL}};
const Segment SGI284715[] = {{&S71585[106], 9}, {NULL}};
const Segment SGI284716[] = {{&S71586[93], 4}, {NULL}};
const Segment SGI284717[] = {{&S71586[13], 9}, {&S71586[23], 22}, {&S71586[67], 17}, {NULL}};
const Segment SGI284718[] = {{&S71586[106], 9}, {NULL}};
const Segment SGI284719[] = {{&S71604[74], 4}, {NULL}};
const Segment SGI284720[] = {{&S71604[13], 14}, {&S71604[48], 17}, {NULL}};
const Segment SGI284721[] = {{&S71605[81], 4}, {NULL}};
const Segment SGI284722[] = {{&S71605[13], 14}, {&S71605[28], 6}, {&S71605[55], 17}, {NULL}};
const Segment SGI284723[] = {{&S71673[74], 4}, {NULL}};
const Segment SGI284724[] = {{&S71673[13], 14}, {&S71673[48], 17}, {NULL}};
const Segment SGI284725[] = {{&S71673[100], 18}, {NULL}};
const Segment SGI284726[] = {{&S71674[81], 4}, {NULL}};
const Segment SGI284727[] = {{&S71674[13], 14}, {&S71674[28], 6}, {&S71674[55], 17}, {NULL}};
const Segment SGI284728[] = {{&S71674[107], 18}, {NULL}};
const Segment SGI284729[] = {{&S71898[81], 4}, {NULL}};
const Segment SGI284730[] = {{&S71898[13], 14}, {&S71898[28], 6}, {&S71898[55], 17}, {NULL}};
const Segment SGI284731[] = {{&S71898[107], 18}, {NULL}};
const Segment SGI284732[] = {{&S71899[74], 4}, {NULL}};
const Segment SGI284733[] = {{&S71899[13], 14}, {&S71899[48], 17}, {NULL}};
const Segment SGI284734[] = {{&S71899[100], 18}, {NULL}};
const Segment SGI284735[] = {{&S71914[90], 4}, {NULL}};
const Segment SGI284736[] = {{&S71914[13], 9}, {&S71914[23], 22}, {&S71914[66], 17}, {NULL}};
const Segment SGI284737[] = {{&S71914[85], 9}, {NULL}};
const Segment SGI284738[] = {{&S71915[90], 4}, {NULL}};
const Segment SGI284739[] = {{&S71915[13], 9}, {&S71915[23], 22}, {&S71915[66], 17}, {NULL}};
const Segment SGI284740[] = {{&S71915[85], 9}, {NULL}};
const Segment SGI284741[] = {{&S71813[93], 4}, {NULL}};
const Segment SGI284742[] = {{&S71813[13], 9}, {&S71813[23], 22}, {&S71813[67], 17}, {NULL}};
const Segment SGI284743[] = {{&S71934[41], 4}, {NULL}};
const Segment SGI284744[] = {{&S71934[13], 14}, {&S71934[28], 25}, {&S71934[75], 17}, {NULL}};
const Segment SGI284745[] = {{&S71935[41], 4}, {NULL}};
const Segment SGI284746[] = {{&S71935[13], 14}, {&S71935[28], 25}, {&S71935[75], 17}, {NULL}};
const Segment SGI284747[] = {{&S71936[48], 4}, {NULL}};
const Segment SGI284748[] = {{&S71936[13], 14}, {&S71936[28], 6}, {&S71936[35], 25}, {&S71936[82], 17}, {NULL}};
const Segment SGI284749[] = {{&S71939[41], 4}, {NULL}};
const Segment SGI284750[] = {{&S71939[13], 14}, {&S71939[28], 25}, {&S71939[75], 17}, {NULL}};
const Segment SGI284751[] = {{&S71941[48], 4}, {NULL}};
const Segment SGI284752[] = {{&S71941[13], 14}, {&S71941[28], 6}, {&S71941[35], 25}, {&S71941[82], 17}, {NULL}};
const Segment SGI284753[] = {{&S80577[117], 3}, {NULL}};
const Segment SGI284754[] = {{&S80577[13], 5}, {&S80577[19], 2}, {&S80577[22], 14}, {&S80577[37], 6}, {&S80577[44], 25}, {&S80577[91], 17}, {NULL}};
const Segment SGI284755[] = {{&S80578[117], 3}, {NULL}};
const Segment SGI284756[] = {{&S80578[13], 5}, {&S80578[19], 2}, {&S80578[22], 14}, {&S80578[37], 6}, {&S80578[44], 25}, {&S80578[91], 17}, {NULL}};
const Segment SGI284757[] = {{&S80579[117], 3}, {NULL}};
const Segment SGI284758[] = {{&S80579[13], 5}, {&S80579[19], 2}, {&S80579[22], 14}, {&S80579[37], 6}, {&S80579[44], 25}, {&S80579[91], 17}, {NULL}};
const Segment SGI284759[] = {{&S80580[117], 3}, {NULL}};
const Segment SGI284760[] = {{&S80580[13], 5}, {&S80580[19], 2}, {&S80580[22], 14}, {&S80580[37], 6}, {&S80580[44], 25}, {&S80580[91], 17}, {NULL}};
const Segment SGI284761[] = {{&S80581[118], 3}, {NULL}};
const Segment SGI284762[] = {{&S80581[13], 5}, {&S80581[19], 2}, {&S80581[22], 14}, {&S80581[37], 6}, {&S80581[44], 26}, {&S80581[92], 17}, {NULL}};
const Segment SGI284763[] = {{&S80582[117], 3}, {NULL}};
const Segment SGI284764[] = {{&S80582[13], 5}, {&S80582[19], 2}, {&S80582[22], 14}, {&S80582[37], 6}, {&S80582[44], 25}, {&S80582[91], 17}, {NULL}};
const Segment SGI284765[] = {{&S72150[98], 4}, {NULL}};
const Segment SGI284766[] = {{&S72150[13], 14}, {&S72150[49], 17}, {NULL}};
const Segment SGI284767[] = {{&S72150[68], 18}, {NULL}};
const Segment SGI284768[] = {{&S72151[105], 4}, {NULL}};
const Segment SGI284769[] = {{&S72151[13], 14}, {&S72151[28], 6}, {&S72151[56], 17}, {NULL}};
const Segment SGI284770[] = {{&S72151[75], 18}, {NULL}};
const Segment SGI284771[] = {{&S72217[91], 4}, {NULL}};
const Segment SGI284772[] = {{&S72217[13], 9}, {&S72217[23], 22}, {&S72217[67], 17}, {NULL}};
const Segment SGI284773[] = {{&S72217[86], 9}, {NULL}};
const Segment SGI284774[] = {{&S72246[90], 4}, {NULL}};
const Segment SGI284775[] = {{&S72246[13], 9}, {&S72246[23], 22}, {&S72246[66], 17}, {NULL}};
const Segment SGI284776[] = {{&S72246[85], 9}, {NULL}};
const Segment SGI284777[] = {{&S72247[90], 4}, {NULL}};
const Segment SGI284778[] = {{&S72247[13], 9}, {&S72247[23], 22}, {&S72247[66], 17}, {NULL}};
const Segment SGI284779[] = {{&S72247[85], 9}, {NULL}};
const Segment SGI284780[] = {{&S72248[90], 4}, {NULL}};
const Segment SGI284781[] = {{&S72248[13], 9}, {&S72248[23], 22}, {&S72248[66], 17}, {NULL}};
const Segment SGI284782[] = {{&S72248[85], 9}, {NULL}};
const Segment SGI284783[] = {{&S72249[90], 4}, {NULL}};
const Segment SGI284784[] = {{&S72249[13], 9}, {&S72249[23], 22}, {&S72249[66], 17}, {NULL}};
const Segment SGI284785[] = {{&S72249[85], 9}, {NULL}};
const Segment SGI284786[] = {{&S72250[90], 4}, {NULL}};
const Segment SGI284787[] = {{&S72250[13], 9}, {&S72250[23], 22}, {&S72250[66], 17}, {NULL}};
const Segment SGI284788[] = {{&S72250[85], 9}, {NULL}};
const Segment SGI284789[] = {{&S72255[93], 4}, {NULL}};
const Segment SGI284790[] = {{&S72255[13], 9}, {&S72255[23], 22}, {&S72255[67], 17}, {NULL}};
const Segment SGI284791[] = {{&S72255[115], 3}, {NULL}};
const Segment SGI284792[] = {{&S72256[93], 4}, {NULL}};
const Segment SGI284793[] = {{&S72256[13], 9}, {&S72256[23], 22}, {&S72256[67], 17}, {NULL}};
const Segment SGI284794[] = {{&S72256[115], 3}, {NULL}};
const Segment SGI284795[] = {{&S80885[8], 3}, {NULL}};
const Segment SGI284796[] = {{&S80885[13], 3}, {&S80885[17], 2}, {&S80885[20], 13}, {&S80885[34], 6}, {&S80885[61], 17}, {NULL}};
const Segment SGI284797[] = {{&S72316[92], 4}, {NULL}};
const Segment SGI284798[] = {{&S72316[13], 9}, {&S72316[23], 22}, {&S72316[67], 17}, {NULL}};
const Segment SGI284799[] = {{&S72316[87], 9}, {NULL}};
const Segment SGI284800[] = {{&S72403[81], 4}, {NULL}};
const Segment SGI284801[] = {{&S72403[13], 14}, {&S72403[48], 17}, {NULL}};
const Segment SGI284802[] = {{&S72403[67], 18}, {NULL}};
const Segment SGI284803[] = {{&S81071[91], 3}, {NULL}};
const Segment SGI284804[] = {{&S81071[13], 7}, {&S81071[21], 2}, {&S81071[24], 15}, {&S81071[40], 6}, {&S81071[67], 17}, {NULL}};
const Segment SGI284805[] = {{&S81071[86], 8}, {NULL}};
const Segment SGI284806[] = {{&S72611[74], 4}, {NULL}};
const Segment SGI284807[] = {{&S72611[13], 14}, {&S72611[48], 17}, {NULL}};
const Segment SGI284808[] = {{&S72656[73], 4}, {NULL}};
const Segment SGI284809[] = {{&S72656[13], 3}, {&S72656[17], 11}, {&S72656[49], 17}, {NULL}};
const Segment SGI284810[] = {{&S72656[68], 9}, {NULL}};
const Segment SGI284811[] = {{&S72657[75], 4}, {NULL}};
const Segment SGI284812[] = {{&S72657[13], 3}, {&S72657[17], 13}, {&S72657[51], 17}, {NULL}};
const Segment SGI284813[] = {{&S72657[70], 9}, {NULL}};
const Segment SGI284814[] = {{&S92103[38], 5}, {NULL}};
const Segment SGI284815[] = {{&S92103[99], 5}, {NULL}};
const Segment SGI284816[] = {{&S72733[105], 4}, {NULL}};
const Segment SGI284817[] = {{&S72733[13], 14}, {&S72733[28], 6}, {&S72733[56], 17}, {NULL}};
const Segment SGI284818[] = {{&S72733[75], 18}, {NULL}};
const Segment SGI284819[] = {{&S72734[98], 4}, {NULL}};
const Segment SGI284820[] = {{&S72734[13], 14}, {&S72734[49], 17}, {NULL}};
const Segment SGI284821[] = {{&S72734[68], 18}, {NULL}};
const Segment SGI284822[] = {{&S81315[8], 3}, {NULL}};
const Segment SGI284823[] = {{&S81315[13], 3}, {&S81315[17], 2}, {&S81315[20], 11}, {&S81315[32], 6}, {&S81315[59], 17}, {NULL}};
const Segment SGI284824[] = {{&S81315[123], 8}, {NULL}};
const Segment SGI284825[] = {{&S81467[261], 3}, {&S81467[384], 3}, {NULL}};
const Segment SGI284826[] = {{&S81467[13], 7}, {&S81467[21], 2}, {&S81467[24], 15}, {&S81467[40], 6}, {&S81467[47], 12}, {&S81467[85], 21}, {&S81467[21], 2}, {&S81467[24], 15}, {&S81467[126], 6}, {&S81467[133], 15}, {&S81467[174], 17}, {&S81467[21], 2}, {&S81467[195], 13}, {&S81467[40], 6}, {&S81467[236], 16}, {&S81467[85], 21}, {&S81467[21], 2}, {&S81467[24], 15}, {&S81467[40], 6}, {&S81467[351], 17}, {&S81467[174], 17}, {&S81467[21], 2}, {&S81467[448], 19}, {&S81467[40], 6}, {&S81467[476], 10}, {NULL}};
const Segment SGI284827[] = {{&S81467[370], 17}, {NULL}};
const Segment SGI284828[] = {{&S81468[94], 3}, {NULL}};
const Segment SGI284829[] = {{&S81468[13], 3}, {&S81468[17], 2}, {&S81468[20], 13}, {&S81468[34], 6}, {&S81468[61], 17}, {NULL}};
const Segment SGI284830[] = {{&S81468[80], 17}, {NULL}};
const Segment SGI284831[] = {{&S81526[87], 3}, {NULL}};
const Segment SGI284832[] = {{&S81526[13], 3}, {&S81526[17], 2}, {&S81526[20], 13}, {&S81526[34], 6}, {&S81526[61], 17}, {NULL}};
const Segment SGI284833[] = {{&S72821[78], 4}, {NULL}};
const Segment SGI284834[] = {{&S72821[13], 3}, {&S72821[17], 13}, {&S72821[52], 17}, {NULL}};
const Segment SGI284835[] = {{&S72821[91], 9}, {NULL}};
const Segment SGI284836[] = {{&S72822[76], 4}, {NULL}};
const Segment SGI284837[] = {{&S72822[13], 3}, {&S72822[17], 11}, {&S72822[50], 17}, {NULL}};
const Segment SGI284838[] = {{&S72822[89], 9}, {NULL}};
const Segment SGI284839[] = {{&S81880[114], 3}, {NULL}};
const Segment SGI284840[] = {{&S81880[13], 7}, {&S81880[21], 2}, {&S81880[24], 15}, {&S81880[40], 6}, {&S81880[68], 17}, {NULL}};
const Segment SGI284841[] = {{&S81880[96], 3}, {NULL}};
const Segment SGI284842[] = {{&S81881[113], 3}, {NULL}};
const Segment SGI284843[] = {{&S81881[13], 7}, {&S81881[21], 2}, {&S81881[24], 15}, {&S81881[40], 6}, {&S81881[67], 17}, {NULL}};
const Segment SGI284844[] = {{&S81881[95], 3}, {NULL}};
const Segment SGI284845[] = {{&S81882[113], 3}, {NULL}};
const Segment SGI284846[] = {{&S81882[13], 7}, {&S81882[21], 2}, {&S81882[24], 15}, {&S81882[40], 6}, {&S81882[67], 17}, {NULL}};
const Segment SGI284847[] = {{&S81882[95], 3}, {NULL}};
const Segment SGI284848[] = {{&S73197[74], 4}, {NULL}};
const Segment SGI284849[] = {{&S73197[13], 14}, {&S73197[48], 17}, {NULL}};
const Segment SGI284850[] = {{&S73197[100], 18}, {NULL}};
const Segment SGI284851[] = {{&S73198[81], 4}, {NULL}};
const Segment SGI284852[] = {{&S73198[13], 14}, {&S73198[28], 6}, {&S73198[55], 17}, {NULL}};
const Segment SGI284853[] = {{&S73198[107], 18}, {NULL}};
const Segment SGI284854[] = {{&S73378[75], 4}, {NULL}};
const Segment SGI284855[] = {{&S73378[13], 3}, {&S73378[17], 13}, {&S73378[51], 17}, {NULL}};
const Segment SGI284856[] = {{&S73378[70], 9}, {NULL}};
const Segment SGI284857[] = {{&S73379[73], 4}, {NULL}};
const Segment SGI284858[] = {{&S73379[13], 3}, {&S73379[17], 11}, {&S73379[49], 17}, {NULL}};
const Segment SGI284859[] = {{&S73379[68], 9}, {NULL}};
const Segment SGI284860[] = {{&S73681[75], 4}, {NULL}};
const Segment SGI284861[] = {{&S73681[13], 14}, {&S73681[49], 17}, {NULL}};
const Segment SGI284862[] = {{&S73681[102], 18}, {NULL}};
const Segment SGI284863[] = {{&S82453[114], 3}, {NULL}};
const Segment SGI284864[] = {{&S82453[13], 7}, {&S82453[21], 2}, {&S82453[24], 15}, {&S82453[40], 6}, {&S82453[68], 17}, {NULL}};
const Segment SGI284865[] = {{&S82453[96], 3}, {NULL}};
const Segment SGI284866[] = {{&S73707[75], 4}, {NULL}};
const Segment SGI284867[] = {{&S73707[13], 14}, {&S73707[49], 17}, {NULL}};
const Segment SGI284868[] = {{&S73707[111], 4}, {NULL}};
const Segment SGI284869[] = {{&S73704[91], 4}, {NULL}};
const Segment SGI284870[] = {{&S73704[13], 9}, {&S73704[23], 22}, {&S73704[67], 17}, {NULL}};
const Segment SGI284871[] = {{&S73704[86], 9}, {NULL}};
const Segment SGI284872[] = {{&S73731[104], 4}, {NULL}};
const Segment SGI284873[] = {{&S73731[13], 4}, {&S73731[18], 2}, {&S73731[21], 27}, {&S73731[49], 6}, {&S73731[77], 17}, {NULL}};
const Segment SGI284874[] = {{&S73731[130], 10}, {NULL}};
const Segment SGI284875[] = {{&S73732[104], 4}, {NULL}};
const Segment SGI284876[] = {{&S73732[13], 4}, {&S73732[18], 2}, {&S73732[21], 27}, {&S73732[49], 6}, {&S73732[77], 17}, {NULL}};
const Segment SGI284877[] = {{&S73732[130], 10}, {NULL}};
const Segment SGI284878[] = {{&S73849[95], 4}, {NULL}};
const Segment SGI284879[] = {{&S73849[13], 14}, {&S73849[48], 17}, {NULL}};
const Segment SGI284880[] = {{&S73849[76], 3}, {NULL}};
const Segment SGI284881[] = {{&S73850[114], 4}, {NULL}};
const Segment SGI284882[] = {{&S73850[13], 7}, {&S73850[21], 2}, {&S73850[24], 15}, {&S73850[40], 6}, {&S73850[68], 17}, {NULL}};
const Segment SGI284883[] = {{&S73850[96], 3}, {NULL}};
const Segment SGI284884[] = {{&S73851[95], 4}, {NULL}};
const Segment SGI284885[] = {{&S73851[13], 14}, {&S73851[49], 17}, {NULL}};
const Segment SGI284886[] = {{&S73851[77], 3}, {NULL}};
const Segment SGI284887[] = {{&S73852[94], 4}, {NULL}};
const Segment SGI284888[] = {{&S73852[13], 14}, {&S73852[48], 17}, {NULL}};
const Segment SGI284889[] = {{&S73852[76], 3}, {NULL}};
const Segment SGI284890[] = {{&S73853[95], 4}, {NULL}};
const Segment SGI284891[] = {{&S73853[13], 14}, {&S73853[49], 17}, {NULL}};
const Segment SGI284892[] = {{&S73853[77], 3}, {NULL}};
const Segment SGI284893[] = {{&S73882[75], 4}, {NULL}};
const Segment SGI284894[] = {{&S73882[13], 3}, {&S73882[17], 11}, {&S73882[50], 17}, {NULL}};
const Segment SGI284895[] = {{&S73882[70], 9}, {NULL}};
const Segment SGI284896[] = {{&S73883[77], 4}, {NULL}};
const Segment SGI284897[] = {{&S73883[13], 3}, {&S73883[17], 13}, {&S73883[52], 17}, {NULL}};
const Segment SGI284898[] = {{&S73883[72], 9}, {NULL}};
const Segment SGI284899[] = {{&S74028[74], 4}, {NULL}};
const Segment SGI284900[] = {{&S74028[13], 14}, {&S74028[48], 17}, {NULL}};
const Segment SGI284901[] = {{&S74028[100], 18}, {NULL}};
const Segment SGI284902[] = {{&S74057[98], 4}, {NULL}};
const Segment SGI284903[] = {{&S74057[13], 14}, {&S74057[49], 17}, {NULL}};
const Segment SGI284904[] = {{&S74057[68], 18}, {NULL}};
const Segment SGI284905[] = {{&S74105[75], 4}, {NULL}};
const Segment SGI284906[] = {{&S74105[13], 14}, {&S74105[49], 17}, {NULL}};
const Segment SGI284907[] = {{&S74105[103], 18}, {NULL}};
const Segment SGI284908[] = {{&S74112[79], 4}, {NULL}};
const Segment SGI284909[] = {{&S74112[13], 14}, {&S74112[28], 6}, {&S74112[55], 17}, {NULL}};
const Segment SGI284910[] = {{&S74112[74], 9}, {NULL}};
const Segment SGI284911[] = {{&S74113[72], 4}, {NULL}};
const Segment SGI284912[] = {{&S74113[13], 14}, {&S74113[48], 17}, {NULL}};
const Segment SGI284913[] = {{&S74113[67], 9}, {NULL}};
const Segment SGI284914[] = {{&S74137[88], 4}, {NULL}};
const Segment SGI284915[] = {{&S74137[13], 14}, {&S74137[28], 6}, {&S74137[55], 17}, {NULL}};
const Segment SGI284916[] = {{&S74137[74], 18}, {NULL}};
const Segment SGI284917[] = {{&S74138[81], 4}, {NULL}};
const Segment SGI284918[] = {{&S74138[13], 14}, {&S74138[48], 17}, {NULL}};
const Segment SGI284919[] = {{&S74138[67], 18}, {NULL}};
const Segment SGI284920[] = {{&S74224[75], 4}, {NULL}};
const Segment SGI284921[] = {{&S74224[13], 14}, {&S74224[49], 17}, {NULL}};
const Segment SGI284922[] = {{&S74315[75], 4}, {NULL}};
const Segment SGI284923[] = {{&S74315[13], 14}, {&S74315[49], 17}, {NULL}};
const Segment SGI284924[] = {{&S74344[98], 4}, {NULL}};
const Segment SGI284925[] = {{&S74344[13], 14}, {&S74344[49], 17}, {NULL}};
const Segment SGI284926[] = {{&S74344[68], 18}, {NULL}};
const Segment SGI284927[] = {{&S74345[105], 4}, {NULL}};
const Segment SGI284928[] = {{&S74345[13], 14}, {&S74345[28], 6}, {&S74345[56], 17}, {NULL}};
const Segment SGI284929[] = {{&S74345[75], 18}, {NULL}};
const Segment SGI284930[] = {{&S74380[97], 4}, {NULL}};
const Segment SGI284931[] = {{&S74380[13], 14}, {&S74380[48], 17}, {NULL}};
const Segment SGI284932[] = {{&S74380[67], 18}, {NULL}};
const Segment SGI284933[] = {{&S74381[104], 4}, {NULL}};
const Segment SGI284934[] = {{&S74381[13], 14}, {&S74381[28], 6}, {&S74381[55], 17}, {NULL}};
const Segment SGI284935[] = {{&S74381[74], 18}, {NULL}};
const Segment SGI284936[] = {{&S74329[75], 4}, {NULL}};
const Segment SGI284937[] = {{&S74329[13], 14}, {&S74329[49], 17}, {NULL}};
const Segment SGI284938[] = {{&S74329[103], 12}, {NULL}};
const Segment SGI284939[] = {{&S74460[112], 4}, {NULL}};
const Segment SGI284940[] = {{&S74460[13], 9}, {&S74460[23], 22}, {&S74460[66], 17}, {NULL}};
const Segment SGI284941[] = {{&S74460[94], 3}, {NULL}};
const Segment SGI284942[] = {{&S74461[113], 4}, {NULL}};
const Segment SGI284943[] = {{&S74461[13], 9}, {&S74461[23], 22}, {&S74461[66], 17}, {NULL}};
const Segment SGI284944[] = {{&S74461[94], 3}, {NULL}};
const Segment SGI284945[] = {{&S74462[112], 4}, {NULL}};
const Segment SGI284946[] = {{&S74462[13], 9}, {&S74462[23], 22}, {&S74462[66], 17}, {NULL}};
const Segment SGI284947[] = {{&S74462[94], 3}, {NULL}};
const Segment SGI284948[] = {{&S74463[112], 4}, {NULL}};
const Segment SGI284949[] = {{&S74463[13], 9}, {&S74463[23], 22}, {&S74463[66], 17}, {NULL}};
const Segment SGI284950[] = {{&S74463[94], 3}, {NULL}};
const Segment SGI284951[] = {{&S74464[113], 4}, {NULL}};
const Segment SGI284952[] = {{&S74464[13], 9}, {&S74464[23], 22}, {&S74464[67], 17}, {NULL}};
const Segment SGI284953[] = {{&S74464[95], 3}, {NULL}};
const Segment SGI284954[] = {{&S74465[113], 4}, {NULL}};
const Segment SGI284955[] = {{&S74465[13], 9}, {&S74465[23], 22}, {&S74465[67], 17}, {NULL}};
const Segment SGI284956[] = {{&S74465[95], 3}, {NULL}};
const Segment SGI284957[] = {{&S74468[113], 4}, {NULL}};
const Segment SGI284958[] = {{&S74468[13], 9}, {&S74468[23], 22}, {&S74468[67], 17}, {NULL}};
const Segment SGI284959[] = {{&S74468[95], 3}, {NULL}};
const Segment SGI284960[] = {{&S74469[112], 4}, {NULL}};
const Segment SGI284961[] = {{&S74469[13], 9}, {&S74469[23], 22}, {&S74469[66], 17}, {NULL}};
const Segment SGI284962[] = {{&S74469[94], 3}, {NULL}};
const Segment SGI284963[] = {{&S74470[123], 4}, {NULL}};
const Segment SGI284964[] = {{&S74470[13], 9}, {&S74470[23], 2}, {&S74470[26], 22}, {&S74470[49], 6}, {&S74470[77], 17}, {NULL}};
const Segment SGI284965[] = {{&S74470[105], 3}, {NULL}};
const Segment SGI284966[] = {{&S74589[74], 4}, {NULL}};
const Segment SGI284967[] = {{&S74589[13], 14}, {&S74589[48], 17}, {NULL}};
const Segment SGI284968[] = {{&S74690[74], 4}, {NULL}};
const Segment SGI284969[] = {{&S74690[13], 14}, {&S74690[48], 17}, {NULL}};
const Segment SGI284970[] = {{&S74873[91], 4}, {NULL}};
const Segment SGI284971[] = {{&S74873[13], 3}, {&S74873[17], 11}, {&S74873[50], 17}, {NULL}};
const Segment SGI284972[] = {{&S74873[69], 12}, {NULL}};
const Segment SGI284973[] = {{&S74874[93], 4}, {NULL}};
const Segment SGI284974[] = {{&S74874[13], 3}, {&S74874[17], 13}, {&S74874[52], 17}, {NULL}};
const Segment SGI284975[] = {{&S74874[71], 12}, {NULL}};
const Segment SGI284976[] = {{&S83577[83], 3}, {NULL}};
const Segment SGI284977[] = {{&S83577[13], 3}, {&S83577[17], 2}, {&S83577[20], 11}, {&S83577[32], 6}, {&S83577[59], 17}, {NULL}};
const Segment SGI284978[] = {{&S83577[78], 8}, {NULL}};
const Segment SGI284979[] = {{&S83578[85], 3}, {NULL}};
const Segment SGI284980[] = {{&S83578[13], 3}, {&S83578[17], 2}, {&S83578[20], 13}, {&S83578[34], 6}, {&S83578[61], 17}, {NULL}};
const Segment SGI284981[] = {{&S83578[80], 8}, {NULL}};
const Segment SGI284982[] = {{&S75047[75], 4}, {NULL}};
const Segment SGI284983[] = {{&S75047[13], 14}, {&S75047[49], 17}, {NULL}};
const Segment SGI284984[] = {{&S75048[81], 4}, {NULL}};
const Segment SGI284985[] = {{&S75048[13], 14}, {&S75048[48], 17}, {NULL}};
const Segment SGI284986[] = {{&S75048[67], 18}, {NULL}};
const Segment SGI284987[] = {{&S75076[136], 4}, {NULL}};
const Segment SGI284988[] = {{&S75076[13], 3}, {&S75076[17], 13}, {&S75076[51], 17}, {&S75076[81], 44}, {NULL}};
const Segment SGI284989[] = {{&S75077[136], 4}, {NULL}};
const Segment SGI284990[] = {{&S75077[13], 3}, {&S75077[17], 13}, {&S75077[51], 17}, {&S75077[81], 44}, {NULL}};
const Segment SGI284991[] = {{&S75078[136], 4}, {NULL}};
const Segment SGI284992[] = {{&S75078[13], 3}, {&S75078[17], 13}, {&S75078[51], 17}, {&S75078[81], 44}, {NULL}};
const Segment SGI284993[] = {{&S75079[136], 4}, {NULL}};
const Segment SGI284994[] = {{&S75079[13], 3}, {&S75079[17], 13}, {&S75079[51], 17}, {&S75079[81], 44}, {NULL}};
const Segment SGI284995[] = {{&S75080[136], 4}, {NULL}};
const Segment SGI284996[] = {{&S75080[13], 3}, {&S75080[17], 13}, {&S75080[51], 17}, {&S75080[81], 44}, {NULL}};
const Segment SGI284997[] = {{&S75081[136], 4}, {NULL}};
const Segment SGI284998[] = {{&S75081[13], 3}, {&S75081[17], 13}, {&S75081[51], 17}, {&S75081[81], 44}, {NULL}};
const Segment SGI284999[] = {{&S75082[136], 4}, {NULL}};
const Segment SGI285000[] = {{&S75082[13], 3}, {&S75082[17], 13}, {&S75082[51], 17}, {&S75082[81], 44}, {NULL}};
const Segment SGI285001[] = {{&S75083[136], 4}, {NULL}};
const Segment SGI285002[] = {{&S75083[13], 3}, {&S75083[17], 13}, {&S75083[51], 17}, {&S75083[81], 44}, {NULL}};
const Segment SGI285003[] = {{&S75084[136], 4}, {NULL}};
const Segment SGI285004[] = {{&S75084[13], 3}, {&S75084[17], 13}, {&S75084[51], 17}, {&S75084[81], 44}, {NULL}};
const Segment SGI285005[] = {{&S75085[136], 4}, {NULL}};
const Segment SGI285006[] = {{&S75085[13], 3}, {&S75085[17], 13}, {&S75085[51], 17}, {&S75085[81], 44}, {NULL}};
const Segment SGI285007[] = {{&S75086[136], 4}, {NULL}};
const Segment SGI285008[] = {{&S75086[13], 3}, {&S75086[17], 13}, {&S75086[51], 17}, {&S75086[81], 44}, {NULL}};
const Segment SGI285009[] = {{&S75087[136], 4}, {NULL}};
const Segment SGI285010[] = {{&S75087[13], 3}, {&S75087[17], 13}, {&S75087[51], 17}, {&S75087[81], 44}, {NULL}};
const Segment SGI285011[] = {{&S75088[136], 4}, {NULL}};
const Segment SGI285012[] = {{&S75088[13], 3}, {&S75088[17], 13}, {&S75088[51], 17}, {&S75088[81], 44}, {NULL}};
const Segment SGI285013[] = {{&S75089[136], 4}, {NULL}};
const Segment SGI285014[] = {{&S75089[13], 3}, {&S75089[17], 13}, {&S75089[51], 17}, {&S75089[81], 44}, {NULL}};
const Segment SGI285015[] = {{&S75090[136], 4}, {NULL}};
const Segment SGI285016[] = {{&S75090[13], 3}, {&S75090[17], 13}, {&S75090[51], 17}, {&S75090[81], 44}, {NULL}};
const Segment SGI285017[] = {{&S75091[136], 4}, {NULL}};
const Segment SGI285018[] = {{&S75091[13], 3}, {&S75091[17], 13}, {&S75091[51], 17}, {&S75091[81], 44}, {NULL}};
const Segment SGI285019[] = {{&S75092[136], 4}, {NULL}};
const Segment SGI285020[] = {{&S75092[13], 3}, {&S75092[17], 13}, {&S75092[51], 17}, {&S75092[81], 44}, {NULL}};
const Segment SGI285021[] = {{&S75093[136], 4}, {NULL}};
const Segment SGI285022[] = {{&S75093[13], 3}, {&S75093[17], 13}, {&S75093[51], 17}, {&S75093[81], 44}, {NULL}};
const Segment SGI285023[] = {{&S75094[136], 4}, {NULL}};
const Segment SGI285024[] = {{&S75094[13], 3}, {&S75094[17], 13}, {&S75094[51], 17}, {&S75094[81], 44}, {NULL}};
const Segment SGI285025[] = {{&S75095[136], 4}, {NULL}};
const Segment SGI285026[] = {{&S75095[13], 3}, {&S75095[17], 13}, {&S75095[51], 17}, {&S75095[81], 44}, {NULL}};
const Segment SGI285027[] = {{&S75096[92], 4}, {NULL}};
const Segment SGI285028[] = {{&S75096[13], 3}, {&S75096[17], 13}, {&S75096[51], 17}, {NULL}};
const Segment SGI285029[] = {{&S75097[136], 4}, {NULL}};
const Segment SGI285030[] = {{&S75097[13], 3}, {&S75097[17], 13}, {&S75097[51], 17}, {&S75097[81], 44}, {NULL}};
const Segment SGI285031[] = {{&S75098[136], 4}, {NULL}};
const Segment SGI285032[] = {{&S75098[13], 3}, {&S75098[17], 13}, {&S75098[51], 17}, {&S75098[81], 44}, {NULL}};
const Segment SGI285033[] = {{&S75099[136], 4}, {NULL}};
const Segment SGI285034[] = {{&S75099[13], 3}, {&S75099[17], 13}, {&S75099[51], 17}, {&S75099[81], 44}, {NULL}};
const Segment SGI285035[] = {{&S75100[136], 4}, {NULL}};
const Segment SGI285036[] = {{&S75100[13], 3}, {&S75100[17], 13}, {&S75100[51], 17}, {&S75100[81], 44}, {NULL}};
const Segment SGI285037[] = {{&S75101[136], 4}, {NULL}};
const Segment SGI285038[] = {{&S75101[13], 3}, {&S75101[17], 13}, {&S75101[51], 17}, {&S75101[81], 44}, {NULL}};
const Segment SGI285039[] = {{&S75102[136], 4}, {NULL}};
const Segment SGI285040[] = {{&S75102[13], 3}, {&S75102[17], 13}, {&S75102[51], 17}, {&S75102[81], 44}, {NULL}};
const Segment SGI285041[] = {{&S75103[136], 4}, {NULL}};
const Segment SGI285042[] = {{&S75103[13], 3}, {&S75103[17], 13}, {&S75103[51], 17}, {&S75103[81], 44}, {&S75103[182], 8}, {NULL}};
const Segment SGI285043[] = {{&S75104[136], 4}, {NULL}};
const Segment SGI285044[] = {{&S75104[13], 3}, {&S75104[17], 13}, {&S75104[51], 17}, {&S75104[81], 44}, {NULL}};
const Segment SGI285045[] = {{&S75105[136], 4}, {NULL}};
const Segment SGI285046[] = {{&S75105[13], 3}, {&S75105[17], 13}, {&S75105[51], 17}, {&S75105[81], 44}, {NULL}};
const Segment SGI285047[] = {{&S75106[136], 4}, {NULL}};
const Segment SGI285048[] = {{&S75106[13], 3}, {&S75106[17], 13}, {&S75106[51], 17}, {&S75106[81], 44}, {NULL}};
const Segment SGI285049[] = {{&S75107[136], 4}, {NULL}};
const Segment SGI285050[] = {{&S75107[13], 3}, {&S75107[17], 13}, {&S75107[51], 17}, {&S75107[81], 44}, {NULL}};
const Segment SGI285051[] = {{&S75108[136], 4}, {NULL}};
const Segment SGI285052[] = {{&S75108[13], 3}, {&S75108[17], 13}, {&S75108[51], 17}, {&S75108[81], 44}, {NULL}};
const Segment SGI285053[] = {{&S75109[136], 4}, {NULL}};
const Segment SGI285054[] = {{&S75109[13], 3}, {&S75109[17], 13}, {&S75109[51], 17}, {&S75109[81], 44}, {NULL}};
const Segment SGI285055[] = {{&S75110[136], 4}, {NULL}};
const Segment SGI285056[] = {{&S75110[13], 3}, {&S75110[17], 13}, {&S75110[51], 17}, {&S75110[81], 44}, {NULL}};
const Segment SGI285057[] = {{&S75111[136], 4}, {NULL}};
const Segment SGI285058[] = {{&S75111[13], 3}, {&S75111[17], 13}, {&S75111[51], 17}, {&S75111[81], 44}, {NULL}};
const Segment SGI285059[] = {{&S75112[136], 4}, {NULL}};
const Segment SGI285060[] = {{&S75112[13], 3}, {&S75112[17], 13}, {&S75112[51], 17}, {&S75112[81], 44}, {NULL}};
const Segment SGI285061[] = {{&S75113[136], 4}, {NULL}};
const Segment SGI285062[] = {{&S75113[13], 3}, {&S75113[17], 13}, {&S75113[51], 17}, {&S75113[81], 44}, {NULL}};
const Segment SGI285063[] = {{&S75114[136], 4}, {NULL}};
const Segment SGI285064[] = {{&S75114[13], 3}, {&S75114[17], 13}, {&S75114[51], 17}, {&S75114[81], 44}, {NULL}};
const Segment SGI285065[] = {{&S75115[136], 4}, {NULL}};
const Segment SGI285066[] = {{&S75115[13], 3}, {&S75115[17], 13}, {&S75115[51], 17}, {&S75115[81], 44}, {NULL}};
const Segment SGI285067[] = {{&S75116[136], 4}, {NULL}};
const Segment SGI285068[] = {{&S75116[13], 3}, {&S75116[17], 13}, {&S75116[51], 17}, {&S75116[81], 44}, {NULL}};
const Segment SGI285069[] = {{&S75117[136], 4}, {NULL}};
const Segment SGI285070[] = {{&S75117[13], 3}, {&S75117[17], 13}, {&S75117[51], 17}, {&S75117[81], 44}, {NULL}};
const Segment SGI285071[] = {{&S75118[136], 4}, {NULL}};
const Segment SGI285072[] = {{&S75118[13], 3}, {&S75118[17], 13}, {&S75118[51], 17}, {&S75118[81], 44}, {NULL}};
const Segment SGI285073[] = {{&S75232[80], 4}, {NULL}};
const Segment SGI285074[] = {{&S75232[13], 3}, {&S75232[17], 13}, {&S75232[52], 17}, {&S75232[141], 14}, {NULL}};
const Segment SGI285075[] = {{&S75232[127], 12}, {NULL}};
const Segment SGI285076[] = {{&S75233[78], 4}, {NULL}};
const Segment SGI285077[] = {{&S75233[13], 3}, {&S75233[17], 11}, {&S75233[50], 17}, {&S75233[139], 14}, {NULL}};
const Segment SGI285078[] = {{&S75233[125], 12}, {NULL}};
const Segment SGI285079[] = {{&S75287[80], 4}, {NULL}};
const Segment SGI285080[] = {{&S75287[13], 14}, {&S75287[28], 6}, {&S75287[56], 17}, {NULL}};
const Segment SGI285081[] = {{&S75287[75], 9}, {NULL}};
const Segment SGI285082[] = {{&S75288[73], 4}, {NULL}};
const Segment SGI285083[] = {{&S75288[13], 14}, {&S75288[49], 17}, {NULL}};
const Segment SGI285084[] = {{&S75288[68], 9}, {NULL}};
const Segment SGI285085[] = {{&S83919[93], 3}, {NULL}};
const Segment SGI285086[] = {{&S83919[13], 7}, {&S83919[21], 2}, {&S83919[24], 15}, {&S83919[40], 6}, {&S83919[67], 17}, {NULL}};
const Segment SGI285087[] = {{&S75514[82], 4}, {NULL}};
const Segment SGI285088[] = {{&S75514[13], 14}, {&S75514[28], 6}, {&S75514[56], 17}, {NULL}};
const Segment SGI285089[] = {{&S75514[110], 18}, {NULL}};
const Segment SGI285090[] = {{&S75515[75], 4}, {NULL}};
const Segment SGI285091[] = {{&S75515[13], 14}, {&S75515[49], 17}, {NULL}};
const Segment SGI285092[] = {{&S75515[103], 18}, {NULL}};
const Segment SGI285093[] = {{&S75583[75], 4}, {NULL}};
const Segment SGI285094[] = {{&S75583[13], 14}, {&S75583[49], 17}, {NULL}};
const Segment SGI285095[] = {{&S75583[24], 3}, {NULL}};
const Segment SGI285096[] = {{&S75657[97], 4}, {NULL}};
const Segment SGI285097[] = {{&S75657[13], 14}, {&S75657[48], 17}, {NULL}};
const Segment SGI285098[] = {{&S75657[67], 18}, {NULL}};
const Segment SGI285099[] = {{&S75638[93], 4}, {NULL}};
const Segment SGI285100[] = {{&S75638[13], 9}, {&S75638[23], 22}, {&S75638[67], 17}, {NULL}};
const Segment SGI285101[] = {{&S75661[81], 4}, {NULL}};
const Segment SGI285102[] = {{&S75661[13], 14}, {&S75661[28], 6}, {&S75661[55], 17}, {NULL}};
const Segment SGI285103[] = {{&S75661[107], 12}, {NULL}};
const Segment SGI285104[] = {{&S75663[74], 4}, {NULL}};
const Segment SGI285105[] = {{&S75663[13], 14}, {&S75663[48], 17}, {NULL}};
const Segment SGI285106[] = {{&S75663[100], 12}, {NULL}};
const Segment SGI285107[] = {{&S70458[7], 3}, {NULL}};
const Segment SGI285108[] = {{&S70458[11], 5}, {NULL}};
const Segment SGI285109[] = {{&S70459[7], 3}, {NULL}};
const Segment SGI285110[] = {{&S70459[11], 5}, {NULL}};
const Segment SGI285111[] = {{&S70460[7], 3}, {NULL}};
const Segment SGI285112[] = {{&S70460[11], 5}, {NULL}};
const Segment SGI285113[] = {{&S84371[122], 3}, {NULL}};
const Segment SGI285114[] = {{&S84371[13], 9}, {&S84371[23], 2}, {&S84371[26], 22}, {&S84371[49], 6}, {&S84371[77], 17}, {NULL}};
const Segment SGI285115[] = {{&S84371[105], 3}, {NULL}};
const Segment SGI285116[] = {{&S84375[94], 3}, {NULL}};
const Segment SGI285117[] = {{&S84375[13], 7}, {&S84375[21], 2}, {&S84375[24], 15}, {&S84375[40], 6}, {&S84375[68], 17}, {NULL}};
const Segment SGI285118[] = {{&S76034[75], 4}, {NULL}};
const Segment SGI285119[] = {{&S76034[13], 14}, {&S76034[49], 17}, {NULL}};
const Segment SGI285120[] = {{&S76034[101], 18}, {NULL}};
const Segment SGI285121[] = {{&S76035[82], 4}, {NULL}};
const Segment SGI285122[] = {{&S76035[13], 14}, {&S76035[28], 6}, {&S76035[56], 17}, {NULL}};
const Segment SGI285123[] = {{&S76035[108], 18}, {NULL}};
const Segment SGI285124[] = {{&S84488[93], 3}, {NULL}};
const Segment SGI285125[] = {{&S84488[13], 7}, {&S84488[21], 2}, {&S84488[24], 15}, {&S84488[40], 6}, {&S84488[67], 17}, {NULL}};
const Segment SGI285126[] = {{&S84496[94], 3}, {NULL}};
const Segment SGI285127[] = {{&S84496[13], 7}, {&S84496[21], 2}, {&S84496[24], 15}, {&S84496[40], 6}, {&S84496[68], 17}, {NULL}};
const Segment SGI285128[] = {{&S76072[82], 4}, {NULL}};
const Segment SGI285129[] = {{&S76072[13], 3}, {&S76072[17], 17}, {&S76072[56], 17}, {NULL}};
const Segment SGI285130[] = {{&S76138[75], 4}, {NULL}};
const Segment SGI285131[] = {{&S76138[13], 14}, {&S76138[49], 17}, {NULL}};
const Segment SGI285132[] = {{&S76138[103], 18}, {NULL}};
const Segment SGI285133[] = {{&S76139[82], 4}, {NULL}};
const Segment SGI285134[] = {{&S76139[13], 14}, {&S76139[28], 6}, {&S76139[56], 17}, {NULL}};
const Segment SGI285135[] = {{&S76139[110], 18}, {NULL}};
const Segment SGI285136[] = {{&S76209[93], 4}, {NULL}};
const Segment SGI285137[] = {{&S76209[13], 3}, {&S76209[17], 13}, {&S76209[52], 17}, {NULL}};
const Segment SGI285138[] = {{&S76209[71], 12}, {NULL}};
const Segment SGI285139[] = {{&S76210[91], 4}, {NULL}};
const Segment SGI285140[] = {{&S76210[13], 3}, {&S76210[17], 11}, {&S76210[50], 17}, {NULL}};
const Segment SGI285141[] = {{&S76210[69], 12}, {NULL}};
const Segment SGI285142[] = {{&S84868[91], 3}, {NULL}};
const Segment SGI285143[] = {{&S84868[13], 7}, {&S84868[21], 2}, {&S84868[24], 15}, {&S84868[40], 6}, {&S84868[67], 17}, {NULL}};
const Segment SGI285144[] = {{&S84868[86], 8}, {NULL}};
const Segment SGI285145[] = {{&S84952[102], 3}, {NULL}};
const Segment SGI285146[] = {{&S84952[13], 9}, {&S84952[23], 2}, {&S84952[26], 22}, {&S84952[49], 6}, {&S84952[76], 17}, {NULL}};
const Segment SGI285147[] = {{&S84953[102], 3}, {NULL}};
const Segment SGI285148[] = {{&S84953[13], 9}, {&S84953[23], 2}, {&S84953[26], 22}, {&S84953[49], 6}, {&S84953[76], 17}, {NULL}};
const Segment SGI285149[] = {{&S84954[102], 3}, {NULL}};
const Segment SGI285150[] = {{&S84954[13], 9}, {&S84954[23], 2}, {&S84954[26], 22}, {&S84954[49], 6}, {&S84954[76], 17}, {NULL}};
const Segment SGI285151[] = {{&S84955[102], 3}, {NULL}};
const Segment SGI285152[] = {{&S84955[13], 9}, {&S84955[23], 2}, {&S84955[26], 22}, {&S84955[49], 6}, {&S84955[76], 17}, {NULL}};
const Segment SGI285153[] = {{&S84956[102], 3}, {NULL}};
const Segment SGI285154[] = {{&S84956[13], 9}, {&S84956[23], 2}, {&S84956[26], 22}, {&S84956[49], 6}, {&S84956[76], 17}, {NULL}};
const Segment SGI285155[] = {{&S76602[94], 4}, {NULL}};
const Segment SGI285156[] = {{&S76602[13], 7}, {&S76602[21], 2}, {&S76602[24], 15}, {&S76602[40], 6}, {&S76602[68], 17}, {NULL}};
const Segment SGI285157[] = {{&S85048[8], 3}, {NULL}};
const Segment SGI285158[] = {{&S85048[13], 3}, {&S85048[17], 2}, {&S85048[20], 11}, {&S85048[32], 6}, {&S85048[59], 17}, {NULL}};
const Segment SGI285159[] = {{&S85048[78], 12}, {NULL}};
const Segment SGI285160[] = {{&S85049[8], 3}, {NULL}};
const Segment SGI285161[] = {{&S85049[13], 3}, {&S85049[17], 2}, {&S85049[20], 13}, {&S85049[34], 6}, {&S85049[61], 17}, {NULL}};
const Segment SGI285162[] = {{&S85049[80], 12}, {NULL}};
const Segment SGI285163[] = {{&S71260[7], 3}, {NULL}};
const Segment SGI285164[] = {{&S71260[11], 5}, {NULL}};
const Segment SGI285165[] = {{&S76816[91], 4}, {NULL}};
const Segment SGI285166[] = {{&S76816[13], 9}, {&S76816[23], 22}, {&S76816[67], 17}, {NULL}};
const Segment SGI285167[] = {{&S76816[86], 9}, {NULL}};
const Segment SGI285168[] = {{&S76817[91], 4}, {NULL}};
const Segment SGI285169[] = {{&S76817[13], 9}, {&S76817[23], 22}, {&S76817[67], 17}, {NULL}};
const Segment SGI285170[] = {{&S76817[86], 9}, {NULL}};
const Segment SGI285171[] = {{&S76818[91], 4}, {NULL}};
const Segment SGI285172[] = {{&S76818[13], 9}, {&S76818[23], 22}, {&S76818[67], 17}, {NULL}};
const Segment SGI285173[] = {{&S76818[86], 9}, {NULL}};
const Segment SGI285174[] = {{&S76819[91], 4}, {NULL}};
const Segment SGI285175[] = {{&S76819[13], 9}, {&S76819[23], 22}, {&S76819[67], 17}, {NULL}};
const Segment SGI285176[] = {{&S76819[86], 9}, {NULL}};
const Segment SGI285177[] = {{&S76820[91], 4}, {NULL}};
const Segment SGI285178[] = {{&S76820[13], 9}, {&S76820[23], 22}, {&S76820[67], 17}, {NULL}};
const Segment SGI285179[] = {{&S76820[86], 9}, {NULL}};
const Segment SGI285180[] = {{&S76905[75], 4}, {NULL}};
const Segment SGI285181[] = {{&S76905[13], 14}, {&S76905[49], 17}, {NULL}};
const Segment SGI285182[] = {{&S76916[108], 4}, {NULL}};
const Segment SGI285183[] = {{&S76916[13], 3}, {&S76916[17], 13}, {&S76916[31], 22}, {&S76916[54], 6}, {&S76916[82], 17}, {NULL}};
const Segment SGI285184[] = {{&S76917[108], 4}, {NULL}};
const Segment SGI285185[] = {{&S76917[13], 3}, {&S76917[17], 13}, {&S76917[31], 22}, {&S76917[54], 6}, {&S76917[82], 17}, {NULL}};
const Segment SGI285186[] = {{&S77194[104], 4}, {NULL}};
const Segment SGI285187[] = {{&S77194[13], 14}, {&S77194[28], 6}, {&S77194[55], 17}, {NULL}};
const Segment SGI285188[] = {{&S77194[74], 18}, {NULL}};
const Segment SGI285189[] = {{&S77195[97], 4}, {NULL}};
const Segment SGI285190[] = {{&S77195[13], 14}, {&S77195[48], 17}, {NULL}};
const Segment SGI285191[] = {{&S77195[67], 18}, {NULL}};
const Segment SGI285192[] = {{&S77270[78], 4}, {NULL}};
const Segment SGI285193[] = {{&S77270[13], 3}, {&S77270[17], 14}, {&S77270[52], 17}, {NULL}};
const Segment SGI285194[] = {{&S77271[77], 4}, {NULL}};
const Segment SGI285195[] = {{&S77271[13], 3}, {&S77271[17], 13}, {&S77271[51], 17}, {NULL}};
const Segment SGI285196[] = {{&S77330[74], 4}, {NULL}};
const Segment SGI285197[] = {{&S77330[13], 14}, {&S77330[48], 17}, {NULL}};
const Segment SGI285198[] = {{&S77330[100], 18}, {NULL}};
const Segment SGI285199[] = {{&S77419[97], 4}, {NULL}};
const Segment SGI285200[] = {{&S77419[13], 14}, {&S77419[48], 17}, {NULL}};
const Segment SGI285201[] = {{&S77419[67], 18}, {NULL}};
const Segment SGI285202[] = {{&S77464[101], 4}, {NULL}};
const Segment SGI285203[] = {{&S77464[13], 7}, {&S77464[21], 2}, {&S77464[24], 15}, {&S77464[40], 6}, {&S77464[68], 17}, {NULL}};
const Segment SGI285204[] = {{&S77464[87], 18}, {NULL}};
const Segment SGI285205[] = {{&S85925[83], 3}, {NULL}};
const Segment SGI285206[] = {{&S85925[13], 3}, {&S85925[17], 2}, {&S85925[20], 11}, {&S85925[32], 6}, {&S85925[59], 17}, {NULL}};
const Segment SGI285207[] = {{&S85925[78], 8}, {NULL}};
const Segment SGI285208[] = {{&S85926[92], 3}, {NULL}};
const Segment SGI285209[] = {{&S85926[13], 3}, {&S85926[17], 2}, {&S85926[20], 19}, {&S85926[41], 6}, {&S85926[68], 17}, {NULL}};
const Segment SGI285210[] = {{&S85926[87], 8}, {NULL}};
const Segment SGI285211[] = {{&S77575[90], 4}, {NULL}};
const Segment SGI285212[] = {{&S77575[13], 9}, {&S77575[23], 22}, {&S77575[66], 17}, {NULL}};
const Segment SGI285213[] = {{&S77575[85], 9}, {NULL}};
const Segment SGI285214[] = {{&S77576[98], 4}, {NULL}};
const Segment SGI285215[] = {{&S77576[13], 14}, {&S77576[49], 17}, {NULL}};
const Segment SGI285216[] = {{&S77576[68], 18}, {NULL}};
const Segment SGI285217[] = {{&S77629[81], 4}, {NULL}};
const Segment SGI285218[] = {{&S77629[13], 14}, {&S77629[48], 17}, {NULL}};
const Segment SGI285219[] = {{&S77629[67], 18}, {NULL}};
const Segment SGI285220[] = {{&S77682[75], 4}, {NULL}};
const Segment SGI285221[] = {{&S77682[13], 14}, {&S77682[49], 17}, {NULL}};
const Segment SGI285222[] = {{&S77682[103], 18}, {NULL}};
const Segment SGI285223[] = {{&S77736[74], 4}, {NULL}};
const Segment SGI285224[] = {{&S77736[13], 14}, {&S77736[48], 17}, {NULL}};
const Segment SGI285225[] = {{&S77736[100], 18}, {NULL}};
const Segment SGI285226[] = {{&S86232[101], 3}, {NULL}};
const Segment SGI285227[] = {{&S86232[13], 7}, {&S86232[21], 2}, {&S86232[24], 15}, {&S86232[40], 6}, {&S86232[68], 17}, {NULL}};
const Segment SGI285228[] = {{&S86232[87], 17}, {NULL}};
const Segment SGI285229[] = {{&S86326[94], 3}, {NULL}};
const Segment SGI285230[] = {{&S86326[13], 7}, {&S86326[21], 2}, {&S86326[24], 15}, {&S86326[40], 6}, {&S86326[68], 17}, {NULL}};
const Segment SGI285231[] = {{&S86423[84], 3}, {NULL}};
const Segment SGI285232[] = {{&S86423[13], 3}, {&S86423[17], 2}, {&S86423[20], 11}, {&S86423[32], 6}, {&S86423[60], 17}, {NULL}};
const Segment SGI285233[] = {{&S86423[79], 8}, {NULL}};
const Segment SGI285234[] = {{&S86424[86], 3}, {NULL}};
const Segment SGI285235[] = {{&S86424[13], 3}, {&S86424[17], 2}, {&S86424[20], 13}, {&S86424[34], 6}, {&S86424[62], 17}, {NULL}};
const Segment SGI285236[] = {{&S86424[81], 8}, {NULL}};
const Segment SGI285237[] = {{&S77880[78], 4}, {NULL}};
const Segment SGI285238[] = {{&S77880[13], 3}, {&S77880[17], 14}, {&S77880[52], 17}, {NULL}};
const Segment SGI285239[] = {{&S77881[77], 4}, {NULL}};
const Segment SGI285240[] = {{&S77881[13], 3}, {&S77881[17], 13}, {&S77881[51], 17}, {NULL}};
const Segment SGI285241[] = {{&S78143[116], 4}, {NULL}};
const Segment SGI285242[] = {{&S78143[13], 3}, {&S78143[17], 13}, {&S78143[31], 37}, {&S78143[90], 17}, {NULL}};
const Segment SGI285243[] = {{&S78144[118], 4}, {NULL}};
const Segment SGI285244[] = {{&S78144[13], 3}, {&S78144[17], 13}, {&S78144[31], 39}, {&S78144[92], 17}, {NULL}};
const Segment SGI285245[] = {{&S78145[118], 4}, {NULL}};
const Segment SGI285246[] = {{&S78145[13], 3}, {&S78145[17], 13}, {&S78145[31], 39}, {&S78145[92], 17}, {NULL}};
const Segment SGI285247[] = {{&S78167[84], 4}, {NULL}};
const Segment SGI285248[] = {{&S78167[13], 3}, {&S78167[17], 2}, {&S78167[20], 11}, {&S78167[32], 6}, {&S78167[60], 17}, {NULL}};
const Segment SGI285249[] = {{&S78167[79], 9}, {NULL}};
const Segment SGI285250[] = {{&S78168[86], 4}, {NULL}};
const Segment SGI285251[] = {{&S78168[13], 3}, {&S78168[17], 2}, {&S78168[20], 13}, {&S78168[34], 6}, {&S78168[62], 17}, {NULL}};
const Segment SGI285252[] = {{&S78168[81], 9}, {NULL}};
const Segment SGI285253[] = {{&S78326[75], 4}, {NULL}};
const Segment SGI285254[] = {{&S78326[13], 14}, {&S78326[49], 17}, {NULL}};
const Segment SGI285255[] = {{&S78326[111], 4}, {NULL}};
const Segment SGI285256[] = {{&S78327[74], 4}, {NULL}};
const Segment SGI285257[] = {{&S78327[13], 14}, {&S78327[48], 17}, {NULL}};
const Segment SGI285258[] = {{&S78327[109], 4}, {NULL}};
const Segment SGI285259[] = {{&S78536[92], 4}, {NULL}};
const Segment SGI285260[] = {{&S78536[13], 9}, {&S78536[23], 22}, {&S78536[67], 17}, {NULL}};
const Segment SGI285261[] = {{&S78536[87], 9}, {NULL}};
const Segment SGI285262[] = {{&S78537[92], 4}, {NULL}};
const Segment SGI285263[] = {{&S78537[13], 9}, {&S78537[23], 22}, {&S78537[67], 17}, {NULL}};
const Segment SGI285264[] = {{&S78537[87], 9}, {NULL}};
const Segment SGI285265[] = {{&S78538[92], 4}, {NULL}};
const Segment SGI285266[] = {{&S78538[13], 9}, {&S78538[23], 22}, {&S78538[67], 17}, {NULL}};
const Segment SGI285267[] = {{&S78538[87], 9}, {NULL}};
const Segment SGI285268[] = {{&S78539[92], 4}, {NULL}};
const Segment SGI285269[] = {{&S78539[13], 9}, {&S78539[23], 22}, {&S78539[67], 17}, {NULL}};
const Segment SGI285270[] = {{&S78539[87], 9}, {NULL}};
const Segment SGI285271[] = {{&S78593[105], 4}, {NULL}};
const Segment SGI285272[] = {{&S78593[13], 3}, {&S78593[17], 13}, {&S78593[31], 26}, {&S78593[79], 17}, {NULL}};
const Segment SGI285273[] = {{&S78594[105], 4}, {NULL}};
const Segment SGI285274[] = {{&S78594[13], 3}, {&S78594[17], 13}, {&S78594[31], 26}, {&S78594[79], 17}, {NULL}};
const Segment SGI285275[] = {{&S78699[96], 4}, {NULL}};
const Segment SGI285276[] = {{&S78699[13], 3}, {&S78699[17], 13}, {&S78699[31], 10}, {&S78699[42], 6}, {&S78699[70], 17}, {NULL}};
const Segment SGI285277[] = {{&S78700[96], 4}, {NULL}};
const Segment SGI285278[] = {{&S78700[13], 3}, {&S78700[17], 13}, {&S78700[31], 10}, {&S78700[42], 6}, {&S78700[70], 17}, {NULL}};
const Segment SGI285279[] = {{&S78938[74], 4}, {NULL}};
const Segment SGI285280[] = {{&S78938[13], 14}, {&S78938[49], 17}, {NULL}};
const Segment SGI285281[] = {{&S78938[69], 9}, {NULL}};
const Segment SGI285282[] = {{&S78939[81], 4}, {NULL}};
const Segment SGI285283[] = {{&S78939[13], 14}, {&S78939[28], 6}, {&S78939[56], 17}, {NULL}};
const Segment SGI285284[] = {{&S78939[76], 9}, {NULL}};
const Segment SGI285285[] = {{&S79127[48], 4}, {NULL}};
const Segment SGI285286[] = {{&S79127[13], 14}, {&S79127[28], 6}, {&S79127[35], 24}, {&S79127[80], 17}, {NULL}};
const Segment SGI285287[] = {{&S79127[36], 16}, {NULL}};
const Segment SGI285288[] = {{&S79129[41], 4}, {NULL}};
const Segment SGI285289[] = {{&S79129[13], 14}, {&S79129[28], 24}, {&S79129[73], 17}, {NULL}};
const Segment SGI285290[] = {{&S79129[29], 16}, {NULL}};
const Segment SGI285291[] = {{&S79132[41], 4}, {NULL}};
const Segment SGI285292[] = {{&S79132[13], 14}, {&S79132[28], 24}, {&S79132[73], 17}, {NULL}};
const Segment SGI285293[] = {{&S79132[29], 16}, {NULL}};
const Segment SGI285294[] = {{&S79135[75], 4}, {NULL}};
const Segment SGI285295[] = {{&S79135[13], 3}, {&S79135[17], 13}, {&S79135[51], 17}, {NULL}};
const Segment SGI285296[] = {{&S79135[70], 9}, {NULL}};
const Segment SGI285297[] = {{&S79136[73], 4}, {NULL}};
const Segment SGI285298[] = {{&S79136[13], 3}, {&S79136[17], 11}, {&S79136[49], 17}, {NULL}};
const Segment SGI285299[] = {{&S79136[68], 9}, {NULL}};
const Segment SGI285300[] = {{&S79037[75], 4}, {NULL}};
const Segment SGI285301[] = {{&S79037[13], 14}, {&S79037[49], 17}, {NULL}};
const Segment SGI285302[] = {{&S79037[102], 12}, {NULL}};
const Segment SGI285303[] = {{&S79269[75], 4}, {NULL}};
const Segment SGI285304[] = {{&S79269[13], 14}, {&S79269[49], 17}, {NULL}};
const Segment SGI285305[] = {{&S79269[103], 18}, {NULL}};
const Segment SGI285306[] = {{&S79271[72], 4}, {NULL}};
const Segment SGI285307[] = {{&S79271[13], 14}, {&S79271[48], 17}, {NULL}};
const Segment SGI285308[] = {{&S79271[67], 9}, {NULL}};
const Segment SGI285309[] = {{&S79272[79], 4}, {NULL}};
const Segment SGI285310[] = {{&S79272[13], 14}, {&S79272[28], 6}, {&S79272[55], 17}, {NULL}};
const Segment SGI285311[] = {{&S79272[74], 9}, {NULL}};
const Segment SGI285312[] = {{&S79106[91], 4}, {NULL}};
const Segment SGI285313[] = {{&S79106[13], 3}, {&S79106[17], 11}, {&S79106[50], 17}, {NULL}};
const Segment SGI285314[] = {{&S79106[69], 12}, {NULL}};
const Segment SGI285315[] = {{&S79107[93], 4}, {NULL}};
const Segment SGI285316[] = {{&S79107[13], 3}, {&S79107[17], 13}, {&S79107[52], 17}, {NULL}};
const Segment SGI285317[] = {{&S79107[71], 12}, {NULL}};
const Segment SGI285318[] = {{&S79158[82], 4}, {NULL}};
const Segment SGI285319[] = {{&S79158[13], 14}, {&S79158[28], 6}, {&S79158[56], 17}, {NULL}};
const Segment SGI285320[] = {{&S79158[110], 12}, {NULL}};
const Segment SGI285321[] = {{&S79159[75], 4}, {NULL}};
const Segment SGI285322[] = {{&S79159[13], 14}, {&S79159[49], 17}, {NULL}};
const Segment SGI285323[] = {{&S79159[103], 12}, {NULL}};
const Segment SGI285324[] = {{&S79415[73], 4}, {NULL}};
const Segment SGI285325[] = {{&S79415[13], 3}, {&S79415[17], 11}, {&S79415[49], 17}, {NULL}};
const Segment SGI285326[] = {{&S79415[68], 9}, {NULL}};
const Segment SGI285327[] = {{&S79416[75], 4}, {NULL}};
const Segment SGI285328[] = {{&S79416[13], 3}, {&S79416[17], 13}, {&S79416[51], 17}, {NULL}};
const Segment SGI285329[] = {{&S79416[70], 9}, {NULL}};
const Segment SGI285330[] = {{&S99424[38], 8}, {NULL}};
const Segment SGI285331[] = {{&S99424[100], 5}, {NULL}};
const Segment SGI285332[] = {{&S79477[98], 4}, {NULL}};
const Segment SGI285333[] = {{&S79477[13], 14}, {&S79477[49], 17}, {NULL}};
const Segment SGI285334[] = {{&S79477[68], 18}, {NULL}};
const Segment SGI285335[] = {{&S79480[75], 4}, {NULL}};
const Segment SGI285336[] = {{&S79480[13], 15}, {&S79480[49], 17}, {NULL}};
const Segment SGI285337[] = {{&S79510[74], 4}, {NULL}};
const Segment SGI285338[] = {{&S79510[13], 14}, {&S79510[48], 17}, {NULL}};
const Segment SGI285339[] = {{&S79510[108], 3}, {NULL}};
const Segment SGI285340[] = {{&S87923[94], 3}, {NULL}};
const Segment SGI285341[] = {{&S87923[13], 7}, {&S87923[21], 2}, {&S87923[24], 15}, {&S87923[40], 6}, {&S87923[68], 17}, {NULL}};
const Segment SGI285342[] = {{&S79819[74], 4}, {NULL}};
const Segment SGI285343[] = {{&S79819[13], 14}, {&S79819[48], 17}, {NULL}};
const Segment SGI285344[] = {{&S79820[97], 4}, {NULL}};
const Segment SGI285345[] = {{&S79820[13], 14}, {&S79820[48], 17}, {NULL}};
const Segment SGI285346[] = {{&S79820[67], 18}, {NULL}};
const Segment SGI285347[] = {{&S79821[104], 4}, {NULL}};
const Segment SGI285348[] = {{&S79821[13], 14}, {&S79821[28], 6}, {&S79821[55], 17}, {NULL}};
const Segment SGI285349[] = {{&S79821[74], 18}, {NULL}};
const Segment SGI285350[] = {{&S79988[98], 4}, {NULL}};
const Segment SGI285351[] = {{&S79988[13], 14}, {&S79988[49], 17}, {NULL}};
const Segment SGI285352[] = {{&S79988[68], 18}, {NULL}};
const Segment SGI285353[] = {{&S88415[122], 3}, {NULL}};
const Segment SGI285354[] = {{&S88415[13], 9}, {&S88415[23], 2}, {&S88415[26], 22}, {&S88415[49], 6}, {&S88415[76], 17}, {NULL}};
const Segment SGI285355[] = {{&S88415[104], 3}, {NULL}};
const Segment SGI285356[] = {{&S88416[123], 3}, {NULL}};
const Segment SGI285357[] = {{&S88416[13], 9}, {&S88416[23], 2}, {&S88416[26], 22}, {&S88416[49], 6}, {&S88416[77], 17}, {NULL}};
const Segment SGI285358[] = {{&S88416[105], 3}, {NULL}};
const Segment SGI285359[] = {{&S88417[122], 3}, {NULL}};
const Segment SGI285360[] = {{&S88417[13], 9}, {&S88417[23], 2}, {&S88417[26], 22}, {&S88417[49], 6}, {&S88417[76], 17}, {NULL}};
const Segment SGI285361[] = {{&S88417[104], 3}, {NULL}};
const Segment SGI285362[] = {{&S79848[82], 4}, {NULL}};
const Segment SGI285363[] = {{&S79848[13], 14}, {&S79848[28], 6}, {&S79848[56], 17}, {NULL}};
const Segment SGI285364[] = {{&S79848[109], 12}, {NULL}};
const Segment SGI285365[] = {{&S79849[75], 4}, {NULL}};
const Segment SGI285366[] = {{&S79849[13], 14}, {&S79849[49], 17}, {NULL}};
const Segment SGI285367[] = {{&S79849[102], 12}, {NULL}};
const Segment SGI285368[] = {{&S80300[96], 4}, {NULL}};
const Segment SGI285369[] = {{&S80300[13], 3}, {&S80300[17], 13}, {&S80300[31], 17}, {&S80300[70], 17}, {NULL}};
const Segment SGI285370[] = {{&S79969[93], 4}, {NULL}};
const Segment SGI285371[] = {{&S79969[13], 9}, {&S79969[23], 22}, {&S79969[67], 17}, {NULL}};
const Segment SGI285372[] = {{&S80105[93], 4}, {NULL}};
const Segment SGI285373[] = {{&S80105[13], 9}, {&S80105[23], 22}, {&S80105[67], 17}, {&S80105[139], 12}, {NULL}};
const Segment SGI285374[] = {{&S80105[120], 8}, {NULL}};
const Segment SGI285375[] = {{&S80106[93], 4}, {NULL}};
const Segment SGI285376[] = {{&S80106[13], 9}, {&S80106[23], 22}, {&S80106[67], 17}, {NULL}};
const Segment SGI285377[] = {{&S80106[120], 8}, {NULL}};
const Segment SGI285378[] = {{&S80660[97], 4}, {NULL}};
const Segment SGI285379[] = {{&S80660[13], 14}, {&S80660[48], 17}, {NULL}};
const Segment SGI285380[] = {{&S80660[67], 18}, {NULL}};
const Segment SGI285381[] = {{&S80693[118], 4}, {NULL}};
const Segment SGI285382[] = {{&S80693[13], 3}, {&S80693[17], 13}, {&S80693[31], 39}, {&S80693[92], 17}, {NULL}};
const Segment SGI285383[] = {{&S80736[91], 4}, {NULL}};
const Segment SGI285384[] = {{&S80736[13], 9}, {&S80736[23], 22}, {&S80736[67], 17}, {NULL}};
const Segment SGI285385[] = {{&S80736[86], 9}, {NULL}};
const Segment SGI285386[] = {{&S80737[91], 4}, {NULL}};
const Segment SGI285387[] = {{&S80737[13], 9}, {&S80737[23], 22}, {&S80737[67], 17}, {NULL}};
const Segment SGI285388[] = {{&S80737[86], 9}, {NULL}};
const Segment SGI285389[] = {{&S80777[74], 4}, {NULL}};
const Segment SGI285390[] = {{&S80777[13], 14}, {&S80777[48], 17}, {NULL}};
const Segment SGI285391[] = {{&S80777[108], 3}, {NULL}};
const Segment SGI285392[] = {{&S89111[100], 3}, {NULL}};
const Segment SGI285393[] = {{&S89111[13], 7}, {&S89111[21], 2}, {&S89111[24], 15}, {&S89111[40], 6}, {&S89111[67], 17}, {NULL}};
const Segment SGI285394[] = {{&S89111[86], 17}, {NULL}};
const Segment SGI285395[] = {{&S80849[108], 4}, {NULL}};
const Segment SGI285396[] = {{&S80849[13], 4}, {&S80849[18], 2}, {&S80849[21], 32}, {&S80849[54], 6}, {&S80849[82], 16}, {NULL}};
const Segment SGI285397[] = {{&S80849[134], 10}, {NULL}};
const Segment SGI285398[] = {{&S81008[105], 4}, {NULL}};
const Segment SGI285399[] = {{&S81008[13], 3}, {&S81008[17], 13}, {&S81008[31], 26}, {&S81008[79], 17}, {NULL}};
const Segment SGI285400[] = {{&S80768[79], 4}, {NULL}};
const Segment SGI285401[] = {{&S80768[13], 3}, {&S80768[17], 14}, {&S80768[53], 17}, {NULL}};
const Segment SGI285402[] = {{&S80769[78], 4}, {NULL}};
const Segment SGI285403[] = {{&S80769[13], 3}, {&S80769[17], 13}, {&S80769[52], 17}, {NULL}};
const Segment SGI285404[] = {{&S89404[8], 3}, {NULL}};
const Segment SGI285405[] = {{&S89404[13], 10}, {&S89404[24], 2}, {&S89404[27], 18}, {&S89404[46], 6}, {&S89404[73], 17}, {NULL}};
const Segment SGI285406[] = {{&S89406[8], 3}, {NULL}};
const Segment SGI285407[] = {{&S89406[13], 9}, {&S89406[23], 2}, {&S89406[26], 22}, {&S89406[49], 6}, {&S89406[76], 17}, {NULL}};
const Segment SGI285408[] = {{&S89406[95], 8}, {NULL}};
const Segment SGI285409[] = {{&S89407[8], 3}, {NULL}};
const Segment SGI285410[] = {{&S89407[13], 9}, {&S89407[23], 2}, {&S89407[26], 22}, {&S89407[49], 6}, {&S89407[76], 17}, {NULL}};
const Segment SGI285411[] = {{&S89407[95], 8}, {NULL}};
const Segment SGI285412[] = {{&S89408[8], 3}, {NULL}};
const Segment SGI285413[] = {{&S89408[13], 9}, {&S89408[23], 2}, {&S89408[26], 22}, {&S89408[49], 6}, {&S89408[76], 17}, {NULL}};
const Segment SGI285414[] = {{&S89408[95], 8}, {NULL}};
const Segment SGI285415[] = {{&S80827[76], 4}, {NULL}};
const Segment SGI285416[] = {{&S80827[13], 15}, {&S80827[50], 17}, {NULL}};
const Segment SGI285417[] = {{&S81055[85], 4}, {NULL}};
const Segment SGI285418[] = {{&S81055[13], 3}, {&S81055[17], 21}, {&S81055[59], 17}, {NULL}};
const Segment SGI285419[] = {{&S81056[85], 4}, {NULL}};
const Segment SGI285420[] = {{&S81056[13], 3}, {&S81056[17], 21}, {&S81056[59], 17}, {NULL}};
const Segment SGI285421[] = {{&S81057[86], 4}, {NULL}};
const Segment SGI285422[] = {{&S81057[13], 3}, {&S81057[17], 22}, {&S81057[60], 17}, {NULL}};
const Segment SGI285423[] = {{&S81058[86], 4}, {NULL}};
const Segment SGI285424[] = {{&S81058[13], 3}, {&S81058[17], 22}, {&S81058[60], 17}, {NULL}};
const Segment SGI285425[] = {{&S81059[86], 4}, {NULL}};
const Segment SGI285426[] = {{&S81059[13], 3}, {&S81059[17], 22}, {&S81059[60], 17}, {NULL}};
const Segment SGI285427[] = {{&S81060[83], 4}, {NULL}};
const Segment SGI285428[] = {{&S81060[13], 3}, {&S81060[17], 19}, {&S81060[57], 17}, {NULL}};
const Segment SGI285429[] = {{&S81061[83], 4}, {NULL}};
const Segment SGI285430[] = {{&S81061[13], 3}, {&S81061[17], 19}, {&S81061[57], 17}, {NULL}};
const Segment SGI285431[] = {{&S81062[86], 4}, {NULL}};
const Segment SGI285432[] = {{&S81062[13], 3}, {&S81062[17], 22}, {&S81062[60], 17}, {NULL}};
const Segment SGI285433[] = {{&S75447[7], 3}, {NULL}};
const Segment SGI285434[] = {{&S75447[11], 4}, {NULL}};
const Segment SGI285435[] = {{&S75448[7], 3}, {NULL}};
const Segment SGI285436[] = {{&S75448[11], 4}, {NULL}};
const Segment SGI285437[] = {{&S81182[97], 4}, {NULL}};
const Segment SGI285438[] = {{&S81182[13], 14}, {&S81182[48], 17}, {NULL}};
const Segment SGI285439[] = {{&S81182[67], 18}, {NULL}};
const Segment SGI285440[] = {{&S81183[104], 4}, {NULL}};
const Segment SGI285441[] = {{&S81183[13], 14}, {&S81183[28], 6}, {&S81183[55], 17}, {NULL}};
const Segment SGI285442[] = {{&S81183[74], 18}, {NULL}};
const Segment SGI285443[] = {{&S81189[90], 4}, {NULL}};
const Segment SGI285444[] = {{&S81189[13], 9}, {&S81189[23], 22}, {&S81189[66], 17}, {NULL}};
const Segment SGI285445[] = {{&S81189[85], 9}, {NULL}};
const Segment SGI285446[] = {{&S81190[90], 4}, {NULL}};
const Segment SGI285447[] = {{&S81190[13], 9}, {&S81190[23], 22}, {&S81190[66], 17}, {NULL}};
const Segment SGI285448[] = {{&S81190[85], 9}, {NULL}};
const Segment SGI285449[] = {{&S81191[90], 4}, {NULL}};
const Segment SGI285450[] = {{&S81191[13], 9}, {&S81191[23], 22}, {&S81191[66], 17}, {NULL}};
const Segment SGI285451[] = {{&S81191[85], 9}, {NULL}};
const Segment SGI285452[] = {{&S81192[90], 4}, {NULL}};
const Segment SGI285453[] = {{&S81192[13], 9}, {&S81192[23], 22}, {&S81192[66], 17}, {NULL}};
const Segment SGI285454[] = {{&S81192[85], 9}, {NULL}};
const Segment SGI285455[] = {{&S81007[92], 4}, {NULL}};
const Segment SGI285456[] = {{&S81007[13], 9}, {&S81007[23], 22}, {&S81007[66], 17}, {NULL}};
const Segment SGI285457[] = {{&S81007[104], 9}, {NULL}};
const Segment SGI285458[] = {{&S89724[100], 3}, {NULL}};
const Segment SGI285459[] = {{&S89724[13], 9}, {&S89724[23], 2}, {&S89724[26], 22}, {&S89724[49], 6}, {&S89724[76], 17}, {NULL}};
const Segment SGI285460[] = {{&S89724[95], 8}, {NULL}};
const Segment SGI285461[] = {{&S81255[79], 4}, {NULL}};
const Segment SGI285462[] = {{&S81255[13], 3}, {&S81255[17], 14}, {&S81255[53], 17}, {NULL}};
const Segment SGI285463[] = {{&S81256[78], 4}, {NULL}};
const Segment SGI285464[] = {{&S81256[13], 3}, {&S81256[17], 13}, {&S81256[52], 17}, {NULL}};
const Segment SGI285465[] = {{&S89862[93], 3}, {NULL}};
const Segment SGI285466[] = {{&S89862[13], 7}, {&S89862[21], 2}, {&S89862[24], 15}, {&S89862[40], 6}, {&S89862[67], 17}, {NULL}};
const Segment SGI285467[] = {{&S81316[76], 4}, {NULL}};
const Segment SGI285468[] = {{&S81316[13], 15}, {&S81316[50], 17}, {NULL}};
const Segment SGI285469[] = {{&S100972[38], 5}, {NULL}};
const Segment SGI285470[] = {{&S100972[100], 5}, {NULL}};
const Segment SGI285471[] = {{&S81385[92], 4}, {NULL}};
const Segment SGI285472[] = {{&S81385[13], 3}, {&S81385[17], 13}, {&S81385[51], 17}, {NULL}};
const Segment SGI285473[] = {{&S81385[70], 12}, {NULL}};
const Segment SGI285474[] = {{&S81386[90], 4}, {NULL}};
const Segment SGI285475[] = {{&S81386[13], 3}, {&S81386[17], 11}, {&S81386[49], 17}, {NULL}};
const Segment SGI285476[] = {{&S81386[68], 12}, {NULL}};
const Segment SGI285477[] = {{&S75964[7], 3}, {NULL}};
const Segment SGI285478[] = {{&S75964[11], 4}, {NULL}};
const Segment SGI285479[] = {{&S81449[90], 4}, {NULL}};
const Segment SGI285480[] = {{&S81449[13], 9}, {&S81449[23], 22}, {&S81449[66], 17}, {NULL}};
const Segment SGI285481[] = {{&S81449[85], 9}, {NULL}};
const Segment SGI285482[] = {{&S81590[74], 4}, {NULL}};
const Segment SGI285483[] = {{&S81590[13], 14}, {&S81590[48], 17}, {NULL}};
const Segment SGI285484[] = {{&S81590[109], 4}, {NULL}};
const Segment SGI285485[] = {{&S81597[75], 4}, {NULL}};
const Segment SGI285486[] = {{&S81597[13], 14}, {&S81597[49], 17}, {NULL}};
const Segment SGI285487[] = {{&S81597[111], 4}, {NULL}};
const Segment SGI285488[] = {{&S81736[74], 4}, {NULL}};
const Segment SGI285489[] = {{&S81736[13], 14}, {&S81736[48], 17}, {NULL}};
const Segment SGI285490[] = {{&S81736[109], 4}, {NULL}};
const Segment SGI285491[] = {{&S81834[93], 4}, {NULL}};
const Segment SGI285492[] = {{&S81834[13], 9}, {&S81834[23], 22}, {&S81834[67], 17}, {NULL}};
const Segment SGI285493[] = {{&S81884[82], 4}, {NULL}};
const Segment SGI285494[] = {{&S81884[13], 14}, {&S81884[28], 6}, {&S81884[56], 17}, {NULL}};
const Segment SGI285495[] = {{&S81884[109], 18}, {NULL}};
const Segment SGI285496[] = {{&S81885[75], 4}, {NULL}};
const Segment SGI285497[] = {{&S81885[13], 14}, {&S81885[49], 17}, {NULL}};
const Segment SGI285498[] = {{&S81885[102], 18}, {NULL}};
const Segment SGI285499[] = {{&S81865[48], 4}, {NULL}};
const Segment SGI285500[] = {{&S81865[13], 14}, {&S81865[28], 6}, {&S81865[35], 26}, {&S81865[83], 17}, {NULL}};
const Segment SGI285501[] = {{&S81865[36], 16}, {NULL}};
const Segment SGI285502[] = {{&S81866[41], 4}, {NULL}};
const Segment SGI285503[] = {{&S81866[13], 14}, {&S81866[28], 26}, {&S81866[76], 17}, {NULL}};
const Segment SGI285504[] = {{&S81866[29], 16}, {NULL}};
const Segment SGI285505[] = {{&S81868[41], 4}, {NULL}};
const Segment SGI285506[] = {{&S81868[13], 14}, {&S81868[28], 26}, {&S81868[76], 17}, {NULL}};
const Segment SGI285507[] = {{&S81868[29], 16}, {NULL}};
const Segment SGI285508[] = {{&S81869[48], 4}, {NULL}};
const Segment SGI285509[] = {{&S81869[13], 14}, {&S81869[28], 6}, {&S81869[35], 26}, {&S81869[83], 17}, {NULL}};
const Segment SGI285510[] = {{&S81869[36], 16}, {NULL}};
const Segment SGI285511[] = {{&S81873[41], 4}, {NULL}};
const Segment SGI285512[] = {{&S81873[13], 14}, {&S81873[28], 26}, {&S81873[76], 17}, {NULL}};
const Segment SGI285513[] = {{&S81873[29], 16}, {NULL}};
const Segment SGI285514[] = {{&S81876[41], 4}, {NULL}};
const Segment SGI285515[] = {{&S81876[13], 14}, {&S81876[28], 26}, {&S81876[76], 17}, {NULL}};
const Segment SGI285516[] = {{&S81876[29], 16}, {NULL}};
const Segment SGI285517[] = {{&S81877[48], 4}, {NULL}};
const Segment SGI285518[] = {{&S81877[13], 14}, {&S81877[28], 6}, {&S81877[35], 26}, {&S81877[83], 17}, {NULL}};
const Segment SGI285519[] = {{&S81877[36], 16}, {NULL}};
const Segment SGI285520[] = {{&S81907[104], 4}, {NULL}};
const Segment SGI285521[] = {{&S81907[13], 4}, {&S81907[18], 2}, {&S81907[21], 27}, {&S81907[49], 6}, {&S81907[77], 17}, {NULL}};
const Segment SGI285522[] = {{&S81907[130], 10}, {NULL}};
const Segment SGI285523[] = {{&S81908[104], 4}, {NULL}};
const Segment SGI285524[] = {{&S81908[13], 4}, {&S81908[18], 2}, {&S81908[21], 27}, {&S81908[49], 6}, {&S81908[77], 17}, {NULL}};
const Segment SGI285525[] = {{&S81908[130], 10}, {NULL}};
const Segment SGI285526[] = {{&S81909[104], 4}, {NULL}};
const Segment SGI285527[] = {{&S81909[13], 4}, {&S81909[18], 2}, {&S81909[21], 27}, {&S81909[49], 6}, {&S81909[77], 17}, {NULL}};
const Segment SGI285528[] = {{&S81909[130], 10}, {NULL}};
const Segment SGI285529[] = {{&S81965[74], 4}, {NULL}};
const Segment SGI285530[] = {{&S81965[13], 14}, {&S81965[48], 17}, {NULL}};
const Segment SGI285531[] = {{&S81965[100], 12}, {NULL}};
const Segment SGI285532[] = {{&S81966[81], 4}, {NULL}};
const Segment SGI285533[] = {{&S81966[13], 14}, {&S81966[28], 6}, {&S81966[55], 17}, {NULL}};
const Segment SGI285534[] = {{&S81966[107], 12}, {NULL}};
const Segment SGI285535[] = {{&S81951[90], 4}, {NULL}};
const Segment SGI285536[] = {{&S81951[13], 9}, {&S81951[23], 22}, {&S81951[66], 17}, {NULL}};
const Segment SGI285537[] = {{&S81951[85], 9}, {NULL}};
const Segment SGI285538[] = {{&S81952[90], 4}, {NULL}};
const Segment SGI285539[] = {{&S81952[13], 9}, {&S81952[23], 22}, {&S81952[66], 17}, {NULL}};
const Segment SGI285540[] = {{&S81952[85], 9}, {NULL}};
const Segment SGI285541[] = {{&S81953[90], 4}, {NULL}};
const Segment SGI285542[] = {{&S81953[13], 9}, {&S81953[23], 22}, {&S81953[66], 17}, {NULL}};
const Segment SGI285543[] = {{&S81953[85], 9}, {NULL}};
const Segment SGI285544[] = {{&S81954[90], 4}, {NULL}};
const Segment SGI285545[] = {{&S81954[13], 9}, {&S81954[23], 22}, {&S81954[66], 17}, {NULL}};
const Segment SGI285546[] = {{&S81954[85], 9}, {NULL}};
const Segment SGI285547[] = {{&S82033[81], 4}, {NULL}};
const Segment SGI285548[] = {{&S82033[13], 14}, {&S82033[48], 17}, {NULL}};
const Segment SGI285549[] = {{&S82033[67], 18}, {NULL}};
const Segment SGI285550[] = {{&S82034[88], 4}, {NULL}};
const Segment SGI285551[] = {{&S82034[13], 14}, {&S82034[28], 6}, {&S82034[55], 17}, {NULL}};
const Segment SGI285552[] = {{&S82034[74], 18}, {NULL}};
const Segment SGI285553[] = {{&S90782[93], 3}, {NULL}};
const Segment SGI285554[] = {{&S90782[13], 7}, {&S90782[21], 2}, {&S90782[24], 15}, {&S90782[40], 6}, {&S90782[67], 17}, {NULL}};
const Segment SGI285555[] = {{&S90783[36], 3}, {NULL}};
const Segment SGI285556[] = {{&S90783[13], 7}, {&S90783[21], 2}, {&S90783[24], 15}, {&S90783[40], 6}, {&S90783[67], 17}, {NULL}};
const Segment SGI285557[] = {{&S82269[94], 4}, {NULL}};
const Segment SGI285558[] = {{&S82269[13], 7}, {&S82269[21], 2}, {&S82269[24], 15}, {&S82269[40], 6}, {&S82269[68], 17}, {NULL}};
const Segment SGI285559[] = {{&S82270[76], 4}, {NULL}};
const Segment SGI285560[] = {{&S82270[13], 15}, {&S82270[50], 17}, {NULL}};
const Segment SGI285561[] = {{&S82368[74], 4}, {NULL}};
const Segment SGI285562[] = {{&S82368[13], 14}, {&S82368[48], 17}, {NULL}};
const Segment SGI285563[] = {{&S82448[94], 4}, {NULL}};
const Segment SGI285564[] = {{&S82448[13], 14}, {&S82448[48], 17}, {NULL}};
const Segment SGI285565[] = {{&S82448[76], 3}, {NULL}};
const Segment SGI285566[] = {{&S82449[94], 4}, {NULL}};
const Segment SGI285567[] = {{&S82449[13], 14}, {&S82449[48], 17}, {NULL}};
const Segment SGI285568[] = {{&S82449[76], 3}, {NULL}};
const Segment SGI285569[] = {{&S82450[101], 4}, {NULL}};
const Segment SGI285570[] = {{&S82450[13], 14}, {&S82450[28], 6}, {&S82450[55], 17}, {NULL}};
const Segment SGI285571[] = {{&S82450[83], 3}, {NULL}};
const Segment SGI285572[] = {{&S82451[101], 4}, {NULL}};
const Segment SGI285573[] = {{&S82451[13], 14}, {&S82451[28], 6}, {&S82451[55], 17}, {NULL}};
const Segment SGI285574[] = {{&S82451[83], 3}, {NULL}};
const Segment SGI285575[] = {{&S82452[95], 4}, {NULL}};
const Segment SGI285576[] = {{&S82452[13], 14}, {&S82452[49], 17}, {NULL}};
const Segment SGI285577[] = {{&S82452[77], 3}, {NULL}};
const Segment SGI285578[] = {{&S82454[114], 4}, {NULL}};
const Segment SGI285579[] = {{&S82454[13], 7}, {&S82454[21], 2}, {&S82454[24], 15}, {&S82454[40], 6}, {&S82454[68], 17}, {NULL}};
const Segment SGI285580[] = {{&S82454[96], 3}, {NULL}};
const Segment SGI285581[] = {{&S82455[95], 4}, {NULL}};
const Segment SGI285582[] = {{&S82455[13], 14}, {&S82455[49], 17}, {NULL}};
const Segment SGI285583[] = {{&S82455[77], 3}, {NULL}};
const Segment SGI285584[] = {{&S82555[75], 4}, {NULL}};
const Segment SGI285585[] = {{&S82555[13], 14}, {&S82555[49], 17}, {NULL}};
const Segment SGI285586[] = {{&S82555[110], 3}, {NULL}};
const Segment SGI285587[] = {{&S82557[74], 4}, {NULL}};
const Segment SGI285588[] = {{&S82557[13], 14}, {&S82557[48], 17}, {NULL}};
const Segment SGI285589[] = {{&S82767[93], 4}, {NULL}};
const Segment SGI285590[] = {{&S82767[13], 3}, {&S82767[17], 13}, {&S82767[52], 17}, {NULL}};
const Segment SGI285591[] = {{&S82767[71], 12}, {NULL}};
const Segment SGI285592[] = {{&S76899[7], 3}, {NULL}};
const Segment SGI285593[] = {{&S76899[11], 4}, {NULL}};
const Segment SGI285594[] = {{&S76900[7], 3}, {NULL}};
const Segment SGI285595[] = {{&S76900[11], 4}, {NULL}};
const Segment SGI285596[] = {{&S82738[90], 4}, {NULL}};
const Segment SGI285597[] = {{&S82738[13], 9}, {&S82738[23], 22}, {&S82738[66], 17}, {NULL}};
const Segment SGI285598[] = {{&S82738[85], 9}, {NULL}};
const Segment SGI285599[] = {{&S82739[90], 4}, {NULL}};
const Segment SGI285600[] = {{&S82739[13], 9}, {&S82739[23], 22}, {&S82739[66], 17}, {NULL}};
const Segment SGI285601[] = {{&S82739[85], 9}, {NULL}};
const Segment SGI285602[] = {{&S82740[90], 4}, {NULL}};
const Segment SGI285603[] = {{&S82740[13], 9}, {&S82740[23], 22}, {&S82740[66], 17}, {NULL}};
const Segment SGI285604[] = {{&S82740[85], 9}, {NULL}};
const Segment SGI285605[] = {{&S82741[90], 4}, {NULL}};
const Segment SGI285606[] = {{&S82741[13], 9}, {&S82741[23], 22}, {&S82741[66], 17}, {NULL}};
const Segment SGI285607[] = {{&S82741[85], 9}, {NULL}};
const Segment SGI285608[] = {{&S91288[95], 3}, {NULL}};
const Segment SGI285609[] = {{&S91288[13], 7}, {&S91288[21], 2}, {&S91288[24], 16}, {&S91288[41], 6}, {&S91288[69], 17}, {NULL}};
const Segment SGI285610[] = {{&S82768[91], 4}, {NULL}};
const Segment SGI285611[] = {{&S82768[13], 3}, {&S82768[17], 11}, {&S82768[50], 17}, {NULL}};
const Segment SGI285612[] = {{&S82768[69], 12}, {NULL}};
const Segment SGI285613[] = {{&S82893[87], 4}, {NULL}};
const Segment SGI285614[] = {{&S82893[13], 14}, {&S82893[49], 17}, {NULL}};
const Segment SGI285615[] = {{&S82893[68], 11}, {NULL}};
const Segment SGI285616[] = {{&S82894[86], 4}, {NULL}};
const Segment SGI285617[] = {{&S82894[13], 14}, {&S82894[48], 17}, {NULL}};
const Segment SGI285618[] = {{&S82894[67], 11}, {NULL}};
const Segment SGI285619[] = {{&S82959[41], 4}, {NULL}};
const Segment SGI285620[] = {{&S82959[13], 14}, {&S82959[28], 25}, {&S82959[74], 17}, {NULL}};
const Segment SGI285621[] = {{&S82959[29], 16}, {NULL}};
const Segment SGI285622[] = {{&S82961[41], 4}, {NULL}};
const Segment SGI285623[] = {{&S82961[13], 14}, {&S82961[28], 25}, {&S82961[74], 17}, {NULL}};
const Segment SGI285624[] = {{&S82961[29], 16}, {NULL}};
const Segment SGI285625[] = {{&S91489[36], 3}, {NULL}};
const Segment SGI285626[] = {{&S91489[13], 7}, {&S91489[21], 2}, {&S91489[24], 15}, {&S91489[40], 6}, {&S91489[67], 17}, {NULL}};
const Segment SGI285627[] = {{&S91489[86], 17}, {NULL}};
const Segment SGI285628[] = {{&S82962[41], 4}, {NULL}};
const Segment SGI285629[] = {{&S82962[13], 14}, {&S82962[28], 25}, {&S82962[74], 17}, {NULL}};
const Segment SGI285630[] = {{&S82962[29], 16}, {NULL}};
const Segment SGI285631[] = {{&S82963[48], 4}, {NULL}};
const Segment SGI285632[] = {{&S82963[13], 14}, {&S82963[28], 6}, {&S82963[35], 25}, {&S82963[81], 17}, {NULL}};
const Segment SGI285633[] = {{&S82963[36], 16}, {NULL}};
const Segment SGI285634[] = {{&S83129[75], 4}, {NULL}};
const Segment SGI285635[] = {{&S83129[13], 14}, {&S83129[49], 17}, {NULL}};
const Segment SGI285636[] = {{&S83129[111], 4}, {NULL}};
const Segment SGI285637[] = {{&S83130[74], 4}, {NULL}};
const Segment SGI285638[] = {{&S83130[13], 14}, {&S83130[48], 17}, {NULL}};
const Segment SGI285639[] = {{&S83130[109], 4}, {NULL}};
const Segment SGI285640[] = {{&S83168[75], 4}, {NULL}};
const Segment SGI285641[] = {{&S83168[13], 14}, {&S83168[49], 17}, {NULL}};
const Segment SGI285642[] = {{&S83168[103], 12}, {NULL}};
const Segment SGI285643[] = {{&S91729[36], 3}, {NULL}};
const Segment SGI285644[] = {{&S91729[13], 7}, {&S91729[21], 2}, {&S91729[24], 15}, {&S91729[40], 6}, {&S91729[67], 17}, {NULL}};
const Segment SGI285645[] = {{&S91729[86], 8}, {NULL}};
const Segment SGI285646[] = {{&S83257[136], 4}, {NULL}};
const Segment SGI285647[] = {{&S83257[13], 3}, {&S83257[17], 13}, {&S83257[51], 17}, {&S83257[81], 44}, {NULL}};
const Segment SGI285648[] = {{&S83258[136], 4}, {NULL}};
const Segment SGI285649[] = {{&S83258[13], 3}, {&S83258[17], 13}, {&S83258[51], 17}, {&S83258[81], 44}, {NULL}};
const Segment SGI285650[] = {{&S83259[136], 4}, {NULL}};
const Segment SGI285651[] = {{&S83259[13], 3}, {&S83259[17], 13}, {&S83259[51], 17}, {&S83259[81], 44}, {NULL}};
const Segment SGI285652[] = {{&S83260[136], 4}, {NULL}};
const Segment SGI285653[] = {{&S83260[13], 3}, {&S83260[17], 13}, {&S83260[51], 17}, {&S83260[81], 44}, {NULL}};
const Segment SGI285654[] = {{&S83261[136], 4}, {NULL}};
const Segment SGI285655[] = {{&S83261[13], 3}, {&S83261[17], 13}, {&S83261[51], 17}, {&S83261[81], 44}, {NULL}};
const Segment SGI285656[] = {{&S83262[136], 4}, {NULL}};
const Segment SGI285657[] = {{&S103285[38], 5}, {NULL}};
const Segment SGI285658[] = {{&S83262[13], 3}, {&S83262[17], 13}, {&S83262[51], 17}, {&S83262[81], 44}, {NULL}};
const Segment SGI285659[] = {{&S103285[100], 5}, {NULL}};
const Segment SGI285660[] = {{&S83263[136], 4}, {NULL}};
const Segment SGI285661[] = {{&S83263[13], 3}, {&S83263[17], 13}, {&S83263[51], 17}, {&S83263[81], 44}, {NULL}};
const Segment SGI285662[] = {{&S83264[136], 4}, {NULL}};
const Segment SGI285663[] = {{&S83264[13], 3}, {&S83264[17], 13}, {&S83264[51], 17}, {&S83264[81], 44}, {NULL}};
const Segment SGI285664[] = {{&S83265[136], 4}, {NULL}};
const Segment SGI285665[] = {{&S83265[13], 3}, {&S83265[17], 13}, {&S83265[51], 17}, {&S83265[81], 44}, {NULL}};
const Segment SGI285666[] = {{&S83266[136], 4}, {NULL}};
const Segment SGI285667[] = {{&S83266[13], 3}, {&S83266[17], 13}, {&S83266[51], 17}, {&S83266[81], 44}, {NULL}};
const Segment SGI285668[] = {{&S83267[136], 4}, {NULL}};
const Segment SGI285669[] = {{&S83267[13], 3}, {&S83267[17], 13}, {&S83267[51], 17}, {&S83267[81], 44}, {NULL}};
const Segment SGI285670[] = {{&S83268[136], 4}, {NULL}};
const Segment SGI285671[] = {{&S83268[13], 3}, {&S83268[17], 13}, {&S83268[51], 17}, {&S83268[81], 44}, {NULL}};
const Segment SGI285672[] = {{&S83269[136], 4}, {NULL}};
const Segment SGI285673[] = {{&S83269[13], 3}, {&S83269[17], 13}, {&S83269[51], 17}, {&S83269[81], 44}, {NULL}};
const Segment SGI285674[] = {{&S83270[136], 4}, {NULL}};
const Segment SGI285675[] = {{&S83270[13], 3}, {&S83270[17], 13}, {&S83270[51], 17}, {&S83270[81], 44}, {NULL}};
const Segment SGI285676[] = {{&S83271[136], 4}, {NULL}};
const Segment SGI285677[] = {{&S83271[13], 3}, {&S83271[17], 13}, {&S83271[51], 17}, {&S83271[81], 44}, {NULL}};
const Segment SGI285678[] = {{&S83272[136], 4}, {NULL}};
const Segment SGI285679[] = {{&S83272[13], 3}, {&S83272[17], 13}, {&S83272[51], 17}, {&S83272[81], 44}, {NULL}};
const Segment SGI285680[] = {{&S83273[136], 4}, {NULL}};
const Segment SGI285681[] = {{&S83273[13], 3}, {&S83273[17], 13}, {&S83273[51], 17}, {&S83273[81], 44}, {NULL}};
const Segment SGI285682[] = {{&S83274[136], 4}, {NULL}};
const Segment SGI285683[] = {{&S83274[13], 3}, {&S83274[17], 13}, {&S83274[51], 17}, {&S83274[81], 44}, {NULL}};
const Segment SGI285684[] = {{&S83275[136], 4}, {NULL}};
const Segment SGI285685[] = {{&S83275[13], 3}, {&S83275[17], 13}, {&S83275[51], 17}, {&S83275[81], 44}, {NULL}};
const Segment SGI285686[] = {{&S83276[136], 4}, {NULL}};
const Segment SGI285687[] = {{&S83276[13], 3}, {&S83276[17], 13}, {&S83276[51], 17}, {&S83276[81], 44}, {NULL}};
const Segment SGI285688[] = {{&S83277[136], 4}, {NULL}};
const Segment SGI285689[] = {{&S83277[13], 3}, {&S83277[17], 13}, {&S83277[51], 17}, {&S83277[81], 44}, {NULL}};
const Segment SGI285690[] = {{&S83278[136], 4}, {NULL}};
const Segment SGI285691[] = {{&S83278[13], 3}, {&S83278[17], 13}, {&S83278[51], 17}, {&S83278[81], 44}, {NULL}};
const Segment SGI285692[] = {{&S83279[136], 4}, {NULL}};
const Segment SGI285693[] = {{&S83279[13], 3}, {&S83279[17], 13}, {&S83279[51], 17}, {&S83279[81], 44}, {NULL}};
const Segment SGI285694[] = {{&S83280[136], 4}, {NULL}};
const Segment SGI285695[] = {{&S83280[13], 3}, {&S83280[17], 13}, {&S83280[51], 17}, {&S83280[81], 44}, {NULL}};
const Segment SGI285696[] = {{&S83281[136], 4}, {NULL}};
const Segment SGI285697[] = {{&S83281[13], 3}, {&S83281[17], 13}, {&S83281[51], 17}, {&S83281[81], 44}, {NULL}};
const Segment SGI285698[] = {{&S83282[136], 4}, {NULL}};
const Segment SGI285699[] = {{&S83282[13], 3}, {&S83282[17], 13}, {&S83282[51], 17}, {&S83282[81], 44}, {NULL}};
const Segment SGI285700[] = {{&S83283[136], 4}, {NULL}};
const Segment SGI285701[] = {{&S83283[13], 3}, {&S83283[17], 13}, {&S83283[51], 17}, {&S83283[81], 44}, {NULL}};
const Segment SGI285702[] = {{&S83284[136], 4}, {NULL}};
const Segment SGI285703[] = {{&S83284[13], 3}, {&S83284[17], 13}, {&S83284[51], 17}, {&S83284[81], 44}, {NULL}};
const Segment SGI285704[] = {{&S83285[136], 4}, {NULL}};
const Segment SGI285705[] = {{&S83285[13], 3}, {&S83285[17], 13}, {&S83285[51], 17}, {&S83285[81], 44}, {NULL}};
const Segment SGI285706[] = {{&S83286[136], 4}, {NULL}};
const Segment SGI285707[] = {{&S83286[13], 3}, {&S83286[17], 13}, {&S83286[51], 17}, {&S83286[81], 44}, {NULL}};
const Segment SGI285708[] = {{&S83287[92], 4}, {NULL}};
const Segment SGI285709[] = {{&S83287[13], 3}, {&S83287[17], 13}, {&S83287[51], 17}, {NULL}};
const Segment SGI285710[] = {{&S83288[136], 4}, {NULL}};
const Segment SGI285711[] = {{&S83288[13], 3}, {&S83288[17], 13}, {&S83288[51], 17}, {&S83288[81], 44}, {NULL}};
const Segment SGI285712[] = {{&S91887[100], 3}, {NULL}};
const Segment SGI285713[] = {{&S91887[13], 7}, {&S91887[21], 2}, {&S91887[24], 15}, {&S91887[40], 6}, {&S91887[67], 17}, {NULL}};
const Segment SGI285714[] = {{&S91887[86], 17}, {NULL}};
const Segment SGI285715[] = {{&S83503[91], 4}, {NULL}};
const Segment SGI285716[] = {{&S83503[13], 9}, {&S83503[23], 22}, {&S83503[67], 17}, {NULL}};
const Segment SGI285717[] = {{&S83503[86], 9}, {NULL}};
const Segment SGI285718[] = {{&S83576[91], 4}, {NULL}};
const Segment SGI285719[] = {{&S83576[13], 9}, {&S83576[23], 22}, {&S83576[66], 17}, {NULL}};
const Segment SGI285720[] = {{&S83576[86], 9}, {NULL}};
const Segment SGI285721[] = {{&S83581[75], 4}, {NULL}};
const Segment SGI285722[] = {{&S83581[13], 15}, {&S83581[49], 17}, {NULL}};
const Segment SGI285723[] = {{&S92090[45], 3}, {NULL}};
const Segment SGI285724[] = {{&S92090[19], 17}, {&S92090[112], 6}, {NULL}};
const Segment SGI285725[] = {{&S92092[37], 3}, {NULL}};
const Segment SGI285726[] = {{&S92094[101], 3}, {NULL}};
const Segment SGI285727[] = {{&S92094[13], 9}, {&S92094[23], 2}, {&S92094[26], 22}, {&S92094[49], 6}, {&S92094[77], 17}, {NULL}};
const Segment SGI285728[] = {{&S92094[96], 8}, {NULL}};
const Segment SGI285729[] = {{&S92104[110], 3}, {NULL}};
const Segment SGI285730[] = {{&S92104[13], 7}, {&S92104[21], 2}, {&S92104[24], 15}, {&S92104[40], 6}, {&S92104[67], 17}, {NULL}};
const Segment SGI285731[] = {{&S92104[86], 9}, {NULL}};
const Segment SGI285732[] = {{&S92105[8], 3}, {NULL}};
const Segment SGI285733[] = {{&S92105[13], 7}, {&S92105[21], 2}, {&S92105[24], 15}, {&S92105[40], 6}, {&S92105[67], 17}, {NULL}};
const Segment SGI285734[] = {{&S92105[86], 9}, {NULL}};
const Segment SGI285735[] = {{&S83816[98], 4}, {NULL}};
const Segment SGI285736[] = {{&S83816[13], 14}, {&S83816[49], 17}, {NULL}};
const Segment SGI285737[] = {{&S83816[68], 18}, {NULL}};
const Segment SGI285738[] = {{&S83817[105], 4}, {NULL}};
const Segment SGI285739[] = {{&S83817[13], 14}, {&S83817[28], 6}, {&S83817[56], 17}, {NULL}};
const Segment SGI285740[] = {{&S83817[75], 18}, {NULL}};
const Segment SGI285741[] = {{&S83680[74], 4}, {NULL}};
const Segment SGI285742[] = {{&S83680[13], 14}, {&S83680[48], 17}, {NULL}};
const Segment SGI285743[] = {{&S83680[100], 12}, {NULL}};
const Segment SGI285744[] = {{&S83681[74], 4}, {NULL}};
const Segment SGI285745[] = {{&S83681[13], 14}, {&S83681[48], 17}, {NULL}};
const Segment SGI285746[] = {{&S83681[100], 12}, {NULL}};
const Segment SGI285747[] = {{&S83920[98], 4}, {NULL}};
const Segment SGI285748[] = {{&S83920[13], 14}, {&S83920[49], 17}, {NULL}};
const Segment SGI285749[] = {{&S83920[68], 18}, {NULL}};
const Segment SGI285750[] = {{&S83921[105], 4}, {NULL}};
const Segment SGI285751[] = {{&S83921[13], 14}, {&S83921[28], 6}, {&S83921[56], 17}, {NULL}};
const Segment SGI285752[] = {{&S83921[75], 18}, {NULL}};
const Segment SGI285753[] = {{&S84083[76], 4}, {NULL}};
const Segment SGI285754[] = {{&S84083[13], 15}, {&S84083[50], 17}, {NULL}};
const Segment SGI285755[] = {{&S84016[75], 4}, {NULL}};
const Segment SGI285756[] = {{&S84016[13], 14}, {&S84016[49], 17}, {NULL}};
const Segment SGI285757[] = {{&S84016[102], 12}, {NULL}};
const Segment SGI285758[] = {{&S84017[82], 4}, {NULL}};
const Segment SGI285759[] = {{&S84017[13], 14}, {&S84017[28], 6}, {&S84017[56], 17}, {NULL}};
const Segment SGI285760[] = {{&S84017[109], 12}, {NULL}};
const Segment SGI285761[] = {{&S84193[58], 4}, {NULL}};
const Segment SGI285762[] = {{&S84193[32], 17}, {NULL}};
const Segment SGI285763[] = {{&S92690[94], 3}, {NULL}};
const Segment SGI285764[] = {{&S92690[13], 7}, {&S92690[21], 2}, {&S92690[24], 15}, {&S92690[40], 6}, {&S92690[68], 17}, {NULL}};
const Segment SGI285765[] = {{&S78422[7], 3}, {NULL}};
const Segment SGI285766[] = {{&S78422[11], 5}, {NULL}};
const Segment SGI285767[] = {{&S84366[91], 4}, {NULL}};
const Segment SGI285768[] = {{&S84366[13], 9}, {&S84366[23], 22}, {&S84366[66], 17}, {NULL}};
const Segment SGI285769[] = {{&S84366[86], 9}, {NULL}};
const Segment SGI285770[] = {{&S84367[91], 4}, {NULL}};
const Segment SGI285771[] = {{&S84367[13], 9}, {&S84367[23], 22}, {&S84367[66], 17}, {NULL}};
const Segment SGI285772[] = {{&S84367[86], 9}, {NULL}};
const Segment SGI285773[] = {{&S84368[91], 4}, {NULL}};
const Segment SGI285774[] = {{&S84368[13], 9}, {&S84368[23], 22}, {&S84368[66], 17}, {NULL}};
const Segment SGI285775[] = {{&S84368[86], 9}, {NULL}};
const Segment SGI285776[] = {{&S84369[91], 4}, {NULL}};
const Segment SGI285777[] = {{&S84369[13], 9}, {&S84369[23], 22}, {&S84369[66], 17}, {NULL}};
const Segment SGI285778[] = {{&S84369[86], 9}, {NULL}};
const Segment SGI285779[] = {{&S84370[113], 4}, {NULL}};
const Segment SGI285780[] = {{&S84370[13], 9}, {&S84370[23], 22}, {&S84370[66], 17}, {NULL}};
const Segment SGI285781[] = {{&S84370[94], 3}, {NULL}};
const Segment SGI285782[] = {{&S84372[113], 4}, {NULL}};
const Segment SGI285783[] = {{&S84372[13], 9}, {&S84372[23], 22}, {&S84372[67], 17}, {NULL}};
const Segment SGI285784[] = {{&S84372[95], 3}, {NULL}};
const Segment SGI285785[] = {{&S84373[113], 4}, {NULL}};
const Segment SGI285786[] = {{&S84373[13], 9}, {&S84373[23], 22}, {&S84373[67], 17}, {NULL}};
const Segment SGI285787[] = {{&S84373[95], 3}, {NULL}};
const Segment SGI285788[] = {{&S84374[112], 4}, {NULL}};
const Segment SGI285789[] = {{&S84374[13], 9}, {&S84374[23], 22}, {&S84374[66], 17}, {NULL}};
const Segment SGI285790[] = {{&S84374[94], 3}, {NULL}};
const Segment SGI285791[] = {{&S84327[93], 4}, {NULL}};
const Segment SGI285792[] = {{&S84327[13], 3}, {&S84327[17], 13}, {&S84327[52], 17}, {NULL}};
const Segment SGI285793[] = {{&S84327[71], 12}, {NULL}};
const Segment SGI285794[] = {{&S92846[94], 3}, {NULL}};
const Segment SGI285795[] = {{&S92846[13], 7}, {&S92846[21], 2}, {&S92846[24], 15}, {&S92846[40], 6}, {&S92846[68], 17}, {NULL}};
const Segment SGI285796[] = {{&S84477[98], 4}, {NULL}};
const Segment SGI285797[] = {{&S84477[13], 14}, {&S84477[49], 17}, {NULL}};
const Segment SGI285798[] = {{&S84477[68], 18}, {NULL}};
const Segment SGI285799[] = {{&S84478[105], 4}, {NULL}};
const Segment SGI285800[] = {{&S84478[13], 14}, {&S84478[28], 6}, {&S84478[56], 17}, {NULL}};
const Segment SGI285801[] = {{&S84478[75], 18}, {NULL}};
const Segment SGI285802[] = {{&S84498[79], 4}, {NULL}};
const Segment SGI285803[] = {{&S84498[13], 3}, {&S84498[17], 14}, {&S84498[53], 17}, {NULL}};
const Segment SGI285804[] = {{&S84499[78], 4}, {NULL}};
const Segment SGI285805[] = {{&S84499[13], 3}, {&S84499[17], 13}, {&S84499[52], 17}, {NULL}};
const Segment SGI285806[] = {{&S104680[38], 5}, {NULL}};
const Segment SGI285807[] = {{&S104680[97], 5}, {NULL}};
const Segment SGI285808[] = {{&S93299[94], 3}, {NULL}};
const Segment SGI285809[] = {{&S93299[13], 7}, {&S93299[21], 2}, {&S93299[24], 15}, {&S93299[40], 6}, {&S93299[68], 17}, {NULL}};
const Segment SGI285810[] = {{&S93299[120], 15}, {NULL}};
const Segment SGI285811[] = {{&S84911[82], 4}, {NULL}};
const Segment SGI285812[] = {{&S84911[13], 3}, {&S84911[17], 14}, {&S84911[54], 17}, {NULL}};
const Segment SGI285813[] = {{&S84912[81], 4}, {NULL}};
const Segment SGI285814[] = {{&S84912[13], 3}, {&S84912[17], 13}, {&S84912[53], 17}, {NULL}};
const Segment SGI285815[] = {{&S105467[38], 5}, {NULL}};
const Segment SGI285816[] = {{&S105467[99], 5}, {NULL}};
const Segment SGI285817[] = {{&S93771[100], 3}, {NULL}};
const Segment SGI285818[] = {{&S93771[13], 7}, {&S93771[21], 2}, {&S93771[24], 15}, {&S93771[40], 6}, {&S93771[67], 17}, {NULL}};
const Segment SGI285819[] = {{&S93771[86], 17}, {NULL}};
const Segment SGI285820[] = {{&S93790[91], 3}, {NULL}};
const Segment SGI285821[] = {{&S93790[13], 3}, {&S93790[17], 2}, {&S93790[20], 17}, {&S93790[38], 6}, {&S93790[65], 17}, {NULL}};
const Segment SGI285822[] = {{&S85357[90], 4}, {NULL}};
const Segment SGI285823[] = {{&S85357[13], 9}, {&S85357[23], 22}, {&S85357[66], 17}, {NULL}};
const Segment SGI285824[] = {{&S85357[85], 9}, {NULL}};
const Segment SGI285825[] = {{&S85358[90], 4}, {NULL}};
const Segment SGI285826[] = {{&S85358[13], 9}, {&S85358[23], 22}, {&S85358[66], 17}, {NULL}};
const Segment SGI285827[] = {{&S85358[85], 9}, {NULL}};
const Segment SGI285828[] = {{&S85359[90], 4}, {NULL}};
const Segment SGI285829[] = {{&S85359[13], 9}, {&S85359[23], 22}, {&S85359[66], 17}, {NULL}};
const Segment SGI285830[] = {{&S85359[85], 9}, {NULL}};
const Segment SGI285831[] = {{&S85369[102], 4}, {NULL}};
const Segment SGI285832[] = {{&S85369[13], 3}, {&S85369[17], 13}, {&S85369[31], 16}, {&S85369[48], 6}, {&S85369[76], 17}, {NULL}};
const Segment SGI285833[] = {{&S85448[109], 4}, {NULL}};
const Segment SGI285834[] = {{&S85448[13], 3}, {&S85448[17], 13}, {&S85448[31], 23}, {&S85448[55], 6}, {&S85448[83], 17}, {NULL}};
const Segment SGI285835[] = {{&S85461[108], 4}, {NULL}};
const Segment SGI285836[] = {{&S85461[13], 3}, {&S85461[17], 13}, {&S85461[31], 22}, {&S85461[54], 6}, {&S85461[82], 17}, {NULL}};
const Segment SGI285837[] = {{&S85462[108], 4}, {NULL}};
const Segment SGI285838[] = {{&S85462[13], 3}, {&S85462[17], 13}, {&S85462[31], 22}, {&S85462[54], 6}, {&S85462[82], 17}, {NULL}};
const Segment SGI285839[] = {{&S85463[108], 4}, {NULL}};
const Segment SGI285840[] = {{&S85463[13], 3}, {&S85463[17], 13}, {&S85463[31], 22}, {&S85463[54], 6}, {&S85463[82], 17}, {NULL}};
const Segment SGI285841[] = {{&S94083[13], 7}, {&S94083[21], 2}, {&S94083[24], 17}, {&S94083[42], 6}, {&S94083[70], 17}, {NULL}};
const Segment SGI285842[] = {{&S85651[97], 4}, {NULL}};
const Segment SGI285843[] = {{&S85651[13], 14}, {&S85651[48], 17}, {NULL}};
const Segment SGI285844[] = {{&S85651[67], 18}, {NULL}};
const Segment SGI285845[] = {{&S85662[65], 4}, {NULL}};
const Segment SGI285846[] = {{&S85662[13], 5}, {&S85662[39], 17}, {NULL}};
const Segment SGI285847[] = {{&S85983[78], 4}, {NULL}};
const Segment SGI285848[] = {{&S85983[13], 3}, {&S85983[17], 11}, {&S85983[50], 17}, {&S85983[137], 23}, {NULL}};
const Segment SGI285849[] = {{&S85983[123], 12}, {NULL}};
const Segment SGI285850[] = {{&S85984[80], 4}, {NULL}};
const Segment SGI285851[] = {{&S85984[13], 3}, {&S85984[17], 13}, {&S85984[52], 17}, {&S85984[139], 23}, {NULL}};
const Segment SGI285852[] = {{&S85984[125], 12}, {NULL}};
const Segment SGI285853[] = {{&S85985[80], 4}, {NULL}};
const Segment SGI285854[] = {{&S85985[13], 3}, {&S85985[17], 13}, {&S85985[52], 17}, {&S85985[139], 22}, {NULL}};
const Segment SGI285855[] = {{&S85985[125], 12}, {NULL}};
const Segment SGI285856[] = {{&S85778[41], 4}, {NULL}};
const Segment SGI285857[] = {{&S85778[13], 14}, {&S85778[28], 25}, {&S85778[74], 17}, {NULL}};
const Segment SGI285858[] = {{&S85778[29], 16}, {NULL}};
const Segment SGI285859[] = {{&S85779[48], 4}, {NULL}};
const Segment SGI285860[] = {{&S85779[13], 14}, {&S85779[28], 6}, {&S85779[35], 25}, {&S85779[81], 17}, {NULL}};
const Segment SGI285861[] = {{&S85779[36], 16}, {NULL}};
const Segment SGI285862[] = {{&S85870[93], 4}, {NULL}};
const Segment SGI285863[] = {{&S85870[13], 3}, {&S85870[17], 13}, {&S85870[52], 17}, {&S85870[143], 10}, {&S85870[154], 16}, {&S85870[171], 23}, {NULL}};
const Segment SGI285864[] = {{&S85870[71], 12}, {NULL}};
const Segment SGI285865[] = {{&S86130[74], 4}, {NULL}};
const Segment SGI285866[] = {{&S86130[13], 14}, {&S86130[48], 17}, {NULL}};
const Segment SGI285867[] = {{&S86130[100], 18}, {NULL}};
const Segment SGI285868[] = {{&S86131[81], 4}, {NULL}};
const Segment SGI285869[] = {{&S86131[13], 14}, {&S86131[28], 6}, {&S86131[55], 17}, {NULL}};
const Segment SGI285870[] = {{&S86131[107], 18}, {NULL}};
const Segment SGI285871[] = {{&S85915[75], 4}, {NULL}};
const Segment SGI285872[] = {{&S85915[13], 14}, {&S85915[49], 17}, {NULL}};
const Segment SGI285873[] = {{&S85915[111], 4}, {NULL}};
const Segment SGI285874[] = {{&S94669[99], 3}, {NULL}};
const Segment SGI285875[] = {{&S94669[13], 3}, {&S94669[17], 2}, {&S94669[20], 11}, {&S94669[32], 6}, {&S94669[59], 17}, {NULL}};
const Segment SGI285876[] = {{&S94669[78], 11}, {NULL}};
const Segment SGI285877[] = {{&S94670[101], 3}, {NULL}};
const Segment SGI285878[] = {{&S94670[13], 3}, {&S94670[17], 2}, {&S94670[20], 13}, {&S94670[34], 6}, {&S94670[61], 17}, {NULL}};
const Segment SGI285879[] = {{&S94670[80], 11}, {NULL}};
const Segment SGI285880[] = {{&S86335[74], 4}, {NULL}};
const Segment SGI285881[] = {{&S86335[13], 14}, {&S86335[48], 17}, {NULL}};
const Segment SGI285882[] = {{&S86335[100], 18}, {NULL}};
const Segment SGI285883[] = {{&S86336[81], 4}, {NULL}};
const Segment SGI285884[] = {{&S86336[13], 14}, {&S86336[28], 6}, {&S86336[55], 17}, {NULL}};
const Segment SGI285885[] = {{&S86336[107], 18}, {NULL}};
const Segment SGI285886[] = {{&S95159[103], 3}, {NULL}};
const Segment SGI285887[] = {{&S95159[13], 9}, {&S95159[23], 2}, {&S95159[26], 22}, {&S95159[49], 6}, {&S95159[77], 17}, {NULL}};
const Segment SGI285888[] = {{&S95160[102], 3}, {NULL}};
const Segment SGI285889[] = {{&S95160[13], 9}, {&S95160[23], 2}, {&S95160[26], 22}, {&S95160[49], 6}, {&S95160[76], 17}, {NULL}};
const Segment SGI285890[] = {{&S95311[94], 3}, {NULL}};
const Segment SGI285891[] = {{&S95311[13], 3}, {&S95311[17], 2}, {&S95311[20], 19}, {&S95311[41], 6}, {&S95311[68], 17}, {NULL}};
const Segment SGI285892[] = {{&S95391[95], 3}, {NULL}};
const Segment SGI285893[] = {{&S95391[13], 7}, {&S95391[21], 2}, {&S95391[24], 16}, {&S95391[41], 6}, {&S95391[69], 17}, {NULL}};
const Segment SGI285894[] = {{&S95394[128], 3}, {NULL}};
const Segment SGI285895[] = {{&S95394[13], 5}, {&S95394[19], 2}, {&S95394[22], 14}, {&S95394[37], 6}, {&S95394[44], 36}, {&S95394[102], 17}, {NULL}};
const Segment SGI285896[] = {{&S95395[128], 3}, {NULL}};
const Segment SGI285897[] = {{&S95395[13], 5}, {&S95395[19], 2}, {&S95395[22], 14}, {&S95395[37], 6}, {&S95395[44], 36}, {&S95395[102], 17}, {NULL}};
const Segment SGI285898[] = {{&S86790[90], 4}, {NULL}};
const Segment SGI285899[] = {{&S86790[13], 9}, {&S86790[23], 22}, {&S86790[66], 17}, {NULL}};
const Segment SGI285900[] = {{&S86790[85], 9}, {NULL}};
const Segment SGI285901[] = {{&S95398[8], 3}, {NULL}};
const Segment SGI285902[] = {{&S95398[13], 7}, {&S95398[21], 2}, {&S95398[24], 15}, {&S95398[40], 6}, {&S95398[67], 17}, {NULL}};
const Segment SGI285903[] = {{&S95398[86], 9}, {NULL}};
const Segment SGI285904[] = {{&S86830[105], 4}, {NULL}};
const Segment SGI285905[] = {{&S86830[13], 3}, {&S86830[17], 13}, {&S86830[31], 26}, {&S86830[79], 17}, {NULL}};
const Segment SGI285906[] = {{&S86831[105], 4}, {NULL}};
const Segment SGI285907[] = {{&S86831[13], 3}, {&S86831[17], 13}, {&S86831[31], 26}, {&S86831[79], 17}, {NULL}};
const Segment SGI285908[] = {{&S86923[90], 4}, {NULL}};
const Segment SGI285909[] = {{&S86923[13], 9}, {&S86923[23], 22}, {&S86923[66], 17}, {NULL}};
const Segment SGI285910[] = {{&S86923[85], 9}, {NULL}};
const Segment SGI285911[] = {{&S87052[75], 4}, {NULL}};
const Segment SGI285912[] = {{&S87052[13], 14}, {&S87052[49], 17}, {NULL}};
const Segment SGI285913[] = {{&S87052[103], 18}, {NULL}};
const Segment SGI285914[] = {{&S87053[82], 4}, {NULL}};
const Segment SGI285915[] = {{&S87053[13], 14}, {&S87053[28], 6}, {&S87053[56], 17}, {NULL}};
const Segment SGI285916[] = {{&S87053[110], 18}, {NULL}};
const Segment SGI285917[] = {{&S87062[92], 4}, {NULL}};
const Segment SGI285918[] = {{&S87062[13], 9}, {&S87062[23], 22}, {&S87062[67], 17}, {NULL}};
const Segment SGI285919[] = {{&S87062[87], 9}, {NULL}};
const Segment SGI285920[] = {{&S87092[75], 4}, {NULL}};
const Segment SGI285921[] = {{&S87092[13], 14}, {&S87092[49], 17}, {NULL}};
const Segment SGI285922[] = {{&S87092[110], 3}, {NULL}};
const Segment SGI285923[] = {{&S87156[106], 4}, {NULL}};
const Segment SGI285924[] = {{&S87156[13], 3}, {&S87156[17], 13}, {&S87156[31], 27}, {&S87156[80], 17}, {NULL}};
const Segment SGI285925[] = {{&S87189[75], 4}, {NULL}};
const Segment SGI285926[] = {{&S87189[13], 15}, {&S87189[49], 17}, {NULL}};
const Segment SGI285927[] = {{&S87385[112], 4}, {NULL}};
const Segment SGI285928[] = {{&S87385[13], 9}, {&S87385[23], 22}, {&S87385[67], 17}, {NULL}};
const Segment SGI285929[] = {{&S87385[86], 11}, {NULL}};
const Segment SGI285930[] = {{&S81787[7], 3}, {NULL}};
const Segment SGI285931[] = {{&S81787[11], 4}, {NULL}};
const Segment SGI285932[] = {{&S87429[74], 4}, {NULL}};
const Segment SGI285933[] = {{&S87429[13], 14}, {&S87429[48], 17}, {NULL}};
const Segment SGI285934[] = {{&S87429[109], 3}, {NULL}};
const Segment SGI285935[] = {{&S87481[79], 4}, {NULL}};
const Segment SGI285936[] = {{&S87481[13], 3}, {&S87481[17], 14}, {&S87481[53], 17}, {NULL}};
const Segment SGI285937[] = {{&S87497[79], 4}, {NULL}};
const Segment SGI285938[] = {{&S87497[13], 3}, {&S87497[17], 14}, {&S87497[53], 17}, {NULL}};
const Segment SGI285939[] = {{&S87553[75], 4}, {NULL}};
const Segment SGI285940[] = {{&S87553[13], 14}, {&S87553[49], 17}, {NULL}};
const Segment SGI285941[] = {{&S87553[101], 18}, {NULL}};
const Segment SGI285942[] = {{&S87554[82], 4}, {NULL}};
const Segment SGI285943[] = {{&S87554[13], 14}, {&S87554[28], 6}, {&S87554[56], 17}, {NULL}};
const Segment SGI285944[] = {{&S87554[108], 18}, {NULL}};
const Segment SGI285945[] = {{&S87631[87], 4}, {NULL}};
const Segment SGI285946[] = {{&S87631[13], 3}, {&S87631[17], 13}, {&S87631[31], 8}, {&S87631[61], 17}, {NULL}};
const Segment SGI285947[] = {{&S87684[82], 4}, {NULL}};
const Segment SGI285948[] = {{&S87684[13], 14}, {&S87684[28], 6}, {&S87684[56], 17}, {NULL}};
const Segment SGI285949[] = {{&S87684[109], 18}, {NULL}};
const Segment SGI285950[] = {{&S87685[75], 4}, {NULL}};
const Segment SGI285951[] = {{&S87685[13], 14}, {&S87685[49], 17}, {NULL}};
const Segment SGI285952[] = {{&S87685[102], 18}, {NULL}};
const Segment SGI285953[] = {{&S87669[75], 4}, {NULL}};
const Segment SGI285954[] = {{&S87669[13], 15}, {&S87669[49], 17}, {NULL}};
const Segment SGI285955[] = {{&S87670[98], 4}, {NULL}};
const Segment SGI285956[] = {{&S87670[13], 14}, {&S87670[49], 17}, {NULL}};
const Segment SGI285957[] = {{&S87670[68], 18}, {NULL}};
const Segment SGI285958[] = {{&S87671[105], 4}, {NULL}};
const Segment SGI285959[] = {{&S87671[13], 14}, {&S87671[28], 6}, {&S87671[56], 17}, {NULL}};
const Segment SGI285960[] = {{&S87671[75], 18}, {NULL}};
const Segment SGI285961[] = {{&S96340[8], 3}, {NULL}};
const Segment SGI285962[] = {{&S96340[13], 9}, {&S96340[23], 2}, {&S96340[26], 22}, {&S96340[49], 6}, {&S96340[76], 17}, {NULL}};
const Segment SGI285963[] = {{&S96340[95], 8}, {NULL}};
const Segment SGI285964[] = {{&S87681[81], 4}, {NULL}};
const Segment SGI285965[] = {{&S87681[13], 14}, {&S87681[28], 6}, {&S87681[55], 17}, {NULL}};
const Segment SGI285966[] = {{&S87681[107], 12}, {NULL}};
const Segment SGI285967[] = {{&S87683[74], 4}, {NULL}};
const Segment SGI285968[] = {{&S87683[13], 14}, {&S87683[48], 17}, {NULL}};
const Segment SGI285969[] = {{&S87683[100], 12}, {NULL}};
const Segment SGI285970[] = {{&S87721[80], 4}, {NULL}};
const Segment SGI285971[] = {{&S87721[13], 14}, {&S87721[28], 6}, {&S87721[56], 17}, {NULL}};
const Segment SGI285972[] = {{&S87721[75], 9}, {NULL}};
const Segment SGI285973[] = {{&S87722[73], 4}, {NULL}};
const Segment SGI285974[] = {{&S87722[13], 14}, {&S87722[49], 17}, {NULL}};
const Segment SGI285975[] = {{&S87722[68], 9}, {NULL}};
const Segment SGI285976[] = {{&S87799[97], 4}, {NULL}};
const Segment SGI285977[] = {{&S87799[13], 14}, {&S87799[48], 17}, {NULL}};
const Segment SGI285978[] = {{&S87799[67], 18}, {NULL}};
const Segment SGI285979[] = {{&S87800[104], 4}, {NULL}};
const Segment SGI285980[] = {{&S87800[13], 14}, {&S87800[28], 6}, {&S87800[55], 17}, {NULL}};
const Segment SGI285981[] = {{&S87800[74], 18}, {NULL}};
const Segment SGI285982[] = {{&S87803[74], 4}, {NULL}};
const Segment SGI285983[] = {{&S87803[13], 14}, {&S87803[48], 17}, {NULL}};
const Segment SGI285984[] = {{&S87803[109], 3}, {NULL}};
const Segment SGI285985[] = {{&S87837[88], 4}, {NULL}};
const Segment SGI285986[] = {{&S87837[13], 14}, {&S87837[28], 6}, {&S87837[55], 17}, {NULL}};
const Segment SGI285987[] = {{&S87837[74], 18}, {NULL}};
const Segment SGI285988[] = {{&S87838[81], 4}, {NULL}};
const Segment SGI285989[] = {{&S87838[13], 14}, {&S87838[48], 17}, {NULL}};
const Segment SGI285990[] = {{&S87838[67], 18}, {NULL}};
const Segment SGI285991[] = {{&S88083[87], 4}, {NULL}};
const Segment SGI285992[] = {{&S88083[13], 3}, {&S88083[17], 13}, {&S88083[31], 8}, {&S88083[61], 17}, {NULL}};
const Segment SGI285993[] = {{&S88210[72], 4}, {NULL}};
const Segment SGI285994[] = {{&S88210[13], 14}, {&S88210[48], 17}, {NULL}};
const Segment SGI285995[] = {{&S88210[67], 9}, {NULL}};
const Segment SGI285996[] = {{&S88211[79], 4}, {NULL}};
const Segment SGI285997[] = {{&S88211[13], 14}, {&S88211[28], 6}, {&S88211[55], 17}, {NULL}};
const Segment SGI285998[] = {{&S88211[74], 9}, {NULL}};
const Segment SGI285999[] = {{&S82822[7], 3}, {NULL}};
const Segment SGI286000[] = {{&S82822[11], 6}, {NULL}};
const Segment SGI286001[] = {{&S88606[90], 4}, {NULL}};
const Segment SGI286002[] = {{&S88606[13], 3}, {&S88606[17], 11}, {&S88606[49], 17}, {NULL}};
const Segment SGI286003[] = {{&S88606[68], 12}, {NULL}};
const Segment SGI286004[] = {{&S97341[100], 3}, {NULL}};
const Segment SGI286005[] = {{&S97341[13], 9}, {&S97341[23], 2}, {&S97341[26], 22}, {&S97341[49], 6}, {&S97341[76], 17}, {NULL}};
const Segment SGI286006[] = {{&S97341[95], 8}, {NULL}};
const Segment SGI286007[] = {{&S83209[7], 3}, {NULL}};
const Segment SGI286008[] = {{&S83209[11], 4}, {NULL}};
const Segment SGI286009[] = {{&S97480[92], 3}, {NULL}};
const Segment SGI286010[] = {{&S97480[13], 7}, {&S97480[21], 2}, {&S97480[24], 15}, {&S97480[40], 6}, {&S97480[68], 17}, {NULL}};
const Segment SGI286011[] = {{&S97480[87], 8}, {NULL}};
const Segment SGI286012[] = {{&S88846[70], 4}, {NULL}};
const Segment SGI286013[] = {{&S88846[12], 12}, {&S88846[44], 17}, {NULL}};
const Segment SGI286014[] = {{&S88983[105], 4}, {NULL}};
const Segment SGI286015[] = {{&S88983[13], 14}, {&S88983[28], 6}, {&S88983[56], 17}, {NULL}};
const Segment SGI286016[] = {{&S88983[75], 18}, {NULL}};
const Segment SGI286017[] = {{&S88984[98], 4}, {NULL}};
const Segment SGI286018[] = {{&S88984[13], 14}, {&S88984[49], 17}, {NULL}};
const Segment SGI286019[] = {{&S88984[68], 18}, {NULL}};
const Segment SGI286020[] = {{&S97806[110], 3}, {NULL}};
const Segment SGI286021[] = {{&S97806[13], 7}, {&S97806[21], 2}, {&S97806[24], 15}, {&S97806[40], 6}, {&S97806[67], 17}, {NULL}};
const Segment SGI286022[] = {{&S97806[86], 9}, {NULL}};
const Segment SGI286023[] = {{&S89017[87], 4}, {NULL}};
const Segment SGI286024[] = {{&S89017[13], 3}, {&S89017[17], 13}, {&S89017[31], 8}, {&S89017[61], 17}, {NULL}};
const Segment SGI286025[] = {{&S83740[7], 3}, {NULL}};
const Segment SGI286026[] = {{&S83740[11], 5}, {NULL}};
const Segment SGI286027[] = {{&S89262[74], 4}, {NULL}};
const Segment SGI286028[] = {{&S89262[13], 14}, {&S89262[48], 17}, {NULL}};
const Segment SGI286029[] = {{&S89262[100], 12}, {NULL}};
const Segment SGI286030[] = {{&S89263[74], 4}, {NULL}};
const Segment SGI286031[] = {{&S89263[13], 14}, {&S89263[48], 17}, {NULL}};
const Segment SGI286032[] = {{&S89263[100], 12}, {NULL}};
const Segment SGI286033[] = {{&S89121[79], 4}, {NULL}};
const Segment SGI286034[] = {{&S89121[13], 3}, {&S89121[17], 14}, {&S89121[53], 17}, {NULL}};
const Segment SGI286035[] = {{&S89122[78], 4}, {NULL}};
const Segment SGI286036[] = {{&S89122[13], 3}, {&S89122[17], 13}, {&S89122[52], 17}, {NULL}};
const Segment SGI286037[] = {{&S89123[75], 4}, {NULL}};
const Segment SGI286038[] = {{&S89123[13], 14}, {&S89123[49], 17}, {NULL}};
const Segment SGI286039[] = {{&S89123[103], 18}, {NULL}};
const Segment SGI286040[] = {{&S89124[82], 4}, {NULL}};
const Segment SGI286041[] = {{&S89124[13], 14}, {&S89124[28], 6}, {&S89124[56], 17}, {NULL}};
const Segment SGI286042[] = {{&S89124[110], 18}, {NULL}};
const Segment SGI286043[] = {{&S89136[90], 4}, {NULL}};
const Segment SGI286044[] = {{&S89136[13], 9}, {&S89136[23], 22}, {&S89136[66], 17}, {NULL}};
const Segment SGI286045[] = {{&S89136[85], 9}, {NULL}};
const Segment SGI286046[] = {{&S98089[8], 3}, {NULL}};
const Segment SGI286047[] = {{&S98089[13], 7}, {&S98089[21], 2}, {&S98089[24], 15}, {&S98089[40], 6}, {&S98089[67], 17}, {NULL}};
const Segment SGI286048[] = {{&S98089[86], 9}, {NULL}};
const Segment SGI286049[] = {{&S89448[74], 4}, {NULL}};
const Segment SGI286050[] = {{&S89448[13], 14}, {&S89448[48], 17}, {NULL}};
const Segment SGI286051[] = {{&S89448[109], 4}, {NULL}};
const Segment SGI286052[] = {{&S89457[87], 4}, {NULL}};
const Segment SGI286053[] = {{&S89457[13], 14}, {&S89457[49], 17}, {NULL}};
const Segment SGI286054[] = {{&S89457[68], 11}, {NULL}};
const Segment SGI286055[] = {{&S84078[7], 3}, {NULL}};
const Segment SGI286056[] = {{&S84078[11], 4}, {NULL}};
const Segment SGI286057[] = {{&S84173[7], 3}, {NULL}};
const Segment SGI286058[] = {{&S84173[11], 4}, {NULL}};
const Segment SGI286059[] = {{&S89573[95], 4}, {NULL}};
const Segment SGI286060[] = {{&S89573[13], 7}, {&S89573[21], 2}, {&S89573[24], 16}, {&S89573[41], 6}, {&S89573[69], 17}, {NULL}};
const Segment SGI286061[] = {{&S2912[72], 4}, {NULL}};
const Segment SGI286062[] = {{&S2912[8], 3}, {NULL}};
const Segment SGI286063[] = {{&S2912[24], 14}, {NULL}};
const Segment SGI286064[] = {{&S2912[7], 1}, {NULL}};
const Segment SGI286065[] = {{&S2912[15], 4}, {&S2912[27], 11}, {NULL}};
const Segment SGI286066[] = {{&S2913[77], 4}, {NULL}};
const Segment SGI286067[] = {{&S2913[8], 3}, {&S2913[8], 3}, {NULL}};
const Segment SGI286068[] = {{&S2913[25], 7}, {NULL}};
const Segment SGI286069[] = {{&S2913[37], 14}, {NULL}};
const Segment SGI286070[] = {{&S2913[7], 1}, {NULL}};
const Segment SGI286071[] = {{&S2913[28], 4}, {&S2913[40], 11}, {NULL}};
const Segment SGI286072[] = {{&S89600[104], 4}, {NULL}};
const Segment SGI286073[] = {{&S89600[13], 4}, {&S89600[18], 2}, {&S89600[21], 27}, {&S89600[49], 6}, {&S89600[77], 17}, {NULL}};
const Segment SGI286074[] = {{&S89600[130], 10}, {NULL}};
const Segment SGI286075[] = {{&S2914[76], 4}, {NULL}};
const Segment SGI286076[] = {{&S2914[8], 3}, {&S2914[8], 3}, {&S2914[8], 3}, {NULL}};
const Segment SGI286077[] = {{&S2914[13], 7}, {&S2914[13], 7}, {&S2914[188], 3}, {NULL}};
const Segment SGI286078[] = {{&S2914[25], 14}, {&S2914[25], 14}, {&S2914[188], 3}, {NULL}};
const Segment SGI286079[] = {{&S89601[104], 4}, {NULL}};
const Segment SGI286080[] = {{&S89601[13], 4}, {&S89601[18], 2}, {&S89601[21], 27}, {&S89601[49], 6}, {&S89601[77], 17}, {NULL}};
const Segment SGI286081[] = {{&S2914[7], 1}, {NULL}};
const Segment SGI286082[] = {{&S89601[130], 10}, {NULL}};
const Segment SGI286083[] = {{&S2914[16], 4}, {&S2914[28], 11}, {&S2914[16], 4}, {&S2914[28], 11}, {NULL}};
const Segment SGI286084[] = {{&S2915[72], 4}, {NULL}};
const Segment SGI286085[] = {{&S2915[8], 3}, {NULL}};
const Segment SGI286086[] = {{&S2915[13], 7}, {NULL}};
const Segment SGI286087[] = {{&S2915[25], 14}, {NULL}};
const Segment SGI286088[] = {{&S2915[7], 1}, {NULL}};
const Segment SGI286089[] = {{&S2915[16], 4}, {&S2915[28], 11}, {NULL}};
const Segment SGI286090[] = {{&S2916[77], 4}, {NULL}};
const Segment SGI286091[] = {{&S2916[28], 14}, {NULL}};
const Segment SGI286092[] = {{&S2916[7], 1}, {NULL}};
const Segment SGI286093[] = {{&S2916[19], 4}, {&S2916[31], 11}, {NULL}};
const Segment SGI286094[] = {{&S2917[73], 4}, {NULL}};
const Segment SGI286095[] = {{&S2917[8], 3}, {NULL}};
const Segment SGI286096[] = {{&S2917[7], 1}, {NULL}};
const Segment SGI286097[] = {{&S2917[16], 4}, {&S2917[28], 11}, {NULL}};
const Segment SGI286098[] = {{&S2918[76], 4}, {NULL}};
const Segment SGI286099[] = {{&S2918[8], 3}, {NULL}};
const Segment SGI286100[] = {{&S2918[13], 7}, {NULL}};
const Segment SGI286101[] = {{&S2918[25], 14}, {NULL}};
const Segment SGI286102[] = {{&S2918[7], 1}, {NULL}};
const Segment SGI286103[] = {{&S2918[16], 4}, {&S2918[28], 11}, {NULL}};
const Segment SGI286104[] = {{&S2919[84], 4}, {NULL}};
const Segment SGI286105[] = {{&S2919[8], 3}, {NULL}};
const Segment SGI286106[] = {{&S2919[13], 7}, {NULL}};
const Segment SGI286107[] = {{&S2919[25], 14}, {NULL}};
const Segment SGI286108[] = {{&S2919[7], 1}, {NULL}};
const Segment SGI286109[] = {{&S2919[16], 4}, {&S2919[28], 11}, {&S2919[44], 2}, {NULL}};
const Segment SGI286110[] = {{&S2920[48], 4}, {NULL}};
const Segment SGI286111[] = {{&S2920[8], 3}, {NULL}};
const Segment SGI286112[] = {{&S2920[7], 1}, {NULL}};
const Segment SGI286113[] = {{&S2920[15], 10}, {NULL}};
const Segment SGI286114[] = {{&S2921[61], 4}, {NULL}};
const Segment SGI286115[] = {{&S2921[8], 3}, {NULL}};
const Segment SGI286116[] = {{&S2921[13], 14}, {NULL}};
const Segment SGI286117[] = {{&S2921[7], 1}, {NULL}};
const Segment SGI286118[] = {{&S2921[16], 11}, {NULL}};
const Segment SGI286119[] = {{&S2922[73], 4}, {NULL}};
const Segment SGI286120[] = {{&S2922[8], 3}, {NULL}};
const Segment SGI286121[] = {{&S2922[13], 7}, {NULL}};
const Segment SGI286122[] = {{&S2922[25], 14}, {NULL}};
const Segment SGI286123[] = {{&S2922[7], 1}, {NULL}};
const Segment SGI286124[] = {{&S2922[16], 4}, {&S2922[28], 11}, {NULL}};
const Segment SGI286125[] = {{&S89709[71], 4}, {NULL}};
const Segment SGI286126[] = {{&S89709[13], 10}, {&S89709[45], 17}, {NULL}};
const Segment SGI286127[] = {{&S89731[74], 4}, {NULL}};
const Segment SGI286128[] = {{&S89731[13], 14}, {&S89731[48], 17}, {NULL}};
const Segment SGI286129[] = {{&S89731[100], 18}, {NULL}};
const Segment SGI286130[] = {{&S89732[81], 4}, {NULL}};
const Segment SGI286131[] = {{&S89732[13], 14}, {&S89732[28], 6}, {&S89732[55], 17}, {NULL}};
const Segment SGI286132[] = {{&S89732[107], 18}, {NULL}};
const Segment SGI286133[] = {{&S89833[113], 4}, {NULL}};
const Segment SGI286134[] = {{&S89833[13], 6}, {&S89833[20], 2}, {&S89833[23], 34}, {&S89833[58], 6}, {&S89833[86], 17}, {NULL}};
const Segment SGI286135[] = {{&S89833[139], 10}, {NULL}};
const Segment SGI286136[] = {{&S98575[83], 3}, {NULL}};
const Segment SGI286137[] = {{&S98575[13], 3}, {&S98575[17], 2}, {&S98575[20], 11}, {&S98575[32], 6}, {&S98575[59], 17}, {NULL}};
const Segment SGI286138[] = {{&S98575[78], 8}, {NULL}};
const Segment SGI286139[] = {{&S89903[90], 4}, {NULL}};
const Segment SGI286140[] = {{&S89903[13], 3}, {&S89903[17], 11}, {&S89903[49], 17}, {NULL}};
const Segment SGI286141[] = {{&S89903[68], 12}, {NULL}};
const Segment SGI286142[] = {{&S89880[74], 4}, {NULL}};
const Segment SGI286143[] = {{&S89880[13], 14}, {&S89880[48], 17}, {NULL}};
const Segment SGI286144[] = {{&S89880[100], 18}, {NULL}};
const Segment SGI286145[] = {{&S89881[81], 4}, {NULL}};
const Segment SGI286146[] = {{&S89881[13], 14}, {&S89881[28], 6}, {&S89881[55], 17}, {NULL}};
const Segment SGI286147[] = {{&S89881[107], 18}, {NULL}};
const Segment SGI286148[] = {{&S90052[93], 4}, {NULL}};
const Segment SGI286149[] = {{&S90052[13], 9}, {&S90052[23], 22}, {&S90052[67], 17}, {NULL}};
const Segment SGI286150[] = {{&S90052[106], 9}, {NULL}};
const Segment SGI286151[] = {{&S90071[113], 4}, {NULL}};
const Segment SGI286152[] = {{&S90071[13], 9}, {&S90071[23], 22}, {&S90071[66], 17}, {NULL}};
const Segment SGI286153[] = {{&S90071[94], 3}, {NULL}};
const Segment SGI286154[] = {{&S90072[113], 4}, {NULL}};
const Segment SGI286155[] = {{&S90072[13], 9}, {&S90072[23], 22}, {&S90072[66], 17}, {NULL}};
const Segment SGI286156[] = {{&S90072[94], 3}, {NULL}};
const Segment SGI286157[] = {{&S90189[97], 4}, {NULL}};
const Segment SGI286158[] = {{&S90189[13], 14}, {&S90189[48], 17}, {NULL}};
const Segment SGI286159[] = {{&S90189[67], 18}, {NULL}};
const Segment SGI286160[] = {{&S90190[104], 4}, {NULL}};
const Segment SGI286161[] = {{&S90190[13], 14}, {&S90190[28], 6}, {&S90190[55], 17}, {NULL}};
const Segment SGI286162[] = {{&S90190[74], 18}, {NULL}};
const Segment SGI286163[] = {{&S90192[75], 4}, {NULL}};
const Segment SGI286164[] = {{&S90192[13], 14}, {&S90192[49], 17}, {NULL}};
const Segment SGI286165[] = {{&S90192[111], 3}, {NULL}};
const Segment SGI286166[] = {{&S84772[7], 3}, {NULL}};
const Segment SGI286167[] = {{&S84772[11], 5}, {NULL}};
const Segment SGI286168[] = {{&S84773[7], 3}, {NULL}};
const Segment SGI286169[] = {{&S84773[11], 8}, {NULL}};
const Segment SGI286170[] = {{&S84774[7], 3}, {NULL}};
const Segment SGI286171[] = {{&S84774[11], 6}, {NULL}};
const Segment SGI286172[] = {{&S99065[93], 3}, {NULL}};
const Segment SGI286173[] = {{&S99065[13], 7}, {&S99065[21], 2}, {&S99065[24], 15}, {&S99065[40], 6}, {&S99065[67], 17}, {NULL}};
const Segment SGI286174[] = {{&S90379[113], 4}, {NULL}};
const Segment SGI286175[] = {{&S90379[13], 9}, {&S90379[23], 22}, {&S90379[66], 17}, {NULL}};
const Segment SGI286176[] = {{&S90379[94], 3}, {NULL}};
const Segment SGI286177[] = {{&S90380[113], 4}, {NULL}};
const Segment SGI286178[] = {{&S90380[13], 9}, {&S90380[23], 22}, {&S90380[67], 17}, {NULL}};
const Segment SGI286179[] = {{&S90380[95], 3}, {NULL}};
const Segment SGI286180[] = {{&S90381[112], 4}, {NULL}};
const Segment SGI286181[] = {{&S90381[13], 9}, {&S90381[23], 22}, {&S90381[66], 17}, {NULL}};
const Segment SGI286182[] = {{&S90381[94], 3}, {NULL}};
const Segment SGI286183[] = {{&S90382[113], 4}, {NULL}};
const Segment SGI286184[] = {{&S90382[13], 9}, {&S90382[23], 22}, {&S90382[66], 17}, {NULL}};
const Segment SGI286185[] = {{&S90382[94], 3}, {NULL}};
const Segment SGI286186[] = {{&S90383[112], 4}, {NULL}};
const Segment SGI286187[] = {{&S90383[13], 9}, {&S90383[23], 22}, {&S90383[66], 17}, {NULL}};
const Segment SGI286188[] = {{&S90383[94], 3}, {NULL}};
const Segment SGI286189[] = {{&S90384[113], 4}, {NULL}};
const Segment SGI286190[] = {{&S90384[13], 9}, {&S90384[23], 22}, {&S90384[66], 17}, {NULL}};
const Segment SGI286191[] = {{&S90384[94], 3}, {NULL}};
const Segment SGI286192[] = {{&S90428[73], 4}, {NULL}};
const Segment SGI286193[] = {{&S90428[13], 3}, {&S90428[17], 8}, {&S90428[47], 17}, {NULL}};
const Segment SGI286194[] = {{&S90428[108], 3}, {NULL}};
const Segment SGI286195[] = {{&S84949[7], 3}, {NULL}};
const Segment SGI286196[] = {{&S84949[11], 5}, {NULL}};
const Segment SGI286197[] = {{&S90499[90], 4}, {NULL}};
const Segment SGI286198[] = {{&S90499[25], 3}, {&S90499[29], 13}, {&S90499[64], 17}, {NULL}};
const Segment SGI286199[] = {{&S90499[0], 12}, {NULL}};
const Segment SGI286200[] = {{&S90500[91], 4}, {NULL}};
const Segment SGI286201[] = {{&S90500[26], 3}, {&S90500[30], 13}, {&S90500[65], 17}, {NULL}};
const Segment SGI286202[] = {{&S90500[0], 13}, {NULL}};
const Segment SGI286203[] = {{&S90573[90], 4}, {NULL}};
const Segment SGI286204[] = {{&S90573[13], 3}, {&S90573[17], 11}, {&S90573[49], 17}, {NULL}};
const Segment SGI286205[] = {{&S90573[68], 12}, {NULL}};
const Segment SGI286206[] = {{&S90574[92], 4}, {NULL}};
const Segment SGI286207[] = {{&S90574[13], 3}, {&S90574[17], 13}, {&S90574[51], 17}, {NULL}};
const Segment SGI286208[] = {{&S90574[70], 12}, {NULL}};
const Segment SGI286209[] = {{&S85040[7], 3}, {NULL}};
const Segment SGI286210[] = {{&S85040[11], 5}, {NULL}};
const Segment SGI286211[] = {{&S90670[136], 4}, {NULL}};
const Segment SGI286212[] = {{&S90670[13], 3}, {&S90670[17], 13}, {&S90670[51], 17}, {&S90670[81], 44}, {NULL}};
const Segment SGI286213[] = {{&S90671[136], 4}, {NULL}};
const Segment SGI286214[] = {{&S90671[13], 3}, {&S90671[17], 13}, {&S90671[51], 17}, {&S90671[81], 44}, {NULL}};
const Segment SGI286215[] = {{&S90672[136], 4}, {NULL}};
const Segment SGI286216[] = {{&S90672[13], 3}, {&S90672[17], 13}, {&S90672[51], 17}, {&S90672[81], 44}, {NULL}};
const Segment SGI286217[] = {{&S90673[136], 4}, {NULL}};
const Segment SGI286218[] = {{&S90673[13], 3}, {&S90673[17], 13}, {&S90673[51], 17}, {&S90673[81], 44}, {NULL}};
const Segment SGI286219[] = {{&S90674[136], 4}, {NULL}};
const Segment SGI286220[] = {{&S90674[13], 3}, {&S90674[17], 13}, {&S90674[51], 17}, {&S90674[81], 44}, {NULL}};
const Segment SGI286221[] = {{&S90675[136], 4}, {NULL}};
const Segment SGI286222[] = {{&S90675[13], 3}, {&S90675[17], 13}, {&S90675[51], 17}, {&S90675[81], 44}, {NULL}};
const Segment SGI286223[] = {{&S90676[136], 4}, {NULL}};
const Segment SGI286224[] = {{&S90676[13], 3}, {&S90676[17], 13}, {&S90676[51], 17}, {&S90676[81], 44}, {NULL}};
const Segment SGI286225[] = {{&S90677[136], 4}, {NULL}};
const Segment SGI286226[] = {{&S90677[13], 3}, {&S90677[17], 13}, {&S90677[51], 17}, {&S90677[81], 44}, {NULL}};
const Segment SGI286227[] = {{&S90678[136], 4}, {NULL}};
const Segment SGI286228[] = {{&S90678[13], 3}, {&S90678[17], 13}, {&S90678[51], 17}, {&S90678[81], 44}, {NULL}};
const Segment SGI286229[] = {{&S90679[136], 4}, {NULL}};
const Segment SGI286230[] = {{&S90679[13], 3}, {&S90679[17], 13}, {&S90679[51], 17}, {&S90679[81], 44}, {NULL}};
const Segment SGI286231[] = {{&S90680[136], 4}, {NULL}};
const Segment SGI286232[] = {{&S90680[13], 3}, {&S90680[17], 13}, {&S90680[51], 17}, {&S90680[81], 44}, {NULL}};
const Segment SGI286233[] = {{&S90681[136], 4}, {NULL}};
const Segment SGI286234[] = {{&S90681[13], 3}, {&S90681[17], 13}, {&S90681[51], 17}, {&S90681[81], 44}, {NULL}};
const Segment SGI286235[] = {{&S90682[136], 4}, {NULL}};
const Segment SGI286236[] = {{&S90682[13], 3}, {&S90682[17], 13}, {&S90682[51], 17}, {&S90682[81], 44}, {NULL}};
const Segment SGI286237[] = {{&S90683[136], 4}, {NULL}};
const Segment SGI286238[] = {{&S90683[13], 3}, {&S90683[17], 13}, {&S90683[51], 17}, {&S90683[81], 44}, {NULL}};
const Segment SGI286239[] = {{&S90684[136], 4}, {NULL}};
const Segment SGI286240[] = {{&S90684[13], 3}, {&S90684[17], 13}, {&S90684[51], 17}, {&S90684[81], 44}, {NULL}};
const Segment SGI286241[] = {{&S90685[136], 4}, {NULL}};
const Segment SGI286242[] = {{&S90685[13], 3}, {&S90685[17], 13}, {&S90685[51], 17}, {&S90685[81], 44}, {NULL}};
const Segment SGI286243[] = {{&S90686[136], 4}, {NULL}};
const Segment SGI286244[] = {{&S90686[13], 3}, {&S90686[17], 13}, {&S90686[51], 17}, {&S90686[81], 44}, {NULL}};
const Segment SGI286245[] = {{&S90687[136], 4}, {NULL}};
const Segment SGI286246[] = {{&S90687[13], 3}, {&S90687[17], 13}, {&S90687[51], 17}, {&S90687[81], 44}, {NULL}};
const Segment SGI286247[] = {{&S90688[136], 4}, {NULL}};
const Segment SGI286248[] = {{&S90688[13], 3}, {&S90688[17], 13}, {&S90688[51], 17}, {&S90688[81], 44}, {NULL}};
const Segment SGI286249[] = {{&S90689[136], 4}, {NULL}};
const Segment SGI286250[] = {{&S90689[13], 3}, {&S90689[17], 13}, {&S90689[51], 17}, {&S90689[81], 44}, {NULL}};
const Segment SGI286251[] = {{&S90690[136], 4}, {NULL}};
const Segment SGI286252[] = {{&S90690[13], 3}, {&S90690[17], 13}, {&S90690[51], 17}, {&S90690[81], 44}, {NULL}};
const Segment SGI286253[] = {{&S90691[136], 4}, {NULL}};
const Segment SGI286254[] = {{&S90691[13], 3}, {&S90691[17], 13}, {&S90691[51], 17}, {&S90691[81], 44}, {NULL}};
const Segment SGI286255[] = {{&S90692[136], 4}, {NULL}};
const Segment SGI286256[] = {{&S90692[13], 3}, {&S90692[17], 13}, {&S90692[51], 17}, {&S90692[81], 44}, {NULL}};
const Segment SGI286257[] = {{&S90693[136], 4}, {NULL}};
const Segment SGI286258[] = {{&S90693[13], 3}, {&S90693[17], 13}, {&S90693[51], 17}, {&S90693[81], 44}, {NULL}};
const Segment SGI286259[] = {{&S90694[136], 4}, {NULL}};
const Segment SGI286260[] = {{&S90694[13], 3}, {&S90694[17], 13}, {&S90694[51], 17}, {&S90694[81], 44}, {NULL}};
const Segment SGI286261[] = {{&S90695[136], 4}, {NULL}};
const Segment SGI286262[] = {{&S90695[13], 3}, {&S90695[17], 13}, {&S90695[51], 17}, {&S90695[81], 44}, {NULL}};
const Segment SGI286263[] = {{&S90696[136], 4}, {NULL}};
const Segment SGI286264[] = {{&S90696[13], 3}, {&S90696[17], 13}, {&S90696[51], 17}, {&S90696[81], 44}, {NULL}};
const Segment SGI286265[] = {{&S90697[136], 4}, {NULL}};
const Segment SGI286266[] = {{&S90697[13], 3}, {&S90697[17], 13}, {&S90697[51], 17}, {&S90697[81], 44}, {NULL}};
const Segment SGI286267[] = {{&S90698[136], 4}, {NULL}};
const Segment SGI286268[] = {{&S90698[13], 3}, {&S90698[17], 13}, {&S90698[51], 17}, {&S90698[81], 44}, {NULL}};
const Segment SGI286269[] = {{&S90699[136], 4}, {NULL}};
const Segment SGI286270[] = {{&S90699[13], 3}, {&S90699[17], 13}, {&S90699[51], 17}, {&S90699[81], 44}, {NULL}};
const Segment SGI286271[] = {{&S90700[136], 4}, {NULL}};
const Segment SGI286272[] = {{&S90700[13], 3}, {&S90700[17], 13}, {&S90700[51], 17}, {&S90700[81], 44}, {NULL}};
const Segment SGI286273[] = {{&S90701[136], 4}, {NULL}};
const Segment SGI286274[] = {{&S90701[13], 3}, {&S90701[17], 13}, {&S90701[51], 17}, {&S90701[81], 44}, {NULL}};
const Segment SGI286275[] = {{&S90702[136], 4}, {NULL}};
const Segment SGI286276[] = {{&S90702[13], 3}, {&S90702[17], 13}, {&S90702[51], 17}, {&S90702[81], 44}, {NULL}};
const Segment SGI286277[] = {{&S90703[136], 4}, {NULL}};
const Segment SGI286278[] = {{&S90703[13], 3}, {&S90703[17], 13}, {&S90703[51], 17}, {&S90703[81], 44}, {NULL}};
const Segment SGI286279[] = {{&S90704[136], 4}, {NULL}};
const Segment SGI286280[] = {{&S90704[13], 3}, {&S90704[17], 13}, {&S90704[51], 17}, {&S90704[81], 44}, {NULL}};
const Segment SGI286281[] = {{&S90705[136], 4}, {NULL}};
const Segment SGI286282[] = {{&S90705[13], 3}, {&S90705[17], 13}, {&S90705[51], 17}, {&S90705[81], 44}, {NULL}};
const Segment SGI286283[] = {{&S90706[136], 4}, {NULL}};
const Segment SGI286284[] = {{&S90706[13], 3}, {&S90706[17], 13}, {&S90706[51], 17}, {&S90706[81], 44}, {NULL}};
const Segment SGI286285[] = {{&S90707[136], 4}, {NULL}};
const Segment SGI286286[] = {{&S90707[13], 3}, {&S90707[17], 13}, {&S90707[51], 17}, {&S90707[81], 44}, {NULL}};
const Segment SGI286287[] = {{&S90708[136], 4}, {NULL}};
const Segment SGI286288[] = {{&S90708[13], 3}, {&S90708[17], 13}, {&S90708[51], 17}, {&S90708[81], 44}, {NULL}};
const Segment SGI286289[] = {{&S90709[136], 4}, {NULL}};
const Segment SGI286290[] = {{&S90709[13], 3}, {&S90709[17], 13}, {&S90709[51], 17}, {&S90709[81], 44}, {NULL}};
const Segment SGI286291[] = {{&S90758[97], 4}, {NULL}};
const Segment SGI286292[] = {{&S90758[13], 14}, {&S90758[48], 17}, {NULL}};
const Segment SGI286293[] = {{&S90758[67], 18}, {NULL}};
const Segment SGI286294[] = {{&S90759[104], 4}, {NULL}};
const Segment SGI286295[] = {{&S90759[13], 14}, {&S90759[28], 6}, {&S90759[55], 17}, {NULL}};
const Segment SGI286296[] = {{&S90759[74], 18}, {NULL}};
const Segment SGI286297[] = {{&S90761[82], 4}, {NULL}};
const Segment SGI286298[] = {{&S90761[13], 14}, {&S90761[49], 17}, {NULL}};
const Segment SGI286299[] = {{&S90761[68], 18}, {NULL}};
const Segment SGI286300[] = {{&S90762[89], 4}, {NULL}};
const Segment SGI286301[] = {{&S90762[13], 14}, {&S90762[28], 6}, {&S90762[56], 17}, {NULL}};
const Segment SGI286302[] = {{&S90762[75], 18}, {NULL}};
const Segment SGI286303[] = {{&S90765[75], 4}, {NULL}};
const Segment SGI286304[] = {{&S90765[13], 14}, {&S90765[49], 17}, {NULL}};
const Segment SGI286305[] = {{&S90765[103], 18}, {NULL}};
const Segment SGI286306[] = {{&S90766[82], 4}, {NULL}};
const Segment SGI286307[] = {{&S90766[13], 14}, {&S90766[28], 6}, {&S90766[56], 17}, {NULL}};
const Segment SGI286308[] = {{&S90766[110], 18}, {NULL}};
const Segment SGI286309[] = {{&S99622[8], 3}, {NULL}};
const Segment SGI286310[] = {{&S99622[13], 3}, {&S99622[17], 2}, {&S99622[20], 16}, {&S99622[37], 6}, {&S99622[64], 17}, {NULL}};
const Segment SGI286311[] = {{&S85338[7], 3}, {NULL}};
const Segment SGI286312[] = {{&S85338[11], 5}, {NULL}};
const Segment SGI286313[] = {{&S85339[7], 3}, {NULL}};
const Segment SGI286314[] = {{&S85339[11], 5}, {NULL}};
const Segment SGI286315[] = {{&S85341[7], 3}, {NULL}};
const Segment SGI286316[] = {{&S85341[11], 4}, {NULL}};
const Segment SGI286317[] = {{&S90949[91], 4}, {NULL}};
const Segment SGI286318[] = {{&S90949[13], 9}, {&S90949[23], 22}, {&S90949[67], 17}, {NULL}};
const Segment SGI286319[] = {{&S90949[86], 9}, {NULL}};
const Segment SGI286320[] = {{&S90997[108], 4}, {NULL}};
const Segment SGI286321[] = {{&S90997[13], 3}, {&S90997[17], 13}, {&S90997[31], 22}, {&S90997[54], 6}, {&S90997[82], 17}, {NULL}};
const Segment SGI286322[] = {{&S85429[7], 3}, {NULL}};
const Segment SGI286323[] = {{&S85429[11], 5}, {NULL}};
const Segment SGI286324[] = {{&S90998[108], 4}, {NULL}};
const Segment SGI286325[] = {{&S90998[13], 3}, {&S90998[17], 13}, {&S90998[31], 22}, {&S90998[54], 6}, {&S90998[82], 17}, {NULL}};
const Segment SGI286326[] = {{&S4177[8], 3}, {NULL}};
const Segment SGI286327[] = {{&S4177[13], 7}, {NULL}};
const Segment SGI286328[] = {{&S4177[25], 14}, {NULL}};
const Segment SGI286329[] = {{&S4177[11], 1}, {NULL}};
const Segment SGI286330[] = {{&S4177[16], 4}, {&S4177[28], 11}, {NULL}};
const Segment SGI286331[] = {{&S4178[73], 4}, {NULL}};
const Segment SGI286332[] = {{&S4178[8], 3}, {NULL}};
const Segment SGI286333[] = {{&S4178[13], 7}, {NULL}};
const Segment SGI286334[] = {{&S4178[25], 14}, {NULL}};
const Segment SGI286335[] = {{&S4178[7], 1}, {NULL}};
const Segment SGI286336[] = {{&S4178[16], 4}, {&S4178[28], 11}, {NULL}};
const Segment SGI286337[] = {{&S4179[76], 4}, {NULL}};
const Segment SGI286338[] = {{&S4179[8], 3}, {NULL}};
const Segment SGI286339[] = {{&S4179[13], 7}, {NULL}};
const Segment SGI286340[] = {{&S4179[25], 14}, {NULL}};
const Segment SGI286341[] = {{&S4179[7], 1}, {NULL}};
const Segment SGI286342[] = {{&S4179[16], 4}, {&S4179[28], 11}, {&S4179[87], 17}, {NULL}};
const Segment SGI286343[] = {{&S4180[70], 4}, {NULL}};
const Segment SGI286344[] = {{&S4180[8], 3}, {NULL}};
const Segment SGI286345[] = {{&S4180[13], 7}, {NULL}};
const Segment SGI286346[] = {{&S4180[25], 14}, {NULL}};
const Segment SGI286347[] = {{&S4180[7], 1}, {&S4180[11], 1}, {NULL}};
const Segment SGI286348[] = {{&S4180[16], 4}, {&S4180[28], 11}, {NULL}};
const Segment SGI286349[] = {{&S4181[73], 4}, {NULL}};
const Segment SGI286350[] = {{&S4181[8], 3}, {NULL}};
const Segment SGI286351[] = {{&S4181[13], 7}, {NULL}};
const Segment SGI286352[] = {{&S4181[25], 14}, {NULL}};
const Segment SGI286353[] = {{&S4181[7], 1}, {NULL}};
const Segment SGI286354[] = {{&S4181[16], 4}, {&S4181[28], 11}, {NULL}};
const Segment SGI286355[] = {{&S4182[56], 4}, {NULL}};
const Segment SGI286356[] = {{&S4182[8], 3}, {NULL}};
const Segment SGI286357[] = {{&S4182[13], 14}, {NULL}};
const Segment SGI286358[] = {{&S4182[7], 1}, {NULL}};
const Segment SGI286359[] = {{&S4182[16], 11}, {NULL}};
const Segment SGI286360[] = {{&S91065[74], 4}, {NULL}};
const Segment SGI286361[] = {{&S91065[13], 14}, {&S91065[48], 17}, {NULL}};
const Segment SGI286362[] = {{&S91065[108], 3}, {NULL}};
const Segment SGI286363[] = {{&S85558[7], 3}, {NULL}};
const Segment SGI286364[] = {{&S85558[11], 4}, {NULL}};
const Segment SGI286365[] = {{&S85562[7], 3}, {NULL}};
const Segment SGI286366[] = {{&S85562[11], 5}, {NULL}};
const Segment SGI286367[] = {{&S85563[7], 3}, {NULL}};
const Segment SGI286368[] = {{&S85563[11], 5}, {NULL}};
const Segment SGI286369[] = {{&S85564[7], 3}, {NULL}};
const Segment SGI286370[] = {{&S85564[11], 5}, {NULL}};
const Segment SGI286371[] = {{&S91136[74], 4}, {NULL}};
const Segment SGI286372[] = {{&S91136[13], 14}, {&S91136[48], 17}, {NULL}};
const Segment SGI286373[] = {{&S91136[100], 18}, {NULL}};
const Segment SGI286374[] = {{&S91137[81], 4}, {NULL}};
const Segment SGI286375[] = {{&S91137[13], 14}, {&S91137[28], 6}, {&S91137[55], 17}, {NULL}};
const Segment SGI286376[] = {{&S91137[107], 18}, {NULL}};
const Segment SGI286377[] = {{&S91166[79], 4}, {NULL}};
const Segment SGI286378[] = {{&S91166[13], 14}, {&S91166[28], 6}, {&S91166[55], 17}, {NULL}};
const Segment SGI286379[] = {{&S91166[74], 9}, {NULL}};
const Segment SGI286380[] = {{&S100189[93], 3}, {NULL}};
const Segment SGI286381[] = {{&S100189[13], 7}, {&S100189[21], 2}, {&S100189[24], 15}, {&S100189[40], 6}, {&S100189[67], 17}, {NULL}};
const Segment SGI286382[] = {{&S91491[75], 4}, {NULL}};
const Segment SGI286383[] = {{&S91491[13], 14}, {&S91491[49], 17}, {NULL}};
const Segment SGI286384[] = {{&S91491[111], 3}, {NULL}};
const Segment SGI286385[] = {{&S91497[81], 4}, {NULL}};
const Segment SGI286386[] = {{&S91497[13], 3}, {&S91497[17], 16}, {&S91497[55], 17}, {NULL}};
const Segment SGI286387[] = {{&S91498[84], 4}, {NULL}};
const Segment SGI286388[] = {{&S91498[13], 3}, {&S91498[17], 19}, {&S91498[58], 17}, {NULL}};
const Segment SGI286389[] = {{&S91499[91], 4}, {NULL}};
const Segment SGI286390[] = {{&S91499[13], 3}, {&S91499[17], 2}, {&S91499[20], 16}, {&S91499[37], 6}, {&S91499[65], 17}, {NULL}};
const Segment SGI286391[] = {{&S91319[92], 4}, {NULL}};
const Segment SGI286392[] = {{&S91319[13], 3}, {&S91319[17], 13}, {&S91319[51], 17}, {NULL}};
const Segment SGI286393[] = {{&S91319[70], 12}, {NULL}};
const Segment SGI286394[] = {{&S91320[90], 4}, {NULL}};
const Segment SGI286395[] = {{&S91320[13], 3}, {&S91320[17], 11}, {&S91320[49], 17}, {NULL}};
const Segment SGI286396[] = {{&S91320[68], 12}, {NULL}};
const Segment SGI286397[] = {{&S91693[92], 4}, {NULL}};
const Segment SGI286398[] = {{&S91693[13], 3}, {&S91693[17], 13}, {&S91693[51], 17}, {NULL}};
const Segment SGI286399[] = {{&S91693[70], 12}, {NULL}};
const Segment SGI286400[] = {{&S91694[90], 4}, {NULL}};
const Segment SGI286401[] = {{&S91694[13], 3}, {&S91694[17], 11}, {&S91694[49], 17}, {NULL}};
const Segment SGI286402[] = {{&S91694[68], 12}, {NULL}};
const Segment SGI286403[] = {{&S91695[92], 4}, {NULL}};
const Segment SGI286404[] = {{&S91695[13], 3}, {&S91695[17], 13}, {&S91695[51], 17}, {NULL}};
const Segment SGI286405[] = {{&S91695[70], 12}, {NULL}};
const Segment SGI286406[] = {{&S91744[86], 4}, {NULL}};
const Segment SGI286407[] = {{&S91744[21], 3}, {&S91744[25], 13}, {&S91744[60], 17}, {NULL}};
const Segment SGI286408[] = {{&S91744[0], 8}, {NULL}};
const Segment SGI286409[] = {{&S86032[7], 3}, {NULL}};
const Segment SGI286410[] = {{&S86032[11], 5}, {NULL}};
const Segment SGI286411[] = {{&S86109[7], 3}, {NULL}};
const Segment SGI286412[] = {{&S86109[11], 4}, {NULL}};
const Segment SGI286413[] = {{&S86113[7], 3}, {NULL}};
const Segment SGI286414[] = {{&S86113[11], 4}, {NULL}};
const Segment SGI286415[] = {{&S86115[7], 3}, {NULL}};
const Segment SGI286416[] = {{&S86115[11], 5}, {NULL}};
const Segment SGI286417[] = {{&S86116[7], 3}, {NULL}};
const Segment SGI286418[] = {{&S86116[11], 8}, {NULL}};
const Segment SGI286419[] = {{&S100766[8], 3}, {NULL}};
const Segment SGI286420[] = {{&S100766[13], 3}, {&S100766[17], 2}, {&S100766[20], 11}, {&S100766[32], 6}, {&S100766[59], 17}, {NULL}};
const Segment SGI286421[] = {{&S100766[78], 8}, {NULL}};
const Segment SGI286422[] = {{&S86323[7], 3}, {NULL}};
const Segment SGI286423[] = {{&S86323[11], 4}, {NULL}};
const Segment SGI286424[] = {{&S92101[79], 4}, {NULL}};
const Segment SGI286425[] = {{&S92101[13], 3}, {&S92101[17], 14}, {&S92101[53], 17}, {NULL}};
const Segment SGI286426[] = {{&S92102[78], 4}, {NULL}};
const Segment SGI286427[] = {{&S92102[13], 3}, {&S92102[17], 13}, {&S92102[52], 17}, {NULL}};
const Segment SGI286428[] = {{&S92163[95], 4}, {NULL}};
const Segment SGI286429[] = {{&S92163[13], 14}, {&S92163[48], 17}, {NULL}};
const Segment SGI286430[] = {{&S92163[76], 4}, {NULL}};
const Segment SGI286431[] = {{&S86549[7], 3}, {NULL}};
const Segment SGI286432[] = {{&S86549[11], 4}, {NULL}};
const Segment SGI286433[] = {{&S101014[94], 3}, {NULL}};
const Segment SGI286434[] = {{&S101014[13], 7}, {&S101014[21], 2}, {&S101014[24], 16}, {&S101014[41], 6}, {&S101014[68], 17}, {NULL}};
const Segment SGI286435[] = {{&S92260[104], 4}, {NULL}};
const Segment SGI286436[] = {{&S92260[13], 14}, {&S92260[28], 6}, {&S92260[55], 17}, {NULL}};
const Segment SGI286437[] = {{&S92260[74], 18}, {NULL}};
const Segment SGI286438[] = {{&S92261[97], 4}, {NULL}};
const Segment SGI286439[] = {{&S92261[13], 14}, {&S92261[48], 17}, {NULL}};
const Segment SGI286440[] = {{&S92261[67], 18}, {NULL}};
const Segment SGI286441[] = {{&S86658[7], 3}, {NULL}};
const Segment SGI286442[] = {{&S86658[11], 6}, {NULL}};
const Segment SGI286443[] = {{&S86659[7], 3}, {NULL}};
const Segment SGI286444[] = {{&S86659[11], 4}, {NULL}};
const Segment SGI286445[] = {{&S86894[7], 3}, {NULL}};
const Segment SGI286446[] = {{&S86894[11], 5}, {NULL}};
const Segment SGI286447[] = {{&S5618[83], 4}, {NULL}};
const Segment SGI286448[] = {{&S5618[8], 3}, {NULL}};
const Segment SGI286449[] = {{&S5618[13], 7}, {NULL}};
const Segment SGI286450[] = {{&S5618[25], 14}, {NULL}};
const Segment SGI286451[] = {{&S5618[7], 1}, {NULL}};
const Segment SGI286452[] = {{&S5618[16], 4}, {&S5618[28], 11}, {NULL}};
const Segment SGI286453[] = {{&S5619[69], 4}, {NULL}};
const Segment SGI286454[] = {{&S5619[8], 3}, {NULL}};
const Segment SGI286455[] = {{&S5619[13], 3}, {NULL}};
const Segment SGI286456[] = {{&S5619[13], 3}, {&S5619[21], 14}, {NULL}};
const Segment SGI286457[] = {{&S5619[7], 1}, {NULL}};
const Segment SGI286458[] = {{&S5619[24], 11}, {NULL}};
const Segment SGI286459[] = {{&S5620[211], 4}, {NULL}};
const Segment SGI286460[] = {{&S5620[8], 3}, {&S5620[8], 3}, {&S5620[147], 3}, {NULL}};
const Segment SGI286461[] = {{&S5620[13], 10}, {&S5620[13], 10}, {&S5620[35], 7}, {NULL}};
const Segment SGI286462[] = {{&S5620[35], 14}, {&S5620[118], 14}, {&S5620[164], 14}, {NULL}};
const Segment SGI286463[] = {{&S5620[7], 1}, {NULL}};
const Segment SGI286464[] = {{&S5620[38], 11}, {&S5620[121], 11}, {&S5620[38], 4}, {&S5620[167], 11}, {NULL}};
const Segment SGI286465[] = {{&S5621[75], 4}, {NULL}};
const Segment SGI286466[] = {{&S5621[8], 3}, {NULL}};
const Segment SGI286467[] = {{&S5621[13], 7}, {NULL}};
const Segment SGI286468[] = {{&S5621[25], 14}, {NULL}};
const Segment SGI286469[] = {{&S5621[7], 1}, {NULL}};
const Segment SGI286470[] = {{&S5621[16], 4}, {&S5621[28], 11}, {&S5621[117], 14}, {NULL}};
const Segment SGI286471[] = {{&S5622[73], 4}, {&S5622[7], 4}, {NULL}};
const Segment SGI286472[] = {{&S5622[8], 3}, {NULL}};
const Segment SGI286473[] = {{&S5622[13], 7}, {NULL}};
const Segment SGI286474[] = {{&S5622[25], 14}, {&S5622[82], 14}, {NULL}};
const Segment SGI286475[] = {{&S5622[7], 1}, {&S5622[7], 1}, {NULL}};
const Segment SGI286476[] = {{&S5622[16], 4}, {&S5622[28], 11}, {&S5622[85], 11}, {NULL}};
const Segment SGI286477[] = {{&S5623[69], 4}, {NULL}};
const Segment SGI286478[] = {{&S5623[8], 3}, {NULL}};
const Segment SGI286479[] = {{&S5623[22], 14}, {NULL}};
const Segment SGI286480[] = {{&S5623[7], 1}, {NULL}};
const Segment SGI286481[] = {{&S5623[15], 4}, {&S5623[25], 11}, {NULL}};
const Segment SGI286482[] = {{&S92672[81], 4}, {NULL}};
const Segment SGI286483[] = {{&S92672[13], 14}, {&S92672[48], 17}, {NULL}};
const Segment SGI286484[] = {{&S92672[67], 18}, {NULL}};
const Segment SGI286485[] = {{&S92673[88], 4}, {NULL}};
const Segment SGI286486[] = {{&S92673[13], 14}, {&S92673[28], 6}, {&S92673[55], 17}, {NULL}};
const Segment SGI286487[] = {{&S92673[74], 18}, {NULL}};
const Segment SGI286488[] = {{&S92674[81], 4}, {NULL}};
const Segment SGI286489[] = {{&S92674[13], 14}, {&S92674[48], 17}, {NULL}};
const Segment SGI286490[] = {{&S92674[67], 18}, {NULL}};
const Segment SGI286491[] = {{&S92675[88], 4}, {NULL}};
const Segment SGI286492[] = {{&S92675[13], 14}, {&S92675[28], 6}, {&S92675[55], 17}, {NULL}};
const Segment SGI286493[] = {{&S92675[74], 18}, {NULL}};
const Segment SGI286494[] = {{&S101368[99], 3}, {NULL}};
const Segment SGI286495[] = {{&S101368[13], 3}, {&S101368[17], 2}, {&S101368[20], 11}, {&S101368[32], 6}, {&S101368[59], 17}, {NULL}};
const Segment SGI286496[] = {{&S101368[78], 11}, {NULL}};
const Segment SGI286497[] = {{&S87039[7], 3}, {NULL}};
const Segment SGI286498[] = {{&S87039[11], 4}, {NULL}};
const Segment SGI286499[] = {{&S92511[74], 4}, {NULL}};
const Segment SGI286500[] = {{&S92511[13], 14}, {&S92511[48], 17}, {NULL}};
const Segment SGI286501[] = {{&S92511[100], 12}, {NULL}};
const Segment SGI286502[] = {{&S101539[103], 3}, {NULL}};
const Segment SGI286503[] = {{&S101539[13], 9}, {&S101539[23], 2}, {&S101539[26], 22}, {&S101539[49], 6}, {&S101539[77], 17}, {NULL}};
const Segment SGI286504[] = {{&S92884[85], 4}, {NULL}};
const Segment SGI286505[] = {{&S92884[13], 9}, {&S92884[23], 15}, {&S92884[59], 17}, {NULL}};
const Segment SGI286506[] = {{&S101672[102], 3}, {NULL}};
const Segment SGI286507[] = {{&S101672[13], 9}, {&S101672[23], 2}, {&S101672[26], 22}, {&S101672[49], 6}, {&S101672[76], 17}, {NULL}};
const Segment SGI286508[] = {{&S93071[81], 4}, {NULL}};
const Segment SGI286509[] = {{&S93071[13], 14}, {&S93071[28], 6}, {&S93071[55], 17}, {NULL}};
const Segment SGI286510[] = {{&S93071[107], 18}, {NULL}};
const Segment SGI286511[] = {{&S93072[74], 4}, {NULL}};
const Segment SGI286512[] = {{&S93072[13], 14}, {&S93072[48], 17}, {NULL}};
const Segment SGI286513[] = {{&S93072[100], 18}, {NULL}};
const Segment SGI286514[] = {{&S101834[87], 3}, {NULL}};
const Segment SGI286515[] = {{&S101834[13], 3}, {&S101834[17], 2}, {&S101834[20], 13}, {&S101834[34], 6}, {&S101834[61], 17}, {NULL}};
const Segment SGI286516[] = {{&S101835[88], 3}, {NULL}};
const Segment SGI286517[] = {{&S101835[13], 3}, {&S101835[17], 2}, {&S101835[20], 14}, {&S101835[35], 6}, {&S101835[62], 17}, {NULL}};
const Segment SGI286518[] = {{&S101836[84], 3}, {NULL}};
const Segment SGI286519[] = {{&S101836[13], 3}, {&S101836[17], 2}, {&S101836[20], 11}, {&S101836[32], 6}, {&S101836[60], 17}, {NULL}};
const Segment SGI286520[] = {{&S101836[79], 8}, {NULL}};
const Segment SGI286521[] = {{&S93140[87], 4}, {NULL}};
const Segment SGI286522[] = {{&S93140[13], 3}, {&S93140[17], 13}, {&S93140[31], 8}, {&S93140[61], 17}, {NULL}};
const Segment SGI286523[] = {{&S87424[7], 3}, {NULL}};
const Segment SGI286524[] = {{&S87424[11], 4}, {NULL}};
const Segment SGI286525[] = {{&S87425[7], 3}, {NULL}};
const Segment SGI286526[] = {{&S87425[11], 4}, {NULL}};
const Segment SGI286527[] = {{&S87426[7], 3}, {NULL}};
const Segment SGI286528[] = {{&S87426[11], 4}, {NULL}};
const Segment SGI286529[] = {{&S87542[7], 3}, {NULL}};
const Segment SGI286530[] = {{&S87542[11], 4}, {NULL}};
const Segment SGI286531[] = {{&S102225[93], 3}, {NULL}};
const Segment SGI286532[] = {{&S102225[13], 7}, {&S102225[21], 2}, {&S102225[24], 15}, {&S102225[40], 6}, {&S102225[67], 17}, {NULL}};
const Segment SGI286533[] = {{&S102246[94], 3}, {NULL}};
const Segment SGI286534[] = {{&S102246[13], 7}, {&S102246[21], 2}, {&S102246[24], 15}, {&S102246[40], 6}, {&S102246[68], 17}, {NULL}};
const Segment SGI286535[] = {{&S93189[90], 4}, {NULL}};
const Segment SGI286536[] = {{&S93189[13], 3}, {&S93189[17], 11}, {&S93189[49], 17}, {NULL}};
const Segment SGI286537[] = {{&S93189[68], 12}, {NULL}};
const Segment SGI286538[] = {{&S102246[120], 9}, {NULL}};
const Segment SGI286539[] = {{&S93764[41], 4}, {NULL}};
const Segment SGI286540[] = {{&S93764[13], 14}, {&S93764[28], 25}, {&S93764[75], 17}, {NULL}};
const Segment SGI286541[] = {{&S93764[29], 16}, {NULL}};
const Segment SGI286542[] = {{&S93765[48], 4}, {NULL}};
const Segment SGI286543[] = {{&S93765[13], 14}, {&S93765[28], 6}, {&S93765[35], 25}, {&S93765[82], 17}, {NULL}};
const Segment SGI286544[] = {{&S93765[36], 16}, {NULL}};
const Segment SGI286545[] = {{&S93766[41], 4}, {NULL}};
const Segment SGI286546[] = {{&S93766[13], 14}, {&S93766[28], 25}, {&S93766[75], 17}, {NULL}};
const Segment SGI286547[] = {{&S93766[29], 16}, {NULL}};
const Segment SGI286548[] = {{&S93767[41], 4}, {NULL}};
const Segment SGI286549[] = {{&S93767[13], 14}, {&S93767[28], 25}, {&S93767[75], 17}, {NULL}};
const Segment SGI286550[] = {{&S93767[29], 16}, {NULL}};
const Segment SGI286551[] = {{&S93818[100], 4}, {NULL}};
const Segment SGI286552[] = {{&S93818[13], 3}, {&S93818[17], 13}, {&S93818[31], 14}, {&S93818[46], 6}, {&S93818[74], 17}, {NULL}};
const Segment SGI286553[] = {{&S93819[100], 4}, {NULL}};
const Segment SGI286554[] = {{&S93819[13], 3}, {&S93819[17], 13}, {&S93819[31], 14}, {&S93819[46], 6}, {&S93819[74], 17}, {NULL}};
const Segment SGI286555[] = {{&S93849[92], 4}, {NULL}};
const Segment SGI286556[] = {{&S93849[13], 7}, {&S93849[21], 2}, {&S93849[24], 15}, {&S93849[40], 6}, {&S93849[68], 17}, {NULL}};
const Segment SGI286557[] = {{&S93849[87], 9}, {NULL}};
const Segment SGI286558[] = {{&S93436[92], 4}, {NULL}};
const Segment SGI286559[] = {{&S93436[13], 9}, {&S93436[23], 22}, {&S93436[66], 17}, {NULL}};
const Segment SGI286560[] = {{&S102463[94], 3}, {NULL}};
const Segment SGI286561[] = {{&S102463[13], 7}, {&S102463[21], 2}, {&S102463[24], 15}, {&S102463[40], 6}, {&S102463[68], 17}, {NULL}};
const Segment SGI286562[] = {{&S102463[119], 15}, {NULL}};
const Segment SGI286563[] = {{&S93978[76], 4}, {NULL}};
const Segment SGI286564[] = {{&S93978[13], 3}, {&S93978[17], 13}, {&S93978[51], 17}, {NULL}};
const Segment SGI286565[] = {{&S93978[71], 9}, {NULL}};
const Segment SGI286566[] = {{&S93979[74], 4}, {NULL}};
const Segment SGI286567[] = {{&S93979[13], 3}, {&S93979[17], 11}, {&S93979[49], 17}, {NULL}};
const Segment SGI286568[] = {{&S93979[69], 9}, {NULL}};
const Segment SGI286569[] = {{&S93980[76], 4}, {NULL}};
const Segment SGI286570[] = {{&S93980[13], 3}, {&S93980[17], 13}, {&S93980[52], 17}, {NULL}};
const Segment SGI286571[] = {{&S93980[71], 9}, {NULL}};
const Segment SGI286572[] = {{&S93981[74], 4}, {NULL}};
const Segment SGI286573[] = {{&S93981[13], 3}, {&S93981[17], 11}, {&S93981[50], 17}, {NULL}};
const Segment SGI286574[] = {{&S93981[69], 9}, {NULL}};
const Segment SGI286575[] = {{&S93991[91], 4}, {NULL}};
const Segment SGI286576[] = {{&S93991[13], 3}, {&S93991[17], 2}, {&S93991[20], 16}, {&S93991[37], 6}, {&S93991[65], 17}, {NULL}};
const Segment SGI286577[] = {{&S93992[94], 4}, {NULL}};
const Segment SGI286578[] = {{&S93992[13], 3}, {&S93992[17], 2}, {&S93992[20], 19}, {&S93992[40], 6}, {&S93992[68], 17}, {NULL}};
const Segment SGI286579[] = {{&S93993[90], 4}, {NULL}};
const Segment SGI286580[] = {{&S93993[13], 9}, {&S93993[23], 22}, {&S93993[66], 17}, {NULL}};
const Segment SGI286581[] = {{&S93993[85], 9}, {NULL}};
const Segment SGI286582[] = {{&S93591[79], 4}, {NULL}};
const Segment SGI286583[] = {{&S93591[13], 3}, {&S93591[17], 14}, {&S93591[53], 17}, {NULL}};
const Segment SGI286584[] = {{&S93594[90], 4}, {NULL}};
const Segment SGI286585[] = {{&S93594[13], 3}, {&S93594[17], 11}, {&S93594[49], 17}, {NULL}};
const Segment SGI286586[] = {{&S93594[68], 12}, {NULL}};
const Segment SGI286587[] = {{&S88290[7], 3}, {NULL}};
const Segment SGI286588[] = {{&S88290[11], 4}, {NULL}};
const Segment SGI286589[] = {{&S94115[114], 4}, {NULL}};
const Segment SGI286590[] = {{&S94115[13], 3}, {&S94115[17], 13}, {&S94115[31], 6}, {&S94115[38], 28}, {&S94115[88], 17}, {NULL}};
const Segment SGI286591[] = {{&S94209[74], 4}, {NULL}};
const Segment SGI286592[] = {{&S94209[13], 3}, {&S94209[17], 11}, {&S94209[50], 17}, {NULL}};
const Segment SGI286593[] = {{&S94209[69], 9}, {NULL}};
const Segment SGI286594[] = {{&S93805[74], 4}, {NULL}};
const Segment SGI286595[] = {{&S94210[76], 4}, {NULL}};
const Segment SGI286596[] = {{&S93805[13], 14}, {&S93805[48], 17}, {NULL}};
const Segment SGI286597[] = {{&S94210[13], 3}, {&S94210[17], 13}, {&S94210[52], 17}, {NULL}};
const Segment SGI286598[] = {{&S93805[100], 18}, {NULL}};
const Segment SGI286599[] = {{&S94210[71], 9}, {NULL}};
const Segment SGI286600[] = {{&S93806[81], 4}, {NULL}};
const Segment SGI286601[] = {{&S93806[13], 14}, {&S93806[28], 6}, {&S93806[55], 17}, {NULL}};
const Segment SGI286602[] = {{&S93806[107], 18}, {NULL}};
const Segment SGI286603[] = {{&S102842[8], 3}, {NULL}};
const Segment SGI286604[] = {{&S102842[13], 7}, {&S102842[21], 2}, {&S102842[24], 15}, {&S102842[40], 6}, {&S102842[67], 17}, {NULL}};
const Segment SGI286605[] = {{&S102859[95], 3}, {NULL}};
const Segment SGI286606[] = {{&S102859[13], 7}, {&S102859[21], 2}, {&S102859[24], 16}, {&S102859[41], 6}, {&S102859[69], 17}, {NULL}};
const Segment SGI286607[] = {{&S102885[93], 3}, {NULL}};
const Segment SGI286608[] = {{&S102885[13], 7}, {&S102885[21], 2}, {&S102885[24], 15}, {&S102885[40], 6}, {&S102885[67], 17}, {NULL}};
const Segment SGI286609[] = {{&S94343[113], 4}, {NULL}};
const Segment SGI286610[] = {{&S94343[13], 9}, {&S94343[23], 22}, {&S94343[66], 17}, {NULL}};
const Segment SGI286611[] = {{&S94343[94], 4}, {NULL}};
const Segment SGI286612[] = {{&S88582[7], 3}, {NULL}};
const Segment SGI286613[] = {{&S88582[11], 4}, {NULL}};
const Segment SGI286614[] = {{&S88583[7], 3}, {NULL}};
const Segment SGI286615[] = {{&S88583[11], 4}, {NULL}};
const Segment SGI286616[] = {{&S88584[7], 3}, {NULL}};
const Segment SGI286617[] = {{&S88584[11], 4}, {NULL}};
const Segment SGI286618[] = {{&S88585[7], 3}, {NULL}};
const Segment SGI286619[] = {{&S88585[11], 5}, {NULL}};
const Segment SGI286620[] = {{&S94423[98], 4}, {NULL}};
const Segment SGI286621[] = {{&S94423[13], 14}, {&S94423[49], 17}, {NULL}};
const Segment SGI286622[] = {{&S94423[68], 18}, {NULL}};
const Segment SGI286623[] = {{&S88695[7], 3}, {NULL}};
const Segment SGI286624[] = {{&S88695[11], 4}, {NULL}};
const Segment SGI286625[] = {{&S88696[7], 3}, {NULL}};
const Segment SGI286626[] = {{&S88696[11], 5}, {NULL}};
const Segment SGI286627[] = {{&S88700[7], 3}, {NULL}};
const Segment SGI286628[] = {{&S88700[11], 4}, {NULL}};
const Segment SGI286629[] = {{&S88701[7], 3}, {NULL}};
const Segment SGI286630[] = {{&S88701[11], 4}, {NULL}};
const Segment SGI286631[] = {{&S103125[87], 3}, {NULL}};
const Segment SGI286632[] = {{&S103125[13], 3}, {&S103125[17], 2}, {&S103125[20], 13}, {&S103125[34], 6}, {&S103125[61], 17}, {NULL}};
const Segment SGI286633[] = {{&S88750[7], 3}, {NULL}};
const Segment SGI286634[] = {{&S88750[11], 4}, {NULL}};
const Segment SGI286635[] = {{&S94001[82], 4}, {NULL}};
const Segment SGI286636[] = {{&S94001[13], 14}, {&S94001[28], 6}, {&S94001[56], 17}, {NULL}};
const Segment SGI286637[] = {{&S94001[109], 18}, {NULL}};
const Segment SGI286638[] = {{&S94002[75], 4}, {NULL}};
const Segment SGI286639[] = {{&S94002[13], 14}, {&S94002[49], 17}, {NULL}};
const Segment SGI286640[] = {{&S94002[102], 18}, {NULL}};
const Segment SGI286641[] = {{&S94004[82], 4}, {NULL}};
const Segment SGI286642[] = {{&S94004[13], 14}, {&S94004[28], 6}, {&S94004[56], 17}, {NULL}};
const Segment SGI286643[] = {{&S94004[109], 18}, {NULL}};
const Segment SGI286644[] = {{&S94005[75], 4}, {NULL}};
const Segment SGI286645[] = {{&S94005[13], 14}, {&S94005[49], 17}, {NULL}};
const Segment SGI286646[] = {{&S94005[102], 18}, {NULL}};
const Segment SGI286647[] = {{&S94041[93], 4}, {NULL}};
const Segment SGI286648[] = {{&S94041[13], 3}, {&S94041[17], 13}, {&S94041[52], 17}, {NULL}};
const Segment SGI286649[] = {{&S94041[71], 12}, {NULL}};
const Segment SGI286650[] = {{&S94689[120], 4}, {NULL}};
const Segment SGI286651[] = {{&S94689[13], 9}, {&S94689[23], 2}, {&S94689[26], 10}, {&S94689[37], 6}, {&S94689[44], 26}, {&S94689[11], 1}, {&S94689[94], 17}, {NULL}};
const Segment SGI286652[] = {{&S103351[100], 3}, {NULL}};
const Segment SGI286653[] = {{&S103351[13], 9}, {&S103351[23], 2}, {&S103351[26], 22}, {&S103351[49], 6}, {&S103351[76], 17}, {NULL}};
const Segment SGI286654[] = {{&S103351[95], 8}, {NULL}};
const Segment SGI286655[] = {{&S94724[101], 4}, {NULL}};
const Segment SGI286656[] = {{&S94724[13], 7}, {&S94724[21], 2}, {&S94724[24], 15}, {&S94724[40], 6}, {&S94724[68], 17}, {NULL}};
const Segment SGI286657[] = {{&S94724[87], 18}, {NULL}};
const Segment SGI286658[] = {{&S88976[7], 3}, {NULL}};
const Segment SGI286659[] = {{&S88976[11], 5}, {NULL}};
const Segment SGI286660[] = {{&S103410[86], 3}, {NULL}};
const Segment SGI286661[] = {{&S103410[13], 3}, {&S103410[17], 2}, {&S103410[20], 11}, {&S103410[32], 6}, {&S103410[60], 17}, {NULL}};
const Segment SGI286662[] = {{&S103410[79], 11}, {NULL}};
const Segment SGI286663[] = {{&S94820[73], 4}, {NULL}};
const Segment SGI286664[] = {{&S94820[13], 3}, {&S94820[17], 8}, {&S94820[47], 17}, {NULL}};
const Segment SGI286665[] = {{&S94820[108], 3}, {NULL}};
const Segment SGI286666[] = {{&S95147[97], 4}, {NULL}};
const Segment SGI286667[] = {{&S95147[13], 14}, {&S95147[48], 17}, {NULL}};
const Segment SGI286668[] = {{&S95147[67], 18}, {NULL}};
const Segment SGI286669[] = {{&S95148[104], 4}, {NULL}};
const Segment SGI286670[] = {{&S95148[13], 14}, {&S95148[28], 6}, {&S95148[55], 17}, {NULL}};
const Segment SGI286671[] = {{&S95148[74], 18}, {NULL}};
const Segment SGI286672[] = {{&S89396[7], 3}, {NULL}};
const Segment SGI286673[] = {{&S89396[11], 5}, {NULL}};
const Segment SGI286674[] = {{&S103672[119], 3}, {NULL}};
const Segment SGI286675[] = {{&S103672[13], 9}, {&S103672[23], 2}, {&S103672[26], 22}, {&S103672[49], 6}, {&S103672[76], 17}, {NULL}};
const Segment SGI286676[] = {{&S103672[95], 9}, {NULL}};
const Segment SGI286677[] = {{&S103673[119], 3}, {NULL}};
const Segment SGI286678[] = {{&S103673[13], 9}, {&S103673[23], 2}, {&S103673[26], 22}, {&S103673[49], 6}, {&S103673[76], 17}, {NULL}};
const Segment SGI286679[] = {{&S103673[95], 9}, {NULL}};
const Segment SGI286680[] = {{&S103727[102], 3}, {NULL}};
const Segment SGI286681[] = {{&S103727[13], 3}, {&S103727[17], 2}, {&S103727[20], 13}, {&S103727[34], 6}, {&S103727[62], 17}, {NULL}};
const Segment SGI286682[] = {{&S103727[81], 11}, {NULL}};
const Segment SGI286683[] = {{&S95385[41], 4}, {NULL}};
const Segment SGI286684[] = {{&S95385[13], 14}, {&S95385[28], 25}, {&S95385[74], 17}, {NULL}};
const Segment SGI286685[] = {{&S95385[29], 16}, {NULL}};
const Segment SGI286686[] = {{&S95386[48], 4}, {NULL}};
const Segment SGI286687[] = {{&S95386[13], 14}, {&S95386[28], 6}, {&S95386[35], 25}, {&S95386[81], 17}, {NULL}};
const Segment SGI286688[] = {{&S95386[36], 16}, {NULL}};
const Segment SGI286689[] = {{&S95388[41], 4}, {NULL}};
const Segment SGI286690[] = {{&S95388[13], 14}, {&S95388[28], 25}, {&S95388[74], 17}, {NULL}};
const Segment SGI286691[] = {{&S95388[29], 16}, {NULL}};
const Segment SGI286692[] = {{&S95393[76], 4}, {NULL}};
const Segment SGI286693[] = {{&S95393[13], 15}, {&S95393[50], 17}, {NULL}};
const Segment SGI286694[] = {{&S89565[7], 3}, {NULL}};
const Segment SGI286695[] = {{&S89565[11], 6}, {NULL}};
const Segment SGI286696[] = {{&S89566[7], 3}, {NULL}};
const Segment SGI286697[] = {{&S89566[11], 8}, {NULL}};
const Segment SGI286698[] = {{&S89705[7], 3}, {NULL}};
const Segment SGI286699[] = {{&S89705[11], 4}, {NULL}};
const Segment SGI286700[] = {{&S89706[7], 3}, {NULL}};
const Segment SGI286701[] = {{&S89706[11], 4}, {NULL}};
const Segment SGI286702[] = {{&S95526[87], 4}, {NULL}};
const Segment SGI286703[] = {{&S95526[13], 3}, {&S95526[17], 13}, {&S95526[31], 8}, {&S95526[61], 17}, {NULL}};
const Segment SGI286704[] = {{&S95200[78], 4}, {NULL}};
const Segment SGI286705[] = {{&S95200[13], 3}, {&S95200[17], 14}, {&S95200[52], 17}, {NULL}};
const Segment SGI286706[] = {{&S8688[63], 4}, {NULL}};
const Segment SGI286707[] = {{&S8688[8], 3}, {NULL}};
const Segment SGI286708[] = {{&S8688[13], 14}, {NULL}};
const Segment SGI286709[] = {{&S8688[7], 1}, {NULL}};
const Segment SGI286710[] = {{&S8688[16], 11}, {NULL}};
const Segment SGI286711[] = {{&S8688[68], 11}, {NULL}};
const Segment SGI286712[] = {{&S104324[70], 3}, {NULL}};
const Segment SGI286713[] = {{&S104324[13], 9}, {&S104324[44], 17}, {NULL}};
const Segment SGI286714[] = {{&S95906[105], 4}, {NULL}};
const Segment SGI286715[] = {{&S95906[13], 14}, {&S95906[28], 6}, {&S95906[56], 17}, {NULL}};
const Segment SGI286716[] = {{&S95906[75], 18}, {NULL}};
const Segment SGI286717[] = {{&S90025[7], 3}, {NULL}};
const Segment SGI286718[] = {{&S90025[11], 4}, {NULL}};
const Segment SGI286719[] = {{&S95988[75], 4}, {NULL}};
const Segment SGI286720[] = {{&S95988[13], 15}, {&S95988[49], 17}, {NULL}};
const Segment SGI286721[] = {{&S95989[75], 4}, {NULL}};
const Segment SGI286722[] = {{&S95989[13], 14}, {&S95989[49], 17}, {NULL}};
const Segment SGI286723[] = {{&S95989[111], 3}, {NULL}};
const Segment SGI286724[] = {{&S96029[92], 4}, {NULL}};
const Segment SGI286725[] = {{&S96029[13], 3}, {&S96029[17], 13}, {&S96029[52], 17}, {NULL}};
const Segment SGI286726[] = {{&S96029[71], 11}, {NULL}};
const Segment SGI286727[] = {{&S96030[90], 4}, {NULL}};
const Segment SGI286728[] = {{&S96030[13], 3}, {&S96030[17], 11}, {&S96030[50], 17}, {NULL}};
const Segment SGI286729[] = {{&S96030[69], 11}, {NULL}};
const Segment SGI286730[] = {{&S9027[63], 4}, {NULL}};
const Segment SGI286731[] = {{&S9027[8], 3}, {NULL}};
const Segment SGI286732[] = {{&S9027[13], 14}, {NULL}};
const Segment SGI286733[] = {{&S9027[7], 1}, {NULL}};
const Segment SGI286734[] = {{&S9027[16], 11}, {NULL}};
const Segment SGI286735[] = {{&S9027[68], 11}, {NULL}};
const Segment SGI286736[] = {{&S9104[68], 4}, {NULL}};
const Segment SGI286737[] = {{&S9104[8], 3}, {NULL}};
const Segment SGI286738[] = {{&S9104[13], 14}, {NULL}};
const Segment SGI286739[] = {{&S9104[7], 1}, {NULL}};
const Segment SGI286740[] = {{&S9104[16], 11}, {&S9104[32], 2}, {NULL}};
const Segment SGI286741[] = {{&S9104[73], 11}, {NULL}};
const Segment SGI286742[] = {{&S90180[7], 3}, {NULL}};
const Segment SGI286743[] = {{&S90180[11], 5}, {NULL}};
const Segment SGI286744[] = {{&S90181[7], 3}, {NULL}};
const Segment SGI286745[] = {{&S90181[11], 5}, {NULL}};
const Segment SGI286746[] = {{&S90182[7], 3}, {NULL}};
const Segment SGI286747[] = {{&S90182[11], 5}, {NULL}};
const Segment SGI286748[] = {{&S90183[7], 3}, {NULL}};
const Segment SGI286749[] = {{&S90183[11], 6}, {NULL}};
const Segment SGI286750[] = {{&S9183[84], 4}, {NULL}};
const Segment SGI286751[] = {{&S9183[8], 3}, {NULL}};
const Segment SGI286752[] = {{&S9183[13], 7}, {NULL}};
const Segment SGI286753[] = {{&S9183[25], 14}, {NULL}};
const Segment SGI286754[] = {{&S9183[7], 1}, {NULL}};
const Segment SGI286755[] = {{&S9183[16], 4}, {&S9183[28], 11}, {NULL}};
const Segment SGI286756[] = {{&S9184[73], 4}, {NULL}};
const Segment SGI286757[] = {{&S9184[8], 3}, {NULL}};
const Segment SGI286758[] = {{&S9184[26], 14}, {NULL}};
const Segment SGI286759[] = {{&S9184[7], 1}, {NULL}};
const Segment SGI286760[] = {{&S9184[17], 4}, {&S9184[29], 11}, {NULL}};
const Segment SGI286761[] = {{&S9186[72], 4}, {NULL}};
const Segment SGI286762[] = {{&S9186[8], 3}, {NULL}};
const Segment SGI286763[] = {{&S9186[13], 7}, {NULL}};
const Segment SGI286764[] = {{&S9186[25], 14}, {NULL}};
const Segment SGI286765[] = {{&S9186[7], 1}, {&S9186[11], 1}, {NULL}};
const Segment SGI286766[] = {{&S9186[16], 4}, {&S9186[28], 11}, {NULL}};
const Segment SGI286767[] = {{&S9187[76], 4}, {NULL}};
const Segment SGI286768[] = {{&S9187[8], 3}, {NULL}};
const Segment SGI286769[] = {{&S9187[13], 7}, {NULL}};
const Segment SGI286770[] = {{&S9187[25], 14}, {NULL}};
const Segment SGI286771[] = {{&S9187[7], 1}, {NULL}};
const Segment SGI286772[] = {{&S9187[16], 4}, {&S9187[28], 11}, {&S9187[108], 17}, {NULL}};
const Segment SGI286773[] = {{&S9188[56], 4}, {NULL}};
const Segment SGI286774[] = {{&S9188[8], 3}, {NULL}};
const Segment SGI286775[] = {{&S9188[13], 7}, {NULL}};
const Segment SGI286776[] = {{&S9188[7], 1}, {NULL}};
const Segment SGI286777[] = {{&S9188[16], 4}, {&S9188[28], 34}, {NULL}};
const Segment SGI286778[] = {{&S9189[76], 4}, {NULL}};
const Segment SGI286779[] = {{&S9189[8], 3}, {NULL}};
const Segment SGI286780[] = {{&S9189[13], 7}, {NULL}};
const Segment SGI286781[] = {{&S9189[25], 14}, {NULL}};
const Segment SGI286782[] = {{&S9189[7], 1}, {NULL}};
const Segment SGI286783[] = {{&S9189[16], 4}, {&S9189[28], 11}, {&S9189[108], 17}, {NULL}};
const Segment SGI286784[] = {{&S9190[75], 4}, {NULL}};
const Segment SGI286785[] = {{&S9190[8], 3}, {NULL}};
const Segment SGI286786[] = {{&S9190[13], 7}, {NULL}};
const Segment SGI286787[] = {{&S9190[25], 14}, {NULL}};
const Segment SGI286788[] = {{&S9190[7], 1}, {NULL}};
const Segment SGI286789[] = {{&S9190[16], 4}, {&S9190[28], 11}, {NULL}};
const Segment SGI286790[] = {{&S9193[8], 3}, {NULL}};
const Segment SGI286791[] = {{&S9193[13], 7}, {NULL}};
const Segment SGI286792[] = {{&S9193[25], 14}, {NULL}};
const Segment SGI286793[] = {{&S9193[16], 4}, {&S9193[28], 11}, {NULL}};
const Segment SGI286794[] = {{&S9193[69], 11}, {NULL}};
const Segment SGI286795[] = {{&S9194[76], 4}, {NULL}};
const Segment SGI286796[] = {{&S9194[8], 3}, {NULL}};
const Segment SGI286797[] = {{&S9194[13], 7}, {NULL}};
const Segment SGI286798[] = {{&S9194[25], 14}, {NULL}};
const Segment SGI286799[] = {{&S9194[7], 1}, {NULL}};
const Segment SGI286800[] = {{&S9194[16], 4}, {&S9194[28], 11}, {&S9194[108], 17}, {NULL}};
const Segment SGI286801[] = {{&S9195[8], 3}, {NULL}};
const Segment SGI286802[] = {{&S9195[13], 7}, {NULL}};
const Segment SGI286803[] = {{&S9195[25], 14}, {NULL}};
const Segment SGI286804[] = {{&S9195[7], 1}, {NULL}};
const Segment SGI286805[] = {{&S9195[16], 4}, {&S9195[28], 11}, {NULL}};
const Segment SGI286806[] = {{&S9198[76], 4}, {NULL}};
const Segment SGI286807[] = {{&S9198[8], 3}, {NULL}};
const Segment SGI286808[] = {{&S9198[13], 7}, {NULL}};
const Segment SGI286809[] = {{&S9198[25], 14}, {NULL}};
const Segment SGI286810[] = {{&S9198[7], 1}, {NULL}};
const Segment SGI286811[] = {{&S9198[16], 4}, {&S9198[28], 11}, {&S9198[108], 17}, {NULL}};
const Segment SGI286812[] = {{&S9199[73], 4}, {NULL}};
const Segment SGI286813[] = {{&S9199[8], 3}, {NULL}};
const Segment SGI286814[] = {{&S9199[13], 7}, {NULL}};
const Segment SGI286815[] = {{&S9199[25], 14}, {NULL}};
const Segment SGI286816[] = {{&S9199[7], 1}, {NULL}};
const Segment SGI286817[] = {{&S9199[16], 4}, {&S9199[28], 11}, {NULL}};
const Segment SGI286818[] = {{&S9201[8], 3}, {NULL}};
const Segment SGI286819[] = {{&S9201[13], 7}, {NULL}};
const Segment SGI286820[] = {{&S9201[16], 4}, {&S9201[28], 17}, {NULL}};
const Segment SGI286821[] = {{&S9202[72], 4}, {NULL}};
const Segment SGI286822[] = {{&S9202[8], 3}, {NULL}};
const Segment SGI286823[] = {{&S9202[13], 7}, {NULL}};
const Segment SGI286824[] = {{&S9202[25], 14}, {NULL}};
const Segment SGI286825[] = {{&S9202[7], 1}, {NULL}};
const Segment SGI286826[] = {{&S9202[16], 4}, {&S9202[28], 11}, {NULL}};
const Segment SGI286827[] = {{&S9203[69], 4}, {NULL}};
const Segment SGI286828[] = {{&S9203[8], 3}, {NULL}};
const Segment SGI286829[] = {{&S9203[13], 7}, {NULL}};
const Segment SGI286830[] = {{&S9203[7], 1}, {NULL}};
const Segment SGI286831[] = {{&S9203[16], 4}, {&S9203[28], 11}, {NULL}};
const Segment SGI286832[] = {{&S9204[72], 5}, {NULL}};
const Segment SGI286833[] = {{&S9204[8], 3}, {NULL}};
const Segment SGI286834[] = {{&S9204[13], 7}, {NULL}};
const Segment SGI286835[] = {{&S9204[25], 14}, {NULL}};
const Segment SGI286836[] = {{&S9204[7], 1}, {NULL}};
const Segment SGI286837[] = {{&S9204[16], 4}, {&S9204[28], 11}, {NULL}};
const Segment SGI286838[] = {{&S9205[76], 4}, {NULL}};
const Segment SGI286839[] = {{&S9205[8], 3}, {NULL}};
const Segment SGI286840[] = {{&S9205[13], 7}, {NULL}};
const Segment SGI286841[] = {{&S9205[25], 14}, {NULL}};
const Segment SGI286842[] = {{&S9205[7], 1}, {NULL}};
const Segment SGI286843[] = {{&S9205[16], 4}, {&S9205[28], 11}, {&S9205[108], 25}, {NULL}};
const Segment SGI286844[] = {{&S95747[78], 4}, {NULL}};
const Segment SGI286845[] = {{&S95747[13], 3}, {&S95747[17], 14}, {&S95747[52], 17}, {NULL}};
const Segment SGI286846[] = {{&S96324[75], 4}, {NULL}};
const Segment SGI286847[] = {{&S96324[13], 14}, {&S96324[49], 17}, {NULL}};
const Segment SGI286848[] = {{&S96324[111], 3}, {NULL}};
const Segment SGI286849[] = {{&S90754[7], 3}, {NULL}};
const Segment SGI286850[] = {{&S90754[11], 4}, {NULL}};
const Segment SGI286851[] = {{&S96566[104], 4}, {NULL}};
const Segment SGI286852[] = {{&S96566[13], 4}, {&S96566[18], 2}, {&S96566[21], 27}, {&S96566[49], 6}, {&S96566[77], 17}, {NULL}};
const Segment SGI286853[] = {{&S96566[130], 10}, {NULL}};
const Segment SGI286854[] = {{&S90945[7], 3}, {NULL}};
const Segment SGI286855[] = {{&S90945[11], 5}, {NULL}};
const Segment SGI286856[] = {{&S90946[7], 3}, {NULL}};
const Segment SGI286857[] = {{&S90946[11], 8}, {NULL}};
const Segment SGI286858[] = {{&S96717[88], 4}, {NULL}};
const Segment SGI286859[] = {{&S96717[13], 15}, {&S96717[50], 17}, {NULL}};
const Segment SGI286860[] = {{&S96717[69], 11}, {NULL}};
const Segment SGI286861[] = {{&S9935[82], 4}, {NULL}};
const Segment SGI286862[] = {{&S9935[8], 3}, {NULL}};
const Segment SGI286863[] = {{&S9935[13], 7}, {NULL}};
const Segment SGI286864[] = {{&S9935[25], 14}, {NULL}};
const Segment SGI286865[] = {{&S9935[7], 1}, {NULL}};
const Segment SGI286866[] = {{&S9935[16], 4}, {&S9935[28], 11}, {NULL}};
const Segment SGI286867[] = {{&S9936[63], 4}, {NULL}};
const Segment SGI286868[] = {{&S9936[8], 3}, {NULL}};
const Segment SGI286869[] = {{&S9936[13], 14}, {NULL}};
const Segment SGI286870[] = {{&S9936[7], 1}, {NULL}};
const Segment SGI286871[] = {{&S9936[16], 11}, {NULL}};
const Segment SGI286872[] = {{&S9936[70], 11}, {NULL}};
const Segment SGI286873[] = {{&S9937[39], 4}, {&S9937[76], 4}, {NULL}};
const Segment SGI286874[] = {{&S9937[8], 3}, {NULL}};
const Segment SGI286875[] = {{&S9937[13], 14}, {NULL}};
const Segment SGI286876[] = {{&S9937[7], 1}, {&S9937[7], 1}, {NULL}};
const Segment SGI286877[] = {{&S9937[16], 11}, {NULL}};
const Segment SGI286878[] = {{&S9937[83], 11}, {NULL}};
const Segment SGI286879[] = {{&S9938[76], 4}, {NULL}};
const Segment SGI286880[] = {{&S9938[8], 3}, {NULL}};
const Segment SGI286881[] = {{&S9938[13], 7}, {NULL}};
const Segment SGI286882[] = {{&S9938[25], 14}, {NULL}};
const Segment SGI286883[] = {{&S9938[7], 1}, {NULL}};
const Segment SGI286884[] = {{&S9938[16], 4}, {&S9938[28], 11}, {&S9938[87], 14}, {NULL}};
const Segment SGI286885[] = {{&S9939[75], 4}, {NULL}};
const Segment SGI286886[] = {{&S9939[8], 3}, {NULL}};
const Segment SGI286887[] = {{&S9939[13], 7}, {NULL}};
const Segment SGI286888[] = {{&S9939[25], 14}, {NULL}};
const Segment SGI286889[] = {{&S9939[7], 1}, {NULL}};
const Segment SGI286890[] = {{&S9939[16], 4}, {&S9939[28], 11}, {&S9939[85], 17}, {NULL}};
const Segment SGI286891[] = {{&S96860[118], 4}, {NULL}};
const Segment SGI286892[] = {{&S96860[13], 3}, {&S96860[17], 13}, {&S96860[31], 39}, {&S96860[92], 17}, {NULL}};
const Segment SGI286893[] = {{&S9940[66], 4}, {NULL}};
const Segment SGI286894[] = {{&S9940[8], 3}, {NULL}};
const Segment SGI286895[] = {{&S9940[13], 14}, {NULL}};
const Segment SGI286896[] = {{&S9940[7], 1}, {NULL}};
const Segment SGI286897[] = {{&S9940[16], 11}, {NULL}};
const Segment SGI286898[] = {{&S9940[73], 11}, {NULL}};
const Segment SGI286899[] = {{&S9941[75], 4}, {NULL}};
const Segment SGI286900[] = {{&S9941[8], 3}, {NULL}};
const Segment SGI286901[] = {{&S9941[13], 7}, {NULL}};
const Segment SGI286902[] = {{&S9941[25], 14}, {NULL}};
const Segment SGI286903[] = {{&S9941[7], 1}, {NULL}};
const Segment SGI286904[] = {{&S9941[16], 4}, {&S9941[28], 11}, {&S9941[106], 14}, {NULL}};
const Segment SGI286905[] = {{&S9942[39], 4}, {&S9942[39], 4}, {NULL}};
const Segment SGI286906[] = {{&S9942[8], 3}, {NULL}};
const Segment SGI286907[] = {{&S9942[13], 14}, {NULL}};
const Segment SGI286908[] = {{&S9942[7], 1}, {&S9942[7], 1}, {NULL}};
const Segment SGI286909[] = {{&S9942[16], 11}, {NULL}};
const Segment SGI286910[] = {{&S9942[83], 11}, {NULL}};
const Segment SGI286911[] = {{&S9943[63], 4}, {NULL}};
const Segment SGI286912[] = {{&S9943[8], 3}, {NULL}};
const Segment SGI286913[] = {{&S9943[13], 14}, {NULL}};
const Segment SGI286914[] = {{&S9943[7], 1}, {NULL}};
const Segment SGI286915[] = {{&S9943[16], 11}, {NULL}};
const Segment SGI286916[] = {{&S9943[70], 11}, {NULL}};
const Segment SGI286917[] = {{&S9944[7], 4}, {&S9944[76], 4}, {NULL}};
const Segment SGI286918[] = {{&S9944[8], 3}, {NULL}};
const Segment SGI286919[] = {{&S9944[13], 14}, {NULL}};
const Segment SGI286920[] = {{&S9944[7], 1}, {&S9944[7], 1}, {NULL}};
const Segment SGI286921[] = {{&S9944[16], 11}, {NULL}};
const Segment SGI286922[] = {{&S9944[83], 11}, {NULL}};
const Segment SGI286923[] = {{&S9945[63], 4}, {NULL}};
const Segment SGI286924[] = {{&S9945[8], 3}, {NULL}};
const Segment SGI286925[] = {{&S9945[13], 14}, {NULL}};
const Segment SGI286926[] = {{&S9945[7], 1}, {NULL}};
const Segment SGI286927[] = {{&S9945[16], 11}, {NULL}};
const Segment SGI286928[] = {{&S9945[70], 11}, {NULL}};
const Segment SGI286929[] = {{&S9946[64], 4}, {NULL}};
const Segment SGI286930[] = {{&S9946[8], 3}, {NULL}};
const Segment SGI286931[] = {{&S9946[13], 14}, {NULL}};
const Segment SGI286932[] = {{&S9946[7], 1}, {NULL}};
const Segment SGI286933[] = {{&S9946[16], 11}, {NULL}};
const Segment SGI286934[] = {{&S9947[66], 4}, {NULL}};
const Segment SGI286935[] = {{&S9947[8], 3}, {NULL}};
const Segment SGI286936[] = {{&S9947[13], 14}, {NULL}};
const Segment SGI286937[] = {{&S9947[7], 1}, {NULL}};
const Segment SGI286938[] = {{&S9947[16], 11}, {NULL}};
const Segment SGI286939[] = {{&S9947[73], 11}, {NULL}};
const Segment SGI286940[] = {{&S9948[63], 4}, {NULL}};
const Segment SGI286941[] = {{&S9948[8], 3}, {NULL}};
const Segment SGI286942[] = {{&S9948[13], 14}, {NULL}};
const Segment SGI286943[] = {{&S9948[7], 1}, {NULL}};
const Segment SGI286944[] = {{&S9948[16], 11}, {NULL}};
const Segment SGI286945[] = {{&S9948[70], 11}, {NULL}};
const Segment SGI286946[] = {{&S9949[67], 4}, {NULL}};
const Segment SGI286947[] = {{&S9949[8], 3}, {NULL}};
const Segment SGI286948[] = {{&S9949[13], 7}, {NULL}};
const Segment SGI286949[] = {{&S9949[25], 14}, {NULL}};
const Segment SGI286950[] = {{&S9949[7], 1}, {NULL}};
const Segment SGI286951[] = {{&S9949[16], 4}, {&S9949[28], 11}, {NULL}};
const Segment SGI286952[] = {{&S9950[63], 4}, {NULL}};
const Segment SGI286953[] = {{&S9950[8], 3}, {NULL}};
const Segment SGI286954[] = {{&S9950[13], 14}, {NULL}};
const Segment SGI286955[] = {{&S9950[7], 1}, {NULL}};
const Segment SGI286956[] = {{&S9950[16], 11}, {NULL}};
const Segment SGI286957[] = {{&S9950[70], 11}, {NULL}};
const Segment SGI286958[] = {{&S9951[75], 4}, {NULL}};
const Segment SGI286959[] = {{&S9951[8], 3}, {NULL}};
const Segment SGI286960[] = {{&S9951[13], 7}, {NULL}};
const Segment SGI286961[] = {{&S9951[25], 14}, {NULL}};
const Segment SGI286962[] = {{&S9951[7], 1}, {NULL}};
const Segment SGI286963[] = {{&S9951[16], 4}, {&S9951[28], 11}, {&S9951[85], 17}, {NULL}};
const Segment SGI286964[] = {{&S91116[7], 3}, {NULL}};
const Segment SGI286965[] = {{&S91116[11], 8}, {NULL}};
const Segment SGI286966[] = {{&S91117[7], 3}, {NULL}};
const Segment SGI286967[] = {{&S91117[11], 6}, {NULL}};
const Segment SGI286968[] = {{&S96948[78], 4}, {NULL}};
const Segment SGI286969[] = {{&S96948[13], 3}, {&S96948[17], 14}, {&S96948[52], 17}, {NULL}};
const Segment SGI286970[] = {{&S96949[77], 4}, {NULL}};
const Segment SGI286971[] = {{&S96949[13], 3}, {&S96949[17], 13}, {&S96949[51], 17}, {NULL}};
const Segment SGI286972[] = {{&S96640[74], 4}, {NULL}};
const Segment SGI286973[] = {{&S96640[13], 14}, {&S96640[48], 17}, {NULL}};
const Segment SGI286974[] = {{&S96640[100], 18}, {NULL}};
const Segment SGI286975[] = {{&S96641[81], 4}, {NULL}};
const Segment SGI286976[] = {{&S96641[13], 14}, {&S96641[28], 6}, {&S96641[55], 17}, {NULL}};
const Segment SGI286977[] = {{&S96641[107], 18}, {NULL}};
const Segment SGI286978[] = {{&S97290[74], 4}, {NULL}};
const Segment SGI286979[] = {{&S97290[13], 14}, {&S97290[48], 17}, {NULL}};
const Segment SGI286980[] = {{&S97290[100], 18}, {NULL}};
const Segment SGI286981[] = {{&S97291[81], 4}, {NULL}};
const Segment SGI286982[] = {{&S97291[13], 14}, {&S97291[28], 6}, {&S97291[55], 17}, {NULL}};
const Segment SGI286983[] = {{&S97291[107], 18}, {NULL}};
const Segment SGI286984[] = {{&S91569[7], 3}, {NULL}};
const Segment SGI286985[] = {{&S91569[11], 5}, {NULL}};
const Segment SGI286986[] = {{&S97665[82], 4}, {NULL}};
const Segment SGI286987[] = {{&S97665[13], 14}, {&S97665[49], 17}, {NULL}};
const Segment SGI286988[] = {{&S97665[68], 18}, {NULL}};
const Segment SGI286989[] = {{&S97670[82], 4}, {NULL}};
const Segment SGI286990[] = {{&S97670[13], 14}, {&S97670[28], 6}, {&S97670[56], 17}, {NULL}};
const Segment SGI286991[] = {{&S97670[110], 18}, {NULL}};
const Segment SGI286992[] = {{&S97671[75], 4}, {NULL}};
const Segment SGI286993[] = {{&S97671[13], 14}, {&S97671[49], 17}, {NULL}};
const Segment SGI286994[] = {{&S97671[103], 18}, {NULL}};
const Segment SGI286995[] = {{&S97683[75], 4}, {NULL}};
const Segment SGI286996[] = {{&S97683[13], 14}, {&S97683[49], 17}, {NULL}};
const Segment SGI286997[] = {{&S97683[111], 4}, {NULL}};
const Segment SGI286998[] = {{&S91880[7], 3}, {NULL}};
const Segment SGI286999[] = {{&S91880[11], 4}, {NULL}};
const Segment SGI287000[] = {{&S97692[74], 4}, {NULL}};
const Segment SGI287001[] = {{&S97692[13], 14}, {&S97692[48], 17}, {NULL}};
const Segment SGI287002[] = {{&S97692[100], 12}, {NULL}};
const Segment SGI287003[] = {{&S97701[78], 4}, {NULL}};
const Segment SGI287004[] = {{&S97701[13], 3}, {&S97701[17], 14}, {&S97701[52], 17}, {NULL}};
const Segment SGI287005[] = {{&S97702[77], 4}, {NULL}};
const Segment SGI287006[] = {{&S97702[13], 3}, {&S97702[17], 13}, {&S97702[51], 17}, {NULL}};
const Segment SGI287007[] = {{&S97773[104], 4}, {NULL}};
const Segment SGI287008[] = {{&S97773[13], 4}, {&S97773[18], 2}, {&S97773[21], 27}, {&S97773[49], 6}, {&S97773[77], 17}, {NULL}};
const Segment SGI287009[] = {{&S97773[130], 10}, {NULL}};
const Segment SGI287010[] = {{&S10921[64], 4}, {NULL}};
const Segment SGI287011[] = {{&S10921[8], 3}, {NULL}};
const Segment SGI287012[] = {{&S10921[13], 14}, {NULL}};
const Segment SGI287013[] = {{&S10921[7], 1}, {NULL}};
const Segment SGI287014[] = {{&S10921[16], 11}, {&S10921[32], 2}, {NULL}};
const Segment SGI287015[] = {{&S10921[69], 11}, {NULL}};
const Segment SGI287016[] = {{&S10938[59], 4}, {NULL}};
const Segment SGI287017[] = {{&S10938[8], 3}, {NULL}};
const Segment SGI287018[] = {{&S10938[13], 14}, {NULL}};
const Segment SGI287019[] = {{&S10938[7], 1}, {NULL}};
const Segment SGI287020[] = {{&S10938[16], 11}, {NULL}};
const Segment SGI287021[] = {{&S10938[64], 11}, {NULL}};
const Segment SGI287022[] = {{&S92088[7], 3}, {NULL}};
const Segment SGI287023[] = {{&S92088[11], 4}, {NULL}};
const Segment SGI287024[] = {{&S92290[7], 3}, {NULL}};
const Segment SGI287025[] = {{&S92290[11], 5}, {NULL}};
const Segment SGI287026[] = {{&S98112[93], 4}, {NULL}};
const Segment SGI287027[] = {{&S98112[13], 3}, {&S98112[17], 13}, {&S98112[52], 17}, {NULL}};
const Segment SGI287028[] = {{&S98112[71], 12}, {NULL}};
const Segment SGI287029[] = {{&S98113[91], 4}, {NULL}};
const Segment SGI287030[] = {{&S98113[13], 3}, {&S98113[17], 11}, {&S98113[50], 17}, {NULL}};
const Segment SGI287031[] = {{&S98113[69], 12}, {NULL}};
const Segment SGI287032[] = {{&S11189[59], 4}, {NULL}};
const Segment SGI287033[] = {{&S11189[8], 3}, {NULL}};
const Segment SGI287034[] = {{&S11189[13], 14}, {NULL}};
const Segment SGI287035[] = {{&S11189[7], 1}, {NULL}};
const Segment SGI287036[] = {{&S11189[16], 11}, {NULL}};
const Segment SGI287037[] = {{&S11189[64], 11}, {NULL}};
const Segment SGI287038[] = {{&S11222[64], 4}, {NULL}};
const Segment SGI287039[] = {{&S11222[8], 3}, {NULL}};
const Segment SGI287040[] = {{&S11222[13], 14}, {NULL}};
const Segment SGI287041[] = {{&S11222[7], 1}, {NULL}};
const Segment SGI287042[] = {{&S11222[16], 11}, {&S11222[32], 2}, {NULL}};
const Segment SGI287043[] = {{&S11222[69], 11}, {NULL}};
const Segment SGI287044[] = {{&S92670[7], 3}, {NULL}};
const Segment SGI287045[] = {{&S92670[11], 4}, {NULL}};
const Segment SGI287046[] = {{&S98410[94], 4}, {NULL}};
const Segment SGI287047[] = {{&S98410[29], 3}, {&S98410[33], 13}, {&S98410[68], 17}, {NULL}};
const Segment SGI287048[] = {{&S98410[0], 16}, {NULL}};
const Segment SGI287049[] = {{&S98414[74], 4}, {NULL}};
const Segment SGI287050[] = {{&S98414[13], 14}, {&S98414[48], 17}, {NULL}};
const Segment SGI287051[] = {{&S98414[108], 3}, {NULL}};
const Segment SGI287052[] = {{&S98566[82], 4}, {NULL}};
const Segment SGI287053[] = {{&S98566[13], 14}, {&S98566[28], 6}, {&S98566[56], 17}, {NULL}};
const Segment SGI287054[] = {{&S98566[110], 18}, {NULL}};
const Segment SGI287055[] = {{&S98567[75], 4}, {NULL}};
const Segment SGI287056[] = {{&S98567[13], 14}, {&S98567[49], 17}, {NULL}};
const Segment SGI287057[] = {{&S98567[103], 18}, {NULL}};
const Segment SGI287058[] = {{&S98606[78], 4}, {NULL}};
const Segment SGI287059[] = {{&S98606[13], 3}, {&S98606[17], 14}, {&S98606[52], 17}, {NULL}};
const Segment SGI287060[] = {{&S98607[77], 4}, {NULL}};
const Segment SGI287061[] = {{&S98607[13], 3}, {&S98607[17], 13}, {&S98607[51], 17}, {NULL}};
const Segment SGI287062[] = {{&S98656[93], 4}, {NULL}};
const Segment SGI287063[] = {{&S98656[13], 9}, {&S98656[23], 22}, {&S98656[67], 17}, {NULL}};
const Segment SGI287064[] = {{&S98675[113], 4}, {NULL}};
const Segment SGI287065[] = {{&S98675[13], 6}, {&S98675[20], 2}, {&S98675[23], 34}, {&S98675[58], 6}, {&S98675[86], 17}, {NULL}};
const Segment SGI287066[] = {{&S98675[139], 10}, {NULL}};
const Segment SGI287067[] = {{&S93056[7], 3}, {NULL}};
const Segment SGI287068[] = {{&S93056[11], 5}, {NULL}};
const Segment SGI287069[] = {{&S12213[59], 4}, {NULL}};
const Segment SGI287070[] = {{&S12213[8], 3}, {NULL}};
const Segment SGI287071[] = {{&S12213[13], 14}, {NULL}};
const Segment SGI287072[] = {{&S12213[7], 1}, {NULL}};
const Segment SGI287073[] = {{&S12213[16], 11}, {NULL}};
const Segment SGI287074[] = {{&S12213[64], 11}, {NULL}};
const Segment SGI287075[] = {{&S12242[64], 4}, {NULL}};
const Segment SGI287076[] = {{&S12242[8], 3}, {NULL}};
const Segment SGI287077[] = {{&S12242[13], 14}, {NULL}};
const Segment SGI287078[] = {{&S12242[7], 1}, {NULL}};
const Segment SGI287079[] = {{&S12242[16], 11}, {&S12242[32], 2}, {NULL}};
const Segment SGI287080[] = {{&S12242[69], 11}, {NULL}};
const Segment SGI287081[] = {{&S93279[7], 3}, {NULL}};
const Segment SGI287082[] = {{&S93279[11], 4}, {NULL}};
const Segment SGI287083[] = {{&S93288[7], 3}, {NULL}};
const Segment SGI287084[] = {{&S93288[11], 4}, {NULL}};
const Segment SGI287085[] = {{&S93296[7], 3}, {NULL}};
const Segment SGI287086[] = {{&S93296[11], 4}, {NULL}};
const Segment SGI287087[] = {{&S99071[75], 4}, {NULL}};
const Segment SGI287088[] = {{&S99071[13], 14}, {&S99071[49], 17}, {NULL}};
const Segment SGI287089[] = {{&S99071[111], 3}, {NULL}};
const Segment SGI287090[] = {{&S99105[90], 4}, {NULL}};
const Segment SGI287091[] = {{&S99105[13], 3}, {&S99105[17], 11}, {&S99105[50], 17}, {NULL}};
const Segment SGI287092[] = {{&S99105[69], 11}, {NULL}};
const Segment SGI287093[] = {{&S99106[92], 4}, {NULL}};
const Segment SGI287094[] = {{&S99106[13], 3}, {&S99106[17], 13}, {&S99106[52], 17}, {NULL}};
const Segment SGI287095[] = {{&S99106[71], 11}, {NULL}};
const Segment SGI287096[] = {{&S12493[59], 4}, {NULL}};
const Segment SGI287097[] = {{&S12493[8], 3}, {NULL}};
const Segment SGI287098[] = {{&S12493[13], 14}, {NULL}};
const Segment SGI287099[] = {{&S12493[7], 1}, {NULL}};
const Segment SGI287100[] = {{&S12493[16], 11}, {NULL}};
const Segment SGI287101[] = {{&S12493[64], 11}, {NULL}};
const Segment SGI287102[] = {{&S12525[64], 4}, {NULL}};
const Segment SGI287103[] = {{&S12525[8], 3}, {NULL}};
const Segment SGI287104[] = {{&S12525[13], 14}, {NULL}};
const Segment SGI287105[] = {{&S12525[7], 1}, {NULL}};
const Segment SGI287106[] = {{&S12525[16], 11}, {&S12525[32], 2}, {NULL}};
const Segment SGI287107[] = {{&S12525[69], 11}, {NULL}};
const Segment SGI287108[] = {{&S99351[80], 4}, {NULL}};
const Segment SGI287109[] = {{&S99351[13], 14}, {&S99351[28], 6}, {&S99351[55], 17}, {NULL}};
const Segment SGI287110[] = {{&S99351[75], 9}, {NULL}};
const Segment SGI287111[] = {{&S12809[56], 4}, {NULL}};
const Segment SGI287112[] = {{&S12809[8], 3}, {NULL}};
const Segment SGI287113[] = {{&S12809[13], 14}, {NULL}};
const Segment SGI287114[] = {{&S12809[7], 1}, {NULL}};
const Segment SGI287115[] = {{&S12809[16], 11}, {NULL}};
const Segment SGI287116[] = {{&S12809[61], 11}, {NULL}};
const Segment SGI287117[] = {{&S99582[75], 4}, {NULL}};
const Segment SGI287118[] = {{&S99582[13], 14}, {&S99582[49], 17}, {NULL}};
const Segment SGI287119[] = {{&S99582[111], 3}, {NULL}};
const Segment SGI287120[] = {{&S99671[91], 4}, {NULL}};
const Segment SGI287121[] = {{&S99671[13], 9}, {&S99671[23], 22}, {&S99671[66], 17}, {NULL}};
const Segment SGI287122[] = {{&S99671[86], 9}, {NULL}};
const Segment SGI287123[] = {{&S99258[82], 4}, {NULL}};
const Segment SGI287124[] = {{&S99258[13], 14}, {&S99258[28], 6}, {&S99258[56], 17}, {NULL}};
const Segment SGI287125[] = {{&S99258[108], 12}, {NULL}};
const Segment SGI287126[] = {{&S99394[81], 4}, {NULL}};
const Segment SGI287127[] = {{&S99394[13], 3}, {&S99394[17], 14}, {&S99394[53], 17}, {NULL}};
const Segment SGI287128[] = {{&S99408[92], 4}, {NULL}};
const Segment SGI287129[] = {{&S99408[13], 9}, {&S99408[23], 22}, {&S99408[66], 17}, {NULL}};
const Segment SGI287130[] = {{&S93971[7], 3}, {NULL}};
const Segment SGI287131[] = {{&S93971[11], 5}, {NULL}};
const Segment SGI287132[] = {{&S93977[7], 3}, {NULL}};
const Segment SGI287133[] = {{&S93977[11], 4}, {NULL}};
const Segment SGI287134[] = {{&S100224[82], 4}, {NULL}};
const Segment SGI287135[] = {{&S100224[13], 14}, {&S100224[49], 17}, {NULL}};
const Segment SGI287136[] = {{&S100224[68], 18}, {NULL}};
const Segment SGI287137[] = {{&S94657[7], 3}, {NULL}};
const Segment SGI287138[] = {{&S94657[11], 4}, {NULL}};
const Segment SGI287139[] = {{&S94892[7], 3}, {NULL}};
const Segment SGI287140[] = {{&S94892[11], 4}, {NULL}};
const Segment SGI287141[] = {{&S100710[102], 4}, {NULL}};
const Segment SGI287142[] = {{&S100710[13], 4}, {&S100710[18], 2}, {&S100710[21], 25}, {&S100710[47], 6}, {&S100710[75], 17}, {NULL}};
const Segment SGI287143[] = {{&S100710[128], 10}, {NULL}};
const Segment SGI287144[] = {{&S100777[82], 4}, {NULL}};
const Segment SGI287145[] = {{&S100777[13], 14}, {&S100777[28], 6}, {&S100777[56], 17}, {NULL}};
const Segment SGI287146[] = {{&S100777[110], 18}, {NULL}};
const Segment SGI287147[] = {{&S100778[75], 4}, {NULL}};
const Segment SGI287148[] = {{&S100778[13], 14}, {&S100778[49], 17}, {NULL}};
const Segment SGI287149[] = {{&S100778[103], 18}, {NULL}};
const Segment SGI287150[] = {{&S100790[82], 4}, {NULL}};
const Segment SGI287151[] = {{&S100790[13], 14}, {&S100790[28], 6}, {&S100790[56], 17}, {NULL}};
const Segment SGI287152[] = {{&S100790[108], 18}, {NULL}};
const Segment SGI287153[] = {{&S100791[75], 4}, {NULL}};
const Segment SGI287154[] = {{&S100791[13], 14}, {&S100791[49], 17}, {NULL}};
const Segment SGI287155[] = {{&S100791[101], 18}, {NULL}};
const Segment SGI287156[] = {{&S100801[79], 4}, {NULL}};
const Segment SGI287157[] = {{&S100801[13], 3}, {&S100801[17], 14}, {&S100801[53], 17}, {NULL}};
const Segment SGI287158[] = {{&S100807[90], 4}, {NULL}};
const Segment SGI287159[] = {{&S100807[13], 3}, {&S100807[17], 11}, {&S100807[50], 17}, {NULL}};
const Segment SGI287160[] = {{&S100807[69], 11}, {NULL}};
const Segment SGI287161[] = {{&S100821[93], 4}, {NULL}};
const Segment SGI287162[] = {{&S100821[13], 3}, {&S100821[17], 13}, {&S100821[52], 17}, {NULL}};
const Segment SGI287163[] = {{&S100821[71], 12}, {NULL}};
const Segment SGI287164[] = {{&S100822[91], 4}, {NULL}};
const Segment SGI287165[] = {{&S100822[13], 3}, {&S100822[17], 11}, {&S100822[50], 17}, {NULL}};
const Segment SGI287166[] = {{&S100822[69], 12}, {NULL}};
const Segment SGI287167[] = {{&S101058[93], 4}, {NULL}};
const Segment SGI287168[] = {{&S101058[13], 9}, {&S101058[23], 22}, {&S101058[67], 17}, {NULL}};
const Segment SGI287169[] = {{&S101058[115], 3}, {NULL}};
const Segment SGI287170[] = {{&S101093[90], 4}, {NULL}};
const Segment SGI287171[] = {{&S101093[13], 9}, {&S101093[23], 22}, {&S101093[66], 17}, {NULL}};
const Segment SGI287172[] = {{&S101093[85], 9}, {NULL}};
const Segment SGI287173[] = {{&S14635[59], 4}, {NULL}};
const Segment SGI287174[] = {{&S14635[8], 3}, {NULL}};
const Segment SGI287175[] = {{&S14635[13], 14}, {NULL}};
const Segment SGI287176[] = {{&S14635[7], 1}, {NULL}};
const Segment SGI287177[] = {{&S14635[16], 11}, {NULL}};
const Segment SGI287178[] = {{&S14635[64], 11}, {NULL}};
const Segment SGI287179[] = {{&S14691[64], 4}, {NULL}};
const Segment SGI287180[] = {{&S14691[8], 3}, {NULL}};
const Segment SGI287181[] = {{&S14691[13], 14}, {NULL}};
const Segment SGI287182[] = {{&S14691[7], 1}, {NULL}};
const Segment SGI287183[] = {{&S14691[16], 11}, {&S14691[32], 2}, {NULL}};
const Segment SGI287184[] = {{&S14691[69], 11}, {NULL}};
const Segment SGI287185[] = {{&S101203[100], 4}, {NULL}};
const Segment SGI287186[] = {{&S101203[13], 3}, {&S101203[17], 2}, {&S101203[20], 11}, {&S101203[32], 6}, {&S101203[60], 17}, {NULL}};
const Segment SGI287187[] = {{&S101203[79], 11}, {NULL}};
const Segment SGI287188[] = {{&S101214[90], 4}, {NULL}};
const Segment SGI287189[] = {{&S101214[13], 9}, {&S101214[23], 2}, {&S101214[26], 9}, {&S101214[36], 6}, {&S101214[64], 17}, {NULL}};
const Segment SGI287190[] = {{&S14803[56], 4}, {NULL}};
const Segment SGI287191[] = {{&S14803[8], 3}, {NULL}};
const Segment SGI287192[] = {{&S14803[13], 14}, {NULL}};
const Segment SGI287193[] = {{&S14803[7], 1}, {NULL}};
const Segment SGI287194[] = {{&S14803[16], 11}, {NULL}};
const Segment SGI287195[] = {{&S14803[61], 11}, {NULL}};
const Segment SGI287196[] = {{&S101488[91], 4}, {NULL}};
const Segment SGI287197[] = {{&S101488[13], 9}, {&S101488[23], 22}, {&S101488[66], 17}, {NULL}};
const Segment SGI287198[] = {{&S101488[86], 9}, {NULL}};
const Segment SGI287199[] = {{&S101513[75], 4}, {NULL}};
const Segment SGI287200[] = {{&S101513[13], 14}, {&S101513[49], 17}, {NULL}};
const Segment SGI287201[] = {{&S101513[111], 3}, {NULL}};
const Segment SGI287202[] = {{&S101514[75], 4}, {NULL}};
const Segment SGI287203[] = {{&S101514[13], 14}, {&S101514[49], 17}, {NULL}};
const Segment SGI287204[] = {{&S101514[111], 3}, {NULL}};
const Segment SGI287205[] = {{&S101516[81], 4}, {NULL}};
const Segment SGI287206[] = {{&S101516[13], 14}, {&S101516[28], 6}, {&S101516[55], 17}, {NULL}};
const Segment SGI287207[] = {{&S101516[107], 18}, {NULL}};
const Segment SGI287208[] = {{&S101517[74], 4}, {NULL}};
const Segment SGI287209[] = {{&S101517[13], 14}, {&S101517[48], 17}, {NULL}};
const Segment SGI287210[] = {{&S101517[100], 18}, {NULL}};
const Segment SGI287211[] = {{&S101525[74], 4}, {NULL}};
const Segment SGI287212[] = {{&S101525[13], 14}, {&S101525[48], 17}, {NULL}};
const Segment SGI287213[] = {{&S101525[100], 12}, {NULL}};
const Segment SGI287214[] = {{&S101698[98], 4}, {NULL}};
const Segment SGI287215[] = {{&S101698[13], 14}, {&S101698[49], 17}, {NULL}};
const Segment SGI287216[] = {{&S101698[68], 18}, {NULL}};
const Segment SGI287217[] = {{&S101699[105], 4}, {NULL}};
const Segment SGI287218[] = {{&S101699[13], 14}, {&S101699[28], 6}, {&S101699[56], 17}, {NULL}};
const Segment SGI287219[] = {{&S101699[75], 18}, {NULL}};
const Segment SGI287220[] = {{&S101702[93], 4}, {NULL}};
const Segment SGI287221[] = {{&S101702[13], 9}, {&S101702[23], 22}, {&S101702[67], 17}, {NULL}};
const Segment SGI287222[] = {{&S101702[115], 3}, {NULL}};
const Segment SGI287223[] = {{&S101703[93], 4}, {NULL}};
const Segment SGI287224[] = {{&S101703[13], 9}, {&S101703[23], 22}, {&S101703[67], 17}, {NULL}};
const Segment SGI287225[] = {{&S101703[115], 3}, {NULL}};
const Segment SGI287226[] = {{&S101716[78], 4}, {NULL}};
const Segment SGI287227[] = {{&S101716[13], 3}, {&S101716[17], 13}, {&S101716[52], 17}, {NULL}};
const Segment SGI287228[] = {{&S101717[79], 4}, {NULL}};
const Segment SGI287229[] = {{&S101717[13], 3}, {&S101717[17], 14}, {&S101717[53], 17}, {NULL}};
const Segment SGI287230[] = {{&S101839[75], 4}, {NULL}};
const Segment SGI287231[] = {{&S101839[13], 14}, {&S101839[49], 17}, {NULL}};
const Segment SGI287232[] = {{&S101839[111], 3}, {NULL}};
const Segment SGI287233[] = {{&S96193[7], 3}, {NULL}};
const Segment SGI287234[] = {{&S96193[11], 7}, {NULL}};
const Segment SGI287235[] = {{&S96312[7], 3}, {NULL}};
const Segment SGI287236[] = {{&S96312[11], 5}, {NULL}};
const Segment SGI287237[] = {{&S96322[7], 3}, {NULL}};
const Segment SGI287238[] = {{&S96322[11], 4}, {NULL}};
const Segment SGI287239[] = {{&S96323[7], 3}, {NULL}};
const Segment SGI287240[] = {{&S96323[11], 5}, {NULL}};
const Segment SGI287241[] = {{&S15852[59], 4}, {NULL}};
const Segment SGI287242[] = {{&S15852[8], 3}, {NULL}};
const Segment SGI287243[] = {{&S15852[13], 14}, {NULL}};
const Segment SGI287244[] = {{&S15852[7], 1}, {NULL}};
const Segment SGI287245[] = {{&S15852[16], 11}, {NULL}};
const Segment SGI287246[] = {{&S15852[64], 11}, {NULL}};
const Segment SGI287247[] = {{&S15895[64], 4}, {NULL}};
const Segment SGI287248[] = {{&S15895[8], 3}, {NULL}};
const Segment SGI287249[] = {{&S15895[13], 14}, {NULL}};
const Segment SGI287250[] = {{&S15895[7], 1}, {NULL}};
const Segment SGI287251[] = {{&S15895[16], 11}, {&S15895[32], 2}, {NULL}};
const Segment SGI287252[] = {{&S15895[69], 11}, {NULL}};
const Segment SGI287253[] = {{&S102485[90], 4}, {NULL}};
const Segment SGI287254[] = {{&S102485[13], 3}, {&S102485[17], 11}, {&S102485[50], 17}, {NULL}};
const Segment SGI287255[] = {{&S102485[69], 11}, {NULL}};
const Segment SGI287256[] = {{&S102488[77], 4}, {NULL}};
const Segment SGI287257[] = {{&S102488[13], 3}, {&S102488[17], 12}, {&S102488[51], 17}, {NULL}};
const Segment SGI287258[] = {{&S102675[75], 4}, {NULL}};
const Segment SGI287259[] = {{&S102675[13], 14}, {&S102675[49], 17}, {NULL}};
const Segment SGI287260[] = {{&S102675[111], 3}, {NULL}};
const Segment SGI287261[] = {{&S96654[7], 3}, {NULL}};
const Segment SGI287262[] = {{&S96654[11], 5}, {NULL}};
const Segment SGI287263[] = {{&S102887[105], 4}, {NULL}};
const Segment SGI287264[] = {{&S102887[13], 14}, {&S102887[28], 6}, {&S102887[56], 17}, {NULL}};
const Segment SGI287265[] = {{&S102887[75], 18}, {NULL}};
const Segment SGI287266[] = {{&S103060[75], 4}, {NULL}};
const Segment SGI287267[] = {{&S103060[13], 14}, {&S103060[49], 17}, {NULL}};
const Segment SGI287268[] = {{&S103060[111], 3}, {NULL}};
const Segment SGI287269[] = {{&S103062[75], 4}, {NULL}};
const Segment SGI287270[] = {{&S103062[13], 14}, {&S103062[49], 17}, {NULL}};
const Segment SGI287271[] = {{&S103062[111], 3}, {NULL}};
const Segment SGI287272[] = {{&S103077[92], 4}, {NULL}};
const Segment SGI287273[] = {{&S103077[13], 3}, {&S103077[17], 13}, {&S103077[51], 17}, {NULL}};
const Segment SGI287274[] = {{&S103077[70], 12}, {NULL}};
const Segment SGI287275[] = {{&S103078[90], 4}, {NULL}};
const Segment SGI287276[] = {{&S103078[13], 3}, {&S103078[17], 11}, {&S103078[49], 17}, {NULL}};
const Segment SGI287277[] = {{&S103078[68], 12}, {NULL}};
const Segment SGI287278[] = {{&S103118[98], 4}, {NULL}};
const Segment SGI287279[] = {{&S103118[13], 14}, {&S103118[28], 6}, {&S103118[56], 17}, {NULL}};
const Segment SGI287280[] = {{&S103118[75], 11}, {NULL}};
const Segment SGI287281[] = {{&S102469[74], 4}, {NULL}};
const Segment SGI287282[] = {{&S102469[13], 14}, {&S102469[48], 17}, {NULL}};
const Segment SGI287283[] = {{&S102469[100], 12}, {NULL}};
const Segment SGI287284[] = {{&S103119[91], 4}, {NULL}};
const Segment SGI287285[] = {{&S103119[13], 14}, {&S103119[49], 17}, {NULL}};
const Segment SGI287286[] = {{&S103119[68], 11}, {NULL}};
const Segment SGI287287[] = {{&S103335[66], 4}, {NULL}};
const Segment SGI287288[] = {{&S103335[13], 5}, {&S103335[40], 17}, {NULL}};
const Segment SGI287289[] = {{&S103337[75], 4}, {NULL}};
const Segment SGI287290[] = {{&S103337[13], 14}, {&S103337[49], 17}, {NULL}};
const Segment SGI287291[] = {{&S103337[111], 3}, {NULL}};
const Segment SGI287292[] = {{&S103341[92], 4}, {NULL}};
const Segment SGI287293[] = {{&S103341[13], 3}, {&S103341[17], 13}, {&S103341[51], 17}, {NULL}};
const Segment SGI287294[] = {{&S103341[70], 12}, {NULL}};
const Segment SGI287295[] = {{&S103342[90], 4}, {NULL}};
const Segment SGI287296[] = {{&S103342[13], 3}, {&S103342[17], 11}, {&S103342[49], 17}, {NULL}};
const Segment SGI287297[] = {{&S103342[68], 12}, {NULL}};
const Segment SGI287298[] = {{&S103690[75], 4}, {NULL}};
const Segment SGI287299[] = {{&S103690[13], 14}, {&S103690[49], 17}, {NULL}};
const Segment SGI287300[] = {{&S103690[111], 3}, {NULL}};
const Segment SGI287301[] = {{&S103725[76], 4}, {NULL}};
const Segment SGI287302[] = {{&S103725[13], 3}, {&S103725[17], 12}, {&S103725[50], 17}, {NULL}};
const Segment SGI287303[] = {{&S103954[74], 4}, {NULL}};
const Segment SGI287304[] = {{&S103954[13], 14}, {&S103954[48], 17}, {NULL}};
const Segment SGI287305[] = {{&S103954[109], 3}, {NULL}};
const Segment SGI287306[] = {{&S103960[82], 4}, {NULL}};
const Segment SGI287307[] = {{&S103960[13], 14}, {&S103960[49], 17}, {NULL}};
const Segment SGI287308[] = {{&S103960[68], 18}, {NULL}};
const Segment SGI287309[] = {{&S103212[136], 4}, {NULL}};
const Segment SGI287310[] = {{&S103212[13], 3}, {&S103212[17], 13}, {&S103212[51], 17}, {&S103212[81], 44}, {NULL}};
const Segment SGI287311[] = {{&S103213[136], 4}, {NULL}};
const Segment SGI287312[] = {{&S103213[13], 3}, {&S103213[17], 13}, {&S103213[51], 17}, {&S103213[81], 44}, {NULL}};
const Segment SGI287313[] = {{&S103214[136], 4}, {NULL}};
const Segment SGI287314[] = {{&S103214[13], 3}, {&S103214[17], 13}, {&S103214[51], 17}, {&S103214[81], 44}, {NULL}};
const Segment SGI287315[] = {{&S103215[136], 4}, {NULL}};
const Segment SGI287316[] = {{&S103215[13], 3}, {&S103215[17], 13}, {&S103215[51], 17}, {&S103215[81], 44}, {NULL}};
const Segment SGI287317[] = {{&S103216[136], 4}, {NULL}};
const Segment SGI287318[] = {{&S103216[13], 3}, {&S103216[17], 13}, {&S103216[51], 17}, {&S103216[81], 44}, {NULL}};
const Segment SGI287319[] = {{&S103505[80], 4}, {NULL}};
const Segment SGI287320[] = {{&S103505[13], 3}, {&S103505[17], 13}, {&S103505[52], 17}, {&S103505[141], 14}, {NULL}};
const Segment SGI287321[] = {{&S103505[127], 12}, {NULL}};
const Segment SGI287322[] = {{&S103506[78], 4}, {NULL}};
const Segment SGI287323[] = {{&S103506[13], 3}, {&S103506[17], 11}, {&S103506[50], 17}, {&S103506[139], 14}, {NULL}};
const Segment SGI287324[] = {{&S103506[125], 12}, {NULL}};
const Segment SGI287325[] = {{&S104362[74], 4}, {NULL}};
const Segment SGI287326[] = {{&S104362[13], 14}, {&S104362[48], 17}, {NULL}};
const Segment SGI287327[] = {{&S104362[109], 4}, {NULL}};
const Segment SGI287328[] = {{&S104764[75], 4}, {NULL}};
const Segment SGI287329[] = {{&S104764[13], 14}, {&S104764[49], 17}, {NULL}};
const Segment SGI287330[] = {{&S104764[111], 3}, {NULL}};
const Segment SGI287331[] = {{&S104363[70], 4}, {NULL}};
const Segment SGI287332[] = {{&S104363[13], 9}, {&S104363[44], 17}, {NULL}};
const Segment SGI287333[] = {{&S104368[69], 4}, {NULL}};
const Segment SGI287334[] = {{&S104368[13], 9}, {&S104368[43], 17}, {NULL}};
const Segment SGI287335[] = {{&S98551[7], 3}, {NULL}};
const Segment SGI287336[] = {{&S98551[11], 4}, {NULL}};
const Segment SGI287337[] = {{&S105488[72], 4}, {NULL}};
const Segment SGI287338[] = {{&S105488[13], 4}, {&S105488[18], 7}, {&S105488[46], 17}, {NULL}};
const Segment SGI287339[] = {{&S105063[136], 4}, {NULL}};
const Segment SGI287340[] = {{&S105063[13], 3}, {&S105063[17], 13}, {&S105063[51], 17}, {&S105063[81], 44}, {NULL}};
const Segment SGI287341[] = {{&S105070[80], 4}, {NULL}};
const Segment SGI287342[] = {{&S105070[13], 3}, {&S105070[17], 13}, {&S105070[52], 17}, {&S105070[139], 14}, {NULL}};
const Segment SGI287343[] = {{&S105070[125], 12}, {NULL}};
const Segment SGI287344[] = {{&S105071[78], 4}, {NULL}};
const Segment SGI287345[] = {{&S105071[13], 3}, {&S105071[17], 11}, {&S105071[50], 17}, {&S105071[137], 21}, {NULL}};
const Segment SGI287346[] = {{&S105071[123], 12}, {NULL}};
const Segment SGI287347[] = {{&S105064[85], 4}, {NULL}};
const Segment SGI287348[] = {{&S105064[13], 9}, {&S105064[23], 15}, {&S105064[59], 17}, {NULL}};
const Segment SGI287349[] = {{&S106010[13], 10}, {&S106010[24], 2}, {&S106010[27], 20}, {&S106010[62], 17}, {NULL}};
const Segment SGI287350[] = {{&S106012[13], 10}, {&S106012[24], 2}, {&S106012[27], 20}, {&S106012[62], 17}, {NULL}};
const Segment SGI287351[] = {{&S106013[13], 10}, {&S106013[24], 2}, {&S106013[27], 20}, {&S106013[62], 17}, {NULL}};
const Segment SGI287352[] = {{&S106014[13], 10}, {&S106014[24], 2}, {&S106014[27], 20}, {&S106014[62], 17}, {NULL}};
const Segment SGI287353[] = {{&S106015[13], 10}, {&S106015[24], 2}, {&S106015[27], 20}, {&S106015[62], 17}, {NULL}};
const Segment SGI287354[] = {{&S106016[13], 10}, {&S106016[24], 2}, {&S106016[27], 20}, {&S106016[62], 17}, {NULL}};
const Segment SGI287355[] = {{&S19369[55], 5}, {NULL}};
const Segment SGI287356[] = {{&S19369[8], 3}, {NULL}};
const Segment SGI287357[] = {{&S19369[13], 14}, {NULL}};
const Segment SGI287358[] = {{&S19369[7], 1}, {NULL}};
const Segment SGI287359[] = {{&S19369[16], 11}, {&S19369[32], 2}, {NULL}};
const Segment SGI287360[] = {{&S19370[50], 5}, {NULL}};
const Segment SGI287361[] = {{&S19370[8], 3}, {NULL}};
const Segment SGI287362[] = {{&S19370[13], 14}, {NULL}};
const Segment SGI287363[] = {{&S19370[7], 1}, {NULL}};
const Segment SGI287364[] = {{&S19370[16], 11}, {NULL}};
const Segment SGI287365[] = {{&S19371[95], 5}, {NULL}};
const Segment SGI287366[] = {{&S19371[8], 3}, {NULL}};
const Segment SGI287367[] = {{&S19371[13], 7}, {NULL}};
const Segment SGI287368[] = {{&S19371[25], 14}, {NULL}};
const Segment SGI287369[] = {{&S19371[7], 1}, {NULL}};
const Segment SGI287370[] = {{&S19371[16], 4}, {&S19371[28], 11}, {NULL}};
const Segment SGI287371[] = {{&S19372[64], 5}, {NULL}};
const Segment SGI287372[] = {{&S19372[8], 3}, {NULL}};
const Segment SGI287373[] = {{&S19372[13], 7}, {NULL}};
const Segment SGI287374[] = {{&S19372[22], 14}, {NULL}};
const Segment SGI287375[] = {{&S19372[7], 1}, {NULL}};
const Segment SGI287376[] = {{&S19372[16], 4}, {&S19372[25], 11}, {&S19372[41], 2}, {NULL}};
const Segment SGI287377[] = {{&S19579[63], 4}, {NULL}};
const Segment SGI287378[] = {{&S19579[8], 3}, {NULL}};
const Segment SGI287379[] = {{&S19579[13], 14}, {NULL}};
const Segment SGI287380[] = {{&S19579[7], 1}, {NULL}};
const Segment SGI287381[] = {{&S19579[16], 11}, {NULL}};
const Segment SGI287382[] = {{&S19579[68], 11}, {NULL}};
const Segment SGI287383[] = {{&S100156[7], 3}, {NULL}};
const Segment SGI287384[] = {{&S100156[11], 5}, {NULL}};
const Segment SGI287385[] = {{&S20004[52], 4}, {NULL}};
const Segment SGI287386[] = {{&S20004[8], 3}, {NULL}};
const Segment SGI287387[] = {{&S20004[13], 14}, {NULL}};
const Segment SGI287388[] = {{&S20004[7], 1}, {NULL}};
const Segment SGI287389[] = {{&S20004[16], 11}, {NULL}};
const Segment SGI287390[] = {{&S20004[57], 11}, {NULL}};
const Segment SGI287391[] = {{&S20144[83], 4}, {NULL}};
const Segment SGI287392[] = {{&S20144[8], 3}, {NULL}};
const Segment SGI287393[] = {{&S20144[13], 9}, {NULL}};
const Segment SGI287394[] = {{&S20144[33], 13}, {NULL}};
const Segment SGI287395[] = {{&S20144[7], 1}, {NULL}};
const Segment SGI287396[] = {{&S20277[63], 4}, {NULL}};
const Segment SGI287397[] = {{&S20277[8], 3}, {NULL}};
const Segment SGI287398[] = {{&S20277[13], 14}, {NULL}};
const Segment SGI287399[] = {{&S20277[7], 1}, {NULL}};
const Segment SGI287400[] = {{&S20277[16], 11}, {NULL}};
const Segment SGI287401[] = {{&S20277[70], 11}, {NULL}};
const Segment SGI287402[] = {{&S20306[63], 4}, {NULL}};
const Segment SGI287403[] = {{&S20306[8], 3}, {NULL}};
const Segment SGI287404[] = {{&S20306[13], 14}, {NULL}};
const Segment SGI287405[] = {{&S20306[7], 1}, {NULL}};
const Segment SGI287406[] = {{&S20306[16], 11}, {NULL}};
const Segment SGI287407[] = {{&S20306[70], 11}, {NULL}};
const Segment SGI287408[] = {{&S20660[59], 5}, {NULL}};
const Segment SGI287409[] = {{&S20660[8], 3}, {NULL}};
const Segment SGI287410[] = {{&S20660[13], 3}, {NULL}};
const Segment SGI287411[] = {{&S20660[7], 1}, {NULL}};
const Segment SGI287412[] = {{&S20661[59], 5}, {NULL}};
const Segment SGI287413[] = {{&S20661[8], 3}, {NULL}};
const Segment SGI287414[] = {{&S20661[13], 3}, {NULL}};
const Segment SGI287415[] = {{&S20661[7], 1}, {NULL}};
const Segment SGI287416[] = {{&S20662[61], 5}, {NULL}};
const Segment SGI287417[] = {{&S20662[8], 3}, {NULL}};
const Segment SGI287418[] = {{&S20662[13], 3}, {NULL}};
const Segment SGI287419[] = {{&S101194[7], 3}, {NULL}};
const Segment SGI287420[] = {{&S101194[11], 4}, {NULL}};
const Segment SGI287421[] = {{&S20955[63], 4}, {NULL}};
const Segment SGI287422[] = {{&S20955[8], 3}, {NULL}};
const Segment SGI287423[] = {{&S20955[13], 14}, {NULL}};
const Segment SGI287424[] = {{&S20955[7], 1}, {NULL}};
const Segment SGI287425[] = {{&S20955[16], 11}, {NULL}};
const Segment SGI287426[] = {{&S20955[70], 11}, {NULL}};
const Segment SGI287427[] = {{&S20956[63], 4}, {NULL}};
const Segment SGI287428[] = {{&S20956[8], 3}, {NULL}};
const Segment SGI287429[] = {{&S20956[13], 14}, {NULL}};
const Segment SGI287430[] = {{&S20956[7], 1}, {NULL}};
const Segment SGI287431[] = {{&S20956[16], 11}, {NULL}};
const Segment SGI287432[] = {{&S20956[70], 11}, {NULL}};
const Segment SGI287433[] = {{&S20957[70], 4}, {NULL}};
const Segment SGI287434[] = {{&S20957[8], 3}, {NULL}};
const Segment SGI287435[] = {{&S20957[13], 7}, {NULL}};
const Segment SGI287436[] = {{&S20957[25], 14}, {NULL}};
const Segment SGI287437[] = {{&S20957[7], 1}, {&S20957[11], 1}, {NULL}};
const Segment SGI287438[] = {{&S20957[16], 4}, {&S20957[28], 11}, {NULL}};
const Segment SGI287439[] = {{&S20958[34], 4}, {NULL}};
const Segment SGI287440[] = {{&S20958[8], 3}, {NULL}};
const Segment SGI287441[] = {{&S20958[13], 14}, {NULL}};
const Segment SGI287442[] = {{&S20958[7], 1}, {NULL}};
const Segment SGI287443[] = {{&S20958[16], 11}, {NULL}};
const Segment SGI287444[] = {{&S20958[80], 11}, {NULL}};
const Segment SGI287445[] = {{&S20959[66], 4}, {NULL}};
const Segment SGI287446[] = {{&S20959[8], 3}, {NULL}};
const Segment SGI287447[] = {{&S20959[13], 14}, {NULL}};
const Segment SGI287448[] = {{&S20959[7], 1}, {NULL}};
const Segment SGI287449[] = {{&S20959[16], 11}, {NULL}};
const Segment SGI287450[] = {{&S20959[73], 11}, {NULL}};
const Segment SGI287451[] = {{&S20960[63], 4}, {NULL}};
const Segment SGI287452[] = {{&S20960[8], 3}, {NULL}};
const Segment SGI287453[] = {{&S20960[13], 14}, {NULL}};
const Segment SGI287454[] = {{&S20960[7], 1}, {NULL}};
const Segment SGI287455[] = {{&S20960[16], 11}, {NULL}};
const Segment SGI287456[] = {{&S20960[70], 11}, {NULL}};
const Segment SGI287457[] = {{&S20961[63], 4}, {NULL}};
const Segment SGI287458[] = {{&S20961[8], 3}, {NULL}};
const Segment SGI287459[] = {{&S20961[13], 14}, {NULL}};
const Segment SGI287460[] = {{&S20961[7], 1}, {NULL}};
const Segment SGI287461[] = {{&S20961[16], 11}, {NULL}};
const Segment SGI287462[] = {{&S20961[70], 11}, {NULL}};
const Segment SGI287463[] = {{&S20962[63], 4}, {NULL}};
const Segment SGI287464[] = {{&S20962[8], 3}, {NULL}};
const Segment SGI287465[] = {{&S20962[13], 14}, {NULL}};
const Segment SGI287466[] = {{&S20962[7], 1}, {NULL}};
const Segment SGI287467[] = {{&S20962[16], 11}, {NULL}};
const Segment SGI287468[] = {{&S20962[70], 11}, {NULL}};
const Segment SGI287469[] = {{&S20963[63], 4}, {NULL}};
const Segment SGI287470[] = {{&S20963[8], 3}, {NULL}};
const Segment SGI287471[] = {{&S20963[13], 14}, {NULL}};
const Segment SGI287472[] = {{&S20963[7], 1}, {NULL}};
const Segment SGI287473[] = {{&S20963[16], 11}, {NULL}};
const Segment SGI287474[] = {{&S20963[70], 11}, {NULL}};
const Segment SGI287475[] = {{&S20965[66], 4}, {NULL}};
const Segment SGI287476[] = {{&S20965[8], 3}, {NULL}};
const Segment SGI287477[] = {{&S20965[13], 14}, {NULL}};
const Segment SGI287478[] = {{&S20965[7], 1}, {NULL}};
const Segment SGI287479[] = {{&S20965[16], 11}, {NULL}};
const Segment SGI287480[] = {{&S20966[75], 4}, {NULL}};
const Segment SGI287481[] = {{&S20966[8], 3}, {NULL}};
const Segment SGI287482[] = {{&S20966[13], 7}, {NULL}};
const Segment SGI287483[] = {{&S20966[25], 14}, {NULL}};
const Segment SGI287484[] = {{&S20966[7], 1}, {NULL}};
const Segment SGI287485[] = {{&S20966[16], 4}, {&S20966[28], 11}, {NULL}};
const Segment SGI287486[] = {{&S101484[7], 3}, {NULL}};
const Segment SGI287487[] = {{&S101484[11], 8}, {NULL}};
const Segment SGI287488[] = {{&S21270[75], 4}, {NULL}};
const Segment SGI287489[] = {{&S21270[8], 3}, {NULL}};
const Segment SGI287490[] = {{&S21270[13], 7}, {NULL}};
const Segment SGI287491[] = {{&S21270[25], 14}, {NULL}};
const Segment SGI287492[] = {{&S21270[7], 1}, {NULL}};
const Segment SGI287493[] = {{&S21270[16], 4}, {&S21270[28], 11}, {NULL}};
const Segment SGI287494[] = {{&S21271[75], 4}, {NULL}};
const Segment SGI287495[] = {{&S21271[8], 3}, {NULL}};
const Segment SGI287496[] = {{&S21271[13], 7}, {NULL}};
const Segment SGI287497[] = {{&S21271[25], 14}, {NULL}};
const Segment SGI287498[] = {{&S21271[7], 1}, {NULL}};
const Segment SGI287499[] = {{&S21271[16], 4}, {&S21271[28], 11}, {&S21271[106], 17}, {NULL}};
const Segment SGI287500[] = {{&S21272[75], 4}, {NULL}};
const Segment SGI287501[] = {{&S21272[8], 3}, {NULL}};
const Segment SGI287502[] = {{&S21272[13], 7}, {NULL}};
const Segment SGI287503[] = {{&S21272[25], 14}, {NULL}};
const Segment SGI287504[] = {{&S21272[7], 1}, {NULL}};
const Segment SGI287505[] = {{&S21272[16], 4}, {&S21272[28], 11}, {&S21272[106], 14}, {NULL}};
const Segment SGI287506[] = {{&S21273[76], 4}, {NULL}};
const Segment SGI287507[] = {{&S21273[8], 3}, {NULL}};
const Segment SGI287508[] = {{&S21273[13], 7}, {NULL}};
const Segment SGI287509[] = {{&S21273[25], 14}, {NULL}};
const Segment SGI287510[] = {{&S21273[7], 1}, {NULL}};
const Segment SGI287511[] = {{&S21273[16], 4}, {&S21273[28], 11}, {&S21273[108], 17}, {NULL}};
const Segment SGI287512[] = {{&S21274[66], 4}, {&S21274[66], 4}, {NULL}};
const Segment SGI287513[] = {{&S21274[8], 3}, {&S21274[8], 3}, {NULL}};
const Segment SGI287514[] = {{&S21274[13], 7}, {&S21274[13], 7}, {NULL}};
const Segment SGI287515[] = {{&S21274[25], 14}, {&S21274[97], 14}, {NULL}};
const Segment SGI287516[] = {{&S21274[7], 1}, {&S21274[7], 1}, {NULL}};
const Segment SGI287517[] = {{&S21274[16], 4}, {&S21274[28], 11}, {&S21274[16], 4}, {&S21274[100], 11}, {NULL}};
const Segment SGI287518[] = {{&S21275[63], 4}, {NULL}};
const Segment SGI287519[] = {{&S21275[8], 3}, {NULL}};
const Segment SGI287520[] = {{&S21275[13], 14}, {NULL}};
const Segment SGI287521[] = {{&S21275[7], 1}, {NULL}};
const Segment SGI287522[] = {{&S21275[16], 11}, {NULL}};
const Segment SGI287523[] = {{&S21276[75], 4}, {NULL}};
const Segment SGI287524[] = {{&S21276[8], 3}, {NULL}};
const Segment SGI287525[] = {{&S21276[13], 7}, {NULL}};
const Segment SGI287526[] = {{&S21276[25], 14}, {NULL}};
const Segment SGI287527[] = {{&S21276[7], 1}, {NULL}};
const Segment SGI287528[] = {{&S21276[16], 4}, {&S21276[28], 11}, {&S21276[107], 14}, {NULL}};
const Segment SGI287529[] = {{&S21299[56], 4}, {NULL}};
const Segment SGI287530[] = {{&S21299[8], 3}, {NULL}};
const Segment SGI287531[] = {{&S21299[13], 14}, {NULL}};
const Segment SGI287532[] = {{&S21299[7], 1}, {NULL}};
const Segment SGI287533[] = {{&S21299[16], 11}, {NULL}};
const Segment SGI287534[] = {{&S21299[61], 11}, {NULL}};
const Segment SGI287535[] = {{&S101657[7], 3}, {NULL}};
const Segment SGI287536[] = {{&S101657[11], 6}, {NULL}};
const Segment SGI287537[] = {{&S101815[7], 3}, {NULL}};
const Segment SGI287538[] = {{&S101815[11], 6}, {NULL}};
const Segment SGI287539[] = {{&S21722[17], 4}, {NULL}};
const Segment SGI287540[] = {{&S21722[68], 4}, {NULL}};
const Segment SGI287541[] = {{&S21722[8], 3}, {NULL}};
const Segment SGI287542[] = {{&S21722[13], 3}, {NULL}};
const Segment SGI287543[] = {{&S21722[7], 1}, {NULL}};
const Segment SGI287544[] = {{&S21723[67], 4}, {NULL}};
const Segment SGI287545[] = {{&S21723[8], 3}, {NULL}};
const Segment SGI287546[] = {{&S21723[13], 3}, {NULL}};
const Segment SGI287547[] = {{&S21723[7], 1}, {NULL}};
const Segment SGI287548[] = {{&S21724[68], 4}, {&S21724[90], 4}, {NULL}};
const Segment SGI287549[] = {{&S21724[8], 3}, {NULL}};
const Segment SGI287550[] = {{&S21724[13], 3}, {NULL}};
const Segment SGI287551[] = {{&S21724[7], 1}, {&S21724[7], 1}, {NULL}};
const Segment SGI287552[] = {{&S21725[68], 4}, {NULL}};
const Segment SGI287553[] = {{&S21725[9], 3}, {NULL}};
const Segment SGI287554[] = {{&S21725[14], 3}, {NULL}};
const Segment SGI287555[] = {{&S21725[8], 1}, {NULL}};
const Segment SGI287556[] = {{&S21726[68], 5}, {&S21726[89], 4}, {NULL}};
const Segment SGI287557[] = {{&S21726[8], 3}, {NULL}};
const Segment SGI287558[] = {{&S21726[13], 3}, {NULL}};
const Segment SGI287559[] = {{&S21726[7], 1}, {&S21726[7], 1}, {NULL}};
const Segment SGI287560[] = {{&S21727[63], 4}, {&S21727[63], 4}, {NULL}};
const Segment SGI287561[] = {{&S21727[8], 3}, {NULL}};
const Segment SGI287562[] = {{&S21727[13], 3}, {NULL}};
const Segment SGI287563[] = {{&S21727[7], 1}, {&S21727[7], 1}, {NULL}};
const Segment SGI287564[] = {{&S21835[52], 4}, {NULL}};
const Segment SGI287565[] = {{&S21835[8], 3}, {NULL}};
const Segment SGI287566[] = {{&S21835[13], 14}, {NULL}};
const Segment SGI287567[] = {{&S21835[7], 1}, {NULL}};
const Segment SGI287568[] = {{&S21835[16], 11}, {NULL}};
const Segment SGI287569[] = {{&S21835[57], 11}, {NULL}};
const Segment SGI287570[] = {{&S21947[8], 3}, {NULL}};
const Segment SGI287571[] = {{&S21947[13], 3}, {NULL}};
const Segment SGI287572[] = {{&S21948[87], 4}, {NULL}};
const Segment SGI287573[] = {{&S21948[8], 3}, {NULL}};
const Segment SGI287574[] = {{&S21948[13], 3}, {NULL}};
const Segment SGI287575[] = {{&S21948[35], 1}, {&S21948[7], 1}, {NULL}};
const Segment SGI287576[] = {{&S21949[65], 4}, {NULL}};
const Segment SGI287577[] = {{&S21949[8], 3}, {NULL}};
const Segment SGI287578[] = {{&S21949[13], 3}, {NULL}};
const Segment SGI287579[] = {{&S21949[71], 10}, {NULL}};
const Segment SGI287580[] = {{&S21950[69], 4}, {NULL}};
const Segment SGI287581[] = {{&S21950[8], 3}, {NULL}};
const Segment SGI287582[] = {{&S21950[13], 3}, {NULL}};
const Segment SGI287583[] = {{&S21950[77], 10}, {NULL}};
const Segment SGI287584[] = {{&S21951[82], 4}, {NULL}};
const Segment SGI287585[] = {{&S21951[8], 3}, {NULL}};
const Segment SGI287586[] = {{&S21951[13], 3}, {NULL}};
const Segment SGI287587[] = {{&S21951[7], 1}, {NULL}};
const Segment SGI287588[] = {{&S21952[8], 3}, {NULL}};
const Segment SGI287589[] = {{&S21952[13], 3}, {NULL}};
const Segment SGI287590[] = {{&S21952[76], 10}, {NULL}};
const Segment SGI287591[] = {{&S21952[61], 11}, {NULL}};
const Segment SGI287592[] = {{&S21953[67], 4}, {NULL}};
const Segment SGI287593[] = {{&S21953[8], 3}, {NULL}};
const Segment SGI287594[] = {{&S21953[13], 3}, {NULL}};
const Segment SGI287595[] = {{&S21953[73], 10}, {NULL}};
const Segment SGI287596[] = {{&S21953[7], 1}, {NULL}};
const Segment SGI287597[] = {{&S21954[69], 4}, {NULL}};
const Segment SGI287598[] = {{&S21954[8], 3}, {NULL}};
const Segment SGI287599[] = {{&S21954[13], 3}, {NULL}};
const Segment SGI287600[] = {{&S21955[68], 4}, {NULL}};
const Segment SGI287601[] = {{&S21955[8], 3}, {NULL}};
const Segment SGI287602[] = {{&S21955[13], 3}, {NULL}};
const Segment SGI287603[] = {{&S21955[77], 10}, {NULL}};
const Segment SGI287604[] = {{&S21955[7], 1}, {NULL}};
const Segment SGI287605[] = {{&S21956[70], 4}, {NULL}};
const Segment SGI287606[] = {{&S21956[8], 3}, {NULL}};
const Segment SGI287607[] = {{&S21956[13], 3}, {NULL}};
const Segment SGI287608[] = {{&S21956[7], 1}, {NULL}};
const Segment SGI287609[] = {{&S21956[81], 12}, {NULL}};
const Segment SGI287610[] = {{&S21961[78], 4}, {NULL}};
const Segment SGI287611[] = {{&S21961[8], 3}, {NULL}};
const Segment SGI287612[] = {{&S21961[13], 3}, {NULL}};
const Segment SGI287613[] = {{&S102217[7], 3}, {NULL}};
const Segment SGI287614[] = {{&S102217[11], 4}, {NULL}};
const Segment SGI287615[] = {{&S22096[52], 4}, {NULL}};
const Segment SGI287616[] = {{&S22096[8], 3}, {NULL}};
const Segment SGI287617[] = {{&S22096[13], 14}, {NULL}};
const Segment SGI287618[] = {{&S22096[7], 1}, {NULL}};
const Segment SGI287619[] = {{&S22096[16], 11}, {NULL}};
const Segment SGI287620[] = {{&S22096[57], 11}, {NULL}};
const Segment SGI287621[] = {{&S8260[0], 168}, {NULL}};
const Segment SGI287622[] = {{&S22524[61], 4}, {NULL}};
const Segment SGI287623[] = {{&S22524[8], 3}, {NULL}};
const Segment SGI287624[] = {{&S22524[13], 14}, {NULL}};
const Segment SGI287625[] = {{&S22524[7], 1}, {NULL}};
const Segment SGI287626[] = {{&S22524[16], 11}, {NULL}};
const Segment SGI287627[] = {{&S22524[68], 10}, {NULL}};
const Segment SGI287628[] = {{&S22528[64], 4}, {NULL}};
const Segment SGI287629[] = {{&S22528[8], 3}, {NULL}};
const Segment SGI287630[] = {{&S22528[13], 14}, {NULL}};
const Segment SGI287631[] = {{&S22528[7], 1}, {NULL}};
const Segment SGI287632[] = {{&S22528[16], 11}, {NULL}};
const Segment SGI287633[] = {{&S22528[71], 10}, {NULL}};
const Segment SGI287634[] = {{&S22541[61], 4}, {NULL}};
const Segment SGI287635[] = {{&S22541[8], 3}, {NULL}};
const Segment SGI287636[] = {{&S22541[13], 14}, {NULL}};
const Segment SGI287637[] = {{&S22541[7], 1}, {NULL}};
const Segment SGI287638[] = {{&S22541[16], 11}, {NULL}};
const Segment SGI287639[] = {{&S22541[68], 10}, {NULL}};
const Segment SGI287640[] = {{&S22551[61], 4}, {NULL}};
const Segment SGI287641[] = {{&S22551[8], 3}, {NULL}};
const Segment SGI287642[] = {{&S22551[13], 14}, {NULL}};
const Segment SGI287643[] = {{&S22551[7], 1}, {NULL}};
const Segment SGI287644[] = {{&S22551[16], 11}, {NULL}};
const Segment SGI287645[] = {{&S22551[68], 10}, {NULL}};
const Segment SGI287646[] = {{&S22642[82], 4}, {NULL}};
const Segment SGI287647[] = {{&S22642[8], 3}, {NULL}};
const Segment SGI287648[] = {{&S22642[13], 9}, {NULL}};
const Segment SGI287649[] = {{&S22642[33], 13}, {NULL}};
const Segment SGI287650[] = {{&S22642[7], 1}, {NULL}};
const Segment SGI287651[] = {{&S22643[81], 4}, {NULL}};
const Segment SGI287652[] = {{&S22643[8], 3}, {NULL}};
const Segment SGI287653[] = {{&S22643[13], 9}, {NULL}};
const Segment SGI287654[] = {{&S22643[33], 19}, {NULL}};
const Segment SGI287655[] = {{&S22643[7], 1}, {NULL}};
const Segment SGI287656[] = {{&S22644[70], 4}, {NULL}};
const Segment SGI287657[] = {{&S22644[8], 3}, {NULL}};
const Segment SGI287658[] = {{&S22644[13], 9}, {NULL}};
const Segment SGI287659[] = {{&S22644[33], 13}, {NULL}};
const Segment SGI287660[] = {{&S22644[7], 1}, {NULL}};
const Segment SGI287661[] = {{&S22645[96], 4}, {NULL}};
const Segment SGI287662[] = {{&S22645[8], 3}, {NULL}};
const Segment SGI287663[] = {{&S22645[13], 9}, {NULL}};
const Segment SGI287664[] = {{&S22645[33], 13}, {NULL}};
const Segment SGI287665[] = {{&S22645[87], 10}, {NULL}};
const Segment SGI287666[] = {{&S22646[8], 3}, {NULL}};
const Segment SGI287667[] = {{&S22646[13], 9}, {NULL}};
const Segment SGI287668[] = {{&S22646[33], 13}, {NULL}};
const Segment SGI287669[] = {{&S22646[11], 1}, {NULL}};
const Segment SGI287670[] = {{&S22815[60], 5}, {NULL}};
const Segment SGI287671[] = {{&S22815[8], 3}, {NULL}};
const Segment SGI287672[] = {{&S22815[13], 14}, {NULL}};
const Segment SGI287673[] = {{&S22815[7], 1}, {NULL}};
const Segment SGI287674[] = {{&S22815[16], 11}, {NULL}};
const Segment SGI287675[] = {{&S22816[65], 5}, {NULL}};
const Segment SGI287676[] = {{&S22816[8], 3}, {NULL}};
const Segment SGI287677[] = {{&S22816[13], 14}, {NULL}};
const Segment SGI287678[] = {{&S22816[7], 1}, {NULL}};
const Segment SGI287679[] = {{&S22816[16], 11}, {&S22816[32], 2}, {NULL}};
const Segment SGI287680[] = {{&S22817[60], 6}, {NULL}};
const Segment SGI287681[] = {{&S22817[8], 3}, {NULL}};
const Segment SGI287682[] = {{&S22817[13], 14}, {NULL}};
const Segment SGI287683[] = {{&S22817[7], 1}, {NULL}};
const Segment SGI287684[] = {{&S22817[16], 11}, {NULL}};
const Segment SGI287685[] = {{&S22818[64], 5}, {NULL}};
const Segment SGI287686[] = {{&S22818[8], 3}, {NULL}};
const Segment SGI287687[] = {{&S22818[13], 7}, {NULL}};
const Segment SGI287688[] = {{&S22818[22], 14}, {NULL}};
const Segment SGI287689[] = {{&S22818[7], 1}, {NULL}};
const Segment SGI287690[] = {{&S22818[16], 4}, {&S22818[25], 11}, {&S22818[41], 2}, {NULL}};
const Segment SGI287691[] = {{&S22819[62], 5}, {NULL}};
const Segment SGI287692[] = {{&S22819[8], 3}, {NULL}};
const Segment SGI287693[] = {{&S22819[13], 14}, {NULL}};
const Segment SGI287694[] = {{&S22819[7], 1}, {NULL}};
const Segment SGI287695[] = {{&S22819[16], 11}, {NULL}};
const Segment SGI287696[] = {{&S22820[56], 5}, {NULL}};
const Segment SGI287697[] = {{&S22820[8], 3}, {NULL}};
const Segment SGI287698[] = {{&S22820[7], 1}, {NULL}};
const Segment SGI287699[] = {{&S22820[15], 46}, {NULL}};
const Segment SGI287700[] = {{&S22821[67], 5}, {NULL}};
const Segment SGI287701[] = {{&S22821[8], 3}, {NULL}};
const Segment SGI287702[] = {{&S22821[13], 7}, {NULL}};
const Segment SGI287703[] = {{&S22821[25], 14}, {NULL}};
const Segment SGI287704[] = {{&S22821[7], 1}, {NULL}};
const Segment SGI287705[] = {{&S22821[16], 4}, {&S22821[28], 11}, {NULL}};
const Segment SGI287706[] = {{&S22823[52], 4}, {NULL}};
const Segment SGI287707[] = {{&S22823[14], 14}, {NULL}};
const Segment SGI287708[] = {{&S22823[8], 1}, {NULL}};
const Segment SGI287709[] = {{&S22823[17], 11}, {NULL}};
const Segment SGI287710[] = {{&S22824[66], 4}, {&S22824[66], 4}, {NULL}};
const Segment SGI287711[] = {{&S22824[8], 3}, {&S22824[8], 3}, {NULL}};
const Segment SGI287712[] = {{&S22824[13], 7}, {&S22824[13], 7}, {NULL}};
const Segment SGI287713[] = {{&S22824[25], 14}, {&S22824[95], 14}, {NULL}};
const Segment SGI287714[] = {{&S22824[7], 1}, {&S22824[7], 1}, {NULL}};
const Segment SGI287715[] = {{&S22824[16], 4}, {&S22824[28], 11}, {&S22824[16], 4}, {&S22824[98], 11}, {NULL}};
const Segment SGI287716[] = {{&S22825[81], 4}, {NULL}};
const Segment SGI287717[] = {{&S22825[8], 3}, {NULL}};
const Segment SGI287718[] = {{&S22825[13], 7}, {NULL}};
const Segment SGI287719[] = {{&S22825[25], 14}, {NULL}};
const Segment SGI287720[] = {{&S22825[7], 1}, {NULL}};
const Segment SGI287721[] = {{&S22825[16], 4}, {&S22825[28], 11}, {&S22825[114], 35}, {NULL}};
const Segment SGI287722[] = {{&S22826[76], 4}, {NULL}};
const Segment SGI287723[] = {{&S22826[8], 3}, {NULL}};
const Segment SGI287724[] = {{&S22826[13], 7}, {NULL}};
const Segment SGI287725[] = {{&S22826[25], 14}, {NULL}};
const Segment SGI287726[] = {{&S22826[7], 1}, {NULL}};
const Segment SGI287727[] = {{&S22826[16], 4}, {&S22826[28], 11}, {&S22826[108], 17}, {NULL}};
const Segment SGI287728[] = {{&S22827[76], 4}, {NULL}};
const Segment SGI287729[] = {{&S22827[8], 3}, {NULL}};
const Segment SGI287730[] = {{&S22827[13], 7}, {NULL}};
const Segment SGI287731[] = {{&S22827[25], 14}, {NULL}};
const Segment SGI287732[] = {{&S22827[7], 1}, {NULL}};
const Segment SGI287733[] = {{&S22827[16], 4}, {&S22827[28], 11}, {&S22827[108], 25}, {NULL}};
const Segment SGI287734[] = {{&S22828[76], 4}, {NULL}};
const Segment SGI287735[] = {{&S22828[8], 3}, {NULL}};
const Segment SGI287736[] = {{&S22828[13], 7}, {NULL}};
const Segment SGI287737[] = {{&S22828[25], 14}, {NULL}};
const Segment SGI287738[] = {{&S22828[7], 1}, {NULL}};
const Segment SGI287739[] = {{&S22828[16], 4}, {&S22828[28], 11}, {&S22828[108], 25}, {NULL}};
const Segment SGI287740[] = {{&S22829[70], 4}, {NULL}};
const Segment SGI287741[] = {{&S22829[8], 3}, {NULL}};
const Segment SGI287742[] = {{&S22829[13], 7}, {NULL}};
const Segment SGI287743[] = {{&S22829[25], 14}, {NULL}};
const Segment SGI287744[] = {{&S22829[7], 1}, {NULL}};
const Segment SGI287745[] = {{&S22829[16], 4}, {&S22829[28], 11}, {NULL}};
const Segment SGI287746[] = {{&S22831[81], 4}, {NULL}};
const Segment SGI287747[] = {{&S22831[8], 3}, {NULL}};
const Segment SGI287748[] = {{&S22831[13], 7}, {NULL}};
const Segment SGI287749[] = {{&S22831[25], 14}, {NULL}};
const Segment SGI287750[] = {{&S22831[7], 1}, {NULL}};
const Segment SGI287751[] = {{&S22831[16], 4}, {&S22831[28], 11}, {&S22831[113], 39}, {NULL}};
const Segment SGI287752[] = {{&S22832[75], 4}, {NULL}};
const Segment SGI287753[] = {{&S22832[8], 3}, {NULL}};
const Segment SGI287754[] = {{&S22832[13], 7}, {NULL}};
const Segment SGI287755[] = {{&S22832[25], 14}, {NULL}};
const Segment SGI287756[] = {{&S22832[7], 1}, {NULL}};
const Segment SGI287757[] = {{&S22832[16], 4}, {&S22832[28], 11}, {NULL}};
const Segment SGI287758[] = {{&S22833[76], 4}, {NULL}};
const Segment SGI287759[] = {{&S22833[8], 3}, {NULL}};
const Segment SGI287760[] = {{&S22833[13], 7}, {NULL}};
const Segment SGI287761[] = {{&S22833[25], 14}, {NULL}};
const Segment SGI287762[] = {{&S22833[7], 1}, {NULL}};
const Segment SGI287763[] = {{&S22833[16], 4}, {&S22833[28], 11}, {&S22833[87], 17}, {NULL}};
const Segment SGI287764[] = {{&S22834[76], 4}, {NULL}};
const Segment SGI287765[] = {{&S22834[8], 3}, {NULL}};
const Segment SGI287766[] = {{&S22834[13], 7}, {NULL}};
const Segment SGI287767[] = {{&S22834[25], 14}, {NULL}};
const Segment SGI287768[] = {{&S22834[7], 1}, {NULL}};
const Segment SGI287769[] = {{&S22834[16], 4}, {&S22834[28], 11}, {&S22834[108], 25}, {NULL}};
const Segment SGI287770[] = {{&S22835[79], 4}, {NULL}};
const Segment SGI287771[] = {{&S22835[29], 14}, {NULL}};
const Segment SGI287772[] = {{&S22835[11], 1}, {NULL}};
const Segment SGI287773[] = {{&S22835[20], 4}, {&S22835[32], 11}, {NULL}};
const Segment SGI287774[] = {{&S22836[76], 4}, {NULL}};
const Segment SGI287775[] = {{&S22836[8], 3}, {NULL}};
const Segment SGI287776[] = {{&S22836[13], 7}, {NULL}};
const Segment SGI287777[] = {{&S22836[25], 14}, {NULL}};
const Segment SGI287778[] = {{&S22836[7], 1}, {NULL}};
const Segment SGI287779[] = {{&S22836[16], 4}, {&S22836[28], 11}, {&S22836[113], 22}, {NULL}};
const Segment SGI287780[] = {{&S23005[53], 5}, {NULL}};
const Segment SGI287781[] = {{&S23005[8], 3}, {NULL}};
const Segment SGI287782[] = {{&S23005[13], 14}, {NULL}};
const Segment SGI287783[] = {{&S23005[7], 1}, {NULL}};
const Segment SGI287784[] = {{&S23005[16], 11}, {NULL}};
const Segment SGI287785[] = {{&S23006[65], 5}, {NULL}};
const Segment SGI287786[] = {{&S23006[8], 3}, {NULL}};
const Segment SGI287787[] = {{&S23006[13], 14}, {NULL}};
const Segment SGI287788[] = {{&S23006[7], 1}, {NULL}};
const Segment SGI287789[] = {{&S23006[16], 11}, {&S23006[32], 2}, {NULL}};
const Segment SGI287790[] = {{&S23007[60], 5}, {&S23007[60], 5}, {NULL}};
const Segment SGI287791[] = {{&S23007[8], 3}, {NULL}};
const Segment SGI287792[] = {{&S23007[13], 14}, {NULL}};
const Segment SGI287793[] = {{&S23007[7], 1}, {&S23007[7], 1}, {NULL}};
const Segment SGI287794[] = {{&S23007[16], 11}, {NULL}};
const Segment SGI287795[] = {{&S23008[106], 5}, {NULL}};
const Segment SGI287796[] = {{&S23008[8], 3}, {NULL}};
const Segment SGI287797[] = {{&S23008[13], 7}, {NULL}};
const Segment SGI287798[] = {{&S23008[25], 14}, {NULL}};
const Segment SGI287799[] = {{&S23008[7], 1}, {NULL}};
const Segment SGI287800[] = {{&S23008[16], 4}, {&S23008[28], 11}, {NULL}};
const Segment SGI287801[] = {{&S23009[74], 5}, {NULL}};
const Segment SGI287802[] = {{&S23009[8], 3}, {NULL}};
const Segment SGI287803[] = {{&S23009[13], 7}, {NULL}};
const Segment SGI287804[] = {{&S23009[22], 14}, {NULL}};
const Segment SGI287805[] = {{&S23009[7], 1}, {NULL}};
const Segment SGI287806[] = {{&S23009[16], 4}, {&S23009[25], 11}, {&S23009[41], 2}, {NULL}};
const Segment SGI287807[] = {{&S23099[55], 5}, {NULL}};
const Segment SGI287808[] = {{&S23099[8], 3}, {NULL}};
const Segment SGI287809[] = {{&S23099[13], 14}, {NULL}};
const Segment SGI287810[] = {{&S23099[7], 1}, {NULL}};
const Segment SGI287811[] = {{&S23099[16], 11}, {&S23099[32], 2}, {NULL}};
const Segment SGI287812[] = {{&S23100[53], 5}, {NULL}};
const Segment SGI287813[] = {{&S23100[8], 3}, {NULL}};
const Segment SGI287814[] = {{&S23100[13], 14}, {NULL}};
const Segment SGI287815[] = {{&S23100[7], 1}, {NULL}};
const Segment SGI287816[] = {{&S23100[16], 11}, {NULL}};
const Segment SGI287817[] = {{&S23101[60], 5}, {&S23101[60], 5}, {NULL}};
const Segment SGI287818[] = {{&S23101[8], 3}, {NULL}};
const Segment SGI287819[] = {{&S23101[13], 14}, {NULL}};
const Segment SGI287820[] = {{&S23101[7], 1}, {&S23101[7], 1}, {NULL}};
const Segment SGI287821[] = {{&S23101[16], 11}, {NULL}};
const Segment SGI287822[] = {{&S23102[65], 5}, {&S23102[65], 5}, {NULL}};
const Segment SGI287823[] = {{&S23102[8], 3}, {NULL}};
const Segment SGI287824[] = {{&S23102[13], 14}, {NULL}};
const Segment SGI287825[] = {{&S23102[7], 1}, {&S23102[7], 1}, {NULL}};
const Segment SGI287826[] = {{&S23102[16], 11}, {&S23102[32], 2}, {NULL}};
const Segment SGI287827[] = {{&S23103[73], 5}, {NULL}};
const Segment SGI287828[] = {{&S23103[8], 3}, {&S23103[8], 3}, {NULL}};
const Segment SGI287829[] = {{&S23103[13], 27}, {NULL}};
const Segment SGI287830[] = {{&S23103[26], 14}, {NULL}};
const Segment SGI287831[] = {{&S23103[7], 1}, {NULL}};
const Segment SGI287832[] = {{&S23103[29], 11}, {NULL}};
const Segment SGI287833[] = {{&S23104[65], 5}, {NULL}};
const Segment SGI287834[] = {{&S23104[8], 3}, {NULL}};
const Segment SGI287835[] = {{&S23104[13], 7}, {NULL}};
const Segment SGI287836[] = {{&S23104[25], 14}, {NULL}};
const Segment SGI287837[] = {{&S23104[7], 1}, {NULL}};
const Segment SGI287838[] = {{&S23104[16], 4}, {&S23104[28], 11}, {NULL}};
const Segment SGI287839[] = {{&S23188[52], 4}, {NULL}};
const Segment SGI287840[] = {{&S23188[8], 3}, {NULL}};
const Segment SGI287841[] = {{&S23188[13], 14}, {NULL}};
const Segment SGI287842[] = {{&S23188[7], 1}, {NULL}};
const Segment SGI287843[] = {{&S23188[16], 11}, {NULL}};
const Segment SGI287844[] = {{&S23188[57], 11}, {NULL}};
const Segment SGI287845[] = {{&S23712[52], 4}, {NULL}};
const Segment SGI287846[] = {{&S23712[8], 3}, {NULL}};
const Segment SGI287847[] = {{&S23712[13], 14}, {NULL}};
const Segment SGI287848[] = {{&S23712[7], 1}, {NULL}};
const Segment SGI287849[] = {{&S23712[16], 11}, {NULL}};
const Segment SGI287850[] = {{&S23712[57], 11}, {NULL}};
const Segment SGI287851[] = {{&S103938[7], 3}, {NULL}};
const Segment SGI287852[] = {{&S103938[11], 5}, {NULL}};
const Segment SGI287853[] = {{&S23966[43], 5}, {NULL}};
const Segment SGI287854[] = {{&S23966[8], 3}, {NULL}};
const Segment SGI287855[] = {{&S23966[13], 14}, {NULL}};
const Segment SGI287856[] = {{&S23966[7], 1}, {NULL}};
const Segment SGI287857[] = {{&S23966[16], 11}, {NULL}};
const Segment SGI287858[] = {{&S23967[55], 5}, {NULL}};
const Segment SGI287859[] = {{&S23967[8], 3}, {NULL}};
const Segment SGI287860[] = {{&S23967[13], 14}, {NULL}};
const Segment SGI287861[] = {{&S23967[7], 1}, {NULL}};
const Segment SGI287862[] = {{&S23967[16], 11}, {&S23967[32], 2}, {NULL}};
const Segment SGI287863[] = {{&S23968[43], 5}, {NULL}};
const Segment SGI287864[] = {{&S23968[8], 3}, {NULL}};
const Segment SGI287865[] = {{&S23968[13], 14}, {NULL}};
const Segment SGI287866[] = {{&S23968[7], 1}, {NULL}};
const Segment SGI287867[] = {{&S23968[16], 11}, {NULL}};
const Segment SGI287868[] = {{&S23969[65], 5}, {NULL}};
const Segment SGI287869[] = {{&S23969[8], 3}, {NULL}};
const Segment SGI287870[] = {{&S23969[24], 14}, {NULL}};
const Segment SGI287871[] = {{&S23969[7], 1}, {NULL}};
const Segment SGI287872[] = {{&S23969[15], 4}, {&S23969[27], 11}, {NULL}};
const Segment SGI287873[] = {{&S23970[75], 5}, {NULL}};
const Segment SGI287874[] = {{&S23970[8], 3}, {NULL}};
const Segment SGI287875[] = {{&S23970[13], 7}, {NULL}};
const Segment SGI287876[] = {{&S23970[25], 14}, {NULL}};
const Segment SGI287877[] = {{&S23970[7], 1}, {NULL}};
const Segment SGI287878[] = {{&S23970[16], 4}, {&S23970[28], 11}, {NULL}};
const Segment SGI287879[] = {{&S23971[72], 5}, {NULL}};
const Segment SGI287880[] = {{&S23971[8], 3}, {NULL}};
const Segment SGI287881[] = {{&S23971[13], 7}, {NULL}};
const Segment SGI287882[] = {{&S23971[25], 14}, {NULL}};
const Segment SGI287883[] = {{&S23971[7], 1}, {NULL}};
const Segment SGI287884[] = {{&S23971[16], 4}, {&S23971[28], 11}, {NULL}};
const Segment SGI287885[] = {{&S24209[64], 4}, {NULL}};
const Segment SGI287886[] = {{&S24209[8], 3}, {NULL}};
const Segment SGI287887[] = {{&S24209[13], 14}, {NULL}};
const Segment SGI287888[] = {{&S24209[7], 1}, {NULL}};
const Segment SGI287889[] = {{&S24209[16], 11}, {NULL}};
const Segment SGI287890[] = {{&S24209[71], 10}, {NULL}};
const Segment SGI287891[] = {{&S24211[61], 4}, {NULL}};
const Segment SGI287892[] = {{&S24211[8], 3}, {NULL}};
const Segment SGI287893[] = {{&S24211[13], 14}, {NULL}};
const Segment SGI287894[] = {{&S24211[7], 1}, {NULL}};
const Segment SGI287895[] = {{&S24211[16], 11}, {NULL}};
const Segment SGI287896[] = {{&S24211[68], 10}, {NULL}};
const Segment SGI287897[] = {{&S24213[61], 4}, {NULL}};
const Segment SGI287898[] = {{&S24213[8], 3}, {NULL}};
const Segment SGI287899[] = {{&S24213[13], 14}, {NULL}};
const Segment SGI287900[] = {{&S24213[7], 1}, {NULL}};
const Segment SGI287901[] = {{&S24213[16], 11}, {NULL}};
const Segment SGI287902[] = {{&S24213[68], 10}, {NULL}};
const Segment SGI287903[] = {{&S24214[61], 4}, {NULL}};
const Segment SGI287904[] = {{&S24214[8], 3}, {NULL}};
const Segment SGI287905[] = {{&S24214[13], 14}, {NULL}};
const Segment SGI287906[] = {{&S24214[7], 1}, {NULL}};
const Segment SGI287907[] = {{&S24214[16], 11}, {NULL}};
const Segment SGI287908[] = {{&S24214[68], 10}, {NULL}};
const Segment SGI287909[] = {{&S24215[61], 4}, {NULL}};
const Segment SGI287910[] = {{&S24215[8], 3}, {NULL}};
const Segment SGI287911[] = {{&S24215[13], 14}, {NULL}};
const Segment SGI287912[] = {{&S24215[7], 1}, {NULL}};
const Segment SGI287913[] = {{&S24215[16], 11}, {NULL}};
const Segment SGI287914[] = {{&S24215[68], 10}, {NULL}};
const Segment SGI287915[] = {{&S24232[55], 5}, {NULL}};
const Segment SGI287916[] = {{&S24232[8], 3}, {NULL}};
const Segment SGI287917[] = {{&S24232[13], 14}, {NULL}};
const Segment SGI287918[] = {{&S24232[7], 1}, {NULL}};
const Segment SGI287919[] = {{&S24232[16], 11}, {&S24232[32], 2}, {NULL}};
const Segment SGI287920[] = {{&S24233[60], 5}, {NULL}};
const Segment SGI287921[] = {{&S24233[8], 3}, {NULL}};
const Segment SGI287922[] = {{&S24233[13], 14}, {NULL}};
const Segment SGI287923[] = {{&S24233[7], 1}, {NULL}};
const Segment SGI287924[] = {{&S24233[16], 11}, {NULL}};
const Segment SGI287925[] = {{&S24234[64], 5}, {NULL}};
const Segment SGI287926[] = {{&S24234[8], 3}, {NULL}};
const Segment SGI287927[] = {{&S24234[13], 7}, {NULL}};
const Segment SGI287928[] = {{&S24234[22], 14}, {NULL}};
const Segment SGI287929[] = {{&S24234[7], 1}, {NULL}};
const Segment SGI287930[] = {{&S24234[16], 4}, {&S24234[25], 11}, {&S24234[41], 2}, {NULL}};
const Segment SGI287931[] = {{&S24235[105], 5}, {NULL}};
const Segment SGI287932[] = {{&S24235[8], 3}, {NULL}};
const Segment SGI287933[] = {{&S24235[13], 7}, {NULL}};
const Segment SGI287934[] = {{&S24235[25], 14}, {NULL}};
const Segment SGI287935[] = {{&S24235[7], 1}, {NULL}};
const Segment SGI287936[] = {{&S24235[16], 4}, {&S24235[28], 11}, {NULL}};
const Segment SGI287937[] = {{&S24260[62], 5}, {&S24260[62], 5}, {NULL}};
const Segment SGI287938[] = {{&S24260[8], 3}, {NULL}};
const Segment SGI287939[] = {{&S24260[13], 14}, {NULL}};
const Segment SGI287940[] = {{&S24260[7], 1}, {&S24260[7], 1}, {NULL}};
const Segment SGI287941[] = {{&S24260[16], 11}, {NULL}};
const Segment SGI287942[] = {{&S24261[67], 5}, {NULL}};
const Segment SGI287943[] = {{&S24261[8], 3}, {NULL}};
const Segment SGI287944[] = {{&S24261[13], 14}, {NULL}};
const Segment SGI287945[] = {{&S24261[7], 1}, {NULL}};
const Segment SGI287946[] = {{&S24261[16], 11}, {&S24261[32], 2}, {NULL}};
const Segment SGI287947[] = {{&S24262[62], 5}, {NULL}};
const Segment SGI287948[] = {{&S24262[8], 3}, {NULL}};
const Segment SGI287949[] = {{&S24262[13], 7}, {NULL}};
const Segment SGI287950[] = {{&S24262[22], 14}, {NULL}};
const Segment SGI287951[] = {{&S24262[7], 1}, {NULL}};
const Segment SGI287952[] = {{&S24262[16], 4}, {&S24262[25], 11}, {NULL}};
const Segment SGI287953[] = {{&S24263[63], 5}, {NULL}};
const Segment SGI287954[] = {{&S24263[8], 3}, {NULL}};
const Segment SGI287955[] = {{&S24263[13], 14}, {NULL}};
const Segment SGI287956[] = {{&S24263[7], 1}, {NULL}};
const Segment SGI287957[] = {{&S24263[16], 11}, {NULL}};
const Segment SGI287958[] = {{&S24264[24], 14}, {NULL}};
const Segment SGI287959[] = {{&S24264[7], 1}, {NULL}};
const Segment SGI287960[] = {{&S24264[15], 4}, {&S24264[27], 11}, {NULL}};
const Segment SGI287961[] = {{&S24288[81], 4}, {NULL}};
const Segment SGI287962[] = {{&S24288[8], 3}, {NULL}};
const Segment SGI287963[] = {{&S24288[13], 9}, {NULL}};
const Segment SGI287964[] = {{&S24288[31], 13}, {NULL}};
const Segment SGI287965[] = {{&S24288[7], 1}, {NULL}};
const Segment SGI287966[] = {{&S24615[50], 5}, {NULL}};
const Segment SGI287967[] = {{&S24615[8], 3}, {NULL}};
const Segment SGI287968[] = {{&S24615[13], 14}, {NULL}};
const Segment SGI287969[] = {{&S24615[7], 1}, {NULL}};
const Segment SGI287970[] = {{&S24615[16], 11}, {NULL}};
const Segment SGI287971[] = {{&S24616[65], 5}, {NULL}};
const Segment SGI287972[] = {{&S24616[8], 3}, {NULL}};
const Segment SGI287973[] = {{&S24616[13], 14}, {NULL}};
const Segment SGI287974[] = {{&S24616[7], 1}, {NULL}};
const Segment SGI287975[] = {{&S24616[16], 11}, {&S24616[32], 2}, {NULL}};
const Segment SGI287976[] = {{&S24617[64], 5}, {NULL}};
const Segment SGI287977[] = {{&S24617[8], 3}, {NULL}};
const Segment SGI287978[] = {{&S24617[13], 7}, {NULL}};
const Segment SGI287979[] = {{&S24617[22], 14}, {NULL}};
const Segment SGI287980[] = {{&S24617[7], 1}, {NULL}};
const Segment SGI287981[] = {{&S24617[16], 4}, {&S24617[25], 11}, {&S24617[41], 2}, {NULL}};
const Segment SGI287982[] = {{&S24618[65], 5}, {NULL}};
const Segment SGI287983[] = {{&S24618[8], 3}, {NULL}};
const Segment SGI287984[] = {{&S24618[13], 7}, {NULL}};
const Segment SGI287985[] = {{&S24618[25], 14}, {NULL}};
const Segment SGI287986[] = {{&S24618[7], 1}, {NULL}};
const Segment SGI287987[] = {{&S24618[16], 4}, {&S24618[28], 11}, {NULL}};
const Segment SGI287988[] = {{&S10431[0], 162}, {NULL}};
const Segment SGI287989[] = {{&S25139[62], 5}, {&S25139[62], 5}, {NULL}};
const Segment SGI287990[] = {{&S25139[8], 3}, {NULL}};
const Segment SGI287991[] = {{&S25139[13], 3}, {NULL}};
const Segment SGI287992[] = {{&S25139[13], 3}, {&S25139[18], 6}, {NULL}};
const Segment SGI287993[] = {{&S25139[7], 1}, {&S25139[7], 1}, {NULL}};
const Segment SGI287994[] = {{&S25140[73], 5}, {NULL}};
const Segment SGI287995[] = {{&S25140[8], 3}, {NULL}};
const Segment SGI287996[] = {{&S25140[13], 3}, {NULL}};
const Segment SGI287997[] = {{&S25140[13], 3}, {&S25140[18], 6}, {NULL}};
const Segment SGI287998[] = {{&S25140[7], 1}, {NULL}};
const Segment SGI287999[] = {{&S25141[55], 5}, {NULL}};
const Segment SGI288000[] = {{&S25141[8], 3}, {NULL}};
const Segment SGI288001[] = {{&S25141[13], 14}, {NULL}};
const Segment SGI288002[] = {{&S25141[7], 1}, {NULL}};
const Segment SGI288003[] = {{&S25141[16], 11}, {&S25141[32], 2}, {NULL}};
const Segment SGI288004[] = {{&S25142[60], 5}, {NULL}};
const Segment SGI288005[] = {{&S25142[8], 3}, {NULL}};
const Segment SGI288006[] = {{&S25142[13], 14}, {NULL}};
const Segment SGI288007[] = {{&S25142[7], 1}, {NULL}};
const Segment SGI288008[] = {{&S25142[16], 11}, {NULL}};
const Segment SGI288009[] = {{&S25143[64], 5}, {NULL}};
const Segment SGI288010[] = {{&S25143[8], 3}, {NULL}};
const Segment SGI288011[] = {{&S25143[13], 7}, {NULL}};
const Segment SGI288012[] = {{&S25143[22], 14}, {NULL}};
const Segment SGI288013[] = {{&S25143[7], 1}, {NULL}};
const Segment SGI288014[] = {{&S25143[16], 4}, {&S25143[25], 11}, {&S25143[41], 2}, {NULL}};
const Segment SGI288015[] = {{&S25144[60], 5}, {NULL}};
const Segment SGI288016[] = {{&S25144[8], 3}, {NULL}};
const Segment SGI288017[] = {{&S25144[13], 14}, {NULL}};
const Segment SGI288018[] = {{&S25144[7], 1}, {NULL}};
const Segment SGI288019[] = {{&S25144[16], 11}, {NULL}};
const Segment SGI288020[] = {{&S25144[66], 13}, {NULL}};
const Segment SGI288021[] = {{&S25145[65], 5}, {NULL}};
const Segment SGI288022[] = {{&S25145[8], 3}, {NULL}};
const Segment SGI288023[] = {{&S25145[13], 14}, {NULL}};
const Segment SGI288024[] = {{&S25145[7], 1}, {NULL}};
const Segment SGI288025[] = {{&S25145[16], 11}, {&S25145[32], 2}, {NULL}};
const Segment SGI288026[] = {{&S25145[71], 13}, {NULL}};
const Segment SGI288027[] = {{&S25146[52], 5}, {NULL}};
const Segment SGI288028[] = {{&S25146[8], 3}, {NULL}};
const Segment SGI288029[] = {{&S25146[13], 7}, {NULL}};
const Segment SGI288030[] = {{&S25146[22], 14}, {NULL}};
const Segment SGI288031[] = {{&S25146[7], 1}, {NULL}};
const Segment SGI288032[] = {{&S25146[16], 4}, {&S25146[25], 11}, {NULL}};
const Segment SGI288033[] = {{&S25147[44], 5}, {NULL}};
const Segment SGI288034[] = {{&S25147[8], 3}, {NULL}};
const Segment SGI288035[] = {{&S25147[13], 14}, {NULL}};
const Segment SGI288036[] = {{&S25147[7], 1}, {NULL}};
const Segment SGI288037[] = {{&S25147[16], 33}, {NULL}};
const Segment SGI288038[] = {{&S25345[60], 5}, {NULL}};
const Segment SGI288039[] = {{&S25345[8], 3}, {NULL}};
const Segment SGI288040[] = {{&S25345[13], 14}, {NULL}};
const Segment SGI288041[] = {{&S25345[7], 1}, {NULL}};
const Segment SGI288042[] = {{&S25345[16], 11}, {NULL}};
const Segment SGI288043[] = {{&S25346[53], 5}, {&S25346[53], 5}, {NULL}};
const Segment SGI288044[] = {{&S25346[8], 3}, {NULL}};
const Segment SGI288045[] = {{&S25346[13], 14}, {NULL}};
const Segment SGI288046[] = {{&S25346[7], 1}, {&S25346[7], 1}, {NULL}};
const Segment SGI288047[] = {{&S25346[16], 11}, {NULL}};
const Segment SGI288048[] = {{&S25347[60], 5}, {NULL}};
const Segment SGI288049[] = {{&S25347[8], 3}, {NULL}};
const Segment SGI288050[] = {{&S25347[13], 14}, {NULL}};
const Segment SGI288051[] = {{&S25347[7], 1}, {NULL}};
const Segment SGI288052[] = {{&S25347[16], 11}, {NULL}};
const Segment SGI288053[] = {{&S25348[65], 5}, {NULL}};
const Segment SGI288054[] = {{&S25348[8], 3}, {NULL}};
const Segment SGI288055[] = {{&S25348[13], 14}, {NULL}};
const Segment SGI288056[] = {{&S25348[7], 1}, {NULL}};
const Segment SGI288057[] = {{&S25348[16], 11}, {&S25348[32], 2}, {NULL}};
const Segment SGI288058[] = {{&S25349[75], 5}, {NULL}};
const Segment SGI288059[] = {{&S25349[8], 3}, {NULL}};
const Segment SGI288060[] = {{&S25349[13], 7}, {NULL}};
const Segment SGI288061[] = {{&S25349[25], 14}, {NULL}};
const Segment SGI288062[] = {{&S25349[7], 1}, {NULL}};
const Segment SGI288063[] = {{&S25349[16], 4}, {&S25349[28], 11}, {NULL}};
const Segment SGI288064[] = {{&S25350[65], 5}, {NULL}};
const Segment SGI288065[] = {{&S25350[8], 3}, {NULL}};
const Segment SGI288066[] = {{&S25350[24], 14}, {NULL}};
const Segment SGI288067[] = {{&S25350[7], 1}, {NULL}};
const Segment SGI288068[] = {{&S25350[15], 4}, {&S25350[27], 11}, {NULL}};
const Segment SGI288069[] = {{&S25351[75], 5}, {NULL}};
const Segment SGI288070[] = {{&S25351[8], 3}, {NULL}};
const Segment SGI288071[] = {{&S25351[13], 7}, {NULL}};
const Segment SGI288072[] = {{&S25351[25], 14}, {NULL}};
const Segment SGI288073[] = {{&S25351[7], 1}, {NULL}};
const Segment SGI288074[] = {{&S25351[16], 4}, {&S25351[28], 11}, {NULL}};
const Segment SGI288075[] = {{&S25361[65], 5}, {NULL}};
const Segment SGI288076[] = {{&S25361[8], 3}, {NULL}};
const Segment SGI288077[] = {{&S25361[13], 14}, {NULL}};
const Segment SGI288078[] = {{&S25361[7], 1}, {NULL}};
const Segment SGI288079[] = {{&S25361[16], 11}, {&S25361[32], 2}, {NULL}};
const Segment SGI288080[] = {{&S25362[87], 5}, {NULL}};
const Segment SGI288081[] = {{&S25362[8], 3}, {NULL}};
const Segment SGI288082[] = {{&S25362[13], 14}, {NULL}};
const Segment SGI288083[] = {{&S25362[7], 1}, {NULL}};
const Segment SGI288084[] = {{&S25362[16], 11}, {NULL}};
const Segment SGI288085[] = {{&S25363[60], 5}, {&S25363[60], 5}, {NULL}};
const Segment SGI288086[] = {{&S25363[8], 3}, {NULL}};
const Segment SGI288087[] = {{&S25363[13], 14}, {NULL}};
const Segment SGI288088[] = {{&S25363[7], 1}, {&S25363[7], 1}, {NULL}};
const Segment SGI288089[] = {{&S25363[16], 11}, {NULL}};
const Segment SGI288090[] = {{&S25364[62], 5}, {NULL}};
const Segment SGI288091[] = {{&S25364[8], 3}, {NULL}};
const Segment SGI288092[] = {{&S25364[13], 7}, {NULL}};
const Segment SGI288093[] = {{&S25364[22], 14}, {NULL}};
const Segment SGI288094[] = {{&S25364[7], 1}, {NULL}};
const Segment SGI288095[] = {{&S25364[16], 4}, {&S25364[25], 11}, {NULL}};
const Segment SGI288096[] = {{&S25372[60], 5}, {NULL}};
const Segment SGI288097[] = {{&S25372[8], 3}, {NULL}};
const Segment SGI288098[] = {{&S25372[13], 14}, {NULL}};
const Segment SGI288099[] = {{&S25372[7], 1}, {NULL}};
const Segment SGI288100[] = {{&S25372[16], 11}, {NULL}};
const Segment SGI288101[] = {{&S25373[67], 5}, {NULL}};
const Segment SGI288102[] = {{&S25373[8], 3}, {NULL}};
const Segment SGI288103[] = {{&S25373[13], 14}, {NULL}};
const Segment SGI288104[] = {{&S25373[7], 1}, {NULL}};
const Segment SGI288105[] = {{&S25373[16], 11}, {&S25373[32], 2}, {NULL}};
const Segment SGI288106[] = {{&S25374[60], 5}, {NULL}};
const Segment SGI288107[] = {{&S25374[8], 3}, {NULL}};
const Segment SGI288108[] = {{&S25374[13], 14}, {NULL}};
const Segment SGI288109[] = {{&S25374[7], 1}, {NULL}};
const Segment SGI288110[] = {{&S25374[16], 11}, {NULL}};
const Segment SGI288111[] = {{&S25374[66], 14}, {NULL}};
const Segment SGI288112[] = {{&S25375[52], 5}, {NULL}};
const Segment SGI288113[] = {{&S25375[8], 3}, {NULL}};
const Segment SGI288114[] = {{&S25375[13], 7}, {NULL}};
const Segment SGI288115[] = {{&S25375[22], 14}, {NULL}};
const Segment SGI288116[] = {{&S25375[7], 1}, {NULL}};
const Segment SGI288117[] = {{&S25375[16], 4}, {&S25375[25], 11}, {NULL}};
const Segment SGI288118[] = {{&S25376[64], 5}, {NULL}};
const Segment SGI288119[] = {{&S25376[8], 3}, {NULL}};
const Segment SGI288120[] = {{&S25376[13], 7}, {NULL}};
const Segment SGI288121[] = {{&S25376[22], 14}, {NULL}};
const Segment SGI288122[] = {{&S25376[7], 1}, {NULL}};
const Segment SGI288123[] = {{&S25376[16], 4}, {&S25376[25], 11}, {&S25376[41], 2}, {NULL}};
const Segment SGI288124[] = {{&S25589[73], 4}, {NULL}};
const Segment SGI288125[] = {{&S25589[8], 3}, {NULL}};
const Segment SGI288126[] = {{&S25589[13], 7}, {NULL}};
const Segment SGI288127[] = {{&S25589[25], 14}, {NULL}};
const Segment SGI288128[] = {{&S25589[7], 1}, {NULL}};
const Segment SGI288129[] = {{&S25589[16], 4}, {&S25589[28], 11}, {NULL}};
const Segment SGI288130[] = {{&S25590[87], 4}, {NULL}};
const Segment SGI288131[] = {{&S25590[20], 3}, {NULL}};
const Segment SGI288132[] = {{&S25590[25], 7}, {NULL}};
const Segment SGI288133[] = {{&S25590[37], 14}, {NULL}};
const Segment SGI288134[] = {{&S25590[19], 1}, {NULL}};
const Segment SGI288135[] = {{&S25590[28], 4}, {&S25590[40], 11}, {NULL}};
const Segment SGI288136[] = {{&S25591[58], 4}, {NULL}};
const Segment SGI288137[] = {{&S25591[8], 3}, {NULL}};
const Segment SGI288138[] = {{&S25591[13], 14}, {NULL}};
const Segment SGI288139[] = {{&S25591[7], 1}, {NULL}};
const Segment SGI288140[] = {{&S25591[16], 11}, {NULL}};
const Segment SGI288141[] = {{&S25592[44], 4}, {NULL}};
const Segment SGI288142[] = {{&S25592[8], 3}, {NULL}};
const Segment SGI288143[] = {{&S25592[13], 14}, {NULL}};
const Segment SGI288144[] = {{&S25592[7], 1}, {NULL}};
const Segment SGI288145[] = {{&S25592[16], 34}, {NULL}};
const Segment SGI288146[] = {{&S25593[50], 4}, {NULL}};
const Segment SGI288147[] = {{&S25593[8], 3}, {NULL}};
const Segment SGI288148[] = {{&S25593[12], 14}, {NULL}};
const Segment SGI288149[] = {{&S25593[7], 1}, {NULL}};
const Segment SGI288150[] = {{&S25593[15], 11}, {NULL}};
const Segment SGI288151[] = {{&S25594[54], 4}, {NULL}};
const Segment SGI288152[] = {{&S25594[8], 3}, {NULL}};
const Segment SGI288153[] = {{&S25594[13], 14}, {NULL}};
const Segment SGI288154[] = {{&S25594[7], 1}, {NULL}};
const Segment SGI288155[] = {{&S25594[16], 11}, {NULL}};
const Segment SGI288156[] = {{&S25595[46], 4}, {NULL}};
const Segment SGI288157[] = {{&S25595[8], 3}, {NULL}};
const Segment SGI288158[] = {{&S25595[7], 1}, {NULL}};
const Segment SGI288159[] = {{&S25595[15], 9}, {NULL}};
const Segment SGI288160[] = {{&S25596[8], 3}, {NULL}};
const Segment SGI288161[] = {{&S25596[13], 7}, {NULL}};
const Segment SGI288162[] = {{&S25596[25], 14}, {NULL}};
const Segment SGI288163[] = {{&S25596[7], 1}, {NULL}};
const Segment SGI288164[] = {{&S25596[16], 4}, {&S25596[28], 11}, {NULL}};
const Segment SGI288165[] = {{&S25712[63], 4}, {NULL}};
const Segment SGI288166[] = {{&S25712[8], 3}, {NULL}};
const Segment SGI288167[] = {{&S25712[13], 14}, {NULL}};
const Segment SGI288168[] = {{&S25712[7], 1}, {NULL}};
const Segment SGI288169[] = {{&S25712[16], 11}, {NULL}};
const Segment SGI288170[] = {{&S25712[68], 11}, {NULL}};
const Segment SGI288171[] = {{&S25819[99], 4}, {NULL}};
const Segment SGI288172[] = {{&S25819[8], 3}, {&S25819[8], 3}, {NULL}};
const Segment SGI288173[] = {{&S25819[0], 11}, {NULL}};
const Segment SGI288174[] = {{&S25819[7], 1}, {NULL}};
const Segment SGI288175[] = {{&S25819[29], 17}, {NULL}};
const Segment SGI288176[] = {{&S25820[8], 3}, {NULL}};
const Segment SGI288177[] = {{&S25820[13], 7}, {NULL}};
const Segment SGI288178[] = {{&S25820[25], 14}, {NULL}};
const Segment SGI288179[] = {{&S25820[16], 4}, {&S25820[28], 11}, {NULL}};
const Segment SGI288180[] = {{&S25821[61], 4}, {&S25821[61], 4}, {NULL}};
const Segment SGI288181[] = {{&S25821[8], 3}, {NULL}};
const Segment SGI288182[] = {{&S25821[13], 14}, {NULL}};
const Segment SGI288183[] = {{&S25821[7], 1}, {&S25821[7], 1}, {NULL}};
const Segment SGI288184[] = {{&S25821[16], 11}, {NULL}};
const Segment SGI288185[] = {{&S25822[66], 4}, {NULL}};
const Segment SGI288186[] = {{&S25822[8], 3}, {NULL}};
const Segment SGI288187[] = {{&S25822[13], 14}, {NULL}};
const Segment SGI288188[] = {{&S25822[7], 1}, {NULL}};
const Segment SGI288189[] = {{&S25822[16], 11}, {&S25822[32], 2}, {NULL}};
const Segment SGI288190[] = {{&S25823[52], 5}, {NULL}};
const Segment SGI288191[] = {{&S25823[8], 3}, {NULL}};
const Segment SGI288192[] = {{&S25823[13], 14}, {NULL}};
const Segment SGI288193[] = {{&S25823[7], 1}, {NULL}};
const Segment SGI288194[] = {{&S25823[16], 11}, {NULL}};
const Segment SGI288195[] = {{&S25824[52], 4}, {NULL}};
const Segment SGI288196[] = {{&S25824[8], 3}, {NULL}};
const Segment SGI288197[] = {{&S25824[13], 14}, {NULL}};
const Segment SGI288198[] = {{&S25824[7], 1}, {NULL}};
const Segment SGI288199[] = {{&S25824[16], 11}, {NULL}};
const Segment SGI288200[] = {{&S25825[66], 4}, {NULL}};
const Segment SGI288201[] = {{&S25825[8], 3}, {NULL}};
const Segment SGI288202[] = {{&S25825[13], 7}, {NULL}};
const Segment SGI288203[] = {{&S25825[25], 14}, {NULL}};
const Segment SGI288204[] = {{&S25825[7], 1}, {NULL}};
const Segment SGI288205[] = {{&S25825[16], 4}, {&S25825[28], 11}, {NULL}};
const Segment SGI288206[] = {{&S25826[52], 4}, {NULL}};
const Segment SGI288207[] = {{&S25826[8], 3}, {NULL}};
const Segment SGI288208[] = {{&S25826[13], 14}, {NULL}};
const Segment SGI288209[] = {{&S25826[7], 1}, {NULL}};
const Segment SGI288210[] = {{&S25826[16], 11}, {NULL}};
const Segment SGI288211[] = {{&S25951[56], 4}, {NULL}};
const Segment SGI288212[] = {{&S25951[8], 3}, {NULL}};
const Segment SGI288213[] = {{&S25951[13], 14}, {NULL}};
const Segment SGI288214[] = {{&S25951[7], 1}, {NULL}};
const Segment SGI288215[] = {{&S25951[16], 11}, {NULL}};
const Segment SGI288216[] = {{&S25951[61], 11}, {NULL}};
const Segment SGI288217[] = {{&S26130[64], 5}, {NULL}};
const Segment SGI288218[] = {{&S26130[8], 3}, {NULL}};
const Segment SGI288219[] = {{&S26130[13], 3}, {NULL}};
const Segment SGI288220[] = {{&S26130[13], 3}, {&S26130[18], 6}, {NULL}};
const Segment SGI288221[] = {{&S26130[7], 1}, {NULL}};
const Segment SGI288222[] = {{&S26150[56], 4}, {NULL}};
const Segment SGI288223[] = {{&S26150[8], 3}, {NULL}};
const Segment SGI288224[] = {{&S26150[13], 14}, {NULL}};
const Segment SGI288225[] = {{&S26150[7], 1}, {NULL}};
const Segment SGI288226[] = {{&S26150[16], 11}, {NULL}};
const Segment SGI288227[] = {{&S26150[61], 11}, {NULL}};
const Segment SGI288228[] = {{&S26175[53], 5}, {NULL}};
const Segment SGI288229[] = {{&S26175[8], 3}, {NULL}};
const Segment SGI288230[] = {{&S26175[13], 14}, {NULL}};
const Segment SGI288231[] = {{&S26175[7], 1}, {NULL}};
const Segment SGI288232[] = {{&S26175[16], 11}, {NULL}};
const Segment SGI288233[] = {{&S26176[62], 5}, {NULL}};
const Segment SGI288234[] = {{&S26176[8], 3}, {NULL}};
const Segment SGI288235[] = {{&S26176[13], 14}, {NULL}};
const Segment SGI288236[] = {{&S26176[7], 1}, {NULL}};
const Segment SGI288237[] = {{&S26176[16], 11}, {NULL}};
const Segment SGI288238[] = {{&S26177[67], 5}, {NULL}};
const Segment SGI288239[] = {{&S26177[8], 3}, {NULL}};
const Segment SGI288240[] = {{&S26177[13], 14}, {NULL}};
const Segment SGI288241[] = {{&S26177[7], 1}, {NULL}};
const Segment SGI288242[] = {{&S26177[16], 11}, {&S26177[32], 2}, {NULL}};
const Segment SGI288243[] = {{&S26178[59], 4}, {NULL}};
const Segment SGI288244[] = {{&S26178[8], 3}, {NULL}};
const Segment SGI288245[] = {{&S26178[13], 14}, {NULL}};
const Segment SGI288246[] = {{&S26178[7], 1}, {NULL}};
const Segment SGI288247[] = {{&S26178[16], 11}, {NULL}};
const Segment SGI288248[] = {{&S26179[74], 4}, {NULL}};
const Segment SGI288249[] = {{&S26179[8], 3}, {NULL}};
const Segment SGI288250[] = {{&S26179[13], 7}, {NULL}};
const Segment SGI288251[] = {{&S26179[25], 14}, {NULL}};
const Segment SGI288252[] = {{&S26179[7], 1}, {NULL}};
const Segment SGI288253[] = {{&S26179[16], 4}, {&S26179[28], 11}, {NULL}};
const Segment SGI288254[] = {{&S26180[7], 4}, {NULL}};
const Segment SGI288255[] = {{&S26180[8], 3}, {NULL}};
const Segment SGI288256[] = {{&S26180[13], 14}, {NULL}};
const Segment SGI288257[] = {{&S26180[7], 1}, {NULL}};
const Segment SGI288258[] = {{&S26180[16], 12}, {&S26180[16], 12}, {NULL}};
const Segment SGI288259[] = {{&S26181[64], 4}, {NULL}};
const Segment SGI288260[] = {{&S26181[8], 3}, {NULL}};
const Segment SGI288261[] = {{&S26181[13], 14}, {NULL}};
const Segment SGI288262[] = {{&S26181[7], 1}, {NULL}};
const Segment SGI288263[] = {{&S26181[16], 11}, {&S26181[32], 2}, {NULL}};
const Segment SGI288264[] = {{&S26182[7], 4}, {NULL}};
const Segment SGI288265[] = {{&S26182[8], 3}, {NULL}};
const Segment SGI288266[] = {{&S26182[13], 14}, {NULL}};
const Segment SGI288267[] = {{&S26182[7], 1}, {NULL}};
const Segment SGI288268[] = {{&S26182[16], 12}, {&S26182[16], 11}, {NULL}};
const Segment SGI288269[] = {{&S26183[61], 4}, {&S26183[61], 4}, {NULL}};
const Segment SGI288270[] = {{&S26183[8], 3}, {NULL}};
const Segment SGI288271[] = {{&S26183[13], 14}, {NULL}};
const Segment SGI288272[] = {{&S26183[7], 1}, {&S26183[7], 1}, {NULL}};
const Segment SGI288273[] = {{&S26183[16], 11}, {NULL}};
const Segment SGI288274[] = {{&S26184[45], 4}, {NULL}};
const Segment SGI288275[] = {{&S26184[8], 3}, {NULL}};
const Segment SGI288276[] = {{&S26184[13], 14}, {NULL}};
const Segment SGI288277[] = {{&S26184[7], 1}, {NULL}};
const Segment SGI288278[] = {{&S26184[16], 12}, {&S26184[16], 11}, {NULL}};
const Segment SGI288279[] = {{&S26185[44], 4}, {NULL}};
const Segment SGI288280[] = {{&S26185[8], 3}, {NULL}};
const Segment SGI288281[] = {{&S26185[13], 14}, {NULL}};
const Segment SGI288282[] = {{&S26185[7], 1}, {NULL}};
const Segment SGI288283[] = {{&S26185[16], 32}, {NULL}};
const Segment SGI288284[] = {{&S26195[60], 5}, {NULL}};
const Segment SGI288285[] = {{&S26195[8], 3}, {NULL}};
const Segment SGI288286[] = {{&S26195[13], 14}, {NULL}};
const Segment SGI288287[] = {{&S26195[7], 1}, {NULL}};
const Segment SGI288288[] = {{&S26195[16], 11}, {NULL}};
const Segment SGI288289[] = {{&S26196[60], 5}, {NULL}};
const Segment SGI288290[] = {{&S26196[8], 3}, {NULL}};
const Segment SGI288291[] = {{&S26196[13], 14}, {NULL}};
const Segment SGI288292[] = {{&S26196[7], 1}, {NULL}};
const Segment SGI288293[] = {{&S26196[16], 11}, {NULL}};
const Segment SGI288294[] = {{&S26196[66], 14}, {NULL}};
const Segment SGI288295[] = {{&S26197[74], 5}, {NULL}};
const Segment SGI288296[] = {{&S26197[17], 3}, {NULL}};
const Segment SGI288297[] = {{&S26197[22], 14}, {NULL}};
const Segment SGI288298[] = {{&S26197[16], 1}, {NULL}};
const Segment SGI288299[] = {{&S26197[25], 11}, {&S26197[41], 2}, {NULL}};
const Segment SGI288300[] = {{&S26208[60], 5}, {NULL}};
const Segment SGI288301[] = {{&S26208[8], 3}, {NULL}};
const Segment SGI288302[] = {{&S26208[13], 14}, {NULL}};
const Segment SGI288303[] = {{&S26208[7], 1}, {NULL}};
const Segment SGI288304[] = {{&S26208[16], 11}, {NULL}};
const Segment SGI288305[] = {{&S26209[64], 5}, {NULL}};
const Segment SGI288306[] = {{&S26209[8], 3}, {NULL}};
const Segment SGI288307[] = {{&S26209[13], 7}, {NULL}};
const Segment SGI288308[] = {{&S26209[22], 14}, {NULL}};
const Segment SGI288309[] = {{&S26209[7], 1}, {NULL}};
const Segment SGI288310[] = {{&S26209[16], 4}, {&S26209[25], 11}, {&S26209[41], 2}, {NULL}};
const Segment SGI288311[] = {{&S26210[74], 5}, {NULL}};
const Segment SGI288312[] = {{&S26210[17], 3}, {NULL}};
const Segment SGI288313[] = {{&S26210[22], 14}, {NULL}};
const Segment SGI288314[] = {{&S26210[16], 1}, {NULL}};
const Segment SGI288315[] = {{&S26210[25], 11}, {&S26210[41], 2}, {NULL}};
const Segment SGI288316[] = {{&S26358[63], 4}, {NULL}};
const Segment SGI288317[] = {{&S26358[8], 3}, {NULL}};
const Segment SGI288318[] = {{&S26358[13], 14}, {NULL}};
const Segment SGI288319[] = {{&S26358[7], 1}, {NULL}};
const Segment SGI288320[] = {{&S26358[16], 11}, {NULL}};
const Segment SGI288321[] = {{&S26358[70], 11}, {NULL}};
const Segment SGI288322[] = {{&S26366[63], 4}, {NULL}};
const Segment SGI288323[] = {{&S26366[8], 3}, {NULL}};
const Segment SGI288324[] = {{&S26366[13], 14}, {NULL}};
const Segment SGI288325[] = {{&S26366[7], 1}, {NULL}};
const Segment SGI288326[] = {{&S26366[16], 11}, {NULL}};
const Segment SGI288327[] = {{&S26366[70], 11}, {NULL}};
const Segment SGI288328[] = {{&S26417[75], 4}, {NULL}};
const Segment SGI288329[] = {{&S26417[8], 3}, {NULL}};
const Segment SGI288330[] = {{&S26417[13], 7}, {NULL}};
const Segment SGI288331[] = {{&S26417[25], 14}, {NULL}};
const Segment SGI288332[] = {{&S26417[7], 1}, {&S26417[11], 1}, {NULL}};
const Segment SGI288333[] = {{&S26417[16], 4}, {&S26417[28], 11}, {NULL}};
const Segment SGI288334[] = {{&S26418[8], 3}, {NULL}};
const Segment SGI288335[] = {{&S26418[13], 7}, {NULL}};
const Segment SGI288336[] = {{&S26418[25], 14}, {NULL}};
const Segment SGI288337[] = {{&S26418[16], 4}, {&S26418[28], 11}, {NULL}};
const Segment SGI288338[] = {{&S26418[68], 11}, {NULL}};
const Segment SGI288339[] = {{&S26419[76], 4}, {NULL}};
const Segment SGI288340[] = {{&S26419[8], 3}, {NULL}};
const Segment SGI288341[] = {{&S26419[13], 7}, {NULL}};
const Segment SGI288342[] = {{&S26419[25], 14}, {NULL}};
const Segment SGI288343[] = {{&S26419[7], 1}, {NULL}};
const Segment SGI288344[] = {{&S26419[16], 4}, {&S26419[28], 11}, {&S26419[117], 17}, {NULL}};
const Segment SGI288345[] = {{&S26420[73], 4}, {NULL}};
const Segment SGI288346[] = {{&S26420[8], 3}, {NULL}};
const Segment SGI288347[] = {{&S26420[13], 7}, {NULL}};
const Segment SGI288348[] = {{&S26420[7], 1}, {NULL}};
const Segment SGI288349[] = {{&S26420[16], 4}, {&S26420[28], 10}, {NULL}};
const Segment SGI288350[] = {{&S26421[56], 4}, {NULL}};
const Segment SGI288351[] = {{&S26421[8], 3}, {NULL}};
const Segment SGI288352[] = {{&S26421[13], 7}, {NULL}};
const Segment SGI288353[] = {{&S26421[7], 1}, {NULL}};
const Segment SGI288354[] = {{&S26421[16], 4}, {&S26421[28], 36}, {NULL}};
const Segment SGI288355[] = {{&S26422[76], 4}, {NULL}};
const Segment SGI288356[] = {{&S26422[8], 3}, {NULL}};
const Segment SGI288357[] = {{&S26422[13], 7}, {NULL}};
const Segment SGI288358[] = {{&S26422[25], 14}, {NULL}};
const Segment SGI288359[] = {{&S26422[7], 1}, {NULL}};
const Segment SGI288360[] = {{&S26422[16], 4}, {&S26422[28], 11}, {&S26422[110], 17}, {NULL}};
const Segment SGI288361[] = {{&S26423[107], 4}, {NULL}};
const Segment SGI288362[] = {{&S26423[8], 3}, {NULL}};
const Segment SGI288363[] = {{&S26423[13], 7}, {NULL}};
const Segment SGI288364[] = {{&S26423[60], 14}, {NULL}};
const Segment SGI288365[] = {{&S26423[7], 1}, {NULL}};
const Segment SGI288366[] = {{&S26423[16], 4}, {&S26423[63], 11}, {NULL}};
const Segment SGI288367[] = {{&S26424[76], 4}, {NULL}};
const Segment SGI288368[] = {{&S26424[8], 3}, {NULL}};
const Segment SGI288369[] = {{&S26424[13], 7}, {NULL}};
const Segment SGI288370[] = {{&S26424[25], 14}, {NULL}};
const Segment SGI288371[] = {{&S26424[7], 1}, {NULL}};
const Segment SGI288372[] = {{&S26424[16], 4}, {&S26424[28], 11}, {NULL}};
const Segment SGI288373[] = {{&S26425[8], 3}, {NULL}};
const Segment SGI288374[] = {{&S26425[13], 7}, {NULL}};
const Segment SGI288375[] = {{&S26425[25], 14}, {NULL}};
const Segment SGI288376[] = {{&S26425[16], 4}, {&S26425[28], 11}, {NULL}};
const Segment SGI288377[] = {{&S26425[68], 11}, {NULL}};
const Segment SGI288378[] = {{&S26427[74], 4}, {NULL}};
const Segment SGI288379[] = {{&S26427[8], 3}, {NULL}};
const Segment SGI288380[] = {{&S26427[7], 1}, {NULL}};
const Segment SGI288381[] = {{&S26427[16], 4}, {&S26427[28], 11}, {NULL}};
const Segment SGI288382[] = {{&S26428[54], 4}, {NULL}};
const Segment SGI288383[] = {{&S26428[8], 3}, {NULL}};
const Segment SGI288384[] = {{&S26428[13], 14}, {NULL}};
const Segment SGI288385[] = {{&S26428[7], 1}, {NULL}};
const Segment SGI288386[] = {{&S26428[16], 11}, {NULL}};
const Segment SGI288387[] = {{&S26429[77], 4}, {NULL}};
const Segment SGI288388[] = {{&S26429[10], 3}, {NULL}};
const Segment SGI288389[] = {{&S26429[15], 7}, {NULL}};
const Segment SGI288390[] = {{&S26429[27], 14}, {NULL}};
const Segment SGI288391[] = {{&S26429[9], 1}, {&S26429[13], 1}, {NULL}};
const Segment SGI288392[] = {{&S26429[18], 4}, {&S26429[30], 11}, {&S26429[105], 9}, {NULL}};
const Segment SGI288393[] = {{&S26430[75], 4}, {NULL}};
const Segment SGI288394[] = {{&S26430[8], 3}, {NULL}};
const Segment SGI288395[] = {{&S26430[13], 7}, {NULL}};
const Segment SGI288396[] = {{&S26430[25], 14}, {NULL}};
const Segment SGI288397[] = {{&S26430[7], 1}, {NULL}};
const Segment SGI288398[] = {{&S26430[16], 4}, {&S26430[28], 11}, {NULL}};
const Segment SGI288399[] = {{&S26554[64], 5}, {NULL}};
const Segment SGI288400[] = {{&S26554[8], 3}, {NULL}};
const Segment SGI288401[] = {{&S26554[13], 3}, {NULL}};
const Segment SGI288402[] = {{&S26554[13], 3}, {&S26554[18], 6}, {NULL}};
const Segment SGI288403[] = {{&S26554[7], 1}, {NULL}};
const Segment SGI288404[] = {{&S26673[59], 4}, {NULL}};
const Segment SGI288405[] = {{&S26673[8], 3}, {NULL}};
const Segment SGI288406[] = {{&S26673[13], 14}, {NULL}};
const Segment SGI288407[] = {{&S26673[7], 1}, {NULL}};
const Segment SGI288408[] = {{&S26673[16], 11}, {NULL}};
const Segment SGI288409[] = {{&S26673[64], 11}, {NULL}};
const Segment SGI288410[] = {{&S26701[64], 4}, {NULL}};
const Segment SGI288411[] = {{&S26701[8], 3}, {NULL}};
const Segment SGI288412[] = {{&S26701[13], 14}, {NULL}};
const Segment SGI288413[] = {{&S26701[7], 1}, {NULL}};
const Segment SGI288414[] = {{&S26701[16], 11}, {&S26701[32], 2}, {NULL}};
const Segment SGI288415[] = {{&S26701[69], 11}, {NULL}};
const Segment SGI288416[] = {{&S26729[52], 5}, {NULL}};
const Segment SGI288417[] = {{&S26729[8], 3}, {NULL}};
const Segment SGI288418[] = {{&S26729[13], 14}, {NULL}};
const Segment SGI288419[] = {{&S26729[7], 1}, {NULL}};
const Segment SGI288420[] = {{&S26729[16], 11}, {NULL}};
const Segment SGI288421[] = {{&S26730[53], 5}, {&S26730[53], 5}, {NULL}};
const Segment SGI288422[] = {{&S26730[8], 3}, {NULL}};
const Segment SGI288423[] = {{&S26730[13], 14}, {NULL}};
const Segment SGI288424[] = {{&S26730[7], 1}, {&S26730[7], 1}, {NULL}};
const Segment SGI288425[] = {{&S26730[16], 11}, {NULL}};
const Segment SGI288426[] = {{&S26731[63], 5}, {NULL}};
const Segment SGI288427[] = {{&S26731[8], 3}, {NULL}};
const Segment SGI288428[] = {{&S26731[13], 14}, {NULL}};
const Segment SGI288429[] = {{&S26731[7], 1}, {NULL}};
const Segment SGI288430[] = {{&S26731[16], 11}, {NULL}};
const Segment SGI288431[] = {{&S26732[75], 5}, {NULL}};
const Segment SGI288432[] = {{&S26732[8], 3}, {NULL}};
const Segment SGI288433[] = {{&S26732[13], 7}, {NULL}};
const Segment SGI288434[] = {{&S26732[25], 14}, {NULL}};
const Segment SGI288435[] = {{&S26732[7], 1}, {NULL}};
const Segment SGI288436[] = {{&S26732[16], 4}, {&S26732[28], 11}, {NULL}};
const Segment SGI288437[] = {{&S12385[0], 166}, {NULL}};
const Segment SGI288438[] = {{&S27806[55], 5}, {NULL}};
const Segment SGI288439[] = {{&S27806[8], 3}, {NULL}};
const Segment SGI288440[] = {{&S27806[13], 14}, {NULL}};
const Segment SGI288441[] = {{&S27806[7], 1}, {NULL}};
const Segment SGI288442[] = {{&S27806[16], 11}, {&S27806[32], 2}, {NULL}};
const Segment SGI288443[] = {{&S27807[43], 5}, {NULL}};
const Segment SGI288444[] = {{&S27807[8], 3}, {NULL}};
const Segment SGI288445[] = {{&S27807[13], 14}, {NULL}};
const Segment SGI288446[] = {{&S27807[7], 1}, {NULL}};
const Segment SGI288447[] = {{&S27807[16], 11}, {NULL}};
const Segment SGI288448[] = {{&S27808[75], 5}, {NULL}};
const Segment SGI288449[] = {{&S27808[8], 3}, {NULL}};
const Segment SGI288450[] = {{&S27808[13], 7}, {NULL}};
const Segment SGI288451[] = {{&S27808[25], 14}, {NULL}};
const Segment SGI288452[] = {{&S27808[7], 1}, {NULL}};
const Segment SGI288453[] = {{&S27808[16], 4}, {&S27808[28], 11}, {NULL}};
const Segment SGI288454[] = {{&S27810[59], 4}, {NULL}};
const Segment SGI288455[] = {{&S27810[8], 3}, {NULL}};
const Segment SGI288456[] = {{&S27810[13], 14}, {NULL}};
const Segment SGI288457[] = {{&S27810[7], 1}, {NULL}};
const Segment SGI288458[] = {{&S27810[16], 11}, {NULL}};
const Segment SGI288459[] = {{&S27810[64], 11}, {NULL}};
const Segment SGI288460[] = {{&S27879[64], 4}, {NULL}};
const Segment SGI288461[] = {{&S27879[8], 3}, {NULL}};
const Segment SGI288462[] = {{&S27879[13], 14}, {NULL}};
const Segment SGI288463[] = {{&S27879[7], 1}, {NULL}};
const Segment SGI288464[] = {{&S27879[16], 11}, {&S27879[32], 2}, {NULL}};
const Segment SGI288465[] = {{&S27879[69], 11}, {NULL}};
const Segment SGI288466[] = {{&S27949[64], 5}, {NULL}};
const Segment SGI288467[] = {{&S27949[8], 3}, {NULL}};
const Segment SGI288468[] = {{&S27949[13], 3}, {NULL}};
const Segment SGI288469[] = {{&S27949[13], 3}, {&S27949[18], 6}, {NULL}};
const Segment SGI288470[] = {{&S27949[7], 1}, {NULL}};
const Segment SGI288471[] = {{&S28348[60], 5}, {NULL}};
const Segment SGI288472[] = {{&S28348[8], 3}, {NULL}};
const Segment SGI288473[] = {{&S28348[13], 14}, {NULL}};
const Segment SGI288474[] = {{&S28348[7], 1}, {NULL}};
const Segment SGI288475[] = {{&S28348[16], 11}, {NULL}};
const Segment SGI288476[] = {{&S28349[53], 5}, {NULL}};
const Segment SGI288477[] = {{&S28349[8], 3}, {NULL}};
const Segment SGI288478[] = {{&S28349[13], 14}, {NULL}};
const Segment SGI288479[] = {{&S28349[7], 1}, {NULL}};
const Segment SGI288480[] = {{&S28349[16], 11}, {NULL}};
const Segment SGI288481[] = {{&S28350[65], 5}, {NULL}};
const Segment SGI288482[] = {{&S28350[8], 3}, {NULL}};
const Segment SGI288483[] = {{&S28350[13], 14}, {NULL}};
const Segment SGI288484[] = {{&S28350[7], 1}, {NULL}};
const Segment SGI288485[] = {{&S28350[16], 11}, {&S28350[32], 2}, {NULL}};
const Segment SGI288486[] = {{&S28351[67], 5}, {NULL}};
const Segment SGI288487[] = {{&S28351[8], 3}, {NULL}};
const Segment SGI288488[] = {{&S28351[13], 7}, {NULL}};
const Segment SGI288489[] = {{&S28351[25], 14}, {NULL}};
const Segment SGI288490[] = {{&S28351[7], 1}, {NULL}};
const Segment SGI288491[] = {{&S28351[16], 4}, {&S28351[28], 11}, {NULL}};
const Segment SGI288492[] = {{&S28450[60], 5}, {NULL}};
const Segment SGI288493[] = {{&S28450[8], 3}, {NULL}};
const Segment SGI288494[] = {{&S28450[13], 14}, {NULL}};
const Segment SGI288495[] = {{&S28450[7], 1}, {NULL}};
const Segment SGI288496[] = {{&S28450[16], 11}, {NULL}};
const Segment SGI288497[] = {{&S28451[65], 5}, {NULL}};
const Segment SGI288498[] = {{&S28451[8], 3}, {NULL}};
const Segment SGI288499[] = {{&S28451[13], 14}, {NULL}};
const Segment SGI288500[] = {{&S28451[7], 1}, {NULL}};
const Segment SGI288501[] = {{&S28451[16], 11}, {&S28451[32], 2}, {NULL}};
const Segment SGI288502[] = {{&S28452[91], 5}, {NULL}};
const Segment SGI288503[] = {{&S28452[8], 3}, {NULL}};
const Segment SGI288504[] = {{&S28452[13], 10}, {NULL}};
const Segment SGI288505[] = {{&S28452[37], 14}, {NULL}};
const Segment SGI288506[] = {{&S28452[7], 1}, {NULL}};
const Segment SGI288507[] = {{&S28452[28], 4}, {&S28452[40], 11}, {NULL}};
const Segment SGI288508[] = {{&S28573[59], 4}, {NULL}};
const Segment SGI288509[] = {{&S28573[8], 3}, {NULL}};
const Segment SGI288510[] = {{&S28573[13], 3}, {NULL}};
const Segment SGI288511[] = {{&S28573[7], 1}, {NULL}};
const Segment SGI288512[] = {{&S28573[64], 11}, {NULL}};
const Segment SGI288513[] = {{&S28740[53], 5}, {NULL}};
const Segment SGI288514[] = {{&S28740[8], 3}, {NULL}};
const Segment SGI288515[] = {{&S28740[13], 14}, {NULL}};
const Segment SGI288516[] = {{&S28740[7], 1}, {NULL}};
const Segment SGI288517[] = {{&S28740[16], 11}, {NULL}};
const Segment SGI288518[] = {{&S28741[60], 5}, {NULL}};
const Segment SGI288519[] = {{&S28741[8], 3}, {NULL}};
const Segment SGI288520[] = {{&S28741[13], 14}, {NULL}};
const Segment SGI288521[] = {{&S28741[7], 1}, {NULL}};
const Segment SGI288522[] = {{&S28741[16], 11}, {NULL}};
const Segment SGI288523[] = {{&S28975[56], 4}, {NULL}};
const Segment SGI288524[] = {{&S28975[8], 3}, {NULL}};
const Segment SGI288525[] = {{&S28975[13], 14}, {NULL}};
const Segment SGI288526[] = {{&S28975[7], 1}, {NULL}};
const Segment SGI288527[] = {{&S28975[16], 11}, {NULL}};
const Segment SGI288528[] = {{&S28975[61], 11}, {NULL}};
const Segment SGI288529[] = {{&S14268[0], 163}, {NULL}};
const Segment SGI288530[] = {{&S29006[65], 5}, {NULL}};
const Segment SGI288531[] = {{&S29006[8], 3}, {NULL}};
const Segment SGI288532[] = {{&S29006[13], 14}, {NULL}};
const Segment SGI288533[] = {{&S29006[7], 1}, {NULL}};
const Segment SGI288534[] = {{&S29006[16], 11}, {&S29006[32], 2}, {NULL}};
const Segment SGI288535[] = {{&S29007[60], 5}, {NULL}};
const Segment SGI288536[] = {{&S29007[8], 3}, {NULL}};
const Segment SGI288537[] = {{&S29007[13], 14}, {NULL}};
const Segment SGI288538[] = {{&S29007[7], 1}, {NULL}};
const Segment SGI288539[] = {{&S29007[16], 11}, {NULL}};
const Segment SGI288540[] = {{&S29008[60], 5}, {&S29008[60], 5}, {NULL}};
const Segment SGI288541[] = {{&S29008[8], 3}, {NULL}};
const Segment SGI288542[] = {{&S29008[13], 14}, {NULL}};
const Segment SGI288543[] = {{&S29008[7], 1}, {&S29008[7], 1}, {NULL}};
const Segment SGI288544[] = {{&S29008[16], 11}, {NULL}};
const Segment SGI288545[] = {{&S29009[62], 5}, {NULL}};
const Segment SGI288546[] = {{&S29009[8], 3}, {NULL}};
const Segment SGI288547[] = {{&S29009[13], 7}, {NULL}};
const Segment SGI288548[] = {{&S29009[22], 14}, {NULL}};
const Segment SGI288549[] = {{&S29009[7], 1}, {NULL}};
const Segment SGI288550[] = {{&S29009[16], 4}, {&S29009[25], 11}, {NULL}};
const Segment SGI288551[] = {{&S29010[65], 5}, {&S29010[65], 5}, {NULL}};
const Segment SGI288552[] = {{&S29010[8], 3}, {NULL}};
const Segment SGI288553[] = {{&S29010[13], 14}, {NULL}};
const Segment SGI288554[] = {{&S29010[7], 1}, {&S29010[7], 1}, {NULL}};
const Segment SGI288555[] = {{&S29010[16], 11}, {&S29010[32], 2}, {NULL}};
const Segment SGI288556[] = {{&S29011[74], 5}, {NULL}};
const Segment SGI288557[] = {{&S29011[8], 3}, {NULL}};
const Segment SGI288558[] = {{&S29011[13], 7}, {NULL}};
const Segment SGI288559[] = {{&S29011[22], 14}, {NULL}};
const Segment SGI288560[] = {{&S29011[7], 1}, {NULL}};
const Segment SGI288561[] = {{&S29011[16], 4}, {&S29011[25], 11}, {&S29011[41], 2}, {NULL}};
const Segment SGI288562[] = {{&S29447[53], 5}, {NULL}};
const Segment SGI288563[] = {{&S29447[8], 3}, {NULL}};
const Segment SGI288564[] = {{&S29447[13], 14}, {NULL}};
const Segment SGI288565[] = {{&S29447[7], 1}, {NULL}};
const Segment SGI288566[] = {{&S29447[16], 11}, {NULL}};
const Segment SGI288567[] = {{&S29448[53], 5}, {NULL}};
const Segment SGI288568[] = {{&S29448[8], 3}, {NULL}};
const Segment SGI288569[] = {{&S29448[13], 14}, {NULL}};
const Segment SGI288570[] = {{&S29448[7], 1}, {NULL}};
const Segment SGI288571[] = {{&S29448[16], 11}, {NULL}};
const Segment SGI288572[] = {{&S29449[53], 5}, {&S29449[53], 5}, {NULL}};
const Segment SGI288573[] = {{&S29449[8], 3}, {NULL}};
const Segment SGI288574[] = {{&S29449[13], 14}, {NULL}};
const Segment SGI288575[] = {{&S29449[7], 1}, {&S29449[7], 1}, {NULL}};
const Segment SGI288576[] = {{&S29449[16], 11}, {NULL}};
const Segment SGI288577[] = {{&S29450[65], 5}, {NULL}};
const Segment SGI288578[] = {{&S29450[8], 3}, {NULL}};
const Segment SGI288579[] = {{&S29450[13], 14}, {NULL}};
const Segment SGI288580[] = {{&S29450[7], 1}, {NULL}};
const Segment SGI288581[] = {{&S29450[16], 11}, {&S29450[32], 2}, {NULL}};
const Segment SGI288582[] = {{&S29451[62], 5}, {NULL}};
const Segment SGI288583[] = {{&S29451[8], 3}, {NULL}};
const Segment SGI288584[] = {{&S29451[13], 7}, {NULL}};
const Segment SGI288585[] = {{&S29451[22], 14}, {NULL}};
const Segment SGI288586[] = {{&S29451[7], 1}, {NULL}};
const Segment SGI288587[] = {{&S29451[16], 4}, {&S29451[25], 11}, {NULL}};
const Segment SGI288588[] = {{&S29452[44], 5}, {NULL}};
const Segment SGI288589[] = {{&S29452[8], 3}, {NULL}};
const Segment SGI288590[] = {{&S29452[13], 14}, {NULL}};
const Segment SGI288591[] = {{&S29452[7], 1}, {NULL}};
const Segment SGI288592[] = {{&S29452[16], 44}, {NULL}};
const Segment SGI288593[] = {{&S29498[65], 5}, {NULL}};
const Segment SGI288594[] = {{&S29498[8], 3}, {NULL}};
const Segment SGI288595[] = {{&S29498[13], 14}, {NULL}};
const Segment SGI288596[] = {{&S29498[7], 1}, {NULL}};
const Segment SGI288597[] = {{&S29498[16], 11}, {&S29498[32], 2}, {NULL}};
const Segment SGI288598[] = {{&S29499[60], 5}, {NULL}};
const Segment SGI288599[] = {{&S29499[8], 3}, {NULL}};
const Segment SGI288600[] = {{&S29499[13], 14}, {NULL}};
const Segment SGI288601[] = {{&S29499[7], 1}, {NULL}};
const Segment SGI288602[] = {{&S29499[16], 11}, {NULL}};
const Segment SGI288603[] = {{&S29500[53], 5}, {&S29500[53], 5}, {NULL}};
const Segment SGI288604[] = {{&S29500[8], 3}, {NULL}};
const Segment SGI288605[] = {{&S29500[13], 14}, {NULL}};
const Segment SGI288606[] = {{&S29500[7], 1}, {&S29500[7], 1}, {NULL}};
const Segment SGI288607[] = {{&S29500[16], 11}, {NULL}};
const Segment SGI288608[] = {{&S29501[65], 5}, {NULL}};
const Segment SGI288609[] = {{&S29501[8], 3}, {NULL}};
const Segment SGI288610[] = {{&S29501[13], 7}, {NULL}};
const Segment SGI288611[] = {{&S29501[25], 14}, {NULL}};
const Segment SGI288612[] = {{&S29501[7], 1}, {NULL}};
const Segment SGI288613[] = {{&S29501[16], 4}, {&S29501[28], 11}, {NULL}};
const Segment SGI288614[] = {{&S29584[62], 5}, {NULL}};
const Segment SGI288615[] = {{&S29584[8], 3}, {NULL}};
const Segment SGI288616[] = {{&S29584[13], 3}, {NULL}};
const Segment SGI288617[] = {{&S29584[13], 3}, {&S29584[18], 6}, {NULL}};
const Segment SGI288618[] = {{&S29584[7], 1}, {NULL}};
const Segment SGI288619[] = {{&S29585[64], 5}, {&S29585[64], 5}, {NULL}};
const Segment SGI288620[] = {{&S29585[8], 3}, {NULL}};
const Segment SGI288621[] = {{&S29585[13], 3}, {NULL}};
const Segment SGI288622[] = {{&S29585[13], 3}, {&S29585[18], 6}, {NULL}};
const Segment SGI288623[] = {{&S29585[7], 1}, {&S29585[7], 1}, {NULL}};
const Segment SGI288624[] = {{&S29605[56], 4}, {NULL}};
const Segment SGI288625[] = {{&S29605[8], 3}, {NULL}};
const Segment SGI288626[] = {{&S29605[13], 14}, {NULL}};
const Segment SGI288627[] = {{&S29605[7], 1}, {NULL}};
const Segment SGI288628[] = {{&S29605[16], 11}, {NULL}};
const Segment SGI288629[] = {{&S29605[61], 11}, {NULL}};
const Segment SGI288630[] = {{&S29709[53], 5}, {NULL}};
const Segment SGI288631[] = {{&S29709[8], 3}, {NULL}};
const Segment SGI288632[] = {{&S29709[13], 14}, {NULL}};
const Segment SGI288633[] = {{&S29709[7], 1}, {NULL}};
const Segment SGI288634[] = {{&S29709[16], 11}, {NULL}};
const Segment SGI288635[] = {{&S29710[53], 5}, {NULL}};
const Segment SGI288636[] = {{&S29710[8], 3}, {NULL}};
const Segment SGI288637[] = {{&S29710[13], 14}, {NULL}};
const Segment SGI288638[] = {{&S29710[7], 1}, {NULL}};
const Segment SGI288639[] = {{&S29710[16], 11}, {NULL}};
const Segment SGI288640[] = {{&S29711[53], 5}, {&S29711[53], 5}, {NULL}};
const Segment SGI288641[] = {{&S29711[8], 3}, {NULL}};
const Segment SGI288642[] = {{&S29711[13], 14}, {NULL}};
const Segment SGI288643[] = {{&S29711[7], 1}, {&S29711[7], 1}, {NULL}};
const Segment SGI288644[] = {{&S29711[16], 11}, {NULL}};
const Segment SGI288645[] = {{&S29827[7], 4}, {NULL}};
const Segment SGI288646[] = {{&S29827[8], 3}, {NULL}};
const Segment SGI288647[] = {{&S29827[13], 7}, {NULL}};
const Segment SGI288648[] = {{&S29827[25], 14}, {NULL}};
const Segment SGI288649[] = {{&S29827[7], 1}, {NULL}};
const Segment SGI288650[] = {{&S29827[16], 4}, {&S29827[28], 11}, {NULL}};
const Segment SGI288651[] = {{&S29828[7], 4}, {&S29828[7], 4}, {NULL}};
const Segment SGI288652[] = {{&S29828[8], 3}, {NULL}};
const Segment SGI288653[] = {{&S29828[13], 14}, {NULL}};
const Segment SGI288654[] = {{&S29828[7], 1}, {&S29828[7], 1}, {NULL}};
const Segment SGI288655[] = {{&S29828[16], 11}, {NULL}};
const Segment SGI288656[] = {{&S29829[7], 4}, {NULL}};
const Segment SGI288657[] = {{&S29829[8], 3}, {NULL}};
const Segment SGI288658[] = {{&S29829[13], 14}, {NULL}};
const Segment SGI288659[] = {{&S29829[7], 1}, {NULL}};
const Segment SGI288660[] = {{&S29829[16], 32}, {NULL}};
const Segment SGI288661[] = {{&S29830[7], 4}, {NULL}};
const Segment SGI288662[] = {{&S29830[8], 3}, {NULL}};
const Segment SGI288663[] = {{&S29830[13], 14}, {NULL}};
const Segment SGI288664[] = {{&S29830[7], 1}, {NULL}};
const Segment SGI288665[] = {{&S29830[16], 11}, {NULL}};
const Segment SGI288666[] = {{&S29831[7], 4}, {NULL}};
const Segment SGI288667[] = {{&S29831[8], 3}, {NULL}};
const Segment SGI288668[] = {{&S29831[13], 14}, {NULL}};
const Segment SGI288669[] = {{&S29831[7], 1}, {NULL}};
const Segment SGI288670[] = {{&S29831[16], 11}, {NULL}};
const Segment SGI288671[] = {{&S29832[7], 4}, {NULL}};
const Segment SGI288672[] = {{&S29832[8], 3}, {NULL}};
const Segment SGI288673[] = {{&S29832[13], 7}, {NULL}};
const Segment SGI288674[] = {{&S29832[25], 14}, {NULL}};
const Segment SGI288675[] = {{&S29832[7], 1}, {NULL}};
const Segment SGI288676[] = {{&S29832[16], 4}, {&S29832[28], 11}, {NULL}};
const Segment SGI288677[] = {{&S15618[49], 3}, {NULL}};
const Segment SGI288678[] = {{&S15618[0], 140}, {NULL}};
const Segment SGI288679[] = {{&S15619[49], 3}, {NULL}};
const Segment SGI288680[] = {{&S15619[0], 140}, {NULL}};
const Segment SGI288681[] = {{&S15622[49], 3}, {NULL}};
const Segment SGI288682[] = {{&S15622[0], 140}, {NULL}};
const Segment SGI288683[] = {{&S15644[49], 3}, {NULL}};
const Segment SGI288684[] = {{&S15644[0], 130}, {NULL}};
const Segment SGI288685[] = {{&S15646[49], 3}, {NULL}};
const Segment SGI288686[] = {{&S15646[0], 130}, {NULL}};
const Segment SGI288687[] = {{&S15647[49], 3}, {NULL}};
const Segment SGI288688[] = {{&S15647[0], 139}, {NULL}};
const Segment SGI288689[] = {{&S15652[49], 3}, {NULL}};
const Segment SGI288690[] = {{&S15652[0], 131}, {NULL}};
const Segment SGI288691[] = {{&S15656[49], 3}, {NULL}};
const Segment SGI288692[] = {{&S15656[0], 140}, {NULL}};
const Segment SGI288693[] = {{&S15658[49], 3}, {NULL}};
const Segment SGI288694[] = {{&S15658[0], 140}, {NULL}};
const Segment SGI288695[] = {{&S15659[49], 3}, {NULL}};
const Segment SGI288696[] = {{&S15659[0], 131}, {NULL}};
const Segment SGI288697[] = {{&S15664[49], 3}, {NULL}};
const Segment SGI288698[] = {{&S15664[0], 140}, {NULL}};
const Segment SGI288699[] = {{&S15667[49], 3}, {NULL}};
const Segment SGI288700[] = {{&S15667[0], 140}, {NULL}};
const Segment SGI288701[] = {{&S15668[49], 3}, {NULL}};
const Segment SGI288702[] = {{&S15668[0], 140}, {NULL}};
const Segment SGI288703[] = {{&S15670[0], 166}, {NULL}};
const Segment SGI288704[] = {{&S30145[56], 4}, {NULL}};
const Segment SGI288705[] = {{&S30145[8], 3}, {NULL}};
const Segment SGI288706[] = {{&S30145[13], 14}, {NULL}};
const Segment SGI288707[] = {{&S30145[7], 1}, {NULL}};
const Segment SGI288708[] = {{&S30145[16], 11}, {NULL}};
const Segment SGI288709[] = {{&S30145[61], 11}, {NULL}};
const Segment SGI288710[] = {{&S30205[59], 4}, {NULL}};
const Segment SGI288711[] = {{&S30205[8], 3}, {NULL}};
const Segment SGI288712[] = {{&S30205[13], 3}, {NULL}};
const Segment SGI288713[] = {{&S30205[7], 1}, {NULL}};
const Segment SGI288714[] = {{&S30206[55], 4}, {NULL}};
const Segment SGI288715[] = {{&S30206[8], 3}, {NULL}};
const Segment SGI288716[] = {{&S30206[13], 3}, {NULL}};
const Segment SGI288717[] = {{&S30206[7], 1}, {NULL}};
const Segment SGI288718[] = {{&S30207[52], 4}, {NULL}};
const Segment SGI288719[] = {{&S30207[7], 3}, {NULL}};
const Segment SGI288720[] = {{&S30207[1], 1}, {NULL}};
const Segment SGI288721[] = {{&S30208[20], 3}, {NULL}};
const Segment SGI288722[] = {{&S30208[25], 3}, {NULL}};
const Segment SGI288723[] = {{&S30208[19], 1}, {NULL}};
const Segment SGI288724[] = {{&S30209[62], 5}, {NULL}};
const Segment SGI288725[] = {{&S30209[8], 3}, {NULL}};
const Segment SGI288726[] = {{&S30209[13], 3}, {NULL}};
const Segment SGI288727[] = {{&S30209[13], 3}, {&S30209[18], 6}, {NULL}};
const Segment SGI288728[] = {{&S30209[7], 1}, {NULL}};
const Segment SGI288729[] = {{&S30210[58], 5}, {NULL}};
const Segment SGI288730[] = {{&S30210[8], 3}, {NULL}};
const Segment SGI288731[] = {{&S30210[13], 14}, {NULL}};
const Segment SGI288732[] = {{&S30210[7], 1}, {NULL}};
const Segment SGI288733[] = {{&S30210[16], 11}, {NULL}};
const Segment SGI288734[] = {{&S30211[60], 5}, {&S30211[60], 5}, {NULL}};
const Segment SGI288735[] = {{&S30211[8], 3}, {NULL}};
const Segment SGI288736[] = {{&S30211[13], 14}, {NULL}};
const Segment SGI288737[] = {{&S30211[7], 1}, {&S30211[7], 1}, {NULL}};
const Segment SGI288738[] = {{&S30211[16], 11}, {NULL}};
const Segment SGI288739[] = {{&S30212[65], 5}, {NULL}};
const Segment SGI288740[] = {{&S30212[8], 3}, {NULL}};
const Segment SGI288741[] = {{&S30212[13], 7}, {NULL}};
const Segment SGI288742[] = {{&S30212[25], 14}, {NULL}};
const Segment SGI288743[] = {{&S30212[7], 1}, {NULL}};
const Segment SGI288744[] = {{&S30212[16], 4}, {&S30212[28], 11}, {NULL}};
const Segment SGI288745[] = {{&S30213[74], 5}, {NULL}};
const Segment SGI288746[] = {{&S30213[8], 3}, {NULL}};
const Segment SGI288747[] = {{&S30213[13], 7}, {NULL}};
const Segment SGI288748[] = {{&S30213[22], 14}, {NULL}};
const Segment SGI288749[] = {{&S30213[7], 1}, {NULL}};
const Segment SGI288750[] = {{&S30213[16], 4}, {&S30213[25], 11}, {&S30213[41], 2}, {NULL}};
const Segment SGI288751[] = {{&S30214[72], 5}, {NULL}};
const Segment SGI288752[] = {{&S30214[8], 3}, {NULL}};
const Segment SGI288753[] = {{&S30214[32], 14}, {NULL}};
const Segment SGI288754[] = {{&S30214[7], 1}, {NULL}};
const Segment SGI288755[] = {{&S30214[35], 11}, {NULL}};
const Segment SGI288756[] = {{&S30503[56], 4}, {NULL}};
const Segment SGI288757[] = {{&S30503[8], 3}, {NULL}};
const Segment SGI288758[] = {{&S30503[13], 14}, {NULL}};
const Segment SGI288759[] = {{&S30503[7], 1}, {NULL}};
const Segment SGI288760[] = {{&S30503[16], 11}, {NULL}};
const Segment SGI288761[] = {{&S30503[61], 11}, {NULL}};
const Segment SGI288762[] = {{&S30576[60], 5}, {NULL}};
const Segment SGI288763[] = {{&S30576[8], 3}, {NULL}};
const Segment SGI288764[] = {{&S30576[13], 14}, {NULL}};
const Segment SGI288765[] = {{&S30576[7], 1}, {NULL}};
const Segment SGI288766[] = {{&S30576[16], 11}, {NULL}};
const Segment SGI288767[] = {{&S30577[60], 5}, {NULL}};
const Segment SGI288768[] = {{&S30577[8], 3}, {NULL}};
const Segment SGI288769[] = {{&S30577[13], 14}, {NULL}};
const Segment SGI288770[] = {{&S30577[7], 1}, {NULL}};
const Segment SGI288771[] = {{&S30577[16], 11}, {NULL}};
const Segment SGI288772[] = {{&S30578[53], 5}, {NULL}};
const Segment SGI288773[] = {{&S30578[8], 3}, {NULL}};
const Segment SGI288774[] = {{&S30578[13], 14}, {NULL}};
const Segment SGI288775[] = {{&S30578[7], 1}, {NULL}};
const Segment SGI288776[] = {{&S30578[16], 11}, {NULL}};
const Segment SGI288777[] = {{&S30578[59], 14}, {NULL}};
const Segment SGI288778[] = {{&S30579[52], 5}, {NULL}};
const Segment SGI288779[] = {{&S30579[8], 3}, {NULL}};
const Segment SGI288780[] = {{&S30579[13], 7}, {NULL}};
const Segment SGI288781[] = {{&S30579[22], 14}, {NULL}};
const Segment SGI288782[] = {{&S30579[7], 1}, {NULL}};
const Segment SGI288783[] = {{&S30579[16], 4}, {&S30579[25], 11}, {NULL}};
const Segment SGI288784[] = {{&S30589[55], 5}, {NULL}};
const Segment SGI288785[] = {{&S30589[8], 3}, {NULL}};
const Segment SGI288786[] = {{&S30589[13], 14}, {NULL}};
const Segment SGI288787[] = {{&S30589[7], 1}, {NULL}};
const Segment SGI288788[] = {{&S30589[16], 11}, {&S30589[32], 2}, {NULL}};
const Segment SGI288789[] = {{&S30590[43], 5}, {NULL}};
const Segment SGI288790[] = {{&S30590[8], 3}, {NULL}};
const Segment SGI288791[] = {{&S30590[13], 14}, {NULL}};
const Segment SGI288792[] = {{&S30590[7], 1}, {NULL}};
const Segment SGI288793[] = {{&S30590[16], 11}, {NULL}};
const Segment SGI288794[] = {{&S30591[53], 5}, {NULL}};
const Segment SGI288795[] = {{&S30591[8], 3}, {NULL}};
const Segment SGI288796[] = {{&S30591[13], 14}, {NULL}};
const Segment SGI288797[] = {{&S30591[7], 1}, {NULL}};
const Segment SGI288798[] = {{&S30591[16], 11}, {NULL}};
const Segment SGI288799[] = {{&S30592[65], 5}, {NULL}};
const Segment SGI288800[] = {{&S30592[8], 3}, {NULL}};
const Segment SGI288801[] = {{&S30592[24], 14}, {NULL}};
const Segment SGI288802[] = {{&S30592[7], 1}, {NULL}};
const Segment SGI288803[] = {{&S30592[15], 4}, {&S30592[27], 11}, {NULL}};
const Segment SGI288804[] = {{&S30593[94], 5}, {NULL}};
const Segment SGI288805[] = {{&S30593[8], 3}, {NULL}};
const Segment SGI288806[] = {{&S30593[13], 7}, {NULL}};
const Segment SGI288807[] = {{&S30593[25], 14}, {NULL}};
const Segment SGI288808[] = {{&S30593[7], 1}, {NULL}};
const Segment SGI288809[] = {{&S30593[16], 4}, {&S30593[28], 11}, {NULL}};
const Segment SGI288810[] = {{&S30594[72], 5}, {NULL}};
const Segment SGI288811[] = {{&S30594[8], 3}, {NULL}};
const Segment SGI288812[] = {{&S30594[13], 7}, {NULL}};
const Segment SGI288813[] = {{&S30594[25], 14}, {NULL}};
const Segment SGI288814[] = {{&S30594[7], 1}, {NULL}};
const Segment SGI288815[] = {{&S30594[16], 4}, {&S30594[28], 11}, {NULL}};
const Segment SGI288816[] = {{&S30595[65], 5}, {NULL}};
const Segment SGI288817[] = {{&S30595[8], 3}, {NULL}};
const Segment SGI288818[] = {{&S30595[13], 14}, {NULL}};
const Segment SGI288819[] = {{&S30595[7], 1}, {NULL}};
const Segment SGI288820[] = {{&S30595[16], 11}, {&S30595[32], 2}, {NULL}};
const Segment SGI288821[] = {{&S30596[53], 5}, {NULL}};
const Segment SGI288822[] = {{&S30596[8], 3}, {NULL}};
const Segment SGI288823[] = {{&S30596[13], 14}, {NULL}};
const Segment SGI288824[] = {{&S30596[7], 1}, {NULL}};
const Segment SGI288825[] = {{&S30596[16], 11}, {NULL}};
const Segment SGI288826[] = {{&S30597[53], 5}, {&S30597[53], 5}, {NULL}};
const Segment SGI288827[] = {{&S30597[8], 3}, {NULL}};
const Segment SGI288828[] = {{&S30597[13], 14}, {NULL}};
const Segment SGI288829[] = {{&S30597[7], 1}, {&S30597[7], 1}, {NULL}};
const Segment SGI288830[] = {{&S30597[16], 11}, {NULL}};
const Segment SGI288831[] = {{&S30598[62], 5}, {NULL}};
const Segment SGI288832[] = {{&S30598[8], 3}, {NULL}};
const Segment SGI288833[] = {{&S30598[13], 3}, {NULL}};
const Segment SGI288834[] = {{&S30598[13], 3}, {&S30598[18], 6}, {NULL}};
const Segment SGI288835[] = {{&S30598[7], 1}, {NULL}};
const Segment SGI288836[] = {{&S30599[64], 5}, {&S30599[64], 5}, {NULL}};
const Segment SGI288837[] = {{&S30599[8], 3}, {NULL}};
const Segment SGI288838[] = {{&S30599[13], 3}, {NULL}};
const Segment SGI288839[] = {{&S30599[13], 3}, {&S30599[18], 6}, {NULL}};
const Segment SGI288840[] = {{&S30599[7], 1}, {&S30599[7], 1}, {NULL}};
const Segment SGI288841[] = {{&S31018[52], 4}, {NULL}};
const Segment SGI288842[] = {{&S31018[8], 3}, {NULL}};
const Segment SGI288843[] = {{&S31018[13], 14}, {NULL}};
const Segment SGI288844[] = {{&S31018[7], 1}, {NULL}};
const Segment SGI288845[] = {{&S31018[16], 11}, {NULL}};
const Segment SGI288846[] = {{&S31018[57], 11}, {NULL}};
const Segment SGI288847[] = {{&S31098[56], 5}, {NULL}};
const Segment SGI288848[] = {{&S31098[8], 3}, {NULL}};
const Segment SGI288849[] = {{&S31098[13], 3}, {NULL}};
const Segment SGI288850[] = {{&S31098[7], 1}, {NULL}};
const Segment SGI288851[] = {{&S31099[56], 5}, {NULL}};
const Segment SGI288852[] = {{&S31099[8], 3}, {NULL}};
const Segment SGI288853[] = {{&S31099[13], 3}, {NULL}};
const Segment SGI288854[] = {{&S31099[7], 1}, {NULL}};
const Segment SGI288855[] = {{&S31100[56], 5}, {NULL}};
const Segment SGI288856[] = {{&S31100[8], 3}, {NULL}};
const Segment SGI288857[] = {{&S31100[13], 3}, {NULL}};
const Segment SGI288858[] = {{&S31101[54], 5}, {&S31101[54], 5}, {NULL}};
const Segment SGI288859[] = {{&S31101[8], 3}, {NULL}};
const Segment SGI288860[] = {{&S31101[13], 3}, {NULL}};
const Segment SGI288861[] = {{&S31101[7], 1}, {&S31101[7], 1}, {NULL}};
const Segment SGI288862[] = {{&S31186[67], 4}, {&S31186[81], 4}, {NULL}};
const Segment SGI288863[] = {{&S31186[81], 4}, {NULL}};
const Segment SGI288864[] = {{&S31186[8], 3}, {NULL}};
const Segment SGI288865[] = {{&S31186[13], 3}, {NULL}};
const Segment SGI288866[] = {{&S31186[109], 10}, {NULL}};
const Segment SGI288867[] = {{&S31186[7], 1}, {&S31186[7], 1}, {NULL}};
const Segment SGI288868[] = {{&S31187[69], 4}, {&S31187[89], 4}, {NULL}};
const Segment SGI288869[] = {{&S31187[8], 3}, {NULL}};
const Segment SGI288870[] = {{&S31187[13], 3}, {NULL}};
const Segment SGI288871[] = {{&S31187[7], 1}, {&S31187[7], 1}, {NULL}};
const Segment SGI288872[] = {{&S31189[69], 4}, {NULL}};
const Segment SGI288873[] = {{&S31189[8], 3}, {NULL}};
const Segment SGI288874[] = {{&S31189[13], 3}, {NULL}};
const Segment SGI288875[] = {{&S31189[97], 10}, {NULL}};
const Segment SGI288876[] = {{&S31189[7], 1}, {NULL}};
const Segment SGI288877[] = {{&S31190[69], 4}, {NULL}};
const Segment SGI288878[] = {{&S31190[8], 3}, {NULL}};
const Segment SGI288879[] = {{&S31190[13], 3}, {NULL}};
const Segment SGI288880[] = {{&S31190[7], 1}, {NULL}};
const Segment SGI288881[] = {{&S31191[64], 4}, {NULL}};
const Segment SGI288882[] = {{&S31191[8], 3}, {NULL}};
const Segment SGI288883[] = {{&S31191[13], 3}, {NULL}};
const Segment SGI288884[] = {{&S31191[7], 1}, {NULL}};
const Segment SGI288885[] = {{&S31192[67], 4}, {&S31192[67], 4}, {NULL}};
const Segment SGI288886[] = {{&S31192[8], 3}, {NULL}};
const Segment SGI288887[] = {{&S31192[13], 3}, {NULL}};
const Segment SGI288888[] = {{&S31192[95], 10}, {NULL}};
const Segment SGI288889[] = {{&S31192[7], 1}, {&S31192[7], 1}, {NULL}};
const Segment SGI288890[] = {{&S31193[68], 4}, {&S31193[90], 4}, {NULL}};
const Segment SGI288891[] = {{&S31193[8], 3}, {NULL}};
const Segment SGI288892[] = {{&S31193[13], 3}, {NULL}};
const Segment SGI288893[] = {{&S31193[7], 1}, {&S31193[7], 1}, {NULL}};
const Segment SGI288894[] = {{&S31194[67], 4}, {&S31194[85], 4}, {NULL}};
const Segment SGI288895[] = {{&S31194[8], 3}, {NULL}};
const Segment SGI288896[] = {{&S31194[13], 3}, {NULL}};
const Segment SGI288897[] = {{&S31194[93], 10}, {NULL}};
const Segment SGI288898[] = {{&S31194[7], 1}, {&S31194[7], 1}, {NULL}};
const Segment SGI288899[] = {{&S31197[68], 4}, {&S31197[68], 4}, {NULL}};
const Segment SGI288900[] = {{&S31197[8], 3}, {NULL}};
const Segment SGI288901[] = {{&S31197[13], 3}, {NULL}};
const Segment SGI288902[] = {{&S31197[7], 1}, {&S31197[7], 1}, {NULL}};
const Segment SGI288903[] = {{&S31198[90], 4}, {NULL}};
const Segment SGI288904[] = {{&S31198[8], 3}, {NULL}};
const Segment SGI288905[] = {{&S31198[13], 3}, {NULL}};
const Segment SGI288906[] = {{&S31198[7], 1}, {NULL}};
const Segment SGI288907[] = {{&S31198[63], 11}, {NULL}};
const Segment SGI288908[] = {{&S31312[56], 4}, {NULL}};
const Segment SGI288909[] = {{&S31312[8], 3}, {NULL}};
const Segment SGI288910[] = {{&S31312[13], 14}, {NULL}};
const Segment SGI288911[] = {{&S31312[7], 1}, {NULL}};
const Segment SGI288912[] = {{&S31312[16], 11}, {NULL}};
const Segment SGI288913[] = {{&S31312[61], 11}, {NULL}};
const Segment SGI288914[] = {{&S31332[56], 5}, {NULL}};
const Segment SGI288915[] = {{&S31332[8], 3}, {NULL}};
const Segment SGI288916[] = {{&S31332[13], 3}, {NULL}};
const Segment SGI288917[] = {{&S31332[7], 1}, {NULL}};
const Segment SGI288918[] = {{&S31333[56], 5}, {NULL}};
const Segment SGI288919[] = {{&S31333[8], 3}, {NULL}};
const Segment SGI288920[] = {{&S31333[13], 3}, {NULL}};
const Segment SGI288921[] = {{&S31333[7], 1}, {NULL}};
const Segment SGI288922[] = {{&S31335[54], 5}, {&S31335[54], 5}, {NULL}};
const Segment SGI288923[] = {{&S31335[8], 3}, {NULL}};
const Segment SGI288924[] = {{&S31335[13], 3}, {NULL}};
const Segment SGI288925[] = {{&S31335[7], 1}, {&S31335[7], 1}, {NULL}};
const Segment SGI288926[] = {{&S31336[67], 5}, {NULL}};
const Segment SGI288927[] = {{&S31336[8], 3}, {&S31336[21], 3}, {NULL}};
const Segment SGI288928[] = {{&S31336[13], 16}, {NULL}};
const Segment SGI288929[] = {{&S31336[26], 3}, {NULL}};
const Segment SGI288930[] = {{&S31336[7], 1}, {NULL}};
const Segment SGI288931[] = {{&S31337[65], 6}, {NULL}};
const Segment SGI288932[] = {{&S31337[8], 3}, {NULL}};
const Segment SGI288933[] = {{&S31337[13], 3}, {NULL}};
const Segment SGI288934[] = {{&S31338[43], 5}, {NULL}};
const Segment SGI288935[] = {{&S31338[8], 3}, {NULL}};
const Segment SGI288936[] = {{&S31338[13], 14}, {NULL}};
const Segment SGI288937[] = {{&S31338[7], 1}, {NULL}};
const Segment SGI288938[] = {{&S31338[16], 11}, {NULL}};
const Segment SGI288939[] = {{&S31339[55], 5}, {NULL}};
const Segment SGI288940[] = {{&S31339[8], 3}, {NULL}};
const Segment SGI288941[] = {{&S31339[13], 14}, {NULL}};
const Segment SGI288942[] = {{&S31339[7], 1}, {NULL}};
const Segment SGI288943[] = {{&S31339[16], 11}, {&S31339[32], 2}, {NULL}};
const Segment SGI288944[] = {{&S31340[60], 5}, {NULL}};
const Segment SGI288945[] = {{&S31340[8], 3}, {NULL}};
const Segment SGI288946[] = {{&S31340[13], 14}, {NULL}};
const Segment SGI288947[] = {{&S31340[7], 1}, {NULL}};
const Segment SGI288948[] = {{&S31340[16], 11}, {NULL}};
const Segment SGI288949[] = {{&S31341[75], 5}, {NULL}};
const Segment SGI288950[] = {{&S31341[8], 3}, {NULL}};
const Segment SGI288951[] = {{&S31341[13], 7}, {NULL}};
const Segment SGI288952[] = {{&S31341[25], 14}, {NULL}};
const Segment SGI288953[] = {{&S31341[7], 1}, {NULL}};
const Segment SGI288954[] = {{&S31341[16], 4}, {&S31341[28], 11}, {NULL}};
const Segment SGI288955[] = {{&S31385[56], 4}, {NULL}};
const Segment SGI288956[] = {{&S31385[8], 3}, {NULL}};
const Segment SGI288957[] = {{&S31385[13], 14}, {NULL}};
const Segment SGI288958[] = {{&S31385[7], 1}, {NULL}};
const Segment SGI288959[] = {{&S31385[16], 11}, {NULL}};
const Segment SGI288960[] = {{&S31385[61], 11}, {NULL}};
const Segment SGI288961[] = {{&S31438[83], 4}, {NULL}};
const Segment SGI288962[] = {{&S31438[8], 3}, {NULL}};
const Segment SGI288963[] = {{&S31438[13], 9}, {NULL}};
const Segment SGI288964[] = {{&S31438[33], 13}, {NULL}};
const Segment SGI288965[] = {{&S31438[7], 1}, {NULL}};
const Segment SGI288966[] = {{&S31477[52], 4}, {NULL}};
const Segment SGI288967[] = {{&S31477[8], 3}, {NULL}};
const Segment SGI288968[] = {{&S31477[13], 14}, {NULL}};
const Segment SGI288969[] = {{&S31477[7], 1}, {NULL}};
const Segment SGI288970[] = {{&S31477[16], 11}, {NULL}};
const Segment SGI288971[] = {{&S31477[57], 11}, {NULL}};
const Segment SGI288972[] = {{&S31601[59], 4}, {NULL}};
const Segment SGI288973[] = {{&S31601[8], 3}, {NULL}};
const Segment SGI288974[] = {{&S31601[13], 14}, {NULL}};
const Segment SGI288975[] = {{&S31601[7], 1}, {NULL}};
const Segment SGI288976[] = {{&S31601[16], 11}, {NULL}};
const Segment SGI288977[] = {{&S31601[64], 11}, {NULL}};
const Segment SGI288978[] = {{&S31653[59], 4}, {NULL}};
const Segment SGI288979[] = {{&S31653[8], 3}, {NULL}};
const Segment SGI288980[] = {{&S31653[13], 14}, {NULL}};
const Segment SGI288981[] = {{&S31653[7], 1}, {NULL}};
const Segment SGI288982[] = {{&S31653[16], 11}, {NULL}};
const Segment SGI288983[] = {{&S31653[64], 11}, {NULL}};
const Segment SGI288984[] = {{&S31674[64], 4}, {NULL}};
const Segment SGI288985[] = {{&S31674[8], 3}, {NULL}};
const Segment SGI288986[] = {{&S31674[13], 14}, {NULL}};
const Segment SGI288987[] = {{&S31674[7], 1}, {NULL}};
const Segment SGI288988[] = {{&S31674[16], 11}, {&S31674[32], 2}, {NULL}};
const Segment SGI288989[] = {{&S31674[69], 11}, {NULL}};
const Segment SGI288990[] = {{&S31701[55], 5}, {NULL}};
const Segment SGI288991[] = {{&S31701[8], 3}, {NULL}};
const Segment SGI288992[] = {{&S31701[13], 14}, {NULL}};
const Segment SGI288993[] = {{&S31701[7], 1}, {NULL}};
const Segment SGI288994[] = {{&S31701[16], 11}, {&S31701[32], 2}, {NULL}};
const Segment SGI288995[] = {{&S31702[60], 5}, {NULL}};
const Segment SGI288996[] = {{&S31702[8], 3}, {NULL}};
const Segment SGI288997[] = {{&S31702[13], 14}, {NULL}};
const Segment SGI288998[] = {{&S31702[7], 1}, {NULL}};
const Segment SGI288999[] = {{&S31702[16], 11}, {NULL}};
const Segment SGI289000[] = {{&S31732[62], 5}, {NULL}};
const Segment SGI289001[] = {{&S31732[8], 3}, {NULL}};
const Segment SGI289002[] = {{&S31732[13], 3}, {NULL}};
const Segment SGI289003[] = {{&S31732[13], 3}, {&S31732[18], 6}, {NULL}};
const Segment SGI289004[] = {{&S31732[7], 1}, {NULL}};
const Segment SGI289005[] = {{&S31863[71], 4}, {NULL}};
const Segment SGI289006[] = {{&S31863[8], 3}, {NULL}};
const Segment SGI289007[] = {{&S31863[13], 14}, {NULL}};
const Segment SGI289008[] = {{&S31863[7], 1}, {NULL}};
const Segment SGI289009[] = {{&S31863[16], 11}, {NULL}};
const Segment SGI289010[] = {{&S31863[78], 11}, {NULL}};
const Segment SGI289011[] = {{&S31896[60], 5}, {NULL}};
const Segment SGI289012[] = {{&S31896[8], 3}, {NULL}};
const Segment SGI289013[] = {{&S31896[13], 14}, {NULL}};
const Segment SGI289014[] = {{&S31896[7], 1}, {NULL}};
const Segment SGI289015[] = {{&S31896[16], 11}, {NULL}};
const Segment SGI289016[] = {{&S31897[65], 5}, {NULL}};
const Segment SGI289017[] = {{&S31897[8], 3}, {NULL}};
const Segment SGI289018[] = {{&S31897[13], 14}, {NULL}};
const Segment SGI289019[] = {{&S31897[7], 1}, {NULL}};
const Segment SGI289020[] = {{&S31897[16], 11}, {&S31897[32], 2}, {NULL}};
const Segment SGI289021[] = {{&S31898[67], 5}, {NULL}};
const Segment SGI289022[] = {{&S31898[8], 3}, {NULL}};
const Segment SGI289023[] = {{&S31898[13], 7}, {NULL}};
const Segment SGI289024[] = {{&S31898[25], 14}, {NULL}};
const Segment SGI289025[] = {{&S31898[7], 1}, {NULL}};
const Segment SGI289026[] = {{&S31898[16], 4}, {&S31898[28], 11}, {NULL}};
const Segment SGI289027[] = {{&S32159[61], 4}, {NULL}};
const Segment SGI289028[] = {{&S32159[8], 3}, {NULL}};
const Segment SGI289029[] = {{&S32159[13], 14}, {NULL}};
const Segment SGI289030[] = {{&S32159[7], 1}, {NULL}};
const Segment SGI289031[] = {{&S32159[16], 11}, {NULL}};
const Segment SGI289032[] = {{&S32159[68], 11}, {NULL}};
const Segment SGI289033[] = {{&S32168[61], 4}, {NULL}};
const Segment SGI289034[] = {{&S32168[8], 3}, {NULL}};
const Segment SGI289035[] = {{&S32168[13], 14}, {NULL}};
const Segment SGI289036[] = {{&S32168[7], 1}, {NULL}};
const Segment SGI289037[] = {{&S32168[16], 11}, {NULL}};
const Segment SGI289038[] = {{&S32168[68], 11}, {NULL}};
const Segment SGI289039[] = {{&S32194[50], 5}, {NULL}};
const Segment SGI289040[] = {{&S32194[8], 3}, {NULL}};
const Segment SGI289041[] = {{&S32194[13], 14}, {NULL}};
const Segment SGI289042[] = {{&S32194[7], 1}, {NULL}};
const Segment SGI289043[] = {{&S32194[16], 11}, {NULL}};
const Segment SGI289044[] = {{&S32195[55], 5}, {NULL}};
const Segment SGI289045[] = {{&S32195[8], 3}, {NULL}};
const Segment SGI289046[] = {{&S32195[13], 14}, {NULL}};
const Segment SGI289047[] = {{&S32195[7], 1}, {NULL}};
const Segment SGI289048[] = {{&S32195[16], 11}, {&S32195[32], 2}, {NULL}};
const Segment SGI289049[] = {{&S32220[69], 5}, {NULL}};
const Segment SGI289050[] = {{&S32220[8], 3}, {NULL}};
const Segment SGI289051[] = {{&S32220[13], 9}, {NULL}};
const Segment SGI289052[] = {{&S32220[30], 13}, {NULL}};
const Segment SGI289053[] = {{&S32220[7], 1}, {NULL}};
const Segment SGI289054[] = {{&S32221[69], 5}, {&S32221[69], 5}, {NULL}};
const Segment SGI289055[] = {{&S32221[8], 3}, {NULL}};
const Segment SGI289056[] = {{&S32221[13], 9}, {NULL}};
const Segment SGI289057[] = {{&S32221[30], 13}, {NULL}};
const Segment SGI289058[] = {{&S32221[7], 1}, {&S32221[7], 1}, {NULL}};
const Segment SGI289059[] = {{&S32222[64], 5}, {NULL}};
const Segment SGI289060[] = {{&S32222[8], 3}, {NULL}};
const Segment SGI289061[] = {{&S32222[13], 3}, {NULL}};
const Segment SGI289062[] = {{&S32222[13], 3}, {&S32222[18], 6}, {NULL}};
const Segment SGI289063[] = {{&S32222[7], 1}, {NULL}};
const Segment SGI289064[] = {{&S32312[53], 5}, {NULL}};
const Segment SGI289065[] = {{&S32312[8], 3}, {NULL}};
const Segment SGI289066[] = {{&S32312[13], 14}, {NULL}};
const Segment SGI289067[] = {{&S32312[7], 1}, {NULL}};
const Segment SGI289068[] = {{&S32312[16], 11}, {NULL}};
const Segment SGI289069[] = {{&S32313[60], 5}, {NULL}};
const Segment SGI289070[] = {{&S32313[8], 3}, {NULL}};
const Segment SGI289071[] = {{&S32313[13], 14}, {NULL}};
const Segment SGI289072[] = {{&S32313[7], 1}, {NULL}};
const Segment SGI289073[] = {{&S32313[16], 11}, {NULL}};
const Segment SGI289074[] = {{&S32314[52], 5}, {NULL}};
const Segment SGI289075[] = {{&S32314[8], 3}, {NULL}};
const Segment SGI289076[] = {{&S32314[13], 7}, {NULL}};
const Segment SGI289077[] = {{&S32314[22], 14}, {NULL}};
const Segment SGI289078[] = {{&S32314[7], 1}, {NULL}};
const Segment SGI289079[] = {{&S32314[16], 4}, {&S32314[25], 11}, {NULL}};
const Segment SGI289080[] = {{&S32315[53], 5}, {NULL}};
const Segment SGI289081[] = {{&S32315[8], 3}, {NULL}};
const Segment SGI289082[] = {{&S32315[13], 14}, {NULL}};
const Segment SGI289083[] = {{&S32315[7], 1}, {NULL}};
const Segment SGI289084[] = {{&S32315[16], 11}, {NULL}};
const Segment SGI289085[] = {{&S32315[59], 13}, {NULL}};
const Segment SGI289086[] = {{&S32316[60], 5}, {NULL}};
const Segment SGI289087[] = {{&S32316[8], 3}, {NULL}};
const Segment SGI289088[] = {{&S32316[13], 14}, {NULL}};
const Segment SGI289089[] = {{&S32316[7], 1}, {NULL}};
const Segment SGI289090[] = {{&S32316[16], 11}, {NULL}};
const Segment SGI289091[] = {{&S32316[66], 13}, {NULL}};
const Segment SGI289092[] = {{&S32404[69], 5}, {NULL}};
const Segment SGI289093[] = {{&S32404[8], 3}, {NULL}};
const Segment SGI289094[] = {{&S32404[13], 9}, {NULL}};
const Segment SGI289095[] = {{&S32404[30], 13}, {NULL}};
const Segment SGI289096[] = {{&S32404[7], 1}, {NULL}};
const Segment SGI289097[] = {{&S32441[99], 4}, {NULL}};
const Segment SGI289098[] = {{&S32441[8], 3}, {NULL}};
const Segment SGI289099[] = {{&S32441[13], 3}, {NULL}};
const Segment SGI289100[] = {{&S32441[13], 3}, {&S32441[71], 6}, {NULL}};
const Segment SGI289101[] = {{&S32441[71], 12}, {NULL}};
const Segment SGI289102[] = {{&S32441[7], 1}, {NULL}};
const Segment SGI289103[] = {{&S32639[53], 5}, {NULL}};
const Segment SGI289104[] = {{&S32639[8], 3}, {NULL}};
const Segment SGI289105[] = {{&S32639[13], 14}, {NULL}};
const Segment SGI289106[] = {{&S32639[7], 1}, {NULL}};
const Segment SGI289107[] = {{&S32639[16], 11}, {NULL}};
const Segment SGI289108[] = {{&S32640[60], 5}, {NULL}};
const Segment SGI289109[] = {{&S32640[8], 3}, {NULL}};
const Segment SGI289110[] = {{&S32640[13], 14}, {NULL}};
const Segment SGI289111[] = {{&S32640[7], 1}, {NULL}};
const Segment SGI289112[] = {{&S32640[16], 11}, {NULL}};
const Segment SGI289113[] = {{&S32641[53], 5}, {NULL}};
const Segment SGI289114[] = {{&S32641[8], 3}, {NULL}};
const Segment SGI289115[] = {{&S32641[13], 14}, {NULL}};
const Segment SGI289116[] = {{&S32641[7], 1}, {NULL}};
const Segment SGI289117[] = {{&S32641[16], 11}, {NULL}};
const Segment SGI289118[] = {{&S32641[59], 14}, {NULL}};
const Segment SGI289119[] = {{&S32642[60], 5}, {NULL}};
const Segment SGI289120[] = {{&S32642[8], 3}, {NULL}};
const Segment SGI289121[] = {{&S32642[13], 14}, {NULL}};
const Segment SGI289122[] = {{&S32642[7], 1}, {NULL}};
const Segment SGI289123[] = {{&S32642[16], 11}, {NULL}};
const Segment SGI289124[] = {{&S32642[66], 14}, {NULL}};
const Segment SGI289125[] = {{&S32643[52], 5}, {NULL}};
const Segment SGI289126[] = {{&S32643[8], 3}, {NULL}};
const Segment SGI289127[] = {{&S32643[13], 7}, {NULL}};
const Segment SGI289128[] = {{&S32643[22], 14}, {NULL}};
const Segment SGI289129[] = {{&S32643[7], 1}, {NULL}};
const Segment SGI289130[] = {{&S32643[16], 4}, {&S32643[25], 11}, {NULL}};
const Segment SGI289131[] = {{&S32671[69], 5}, {NULL}};
const Segment SGI289132[] = {{&S32671[8], 3}, {NULL}};
const Segment SGI289133[] = {{&S32671[13], 9}, {NULL}};
const Segment SGI289134[] = {{&S32671[30], 13}, {NULL}};
const Segment SGI289135[] = {{&S32671[7], 1}, {NULL}};
const Segment SGI289136[] = {{&S32720[56], 4}, {NULL}};
const Segment SGI289137[] = {{&S32720[8], 3}, {NULL}};
const Segment SGI289138[] = {{&S32720[13], 14}, {NULL}};
const Segment SGI289139[] = {{&S32720[7], 1}, {NULL}};
const Segment SGI289140[] = {{&S32720[16], 11}, {NULL}};
const Segment SGI289141[] = {{&S32720[61], 11}, {NULL}};
const Segment SGI289142[] = {{&S32797[56], 4}, {NULL}};
const Segment SGI289143[] = {{&S32797[8], 3}, {NULL}};
const Segment SGI289144[] = {{&S32797[13], 14}, {NULL}};
const Segment SGI289145[] = {{&S32797[7], 1}, {NULL}};
const Segment SGI289146[] = {{&S32797[16], 11}, {NULL}};
const Segment SGI289147[] = {{&S32797[61], 11}, {NULL}};
const Segment SGI289148[] = {{&S32906[69], 5}, {NULL}};
const Segment SGI289149[] = {{&S32906[8], 3}, {NULL}};
const Segment SGI289150[] = {{&S32906[13], 9}, {NULL}};
const Segment SGI289151[] = {{&S32906[30], 13}, {NULL}};
const Segment SGI289152[] = {{&S32906[7], 1}, {NULL}};
const Segment SGI289153[] = {{&S32907[69], 5}, {&S32907[69], 5}, {NULL}};
const Segment SGI289154[] = {{&S32907[8], 3}, {NULL}};
const Segment SGI289155[] = {{&S32907[13], 9}, {NULL}};
const Segment SGI289156[] = {{&S32907[30], 13}, {NULL}};
const Segment SGI289157[] = {{&S32907[7], 1}, {&S32907[7], 1}, {NULL}};
const Segment SGI289158[] = {{&S32942[63], 4}, {NULL}};
const Segment SGI289159[] = {{&S32942[8], 3}, {NULL}};
const Segment SGI289160[] = {{&S32942[13], 14}, {NULL}};
const Segment SGI289161[] = {{&S32942[7], 1}, {NULL}};
const Segment SGI289162[] = {{&S32942[16], 11}, {NULL}};
const Segment SGI289163[] = {{&S32942[70], 11}, {NULL}};
const Segment SGI289164[] = {{&S33033[56], 4}, {NULL}};
const Segment SGI289165[] = {{&S33033[8], 3}, {NULL}};
const Segment SGI289166[] = {{&S33033[13], 14}, {NULL}};
const Segment SGI289167[] = {{&S33033[7], 1}, {NULL}};
const Segment SGI289168[] = {{&S33033[16], 11}, {NULL}};
const Segment SGI289169[] = {{&S33033[61], 11}, {NULL}};
const Segment SGI289170[] = {{&S33176[68], 4}, {NULL}};
const Segment SGI289171[] = {{&S33176[8], 3}, {NULL}};
const Segment SGI289172[] = {{&S33176[13], 3}, {NULL}};
const Segment SGI289173[] = {{&S33176[74], 10}, {NULL}};
const Segment SGI289174[] = {{&S33176[7], 1}, {NULL}};
const Segment SGI289175[] = {{&S33177[17], 1}, {NULL}};
const Segment SGI289176[] = {{&S33177[61], 4}, {NULL}};
const Segment SGI289177[] = {{&S33177[8], 3}, {NULL}};
const Segment SGI289178[] = {{&S33177[13], 3}, {NULL}};
const Segment SGI289179[] = {{&S33177[7], 1}, {NULL}};
const Segment SGI289180[] = {{&S33178[74], 4}, {&S33178[74], 4}, {NULL}};
const Segment SGI289181[] = {{&S33178[8], 3}, {NULL}};
const Segment SGI289182[] = {{&S33178[13], 3}, {NULL}};
const Segment SGI289183[] = {{&S33178[7], 1}, {&S33178[7], 1}, {NULL}};
const Segment SGI289184[] = {{&S33179[60], 4}, {NULL}};
const Segment SGI289185[] = {{&S33179[8], 3}, {NULL}};
const Segment SGI289186[] = {{&S33179[13], 3}, {NULL}};
const Segment SGI289187[] = {{&S33179[7], 1}, {NULL}};
const Segment SGI289188[] = {{&S33180[70], 4}, {NULL}};
const Segment SGI289189[] = {{&S33180[8], 3}, {NULL}};
const Segment SGI289190[] = {{&S33180[13], 3}, {NULL}};
const Segment SGI289191[] = {{&S33180[81], 10}, {NULL}};
const Segment SGI289192[] = {{&S33181[65], 4}, {NULL}};
const Segment SGI289193[] = {{&S33181[8], 3}, {NULL}};
const Segment SGI289194[] = {{&S33181[13], 3}, {NULL}};
const Segment SGI289195[] = {{&S33181[76], 10}, {NULL}};
const Segment SGI289196[] = {{&S33181[7], 1}, {NULL}};
const Segment SGI289197[] = {{&S33182[69], 4}, {NULL}};
const Segment SGI289198[] = {{&S33182[8], 3}, {NULL}};
const Segment SGI289199[] = {{&S33182[13], 3}, {NULL}};
const Segment SGI289200[] = {{&S33182[7], 1}, {NULL}};
const Segment SGI289201[] = {{&S33182[24], 12}, {NULL}};
const Segment SGI289202[] = {{&S33183[8], 3}, {NULL}};
const Segment SGI289203[] = {{&S33183[13], 3}, {NULL}};
const Segment SGI289204[] = {{&S33183[60], 11}, {NULL}};
const Segment SGI289205[] = {{&S33184[8], 3}, {NULL}};
const Segment SGI289206[] = {{&S33184[13], 3}, {NULL}};
const Segment SGI289207[] = {{&S33184[72], 10}, {NULL}};
const Segment SGI289208[] = {{&S33184[57], 11}, {NULL}};
const Segment SGI289209[] = {{&S33185[127], 4}, {NULL}};
const Segment SGI289210[] = {{&S33185[8], 3}, {&S33185[8], 3}, {NULL}};
const Segment SGI289211[] = {{&S33185[13], 7}, {&S33185[78], 3}, {NULL}};
const Segment SGI289212[] = {{&S33185[25], 14}, {&S33185[78], 3}, {NULL}};
const Segment SGI289213[] = {{&S33185[7], 1}, {NULL}};
const Segment SGI289214[] = {{&S33185[16], 4}, {&S33185[28], 11}, {NULL}};
const Segment SGI289215[] = {{&S33186[69], 5}, {NULL}};
const Segment SGI289216[] = {{&S33186[8], 3}, {NULL}};
const Segment SGI289217[] = {{&S33186[13], 9}, {NULL}};
const Segment SGI289218[] = {{&S33186[30], 13}, {NULL}};
const Segment SGI289219[] = {{&S33186[7], 1}, {NULL}};
const Segment SGI289220[] = {{&S33191[76], 4}, {NULL}};
const Segment SGI289221[] = {{&S33191[8], 3}, {NULL}};
const Segment SGI289222[] = {{&S33191[13], 7}, {NULL}};
const Segment SGI289223[] = {{&S33191[25], 14}, {NULL}};
const Segment SGI289224[] = {{&S33191[7], 1}, {NULL}};
const Segment SGI289225[] = {{&S33191[16], 4}, {&S33191[28], 11}, {NULL}};
const Segment SGI289226[] = {{&S33192[8], 3}, {NULL}};
const Segment SGI289227[] = {{&S33192[13], 7}, {NULL}};
const Segment SGI289228[] = {{&S33192[25], 14}, {NULL}};
const Segment SGI289229[] = {{&S33192[16], 4}, {&S33192[28], 11}, {NULL}};
const Segment SGI289230[] = {{&S33258[7], 4}, {NULL}};
const Segment SGI289231[] = {{&S33258[8], 3}, {NULL}};
const Segment SGI289232[] = {{&S33258[13], 3}, {NULL}};
const Segment SGI289233[] = {{&S33258[7], 1}, {NULL}};
const Segment SGI289234[] = {{&S33259[7], 4}, {NULL}};
const Segment SGI289235[] = {{&S33259[8], 3}, {NULL}};
const Segment SGI289236[] = {{&S33259[13], 3}, {NULL}};
const Segment SGI289237[] = {{&S33259[7], 1}, {NULL}};
const Segment SGI289238[] = {{&S33284[60], 5}, {NULL}};
const Segment SGI289239[] = {{&S33284[8], 3}, {NULL}};
const Segment SGI289240[] = {{&S33284[13], 14}, {NULL}};
const Segment SGI289241[] = {{&S33284[7], 1}, {NULL}};
const Segment SGI289242[] = {{&S33284[16], 11}, {NULL}};
const Segment SGI289243[] = {{&S33285[65], 5}, {NULL}};
const Segment SGI289244[] = {{&S33285[8], 3}, {NULL}};
const Segment SGI289245[] = {{&S33285[13], 14}, {NULL}};
const Segment SGI289246[] = {{&S33285[7], 1}, {NULL}};
const Segment SGI289247[] = {{&S33285[16], 11}, {&S33285[32], 2}, {NULL}};
const Segment SGI289248[] = {{&S33309[55], 5}, {NULL}};
const Segment SGI289249[] = {{&S33309[8], 3}, {NULL}};
const Segment SGI289250[] = {{&S33309[13], 14}, {NULL}};
const Segment SGI289251[] = {{&S33309[7], 1}, {NULL}};
const Segment SGI289252[] = {{&S33309[16], 11}, {&S33309[32], 2}, {NULL}};
const Segment SGI289253[] = {{&S33310[60], 5}, {NULL}};
const Segment SGI289254[] = {{&S33310[8], 3}, {NULL}};
const Segment SGI289255[] = {{&S33310[13], 14}, {NULL}};
const Segment SGI289256[] = {{&S33310[7], 1}, {NULL}};
const Segment SGI289257[] = {{&S33310[16], 11}, {NULL}};
const Segment SGI289258[] = {{&S33311[53], 5}, {NULL}};
const Segment SGI289259[] = {{&S33311[8], 3}, {NULL}};
const Segment SGI289260[] = {{&S33311[13], 14}, {NULL}};
const Segment SGI289261[] = {{&S33311[7], 1}, {NULL}};
const Segment SGI289262[] = {{&S33311[16], 11}, {NULL}};
const Segment SGI289263[] = {{&S33311[59], 13}, {NULL}};
const Segment SGI289264[] = {{&S33312[65], 5}, {NULL}};
const Segment SGI289265[] = {{&S33312[8], 3}, {NULL}};
const Segment SGI289266[] = {{&S33312[13], 14}, {NULL}};
const Segment SGI289267[] = {{&S33312[7], 1}, {NULL}};
const Segment SGI289268[] = {{&S33312[16], 11}, {&S33312[32], 2}, {NULL}};
const Segment SGI289269[] = {{&S33312[71], 13}, {NULL}};
const Segment SGI289270[] = {{&S33325[57], 4}, {NULL}};
const Segment SGI289271[] = {{&S33325[8], 3}, {NULL}};
const Segment SGI289272[] = {{&S33325[13], 3}, {NULL}};
const Segment SGI289273[] = {{&S33325[7], 1}, {NULL}};
const Segment SGI289274[] = {{&S33325[62], 11}, {NULL}};
const Segment SGI289275[] = {{&S33447[56], 4}, {NULL}};
const Segment SGI289276[] = {{&S33447[8], 3}, {NULL}};
const Segment SGI289277[] = {{&S33447[13], 14}, {NULL}};
const Segment SGI289278[] = {{&S33447[7], 1}, {NULL}};
const Segment SGI289279[] = {{&S33447[16], 11}, {NULL}};
const Segment SGI289280[] = {{&S33447[61], 11}, {NULL}};
const Segment SGI289281[] = {{&S33512[53], 5}, {NULL}};
const Segment SGI289282[] = {{&S33512[8], 3}, {NULL}};
const Segment SGI289283[] = {{&S33512[13], 14}, {NULL}};
const Segment SGI289284[] = {{&S33512[7], 1}, {NULL}};
const Segment SGI289285[] = {{&S33512[16], 11}, {NULL}};
const Segment SGI289286[] = {{&S33513[53], 5}, {&S33513[53], 5}, {NULL}};
const Segment SGI289287[] = {{&S33513[8], 3}, {NULL}};
const Segment SGI289288[] = {{&S33513[13], 14}, {NULL}};
const Segment SGI289289[] = {{&S33513[7], 1}, {&S33513[7], 1}, {NULL}};
const Segment SGI289290[] = {{&S33513[16], 11}, {NULL}};
const Segment SGI289291[] = {{&S33515[73], 5}, {NULL}};
const Segment SGI289292[] = {{&S33515[8], 3}, {NULL}};
const Segment SGI289293[] = {{&S33515[13], 7}, {NULL}};
const Segment SGI289294[] = {{&S33515[25], 14}, {NULL}};
const Segment SGI289295[] = {{&S33515[7], 1}, {NULL}};
const Segment SGI289296[] = {{&S33515[16], 4}, {&S33515[28], 11}, {NULL}};
const Segment SGI289297[] = {{&S33517[43], 5}, {NULL}};
const Segment SGI289298[] = {{&S33517[8], 3}, {NULL}};
const Segment SGI289299[] = {{&S33517[13], 14}, {NULL}};
const Segment SGI289300[] = {{&S33517[7], 1}, {NULL}};
const Segment SGI289301[] = {{&S33517[16], 11}, {NULL}};
const Segment SGI289302[] = {{&S33518[65], 5}, {NULL}};
const Segment SGI289303[] = {{&S33518[8], 3}, {NULL}};
const Segment SGI289304[] = {{&S33518[13], 14}, {NULL}};
const Segment SGI289305[] = {{&S33518[7], 1}, {NULL}};
const Segment SGI289306[] = {{&S33518[16], 11}, {&S33518[32], 2}, {NULL}};
const Segment SGI289307[] = {{&S33519[60], 5}, {NULL}};
const Segment SGI289308[] = {{&S33519[8], 3}, {NULL}};
const Segment SGI289309[] = {{&S33519[13], 14}, {NULL}};
const Segment SGI289310[] = {{&S33519[7], 1}, {NULL}};
const Segment SGI289311[] = {{&S33519[16], 11}, {NULL}};
const Segment SGI289312[] = {{&S33520[53], 5}, {NULL}};
const Segment SGI289313[] = {{&S33520[8], 3}, {NULL}};
const Segment SGI289314[] = {{&S33520[13], 14}, {NULL}};
const Segment SGI289315[] = {{&S33520[7], 1}, {NULL}};
const Segment SGI289316[] = {{&S33520[16], 11}, {NULL}};
const Segment SGI289317[] = {{&S33520[59], 14}, {NULL}};
const Segment SGI289318[] = {{&S33521[72], 5}, {NULL}};
const Segment SGI289319[] = {{&S33521[8], 3}, {NULL}};
const Segment SGI289320[] = {{&S33521[13], 7}, {NULL}};
const Segment SGI289321[] = {{&S33521[25], 14}, {NULL}};
const Segment SGI289322[] = {{&S33521[7], 1}, {NULL}};
const Segment SGI289323[] = {{&S33521[16], 4}, {&S33521[28], 11}, {NULL}};
const Segment SGI289324[] = {{&S33651[69], 5}, {NULL}};
const Segment SGI289325[] = {{&S33651[8], 3}, {NULL}};
const Segment SGI289326[] = {{&S33651[13], 9}, {NULL}};
const Segment SGI289327[] = {{&S33651[30], 13}, {NULL}};
const Segment SGI289328[] = {{&S33651[7], 1}, {NULL}};
const Segment SGI289329[] = {{&S33652[69], 5}, {NULL}};
const Segment SGI289330[] = {{&S33652[8], 3}, {NULL}};
const Segment SGI289331[] = {{&S33652[13], 9}, {NULL}};
const Segment SGI289332[] = {{&S33652[30], 13}, {NULL}};
const Segment SGI289333[] = {{&S33652[7], 1}, {NULL}};
const Segment SGI289334[] = {{&S33907[52], 4}, {NULL}};
const Segment SGI289335[] = {{&S33907[8], 3}, {NULL}};
const Segment SGI289336[] = {{&S33907[13], 14}, {NULL}};
const Segment SGI289337[] = {{&S33907[7], 1}, {NULL}};
const Segment SGI289338[] = {{&S33907[16], 11}, {NULL}};
const Segment SGI289339[] = {{&S33907[57], 11}, {NULL}};
const Segment SGI289340[] = {{&S33944[53], 5}, {NULL}};
const Segment SGI289341[] = {{&S33944[8], 3}, {NULL}};
const Segment SGI289342[] = {{&S33944[13], 14}, {NULL}};
const Segment SGI289343[] = {{&S33944[7], 1}, {NULL}};
const Segment SGI289344[] = {{&S33944[16], 11}, {NULL}};
const Segment SGI289345[] = {{&S33945[65], 5}, {NULL}};
const Segment SGI289346[] = {{&S33945[8], 3}, {NULL}};
const Segment SGI289347[] = {{&S33945[13], 14}, {NULL}};
const Segment SGI289348[] = {{&S33945[7], 1}, {NULL}};
const Segment SGI289349[] = {{&S33945[16], 11}, {&S33945[32], 2}, {NULL}};
const Segment SGI289350[] = {{&S33960[44], 5}, {NULL}};
const Segment SGI289351[] = {{&S33960[8], 3}, {NULL}};
const Segment SGI289352[] = {{&S33960[13], 3}, {NULL}};
const Segment SGI289353[] = {{&S33960[7], 1}, {NULL}};
const Segment SGI289354[] = {{&S33961[56], 5}, {NULL}};
const Segment SGI289355[] = {{&S33961[8], 3}, {NULL}};
const Segment SGI289356[] = {{&S33961[13], 3}, {NULL}};
const Segment SGI289357[] = {{&S33961[7], 1}, {NULL}};
const Segment SGI289358[] = {{&S34019[64], 5}, {NULL}};
const Segment SGI289359[] = {{&S34019[8], 3}, {NULL}};
const Segment SGI289360[] = {{&S34019[13], 3}, {NULL}};
const Segment SGI289361[] = {{&S34019[13], 3}, {&S34019[18], 6}, {NULL}};
const Segment SGI289362[] = {{&S34019[7], 1}, {NULL}};
const Segment SGI289363[] = {{&S34043[69], 5}, {NULL}};
const Segment SGI289364[] = {{&S34043[8], 3}, {NULL}};
const Segment SGI289365[] = {{&S34043[13], 9}, {NULL}};
const Segment SGI289366[] = {{&S34043[30], 13}, {NULL}};
const Segment SGI289367[] = {{&S34043[7], 1}, {NULL}};
const Segment SGI289368[] = {{&S34467[65], 5}, {NULL}};
const Segment SGI289369[] = {{&S34467[8], 3}, {NULL}};
const Segment SGI289370[] = {{&S34467[13], 14}, {NULL}};
const Segment SGI289371[] = {{&S34467[7], 1}, {NULL}};
const Segment SGI289372[] = {{&S34467[16], 11}, {&S34467[32], 2}, {NULL}};
const Segment SGI289373[] = {{&S34468[60], 5}, {NULL}};
const Segment SGI289374[] = {{&S34468[8], 3}, {NULL}};
const Segment SGI289375[] = {{&S34468[13], 14}, {NULL}};
const Segment SGI289376[] = {{&S34468[7], 1}, {NULL}};
const Segment SGI289377[] = {{&S34468[16], 11}, {NULL}};
const Segment SGI289378[] = {{&S34469[65], 5}, {NULL}};
const Segment SGI289379[] = {{&S34469[8], 3}, {NULL}};
const Segment SGI289380[] = {{&S34469[13], 14}, {NULL}};
const Segment SGI289381[] = {{&S34469[7], 1}, {NULL}};
const Segment SGI289382[] = {{&S34469[16], 11}, {&S34469[32], 2}, {NULL}};
const Segment SGI289383[] = {{&S34469[71], 13}, {NULL}};
const Segment SGI289384[] = {{&S34470[60], 5}, {NULL}};
const Segment SGI289385[] = {{&S34470[8], 3}, {NULL}};
const Segment SGI289386[] = {{&S34470[13], 14}, {NULL}};
const Segment SGI289387[] = {{&S34470[7], 1}, {NULL}};
const Segment SGI289388[] = {{&S34470[16], 11}, {NULL}};
const Segment SGI289389[] = {{&S34470[66], 13}, {NULL}};
const Segment SGI289390[] = {{&S34510[60], 5}, {NULL}};
const Segment SGI289391[] = {{&S34510[8], 3}, {NULL}};
const Segment SGI289392[] = {{&S34510[13], 14}, {NULL}};
const Segment SGI289393[] = {{&S34510[7], 1}, {NULL}};
const Segment SGI289394[] = {{&S34510[16], 11}, {NULL}};
const Segment SGI289395[] = {{&S34511[65], 5}, {NULL}};
const Segment SGI289396[] = {{&S34511[8], 3}, {NULL}};
const Segment SGI289397[] = {{&S34511[13], 14}, {NULL}};
const Segment SGI289398[] = {{&S34511[7], 1}, {NULL}};
const Segment SGI289399[] = {{&S34511[16], 11}, {&S34511[32], 2}, {NULL}};
const Segment SGI289400[] = {{&S34512[65], 5}, {NULL}};
const Segment SGI289401[] = {{&S34512[8], 3}, {NULL}};
const Segment SGI289402[] = {{&S34512[13], 14}, {NULL}};
const Segment SGI289403[] = {{&S34512[7], 1}, {NULL}};
const Segment SGI289404[] = {{&S34512[16], 11}, {&S34512[32], 2}, {NULL}};
const Segment SGI289405[] = {{&S34512[71], 13}, {NULL}};
const Segment SGI289406[] = {{&S34513[60], 5}, {NULL}};
const Segment SGI289407[] = {{&S34513[8], 3}, {NULL}};
const Segment SGI289408[] = {{&S34513[13], 14}, {NULL}};
const Segment SGI289409[] = {{&S34513[7], 1}, {NULL}};
const Segment SGI289410[] = {{&S34513[16], 11}, {NULL}};
const Segment SGI289411[] = {{&S34513[66], 13}, {NULL}};
const Segment SGI289412[] = {{&S34704[65], 5}, {NULL}};
const Segment SGI289413[] = {{&S34704[8], 3}, {NULL}};
const Segment SGI289414[] = {{&S34704[13], 14}, {NULL}};
const Segment SGI289415[] = {{&S34704[7], 1}, {NULL}};
const Segment SGI289416[] = {{&S34704[16], 11}, {&S34704[32], 2}, {NULL}};
const Segment SGI289417[] = {{&S34705[53], 5}, {NULL}};
const Segment SGI289418[] = {{&S34705[8], 3}, {NULL}};
const Segment SGI289419[] = {{&S34705[13], 14}, {NULL}};
const Segment SGI289420[] = {{&S34705[7], 1}, {NULL}};
const Segment SGI289421[] = {{&S34705[16], 11}, {NULL}};
const Segment SGI289422[] = {{&S34706[60], 5}, {NULL}};
const Segment SGI289423[] = {{&S34706[8], 3}, {NULL}};
const Segment SGI289424[] = {{&S34706[13], 14}, {NULL}};
const Segment SGI289425[] = {{&S34706[7], 1}, {NULL}};
const Segment SGI289426[] = {{&S34706[16], 11}, {NULL}};
const Segment SGI289427[] = {{&S34706[66], 14}, {NULL}};
const Segment SGI289428[] = {{&S34802[69], 5}, {NULL}};
const Segment SGI289429[] = {{&S34802[8], 3}, {NULL}};
const Segment SGI289430[] = {{&S34802[13], 9}, {NULL}};
const Segment SGI289431[] = {{&S34802[30], 13}, {NULL}};
const Segment SGI289432[] = {{&S34802[7], 1}, {NULL}};
const Segment SGI289433[] = {{&S34803[69], 5}, {NULL}};
const Segment SGI289434[] = {{&S34803[8], 3}, {NULL}};
const Segment SGI289435[] = {{&S34803[13], 9}, {NULL}};
const Segment SGI289436[] = {{&S34803[30], 13}, {NULL}};
const Segment SGI289437[] = {{&S34803[7], 1}, {NULL}};
const Segment SGI289438[] = {{&S34814[69], 5}, {NULL}};
const Segment SGI289439[] = {{&S34814[8], 3}, {NULL}};
const Segment SGI289440[] = {{&S34814[13], 9}, {NULL}};
const Segment SGI289441[] = {{&S34814[30], 13}, {NULL}};
const Segment SGI289442[] = {{&S34814[7], 1}, {NULL}};
const Segment SGI289443[] = {{&S34986[69], 5}, {NULL}};
const Segment SGI289444[] = {{&S34986[8], 3}, {NULL}};
const Segment SGI289445[] = {{&S34986[13], 9}, {NULL}};
const Segment SGI289446[] = {{&S34986[30], 13}, {NULL}};
const Segment SGI289447[] = {{&S34986[7], 1}, {NULL}};
const Segment SGI289448[] = {{&S34987[73], 5}, {NULL}};
const Segment SGI289449[] = {{&S34987[34], 13}, {NULL}};
const Segment SGI289450[] = {{&S34987[11], 1}, {NULL}};
const Segment SGI289451[] = {{&S35012[69], 5}, {NULL}};
const Segment SGI289452[] = {{&S35012[8], 3}, {NULL}};
const Segment SGI289453[] = {{&S35012[13], 9}, {NULL}};
const Segment SGI289454[] = {{&S35012[30], 13}, {NULL}};
const Segment SGI289455[] = {{&S35012[7], 1}, {NULL}};
const Segment SGI289456[] = {{&S35176[67], 4}, {NULL}};
const Segment SGI289457[] = {{&S35176[8], 3}, {NULL}};
const Segment SGI289458[] = {{&S35176[13], 3}, {NULL}};
const Segment SGI289459[] = {{&S35176[77], 10}, {NULL}};
const Segment SGI289460[] = {{&S35176[7], 1}, {NULL}};
const Segment SGI289461[] = {{&S35177[64], 4}, {NULL}};
const Segment SGI289462[] = {{&S35177[8], 3}, {NULL}};
const Segment SGI289463[] = {{&S35177[13], 3}, {NULL}};
const Segment SGI289464[] = {{&S35177[7], 1}, {NULL}};
const Segment SGI289465[] = {{&S35362[69], 5}, {NULL}};
const Segment SGI289466[] = {{&S35362[8], 3}, {NULL}};
const Segment SGI289467[] = {{&S35362[13], 9}, {NULL}};
const Segment SGI289468[] = {{&S35362[30], 13}, {NULL}};
const Segment SGI289469[] = {{&S35362[7], 1}, {NULL}};
const Segment SGI289470[] = {{&S35404[69], 5}, {NULL}};
const Segment SGI289471[] = {{&S35404[8], 3}, {NULL}};
const Segment SGI289472[] = {{&S35404[13], 9}, {NULL}};
const Segment SGI289473[] = {{&S35404[30], 13}, {NULL}};
const Segment SGI289474[] = {{&S35404[7], 1}, {NULL}};
const Segment SGI289475[] = {{&S35496[52], 4}, {NULL}};
const Segment SGI289476[] = {{&S35496[8], 3}, {NULL}};
const Segment SGI289477[] = {{&S35496[13], 14}, {NULL}};
const Segment SGI289478[] = {{&S35496[7], 1}, {NULL}};
const Segment SGI289479[] = {{&S35496[16], 11}, {NULL}};
const Segment SGI289480[] = {{&S35496[57], 11}, {NULL}};
const Segment SGI289481[] = {{&S35518[69], 5}, {NULL}};
const Segment SGI289482[] = {{&S35518[8], 3}, {NULL}};
const Segment SGI289483[] = {{&S35518[13], 9}, {NULL}};
const Segment SGI289484[] = {{&S35518[30], 13}, {NULL}};
const Segment SGI289485[] = {{&S35518[7], 1}, {NULL}};
const Segment SGI289486[] = {{&S35631[52], 4}, {NULL}};
const Segment SGI289487[] = {{&S35631[8], 3}, {NULL}};
const Segment SGI289488[] = {{&S35631[13], 14}, {NULL}};
const Segment SGI289489[] = {{&S35631[7], 1}, {NULL}};
const Segment SGI289490[] = {{&S35631[16], 11}, {NULL}};
const Segment SGI289491[] = {{&S35631[57], 11}, {NULL}};
const Segment SGI289492[] = {{&S36203[69], 5}, {NULL}};
const Segment SGI289493[] = {{&S36203[8], 3}, {NULL}};
const Segment SGI289494[] = {{&S36203[13], 9}, {NULL}};
const Segment SGI289495[] = {{&S36203[30], 13}, {NULL}};
const Segment SGI289496[] = {{&S36203[7], 1}, {NULL}};
const Segment SGI289497[] = {{&S36204[69], 5}, {&S36204[69], 5}, {NULL}};
const Segment SGI289498[] = {{&S36204[8], 3}, {NULL}};
const Segment SGI289499[] = {{&S36204[13], 9}, {NULL}};
const Segment SGI289500[] = {{&S36204[30], 13}, {NULL}};
const Segment SGI289501[] = {{&S36204[7], 1}, {&S36204[7], 1}, {NULL}};
const Segment SGI289502[] = {{&S36205[52], 5}, {NULL}};
const Segment SGI289503[] = {{&S36205[8], 3}, {NULL}};
const Segment SGI289504[] = {{&S36205[13], 14}, {NULL}};
const Segment SGI289505[] = {{&S36205[7], 1}, {NULL}};
const Segment SGI289506[] = {{&S36205[16], 11}, {NULL}};
const Segment SGI289507[] = {{&S36206[52], 8}, {NULL}};
const Segment SGI289508[] = {{&S36206[8], 3}, {NULL}};
const Segment SGI289509[] = {{&S36206[13], 14}, {NULL}};
const Segment SGI289510[] = {{&S36206[7], 1}, {NULL}};
const Segment SGI289511[] = {{&S36206[16], 11}, {NULL}};
const Segment SGI289512[] = {{&S36207[54], 4}, {NULL}};
const Segment SGI289513[] = {{&S36207[13], 14}, {NULL}};
const Segment SGI289514[] = {{&S36207[7], 1}, {NULL}};
const Segment SGI289515[] = {{&S36207[16], 11}, {NULL}};
const Segment SGI289516[] = {{&S36208[8], 4}, {NULL}};
const Segment SGI289517[] = {{&S36208[9], 3}, {NULL}};
const Segment SGI289518[] = {{&S36208[14], 14}, {NULL}};
const Segment SGI289519[] = {{&S36208[8], 1}, {NULL}};
const Segment SGI289520[] = {{&S36208[17], 11}, {NULL}};
const Segment SGI289521[] = {{&S36209[7], 4}, {NULL}};
const Segment SGI289522[] = {{&S36209[8], 3}, {NULL}};
const Segment SGI289523[] = {{&S36209[13], 7}, {NULL}};
const Segment SGI289524[] = {{&S36209[25], 14}, {NULL}};
const Segment SGI289525[] = {{&S36209[7], 1}, {NULL}};
const Segment SGI289526[] = {{&S36209[16], 4}, {&S36209[28], 11}, {NULL}};
const Segment SGI289527[] = {{&S36596[53], 5}, {&S36596[53], 5}, {NULL}};
const Segment SGI289528[] = {{&S36596[8], 3}, {NULL}};
const Segment SGI289529[] = {{&S36596[13], 14}, {NULL}};
const Segment SGI289530[] = {{&S36596[7], 1}, {&S36596[7], 1}, {NULL}};
const Segment SGI289531[] = {{&S36596[16], 11}, {NULL}};
const Segment SGI289532[] = {{&S36597[66], 5}, {NULL}};
const Segment SGI289533[] = {{&S36597[8], 3}, {&S36597[8], 3}, {NULL}};
const Segment SGI289534[] = {{&S36597[13], 27}, {NULL}};
const Segment SGI289535[] = {{&S36597[26], 14}, {NULL}};
const Segment SGI289536[] = {{&S36597[7], 1}, {NULL}};
const Segment SGI289537[] = {{&S36597[29], 11}, {NULL}};
const Segment SGI289538[] = {{&S36598[51], 5}, {NULL}};
const Segment SGI289539[] = {{&S36598[8], 3}, {NULL}};
const Segment SGI289540[] = {{&S36598[13], 14}, {NULL}};
const Segment SGI289541[] = {{&S36598[7], 1}, {NULL}};
const Segment SGI289542[] = {{&S36598[16], 11}, {NULL}};
const Segment SGI289543[] = {{&S36599[65], 5}, {NULL}};
const Segment SGI289544[] = {{&S36599[8], 3}, {NULL}};
const Segment SGI289545[] = {{&S36599[13], 7}, {NULL}};
const Segment SGI289546[] = {{&S36599[25], 14}, {NULL}};
const Segment SGI289547[] = {{&S36599[7], 1}, {NULL}};
const Segment SGI289548[] = {{&S36599[16], 4}, {&S36599[28], 11}, {NULL}};
const Segment SGI289549[] = {{&S36600[73], 5}, {NULL}};
const Segment SGI289550[] = {{&S36600[8], 3}, {NULL}};
const Segment SGI289551[] = {{&S36600[13], 7}, {NULL}};
const Segment SGI289552[] = {{&S36600[25], 14}, {NULL}};
const Segment SGI289553[] = {{&S36600[7], 1}, {NULL}};
const Segment SGI289554[] = {{&S36600[16], 4}, {&S36600[28], 11}, {NULL}};
const Segment SGI289555[] = {{&S36601[43], 5}, {NULL}};
const Segment SGI289556[] = {{&S36601[8], 3}, {NULL}};
const Segment SGI289557[] = {{&S36601[13], 14}, {NULL}};
const Segment SGI289558[] = {{&S36601[7], 1}, {NULL}};
const Segment SGI289559[] = {{&S36601[16], 11}, {NULL}};
const Segment SGI289560[] = {{&S36609[54], 5}, {NULL}};
const Segment SGI289561[] = {{&S36609[8], 3}, {NULL}};
const Segment SGI289562[] = {{&S36609[13], 3}, {NULL}};
const Segment SGI289563[] = {{&S36609[7], 1}, {NULL}};
const Segment SGI289564[] = {{&S36610[74], 5}, {NULL}};
const Segment SGI289565[] = {{&S36610[8], 3}, {NULL}};
const Segment SGI289566[] = {{&S36610[13], 3}, {NULL}};
const Segment SGI289567[] = {{&S36610[7], 1}, {NULL}};
const Segment SGI289568[] = {{&S36611[54], 5}, {&S36611[54], 5}, {NULL}};
const Segment SGI289569[] = {{&S36611[8], 3}, {NULL}};
const Segment SGI289570[] = {{&S36611[13], 3}, {NULL}};
const Segment SGI289571[] = {{&S36611[7], 1}, {&S36611[7], 1}, {NULL}};
const Segment SGI289572[] = {{&S36665[64], 5}, {NULL}};
const Segment SGI289573[] = {{&S36665[8], 3}, {NULL}};
const Segment SGI289574[] = {{&S36665[13], 3}, {NULL}};
const Segment SGI289575[] = {{&S36665[7], 1}, {NULL}};
const Segment SGI289576[] = {{&S36666[56], 5}, {NULL}};
const Segment SGI289577[] = {{&S36666[8], 3}, {NULL}};
const Segment SGI289578[] = {{&S36666[13], 3}, {NULL}};
const Segment SGI289579[] = {{&S36666[7], 1}, {NULL}};
const Segment SGI289580[] = {{&S36694[54], 4}, {&S36694[54], 4}, {NULL}};
const Segment SGI289581[] = {{&S36694[8], 3}, {NULL}};
const Segment SGI289582[] = {{&S36694[13], 14}, {NULL}};
const Segment SGI289583[] = {{&S36694[7], 1}, {&S36694[7], 1}, {NULL}};
const Segment SGI289584[] = {{&S36694[16], 11}, {NULL}};
const Segment SGI289585[] = {{&S36695[52], 4}, {NULL}};
const Segment SGI289586[] = {{&S36695[8], 3}, {NULL}};
const Segment SGI289587[] = {{&S36695[13], 14}, {NULL}};
const Segment SGI289588[] = {{&S36695[7], 1}, {NULL}};
const Segment SGI289589[] = {{&S36695[16], 11}, {NULL}};
const Segment SGI289590[] = {{&S36696[63], 4}, {NULL}};
const Segment SGI289591[] = {{&S36696[8], 3}, {NULL}};
const Segment SGI289592[] = {{&S36696[13], 7}, {NULL}};
const Segment SGI289593[] = {{&S36696[22], 14}, {NULL}};
const Segment SGI289594[] = {{&S36696[7], 1}, {NULL}};
const Segment SGI289595[] = {{&S36696[16], 4}, {&S36696[25], 11}, {NULL}};
const Segment SGI289596[] = {{&S36697[44], 4}, {NULL}};
const Segment SGI289597[] = {{&S36697[8], 3}, {NULL}};
const Segment SGI289598[] = {{&S36697[13], 14}, {NULL}};
const Segment SGI289599[] = {{&S36697[7], 1}, {NULL}};
const Segment SGI289600[] = {{&S36697[16], 34}, {NULL}};
const Segment SGI289601[] = {{&S36698[54], 4}, {NULL}};
const Segment SGI289602[] = {{&S36698[8], 3}, {NULL}};
const Segment SGI289603[] = {{&S36698[13], 14}, {NULL}};
const Segment SGI289604[] = {{&S36698[7], 1}, {NULL}};
const Segment SGI289605[] = {{&S36698[16], 11}, {NULL}};
const Segment SGI289606[] = {{&S36702[55], 4}, {NULL}};
const Segment SGI289607[] = {{&S36702[8], 3}, {NULL}};
const Segment SGI289608[] = {{&S36702[13], 3}, {NULL}};
const Segment SGI289609[] = {{&S36702[7], 1}, {NULL}};
const Segment SGI289610[] = {{&S36702[60], 11}, {NULL}};
const Segment SGI289611[] = {{&S36801[52], 4}, {NULL}};
const Segment SGI289612[] = {{&S36801[8], 3}, {NULL}};
const Segment SGI289613[] = {{&S36801[13], 14}, {NULL}};
const Segment SGI289614[] = {{&S36801[7], 1}, {NULL}};
const Segment SGI289615[] = {{&S36801[16], 11}, {NULL}};
const Segment SGI289616[] = {{&S36801[57], 11}, {NULL}};
const Segment SGI289617[] = {{&S36942[65], 5}, {NULL}};
const Segment SGI289618[] = {{&S36942[8], 3}, {NULL}};
const Segment SGI289619[] = {{&S36942[13], 14}, {NULL}};
const Segment SGI289620[] = {{&S36942[7], 1}, {NULL}};
const Segment SGI289621[] = {{&S36942[16], 11}, {&S36942[32], 2}, {NULL}};
const Segment SGI289622[] = {{&S36943[53], 5}, {NULL}};
const Segment SGI289623[] = {{&S36943[8], 3}, {NULL}};
const Segment SGI289624[] = {{&S36943[13], 14}, {NULL}};
const Segment SGI289625[] = {{&S36943[7], 1}, {NULL}};
const Segment SGI289626[] = {{&S36943[16], 11}, {NULL}};
const Segment SGI289627[] = {{&S36944[53], 5}, {NULL}};
const Segment SGI289628[] = {{&S36944[8], 3}, {NULL}};
const Segment SGI289629[] = {{&S36944[13], 14}, {NULL}};
const Segment SGI289630[] = {{&S36944[7], 1}, {NULL}};
const Segment SGI289631[] = {{&S36944[16], 11}, {NULL}};
const Segment SGI289632[] = {{&S36944[59], 13}, {NULL}};
const Segment SGI289633[] = {{&S36945[65], 5}, {NULL}};
const Segment SGI289634[] = {{&S36945[8], 3}, {NULL}};
const Segment SGI289635[] = {{&S36945[13], 14}, {NULL}};
const Segment SGI289636[] = {{&S36945[7], 1}, {NULL}};
const Segment SGI289637[] = {{&S36945[16], 11}, {&S36945[32], 2}, {NULL}};
const Segment SGI289638[] = {{&S36945[71], 13}, {NULL}};
const Segment SGI289639[] = {{&S37036[66], 4}, {NULL}};
const Segment SGI289640[] = {{&S37036[8], 3}, {NULL}};
const Segment SGI289641[] = {{&S37036[13], 3}, {NULL}};
const Segment SGI289642[] = {{&S37036[60], 10}, {NULL}};
const Segment SGI289643[] = {{&S37036[7], 1}, {NULL}};
const Segment SGI289644[] = {{&S37036[60], 7}, {NULL}};
const Segment SGI289645[] = {{&S37170[69], 5}, {NULL}};
const Segment SGI289646[] = {{&S37170[8], 3}, {NULL}};
const Segment SGI289647[] = {{&S37170[13], 9}, {NULL}};
const Segment SGI289648[] = {{&S37170[30], 13}, {NULL}};
const Segment SGI289649[] = {{&S37170[7], 1}, {NULL}};
const Segment SGI289650[] = {{&S37171[69], 5}, {&S37171[69], 5}, {NULL}};
const Segment SGI289651[] = {{&S37171[8], 3}, {NULL}};
const Segment SGI289652[] = {{&S37171[13], 9}, {NULL}};
const Segment SGI289653[] = {{&S37171[30], 13}, {NULL}};
const Segment SGI289654[] = {{&S37171[7], 1}, {&S37171[7], 1}, {NULL}};
const Segment SGI289655[] = {{&S37172[67], 5}, {NULL}};
const Segment SGI289656[] = {{&S37172[8], 3}, {NULL}};
const Segment SGI289657[] = {{&S37172[13], 14}, {NULL}};
const Segment SGI289658[] = {{&S37172[7], 1}, {NULL}};
const Segment SGI289659[] = {{&S37172[16], 11}, {&S37172[32], 2}, {NULL}};
const Segment SGI289660[] = {{&S37173[55], 5}, {&S37173[55], 5}, {NULL}};
const Segment SGI289661[] = {{&S37173[8], 3}, {NULL}};
const Segment SGI289662[] = {{&S37173[13], 14}, {NULL}};
const Segment SGI289663[] = {{&S37173[7], 1}, {&S37173[7], 1}, {NULL}};
const Segment SGI289664[] = {{&S37173[16], 11}, {NULL}};
const Segment SGI289665[] = {{&S37174[74], 5}, {NULL}};
const Segment SGI289666[] = {{&S37174[8], 3}, {NULL}};
const Segment SGI289667[] = {{&S37174[13], 7}, {NULL}};
const Segment SGI289668[] = {{&S37174[25], 14}, {NULL}};
const Segment SGI289669[] = {{&S37174[7], 1}, {NULL}};
const Segment SGI289670[] = {{&S37174[16], 4}, {&S37174[28], 11}, {NULL}};
const Segment SGI289671[] = {{&S37175[56], 5}, {NULL}};
const Segment SGI289672[] = {{&S37175[8], 3}, {NULL}};
const Segment SGI289673[] = {{&S37175[13], 14}, {NULL}};
const Segment SGI289674[] = {{&S37175[7], 1}, {NULL}};
const Segment SGI289675[] = {{&S37175[16], 11}, {NULL}};
const Segment SGI289676[] = {{&S37176[55], 5}, {NULL}};
const Segment SGI289677[] = {{&S37176[8], 3}, {NULL}};
const Segment SGI289678[] = {{&S37176[13], 14}, {NULL}};
const Segment SGI289679[] = {{&S37176[7], 1}, {NULL}};
const Segment SGI289680[] = {{&S37176[16], 11}, {NULL}};
const Segment SGI289681[] = {{&S37193[52], 4}, {NULL}};
const Segment SGI289682[] = {{&S37193[8], 3}, {NULL}};
const Segment SGI289683[] = {{&S37193[13], 14}, {NULL}};
const Segment SGI289684[] = {{&S37193[7], 1}, {NULL}};
const Segment SGI289685[] = {{&S37193[16], 11}, {NULL}};
const Segment SGI289686[] = {{&S37193[57], 11}, {NULL}};
const Segment SGI289687[] = {{&S37262[71], 5}, {&S37262[71], 5}, {NULL}};
const Segment SGI289688[] = {{&S37262[8], 3}, {NULL}};
const Segment SGI289689[] = {{&S37262[13], 9}, {NULL}};
const Segment SGI289690[] = {{&S37262[30], 13}, {NULL}};
const Segment SGI289691[] = {{&S37262[7], 1}, {&S37262[7], 1}, {NULL}};
const Segment SGI289692[] = {{&S37263[71], 5}, {NULL}};
const Segment SGI289693[] = {{&S37263[8], 3}, {NULL}};
const Segment SGI289694[] = {{&S37263[13], 9}, {NULL}};
const Segment SGI289695[] = {{&S37263[30], 13}, {NULL}};
const Segment SGI289696[] = {{&S37263[7], 1}, {NULL}};
const Segment SGI289697[] = {{&S37264[73], 5}, {NULL}};
const Segment SGI289698[] = {{&S37264[8], 3}, {NULL}};
const Segment SGI289699[] = {{&S37264[13], 9}, {NULL}};
const Segment SGI289700[] = {{&S37264[30], 15}, {NULL}};
const Segment SGI289701[] = {{&S37264[7], 1}, {NULL}};
const Segment SGI289702[] = {{&S37300[57], 4}, {NULL}};
const Segment SGI289703[] = {{&S37300[8], 3}, {NULL}};
const Segment SGI289704[] = {{&S37300[13], 3}, {NULL}};
const Segment SGI289705[] = {{&S37300[7], 1}, {NULL}};
const Segment SGI289706[] = {{&S37300[62], 11}, {NULL}};
const Segment SGI289707[] = {{&S37302[73], 5}, {NULL}};
const Segment SGI289708[] = {{&S37302[8], 3}, {NULL}};
const Segment SGI289709[] = {{&S37302[13], 7}, {NULL}};
const Segment SGI289710[] = {{&S37302[25], 14}, {NULL}};
const Segment SGI289711[] = {{&S37302[7], 1}, {NULL}};
const Segment SGI289712[] = {{&S37302[16], 4}, {&S37302[28], 11}, {NULL}};
const Segment SGI289713[] = {{&S37303[70], 4}, {NULL}};
const Segment SGI289714[] = {{&S37303[8], 3}, {NULL}};
const Segment SGI289715[] = {{&S37303[13], 7}, {NULL}};
const Segment SGI289716[] = {{&S37303[7], 1}, {NULL}};
const Segment SGI289717[] = {{&S37303[16], 4}, {&S37303[28], 10}, {NULL}};
const Segment SGI289718[] = {{&S37304[63], 4}, {NULL}};
const Segment SGI289719[] = {{&S37304[8], 3}, {NULL}};
const Segment SGI289720[] = {{&S37304[13], 7}, {NULL}};
const Segment SGI289721[] = {{&S37304[25], 14}, {NULL}};
const Segment SGI289722[] = {{&S37304[7], 1}, {NULL}};
const Segment SGI289723[] = {{&S37304[16], 4}, {&S37304[28], 11}, {NULL}};
const Segment SGI289724[] = {{&S37305[71], 4}, {NULL}};
const Segment SGI289725[] = {{&S37305[8], 3}, {NULL}};
const Segment SGI289726[] = {{&S37305[13], 7}, {NULL}};
const Segment SGI289727[] = {{&S37305[25], 14}, {NULL}};
const Segment SGI289728[] = {{&S37305[7], 1}, {NULL}};
const Segment SGI289729[] = {{&S37305[16], 4}, {&S37305[28], 11}, {NULL}};
const Segment SGI289730[] = {{&S37321[60], 5}, {NULL}};
const Segment SGI289731[] = {{&S37321[8], 3}, {NULL}};
const Segment SGI289732[] = {{&S37321[13], 14}, {NULL}};
const Segment SGI289733[] = {{&S37321[7], 1}, {NULL}};
const Segment SGI289734[] = {{&S37321[16], 11}, {NULL}};
const Segment SGI289735[] = {{&S37322[53], 5}, {NULL}};
const Segment SGI289736[] = {{&S37322[8], 3}, {NULL}};
const Segment SGI289737[] = {{&S37322[13], 14}, {NULL}};
const Segment SGI289738[] = {{&S37322[7], 1}, {NULL}};
const Segment SGI289739[] = {{&S37322[16], 11}, {NULL}};
const Segment SGI289740[] = {{&S37323[53], 5}, {NULL}};
const Segment SGI289741[] = {{&S37323[8], 3}, {NULL}};
const Segment SGI289742[] = {{&S37323[13], 14}, {NULL}};
const Segment SGI289743[] = {{&S37323[7], 1}, {NULL}};
const Segment SGI289744[] = {{&S37323[16], 11}, {NULL}};
const Segment SGI289745[] = {{&S37323[59], 13}, {NULL}};
const Segment SGI289746[] = {{&S37332[55], 5}, {NULL}};
const Segment SGI289747[] = {{&S37332[8], 3}, {NULL}};
const Segment SGI289748[] = {{&S37332[13], 14}, {NULL}};
const Segment SGI289749[] = {{&S37332[7], 1}, {NULL}};
const Segment SGI289750[] = {{&S37332[16], 11}, {&S37332[32], 2}, {NULL}};
const Segment SGI289751[] = {{&S37333[53], 5}, {NULL}};
const Segment SGI289752[] = {{&S37333[8], 3}, {NULL}};
const Segment SGI289753[] = {{&S37333[13], 14}, {NULL}};
const Segment SGI289754[] = {{&S37333[7], 1}, {NULL}};
const Segment SGI289755[] = {{&S37333[16], 11}, {NULL}};
const Segment SGI289756[] = {{&S37334[72], 5}, {NULL}};
const Segment SGI289757[] = {{&S37334[8], 3}, {NULL}};
const Segment SGI289758[] = {{&S37334[13], 7}, {NULL}};
const Segment SGI289759[] = {{&S37334[25], 14}, {NULL}};
const Segment SGI289760[] = {{&S37334[7], 1}, {NULL}};
const Segment SGI289761[] = {{&S37334[16], 4}, {&S37334[28], 11}, {NULL}};
const Segment SGI289762[] = {{&S37388[71], 5}, {NULL}};
const Segment SGI289763[] = {{&S37388[8], 3}, {NULL}};
const Segment SGI289764[] = {{&S37388[13], 9}, {NULL}};
const Segment SGI289765[] = {{&S37388[30], 13}, {NULL}};
const Segment SGI289766[] = {{&S37388[7], 1}, {NULL}};
const Segment SGI289767[] = {{&S37389[71], 5}, {&S37389[71], 5}, {NULL}};
const Segment SGI289768[] = {{&S37389[8], 3}, {NULL}};
const Segment SGI289769[] = {{&S37389[13], 9}, {NULL}};
const Segment SGI289770[] = {{&S37389[30], 13}, {NULL}};
const Segment SGI289771[] = {{&S37389[7], 1}, {&S37389[7], 1}, {NULL}};
const Segment SGI289772[] = {{&S37390[73], 5}, {NULL}};
const Segment SGI289773[] = {{&S37390[8], 3}, {NULL}};
const Segment SGI289774[] = {{&S37390[13], 9}, {NULL}};
const Segment SGI289775[] = {{&S37390[30], 15}, {NULL}};
const Segment SGI289776[] = {{&S37390[7], 1}, {NULL}};
const Segment SGI289777[] = {{&S37713[74], 4}, {NULL}};
const Segment SGI289778[] = {{&S37713[8], 3}, {NULL}};
const Segment SGI289779[] = {{&S37713[13], 9}, {NULL}};
const Segment SGI289780[] = {{&S37713[30], 13}, {NULL}};
const Segment SGI289781[] = {{&S37713[7], 1}, {NULL}};
const Segment SGI289782[] = {{&S37714[74], 4}, {NULL}};
const Segment SGI289783[] = {{&S37714[8], 3}, {NULL}};
const Segment SGI289784[] = {{&S37714[13], 9}, {NULL}};
const Segment SGI289785[] = {{&S37714[30], 15}, {NULL}};
const Segment SGI289786[] = {{&S37714[7], 1}, {NULL}};
const Segment SGI289787[] = {{&S37722[8], 3}, {NULL}};
const Segment SGI289788[] = {{&S37722[13], 7}, {NULL}};
const Segment SGI289789[] = {{&S37722[25], 14}, {NULL}};
const Segment SGI289790[] = {{&S37722[16], 4}, {&S37722[28], 11}, {NULL}};
const Segment SGI289791[] = {{&S37725[8], 3}, {NULL}};
const Segment SGI289792[] = {{&S37725[13], 7}, {NULL}};
const Segment SGI289793[] = {{&S37725[25], 14}, {NULL}};
const Segment SGI289794[] = {{&S37725[100], 5}, {NULL}};
const Segment SGI289795[] = {{&S37725[16], 4}, {&S37725[28], 11}, {NULL}};
const Segment SGI289796[] = {{&S37726[8], 3}, {NULL}};
const Segment SGI289797[] = {{&S37726[13], 7}, {NULL}};
const Segment SGI289798[] = {{&S37726[25], 14}, {NULL}};
const Segment SGI289799[] = {{&S37726[77], 5}, {NULL}};
const Segment SGI289800[] = {{&S37726[16], 4}, {&S37726[28], 11}, {NULL}};
const Segment SGI289801[] = {{&S37727[8], 3}, {NULL}};
const Segment SGI289802[] = {{&S37727[13], 7}, {NULL}};
const Segment SGI289803[] = {{&S37727[25], 14}, {NULL}};
const Segment SGI289804[] = {{&S37727[77], 8}, {NULL}};
const Segment SGI289805[] = {{&S37727[16], 4}, {&S37727[28], 11}, {NULL}};
const Segment SGI289806[] = {{&S37728[8], 3}, {NULL}};
const Segment SGI289807[] = {{&S37728[13], 7}, {NULL}};
const Segment SGI289808[] = {{&S37728[25], 14}, {NULL}};
const Segment SGI289809[] = {{&S37728[77], 12}, {NULL}};
const Segment SGI289810[] = {{&S37728[16], 4}, {&S37728[28], 11}, {NULL}};
const Segment SGI289811[] = {{&S37729[8], 3}, {NULL}};
const Segment SGI289812[] = {{&S37729[13], 7}, {NULL}};
const Segment SGI289813[] = {{&S37729[25], 14}, {NULL}};
const Segment SGI289814[] = {{&S37729[100], 6}, {NULL}};
const Segment SGI289815[] = {{&S37729[16], 4}, {&S37729[28], 11}, {NULL}};
const Segment SGI289816[] = {{&S37730[8], 3}, {NULL}};
const Segment SGI289817[] = {{&S37730[13], 7}, {NULL}};
const Segment SGI289818[] = {{&S37730[25], 14}, {NULL}};
const Segment SGI289819[] = {{&S37730[77], 13}, {NULL}};
const Segment SGI289820[] = {{&S37730[16], 4}, {&S37730[28], 11}, {NULL}};
const Segment SGI289821[] = {{&S37731[8], 3}, {NULL}};
const Segment SGI289822[] = {{&S37731[13], 7}, {NULL}};
const Segment SGI289823[] = {{&S37731[25], 14}, {NULL}};
const Segment SGI289824[] = {{&S37731[75], 2}, {NULL}};
const Segment SGI289825[] = {{&S37731[16], 4}, {&S37731[28], 11}, {NULL}};
const Segment SGI289826[] = {{&S37732[8], 3}, {NULL}};
const Segment SGI289827[] = {{&S37732[13], 7}, {NULL}};
const Segment SGI289828[] = {{&S37732[25], 14}, {NULL}};
const Segment SGI289829[] = {{&S37732[71], 1}, {NULL}};
const Segment SGI289830[] = {{&S37732[16], 4}, {&S37732[28], 11}, {NULL}};
const Segment SGI289831[] = {{&S37734[8], 3}, {NULL}};
const Segment SGI289832[] = {{&S37734[13], 14}, {NULL}};
const Segment SGI289833[] = {{&S37734[52], 12}, {NULL}};
const Segment SGI289834[] = {{&S37734[16], 11}, {NULL}};
const Segment SGI289835[] = {{&S37735[8], 3}, {NULL}};
const Segment SGI289836[] = {{&S37735[13], 7}, {NULL}};
const Segment SGI289837[] = {{&S37735[25], 14}, {NULL}};
const Segment SGI289838[] = {{&S37735[72], 53}, {NULL}};
const Segment SGI289839[] = {{&S37735[16], 4}, {&S37735[28], 11}, {NULL}};
const Segment SGI289840[] = {{&S37876[69], 5}, {NULL}};
const Segment SGI289841[] = {{&S37876[8], 3}, {NULL}};
const Segment SGI289842[] = {{&S37876[13], 9}, {NULL}};
const Segment SGI289843[] = {{&S37876[30], 13}, {NULL}};
const Segment SGI289844[] = {{&S37876[7], 1}, {NULL}};
const Segment SGI289845[] = {{&S37877[69], 5}, {&S37877[69], 5}, {NULL}};
const Segment SGI289846[] = {{&S37877[8], 3}, {NULL}};
const Segment SGI289847[] = {{&S37877[13], 9}, {NULL}};
const Segment SGI289848[] = {{&S37877[30], 13}, {NULL}};
const Segment SGI289849[] = {{&S37877[7], 1}, {&S37877[7], 1}, {NULL}};
const Segment SGI289850[] = {{&S37949[56], 4}, {NULL}};
const Segment SGI289851[] = {{&S37949[8], 3}, {NULL}};
const Segment SGI289852[] = {{&S37949[13], 14}, {NULL}};
const Segment SGI289853[] = {{&S37949[7], 1}, {NULL}};
const Segment SGI289854[] = {{&S37949[16], 11}, {NULL}};
const Segment SGI289855[] = {{&S37949[61], 11}, {NULL}};
const Segment SGI289856[] = {{&S38052[65], 5}, {NULL}};
const Segment SGI289857[] = {{&S38052[8], 3}, {NULL}};
const Segment SGI289858[] = {{&S38052[13], 14}, {NULL}};
const Segment SGI289859[] = {{&S38052[7], 1}, {NULL}};
const Segment SGI289860[] = {{&S38052[16], 11}, {&S38052[32], 2}, {NULL}};
const Segment SGI289861[] = {{&S38053[60], 5}, {NULL}};
const Segment SGI289862[] = {{&S38053[8], 3}, {NULL}};
const Segment SGI289863[] = {{&S38053[13], 14}, {NULL}};
const Segment SGI289864[] = {{&S38053[7], 1}, {NULL}};
const Segment SGI289865[] = {{&S38053[16], 11}, {NULL}};
const Segment SGI289866[] = {{&S38086[54], 4}, {NULL}};
const Segment SGI289867[] = {{&S38086[8], 3}, {NULL}};
const Segment SGI289868[] = {{&S38086[13], 14}, {NULL}};
const Segment SGI289869[] = {{&S38086[7], 1}, {NULL}};
const Segment SGI289870[] = {{&S38086[16], 11}, {NULL}};
const Segment SGI289871[] = {{&S38087[75], 4}, {NULL}};
const Segment SGI289872[] = {{&S38087[8], 3}, {NULL}};
const Segment SGI289873[] = {{&S38087[13], 7}, {NULL}};
const Segment SGI289874[] = {{&S38087[25], 14}, {NULL}};
const Segment SGI289875[] = {{&S38087[7], 1}, {NULL}};
const Segment SGI289876[] = {{&S38087[16], 4}, {&S38087[28], 11}, {NULL}};
const Segment SGI289877[] = {{&S38088[66], 4}, {NULL}};
const Segment SGI289878[] = {{&S38088[8], 3}, {NULL}};
const Segment SGI289879[] = {{&S38088[13], 14}, {NULL}};
const Segment SGI289880[] = {{&S38088[7], 1}, {NULL}};
const Segment SGI289881[] = {{&S38088[16], 11}, {&S38088[32], 2}, {NULL}};
const Segment SGI289882[] = {{&S38089[52], 4}, {&S38089[52], 4}, {NULL}};
const Segment SGI289883[] = {{&S38089[8], 3}, {NULL}};
const Segment SGI289884[] = {{&S38089[13], 14}, {NULL}};
const Segment SGI289885[] = {{&S38089[7], 1}, {&S38089[7], 1}, {NULL}};
const Segment SGI289886[] = {{&S38089[16], 11}, {NULL}};
const Segment SGI289887[] = {{&S38225[69], 5}, {NULL}};
const Segment SGI289888[] = {{&S38225[8], 3}, {NULL}};
const Segment SGI289889[] = {{&S38225[13], 9}, {NULL}};
const Segment SGI289890[] = {{&S38225[30], 13}, {NULL}};
const Segment SGI289891[] = {{&S38225[7], 1}, {NULL}};
const Segment SGI289892[] = {{&S38226[69], 5}, {&S38226[69], 5}, {NULL}};
const Segment SGI289893[] = {{&S38226[8], 3}, {NULL}};
const Segment SGI289894[] = {{&S38226[13], 9}, {NULL}};
const Segment SGI289895[] = {{&S38226[30], 13}, {NULL}};
const Segment SGI289896[] = {{&S38226[7], 1}, {&S38226[7], 1}, {NULL}};
const Segment SGI289897[] = {{&S38243[62], 5}, {NULL}};
const Segment SGI289898[] = {{&S38243[8], 3}, {NULL}};
const Segment SGI289899[] = {{&S38243[13], 3}, {NULL}};
const Segment SGI289900[] = {{&S38243[13], 3}, {&S38243[18], 6}, {NULL}};
const Segment SGI289901[] = {{&S38243[7], 1}, {NULL}};
const Segment SGI289902[] = {{&S38244[62], 5}, {&S38244[62], 5}, {NULL}};
const Segment SGI289903[] = {{&S38244[8], 3}, {NULL}};
const Segment SGI289904[] = {{&S38244[13], 3}, {NULL}};
const Segment SGI289905[] = {{&S38244[13], 3}, {&S38244[18], 6}, {NULL}};
const Segment SGI289906[] = {{&S38244[7], 1}, {&S38244[7], 1}, {NULL}};
const Segment SGI289907[] = {{&S38335[53], 5}, {NULL}};
const Segment SGI289908[] = {{&S38335[8], 3}, {NULL}};
const Segment SGI289909[] = {{&S38335[13], 14}, {NULL}};
const Segment SGI289910[] = {{&S38335[7], 1}, {NULL}};
const Segment SGI289911[] = {{&S38335[16], 11}, {NULL}};
const Segment SGI289912[] = {{&S38336[53], 5}, {&S38336[53], 5}, {NULL}};
const Segment SGI289913[] = {{&S38336[8], 3}, {NULL}};
const Segment SGI289914[] = {{&S38336[13], 14}, {NULL}};
const Segment SGI289915[] = {{&S38336[7], 1}, {&S38336[7], 1}, {NULL}};
const Segment SGI289916[] = {{&S38336[16], 11}, {NULL}};
const Segment SGI289917[] = {{&S38381[69], 5}, {NULL}};
const Segment SGI289918[] = {{&S38381[8], 3}, {NULL}};
const Segment SGI289919[] = {{&S38381[13], 9}, {NULL}};
const Segment SGI289920[] = {{&S38381[30], 13}, {NULL}};
const Segment SGI289921[] = {{&S38381[7], 1}, {NULL}};
const Segment SGI289922[] = {{&S38440[76], 4}, {NULL}};
const Segment SGI289923[] = {{&S38440[8], 3}, {NULL}};
const Segment SGI289924[] = {{&S38440[13], 7}, {NULL}};
const Segment SGI289925[] = {{&S38440[25], 14}, {NULL}};
const Segment SGI289926[] = {{&S38440[7], 1}, {NULL}};
const Segment SGI289927[] = {{&S38440[16], 4}, {&S38440[28], 11}, {NULL}};
const Segment SGI289928[] = {{&S38604[54], 4}, {NULL}};
const Segment SGI289929[] = {{&S38604[8], 3}, {NULL}};
const Segment SGI289930[] = {{&S38604[13], 14}, {NULL}};
const Segment SGI289931[] = {{&S38604[7], 1}, {NULL}};
const Segment SGI289932[] = {{&S38604[16], 11}, {NULL}};
const Segment SGI289933[] = {{&S38605[44], 4}, {NULL}};
const Segment SGI289934[] = {{&S38605[8], 3}, {NULL}};
const Segment SGI289935[] = {{&S38605[13], 14}, {NULL}};
const Segment SGI289936[] = {{&S38605[7], 1}, {NULL}};
const Segment SGI289937[] = {{&S38605[16], 34}, {NULL}};
const Segment SGI289938[] = {{&S38606[54], 4}, {NULL}};
const Segment SGI289939[] = {{&S38606[8], 3}, {NULL}};
const Segment SGI289940[] = {{&S38606[13], 14}, {NULL}};
const Segment SGI289941[] = {{&S38606[7], 1}, {NULL}};
const Segment SGI289942[] = {{&S38606[16], 11}, {NULL}};
const Segment SGI289943[] = {{&S38607[73], 4}, {NULL}};
const Segment SGI289944[] = {{&S38607[8], 3}, {NULL}};
const Segment SGI289945[] = {{&S38607[13], 7}, {NULL}};
const Segment SGI289946[] = {{&S38607[25], 14}, {NULL}};
const Segment SGI289947[] = {{&S38607[7], 1}, {NULL}};
const Segment SGI289948[] = {{&S38607[16], 4}, {&S38607[28], 11}, {NULL}};
const Segment SGI289949[] = {{&S38608[73], 4}, {NULL}};
const Segment SGI289950[] = {{&S38608[8], 3}, {NULL}};
const Segment SGI289951[] = {{&S38608[13], 7}, {NULL}};
const Segment SGI289952[] = {{&S38608[25], 14}, {NULL}};
const Segment SGI289953[] = {{&S38608[7], 1}, {NULL}};
const Segment SGI289954[] = {{&S38608[16], 4}, {&S38608[28], 11}, {NULL}};
const Segment SGI289955[] = {{&S38763[60], 5}, {NULL}};
const Segment SGI289956[] = {{&S38763[8], 3}, {NULL}};
const Segment SGI289957[] = {{&S38763[13], 14}, {NULL}};
const Segment SGI289958[] = {{&S38763[7], 1}, {NULL}};
const Segment SGI289959[] = {{&S38763[16], 11}, {NULL}};
const Segment SGI289960[] = {{&S38763[66], 14}, {NULL}};
const Segment SGI289961[] = {{&S38764[60], 5}, {NULL}};
const Segment SGI289962[] = {{&S38764[8], 3}, {NULL}};
const Segment SGI289963[] = {{&S38764[13], 14}, {NULL}};
const Segment SGI289964[] = {{&S38764[7], 1}, {NULL}};
const Segment SGI289965[] = {{&S38764[16], 11}, {NULL}};
const Segment SGI289966[] = {{&S38764[66], 17}, {NULL}};
const Segment SGI289967[] = {{&S38817[92], 4}, {NULL}};
const Segment SGI289968[] = {{&S38817[8], 3}, {NULL}};
const Segment SGI289969[] = {{&S38817[13], 3}, {NULL}};
const Segment SGI289970[] = {{&S38817[13], 3}, {&S38817[61], 6}, {NULL}};
const Segment SGI289971[] = {{&S38817[61], 12}, {NULL}};
const Segment SGI289972[] = {{&S38817[7], 1}, {NULL}};
const Segment SGI289973[] = {{&S38932[69], 5}, {NULL}};
const Segment SGI289974[] = {{&S38932[8], 3}, {NULL}};
const Segment SGI289975[] = {{&S38932[13], 9}, {NULL}};
const Segment SGI289976[] = {{&S38932[30], 13}, {NULL}};
const Segment SGI289977[] = {{&S38932[7], 1}, {NULL}};
const Segment SGI289978[] = {{&S38933[69], 5}, {&S38933[69], 5}, {NULL}};
const Segment SGI289979[] = {{&S38933[8], 3}, {NULL}};
const Segment SGI289980[] = {{&S38933[13], 9}, {NULL}};
const Segment SGI289981[] = {{&S38933[30], 13}, {NULL}};
const Segment SGI289982[] = {{&S38933[7], 1}, {&S38933[7], 1}, {NULL}};
const Segment SGI289983[] = {{&S38987[69], 5}, {NULL}};
const Segment SGI289984[] = {{&S38987[8], 3}, {NULL}};
const Segment SGI289985[] = {{&S38987[13], 9}, {NULL}};
const Segment SGI289986[] = {{&S38987[30], 13}, {NULL}};
const Segment SGI289987[] = {{&S38987[7], 1}, {NULL}};
const Segment SGI289988[] = {{&S39453[70], 4}, {NULL}};
const Segment SGI289989[] = {{&S39453[8], 3}, {NULL}};
const Segment SGI289990[] = {{&S39453[13], 9}, {NULL}};
const Segment SGI289991[] = {{&S39453[30], 13}, {NULL}};
const Segment SGI289992[] = {{&S39453[7], 1}, {NULL}};
const Segment SGI289993[] = {{&S39454[70], 4}, {&S39454[70], 4}, {NULL}};
const Segment SGI289994[] = {{&S39454[8], 3}, {NULL}};
const Segment SGI289995[] = {{&S39454[13], 9}, {NULL}};
const Segment SGI289996[] = {{&S39454[30], 13}, {NULL}};
const Segment SGI289997[] = {{&S39454[7], 1}, {&S39454[7], 1}, {NULL}};
const Segment SGI289998[] = {{&S39455[70], 4}, {NULL}};
const Segment SGI289999[] = {{&S39455[8], 3}, {NULL}};
const Segment SGI290000[] = {{&S39455[13], 9}, {NULL}};
const Segment SGI290001[] = {{&S39455[30], 13}, {NULL}};
const Segment SGI290002[] = {{&S39455[7], 1}, {&S39455[11], 1}, {NULL}};
const Segment SGI290003[] = {{&S39473[62], 5}, {NULL}};
const Segment SGI290004[] = {{&S39473[8], 3}, {NULL}};
const Segment SGI290005[] = {{&S39473[13], 3}, {NULL}};
const Segment SGI290006[] = {{&S39473[13], 3}, {&S39473[18], 6}, {NULL}};
const Segment SGI290007[] = {{&S39473[7], 1}, {NULL}};
const Segment SGI290008[] = {{&S39541[69], 5}, {NULL}};
const Segment SGI290009[] = {{&S39541[8], 3}, {NULL}};
const Segment SGI290010[] = {{&S39541[13], 9}, {NULL}};
const Segment SGI290011[] = {{&S39541[30], 13}, {NULL}};
const Segment SGI290012[] = {{&S39541[7], 1}, {NULL}};
const Segment SGI290013[] = {{&S39690[60], 5}, {NULL}};
const Segment SGI290014[] = {{&S39690[8], 3}, {NULL}};
const Segment SGI290015[] = {{&S39690[13], 14}, {NULL}};
const Segment SGI290016[] = {{&S39690[7], 1}, {NULL}};
const Segment SGI290017[] = {{&S39690[16], 11}, {NULL}};
const Segment SGI290018[] = {{&S39690[68], 13}, {NULL}};
const Segment SGI290019[] = {{&S39691[65], 5}, {NULL}};
const Segment SGI290020[] = {{&S39691[8], 3}, {NULL}};
const Segment SGI290021[] = {{&S39691[13], 14}, {NULL}};
const Segment SGI290022[] = {{&S39691[7], 1}, {NULL}};
const Segment SGI290023[] = {{&S39691[16], 11}, {&S39691[32], 2}, {NULL}};
const Segment SGI290024[] = {{&S39691[77], 13}, {NULL}};
const Segment SGI290025[] = {{&S39692[69], 5}, {NULL}};
const Segment SGI290026[] = {{&S39692[8], 3}, {NULL}};
const Segment SGI290027[] = {{&S39692[13], 9}, {NULL}};
const Segment SGI290028[] = {{&S39692[30], 13}, {NULL}};
const Segment SGI290029[] = {{&S39692[7], 1}, {NULL}};
const Segment SGI290030[] = {{&S39693[69], 5}, {&S39693[69], 5}, {NULL}};
const Segment SGI290031[] = {{&S39693[8], 3}, {NULL}};
const Segment SGI290032[] = {{&S39693[13], 9}, {NULL}};
const Segment SGI290033[] = {{&S39693[30], 13}, {NULL}};
const Segment SGI290034[] = {{&S39693[7], 1}, {&S39693[7], 1}, {NULL}};
const Segment SGI290035[] = {{&S39698[82], 5}, {NULL}};
const Segment SGI290036[] = {{&S39698[8], 3}, {NULL}};
const Segment SGI290037[] = {{&S39698[13], 9}, {NULL}};
const Segment SGI290038[] = {{&S39698[33], 15}, {NULL}};
const Segment SGI290039[] = {{&S39698[7], 1}, {NULL}};
const Segment SGI290040[] = {{&S39699[82], 5}, {NULL}};
const Segment SGI290041[] = {{&S39699[8], 3}, {NULL}};
const Segment SGI290042[] = {{&S39699[13], 9}, {NULL}};
const Segment SGI290043[] = {{&S39699[33], 15}, {NULL}};
const Segment SGI290044[] = {{&S39699[7], 1}, {NULL}};
const Segment SGI290045[] = {{&S39700[62], 5}, {NULL}};
const Segment SGI290046[] = {{&S39700[8], 3}, {NULL}};
const Segment SGI290047[] = {{&S39700[13], 3}, {NULL}};
const Segment SGI290048[] = {{&S39700[13], 3}, {&S39700[18], 6}, {NULL}};
const Segment SGI290049[] = {{&S39700[7], 1}, {NULL}};
const Segment SGI290050[] = {{&S39701[64], 5}, {&S39701[64], 5}, {NULL}};
const Segment SGI290051[] = {{&S39701[8], 3}, {NULL}};
const Segment SGI290052[] = {{&S39701[13], 3}, {NULL}};
const Segment SGI290053[] = {{&S39701[13], 3}, {&S39701[18], 6}, {NULL}};
const Segment SGI290054[] = {{&S39701[7], 1}, {&S39701[7], 1}, {NULL}};
const Segment SGI290055[] = {{&S39917[66], 4}, {NULL}};
const Segment SGI290056[] = {{&S39917[8], 3}, {NULL}};
const Segment SGI290057[] = {{&S39917[13], 9}, {NULL}};
const Segment SGI290058[] = {{&S39917[33], 15}, {NULL}};
const Segment SGI290059[] = {{&S39917[7], 1}, {NULL}};
const Segment SGI290060[] = {{&S39918[78], 4}, {NULL}};
const Segment SGI290061[] = {{&S39918[8], 3}, {NULL}};
const Segment SGI290062[] = {{&S39918[13], 9}, {NULL}};
const Segment SGI290063[] = {{&S39918[7], 1}, {NULL}};
const Segment SGI290064[] = {{&S39919[158], 4}, {NULL}};
const Segment SGI290065[] = {{&S39919[8], 3}, {&S39919[8], 3}, {NULL}};
const Segment SGI290066[] = {{&S39919[13], 9}, {&S39919[13], 9}, {NULL}};
const Segment SGI290067[] = {{&S39919[33], 13}, {&S39919[33], 13}, {NULL}};
const Segment SGI290068[] = {{&S39919[7], 1}, {NULL}};
const Segment SGI290069[] = {{&S39920[82], 4}, {NULL}};
const Segment SGI290070[] = {{&S39920[8], 3}, {NULL}};
const Segment SGI290071[] = {{&S39920[13], 9}, {NULL}};
const Segment SGI290072[] = {{&S39920[33], 13}, {NULL}};
const Segment SGI290073[] = {{&S39920[7], 1}, {NULL}};
const Segment SGI290074[] = {{&S39957[64], 5}, {NULL}};
const Segment SGI290075[] = {{&S39957[8], 3}, {NULL}};
const Segment SGI290076[] = {{&S39957[13], 3}, {NULL}};
const Segment SGI290077[] = {{&S39957[7], 1}, {NULL}};
const Segment SGI290078[] = {{&S39958[54], 5}, {NULL}};
const Segment SGI290079[] = {{&S39958[8], 3}, {NULL}};
const Segment SGI290080[] = {{&S39958[13], 3}, {NULL}};
const Segment SGI290081[] = {{&S39958[7], 1}, {NULL}};
const Segment SGI290082[] = {{&S40090[69], 5}, {NULL}};
const Segment SGI290083[] = {{&S40090[8], 3}, {NULL}};
const Segment SGI290084[] = {{&S40090[13], 9}, {NULL}};
const Segment SGI290085[] = {{&S40090[30], 13}, {NULL}};
const Segment SGI290086[] = {{&S40090[7], 1}, {NULL}};
const Segment SGI290087[] = {{&S40092[71], 5}, {NULL}};
const Segment SGI290088[] = {{&S40092[8], 3}, {NULL}};
const Segment SGI290089[] = {{&S40092[13], 9}, {NULL}};
const Segment SGI290090[] = {{&S40092[30], 13}, {NULL}};
const Segment SGI290091[] = {{&S40092[7], 1}, {NULL}};
const Segment SGI290092[] = {{&S40177[76], 4}, {NULL}};
const Segment SGI290093[] = {{&S40177[8], 3}, {NULL}};
const Segment SGI290094[] = {{&S40177[13], 7}, {NULL}};
const Segment SGI290095[] = {{&S40177[25], 14}, {NULL}};
const Segment SGI290096[] = {{&S40177[7], 1}, {NULL}};
const Segment SGI290097[] = {{&S40177[16], 4}, {&S40177[28], 11}, {NULL}};
const Segment SGI290098[] = {{&S40178[7], 4}, {NULL}};
const Segment SGI290099[] = {{&S40178[8], 3}, {NULL}};
const Segment SGI290100[] = {{&S40178[13], 9}, {NULL}};
const Segment SGI290101[] = {{&S40178[7], 1}, {NULL}};
const Segment SGI290102[] = {{&S40179[7], 4}, {NULL}};
const Segment SGI290103[] = {{&S40179[8], 3}, {NULL}};
const Segment SGI290104[] = {{&S40179[13], 9}, {NULL}};
const Segment SGI290105[] = {{&S40179[30], 13}, {NULL}};
const Segment SGI290106[] = {{&S40179[7], 1}, {NULL}};
const Segment SGI290107[] = {{&S40278[53], 5}, {NULL}};
const Segment SGI290108[] = {{&S40278[8], 3}, {NULL}};
const Segment SGI290109[] = {{&S40278[13], 14}, {NULL}};
const Segment SGI290110[] = {{&S40278[7], 1}, {NULL}};
const Segment SGI290111[] = {{&S40278[16], 11}, {NULL}};
const Segment SGI290112[] = {{&S40278[59], 13}, {NULL}};
const Segment SGI290113[] = {{&S40279[60], 5}, {NULL}};
const Segment SGI290114[] = {{&S40279[8], 3}, {NULL}};
const Segment SGI290115[] = {{&S40279[13], 14}, {NULL}};
const Segment SGI290116[] = {{&S40279[7], 1}, {NULL}};
const Segment SGI290117[] = {{&S40279[16], 11}, {NULL}};
const Segment SGI290118[] = {{&S40279[66], 13}, {NULL}};
const Segment SGI290119[] = {{&S40282[88], 4}, {NULL}};
const Segment SGI290120[] = {{&S40282[8], 3}, {NULL}};
const Segment SGI290121[] = {{&S40282[13], 3}, {NULL}};
const Segment SGI290122[] = {{&S40282[13], 3}, {&S40282[60], 6}, {NULL}};
const Segment SGI290123[] = {{&S40282[60], 10}, {NULL}};
const Segment SGI290124[] = {{&S40282[7], 1}, {NULL}};
const Segment SGI290125[] = {{&S40283[84], 5}, {NULL}};
const Segment SGI290126[] = {{&S40283[8], 3}, {NULL}};
const Segment SGI290127[] = {{&S40283[13], 3}, {NULL}};
const Segment SGI290128[] = {{&S40283[13], 3}, {&S40283[57], 6}, {NULL}};
const Segment SGI290129[] = {{&S40283[57], 11}, {NULL}};
const Segment SGI290130[] = {{&S40283[7], 1}, {NULL}};
const Segment SGI290131[] = {{&S40284[85], 4}, {NULL}};
const Segment SGI290132[] = {{&S40284[8], 3}, {NULL}};
const Segment SGI290133[] = {{&S40284[13], 3}, {NULL}};
const Segment SGI290134[] = {{&S40284[13], 3}, {&S40284[58], 6}, {NULL}};
const Segment SGI290135[] = {{&S40284[58], 11}, {NULL}};
const Segment SGI290136[] = {{&S40284[7], 1}, {NULL}};
const Segment SGI290137[] = {{&S40304[69], 5}, {NULL}};
const Segment SGI290138[] = {{&S40304[8], 3}, {NULL}};
const Segment SGI290139[] = {{&S40304[13], 9}, {NULL}};
const Segment SGI290140[] = {{&S40304[30], 13}, {NULL}};
const Segment SGI290141[] = {{&S40304[7], 1}, {NULL}};
const Segment SGI290142[] = {{&S40342[65], 5}, {NULL}};
const Segment SGI290143[] = {{&S40342[8], 3}, {NULL}};
const Segment SGI290144[] = {{&S40342[13], 14}, {NULL}};
const Segment SGI290145[] = {{&S40342[7], 1}, {NULL}};
const Segment SGI290146[] = {{&S40342[16], 11}, {&S40342[32], 2}, {NULL}};
const Segment SGI290147[] = {{&S40342[71], 13}, {NULL}};
const Segment SGI290148[] = {{&S40343[60], 5}, {NULL}};
const Segment SGI290149[] = {{&S40343[8], 3}, {NULL}};
const Segment SGI290150[] = {{&S40343[13], 14}, {NULL}};
const Segment SGI290151[] = {{&S40343[7], 1}, {NULL}};
const Segment SGI290152[] = {{&S40343[16], 11}, {NULL}};
const Segment SGI290153[] = {{&S40343[66], 13}, {NULL}};
const Segment SGI290154[] = {{&S40375[56], 5}, {NULL}};
const Segment SGI290155[] = {{&S40375[8], 3}, {NULL}};
const Segment SGI290156[] = {{&S40375[13], 3}, {NULL}};
const Segment SGI290157[] = {{&S40375[7], 1}, {NULL}};
const Segment SGI290158[] = {{&S40376[54], 5}, {NULL}};
const Segment SGI290159[] = {{&S40376[8], 3}, {NULL}};
const Segment SGI290160[] = {{&S40376[13], 3}, {NULL}};
const Segment SGI290161[] = {{&S40376[7], 1}, {NULL}};
const Segment SGI290162[] = {{&S40377[54], 5}, {&S40377[54], 5}, {NULL}};
const Segment SGI290163[] = {{&S40377[8], 3}, {NULL}};
const Segment SGI290164[] = {{&S40377[13], 3}, {NULL}};
const Segment SGI290165[] = {{&S40377[7], 1}, {&S40377[7], 1}, {NULL}};
const Segment SGI290166[] = {{&S40534[97], 4}, {NULL}};
const Segment SGI290167[] = {{&S40534[8], 3}, {NULL}};
const Segment SGI290168[] = {{&S40534[13], 9}, {NULL}};
const Segment SGI290169[] = {{&S40534[31], 13}, {NULL}};
const Segment SGI290170[] = {{&S40534[7], 1}, {NULL}};
const Segment SGI290171[] = {{&S40534[71], 10}, {NULL}};
const Segment SGI290172[] = {{&S40535[97], 5}, {NULL}};
const Segment SGI290173[] = {{&S40535[8], 3}, {NULL}};
const Segment SGI290174[] = {{&S40535[13], 9}, {NULL}};
const Segment SGI290175[] = {{&S40535[31], 13}, {NULL}};
const Segment SGI290176[] = {{&S40535[7], 1}, {NULL}};
const Segment SGI290177[] = {{&S40535[71], 10}, {NULL}};
const Segment SGI290178[] = {{&S40736[62], 4}, {NULL}};
const Segment SGI290179[] = {{&S40736[8], 3}, {NULL}};
const Segment SGI290180[] = {{&S40736[13], 7}, {NULL}};
const Segment SGI290181[] = {{&S40736[22], 14}, {NULL}};
const Segment SGI290182[] = {{&S40736[7], 1}, {NULL}};
const Segment SGI290183[] = {{&S40736[16], 4}, {&S40736[25], 11}, {NULL}};
const Segment SGI290184[] = {{&S40737[54], 4}, {NULL}};
const Segment SGI290185[] = {{&S40737[8], 3}, {NULL}};
const Segment SGI290186[] = {{&S40737[13], 14}, {NULL}};
const Segment SGI290187[] = {{&S40737[7], 1}, {NULL}};
const Segment SGI290188[] = {{&S40737[16], 11}, {NULL}};
const Segment SGI290189[] = {{&S40738[80], 4}, {NULL}};
const Segment SGI290190[] = {{&S40738[8], 3}, {NULL}};
const Segment SGI290191[] = {{&S40738[13], 14}, {NULL}};
const Segment SGI290192[] = {{&S40738[7], 1}, {NULL}};
const Segment SGI290193[] = {{&S40738[16], 11}, {&S40738[32], 2}, {NULL}};
const Segment SGI290194[] = {{&S40739[60], 4}, {NULL}};
const Segment SGI290195[] = {{&S40739[8], 3}, {NULL}};
const Segment SGI290196[] = {{&S40739[13], 22}, {NULL}};
const Segment SGI290197[] = {{&S40739[21], 14}, {NULL}};
const Segment SGI290198[] = {{&S40739[7], 1}, {NULL}};
const Segment SGI290199[] = {{&S40739[16], 19}, {NULL}};
const Segment SGI290200[] = {{&S40755[113], 4}, {NULL}};
const Segment SGI290201[] = {{&S40755[8], 3}, {NULL}};
const Segment SGI290202[] = {{&S40755[13], 9}, {NULL}};
const Segment SGI290203[] = {{&S40755[33], 15}, {NULL}};
const Segment SGI290204[] = {{&S40755[7], 1}, {NULL}};
const Segment SGI290205[] = {{&S40756[77], 4}, {NULL}};
const Segment SGI290206[] = {{&S40756[8], 3}, {NULL}};
const Segment SGI290207[] = {{&S40756[13], 9}, {NULL}};
const Segment SGI290208[] = {{&S40756[31], 13}, {NULL}};
const Segment SGI290209[] = {{&S40756[7], 1}, {NULL}};
const Segment SGI290210[] = {{&S40810[71], 4}, {NULL}};
const Segment SGI290211[] = {{&S40810[8], 3}, {NULL}};
const Segment SGI290212[] = {{&S40810[13], 14}, {NULL}};
const Segment SGI290213[] = {{&S40810[7], 1}, {NULL}};
const Segment SGI290214[] = {{&S40810[16], 11}, {NULL}};
const Segment SGI290215[] = {{&S40810[78], 10}, {NULL}};
const Segment SGI290216[] = {{&S40813[61], 4}, {NULL}};
const Segment SGI290217[] = {{&S40813[8], 3}, {NULL}};
const Segment SGI290218[] = {{&S40813[13], 14}, {NULL}};
const Segment SGI290219[] = {{&S40813[7], 1}, {NULL}};
const Segment SGI290220[] = {{&S40813[16], 11}, {NULL}};
const Segment SGI290221[] = {{&S40813[68], 10}, {NULL}};
const Segment SGI290222[] = {{&S40854[65], 5}, {NULL}};
const Segment SGI290223[] = {{&S40854[8], 3}, {NULL}};
const Segment SGI290224[] = {{&S40854[13], 14}, {NULL}};
const Segment SGI290225[] = {{&S40854[7], 1}, {NULL}};
const Segment SGI290226[] = {{&S40854[16], 11}, {&S40854[32], 2}, {NULL}};
const Segment SGI290227[] = {{&S40855[53], 5}, {&S40855[53], 5}, {NULL}};
const Segment SGI290228[] = {{&S40855[8], 3}, {NULL}};
const Segment SGI290229[] = {{&S40855[13], 14}, {NULL}};
const Segment SGI290230[] = {{&S40855[7], 1}, {&S40855[7], 1}, {NULL}};
const Segment SGI290231[] = {{&S40855[16], 11}, {NULL}};
const Segment SGI290232[] = {{&S40856[60], 5}, {NULL}};
const Segment SGI290233[] = {{&S40856[8], 3}, {NULL}};
const Segment SGI290234[] = {{&S40856[13], 14}, {NULL}};
const Segment SGI290235[] = {{&S40856[7], 1}, {NULL}};
const Segment SGI290236[] = {{&S40856[16], 11}, {NULL}};
const Segment SGI290237[] = {{&S40865[61], 4}, {NULL}};
const Segment SGI290238[] = {{&S40865[8], 3}, {NULL}};
const Segment SGI290239[] = {{&S40865[13], 14}, {NULL}};
const Segment SGI290240[] = {{&S40865[7], 1}, {NULL}};
const Segment SGI290241[] = {{&S40865[16], 11}, {NULL}};
const Segment SGI290242[] = {{&S40865[68], 11}, {NULL}};
const Segment SGI290243[] = {{&S40868[61], 4}, {NULL}};
const Segment SGI290244[] = {{&S40868[8], 3}, {NULL}};
const Segment SGI290245[] = {{&S40868[13], 14}, {NULL}};
const Segment SGI290246[] = {{&S40868[7], 1}, {NULL}};
const Segment SGI290247[] = {{&S40868[16], 11}, {NULL}};
const Segment SGI290248[] = {{&S40868[68], 11}, {NULL}};
const Segment SGI290249[] = {{&S40888[89], 4}, {NULL}};
const Segment SGI290250[] = {{&S40888[8], 3}, {NULL}};
const Segment SGI290251[] = {{&S40888[13], 9}, {NULL}};
const Segment SGI290252[] = {{&S40888[33], 13}, {NULL}};
const Segment SGI290253[] = {{&S40888[7], 1}, {NULL}};
const Segment SGI290254[] = {{&S40889[81], 4}, {NULL}};
const Segment SGI290255[] = {{&S40889[8], 3}, {NULL}};
const Segment SGI290256[] = {{&S40889[13], 9}, {NULL}};
const Segment SGI290257[] = {{&S40889[30], 15}, {NULL}};
const Segment SGI290258[] = {{&S40889[7], 1}, {NULL}};
const Segment SGI290259[] = {{&S40920[61], 4}, {NULL}};
const Segment SGI290260[] = {{&S40920[8], 3}, {NULL}};
const Segment SGI290261[] = {{&S40920[13], 14}, {NULL}};
const Segment SGI290262[] = {{&S40920[7], 1}, {NULL}};
const Segment SGI290263[] = {{&S40920[16], 11}, {NULL}};
const Segment SGI290264[] = {{&S40922[56], 4}, {NULL}};
const Segment SGI290265[] = {{&S40922[8], 3}, {NULL}};
const Segment SGI290266[] = {{&S40922[13], 7}, {NULL}};
const Segment SGI290267[] = {{&S40922[7], 1}, {NULL}};
const Segment SGI290268[] = {{&S40922[16], 4}, {&S40922[28], 37}, {NULL}};
const Segment SGI290269[] = {{&S40923[75], 4}, {NULL}};
const Segment SGI290270[] = {{&S40923[8], 3}, {NULL}};
const Segment SGI290271[] = {{&S40923[13], 7}, {NULL}};
const Segment SGI290272[] = {{&S40923[25], 14}, {NULL}};
const Segment SGI290273[] = {{&S40923[7], 1}, {NULL}};
const Segment SGI290274[] = {{&S40923[16], 4}, {&S40923[28], 11}, {NULL}};
const Segment SGI290275[] = {{&S40926[68], 4}, {NULL}};
const Segment SGI290276[] = {{&S40926[8], 3}, {NULL}};
const Segment SGI290277[] = {{&S40926[13], 7}, {NULL}};
const Segment SGI290278[] = {{&S40926[25], 14}, {NULL}};
const Segment SGI290279[] = {{&S40926[7], 1}, {NULL}};
const Segment SGI290280[] = {{&S40926[16], 4}, {&S40926[28], 11}, {NULL}};
const Segment SGI290281[] = {{&S40927[72], 4}, {NULL}};
const Segment SGI290282[] = {{&S40927[8], 3}, {NULL}};
const Segment SGI290283[] = {{&S40927[7], 1}, {NULL}};
const Segment SGI290284[] = {{&S40927[16], 4}, {&S40927[28], 11}, {NULL}};
const Segment SGI290285[] = {{&S40947[60], 5}, {NULL}};
const Segment SGI290286[] = {{&S40947[8], 3}, {NULL}};
const Segment SGI290287[] = {{&S40947[13], 14}, {NULL}};
const Segment SGI290288[] = {{&S40947[7], 1}, {NULL}};
const Segment SGI290289[] = {{&S40947[16], 11}, {NULL}};
const Segment SGI290290[] = {{&S40947[66], 13}, {NULL}};
const Segment SGI290291[] = {{&S40949[65], 5}, {NULL}};
const Segment SGI290292[] = {{&S40949[8], 3}, {NULL}};
const Segment SGI290293[] = {{&S40949[13], 14}, {NULL}};
const Segment SGI290294[] = {{&S40949[7], 1}, {NULL}};
const Segment SGI290295[] = {{&S40949[16], 11}, {&S40949[32], 2}, {NULL}};
const Segment SGI290296[] = {{&S40949[71], 13}, {NULL}};
const Segment SGI290297[] = {{&S41065[90], 4}, {NULL}};
const Segment SGI290298[] = {{&S41065[8], 3}, {NULL}};
const Segment SGI290299[] = {{&S41065[13], 3}, {NULL}};
const Segment SGI290300[] = {{&S41065[13], 3}, {&S41065[60], 6}, {NULL}};
const Segment SGI290301[] = {{&S41065[60], 11}, {NULL}};
const Segment SGI290302[] = {{&S41065[7], 1}, {NULL}};
const Segment SGI290303[] = {{&S41072[83], 4}, {NULL}};
const Segment SGI290304[] = {{&S41072[8], 3}, {NULL}};
const Segment SGI290305[] = {{&S41072[13], 9}, {NULL}};
const Segment SGI290306[] = {{&S41072[33], 13}, {NULL}};
const Segment SGI290307[] = {{&S41072[7], 1}, {NULL}};
const Segment SGI290308[] = {{&S41084[60], 5}, {NULL}};
const Segment SGI290309[] = {{&S41084[8], 3}, {NULL}};
const Segment SGI290310[] = {{&S41084[13], 14}, {NULL}};
const Segment SGI290311[] = {{&S41084[7], 1}, {NULL}};
const Segment SGI290312[] = {{&S41084[16], 11}, {NULL}};
const Segment SGI290313[] = {{&S41085[57], 5}, {NULL}};
const Segment SGI290314[] = {{&S41085[8], 3}, {NULL}};
const Segment SGI290315[] = {{&S41085[13], 14}, {NULL}};
const Segment SGI290316[] = {{&S41085[7], 1}, {NULL}};
const Segment SGI290317[] = {{&S41085[16], 11}, {&S41085[32], 2}, {NULL}};
const Segment SGI290318[] = {{&S41086[65], 5}, {NULL}};
const Segment SGI290319[] = {{&S41086[8], 3}, {NULL}};
const Segment SGI290320[] = {{&S41086[13], 14}, {NULL}};
const Segment SGI290321[] = {{&S41086[7], 1}, {NULL}};
const Segment SGI290322[] = {{&S41086[16], 11}, {&S41086[32], 2}, {NULL}};
const Segment SGI290323[] = {{&S41086[71], 13}, {NULL}};
const Segment SGI290324[] = {{&S41087[60], 5}, {NULL}};
const Segment SGI290325[] = {{&S41087[8], 3}, {NULL}};
const Segment SGI290326[] = {{&S41087[13], 14}, {NULL}};
const Segment SGI290327[] = {{&S41087[7], 1}, {NULL}};
const Segment SGI290328[] = {{&S41087[16], 11}, {NULL}};
const Segment SGI290329[] = {{&S41087[66], 13}, {NULL}};
const Segment SGI290330[] = {{&S41113[67], 5}, {NULL}};
const Segment SGI290331[] = {{&S41113[8], 3}, {NULL}};
const Segment SGI290332[] = {{&S41113[13], 14}, {NULL}};
const Segment SGI290333[] = {{&S41113[7], 1}, {NULL}};
const Segment SGI290334[] = {{&S41113[16], 11}, {&S41113[32], 2}, {NULL}};
const Segment SGI290335[] = {{&S41216[65], 5}, {NULL}};
const Segment SGI290336[] = {{&S41216[8], 3}, {NULL}};
const Segment SGI290337[] = {{&S41216[13], 14}, {NULL}};
const Segment SGI290338[] = {{&S41216[7], 1}, {NULL}};
const Segment SGI290339[] = {{&S41216[16], 11}, {&S41216[32], 2}, {NULL}};
const Segment SGI290340[] = {{&S41216[71], 13}, {NULL}};
const Segment SGI290341[] = {{&S41217[60], 5}, {NULL}};
const Segment SGI290342[] = {{&S41217[8], 3}, {NULL}};
const Segment SGI290343[] = {{&S41217[13], 14}, {NULL}};
const Segment SGI290344[] = {{&S41217[7], 1}, {NULL}};
const Segment SGI290345[] = {{&S41217[16], 11}, {NULL}};
const Segment SGI290346[] = {{&S41217[66], 13}, {NULL}};
const Segment SGI290347[] = {{&S41257[56], 5}, {NULL}};
const Segment SGI290348[] = {{&S41257[8], 3}, {NULL}};
const Segment SGI290349[] = {{&S41257[13], 3}, {NULL}};
const Segment SGI290350[] = {{&S41257[7], 1}, {NULL}};
const Segment SGI290351[] = {{&S41258[64], 5}, {NULL}};
const Segment SGI290352[] = {{&S41258[8], 3}, {NULL}};
const Segment SGI290353[] = {{&S41258[13], 3}, {NULL}};
const Segment SGI290354[] = {{&S41258[7], 1}, {NULL}};
const Segment SGI290355[] = {{&S41380[73], 4}, {NULL}};
const Segment SGI290356[] = {{&S41380[8], 3}, {NULL}};
const Segment SGI290357[] = {{&S41380[13], 7}, {NULL}};
const Segment SGI290358[] = {{&S41380[25], 14}, {NULL}};
const Segment SGI290359[] = {{&S41380[7], 1}, {NULL}};
const Segment SGI290360[] = {{&S41380[16], 4}, {&S41380[28], 11}, {NULL}};
const Segment SGI290361[] = {{&S41381[75], 4}, {NULL}};
const Segment SGI290362[] = {{&S41381[8], 3}, {NULL}};
const Segment SGI290363[] = {{&S41381[27], 14}, {NULL}};
const Segment SGI290364[] = {{&S41381[7], 1}, {NULL}};
const Segment SGI290365[] = {{&S41381[18], 4}, {&S41381[30], 11}, {NULL}};
const Segment SGI290366[] = {{&S41382[73], 4}, {NULL}};
const Segment SGI290367[] = {{&S41382[8], 3}, {NULL}};
const Segment SGI290368[] = {{&S41382[13], 7}, {NULL}};
const Segment SGI290369[] = {{&S41382[25], 14}, {NULL}};
const Segment SGI290370[] = {{&S41382[7], 1}, {NULL}};
const Segment SGI290371[] = {{&S41382[16], 4}, {&S41382[28], 11}, {NULL}};
const Segment SGI290372[] = {{&S41383[70], 4}, {NULL}};
const Segment SGI290373[] = {{&S41383[8], 3}, {NULL}};
const Segment SGI290374[] = {{&S41383[13], 7}, {NULL}};
const Segment SGI290375[] = {{&S41383[7], 1}, {NULL}};
const Segment SGI290376[] = {{&S41383[16], 4}, {&S41383[28], 10}, {NULL}};
const Segment SGI290377[] = {{&S41384[71], 4}, {NULL}};
const Segment SGI290378[] = {{&S41384[8], 3}, {NULL}};
const Segment SGI290379[] = {{&S41384[13], 7}, {NULL}};
const Segment SGI290380[] = {{&S41384[25], 14}, {NULL}};
const Segment SGI290381[] = {{&S41384[7], 1}, {NULL}};
const Segment SGI290382[] = {{&S41384[16], 4}, {&S41384[28], 11}, {NULL}};
const Segment SGI290383[] = {{&S41454[64], 5}, {NULL}};
const Segment SGI290384[] = {{&S41454[8], 3}, {NULL}};
const Segment SGI290385[] = {{&S41454[13], 3}, {NULL}};
const Segment SGI290386[] = {{&S41454[7], 1}, {NULL}};
const Segment SGI290387[] = {{&S41455[54], 5}, {NULL}};
const Segment SGI290388[] = {{&S41455[8], 3}, {NULL}};
const Segment SGI290389[] = {{&S41455[13], 3}, {NULL}};
const Segment SGI290390[] = {{&S41455[7], 1}, {NULL}};
const Segment SGI290391[] = {{&S41456[54], 5}, {&S41456[54], 5}, {NULL}};
const Segment SGI290392[] = {{&S41456[8], 3}, {NULL}};
const Segment SGI290393[] = {{&S41456[13], 3}, {NULL}};
const Segment SGI290394[] = {{&S41456[7], 1}, {&S41456[7], 1}, {NULL}};
const Segment SGI290395[] = {{&S41629[56], 5}, {NULL}};
const Segment SGI290396[] = {{&S41629[8], 3}, {NULL}};
const Segment SGI290397[] = {{&S41629[13], 3}, {NULL}};
const Segment SGI290398[] = {{&S41629[7], 1}, {NULL}};
const Segment SGI290399[] = {{&S41630[64], 5}, {NULL}};
const Segment SGI290400[] = {{&S41630[8], 3}, {NULL}};
const Segment SGI290401[] = {{&S41630[13], 3}, {NULL}};
const Segment SGI290402[] = {{&S41630[7], 1}, {NULL}};
const Segment SGI290403[] = {{&S41671[60], 5}, {NULL}};
const Segment SGI290404[] = {{&S41671[8], 3}, {NULL}};
const Segment SGI290405[] = {{&S41671[13], 14}, {NULL}};
const Segment SGI290406[] = {{&S41671[7], 1}, {NULL}};
const Segment SGI290407[] = {{&S41671[16], 11}, {NULL}};
const Segment SGI290408[] = {{&S41671[66], 13}, {NULL}};
const Segment SGI290409[] = {{&S41818[53], 5}, {NULL}};
const Segment SGI290410[] = {{&S41818[8], 3}, {NULL}};
const Segment SGI290411[] = {{&S41818[13], 14}, {NULL}};
const Segment SGI290412[] = {{&S41818[7], 1}, {NULL}};
const Segment SGI290413[] = {{&S41818[16], 11}, {NULL}};
const Segment SGI290414[] = {{&S41819[53], 5}, {&S41819[53], 5}, {NULL}};
const Segment SGI290415[] = {{&S41819[8], 3}, {NULL}};
const Segment SGI290416[] = {{&S41819[13], 14}, {NULL}};
const Segment SGI290417[] = {{&S41819[7], 1}, {&S41819[7], 1}, {NULL}};
const Segment SGI290418[] = {{&S41819[16], 11}, {NULL}};
const Segment SGI290419[] = {{&S41820[65], 5}, {NULL}};
const Segment SGI290420[] = {{&S41820[8], 3}, {NULL}};
const Segment SGI290421[] = {{&S41820[13], 14}, {NULL}};
const Segment SGI290422[] = {{&S41820[7], 1}, {NULL}};
const Segment SGI290423[] = {{&S41820[16], 11}, {&S41820[32], 2}, {NULL}};
const Segment SGI290424[] = {{&S41841[65], 5}, {NULL}};
const Segment SGI290425[] = {{&S41841[8], 3}, {NULL}};
const Segment SGI290426[] = {{&S41841[13], 14}, {NULL}};
const Segment SGI290427[] = {{&S41841[7], 1}, {NULL}};
const Segment SGI290428[] = {{&S41841[16], 11}, {&S41841[32], 2}, {NULL}};
const Segment SGI290429[] = {{&S41842[60], 5}, {NULL}};
const Segment SGI290430[] = {{&S41842[8], 3}, {NULL}};
const Segment SGI290431[] = {{&S41842[13], 14}, {NULL}};
const Segment SGI290432[] = {{&S41842[7], 1}, {NULL}};
const Segment SGI290433[] = {{&S41842[16], 11}, {NULL}};
const Segment SGI290434[] = {{&S41886[66], 4}, {NULL}};
const Segment SGI290435[] = {{&S41886[8], 3}, {NULL}};
const Segment SGI290436[] = {{&S41886[13], 3}, {NULL}};
const Segment SGI290437[] = {{&S41886[7], 1}, {NULL}};
const Segment SGI290438[] = {{&S41940[60], 5}, {NULL}};
const Segment SGI290439[] = {{&S41940[8], 3}, {NULL}};
const Segment SGI290440[] = {{&S41940[13], 14}, {NULL}};
const Segment SGI290441[] = {{&S41940[7], 1}, {NULL}};
const Segment SGI290442[] = {{&S41940[16], 11}, {NULL}};
const Segment SGI290443[] = {{&S41941[67], 5}, {NULL}};
const Segment SGI290444[] = {{&S41941[8], 3}, {NULL}};
const Segment SGI290445[] = {{&S41941[13], 14}, {NULL}};
const Segment SGI290446[] = {{&S41941[7], 1}, {NULL}};
const Segment SGI290447[] = {{&S41941[16], 11}, {&S41941[32], 2}, {NULL}};
const Segment SGI290448[] = {{&S41942[60], 5}, {NULL}};
const Segment SGI290449[] = {{&S41942[8], 3}, {NULL}};
const Segment SGI290450[] = {{&S41942[13], 14}, {NULL}};
const Segment SGI290451[] = {{&S41942[7], 1}, {NULL}};
const Segment SGI290452[] = {{&S41942[16], 11}, {NULL}};
const Segment SGI290453[] = {{&S41942[66], 14}, {NULL}};
const Segment SGI290454[] = {{&S41965[7], 4}, {NULL}};
const Segment SGI290455[] = {{&S41965[8], 3}, {NULL}};
const Segment SGI290456[] = {{&S41965[13], 14}, {NULL}};
const Segment SGI290457[] = {{&S41965[7], 1}, {NULL}};
const Segment SGI290458[] = {{&S41965[16], 11}, {NULL}};
const Segment SGI290459[] = {{&S42111[65], 4}, {NULL}};
const Segment SGI290460[] = {{&S42111[8], 3}, {NULL}};
const Segment SGI290461[] = {{&S42111[24], 14}, {NULL}};
const Segment SGI290462[] = {{&S42111[7], 1}, {NULL}};
const Segment SGI290463[] = {{&S42111[15], 4}, {&S42111[27], 11}, {NULL}};
const Segment SGI290464[] = {{&S42112[75], 4}, {NULL}};
const Segment SGI290465[] = {{&S42112[8], 3}, {NULL}};
const Segment SGI290466[] = {{&S42112[13], 7}, {NULL}};
const Segment SGI290467[] = {{&S42112[25], 14}, {NULL}};
const Segment SGI290468[] = {{&S42112[7], 1}, {NULL}};
const Segment SGI290469[] = {{&S42112[16], 4}, {&S42112[28], 11}, {NULL}};
const Segment SGI290470[] = {{&S42113[64], 4}, {NULL}};
const Segment SGI290471[] = {{&S42113[20], 3}, {NULL}};
const Segment SGI290472[] = {{&S42113[25], 14}, {NULL}};
const Segment SGI290473[] = {{&S42113[19], 1}, {NULL}};
const Segment SGI290474[] = {{&S42113[28], 11}, {NULL}};
const Segment SGI290475[] = {{&S42114[52], 4}, {&S42114[52], 4}, {NULL}};
const Segment SGI290476[] = {{&S42114[8], 3}, {NULL}};
const Segment SGI290477[] = {{&S42114[13], 14}, {NULL}};
const Segment SGI290478[] = {{&S42114[7], 1}, {&S42114[7], 1}, {NULL}};
const Segment SGI290479[] = {{&S42114[16], 11}, {NULL}};
const Segment SGI290480[] = {{&S42115[75], 4}, {NULL}};
const Segment SGI290481[] = {{&S42115[8], 3}, {NULL}};
const Segment SGI290482[] = {{&S42115[13], 7}, {NULL}};
const Segment SGI290483[] = {{&S42115[25], 14}, {NULL}};
const Segment SGI290484[] = {{&S42115[7], 1}, {NULL}};
const Segment SGI290485[] = {{&S42115[16], 4}, {&S42115[28], 11}, {NULL}};
const Segment SGI290486[] = {{&S42116[59], 4}, {NULL}};
const Segment SGI290487[] = {{&S42116[8], 3}, {NULL}};
const Segment SGI290488[] = {{&S42116[13], 14}, {NULL}};
const Segment SGI290489[] = {{&S42116[7], 1}, {NULL}};
const Segment SGI290490[] = {{&S42116[16], 11}, {NULL}};
const Segment SGI290491[] = {{&S42207[60], 5}, {NULL}};
const Segment SGI290492[] = {{&S42207[8], 3}, {NULL}};
const Segment SGI290493[] = {{&S42207[13], 14}, {NULL}};
const Segment SGI290494[] = {{&S42207[7], 1}, {NULL}};
const Segment SGI290495[] = {{&S42207[16], 11}, {NULL}};
const Segment SGI290496[] = {{&S42207[66], 13}, {NULL}};
const Segment SGI290497[] = {{&S42209[75], 4}, {NULL}};
const Segment SGI290498[] = {{&S42209[8], 3}, {NULL}};
const Segment SGI290499[] = {{&S42209[13], 7}, {NULL}};
const Segment SGI290500[] = {{&S42209[25], 14}, {NULL}};
const Segment SGI290501[] = {{&S42209[7], 1}, {NULL}};
const Segment SGI290502[] = {{&S42209[16], 4}, {&S42209[28], 11}, {NULL}};
const Segment SGI290503[] = {{&S42210[59], 4}, {&S42210[59], 4}, {NULL}};
const Segment SGI290504[] = {{&S42210[8], 3}, {NULL}};
const Segment SGI290505[] = {{&S42210[13], 14}, {NULL}};
const Segment SGI290506[] = {{&S42210[7], 1}, {&S42210[7], 1}, {NULL}};
const Segment SGI290507[] = {{&S42210[16], 11}, {NULL}};
const Segment SGI290508[] = {{&S42211[64], 4}, {NULL}};
const Segment SGI290509[] = {{&S42211[8], 3}, {NULL}};
const Segment SGI290510[] = {{&S42211[13], 14}, {NULL}};
const Segment SGI290511[] = {{&S42211[7], 1}, {NULL}};
const Segment SGI290512[] = {{&S42211[16], 11}, {&S42211[32], 2}, {NULL}};
const Segment SGI290513[] = {{&S42212[59], 8}, {NULL}};
const Segment SGI290514[] = {{&S42212[8], 3}, {NULL}};
const Segment SGI290515[] = {{&S42212[13], 14}, {NULL}};
const Segment SGI290516[] = {{&S42212[7], 1}, {NULL}};
const Segment SGI290517[] = {{&S42212[16], 11}, {NULL}};
const Segment SGI290518[] = {{&S42213[75], 4}, {NULL}};
const Segment SGI290519[] = {{&S42213[8], 3}, {NULL}};
const Segment SGI290520[] = {{&S42213[13], 7}, {NULL}};
const Segment SGI290521[] = {{&S42213[22], 14}, {NULL}};
const Segment SGI290522[] = {{&S42213[7], 1}, {NULL}};
const Segment SGI290523[] = {{&S42213[16], 4}, {&S42213[25], 11}, {&S42213[41], 2}, {NULL}};
const Segment SGI290524[] = {{&S42214[54], 4}, {NULL}};
const Segment SGI290525[] = {{&S42214[8], 3}, {NULL}};
const Segment SGI290526[] = {{&S42214[13], 14}, {NULL}};
const Segment SGI290527[] = {{&S42214[7], 1}, {NULL}};
const Segment SGI290528[] = {{&S42214[16], 11}, {NULL}};
const Segment SGI290529[] = {{&S42215[32], 4}, {NULL}};
const Segment SGI290530[] = {{&S42215[8], 3}, {NULL}};
const Segment SGI290531[] = {{&S42215[13], 10}, {NULL}};
const Segment SGI290532[] = {{&S42215[40], 14}, {NULL}};
const Segment SGI290533[] = {{&S42215[7], 1}, {NULL}};
const Segment SGI290534[] = {{&S42215[43], 11}, {NULL}};
const Segment SGI290535[] = {{&S42216[8], 3}, {NULL}};
const Segment SGI290536[] = {{&S42216[13], 7}, {NULL}};
const Segment SGI290537[] = {{&S42216[25], 14}, {NULL}};
const Segment SGI290538[] = {{&S42216[7], 1}, {NULL}};
const Segment SGI290539[] = {{&S42216[16], 4}, {&S42216[28], 11}, {NULL}};
const Segment SGI290540[] = {{&S42458[81], 4}, {NULL}};
const Segment SGI290541[] = {{&S42458[8], 3}, {NULL}};
const Segment SGI290542[] = {{&S42458[13], 9}, {NULL}};
const Segment SGI290543[] = {{&S42458[31], 13}, {NULL}};
const Segment SGI290544[] = {{&S42458[7], 1}, {NULL}};
const Segment SGI290545[] = {{&S42639[55], 4}, {NULL}};
const Segment SGI290546[] = {{&S42639[8], 3}, {NULL}};
const Segment SGI290547[] = {{&S42639[13], 3}, {NULL}};
const Segment SGI290548[] = {{&S42639[7], 1}, {NULL}};
const Segment SGI290549[] = {{&S42639[60], 11}, {NULL}};
const Segment SGI290550[] = {{&S42870[99], 4}, {NULL}};
const Segment SGI290551[] = {{&S42870[8], 3}, {NULL}};
const Segment SGI290552[] = {{&S42870[13], 9}, {NULL}};
const Segment SGI290553[] = {{&S42870[33], 13}, {NULL}};
const Segment SGI290554[] = {{&S42870[7], 1}, {NULL}};
const Segment SGI290555[] = {{&S42870[73], 10}, {NULL}};
const Segment SGI290556[] = {{&S42882[89], 4}, {NULL}};
const Segment SGI290557[] = {{&S42882[8], 3}, {NULL}};
const Segment SGI290558[] = {{&S42882[13], 3}, {NULL}};
const Segment SGI290559[] = {{&S42882[83], 10}, {NULL}};
const Segment SGI290560[] = {{&S42882[7], 1}, {NULL}};
const Segment SGI290561[] = {{&S42882[83], 7}, {NULL}};
const Segment SGI290562[] = {{&S42885[70], 4}, {NULL}};
const Segment SGI290563[] = {{&S42885[8], 3}, {NULL}};
const Segment SGI290564[] = {{&S42885[13], 9}, {NULL}};
const Segment SGI290565[] = {{&S42885[30], 13}, {NULL}};
const Segment SGI290566[] = {{&S42885[7], 1}, {NULL}};
const Segment SGI290567[] = {{&S42890[60], 5}, {NULL}};
const Segment SGI290568[] = {{&S42890[8], 3}, {NULL}};
const Segment SGI290569[] = {{&S42890[13], 14}, {NULL}};
const Segment SGI290570[] = {{&S42890[7], 1}, {NULL}};
const Segment SGI290571[] = {{&S42890[16], 11}, {NULL}};
const Segment SGI290572[] = {{&S42890[68], 13}, {NULL}};
const Segment SGI290573[] = {{&S42891[65], 5}, {NULL}};
const Segment SGI290574[] = {{&S42891[8], 3}, {NULL}};
const Segment SGI290575[] = {{&S42891[13], 14}, {NULL}};
const Segment SGI290576[] = {{&S42891[7], 1}, {NULL}};
const Segment SGI290577[] = {{&S42891[16], 11}, {&S42891[32], 2}, {NULL}};
const Segment SGI290578[] = {{&S42891[77], 13}, {NULL}};
const Segment SGI290579[] = {{&S42962[65], 5}, {NULL}};
const Segment SGI290580[] = {{&S42962[8], 3}, {NULL}};
const Segment SGI290581[] = {{&S42962[13], 14}, {NULL}};
const Segment SGI290582[] = {{&S42962[7], 1}, {NULL}};
const Segment SGI290583[] = {{&S42962[16], 11}, {&S42962[32], 2}, {NULL}};
const Segment SGI290584[] = {{&S42963[53], 5}, {NULL}};
const Segment SGI290585[] = {{&S42963[8], 3}, {NULL}};
const Segment SGI290586[] = {{&S42963[13], 14}, {NULL}};
const Segment SGI290587[] = {{&S42963[7], 1}, {NULL}};
const Segment SGI290588[] = {{&S42963[16], 11}, {NULL}};
const Segment SGI290589[] = {{&S42964[53], 5}, {NULL}};
const Segment SGI290590[] = {{&S42964[8], 3}, {NULL}};
const Segment SGI290591[] = {{&S42964[13], 14}, {NULL}};
const Segment SGI290592[] = {{&S42964[7], 1}, {NULL}};
const Segment SGI290593[] = {{&S42964[16], 11}, {NULL}};
const Segment SGI290594[] = {{&S42964[59], 14}, {NULL}};
const Segment SGI290595[] = {{&S42974[75], 5}, {NULL}};
const Segment SGI290596[] = {{&S42974[8], 3}, {NULL}};
const Segment SGI290597[] = {{&S42974[13], 14}, {NULL}};
const Segment SGI290598[] = {{&S42974[7], 1}, {NULL}};
const Segment SGI290599[] = {{&S42974[16], 11}, {NULL}};
const Segment SGI290600[] = {{&S42974[45], 13}, {NULL}};
const Segment SGI290601[] = {{&S42975[75], 5}, {NULL}};
const Segment SGI290602[] = {{&S42975[8], 3}, {NULL}};
const Segment SGI290603[] = {{&S42975[13], 14}, {NULL}};
const Segment SGI290604[] = {{&S42975[7], 1}, {NULL}};
const Segment SGI290605[] = {{&S42975[16], 11}, {NULL}};
const Segment SGI290606[] = {{&S42975[45], 13}, {NULL}};
const Segment SGI290607[] = {{&S42976[75], 5}, {NULL}};
const Segment SGI290608[] = {{&S42976[8], 3}, {NULL}};
const Segment SGI290609[] = {{&S42976[13], 14}, {NULL}};
const Segment SGI290610[] = {{&S42976[7], 1}, {NULL}};
const Segment SGI290611[] = {{&S42976[16], 11}, {NULL}};
const Segment SGI290612[] = {{&S42976[45], 13}, {NULL}};
const Segment SGI290613[] = {{&S42977[75], 5}, {NULL}};
const Segment SGI290614[] = {{&S42977[8], 3}, {NULL}};
const Segment SGI290615[] = {{&S42977[13], 14}, {NULL}};
const Segment SGI290616[] = {{&S42977[7], 1}, {NULL}};
const Segment SGI290617[] = {{&S42977[16], 11}, {NULL}};
const Segment SGI290618[] = {{&S42977[45], 13}, {NULL}};
const Segment SGI290619[] = {{&S42978[75], 5}, {NULL}};
const Segment SGI290620[] = {{&S42978[8], 3}, {NULL}};
const Segment SGI290621[] = {{&S42978[13], 14}, {NULL}};
const Segment SGI290622[] = {{&S42978[7], 1}, {NULL}};
const Segment SGI290623[] = {{&S42978[16], 11}, {NULL}};
const Segment SGI290624[] = {{&S42978[45], 13}, {NULL}};
const Segment SGI290625[] = {{&S42979[75], 5}, {NULL}};
const Segment SGI290626[] = {{&S42979[8], 3}, {NULL}};
const Segment SGI290627[] = {{&S42979[13], 14}, {NULL}};
const Segment SGI290628[] = {{&S42979[7], 1}, {NULL}};
const Segment SGI290629[] = {{&S42979[16], 11}, {NULL}};
const Segment SGI290630[] = {{&S42979[45], 13}, {NULL}};
const Segment SGI290631[] = {{&S42980[75], 5}, {NULL}};
const Segment SGI290632[] = {{&S42980[8], 3}, {NULL}};
const Segment SGI290633[] = {{&S42980[13], 14}, {NULL}};
const Segment SGI290634[] = {{&S42980[7], 1}, {NULL}};
const Segment SGI290635[] = {{&S42980[16], 11}, {NULL}};
const Segment SGI290636[] = {{&S42980[45], 13}, {NULL}};
const Segment SGI290637[] = {{&S42981[75], 5}, {NULL}};
const Segment SGI290638[] = {{&S42981[8], 3}, {NULL}};
const Segment SGI290639[] = {{&S42981[13], 14}, {NULL}};
const Segment SGI290640[] = {{&S42981[7], 1}, {NULL}};
const Segment SGI290641[] = {{&S42981[16], 11}, {NULL}};
const Segment SGI290642[] = {{&S42981[45], 13}, {NULL}};
const Segment SGI290643[] = {{&S42982[75], 5}, {NULL}};
const Segment SGI290644[] = {{&S42982[8], 3}, {NULL}};
const Segment SGI290645[] = {{&S42982[13], 14}, {NULL}};
const Segment SGI290646[] = {{&S42982[7], 1}, {NULL}};
const Segment SGI290647[] = {{&S42982[16], 11}, {NULL}};
const Segment SGI290648[] = {{&S42982[45], 13}, {NULL}};
const Segment SGI290649[] = {{&S42983[70], 4}, {NULL}};
const Segment SGI290650[] = {{&S42983[8], 3}, {NULL}};
const Segment SGI290651[] = {{&S42983[13], 9}, {NULL}};
const Segment SGI290652[] = {{&S42983[30], 13}, {NULL}};
const Segment SGI290653[] = {{&S42983[7], 1}, {NULL}};
const Segment SGI290654[] = {{&S43364[56], 4}, {NULL}};
const Segment SGI290655[] = {{&S43364[8], 3}, {NULL}};
const Segment SGI290656[] = {{&S43364[13], 14}, {NULL}};
const Segment SGI290657[] = {{&S43364[7], 1}, {NULL}};
const Segment SGI290658[] = {{&S43364[16], 11}, {NULL}};
const Segment SGI290659[] = {{&S43364[61], 11}, {NULL}};
const Segment SGI290660[] = {{&S43420[65], 5}, {NULL}};
const Segment SGI290661[] = {{&S43420[8], 3}, {NULL}};
const Segment SGI290662[] = {{&S43420[13], 14}, {NULL}};
const Segment SGI290663[] = {{&S43420[7], 1}, {NULL}};
const Segment SGI290664[] = {{&S43420[16], 11}, {&S43420[32], 2}, {NULL}};
const Segment SGI290665[] = {{&S43420[75], 13}, {NULL}};
const Segment SGI290666[] = {{&S43421[60], 5}, {NULL}};
const Segment SGI290667[] = {{&S43421[8], 3}, {NULL}};
const Segment SGI290668[] = {{&S43421[13], 14}, {NULL}};
const Segment SGI290669[] = {{&S43421[7], 1}, {NULL}};
const Segment SGI290670[] = {{&S43421[16], 11}, {NULL}};
const Segment SGI290671[] = {{&S43421[66], 13}, {NULL}};
const Segment SGI290672[] = {{&S43509[64], 4}, {NULL}};
const Segment SGI290673[] = {{&S43509[8], 3}, {NULL}};
const Segment SGI290674[] = {{&S43509[13], 3}, {NULL}};
const Segment SGI290675[] = {{&S43509[7], 1}, {NULL}};
const Segment SGI290676[] = {{&S43509[71], 10}, {NULL}};
const Segment SGI290677[] = {{&S43553[55], 5}, {NULL}};
const Segment SGI290678[] = {{&S43553[8], 3}, {NULL}};
const Segment SGI290679[] = {{&S43553[13], 14}, {NULL}};
const Segment SGI290680[] = {{&S43553[7], 1}, {NULL}};
const Segment SGI290681[] = {{&S43553[16], 11}, {&S43553[32], 2}, {NULL}};
const Segment SGI290682[] = {{&S43554[50], 5}, {NULL}};
const Segment SGI290683[] = {{&S43554[8], 3}, {NULL}};
const Segment SGI290684[] = {{&S43554[13], 14}, {NULL}};
const Segment SGI290685[] = {{&S43554[7], 1}, {NULL}};
const Segment SGI290686[] = {{&S43554[16], 11}, {NULL}};
const Segment SGI290687[] = {{&S43567[70], 4}, {NULL}};
const Segment SGI290688[] = {{&S43567[8], 3}, {NULL}};
const Segment SGI290689[] = {{&S43567[13], 9}, {NULL}};
const Segment SGI290690[] = {{&S43567[30], 13}, {NULL}};
const Segment SGI290691[] = {{&S43567[7], 1}, {NULL}};
const Segment SGI290692[] = {{&S43568[65], 5}, {NULL}};
const Segment SGI290693[] = {{&S43568[8], 3}, {NULL}};
const Segment SGI290694[] = {{&S43568[13], 14}, {NULL}};
const Segment SGI290695[] = {{&S43568[7], 1}, {NULL}};
const Segment SGI290696[] = {{&S43568[16], 11}, {&S43568[32], 2}, {NULL}};
const Segment SGI290697[] = {{&S43569[60], 5}, {NULL}};
const Segment SGI290698[] = {{&S43569[8], 3}, {NULL}};
const Segment SGI290699[] = {{&S43569[13], 14}, {NULL}};
const Segment SGI290700[] = {{&S43569[7], 1}, {NULL}};
const Segment SGI290701[] = {{&S43569[16], 11}, {NULL}};
const Segment SGI290702[] = {{&S43594[90], 4}, {NULL}};
const Segment SGI290703[] = {{&S43594[8], 3}, {NULL}};
const Segment SGI290704[] = {{&S43594[13], 3}, {NULL}};
const Segment SGI290705[] = {{&S43594[13], 3}, {&S43594[60], 6}, {NULL}};
const Segment SGI290706[] = {{&S43594[60], 11}, {NULL}};
const Segment SGI290707[] = {{&S43594[7], 1}, {NULL}};
const Segment SGI290708[] = {{&S43612[67], 5}, {NULL}};
const Segment SGI290709[] = {{&S43612[8], 3}, {NULL}};
const Segment SGI290710[] = {{&S43612[13], 14}, {NULL}};
const Segment SGI290711[] = {{&S43612[7], 1}, {NULL}};
const Segment SGI290712[] = {{&S43612[16], 11}, {&S43612[32], 2}, {NULL}};
const Segment SGI290713[] = {{&S43613[57], 5}, {NULL}};
const Segment SGI290714[] = {{&S43613[8], 3}, {NULL}};
const Segment SGI290715[] = {{&S43613[13], 14}, {NULL}};
const Segment SGI290716[] = {{&S43613[7], 1}, {NULL}};
const Segment SGI290717[] = {{&S43613[16], 11}, {NULL}};
const Segment SGI290718[] = {{&S43652[53], 5}, {NULL}};
const Segment SGI290719[] = {{&S43652[8], 3}, {NULL}};
const Segment SGI290720[] = {{&S43652[13], 14}, {NULL}};
const Segment SGI290721[] = {{&S43652[7], 1}, {NULL}};
const Segment SGI290722[] = {{&S43652[16], 11}, {NULL}};
const Segment SGI290723[] = {{&S43653[65], 5}, {NULL}};
const Segment SGI290724[] = {{&S43653[8], 3}, {NULL}};
const Segment SGI290725[] = {{&S43653[13], 14}, {NULL}};
const Segment SGI290726[] = {{&S43653[7], 1}, {NULL}};
const Segment SGI290727[] = {{&S43653[16], 11}, {&S43653[32], 2}, {NULL}};
const Segment SGI290728[] = {{&S43914[55], 4}, {NULL}};
const Segment SGI290729[] = {{&S43914[8], 3}, {NULL}};
const Segment SGI290730[] = {{&S43914[13], 3}, {NULL}};
const Segment SGI290731[] = {{&S43914[7], 1}, {NULL}};
const Segment SGI290732[] = {{&S43914[60], 11}, {NULL}};
const Segment SGI290733[] = {{&S43934[60], 5}, {NULL}};
const Segment SGI290734[] = {{&S43934[8], 3}, {NULL}};
const Segment SGI290735[] = {{&S43934[13], 14}, {NULL}};
const Segment SGI290736[] = {{&S43934[7], 1}, {NULL}};
const Segment SGI290737[] = {{&S43934[16], 11}, {NULL}};
const Segment SGI290738[] = {{&S43934[66], 13}, {NULL}};
const Segment SGI290739[] = {{&S44340[60], 5}, {NULL}};
const Segment SGI290740[] = {{&S44340[8], 3}, {NULL}};
const Segment SGI290741[] = {{&S44340[13], 14}, {NULL}};
const Segment SGI290742[] = {{&S44340[7], 1}, {NULL}};
const Segment SGI290743[] = {{&S44340[16], 11}, {NULL}};
const Segment SGI290744[] = {{&S44340[66], 13}, {NULL}};
const Segment SGI290745[] = {{&S44422[65], 5}, {NULL}};
const Segment SGI290746[] = {{&S44422[8], 3}, {NULL}};
const Segment SGI290747[] = {{&S44422[13], 14}, {NULL}};
const Segment SGI290748[] = {{&S44422[7], 1}, {NULL}};
const Segment SGI290749[] = {{&S44422[16], 11}, {&S44422[32], 2}, {NULL}};
const Segment SGI290750[] = {{&S44423[53], 5}, {NULL}};
const Segment SGI290751[] = {{&S44423[8], 3}, {NULL}};
const Segment SGI290752[] = {{&S44423[13], 14}, {NULL}};
const Segment SGI290753[] = {{&S44423[7], 1}, {NULL}};
const Segment SGI290754[] = {{&S44423[16], 11}, {NULL}};
const Segment SGI290755[] = {{&S44440[69], 5}, {NULL}};
const Segment SGI290756[] = {{&S44440[8], 3}, {NULL}};
const Segment SGI290757[] = {{&S44440[13], 9}, {NULL}};
const Segment SGI290758[] = {{&S44440[30], 13}, {NULL}};
const Segment SGI290759[] = {{&S44440[7], 1}, {NULL}};
const Segment SGI290760[] = {{&S44444[69], 4}, {NULL}};
const Segment SGI290761[] = {{&S44444[8], 3}, {NULL}};
const Segment SGI290762[] = {{&S44444[13], 3}, {NULL}};
const Segment SGI290763[] = {{&S44444[13], 3}, {&S44444[21], 7}, {NULL}};
const Segment SGI290764[] = {{&S44444[7], 1}, {NULL}};
const Segment SGI290765[] = {{&S44543[53], 5}, {NULL}};
const Segment SGI290766[] = {{&S44543[8], 3}, {NULL}};
const Segment SGI290767[] = {{&S44543[13], 14}, {NULL}};
const Segment SGI290768[] = {{&S44543[7], 1}, {NULL}};
const Segment SGI290769[] = {{&S44543[16], 11}, {NULL}};
const Segment SGI290770[] = {{&S44543[59], 13}, {NULL}};
const Segment SGI290771[] = {{&S44544[60], 5}, {NULL}};
const Segment SGI290772[] = {{&S44544[8], 3}, {NULL}};
const Segment SGI290773[] = {{&S44544[13], 14}, {NULL}};
const Segment SGI290774[] = {{&S44544[7], 1}, {NULL}};
const Segment SGI290775[] = {{&S44544[16], 11}, {NULL}};
const Segment SGI290776[] = {{&S44544[66], 13}, {NULL}};
const Segment SGI290777[] = {{&S44571[53], 4}, {NULL}};
const Segment SGI290778[] = {{&S44571[8], 3}, {NULL}};
const Segment SGI290779[] = {{&S44571[13], 3}, {NULL}};
const Segment SGI290780[] = {{&S44571[7], 1}, {NULL}};
const Segment SGI290781[] = {{&S44571[58], 11}, {NULL}};
const Segment SGI290782[] = {{&S44583[70], 4}, {NULL}};
const Segment SGI290783[] = {{&S44583[8], 3}, {NULL}};
const Segment SGI290784[] = {{&S44583[13], 9}, {NULL}};
const Segment SGI290785[] = {{&S44583[30], 13}, {NULL}};
const Segment SGI290786[] = {{&S44583[7], 1}, {NULL}};
const Segment SGI290787[] = {{&S44628[56], 4}, {NULL}};
const Segment SGI290788[] = {{&S44628[8], 3}, {NULL}};
const Segment SGI290789[] = {{&S44628[13], 14}, {NULL}};
const Segment SGI290790[] = {{&S44628[7], 1}, {NULL}};
const Segment SGI290791[] = {{&S44628[16], 11}, {NULL}};
const Segment SGI290792[] = {{&S44628[61], 11}, {NULL}};
const Segment SGI290793[] = {{&S44644[65], 5}, {NULL}};
const Segment SGI290794[] = {{&S44644[8], 3}, {NULL}};
const Segment SGI290795[] = {{&S44644[13], 14}, {NULL}};
const Segment SGI290796[] = {{&S44644[7], 1}, {NULL}};
const Segment SGI290797[] = {{&S44644[16], 11}, {&S44644[32], 2}, {NULL}};
const Segment SGI290798[] = {{&S44644[71], 13}, {NULL}};
const Segment SGI290799[] = {{&S44645[60], 5}, {NULL}};
const Segment SGI290800[] = {{&S44645[8], 3}, {NULL}};
const Segment SGI290801[] = {{&S44645[13], 14}, {NULL}};
const Segment SGI290802[] = {{&S44645[7], 1}, {NULL}};
const Segment SGI290803[] = {{&S44645[16], 11}, {NULL}};
const Segment SGI290804[] = {{&S44645[66], 13}, {NULL}};
const Segment SGI290805[] = {{&S43261[8], 3}, {NULL}};
const Segment SGI290806[] = {{&S43261[33], 6}, {NULL}};
const Segment SGI290807[] = {{&S43261[69], 6}, {NULL}};
const Segment SGI290808[] = {{&S43261[13], 26}, {NULL}};
const Segment SGI290809[] = {{&S43261[33], 2}, {NULL}};
const Segment SGI290810[] = {{&S43262[8], 3}, {NULL}};
const Segment SGI290811[] = {{&S43262[33], 6}, {NULL}};
const Segment SGI290812[] = {{&S43262[69], 6}, {NULL}};
const Segment SGI290813[] = {{&S43262[13], 26}, {NULL}};
const Segment SGI290814[] = {{&S43262[33], 2}, {NULL}};
const Segment SGI290815[] = {{&S44726[53], 5}, {NULL}};
const Segment SGI290816[] = {{&S44726[8], 3}, {NULL}};
const Segment SGI290817[] = {{&S44726[13], 14}, {NULL}};
const Segment SGI290818[] = {{&S44726[7], 1}, {NULL}};
const Segment SGI290819[] = {{&S44726[16], 11}, {NULL}};
const Segment SGI290820[] = {{&S44727[65], 5}, {NULL}};
const Segment SGI290821[] = {{&S44727[8], 3}, {NULL}};
const Segment SGI290822[] = {{&S44727[13], 14}, {NULL}};
const Segment SGI290823[] = {{&S44727[7], 1}, {NULL}};
const Segment SGI290824[] = {{&S44727[16], 11}, {&S44727[32], 2}, {NULL}};
const Segment SGI290825[] = {{&S44728[60], 5}, {&S44728[60], 5}, {NULL}};
const Segment SGI290826[] = {{&S44728[8], 3}, {NULL}};
const Segment SGI290827[] = {{&S44728[13], 14}, {NULL}};
const Segment SGI290828[] = {{&S44728[7], 1}, {&S44728[7], 1}, {NULL}};
const Segment SGI290829[] = {{&S44728[16], 11}, {NULL}};
const Segment SGI290830[] = {{&S44822[53], 5}, {NULL}};
const Segment SGI290831[] = {{&S44822[8], 3}, {NULL}};
const Segment SGI290832[] = {{&S44822[13], 14}, {NULL}};
const Segment SGI290833[] = {{&S44822[7], 1}, {NULL}};
const Segment SGI290834[] = {{&S44822[16], 11}, {NULL}};
const Segment SGI290835[] = {{&S44822[59], 14}, {NULL}};
const Segment SGI290836[] = {{&S44823[53], 5}, {NULL}};
const Segment SGI290837[] = {{&S44823[8], 3}, {NULL}};
const Segment SGI290838[] = {{&S44823[13], 14}, {NULL}};
const Segment SGI290839[] = {{&S44823[7], 1}, {NULL}};
const Segment SGI290840[] = {{&S44823[16], 11}, {NULL}};
const Segment SGI290841[] = {{&S44823[59], 17}, {NULL}};
const Segment SGI290842[] = {{&S44824[60], 5}, {NULL}};
const Segment SGI290843[] = {{&S44824[8], 3}, {NULL}};
const Segment SGI290844[] = {{&S44824[13], 14}, {NULL}};
const Segment SGI290845[] = {{&S44824[7], 1}, {NULL}};
const Segment SGI290846[] = {{&S44824[16], 11}, {NULL}};
const Segment SGI290847[] = {{&S44824[66], 14}, {NULL}};
const Segment SGI290848[] = {{&S44958[75], 4}, {NULL}};
const Segment SGI290849[] = {{&S44958[8], 3}, {NULL}};
const Segment SGI290850[] = {{&S44958[13], 7}, {NULL}};
const Segment SGI290851[] = {{&S44958[25], 14}, {NULL}};
const Segment SGI290852[] = {{&S44958[7], 1}, {NULL}};
const Segment SGI290853[] = {{&S44958[16], 4}, {&S44958[28], 11}, {NULL}};
const Segment SGI290854[] = {{&S44959[61], 4}, {NULL}};
const Segment SGI290855[] = {{&S44959[8], 3}, {NULL}};
const Segment SGI290856[] = {{&S44959[13], 14}, {NULL}};
const Segment SGI290857[] = {{&S44959[7], 1}, {NULL}};
const Segment SGI290858[] = {{&S44959[16], 11}, {NULL}};
const Segment SGI290859[] = {{&S45101[79], 4}, {NULL}};
const Segment SGI290860[] = {{&S45101[8], 3}, {NULL}};
const Segment SGI290861[] = {{&S45101[13], 9}, {NULL}};
const Segment SGI290862[] = {{&S45101[33], 13}, {NULL}};
const Segment SGI290863[] = {{&S45101[7], 1}, {NULL}};
const Segment SGI290864[] = {{&S45119[66], 4}, {NULL}};
const Segment SGI290865[] = {{&S45119[8], 3}, {NULL}};
const Segment SGI290866[] = {{&S45119[13], 14}, {NULL}};
const Segment SGI290867[] = {{&S45119[7], 1}, {NULL}};
const Segment SGI290868[] = {{&S45119[16], 11}, {&S45119[32], 2}, {NULL}};
const Segment SGI290869[] = {{&S45120[67], 4}, {NULL}};
const Segment SGI290870[] = {{&S45120[8], 3}, {NULL}};
const Segment SGI290871[] = {{&S45120[13], 14}, {NULL}};
const Segment SGI290872[] = {{&S45120[7], 1}, {NULL}};
const Segment SGI290873[] = {{&S45120[16], 11}, {&S45120[32], 2}, {NULL}};
const Segment SGI290874[] = {{&S45121[61], 4}, {NULL}};
const Segment SGI290875[] = {{&S45121[8], 3}, {NULL}};
const Segment SGI290876[] = {{&S45121[13], 14}, {NULL}};
const Segment SGI290877[] = {{&S45121[7], 1}, {NULL}};
const Segment SGI290878[] = {{&S45121[16], 11}, {NULL}};
const Segment SGI290879[] = {{&S45122[48], 4}, {NULL}};
const Segment SGI290880[] = {{&S45122[8], 3}, {NULL}};
const Segment SGI290881[] = {{&S45122[7], 1}, {NULL}};
const Segment SGI290882[] = {{&S45122[15], 9}, {NULL}};
const Segment SGI290883[] = {{&S45124[8], 3}, {NULL}};
const Segment SGI290884[] = {{&S45124[13], 7}, {NULL}};
const Segment SGI290885[] = {{&S45124[25], 14}, {NULL}};
const Segment SGI290886[] = {{&S45124[7], 1}, {NULL}};
const Segment SGI290887[] = {{&S45124[16], 4}, {&S45124[28], 11}, {NULL}};
const Segment SGI290888[] = {{&S45125[59], 4}, {&S45125[59], 4}, {NULL}};
const Segment SGI290889[] = {{&S45125[8], 3}, {NULL}};
const Segment SGI290890[] = {{&S45125[13], 14}, {NULL}};
const Segment SGI290891[] = {{&S45125[7], 1}, {&S45125[7], 1}, {NULL}};
const Segment SGI290892[] = {{&S45125[16], 11}, {NULL}};
const Segment SGI290893[] = {{&S45126[73], 4}, {NULL}};
const Segment SGI290894[] = {{&S45126[8], 3}, {NULL}};
const Segment SGI290895[] = {{&S45126[13], 7}, {NULL}};
const Segment SGI290896[] = {{&S45126[25], 14}, {NULL}};
const Segment SGI290897[] = {{&S45126[7], 1}, {NULL}};
const Segment SGI290898[] = {{&S45126[16], 4}, {&S45126[28], 11}, {NULL}};
const Segment SGI290899[] = {{&S45176[56], 5}, {NULL}};
const Segment SGI290900[] = {{&S45176[8], 3}, {NULL}};
const Segment SGI290901[] = {{&S45176[13], 3}, {NULL}};
const Segment SGI290902[] = {{&S45176[7], 1}, {NULL}};
const Segment SGI290903[] = {{&S45177[64], 5}, {NULL}};
const Segment SGI290904[] = {{&S45177[8], 3}, {NULL}};
const Segment SGI290905[] = {{&S45177[13], 3}, {NULL}};
const Segment SGI290906[] = {{&S45177[7], 1}, {NULL}};
const Segment SGI290907[] = {{&S45178[58], 5}, {NULL}};
const Segment SGI290908[] = {{&S45178[8], 3}, {NULL}};
const Segment SGI290909[] = {{&S45178[13], 3}, {NULL}};
const Segment SGI290910[] = {{&S45183[64], 5}, {NULL}};
const Segment SGI290911[] = {{&S45183[8], 3}, {NULL}};
const Segment SGI290912[] = {{&S45183[13], 3}, {NULL}};
const Segment SGI290913[] = {{&S45183[7], 1}, {NULL}};
const Segment SGI290914[] = {{&S45184[56], 5}, {NULL}};
const Segment SGI290915[] = {{&S45184[8], 3}, {NULL}};
const Segment SGI290916[] = {{&S45184[13], 3}, {NULL}};
const Segment SGI290917[] = {{&S45184[7], 1}, {NULL}};
const Segment SGI290918[] = {{&S45185[112], 5}, {NULL}};
const Segment SGI290919[] = {{&S45185[8], 3}, {&S45185[8], 3}, {NULL}};
const Segment SGI290920[] = {{&S45185[13], 3}, {&S45185[13], 3}, {NULL}};
const Segment SGI290921[] = {{&S45185[7], 1}, {NULL}};
const Segment SGI290922[] = {{&S45186[58], 5}, {NULL}};
const Segment SGI290923[] = {{&S45186[8], 3}, {NULL}};
const Segment SGI290924[] = {{&S45186[13], 3}, {NULL}};
const Segment SGI290925[] = {{&S45340[62], 4}, {NULL}};
const Segment SGI290926[] = {{&S45340[8], 3}, {NULL}};
const Segment SGI290927[] = {{&S45340[13], 3}, {NULL}};
const Segment SGI290928[] = {{&S45340[7], 1}, {NULL}};
const Segment SGI290929[] = {{&S45340[69], 10}, {NULL}};
const Segment SGI290930[] = {{&S45398[60], 5}, {NULL}};
const Segment SGI290931[] = {{&S45398[8], 3}, {NULL}};
const Segment SGI290932[] = {{&S45398[13], 14}, {NULL}};
const Segment SGI290933[] = {{&S45398[7], 1}, {NULL}};
const Segment SGI290934[] = {{&S45398[16], 11}, {NULL}};
const Segment SGI290935[] = {{&S45398[66], 13}, {NULL}};
const Segment SGI290936[] = {{&S45399[53], 5}, {NULL}};
const Segment SGI290937[] = {{&S45399[8], 3}, {NULL}};
const Segment SGI290938[] = {{&S45399[13], 14}, {NULL}};
const Segment SGI290939[] = {{&S45399[7], 1}, {NULL}};
const Segment SGI290940[] = {{&S45399[16], 11}, {NULL}};
const Segment SGI290941[] = {{&S45399[59], 13}, {NULL}};
const Segment SGI290942[] = {{&S45414[75], 4}, {NULL}};
const Segment SGI290943[] = {{&S45414[8], 3}, {NULL}};
const Segment SGI290944[] = {{&S45414[13], 9}, {NULL}};
const Segment SGI290945[] = {{&S45414[7], 1}, {NULL}};
const Segment SGI290946[] = {{&S45415[8], 3}, {NULL}};
const Segment SGI290947[] = {{&S45415[13], 9}, {NULL}};
const Segment SGI290948[] = {{&S45415[31], 15}, {NULL}};
const Segment SGI290949[] = {{&S45415[75], 11}, {NULL}};
const Segment SGI290950[] = {{&S45416[78], 4}, {NULL}};
const Segment SGI290951[] = {{&S45416[8], 3}, {NULL}};
const Segment SGI290952[] = {{&S45416[13], 9}, {NULL}};
const Segment SGI290953[] = {{&S45416[31], 15}, {NULL}};
const Segment SGI290954[] = {{&S45416[7], 1}, {NULL}};
const Segment SGI290955[] = {{&S45417[7], 4}, {NULL}};
const Segment SGI290956[] = {{&S45417[8], 3}, {NULL}};
const Segment SGI290957[] = {{&S45417[13], 9}, {NULL}};
const Segment SGI290958[] = {{&S45417[30], 13}, {NULL}};
const Segment SGI290959[] = {{&S45417[7], 1}, {NULL}};
const Segment SGI290960[] = {{&S45560[60], 5}, {NULL}};
const Segment SGI290961[] = {{&S45560[8], 3}, {NULL}};
const Segment SGI290962[] = {{&S45560[13], 14}, {NULL}};
const Segment SGI290963[] = {{&S45560[7], 1}, {NULL}};
const Segment SGI290964[] = {{&S45560[16], 11}, {NULL}};
const Segment SGI290965[] = {{&S45560[66], 13}, {NULL}};
const Segment SGI290966[] = {{&S45629[99], 4}, {NULL}};
const Segment SGI290967[] = {{&S45629[8], 3}, {NULL}};
const Segment SGI290968[] = {{&S45629[13], 9}, {NULL}};
const Segment SGI290969[] = {{&S45629[33], 13}, {NULL}};
const Segment SGI290970[] = {{&S45629[7], 1}, {NULL}};
const Segment SGI290971[] = {{&S45629[73], 10}, {NULL}};
const Segment SGI290972[] = {{&S45630[69], 5}, {NULL}};
const Segment SGI290973[] = {{&S45630[8], 3}, {NULL}};
const Segment SGI290974[] = {{&S45630[13], 9}, {NULL}};
const Segment SGI290975[] = {{&S45630[30], 13}, {NULL}};
const Segment SGI290976[] = {{&S45630[7], 1}, {NULL}};
const Segment SGI290977[] = {{&S45631[92], 4}, {NULL}};
const Segment SGI290978[] = {{&S45631[8], 3}, {NULL}};
const Segment SGI290979[] = {{&S45631[13], 9}, {NULL}};
const Segment SGI290980[] = {{&S45631[33], 13}, {NULL}};
const Segment SGI290981[] = {{&S45631[7], 1}, {NULL}};
const Segment SGI290982[] = {{&S45632[80], 4}, {NULL}};
const Segment SGI290983[] = {{&S45632[8], 3}, {NULL}};
const Segment SGI290984[] = {{&S45632[13], 9}, {NULL}};
const Segment SGI290985[] = {{&S45632[33], 15}, {NULL}};
const Segment SGI290986[] = {{&S45632[7], 1}, {NULL}};
const Segment SGI290987[] = {{&S45741[52], 4}, {NULL}};
const Segment SGI290988[] = {{&S45741[8], 3}, {NULL}};
const Segment SGI290989[] = {{&S45741[13], 14}, {NULL}};
const Segment SGI290990[] = {{&S45741[7], 1}, {NULL}};
const Segment SGI290991[] = {{&S45741[16], 11}, {NULL}};
const Segment SGI290992[] = {{&S45742[70], 4}, {NULL}};
const Segment SGI290993[] = {{&S45742[8], 3}, {NULL}};
const Segment SGI290994[] = {{&S45742[13], 14}, {NULL}};
const Segment SGI290995[] = {{&S45742[7], 1}, {NULL}};
const Segment SGI290996[] = {{&S45742[16], 11}, {&S45742[32], 2}, {NULL}};
const Segment SGI290997[] = {{&S45743[80], 4}, {NULL}};
const Segment SGI290998[] = {{&S45743[8], 3}, {NULL}};
const Segment SGI290999[] = {{&S45743[13], 7}, {NULL}};
const Segment SGI291000[] = {{&S45743[25], 14}, {NULL}};
const Segment SGI291001[] = {{&S45743[7], 1}, {NULL}};
const Segment SGI291002[] = {{&S45743[16], 4}, {&S45743[28], 11}, {NULL}};
const Segment SGI291003[] = {{&S45801[99], 4}, {NULL}};
const Segment SGI291004[] = {{&S45801[8], 3}, {NULL}};
const Segment SGI291005[] = {{&S45801[13], 9}, {NULL}};
const Segment SGI291006[] = {{&S45801[33], 13}, {NULL}};
const Segment SGI291007[] = {{&S45801[7], 1}, {NULL}};
const Segment SGI291008[] = {{&S45801[73], 10}, {NULL}};
const Segment SGI291009[] = {{&S45802[99], 5}, {NULL}};
const Segment SGI291010[] = {{&S45802[8], 3}, {NULL}};
const Segment SGI291011[] = {{&S45802[13], 9}, {NULL}};
const Segment SGI291012[] = {{&S45802[33], 13}, {NULL}};
const Segment SGI291013[] = {{&S45802[7], 1}, {NULL}};
const Segment SGI291014[] = {{&S45802[73], 10}, {NULL}};
const Segment SGI291015[] = {{&S45826[53], 5}, {NULL}};
const Segment SGI291016[] = {{&S45826[8], 3}, {NULL}};
const Segment SGI291017[] = {{&S45826[13], 14}, {NULL}};
const Segment SGI291018[] = {{&S45826[7], 1}, {NULL}};
const Segment SGI291019[] = {{&S45826[16], 11}, {NULL}};
const Segment SGI291020[] = {{&S45826[59], 13}, {NULL}};
const Segment SGI291021[] = {{&S45827[60], 5}, {NULL}};
const Segment SGI291022[] = {{&S45827[8], 3}, {NULL}};
const Segment SGI291023[] = {{&S45827[13], 14}, {NULL}};
const Segment SGI291024[] = {{&S45827[7], 1}, {NULL}};
const Segment SGI291025[] = {{&S45827[16], 11}, {NULL}};
const Segment SGI291026[] = {{&S45827[66], 13}, {NULL}};
const Segment SGI291027[] = {{&S45867[56], 5}, {NULL}};
const Segment SGI291028[] = {{&S45867[8], 3}, {NULL}};
const Segment SGI291029[] = {{&S45867[13], 3}, {NULL}};
const Segment SGI291030[] = {{&S45867[7], 1}, {NULL}};
const Segment SGI291031[] = {{&S45868[64], 5}, {NULL}};
const Segment SGI291032[] = {{&S45868[8], 3}, {NULL}};
const Segment SGI291033[] = {{&S45868[13], 3}, {NULL}};
const Segment SGI291034[] = {{&S45868[7], 1}, {NULL}};
const Segment SGI291035[] = {{&S45869[58], 5}, {NULL}};
const Segment SGI291036[] = {{&S45869[8], 3}, {NULL}};
const Segment SGI291037[] = {{&S45869[13], 3}, {NULL}};
const Segment SGI291038[] = {{&S45916[57], 4}, {NULL}};
const Segment SGI291039[] = {{&S45916[8], 3}, {NULL}};
const Segment SGI291040[] = {{&S45916[13], 3}, {NULL}};
const Segment SGI291041[] = {{&S45916[7], 1}, {NULL}};
const Segment SGI291042[] = {{&S45916[62], 11}, {NULL}};
const Segment SGI291043[] = {{&S45950[61], 4}, {NULL}};
const Segment SGI291044[] = {{&S45950[8], 3}, {NULL}};
const Segment SGI291045[] = {{&S45950[13], 14}, {NULL}};
const Segment SGI291046[] = {{&S45950[7], 1}, {NULL}};
const Segment SGI291047[] = {{&S45950[16], 11}, {NULL}};
const Segment SGI291048[] = {{&S45950[68], 10}, {NULL}};
const Segment SGI291049[] = {{&S46040[70], 5}, {NULL}};
const Segment SGI291050[] = {{&S46040[8], 3}, {NULL}};
const Segment SGI291051[] = {{&S46040[13], 9}, {NULL}};
const Segment SGI291052[] = {{&S46040[30], 13}, {NULL}};
const Segment SGI291053[] = {{&S46040[7], 1}, {NULL}};
const Segment SGI291054[] = {{&S46089[70], 4}, {NULL}};
const Segment SGI291055[] = {{&S46089[8], 3}, {NULL}};
const Segment SGI291056[] = {{&S46089[13], 9}, {NULL}};
const Segment SGI291057[] = {{&S46089[30], 13}, {NULL}};
const Segment SGI291058[] = {{&S46089[7], 1}, {NULL}};
const Segment SGI291059[] = {{&S46348[55], 4}, {NULL}};
const Segment SGI291060[] = {{&S46348[8], 3}, {NULL}};
const Segment SGI291061[] = {{&S46348[13], 3}, {NULL}};
const Segment SGI291062[] = {{&S46348[7], 1}, {NULL}};
const Segment SGI291063[] = {{&S46348[60], 11}, {NULL}};
const Segment SGI291064[] = {{&S46470[53], 4}, {NULL}};
const Segment SGI291065[] = {{&S46470[8], 3}, {NULL}};
const Segment SGI291066[] = {{&S46470[13], 3}, {NULL}};
const Segment SGI291067[] = {{&S46470[7], 1}, {NULL}};
const Segment SGI291068[] = {{&S46471[56], 4}, {NULL}};
const Segment SGI291069[] = {{&S46471[8], 3}, {NULL}};
const Segment SGI291070[] = {{&S46471[14], 3}, {NULL}};
const Segment SGI291071[] = {{&S46471[7], 1}, {NULL}};
const Segment SGI291072[] = {{&S46472[63], 4}, {NULL}};
const Segment SGI291073[] = {{&S46472[8], 3}, {NULL}};
const Segment SGI291074[] = {{&S46472[13], 3}, {NULL}};
const Segment SGI291075[] = {{&S46472[7], 1}, {NULL}};
const Segment SGI291076[] = {{&S46648[60], 5}, {NULL}};
const Segment SGI291077[] = {{&S46648[8], 3}, {NULL}};
const Segment SGI291078[] = {{&S46648[13], 14}, {NULL}};
const Segment SGI291079[] = {{&S46648[7], 1}, {NULL}};
const Segment SGI291080[] = {{&S46648[16], 11}, {NULL}};
const Segment SGI291081[] = {{&S46648[66], 13}, {NULL}};
const Segment SGI291082[] = {{&S46650[65], 5}, {NULL}};
const Segment SGI291083[] = {{&S46650[8], 3}, {NULL}};
const Segment SGI291084[] = {{&S46650[13], 14}, {NULL}};
const Segment SGI291085[] = {{&S46650[7], 1}, {NULL}};
const Segment SGI291086[] = {{&S46650[16], 11}, {&S46650[32], 2}, {NULL}};
const Segment SGI291087[] = {{&S46650[71], 13}, {NULL}};
const Segment SGI291088[] = {{&S46746[70], 4}, {NULL}};
const Segment SGI291089[] = {{&S46746[8], 3}, {NULL}};
const Segment SGI291090[] = {{&S46746[13], 9}, {NULL}};
const Segment SGI291091[] = {{&S46746[30], 13}, {NULL}};
const Segment SGI291092[] = {{&S46746[7], 1}, {NULL}};
const Segment SGI291093[] = {{&S46859[53], 5}, {NULL}};
const Segment SGI291094[] = {{&S46859[8], 3}, {NULL}};
const Segment SGI291095[] = {{&S46859[13], 14}, {NULL}};
const Segment SGI291096[] = {{&S46859[7], 1}, {NULL}};
const Segment SGI291097[] = {{&S46859[16], 11}, {NULL}};
const Segment SGI291098[] = {{&S46859[59], 13}, {NULL}};
const Segment SGI291099[] = {{&S46860[60], 5}, {NULL}};
const Segment SGI291100[] = {{&S46860[8], 3}, {NULL}};
const Segment SGI291101[] = {{&S46860[13], 14}, {NULL}};
const Segment SGI291102[] = {{&S46860[7], 1}, {NULL}};
const Segment SGI291103[] = {{&S46860[16], 11}, {NULL}};
const Segment SGI291104[] = {{&S46860[66], 13}, {NULL}};
const Segment SGI291105[] = {{&S47051[90], 4}, {NULL}};
const Segment SGI291106[] = {{&S47051[8], 3}, {NULL}};
const Segment SGI291107[] = {{&S47051[13], 3}, {NULL}};
const Segment SGI291108[] = {{&S47051[13], 3}, {&S47051[62], 6}, {NULL}};
const Segment SGI291109[] = {{&S47051[62], 11}, {NULL}};
const Segment SGI291110[] = {{&S47051[7], 1}, {NULL}};
const Segment SGI291111[] = {{&S47053[57], 5}, {NULL}};
const Segment SGI291112[] = {{&S47053[8], 3}, {NULL}};
const Segment SGI291113[] = {{&S47053[13], 3}, {NULL}};
const Segment SGI291114[] = {{&S47053[13], 3}, {&S47053[18], 5}, {NULL}};
const Segment SGI291115[] = {{&S47053[7], 1}, {NULL}};
const Segment SGI291116[] = {{&S47085[78], 4}, {NULL}};
const Segment SGI291117[] = {{&S47085[8], 3}, {NULL}};
const Segment SGI291118[] = {{&S47085[13], 7}, {NULL}};
const Segment SGI291119[] = {{&S47085[25], 14}, {NULL}};
const Segment SGI291120[] = {{&S47085[7], 1}, {NULL}};
const Segment SGI291121[] = {{&S47085[16], 4}, {&S47085[28], 11}, {NULL}};
const Segment SGI291122[] = {{&S47086[72], 4}, {NULL}};
const Segment SGI291123[] = {{&S47086[8], 3}, {NULL}};
const Segment SGI291124[] = {{&S47086[13], 7}, {NULL}};
const Segment SGI291125[] = {{&S47086[25], 14}, {NULL}};
const Segment SGI291126[] = {{&S47086[7], 1}, {NULL}};
const Segment SGI291127[] = {{&S47086[16], 4}, {&S47086[28], 11}, {NULL}};
const Segment SGI291128[] = {{&S47087[84], 4}, {NULL}};
const Segment SGI291129[] = {{&S47087[8], 3}, {NULL}};
const Segment SGI291130[] = {{&S47087[13], 7}, {NULL}};
const Segment SGI291131[] = {{&S47087[7], 1}, {NULL}};
const Segment SGI291132[] = {{&S47087[16], 4}, {&S47087[28], 12}, {NULL}};
const Segment SGI291133[] = {{&S47100[70], 4}, {NULL}};
const Segment SGI291134[] = {{&S47100[8], 3}, {NULL}};
const Segment SGI291135[] = {{&S47100[13], 9}, {NULL}};
const Segment SGI291136[] = {{&S47100[30], 15}, {NULL}};
const Segment SGI291137[] = {{&S47100[7], 1}, {NULL}};
const Segment SGI291138[] = {{&S47100[75], 11}, {NULL}};
const Segment SGI291139[] = {{&S47385[72], 4}, {NULL}};
const Segment SGI291140[] = {{&S47385[8], 3}, {NULL}};
const Segment SGI291141[] = {{&S47385[13], 7}, {NULL}};
const Segment SGI291142[] = {{&S47385[25], 14}, {NULL}};
const Segment SGI291143[] = {{&S47385[7], 1}, {NULL}};
const Segment SGI291144[] = {{&S47385[16], 4}, {&S47385[28], 11}, {NULL}};
const Segment SGI291145[] = {{&S47386[82], 4}, {NULL}};
const Segment SGI291146[] = {{&S47386[8], 3}, {NULL}};
const Segment SGI291147[] = {{&S47386[13], 7}, {NULL}};
const Segment SGI291148[] = {{&S47386[7], 1}, {NULL}};
const Segment SGI291149[] = {{&S47386[16], 4}, {&S47386[28], 15}, {NULL}};
const Segment SGI291150[] = {{&S47387[84], 4}, {NULL}};
const Segment SGI291151[] = {{&S47387[8], 3}, {NULL}};
const Segment SGI291152[] = {{&S47387[13], 7}, {NULL}};
const Segment SGI291153[] = {{&S47387[7], 1}, {NULL}};
const Segment SGI291154[] = {{&S47387[16], 4}, {&S47387[28], 12}, {NULL}};
const Segment SGI291155[] = {{&S47393[75], 4}, {NULL}};
const Segment SGI291156[] = {{&S47393[8], 3}, {NULL}};
const Segment SGI291157[] = {{&S47393[13], 7}, {NULL}};
const Segment SGI291158[] = {{&S47393[25], 14}, {NULL}};
const Segment SGI291159[] = {{&S47393[7], 1}, {NULL}};
const Segment SGI291160[] = {{&S47393[16], 4}, {&S47393[28], 11}, {NULL}};
const Segment SGI291161[] = {{&S47393[85], 13}, {NULL}};
const Segment SGI291162[] = {{&S47548[53], 5}, {NULL}};
const Segment SGI291163[] = {{&S47548[8], 3}, {NULL}};
const Segment SGI291164[] = {{&S47548[13], 14}, {NULL}};
const Segment SGI291165[] = {{&S47548[7], 1}, {NULL}};
const Segment SGI291166[] = {{&S47548[16], 11}, {NULL}};
const Segment SGI291167[] = {{&S47549[65], 5}, {NULL}};
const Segment SGI291168[] = {{&S47549[8], 3}, {NULL}};
const Segment SGI291169[] = {{&S47549[13], 14}, {NULL}};
const Segment SGI291170[] = {{&S47549[7], 1}, {NULL}};
const Segment SGI291171[] = {{&S47549[16], 11}, {&S47549[32], 2}, {NULL}};
const Segment SGI291172[] = {{&S47700[82], 4}, {NULL}};
const Segment SGI291173[] = {{&S47700[8], 3}, {NULL}};
const Segment SGI291174[] = {{&S47700[13], 9}, {NULL}};
const Segment SGI291175[] = {{&S47700[33], 13}, {NULL}};
const Segment SGI291176[] = {{&S47700[7], 1}, {NULL}};
const Segment SGI291177[] = {{&S47825[59], 5}, {NULL}};
const Segment SGI291178[] = {{&S47825[8], 3}, {NULL}};
const Segment SGI291179[] = {{&S47825[13], 9}, {NULL}};
const Segment SGI291180[] = {{&S47825[30], 13}, {NULL}};
const Segment SGI291181[] = {{&S47825[7], 1}, {NULL}};
const Segment SGI291182[] = {{&S47835[75], 4}, {NULL}};
const Segment SGI291183[] = {{&S47835[8], 3}, {NULL}};
const Segment SGI291184[] = {{&S47835[13], 7}, {NULL}};
const Segment SGI291185[] = {{&S47835[25], 14}, {NULL}};
const Segment SGI291186[] = {{&S47835[7], 1}, {NULL}};
const Segment SGI291187[] = {{&S47835[16], 4}, {&S47835[28], 11}, {NULL}};
const Segment SGI291188[] = {{&S48004[70], 4}, {NULL}};
const Segment SGI291189[] = {{&S48004[8], 3}, {NULL}};
const Segment SGI291190[] = {{&S48004[13], 9}, {NULL}};
const Segment SGI291191[] = {{&S48004[30], 13}, {NULL}};
const Segment SGI291192[] = {{&S48004[7], 1}, {NULL}};
const Segment SGI291193[] = {{&S48019[69], 5}, {NULL}};
const Segment SGI291194[] = {{&S48019[8], 3}, {NULL}};
const Segment SGI291195[] = {{&S48019[13], 9}, {NULL}};
const Segment SGI291196[] = {{&S48019[28], 13}, {NULL}};
const Segment SGI291197[] = {{&S48019[7], 1}, {NULL}};
const Segment SGI291198[] = {{&S32534[0], 124}, {NULL}};
const Segment SGI291199[] = {{&S32535[0], 107}, {NULL}};
const Segment SGI291200[] = {{&S32536[0], 108}, {NULL}};
const Segment SGI291201[] = {{&S32537[0], 137}, {NULL}};
const Segment SGI291202[] = {{&S32538[0], 132}, {NULL}};
const Segment SGI291203[] = {{&S32539[0], 137}, {NULL}};
const Segment SGI291204[] = {{&S32540[0], 137}, {NULL}};
const Segment SGI291205[] = {{&S32541[0], 137}, {NULL}};
const Segment SGI291206[] = {{&S32542[0], 137}, {NULL}};
const Segment SGI291207[] = {{&S32543[0], 137}, {NULL}};
const Segment SGI291208[] = {{&S32544[0], 137}, {NULL}};
const Segment SGI291209[] = {{&S32545[0], 137}, {NULL}};
const Segment SGI291210[] = {{&S32546[0], 137}, {NULL}};
const Segment SGI291211[] = {{&S32547[0], 137}, {NULL}};
const Segment SGI291212[] = {{&S32548[0], 137}, {NULL}};
const Segment SGI291213[] = {{&S32549[0], 137}, {NULL}};
const Segment SGI291214[] = {{&S32550[0], 137}, {NULL}};
const Segment SGI291215[] = {{&S32551[0], 137}, {NULL}};
const Segment SGI291216[] = {{&S32552[0], 137}, {NULL}};
const Segment SGI291217[] = {{&S48063[53], 4}, {NULL}};
const Segment SGI291218[] = {{&S48063[8], 3}, {NULL}};
const Segment SGI291219[] = {{&S48063[13], 3}, {NULL}};
const Segment SGI291220[] = {{&S48063[7], 1}, {NULL}};
const Segment SGI291221[] = {{&S48063[58], 11}, {NULL}};
const Segment SGI291222[] = {{&S32553[0], 137}, {NULL}};
const Segment SGI291223[] = {{&S32554[0], 137}, {NULL}};
const Segment SGI291224[] = {{&S32555[0], 137}, {NULL}};
const Segment SGI291225[] = {{&S32556[0], 106}, {NULL}};
const Segment SGI291226[] = {{&S32557[0], 137}, {NULL}};
const Segment SGI291227[] = {{&S32558[0], 137}, {NULL}};
const Segment SGI291228[] = {{&S32559[0], 112}, {NULL}};
const Segment SGI291229[] = {{&S32560[0], 137}, {NULL}};
const Segment SGI291230[] = {{&S32561[0], 137}, {NULL}};
const Segment SGI291231[] = {{&S32562[0], 137}, {NULL}};
const Segment SGI291232[] = {{&S32563[0], 137}, {NULL}};
const Segment SGI291233[] = {{&S32564[0], 137}, {NULL}};
const Segment SGI291234[] = {{&S32565[0], 137}, {NULL}};
const Segment SGI291235[] = {{&S32566[0], 137}, {NULL}};
const Segment SGI291236[] = {{&S32567[0], 137}, {NULL}};
const Segment SGI291237[] = {{&S32568[0], 137}, {NULL}};
const Segment SGI291238[] = {{&S32569[0], 137}, {NULL}};
const Segment SGI291239[] = {{&S32570[0], 137}, {NULL}};
const Segment SGI291240[] = {{&S32571[0], 120}, {NULL}};
const Segment SGI291241[] = {{&S32572[0], 137}, {NULL}};
const Segment SGI291242[] = {{&S32573[0], 137}, {NULL}};
const Segment SGI291243[] = {{&S32574[0], 137}, {NULL}};
const Segment SGI291244[] = {{&S32575[0], 137}, {NULL}};
const Segment SGI291245[] = {{&S32576[0], 137}, {NULL}};
const Segment SGI291246[] = {{&S48091[79], 4}, {NULL}};
const Segment SGI291247[] = {{&S48091[8], 3}, {NULL}};
const Segment SGI291248[] = {{&S48091[13], 3}, {NULL}};
const Segment SGI291249[] = {{&S32577[0], 137}, {NULL}};
const Segment SGI291250[] = {{&S48091[7], 1}, {NULL}};
const Segment SGI291251[] = {{&S48091[84], 17}, {NULL}};
const Segment SGI291252[] = {{&S32578[0], 137}, {NULL}};
const Segment SGI291253[] = {{&S48223[91], 4}, {NULL}};
const Segment SGI291254[] = {{&S48223[8], 3}, {NULL}};
const Segment SGI291255[] = {{&S48223[13], 3}, {NULL}};
const Segment SGI291256[] = {{&S48223[13], 3}, {&S48223[63], 6}, {NULL}};
const Segment SGI291257[] = {{&S48223[63], 11}, {NULL}};
const Segment SGI291258[] = {{&S48223[7], 1}, {NULL}};
const Segment SGI291259[] = {{&S48379[74], 4}, {NULL}};
const Segment SGI291260[] = {{&S48379[8], 3}, {NULL}};
const Segment SGI291261[] = {{&S48379[13], 9}, {NULL}};
const Segment SGI291262[] = {{&S48379[30], 15}, {NULL}};
const Segment SGI291263[] = {{&S48379[7], 1}, {NULL}};
const Segment SGI291264[] = {{&S48379[79], 11}, {NULL}};
const Segment SGI291265[] = {{&S48464[56], 4}, {NULL}};
const Segment SGI291266[] = {{&S48464[8], 3}, {NULL}};
const Segment SGI291267[] = {{&S48464[13], 14}, {NULL}};
const Segment SGI291268[] = {{&S48464[7], 1}, {NULL}};
const Segment SGI291269[] = {{&S48464[16], 11}, {NULL}};
const Segment SGI291270[] = {{&S48464[61], 11}, {NULL}};
const Segment SGI291271[] = {{&S46824[8], 3}, {NULL}};
const Segment SGI291272[] = {{&S46824[33], 6}, {NULL}};
const Segment SGI291273[] = {{&S46824[69], 6}, {NULL}};
const Segment SGI291274[] = {{&S46824[13], 26}, {NULL}};
const Segment SGI291275[] = {{&S46824[33], 2}, {NULL}};
const Segment SGI291276[] = {{&S46825[8], 3}, {NULL}};
const Segment SGI291277[] = {{&S46825[33], 6}, {NULL}};
const Segment SGI291278[] = {{&S46825[69], 6}, {NULL}};
const Segment SGI291279[] = {{&S46825[13], 26}, {NULL}};
const Segment SGI291280[] = {{&S46825[33], 2}, {NULL}};
const Segment SGI291281[] = {{&S48759[69], 5}, {NULL}};
const Segment SGI291282[] = {{&S48759[8], 3}, {NULL}};
const Segment SGI291283[] = {{&S48759[13], 9}, {NULL}};
const Segment SGI291284[] = {{&S48759[30], 13}, {NULL}};
const Segment SGI291285[] = {{&S48759[7], 1}, {NULL}};
const Segment SGI291286[] = {{&S48783[60], 5}, {NULL}};
const Segment SGI291287[] = {{&S48783[8], 3}, {NULL}};
const Segment SGI291288[] = {{&S48783[13], 14}, {NULL}};
const Segment SGI291289[] = {{&S48783[7], 1}, {NULL}};
const Segment SGI291290[] = {{&S48783[16], 11}, {NULL}};
const Segment SGI291291[] = {{&S48784[60], 5}, {NULL}};
const Segment SGI291292[] = {{&S48784[8], 3}, {NULL}};
const Segment SGI291293[] = {{&S48784[13], 14}, {NULL}};
const Segment SGI291294[] = {{&S48784[7], 1}, {NULL}};
const Segment SGI291295[] = {{&S48784[16], 11}, {NULL}};
const Segment SGI291296[] = {{&S48784[66], 13}, {NULL}};
const Segment SGI291297[] = {{&S48785[74], 5}, {NULL}};
const Segment SGI291298[] = {{&S48785[17], 3}, {NULL}};
const Segment SGI291299[] = {{&S48785[22], 14}, {NULL}};
const Segment SGI291300[] = {{&S48785[16], 1}, {NULL}};
const Segment SGI291301[] = {{&S48785[25], 11}, {&S48785[41], 2}, {NULL}};
const Segment SGI291302[] = {{&S48808[53], 4}, {NULL}};
const Segment SGI291303[] = {{&S48808[8], 3}, {NULL}};
const Segment SGI291304[] = {{&S48808[13], 3}, {NULL}};
const Segment SGI291305[] = {{&S48808[7], 1}, {NULL}};
const Segment SGI291306[] = {{&S48809[57], 4}, {&S48809[57], 4}, {NULL}};
const Segment SGI291307[] = {{&S48809[8], 3}, {NULL}};
const Segment SGI291308[] = {{&S48809[13], 3}, {NULL}};
const Segment SGI291309[] = {{&S48809[7], 1}, {&S48809[7], 1}, {NULL}};
const Segment SGI291310[] = {{&S48810[53], 4}, {NULL}};
const Segment SGI291311[] = {{&S48810[8], 3}, {NULL}};
const Segment SGI291312[] = {{&S48810[13], 3}, {NULL}};
const Segment SGI291313[] = {{&S48810[7], 1}, {NULL}};
const Segment SGI291314[] = {{&S49182[65], 5}, {NULL}};
const Segment SGI291315[] = {{&S49182[8], 3}, {NULL}};
const Segment SGI291316[] = {{&S49182[13], 14}, {NULL}};
const Segment SGI291317[] = {{&S49182[7], 1}, {NULL}};
const Segment SGI291318[] = {{&S49182[16], 11}, {&S49182[32], 2}, {NULL}};
const Segment SGI291319[] = {{&S49183[60], 5}, {NULL}};
const Segment SGI291320[] = {{&S49183[8], 3}, {NULL}};
const Segment SGI291321[] = {{&S49183[13], 14}, {NULL}};
const Segment SGI291322[] = {{&S49183[7], 1}, {NULL}};
const Segment SGI291323[] = {{&S49183[16], 11}, {NULL}};
const Segment SGI291324[] = {{&S49250[53], 5}, {NULL}};
const Segment SGI291325[] = {{&S49250[8], 3}, {NULL}};
const Segment SGI291326[] = {{&S49250[13], 14}, {NULL}};
const Segment SGI291327[] = {{&S49250[7], 1}, {NULL}};
const Segment SGI291328[] = {{&S49250[16], 11}, {NULL}};
const Segment SGI291329[] = {{&S49250[59], 14}, {NULL}};
const Segment SGI291330[] = {{&S49251[53], 5}, {NULL}};
const Segment SGI291331[] = {{&S49251[8], 3}, {NULL}};
const Segment SGI291332[] = {{&S49251[13], 14}, {NULL}};
const Segment SGI291333[] = {{&S49251[7], 1}, {NULL}};
const Segment SGI291334[] = {{&S49251[16], 11}, {NULL}};
const Segment SGI291335[] = {{&S49251[59], 17}, {NULL}};
const Segment SGI291336[] = {{&S49357[54], 5}, {NULL}};
const Segment SGI291337[] = {{&S49357[8], 3}, {NULL}};
const Segment SGI291338[] = {{&S49357[13], 3}, {NULL}};
const Segment SGI291339[] = {{&S49357[7], 1}, {NULL}};
const Segment SGI291340[] = {{&S49357[60], 13}, {NULL}};
const Segment SGI291341[] = {{&S49368[69], 5}, {NULL}};
const Segment SGI291342[] = {{&S49368[8], 3}, {NULL}};
const Segment SGI291343[] = {{&S49368[13], 9}, {NULL}};
const Segment SGI291344[] = {{&S49368[30], 13}, {NULL}};
const Segment SGI291345[] = {{&S49368[7], 1}, {NULL}};
const Segment SGI291346[] = {{&S49414[82], 4}, {NULL}};
const Segment SGI291347[] = {{&S49414[8], 3}, {NULL}};
const Segment SGI291348[] = {{&S49414[13], 7}, {NULL}};
const Segment SGI291349[] = {{&S49414[25], 14}, {NULL}};
const Segment SGI291350[] = {{&S49414[7], 1}, {NULL}};
const Segment SGI291351[] = {{&S49414[16], 4}, {&S49414[28], 11}, {NULL}};
const Segment SGI291352[] = {{&S49415[61], 4}, {NULL}};
const Segment SGI291353[] = {{&S49415[8], 3}, {NULL}};
const Segment SGI291354[] = {{&S49415[13], 14}, {NULL}};
const Segment SGI291355[] = {{&S49415[7], 1}, {NULL}};
const Segment SGI291356[] = {{&S49415[16], 11}, {NULL}};
const Segment SGI291357[] = {{&S49416[8], 3}, {NULL}};
const Segment SGI291358[] = {{&S49416[13], 7}, {NULL}};
const Segment SGI291359[] = {{&S49416[25], 14}, {NULL}};
const Segment SGI291360[] = {{&S49416[16], 4}, {&S49416[28], 11}, {NULL}};
const Segment SGI291361[] = {{&S49416[68], 11}, {NULL}};
const Segment SGI291362[] = {{&S49417[53], 5}, {NULL}};
const Segment SGI291363[] = {{&S49417[8], 3}, {NULL}};
const Segment SGI291364[] = {{&S49417[13], 14}, {NULL}};
const Segment SGI291365[] = {{&S49417[7], 1}, {NULL}};
const Segment SGI291366[] = {{&S49417[16], 11}, {NULL}};
const Segment SGI291367[] = {{&S49931[72], 4}, {NULL}};
const Segment SGI291368[] = {{&S49931[8], 3}, {NULL}};
const Segment SGI291369[] = {{&S49931[13], 9}, {NULL}};
const Segment SGI291370[] = {{&S49931[7], 1}, {NULL}};
const Segment SGI291371[] = {{&S49931[79], 10}, {NULL}};
const Segment SGI291372[] = {{&S50182[53], 5}, {NULL}};
const Segment SGI291373[] = {{&S50182[8], 3}, {NULL}};
const Segment SGI291374[] = {{&S50182[13], 14}, {NULL}};
const Segment SGI291375[] = {{&S50182[7], 1}, {NULL}};
const Segment SGI291376[] = {{&S50182[16], 11}, {NULL}};
const Segment SGI291377[] = {{&S50182[61], 13}, {NULL}};
const Segment SGI291378[] = {{&S50183[60], 5}, {NULL}};
const Segment SGI291379[] = {{&S50183[8], 3}, {NULL}};
const Segment SGI291380[] = {{&S50183[13], 14}, {NULL}};
const Segment SGI291381[] = {{&S50183[7], 1}, {NULL}};
const Segment SGI291382[] = {{&S50183[16], 11}, {NULL}};
const Segment SGI291383[] = {{&S50183[68], 13}, {NULL}};
const Segment SGI291384[] = {{&S34953[19], 7}, {NULL}};
const Segment SGI291385[] = {{&S34953[0], 33}, {NULL}};
const Segment SGI291386[] = {{&S50398[97], 4}, {NULL}};
const Segment SGI291387[] = {{&S50398[8], 3}, {NULL}};
const Segment SGI291388[] = {{&S50398[13], 9}, {NULL}};
const Segment SGI291389[] = {{&S50398[33], 13}, {NULL}};
const Segment SGI291390[] = {{&S50398[7], 1}, {NULL}};
const Segment SGI291391[] = {{&S50398[73], 10}, {NULL}};
const Segment SGI291392[] = {{&S50399[99], 5}, {NULL}};
const Segment SGI291393[] = {{&S50399[8], 3}, {NULL}};
const Segment SGI291394[] = {{&S50399[13], 9}, {NULL}};
const Segment SGI291395[] = {{&S50399[33], 13}, {NULL}};
const Segment SGI291396[] = {{&S50399[7], 1}, {NULL}};
const Segment SGI291397[] = {{&S50399[73], 10}, {NULL}};
const Segment SGI291398[] = {{&S50416[60], 5}, {NULL}};
const Segment SGI291399[] = {{&S50416[8], 3}, {NULL}};
const Segment SGI291400[] = {{&S50416[13], 14}, {NULL}};
const Segment SGI291401[] = {{&S50416[7], 1}, {NULL}};
const Segment SGI291402[] = {{&S50416[16], 11}, {NULL}};
const Segment SGI291403[] = {{&S50416[66], 14}, {NULL}};
const Segment SGI291404[] = {{&S50417[60], 5}, {NULL}};
const Segment SGI291405[] = {{&S50417[8], 3}, {NULL}};
const Segment SGI291406[] = {{&S50417[13], 14}, {NULL}};
const Segment SGI291407[] = {{&S50417[7], 1}, {NULL}};
const Segment SGI291408[] = {{&S50417[16], 11}, {NULL}};
const Segment SGI291409[] = {{&S50417[66], 17}, {NULL}};
const Segment SGI291410[] = {{&S50627[53], 5}, {NULL}};
const Segment SGI291411[] = {{&S50627[8], 3}, {NULL}};
const Segment SGI291412[] = {{&S50627[13], 14}, {NULL}};
const Segment SGI291413[] = {{&S50627[7], 1}, {NULL}};
const Segment SGI291414[] = {{&S50627[16], 11}, {NULL}};
const Segment SGI291415[] = {{&S50627[59], 13}, {NULL}};
const Segment SGI291416[] = {{&S50628[60], 5}, {NULL}};
const Segment SGI291417[] = {{&S50628[8], 3}, {NULL}};
const Segment SGI291418[] = {{&S50628[13], 14}, {NULL}};
const Segment SGI291419[] = {{&S50628[7], 1}, {NULL}};
const Segment SGI291420[] = {{&S50628[16], 11}, {NULL}};
const Segment SGI291421[] = {{&S50628[66], 13}, {NULL}};
const Segment SGI291422[] = {{&S50629[69], 5}, {NULL}};
const Segment SGI291423[] = {{&S50629[8], 3}, {NULL}};
const Segment SGI291424[] = {{&S50629[13], 9}, {NULL}};
const Segment SGI291425[] = {{&S50629[30], 13}, {NULL}};
const Segment SGI291426[] = {{&S50629[7], 1}, {NULL}};
const Segment SGI291427[] = {{&S50635[69], 5}, {NULL}};
const Segment SGI291428[] = {{&S50635[8], 3}, {NULL}};
const Segment SGI291429[] = {{&S50635[13], 9}, {NULL}};
const Segment SGI291430[] = {{&S50635[30], 13}, {NULL}};
const Segment SGI291431[] = {{&S50635[7], 1}, {NULL}};
const Segment SGI291432[] = {{&S50636[8], 3}, {NULL}};
const Segment SGI291433[] = {{&S50636[13], 9}, {NULL}};
const Segment SGI291434[] = {{&S50636[30], 13}, {NULL}};
const Segment SGI291435[] = {{&S50636[7], 1}, {NULL}};
const Segment SGI291436[] = {{&S50796[69], 5}, {NULL}};
const Segment SGI291437[] = {{&S50796[8], 3}, {NULL}};
const Segment SGI291438[] = {{&S50796[13], 9}, {NULL}};
const Segment SGI291439[] = {{&S50796[30], 13}, {NULL}};
const Segment SGI291440[] = {{&S50796[7], 1}, {NULL}};
const Segment SGI291441[] = {{&S50815[61], 4}, {NULL}};
const Segment SGI291442[] = {{&S50815[8], 3}, {NULL}};
const Segment SGI291443[] = {{&S50815[13], 14}, {NULL}};
const Segment SGI291444[] = {{&S50815[7], 1}, {NULL}};
const Segment SGI291445[] = {{&S50815[16], 11}, {NULL}};
const Segment SGI291446[] = {{&S50815[68], 11}, {NULL}};
const Segment SGI291447[] = {{&S50857[68], 5}, {NULL}};
const Segment SGI291448[] = {{&S50857[8], 3}, {NULL}};
const Segment SGI291449[] = {{&S50857[13], 14}, {NULL}};
const Segment SGI291450[] = {{&S50857[7], 1}, {NULL}};
const Segment SGI291451[] = {{&S50857[16], 11}, {NULL}};
const Segment SGI291452[] = {{&S50857[38], 13}, {NULL}};
const Segment SGI291453[] = {{&S50858[68], 5}, {NULL}};
const Segment SGI291454[] = {{&S50858[8], 3}, {NULL}};
const Segment SGI291455[] = {{&S50858[13], 14}, {NULL}};
const Segment SGI291456[] = {{&S50858[7], 1}, {NULL}};
const Segment SGI291457[] = {{&S50858[16], 11}, {NULL}};
const Segment SGI291458[] = {{&S50858[38], 13}, {NULL}};
const Segment SGI291459[] = {{&S50859[68], 5}, {NULL}};
const Segment SGI291460[] = {{&S50859[8], 3}, {NULL}};
const Segment SGI291461[] = {{&S50859[13], 14}, {NULL}};
const Segment SGI291462[] = {{&S50859[7], 1}, {NULL}};
const Segment SGI291463[] = {{&S50859[16], 11}, {NULL}};
const Segment SGI291464[] = {{&S50859[38], 13}, {NULL}};
const Segment SGI291465[] = {{&S50860[68], 5}, {NULL}};
const Segment SGI291466[] = {{&S50860[8], 3}, {NULL}};
const Segment SGI291467[] = {{&S50860[13], 14}, {NULL}};
const Segment SGI291468[] = {{&S50860[7], 1}, {NULL}};
const Segment SGI291469[] = {{&S50860[16], 11}, {NULL}};
const Segment SGI291470[] = {{&S50860[38], 13}, {NULL}};
const Segment SGI291471[] = {{&S50861[68], 5}, {NULL}};
const Segment SGI291472[] = {{&S50861[8], 3}, {NULL}};
const Segment SGI291473[] = {{&S50861[13], 14}, {NULL}};
const Segment SGI291474[] = {{&S50861[7], 1}, {NULL}};
const Segment SGI291475[] = {{&S50861[16], 11}, {NULL}};
const Segment SGI291476[] = {{&S50861[38], 13}, {NULL}};
const Segment SGI291477[] = {{&S50862[68], 5}, {NULL}};
const Segment SGI291478[] = {{&S50862[8], 3}, {NULL}};
const Segment SGI291479[] = {{&S50862[13], 14}, {NULL}};
const Segment SGI291480[] = {{&S50862[7], 1}, {NULL}};
const Segment SGI291481[] = {{&S50862[16], 11}, {NULL}};
const Segment SGI291482[] = {{&S50862[38], 13}, {NULL}};
const Segment SGI291483[] = {{&S50863[68], 5}, {NULL}};
const Segment SGI291484[] = {{&S50863[8], 3}, {NULL}};
const Segment SGI291485[] = {{&S50863[13], 14}, {NULL}};
const Segment SGI291486[] = {{&S50863[7], 1}, {NULL}};
const Segment SGI291487[] = {{&S50863[16], 11}, {NULL}};
const Segment SGI291488[] = {{&S50863[38], 13}, {NULL}};
const Segment SGI291489[] = {{&S50864[68], 5}, {NULL}};
const Segment SGI291490[] = {{&S50864[8], 3}, {NULL}};
const Segment SGI291491[] = {{&S50864[13], 14}, {NULL}};
const Segment SGI291492[] = {{&S50864[7], 1}, {NULL}};
const Segment SGI291493[] = {{&S50864[16], 11}, {NULL}};
const Segment SGI291494[] = {{&S50864[38], 13}, {NULL}};
const Segment SGI291495[] = {{&S50865[68], 5}, {NULL}};
const Segment SGI291496[] = {{&S50865[8], 3}, {NULL}};
const Segment SGI291497[] = {{&S50865[13], 14}, {NULL}};
const Segment SGI291498[] = {{&S50865[7], 1}, {NULL}};
const Segment SGI291499[] = {{&S50865[16], 11}, {NULL}};
const Segment SGI291500[] = {{&S50865[38], 13}, {NULL}};
const Segment SGI291501[] = {{&S50923[53], 5}, {NULL}};
const Segment SGI291502[] = {{&S50923[8], 3}, {NULL}};
const Segment SGI291503[] = {{&S50923[13], 14}, {NULL}};
const Segment SGI291504[] = {{&S50923[7], 1}, {NULL}};
const Segment SGI291505[] = {{&S50923[16], 11}, {NULL}};
const Segment SGI291506[] = {{&S50923[59], 13}, {NULL}};
const Segment SGI291507[] = {{&S50924[60], 5}, {NULL}};
const Segment SGI291508[] = {{&S50924[8], 3}, {NULL}};
const Segment SGI291509[] = {{&S50924[13], 14}, {NULL}};
const Segment SGI291510[] = {{&S50924[7], 1}, {NULL}};
const Segment SGI291511[] = {{&S50924[16], 11}, {NULL}};
const Segment SGI291512[] = {{&S50924[66], 13}, {NULL}};
const Segment SGI291513[] = {{&S50938[69], 5}, {NULL}};
const Segment SGI291514[] = {{&S50938[8], 3}, {NULL}};
const Segment SGI291515[] = {{&S50938[13], 9}, {NULL}};
const Segment SGI291516[] = {{&S50938[30], 13}, {NULL}};
const Segment SGI291517[] = {{&S50938[7], 1}, {NULL}};
const Segment SGI291518[] = {{&S50979[60], 5}, {NULL}};
const Segment SGI291519[] = {{&S50979[8], 3}, {NULL}};
const Segment SGI291520[] = {{&S50979[13], 14}, {NULL}};
const Segment SGI291521[] = {{&S50979[7], 1}, {NULL}};
const Segment SGI291522[] = {{&S50979[16], 11}, {NULL}};
const Segment SGI291523[] = {{&S50979[68], 13}, {NULL}};
const Segment SGI291524[] = {{&S51379[76], 4}, {NULL}};
const Segment SGI291525[] = {{&S51379[8], 3}, {NULL}};
const Segment SGI291526[] = {{&S51379[13], 7}, {NULL}};
const Segment SGI291527[] = {{&S51379[25], 14}, {NULL}};
const Segment SGI291528[] = {{&S51379[7], 1}, {NULL}};
const Segment SGI291529[] = {{&S51379[16], 4}, {&S51379[28], 11}, {NULL}};
const Segment SGI291530[] = {{&S51379[85], 12}, {NULL}};
const Segment SGI291531[] = {{&S51381[76], 4}, {NULL}};
const Segment SGI291532[] = {{&S51381[8], 3}, {NULL}};
const Segment SGI291533[] = {{&S51381[13], 7}, {NULL}};
const Segment SGI291534[] = {{&S51381[25], 14}, {NULL}};
const Segment SGI291535[] = {{&S51381[7], 1}, {NULL}};
const Segment SGI291536[] = {{&S51381[16], 4}, {&S51381[28], 11}, {NULL}};
const Segment SGI291537[] = {{&S51381[85], 12}, {NULL}};
const Segment SGI291538[] = {{&S51463[90], 4}, {NULL}};
const Segment SGI291539[] = {{&S51463[8], 3}, {NULL}};
const Segment SGI291540[] = {{&S51463[13], 3}, {NULL}};
const Segment SGI291541[] = {{&S51463[13], 3}, {&S51463[63], 6}, {NULL}};
const Segment SGI291542[] = {{&S51463[63], 11}, {NULL}};
const Segment SGI291543[] = {{&S51463[7], 1}, {NULL}};
const Segment SGI291544[] = {{&S51511[60], 5}, {NULL}};
const Segment SGI291545[] = {{&S51511[8], 3}, {NULL}};
const Segment SGI291546[] = {{&S51511[13], 14}, {NULL}};
const Segment SGI291547[] = {{&S51511[7], 1}, {NULL}};
const Segment SGI291548[] = {{&S51511[16], 11}, {NULL}};
const Segment SGI291549[] = {{&S51511[66], 13}, {NULL}};
const Segment SGI291550[] = {{&S51512[65], 5}, {NULL}};
const Segment SGI291551[] = {{&S51512[8], 3}, {NULL}};
const Segment SGI291552[] = {{&S51512[13], 14}, {NULL}};
const Segment SGI291553[] = {{&S51512[7], 1}, {NULL}};
const Segment SGI291554[] = {{&S51512[16], 11}, {&S51512[32], 2}, {NULL}};
const Segment SGI291555[] = {{&S51512[71], 13}, {NULL}};
const Segment SGI291556[] = {{&S51566[60], 5}, {NULL}};
const Segment SGI291557[] = {{&S51566[8], 3}, {NULL}};
const Segment SGI291558[] = {{&S51566[13], 14}, {NULL}};
const Segment SGI291559[] = {{&S51566[7], 1}, {NULL}};
const Segment SGI291560[] = {{&S51566[16], 11}, {NULL}};
const Segment SGI291561[] = {{&S51566[66], 14}, {NULL}};
const Segment SGI291562[] = {{&S51567[82], 4}, {NULL}};
const Segment SGI291563[] = {{&S51567[8], 3}, {NULL}};
const Segment SGI291564[] = {{&S51567[13], 9}, {NULL}};
const Segment SGI291565[] = {{&S51567[33], 13}, {NULL}};
const Segment SGI291566[] = {{&S51567[7], 1}, {NULL}};
const Segment SGI291567[] = {{&S51596[55], 4}, {NULL}};
const Segment SGI291568[] = {{&S51596[8], 3}, {NULL}};
const Segment SGI291569[] = {{&S51596[13], 3}, {NULL}};
const Segment SGI291570[] = {{&S51596[7], 1}, {NULL}};
const Segment SGI291571[] = {{&S51597[57], 4}, {NULL}};
const Segment SGI291572[] = {{&S51597[8], 3}, {NULL}};
const Segment SGI291573[] = {{&S51597[13], 3}, {NULL}};
const Segment SGI291574[] = {{&S51597[7], 1}, {NULL}};
const Segment SGI291575[] = {{&S51598[53], 5}, {NULL}};
const Segment SGI291576[] = {{&S51598[8], 3}, {NULL}};
const Segment SGI291577[] = {{&S51598[13], 14}, {NULL}};
const Segment SGI291578[] = {{&S51598[7], 1}, {NULL}};
const Segment SGI291579[] = {{&S51598[16], 11}, {NULL}};
const Segment SGI291580[] = {{&S51598[61], 13}, {NULL}};
const Segment SGI291581[] = {{&S51600[53], 5}, {NULL}};
const Segment SGI291582[] = {{&S51600[8], 3}, {NULL}};
const Segment SGI291583[] = {{&S51600[13], 14}, {NULL}};
const Segment SGI291584[] = {{&S51600[7], 1}, {NULL}};
const Segment SGI291585[] = {{&S51600[16], 11}, {NULL}};
const Segment SGI291586[] = {{&S51600[61], 13}, {NULL}};
const Segment SGI291587[] = {{&S51602[69], 5}, {NULL}};
const Segment SGI291588[] = {{&S51602[8], 3}, {NULL}};
const Segment SGI291589[] = {{&S51602[13], 9}, {NULL}};
const Segment SGI291590[] = {{&S51602[30], 13}, {NULL}};
const Segment SGI291591[] = {{&S51602[7], 1}, {NULL}};
const Segment SGI291592[] = {{&S51605[88], 4}, {NULL}};
const Segment SGI291593[] = {{&S51605[8], 3}, {NULL}};
const Segment SGI291594[] = {{&S51605[13], 3}, {NULL}};
const Segment SGI291595[] = {{&S51605[13], 3}, {&S51605[61], 6}, {NULL}};
const Segment SGI291596[] = {{&S51605[61], 11}, {NULL}};
const Segment SGI291597[] = {{&S51605[7], 1}, {NULL}};
const Segment SGI291598[] = {{&S51754[60], 5}, {NULL}};
const Segment SGI291599[] = {{&S51754[8], 3}, {NULL}};
const Segment SGI291600[] = {{&S51754[13], 14}, {NULL}};
const Segment SGI291601[] = {{&S51754[7], 1}, {NULL}};
const Segment SGI291602[] = {{&S51754[16], 11}, {NULL}};
const Segment SGI291603[] = {{&S51754[66], 14}, {NULL}};
const Segment SGI291604[] = {{&S51755[60], 5}, {NULL}};
const Segment SGI291605[] = {{&S51755[8], 3}, {NULL}};
const Segment SGI291606[] = {{&S51755[13], 14}, {NULL}};
const Segment SGI291607[] = {{&S51755[7], 1}, {NULL}};
const Segment SGI291608[] = {{&S51755[16], 11}, {NULL}};
const Segment SGI291609[] = {{&S51755[66], 14}, {NULL}};
const Segment SGI291610[] = {{&S51764[7], 4}, {NULL}};
const Segment SGI291611[] = {{&S51764[7], 1}, {NULL}};
const Segment SGI291612[] = {{&S51764[16], 4}, {&S51764[28], 2}, {NULL}};
const Segment SGI291613[] = {{&S51765[64], 4}, {NULL}};
const Segment SGI291614[] = {{&S51765[8], 3}, {NULL}};
const Segment SGI291615[] = {{&S51765[13], 7}, {NULL}};
const Segment SGI291616[] = {{&S51765[7], 1}, {NULL}};
const Segment SGI291617[] = {{&S51765[16], 4}, {&S51765[28], 2}, {NULL}};
const Segment SGI291618[] = {{&S51766[70], 4}, {NULL}};
const Segment SGI291619[] = {{&S51766[8], 3}, {NULL}};
const Segment SGI291620[] = {{&S51766[13], 7}, {NULL}};
const Segment SGI291621[] = {{&S51766[7], 1}, {NULL}};
const Segment SGI291622[] = {{&S51766[16], 4}, {&S51766[28], 8}, {NULL}};
const Segment SGI291623[] = {{&S51771[69], 5}, {NULL}};
const Segment SGI291624[] = {{&S51771[8], 3}, {NULL}};
const Segment SGI291625[] = {{&S51771[13], 9}, {NULL}};
const Segment SGI291626[] = {{&S51771[30], 13}, {NULL}};
const Segment SGI291627[] = {{&S51771[7], 1}, {NULL}};
const Segment SGI291628[] = {{&S51791[53], 5}, {NULL}};
const Segment SGI291629[] = {{&S51791[8], 3}, {NULL}};
const Segment SGI291630[] = {{&S51791[13], 14}, {NULL}};
const Segment SGI291631[] = {{&S51791[7], 1}, {NULL}};
const Segment SGI291632[] = {{&S51791[16], 11}, {NULL}};
const Segment SGI291633[] = {{&S51791[59], 13}, {NULL}};
const Segment SGI291634[] = {{&S51925[65], 5}, {NULL}};
const Segment SGI291635[] = {{&S51925[8], 3}, {NULL}};
const Segment SGI291636[] = {{&S51925[13], 14}, {NULL}};
const Segment SGI291637[] = {{&S51925[7], 1}, {NULL}};
const Segment SGI291638[] = {{&S51925[16], 11}, {&S51925[32], 2}, {NULL}};
const Segment SGI291639[] = {{&S51925[77], 13}, {NULL}};
const Segment SGI291640[] = {{&S51926[60], 5}, {NULL}};
const Segment SGI291641[] = {{&S51926[8], 3}, {NULL}};
const Segment SGI291642[] = {{&S51926[13], 14}, {NULL}};
const Segment SGI291643[] = {{&S51926[7], 1}, {NULL}};
const Segment SGI291644[] = {{&S51926[16], 11}, {NULL}};
const Segment SGI291645[] = {{&S51926[68], 13}, {NULL}};
const Segment SGI291646[] = {{&S36632[0], 124}, {NULL}};
const Segment SGI291647[] = {{&S36633[0], 106}, {NULL}};
const Segment SGI291648[] = {{&S36634[0], 106}, {NULL}};
const Segment SGI291649[] = {{&S36635[0], 132}, {NULL}};
const Segment SGI291650[] = {{&S36636[0], 106}, {NULL}};
const Segment SGI291651[] = {{&S36637[0], 137}, {NULL}};
const Segment SGI291652[] = {{&S36638[0], 137}, {NULL}};
const Segment SGI291653[] = {{&S36639[0], 137}, {NULL}};
const Segment SGI291654[] = {{&S36640[0], 137}, {NULL}};
const Segment SGI291655[] = {{&S36641[0], 137}, {NULL}};
const Segment SGI291656[] = {{&S36642[0], 137}, {NULL}};
const Segment SGI291657[] = {{&S36643[0], 137}, {NULL}};
const Segment SGI291658[] = {{&S36644[0], 137}, {NULL}};
const Segment SGI291659[] = {{&S36645[0], 137}, {NULL}};
const Segment SGI291660[] = {{&S36646[0], 137}, {NULL}};
const Segment SGI291661[] = {{&S36647[0], 137}, {NULL}};
const Segment SGI291662[] = {{&S36648[0], 137}, {NULL}};
const Segment SGI291663[] = {{&S36649[0], 137}, {NULL}};
const Segment SGI291664[] = {{&S36650[0], 137}, {NULL}};
const Segment SGI291665[] = {{&S36651[0], 137}, {NULL}};
const Segment SGI291666[] = {{&S36652[0], 112}, {NULL}};
const Segment SGI291667[] = {{&S36653[0], 137}, {NULL}};
const Segment SGI291668[] = {{&S36654[0], 137}, {NULL}};
const Segment SGI291669[] = {{&S36655[0], 137}, {NULL}};
const Segment SGI291670[] = {{&S36656[0], 120}, {NULL}};
const Segment SGI291671[] = {{&S36657[0], 137}, {NULL}};
const Segment SGI291672[] = {{&S36658[0], 137}, {NULL}};
const Segment SGI291673[] = {{&S36659[0], 137}, {NULL}};
const Segment SGI291674[] = {{&S36660[0], 137}, {NULL}};
const Segment SGI291675[] = {{&S36661[0], 137}, {NULL}};
const Segment SGI291676[] = {{&S36662[0], 137}, {NULL}};
const Segment SGI291677[] = {{&S36663[0], 137}, {NULL}};
const Segment SGI291678[] = {{&S52253[69], 5}, {NULL}};
const Segment SGI291679[] = {{&S52253[8], 3}, {NULL}};
const Segment SGI291680[] = {{&S52253[13], 9}, {NULL}};
const Segment SGI291681[] = {{&S52253[30], 13}, {NULL}};
const Segment SGI291682[] = {{&S52253[7], 1}, {NULL}};
const Segment SGI291683[] = {{&S52277[75], 4}, {NULL}};
const Segment SGI291684[] = {{&S52277[8], 3}, {NULL}};
const Segment SGI291685[] = {{&S52277[13], 7}, {NULL}};
const Segment SGI291686[] = {{&S52277[25], 14}, {NULL}};
const Segment SGI291687[] = {{&S52277[7], 1}, {NULL}};
const Segment SGI291688[] = {{&S52277[16], 4}, {&S52277[28], 11}, {&S52277[85], 17}, {NULL}};
const Segment SGI291689[] = {{&S52278[71], 4}, {NULL}};
const Segment SGI291690[] = {{&S52278[8], 3}, {NULL}};
const Segment SGI291691[] = {{&S52278[13], 7}, {NULL}};
const Segment SGI291692[] = {{&S52278[25], 14}, {NULL}};
const Segment SGI291693[] = {{&S52278[7], 1}, {NULL}};
const Segment SGI291694[] = {{&S52278[16], 4}, {&S52278[28], 11}, {NULL}};
const Segment SGI291695[] = {{&S52305[66], 4}, {NULL}};
const Segment SGI291696[] = {{&S52305[8], 3}, {NULL}};
const Segment SGI291697[] = {{&S52305[13], 3}, {NULL}};
const Segment SGI291698[] = {{&S52305[60], 10}, {NULL}};
const Segment SGI291699[] = {{&S52305[7], 1}, {NULL}};
const Segment SGI291700[] = {{&S52305[60], 7}, {NULL}};
const Segment SGI291701[] = {{&S52341[54], 5}, {NULL}};
const Segment SGI291702[] = {{&S52341[8], 3}, {NULL}};
const Segment SGI291703[] = {{&S52341[13], 3}, {NULL}};
const Segment SGI291704[] = {{&S52341[7], 1}, {NULL}};
const Segment SGI291705[] = {{&S52341[60], 13}, {NULL}};
const Segment SGI291706[] = {{&S52524[78], 5}, {NULL}};
const Segment SGI291707[] = {{&S52524[17], 3}, {NULL}};
const Segment SGI291708[] = {{&S52524[22], 9}, {NULL}};
const Segment SGI291709[] = {{&S52524[39], 13}, {NULL}};
const Segment SGI291710[] = {{&S52524[16], 1}, {NULL}};
const Segment SGI291711[] = {{&S52603[54], 4}, {&S52603[54], 4}, {NULL}};
const Segment SGI291712[] = {{&S52603[8], 3}, {NULL}};
const Segment SGI291713[] = {{&S52603[13], 14}, {NULL}};
const Segment SGI291714[] = {{&S52603[7], 1}, {&S52603[7], 1}, {NULL}};
const Segment SGI291715[] = {{&S52603[16], 11}, {NULL}};
const Segment SGI291716[] = {{&S52604[59], 4}, {NULL}};
const Segment SGI291717[] = {{&S52604[8], 3}, {NULL}};
const Segment SGI291718[] = {{&S52604[13], 14}, {NULL}};
const Segment SGI291719[] = {{&S52604[7], 1}, {NULL}};
const Segment SGI291720[] = {{&S52604[16], 11}, {NULL}};
const Segment SGI291721[] = {{&S52638[83], 4}, {NULL}};
const Segment SGI291722[] = {{&S52638[8], 3}, {NULL}};
const Segment SGI291723[] = {{&S52638[13], 9}, {NULL}};
const Segment SGI291724[] = {{&S52638[33], 13}, {NULL}};
const Segment SGI291725[] = {{&S52638[7], 1}, {NULL}};
const Segment SGI291726[] = {{&S52876[50], 4}, {NULL}};
const Segment SGI291727[] = {{&S52876[8], 3}, {NULL}};
const Segment SGI291728[] = {{&S52876[13], 3}, {NULL}};
const Segment SGI291729[] = {{&S52876[7], 1}, {NULL}};
const Segment SGI291730[] = {{&S52876[57], 13}, {NULL}};
const Segment SGI291731[] = {{&S52982[53], 5}, {NULL}};
const Segment SGI291732[] = {{&S52982[8], 3}, {NULL}};
const Segment SGI291733[] = {{&S52982[13], 14}, {NULL}};
const Segment SGI291734[] = {{&S52982[7], 1}, {NULL}};
const Segment SGI291735[] = {{&S52982[16], 11}, {NULL}};
const Segment SGI291736[] = {{&S52982[61], 13}, {NULL}};
const Segment SGI291737[] = {{&S52983[60], 5}, {NULL}};
const Segment SGI291738[] = {{&S52983[8], 3}, {NULL}};
const Segment SGI291739[] = {{&S52983[13], 14}, {NULL}};
const Segment SGI291740[] = {{&S52983[7], 1}, {NULL}};
const Segment SGI291741[] = {{&S52983[16], 11}, {NULL}};
const Segment SGI291742[] = {{&S52983[68], 13}, {NULL}};
const Segment SGI291743[] = {{&S53054[53], 5}, {NULL}};
const Segment SGI291744[] = {{&S53054[8], 3}, {NULL}};
const Segment SGI291745[] = {{&S53054[13], 14}, {NULL}};
const Segment SGI291746[] = {{&S53054[7], 1}, {NULL}};
const Segment SGI291747[] = {{&S53054[16], 11}, {NULL}};
const Segment SGI291748[] = {{&S53054[59], 13}, {NULL}};
const Segment SGI291749[] = {{&S53106[69], 5}, {NULL}};
const Segment SGI291750[] = {{&S53106[8], 3}, {NULL}};
const Segment SGI291751[] = {{&S53106[13], 9}, {NULL}};
const Segment SGI291752[] = {{&S53106[30], 13}, {NULL}};
const Segment SGI291753[] = {{&S53106[7], 1}, {NULL}};
const Segment SGI291754[] = {{&S53108[53], 5}, {NULL}};
const Segment SGI291755[] = {{&S53108[8], 3}, {NULL}};
const Segment SGI291756[] = {{&S53108[13], 14}, {NULL}};
const Segment SGI291757[] = {{&S53108[7], 1}, {NULL}};
const Segment SGI291758[] = {{&S53108[16], 11}, {NULL}};
const Segment SGI291759[] = {{&S53108[59], 13}, {NULL}};
const Segment SGI291760[] = {{&S53109[65], 5}, {NULL}};
const Segment SGI291761[] = {{&S53109[8], 3}, {NULL}};
const Segment SGI291762[] = {{&S53109[13], 14}, {NULL}};
const Segment SGI291763[] = {{&S53109[7], 1}, {NULL}};
const Segment SGI291764[] = {{&S53109[16], 11}, {&S53109[32], 2}, {NULL}};
const Segment SGI291765[] = {{&S53109[77], 13}, {NULL}};
const Segment SGI291766[] = {{&S53110[60], 5}, {NULL}};
const Segment SGI291767[] = {{&S53110[8], 3}, {NULL}};
const Segment SGI291768[] = {{&S53110[13], 14}, {NULL}};
const Segment SGI291769[] = {{&S53110[7], 1}, {NULL}};
const Segment SGI291770[] = {{&S53110[16], 11}, {NULL}};
const Segment SGI291771[] = {{&S53110[68], 13}, {NULL}};
const Segment SGI291772[] = {{&S53122[56], 5}, {NULL}};
const Segment SGI291773[] = {{&S53122[8], 3}, {NULL}};
const Segment SGI291774[] = {{&S53122[13], 14}, {NULL}};
const Segment SGI291775[] = {{&S53122[7], 1}, {NULL}};
const Segment SGI291776[] = {{&S53122[16], 11}, {NULL}};
const Segment SGI291777[] = {{&S53250[75], 4}, {NULL}};
const Segment SGI291778[] = {{&S53250[8], 3}, {NULL}};
const Segment SGI291779[] = {{&S53250[13], 7}, {NULL}};
const Segment SGI291780[] = {{&S53250[25], 14}, {NULL}};
const Segment SGI291781[] = {{&S53250[7], 1}, {NULL}};
const Segment SGI291782[] = {{&S53250[16], 4}, {&S53250[28], 11}, {NULL}};
const Segment SGI291783[] = {{&S53258[70], 4}, {NULL}};
const Segment SGI291784[] = {{&S53258[8], 3}, {NULL}};
const Segment SGI291785[] = {{&S53258[13], 3}, {NULL}};
const Segment SGI291786[] = {{&S53258[13], 3}, {&S53258[21], 7}, {NULL}};
const Segment SGI291787[] = {{&S53258[7], 1}, {NULL}};
const Segment SGI291788[] = {{&S53291[59], 4}, {NULL}};
const Segment SGI291789[] = {{&S53291[8], 3}, {NULL}};
const Segment SGI291790[] = {{&S53291[13], 3}, {NULL}};
const Segment SGI291791[] = {{&S53291[7], 1}, {NULL}};
const Segment SGI291792[] = {{&S53291[64], 11}, {NULL}};
const Segment SGI291793[] = {{&S53297[75], 4}, {NULL}};
const Segment SGI291794[] = {{&S53297[8], 3}, {NULL}};
const Segment SGI291795[] = {{&S53297[13], 7}, {NULL}};
const Segment SGI291796[] = {{&S53297[25], 14}, {NULL}};
const Segment SGI291797[] = {{&S53297[7], 1}, {NULL}};
const Segment SGI291798[] = {{&S53297[16], 4}, {&S53297[28], 11}, {NULL}};
const Segment SGI291799[] = {{&S53474[75], 4}, {NULL}};
const Segment SGI291800[] = {{&S53474[8], 3}, {NULL}};
const Segment SGI291801[] = {{&S53474[13], 7}, {NULL}};
const Segment SGI291802[] = {{&S53474[25], 14}, {NULL}};
const Segment SGI291803[] = {{&S53474[7], 1}, {NULL}};
const Segment SGI291804[] = {{&S53474[16], 4}, {&S53474[28], 11}, {NULL}};
const Segment SGI291805[] = {{&S53623[70], 4}, {NULL}};
const Segment SGI291806[] = {{&S53623[8], 3}, {NULL}};
const Segment SGI291807[] = {{&S53623[13], 3}, {NULL}};
const Segment SGI291808[] = {{&S53623[7], 1}, {NULL}};
const Segment SGI291809[] = {{&S53623[79], 13}, {NULL}};
const Segment SGI291810[] = {{&S38478[39], 3}, {NULL}};
const Segment SGI291811[] = {{&S38478[0], 139}, {NULL}};
const Segment SGI291812[] = {{&S38480[39], 3}, {NULL}};
const Segment SGI291813[] = {{&S38480[0], 139}, {NULL}};
const Segment SGI291814[] = {{&S53740[54], 5}, {NULL}};
const Segment SGI291815[] = {{&S53740[8], 3}, {NULL}};
const Segment SGI291816[] = {{&S53740[13], 3}, {NULL}};
const Segment SGI291817[] = {{&S53740[7], 1}, {NULL}};
const Segment SGI291818[] = {{&S53740[60], 13}, {NULL}};
const Segment SGI291819[] = {{&S53757[60], 5}, {NULL}};
const Segment SGI291820[] = {{&S53757[8], 3}, {NULL}};
const Segment SGI291821[] = {{&S53757[13], 14}, {NULL}};
const Segment SGI291822[] = {{&S53757[7], 1}, {NULL}};
const Segment SGI291823[] = {{&S53757[16], 11}, {NULL}};
const Segment SGI291824[] = {{&S53757[66], 14}, {NULL}};
const Segment SGI291825[] = {{&S53761[57], 4}, {NULL}};
const Segment SGI291826[] = {{&S53761[8], 3}, {NULL}};
const Segment SGI291827[] = {{&S53761[13], 3}, {NULL}};
const Segment SGI291828[] = {{&S53761[7], 1}, {NULL}};
const Segment SGI291829[] = {{&S53761[62], 11}, {NULL}};
const Segment SGI291830[] = {{&S53890[55], 5}, {NULL}};
const Segment SGI291831[] = {{&S53890[8], 3}, {NULL}};
const Segment SGI291832[] = {{&S53890[13], 3}, {NULL}};
const Segment SGI291833[] = {{&S53890[13], 3}, {&S53890[18], 5}, {NULL}};
const Segment SGI291834[] = {{&S53890[7], 1}, {NULL}};
const Segment SGI291835[] = {{&S53959[53], 5}, {NULL}};
const Segment SGI291836[] = {{&S53959[8], 3}, {NULL}};
const Segment SGI291837[] = {{&S53959[13], 14}, {NULL}};
const Segment SGI291838[] = {{&S53959[7], 1}, {NULL}};
const Segment SGI291839[] = {{&S53959[16], 11}, {NULL}};
const Segment SGI291840[] = {{&S53959[59], 13}, {NULL}};
const Segment SGI291841[] = {{&S53971[70], 4}, {NULL}};
const Segment SGI291842[] = {{&S53971[8], 3}, {NULL}};
const Segment SGI291843[] = {{&S53971[13], 9}, {NULL}};
const Segment SGI291844[] = {{&S53971[7], 1}, {NULL}};
const Segment SGI291845[] = {{&S53971[77], 11}, {NULL}};
const Segment SGI291846[] = {{&S54010[60], 5}, {NULL}};
const Segment SGI291847[] = {{&S54010[8], 3}, {NULL}};
const Segment SGI291848[] = {{&S54010[13], 14}, {NULL}};
const Segment SGI291849[] = {{&S54010[7], 1}, {NULL}};
const Segment SGI291850[] = {{&S54010[16], 11}, {NULL}};
const Segment SGI291851[] = {{&S54010[66], 13}, {NULL}};
const Segment SGI291852[] = {{&S54055[54], 5}, {NULL}};
const Segment SGI291853[] = {{&S54055[8], 3}, {NULL}};
const Segment SGI291854[] = {{&S54055[13], 3}, {NULL}};
const Segment SGI291855[] = {{&S54055[7], 1}, {NULL}};
const Segment SGI291856[] = {{&S54055[60], 13}, {NULL}};
const Segment SGI291857[] = {{&S54156[65], 5}, {NULL}};
const Segment SGI291858[] = {{&S54156[8], 3}, {NULL}};
const Segment SGI291859[] = {{&S54156[13], 14}, {NULL}};
const Segment SGI291860[] = {{&S54156[7], 1}, {NULL}};
const Segment SGI291861[] = {{&S54156[16], 11}, {&S54156[32], 2}, {NULL}};
const Segment SGI291862[] = {{&S54156[71], 13}, {NULL}};
const Segment SGI291863[] = {{&S54157[60], 5}, {NULL}};
const Segment SGI291864[] = {{&S54157[8], 3}, {NULL}};
const Segment SGI291865[] = {{&S54157[13], 14}, {NULL}};
const Segment SGI291866[] = {{&S54157[7], 1}, {NULL}};
const Segment SGI291867[] = {{&S54157[16], 11}, {NULL}};
const Segment SGI291868[] = {{&S54157[66], 13}, {NULL}};
const Segment SGI291869[] = {{&S54216[66], 5}, {&S54216[66], 5}, {NULL}};
const Segment SGI291870[] = {{&S54216[8], 3}, {NULL}};
const Segment SGI291871[] = {{&S54216[13], 3}, {NULL}};
const Segment SGI291872[] = {{&S54216[13], 3}, {&S54216[18], 6}, {NULL}};
const Segment SGI291873[] = {{&S54216[7], 1}, {&S54216[7], 1}, {NULL}};
const Segment SGI291874[] = {{&S54218[64], 5}, {NULL}};
const Segment SGI291875[] = {{&S54218[8], 3}, {NULL}};
const Segment SGI291876[] = {{&S54218[13], 3}, {NULL}};
const Segment SGI291877[] = {{&S54218[13], 3}, {&S54218[18], 6}, {NULL}};
const Segment SGI291878[] = {{&S54218[7], 1}, {NULL}};
const Segment SGI291879[] = {{&S54219[65], 5}, {NULL}};
const Segment SGI291880[] = {{&S54219[8], 3}, {NULL}};
const Segment SGI291881[] = {{&S54219[13], 14}, {NULL}};
const Segment SGI291882[] = {{&S54219[7], 1}, {NULL}};
const Segment SGI291883[] = {{&S54219[16], 11}, {&S54219[32], 2}, {NULL}};
const Segment SGI291884[] = {{&S54219[77], 13}, {NULL}};
const Segment SGI291885[] = {{&S54220[60], 5}, {NULL}};
const Segment SGI291886[] = {{&S54220[8], 3}, {NULL}};
const Segment SGI291887[] = {{&S54220[13], 14}, {NULL}};
const Segment SGI291888[] = {{&S54220[7], 1}, {NULL}};
const Segment SGI291889[] = {{&S54220[16], 11}, {NULL}};
const Segment SGI291890[] = {{&S54220[68], 13}, {NULL}};
const Segment SGI291891[] = {{&S54334[65], 5}, {NULL}};
const Segment SGI291892[] = {{&S54334[8], 3}, {NULL}};
const Segment SGI291893[] = {{&S54334[13], 14}, {NULL}};
const Segment SGI291894[] = {{&S54334[7], 1}, {NULL}};
const Segment SGI291895[] = {{&S54334[16], 11}, {&S54334[32], 2}, {NULL}};
const Segment SGI291896[] = {{&S54334[75], 13}, {NULL}};
const Segment SGI291897[] = {{&S54335[60], 5}, {NULL}};
const Segment SGI291898[] = {{&S54335[8], 3}, {NULL}};
const Segment SGI291899[] = {{&S54335[13], 14}, {NULL}};
const Segment SGI291900[] = {{&S54335[7], 1}, {NULL}};
const Segment SGI291901[] = {{&S54335[16], 11}, {NULL}};
const Segment SGI291902[] = {{&S54335[68], 13}, {NULL}};
const Segment SGI291903[] = {{&S54362[69], 5}, {NULL}};
const Segment SGI291904[] = {{&S54362[8], 3}, {NULL}};
const Segment SGI291905[] = {{&S54362[13], 9}, {NULL}};
const Segment SGI291906[] = {{&S54362[30], 13}, {NULL}};
const Segment SGI291907[] = {{&S54362[7], 1}, {NULL}};
const Segment SGI291908[] = {{&S54424[75], 4}, {NULL}};
const Segment SGI291909[] = {{&S54424[8], 3}, {NULL}};
const Segment SGI291910[] = {{&S54424[13], 7}, {NULL}};
const Segment SGI291911[] = {{&S54424[25], 14}, {NULL}};
const Segment SGI291912[] = {{&S54424[7], 1}, {NULL}};
const Segment SGI291913[] = {{&S54424[16], 4}, {&S54424[28], 11}, {NULL}};
const Segment SGI291914[] = {{&S54517[69], 5}, {NULL}};
const Segment SGI291915[] = {{&S54517[8], 3}, {NULL}};
const Segment SGI291916[] = {{&S54517[13], 9}, {NULL}};
const Segment SGI291917[] = {{&S54517[30], 13}, {NULL}};
const Segment SGI291918[] = {{&S54517[7], 1}, {NULL}};
const Segment SGI291919[] = {{&S54577[64], 5}, {NULL}};
const Segment SGI291920[] = {{&S54577[8], 3}, {NULL}};
const Segment SGI291921[] = {{&S54577[13], 3}, {NULL}};
const Segment SGI291922[] = {{&S54577[7], 1}, {NULL}};
const Segment SGI291923[] = {{&S54578[56], 5}, {NULL}};
const Segment SGI291924[] = {{&S54578[8], 3}, {NULL}};
const Segment SGI291925[] = {{&S54578[13], 3}, {NULL}};
const Segment SGI291926[] = {{&S54578[7], 1}, {NULL}};
const Segment SGI291927[] = {{&S54604[59], 4}, {NULL}};
const Segment SGI291928[] = {{&S54604[8], 3}, {NULL}};
const Segment SGI291929[] = {{&S54604[13], 14}, {NULL}};
const Segment SGI291930[] = {{&S54604[7], 1}, {NULL}};
const Segment SGI291931[] = {{&S54604[16], 11}, {NULL}};
const Segment SGI291932[] = {{&S54605[64], 4}, {NULL}};
const Segment SGI291933[] = {{&S54605[8], 3}, {NULL}};
const Segment SGI291934[] = {{&S54605[13], 14}, {NULL}};
const Segment SGI291935[] = {{&S54605[7], 1}, {NULL}};
const Segment SGI291936[] = {{&S54605[16], 11}, {&S54605[32], 2}, {NULL}};
const Segment SGI291937[] = {{&S54606[54], 4}, {NULL}};
const Segment SGI291938[] = {{&S54606[8], 3}, {NULL}};
const Segment SGI291939[] = {{&S54606[13], 14}, {NULL}};
const Segment SGI291940[] = {{&S54606[7], 1}, {&S54606[11], 1}, {NULL}};
const Segment SGI291941[] = {{&S54606[16], 11}, {NULL}};
const Segment SGI291942[] = {{&S54608[53], 5}, {NULL}};
const Segment SGI291943[] = {{&S54608[8], 3}, {NULL}};
const Segment SGI291944[] = {{&S54608[13], 14}, {NULL}};
const Segment SGI291945[] = {{&S54608[7], 1}, {NULL}};
const Segment SGI291946[] = {{&S54608[16], 11}, {NULL}};
const Segment SGI291947[] = {{&S54772[53], 5}, {NULL}};
const Segment SGI291948[] = {{&S54772[8], 3}, {NULL}};
const Segment SGI291949[] = {{&S54772[13], 14}, {NULL}};
const Segment SGI291950[] = {{&S54772[7], 1}, {NULL}};
const Segment SGI291951[] = {{&S54772[16], 11}, {NULL}};
const Segment SGI291952[] = {{&S54772[59], 13}, {NULL}};
const Segment SGI291953[] = {{&S54986[54], 5}, {NULL}};
const Segment SGI291954[] = {{&S54986[8], 3}, {NULL}};
const Segment SGI291955[] = {{&S54986[13], 3}, {NULL}};
const Segment SGI291956[] = {{&S54986[7], 1}, {NULL}};
const Segment SGI291957[] = {{&S54986[60], 13}, {NULL}};
const Segment SGI291958[] = {{&S55100[65], 4}, {NULL}};
const Segment SGI291959[] = {{&S55100[8], 3}, {NULL}};
const Segment SGI291960[] = {{&S55100[24], 14}, {NULL}};
const Segment SGI291961[] = {{&S55100[7], 1}, {NULL}};
const Segment SGI291962[] = {{&S55100[15], 4}, {&S55100[27], 11}, {NULL}};
const Segment SGI291963[] = {{&S55101[52], 4}, {&S55101[52], 4}, {NULL}};
const Segment SGI291964[] = {{&S55101[8], 3}, {NULL}};
const Segment SGI291965[] = {{&S55101[13], 14}, {NULL}};
const Segment SGI291966[] = {{&S55101[7], 1}, {&S55101[7], 1}, {NULL}};
const Segment SGI291967[] = {{&S55101[16], 11}, {NULL}};
const Segment SGI291968[] = {{&S55102[76], 4}, {NULL}};
const Segment SGI291969[] = {{&S55102[8], 3}, {NULL}};
const Segment SGI291970[] = {{&S55102[13], 7}, {NULL}};
const Segment SGI291971[] = {{&S55102[25], 14}, {NULL}};
const Segment SGI291972[] = {{&S55102[7], 1}, {NULL}};
const Segment SGI291973[] = {{&S55102[16], 4}, {&S55102[28], 11}, {NULL}};
const Segment SGI291974[] = {{&S55103[54], 4}, {NULL}};
const Segment SGI291975[] = {{&S55103[8], 3}, {NULL}};
const Segment SGI291976[] = {{&S55103[13], 14}, {NULL}};
const Segment SGI291977[] = {{&S55103[7], 1}, {NULL}};
const Segment SGI291978[] = {{&S55103[16], 11}, {NULL}};
const Segment SGI291979[] = {{&S55143[8], 3}, {NULL}};
const Segment SGI291980[] = {{&S55143[13], 7}, {NULL}};
const Segment SGI291981[] = {{&S55143[25], 14}, {NULL}};
const Segment SGI291982[] = {{&S55143[7], 1}, {NULL}};
const Segment SGI291983[] = {{&S55143[16], 4}, {&S55143[28], 11}, {NULL}};
const Segment SGI291984[] = {{&S55143[88], 13}, {NULL}};
const Segment SGI291985[] = {{&S55207[60], 5}, {NULL}};
const Segment SGI291986[] = {{&S55207[8], 3}, {NULL}};
const Segment SGI291987[] = {{&S55207[13], 14}, {NULL}};
const Segment SGI291988[] = {{&S55207[7], 1}, {NULL}};
const Segment SGI291989[] = {{&S55207[16], 11}, {NULL}};
const Segment SGI291990[] = {{&S55207[66], 14}, {NULL}};
const Segment SGI291991[] = {{&S55210[76], 4}, {NULL}};
const Segment SGI291992[] = {{&S55210[8], 3}, {NULL}};
const Segment SGI291993[] = {{&S55210[13], 7}, {NULL}};
const Segment SGI291994[] = {{&S55210[25], 14}, {NULL}};
const Segment SGI291995[] = {{&S55210[7], 1}, {NULL}};
const Segment SGI291996[] = {{&S55210[16], 4}, {&S55210[28], 11}, {NULL}};
const Segment SGI291997[] = {{&S55210[85], 12}, {NULL}};
const Segment SGI291998[] = {{&S55314[69], 5}, {NULL}};
const Segment SGI291999[] = {{&S55314[8], 3}, {NULL}};
const Segment SGI292000[] = {{&S55314[13], 9}, {NULL}};
const Segment SGI292001[] = {{&S55314[30], 13}, {NULL}};
const Segment SGI292002[] = {{&S55314[7], 1}, {NULL}};
const Segment SGI292003[] = {{&S55394[83], 4}, {NULL}};
const Segment SGI292004[] = {{&S55394[8], 3}, {NULL}};
const Segment SGI292005[] = {{&S55394[13], 9}, {NULL}};
const Segment SGI292006[] = {{&S55394[33], 13}, {NULL}};
const Segment SGI292007[] = {{&S55394[7], 1}, {NULL}};
const Segment SGI292008[] = {{&S55433[64], 4}, {NULL}};
const Segment SGI292009[] = {{&S55433[8], 3}, {NULL}};
const Segment SGI292010[] = {{&S55433[13], 3}, {NULL}};
const Segment SGI292011[] = {{&S55433[58], 10}, {NULL}};
const Segment SGI292012[] = {{&S55433[7], 1}, {NULL}};
const Segment SGI292013[] = {{&S55433[58], 7}, {NULL}};
const Segment SGI292014[] = {{&S55496[53], 5}, {NULL}};
const Segment SGI292015[] = {{&S55496[8], 3}, {NULL}};
const Segment SGI292016[] = {{&S55496[13], 14}, {NULL}};
const Segment SGI292017[] = {{&S55496[7], 1}, {NULL}};
const Segment SGI292018[] = {{&S55496[16], 11}, {NULL}};
const Segment SGI292019[] = {{&S55604[55], 4}, {NULL}};
const Segment SGI292020[] = {{&S55604[8], 3}, {NULL}};
const Segment SGI292021[] = {{&S55604[13], 3}, {NULL}};
const Segment SGI292022[] = {{&S55604[7], 1}, {NULL}};
const Segment SGI292023[] = {{&S55604[60], 11}, {NULL}};
const Segment SGI292024[] = {{&S55644[60], 5}, {NULL}};
const Segment SGI292025[] = {{&S55644[8], 3}, {NULL}};
const Segment SGI292026[] = {{&S55644[13], 14}, {NULL}};
const Segment SGI292027[] = {{&S55644[7], 1}, {NULL}};
const Segment SGI292028[] = {{&S55644[16], 11}, {NULL}};
const Segment SGI292029[] = {{&S55644[68], 13}, {NULL}};
const Segment SGI292030[] = {{&S55645[65], 5}, {NULL}};
const Segment SGI292031[] = {{&S55645[8], 3}, {NULL}};
const Segment SGI292032[] = {{&S55645[13], 14}, {NULL}};
const Segment SGI292033[] = {{&S55645[7], 1}, {NULL}};
const Segment SGI292034[] = {{&S55645[16], 11}, {&S55645[32], 2}, {NULL}};
const Segment SGI292035[] = {{&S55645[77], 13}, {NULL}};
const Segment SGI292036[] = {{&S55665[61], 4}, {NULL}};
const Segment SGI292037[] = {{&S55665[8], 3}, {NULL}};
const Segment SGI292038[] = {{&S55665[13], 14}, {NULL}};
const Segment SGI292039[] = {{&S55665[7], 1}, {NULL}};
const Segment SGI292040[] = {{&S55665[16], 11}, {NULL}};
const Segment SGI292041[] = {{&S55681[70], 4}, {NULL}};
const Segment SGI292042[] = {{&S55681[8], 3}, {NULL}};
const Segment SGI292043[] = {{&S55681[13], 3}, {NULL}};
const Segment SGI292044[] = {{&S55681[13], 3}, {&S55681[21], 7}, {NULL}};
const Segment SGI292045[] = {{&S55681[7], 1}, {NULL}};
const Segment SGI292046[] = {{&S55682[8], 3}, {NULL}};
const Segment SGI292047[] = {{&S55682[13], 3}, {NULL}};
const Segment SGI292048[] = {{&S55682[13], 3}, {&S55682[21], 7}, {NULL}};
const Segment SGI292049[] = {{&S55682[61], 11}, {NULL}};
const Segment SGI292050[] = {{&S55752[53], 5}, {NULL}};
const Segment SGI292051[] = {{&S55752[8], 3}, {NULL}};
const Segment SGI292052[] = {{&S55752[13], 14}, {NULL}};
const Segment SGI292053[] = {{&S55752[7], 1}, {NULL}};
const Segment SGI292054[] = {{&S55752[16], 11}, {NULL}};
const Segment SGI292055[] = {{&S55752[59], 13}, {NULL}};
const Segment SGI292056[] = {{&S55854[53], 5}, {NULL}};
const Segment SGI292057[] = {{&S55854[8], 3}, {NULL}};
const Segment SGI292058[] = {{&S55854[13], 14}, {NULL}};
const Segment SGI292059[] = {{&S55854[7], 1}, {NULL}};
const Segment SGI292060[] = {{&S55854[16], 11}, {NULL}};
const Segment SGI292061[] = {{&S55863[53], 5}, {NULL}};
const Segment SGI292062[] = {{&S55863[8], 3}, {NULL}};
const Segment SGI292063[] = {{&S55863[13], 14}, {NULL}};
const Segment SGI292064[] = {{&S55863[7], 1}, {NULL}};
const Segment SGI292065[] = {{&S55863[16], 11}, {NULL}};
const Segment SGI292066[] = {{&S55868[67], 5}, {NULL}};
const Segment SGI292067[] = {{&S55868[8], 3}, {NULL}};
const Segment SGI292068[] = {{&S55868[7], 1}, {NULL}};
const Segment SGI292069[] = {{&S55868[15], 4}, {&S55868[27], 6}, {NULL}};
const Segment SGI292070[] = {{&S55875[74], 4}, {NULL}};
const Segment SGI292071[] = {{&S55875[8], 3}, {NULL}};
const Segment SGI292072[] = {{&S55875[13], 7}, {NULL}};
const Segment SGI292073[] = {{&S55875[7], 1}, {NULL}};
const Segment SGI292074[] = {{&S55875[16], 4}, {&S55875[28], 7}, {NULL}};
const Segment SGI292075[] = {{&S55875[83], 13}, {NULL}};
const Segment SGI292076[] = {{&S55876[69], 4}, {NULL}};
const Segment SGI292077[] = {{&S55876[8], 3}, {NULL}};
const Segment SGI292078[] = {{&S55876[13], 7}, {NULL}};
const Segment SGI292079[] = {{&S55876[7], 1}, {NULL}};
const Segment SGI292080[] = {{&S55876[16], 4}, {&S55876[28], 2}, {NULL}};
const Segment SGI292081[] = {{&S55876[78], 13}, {NULL}};
const Segment SGI292082[] = {{&S55877[75], 4}, {NULL}};
const Segment SGI292083[] = {{&S55877[8], 3}, {NULL}};
const Segment SGI292084[] = {{&S55877[13], 7}, {NULL}};
const Segment SGI292085[] = {{&S55877[7], 1}, {NULL}};
const Segment SGI292086[] = {{&S55877[16], 4}, {&S55877[28], 8}, {NULL}};
const Segment SGI292087[] = {{&S55877[84], 13}, {NULL}};
const Segment SGI292088[] = {{&S55878[62], 5}, {NULL}};
const Segment SGI292089[] = {{&S55878[8], 3}, {NULL}};
const Segment SGI292090[] = {{&S55878[13], 14}, {NULL}};
const Segment SGI292091[] = {{&S55878[7], 1}, {NULL}};
const Segment SGI292092[] = {{&S55878[16], 11}, {NULL}};
const Segment SGI292093[] = {{&S55878[70], 13}, {NULL}};
const Segment SGI292094[] = {{&S55879[67], 5}, {NULL}};
const Segment SGI292095[] = {{&S55879[8], 3}, {NULL}};
const Segment SGI292096[] = {{&S55879[13], 14}, {NULL}};
const Segment SGI292097[] = {{&S55879[7], 1}, {NULL}};
const Segment SGI292098[] = {{&S55879[16], 11}, {&S55879[32], 2}, {NULL}};
const Segment SGI292099[] = {{&S55879[79], 13}, {NULL}};
const Segment SGI292100[] = {{&S56010[53], 5}, {NULL}};
const Segment SGI292101[] = {{&S56010[8], 3}, {NULL}};
const Segment SGI292102[] = {{&S56010[13], 14}, {NULL}};
const Segment SGI292103[] = {{&S56010[7], 1}, {NULL}};
const Segment SGI292104[] = {{&S56010[16], 11}, {NULL}};
const Segment SGI292105[] = {{&S56010[61], 13}, {NULL}};
const Segment SGI292106[] = {{&S56011[65], 5}, {NULL}};
const Segment SGI292107[] = {{&S56011[8], 3}, {NULL}};
const Segment SGI292108[] = {{&S56011[13], 14}, {NULL}};
const Segment SGI292109[] = {{&S56011[7], 1}, {NULL}};
const Segment SGI292110[] = {{&S56011[16], 11}, {&S56011[32], 2}, {NULL}};
const Segment SGI292111[] = {{&S56011[77], 13}, {NULL}};
const Segment SGI292112[] = {{&S56161[53], 5}, {NULL}};
const Segment SGI292113[] = {{&S56161[8], 3}, {NULL}};
const Segment SGI292114[] = {{&S56161[13], 14}, {NULL}};
const Segment SGI292115[] = {{&S56161[7], 1}, {NULL}};
const Segment SGI292116[] = {{&S56161[16], 11}, {NULL}};
const Segment SGI292117[] = {{&S56279[53], 5}, {NULL}};
const Segment SGI292118[] = {{&S56279[8], 3}, {NULL}};
const Segment SGI292119[] = {{&S56279[13], 14}, {NULL}};
const Segment SGI292120[] = {{&S56279[7], 1}, {NULL}};
const Segment SGI292121[] = {{&S56279[16], 11}, {NULL}};
const Segment SGI292122[] = {{&S56279[59], 13}, {NULL}};
const Segment SGI292123[] = {{&S56280[60], 5}, {NULL}};
const Segment SGI292124[] = {{&S56280[8], 3}, {NULL}};
const Segment SGI292125[] = {{&S56280[13], 14}, {NULL}};
const Segment SGI292126[] = {{&S56280[7], 1}, {NULL}};
const Segment SGI292127[] = {{&S56280[16], 11}, {NULL}};
const Segment SGI292128[] = {{&S56280[66], 13}, {NULL}};
const Segment SGI292129[] = {{&S56727[50], 4}, {NULL}};
const Segment SGI292130[] = {{&S56727[8], 3}, {NULL}};
const Segment SGI292131[] = {{&S56727[13], 3}, {NULL}};
const Segment SGI292132[] = {{&S56727[7], 1}, {NULL}};
const Segment SGI292133[] = {{&S56727[57], 10}, {NULL}};
const Segment SGI292134[] = {{&S56740[68], 4}, {NULL}};
const Segment SGI292135[] = {{&S56740[8], 3}, {NULL}};
const Segment SGI292136[] = {{&S56740[13], 9}, {NULL}};
const Segment SGI292137[] = {{&S56740[30], 13}, {NULL}};
const Segment SGI292138[] = {{&S56740[7], 1}, {NULL}};
const Segment SGI292139[] = {{&S56741[70], 4}, {&S56741[70], 4}, {NULL}};
const Segment SGI292140[] = {{&S56741[8], 3}, {NULL}};
const Segment SGI292141[] = {{&S56741[13], 9}, {NULL}};
const Segment SGI292142[] = {{&S56741[30], 13}, {NULL}};
const Segment SGI292143[] = {{&S56741[7], 1}, {&S56741[7], 1}, {NULL}};
const Segment SGI292144[] = {{&S56861[53], 5}, {NULL}};
const Segment SGI292145[] = {{&S56861[8], 3}, {NULL}};
const Segment SGI292146[] = {{&S56861[13], 14}, {NULL}};
const Segment SGI292147[] = {{&S56861[7], 1}, {NULL}};
const Segment SGI292148[] = {{&S56861[16], 11}, {NULL}};
const Segment SGI292149[] = {{&S56905[60], 5}, {NULL}};
const Segment SGI292150[] = {{&S56905[8], 3}, {NULL}};
const Segment SGI292151[] = {{&S56905[13], 14}, {NULL}};
const Segment SGI292152[] = {{&S56905[7], 1}, {NULL}};
const Segment SGI292153[] = {{&S56905[16], 11}, {NULL}};
const Segment SGI292154[] = {{&S56905[68], 13}, {NULL}};
const Segment SGI292155[] = {{&S56993[79], 4}, {NULL}};
const Segment SGI292156[] = {{&S56993[8], 3}, {NULL}};
const Segment SGI292157[] = {{&S56993[13], 3}, {NULL}};
const Segment SGI292158[] = {{&S56993[7], 1}, {NULL}};
const Segment SGI292159[] = {{&S56994[52], 4}, {NULL}};
const Segment SGI292160[] = {{&S56994[8], 3}, {NULL}};
const Segment SGI292161[] = {{&S56994[13], 3}, {NULL}};
const Segment SGI292162[] = {{&S56994[7], 1}, {NULL}};
const Segment SGI292163[] = {{&S56995[55], 4}, {&S56995[55], 4}, {NULL}};
const Segment SGI292164[] = {{&S56995[8], 3}, {NULL}};
const Segment SGI292165[] = {{&S56995[13], 3}, {NULL}};
const Segment SGI292166[] = {{&S56995[7], 1}, {&S56995[7], 1}, {NULL}};
const Segment SGI292167[] = {{&S56998[53], 5}, {NULL}};
const Segment SGI292168[] = {{&S56998[8], 3}, {NULL}};
const Segment SGI292169[] = {{&S56998[13], 14}, {NULL}};
const Segment SGI292170[] = {{&S56998[7], 1}, {NULL}};
const Segment SGI292171[] = {{&S56998[16], 11}, {NULL}};
const Segment SGI292172[] = {{&S57037[99], 5}, {NULL}};
const Segment SGI292173[] = {{&S57037[8], 3}, {NULL}};
const Segment SGI292174[] = {{&S57037[13], 14}, {NULL}};
const Segment SGI292175[] = {{&S57037[13], 14}, {&S57037[13], 14}, {NULL}};
const Segment SGI292176[] = {{&S57037[7], 1}, {NULL}};
const Segment SGI292177[] = {{&S57037[16], 11}, {&S57037[16], 11}, {&S57037[48], 2}, {NULL}};
const Segment SGI292178[] = {{&S57038[87], 5}, {NULL}};
const Segment SGI292179[] = {{&S57038[8], 3}, {NULL}};
const Segment SGI292180[] = {{&S57038[13], 14}, {NULL}};
const Segment SGI292181[] = {{&S57038[13], 14}, {&S57038[13], 14}, {NULL}};
const Segment SGI292182[] = {{&S57038[7], 1}, {NULL}};
const Segment SGI292183[] = {{&S57038[16], 11}, {&S57038[16], 11}, {NULL}};
const Segment SGI292184[] = {{&S57039[83], 5}, {NULL}};
const Segment SGI292185[] = {{&S57039[8], 3}, {NULL}};
const Segment SGI292186[] = {{&S57039[13], 14}, {NULL}};
const Segment SGI292187[] = {{&S57039[7], 1}, {NULL}};
const Segment SGI292188[] = {{&S57039[16], 11}, {&S57039[32], 2}, {NULL}};
const Segment SGI292189[] = {{&S57040[71], 5}, {NULL}};
const Segment SGI292190[] = {{&S57040[8], 3}, {NULL}};
const Segment SGI292191[] = {{&S57040[13], 14}, {NULL}};
const Segment SGI292192[] = {{&S57040[7], 1}, {NULL}};
const Segment SGI292193[] = {{&S57040[16], 11}, {NULL}};
const Segment SGI292194[] = {{&S57041[53], 5}, {NULL}};
const Segment SGI292195[] = {{&S57041[8], 3}, {NULL}};
const Segment SGI292196[] = {{&S57041[13], 14}, {NULL}};
const Segment SGI292197[] = {{&S57041[7], 1}, {NULL}};
const Segment SGI292198[] = {{&S57041[16], 11}, {NULL}};
const Segment SGI292199[] = {{&S41512[39], 3}, {NULL}};
const Segment SGI292200[] = {{&S41512[0], 138}, {NULL}};
const Segment SGI292201[] = {{&S57184[50], 5}, {NULL}};
const Segment SGI292202[] = {{&S57184[8], 3}, {NULL}};
const Segment SGI292203[] = {{&S57184[13], 14}, {NULL}};
const Segment SGI292204[] = {{&S57184[7], 1}, {NULL}};
const Segment SGI292205[] = {{&S57184[16], 11}, {NULL}};
const Segment SGI292206[] = {{&S57184[69], 14}, {NULL}};
const Segment SGI292207[] = {{&S57189[74], 4}, {NULL}};
const Segment SGI292208[] = {{&S57189[8], 3}, {NULL}};
const Segment SGI292209[] = {{&S57189[13], 3}, {NULL}};
const Segment SGI292210[] = {{&S57189[13], 3}, {&S57189[84], 6}, {NULL}};
const Segment SGI292211[] = {{&S57189[91], 6}, {NULL}};
const Segment SGI292212[] = {{&S57189[7], 1}, {NULL}};
const Segment SGI292213[] = {{&S41545[0], 95}, {NULL}};
const Segment SGI292214[] = {{&S57280[53], 5}, {NULL}};
const Segment SGI292215[] = {{&S57280[8], 3}, {NULL}};
const Segment SGI292216[] = {{&S57280[13], 14}, {NULL}};
const Segment SGI292217[] = {{&S57280[7], 1}, {NULL}};
const Segment SGI292218[] = {{&S57280[16], 11}, {NULL}};
const Segment SGI292219[] = {{&S57291[76], 4}, {NULL}};
const Segment SGI292220[] = {{&S57291[8], 3}, {NULL}};
const Segment SGI292221[] = {{&S57291[13], 7}, {NULL}};
const Segment SGI292222[] = {{&S57291[25], 14}, {NULL}};
const Segment SGI292223[] = {{&S57291[7], 1}, {NULL}};
const Segment SGI292224[] = {{&S57291[16], 4}, {&S57291[28], 11}, {NULL}};
const Segment SGI292225[] = {{&S57297[64], 5}, {NULL}};
const Segment SGI292226[] = {{&S57297[8], 3}, {NULL}};
const Segment SGI292227[] = {{&S57297[13], 3}, {NULL}};
const Segment SGI292228[] = {{&S57297[7], 1}, {NULL}};
const Segment SGI292229[] = {{&S57298[56], 5}, {NULL}};
const Segment SGI292230[] = {{&S57298[8], 3}, {NULL}};
const Segment SGI292231[] = {{&S57298[13], 3}, {NULL}};
const Segment SGI292232[] = {{&S57298[7], 1}, {NULL}};
const Segment SGI292233[] = {{&S57384[53], 5}, {NULL}};
const Segment SGI292234[] = {{&S57384[8], 3}, {NULL}};
const Segment SGI292235[] = {{&S57384[13], 14}, {NULL}};
const Segment SGI292236[] = {{&S57384[7], 1}, {NULL}};
const Segment SGI292237[] = {{&S57384[16], 11}, {NULL}};
const Segment SGI292238[] = {{&S57384[59], 9}, {NULL}};
const Segment SGI292239[] = {{&S57385[54], 5}, {NULL}};
const Segment SGI292240[] = {{&S57385[8], 3}, {NULL}};
const Segment SGI292241[] = {{&S57385[13], 3}, {NULL}};
const Segment SGI292242[] = {{&S57385[7], 1}, {NULL}};
const Segment SGI292243[] = {{&S57385[60], 13}, {NULL}};
const Segment SGI292244[] = {{&S57471[47], 5}, {NULL}};
const Segment SGI292245[] = {{&S57471[8], 3}, {NULL}};
const Segment SGI292246[] = {{&S57471[13], 3}, {NULL}};
const Segment SGI292247[] = {{&S57471[13], 3}, {&S57471[18], 7}, {NULL}};
const Segment SGI292248[] = {{&S57471[7], 1}, {NULL}};
const Segment SGI292249[] = {{&S57487[53], 5}, {NULL}};
const Segment SGI292250[] = {{&S57487[8], 3}, {NULL}};
const Segment SGI292251[] = {{&S57487[13], 14}, {NULL}};
const Segment SGI292252[] = {{&S57487[7], 1}, {NULL}};
const Segment SGI292253[] = {{&S57487[16], 11}, {NULL}};
const Segment SGI292254[] = {{&S57508[78], 4}, {NULL}};
const Segment SGI292255[] = {{&S57508[8], 3}, {NULL}};
const Segment SGI292256[] = {{&S57508[13], 9}, {NULL}};
const Segment SGI292257[] = {{&S57508[7], 1}, {NULL}};
const Segment SGI292258[] = {{&S57509[82], 4}, {NULL}};
const Segment SGI292259[] = {{&S57509[8], 3}, {NULL}};
const Segment SGI292260[] = {{&S57509[13], 9}, {NULL}};
const Segment SGI292261[] = {{&S57509[31], 15}, {NULL}};
const Segment SGI292262[] = {{&S57509[7], 1}, {&S57509[11], 1}, {NULL}};
const Segment SGI292263[] = {{&S57510[73], 4}, {NULL}};
const Segment SGI292264[] = {{&S57510[8], 3}, {NULL}};
const Segment SGI292265[] = {{&S57510[13], 9}, {NULL}};
const Segment SGI292266[] = {{&S57510[31], 15}, {NULL}};
const Segment SGI292267[] = {{&S57510[7], 1}, {NULL}};
const Segment SGI292268[] = {{&S57521[86], 4}, {NULL}};
const Segment SGI292269[] = {{&S57521[8], 3}, {NULL}};
const Segment SGI292270[] = {{&S57521[13], 3}, {NULL}};
const Segment SGI292271[] = {{&S57521[13], 3}, {&S57521[58], 6}, {NULL}};
const Segment SGI292272[] = {{&S57521[58], 11}, {NULL}};
const Segment SGI292273[] = {{&S57521[7], 1}, {NULL}};
const Segment SGI292274[] = {{&S57524[53], 5}, {NULL}};
const Segment SGI292275[] = {{&S57524[8], 3}, {NULL}};
const Segment SGI292276[] = {{&S57524[13], 14}, {NULL}};
const Segment SGI292277[] = {{&S57524[7], 1}, {NULL}};
const Segment SGI292278[] = {{&S57524[16], 11}, {NULL}};
const Segment SGI292279[] = {{&S57677[81], 4}, {NULL}};
const Segment SGI292280[] = {{&S57677[8], 3}, {NULL}};
const Segment SGI292281[] = {{&S57677[13], 3}, {NULL}};
const Segment SGI292282[] = {{&S57677[13], 3}, {&S57677[60], 6}, {NULL}};
const Segment SGI292283[] = {{&S57677[7], 1}, {NULL}};
const Segment SGI292284[] = {{&S57780[69], 5}, {NULL}};
const Segment SGI292285[] = {{&S57780[8], 3}, {NULL}};
const Segment SGI292286[] = {{&S57780[13], 9}, {NULL}};
const Segment SGI292287[] = {{&S57780[30], 13}, {NULL}};
const Segment SGI292288[] = {{&S57780[7], 1}, {NULL}};
const Segment SGI292289[] = {{&S57927[65], 5}, {NULL}};
const Segment SGI292290[] = {{&S57927[8], 3}, {NULL}};
const Segment SGI292291[] = {{&S57927[13], 14}, {NULL}};
const Segment SGI292292[] = {{&S57927[7], 1}, {NULL}};
const Segment SGI292293[] = {{&S57927[16], 11}, {&S57927[32], 2}, {NULL}};
const Segment SGI292294[] = {{&S57927[75], 13}, {NULL}};
const Segment SGI292295[] = {{&S57928[60], 5}, {NULL}};
const Segment SGI292296[] = {{&S57928[8], 3}, {NULL}};
const Segment SGI292297[] = {{&S57928[13], 14}, {NULL}};
const Segment SGI292298[] = {{&S57928[7], 1}, {NULL}};
const Segment SGI292299[] = {{&S57928[16], 11}, {NULL}};
const Segment SGI292300[] = {{&S57928[66], 13}, {NULL}};
const Segment SGI292301[] = {{&S57940[60], 5}, {NULL}};
const Segment SGI292302[] = {{&S57940[8], 3}, {NULL}};
const Segment SGI292303[] = {{&S57940[13], 14}, {NULL}};
const Segment SGI292304[] = {{&S57940[7], 1}, {NULL}};
const Segment SGI292305[] = {{&S57940[16], 11}, {NULL}};
const Segment SGI292306[] = {{&S57941[53], 5}, {NULL}};
const Segment SGI292307[] = {{&S57941[8], 3}, {NULL}};
const Segment SGI292308[] = {{&S57941[13], 14}, {NULL}};
const Segment SGI292309[] = {{&S57941[7], 1}, {NULL}};
const Segment SGI292310[] = {{&S57941[16], 11}, {NULL}};
const Segment SGI292311[] = {{&S42163[0], 111}, {NULL}};
const Segment SGI292312[] = {{&S58041[53], 5}, {NULL}};
const Segment SGI292313[] = {{&S58041[8], 3}, {NULL}};
const Segment SGI292314[] = {{&S58041[13], 14}, {NULL}};
const Segment SGI292315[] = {{&S58041[7], 1}, {NULL}};
const Segment SGI292316[] = {{&S58041[16], 11}, {NULL}};
const Segment SGI292317[] = {{&S58045[53], 5}, {NULL}};
const Segment SGI292318[] = {{&S58045[8], 3}, {NULL}};
const Segment SGI292319[] = {{&S58045[13], 14}, {NULL}};
const Segment SGI292320[] = {{&S58045[7], 1}, {NULL}};
const Segment SGI292321[] = {{&S58045[16], 11}, {NULL}};
const Segment SGI292322[] = {{&S58163[53], 5}, {NULL}};
const Segment SGI292323[] = {{&S58163[8], 3}, {NULL}};
const Segment SGI292324[] = {{&S58163[13], 14}, {NULL}};
const Segment SGI292325[] = {{&S58163[7], 1}, {NULL}};
const Segment SGI292326[] = {{&S58163[16], 11}, {NULL}};
const Segment SGI292327[] = {{&S58163[59], 13}, {NULL}};
const Segment SGI292328[] = {{&S58164[65], 5}, {NULL}};
const Segment SGI292329[] = {{&S58164[8], 3}, {NULL}};
const Segment SGI292330[] = {{&S58164[13], 14}, {NULL}};
const Segment SGI292331[] = {{&S58164[7], 1}, {NULL}};
const Segment SGI292332[] = {{&S58164[16], 11}, {&S58164[32], 2}, {NULL}};
const Segment SGI292333[] = {{&S58164[71], 13}, {NULL}};
const Segment SGI292334[] = {{&S58229[53], 5}, {NULL}};
const Segment SGI292335[] = {{&S58229[8], 3}, {NULL}};
const Segment SGI292336[] = {{&S58229[13], 14}, {NULL}};
const Segment SGI292337[] = {{&S58229[7], 1}, {NULL}};
const Segment SGI292338[] = {{&S58229[16], 11}, {NULL}};
const Segment SGI292339[] = {{&S58308[64], 4}, {NULL}};
const Segment SGI292340[] = {{&S58308[8], 3}, {NULL}};
const Segment SGI292341[] = {{&S58308[13], 3}, {NULL}};
const Segment SGI292342[] = {{&S58308[13], 3}, {&S58308[74], 6}, {NULL}};
const Segment SGI292343[] = {{&S58308[81], 4}, {NULL}};
const Segment SGI292344[] = {{&S58308[7], 1}, {NULL}};
const Segment SGI292345[] = {{&S58309[8], 3}, {NULL}};
const Segment SGI292346[] = {{&S58309[13], 3}, {NULL}};
const Segment SGI292347[] = {{&S58309[13], 3}, {&S58309[77], 6}, {NULL}};
const Segment SGI292348[] = {{&S58309[84], 4}, {NULL}};
const Segment SGI292349[] = {{&S58309[62], 11}, {NULL}};
const Segment SGI292350[] = {{&S58367[54], 5}, {NULL}};
const Segment SGI292351[] = {{&S58367[8], 3}, {NULL}};
const Segment SGI292352[] = {{&S58367[13], 3}, {NULL}};
const Segment SGI292353[] = {{&S58367[7], 1}, {NULL}};
const Segment SGI292354[] = {{&S58367[60], 13}, {NULL}};
const Segment SGI292355[] = {{&S58375[55], 5}, {NULL}};
const Segment SGI292356[] = {{&S58375[8], 3}, {NULL}};
const Segment SGI292357[] = {{&S58375[13], 14}, {NULL}};
const Segment SGI292358[] = {{&S58375[7], 1}, {NULL}};
const Segment SGI292359[] = {{&S58375[16], 11}, {NULL}};
const Segment SGI292360[] = {{&S58375[63], 9}, {NULL}};
const Segment SGI292361[] = {{&S58383[75], 4}, {NULL}};
const Segment SGI292362[] = {{&S58383[8], 3}, {NULL}};
const Segment SGI292363[] = {{&S58383[13], 7}, {NULL}};
const Segment SGI292364[] = {{&S58383[25], 14}, {NULL}};
const Segment SGI292365[] = {{&S58383[7], 1}, {NULL}};
const Segment SGI292366[] = {{&S58383[16], 4}, {&S58383[28], 11}, {NULL}};
const Segment SGI292367[] = {{&S58383[82], 13}, {NULL}};
const Segment SGI292368[] = {{&S58384[76], 4}, {NULL}};
const Segment SGI292369[] = {{&S58384[8], 3}, {NULL}};
const Segment SGI292370[] = {{&S58384[13], 7}, {NULL}};
const Segment SGI292371[] = {{&S58384[25], 14}, {NULL}};
const Segment SGI292372[] = {{&S58384[7], 1}, {NULL}};
const Segment SGI292373[] = {{&S58384[16], 4}, {&S58384[28], 11}, {NULL}};
const Segment SGI292374[] = {{&S58384[84], 16}, {NULL}};
const Segment SGI292375[] = {{&S58385[76], 4}, {NULL}};
const Segment SGI292376[] = {{&S58385[8], 3}, {NULL}};
const Segment SGI292377[] = {{&S58385[13], 7}, {NULL}};
const Segment SGI292378[] = {{&S58385[25], 14}, {NULL}};
const Segment SGI292379[] = {{&S58385[7], 1}, {NULL}};
const Segment SGI292380[] = {{&S58385[16], 4}, {&S58385[28], 11}, {NULL}};
const Segment SGI292381[] = {{&S58385[84], 13}, {NULL}};
const Segment SGI292382[] = {{&S58394[53], 5}, {NULL}};
const Segment SGI292383[] = {{&S58394[8], 3}, {NULL}};
const Segment SGI292384[] = {{&S58394[13], 14}, {NULL}};
const Segment SGI292385[] = {{&S58394[7], 1}, {NULL}};
const Segment SGI292386[] = {{&S58394[16], 11}, {NULL}};
const Segment SGI292387[] = {{&S58427[71], 4}, {NULL}};
const Segment SGI292388[] = {{&S58427[8], 3}, {NULL}};
const Segment SGI292389[] = {{&S58427[13], 3}, {NULL}};
const Segment SGI292390[] = {{&S58427[13], 3}, {&S58427[21], 7}, {NULL}};
const Segment SGI292391[] = {{&S58427[7], 1}, {NULL}};
const Segment SGI292392[] = {{&S58444[53], 5}, {NULL}};
const Segment SGI292393[] = {{&S58444[8], 3}, {NULL}};
const Segment SGI292394[] = {{&S58444[13], 14}, {NULL}};
const Segment SGI292395[] = {{&S58444[7], 1}, {NULL}};
const Segment SGI292396[] = {{&S58444[16], 11}, {NULL}};
const Segment SGI292397[] = {{&S58593[53], 5}, {NULL}};
const Segment SGI292398[] = {{&S58593[8], 3}, {NULL}};
const Segment SGI292399[] = {{&S58593[13], 14}, {NULL}};
const Segment SGI292400[] = {{&S58593[7], 1}, {NULL}};
const Segment SGI292401[] = {{&S58593[16], 11}, {NULL}};
const Segment SGI292402[] = {{&S58605[57], 5}, {NULL}};
const Segment SGI292403[] = {{&S58605[8], 3}, {NULL}};
const Segment SGI292404[] = {{&S58605[13], 9}, {NULL}};
const Segment SGI292405[] = {{&S58605[28], 13}, {NULL}};
const Segment SGI292406[] = {{&S58605[7], 1}, {NULL}};
const Segment SGI292407[] = {{&S58629[68], 5}, {NULL}};
const Segment SGI292408[] = {{&S58629[8], 3}, {NULL}};
const Segment SGI292409[] = {{&S58629[13], 14}, {NULL}};
const Segment SGI292410[] = {{&S58629[7], 1}, {NULL}};
const Segment SGI292411[] = {{&S58629[16], 11}, {NULL}};
const Segment SGI292412[] = {{&S58629[38], 13}, {NULL}};
const Segment SGI292413[] = {{&S58630[68], 5}, {NULL}};
const Segment SGI292414[] = {{&S58630[8], 3}, {NULL}};
const Segment SGI292415[] = {{&S58630[13], 14}, {NULL}};
const Segment SGI292416[] = {{&S58630[7], 1}, {NULL}};
const Segment SGI292417[] = {{&S58630[16], 11}, {NULL}};
const Segment SGI292418[] = {{&S58630[38], 13}, {NULL}};
const Segment SGI292419[] = {{&S58631[68], 5}, {NULL}};
const Segment SGI292420[] = {{&S58631[8], 3}, {NULL}};
const Segment SGI292421[] = {{&S58631[13], 14}, {NULL}};
const Segment SGI292422[] = {{&S58631[7], 1}, {NULL}};
const Segment SGI292423[] = {{&S58631[16], 11}, {NULL}};
const Segment SGI292424[] = {{&S58631[38], 13}, {NULL}};
const Segment SGI292425[] = {{&S58632[68], 5}, {NULL}};
const Segment SGI292426[] = {{&S58632[8], 3}, {NULL}};
const Segment SGI292427[] = {{&S58632[13], 14}, {NULL}};
const Segment SGI292428[] = {{&S58632[7], 1}, {NULL}};
const Segment SGI292429[] = {{&S58632[16], 11}, {NULL}};
const Segment SGI292430[] = {{&S58632[38], 13}, {NULL}};
const Segment SGI292431[] = {{&S58633[68], 5}, {NULL}};
const Segment SGI292432[] = {{&S58633[8], 3}, {NULL}};
const Segment SGI292433[] = {{&S58633[13], 14}, {NULL}};
const Segment SGI292434[] = {{&S58633[7], 1}, {NULL}};
const Segment SGI292435[] = {{&S58633[16], 11}, {NULL}};
const Segment SGI292436[] = {{&S58633[38], 13}, {NULL}};
const Segment SGI292437[] = {{&S58634[68], 5}, {NULL}};
const Segment SGI292438[] = {{&S58634[8], 3}, {NULL}};
const Segment SGI292439[] = {{&S58634[13], 14}, {NULL}};
const Segment SGI292440[] = {{&S58634[7], 1}, {NULL}};
const Segment SGI292441[] = {{&S58634[16], 11}, {NULL}};
const Segment SGI292442[] = {{&S58634[38], 13}, {NULL}};
const Segment SGI292443[] = {{&S58635[68], 5}, {NULL}};
const Segment SGI292444[] = {{&S58635[8], 3}, {NULL}};
const Segment SGI292445[] = {{&S58635[13], 14}, {NULL}};
const Segment SGI292446[] = {{&S58635[7], 1}, {NULL}};
const Segment SGI292447[] = {{&S58635[16], 11}, {NULL}};
const Segment SGI292448[] = {{&S58635[38], 13}, {NULL}};
const Segment SGI292449[] = {{&S58636[68], 5}, {NULL}};
const Segment SGI292450[] = {{&S58636[8], 3}, {NULL}};
const Segment SGI292451[] = {{&S58636[13], 14}, {NULL}};
const Segment SGI292452[] = {{&S58636[7], 1}, {NULL}};
const Segment SGI292453[] = {{&S58636[16], 11}, {NULL}};
const Segment SGI292454[] = {{&S58636[38], 13}, {NULL}};
const Segment SGI292455[] = {{&S58637[68], 5}, {NULL}};
const Segment SGI292456[] = {{&S58637[8], 3}, {NULL}};
const Segment SGI292457[] = {{&S58637[13], 14}, {NULL}};
const Segment SGI292458[] = {{&S58637[7], 1}, {NULL}};
const Segment SGI292459[] = {{&S58637[16], 11}, {NULL}};
const Segment SGI292460[] = {{&S58637[38], 13}, {NULL}};
const Segment SGI292461[] = {{&S58638[90], 4}, {NULL}};
const Segment SGI292462[] = {{&S58638[8], 3}, {NULL}};
const Segment SGI292463[] = {{&S58638[13], 3}, {NULL}};
const Segment SGI292464[] = {{&S58638[59], 10}, {NULL}};
const Segment SGI292465[] = {{&S58638[7], 1}, {NULL}};
const Segment SGI292466[] = {{&S58638[59], 7}, {NULL}};
const Segment SGI292467[] = {{&S58690[59], 4}, {NULL}};
const Segment SGI292468[] = {{&S58690[8], 3}, {NULL}};
const Segment SGI292469[] = {{&S58690[13], 14}, {NULL}};
const Segment SGI292470[] = {{&S58690[7], 1}, {NULL}};
const Segment SGI292471[] = {{&S58690[16], 11}, {NULL}};
const Segment SGI292472[] = {{&S58731[67], 4}, {NULL}};
const Segment SGI292473[] = {{&S58731[8], 3}, {NULL}};
const Segment SGI292474[] = {{&S58731[13], 3}, {NULL}};
const Segment SGI292475[] = {{&S58731[61], 10}, {NULL}};
const Segment SGI292476[] = {{&S58731[7], 1}, {NULL}};
const Segment SGI292477[] = {{&S58731[61], 7}, {NULL}};
const Segment SGI292478[] = {{&S59092[53], 5}, {NULL}};
const Segment SGI292479[] = {{&S59092[8], 3}, {NULL}};
const Segment SGI292480[] = {{&S59092[13], 14}, {NULL}};
const Segment SGI292481[] = {{&S59092[7], 1}, {NULL}};
const Segment SGI292482[] = {{&S59092[16], 11}, {NULL}};
const Segment SGI292483[] = {{&S59133[60], 5}, {NULL}};
const Segment SGI292484[] = {{&S59133[8], 3}, {NULL}};
const Segment SGI292485[] = {{&S59133[13], 14}, {NULL}};
const Segment SGI292486[] = {{&S59133[7], 1}, {NULL}};
const Segment SGI292487[] = {{&S59133[16], 11}, {NULL}};
const Segment SGI292488[] = {{&S59133[66], 13}, {NULL}};
const Segment SGI292489[] = {{&S59145[54], 4}, {NULL}};
const Segment SGI292490[] = {{&S59145[8], 3}, {NULL}};
const Segment SGI292491[] = {{&S59145[13], 14}, {NULL}};
const Segment SGI292492[] = {{&S59145[61], 5}, {NULL}};
const Segment SGI292493[] = {{&S59145[7], 1}, {NULL}};
const Segment SGI292494[] = {{&S59145[16], 11}, {NULL}};
const Segment SGI292495[] = {{&S59167[53], 5}, {NULL}};
const Segment SGI292496[] = {{&S59167[8], 3}, {NULL}};
const Segment SGI292497[] = {{&S59167[13], 14}, {NULL}};
const Segment SGI292498[] = {{&S59167[7], 1}, {NULL}};
const Segment SGI292499[] = {{&S59167[16], 11}, {NULL}};
const Segment SGI292500[] = {{&S59550[70], 4}, {NULL}};
const Segment SGI292501[] = {{&S59550[8], 3}, {NULL}};
const Segment SGI292502[] = {{&S59550[13], 7}, {NULL}};
const Segment SGI292503[] = {{&S59550[7], 1}, {NULL}};
const Segment SGI292504[] = {{&S59550[16], 4}, {&S59550[28], 8}, {NULL}};
const Segment SGI292505[] = {{&S59551[66], 4}, {NULL}};
const Segment SGI292506[] = {{&S59551[8], 3}, {NULL}};
const Segment SGI292507[] = {{&S59551[13], 7}, {NULL}};
const Segment SGI292508[] = {{&S59551[7], 1}, {NULL}};
const Segment SGI292509[] = {{&S59551[16], 4}, {&S59551[28], 2}, {NULL}};
const Segment SGI292510[] = {{&S59552[53], 5}, {NULL}};
const Segment SGI292511[] = {{&S59552[8], 3}, {NULL}};
const Segment SGI292512[] = {{&S59552[13], 14}, {NULL}};
const Segment SGI292513[] = {{&S59552[7], 1}, {NULL}};
const Segment SGI292514[] = {{&S59552[16], 11}, {NULL}};
const Segment SGI292515[] = {{&S59552[61], 13}, {NULL}};
const Segment SGI292516[] = {{&S59554[60], 5}, {NULL}};
const Segment SGI292517[] = {{&S59554[8], 3}, {NULL}};
const Segment SGI292518[] = {{&S59554[13], 14}, {NULL}};
const Segment SGI292519[] = {{&S59554[7], 1}, {NULL}};
const Segment SGI292520[] = {{&S59554[16], 11}, {NULL}};
const Segment SGI292521[] = {{&S59554[68], 13}, {NULL}};
const Segment SGI292522[] = {{&S59720[53], 5}, {NULL}};
const Segment SGI292523[] = {{&S59720[8], 3}, {NULL}};
const Segment SGI292524[] = {{&S59720[13], 14}, {NULL}};
const Segment SGI292525[] = {{&S59720[7], 1}, {NULL}};
const Segment SGI292526[] = {{&S59720[16], 11}, {NULL}};
const Segment SGI292527[] = {{&S59730[88], 4}, {NULL}};
const Segment SGI292528[] = {{&S59730[8], 3}, {NULL}};
const Segment SGI292529[] = {{&S59730[13], 3}, {NULL}};
const Segment SGI292530[] = {{&S59730[13], 3}, {&S59730[60], 6}, {NULL}};
const Segment SGI292531[] = {{&S59730[60], 11}, {NULL}};
const Segment SGI292532[] = {{&S59730[7], 1}, {NULL}};
const Segment SGI292533[] = {{&S59731[17], 1}, {NULL}};
const Segment SGI292534[] = {{&S59731[85], 4}, {NULL}};
const Segment SGI292535[] = {{&S59731[8], 3}, {NULL}};
const Segment SGI292536[] = {{&S59731[13], 3}, {NULL}};
const Segment SGI292537[] = {{&S59731[13], 3}, {&S59731[57], 6}, {NULL}};
const Segment SGI292538[] = {{&S59731[57], 11}, {NULL}};
const Segment SGI292539[] = {{&S59731[7], 1}, {NULL}};
const Segment SGI292540[] = {{&S59768[60], 5}, {NULL}};
const Segment SGI292541[] = {{&S59768[8], 3}, {NULL}};
const Segment SGI292542[] = {{&S59768[13], 14}, {NULL}};
const Segment SGI292543[] = {{&S59768[7], 1}, {NULL}};
const Segment SGI292544[] = {{&S59768[16], 11}, {NULL}};
const Segment SGI292545[] = {{&S59768[66], 13}, {NULL}};
const Segment SGI292546[] = {{&S59815[61], 4}, {NULL}};
const Segment SGI292547[] = {{&S59815[8], 3}, {NULL}};
const Segment SGI292548[] = {{&S59815[13], 14}, {NULL}};
const Segment SGI292549[] = {{&S59815[7], 1}, {NULL}};
const Segment SGI292550[] = {{&S59815[16], 11}, {NULL}};
const Segment SGI292551[] = {{&S59816[54], 4}, {&S59816[54], 4}, {NULL}};
const Segment SGI292552[] = {{&S59816[8], 3}, {NULL}};
const Segment SGI292553[] = {{&S59816[13], 14}, {NULL}};
const Segment SGI292554[] = {{&S59816[7], 1}, {&S59816[7], 1}, {NULL}};
const Segment SGI292555[] = {{&S59816[16], 11}, {NULL}};
const Segment SGI292556[] = {{&S59817[70], 4}, {NULL}};
const Segment SGI292557[] = {{&S59817[8], 3}, {NULL}};
const Segment SGI292558[] = {{&S59817[13], 14}, {NULL}};
const Segment SGI292559[] = {{&S59817[7], 1}, {NULL}};
const Segment SGI292560[] = {{&S59817[16], 11}, {&S59817[32], 2}, {NULL}};
const Segment SGI292561[] = {{&S59818[63], 4}, {NULL}};
const Segment SGI292562[] = {{&S59818[8], 3}, {NULL}};
const Segment SGI292563[] = {{&S59818[13], 7}, {NULL}};
const Segment SGI292564[] = {{&S59818[22], 14}, {NULL}};
const Segment SGI292565[] = {{&S59818[7], 1}, {NULL}};
const Segment SGI292566[] = {{&S59818[16], 4}, {&S59818[25], 11}, {NULL}};
const Segment SGI292567[] = {{&S59829[53], 5}, {NULL}};
const Segment SGI292568[] = {{&S59829[8], 3}, {NULL}};
const Segment SGI292569[] = {{&S59829[13], 14}, {NULL}};
const Segment SGI292570[] = {{&S59829[7], 1}, {NULL}};
const Segment SGI292571[] = {{&S59829[16], 11}, {NULL}};
const Segment SGI292572[] = {{&S59829[59], 13}, {NULL}};
const Segment SGI292573[] = {{&S59951[65], 5}, {NULL}};
const Segment SGI292574[] = {{&S59951[8], 3}, {NULL}};
const Segment SGI292575[] = {{&S59951[13], 14}, {NULL}};
const Segment SGI292576[] = {{&S59951[7], 1}, {NULL}};
const Segment SGI292577[] = {{&S59951[16], 11}, {&S59951[32], 2}, {NULL}};
const Segment SGI292578[] = {{&S59951[75], 13}, {NULL}};
const Segment SGI292579[] = {{&S59952[60], 5}, {NULL}};
const Segment SGI292580[] = {{&S59952[8], 3}, {NULL}};
const Segment SGI292581[] = {{&S59952[13], 14}, {NULL}};
const Segment SGI292582[] = {{&S59952[7], 1}, {NULL}};
const Segment SGI292583[] = {{&S59952[16], 11}, {NULL}};
const Segment SGI292584[] = {{&S59952[66], 13}, {NULL}};
const Segment SGI292585[] = {{&S60081[60], 5}, {NULL}};
const Segment SGI292586[] = {{&S60081[8], 3}, {NULL}};
const Segment SGI292587[] = {{&S60081[13], 14}, {NULL}};
const Segment SGI292588[] = {{&S60081[7], 1}, {NULL}};
const Segment SGI292589[] = {{&S60081[16], 11}, {NULL}};
const Segment SGI292590[] = {{&S60081[66], 13}, {NULL}};
const Segment SGI292591[] = {{&S60117[60], 5}, {NULL}};
const Segment SGI292592[] = {{&S60117[8], 3}, {NULL}};
const Segment SGI292593[] = {{&S60117[13], 14}, {NULL}};
const Segment SGI292594[] = {{&S60117[7], 1}, {NULL}};
const Segment SGI292595[] = {{&S60117[16], 11}, {NULL}};
const Segment SGI292596[] = {{&S60117[66], 13}, {NULL}};
const Segment SGI292597[] = {{&S60128[75], 5}, {NULL}};
const Segment SGI292598[] = {{&S60128[29], 3}, {NULL}};
const Segment SGI292599[] = {{&S60128[34], 3}, {NULL}};
const Segment SGI292600[] = {{&S60128[8], 5}, {NULL}};
const Segment SGI292601[] = {{&S60128[7], 1}, {NULL}};
const Segment SGI292602[] = {{&S60224[76], 4}, {NULL}};
const Segment SGI292603[] = {{&S60224[8], 3}, {NULL}};
const Segment SGI292604[] = {{&S60224[13], 7}, {NULL}};
const Segment SGI292605[] = {{&S60224[25], 14}, {NULL}};
const Segment SGI292606[] = {{&S60224[7], 1}, {NULL}};
const Segment SGI292607[] = {{&S60224[16], 4}, {&S60224[28], 11}, {NULL}};
const Segment SGI292608[] = {{&S60224[86], 9}, {NULL}};
const Segment SGI292609[] = {{&S60225[76], 4}, {NULL}};
const Segment SGI292610[] = {{&S60225[8], 3}, {NULL}};
const Segment SGI292611[] = {{&S60225[13], 7}, {NULL}};
const Segment SGI292612[] = {{&S60225[25], 14}, {NULL}};
const Segment SGI292613[] = {{&S60225[7], 1}, {NULL}};
const Segment SGI292614[] = {{&S60225[16], 4}, {&S60225[28], 11}, {NULL}};
const Segment SGI292615[] = {{&S60225[86], 9}, {NULL}};
const Segment SGI292616[] = {{&S60271[60], 5}, {NULL}};
const Segment SGI292617[] = {{&S60271[8], 3}, {NULL}};
const Segment SGI292618[] = {{&S60271[13], 14}, {NULL}};
const Segment SGI292619[] = {{&S60271[7], 1}, {NULL}};
const Segment SGI292620[] = {{&S60271[16], 11}, {NULL}};
const Segment SGI292621[] = {{&S60271[68], 13}, {NULL}};
const Segment SGI292622[] = {{&S44317[39], 3}, {NULL}};
const Segment SGI292623[] = {{&S44317[0], 138}, {NULL}};
const Segment SGI292624[] = {{&S60485[69], 5}, {NULL}};
const Segment SGI292625[] = {{&S60485[8], 3}, {NULL}};
const Segment SGI292626[] = {{&S60485[13], 9}, {NULL}};
const Segment SGI292627[] = {{&S60485[30], 13}, {NULL}};
const Segment SGI292628[] = {{&S60485[7], 1}, {NULL}};
const Segment SGI292629[] = {{&S60485[75], 13}, {NULL}};
const Segment SGI292630[] = {{&S60496[69], 5}, {NULL}};
const Segment SGI292631[] = {{&S60496[8], 3}, {NULL}};
const Segment SGI292632[] = {{&S60496[13], 9}, {NULL}};
const Segment SGI292633[] = {{&S60496[30], 13}, {NULL}};
const Segment SGI292634[] = {{&S60496[7], 1}, {NULL}};
const Segment SGI292635[] = {{&S60497[69], 5}, {&S60497[69], 5}, {NULL}};
const Segment SGI292636[] = {{&S60497[8], 3}, {NULL}};
const Segment SGI292637[] = {{&S60497[13], 9}, {NULL}};
const Segment SGI292638[] = {{&S60497[30], 13}, {NULL}};
const Segment SGI292639[] = {{&S60497[7], 1}, {&S60497[7], 1}, {NULL}};
const Segment SGI292640[] = {{&S60500[69], 4}, {NULL}};
const Segment SGI292641[] = {{&S60500[8], 3}, {NULL}};
const Segment SGI292642[] = {{&S60500[13], 3}, {NULL}};
const Segment SGI292643[] = {{&S60500[13], 3}, {&S60500[21], 7}, {NULL}};
const Segment SGI292644[] = {{&S60500[7], 1}, {NULL}};
const Segment SGI292645[] = {{&S60501[53], 5}, {NULL}};
const Segment SGI292646[] = {{&S60501[8], 3}, {NULL}};
const Segment SGI292647[] = {{&S60501[13], 14}, {NULL}};
const Segment SGI292648[] = {{&S60501[7], 1}, {NULL}};
const Segment SGI292649[] = {{&S60501[16], 11}, {NULL}};
const Segment SGI292650[] = {{&S60501[61], 13}, {NULL}};
const Segment SGI292651[] = {{&S60502[65], 5}, {NULL}};
const Segment SGI292652[] = {{&S60502[8], 3}, {NULL}};
const Segment SGI292653[] = {{&S60502[13], 14}, {NULL}};
const Segment SGI292654[] = {{&S60502[7], 1}, {NULL}};
const Segment SGI292655[] = {{&S60502[16], 11}, {&S60502[32], 2}, {NULL}};
const Segment SGI292656[] = {{&S60502[77], 13}, {NULL}};
const Segment SGI292657[] = {{&S60631[53], 5}, {NULL}};
const Segment SGI292658[] = {{&S60631[8], 3}, {NULL}};
const Segment SGI292659[] = {{&S60631[13], 14}, {NULL}};
const Segment SGI292660[] = {{&S60631[7], 1}, {NULL}};
const Segment SGI292661[] = {{&S60631[16], 11}, {NULL}};
const Segment SGI292662[] = {{&S60712[53], 5}, {NULL}};
const Segment SGI292663[] = {{&S60712[8], 3}, {NULL}};
const Segment SGI292664[] = {{&S60712[13], 14}, {NULL}};
const Segment SGI292665[] = {{&S60712[7], 1}, {NULL}};
const Segment SGI292666[] = {{&S60712[16], 11}, {NULL}};
const Segment SGI292667[] = {{&S60713[55], 5}, {NULL}};
const Segment SGI292668[] = {{&S60713[8], 3}, {NULL}};
const Segment SGI292669[] = {{&S60713[13], 3}, {NULL}};
const Segment SGI292670[] = {{&S60713[13], 3}, {&S60713[18], 5}, {NULL}};
const Segment SGI292671[] = {{&S60713[7], 1}, {NULL}};
const Segment SGI292672[] = {{&S60954[53], 5}, {NULL}};
const Segment SGI292673[] = {{&S60954[8], 3}, {NULL}};
const Segment SGI292674[] = {{&S60954[13], 14}, {NULL}};
const Segment SGI292675[] = {{&S60954[7], 1}, {NULL}};
const Segment SGI292676[] = {{&S60954[16], 11}, {NULL}};
const Segment SGI292677[] = {{&S60954[59], 13}, {NULL}};
const Segment SGI292678[] = {{&S60955[65], 5}, {NULL}};
const Segment SGI292679[] = {{&S60955[8], 3}, {NULL}};
const Segment SGI292680[] = {{&S60955[13], 14}, {NULL}};
const Segment SGI292681[] = {{&S60955[7], 1}, {NULL}};
const Segment SGI292682[] = {{&S60955[16], 11}, {&S60955[32], 2}, {NULL}};
const Segment SGI292683[] = {{&S60955[75], 13}, {NULL}};
const Segment SGI292684[] = {{&S60986[53], 5}, {NULL}};
const Segment SGI292685[] = {{&S60986[8], 3}, {NULL}};
const Segment SGI292686[] = {{&S60986[13], 14}, {NULL}};
const Segment SGI292687[] = {{&S60986[7], 1}, {NULL}};
const Segment SGI292688[] = {{&S60986[16], 11}, {NULL}};
const Segment SGI292689[] = {{&S60986[61], 13}, {NULL}};
const Segment SGI292690[] = {{&S60988[60], 5}, {NULL}};
const Segment SGI292691[] = {{&S60988[8], 3}, {NULL}};
const Segment SGI292692[] = {{&S60988[13], 14}, {NULL}};
const Segment SGI292693[] = {{&S60988[7], 1}, {NULL}};
const Segment SGI292694[] = {{&S60988[16], 11}, {NULL}};
const Segment SGI292695[] = {{&S60988[68], 13}, {NULL}};
const Segment SGI292696[] = {{&S61079[76], 4}, {NULL}};
const Segment SGI292697[] = {{&S61079[8], 3}, {NULL}};
const Segment SGI292698[] = {{&S61079[13], 7}, {NULL}};
const Segment SGI292699[] = {{&S61079[25], 14}, {NULL}};
const Segment SGI292700[] = {{&S61079[7], 1}, {NULL}};
const Segment SGI292701[] = {{&S61079[16], 4}, {&S61079[28], 11}, {NULL}};
const Segment SGI292702[] = {{&S61079[84], 16}, {NULL}};
const Segment SGI292703[] = {{&S61080[73], 4}, {NULL}};
const Segment SGI292704[] = {{&S61080[8], 3}, {NULL}};
const Segment SGI292705[] = {{&S61080[13], 7}, {NULL}};
const Segment SGI292706[] = {{&S61080[25], 14}, {NULL}};
const Segment SGI292707[] = {{&S61080[7], 1}, {NULL}};
const Segment SGI292708[] = {{&S61080[16], 4}, {&S61080[28], 11}, {NULL}};
const Segment SGI292709[] = {{&S61080[81], 13}, {NULL}};
const Segment SGI292710[] = {{&S61145[74], 4}, {NULL}};
const Segment SGI292711[] = {{&S61145[8], 3}, {NULL}};
const Segment SGI292712[] = {{&S61145[13], 9}, {NULL}};
const Segment SGI292713[] = {{&S61145[30], 15}, {NULL}};
const Segment SGI292714[] = {{&S61145[7], 1}, {NULL}};
const Segment SGI292715[] = {{&S61145[79], 11}, {NULL}};
const Segment SGI292716[] = {{&S61146[68], 4}, {NULL}};
const Segment SGI292717[] = {{&S61146[8], 3}, {NULL}};
const Segment SGI292718[] = {{&S61146[13], 9}, {NULL}};
const Segment SGI292719[] = {{&S61146[30], 13}, {NULL}};
const Segment SGI292720[] = {{&S61146[7], 1}, {NULL}};
const Segment SGI292721[] = {{&S61226[75], 4}, {NULL}};
const Segment SGI292722[] = {{&S61226[8], 3}, {NULL}};
const Segment SGI292723[] = {{&S61226[13], 7}, {NULL}};
const Segment SGI292724[] = {{&S61226[25], 14}, {NULL}};
const Segment SGI292725[] = {{&S61226[7], 1}, {NULL}};
const Segment SGI292726[] = {{&S61226[16], 4}, {&S61226[28], 11}, {NULL}};
const Segment SGI292727[] = {{&S61387[65], 5}, {NULL}};
const Segment SGI292728[] = {{&S61387[8], 3}, {NULL}};
const Segment SGI292729[] = {{&S61387[13], 14}, {NULL}};
const Segment SGI292730[] = {{&S61387[7], 1}, {NULL}};
const Segment SGI292731[] = {{&S61387[16], 11}, {&S61387[32], 2}, {NULL}};
const Segment SGI292732[] = {{&S61387[77], 13}, {NULL}};
const Segment SGI292733[] = {{&S61388[60], 5}, {NULL}};
const Segment SGI292734[] = {{&S61388[8], 3}, {NULL}};
const Segment SGI292735[] = {{&S61388[13], 14}, {NULL}};
const Segment SGI292736[] = {{&S61388[7], 1}, {NULL}};
const Segment SGI292737[] = {{&S61388[16], 11}, {NULL}};
const Segment SGI292738[] = {{&S61388[68], 13}, {NULL}};
const Segment SGI292739[] = {{&S61429[69], 5}, {NULL}};
const Segment SGI292740[] = {{&S61429[8], 3}, {NULL}};
const Segment SGI292741[] = {{&S61429[13], 9}, {NULL}};
const Segment SGI292742[] = {{&S61429[30], 13}, {NULL}};
const Segment SGI292743[] = {{&S61429[7], 1}, {NULL}};
const Segment SGI292744[] = {{&S61429[75], 13}, {NULL}};
const Segment SGI292745[] = {{&S61595[75], 4}, {NULL}};
const Segment SGI292746[] = {{&S61595[8], 3}, {NULL}};
const Segment SGI292747[] = {{&S61595[13], 7}, {NULL}};
const Segment SGI292748[] = {{&S61595[25], 14}, {NULL}};
const Segment SGI292749[] = {{&S61595[7], 1}, {NULL}};
const Segment SGI292750[] = {{&S61595[16], 4}, {&S61595[28], 11}, {NULL}};
const Segment SGI292751[] = {{&S61595[82], 9}, {NULL}};
const Segment SGI292752[] = {{&S61596[76], 4}, {NULL}};
const Segment SGI292753[] = {{&S61596[8], 3}, {NULL}};
const Segment SGI292754[] = {{&S61596[13], 7}, {NULL}};
const Segment SGI292755[] = {{&S61596[25], 14}, {NULL}};
const Segment SGI292756[] = {{&S61596[7], 1}, {NULL}};
const Segment SGI292757[] = {{&S61596[16], 4}, {&S61596[28], 11}, {NULL}};
const Segment SGI292758[] = {{&S61596[84], 9}, {NULL}};
const Segment SGI292759[] = {{&S61617[62], 4}, {NULL}};
const Segment SGI292760[] = {{&S61617[8], 3}, {NULL}};
const Segment SGI292761[] = {{&S61617[13], 3}, {NULL}};
const Segment SGI292762[] = {{&S61617[7], 1}, {NULL}};
const Segment SGI292763[] = {{&S61617[67], 10}, {NULL}};
const Segment SGI292764[] = {{&S61722[54], 5}, {NULL}};
const Segment SGI292765[] = {{&S61722[8], 3}, {NULL}};
const Segment SGI292766[] = {{&S61722[13], 15}, {NULL}};
const Segment SGI292767[] = {{&S61722[7], 1}, {NULL}};
const Segment SGI292768[] = {{&S61722[16], 12}, {NULL}};
const Segment SGI292769[] = {{&S61799[78], 4}, {NULL}};
const Segment SGI292770[] = {{&S61799[8], 3}, {NULL}};
const Segment SGI292771[] = {{&S61799[13], 7}, {NULL}};
const Segment SGI292772[] = {{&S61799[25], 14}, {NULL}};
const Segment SGI292773[] = {{&S61799[7], 1}, {NULL}};
const Segment SGI292774[] = {{&S61799[16], 4}, {&S61799[28], 11}, {NULL}};
const Segment SGI292775[] = {{&S61799[87], 13}, {NULL}};
const Segment SGI292776[] = {{&S61800[79], 4}, {NULL}};
const Segment SGI292777[] = {{&S61800[8], 3}, {NULL}};
const Segment SGI292778[] = {{&S61800[13], 7}, {NULL}};
const Segment SGI292779[] = {{&S61800[25], 14}, {NULL}};
const Segment SGI292780[] = {{&S61800[7], 1}, {NULL}};
const Segment SGI292781[] = {{&S61800[16], 4}, {&S61800[28], 11}, {NULL}};
const Segment SGI292782[] = {{&S61800[89], 13}, {NULL}};
const Segment SGI292783[] = {{&S61878[73], 4}, {NULL}};
const Segment SGI292784[] = {{&S61878[8], 3}, {NULL}};
const Segment SGI292785[] = {{&S61878[13], 7}, {NULL}};
const Segment SGI292786[] = {{&S61878[25], 14}, {NULL}};
const Segment SGI292787[] = {{&S61878[7], 1}, {NULL}};
const Segment SGI292788[] = {{&S61878[16], 4}, {&S61878[28], 11}, {NULL}};
const Segment SGI292789[] = {{&S61905[53], 5}, {NULL}};
const Segment SGI292790[] = {{&S61905[8], 3}, {NULL}};
const Segment SGI292791[] = {{&S61905[13], 14}, {NULL}};
const Segment SGI292792[] = {{&S61905[7], 1}, {NULL}};
const Segment SGI292793[] = {{&S61905[16], 11}, {NULL}};
const Segment SGI292794[] = {{&S61905[59], 14}, {NULL}};
const Segment SGI292795[] = {{&S61906[53], 5}, {NULL}};
const Segment SGI292796[] = {{&S61906[8], 3}, {NULL}};
const Segment SGI292797[] = {{&S61906[13], 14}, {NULL}};
const Segment SGI292798[] = {{&S61906[7], 1}, {NULL}};
const Segment SGI292799[] = {{&S61906[16], 11}, {NULL}};
const Segment SGI292800[] = {{&S61906[59], 14}, {NULL}};
const Segment SGI292801[] = {{&S62047[75], 5}, {NULL}};
const Segment SGI292802[] = {{&S62047[8], 3}, {NULL}};
const Segment SGI292803[] = {{&S62047[13], 14}, {NULL}};
const Segment SGI292804[] = {{&S62047[7], 1}, {NULL}};
const Segment SGI292805[] = {{&S62047[16], 11}, {NULL}};
const Segment SGI292806[] = {{&S62047[45], 13}, {NULL}};
const Segment SGI292807[] = {{&S62048[75], 5}, {NULL}};
const Segment SGI292808[] = {{&S62048[8], 3}, {NULL}};
const Segment SGI292809[] = {{&S62048[13], 14}, {NULL}};
const Segment SGI292810[] = {{&S62048[7], 1}, {NULL}};
const Segment SGI292811[] = {{&S62048[16], 11}, {NULL}};
const Segment SGI292812[] = {{&S62048[45], 13}, {NULL}};
const Segment SGI292813[] = {{&S62049[75], 5}, {NULL}};
const Segment SGI292814[] = {{&S62049[8], 3}, {NULL}};
const Segment SGI292815[] = {{&S62049[13], 14}, {NULL}};
const Segment SGI292816[] = {{&S62049[7], 1}, {NULL}};
const Segment SGI292817[] = {{&S62049[16], 11}, {NULL}};
const Segment SGI292818[] = {{&S62049[45], 13}, {NULL}};
const Segment SGI292819[] = {{&S62050[75], 5}, {NULL}};
const Segment SGI292820[] = {{&S62050[8], 3}, {NULL}};
const Segment SGI292821[] = {{&S62050[13], 14}, {NULL}};
const Segment SGI292822[] = {{&S62050[7], 1}, {NULL}};
const Segment SGI292823[] = {{&S62050[16], 11}, {NULL}};
const Segment SGI292824[] = {{&S62050[45], 13}, {NULL}};
const Segment SGI292825[] = {{&S62051[75], 5}, {NULL}};
const Segment SGI292826[] = {{&S62051[8], 3}, {NULL}};
const Segment SGI292827[] = {{&S62051[13], 14}, {NULL}};
const Segment SGI292828[] = {{&S62051[7], 1}, {NULL}};
const Segment SGI292829[] = {{&S62051[16], 11}, {NULL}};
const Segment SGI292830[] = {{&S62051[45], 13}, {NULL}};
const Segment SGI292831[] = {{&S62052[75], 5}, {NULL}};
const Segment SGI292832[] = {{&S62052[8], 3}, {NULL}};
const Segment SGI292833[] = {{&S62052[13], 14}, {NULL}};
const Segment SGI292834[] = {{&S62052[7], 1}, {NULL}};
const Segment SGI292835[] = {{&S62052[16], 11}, {NULL}};
const Segment SGI292836[] = {{&S62052[45], 13}, {NULL}};
const Segment SGI292837[] = {{&S62053[75], 5}, {NULL}};
const Segment SGI292838[] = {{&S62053[8], 3}, {NULL}};
const Segment SGI292839[] = {{&S62053[13], 14}, {NULL}};
const Segment SGI292840[] = {{&S62053[7], 1}, {NULL}};
const Segment SGI292841[] = {{&S62053[16], 11}, {NULL}};
const Segment SGI292842[] = {{&S62053[45], 13}, {NULL}};
const Segment SGI292843[] = {{&S62054[75], 5}, {NULL}};
const Segment SGI292844[] = {{&S62054[8], 3}, {NULL}};
const Segment SGI292845[] = {{&S62054[13], 14}, {NULL}};
const Segment SGI292846[] = {{&S62054[7], 1}, {NULL}};
const Segment SGI292847[] = {{&S62054[16], 11}, {NULL}};
const Segment SGI292848[] = {{&S62054[45], 13}, {NULL}};
const Segment SGI292849[] = {{&S62062[67], 4}, {NULL}};
const Segment SGI292850[] = {{&S62062[8], 3}, {NULL}};
const Segment SGI292851[] = {{&S62062[13], 7}, {NULL}};
const Segment SGI292852[] = {{&S62062[7], 1}, {NULL}};
const Segment SGI292853[] = {{&S62062[16], 4}, {&S62062[28], 5}, {NULL}};
const Segment SGI292854[] = {{&S62116[75], 4}, {NULL}};
const Segment SGI292855[] = {{&S62116[8], 3}, {NULL}};
const Segment SGI292856[] = {{&S62116[13], 9}, {NULL}};
const Segment SGI292857[] = {{&S62116[7], 1}, {NULL}};
const Segment SGI292858[] = {{&S62117[82], 4}, {NULL}};
const Segment SGI292859[] = {{&S62117[8], 3}, {NULL}};
const Segment SGI292860[] = {{&S62117[13], 9}, {NULL}};
const Segment SGI292861[] = {{&S62117[31], 15}, {NULL}};
const Segment SGI292862[] = {{&S62117[7], 1}, {NULL}};
const Segment SGI292863[] = {{&S62229[54], 5}, {NULL}};
const Segment SGI292864[] = {{&S62229[8], 3}, {NULL}};
const Segment SGI292865[] = {{&S62229[13], 15}, {NULL}};
const Segment SGI292866[] = {{&S62229[7], 1}, {NULL}};
const Segment SGI292867[] = {{&S62229[16], 12}, {NULL}};
const Segment SGI292868[] = {{&S62260[54], 5}, {NULL}};
const Segment SGI292869[] = {{&S62260[8], 3}, {NULL}};
const Segment SGI292870[] = {{&S62260[13], 3}, {NULL}};
const Segment SGI292871[] = {{&S62260[7], 1}, {NULL}};
const Segment SGI292872[] = {{&S62260[60], 13}, {NULL}};
const Segment SGI292873[] = {{&S62378[57], 5}, {NULL}};
const Segment SGI292874[] = {{&S62378[8], 3}, {NULL}};
const Segment SGI292875[] = {{&S62378[13], 9}, {NULL}};
const Segment SGI292876[] = {{&S62378[28], 13}, {NULL}};
const Segment SGI292877[] = {{&S62378[7], 1}, {NULL}};
const Segment SGI292878[] = {{&S62387[80], 4}, {NULL}};
const Segment SGI292879[] = {{&S62387[8], 3}, {NULL}};
const Segment SGI292880[] = {{&S62387[13], 9}, {NULL}};
const Segment SGI292881[] = {{&S62387[33], 13}, {NULL}};
const Segment SGI292882[] = {{&S62387[7], 1}, {NULL}};
const Segment SGI292883[] = {{&S62465[53], 5}, {NULL}};
const Segment SGI292884[] = {{&S62465[8], 3}, {NULL}};
const Segment SGI292885[] = {{&S62465[13], 14}, {NULL}};
const Segment SGI292886[] = {{&S62465[7], 1}, {NULL}};
const Segment SGI292887[] = {{&S62465[16], 11}, {NULL}};
const Segment SGI292888[] = {{&S62465[59], 13}, {NULL}};
const Segment SGI292889[] = {{&S62467[53], 5}, {NULL}};
const Segment SGI292890[] = {{&S62467[8], 3}, {NULL}};
const Segment SGI292891[] = {{&S62467[13], 14}, {NULL}};
const Segment SGI292892[] = {{&S62467[7], 1}, {NULL}};
const Segment SGI292893[] = {{&S62467[16], 11}, {NULL}};
const Segment SGI292894[] = {{&S62467[59], 13}, {NULL}};
const Segment SGI292895[] = {{&S62474[69], 5}, {NULL}};
const Segment SGI292896[] = {{&S62474[8], 3}, {NULL}};
const Segment SGI292897[] = {{&S62474[13], 9}, {NULL}};
const Segment SGI292898[] = {{&S62474[30], 13}, {NULL}};
const Segment SGI292899[] = {{&S62474[7], 1}, {NULL}};
const Segment SGI292900[] = {{&S62474[75], 13}, {NULL}};
const Segment SGI292901[] = {{&S62688[53], 5}, {NULL}};
const Segment SGI292902[] = {{&S62688[8], 3}, {NULL}};
const Segment SGI292903[] = {{&S62688[13], 14}, {NULL}};
const Segment SGI292904[] = {{&S62688[7], 1}, {NULL}};
const Segment SGI292905[] = {{&S62688[16], 11}, {NULL}};
const Segment SGI292906[] = {{&S62688[59], 14}, {NULL}};
const Segment SGI292907[] = {{&S62771[69], 5}, {NULL}};
const Segment SGI292908[] = {{&S62771[8], 3}, {NULL}};
const Segment SGI292909[] = {{&S62771[13], 9}, {NULL}};
const Segment SGI292910[] = {{&S62771[30], 13}, {NULL}};
const Segment SGI292911[] = {{&S62771[7], 1}, {NULL}};
const Segment SGI292912[] = {{&S62771[75], 13}, {NULL}};
const Segment SGI292913[] = {{&S60749[8], 3}, {NULL}};
const Segment SGI292914[] = {{&S60749[33], 4}, {NULL}};
const Segment SGI292915[] = {{&S60749[63], 6}, {NULL}};
const Segment SGI292916[] = {{&S60749[13], 24}, {NULL}};
const Segment SGI292917[] = {{&S60749[33], 2}, {NULL}};
const Segment SGI292918[] = {{&S62843[53], 5}, {NULL}};
const Segment SGI292919[] = {{&S62843[8], 3}, {NULL}};
const Segment SGI292920[] = {{&S62843[13], 14}, {NULL}};
const Segment SGI292921[] = {{&S62843[7], 1}, {NULL}};
const Segment SGI292922[] = {{&S62843[16], 11}, {NULL}};
const Segment SGI292923[] = {{&S62843[61], 13}, {NULL}};
const Segment SGI292924[] = {{&S62902[68], 4}, {NULL}};
const Segment SGI292925[] = {{&S62902[8], 3}, {NULL}};
const Segment SGI292926[] = {{&S62902[13], 3}, {NULL}};
const Segment SGI292927[] = {{&S62902[13], 3}, {&S62902[21], 5}, {NULL}};
const Segment SGI292928[] = {{&S62902[7], 1}, {NULL}};
const Segment SGI292929[] = {{&S62909[60], 5}, {NULL}};
const Segment SGI292930[] = {{&S62909[8], 3}, {NULL}};
const Segment SGI292931[] = {{&S62909[13], 14}, {NULL}};
const Segment SGI292932[] = {{&S62909[7], 1}, {NULL}};
const Segment SGI292933[] = {{&S62909[16], 11}, {NULL}};
const Segment SGI292934[] = {{&S62909[66], 13}, {NULL}};
const Segment SGI292935[] = {{&S62988[53], 5}, {NULL}};
const Segment SGI292936[] = {{&S62988[8], 3}, {NULL}};
const Segment SGI292937[] = {{&S62988[13], 14}, {NULL}};
const Segment SGI292938[] = {{&S62988[7], 1}, {NULL}};
const Segment SGI292939[] = {{&S62988[16], 11}, {NULL}};
const Segment SGI292940[] = {{&S62988[61], 13}, {NULL}};
const Segment SGI292941[] = {{&S62989[65], 5}, {NULL}};
const Segment SGI292942[] = {{&S62989[8], 3}, {NULL}};
const Segment SGI292943[] = {{&S62989[13], 14}, {NULL}};
const Segment SGI292944[] = {{&S62989[7], 1}, {NULL}};
const Segment SGI292945[] = {{&S62989[16], 11}, {&S62989[32], 2}, {NULL}};
const Segment SGI292946[] = {{&S62989[77], 13}, {NULL}};
const Segment SGI292947[] = {{&S63137[99], 4}, {NULL}};
const Segment SGI292948[] = {{&S63137[8], 3}, {NULL}};
const Segment SGI292949[] = {{&S63137[13], 9}, {NULL}};
const Segment SGI292950[] = {{&S63137[31], 15}, {NULL}};
const Segment SGI292951[] = {{&S63137[7], 1}, {NULL}};
const Segment SGI292952[] = {{&S63137[73], 10}, {NULL}};
const Segment SGI292953[] = {{&S63138[92], 4}, {NULL}};
const Segment SGI292954[] = {{&S63138[8], 3}, {NULL}};
const Segment SGI292955[] = {{&S63138[13], 9}, {NULL}};
const Segment SGI292956[] = {{&S63138[7], 1}, {NULL}};
const Segment SGI292957[] = {{&S63138[68], 10}, {NULL}};
const Segment SGI292958[] = {{&S63146[53], 5}, {NULL}};
const Segment SGI292959[] = {{&S63146[8], 3}, {NULL}};
const Segment SGI292960[] = {{&S63146[13], 14}, {NULL}};
const Segment SGI292961[] = {{&S63146[7], 1}, {NULL}};
const Segment SGI292962[] = {{&S63146[16], 11}, {NULL}};
const Segment SGI292963[] = {{&S63146[59], 13}, {NULL}};
const Segment SGI292964[] = {{&S63147[65], 5}, {NULL}};
const Segment SGI292965[] = {{&S63147[8], 3}, {NULL}};
const Segment SGI292966[] = {{&S63147[13], 14}, {NULL}};
const Segment SGI292967[] = {{&S63147[7], 1}, {NULL}};
const Segment SGI292968[] = {{&S63147[16], 11}, {&S63147[32], 2}, {NULL}};
const Segment SGI292969[] = {{&S63147[71], 13}, {NULL}};
const Segment SGI292970[] = {{&S63230[69], 5}, {NULL}};
const Segment SGI292971[] = {{&S63230[8], 3}, {NULL}};
const Segment SGI292972[] = {{&S63230[13], 9}, {NULL}};
const Segment SGI292973[] = {{&S63230[30], 13}, {NULL}};
const Segment SGI292974[] = {{&S63230[7], 1}, {NULL}};
const Segment SGI292975[] = {{&S63230[75], 13}, {NULL}};
const Segment SGI292976[] = {{&S63266[60], 5}, {NULL}};
const Segment SGI292977[] = {{&S63266[8], 3}, {NULL}};
const Segment SGI292978[] = {{&S63266[13], 14}, {NULL}};
const Segment SGI292979[] = {{&S63266[7], 1}, {NULL}};
const Segment SGI292980[] = {{&S63266[16], 11}, {NULL}};
const Segment SGI292981[] = {{&S63266[66], 14}, {NULL}};
const Segment SGI292982[] = {{&S63291[53], 5}, {NULL}};
const Segment SGI292983[] = {{&S63291[8], 3}, {NULL}};
const Segment SGI292984[] = {{&S63291[13], 14}, {NULL}};
const Segment SGI292985[] = {{&S63291[7], 1}, {NULL}};
const Segment SGI292986[] = {{&S63291[16], 11}, {NULL}};
const Segment SGI292987[] = {{&S63448[53], 5}, {NULL}};
const Segment SGI292988[] = {{&S63448[8], 3}, {NULL}};
const Segment SGI292989[] = {{&S63448[13], 14}, {NULL}};
const Segment SGI292990[] = {{&S63448[7], 1}, {NULL}};
const Segment SGI292991[] = {{&S63448[16], 11}, {NULL}};
const Segment SGI292992[] = {{&S63448[59], 13}, {NULL}};
const Segment SGI292993[] = {{&S63449[65], 5}, {NULL}};
const Segment SGI292994[] = {{&S63449[8], 3}, {NULL}};
const Segment SGI292995[] = {{&S63449[13], 14}, {NULL}};
const Segment SGI292996[] = {{&S63449[7], 1}, {NULL}};
const Segment SGI292997[] = {{&S63449[16], 11}, {&S63449[32], 2}, {NULL}};
const Segment SGI292998[] = {{&S63449[75], 13}, {NULL}};
const Segment SGI292999[] = {{&S63450[60], 5}, {NULL}};
const Segment SGI293000[] = {{&S63450[8], 3}, {NULL}};
const Segment SGI293001[] = {{&S63450[13], 14}, {NULL}};
const Segment SGI293002[] = {{&S63450[7], 1}, {NULL}};
const Segment SGI293003[] = {{&S63450[16], 11}, {NULL}};
const Segment SGI293004[] = {{&S63450[66], 13}, {NULL}};
const Segment SGI293005[] = {{&S63466[67], 4}, {NULL}};
const Segment SGI293006[] = {{&S63466[8], 3}, {NULL}};
const Segment SGI293007[] = {{&S63466[13], 3}, {NULL}};
const Segment SGI293008[] = {{&S63466[13], 3}, {&S63466[21], 7}, {NULL}};
const Segment SGI293009[] = {{&S63466[7], 1}, {NULL}};
const Segment SGI293010[] = {{&S63467[76], 4}, {NULL}};
const Segment SGI293011[] = {{&S63467[8], 3}, {NULL}};
const Segment SGI293012[] = {{&S63467[13], 7}, {NULL}};
const Segment SGI293013[] = {{&S63467[25], 14}, {NULL}};
const Segment SGI293014[] = {{&S63467[7], 1}, {NULL}};
const Segment SGI293015[] = {{&S63467[16], 4}, {&S63467[28], 11}, {NULL}};
const Segment SGI293016[] = {{&S63467[84], 13}, {NULL}};
const Segment SGI293017[] = {{&S63597[53], 5}, {NULL}};
const Segment SGI293018[] = {{&S63597[8], 3}, {NULL}};
const Segment SGI293019[] = {{&S63597[13], 14}, {NULL}};
const Segment SGI293020[] = {{&S63597[7], 1}, {NULL}};
const Segment SGI293021[] = {{&S63597[16], 11}, {NULL}};
const Segment SGI293022[] = {{&S63659[54], 5}, {NULL}};
const Segment SGI293023[] = {{&S63659[8], 3}, {NULL}};
const Segment SGI293024[] = {{&S63659[13], 15}, {NULL}};
const Segment SGI293025[] = {{&S63659[7], 1}, {NULL}};
const Segment SGI293026[] = {{&S63659[16], 12}, {NULL}};
const Segment SGI293027[] = {{&S63741[60], 5}, {NULL}};
const Segment SGI293028[] = {{&S63741[8], 3}, {NULL}};
const Segment SGI293029[] = {{&S63741[13], 14}, {NULL}};
const Segment SGI293030[] = {{&S63741[7], 1}, {NULL}};
const Segment SGI293031[] = {{&S63741[16], 11}, {NULL}};
const Segment SGI293032[] = {{&S63741[66], 13}, {NULL}};
const Segment SGI293033[] = {{&S63825[68], 4}, {NULL}};
const Segment SGI293034[] = {{&S63825[8], 3}, {NULL}};
const Segment SGI293035[] = {{&S63825[13], 3}, {NULL}};
const Segment SGI293036[] = {{&S63825[13], 3}, {&S63825[21], 5}, {NULL}};
const Segment SGI293037[] = {{&S63825[7], 1}, {NULL}};
const Segment SGI293038[] = {{&S63870[57], 5}, {NULL}};
const Segment SGI293039[] = {{&S63870[8], 3}, {NULL}};
const Segment SGI293040[] = {{&S63870[13], 3}, {NULL}};
const Segment SGI293041[] = {{&S63870[13], 3}, {&S63870[18], 7}, {NULL}};
const Segment SGI293042[] = {{&S63870[7], 1}, {NULL}};
const Segment SGI293043[] = {{&S63942[76], 4}, {NULL}};
const Segment SGI293044[] = {{&S63942[8], 3}, {NULL}};
const Segment SGI293045[] = {{&S63942[13], 7}, {NULL}};
const Segment SGI293046[] = {{&S63942[25], 14}, {NULL}};
const Segment SGI293047[] = {{&S63942[7], 1}, {NULL}};
const Segment SGI293048[] = {{&S63942[16], 4}, {&S63942[28], 11}, {NULL}};
const Segment SGI293049[] = {{&S63942[85], 12}, {NULL}};
const Segment SGI293050[] = {{&S63999[69], 4}, {NULL}};
const Segment SGI293051[] = {{&S63999[8], 3}, {NULL}};
const Segment SGI293052[] = {{&S63999[13], 3}, {NULL}};
const Segment SGI293053[] = {{&S63999[7], 1}, {NULL}};
const Segment SGI293054[] = {{&S64089[73], 4}, {NULL}};
const Segment SGI293055[] = {{&S64089[8], 3}, {NULL}};
const Segment SGI293056[] = {{&S64089[13], 7}, {NULL}};
const Segment SGI293057[] = {{&S64089[25], 14}, {NULL}};
const Segment SGI293058[] = {{&S64089[7], 1}, {NULL}};
const Segment SGI293059[] = {{&S64089[16], 4}, {&S64089[28], 11}, {NULL}};
const Segment SGI293060[] = {{&S64169[55], 5}, {NULL}};
const Segment SGI293061[] = {{&S64169[8], 3}, {NULL}};
const Segment SGI293062[] = {{&S64169[13], 14}, {NULL}};
const Segment SGI293063[] = {{&S64169[7], 1}, {NULL}};
const Segment SGI293064[] = {{&S64169[16], 11}, {NULL}};
const Segment SGI293065[] = {{&S64169[63], 9}, {NULL}};
const Segment SGI293066[] = {{&S64170[55], 5}, {NULL}};
const Segment SGI293067[] = {{&S64170[8], 3}, {NULL}};
const Segment SGI293068[] = {{&S64170[13], 14}, {NULL}};
const Segment SGI293069[] = {{&S64170[7], 1}, {NULL}};
const Segment SGI293070[] = {{&S64170[16], 11}, {NULL}};
const Segment SGI293071[] = {{&S64170[63], 9}, {NULL}};
const Segment SGI293072[] = {{&S64184[53], 5}, {NULL}};
const Segment SGI293073[] = {{&S64184[8], 3}, {NULL}};
const Segment SGI293074[] = {{&S64184[13], 14}, {NULL}};
const Segment SGI293075[] = {{&S64184[7], 1}, {NULL}};
const Segment SGI293076[] = {{&S64184[16], 11}, {NULL}};
const Segment SGI293077[] = {{&S64184[59], 14}, {NULL}};
const Segment SGI293078[] = {{&S64232[60], 5}, {NULL}};
const Segment SGI293079[] = {{&S64232[8], 3}, {NULL}};
const Segment SGI293080[] = {{&S64232[13], 14}, {NULL}};
const Segment SGI293081[] = {{&S64232[7], 1}, {NULL}};
const Segment SGI293082[] = {{&S64232[16], 11}, {NULL}};
const Segment SGI293083[] = {{&S64232[66], 13}, {NULL}};
const Segment SGI293084[] = {{&S64233[65], 5}, {NULL}};
const Segment SGI293085[] = {{&S64233[8], 3}, {NULL}};
const Segment SGI293086[] = {{&S64233[13], 14}, {NULL}};
const Segment SGI293087[] = {{&S64233[7], 1}, {NULL}};
const Segment SGI293088[] = {{&S64233[16], 11}, {&S64233[32], 2}, {NULL}};
const Segment SGI293089[] = {{&S64233[77], 13}, {NULL}};
const Segment SGI293090[] = {{&S64434[54], 5}, {NULL}};
const Segment SGI293091[] = {{&S64434[8], 3}, {NULL}};
const Segment SGI293092[] = {{&S64434[13], 15}, {NULL}};
const Segment SGI293093[] = {{&S64434[7], 1}, {NULL}};
const Segment SGI293094[] = {{&S64434[16], 12}, {NULL}};
const Segment SGI293095[] = {{&S64525[69], 5}, {NULL}};
const Segment SGI293096[] = {{&S64525[8], 3}, {NULL}};
const Segment SGI293097[] = {{&S64525[13], 9}, {NULL}};
const Segment SGI293098[] = {{&S64525[30], 13}, {NULL}};
const Segment SGI293099[] = {{&S64525[7], 1}, {NULL}};
const Segment SGI293100[] = {{&S64526[69], 5}, {&S64526[69], 5}, {NULL}};
const Segment SGI293101[] = {{&S64526[8], 3}, {NULL}};
const Segment SGI293102[] = {{&S64526[13], 9}, {NULL}};
const Segment SGI293103[] = {{&S64526[30], 13}, {NULL}};
const Segment SGI293104[] = {{&S64526[7], 1}, {&S64526[7], 1}, {NULL}};
const Segment SGI293105[] = {{&S64546[69], 5}, {NULL}};
const Segment SGI293106[] = {{&S64546[8], 3}, {NULL}};
const Segment SGI293107[] = {{&S64546[13], 9}, {NULL}};
const Segment SGI293108[] = {{&S64546[30], 13}, {NULL}};
const Segment SGI293109[] = {{&S64546[7], 1}, {NULL}};
const Segment SGI293110[] = {{&S64546[75], 13}, {NULL}};
const Segment SGI293111[] = {{&S64619[54], 5}, {NULL}};
const Segment SGI293112[] = {{&S64619[8], 3}, {NULL}};
const Segment SGI293113[] = {{&S64619[13], 15}, {NULL}};
const Segment SGI293114[] = {{&S64619[7], 1}, {NULL}};
const Segment SGI293115[] = {{&S64619[16], 12}, {NULL}};
const Segment SGI293116[] = {{&S64648[38], 4}, {NULL}};
const Segment SGI293117[] = {{&S64648[8], 3}, {NULL}};
const Segment SGI293118[] = {{&S64648[13], 10}, {NULL}};
const Segment SGI293119[] = {{&S64648[7], 1}, {NULL}};
const Segment SGI293120[] = {{&S64648[16], 7}, {NULL}};
const Segment SGI293121[] = {{&S64649[44], 4}, {NULL}};
const Segment SGI293122[] = {{&S64649[8], 3}, {NULL}};
const Segment SGI293123[] = {{&S64649[13], 7}, {NULL}};
const Segment SGI293124[] = {{&S64649[7], 1}, {NULL}};
const Segment SGI293125[] = {{&S64649[16], 4}, {NULL}};
const Segment SGI293126[] = {{&S64651[54], 4}, {NULL}};
const Segment SGI293127[] = {{&S64651[8], 3}, {NULL}};
const Segment SGI293128[] = {{&S64651[13], 3}, {NULL}};
const Segment SGI293129[] = {{&S64651[13], 3}, {&S64651[18], 5}, {NULL}};
const Segment SGI293130[] = {{&S64651[7], 1}, {NULL}};
const Segment SGI293131[] = {{&S64678[55], 5}, {NULL}};
const Segment SGI293132[] = {{&S64678[8], 3}, {NULL}};
const Segment SGI293133[] = {{&S64678[13], 3}, {NULL}};
const Segment SGI293134[] = {{&S64678[13], 3}, {&S64678[18], 5}, {NULL}};
const Segment SGI293135[] = {{&S64678[7], 1}, {NULL}};
const Segment SGI293136[] = {{&S64771[54], 5}, {NULL}};
const Segment SGI293137[] = {{&S64771[8], 3}, {NULL}};
const Segment SGI293138[] = {{&S64771[13], 15}, {NULL}};
const Segment SGI293139[] = {{&S64771[7], 1}, {NULL}};
const Segment SGI293140[] = {{&S64771[16], 12}, {NULL}};
const Segment SGI293141[] = {{&S64879[69], 5}, {NULL}};
const Segment SGI293142[] = {{&S64879[8], 3}, {NULL}};
const Segment SGI293143[] = {{&S64879[13], 9}, {NULL}};
const Segment SGI293144[] = {{&S64879[30], 13}, {NULL}};
const Segment SGI293145[] = {{&S64879[7], 1}, {NULL}};
const Segment SGI293146[] = {{&S64981[75], 4}, {NULL}};
const Segment SGI293147[] = {{&S64981[8], 3}, {NULL}};
const Segment SGI293148[] = {{&S64981[13], 7}, {NULL}};
const Segment SGI293149[] = {{&S64981[25], 14}, {NULL}};
const Segment SGI293150[] = {{&S64981[7], 1}, {NULL}};
const Segment SGI293151[] = {{&S64981[16], 4}, {&S64981[28], 11}, {NULL}};
const Segment SGI293152[] = {{&S64982[65], 4}, {NULL}};
const Segment SGI293153[] = {{&S64982[8], 3}, {NULL}};
const Segment SGI293154[] = {{&S64982[13], 7}, {NULL}};
const Segment SGI293155[] = {{&S64982[7], 1}, {NULL}};
const Segment SGI293156[] = {{&S64982[16], 4}, {&S64982[26], 7}, {NULL}};
const Segment SGI293157[] = {{&S64983[77], 4}, {NULL}};
const Segment SGI293158[] = {{&S64983[8], 3}, {NULL}};
const Segment SGI293159[] = {{&S64983[13], 7}, {NULL}};
const Segment SGI293160[] = {{&S64983[7], 1}, {NULL}};
const Segment SGI293161[] = {{&S64983[16], 4}, {&S64983[28], 13}, {NULL}};
const Segment SGI293162[] = {{&S64984[75], 4}, {NULL}};
const Segment SGI293163[] = {{&S64984[8], 3}, {NULL}};
const Segment SGI293164[] = {{&S64984[13], 7}, {NULL}};
const Segment SGI293165[] = {{&S64984[25], 14}, {NULL}};
const Segment SGI293166[] = {{&S64984[7], 1}, {NULL}};
const Segment SGI293167[] = {{&S64984[16], 4}, {&S64984[28], 11}, {NULL}};
const Segment SGI293168[] = {{&S64985[88], 4}, {NULL}};
const Segment SGI293169[] = {{&S64985[8], 3}, {NULL}};
const Segment SGI293170[] = {{&S64985[13], 7}, {NULL}};
const Segment SGI293171[] = {{&S64985[7], 1}, {NULL}};
const Segment SGI293172[] = {{&S64985[16], 4}, {NULL}};
const Segment SGI293173[] = {{&S65082[54], 5}, {NULL}};
const Segment SGI293174[] = {{&S65082[8], 3}, {NULL}};
const Segment SGI293175[] = {{&S65082[13], 15}, {NULL}};
const Segment SGI293176[] = {{&S65082[7], 1}, {NULL}};
const Segment SGI293177[] = {{&S65082[16], 12}, {NULL}};
const Segment SGI293178[] = {{&S62310[8], 3}, {NULL}};
const Segment SGI293179[] = {{&S62310[31], 4}, {NULL}};
const Segment SGI293180[] = {{&S62310[58], 6}, {NULL}};
const Segment SGI293181[] = {{&S62310[13], 22}, {NULL}};
const Segment SGI293182[] = {{&S65131[69], 5}, {NULL}};
const Segment SGI293183[] = {{&S65131[8], 3}, {NULL}};
const Segment SGI293184[] = {{&S65131[13], 9}, {NULL}};
const Segment SGI293185[] = {{&S65131[30], 13}, {NULL}};
const Segment SGI293186[] = {{&S65131[7], 1}, {NULL}};
const Segment SGI293187[] = {{&S65131[75], 13}, {NULL}};
const Segment SGI293188[] = {{&S65219[54], 5}, {NULL}};
const Segment SGI293189[] = {{&S65219[8], 3}, {NULL}};
const Segment SGI293190[] = {{&S65219[13], 15}, {NULL}};
const Segment SGI293191[] = {{&S65219[7], 1}, {NULL}};
const Segment SGI293192[] = {{&S65219[16], 12}, {NULL}};
const Segment SGI293193[] = {{&S65232[69], 5}, {NULL}};
const Segment SGI293194[] = {{&S65232[8], 3}, {NULL}};
const Segment SGI293195[] = {{&S65232[13], 9}, {NULL}};
const Segment SGI293196[] = {{&S65232[30], 13}, {NULL}};
const Segment SGI293197[] = {{&S65232[7], 1}, {NULL}};
const Segment SGI293198[] = {{&S65232[75], 13}, {NULL}};
const Segment SGI293199[] = {{&S65236[60], 5}, {NULL}};
const Segment SGI293200[] = {{&S65236[8], 3}, {NULL}};
const Segment SGI293201[] = {{&S65236[13], 14}, {NULL}};
const Segment SGI293202[] = {{&S65236[7], 1}, {NULL}};
const Segment SGI293203[] = {{&S65236[16], 11}, {NULL}};
const Segment SGI293204[] = {{&S65236[66], 14}, {NULL}};
const Segment SGI293205[] = {{&S65237[57], 5}, {NULL}};
const Segment SGI293206[] = {{&S65237[8], 3}, {NULL}};
const Segment SGI293207[] = {{&S65237[13], 3}, {NULL}};
const Segment SGI293208[] = {{&S65237[13], 3}, {&S65237[18], 7}, {NULL}};
const Segment SGI293209[] = {{&S65237[7], 1}, {NULL}};
const Segment SGI293210[] = {{&S65273[57], 5}, {NULL}};
const Segment SGI293211[] = {{&S65273[8], 3}, {NULL}};
const Segment SGI293212[] = {{&S65273[13], 3}, {NULL}};
const Segment SGI293213[] = {{&S65273[13], 3}, {&S65273[18], 7}, {NULL}};
const Segment SGI293214[] = {{&S65273[7], 1}, {NULL}};
const Segment SGI293215[] = {{&S65322[54], 5}, {NULL}};
const Segment SGI293216[] = {{&S65322[8], 3}, {NULL}};
const Segment SGI293217[] = {{&S65322[13], 15}, {NULL}};
const Segment SGI293218[] = {{&S65322[7], 1}, {NULL}};
const Segment SGI293219[] = {{&S65322[16], 12}, {NULL}};
const Segment SGI293220[] = {{&S48916[55], 16}, {NULL}};
const Segment SGI293221[] = {{&S48916[0], 91}, {NULL}};
const Segment SGI293222[] = {{&S48917[55], 9}, {NULL}};
const Segment SGI293223[] = {{&S48917[0], 88}, {NULL}};
const Segment SGI293224[] = {{&S48918[0], 110}, {NULL}};
const Segment SGI293225[] = {{&S48919[0], 110}, {NULL}};
const Segment SGI293226[] = {{&S48920[55], 3}, {NULL}};
const Segment SGI293227[] = {{&S48920[0], 96}, {NULL}};
const Segment SGI293228[] = {{&S48922[55], 9}, {NULL}};
const Segment SGI293229[] = {{&S48922[0], 84}, {NULL}};
const Segment SGI293230[] = {{&S48924[73], 8}, {NULL}};
const Segment SGI293231[] = {{&S48924[0], 81}, {NULL}};
const Segment SGI293232[] = {{&S48925[0], 110}, {NULL}};
const Segment SGI293233[] = {{&S48926[0], 110}, {NULL}};
const Segment SGI293234[] = {{&S48927[0], 110}, {NULL}};
const Segment SGI293235[] = {{&S48928[0], 110}, {NULL}};
const Segment SGI293236[] = {{&S48929[0], 110}, {NULL}};
const Segment SGI293237[] = {{&S48930[0], 110}, {NULL}};
const Segment SGI293238[] = {{&S48931[0], 110}, {NULL}};
const Segment SGI293239[] = {{&S48932[0], 104}, {NULL}};
const Segment SGI293240[] = {{&S48933[73], 7}, {NULL}};
const Segment SGI293241[] = {{&S48933[0], 104}, {NULL}};
const Segment SGI293242[] = {{&S48934[0], 110}, {NULL}};
const Segment SGI293243[] = {{&S48935[0], 110}, {NULL}};
const Segment SGI293244[] = {{&S48937[0], 104}, {NULL}};
const Segment SGI293245[] = {{&S48938[55], 3}, {NULL}};
const Segment SGI293246[] = {{&S48938[0], 78}, {NULL}};
const Segment SGI293247[] = {{&S48939[55], 13}, {NULL}};
const Segment SGI293248[] = {{&S48939[0], 88}, {NULL}};
const Segment SGI293249[] = {{&S65465[76], 4}, {NULL}};
const Segment SGI293250[] = {{&S65465[8], 3}, {NULL}};
const Segment SGI293251[] = {{&S65465[13], 7}, {NULL}};
const Segment SGI293252[] = {{&S65465[25], 14}, {NULL}};
const Segment SGI293253[] = {{&S65465[7], 1}, {NULL}};
const Segment SGI293254[] = {{&S65465[16], 4}, {&S65465[28], 11}, {NULL}};
const Segment SGI293255[] = {{&S65465[86], 13}, {NULL}};
const Segment SGI293256[] = {{&S65485[55], 5}, {NULL}};
const Segment SGI293257[] = {{&S65485[8], 3}, {NULL}};
const Segment SGI293258[] = {{&S65485[13], 14}, {NULL}};
const Segment SGI293259[] = {{&S65485[7], 1}, {NULL}};
const Segment SGI293260[] = {{&S65485[16], 11}, {NULL}};
const Segment SGI293261[] = {{&S65485[63], 13}, {NULL}};
const Segment SGI293262[] = {{&S65486[67], 5}, {NULL}};
const Segment SGI293263[] = {{&S65486[8], 3}, {NULL}};
const Segment SGI293264[] = {{&S65486[13], 14}, {NULL}};
const Segment SGI293265[] = {{&S65486[7], 1}, {NULL}};
const Segment SGI293266[] = {{&S65486[16], 11}, {&S65486[32], 2}, {NULL}};
const Segment SGI293267[] = {{&S65486[79], 13}, {NULL}};
const Segment SGI293268[] = {{&S65525[57], 5}, {NULL}};
const Segment SGI293269[] = {{&S65525[8], 3}, {NULL}};
const Segment SGI293270[] = {{&S65525[13], 3}, {NULL}};
const Segment SGI293271[] = {{&S65525[13], 3}, {&S65525[18], 7}, {NULL}};
const Segment SGI293272[] = {{&S65525[7], 1}, {NULL}};
const Segment SGI293273[] = {{&S65545[69], 5}, {NULL}};
const Segment SGI293274[] = {{&S65545[8], 3}, {NULL}};
const Segment SGI293275[] = {{&S65545[13], 9}, {NULL}};
const Segment SGI293276[] = {{&S65545[30], 13}, {NULL}};
const Segment SGI293277[] = {{&S65545[7], 1}, {NULL}};
const Segment SGI293278[] = {{&S65673[60], 5}, {NULL}};
const Segment SGI293279[] = {{&S65673[8], 3}, {NULL}};
const Segment SGI293280[] = {{&S65673[13], 14}, {NULL}};
const Segment SGI293281[] = {{&S65673[7], 1}, {NULL}};
const Segment SGI293282[] = {{&S65673[16], 11}, {NULL}};
const Segment SGI293283[] = {{&S65673[66], 13}, {NULL}};
const Segment SGI293284[] = {{&S65674[65], 5}, {NULL}};
const Segment SGI293285[] = {{&S65674[8], 3}, {NULL}};
const Segment SGI293286[] = {{&S65674[13], 14}, {NULL}};
const Segment SGI293287[] = {{&S65674[7], 1}, {NULL}};
const Segment SGI293288[] = {{&S65674[16], 11}, {&S65674[32], 2}, {NULL}};
const Segment SGI293289[] = {{&S65674[75], 13}, {NULL}};
const Segment SGI293290[] = {{&S65706[60], 5}, {NULL}};
const Segment SGI293291[] = {{&S65706[8], 3}, {NULL}};
const Segment SGI293292[] = {{&S65706[13], 14}, {NULL}};
const Segment SGI293293[] = {{&S65706[7], 1}, {NULL}};
const Segment SGI293294[] = {{&S65706[16], 11}, {NULL}};
const Segment SGI293295[] = {{&S65706[66], 14}, {NULL}};
const Segment SGI293296[] = {{&S65707[60], 5}, {NULL}};
const Segment SGI293297[] = {{&S65707[8], 3}, {NULL}};
const Segment SGI293298[] = {{&S65707[13], 14}, {NULL}};
const Segment SGI293299[] = {{&S65707[7], 1}, {NULL}};
const Segment SGI293300[] = {{&S65707[16], 11}, {NULL}};
const Segment SGI293301[] = {{&S65707[66], 14}, {NULL}};
const Segment SGI293302[] = {{&S65711[54], 5}, {NULL}};
const Segment SGI293303[] = {{&S65711[8], 3}, {NULL}};
const Segment SGI293304[] = {{&S65711[13], 15}, {NULL}};
const Segment SGI293305[] = {{&S65711[7], 1}, {NULL}};
const Segment SGI293306[] = {{&S65711[16], 12}, {NULL}};
const Segment SGI293307[] = {{&S65754[57], 4}, {NULL}};
const Segment SGI293308[] = {{&S65754[8], 3}, {NULL}};
const Segment SGI293309[] = {{&S65754[13], 10}, {NULL}};
const Segment SGI293310[] = {{&S65754[7], 1}, {NULL}};
const Segment SGI293311[] = {{&S65754[16], 7}, {NULL}};
const Segment SGI293312[] = {{&S65755[72], 4}, {NULL}};
const Segment SGI293313[] = {{&S65755[8], 3}, {NULL}};
const Segment SGI293314[] = {{&S65755[13], 7}, {NULL}};
const Segment SGI293315[] = {{&S65755[7], 1}, {NULL}};
const Segment SGI293316[] = {{&S65755[16], 4}, {&S65755[28], 8}, {NULL}};
const Segment SGI293317[] = {{&S65756[63], 4}, {NULL}};
const Segment SGI293318[] = {{&S65756[8], 3}, {NULL}};
const Segment SGI293319[] = {{&S65756[13], 7}, {NULL}};
const Segment SGI293320[] = {{&S65756[7], 1}, {NULL}};
const Segment SGI293321[] = {{&S65756[16], 4}, {&S65756[28], 2}, {NULL}};
const Segment SGI293322[] = {{&S65842[60], 5}, {NULL}};
const Segment SGI293323[] = {{&S65842[8], 3}, {NULL}};
const Segment SGI293324[] = {{&S65842[13], 14}, {NULL}};
const Segment SGI293325[] = {{&S65842[7], 1}, {NULL}};
const Segment SGI293326[] = {{&S65842[16], 11}, {NULL}};
const Segment SGI293327[] = {{&S65842[70], 14}, {NULL}};
const Segment SGI293328[] = {{&S65880[69], 5}, {NULL}};
const Segment SGI293329[] = {{&S65880[8], 3}, {NULL}};
const Segment SGI293330[] = {{&S65880[13], 9}, {NULL}};
const Segment SGI293331[] = {{&S65880[30], 13}, {NULL}};
const Segment SGI293332[] = {{&S65880[7], 1}, {NULL}};
const Segment SGI293333[] = {{&S65962[78], 4}, {NULL}};
const Segment SGI293334[] = {{&S65962[8], 3}, {NULL}};
const Segment SGI293335[] = {{&S65962[13], 9}, {NULL}};
const Segment SGI293336[] = {{&S65962[7], 1}, {NULL}};
const Segment SGI293337[] = {{&S65962[88], 9}, {NULL}};
const Segment SGI293338[] = {{&S65963[83], 4}, {NULL}};
const Segment SGI293339[] = {{&S65963[8], 3}, {NULL}};
const Segment SGI293340[] = {{&S65963[13], 9}, {NULL}};
const Segment SGI293341[] = {{&S65963[31], 15}, {NULL}};
const Segment SGI293342[] = {{&S65963[7], 1}, {NULL}};
const Segment SGI293343[] = {{&S65963[93], 9}, {NULL}};
const Segment SGI293344[] = {{&S65973[69], 5}, {NULL}};
const Segment SGI293345[] = {{&S65973[8], 3}, {NULL}};
const Segment SGI293346[] = {{&S65973[13], 9}, {NULL}};
const Segment SGI293347[] = {{&S65973[30], 13}, {NULL}};
const Segment SGI293348[] = {{&S65973[7], 1}, {NULL}};
const Segment SGI293349[] = {{&S65973[75], 13}, {NULL}};
const Segment SGI293350[] = {{&S65996[53], 5}, {NULL}};
const Segment SGI293351[] = {{&S65996[8], 3}, {NULL}};
const Segment SGI293352[] = {{&S65996[13], 14}, {NULL}};
const Segment SGI293353[] = {{&S65996[7], 1}, {NULL}};
const Segment SGI293354[] = {{&S65996[16], 11}, {NULL}};
const Segment SGI293355[] = {{&S65996[59], 9}, {NULL}};
const Segment SGI293356[] = {{&S65997[53], 5}, {NULL}};
const Segment SGI293357[] = {{&S65997[8], 3}, {NULL}};
const Segment SGI293358[] = {{&S65997[13], 14}, {NULL}};
const Segment SGI293359[] = {{&S65997[7], 1}, {NULL}};
const Segment SGI293360[] = {{&S65997[16], 11}, {NULL}};
const Segment SGI293361[] = {{&S65997[59], 9}, {NULL}};
const Segment SGI293362[] = {{&S65998[57], 5}, {NULL}};
const Segment SGI293363[] = {{&S65998[8], 3}, {NULL}};
const Segment SGI293364[] = {{&S65998[13], 3}, {NULL}};
const Segment SGI293365[] = {{&S65998[13], 3}, {&S65998[18], 7}, {NULL}};
const Segment SGI293366[] = {{&S65998[7], 1}, {NULL}};
const Segment SGI293367[] = {{&S66033[75], 4}, {NULL}};
const Segment SGI293368[] = {{&S66033[8], 3}, {NULL}};
const Segment SGI293369[] = {{&S66033[13], 7}, {NULL}};
const Segment SGI293370[] = {{&S66033[25], 14}, {NULL}};
const Segment SGI293371[] = {{&S66033[7], 1}, {NULL}};
const Segment SGI293372[] = {{&S66033[16], 4}, {&S66033[28], 11}, {NULL}};
const Segment SGI293373[] = {{&S66033[82], 13}, {NULL}};
const Segment SGI293374[] = {{&S66034[76], 4}, {NULL}};
const Segment SGI293375[] = {{&S66034[8], 3}, {NULL}};
const Segment SGI293376[] = {{&S66034[13], 7}, {NULL}};
const Segment SGI293377[] = {{&S66034[25], 14}, {NULL}};
const Segment SGI293378[] = {{&S66034[7], 1}, {NULL}};
const Segment SGI293379[] = {{&S66034[16], 4}, {&S66034[28], 11}, {NULL}};
const Segment SGI293380[] = {{&S66034[84], 16}, {NULL}};
const Segment SGI293381[] = {{&S66066[76], 4}, {NULL}};
const Segment SGI293382[] = {{&S66066[8], 3}, {NULL}};
const Segment SGI293383[] = {{&S66066[13], 7}, {NULL}};
const Segment SGI293384[] = {{&S66066[25], 14}, {NULL}};
const Segment SGI293385[] = {{&S66066[7], 1}, {NULL}};
const Segment SGI293386[] = {{&S66066[16], 4}, {&S66066[28], 11}, {NULL}};
const Segment SGI293387[] = {{&S66067[121], 4}, {NULL}};
const Segment SGI293388[] = {{&S66067[8], 3}, {NULL}};
const Segment SGI293389[] = {{&S66067[13], 7}, {NULL}};
const Segment SGI293390[] = {{&S66067[7], 1}, {NULL}};
const Segment SGI293391[] = {{&S66067[16], 4}, {&S66067[16], 4}, {NULL}};
const Segment SGI293392[] = {{&S66068[41], 4}, {NULL}};
const Segment SGI293393[] = {{&S66068[8], 3}, {NULL}};
const Segment SGI293394[] = {{&S66068[13], 7}, {NULL}};
const Segment SGI293395[] = {{&S66068[7], 1}, {NULL}};
const Segment SGI293396[] = {{&S66068[16], 4}, {&S66068[25], 20}, {NULL}};
const Segment SGI293397[] = {{&S66183[97], 4}, {NULL}};
const Segment SGI293398[] = {{&S66183[8], 3}, {NULL}};
const Segment SGI293399[] = {{&S66183[13], 7}, {NULL}};
const Segment SGI293400[] = {{&S66183[25], 14}, {NULL}};
const Segment SGI293401[] = {{&S66183[7], 1}, {NULL}};
const Segment SGI293402[] = {{&S66183[16], 4}, {&S66183[28], 11}, {NULL}};
const Segment SGI293403[] = {{&S66183[71], 13}, {NULL}};
const Segment SGI293404[] = {{&S66185[8], 3}, {NULL}};
const Segment SGI293405[] = {{&S66185[13], 7}, {NULL}};
const Segment SGI293406[] = {{&S66185[25], 14}, {NULL}};
const Segment SGI293407[] = {{&S66185[78], 11}, {NULL}};
const Segment SGI293408[] = {{&S66185[16], 4}, {&S66185[28], 11}, {NULL}};
const Segment SGI293409[] = {{&S66185[79], 13}, {NULL}};
const Segment SGI293410[] = {{&S66209[53], 5}, {NULL}};
const Segment SGI293411[] = {{&S66209[8], 3}, {NULL}};
const Segment SGI293412[] = {{&S66209[13], 14}, {NULL}};
const Segment SGI293413[] = {{&S66209[7], 1}, {NULL}};
const Segment SGI293414[] = {{&S66209[16], 11}, {NULL}};
const Segment SGI293415[] = {{&S66209[59], 14}, {NULL}};
const Segment SGI293416[] = {{&S66306[57], 5}, {NULL}};
const Segment SGI293417[] = {{&S66306[8], 3}, {NULL}};
const Segment SGI293418[] = {{&S66306[13], 3}, {NULL}};
const Segment SGI293419[] = {{&S66306[13], 3}, {&S66306[18], 7}, {NULL}};
const Segment SGI293420[] = {{&S66306[7], 1}, {NULL}};
const Segment SGI293421[] = {{&S66335[69], 5}, {NULL}};
const Segment SGI293422[] = {{&S66335[8], 3}, {NULL}};
const Segment SGI293423[] = {{&S66335[13], 9}, {NULL}};
const Segment SGI293424[] = {{&S66335[30], 13}, {NULL}};
const Segment SGI293425[] = {{&S66335[7], 1}, {NULL}};
const Segment SGI293426[] = {{&S66335[75], 13}, {NULL}};
const Segment SGI293427[] = {{&S66355[66], 4}, {NULL}};
const Segment SGI293428[] = {{&S66355[8], 3}, {NULL}};
const Segment SGI293429[] = {{&S66355[13], 3}, {NULL}};
const Segment SGI293430[] = {{&S66355[7], 1}, {NULL}};
const Segment SGI293431[] = {{&S66355[73], 11}, {NULL}};
const Segment SGI293432[] = {{&S63811[8], 3}, {NULL}};
const Segment SGI293433[] = {{&S63811[13], 15}, {NULL}};
const Segment SGI293434[] = {{&S63811[24], 2}, {NULL}};
const Segment SGI293435[] = {{&S63811[24], 4}, {NULL}};
const Segment SGI293436[] = {{&S66499[76], 4}, {NULL}};
const Segment SGI293437[] = {{&S66499[8], 3}, {NULL}};
const Segment SGI293438[] = {{&S66499[13], 7}, {NULL}};
const Segment SGI293439[] = {{&S66499[25], 14}, {NULL}};
const Segment SGI293440[] = {{&S66499[7], 1}, {NULL}};
const Segment SGI293441[] = {{&S66499[16], 4}, {&S66499[28], 11}, {NULL}};
const Segment SGI293442[] = {{&S66576[57], 4}, {NULL}};
const Segment SGI293443[] = {{&S66576[8], 3}, {NULL}};
const Segment SGI293444[] = {{&S66576[13], 3}, {NULL}};
const Segment SGI293445[] = {{&S66576[13], 3}, {&S66576[18], 7}, {NULL}};
const Segment SGI293446[] = {{&S66576[7], 1}, {NULL}};
const Segment SGI293447[] = {{&S66743[64], 5}, {NULL}};
const Segment SGI293448[] = {{&S66743[8], 3}, {NULL}};
const Segment SGI293449[] = {{&S66743[13], 3}, {NULL}};
const Segment SGI293450[] = {{&S66743[13], 3}, {&S66743[18], 6}, {NULL}};
const Segment SGI293451[] = {{&S66743[7], 1}, {NULL}};
const Segment SGI293452[] = {{&S66946[54], 5}, {NULL}};
const Segment SGI293453[] = {{&S66946[8], 3}, {NULL}};
const Segment SGI293454[] = {{&S66946[13], 15}, {NULL}};
const Segment SGI293455[] = {{&S66946[7], 1}, {NULL}};
const Segment SGI293456[] = {{&S66946[16], 12}, {NULL}};
const Segment SGI293457[] = {{&S67096[68], 4}, {NULL}};
const Segment SGI293458[] = {{&S67096[8], 3}, {NULL}};
const Segment SGI293459[] = {{&S67096[13], 3}, {NULL}};
const Segment SGI293460[] = {{&S67096[13], 3}, {&S67096[76], 6}, {NULL}};
const Segment SGI293461[] = {{&S67096[7], 1}, {NULL}};
const Segment SGI293462[] = {{&S67114[43], 5}, {NULL}};
const Segment SGI293463[] = {{&S67114[8], 3}, {NULL}};
const Segment SGI293464[] = {{&S67114[13], 14}, {NULL}};
const Segment SGI293465[] = {{&S67114[7], 1}, {NULL}};
const Segment SGI293466[] = {{&S67114[16], 11}, {NULL}};
const Segment SGI293467[] = {{&S67114[62], 14}, {NULL}};
const Segment SGI293468[] = {{&S50756[0], 47}, {NULL}};
const Segment SGI293469[] = {{&S50757[0], 47}, {NULL}};
const Segment SGI293470[] = {{&S50758[0], 47}, {NULL}};
const Segment SGI293471[] = {{&S50759[0], 50}, {NULL}};
const Segment SGI293472[] = {{&S50760[0], 47}, {NULL}};
const Segment SGI293473[] = {{&S50761[0], 50}, {NULL}};
const Segment SGI293474[] = {{&S50762[0], 47}, {NULL}};
const Segment SGI293475[] = {{&S50763[0], 47}, {NULL}};
const Segment SGI293476[] = {{&S50764[0], 48}, {NULL}};
const Segment SGI293477[] = {{&S50765[0], 47}, {NULL}};
const Segment SGI293478[] = {{&S50766[0], 47}, {NULL}};
const Segment SGI293479[] = {{&S50767[0], 47}, {NULL}};
const Segment SGI293480[] = {{&S50768[0], 47}, {NULL}};
const Segment SGI293481[] = {{&S50769[0], 47}, {NULL}};
const Segment SGI293482[] = {{&S67285[53], 5}, {NULL}};
const Segment SGI293483[] = {{&S67285[8], 3}, {NULL}};
const Segment SGI293484[] = {{&S67285[13], 14}, {NULL}};
const Segment SGI293485[] = {{&S67285[7], 1}, {NULL}};
const Segment SGI293486[] = {{&S67285[16], 11}, {NULL}};
const Segment SGI293487[] = {{&S67285[59], 13}, {NULL}};
const Segment SGI293488[] = {{&S67286[65], 5}, {NULL}};
const Segment SGI293489[] = {{&S67286[8], 3}, {NULL}};
const Segment SGI293490[] = {{&S67286[13], 14}, {NULL}};
const Segment SGI293491[] = {{&S67286[7], 1}, {NULL}};
const Segment SGI293492[] = {{&S67286[16], 11}, {&S67286[32], 2}, {NULL}};
const Segment SGI293493[] = {{&S67286[75], 13}, {NULL}};
const Segment SGI293494[] = {{&S67323[53], 5}, {NULL}};
const Segment SGI293495[] = {{&S67323[8], 3}, {NULL}};
const Segment SGI293496[] = {{&S67323[13], 14}, {NULL}};
const Segment SGI293497[] = {{&S67323[7], 1}, {NULL}};
const Segment SGI293498[] = {{&S67323[16], 11}, {NULL}};
const Segment SGI293499[] = {{&S67323[59], 13}, {NULL}};
const Segment SGI293500[] = {{&S67363[69], 5}, {NULL}};
const Segment SGI293501[] = {{&S67363[8], 3}, {NULL}};
const Segment SGI293502[] = {{&S67363[13], 9}, {NULL}};
const Segment SGI293503[] = {{&S67363[30], 13}, {NULL}};
const Segment SGI293504[] = {{&S67363[7], 1}, {NULL}};
const Segment SGI293505[] = {{&S67363[75], 13}, {NULL}};
const Segment SGI293506[] = {{&S67377[76], 4}, {NULL}};
const Segment SGI293507[] = {{&S67377[8], 3}, {NULL}};
const Segment SGI293508[] = {{&S67377[13], 7}, {NULL}};
const Segment SGI293509[] = {{&S67377[25], 14}, {NULL}};
const Segment SGI293510[] = {{&S67377[7], 1}, {NULL}};
const Segment SGI293511[] = {{&S67377[16], 4}, {&S67377[28], 11}, {NULL}};
const Segment SGI293512[] = {{&S67377[84], 9}, {NULL}};
const Segment SGI293513[] = {{&S67475[56], 5}, {NULL}};
const Segment SGI293514[] = {{&S67475[8], 3}, {NULL}};
const Segment SGI293515[] = {{&S67475[13], 3}, {NULL}};
const Segment SGI293516[] = {{&S67475[7], 1}, {NULL}};
const Segment SGI293517[] = {{&S67532[69], 4}, {NULL}};
const Segment SGI293518[] = {{&S67532[8], 3}, {NULL}};
const Segment SGI293519[] = {{&S67532[13], 9}, {NULL}};
const Segment SGI293520[] = {{&S67532[28], 13}, {NULL}};
const Segment SGI293521[] = {{&S67532[7], 1}, {NULL}};
const Segment SGI293522[] = {{&S67574[54], 5}, {NULL}};
const Segment SGI293523[] = {{&S67574[8], 3}, {NULL}};
const Segment SGI293524[] = {{&S67574[13], 15}, {NULL}};
const Segment SGI293525[] = {{&S67574[7], 1}, {NULL}};
const Segment SGI293526[] = {{&S67574[16], 12}, {NULL}};
const Segment SGI293527[] = {{&S67667[69], 5}, {NULL}};
const Segment SGI293528[] = {{&S67667[8], 3}, {NULL}};
const Segment SGI293529[] = {{&S67667[13], 9}, {NULL}};
const Segment SGI293530[] = {{&S67667[30], 13}, {NULL}};
const Segment SGI293531[] = {{&S67667[7], 1}, {NULL}};
const Segment SGI293532[] = {{&S67667[75], 13}, {NULL}};
const Segment SGI293533[] = {{&S67979[69], 5}, {NULL}};
const Segment SGI293534[] = {{&S67979[8], 3}, {NULL}};
const Segment SGI293535[] = {{&S67979[13], 9}, {NULL}};
const Segment SGI293536[] = {{&S67979[30], 13}, {NULL}};
const Segment SGI293537[] = {{&S67979[7], 1}, {NULL}};
const Segment SGI293538[] = {{&S67979[75], 13}, {NULL}};
const Segment SGI293539[] = {{&S67991[63], 4}, {NULL}};
const Segment SGI293540[] = {{&S67991[8], 3}, {NULL}};
const Segment SGI293541[] = {{&S67991[13], 3}, {NULL}};
const Segment SGI293542[] = {{&S67991[13], 3}, {&S67991[18], 6}, {NULL}};
const Segment SGI293543[] = {{&S67991[7], 1}, {NULL}};
const Segment SGI293544[] = {{&S68023[56], 5}, {NULL}};
const Segment SGI293545[] = {{&S68023[8], 3}, {NULL}};
const Segment SGI293546[] = {{&S68023[13], 3}, {NULL}};
const Segment SGI293547[] = {{&S68023[7], 1}, {NULL}};
const Segment SGI293548[] = {{&S68023[64], 13}, {NULL}};
const Segment SGI293549[] = {{&S68168[53], 5}, {NULL}};
const Segment SGI293550[] = {{&S68168[8], 3}, {NULL}};
const Segment SGI293551[] = {{&S68168[13], 14}, {NULL}};
const Segment SGI293552[] = {{&S68168[7], 1}, {NULL}};
const Segment SGI293553[] = {{&S68168[16], 11}, {NULL}};
const Segment SGI293554[] = {{&S68228[57], 5}, {NULL}};
const Segment SGI293555[] = {{&S68228[8], 3}, {NULL}};
const Segment SGI293556[] = {{&S68228[13], 3}, {NULL}};
const Segment SGI293557[] = {{&S68228[13], 3}, {&S68228[18], 7}, {NULL}};
const Segment SGI293558[] = {{&S68228[7], 1}, {NULL}};
const Segment SGI293559[] = {{&S68231[69], 5}, {NULL}};
const Segment SGI293560[] = {{&S68231[8], 3}, {NULL}};
const Segment SGI293561[] = {{&S68231[13], 9}, {NULL}};
const Segment SGI293562[] = {{&S68231[30], 13}, {NULL}};
const Segment SGI293563[] = {{&S68231[7], 1}, {NULL}};
const Segment SGI293564[] = {{&S68231[75], 13}, {NULL}};
const Segment SGI293565[] = {{&S68278[69], 5}, {NULL}};
const Segment SGI293566[] = {{&S68278[8], 3}, {NULL}};
const Segment SGI293567[] = {{&S68278[13], 9}, {NULL}};
const Segment SGI293568[] = {{&S68278[30], 13}, {NULL}};
const Segment SGI293569[] = {{&S68278[7], 1}, {NULL}};
const Segment SGI293570[] = {{&S68278[75], 13}, {NULL}};
const Segment SGI293571[] = {{&S65431[8], 3}, {NULL}};
const Segment SGI293572[] = {{&S65431[39], 5}, {NULL}};
const Segment SGI293573[] = {{&S65431[68], 6}, {NULL}};
const Segment SGI293574[] = {{&S65431[13], 31}, {NULL}};
const Segment SGI293575[] = {{&S65431[62], 5}, {NULL}};
const Segment SGI293576[] = {{&S68415[73], 5}, {NULL}};
const Segment SGI293577[] = {{&S68415[8], 3}, {NULL}};
const Segment SGI293578[] = {{&S68415[13], 9}, {NULL}};
const Segment SGI293579[] = {{&S68415[31], 15}, {NULL}};
const Segment SGI293580[] = {{&S68415[7], 1}, {NULL}};
const Segment SGI293581[] = {{&S68416[75], 4}, {NULL}};
const Segment SGI293582[] = {{&S68416[8], 3}, {NULL}};
const Segment SGI293583[] = {{&S68416[13], 9}, {NULL}};
const Segment SGI293584[] = {{&S68416[31], 15}, {NULL}};
const Segment SGI293585[] = {{&S68416[7], 1}, {NULL}};
const Segment SGI293586[] = {{&S68455[75], 4}, {NULL}};
const Segment SGI293587[] = {{&S68455[8], 3}, {NULL}};
const Segment SGI293588[] = {{&S68455[13], 7}, {NULL}};
const Segment SGI293589[] = {{&S68455[25], 14}, {NULL}};
const Segment SGI293590[] = {{&S68455[7], 1}, {NULL}};
const Segment SGI293591[] = {{&S68455[16], 4}, {&S68455[28], 11}, {NULL}};
const Segment SGI293592[] = {{&S68455[82], 9}, {NULL}};
const Segment SGI293593[] = {{&S68486[109], 4}, {NULL}};
const Segment SGI293594[] = {{&S68486[8], 3}, {NULL}};
const Segment SGI293595[] = {{&S68486[13], 9}, {NULL}};
const Segment SGI293596[] = {{&S68486[31], 13}, {NULL}};
const Segment SGI293597[] = {{&S68486[7], 1}, {NULL}};
const Segment SGI293598[] = {{&S68486[71], 10}, {NULL}};
const Segment SGI293599[] = {{&S68615[74], 4}, {NULL}};
const Segment SGI293600[] = {{&S68615[8], 3}, {NULL}};
const Segment SGI293601[] = {{&S68615[13], 9}, {NULL}};
const Segment SGI293602[] = {{&S68615[30], 15}, {NULL}};
const Segment SGI293603[] = {{&S68615[7], 1}, {NULL}};
const Segment SGI293604[] = {{&S68615[79], 11}, {NULL}};
const Segment SGI293605[] = {{&S68625[55], 5}, {NULL}};
const Segment SGI293606[] = {{&S68625[8], 3}, {NULL}};
const Segment SGI293607[] = {{&S68625[13], 3}, {NULL}};
const Segment SGI293608[] = {{&S68625[13], 3}, {&S68625[18], 5}, {NULL}};
const Segment SGI293609[] = {{&S68625[7], 1}, {NULL}};
const Segment SGI293610[] = {{&S68650[53], 5}, {NULL}};
const Segment SGI293611[] = {{&S68650[8], 3}, {NULL}};
const Segment SGI293612[] = {{&S68650[13], 14}, {NULL}};
const Segment SGI293613[] = {{&S68650[7], 1}, {NULL}};
const Segment SGI293614[] = {{&S68650[16], 11}, {NULL}};
const Segment SGI293615[] = {{&S68650[59], 13}, {NULL}};
const Segment SGI293616[] = {{&S68694[55], 5}, {NULL}};
const Segment SGI293617[] = {{&S68694[8], 3}, {NULL}};
const Segment SGI293618[] = {{&S68694[13], 3}, {NULL}};
const Segment SGI293619[] = {{&S68694[13], 3}, {&S68694[18], 5}, {NULL}};
const Segment SGI293620[] = {{&S68694[7], 1}, {NULL}};
const Segment SGI293621[] = {{&S68714[7], 4}, {NULL}};
const Segment SGI293622[] = {{&S68714[8], 3}, {NULL}};
const Segment SGI293623[] = {{&S68714[13], 14}, {NULL}};
const Segment SGI293624[] = {{&S68714[7], 1}, {NULL}};
const Segment SGI293625[] = {{&S68714[16], 11}, {NULL}};
const Segment SGI293626[] = {{&S68715[7], 4}, {NULL}};
const Segment SGI293627[] = {{&S68715[8], 3}, {NULL}};
const Segment SGI293628[] = {{&S68715[13], 14}, {NULL}};
const Segment SGI293629[] = {{&S68715[7], 1}, {NULL}};
const Segment SGI293630[] = {{&S68715[16], 64}, {NULL}};
const Segment SGI293631[] = {{&S68733[94], 4}, {NULL}};
const Segment SGI293632[] = {{&S68733[8], 3}, {NULL}};
const Segment SGI293633[] = {{&S68733[13], 7}, {NULL}};
const Segment SGI293634[] = {{&S68733[25], 14}, {NULL}};
const Segment SGI293635[] = {{&S68733[7], 1}, {NULL}};
const Segment SGI293636[] = {{&S68733[16], 4}, {&S68733[28], 11}, {NULL}};
const Segment SGI293637[] = {{&S68733[101], 8}, {NULL}};
const Segment SGI293638[] = {{&S68744[62], 5}, {NULL}};
const Segment SGI293639[] = {{&S68744[8], 3}, {NULL}};
const Segment SGI293640[] = {{&S68744[13], 3}, {NULL}};
const Segment SGI293641[] = {{&S68744[13], 3}, {&S68744[18], 6}, {NULL}};
const Segment SGI293642[] = {{&S68744[7], 1}, {NULL}};
const Segment SGI293643[] = {{&S68788[66], 5}, {NULL}};
const Segment SGI293644[] = {{&S68788[8], 3}, {NULL}};
const Segment SGI293645[] = {{&S68788[13], 7}, {NULL}};
const Segment SGI293646[] = {{&S68788[25], 14}, {NULL}};
const Segment SGI293647[] = {{&S68788[7], 1}, {NULL}};
const Segment SGI293648[] = {{&S68788[16], 4}, {&S68788[28], 11}, {NULL}};
const Segment SGI293649[] = {{&S68789[72], 4}, {NULL}};
const Segment SGI293650[] = {{&S68789[8], 3}, {NULL}};
const Segment SGI293651[] = {{&S68789[13], 7}, {NULL}};
const Segment SGI293652[] = {{&S68789[25], 14}, {NULL}};
const Segment SGI293653[] = {{&S68789[7], 1}, {NULL}};
const Segment SGI293654[] = {{&S68789[16], 4}, {&S68789[28], 11}, {NULL}};
const Segment SGI293655[] = {{&S68790[71], 4}, {NULL}};
const Segment SGI293656[] = {{&S68790[8], 3}, {NULL}};
const Segment SGI293657[] = {{&S68790[13], 7}, {NULL}};
const Segment SGI293658[] = {{&S68790[25], 14}, {NULL}};
const Segment SGI293659[] = {{&S68790[7], 1}, {NULL}};
const Segment SGI293660[] = {{&S68790[16], 4}, {&S68790[28], 11}, {NULL}};
const Segment SGI293661[] = {{&S68972[8], 3}, {NULL}};
const Segment SGI293662[] = {{&S68972[13], 3}, {NULL}};
const Segment SGI293663[] = {{&S68972[13], 3}, {&S68972[61], 6}, {NULL}};
const Segment SGI293664[] = {{&S68972[61], 12}, {NULL}};
const Segment SGI293665[] = {{&S68972[7], 1}, {NULL}};
const Segment SGI293666[] = {{&S69067[59], 4}, {NULL}};
const Segment SGI293667[] = {{&S69067[8], 3}, {NULL}};
const Segment SGI293668[] = {{&S69067[13], 14}, {NULL}};
const Segment SGI293669[] = {{&S69067[7], 1}, {NULL}};
const Segment SGI293670[] = {{&S69067[16], 11}, {NULL}};
const Segment SGI293671[] = {{&S69068[75], 4}, {NULL}};
const Segment SGI293672[] = {{&S69068[8], 3}, {NULL}};
const Segment SGI293673[] = {{&S69068[13], 7}, {NULL}};
const Segment SGI293674[] = {{&S69068[25], 14}, {NULL}};
const Segment SGI293675[] = {{&S69068[7], 1}, {NULL}};
const Segment SGI293676[] = {{&S69068[16], 4}, {&S69068[28], 11}, {NULL}};
const Segment SGI293677[] = {{&S69069[66], 4}, {NULL}};
const Segment SGI293678[] = {{&S69069[8], 3}, {NULL}};
const Segment SGI293679[] = {{&S69069[13], 9}, {NULL}};
const Segment SGI293680[] = {{&S69069[28], 13}, {NULL}};
const Segment SGI293681[] = {{&S69069[7], 1}, {NULL}};
const Segment SGI293682[] = {{&S69134[109], 4}, {NULL}};
const Segment SGI293683[] = {{&S69134[8], 3}, {NULL}};
const Segment SGI293684[] = {{&S69134[13], 9}, {NULL}};
const Segment SGI293685[] = {{&S69134[31], 13}, {NULL}};
const Segment SGI293686[] = {{&S69134[7], 1}, {NULL}};
const Segment SGI293687[] = {{&S69134[71], 10}, {NULL}};
const Segment SGI293688[] = {{&S52584[0], 124}, {NULL}};
const Segment SGI293689[] = {{&S52585[82], 16}, {NULL}};
const Segment SGI293690[] = {{&S52585[0], 98}, {NULL}};
const Segment SGI293691[] = {{&S52586[0], 108}, {NULL}};
const Segment SGI293692[] = {{&S52587[0], 124}, {NULL}};
const Segment SGI293693[] = {{&S52588[0], 124}, {NULL}};
const Segment SGI293694[] = {{&S52589[0], 124}, {NULL}};
const Segment SGI293695[] = {{&S52591[0], 98}, {NULL}};
const Segment SGI293696[] = {{&S52592[0], 118}, {NULL}};
const Segment SGI293697[] = {{&S52594[0], 94}, {NULL}};
const Segment SGI293698[] = {{&S52595[96], 23}, {NULL}};
const Segment SGI293699[] = {{&S52595[0], 140}, {NULL}};
const Segment SGI293700[] = {{&S52596[0], 98}, {NULL}};
const Segment SGI293701[] = {{&S52597[0], 98}, {NULL}};
const Segment SGI293702[] = {{&S52598[0], 118}, {NULL}};
const Segment SGI293703[] = {{&S52599[0], 124}, {NULL}};
const Segment SGI293704[] = {{&S52600[96], 23}, {NULL}};
const Segment SGI293705[] = {{&S52600[0], 140}, {NULL}};
const Segment SGI293706[] = {{&S52602[0], 124}, {NULL}};
const Segment SGI293707[] = {{&S52694[0], 89}, {NULL}};
const Segment SGI293708[] = {{&S52738[0], 117}, {NULL}};
const Segment SGI293709[] = {{&S52740[0], 106}, {NULL}};
const Segment SGI293710[] = {{&S69259[67], 5}, {NULL}};
const Segment SGI293711[] = {{&S69259[8], 3}, {NULL}};
const Segment SGI293712[] = {{&S69259[13], 9}, {NULL}};
const Segment SGI293713[] = {{&S69259[28], 13}, {NULL}};
const Segment SGI293714[] = {{&S69259[7], 1}, {NULL}};
const Segment SGI293715[] = {{&S52751[0], 112}, {NULL}};
const Segment SGI293716[] = {{&S69296[54], 5}, {NULL}};
const Segment SGI293717[] = {{&S69296[8], 3}, {NULL}};
const Segment SGI293718[] = {{&S69296[13], 15}, {NULL}};
const Segment SGI293719[] = {{&S69296[7], 1}, {NULL}};
const Segment SGI293720[] = {{&S69296[16], 12}, {NULL}};
const Segment SGI293721[] = {{&S69327[58], 5}, {NULL}};
const Segment SGI293722[] = {{&S69327[8], 3}, {NULL}};
const Segment SGI293723[] = {{&S69327[13], 3}, {NULL}};
const Segment SGI293724[] = {{&S69327[13], 3}, {&S69327[18], 7}, {NULL}};
const Segment SGI293725[] = {{&S69327[7], 1}, {NULL}};
const Segment SGI293726[] = {{&S69470[69], 5}, {NULL}};
const Segment SGI293727[] = {{&S69470[8], 3}, {NULL}};
const Segment SGI293728[] = {{&S69470[13], 9}, {NULL}};
const Segment SGI293729[] = {{&S69470[30], 13}, {NULL}};
const Segment SGI293730[] = {{&S69470[7], 1}, {NULL}};
const Segment SGI293731[] = {{&S69470[75], 13}, {NULL}};
const Segment SGI293732[] = {{&S69501[76], 4}, {NULL}};
const Segment SGI293733[] = {{&S69501[8], 3}, {NULL}};
const Segment SGI293734[] = {{&S69501[13], 7}, {NULL}};
const Segment SGI293735[] = {{&S69501[25], 14}, {NULL}};
const Segment SGI293736[] = {{&S69501[7], 1}, {NULL}};
const Segment SGI293737[] = {{&S69501[16], 4}, {&S69501[28], 11}, {NULL}};
const Segment SGI293738[] = {{&S69501[86], 9}, {NULL}};
const Segment SGI293739[] = {{&S69580[111], 4}, {NULL}};
const Segment SGI293740[] = {{&S69580[8], 3}, {NULL}};
const Segment SGI293741[] = {{&S69580[13], 9}, {NULL}};
const Segment SGI293742[] = {{&S69580[33], 13}, {NULL}};
const Segment SGI293743[] = {{&S69580[7], 1}, {NULL}};
const Segment SGI293744[] = {{&S69580[73], 10}, {NULL}};
const Segment SGI293745[] = {{&S69635[70], 4}, {NULL}};
const Segment SGI293746[] = {{&S69635[8], 3}, {NULL}};
const Segment SGI293747[] = {{&S69635[13], 9}, {NULL}};
const Segment SGI293748[] = {{&S69635[30], 13}, {NULL}};
const Segment SGI293749[] = {{&S69635[7], 1}, {NULL}};
const Segment SGI293750[] = {{&S69636[70], 4}, {&S69636[70], 4}, {NULL}};
const Segment SGI293751[] = {{&S69636[8], 3}, {NULL}};
const Segment SGI293752[] = {{&S69636[13], 9}, {NULL}};
const Segment SGI293753[] = {{&S69636[30], 13}, {NULL}};
const Segment SGI293754[] = {{&S69636[7], 1}, {&S69636[7], 1}, {NULL}};
const Segment SGI293755[] = {{&S69645[53], 5}, {NULL}};
const Segment SGI293756[] = {{&S69645[8], 3}, {NULL}};
const Segment SGI293757[] = {{&S69645[13], 14}, {NULL}};
const Segment SGI293758[] = {{&S69645[7], 1}, {NULL}};
const Segment SGI293759[] = {{&S69645[16], 11}, {NULL}};
const Segment SGI293760[] = {{&S69645[59], 14}, {NULL}};
const Segment SGI293761[] = {{&S69646[53], 5}, {NULL}};
const Segment SGI293762[] = {{&S69646[8], 3}, {NULL}};
const Segment SGI293763[] = {{&S69646[13], 14}, {NULL}};
const Segment SGI293764[] = {{&S69646[7], 1}, {NULL}};
const Segment SGI293765[] = {{&S69646[16], 11}, {NULL}};
const Segment SGI293766[] = {{&S69646[59], 14}, {NULL}};
const Segment SGI293767[] = {{&S69688[53], 5}, {NULL}};
const Segment SGI293768[] = {{&S69688[8], 3}, {NULL}};
const Segment SGI293769[] = {{&S69688[13], 14}, {NULL}};
const Segment SGI293770[] = {{&S69688[7], 1}, {NULL}};
const Segment SGI293771[] = {{&S69688[16], 11}, {NULL}};
const Segment SGI293772[] = {{&S69688[59], 14}, {NULL}};
const Segment SGI293773[] = {{&S69764[64], 4}, {NULL}};
const Segment SGI293774[] = {{&S69764[8], 3}, {NULL}};
const Segment SGI293775[] = {{&S69764[13], 14}, {NULL}};
const Segment SGI293776[] = {{&S69764[7], 1}, {NULL}};
const Segment SGI293777[] = {{&S69764[16], 11}, {&S69764[32], 2}, {NULL}};
const Segment SGI293778[] = {{&S69764[73], 12}, {NULL}};
const Segment SGI293779[] = {{&S69765[52], 4}, {NULL}};
const Segment SGI293780[] = {{&S69765[8], 3}, {NULL}};
const Segment SGI293781[] = {{&S69765[13], 14}, {NULL}};
const Segment SGI293782[] = {{&S69765[7], 1}, {NULL}};
const Segment SGI293783[] = {{&S69765[16], 11}, {NULL}};
const Segment SGI293784[] = {{&S69765[57], 12}, {NULL}};
const Segment SGI293785[] = {{&S69798[56], 5}, {NULL}};
const Segment SGI293786[] = {{&S69798[8], 3}, {NULL}};
const Segment SGI293787[] = {{&S69798[13], 15}, {NULL}};
const Segment SGI293788[] = {{&S69798[7], 1}, {NULL}};
const Segment SGI293789[] = {{&S69798[16], 12}, {NULL}};
const Segment SGI293790[] = {{&S69827[62], 4}, {NULL}};
const Segment SGI293791[] = {{&S69827[8], 3}, {NULL}};
const Segment SGI293792[] = {{&S69827[13], 7}, {NULL}};
const Segment SGI293793[] = {{&S69827[7], 1}, {NULL}};
const Segment SGI293794[] = {{&S69827[16], 4}, {&S69827[28], 2}, {NULL}};
const Segment SGI293795[] = {{&S69828[62], 4}, {NULL}};
const Segment SGI293796[] = {{&S69828[8], 3}, {NULL}};
const Segment SGI293797[] = {{&S69828[13], 7}, {NULL}};
const Segment SGI293798[] = {{&S69828[7], 1}, {NULL}};
const Segment SGI293799[] = {{&S69828[16], 4}, {&S69828[28], 2}, {NULL}};
const Segment SGI293800[] = {{&S69829[61], 4}, {NULL}};
const Segment SGI293801[] = {{&S69829[8], 3}, {NULL}};
const Segment SGI293802[] = {{&S69829[13], 7}, {NULL}};
const Segment SGI293803[] = {{&S69829[7], 1}, {NULL}};
const Segment SGI293804[] = {{&S69829[16], 4}, {&S69829[28], 8}, {NULL}};
const Segment SGI293805[] = {{&S69830[63], 5}, {NULL}};
const Segment SGI293806[] = {{&S69830[8], 3}, {NULL}};
const Segment SGI293807[] = {{&S69830[13], 7}, {NULL}};
const Segment SGI293808[] = {{&S69830[7], 1}, {NULL}};
const Segment SGI293809[] = {{&S69830[16], 4}, {&S69830[28], 8}, {NULL}};
const Segment SGI293810[] = {{&S69831[68], 4}, {NULL}};
const Segment SGI293811[] = {{&S69831[8], 3}, {NULL}};
const Segment SGI293812[] = {{&S69831[13], 7}, {NULL}};
const Segment SGI293813[] = {{&S69831[7], 1}, {NULL}};
const Segment SGI293814[] = {{&S69831[16], 4}, {&S69831[28], 8}, {NULL}};
const Segment SGI293815[] = {{&S69832[57], 5}, {NULL}};
const Segment SGI293816[] = {{&S69832[8], 3}, {NULL}};
const Segment SGI293817[] = {{&S69832[13], 7}, {NULL}};
const Segment SGI293818[] = {{&S69832[7], 1}, {NULL}};
const Segment SGI293819[] = {{&S69832[16], 4}, {&S69832[28], 2}, {NULL}};
const Segment SGI293820[] = {{&S69967[70], 4}, {NULL}};
const Segment SGI293821[] = {{&S69967[8], 3}, {NULL}};
const Segment SGI293822[] = {{&S69967[13], 14}, {NULL}};
const Segment SGI293823[] = {{&S69967[7], 1}, {NULL}};
const Segment SGI293824[] = {{&S69967[16], 11}, {NULL}};
const Segment SGI293825[] = {{&S69968[73], 4}, {NULL}};
const Segment SGI293826[] = {{&S69968[8], 3}, {NULL}};
const Segment SGI293827[] = {{&S69968[13], 7}, {NULL}};
const Segment SGI293828[] = {{&S69968[25], 14}, {NULL}};
const Segment SGI293829[] = {{&S69968[7], 1}, {NULL}};
const Segment SGI293830[] = {{&S69968[16], 4}, {&S69968[28], 11}, {NULL}};
const Segment SGI293831[] = {{&S70072[94], 4}, {NULL}};
const Segment SGI293832[] = {{&S70072[8], 3}, {NULL}};
const Segment SGI293833[] = {{&S70072[13], 7}, {NULL}};
const Segment SGI293834[] = {{&S70072[25], 14}, {NULL}};
const Segment SGI293835[] = {{&S70072[7], 1}, {NULL}};
const Segment SGI293836[] = {{&S70072[16], 4}, {&S70072[28], 11}, {NULL}};
const Segment SGI293837[] = {{&S70072[101], 8}, {NULL}};
const Segment SGI293838[] = {{&S70112[53], 5}, {NULL}};
const Segment SGI293839[] = {{&S70112[8], 3}, {NULL}};
const Segment SGI293840[] = {{&S70112[13], 14}, {NULL}};
const Segment SGI293841[] = {{&S70112[7], 1}, {NULL}};
const Segment SGI293842[] = {{&S70112[16], 11}, {NULL}};
const Segment SGI293843[] = {{&S70112[59], 14}, {NULL}};
const Segment SGI293844[] = {{&S70123[69], 5}, {NULL}};
const Segment SGI293845[] = {{&S70123[8], 3}, {NULL}};
const Segment SGI293846[] = {{&S70123[13], 9}, {NULL}};
const Segment SGI293847[] = {{&S70123[30], 13}, {NULL}};
const Segment SGI293848[] = {{&S70123[7], 1}, {NULL}};
const Segment SGI293849[] = {{&S70123[75], 13}, {NULL}};
const Segment SGI293850[] = {{&S70365[67], 4}, {NULL}};
const Segment SGI293851[] = {{&S70365[8], 3}, {NULL}};
const Segment SGI293852[] = {{&S70365[13], 3}, {NULL}};
const Segment SGI293853[] = {{&S70365[13], 3}, {&S70365[21], 5}, {NULL}};
const Segment SGI293854[] = {{&S70365[7], 1}, {NULL}};
const Segment SGI293855[] = {{&S70435[57], 5}, {NULL}};
const Segment SGI293856[] = {{&S70435[8], 3}, {NULL}};
const Segment SGI293857[] = {{&S70435[13], 3}, {NULL}};
const Segment SGI293858[] = {{&S70435[13], 3}, {&S70435[18], 7}, {NULL}};
const Segment SGI293859[] = {{&S70435[7], 1}, {NULL}};
const Segment SGI293860[] = {{&S70466[55], 5}, {NULL}};
const Segment SGI293861[] = {{&S70466[8], 3}, {NULL}};
const Segment SGI293862[] = {{&S70466[13], 3}, {NULL}};
const Segment SGI293863[] = {{&S70466[13], 3}, {&S70466[18], 5}, {NULL}};
const Segment SGI293864[] = {{&S70466[7], 1}, {NULL}};
const Segment SGI293865[] = {{&S70699[53], 5}, {NULL}};
const Segment SGI293866[] = {{&S70699[8], 3}, {NULL}};
const Segment SGI293867[] = {{&S70699[13], 14}, {NULL}};
const Segment SGI293868[] = {{&S70699[7], 1}, {NULL}};
const Segment SGI293869[] = {{&S70699[16], 11}, {NULL}};
const Segment SGI293870[] = {{&S70699[59], 13}, {NULL}};
const Segment SGI293871[] = {{&S70700[65], 5}, {NULL}};
const Segment SGI293872[] = {{&S70700[8], 3}, {NULL}};
const Segment SGI293873[] = {{&S70700[13], 14}, {NULL}};
const Segment SGI293874[] = {{&S70700[7], 1}, {NULL}};
const Segment SGI293875[] = {{&S70700[16], 11}, {&S70700[32], 2}, {NULL}};
const Segment SGI293876[] = {{&S70700[75], 13}, {NULL}};
const Segment SGI293877[] = {{&S68052[8], 3}, {NULL}};
const Segment SGI293878[] = {{&S68052[33], 4}, {NULL}};
const Segment SGI293879[] = {{&S68052[63], 6}, {NULL}};
const Segment SGI293880[] = {{&S68052[13], 24}, {NULL}};
const Segment SGI293881[] = {{&S68052[33], 2}, {NULL}};
const Segment SGI293882[] = {{&S70898[56], 4}, {NULL}};
const Segment SGI293883[] = {{&S70898[8], 3}, {NULL}};
const Segment SGI293884[] = {{&S70898[13], 3}, {NULL}};
const Segment SGI293885[] = {{&S70898[7], 1}, {NULL}};
const Segment SGI293886[] = {{&S70898[64], 13}, {NULL}};
const Segment SGI293887[] = {{&S71013[60], 5}, {NULL}};
const Segment SGI293888[] = {{&S71013[8], 3}, {NULL}};
const Segment SGI293889[] = {{&S71013[13], 14}, {NULL}};
const Segment SGI293890[] = {{&S71013[7], 1}, {NULL}};
const Segment SGI293891[] = {{&S71013[16], 11}, {NULL}};
const Segment SGI293892[] = {{&S71013[66], 8}, {NULL}};
const Segment SGI293893[] = {{&S71097[87], 4}, {NULL}};
const Segment SGI293894[] = {{&S71097[8], 3}, {NULL}};
const Segment SGI293895[] = {{&S71097[13], 3}, {NULL}};
const Segment SGI293896[] = {{&S71097[13], 3}, {&S71097[60], 6}, {NULL}};
const Segment SGI293897[] = {{&S71097[60], 11}, {NULL}};
const Segment SGI293898[] = {{&S71097[7], 1}, {NULL}};
const Segment SGI293899[] = {{&S71178[73], 4}, {NULL}};
const Segment SGI293900[] = {{&S71178[8], 3}, {NULL}};
const Segment SGI293901[] = {{&S71178[13], 7}, {NULL}};
const Segment SGI293902[] = {{&S71178[25], 14}, {NULL}};
const Segment SGI293903[] = {{&S71178[7], 1}, {NULL}};
const Segment SGI293904[] = {{&S71178[16], 4}, {&S71178[28], 11}, {NULL}};
const Segment SGI293905[] = {{&S71179[70], 4}, {NULL}};
const Segment SGI293906[] = {{&S71179[8], 3}, {NULL}};
const Segment SGI293907[] = {{&S71179[13], 7}, {NULL}};
const Segment SGI293908[] = {{&S71179[25], 14}, {NULL}};
const Segment SGI293909[] = {{&S71179[7], 1}, {NULL}};
const Segment SGI293910[] = {{&S71179[16], 4}, {&S71179[28], 11}, {NULL}};
const Segment SGI293911[] = {{&S71248[75], 4}, {NULL}};
const Segment SGI293912[] = {{&S71248[8], 3}, {NULL}};
const Segment SGI293913[] = {{&S71248[13], 7}, {NULL}};
const Segment SGI293914[] = {{&S71248[25], 14}, {NULL}};
const Segment SGI293915[] = {{&S71248[7], 1}, {NULL}};
const Segment SGI293916[] = {{&S71248[16], 4}, {&S71248[28], 11}, {NULL}};
const Segment SGI293917[] = {{&S71248[82], 9}, {NULL}};
const Segment SGI293918[] = {{&S71249[75], 4}, {NULL}};
const Segment SGI293919[] = {{&S71249[8], 3}, {NULL}};
const Segment SGI293920[] = {{&S71249[13], 7}, {NULL}};
const Segment SGI293921[] = {{&S71249[25], 14}, {NULL}};
const Segment SGI293922[] = {{&S71249[7], 1}, {NULL}};
const Segment SGI293923[] = {{&S71249[16], 4}, {&S71249[28], 11}, {NULL}};
const Segment SGI293924[] = {{&S71249[82], 9}, {NULL}};
const Segment SGI293925[] = {{&S71250[70], 4}, {NULL}};
const Segment SGI293926[] = {{&S71250[8], 3}, {NULL}};
const Segment SGI293927[] = {{&S71250[13], 9}, {NULL}};
const Segment SGI293928[] = {{&S71250[30], 13}, {NULL}};
const Segment SGI293929[] = {{&S71250[7], 1}, {NULL}};
const Segment SGI293930[] = {{&S71250[77], 10}, {NULL}};
const Segment SGI293931[] = {{&S71300[55], 4}, {NULL}};
const Segment SGI293932[] = {{&S71300[8], 3}, {NULL}};
const Segment SGI293933[] = {{&S71300[13], 3}, {NULL}};
const Segment SGI293934[] = {{&S71300[7], 1}, {NULL}};
const Segment SGI293935[] = {{&S71300[62], 13}, {NULL}};
const Segment SGI293936[] = {{&S71331[55], 5}, {NULL}};
const Segment SGI293937[] = {{&S71331[8], 3}, {NULL}};
const Segment SGI293938[] = {{&S71331[13], 3}, {NULL}};
const Segment SGI293939[] = {{&S71331[13], 3}, {&S71331[18], 5}, {NULL}};
const Segment SGI293940[] = {{&S71331[7], 1}, {NULL}};
const Segment SGI293941[] = {{&S71339[85], 4}, {NULL}};
const Segment SGI293942[] = {{&S71339[8], 3}, {NULL}};
const Segment SGI293943[] = {{&S71339[13], 9}, {NULL}};
const Segment SGI293944[] = {{&S71339[31], 15}, {NULL}};
const Segment SGI293945[] = {{&S71339[7], 1}, {NULL}};
const Segment SGI293946[] = {{&S71339[94], 13}, {NULL}};
const Segment SGI293947[] = {{&S71340[80], 4}, {NULL}};
const Segment SGI293948[] = {{&S71340[8], 3}, {NULL}};
const Segment SGI293949[] = {{&S71340[13], 9}, {NULL}};
const Segment SGI293950[] = {{&S71340[7], 1}, {NULL}};
const Segment SGI293951[] = {{&S71340[89], 13}, {NULL}};
const Segment SGI293952[] = {{&S71408[69], 4}, {NULL}};
const Segment SGI293953[] = {{&S71408[8], 3}, {NULL}};
const Segment SGI293954[] = {{&S71408[15], 3}, {&S71408[23], 7}, {NULL}};
const Segment SGI293955[] = {{&S71408[7], 1}, {NULL}};
const Segment SGI293956[] = {{&S71409[67], 4}, {NULL}};
const Segment SGI293957[] = {{&S71409[8], 3}, {NULL}};
const Segment SGI293958[] = {{&S71409[13], 3}, {NULL}};
const Segment SGI293959[] = {{&S71409[13], 3}, {&S71409[21], 7}, {NULL}};
const Segment SGI293960[] = {{&S71409[7], 1}, {NULL}};
const Segment SGI293961[] = {{&S71628[70], 4}, {NULL}};
const Segment SGI293962[] = {{&S71628[8], 3}, {NULL}};
const Segment SGI293963[] = {{&S71628[13], 3}, {NULL}};
const Segment SGI293964[] = {{&S71628[13], 3}, {&S71628[21], 7}, {NULL}};
const Segment SGI293965[] = {{&S71628[7], 1}, {NULL}};
const Segment SGI293966[] = {{&S71712[54], 5}, {NULL}};
const Segment SGI293967[] = {{&S71712[8], 3}, {NULL}};
const Segment SGI293968[] = {{&S71712[13], 3}, {NULL}};
const Segment SGI293969[] = {{&S71712[7], 1}, {NULL}};
const Segment SGI293970[] = {{&S71712[62], 13}, {NULL}};
const Segment SGI293971[] = {{&S71859[65], 5}, {NULL}};
const Segment SGI293972[] = {{&S71859[8], 3}, {NULL}};
const Segment SGI293973[] = {{&S71859[13], 14}, {NULL}};
const Segment SGI293974[] = {{&S71859[7], 1}, {NULL}};
const Segment SGI293975[] = {{&S71859[16], 11}, {&S71859[32], 2}, {NULL}};
const Segment SGI293976[] = {{&S71859[77], 13}, {NULL}};
const Segment SGI293977[] = {{&S71860[53], 5}, {NULL}};
const Segment SGI293978[] = {{&S71860[8], 3}, {NULL}};
const Segment SGI293979[] = {{&S71860[13], 14}, {NULL}};
const Segment SGI293980[] = {{&S71860[7], 1}, {NULL}};
const Segment SGI293981[] = {{&S71860[16], 11}, {NULL}};
const Segment SGI293982[] = {{&S71860[61], 13}, {NULL}};
const Segment SGI293983[] = {{&S55502[0], 46}, {NULL}};
const Segment SGI293984[] = {{&S55503[0], 67}, {NULL}};
const Segment SGI293985[] = {{&S55504[0], 67}, {NULL}};
const Segment SGI293986[] = {{&S55505[18], 16}, {NULL}};
const Segment SGI293987[] = {{&S55505[0], 88}, {NULL}};
const Segment SGI293988[] = {{&S55506[0], 54}, {NULL}};
const Segment SGI293989[] = {{&S55507[0], 76}, {NULL}};
const Segment SGI293990[] = {{&S55508[0], 73}, {NULL}};
const Segment SGI293991[] = {{&S55509[0], 73}, {NULL}};
const Segment SGI293992[] = {{&S55510[0], 73}, {NULL}};
const Segment SGI293993[] = {{&S55511[0], 73}, {NULL}};
const Segment SGI293994[] = {{&S55512[0], 73}, {NULL}};
const Segment SGI293995[] = {{&S55513[0], 73}, {NULL}};
const Segment SGI293996[] = {{&S55514[0], 73}, {NULL}};
const Segment SGI293997[] = {{&S55515[0], 73}, {NULL}};
const Segment SGI293998[] = {{&S55516[0], 76}, {NULL}};
const Segment SGI293999[] = {{&S55517[0], 73}, {NULL}};
const Segment SGI294000[] = {{&S55518[0], 73}, {NULL}};
const Segment SGI294001[] = {{&S55519[0], 76}, {NULL}};
const Segment SGI294002[] = {{&S55522[96], 16}, {NULL}};
const Segment SGI294003[] = {{&S55522[0], 133}, {NULL}};
const Segment SGI294004[] = {{&S55523[96], 16}, {NULL}};
const Segment SGI294005[] = {{&S55523[0], 133}, {NULL}};
const Segment SGI294006[] = {{&S55524[96], 16}, {NULL}};
const Segment SGI294007[] = {{&S55524[0], 133}, {NULL}};
const Segment SGI294008[] = {{&S55525[87], 7}, {NULL}};
const Segment SGI294009[] = {{&S55525[0], 134}, {NULL}};
const Segment SGI294010[] = {{&S55526[0], 85}, {NULL}};
const Segment SGI294011[] = {{&S55527[0], 117}, {NULL}};
const Segment SGI294012[] = {{&S55528[0], 117}, {NULL}};
const Segment SGI294013[] = {{&S55529[0], 117}, {NULL}};
const Segment SGI294014[] = {{&S55530[96], 16}, {NULL}};
const Segment SGI294015[] = {{&S55530[0], 133}, {NULL}};
const Segment SGI294016[] = {{&S55531[0], 87}, {NULL}};
const Segment SGI294017[] = {{&S55532[0], 91}, {NULL}};
const Segment SGI294018[] = {{&S55533[82], 17}, {NULL}};
const Segment SGI294019[] = {{&S55533[0], 99}, {NULL}};
const Segment SGI294020[] = {{&S55534[0], 117}, {NULL}};
const Segment SGI294021[] = {{&S55535[0], 117}, {NULL}};
const Segment SGI294022[] = {{&S55536[0], 117}, {NULL}};
const Segment SGI294023[] = {{&S55537[0], 117}, {NULL}};
const Segment SGI294024[] = {{&S55538[0], 117}, {NULL}};
const Segment SGI294025[] = {{&S55539[0], 117}, {NULL}};
const Segment SGI294026[] = {{&S55540[0], 117}, {NULL}};
const Segment SGI294027[] = {{&S55541[0], 111}, {NULL}};
const Segment SGI294028[] = {{&S55542[0], 111}, {NULL}};
const Segment SGI294029[] = {{&S55543[0], 85}, {NULL}};
const Segment SGI294030[] = {{&S55544[0], 91}, {NULL}};
const Segment SGI294031[] = {{&S55545[0], 91}, {NULL}};
const Segment SGI294032[] = {{&S55546[0], 117}, {NULL}};
const Segment SGI294033[] = {{&S55547[0], 117}, {NULL}};
const Segment SGI294034[] = {{&S55548[96], 16}, {NULL}};
const Segment SGI294035[] = {{&S55548[0], 133}, {NULL}};
const Segment SGI294036[] = {{&S55549[0], 150}, {NULL}};
const Segment SGI294037[] = {{&S55550[0], 91}, {NULL}};
const Segment SGI294038[] = {{&S72122[65], 5}, {NULL}};
const Segment SGI294039[] = {{&S72122[8], 3}, {NULL}};
const Segment SGI294040[] = {{&S72122[13], 14}, {NULL}};
const Segment SGI294041[] = {{&S72122[7], 1}, {NULL}};
const Segment SGI294042[] = {{&S72122[16], 11}, {&S72122[32], 2}, {NULL}};
const Segment SGI294043[] = {{&S72122[75], 13}, {NULL}};
const Segment SGI294044[] = {{&S72123[53], 5}, {NULL}};
const Segment SGI294045[] = {{&S72123[8], 3}, {NULL}};
const Segment SGI294046[] = {{&S72123[13], 14}, {NULL}};
const Segment SGI294047[] = {{&S72123[7], 1}, {NULL}};
const Segment SGI294048[] = {{&S72123[16], 11}, {NULL}};
const Segment SGI294049[] = {{&S72123[61], 13}, {NULL}};
const Segment SGI294050[] = {{&S72154[76], 4}, {NULL}};
const Segment SGI294051[] = {{&S72154[8], 3}, {NULL}};
const Segment SGI294052[] = {{&S72154[13], 7}, {NULL}};
const Segment SGI294053[] = {{&S72154[25], 14}, {NULL}};
const Segment SGI294054[] = {{&S72154[7], 1}, {NULL}};
const Segment SGI294055[] = {{&S72154[16], 4}, {&S72154[28], 11}, {NULL}};
const Segment SGI294056[] = {{&S72154[86], 9}, {NULL}};
const Segment SGI294057[] = {{&S72276[55], 5}, {NULL}};
const Segment SGI294058[] = {{&S72276[8], 3}, {NULL}};
const Segment SGI294059[] = {{&S72276[13], 3}, {NULL}};
const Segment SGI294060[] = {{&S72276[13], 3}, {&S72276[18], 5}, {NULL}};
const Segment SGI294061[] = {{&S72276[7], 1}, {NULL}};
const Segment SGI294062[] = {{&S72317[68], 4}, {NULL}};
const Segment SGI294063[] = {{&S72317[8], 3}, {NULL}};
const Segment SGI294064[] = {{&S72317[13], 9}, {NULL}};
const Segment SGI294065[] = {{&S72317[28], 13}, {NULL}};
const Segment SGI294066[] = {{&S72317[7], 1}, {NULL}};
const Segment SGI294067[] = {{&S72345[75], 4}, {NULL}};
const Segment SGI294068[] = {{&S72345[8], 3}, {NULL}};
const Segment SGI294069[] = {{&S72345[13], 14}, {NULL}};
const Segment SGI294070[] = {{&S72345[13], 14}, {&S72345[35], 13}, {NULL}};
const Segment SGI294071[] = {{&S72345[7], 1}, {NULL}};
const Segment SGI294072[] = {{&S72345[16], 11}, {NULL}};
const Segment SGI294073[] = {{&S72346[60], 5}, {NULL}};
const Segment SGI294074[] = {{&S72346[8], 3}, {NULL}};
const Segment SGI294075[] = {{&S72346[13], 14}, {NULL}};
const Segment SGI294076[] = {{&S72346[7], 1}, {NULL}};
const Segment SGI294077[] = {{&S72346[16], 11}, {NULL}};
const Segment SGI294078[] = {{&S72346[66], 13}, {NULL}};
const Segment SGI294079[] = {{&S72404[68], 4}, {NULL}};
const Segment SGI294080[] = {{&S72404[8], 3}, {NULL}};
const Segment SGI294081[] = {{&S72404[13], 3}, {NULL}};
const Segment SGI294082[] = {{&S72404[13], 3}, {&S72404[21], 5}, {NULL}};
const Segment SGI294083[] = {{&S72404[7], 1}, {NULL}};
const Segment SGI294084[] = {{&S72405[111], 4}, {NULL}};
const Segment SGI294085[] = {{&S72405[8], 3}, {NULL}};
const Segment SGI294086[] = {{&S72405[13], 9}, {NULL}};
const Segment SGI294087[] = {{&S72405[33], 13}, {NULL}};
const Segment SGI294088[] = {{&S72405[7], 1}, {NULL}};
const Segment SGI294089[] = {{&S72405[73], 10}, {NULL}};
const Segment SGI294090[] = {{&S72506[7], 4}, {NULL}};
const Segment SGI294091[] = {{&S72506[8], 3}, {NULL}};
const Segment SGI294092[] = {{&S72506[13], 7}, {NULL}};
const Segment SGI294093[] = {{&S72506[25], 14}, {NULL}};
const Segment SGI294094[] = {{&S72506[7], 1}, {NULL}};
const Segment SGI294095[] = {{&S72506[16], 4}, {&S72506[28], 11}, {NULL}};
const Segment SGI294096[] = {{&S72512[69], 5}, {NULL}};
const Segment SGI294097[] = {{&S72512[8], 3}, {NULL}};
const Segment SGI294098[] = {{&S72512[13], 9}, {NULL}};
const Segment SGI294099[] = {{&S72512[30], 13}, {NULL}};
const Segment SGI294100[] = {{&S72512[7], 1}, {NULL}};
const Segment SGI294101[] = {{&S72512[75], 13}, {NULL}};
const Segment SGI294102[] = {{&S72570[95], 4}, {NULL}};
const Segment SGI294103[] = {{&S72570[40], 3}, {NULL}};
const Segment SGI294104[] = {{&S72570[45], 3}, {NULL}};
const Segment SGI294105[] = {{&S72570[7], 1}, {&S72570[39], 1}, {NULL}};
const Segment SGI294106[] = {{&S72571[64], 4}, {NULL}};
const Segment SGI294107[] = {{&S72571[8], 3}, {NULL}};
const Segment SGI294108[] = {{&S72571[13], 3}, {NULL}};
const Segment SGI294109[] = {{&S72571[7], 1}, {NULL}};
const Segment SGI294110[] = {{&S72572[58], 5}, {NULL}};
const Segment SGI294111[] = {{&S72572[8], 3}, {NULL}};
const Segment SGI294112[] = {{&S72572[13], 3}, {NULL}};
const Segment SGI294113[] = {{&S72572[7], 1}, {NULL}};
const Segment SGI294114[] = {{&S72618[53], 5}, {NULL}};
const Segment SGI294115[] = {{&S72618[8], 3}, {NULL}};
const Segment SGI294116[] = {{&S72618[13], 14}, {NULL}};
const Segment SGI294117[] = {{&S72618[7], 1}, {NULL}};
const Segment SGI294118[] = {{&S72618[16], 11}, {NULL}};
const Segment SGI294119[] = {{&S72618[59], 13}, {NULL}};
const Segment SGI294120[] = {{&S72658[57], 5}, {NULL}};
const Segment SGI294121[] = {{&S72658[8], 3}, {NULL}};
const Segment SGI294122[] = {{&S72658[13], 3}, {NULL}};
const Segment SGI294123[] = {{&S72658[13], 3}, {&S72658[18], 7}, {NULL}};
const Segment SGI294124[] = {{&S72658[7], 1}, {NULL}};
const Segment SGI294125[] = {{&S72742[69], 5}, {NULL}};
const Segment SGI294126[] = {{&S72742[8], 3}, {NULL}};
const Segment SGI294127[] = {{&S72742[13], 9}, {NULL}};
const Segment SGI294128[] = {{&S72742[30], 13}, {NULL}};
const Segment SGI294129[] = {{&S72742[7], 1}, {NULL}};
const Segment SGI294130[] = {{&S72742[77], 13}, {NULL}};
const Segment SGI294131[] = {{&S72859[77], 4}, {NULL}};
const Segment SGI294132[] = {{&S72859[8], 3}, {NULL}};
const Segment SGI294133[] = {{&S72859[13], 7}, {NULL}};
const Segment SGI294134[] = {{&S72859[7], 1}, {NULL}};
const Segment SGI294135[] = {{&S72859[16], 4}, {&S72859[28], 15}, {NULL}};
const Segment SGI294136[] = {{&S72860[54], 4}, {NULL}};
const Segment SGI294137[] = {{&S72860[8], 3}, {NULL}};
const Segment SGI294138[] = {{&S72860[13], 14}, {NULL}};
const Segment SGI294139[] = {{&S72860[7], 1}, {NULL}};
const Segment SGI294140[] = {{&S72860[16], 11}, {NULL}};
const Segment SGI294141[] = {{&S72861[73], 4}, {NULL}};
const Segment SGI294142[] = {{&S72861[8], 3}, {NULL}};
const Segment SGI294143[] = {{&S72861[13], 7}, {NULL}};
const Segment SGI294144[] = {{&S72861[25], 14}, {NULL}};
const Segment SGI294145[] = {{&S72861[7], 1}, {NULL}};
const Segment SGI294146[] = {{&S72861[16], 4}, {&S72861[28], 11}, {NULL}};
const Segment SGI294147[] = {{&S72952[53], 5}, {NULL}};
const Segment SGI294148[] = {{&S72952[8], 3}, {NULL}};
const Segment SGI294149[] = {{&S72952[13], 14}, {NULL}};
const Segment SGI294150[] = {{&S72952[7], 1}, {NULL}};
const Segment SGI294151[] = {{&S72952[16], 11}, {NULL}};
const Segment SGI294152[] = {{&S72952[61], 13}, {NULL}};
const Segment SGI294153[] = {{&S72993[66], 4}, {NULL}};
const Segment SGI294154[] = {{&S72993[8], 3}, {NULL}};
const Segment SGI294155[] = {{&S72993[13], 3}, {NULL}};
const Segment SGI294156[] = {{&S72993[13], 3}, {&S72993[21], 5}, {NULL}};
const Segment SGI294157[] = {{&S72993[7], 1}, {NULL}};
const Segment SGI294158[] = {{&S73044[69], 5}, {NULL}};
const Segment SGI294159[] = {{&S73044[8], 3}, {NULL}};
const Segment SGI294160[] = {{&S73044[13], 9}, {NULL}};
const Segment SGI294161[] = {{&S73044[30], 13}, {NULL}};
const Segment SGI294162[] = {{&S73044[7], 1}, {NULL}};
const Segment SGI294163[] = {{&S73044[75], 13}, {NULL}};
const Segment SGI294164[] = {{&S73109[63], 4}, {NULL}};
const Segment SGI294165[] = {{&S73109[8], 3}, {NULL}};
const Segment SGI294166[] = {{&S73109[13], 14}, {NULL}};
const Segment SGI294167[] = {{&S73109[7], 1}, {NULL}};
const Segment SGI294168[] = {{&S73109[16], 11}, {NULL}};
const Segment SGI294169[] = {{&S73109[68], 11}, {NULL}};
const Segment SGI294170[] = {{&S73380[76], 4}, {NULL}};
const Segment SGI294171[] = {{&S73380[8], 3}, {NULL}};
const Segment SGI294172[] = {{&S73380[13], 7}, {NULL}};
const Segment SGI294173[] = {{&S73380[25], 14}, {NULL}};
const Segment SGI294174[] = {{&S73380[7], 1}, {NULL}};
const Segment SGI294175[] = {{&S73380[16], 4}, {&S73380[28], 11}, {NULL}};
const Segment SGI294176[] = {{&S73380[84], 9}, {NULL}};
const Segment SGI294177[] = {{&S73381[76], 4}, {NULL}};
const Segment SGI294178[] = {{&S73381[8], 3}, {NULL}};
const Segment SGI294179[] = {{&S73381[13], 7}, {NULL}};
const Segment SGI294180[] = {{&S73381[25], 14}, {NULL}};
const Segment SGI294181[] = {{&S73381[7], 1}, {NULL}};
const Segment SGI294182[] = {{&S73381[16], 4}, {&S73381[28], 11}, {NULL}};
const Segment SGI294183[] = {{&S73381[84], 9}, {NULL}};
const Segment SGI294184[] = {{&S73430[54], 5}, {NULL}};
const Segment SGI294185[] = {{&S73430[8], 3}, {NULL}};
const Segment SGI294186[] = {{&S73430[13], 3}, {NULL}};
const Segment SGI294187[] = {{&S73430[7], 1}, {NULL}};
const Segment SGI294188[] = {{&S73430[60], 13}, {NULL}};
const Segment SGI294189[] = {{&S73529[7], 4}, {NULL}};
const Segment SGI294190[] = {{&S73529[8], 3}, {NULL}};
const Segment SGI294191[] = {{&S73529[13], 14}, {NULL}};
const Segment SGI294192[] = {{&S73529[7], 1}, {NULL}};
const Segment SGI294193[] = {{&S73529[16], 11}, {NULL}};
const Segment SGI294194[] = {{&S73530[7], 4}, {NULL}};
const Segment SGI294195[] = {{&S73530[8], 3}, {NULL}};
const Segment SGI294196[] = {{&S73530[13], 7}, {NULL}};
const Segment SGI294197[] = {{&S73530[25], 14}, {NULL}};
const Segment SGI294198[] = {{&S73530[7], 1}, {NULL}};
const Segment SGI294199[] = {{&S73530[16], 4}, {&S73530[28], 11}, {NULL}};
const Segment SGI294200[] = {{&S73637[67], 4}, {NULL}};
const Segment SGI294201[] = {{&S73637[8], 3}, {NULL}};
const Segment SGI294202[] = {{&S73637[13], 3}, {NULL}};
const Segment SGI294203[] = {{&S73637[7], 1}, {NULL}};
const Segment SGI294204[] = {{&S73637[74], 9}, {NULL}};
const Segment SGI294205[] = {{&S73655[43], 5}, {NULL}};
const Segment SGI294206[] = {{&S73655[8], 3}, {NULL}};
const Segment SGI294207[] = {{&S73655[13], 14}, {NULL}};
const Segment SGI294208[] = {{&S73655[7], 1}, {NULL}};
const Segment SGI294209[] = {{&S73655[16], 11}, {NULL}};
const Segment SGI294210[] = {{&S73655[62], 14}, {NULL}};
const Segment SGI294211[] = {{&S73813[55], 5}, {NULL}};
const Segment SGI294212[] = {{&S73813[8], 3}, {NULL}};
const Segment SGI294213[] = {{&S73813[13], 3}, {NULL}};
const Segment SGI294214[] = {{&S73813[13], 3}, {&S73813[18], 5}, {NULL}};
const Segment SGI294215[] = {{&S73813[7], 1}, {NULL}};
const Segment SGI294216[] = {{&S73910[53], 5}, {NULL}};
const Segment SGI294217[] = {{&S73910[8], 3}, {NULL}};
const Segment SGI294218[] = {{&S73910[13], 14}, {NULL}};
const Segment SGI294219[] = {{&S73910[7], 1}, {NULL}};
const Segment SGI294220[] = {{&S73910[16], 11}, {NULL}};
const Segment SGI294221[] = {{&S73910[59], 13}, {NULL}};
const Segment SGI294222[] = {{&S73912[8], 3}, {NULL}};
const Segment SGI294223[] = {{&S73912[13], 3}, {NULL}};
const Segment SGI294224[] = {{&S73912[12], 1}, {NULL}};
const Segment SGI294225[] = {{&S73913[8], 3}, {NULL}};
const Segment SGI294226[] = {{&S73913[13], 3}, {NULL}};
const Segment SGI294227[] = {{&S73913[59], 1}, {NULL}};
const Segment SGI294228[] = {{&S73914[8], 3}, {NULL}};
const Segment SGI294229[] = {{&S73914[13], 3}, {NULL}};
const Segment SGI294230[] = {{&S73984[72], 4}, {NULL}};
const Segment SGI294231[] = {{&S73984[8], 3}, {NULL}};
const Segment SGI294232[] = {{&S73984[13], 3}, {NULL}};
const Segment SGI294233[] = {{&S73984[7], 1}, {NULL}};
const Segment SGI294234[] = {{&S73984[77], 17}, {NULL}};
const Segment SGI294235[] = {{&S74004[60], 5}, {NULL}};
const Segment SGI294236[] = {{&S74004[8], 3}, {NULL}};
const Segment SGI294237[] = {{&S74004[13], 14}, {NULL}};
const Segment SGI294238[] = {{&S74004[7], 1}, {NULL}};
const Segment SGI294239[] = {{&S74004[16], 11}, {NULL}};
const Segment SGI294240[] = {{&S74004[66], 9}, {NULL}};
const Segment SGI294241[] = {{&S74005[53], 5}, {NULL}};
const Segment SGI294242[] = {{&S74005[8], 3}, {NULL}};
const Segment SGI294243[] = {{&S74005[13], 14}, {NULL}};
const Segment SGI294244[] = {{&S74005[7], 1}, {NULL}};
const Segment SGI294245[] = {{&S74005[16], 11}, {NULL}};
const Segment SGI294246[] = {{&S74005[59], 9}, {NULL}};
const Segment SGI294247[] = {{&S74013[55], 5}, {NULL}};
const Segment SGI294248[] = {{&S74013[8], 3}, {NULL}};
const Segment SGI294249[] = {{&S74013[13], 3}, {NULL}};
const Segment SGI294250[] = {{&S74013[13], 3}, {&S74013[18], 5}, {NULL}};
const Segment SGI294251[] = {{&S74013[7], 1}, {NULL}};
const Segment SGI294252[] = {{&S74060[68], 4}, {NULL}};
const Segment SGI294253[] = {{&S74060[8], 3}, {NULL}};
const Segment SGI294254[] = {{&S74060[13], 3}, {NULL}};
const Segment SGI294255[] = {{&S74060[13], 3}, {&S74060[21], 5}, {NULL}};
const Segment SGI294256[] = {{&S74060[7], 1}, {NULL}};
const Segment SGI294257[] = {{&S74075[53], 5}, {NULL}};
const Segment SGI294258[] = {{&S74075[8], 3}, {NULL}};
const Segment SGI294259[] = {{&S74075[13], 14}, {NULL}};
const Segment SGI294260[] = {{&S74075[7], 1}, {NULL}};
const Segment SGI294261[] = {{&S74075[16], 11}, {NULL}};
const Segment SGI294262[] = {{&S74075[63], 14}, {NULL}};
const Segment SGI294263[] = {{&S74079[53], 5}, {NULL}};
const Segment SGI294264[] = {{&S74079[8], 3}, {NULL}};
const Segment SGI294265[] = {{&S74079[13], 14}, {NULL}};
const Segment SGI294266[] = {{&S74079[7], 1}, {NULL}};
const Segment SGI294267[] = {{&S74079[16], 11}, {NULL}};
const Segment SGI294268[] = {{&S74079[59], 14}, {NULL}};
const Segment SGI294269[] = {{&S74108[55], 5}, {NULL}};
const Segment SGI294270[] = {{&S74108[8], 3}, {NULL}};
const Segment SGI294271[] = {{&S74108[13], 3}, {NULL}};
const Segment SGI294272[] = {{&S74108[13], 3}, {&S74108[18], 5}, {NULL}};
const Segment SGI294273[] = {{&S74108[7], 1}, {NULL}};
const Segment SGI294274[] = {{&S74139[78], 4}, {NULL}};
const Segment SGI294275[] = {{&S74139[8], 3}, {NULL}};
const Segment SGI294276[] = {{&S74139[13], 7}, {NULL}};
const Segment SGI294277[] = {{&S74139[25], 14}, {NULL}};
const Segment SGI294278[] = {{&S74139[7], 1}, {NULL}};
const Segment SGI294279[] = {{&S74139[16], 4}, {&S74139[28], 11}, {NULL}};
const Segment SGI294280[] = {{&S74139[87], 13}, {NULL}};
const Segment SGI294281[] = {{&S74167[78], 4}, {NULL}};
const Segment SGI294282[] = {{&S74167[8], 3}, {NULL}};
const Segment SGI294283[] = {{&S74167[13], 9}, {NULL}};
const Segment SGI294284[] = {{&S74167[31], 15}, {NULL}};
const Segment SGI294285[] = {{&S74167[7], 1}, {NULL}};
const Segment SGI294286[] = {{&S74293[54], 5}, {NULL}};
const Segment SGI294287[] = {{&S74293[8], 3}, {NULL}};
const Segment SGI294288[] = {{&S74293[13], 3}, {NULL}};
const Segment SGI294289[] = {{&S74293[7], 1}, {NULL}};
const Segment SGI294290[] = {{&S74293[60], 13}, {NULL}};
const Segment SGI294291[] = {{&S74299[50], 5}, {NULL}};
const Segment SGI294292[] = {{&S74299[8], 3}, {NULL}};
const Segment SGI294293[] = {{&S74299[13], 14}, {NULL}};
const Segment SGI294294[] = {{&S74299[7], 1}, {NULL}};
const Segment SGI294295[] = {{&S74299[16], 11}, {NULL}};
const Segment SGI294296[] = {{&S74299[69], 14}, {NULL}};
const Segment SGI294297[] = {{&S74322[57], 5}, {NULL}};
const Segment SGI294298[] = {{&S74322[8], 3}, {NULL}};
const Segment SGI294299[] = {{&S74322[13], 3}, {NULL}};
const Segment SGI294300[] = {{&S74322[13], 3}, {&S74322[18], 7}, {NULL}};
const Segment SGI294301[] = {{&S74322[7], 1}, {NULL}};
const Segment SGI294302[] = {{&S74349[53], 5}, {NULL}};
const Segment SGI294303[] = {{&S74349[8], 3}, {NULL}};
const Segment SGI294304[] = {{&S74349[13], 14}, {NULL}};
const Segment SGI294305[] = {{&S74349[7], 1}, {NULL}};
const Segment SGI294306[] = {{&S74349[16], 11}, {NULL}};
const Segment SGI294307[] = {{&S74349[61], 13}, {NULL}};
const Segment SGI294308[] = {{&S74603[83], 4}, {NULL}};
const Segment SGI294309[] = {{&S74603[8], 3}, {NULL}};
const Segment SGI294310[] = {{&S74603[13], 7}, {NULL}};
const Segment SGI294311[] = {{&S74603[25], 14}, {NULL}};
const Segment SGI294312[] = {{&S74603[7], 1}, {NULL}};
const Segment SGI294313[] = {{&S74603[16], 4}, {&S74603[28], 11}, {NULL}};
const Segment SGI294314[] = {{&S74603[66], 9}, {NULL}};
const Segment SGI294315[] = {{&S58167[0], 73}, {NULL}};
const Segment SGI294316[] = {{&S58182[0], 74}, {NULL}};
const Segment SGI294317[] = {{&S75134[55], 4}, {NULL}};
const Segment SGI294318[] = {{&S75134[8], 3}, {NULL}};
const Segment SGI294319[] = {{&S75134[13], 14}, {NULL}};
const Segment SGI294320[] = {{&S75134[7], 1}, {NULL}};
const Segment SGI294321[] = {{&S75134[16], 11}, {NULL}};
const Segment SGI294322[] = {{&S75134[63], 13}, {NULL}};
const Segment SGI294323[] = {{&S59056[0], 121}, {NULL}};
const Segment SGI294324[] = {{&S59057[0], 121}, {NULL}};
const Segment SGI294325[] = {{&S59058[0], 121}, {NULL}};
const Segment SGI294326[] = {{&S59059[0], 121}, {NULL}};
const Segment SGI294327[] = {{&S59060[0], 121}, {NULL}};
const Segment SGI294328[] = {{&S59062[0], 95}, {NULL}};
const Segment SGI294329[] = {{&S59064[0], 121}, {NULL}};
const Segment SGI294330[] = {{&S59065[0], 121}, {NULL}};
const Segment SGI294331[] = {{&S59066[0], 121}, {NULL}};
const Segment SGI294332[] = {{&S59068[0], 121}, {NULL}};
const Segment SGI294333[] = {{&S59069[0], 121}, {NULL}};
const Segment SGI294334[] = {{&S59070[0], 121}, {NULL}};
const Segment SGI294335[] = {{&S59071[0], 121}, {NULL}};
const Segment SGI294336[] = {{&S75410[76], 4}, {NULL}};
const Segment SGI294337[] = {{&S75410[8], 3}, {NULL}};
const Segment SGI294338[] = {{&S75410[13], 9}, {NULL}};
const Segment SGI294339[] = {{&S75410[31], 13}, {NULL}};
const Segment SGI294340[] = {{&S75410[7], 1}, {NULL}};
const Segment SGI294341[] = {{&S75410[81], 9}, {NULL}};
const Segment SGI294342[] = {{&S75411[57], 5}, {NULL}};
const Segment SGI294343[] = {{&S75411[8], 3}, {NULL}};
const Segment SGI294344[] = {{&S75411[13], 3}, {NULL}};
const Segment SGI294345[] = {{&S75411[13], 3}, {&S75411[18], 7}, {NULL}};
const Segment SGI294346[] = {{&S75411[7], 1}, {NULL}};
const Segment SGI294347[] = {{&S75483[65], 4}, {NULL}};
const Segment SGI294348[] = {{&S75483[8], 3}, {NULL}};
const Segment SGI294349[] = {{&S75483[13], 3}, {NULL}};
const Segment SGI294350[] = {{&S75483[59], 10}, {NULL}};
const Segment SGI294351[] = {{&S75483[7], 1}, {NULL}};
const Segment SGI294352[] = {{&S75483[59], 7}, {NULL}};
const Segment SGI294353[] = {{&S75554[69], 5}, {NULL}};
const Segment SGI294354[] = {{&S75554[8], 3}, {NULL}};
const Segment SGI294355[] = {{&S75554[13], 9}, {NULL}};
const Segment SGI294356[] = {{&S75554[30], 13}, {NULL}};
const Segment SGI294357[] = {{&S75554[7], 1}, {NULL}};
const Segment SGI294358[] = {{&S75554[77], 13}, {NULL}};
const Segment SGI294359[] = {{&S75698[60], 5}, {NULL}};
const Segment SGI294360[] = {{&S75698[8], 3}, {NULL}};
const Segment SGI294361[] = {{&S75698[13], 14}, {NULL}};
const Segment SGI294362[] = {{&S75698[7], 1}, {NULL}};
const Segment SGI294363[] = {{&S75698[16], 11}, {NULL}};
const Segment SGI294364[] = {{&S75698[66], 14}, {NULL}};
const Segment SGI294365[] = {{&S75972[76], 4}, {NULL}};
const Segment SGI294366[] = {{&S75972[8], 3}, {NULL}};
const Segment SGI294367[] = {{&S75972[13], 7}, {NULL}};
const Segment SGI294368[] = {{&S75972[7], 1}, {NULL}};
const Segment SGI294369[] = {{&S75972[16], 4}, {&S75972[28], 12}, {NULL}};
const Segment SGI294370[] = {{&S76068[76], 4}, {NULL}};
const Segment SGI294371[] = {{&S76068[8], 3}, {NULL}};
const Segment SGI294372[] = {{&S76068[13], 7}, {NULL}};
const Segment SGI294373[] = {{&S76068[25], 14}, {NULL}};
const Segment SGI294374[] = {{&S76068[7], 1}, {NULL}};
const Segment SGI294375[] = {{&S76068[16], 4}, {&S76068[28], 11}, {NULL}};
const Segment SGI294376[] = {{&S76141[70], 4}, {NULL}};
const Segment SGI294377[] = {{&S76141[8], 3}, {NULL}};
const Segment SGI294378[] = {{&S76141[13], 9}, {NULL}};
const Segment SGI294379[] = {{&S76141[30], 13}, {NULL}};
const Segment SGI294380[] = {{&S76141[7], 1}, {NULL}};
const Segment SGI294381[] = {{&S76271[57], 4}, {NULL}};
const Segment SGI294382[] = {{&S76271[8], 3}, {NULL}};
const Segment SGI294383[] = {{&S76271[13], 7}, {NULL}};
const Segment SGI294384[] = {{&S76271[7], 1}, {NULL}};
const Segment SGI294385[] = {{&S76271[16], 4}, {&S76271[28], 2}, {NULL}};
const Segment SGI294386[] = {{&S76272[8], 3}, {NULL}};
const Segment SGI294387[] = {{&S76272[13], 3}, {NULL}};
const Segment SGI294388[] = {{&S76272[24], 2}, {NULL}};
const Segment SGI294389[] = {{&S76272[53], 11}, {NULL}};
const Segment SGI294390[] = {{&S76379[75], 4}, {NULL}};
const Segment SGI294391[] = {{&S76379[8], 3}, {NULL}};
const Segment SGI294392[] = {{&S76379[13], 3}, {NULL}};
const Segment SGI294393[] = {{&S76379[13], 3}, {&S76379[61], 6}, {NULL}};
const Segment SGI294394[] = {{&S76379[7], 1}, {NULL}};
const Segment SGI294395[] = {{&S76384[55], 5}, {NULL}};
const Segment SGI294396[] = {{&S76384[8], 3}, {NULL}};
const Segment SGI294397[] = {{&S76384[13], 14}, {NULL}};
const Segment SGI294398[] = {{&S76384[7], 1}, {NULL}};
const Segment SGI294399[] = {{&S76384[16], 11}, {NULL}};
const Segment SGI294400[] = {{&S76384[63], 9}, {NULL}};
const Segment SGI294401[] = {{&S76525[55], 5}, {NULL}};
const Segment SGI294402[] = {{&S76525[8], 3}, {NULL}};
const Segment SGI294403[] = {{&S76525[13], 3}, {NULL}};
const Segment SGI294404[] = {{&S76525[13], 3}, {&S76525[18], 5}, {NULL}};
const Segment SGI294405[] = {{&S76525[7], 1}, {NULL}};
const Segment SGI294406[] = {{&S76547[75], 4}, {NULL}};
const Segment SGI294407[] = {{&S76547[8], 3}, {NULL}};
const Segment SGI294408[] = {{&S76547[13], 7}, {NULL}};
const Segment SGI294409[] = {{&S76547[25], 14}, {NULL}};
const Segment SGI294410[] = {{&S76547[7], 1}, {NULL}};
const Segment SGI294411[] = {{&S76547[16], 4}, {&S76547[28], 11}, {NULL}};
const Segment SGI294412[] = {{&S76547[82], 13}, {NULL}};
const Segment SGI294413[] = {{&S76552[58], 5}, {NULL}};
const Segment SGI294414[] = {{&S76552[8], 3}, {NULL}};
const Segment SGI294415[] = {{&S76552[13], 3}, {NULL}};
const Segment SGI294416[] = {{&S76552[7], 1}, {NULL}};
const Segment SGI294417[] = {{&S76552[66], 13}, {NULL}};
const Segment SGI294418[] = {{&S76571[52], 4}, {NULL}};
const Segment SGI294419[] = {{&S76571[8], 3}, {NULL}};
const Segment SGI294420[] = {{&S76571[13], 14}, {NULL}};
const Segment SGI294421[] = {{&S76571[7], 1}, {NULL}};
const Segment SGI294422[] = {{&S76571[16], 11}, {NULL}};
const Segment SGI294423[] = {{&S76571[57], 13}, {NULL}};
const Segment SGI294424[] = {{&S76621[67], 4}, {NULL}};
const Segment SGI294425[] = {{&S76621[8], 3}, {NULL}};
const Segment SGI294426[] = {{&S76621[13], 7}, {NULL}};
const Segment SGI294427[] = {{&S76621[7], 1}, {NULL}};
const Segment SGI294428[] = {{&S76621[16], 4}, {&S76621[28], 2}, {NULL}};
const Segment SGI294429[] = {{&S76655[69], 5}, {NULL}};
const Segment SGI294430[] = {{&S76655[8], 3}, {NULL}};
const Segment SGI294431[] = {{&S76655[13], 9}, {NULL}};
const Segment SGI294432[] = {{&S76655[30], 13}, {NULL}};
const Segment SGI294433[] = {{&S76655[7], 1}, {NULL}};
const Segment SGI294434[] = {{&S76779[72], 4}, {NULL}};
const Segment SGI294435[] = {{&S76779[8], 3}, {NULL}};
const Segment SGI294436[] = {{&S76779[13], 7}, {NULL}};
const Segment SGI294437[] = {{&S76779[25], 14}, {NULL}};
const Segment SGI294438[] = {{&S76779[7], 1}, {NULL}};
const Segment SGI294439[] = {{&S76779[16], 4}, {&S76779[28], 11}, {NULL}};
const Segment SGI294440[] = {{&S76801[82], 4}, {NULL}};
const Segment SGI294441[] = {{&S76801[8], 3}, {NULL}};
const Segment SGI294442[] = {{&S76801[13], 9}, {NULL}};
const Segment SGI294443[] = {{&S76801[33], 13}, {NULL}};
const Segment SGI294444[] = {{&S76801[7], 1}, {NULL}};
const Segment SGI294445[] = {{&S76874[53], 5}, {NULL}};
const Segment SGI294446[] = {{&S76874[8], 3}, {NULL}};
const Segment SGI294447[] = {{&S76874[13], 14}, {NULL}};
const Segment SGI294448[] = {{&S76874[7], 1}, {NULL}};
const Segment SGI294449[] = {{&S76874[16], 11}, {NULL}};
const Segment SGI294450[] = {{&S76874[59], 13}, {NULL}};
const Segment SGI294451[] = {{&S76875[65], 5}, {NULL}};
const Segment SGI294452[] = {{&S76875[8], 3}, {NULL}};
const Segment SGI294453[] = {{&S76875[13], 14}, {NULL}};
const Segment SGI294454[] = {{&S76875[7], 1}, {NULL}};
const Segment SGI294455[] = {{&S76875[16], 11}, {&S76875[32], 2}, {NULL}};
const Segment SGI294456[] = {{&S76875[75], 13}, {NULL}};
const Segment SGI294457[] = {{&S76907[62], 5}, {NULL}};
const Segment SGI294458[] = {{&S76907[8], 3}, {NULL}};
const Segment SGI294459[] = {{&S76907[13], 14}, {NULL}};
const Segment SGI294460[] = {{&S76907[7], 1}, {NULL}};
const Segment SGI294461[] = {{&S76907[16], 11}, {NULL}};
const Segment SGI294462[] = {{&S76907[70], 13}, {NULL}};
const Segment SGI294463[] = {{&S76909[55], 5}, {NULL}};
const Segment SGI294464[] = {{&S76909[8], 3}, {NULL}};
const Segment SGI294465[] = {{&S76909[13], 14}, {NULL}};
const Segment SGI294466[] = {{&S76909[7], 1}, {NULL}};
const Segment SGI294467[] = {{&S76909[16], 11}, {NULL}};
const Segment SGI294468[] = {{&S76909[63], 13}, {NULL}};
const Segment SGI294469[] = {{&S77272[54], 4}, {NULL}};
const Segment SGI294470[] = {{&S77272[8], 3}, {NULL}};
const Segment SGI294471[] = {{&S77272[13], 14}, {NULL}};
const Segment SGI294472[] = {{&S77272[61], 5}, {NULL}};
const Segment SGI294473[] = {{&S77272[7], 1}, {NULL}};
const Segment SGI294474[] = {{&S77272[16], 11}, {NULL}};
const Segment SGI294475[] = {{&S77385[82], 4}, {NULL}};
const Segment SGI294476[] = {{&S77385[8], 3}, {NULL}};
const Segment SGI294477[] = {{&S77385[13], 9}, {NULL}};
const Segment SGI294478[] = {{&S77385[33], 13}, {NULL}};
const Segment SGI294479[] = {{&S77385[7], 1}, {NULL}};
const Segment SGI294480[] = {{&S77385[89], 9}, {NULL}};
const Segment SGI294481[] = {{&S77386[65], 4}, {NULL}};
const Segment SGI294482[] = {{&S77386[8], 3}, {NULL}};
const Segment SGI294483[] = {{&S77386[13], 3}, {NULL}};
const Segment SGI294484[] = {{&S77386[7], 1}, {NULL}};
const Segment SGI294485[] = {{&S77391[69], 5}, {NULL}};
const Segment SGI294486[] = {{&S77391[8], 3}, {NULL}};
const Segment SGI294487[] = {{&S77391[13], 9}, {NULL}};
const Segment SGI294488[] = {{&S77391[30], 13}, {NULL}};
const Segment SGI294489[] = {{&S77391[7], 1}, {NULL}};
const Segment SGI294490[] = {{&S77391[77], 13}, {NULL}};
const Segment SGI294491[] = {{&S61262[0], 97}, {NULL}};
const Segment SGI294492[] = {{&S74531[8], 3}, {NULL}};
const Segment SGI294493[] = {{&S74531[83], 5}, {NULL}};
const Segment SGI294494[] = {{&S74531[76], 6}, {NULL}};
const Segment SGI294495[] = {{&S74531[13], 39}, {NULL}};
const Segment SGI294496[] = {{&S74531[70], 5}, {NULL}};
const Segment SGI294497[] = {{&S77531[58], 4}, {NULL}};
const Segment SGI294498[] = {{&S77531[8], 3}, {NULL}};
const Segment SGI294499[] = {{&S77531[13], 3}, {NULL}};
const Segment SGI294500[] = {{&S77531[13], 3}, {&S77531[18], 7}, {NULL}};
const Segment SGI294501[] = {{&S77531[7], 1}, {NULL}};
const Segment SGI294502[] = {{&S77590[72], 4}, {NULL}};
const Segment SGI294503[] = {{&S77590[8], 3}, {NULL}};
const Segment SGI294504[] = {{&S77590[13], 7}, {NULL}};
const Segment SGI294505[] = {{&S77590[25], 14}, {NULL}};
const Segment SGI294506[] = {{&S77590[7], 1}, {NULL}};
const Segment SGI294507[] = {{&S77590[16], 4}, {&S77590[28], 11}, {NULL}};
const Segment SGI294508[] = {{&S77594[70], 4}, {NULL}};
const Segment SGI294509[] = {{&S77594[8], 3}, {NULL}};
const Segment SGI294510[] = {{&S77594[13], 9}, {NULL}};
const Segment SGI294511[] = {{&S77594[30], 13}, {NULL}};
const Segment SGI294512[] = {{&S77594[7], 1}, {NULL}};
const Segment SGI294513[] = {{&S77633[55], 5}, {NULL}};
const Segment SGI294514[] = {{&S77633[8], 3}, {NULL}};
const Segment SGI294515[] = {{&S77633[13], 14}, {NULL}};
const Segment SGI294516[] = {{&S77633[7], 1}, {NULL}};
const Segment SGI294517[] = {{&S77633[16], 11}, {NULL}};
const Segment SGI294518[] = {{&S77633[63], 13}, {NULL}};
const Segment SGI294519[] = {{&S77634[67], 5}, {NULL}};
const Segment SGI294520[] = {{&S77634[8], 3}, {NULL}};
const Segment SGI294521[] = {{&S77634[13], 14}, {NULL}};
const Segment SGI294522[] = {{&S77634[7], 1}, {NULL}};
const Segment SGI294523[] = {{&S77634[16], 11}, {&S77634[32], 2}, {NULL}};
const Segment SGI294524[] = {{&S77634[79], 13}, {NULL}};
const Segment SGI294525[] = {{&S77768[80], 4}, {NULL}};
const Segment SGI294526[] = {{&S77768[8], 3}, {NULL}};
const Segment SGI294527[] = {{&S77768[13], 9}, {NULL}};
const Segment SGI294528[] = {{&S77768[33], 13}, {NULL}};
const Segment SGI294529[] = {{&S77768[7], 1}, {NULL}};
const Segment SGI294530[] = {{&S77869[76], 4}, {NULL}};
const Segment SGI294531[] = {{&S77869[8], 3}, {NULL}};
const Segment SGI294532[] = {{&S77869[13], 7}, {NULL}};
const Segment SGI294533[] = {{&S77869[25], 14}, {NULL}};
const Segment SGI294534[] = {{&S77869[7], 1}, {NULL}};
const Segment SGI294535[] = {{&S77869[16], 4}, {&S77869[28], 11}, {NULL}};
const Segment SGI294536[] = {{&S77869[86], 9}, {NULL}};
const Segment SGI294537[] = {{&S77961[64], 4}, {NULL}};
const Segment SGI294538[] = {{&S77961[8], 3}, {NULL}};
const Segment SGI294539[] = {{&S77961[13], 3}, {NULL}};
const Segment SGI294540[] = {{&S77961[13], 3}, {&S77961[73], 6}, {NULL}};
const Segment SGI294541[] = {{&S77961[7], 1}, {NULL}};
const Segment SGI294542[] = {{&S77963[60], 5}, {NULL}};
const Segment SGI294543[] = {{&S77963[8], 3}, {NULL}};
const Segment SGI294544[] = {{&S77963[13], 14}, {NULL}};
const Segment SGI294545[] = {{&S77963[7], 1}, {NULL}};
const Segment SGI294546[] = {{&S77963[16], 11}, {NULL}};
const Segment SGI294547[] = {{&S77963[70], 14}, {NULL}};
const Segment SGI294548[] = {{&S78011[62], 4}, {NULL}};
const Segment SGI294549[] = {{&S78011[8], 3}, {NULL}};
const Segment SGI294550[] = {{&S78011[13], 3}, {NULL}};
const Segment SGI294551[] = {{&S78011[7], 1}, {NULL}};
const Segment SGI294552[] = {{&S78011[67], 10}, {NULL}};
const Segment SGI294553[] = {{&S78127[80], 4}, {NULL}};
const Segment SGI294554[] = {{&S78127[8], 3}, {NULL}};
const Segment SGI294555[] = {{&S78127[13], 7}, {NULL}};
const Segment SGI294556[] = {{&S78127[25], 14}, {NULL}};
const Segment SGI294557[] = {{&S78127[7], 1}, {NULL}};
const Segment SGI294558[] = {{&S78127[16], 4}, {&S78127[28], 11}, {NULL}};
const Segment SGI294559[] = {{&S78127[90], 9}, {NULL}};
const Segment SGI294560[] = {{&S78187[68], 4}, {NULL}};
const Segment SGI294561[] = {{&S78187[8], 3}, {NULL}};
const Segment SGI294562[] = {{&S78187[13], 3}, {NULL}};
const Segment SGI294563[] = {{&S78187[7], 1}, {NULL}};
const Segment SGI294564[] = {{&S78187[78], 9}, {NULL}};
const Segment SGI294565[] = {{&S78228[54], 4}, {NULL}};
const Segment SGI294566[] = {{&S78228[8], 3}, {NULL}};
const Segment SGI294567[] = {{&S78228[13], 14}, {NULL}};
const Segment SGI294568[] = {{&S78228[7], 1}, {NULL}};
const Segment SGI294569[] = {{&S78228[16], 11}, {NULL}};
const Segment SGI294570[] = {{&S78228[61], 13}, {NULL}};
const Segment SGI294571[] = {{&S78331[57], 4}, {NULL}};
const Segment SGI294572[] = {{&S78331[8], 3}, {NULL}};
const Segment SGI294573[] = {{&S78331[13], 3}, {NULL}};
const Segment SGI294574[] = {{&S78331[13], 3}, {&S78331[18], 7}, {NULL}};
const Segment SGI294575[] = {{&S78331[7], 1}, {NULL}};
const Segment SGI294576[] = {{&S78344[83], 4}, {NULL}};
const Segment SGI294577[] = {{&S78344[8], 3}, {NULL}};
const Segment SGI294578[] = {{&S78344[13], 9}, {NULL}};
const Segment SGI294579[] = {{&S78344[33], 13}, {NULL}};
const Segment SGI294580[] = {{&S78344[7], 1}, {NULL}};
const Segment SGI294581[] = {{&S78344[91], 9}, {NULL}};
const Segment SGI294582[] = {{&S78379[76], 4}, {NULL}};
const Segment SGI294583[] = {{&S78379[8], 3}, {NULL}};
const Segment SGI294584[] = {{&S78379[13], 7}, {NULL}};
const Segment SGI294585[] = {{&S78379[25], 14}, {NULL}};
const Segment SGI294586[] = {{&S78379[7], 1}, {NULL}};
const Segment SGI294587[] = {{&S78379[16], 4}, {&S78379[28], 11}, {NULL}};
const Segment SGI294588[] = {{&S78379[84], 9}, {NULL}};
const Segment SGI294589[] = {{&S62131[0], 54}, {NULL}};
const Segment SGI294590[] = {{&S62132[0], 51}, {NULL}};
const Segment SGI294591[] = {{&S62133[0], 51}, {NULL}};
const Segment SGI294592[] = {{&S62134[0], 51}, {NULL}};
const Segment SGI294593[] = {{&S62135[0], 54}, {NULL}};
const Segment SGI294594[] = {{&S62136[0], 51}, {NULL}};
const Segment SGI294595[] = {{&S62137[0], 51}, {NULL}};
const Segment SGI294596[] = {{&S62138[0], 51}, {NULL}};
const Segment SGI294597[] = {{&S62139[0], 51}, {NULL}};
const Segment SGI294598[] = {{&S62140[0], 51}, {NULL}};
const Segment SGI294599[] = {{&S62141[0], 51}, {NULL}};
const Segment SGI294600[] = {{&S62142[0], 51}, {NULL}};
const Segment SGI294601[] = {{&S62143[0], 51}, {NULL}};
const Segment SGI294602[] = {{&S62144[0], 52}, {NULL}};
const Segment SGI294603[] = {{&S62145[0], 80}, {NULL}};
const Segment SGI294604[] = {{&S78528[8], 3}, {NULL}};
const Segment SGI294605[] = {{&S78528[13], 7}, {NULL}};
const Segment SGI294606[] = {{&S78528[25], 14}, {NULL}};
const Segment SGI294607[] = {{&S78528[11], 1}, {NULL}};
const Segment SGI294608[] = {{&S78528[16], 4}, {&S78528[28], 11}, {NULL}};
const Segment SGI294609[] = {{&S78529[8], 3}, {NULL}};
const Segment SGI294610[] = {{&S78529[13], 7}, {NULL}};
const Segment SGI294611[] = {{&S78529[25], 14}, {NULL}};
const Segment SGI294612[] = {{&S78529[11], 1}, {&S78529[11], 1}, {NULL}};
const Segment SGI294613[] = {{&S78529[16], 4}, {&S78529[28], 11}, {NULL}};
const Segment SGI294614[] = {{&S78578[64], 4}, {NULL}};
const Segment SGI294615[] = {{&S78578[8], 3}, {NULL}};
const Segment SGI294616[] = {{&S78578[13], 7}, {NULL}};
const Segment SGI294617[] = {{&S78578[7], 1}, {NULL}};
const Segment SGI294618[] = {{&S78578[16], 4}, {&S78578[28], 2}, {NULL}};
const Segment SGI294619[] = {{&S78620[72], 4}, {NULL}};
const Segment SGI294620[] = {{&S78620[8], 3}, {NULL}};
const Segment SGI294621[] = {{&S78620[13], 7}, {NULL}};
const Segment SGI294622[] = {{&S78620[25], 14}, {NULL}};
const Segment SGI294623[] = {{&S78620[7], 1}, {NULL}};
const Segment SGI294624[] = {{&S78620[16], 4}, {&S78620[28], 11}, {NULL}};
const Segment SGI294625[] = {{&S78620[79], 9}, {NULL}};
const Segment SGI294626[] = {{&S62475[0], 55}, {NULL}};
const Segment SGI294627[] = {{&S62476[18], 23}, {NULL}};
const Segment SGI294628[] = {{&S62476[0], 120}, {NULL}};
const Segment SGI294629[] = {{&S62477[63], 16}, {NULL}};
const Segment SGI294630[] = {{&S62477[0], 79}, {NULL}};
const Segment SGI294631[] = {{&S62478[0], 55}, {NULL}};
const Segment SGI294632[] = {{&S62479[0], 75}, {NULL}};
const Segment SGI294633[] = {{&S62480[0], 75}, {NULL}};
const Segment SGI294634[] = {{&S62481[18], 23}, {NULL}};
const Segment SGI294635[] = {{&S62481[0], 121}, {NULL}};
const Segment SGI294636[] = {{&S62482[0], 63}, {NULL}};
const Segment SGI294637[] = {{&S62483[0], 63}, {NULL}};
const Segment SGI294638[] = {{&S62484[0], 55}, {NULL}};
const Segment SGI294639[] = {{&S62485[0], 81}, {NULL}};
const Segment SGI294640[] = {{&S62486[0], 55}, {NULL}};
const Segment SGI294641[] = {{&S62487[0], 81}, {NULL}};
const Segment SGI294642[] = {{&S62488[0], 81}, {NULL}};
const Segment SGI294643[] = {{&S62489[0], 55}, {NULL}};
const Segment SGI294644[] = {{&S62490[0], 55}, {NULL}};
const Segment SGI294645[] = {{&S62491[0], 81}, {NULL}};
const Segment SGI294646[] = {{&S78693[53], 5}, {NULL}};
const Segment SGI294647[] = {{&S78693[8], 3}, {NULL}};
const Segment SGI294648[] = {{&S78693[13], 14}, {NULL}};
const Segment SGI294649[] = {{&S62492[0], 55}, {NULL}};
const Segment SGI294650[] = {{&S78693[7], 1}, {NULL}};
const Segment SGI294651[] = {{&S78693[16], 11}, {NULL}};
const Segment SGI294652[] = {{&S78693[59], 9}, {NULL}};
const Segment SGI294653[] = {{&S78694[60], 5}, {NULL}};
const Segment SGI294654[] = {{&S78694[8], 3}, {NULL}};
const Segment SGI294655[] = {{&S78694[13], 14}, {NULL}};
const Segment SGI294656[] = {{&S78694[7], 1}, {NULL}};
const Segment SGI294657[] = {{&S78694[16], 11}, {NULL}};
const Segment SGI294658[] = {{&S78694[66], 9}, {NULL}};
const Segment SGI294659[] = {{&S78808[60], 5}, {NULL}};
const Segment SGI294660[] = {{&S78808[8], 3}, {NULL}};
const Segment SGI294661[] = {{&S78808[13], 14}, {NULL}};
const Segment SGI294662[] = {{&S78808[7], 1}, {NULL}};
const Segment SGI294663[] = {{&S78808[16], 11}, {NULL}};
const Segment SGI294664[] = {{&S78808[66], 13}, {NULL}};
const Segment SGI294665[] = {{&S78809[60], 5}, {NULL}};
const Segment SGI294666[] = {{&S78809[8], 3}, {NULL}};
const Segment SGI294667[] = {{&S78809[13], 14}, {NULL}};
const Segment SGI294668[] = {{&S78809[7], 1}, {NULL}};
const Segment SGI294669[] = {{&S78809[16], 11}, {NULL}};
const Segment SGI294670[] = {{&S78809[66], 13}, {NULL}};
const Segment SGI294671[] = {{&S78818[75], 4}, {NULL}};
const Segment SGI294672[] = {{&S78818[8], 3}, {NULL}};
const Segment SGI294673[] = {{&S78818[13], 7}, {NULL}};
const Segment SGI294674[] = {{&S78818[25], 14}, {NULL}};
const Segment SGI294675[] = {{&S78818[7], 1}, {NULL}};
const Segment SGI294676[] = {{&S78818[16], 4}, {&S78818[28], 11}, {NULL}};
const Segment SGI294677[] = {{&S78818[84], 9}, {NULL}};
const Segment SGI294678[] = {{&S79146[68], 4}, {NULL}};
const Segment SGI294679[] = {{&S79146[8], 3}, {NULL}};
const Segment SGI294680[] = {{&S79146[13], 3}, {NULL}};
const Segment SGI294681[] = {{&S79146[7], 1}, {NULL}};
const Segment SGI294682[] = {{&S79190[73], 4}, {NULL}};
const Segment SGI294683[] = {{&S79190[8], 3}, {NULL}};
const Segment SGI294684[] = {{&S79190[13], 7}, {NULL}};
const Segment SGI294685[] = {{&S79190[25], 14}, {NULL}};
const Segment SGI294686[] = {{&S79190[7], 1}, {NULL}};
const Segment SGI294687[] = {{&S79190[16], 4}, {&S79190[28], 11}, {NULL}};
const Segment SGI294688[] = {{&S79190[78], 13}, {NULL}};
const Segment SGI294689[] = {{&S79201[76], 4}, {NULL}};
const Segment SGI294690[] = {{&S79201[8], 3}, {NULL}};
const Segment SGI294691[] = {{&S79201[13], 7}, {NULL}};
const Segment SGI294692[] = {{&S79201[25], 14}, {NULL}};
const Segment SGI294693[] = {{&S79201[7], 1}, {NULL}};
const Segment SGI294694[] = {{&S79201[16], 4}, {&S79201[28], 11}, {NULL}};
const Segment SGI294695[] = {{&S79201[86], 9}, {NULL}};
const Segment SGI294696[] = {{&S79270[57], 5}, {NULL}};
const Segment SGI294697[] = {{&S79270[8], 3}, {NULL}};
const Segment SGI294698[] = {{&S79270[13], 3}, {NULL}};
const Segment SGI294699[] = {{&S79270[13], 3}, {&S79270[18], 7}, {NULL}};
const Segment SGI294700[] = {{&S79270[7], 1}, {NULL}};
const Segment SGI294701[] = {{&S79275[54], 4}, {NULL}};
const Segment SGI294702[] = {{&S79275[8], 3}, {NULL}};
const Segment SGI294703[] = {{&S79275[13], 14}, {NULL}};
const Segment SGI294704[] = {{&S79275[7], 1}, {NULL}};
const Segment SGI294705[] = {{&S79275[16], 11}, {NULL}};
const Segment SGI294706[] = {{&S79281[76], 4}, {NULL}};
const Segment SGI294707[] = {{&S79281[8], 3}, {NULL}};
const Segment SGI294708[] = {{&S79281[13], 7}, {NULL}};
const Segment SGI294709[] = {{&S79281[25], 14}, {NULL}};
const Segment SGI294710[] = {{&S79281[7], 1}, {NULL}};
const Segment SGI294711[] = {{&S79281[16], 4}, {&S79281[28], 11}, {NULL}};
const Segment SGI294712[] = {{&S79281[85], 12}, {NULL}};
const Segment SGI294713[] = {{&S79312[65], 4}, {NULL}};
const Segment SGI294714[] = {{&S79312[8], 3}, {NULL}};
const Segment SGI294715[] = {{&S79312[13], 3}, {NULL}};
const Segment SGI294716[] = {{&S79312[13], 3}, {&S79312[21], 7}, {NULL}};
const Segment SGI294717[] = {{&S79312[7], 1}, {NULL}};
const Segment SGI294718[] = {{&S79367[75], 4}, {NULL}};
const Segment SGI294719[] = {{&S79367[8], 3}, {NULL}};
const Segment SGI294720[] = {{&S79367[13], 7}, {NULL}};
const Segment SGI294721[] = {{&S79367[25], 14}, {NULL}};
const Segment SGI294722[] = {{&S79367[7], 1}, {NULL}};
const Segment SGI294723[] = {{&S79367[16], 4}, {&S79367[28], 11}, {NULL}};
const Segment SGI294724[] = {{&S79367[82], 13}, {NULL}};
const Segment SGI294725[] = {{&S79417[75], 4}, {NULL}};
const Segment SGI294726[] = {{&S79417[8], 3}, {NULL}};
const Segment SGI294727[] = {{&S79417[13], 7}, {NULL}};
const Segment SGI294728[] = {{&S79417[25], 14}, {NULL}};
const Segment SGI294729[] = {{&S79417[7], 1}, {NULL}};
const Segment SGI294730[] = {{&S79417[16], 4}, {&S79417[28], 11}, {NULL}};
const Segment SGI294731[] = {{&S79417[82], 8}, {NULL}};
const Segment SGI294732[] = {{&S79596[76], 4}, {NULL}};
const Segment SGI294733[] = {{&S79596[8], 3}, {NULL}};
const Segment SGI294734[] = {{&S79596[13], 7}, {NULL}};
const Segment SGI294735[] = {{&S79596[25], 14}, {NULL}};
const Segment SGI294736[] = {{&S79596[7], 1}, {NULL}};
const Segment SGI294737[] = {{&S79596[16], 4}, {&S79596[28], 11}, {NULL}};
const Segment SGI294738[] = {{&S79596[84], 9}, {NULL}};
const Segment SGI294739[] = {{&S79621[75], 4}, {NULL}};
const Segment SGI294740[] = {{&S79621[8], 3}, {NULL}};
const Segment SGI294741[] = {{&S79621[13], 7}, {NULL}};
const Segment SGI294742[] = {{&S79621[25], 14}, {NULL}};
const Segment SGI294743[] = {{&S79621[7], 1}, {NULL}};
const Segment SGI294744[] = {{&S79621[16], 4}, {&S79621[28], 11}, {NULL}};
const Segment SGI294745[] = {{&S79621[82], 8}, {NULL}};
const Segment SGI294746[] = {{&S79622[66], 4}, {NULL}};
const Segment SGI294747[] = {{&S79622[8], 3}, {NULL}};
const Segment SGI294748[] = {{&S79622[13], 9}, {NULL}};
const Segment SGI294749[] = {{&S79622[28], 13}, {NULL}};
const Segment SGI294750[] = {{&S79622[7], 1}, {NULL}};
const Segment SGI294751[] = {{&S79628[55], 4}, {NULL}};
const Segment SGI294752[] = {{&S79628[8], 3}, {NULL}};
const Segment SGI294753[] = {{&S79628[13], 3}, {NULL}};
const Segment SGI294754[] = {{&S79628[13], 3}, {&S79628[18], 7}, {NULL}};
const Segment SGI294755[] = {{&S79628[7], 1}, {NULL}};
const Segment SGI294756[] = {{&S79884[53], 5}, {NULL}};
const Segment SGI294757[] = {{&S79884[8], 3}, {NULL}};
const Segment SGI294758[] = {{&S79884[13], 14}, {NULL}};
const Segment SGI294759[] = {{&S79884[7], 1}, {NULL}};
const Segment SGI294760[] = {{&S79884[16], 11}, {NULL}};
const Segment SGI294761[] = {{&S79884[61], 13}, {NULL}};
const Segment SGI294762[] = {{&S79885[65], 5}, {NULL}};
const Segment SGI294763[] = {{&S79885[8], 3}, {NULL}};
const Segment SGI294764[] = {{&S79885[13], 14}, {NULL}};
const Segment SGI294765[] = {{&S79885[7], 1}, {NULL}};
const Segment SGI294766[] = {{&S79885[16], 11}, {&S79885[32], 2}, {NULL}};
const Segment SGI294767[] = {{&S79885[77], 13}, {NULL}};
const Segment SGI294768[] = {{&S79958[60], 5}, {NULL}};
const Segment SGI294769[] = {{&S79958[8], 3}, {NULL}};
const Segment SGI294770[] = {{&S79958[13], 14}, {NULL}};
const Segment SGI294771[] = {{&S79958[7], 1}, {NULL}};
const Segment SGI294772[] = {{&S79958[16], 11}, {NULL}};
const Segment SGI294773[] = {{&S79958[66], 13}, {NULL}};
const Segment SGI294774[] = {{&S76550[8], 3}, {NULL}};
const Segment SGI294775[] = {{&S76550[33], 4}, {NULL}};
const Segment SGI294776[] = {{&S76550[63], 6}, {NULL}};
const Segment SGI294777[] = {{&S76550[13], 24}, {NULL}};
const Segment SGI294778[] = {{&S76550[33], 2}, {NULL}};
const Segment SGI294779[] = {{&S79996[53], 5}, {NULL}};
const Segment SGI294780[] = {{&S79996[8], 3}, {NULL}};
const Segment SGI294781[] = {{&S79996[13], 14}, {NULL}};
const Segment SGI294782[] = {{&S79996[7], 1}, {NULL}};
const Segment SGI294783[] = {{&S79996[16], 11}, {NULL}};
const Segment SGI294784[] = {{&S79996[59], 13}, {NULL}};
const Segment SGI294785[] = {{&S79997[65], 5}, {NULL}};
const Segment SGI294786[] = {{&S79997[8], 3}, {NULL}};
const Segment SGI294787[] = {{&S79997[13], 14}, {NULL}};
const Segment SGI294788[] = {{&S79997[7], 1}, {NULL}};
const Segment SGI294789[] = {{&S79997[16], 11}, {&S79997[32], 2}, {NULL}};
const Segment SGI294790[] = {{&S79997[75], 13}, {NULL}};
const Segment SGI294791[] = {{&S80051[94], 4}, {NULL}};
const Segment SGI294792[] = {{&S80051[8], 3}, {NULL}};
const Segment SGI294793[] = {{&S80051[13], 7}, {NULL}};
const Segment SGI294794[] = {{&S80051[25], 14}, {NULL}};
const Segment SGI294795[] = {{&S80051[7], 1}, {NULL}};
const Segment SGI294796[] = {{&S80051[16], 4}, {&S80051[28], 11}, {NULL}};
const Segment SGI294797[] = {{&S80051[101], 8}, {NULL}};
const Segment SGI294798[] = {{&S80066[55], 5}, {NULL}};
const Segment SGI294799[] = {{&S80066[8], 3}, {NULL}};
const Segment SGI294800[] = {{&S80066[13], 14}, {NULL}};
const Segment SGI294801[] = {{&S80066[7], 1}, {NULL}};
const Segment SGI294802[] = {{&S80066[16], 11}, {NULL}};
const Segment SGI294803[] = {{&S80066[63], 9}, {NULL}};
const Segment SGI294804[] = {{&S80136[94], 4}, {NULL}};
const Segment SGI294805[] = {{&S80136[8], 3}, {NULL}};
const Segment SGI294806[] = {{&S80136[13], 7}, {NULL}};
const Segment SGI294807[] = {{&S80136[25], 14}, {NULL}};
const Segment SGI294808[] = {{&S80136[7], 1}, {NULL}};
const Segment SGI294809[] = {{&S80136[16], 4}, {&S80136[28], 11}, {NULL}};
const Segment SGI294810[] = {{&S80136[101], 8}, {NULL}};
const Segment SGI294811[] = {{&S80140[7], 4}, {NULL}};
const Segment SGI294812[] = {{&S80140[8], 3}, {NULL}};
const Segment SGI294813[] = {{&S80140[13], 9}, {NULL}};
const Segment SGI294814[] = {{&S80140[30], 13}, {NULL}};
const Segment SGI294815[] = {{&S80140[7], 1}, {NULL}};
const Segment SGI294816[] = {{&S80147[53], 5}, {NULL}};
const Segment SGI294817[] = {{&S80147[8], 3}, {NULL}};
const Segment SGI294818[] = {{&S80147[13], 14}, {NULL}};
const Segment SGI294819[] = {{&S80147[7], 1}, {NULL}};
const Segment SGI294820[] = {{&S80147[16], 11}, {NULL}};
const Segment SGI294821[] = {{&S80147[61], 13}, {NULL}};
const Segment SGI294822[] = {{&S80148[65], 5}, {NULL}};
const Segment SGI294823[] = {{&S80148[8], 3}, {NULL}};
const Segment SGI294824[] = {{&S80148[13], 14}, {NULL}};
const Segment SGI294825[] = {{&S80148[7], 1}, {NULL}};
const Segment SGI294826[] = {{&S80148[16], 11}, {&S80148[32], 2}, {NULL}};
const Segment SGI294827[] = {{&S80148[77], 13}, {NULL}};
const Segment SGI294828[] = {{&S63841[0], 53}, {NULL}};
const Segment SGI294829[] = {{&S63842[0], 53}, {NULL}};
const Segment SGI294830[] = {{&S63843[0], 53}, {NULL}};
const Segment SGI294831[] = {{&S63844[0], 56}, {NULL}};
const Segment SGI294832[] = {{&S63845[0], 53}, {NULL}};
const Segment SGI294833[] = {{&S63846[0], 56}, {NULL}};
const Segment SGI294834[] = {{&S63847[0], 53}, {NULL}};
const Segment SGI294835[] = {{&S63848[0], 53}, {NULL}};
const Segment SGI294836[] = {{&S63849[0], 53}, {NULL}};
const Segment SGI294837[] = {{&S63850[0], 53}, {NULL}};
const Segment SGI294838[] = {{&S80205[65], 4}, {NULL}};
const Segment SGI294839[] = {{&S80205[8], 3}, {NULL}};
const Segment SGI294840[] = {{&S80205[13], 3}, {NULL}};
const Segment SGI294841[] = {{&S80205[7], 1}, {NULL}};
const Segment SGI294842[] = {{&S80205[72], 13}, {NULL}};
const Segment SGI294843[] = {{&S80218[69], 5}, {NULL}};
const Segment SGI294844[] = {{&S80218[8], 3}, {NULL}};
const Segment SGI294845[] = {{&S80218[13], 9}, {NULL}};
const Segment SGI294846[] = {{&S80218[30], 13}, {NULL}};
const Segment SGI294847[] = {{&S80218[7], 1}, {NULL}};
const Segment SGI294848[] = {{&S80218[77], 13}, {NULL}};
const Segment SGI294849[] = {{&S63894[0], 76}, {NULL}};
const Segment SGI294850[] = {{&S63895[80], 14}, {NULL}};
const Segment SGI294851[] = {{&S63895[0], 114}, {NULL}};
const Segment SGI294852[] = {{&S63896[0], 80}, {NULL}};
const Segment SGI294853[] = {{&S63897[80], 14}, {NULL}};
const Segment SGI294854[] = {{&S63897[0], 114}, {NULL}};
const Segment SGI294855[] = {{&S63898[0], 107}, {NULL}};
const Segment SGI294856[] = {{&S63899[0], 112}, {NULL}};
const Segment SGI294857[] = {{&S63900[0], 139}, {NULL}};
const Segment SGI294858[] = {{&S63901[0], 112}, {NULL}};
const Segment SGI294859[] = {{&S54745[0], 9}, {NULL}};
const Segment SGI294860[] = {{&S54746[0], 9}, {NULL}};
const Segment SGI294861[] = {{&S54747[0], 9}, {NULL}};
const Segment SGI294862[] = {{&S54748[0], 9}, {NULL}};
const Segment SGI294863[] = {{&S54749[0], 9}, {NULL}};
const Segment SGI294864[] = {{&S54750[0], 9}, {NULL}};
const Segment SGI294865[] = {{&S54751[0], 9}, {NULL}};
const Segment SGI294866[] = {{&S54752[0], 9}, {NULL}};
const Segment SGI294867[] = {{&S54753[0], 9}, {NULL}};
const Segment SGI294868[] = {{&S54754[0], 9}, {NULL}};
const Segment SGI294869[] = {{&S54755[0], 9}, {NULL}};
const Segment SGI294870[] = {{&S54756[0], 9}, {NULL}};
const Segment SGI294871[] = {{&S54757[0], 9}, {NULL}};
const Segment SGI294872[] = {{&S54758[0], 9}, {NULL}};
const Segment SGI294873[] = {{&S54759[0], 9}, {NULL}};
const Segment SGI294874[] = {{&S54760[0], 9}, {NULL}};
const Segment SGI294875[] = {{&S54761[0], 9}, {NULL}};
const Segment SGI294876[] = {{&S54762[0], 9}, {NULL}};
const Segment SGI294877[] = {{&S54763[0], 9}, {NULL}};
const Segment SGI294878[] = {{&S54764[0], 9}, {NULL}};
const Segment SGI294879[] = {{&S54765[0], 9}, {NULL}};
const Segment SGI294880[] = {{&S54766[0], 9}, {NULL}};
const Segment SGI294881[] = {{&S54767[0], 9}, {NULL}};
const Segment SGI294882[] = {{&S54768[0], 9}, {NULL}};
const Segment SGI294883[] = {{&S54769[0], 9}, {NULL}};
const Segment SGI294884[] = {{&S54770[0], 9}, {NULL}};
const Segment SGI294885[] = {{&S54771[0], 9}, {NULL}};
const Segment SGI294886[] = {{&S80277[52], 4}, {NULL}};
const Segment SGI294887[] = {{&S80277[8], 3}, {NULL}};
const Segment SGI294888[] = {{&S80277[13], 14}, {NULL}};
const Segment SGI294889[] = {{&S80277[7], 1}, {NULL}};
const Segment SGI294890[] = {{&S80277[16], 11}, {NULL}};
const Segment SGI294891[] = {{&S80277[57], 12}, {NULL}};
const Segment SGI294892[] = {{&S80406[54], 4}, {NULL}};
const Segment SGI294893[] = {{&S80406[8], 3}, {NULL}};
const Segment SGI294894[] = {{&S80406[13], 14}, {NULL}};
const Segment SGI294895[] = {{&S80406[7], 1}, {NULL}};
const Segment SGI294896[] = {{&S80406[16], 11}, {NULL}};
const Segment SGI294897[] = {{&S80406[61], 13}, {NULL}};
const Segment SGI294898[] = {{&S80739[70], 4}, {NULL}};
const Segment SGI294899[] = {{&S80739[8], 3}, {NULL}};
const Segment SGI294900[] = {{&S80739[13], 3}, {NULL}};
const Segment SGI294901[] = {{&S80739[13], 3}, {&S80739[21], 7}, {NULL}};
const Segment SGI294902[] = {{&S80739[7], 1}, {NULL}};
const Segment SGI294903[] = {{&S80753[56], 4}, {NULL}};
const Segment SGI294904[] = {{&S80753[8], 3}, {NULL}};
const Segment SGI294905[] = {{&S80753[13], 14}, {NULL}};
const Segment SGI294906[] = {{&S80753[7], 1}, {NULL}};
const Segment SGI294907[] = {{&S80753[16], 11}, {NULL}};
const Segment SGI294908[] = {{&S80753[61], 11}, {NULL}};
const Segment SGI294909[] = {{&S80786[92], 5}, {NULL}};
const Segment SGI294910[] = {{&S80786[8], 3}, {NULL}};
const Segment SGI294911[] = {{&S80786[13], 14}, {NULL}};
const Segment SGI294912[] = {{&S80786[13], 14}, {&S80786[13], 14}, {NULL}};
const Segment SGI294913[] = {{&S80786[7], 1}, {NULL}};
const Segment SGI294914[] = {{&S80786[16], 11}, {&S80786[16], 11}, {NULL}};
const Segment SGI294915[] = {{&S80787[76], 5}, {NULL}};
const Segment SGI294916[] = {{&S80787[8], 3}, {NULL}};
const Segment SGI294917[] = {{&S80787[13], 14}, {NULL}};
const Segment SGI294918[] = {{&S80787[7], 1}, {NULL}};
const Segment SGI294919[] = {{&S80787[16], 11}, {NULL}};
const Segment SGI294920[] = {{&S80823[66], 4}, {NULL}};
const Segment SGI294921[] = {{&S80823[8], 3}, {NULL}};
const Segment SGI294922[] = {{&S80823[13], 9}, {NULL}};
const Segment SGI294923[] = {{&S80823[28], 13}, {NULL}};
const Segment SGI294924[] = {{&S80823[7], 1}, {NULL}};
const Segment SGI294925[] = {{&S80824[91], 4}, {NULL}};
const Segment SGI294926[] = {{&S80824[8], 3}, {NULL}};
const Segment SGI294927[] = {{&S80824[13], 3}, {NULL}};
const Segment SGI294928[] = {{&S80824[60], 12}, {NULL}};
const Segment SGI294929[] = {{&S80824[7], 1}, {NULL}};
const Segment SGI294930[] = {{&S80920[56], 5}, {NULL}};
const Segment SGI294931[] = {{&S80920[8], 3}, {NULL}};
const Segment SGI294932[] = {{&S80920[13], 3}, {NULL}};
const Segment SGI294933[] = {{&S80920[7], 1}, {NULL}};
const Segment SGI294934[] = {{&S80920[62], 13}, {NULL}};
const Segment SGI294935[] = {{&S80990[8], 3}, {NULL}};
const Segment SGI294936[] = {{&S80990[13], 7}, {NULL}};
const Segment SGI294937[] = {{&S80990[25], 14}, {NULL}};
const Segment SGI294938[] = {{&S80990[76], 4}, {NULL}};
const Segment SGI294939[] = {{&S80990[16], 4}, {&S80990[28], 11}, {NULL}};
const Segment SGI294940[] = {{&S80991[8], 3}, {NULL}};
const Segment SGI294941[] = {{&S80991[13], 7}, {NULL}};
const Segment SGI294942[] = {{&S80991[25], 14}, {NULL}};
const Segment SGI294943[] = {{&S80991[100], 5}, {NULL}};
const Segment SGI294944[] = {{&S80991[16], 4}, {&S80991[28], 11}, {NULL}};
const Segment SGI294945[] = {{&S80992[8], 3}, {NULL}};
const Segment SGI294946[] = {{&S80992[13], 7}, {NULL}};
const Segment SGI294947[] = {{&S80992[25], 14}, {NULL}};
const Segment SGI294948[] = {{&S80992[16], 4}, {&S80992[28], 11}, {NULL}};
const Segment SGI294949[] = {{&S64703[0], 50}, {NULL}};
const Segment SGI294950[] = {{&S64704[0], 56}, {NULL}};
const Segment SGI294951[] = {{&S64705[0], 51}, {NULL}};
const Segment SGI294952[] = {{&S64706[0], 57}, {NULL}};
const Segment SGI294953[] = {{&S64707[0], 56}, {NULL}};
const Segment SGI294954[] = {{&S64708[0], 48}, {NULL}};
const Segment SGI294955[] = {{&S64709[0], 59}, {NULL}};
const Segment SGI294956[] = {{&S64710[0], 54}, {NULL}};
const Segment SGI294957[] = {{&S64711[0], 57}, {NULL}};
const Segment SGI294958[] = {{&S64712[0], 54}, {NULL}};
const Segment SGI294959[] = {{&S64713[0], 57}, {NULL}};
const Segment SGI294960[] = {{&S64714[0], 58}, {NULL}};
const Segment SGI294961[] = {{&S64715[0], 48}, {NULL}};
const Segment SGI294962[] = {{&S64716[0], 52}, {NULL}};
const Segment SGI294963[] = {{&S64717[0], 57}, {NULL}};
const Segment SGI294964[] = {{&S64718[0], 59}, {NULL}};
const Segment SGI294965[] = {{&S64719[0], 60}, {NULL}};
const Segment SGI294966[] = {{&S64720[0], 53}, {NULL}};
const Segment SGI294967[] = {{&S64721[0], 54}, {NULL}};
const Segment SGI294968[] = {{&S64722[0], 46}, {NULL}};
const Segment SGI294969[] = {{&S64723[0], 51}, {NULL}};
const Segment SGI294970[] = {{&S64724[0], 54}, {NULL}};
const Segment SGI294971[] = {{&S64725[0], 54}, {NULL}};
const Segment SGI294972[] = {{&S64726[0], 45}, {NULL}};
const Segment SGI294973[] = {{&S64727[0], 53}, {NULL}};
const Segment SGI294974[] = {{&S64728[0], 50}, {NULL}};
const Segment SGI294975[] = {{&S64729[0], 64}, {NULL}};
const Segment SGI294976[] = {{&S64730[0], 58}, {NULL}};
const Segment SGI294977[] = {{&S64731[0], 51}, {NULL}};
const Segment SGI294978[] = {{&S81126[73], 4}, {NULL}};
const Segment SGI294979[] = {{&S81126[8], 3}, {NULL}};
const Segment SGI294980[] = {{&S81126[13], 7}, {NULL}};
const Segment SGI294981[] = {{&S81126[25], 14}, {NULL}};
const Segment SGI294982[] = {{&S81126[7], 1}, {NULL}};
const Segment SGI294983[] = {{&S81126[16], 4}, {&S81126[28], 11}, {NULL}};
const Segment SGI294984[] = {{&S81127[32], 4}, {NULL}};
const Segment SGI294985[] = {{&S81127[8], 3}, {NULL}};
const Segment SGI294986[] = {{&S81127[13], 10}, {NULL}};
const Segment SGI294987[] = {{&S81127[7], 1}, {NULL}};
const Segment SGI294988[] = {{&S81127[16], 22}, {NULL}};
const Segment SGI294989[] = {{&S81133[60], 5}, {NULL}};
const Segment SGI294990[] = {{&S81133[8], 3}, {NULL}};
const Segment SGI294991[] = {{&S81133[13], 14}, {NULL}};
const Segment SGI294992[] = {{&S81133[7], 1}, {NULL}};
const Segment SGI294993[] = {{&S81133[16], 11}, {NULL}};
const Segment SGI294994[] = {{&S81133[66], 13}, {NULL}};
const Segment SGI294995[] = {{&S81134[65], 5}, {NULL}};
const Segment SGI294996[] = {{&S81134[8], 3}, {NULL}};
const Segment SGI294997[] = {{&S81134[13], 14}, {NULL}};
const Segment SGI294998[] = {{&S81134[7], 1}, {NULL}};
const Segment SGI294999[] = {{&S81134[16], 11}, {&S81134[32], 2}, {NULL}};
const Segment SGI295000[] = {{&S81134[75], 13}, {NULL}};
const Segment SGI295001[] = {{&S81554[53], 5}, {NULL}};
const Segment SGI295002[] = {{&S81554[8], 3}, {NULL}};
const Segment SGI295003[] = {{&S81554[13], 14}, {NULL}};
const Segment SGI295004[] = {{&S81554[7], 1}, {NULL}};
const Segment SGI295005[] = {{&S81554[16], 11}, {NULL}};
const Segment SGI295006[] = {{&S81554[59], 13}, {NULL}};
const Segment SGI295007[] = {{&S81608[53], 5}, {NULL}};
const Segment SGI295008[] = {{&S81608[8], 3}, {NULL}};
const Segment SGI295009[] = {{&S81608[13], 14}, {NULL}};
const Segment SGI295010[] = {{&S81608[7], 1}, {NULL}};
const Segment SGI295011[] = {{&S81608[16], 11}, {NULL}};
const Segment SGI295012[] = {{&S81608[59], 14}, {NULL}};
const Segment SGI295013[] = {{&S81617[99], 4}, {NULL}};
const Segment SGI295014[] = {{&S81617[8], 3}, {NULL}};
const Segment SGI295015[] = {{&S81617[13], 9}, {NULL}};
const Segment SGI295016[] = {{&S81617[33], 13}, {NULL}};
const Segment SGI295017[] = {{&S81617[7], 1}, {NULL}};
const Segment SGI295018[] = {{&S81617[73], 10}, {NULL}};
const Segment SGI295019[] = {{&S81674[87], 4}, {NULL}};
const Segment SGI295020[] = {{&S81674[8], 3}, {NULL}};
const Segment SGI295021[] = {{&S81674[13], 3}, {NULL}};
const Segment SGI295022[] = {{&S81674[13], 3}, {&S81674[60], 6}, {NULL}};
const Segment SGI295023[] = {{&S81674[60], 11}, {NULL}};
const Segment SGI295024[] = {{&S81674[7], 1}, {NULL}};
const Segment SGI295025[] = {{&S81675[77], 4}, {NULL}};
const Segment SGI295026[] = {{&S81675[8], 3}, {NULL}};
const Segment SGI295027[] = {{&S81675[13], 3}, {NULL}};
const Segment SGI295028[] = {{&S81675[13], 3}, {&S81675[21], 7}, {NULL}};
const Segment SGI295029[] = {{&S81675[7], 1}, {NULL}};
const Segment SGI295030[] = {{&S81800[7], 4}, {NULL}};
const Segment SGI295031[] = {{&S81800[8], 3}, {NULL}};
const Segment SGI295032[] = {{&S81800[13], 3}, {NULL}};
const Segment SGI295033[] = {{&S81800[7], 1}, {NULL}};
const Segment SGI295034[] = {{&S81800[68], 13}, {NULL}};
const Segment SGI295035[] = {{&S78489[8], 3}, {NULL}};
const Segment SGI295036[] = {{&S78489[33], 4}, {NULL}};
const Segment SGI295037[] = {{&S78489[63], 6}, {NULL}};
const Segment SGI295038[] = {{&S78489[13], 24}, {NULL}};
const Segment SGI295039[] = {{&S78489[33], 2}, {NULL}};
const Segment SGI295040[] = {{&S81958[69], 5}, {NULL}};
const Segment SGI295041[] = {{&S81958[8], 3}, {NULL}};
const Segment SGI295042[] = {{&S81958[13], 9}, {NULL}};
const Segment SGI295043[] = {{&S81958[30], 13}, {NULL}};
const Segment SGI295044[] = {{&S81958[7], 1}, {NULL}};
const Segment SGI295045[] = {{&S81958[77], 13}, {NULL}};
const Segment SGI295046[] = {{&S82046[75], 4}, {NULL}};
const Segment SGI295047[] = {{&S82046[8], 3}, {NULL}};
const Segment SGI295048[] = {{&S82046[13], 7}, {NULL}};
const Segment SGI295049[] = {{&S82046[25], 14}, {NULL}};
const Segment SGI295050[] = {{&S82046[7], 1}, {NULL}};
const Segment SGI295051[] = {{&S82046[16], 4}, {&S82046[28], 11}, {NULL}};
const Segment SGI295052[] = {{&S82046[84], 9}, {NULL}};
const Segment SGI295053[] = {{&S82103[73], 4}, {NULL}};
const Segment SGI295054[] = {{&S82103[8], 3}, {NULL}};
const Segment SGI295055[] = {{&S82103[13], 7}, {NULL}};
const Segment SGI295056[] = {{&S82103[25], 14}, {NULL}};
const Segment SGI295057[] = {{&S82103[7], 1}, {NULL}};
const Segment SGI295058[] = {{&S82103[16], 4}, {&S82103[28], 11}, {NULL}};
const Segment SGI295059[] = {{&S82105[52], 4}, {NULL}};
const Segment SGI295060[] = {{&S82105[8], 3}, {NULL}};
const Segment SGI295061[] = {{&S82105[13], 14}, {NULL}};
const Segment SGI295062[] = {{&S82105[7], 1}, {NULL}};
const Segment SGI295063[] = {{&S82105[16], 11}, {NULL}};
const Segment SGI295064[] = {{&S82105[57], 13}, {NULL}};
const Segment SGI295065[] = {{&S65653[0], 71}, {NULL}};
const Segment SGI295066[] = {{&S82200[69], 5}, {NULL}};
const Segment SGI295067[] = {{&S82200[8], 3}, {NULL}};
const Segment SGI295068[] = {{&S82200[13], 9}, {NULL}};
const Segment SGI295069[] = {{&S82200[30], 13}, {NULL}};
const Segment SGI295070[] = {{&S82200[7], 1}, {NULL}};
const Segment SGI295071[] = {{&S82200[75], 13}, {NULL}};
const Segment SGI295072[] = {{&S82272[72], 4}, {NULL}};
const Segment SGI295073[] = {{&S82272[8], 3}, {NULL}};
const Segment SGI295074[] = {{&S82272[13], 7}, {NULL}};
const Segment SGI295075[] = {{&S82272[25], 14}, {NULL}};
const Segment SGI295076[] = {{&S82272[7], 1}, {NULL}};
const Segment SGI295077[] = {{&S82272[16], 4}, {&S82272[28], 11}, {NULL}};
const Segment SGI295078[] = {{&S82272[79], 13}, {NULL}};
const Segment SGI295079[] = {{&S78937[8], 3}, {NULL}};
const Segment SGI295080[] = {{&S78937[33], 4}, {NULL}};
const Segment SGI295081[] = {{&S78937[63], 6}, {NULL}};
const Segment SGI295082[] = {{&S78937[13], 24}, {NULL}};
const Segment SGI295083[] = {{&S78937[33], 2}, {NULL}};
const Segment SGI295084[] = {{&S65856[0], 44}, {NULL}};
const Segment SGI295085[] = {{&S65857[0], 52}, {NULL}};
const Segment SGI295086[] = {{&S65858[69], 5}, {NULL}};
const Segment SGI295087[] = {{&S65858[0], 82}, {NULL}};
const Segment SGI295088[] = {{&S65859[0], 64}, {NULL}};
const Segment SGI295089[] = {{&S65860[0], 44}, {NULL}};
const Segment SGI295090[] = {{&S65861[0], 59}, {NULL}};
const Segment SGI295091[] = {{&S65862[0], 44}, {NULL}};
const Segment SGI295092[] = {{&S65863[0], 44}, {NULL}};
const Segment SGI295093[] = {{&S65864[0], 44}, {NULL}};
const Segment SGI295094[] = {{&S65865[0], 47}, {NULL}};
const Segment SGI295095[] = {{&S65866[0], 47}, {NULL}};
const Segment SGI295096[] = {{&S65867[0], 44}, {NULL}};
const Segment SGI295097[] = {{&S65868[0], 44}, {NULL}};
const Segment SGI295098[] = {{&S65869[0], 44}, {NULL}};
const Segment SGI295099[] = {{&S65870[0], 44}, {NULL}};
const Segment SGI295100[] = {{&S65871[0], 44}, {NULL}};
const Segment SGI295101[] = {{&S65872[0], 47}, {NULL}};
const Segment SGI295102[] = {{&S65873[0], 59}, {NULL}};
const Segment SGI295103[] = {{&S65891[0], 67}, {NULL}};
const Segment SGI295104[] = {{&S65892[0], 67}, {NULL}};
const Segment SGI295105[] = {{&S65893[0], 67}, {NULL}};
const Segment SGI295106[] = {{&S65894[0], 70}, {NULL}};
const Segment SGI295107[] = {{&S65895[0], 64}, {NULL}};
const Segment SGI295108[] = {{&S65896[0], 70}, {NULL}};
const Segment SGI295109[] = {{&S65897[0], 67}, {NULL}};
const Segment SGI295110[] = {{&S65898[0], 64}, {NULL}};
const Segment SGI295111[] = {{&S65899[0], 70}, {NULL}};
const Segment SGI295112[] = {{&S65900[0], 64}, {NULL}};
const Segment SGI295113[] = {{&S65901[0], 70}, {NULL}};
const Segment SGI295114[] = {{&S82278[52], 4}, {NULL}};
const Segment SGI295115[] = {{&S82278[8], 3}, {NULL}};
const Segment SGI295116[] = {{&S82278[13], 14}, {NULL}};
const Segment SGI295117[] = {{&S82278[7], 1}, {NULL}};
const Segment SGI295118[] = {{&S82278[16], 11}, {NULL}};
const Segment SGI295119[] = {{&S82278[57], 13}, {NULL}};
const Segment SGI295120[] = {{&S82282[52], 4}, {NULL}};
const Segment SGI295121[] = {{&S82282[8], 3}, {NULL}};
const Segment SGI295122[] = {{&S82282[13], 14}, {NULL}};
const Segment SGI295123[] = {{&S82282[7], 1}, {NULL}};
const Segment SGI295124[] = {{&S82282[16], 11}, {NULL}};
const Segment SGI295125[] = {{&S82282[57], 12}, {NULL}};
const Segment SGI295126[] = {{&S82283[65], 4}, {NULL}};
const Segment SGI295127[] = {{&S82283[8], 3}, {NULL}};
const Segment SGI295128[] = {{&S82283[13], 3}, {NULL}};
const Segment SGI295129[] = {{&S82283[7], 1}, {NULL}};
const Segment SGI295130[] = {{&S82333[69], 5}, {NULL}};
const Segment SGI295131[] = {{&S82333[8], 3}, {NULL}};
const Segment SGI295132[] = {{&S82333[13], 9}, {NULL}};
const Segment SGI295133[] = {{&S82333[30], 13}, {NULL}};
const Segment SGI295134[] = {{&S82333[7], 1}, {NULL}};
const Segment SGI295135[] = {{&S82392[60], 5}, {NULL}};
const Segment SGI295136[] = {{&S82392[8], 3}, {NULL}};
const Segment SGI295137[] = {{&S82392[13], 7}, {NULL}};
const Segment SGI295138[] = {{&S82392[7], 1}, {NULL}};
const Segment SGI295139[] = {{&S82392[16], 4}, {&S82392[28], 5}, {NULL}};
const Segment SGI295140[] = {{&S82413[57], 5}, {NULL}};
const Segment SGI295141[] = {{&S82413[8], 3}, {NULL}};
const Segment SGI295142[] = {{&S82413[13], 3}, {NULL}};
const Segment SGI295143[] = {{&S82413[13], 3}, {&S82413[18], 7}, {NULL}};
const Segment SGI295144[] = {{&S82413[7], 1}, {NULL}};
const Segment SGI295145[] = {{&S82457[54], 4}, {NULL}};
const Segment SGI295146[] = {{&S82457[8], 3}, {NULL}};
const Segment SGI295147[] = {{&S82457[13], 14}, {NULL}};
const Segment SGI295148[] = {{&S82457[7], 1}, {NULL}};
const Segment SGI295149[] = {{&S82457[16], 11}, {NULL}};
const Segment SGI295150[] = {{&S82457[61], 13}, {NULL}};
const Segment SGI295151[] = {{&S82459[53], 5}, {NULL}};
const Segment SGI295152[] = {{&S82459[8], 3}, {NULL}};
const Segment SGI295153[] = {{&S82459[13], 14}, {NULL}};
const Segment SGI295154[] = {{&S82459[7], 1}, {NULL}};
const Segment SGI295155[] = {{&S82459[16], 11}, {NULL}};
const Segment SGI295156[] = {{&S82566[63], 4}, {NULL}};
const Segment SGI295157[] = {{&S82566[8], 3}, {NULL}};
const Segment SGI295158[] = {{&S82566[13], 3}, {NULL}};
const Segment SGI295159[] = {{&S82566[13], 3}, {&S82566[71], 6}, {NULL}};
const Segment SGI295160[] = {{&S82566[7], 1}, {NULL}};
const Segment SGI295161[] = {{&S66386[0], 54}, {NULL}};
const Segment SGI295162[] = {{&S66387[0], 54}, {NULL}};
const Segment SGI295163[] = {{&S66388[0], 54}, {NULL}};
const Segment SGI295164[] = {{&S66389[0], 54}, {NULL}};
const Segment SGI295165[] = {{&S66390[0], 54}, {NULL}};
const Segment SGI295166[] = {{&S66391[0], 54}, {NULL}};
const Segment SGI295167[] = {{&S66392[0], 54}, {NULL}};
const Segment SGI295168[] = {{&S66393[0], 57}, {NULL}};
const Segment SGI295169[] = {{&S66394[0], 54}, {NULL}};
const Segment SGI295170[] = {{&S66395[0], 54}, {NULL}};
const Segment SGI295171[] = {{&S66396[0], 54}, {NULL}};
const Segment SGI295172[] = {{&S82677[79], 4}, {NULL}};
const Segment SGI295173[] = {{&S82677[8], 3}, {NULL}};
const Segment SGI295174[] = {{&S82677[13], 7}, {NULL}};
const Segment SGI295175[] = {{&S82677[25], 14}, {NULL}};
const Segment SGI295176[] = {{&S82677[7], 1}, {NULL}};
const Segment SGI295177[] = {{&S82677[16], 4}, {&S82677[28], 11}, {NULL}};
const Segment SGI295178[] = {{&S82677[87], 9}, {NULL}};
const Segment SGI295179[] = {{&S82678[54], 4}, {NULL}};
const Segment SGI295180[] = {{&S82678[8], 3}, {NULL}};
const Segment SGI295181[] = {{&S82678[13], 14}, {NULL}};
const Segment SGI295182[] = {{&S82678[7], 1}, {NULL}};
const Segment SGI295183[] = {{&S82678[16], 11}, {NULL}};
const Segment SGI295184[] = {{&S82678[61], 13}, {NULL}};
const Segment SGI295185[] = {{&S82742[75], 4}, {NULL}};
const Segment SGI295186[] = {{&S82742[8], 3}, {NULL}};
const Segment SGI295187[] = {{&S82742[13], 7}, {NULL}};
const Segment SGI295188[] = {{&S82742[25], 14}, {NULL}};
const Segment SGI295189[] = {{&S82742[7], 1}, {NULL}};
const Segment SGI295190[] = {{&S82742[16], 4}, {&S82742[28], 11}, {NULL}};
const Segment SGI295191[] = {{&S82742[82], 13}, {NULL}};
const Segment SGI295192[] = {{&S82753[54], 4}, {NULL}};
const Segment SGI295193[] = {{&S82753[8], 3}, {NULL}};
const Segment SGI295194[] = {{&S82753[13], 14}, {NULL}};
const Segment SGI295195[] = {{&S82753[7], 1}, {NULL}};
const Segment SGI295196[] = {{&S82753[16], 11}, {NULL}};
const Segment SGI295197[] = {{&S66539[0], 96}, {NULL}};
const Segment SGI295198[] = {{&S66540[0], 71}, {NULL}};
const Segment SGI295199[] = {{&S66541[0], 73}, {NULL}};
const Segment SGI295200[] = {{&S66542[0], 96}, {NULL}};
const Segment SGI295201[] = {{&S66543[0], 96}, {NULL}};
const Segment SGI295202[] = {{&S66544[0], 79}, {NULL}};
const Segment SGI295203[] = {{&S66545[4], 7}, {NULL}};
const Segment SGI295204[] = {{&S66545[0], 97}, {NULL}};
const Segment SGI295205[] = {{&S66546[80], 5}, {NULL}};
const Segment SGI295206[] = {{&S66546[0], 105}, {NULL}};
const Segment SGI295207[] = {{&S66547[80], 5}, {NULL}};
const Segment SGI295208[] = {{&S66547[0], 105}, {NULL}};
const Segment SGI295209[] = {{&S82831[80], 4}, {NULL}};
const Segment SGI295210[] = {{&S82831[8], 3}, {NULL}};
const Segment SGI295211[] = {{&S82831[13], 3}, {NULL}};
const Segment SGI295212[] = {{&S82831[13], 3}, {&S82831[33], 6}, {NULL}};
const Segment SGI295213[] = {{&S82831[7], 1}, {NULL}};
const Segment SGI295214[] = {{&S82900[70], 4}, {NULL}};
const Segment SGI295215[] = {{&S82900[8], 3}, {NULL}};
const Segment SGI295216[] = {{&S82900[13], 9}, {NULL}};
const Segment SGI295217[] = {{&S82900[30], 13}, {NULL}};
const Segment SGI295218[] = {{&S82900[7], 1}, {NULL}};
const Segment SGI295219[] = {{&S82905[22], 3}, {NULL}};
const Segment SGI295220[] = {{&S82905[27], 9}, {NULL}};
const Segment SGI295221[] = {{&S82905[21], 1}, {NULL}};
const Segment SGI295222[] = {{&S82986[60], 5}, {NULL}};
const Segment SGI295223[] = {{&S82986[8], 3}, {NULL}};
const Segment SGI295224[] = {{&S82986[13], 14}, {NULL}};
const Segment SGI295225[] = {{&S82986[7], 1}, {NULL}};
const Segment SGI295226[] = {{&S82986[16], 11}, {NULL}};
const Segment SGI295227[] = {{&S82986[68], 13}, {NULL}};
const Segment SGI295228[] = {{&S83141[89], 4}, {NULL}};
const Segment SGI295229[] = {{&S83141[8], 3}, {NULL}};
const Segment SGI295230[] = {{&S83141[13], 3}, {NULL}};
const Segment SGI295231[] = {{&S83141[13], 3}, {&S83141[61], 6}, {NULL}};
const Segment SGI295232[] = {{&S83141[61], 11}, {NULL}};
const Segment SGI295233[] = {{&S83141[7], 1}, {NULL}};
const Segment SGI295234[] = {{&S83142[67], 4}, {NULL}};
const Segment SGI295235[] = {{&S83142[8], 3}, {NULL}};
const Segment SGI295236[] = {{&S83142[13], 3}, {NULL}};
const Segment SGI295237[] = {{&S83142[7], 1}, {NULL}};
const Segment SGI295238[] = {{&S83142[76], 9}, {NULL}};
const Segment SGI295239[] = {{&S67078[0], 97}, {NULL}};
const Segment SGI295240[] = {{&S67079[0], 97}, {NULL}};
const Segment SGI295241[] = {{&S67080[73], 5}, {NULL}};
const Segment SGI295242[] = {{&S67080[0], 86}, {NULL}};
const Segment SGI295243[] = {{&S67081[0], 95}, {NULL}};
const Segment SGI295244[] = {{&S67082[0], 97}, {NULL}};
const Segment SGI295245[] = {{&S67083[4], 5}, {NULL}};
const Segment SGI295246[] = {{&S67083[0], 93}, {NULL}};
const Segment SGI295247[] = {{&S67084[0], 99}, {NULL}};
const Segment SGI295248[] = {{&S67085[0], 85}, {NULL}};
const Segment SGI295249[] = {{&S67086[0], 97}, {NULL}};
const Segment SGI295250[] = {{&S67087[0], 73}, {NULL}};
const Segment SGI295251[] = {{&S67088[0], 99}, {NULL}};
const Segment SGI295252[] = {{&S67089[0], 97}, {NULL}};
const Segment SGI295253[] = {{&S67090[0], 97}, {NULL}};
const Segment SGI295254[] = {{&S67091[0], 71}, {NULL}};
const Segment SGI295255[] = {{&S67092[0], 99}, {NULL}};
const Segment SGI295256[] = {{&S67093[0], 99}, {NULL}};
const Segment SGI295257[] = {{&S67094[0], 100}, {NULL}};
const Segment SGI295258[] = {{&S67095[0], 97}, {NULL}};
const Segment SGI295259[] = {{&S83392[79], 4}, {NULL}};
const Segment SGI295260[] = {{&S83392[8], 3}, {NULL}};
const Segment SGI295261[] = {{&S83392[13], 7}, {NULL}};
const Segment SGI295262[] = {{&S83392[25], 14}, {NULL}};
const Segment SGI295263[] = {{&S83392[7], 1}, {NULL}};
const Segment SGI295264[] = {{&S83392[16], 4}, {&S83392[28], 11}, {NULL}};
const Segment SGI295265[] = {{&S83392[87], 9}, {NULL}};
const Segment SGI295266[] = {{&S83395[60], 5}, {NULL}};
const Segment SGI295267[] = {{&S83395[8], 3}, {NULL}};
const Segment SGI295268[] = {{&S83395[13], 14}, {NULL}};
const Segment SGI295269[] = {{&S83395[7], 1}, {NULL}};
const Segment SGI295270[] = {{&S83395[16], 11}, {NULL}};
const Segment SGI295271[] = {{&S83395[66], 13}, {NULL}};
const Segment SGI295272[] = {{&S83403[7], 4}, {NULL}};
const Segment SGI295273[] = {{&S83403[8], 3}, {NULL}};
const Segment SGI295274[] = {{&S83403[13], 9}, {NULL}};
const Segment SGI295275[] = {{&S83403[30], 13}, {NULL}};
const Segment SGI295276[] = {{&S83403[7], 1}, {NULL}};
const Segment SGI295277[] = {{&S83495[52], 4}, {NULL}};
const Segment SGI295278[] = {{&S83495[8], 3}, {NULL}};
const Segment SGI295279[] = {{&S83495[13], 14}, {NULL}};
const Segment SGI295280[] = {{&S83495[7], 1}, {NULL}};
const Segment SGI295281[] = {{&S83495[16], 11}, {NULL}};
const Segment SGI295282[] = {{&S83521[75], 4}, {NULL}};
const Segment SGI295283[] = {{&S83521[8], 3}, {NULL}};
const Segment SGI295284[] = {{&S83521[13], 7}, {NULL}};
const Segment SGI295285[] = {{&S83521[25], 14}, {NULL}};
const Segment SGI295286[] = {{&S83521[7], 1}, {NULL}};
const Segment SGI295287[] = {{&S83521[16], 4}, {&S83521[28], 11}, {NULL}};
const Segment SGI295288[] = {{&S83521[80], 8}, {NULL}};
const Segment SGI295289[] = {{&S67513[0], 59}, {NULL}};
const Segment SGI295290[] = {{&S67514[18], 27}, {NULL}};
const Segment SGI295291[] = {{&S67514[0], 124}, {NULL}};
const Segment SGI295292[] = {{&S67515[0], 62}, {NULL}};
const Segment SGI295293[] = {{&S67516[0], 62}, {NULL}};
const Segment SGI295294[] = {{&S67517[0], 59}, {NULL}};
const Segment SGI295295[] = {{&S67518[0], 66}, {NULL}};
const Segment SGI295296[] = {{&S67519[0], 59}, {NULL}};
const Segment SGI295297[] = {{&S67520[0], 59}, {NULL}};
const Segment SGI295298[] = {{&S67521[0], 59}, {NULL}};
const Segment SGI295299[] = {{&S67522[0], 62}, {NULL}};
const Segment SGI295300[] = {{&S67523[0], 66}, {NULL}};
const Segment SGI295301[] = {{&S67524[0], 59}, {NULL}};
const Segment SGI295302[] = {{&S67525[0], 59}, {NULL}};
const Segment SGI295303[] = {{&S67526[0], 59}, {NULL}};
const Segment SGI295304[] = {{&S67527[0], 88}, {NULL}};
const Segment SGI295305[] = {{&S67528[0], 59}, {NULL}};
const Segment SGI295306[] = {{&S83843[75], 4}, {NULL}};
const Segment SGI295307[] = {{&S83843[8], 3}, {NULL}};
const Segment SGI295308[] = {{&S83843[13], 7}, {NULL}};
const Segment SGI295309[] = {{&S83843[25], 14}, {NULL}};
const Segment SGI295310[] = {{&S83843[7], 1}, {NULL}};
const Segment SGI295311[] = {{&S83843[16], 4}, {&S83843[28], 11}, {NULL}};
const Segment SGI295312[] = {{&S83843[82], 8}, {NULL}};
const Segment SGI295313[] = {{&S83844[75], 4}, {NULL}};
const Segment SGI295314[] = {{&S83844[8], 3}, {NULL}};
const Segment SGI295315[] = {{&S83844[13], 7}, {NULL}};
const Segment SGI295316[] = {{&S83844[25], 14}, {NULL}};
const Segment SGI295317[] = {{&S83844[7], 1}, {NULL}};
const Segment SGI295318[] = {{&S83844[16], 4}, {&S83844[28], 11}, {NULL}};
const Segment SGI295319[] = {{&S83844[82], 8}, {NULL}};
const Segment SGI295320[] = {{&S83863[59], 5}, {NULL}};
const Segment SGI295321[] = {{&S83863[8], 3}, {NULL}};
const Segment SGI295322[] = {{&S83863[13], 13}, {NULL}};
const Segment SGI295323[] = {{&S83863[7], 1}, {NULL}};
const Segment SGI295324[] = {{&S67696[73], 11}, {NULL}};
const Segment SGI295325[] = {{&S67696[0], 84}, {NULL}};
const Segment SGI295326[] = {{&S67697[55], 12}, {NULL}};
const Segment SGI295327[] = {{&S67697[0], 87}, {NULL}};
const Segment SGI295328[] = {{&S67698[55], 4}, {NULL}};
const Segment SGI295329[] = {{&S67698[0], 79}, {NULL}};
const Segment SGI295330[] = {{&S67699[55], 13}, {NULL}};
const Segment SGI295331[] = {{&S67699[0], 88}, {NULL}};
const Segment SGI295332[] = {{&S67700[55], 10}, {NULL}};
const Segment SGI295333[] = {{&S67700[0], 85}, {NULL}};
const Segment SGI295334[] = {{&S67701[55], 12}, {NULL}};
const Segment SGI295335[] = {{&S67701[0], 91}, {NULL}};
const Segment SGI295336[] = {{&S67746[33], 16}, {NULL}};
const Segment SGI295337[] = {{&S67746[0], 82}, {NULL}};
const Segment SGI295338[] = {{&S67747[33], 16}, {NULL}};
const Segment SGI295339[] = {{&S67747[0], 82}, {NULL}};
const Segment SGI295340[] = {{&S67748[33], 5}, {NULL}};
const Segment SGI295341[] = {{&S67748[0], 71}, {NULL}};
const Segment SGI295342[] = {{&S67749[33], 16}, {NULL}};
const Segment SGI295343[] = {{&S67749[0], 82}, {NULL}};
const Segment SGI295344[] = {{&S67750[33], 16}, {NULL}};
const Segment SGI295345[] = {{&S67750[0], 81}, {NULL}};
const Segment SGI295346[] = {{&S67751[33], 16}, {NULL}};
const Segment SGI295347[] = {{&S67751[0], 81}, {NULL}};
const Segment SGI295348[] = {{&S67752[33], 16}, {NULL}};
const Segment SGI295349[] = {{&S67752[0], 81}, {NULL}};
const Segment SGI295350[] = {{&S67753[33], 16}, {NULL}};
const Segment SGI295351[] = {{&S67753[0], 81}, {NULL}};
const Segment SGI295352[] = {{&S67754[33], 16}, {NULL}};
const Segment SGI295353[] = {{&S67754[0], 82}, {NULL}};
const Segment SGI295354[] = {{&S67755[33], 16}, {NULL}};
const Segment SGI295355[] = {{&S67755[0], 81}, {NULL}};
const Segment SGI295356[] = {{&S67756[33], 5}, {NULL}};
const Segment SGI295357[] = {{&S67756[0], 71}, {NULL}};
const Segment SGI295358[] = {{&S67757[33], 16}, {NULL}};
const Segment SGI295359[] = {{&S67757[0], 81}, {NULL}};
const Segment SGI295360[] = {{&S67758[33], 16}, {NULL}};
const Segment SGI295361[] = {{&S67758[0], 81}, {NULL}};
const Segment SGI295362[] = {{&S83999[53], 5}, {NULL}};
const Segment SGI295363[] = {{&S83999[8], 3}, {NULL}};
const Segment SGI295364[] = {{&S83999[13], 14}, {NULL}};
const Segment SGI295365[] = {{&S83999[7], 1}, {NULL}};
const Segment SGI295366[] = {{&S83999[16], 11}, {NULL}};
const Segment SGI295367[] = {{&S83999[63], 14}, {NULL}};
const Segment SGI295368[] = {{&S84040[67], 5}, {NULL}};
const Segment SGI295369[] = {{&S84040[8], 3}, {NULL}};
const Segment SGI295370[] = {{&S84040[13], 9}, {NULL}};
const Segment SGI295371[] = {{&S84040[28], 13}, {NULL}};
const Segment SGI295372[] = {{&S84040[7], 1}, {NULL}};
const Segment SGI295373[] = {{&S84088[69], 5}, {NULL}};
const Segment SGI295374[] = {{&S84088[8], 3}, {NULL}};
const Segment SGI295375[] = {{&S84088[13], 9}, {NULL}};
const Segment SGI295376[] = {{&S84088[30], 13}, {NULL}};
const Segment SGI295377[] = {{&S84088[7], 1}, {NULL}};
const Segment SGI295378[] = {{&S84088[75], 13}, {NULL}};
const Segment SGI295379[] = {{&S84091[64], 4}, {NULL}};
const Segment SGI295380[] = {{&S84091[8], 3}, {NULL}};
const Segment SGI295381[] = {{&S84091[13], 14}, {NULL}};
const Segment SGI295382[] = {{&S84091[7], 1}, {NULL}};
const Segment SGI295383[] = {{&S84091[16], 11}, {&S84091[32], 2}, {NULL}};
const Segment SGI295384[] = {{&S84091[73], 12}, {NULL}};
const Segment SGI295385[] = {{&S84186[69], 5}, {NULL}};
const Segment SGI295386[] = {{&S84186[8], 3}, {NULL}};
const Segment SGI295387[] = {{&S84186[13], 9}, {NULL}};
const Segment SGI295388[] = {{&S84186[30], 13}, {NULL}};
const Segment SGI295389[] = {{&S84186[7], 1}, {NULL}};
const Segment SGI295390[] = {{&S84186[75], 13}, {NULL}};
const Segment SGI295391[] = {{&S84187[69], 5}, {NULL}};
const Segment SGI295392[] = {{&S84187[8], 3}, {NULL}};
const Segment SGI295393[] = {{&S84187[13], 9}, {NULL}};
const Segment SGI295394[] = {{&S84187[30], 13}, {NULL}};
const Segment SGI295395[] = {{&S84187[7], 1}, {NULL}};
const Segment SGI295396[] = {{&S84187[77], 13}, {NULL}};
const Segment SGI295397[] = {{&S67860[0], 46}, {NULL}};
const Segment SGI295398[] = {{&S67861[0], 43}, {NULL}};
const Segment SGI295399[] = {{&S84393[76], 4}, {NULL}};
const Segment SGI295400[] = {{&S84393[8], 3}, {NULL}};
const Segment SGI295401[] = {{&S84393[13], 7}, {NULL}};
const Segment SGI295402[] = {{&S84393[25], 14}, {NULL}};
const Segment SGI295403[] = {{&S84393[7], 1}, {NULL}};
const Segment SGI295404[] = {{&S84393[16], 4}, {&S84393[28], 11}, {NULL}};
const Segment SGI295405[] = {{&S84393[86], 9}, {NULL}};
const Segment SGI295406[] = {{&S84408[57], 5}, {NULL}};
const Segment SGI295407[] = {{&S84408[8], 3}, {NULL}};
const Segment SGI295408[] = {{&S84408[13], 9}, {NULL}};
const Segment SGI295409[] = {{&S84408[28], 13}, {NULL}};
const Segment SGI295410[] = {{&S84408[7], 1}, {NULL}};
const Segment SGI295411[] = {{&S84451[57], 5}, {NULL}};
const Segment SGI295412[] = {{&S84451[8], 3}, {NULL}};
const Segment SGI295413[] = {{&S84451[13], 3}, {NULL}};
const Segment SGI295414[] = {{&S84451[13], 3}, {&S84451[18], 7}, {NULL}};
const Segment SGI295415[] = {{&S84451[7], 1}, {NULL}};
const Segment SGI295416[] = {{&S84500[53], 5}, {NULL}};
const Segment SGI295417[] = {{&S84500[8], 3}, {NULL}};
const Segment SGI295418[] = {{&S84500[13], 14}, {NULL}};
const Segment SGI295419[] = {{&S84500[7], 1}, {NULL}};
const Segment SGI295420[] = {{&S84500[16], 11}, {NULL}};
const Segment SGI295421[] = {{&S84500[63], 14}, {NULL}};
const Segment SGI295422[] = {{&S84501[75], 4}, {NULL}};
const Segment SGI295423[] = {{&S84501[8], 3}, {NULL}};
const Segment SGI295424[] = {{&S84501[13], 7}, {NULL}};
const Segment SGI295425[] = {{&S84501[25], 14}, {NULL}};
const Segment SGI295426[] = {{&S84501[7], 1}, {NULL}};
const Segment SGI295427[] = {{&S84501[16], 4}, {&S84501[28], 11}, {NULL}};
const Segment SGI295428[] = {{&S84501[84], 13}, {NULL}};
const Segment SGI295429[] = {{&S84505[57], 5}, {NULL}};
const Segment SGI295430[] = {{&S84505[8], 3}, {NULL}};
const Segment SGI295431[] = {{&S84505[13], 3}, {NULL}};
const Segment SGI295432[] = {{&S84505[13], 3}, {&S84505[18], 7}, {NULL}};
const Segment SGI295433[] = {{&S84505[7], 1}, {NULL}};
const Segment SGI295434[] = {{&S84598[65], 5}, {NULL}};
const Segment SGI295435[] = {{&S84598[8], 3}, {NULL}};
const Segment SGI295436[] = {{&S84598[13], 14}, {NULL}};
const Segment SGI295437[] = {{&S84598[7], 1}, {NULL}};
const Segment SGI295438[] = {{&S84598[16], 11}, {&S84598[32], 2}, {NULL}};
const Segment SGI295439[] = {{&S84598[75], 13}, {NULL}};
const Segment SGI295440[] = {{&S84599[60], 5}, {NULL}};
const Segment SGI295441[] = {{&S84599[8], 3}, {NULL}};
const Segment SGI295442[] = {{&S84599[13], 14}, {NULL}};
const Segment SGI295443[] = {{&S84599[7], 1}, {NULL}};
const Segment SGI295444[] = {{&S84599[16], 11}, {NULL}};
const Segment SGI295445[] = {{&S84599[66], 13}, {NULL}};
const Segment SGI295446[] = {{&S84673[56], 4}, {NULL}};
const Segment SGI295447[] = {{&S84673[8], 3}, {NULL}};
const Segment SGI295448[] = {{&S84673[13], 3}, {NULL}};
const Segment SGI295449[] = {{&S84673[13], 3}, {&S84673[18], 5}, {NULL}};
const Segment SGI295450[] = {{&S84673[7], 1}, {NULL}};
const Segment SGI295451[] = {{&S84753[83], 4}, {NULL}};
const Segment SGI295452[] = {{&S84753[8], 3}, {NULL}};
const Segment SGI295453[] = {{&S84753[13], 9}, {NULL}};
const Segment SGI295454[] = {{&S84753[33], 13}, {NULL}};
const Segment SGI295455[] = {{&S84753[7], 1}, {NULL}};
const Segment SGI295456[] = {{&S68490[0], 71}, {NULL}};
const Segment SGI295457[] = {{&S68590[55], 16}, {NULL}};
const Segment SGI295458[] = {{&S68590[0], 91}, {NULL}};
const Segment SGI295459[] = {{&S68591[55], 16}, {NULL}};
const Segment SGI295460[] = {{&S68591[0], 95}, {NULL}};
const Segment SGI295461[] = {{&S68592[0], 111}, {NULL}};
const Segment SGI295462[] = {{&S68593[55], 16}, {NULL}};
const Segment SGI295463[] = {{&S68593[0], 102}, {NULL}};
const Segment SGI295464[] = {{&S84858[53], 4}, {NULL}};
const Segment SGI295465[] = {{&S84858[8], 3}, {NULL}};
const Segment SGI295466[] = {{&S84858[13], 13}, {NULL}};
const Segment SGI295467[] = {{&S84858[7], 1}, {NULL}};
const Segment SGI295468[] = {{&S84858[16], 10}, {NULL}};
const Segment SGI295469[] = {{&S84863[78], 4}, {NULL}};
const Segment SGI295470[] = {{&S84863[8], 3}, {NULL}};
const Segment SGI295471[] = {{&S84863[13], 7}, {NULL}};
const Segment SGI295472[] = {{&S84863[25], 14}, {NULL}};
const Segment SGI295473[] = {{&S84863[7], 1}, {NULL}};
const Segment SGI295474[] = {{&S84863[16], 4}, {&S84863[28], 11}, {NULL}};
const Segment SGI295475[] = {{&S84863[87], 13}, {NULL}};
const Segment SGI295476[] = {{&S84877[75], 4}, {NULL}};
const Segment SGI295477[] = {{&S84877[8], 3}, {NULL}};
const Segment SGI295478[] = {{&S84877[13], 7}, {NULL}};
const Segment SGI295479[] = {{&S84877[25], 14}, {NULL}};
const Segment SGI295480[] = {{&S84877[7], 1}, {NULL}};
const Segment SGI295481[] = {{&S84877[16], 4}, {&S84877[28], 11}, {NULL}};
const Segment SGI295482[] = {{&S84877[84], 9}, {NULL}};
const Segment SGI295483[] = {{&S84926[69], 5}, {NULL}};
const Segment SGI295484[] = {{&S84926[8], 3}, {NULL}};
const Segment SGI295485[] = {{&S84926[13], 9}, {NULL}};
const Segment SGI295486[] = {{&S84926[30], 13}, {NULL}};
const Segment SGI295487[] = {{&S84926[7], 1}, {NULL}};
const Segment SGI295488[] = {{&S84951[61], 4}, {NULL}};
const Segment SGI295489[] = {{&S84951[8], 3}, {NULL}};
const Segment SGI295490[] = {{&S84951[13], 14}, {NULL}};
const Segment SGI295491[] = {{&S84951[7], 1}, {NULL}};
const Segment SGI295492[] = {{&S84951[16], 11}, {NULL}};
const Segment SGI295493[] = {{&S84951[68], 8}, {NULL}};
const Segment SGI295494[] = {{&S68955[0], 95}, {NULL}};
const Segment SGI295495[] = {{&S68956[79], 5}, {NULL}};
const Segment SGI295496[] = {{&S68956[0], 104}, {NULL}};
const Segment SGI295497[] = {{&S68957[0], 72}, {NULL}};
const Segment SGI295498[] = {{&S68958[79], 5}, {NULL}};
const Segment SGI295499[] = {{&S68958[0], 104}, {NULL}};
const Segment SGI295500[] = {{&S68959[79], 5}, {NULL}};
const Segment SGI295501[] = {{&S68959[0], 104}, {NULL}};
const Segment SGI295502[] = {{&S68960[0], 95}, {NULL}};
const Segment SGI295503[] = {{&S68961[0], 95}, {NULL}};
const Segment SGI295504[] = {{&S68962[0], 121}, {NULL}};
const Segment SGI295505[] = {{&S68963[0], 98}, {NULL}};
const Segment SGI295506[] = {{&S68964[0], 70}, {NULL}};
const Segment SGI295507[] = {{&S68965[0], 95}, {NULL}};
const Segment SGI295508[] = {{&S82039[8], 3}, {NULL}};
const Segment SGI295509[] = {{&S82039[37], 4}, {NULL}};
const Segment SGI295510[] = {{&S82039[64], 6}, {NULL}};
const Segment SGI295511[] = {{&S82039[13], 28}, {NULL}};
const Segment SGI295512[] = {{&S85251[67], 4}, {NULL}};
const Segment SGI295513[] = {{&S85251[8], 3}, {NULL}};
const Segment SGI295514[] = {{&S85251[13], 7}, {NULL}};
const Segment SGI295515[] = {{&S85251[7], 1}, {NULL}};
const Segment SGI295516[] = {{&S85251[16], 4}, {&S85251[28], 2}, {NULL}};
const Segment SGI295517[] = {{&S85251[77], 9}, {NULL}};
const Segment SGI295518[] = {{&S85252[73], 4}, {NULL}};
const Segment SGI295519[] = {{&S85252[8], 3}, {NULL}};
const Segment SGI295520[] = {{&S85252[13], 7}, {NULL}};
const Segment SGI295521[] = {{&S85252[7], 1}, {NULL}};
const Segment SGI295522[] = {{&S85252[16], 4}, {&S85252[28], 8}, {NULL}};
const Segment SGI295523[] = {{&S85252[83], 9}, {NULL}};
const Segment SGI295524[] = {{&S85347[73], 4}, {NULL}};
const Segment SGI295525[] = {{&S85347[8], 3}, {NULL}};
const Segment SGI295526[] = {{&S85347[13], 7}, {NULL}};
const Segment SGI295527[] = {{&S85347[25], 14}, {NULL}};
const Segment SGI295528[] = {{&S85347[7], 1}, {NULL}};
const Segment SGI295529[] = {{&S85347[16], 4}, {&S85347[28], 11}, {NULL}};
const Segment SGI295530[] = {{&S85386[67], 5}, {NULL}};
const Segment SGI295531[] = {{&S85386[8], 3}, {NULL}};
const Segment SGI295532[] = {{&S85386[13], 9}, {NULL}};
const Segment SGI295533[] = {{&S85386[28], 13}, {NULL}};
const Segment SGI295534[] = {{&S85386[7], 1}, {NULL}};
const Segment SGI295535[] = {{&S85466[75], 4}, {NULL}};
const Segment SGI295536[] = {{&S85466[8], 3}, {NULL}};
const Segment SGI295537[] = {{&S85466[13], 7}, {NULL}};
const Segment SGI295538[] = {{&S85466[25], 14}, {NULL}};
const Segment SGI295539[] = {{&S85466[7], 1}, {NULL}};
const Segment SGI295540[] = {{&S85466[16], 4}, {&S85466[28], 11}, {NULL}};
const Segment SGI295541[] = {{&S85466[80], 8}, {NULL}};
const Segment SGI295542[] = {{&S85658[76], 4}, {NULL}};
const Segment SGI295543[] = {{&S85658[8], 3}, {NULL}};
const Segment SGI295544[] = {{&S85658[13], 7}, {NULL}};
const Segment SGI295545[] = {{&S85658[7], 1}, {NULL}};
const Segment SGI295546[] = {{&S85658[16], 4}, {&S85658[28], 12}, {NULL}};
const Segment SGI295547[] = {{&S85661[60], 5}, {NULL}};
const Segment SGI295548[] = {{&S85661[8], 3}, {NULL}};
const Segment SGI295549[] = {{&S85661[13], 14}, {NULL}};
const Segment SGI295550[] = {{&S85661[7], 1}, {NULL}};
const Segment SGI295551[] = {{&S85661[16], 11}, {NULL}};
const Segment SGI295552[] = {{&S85661[66], 13}, {NULL}};
const Segment SGI295553[] = {{&S85789[54], 4}, {NULL}};
const Segment SGI295554[] = {{&S85789[8], 3}, {NULL}};
const Segment SGI295555[] = {{&S85789[13], 14}, {NULL}};
const Segment SGI295556[] = {{&S85789[7], 1}, {NULL}};
const Segment SGI295557[] = {{&S85789[16], 11}, {NULL}};
const Segment SGI295558[] = {{&S85789[61], 12}, {NULL}};
const Segment SGI295559[] = {{&S85790[64], 4}, {NULL}};
const Segment SGI295560[] = {{&S85790[8], 3}, {NULL}};
const Segment SGI295561[] = {{&S85790[13], 14}, {NULL}};
const Segment SGI295562[] = {{&S85790[7], 1}, {NULL}};
const Segment SGI295563[] = {{&S85790[16], 11}, {&S85790[32], 2}, {NULL}};
const Segment SGI295564[] = {{&S85790[75], 12}, {NULL}};
const Segment SGI295565[] = {{&S85791[61], 4}, {NULL}};
const Segment SGI295566[] = {{&S85791[8], 3}, {NULL}};
const Segment SGI295567[] = {{&S85791[13], 14}, {NULL}};
const Segment SGI295568[] = {{&S85791[7], 1}, {NULL}};
const Segment SGI295569[] = {{&S85791[16], 11}, {NULL}};
const Segment SGI295570[] = {{&S85791[68], 13}, {NULL}};
const Segment SGI295571[] = {{&S85792[54], 4}, {NULL}};
const Segment SGI295572[] = {{&S85792[8], 3}, {NULL}};
const Segment SGI295573[] = {{&S85792[13], 14}, {NULL}};
const Segment SGI295574[] = {{&S85792[7], 1}, {NULL}};
const Segment SGI295575[] = {{&S85792[16], 11}, {NULL}};
const Segment SGI295576[] = {{&S85792[61], 13}, {NULL}};
const Segment SGI295577[] = {{&S82743[8], 3}, {NULL}};
const Segment SGI295578[] = {{&S82743[37], 4}, {NULL}};
const Segment SGI295579[] = {{&S82743[68], 6}, {NULL}};
const Segment SGI295580[] = {{&S82743[13], 30}, {NULL}};
const Segment SGI295581[] = {{&S82744[8], 3}, {NULL}};
const Segment SGI295582[] = {{&S82744[37], 6}, {NULL}};
const Segment SGI295583[] = {{&S82744[68], 6}, {NULL}};
const Segment SGI295584[] = {{&S82744[13], 30}, {NULL}};
const Segment SGI295585[] = {{&S82745[8], 3}, {NULL}};
const Segment SGI295586[] = {{&S82745[37], 6}, {NULL}};
const Segment SGI295587[] = {{&S82745[68], 6}, {NULL}};
const Segment SGI295588[] = {{&S82745[13], 30}, {NULL}};
const Segment SGI295589[] = {{&S85850[69], 5}, {NULL}};
const Segment SGI295590[] = {{&S85850[8], 3}, {NULL}};
const Segment SGI295591[] = {{&S85850[13], 9}, {NULL}};
const Segment SGI295592[] = {{&S85850[30], 13}, {NULL}};
const Segment SGI295593[] = {{&S85850[7], 1}, {NULL}};
const Segment SGI295594[] = {{&S85850[75], 13}, {NULL}};
const Segment SGI295595[] = {{&S85912[94], 4}, {NULL}};
const Segment SGI295596[] = {{&S85912[8], 3}, {NULL}};
const Segment SGI295597[] = {{&S85912[13], 7}, {NULL}};
const Segment SGI295598[] = {{&S85912[25], 14}, {NULL}};
const Segment SGI295599[] = {{&S85912[7], 1}, {NULL}};
const Segment SGI295600[] = {{&S85912[16], 4}, {&S85912[28], 11}, {NULL}};
const Segment SGI295601[] = {{&S85912[101], 8}, {NULL}};
const Segment SGI295602[] = {{&S86119[53], 5}, {NULL}};
const Segment SGI295603[] = {{&S86119[8], 3}, {NULL}};
const Segment SGI295604[] = {{&S86119[13], 14}, {NULL}};
const Segment SGI295605[] = {{&S86119[7], 1}, {NULL}};
const Segment SGI295606[] = {{&S86119[16], 11}, {NULL}};
const Segment SGI295607[] = {{&S86119[59], 13}, {NULL}};
const Segment SGI295608[] = {{&S83135[8], 3}, {NULL}};
const Segment SGI295609[] = {{&S83135[33], 4}, {NULL}};
const Segment SGI295610[] = {{&S83135[63], 6}, {NULL}};
const Segment SGI295611[] = {{&S83135[13], 24}, {NULL}};
const Segment SGI295612[] = {{&S83135[33], 2}, {NULL}};
const Segment SGI295613[] = {{&S86238[69], 5}, {NULL}};
const Segment SGI295614[] = {{&S86238[8], 3}, {NULL}};
const Segment SGI295615[] = {{&S86238[13], 9}, {NULL}};
const Segment SGI295616[] = {{&S86238[30], 13}, {NULL}};
const Segment SGI295617[] = {{&S86238[7], 1}, {NULL}};
const Segment SGI295618[] = {{&S86238[75], 13}, {NULL}};
const Segment SGI295619[] = {{&S86333[69], 5}, {NULL}};
const Segment SGI295620[] = {{&S86333[8], 3}, {NULL}};
const Segment SGI295621[] = {{&S86333[13], 9}, {NULL}};
const Segment SGI295622[] = {{&S86333[30], 13}, {NULL}};
const Segment SGI295623[] = {{&S86333[7], 1}, {NULL}};
const Segment SGI295624[] = {{&S86333[77], 13}, {NULL}};
const Segment SGI295625[] = {{&S70073[33], 15}, {NULL}};
const Segment SGI295626[] = {{&S70073[0], 81}, {NULL}};
const Segment SGI295627[] = {{&S70074[33], 15}, {NULL}};
const Segment SGI295628[] = {{&S70074[0], 81}, {NULL}};
const Segment SGI295629[] = {{&S70075[33], 15}, {NULL}};
const Segment SGI295630[] = {{&S70075[0], 81}, {NULL}};
const Segment SGI295631[] = {{&S70076[33], 15}, {NULL}};
const Segment SGI295632[] = {{&S70076[0], 80}, {NULL}};
const Segment SGI295633[] = {{&S70077[33], 15}, {NULL}};
const Segment SGI295634[] = {{&S70077[0], 80}, {NULL}};
const Segment SGI295635[] = {{&S70078[33], 15}, {NULL}};
const Segment SGI295636[] = {{&S70078[0], 80}, {NULL}};
const Segment SGI295637[] = {{&S70081[33], 15}, {NULL}};
const Segment SGI295638[] = {{&S70081[0], 80}, {NULL}};
const Segment SGI295639[] = {{&S86563[54], 4}, {NULL}};
const Segment SGI295640[] = {{&S86563[8], 3}, {NULL}};
const Segment SGI295641[] = {{&S86563[13], 14}, {NULL}};
const Segment SGI295642[] = {{&S86563[7], 1}, {NULL}};
const Segment SGI295643[] = {{&S86563[16], 11}, {NULL}};
const Segment SGI295644[] = {{&S86564[55], 4}, {NULL}};
const Segment SGI295645[] = {{&S86564[8], 3}, {NULL}};
const Segment SGI295646[] = {{&S86564[13], 3}, {NULL}};
const Segment SGI295647[] = {{&S86564[13], 3}, {&S86564[18], 7}, {NULL}};
const Segment SGI295648[] = {{&S86564[7], 1}, {NULL}};
const Segment SGI295649[] = {{&S86678[70], 4}, {NULL}};
const Segment SGI295650[] = {{&S86678[8], 3}, {NULL}};
const Segment SGI295651[] = {{&S86678[13], 7}, {NULL}};
const Segment SGI295652[] = {{&S86678[25], 14}, {NULL}};
const Segment SGI295653[] = {{&S86678[7], 1}, {NULL}};
const Segment SGI295654[] = {{&S86678[16], 4}, {&S86678[28], 11}, {NULL}};
const Segment SGI295655[] = {{&S86714[54], 4}, {NULL}};
const Segment SGI295656[] = {{&S86714[8], 3}, {NULL}};
const Segment SGI295657[] = {{&S86714[13], 14}, {NULL}};
const Segment SGI295658[] = {{&S86714[7], 1}, {NULL}};
const Segment SGI295659[] = {{&S86714[16], 11}, {NULL}};
const Segment SGI295660[] = {{&S86714[61], 13}, {NULL}};
const Segment SGI295661[] = {{&S86915[61], 4}, {NULL}};
const Segment SGI295662[] = {{&S86915[8], 3}, {NULL}};
const Segment SGI295663[] = {{&S86915[13], 14}, {NULL}};
const Segment SGI295664[] = {{&S86915[7], 1}, {NULL}};
const Segment SGI295665[] = {{&S86915[16], 11}, {NULL}};
const Segment SGI295666[] = {{&S86915[68], 13}, {NULL}};
const Segment SGI295667[] = {{&S86916[54], 4}, {NULL}};
const Segment SGI295668[] = {{&S86916[8], 3}, {NULL}};
const Segment SGI295669[] = {{&S86916[13], 14}, {NULL}};
const Segment SGI295670[] = {{&S86916[7], 1}, {NULL}};
const Segment SGI295671[] = {{&S86916[16], 11}, {NULL}};
const Segment SGI295672[] = {{&S86916[61], 13}, {NULL}};
const Segment SGI295673[] = {{&S86950[67], 5}, {NULL}};
const Segment SGI295674[] = {{&S86950[8], 3}, {NULL}};
const Segment SGI295675[] = {{&S86950[13], 9}, {NULL}};
const Segment SGI295676[] = {{&S86950[28], 13}, {NULL}};
const Segment SGI295677[] = {{&S86950[7], 1}, {NULL}};
const Segment SGI295678[] = {{&S86950[73], 13}, {NULL}};
const Segment SGI295679[] = {{&S87050[54], 4}, {NULL}};
const Segment SGI295680[] = {{&S87050[8], 3}, {NULL}};
const Segment SGI295681[] = {{&S87050[13], 14}, {NULL}};
const Segment SGI295682[] = {{&S87050[7], 1}, {NULL}};
const Segment SGI295683[] = {{&S87050[16], 11}, {NULL}};
const Segment SGI295684[] = {{&S87050[61], 13}, {NULL}};
const Segment SGI295685[] = {{&S87051[54], 4}, {NULL}};
const Segment SGI295686[] = {{&S87051[8], 3}, {NULL}};
const Segment SGI295687[] = {{&S87051[13], 14}, {NULL}};
const Segment SGI295688[] = {{&S87051[7], 1}, {NULL}};
const Segment SGI295689[] = {{&S87051[16], 11}, {NULL}};
const Segment SGI295690[] = {{&S87051[61], 13}, {NULL}};
const Segment SGI295691[] = {{&S87073[53], 5}, {NULL}};
const Segment SGI295692[] = {{&S87073[8], 3}, {NULL}};
const Segment SGI295693[] = {{&S87073[13], 14}, {NULL}};
const Segment SGI295694[] = {{&S87073[7], 1}, {NULL}};
const Segment SGI295695[] = {{&S87073[16], 11}, {NULL}};
const Segment SGI295696[] = {{&S87073[59], 8}, {NULL}};
const Segment SGI295697[] = {{&S84093[8], 3}, {NULL}};
const Segment SGI295698[] = {{&S84093[37], 5}, {NULL}};
const Segment SGI295699[] = {{&S84093[66], 6}, {NULL}};
const Segment SGI295700[] = {{&S84093[13], 29}, {NULL}};
const Segment SGI295701[] = {{&S84094[8], 3}, {NULL}};
const Segment SGI295702[] = {{&S84094[37], 3}, {NULL}};
const Segment SGI295703[] = {{&S84094[62], 6}, {NULL}};
const Segment SGI295704[] = {{&S84094[13], 27}, {NULL}};
const Segment SGI295705[] = {{&S87299[71], 5}, {NULL}};
const Segment SGI295706[] = {{&S87299[8], 3}, {NULL}};
const Segment SGI295707[] = {{&S87299[13], 9}, {NULL}};
const Segment SGI295708[] = {{&S87299[30], 13}, {NULL}};
const Segment SGI295709[] = {{&S87299[7], 1}, {NULL}};
const Segment SGI295710[] = {{&S87299[79], 13}, {NULL}};
const Segment SGI295711[] = {{&S87309[61], 4}, {NULL}};
const Segment SGI295712[] = {{&S87309[8], 3}, {NULL}};
const Segment SGI295713[] = {{&S87309[13], 14}, {NULL}};
const Segment SGI295714[] = {{&S87309[7], 1}, {NULL}};
const Segment SGI295715[] = {{&S87309[16], 11}, {NULL}};
const Segment SGI295716[] = {{&S87309[68], 13}, {NULL}};
const Segment SGI295717[] = {{&S87431[53], 4}, {NULL}};
const Segment SGI295718[] = {{&S87431[8], 3}, {NULL}};
const Segment SGI295719[] = {{&S87431[13], 3}, {NULL}};
const Segment SGI295720[] = {{&S87431[7], 1}, {NULL}};
const Segment SGI295721[] = {{&S87431[58], 13}, {NULL}};
const Segment SGI295722[] = {{&S87432[69], 5}, {NULL}};
const Segment SGI295723[] = {{&S87432[8], 3}, {NULL}};
const Segment SGI295724[] = {{&S87432[13], 9}, {NULL}};
const Segment SGI295725[] = {{&S87432[30], 13}, {NULL}};
const Segment SGI295726[] = {{&S87432[7], 1}, {NULL}};
const Segment SGI295727[] = {{&S87432[77], 13}, {NULL}};
const Segment SGI295728[] = {{&S87558[67], 5}, {NULL}};
const Segment SGI295729[] = {{&S87558[8], 3}, {NULL}};
const Segment SGI295730[] = {{&S87558[13], 14}, {NULL}};
const Segment SGI295731[] = {{&S87558[7], 1}, {NULL}};
const Segment SGI295732[] = {{&S87558[16], 11}, {&S87558[32], 2}, {NULL}};
const Segment SGI295733[] = {{&S87558[79], 13}, {NULL}};
const Segment SGI295734[] = {{&S87559[55], 5}, {NULL}};
const Segment SGI295735[] = {{&S87559[8], 3}, {NULL}};
const Segment SGI295736[] = {{&S87559[13], 14}, {NULL}};
const Segment SGI295737[] = {{&S87559[7], 1}, {NULL}};
const Segment SGI295738[] = {{&S87559[16], 11}, {NULL}};
const Segment SGI295739[] = {{&S87559[63], 13}, {NULL}};
const Segment SGI295740[] = {{&S87566[70], 4}, {NULL}};
const Segment SGI295741[] = {{&S87566[8], 3}, {NULL}};
const Segment SGI295742[] = {{&S87566[13], 7}, {NULL}};
const Segment SGI295743[] = {{&S87566[7], 1}, {NULL}};
const Segment SGI295744[] = {{&S87566[16], 4}, {&S87566[28], 6}, {NULL}};
const Segment SGI295745[] = {{&S87679[70], 4}, {NULL}};
const Segment SGI295746[] = {{&S87679[8], 3}, {NULL}};
const Segment SGI295747[] = {{&S87679[13], 3}, {NULL}};
const Segment SGI295748[] = {{&S87679[13], 3}, {&S87679[21], 7}, {NULL}};
const Segment SGI295749[] = {{&S87679[7], 1}, {NULL}};
const Segment SGI295750[] = {{&S87833[102], 4}, {NULL}};
const Segment SGI295751[] = {{&S87833[8], 3}, {NULL}};
const Segment SGI295752[] = {{&S87833[13], 9}, {NULL}};
const Segment SGI295753[] = {{&S87833[33], 13}, {NULL}};
const Segment SGI295754[] = {{&S87833[7], 1}, {NULL}};
const Segment SGI295755[] = {{&S87833[109], 8}, {NULL}};
const Segment SGI295756[] = {{&S87928[61], 4}, {NULL}};
const Segment SGI295757[] = {{&S87928[8], 3}, {NULL}};
const Segment SGI295758[] = {{&S87928[13], 14}, {NULL}};
const Segment SGI295759[] = {{&S87928[7], 1}, {NULL}};
const Segment SGI295760[] = {{&S87928[16], 11}, {NULL}};
const Segment SGI295761[] = {{&S87928[68], 13}, {NULL}};
const Segment SGI295762[] = {{&S87954[73], 4}, {NULL}};
const Segment SGI295763[] = {{&S87954[8], 3}, {NULL}};
const Segment SGI295764[] = {{&S87954[13], 7}, {NULL}};
const Segment SGI295765[] = {{&S87954[25], 14}, {NULL}};
const Segment SGI295766[] = {{&S87954[7], 1}, {NULL}};
const Segment SGI295767[] = {{&S87954[16], 4}, {&S87954[28], 11}, {NULL}};
const Segment SGI295768[] = {{&S88063[73], 4}, {NULL}};
const Segment SGI295769[] = {{&S88063[8], 3}, {NULL}};
const Segment SGI295770[] = {{&S88063[13], 7}, {NULL}};
const Segment SGI295771[] = {{&S88063[25], 14}, {NULL}};
const Segment SGI295772[] = {{&S88063[7], 1}, {NULL}};
const Segment SGI295773[] = {{&S88063[16], 4}, {&S88063[28], 11}, {NULL}};
const Segment SGI295774[] = {{&S88064[69], 4}, {NULL}};
const Segment SGI295775[] = {{&S88064[8], 3}, {NULL}};
const Segment SGI295776[] = {{&S88064[13], 14}, {NULL}};
const Segment SGI295777[] = {{&S88064[7], 1}, {NULL}};
const Segment SGI295778[] = {{&S88064[16], 11}, {&S88064[32], 2}, {NULL}};
const Segment SGI295779[] = {{&S71404[0], 91}, {NULL}};
const Segment SGI295780[] = {{&S88198[57], 5}, {NULL}};
const Segment SGI295781[] = {{&S88198[8], 3}, {NULL}};
const Segment SGI295782[] = {{&S88198[13], 3}, {NULL}};
const Segment SGI295783[] = {{&S88198[13], 3}, {&S88198[18], 7}, {NULL}};
const Segment SGI295784[] = {{&S88198[7], 1}, {NULL}};
const Segment SGI295785[] = {{&S88300[74], 4}, {NULL}};
const Segment SGI295786[] = {{&S88300[8], 3}, {NULL}};
const Segment SGI295787[] = {{&S88300[13], 7}, {NULL}};
const Segment SGI295788[] = {{&S88300[25], 14}, {NULL}};
const Segment SGI295789[] = {{&S88300[7], 1}, {NULL}};
const Segment SGI295790[] = {{&S88300[16], 4}, {&S88300[28], 11}, {NULL}};
const Segment SGI295791[] = {{&S88301[64], 4}, {NULL}};
const Segment SGI295792[] = {{&S88301[8], 3}, {NULL}};
const Segment SGI295793[] = {{&S88301[13], 14}, {NULL}};
const Segment SGI295794[] = {{&S88301[7], 1}, {NULL}};
const Segment SGI295795[] = {{&S88301[16], 11}, {&S88301[32], 2}, {NULL}};
const Segment SGI295796[] = {{&S88301[73], 12}, {NULL}};
const Segment SGI295797[] = {{&S88302[59], 4}, {NULL}};
const Segment SGI295798[] = {{&S88302[8], 3}, {NULL}};
const Segment SGI295799[] = {{&S88302[13], 14}, {NULL}};
const Segment SGI295800[] = {{&S88302[7], 1}, {NULL}};
const Segment SGI295801[] = {{&S88302[16], 11}, {NULL}};
const Segment SGI295802[] = {{&S88302[64], 12}, {NULL}};
const Segment SGI295803[] = {{&S88307[8], 3}, {NULL}};
const Segment SGI295804[] = {{&S88307[13], 9}, {NULL}};
const Segment SGI295805[] = {{&S88308[8], 3}, {NULL}};
const Segment SGI295806[] = {{&S88308[13], 9}, {NULL}};
const Segment SGI295807[] = {{&S88308[31], 15}, {NULL}};
const Segment SGI295808[] = {{&S88446[75], 4}, {NULL}};
const Segment SGI295809[] = {{&S88446[8], 3}, {NULL}};
const Segment SGI295810[] = {{&S88446[13], 7}, {NULL}};
const Segment SGI295811[] = {{&S88446[25], 14}, {NULL}};
const Segment SGI295812[] = {{&S88446[7], 1}, {NULL}};
const Segment SGI295813[] = {{&S88446[16], 4}, {&S88446[28], 11}, {NULL}};
const Segment SGI295814[] = {{&S71635[0], 77}, {NULL}};
const Segment SGI295815[] = {{&S71636[0], 54}, {NULL}};
const Segment SGI295816[] = {{&S71637[0], 51}, {NULL}};
const Segment SGI295817[] = {{&S71638[0], 80}, {NULL}};
const Segment SGI295818[] = {{&S71639[0], 77}, {NULL}};
const Segment SGI295819[] = {{&S71640[0], 77}, {NULL}};
const Segment SGI295820[] = {{&S71641[0], 77}, {NULL}};
const Segment SGI295821[] = {{&S71642[0], 77}, {NULL}};
const Segment SGI295822[] = {{&S71643[0], 77}, {NULL}};
const Segment SGI295823[] = {{&S88525[69], 5}, {NULL}};
const Segment SGI295824[] = {{&S88525[8], 3}, {NULL}};
const Segment SGI295825[] = {{&S88525[13], 9}, {NULL}};
const Segment SGI295826[] = {{&S88525[30], 13}, {NULL}};
const Segment SGI295827[] = {{&S88525[7], 1}, {NULL}};
const Segment SGI295828[] = {{&S71733[0], 58}, {NULL}};
const Segment SGI295829[] = {{&S71734[0], 58}, {NULL}};
const Segment SGI295830[] = {{&S71735[0], 58}, {NULL}};
const Segment SGI295831[] = {{&S71736[0], 58}, {NULL}};
const Segment SGI295832[] = {{&S71737[0], 61}, {NULL}};
const Segment SGI295833[] = {{&S71738[0], 58}, {NULL}};
const Segment SGI295834[] = {{&S71739[0], 58}, {NULL}};
const Segment SGI295835[] = {{&S71740[0], 58}, {NULL}};
const Segment SGI295836[] = {{&S71741[0], 58}, {NULL}};
const Segment SGI295837[] = {{&S71742[0], 59}, {NULL}};
const Segment SGI295838[] = {{&S71743[0], 58}, {NULL}};
const Segment SGI295839[] = {{&S71744[0], 61}, {NULL}};
const Segment SGI295840[] = {{&S88588[81], 4}, {NULL}};
const Segment SGI295841[] = {{&S88588[8], 3}, {NULL}};
const Segment SGI295842[] = {{&S88588[13], 9}, {NULL}};
const Segment SGI295843[] = {{&S88588[31], 13}, {NULL}};
const Segment SGI295844[] = {{&S71745[0], 58}, {NULL}};
const Segment SGI295845[] = {{&S88588[7], 1}, {NULL}};
const Segment SGI295846[] = {{&S88588[89], 9}, {NULL}};
const Segment SGI295847[] = {{&S71746[0], 58}, {NULL}};
const Segment SGI295848[] = {{&S85449[8], 3}, {NULL}};
const Segment SGI295849[] = {{&S85449[37], 4}, {NULL}};
const Segment SGI295850[] = {{&S85449[64], 6}, {NULL}};
const Segment SGI295851[] = {{&S85449[13], 28}, {NULL}};
const Segment SGI295852[] = {{&S85450[8], 3}, {NULL}};
const Segment SGI295853[] = {{&S85450[37], 4}, {NULL}};
const Segment SGI295854[] = {{&S85450[66], 6}, {NULL}};
const Segment SGI295855[] = {{&S85450[13], 28}, {NULL}};
const Segment SGI295856[] = {{&S85450[59], 6}, {NULL}};
const Segment SGI295857[] = {{&S88712[55], 5}, {NULL}};
const Segment SGI295858[] = {{&S88712[8], 3}, {NULL}};
const Segment SGI295859[] = {{&S88712[13], 14}, {NULL}};
const Segment SGI295860[] = {{&S88712[7], 1}, {NULL}};
const Segment SGI295861[] = {{&S88712[16], 11}, {NULL}};
const Segment SGI295862[] = {{&S88712[63], 13}, {NULL}};
const Segment SGI295863[] = {{&S88718[54], 4}, {NULL}};
const Segment SGI295864[] = {{&S88718[8], 3}, {NULL}};
const Segment SGI295865[] = {{&S88718[13], 14}, {NULL}};
const Segment SGI295866[] = {{&S88718[7], 1}, {NULL}};
const Segment SGI295867[] = {{&S88718[16], 11}, {NULL}};
const Segment SGI295868[] = {{&S85571[8], 3}, {NULL}};
const Segment SGI295869[] = {{&S85571[13], 15}, {NULL}};
const Segment SGI295870[] = {{&S85571[24], 2}, {NULL}};
const Segment SGI295871[] = {{&S85571[24], 4}, {NULL}};
const Segment SGI295872[] = {{&S88725[61], 4}, {NULL}};
const Segment SGI295873[] = {{&S88725[8], 3}, {NULL}};
const Segment SGI295874[] = {{&S88725[13], 14}, {NULL}};
const Segment SGI295875[] = {{&S88725[7], 1}, {NULL}};
const Segment SGI295876[] = {{&S88725[16], 11}, {NULL}};
const Segment SGI295877[] = {{&S88725[68], 13}, {NULL}};
const Segment SGI295878[] = {{&S88727[70], 4}, {NULL}};
const Segment SGI295879[] = {{&S88727[8], 3}, {NULL}};
const Segment SGI295880[] = {{&S88727[13], 7}, {NULL}};
const Segment SGI295881[] = {{&S88727[7], 1}, {NULL}};
const Segment SGI295882[] = {{&S88727[16], 4}, {&S88727[28], 5}, {NULL}};
const Segment SGI295883[] = {{&S88838[73], 4}, {NULL}};
const Segment SGI295884[] = {{&S88838[8], 3}, {NULL}};
const Segment SGI295885[] = {{&S88838[13], 7}, {NULL}};
const Segment SGI295886[] = {{&S88838[25], 14}, {NULL}};
const Segment SGI295887[] = {{&S88838[7], 1}, {NULL}};
const Segment SGI295888[] = {{&S88838[16], 4}, {&S88838[28], 11}, {NULL}};
const Segment SGI295889[] = {{&S88838[81], 13}, {NULL}};
const Segment SGI295890[] = {{&S88849[75], 4}, {NULL}};
const Segment SGI295891[] = {{&S88849[8], 3}, {NULL}};
const Segment SGI295892[] = {{&S88849[13], 7}, {NULL}};
const Segment SGI295893[] = {{&S88849[25], 14}, {NULL}};
const Segment SGI295894[] = {{&S88849[7], 1}, {NULL}};
const Segment SGI295895[] = {{&S88849[16], 4}, {&S88849[28], 11}, {NULL}};
const Segment SGI295896[] = {{&S72109[87], 10}, {NULL}};
const Segment SGI295897[] = {{&S72109[0], 137}, {NULL}};
const Segment SGI295898[] = {{&S72110[0], 145}, {NULL}};
const Segment SGI295899[] = {{&S72111[96], 17}, {NULL}};
const Segment SGI295900[] = {{&S72111[0], 135}, {NULL}};
const Segment SGI295901[] = {{&S72112[0], 120}, {NULL}};
const Segment SGI295902[] = {{&S72113[96], 17}, {NULL}};
const Segment SGI295903[] = {{&S72113[0], 134}, {NULL}};
const Segment SGI295904[] = {{&S72114[0], 95}, {NULL}};
const Segment SGI295905[] = {{&S72115[0], 94}, {NULL}};
const Segment SGI295906[] = {{&S88988[52], 4}, {NULL}};
const Segment SGI295907[] = {{&S88988[8], 3}, {NULL}};
const Segment SGI295908[] = {{&S88988[13], 14}, {NULL}};
const Segment SGI295909[] = {{&S88988[7], 1}, {NULL}};
const Segment SGI295910[] = {{&S88988[16], 11}, {NULL}};
const Segment SGI295911[] = {{&S88988[57], 12}, {NULL}};
const Segment SGI295912[] = {{&S88992[60], 5}, {NULL}};
const Segment SGI295913[] = {{&S88992[8], 3}, {NULL}};
const Segment SGI295914[] = {{&S88992[13], 14}, {NULL}};
const Segment SGI295915[] = {{&S88992[7], 1}, {NULL}};
const Segment SGI295916[] = {{&S88992[16], 11}, {NULL}};
const Segment SGI295917[] = {{&S88992[70], 14}, {NULL}};
const Segment SGI295918[] = {{&S89113[70], 4}, {NULL}};
const Segment SGI295919[] = {{&S89113[8], 3}, {NULL}};
const Segment SGI295920[] = {{&S89113[13], 7}, {NULL}};
const Segment SGI295921[] = {{&S89113[7], 1}, {NULL}};
const Segment SGI295922[] = {{&S89113[16], 4}, {&S89113[28], 5}, {NULL}};
const Segment SGI295923[] = {{&S89114[75], 4}, {NULL}};
const Segment SGI295924[] = {{&S89114[8], 3}, {NULL}};
const Segment SGI295925[] = {{&S89114[13], 7}, {NULL}};
const Segment SGI295926[] = {{&S89114[25], 14}, {NULL}};
const Segment SGI295927[] = {{&S89114[7], 1}, {NULL}};
const Segment SGI295928[] = {{&S89114[16], 4}, {&S89114[28], 11}, {NULL}};
const Segment SGI295929[] = {{&S89115[63], 4}, {NULL}};
const Segment SGI295930[] = {{&S89115[8], 3}, {NULL}};
const Segment SGI295931[] = {{&S89115[13], 14}, {NULL}};
const Segment SGI295932[] = {{&S89115[7], 1}, {NULL}};
const Segment SGI295933[] = {{&S89115[16], 11}, {NULL}};
const Segment SGI295934[] = {{&S89115[70], 11}, {NULL}};
const Segment SGI295935[] = {{&S89130[75], 4}, {NULL}};
const Segment SGI295936[] = {{&S89130[8], 3}, {NULL}};
const Segment SGI295937[] = {{&S89130[13], 7}, {NULL}};
const Segment SGI295938[] = {{&S89130[25], 14}, {NULL}};
const Segment SGI295939[] = {{&S89130[7], 1}, {NULL}};
const Segment SGI295940[] = {{&S89130[16], 4}, {&S89130[28], 11}, {NULL}};
const Segment SGI295941[] = {{&S72343[32], 4}, {NULL}};
const Segment SGI295942[] = {{&S72343[0], 49}, {NULL}};
const Segment SGI295943[] = {{&S72366[79], 9}, {NULL}};
const Segment SGI295944[] = {{&S72366[0], 128}, {NULL}};
const Segment SGI295945[] = {{&S72367[18], 9}, {NULL}};
const Segment SGI295946[] = {{&S72367[0], 128}, {NULL}};
const Segment SGI295947[] = {{&S72368[0], 121}, {NULL}};
const Segment SGI295948[] = {{&S72369[0], 95}, {NULL}};
const Segment SGI295949[] = {{&S72370[79], 5}, {NULL}};
const Segment SGI295950[] = {{&S72370[0], 104}, {NULL}};
const Segment SGI295951[] = {{&S72371[0], 98}, {NULL}};
const Segment SGI295952[] = {{&S72372[0], 70}, {NULL}};
const Segment SGI295953[] = {{&S72373[79], 5}, {NULL}};
const Segment SGI295954[] = {{&S72373[0], 110}, {NULL}};
const Segment SGI295955[] = {{&S72374[79], 5}, {NULL}};
const Segment SGI295956[] = {{&S72374[0], 105}, {NULL}};
const Segment SGI295957[] = {{&S89247[57], 5}, {NULL}};
const Segment SGI295958[] = {{&S89247[8], 3}, {NULL}};
const Segment SGI295959[] = {{&S89247[13], 3}, {NULL}};
const Segment SGI295960[] = {{&S89247[13], 3}, {&S89247[18], 7}, {NULL}};
const Segment SGI295961[] = {{&S89247[7], 1}, {NULL}};
const Segment SGI295962[] = {{&S89248[52], 4}, {NULL}};
const Segment SGI295963[] = {{&S89248[8], 3}, {NULL}};
const Segment SGI295964[] = {{&S89248[13], 14}, {NULL}};
const Segment SGI295965[] = {{&S89248[7], 1}, {NULL}};
const Segment SGI295966[] = {{&S89248[16], 11}, {NULL}};
const Segment SGI295967[] = {{&S89248[57], 12}, {NULL}};
const Segment SGI295968[] = {{&S89249[64], 4}, {NULL}};
const Segment SGI295969[] = {{&S89249[8], 3}, {NULL}};
const Segment SGI295970[] = {{&S89249[13], 14}, {NULL}};
const Segment SGI295971[] = {{&S89249[7], 1}, {NULL}};
const Segment SGI295972[] = {{&S89249[16], 11}, {&S89249[32], 2}, {NULL}};
const Segment SGI295973[] = {{&S89249[73], 12}, {NULL}};
const Segment SGI295974[] = {{&S89266[75], 4}, {NULL}};
const Segment SGI295975[] = {{&S89266[8], 3}, {NULL}};
const Segment SGI295976[] = {{&S89266[13], 7}, {NULL}};
const Segment SGI295977[] = {{&S89266[25], 14}, {NULL}};
const Segment SGI295978[] = {{&S89266[7], 1}, {NULL}};
const Segment SGI295979[] = {{&S89266[16], 4}, {&S89266[28], 11}, {NULL}};
const Segment SGI295980[] = {{&S86033[8], 3}, {NULL}};
const Segment SGI295981[] = {{&S86033[37], 4}, {NULL}};
const Segment SGI295982[] = {{&S86033[64], 6}, {NULL}};
const Segment SGI295983[] = {{&S86033[13], 28}, {NULL}};
const Segment SGI295984[] = {{&S89417[69], 5}, {NULL}};
const Segment SGI295985[] = {{&S89417[8], 3}, {NULL}};
const Segment SGI295986[] = {{&S89417[13], 9}, {NULL}};
const Segment SGI295987[] = {{&S89417[30], 13}, {NULL}};
const Segment SGI295988[] = {{&S89417[7], 1}, {NULL}};
const Segment SGI295989[] = {{&S89417[75], 13}, {NULL}};
const Segment SGI295990[] = {{&S89418[65], 5}, {NULL}};
const Segment SGI295991[] = {{&S89418[8], 3}, {NULL}};
const Segment SGI295992[] = {{&S89418[13], 14}, {NULL}};
const Segment SGI295993[] = {{&S89418[7], 1}, {NULL}};
const Segment SGI295994[] = {{&S89418[16], 11}, {&S89418[32], 2}, {NULL}};
const Segment SGI295995[] = {{&S89418[75], 13}, {NULL}};
const Segment SGI295996[] = {{&S89419[53], 5}, {NULL}};
const Segment SGI295997[] = {{&S89419[8], 3}, {NULL}};
const Segment SGI295998[] = {{&S89419[13], 14}, {NULL}};
const Segment SGI295999[] = {{&S89419[7], 1}, {NULL}};
const Segment SGI296000[] = {{&S89419[16], 11}, {NULL}};
const Segment SGI296001[] = {{&S89419[59], 13}, {NULL}};
const Segment SGI296002[] = {{&S72932[33], 14}, {NULL}};
const Segment SGI296003[] = {{&S72932[0], 79}, {NULL}};
const Segment SGI296004[] = {{&S72933[33], 14}, {NULL}};
const Segment SGI296005[] = {{&S72933[0], 79}, {NULL}};
const Segment SGI296006[] = {{&S72934[33], 14}, {NULL}};
const Segment SGI296007[] = {{&S72934[0], 79}, {NULL}};
const Segment SGI296008[] = {{&S89575[70], 4}, {NULL}};
const Segment SGI296009[] = {{&S89575[8], 3}, {NULL}};
const Segment SGI296010[] = {{&S89575[13], 9}, {NULL}};
const Segment SGI296011[] = {{&S89575[30], 13}, {NULL}};
const Segment SGI296012[] = {{&S89575[7], 1}, {NULL}};
const Segment SGI296013[] = {{&S89575[77], 13}, {NULL}};
const Segment SGI296014[] = {{&S89576[8], 3}, {NULL}};
const Segment SGI296015[] = {{&S89576[13], 9}, {NULL}};
const Segment SGI296016[] = {{&S89576[30], 13}, {NULL}};
const Segment SGI296017[] = {{&S89576[71], 12}, {NULL}};
const Segment SGI296018[] = {{&S89576[73], 13}, {NULL}};
const Segment SGI296019[] = {{&S89578[83], 4}, {NULL}};
const Segment SGI296020[] = {{&S89578[8], 3}, {NULL}};
const Segment SGI296021[] = {{&S89578[13], 9}, {NULL}};
const Segment SGI296022[] = {{&S89578[33], 13}, {NULL}};
const Segment SGI296023[] = {{&S89578[7], 1}, {NULL}};
const Segment SGI296024[] = {{&S89578[91], 9}, {NULL}};
const Segment SGI296025[] = {{&S73097[55], 13}, {NULL}};
const Segment SGI296026[] = {{&S73097[0], 92}, {NULL}};
const Segment SGI296027[] = {{&S73098[0], 118}, {NULL}};
const Segment SGI296028[] = {{&S73099[0], 118}, {NULL}};
const Segment SGI296029[] = {{&S73100[0], 118}, {NULL}};
const Segment SGI296030[] = {{&S73101[55], 17}, {NULL}};
const Segment SGI296031[] = {{&S73101[0], 96}, {NULL}};
const Segment SGI296032[] = {{&S73102[0], 118}, {NULL}};
const Segment SGI296033[] = {{&S73103[0], 118}, {NULL}};
const Segment SGI296034[] = {{&S73105[0], 118}, {NULL}};
const Segment SGI296035[] = {{&S73141[0], 92}, {NULL}};
const Segment SGI296036[] = {{&S89718[66], 4}, {NULL}};
const Segment SGI296037[] = {{&S89718[8], 3}, {NULL}};
const Segment SGI296038[] = {{&S89718[13], 9}, {NULL}};
const Segment SGI296039[] = {{&S89718[33], 15}, {NULL}};
const Segment SGI296040[] = {{&S89718[7], 1}, {NULL}};
const Segment SGI296041[] = {{&S89728[65], 5}, {NULL}};
const Segment SGI296042[] = {{&S89728[8], 3}, {NULL}};
const Segment SGI296043[] = {{&S89728[13], 14}, {NULL}};
const Segment SGI296044[] = {{&S89728[7], 1}, {NULL}};
const Segment SGI296045[] = {{&S89728[16], 11}, {&S89728[32], 2}, {NULL}};
const Segment SGI296046[] = {{&S89728[77], 13}, {NULL}};
const Segment SGI296047[] = {{&S89729[53], 5}, {NULL}};
const Segment SGI296048[] = {{&S89729[8], 3}, {NULL}};
const Segment SGI296049[] = {{&S89729[13], 14}, {NULL}};
const Segment SGI296050[] = {{&S89729[7], 1}, {NULL}};
const Segment SGI296051[] = {{&S89729[16], 11}, {NULL}};
const Segment SGI296052[] = {{&S89729[61], 13}, {NULL}};
const Segment SGI296053[] = {{&S89783[69], 5}, {NULL}};
const Segment SGI296054[] = {{&S89783[8], 3}, {NULL}};
const Segment SGI296055[] = {{&S89783[13], 9}, {NULL}};
const Segment SGI296056[] = {{&S89783[30], 13}, {NULL}};
const Segment SGI296057[] = {{&S89783[7], 1}, {NULL}};
const Segment SGI296058[] = {{&S89783[75], 13}, {NULL}};
const Segment SGI296059[] = {{&S73332[0], 70}, {NULL}};
const Segment SGI296060[] = {{&S73333[87], 4}, {NULL}};
const Segment SGI296061[] = {{&S73333[0], 111}, {NULL}};
const Segment SGI296062[] = {{&S73334[87], 4}, {NULL}};
const Segment SGI296063[] = {{&S73334[0], 111}, {NULL}};
const Segment SGI296064[] = {{&S73335[0], 94}, {NULL}};
const Segment SGI296065[] = {{&S73336[87], 4}, {NULL}};
const Segment SGI296066[] = {{&S73336[0], 111}, {NULL}};
const Segment SGI296067[] = {{&S89866[7], 4}, {NULL}};
const Segment SGI296068[] = {{&S89866[8], 3}, {NULL}};
const Segment SGI296069[] = {{&S89866[13], 14}, {NULL}};
const Segment SGI296070[] = {{&S73337[0], 94}, {NULL}};
const Segment SGI296071[] = {{&S89866[7], 1}, {NULL}};
const Segment SGI296072[] = {{&S89866[16], 11}, {NULL}};
const Segment SGI296073[] = {{&S89866[64], 13}, {NULL}};
const Segment SGI296074[] = {{&S86675[8], 3}, {NULL}};
const Segment SGI296075[] = {{&S86675[33], 4}, {NULL}};
const Segment SGI296076[] = {{&S86675[63], 6}, {NULL}};
const Segment SGI296077[] = {{&S86675[13], 24}, {NULL}};
const Segment SGI296078[] = {{&S86675[33], 2}, {NULL}};
const Segment SGI296079[] = {{&S73531[33], 8}, {NULL}};
const Segment SGI296080[] = {{&S73531[0], 74}, {NULL}};
const Segment SGI296081[] = {{&S73532[33], 8}, {NULL}};
const Segment SGI296082[] = {{&S73532[0], 74}, {NULL}};
const Segment SGI296083[] = {{&S73533[33], 8}, {NULL}};
const Segment SGI296084[] = {{&S73533[0], 73}, {NULL}};
const Segment SGI296085[] = {{&S73534[33], 8}, {NULL}};
const Segment SGI296086[] = {{&S73534[0], 73}, {NULL}};
const Segment SGI296087[] = {{&S73535[33], 8}, {NULL}};
const Segment SGI296088[] = {{&S73535[0], 73}, {NULL}};
const Segment SGI296089[] = {{&S73536[33], 8}, {NULL}};
const Segment SGI296090[] = {{&S73536[0], 73}, {NULL}};
const Segment SGI296091[] = {{&S73537[33], 8}, {NULL}};
const Segment SGI296092[] = {{&S73537[0], 74}, {NULL}};
const Segment SGI296093[] = {{&S73538[33], 8}, {NULL}};
const Segment SGI296094[] = {{&S73538[0], 73}, {NULL}};
const Segment SGI296095[] = {{&S90037[71], 4}, {NULL}};
const Segment SGI296096[] = {{&S90037[8], 3}, {NULL}};
const Segment SGI296097[] = {{&S90037[13], 9}, {NULL}};
const Segment SGI296098[] = {{&S90037[28], 13}, {NULL}};
const Segment SGI296099[] = {{&S90037[7], 1}, {NULL}};
const Segment SGI296100[] = {{&S90201[60], 5}, {NULL}};
const Segment SGI296101[] = {{&S90201[8], 3}, {NULL}};
const Segment SGI296102[] = {{&S90201[13], 14}, {NULL}};
const Segment SGI296103[] = {{&S90201[7], 1}, {NULL}};
const Segment SGI296104[] = {{&S90201[16], 11}, {NULL}};
const Segment SGI296105[] = {{&S90201[66], 13}, {NULL}};
const Segment SGI296106[] = {{&S90204[102], 4}, {NULL}};
const Segment SGI296107[] = {{&S90204[8], 3}, {NULL}};
const Segment SGI296108[] = {{&S90204[13], 9}, {NULL}};
const Segment SGI296109[] = {{&S90204[33], 13}, {NULL}};
const Segment SGI296110[] = {{&S90204[7], 1}, {NULL}};
const Segment SGI296111[] = {{&S90204[109], 8}, {NULL}};
const Segment SGI296112[] = {{&S90206[76], 4}, {NULL}};
const Segment SGI296113[] = {{&S90206[8], 3}, {NULL}};
const Segment SGI296114[] = {{&S90206[13], 7}, {NULL}};
const Segment SGI296115[] = {{&S90206[25], 14}, {NULL}};
const Segment SGI296116[] = {{&S90206[7], 1}, {NULL}};
const Segment SGI296117[] = {{&S90206[16], 4}, {&S90206[28], 11}, {NULL}};
const Segment SGI296118[] = {{&S90206[84], 9}, {NULL}};
const Segment SGI296119[] = {{&S90216[71], 4}, {NULL}};
const Segment SGI296120[] = {{&S90216[8], 3}, {NULL}};
const Segment SGI296121[] = {{&S90216[13], 7}, {NULL}};
const Segment SGI296122[] = {{&S90216[25], 14}, {NULL}};
const Segment SGI296123[] = {{&S90216[7], 1}, {NULL}};
const Segment SGI296124[] = {{&S90216[16], 4}, {&S90216[28], 11}, {NULL}};
const Segment SGI296125[] = {{&S90217[66], 5}, {NULL}};
const Segment SGI296126[] = {{&S90217[8], 3}, {NULL}};
const Segment SGI296127[] = {{&S90217[13], 7}, {NULL}};
const Segment SGI296128[] = {{&S90217[25], 14}, {NULL}};
const Segment SGI296129[] = {{&S90217[7], 1}, {NULL}};
const Segment SGI296130[] = {{&S90217[16], 4}, {&S90217[28], 11}, {NULL}};
const Segment SGI296131[] = {{&S90269[76], 4}, {NULL}};
const Segment SGI296132[] = {{&S90269[8], 3}, {NULL}};
const Segment SGI296133[] = {{&S90269[13], 7}, {NULL}};
const Segment SGI296134[] = {{&S90269[25], 14}, {NULL}};
const Segment SGI296135[] = {{&S90269[7], 1}, {NULL}};
const Segment SGI296136[] = {{&S90269[16], 4}, {&S90269[28], 11}, {NULL}};
const Segment SGI296137[] = {{&S90269[86], 9}, {NULL}};
const Segment SGI296138[] = {{&S90376[72], 4}, {NULL}};
const Segment SGI296139[] = {{&S90376[8], 3}, {NULL}};
const Segment SGI296140[] = {{&S90376[13], 3}, {NULL}};
const Segment SGI296141[] = {{&S90376[7], 1}, {NULL}};
const Segment SGI296142[] = {{&S90376[77], 17}, {NULL}};
const Segment SGI296143[] = {{&S90386[60], 5}, {NULL}};
const Segment SGI296144[] = {{&S90386[8], 3}, {NULL}};
const Segment SGI296145[] = {{&S90386[13], 14}, {NULL}};
const Segment SGI296146[] = {{&S90386[7], 1}, {NULL}};
const Segment SGI296147[] = {{&S90386[16], 11}, {NULL}};
const Segment SGI296148[] = {{&S90386[66], 13}, {NULL}};
const Segment SGI296149[] = {{&S90389[7], 4}, {NULL}};
const Segment SGI296150[] = {{&S90389[8], 3}, {NULL}};
const Segment SGI296151[] = {{&S90389[13], 9}, {NULL}};
const Segment SGI296152[] = {{&S90389[28], 13}, {NULL}};
const Segment SGI296153[] = {{&S90389[7], 1}, {NULL}};
const Segment SGI296154[] = {{&S90393[66], 4}, {NULL}};
const Segment SGI296155[] = {{&S90393[8], 3}, {NULL}};
const Segment SGI296156[] = {{&S90393[13], 9}, {NULL}};
const Segment SGI296157[] = {{&S90393[28], 13}, {NULL}};
const Segment SGI296158[] = {{&S90393[7], 1}, {NULL}};
const Segment SGI296159[] = {{&S90552[82], 4}, {NULL}};
const Segment SGI296160[] = {{&S90552[8], 3}, {NULL}};
const Segment SGI296161[] = {{&S90552[13], 9}, {NULL}};
const Segment SGI296162[] = {{&S90552[33], 13}, {NULL}};
const Segment SGI296163[] = {{&S90552[7], 1}, {NULL}};
const Segment SGI296164[] = {{&S90552[89], 8}, {NULL}};
const Segment SGI296165[] = {{&S90553[68], 4}, {NULL}};
const Segment SGI296166[] = {{&S90553[8], 3}, {NULL}};
const Segment SGI296167[] = {{&S90553[13], 9}, {NULL}};
const Segment SGI296168[] = {{&S90553[30], 13}, {NULL}};
const Segment SGI296169[] = {{&S90553[7], 1}, {NULL}};
const Segment SGI296170[] = {{&S90553[73], 13}, {NULL}};
const Segment SGI296171[] = {{&S90562[69], 5}, {NULL}};
const Segment SGI296172[] = {{&S90562[8], 3}, {NULL}};
const Segment SGI296173[] = {{&S90562[13], 9}, {NULL}};
const Segment SGI296174[] = {{&S90562[30], 13}, {NULL}};
const Segment SGI296175[] = {{&S90562[7], 1}, {NULL}};
const Segment SGI296176[] = {{&S90562[75], 13}, {NULL}};
const Segment SGI296177[] = {{&S90572[53], 5}, {NULL}};
const Segment SGI296178[] = {{&S90572[8], 3}, {NULL}};
const Segment SGI296179[] = {{&S90572[13], 14}, {NULL}};
const Segment SGI296180[] = {{&S90572[7], 1}, {NULL}};
const Segment SGI296181[] = {{&S90572[16], 11}, {NULL}};
const Segment SGI296182[] = {{&S90572[59], 13}, {NULL}};
const Segment SGI296183[] = {{&S74098[0], 77}, {NULL}};
const Segment SGI296184[] = {{&S74099[0], 82}, {NULL}};
const Segment SGI296185[] = {{&S74100[80], 16}, {NULL}};
const Segment SGI296186[] = {{&S74100[0], 117}, {NULL}};
const Segment SGI296187[] = {{&S87435[8], 3}, {NULL}};
const Segment SGI296188[] = {{&S87435[37], 4}, {NULL}};
const Segment SGI296189[] = {{&S87435[66], 6}, {NULL}};
const Segment SGI296190[] = {{&S87435[13], 28}, {NULL}};
const Segment SGI296191[] = {{&S87435[59], 6}, {NULL}};
const Segment SGI296192[] = {{&S90767[83], 4}, {NULL}};
const Segment SGI296193[] = {{&S90767[8], 3}, {NULL}};
const Segment SGI296194[] = {{&S90767[13], 9}, {NULL}};
const Segment SGI296195[] = {{&S90767[33], 13}, {NULL}};
const Segment SGI296196[] = {{&S90767[7], 1}, {NULL}};
const Segment SGI296197[] = {{&S90767[91], 9}, {NULL}};
const Segment SGI296198[] = {{&S90781[83], 4}, {NULL}};
const Segment SGI296199[] = {{&S90781[8], 3}, {NULL}};
const Segment SGI296200[] = {{&S90781[13], 9}, {NULL}};
const Segment SGI296201[] = {{&S90781[33], 13}, {NULL}};
const Segment SGI296202[] = {{&S90781[7], 1}, {NULL}};
const Segment SGI296203[] = {{&S90781[91], 9}, {NULL}};
const Segment SGI296204[] = {{&S74409[0], 95}, {NULL}};
const Segment SGI296205[] = {{&S90950[70], 4}, {NULL}};
const Segment SGI296206[] = {{&S90950[8], 3}, {NULL}};
const Segment SGI296207[] = {{&S90950[13], 9}, {NULL}};
const Segment SGI296208[] = {{&S90950[30], 13}, {NULL}};
const Segment SGI296209[] = {{&S90950[7], 1}, {NULL}};
const Segment SGI296210[] = {{&S90950[77], 13}, {NULL}};
const Segment SGI296211[] = {{&S90961[60], 5}, {NULL}};
const Segment SGI296212[] = {{&S90961[8], 3}, {NULL}};
const Segment SGI296213[] = {{&S90961[13], 14}, {NULL}};
const Segment SGI296214[] = {{&S90961[7], 1}, {NULL}};
const Segment SGI296215[] = {{&S90961[16], 11}, {NULL}};
const Segment SGI296216[] = {{&S90961[68], 13}, {NULL}};
const Segment SGI296217[] = {{&S74489[0], 95}, {NULL}};
const Segment SGI296218[] = {{&S74520[0], 62}, {NULL}};
const Segment SGI296219[] = {{&S74521[0], 68}, {NULL}};
const Segment SGI296220[] = {{&S74522[0], 62}, {NULL}};
const Segment SGI296221[] = {{&S74523[0], 68}, {NULL}};
const Segment SGI296222[] = {{&S74524[0], 68}, {NULL}};
const Segment SGI296223[] = {{&S74525[0], 68}, {NULL}};
const Segment SGI296224[] = {{&S74526[0], 64}, {NULL}};
const Segment SGI296225[] = {{&S74527[0], 62}, {NULL}};
const Segment SGI296226[] = {{&S74564[0], 80}, {NULL}};
const Segment SGI296227[] = {{&S74565[0], 109}, {NULL}};
const Segment SGI296228[] = {{&S74566[0], 81}, {NULL}};
const Segment SGI296229[] = {{&S74567[0], 99}, {NULL}};
const Segment SGI296230[] = {{&S74568[0], 99}, {NULL}};
const Segment SGI296231[] = {{&S74569[0], 99}, {NULL}};
const Segment SGI296232[] = {{&S74570[0], 99}, {NULL}};
const Segment SGI296233[] = {{&S74571[0], 75}, {NULL}};
const Segment SGI296234[] = {{&S74572[0], 102}, {NULL}};
const Segment SGI296235[] = {{&S74573[0], 99}, {NULL}};
const Segment SGI296236[] = {{&S74574[0], 81}, {NULL}};
const Segment SGI296237[] = {{&S65710[0], 9}, {NULL}};
const Segment SGI296238[] = {{&S91128[7], 4}, {NULL}};
const Segment SGI296239[] = {{&S91128[8], 3}, {NULL}};
const Segment SGI296240[] = {{&S91128[13], 14}, {NULL}};
const Segment SGI296241[] = {{&S91128[7], 1}, {NULL}};
const Segment SGI296242[] = {{&S91128[16], 11}, {&S91128[32], 2}, {NULL}};
const Segment SGI296243[] = {{&S91128[73], 12}, {NULL}};
const Segment SGI296244[] = {{&S91129[7], 4}, {NULL}};
const Segment SGI296245[] = {{&S91129[8], 3}, {NULL}};
const Segment SGI296246[] = {{&S91129[13], 14}, {NULL}};
const Segment SGI296247[] = {{&S91129[7], 1}, {NULL}};
const Segment SGI296248[] = {{&S91129[16], 11}, {NULL}};
const Segment SGI296249[] = {{&S91129[64], 12}, {NULL}};
const Segment SGI296250[] = {{&S91139[54], 4}, {NULL}};
const Segment SGI296251[] = {{&S91139[8], 3}, {NULL}};
const Segment SGI296252[] = {{&S91139[13], 14}, {NULL}};
const Segment SGI296253[] = {{&S91139[7], 1}, {NULL}};
const Segment SGI296254[] = {{&S91139[16], 11}, {NULL}};
const Segment SGI296255[] = {{&S91139[61], 13}, {NULL}};
const Segment SGI296256[] = {{&S91165[57], 5}, {NULL}};
const Segment SGI296257[] = {{&S91165[8], 3}, {NULL}};
const Segment SGI296258[] = {{&S91165[13], 3}, {NULL}};
const Segment SGI296259[] = {{&S91165[13], 3}, {&S91165[18], 7}, {NULL}};
const Segment SGI296260[] = {{&S91165[7], 1}, {NULL}};
const Segment SGI296261[] = {{&S91287[53], 4}, {NULL}};
const Segment SGI296262[] = {{&S91287[8], 3}, {NULL}};
const Segment SGI296263[] = {{&S91287[13], 3}, {NULL}};
const Segment SGI296264[] = {{&S91287[7], 1}, {NULL}};
const Segment SGI296265[] = {{&S91287[58], 13}, {NULL}};
const Segment SGI296266[] = {{&S91289[77], 4}, {NULL}};
const Segment SGI296267[] = {{&S91289[8], 3}, {NULL}};
const Segment SGI296268[] = {{&S91289[13], 3}, {NULL}};
const Segment SGI296269[] = {{&S91289[7], 1}, {NULL}};
const Segment SGI296270[] = {{&S91289[85], 9}, {NULL}};
const Segment SGI296271[] = {{&S91290[59], 4}, {NULL}};
const Segment SGI296272[] = {{&S91290[8], 3}, {NULL}};
const Segment SGI296273[] = {{&S91290[13], 14}, {NULL}};
const Segment SGI296274[] = {{&S91290[7], 1}, {NULL}};
const Segment SGI296275[] = {{&S91290[16], 11}, {NULL}};
const Segment SGI296276[] = {{&S91290[64], 13}, {NULL}};
const Segment SGI296277[] = {{&S74897[0], 52}, {NULL}};
const Segment SGI296278[] = {{&S91291[52], 4}, {NULL}};
const Segment SGI296279[] = {{&S91291[8], 3}, {NULL}};
const Segment SGI296280[] = {{&S91291[13], 14}, {NULL}};
const Segment SGI296281[] = {{&S91291[7], 1}, {NULL}};
const Segment SGI296282[] = {{&S74898[0], 51}, {NULL}};
const Segment SGI296283[] = {{&S91291[16], 11}, {NULL}};
const Segment SGI296284[] = {{&S91291[57], 16}, {NULL}};
const Segment SGI296285[] = {{&S74899[0], 55}, {NULL}};
const Segment SGI296286[] = {{&S74900[0], 50}, {NULL}};
const Segment SGI296287[] = {{&S91293[59], 4}, {NULL}};
const Segment SGI296288[] = {{&S91293[8], 3}, {NULL}};
const Segment SGI296289[] = {{&S91293[13], 14}, {NULL}};
const Segment SGI296290[] = {{&S91293[7], 1}, {NULL}};
const Segment SGI296291[] = {{&S91293[16], 11}, {NULL}};
const Segment SGI296292[] = {{&S91293[64], 12}, {NULL}};
const Segment SGI296293[] = {{&S74901[0], 57}, {NULL}};
const Segment SGI296294[] = {{&S74902[0], 56}, {NULL}};
const Segment SGI296295[] = {{&S91294[64], 4}, {NULL}};
const Segment SGI296296[] = {{&S91294[8], 3}, {NULL}};
const Segment SGI296297[] = {{&S91294[13], 14}, {NULL}};
const Segment SGI296298[] = {{&S91294[7], 1}, {NULL}};
const Segment SGI296299[] = {{&S91294[16], 11}, {&S91294[32], 2}, {NULL}};
const Segment SGI296300[] = {{&S91294[73], 12}, {NULL}};
const Segment SGI296301[] = {{&S74903[0], 49}, {NULL}};
const Segment SGI296302[] = {{&S74904[0], 51}, {NULL}};
const Segment SGI296303[] = {{&S74905[0], 56}, {NULL}};
const Segment SGI296304[] = {{&S74906[0], 52}, {NULL}};
const Segment SGI296305[] = {{&S91297[86], 4}, {NULL}};
const Segment SGI296306[] = {{&S91297[8], 3}, {NULL}};
const Segment SGI296307[] = {{&S91297[13], 3}, {NULL}};
const Segment SGI296308[] = {{&S91297[60], 10}, {NULL}};
const Segment SGI296309[] = {{&S91297[7], 1}, {NULL}};
const Segment SGI296310[] = {{&S91297[60], 7}, {NULL}};
const Segment SGI296311[] = {{&S74907[0], 52}, {NULL}};
const Segment SGI296312[] = {{&S74908[0], 54}, {NULL}};
const Segment SGI296313[] = {{&S74909[0], 54}, {NULL}};
const Segment SGI296314[] = {{&S74910[0], 56}, {NULL}};
const Segment SGI296315[] = {{&S74911[0], 56}, {NULL}};
const Segment SGI296316[] = {{&S74912[0], 56}, {NULL}};
const Segment SGI296317[] = {{&S74913[0], 52}, {NULL}};
const Segment SGI296318[] = {{&S91302[61], 4}, {NULL}};
const Segment SGI296319[] = {{&S91302[8], 3}, {NULL}};
const Segment SGI296320[] = {{&S91302[13], 14}, {NULL}};
const Segment SGI296321[] = {{&S91302[7], 1}, {NULL}};
const Segment SGI296322[] = {{&S91302[16], 11}, {NULL}};
const Segment SGI296323[] = {{&S91302[68], 12}, {NULL}};
const Segment SGI296324[] = {{&S74914[0], 56}, {NULL}};
const Segment SGI296325[] = {{&S74915[0], 54}, {NULL}};
const Segment SGI296326[] = {{&S91303[66], 4}, {NULL}};
const Segment SGI296327[] = {{&S91303[8], 3}, {NULL}};
const Segment SGI296328[] = {{&S91303[13], 14}, {NULL}};
const Segment SGI296329[] = {{&S91303[7], 1}, {NULL}};
const Segment SGI296330[] = {{&S91303[16], 11}, {&S91303[32], 2}, {NULL}};
const Segment SGI296331[] = {{&S91303[77], 12}, {NULL}};
const Segment SGI296332[] = {{&S91318[60], 5}, {NULL}};
const Segment SGI296333[] = {{&S91318[8], 3}, {NULL}};
const Segment SGI296334[] = {{&S91318[13], 14}, {NULL}};
const Segment SGI296335[] = {{&S91318[7], 1}, {NULL}};
const Segment SGI296336[] = {{&S91318[16], 11}, {NULL}};
const Segment SGI296337[] = {{&S91318[66], 13}, {NULL}};
const Segment SGI296338[] = {{&S88260[8], 3}, {NULL}};
const Segment SGI296339[] = {{&S88260[13], 15}, {NULL}};
const Segment SGI296340[] = {{&S88260[24], 2}, {NULL}};
const Segment SGI296341[] = {{&S88260[24], 4}, {NULL}};
const Segment SGI296342[] = {{&S91502[7], 4}, {NULL}};
const Segment SGI296343[] = {{&S91502[8], 3}, {NULL}};
const Segment SGI296344[] = {{&S91502[13], 14}, {NULL}};
const Segment SGI296345[] = {{&S91502[7], 1}, {NULL}};
const Segment SGI296346[] = {{&S91502[16], 11}, {NULL}};
const Segment SGI296347[] = {{&S91502[64], 13}, {NULL}};
const Segment SGI296348[] = {{&S91505[61], 4}, {NULL}};
const Segment SGI296349[] = {{&S91505[8], 3}, {NULL}};
const Segment SGI296350[] = {{&S91505[13], 14}, {NULL}};
const Segment SGI296351[] = {{&S91505[7], 1}, {NULL}};
const Segment SGI296352[] = {{&S91505[16], 11}, {NULL}};
const Segment SGI296353[] = {{&S91505[68], 13}, {NULL}};
const Segment SGI296354[] = {{&S75124[0], 55}, {NULL}};
const Segment SGI296355[] = {{&S75125[0], 50}, {NULL}};
const Segment SGI296356[] = {{&S75126[0], 55}, {NULL}};
const Segment SGI296357[] = {{&S75127[0], 58}, {NULL}};
const Segment SGI296358[] = {{&S75128[0], 55}, {NULL}};
const Segment SGI296359[] = {{&S91523[63], 4}, {NULL}};
const Segment SGI296360[] = {{&S91523[8], 3}, {NULL}};
const Segment SGI296361[] = {{&S91523[13], 7}, {NULL}};
const Segment SGI296362[] = {{&S91523[25], 14}, {NULL}};
const Segment SGI296363[] = {{&S91523[7], 1}, {NULL}};
const Segment SGI296364[] = {{&S91523[16], 4}, {&S91523[28], 11}, {NULL}};
const Segment SGI296365[] = {{&S75225[43], 17}, {NULL}};
const Segment SGI296366[] = {{&S75225[0], 105}, {NULL}};
const Segment SGI296367[] = {{&S75226[43], 17}, {NULL}};
const Segment SGI296368[] = {{&S75226[0], 105}, {NULL}};
const Segment SGI296369[] = {{&S91619[75], 4}, {NULL}};
const Segment SGI296370[] = {{&S91619[8], 3}, {NULL}};
const Segment SGI296371[] = {{&S91619[13], 7}, {NULL}};
const Segment SGI296372[] = {{&S91619[25], 14}, {NULL}};
const Segment SGI296373[] = {{&S91619[7], 1}, {NULL}};
const Segment SGI296374[] = {{&S91619[16], 4}, {&S91619[28], 11}, {NULL}};
const Segment SGI296375[] = {{&S75268[65], 13}, {NULL}};
const Segment SGI296376[] = {{&S75268[0], 135}, {NULL}};
const Segment SGI296377[] = {{&S91700[74], 4}, {NULL}};
const Segment SGI296378[] = {{&S91700[8], 3}, {NULL}};
const Segment SGI296379[] = {{&S91700[13], 3}, {NULL}};
const Segment SGI296380[] = {{&S91700[13], 3}, {&S91700[60], 6}, {NULL}};
const Segment SGI296381[] = {{&S91700[7], 1}, {NULL}};
const Segment SGI296382[] = {{&S91716[68], 4}, {NULL}};
const Segment SGI296383[] = {{&S91716[8], 3}, {NULL}};
const Segment SGI296384[] = {{&S91716[13], 9}, {NULL}};
const Segment SGI296385[] = {{&S91716[28], 13}, {NULL}};
const Segment SGI296386[] = {{&S91716[7], 1}, {NULL}};
const Segment SGI296387[] = {{&S91725[60], 5}, {NULL}};
const Segment SGI296388[] = {{&S91725[8], 3}, {NULL}};
const Segment SGI296389[] = {{&S91725[13], 14}, {NULL}};
const Segment SGI296390[] = {{&S91725[7], 1}, {NULL}};
const Segment SGI296391[] = {{&S91725[16], 11}, {NULL}};
const Segment SGI296392[] = {{&S91725[70], 14}, {NULL}};
const Segment SGI296393[] = {{&S91737[84], 4}, {NULL}};
const Segment SGI296394[] = {{&S91737[8], 3}, {NULL}};
const Segment SGI296395[] = {{&S91737[13], 7}, {NULL}};
const Segment SGI296396[] = {{&S91737[25], 14}, {NULL}};
const Segment SGI296397[] = {{&S91737[7], 1}, {NULL}};
const Segment SGI296398[] = {{&S91737[16], 4}, {&S91737[28], 11}, {NULL}};
const Segment SGI296399[] = {{&S91737[68], 8}, {NULL}};
const Segment SGI296400[] = {{&S91767[75], 4}, {NULL}};
const Segment SGI296401[] = {{&S91767[8], 3}, {NULL}};
const Segment SGI296402[] = {{&S91767[13], 7}, {NULL}};
const Segment SGI296403[] = {{&S91767[25], 14}, {NULL}};
const Segment SGI296404[] = {{&S91767[7], 1}, {NULL}};
const Segment SGI296405[] = {{&S91767[16], 4}, {&S91767[28], 11}, {NULL}};
const Segment SGI296406[] = {{&S91767[82], 9}, {NULL}};
const Segment SGI296407[] = {{&S91906[56], 5}, {NULL}};
const Segment SGI296408[] = {{&S91906[8], 3}, {NULL}};
const Segment SGI296409[] = {{&S91906[13], 3}, {NULL}};
const Segment SGI296410[] = {{&S91906[13], 3}, {&S91906[18], 7}, {NULL}};
const Segment SGI296411[] = {{&S91906[7], 1}, {NULL}};
const Segment SGI296412[] = {{&S91907[68], 4}, {NULL}};
const Segment SGI296413[] = {{&S91907[8], 3}, {NULL}};
const Segment SGI296414[] = {{&S91907[13], 9}, {NULL}};
const Segment SGI296415[] = {{&S91907[28], 13}, {NULL}};
const Segment SGI296416[] = {{&S91907[7], 1}, {NULL}};
const Segment SGI296417[] = {{&S91915[53], 5}, {NULL}};
const Segment SGI296418[] = {{&S91915[8], 3}, {NULL}};
const Segment SGI296419[] = {{&S91915[13], 14}, {NULL}};
const Segment SGI296420[] = {{&S91915[7], 1}, {NULL}};
const Segment SGI296421[] = {{&S91915[16], 11}, {NULL}};
const Segment SGI296422[] = {{&S91915[63], 14}, {NULL}};
const Segment SGI296423[] = {{&S75570[31], 6}, {NULL}};
const Segment SGI296424[] = {{&S75570[0], 70}, {NULL}};
const Segment SGI296425[] = {{&S75571[31], 6}, {NULL}};
const Segment SGI296426[] = {{&S75571[0], 70}, {NULL}};
const Segment SGI296427[] = {{&S75572[33], 6}, {NULL}};
const Segment SGI296428[] = {{&S75572[0], 72}, {NULL}};
const Segment SGI296429[] = {{&S75573[31], 6}, {NULL}};
const Segment SGI296430[] = {{&S75573[0], 70}, {NULL}};
const Segment SGI296431[] = {{&S75574[31], 6}, {NULL}};
const Segment SGI296432[] = {{&S75574[0], 71}, {NULL}};
const Segment SGI296433[] = {{&S75575[31], 6}, {NULL}};
const Segment SGI296434[] = {{&S75575[0], 70}, {NULL}};
const Segment SGI296435[] = {{&S75576[31], 6}, {NULL}};
const Segment SGI296436[] = {{&S75576[0], 69}, {NULL}};
const Segment SGI296437[] = {{&S75577[33], 8}, {NULL}};
const Segment SGI296438[] = {{&S75577[0], 73}, {NULL}};
const Segment SGI296439[] = {{&S75578[33], 8}, {NULL}};
const Segment SGI296440[] = {{&S75578[0], 73}, {NULL}};
const Segment SGI296441[] = {{&S75579[33], 8}, {NULL}};
const Segment SGI296442[] = {{&S75579[0], 73}, {NULL}};
const Segment SGI296443[] = {{&S75580[31], 6}, {NULL}};
const Segment SGI296444[] = {{&S75580[0], 69}, {NULL}};
const Segment SGI296445[] = {{&S92013[85], 4}, {NULL}};
const Segment SGI296446[] = {{&S92013[8], 3}, {NULL}};
const Segment SGI296447[] = {{&S92013[13], 7}, {NULL}};
const Segment SGI296448[] = {{&S92013[25], 14}, {NULL}};
const Segment SGI296449[] = {{&S92013[7], 1}, {NULL}};
const Segment SGI296450[] = {{&S92013[16], 4}, {&S92013[28], 11}, {NULL}};
const Segment SGI296451[] = {{&S92013[69], 8}, {NULL}};
const Segment SGI296452[] = {{&S92021[67], 5}, {NULL}};
const Segment SGI296453[] = {{&S92021[8], 3}, {NULL}};
const Segment SGI296454[] = {{&S92021[13], 9}, {NULL}};
const Segment SGI296455[] = {{&S92021[28], 13}, {NULL}};
const Segment SGI296456[] = {{&S92021[7], 1}, {NULL}};
const Segment SGI296457[] = {{&S75771[33], 12}, {NULL}};
const Segment SGI296458[] = {{&S75771[0], 78}, {NULL}};
const Segment SGI296459[] = {{&S75772[33], 12}, {NULL}};
const Segment SGI296460[] = {{&S75772[0], 78}, {NULL}};
const Segment SGI296461[] = {{&S75773[33], 12}, {NULL}};
const Segment SGI296462[] = {{&S75773[0], 78}, {NULL}};
const Segment SGI296463[] = {{&S75774[33], 12}, {NULL}};
const Segment SGI296464[] = {{&S75774[0], 78}, {NULL}};
const Segment SGI296465[] = {{&S75775[33], 12}, {NULL}};
const Segment SGI296466[] = {{&S75775[0], 80}, {NULL}};
const Segment SGI296467[] = {{&S75776[33], 12}, {NULL}};
const Segment SGI296468[] = {{&S75776[0], 78}, {NULL}};
const Segment SGI296469[] = {{&S75777[33], 12}, {NULL}};
const Segment SGI296470[] = {{&S75777[0], 78}, {NULL}};
const Segment SGI296471[] = {{&S75778[33], 12}, {NULL}};
const Segment SGI296472[] = {{&S75778[0], 77}, {NULL}};
const Segment SGI296473[] = {{&S75779[33], 12}, {NULL}};
const Segment SGI296474[] = {{&S75779[0], 77}, {NULL}};
const Segment SGI296475[] = {{&S75780[33], 12}, {NULL}};
const Segment SGI296476[] = {{&S75780[0], 78}, {NULL}};
const Segment SGI296477[] = {{&S75781[33], 12}, {NULL}};
const Segment SGI296478[] = {{&S75781[0], 77}, {NULL}};
const Segment SGI296479[] = {{&S92340[8], 3}, {NULL}};
const Segment SGI296480[] = {{&S92340[13], 7}, {NULL}};
const Segment SGI296481[] = {{&S92340[7], 1}, {NULL}};
const Segment SGI296482[] = {{&S92340[16], 37}, {NULL}};
const Segment SGI296483[] = {{&S92496[54], 4}, {NULL}};
const Segment SGI296484[] = {{&S92496[8], 3}, {NULL}};
const Segment SGI296485[] = {{&S92496[13], 14}, {NULL}};
const Segment SGI296486[] = {{&S92496[7], 1}, {NULL}};
const Segment SGI296487[] = {{&S92496[16], 11}, {NULL}};
const Segment SGI296488[] = {{&S92508[65], 5}, {NULL}};
const Segment SGI296489[] = {{&S92508[8], 3}, {NULL}};
const Segment SGI296490[] = {{&S92508[13], 14}, {NULL}};
const Segment SGI296491[] = {{&S92508[7], 1}, {NULL}};
const Segment SGI296492[] = {{&S92508[16], 11}, {&S92508[32], 2}, {NULL}};
const Segment SGI296493[] = {{&S92508[75], 13}, {NULL}};
const Segment SGI296494[] = {{&S92509[53], 5}, {NULL}};
const Segment SGI296495[] = {{&S92509[8], 3}, {NULL}};
const Segment SGI296496[] = {{&S92509[13], 14}, {NULL}};
const Segment SGI296497[] = {{&S92509[7], 1}, {NULL}};
const Segment SGI296498[] = {{&S92509[16], 11}, {NULL}};
const Segment SGI296499[] = {{&S92509[59], 13}, {NULL}};
const Segment SGI296500[] = {{&S92522[57], 5}, {NULL}};
const Segment SGI296501[] = {{&S92522[8], 3}, {NULL}};
const Segment SGI296502[] = {{&S92522[13], 3}, {NULL}};
const Segment SGI296503[] = {{&S92522[13], 3}, {&S92522[18], 7}, {NULL}};
const Segment SGI296504[] = {{&S92522[7], 1}, {NULL}};
const Segment SGI296505[] = {{&S92523[94], 4}, {NULL}};
const Segment SGI296506[] = {{&S92523[8], 3}, {NULL}};
const Segment SGI296507[] = {{&S92523[13], 7}, {NULL}};
const Segment SGI296508[] = {{&S92523[25], 14}, {NULL}};
const Segment SGI296509[] = {{&S92523[7], 1}, {NULL}};
const Segment SGI296510[] = {{&S92523[16], 4}, {&S92523[28], 11}, {NULL}};
const Segment SGI296511[] = {{&S92523[101], 8}, {NULL}};
const Segment SGI296512[] = {{&S92527[85], 4}, {NULL}};
const Segment SGI296513[] = {{&S92527[8], 3}, {NULL}};
const Segment SGI296514[] = {{&S92527[13], 3}, {NULL}};
const Segment SGI296515[] = {{&S92527[13], 3}, {&S92527[58], 6}, {NULL}};
const Segment SGI296516[] = {{&S92527[58], 11}, {NULL}};
const Segment SGI296517[] = {{&S92527[7], 1}, {NULL}};
const Segment SGI296518[] = {{&S92558[74], 4}, {NULL}};
const Segment SGI296519[] = {{&S92558[8], 3}, {NULL}};
const Segment SGI296520[] = {{&S92558[13], 7}, {NULL}};
const Segment SGI296521[] = {{&S92558[25], 14}, {NULL}};
const Segment SGI296522[] = {{&S92558[7], 1}, {NULL}};
const Segment SGI296523[] = {{&S92558[16], 4}, {&S92558[28], 11}, {NULL}};
const Segment SGI296524[] = {{&S76101[0], 56}, {NULL}};
const Segment SGI296525[] = {{&S76102[0], 53}, {NULL}};
const Segment SGI296526[] = {{&S76103[0], 53}, {NULL}};
const Segment SGI296527[] = {{&S76104[0], 53}, {NULL}};
const Segment SGI296528[] = {{&S76105[0], 53}, {NULL}};
const Segment SGI296529[] = {{&S76106[0], 53}, {NULL}};
const Segment SGI296530[] = {{&S92613[69], 5}, {NULL}};
const Segment SGI296531[] = {{&S92613[8], 3}, {NULL}};
const Segment SGI296532[] = {{&S92613[13], 9}, {NULL}};
const Segment SGI296533[] = {{&S92613[30], 13}, {NULL}};
const Segment SGI296534[] = {{&S92613[7], 1}, {NULL}};
const Segment SGI296535[] = {{&S92686[90], 4}, {NULL}};
const Segment SGI296536[] = {{&S92686[8], 3}, {NULL}};
const Segment SGI296537[] = {{&S92686[13], 3}, {NULL}};
const Segment SGI296538[] = {{&S92686[13], 3}, {&S92686[60], 6}, {NULL}};
const Segment SGI296539[] = {{&S92686[60], 11}, {NULL}};
const Segment SGI296540[] = {{&S92686[7], 1}, {NULL}};
const Segment SGI296541[] = {{&S92687[73], 4}, {NULL}};
const Segment SGI296542[] = {{&S92687[8], 3}, {NULL}};
const Segment SGI296543[] = {{&S92687[13], 7}, {NULL}};
const Segment SGI296544[] = {{&S92687[25], 14}, {NULL}};
const Segment SGI296545[] = {{&S92687[7], 1}, {NULL}};
const Segment SGI296546[] = {{&S92687[16], 4}, {&S92687[28], 11}, {NULL}};
const Segment SGI296547[] = {{&S76223[0], 107}, {NULL}};
const Segment SGI296548[] = {{&S76224[0], 107}, {NULL}};
const Segment SGI296549[] = {{&S92782[37], 6}, {NULL}};
const Segment SGI296550[] = {{&S92782[8], 3}, {NULL}};
const Segment SGI296551[] = {{&S92782[13], 7}, {NULL}};
const Segment SGI296552[] = {{&S92782[7], 1}, {NULL}};
const Segment SGI296553[] = {{&S92782[16], 41}, {NULL}};
const Segment SGI296554[] = {{&S92783[37], 6}, {NULL}};
const Segment SGI296555[] = {{&S92783[8], 3}, {NULL}};
const Segment SGI296556[] = {{&S92783[13], 7}, {NULL}};
const Segment SGI296557[] = {{&S92783[7], 1}, {NULL}};
const Segment SGI296558[] = {{&S92783[16], 41}, {NULL}};
const Segment SGI296559[] = {{&S76424[80], 21}, {NULL}};
const Segment SGI296560[] = {{&S76424[0], 122}, {NULL}};
const Segment SGI296561[] = {{&S76425[0], 102}, {NULL}};
const Segment SGI296562[] = {{&S76426[80], 21}, {NULL}};
const Segment SGI296563[] = {{&S76426[0], 122}, {NULL}};
const Segment SGI296564[] = {{&S76427[80], 21}, {NULL}};
const Segment SGI296565[] = {{&S76427[0], 122}, {NULL}};
const Segment SGI296566[] = {{&S76428[0], 80}, {NULL}};
const Segment SGI296567[] = {{&S93004[69], 5}, {NULL}};
const Segment SGI296568[] = {{&S93004[8], 3}, {NULL}};
const Segment SGI296569[] = {{&S93004[13], 9}, {NULL}};
const Segment SGI296570[] = {{&S93004[30], 13}, {NULL}};
const Segment SGI296571[] = {{&S93004[7], 1}, {NULL}};
const Segment SGI296572[] = {{&S76675[0], 76}, {NULL}};
const Segment SGI296573[] = {{&S76676[0], 54}, {NULL}};
const Segment SGI296574[] = {{&S76677[0], 57}, {NULL}};
const Segment SGI296575[] = {{&S76678[0], 62}, {NULL}};
const Segment SGI296576[] = {{&S76679[0], 54}, {NULL}};
const Segment SGI296577[] = {{&S76680[0], 54}, {NULL}};
const Segment SGI296578[] = {{&S76681[0], 54}, {NULL}};
const Segment SGI296579[] = {{&S76682[0], 61}, {NULL}};
const Segment SGI296580[] = {{&S76683[0], 54}, {NULL}};
const Segment SGI296581[] = {{&S76689[0], 92}, {NULL}};
const Segment SGI296582[] = {{&S76871[0], 64}, {NULL}};
const Segment SGI296583[] = {{&S90043[8], 3}, {NULL}};
const Segment SGI296584[] = {{&S90043[37], 5}, {NULL}};
const Segment SGI296585[] = {{&S90043[66], 6}, {NULL}};
const Segment SGI296586[] = {{&S90043[13], 29}, {NULL}};
const Segment SGI296587[] = {{&S90043[60], 5}, {NULL}};
const Segment SGI296588[] = {{&S76978[0], 95}, {NULL}};
const Segment SGI296589[] = {{&S77062[55], 20}, {NULL}};
const Segment SGI296590[] = {{&S77062[0], 95}, {NULL}};
const Segment SGI296591[] = {{&S77063[55], 20}, {NULL}};
const Segment SGI296592[] = {{&S77063[0], 95}, {NULL}};
const Segment SGI296593[] = {{&S77064[55], 20}, {NULL}};
const Segment SGI296594[] = {{&S77064[0], 99}, {NULL}};
const Segment SGI296595[] = {{&S93426[69], 5}, {NULL}};
const Segment SGI296596[] = {{&S93426[8], 3}, {NULL}};
const Segment SGI296597[] = {{&S93426[13], 9}, {NULL}};
const Segment SGI296598[] = {{&S93426[30], 13}, {NULL}};
const Segment SGI296599[] = {{&S93426[7], 1}, {NULL}};
const Segment SGI296600[] = {{&S93542[7], 4}, {NULL}};
const Segment SGI296601[] = {{&S93542[8], 3}, {NULL}};
const Segment SGI296602[] = {{&S93542[13], 14}, {NULL}};
const Segment SGI296603[] = {{&S93542[7], 1}, {NULL}};
const Segment SGI296604[] = {{&S93542[16], 11}, {NULL}};
const Segment SGI296605[] = {{&S93542[64], 8}, {NULL}};
const Segment SGI296606[] = {{&S93548[54], 4}, {NULL}};
const Segment SGI296607[] = {{&S93548[8], 3}, {NULL}};
const Segment SGI296608[] = {{&S93548[13], 14}, {NULL}};
const Segment SGI296609[] = {{&S93548[7], 1}, {NULL}};
const Segment SGI296610[] = {{&S93548[16], 11}, {NULL}};
const Segment SGI296611[] = {{&S93548[61], 13}, {NULL}};
const Segment SGI296612[] = {{&S77179[0], 78}, {NULL}};
const Segment SGI296613[] = {{&S77180[0], 99}, {NULL}};
const Segment SGI296614[] = {{&S77181[80], 9}, {NULL}};
const Segment SGI296615[] = {{&S77181[0], 129}, {NULL}};
const Segment SGI296616[] = {{&S77182[0], 84}, {NULL}};
const Segment SGI296617[] = {{&S77183[0], 71}, {NULL}};
const Segment SGI296618[] = {{&S77185[0], 70}, {NULL}};
const Segment SGI296619[] = {{&S77186[79], 5}, {NULL}};
const Segment SGI296620[] = {{&S77186[0], 105}, {NULL}};
const Segment SGI296621[] = {{&S77187[0], 122}, {NULL}};
const Segment SGI296622[] = {{&S77188[0], 98}, {NULL}};
const Segment SGI296623[] = {{&S77189[79], 9}, {NULL}};
const Segment SGI296624[] = {{&S77189[0], 128}, {NULL}};
const Segment SGI296625[] = {{&S77191[0], 85}, {NULL}};
const Segment SGI296626[] = {{&S77192[0], 81}, {NULL}};
const Segment SGI296627[] = {{&S77193[9], 7}, {NULL}};
const Segment SGI296628[] = {{&S77193[0], 116}, {NULL}};
const Segment SGI296629[] = {{&S93783[54], 4}, {NULL}};
const Segment SGI296630[] = {{&S93783[8], 3}, {NULL}};
const Segment SGI296631[] = {{&S93783[13], 14}, {NULL}};
const Segment SGI296632[] = {{&S93783[7], 1}, {NULL}};
const Segment SGI296633[] = {{&S93783[16], 11}, {NULL}};
const Segment SGI296634[] = {{&S93783[61], 16}, {NULL}};
const Segment SGI296635[] = {{&S93784[61], 4}, {NULL}};
const Segment SGI296636[] = {{&S93784[8], 3}, {NULL}};
const Segment SGI296637[] = {{&S93784[13], 14}, {NULL}};
const Segment SGI296638[] = {{&S93784[7], 1}, {NULL}};
const Segment SGI296639[] = {{&S93784[16], 11}, {NULL}};
const Segment SGI296640[] = {{&S93784[68], 13}, {NULL}};
const Segment SGI296641[] = {{&S93799[70], 4}, {NULL}};
const Segment SGI296642[] = {{&S93799[8], 3}, {NULL}};
const Segment SGI296643[] = {{&S93799[13], 9}, {NULL}};
const Segment SGI296644[] = {{&S93799[30], 13}, {NULL}};
const Segment SGI296645[] = {{&S93799[7], 1}, {NULL}};
const Segment SGI296646[] = {{&S93799[77], 13}, {NULL}};
const Segment SGI296647[] = {{&S93891[67], 5}, {NULL}};
const Segment SGI296648[] = {{&S93891[8], 3}, {NULL}};
const Segment SGI296649[] = {{&S93891[13], 9}, {NULL}};
const Segment SGI296650[] = {{&S93891[28], 13}, {NULL}};
const Segment SGI296651[] = {{&S93891[7], 1}, {NULL}};
const Segment SGI296652[] = {{&S94036[75], 4}, {NULL}};
const Segment SGI296653[] = {{&S94036[8], 3}, {NULL}};
const Segment SGI296654[] = {{&S94036[13], 7}, {NULL}};
const Segment SGI296655[] = {{&S94036[25], 14}, {NULL}};
const Segment SGI296656[] = {{&S94036[7], 1}, {NULL}};
const Segment SGI296657[] = {{&S94036[16], 4}, {&S94036[28], 11}, {NULL}};
const Segment SGI296658[] = {{&S94036[84], 9}, {NULL}};
const Segment SGI296659[] = {{&S94037[64], 4}, {NULL}};
const Segment SGI296660[] = {{&S94037[8], 3}, {NULL}};
const Segment SGI296661[] = {{&S94037[13], 14}, {NULL}};
const Segment SGI296662[] = {{&S94037[7], 1}, {NULL}};
const Segment SGI296663[] = {{&S94037[16], 11}, {NULL}};
const Segment SGI296664[] = {{&S94038[69], 4}, {NULL}};
const Segment SGI296665[] = {{&S94038[8], 3}, {NULL}};
const Segment SGI296666[] = {{&S94038[13], 14}, {NULL}};
const Segment SGI296667[] = {{&S94038[7], 1}, {NULL}};
const Segment SGI296668[] = {{&S94038[16], 11}, {&S94038[32], 2}, {NULL}};
const Segment SGI296669[] = {{&S94050[60], 5}, {NULL}};
const Segment SGI296670[] = {{&S94050[8], 3}, {NULL}};
const Segment SGI296671[] = {{&S94050[13], 14}, {NULL}};
const Segment SGI296672[] = {{&S94050[7], 1}, {NULL}};
const Segment SGI296673[] = {{&S94050[16], 11}, {NULL}};
const Segment SGI296674[] = {{&S94050[66], 8}, {NULL}};
const Segment SGI296675[] = {{&S77703[43], 15}, {NULL}};
const Segment SGI296676[] = {{&S77703[0], 103}, {NULL}};
const Segment SGI296677[] = {{&S94226[69], 5}, {NULL}};
const Segment SGI296678[] = {{&S94226[8], 3}, {NULL}};
const Segment SGI296679[] = {{&S94226[13], 9}, {NULL}};
const Segment SGI296680[] = {{&S94226[30], 13}, {NULL}};
const Segment SGI296681[] = {{&S94226[7], 1}, {NULL}};
const Segment SGI296682[] = {{&S94226[77], 13}, {NULL}};
const Segment SGI296683[] = {{&S77805[0], 59}, {NULL}};
const Segment SGI296684[] = {{&S77806[0], 54}, {NULL}};
const Segment SGI296685[] = {{&S77807[0], 54}, {NULL}};
const Segment SGI296686[] = {{&S77808[0], 58}, {NULL}};
const Segment SGI296687[] = {{&S77809[0], 60}, {NULL}};
const Segment SGI296688[] = {{&S77810[0], 60}, {NULL}};
const Segment SGI296689[] = {{&S77811[0], 59}, {NULL}};
const Segment SGI296690[] = {{&S77812[0], 59}, {NULL}};
const Segment SGI296691[] = {{&S77813[0], 54}, {NULL}};
const Segment SGI296692[] = {{&S77814[0], 61}, {NULL}};
const Segment SGI296693[] = {{&S94311[7], 4}, {NULL}};
const Segment SGI296694[] = {{&S94311[8], 3}, {NULL}};
const Segment SGI296695[] = {{&S94311[13], 9}, {NULL}};
const Segment SGI296696[] = {{&S94311[30], 13}, {NULL}};
const Segment SGI296697[] = {{&S94311[7], 1}, {NULL}};
const Segment SGI296698[] = {{&S94311[73], 13}, {NULL}};
const Segment SGI296699[] = {{&S77879[0], 92}, {NULL}};
const Segment SGI296700[] = {{&S78169[33], 8}, {NULL}};
const Segment SGI296701[] = {{&S78169[0], 74}, {NULL}};
const Segment SGI296702[] = {{&S78170[33], 8}, {NULL}};
const Segment SGI296703[] = {{&S78170[0], 73}, {NULL}};
const Segment SGI296704[] = {{&S78171[33], 8}, {NULL}};
const Segment SGI296705[] = {{&S78171[0], 75}, {NULL}};
const Segment SGI296706[] = {{&S78172[33], 8}, {NULL}};
const Segment SGI296707[] = {{&S78172[0], 74}, {NULL}};
const Segment SGI296708[] = {{&S78173[33], 8}, {NULL}};
const Segment SGI296709[] = {{&S78173[0], 74}, {NULL}};
const Segment SGI296710[] = {{&S78174[33], 8}, {NULL}};
const Segment SGI296711[] = {{&S78174[0], 74}, {NULL}};
const Segment SGI296712[] = {{&S78175[33], 8}, {NULL}};
const Segment SGI296713[] = {{&S78175[0], 74}, {NULL}};
const Segment SGI296714[] = {{&S78176[33], 8}, {NULL}};
const Segment SGI296715[] = {{&S78176[0], 74}, {NULL}};
const Segment SGI296716[] = {{&S78177[33], 8}, {NULL}};
const Segment SGI296717[] = {{&S78177[0], 73}, {NULL}};
const Segment SGI296718[] = {{&S78178[33], 8}, {NULL}};
const Segment SGI296719[] = {{&S78178[0], 73}, {NULL}};
const Segment SGI296720[] = {{&S94679[63], 4}, {NULL}};
const Segment SGI296721[] = {{&S94679[8], 3}, {NULL}};
const Segment SGI296722[] = {{&S94679[13], 14}, {NULL}};
const Segment SGI296723[] = {{&S94679[7], 1}, {NULL}};
const Segment SGI296724[] = {{&S94679[16], 11}, {NULL}};
const Segment SGI296725[] = {{&S94679[70], 11}, {NULL}};
const Segment SGI296726[] = {{&S94729[67], 5}, {NULL}};
const Segment SGI296727[] = {{&S94729[8], 3}, {NULL}};
const Segment SGI296728[] = {{&S94729[13], 9}, {NULL}};
const Segment SGI296729[] = {{&S94729[28], 13}, {NULL}};
const Segment SGI296730[] = {{&S94729[7], 1}, {NULL}};
const Segment SGI296731[] = {{&S94741[76], 4}, {NULL}};
const Segment SGI296732[] = {{&S94741[8], 3}, {NULL}};
const Segment SGI296733[] = {{&S94741[13], 7}, {NULL}};
const Segment SGI296734[] = {{&S94741[25], 14}, {NULL}};
const Segment SGI296735[] = {{&S94741[7], 1}, {NULL}};
const Segment SGI296736[] = {{&S94741[16], 4}, {&S94741[28], 11}, {NULL}};
const Segment SGI296737[] = {{&S94741[85], 12}, {NULL}};
const Segment SGI296738[] = {{&S78435[0], 67}, {NULL}};
const Segment SGI296739[] = {{&S78436[0], 64}, {NULL}};
const Segment SGI296740[] = {{&S78437[0], 67}, {NULL}};
const Segment SGI296741[] = {{&S78438[0], 64}, {NULL}};
const Segment SGI296742[] = {{&S78439[0], 67}, {NULL}};
const Segment SGI296743[] = {{&S78440[0], 73}, {NULL}};
const Segment SGI296744[] = {{&S78441[0], 73}, {NULL}};
const Segment SGI296745[] = {{&S78442[0], 64}, {NULL}};
const Segment SGI296746[] = {{&S78443[0], 73}, {NULL}};
const Segment SGI296747[] = {{&S78478[80], 5}, {NULL}};
const Segment SGI296748[] = {{&S78478[0], 106}, {NULL}};
const Segment SGI296749[] = {{&S78479[18], 9}, {NULL}};
const Segment SGI296750[] = {{&S78479[0], 129}, {NULL}};
const Segment SGI296751[] = {{&S78480[80], 5}, {NULL}};
const Segment SGI296752[] = {{&S78480[0], 105}, {NULL}};
const Segment SGI296753[] = {{&S78481[0], 99}, {NULL}};
const Segment SGI296754[] = {{&S78482[80], 5}, {NULL}};
const Segment SGI296755[] = {{&S78482[0], 105}, {NULL}};
const Segment SGI296756[] = {{&S94851[69], 5}, {NULL}};
const Segment SGI296757[] = {{&S94851[8], 3}, {NULL}};
const Segment SGI296758[] = {{&S94851[13], 9}, {NULL}};
const Segment SGI296759[] = {{&S94851[30], 13}, {NULL}};
const Segment SGI296760[] = {{&S94851[7], 1}, {NULL}};
const Segment SGI296761[] = {{&S78531[0], 66}, {NULL}};
const Segment SGI296762[] = {{&S78532[0], 66}, {NULL}};
const Segment SGI296763[] = {{&S78533[0], 66}, {NULL}};
const Segment SGI296764[] = {{&S94901[69], 5}, {NULL}};
const Segment SGI296765[] = {{&S94901[8], 3}, {NULL}};
const Segment SGI296766[] = {{&S94901[13], 9}, {NULL}};
const Segment SGI296767[] = {{&S94901[30], 13}, {NULL}};
const Segment SGI296768[] = {{&S94901[7], 1}, {NULL}};
const Segment SGI296769[] = {{&S94901[75], 13}, {NULL}};
const Segment SGI296770[] = {{&S94925[53], 5}, {NULL}};
const Segment SGI296771[] = {{&S94925[8], 3}, {NULL}};
const Segment SGI296772[] = {{&S94925[13], 14}, {NULL}};
const Segment SGI296773[] = {{&S94925[7], 1}, {NULL}};
const Segment SGI296774[] = {{&S94925[16], 11}, {NULL}};
const Segment SGI296775[] = {{&S94925[63], 14}, {NULL}};
const Segment SGI296776[] = {{&S78719[4], 20}, {NULL}};
const Segment SGI296777[] = {{&S78719[0], 139}, {NULL}};
const Segment SGI296778[] = {{&S78720[0], 128}, {NULL}};
const Segment SGI296779[] = {{&S78721[0], 102}, {NULL}};
const Segment SGI296780[] = {{&S78722[0], 128}, {NULL}};
const Segment SGI296781[] = {{&S78723[0], 128}, {NULL}};
const Segment SGI296782[] = {{&S78724[96], 27}, {NULL}};
const Segment SGI296783[] = {{&S78724[0], 144}, {NULL}};
const Segment SGI296784[] = {{&S78784[43], 9}, {NULL}};
const Segment SGI296785[] = {{&S78784[0], 97}, {NULL}};
const Segment SGI296786[] = {{&S78803[0], 63}, {NULL}};
const Segment SGI296787[] = {{&S95008[90], 4}, {NULL}};
const Segment SGI296788[] = {{&S95008[8], 3}, {NULL}};
const Segment SGI296789[] = {{&S95008[13], 3}, {NULL}};
const Segment SGI296790[] = {{&S95008[13], 3}, {&S95008[62], 6}, {NULL}};
const Segment SGI296791[] = {{&S95008[62], 12}, {NULL}};
const Segment SGI296792[] = {{&S95008[7], 1}, {NULL}};
const Segment SGI296793[] = {{&S78890[33], 8}, {NULL}};
const Segment SGI296794[] = {{&S78890[0], 73}, {NULL}};
const Segment SGI296795[] = {{&S78891[33], 8}, {NULL}};
const Segment SGI296796[] = {{&S78891[0], 73}, {NULL}};
const Segment SGI296797[] = {{&S78892[33], 8}, {NULL}};
const Segment SGI296798[] = {{&S78892[0], 73}, {NULL}};
const Segment SGI296799[] = {{&S78926[0], 71}, {NULL}};
const Segment SGI296800[] = {{&S78974[0], 71}, {NULL}};
const Segment SGI296801[] = {{&S79076[0], 51}, {NULL}};
const Segment SGI296802[] = {{&S79077[0], 51}, {NULL}};
const Segment SGI296803[] = {{&S95165[76], 4}, {NULL}};
const Segment SGI296804[] = {{&S95165[8], 3}, {NULL}};
const Segment SGI296805[] = {{&S95165[13], 7}, {NULL}};
const Segment SGI296806[] = {{&S95165[25], 14}, {NULL}};
const Segment SGI296807[] = {{&S95165[7], 1}, {NULL}};
const Segment SGI296808[] = {{&S95165[16], 4}, {&S95165[28], 11}, {NULL}};
const Segment SGI296809[] = {{&S95165[84], 9}, {NULL}};
const Segment SGI296810[] = {{&S79188[0], 82}, {NULL}};
const Segment SGI296811[] = {{&S79189[0], 93}, {NULL}};
const Segment SGI296812[] = {{&S92152[8], 3}, {NULL}};
const Segment SGI296813[] = {{&S92152[43], 5}, {NULL}};
const Segment SGI296814[] = {{&S92152[72], 6}, {NULL}};
const Segment SGI296815[] = {{&S92152[13], 35}, {NULL}};
const Segment SGI296816[] = {{&S92152[66], 5}, {NULL}};
const Segment SGI296817[] = {{&S79236[0], 82}, {NULL}};
const Segment SGI296818[] = {{&S79408[0], 111}, {NULL}};
const Segment SGI296819[] = {{&S79409[0], 91}, {NULL}};
const Segment SGI296820[] = {{&S79410[0], 111}, {NULL}};
const Segment SGI296821[] = {{&S95400[83], 4}, {NULL}};
const Segment SGI296822[] = {{&S95400[8], 3}, {NULL}};
const Segment SGI296823[] = {{&S95400[13], 9}, {NULL}};
const Segment SGI296824[] = {{&S95400[33], 13}, {NULL}};
const Segment SGI296825[] = {{&S95400[7], 1}, {NULL}};
const Segment SGI296826[] = {{&S95400[91], 9}, {NULL}};
const Segment SGI296827[] = {{&S95414[7], 4}, {NULL}};
const Segment SGI296828[] = {{&S95414[8], 3}, {NULL}};
const Segment SGI296829[] = {{&S95414[13], 9}, {NULL}};
const Segment SGI296830[] = {{&S95414[30], 13}, {NULL}};
const Segment SGI296831[] = {{&S95414[7], 1}, {NULL}};
const Segment SGI296832[] = {{&S95414[73], 13}, {NULL}};
const Segment SGI296833[] = {{&S79616[0], 58}, {NULL}};
const Segment SGI296834[] = {{&S79617[0], 56}, {NULL}};
const Segment SGI296835[] = {{&S79618[0], 58}, {NULL}};
const Segment SGI296836[] = {{&S79649[43], 16}, {NULL}};
const Segment SGI296837[] = {{&S79649[0], 104}, {NULL}};
const Segment SGI296838[] = {{&S95640[61], 4}, {NULL}};
const Segment SGI296839[] = {{&S95640[8], 3}, {NULL}};
const Segment SGI296840[] = {{&S95640[13], 14}, {NULL}};
const Segment SGI296841[] = {{&S95640[7], 1}, {NULL}};
const Segment SGI296842[] = {{&S95640[16], 11}, {NULL}};
const Segment SGI296843[] = {{&S95640[68], 13}, {NULL}};
const Segment SGI296844[] = {{&S79750[0], 71}, {NULL}};
const Segment SGI296845[] = {{&S95701[85], 4}, {NULL}};
const Segment SGI296846[] = {{&S95701[8], 3}, {NULL}};
const Segment SGI296847[] = {{&S95701[13], 7}, {NULL}};
const Segment SGI296848[] = {{&S95701[25], 14}, {NULL}};
const Segment SGI296849[] = {{&S95701[7], 1}, {NULL}};
const Segment SGI296850[] = {{&S95701[16], 4}, {&S95701[28], 11}, {NULL}};
const Segment SGI296851[] = {{&S95701[69], 8}, {NULL}};
const Segment SGI296852[] = {{&S95703[54], 4}, {NULL}};
const Segment SGI296853[] = {{&S95703[8], 3}, {NULL}};
const Segment SGI296854[] = {{&S95703[13], 14}, {NULL}};
const Segment SGI296855[] = {{&S95703[7], 1}, {NULL}};
const Segment SGI296856[] = {{&S95703[16], 11}, {NULL}};
const Segment SGI296857[] = {{&S95703[61], 13}, {NULL}};
const Segment SGI296858[] = {{&S95706[7], 4}, {NULL}};
const Segment SGI296859[] = {{&S95706[8], 3}, {NULL}};
const Segment SGI296860[] = {{&S95706[13], 14}, {NULL}};
const Segment SGI296861[] = {{&S95706[7], 1}, {NULL}};
const Segment SGI296862[] = {{&S95706[16], 11}, {NULL}};
const Segment SGI296863[] = {{&S95709[65], 5}, {NULL}};
const Segment SGI296864[] = {{&S95709[8], 3}, {NULL}};
const Segment SGI296865[] = {{&S95709[13], 14}, {NULL}};
const Segment SGI296866[] = {{&S95709[7], 1}, {NULL}};
const Segment SGI296867[] = {{&S95709[16], 11}, {&S95709[32], 2}, {NULL}};
const Segment SGI296868[] = {{&S95709[77], 13}, {NULL}};
const Segment SGI296869[] = {{&S95712[80], 4}, {NULL}};
const Segment SGI296870[] = {{&S95712[8], 3}, {NULL}};
const Segment SGI296871[] = {{&S95712[13], 9}, {NULL}};
const Segment SGI296872[] = {{&S95712[31], 13}, {NULL}};
const Segment SGI296873[] = {{&S95712[7], 1}, {NULL}};
const Segment SGI296874[] = {{&S80054[0], 66}, {NULL}};
const Segment SGI296875[] = {{&S80055[0], 70}, {NULL}};
const Segment SGI296876[] = {{&S80056[0], 70}, {NULL}};
const Segment SGI296877[] = {{&S80057[0], 70}, {NULL}};
const Segment SGI296878[] = {{&S80058[0], 70}, {NULL}};
const Segment SGI296879[] = {{&S80122[0], 58}, {NULL}};
const Segment SGI296880[] = {{&S80123[0], 58}, {NULL}};
const Segment SGI296881[] = {{&S80124[0], 58}, {NULL}};
const Segment SGI296882[] = {{&S80125[0], 58}, {NULL}};
const Segment SGI296883[] = {{&S80126[0], 58}, {NULL}};
const Segment SGI296884[] = {{&S95879[69], 5}, {NULL}};
const Segment SGI296885[] = {{&S95879[8], 3}, {NULL}};
const Segment SGI296886[] = {{&S95879[13], 9}, {NULL}};
const Segment SGI296887[] = {{&S95879[30], 13}, {NULL}};
const Segment SGI296888[] = {{&S95879[7], 1}, {NULL}};
const Segment SGI296889[] = {{&S95879[75], 13}, {NULL}};
const Segment SGI296890[] = {{&S80262[0], 87}, {NULL}};
const Segment SGI296891[] = {{&S95996[69], 5}, {NULL}};
const Segment SGI296892[] = {{&S95996[8], 3}, {NULL}};
const Segment SGI296893[] = {{&S95996[13], 9}, {NULL}};
const Segment SGI296894[] = {{&S95996[30], 13}, {NULL}};
const Segment SGI296895[] = {{&S95996[7], 1}, {NULL}};
const Segment SGI296896[] = {{&S95996[75], 13}, {NULL}};
const Segment SGI296897[] = {{&S95998[68], 4}, {NULL}};
const Segment SGI296898[] = {{&S95998[8], 3}, {NULL}};
const Segment SGI296899[] = {{&S95998[13], 9}, {NULL}};
const Segment SGI296900[] = {{&S95998[30], 13}, {NULL}};
const Segment SGI296901[] = {{&S95998[7], 1}, {NULL}};
const Segment SGI296902[] = {{&S95998[73], 13}, {NULL}};
const Segment SGI296903[] = {{&S96032[53], 5}, {NULL}};
const Segment SGI296904[] = {{&S96032[8], 3}, {NULL}};
const Segment SGI296905[] = {{&S96032[13], 14}, {NULL}};
const Segment SGI296906[] = {{&S96032[7], 1}, {NULL}};
const Segment SGI296907[] = {{&S96032[16], 11}, {NULL}};
const Segment SGI296908[] = {{&S96032[59], 8}, {NULL}};
const Segment SGI296909[] = {{&S80395[0], 85}, {NULL}};
const Segment SGI296910[] = {{&S80396[0], 82}, {NULL}};
const Segment SGI296911[] = {{&S80397[0], 75}, {NULL}};
const Segment SGI296912[] = {{&S96128[57], 5}, {NULL}};
const Segment SGI296913[] = {{&S96128[8], 3}, {NULL}};
const Segment SGI296914[] = {{&S96128[13], 3}, {NULL}};
const Segment SGI296915[] = {{&S96128[13], 3}, {&S96128[18], 7}, {NULL}};
const Segment SGI296916[] = {{&S96128[7], 1}, {NULL}};
const Segment SGI296917[] = {{&S96128[63], 13}, {NULL}};
const Segment SGI296918[] = {{&S80554[43], 8}, {NULL}};
const Segment SGI296919[] = {{&S80554[0], 103}, {NULL}};
const Segment SGI296920[] = {{&S80555[43], 8}, {NULL}};
const Segment SGI296921[] = {{&S80555[0], 103}, {NULL}};
const Segment SGI296922[] = {{&S96194[69], 5}, {NULL}};
const Segment SGI296923[] = {{&S96194[8], 3}, {NULL}};
const Segment SGI296924[] = {{&S96194[13], 9}, {NULL}};
const Segment SGI296925[] = {{&S96194[30], 13}, {NULL}};
const Segment SGI296926[] = {{&S96194[7], 1}, {NULL}};
const Segment SGI296927[] = {{&S80731[0], 82}, {NULL}};
const Segment SGI296928[] = {{&S80732[0], 93}, {NULL}};
const Segment SGI296929[] = {{&S96334[102], 4}, {NULL}};
const Segment SGI296930[] = {{&S96334[8], 3}, {NULL}};
const Segment SGI296931[] = {{&S96334[13], 9}, {NULL}};
const Segment SGI296932[] = {{&S96334[33], 13}, {NULL}};
const Segment SGI296933[] = {{&S96334[7], 1}, {NULL}};
const Segment SGI296934[] = {{&S96334[109], 8}, {NULL}};
const Segment SGI296935[] = {{&S96337[55], 4}, {NULL}};
const Segment SGI296936[] = {{&S96337[8], 3}, {NULL}};
const Segment SGI296937[] = {{&S96337[13], 3}, {NULL}};
const Segment SGI296938[] = {{&S96337[13], 3}, {&S96337[18], 7}, {NULL}};
const Segment SGI296939[] = {{&S96337[7], 1}, {NULL}};
const Segment SGI296940[] = {{&S80817[33], 15}, {NULL}};
const Segment SGI296941[] = {{&S80817[0], 81}, {NULL}};
const Segment SGI296942[] = {{&S80818[33], 15}, {NULL}};
const Segment SGI296943[] = {{&S80818[0], 81}, {NULL}};
const Segment SGI296944[] = {{&S80819[33], 15}, {NULL}};
const Segment SGI296945[] = {{&S80819[0], 80}, {NULL}};
const Segment SGI296946[] = {{&S80820[33], 15}, {NULL}};
const Segment SGI296947[] = {{&S80820[0], 80}, {NULL}};
const Segment SGI296948[] = {{&S80821[33], 15}, {NULL}};
const Segment SGI296949[] = {{&S80821[0], 80}, {NULL}};
const Segment SGI296950[] = {{&S80822[33], 15}, {NULL}};
const Segment SGI296951[] = {{&S80822[0], 81}, {NULL}};
const Segment SGI296952[] = {{&S80892[33], 5}, {NULL}};
const Segment SGI296953[] = {{&S80892[0], 70}, {NULL}};
const Segment SGI296954[] = {{&S80893[33], 5}, {NULL}};
const Segment SGI296955[] = {{&S80893[0], 70}, {NULL}};
const Segment SGI296956[] = {{&S80894[33], 5}, {NULL}};
const Segment SGI296957[] = {{&S80894[0], 70}, {NULL}};
const Segment SGI296958[] = {{&S96481[57], 5}, {NULL}};
const Segment SGI296959[] = {{&S96481[8], 3}, {NULL}};
const Segment SGI296960[] = {{&S96481[13], 3}, {NULL}};
const Segment SGI296961[] = {{&S96481[13], 3}, {&S96481[18], 7}, {NULL}};
const Segment SGI296962[] = {{&S96481[7], 1}, {NULL}};
const Segment SGI296963[] = {{&S96496[71], 4}, {NULL}};
const Segment SGI296964[] = {{&S96496[8], 3}, {NULL}};
const Segment SGI296965[] = {{&S96496[13], 3}, {NULL}};
const Segment SGI296966[] = {{&S96496[7], 1}, {NULL}};
const Segment SGI296967[] = {{&S96496[21], 10}, {NULL}};
const Segment SGI296968[] = {{&S96542[69], 5}, {NULL}};
const Segment SGI296969[] = {{&S96542[8], 3}, {NULL}};
const Segment SGI296970[] = {{&S96542[13], 9}, {NULL}};
const Segment SGI296971[] = {{&S96542[30], 13}, {NULL}};
const Segment SGI296972[] = {{&S96542[7], 1}, {NULL}};
const Segment SGI296973[] = {{&S96624[82], 4}, {NULL}};
const Segment SGI296974[] = {{&S96624[8], 3}, {NULL}};
const Segment SGI296975[] = {{&S96624[13], 9}, {NULL}};
const Segment SGI296976[] = {{&S96624[33], 13}, {NULL}};
const Segment SGI296977[] = {{&S96624[7], 1}, {NULL}};
const Segment SGI296978[] = {{&S96624[89], 8}, {NULL}};
const Segment SGI296979[] = {{&S96629[89], 4}, {NULL}};
const Segment SGI296980[] = {{&S96629[8], 3}, {NULL}};
const Segment SGI296981[] = {{&S96629[13], 3}, {NULL}};
const Segment SGI296982[] = {{&S96629[13], 3}, {&S96629[61], 6}, {NULL}};
const Segment SGI296983[] = {{&S96629[61], 12}, {NULL}};
const Segment SGI296984[] = {{&S96629[7], 1}, {NULL}};
const Segment SGI296985[] = {{&S96651[7], 4}, {NULL}};
const Segment SGI296986[] = {{&S96651[8], 3}, {NULL}};
const Segment SGI296987[] = {{&S96651[13], 14}, {NULL}};
const Segment SGI296988[] = {{&S96651[7], 1}, {NULL}};
const Segment SGI296989[] = {{&S96651[16], 11}, {NULL}};
const Segment SGI296990[] = {{&S96651[57], 16}, {NULL}};
const Segment SGI296991[] = {{&S96715[79], 4}, {NULL}};
const Segment SGI296992[] = {{&S96715[8], 3}, {NULL}};
const Segment SGI296993[] = {{&S96715[13], 3}, {NULL}};
const Segment SGI296994[] = {{&S96715[13], 3}, {&S96715[60], 6}, {NULL}};
const Segment SGI296995[] = {{&S96715[60], 11}, {NULL}};
const Segment SGI296996[] = {{&S96715[7], 1}, {NULL}};
const Segment SGI296997[] = {{&S81383[0], 90}, {NULL}};
const Segment SGI296998[] = {{&S81441[0], 98}, {NULL}};
const Segment SGI296999[] = {{&S81442[0], 98}, {NULL}};
const Segment SGI297000[] = {{&S81443[0], 70}, {NULL}};
const Segment SGI297001[] = {{&S96966[71], 5}, {NULL}};
const Segment SGI297002[] = {{&S96966[8], 3}, {NULL}};
const Segment SGI297003[] = {{&S96966[13], 9}, {NULL}};
const Segment SGI297004[] = {{&S96966[30], 13}, {NULL}};
const Segment SGI297005[] = {{&S96966[7], 1}, {NULL}};
const Segment SGI297006[] = {{&S96966[79], 13}, {NULL}};
const Segment SGI297007[] = {{&S96973[54], 4}, {NULL}};
const Segment SGI297008[] = {{&S96973[8], 3}, {NULL}};
const Segment SGI297009[] = {{&S96973[13], 14}, {NULL}};
const Segment SGI297010[] = {{&S96973[7], 1}, {NULL}};
const Segment SGI297011[] = {{&S96973[16], 11}, {NULL}};
const Segment SGI297012[] = {{&S96973[61], 8}, {NULL}};
const Segment SGI297013[] = {{&S96975[84], 4}, {NULL}};
const Segment SGI297014[] = {{&S96975[8], 3}, {NULL}};
const Segment SGI297015[] = {{&S96975[13], 7}, {NULL}};
const Segment SGI297016[] = {{&S96975[25], 14}, {NULL}};
const Segment SGI297017[] = {{&S96975[7], 1}, {NULL}};
const Segment SGI297018[] = {{&S96975[16], 4}, {&S96975[28], 11}, {NULL}};
const Segment SGI297019[] = {{&S96975[68], 8}, {NULL}};
const Segment SGI297020[] = {{&S96980[54], 4}, {NULL}};
const Segment SGI297021[] = {{&S96980[8], 3}, {NULL}};
const Segment SGI297022[] = {{&S96980[13], 3}, {NULL}};
const Segment SGI297023[] = {{&S96980[13], 3}, {&S96980[18], 5}, {NULL}};
const Segment SGI297024[] = {{&S96980[7], 1}, {NULL}};
const Segment SGI297025[] = {{&S96981[59], 4}, {NULL}};
const Segment SGI297026[] = {{&S96981[8], 3}, {NULL}};
const Segment SGI297027[] = {{&S96981[13], 14}, {NULL}};
const Segment SGI297028[] = {{&S96981[7], 1}, {NULL}};
const Segment SGI297029[] = {{&S96981[16], 11}, {NULL}};
const Segment SGI297030[] = {{&S96981[64], 13}, {NULL}};
const Segment SGI297031[] = {{&S96997[70], 4}, {NULL}};
const Segment SGI297032[] = {{&S96997[8], 3}, {NULL}};
const Segment SGI297033[] = {{&S96997[13], 7}, {NULL}};
const Segment SGI297034[] = {{&S96997[7], 1}, {NULL}};
const Segment SGI297035[] = {{&S96997[16], 4}, {&S96997[28], 5}, {NULL}};
const Segment SGI297036[] = {{&S96997[80], 13}, {NULL}};
const Segment SGI297037[] = {{&S97275[52], 4}, {NULL}};
const Segment SGI297038[] = {{&S97275[8], 3}, {NULL}};
const Segment SGI297039[] = {{&S97275[13], 14}, {NULL}};
const Segment SGI297040[] = {{&S97275[7], 1}, {NULL}};
const Segment SGI297041[] = {{&S97275[16], 11}, {NULL}};
const Segment SGI297042[] = {{&S97275[57], 13}, {NULL}};
const Segment SGI297043[] = {{&S97279[58], 4}, {NULL}};
const Segment SGI297044[] = {{&S97279[8], 3}, {NULL}};
const Segment SGI297045[] = {{&S97279[13], 3}, {NULL}};
const Segment SGI297046[] = {{&S97279[13], 3}, {&S97279[18], 7}, {NULL}};
const Segment SGI297047[] = {{&S97279[7], 1}, {NULL}};
const Segment SGI297048[] = {{&S97283[70], 4}, {NULL}};
const Segment SGI297049[] = {{&S97283[8], 3}, {NULL}};
const Segment SGI297050[] = {{&S97283[13], 9}, {NULL}};
const Segment SGI297051[] = {{&S97283[30], 13}, {NULL}};
const Segment SGI297052[] = {{&S97283[7], 1}, {NULL}};
const Segment SGI297053[] = {{&S97283[77], 13}, {NULL}};
const Segment SGI297054[] = {{&S97288[52], 4}, {NULL}};
const Segment SGI297055[] = {{&S97288[8], 3}, {NULL}};
const Segment SGI297056[] = {{&S97288[13], 14}, {NULL}};
const Segment SGI297057[] = {{&S97288[7], 1}, {NULL}};
const Segment SGI297058[] = {{&S97288[16], 11}, {NULL}};
const Segment SGI297059[] = {{&S97288[57], 12}, {NULL}};
const Segment SGI297060[] = {{&S97289[7], 4}, {NULL}};
const Segment SGI297061[] = {{&S97289[8], 3}, {NULL}};
const Segment SGI297062[] = {{&S97289[13], 3}, {NULL}};
const Segment SGI297063[] = {{&S97289[13], 3}, {&S97289[18], 5}, {NULL}};
const Segment SGI297064[] = {{&S97289[7], 1}, {NULL}};
const Segment SGI297065[] = {{&S97294[76], 4}, {NULL}};
const Segment SGI297066[] = {{&S97294[8], 3}, {NULL}};
const Segment SGI297067[] = {{&S97294[13], 7}, {NULL}};
const Segment SGI297068[] = {{&S97294[25], 14}, {NULL}};
const Segment SGI297069[] = {{&S97294[7], 1}, {NULL}};
const Segment SGI297070[] = {{&S97294[16], 4}, {&S97294[28], 11}, {NULL}};
const Segment SGI297071[] = {{&S97294[84], 9}, {NULL}};
const Segment SGI297072[] = {{&S97300[64], 4}, {NULL}};
const Segment SGI297073[] = {{&S97300[8], 3}, {NULL}};
const Segment SGI297074[] = {{&S97300[13], 14}, {NULL}};
const Segment SGI297075[] = {{&S97300[7], 1}, {NULL}};
const Segment SGI297076[] = {{&S97300[16], 11}, {&S97300[32], 2}, {NULL}};
const Segment SGI297077[] = {{&S97300[73], 12}, {NULL}};
const Segment SGI297078[] = {{&S97319[53], 5}, {NULL}};
const Segment SGI297079[] = {{&S97319[8], 3}, {NULL}};
const Segment SGI297080[] = {{&S97319[13], 14}, {NULL}};
const Segment SGI297081[] = {{&S97319[7], 1}, {NULL}};
const Segment SGI297082[] = {{&S97319[16], 11}, {NULL}};
const Segment SGI297083[] = {{&S97319[63], 14}, {NULL}};
const Segment SGI297084[] = {{&S81786[0], 92}, {NULL}};
const Segment SGI297085[] = {{&S97371[57], 5}, {NULL}};
const Segment SGI297086[] = {{&S97371[8], 3}, {NULL}};
const Segment SGI297087[] = {{&S97371[13], 3}, {NULL}};
const Segment SGI297088[] = {{&S97371[13], 3}, {&S97371[18], 7}, {NULL}};
const Segment SGI297089[] = {{&S97371[7], 1}, {NULL}};
const Segment SGI297090[] = {{&S97371[63], 13}, {NULL}};
const Segment SGI297091[] = {{&S97376[57], 5}, {NULL}};
const Segment SGI297092[] = {{&S97376[8], 3}, {NULL}};
const Segment SGI297093[] = {{&S97376[13], 3}, {NULL}};
const Segment SGI297094[] = {{&S97376[13], 3}, {&S97376[18], 7}, {NULL}};
const Segment SGI297095[] = {{&S97376[7], 1}, {NULL}};
const Segment SGI297096[] = {{&S81849[0], 55}, {NULL}};
const Segment SGI297097[] = {{&S81850[0], 57}, {NULL}};
const Segment SGI297098[] = {{&S81851[0], 58}, {NULL}};
const Segment SGI297099[] = {{&S81852[0], 53}, {NULL}};
const Segment SGI297100[] = {{&S81853[0], 59}, {NULL}};
const Segment SGI297101[] = {{&S81854[0], 59}, {NULL}};
const Segment SGI297102[] = {{&S81855[0], 59}, {NULL}};
const Segment SGI297103[] = {{&S97425[7], 4}, {NULL}};
const Segment SGI297104[] = {{&S97425[8], 3}, {NULL}};
const Segment SGI297105[] = {{&S97425[13], 14}, {NULL}};
const Segment SGI297106[] = {{&S97425[7], 1}, {NULL}};
const Segment SGI297107[] = {{&S97425[16], 11}, {NULL}};
const Segment SGI297108[] = {{&S97425[64], 13}, {NULL}};
const Segment SGI297109[] = {{&S81856[0], 58}, {NULL}};
const Segment SGI297110[] = {{&S81857[0], 53}, {NULL}};
const Segment SGI297111[] = {{&S81858[25], 1}, {NULL}};
const Segment SGI297112[] = {{&S81858[0], 48}, {NULL}};
const Segment SGI297113[] = {{&S81878[0], 15}, {NULL}};
const Segment SGI297114[] = {{&S97475[13], 14}, {NULL}};
const Segment SGI297115[] = {{&S97475[7], 1}, {NULL}};
const Segment SGI297116[] = {{&S97475[16], 11}, {NULL}};
const Segment SGI297117[] = {{&S97476[8], 3}, {NULL}};
const Segment SGI297118[] = {{&S97476[13], 7}, {NULL}};
const Segment SGI297119[] = {{&S97476[25], 14}, {NULL}};
const Segment SGI297120[] = {{&S97476[76], 5}, {NULL}};
const Segment SGI297121[] = {{&S97476[16], 4}, {&S97476[28], 11}, {NULL}};
const Segment SGI297122[] = {{&S97710[77], 4}, {NULL}};
const Segment SGI297123[] = {{&S97710[8], 3}, {NULL}};
const Segment SGI297124[] = {{&S97710[13], 9}, {NULL}};
const Segment SGI297125[] = {{&S97710[7], 1}, {NULL}};
const Segment SGI297126[] = {{&S97756[75], 4}, {NULL}};
const Segment SGI297127[] = {{&S97756[8], 3}, {NULL}};
const Segment SGI297128[] = {{&S97756[13], 7}, {NULL}};
const Segment SGI297129[] = {{&S97756[25], 14}, {NULL}};
const Segment SGI297130[] = {{&S97756[7], 1}, {NULL}};
const Segment SGI297131[] = {{&S97756[16], 4}, {&S97756[28], 11}, {NULL}};
const Segment SGI297132[] = {{&S97756[84], 9}, {NULL}};
const Segment SGI297133[] = {{&S82224[43], 13}, {NULL}};
const Segment SGI297134[] = {{&S82224[0], 101}, {NULL}};
const Segment SGI297135[] = {{&S82225[43], 13}, {NULL}};
const Segment SGI297136[] = {{&S82225[0], 101}, {NULL}};
const Segment SGI297137[] = {{&S97897[69], 5}, {NULL}};
const Segment SGI297138[] = {{&S97897[8], 3}, {NULL}};
const Segment SGI297139[] = {{&S97897[13], 9}, {NULL}};
const Segment SGI297140[] = {{&S97897[30], 13}, {NULL}};
const Segment SGI297141[] = {{&S97897[7], 1}, {NULL}};
const Segment SGI297142[] = {{&S82260[0], 85}, {NULL}};
const Segment SGI297143[] = {{&S82261[0], 89}, {NULL}};
const Segment SGI297144[] = {{&S82501[43], 15}, {NULL}};
const Segment SGI297145[] = {{&S82501[0], 103}, {NULL}};
const Segment SGI297146[] = {{&S82502[43], 15}, {NULL}};
const Segment SGI297147[] = {{&S82502[0], 103}, {NULL}};
const Segment SGI297148[] = {{&S82528[18], 9}, {NULL}};
const Segment SGI297149[] = {{&S82528[0], 139}, {NULL}};
const Segment SGI297150[] = {{&S98109[73], 4}, {NULL}};
const Segment SGI297151[] = {{&S98109[8], 3}, {NULL}};
const Segment SGI297152[] = {{&S98109[13], 7}, {NULL}};
const Segment SGI297153[] = {{&S98109[25], 14}, {NULL}};
const Segment SGI297154[] = {{&S98109[7], 1}, {NULL}};
const Segment SGI297155[] = {{&S98109[16], 4}, {&S98109[28], 11}, {NULL}};
const Segment SGI297156[] = {{&S98114[8], 3}, {NULL}};
const Segment SGI297157[] = {{&S98114[13], 7}, {NULL}};
const Segment SGI297158[] = {{&S98114[25], 14}, {NULL}};
const Segment SGI297159[] = {{&S98114[7], 1}, {NULL}};
const Segment SGI297160[] = {{&S98114[16], 4}, {&S98114[28], 11}, {NULL}};
const Segment SGI297161[] = {{&S98114[68], 5}, {NULL}};
const Segment SGI297162[] = {{&S98132[60], 5}, {NULL}};
const Segment SGI297163[] = {{&S98132[8], 3}, {NULL}};
const Segment SGI297164[] = {{&S98132[13], 14}, {NULL}};
const Segment SGI297165[] = {{&S98132[7], 1}, {NULL}};
const Segment SGI297166[] = {{&S98132[16], 11}, {NULL}};
const Segment SGI297167[] = {{&S98132[70], 14}, {NULL}};
const Segment SGI297168[] = {{&S82770[33], 35}, {NULL}};
const Segment SGI297169[] = {{&S82770[0], 100}, {NULL}};
const Segment SGI297170[] = {{&S82771[33], 35}, {NULL}};
const Segment SGI297171[] = {{&S82771[0], 100}, {NULL}};
const Segment SGI297172[] = {{&S82772[33], 35}, {NULL}};
const Segment SGI297173[] = {{&S82772[0], 101}, {NULL}};
const Segment SGI297174[] = {{&S82823[0], 96}, {NULL}};
const Segment SGI297175[] = {{&S82824[0], 103}, {NULL}};
const Segment SGI297176[] = {{&S82854[0], 95}, {NULL}};
const Segment SGI297177[] = {{&S98574[88], 4}, {NULL}};
const Segment SGI297178[] = {{&S98574[8], 3}, {NULL}};
const Segment SGI297179[] = {{&S98574[13], 3}, {NULL}};
const Segment SGI297180[] = {{&S98574[13], 3}, {&S98574[60], 6}, {NULL}};
const Segment SGI297181[] = {{&S98574[60], 11}, {NULL}};
const Segment SGI297182[] = {{&S98574[7], 1}, {NULL}};
const Segment SGI297183[] = {{&S98581[54], 4}, {NULL}};
const Segment SGI297184[] = {{&S98581[8], 3}, {NULL}};
const Segment SGI297185[] = {{&S98581[13], 3}, {NULL}};
const Segment SGI297186[] = {{&S98581[13], 3}, {&S98581[18], 5}, {NULL}};
const Segment SGI297187[] = {{&S98581[7], 1}, {NULL}};
const Segment SGI297188[] = {{&S98608[65], 4}, {NULL}};
const Segment SGI297189[] = {{&S98608[8], 3}, {NULL}};
const Segment SGI297190[] = {{&S98608[13], 3}, {NULL}};
const Segment SGI297191[] = {{&S98608[13], 3}, {&S98608[21], 5}, {NULL}};
const Segment SGI297192[] = {{&S98608[7], 1}, {NULL}};
const Segment SGI297193[] = {{&S98614[83], 4}, {NULL}};
const Segment SGI297194[] = {{&S98614[8], 3}, {NULL}};
const Segment SGI297195[] = {{&S98614[13], 9}, {NULL}};
const Segment SGI297196[] = {{&S98614[33], 13}, {NULL}};
const Segment SGI297197[] = {{&S98614[7], 1}, {NULL}};
const Segment SGI297198[] = {{&S98614[91], 9}, {NULL}};
const Segment SGI297199[] = {{&S98624[7], 4}, {NULL}};
const Segment SGI297200[] = {{&S98624[8], 3}, {NULL}};
const Segment SGI297201[] = {{&S98624[13], 14}, {NULL}};
const Segment SGI297202[] = {{&S98624[7], 1}, {NULL}};
const Segment SGI297203[] = {{&S98624[16], 11}, {NULL}};
const Segment SGI297204[] = {{&S98624[57], 13}, {NULL}};
const Segment SGI297205[] = {{&S98625[53], 5}, {NULL}};
const Segment SGI297206[] = {{&S98625[8], 3}, {NULL}};
const Segment SGI297207[] = {{&S98625[13], 14}, {NULL}};
const Segment SGI297208[] = {{&S98625[7], 1}, {NULL}};
const Segment SGI297209[] = {{&S98625[16], 11}, {NULL}};
const Segment SGI297210[] = {{&S98625[59], 13}, {NULL}};
const Segment SGI297211[] = {{&S82976[33], 12}, {NULL}};
const Segment SGI297212[] = {{&S82976[0], 78}, {NULL}};
const Segment SGI297213[] = {{&S82977[33], 12}, {NULL}};
const Segment SGI297214[] = {{&S82977[0], 77}, {NULL}};
const Segment SGI297215[] = {{&S82978[33], 12}, {NULL}};
const Segment SGI297216[] = {{&S82978[0], 77}, {NULL}};
const Segment SGI297217[] = {{&S82979[33], 12}, {NULL}};
const Segment SGI297218[] = {{&S82979[0], 78}, {NULL}};
const Segment SGI297219[] = {{&S82980[33], 12}, {NULL}};
const Segment SGI297220[] = {{&S82980[0], 77}, {NULL}};
const Segment SGI297221[] = {{&S82981[33], 12}, {NULL}};
const Segment SGI297222[] = {{&S82981[0], 77}, {NULL}};
const Segment SGI297223[] = {{&S83003[43], 9}, {NULL}};
const Segment SGI297224[] = {{&S83003[0], 97}, {NULL}};
const Segment SGI297225[] = {{&S83004[43], 10}, {NULL}};
const Segment SGI297226[] = {{&S83004[0], 98}, {NULL}};
const Segment SGI297227[] = {{&S83038[78], 8}, {NULL}};
const Segment SGI297228[] = {{&S83038[0], 126}, {NULL}};
const Segment SGI297229[] = {{&S83039[0], 87}, {NULL}};
const Segment SGI297230[] = {{&S83059[33], 5}, {NULL}};
const Segment SGI297231[] = {{&S83059[0], 71}, {NULL}};
const Segment SGI297232[] = {{&S83060[33], 5}, {NULL}};
const Segment SGI297233[] = {{&S83060[0], 71}, {NULL}};
const Segment SGI297234[] = {{&S83061[33], 5}, {NULL}};
const Segment SGI297235[] = {{&S83061[0], 71}, {NULL}};
const Segment SGI297236[] = {{&S83062[33], 5}, {NULL}};
const Segment SGI297237[] = {{&S83062[0], 71}, {NULL}};
const Segment SGI297238[] = {{&S83123[18], 8}, {NULL}};
const Segment SGI297239[] = {{&S83123[0], 128}, {NULL}};
const Segment SGI297240[] = {{&S83124[80], 8}, {NULL}};
const Segment SGI297241[] = {{&S83124[0], 128}, {NULL}};
const Segment SGI297242[] = {{&S98778[67], 5}, {NULL}};
const Segment SGI297243[] = {{&S98778[8], 3}, {NULL}};
const Segment SGI297244[] = {{&S98778[13], 9}, {NULL}};
const Segment SGI297245[] = {{&S98778[28], 13}, {NULL}};
const Segment SGI297246[] = {{&S98778[7], 1}, {NULL}};
const Segment SGI297247[] = {{&S83125[0], 70}, {NULL}};
const Segment SGI297248[] = {{&S83291[89], 4}, {NULL}};
const Segment SGI297249[] = {{&S83291[0], 93}, {NULL}};
const Segment SGI297250[] = {{&S83292[0], 86}, {NULL}};
const Segment SGI297251[] = {{&S83293[0], 91}, {NULL}};
const Segment SGI297252[] = {{&S95415[8], 3}, {NULL}};
const Segment SGI297253[] = {{&S95415[71], 5}, {NULL}};
const Segment SGI297254[] = {{&S95415[64], 6}, {NULL}};
const Segment SGI297255[] = {{&S95415[13], 24}, {NULL}};
const Segment SGI297256[] = {{&S95415[55], 5}, {NULL}};
const Segment SGI297257[] = {{&S83378[0], 54}, {NULL}};
const Segment SGI297258[] = {{&S83379[0], 53}, {NULL}};
const Segment SGI297259[] = {{&S83380[0], 58}, {NULL}};
const Segment SGI297260[] = {{&S83381[0], 58}, {NULL}};
const Segment SGI297261[] = {{&S99074[57], 4}, {NULL}};
const Segment SGI297262[] = {{&S99074[8], 3}, {NULL}};
const Segment SGI297263[] = {{&S99074[13], 14}, {NULL}};
const Segment SGI297264[] = {{&S99074[7], 1}, {NULL}};
const Segment SGI297265[] = {{&S99074[16], 11}, {NULL}};
const Segment SGI297266[] = {{&S99083[75], 4}, {NULL}};
const Segment SGI297267[] = {{&S99083[8], 3}, {NULL}};
const Segment SGI297268[] = {{&S99083[13], 7}, {NULL}};
const Segment SGI297269[] = {{&S99083[25], 14}, {NULL}};
const Segment SGI297270[] = {{&S99083[7], 1}, {NULL}};
const Segment SGI297271[] = {{&S99083[16], 4}, {&S99083[28], 11}, {NULL}};
const Segment SGI297272[] = {{&S99083[84], 9}, {NULL}};
const Segment SGI297273[] = {{&S99084[61], 4}, {NULL}};
const Segment SGI297274[] = {{&S99084[8], 3}, {NULL}};
const Segment SGI297275[] = {{&S99084[13], 14}, {NULL}};
const Segment SGI297276[] = {{&S99084[7], 1}, {NULL}};
const Segment SGI297277[] = {{&S99084[16], 11}, {NULL}};
const Segment SGI297278[] = {{&S99084[68], 12}, {NULL}};
const Segment SGI297279[] = {{&S83497[0], 89}, {NULL}};
const Segment SGI297280[] = {{&S99113[76], 5}, {NULL}};
const Segment SGI297281[] = {{&S99113[8], 3}, {NULL}};
const Segment SGI297282[] = {{&S99113[13], 3}, {NULL}};
const Segment SGI297283[] = {{&S99113[13], 3}, {&S99113[57], 6}, {NULL}};
const Segment SGI297284[] = {{&S99113[57], 11}, {NULL}};
const Segment SGI297285[] = {{&S99113[7], 1}, {NULL}};
const Segment SGI297286[] = {{&S83596[33], 12}, {NULL}};
const Segment SGI297287[] = {{&S83596[0], 77}, {NULL}};
const Segment SGI297288[] = {{&S83597[33], 12}, {NULL}};
const Segment SGI297289[] = {{&S83597[0], 77}, {NULL}};
const Segment SGI297290[] = {{&S83598[33], 12}, {NULL}};
const Segment SGI297291[] = {{&S83598[0], 77}, {NULL}};
const Segment SGI297292[] = {{&S99293[67], 5}, {NULL}};
const Segment SGI297293[] = {{&S99293[8], 3}, {NULL}};
const Segment SGI297294[] = {{&S99293[13], 9}, {NULL}};
const Segment SGI297295[] = {{&S99293[28], 13}, {NULL}};
const Segment SGI297296[] = {{&S99293[7], 1}, {NULL}};
const Segment SGI297297[] = {{&S99293[73], 13}, {NULL}};
const Segment SGI297298[] = {{&S83738[0], 69}, {NULL}};
const Segment SGI297299[] = {{&S83803[65], 6}, {NULL}};
const Segment SGI297300[] = {{&S83803[0], 87}, {NULL}};
const Segment SGI297301[] = {{&S83804[65], 6}, {NULL}};
const Segment SGI297302[] = {{&S83804[0], 87}, {NULL}};
const Segment SGI297303[] = {{&S83805[65], 6}, {NULL}};
const Segment SGI297304[] = {{&S83805[0], 87}, {NULL}};
const Segment SGI297305[] = {{&S83806[65], 6}, {NULL}};
const Segment SGI297306[] = {{&S83806[0], 87}, {NULL}};
const Segment SGI297307[] = {{&S83807[65], 6}, {NULL}};
const Segment SGI297308[] = {{&S83807[0], 87}, {NULL}};
const Segment SGI297309[] = {{&S83808[65], 6}, {NULL}};
const Segment SGI297310[] = {{&S83808[0], 90}, {NULL}};
const Segment SGI297311[] = {{&S83809[65], 6}, {NULL}};
const Segment SGI297312[] = {{&S83809[0], 87}, {NULL}};
const Segment SGI297313[] = {{&S83810[65], 6}, {NULL}};
const Segment SGI297314[] = {{&S83810[0], 90}, {NULL}};
const Segment SGI297315[] = {{&S83811[65], 6}, {NULL}};
const Segment SGI297316[] = {{&S83811[0], 87}, {NULL}};
const Segment SGI297317[] = {{&S83893[0], 57}, {NULL}};
const Segment SGI297318[] = {{&S83894[0], 52}, {NULL}};
const Segment SGI297319[] = {{&S83895[0], 50}, {NULL}};
const Segment SGI297320[] = {{&S83896[0], 54}, {NULL}};
const Segment SGI297321[] = {{&S83897[0], 51}, {NULL}};
const Segment SGI297322[] = {{&S83898[0], 54}, {NULL}};
const Segment SGI297323[] = {{&S83899[0], 54}, {NULL}};
const Segment SGI297324[] = {{&S83900[0], 51}, {NULL}};
const Segment SGI297325[] = {{&S83901[0], 58}, {NULL}};
const Segment SGI297326[] = {{&S83902[0], 56}, {NULL}};
const Segment SGI297327[] = {{&S83903[0], 54}, {NULL}};
const Segment SGI297328[] = {{&S83904[0], 54}, {NULL}};
const Segment SGI297329[] = {{&S83905[0], 56}, {NULL}};
const Segment SGI297330[] = {{&S83906[0], 58}, {NULL}};
const Segment SGI297331[] = {{&S83907[0], 58}, {NULL}};
const Segment SGI297332[] = {{&S99621[83], 4}, {NULL}};
const Segment SGI297333[] = {{&S99621[8], 3}, {NULL}};
const Segment SGI297334[] = {{&S99621[13], 9}, {NULL}};
const Segment SGI297335[] = {{&S99621[33], 13}, {NULL}};
const Segment SGI297336[] = {{&S99621[7], 1}, {NULL}};
const Segment SGI297337[] = {{&S99621[91], 9}, {NULL}};
const Segment SGI297338[] = {{&S99628[88], 4}, {NULL}};
const Segment SGI297339[] = {{&S99628[8], 3}, {NULL}};
const Segment SGI297340[] = {{&S99628[13], 3}, {NULL}};
const Segment SGI297341[] = {{&S99628[13], 3}, {&S99628[60], 6}, {NULL}};
const Segment SGI297342[] = {{&S99628[60], 11}, {NULL}};
const Segment SGI297343[] = {{&S99628[7], 1}, {NULL}};
const Segment SGI297344[] = {{&S99631[66], 4}, {NULL}};
const Segment SGI297345[] = {{&S99631[8], 3}, {NULL}};
const Segment SGI297346[] = {{&S99631[13], 9}, {NULL}};
const Segment SGI297347[] = {{&S99631[28], 13}, {NULL}};
const Segment SGI297348[] = {{&S99631[7], 1}, {NULL}};
const Segment SGI297349[] = {{&S99652[82], 4}, {NULL}};
const Segment SGI297350[] = {{&S99652[8], 3}, {NULL}};
const Segment SGI297351[] = {{&S99652[13], 9}, {NULL}};
const Segment SGI297352[] = {{&S99652[33], 13}, {NULL}};
const Segment SGI297353[] = {{&S99652[7], 1}, {NULL}};
const Segment SGI297354[] = {{&S99691[67], 5}, {NULL}};
const Segment SGI297355[] = {{&S99691[8], 3}, {NULL}};
const Segment SGI297356[] = {{&S99691[13], 9}, {NULL}};
const Segment SGI297357[] = {{&S99691[28], 13}, {NULL}};
const Segment SGI297358[] = {{&S99691[7], 1}, {NULL}};
const Segment SGI297359[] = {{&S96427[8], 3}, {NULL}};
const Segment SGI297360[] = {{&S96427[92], 3}, {NULL}};
const Segment SGI297361[] = {{&S96427[85], 6}, {NULL}};
const Segment SGI297362[] = {{&S96427[13], 45}, {NULL}};
const Segment SGI297363[] = {{&S96427[76], 5}, {NULL}};
const Segment SGI297364[] = {{&S84215[43], 6}, {NULL}};
const Segment SGI297365[] = {{&S84215[0], 94}, {NULL}};
const Segment SGI297366[] = {{&S84265[0], 85}, {NULL}};
const Segment SGI297367[] = {{&S84266[0], 81}, {NULL}};
const Segment SGI297368[] = {{&S84267[0], 81}, {NULL}};
const Segment SGI297369[] = {{&S100041[80], 4}, {NULL}};
const Segment SGI297370[] = {{&S100041[8], 3}, {NULL}};
const Segment SGI297371[] = {{&S100041[13], 3}, {NULL}};
const Segment SGI297372[] = {{&S100041[7], 1}, {NULL}};
const Segment SGI297373[] = {{&S84364[0], 92}, {NULL}};
const Segment SGI297374[] = {{&S84382[0], 55}, {NULL}};
const Segment SGI297375[] = {{&S84383[0], 55}, {NULL}};
const Segment SGI297376[] = {{&S84384[0], 62}, {NULL}};
const Segment SGI297377[] = {{&S84385[0], 52}, {NULL}};
const Segment SGI297378[] = {{&S100169[75], 4}, {NULL}};
const Segment SGI297379[] = {{&S100169[8], 3}, {NULL}};
const Segment SGI297380[] = {{&S100169[13], 7}, {NULL}};
const Segment SGI297381[] = {{&S100169[25], 14}, {NULL}};
const Segment SGI297382[] = {{&S100169[7], 1}, {NULL}};
const Segment SGI297383[] = {{&S100169[16], 4}, {&S100169[28], 11}, {NULL}};
const Segment SGI297384[] = {{&S100170[67], 4}, {NULL}};
const Segment SGI297385[] = {{&S100170[8], 3}, {NULL}};
const Segment SGI297386[] = {{&S100170[13], 7}, {NULL}};
const Segment SGI297387[] = {{&S100170[7], 1}, {NULL}};
const Segment SGI297388[] = {{&S100170[16], 4}, {&S100170[28], 5}, {NULL}};
const Segment SGI297389[] = {{&S100178[61], 4}, {NULL}};
const Segment SGI297390[] = {{&S100178[8], 3}, {NULL}};
const Segment SGI297391[] = {{&S100178[13], 14}, {NULL}};
const Segment SGI297392[] = {{&S100178[7], 1}, {NULL}};
const Segment SGI297393[] = {{&S100178[16], 11}, {NULL}};
const Segment SGI297394[] = {{&S100178[68], 16}, {NULL}};
const Segment SGI297395[] = {{&S100197[7], 4}, {NULL}};
const Segment SGI297396[] = {{&S100197[8], 3}, {NULL}};
const Segment SGI297397[] = {{&S100197[13], 14}, {NULL}};
const Segment SGI297398[] = {{&S100197[7], 1}, {NULL}};
const Segment SGI297399[] = {{&S100197[16], 11}, {NULL}};
const Segment SGI297400[] = {{&S100197[57], 8}, {NULL}};
const Segment SGI297401[] = {{&S100212[54], 4}, {NULL}};
const Segment SGI297402[] = {{&S100212[8], 3}, {NULL}};
const Segment SGI297403[] = {{&S100212[13], 14}, {NULL}};
const Segment SGI297404[] = {{&S100212[7], 1}, {NULL}};
const Segment SGI297405[] = {{&S100212[16], 11}, {NULL}};
const Segment SGI297406[] = {{&S100212[61], 16}, {NULL}};
const Segment SGI297407[] = {{&S100218[82], 4}, {NULL}};
const Segment SGI297408[] = {{&S100218[8], 3}, {NULL}};
const Segment SGI297409[] = {{&S100218[13], 9}, {NULL}};
const Segment SGI297410[] = {{&S100218[33], 13}, {NULL}};
const Segment SGI297411[] = {{&S100218[7], 1}, {NULL}};
const Segment SGI297412[] = {{&S100218[89], 8}, {NULL}};
const Segment SGI297413[] = {{&S84449[43], 9}, {NULL}};
const Segment SGI297414[] = {{&S84449[0], 97}, {NULL}};
const Segment SGI297415[] = {{&S84450[43], 9}, {NULL}};
const Segment SGI297416[] = {{&S84450[0], 97}, {NULL}};
const Segment SGI297417[] = {{&S100250[64], 4}, {NULL}};
const Segment SGI297418[] = {{&S100250[8], 3}, {NULL}};
const Segment SGI297419[] = {{&S100250[13], 7}, {NULL}};
const Segment SGI297420[] = {{&S100250[7], 1}, {NULL}};
const Segment SGI297421[] = {{&S100250[16], 4}, {&S100250[28], 2}, {NULL}};
const Segment SGI297422[] = {{&S100256[68], 4}, {NULL}};
const Segment SGI297423[] = {{&S100256[8], 3}, {NULL}};
const Segment SGI297424[] = {{&S100256[13], 9}, {NULL}};
const Segment SGI297425[] = {{&S100256[30], 13}, {NULL}};
const Segment SGI297426[] = {{&S100256[7], 1}, {NULL}};
const Segment SGI297427[] = {{&S100256[73], 8}, {NULL}};
const Segment SGI297428[] = {{&S100270[55], 5}, {NULL}};
const Segment SGI297429[] = {{&S100270[8], 3}, {NULL}};
const Segment SGI297430[] = {{&S100270[13], 14}, {NULL}};
const Segment SGI297431[] = {{&S100270[7], 1}, {NULL}};
const Segment SGI297432[] = {{&S100270[16], 11}, {NULL}};
const Segment SGI297433[] = {{&S100270[63], 13}, {NULL}};
const Segment SGI297434[] = {{&S100284[57], 4}, {NULL}};
const Segment SGI297435[] = {{&S100284[8], 3}, {NULL}};
const Segment SGI297436[] = {{&S100284[13], 3}, {NULL}};
const Segment SGI297437[] = {{&S100284[7], 1}, {NULL}};
const Segment SGI297438[] = {{&S100284[18], 10}, {NULL}};
const Segment SGI297439[] = {{&S84528[43], 9}, {NULL}};
const Segment SGI297440[] = {{&S84528[0], 97}, {NULL}};
const Segment SGI297441[] = {{&S84537[43], 6}, {NULL}};
const Segment SGI297442[] = {{&S84537[0], 97}, {NULL}};
const Segment SGI297443[] = {{&S84538[43], 6}, {NULL}};
const Segment SGI297444[] = {{&S84538[0], 97}, {NULL}};
const Segment SGI297445[] = {{&S84580[55], 12}, {NULL}};
const Segment SGI297446[] = {{&S84580[0], 87}, {NULL}};
const Segment SGI297447[] = {{&S84581[55], 12}, {NULL}};
const Segment SGI297448[] = {{&S84581[0], 87}, {NULL}};
const Segment SGI297449[] = {{&S100407[84], 4}, {NULL}};
const Segment SGI297450[] = {{&S100407[8], 3}, {NULL}};
const Segment SGI297451[] = {{&S100407[13], 7}, {NULL}};
const Segment SGI297452[] = {{&S100407[25], 14}, {NULL}};
const Segment SGI297453[] = {{&S100407[7], 1}, {NULL}};
const Segment SGI297454[] = {{&S100407[16], 4}, {&S100407[28], 11}, {NULL}};
const Segment SGI297455[] = {{&S100407[68], 8}, {NULL}};
const Segment SGI297456[] = {{&S84718[33], 35}, {NULL}};
const Segment SGI297457[] = {{&S84718[0], 100}, {NULL}};
const Segment SGI297458[] = {{&S84719[33], 35}, {NULL}};
const Segment SGI297459[] = {{&S84719[0], 100}, {NULL}};
const Segment SGI297460[] = {{&S84720[33], 35}, {NULL}};
const Segment SGI297461[] = {{&S84720[0], 100}, {NULL}};
const Segment SGI297462[] = {{&S100600[69], 5}, {NULL}};
const Segment SGI297463[] = {{&S100600[8], 3}, {NULL}};
const Segment SGI297464[] = {{&S100600[13], 9}, {NULL}};
const Segment SGI297465[] = {{&S100600[30], 13}, {NULL}};
const Segment SGI297466[] = {{&S100600[7], 1}, {NULL}};
const Segment SGI297467[] = {{&S100609[76], 4}, {NULL}};
const Segment SGI297468[] = {{&S100609[8], 3}, {NULL}};
const Segment SGI297469[] = {{&S100609[13], 7}, {NULL}};
const Segment SGI297470[] = {{&S100609[25], 14}, {NULL}};
const Segment SGI297471[] = {{&S100609[7], 1}, {NULL}};
const Segment SGI297472[] = {{&S100609[16], 4}, {&S100609[28], 11}, {NULL}};
const Segment SGI297473[] = {{&S84848[0], 88}, {NULL}};
const Segment SGI297474[] = {{&S85202[43], 6}, {NULL}};
const Segment SGI297475[] = {{&S85202[0], 97}, {NULL}};
const Segment SGI297476[] = {{&S85203[43], 6}, {NULL}};
const Segment SGI297477[] = {{&S85203[0], 97}, {NULL}};
const Segment SGI297478[] = {{&S75566[0], 9}, {NULL}};
const Segment SGI297479[] = {{&S100770[82], 4}, {NULL}};
const Segment SGI297480[] = {{&S100770[8], 3}, {NULL}};
const Segment SGI297481[] = {{&S100770[13], 9}, {NULL}};
const Segment SGI297482[] = {{&S100770[33], 13}, {NULL}};
const Segment SGI297483[] = {{&S100770[7], 1}, {NULL}};
const Segment SGI297484[] = {{&S100770[89], 8}, {NULL}};
const Segment SGI297485[] = {{&S85314[43], 17}, {NULL}};
const Segment SGI297486[] = {{&S85314[0], 105}, {NULL}};
const Segment SGI297487[] = {{&S85315[43], 17}, {NULL}};
const Segment SGI297488[] = {{&S85315[0], 105}, {NULL}};
const Segment SGI297489[] = {{&S85331[0], 70}, {NULL}};
const Segment SGI297490[] = {{&S85332[0], 88}, {NULL}};
const Segment SGI297491[] = {{&S85333[0], 75}, {NULL}};
const Segment SGI297492[] = {{&S100808[82], 4}, {NULL}};
const Segment SGI297493[] = {{&S100808[8], 3}, {NULL}};
const Segment SGI297494[] = {{&S100808[13], 9}, {NULL}};
const Segment SGI297495[] = {{&S100808[33], 13}, {NULL}};
const Segment SGI297496[] = {{&S100808[7], 1}, {NULL}};
const Segment SGI297497[] = {{&S100809[85], 4}, {NULL}};
const Segment SGI297498[] = {{&S100809[8], 3}, {NULL}};
const Segment SGI297499[] = {{&S100809[13], 7}, {NULL}};
const Segment SGI297500[] = {{&S100809[25], 14}, {NULL}};
const Segment SGI297501[] = {{&S100809[7], 1}, {NULL}};
const Segment SGI297502[] = {{&S100809[16], 4}, {&S100809[28], 11}, {NULL}};
const Segment SGI297503[] = {{&S100809[69], 8}, {NULL}};
const Segment SGI297504[] = {{&S85417[79], 8}, {NULL}};
const Segment SGI297505[] = {{&S85417[0], 127}, {NULL}};
const Segment SGI297506[] = {{&S85418[0], 94}, {NULL}};
const Segment SGI297507[] = {{&S85419[0], 94}, {NULL}};
const Segment SGI297508[] = {{&S85420[0], 94}, {NULL}};
const Segment SGI297509[] = {{&S97694[8], 3}, {NULL}};
const Segment SGI297510[] = {{&S97694[60], 3}, {NULL}};
const Segment SGI297511[] = {{&S97694[64], 6}, {NULL}};
const Segment SGI297512[] = {{&S97694[13], 29}, {NULL}};
const Segment SGI297513[] = {{&S101013[88], 4}, {NULL}};
const Segment SGI297514[] = {{&S101013[8], 3}, {NULL}};
const Segment SGI297515[] = {{&S101013[13], 3}, {NULL}};
const Segment SGI297516[] = {{&S101013[58], 10}, {NULL}};
const Segment SGI297517[] = {{&S101013[7], 1}, {NULL}};
const Segment SGI297518[] = {{&S101013[58], 7}, {NULL}};
const Segment SGI297519[] = {{&S101017[60], 5}, {NULL}};
const Segment SGI297520[] = {{&S101017[8], 3}, {NULL}};
const Segment SGI297521[] = {{&S101017[13], 14}, {NULL}};
const Segment SGI297522[] = {{&S101017[7], 1}, {NULL}};
const Segment SGI297523[] = {{&S101017[16], 11}, {NULL}};
const Segment SGI297524[] = {{&S101017[68], 13}, {NULL}};
const Segment SGI297525[] = {{&S101038[99], 4}, {NULL}};
const Segment SGI297526[] = {{&S101038[8], 3}, {NULL}};
const Segment SGI297527[] = {{&S101038[13], 3}, {NULL}};
const Segment SGI297528[] = {{&S101038[57], 11}, {NULL}};
const Segment SGI297529[] = {{&S101038[7], 1}, {NULL}};
const Segment SGI297530[] = {{&S101038[57], 7}, {NULL}};
const Segment SGI297531[] = {{&S101039[68], 4}, {NULL}};
const Segment SGI297532[] = {{&S101039[8], 3}, {NULL}};
const Segment SGI297533[] = {{&S101039[13], 3}, {NULL}};
const Segment SGI297534[] = {{&S101039[7], 1}, {NULL}};
const Segment SGI297535[] = {{&S101039[78], 9}, {NULL}};
const Segment SGI297536[] = {{&S101043[56], 4}, {NULL}};
const Segment SGI297537[] = {{&S101043[8], 3}, {NULL}};
const Segment SGI297538[] = {{&S101043[13], 3}, {NULL}};
const Segment SGI297539[] = {{&S101043[13], 3}, {&S101043[18], 7}, {NULL}};
const Segment SGI297540[] = {{&S101043[7], 1}, {NULL}};
const Segment SGI297541[] = {{&S101044[7], 4}, {NULL}};
const Segment SGI297542[] = {{&S101044[8], 3}, {NULL}};
const Segment SGI297543[] = {{&S101044[13], 14}, {NULL}};
const Segment SGI297544[] = {{&S101044[7], 1}, {NULL}};
const Segment SGI297545[] = {{&S101044[16], 11}, {NULL}};
const Segment SGI297546[] = {{&S101044[57], 8}, {NULL}};
const Segment SGI297547[] = {{&S101051[64], 4}, {NULL}};
const Segment SGI297548[] = {{&S101051[8], 3}, {NULL}};
const Segment SGI297549[] = {{&S101051[13], 14}, {NULL}};
const Segment SGI297550[] = {{&S101051[7], 1}, {NULL}};
const Segment SGI297551[] = {{&S101051[16], 11}, {&S101051[32], 2}, {NULL}};
const Segment SGI297552[] = {{&S101051[73], 12}, {NULL}};
const Segment SGI297553[] = {{&S101082[73], 4}, {NULL}};
const Segment SGI297554[] = {{&S101082[8], 3}, {NULL}};
const Segment SGI297555[] = {{&S101082[13], 7}, {NULL}};
const Segment SGI297556[] = {{&S101082[25], 14}, {NULL}};
const Segment SGI297557[] = {{&S101082[7], 1}, {NULL}};
const Segment SGI297558[] = {{&S101082[16], 4}, {&S101082[28], 11}, {NULL}};
const Segment SGI297559[] = {{&S101082[78], 8}, {NULL}};
const Segment SGI297560[] = {{&S101202[89], 4}, {NULL}};
const Segment SGI297561[] = {{&S101202[8], 3}, {NULL}};
const Segment SGI297562[] = {{&S101202[13], 3}, {NULL}};
const Segment SGI297563[] = {{&S101202[61], 10}, {NULL}};
const Segment SGI297564[] = {{&S101202[7], 1}, {NULL}};
const Segment SGI297565[] = {{&S101202[61], 7}, {NULL}};
const Segment SGI297566[] = {{&S101342[94], 4}, {NULL}};
const Segment SGI297567[] = {{&S101342[8], 3}, {NULL}};
const Segment SGI297568[] = {{&S101342[13], 7}, {NULL}};
const Segment SGI297569[] = {{&S101342[25], 14}, {NULL}};
const Segment SGI297570[] = {{&S101342[7], 1}, {NULL}};
const Segment SGI297571[] = {{&S101342[16], 4}, {&S101342[28], 11}, {NULL}};
const Segment SGI297572[] = {{&S101342[101], 8}, {NULL}};
const Segment SGI297573[] = {{&S85895[18], 9}, {NULL}};
const Segment SGI297574[] = {{&S85895[0], 129}, {NULL}};
const Segment SGI297575[] = {{&S85896[0], 71}, {NULL}};
const Segment SGI297576[] = {{&S85897[0], 62}, {NULL}};
const Segment SGI297577[] = {{&S85922[0], 99}, {NULL}};
const Segment SGI297578[] = {{&S85934[0], 53}, {NULL}};
const Segment SGI297579[] = {{&S85935[0], 46}, {NULL}};
const Segment SGI297580[] = {{&S85936[0], 53}, {NULL}};
const Segment SGI297581[] = {{&S85975[41], 7}, {NULL}};
const Segment SGI297582[] = {{&S85975[0], 93}, {NULL}};
const Segment SGI297583[] = {{&S85976[43], 7}, {NULL}};
const Segment SGI297584[] = {{&S85976[0], 95}, {NULL}};
const Segment SGI297585[] = {{&S101688[88], 4}, {NULL}};
const Segment SGI297586[] = {{&S101688[8], 3}, {NULL}};
const Segment SGI297587[] = {{&S101688[13], 3}, {NULL}};
const Segment SGI297588[] = {{&S101688[13], 3}, {&S101688[60], 6}, {NULL}};
const Segment SGI297589[] = {{&S101688[60], 12}, {NULL}};
const Segment SGI297590[] = {{&S101688[7], 1}, {NULL}};
const Segment SGI297591[] = {{&S86030[0], 55}, {NULL}};
const Segment SGI297592[] = {{&S86031[0], 48}, {NULL}};
const Segment SGI297593[] = {{&S101842[8], 3}, {NULL}};
const Segment SGI297594[] = {{&S101842[13], 7}, {NULL}};
const Segment SGI297595[] = {{&S101842[25], 14}, {NULL}};
const Segment SGI297596[] = {{&S101842[7], 1}, {NULL}};
const Segment SGI297597[] = {{&S101842[16], 4}, {&S101842[28], 11}, {NULL}};
const Segment SGI297598[] = {{&S101842[68], 5}, {NULL}};
const Segment SGI297599[] = {{&S98638[8], 3}, {NULL}};
const Segment SGI297600[] = {{&S98638[63], 6}, {NULL}};
const Segment SGI297601[] = {{&S98638[13], 28}, {NULL}};
const Segment SGI297602[] = {{&S102059[8], 3}, {NULL}};
const Segment SGI297603[] = {{&S102059[13], 7}, {NULL}};
const Segment SGI297604[] = {{&S102059[16], 4}, {&S102059[28], 10}, {NULL}};
const Segment SGI297605[] = {{&S86317[0], 64}, {NULL}};
const Segment SGI297606[] = {{&S86327[0], 83}, {NULL}};
const Segment SGI297607[] = {{&S86328[0], 81}, {NULL}};
const Segment SGI297608[] = {{&S86416[0], 56}, {NULL}};
const Segment SGI297609[] = {{&S102227[8], 3}, {NULL}};
const Segment SGI297610[] = {{&S102227[13], 7}, {NULL}};
const Segment SGI297611[] = {{&S102227[25], 14}, {NULL}};
const Segment SGI297612[] = {{&S86417[0], 56}, {NULL}};
const Segment SGI297613[] = {{&S102227[7], 1}, {NULL}};
const Segment SGI297614[] = {{&S102227[16], 4}, {&S102227[28], 11}, {NULL}};
const Segment SGI297615[] = {{&S102227[68], 5}, {NULL}};
const Segment SGI297616[] = {{&S102229[80], 4}, {NULL}};
const Segment SGI297617[] = {{&S102229[8], 3}, {NULL}};
const Segment SGI297618[] = {{&S102229[13], 9}, {NULL}};
const Segment SGI297619[] = {{&S102229[33], 13}, {NULL}};
const Segment SGI297620[] = {{&S102229[7], 1}, {NULL}};
const Segment SGI297621[] = {{&S102229[85], 8}, {NULL}};
const Segment SGI297622[] = {{&S102318[77], 4}, {NULL}};
const Segment SGI297623[] = {{&S102318[8], 3}, {NULL}};
const Segment SGI297624[] = {{&S102318[13], 3}, {NULL}};
const Segment SGI297625[] = {{&S102318[7], 1}, {NULL}};
const Segment SGI297626[] = {{&S102318[56], 7}, {NULL}};
const Segment SGI297627[] = {{&S86552[18], 9}, {NULL}};
const Segment SGI297628[] = {{&S86552[0], 139}, {NULL}};
const Segment SGI297629[] = {{&S86574[49], 7}, {NULL}};
const Segment SGI297630[] = {{&S86574[0], 140}, {NULL}};
const Segment SGI297631[] = {{&S86791[0], 67}, {NULL}};
const Segment SGI297632[] = {{&S86792[0], 67}, {NULL}};
const Segment SGI297633[] = {{&S86841[43], 3}, {NULL}};
const Segment SGI297634[] = {{&S86841[0], 94}, {NULL}};
const Segment SGI297635[] = {{&S86842[43], 3}, {NULL}};
const Segment SGI297636[] = {{&S86842[0], 94}, {NULL}};
const Segment SGI297637[] = {{&S86897[0], 81}, {NULL}};
const Segment SGI297638[] = {{&S86913[0], 90}, {NULL}};
const Segment SGI297639[] = {{&S86937[0], 60}, {NULL}};
const Segment SGI297640[] = {{&S86938[0], 60}, {NULL}};
const Segment SGI297641[] = {{&S87067[0], 83}, {NULL}};
const Segment SGI297642[] = {{&S87185[0], 75}, {NULL}};
const Segment SGI297643[] = {{&S99593[8], 3}, {NULL}};
const Segment SGI297644[] = {{&S99593[71], 5}, {NULL}};
const Segment SGI297645[] = {{&S99593[64], 6}, {NULL}};
const Segment SGI297646[] = {{&S99593[13], 24}, {NULL}};
const Segment SGI297647[] = {{&S99593[55], 5}, {NULL}};
const Segment SGI297648[] = {{&S77815[0], 9}, {NULL}};
const Segment SGI297649[] = {{&S87382[43], 8}, {NULL}};
const Segment SGI297650[] = {{&S87382[0], 103}, {NULL}};
const Segment SGI297651[] = {{&S87383[43], 8}, {NULL}};
const Segment SGI297652[] = {{&S87383[0], 103}, {NULL}};
const Segment SGI297653[] = {{&S87389[43], 15}, {NULL}};
const Segment SGI297654[] = {{&S87389[0], 103}, {NULL}};
const Segment SGI297655[] = {{&S87521[0], 71}, {NULL}};
const Segment SGI297656[] = {{&S87687[0], 66}, {NULL}};
const Segment SGI297657[] = {{&S87704[33], 5}, {NULL}};
const Segment SGI297658[] = {{&S87704[0], 71}, {NULL}};
const Segment SGI297659[] = {{&S87705[33], 5}, {NULL}};
const Segment SGI297660[] = {{&S87705[0], 71}, {NULL}};
const Segment SGI297661[] = {{&S87732[43], 8}, {NULL}};
const Segment SGI297662[] = {{&S87732[0], 105}, {NULL}};
const Segment SGI297663[] = {{&S87733[43], 8}, {NULL}};
const Segment SGI297664[] = {{&S87733[0], 105}, {NULL}};
const Segment SGI297665[] = {{&S88039[0], 51}, {NULL}};
const Segment SGI297666[] = {{&S88162[0], 60}, {NULL}};
const Segment SGI297667[] = {{&S88163[0], 57}, {NULL}};
const Segment SGI297668[] = {{&S88164[0], 58}, {NULL}};
const Segment SGI297669[] = {{&S88165[0], 53}, {NULL}};
const Segment SGI297670[] = {{&S88166[0], 53}, {NULL}};
const Segment SGI297671[] = {{&S88167[0], 57}, {NULL}};
const Segment SGI297672[] = {{&S88168[0], 63}, {NULL}};
const Segment SGI297673[] = {{&S88169[0], 52}, {NULL}};
const Segment SGI297674[] = {{&S88170[0], 59}, {NULL}};
const Segment SGI297675[] = {{&S88171[0], 52}, {NULL}};
const Segment SGI297676[] = {{&S88172[0], 58}, {NULL}};
const Segment SGI297677[] = {{&S88173[0], 57}, {NULL}};
const Segment SGI297678[] = {{&S88174[0], 57}, {NULL}};
const Segment SGI297679[] = {{&S88175[0], 57}, {NULL}};
const Segment SGI297680[] = {{&S78806[0], 9}, {NULL}};
const Segment SGI297681[] = {{&S88717[0], 64}, {NULL}};
const Segment SGI297682[] = {{&S88818[0], 99}, {NULL}};
const Segment SGI297683[] = {{&S88819[0], 74}, {NULL}};
const Segment SGI297684[] = {{&S88820[0], 99}, {NULL}};
const Segment SGI297685[] = {{&S88941[43], 3}, {NULL}};
const Segment SGI297686[] = {{&S88941[0], 94}, {NULL}};
const Segment SGI297687[] = {{&S88942[43], 3}, {NULL}};
const Segment SGI297688[] = {{&S88942[0], 94}, {NULL}};
const Segment SGI297689[] = {{&S89131[0], 81}, {NULL}};
const Segment SGI297690[] = {{&S105080[53], 5}, {NULL}};
const Segment SGI297691[] = {{&S105080[8], 3}, {NULL}};
const Segment SGI297692[] = {{&S105080[13], 3}, {NULL}};
const Segment SGI297693[] = {{&S105080[7], 1}, {NULL}};
const Segment SGI297694[] = {{&S105081[75], 5}, {NULL}};
const Segment SGI297695[] = {{&S105081[8], 3}, {NULL}};
const Segment SGI297696[] = {{&S105081[15], 3}, {NULL}};
const Segment SGI297697[] = {{&S105081[7], 1}, {NULL}};
const Segment SGI297698[] = {{&S105081[33], 2}, {NULL}};
const Segment SGI297699[] = {{&S89301[43], 6}, {NULL}};
const Segment SGI297700[] = {{&S89301[0], 94}, {NULL}};
const Segment SGI297701[] = {{&S89358[43], 6}, {NULL}};
const Segment SGI297702[] = {{&S89358[0], 97}, {NULL}};
const Segment SGI297703[] = {{&S89359[43], 6}, {NULL}};
const Segment SGI297704[] = {{&S89359[0], 97}, {NULL}};
const Segment SGI297705[] = {{&S89410[0], 88}, {NULL}};
const Segment SGI297706[] = {{&S79619[0], 9}, {NULL}};
const Segment SGI297707[] = {{&S105635[70], 4}, {NULL}};
const Segment SGI297708[] = {{&S105635[8], 3}, {&S105635[8], 3}, {NULL}};
const Segment SGI297709[] = {{&S105635[13], 3}, {&S105635[21], 5}, {NULL}};
const Segment SGI297710[] = {{&S105635[13], 3}, {NULL}};
const Segment SGI297711[] = {{&S105635[7], 1}, {NULL}};
const Segment SGI297712[] = {{&S89591[0], 55}, {NULL}};
const Segment SGI297713[] = {{&S89592[0], 64}, {NULL}};
const Segment SGI297714[] = {{&S89593[0], 54}, {NULL}};
const Segment SGI297715[] = {{&S89736[0], 85}, {NULL}};
const Segment SGI297716[] = {{&S80043[0], 9}, {NULL}};
const Segment SGI297717[] = {{&S89874[0], 64}, {NULL}};
const Segment SGI297718[] = {{&S80121[0], 9}, {NULL}};
const Segment SGI297719[] = {{&S80127[0], 9}, {NULL}};
const Segment SGI297720[] = {{&S80204[0], 9}, {NULL}};
const Segment SGI297721[] = {{&S105993[56], 5}, {NULL}};
const Segment SGI297722[] = {{&S105993[8], 3}, {NULL}};
const Segment SGI297723[] = {{&S105993[13], 3}, {NULL}};
const Segment SGI297724[] = {{&S105993[7], 1}, {NULL}};
const Segment SGI297725[] = {{&S105994[56], 5}, {NULL}};
const Segment SGI297726[] = {{&S105994[8], 3}, {NULL}};
const Segment SGI297727[] = {{&S105994[13], 7}, {NULL}};
const Segment SGI297728[] = {{&S105994[7], 1}, {NULL}};
const Segment SGI297729[] = {{&S105994[16], 4}, {&S105994[16], 4}, {NULL}};
const Segment SGI297730[] = {{&S90028[0], 73}, {NULL}};
const Segment SGI297731[] = {{&S90134[43], 8}, {NULL}};
const Segment SGI297732[] = {{&S90134[0], 103}, {NULL}};
const Segment SGI297733[] = {{&S80460[0], 9}, {NULL}};
const Segment SGI297734[] = {{&S90171[0], 94}, {NULL}};
const Segment SGI297735[] = {{&S90172[0], 94}, {NULL}};
const Segment SGI297736[] = {{&S80511[0], 9}, {NULL}};
const Segment SGI297737[] = {{&S80583[0], 9}, {NULL}};
const Segment SGI297738[] = {{&S90332[43], 6}, {NULL}};
const Segment SGI297739[] = {{&S90332[0], 97}, {NULL}};
const Segment SGI297740[] = {{&S90333[43], 6}, {NULL}};
const Segment SGI297741[] = {{&S90333[0], 97}, {NULL}};
const Segment SGI297742[] = {{&S90353[79], 9}, {NULL}};
const Segment SGI297743[] = {{&S90353[0], 128}, {NULL}};
const Segment SGI297744[] = {{&S90388[0], 90}, {NULL}};
const Segment SGI297745[] = {{&S80730[0], 9}, {NULL}};
const Segment SGI297746[] = {{&S80735[0], 9}, {NULL}};
const Segment SGI297747[] = {{&S80738[0], 9}, {NULL}};
const Segment SGI297748[] = {{&S80807[0], 9}, {NULL}};
const Segment SGI297749[] = {{&S90560[0], 91}, {NULL}};
const Segment SGI297750[] = {{&S103383[8], 3}, {NULL}};
const Segment SGI297751[] = {{&S103383[39], 3}, {NULL}};
const Segment SGI297752[] = {{&S103383[63], 6}, {NULL}};
const Segment SGI297753[] = {{&S103383[13], 29}, {NULL}};
const Segment SGI297754[] = {{&S90616[43], 9}, {NULL}};
const Segment SGI297755[] = {{&S90616[0], 97}, {NULL}};
const Segment SGI297756[] = {{&S80936[0], 9}, {NULL}};
const Segment SGI297757[] = {{&S90653[43], 6}, {NULL}};
const Segment SGI297758[] = {{&S90653[0], 97}, {NULL}};
const Segment SGI297759[] = {{&S90738[0], 85}, {NULL}};
const Segment SGI297760[] = {{&S90739[0], 91}, {NULL}};
const Segment SGI297761[] = {{&S90740[87], 7}, {NULL}};
const Segment SGI297762[] = {{&S90740[0], 134}, {NULL}};
const Segment SGI297763[] = {{&S90896[43], 3}, {NULL}};
const Segment SGI297764[] = {{&S90896[0], 94}, {NULL}};
const Segment SGI297765[] = {{&S90907[43], 9}, {NULL}};
const Segment SGI297766[] = {{&S90907[0], 97}, {NULL}};
const Segment SGI297767[] = {{&S90908[43], 9}, {NULL}};
const Segment SGI297768[] = {{&S90908[0], 97}, {NULL}};
const Segment SGI297769[] = {{&S90926[43], 17}, {NULL}};
const Segment SGI297770[] = {{&S90926[0], 105}, {NULL}};
const Segment SGI297771[] = {{&S90937[0], 52}, {NULL}};
const Segment SGI297772[] = {{&S90938[0], 54}, {NULL}};
const Segment SGI297773[] = {{&S90939[0], 54}, {NULL}};
const Segment SGI297774[] = {{&S90964[0], 101}, {NULL}};
const Segment SGI297775[] = {{&S90965[0], 101}, {NULL}};
const Segment SGI297776[] = {{&S91058[0], 100}, {NULL}};
const Segment SGI297777[] = {{&S91059[0], 100}, {NULL}};
const Segment SGI297778[] = {{&S91450[0], 70}, {NULL}};
const Segment SGI297779[] = {{&S91485[0], 88}, {NULL}};
const Segment SGI297780[] = {{&S91539[33], 6}, {NULL}};
const Segment SGI297781[] = {{&S91539[0], 71}, {NULL}};
const Segment SGI297782[] = {{&S91540[33], 6}, {NULL}};
const Segment SGI297783[] = {{&S91540[0], 71}, {NULL}};
const Segment SGI297784[] = {{&S91573[43], 9}, {NULL}};
const Segment SGI297785[] = {{&S91573[0], 97}, {NULL}};
const Segment SGI297786[] = {{&S91679[0], 57}, {NULL}};
const Segment SGI297787[] = {{&S91680[0], 60}, {NULL}};
const Segment SGI297788[] = {{&S91681[0], 57}, {NULL}};
const Segment SGI297789[] = {{&S91682[0], 57}, {NULL}};
const Segment SGI297790[] = {{&S81950[0], 9}, {NULL}};
const Segment SGI297791[] = {{&S91701[0], 96}, {NULL}};
const Segment SGI297792[] = {{&S91872[0], 44}, {NULL}};
const Segment SGI297793[] = {{&S91873[0], 77}, {NULL}};
const Segment SGI297794[] = {{&S104372[8], 3}, {NULL}};
const Segment SGI297795[] = {{&S104372[39], 3}, {NULL}};
const Segment SGI297796[] = {{&S104372[63], 6}, {NULL}};
const Segment SGI297797[] = {{&S104372[13], 29}, {NULL}};
const Segment SGI297798[] = {{&S91958[43], 13}, {NULL}};
const Segment SGI297799[] = {{&S91958[0], 101}, {NULL}};
const Segment SGI297800[] = {{&S92061[0], 71}, {NULL}};
const Segment SGI297801[] = {{&S92062[80], 5}, {NULL}};
const Segment SGI297802[] = {{&S92062[0], 105}, {NULL}};
const Segment SGI297803[] = {{&S92068[0], 96}, {NULL}};
const Segment SGI297804[] = {{&S92070[0], 73}, {NULL}};
const Segment SGI297805[] = {{&S92109[0], 67}, {NULL}};
const Segment SGI297806[] = {{&S92110[0], 69}, {NULL}};
const Segment SGI297807[] = {{&S92111[0], 69}, {NULL}};
const Segment SGI297808[] = {{&S92202[43], 6}, {NULL}};
const Segment SGI297809[] = {{&S92202[0], 97}, {NULL}};
const Segment SGI297810[] = {{&S92203[43], 6}, {NULL}};
const Segment SGI297811[] = {{&S92203[0], 97}, {NULL}};
const Segment SGI297812[] = {{&S82522[0], 9}, {NULL}};
const Segment SGI297813[] = {{&S92276[55], 12}, {NULL}};
const Segment SGI297814[] = {{&S92276[0], 87}, {NULL}};
const Segment SGI297815[] = {{&S92303[0], 90}, {NULL}};
const Segment SGI297816[] = {{&S92443[43], 9}, {NULL}};
const Segment SGI297817[] = {{&S92443[0], 97}, {NULL}};
const Segment SGI297818[] = {{&S82886[0], 9}, {NULL}};
const Segment SGI297819[] = {{&S82887[0], 9}, {NULL}};
const Segment SGI297820[] = {{&S82888[0], 9}, {NULL}};
const Segment SGI297821[] = {{&S92528[33], 3}, {NULL}};
const Segment SGI297822[] = {{&S92528[0], 69}, {NULL}};
const Segment SGI297823[] = {{&S92529[33], 3}, {NULL}};
const Segment SGI297824[] = {{&S92529[0], 68}, {NULL}};
const Segment SGI297825[] = {{&S92530[33], 3}, {NULL}};
const Segment SGI297826[] = {{&S92530[0], 68}, {NULL}};
const Segment SGI297827[] = {{&S82956[0], 9}, {NULL}};
const Segment SGI297828[] = {{&S92577[43], 36}, {NULL}};
const Segment SGI297829[] = {{&S92577[0], 124}, {NULL}};
const Segment SGI297830[] = {{&S92609[43], 6}, {NULL}};
const Segment SGI297831[] = {{&S92609[0], 97}, {NULL}};
const Segment SGI297832[] = {{&S92637[43], 8}, {NULL}};
const Segment SGI297833[] = {{&S92637[0], 105}, {NULL}};
const Segment SGI297834[] = {{&S92638[43], 8}, {NULL}};
const Segment SGI297835[] = {{&S92638[0], 105}, {NULL}};
const Segment SGI297836[] = {{&S83207[0], 9}, {NULL}};
const Segment SGI297837[] = {{&S92822[0], 73}, {NULL}};
const Segment SGI297838[] = {{&S92854[0], 88}, {NULL}};
const Segment SGI297839[] = {{&S83494[0], 9}, {NULL}};
const Segment SGI297840[] = {{&S92942[43], 36}, {NULL}};
const Segment SGI297841[] = {{&S92942[0], 124}, {NULL}};
const Segment SGI297842[] = {{&S83744[0], 9}, {NULL}};
const Segment SGI297843[] = {{&S93254[82], 9}, {NULL}};
const Segment SGI297844[] = {{&S93254[0], 91}, {NULL}};
const Segment SGI297845[] = {{&S93258[65], 9}, {NULL}};
const Segment SGI297846[] = {{&S93258[0], 131}, {NULL}};
const Segment SGI297847[] = {{&S93259[65], 9}, {NULL}};
const Segment SGI297848[] = {{&S93259[0], 131}, {NULL}};
const Segment SGI297849[] = {{&S93260[65], 9}, {NULL}};
const Segment SGI297850[] = {{&S93260[0], 131}, {NULL}};
const Segment SGI297851[] = {{&S93434[0], 102}, {NULL}};
const Segment SGI297852[] = {{&S93435[0], 102}, {NULL}};
const Segment SGI297853[] = {{&S93498[0], 69}, {NULL}};
const Segment SGI297854[] = {{&S93510[0], 73}, {NULL}};
const Segment SGI297855[] = {{&S93511[0], 69}, {NULL}};
const Segment SGI297856[] = {{&S93512[80], 10}, {NULL}};
const Segment SGI297857[] = {{&S93512[0], 130}, {NULL}};
const Segment SGI297858[] = {{&S93513[80], 9}, {NULL}};
const Segment SGI297859[] = {{&S93513[0], 129}, {NULL}};
const Segment SGI297860[] = {{&S106040[8], 3}, {NULL}};
const Segment SGI297861[] = {{&S106040[13], 15}, {NULL}};
const Segment SGI297862[] = {{&S106040[24], 2}, {NULL}};
const Segment SGI297863[] = {{&S106040[24], 4}, {NULL}};
const Segment SGI297864[] = {{&S93531[79], 8}, {NULL}};
const Segment SGI297865[] = {{&S93531[0], 127}, {NULL}};
const Segment SGI297866[] = {{&S84174[0], 9}, {NULL}};
const Segment SGI297867[] = {{&S93716[0], 75}, {NULL}};
const Segment SGI297868[] = {{&S93717[0], 107}, {NULL}};
const Segment SGI297869[] = {{&S93718[0], 71}, {NULL}};
const Segment SGI297870[] = {{&S93719[80], 9}, {NULL}};
const Segment SGI297871[] = {{&S93719[0], 129}, {NULL}};
const Segment SGI297872[] = {{&S93728[80], 9}, {NULL}};
const Segment SGI297873[] = {{&S93728[0], 129}, {NULL}};
const Segment SGI297874[] = {{&S93729[98], 9}, {NULL}};
const Segment SGI297875[] = {{&S93729[0], 147}, {NULL}};
const Segment SGI297876[] = {{&S93730[80], 10}, {NULL}};
const Segment SGI297877[] = {{&S93730[0], 130}, {NULL}};
const Segment SGI297878[] = {{&S93731[0], 73}, {NULL}};
const Segment SGI297879[] = {{&S93732[0], 71}, {NULL}};
const Segment SGI297880[] = {{&S93906[43], 8}, {NULL}};
const Segment SGI297881[] = {{&S93906[0], 96}, {NULL}};
const Segment SGI297882[] = {{&S93969[0], 91}, {NULL}};
const Segment SGI297883[] = {{&S94009[0], 64}, {NULL}};
const Segment SGI297884[] = {{&S94010[0], 64}, {NULL}};
const Segment SGI297885[] = {{&S84474[0], 9}, {NULL}};
const Segment SGI297886[] = {{&S84476[0], 9}, {NULL}};
const Segment SGI297887[] = {{&S94072[43], 16}, {NULL}};
const Segment SGI297888[] = {{&S94072[0], 104}, {NULL}};
const Segment SGI297889[] = {{&S84585[0], 9}, {NULL}};
const Segment SGI297890[] = {{&S94201[0], 73}, {NULL}};
const Segment SGI297891[] = {{&S94214[0], 57}, {NULL}};
const Segment SGI297892[] = {{&S84672[0], 9}, {NULL}};
const Segment SGI297893[] = {{&S94274[0], 70}, {NULL}};
const Segment SGI297894[] = {{&S94275[0], 67}, {NULL}};
const Segment SGI297895[] = {{&S94315[41], 7}, {NULL}};
const Segment SGI297896[] = {{&S94315[0], 93}, {NULL}};
const Segment SGI297897[] = {{&S94349[43], 8}, {NULL}};
const Segment SGI297898[] = {{&S94349[0], 105}, {NULL}};
const Segment SGI297899[] = {{&S94381[43], 6}, {NULL}};
const Segment SGI297900[] = {{&S94381[0], 94}, {NULL}};
const Segment SGI297901[] = {{&S94393[0], 87}, {NULL}};
const Segment SGI297902[] = {{&S94433[0], 74}, {NULL}};
const Segment SGI297903[] = {{&S94434[0], 74}, {NULL}};
const Segment SGI297904[] = {{&S94435[0], 70}, {NULL}};
const Segment SGI297905[] = {{&S94436[0], 70}, {NULL}};
const Segment SGI297906[] = {{&S94437[0], 67}, {NULL}};
const Segment SGI297907[] = {{&S94438[0], 73}, {NULL}};
const Segment SGI297908[] = {{&S94439[0], 67}, {NULL}};
const Segment SGI297909[] = {{&S94440[0], 67}, {NULL}};
const Segment SGI297910[] = {{&S94458[0], 91}, {NULL}};
const Segment SGI297911[] = {{&S94530[43], 13}, {NULL}};
const Segment SGI297912[] = {{&S94530[0], 101}, {NULL}};
const Segment SGI297913[] = {{&S84849[0], 9}, {NULL}};
const Segment SGI297914[] = {{&S94622[79], 5}, {NULL}};
const Segment SGI297915[] = {{&S94622[0], 105}, {NULL}};
const Segment SGI297916[] = {{&S94623[0], 99}, {NULL}};
const Segment SGI297917[] = {{&S94624[0], 70}, {NULL}};
const Segment SGI297918[] = {{&S94625[0], 122}, {NULL}};
const Segment SGI297919[] = {{&S94637[18], 9}, {NULL}};
const Segment SGI297920[] = {{&S94637[0], 128}, {NULL}};
const Segment SGI297921[] = {{&S94644[0], 71}, {NULL}};
const Segment SGI297922[] = {{&S84950[0], 9}, {NULL}};
const Segment SGI297923[] = {{&S94687[76], 9}, {NULL}};
const Segment SGI297924[] = {{&S94687[0], 101}, {NULL}};
const Segment SGI297925[] = {{&S94688[0], 94}, {NULL}};
const Segment SGI297926[] = {{&S94771[43], 12}, {NULL}};
const Segment SGI297927[] = {{&S94771[0], 100}, {NULL}};
const Segment SGI297928[] = {{&S94876[80], 9}, {NULL}};
const Segment SGI297929[] = {{&S94876[0], 129}, {NULL}};
const Segment SGI297930[] = {{&S94905[18], 9}, {NULL}};
const Segment SGI297931[] = {{&S94905[0], 127}, {NULL}};
const Segment SGI297932[] = {{&S95100[43], 9}, {NULL}};
const Segment SGI297933[] = {{&S95100[0], 97}, {NULL}};
const Segment SGI297934[] = {{&S95120[0], 55}, {NULL}};
const Segment SGI297935[] = {{&S95121[0], 53}, {NULL}};
const Segment SGI297936[] = {{&S95122[0], 60}, {NULL}};
const Segment SGI297937[] = {{&S95123[0], 55}, {NULL}};
const Segment SGI297938[] = {{&S95124[0], 62}, {NULL}};
const Segment SGI297939[] = {{&S95162[0], 71}, {NULL}};
const Segment SGI297940[] = {{&S95163[0], 74}, {NULL}};
const Segment SGI297941[] = {{&S95198[0], 67}, {NULL}};
const Segment SGI297942[] = {{&S95326[43], 6}, {NULL}};
const Segment SGI297943[] = {{&S95326[0], 97}, {NULL}};
const Segment SGI297944[] = {{&S85781[0], 9}, {NULL}};
const Segment SGI297945[] = {{&S95445[0], 51}, {NULL}};
const Segment SGI297946[] = {{&S95446[0], 51}, {NULL}};
const Segment SGI297947[] = {{&S86026[0], 9}, {NULL}};
const Segment SGI297948[] = {{&S86114[0], 9}, {NULL}};
const Segment SGI297949[] = {{&S95684[0], 80}, {NULL}};
const Segment SGI297950[] = {{&S95685[0], 137}, {NULL}};
const Segment SGI297951[] = {{&S95798[48], 5}, {NULL}};
const Segment SGI297952[] = {{&S95798[0], 75}, {NULL}};
const Segment SGI297953[] = {{&S95921[43], 9}, {NULL}};
const Segment SGI297954[] = {{&S95921[0], 97}, {NULL}};
const Segment SGI297955[] = {{&S95963[0], 121}, {NULL}};
const Segment SGI297956[] = {{&S95964[0], 122}, {NULL}};
const Segment SGI297957[] = {{&S96051[0], 92}, {NULL}};
const Segment SGI297958[] = {{&S86662[0], 9}, {NULL}};
const Segment SGI297959[] = {{&S96285[0], 62}, {NULL}};
const Segment SGI297960[] = {{&S96286[0], 61}, {NULL}};
const Segment SGI297961[] = {{&S96595[0], 96}, {NULL}};
const Segment SGI297962[] = {{&S96620[55], 9}, {NULL}};
const Segment SGI297963[] = {{&S96620[0], 84}, {NULL}};
const Segment SGI297964[] = {{&S96638[0], 98}, {NULL}};
const Segment SGI297965[] = {{&S96685[0], 68}, {NULL}};
const Segment SGI297966[] = {{&S96755[43], 6}, {NULL}};
const Segment SGI297967[] = {{&S96755[0], 94}, {NULL}};
const Segment SGI297968[] = {{&S96912[0], 70}, {NULL}};
const Segment SGI297969[] = {{&S96999[0], 67}, {NULL}};
const Segment SGI297970[] = {{&S97025[33], 5}, {NULL}};
const Segment SGI297971[] = {{&S97025[0], 71}, {NULL}};
const Segment SGI297972[] = {{&S97034[0], 73}, {NULL}};
const Segment SGI297973[] = {{&S97105[49], 5}, {NULL}};
const Segment SGI297974[] = {{&S97105[0], 99}, {NULL}};
const Segment SGI297975[] = {{&S97108[43], 9}, {NULL}};
const Segment SGI297976[] = {{&S97108[0], 97}, {NULL}};
const Segment SGI297977[] = {{&S87666[0], 9}, {NULL}};
const Segment SGI297978[] = {{&S97191[43], 13}, {NULL}};
const Segment SGI297979[] = {{&S97191[0], 101}, {NULL}};
const Segment SGI297980[] = {{&S97243[0], 46}, {NULL}};
const Segment SGI297981[] = {{&S97244[0], 52}, {NULL}};
const Segment SGI297982[] = {{&S97245[0], 53}, {NULL}};
const Segment SGI297983[] = {{&S97249[0], 71}, {NULL}};
const Segment SGI297984[] = {{&S97250[80], 9}, {NULL}};
const Segment SGI297985[] = {{&S97250[0], 129}, {NULL}};
const Segment SGI297986[] = {{&S97251[0], 73}, {NULL}};
const Segment SGI297987[] = {{&S97258[0], 70}, {NULL}};
const Segment SGI297988[] = {{&S97259[0], 69}, {NULL}};
const Segment SGI297989[] = {{&S87913[0], 9}, {NULL}};
const Segment SGI297990[] = {{&S97409[0], 101}, {NULL}};
const Segment SGI297991[] = {{&S97434[43], 12}, {NULL}};
const Segment SGI297992[] = {{&S97434[0], 100}, {NULL}};
const Segment SGI297993[] = {{&S97463[0], 85}, {NULL}};
const Segment SGI297994[] = {{&S88048[0], 9}, {NULL}};
const Segment SGI297995[] = {{&S97638[0], 70}, {NULL}};
const Segment SGI297996[] = {{&S97639[0], 71}, {NULL}};
const Segment SGI297997[] = {{&S97640[0], 122}, {NULL}};
const Segment SGI297998[] = {{&S97642[0], 70}, {NULL}};
const Segment SGI297999[] = {{&S97675[0], 81}, {NULL}};
const Segment SGI298000[] = {{&S97687[0], 66}, {NULL}};
const Segment SGI298001[] = {{&S97802[33], 5}, {NULL}};
const Segment SGI298002[] = {{&S97802[0], 73}, {NULL}};
const Segment SGI298003[] = {{&S97810[43], 7}, {NULL}};
const Segment SGI298004[] = {{&S97810[0], 95}, {NULL}};
const Segment SGI298005[] = {{&S97862[43], 3}, {NULL}};
const Segment SGI298006[] = {{&S97862[0], 94}, {NULL}};
const Segment SGI298007[] = {{&S97948[43], 6}, {NULL}};
const Segment SGI298008[] = {{&S97948[0], 94}, {NULL}};
const Segment SGI298009[] = {{&S97950[43], 6}, {NULL}};
const Segment SGI298010[] = {{&S97950[0], 97}, {NULL}};
const Segment SGI298011[] = {{&S97969[43], 9}, {NULL}};
const Segment SGI298012[] = {{&S97969[0], 97}, {NULL}};
const Segment SGI298013[] = {{&S98030[43], 6}, {NULL}};
const Segment SGI298014[] = {{&S98030[0], 97}, {NULL}};
const Segment SGI298015[] = {{&S88413[0], 9}, {NULL}};
const Segment SGI298016[] = {{&S98069[0], 95}, {NULL}};
const Segment SGI298017[] = {{&S98192[33], 5}, {NULL}};
const Segment SGI298018[] = {{&S98192[0], 73}, {NULL}};
const Segment SGI298019[] = {{&S98222[33], 5}, {NULL}};
const Segment SGI298020[] = {{&S98222[0], 73}, {NULL}};
const Segment SGI298021[] = {{&S98495[43], 8}, {NULL}};
const Segment SGI298022[] = {{&S98495[0], 103}, {NULL}};
const Segment SGI298023[] = {{&S98515[0], 76}, {NULL}};
const Segment SGI298024[] = {{&S98517[0], 87}, {NULL}};
const Segment SGI298025[] = {{&S98518[0], 89}, {NULL}};
const Segment SGI298026[] = {{&S98609[0], 81}, {NULL}};
const Segment SGI298027[] = {{&S88698[0], 9}, {NULL}};
const Segment SGI298028[] = {{&S98650[18], 9}, {NULL}};
const Segment SGI298029[] = {{&S98650[0], 127}, {NULL}};
const Segment SGI298030[] = {{&S98696[0], 79}, {NULL}};
const Segment SGI298031[] = {{&S98723[49], 9}, {NULL}};
const Segment SGI298032[] = {{&S98723[0], 103}, {NULL}};
const Segment SGI298033[] = {{&S98853[43], 6}, {NULL}};
const Segment SGI298034[] = {{&S98853[0], 97}, {NULL}};
const Segment SGI298035[] = {{&S98979[43], 13}, {NULL}};
const Segment SGI298036[] = {{&S98979[0], 101}, {NULL}};
const Segment SGI298037[] = {{&S88978[0], 9}, {NULL}};
const Segment SGI298038[] = {{&S99035[0], 71}, {NULL}};
const Segment SGI298039[] = {{&S99036[18], 8}, {NULL}};
const Segment SGI298040[] = {{&S99036[0], 127}, {NULL}};
const Segment SGI298041[] = {{&S99055[0], 82}, {NULL}};
const Segment SGI298042[] = {{&S99070[0], 75}, {NULL}};
const Segment SGI298043[] = {{&S99076[0], 67}, {NULL}};
const Segment SGI298044[] = {{&S99172[43], 5}, {NULL}};
const Segment SGI298045[] = {{&S99172[0], 69}, {NULL}};
const Segment SGI298046[] = {{&S99200[0], 81}, {NULL}};
const Segment SGI298047[] = {{&S99403[43], 6}, {NULL}};
const Segment SGI298048[] = {{&S99403[0], 106}, {NULL}};
const Segment SGI298049[] = {{&S99527[0], 70}, {NULL}};
const Segment SGI298050[] = {{&S99528[79], 5}, {NULL}};
const Segment SGI298051[] = {{&S99528[0], 104}, {NULL}};
const Segment SGI298052[] = {{&S99557[0], 174}, {NULL}};
const Segment SGI298053[] = {{&S99558[0], 186}, {NULL}};
const Segment SGI298054[] = {{&S99584[0], 81}, {NULL}};
const Segment SGI298055[] = {{&S7996[0], 85}, {NULL}};
const Segment SGI298056[] = {{&S99693[0], 73}, {NULL}};
const Segment SGI298057[] = {{&S99694[0], 67}, {NULL}};
const Segment SGI298058[] = {{&S99861[43], 7}, {NULL}};
const Segment SGI298059[] = {{&S99861[0], 95}, {NULL}};
const Segment SGI298060[] = {{&S99996[43], 5}, {NULL}};
const Segment SGI298061[] = {{&S99996[0], 93}, {NULL}};
const Segment SGI298062[] = {{&S100113[55], 12}, {NULL}};
const Segment SGI298063[] = {{&S100113[0], 87}, {NULL}};
const Segment SGI298064[] = {{&S100114[55], 14}, {NULL}};
const Segment SGI298065[] = {{&S100114[0], 89}, {NULL}};
const Segment SGI298066[] = {{&S100115[55], 14}, {NULL}};
const Segment SGI298067[] = {{&S100115[0], 93}, {NULL}};
const Segment SGI298068[] = {{&S90184[0], 9}, {NULL}};
const Segment SGI298069[] = {{&S100150[0], 88}, {NULL}};
const Segment SGI298070[] = {{&S100151[0], 85}, {NULL}};
const Segment SGI298071[] = {{&S100152[0], 88}, {NULL}};
const Segment SGI298072[] = {{&S100167[18], 9}, {NULL}};
const Segment SGI298073[] = {{&S100167[0], 139}, {NULL}};
const Segment SGI298074[] = {{&S100196[0], 66}, {NULL}};
const Segment SGI298075[] = {{&S100586[0], 100}, {NULL}};
const Segment SGI298076[] = {{&S100724[0], 71}, {NULL}};
const Segment SGI298077[] = {{&S100731[0], 71}, {NULL}};
const Segment SGI298078[] = {{&S100753[79], 9}, {NULL}};
const Segment SGI298079[] = {{&S100753[0], 128}, {NULL}};
const Segment SGI298080[] = {{&S100847[0], 70}, {NULL}};
const Segment SGI298081[] = {{&S90948[0], 9}, {NULL}};
const Segment SGI298082[] = {{&S100945[0], 79}, {NULL}};
const Segment SGI298083[] = {{&S100947[0], 94}, {NULL}};
const Segment SGI298084[] = {{&S100995[0], 63}, {NULL}};
const Segment SGI298085[] = {{&S100996[0], 63}, {NULL}};
const Segment SGI298086[] = {{&S91121[0], 9}, {NULL}};
const Segment SGI298087[] = {{&S101132[0], 109}, {NULL}};
const Segment SGI298088[] = {{&S101133[0], 117}, {NULL}};
const Segment SGI298089[] = {{&S101146[0], 46}, {NULL}};
const Segment SGI298090[] = {{&S101147[0], 98}, {NULL}};
const Segment SGI298091[] = {{&S101182[0], 73}, {NULL}};
const Segment SGI298092[] = {{&S101186[0], 92}, {NULL}};
const Segment SGI298093[] = {{&S101278[0], 100}, {NULL}};
const Segment SGI298094[] = {{&S101279[65], 6}, {NULL}};
const Segment SGI298095[] = {{&S101279[0], 71}, {NULL}};
const Segment SGI298096[] = {{&S101280[0], 87}, {NULL}};
const Segment SGI298097[] = {{&S101284[79], 9}, {NULL}};
const Segment SGI298098[] = {{&S101284[0], 128}, {NULL}};
const Segment SGI298099[] = {{&S101313[0], 76}, {NULL}};
const Segment SGI298100[] = {{&S101316[0], 140}, {NULL}};
const Segment SGI298101[] = {{&S101317[0], 87}, {NULL}};
const Segment SGI298102[] = {{&S101330[0], 130}, {NULL}};
const Segment SGI298103[] = {{&S101378[48], 10}, {NULL}};
const Segment SGI298104[] = {{&S101378[0], 81}, {NULL}};
const Segment SGI298105[] = {{&S101476[0], 90}, {NULL}};
const Segment SGI298106[] = {{&S101500[0], 82}, {NULL}};
const Segment SGI298107[] = {{&S101519[0], 63}, {NULL}};
const Segment SGI298108[] = {{&S101535[0], 66}, {NULL}};
const Segment SGI298109[] = {{&S101603[79], 9}, {NULL}};
const Segment SGI298110[] = {{&S101603[0], 128}, {NULL}};
const Segment SGI298111[] = {{&S101632[0], 70}, {NULL}};
const Segment SGI298112[] = {{&S101637[18], 9}, {NULL}};
const Segment SGI298113[] = {{&S101637[0], 128}, {NULL}};
const Segment SGI298114[] = {{&S101690[0], 63}, {NULL}};
const Segment SGI298115[] = {{&S101691[0], 63}, {NULL}};
const Segment SGI298116[] = {{&S101714[0], 103}, {NULL}};
const Segment SGI298117[] = {{&S91881[0], 9}, {NULL}};
const Segment SGI298118[] = {{&S91882[0], 9}, {NULL}};
const Segment SGI298119[] = {{&S101805[0], 82}, {NULL}};
const Segment SGI298120[] = {{&S101821[0], 85}, {NULL}};
const Segment SGI298121[] = {{&S101824[18], 9}, {NULL}};
const Segment SGI298122[] = {{&S101824[0], 139}, {NULL}};
const Segment SGI298123[] = {{&S101832[0], 44}, {NULL}};
const Segment SGI298124[] = {{&S101957[0], 70}, {NULL}};
const Segment SGI298125[] = {{&S101958[0], 71}, {NULL}};
const Segment SGI298126[] = {{&S101960[0], 122}, {NULL}};
const Segment SGI298127[] = {{&S101961[0], 53}, {NULL}};
const Segment SGI298128[] = {{&S101962[0], 57}, {NULL}};
const Segment SGI298129[] = {{&S101963[0], 57}, {NULL}};
const Segment SGI298130[] = {{&S101973[79], 8}, {NULL}};
const Segment SGI298131[] = {{&S101973[0], 127}, {NULL}};
const Segment SGI298132[] = {{&S101974[0], 70}, {NULL}};
const Segment SGI298133[] = {{&S101993[0], 96}, {NULL}};
const Segment SGI298134[] = {{&S101994[0], 96}, {NULL}};
const Segment SGI298135[] = {{&S92087[0], 9}, {NULL}};
const Segment SGI298136[] = {{&S92089[0], 9}, {NULL}};
const Segment SGI298137[] = {{&S102028[18], 9}, {NULL}};
const Segment SGI298138[] = {{&S102028[0], 116}, {NULL}};
const Segment SGI298139[] = {{&S102038[0], 75}, {NULL}};
const Segment SGI298140[] = {{&S102053[0], 71}, {NULL}};
const Segment SGI298141[] = {{&S102126[48], 5}, {NULL}};
const Segment SGI298142[] = {{&S102126[0], 75}, {NULL}};
const Segment SGI298143[] = {{&S102176[0], 97}, {NULL}};
const Segment SGI298144[] = {{&S102195[0], 194}, {NULL}};
const Segment SGI298145[] = {{&S102196[0], 195}, {NULL}};
const Segment SGI298146[] = {{&S102206[0], 88}, {NULL}};
const Segment SGI298147[] = {{&S92295[0], 9}, {NULL}};
const Segment SGI298148[] = {{&S92297[0], 9}, {NULL}};
const Segment SGI298149[] = {{&S102329[0], 96}, {NULL}};
const Segment SGI298150[] = {{&S102343[0], 69}, {NULL}};
const Segment SGI298151[] = {{&S102344[0], 72}, {NULL}};
const Segment SGI298152[] = {{&S102381[79], 8}, {NULL}};
const Segment SGI298153[] = {{&S102381[0], 127}, {NULL}};
const Segment SGI298154[] = {{&S102435[0], 63}, {NULL}};
const Segment SGI298155[] = {{&S102436[0], 63}, {NULL}};
const Segment SGI298156[] = {{&S102545[0], 108}, {NULL}};
const Segment SGI298157[] = {{&S102546[0], 71}, {NULL}};
const Segment SGI298158[] = {{&S102547[18], 9}, {NULL}};
const Segment SGI298159[] = {{&S102547[0], 128}, {NULL}};
const Segment SGI298160[] = {{&S102577[0], 97}, {NULL}};
const Segment SGI298161[] = {{&S102763[0], 70}, {NULL}};
const Segment SGI298162[] = {{&S102779[0], 80}, {NULL}};
const Segment SGI298163[] = {{&S102806[0], 70}, {NULL}};
const Segment SGI298164[] = {{&S102826[0], 47}, {NULL}};
const Segment SGI298165[] = {{&S92841[0], 9}, {NULL}};
const Segment SGI298166[] = {{&S102957[0], 75}, {NULL}};
const Segment SGI298167[] = {{&S102964[65], 5}, {NULL}};
const Segment SGI298168[] = {{&S102964[0], 107}, {NULL}};
const Segment SGI298169[] = {{&S102965[65], 9}, {NULL}};
const Segment SGI298170[] = {{&S102965[0], 131}, {NULL}};
const Segment SGI298171[] = {{&S102971[0], 71}, {NULL}};
const Segment SGI298172[] = {{&S103008[18], 9}, {NULL}};
const Segment SGI298173[] = {{&S103008[0], 129}, {NULL}};
const Segment SGI298174[] = {{&S103024[0], 98}, {NULL}};
const Segment SGI298175[] = {{&S103031[0], 97}, {NULL}};
const Segment SGI298176[] = {{&S103046[17], 8}, {NULL}};
const Segment SGI298177[] = {{&S103046[0], 141}, {NULL}};
const Segment SGI298178[] = {{&S93285[0], 9}, {NULL}};
const Segment SGI298179[] = {{&S93286[0], 9}, {NULL}};
const Segment SGI298180[] = {{&S103199[48], 6}, {NULL}};
const Segment SGI298181[] = {{&S103199[0], 77}, {NULL}};
const Segment SGI298182[] = {{&S103218[0], 121}, {NULL}};
const Segment SGI298183[] = {{&S103219[0], 105}, {NULL}};
const Segment SGI298184[] = {{&S103220[0], 105}, {NULL}};
const Segment SGI298185[] = {{&S103221[0], 121}, {NULL}};
const Segment SGI298186[] = {{&S103275[0], 62}, {NULL}};
const Segment SGI298187[] = {{&S103276[0], 58}, {NULL}};
const Segment SGI298188[] = {{&S103296[0], 70}, {NULL}};
const Segment SGI298189[] = {{&S103310[0], 96}, {NULL}};
const Segment SGI298190[] = {{&S103321[0], 60}, {NULL}};
const Segment SGI298191[] = {{&S103322[0], 70}, {NULL}};
const Segment SGI298192[] = {{&S103323[79], 9}, {NULL}};
const Segment SGI298193[] = {{&S103323[0], 128}, {NULL}};
const Segment SGI298194[] = {{&S103357[0], 99}, {NULL}};
const Segment SGI298195[] = {{&S103399[66], 7}, {NULL}};
const Segment SGI298196[] = {{&S103399[0], 87}, {NULL}};
const Segment SGI298197[] = {{&S103400[71], 7}, {NULL}};
const Segment SGI298198[] = {{&S103400[0], 92}, {NULL}};
const Segment SGI298199[] = {{&S103404[31], 10}, {NULL}};
const Segment SGI298200[] = {{&S103404[0], 74}, {NULL}};
const Segment SGI298201[] = {{&S93533[0], 9}, {NULL}};
const Segment SGI298202[] = {{&S103510[0], 75}, {NULL}};
const Segment SGI298203[] = {{&S103511[66], 17}, {NULL}};
const Segment SGI298204[] = {{&S103511[0], 83}, {NULL}};
const Segment SGI298205[] = {{&S103512[79], 10}, {NULL}};
const Segment SGI298206[] = {{&S103512[0], 129}, {NULL}};
const Segment SGI298207[] = {{&S103516[18], 10}, {NULL}};
const Segment SGI298208[] = {{&S103516[0], 130}, {NULL}};
const Segment SGI298209[] = {{&S103517[18], 10}, {NULL}};
const Segment SGI298210[] = {{&S103517[0], 129}, {NULL}};
const Segment SGI298211[] = {{&S103519[0], 91}, {NULL}};
const Segment SGI298212[] = {{&S103520[0], 88}, {NULL}};
const Segment SGI298213[] = {{&S103544[0], 71}, {NULL}};
const Segment SGI298214[] = {{&S103569[0], 86}, {NULL}};
const Segment SGI298215[] = {{&S103591[0], 72}, {NULL}};
const Segment SGI298216[] = {{&S103644[73], 7}, {NULL}};
const Segment SGI298217[] = {{&S103644[0], 80}, {NULL}};
const Segment SGI298218[] = {{&S103669[0], 74}, {NULL}};
const Segment SGI298219[] = {{&S103837[18], 9}, {NULL}};
const Segment SGI298220[] = {{&S103837[0], 145}, {NULL}};
const Segment SGI298221[] = {{&S93756[0], 9}, {NULL}};
const Segment SGI298222[] = {{&S93757[0], 9}, {NULL}};
const Segment SGI298223[] = {{&S103860[0], 88}, {NULL}};
const Segment SGI298224[] = {{&S103866[0], 73}, {NULL}};
const Segment SGI298225[] = {{&S103867[0], 77}, {NULL}};
const Segment SGI298226[] = {{&S103881[0], 77}, {NULL}};
const Segment SGI298227[] = {{&S103930[0], 77}, {NULL}};
const Segment SGI298228[] = {{&S104003[0], 81}, {NULL}};
const Segment SGI298229[] = {{&S104028[0], 64}, {NULL}};
const Segment SGI298230[] = {{&S104164[55], 18}, {NULL}};
const Segment SGI298231[] = {{&S104164[0], 93}, {NULL}};
const Segment SGI298232[] = {{&S104169[0], 55}, {NULL}};
const Segment SGI298233[] = {{&S104179[96], 9}, {NULL}};
const Segment SGI298234[] = {{&S104179[0], 145}, {NULL}};
const Segment SGI298235[] = {{&S104184[0], 74}, {NULL}};
const Segment SGI298236[] = {{&S104217[0], 172}, {NULL}};
const Segment SGI298237[] = {{&S104226[18], 8}, {NULL}};
const Segment SGI298238[] = {{&S104226[0], 140}, {NULL}};
const Segment SGI298239[] = {{&S104244[0], 189}, {NULL}};
const Segment SGI298240[] = {{&S104307[79], 10}, {NULL}};
const Segment SGI298241[] = {{&S104307[0], 129}, {NULL}};
const Segment SGI298242[] = {{&S104313[0], 102}, {NULL}};
const Segment SGI298243[] = {{&S104338[0], 63}, {NULL}};
const Segment SGI298244[] = {{&S104408[0], 64}, {NULL}};
const Segment SGI298245[] = {{&S104564[0], 70}, {NULL}};
const Segment SGI298246[] = {{&S104565[0], 74}, {NULL}};
const Segment SGI298247[] = {{&S104571[18], 10}, {NULL}};
const Segment SGI298248[] = {{&S104571[0], 130}, {NULL}};
const Segment SGI298249[] = {{&S104580[0], 86}, {NULL}};
const Segment SGI298250[] = {{&S104595[0], 70}, {NULL}};
const Segment SGI298251[] = {{&S104635[0], 70}, {NULL}};
const Segment SGI298252[] = {{&S94667[0], 9}, {NULL}};
const Segment SGI298253[] = {{&S104644[0], 122}, {NULL}};
const Segment SGI298254[] = {{&S94668[0], 9}, {NULL}};
const Segment SGI298255[] = {{&S104645[0], 71}, {NULL}};
const Segment SGI298256[] = {{&S104661[0], 70}, {NULL}};
const Segment SGI298257[] = {{&S104685[0], 81}, {NULL}};
const Segment SGI298258[] = {{&S104693[0], 87}, {NULL}};
const Segment SGI298259[] = {{&S104745[0], 96}, {NULL}};
const Segment SGI298260[] = {{&S104778[0], 73}, {NULL}};
const Segment SGI298261[] = {{&S104779[55], 5}, {NULL}};
const Segment SGI298262[] = {{&S104779[0], 80}, {NULL}};
const Segment SGI298263[] = {{&S104853[0], 73}, {NULL}};
const Segment SGI298264[] = {{&S104854[17], 9}, {NULL}};
const Segment SGI298265[] = {{&S104854[0], 33}, {NULL}};
const Segment SGI298266[] = {{&S104862[0], 64}, {NULL}};
const Segment SGI298267[] = {{&S95145[0], 9}, {NULL}};
const Segment SGI298268[] = {{&S105098[0], 86}, {NULL}};
const Segment SGI298269[] = {{&S105109[0], 74}, {NULL}};
const Segment SGI298270[] = {{&S105133[0], 77}, {NULL}};
const Segment SGI298271[] = {{&S105157[0], 87}, {NULL}};
const Segment SGI298272[] = {{&S105175[0], 101}, {NULL}};
const Segment SGI298273[] = {{&S105214[0], 73}, {NULL}};
const Segment SGI298274[] = {{&S105224[18], 8}, {NULL}};
const Segment SGI298275[] = {{&S105224[0], 139}, {NULL}};
const Segment SGI298276[] = {{&S105233[0], 91}, {NULL}};
const Segment SGI298277[] = {{&S105338[0], 92}, {NULL}};
const Segment SGI298278[] = {{&S105350[0], 102}, {NULL}};
const Segment SGI298279[] = {{&S95380[0], 9}, {NULL}};
const Segment SGI298280[] = {{&S105404[0], 74}, {NULL}};
const Segment SGI298281[] = {{&S105405[0], 73}, {NULL}};
const Segment SGI298282[] = {{&S105416[0], 68}, {NULL}};
const Segment SGI298283[] = {{&S105417[0], 134}, {NULL}};
const Segment SGI298284[] = {{&S105421[0], 70}, {NULL}};
const Segment SGI298285[] = {{&S105430[0], 73}, {NULL}};
const Segment SGI298286[] = {{&S105441[48], 9}, {NULL}};
const Segment SGI298287[] = {{&S105441[0], 122}, {NULL}};
const Segment SGI298288[] = {{&S105572[19], 9}, {NULL}};
const Segment SGI298289[] = {{&S105572[0], 28}, {NULL}};
const Segment SGI298290[] = {{&S95692[0], 9}, {NULL}};
const Segment SGI298291[] = {{&S105574[0], 159}, {NULL}};
const Segment SGI298292[] = {{&S95694[0], 9}, {NULL}};
const Segment SGI298293[] = {{&S95985[0], 9}, {NULL}};
const Segment SGI298294[] = {{&S96655[0], 9}, {NULL}};
const Segment SGI298295[] = {{&S96943[0], 9}, {NULL}};
const Segment SGI298296[] = {{&S97267[0], 9}, {NULL}};
const Segment SGI298297[] = {{&S97271[0], 9}, {NULL}};
const Segment SGI298298[] = {{&S97272[0], 9}, {NULL}};
const Segment SGI298299[] = {{&S97663[0], 9}, {NULL}};
const Segment SGI298300[] = {{&S98085[0], 9}, {NULL}};
const Segment SGI298301[] = {{&S99577[0], 9}, {NULL}};
const Segment SGI298302[] = {{&S99579[0], 9}, {NULL}};
const Segment SGI298303[] = {{&S99592[0], 9}, {NULL}};
const Segment SGI298304[] = {{&S100318[0], 9}, {NULL}};
const Segment SGI298305[] = {{&S100756[0], 9}, {NULL}};
const Segment SGI298306[] = {{&S100757[0], 9}, {NULL}};
const Segment SGI298307[] = {{&S100987[0], 9}, {NULL}};
const Segment SGI298308[] = {{&S100988[0], 9}, {NULL}};
const Segment SGI298309[] = {{&S101192[0], 9}, {NULL}};
const Segment SGI298310[] = {{&S101193[0], 9}, {NULL}};
const Segment SGI298311[] = {{&S101332[0], 9}, {NULL}};
const Segment SGI298312[] = {{&S101485[0], 9}, {NULL}};
const Segment SGI298313[] = {{&S101486[0], 9}, {NULL}};
const Segment SGI298314[] = {{&S101677[0], 9}, {NULL}};
const Segment SGI298315[] = {{&S101833[0], 9}, {NULL}};
const Segment SGI298316[] = {{&S102027[0], 9}, {NULL}};
const Segment SGI298317[] = {{&S102219[0], 9}, {NULL}};
const Segment SGI298318[] = {{&S102228[0], 9}, {NULL}};
const Segment SGI298319[] = {{&S4710[0], 28}, {NULL}};
const Segment SGI298320[] = {{&S4712[0], 13}, {NULL}};
const Segment SGI298321[] = {{&S4713[0], 27}, {NULL}};
const Segment SGI298322[] = {{&S4718[0], 13}, {NULL}};
const Segment SGI298323[] = {{&S4719[0], 38}, {NULL}};
const Segment SGI298324[] = {{&S4720[0], 25}, {NULL}};
const Segment SGI298325[] = {{&S4721[0], 56}, {NULL}};
const Segment SGI298326[] = {{&S4723[0], 17}, {NULL}};
const Segment SGI298327[] = {{&S4724[0], 61}, {NULL}};
const Segment SGI298328[] = {{&S4725[0], 61}, {NULL}};
const Segment SGI298329[] = {{&S4730[0], 20}, {NULL}};
const Segment SGI298330[] = {{&S4731[0], 17}, {NULL}};
const Segment SGI298331[] = {{&S4732[0], 17}, {NULL}};
const Segment SGI298332[] = {{&S102401[0], 9}, {NULL}};
const Segment SGI298333[] = {{&S102402[0], 9}, {NULL}};
const Segment SGI298334[] = {{&S102406[0], 9}, {NULL}};
const Segment SGI298335[] = {{&S102410[0], 9}, {NULL}};
const Segment SGI298336[] = {{&S102626[0], 9}, {NULL}};
const Segment SGI298337[] = {{&S103058[0], 9}, {NULL}};
const Segment SGI298338[] = {{&S103331[0], 9}, {NULL}};
const Segment SGI298339[] = {{&S103333[0], 9}, {NULL}};
const Segment SGI298340[] = {{&S103908[0], 9}, {NULL}};
const Segment SGI298341[] = {{&S104306[0], 9}, {NULL}};
const Segment SGI298342[] = {{&S104739[0], 9}, {NULL}};
const Segment SGI298343[] = {{&S105414[0], 9}, {NULL}};
const Segment SGI298344[] = {{&S105426[0], 9}, {NULL}};
const Segment SGI298345[] = {{&S105427[0], 9}, {NULL}};
const Segment SGI298346[] = {{&S154[30], 3}, {NULL}};
const Segment SGI298347[] = {{&S154[8], 3}, {NULL}};
const Segment SGI298348[] = {{&S154[13], 10}, {NULL}};
const Segment SGI298349[] = {{&S154[42], 7}, {NULL}};
const Segment SGI298350[] = {{&S154[35], 14}, {NULL}};
const Segment SGI298351[] = {{&S154[64], 15}, {NULL}};
const Segment SGI298352[] = {{&S155[30], 3}, {&S155[94], 3}, {NULL}};
const Segment SGI298353[] = {{&S155[8], 3}, {&S155[8], 3}, {NULL}};
const Segment SGI298354[] = {{&S155[13], 10}, {&S155[13], 10}, {NULL}};
const Segment SGI298355[] = {{&S155[42], 7}, {&S155[106], 7}, {NULL}};
const Segment SGI298356[] = {{&S155[35], 14}, {&S155[99], 14}, {NULL}};
const Segment SGI298357[] = {{&S155[115], 3}, {NULL}};
const Segment SGI298358[] = {{&S156[30], 3}, {NULL}};
const Segment SGI298359[] = {{&S156[8], 3}, {NULL}};
const Segment SGI298360[] = {{&S156[13], 10}, {NULL}};
const Segment SGI298361[] = {{&S156[42], 7}, {NULL}};
const Segment SGI298362[] = {{&S156[35], 14}, {NULL}};
const Segment SGI298363[] = {{&S157[30], 3}, {NULL}};
const Segment SGI298364[] = {{&S157[8], 3}, {NULL}};
const Segment SGI298365[] = {{&S157[13], 10}, {NULL}};
const Segment SGI298366[] = {{&S157[42], 7}, {NULL}};
const Segment SGI298367[] = {{&S157[35], 14}, {NULL}};
const Segment SGI298368[] = {{&S157[64], 3}, {NULL}};
const Segment SGI298369[] = {{&S158[30], 3}, {NULL}};
const Segment SGI298370[] = {{&S158[8], 3}, {NULL}};
const Segment SGI298371[] = {{&S158[13], 10}, {NULL}};
const Segment SGI298372[] = {{&S158[42], 7}, {NULL}};
const Segment SGI298373[] = {{&S159[30], 3}, {NULL}};
const Segment SGI298374[] = {{&S159[8], 3}, {NULL}};
const Segment SGI298375[] = {{&S159[13], 10}, {NULL}};
const Segment SGI298376[] = {{&S159[42], 7}, {NULL}};
const Segment SGI298377[] = {{&S159[35], 14}, {NULL}};
const Segment SGI298378[] = {{&S159[64], 9}, {NULL}};
const Segment SGI298379[] = {{&S160[31], 3}, {NULL}};
const Segment SGI298380[] = {{&S160[43], 7}, {NULL}};
const Segment SGI298381[] = {{&S161[30], 3}, {NULL}};
const Segment SGI298382[] = {{&S161[8], 3}, {NULL}};
const Segment SGI298383[] = {{&S161[13], 10}, {NULL}};
const Segment SGI298384[] = {{&S161[42], 7}, {NULL}};
const Segment SGI298385[] = {{&S161[35], 14}, {NULL}};
const Segment SGI298386[] = {{&S161[102], 15}, {NULL}};
const Segment SGI298387[] = {{&S162[30], 3}, {&S162[94], 3}, {NULL}};
const Segment SGI298388[] = {{&S162[8], 3}, {&S162[8], 3}, {NULL}};
const Segment SGI298389[] = {{&S162[13], 10}, {&S162[13], 10}, {NULL}};
const Segment SGI298390[] = {{&S162[42], 7}, {&S162[42], 7}, {NULL}};
const Segment SGI298391[] = {{&S162[35], 14}, {&S162[35], 14}, {NULL}};
const Segment SGI298392[] = {{&S162[160], 15}, {NULL}};
const Segment SGI298393[] = {{&S162[115], 3}, {NULL}};
const Segment SGI298394[] = {{&S163[30], 3}, {NULL}};
const Segment SGI298395[] = {{&S163[8], 3}, {NULL}};
const Segment SGI298396[] = {{&S163[13], 10}, {NULL}};
const Segment SGI298397[] = {{&S163[42], 7}, {NULL}};
const Segment SGI298398[] = {{&S163[35], 14}, {NULL}};
const Segment SGI298399[] = {{&S164[30], 3}, {NULL}};
const Segment SGI298400[] = {{&S164[8], 3}, {NULL}};
const Segment SGI298401[] = {{&S164[13], 10}, {NULL}};
const Segment SGI298402[] = {{&S164[42], 7}, {NULL}};
const Segment SGI298403[] = {{&S164[35], 14}, {NULL}};
const Segment SGI298404[] = {{&S164[64], 3}, {NULL}};
const Segment SGI298405[] = {{&S165[30], 3}, {NULL}};
const Segment SGI298406[] = {{&S165[8], 3}, {NULL}};
const Segment SGI298407[] = {{&S165[13], 10}, {NULL}};
const Segment SGI298408[] = {{&S165[42], 7}, {NULL}};
const Segment SGI298409[] = {{&S165[35], 14}, {NULL}};
const Segment SGI298410[] = {{&S165[51], 6}, {NULL}};
const Segment SGI298411[] = {{&S166[30], 3}, {&S166[106], 3}, {&S166[106], 3}, {NULL}};
const Segment SGI298412[] = {{&S166[8], 3}, {&S166[8], 3}, {&S166[8], 3}, {NULL}};
const Segment SGI298413[] = {{&S166[13], 10}, {&S166[13], 10}, {&S166[13], 10}, {NULL}};
const Segment SGI298414[] = {{&S166[42], 7}, {&S166[42], 7}, {&S166[42], 7}, {NULL}};
const Segment SGI298415[] = {{&S166[35], 14}, {&S166[35], 14}, {&S166[35], 14}, {NULL}};
const Segment SGI298416[] = {{&S166[127], 3}, {&S166[127], 3}, {NULL}};
const Segment SGI298417[] = {{&S167[30], 3}, {&S167[94], 3}, {NULL}};
const Segment SGI298418[] = {{&S167[8], 3}, {&S167[8], 3}, {NULL}};
const Segment SGI298419[] = {{&S167[13], 10}, {&S167[13], 10}, {NULL}};
const Segment SGI298420[] = {{&S167[42], 7}, {&S167[42], 7}, {NULL}};
const Segment SGI298421[] = {{&S167[35], 14}, {NULL}};
const Segment SGI298422[] = {{&S168[30], 3}, {NULL}};
const Segment SGI298423[] = {{&S168[8], 3}, {NULL}};
const Segment SGI298424[] = {{&S168[13], 10}, {NULL}};
const Segment SGI298425[] = {{&S168[42], 7}, {NULL}};
const Segment SGI298426[] = {{&S168[35], 14}, {NULL}};
const Segment SGI298427[] = {{&S168[51], 5}, {NULL}};
const Segment SGI298428[] = {{&S169[30], 3}, {&S169[94], 3}, {NULL}};
const Segment SGI298429[] = {{&S169[8], 3}, {&S169[8], 3}, {NULL}};
const Segment SGI298430[] = {{&S169[13], 10}, {&S169[13], 10}, {NULL}};
const Segment SGI298431[] = {{&S169[42], 7}, {&S169[42], 7}, {NULL}};
const Segment SGI298432[] = {{&S169[35], 14}, {&S169[35], 14}, {NULL}};
const Segment SGI298433[] = {{&S169[122], 9}, {NULL}};
const Segment SGI298434[] = {{&S169[115], 3}, {NULL}};
const Segment SGI298435[] = {{&S170[30], 3}, {NULL}};
const Segment SGI298436[] = {{&S170[8], 3}, {NULL}};
const Segment SGI298437[] = {{&S170[13], 10}, {NULL}};
const Segment SGI298438[] = {{&S170[42], 7}, {NULL}};
const Segment SGI298439[] = {{&S171[30], 3}, {NULL}};
const Segment SGI298440[] = {{&S171[8], 3}, {NULL}};
const Segment SGI298441[] = {{&S171[13], 10}, {NULL}};
const Segment SGI298442[] = {{&S171[42], 7}, {NULL}};
const Segment SGI298443[] = {{&S171[35], 14}, {NULL}};
const Segment SGI298444[] = {{&S171[102], 15}, {&S171[102], 15}, {NULL}};
const Segment SGI298445[] = {{&S172[30], 3}, {&S172[94], 3}, {NULL}};
const Segment SGI298446[] = {{&S172[8], 3}, {&S172[8], 3}, {NULL}};
const Segment SGI298447[] = {{&S172[13], 10}, {&S172[13], 10}, {NULL}};
const Segment SGI298448[] = {{&S172[42], 7}, {NULL}};
const Segment SGI298449[] = {{&S172[35], 14}, {NULL}};
const Segment SGI298450[] = {{&S172[99], 5}, {NULL}};
const Segment SGI298451[] = {{&S173[30], 3}, {&S173[30], 3}, {NULL}};
const Segment SGI298452[] = {{&S173[8], 3}, {&S173[8], 3}, {NULL}};
const Segment SGI298453[] = {{&S173[13], 10}, {&S173[13], 10}, {NULL}};
const Segment SGI298454[] = {{&S173[42], 7}, {NULL}};
const Segment SGI298455[] = {{&S173[35], 14}, {NULL}};
const Segment SGI298456[] = {{&S173[99], 5}, {NULL}};
const Segment SGI298457[] = {{&S174[30], 3}, {&S174[94], 3}, {NULL}};
const Segment SGI298458[] = {{&S174[8], 3}, {&S174[8], 3}, {NULL}};
const Segment SGI298459[] = {{&S174[13], 10}, {&S174[13], 10}, {NULL}};
const Segment SGI298460[] = {{&S174[42], 7}, {&S174[42], 7}, {NULL}};
const Segment SGI298461[] = {{&S174[35], 14}, {&S174[35], 14}, {NULL}};
const Segment SGI298462[] = {{&S174[205], 4}, {NULL}};
const Segment SGI298463[] = {{&S174[115], 3}, {NULL}};
const Segment SGI298464[] = {{&S175[30], 3}, {NULL}};
const Segment SGI298465[] = {{&S175[8], 3}, {NULL}};
const Segment SGI298466[] = {{&S175[13], 10}, {NULL}};
const Segment SGI298467[] = {{&S175[42], 7}, {NULL}};
const Segment SGI298468[] = {{&S175[35], 14}, {NULL}};
const Segment SGI298469[] = {{&S175[128], 4}, {NULL}};
const Segment SGI298470[] = {{&S176[30], 3}, {NULL}};
const Segment SGI298471[] = {{&S176[8], 3}, {NULL}};
const Segment SGI298472[] = {{&S176[13], 10}, {NULL}};
const Segment SGI298473[] = {{&S176[42], 7}, {NULL}};
const Segment SGI298474[] = {{&S176[35], 14}, {NULL}};
const Segment SGI298475[] = {{&S176[102], 15}, {NULL}};
const Segment SGI298476[] = {{&S176[187], 4}, {NULL}};
const Segment SGI298477[] = {{&S177[30], 3}, {NULL}};
const Segment SGI298478[] = {{&S177[8], 3}, {NULL}};
const Segment SGI298479[] = {{&S177[13], 10}, {NULL}};
const Segment SGI298480[] = {{&S177[42], 7}, {NULL}};
const Segment SGI298481[] = {{&S177[35], 14}, {NULL}};
const Segment SGI298482[] = {{&S177[181], 4}, {NULL}};
const Segment SGI298483[] = {{&S177[64], 9}, {NULL}};
const Segment SGI298484[] = {{&S178[30], 3}, {&S178[30], 3}, {NULL}};
const Segment SGI298485[] = {{&S178[8], 3}, {NULL}};
const Segment SGI298486[] = {{&S178[13], 10}, {NULL}};
const Segment SGI298487[] = {{&S178[42], 7}, {NULL}};
const Segment SGI298488[] = {{&S178[35], 14}, {NULL}};
const Segment SGI298489[] = {{&S179[31], 3}, {NULL}};
const Segment SGI298490[] = {{&S179[43], 7}, {NULL}};
const Segment SGI298491[] = {{&S180[30], 3}, {NULL}};
const Segment SGI298492[] = {{&S180[8], 3}, {NULL}};
const Segment SGI298493[] = {{&S180[13], 10}, {NULL}};
const Segment SGI298494[] = {{&S180[42], 7}, {NULL}};
const Segment SGI298495[] = {{&S180[35], 14}, {NULL}};
const Segment SGI298496[] = {{&S180[79], 15}, {NULL}};
const Segment SGI298497[] = {{&S180[64], 9}, {NULL}};
const Segment SGI298498[] = {{&S181[30], 3}, {NULL}};
const Segment SGI298499[] = {{&S181[8], 3}, {NULL}};
const Segment SGI298500[] = {{&S181[13], 10}, {NULL}};
const Segment SGI298501[] = {{&S181[42], 7}, {NULL}};
const Segment SGI298502[] = {{&S181[35], 14}, {NULL}};
const Segment SGI298503[] = {{&S181[107], 15}, {&S181[107], 15}, {NULL}};
const Segment SGI298504[] = {{&S181[64], 3}, {NULL}};
const Segment SGI298505[] = {{&S182[30], 3}, {NULL}};
const Segment SGI298506[] = {{&S182[8], 3}, {NULL}};
const Segment SGI298507[] = {{&S182[13], 10}, {NULL}};
const Segment SGI298508[] = {{&S182[42], 7}, {NULL}};
const Segment SGI298509[] = {{&S182[35], 14}, {NULL}};
const Segment SGI298510[] = {{&S182[107], 15}, {NULL}};
const Segment SGI298511[] = {{&S182[64], 3}, {NULL}};
const Segment SGI298512[] = {{&S183[30], 3}, {&S183[101], 3}, {NULL}};
const Segment SGI298513[] = {{&S183[8], 3}, {&S183[8], 3}, {NULL}};
const Segment SGI298514[] = {{&S183[13], 10}, {&S183[13], 10}, {NULL}};
const Segment SGI298515[] = {{&S183[42], 7}, {&S183[113], 7}, {NULL}};
const Segment SGI298516[] = {{&S183[35], 14}, {&S183[106], 14}, {NULL}};
const Segment SGI298517[] = {{&S183[51], 6}, {NULL}};
const Segment SGI298518[] = {{&S183[122], 3}, {NULL}};
const Segment SGI298519[] = {{&S184[30], 3}, {&S184[99], 3}, {NULL}};
const Segment SGI298520[] = {{&S184[8], 3}, {&S184[8], 3}, {NULL}};
const Segment SGI298521[] = {{&S184[13], 10}, {&S184[13], 10}, {NULL}};
const Segment SGI298522[] = {{&S184[42], 7}, {&S184[42], 7}, {NULL}};
const Segment SGI298523[] = {{&S184[35], 14}, {&S184[35], 14}, {NULL}};
const Segment SGI298524[] = {{&S184[64], 3}, {&S184[64], 3}, {NULL}};
const Segment SGI298525[] = {{&S185[30], 3}, {&S185[94], 3}, {NULL}};
const Segment SGI298526[] = {{&S185[8], 3}, {&S185[8], 3}, {NULL}};
const Segment SGI298527[] = {{&S185[13], 10}, {&S185[13], 10}, {NULL}};
const Segment SGI298528[] = {{&S185[42], 7}, {NULL}};
const Segment SGI298529[] = {{&S185[35], 14}, {NULL}};
const Segment SGI298530[] = {{&S185[99], 5}, {NULL}};
const Segment SGI298531[] = {{&S186[30], 3}, {NULL}};
const Segment SGI298532[] = {{&S186[8], 3}, {&S186[8], 3}, {&S186[8], 3}, {NULL}};
const Segment SGI298533[] = {{&S186[13], 10}, {NULL}};
const Segment SGI298534[] = {{&S186[42], 7}, {NULL}};
const Segment SGI298535[] = {{&S186[35], 14}, {NULL}};
const Segment SGI298536[] = {{&S187[30], 3}, {&S187[30], 3}, {&S187[30], 3}, {NULL}};
const Segment SGI298537[] = {{&S187[8], 3}, {NULL}};
const Segment SGI298538[] = {{&S187[13], 10}, {NULL}};
const Segment SGI298539[] = {{&S187[42], 7}, {NULL}};
const Segment SGI298540[] = {{&S187[35], 14}, {NULL}};
const Segment SGI298541[] = {{&S188[30], 3}, {&S188[40], 3}, {NULL}};
const Segment SGI298542[] = {{&S188[8], 3}, {NULL}};
const Segment SGI298543[] = {{&S188[13], 10}, {NULL}};
const Segment SGI298544[] = {{&S188[57], 7}, {NULL}};
const Segment SGI298545[] = {{&S188[50], 14}, {NULL}};
const Segment SGI298546[] = {{&S189[30], 3}, {NULL}};
const Segment SGI298547[] = {{&S189[8], 3}, {&S189[8], 3}, {NULL}};
const Segment SGI298548[] = {{&S189[13], 10}, {NULL}};
const Segment SGI298549[] = {{&S189[42], 7}, {NULL}};
const Segment SGI298550[] = {{&S189[35], 14}, {NULL}};
const Segment SGI298551[] = {{&S190[30], 3}, {&S190[94], 3}, {NULL}};
const Segment SGI298552[] = {{&S190[8], 3}, {&S190[8], 3}, {NULL}};
const Segment SGI298553[] = {{&S190[13], 10}, {&S190[13], 10}, {NULL}};
const Segment SGI298554[] = {{&S190[42], 7}, {&S190[42], 7}, {NULL}};
const Segment SGI298555[] = {{&S190[35], 14}, {&S190[35], 14}, {NULL}};
const Segment SGI298556[] = {{&S190[160], 15}, {NULL}};
const Segment SGI298557[] = {{&S190[245], 4}, {NULL}};
const Segment SGI298558[] = {{&S190[115], 3}, {NULL}};
const Segment SGI298559[] = {{&S191[30], 3}, {&S191[106], 3}, {NULL}};
const Segment SGI298560[] = {{&S191[8], 3}, {&S191[8], 3}, {NULL}};
const Segment SGI298561[] = {{&S191[13], 10}, {&S191[13], 10}, {NULL}};
const Segment SGI298562[] = {{&S191[42], 7}, {&S191[118], 7}, {NULL}};
const Segment SGI298563[] = {{&S191[35], 14}, {&S191[111], 14}, {NULL}};
const Segment SGI298564[] = {{&S191[51], 5}, {NULL}};
const Segment SGI298565[] = {{&S191[127], 3}, {NULL}};
const Segment SGI298566[] = {{&S192[30], 3}, {NULL}};
const Segment SGI298567[] = {{&S192[8], 3}, {NULL}};
const Segment SGI298568[] = {{&S192[13], 10}, {NULL}};
const Segment SGI298569[] = {{&S192[42], 7}, {NULL}};
const Segment SGI298570[] = {{&S193[24], 3}, {NULL}};
const Segment SGI298571[] = {{&S193[36], 7}, {NULL}};
const Segment SGI298572[] = {{&S193[29], 14}, {NULL}};
const Segment SGI298573[] = {{&S194[30], 3}, {NULL}};
const Segment SGI298574[] = {{&S194[8], 3}, {NULL}};
const Segment SGI298575[] = {{&S194[13], 20}, {NULL}};
const Segment SGI298576[] = {{&S194[42], 7}, {NULL}};
const Segment SGI298577[] = {{&S194[35], 14}, {NULL}};
const Segment SGI298578[] = {{&S195[30], 3}, {NULL}};
const Segment SGI298579[] = {{&S195[8], 3}, {NULL}};
const Segment SGI298580[] = {{&S195[13], 10}, {NULL}};
const Segment SGI298581[] = {{&S195[42], 7}, {NULL}};
const Segment SGI298582[] = {{&S195[35], 14}, {NULL}};
const Segment SGI298583[] = {{&S195[102], 15}, {&S195[102], 15}, {&S195[102], 15}, {NULL}};
const Segment SGI298584[] = {{&S196[30], 14}, {NULL}};
const Segment SGI298585[] = {{&S196[8], 3}, {NULL}};
const Segment SGI298586[] = {{&S196[13], 10}, {NULL}};
const Segment SGI298587[] = {{&S196[53], 7}, {NULL}};
const Segment SGI298588[] = {{&S197[30], 3}, {NULL}};
const Segment SGI298589[] = {{&S197[8], 3}, {NULL}};
const Segment SGI298590[] = {{&S197[13], 10}, {NULL}};
const Segment SGI298591[] = {{&S197[42], 7}, {NULL}};
const Segment SGI298592[] = {{&S197[35], 14}, {NULL}};
const Segment SGI298593[] = {{&S197[51], 6}, {NULL}};
const Segment SGI298594[] = {{&S197[58], 3}, {NULL}};
const Segment SGI298595[] = {{&S198[30], 3}, {&S198[94], 3}, {NULL}};
const Segment SGI298596[] = {{&S198[8], 3}, {&S198[8], 3}, {NULL}};
const Segment SGI298597[] = {{&S198[13], 10}, {&S198[13], 10}, {NULL}};
const Segment SGI298598[] = {{&S198[42], 7}, {&S198[42], 3}, {NULL}};
const Segment SGI298599[] = {{&S198[35], 14}, {NULL}};
const Segment SGI298600[] = {{&S199[30], 3}, {NULL}};
const Segment SGI298601[] = {{&S199[8], 3}, {NULL}};
const Segment SGI298602[] = {{&S199[13], 10}, {NULL}};
const Segment SGI298603[] = {{&S199[42], 7}, {&S199[98], 5}, {NULL}};
const Segment SGI298604[] = {{&S199[35], 14}, {NULL}};
const Segment SGI298605[] = {{&S200[30], 3}, {&S200[30], 3}, {&S200[190], 3}, {NULL}};
const Segment SGI298606[] = {{&S200[8], 3}, {NULL}};
const Segment SGI298607[] = {{&S200[13], 10}, {NULL}};
const Segment SGI298608[] = {{&S200[42], 7}, {NULL}};
const Segment SGI298609[] = {{&S200[35], 14}, {NULL}};
const Segment SGI298610[] = {{&S201[31], 14}, {NULL}};
const Segment SGI298611[] = {{&S201[8], 3}, {NULL}};
const Segment SGI298612[] = {{&S201[13], 11}, {NULL}};
const Segment SGI298613[] = {{&S201[65], 32}, {NULL}};
const Segment SGI298614[] = {{&S202[30], 3}, {&S202[30], 3}, {&S202[142], 3}, {NULL}};
const Segment SGI298615[] = {{&S202[8], 3}, {&S202[8], 3}, {&S202[8], 3}, {NULL}};
const Segment SGI298616[] = {{&S202[13], 10}, {&S202[13], 10}, {&S202[13], 10}, {NULL}};
const Segment SGI298617[] = {{&S202[42], 7}, {&S202[42], 7}, {NULL}};
const Segment SGI298618[] = {{&S202[35], 14}, {&S202[35], 14}, {NULL}};
const Segment SGI298619[] = {{&S202[99], 5}, {NULL}};
const Segment SGI298620[] = {{&S202[163], 3}, {NULL}};
const Segment SGI298621[] = {{&S203[30], 3}, {NULL}};
const Segment SGI298622[] = {{&S203[8], 3}, {NULL}};
const Segment SGI298623[] = {{&S203[13], 10}, {NULL}};
const Segment SGI298624[] = {{&S203[42], 7}, {NULL}};
const Segment SGI298625[] = {{&S203[35], 14}, {NULL}};
const Segment SGI298626[] = {{&S203[51], 6}, {NULL}};
const Segment SGI298627[] = {{&S204[30], 3}, {NULL}};
const Segment SGI298628[] = {{&S204[8], 3}, {NULL}};
const Segment SGI298629[] = {{&S204[13], 10}, {NULL}};
const Segment SGI298630[] = {{&S204[42], 7}, {NULL}};
const Segment SGI298631[] = {{&S204[35], 14}, {NULL}};
const Segment SGI298632[] = {{&S204[153], 5}, {&S204[153], 5}, {NULL}};
const Segment SGI298633[] = {{&S205[30], 3}, {NULL}};
const Segment SGI298634[] = {{&S205[8], 3}, {NULL}};
const Segment SGI298635[] = {{&S205[13], 10}, {NULL}};
const Segment SGI298636[] = {{&S205[42], 7}, {NULL}};
const Segment SGI298637[] = {{&S205[35], 14}, {NULL}};
const Segment SGI298638[] = {{&S206[30], 3}, {NULL}};
const Segment SGI298639[] = {{&S206[8], 3}, {NULL}};
const Segment SGI298640[] = {{&S206[13], 10}, {NULL}};
const Segment SGI298641[] = {{&S206[42], 7}, {&S206[98], 5}, {&S206[98], 5}, {NULL}};
const Segment SGI298642[] = {{&S206[35], 14}, {NULL}};
const Segment SGI298643[] = {{&S207[30], 3}, {&S207[30], 3}, {NULL}};
const Segment SGI298644[] = {{&S207[8], 3}, {NULL}};
const Segment SGI298645[] = {{&S207[13], 10}, {NULL}};
const Segment SGI298646[] = {{&S207[42], 7}, {NULL}};
const Segment SGI298647[] = {{&S207[35], 14}, {NULL}};
const Segment SGI298648[] = {{&S210[30], 3}, {&S210[30], 3}, {&S210[72], 3}, {&S210[82], 3}, {&S210[92], 3}, {NULL}};
const Segment SGI298649[] = {{&S210[8], 3}, {&S210[107], 3}, {&S210[8], 3}, {NULL}};
const Segment SGI298650[] = {{&S210[13], 10}, {NULL}};
const Segment SGI298651[] = {{&S210[42], 7}, {NULL}};
const Segment SGI298652[] = {{&S210[35], 14}, {NULL}};
const Segment SGI298653[] = {{&S210[51], 3}, {NULL}};
const Segment SGI298654[] = {{&S211[30], 3}, {NULL}};
const Segment SGI298655[] = {{&S211[8], 3}, {NULL}};
const Segment SGI298656[] = {{&S211[13], 10}, {NULL}};
const Segment SGI298657[] = {{&S211[42], 7}, {&S211[101], 20}, {NULL}};
const Segment SGI298658[] = {{&S211[35], 14}, {NULL}};
const Segment SGI298659[] = {{&S211[102], 15}, {NULL}};
const Segment SGI298660[] = {{&S212[30], 3}, {NULL}};
const Segment SGI298661[] = {{&S212[8], 3}, {NULL}};
const Segment SGI298662[] = {{&S212[13], 10}, {NULL}};
const Segment SGI298663[] = {{&S212[42], 7}, {NULL}};
const Segment SGI298664[] = {{&S212[35], 14}, {NULL}};
const Segment SGI298665[] = {{&S212[196], 9}, {NULL}};
const Segment SGI298666[] = {{&S212[64], 3}, {NULL}};
const Segment SGI298667[] = {{&S214[30], 3}, {NULL}};
const Segment SGI298668[] = {{&S214[8], 3}, {NULL}};
const Segment SGI298669[] = {{&S214[13], 10}, {NULL}};
const Segment SGI298670[] = {{&S214[42], 7}, {NULL}};
const Segment SGI298671[] = {{&S214[35], 14}, {NULL}};
const Segment SGI298672[] = {{&S214[103], 9}, {NULL}};
const Segment SGI298673[] = {{&S215[30], 3}, {NULL}};
const Segment SGI298674[] = {{&S215[8], 3}, {NULL}};
const Segment SGI298675[] = {{&S215[13], 10}, {NULL}};
const Segment SGI298676[] = {{&S215[42], 7}, {NULL}};
const Segment SGI298677[] = {{&S215[35], 14}, {NULL}};
const Segment SGI298678[] = {{&S215[138], 15}, {NULL}};
const Segment SGI298679[] = {{&S215[51], 6}, {NULL}};
const Segment SGI298680[] = {{&S216[30], 3}, {&S216[30], 3}, {NULL}};
const Segment SGI298681[] = {{&S216[8], 3}, {&S216[8], 3}, {NULL}};
const Segment SGI298682[] = {{&S216[13], 10}, {&S216[13], 10}, {NULL}};
const Segment SGI298683[] = {{&S216[42], 7}, {NULL}};
const Segment SGI298684[] = {{&S216[35], 14}, {NULL}};
const Segment SGI298685[] = {{&S216[166], 15}, {NULL}};
const Segment SGI298686[] = {{&S216[99], 5}, {NULL}};
const Segment SGI298687[] = {{&S217[30], 3}, {NULL}};
const Segment SGI298688[] = {{&S217[8], 3}, {NULL}};
const Segment SGI298689[] = {{&S217[13], 10}, {NULL}};
const Segment SGI298690[] = {{&S217[42], 7}, {&S217[75], 16}, {NULL}};
const Segment SGI298691[] = {{&S217[35], 14}, {NULL}};
const Segment SGI298692[] = {{&S218[30], 3}, {&S218[99], 3}, {NULL}};
const Segment SGI298693[] = {{&S218[8], 3}, {&S218[8], 3}, {NULL}};
const Segment SGI298694[] = {{&S218[13], 10}, {&S218[13], 10}, {NULL}};
const Segment SGI298695[] = {{&S218[42], 7}, {&S218[42], 3}, {NULL}};
const Segment SGI298696[] = {{&S218[35], 14}, {NULL}};
const Segment SGI298697[] = {{&S218[64], 3}, {NULL}};
const Segment SGI298698[] = {{&S219[30], 3}, {&S219[94], 3}, {NULL}};
const Segment SGI298699[] = {{&S219[8], 3}, {&S219[8], 3}, {NULL}};
const Segment SGI298700[] = {{&S219[13], 10}, {&S219[13], 10}, {NULL}};
const Segment SGI298701[] = {{&S219[42], 7}, {NULL}};
const Segment SGI298702[] = {{&S219[35], 14}, {NULL}};
const Segment SGI298703[] = {{&S221[30], 3}, {&S221[70], 3}, {&S221[30], 3}, {NULL}};
const Segment SGI298704[] = {{&S221[8], 3}, {NULL}};
const Segment SGI298705[] = {{&S221[13], 10}, {NULL}};
const Segment SGI298706[] = {{&S221[42], 7}, {NULL}};
const Segment SGI298707[] = {{&S221[35], 14}, {NULL}};
const Segment SGI298708[] = {{&S222[30], 3}, {&S222[94], 3}, {&S222[94], 3}, {NULL}};
const Segment SGI298709[] = {{&S222[8], 3}, {&S222[8], 3}, {&S222[8], 3}, {NULL}};
const Segment SGI298710[] = {{&S222[13], 10}, {&S222[77], 10}, {&S222[77], 10}, {NULL}};
const Segment SGI298711[] = {{&S222[42], 7}, {&S222[42], 3}, {&S222[42], 3}, {NULL}};
const Segment SGI298712[] = {{&S222[35], 14}, {NULL}};
const Segment SGI298713[] = {{&S223[30], 3}, {&S223[156], 3}, {&S223[156], 3}, {NULL}};
const Segment SGI298714[] = {{&S223[8], 3}, {NULL}};
const Segment SGI298715[] = {{&S223[13], 10}, {NULL}};
const Segment SGI298716[] = {{&S223[42], 7}, {NULL}};
const Segment SGI298717[] = {{&S223[35], 14}, {NULL}};
const Segment SGI298718[] = {{&S224[30], 3}, {NULL}};
const Segment SGI298719[] = {{&S224[8], 3}, {NULL}};
const Segment SGI298720[] = {{&S224[13], 10}, {NULL}};
const Segment SGI298721[] = {{&S224[42], 7}, {&S224[58], 16}, {NULL}};
const Segment SGI298722[] = {{&S224[35], 14}, {NULL}};
const Segment SGI298723[] = {{&S225[30], 3}, {&S225[94], 3}, {&S225[152], 3}, {NULL}};
const Segment SGI298724[] = {{&S225[8], 3}, {&S225[8], 3}, {&S225[8], 3}, {NULL}};
const Segment SGI298725[] = {{&S225[13], 10}, {&S225[13], 10}, {&S225[13], 10}, {NULL}};
const Segment SGI298726[] = {{&S225[42], 7}, {&S225[42], 7}, {NULL}};
const Segment SGI298727[] = {{&S225[35], 14}, {&S225[35], 14}, {NULL}};
const Segment SGI298728[] = {{&S225[115], 3}, {NULL}};
const Segment SGI298729[] = {{&S226[27], 3}, {NULL}};
const Segment SGI298730[] = {{&S226[8], 3}, {NULL}};
const Segment SGI298731[] = {{&S226[13], 7}, {NULL}};
const Segment SGI298732[] = {{&S226[39], 7}, {NULL}};
const Segment SGI298733[] = {{&S226[32], 14}, {NULL}};
const Segment SGI298734[] = {{&S227[30], 3}, {NULL}};
const Segment SGI298735[] = {{&S227[8], 3}, {NULL}};
const Segment SGI298736[] = {{&S227[13], 10}, {NULL}};
const Segment SGI298737[] = {{&S227[42], 7}, {NULL}};
const Segment SGI298738[] = {{&S227[35], 14}, {NULL}};
const Segment SGI298739[] = {{&S227[102], 15}, {&S227[102], 15}, {NULL}};
const Segment SGI298740[] = {{&S227[208], 4}, {NULL}};
const Segment SGI298741[] = {{&S228[8], 3}, {&S228[94], 3}, {NULL}};
const Segment SGI298742[] = {{&S228[8], 3}, {&S228[72], 3}, {NULL}};
const Segment SGI298743[] = {{&S228[13], 10}, {&S228[13], 10}, {NULL}};
const Segment SGI298744[] = {{&S228[42], 7}, {&S228[42], 7}, {NULL}};
const Segment SGI298745[] = {{&S228[35], 14}, {&S228[35], 14}, {NULL}};
const Segment SGI298746[] = {{&S228[115], 3}, {NULL}};
const Segment SGI298747[] = {{&S229[30], 3}, {&S229[158], 3}, {NULL}};
const Segment SGI298748[] = {{&S229[8], 3}, {&S229[8], 3}, {NULL}};
const Segment SGI298749[] = {{&S229[13], 10}, {&S229[13], 10}, {NULL}};
const Segment SGI298750[] = {{&S229[42], 7}, {NULL}};
const Segment SGI298751[] = {{&S229[35], 14}, {NULL}};
const Segment SGI298752[] = {{&S229[163], 5}, {NULL}};
const Segment SGI298753[] = {{&S230[30], 3}, {&S230[72], 3}, {&S230[30], 3}, {NULL}};
const Segment SGI298754[] = {{&S230[8], 3}, {NULL}};
const Segment SGI298755[] = {{&S230[13], 10}, {NULL}};
const Segment SGI298756[] = {{&S230[42], 7}, {NULL}};
const Segment SGI298757[] = {{&S230[35], 14}, {NULL}};
const Segment SGI298758[] = {{&S231[30], 3}, {&S231[230], 3}, {NULL}};
const Segment SGI298759[] = {{&S231[8], 3}, {NULL}};
const Segment SGI298760[] = {{&S231[13], 10}, {NULL}};
const Segment SGI298761[] = {{&S231[42], 7}, {NULL}};
const Segment SGI298762[] = {{&S231[35], 14}, {NULL}};
const Segment SGI298763[] = {{&S231[64], 9}, {NULL}};
const Segment SGI298764[] = {{&S232[30], 3}, {&S232[94], 3}, {&S232[30], 3}, {NULL}};
const Segment SGI298765[] = {{&S232[8], 3}, {&S232[8], 3}, {&S232[8], 3}, {NULL}};
const Segment SGI298766[] = {{&S232[13], 10}, {&S232[13], 10}, {&S232[13], 10}, {NULL}};
const Segment SGI298767[] = {{&S232[42], 7}, {NULL}};
const Segment SGI298768[] = {{&S232[35], 14}, {NULL}};
const Segment SGI298769[] = {{&S232[99], 5}, {&S232[99], 5}, {NULL}};
const Segment SGI298770[] = {{&S233[30], 3}, {NULL}};
const Segment SGI298771[] = {{&S233[8], 3}, {NULL}};
const Segment SGI298772[] = {{&S233[13], 10}, {NULL}};
const Segment SGI298773[] = {{&S233[42], 7}, {&S233[137], 20}, {NULL}};
const Segment SGI298774[] = {{&S233[35], 14}, {NULL}};
const Segment SGI298775[] = {{&S233[138], 15}, {NULL}};
const Segment SGI298776[] = {{&S233[51], 3}, {NULL}};
const Segment SGI298777[] = {{&S234[30], 3}, {NULL}};
const Segment SGI298778[] = {{&S234[8], 3}, {NULL}};
const Segment SGI298779[] = {{&S234[13], 10}, {NULL}};
const Segment SGI298780[] = {{&S234[42], 7}, {NULL}};
const Segment SGI298781[] = {{&S234[35], 14}, {NULL}};
const Segment SGI298782[] = {{&S234[89], 4}, {NULL}};
const Segment SGI298783[] = {{&S234[64], 3}, {NULL}};
const Segment SGI298784[] = {{&S235[30], 3}, {&S235[69], 3}, {&S235[69], 3}, {&S235[69], 3}, {NULL}};
const Segment SGI298785[] = {{&S235[8], 3}, {NULL}};
const Segment SGI298786[] = {{&S235[13], 10}, {NULL}};
const Segment SGI298787[] = {{&S235[42], 7}, {NULL}};
const Segment SGI298788[] = {{&S235[35], 14}, {NULL}};
const Segment SGI298789[] = {{&S236[30], 3}, {&S236[176], 3}, {NULL}};
const Segment SGI298790[] = {{&S236[8], 3}, {NULL}};
const Segment SGI298791[] = {{&S236[13], 10}, {NULL}};
const Segment SGI298792[] = {{&S236[42], 7}, {NULL}};
const Segment SGI298793[] = {{&S236[35], 14}, {NULL}};
const Segment SGI298794[] = {{&S237[30], 3}, {&S237[40], 3}, {&S237[50], 3}, {NULL}};
const Segment SGI298795[] = {{&S237[8], 3}, {NULL}};
const Segment SGI298796[] = {{&S237[13], 10}, {NULL}};
const Segment SGI298797[] = {{&S237[62], 7}, {NULL}};
const Segment SGI298798[] = {{&S237[55], 14}, {NULL}};
const Segment SGI298799[] = {{&S237[71], 3}, {NULL}};
const Segment SGI298800[] = {{&S238[30], 3}, {NULL}};
const Segment SGI298801[] = {{&S238[8], 3}, {NULL}};
const Segment SGI298802[] = {{&S238[13], 10}, {NULL}};
const Segment SGI298803[] = {{&S238[42], 7}, {NULL}};
const Segment SGI298804[] = {{&S238[35], 14}, {NULL}};
const Segment SGI298805[] = {{&S238[51], 6}, {NULL}};
const Segment SGI298806[] = {{&S238[71], 3}, {NULL}};
const Segment SGI298807[] = {{&S239[30], 3}, {NULL}};
const Segment SGI298808[] = {{&S239[8], 3}, {NULL}};
const Segment SGI298809[] = {{&S239[13], 10}, {NULL}};
const Segment SGI298810[] = {{&S239[55], 7}, {NULL}};
const Segment SGI298811[] = {{&S239[48], 14}, {NULL}};
const Segment SGI298812[] = {{&S240[30], 3}, {NULL}};
const Segment SGI298813[] = {{&S240[8], 3}, {NULL}};
const Segment SGI298814[] = {{&S240[13], 10}, {NULL}};
const Segment SGI298815[] = {{&S240[42], 7}, {NULL}};
const Segment SGI298816[] = {{&S240[35], 14}, {NULL}};
const Segment SGI298817[] = {{&S240[131], 15}, {NULL}};
const Segment SGI298818[] = {{&S241[30], 3}, {NULL}};
const Segment SGI298819[] = {{&S241[8], 3}, {NULL}};
const Segment SGI298820[] = {{&S241[13], 10}, {NULL}};
const Segment SGI298821[] = {{&S241[55], 7}, {NULL}};
const Segment SGI298822[] = {{&S241[48], 14}, {NULL}};
const Segment SGI298823[] = {{&S241[64], 3}, {NULL}};
const Segment SGI298824[] = {{&S242[30], 3}, {NULL}};
const Segment SGI298825[] = {{&S242[8], 3}, {NULL}};
const Segment SGI298826[] = {{&S242[13], 10}, {NULL}};
const Segment SGI298827[] = {{&S242[55], 7}, {NULL}};
const Segment SGI298828[] = {{&S242[48], 14}, {NULL}};
const Segment SGI298829[] = {{&S242[64], 3}, {NULL}};
const Segment SGI298830[] = {{&S243[30], 3}, {NULL}};
const Segment SGI298831[] = {{&S243[8], 3}, {NULL}};
const Segment SGI298832[] = {{&S243[13], 10}, {NULL}};
const Segment SGI298833[] = {{&S243[42], 7}, {NULL}};
const Segment SGI298834[] = {{&S243[35], 14}, {NULL}};
const Segment SGI298835[] = {{&S243[160], 3}, {NULL}};
const Segment SGI298836[] = {{&S244[30], 3}, {&S244[94], 3}, {NULL}};
const Segment SGI298837[] = {{&S244[8], 3}, {&S244[8], 3}, {NULL}};
const Segment SGI298838[] = {{&S244[13], 10}, {&S244[13], 10}, {NULL}};
const Segment SGI298839[] = {{&S244[42], 7}, {&S244[42], 7}, {NULL}};
const Segment SGI298840[] = {{&S244[35], 14}, {&S244[35], 14}, {NULL}};
const Segment SGI298841[] = {{&S245[30], 3}, {&S245[102], 3}, {NULL}};
const Segment SGI298842[] = {{&S245[8], 3}, {&S245[8], 3}, {NULL}};
const Segment SGI298843[] = {{&S245[13], 10}, {&S245[13], 10}, {NULL}};
const Segment SGI298844[] = {{&S245[42], 7}, {NULL}};
const Segment SGI298845[] = {{&S245[35], 14}, {NULL}};
const Segment SGI298846[] = {{&S246[30], 3}, {&S246[69], 3}, {&S246[69], 3}, {NULL}};
const Segment SGI298847[] = {{&S246[8], 3}, {NULL}};
const Segment SGI298848[] = {{&S246[13], 10}, {NULL}};
const Segment SGI298849[] = {{&S246[42], 7}, {NULL}};
const Segment SGI298850[] = {{&S246[35], 14}, {NULL}};
const Segment SGI298851[] = {{&S247[30], 3}, {&S247[103], 3}, {NULL}};
const Segment SGI298852[] = {{&S247[8], 3}, {&S247[8], 3}, {NULL}};
const Segment SGI298853[] = {{&S247[13], 10}, {&S247[13], 10}, {NULL}};
const Segment SGI298854[] = {{&S247[42], 7}, {&S247[42], 7}, {&S247[139], 20}, {NULL}};
const Segment SGI298855[] = {{&S247[35], 14}, {&S247[35], 14}, {NULL}};
const Segment SGI298856[] = {{&S247[140], 15}, {&S247[140], 15}, {NULL}};
const Segment SGI298857[] = {{&S247[124], 3}, {NULL}};
const Segment SGI298858[] = {{&S248[30], 3}, {NULL}};
const Segment SGI298859[] = {{&S248[8], 3}, {NULL}};
const Segment SGI298860[] = {{&S248[13], 10}, {NULL}};
const Segment SGI298861[] = {{&S248[42], 7}, {&S248[75], 20}, {NULL}};
const Segment SGI298862[] = {{&S248[35], 14}, {NULL}};
const Segment SGI298863[] = {{&S248[76], 15}, {&S248[76], 15}, {NULL}};
const Segment SGI298864[] = {{&S248[51], 3}, {NULL}};
const Segment SGI298865[] = {{&S249[21], 3}, {NULL}};
const Segment SGI298866[] = {{&S249[8], 3}, {NULL}};
const Segment SGI298867[] = {{&S249[11], 1}, {NULL}};
const Segment SGI298868[] = {{&S249[33], 7}, {NULL}};
const Segment SGI298869[] = {{&S249[26], 14}, {NULL}};
const Segment SGI298870[] = {{&S250[30], 3}, {NULL}};
const Segment SGI298871[] = {{&S250[8], 3}, {NULL}};
const Segment SGI298872[] = {{&S250[13], 10}, {NULL}};
const Segment SGI298873[] = {{&S250[42], 7}, {NULL}};
const Segment SGI298874[] = {{&S250[35], 14}, {NULL}};
const Segment SGI298875[] = {{&S250[64], 3}, {NULL}};
const Segment SGI298876[] = {{&S251[30], 3}, {&S251[82], 3}, {&S251[82], 3}, {NULL}};
const Segment SGI298877[] = {{&S251[8], 3}, {&S251[8], 3}, {NULL}};
const Segment SGI298878[] = {{&S251[13], 10}, {&S251[13], 10}, {NULL}};
const Segment SGI298879[] = {{&S251[42], 7}, {&S251[42], 7}, {NULL}};
const Segment SGI298880[] = {{&S251[35], 14}, {&S251[35], 14}, {NULL}};
const Segment SGI298881[] = {{&S252[30], 3}, {&S252[128], 3}, {NULL}};
const Segment SGI298882[] = {{&S252[8], 3}, {NULL}};
const Segment SGI298883[] = {{&S252[13], 10}, {NULL}};
const Segment SGI298884[] = {{&S252[42], 7}, {NULL}};
const Segment SGI298885[] = {{&S252[35], 14}, {NULL}};
const Segment SGI298886[] = {{&S253[30], 3}, {NULL}};
const Segment SGI298887[] = {{&S253[8], 3}, {NULL}};
const Segment SGI298888[] = {{&S253[13], 10}, {NULL}};
const Segment SGI298889[] = {{&S253[42], 7}, {&S253[166], 5}, {&S253[166], 5}, {NULL}};
const Segment SGI298890[] = {{&S253[35], 14}, {NULL}};
const Segment SGI298891[] = {{&S253[110], 15}, {NULL}};
const Segment SGI298892[] = {{&S254[30], 3}, {NULL}};
const Segment SGI298893[] = {{&S254[8], 3}, {NULL}};
const Segment SGI298894[] = {{&S254[13], 10}, {NULL}};
const Segment SGI298895[] = {{&S254[42], 7}, {NULL}};
const Segment SGI298896[] = {{&S254[35], 14}, {NULL}};
const Segment SGI298897[] = {{&S254[102], 15}, {NULL}};
const Segment SGI298898[] = {{&S254[128], 3}, {NULL}};
const Segment SGI298899[] = {{&S255[30], 3}, {NULL}};
const Segment SGI298900[] = {{&S255[8], 3}, {NULL}};
const Segment SGI298901[] = {{&S255[13], 10}, {NULL}};
const Segment SGI298902[] = {{&S255[42], 7}, {NULL}};
const Segment SGI298903[] = {{&S255[35], 14}, {NULL}};
const Segment SGI298904[] = {{&S255[51], 5}, {NULL}};
const Segment SGI298905[] = {{&S256[30], 3}, {NULL}};
const Segment SGI298906[] = {{&S256[8], 3}, {&S256[8], 3}, {&S256[8], 3}, {&S256[8], 3}, {&S256[8], 3}, {NULL}};
const Segment SGI298907[] = {{&S256[13], 10}, {NULL}};
const Segment SGI298908[] = {{&S256[42], 7}, {NULL}};
const Segment SGI298909[] = {{&S256[35], 14}, {NULL}};
const Segment SGI298910[] = {{&S257[30], 3}, {NULL}};
const Segment SGI298911[] = {{&S257[8], 3}, {&S257[8], 3}, {NULL}};
const Segment SGI298912[] = {{&S257[13], 10}, {&S257[13], 10}, {NULL}};
const Segment SGI298913[] = {{&S257[42], 7}, {NULL}};
const Segment SGI298914[] = {{&S257[35], 14}, {NULL}};
const Segment SGI298915[] = {{&S258[30], 3}, {NULL}};
const Segment SGI298916[] = {{&S258[8], 3}, {NULL}};
const Segment SGI298917[] = {{&S258[13], 10}, {NULL}};
const Segment SGI298918[] = {{&S258[42], 7}, {&S258[90], 3}, {&S258[90], 3}, {NULL}};
const Segment SGI298919[] = {{&S258[35], 14}, {NULL}};
const Segment SGI298920[] = {{&S259[30], 3}, {&S259[130], 3}, {&S259[178], 3}, {NULL}};
const Segment SGI298921[] = {{&S259[8], 3}, {&S259[8], 3}, {&S259[8], 3}, {NULL}};
const Segment SGI298922[] = {{&S259[13], 10}, {&S259[13], 10}, {&S259[13], 10}, {NULL}};
const Segment SGI298923[] = {{&S259[42], 7}, {&S259[42], 7}, {NULL}};
const Segment SGI298924[] = {{&S259[35], 14}, {&S259[35], 14}, {NULL}};
const Segment SGI298925[] = {{&S259[135], 5}, {NULL}};
const Segment SGI298926[] = {{&S259[199], 3}, {NULL}};
const Segment SGI298927[] = {{&S260[30], 3}, {&S260[94], 3}, {NULL}};
const Segment SGI298928[] = {{&S260[8], 3}, {&S260[8], 3}, {NULL}};
const Segment SGI298929[] = {{&S260[13], 10}, {&S260[13], 10}, {NULL}};
const Segment SGI298930[] = {{&S260[42], 7}, {NULL}};
const Segment SGI298931[] = {{&S260[35], 14}, {NULL}};
const Segment SGI298932[] = {{&S260[158], 15}, {NULL}};
const Segment SGI298933[] = {{&S260[99], 5}, {NULL}};
const Segment SGI298934[] = {{&S261[30], 3}, {NULL}};
const Segment SGI298935[] = {{&S261[8], 3}, {NULL}};
const Segment SGI298936[] = {{&S261[13], 10}, {NULL}};
const Segment SGI298937[] = {{&S261[42], 7}, {&S261[145], 20}, {NULL}};
const Segment SGI298938[] = {{&S261[35], 14}, {NULL}};
const Segment SGI298939[] = {{&S261[146], 15}, {&S261[146], 15}, {NULL}};
const Segment SGI298940[] = {{&S262[20], 3}, {NULL}};
const Segment SGI298941[] = {{&S262[25], 7}, {NULL}};
const Segment SGI298942[] = {{&S262[44], 7}, {NULL}};
const Segment SGI298943[] = {{&S262[37], 14}, {NULL}};
const Segment SGI298944[] = {{&S263[30], 3}, {NULL}};
const Segment SGI298945[] = {{&S263[8], 3}, {NULL}};
const Segment SGI298946[] = {{&S263[13], 10}, {NULL}};
const Segment SGI298947[] = {{&S263[42], 7}, {NULL}};
const Segment SGI298948[] = {{&S264[75], 14}, {NULL}};
const Segment SGI298949[] = {{&S264[8], 3}, {NULL}};
const Segment SGI298950[] = {{&S264[13], 10}, {NULL}};
const Segment SGI298951[] = {{&S264[61], 7}, {NULL}};
const Segment SGI298952[] = {{&S265[30], 3}, {&S265[76], 3}, {NULL}};
const Segment SGI298953[] = {{&S265[8], 3}, {&S265[8], 3}, {NULL}};
const Segment SGI298954[] = {{&S265[13], 10}, {&S265[13], 10}, {NULL}};
const Segment SGI298955[] = {{&S265[42], 7}, {&S265[42], 7}, {NULL}};
const Segment SGI298956[] = {{&S265[35], 14}, {&S265[35], 14}, {NULL}};
const Segment SGI298957[] = {{&S265[157], 3}, {NULL}};
const Segment SGI298958[] = {{&S266[30], 3}, {&S266[94], 3}, {&S266[94], 3}, {&S266[94], 3}, {NULL}};
const Segment SGI298959[] = {{&S266[8], 3}, {&S266[8], 3}, {&S266[8], 3}, {&S266[8], 3}, {NULL}};
const Segment SGI298960[] = {{&S266[13], 10}, {&S266[13], 10}, {&S266[13], 10}, {&S266[13], 10}, {NULL}};
const Segment SGI298961[] = {{&S266[42], 7}, {NULL}};
const Segment SGI298962[] = {{&S266[35], 14}, {NULL}};
const Segment SGI298963[] = {{&S266[99], 5}, {&S266[99], 5}, {&S266[99], 5}, {NULL}};
const Segment SGI298964[] = {{&S267[30], 3}, {NULL}};
const Segment SGI298965[] = {{&S267[8], 3}, {NULL}};
const Segment SGI298966[] = {{&S267[13], 10}, {NULL}};
const Segment SGI298967[] = {{&S267[42], 7}, {NULL}};
const Segment SGI298968[] = {{&S267[35], 14}, {NULL}};
const Segment SGI298969[] = {{&S267[131], 15}, {NULL}};
const Segment SGI298970[] = {{&S267[187], 4}, {NULL}};
const Segment SGI298971[] = {{&S267[152], 9}, {NULL}};
const Segment SGI298972[] = {{&S268[30], 3}, {&S268[81], 3}, {NULL}};
const Segment SGI298973[] = {{&S268[8], 3}, {&S268[8], 3}, {NULL}};
const Segment SGI298974[] = {{&S268[13], 10}, {NULL}};
const Segment SGI298975[] = {{&S268[42], 7}, {&S268[42], 7}, {NULL}};
const Segment SGI298976[] = {{&S268[35], 14}, {&S268[35], 14}, {NULL}};
const Segment SGI298977[] = {{&S268[102], 3}, {NULL}};
const Segment SGI298978[] = {{&S269[30], 3}, {&S269[94], 3}, {NULL}};
const Segment SGI298979[] = {{&S269[8], 3}, {&S269[8], 3}, {NULL}};
const Segment SGI298980[] = {{&S269[13], 10}, {&S269[13], 10}, {NULL}};
const Segment SGI298981[] = {{&S269[42], 7}, {&S269[42], 7}, {NULL}};
const Segment SGI298982[] = {{&S269[35], 14}, {&S269[35], 14}, {NULL}};
const Segment SGI298983[] = {{&S269[160], 15}, {&S269[160], 15}, {NULL}};
const Segment SGI298984[] = {{&S269[115], 3}, {NULL}};
const Segment SGI298985[] = {{&S270[30], 3}, {NULL}};
const Segment SGI298986[] = {{&S270[8], 3}, {NULL}};
const Segment SGI298987[] = {{&S270[13], 10}, {NULL}};
const Segment SGI298988[] = {{&S270[42], 7}, {NULL}};
const Segment SGI298989[] = {{&S270[35], 14}, {NULL}};
const Segment SGI298990[] = {{&S270[74], 3}, {NULL}};
const Segment SGI298991[] = {{&S271[30], 3}, {NULL}};
const Segment SGI298992[] = {{&S271[8], 3}, {NULL}};
const Segment SGI298993[] = {{&S271[13], 10}, {NULL}};
const Segment SGI298994[] = {{&S271[42], 7}, {NULL}};
const Segment SGI298995[] = {{&S271[35], 14}, {NULL}};
const Segment SGI298996[] = {{&S271[64], 3}, {NULL}};
const Segment SGI298997[] = {{&S271[79], 3}, {NULL}};
const Segment SGI298998[] = {{&S272[30], 3}, {&S272[94], 3}, {&S272[94], 3}, {NULL}};
const Segment SGI298999[] = {{&S272[8], 3}, {NULL}};
const Segment SGI299000[] = {{&S272[13], 10}, {NULL}};
const Segment SGI299001[] = {{&S272[42], 7}, {&S272[106], 3}, {&S272[106], 3}, {NULL}};
const Segment SGI299002[] = {{&S272[35], 14}, {NULL}};
const Segment SGI299003[] = {{&S273[30], 3}, {NULL}};
const Segment SGI299004[] = {{&S273[8], 3}, {NULL}};
const Segment SGI299005[] = {{&S273[13], 10}, {NULL}};
const Segment SGI299006[] = {{&S273[42], 7}, {&S273[71], 3}, {NULL}};
const Segment SGI299007[] = {{&S273[35], 14}, {NULL}};
const Segment SGI299008[] = {{&S274[30], 3}, {NULL}};
const Segment SGI299009[] = {{&S274[8], 3}, {&S274[8], 3}, {&S274[8], 3}, {&S274[8], 3}, {NULL}};
const Segment SGI299010[] = {{&S274[13], 10}, {NULL}};
const Segment SGI299011[] = {{&S274[42], 7}, {NULL}};
const Segment SGI299012[] = {{&S274[35], 14}, {NULL}};
const Segment SGI299013[] = {{&S275[30], 3}, {NULL}};
const Segment SGI299014[] = {{&S275[8], 3}, {NULL}};
const Segment SGI299015[] = {{&S275[13], 10}, {NULL}};
const Segment SGI299016[] = {{&S275[42], 7}, {NULL}};
const Segment SGI299017[] = {{&S275[35], 14}, {NULL}};
const Segment SGI299018[] = {{&S275[138], 15}, {NULL}};
const Segment SGI299019[] = {{&S275[179], 4}, {NULL}};
const Segment SGI299020[] = {{&S275[51], 6}, {NULL}};
const Segment SGI299021[] = {{&S276[30], 3}, {NULL}};
const Segment SGI299022[] = {{&S276[8], 3}, {NULL}};
const Segment SGI299023[] = {{&S276[13], 10}, {NULL}};
const Segment SGI299024[] = {{&S276[42], 7}, {NULL}};
const Segment SGI299025[] = {{&S276[35], 14}, {NULL}};
const Segment SGI299026[] = {{&S276[74], 3}, {NULL}};
const Segment SGI299027[] = {{&S277[30], 3}, {&S277[177], 3}, {&S277[177], 3}, {NULL}};
const Segment SGI299028[] = {{&S277[8], 3}, {&S277[8], 3}, {&S277[8], 3}, {NULL}};
const Segment SGI299029[] = {{&S277[13], 10}, {&S277[13], 10}, {&S277[13], 10}, {NULL}};
const Segment SGI299030[] = {{&S277[42], 7}, {NULL}};
const Segment SGI299031[] = {{&S277[35], 14}, {NULL}};
const Segment SGI299032[] = {{&S277[182], 5}, {&S277[182], 5}, {NULL}};
const Segment SGI299033[] = {{&S278[8], 3}, {NULL}};
const Segment SGI299034[] = {{&S279[30], 3}, {NULL}};
const Segment SGI299035[] = {{&S279[8], 3}, {NULL}};
const Segment SGI299036[] = {{&S279[13], 10}, {NULL}};
const Segment SGI299037[] = {{&S279[42], 7}, {NULL}};
const Segment SGI299038[] = {{&S279[35], 14}, {NULL}};
const Segment SGI299039[] = {{&S279[64], 3}, {NULL}};
const Segment SGI299040[] = {{&S279[79], 3}, {NULL}};
const Segment SGI299041[] = {{&S280[30], 3}, {NULL}};
const Segment SGI299042[] = {{&S280[8], 3}, {NULL}};
const Segment SGI299043[] = {{&S280[13], 10}, {NULL}};
const Segment SGI299044[] = {{&S280[42], 7}, {NULL}};
const Segment SGI299045[] = {{&S280[35], 14}, {NULL}};
const Segment SGI299046[] = {{&S280[138], 15}, {NULL}};
const Segment SGI299047[] = {{&S280[51], 6}, {NULL}};
const Segment SGI299048[] = {{&S281[30], 3}, {NULL}};
const Segment SGI299049[] = {{&S281[8], 3}, {NULL}};
const Segment SGI299050[] = {{&S281[13], 10}, {NULL}};
const Segment SGI299051[] = {{&S281[42], 7}, {NULL}};
const Segment SGI299052[] = {{&S281[35], 14}, {NULL}};
const Segment SGI299053[] = {{&S281[138], 15}, {NULL}};
const Segment SGI299054[] = {{&S281[51], 6}, {NULL}};
const Segment SGI299055[] = {{&S282[30], 3}, {NULL}};
const Segment SGI299056[] = {{&S282[8], 3}, {NULL}};
const Segment SGI299057[] = {{&S282[13], 10}, {NULL}};
const Segment SGI299058[] = {{&S282[42], 7}, {NULL}};
const Segment SGI299059[] = {{&S282[35], 14}, {NULL}};
const Segment SGI299060[] = {{&S282[51], 6}, {NULL}};
const Segment SGI299061[] = {{&S283[30], 3}, {NULL}};
const Segment SGI299062[] = {{&S283[8], 3}, {NULL}};
const Segment SGI299063[] = {{&S283[13], 10}, {NULL}};
const Segment SGI299064[] = {{&S283[42], 7}, {NULL}};
const Segment SGI299065[] = {{&S283[35], 14}, {NULL}};
const Segment SGI299066[] = {{&S283[131], 15}, {&S283[131], 15}, {NULL}};
const Segment SGI299067[] = {{&S283[210], 9}, {NULL}};
const Segment SGI299068[] = {{&S284[30], 3}, {&S284[131], 3}, {NULL}};
const Segment SGI299069[] = {{&S284[8], 3}, {&S284[8], 3}, {NULL}};
const Segment SGI299070[] = {{&S284[13], 10}, {&S284[13], 10}, {NULL}};
const Segment SGI299071[] = {{&S284[42], 7}, {&S284[42], 7}, {NULL}};
const Segment SGI299072[] = {{&S284[35], 14}, {&S284[35], 14}, {NULL}};
const Segment SGI299073[] = {{&S284[226], 15}, {NULL}};
const Segment SGI299074[] = {{&S284[51], 6}, {NULL}};
const Segment SGI299075[] = {{&S284[152], 3}, {NULL}};
const Segment SGI299076[] = {{&S285[30], 3}, {NULL}};
const Segment SGI299077[] = {{&S285[8], 3}, {NULL}};
const Segment SGI299078[] = {{&S285[13], 10}, {NULL}};
const Segment SGI299079[] = {{&S285[55], 7}, {NULL}};
const Segment SGI299080[] = {{&S285[48], 14}, {NULL}};
const Segment SGI299081[] = {{&S285[64], 3}, {&S285[64], 3}, {NULL}};
const Segment SGI299082[] = {{&S286[30], 3}, {NULL}};
const Segment SGI299083[] = {{&S286[8], 3}, {NULL}};
const Segment SGI299084[] = {{&S286[13], 10}, {NULL}};
const Segment SGI299085[] = {{&S286[42], 7}, {NULL}};
const Segment SGI299086[] = {{&S286[35], 14}, {NULL}};
const Segment SGI299087[] = {{&S286[51], 3}, {NULL}};
const Segment SGI299088[] = {{&S287[72], 14}, {NULL}};
const Segment SGI299089[] = {{&S287[8], 3}, {NULL}};
const Segment SGI299090[] = {{&S287[13], 10}, {NULL}};
const Segment SGI299091[] = {{&S287[61], 4}, {NULL}};
const Segment SGI299092[] = {{&S288[37], 3}, {NULL}};
const Segment SGI299093[] = {{&S288[49], 7}, {NULL}};
const Segment SGI299094[] = {{&S288[42], 14}, {NULL}};
const Segment SGI299095[] = {{&S288[117], 15}, {NULL}};
const Segment SGI299096[] = {{&S289[30], 3}, {NULL}};
const Segment SGI299097[] = {{&S289[8], 3}, {NULL}};
const Segment SGI299098[] = {{&S289[13], 10}, {NULL}};
const Segment SGI299099[] = {{&S289[42], 7}, {&S289[200], 15}, {NULL}};
const Segment SGI299100[] = {{&S289[35], 14}, {NULL}};
const Segment SGI299101[] = {{&S290[30], 3}, {&S290[108], 3}, {&S290[119], 3}, {&S290[30], 3}, {&S290[139], 3}, {NULL}};
const Segment SGI299102[] = {{&S290[8], 3}, {&S290[154], 3}, {&S290[8], 3}, {NULL}};
const Segment SGI299103[] = {{&S290[13], 10}, {NULL}};
const Segment SGI299104[] = {{&S290[42], 7}, {NULL}};
const Segment SGI299105[] = {{&S290[35], 14}, {NULL}};
const Segment SGI299106[] = {{&S291[30], 3}, {NULL}};
const Segment SGI299107[] = {{&S291[8], 3}, {NULL}};
const Segment SGI299108[] = {{&S291[13], 10}, {NULL}};
const Segment SGI299109[] = {{&S291[42], 7}, {NULL}};
const Segment SGI299110[] = {{&S291[35], 14}, {NULL}};
const Segment SGI299111[] = {{&S291[142], 3}, {&S291[142], 3}, {NULL}};
const Segment SGI299112[] = {{&S292[30], 3}, {NULL}};
const Segment SGI299113[] = {{&S292[8], 3}, {NULL}};
const Segment SGI299114[] = {{&S292[13], 10}, {NULL}};
const Segment SGI299115[] = {{&S292[42], 7}, {NULL}};
const Segment SGI299116[] = {{&S292[35], 14}, {NULL}};
const Segment SGI299117[] = {{&S292[51], 6}, {NULL}};
const Segment SGI299118[] = {{&S292[151], 9}, {NULL}};
const Segment SGI299119[] = {{&S293[30], 3}, {NULL}};
const Segment SGI299120[] = {{&S293[8], 3}, {NULL}};
const Segment SGI299121[] = {{&S293[13], 10}, {NULL}};
const Segment SGI299122[] = {{&S293[55], 7}, {NULL}};
const Segment SGI299123[] = {{&S293[48], 14}, {NULL}};
const Segment SGI299124[] = {{&S293[64], 3}, {&S293[64], 3}, {NULL}};
const Segment SGI299125[] = {{&S294[30], 3}, {NULL}};
const Segment SGI299126[] = {{&S294[8], 3}, {NULL}};
const Segment SGI299127[] = {{&S294[13], 10}, {NULL}};
const Segment SGI299128[] = {{&S294[55], 7}, {NULL}};
const Segment SGI299129[] = {{&S294[48], 14}, {NULL}};
const Segment SGI299130[] = {{&S294[64], 3}, {&S294[64], 3}, {NULL}};
const Segment SGI299131[] = {{&S295[30], 3}, {NULL}};
const Segment SGI299132[] = {{&S295[8], 3}, {NULL}};
const Segment SGI299133[] = {{&S295[13], 10}, {NULL}};
const Segment SGI299134[] = {{&S295[55], 7}, {NULL}};
const Segment SGI299135[] = {{&S295[48], 14}, {NULL}};
const Segment SGI299136[] = {{&S295[64], 3}, {&S295[64], 3}, {NULL}};
const Segment SGI299137[] = {{&S296[30], 3}, {&S296[101], 3}, {NULL}};
const Segment SGI299138[] = {{&S296[8], 3}, {&S296[8], 3}, {NULL}};
const Segment SGI299139[] = {{&S296[13], 10}, {&S296[13], 10}, {NULL}};
const Segment SGI299140[] = {{&S296[42], 7}, {&S296[42], 7}, {NULL}};
const Segment SGI299141[] = {{&S296[35], 14}, {&S296[35], 14}, {NULL}};
const Segment SGI299142[] = {{&S296[196], 15}, {NULL}};
const Segment SGI299143[] = {{&S296[268], 9}, {NULL}};
const Segment SGI299144[] = {{&S296[122], 3}, {NULL}};
const Segment SGI299145[] = {{&S297[33], 3}, {NULL}};
const Segment SGI299146[] = {{&S297[8], 3}, {NULL}};
const Segment SGI299147[] = {{&S297[13], 13}, {NULL}};
const Segment SGI299148[] = {{&S297[45], 7}, {NULL}};
const Segment SGI299149[] = {{&S297[38], 14}, {NULL}};
const Segment SGI299150[] = {{&S298[30], 3}, {&S298[209], 3}, {&S298[209], 3}, {NULL}};
const Segment SGI299151[] = {{&S298[8], 3}, {&S298[8], 3}, {&S298[8], 3}, {NULL}};
const Segment SGI299152[] = {{&S298[13], 10}, {&S298[13], 10}, {&S298[13], 10}, {NULL}};
const Segment SGI299153[] = {{&S298[42], 7}, {&S298[42], 3}, {&S298[42], 3}, {NULL}};
const Segment SGI299154[] = {{&S298[35], 14}, {NULL}};
const Segment SGI299155[] = {{&S299[46], 3}, {&S299[46], 3}, {NULL}};
const Segment SGI299156[] = {{&S299[8], 3}, {NULL}};
const Segment SGI299157[] = {{&S299[13], 10}, {NULL}};
const Segment SGI299158[] = {{&S299[32], 7}, {&S299[32], 7}, {NULL}};
const Segment SGI299159[] = {{&S299[25], 14}, {&S299[25], 14}, {NULL}};
const Segment SGI299160[] = {{&S300[18], 3}, {NULL}};
const Segment SGI299161[] = {{&S300[8], 3}, {NULL}};
const Segment SGI299162[] = {{&S300[13], 8}, {NULL}};
const Segment SGI299163[] = {{&S300[30], 7}, {NULL}};
const Segment SGI299164[] = {{&S301[30], 3}, {NULL}};
const Segment SGI299165[] = {{&S301[8], 3}, {NULL}};
const Segment SGI299166[] = {{&S301[13], 20}, {NULL}};
const Segment SGI299167[] = {{&S301[42], 7}, {NULL}};
const Segment SGI299168[] = {{&S301[35], 14}, {NULL}};
const Segment SGI299169[] = {{&S302[30], 3}, {NULL}};
const Segment SGI299170[] = {{&S302[8], 3}, {NULL}};
const Segment SGI299171[] = {{&S302[13], 10}, {NULL}};
const Segment SGI299172[] = {{&S302[42], 7}, {NULL}};
const Segment SGI299173[] = {{&S303[30], 3}, {NULL}};
const Segment SGI299174[] = {{&S303[8], 3}, {NULL}};
const Segment SGI299175[] = {{&S303[13], 10}, {NULL}};
const Segment SGI299176[] = {{&S303[42], 7}, {NULL}};
const Segment SGI299177[] = {{&S303[35], 14}, {NULL}};
const Segment SGI299178[] = {{&S303[131], 15}, {NULL}};
const Segment SGI299179[] = {{&S305[30], 3}, {NULL}};
const Segment SGI299180[] = {{&S305[8], 3}, {NULL}};
const Segment SGI299181[] = {{&S305[13], 10}, {NULL}};
const Segment SGI299182[] = {{&S305[42], 7}, {NULL}};
const Segment SGI299183[] = {{&S305[35], 14}, {NULL}};
const Segment SGI299184[] = {{&S305[51], 5}, {NULL}};
const Segment SGI299185[] = {{&S306[30], 3}, {NULL}};
const Segment SGI299186[] = {{&S306[8], 3}, {NULL}};
const Segment SGI299187[] = {{&S306[13], 10}, {NULL}};
const Segment SGI299188[] = {{&S306[42], 7}, {NULL}};
const Segment SGI299189[] = {{&S306[35], 14}, {NULL}};
const Segment SGI299190[] = {{&S306[131], 15}, {NULL}};
const Segment SGI299191[] = {{&S306[152], 9}, {NULL}};
const Segment SGI299192[] = {{&S307[30], 3}, {NULL}};
const Segment SGI299193[] = {{&S307[8], 3}, {NULL}};
const Segment SGI299194[] = {{&S307[13], 10}, {NULL}};
const Segment SGI299195[] = {{&S307[42], 7}, {NULL}};
const Segment SGI299196[] = {{&S307[35], 14}, {NULL}};
const Segment SGI299197[] = {{&S307[51], 6}, {NULL}};
const Segment SGI299198[] = {{&S308[30], 3}, {NULL}};
const Segment SGI299199[] = {{&S308[8], 3}, {NULL}};
const Segment SGI299200[] = {{&S308[13], 10}, {NULL}};
const Segment SGI299201[] = {{&S308[42], 7}, {NULL}};
const Segment SGI299202[] = {{&S308[35], 14}, {NULL}};
const Segment SGI299203[] = {{&S309[30], 3}, {NULL}};
const Segment SGI299204[] = {{&S309[8], 3}, {NULL}};
const Segment SGI299205[] = {{&S309[13], 10}, {NULL}};
const Segment SGI299206[] = {{&S309[42], 7}, {NULL}};
const Segment SGI299207[] = {{&S309[35], 14}, {NULL}};
const Segment SGI299208[] = {{&S309[143], 15}, {NULL}};
const Segment SGI299209[] = {{&S309[51], 5}, {NULL}};
const Segment SGI299210[] = {{&S309[184], 9}, {NULL}};
const Segment SGI299211[] = {{&S310[30], 3}, {NULL}};
const Segment SGI299212[] = {{&S310[8], 3}, {NULL}};
const Segment SGI299213[] = {{&S310[13], 10}, {NULL}};
const Segment SGI299214[] = {{&S310[42], 7}, {NULL}};
const Segment SGI299215[] = {{&S310[35], 14}, {NULL}};
const Segment SGI299216[] = {{&S310[138], 15}, {NULL}};
const Segment SGI299217[] = {{&S310[159], 4}, {NULL}};
const Segment SGI299218[] = {{&S310[51], 6}, {NULL}};
const Segment SGI299219[] = {{&S311[30], 3}, {NULL}};
const Segment SGI299220[] = {{&S311[8], 3}, {NULL}};
const Segment SGI299221[] = {{&S311[13], 10}, {NULL}};
const Segment SGI299222[] = {{&S311[42], 7}, {NULL}};
const Segment SGI299223[] = {{&S311[35], 14}, {NULL}};
const Segment SGI299224[] = {{&S311[138], 15}, {NULL}};
const Segment SGI299225[] = {{&S311[51], 6}, {NULL}};
const Segment SGI299226[] = {{&S312[30], 3}, {NULL}};
const Segment SGI299227[] = {{&S312[8], 3}, {NULL}};
const Segment SGI299228[] = {{&S312[13], 10}, {NULL}};
const Segment SGI299229[] = {{&S312[42], 7}, {NULL}};
const Segment SGI299230[] = {{&S312[35], 14}, {NULL}};
const Segment SGI299231[] = {{&S312[131], 15}, {&S312[131], 15}, {NULL}};
const Segment SGI299232[] = {{&S312[200], 9}, {NULL}};
const Segment SGI299233[] = {{&S313[30], 3}, {&S313[101], 3}, {NULL}};
const Segment SGI299234[] = {{&S313[8], 3}, {&S313[8], 3}, {NULL}};
const Segment SGI299235[] = {{&S313[13], 10}, {&S313[13], 10}, {NULL}};
const Segment SGI299236[] = {{&S313[42], 7}, {&S313[113], 7}, {NULL}};
const Segment SGI299237[] = {{&S313[35], 14}, {&S313[106], 14}, {NULL}};
const Segment SGI299238[] = {{&S313[196], 15}, {NULL}};
const Segment SGI299239[] = {{&S313[51], 6}, {NULL}};
const Segment SGI299240[] = {{&S313[122], 3}, {NULL}};
const Segment SGI299241[] = {{&S314[30], 3}, {NULL}};
const Segment SGI299242[] = {{&S314[8], 3}, {NULL}};
const Segment SGI299243[] = {{&S314[13], 10}, {NULL}};
const Segment SGI299244[] = {{&S314[42], 7}, {NULL}};
const Segment SGI299245[] = {{&S314[35], 14}, {NULL}};
const Segment SGI299246[] = {{&S314[131], 15}, {&S314[131], 15}, {NULL}};
const Segment SGI299247[] = {{&S315[30], 3}, {&S315[106], 3}, {NULL}};
const Segment SGI299248[] = {{&S315[8], 3}, {&S315[8], 3}, {NULL}};
const Segment SGI299249[] = {{&S315[13], 10}, {&S315[13], 10}, {NULL}};
const Segment SGI299250[] = {{&S315[42], 7}, {&S315[118], 7}, {NULL}};
const Segment SGI299251[] = {{&S315[35], 14}, {&S315[111], 14}, {NULL}};
const Segment SGI299252[] = {{&S315[201], 15}, {NULL}};
const Segment SGI299253[] = {{&S315[51], 5}, {NULL}};
const Segment SGI299254[] = {{&S315[127], 3}, {NULL}};
const Segment SGI299255[] = {{&S316[31], 3}, {NULL}};
const Segment SGI299256[] = {{&S316[43], 7}, {NULL}};
const Segment SGI299257[] = {{&S317[30], 3}, {NULL}};
const Segment SGI299258[] = {{&S317[8], 3}, {NULL}};
const Segment SGI299259[] = {{&S317[13], 10}, {NULL}};
const Segment SGI299260[] = {{&S317[42], 7}, {NULL}};
const Segment SGI299261[] = {{&S317[35], 14}, {NULL}};
const Segment SGI299262[] = {{&S317[143], 15}, {NULL}};
const Segment SGI299263[] = {{&S317[51], 5}, {NULL}};
const Segment SGI299264[] = {{&S318[30], 3}, {NULL}};
const Segment SGI299265[] = {{&S318[8], 3}, {NULL}};
const Segment SGI299266[] = {{&S318[13], 10}, {NULL}};
const Segment SGI299267[] = {{&S318[42], 7}, {NULL}};
const Segment SGI299268[] = {{&S318[35], 14}, {NULL}};
const Segment SGI299269[] = {{&S318[148], 9}, {NULL}};
const Segment SGI299270[] = {{&S318[64], 3}, {NULL}};
const Segment SGI299271[] = {{&S319[30], 3}, {&S319[30], 3}, {&S319[30], 3}, {NULL}};
const Segment SGI299272[] = {{&S319[8], 3}, {NULL}};
const Segment SGI299273[] = {{&S319[13], 10}, {NULL}};
const Segment SGI299274[] = {{&S319[42], 7}, {NULL}};
const Segment SGI299275[] = {{&S319[35], 14}, {NULL}};
const Segment SGI299276[] = {{&S319[131], 15}, {NULL}};
const Segment SGI299277[] = {{&S320[30], 3}, {NULL}};
const Segment SGI299278[] = {{&S320[8], 3}, {NULL}};
const Segment SGI299279[] = {{&S320[13], 10}, {NULL}};
const Segment SGI299280[] = {{&S320[42], 7}, {NULL}};
const Segment SGI299281[] = {{&S320[35], 14}, {NULL}};
const Segment SGI299282[] = {{&S320[151], 9}, {NULL}};
const Segment SGI299283[] = {{&S321[30], 3}, {NULL}};
const Segment SGI299284[] = {{&S321[8], 3}, {NULL}};
const Segment SGI299285[] = {{&S321[13], 10}, {NULL}};
const Segment SGI299286[] = {{&S321[42], 7}, {NULL}};
const Segment SGI299287[] = {{&S321[35], 14}, {NULL}};
const Segment SGI299288[] = {{&S321[143], 15}, {&S321[143], 15}, {NULL}};
const Segment SGI299289[] = {{&S321[51], 5}, {NULL}};
const Segment SGI299290[] = {{&S322[30], 3}, {&S322[30], 3}, {&S322[30], 3}, {NULL}};
const Segment SGI299291[] = {{&S322[8], 3}, {NULL}};
const Segment SGI299292[] = {{&S322[13], 10}, {NULL}};
const Segment SGI299293[] = {{&S322[42], 7}, {NULL}};
const Segment SGI299294[] = {{&S322[35], 14}, {NULL}};
const Segment SGI299295[] = {{&S322[131], 15}, {NULL}};
const Segment SGI299296[] = {{&S322[164], 9}, {NULL}};
const Segment SGI299297[] = {{&S323[30], 3}, {NULL}};
const Segment SGI299298[] = {{&S323[8], 3}, {NULL}};
const Segment SGI299299[] = {{&S323[13], 10}, {NULL}};
const Segment SGI299300[] = {{&S323[42], 7}, {NULL}};
const Segment SGI299301[] = {{&S323[35], 14}, {NULL}};
const Segment SGI299302[] = {{&S323[64], 3}, {NULL}};
const Segment SGI299303[] = {{&S324[30], 3}, {NULL}};
const Segment SGI299304[] = {{&S324[8], 3}, {NULL}};
const Segment SGI299305[] = {{&S324[13], 10}, {NULL}};
const Segment SGI299306[] = {{&S324[42], 7}, {NULL}};
const Segment SGI299307[] = {{&S324[35], 14}, {NULL}};
const Segment SGI299308[] = {{&S324[131], 15}, {NULL}};
const Segment SGI299309[] = {{&S324[167], 4}, {NULL}};
const Segment SGI299310[] = {{&S324[152], 9}, {NULL}};
const Segment SGI299311[] = {{&S325[30], 3}, {NULL}};
const Segment SGI299312[] = {{&S325[8], 3}, {NULL}};
const Segment SGI299313[] = {{&S325[13], 10}, {NULL}};
const Segment SGI299314[] = {{&S325[42], 7}, {NULL}};
const Segment SGI299315[] = {{&S325[35], 14}, {NULL}};
const Segment SGI299316[] = {{&S325[151], 15}, {&S325[151], 15}, {NULL}};
const Segment SGI299317[] = {{&S325[51], 5}, {NULL}};
const Segment SGI299318[] = {{&S325[200], 9}, {NULL}};
const Segment SGI299319[] = {{&S326[30], 3}, {NULL}};
const Segment SGI299320[] = {{&S326[8], 3}, {NULL}};
const Segment SGI299321[] = {{&S326[13], 10}, {NULL}};
const Segment SGI299322[] = {{&S326[42], 7}, {NULL}};
const Segment SGI299323[] = {{&S326[35], 14}, {NULL}};
const Segment SGI299324[] = {{&S326[143], 15}, {NULL}};
const Segment SGI299325[] = {{&S326[174], 4}, {NULL}};
const Segment SGI299326[] = {{&S326[51], 5}, {NULL}};
const Segment SGI299327[] = {{&S327[30], 3}, {NULL}};
const Segment SGI299328[] = {{&S327[8], 3}, {NULL}};
const Segment SGI299329[] = {{&S327[13], 10}, {NULL}};
const Segment SGI299330[] = {{&S327[42], 7}, {NULL}};
const Segment SGI299331[] = {{&S327[35], 14}, {NULL}};
const Segment SGI299332[] = {{&S327[131], 15}, {NULL}};
const Segment SGI299333[] = {{&S327[152], 4}, {NULL}};
const Segment SGI299334[] = {{&S328[30], 3}, {NULL}};
const Segment SGI299335[] = {{&S328[8], 3}, {NULL}};
const Segment SGI299336[] = {{&S328[13], 10}, {NULL}};
const Segment SGI299337[] = {{&S328[42], 7}, {NULL}};
const Segment SGI299338[] = {{&S328[35], 14}, {NULL}};
const Segment SGI299339[] = {{&S328[51], 6}, {NULL}};
const Segment SGI299340[] = {{&S328[150], 9}, {NULL}};
const Segment SGI299341[] = {{&S329[30], 3}, {&S329[106], 3}, {NULL}};
const Segment SGI299342[] = {{&S329[8], 3}, {&S329[8], 3}, {NULL}};
const Segment SGI299343[] = {{&S329[13], 10}, {&S329[13], 10}, {NULL}};
const Segment SGI299344[] = {{&S329[42], 7}, {&S329[118], 7}, {NULL}};
const Segment SGI299345[] = {{&S329[35], 14}, {&S329[111], 14}, {NULL}};
const Segment SGI299346[] = {{&S329[201], 15}, {NULL}};
const Segment SGI299347[] = {{&S329[51], 5}, {NULL}};
const Segment SGI299348[] = {{&S329[222], 9}, {NULL}};
const Segment SGI299349[] = {{&S329[127], 3}, {NULL}};
const Segment SGI299350[] = {{&S330[30], 3}, {&S330[94], 3}, {NULL}};
const Segment SGI299351[] = {{&S330[8], 3}, {&S330[8], 3}, {NULL}};
const Segment SGI299352[] = {{&S330[13], 10}, {&S330[13], 10}, {NULL}};
const Segment SGI299353[] = {{&S330[42], 7}, {&S330[106], 7}, {NULL}};
const Segment SGI299354[] = {{&S330[35], 14}, {&S330[99], 14}, {NULL}};
const Segment SGI299355[] = {{&S330[189], 15}, {NULL}};
const Segment SGI299356[] = {{&S330[232], 9}, {NULL}};
const Segment SGI299357[] = {{&S330[115], 3}, {NULL}};
const Segment SGI299358[] = {{&S331[30], 3}, {NULL}};
const Segment SGI299359[] = {{&S331[8], 3}, {NULL}};
const Segment SGI299360[] = {{&S331[13], 10}, {NULL}};
const Segment SGI299361[] = {{&S331[42], 7}, {NULL}};
const Segment SGI299362[] = {{&S331[35], 14}, {NULL}};
const Segment SGI299363[] = {{&S331[136], 15}, {NULL}};
const Segment SGI299364[] = {{&S331[202], 9}, {NULL}};
const Segment SGI299365[] = {{&S331[64], 3}, {NULL}};
const Segment SGI299366[] = {{&S332[30], 3}, {NULL}};
const Segment SGI299367[] = {{&S332[8], 3}, {NULL}};
const Segment SGI299368[] = {{&S332[13], 10}, {NULL}};
const Segment SGI299369[] = {{&S332[42], 7}, {NULL}};
const Segment SGI299370[] = {{&S332[35], 14}, {NULL}};
const Segment SGI299371[] = {{&S332[136], 15}, {NULL}};
const Segment SGI299372[] = {{&S332[64], 3}, {NULL}};
const Segment SGI299373[] = {{&S333[30], 3}, {NULL}};
const Segment SGI299374[] = {{&S333[8], 3}, {NULL}};
const Segment SGI299375[] = {{&S333[13], 10}, {NULL}};
const Segment SGI299376[] = {{&S333[42], 7}, {NULL}};
const Segment SGI299377[] = {{&S333[35], 14}, {NULL}};
const Segment SGI299378[] = {{&S333[51], 5}, {NULL}};
const Segment SGI299379[] = {{&S333[163], 9}, {NULL}};
const Segment SGI299380[] = {{&S334[30], 3}, {&S334[101], 3}, {NULL}};
const Segment SGI299381[] = {{&S334[8], 3}, {&S334[8], 3}, {NULL}};
const Segment SGI299382[] = {{&S334[13], 10}, {&S334[13], 10}, {NULL}};
const Segment SGI299383[] = {{&S334[42], 7}, {&S334[113], 7}, {NULL}};
const Segment SGI299384[] = {{&S334[35], 14}, {&S334[106], 14}, {NULL}};
const Segment SGI299385[] = {{&S334[51], 6}, {NULL}};
const Segment SGI299386[] = {{&S334[122], 3}, {NULL}};
const Segment SGI299387[] = {{&S335[30], 3}, {&S335[30], 3}, {NULL}};
const Segment SGI299388[] = {{&S335[8], 3}, {NULL}};
const Segment SGI299389[] = {{&S335[13], 10}, {NULL}};
const Segment SGI299390[] = {{&S335[42], 7}, {NULL}};
const Segment SGI299391[] = {{&S335[35], 14}, {NULL}};
const Segment SGI299392[] = {{&S335[131], 15}, {NULL}};
const Segment SGI299393[] = {{&S336[29], 3}, {NULL}};
const Segment SGI299394[] = {{&S336[8], 3}, {NULL}};
const Segment SGI299395[] = {{&S336[41], 7}, {NULL}};
const Segment SGI299396[] = {{&S336[34], 14}, {NULL}};
const Segment SGI299397[] = {{&S336[135], 15}, {NULL}};
const Segment SGI299398[] = {{&S336[50], 6}, {NULL}};
const Segment SGI299399[] = {{&S337[30], 3}, {NULL}};
const Segment SGI299400[] = {{&S337[8], 3}, {NULL}};
const Segment SGI299401[] = {{&S337[13], 10}, {NULL}};
const Segment SGI299402[] = {{&S337[42], 7}, {NULL}};
const Segment SGI299403[] = {{&S337[35], 14}, {NULL}};
const Segment SGI299404[] = {{&S337[132], 15}, {NULL}};
const Segment SGI299405[] = {{&S338[30], 3}, {&S338[94], 3}, {NULL}};
const Segment SGI299406[] = {{&S338[8], 3}, {&S338[8], 3}, {NULL}};
const Segment SGI299407[] = {{&S338[13], 10}, {&S338[13], 10}, {NULL}};
const Segment SGI299408[] = {{&S338[42], 7}, {&S338[106], 7}, {NULL}};
const Segment SGI299409[] = {{&S338[35], 14}, {&S338[99], 14}, {NULL}};
const Segment SGI299410[] = {{&S338[115], 3}, {NULL}};
const Segment SGI299411[] = {{&S339[30], 3}, {&S339[94], 3}, {NULL}};
const Segment SGI299412[] = {{&S339[8], 3}, {&S339[8], 3}, {NULL}};
const Segment SGI299413[] = {{&S339[13], 10}, {&S339[13], 10}, {NULL}};
const Segment SGI299414[] = {{&S339[42], 7}, {&S339[106], 7}, {NULL}};
const Segment SGI299415[] = {{&S339[35], 14}, {&S339[99], 14}, {NULL}};
const Segment SGI299416[] = {{&S339[189], 15}, {&S339[189], 15}, {NULL}};
const Segment SGI299417[] = {{&S339[115], 3}, {NULL}};
const Segment SGI299418[] = {{&S340[30], 3}, {&S340[172], 3}, {&S340[172], 3}, {NULL}};
const Segment SGI299419[] = {{&S340[8], 3}, {NULL}};
const Segment SGI299420[] = {{&S340[13], 10}, {NULL}};
const Segment SGI299421[] = {{&S340[42], 7}, {NULL}};
const Segment SGI299422[] = {{&S340[35], 14}, {NULL}};
const Segment SGI299423[] = {{&S340[131], 15}, {NULL}};
const Segment SGI299424[] = {{&S341[30], 3}, {&S341[30], 3}, {NULL}};
const Segment SGI299425[] = {{&S341[8], 3}, {NULL}};
const Segment SGI299426[] = {{&S341[13], 10}, {NULL}};
const Segment SGI299427[] = {{&S341[42], 7}, {NULL}};
const Segment SGI299428[] = {{&S341[35], 14}, {NULL}};
const Segment SGI299429[] = {{&S341[138], 15}, {NULL}};
const Segment SGI299430[] = {{&S341[51], 6}, {NULL}};
const Segment SGI299431[] = {{&S342[30], 3}, {NULL}};
const Segment SGI299432[] = {{&S342[8], 3}, {NULL}};
const Segment SGI299433[] = {{&S342[13], 10}, {NULL}};
const Segment SGI299434[] = {{&S342[42], 7}, {NULL}};
const Segment SGI299435[] = {{&S342[35], 14}, {NULL}};
const Segment SGI299436[] = {{&S342[151], 4}, {NULL}};
const Segment SGI299437[] = {{&S343[30], 3}, {NULL}};
const Segment SGI299438[] = {{&S343[8], 3}, {NULL}};
const Segment SGI299439[] = {{&S343[13], 10}, {NULL}};
const Segment SGI299440[] = {{&S343[42], 7}, {NULL}};
const Segment SGI299441[] = {{&S343[35], 14}, {NULL}};
const Segment SGI299442[] = {{&S343[178], 3}, {NULL}};
const Segment SGI299443[] = {{&S344[30], 3}, {NULL}};
const Segment SGI299444[] = {{&S344[8], 3}, {NULL}};
const Segment SGI299445[] = {{&S344[13], 10}, {NULL}};
const Segment SGI299446[] = {{&S344[42], 7}, {NULL}};
const Segment SGI299447[] = {{&S344[35], 14}, {NULL}};
const Segment SGI299448[] = {{&S344[143], 15}, {NULL}};
const Segment SGI299449[] = {{&S344[199], 4}, {NULL}};
const Segment SGI299450[] = {{&S344[51], 5}, {NULL}};
const Segment SGI299451[] = {{&S344[184], 9}, {NULL}};
const Segment SGI299452[] = {{&S345[30], 3}, {&S345[30], 3}, {NULL}};
const Segment SGI299453[] = {{&S345[8], 3}, {NULL}};
const Segment SGI299454[] = {{&S345[13], 10}, {NULL}};
const Segment SGI299455[] = {{&S345[42], 7}, {NULL}};
const Segment SGI299456[] = {{&S345[35], 14}, {NULL}};
const Segment SGI299457[] = {{&S345[131], 15}, {NULL}};
const Segment SGI299458[] = {{&S345[164], 9}, {NULL}};
const Segment SGI299459[] = {{&S346[30], 3}, {&S346[30], 3}, {NULL}};
const Segment SGI299460[] = {{&S346[8], 3}, {NULL}};
const Segment SGI299461[] = {{&S346[13], 10}, {NULL}};
const Segment SGI299462[] = {{&S346[42], 7}, {NULL}};
const Segment SGI299463[] = {{&S346[35], 14}, {NULL}};
const Segment SGI299464[] = {{&S346[136], 15}, {NULL}};
const Segment SGI299465[] = {{&S346[64], 3}, {NULL}};
const Segment SGI299466[] = {{&S347[30], 3}, {&S347[94], 3}, {NULL}};
const Segment SGI299467[] = {{&S347[8], 3}, {&S347[72], 3}, {NULL}};
const Segment SGI299468[] = {{&S347[13], 10}, {&S347[13], 10}, {NULL}};
const Segment SGI299469[] = {{&S347[42], 7}, {&S347[42], 7}, {NULL}};
const Segment SGI299470[] = {{&S347[35], 14}, {&S347[35], 14}, {NULL}};
const Segment SGI299471[] = {{&S347[195], 15}, {NULL}};
const Segment SGI299472[] = {{&S348[30], 14}, {NULL}};
const Segment SGI299473[] = {{&S348[8], 3}, {NULL}};
const Segment SGI299474[] = {{&S348[13], 10}, {NULL}};
const Segment SGI299475[] = {{&S348[53], 7}, {NULL}};
const Segment SGI299476[] = {{&S349[18], 3}, {NULL}};
const Segment SGI299477[] = {{&S349[8], 3}, {NULL}};
const Segment SGI299478[] = {{&S349[13], 8}, {NULL}};
const Segment SGI299479[] = {{&S349[30], 7}, {NULL}};
const Segment SGI299480[] = {{&S349[23], 14}, {NULL}};
const Segment SGI299481[] = {{&S350[30], 3}, {NULL}};
const Segment SGI299482[] = {{&S350[8], 3}, {NULL}};
const Segment SGI299483[] = {{&S350[13], 10}, {NULL}};
const Segment SGI299484[] = {{&S350[42], 7}, {NULL}};
const Segment SGI299485[] = {{&S350[35], 14}, {NULL}};
const Segment SGI299486[] = {{&S350[166], 4}, {NULL}};
const Segment SGI299487[] = {{&S350[151], 9}, {NULL}};
const Segment SGI299488[] = {{&S351[30], 3}, {&S351[207], 3}, {NULL}};
const Segment SGI299489[] = {{&S351[8], 3}, {NULL}};
const Segment SGI299490[] = {{&S351[13], 10}, {NULL}};
const Segment SGI299491[] = {{&S351[42], 7}, {NULL}};
const Segment SGI299492[] = {{&S351[35], 14}, {NULL}};
const Segment SGI299493[] = {{&S351[131], 15}, {NULL}};
const Segment SGI299494[] = {{&S351[164], 9}, {NULL}};
const Segment SGI299495[] = {{&S352[30], 3}, {&S352[94], 3}, {NULL}};
const Segment SGI299496[] = {{&S352[8], 3}, {&S352[8], 3}, {NULL}};
const Segment SGI299497[] = {{&S352[13], 10}, {&S352[13], 10}, {NULL}};
const Segment SGI299498[] = {{&S352[42], 7}, {&S352[106], 7}, {NULL}};
const Segment SGI299499[] = {{&S352[35], 14}, {&S352[99], 14}, {NULL}};
const Segment SGI299500[] = {{&S352[189], 15}, {NULL}};
const Segment SGI299501[] = {{&S352[220], 4}, {NULL}};
const Segment SGI299502[] = {{&S352[115], 3}, {NULL}};
const Segment SGI299503[] = {{&S353[31], 14}, {NULL}};
const Segment SGI299504[] = {{&S353[8], 3}, {NULL}};
const Segment SGI299505[] = {{&S353[13], 11}, {NULL}};
const Segment SGI299506[] = {{&S353[65], 31}, {NULL}};
const Segment SGI299507[] = {{&S354[31], 14}, {NULL}};
const Segment SGI299508[] = {{&S354[8], 3}, {NULL}};
const Segment SGI299509[] = {{&S354[13], 11}, {NULL}};
const Segment SGI299510[] = {{&S354[65], 17}, {NULL}};
const Segment SGI299511[] = {{&S355[30], 3}, {NULL}};
const Segment SGI299512[] = {{&S355[8], 3}, {NULL}};
const Segment SGI299513[] = {{&S355[13], 10}, {NULL}};
const Segment SGI299514[] = {{&S355[42], 7}, {NULL}};
const Segment SGI299515[] = {{&S355[35], 14}, {NULL}};
const Segment SGI299516[] = {{&S355[148], 15}, {NULL}};
const Segment SGI299517[] = {{&S355[51], 5}, {NULL}};
const Segment SGI299518[] = {{&S355[76], 3}, {NULL}};
const Segment SGI299519[] = {{&S356[30], 3}, {NULL}};
const Segment SGI299520[] = {{&S356[8], 3}, {NULL}};
const Segment SGI299521[] = {{&S356[13], 10}, {NULL}};
const Segment SGI299522[] = {{&S356[42], 7}, {NULL}};
const Segment SGI299523[] = {{&S356[35], 14}, {NULL}};
const Segment SGI299524[] = {{&S356[138], 15}, {NULL}};
const Segment SGI299525[] = {{&S356[51], 6}, {NULL}};
const Segment SGI299526[] = {{&S356[181], 9}, {NULL}};
const Segment SGI299527[] = {{&S357[30], 14}, {NULL}};
const Segment SGI299528[] = {{&S357[8], 3}, {NULL}};
const Segment SGI299529[] = {{&S357[13], 10}, {NULL}};
const Segment SGI299530[] = {{&S357[53], 7}, {NULL}};
const Segment SGI299531[] = {{&S358[30], 3}, {NULL}};
const Segment SGI299532[] = {{&S358[8], 3}, {NULL}};
const Segment SGI299533[] = {{&S358[13], 10}, {NULL}};
const Segment SGI299534[] = {{&S358[42], 7}, {&S358[42], 7}, {NULL}};
const Segment SGI299535[] = {{&S358[35], 14}, {&S358[35], 14}, {NULL}};
const Segment SGI299536[] = {{&S359[30], 3}, {&S359[30], 3}, {NULL}};
const Segment SGI299537[] = {{&S359[8], 3}, {NULL}};
const Segment SGI299538[] = {{&S359[13], 10}, {NULL}};
const Segment SGI299539[] = {{&S359[42], 7}, {NULL}};
const Segment SGI299540[] = {{&S359[35], 14}, {NULL}};
const Segment SGI299541[] = {{&S359[51], 6}, {NULL}};
const Segment SGI299542[] = {{&S360[30], 3}, {NULL}};
const Segment SGI299543[] = {{&S360[8], 3}, {NULL}};
const Segment SGI299544[] = {{&S360[13], 10}, {NULL}};
const Segment SGI299545[] = {{&S360[42], 7}, {&S360[58], 16}, {NULL}};
const Segment SGI299546[] = {{&S360[35], 14}, {NULL}};
const Segment SGI299547[] = {{&S361[30], 3}, {&S361[40], 3}, {NULL}};
const Segment SGI299548[] = {{&S361[8], 3}, {NULL}};
const Segment SGI299549[] = {{&S361[13], 10}, {NULL}};
const Segment SGI299550[] = {{&S361[61], 7}, {NULL}};
const Segment SGI299551[] = {{&S361[54], 14}, {NULL}};
const Segment SGI299552[] = {{&S361[157], 15}, {NULL}};
const Segment SGI299553[] = {{&S361[70], 6}, {NULL}};
const Segment SGI299554[] = {{&S362[30], 3}, {&S362[94], 3}, {NULL}};
const Segment SGI299555[] = {{&S362[8], 3}, {&S362[8], 3}, {NULL}};
const Segment SGI299556[] = {{&S362[13], 10}, {&S362[13], 10}, {NULL}};
const Segment SGI299557[] = {{&S362[42], 7}, {&S362[106], 7}, {NULL}};
const Segment SGI299558[] = {{&S362[35], 14}, {&S362[99], 14}, {NULL}};
const Segment SGI299559[] = {{&S362[209], 9}, {NULL}};
const Segment SGI299560[] = {{&S362[115], 3}, {NULL}};
const Segment SGI299561[] = {{&S363[30], 3}, {NULL}};
const Segment SGI299562[] = {{&S363[8], 3}, {NULL}};
const Segment SGI299563[] = {{&S363[13], 10}, {NULL}};
const Segment SGI299564[] = {{&S363[42], 7}, {NULL}};
const Segment SGI299565[] = {{&S363[35], 14}, {NULL}};
const Segment SGI299566[] = {{&S363[51], 6}, {NULL}};
const Segment SGI299567[] = {{&S363[71], 3}, {NULL}};
const Segment SGI299568[] = {{&S364[30], 3}, {NULL}};
const Segment SGI299569[] = {{&S364[8], 3}, {NULL}};
const Segment SGI299570[] = {{&S364[13], 10}, {NULL}};
const Segment SGI299571[] = {{&S364[42], 7}, {NULL}};
const Segment SGI299572[] = {{&S364[35], 14}, {NULL}};
const Segment SGI299573[] = {{&S364[131], 15}, {NULL}};
const Segment SGI299574[] = {{&S364[209], 5}, {NULL}};
const Segment SGI299575[] = {{&S365[30], 3}, {NULL}};
const Segment SGI299576[] = {{&S365[8], 3}, {NULL}};
const Segment SGI299577[] = {{&S365[13], 10}, {NULL}};
const Segment SGI299578[] = {{&S365[42], 7}, {&S365[211], 10}, {NULL}};
const Segment SGI299579[] = {{&S365[35], 14}, {NULL}};
const Segment SGI299580[] = {{&S365[136], 15}, {NULL}};
const Segment SGI299581[] = {{&S365[64], 3}, {NULL}};
const Segment SGI299582[] = {{&S366[30], 3}, {NULL}};
const Segment SGI299583[] = {{&S366[8], 3}, {NULL}};
const Segment SGI299584[] = {{&S366[13], 10}, {NULL}};
const Segment SGI299585[] = {{&S366[42], 7}, {&S366[206], 10}, {NULL}};
const Segment SGI299586[] = {{&S366[35], 14}, {NULL}};
const Segment SGI299587[] = {{&S366[131], 15}, {NULL}};
const Segment SGI299588[] = {{&S367[30], 3}, {&S367[101], 3}, {NULL}};
const Segment SGI299589[] = {{&S367[8], 3}, {&S367[79], 3}, {NULL}};
const Segment SGI299590[] = {{&S367[13], 10}, {&S367[13], 10}, {NULL}};
const Segment SGI299591[] = {{&S367[42], 7}, {&S367[42], 7}, {NULL}};
const Segment SGI299592[] = {{&S367[35], 14}, {&S367[35], 14}, {NULL}};
const Segment SGI299593[] = {{&S367[202], 15}, {NULL}};
const Segment SGI299594[] = {{&S367[51], 6}, {NULL}};
const Segment SGI299595[] = {{&S368[30], 3}, {&S368[94], 3}, {&S368[30], 3}, {NULL}};
const Segment SGI299596[] = {{&S368[8], 3}, {&S368[8], 3}, {NULL}};
const Segment SGI299597[] = {{&S368[13], 10}, {&S368[13], 10}, {NULL}};
const Segment SGI299598[] = {{&S368[42], 7}, {&S368[106], 7}, {NULL}};
const Segment SGI299599[] = {{&S368[35], 14}, {&S368[99], 14}, {NULL}};
const Segment SGI299600[] = {{&S368[189], 15}, {NULL}};
const Segment SGI299601[] = {{&S368[115], 3}, {NULL}};
const Segment SGI299602[] = {{&S369[30], 3}, {&S369[92], 3}, {NULL}};
const Segment SGI299603[] = {{&S369[8], 3}, {NULL}};
const Segment SGI299604[] = {{&S369[13], 10}, {NULL}};
const Segment SGI299605[] = {{&S369[42], 7}, {NULL}};
const Segment SGI299606[] = {{&S369[35], 14}, {NULL}};
const Segment SGI299607[] = {{&S369[51], 6}, {NULL}};
const Segment SGI299608[] = {{&S370[30], 3}, {NULL}};
const Segment SGI299609[] = {{&S370[8], 3}, {NULL}};
const Segment SGI299610[] = {{&S370[13], 10}, {NULL}};
const Segment SGI299611[] = {{&S370[42], 7}, {NULL}};
const Segment SGI299612[] = {{&S370[35], 14}, {NULL}};
const Segment SGI299613[] = {{&S370[136], 15}, {&S370[136], 15}, {NULL}};
const Segment SGI299614[] = {{&S370[64], 3}, {NULL}};
const Segment SGI299615[] = {{&S371[30], 3}, {&S371[207], 3}, {NULL}};
const Segment SGI299616[] = {{&S371[8], 3}, {NULL}};
const Segment SGI299617[] = {{&S371[13], 10}, {NULL}};
const Segment SGI299618[] = {{&S371[42], 7}, {NULL}};
const Segment SGI299619[] = {{&S371[35], 14}, {NULL}};
const Segment SGI299620[] = {{&S371[131], 15}, {NULL}};
const Segment SGI299621[] = {{&S371[187], 9}, {NULL}};
const Segment SGI299622[] = {{&S372[30], 3}, {&S372[30], 3}, {&S372[30], 3}, {NULL}};
const Segment SGI299623[] = {{&S372[8], 3}, {NULL}};
const Segment SGI299624[] = {{&S372[13], 10}, {NULL}};
const Segment SGI299625[] = {{&S372[42], 7}, {NULL}};
const Segment SGI299626[] = {{&S372[35], 14}, {NULL}};
const Segment SGI299627[] = {{&S372[136], 15}, {NULL}};
const Segment SGI299628[] = {{&S372[64], 3}, {NULL}};
const Segment SGI299629[] = {{&S373[30], 3}, {NULL}};
const Segment SGI299630[] = {{&S373[8], 3}, {NULL}};
const Segment SGI299631[] = {{&S373[13], 10}, {NULL}};
const Segment SGI299632[] = {{&S373[42], 7}, {NULL}};
const Segment SGI299633[] = {{&S373[35], 14}, {NULL}};
const Segment SGI299634[] = {{&S373[170], 4}, {NULL}};
const Segment SGI299635[] = {{&S373[51], 6}, {NULL}};
const Segment SGI299636[] = {{&S374[30], 3}, {NULL}};
const Segment SGI299637[] = {{&S374[8], 3}, {&S374[8], 3}, {NULL}};
const Segment SGI299638[] = {{&S374[13], 10}, {NULL}};
const Segment SGI299639[] = {{&S374[42], 7}, {NULL}};
const Segment SGI299640[] = {{&S374[35], 14}, {NULL}};
const Segment SGI299641[] = {{&S374[144], 15}, {NULL}};
const Segment SGI299642[] = {{&S375[30], 3}, {&S375[106], 3}, {NULL}};
const Segment SGI299643[] = {{&S375[8], 3}, {&S375[8], 3}, {NULL}};
const Segment SGI299644[] = {{&S375[13], 10}, {&S375[13], 10}, {NULL}};
const Segment SGI299645[] = {{&S375[42], 7}, {&S375[118], 7}, {NULL}};
const Segment SGI299646[] = {{&S375[35], 14}, {&S375[111], 14}, {NULL}};
const Segment SGI299647[] = {{&S375[51], 5}, {NULL}};
const Segment SGI299648[] = {{&S375[127], 3}, {NULL}};
const Segment SGI299649[] = {{&S376[30], 3}, {&S376[187], 3}, {&S376[187], 3}, {NULL}};
const Segment SGI299650[] = {{&S376[8], 3}, {NULL}};
const Segment SGI299651[] = {{&S376[13], 10}, {NULL}};
const Segment SGI299652[] = {{&S376[42], 7}, {NULL}};
const Segment SGI299653[] = {{&S376[35], 14}, {NULL}};
const Segment SGI299654[] = {{&S376[131], 15}, {NULL}};
const Segment SGI299655[] = {{&S376[164], 9}, {NULL}};
const Segment SGI299656[] = {{&S377[30], 3}, {&S377[192], 3}, {NULL}};
const Segment SGI299657[] = {{&S377[8], 3}, {NULL}};
const Segment SGI299658[] = {{&S377[13], 10}, {NULL}};
const Segment SGI299659[] = {{&S377[42], 7}, {NULL}};
const Segment SGI299660[] = {{&S377[35], 14}, {NULL}};
const Segment SGI299661[] = {{&S377[131], 15}, {NULL}};
const Segment SGI299662[] = {{&S378[30], 3}, {NULL}};
const Segment SGI299663[] = {{&S378[8], 3}, {NULL}};
const Segment SGI299664[] = {{&S378[13], 10}, {NULL}};
const Segment SGI299665[] = {{&S378[42], 7}, {NULL}};
const Segment SGI299666[] = {{&S378[35], 14}, {NULL}};
const Segment SGI299667[] = {{&S378[136], 15}, {&S378[136], 15}, {NULL}};
const Segment SGI299668[] = {{&S378[205], 9}, {NULL}};
const Segment SGI299669[] = {{&S378[64], 3}, {NULL}};
const Segment SGI299670[] = {{&S379[30], 3}, {NULL}};
const Segment SGI299671[] = {{&S379[8], 3}, {NULL}};
const Segment SGI299672[] = {{&S379[13], 10}, {NULL}};
const Segment SGI299673[] = {{&S379[42], 7}, {&S379[171], 34}, {NULL}};
const Segment SGI299674[] = {{&S379[35], 14}, {NULL}};
const Segment SGI299675[] = {{&S379[131], 15}, {NULL}};
const Segment SGI299676[] = {{&S380[30], 3}, {&S380[118], 3}, {NULL}};
const Segment SGI299677[] = {{&S380[8], 3}, {&S380[8], 3}, {NULL}};
const Segment SGI299678[] = {{&S380[13], 10}, {&S380[13], 10}, {NULL}};
const Segment SGI299679[] = {{&S380[42], 7}, {&S380[130], 7}, {NULL}};
const Segment SGI299680[] = {{&S380[35], 14}, {&S380[123], 14}, {NULL}};
const Segment SGI299681[] = {{&S380[173], 15}, {NULL}};
const Segment SGI299682[] = {{&S380[139], 3}, {NULL}};
const Segment SGI299683[] = {{&S381[30], 3}, {&S381[30], 3}, {NULL}};
const Segment SGI299684[] = {{&S381[8], 3}, {&S381[8], 3}, {NULL}};
const Segment SGI299685[] = {{&S381[13], 10}, {&S381[13], 10}, {NULL}};
const Segment SGI299686[] = {{&S381[42], 7}, {&S381[42], 7}, {NULL}};
const Segment SGI299687[] = {{&S381[35], 14}, {&S381[35], 14}, {NULL}};
const Segment SGI299688[] = {{&S381[131], 15}, {&S381[131], 15}, {NULL}};
const Segment SGI299689[] = {{&S382[30], 3}, {&S382[30], 3}, {&S382[30], 3}, {NULL}};
const Segment SGI299690[] = {{&S382[8], 3}, {NULL}};
const Segment SGI299691[] = {{&S382[13], 10}, {NULL}};
const Segment SGI299692[] = {{&S382[42], 7}, {NULL}};
const Segment SGI299693[] = {{&S382[35], 14}, {NULL}};
const Segment SGI299694[] = {{&S382[136], 15}, {NULL}};
const Segment SGI299695[] = {{&S382[169], 9}, {NULL}};
const Segment SGI299696[] = {{&S382[64], 3}, {NULL}};
const Segment SGI299697[] = {{&S383[30], 3}, {NULL}};
const Segment SGI299698[] = {{&S383[8], 3}, {NULL}};
const Segment SGI299699[] = {{&S383[13], 10}, {NULL}};
const Segment SGI299700[] = {{&S383[42], 7}, {NULL}};
const Segment SGI299701[] = {{&S383[35], 14}, {NULL}};
const Segment SGI299702[] = {{&S383[163], 4}, {NULL}};
const Segment SGI299703[] = {{&S383[51], 5}, {NULL}};
const Segment SGI299704[] = {{&S384[30], 3}, {NULL}};
const Segment SGI299705[] = {{&S384[8], 3}, {NULL}};
const Segment SGI299706[] = {{&S384[13], 10}, {NULL}};
const Segment SGI299707[] = {{&S384[42], 7}, {NULL}};
const Segment SGI299708[] = {{&S384[35], 14}, {NULL}};
const Segment SGI299709[] = {{&S384[51], 5}, {NULL}};
const Segment SGI299710[] = {{&S384[76], 3}, {NULL}};
const Segment SGI299711[] = {{&S385[30], 3}, {&S385[94], 3}, {&S385[30], 3}, {NULL}};
const Segment SGI299712[] = {{&S385[8], 3}, {&S385[8], 3}, {NULL}};
const Segment SGI299713[] = {{&S385[13], 10}, {&S385[13], 10}, {NULL}};
const Segment SGI299714[] = {{&S385[42], 7}, {&S385[106], 7}, {NULL}};
const Segment SGI299715[] = {{&S385[35], 14}, {&S385[99], 14}, {NULL}};
const Segment SGI299716[] = {{&S385[189], 15}, {NULL}};
const Segment SGI299717[] = {{&S385[222], 9}, {NULL}};
const Segment SGI299718[] = {{&S385[115], 3}, {NULL}};
const Segment SGI299719[] = {{&S386[30], 3}, {NULL}};
const Segment SGI299720[] = {{&S386[8], 3}, {NULL}};
const Segment SGI299721[] = {{&S386[13], 10}, {NULL}};
const Segment SGI299722[] = {{&S386[42], 7}, {NULL}};
const Segment SGI299723[] = {{&S386[35], 14}, {NULL}};
const Segment SGI299724[] = {{&S386[51], 5}, {NULL}};
const Segment SGI299725[] = {{&S386[180], 9}, {NULL}};
const Segment SGI299726[] = {{&S386[76], 3}, {NULL}};
const Segment SGI299727[] = {{&S387[30], 3}, {&S387[106], 3}, {&S387[106], 3}, {NULL}};
const Segment SGI299728[] = {{&S387[8], 3}, {&S387[8], 3}, {&S387[8], 3}, {NULL}};
const Segment SGI299729[] = {{&S387[13], 10}, {&S387[13], 10}, {&S387[13], 10}, {NULL}};
const Segment SGI299730[] = {{&S387[42], 7}, {&S387[118], 7}, {&S387[118], 7}, {NULL}};
const Segment SGI299731[] = {{&S387[35], 14}, {&S387[111], 14}, {&S387[111], 14}, {NULL}};
const Segment SGI299732[] = {{&S387[315], 15}, {NULL}};
const Segment SGI299733[] = {{&S387[127], 3}, {&S387[127], 3}, {NULL}};
const Segment SGI299734[] = {{&S388[30], 3}, {NULL}};
const Segment SGI299735[] = {{&S388[8], 3}, {NULL}};
const Segment SGI299736[] = {{&S388[13], 10}, {NULL}};
const Segment SGI299737[] = {{&S388[55], 7}, {NULL}};
const Segment SGI299738[] = {{&S388[48], 14}, {NULL}};
const Segment SGI299739[] = {{&S388[64], 3}, {NULL}};
const Segment SGI299740[] = {{&S389[30], 3}, {&S389[30], 3}, {NULL}};
const Segment SGI299741[] = {{&S389[8], 3}, {&S389[8], 3}, {NULL}};
const Segment SGI299742[] = {{&S389[13], 10}, {&S389[13], 10}, {NULL}};
const Segment SGI299743[] = {{&S389[42], 7}, {&S389[42], 7}, {NULL}};
const Segment SGI299744[] = {{&S389[35], 14}, {&S389[35], 14}, {NULL}};
const Segment SGI299745[] = {{&S389[138], 15}, {&S389[138], 15}, {NULL}};
const Segment SGI299746[] = {{&S389[51], 6}, {NULL}};
const Segment SGI299747[] = {{&S390[30], 3}, {NULL}};
const Segment SGI299748[] = {{&S390[8], 3}, {NULL}};
const Segment SGI299749[] = {{&S390[13], 10}, {NULL}};
const Segment SGI299750[] = {{&S390[42], 7}, {NULL}};
const Segment SGI299751[] = {{&S390[35], 14}, {NULL}};
const Segment SGI299752[] = {{&S390[148], 15}, {NULL}};
const Segment SGI299753[] = {{&S390[51], 5}, {NULL}};
const Segment SGI299754[] = {{&S390[189], 9}, {NULL}};
const Segment SGI299755[] = {{&S390[76], 3}, {NULL}};
const Segment SGI299756[] = {{&S391[30], 3}, {NULL}};
const Segment SGI299757[] = {{&S391[8], 3}, {NULL}};
const Segment SGI299758[] = {{&S391[13], 10}, {NULL}};
const Segment SGI299759[] = {{&S391[42], 7}, {NULL}};
const Segment SGI299760[] = {{&S391[35], 14}, {NULL}};
const Segment SGI299761[] = {{&S391[131], 15}, {NULL}};
const Segment SGI299762[] = {{&S391[224], 5}, {NULL}};
const Segment SGI299763[] = {{&S391[203], 9}, {NULL}};
const Segment SGI299764[] = {{&S392[30], 3}, {&S392[86], 3}, {NULL}};
const Segment SGI299765[] = {{&S392[8], 3}, {&S392[8], 3}, {NULL}};
const Segment SGI299766[] = {{&S392[13], 10}, {&S392[13], 10}, {NULL}};
const Segment SGI299767[] = {{&S392[42], 7}, {&S392[98], 7}, {NULL}};
const Segment SGI299768[] = {{&S392[35], 14}, {&S392[91], 14}, {NULL}};
const Segment SGI299769[] = {{&S392[51], 3}, {&S392[51], 3}, {NULL}};
const Segment SGI299770[] = {{&S393[30], 3}, {&S393[189], 3}, {NULL}};
const Segment SGI299771[] = {{&S393[8], 3}, {NULL}};
const Segment SGI299772[] = {{&S393[13], 10}, {NULL}};
const Segment SGI299773[] = {{&S393[42], 7}, {NULL}};
const Segment SGI299774[] = {{&S393[35], 14}, {NULL}};
const Segment SGI299775[] = {{&S393[131], 15}, {NULL}};
const Segment SGI299776[] = {{&S394[30], 3}, {NULL}};
const Segment SGI299777[] = {{&S394[8], 3}, {NULL}};
const Segment SGI299778[] = {{&S394[13], 10}, {NULL}};
const Segment SGI299779[] = {{&S394[42], 14}, {NULL}};
const Segment SGI299780[] = {{&S394[138], 15}, {NULL}};
const Segment SGI299781[] = {{&S395[30], 3}, {NULL}};
const Segment SGI299782[] = {{&S395[8], 3}, {NULL}};
const Segment SGI299783[] = {{&S395[13], 10}, {NULL}};
const Segment SGI299784[] = {{&S395[42], 7}, {NULL}};
const Segment SGI299785[] = {{&S395[35], 14}, {NULL}};
const Segment SGI299786[] = {{&S395[146], 15}, {NULL}};
const Segment SGI299787[] = {{&S395[51], 6}, {NULL}};
const Segment SGI299788[] = {{&S395[182], 3}, {NULL}};
const Segment SGI299789[] = {{&S396[30], 3}, {&S396[30], 3}, {NULL}};
const Segment SGI299790[] = {{&S396[8], 3}, {NULL}};
const Segment SGI299791[] = {{&S396[13], 10}, {NULL}};
const Segment SGI299792[] = {{&S396[42], 7}, {NULL}};
const Segment SGI299793[] = {{&S396[35], 14}, {NULL}};
const Segment SGI299794[] = {{&S397[30], 3}, {NULL}};
const Segment SGI299795[] = {{&S397[8], 3}, {NULL}};
const Segment SGI299796[] = {{&S397[13], 10}, {NULL}};
const Segment SGI299797[] = {{&S397[42], 7}, {&S397[145], 20}, {NULL}};
const Segment SGI299798[] = {{&S397[35], 14}, {NULL}};
const Segment SGI299799[] = {{&S397[146], 15}, {NULL}};
const Segment SGI299800[] = {{&S398[30], 3}, {NULL}};
const Segment SGI299801[] = {{&S398[8], 3}, {NULL}};
const Segment SGI299802[] = {{&S398[13], 10}, {NULL}};
const Segment SGI299803[] = {{&S398[42], 7}, {&S398[93], 20}, {NULL}};
const Segment SGI299804[] = {{&S398[35], 14}, {NULL}};
const Segment SGI299805[] = {{&S398[94], 15}, {NULL}};
const Segment SGI299806[] = {{&S398[51], 3}, {NULL}};
const Segment SGI299807[] = {{&S399[30], 3}, {&S399[101], 3}, {NULL}};
const Segment SGI299808[] = {{&S399[8], 3}, {&S399[8], 3}, {NULL}};
const Segment SGI299809[] = {{&S399[13], 10}, {&S399[13], 10}, {NULL}};
const Segment SGI299810[] = {{&S399[42], 7}, {&S399[113], 7}, {NULL}};
const Segment SGI299811[] = {{&S399[35], 14}, {&S399[106], 14}, {NULL}};
const Segment SGI299812[] = {{&S399[196], 15}, {NULL}};
const Segment SGI299813[] = {{&S399[229], 4}, {NULL}};
const Segment SGI299814[] = {{&S399[51], 6}, {NULL}};
const Segment SGI299815[] = {{&S399[122], 3}, {NULL}};
const Segment SGI299816[] = {{&S400[30], 3}, {NULL}};
const Segment SGI299817[] = {{&S400[8], 3}, {NULL}};
const Segment SGI299818[] = {{&S400[13], 10}, {NULL}};
const Segment SGI299819[] = {{&S400[42], 7}, {NULL}};
const Segment SGI299820[] = {{&S401[30], 3}, {&S401[94], 3}, {NULL}};
const Segment SGI299821[] = {{&S401[8], 3}, {&S401[8], 3}, {NULL}};
const Segment SGI299822[] = {{&S401[13], 10}, {&S401[13], 10}, {NULL}};
const Segment SGI299823[] = {{&S401[42], 7}, {&S401[106], 7}, {NULL}};
const Segment SGI299824[] = {{&S401[35], 14}, {&S401[99], 14}, {NULL}};
const Segment SGI299825[] = {{&S401[189], 15}, {NULL}};
const Segment SGI299826[] = {{&S401[237], 4}, {NULL}};
const Segment SGI299827[] = {{&S401[222], 9}, {NULL}};
const Segment SGI299828[] = {{&S401[115], 3}, {NULL}};
const Segment SGI299829[] = {{&S402[30], 3}, {NULL}};
const Segment SGI299830[] = {{&S402[8], 3}, {NULL}};
const Segment SGI299831[] = {{&S402[13], 10}, {NULL}};
const Segment SGI299832[] = {{&S402[42], 7}, {&S402[186], 34}, {NULL}};
const Segment SGI299833[] = {{&S402[35], 14}, {NULL}};
const Segment SGI299834[] = {{&S402[131], 15}, {NULL}};
const Segment SGI299835[] = {{&S402[164], 9}, {NULL}};
const Segment SGI299836[] = {{&S403[30], 3}, {&S403[30], 3}, {&S403[30], 3}, {NULL}};
const Segment SGI299837[] = {{&S403[8], 3}, {NULL}};
const Segment SGI299838[] = {{&S403[13], 10}, {NULL}};
const Segment SGI299839[] = {{&S403[42], 7}, {NULL}};
const Segment SGI299840[] = {{&S403[35], 14}, {NULL}};
const Segment SGI299841[] = {{&S403[138], 15}, {NULL}};
const Segment SGI299842[] = {{&S403[51], 6}, {NULL}};
const Segment SGI299843[] = {{&S404[30], 3}, {&S404[69], 3}, {NULL}};
const Segment SGI299844[] = {{&S404[8], 3}, {NULL}};
const Segment SGI299845[] = {{&S404[13], 10}, {NULL}};
const Segment SGI299846[] = {{&S404[42], 7}, {NULL}};
const Segment SGI299847[] = {{&S404[35], 14}, {NULL}};
const Segment SGI299848[] = {{&S405[30], 3}, {NULL}};
const Segment SGI299849[] = {{&S405[8], 3}, {NULL}};
const Segment SGI299850[] = {{&S405[13], 10}, {NULL}};
const Segment SGI299851[] = {{&S405[42], 7}, {NULL}};
const Segment SGI299852[] = {{&S405[35], 14}, {NULL}};
const Segment SGI299853[] = {{&S405[131], 15}, {NULL}};
const Segment SGI299854[] = {{&S405[182], 3}, {NULL}};
const Segment SGI299855[] = {{&S405[162], 9}, {NULL}};
const Segment SGI299856[] = {{&S406[30], 3}, {NULL}};
const Segment SGI299857[] = {{&S406[8], 3}, {NULL}};
const Segment SGI299858[] = {{&S406[13], 10}, {NULL}};
const Segment SGI299859[] = {{&S406[42], 7}, {NULL}};
const Segment SGI299860[] = {{&S406[35], 14}, {NULL}};
const Segment SGI299861[] = {{&S406[143], 15}, {NULL}};
const Segment SGI299862[] = {{&S406[51], 6}, {NULL}};
const Segment SGI299863[] = {{&S406[71], 3}, {NULL}};
const Segment SGI299864[] = {{&S407[30], 3}, {&S407[106], 3}, {NULL}};
const Segment SGI299865[] = {{&S407[8], 3}, {&S407[84], 3}, {NULL}};
const Segment SGI299866[] = {{&S407[13], 10}, {&S407[13], 10}, {NULL}};
const Segment SGI299867[] = {{&S407[42], 7}, {&S407[42], 7}, {NULL}};
const Segment SGI299868[] = {{&S407[35], 14}, {&S407[35], 14}, {NULL}};
const Segment SGI299869[] = {{&S407[207], 15}, {NULL}};
const Segment SGI299870[] = {{&S407[51], 5}, {NULL}};
const Segment SGI299871[] = {{&S408[30], 3}, {&S408[113], 3}, {&S408[113], 3}, {NULL}};
const Segment SGI299872[] = {{&S408[8], 3}, {&S408[8], 3}, {&S408[8], 3}, {NULL}};
const Segment SGI299873[] = {{&S408[13], 10}, {&S408[13], 10}, {&S408[13], 10}, {NULL}};
const Segment SGI299874[] = {{&S408[42], 7}, {&S408[125], 7}, {&S408[125], 7}, {NULL}};
const Segment SGI299875[] = {{&S408[35], 14}, {&S408[118], 14}, {&S408[118], 14}, {NULL}};
const Segment SGI299876[] = {{&S408[322], 15}, {NULL}};
const Segment SGI299877[] = {{&S408[51], 6}, {NULL}};
const Segment SGI299878[] = {{&S408[134], 3}, {&S408[134], 3}, {NULL}};
const Segment SGI299879[] = {{&S409[30], 3}, {NULL}};
const Segment SGI299880[] = {{&S409[8], 3}, {NULL}};
const Segment SGI299881[] = {{&S409[13], 10}, {NULL}};
const Segment SGI299882[] = {{&S409[42], 7}, {NULL}};
const Segment SGI299883[] = {{&S409[35], 14}, {NULL}};
const Segment SGI299884[] = {{&S409[109], 6}, {NULL}};
const Segment SGI299885[] = {{&S409[84], 3}, {NULL}};
const Segment SGI299886[] = {{&S410[30], 3}, {&S410[30], 3}, {NULL}};
const Segment SGI299887[] = {{&S410[8], 3}, {NULL}};
const Segment SGI299888[] = {{&S410[13], 10}, {NULL}};
const Segment SGI299889[] = {{&S410[42], 7}, {&S410[203], 7}, {NULL}};
const Segment SGI299890[] = {{&S410[35], 14}, {&S410[196], 14}, {NULL}};
const Segment SGI299891[] = {{&S410[146], 15}, {NULL}};
const Segment SGI299892[] = {{&S410[51], 6}, {NULL}};
const Segment SGI299893[] = {{&S411[30], 3}, {NULL}};
const Segment SGI299894[] = {{&S411[8], 3}, {NULL}};
const Segment SGI299895[] = {{&S411[13], 10}, {NULL}};
const Segment SGI299896[] = {{&S411[42], 7}, {NULL}};
const Segment SGI299897[] = {{&S411[35], 14}, {NULL}};
const Segment SGI299898[] = {{&S411[139], 15}, {&S411[139], 15}, {NULL}};
const Segment SGI299899[] = {{&S411[218], 4}, {NULL}};
const Segment SGI299900[] = {{&S412[30], 3}, {NULL}};
const Segment SGI299901[] = {{&S412[8], 3}, {&S412[8], 3}, {&S412[8], 3}, {NULL}};
const Segment SGI299902[] = {{&S412[13], 10}, {NULL}};
const Segment SGI299903[] = {{&S412[42], 7}, {NULL}};
const Segment SGI299904[] = {{&S412[35], 14}, {NULL}};
const Segment SGI299905[] = {{&S413[30], 3}, {&S413[132], 3}, {NULL}};
const Segment SGI299906[] = {{&S413[8], 3}, {&S413[8], 3}, {NULL}};
const Segment SGI299907[] = {{&S413[13], 10}, {NULL}};
const Segment SGI299908[] = {{&S413[42], 7}, {&S413[42], 7}, {&S413[144], 7}, {NULL}};
const Segment SGI299909[] = {{&S413[35], 14}, {&S413[35], 14}, {&S413[137], 14}, {NULL}};
const Segment SGI299910[] = {{&S413[186], 15}, {NULL}};
const Segment SGI299911[] = {{&S413[51], 6}, {NULL}};
const Segment SGI299912[] = {{&S413[153], 3}, {NULL}};
const Segment SGI299913[] = {{&S414[30], 3}, {NULL}};
const Segment SGI299914[] = {{&S414[8], 3}, {NULL}};
const Segment SGI299915[] = {{&S414[13], 10}, {NULL}};
const Segment SGI299916[] = {{&S414[42], 7}, {NULL}};
const Segment SGI299917[] = {{&S414[35], 14}, {NULL}};
const Segment SGI299918[] = {{&S414[138], 15}, {&S414[138], 15}, {NULL}};
const Segment SGI299919[] = {{&S414[51], 6}, {NULL}};
const Segment SGI299920[] = {{&S415[30], 3}, {NULL}};
const Segment SGI299921[] = {{&S415[8], 3}, {NULL}};
const Segment SGI299922[] = {{&S415[13], 10}, {NULL}};
const Segment SGI299923[] = {{&S415[42], 7}, {NULL}};
const Segment SGI299924[] = {{&S415[35], 14}, {NULL}};
const Segment SGI299925[] = {{&S415[136], 15}, {NULL}};
const Segment SGI299926[] = {{&S415[214], 5}, {NULL}};
const Segment SGI299927[] = {{&S415[64], 3}, {NULL}};
const Segment SGI299928[] = {{&S416[72], 14}, {NULL}};
const Segment SGI299929[] = {{&S416[8], 3}, {NULL}};
const Segment SGI299930[] = {{&S416[13], 10}, {NULL}};
const Segment SGI299931[] = {{&S416[61], 4}, {NULL}};
const Segment SGI299932[] = {{&S417[72], 14}, {NULL}};
const Segment SGI299933[] = {{&S417[8], 3}, {NULL}};
const Segment SGI299934[] = {{&S417[13], 10}, {NULL}};
const Segment SGI299935[] = {{&S417[61], 4}, {NULL}};
const Segment SGI299936[] = {{&S418[30], 3}, {NULL}};
const Segment SGI299937[] = {{&S418[42], 7}, {NULL}};
const Segment SGI299938[] = {{&S418[35], 14}, {NULL}};
const Segment SGI299939[] = {{&S418[138], 15}, {NULL}};
const Segment SGI299940[] = {{&S418[51], 6}, {NULL}};
const Segment SGI299941[] = {{&S419[30], 3}, {NULL}};
const Segment SGI299942[] = {{&S419[8], 3}, {NULL}};
const Segment SGI299943[] = {{&S419[13], 10}, {NULL}};
const Segment SGI299944[] = {{&S419[42], 7}, {NULL}};
const Segment SGI299945[] = {{&S419[35], 14}, {NULL}};
const Segment SGI299946[] = {{&S419[139], 15}, {NULL}};
const Segment SGI299947[] = {{&S419[177], 3}, {NULL}};
const Segment SGI299948[] = {{&S420[30], 3}, {NULL}};
const Segment SGI299949[] = {{&S420[8], 3}, {NULL}};
const Segment SGI299950[] = {{&S420[13], 20}, {NULL}};
const Segment SGI299951[] = {{&S420[42], 7}, {NULL}};
const Segment SGI299952[] = {{&S420[35], 14}, {NULL}};
const Segment SGI299953[] = {{&S420[138], 15}, {NULL}};
const Segment SGI299954[] = {{&S420[51], 6}, {NULL}};
const Segment SGI299955[] = {{&S421[30], 3}, {NULL}};
const Segment SGI299956[] = {{&S421[8], 3}, {NULL}};
const Segment SGI299957[] = {{&S421[13], 10}, {NULL}};
const Segment SGI299958[] = {{&S421[42], 7}, {NULL}};
const Segment SGI299959[] = {{&S421[35], 14}, {NULL}};
const Segment SGI299960[] = {{&S421[64], 15}, {&S421[64], 15}, {NULL}};
const Segment SGI299961[] = {{&S421[90], 6}, {NULL}};
const Segment SGI299962[] = {{&S421[85], 3}, {NULL}};
const Segment SGI299963[] = {{&S422[30], 3}, {&S422[106], 3}, {&S422[106], 3}, {NULL}};
const Segment SGI299964[] = {{&S422[8], 3}, {&S422[8], 3}, {&S422[8], 3}, {NULL}};
const Segment SGI299965[] = {{&S422[13], 10}, {&S422[13], 10}, {&S422[13], 10}, {NULL}};
const Segment SGI299966[] = {{&S422[42], 7}, {&S422[118], 7}, {&S422[118], 7}, {NULL}};
const Segment SGI299967[] = {{&S422[35], 14}, {&S422[111], 14}, {&S422[111], 14}, {NULL}};
const Segment SGI299968[] = {{&S422[315], 15}, {NULL}};
const Segment SGI299969[] = {{&S422[336], 9}, {NULL}};
const Segment SGI299970[] = {{&S422[127], 3}, {&S422[127], 3}, {NULL}};
const Segment SGI299971[] = {{&S423[30], 3}, {NULL}};
const Segment SGI299972[] = {{&S423[8], 3}, {NULL}};
const Segment SGI299973[] = {{&S423[13], 10}, {NULL}};
const Segment SGI299974[] = {{&S423[42], 7}, {NULL}};
const Segment SGI299975[] = {{&S423[35], 14}, {NULL}};
const Segment SGI299976[] = {{&S423[131], 15}, {NULL}};
const Segment SGI299977[] = {{&S423[162], 3}, {NULL}};
const Segment SGI299978[] = {{&S424[30], 3}, {NULL}};
const Segment SGI299979[] = {{&S424[8], 3}, {NULL}};
const Segment SGI299980[] = {{&S424[13], 10}, {NULL}};
const Segment SGI299981[] = {{&S424[42], 7}, {NULL}};
const Segment SGI299982[] = {{&S424[35], 14}, {NULL}};
const Segment SGI299983[] = {{&S424[138], 15}, {NULL}};
const Segment SGI299984[] = {{&S424[51], 6}, {NULL}};
const Segment SGI299985[] = {{&S424[169], 3}, {NULL}};
const Segment SGI299986[] = {{&S425[30], 3}, {&S425[101], 3}, {NULL}};
const Segment SGI299987[] = {{&S425[8], 3}, {&S425[79], 3}, {NULL}};
const Segment SGI299988[] = {{&S425[13], 10}, {&S425[84], 10}, {NULL}};
const Segment SGI299989[] = {{&S425[42], 7}, {&S425[42], 3}, {NULL}};
const Segment SGI299990[] = {{&S425[35], 14}, {NULL}};
const Segment SGI299991[] = {{&S425[51], 6}, {NULL}};
const Segment SGI299992[] = {{&S426[30], 3}, {&S426[40], 3}, {NULL}};
const Segment SGI299993[] = {{&S426[8], 3}, {NULL}};
const Segment SGI299994[] = {{&S426[13], 10}, {NULL}};
const Segment SGI299995[] = {{&S426[61], 7}, {NULL}};
const Segment SGI299996[] = {{&S426[54], 14}, {NULL}};
const Segment SGI299997[] = {{&S426[70], 6}, {NULL}};
const Segment SGI299998[] = {{&S427[30], 3}, {&S427[114], 3}, {NULL}};
const Segment SGI299999[] = {{&S427[8], 3}, {&S427[8], 3}, {NULL}};
const Segment SGI300000[] = {{&S427[13], 10}, {&S427[13], 10}, {NULL}};
const Segment SGI300001[] = {{&S427[42], 7}, {&S427[126], 7}, {NULL}};
const Segment SGI300002[] = {{&S427[35], 14}, {&S427[119], 14}, {NULL}};
const Segment SGI300003[] = {{&S427[209], 15}, {NULL}};
const Segment SGI300004[] = {{&S427[240], 4}, {NULL}};
const Segment SGI300005[] = {{&S427[51], 5}, {NULL}};
const Segment SGI300006[] = {{&S427[135], 3}, {NULL}};
const Segment SGI300007[] = {{&S428[30], 3}, {&S428[30], 3}, {NULL}};
const Segment SGI300008[] = {{&S428[8], 3}, {NULL}};
const Segment SGI300009[] = {{&S428[13], 10}, {NULL}};
const Segment SGI300010[] = {{&S428[42], 7}, {NULL}};
const Segment SGI300011[] = {{&S428[35], 14}, {NULL}};
const Segment SGI300012[] = {{&S428[143], 15}, {NULL}};
const Segment SGI300013[] = {{&S428[51], 5}, {NULL}};
const Segment SGI300014[] = {{&S429[30], 3}, {NULL}};
const Segment SGI300015[] = {{&S429[8], 3}, {NULL}};
const Segment SGI300016[] = {{&S429[13], 10}, {NULL}};
const Segment SGI300017[] = {{&S429[42], 7}, {NULL}};
const Segment SGI300018[] = {{&S429[35], 14}, {NULL}};
const Segment SGI300019[] = {{&S429[169], 4}, {NULL}};
const Segment SGI300020[] = {{&S429[51], 6}, {NULL}};
const Segment SGI300021[] = {{&S429[71], 3}, {NULL}};
const Segment SGI300022[] = {{&S430[30], 3}, {NULL}};
const Segment SGI300023[] = {{&S430[8], 3}, {NULL}};
const Segment SGI300024[] = {{&S430[13], 10}, {NULL}};
const Segment SGI300025[] = {{&S430[42], 7}, {NULL}};
const Segment SGI300026[] = {{&S430[35], 14}, {NULL}};
const Segment SGI300027[] = {{&S430[141], 3}, {NULL}};
const Segment SGI300028[] = {{&S431[30], 3}, {NULL}};
const Segment SGI300029[] = {{&S431[8], 3}, {NULL}};
const Segment SGI300030[] = {{&S431[13], 10}, {NULL}};
const Segment SGI300031[] = {{&S431[42], 7}, {NULL}};
const Segment SGI300032[] = {{&S431[35], 14}, {NULL}};
const Segment SGI300033[] = {{&S431[131], 15}, {NULL}};
const Segment SGI300034[] = {{&S431[162], 3}, {NULL}};
const Segment SGI300035[] = {{&S432[30], 3}, {NULL}};
const Segment SGI300036[] = {{&S432[8], 3}, {NULL}};
const Segment SGI300037[] = {{&S432[13], 10}, {NULL}};
const Segment SGI300038[] = {{&S432[42], 7}, {NULL}};
const Segment SGI300039[] = {{&S432[35], 14}, {NULL}};
const Segment SGI300040[] = {{&S432[138], 15}, {NULL}};
const Segment SGI300041[] = {{&S432[51], 6}, {NULL}};
const Segment SGI300042[] = {{&S432[169], 3}, {NULL}};
const Segment SGI300043[] = {{&S433[30], 3}, {NULL}};
const Segment SGI300044[] = {{&S433[8], 3}, {&S433[8], 3}, {NULL}};
const Segment SGI300045[] = {{&S433[13], 10}, {NULL}};
const Segment SGI300046[] = {{&S433[42], 7}, {NULL}};
const Segment SGI300047[] = {{&S433[35], 14}, {NULL}};
const Segment SGI300048[] = {{&S433[151], 15}, {NULL}};
const Segment SGI300049[] = {{&S433[51], 6}, {NULL}};
const Segment SGI300050[] = {{&S434[30], 3}, {&S434[30], 3}, {NULL}};
const Segment SGI300051[] = {{&S434[8], 3}, {NULL}};
const Segment SGI300052[] = {{&S434[13], 10}, {NULL}};
const Segment SGI300053[] = {{&S434[42], 7}, {NULL}};
const Segment SGI300054[] = {{&S434[35], 14}, {NULL}};
const Segment SGI300055[] = {{&S434[131], 15}, {NULL}};
const Segment SGI300056[] = {{&S434[164], 4}, {NULL}};
const Segment SGI300057[] = {{&S435[30], 3}, {&S435[102], 3}, {NULL}};
const Segment SGI300058[] = {{&S435[8], 3}, {&S435[8], 3}, {NULL}};
const Segment SGI300059[] = {{&S435[13], 10}, {&S435[13], 10}, {NULL}};
const Segment SGI300060[] = {{&S435[42], 7}, {&S435[114], 7}, {NULL}};
const Segment SGI300061[] = {{&S435[35], 14}, {&S435[107], 14}, {NULL}};
const Segment SGI300062[] = {{&S435[197], 15}, {&S435[197], 15}, {NULL}};
const Segment SGI300063[] = {{&S435[252], 9}, {NULL}};
const Segment SGI300064[] = {{&S435[123], 3}, {NULL}};
const Segment SGI300065[] = {{&S436[30], 3}, {&S436[106], 3}, {&S436[106], 3}, {NULL}};
const Segment SGI300066[] = {{&S436[8], 3}, {&S436[8], 3}, {&S436[8], 3}, {NULL}};
const Segment SGI300067[] = {{&S436[13], 10}, {&S436[13], 10}, {&S436[13], 10}, {NULL}};
const Segment SGI300068[] = {{&S436[42], 7}, {&S436[118], 7}, {&S436[118], 7}, {NULL}};
const Segment SGI300069[] = {{&S436[35], 14}, {&S436[111], 14}, {&S436[111], 14}, {NULL}};
const Segment SGI300070[] = {{&S436[127], 3}, {&S436[127], 3}, {NULL}};
const Segment SGI300071[] = {{&S437[30], 3}, {NULL}};
const Segment SGI300072[] = {{&S437[8], 3}, {NULL}};
const Segment SGI300073[] = {{&S437[13], 10}, {NULL}};
const Segment SGI300074[] = {{&S437[42], 7}, {NULL}};
const Segment SGI300075[] = {{&S437[35], 14}, {NULL}};
const Segment SGI300076[] = {{&S437[161], 15}, {&S437[161], 15}, {NULL}};
const Segment SGI300077[] = {{&S437[270], 4}, {NULL}};
const Segment SGI300078[] = {{&S437[51], 5}, {NULL}};
const Segment SGI300079[] = {{&S437[255], 9}, {NULL}};
const Segment SGI300080[] = {{&S438[30], 3}, {&S438[87], 3}, {NULL}};
const Segment SGI300081[] = {{&S438[8], 3}, {&S438[154], 3}, {NULL}};
const Segment SGI300082[] = {{&S438[13], 10}, {&S438[13], 10}, {NULL}};
const Segment SGI300083[] = {{&S438[42], 7}, {&S438[221], 2}, {NULL}};
const Segment SGI300084[] = {{&S438[35], 14}, {NULL}};
const Segment SGI300085[] = {{&S438[138], 15}, {NULL}};
const Segment SGI300086[] = {{&S438[51], 6}, {NULL}};
const Segment SGI300087[] = {{&S439[30], 3}, {NULL}};
const Segment SGI300088[] = {{&S439[8], 3}, {NULL}};
const Segment SGI300089[] = {{&S439[13], 10}, {NULL}};
const Segment SGI300090[] = {{&S439[42], 14}, {NULL}};
const Segment SGI300091[] = {{&S439[138], 15}, {NULL}};
const Segment SGI300092[] = {{&S439[204], 9}, {NULL}};
const Segment SGI300093[] = {{&S440[30], 3}, {NULL}};
const Segment SGI300094[] = {{&S440[8], 3}, {NULL}};
const Segment SGI300095[] = {{&S440[13], 10}, {NULL}};
const Segment SGI300096[] = {{&S440[55], 7}, {NULL}};
const Segment SGI300097[] = {{&S440[48], 14}, {NULL}};
const Segment SGI300098[] = {{&S440[64], 3}, {&S440[64], 3}, {NULL}};
const Segment SGI300099[] = {{&S441[30], 3}, {NULL}};
const Segment SGI300100[] = {{&S441[8], 3}, {NULL}};
const Segment SGI300101[] = {{&S441[13], 10}, {NULL}};
const Segment SGI300102[] = {{&S441[42], 7}, {NULL}};
const Segment SGI300103[] = {{&S441[35], 14}, {NULL}};
const Segment SGI300104[] = {{&S441[131], 15}, {NULL}};
const Segment SGI300105[] = {{&S441[203], 4}, {NULL}};
const Segment SGI300106[] = {{&S441[219], 5}, {NULL}};
const Segment SGI300107[] = {{&S442[30], 3}, {NULL}};
const Segment SGI300108[] = {{&S442[8], 3}, {NULL}};
const Segment SGI300109[] = {{&S442[13], 10}, {NULL}};
const Segment SGI300110[] = {{&S442[42], 7}, {&S442[166], 18}, {NULL}};
const Segment SGI300111[] = {{&S442[35], 14}, {NULL}};
const Segment SGI300112[] = {{&S442[138], 15}, {NULL}};
const Segment SGI300113[] = {{&S442[51], 6}, {NULL}};
const Segment SGI300114[] = {{&S443[30], 3}, {&S443[161], 3}, {NULL}};
const Segment SGI300115[] = {{&S443[8], 3}, {NULL}};
const Segment SGI300116[] = {{&S443[13], 10}, {NULL}};
const Segment SGI300117[] = {{&S443[42], 7}, {NULL}};
const Segment SGI300118[] = {{&S443[35], 14}, {NULL}};
const Segment SGI300119[] = {{&S443[131], 9}, {NULL}};
const Segment SGI300120[] = {{&S444[30], 3}, {NULL}};
const Segment SGI300121[] = {{&S444[8], 3}, {NULL}};
const Segment SGI300122[] = {{&S444[13], 10}, {NULL}};
const Segment SGI300123[] = {{&S444[42], 7}, {NULL}};
const Segment SGI300124[] = {{&S444[35], 14}, {NULL}};
const Segment SGI300125[] = {{&S444[178], 4}, {NULL}};
const Segment SGI300126[] = {{&S444[51], 5}, {NULL}};
const Segment SGI300127[] = {{&S444[163], 9}, {NULL}};
const Segment SGI300128[] = {{&S445[30], 3}, {NULL}};
const Segment SGI300129[] = {{&S445[8], 3}, {NULL}};
const Segment SGI300130[] = {{&S445[13], 10}, {NULL}};
const Segment SGI300131[] = {{&S445[42], 7}, {NULL}};
const Segment SGI300132[] = {{&S445[35], 14}, {NULL}};
const Segment SGI300133[] = {{&S445[181], 15}, {NULL}};
const Segment SGI300134[] = {{&S445[64], 3}, {&S445[64], 3}, {NULL}};
const Segment SGI300135[] = {{&S446[49], 3}, {NULL}};
const Segment SGI300136[] = {{&S446[8], 3}, {NULL}};
const Segment SGI300137[] = {{&S446[13], 29}, {NULL}};
const Segment SGI300138[] = {{&S446[61], 7}, {NULL}};
const Segment SGI300139[] = {{&S446[54], 14}, {NULL}};
const Segment SGI300140[] = {{&S446[150], 15}, {NULL}};
const Segment SGI300141[] = {{&S447[30], 3}, {&S447[113], 3}, {NULL}};
const Segment SGI300142[] = {{&S447[8], 3}, {&S447[8], 3}, {NULL}};
const Segment SGI300143[] = {{&S447[13], 10}, {NULL}};
const Segment SGI300144[] = {{&S447[42], 7}, {&S447[42], 7}, {&S447[125], 7}, {NULL}};
const Segment SGI300145[] = {{&S447[35], 14}, {&S447[35], 14}, {&S447[118], 14}, {NULL}};
const Segment SGI300146[] = {{&S447[167], 15}, {NULL}};
const Segment SGI300147[] = {{&S447[134], 3}, {NULL}};
const Segment SGI300148[] = {{&S449[30], 3}, {NULL}};
const Segment SGI300149[] = {{&S449[8], 3}, {NULL}};
const Segment SGI300150[] = {{&S449[13], 10}, {NULL}};
const Segment SGI300151[] = {{&S449[42], 7}, {NULL}};
const Segment SGI300152[] = {{&S449[35], 14}, {NULL}};
const Segment SGI300153[] = {{&S449[131], 15}, {NULL}};
const Segment SGI300154[] = {{&S449[152], 9}, {NULL}};
const Segment SGI300155[] = {{&S449[177], 3}, {NULL}};
const Segment SGI300156[] = {{&S450[8], 3}, {NULL}};
const Segment SGI300157[] = {{&S450[51], 6}, {NULL}};
const Segment SGI300158[] = {{&S451[30], 3}, {NULL}};
const Segment SGI300159[] = {{&S451[8], 3}, {NULL}};
const Segment SGI300160[] = {{&S451[13], 10}, {NULL}};
const Segment SGI300161[] = {{&S451[42], 7}, {NULL}};
const Segment SGI300162[] = {{&S451[35], 14}, {NULL}};
const Segment SGI300163[] = {{&S451[183], 15}, {&S451[183], 15}, {NULL}};
const Segment SGI300164[] = {{&S451[349], 4}, {NULL}};
const Segment SGI300165[] = {{&S451[334], 9}, {NULL}};
const Segment SGI300166[] = {{&S452[30], 3}, {NULL}};
const Segment SGI300167[] = {{&S452[8], 3}, {NULL}};
const Segment SGI300168[] = {{&S452[13], 10}, {NULL}};
const Segment SGI300169[] = {{&S452[42], 7}, {NULL}};
const Segment SGI300170[] = {{&S452[35], 14}, {NULL}};
const Segment SGI300171[] = {{&S453[30], 3}, {NULL}};
const Segment SGI300172[] = {{&S453[8], 3}, {NULL}};
const Segment SGI300173[] = {{&S453[13], 10}, {NULL}};
const Segment SGI300174[] = {{&S453[42], 7}, {NULL}};
const Segment SGI300175[] = {{&S453[35], 14}, {NULL}};
const Segment SGI300176[] = {{&S453[138], 15}, {NULL}};
const Segment SGI300177[] = {{&S453[51], 6}, {NULL}};
const Segment SGI300178[] = {{&S454[30], 3}, {&S454[106], 3}, {NULL}};
const Segment SGI300179[] = {{&S454[8], 3}, {&S454[8], 3}, {NULL}};
const Segment SGI300180[] = {{&S454[13], 10}, {&S454[13], 10}, {NULL}};
const Segment SGI300181[] = {{&S454[42], 7}, {&S454[118], 7}, {NULL}};
const Segment SGI300182[] = {{&S454[35], 14}, {&S454[111], 14}, {NULL}};
const Segment SGI300183[] = {{&S454[201], 15}, {&S454[201], 15}, {NULL}};
const Segment SGI300184[] = {{&S454[51], 5}, {NULL}};
const Segment SGI300185[] = {{&S454[127], 3}, {NULL}};
const Segment SGI300186[] = {{&S455[30], 3}, {NULL}};
const Segment SGI300187[] = {{&S455[8], 3}, {NULL}};
const Segment SGI300188[] = {{&S455[13], 10}, {NULL}};
const Segment SGI300189[] = {{&S455[55], 7}, {NULL}};
const Segment SGI300190[] = {{&S455[48], 14}, {NULL}};
const Segment SGI300191[] = {{&S455[64], 3}, {&S455[64], 3}, {NULL}};
const Segment SGI300192[] = {{&S456[30], 3}, {&S456[106], 3}, {NULL}};
const Segment SGI300193[] = {{&S456[8], 3}, {&S456[84], 3}, {NULL}};
const Segment SGI300194[] = {{&S456[13], 10}, {&S456[89], 10}, {NULL}};
const Segment SGI300195[] = {{&S456[42], 7}, {&S456[42], 3}, {NULL}};
const Segment SGI300196[] = {{&S456[35], 14}, {NULL}};
const Segment SGI300197[] = {{&S456[51], 5}, {NULL}};
const Segment SGI300198[] = {{&S457[30], 3}, {&S457[120], 3}, {NULL}};
const Segment SGI300199[] = {{&S457[8], 3}, {&S457[8], 3}, {NULL}};
const Segment SGI300200[] = {{&S457[13], 10}, {&S457[13], 10}, {NULL}};
const Segment SGI300201[] = {{&S457[42], 7}, {&S457[132], 7}, {NULL}};
const Segment SGI300202[] = {{&S457[35], 14}, {&S457[125], 14}, {NULL}};
const Segment SGI300203[] = {{&S457[51], 6}, {NULL}};
const Segment SGI300204[] = {{&S457[71], 3}, {&S457[71], 3}, {NULL}};
const Segment SGI300205[] = {{&S458[30], 3}, {NULL}};
const Segment SGI300206[] = {{&S458[8], 3}, {NULL}};
const Segment SGI300207[] = {{&S458[13], 10}, {NULL}};
const Segment SGI300208[] = {{&S458[42], 7}, {NULL}};
const Segment SGI300209[] = {{&S458[35], 14}, {NULL}};
const Segment SGI300210[] = {{&S458[143], 15}, {NULL}};
const Segment SGI300211[] = {{&S458[209], 4}, {NULL}};
const Segment SGI300212[] = {{&S458[51], 6}, {NULL}};
const Segment SGI300213[] = {{&S458[71], 3}, {NULL}};
const Segment SGI300214[] = {{&S459[30], 3}, {&S459[30], 3}, {NULL}};
const Segment SGI300215[] = {{&S459[8], 3}, {NULL}};
const Segment SGI300216[] = {{&S459[13], 10}, {NULL}};
const Segment SGI300217[] = {{&S459[42], 7}, {NULL}};
const Segment SGI300218[] = {{&S459[35], 14}, {NULL}};
const Segment SGI300219[] = {{&S459[136], 15}, {NULL}};
const Segment SGI300220[] = {{&S459[169], 9}, {NULL}};
const Segment SGI300221[] = {{&S459[64], 3}, {NULL}};
const Segment SGI300222[] = {{&S460[30], 3}, {&S460[30], 3}, {NULL}};
const Segment SGI300223[] = {{&S460[8], 3}, {NULL}};
const Segment SGI300224[] = {{&S460[13], 10}, {NULL}};
const Segment SGI300225[] = {{&S460[42], 7}, {NULL}};
const Segment SGI300226[] = {{&S460[35], 14}, {NULL}};
const Segment SGI300227[] = {{&S460[143], 15}, {NULL}};
const Segment SGI300228[] = {{&S460[51], 5}, {NULL}};
const Segment SGI300229[] = {{&S460[184], 9}, {NULL}};
const Segment SGI300230[] = {{&S461[30], 3}, {&S461[101], 3}, {NULL}};
const Segment SGI300231[] = {{&S461[8], 3}, {&S461[8], 3}, {&S461[249], 3}, {NULL}};
const Segment SGI300232[] = {{&S461[13], 10}, {&S461[13], 10}, {NULL}};
const Segment SGI300233[] = {{&S461[42], 7}, {&S461[113], 7}, {NULL}};
const Segment SGI300234[] = {{&S461[35], 14}, {&S461[106], 14}, {NULL}};
const Segment SGI300235[] = {{&S461[51], 6}, {NULL}};
const Segment SGI300236[] = {{&S461[122], 3}, {NULL}};
const Segment SGI300237[] = {{&S462[30], 3}, {&S462[101], 3}, {&S462[30], 3}, {NULL}};
const Segment SGI300238[] = {{&S462[8], 3}, {&S462[79], 3}, {&S462[79], 3}, {NULL}};
const Segment SGI300239[] = {{&S462[13], 10}, {&S462[84], 10}, {&S462[84], 10}, {NULL}};
const Segment SGI300240[] = {{&S462[42], 7}, {&S462[42], 3}, {&S462[42], 3}, {NULL}};
const Segment SGI300241[] = {{&S462[35], 14}, {NULL}};
const Segment SGI300242[] = {{&S462[51], 6}, {NULL}};
const Segment SGI300243[] = {{&S464[30], 3}, {NULL}};
const Segment SGI300244[] = {{&S464[8], 3}, {NULL}};
const Segment SGI300245[] = {{&S464[13], 10}, {NULL}};
const Segment SGI300246[] = {{&S464[56], 7}, {NULL}};
const Segment SGI300247[] = {{&S464[49], 14}, {NULL}};
const Segment SGI300248[] = {{&S464[65], 6}, {NULL}};
const Segment SGI300249[] = {{&S465[30], 3}, {NULL}};
const Segment SGI300250[] = {{&S465[8], 3}, {NULL}};
const Segment SGI300251[] = {{&S465[13], 10}, {NULL}};
const Segment SGI300252[] = {{&S465[42], 7}, {NULL}};
const Segment SGI300253[] = {{&S465[35], 14}, {NULL}};
const Segment SGI300254[] = {{&S465[51], 5}, {NULL}};
const Segment SGI300255[] = {{&S466[30], 3}, {NULL}};
const Segment SGI300256[] = {{&S466[8], 3}, {&S466[8], 3}, {NULL}};
const Segment SGI300257[] = {{&S466[13], 10}, {&S466[13], 10}, {NULL}};
const Segment SGI300258[] = {{&S466[42], 7}, {&S466[96], 7}, {NULL}};
const Segment SGI300259[] = {{&S466[35], 14}, {&S466[89], 14}, {NULL}};
const Segment SGI300260[] = {{&S466[105], 3}, {NULL}};
const Segment SGI300261[] = {{&S467[30], 3}, {NULL}};
const Segment SGI300262[] = {{&S467[8], 3}, {NULL}};
const Segment SGI300263[] = {{&S467[13], 10}, {NULL}};
const Segment SGI300264[] = {{&S467[42], 7}, {NULL}};
const Segment SGI300265[] = {{&S467[35], 14}, {NULL}};
const Segment SGI300266[] = {{&S467[51], 5}, {NULL}};
const Segment SGI300267[] = {{&S468[30], 3}, {NULL}};
const Segment SGI300268[] = {{&S468[8], 3}, {NULL}};
const Segment SGI300269[] = {{&S468[13], 10}, {NULL}};
const Segment SGI300270[] = {{&S468[78], 7}, {NULL}};
const Segment SGI300271[] = {{&S468[71], 14}, {NULL}};
const Segment SGI300272[] = {{&S468[125], 15}, {&S468[125], 15}, {NULL}};
const Segment SGI300273[] = {{&S469[30], 3}, {NULL}};
const Segment SGI300274[] = {{&S469[8], 3}, {NULL}};
const Segment SGI300275[] = {{&S469[13], 10}, {NULL}};
const Segment SGI300276[] = {{&S469[42], 7}, {NULL}};
const Segment SGI300277[] = {{&S470[30], 3}, {NULL}};
const Segment SGI300278[] = {{&S470[8], 3}, {NULL}};
const Segment SGI300279[] = {{&S470[13], 10}, {NULL}};
const Segment SGI300280[] = {{&S470[42], 7}, {NULL}};
const Segment SGI300281[] = {{&S470[35], 14}, {NULL}};
const Segment SGI300282[] = {{&S471[30], 3}, {NULL}};
const Segment SGI300283[] = {{&S471[8], 3}, {NULL}};
const Segment SGI300284[] = {{&S471[13], 10}, {NULL}};
const Segment SGI300285[] = {{&S471[42], 7}, {NULL}};
const Segment SGI300286[] = {{&S471[35], 14}, {NULL}};
const Segment SGI300287[] = {{&S472[30], 3}, {NULL}};
const Segment SGI300288[] = {{&S472[8], 3}, {NULL}};
const Segment SGI300289[] = {{&S472[13], 10}, {NULL}};
const Segment SGI300290[] = {{&S472[44], 7}, {NULL}};
const Segment SGI300291[] = {{&S472[37], 14}, {NULL}};
const Segment SGI300292[] = {{&S472[53], 3}, {NULL}};
const Segment SGI300293[] = {{&S473[30], 3}, {NULL}};
const Segment SGI300294[] = {{&S473[8], 3}, {NULL}};
const Segment SGI300295[] = {{&S473[13], 10}, {NULL}};
const Segment SGI300296[] = {{&S473[42], 7}, {NULL}};
const Segment SGI300297[] = {{&S473[35], 14}, {NULL}};
const Segment SGI300298[] = {{&S473[51], 6}, {NULL}};
const Segment SGI300299[] = {{&S474[30], 3}, {NULL}};
const Segment SGI300300[] = {{&S474[8], 3}, {NULL}};
const Segment SGI300301[] = {{&S474[13], 10}, {NULL}};
const Segment SGI300302[] = {{&S474[42], 7}, {NULL}};
const Segment SGI300303[] = {{&S474[35], 14}, {NULL}};
const Segment SGI300304[] = {{&S474[52], 3}, {NULL}};
const Segment SGI300305[] = {{&S475[30], 3}, {NULL}};
const Segment SGI300306[] = {{&S475[8], 3}, {NULL}};
const Segment SGI300307[] = {{&S475[13], 10}, {NULL}};
const Segment SGI300308[] = {{&S475[42], 7}, {NULL}};
const Segment SGI300309[] = {{&S475[35], 14}, {NULL}};
const Segment SGI300310[] = {{&S475[52], 3}, {NULL}};
const Segment SGI300311[] = {{&S476[31], 3}, {NULL}};
const Segment SGI300312[] = {{&S476[8], 3}, {&S476[8], 3}, {NULL}};
const Segment SGI300313[] = {{&S476[0], 11}, {NULL}};
const Segment SGI300314[] = {{&S476[43], 7}, {NULL}};
const Segment SGI300315[] = {{&S476[36], 14}, {NULL}};
const Segment SGI300316[] = {{&S476[57], 3}, {NULL}};
const Segment SGI300317[] = {{&S477[30], 3}, {&S477[81], 3}, {NULL}};
const Segment SGI300318[] = {{&S477[8], 3}, {&S477[8], 3}, {NULL}};
const Segment SGI300319[] = {{&S477[13], 10}, {&S477[13], 10}, {NULL}};
const Segment SGI300320[] = {{&S477[42], 7}, {&S477[42], 7}, {NULL}};
const Segment SGI300321[] = {{&S477[35], 14}, {&S477[35], 14}, {NULL}};
const Segment SGI300322[] = {{&S477[102], 3}, {NULL}};
const Segment SGI300323[] = {{&S478[27], 3}, {NULL}};
const Segment SGI300324[] = {{&S478[8], 3}, {NULL}};
const Segment SGI300325[] = {{&S478[13], 7}, {NULL}};
const Segment SGI300326[] = {{&S478[39], 7}, {NULL}};
const Segment SGI300327[] = {{&S478[32], 14}, {NULL}};
const Segment SGI300328[] = {{&S478[48], 3}, {NULL}};
const Segment SGI300329[] = {{&S479[30], 3}, {&S479[94], 3}, {NULL}};
const Segment SGI300330[] = {{&S479[8], 3}, {&S479[8], 3}, {NULL}};
const Segment SGI300331[] = {{&S479[13], 10}, {&S479[13], 10}, {NULL}};
const Segment SGI300332[] = {{&S479[42], 7}, {&S479[42], 7}, {NULL}};
const Segment SGI300333[] = {{&S479[35], 14}, {&S479[35], 14}, {NULL}};
const Segment SGI300334[] = {{&S479[160], 15}, {NULL}};
const Segment SGI300335[] = {{&S479[115], 3}, {NULL}};
const Segment SGI300336[] = {{&S480[30], 3}, {&S480[94], 3}, {NULL}};
const Segment SGI300337[] = {{&S480[8], 3}, {&S480[8], 3}, {NULL}};
const Segment SGI300338[] = {{&S480[13], 10}, {&S480[13], 10}, {NULL}};
const Segment SGI300339[] = {{&S480[42], 7}, {&S480[106], 7}, {NULL}};
const Segment SGI300340[] = {{&S480[35], 14}, {&S480[99], 14}, {NULL}};
const Segment SGI300341[] = {{&S480[115], 3}, {NULL}};
const Segment SGI300342[] = {{&S481[31], 3}, {NULL}};
const Segment SGI300343[] = {{&S481[43], 7}, {NULL}};
const Segment SGI300344[] = {{&S482[30], 3}, {NULL}};
const Segment SGI300345[] = {{&S482[8], 3}, {NULL}};
const Segment SGI300346[] = {{&S482[13], 10}, {NULL}};
const Segment SGI300347[] = {{&S482[42], 7}, {NULL}};
const Segment SGI300348[] = {{&S482[35], 14}, {NULL}};
const Segment SGI300349[] = {{&S482[51], 6}, {NULL}};
const Segment SGI300350[] = {{&S482[58], 3}, {NULL}};
const Segment SGI300351[] = {{&S483[30], 3}, {NULL}};
const Segment SGI300352[] = {{&S483[8], 3}, {NULL}};
const Segment SGI300353[] = {{&S483[13], 10}, {NULL}};
const Segment SGI300354[] = {{&S483[42], 7}, {NULL}};
const Segment SGI300355[] = {{&S484[30], 3}, {NULL}};
const Segment SGI300356[] = {{&S484[8], 3}, {NULL}};
const Segment SGI300357[] = {{&S484[13], 10}, {NULL}};
const Segment SGI300358[] = {{&S484[42], 7}, {NULL}};
const Segment SGI300359[] = {{&S484[35], 14}, {NULL}};
const Segment SGI300360[] = {{&S484[89], 15}, {NULL}};
const Segment SGI300361[] = {{&S485[30], 3}, {NULL}};
const Segment SGI300362[] = {{&S485[8], 3}, {NULL}};
const Segment SGI300363[] = {{&S485[13], 10}, {NULL}};
const Segment SGI300364[] = {{&S485[42], 7}, {NULL}};
const Segment SGI300365[] = {{&S485[35], 14}, {NULL}};
const Segment SGI300366[] = {{&S485[51], 6}, {NULL}};
const Segment SGI300367[] = {{&S486[31], 3}, {NULL}};
const Segment SGI300368[] = {{&S486[8], 3}, {NULL}};
const Segment SGI300369[] = {{&S486[13], 11}, {NULL}};
const Segment SGI300370[] = {{&S486[43], 7}, {NULL}};
const Segment SGI300371[] = {{&S487[30], 3}, {&S487[106], 3}, {&S487[106], 3}, {NULL}};
const Segment SGI300372[] = {{&S487[8], 3}, {&S487[8], 3}, {&S487[8], 3}, {NULL}};
const Segment SGI300373[] = {{&S487[13], 10}, {&S487[13], 10}, {&S487[13], 10}, {NULL}};
const Segment SGI300374[] = {{&S487[42], 7}, {&S487[42], 7}, {&S487[42], 7}, {NULL}};
const Segment SGI300375[] = {{&S487[35], 14}, {&S487[35], 14}, {&S487[35], 14}, {NULL}};
const Segment SGI300376[] = {{&S487[127], 3}, {&S487[127], 3}, {NULL}};
const Segment SGI300377[] = {{&S488[30], 3}, {&S488[81], 3}, {NULL}};
const Segment SGI300378[] = {{&S488[8], 3}, {&S488[8], 3}, {NULL}};
const Segment SGI300379[] = {{&S488[13], 10}, {&S488[13], 10}, {NULL}};
const Segment SGI300380[] = {{&S488[42], 7}, {&S488[93], 7}, {NULL}};
const Segment SGI300381[] = {{&S488[35], 14}, {NULL}};
const Segment SGI300382[] = {{&S488[112], 9}, {NULL}};
const Segment SGI300383[] = {{&S489[30], 3}, {NULL}};
const Segment SGI300384[] = {{&S489[8], 3}, {NULL}};
const Segment SGI300385[] = {{&S489[13], 10}, {NULL}};
const Segment SGI300386[] = {{&S489[42], 7}, {NULL}};
const Segment SGI300387[] = {{&S489[35], 14}, {NULL}};
const Segment SGI300388[] = {{&S489[51], 9}, {NULL}};
const Segment SGI300389[] = {{&S490[31], 3}, {NULL}};
const Segment SGI300390[] = {{&S490[8], 3}, {NULL}};
const Segment SGI300391[] = {{&S490[13], 11}, {NULL}};
const Segment SGI300392[] = {{&S490[43], 7}, {NULL}};
const Segment SGI300393[] = {{&S490[36], 14}, {NULL}};
const Segment SGI300394[] = {{&S491[30], 3}, {NULL}};
const Segment SGI300395[] = {{&S491[8], 3}, {NULL}};
const Segment SGI300396[] = {{&S491[13], 10}, {NULL}};
const Segment SGI300397[] = {{&S491[42], 7}, {NULL}};
const Segment SGI300398[] = {{&S491[35], 14}, {NULL}};
const Segment SGI300399[] = {{&S491[102], 15}, {NULL}};
const Segment SGI300400[] = {{&S491[187], 4}, {NULL}};
const Segment SGI300401[] = {{&S492[30], 3}, {NULL}};
const Segment SGI300402[] = {{&S492[8], 3}, {NULL}};
const Segment SGI300403[] = {{&S492[13], 10}, {NULL}};
const Segment SGI300404[] = {{&S492[42], 7}, {NULL}};
const Segment SGI300405[] = {{&S492[35], 14}, {NULL}};
const Segment SGI300406[] = {{&S492[147], 4}, {NULL}};
const Segment SGI300407[] = {{&S493[30], 3}, {&S493[81], 3}, {NULL}};
const Segment SGI300408[] = {{&S493[8], 3}, {&S493[8], 3}, {NULL}};
const Segment SGI300409[] = {{&S493[13], 10}, {&S493[13], 10}, {NULL}};
const Segment SGI300410[] = {{&S493[42], 7}, {&S493[93], 7}, {NULL}};
const Segment SGI300411[] = {{&S493[35], 14}, {NULL}};
const Segment SGI300412[] = {{&S494[30], 3}, {&S494[30], 3}, {&S494[30], 3}, {NULL}};
const Segment SGI300413[] = {{&S494[8], 3}, {NULL}};
const Segment SGI300414[] = {{&S494[13], 10}, {NULL}};
const Segment SGI300415[] = {{&S494[42], 7}, {NULL}};
const Segment SGI300416[] = {{&S494[35], 14}, {NULL}};
const Segment SGI300417[] = {{&S495[30], 3}, {&S495[56], 3}, {NULL}};
const Segment SGI300418[] = {{&S495[8], 3}, {NULL}};
const Segment SGI300419[] = {{&S495[13], 10}, {NULL}};
const Segment SGI300420[] = {{&S495[42], 7}, {NULL}};
const Segment SGI300421[] = {{&S495[35], 14}, {NULL}};
const Segment SGI300422[] = {{&S496[30], 3}, {NULL}};
const Segment SGI300423[] = {{&S496[8], 3}, {NULL}};
const Segment SGI300424[] = {{&S496[13], 10}, {NULL}};
const Segment SGI300425[] = {{&S496[42], 4}, {NULL}};
const Segment SGI300426[] = {{&S497[30], 3}, {NULL}};
const Segment SGI300427[] = {{&S497[8], 3}, {NULL}};
const Segment SGI300428[] = {{&S497[13], 10}, {NULL}};
const Segment SGI300429[] = {{&S497[42], 7}, {&S497[85], 5}, {&S497[85], 5}, {NULL}};
const Segment SGI300430[] = {{&S497[35], 14}, {NULL}};
const Segment SGI300431[] = {{&S498[30], 3}, {NULL}};
const Segment SGI300432[] = {{&S498[8], 3}, {NULL}};
const Segment SGI300433[] = {{&S498[13], 10}, {NULL}};
const Segment SGI300434[] = {{&S498[42], 7}, {NULL}};
const Segment SGI300435[] = {{&S498[35], 14}, {NULL}};
const Segment SGI300436[] = {{&S498[56], 9}, {NULL}};
const Segment SGI300437[] = {{&S498[51], 3}, {NULL}};
const Segment SGI300438[] = {{&S499[30], 3}, {NULL}};
const Segment SGI300439[] = {{&S499[8], 3}, {NULL}};
const Segment SGI300440[] = {{&S499[13], 7}, {NULL}};
const Segment SGI300441[] = {{&S499[42], 7}, {NULL}};
const Segment SGI300442[] = {{&S500[30], 3}, {&S500[80], 3}, {NULL}};
const Segment SGI300443[] = {{&S500[8], 3}, {&S500[8], 3}, {NULL}};
const Segment SGI300444[] = {{&S500[13], 10}, {&S500[13], 10}, {NULL}};
const Segment SGI300445[] = {{&S500[42], 7}, {&S500[42], 3}, {NULL}};
const Segment SGI300446[] = {{&S500[35], 14}, {NULL}};
const Segment SGI300447[] = {{&S501[30], 3}, {NULL}};
const Segment SGI300448[] = {{&S501[8], 3}, {NULL}};
const Segment SGI300449[] = {{&S501[13], 10}, {NULL}};
const Segment SGI300450[] = {{&S501[42], 7}, {NULL}};
const Segment SGI300451[] = {{&S501[35], 14}, {NULL}};
const Segment SGI300452[] = {{&S502[30], 3}, {NULL}};
const Segment SGI300453[] = {{&S502[8], 3}, {NULL}};
const Segment SGI300454[] = {{&S502[13], 10}, {NULL}};
const Segment SGI300455[] = {{&S502[42], 7}, {NULL}};
const Segment SGI300456[] = {{&S502[35], 14}, {NULL}};
const Segment SGI300457[] = {{&S502[94], 15}, {NULL}};
const Segment SGI300458[] = {{&S502[51], 3}, {NULL}};
const Segment SGI300459[] = {{&S503[30], 3}, {NULL}};
const Segment SGI300460[] = {{&S503[8], 3}, {NULL}};
const Segment SGI300461[] = {{&S503[13], 10}, {NULL}};
const Segment SGI300462[] = {{&S503[42], 8}, {NULL}};
const Segment SGI300463[] = {{&S504[30], 3}, {NULL}};
const Segment SGI300464[] = {{&S504[8], 3}, {NULL}};
const Segment SGI300465[] = {{&S504[13], 10}, {NULL}};
const Segment SGI300466[] = {{&S504[42], 7}, {NULL}};
const Segment SGI300467[] = {{&S504[35], 14}, {NULL}};
const Segment SGI300468[] = {{&S504[153], 5}, {&S504[153], 5}, {NULL}};
const Segment SGI300469[] = {{&S505[55], 3}, {NULL}};
const Segment SGI300470[] = {{&S505[8], 3}, {&S505[8], 3}, {NULL}};
const Segment SGI300471[] = {{&S505[13], 10}, {&S505[13], 10}, {NULL}};
const Segment SGI300472[] = {{&S505[67], 7}, {&S505[67], 7}, {NULL}};
const Segment SGI300473[] = {{&S505[60], 14}, {&S505[60], 14}, {NULL}};
const Segment SGI300474[] = {{&S506[30], 3}, {NULL}};
const Segment SGI300475[] = {{&S506[8], 3}, {NULL}};
const Segment SGI300476[] = {{&S506[13], 10}, {NULL}};
const Segment SGI300477[] = {{&S506[42], 7}, {NULL}};
const Segment SGI300478[] = {{&S506[35], 14}, {NULL}};
const Segment SGI300479[] = {{&S507[30], 3}, {NULL}};
const Segment SGI300480[] = {{&S507[8], 3}, {NULL}};
const Segment SGI300481[] = {{&S507[13], 10}, {NULL}};
const Segment SGI300482[] = {{&S507[42], 7}, {&S507[75], 16}, {NULL}};
const Segment SGI300483[] = {{&S507[35], 14}, {NULL}};
const Segment SGI300484[] = {{&S508[30], 3}, {NULL}};
const Segment SGI300485[] = {{&S508[8], 3}, {&S508[8], 3}, {&S508[8], 3}, {NULL}};
const Segment SGI300486[] = {{&S508[13], 10}, {NULL}};
const Segment SGI300487[] = {{&S508[42], 7}, {NULL}};
const Segment SGI300488[] = {{&S508[35], 14}, {NULL}};
const Segment SGI300489[] = {{&S509[30], 3}, {&S509[30], 3}, {NULL}};
const Segment SGI300490[] = {{&S509[8], 3}, {&S509[8], 3}, {NULL}};
const Segment SGI300491[] = {{&S509[13], 10}, {&S509[13], 10}, {NULL}};
const Segment SGI300492[] = {{&S509[42], 7}, {NULL}};
const Segment SGI300493[] = {{&S509[35], 14}, {NULL}};
const Segment SGI300494[] = {{&S509[99], 5}, {NULL}};
const Segment SGI300495[] = {{&S510[30], 3}, {&S510[81], 3}, {NULL}};
const Segment SGI300496[] = {{&S510[8], 3}, {&S510[8], 3}, {NULL}};
const Segment SGI300497[] = {{&S510[13], 10}, {&S510[13], 10}, {NULL}};
const Segment SGI300498[] = {{&S510[42], 7}, {&S510[42], 7}, {NULL}};
const Segment SGI300499[] = {{&S510[35], 14}, {NULL}};
const Segment SGI300500[] = {{&S511[30], 3}, {NULL}};
const Segment SGI300501[] = {{&S511[8], 3}, {NULL}};
const Segment SGI300502[] = {{&S511[13], 10}, {NULL}};
const Segment SGI300503[] = {{&S511[42], 7}, {NULL}};
const Segment SGI300504[] = {{&S512[30], 3}, {&S512[87], 3}, {&S512[87], 3}, {NULL}};
const Segment SGI300505[] = {{&S512[8], 3}, {NULL}};
const Segment SGI300506[] = {{&S512[13], 10}, {NULL}};
const Segment SGI300507[] = {{&S512[42], 7}, {NULL}};
const Segment SGI300508[] = {{&S512[35], 14}, {NULL}};
const Segment SGI300509[] = {{&S513[30], 3}, {NULL}};
const Segment SGI300510[] = {{&S513[8], 3}, {NULL}};
const Segment SGI300511[] = {{&S513[13], 10}, {NULL}};
const Segment SGI300512[] = {{&S513[42], 7}, {NULL}};
const Segment SGI300513[] = {{&S513[35], 14}, {NULL}};
const Segment SGI300514[] = {{&S513[51], 3}, {NULL}};
const Segment SGI300515[] = {{&S514[30], 3}, {&S514[30], 3}, {NULL}};
const Segment SGI300516[] = {{&S514[8], 3}, {&S514[8], 3}, {NULL}};
const Segment SGI300517[] = {{&S514[13], 10}, {&S514[13], 10}, {NULL}};
const Segment SGI300518[] = {{&S514[42], 7}, {NULL}};
const Segment SGI300519[] = {{&S514[35], 14}, {NULL}};
const Segment SGI300520[] = {{&S515[30], 3}, {&S515[164], 3}, {NULL}};
const Segment SGI300521[] = {{&S515[8], 3}, {&S515[8], 3}, {NULL}};
const Segment SGI300522[] = {{&S515[13], 10}, {&S515[13], 10}, {NULL}};
const Segment SGI300523[] = {{&S515[42], 7}, {&S515[176], 7}, {NULL}};
const Segment SGI300524[] = {{&S515[35], 14}, {NULL}};
const Segment SGI300525[] = {{&S516[30], 3}, {&S516[30], 3}, {NULL}};
const Segment SGI300526[] = {{&S516[8], 3}, {&S516[8], 3}, {NULL}};
const Segment SGI300527[] = {{&S516[13], 10}, {&S516[13], 10}, {NULL}};
const Segment SGI300528[] = {{&S516[42], 7}, {&S516[42], 7}, {NULL}};
const Segment SGI300529[] = {{&S516[35], 14}, {NULL}};
const Segment SGI300530[] = {{&S517[30], 3}, {&S517[75], 3}, {NULL}};
const Segment SGI300531[] = {{&S517[8], 3}, {NULL}};
const Segment SGI300532[] = {{&S517[13], 10}, {NULL}};
const Segment SGI300533[] = {{&S517[42], 7}, {NULL}};
const Segment SGI300534[] = {{&S517[35], 14}, {NULL}};
const Segment SGI300535[] = {{&S519[30], 3}, {NULL}};
const Segment SGI300536[] = {{&S519[8], 3}, {NULL}};
const Segment SGI300537[] = {{&S519[13], 10}, {NULL}};
const Segment SGI300538[] = {{&S519[42], 3}, {NULL}};
const Segment SGI300539[] = {{&S520[30], 3}, {&S520[30], 3}, {NULL}};
const Segment SGI300540[] = {{&S520[8], 3}, {NULL}};
const Segment SGI300541[] = {{&S520[13], 10}, {NULL}};
const Segment SGI300542[] = {{&S520[42], 7}, {NULL}};
const Segment SGI300543[] = {{&S520[35], 14}, {NULL}};
const Segment SGI300544[] = {{&S521[29], 3}, {NULL}};
const Segment SGI300545[] = {{&S521[8], 3}, {NULL}};
const Segment SGI300546[] = {{&S521[13], 19}, {NULL}};
const Segment SGI300547[] = {{&S521[41], 7}, {NULL}};
const Segment SGI300548[] = {{&S521[34], 14}, {NULL}};
const Segment SGI300549[] = {{&S522[30], 3}, {NULL}};
const Segment SGI300550[] = {{&S522[8], 3}, {NULL}};
const Segment SGI300551[] = {{&S522[13], 10}, {NULL}};
const Segment SGI300552[] = {{&S522[42], 7}, {NULL}};
const Segment SGI300553[] = {{&S522[35], 14}, {NULL}};
const Segment SGI300554[] = {{&S522[51], 5}, {NULL}};
const Segment SGI300555[] = {{&S522[63], 3}, {NULL}};
const Segment SGI300556[] = {{&S523[31], 11}, {NULL}};
const Segment SGI300557[] = {{&S523[8], 3}, {NULL}};
const Segment SGI300558[] = {{&S523[13], 11}, {NULL}};
const Segment SGI300559[] = {{&S523[62], 32}, {NULL}};
const Segment SGI300560[] = {{&S524[30], 3}, {&S524[145], 3}, {&S524[145], 3}, {NULL}};
const Segment SGI300561[] = {{&S524[8], 3}, {&S524[8], 3}, {&S524[8], 3}, {NULL}};
const Segment SGI300562[] = {{&S524[13], 10}, {&S524[13], 10}, {&S524[13], 10}, {NULL}};
const Segment SGI300563[] = {{&S524[42], 7}, {NULL}};
const Segment SGI300564[] = {{&S524[35], 14}, {NULL}};
const Segment SGI300565[] = {{&S524[150], 5}, {&S524[150], 5}, {NULL}};
const Segment SGI300566[] = {{&S525[30], 3}, {NULL}};
const Segment SGI300567[] = {{&S525[8], 3}, {NULL}};
const Segment SGI300568[] = {{&S525[13], 10}, {NULL}};
const Segment SGI300569[] = {{&S525[42], 7}, {NULL}};
const Segment SGI300570[] = {{&S525[35], 14}, {NULL}};
const Segment SGI300571[] = {{&S525[51], 6}, {NULL}};
const Segment SGI300572[] = {{&S526[30], 3}, {NULL}};
const Segment SGI300573[] = {{&S526[8], 3}, {NULL}};
const Segment SGI300574[] = {{&S526[13], 10}, {NULL}};
const Segment SGI300575[] = {{&S526[42], 7}, {&S526[71], 30}, {NULL}};
const Segment SGI300576[] = {{&S526[35], 14}, {NULL}};
const Segment SGI300577[] = {{&S527[30], 3}, {NULL}};
const Segment SGI300578[] = {{&S527[8], 3}, {NULL}};
const Segment SGI300579[] = {{&S527[13], 10}, {NULL}};
const Segment SGI300580[] = {{&S527[42], 7}, {&S527[145], 20}, {NULL}};
const Segment SGI300581[] = {{&S527[35], 14}, {NULL}};
const Segment SGI300582[] = {{&S527[146], 15}, {NULL}};
const Segment SGI300583[] = {{&S528[30], 3}, {NULL}};
const Segment SGI300584[] = {{&S528[8], 3}, {NULL}};
const Segment SGI300585[] = {{&S528[13], 10}, {NULL}};
const Segment SGI300586[] = {{&S528[42], 7}, {NULL}};
const Segment SGI300587[] = {{&S528[35], 14}, {NULL}};
const Segment SGI300588[] = {{&S529[30], 3}, {NULL}};
const Segment SGI300589[] = {{&S529[8], 3}, {NULL}};
const Segment SGI300590[] = {{&S529[13], 10}, {NULL}};
const Segment SGI300591[] = {{&S529[42], 7}, {NULL}};
const Segment SGI300592[] = {{&S529[35], 14}, {NULL}};
const Segment SGI300593[] = {{&S529[51], 6}, {NULL}};
const Segment SGI300594[] = {{&S529[58], 3}, {NULL}};
const Segment SGI300595[] = {{&S530[30], 3}, {NULL}};
const Segment SGI300596[] = {{&S530[8], 3}, {NULL}};
const Segment SGI300597[] = {{&S530[13], 10}, {NULL}};
const Segment SGI300598[] = {{&S530[42], 9}, {NULL}};
const Segment SGI300599[] = {{&S530[53], 3}, {NULL}};
const Segment SGI300600[] = {{&S531[30], 3}, {&S531[30], 3}, {NULL}};
const Segment SGI300601[] = {{&S531[8], 3}, {&S531[8], 3}, {NULL}};
const Segment SGI300602[] = {{&S531[13], 10}, {&S531[13], 10}, {NULL}};
const Segment SGI300603[] = {{&S531[42], 7}, {&S531[42], 7}, {NULL}};
const Segment SGI300604[] = {{&S531[35], 14}, {&S531[35], 14}, {NULL}};
const Segment SGI300605[] = {{&S531[102], 3}, {NULL}};
const Segment SGI300606[] = {{&S532[31], 3}, {NULL}};
const Segment SGI300607[] = {{&S532[43], 7}, {NULL}};
const Segment SGI300608[] = {{&S533[29], 3}, {NULL}};
const Segment SGI300609[] = {{&S533[8], 3}, {NULL}};
const Segment SGI300610[] = {{&S533[13], 10}, {NULL}};
const Segment SGI300611[] = {{&S533[42], 7}, {NULL}};
const Segment SGI300612[] = {{&S533[35], 14}, {NULL}};
const Segment SGI300613[] = {{&S534[30], 3}, {NULL}};
const Segment SGI300614[] = {{&S534[8], 3}, {NULL}};
const Segment SGI300615[] = {{&S534[13], 10}, {NULL}};
const Segment SGI300616[] = {{&S534[42], 7}, {NULL}};
const Segment SGI300617[] = {{&S534[35], 14}, {NULL}};
const Segment SGI300618[] = {{&S534[51], 3}, {NULL}};
const Segment SGI300619[] = {{&S535[30], 3}, {NULL}};
const Segment SGI300620[] = {{&S535[8], 3}, {NULL}};
const Segment SGI300621[] = {{&S535[13], 10}, {NULL}};
const Segment SGI300622[] = {{&S535[42], 7}, {NULL}};
const Segment SGI300623[] = {{&S535[35], 14}, {NULL}};
const Segment SGI300624[] = {{&S535[94], 15}, {&S535[94], 15}, {NULL}};
const Segment SGI300625[] = {{&S535[51], 3}, {NULL}};
const Segment SGI300626[] = {{&S536[30], 3}, {&S536[93], 3}, {NULL}};
const Segment SGI300627[] = {{&S536[8], 3}, {&S536[8], 3}, {NULL}};
const Segment SGI300628[] = {{&S536[13], 10}, {&S536[13], 10}, {NULL}};
const Segment SGI300629[] = {{&S536[42], 7}, {&S536[105], 7}, {NULL}};
const Segment SGI300630[] = {{&S536[35], 14}, {&S536[98], 14}, {NULL}};
const Segment SGI300631[] = {{&S536[51], 5}, {NULL}};
const Segment SGI300632[] = {{&S536[114], 3}, {NULL}};
const Segment SGI300633[] = {{&S537[30], 3}, {NULL}};
const Segment SGI300634[] = {{&S537[8], 3}, {NULL}};
const Segment SGI300635[] = {{&S537[13], 10}, {NULL}};
const Segment SGI300636[] = {{&S537[42], 7}, {NULL}};
const Segment SGI300637[] = {{&S537[35], 14}, {NULL}};
const Segment SGI300638[] = {{&S537[51], 15}, {NULL}};
const Segment SGI300639[] = {{&S538[29], 3}, {NULL}};
const Segment SGI300640[] = {{&S538[8], 3}, {NULL}};
const Segment SGI300641[] = {{&S538[13], 10}, {NULL}};
const Segment SGI300642[] = {{&S538[42], 7}, {NULL}};
const Segment SGI300643[] = {{&S538[35], 14}, {NULL}};
const Segment SGI300644[] = {{&S539[30], 3}, {&S539[81], 3}, {NULL}};
const Segment SGI300645[] = {{&S539[8], 3}, {&S539[8], 3}, {NULL}};
const Segment SGI300646[] = {{&S539[13], 10}, {&S539[13], 10}, {NULL}};
const Segment SGI300647[] = {{&S539[42], 7}, {&S539[42], 7}, {NULL}};
const Segment SGI300648[] = {{&S539[35], 14}, {&S539[35], 14}, {NULL}};
const Segment SGI300649[] = {{&S539[219], 4}, {NULL}};
const Segment SGI300650[] = {{&S539[102], 3}, {NULL}};
const Segment SGI300651[] = {{&S540[30], 3}, {&S540[81], 3}, {&S540[81], 3}, {&S540[81], 3}, {NULL}};
const Segment SGI300652[] = {{&S540[8], 3}, {&S540[8], 3}, {&S540[8], 3}, {&S540[8], 3}, {NULL}};
const Segment SGI300653[] = {{&S540[13], 10}, {&S540[13], 10}, {&S540[13], 10}, {&S540[13], 10}, {NULL}};
const Segment SGI300654[] = {{&S540[42], 7}, {NULL}};
const Segment SGI300655[] = {{&S540[35], 14}, {NULL}};
const Segment SGI300656[] = {{&S540[86], 5}, {&S540[86], 5}, {&S540[86], 5}, {NULL}};
const Segment SGI300657[] = {{&S541[30], 3}, {NULL}};
const Segment SGI300658[] = {{&S541[8], 3}, {NULL}};
const Segment SGI300659[] = {{&S541[13], 10}, {NULL}};
const Segment SGI300660[] = {{&S541[42], 7}, {NULL}};
const Segment SGI300661[] = {{&S541[35], 14}, {NULL}};
const Segment SGI300662[] = {{&S541[51], 3}, {&S541[51], 3}, {NULL}};
const Segment SGI300663[] = {{&S542[30], 3}, {&S542[81], 3}, {NULL}};
const Segment SGI300664[] = {{&S542[8], 3}, {&S542[8], 3}, {NULL}};
const Segment SGI300665[] = {{&S542[13], 10}, {&S542[13], 10}, {NULL}};
const Segment SGI300666[] = {{&S542[42], 7}, {&S542[42], 7}, {NULL}};
const Segment SGI300667[] = {{&S542[35], 14}, {&S542[35], 14}, {NULL}};
const Segment SGI300668[] = {{&S542[109], 9}, {NULL}};
const Segment SGI300669[] = {{&S542[102], 3}, {NULL}};
const Segment SGI300670[] = {{&S543[30], 3}, {&S543[216], 3}, {NULL}};
const Segment SGI300671[] = {{&S543[8], 3}, {NULL}};
const Segment SGI300672[] = {{&S543[13], 10}, {NULL}};
const Segment SGI300673[] = {{&S543[56], 7}, {NULL}};
const Segment SGI300674[] = {{&S543[49], 14}, {NULL}};
const Segment SGI300675[] = {{&S544[30], 3}, {NULL}};
const Segment SGI300676[] = {{&S544[8], 3}, {NULL}};
const Segment SGI300677[] = {{&S544[13], 7}, {NULL}};
const Segment SGI300678[] = {{&S544[42], 7}, {NULL}};
const Segment SGI300679[] = {{&S544[35], 14}, {NULL}};
const Segment SGI300680[] = {{&S545[30], 3}, {NULL}};
const Segment SGI300681[] = {{&S545[8], 3}, {NULL}};
const Segment SGI300682[] = {{&S545[13], 10}, {NULL}};
const Segment SGI300683[] = {{&S545[42], 7}, {NULL}};
const Segment SGI300684[] = {{&S546[30], 3}, {&S546[86], 3}, {NULL}};
const Segment SGI300685[] = {{&S546[8], 3}, {&S546[8], 3}, {NULL}};
const Segment SGI300686[] = {{&S546[13], 10}, {&S546[13], 10}, {NULL}};
const Segment SGI300687[] = {{&S546[42], 7}, {&S546[42], 7}, {NULL}};
const Segment SGI300688[] = {{&S546[35], 14}, {&S546[35], 14}, {NULL}};
const Segment SGI300689[] = {{&S546[51], 3}, {&S546[51], 3}, {NULL}};
const Segment SGI300690[] = {{&S547[30], 3}, {&S547[101], 3}, {NULL}};
const Segment SGI300691[] = {{&S547[8], 3}, {&S547[8], 3}, {NULL}};
const Segment SGI300692[] = {{&S547[13], 10}, {&S547[13], 10}, {NULL}};
const Segment SGI300693[] = {{&S547[42], 7}, {NULL}};
const Segment SGI300694[] = {{&S547[35], 14}, {NULL}};
const Segment SGI300695[] = {{&S547[106], 5}, {NULL}};
const Segment SGI300696[] = {{&S548[30], 3}, {&S548[88], 3}, {NULL}};
const Segment SGI300697[] = {{&S548[8], 3}, {&S548[8], 3}, {NULL}};
const Segment SGI300698[] = {{&S548[13], 10}, {&S548[13], 10}, {NULL}};
const Segment SGI300699[] = {{&S548[42], 7}, {&S548[100], 7}, {NULL}};
const Segment SGI300700[] = {{&S548[35], 14}, {&S548[93], 14}, {NULL}};
const Segment SGI300701[] = {{&S548[51], 6}, {NULL}};
const Segment SGI300702[] = {{&S548[109], 3}, {NULL}};
const Segment SGI300703[] = {{&S549[30], 3}, {NULL}};
const Segment SGI300704[] = {{&S549[8], 3}, {NULL}};
const Segment SGI300705[] = {{&S549[13], 10}, {NULL}};
const Segment SGI300706[] = {{&S549[42], 7}, {NULL}};
const Segment SGI300707[] = {{&S549[35], 14}, {NULL}};
const Segment SGI300708[] = {{&S549[145], 5}, {&S549[145], 5}, {NULL}};
const Segment SGI300709[] = {{&S549[51], 3}, {NULL}};
const Segment SGI300710[] = {{&S550[8], 3}, {&S550[94], 3}, {NULL}};
const Segment SGI300711[] = {{&S550[8], 3}, {&S550[72], 3}, {NULL}};
const Segment SGI300712[] = {{&S550[13], 10}, {&S550[13], 10}, {NULL}};
const Segment SGI300713[] = {{&S550[42], 7}, {&S550[42], 7}, {NULL}};
const Segment SGI300714[] = {{&S550[35], 14}, {&S550[35], 14}, {NULL}};
const Segment SGI300715[] = {{&S550[115], 3}, {NULL}};
const Segment SGI300716[] = {{&S551[30], 3}, {&S551[81], 3}, {NULL}};
const Segment SGI300717[] = {{&S551[8], 3}, {&S551[8], 3}, {NULL}};
const Segment SGI300718[] = {{&S551[13], 10}, {&S551[13], 10}, {NULL}};
const Segment SGI300719[] = {{&S551[42], 7}, {&S551[42], 7}, {NULL}};
const Segment SGI300720[] = {{&S551[35], 14}, {&S551[35], 14}, {NULL}};
const Segment SGI300721[] = {{&S551[194], 15}, {NULL}};
const Segment SGI300722[] = {{&S551[235], 4}, {NULL}};
const Segment SGI300723[] = {{&S551[102], 3}, {NULL}};
const Segment SGI300724[] = {{&S552[30], 3}, {NULL}};
const Segment SGI300725[] = {{&S552[8], 3}, {NULL}};
const Segment SGI300726[] = {{&S552[13], 10}, {NULL}};
const Segment SGI300727[] = {{&S552[42], 7}, {NULL}};
const Segment SGI300728[] = {{&S552[35], 14}, {NULL}};
const Segment SGI300729[] = {{&S552[131], 15}, {NULL}};
const Segment SGI300730[] = {{&S552[152], 9}, {NULL}};
const Segment SGI300731[] = {{&S553[30], 3}, {NULL}};
const Segment SGI300732[] = {{&S553[8], 3}, {NULL}};
const Segment SGI300733[] = {{&S553[13], 10}, {NULL}};
const Segment SGI300734[] = {{&S553[42], 7}, {&S553[73], 20}, {NULL}};
const Segment SGI300735[] = {{&S553[35], 14}, {NULL}};
const Segment SGI300736[] = {{&S553[74], 15}, {NULL}};
const Segment SGI300737[] = {{&S553[51], 3}, {NULL}};
const Segment SGI300738[] = {{&S554[30], 3}, {&S554[86], 3}, {NULL}};
const Segment SGI300739[] = {{&S554[8], 3}, {&S554[8], 3}, {NULL}};
const Segment SGI300740[] = {{&S554[13], 10}, {&S554[13], 10}, {NULL}};
const Segment SGI300741[] = {{&S554[42], 7}, {NULL}};
const Segment SGI300742[] = {{&S554[35], 14}, {NULL}};
const Segment SGI300743[] = {{&S554[91], 5}, {NULL}};
const Segment SGI300744[] = {{&S554[51], 3}, {NULL}};
const Segment SGI300745[] = {{&S555[30], 3}, {NULL}};
const Segment SGI300746[] = {{&S555[8], 3}, {NULL}};
const Segment SGI300747[] = {{&S555[13], 10}, {NULL}};
const Segment SGI300748[] = {{&S555[42], 7}, {&S555[42], 7}, {NULL}};
const Segment SGI300749[] = {{&S555[35], 14}, {&S555[35], 14}, {NULL}};
const Segment SGI300750[] = {{&S556[30], 3}, {NULL}};
const Segment SGI300751[] = {{&S556[8], 3}, {NULL}};
const Segment SGI300752[] = {{&S556[13], 10}, {NULL}};
const Segment SGI300753[] = {{&S556[55], 7}, {NULL}};
const Segment SGI300754[] = {{&S556[48], 14}, {NULL}};
const Segment SGI300755[] = {{&S556[64], 3}, {NULL}};
const Segment SGI300756[] = {{&S557[30], 3}, {NULL}};
const Segment SGI300757[] = {{&S557[8], 3}, {&S557[8], 3}, {NULL}};
const Segment SGI300758[] = {{&S557[13], 10}, {NULL}};
const Segment SGI300759[] = {{&S557[42], 7}, {NULL}};
const Segment SGI300760[] = {{&S557[35], 14}, {NULL}};
const Segment SGI300761[] = {{&S558[30], 3}, {NULL}};
const Segment SGI300762[] = {{&S558[8], 3}, {NULL}};
const Segment SGI300763[] = {{&S558[13], 10}, {NULL}};
const Segment SGI300764[] = {{&S558[42], 7}, {NULL}};
const Segment SGI300765[] = {{&S558[35], 14}, {NULL}};
const Segment SGI300766[] = {{&S558[149], 3}, {&S558[149], 3}, {NULL}};
const Segment SGI300767[] = {{&S560[30], 3}, {&S560[102], 3}, {NULL}};
const Segment SGI300768[] = {{&S560[8], 3}, {&S560[8], 3}, {NULL}};
const Segment SGI300769[] = {{&S560[13], 10}, {&S560[13], 10}, {NULL}};
const Segment SGI300770[] = {{&S560[42], 7}, {NULL}};
const Segment SGI300771[] = {{&S560[35], 14}, {NULL}};
const Segment SGI300772[] = {{&S561[30], 3}, {NULL}};
const Segment SGI300773[] = {{&S561[8], 3}, {NULL}};
const Segment SGI300774[] = {{&S561[13], 10}, {NULL}};
const Segment SGI300775[] = {{&S561[42], 7}, {NULL}};
const Segment SGI300776[] = {{&S561[35], 14}, {NULL}};
const Segment SGI300777[] = {{&S561[109], 3}, {NULL}};
const Segment SGI300778[] = {{&S562[30], 3}, {NULL}};
const Segment SGI300779[] = {{&S562[8], 3}, {NULL}};
const Segment SGI300780[] = {{&S562[13], 10}, {NULL}};
const Segment SGI300781[] = {{&S562[42], 7}, {&S562[119], 20}, {NULL}};
const Segment SGI300782[] = {{&S562[35], 14}, {NULL}};
const Segment SGI300783[] = {{&S562[120], 15}, {&S562[120], 15}, {NULL}};
const Segment SGI300784[] = {{&S562[51], 3}, {NULL}};
const Segment SGI300785[] = {{&S563[5], 3}, {NULL}};
const Segment SGI300786[] = {{&S563[17], 4}, {NULL}};
const Segment SGI300787[] = {{&S564[30], 3}, {NULL}};
const Segment SGI300788[] = {{&S564[8], 3}, {NULL}};
const Segment SGI300789[] = {{&S564[13], 10}, {NULL}};
const Segment SGI300790[] = {{&S564[42], 7}, {NULL}};
const Segment SGI300791[] = {{&S564[35], 14}, {NULL}};
const Segment SGI300792[] = {{&S564[117], 15}, {NULL}};
const Segment SGI300793[] = {{&S564[176], 3}, {NULL}};
const Segment SGI300794[] = {{&S565[30], 3}, {NULL}};
const Segment SGI300795[] = {{&S565[8], 3}, {NULL}};
const Segment SGI300796[] = {{&S565[13], 10}, {NULL}};
const Segment SGI300797[] = {{&S565[42], 7}, {&S565[145], 20}, {NULL}};
const Segment SGI300798[] = {{&S565[35], 14}, {NULL}};
const Segment SGI300799[] = {{&S565[146], 15}, {&S565[146], 15}, {NULL}};
const Segment SGI300800[] = {{&S566[30], 3}, {NULL}};
const Segment SGI300801[] = {{&S566[8], 3}, {NULL}};
const Segment SGI300802[] = {{&S566[13], 10}, {NULL}};
const Segment SGI300803[] = {{&S566[42], 7}, {NULL}};
const Segment SGI300804[] = {{&S566[35], 14}, {NULL}};
const Segment SGI300805[] = {{&S566[89], 4}, {NULL}};
const Segment SGI300806[] = {{&S566[64], 3}, {NULL}};
const Segment SGI300807[] = {{&S567[30], 3}, {&S567[94], 3}, {NULL}};
const Segment SGI300808[] = {{&S567[8], 3}, {&S567[8], 3}, {NULL}};
const Segment SGI300809[] = {{&S567[13], 10}, {&S567[13], 10}, {NULL}};
const Segment SGI300810[] = {{&S567[42], 7}, {&S567[42], 7}, {NULL}};
const Segment SGI300811[] = {{&S567[35], 14}, {&S567[35], 14}, {NULL}};
const Segment SGI300812[] = {{&S568[75], 11}, {NULL}};
const Segment SGI300813[] = {{&S568[8], 3}, {NULL}};
const Segment SGI300814[] = {{&S568[13], 10}, {NULL}};
const Segment SGI300815[] = {{&S568[61], 7}, {NULL}};
const Segment SGI300816[] = {{&S569[30], 3}, {NULL}};
const Segment SGI300817[] = {{&S569[8], 3}, {NULL}};
const Segment SGI300818[] = {{&S569[13], 10}, {NULL}};
const Segment SGI300819[] = {{&S569[42], 7}, {NULL}};
const Segment SGI300820[] = {{&S569[35], 14}, {NULL}};
const Segment SGI300821[] = {{&S569[138], 15}, {NULL}};
const Segment SGI300822[] = {{&S569[51], 6}, {NULL}};
const Segment SGI300823[] = {{&S570[32], 3}, {NULL}};
const Segment SGI300824[] = {{&S570[10], 3}, {NULL}};
const Segment SGI300825[] = {{&S570[15], 10}, {NULL}};
const Segment SGI300826[] = {{&S570[44], 7}, {&S570[146], 3}, {NULL}};
const Segment SGI300827[] = {{&S570[37], 14}, {NULL}};
const Segment SGI300828[] = {{&S571[30], 3}, {NULL}};
const Segment SGI300829[] = {{&S571[8], 3}, {NULL}};
const Segment SGI300830[] = {{&S571[13], 10}, {NULL}};
const Segment SGI300831[] = {{&S571[42], 7}, {NULL}};
const Segment SGI300832[] = {{&S571[35], 14}, {NULL}};
const Segment SGI300833[] = {{&S571[74], 3}, {NULL}};
const Segment SGI300834[] = {{&S572[30], 3}, {NULL}};
const Segment SGI300835[] = {{&S572[8], 3}, {&S572[8], 3}, {NULL}};
const Segment SGI300836[] = {{&S572[13], 10}, {&S572[13], 10}, {NULL}};
const Segment SGI300837[] = {{&S572[42], 7}, {NULL}};
const Segment SGI300838[] = {{&S572[35], 14}, {NULL}};
const Segment SGI300839[] = {{&S573[30], 11}, {NULL}};
const Segment SGI300840[] = {{&S573[8], 3}, {NULL}};
const Segment SGI300841[] = {{&S573[13], 10}, {NULL}};
const Segment SGI300842[] = {{&S573[50], 7}, {NULL}};
const Segment SGI300843[] = {{&S574[30], 3}, {NULL}};
const Segment SGI300844[] = {{&S574[8], 3}, {NULL}};
const Segment SGI300845[] = {{&S574[13], 10}, {NULL}};
const Segment SGI300846[] = {{&S574[42], 7}, {NULL}};
const Segment SGI300847[] = {{&S574[35], 14}, {NULL}};
const Segment SGI300848[] = {{&S574[64], 3}, {NULL}};
const Segment SGI300849[] = {{&S574[79], 3}, {NULL}};
const Segment SGI300850[] = {{&S575[30], 3}, {NULL}};
const Segment SGI300851[] = {{&S575[8], 3}, {NULL}};
const Segment SGI300852[] = {{&S575[13], 10}, {NULL}};
const Segment SGI300853[] = {{&S575[42], 7}, {NULL}};
const Segment SGI300854[] = {{&S575[35], 14}, {NULL}};
const Segment SGI300855[] = {{&S575[74], 3}, {NULL}};
const Segment SGI300856[] = {{&S576[30], 3}, {NULL}};
const Segment SGI300857[] = {{&S576[8], 3}, {&S576[8], 3}, {NULL}};
const Segment SGI300858[] = {{&S576[13], 10}, {&S576[13], 10}, {NULL}};
const Segment SGI300859[] = {{&S576[42], 7}, {&S576[42], 7}, {NULL}};
const Segment SGI300860[] = {{&S576[35], 14}, {&S576[35], 14}, {NULL}};
const Segment SGI300861[] = {{&S576[92], 3}, {NULL}};
const Segment SGI300862[] = {{&S577[30], 3}, {&S577[81], 3}, {NULL}};
const Segment SGI300863[] = {{&S577[8], 3}, {&S577[8], 3}, {NULL}};
const Segment SGI300864[] = {{&S577[13], 10}, {&S577[13], 10}, {NULL}};
const Segment SGI300865[] = {{&S577[42], 7}, {&S577[42], 7}, {&S577[144], 5}, {&S577[144], 5}, {NULL}};
const Segment SGI300866[] = {{&S577[35], 14}, {&S577[35], 14}, {NULL}};
const Segment SGI300867[] = {{&S577[102], 3}, {NULL}};
const Segment SGI300868[] = {{&S578[30], 3}, {NULL}};
const Segment SGI300869[] = {{&S578[8], 3}, {NULL}};
const Segment SGI300870[] = {{&S578[13], 10}, {NULL}};
const Segment SGI300871[] = {{&S578[42], 7}, {NULL}};
const Segment SGI300872[] = {{&S578[35], 14}, {NULL}};
const Segment SGI300873[] = {{&S578[103], 15}, {NULL}};
const Segment SGI300874[] = {{&S578[51], 5}, {NULL}};
const Segment SGI300875[] = {{&S579[30], 3}, {NULL}};
const Segment SGI300876[] = {{&S579[8], 3}, {NULL}};
const Segment SGI300877[] = {{&S579[13], 10}, {NULL}};
const Segment SGI300878[] = {{&S579[42], 7}, {NULL}};
const Segment SGI300879[] = {{&S579[35], 14}, {NULL}};
const Segment SGI300880[] = {{&S579[146], 4}, {NULL}};
const Segment SGI300881[] = {{&S579[61], 9}, {NULL}};
const Segment SGI300882[] = {{&S580[30], 3}, {&S580[81], 3}, {NULL}};
const Segment SGI300883[] = {{&S580[8], 3}, {&S580[8], 3}, {NULL}};
const Segment SGI300884[] = {{&S580[13], 10}, {&S580[13], 10}, {NULL}};
const Segment SGI300885[] = {{&S580[42], 7}, {&S580[42], 7}, {NULL}};
const Segment SGI300886[] = {{&S580[35], 14}, {&S580[35], 14}, {NULL}};
const Segment SGI300887[] = {{&S580[148], 15}, {&S580[148], 15}, {NULL}};
const Segment SGI300888[] = {{&S580[102], 3}, {NULL}};
const Segment SGI300889[] = {{&S581[30], 3}, {NULL}};
const Segment SGI300890[] = {{&S581[8], 3}, {NULL}};
const Segment SGI300891[] = {{&S581[13], 10}, {NULL}};
const Segment SGI300892[] = {{&S581[42], 7}, {NULL}};
const Segment SGI300893[] = {{&S581[35], 14}, {NULL}};
const Segment SGI300894[] = {{&S582[30], 3}, {NULL}};
const Segment SGI300895[] = {{&S582[8], 3}, {NULL}};
const Segment SGI300896[] = {{&S582[13], 10}, {NULL}};
const Segment SGI300897[] = {{&S582[42], 7}, {NULL}};
const Segment SGI300898[] = {{&S582[35], 14}, {NULL}};
const Segment SGI300899[] = {{&S582[138], 15}, {NULL}};
const Segment SGI300900[] = {{&S582[179], 4}, {NULL}};
const Segment SGI300901[] = {{&S582[51], 6}, {NULL}};
const Segment SGI300902[] = {{&S583[30], 3}, {&S583[30], 3}, {&S583[150], 3}, {NULL}};
const Segment SGI300903[] = {{&S583[8], 3}, {&S583[8], 3}, {&S583[8], 3}, {NULL}};
const Segment SGI300904[] = {{&S583[13], 10}, {&S583[13], 10}, {&S583[13], 10}, {NULL}};
const Segment SGI300905[] = {{&S583[42], 7}, {&S583[42], 7}, {NULL}};
const Segment SGI300906[] = {{&S583[35], 14}, {&S583[35], 14}, {NULL}};
const Segment SGI300907[] = {{&S583[99], 5}, {NULL}};
const Segment SGI300908[] = {{&S583[171], 3}, {NULL}};
const Segment SGI300909[] = {{&S584[30], 3}, {NULL}};
const Segment SGI300910[] = {{&S584[8], 3}, {NULL}};
const Segment SGI300911[] = {{&S584[13], 10}, {NULL}};
const Segment SGI300912[] = {{&S584[42], 7}, {NULL}};
const Segment SGI300913[] = {{&S584[35], 14}, {NULL}};
const Segment SGI300914[] = {{&S584[107], 15}, {NULL}};
const Segment SGI300915[] = {{&S584[57], 4}, {NULL}};
const Segment SGI300916[] = {{&S584[52], 3}, {NULL}};
const Segment SGI300917[] = {{&S585[30], 3}, {NULL}};
const Segment SGI300918[] = {{&S585[8], 3}, {NULL}};
const Segment SGI300919[] = {{&S585[13], 10}, {NULL}};
const Segment SGI300920[] = {{&S585[42], 7}, {NULL}};
const Segment SGI300921[] = {{&S585[35], 14}, {NULL}};
const Segment SGI300922[] = {{&S585[104], 15}, {NULL}};
const Segment SGI300923[] = {{&S585[51], 6}, {NULL}};
const Segment SGI300924[] = {{&S585[59], 3}, {NULL}};
const Segment SGI300925[] = {{&S586[30], 3}, {NULL}};
const Segment SGI300926[] = {{&S586[8], 3}, {NULL}};
const Segment SGI300927[] = {{&S586[13], 10}, {NULL}};
const Segment SGI300928[] = {{&S586[42], 7}, {NULL}};
const Segment SGI300929[] = {{&S586[35], 14}, {NULL}};
const Segment SGI300930[] = {{&S587[30], 3}, {NULL}};
const Segment SGI300931[] = {{&S587[8], 3}, {NULL}};
const Segment SGI300932[] = {{&S587[13], 10}, {NULL}};
const Segment SGI300933[] = {{&S587[42], 7}, {NULL}};
const Segment SGI300934[] = {{&S587[35], 14}, {NULL}};
const Segment SGI300935[] = {{&S587[130], 15}, {NULL}};
const Segment SGI300936[] = {{&S588[30], 3}, {&S588[132], 3}, {NULL}};
const Segment SGI300937[] = {{&S588[8], 3}, {&S588[8], 3}, {NULL}};
const Segment SGI300938[] = {{&S588[13], 10}, {&S588[13], 10}, {NULL}};
const Segment SGI300939[] = {{&S588[42], 7}, {&S588[42], 7}, {NULL}};
const Segment SGI300940[] = {{&S588[35], 14}, {&S588[35], 14}, {NULL}};
const Segment SGI300941[] = {{&S588[227], 15}, {NULL}};
const Segment SGI300942[] = {{&S588[51], 6}, {NULL}};
const Segment SGI300943[] = {{&S588[153], 3}, {NULL}};
const Segment SGI300944[] = {{&S589[30], 3}, {&S589[103], 3}, {NULL}};
const Segment SGI300945[] = {{&S589[8], 3}, {&S589[8], 3}, {NULL}};
const Segment SGI300946[] = {{&S589[13], 10}, {&S589[13], 10}, {NULL}};
const Segment SGI300947[] = {{&S589[42], 7}, {&S589[42], 7}, {&S589[139], 20}, {NULL}};
const Segment SGI300948[] = {{&S589[35], 14}, {&S589[35], 14}, {NULL}};
const Segment SGI300949[] = {{&S589[140], 15}, {&S589[140], 15}, {NULL}};
const Segment SGI300950[] = {{&S589[124], 3}, {NULL}};
const Segment SGI300951[] = {{&S590[79], 11}, {NULL}};
const Segment SGI300952[] = {{&S590[8], 3}, {NULL}};
const Segment SGI300953[] = {{&S590[13], 10}, {NULL}};
const Segment SGI300954[] = {{&S590[65], 7}, {NULL}};
const Segment SGI300955[] = {{&S591[30], 3}, {NULL}};
const Segment SGI300956[] = {{&S591[8], 3}, {NULL}};
const Segment SGI300957[] = {{&S591[13], 10}, {NULL}};
const Segment SGI300958[] = {{&S591[42], 7}, {NULL}};
const Segment SGI300959[] = {{&S591[35], 14}, {NULL}};
const Segment SGI300960[] = {{&S591[124], 15}, {NULL}};
const Segment SGI300961[] = {{&S591[52], 3}, {NULL}};
const Segment SGI300962[] = {{&S592[30], 3}, {NULL}};
const Segment SGI300963[] = {{&S592[8], 3}, {NULL}};
const Segment SGI300964[] = {{&S592[13], 10}, {NULL}};
const Segment SGI300965[] = {{&S592[42], 7}, {NULL}};
const Segment SGI300966[] = {{&S593[30], 3}, {&S593[94], 3}, {NULL}};
const Segment SGI300967[] = {{&S593[8], 3}, {&S593[8], 3}, {NULL}};
const Segment SGI300968[] = {{&S593[13], 10}, {&S593[13], 10}, {NULL}};
const Segment SGI300969[] = {{&S593[42], 7}, {&S593[106], 7}, {NULL}};
const Segment SGI300970[] = {{&S593[35], 14}, {&S593[99], 14}, {NULL}};
const Segment SGI300971[] = {{&S593[189], 15}, {NULL}};
const Segment SGI300972[] = {{&S593[115], 3}, {NULL}};
const Segment SGI300973[] = {{&S594[30], 3}, {NULL}};
const Segment SGI300974[] = {{&S594[8], 3}, {NULL}};
const Segment SGI300975[] = {{&S594[13], 10}, {NULL}};
const Segment SGI300976[] = {{&S594[42], 7}, {NULL}};
const Segment SGI300977[] = {{&S594[35], 14}, {NULL}};
const Segment SGI300978[] = {{&S594[138], 15}, {NULL}};
const Segment SGI300979[] = {{&S594[51], 6}, {NULL}};
const Segment SGI300980[] = {{&S595[30], 3}, {NULL}};
const Segment SGI300981[] = {{&S595[8], 3}, {NULL}};
const Segment SGI300982[] = {{&S595[13], 10}, {NULL}};
const Segment SGI300983[] = {{&S595[42], 7}, {NULL}};
const Segment SGI300984[] = {{&S595[35], 14}, {NULL}};
const Segment SGI300985[] = {{&S595[59], 4}, {NULL}};
const Segment SGI300986[] = {{&S596[30], 3}, {NULL}};
const Segment SGI300987[] = {{&S596[8], 3}, {NULL}};
const Segment SGI300988[] = {{&S596[13], 10}, {NULL}};
const Segment SGI300989[] = {{&S596[42], 7}, {NULL}};
const Segment SGI300990[] = {{&S596[35], 14}, {NULL}};
const Segment SGI300991[] = {{&S596[131], 15}, {NULL}};
const Segment SGI300992[] = {{&S597[30], 3}, {NULL}};
const Segment SGI300993[] = {{&S597[8], 3}, {NULL}};
const Segment SGI300994[] = {{&S597[13], 10}, {NULL}};
const Segment SGI300995[] = {{&S597[42], 7}, {NULL}};
const Segment SGI300996[] = {{&S597[35], 14}, {NULL}};
const Segment SGI300997[] = {{&S597[131], 15}, {NULL}};
const Segment SGI300998[] = {{&S597[162], 9}, {NULL}};
const Segment SGI300999[] = {{&S598[30], 3}, {NULL}};
const Segment SGI301000[] = {{&S598[8], 3}, {NULL}};
const Segment SGI301001[] = {{&S598[13], 10}, {NULL}};
const Segment SGI301002[] = {{&S598[42], 7}, {NULL}};
const Segment SGI301003[] = {{&S598[35], 14}, {NULL}};
const Segment SGI301004[] = {{&S598[143], 15}, {NULL}};
const Segment SGI301005[] = {{&S598[51], 5}, {NULL}};
const Segment SGI301006[] = {{&S599[30], 3}, {NULL}};
const Segment SGI301007[] = {{&S599[8], 3}, {NULL}};
const Segment SGI301008[] = {{&S599[13], 10}, {NULL}};
const Segment SGI301009[] = {{&S599[42], 7}, {NULL}};
const Segment SGI301010[] = {{&S599[35], 14}, {NULL}};
const Segment SGI301011[] = {{&S599[51], 6}, {NULL}};
const Segment SGI301012[] = {{&S600[30], 3}, {NULL}};
const Segment SGI301013[] = {{&S600[8], 3}, {NULL}};
const Segment SGI301014[] = {{&S600[13], 10}, {NULL}};
const Segment SGI301015[] = {{&S600[42], 7}, {NULL}};
const Segment SGI301016[] = {{&S600[35], 14}, {NULL}};
const Segment SGI301017[] = {{&S600[131], 15}, {NULL}};
const Segment SGI301018[] = {{&S600[162], 4}, {NULL}};
const Segment SGI301019[] = {{&S601[30], 3}, {NULL}};
const Segment SGI301020[] = {{&S601[8], 3}, {NULL}};
const Segment SGI301021[] = {{&S601[13], 10}, {NULL}};
const Segment SGI301022[] = {{&S601[42], 7}, {NULL}};
const Segment SGI301023[] = {{&S601[35], 14}, {NULL}};
const Segment SGI301024[] = {{&S601[138], 15}, {NULL}};
const Segment SGI301025[] = {{&S601[169], 4}, {NULL}};
const Segment SGI301026[] = {{&S601[51], 6}, {NULL}};
const Segment SGI301027[] = {{&S602[30], 3}, {NULL}};
const Segment SGI301028[] = {{&S602[8], 3}, {NULL}};
const Segment SGI301029[] = {{&S602[13], 10}, {NULL}};
const Segment SGI301030[] = {{&S602[42], 7}, {NULL}};
const Segment SGI301031[] = {{&S602[35], 14}, {NULL}};
const Segment SGI301032[] = {{&S602[131], 15}, {&S602[131], 15}, {NULL}};
const Segment SGI301033[] = {{&S603[30], 3}, {NULL}};
const Segment SGI301034[] = {{&S603[8], 3}, {NULL}};
const Segment SGI301035[] = {{&S603[13], 10}, {NULL}};
const Segment SGI301036[] = {{&S603[42], 7}, {NULL}};
const Segment SGI301037[] = {{&S603[35], 14}, {NULL}};
const Segment SGI301038[] = {{&S603[143], 15}, {NULL}};
const Segment SGI301039[] = {{&S603[174], 4}, {NULL}};
const Segment SGI301040[] = {{&S603[51], 5}, {NULL}};
const Segment SGI301041[] = {{&S604[30], 3}, {NULL}};
const Segment SGI301042[] = {{&S604[8], 3}, {NULL}};
const Segment SGI301043[] = {{&S604[13], 10}, {NULL}};
const Segment SGI301044[] = {{&S604[42], 7}, {NULL}};
const Segment SGI301045[] = {{&S604[35], 14}, {NULL}};
const Segment SGI301046[] = {{&S604[131], 15}, {NULL}};
const Segment SGI301047[] = {{&S604[177], 4}, {NULL}};
const Segment SGI301048[] = {{&S604[162], 9}, {NULL}};
const Segment SGI301049[] = {{&S605[30], 3}, {NULL}};
const Segment SGI301050[] = {{&S605[8], 3}, {NULL}};
const Segment SGI301051[] = {{&S605[13], 10}, {NULL}};
const Segment SGI301052[] = {{&S605[42], 7}, {NULL}};
const Segment SGI301053[] = {{&S605[35], 14}, {NULL}};
const Segment SGI301054[] = {{&S605[151], 9}, {NULL}};
const Segment SGI301055[] = {{&S606[30], 3}, {NULL}};
const Segment SGI301056[] = {{&S606[8], 3}, {NULL}};
const Segment SGI301057[] = {{&S606[13], 10}, {NULL}};
const Segment SGI301058[] = {{&S606[42], 7}, {NULL}};
const Segment SGI301059[] = {{&S606[35], 14}, {NULL}};
const Segment SGI301060[] = {{&S606[143], 15}, {&S606[143], 15}, {NULL}};
const Segment SGI301061[] = {{&S606[51], 5}, {NULL}};
const Segment SGI301062[] = {{&S607[30], 3}, {NULL}};
const Segment SGI301063[] = {{&S607[8], 3}, {NULL}};
const Segment SGI301064[] = {{&S607[13], 10}, {NULL}};
const Segment SGI301065[] = {{&S607[42], 7}, {NULL}};
const Segment SGI301066[] = {{&S607[35], 14}, {NULL}};
const Segment SGI301067[] = {{&S607[143], 15}, {&S607[143], 15}, {NULL}};
const Segment SGI301068[] = {{&S607[51], 5}, {NULL}};
const Segment SGI301069[] = {{&S607[192], 9}, {NULL}};
const Segment SGI301070[] = {{&S608[30], 3}, {NULL}};
const Segment SGI301071[] = {{&S608[8], 3}, {NULL}};
const Segment SGI301072[] = {{&S608[13], 10}, {NULL}};
const Segment SGI301073[] = {{&S608[42], 7}, {&S608[186], 30}, {NULL}};
const Segment SGI301074[] = {{&S608[35], 14}, {NULL}};
const Segment SGI301075[] = {{&S608[138], 15}, {NULL}};
const Segment SGI301076[] = {{&S608[51], 6}, {NULL}};
const Segment SGI301077[] = {{&S609[30], 3}, {NULL}};
const Segment SGI301078[] = {{&S609[8], 3}, {NULL}};
const Segment SGI301079[] = {{&S609[13], 10}, {NULL}};
const Segment SGI301080[] = {{&S609[42], 7}, {NULL}};
const Segment SGI301081[] = {{&S609[35], 14}, {NULL}};
const Segment SGI301082[] = {{&S609[143], 15}, {NULL}};
const Segment SGI301083[] = {{&S609[189], 4}, {NULL}};
const Segment SGI301084[] = {{&S609[51], 5}, {NULL}};
const Segment SGI301085[] = {{&S609[174], 9}, {NULL}};
const Segment SGI301086[] = {{&S610[30], 3}, {NULL}};
const Segment SGI301087[] = {{&S610[8], 3}, {NULL}};
const Segment SGI301088[] = {{&S610[13], 10}, {NULL}};
const Segment SGI301089[] = {{&S610[42], 7}, {NULL}};
const Segment SGI301090[] = {{&S610[35], 14}, {NULL}};
const Segment SGI301091[] = {{&S610[131], 15}, {&S610[131], 15}, {NULL}};
const Segment SGI301092[] = {{&S610[190], 9}, {NULL}};
const Segment SGI301093[] = {{&S611[30], 3}, {NULL}};
const Segment SGI301094[] = {{&S611[8], 3}, {NULL}};
const Segment SGI301095[] = {{&S611[13], 10}, {NULL}};
const Segment SGI301096[] = {{&S611[42], 7}, {NULL}};
const Segment SGI301097[] = {{&S611[35], 14}, {NULL}};
const Segment SGI301098[] = {{&S611[143], 15}, {NULL}};
const Segment SGI301099[] = {{&S611[51], 5}, {NULL}};
const Segment SGI301100[] = {{&S611[184], 9}, {NULL}};
const Segment SGI301101[] = {{&S612[30], 3}, {NULL}};
const Segment SGI301102[] = {{&S612[8], 3}, {NULL}};
const Segment SGI301103[] = {{&S612[13], 10}, {NULL}};
const Segment SGI301104[] = {{&S612[42], 7}, {NULL}};
const Segment SGI301105[] = {{&S612[35], 14}, {NULL}};
const Segment SGI301106[] = {{&S613[30], 3}, {NULL}};
const Segment SGI301107[] = {{&S613[8], 3}, {NULL}};
const Segment SGI301108[] = {{&S613[13], 10}, {NULL}};
const Segment SGI301109[] = {{&S613[42], 7}, {&S613[184], 30}, {NULL}};
const Segment SGI301110[] = {{&S613[35], 14}, {NULL}};
const Segment SGI301111[] = {{&S613[131], 15}, {NULL}};
const Segment SGI301112[] = {{&S613[162], 9}, {NULL}};
const Segment SGI301113[] = {{&S614[30], 3}, {NULL}};
const Segment SGI301114[] = {{&S614[8], 3}, {NULL}};
const Segment SGI301115[] = {{&S614[13], 10}, {NULL}};
const Segment SGI301116[] = {{&S614[42], 7}, {&S614[176], 30}, {NULL}};
const Segment SGI301117[] = {{&S614[35], 14}, {NULL}};
const Segment SGI301118[] = {{&S614[138], 15}, {NULL}};
const Segment SGI301119[] = {{&S614[51], 6}, {NULL}};
const Segment SGI301120[] = {{&S615[30], 3}, {NULL}};
const Segment SGI301121[] = {{&S615[8], 3}, {NULL}};
const Segment SGI301122[] = {{&S615[13], 10}, {NULL}};
const Segment SGI301123[] = {{&S615[42], 7}, {&S615[169], 30}, {NULL}};
const Segment SGI301124[] = {{&S615[35], 14}, {NULL}};
const Segment SGI301125[] = {{&S615[131], 15}, {NULL}};
const Segment SGI301126[] = {{&S616[30], 3}, {&S616[101], 3}, {NULL}};
const Segment SGI301127[] = {{&S616[8], 3}, {&S616[8], 3}, {NULL}};
const Segment SGI301128[] = {{&S616[13], 10}, {&S616[13], 10}, {NULL}};
const Segment SGI301129[] = {{&S616[42], 7}, {&S616[113], 7}, {NULL}};
const Segment SGI301130[] = {{&S616[35], 14}, {&S616[106], 14}, {NULL}};
const Segment SGI301131[] = {{&S616[196], 15}, {NULL}};
const Segment SGI301132[] = {{&S616[51], 6}, {NULL}};
const Segment SGI301133[] = {{&S616[122], 3}, {NULL}};
const Segment SGI301134[] = {{&S617[30], 3}, {&S617[106], 3}, {NULL}};
const Segment SGI301135[] = {{&S617[8], 3}, {&S617[8], 3}, {NULL}};
const Segment SGI301136[] = {{&S617[13], 10}, {&S617[13], 10}, {NULL}};
const Segment SGI301137[] = {{&S617[42], 7}, {&S617[118], 7}, {NULL}};
const Segment SGI301138[] = {{&S617[35], 14}, {&S617[111], 14}, {NULL}};
const Segment SGI301139[] = {{&S617[201], 15}, {NULL}};
const Segment SGI301140[] = {{&S617[51], 5}, {NULL}};
const Segment SGI301141[] = {{&S617[127], 3}, {NULL}};
const Segment SGI301142[] = {{&S618[30], 3}, {NULL}};
const Segment SGI301143[] = {{&S618[8], 3}, {NULL}};
const Segment SGI301144[] = {{&S618[13], 10}, {NULL}};
const Segment SGI301145[] = {{&S618[42], 7}, {NULL}};
const Segment SGI301146[] = {{&S618[35], 14}, {NULL}};
const Segment SGI301147[] = {{&S619[30], 3}, {NULL}};
const Segment SGI301148[] = {{&S619[8], 3}, {NULL}};
const Segment SGI301149[] = {{&S619[13], 10}, {NULL}};
const Segment SGI301150[] = {{&S619[42], 7}, {NULL}};
const Segment SGI301151[] = {{&S619[35], 14}, {NULL}};
const Segment SGI301152[] = {{&S619[51], 5}, {NULL}};
const Segment SGI301153[] = {{&S620[30], 3}, {NULL}};
const Segment SGI301154[] = {{&S620[8], 3}, {NULL}};
const Segment SGI301155[] = {{&S620[13], 10}, {NULL}};
const Segment SGI301156[] = {{&S620[42], 7}, {NULL}};
const Segment SGI301157[] = {{&S620[35], 14}, {NULL}};
const Segment SGI301158[] = {{&S620[64], 3}, {NULL}};
const Segment SGI301159[] = {{&S621[30], 3}, {NULL}};
const Segment SGI301160[] = {{&S621[8], 3}, {NULL}};
const Segment SGI301161[] = {{&S621[13], 10}, {NULL}};
const Segment SGI301162[] = {{&S621[42], 7}, {NULL}};
const Segment SGI301163[] = {{&S621[35], 14}, {NULL}};
const Segment SGI301164[] = {{&S621[51], 5}, {NULL}};
const Segment SGI301165[] = {{&S621[76], 3}, {NULL}};
const Segment SGI301166[] = {{&S622[30], 3}, {NULL}};
const Segment SGI301167[] = {{&S622[8], 3}, {NULL}};
const Segment SGI301168[] = {{&S622[13], 10}, {NULL}};
const Segment SGI301169[] = {{&S622[42], 7}, {NULL}};
const Segment SGI301170[] = {{&S622[35], 14}, {NULL}};
const Segment SGI301171[] = {{&S622[51], 6}, {NULL}};
const Segment SGI301172[] = {{&S623[30], 3}, {NULL}};
const Segment SGI301173[] = {{&S623[8], 3}, {NULL}};
const Segment SGI301174[] = {{&S623[13], 10}, {NULL}};
const Segment SGI301175[] = {{&S623[42], 7}, {NULL}};
const Segment SGI301176[] = {{&S623[35], 14}, {NULL}};
const Segment SGI301177[] = {{&S623[148], 9}, {NULL}};
const Segment SGI301178[] = {{&S623[64], 3}, {NULL}};
const Segment SGI301179[] = {{&S624[30], 3}, {NULL}};
const Segment SGI301180[] = {{&S624[8], 3}, {NULL}};
const Segment SGI301181[] = {{&S624[13], 10}, {NULL}};
const Segment SGI301182[] = {{&S624[42], 7}, {&S624[155], 30}, {NULL}};
const Segment SGI301183[] = {{&S624[35], 14}, {NULL}};
const Segment SGI301184[] = {{&S624[51], 6}, {NULL}};
const Segment SGI301185[] = {{&S625[30], 3}, {NULL}};
const Segment SGI301186[] = {{&S625[8], 3}, {NULL}};
const Segment SGI301187[] = {{&S625[13], 10}, {NULL}};
const Segment SGI301188[] = {{&S625[42], 7}, {&S625[178], 30}, {NULL}};
const Segment SGI301189[] = {{&S625[35], 14}, {NULL}};
const Segment SGI301190[] = {{&S625[138], 15}, {NULL}};
const Segment SGI301191[] = {{&S625[51], 6}, {NULL}};
const Segment SGI301192[] = {{&S626[30], 3}, {NULL}};
const Segment SGI301193[] = {{&S626[8], 3}, {NULL}};
const Segment SGI301194[] = {{&S626[13], 10}, {NULL}};
const Segment SGI301195[] = {{&S626[42], 7}, {&S626[176], 30}, {NULL}};
const Segment SGI301196[] = {{&S626[35], 14}, {NULL}};
const Segment SGI301197[] = {{&S626[138], 15}, {NULL}};
const Segment SGI301198[] = {{&S626[51], 6}, {NULL}};
const Segment SGI301199[] = {{&S627[30], 3}, {NULL}};
const Segment SGI301200[] = {{&S627[8], 3}, {NULL}};
const Segment SGI301201[] = {{&S627[13], 10}, {NULL}};
const Segment SGI301202[] = {{&S627[42], 7}, {NULL}};
const Segment SGI301203[] = {{&S627[35], 14}, {NULL}};
const Segment SGI301204[] = {{&S627[51], 6}, {NULL}};
const Segment SGI301205[] = {{&S627[150], 9}, {NULL}};
const Segment SGI301206[] = {{&S628[30], 4}, {NULL}};
const Segment SGI301207[] = {{&S628[8], 3}, {NULL}};
const Segment SGI301208[] = {{&S628[13], 10}, {NULL}};
const Segment SGI301209[] = {{&S628[43], 7}, {NULL}};
const Segment SGI301210[] = {{&S629[30], 3}, {NULL}};
const Segment SGI301211[] = {{&S629[8], 3}, {NULL}};
const Segment SGI301212[] = {{&S629[13], 10}, {NULL}};
const Segment SGI301213[] = {{&S629[42], 7}, {&S629[184], 30}, {NULL}};
const Segment SGI301214[] = {{&S629[35], 14}, {NULL}};
const Segment SGI301215[] = {{&S629[131], 15}, {NULL}};
const Segment SGI301216[] = {{&S629[162], 9}, {NULL}};
const Segment SGI301217[] = {{&S630[30], 3}, {NULL}};
const Segment SGI301218[] = {{&S630[8], 3}, {NULL}};
const Segment SGI301219[] = {{&S630[13], 10}, {NULL}};
const Segment SGI301220[] = {{&S630[42], 7}, {&S630[155], 30}, {NULL}};
const Segment SGI301221[] = {{&S630[35], 14}, {NULL}};
const Segment SGI301222[] = {{&S630[51], 6}, {NULL}};
const Segment SGI301223[] = {{&S631[30], 3}, {NULL}};
const Segment SGI301224[] = {{&S631[8], 3}, {NULL}};
const Segment SGI301225[] = {{&S631[13], 10}, {NULL}};
const Segment SGI301226[] = {{&S631[42], 7}, {&S631[186], 30}, {NULL}};
const Segment SGI301227[] = {{&S631[35], 14}, {NULL}};
const Segment SGI301228[] = {{&S631[131], 15}, {NULL}};
const Segment SGI301229[] = {{&S632[30], 3}, {&S632[94], 3}, {NULL}};
const Segment SGI301230[] = {{&S632[8], 3}, {&S632[8], 3}, {NULL}};
const Segment SGI301231[] = {{&S632[13], 10}, {&S632[13], 10}, {NULL}};
const Segment SGI301232[] = {{&S632[42], 7}, {&S632[106], 7}, {NULL}};
const Segment SGI301233[] = {{&S632[35], 14}, {&S632[99], 14}, {NULL}};
const Segment SGI301234[] = {{&S632[204], 15}, {NULL}};
const Segment SGI301235[] = {{&S632[189], 9}, {NULL}};
const Segment SGI301236[] = {{&S632[115], 3}, {NULL}};
const Segment SGI301237[] = {{&S633[30], 3}, {NULL}};
const Segment SGI301238[] = {{&S633[8], 3}, {NULL}};
const Segment SGI301239[] = {{&S633[13], 10}, {NULL}};
const Segment SGI301240[] = {{&S633[42], 7}, {NULL}};
const Segment SGI301241[] = {{&S633[35], 14}, {NULL}};
const Segment SGI301242[] = {{&S633[51], 5}, {NULL}};
const Segment SGI301243[] = {{&S633[163], 9}, {NULL}};
const Segment SGI301244[] = {{&S634[30], 3}, {NULL}};
const Segment SGI301245[] = {{&S634[8], 3}, {NULL}};
const Segment SGI301246[] = {{&S634[13], 10}, {NULL}};
const Segment SGI301247[] = {{&S634[42], 7}, {NULL}};
const Segment SGI301248[] = {{&S634[35], 14}, {NULL}};
const Segment SGI301249[] = {{&S634[131], 15}, {&S634[131], 15}, {NULL}};
const Segment SGI301250[] = {{&S634[205], 4}, {NULL}};
const Segment SGI301251[] = {{&S634[180], 9}, {NULL}};
const Segment SGI301252[] = {{&S635[30], 3}, {&S635[94], 3}, {NULL}};
const Segment SGI301253[] = {{&S635[8], 3}, {&S635[8], 3}, {NULL}};
const Segment SGI301254[] = {{&S635[13], 10}, {&S635[13], 10}, {NULL}};
const Segment SGI301255[] = {{&S635[42], 7}, {&S635[106], 7}, {NULL}};
const Segment SGI301256[] = {{&S635[35], 14}, {&S635[99], 14}, {NULL}};
const Segment SGI301257[] = {{&S635[202], 9}, {NULL}};
const Segment SGI301258[] = {{&S635[115], 3}, {NULL}};
const Segment SGI301259[] = {{&S636[30], 3}, {NULL}};
const Segment SGI301260[] = {{&S636[8], 3}, {NULL}};
const Segment SGI301261[] = {{&S636[13], 10}, {NULL}};
const Segment SGI301262[] = {{&S636[42], 7}, {&S636[184], 30}, {NULL}};
const Segment SGI301263[] = {{&S636[35], 14}, {NULL}};
const Segment SGI301264[] = {{&S636[131], 15}, {NULL}};
const Segment SGI301265[] = {{&S636[162], 9}, {NULL}};
const Segment SGI301266[] = {{&S637[30], 3}, {NULL}};
const Segment SGI301267[] = {{&S637[8], 3}, {NULL}};
const Segment SGI301268[] = {{&S637[13], 10}, {NULL}};
const Segment SGI301269[] = {{&S637[42], 7}, {NULL}};
const Segment SGI301270[] = {{&S637[35], 14}, {NULL}};
const Segment SGI301271[] = {{&S637[158], 4}, {NULL}};
const Segment SGI301272[] = {{&S637[51], 6}, {NULL}};
const Segment SGI301273[] = {{&S638[30], 3}, {&S638[94], 3}, {NULL}};
const Segment SGI301274[] = {{&S638[8], 3}, {&S638[8], 3}, {NULL}};
const Segment SGI301275[] = {{&S638[13], 10}, {&S638[13], 10}, {NULL}};
const Segment SGI301276[] = {{&S638[42], 7}, {&S638[106], 7}, {NULL}};
const Segment SGI301277[] = {{&S638[35], 14}, {&S638[99], 14}, {NULL}};
const Segment SGI301278[] = {{&S638[115], 3}, {NULL}};
const Segment SGI301279[] = {{&S639[30], 3}, {NULL}};
const Segment SGI301280[] = {{&S639[8], 3}, {NULL}};
const Segment SGI301281[] = {{&S639[13], 10}, {NULL}};
const Segment SGI301282[] = {{&S639[42], 7}, {&S639[169], 30}, {NULL}};
const Segment SGI301283[] = {{&S639[35], 14}, {NULL}};
const Segment SGI301284[] = {{&S639[131], 15}, {NULL}};
const Segment SGI301285[] = {{&S640[30], 3}, {NULL}};
const Segment SGI301286[] = {{&S640[8], 3}, {NULL}};
const Segment SGI301287[] = {{&S640[13], 10}, {NULL}};
const Segment SGI301288[] = {{&S640[42], 7}, {&S640[186], 30}, {NULL}};
const Segment SGI301289[] = {{&S640[35], 14}, {NULL}};
const Segment SGI301290[] = {{&S640[138], 15}, {NULL}};
const Segment SGI301291[] = {{&S640[51], 6}, {NULL}};
const Segment SGI301292[] = {{&S641[30], 3}, {NULL}};
const Segment SGI301293[] = {{&S641[8], 3}, {NULL}};
const Segment SGI301294[] = {{&S641[13], 10}, {NULL}};
const Segment SGI301295[] = {{&S641[42], 7}, {&S641[187], 30}, {NULL}};
const Segment SGI301296[] = {{&S641[35], 14}, {NULL}};
const Segment SGI301297[] = {{&S642[30], 3}, {NULL}};
const Segment SGI301298[] = {{&S642[8], 3}, {NULL}};
const Segment SGI301299[] = {{&S642[13], 10}, {NULL}};
const Segment SGI301300[] = {{&S642[42], 7}, {NULL}};
const Segment SGI301301[] = {{&S642[35], 14}, {NULL}};
const Segment SGI301302[] = {{&S642[51], 6}, {NULL}};
const Segment SGI301303[] = {{&S642[71], 3}, {NULL}};
const Segment SGI301304[] = {{&S643[30], 3}, {NULL}};
const Segment SGI301305[] = {{&S643[8], 3}, {NULL}};
const Segment SGI301306[] = {{&S643[13], 10}, {NULL}};
const Segment SGI301307[] = {{&S643[42], 7}, {&S643[165], 30}, {NULL}};
const Segment SGI301308[] = {{&S643[35], 14}, {NULL}};
const Segment SGI301309[] = {{&S643[51], 6}, {NULL}};
const Segment SGI301310[] = {{&S644[30], 3}, {&S644[101], 3}, {NULL}};
const Segment SGI301311[] = {{&S644[8], 3}, {&S644[8], 3}, {NULL}};
const Segment SGI301312[] = {{&S644[13], 10}, {&S644[13], 10}, {NULL}};
const Segment SGI301313[] = {{&S644[42], 7}, {&S644[113], 7}, {NULL}};
const Segment SGI301314[] = {{&S644[35], 14}, {&S644[106], 14}, {NULL}};
const Segment SGI301315[] = {{&S644[51], 6}, {NULL}};
const Segment SGI301316[] = {{&S644[122], 3}, {NULL}};
const Segment SGI301317[] = {{&S645[30], 3}, {NULL}};
const Segment SGI301318[] = {{&S645[8], 3}, {NULL}};
const Segment SGI301319[] = {{&S645[13], 10}, {NULL}};
const Segment SGI301320[] = {{&S645[42], 7}, {NULL}};
const Segment SGI301321[] = {{&S645[35], 14}, {NULL}};
const Segment SGI301322[] = {{&S645[136], 15}, {NULL}};
const Segment SGI301323[] = {{&S645[64], 3}, {NULL}};
const Segment SGI301324[] = {{&S646[30], 3}, {NULL}};
const Segment SGI301325[] = {{&S646[8], 3}, {NULL}};
const Segment SGI301326[] = {{&S646[13], 10}, {NULL}};
const Segment SGI301327[] = {{&S646[42], 7}, {&S646[196], 30}, {NULL}};
const Segment SGI301328[] = {{&S646[35], 14}, {NULL}};
const Segment SGI301329[] = {{&S646[138], 15}, {NULL}};
const Segment SGI301330[] = {{&S646[179], 4}, {NULL}};
const Segment SGI301331[] = {{&S646[51], 6}, {NULL}};
const Segment SGI301332[] = {{&S647[30], 3}, {NULL}};
const Segment SGI301333[] = {{&S647[8], 3}, {NULL}};
const Segment SGI301334[] = {{&S647[13], 10}, {NULL}};
const Segment SGI301335[] = {{&S647[42], 7}, {&S647[148], 30}, {NULL}};
const Segment SGI301336[] = {{&S647[35], 14}, {NULL}};
const Segment SGI301337[] = {{&S648[30], 3}, {&S648[30], 3}, {NULL}};
const Segment SGI301338[] = {{&S648[8], 3}, {NULL}};
const Segment SGI301339[] = {{&S648[13], 10}, {NULL}};
const Segment SGI301340[] = {{&S648[42], 7}, {NULL}};
const Segment SGI301341[] = {{&S648[35], 14}, {NULL}};
const Segment SGI301342[] = {{&S648[131], 15}, {NULL}};
const Segment SGI301343[] = {{&S649[30], 3}, {NULL}};
const Segment SGI301344[] = {{&S649[8], 3}, {NULL}};
const Segment SGI301345[] = {{&S649[13], 10}, {NULL}};
const Segment SGI301346[] = {{&S649[42], 7}, {&S649[169], 30}, {NULL}};
const Segment SGI301347[] = {{&S649[35], 14}, {NULL}};
const Segment SGI301348[] = {{&S649[131], 15}, {NULL}};
const Segment SGI301349[] = {{&S650[30], 3}, {&S650[94], 3}, {NULL}};
const Segment SGI301350[] = {{&S650[8], 3}, {&S650[8], 3}, {NULL}};
const Segment SGI301351[] = {{&S650[13], 10}, {&S650[13], 10}, {NULL}};
const Segment SGI301352[] = {{&S650[42], 7}, {&S650[106], 7}, {NULL}};
const Segment SGI301353[] = {{&S650[35], 14}, {NULL}};
const Segment SGI301354[] = {{&S651[28], 3}, {NULL}};
const Segment SGI301355[] = {{&S651[8], 3}, {NULL}};
const Segment SGI301356[] = {{&S651[39], 7}, {NULL}};
const Segment SGI301357[] = {{&S651[32], 14}, {NULL}};
const Segment SGI301358[] = {{&S651[47], 3}, {NULL}};
const Segment SGI301359[] = {{&S652[30], 3}, {NULL}};
const Segment SGI301360[] = {{&S652[8], 3}, {NULL}};
const Segment SGI301361[] = {{&S652[13], 10}, {NULL}};
const Segment SGI301362[] = {{&S652[42], 7}, {&S652[174], 30}, {NULL}};
const Segment SGI301363[] = {{&S652[35], 14}, {NULL}};
const Segment SGI301364[] = {{&S652[131], 15}, {NULL}};
const Segment SGI301365[] = {{&S652[152], 9}, {NULL}};
const Segment SGI301366[] = {{&S653[30], 3}, {NULL}};
const Segment SGI301367[] = {{&S653[8], 3}, {NULL}};
const Segment SGI301368[] = {{&S653[13], 10}, {NULL}};
const Segment SGI301369[] = {{&S653[42], 7}, {NULL}};
const Segment SGI301370[] = {{&S653[35], 14}, {NULL}};
const Segment SGI301371[] = {{&S653[131], 15}, {NULL}};
const Segment SGI301372[] = {{&S653[163], 3}, {NULL}};
const Segment SGI301373[] = {{&S654[30], 3}, {&S654[30], 3}, {NULL}};
const Segment SGI301374[] = {{&S654[8], 3}, {NULL}};
const Segment SGI301375[] = {{&S654[13], 10}, {NULL}};
const Segment SGI301376[] = {{&S654[42], 7}, {NULL}};
const Segment SGI301377[] = {{&S654[35], 14}, {NULL}};
const Segment SGI301378[] = {{&S655[30], 3}, {&S655[147], 3}, {NULL}};
const Segment SGI301379[] = {{&S655[8], 3}, {NULL}};
const Segment SGI301380[] = {{&S655[13], 10}, {NULL}};
const Segment SGI301381[] = {{&S655[42], 7}, {NULL}};
const Segment SGI301382[] = {{&S655[35], 14}, {NULL}};
const Segment SGI301383[] = {{&S655[131], 15}, {NULL}};
const Segment SGI301384[] = {{&S656[30], 3}, {NULL}};
const Segment SGI301385[] = {{&S656[8], 3}, {NULL}};
const Segment SGI301386[] = {{&S656[13], 10}, {NULL}};
const Segment SGI301387[] = {{&S656[42], 7}, {NULL}};
const Segment SGI301388[] = {{&S656[35], 14}, {NULL}};
const Segment SGI301389[] = {{&S656[146], 15}, {NULL}};
const Segment SGI301390[] = {{&S656[51], 6}, {NULL}};
const Segment SGI301391[] = {{&S656[182], 3}, {NULL}};
const Segment SGI301392[] = {{&S657[30], 3}, {NULL}};
const Segment SGI301393[] = {{&S657[8], 3}, {NULL}};
const Segment SGI301394[] = {{&S657[13], 10}, {NULL}};
const Segment SGI301395[] = {{&S657[42], 7}, {&S657[166], 18}, {NULL}};
const Segment SGI301396[] = {{&S657[35], 14}, {NULL}};
const Segment SGI301397[] = {{&S657[138], 15}, {NULL}};
const Segment SGI301398[] = {{&S657[51], 6}, {NULL}};
const Segment SGI301399[] = {{&S658[30], 3}, {NULL}};
const Segment SGI301400[] = {{&S658[8], 3}, {NULL}};
const Segment SGI301401[] = {{&S658[13], 10}, {NULL}};
const Segment SGI301402[] = {{&S658[42], 7}, {&S658[186], 30}, {NULL}};
const Segment SGI301403[] = {{&S658[35], 14}, {NULL}};
const Segment SGI301404[] = {{&S658[138], 15}, {NULL}};
const Segment SGI301405[] = {{&S658[51], 6}, {NULL}};
const Segment SGI301406[] = {{&S659[30], 3}, {NULL}};
const Segment SGI301407[] = {{&S659[8], 3}, {NULL}};
const Segment SGI301408[] = {{&S659[13], 10}, {NULL}};
const Segment SGI301409[] = {{&S659[42], 7}, {NULL}};
const Segment SGI301410[] = {{&S659[35], 14}, {NULL}};
const Segment SGI301411[] = {{&S659[131], 15}, {NULL}};
const Segment SGI301412[] = {{&S659[167], 3}, {NULL}};
const Segment SGI301413[] = {{&S660[30], 3}, {NULL}};
const Segment SGI301414[] = {{&S660[8], 3}, {NULL}};
const Segment SGI301415[] = {{&S660[13], 10}, {NULL}};
const Segment SGI301416[] = {{&S660[42], 7}, {NULL}};
const Segment SGI301417[] = {{&S660[35], 14}, {NULL}};
const Segment SGI301418[] = {{&S660[136], 15}, {NULL}};
const Segment SGI301419[] = {{&S660[177], 9}, {NULL}};
const Segment SGI301420[] = {{&S660[64], 3}, {NULL}};
const Segment SGI301421[] = {{&S661[30], 3}, {&S661[30], 3}, {NULL}};
const Segment SGI301422[] = {{&S661[8], 3}, {NULL}};
const Segment SGI301423[] = {{&S661[13], 10}, {NULL}};
const Segment SGI301424[] = {{&S661[42], 7}, {NULL}};
const Segment SGI301425[] = {{&S661[35], 14}, {NULL}};
const Segment SGI301426[] = {{&S661[131], 15}, {NULL}};
const Segment SGI301427[] = {{&S661[184], 9}, {NULL}};
const Segment SGI301428[] = {{&S662[30], 3}, {NULL}};
const Segment SGI301429[] = {{&S662[8], 3}, {NULL}};
const Segment SGI301430[] = {{&S662[13], 10}, {NULL}};
const Segment SGI301431[] = {{&S662[42], 7}, {&S662[163], 30}, {NULL}};
const Segment SGI301432[] = {{&S662[35], 14}, {NULL}};
const Segment SGI301433[] = {{&S662[141], 9}, {NULL}};
const Segment SGI301434[] = {{&S663[30], 3}, {NULL}};
const Segment SGI301435[] = {{&S663[8], 3}, {NULL}};
const Segment SGI301436[] = {{&S663[13], 10}, {NULL}};
const Segment SGI301437[] = {{&S663[55], 7}, {NULL}};
const Segment SGI301438[] = {{&S663[48], 14}, {NULL}};
const Segment SGI301439[] = {{&S663[64], 3}, {NULL}};
const Segment SGI301440[] = {{&S664[30], 3}, {NULL}};
const Segment SGI301441[] = {{&S664[8], 3}, {NULL}};
const Segment SGI301442[] = {{&S664[13], 10}, {NULL}};
const Segment SGI301443[] = {{&S664[42], 7}, {&S664[179], 30}, {NULL}};
const Segment SGI301444[] = {{&S664[35], 14}, {NULL}};
const Segment SGI301445[] = {{&S664[131], 15}, {NULL}};
const Segment SGI301446[] = {{&S665[30], 3}, {&S665[106], 3}, {NULL}};
const Segment SGI301447[] = {{&S665[8], 3}, {&S665[8], 3}, {NULL}};
const Segment SGI301448[] = {{&S665[13], 10}, {&S665[13], 10}, {NULL}};
const Segment SGI301449[] = {{&S665[42], 7}, {&S665[118], 7}, {NULL}};
const Segment SGI301450[] = {{&S665[35], 14}, {&S665[111], 14}, {NULL}};
const Segment SGI301451[] = {{&S665[201], 15}, {NULL}};
const Segment SGI301452[] = {{&S665[51], 5}, {NULL}};
const Segment SGI301453[] = {{&S665[222], 9}, {NULL}};
const Segment SGI301454[] = {{&S665[127], 3}, {NULL}};
const Segment SGI301455[] = {{&S666[30], 3}, {NULL}};
const Segment SGI301456[] = {{&S666[8], 3}, {NULL}};
const Segment SGI301457[] = {{&S666[13], 10}, {NULL}};
const Segment SGI301458[] = {{&S666[42], 7}, {&S666[145], 20}, {NULL}};
const Segment SGI301459[] = {{&S666[35], 14}, {NULL}};
const Segment SGI301460[] = {{&S666[146], 15}, {NULL}};
const Segment SGI301461[] = {{&S667[30], 3}, {NULL}};
const Segment SGI301462[] = {{&S667[8], 3}, {NULL}};
const Segment SGI301463[] = {{&S667[13], 10}, {NULL}};
const Segment SGI301464[] = {{&S667[42], 7}, {&S667[186], 30}, {NULL}};
const Segment SGI301465[] = {{&S667[35], 14}, {NULL}};
const Segment SGI301466[] = {{&S667[138], 15}, {NULL}};
const Segment SGI301467[] = {{&S667[51], 6}, {NULL}};
const Segment SGI301468[] = {{&S668[30], 3}, {NULL}};
const Segment SGI301469[] = {{&S668[8], 3}, {NULL}};
const Segment SGI301470[] = {{&S668[13], 10}, {NULL}};
const Segment SGI301471[] = {{&S668[42], 7}, {NULL}};
const Segment SGI301472[] = {{&S668[35], 14}, {NULL}};
const Segment SGI301473[] = {{&S668[131], 15}, {NULL}};
const Segment SGI301474[] = {{&S668[209], 5}, {NULL}};
const Segment SGI301475[] = {{&S669[30], 3}, {NULL}};
const Segment SGI301476[] = {{&S669[8], 3}, {NULL}};
const Segment SGI301477[] = {{&S669[13], 10}, {NULL}};
const Segment SGI301478[] = {{&S669[42], 7}, {&S669[159], 30}, {NULL}};
const Segment SGI301479[] = {{&S669[35], 14}, {NULL}};
const Segment SGI301480[] = {{&S669[131], 15}, {NULL}};
const Segment SGI301481[] = {{&S670[30], 3}, {NULL}};
const Segment SGI301482[] = {{&S670[8], 3}, {NULL}};
const Segment SGI301483[] = {{&S670[13], 10}, {NULL}};
const Segment SGI301484[] = {{&S670[42], 7}, {&S670[167], 30}, {NULL}};
const Segment SGI301485[] = {{&S670[35], 14}, {NULL}};
const Segment SGI301486[] = {{&S671[30], 3}, {&S671[94], 3}, {NULL}};
const Segment SGI301487[] = {{&S671[8], 3}, {&S671[8], 3}, {NULL}};
const Segment SGI301488[] = {{&S671[13], 10}, {&S671[13], 10}, {NULL}};
const Segment SGI301489[] = {{&S671[42], 7}, {&S671[106], 7}, {NULL}};
const Segment SGI301490[] = {{&S671[35], 14}, {NULL}};
const Segment SGI301491[] = {{&S671[214], 9}, {NULL}};
const Segment SGI301492[] = {{&S672[30], 3}, {NULL}};
const Segment SGI301493[] = {{&S672[8], 3}, {NULL}};
const Segment SGI301494[] = {{&S672[13], 10}, {NULL}};
const Segment SGI301495[] = {{&S672[42], 7}, {&S672[179], 30}, {NULL}};
const Segment SGI301496[] = {{&S672[35], 14}, {NULL}};
const Segment SGI301497[] = {{&S672[131], 15}, {NULL}};
const Segment SGI301498[] = {{&S672[162], 4}, {NULL}};
const Segment SGI301499[] = {{&S673[30], 3}, {NULL}};
const Segment SGI301500[] = {{&S673[8], 3}, {NULL}};
const Segment SGI301501[] = {{&S673[13], 10}, {NULL}};
const Segment SGI301502[] = {{&S673[42], 7}, {&S673[179], 30}, {NULL}};
const Segment SGI301503[] = {{&S673[35], 14}, {NULL}};
const Segment SGI301504[] = {{&S673[131], 15}, {NULL}};
const Segment SGI301505[] = {{&S674[30], 3}, {NULL}};
const Segment SGI301506[] = {{&S674[8], 3}, {NULL}};
const Segment SGI301507[] = {{&S674[13], 10}, {NULL}};
const Segment SGI301508[] = {{&S674[42], 7}, {NULL}};
const Segment SGI301509[] = {{&S674[35], 14}, {NULL}};
const Segment SGI301510[] = {{&S674[151], 4}, {NULL}};
const Segment SGI301511[] = {{&S674[174], 9}, {NULL}};
const Segment SGI301512[] = {{&S675[30], 3}, {NULL}};
const Segment SGI301513[] = {{&S675[8], 3}, {NULL}};
const Segment SGI301514[] = {{&S675[13], 10}, {NULL}};
const Segment SGI301515[] = {{&S675[42], 7}, {&S675[165], 30}, {NULL}};
const Segment SGI301516[] = {{&S675[35], 14}, {NULL}};
const Segment SGI301517[] = {{&S675[51], 6}, {NULL}};
const Segment SGI301518[] = {{&S676[30], 3}, {NULL}};
const Segment SGI301519[] = {{&S676[8], 3}, {NULL}};
const Segment SGI301520[] = {{&S676[13], 10}, {NULL}};
const Segment SGI301521[] = {{&S676[42], 7}, {NULL}};
const Segment SGI301522[] = {{&S676[35], 14}, {NULL}};
const Segment SGI301523[] = {{&S676[51], 5}, {NULL}};
const Segment SGI301524[] = {{&S677[21], 3}, {NULL}};
const Segment SGI301525[] = {{&S677[8], 3}, {NULL}};
const Segment SGI301526[] = {{&S677[11], 1}, {NULL}};
const Segment SGI301527[] = {{&S677[33], 7}, {NULL}};
const Segment SGI301528[] = {{&S677[26], 14}, {NULL}};
const Segment SGI301529[] = {{&S677[129], 15}, {&S677[129], 15}, {NULL}};
const Segment SGI301530[] = {{&S677[42], 6}, {NULL}};
const Segment SGI301531[] = {{&S678[30], 3}, {NULL}};
const Segment SGI301532[] = {{&S678[8], 3}, {NULL}};
const Segment SGI301533[] = {{&S678[13], 10}, {NULL}};
const Segment SGI301534[] = {{&S678[42], 7}, {NULL}};
const Segment SGI301535[] = {{&S678[35], 14}, {NULL}};
const Segment SGI301536[] = {{&S678[131], 15}, {NULL}};
const Segment SGI301537[] = {{&S678[162], 3}, {NULL}};
const Segment SGI301538[] = {{&S679[30], 3}, {NULL}};
const Segment SGI301539[] = {{&S679[8], 3}, {NULL}};
const Segment SGI301540[] = {{&S679[13], 10}, {NULL}};
const Segment SGI301541[] = {{&S679[42], 7}, {NULL}};
const Segment SGI301542[] = {{&S679[35], 14}, {NULL}};
const Segment SGI301543[] = {{&S679[148], 15}, {NULL}};
const Segment SGI301544[] = {{&S679[51], 5}, {NULL}};
const Segment SGI301545[] = {{&S679[76], 3}, {NULL}};
const Segment SGI301546[] = {{&S680[30], 3}, {NULL}};
const Segment SGI301547[] = {{&S680[8], 3}, {NULL}};
const Segment SGI301548[] = {{&S680[13], 10}, {NULL}};
const Segment SGI301549[] = {{&S680[42], 7}, {&S680[196], 30}, {NULL}};
const Segment SGI301550[] = {{&S680[35], 14}, {NULL}};
const Segment SGI301551[] = {{&S680[138], 15}, {NULL}};
const Segment SGI301552[] = {{&S680[179], 4}, {NULL}};
const Segment SGI301553[] = {{&S680[51], 6}, {NULL}};
const Segment SGI301554[] = {{&S681[30], 3}, {NULL}};
const Segment SGI301555[] = {{&S681[8], 3}, {NULL}};
const Segment SGI301556[] = {{&S681[13], 10}, {NULL}};
const Segment SGI301557[] = {{&S681[42], 7}, {&S681[194], 30}, {NULL}};
const Segment SGI301558[] = {{&S681[35], 14}, {NULL}};
const Segment SGI301559[] = {{&S681[131], 15}, {NULL}};
const Segment SGI301560[] = {{&S681[172], 9}, {NULL}};
const Segment SGI301561[] = {{&S682[30], 3}, {NULL}};
const Segment SGI301562[] = {{&S682[8], 3}, {NULL}};
const Segment SGI301563[] = {{&S682[13], 10}, {NULL}};
const Segment SGI301564[] = {{&S682[42], 7}, {NULL}};
const Segment SGI301565[] = {{&S682[35], 14}, {NULL}};
const Segment SGI301566[] = {{&S682[163], 4}, {NULL}};
const Segment SGI301567[] = {{&S682[51], 5}, {NULL}};
const Segment SGI301568[] = {{&S683[30], 3}, {NULL}};
const Segment SGI301569[] = {{&S683[8], 3}, {NULL}};
const Segment SGI301570[] = {{&S683[13], 10}, {NULL}};
const Segment SGI301571[] = {{&S683[42], 7}, {NULL}};
const Segment SGI301572[] = {{&S683[35], 14}, {NULL}};
const Segment SGI301573[] = {{&S683[138], 15}, {NULL}};
const Segment SGI301574[] = {{&S683[51], 6}, {NULL}};
const Segment SGI301575[] = {{&S683[179], 3}, {NULL}};
const Segment SGI301576[] = {{&S684[30], 3}, {NULL}};
const Segment SGI301577[] = {{&S684[8], 3}, {NULL}};
const Segment SGI301578[] = {{&S684[13], 10}, {NULL}};
const Segment SGI301579[] = {{&S684[42], 7}, {NULL}};
const Segment SGI301580[] = {{&S684[35], 14}, {NULL}};
const Segment SGI301581[] = {{&S684[155], 15}, {NULL}};
const Segment SGI301582[] = {{&S684[51], 6}, {NULL}};
const Segment SGI301583[] = {{&S684[188], 9}, {NULL}};
const Segment SGI301584[] = {{&S685[31], 3}, {NULL}};
const Segment SGI301585[] = {{&S685[8], 3}, {NULL}};
const Segment SGI301586[] = {{&S685[43], 7}, {NULL}};
const Segment SGI301587[] = {{&S685[36], 14}, {NULL}};
const Segment SGI301588[] = {{&S686[30], 3}, {NULL}};
const Segment SGI301589[] = {{&S686[8], 3}, {NULL}};
const Segment SGI301590[] = {{&S686[13], 10}, {NULL}};
const Segment SGI301591[] = {{&S686[42], 7}, {NULL}};
const Segment SGI301592[] = {{&S686[35], 14}, {NULL}};
const Segment SGI301593[] = {{&S686[164], 3}, {NULL}};
const Segment SGI301594[] = {{&S687[30], 3}, {NULL}};
const Segment SGI301595[] = {{&S687[8], 3}, {&S687[8], 3}, {&S687[8], 3}, {NULL}};
const Segment SGI301596[] = {{&S687[13], 10}, {NULL}};
const Segment SGI301597[] = {{&S687[42], 7}, {NULL}};
const Segment SGI301598[] = {{&S687[35], 14}, {NULL}};
const Segment SGI301599[] = {{&S688[30], 3}, {NULL}};
const Segment SGI301600[] = {{&S688[8], 3}, {NULL}};
const Segment SGI301601[] = {{&S688[13], 10}, {NULL}};
const Segment SGI301602[] = {{&S688[42], 7}, {&S688[130], 20}, {NULL}};
const Segment SGI301603[] = {{&S688[35], 14}, {NULL}};
const Segment SGI301604[] = {{&S688[131], 15}, {&S688[131], 15}, {NULL}};
const Segment SGI301605[] = {{&S689[30], 3}, {NULL}};
const Segment SGI301606[] = {{&S689[8], 3}, {NULL}};
const Segment SGI301607[] = {{&S689[13], 10}, {NULL}};
const Segment SGI301608[] = {{&S689[42], 7}, {NULL}};
const Segment SGI301609[] = {{&S689[35], 14}, {NULL}};
const Segment SGI301610[] = {{&S689[131], 15}, {&S689[131], 15}, {NULL}};
const Segment SGI301611[] = {{&S689[190], 4}, {NULL}};
const Segment SGI301612[] = {{&S691[30], 3}, {NULL}};
const Segment SGI301613[] = {{&S691[8], 3}, {NULL}};
const Segment SGI301614[] = {{&S691[13], 10}, {NULL}};
const Segment SGI301615[] = {{&S691[42], 7}, {NULL}};
const Segment SGI301616[] = {{&S691[35], 14}, {NULL}};
const Segment SGI301617[] = {{&S692[30], 3}, {NULL}};
const Segment SGI301618[] = {{&S692[8], 3}, {NULL}};
const Segment SGI301619[] = {{&S692[13], 10}, {NULL}};
const Segment SGI301620[] = {{&S692[42], 7}, {NULL}};
const Segment SGI301621[] = {{&S692[35], 14}, {NULL}};
const Segment SGI301622[] = {{&S692[96], 15}, {NULL}};
const Segment SGI301623[] = {{&S692[64], 3}, {NULL}};
const Segment SGI301624[] = {{&S693[31], 3}, {NULL}};
const Segment SGI301625[] = {{&S693[43], 7}, {NULL}};
const Segment SGI301626[] = {{&S694[30], 3}, {NULL}};
const Segment SGI301627[] = {{&S694[8], 3}, {NULL}};
const Segment SGI301628[] = {{&S694[13], 10}, {NULL}};
const Segment SGI301629[] = {{&S694[42], 7}, {NULL}};
const Segment SGI301630[] = {{&S695[30], 3}, {NULL}};
const Segment SGI301631[] = {{&S695[8], 3}, {NULL}};
const Segment SGI301632[] = {{&S695[13], 10}, {NULL}};
const Segment SGI301633[] = {{&S695[42], 7}, {NULL}};
const Segment SGI301634[] = {{&S695[35], 14}, {NULL}};
const Segment SGI301635[] = {{&S695[91], 15}, {NULL}};
const Segment SGI301636[] = {{&S696[30], 3}, {NULL}};
const Segment SGI301637[] = {{&S696[8], 3}, {NULL}};
const Segment SGI301638[] = {{&S696[13], 10}, {NULL}};
const Segment SGI301639[] = {{&S696[42], 7}, {NULL}};
const Segment SGI301640[] = {{&S696[35], 14}, {NULL}};
const Segment SGI301641[] = {{&S696[103], 15}, {NULL}};
const Segment SGI301642[] = {{&S696[51], 5}, {NULL}};
const Segment SGI301643[] = {{&S697[30], 3}, {NULL}};
const Segment SGI301644[] = {{&S697[8], 3}, {NULL}};
const Segment SGI301645[] = {{&S697[13], 10}, {NULL}};
const Segment SGI301646[] = {{&S697[42], 7}, {NULL}};
const Segment SGI301647[] = {{&S697[35], 14}, {NULL}};
const Segment SGI301648[] = {{&S697[51], 6}, {NULL}};
const Segment SGI301649[] = {{&S698[30], 3}, {NULL}};
const Segment SGI301650[] = {{&S698[8], 3}, {NULL}};
const Segment SGI301651[] = {{&S698[13], 10}, {NULL}};
const Segment SGI301652[] = {{&S698[42], 7}, {NULL}};
const Segment SGI301653[] = {{&S698[35], 14}, {NULL}};
const Segment SGI301654[] = {{&S698[91], 15}, {NULL}};
const Segment SGI301655[] = {{&S698[112], 9}, {NULL}};
const Segment SGI301656[] = {{&S699[30], 3}, {&S699[46], 3}, {NULL}};
const Segment SGI301657[] = {{&S699[8], 3}, {&S699[8], 3}, {NULL}};
const Segment SGI301658[] = {{&S699[13], 10}, {&S699[13], 10}, {NULL}};
const Segment SGI301659[] = {{&S699[42], 7}, {&S699[113], 7}, {NULL}};
const Segment SGI301660[] = {{&S699[35], 14}, {&S699[106], 14}, {NULL}};
const Segment SGI301661[] = {{&S699[156], 15}, {NULL}};
const Segment SGI301662[] = {{&S699[51], 6}, {NULL}};
const Segment SGI301663[] = {{&S699[122], 3}, {NULL}};
const Segment SGI301664[] = {{&S700[30], 3}, {NULL}};
const Segment SGI301665[] = {{&S700[8], 3}, {NULL}};
const Segment SGI301666[] = {{&S700[13], 10}, {NULL}};
const Segment SGI301667[] = {{&S700[42], 7}, {&S700[177], 5}, {NULL}};
const Segment SGI301668[] = {{&S700[35], 14}, {NULL}};
const Segment SGI301669[] = {{&S700[91], 15}, {NULL}};
const Segment SGI301670[] = {{&S701[30], 3}, {&S701[46], 3}, {NULL}};
const Segment SGI301671[] = {{&S701[8], 3}, {&S701[8], 3}, {NULL}};
const Segment SGI301672[] = {{&S701[13], 10}, {&S701[13], 10}, {NULL}};
const Segment SGI301673[] = {{&S701[42], 7}, {&S701[106], 7}, {NULL}};
const Segment SGI301674[] = {{&S701[35], 14}, {&S701[99], 14}, {NULL}};
const Segment SGI301675[] = {{&S701[115], 3}, {NULL}};
const Segment SGI301676[] = {{&S702[30], 3}, {NULL}};
const Segment SGI301677[] = {{&S702[8], 3}, {NULL}};
const Segment SGI301678[] = {{&S702[13], 10}, {NULL}};
const Segment SGI301679[] = {{&S702[42], 7}, {NULL}};
const Segment SGI301680[] = {{&S702[35], 14}, {NULL}};
const Segment SGI301681[] = {{&S703[30], 3}, {NULL}};
const Segment SGI301682[] = {{&S703[8], 3}, {NULL}};
const Segment SGI301683[] = {{&S703[13], 10}, {NULL}};
const Segment SGI301684[] = {{&S703[42], 7}, {NULL}};
const Segment SGI301685[] = {{&S703[35], 14}, {NULL}};
const Segment SGI301686[] = {{&S703[98], 15}, {NULL}};
const Segment SGI301687[] = {{&S703[169], 4}, {NULL}};
const Segment SGI301688[] = {{&S703[51], 6}, {NULL}};
const Segment SGI301689[] = {{&S704[72], 14}, {NULL}};
const Segment SGI301690[] = {{&S704[8], 3}, {NULL}};
const Segment SGI301691[] = {{&S704[13], 10}, {NULL}};
const Segment SGI301692[] = {{&S704[61], 4}, {NULL}};
const Segment SGI301693[] = {{&S705[30], 3}, {NULL}};
const Segment SGI301694[] = {{&S705[8], 3}, {NULL}};
const Segment SGI301695[] = {{&S705[13], 10}, {NULL}};
const Segment SGI301696[] = {{&S705[42], 7}, {NULL}};
const Segment SGI301697[] = {{&S705[35], 14}, {NULL}};
const Segment SGI301698[] = {{&S705[91], 15}, {NULL}};
const Segment SGI301699[] = {{&S705[162], 4}, {NULL}};
const Segment SGI301700[] = {{&S706[30], 3}, {NULL}};
const Segment SGI301701[] = {{&S706[8], 3}, {NULL}};
const Segment SGI301702[] = {{&S706[13], 10}, {NULL}};
const Segment SGI301703[] = {{&S706[42], 7}, {NULL}};
const Segment SGI301704[] = {{&S706[35], 14}, {NULL}};
const Segment SGI301705[] = {{&S706[141], 4}, {NULL}};
const Segment SGI301706[] = {{&S707[30], 3}, {NULL}};
const Segment SGI301707[] = {{&S707[8], 3}, {NULL}};
const Segment SGI301708[] = {{&S707[13], 10}, {NULL}};
const Segment SGI301709[] = {{&S707[42], 7}, {NULL}};
const Segment SGI301710[] = {{&S707[35], 14}, {NULL}};
const Segment SGI301711[] = {{&S707[98], 15}, {&S707[98], 15}, {NULL}};
const Segment SGI301712[] = {{&S707[190], 4}, {NULL}};
const Segment SGI301713[] = {{&S707[51], 6}, {NULL}};
const Segment SGI301714[] = {{&S708[8], 3}, {NULL}};
const Segment SGI301715[] = {{&S708[13], 10}, {NULL}};
const Segment SGI301716[] = {{&S708[41], 7}, {NULL}};
const Segment SGI301717[] = {{&S709[30], 3}, {NULL}};
const Segment SGI301718[] = {{&S709[8], 3}, {NULL}};
const Segment SGI301719[] = {{&S709[13], 10}, {NULL}};
const Segment SGI301720[] = {{&S709[42], 7}, {NULL}};
const Segment SGI301721[] = {{&S709[35], 14}, {NULL}};
const Segment SGI301722[] = {{&S709[76], 3}, {NULL}};
const Segment SGI301723[] = {{&S710[72], 14}, {NULL}};
const Segment SGI301724[] = {{&S710[8], 3}, {NULL}};
const Segment SGI301725[] = {{&S710[13], 10}, {NULL}};
const Segment SGI301726[] = {{&S710[61], 4}, {NULL}};
const Segment SGI301727[] = {{&S711[30], 3}, {NULL}};
const Segment SGI301728[] = {{&S711[8], 3}, {NULL}};
const Segment SGI301729[] = {{&S711[13], 10}, {NULL}};
const Segment SGI301730[] = {{&S711[42], 7}, {NULL}};
const Segment SGI301731[] = {{&S711[35], 14}, {NULL}};
const Segment SGI301732[] = {{&S711[51], 5}, {NULL}};
const Segment SGI301733[] = {{&S712[30], 3}, {&S712[30], 3}, {NULL}};
const Segment SGI301734[] = {{&S712[8], 3}, {&S712[8], 3}, {NULL}};
const Segment SGI301735[] = {{&S712[13], 10}, {&S712[13], 10}, {NULL}};
const Segment SGI301736[] = {{&S712[42], 7}, {NULL}};
const Segment SGI301737[] = {{&S712[35], 14}, {NULL}};
const Segment SGI301738[] = {{&S712[142], 15}, {NULL}};
const Segment SGI301739[] = {{&S712[99], 5}, {NULL}};
const Segment SGI301740[] = {{&S713[30], 3}, {NULL}};
const Segment SGI301741[] = {{&S713[8], 3}, {NULL}};
const Segment SGI301742[] = {{&S713[13], 10}, {NULL}};
const Segment SGI301743[] = {{&S713[42], 7}, {NULL}};
const Segment SGI301744[] = {{&S713[35], 14}, {NULL}};
const Segment SGI301745[] = {{&S713[103], 15}, {NULL}};
const Segment SGI301746[] = {{&S713[174], 4}, {NULL}};
const Segment SGI301747[] = {{&S713[51], 5}, {NULL}};
const Segment SGI301748[] = {{&S714[72], 14}, {NULL}};
const Segment SGI301749[] = {{&S714[8], 3}, {NULL}};
const Segment SGI301750[] = {{&S714[13], 10}, {NULL}};
const Segment SGI301751[] = {{&S714[61], 4}, {NULL}};
const Segment SGI301752[] = {{&S715[72], 14}, {NULL}};
const Segment SGI301753[] = {{&S715[8], 3}, {NULL}};
const Segment SGI301754[] = {{&S715[13], 10}, {NULL}};
const Segment SGI301755[] = {{&S715[61], 4}, {NULL}};
const Segment SGI301756[] = {{&S716[30], 3}, {NULL}};
const Segment SGI301757[] = {{&S716[8], 3}, {NULL}};
const Segment SGI301758[] = {{&S716[13], 10}, {NULL}};
const Segment SGI301759[] = {{&S716[42], 7}, {NULL}};
const Segment SGI301760[] = {{&S716[35], 14}, {NULL}};
const Segment SGI301761[] = {{&S716[64], 6}, {NULL}};
const Segment SGI301762[] = {{&S717[27], 3}, {NULL}};
const Segment SGI301763[] = {{&S717[8], 3}, {NULL}};
const Segment SGI301764[] = {{&S717[13], 7}, {NULL}};
const Segment SGI301765[] = {{&S717[39], 7}, {NULL}};
const Segment SGI301766[] = {{&S718[72], 14}, {NULL}};
const Segment SGI301767[] = {{&S718[8], 3}, {NULL}};
const Segment SGI301768[] = {{&S718[13], 10}, {NULL}};
const Segment SGI301769[] = {{&S718[61], 4}, {NULL}};
const Segment SGI301770[] = {{&S719[30], 3}, {&S719[46], 3}, {NULL}};
const Segment SGI301771[] = {{&S719[8], 3}, {&S719[8], 3}, {NULL}};
const Segment SGI301772[] = {{&S719[13], 10}, {&S719[13], 10}, {NULL}};
const Segment SGI301773[] = {{&S719[42], 7}, {&S719[111], 7}, {NULL}};
const Segment SGI301774[] = {{&S719[35], 14}, {&S719[104], 14}, {NULL}};
const Segment SGI301775[] = {{&S719[154], 15}, {NULL}};
const Segment SGI301776[] = {{&S719[64], 3}, {&S719[64], 3}, {NULL}};
const Segment SGI301777[] = {{&S720[30], 3}, {NULL}};
const Segment SGI301778[] = {{&S720[8], 3}, {NULL}};
const Segment SGI301779[] = {{&S720[13], 10}, {NULL}};
const Segment SGI301780[] = {{&S720[42], 7}, {NULL}};
const Segment SGI301781[] = {{&S720[35], 14}, {NULL}};
const Segment SGI301782[] = {{&S720[154], 4}, {NULL}};
const Segment SGI301783[] = {{&S720[51], 6}, {NULL}};
const Segment SGI301784[] = {{&S721[30], 3}, {&S721[46], 3}, {NULL}};
const Segment SGI301785[] = {{&S721[8], 3}, {&S721[8], 3}, {NULL}};
const Segment SGI301786[] = {{&S721[13], 10}, {&S721[13], 10}, {NULL}};
const Segment SGI301787[] = {{&S721[42], 7}, {&S721[113], 7}, {NULL}};
const Segment SGI301788[] = {{&S721[35], 14}, {&S721[106], 14}, {NULL}};
const Segment SGI301789[] = {{&S721[156], 15}, {NULL}};
const Segment SGI301790[] = {{&S721[227], 4}, {NULL}};
const Segment SGI301791[] = {{&S721[51], 6}, {NULL}};
const Segment SGI301792[] = {{&S721[122], 3}, {NULL}};
const Segment SGI301793[] = {{&S722[30], 3}, {NULL}};
const Segment SGI301794[] = {{&S722[8], 3}, {NULL}};
const Segment SGI301795[] = {{&S722[13], 10}, {NULL}};
const Segment SGI301796[] = {{&S722[42], 7}, {NULL}};
const Segment SGI301797[] = {{&S722[35], 14}, {NULL}};
const Segment SGI301798[] = {{&S722[51], 5}, {NULL}};
const Segment SGI301799[] = {{&S722[103], 9}, {NULL}};
const Segment SGI301800[] = {{&S723[30], 3}, {&S723[46], 3}, {NULL}};
const Segment SGI301801[] = {{&S723[8], 3}, {&S723[8], 3}, {NULL}};
const Segment SGI301802[] = {{&S723[13], 10}, {&S723[13], 10}, {NULL}};
const Segment SGI301803[] = {{&S723[42], 7}, {&S723[106], 7}, {NULL}};
const Segment SGI301804[] = {{&S723[35], 14}, {&S723[99], 14}, {NULL}};
const Segment SGI301805[] = {{&S723[201], 4}, {NULL}};
const Segment SGI301806[] = {{&S723[115], 3}, {NULL}};
const Segment SGI301807[] = {{&S724[72], 14}, {NULL}};
const Segment SGI301808[] = {{&S724[8], 3}, {NULL}};
const Segment SGI301809[] = {{&S724[13], 10}, {NULL}};
const Segment SGI301810[] = {{&S724[61], 4}, {NULL}};
const Segment SGI301811[] = {{&S725[30], 3}, {NULL}};
const Segment SGI301812[] = {{&S725[8], 3}, {NULL}};
const Segment SGI301813[] = {{&S725[13], 10}, {NULL}};
const Segment SGI301814[] = {{&S725[42], 7}, {NULL}};
const Segment SGI301815[] = {{&S726[30], 3}, {&S726[130], 3}, {NULL}};
const Segment SGI301816[] = {{&S726[8], 3}, {&S726[108], 3}, {NULL}};
const Segment SGI301817[] = {{&S726[13], 10}, {&S726[35], 7}, {NULL}};
const Segment SGI301818[] = {{&S726[42], 7}, {&S726[142], 7}, {NULL}};
const Segment SGI301819[] = {{&S726[35], 14}, {&S726[135], 14}, {NULL}};
const Segment SGI301820[] = {{&S726[64], 3}, {NULL}};
const Segment SGI301821[] = {{&S727[30], 3}, {NULL}};
const Segment SGI301822[] = {{&S727[8], 3}, {NULL}};
const Segment SGI301823[] = {{&S727[13], 10}, {NULL}};
const Segment SGI301824[] = {{&S727[42], 7}, {NULL}};
const Segment SGI301825[] = {{&S727[35], 14}, {NULL}};
const Segment SGI301826[] = {{&S727[111], 9}, {NULL}};
const Segment SGI301827[] = {{&S729[27], 3}, {NULL}};
const Segment SGI301828[] = {{&S729[8], 3}, {NULL}};
const Segment SGI301829[] = {{&S729[13], 7}, {NULL}};
const Segment SGI301830[] = {{&S729[39], 7}, {NULL}};
const Segment SGI301831[] = {{&S729[32], 14}, {NULL}};
const Segment SGI301832[] = {{&S730[17], 3}, {NULL}};
const Segment SGI301833[] = {{&S730[29], 7}, {NULL}};
const Segment SGI301834[] = {{&S730[22], 14}, {NULL}};
const Segment SGI301835[] = {{&S731[30], 3}, {&S731[76], 3}, {NULL}};
const Segment SGI301836[] = {{&S731[8], 3}, {NULL}};
const Segment SGI301837[] = {{&S731[13], 10}, {NULL}};
const Segment SGI301838[] = {{&S731[42], 7}, {NULL}};
const Segment SGI301839[] = {{&S731[35], 14}, {NULL}};
const Segment SGI301840[] = {{&S731[51], 6}, {NULL}};
const Segment SGI301841[] = {{&S732[30], 3}, {&S732[30], 3}, {NULL}};
const Segment SGI301842[] = {{&S732[8], 3}, {NULL}};
const Segment SGI301843[] = {{&S732[13], 10}, {NULL}};
const Segment SGI301844[] = {{&S732[42], 7}, {NULL}};
const Segment SGI301845[] = {{&S732[35], 14}, {NULL}};
const Segment SGI301846[] = {{&S733[30], 3}, {&S733[46], 3}, {NULL}};
const Segment SGI301847[] = {{&S733[8], 3}, {&S733[8], 3}, {NULL}};
const Segment SGI301848[] = {{&S733[13], 10}, {&S733[13], 10}, {NULL}};
const Segment SGI301849[] = {{&S733[42], 7}, {&S733[106], 7}, {NULL}};
const Segment SGI301850[] = {{&S733[35], 14}, {&S733[99], 14}, {NULL}};
const Segment SGI301851[] = {{&S733[149], 15}, {NULL}};
const Segment SGI301852[] = {{&S733[220], 4}, {NULL}};
const Segment SGI301853[] = {{&S733[115], 3}, {NULL}};
const Segment SGI301854[] = {{&S734[71], 14}, {NULL}};
const Segment SGI301855[] = {{&S734[8], 3}, {NULL}};
const Segment SGI301856[] = {{&S734[13], 10}, {NULL}};
const Segment SGI301857[] = {{&S734[60], 4}, {NULL}};
const Segment SGI301858[] = {{&S735[30], 3}, {NULL}};
const Segment SGI301859[] = {{&S735[8], 3}, {NULL}};
const Segment SGI301860[] = {{&S735[13], 10}, {NULL}};
const Segment SGI301861[] = {{&S735[42], 7}, {NULL}};
const Segment SGI301862[] = {{&S735[35], 14}, {NULL}};
const Segment SGI301863[] = {{&S735[101], 6}, {NULL}};
const Segment SGI301864[] = {{&S735[76], 3}, {NULL}};
const Segment SGI301865[] = {{&S736[71], 14}, {NULL}};
const Segment SGI301866[] = {{&S736[8], 3}, {NULL}};
const Segment SGI301867[] = {{&S736[13], 10}, {NULL}};
const Segment SGI301868[] = {{&S736[60], 4}, {NULL}};
const Segment SGI301869[] = {{&S737[30], 3}, {NULL}};
const Segment SGI301870[] = {{&S737[8], 3}, {NULL}};
const Segment SGI301871[] = {{&S737[13], 10}, {NULL}};
const Segment SGI301872[] = {{&S737[42], 7}, {NULL}};
const Segment SGI301873[] = {{&S737[35], 14}, {NULL}};
const Segment SGI301874[] = {{&S737[91], 15}, {NULL}};
const Segment SGI301875[] = {{&S737[112], 4}, {NULL}};
const Segment SGI301876[] = {{&S737[122], 9}, {NULL}};
const Segment SGI301877[] = {{&S738[30], 3}, {NULL}};
const Segment SGI301878[] = {{&S738[8], 3}, {NULL}};
const Segment SGI301879[] = {{&S738[13], 10}, {NULL}};
const Segment SGI301880[] = {{&S738[42], 7}, {NULL}};
const Segment SGI301881[] = {{&S738[35], 14}, {NULL}};
const Segment SGI301882[] = {{&S738[106], 15}, {NULL}};
const Segment SGI301883[] = {{&S738[51], 6}, {NULL}};
const Segment SGI301884[] = {{&S739[30], 3}, {NULL}};
const Segment SGI301885[] = {{&S739[8], 3}, {NULL}};
const Segment SGI301886[] = {{&S739[13], 10}, {NULL}};
const Segment SGI301887[] = {{&S739[42], 7}, {NULL}};
const Segment SGI301888[] = {{&S739[35], 14}, {NULL}};
const Segment SGI301889[] = {{&S739[106], 15}, {&S739[106], 15}, {NULL}};
const Segment SGI301890[] = {{&S739[51], 6}, {NULL}};
const Segment SGI301891[] = {{&S740[30], 3}, {&S740[46], 3}, {NULL}};
const Segment SGI301892[] = {{&S740[8], 3}, {&S740[8], 3}, {NULL}};
const Segment SGI301893[] = {{&S740[13], 10}, {&S740[13], 10}, {NULL}};
const Segment SGI301894[] = {{&S740[42], 7}, {&S740[114], 7}, {NULL}};
const Segment SGI301895[] = {{&S740[35], 14}, {&S740[107], 14}, {NULL}};
const Segment SGI301896[] = {{&S740[157], 15}, {NULL}};
const Segment SGI301897[] = {{&S740[123], 3}, {NULL}};
const Segment SGI301898[] = {{&S741[30], 3}, {NULL}};
const Segment SGI301899[] = {{&S741[8], 3}, {NULL}};
const Segment SGI301900[] = {{&S741[13], 10}, {NULL}};
const Segment SGI301901[] = {{&S741[55], 7}, {NULL}};
const Segment SGI301902[] = {{&S741[48], 14}, {NULL}};
const Segment SGI301903[] = {{&S741[64], 3}, {NULL}};
const Segment SGI301904[] = {{&S742[30], 3}, {&S742[46], 3}, {NULL}};
const Segment SGI301905[] = {{&S742[8], 3}, {&S742[8], 3}, {NULL}};
const Segment SGI301906[] = {{&S742[13], 10}, {&S742[13], 10}, {NULL}};
const Segment SGI301907[] = {{&S742[42], 7}, {&S742[126], 7}, {NULL}};
const Segment SGI301908[] = {{&S742[35], 14}, {&S742[119], 14}, {NULL}};
const Segment SGI301909[] = {{&S742[169], 15}, {NULL}};
const Segment SGI301910[] = {{&S742[51], 5}, {NULL}};
const Segment SGI301911[] = {{&S742[135], 3}, {NULL}};
const Segment SGI301912[] = {{&S743[30], 3}, {&S743[46], 3}, {NULL}};
const Segment SGI301913[] = {{&S743[8], 3}, {&S743[8], 3}, {NULL}};
const Segment SGI301914[] = {{&S743[13], 10}, {&S743[13], 10}, {NULL}};
const Segment SGI301915[] = {{&S743[42], 7}, {&S743[121], 7}, {NULL}};
const Segment SGI301916[] = {{&S743[35], 14}, {&S743[114], 14}, {NULL}};
const Segment SGI301917[] = {{&S743[164], 15}, {&S743[164], 15}, {NULL}};
const Segment SGI301918[] = {{&S743[51], 6}, {NULL}};
const Segment SGI301919[] = {{&S743[130], 3}, {NULL}};
const Segment SGI301920[] = {{&S744[30], 3}, {&S744[46], 3}, {NULL}};
const Segment SGI301921[] = {{&S744[8], 3}, {&S744[8], 3}, {NULL}};
const Segment SGI301922[] = {{&S744[13], 10}, {&S744[13], 10}, {NULL}};
const Segment SGI301923[] = {{&S744[42], 7}, {&S744[121], 7}, {NULL}};
const Segment SGI301924[] = {{&S744[35], 14}, {&S744[114], 14}, {NULL}};
const Segment SGI301925[] = {{&S744[51], 6}, {NULL}};
const Segment SGI301926[] = {{&S744[130], 3}, {NULL}};
const Segment SGI301927[] = {{&S745[30], 3}, {NULL}};
const Segment SGI301928[] = {{&S745[8], 3}, {NULL}};
const Segment SGI301929[] = {{&S745[13], 10}, {NULL}};
const Segment SGI301930[] = {{&S745[42], 7}, {NULL}};
const Segment SGI301931[] = {{&S745[35], 14}, {NULL}};
const Segment SGI301932[] = {{&S745[99], 15}, {NULL}};
const Segment SGI301933[] = {{&S745[175], 3}, {NULL}};
const Segment SGI301934[] = {{&S746[30], 3}, {NULL}};
const Segment SGI301935[] = {{&S746[8], 3}, {NULL}};
const Segment SGI301936[] = {{&S746[13], 10}, {NULL}};
const Segment SGI301937[] = {{&S746[42], 7}, {NULL}};
const Segment SGI301938[] = {{&S746[35], 14}, {NULL}};
const Segment SGI301939[] = {{&S746[111], 15}, {NULL}};
const Segment SGI301940[] = {{&S746[51], 5}, {NULL}};
const Segment SGI301941[] = {{&S746[132], 9}, {NULL}};
const Segment SGI301942[] = {{&S747[72], 14}, {NULL}};
const Segment SGI301943[] = {{&S747[8], 3}, {NULL}};
const Segment SGI301944[] = {{&S747[13], 10}, {NULL}};
const Segment SGI301945[] = {{&S747[61], 4}, {NULL}};
const Segment SGI301946[] = {{&S748[72], 14}, {NULL}};
const Segment SGI301947[] = {{&S748[8], 3}, {NULL}};
const Segment SGI301948[] = {{&S748[13], 10}, {NULL}};
const Segment SGI301949[] = {{&S748[61], 4}, {NULL}};
const Segment SGI301950[] = {{&S749[72], 14}, {NULL}};
const Segment SGI301951[] = {{&S749[8], 3}, {NULL}};
const Segment SGI301952[] = {{&S749[13], 10}, {NULL}};
const Segment SGI301953[] = {{&S749[61], 4}, {NULL}};
const Segment SGI301954[] = {{&S750[30], 3}, {NULL}};
const Segment SGI301955[] = {{&S750[8], 3}, {NULL}};
const Segment SGI301956[] = {{&S750[13], 10}, {NULL}};
const Segment SGI301957[] = {{&S750[42], 7}, {&S750[206], 14}, {NULL}};
const Segment SGI301958[] = {{&S750[35], 14}, {NULL}};
const Segment SGI301959[] = {{&S750[51], 6}, {NULL}};
const Segment SGI301960[] = {{&S751[30], 3}, {&S751[46], 3}, {NULL}};
const Segment SGI301961[] = {{&S751[8], 3}, {&S751[8], 3}, {NULL}};
const Segment SGI301962[] = {{&S751[13], 10}, {&S751[13], 10}, {NULL}};
const Segment SGI301963[] = {{&S751[42], 7}, {&S751[114], 7}, {NULL}};
const Segment SGI301964[] = {{&S751[35], 14}, {&S751[107], 14}, {NULL}};
const Segment SGI301965[] = {{&S751[157], 15}, {NULL}};
const Segment SGI301966[] = {{&S751[178], 9}, {NULL}};
const Segment SGI301967[] = {{&S751[123], 3}, {NULL}};
const Segment SGI301968[] = {{&S752[30], 3}, {&S752[46], 3}, {NULL}};
const Segment SGI301969[] = {{&S752[8], 3}, {&S752[8], 3}, {NULL}};
const Segment SGI301970[] = {{&S752[13], 10}, {&S752[13], 10}, {NULL}};
const Segment SGI301971[] = {{&S752[42], 7}, {&S752[98], 7}, {NULL}};
const Segment SGI301972[] = {{&S752[35], 14}, {&S752[91], 14}, {NULL}};
const Segment SGI301973[] = {{&S752[51], 3}, {&S752[51], 3}, {NULL}};
const Segment SGI301974[] = {{&S753[30], 3}, {NULL}};
const Segment SGI301975[] = {{&S753[8], 3}, {NULL}};
const Segment SGI301976[] = {{&S753[13], 10}, {NULL}};
const Segment SGI301977[] = {{&S753[42], 6}, {NULL}};
const Segment SGI301978[] = {{&S754[30], 3}, {NULL}};
const Segment SGI301979[] = {{&S754[8], 3}, {NULL}};
const Segment SGI301980[] = {{&S754[13], 10}, {NULL}};
const Segment SGI301981[] = {{&S754[42], 7}, {NULL}};
const Segment SGI301982[] = {{&S754[35], 14}, {NULL}};
const Segment SGI301983[] = {{&S754[106], 15}, {NULL}};
const Segment SGI301984[] = {{&S754[51], 6}, {NULL}};
const Segment SGI301985[] = {{&S754[208], 3}, {NULL}};
const Segment SGI301986[] = {{&S755[30], 3}, {&S755[30], 3}, {NULL}};
const Segment SGI301987[] = {{&S755[8], 3}, {&S755[8], 3}, {NULL}};
const Segment SGI301988[] = {{&S755[13], 10}, {&S755[13], 10}, {NULL}};
const Segment SGI301989[] = {{&S755[42], 7}, {NULL}};
const Segment SGI301990[] = {{&S755[35], 14}, {NULL}};
const Segment SGI301991[] = {{&S755[99], 5}, {NULL}};
const Segment SGI301992[] = {{&S756[30], 3}, {NULL}};
const Segment SGI301993[] = {{&S756[8], 3}, {NULL}};
const Segment SGI301994[] = {{&S756[13], 10}, {NULL}};
const Segment SGI301995[] = {{&S756[42], 7}, {NULL}};
const Segment SGI301996[] = {{&S756[35], 14}, {NULL}};
const Segment SGI301997[] = {{&S756[154], 3}, {NULL}};
const Segment SGI301998[] = {{&S757[72], 14}, {NULL}};
const Segment SGI301999[] = {{&S757[8], 3}, {NULL}};
const Segment SGI302000[] = {{&S757[13], 10}, {NULL}};
const Segment SGI302001[] = {{&S757[61], 4}, {NULL}};
const Segment SGI302002[] = {{&S758[72], 14}, {NULL}};
const Segment SGI302003[] = {{&S758[8], 3}, {NULL}};
const Segment SGI302004[] = {{&S758[13], 10}, {NULL}};
const Segment SGI302005[] = {{&S758[61], 4}, {NULL}};
const Segment SGI302006[] = {{&S759[30], 14}, {NULL}};
const Segment SGI302007[] = {{&S759[8], 3}, {NULL}};
const Segment SGI302008[] = {{&S759[13], 10}, {NULL}};
const Segment SGI302009[] = {{&S759[53], 7}, {NULL}};
const Segment SGI302010[] = {{&S760[30], 3}, {&S760[114], 3}, {&S760[170], 3}, {NULL}};
const Segment SGI302011[] = {{&S760[8], 3}, {&S760[8], 3}, {&S760[8], 3}, {NULL}};
const Segment SGI302012[] = {{&S760[13], 10}, {&S760[13], 10}, {&S760[13], 10}, {NULL}};
const Segment SGI302013[] = {{&S760[42], 7}, {&S760[42], 7}, {&S760[182], 7}, {NULL}};
const Segment SGI302014[] = {{&S760[35], 14}, {&S760[35], 14}, {&S760[175], 14}, {NULL}};
const Segment SGI302015[] = {{&S760[135], 3}, {NULL}};
const Segment SGI302016[] = {{&S761[72], 14}, {NULL}};
const Segment SGI302017[] = {{&S761[8], 3}, {NULL}};
const Segment SGI302018[] = {{&S761[13], 10}, {NULL}};
const Segment SGI302019[] = {{&S761[61], 4}, {NULL}};
const Segment SGI302020[] = {{&S762[21], 3}, {NULL}};
const Segment SGI302021[] = {{&S762[8], 3}, {NULL}};
const Segment SGI302022[] = {{&S762[11], 1}, {NULL}};
const Segment SGI302023[] = {{&S762[33], 7}, {NULL}};
const Segment SGI302024[] = {{&S762[26], 14}, {NULL}};
const Segment SGI302025[] = {{&S763[72], 14}, {NULL}};
const Segment SGI302026[] = {{&S763[8], 3}, {NULL}};
const Segment SGI302027[] = {{&S763[13], 10}, {NULL}};
const Segment SGI302028[] = {{&S763[61], 4}, {NULL}};
const Segment SGI302029[] = {{&S764[30], 3}, {&S764[46], 3}, {NULL}};
const Segment SGI302030[] = {{&S764[8], 3}, {&S764[8], 3}, {NULL}};
const Segment SGI302031[] = {{&S764[13], 10}, {&S764[13], 10}, {NULL}};
const Segment SGI302032[] = {{&S764[42], 7}, {&S764[126], 7}, {NULL}};
const Segment SGI302033[] = {{&S764[35], 14}, {&S764[119], 14}, {NULL}};
const Segment SGI302034[] = {{&S764[51], 5}, {NULL}};
const Segment SGI302035[] = {{&S764[135], 3}, {NULL}};
const Segment SGI302036[] = {{&S765[72], 14}, {NULL}};
const Segment SGI302037[] = {{&S765[8], 3}, {NULL}};
const Segment SGI302038[] = {{&S765[13], 10}, {NULL}};
const Segment SGI302039[] = {{&S765[61], 4}, {NULL}};
const Segment SGI302040[] = {{&S766[72], 14}, {NULL}};
const Segment SGI302041[] = {{&S766[8], 3}, {NULL}};
const Segment SGI302042[] = {{&S766[13], 10}, {NULL}};
const Segment SGI302043[] = {{&S766[61], 4}, {NULL}};
const Segment SGI302044[] = {{&S767[72], 14}, {NULL}};
const Segment SGI302045[] = {{&S767[8], 3}, {NULL}};
const Segment SGI302046[] = {{&S767[13], 10}, {NULL}};
const Segment SGI302047[] = {{&S767[61], 4}, {NULL}};
const Segment SGI302048[] = {{&S768[30], 3}, {&S768[102], 3}, {NULL}};
const Segment SGI302049[] = {{&S768[8], 3}, {&S768[8], 3}, {NULL}};
const Segment SGI302050[] = {{&S768[13], 10}, {&S768[13], 10}, {NULL}};
const Segment SGI302051[] = {{&S768[42], 7}, {NULL}};
const Segment SGI302052[] = {{&S768[35], 14}, {NULL}};
const Segment SGI302053[] = {{&S768[150], 15}, {NULL}};
const Segment SGI302054[] = {{&S769[30], 3}, {&S769[46], 3}, {NULL}};
const Segment SGI302055[] = {{&S769[8], 3}, {&S769[8], 3}, {NULL}};
const Segment SGI302056[] = {{&S769[13], 10}, {&S769[13], 10}, {NULL}};
const Segment SGI302057[] = {{&S769[42], 7}, {&S769[106], 7}, {NULL}};
const Segment SGI302058[] = {{&S769[35], 14}, {&S769[99], 14}, {NULL}};
const Segment SGI302059[] = {{&S770[72], 14}, {NULL}};
const Segment SGI302060[] = {{&S770[8], 3}, {NULL}};
const Segment SGI302061[] = {{&S770[13], 10}, {NULL}};
const Segment SGI302062[] = {{&S770[61], 4}, {NULL}};
const Segment SGI302063[] = {{&S771[72], 14}, {NULL}};
const Segment SGI302064[] = {{&S771[8], 3}, {NULL}};
const Segment SGI302065[] = {{&S771[13], 10}, {NULL}};
const Segment SGI302066[] = {{&S771[61], 4}, {NULL}};
const Segment SGI302067[] = {{&S772[72], 14}, {NULL}};
const Segment SGI302068[] = {{&S772[8], 3}, {NULL}};
const Segment SGI302069[] = {{&S772[13], 10}, {NULL}};
const Segment SGI302070[] = {{&S772[61], 4}, {NULL}};
const Segment SGI302071[] = {{&S773[72], 14}, {NULL}};
const Segment SGI302072[] = {{&S773[8], 3}, {NULL}};
const Segment SGI302073[] = {{&S773[13], 10}, {NULL}};
const Segment SGI302074[] = {{&S773[61], 4}, {NULL}};
const Segment SGI302075[] = {{&S774[72], 14}, {NULL}};
const Segment SGI302076[] = {{&S774[8], 3}, {NULL}};
const Segment SGI302077[] = {{&S774[13], 10}, {NULL}};
const Segment SGI302078[] = {{&S774[61], 4}, {NULL}};
const Segment SGI302079[] = {{&S775[30], 3}, {NULL}};
const Segment SGI302080[] = {{&S775[8], 3}, {NULL}};
const Segment SGI302081[] = {{&S775[13], 10}, {NULL}};
const Segment SGI302082[] = {{&S775[42], 7}, {NULL}};
const Segment SGI302083[] = {{&S775[35], 14}, {NULL}};
const Segment SGI302084[] = {{&S775[99], 15}, {NULL}};
const Segment SGI302085[] = {{&S775[185], 4}, {NULL}};
const Segment SGI302086[] = {{&S775[120], 9}, {NULL}};
const Segment SGI302087[] = {{&S776[30], 3}, {NULL}};
const Segment SGI302088[] = {{&S776[8], 3}, {NULL}};
const Segment SGI302089[] = {{&S776[13], 10}, {NULL}};
const Segment SGI302090[] = {{&S776[55], 7}, {NULL}};
const Segment SGI302091[] = {{&S776[48], 14}, {NULL}};
const Segment SGI302092[] = {{&S776[64], 3}, {&S776[64], 3}, {NULL}};
const Segment SGI302093[] = {{&S777[72], 14}, {NULL}};
const Segment SGI302094[] = {{&S777[8], 3}, {NULL}};
const Segment SGI302095[] = {{&S777[13], 10}, {NULL}};
const Segment SGI302096[] = {{&S777[61], 4}, {NULL}};
const Segment SGI302097[] = {{&S778[30], 3}, {&S778[46], 3}, {NULL}};
const Segment SGI302098[] = {{&S778[8], 3}, {&S778[8], 3}, {NULL}};
const Segment SGI302099[] = {{&S778[13], 10}, {NULL}};
const Segment SGI302100[] = {{&S778[42], 7}, {&S778[106], 7}, {NULL}};
const Segment SGI302101[] = {{&S778[35], 14}, {&S778[99], 14}, {NULL}};
const Segment SGI302102[] = {{&S778[115], 3}, {NULL}};
const Segment SGI302103[] = {{&S779[72], 14}, {NULL}};
const Segment SGI302104[] = {{&S779[8], 3}, {NULL}};
const Segment SGI302105[] = {{&S779[13], 10}, {NULL}};
const Segment SGI302106[] = {{&S779[61], 4}, {NULL}};
const Segment SGI302107[] = {{&S780[30], 3}, {NULL}};
const Segment SGI302108[] = {{&S780[8], 3}, {NULL}};
const Segment SGI302109[] = {{&S780[13], 10}, {NULL}};
const Segment SGI302110[] = {{&S780[55], 7}, {NULL}};
const Segment SGI302111[] = {{&S780[48], 14}, {NULL}};
const Segment SGI302112[] = {{&S780[64], 3}, {&S780[64], 3}, {NULL}};
const Segment SGI302113[] = {{&S781[30], 3}, {NULL}};
const Segment SGI302114[] = {{&S781[8], 3}, {NULL}};
const Segment SGI302115[] = {{&S781[13], 10}, {NULL}};
const Segment SGI302116[] = {{&S781[42], 7}, {NULL}};
const Segment SGI302117[] = {{&S781[35], 14}, {NULL}};
const Segment SGI302118[] = {{&S781[109], 6}, {NULL}};
const Segment SGI302119[] = {{&S781[84], 3}, {NULL}};
const Segment SGI302120[] = {{&S782[72], 14}, {NULL}};
const Segment SGI302121[] = {{&S782[8], 3}, {NULL}};
const Segment SGI302122[] = {{&S782[13], 10}, {NULL}};
const Segment SGI302123[] = {{&S782[61], 4}, {NULL}};
const Segment SGI302124[] = {{&S783[30], 3}, {NULL}};
const Segment SGI302125[] = {{&S783[8], 3}, {NULL}};
const Segment SGI302126[] = {{&S783[13], 10}, {NULL}};
const Segment SGI302127[] = {{&S783[42], 7}, {NULL}};
const Segment SGI302128[] = {{&S783[35], 14}, {NULL}};
const Segment SGI302129[] = {{&S783[131], 15}, {NULL}};
const Segment SGI302130[] = {{&S784[30], 3}, {&S784[30], 3}, {NULL}};
const Segment SGI302131[] = {{&S784[8], 3}, {&S784[222], 3}, {NULL}};
const Segment SGI302132[] = {{&S784[13], 10}, {&S784[13], 10}, {NULL}};
const Segment SGI302133[] = {{&S784[42], 7}, {&S784[256], 7}, {NULL}};
const Segment SGI302134[] = {{&S784[35], 14}, {&S784[249], 14}, {NULL}};
const Segment SGI302135[] = {{&S784[278], 6}, {NULL}};
const Segment SGI302136[] = {{&S785[72], 14}, {NULL}};
const Segment SGI302137[] = {{&S785[8], 3}, {NULL}};
const Segment SGI302138[] = {{&S785[13], 10}, {NULL}};
const Segment SGI302139[] = {{&S785[61], 4}, {NULL}};
const Segment SGI302140[] = {{&S786[30], 3}, {&S786[197], 3}, {&S786[30], 3}, {&S786[30], 3}, {NULL}};
const Segment SGI302141[] = {{&S786[8], 3}, {&S786[8], 3}, {&S786[8], 3}, {&S786[8], 3}, {NULL}};
const Segment SGI302142[] = {{&S786[13], 10}, {&S786[13], 10}, {&S786[13], 10}, {&S786[13], 10}, {NULL}};
const Segment SGI302143[] = {{&S786[42], 7}, {&S786[42], 7}, {&S786[42], 7}, {&S786[313], 7}, {NULL}};
const Segment SGI302144[] = {{&S786[35], 14}, {NULL}};
const Segment SGI302145[] = {{&S787[29], 3}, {NULL}};
const Segment SGI302146[] = {{&S787[8], 3}, {NULL}};
const Segment SGI302147[] = {{&S787[13], 9}, {NULL}};
const Segment SGI302148[] = {{&S787[41], 7}, {NULL}};
const Segment SGI302149[] = {{&S788[30], 3}, {NULL}};
const Segment SGI302150[] = {{&S788[8], 3}, {NULL}};
const Segment SGI302151[] = {{&S788[13], 10}, {NULL}};
const Segment SGI302152[] = {{&S788[42], 7}, {NULL}};
const Segment SGI302153[] = {{&S788[35], 14}, {NULL}};
const Segment SGI302154[] = {{&S789[30], 3}, {NULL}};
const Segment SGI302155[] = {{&S789[8], 3}, {NULL}};
const Segment SGI302156[] = {{&S789[13], 10}, {NULL}};
const Segment SGI302157[] = {{&S789[42], 7}, {NULL}};
const Segment SGI302158[] = {{&S790[30], 3}, {NULL}};
const Segment SGI302159[] = {{&S790[8], 3}, {NULL}};
const Segment SGI302160[] = {{&S790[13], 7}, {NULL}};
const Segment SGI302161[] = {{&S790[42], 7}, {NULL}};
const Segment SGI302162[] = {{&S790[35], 14}, {NULL}};
const Segment SGI302163[] = {{&S791[30], 3}, {NULL}};
const Segment SGI302164[] = {{&S791[8], 3}, {NULL}};
const Segment SGI302165[] = {{&S791[13], 10}, {NULL}};
const Segment SGI302166[] = {{&S791[42], 7}, {NULL}};
const Segment SGI302167[] = {{&S792[30], 3}, {NULL}};
const Segment SGI302168[] = {{&S792[8], 3}, {NULL}};
const Segment SGI302169[] = {{&S792[13], 10}, {NULL}};
const Segment SGI302170[] = {{&S792[42], 7}, {NULL}};
const Segment SGI302171[] = {{&S792[35], 14}, {NULL}};
const Segment SGI302172[] = {{&S792[78], 15}, {NULL}};
const Segment SGI302173[] = {{&S793[30], 3}, {NULL}};
const Segment SGI302174[] = {{&S793[8], 3}, {NULL}};
const Segment SGI302175[] = {{&S793[13], 10}, {NULL}};
const Segment SGI302176[] = {{&S793[43], 8}, {NULL}};
const Segment SGI302177[] = {{&S793[98], 9}, {NULL}};
const Segment SGI302178[] = {{&S794[30], 3}, {NULL}};
const Segment SGI302179[] = {{&S794[8], 3}, {NULL}};
const Segment SGI302180[] = {{&S794[13], 10}, {NULL}};
const Segment SGI302181[] = {{&S794[42], 7}, {NULL}};
const Segment SGI302182[] = {{&S794[35], 14}, {NULL}};
const Segment SGI302183[] = {{&S795[30], 3}, {NULL}};
const Segment SGI302184[] = {{&S795[8], 3}, {NULL}};
const Segment SGI302185[] = {{&S795[13], 10}, {NULL}};
const Segment SGI302186[] = {{&S795[42], 7}, {NULL}};
const Segment SGI302187[] = {{&S795[35], 14}, {NULL}};
const Segment SGI302188[] = {{&S795[83], 15}, {NULL}};
const Segment SGI302189[] = {{&S795[51], 3}, {NULL}};
const Segment SGI302190[] = {{&S796[30], 3}, {NULL}};
const Segment SGI302191[] = {{&S796[8], 3}, {NULL}};
const Segment SGI302192[] = {{&S796[13], 10}, {NULL}};
const Segment SGI302193[] = {{&S796[42], 7}, {NULL}};
const Segment SGI302194[] = {{&S796[35], 14}, {NULL}};
const Segment SGI302195[] = {{&S796[85], 15}, {NULL}};
const Segment SGI302196[] = {{&S796[51], 6}, {NULL}};
const Segment SGI302197[] = {{&S797[30], 3}, {NULL}};
const Segment SGI302198[] = {{&S797[8], 3}, {NULL}};
const Segment SGI302199[] = {{&S797[13], 10}, {NULL}};
const Segment SGI302200[] = {{&S797[42], 7}, {NULL}};
const Segment SGI302201[] = {{&S797[35], 14}, {NULL}};
const Segment SGI302202[] = {{&S797[51], 3}, {NULL}};
const Segment SGI302203[] = {{&S798[30], 3}, {NULL}};
const Segment SGI302204[] = {{&S798[8], 3}, {NULL}};
const Segment SGI302205[] = {{&S798[13], 10}, {NULL}};
const Segment SGI302206[] = {{&S798[42], 7}, {NULL}};
const Segment SGI302207[] = {{&S798[35], 14}, {NULL}};
const Segment SGI302208[] = {{&S798[78], 15}, {NULL}};
const Segment SGI302209[] = {{&S798[119], 9}, {NULL}};
const Segment SGI302210[] = {{&S799[30], 3}, {NULL}};
const Segment SGI302211[] = {{&S799[8], 3}, {NULL}};
const Segment SGI302212[] = {{&S799[13], 10}, {NULL}};
const Segment SGI302213[] = {{&S799[42], 7}, {NULL}};
const Segment SGI302214[] = {{&S799[35], 14}, {NULL}};
const Segment SGI302215[] = {{&S799[51], 6}, {NULL}};
const Segment SGI302216[] = {{&S800[31], 3}, {NULL}};
const Segment SGI302217[] = {{&S800[43], 7}, {NULL}};
const Segment SGI302218[] = {{&S801[30], 3}, {&S801[46], 3}, {NULL}};
const Segment SGI302219[] = {{&S801[8], 3}, {&S801[8], 3}, {NULL}};
const Segment SGI302220[] = {{&S801[13], 10}, {&S801[13], 10}, {NULL}};
const Segment SGI302221[] = {{&S801[42], 7}, {&S801[93], 7}, {NULL}};
const Segment SGI302222[] = {{&S801[35], 14}, {&S801[86], 14}, {NULL}};
const Segment SGI302223[] = {{&S801[109], 15}, {NULL}};
const Segment SGI302224[] = {{&S801[102], 3}, {NULL}};
const Segment SGI302225[] = {{&S802[30], 3}, {NULL}};
const Segment SGI302226[] = {{&S802[8], 3}, {NULL}};
const Segment SGI302227[] = {{&S802[13], 10}, {NULL}};
const Segment SGI302228[] = {{&S802[42], 7}, {NULL}};
const Segment SGI302229[] = {{&S802[35], 14}, {NULL}};
const Segment SGI302230[] = {{&S802[103], 15}, {NULL}};
const Segment SGI302231[] = {{&S802[51], 5}, {NULL}};
const Segment SGI302232[] = {{&S803[30], 3}, {&S803[30], 3}, {NULL}};
const Segment SGI302233[] = {{&S803[8], 3}, {&S803[8], 3}, {NULL}};
const Segment SGI302234[] = {{&S803[13], 10}, {&S803[13], 10}, {NULL}};
const Segment SGI302235[] = {{&S803[42], 7}, {&S803[42], 7}, {NULL}};
const Segment SGI302236[] = {{&S803[35], 14}, {NULL}};
const Segment SGI302237[] = {{&S804[30], 3}, {NULL}};
const Segment SGI302238[] = {{&S804[8], 3}, {NULL}};
const Segment SGI302239[] = {{&S804[13], 10}, {NULL}};
const Segment SGI302240[] = {{&S804[42], 7}, {NULL}};
const Segment SGI302241[] = {{&S804[35], 14}, {NULL}};
const Segment SGI302242[] = {{&S804[103], 15}, {NULL}};
const Segment SGI302243[] = {{&S804[51], 5}, {NULL}};
const Segment SGI302244[] = {{&S804[124], 9}, {NULL}};
const Segment SGI302245[] = {{&S805[30], 3}, {NULL}};
const Segment SGI302246[] = {{&S805[8], 3}, {NULL}};
const Segment SGI302247[] = {{&S805[13], 10}, {NULL}};
const Segment SGI302248[] = {{&S805[42], 7}, {NULL}};
const Segment SGI302249[] = {{&S805[35], 14}, {NULL}};
const Segment SGI302250[] = {{&S805[91], 15}, {NULL}};
const Segment SGI302251[] = {{&S805[162], 4}, {NULL}};
const Segment SGI302252[] = {{&S806[30], 3}, {NULL}};
const Segment SGI302253[] = {{&S806[8], 3}, {NULL}};
const Segment SGI302254[] = {{&S806[13], 10}, {NULL}};
const Segment SGI302255[] = {{&S806[42], 7}, {NULL}};
const Segment SGI302256[] = {{&S806[35], 14}, {NULL}};
const Segment SGI302257[] = {{&S806[85], 15}, {NULL}};
const Segment SGI302258[] = {{&S806[51], 6}, {NULL}};
const Segment SGI302259[] = {{&S807[30], 3}, {NULL}};
const Segment SGI302260[] = {{&S807[8], 3}, {NULL}};
const Segment SGI302261[] = {{&S807[13], 10}, {NULL}};
const Segment SGI302262[] = {{&S807[42], 7}, {NULL}};
const Segment SGI302263[] = {{&S807[35], 14}, {NULL}};
const Segment SGI302264[] = {{&S807[98], 15}, {NULL}};
const Segment SGI302265[] = {{&S807[169], 4}, {NULL}};
const Segment SGI302266[] = {{&S807[51], 6}, {NULL}};
const Segment SGI302267[] = {{&S808[30], 3}, {NULL}};
const Segment SGI302268[] = {{&S808[8], 3}, {NULL}};
const Segment SGI302269[] = {{&S808[13], 10}, {NULL}};
const Segment SGI302270[] = {{&S808[42], 7}, {&S808[169], 30}, {NULL}};
const Segment SGI302271[] = {{&S808[35], 14}, {NULL}};
const Segment SGI302272[] = {{&S808[91], 15}, {NULL}};
const Segment SGI302273[] = {{&S809[30], 3}, {NULL}};
const Segment SGI302274[] = {{&S809[8], 3}, {NULL}};
const Segment SGI302275[] = {{&S809[13], 10}, {NULL}};
const Segment SGI302276[] = {{&S809[42], 7}, {NULL}};
const Segment SGI302277[] = {{&S809[35], 14}, {NULL}};
const Segment SGI302278[] = {{&S809[141], 4}, {NULL}};
const Segment SGI302279[] = {{&S810[30], 3}, {&S810[30], 3}, {&S810[30], 3}, {NULL}};
const Segment SGI302280[] = {{&S810[8], 3}, {&S810[8], 3}, {&S810[8], 3}, {NULL}};
const Segment SGI302281[] = {{&S810[13], 10}, {&S810[13], 10}, {&S810[13], 10}, {NULL}};
const Segment SGI302282[] = {{&S810[42], 7}, {&S810[42], 7}, {&S810[42], 7}, {NULL}};
const Segment SGI302283[] = {{&S810[35], 14}, {NULL}};
const Segment SGI302284[] = {{&S811[30], 3}, {NULL}};
const Segment SGI302285[] = {{&S811[8], 3}, {NULL}};
const Segment SGI302286[] = {{&S811[13], 10}, {NULL}};
const Segment SGI302287[] = {{&S811[42], 7}, {NULL}};
const Segment SGI302288[] = {{&S811[35], 14}, {NULL}};
const Segment SGI302289[] = {{&S811[103], 15}, {NULL}};
const Segment SGI302290[] = {{&S811[174], 4}, {NULL}};
const Segment SGI302291[] = {{&S811[51], 5}, {NULL}};
const Segment SGI302292[] = {{&S812[30], 3}, {NULL}};
const Segment SGI302293[] = {{&S812[8], 3}, {NULL}};
const Segment SGI302294[] = {{&S812[13], 10}, {NULL}};
const Segment SGI302295[] = {{&S812[42], 7}, {NULL}};
const Segment SGI302296[] = {{&S812[35], 14}, {NULL}};
const Segment SGI302297[] = {{&S812[78], 15}, {NULL}};
const Segment SGI302298[] = {{&S813[30], 3}, {NULL}};
const Segment SGI302299[] = {{&S813[8], 3}, {NULL}};
const Segment SGI302300[] = {{&S813[13], 10}, {NULL}};
const Segment SGI302301[] = {{&S813[42], 7}, {&S813[148], 30}, {NULL}};
const Segment SGI302302[] = {{&S813[35], 14}, {NULL}};
const Segment SGI302303[] = {{&S814[29], 3}, {NULL}};
const Segment SGI302304[] = {{&S814[8], 3}, {NULL}};
const Segment SGI302305[] = {{&S814[13], 34}, {NULL}};
const Segment SGI302306[] = {{&S814[40], 7}, {NULL}};
const Segment SGI302307[] = {{&S815[30], 3}, {NULL}};
const Segment SGI302308[] = {{&S815[8], 3}, {NULL}};
const Segment SGI302309[] = {{&S815[13], 10}, {NULL}};
const Segment SGI302310[] = {{&S815[42], 7}, {&S815[169], 30}, {NULL}};
const Segment SGI302311[] = {{&S815[35], 14}, {NULL}};
const Segment SGI302312[] = {{&S815[91], 15}, {NULL}};
const Segment SGI302313[] = {{&S816[30], 3}, {NULL}};
const Segment SGI302314[] = {{&S816[8], 3}, {NULL}};
const Segment SGI302315[] = {{&S816[13], 7}, {NULL}};
const Segment SGI302316[] = {{&S816[42], 7}, {NULL}};
const Segment SGI302317[] = {{&S817[30], 3}, {NULL}};
const Segment SGI302318[] = {{&S817[8], 3}, {NULL}};
const Segment SGI302319[] = {{&S817[13], 10}, {NULL}};
const Segment SGI302320[] = {{&S817[42], 7}, {NULL}};
const Segment SGI302321[] = {{&S817[35], 14}, {NULL}};
const Segment SGI302322[] = {{&S817[51], 5}, {NULL}};
const Segment SGI302323[] = {{&S818[27], 3}, {NULL}};
const Segment SGI302324[] = {{&S818[8], 3}, {NULL}};
const Segment SGI302325[] = {{&S818[13], 7}, {NULL}};
const Segment SGI302326[] = {{&S818[39], 7}, {NULL}};
const Segment SGI302327[] = {{&S819[30], 3}, {NULL}};
const Segment SGI302328[] = {{&S819[8], 3}, {NULL}};
const Segment SGI302329[] = {{&S819[13], 10}, {NULL}};
const Segment SGI302330[] = {{&S819[42], 7}, {NULL}};
const Segment SGI302331[] = {{&S819[35], 14}, {NULL}};
const Segment SGI302332[] = {{&S819[78], 15}, {NULL}};
const Segment SGI302333[] = {{&S819[154], 4}, {NULL}};
const Segment SGI302334[] = {{&S819[99], 9}, {NULL}};
const Segment SGI302335[] = {{&S820[30], 3}, {&S820[46], 3}, {NULL}};
const Segment SGI302336[] = {{&S820[8], 3}, {&S820[8], 3}, {NULL}};
const Segment SGI302337[] = {{&S820[13], 10}, {&S820[13], 10}, {NULL}};
const Segment SGI302338[] = {{&S820[42], 7}, {&S820[98], 7}, {NULL}};
const Segment SGI302339[] = {{&S820[35], 14}, {&S820[91], 14}, {NULL}};
const Segment SGI302340[] = {{&S820[51], 3}, {&S820[51], 3}, {NULL}};
const Segment SGI302341[] = {{&S821[30], 3}, {NULL}};
const Segment SGI302342[] = {{&S821[8], 3}, {&S821[8], 3}, {&S821[8], 3}, {NULL}};
const Segment SGI302343[] = {{&S821[13], 10}, {NULL}};
const Segment SGI302344[] = {{&S821[42], 7}, {NULL}};
const Segment SGI302345[] = {{&S821[35], 14}, {NULL}};
const Segment SGI302346[] = {{&S822[30], 3}, {NULL}};
const Segment SGI302347[] = {{&S822[8], 3}, {NULL}};
const Segment SGI302348[] = {{&S822[13], 10}, {NULL}};
const Segment SGI302349[] = {{&S822[42], 7}, {NULL}};
const Segment SGI302350[] = {{&S822[35], 14}, {NULL}};
const Segment SGI302351[] = {{&S822[98], 15}, {&S822[98], 15}, {NULL}};
const Segment SGI302352[] = {{&S822[190], 4}, {NULL}};
const Segment SGI302353[] = {{&S822[51], 6}, {NULL}};
const Segment SGI302354[] = {{&S823[29], 3}, {NULL}};
const Segment SGI302355[] = {{&S823[8], 3}, {NULL}};
const Segment SGI302356[] = {{&S823[13], 10}, {NULL}};
const Segment SGI302357[] = {{&S823[42], 7}, {NULL}};
const Segment SGI302358[] = {{&S823[35], 14}, {NULL}};
const Segment SGI302359[] = {{&S824[30], 3}, {NULL}};
const Segment SGI302360[] = {{&S824[41], 6}, {NULL}};
const Segment SGI302361[] = {{&S825[29], 3}, {NULL}};
const Segment SGI302362[] = {{&S825[8], 3}, {NULL}};
const Segment SGI302363[] = {{&S825[42], 7}, {NULL}};
const Segment SGI302364[] = {{&S826[29], 3}, {NULL}};
const Segment SGI302365[] = {{&S826[8], 3}, {NULL}};
const Segment SGI302366[] = {{&S826[13], 10}, {NULL}};
const Segment SGI302367[] = {{&S826[42], 7}, {NULL}};
const Segment SGI302368[] = {{&S826[35], 14}, {NULL}};
const Segment SGI302369[] = {{&S827[30], 3}, {&S827[46], 3}, {NULL}};
const Segment SGI302370[] = {{&S827[8], 3}, {&S827[8], 3}, {NULL}};
const Segment SGI302371[] = {{&S827[13], 10}, {&S827[13], 10}, {NULL}};
const Segment SGI302372[] = {{&S827[42], 7}, {&S827[108], 7}, {NULL}};
const Segment SGI302373[] = {{&S827[35], 14}, {&S827[101], 14}, {NULL}};
const Segment SGI302374[] = {{&S827[151], 15}, {NULL}};
const Segment SGI302375[] = {{&S827[51], 3}, {&S827[51], 3}, {NULL}};
const Segment SGI302376[] = {{&S828[30], 3}, {&S828[46], 3}, {NULL}};
const Segment SGI302377[] = {{&S828[8], 3}, {&S828[8], 3}, {NULL}};
const Segment SGI302378[] = {{&S828[13], 10}, {&S828[13], 10}, {NULL}};
const Segment SGI302379[] = {{&S828[42], 7}, {&S828[93], 7}, {NULL}};
const Segment SGI302380[] = {{&S828[35], 14}, {&S828[86], 14}, {NULL}};
const Segment SGI302381[] = {{&S828[136], 15}, {NULL}};
const Segment SGI302382[] = {{&S828[177], 9}, {NULL}};
const Segment SGI302383[] = {{&S828[102], 3}, {NULL}};
const Segment SGI302384[] = {{&S829[30], 3}, {NULL}};
const Segment SGI302385[] = {{&S829[8], 3}, {NULL}};
const Segment SGI302386[] = {{&S829[13], 10}, {NULL}};
const Segment SGI302387[] = {{&S829[42], 7}, {&S829[169], 30}, {NULL}};
const Segment SGI302388[] = {{&S829[35], 14}, {NULL}};
const Segment SGI302389[] = {{&S829[91], 15}, {NULL}};
const Segment SGI302390[] = {{&S830[30], 3}, {NULL}};
const Segment SGI302391[] = {{&S830[8], 3}, {NULL}};
const Segment SGI302392[] = {{&S830[13], 10}, {NULL}};
const Segment SGI302393[] = {{&S830[42], 7}, {&S830[148], 30}, {NULL}};
const Segment SGI302394[] = {{&S830[35], 14}, {NULL}};
const Segment SGI302395[] = {{&S831[30], 3}, {&S831[46], 3}, {NULL}};
const Segment SGI302396[] = {{&S831[8], 3}, {&S831[8], 3}, {NULL}};
const Segment SGI302397[] = {{&S831[13], 10}, {&S831[13], 10}, {NULL}};
const Segment SGI302398[] = {{&S831[42], 7}, {&S831[136], 7}, {NULL}};
const Segment SGI302399[] = {{&S831[35], 14}, {&S831[129], 14}, {NULL}};
const Segment SGI302400[] = {{&S831[179], 15}, {NULL}};
const Segment SGI302401[] = {{&S831[220], 4}, {NULL}};
const Segment SGI302402[] = {{&S831[145], 3}, {NULL}};
const Segment SGI302403[] = {{&S832[30], 3}, {NULL}};
const Segment SGI302404[] = {{&S832[8], 3}, {NULL}};
const Segment SGI302405[] = {{&S832[13], 10}, {NULL}};
const Segment SGI302406[] = {{&S832[42], 7}, {&S832[148], 30}, {NULL}};
const Segment SGI302407[] = {{&S832[35], 14}, {NULL}};
const Segment SGI302408[] = {{&S833[30], 3}, {&S833[46], 3}, {NULL}};
const Segment SGI302409[] = {{&S833[8], 3}, {&S833[8], 3}, {NULL}};
const Segment SGI302410[] = {{&S833[13], 10}, {&S833[13], 10}, {NULL}};
const Segment SGI302411[] = {{&S833[42], 7}, {&S833[113], 7}, {NULL}};
const Segment SGI302412[] = {{&S833[35], 14}, {&S833[106], 14}, {NULL}};
const Segment SGI302413[] = {{&S833[51], 6}, {NULL}};
const Segment SGI302414[] = {{&S833[122], 3}, {NULL}};
const Segment SGI302415[] = {{&S834[24], 3}, {NULL}};
const Segment SGI302416[] = {{&S834[36], 7}, {NULL}};
const Segment SGI302417[] = {{&S834[29], 14}, {NULL}};
const Segment SGI302418[] = {{&S834[85], 15}, {NULL}};
const Segment SGI302419[] = {{&S835[30], 3}, {NULL}};
const Segment SGI302420[] = {{&S835[8], 3}, {NULL}};
const Segment SGI302421[] = {{&S835[13], 10}, {NULL}};
const Segment SGI302422[] = {{&S835[42], 7}, {&S835[159], 5}, {NULL}};
const Segment SGI302423[] = {{&S835[35], 14}, {NULL}};
const Segment SGI302424[] = {{&S835[88], 15}, {NULL}};
const Segment SGI302425[] = {{&S836[30], 3}, {NULL}};
const Segment SGI302426[] = {{&S836[8], 3}, {NULL}};
const Segment SGI302427[] = {{&S836[13], 10}, {NULL}};
const Segment SGI302428[] = {{&S836[42], 6}, {NULL}};
const Segment SGI302429[] = {{&S837[30], 3}, {NULL}};
const Segment SGI302430[] = {{&S837[8], 3}, {NULL}};
const Segment SGI302431[] = {{&S837[13], 10}, {NULL}};
const Segment SGI302432[] = {{&S837[42], 7}, {NULL}};
const Segment SGI302433[] = {{&S837[35], 14}, {NULL}};
const Segment SGI302434[] = {{&S837[51], 5}, {NULL}};
const Segment SGI302435[] = {{&S837[76], 9}, {NULL}};
const Segment SGI302436[] = {{&S838[30], 3}, {NULL}};
const Segment SGI302437[] = {{&S838[8], 3}, {NULL}};
const Segment SGI302438[] = {{&S838[13], 10}, {NULL}};
const Segment SGI302439[] = {{&S838[42], 7}, {&S838[176], 30}, {NULL}};
const Segment SGI302440[] = {{&S838[35], 14}, {NULL}};
const Segment SGI302441[] = {{&S838[98], 15}, {NULL}};
const Segment SGI302442[] = {{&S838[51], 6}, {NULL}};
const Segment SGI302443[] = {{&S839[30], 3}, {NULL}};
const Segment SGI302444[] = {{&S839[8], 3}, {NULL}};
const Segment SGI302445[] = {{&S839[13], 10}, {NULL}};
const Segment SGI302446[] = {{&S839[55], 7}, {NULL}};
const Segment SGI302447[] = {{&S839[48], 14}, {NULL}};
const Segment SGI302448[] = {{&S839[64], 3}, {NULL}};
const Segment SGI302449[] = {{&S840[30], 3}, {NULL}};
const Segment SGI302450[] = {{&S840[8], 3}, {NULL}};
const Segment SGI302451[] = {{&S840[13], 10}, {NULL}};
const Segment SGI302452[] = {{&S840[42], 7}, {NULL}};
const Segment SGI302453[] = {{&S840[35], 14}, {NULL}};
const Segment SGI302454[] = {{&S840[93], 15}, {&S840[93], 15}, {NULL}};
const Segment SGI302455[] = {{&S840[51], 6}, {NULL}};
const Segment SGI302456[] = {{&S841[30], 3}, {&S841[46], 3}, {NULL}};
const Segment SGI302457[] = {{&S841[8], 3}, {&S841[8], 3}, {NULL}};
const Segment SGI302458[] = {{&S841[13], 10}, {&S841[13], 10}, {NULL}};
const Segment SGI302459[] = {{&S841[42], 7}, {&S841[101], 7}, {NULL}};
const Segment SGI302460[] = {{&S841[35], 14}, {&S841[94], 14}, {NULL}};
const Segment SGI302461[] = {{&S841[110], 3}, {NULL}};
const Segment SGI302462[] = {{&S842[30], 3}, {NULL}};
const Segment SGI302463[] = {{&S842[8], 3}, {NULL}};
const Segment SGI302464[] = {{&S842[13], 10}, {NULL}};
const Segment SGI302465[] = {{&S842[42], 7}, {&S842[191], 14}, {&S842[191], 14}, {NULL}};
const Segment SGI302466[] = {{&S842[35], 14}, {NULL}};
const Segment SGI302467[] = {{&S842[51], 6}, {NULL}};
const Segment SGI302468[] = {{&S843[30], 3}, {&S843[46], 3}, {NULL}};
const Segment SGI302469[] = {{&S843[8], 3}, {&S843[8], 3}, {NULL}};
const Segment SGI302470[] = {{&S843[13], 10}, {&S843[13], 10}, {NULL}};
const Segment SGI302471[] = {{&S843[42], 7}, {&S843[108], 7}, {NULL}};
const Segment SGI302472[] = {{&S843[35], 14}, {&S843[101], 14}, {NULL}};
const Segment SGI302473[] = {{&S843[151], 15}, {&S843[151], 15}, {NULL}};
const Segment SGI302474[] = {{&S843[51], 6}, {NULL}};
const Segment SGI302475[] = {{&S843[117], 3}, {NULL}};
const Segment SGI302476[] = {{&S844[30], 3}, {&S844[46], 3}, {NULL}};
const Segment SGI302477[] = {{&S844[8], 3}, {&S844[8], 3}, {NULL}};
const Segment SGI302478[] = {{&S844[13], 10}, {&S844[13], 10}, {NULL}};
const Segment SGI302479[] = {{&S844[42], 7}, {&S844[108], 7}, {NULL}};
const Segment SGI302480[] = {{&S844[35], 14}, {&S844[101], 14}, {NULL}};
const Segment SGI302481[] = {{&S844[151], 15}, {NULL}};
const Segment SGI302482[] = {{&S844[51], 6}, {NULL}};
const Segment SGI302483[] = {{&S844[117], 3}, {NULL}};
const Segment SGI302484[] = {{&S845[72], 14}, {NULL}};
const Segment SGI302485[] = {{&S845[8], 3}, {NULL}};
const Segment SGI302486[] = {{&S845[13], 10}, {NULL}};
const Segment SGI302487[] = {{&S845[61], 4}, {NULL}};
const Segment SGI302488[] = {{&S846[30], 3}, {NULL}};
const Segment SGI302489[] = {{&S846[8], 3}, {NULL}};
const Segment SGI302490[] = {{&S846[13], 10}, {NULL}};
const Segment SGI302491[] = {{&S846[42], 7}, {NULL}};
const Segment SGI302492[] = {{&S846[35], 14}, {NULL}};
const Segment SGI302493[] = {{&S846[154], 9}, {NULL}};
const Segment SGI302494[] = {{&S846[51], 3}, {NULL}};
const Segment SGI302495[] = {{&S847[30], 3}, {NULL}};
const Segment SGI302496[] = {{&S847[8], 3}, {NULL}};
const Segment SGI302497[] = {{&S847[13], 10}, {NULL}};
const Segment SGI302498[] = {{&S847[42], 7}, {NULL}};
const Segment SGI302499[] = {{&S847[35], 14}, {NULL}};
const Segment SGI302500[] = {{&S847[86], 15}, {NULL}};
const Segment SGI302501[] = {{&S847[182], 3}, {NULL}};
const Segment SGI302502[] = {{&S848[30], 3}, {&S848[46], 3}, {NULL}};
const Segment SGI302503[] = {{&S848[8], 3}, {&S848[8], 3}, {NULL}};
const Segment SGI302504[] = {{&S848[13], 10}, {&S848[13], 10}, {NULL}};
const Segment SGI302505[] = {{&S848[42], 7}, {&S848[113], 7}, {NULL}};
const Segment SGI302506[] = {{&S848[35], 14}, {&S848[106], 14}, {NULL}};
const Segment SGI302507[] = {{&S848[156], 15}, {NULL}};
const Segment SGI302508[] = {{&S848[51], 5}, {NULL}};
const Segment SGI302509[] = {{&S848[122], 3}, {NULL}};
const Segment SGI302510[] = {{&S849[30], 3}, {NULL}};
const Segment SGI302511[] = {{&S849[8], 3}, {NULL}};
const Segment SGI302512[] = {{&S849[13], 10}, {NULL}};
const Segment SGI302513[] = {{&S849[42], 7}, {NULL}};
const Segment SGI302514[] = {{&S849[35], 14}, {NULL}};
const Segment SGI302515[] = {{&S849[98], 15}, {NULL}};
const Segment SGI302516[] = {{&S849[51], 6}, {NULL}};
const Segment SGI302517[] = {{&S849[188], 3}, {NULL}};
const Segment SGI302518[] = {{&S850[30], 3}, {NULL}};
const Segment SGI302519[] = {{&S850[8], 3}, {NULL}};
const Segment SGI302520[] = {{&S850[13], 10}, {NULL}};
const Segment SGI302521[] = {{&S850[42], 7}, {NULL}};
const Segment SGI302522[] = {{&S850[35], 14}, {NULL}};
const Segment SGI302523[] = {{&S850[152], 3}, {NULL}};
const Segment SGI302524[] = {{&S851[43], 3}, {NULL}};
const Segment SGI302525[] = {{&S851[8], 3}, {&S851[21], 3}, {NULL}};
const Segment SGI302526[] = {{&S851[13], 20}, {NULL}};
const Segment SGI302527[] = {{&S851[55], 7}, {NULL}};
const Segment SGI302528[] = {{&S851[48], 14}, {NULL}};
const Segment SGI302529[] = {{&S852[30], 3}, {NULL}};
const Segment SGI302530[] = {{&S852[8], 3}, {NULL}};
const Segment SGI302531[] = {{&S852[13], 10}, {NULL}};
const Segment SGI302532[] = {{&S852[42], 7}, {NULL}};
const Segment SGI302533[] = {{&S852[35], 14}, {NULL}};
const Segment SGI302534[] = {{&S852[51], 6}, {NULL}};
const Segment SGI302535[] = {{&S852[70], 3}, {NULL}};
const Segment SGI302536[] = {{&S853[30], 3}, {NULL}};
const Segment SGI302537[] = {{&S853[8], 3}, {NULL}};
const Segment SGI302538[] = {{&S853[13], 10}, {NULL}};
const Segment SGI302539[] = {{&S853[42], 7}, {NULL}};
const Segment SGI302540[] = {{&S853[35], 14}, {NULL}};
const Segment SGI302541[] = {{&S853[78], 9}, {NULL}};
const Segment SGI302542[] = {{&S854[30], 3}, {&S854[46], 3}, {NULL}};
const Segment SGI302543[] = {{&S854[8], 3}, {&S854[8], 3}, {NULL}};
const Segment SGI302544[] = {{&S854[13], 10}, {&S854[13], 10}, {NULL}};
const Segment SGI302545[] = {{&S854[42], 7}, {&S854[113], 7}, {NULL}};
const Segment SGI302546[] = {{&S854[35], 14}, {&S854[106], 14}, {NULL}};
const Segment SGI302547[] = {{&S854[51], 5}, {NULL}};
const Segment SGI302548[] = {{&S854[122], 3}, {NULL}};
const Segment SGI302549[] = {{&S855[72], 14}, {NULL}};
const Segment SGI302550[] = {{&S855[8], 3}, {NULL}};
const Segment SGI302551[] = {{&S855[13], 10}, {NULL}};
const Segment SGI302552[] = {{&S855[61], 4}, {NULL}};
const Segment SGI302553[] = {{&S856[25], 3}, {NULL}};
const Segment SGI302554[] = {{&S856[8], 3}, {NULL}};
const Segment SGI302555[] = {{&S856[13], 10}, {NULL}};
const Segment SGI302556[] = {{&S856[47], 7}, {NULL}};
const Segment SGI302557[] = {{&S856[40], 14}, {NULL}};
const Segment SGI302558[] = {{&S857[72], 14}, {NULL}};
const Segment SGI302559[] = {{&S857[8], 3}, {NULL}};
const Segment SGI302560[] = {{&S857[13], 10}, {NULL}};
const Segment SGI302561[] = {{&S857[61], 4}, {NULL}};
const Segment SGI302562[] = {{&S858[72], 14}, {NULL}};
const Segment SGI302563[] = {{&S858[8], 3}, {NULL}};
const Segment SGI302564[] = {{&S858[13], 10}, {NULL}};
const Segment SGI302565[] = {{&S858[61], 4}, {NULL}};
const Segment SGI302566[] = {{&S859[30], 3}, {NULL}};
const Segment SGI302567[] = {{&S859[8], 3}, {NULL}};
const Segment SGI302568[] = {{&S859[13], 10}, {NULL}};
const Segment SGI302569[] = {{&S859[42], 7}, {NULL}};
const Segment SGI302570[] = {{&S859[35], 14}, {NULL}};
const Segment SGI302571[] = {{&S859[103], 15}, {NULL}};
const Segment SGI302572[] = {{&S859[209], 4}, {NULL}};
const Segment SGI302573[] = {{&S859[51], 5}, {NULL}};
const Segment SGI302574[] = {{&S859[124], 9}, {NULL}};
const Segment SGI302575[] = {{&S860[67], 3}, {NULL}};
const Segment SGI302576[] = {{&S860[45], 3}, {NULL}};
const Segment SGI302577[] = {{&S860[50], 10}, {NULL}};
const Segment SGI302578[] = {{&S860[28], 6}, {&S860[79], 7}, {NULL}};
const Segment SGI302579[] = {{&S861[30], 3}, {NULL}};
const Segment SGI302580[] = {{&S861[8], 3}, {NULL}};
const Segment SGI302581[] = {{&S861[13], 10}, {NULL}};
const Segment SGI302582[] = {{&S861[42], 7}, {NULL}};
const Segment SGI302583[] = {{&S861[35], 14}, {NULL}};
const Segment SGI302584[] = {{&S861[125], 4}, {NULL}};
const Segment SGI302585[] = {{&S861[51], 6}, {NULL}};
const Segment SGI302586[] = {{&S862[32], 3}, {NULL}};
const Segment SGI302587[] = {{&S862[10], 3}, {NULL}};
const Segment SGI302588[] = {{&S862[15], 10}, {NULL}};
const Segment SGI302589[] = {{&S862[44], 7}, {&S862[84], 6}, {NULL}};
const Segment SGI302590[] = {{&S863[30], 3}, {NULL}};
const Segment SGI302591[] = {{&S863[8], 3}, {NULL}};
const Segment SGI302592[] = {{&S863[13], 10}, {NULL}};
const Segment SGI302593[] = {{&S863[42], 6}, {NULL}};
const Segment SGI302594[] = {{&S864[30], 14}, {NULL}};
const Segment SGI302595[] = {{&S864[8], 3}, {NULL}};
const Segment SGI302596[] = {{&S864[13], 10}, {NULL}};
const Segment SGI302597[] = {{&S864[53], 7}, {NULL}};
const Segment SGI302598[] = {{&S865[30], 14}, {NULL}};
const Segment SGI302599[] = {{&S865[8], 3}, {NULL}};
const Segment SGI302600[] = {{&S865[13], 10}, {NULL}};
const Segment SGI302601[] = {{&S865[53], 7}, {NULL}};
const Segment SGI302602[] = {{&S866[30], 3}, {NULL}};
const Segment SGI302603[] = {{&S866[8], 3}, {NULL}};
const Segment SGI302604[] = {{&S866[13], 10}, {NULL}};
const Segment SGI302605[] = {{&S866[42], 7}, {&S866[206], 14}, {NULL}};
const Segment SGI302606[] = {{&S866[35], 14}, {NULL}};
const Segment SGI302607[] = {{&S866[51], 6}, {NULL}};
const Segment SGI302608[] = {{&S867[30], 3}, {NULL}};
const Segment SGI302609[] = {{&S867[8], 3}, {NULL}};
const Segment SGI302610[] = {{&S867[13], 149}, {NULL}};
const Segment SGI302611[] = {{&S867[42], 120}, {NULL}};
const Segment SGI302612[] = {{&S867[78], 15}, {NULL}};
const Segment SGI302613[] = {{&S867[99], 9}, {NULL}};
const Segment SGI302614[] = {{&S868[30], 3}, {NULL}};
const Segment SGI302615[] = {{&S868[8], 3}, {NULL}};
const Segment SGI302616[] = {{&S868[13], 10}, {NULL}};
const Segment SGI302617[] = {{&S868[42], 7}, {NULL}};
const Segment SGI302618[] = {{&S868[35], 14}, {NULL}};
const Segment SGI302619[] = {{&S869[30], 3}, {NULL}};
const Segment SGI302620[] = {{&S869[8], 3}, {NULL}};
const Segment SGI302621[] = {{&S869[13], 10}, {NULL}};
const Segment SGI302622[] = {{&S869[42], 7}, {NULL}};
const Segment SGI302623[] = {{&S869[35], 14}, {NULL}};
const Segment SGI302624[] = {{&S869[51], 6}, {NULL}};
const Segment SGI302625[] = {{&S870[31], 3}, {NULL}};
const Segment SGI302626[] = {{&S870[43], 7}, {NULL}};
const Segment SGI302627[] = {{&S871[30], 3}, {NULL}};
const Segment SGI302628[] = {{&S871[8], 3}, {NULL}};
const Segment SGI302629[] = {{&S871[13], 10}, {NULL}};
const Segment SGI302630[] = {{&S871[42], 7}, {NULL}};
const Segment SGI302631[] = {{&S871[35], 14}, {NULL}};
const Segment SGI302632[] = {{&S871[51], 5}, {NULL}};
const Segment SGI302633[] = {{&S872[30], 3}, {NULL}};
const Segment SGI302634[] = {{&S872[8], 3}, {NULL}};
const Segment SGI302635[] = {{&S872[13], 10}, {NULL}};
const Segment SGI302636[] = {{&S872[42], 7}, {NULL}};
const Segment SGI302637[] = {{&S872[35], 14}, {NULL}};
const Segment SGI302638[] = {{&S872[131], 15}, {NULL}};
const Segment SGI302639[] = {{&S873[30], 3}, {NULL}};
const Segment SGI302640[] = {{&S873[8], 3}, {NULL}};
const Segment SGI302641[] = {{&S873[13], 10}, {NULL}};
const Segment SGI302642[] = {{&S873[42], 7}, {NULL}};
const Segment SGI302643[] = {{&S874[24], 3}, {NULL}};
const Segment SGI302644[] = {{&S874[36], 7}, {NULL}};
const Segment SGI302645[] = {{&S874[29], 14}, {NULL}};
const Segment SGI302646[] = {{&S874[45], 6}, {NULL}};
const Segment SGI302647[] = {{&S875[21], 3}, {NULL}};
const Segment SGI302648[] = {{&S875[8], 3}, {NULL}};
const Segment SGI302649[] = {{&S875[11], 1}, {NULL}};
const Segment SGI302650[] = {{&S875[33], 7}, {NULL}};
const Segment SGI302651[] = {{&S875[26], 14}, {NULL}};
const Segment SGI302652[] = {{&S876[30], 3}, {NULL}};
const Segment SGI302653[] = {{&S876[8], 3}, {NULL}};
const Segment SGI302654[] = {{&S876[13], 10}, {NULL}};
const Segment SGI302655[] = {{&S876[42], 7}, {NULL}};
const Segment SGI302656[] = {{&S876[35], 14}, {NULL}};
const Segment SGI302657[] = {{&S876[131], 15}, {NULL}};
const Segment SGI302658[] = {{&S876[51], 6}, {NULL}};
const Segment SGI302659[] = {{&S877[30], 3}, {NULL}};
const Segment SGI302660[] = {{&S877[8], 3}, {NULL}};
const Segment SGI302661[] = {{&S877[13], 10}, {NULL}};
const Segment SGI302662[] = {{&S877[42], 7}, {NULL}};
const Segment SGI302663[] = {{&S877[35], 14}, {NULL}};
const Segment SGI302664[] = {{&S877[143], 15}, {NULL}};
const Segment SGI302665[] = {{&S877[164], 4}, {NULL}};
const Segment SGI302666[] = {{&S877[51], 5}, {NULL}};
const Segment SGI302667[] = {{&S877[174], 9}, {NULL}};
const Segment SGI302668[] = {{&S878[30], 3}, {NULL}};
const Segment SGI302669[] = {{&S878[8], 3}, {NULL}};
const Segment SGI302670[] = {{&S878[13], 10}, {NULL}};
const Segment SGI302671[] = {{&S878[42], 7}, {NULL}};
const Segment SGI302672[] = {{&S878[35], 14}, {NULL}};
const Segment SGI302673[] = {{&S878[136], 15}, {NULL}};
const Segment SGI302674[] = {{&S878[51], 5}, {NULL}};
const Segment SGI302675[] = {{&S879[30], 3}, {NULL}};
const Segment SGI302676[] = {{&S879[8], 3}, {NULL}};
const Segment SGI302677[] = {{&S879[13], 10}, {NULL}};
const Segment SGI302678[] = {{&S879[42], 7}, {&S879[58], 16}, {NULL}};
const Segment SGI302679[] = {{&S879[35], 14}, {NULL}};
const Segment SGI302680[] = {{&S880[30], 3}, {NULL}};
const Segment SGI302681[] = {{&S880[8], 3}, {NULL}};
const Segment SGI302682[] = {{&S880[13], 10}, {NULL}};
const Segment SGI302683[] = {{&S880[42], 7}, {NULL}};
const Segment SGI302684[] = {{&S880[35], 14}, {NULL}};
const Segment SGI302685[] = {{&S880[78], 15}, {NULL}};
const Segment SGI302686[] = {{&S880[124], 4}, {NULL}};
const Segment SGI302687[] = {{&S880[51], 6}, {NULL}};
const Segment SGI302688[] = {{&S881[30], 3}, {NULL}};
const Segment SGI302689[] = {{&S881[8], 3}, {NULL}};
const Segment SGI302690[] = {{&S881[13], 10}, {NULL}};
const Segment SGI302691[] = {{&S881[42], 7}, {NULL}};
const Segment SGI302692[] = {{&S882[21], 3}, {NULL}};
const Segment SGI302693[] = {{&S882[8], 3}, {NULL}};
const Segment SGI302694[] = {{&S882[11], 1}, {NULL}};
const Segment SGI302695[] = {{&S882[33], 7}, {NULL}};
const Segment SGI302696[] = {{&S882[26], 14}, {NULL}};
const Segment SGI302697[] = {{&S882[42], 6}, {NULL}};
const Segment SGI302698[] = {{&S883[30], 3}, {NULL}};
const Segment SGI302699[] = {{&S883[8], 3}, {NULL}};
const Segment SGI302700[] = {{&S883[13], 10}, {NULL}};
const Segment SGI302701[] = {{&S883[42], 7}, {NULL}};
const Segment SGI302702[] = {{&S883[35], 14}, {NULL}};
const Segment SGI302703[] = {{&S883[72], 4}, {NULL}};
const Segment SGI302704[] = {{&S884[24], 3}, {NULL}};
const Segment SGI302705[] = {{&S884[36], 7}, {NULL}};
const Segment SGI302706[] = {{&S884[29], 14}, {NULL}};
const Segment SGI302707[] = {{&S885[30], 3}, {NULL}};
const Segment SGI302708[] = {{&S885[8], 3}, {NULL}};
const Segment SGI302709[] = {{&S885[13], 10}, {NULL}};
const Segment SGI302710[] = {{&S885[42], 7}, {&S885[79], 5}, {NULL}};
const Segment SGI302711[] = {{&S885[35], 14}, {NULL}};
const Segment SGI302712[] = {{&S886[30], 14}, {NULL}};
const Segment SGI302713[] = {{&S886[8], 3}, {NULL}};
const Segment SGI302714[] = {{&S886[13], 10}, {NULL}};
const Segment SGI302715[] = {{&S886[53], 7}, {NULL}};
const Segment SGI302716[] = {{&S887[31], 14}, {NULL}};
const Segment SGI302717[] = {{&S887[8], 3}, {NULL}};
const Segment SGI302718[] = {{&S887[13], 11}, {NULL}};
const Segment SGI302719[] = {{&S887[65], 31}, {NULL}};
const Segment SGI302720[] = {{&S888[31], 3}, {NULL}};
const Segment SGI302721[] = {{&S888[8], 3}, {NULL}};
const Segment SGI302722[] = {{&S888[43], 7}, {NULL}};
const Segment SGI302723[] = {{&S888[36], 14}, {NULL}};
const Segment SGI302724[] = {{&S889[30], 3}, {NULL}};
const Segment SGI302725[] = {{&S889[8], 3}, {NULL}};
const Segment SGI302726[] = {{&S889[13], 10}, {NULL}};
const Segment SGI302727[] = {{&S889[42], 2}, {NULL}};
const Segment SGI302728[] = {{&S890[30], 3}, {NULL}};
const Segment SGI302729[] = {{&S890[8], 3}, {NULL}};
const Segment SGI302730[] = {{&S890[13], 10}, {NULL}};
const Segment SGI302731[] = {{&S890[42], 7}, {&S890[86], 5}, {NULL}};
const Segment SGI302732[] = {{&S890[35], 14}, {NULL}};
const Segment SGI302733[] = {{&S890[51], 6}, {NULL}};
const Segment SGI302734[] = {{&S891[30], 3}, {NULL}};
const Segment SGI302735[] = {{&S891[8], 3}, {NULL}};
const Segment SGI302736[] = {{&S891[13], 10}, {NULL}};
const Segment SGI302737[] = {{&S891[42], 7}, {NULL}};
const Segment SGI302738[] = {{&S891[35], 14}, {NULL}};
const Segment SGI302739[] = {{&S891[70], 5}, {NULL}};
const Segment SGI302740[] = {{&S892[30], 3}, {NULL}};
const Segment SGI302741[] = {{&S892[8], 3}, {NULL}};
const Segment SGI302742[] = {{&S892[13], 10}, {NULL}};
const Segment SGI302743[] = {{&S892[42], 7}, {&S892[78], 9}, {NULL}};
const Segment SGI302744[] = {{&S892[35], 14}, {NULL}};
const Segment SGI302745[] = {{&S892[51], 6}, {NULL}};
const Segment SGI302746[] = {{&S893[31], 14}, {NULL}};
const Segment SGI302747[] = {{&S893[8], 3}, {NULL}};
const Segment SGI302748[] = {{&S893[13], 11}, {NULL}};
const Segment SGI302749[] = {{&S893[65], 17}, {NULL}};
const Segment SGI302750[] = {{&S894[30], 3}, {NULL}};
const Segment SGI302751[] = {{&S894[8], 3}, {NULL}};
const Segment SGI302752[] = {{&S894[13], 10}, {NULL}};
const Segment SGI302753[] = {{&S894[42], 7}, {NULL}};
const Segment SGI302754[] = {{&S894[35], 14}, {NULL}};
const Segment SGI302755[] = {{&S894[131], 15}, {NULL}};
const Segment SGI302756[] = {{&S895[30], 3}, {NULL}};
const Segment SGI302757[] = {{&S895[8], 3}, {NULL}};
const Segment SGI302758[] = {{&S895[13], 10}, {NULL}};
const Segment SGI302759[] = {{&S895[42], 7}, {NULL}};
const Segment SGI302760[] = {{&S895[35], 14}, {NULL}};
const Segment SGI302761[] = {{&S895[51], 3}, {NULL}};
const Segment SGI302762[] = {{&S896[72], 14}, {NULL}};
const Segment SGI302763[] = {{&S896[8], 3}, {NULL}};
const Segment SGI302764[] = {{&S896[13], 10}, {NULL}};
const Segment SGI302765[] = {{&S896[61], 4}, {NULL}};
const Segment SGI302766[] = {{&S897[30], 3}, {NULL}};
const Segment SGI302767[] = {{&S897[8], 3}, {NULL}};
const Segment SGI302768[] = {{&S897[13], 10}, {NULL}};
const Segment SGI302769[] = {{&S897[42], 7}, {NULL}};
const Segment SGI302770[] = {{&S897[35], 14}, {NULL}};
const Segment SGI302771[] = {{&S897[131], 15}, {NULL}};
const Segment SGI302772[] = {{&S897[193], 9}, {NULL}};
const Segment SGI302773[] = {{&S898[30], 3}, {NULL}};
const Segment SGI302774[] = {{&S898[8], 3}, {NULL}};
const Segment SGI302775[] = {{&S898[13], 10}, {NULL}};
const Segment SGI302776[] = {{&S898[42], 6}, {NULL}};
const Segment SGI302777[] = {{&S898[50], 6}, {NULL}};
const Segment SGI302778[] = {{&S900[30], 3}, {NULL}};
const Segment SGI302779[] = {{&S900[8], 3}, {NULL}};
const Segment SGI302780[] = {{&S900[13], 10}, {NULL}};
const Segment SGI302781[] = {{&S900[42], 9}, {NULL}};
const Segment SGI302782[] = {{&S900[53], 3}, {NULL}};
const Segment SGI302783[] = {{&S901[30], 3}, {NULL}};
const Segment SGI302784[] = {{&S901[8], 3}, {NULL}};
const Segment SGI302785[] = {{&S901[13], 10}, {NULL}};
const Segment SGI302786[] = {{&S901[42], 7}, {NULL}};
const Segment SGI302787[] = {{&S901[35], 14}, {NULL}};
const Segment SGI302788[] = {{&S901[51], 3}, {NULL}};
const Segment SGI302789[] = {{&S902[24], 3}, {NULL}};
const Segment SGI302790[] = {{&S902[36], 7}, {NULL}};
const Segment SGI302791[] = {{&S902[29], 14}, {NULL}};
const Segment SGI302792[] = {{&S902[45], 5}, {NULL}};
const Segment SGI302793[] = {{&S903[30], 3}, {NULL}};
const Segment SGI302794[] = {{&S903[8], 3}, {NULL}};
const Segment SGI302795[] = {{&S903[13], 10}, {NULL}};
const Segment SGI302796[] = {{&S903[42], 7}, {NULL}};
const Segment SGI302797[] = {{&S903[35], 14}, {NULL}};
const Segment SGI302798[] = {{&S903[138], 15}, {NULL}};
const Segment SGI302799[] = {{&S903[51], 6}, {NULL}};
const Segment SGI302800[] = {{&S905[30], 3}, {&S905[101], 3}, {NULL}};
const Segment SGI302801[] = {{&S905[8], 3}, {&S905[79], 3}, {NULL}};
const Segment SGI302802[] = {{&S905[13], 10}, {&S905[13], 10}, {NULL}};
const Segment SGI302803[] = {{&S905[42], 7}, {&S905[113], 7}, {NULL}};
const Segment SGI302804[] = {{&S905[35], 14}, {NULL}};
const Segment SGI302805[] = {{&S905[51], 6}, {NULL}};
const Segment SGI302806[] = {{&S906[30], 3}, {&S906[94], 3}, {NULL}};
const Segment SGI302807[] = {{&S906[8], 3}, {&S906[72], 3}, {NULL}};
const Segment SGI302808[] = {{&S906[13], 10}, {&S906[13], 10}, {NULL}};
const Segment SGI302809[] = {{&S906[42], 7}, {&S906[106], 7}, {NULL}};
const Segment SGI302810[] = {{&S906[35], 14}, {NULL}};
const Segment SGI302811[] = {{&S908[30], 3}, {&S908[94], 3}, {NULL}};
const Segment SGI302812[] = {{&S908[8], 3}, {&S908[72], 3}, {NULL}};
const Segment SGI302813[] = {{&S908[13], 10}, {&S908[13], 10}, {NULL}};
const Segment SGI302814[] = {{&S908[42], 7}, {NULL}};
const Segment SGI302815[] = {{&S908[35], 14}, {NULL}};
const Segment SGI302816[] = {{&S909[30], 3}, {NULL}};
const Segment SGI302817[] = {{&S909[8], 3}, {NULL}};
const Segment SGI302818[] = {{&S909[13], 10}, {NULL}};
const Segment SGI302819[] = {{&S909[42], 7}, {NULL}};
const Segment SGI302820[] = {{&S909[35], 14}, {NULL}};
const Segment SGI302821[] = {{&S909[51], 6}, {NULL}};
const Segment SGI302822[] = {{&S911[30], 3}, {&S911[69], 3}, {NULL}};
const Segment SGI302823[] = {{&S911[8], 3}, {NULL}};
const Segment SGI302824[] = {{&S911[13], 10}, {NULL}};
const Segment SGI302825[] = {{&S911[42], 7}, {NULL}};
const Segment SGI302826[] = {{&S911[35], 14}, {NULL}};
const Segment SGI302827[] = {{&S912[30], 3}, {NULL}};
const Segment SGI302828[] = {{&S912[8], 3}, {NULL}};
const Segment SGI302829[] = {{&S912[13], 7}, {NULL}};
const Segment SGI302830[] = {{&S912[42], 7}, {NULL}};
const Segment SGI302831[] = {{&S912[35], 14}, {NULL}};
const Segment SGI302832[] = {{&S913[18], 3}, {NULL}};
const Segment SGI302833[] = {{&S913[8], 3}, {NULL}};
const Segment SGI302834[] = {{&S913[13], 8}, {NULL}};
const Segment SGI302835[] = {{&S913[30], 7}, {NULL}};
const Segment SGI302836[] = {{&S913[23], 14}, {NULL}};
const Segment SGI302837[] = {{&S915[30], 3}, {NULL}};
const Segment SGI302838[] = {{&S915[8], 3}, {NULL}};
const Segment SGI302839[] = {{&S915[13], 10}, {NULL}};
const Segment SGI302840[] = {{&S915[42], 14}, {NULL}};
const Segment SGI302841[] = {{&S916[21], 3}, {NULL}};
const Segment SGI302842[] = {{&S916[8], 3}, {NULL}};
const Segment SGI302843[] = {{&S916[11], 1}, {NULL}};
const Segment SGI302844[] = {{&S916[33], 7}, {NULL}};
const Segment SGI302845[] = {{&S916[26], 14}, {NULL}};
const Segment SGI302846[] = {{&S916[42], 5}, {NULL}};
const Segment SGI302847[] = {{&S917[30], 3}, {NULL}};
const Segment SGI302848[] = {{&S917[8], 3}, {NULL}};
const Segment SGI302849[] = {{&S917[13], 10}, {NULL}};
const Segment SGI302850[] = {{&S917[42], 6}, {NULL}};
const Segment SGI302851[] = {{&S917[50], 6}, {NULL}};
const Segment SGI302852[] = {{&S918[27], 3}, {NULL}};
const Segment SGI302853[] = {{&S918[8], 3}, {NULL}};
const Segment SGI302854[] = {{&S918[13], 7}, {NULL}};
const Segment SGI302855[] = {{&S918[39], 7}, {NULL}};
const Segment SGI302856[] = {{&S918[32], 14}, {NULL}};
const Segment SGI302857[] = {{&S919[5], 3}, {&S919[5], 3}, {NULL}};
const Segment SGI302858[] = {{&S919[39], 7}, {NULL}};
const Segment SGI302859[] = {{&S919[32], 14}, {NULL}};
const Segment SGI302860[] = {{&S919[48], 6}, {NULL}};
const Segment SGI302861[] = {{&S920[31], 3}, {NULL}};
const Segment SGI302862[] = {{&S920[8], 3}, {NULL}};
const Segment SGI302863[] = {{&S920[43], 7}, {NULL}};
const Segment SGI302864[] = {{&S920[36], 14}, {NULL}};
const Segment SGI302865[] = {{&S920[52], 6}, {NULL}};
const Segment SGI302866[] = {{&S921[30], 3}, {NULL}};
const Segment SGI302867[] = {{&S921[8], 3}, {NULL}};
const Segment SGI302868[] = {{&S921[13], 10}, {NULL}};
const Segment SGI302869[] = {{&S921[42], 7}, {NULL}};
const Segment SGI302870[] = {{&S921[35], 14}, {NULL}};
const Segment SGI302871[] = {{&S921[131], 15}, {NULL}};
const Segment SGI302872[] = {{&S921[152], 9}, {NULL}};
const Segment SGI302873[] = {{&S923[30], 3}, {&S923[76], 3}, {NULL}};
const Segment SGI302874[] = {{&S923[8], 3}, {NULL}};
const Segment SGI302875[] = {{&S923[13], 10}, {NULL}};
const Segment SGI302876[] = {{&S923[42], 7}, {NULL}};
const Segment SGI302877[] = {{&S923[35], 14}, {NULL}};
const Segment SGI302878[] = {{&S923[51], 6}, {NULL}};
const Segment SGI302879[] = {{&S924[30], 3}, {&S924[101], 3}, {NULL}};
const Segment SGI302880[] = {{&S924[8], 3}, {&S924[79], 3}, {NULL}};
const Segment SGI302881[] = {{&S924[13], 10}, {&S924[13], 10}, {NULL}};
const Segment SGI302882[] = {{&S924[42], 7}, {&S924[113], 7}, {NULL}};
const Segment SGI302883[] = {{&S924[35], 14}, {NULL}};
const Segment SGI302884[] = {{&S924[51], 6}, {NULL}};
const Segment SGI302885[] = {{&S925[5], 3}, {&S925[5], 3}, {NULL}};
const Segment SGI302886[] = {{&S925[39], 7}, {NULL}};
const Segment SGI302887[] = {{&S925[32], 14}, {NULL}};
const Segment SGI302888[] = {{&S926[30], 3}, {&S926[94], 3}, {NULL}};
const Segment SGI302889[] = {{&S926[8], 3}, {&S926[8], 3}, {NULL}};
const Segment SGI302890[] = {{&S926[13], 10}, {&S926[13], 10}, {NULL}};
const Segment SGI302891[] = {{&S926[42], 7}, {&S926[106], 7}, {NULL}};
const Segment SGI302892[] = {{&S926[35], 14}, {&S926[99], 14}, {NULL}};
const Segment SGI302893[] = {{&S926[115], 3}, {NULL}};
const Segment SGI302894[] = {{&S928[30], 3}, {NULL}};
const Segment SGI302895[] = {{&S928[8], 3}, {NULL}};
const Segment SGI302896[] = {{&S928[13], 10}, {NULL}};
const Segment SGI302897[] = {{&S928[42], 7}, {NULL}};
const Segment SGI302898[] = {{&S928[35], 14}, {NULL}};
const Segment SGI302899[] = {{&S928[143], 15}, {NULL}};
const Segment SGI302900[] = {{&S928[51], 5}, {NULL}};
const Segment SGI302901[] = {{&S928[184], 9}, {NULL}};
const Segment SGI302902[] = {{&S929[36], 3}, {NULL}};
const Segment SGI302903[] = {{&S929[48], 2}, {NULL}};
const Segment SGI302904[] = {{&S930[30], 3}, {NULL}};
const Segment SGI302905[] = {{&S930[8], 3}, {NULL}};
const Segment SGI302906[] = {{&S930[13], 10}, {NULL}};
const Segment SGI302907[] = {{&S930[42], 7}, {&S930[58], 14}, {NULL}};
const Segment SGI302908[] = {{&S930[35], 14}, {NULL}};
const Segment SGI302909[] = {{&S931[30], 3}, {&S931[94], 3}, {NULL}};
const Segment SGI302910[] = {{&S931[8], 3}, {&S931[72], 3}, {NULL}};
const Segment SGI302911[] = {{&S931[13], 10}, {&S931[13], 10}, {NULL}};
const Segment SGI302912[] = {{&S931[42], 7}, {NULL}};
const Segment SGI302913[] = {{&S931[35], 14}, {NULL}};
const Segment SGI302914[] = {{&S931[99], 5}, {NULL}};
const Segment SGI302915[] = {{&S933[30], 3}, {NULL}};
const Segment SGI302916[] = {{&S933[8], 3}, {NULL}};
const Segment SGI302917[] = {{&S933[13], 10}, {NULL}};
const Segment SGI302918[] = {{&S933[42], 7}, {NULL}};
const Segment SGI302919[] = {{&S933[35], 14}, {NULL}};
const Segment SGI302920[] = {{&S933[64], 3}, {NULL}};
const Segment SGI302921[] = {{&S934[30], 3}, {NULL}};
const Segment SGI302922[] = {{&S934[8], 3}, {NULL}};
const Segment SGI302923[] = {{&S934[13], 10}, {NULL}};
const Segment SGI302924[] = {{&S934[42], 7}, {&S934[42], 3}, {NULL}};
const Segment SGI302925[] = {{&S934[35], 14}, {NULL}};
const Segment SGI302926[] = {{&S934[51], 6}, {NULL}};
const Segment SGI302927[] = {{&S935[30], 3}, {&S935[8], 3}, {NULL}};
const Segment SGI302928[] = {{&S935[8], 3}, {&S935[79], 3}, {NULL}};
const Segment SGI302929[] = {{&S935[13], 10}, {&S935[13], 10}, {NULL}};
const Segment SGI302930[] = {{&S935[42], 7}, {&S935[42], 3}, {NULL}};
const Segment SGI302931[] = {{&S935[35], 14}, {NULL}};
const Segment SGI302932[] = {{&S935[51], 6}, {NULL}};
const Segment SGI302933[] = {{&S936[30], 3}, {&S936[94], 3}, {&S936[94], 3}, {NULL}};
const Segment SGI302934[] = {{&S936[8], 3}, {&S936[72], 3}, {&S936[72], 3}, {NULL}};
const Segment SGI302935[] = {{&S936[13], 10}, {&S936[13], 10}, {&S936[13], 10}, {NULL}};
const Segment SGI302936[] = {{&S936[42], 7}, {&S936[106], 7}, {&S936[106], 7}, {NULL}};
const Segment SGI302937[] = {{&S936[35], 14}, {NULL}};
const Segment SGI302938[] = {{&S938[30], 3}, {&S938[86], 3}, {NULL}};
const Segment SGI302939[] = {{&S938[8], 3}, {&S938[8], 3}, {NULL}};
const Segment SGI302940[] = {{&S938[13], 10}, {&S938[13], 10}, {NULL}};
const Segment SGI302941[] = {{&S938[42], 7}, {&S938[98], 7}, {NULL}};
const Segment SGI302942[] = {{&S938[35], 14}, {&S938[91], 14}, {NULL}};
const Segment SGI302943[] = {{&S938[51], 3}, {&S938[51], 3}, {NULL}};
const Segment SGI302944[] = {{&S940[30], 3}, {NULL}};
const Segment SGI302945[] = {{&S940[8], 3}, {NULL}};
const Segment SGI302946[] = {{&S940[13], 10}, {NULL}};
const Segment SGI302947[] = {{&S940[42], 7}, {NULL}};
const Segment SGI302948[] = {{&S940[35], 14}, {NULL}};
const Segment SGI302949[] = {{&S940[109], 6}, {NULL}};
const Segment SGI302950[] = {{&S940[84], 3}, {NULL}};
const Segment SGI302951[] = {{&S941[30], 4}, {NULL}};
const Segment SGI302952[] = {{&S941[8], 3}, {NULL}};
const Segment SGI302953[] = {{&S941[13], 10}, {NULL}};
const Segment SGI302954[] = {{&S941[43], 7}, {NULL}};
const Segment SGI302955[] = {{&S941[36], 14}, {NULL}};
const Segment SGI302956[] = {{&S943[5], 3}, {&S943[5], 3}, {NULL}};
const Segment SGI302957[] = {{&S943[30], 2}, {NULL}};
const Segment SGI302958[] = {{&S944[72], 14}, {NULL}};
const Segment SGI302959[] = {{&S944[8], 3}, {NULL}};
const Segment SGI302960[] = {{&S944[13], 10}, {NULL}};
const Segment SGI302961[] = {{&S944[61], 4}, {NULL}};
const Segment SGI302962[] = {{&S945[72], 15}, {NULL}};
const Segment SGI302963[] = {{&S945[8], 3}, {NULL}};
const Segment SGI302964[] = {{&S945[13], 10}, {NULL}};
const Segment SGI302965[] = {{&S945[61], 4}, {NULL}};
const Segment SGI302966[] = {{&S946[30], 3}, {NULL}};
const Segment SGI302967[] = {{&S946[8], 3}, {NULL}};
const Segment SGI302968[] = {{&S946[13], 10}, {NULL}};
const Segment SGI302969[] = {{&S946[42], 7}, {&S946[91], 5}, {NULL}};
const Segment SGI302970[] = {{&S946[35], 14}, {NULL}};
const Segment SGI302971[] = {{&S946[51], 5}, {NULL}};
const Segment SGI302972[] = {{&S947[30], 3}, {NULL}};
const Segment SGI302973[] = {{&S947[8], 3}, {NULL}};
const Segment SGI302974[] = {{&S947[13], 10}, {NULL}};
const Segment SGI302975[] = {{&S947[42], 7}, {&S947[137], 20}, {NULL}};
const Segment SGI302976[] = {{&S947[35], 14}, {NULL}};
const Segment SGI302977[] = {{&S947[138], 15}, {NULL}};
const Segment SGI302978[] = {{&S947[51], 3}, {NULL}};
const Segment SGI302979[] = {{&S948[31], 3}, {NULL}};
const Segment SGI302980[] = {{&S948[8], 3}, {NULL}};
const Segment SGI302981[] = {{&S948[43], 7}, {NULL}};
const Segment SGI302982[] = {{&S948[36], 14}, {NULL}};
const Segment SGI302983[] = {{&S948[52], 5}, {NULL}};
const Segment SGI302984[] = {{&S949[30], 3}, {NULL}};
const Segment SGI302985[] = {{&S949[8], 3}, {NULL}};
const Segment SGI302986[] = {{&S949[13], 10}, {NULL}};
const Segment SGI302987[] = {{&S949[42], 7}, {NULL}};
const Segment SGI302988[] = {{&S949[35], 14}, {NULL}};
const Segment SGI302989[] = {{&S949[129], 15}, {NULL}};
const Segment SGI302990[] = {{&S949[102], 6}, {NULL}};
const Segment SGI302991[] = {{&S949[51], 3}, {NULL}};
const Segment SGI302992[] = {{&S950[30], 3}, {NULL}};
const Segment SGI302993[] = {{&S950[8], 3}, {NULL}};
const Segment SGI302994[] = {{&S950[13], 10}, {NULL}};
const Segment SGI302995[] = {{&S950[42], 7}, {NULL}};
const Segment SGI302996[] = {{&S950[35], 14}, {NULL}};
const Segment SGI302997[] = {{&S950[74], 3}, {NULL}};
const Segment SGI302998[] = {{&S951[30], 3}, {NULL}};
const Segment SGI302999[] = {{&S951[8], 3}, {NULL}};
const Segment SGI303000[] = {{&S951[13], 10}, {NULL}};
const Segment SGI303001[] = {{&S951[42], 7}, {NULL}};
const Segment SGI303002[] = {{&S951[35], 14}, {NULL}};
const Segment SGI303003[] = {{&S951[51], 6}, {NULL}};
const Segment SGI303004[] = {{&S951[81], 3}, {NULL}};
const Segment SGI303005[] = {{&S952[30], 3}, {&S952[30], 3}, {NULL}};
const Segment SGI303006[] = {{&S952[8], 3}, {NULL}};
const Segment SGI303007[] = {{&S952[13], 10}, {NULL}};
const Segment SGI303008[] = {{&S952[42], 7}, {NULL}};
const Segment SGI303009[] = {{&S952[35], 14}, {NULL}};
const Segment SGI303010[] = {{&S952[51], 6}, {NULL}};
const Segment SGI303011[] = {{&S953[72], 14}, {NULL}};
const Segment SGI303012[] = {{&S953[8], 3}, {NULL}};
const Segment SGI303013[] = {{&S953[13], 10}, {NULL}};
const Segment SGI303014[] = {{&S953[61], 4}, {NULL}};
const Segment SGI303015[] = {{&S954[72], 15}, {NULL}};
const Segment SGI303016[] = {{&S954[8], 3}, {NULL}};
const Segment SGI303017[] = {{&S954[13], 10}, {NULL}};
const Segment SGI303018[] = {{&S954[61], 4}, {NULL}};
const Segment SGI303019[] = {{&S955[29], 3}, {NULL}};
const Segment SGI303020[] = {{&S955[8], 3}, {NULL}};
const Segment SGI303021[] = {{&S955[13], 52}, {NULL}};
const Segment SGI303022[] = {{&S955[40], 21}, {NULL}};
const Segment SGI303023[] = {{&S956[30], 4}, {NULL}};
const Segment SGI303024[] = {{&S956[8], 3}, {NULL}};
const Segment SGI303025[] = {{&S956[13], 10}, {NULL}};
const Segment SGI303026[] = {{&S956[43], 7}, {&S956[59], 16}, {NULL}};
const Segment SGI303027[] = {{&S956[36], 14}, {NULL}};
const Segment SGI303028[] = {{&S958[30], 4}, {NULL}};
const Segment SGI303029[] = {{&S958[8], 3}, {NULL}};
const Segment SGI303030[] = {{&S958[13], 10}, {NULL}};
const Segment SGI303031[] = {{&S958[43], 7}, {NULL}};
const Segment SGI303032[] = {{&S958[36], 14}, {NULL}};
const Segment SGI303033[] = {{&S958[52], 6}, {NULL}};
const Segment SGI303034[] = {{&S959[31], 15}, {NULL}};
const Segment SGI303035[] = {{&S959[8], 3}, {NULL}};
const Segment SGI303036[] = {{&S959[13], 11}, {NULL}};
const Segment SGI303037[] = {{&S959[66], 31}, {NULL}};
const Segment SGI303038[] = {{&S960[30], 3}, {NULL}};
const Segment SGI303039[] = {{&S960[8], 3}, {NULL}};
const Segment SGI303040[] = {{&S960[13], 10}, {NULL}};
const Segment SGI303041[] = {{&S960[42], 7}, {NULL}};
const Segment SGI303042[] = {{&S960[35], 14}, {NULL}};
const Segment SGI303043[] = {{&S960[51], 6}, {NULL}};
const Segment SGI303044[] = {{&S960[81], 3}, {NULL}};
const Segment SGI303045[] = {{&S961[30], 3}, {NULL}};
const Segment SGI303046[] = {{&S961[8], 3}, {NULL}};
const Segment SGI303047[] = {{&S961[13], 10}, {NULL}};
const Segment SGI303048[] = {{&S961[42], 7}, {NULL}};
const Segment SGI303049[] = {{&S961[35], 14}, {NULL}};
const Segment SGI303050[] = {{&S961[74], 3}, {NULL}};
const Segment SGI303051[] = {{&S962[30], 4}, {NULL}};
const Segment SGI303052[] = {{&S962[8], 3}, {NULL}};
const Segment SGI303053[] = {{&S962[13], 10}, {NULL}};
const Segment SGI303054[] = {{&S962[43], 7}, {NULL}};
const Segment SGI303055[] = {{&S964[30], 3}, {NULL}};
const Segment SGI303056[] = {{&S964[8], 3}, {&S964[79], 3}, {NULL}};
const Segment SGI303057[] = {{&S964[13], 10}, {NULL}};
const Segment SGI303058[] = {{&S964[42], 7}, {NULL}};
const Segment SGI303059[] = {{&S964[35], 14}, {NULL}};
const Segment SGI303060[] = {{&S964[51], 6}, {NULL}};
const Segment SGI303061[] = {{&S965[30], 3}, {NULL}};
const Segment SGI303062[] = {{&S965[8], 3}, {NULL}};
const Segment SGI303063[] = {{&S965[13], 10}, {NULL}};
const Segment SGI303064[] = {{&S965[42], 7}, {NULL}};
const Segment SGI303065[] = {{&S965[35], 14}, {NULL}};
const Segment SGI303066[] = {{&S965[131], 9}, {NULL}};
const Segment SGI303067[] = {{&S966[30], 3}, {NULL}};
const Segment SGI303068[] = {{&S966[8], 3}, {NULL}};
const Segment SGI303069[] = {{&S966[13], 10}, {NULL}};
const Segment SGI303070[] = {{&S966[55], 7}, {NULL}};
const Segment SGI303071[] = {{&S966[48], 14}, {NULL}};
const Segment SGI303072[] = {{&S966[64], 3}, {&S966[64], 3}, {NULL}};
const Segment SGI303073[] = {{&S967[30], 3}, {NULL}};
const Segment SGI303074[] = {{&S967[8], 3}, {NULL}};
const Segment SGI303075[] = {{&S967[13], 10}, {NULL}};
const Segment SGI303076[] = {{&S967[42], 7}, {NULL}};
const Segment SGI303077[] = {{&S967[35], 14}, {NULL}};
const Segment SGI303078[] = {{&S967[109], 6}, {NULL}};
const Segment SGI303079[] = {{&S967[84], 3}, {NULL}};
const Segment SGI303080[] = {{&S968[30], 15}, {NULL}};
const Segment SGI303081[] = {{&S968[8], 3}, {NULL}};
const Segment SGI303082[] = {{&S968[13], 10}, {NULL}};
const Segment SGI303083[] = {{&S968[54], 7}, {NULL}};
const Segment SGI303084[] = {{&S969[29], 3}, {NULL}};
const Segment SGI303085[] = {{&S969[9], 3}, {NULL}};
const Segment SGI303086[] = {{&S969[40], 21}, {NULL}};
const Segment SGI303087[] = {{&S970[30], 4}, {NULL}};
const Segment SGI303088[] = {{&S970[8], 3}, {NULL}};
const Segment SGI303089[] = {{&S970[13], 10}, {NULL}};
const Segment SGI303090[] = {{&S970[43], 7}, {NULL}};
const Segment SGI303091[] = {{&S970[36], 14}, {NULL}};
const Segment SGI303092[] = {{&S970[139], 15}, {NULL}};
const Segment SGI303093[] = {{&S970[52], 6}, {NULL}};
const Segment SGI303094[] = {{&S971[31], 15}, {NULL}};
const Segment SGI303095[] = {{&S971[8], 3}, {NULL}};
const Segment SGI303096[] = {{&S971[13], 11}, {NULL}};
const Segment SGI303097[] = {{&S971[66], 31}, {NULL}};
const Segment SGI303098[] = {{&S972[30], 15}, {NULL}};
const Segment SGI303099[] = {{&S972[8], 3}, {NULL}};
const Segment SGI303100[] = {{&S972[13], 10}, {NULL}};
const Segment SGI303101[] = {{&S972[54], 7}, {NULL}};
const Segment SGI303102[] = {{&S973[72], 15}, {NULL}};
const Segment SGI303103[] = {{&S973[8], 3}, {NULL}};
const Segment SGI303104[] = {{&S973[13], 10}, {NULL}};
const Segment SGI303105[] = {{&S973[61], 4}, {NULL}};
const Segment SGI303106[] = {{&S974[30], 3}, {&S974[101], 3}, {NULL}};
const Segment SGI303107[] = {{&S974[8], 3}, {&S974[79], 3}, {NULL}};
const Segment SGI303108[] = {{&S974[13], 10}, {&S974[13], 10}, {NULL}};
const Segment SGI303109[] = {{&S974[42], 7}, {NULL}};
const Segment SGI303110[] = {{&S974[35], 14}, {NULL}};
const Segment SGI303111[] = {{&S974[51], 6}, {NULL}};
const Segment SGI303112[] = {{&S974[106], 5}, {NULL}};
const Segment SGI303113[] = {{&S975[30], 4}, {NULL}};
const Segment SGI303114[] = {{&S975[8], 3}, {NULL}};
const Segment SGI303115[] = {{&S975[13], 10}, {NULL}};
const Segment SGI303116[] = {{&S975[43], 7}, {NULL}};
const Segment SGI303117[] = {{&S975[36], 14}, {NULL}};
const Segment SGI303118[] = {{&S975[119], 15}, {NULL}};
const Segment SGI303119[] = {{&S976[5], 3}, {&S976[27], 4}, {NULL}};
const Segment SGI303120[] = {{&S976[40], 7}, {NULL}};
const Segment SGI303121[] = {{&S976[33], 14}, {NULL}};
const Segment SGI303122[] = {{&S976[49], 6}, {NULL}};
const Segment SGI303123[] = {{&S977[30], 15}, {NULL}};
const Segment SGI303124[] = {{&S977[8], 3}, {NULL}};
const Segment SGI303125[] = {{&S977[13], 10}, {NULL}};
const Segment SGI303126[] = {{&S977[54], 7}, {NULL}};
const Segment SGI303127[] = {{&S978[72], 15}, {NULL}};
const Segment SGI303128[] = {{&S978[8], 3}, {NULL}};
const Segment SGI303129[] = {{&S978[13], 10}, {NULL}};
const Segment SGI303130[] = {{&S978[61], 4}, {NULL}};
const Segment SGI303131[] = {{&S979[31], 15}, {NULL}};
const Segment SGI303132[] = {{&S979[8], 3}, {NULL}};
const Segment SGI303133[] = {{&S979[13], 11}, {NULL}};
const Segment SGI303134[] = {{&S979[66], 31}, {NULL}};
const Segment SGI303135[] = {{&S980[31], 3}, {NULL}};
const Segment SGI303136[] = {{&S980[9], 3}, {NULL}};
const Segment SGI303137[] = {{&S980[14], 10}, {NULL}};
const Segment SGI303138[] = {{&S980[43], 7}, {NULL}};
const Segment SGI303139[] = {{&S980[36], 14}, {NULL}};
const Segment SGI303140[] = {{&S980[193], 9}, {NULL}};
const Segment SGI303141[] = {{&S981[31], 3}, {NULL}};
const Segment SGI303142[] = {{&S981[9], 3}, {NULL}};
const Segment SGI303143[] = {{&S981[14], 10}, {NULL}};
const Segment SGI303144[] = {{&S981[43], 7}, {NULL}};
const Segment SGI303145[] = {{&S981[36], 14}, {NULL}};
const Segment SGI303146[] = {{&S981[164], 4}, {NULL}};
const Segment SGI303147[] = {{&S981[52], 5}, {NULL}};
const Segment SGI303148[] = {{&S981[174], 9}, {NULL}};
const Segment SGI303149[] = {{&S982[31], 3}, {NULL}};
const Segment SGI303150[] = {{&S982[9], 3}, {NULL}};
const Segment SGI303151[] = {{&S982[14], 10}, {NULL}};
const Segment SGI303152[] = {{&S982[43], 7}, {NULL}};
const Segment SGI303153[] = {{&S982[36], 14}, {NULL}};
const Segment SGI303154[] = {{&S982[159], 4}, {NULL}};
const Segment SGI303155[] = {{&S982[52], 6}, {NULL}};
const Segment SGI303156[] = {{&S983[30], 3}, {NULL}};
const Segment SGI303157[] = {{&S983[8], 3}, {NULL}};
const Segment SGI303158[] = {{&S983[13], 10}, {NULL}};
const Segment SGI303159[] = {{&S983[42], 8}, {NULL}};
const Segment SGI303160[] = {{&S983[52], 6}, {NULL}};
const Segment SGI303161[] = {{&S984[31], 15}, {NULL}};
const Segment SGI303162[] = {{&S984[8], 3}, {NULL}};
const Segment SGI303163[] = {{&S984[13], 11}, {NULL}};
const Segment SGI303164[] = {{&S984[66], 31}, {NULL}};
const Segment SGI303165[] = {{&S985[30], 3}, {NULL}};
const Segment SGI303166[] = {{&S985[8], 3}, {&S985[72], 3}, {NULL}};
const Segment SGI303167[] = {{&S985[13], 10}, {NULL}};
const Segment SGI303168[] = {{&S985[42], 7}, {NULL}};
const Segment SGI303169[] = {{&S985[35], 14}, {NULL}};
const Segment SGI303170[] = {{&S986[30], 3}, {&S986[101], 3}, {NULL}};
const Segment SGI303171[] = {{&S986[8], 3}, {&S986[79], 3}, {NULL}};
const Segment SGI303172[] = {{&S986[13], 10}, {&S986[13], 10}, {NULL}};
const Segment SGI303173[] = {{&S986[42], 7}, {&S986[113], 2}, {NULL}};
const Segment SGI303174[] = {{&S986[35], 14}, {NULL}};
const Segment SGI303175[] = {{&S986[51], 6}, {NULL}};
const Segment SGI303176[] = {{&S988[30], 3}, {NULL}};
const Segment SGI303177[] = {{&S988[8], 3}, {NULL}};
const Segment SGI303178[] = {{&S988[13], 10}, {NULL}};
const Segment SGI303179[] = {{&S988[42], 7}, {NULL}};
const Segment SGI303180[] = {{&S988[35], 14}, {NULL}};
const Segment SGI303181[] = {{&S988[131], 15}, {&S988[131], 15}, {NULL}};
const Segment SGI303182[] = {{&S990[72], 15}, {NULL}};
const Segment SGI303183[] = {{&S990[8], 3}, {NULL}};
const Segment SGI303184[] = {{&S990[13], 10}, {NULL}};
const Segment SGI303185[] = {{&S990[61], 4}, {NULL}};
const Segment SGI303186[] = {{&S991[30], 15}, {NULL}};
const Segment SGI303187[] = {{&S991[8], 3}, {NULL}};
const Segment SGI303188[] = {{&S991[13], 10}, {NULL}};
const Segment SGI303189[] = {{&S991[54], 7}, {NULL}};
const Segment SGI303190[] = {{&S992[72], 15}, {NULL}};
const Segment SGI303191[] = {{&S992[8], 3}, {NULL}};
const Segment SGI303192[] = {{&S992[13], 10}, {NULL}};
const Segment SGI303193[] = {{&S992[61], 4}, {NULL}};
const Segment SGI303194[] = {{&S993[30], 3}, {NULL}};
const Segment SGI303195[] = {{&S994[30], 3}, {&S994[101], 3}, {NULL}};
const Segment SGI303196[] = {{&S994[8], 3}, {&S994[79], 3}, {NULL}};
const Segment SGI303197[] = {{&S994[13], 10}, {&S994[13], 10}, {NULL}};
const Segment SGI303198[] = {{&S994[42], 7}, {&S994[113], 7}, {NULL}};
const Segment SGI303199[] = {{&S994[35], 14}, {&S994[106], 14}, {NULL}};
const Segment SGI303200[] = {{&S994[51], 6}, {&S994[51], 6}, {NULL}};
const Segment SGI303201[] = {{&S995[31], 15}, {NULL}};
const Segment SGI303202[] = {{&S995[8], 3}, {NULL}};
const Segment SGI303203[] = {{&S995[13], 11}, {NULL}};
const Segment SGI303204[] = {{&S995[66], 31}, {NULL}};
const Segment SGI303205[] = {{&S996[72], 15}, {NULL}};
const Segment SGI303206[] = {{&S996[8], 3}, {NULL}};
const Segment SGI303207[] = {{&S996[13], 10}, {NULL}};
const Segment SGI303208[] = {{&S996[61], 4}, {NULL}};
const Segment SGI303209[] = {{&S997[30], 15}, {NULL}};
const Segment SGI303210[] = {{&S997[8], 3}, {NULL}};
const Segment SGI303211[] = {{&S997[13], 10}, {NULL}};
const Segment SGI303212[] = {{&S997[54], 7}, {NULL}};
const Segment SGI303213[] = {{&S998[30], 3}, {&S998[30], 3}, {NULL}};
const Segment SGI303214[] = {{&S998[8], 3}, {NULL}};
const Segment SGI303215[] = {{&S998[13], 10}, {NULL}};
const Segment SGI303216[] = {{&S998[42], 7}, {NULL}};
const Segment SGI303217[] = {{&S998[35], 14}, {NULL}};
const Segment SGI303218[] = {{&S999[30], 3}, {NULL}};
const Segment SGI303219[] = {{&S999[8], 3}, {NULL}};
const Segment SGI303220[] = {{&S999[13], 10}, {NULL}};
const Segment SGI303221[] = {{&S999[55], 7}, {NULL}};
const Segment SGI303222[] = {{&S999[48], 14}, {NULL}};
const Segment SGI303223[] = {{&S999[64], 3}, {&S999[64], 3}, {NULL}};
const Segment SGI303224[] = {{&S1000[30], 3}, {&S1000[106], 3}, {&S1000[106], 3}, {NULL}};
const Segment SGI303225[] = {{&S1000[8], 3}, {&S1000[84], 3}, {&S1000[84], 3}, {NULL}};
const Segment SGI303226[] = {{&S1000[13], 10}, {&S1000[13], 10}, {&S1000[13], 10}, {NULL}};
const Segment SGI303227[] = {{&S1000[42], 7}, {&S1000[118], 7}, {&S1000[118], 7}, {NULL}};
const Segment SGI303228[] = {{&S1000[35], 14}, {NULL}};
const Segment SGI303229[] = {{&S1000[51], 5}, {NULL}};
const Segment SGI303230[] = {{&S1001[30], 4}, {NULL}};
const Segment SGI303231[] = {{&S1001[8], 3}, {NULL}};
const Segment SGI303232[] = {{&S1001[13], 10}, {NULL}};
const Segment SGI303233[] = {{&S1001[43], 7}, {&S1001[59], 14}, {NULL}};
const Segment SGI303234[] = {{&S1001[36], 14}, {NULL}};
const Segment SGI303235[] = {{&S1002[30], 4}, {NULL}};
const Segment SGI303236[] = {{&S1002[8], 3}, {NULL}};
const Segment SGI303237[] = {{&S1002[13], 10}, {NULL}};
const Segment SGI303238[] = {{&S1002[43], 7}, {NULL}};
const Segment SGI303239[] = {{&S1002[36], 14}, {NULL}};
const Segment SGI303240[] = {{&S1002[132], 15}, {NULL}};
const Segment SGI303241[] = {{&S1002[153], 9}, {NULL}};
const Segment SGI303242[] = {{&S1003[30], 4}, {NULL}};
const Segment SGI303243[] = {{&S1003[8], 3}, {NULL}};
const Segment SGI303244[] = {{&S1003[13], 10}, {NULL}};
const Segment SGI303245[] = {{&S1003[43], 7}, {NULL}};
const Segment SGI303246[] = {{&S1003[36], 14}, {NULL}};
const Segment SGI303247[] = {{&S1003[132], 15}, {NULL}};
const Segment SGI303248[] = {{&S1004[18], 3}, {NULL}};
const Segment SGI303249[] = {{&S1004[8], 3}, {NULL}};
const Segment SGI303250[] = {{&S1004[13], 8}, {NULL}};
const Segment SGI303251[] = {{&S1004[30], 7}, {NULL}};
const Segment SGI303252[] = {{&S1004[23], 14}, {NULL}};
const Segment SGI303253[] = {{&S1004[39], 6}, {NULL}};
const Segment SGI303254[] = {{&S1005[30], 4}, {NULL}};
const Segment SGI303255[] = {{&S1005[8], 3}, {NULL}};
const Segment SGI303256[] = {{&S1005[13], 10}, {NULL}};
const Segment SGI303257[] = {{&S1005[43], 7}, {NULL}};
const Segment SGI303258[] = {{&S1005[36], 14}, {NULL}};
const Segment SGI303259[] = {{&S1006[31], 15}, {NULL}};
const Segment SGI303260[] = {{&S1006[8], 3}, {NULL}};
const Segment SGI303261[] = {{&S1006[13], 11}, {NULL}};
const Segment SGI303262[] = {{&S1006[66], 31}, {NULL}};
const Segment SGI303263[] = {{&S1007[72], 15}, {NULL}};
const Segment SGI303264[] = {{&S1007[8], 3}, {NULL}};
const Segment SGI303265[] = {{&S1007[13], 10}, {NULL}};
const Segment SGI303266[] = {{&S1007[61], 4}, {NULL}};
const Segment SGI303267[] = {{&S1008[30], 3}, {NULL}};
const Segment SGI303268[] = {{&S1008[8], 3}, {NULL}};
const Segment SGI303269[] = {{&S1008[13], 10}, {NULL}};
const Segment SGI303270[] = {{&S1008[42], 7}, {NULL}};
const Segment SGI303271[] = {{&S1008[35], 14}, {NULL}};
const Segment SGI303272[] = {{&S1008[131], 15}, {NULL}};
const Segment SGI303273[] = {{&S1008[162], 4}, {NULL}};
const Segment SGI303274[] = {{&S1009[30], 3}, {NULL}};
const Segment SGI303275[] = {{&S1009[8], 3}, {NULL}};
const Segment SGI303276[] = {{&S1009[13], 10}, {NULL}};
const Segment SGI303277[] = {{&S1009[42], 7}, {&S1009[71], 34}, {NULL}};
const Segment SGI303278[] = {{&S1009[35], 14}, {NULL}};
const Segment SGI303279[] = {{&S1010[31], 15}, {NULL}};
const Segment SGI303280[] = {{&S1010[8], 3}, {NULL}};
const Segment SGI303281[] = {{&S1010[13], 11}, {NULL}};
const Segment SGI303282[] = {{&S1010[66], 31}, {NULL}};
const Segment SGI303283[] = {{&S1011[30], 15}, {NULL}};
const Segment SGI303284[] = {{&S1011[8], 3}, {NULL}};
const Segment SGI303285[] = {{&S1011[13], 10}, {NULL}};
const Segment SGI303286[] = {{&S1011[54], 7}, {NULL}};
const Segment SGI303287[] = {{&S1012[30], 3}, {NULL}};
const Segment SGI303288[] = {{&S1012[8], 3}, {NULL}};
const Segment SGI303289[] = {{&S1012[13], 10}, {NULL}};
const Segment SGI303290[] = {{&S1012[42], 7}, {NULL}};
const Segment SGI303291[] = {{&S1012[35], 14}, {NULL}};
const Segment SGI303292[] = {{&S1013[30], 3}, {&S1013[94], 3}, {NULL}};
const Segment SGI303293[] = {{&S1013[8], 3}, {&S1013[72], 3}, {NULL}};
const Segment SGI303294[] = {{&S1013[13], 10}, {&S1013[13], 10}, {NULL}};
const Segment SGI303295[] = {{&S1013[42], 7}, {&S1013[106], 7}, {NULL}};
const Segment SGI303296[] = {{&S1013[35], 14}, {NULL}};
const Segment SGI303297[] = {{&S1014[72], 15}, {NULL}};
const Segment SGI303298[] = {{&S1014[8], 3}, {NULL}};
const Segment SGI303299[] = {{&S1014[13], 10}, {NULL}};
const Segment SGI303300[] = {{&S1014[61], 4}, {NULL}};
const Segment SGI303301[] = {{&S1015[31], 15}, {NULL}};
const Segment SGI303302[] = {{&S1015[8], 3}, {NULL}};
const Segment SGI303303[] = {{&S1015[13], 11}, {NULL}};
const Segment SGI303304[] = {{&S1015[66], 31}, {NULL}};
const Segment SGI303305[] = {{&S1016[30], 3}, {NULL}};
const Segment SGI303306[] = {{&S1016[8], 3}, {NULL}};
const Segment SGI303307[] = {{&S1016[13], 10}, {NULL}};
const Segment SGI303308[] = {{&S1016[42], 7}, {NULL}};
const Segment SGI303309[] = {{&S1016[35], 14}, {NULL}};
const Segment SGI303310[] = {{&S1017[30], 15}, {NULL}};
const Segment SGI303311[] = {{&S1017[8], 3}, {NULL}};
const Segment SGI303312[] = {{&S1017[13], 10}, {NULL}};
const Segment SGI303313[] = {{&S1017[54], 7}, {NULL}};
const Segment SGI303314[] = {{&S1018[18], 3}, {NULL}};
const Segment SGI303315[] = {{&S1018[8], 3}, {NULL}};
const Segment SGI303316[] = {{&S1018[13], 8}, {NULL}};
const Segment SGI303317[] = {{&S1018[30], 7}, {NULL}};
const Segment SGI303318[] = {{&S1018[23], 14}, {NULL}};
const Segment SGI303319[] = {{&S1018[39], 5}, {NULL}};
const Segment SGI303320[] = {{&S1019[72], 15}, {NULL}};
const Segment SGI303321[] = {{&S1019[8], 3}, {NULL}};
const Segment SGI303322[] = {{&S1019[13], 10}, {NULL}};
const Segment SGI303323[] = {{&S1019[61], 4}, {NULL}};
const Segment SGI303324[] = {{&S1020[30], 3}, {NULL}};
const Segment SGI303325[] = {{&S1020[8], 3}, {NULL}};
const Segment SGI303326[] = {{&S1020[13], 10}, {NULL}};
const Segment SGI303327[] = {{&S1020[42], 7}, {NULL}};
const Segment SGI303328[] = {{&S1020[35], 14}, {NULL}};
const Segment SGI303329[] = {{&S1020[51], 3}, {NULL}};
const Segment SGI303330[] = {{&S1024[30], 3}, {NULL}};
const Segment SGI303331[] = {{&S1024[8], 3}, {NULL}};
const Segment SGI303332[] = {{&S1024[13], 10}, {NULL}};
const Segment SGI303333[] = {{&S1024[42], 7}, {NULL}};
const Segment SGI303334[] = {{&S1024[35], 14}, {NULL}};
const Segment SGI303335[] = {{&S1025[30], 3}, {NULL}};
const Segment SGI303336[] = {{&S1025[8], 3}, {NULL}};
const Segment SGI303337[] = {{&S1025[13], 10}, {NULL}};
const Segment SGI303338[] = {{&S1025[42], 7}, {NULL}};
const Segment SGI303339[] = {{&S1026[30], 3}, {&S1026[30], 1}, {NULL}};
const Segment SGI303340[] = {{&S1026[8], 3}, {&S1026[8], 3}, {NULL}};
const Segment SGI303341[] = {{&S1026[13], 10}, {NULL}};
const Segment SGI303342[] = {{&S1026[42], 7}, {&S1026[43], 2}, {NULL}};
const Segment SGI303343[] = {{&S1026[35], 14}, {NULL}};
const Segment SGI303344[] = {{&S1026[101], 3}, {NULL}};
const Segment SGI303345[] = {{&S1027[30], 3}, {NULL}};
const Segment SGI303346[] = {{&S1027[8], 3}, {NULL}};
const Segment SGI303347[] = {{&S1027[13], 10}, {NULL}};
const Segment SGI303348[] = {{&S1027[42], 7}, {NULL}};
const Segment SGI303349[] = {{&S1027[35], 14}, {NULL}};
const Segment SGI303350[] = {{&S1027[56], 15}, {NULL}};
const Segment SGI303351[] = {{&S1027[51], 3}, {NULL}};
const Segment SGI303352[] = {{&S1028[30], 3}, {NULL}};
const Segment SGI303353[] = {{&S1028[8], 3}, {NULL}};
const Segment SGI303354[] = {{&S1028[13], 10}, {NULL}};
const Segment SGI303355[] = {{&S1028[42], 7}, {NULL}};
const Segment SGI303356[] = {{&S1028[35], 14}, {NULL}};
const Segment SGI303357[] = {{&S1028[51], 5}, {NULL}};
const Segment SGI303358[] = {{&S1029[31], 3}, {NULL}};
const Segment SGI303359[] = {{&S1029[8], 3}, {NULL}};
const Segment SGI303360[] = {{&S1029[13], 11}, {NULL}};
const Segment SGI303361[] = {{&S1029[43], 7}, {NULL}};
const Segment SGI303362[] = {{&S1030[30], 3}, {NULL}};
const Segment SGI303363[] = {{&S1030[8], 3}, {NULL}};
const Segment SGI303364[] = {{&S1030[13], 10}, {NULL}};
const Segment SGI303365[] = {{&S1030[42], 3}, {NULL}};
const Segment SGI303366[] = {{&S1031[30], 3}, {&S1031[40], 3}, {NULL}};
const Segment SGI303367[] = {{&S1031[8], 3}, {NULL}};
const Segment SGI303368[] = {{&S1031[13], 10}, {NULL}};
const Segment SGI303369[] = {{&S1031[52], 7}, {NULL}};
const Segment SGI303370[] = {{&S1032[30], 3}, {&S1032[81], 3}, {NULL}};
const Segment SGI303371[] = {{&S1032[8], 3}, {&S1032[8], 3}, {NULL}};
const Segment SGI303372[] = {{&S1032[13], 10}, {&S1032[13], 10}, {NULL}};
const Segment SGI303373[] = {{&S1032[42], 7}, {&S1032[42], 3}, {NULL}};
const Segment SGI303374[] = {{&S1032[35], 14}, {NULL}};
const Segment SGI303375[] = {{&S1033[30], 3}, {NULL}};
const Segment SGI303376[] = {{&S1033[8], 3}, {NULL}};
const Segment SGI303377[] = {{&S1033[13], 60}, {NULL}};
const Segment SGI303378[] = {{&S1033[42], 7}, {NULL}};
const Segment SGI303379[] = {{&S1033[51], 3}, {NULL}};
const Segment SGI303380[] = {{&S1034[30], 3}, {&S1034[30], 3}, {NULL}};
const Segment SGI303381[] = {{&S1034[8], 3}, {&S1034[8], 3}, {NULL}};
const Segment SGI303382[] = {{&S1034[13], 10}, {&S1034[13], 10}, {NULL}};
const Segment SGI303383[] = {{&S1034[42], 7}, {&S1034[42], 7}, {NULL}};
const Segment SGI303384[] = {{&S1034[35], 14}, {&S1034[35], 14}, {NULL}};
const Segment SGI303385[] = {{&S1034[51], 3}, {&S1034[51], 3}, {NULL}};
const Segment SGI303386[] = {{&S1035[30], 3}, {NULL}};
const Segment SGI303387[] = {{&S1035[8], 3}, {NULL}};
const Segment SGI303388[] = {{&S1035[13], 10}, {NULL}};
const Segment SGI303389[] = {{&S1035[42], 7}, {NULL}};
const Segment SGI303390[] = {{&S1035[35], 14}, {NULL}};
const Segment SGI303391[] = {{&S1035[51], 3}, {NULL}};
const Segment SGI303392[] = {{&S1036[30], 3}, {NULL}};
const Segment SGI303393[] = {{&S1036[8], 3}, {NULL}};
const Segment SGI303394[] = {{&S1036[13], 10}, {NULL}};
const Segment SGI303395[] = {{&S1036[42], 7}, {NULL}};
const Segment SGI303396[] = {{&S1036[35], 14}, {NULL}};
const Segment SGI303397[] = {{&S1036[51], 5}, {NULL}};
const Segment SGI303398[] = {{&S1036[63], 3}, {NULL}};
const Segment SGI303399[] = {{&S1037[30], 3}, {NULL}};
const Segment SGI303400[] = {{&S1037[8], 3}, {NULL}};
const Segment SGI303401[] = {{&S1037[13], 10}, {NULL}};
const Segment SGI303402[] = {{&S1037[42], 7}, {NULL}};
const Segment SGI303403[] = {{&S1037[35], 14}, {NULL}};
const Segment SGI303404[] = {{&S1038[30], 3}, {NULL}};
const Segment SGI303405[] = {{&S1038[8], 3}, {NULL}};
const Segment SGI303406[] = {{&S1038[13], 10}, {NULL}};
const Segment SGI303407[] = {{&S1038[42], 7}, {NULL}};
const Segment SGI303408[] = {{&S1038[35], 14}, {NULL}};
const Segment SGI303409[] = {{&S1038[51], 6}, {NULL}};
const Segment SGI303410[] = {{&S1038[58], 3}, {NULL}};
const Segment SGI303411[] = {{&S1039[30], 3}, {&S1039[30], 3}, {NULL}};
const Segment SGI303412[] = {{&S1039[8], 3}, {&S1039[8], 3}, {NULL}};
const Segment SGI303413[] = {{&S1039[13], 10}, {&S1039[13], 10}, {NULL}};
const Segment SGI303414[] = {{&S1039[42], 7}, {&S1039[93], 7}, {NULL}};
const Segment SGI303415[] = {{&S1039[35], 14}, {&S1039[86], 14}, {NULL}};
const Segment SGI303416[] = {{&S1039[102], 3}, {NULL}};
const Segment SGI303417[] = {{&S1040[30], 3}, {NULL}};
const Segment SGI303418[] = {{&S1040[8], 3}, {NULL}};
const Segment SGI303419[] = {{&S1040[13], 7}, {NULL}};
const Segment SGI303420[] = {{&S1040[42], 7}, {NULL}};
const Segment SGI303421[] = {{&S1040[35], 14}, {NULL}};
const Segment SGI303422[] = {{&S1040[51], 3}, {NULL}};
const Segment SGI303423[] = {{&S1041[30], 3}, {&S1041[30], 3}, {NULL}};
const Segment SGI303424[] = {{&S1041[8], 3}, {&S1041[8], 3}, {NULL}};
const Segment SGI303425[] = {{&S1041[13], 10}, {&S1041[13], 10}, {NULL}};
const Segment SGI303426[] = {{&S1041[42], 7}, {NULL}};
const Segment SGI303427[] = {{&S1041[35], 14}, {NULL}};
const Segment SGI303428[] = {{&S1041[51], 3}, {NULL}};
const Segment SGI303429[] = {{&S1042[30], 3}, {NULL}};
const Segment SGI303430[] = {{&S1042[8], 3}, {NULL}};
const Segment SGI303431[] = {{&S1042[13], 10}, {NULL}};
const Segment SGI303432[] = {{&S1042[42], 7}, {NULL}};
const Segment SGI303433[] = {{&S1043[46], 3}, {&S1043[46], 3}, {NULL}};
const Segment SGI303434[] = {{&S1043[8], 3}, {NULL}};
const Segment SGI303435[] = {{&S1043[13], 10}, {NULL}};
const Segment SGI303436[] = {{&S1043[32], 7}, {&S1043[32], 7}, {NULL}};
const Segment SGI303437[] = {{&S1043[25], 14}, {&S1043[25], 14}, {NULL}};
const Segment SGI303438[] = {{&S1044[30], 3}, {NULL}};
const Segment SGI303439[] = {{&S1044[8], 3}, {NULL}};
const Segment SGI303440[] = {{&S1044[13], 10}, {NULL}};
const Segment SGI303441[] = {{&S1044[42], 7}, {NULL}};
const Segment SGI303442[] = {{&S1045[30], 3}, {&S1045[30], 3}, {NULL}};
const Segment SGI303443[] = {{&S1045[8], 3}, {&S1045[8], 3}, {NULL}};
const Segment SGI303444[] = {{&S1045[13], 10}, {&S1045[13], 10}, {NULL}};
const Segment SGI303445[] = {{&S1045[42], 7}, {&S1045[42], 7}, {NULL}};
const Segment SGI303446[] = {{&S1045[35], 14}, {&S1045[35], 14}, {NULL}};
const Segment SGI303447[] = {{&S1045[102], 3}, {NULL}};
const Segment SGI303448[] = {{&S1046[30], 3}, {NULL}};
const Segment SGI303449[] = {{&S1046[8], 3}, {NULL}};
const Segment SGI303450[] = {{&S1046[13], 10}, {NULL}};
const Segment SGI303451[] = {{&S1046[42], 7}, {NULL}};
const Segment SGI303452[] = {{&S1046[35], 14}, {NULL}};
const Segment SGI303453[] = {{&S1046[56], 9}, {NULL}};
const Segment SGI303454[] = {{&S1046[51], 3}, {NULL}};
const Segment SGI303455[] = {{&S1047[30], 3}, {&S1047[30], 3}, {NULL}};
const Segment SGI303456[] = {{&S1047[8], 3}, {&S1047[8], 3}, {NULL}};
const Segment SGI303457[] = {{&S1047[13], 10}, {&S1047[13], 10}, {NULL}};
const Segment SGI303458[] = {{&S1047[42], 7}, {&S1047[98], 7}, {NULL}};
const Segment SGI303459[] = {{&S1047[35], 14}, {&S1047[91], 14}, {NULL}};
const Segment SGI303460[] = {{&S1047[51], 3}, {&S1047[51], 3}, {NULL}};
const Segment SGI303461[] = {{&S1048[30], 3}, {&S1048[30], 3}, {NULL}};
const Segment SGI303462[] = {{&S1048[8], 3}, {&S1048[8], 3}, {NULL}};
const Segment SGI303463[] = {{&S1048[13], 10}, {&S1048[13], 10}, {NULL}};
const Segment SGI303464[] = {{&S1048[42], 7}, {&S1048[42], 7}, {NULL}};
const Segment SGI303465[] = {{&S1048[35], 14}, {&S1048[35], 14}, {NULL}};
const Segment SGI303466[] = {{&S1048[152], 15}, {NULL}};
const Segment SGI303467[] = {{&S1048[51], 3}, {&S1048[51], 3}, {NULL}};
const Segment SGI303468[] = {{&S1049[29], 3}, {NULL}};
const Segment SGI303469[] = {{&S1049[41], 7}, {NULL}};
const Segment SGI303470[] = {{&S1049[34], 14}, {NULL}};
const Segment SGI303471[] = {{&S1050[42], 3}, {NULL}};
const Segment SGI303472[] = {{&S1050[20], 3}, {NULL}};
const Segment SGI303473[] = {{&S1050[25], 10}, {NULL}};
const Segment SGI303474[] = {{&S1050[53], 3}, {NULL}};
const Segment SGI303475[] = {{&S1051[8], 3}, {NULL}};
const Segment SGI303476[] = {{&S1051[13], 8}, {NULL}};
const Segment SGI303477[] = {{&S1051[30], 7}, {NULL}};
const Segment SGI303478[] = {{&S1051[23], 14}, {NULL}};
const Segment SGI303479[] = {{&S1052[30], 3}, {NULL}};
const Segment SGI303480[] = {{&S1052[8], 3}, {NULL}};
const Segment SGI303481[] = {{&S1052[13], 10}, {NULL}};
const Segment SGI303482[] = {{&S1052[42], 7}, {NULL}};
const Segment SGI303483[] = {{&S1052[35], 14}, {NULL}};
const Segment SGI303484[] = {{&S1053[18], 3}, {NULL}};
const Segment SGI303485[] = {{&S1053[8], 3}, {NULL}};
const Segment SGI303486[] = {{&S1053[13], 8}, {NULL}};
const Segment SGI303487[] = {{&S1053[30], 7}, {NULL}};
const Segment SGI303488[] = {{&S1054[30], 3}, {NULL}};
const Segment SGI303489[] = {{&S1054[8], 3}, {NULL}};
const Segment SGI303490[] = {{&S1054[13], 10}, {NULL}};
const Segment SGI303491[] = {{&S1054[42], 7}, {NULL}};
const Segment SGI303492[] = {{&S1054[35], 14}, {NULL}};
const Segment SGI303493[] = {{&S1054[76], 4}, {NULL}};
const Segment SGI303494[] = {{&S1054[51], 3}, {NULL}};
const Segment SGI303495[] = {{&S1055[31], 3}, {NULL}};
const Segment SGI303496[] = {{&S1055[8], 3}, {NULL}};
const Segment SGI303497[] = {{&S1055[43], 7}, {NULL}};
const Segment SGI303498[] = {{&S1056[30], 3}, {NULL}};
const Segment SGI303499[] = {{&S1056[8], 3}, {NULL}};
const Segment SGI303500[] = {{&S1056[13], 10}, {NULL}};
const Segment SGI303501[] = {{&S1056[42], 7}, {NULL}};
const Segment SGI303502[] = {{&S1056[35], 14}, {NULL}};
const Segment SGI303503[] = {{&S1056[51], 3}, {NULL}};
const Segment SGI303504[] = {{&S1057[27], 3}, {NULL}};
const Segment SGI303505[] = {{&S1057[8], 3}, {NULL}};
const Segment SGI303506[] = {{&S1057[13], 7}, {NULL}};
const Segment SGI303507[] = {{&S1057[39], 7}, {NULL}};
const Segment SGI303508[] = {{&S1057[32], 14}, {NULL}};
const Segment SGI303509[] = {{&S1057[48], 3}, {NULL}};
const Segment SGI303510[] = {{&S1058[30], 3}, {NULL}};
const Segment SGI303511[] = {{&S1058[8], 3}, {NULL}};
const Segment SGI303512[] = {{&S1058[13], 10}, {NULL}};
const Segment SGI303513[] = {{&S1058[42], 7}, {&S1058[75], 5}, {NULL}};
const Segment SGI303514[] = {{&S1058[35], 14}, {NULL}};
const Segment SGI303515[] = {{&S1058[51], 3}, {NULL}};
const Segment SGI303516[] = {{&S1059[29], 3}, {NULL}};
const Segment SGI303517[] = {{&S1059[41], 7}, {NULL}};
const Segment SGI303518[] = {{&S1059[34], 14}, {NULL}};
const Segment SGI303519[] = {{&S1060[30], 3}, {&S1060[30], 3}, {&S1060[30], 3}, {NULL}};
const Segment SGI303520[] = {{&S1060[8], 3}, {&S1060[8], 3}, {&S1060[8], 3}, {NULL}};
const Segment SGI303521[] = {{&S1060[13], 10}, {&S1060[13], 10}, {&S1060[13], 10}, {NULL}};
const Segment SGI303522[] = {{&S1060[42], 7}, {&S1060[42], 7}, {&S1060[42], 7}, {NULL}};
const Segment SGI303523[] = {{&S1060[35], 14}, {&S1060[35], 14}, {&S1060[35], 14}, {NULL}};
const Segment SGI303524[] = {{&S1060[51], 3}, {&S1060[51], 3}, {&S1060[51], 3}, {NULL}};
const Segment SGI303525[] = {{&S1061[30], 3}, {&S1061[30], 3}, {NULL}};
const Segment SGI303526[] = {{&S1061[8], 3}, {NULL}};
const Segment SGI303527[] = {{&S1061[13], 10}, {NULL}};
const Segment SGI303528[] = {{&S1061[42], 7}, {NULL}};
const Segment SGI303529[] = {{&S1061[35], 14}, {NULL}};
const Segment SGI303530[] = {{&S1061[51], 3}, {NULL}};
const Segment SGI303531[] = {{&S1062[30], 3}, {NULL}};
const Segment SGI303532[] = {{&S1062[8], 3}, {NULL}};
const Segment SGI303533[] = {{&S1062[13], 10}, {NULL}};
const Segment SGI303534[] = {{&S1062[42], 7}, {NULL}};
const Segment SGI303535[] = {{&S1062[35], 14}, {NULL}};
const Segment SGI303536[] = {{&S1062[94], 15}, {&S1062[94], 15}, {NULL}};
const Segment SGI303537[] = {{&S1062[51], 3}, {NULL}};
const Segment SGI303538[] = {{&S1063[30], 3}, {NULL}};
const Segment SGI303539[] = {{&S1063[8], 3}, {&S1063[8], 3}, {&S1063[8], 3}, {NULL}};
const Segment SGI303540[] = {{&S1063[13], 10}, {NULL}};
const Segment SGI303541[] = {{&S1063[42], 7}, {NULL}};
const Segment SGI303542[] = {{&S1063[35], 14}, {NULL}};
const Segment SGI303543[] = {{&S1063[51], 3}, {NULL}};
const Segment SGI303544[] = {{&S1064[31], 3}, {NULL}};
const Segment SGI303545[] = {{&S1064[43], 7}, {NULL}};
const Segment SGI303546[] = {{&S1065[30], 3}, {&S1065[30], 3}, {NULL}};
const Segment SGI303547[] = {{&S1065[8], 3}, {&S1065[8], 3}, {NULL}};
const Segment SGI303548[] = {{&S1065[13], 10}, {&S1065[13], 10}, {NULL}};
const Segment SGI303549[] = {{&S1065[42], 3}, {&S1065[102], 7}, {NULL}};
const Segment SGI303550[] = {{&S1065[95], 14}, {NULL}};
const Segment SGI303551[] = {{&S1065[111], 3}, {NULL}};
const Segment SGI303552[] = {{&S1066[27], 3}, {NULL}};
const Segment SGI303553[] = {{&S1066[8], 3}, {NULL}};
const Segment SGI303554[] = {{&S1066[13], 7}, {NULL}};
const Segment SGI303555[] = {{&S1066[39], 7}, {NULL}};
const Segment SGI303556[] = {{&S1067[30], 3}, {NULL}};
const Segment SGI303557[] = {{&S1067[8], 3}, {NULL}};
const Segment SGI303558[] = {{&S1067[13], 10}, {NULL}};
const Segment SGI303559[] = {{&S1067[42], 8}, {NULL}};
const Segment SGI303560[] = {{&S1068[30], 3}, {NULL}};
const Segment SGI303561[] = {{&S1068[8], 3}, {NULL}};
const Segment SGI303562[] = {{&S1068[13], 10}, {NULL}};
const Segment SGI303563[] = {{&S1068[42], 7}, {NULL}};
const Segment SGI303564[] = {{&S1068[35], 14}, {NULL}};
const Segment SGI303565[] = {{&S1068[131], 15}, {NULL}};
const Segment SGI303566[] = {{&S1069[30], 3}, {&S1069[90], 3}, {NULL}};
const Segment SGI303567[] = {{&S1069[8], 3}, {&S1069[8], 3}, {NULL}};
const Segment SGI303568[] = {{&S1069[13], 10}, {&S1069[13], 10}, {NULL}};
const Segment SGI303569[] = {{&S1069[42], 3}, {&S1069[102], 7}, {NULL}};
const Segment SGI303570[] = {{&S1069[95], 14}, {NULL}};
const Segment SGI303571[] = {{&S1069[111], 3}, {NULL}};
const Segment SGI303572[] = {{&S1070[30], 3}, {NULL}};
const Segment SGI303573[] = {{&S1070[8], 3}, {NULL}};
const Segment SGI303574[] = {{&S1070[13], 10}, {NULL}};
const Segment SGI303575[] = {{&S1070[42], 7}, {NULL}};
const Segment SGI303576[] = {{&S1070[35], 14}, {NULL}};
const Segment SGI303577[] = {{&S1070[51], 6}, {NULL}};
const Segment SGI303578[] = {{&S1070[58], 3}, {NULL}};
const Segment SGI303579[] = {{&S1071[30], 3}, {NULL}};
const Segment SGI303580[] = {{&S1071[8], 3}, {NULL}};
const Segment SGI303581[] = {{&S1071[13], 10}, {NULL}};
const Segment SGI303582[] = {{&S1071[42], 7}, {&S1071[63], 30}, {NULL}};
const Segment SGI303583[] = {{&S1071[35], 14}, {NULL}};
const Segment SGI303584[] = {{&S1071[51], 3}, {NULL}};
const Segment SGI303585[] = {{&S1072[30], 4}, {&S1072[82], 3}, {NULL}};
const Segment SGI303586[] = {{&S1072[8], 3}, {&S1072[8], 3}, {NULL}};
const Segment SGI303587[] = {{&S1072[13], 10}, {&S1072[13], 10}, {NULL}};
const Segment SGI303588[] = {{&S1072[43], 7}, {&S1072[94], 7}, {NULL}};
const Segment SGI303589[] = {{&S1072[36], 14}, {&S1072[87], 14}, {NULL}};
const Segment SGI303590[] = {{&S1072[103], 3}, {NULL}};
const Segment SGI303591[] = {{&S1073[30], 3}, {&S1073[30], 3}, {NULL}};
const Segment SGI303592[] = {{&S1073[8], 3}, {&S1073[8], 3}, {NULL}};
const Segment SGI303593[] = {{&S1073[13], 10}, {&S1073[69], 10}, {NULL}};
const Segment SGI303594[] = {{&S1073[42], 7}, {&S1073[42], 3}, {NULL}};
const Segment SGI303595[] = {{&S1073[35], 14}, {NULL}};
const Segment SGI303596[] = {{&S1073[51], 3}, {NULL}};
const Segment SGI303597[] = {{&S1074[30], 3}, {NULL}};
const Segment SGI303598[] = {{&S1074[8], 3}, {NULL}};
const Segment SGI303599[] = {{&S1074[13], 10}, {NULL}};
const Segment SGI303600[] = {{&S1074[42], 7}, {&S1074[75], 5}, {NULL}};
const Segment SGI303601[] = {{&S1074[35], 14}, {NULL}};
const Segment SGI303602[] = {{&S1074[82], 9}, {NULL}};
const Segment SGI303603[] = {{&S1074[51], 3}, {NULL}};
const Segment SGI303604[] = {{&S1075[30], 3}, {NULL}};
const Segment SGI303605[] = {{&S1075[8], 3}, {NULL}};
const Segment SGI303606[] = {{&S1075[13], 10}, {NULL}};
const Segment SGI303607[] = {{&S1075[42], 7}, {NULL}};
const Segment SGI303608[] = {{&S1075[35], 14}, {NULL}};
const Segment SGI303609[] = {{&S1075[51], 6}, {NULL}};
const Segment SGI303610[] = {{&S1076[8], 3}, {NULL}};
const Segment SGI303611[] = {{&S1076[13], 10}, {NULL}};
const Segment SGI303612[] = {{&S1076[41], 3}, {NULL}};
const Segment SGI303613[] = {{&S1076[46], 3}, {NULL}};
const Segment SGI303614[] = {{&S1077[29], 3}, {NULL}};
const Segment SGI303615[] = {{&S1077[41], 7}, {NULL}};
const Segment SGI303616[] = {{&S1077[34], 14}, {NULL}};
const Segment SGI303617[] = {{&S1077[50], 3}, {NULL}};
const Segment SGI303618[] = {{&S1078[31], 3}, {NULL}};
const Segment SGI303619[] = {{&S1078[8], 3}, {NULL}};
const Segment SGI303620[] = {{&S1078[13], 11}, {NULL}};
const Segment SGI303621[] = {{&S1078[43], 7}, {NULL}};
const Segment SGI303622[] = {{&S1079[30], 3}, {NULL}};
const Segment SGI303623[] = {{&S1079[8], 3}, {NULL}};
const Segment SGI303624[] = {{&S1079[13], 10}, {NULL}};
const Segment SGI303625[] = {{&S1079[42], 7}, {&S1079[137], 20}, {NULL}};
const Segment SGI303626[] = {{&S1079[35], 14}, {NULL}};
const Segment SGI303627[] = {{&S1079[138], 15}, {NULL}};
const Segment SGI303628[] = {{&S1079[51], 3}, {NULL}};
const Segment SGI303629[] = {{&S1080[30], 8}, {NULL}};
const Segment SGI303630[] = {{&S1080[8], 3}, {NULL}};
const Segment SGI303631[] = {{&S1080[13], 10}, {NULL}};
const Segment SGI303632[] = {{&S1080[47], 7}, {NULL}};
const Segment SGI303633[] = {{&S1081[30], 3}, {NULL}};
const Segment SGI303634[] = {{&S1081[8], 3}, {NULL}};
const Segment SGI303635[] = {{&S1081[13], 10}, {NULL}};
const Segment SGI303636[] = {{&S1081[42], 7}, {NULL}};
const Segment SGI303637[] = {{&S1081[35], 14}, {NULL}};
const Segment SGI303638[] = {{&S1082[30], 3}, {&S1082[30], 3}, {NULL}};
const Segment SGI303639[] = {{&S1082[8], 3}, {&S1082[8], 3}, {NULL}};
const Segment SGI303640[] = {{&S1082[13], 10}, {&S1082[13], 10}, {NULL}};
const Segment SGI303641[] = {{&S1082[42], 7}, {&S1082[93], 7}, {NULL}};
const Segment SGI303642[] = {{&S1082[35], 14}, {&S1082[86], 14}, {NULL}};
const Segment SGI303643[] = {{&S1082[102], 3}, {NULL}};
const Segment SGI303644[] = {{&S1083[30], 3}, {NULL}};
const Segment SGI303645[] = {{&S1083[8], 3}, {NULL}};
const Segment SGI303646[] = {{&S1083[13], 10}, {NULL}};
const Segment SGI303647[] = {{&S1083[42], 9}, {NULL}};
const Segment SGI303648[] = {{&S1083[53], 3}, {NULL}};
const Segment SGI303649[] = {{&S1084[30], 3}, {NULL}};
const Segment SGI303650[] = {{&S1084[8], 3}, {NULL}};
const Segment SGI303651[] = {{&S1084[13], 10}, {NULL}};
const Segment SGI303652[] = {{&S1084[42], 9}, {NULL}};
const Segment SGI303653[] = {{&S1084[53], 3}, {NULL}};
const Segment SGI303654[] = {{&S1085[30], 3}, {NULL}};
const Segment SGI303655[] = {{&S1085[8], 3}, {NULL}};
const Segment SGI303656[] = {{&S1085[13], 10}, {NULL}};
const Segment SGI303657[] = {{&S1085[42], 7}, {NULL}};
const Segment SGI303658[] = {{&S1085[35], 14}, {NULL}};
const Segment SGI303659[] = {{&S1085[138], 15}, {NULL}};
const Segment SGI303660[] = {{&S1085[51], 6}, {NULL}};
const Segment SGI303661[] = {{&S1086[30], 3}, {&S1086[30], 3}, {NULL}};
const Segment SGI303662[] = {{&S1086[8], 3}, {&S1086[8], 3}, {NULL}};
const Segment SGI303663[] = {{&S1086[13], 10}, {&S1086[13], 10}, {NULL}};
const Segment SGI303664[] = {{&S1086[42], 7}, {&S1086[42], 7}, {&S1086[131], 20}, {NULL}};
const Segment SGI303665[] = {{&S1086[35], 14}, {&S1086[35], 14}, {NULL}};
const Segment SGI303666[] = {{&S1086[132], 15}, {&S1086[132], 15}, {NULL}};
const Segment SGI303667[] = {{&S1086[51], 3}, {&S1086[51], 3}, {NULL}};
const Segment SGI303668[] = {{&S1087[30], 3}, {NULL}};
const Segment SGI303669[] = {{&S1087[8], 3}, {NULL}};
const Segment SGI303670[] = {{&S1087[13], 10}, {NULL}};
const Segment SGI303671[] = {{&S1087[42], 7}, {NULL}};
const Segment SGI303672[] = {{&S1087[35], 14}, {NULL}};
const Segment SGI303673[] = {{&S1087[51], 3}, {&S1087[51], 3}, {NULL}};
const Segment SGI303674[] = {{&S1088[30], 3}, {NULL}};
const Segment SGI303675[] = {{&S1088[8], 3}, {NULL}};
const Segment SGI303676[] = {{&S1088[13], 10}, {NULL}};
const Segment SGI303677[] = {{&S1088[42], 9}, {NULL}};
const Segment SGI303678[] = {{&S1088[53], 3}, {NULL}};
const Segment SGI303679[] = {{&S1089[30], 3}, {NULL}};
const Segment SGI303680[] = {{&S1089[8], 3}, {NULL}};
const Segment SGI303681[] = {{&S1089[13], 10}, {NULL}};
const Segment SGI303682[] = {{&S1089[42], 7}, {NULL}};
const Segment SGI303683[] = {{&S1090[30], 3}, {NULL}};
const Segment SGI303684[] = {{&S1090[8], 3}, {NULL}};
const Segment SGI303685[] = {{&S1090[13], 10}, {NULL}};
const Segment SGI303686[] = {{&S1090[42], 7}, {NULL}};
const Segment SGI303687[] = {{&S1090[35], 14}, {NULL}};
const Segment SGI303688[] = {{&S1091[30], 3}, {&S1091[30], 3}, {&S1091[30], 3}, {NULL}};
const Segment SGI303689[] = {{&S1091[8], 3}, {&S1091[8], 3}, {&S1091[8], 3}, {NULL}};
const Segment SGI303690[] = {{&S1091[13], 10}, {&S1091[13], 10}, {&S1091[13], 10}, {NULL}};
const Segment SGI303691[] = {{&S1091[42], 7}, {&S1091[42], 7}, {&S1091[42], 7}, {NULL}};
const Segment SGI303692[] = {{&S1091[35], 14}, {&S1091[35], 14}, {&S1091[35], 14}, {NULL}};
const Segment SGI303693[] = {{&S1091[114], 3}, {&S1091[114], 3}, {NULL}};
const Segment SGI303694[] = {{&S1092[30], 3}, {NULL}};
const Segment SGI303695[] = {{&S1092[8], 3}, {NULL}};
const Segment SGI303696[] = {{&S1092[13], 10}, {NULL}};
const Segment SGI303697[] = {{&S1092[42], 7}, {NULL}};
const Segment SGI303698[] = {{&S1092[35], 14}, {NULL}};
const Segment SGI303699[] = {{&S1092[61], 3}, {NULL}};
const Segment SGI303700[] = {{&S1092[51], 3}, {NULL}};
const Segment SGI303701[] = {{&S1093[18], 3}, {NULL}};
const Segment SGI303702[] = {{&S1093[8], 3}, {NULL}};
const Segment SGI303703[] = {{&S1093[13], 8}, {NULL}};
const Segment SGI303704[] = {{&S1093[30], 7}, {NULL}};
const Segment SGI303705[] = {{&S1093[23], 14}, {NULL}};
const Segment SGI303706[] = {{&S1094[8], 3}, {&S1094[94], 3}, {NULL}};
const Segment SGI303707[] = {{&S1094[8], 3}, {&S1094[72], 3}, {NULL}};
const Segment SGI303708[] = {{&S1094[13], 10}, {&S1094[13], 10}, {NULL}};
const Segment SGI303709[] = {{&S1094[42], 7}, {&S1094[42], 7}, {NULL}};
const Segment SGI303710[] = {{&S1094[35], 14}, {&S1094[35], 14}, {NULL}};
const Segment SGI303711[] = {{&S1094[115], 3}, {NULL}};
const Segment SGI303712[] = {{&S1095[8], 3}, {&S1095[8], 3}, {NULL}};
const Segment SGI303713[] = {{&S1095[8], 3}, {&S1095[72], 3}, {NULL}};
const Segment SGI303714[] = {{&S1095[13], 10}, {&S1095[13], 10}, {NULL}};
const Segment SGI303715[] = {{&S1095[42], 7}, {&S1095[42], 7}, {NULL}};
const Segment SGI303716[] = {{&S1095[35], 14}, {&S1095[35], 14}, {NULL}};
const Segment SGI303717[] = {{&S1095[115], 3}, {NULL}};
const Segment SGI303718[] = {{&S1096[30], 3}, {NULL}};
const Segment SGI303719[] = {{&S1096[8], 3}, {NULL}};
const Segment SGI303720[] = {{&S1096[13], 10}, {NULL}};
const Segment SGI303721[] = {{&S1096[42], 7}, {&S1096[145], 20}, {NULL}};
const Segment SGI303722[] = {{&S1096[35], 14}, {NULL}};
const Segment SGI303723[] = {{&S1096[146], 15}, {NULL}};
const Segment SGI303724[] = {{&S1097[33], 3}, {NULL}};
const Segment SGI303725[] = {{&S1097[8], 3}, {NULL}};
const Segment SGI303726[] = {{&S1097[13], 10}, {NULL}};
const Segment SGI303727[] = {{&S1097[45], 7}, {NULL}};
const Segment SGI303728[] = {{&S1097[80], 6}, {NULL}};
const Segment SGI303729[] = {{&S1098[30], 3}, {NULL}};
const Segment SGI303730[] = {{&S1098[8], 3}, {NULL}};
const Segment SGI303731[] = {{&S1098[13], 10}, {NULL}};
const Segment SGI303732[] = {{&S1098[42], 7}, {NULL}};
const Segment SGI303733[] = {{&S1098[35], 14}, {NULL}};
const Segment SGI303734[] = {{&S1098[110], 15}, {NULL}};
const Segment SGI303735[] = {{&S1098[151], 4}, {NULL}};
const Segment SGI303736[] = {{&S1098[51], 3}, {NULL}};
const Segment SGI303737[] = {{&S1099[30], 3}, {NULL}};
const Segment SGI303738[] = {{&S1099[8], 3}, {NULL}};
const Segment SGI303739[] = {{&S1099[13], 10}, {NULL}};
const Segment SGI303740[] = {{&S1099[42], 7}, {NULL}};
const Segment SGI303741[] = {{&S1099[35], 14}, {NULL}};
const Segment SGI303742[] = {{&S1099[131], 15}, {NULL}};
const Segment SGI303743[] = {{&S1099[152], 9}, {NULL}};
const Segment SGI303744[] = {{&S1100[30], 3}, {NULL}};
const Segment SGI303745[] = {{&S1100[8], 3}, {&S1100[8], 3}, {NULL}};
const Segment SGI303746[] = {{&S1100[13], 10}, {NULL}};
const Segment SGI303747[] = {{&S1100[42], 7}, {NULL}};
const Segment SGI303748[] = {{&S1100[35], 14}, {NULL}};
const Segment SGI303749[] = {{&S1100[51], 3}, {NULL}};
const Segment SGI303750[] = {{&S1101[30], 3}, {&S1101[30], 3}, {NULL}};
const Segment SGI303751[] = {{&S1101[8], 3}, {&S1101[8], 3}, {NULL}};
const Segment SGI303752[] = {{&S1101[13], 10}, {&S1101[13], 10}, {NULL}};
const Segment SGI303753[] = {{&S1101[42], 7}, {&S1101[42], 7}, {NULL}};
const Segment SGI303754[] = {{&S1101[35], 14}, {&S1101[35], 14}, {NULL}};
const Segment SGI303755[] = {{&S1101[134], 4}, {NULL}};
const Segment SGI303756[] = {{&S1101[51], 3}, {&S1101[51], 3}, {NULL}};
const Segment SGI303757[] = {{&S1102[30], 3}, {&S1102[81], 3}, {NULL}};
const Segment SGI303758[] = {{&S1102[8], 3}, {&S1102[8], 3}, {NULL}};
const Segment SGI303759[] = {{&S1102[13], 10}, {&S1102[13], 10}, {NULL}};
const Segment SGI303760[] = {{&S1102[42], 7}, {&S1102[93], 7}, {NULL}};
const Segment SGI303761[] = {{&S1102[35], 14}, {&S1102[86], 14}, {NULL}};
const Segment SGI303762[] = {{&S1102[102], 3}, {NULL}};
const Segment SGI303763[] = {{&S1103[30], 3}, {NULL}};
const Segment SGI303764[] = {{&S1103[8], 3}, {&S1103[8], 3}, {&S1103[8], 3}, {NULL}};
const Segment SGI303765[] = {{&S1103[13], 10}, {NULL}};
const Segment SGI303766[] = {{&S1103[42], 7}, {NULL}};
const Segment SGI303767[] = {{&S1103[35], 14}, {NULL}};
const Segment SGI303768[] = {{&S1103[206], 15}, {NULL}};
const Segment SGI303769[] = {{&S1103[51], 3}, {NULL}};
const Segment SGI303770[] = {{&S1104[30], 3}, {NULL}};
const Segment SGI303771[] = {{&S1104[8], 3}, {NULL}};
const Segment SGI303772[] = {{&S1104[13], 10}, {NULL}};
const Segment SGI303773[] = {{&S1104[42], 7}, {&S1104[75], 12}, {NULL}};
const Segment SGI303774[] = {{&S1104[35], 14}, {NULL}};
const Segment SGI303775[] = {{&S1104[51], 3}, {NULL}};
const Segment SGI303776[] = {{&S1105[30], 3}, {NULL}};
const Segment SGI303777[] = {{&S1105[8], 3}, {NULL}};
const Segment SGI303778[] = {{&S1105[13], 59}, {NULL}};
const Segment SGI303779[] = {{&S1105[42], 7}, {NULL}};
const Segment SGI303780[] = {{&S1105[51], 3}, {NULL}};
const Segment SGI303781[] = {{&S1106[30], 3}, {NULL}};
const Segment SGI303782[] = {{&S1106[8], 3}, {NULL}};
const Segment SGI303783[] = {{&S1106[13], 10}, {NULL}};
const Segment SGI303784[] = {{&S1106[42], 7}, {NULL}};
const Segment SGI303785[] = {{&S1106[35], 14}, {NULL}};
const Segment SGI303786[] = {{&S1106[51], 6}, {NULL}};
const Segment SGI303787[] = {{&S1107[30], 3}, {&S1107[30], 3}, {NULL}};
const Segment SGI303788[] = {{&S1107[8], 3}, {&S1107[8], 3}, {NULL}};
const Segment SGI303789[] = {{&S1107[13], 10}, {&S1107[13], 10}, {NULL}};
const Segment SGI303790[] = {{&S1107[42], 7}, {&S1107[42], 7}, {NULL}};
const Segment SGI303791[] = {{&S1107[35], 14}, {&S1107[35], 14}, {NULL}};
const Segment SGI303792[] = {{&S1107[196], 15}, {NULL}};
const Segment SGI303793[] = {{&S1107[115], 3}, {NULL}};
const Segment SGI303794[] = {{&S1108[30], 3}, {NULL}};
const Segment SGI303795[] = {{&S1108[8], 3}, {&S1108[8], 3}, {&S1108[8], 3}, {&S1108[8], 3}, {NULL}};
const Segment SGI303796[] = {{&S1108[13], 10}, {NULL}};
const Segment SGI303797[] = {{&S1108[42], 7}, {NULL}};
const Segment SGI303798[] = {{&S1108[35], 14}, {NULL}};
const Segment SGI303799[] = {{&S1108[51], 3}, {NULL}};
const Segment SGI303800[] = {{&S1109[30], 3}, {NULL}};
const Segment SGI303801[] = {{&S1109[8], 3}, {NULL}};
const Segment SGI303802[] = {{&S1109[13], 10}, {NULL}};
const Segment SGI303803[] = {{&S1109[42], 7}, {NULL}};
const Segment SGI303804[] = {{&S1109[35], 14}, {NULL}};
const Segment SGI303805[] = {{&S1109[131], 15}, {NULL}};
const Segment SGI303806[] = {{&S1110[30], 3}, {NULL}};
const Segment SGI303807[] = {{&S1110[8], 3}, {NULL}};
const Segment SGI303808[] = {{&S1110[13], 10}, {NULL}};
const Segment SGI303809[] = {{&S1110[42], 7}, {NULL}};
const Segment SGI303810[] = {{&S1110[35], 14}, {NULL}};
const Segment SGI303811[] = {{&S1110[131], 9}, {NULL}};
const Segment SGI303812[] = {{&S1111[30], 3}, {NULL}};
const Segment SGI303813[] = {{&S1111[8], 3}, {NULL}};
const Segment SGI303814[] = {{&S1111[13], 10}, {NULL}};
const Segment SGI303815[] = {{&S1111[42], 7}, {&S1111[137], 20}, {NULL}};
const Segment SGI303816[] = {{&S1111[35], 14}, {NULL}};
const Segment SGI303817[] = {{&S1111[138], 15}, {&S1111[138], 15}, {NULL}};
const Segment SGI303818[] = {{&S1111[51], 3}, {NULL}};
const Segment SGI303819[] = {{&S1112[30], 3}, {NULL}};
const Segment SGI303820[] = {{&S1112[8], 3}, {NULL}};
const Segment SGI303821[] = {{&S1112[13], 10}, {NULL}};
const Segment SGI303822[] = {{&S1112[42], 4}, {NULL}};
const Segment SGI303823[] = {{&S1113[30], 3}, {NULL}};
const Segment SGI303824[] = {{&S1113[8], 3}, {&S1113[8], 3}, {&S1113[8], 3}, {NULL}};
const Segment SGI303825[] = {{&S1113[13], 10}, {NULL}};
const Segment SGI303826[] = {{&S1113[42], 7}, {NULL}};
const Segment SGI303827[] = {{&S1113[35], 14}, {NULL}};
const Segment SGI303828[] = {{&S1114[30], 3}, {NULL}};
const Segment SGI303829[] = {{&S1114[8], 3}, {NULL}};
const Segment SGI303830[] = {{&S1114[13], 10}, {NULL}};
const Segment SGI303831[] = {{&S1114[42], 7}, {NULL}};
const Segment SGI303832[] = {{&S1114[35], 14}, {NULL}};
const Segment SGI303833[] = {{&S1114[136], 15}, {NULL}};
const Segment SGI303834[] = {{&S1114[157], 9}, {NULL}};
const Segment SGI303835[] = {{&S1114[64], 3}, {NULL}};
const Segment SGI303836[] = {{&S1115[30], 3}, {&S1115[30], 3}, {NULL}};
const Segment SGI303837[] = {{&S1115[8], 3}, {&S1115[8], 3}, {NULL}};
const Segment SGI303838[] = {{&S1115[13], 10}, {&S1115[69], 3}, {NULL}};
const Segment SGI303839[] = {{&S1115[42], 7}, {NULL}};
const Segment SGI303840[] = {{&S1115[35], 14}, {&S1115[69], 3}, {NULL}};
const Segment SGI303841[] = {{&S1115[51], 3}, {NULL}};
const Segment SGI303842[] = {{&S1116[32], 3}, {NULL}};
const Segment SGI303843[] = {{&S1116[10], 3}, {NULL}};
const Segment SGI303844[] = {{&S1116[15], 10}, {NULL}};
const Segment SGI303845[] = {{&S1116[44], 7}, {&S1116[103], 3}, {NULL}};
const Segment SGI303846[] = {{&S1116[37], 14}, {NULL}};
const Segment SGI303847[] = {{&S1117[5], 1}, {&S1117[47], 3}, {NULL}};
const Segment SGI303848[] = {{&S1117[25], 3}, {NULL}};
const Segment SGI303849[] = {{&S1117[30], 10}, {NULL}};
const Segment SGI303850[] = {{&S1117[59], 7}, {NULL}};
const Segment SGI303851[] = {{&S1118[30], 3}, {NULL}};
const Segment SGI303852[] = {{&S1118[8], 3}, {NULL}};
const Segment SGI303853[] = {{&S1118[13], 10}, {NULL}};
const Segment SGI303854[] = {{&S1118[42], 7}, {&S1118[145], 20}, {NULL}};
const Segment SGI303855[] = {{&S1118[35], 14}, {NULL}};
const Segment SGI303856[] = {{&S1118[146], 15}, {&S1118[146], 15}, {NULL}};
const Segment SGI303857[] = {{&S1119[30], 3}, {NULL}};
const Segment SGI303858[] = {{&S1119[8], 3}, {NULL}};
const Segment SGI303859[] = {{&S1119[13], 10}, {NULL}};
const Segment SGI303860[] = {{&S1119[42], 7}, {NULL}};
const Segment SGI303861[] = {{&S1119[35], 14}, {NULL}};
const Segment SGI303862[] = {{&S1119[51], 3}, {NULL}};
const Segment SGI303863[] = {{&S1119[104], 3}, {NULL}};
const Segment SGI303864[] = {{&S1120[30], 3}, {&S1120[30], 3}, {&S1120[30], 3}, {&S1120[30], 3}, {NULL}};
const Segment SGI303865[] = {{&S1120[8], 3}, {&S1120[8], 3}, {&S1120[8], 3}, {&S1120[8], 3}, {NULL}};
const Segment SGI303866[] = {{&S1120[13], 10}, {&S1120[13], 10}, {&S1120[13], 10}, {&S1120[13], 10}, {NULL}};
const Segment SGI303867[] = {{&S1120[42], 7}, {&S1120[42], 7}, {&S1120[42], 7}, {&S1120[42], 7}, {NULL}};
const Segment SGI303868[] = {{&S1120[35], 14}, {&S1120[35], 14}, {&S1120[35], 14}, {&S1120[35], 14}, {NULL}};
const Segment SGI303869[] = {{&S1120[142], 3}, {&S1120[142], 3}, {&S1120[142], 3}, {NULL}};
const Segment SGI303870[] = {{&S1121[75], 13}, {NULL}};
const Segment SGI303871[] = {{&S1121[8], 3}, {NULL}};
const Segment SGI303872[] = {{&S1121[13], 10}, {NULL}};
const Segment SGI303873[] = {{&S1121[61], 7}, {NULL}};
const Segment SGI303874[] = {{&S1122[30], 3}, {NULL}};
const Segment SGI303875[] = {{&S1122[8], 3}, {NULL}};
const Segment SGI303876[] = {{&S1122[13], 10}, {NULL}};
const Segment SGI303877[] = {{&S1122[42], 7}, {NULL}};
const Segment SGI303878[] = {{&S1122[35], 14}, {NULL}};
const Segment SGI303879[] = {{&S1122[138], 15}, {NULL}};
const Segment SGI303880[] = {{&S1122[179], 4}, {NULL}};
const Segment SGI303881[] = {{&S1122[51], 6}, {NULL}};
const Segment SGI303882[] = {{&S1123[30], 3}, {NULL}};
const Segment SGI303883[] = {{&S1123[8], 3}, {NULL}};
const Segment SGI303884[] = {{&S1123[13], 10}, {NULL}};
const Segment SGI303885[] = {{&S1123[42], 7}, {NULL}};
const Segment SGI303886[] = {{&S1123[35], 14}, {NULL}};
const Segment SGI303887[] = {{&S1123[51], 3}, {NULL}};
const Segment SGI303888[] = {{&S1123[66], 3}, {NULL}};
const Segment SGI303889[] = {{&S1124[30], 3}, {NULL}};
const Segment SGI303890[] = {{&S1124[8], 3}, {NULL}};
const Segment SGI303891[] = {{&S1124[13], 10}, {NULL}};
const Segment SGI303892[] = {{&S1124[42], 7}, {NULL}};
const Segment SGI303893[] = {{&S1124[35], 14}, {NULL}};
const Segment SGI303894[] = {{&S1124[138], 15}, {NULL}};
const Segment SGI303895[] = {{&S1124[51], 6}, {NULL}};
const Segment SGI303896[] = {{&S1125[31], 13}, {NULL}};
const Segment SGI303897[] = {{&S1125[8], 3}, {NULL}};
const Segment SGI303898[] = {{&S1125[13], 11}, {NULL}};
const Segment SGI303899[] = {{&S1125[64], 32}, {NULL}};
const Segment SGI303900[] = {{&S1126[30], 3}, {NULL}};
const Segment SGI303901[] = {{&S1126[8], 3}, {NULL}};
const Segment SGI303902[] = {{&S1126[13], 10}, {NULL}};
const Segment SGI303903[] = {{&S1126[42], 7}, {NULL}};
const Segment SGI303904[] = {{&S1126[35], 14}, {NULL}};
const Segment SGI303905[] = {{&S1126[131], 15}, {NULL}};
const Segment SGI303906[] = {{&S1126[172], 4}, {NULL}};
const Segment SGI303907[] = {{&S1127[75], 13}, {NULL}};
const Segment SGI303908[] = {{&S1127[8], 3}, {NULL}};
const Segment SGI303909[] = {{&S1127[13], 10}, {NULL}};
const Segment SGI303910[] = {{&S1127[61], 7}, {NULL}};
const Segment SGI303911[] = {{&S1128[8], 3}, {NULL}};
const Segment SGI303912[] = {{&S1128[13], 10}, {NULL}};
const Segment SGI303913[] = {{&S1128[42], 7}, {NULL}};
const Segment SGI303914[] = {{&S1128[35], 14}, {NULL}};
const Segment SGI303915[] = {{&S1128[51], 3}, {NULL}};
const Segment SGI303916[] = {{&S1129[31], 3}, {NULL}};
const Segment SGI303917[] = {{&S1129[8], 3}, {NULL}};
const Segment SGI303918[] = {{&S1129[13], 11}, {NULL}};
const Segment SGI303919[] = {{&S1129[44], 7}, {NULL}};
const Segment SGI303920[] = {{&S1129[54], 3}, {NULL}};
const Segment SGI303921[] = {{&S1130[30], 3}, {NULL}};
const Segment SGI303922[] = {{&S1130[8], 3}, {NULL}};
const Segment SGI303923[] = {{&S1130[13], 10}, {NULL}};
const Segment SGI303924[] = {{&S1130[42], 7}, {NULL}};
const Segment SGI303925[] = {{&S1130[35], 14}, {NULL}};
const Segment SGI303926[] = {{&S1130[131], 15}, {&S1130[131], 15}, {NULL}};
const Segment SGI303927[] = {{&S1130[210], 9}, {NULL}};
const Segment SGI303928[] = {{&S1131[30], 3}, {&S1131[30], 3}, {NULL}};
const Segment SGI303929[] = {{&S1131[8], 3}, {&S1131[8], 3}, {NULL}};
const Segment SGI303930[] = {{&S1131[13], 10}, {&S1131[13], 10}, {NULL}};
const Segment SGI303931[] = {{&S1131[42], 7}, {&S1131[42], 7}, {NULL}};
const Segment SGI303932[] = {{&S1131[35], 14}, {&S1131[35], 14}, {NULL}};
const Segment SGI303933[] = {{&S1131[226], 15}, {NULL}};
const Segment SGI303934[] = {{&S1131[51], 6}, {NULL}};
const Segment SGI303935[] = {{&S1131[152], 3}, {NULL}};
const Segment SGI303936[] = {{&S1132[30], 3}, {NULL}};
const Segment SGI303937[] = {{&S1132[8], 3}, {NULL}};
const Segment SGI303938[] = {{&S1132[13], 10}, {NULL}};
const Segment SGI303939[] = {{&S1132[42], 7}, {NULL}};
const Segment SGI303940[] = {{&S1132[35], 14}, {NULL}};
const Segment SGI303941[] = {{&S1133[31], 3}, {NULL}};
const Segment SGI303942[] = {{&S1133[8], 3}, {NULL}};
const Segment SGI303943[] = {{&S1133[13], 23}, {NULL}};
const Segment SGI303944[] = {{&S1133[55], 7}, {NULL}};
const Segment SGI303945[] = {{&S1134[32], 3}, {NULL}};
const Segment SGI303946[] = {{&S1134[8], 3}, {NULL}};
const Segment SGI303947[] = {{&S1134[80], 7}, {NULL}};
const Segment SGI303948[] = {{&S1134[73], 14}, {NULL}};
const Segment SGI303949[] = {{&S1135[30], 3}, {NULL}};
const Segment SGI303950[] = {{&S1135[8], 3}, {NULL}};
const Segment SGI303951[] = {{&S1135[13], 10}, {NULL}};
const Segment SGI303952[] = {{&S1135[42], 7}, {NULL}};
const Segment SGI303953[] = {{&S1135[35], 14}, {NULL}};
const Segment SGI303954[] = {{&S1135[131], 15}, {NULL}};
const Segment SGI303955[] = {{&S1135[187], 4}, {NULL}};
const Segment SGI303956[] = {{&S1135[152], 9}, {NULL}};
const Segment SGI303957[] = {{&S1136[30], 3}, {&S1136[30], 3}, {NULL}};
const Segment SGI303958[] = {{&S1136[8], 3}, {&S1136[8], 3}, {NULL}};
const Segment SGI303959[] = {{&S1136[13], 10}, {&S1136[13], 10}, {NULL}};
const Segment SGI303960[] = {{&S1136[42], 7}, {&S1136[42], 7}, {&S1136[139], 20}, {NULL}};
const Segment SGI303961[] = {{&S1136[35], 14}, {&S1136[35], 14}, {NULL}};
const Segment SGI303962[] = {{&S1136[140], 15}, {NULL}};
const Segment SGI303963[] = {{&S1136[51], 3}, {&S1136[51], 3}, {NULL}};
const Segment SGI303964[] = {{&S1137[30], 3}, {NULL}};
const Segment SGI303965[] = {{&S1137[8], 3}, {NULL}};
const Segment SGI303966[] = {{&S1137[13], 10}, {NULL}};
const Segment SGI303967[] = {{&S1137[42], 7}, {NULL}};
const Segment SGI303968[] = {{&S1137[35], 14}, {NULL}};
const Segment SGI303969[] = {{&S1137[144], 4}, {NULL}};
const Segment SGI303970[] = {{&S1138[30], 3}, {NULL}};
const Segment SGI303971[] = {{&S1138[8], 3}, {NULL}};
const Segment SGI303972[] = {{&S1138[13], 10}, {NULL}};
const Segment SGI303973[] = {{&S1138[42], 7}, {NULL}};
const Segment SGI303974[] = {{&S1138[35], 14}, {NULL}};
const Segment SGI303975[] = {{&S1138[51], 6}, {NULL}};
const Segment SGI303976[] = {{&S1138[151], 9}, {NULL}};
const Segment SGI303977[] = {{&S1139[30], 3}, {&S1139[30], 3}, {NULL}};
const Segment SGI303978[] = {{&S1139[8], 3}, {&S1139[8], 3}, {NULL}};
const Segment SGI303979[] = {{&S1139[13], 10}, {&S1139[13], 10}, {NULL}};
const Segment SGI303980[] = {{&S1139[42], 7}, {&S1139[42], 7}, {NULL}};
const Segment SGI303981[] = {{&S1139[35], 14}, {&S1139[35], 14}, {NULL}};
const Segment SGI303982[] = {{&S1139[196], 15}, {NULL}};
const Segment SGI303983[] = {{&S1139[268], 9}, {NULL}};
const Segment SGI303984[] = {{&S1139[122], 3}, {NULL}};
const Segment SGI303985[] = {{&S1140[30], 3}, {NULL}};
const Segment SGI303986[] = {{&S1140[8], 3}, {NULL}};
const Segment SGI303987[] = {{&S1140[13], 10}, {NULL}};
const Segment SGI303988[] = {{&S1140[42], 7}, {NULL}};
const Segment SGI303989[] = {{&S2365[30], 4}, {NULL}};
const Segment SGI303990[] = {{&S2365[8], 3}, {NULL}};
const Segment SGI303991[] = {{&S2365[13], 10}, {NULL}};
const Segment SGI303992[] = {{&S2365[43], 7}, {NULL}};
const Segment SGI303993[] = {{&S2365[36], 14}, {NULL}};
const Segment SGI303994[] = {{&S2365[52], 6}, {NULL}};
const Segment SGI303995[] = {{&S2366[30], 4}, {NULL}};
const Segment SGI303996[] = {{&S2366[8], 3}, {NULL}};
const Segment SGI303997[] = {{&S2366[13], 10}, {NULL}};
const Segment SGI303998[] = {{&S2366[43], 7}, {NULL}};
const Segment SGI303999[] = {{&S2366[36], 14}, {NULL}};
const Segment SGI304000[] = {{&S2367[30], 4}, {NULL}};
const Segment SGI304001[] = {{&S2367[8], 3}, {NULL}};
const Segment SGI304002[] = {{&S2367[13], 10}, {NULL}};
const Segment SGI304003[] = {{&S2367[43], 7}, {NULL}};
const Segment SGI304004[] = {{&S2367[36], 14}, {NULL}};
const Segment SGI304005[] = {{&S2368[30], 4}, {NULL}};
const Segment SGI304006[] = {{&S2368[8], 3}, {NULL}};
const Segment SGI304007[] = {{&S2368[13], 10}, {NULL}};
const Segment SGI304008[] = {{&S2368[43], 7}, {NULL}};
const Segment SGI304009[] = {{&S2368[36], 14}, {NULL}};
const Segment SGI304010[] = {{&S2368[52], 5}, {NULL}};
const Segment SGI304011[] = {{&S2369[30], 4}, {NULL}};
const Segment SGI304012[] = {{&S2369[8], 3}, {NULL}};
const Segment SGI304013[] = {{&S2369[13], 10}, {NULL}};
const Segment SGI304014[] = {{&S2369[43], 7}, {NULL}};
const Segment SGI304015[] = {{&S2369[36], 14}, {NULL}};
const Segment SGI304016[] = {{&S2369[77], 3}, {NULL}};
const Segment SGI304017[] = {{&S2370[30], 4}, {NULL}};
const Segment SGI304018[] = {{&S2370[8], 3}, {NULL}};
const Segment SGI304019[] = {{&S2370[13], 10}, {NULL}};
const Segment SGI304020[] = {{&S2370[43], 7}, {NULL}};
const Segment SGI304021[] = {{&S2370[36], 14}, {NULL}};
const Segment SGI304022[] = {{&S2371[31], 4}, {NULL}};
const Segment SGI304023[] = {{&S2371[44], 7}, {NULL}};
const Segment SGI304024[] = {{&S2372[30], 4}, {NULL}};
const Segment SGI304025[] = {{&S2372[8], 3}, {NULL}};
const Segment SGI304026[] = {{&S2372[13], 10}, {NULL}};
const Segment SGI304027[] = {{&S2372[43], 7}, {NULL}};
const Segment SGI304028[] = {{&S2373[30], 4}, {NULL}};
const Segment SGI304029[] = {{&S2373[8], 3}, {NULL}};
const Segment SGI304030[] = {{&S2373[13], 10}, {NULL}};
const Segment SGI304031[] = {{&S2373[43], 7}, {NULL}};
const Segment SGI304032[] = {{&S2373[36], 14}, {NULL}};
const Segment SGI304033[] = {{&S2373[52], 3}, {NULL}};
const Segment SGI304034[] = {{&S2374[30], 4}, {NULL}};
const Segment SGI304035[] = {{&S2374[8], 3}, {NULL}};
const Segment SGI304036[] = {{&S2374[13], 10}, {NULL}};
const Segment SGI304037[] = {{&S2374[43], 7}, {NULL}};
const Segment SGI304038[] = {{&S2374[36], 14}, {NULL}};
const Segment SGI304039[] = {{&S2374[139], 15}, {NULL}};
const Segment SGI304040[] = {{&S2374[52], 6}, {NULL}};
const Segment SGI304041[] = {{&S2375[30], 4}, {NULL}};
const Segment SGI304042[] = {{&S2375[8], 3}, {NULL}};
const Segment SGI304043[] = {{&S2375[13], 10}, {NULL}};
const Segment SGI304044[] = {{&S2375[43], 7}, {NULL}};
const Segment SGI304045[] = {{&S2375[36], 14}, {NULL}};
const Segment SGI304046[] = {{&S2375[132], 15}, {NULL}};
const Segment SGI304047[] = {{&S2376[21], 4}, {NULL}};
const Segment SGI304048[] = {{&S2376[8], 3}, {NULL}};
const Segment SGI304049[] = {{&S2376[11], 1}, {NULL}};
const Segment SGI304050[] = {{&S2376[34], 7}, {NULL}};
const Segment SGI304051[] = {{&S2376[27], 14}, {NULL}};
const Segment SGI304052[] = {{&S2376[43], 6}, {NULL}};
const Segment SGI304053[] = {{&S2377[30], 4}, {&S2377[73], 3}, {NULL}};
const Segment SGI304054[] = {{&S2377[8], 3}, {NULL}};
const Segment SGI304055[] = {{&S2377[13], 10}, {NULL}};
const Segment SGI304056[] = {{&S2377[43], 7}, {NULL}};
const Segment SGI304057[] = {{&S2377[36], 14}, {NULL}};
const Segment SGI304058[] = {{&S2378[30], 4}, {NULL}};
const Segment SGI304059[] = {{&S2378[8], 3}, {NULL}};
const Segment SGI304060[] = {{&S2378[13], 10}, {NULL}};
const Segment SGI304061[] = {{&S2378[43], 7}, {NULL}};
const Segment SGI304062[] = {{&S2378[36], 14}, {NULL}};
const Segment SGI304063[] = {{&S2378[52], 6}, {NULL}};
const Segment SGI304064[] = {{&S2378[82], 3}, {NULL}};
const Segment SGI304065[] = {{&S2379[30], 4}, {NULL}};
const Segment SGI304066[] = {{&S2379[8], 3}, {NULL}};
const Segment SGI304067[] = {{&S2379[13], 10}, {NULL}};
const Segment SGI304068[] = {{&S2379[43], 7}, {NULL}};
const Segment SGI304069[] = {{&S2379[36], 14}, {NULL}};
const Segment SGI304070[] = {{&S2379[75], 3}, {NULL}};
const Segment SGI304071[] = {{&S2380[30], 4}, {NULL}};
const Segment SGI304072[] = {{&S2380[8], 3}, {NULL}};
const Segment SGI304073[] = {{&S2380[13], 10}, {NULL}};
const Segment SGI304074[] = {{&S2380[43], 2}, {NULL}};
const Segment SGI304075[] = {{&S2381[30], 4}, {NULL}};
const Segment SGI304076[] = {{&S2381[8], 3}, {NULL}};
const Segment SGI304077[] = {{&S2381[13], 10}, {NULL}};
const Segment SGI304078[] = {{&S2381[43], 7}, {NULL}};
const Segment SGI304079[] = {{&S2381[36], 14}, {NULL}};
const Segment SGI304080[] = {{&S2381[75], 3}, {NULL}};
const Segment SGI304081[] = {{&S2382[30], 4}, {NULL}};
const Segment SGI304082[] = {{&S2382[8], 3}, {NULL}};
const Segment SGI304083[] = {{&S2382[13], 10}, {NULL}};
const Segment SGI304084[] = {{&S2382[43], 7}, {NULL}};
const Segment SGI304085[] = {{&S2382[36], 14}, {NULL}};
const Segment SGI304086[] = {{&S2382[52], 6}, {NULL}};
const Segment SGI304087[] = {{&S2382[82], 3}, {NULL}};
const Segment SGI304088[] = {{&S2383[30], 4}, {NULL}};
const Segment SGI304089[] = {{&S2383[8], 3}, {NULL}};
const Segment SGI304090[] = {{&S2383[13], 10}, {NULL}};
const Segment SGI304091[] = {{&S2383[43], 7}, {NULL}};
const Segment SGI304092[] = {{&S2384[21], 4}, {NULL}};
const Segment SGI304093[] = {{&S2384[8], 3}, {NULL}};
const Segment SGI304094[] = {{&S2384[11], 1}, {NULL}};
const Segment SGI304095[] = {{&S2384[34], 7}, {NULL}};
const Segment SGI304096[] = {{&S2384[27], 14}, {NULL}};
const Segment SGI304097[] = {{&S2385[30], 4}, {NULL}};
const Segment SGI304098[] = {{&S2385[8], 3}, {NULL}};
const Segment SGI304099[] = {{&S2385[13], 10}, {NULL}};
const Segment SGI304100[] = {{&S2385[43], 7}, {NULL}};
const Segment SGI304101[] = {{&S2385[36], 14}, {NULL}};
const Segment SGI304102[] = {{&S2385[52], 6}, {NULL}};
const Segment SGI304103[] = {{&S2386[30], 4}, {NULL}};
const Segment SGI304104[] = {{&S2386[8], 3}, {NULL}};
const Segment SGI304105[] = {{&S2386[13], 10}, {NULL}};
const Segment SGI304106[] = {{&S2386[43], 7}, {NULL}};
const Segment SGI304107[] = {{&S2386[36], 14}, {NULL}};
const Segment SGI304108[] = {{&S2386[73], 3}, {NULL}};
const Segment SGI304109[] = {{&S2387[30], 4}, {NULL}};
const Segment SGI304110[] = {{&S2387[8], 3}, {NULL}};
const Segment SGI304111[] = {{&S2387[13], 10}, {NULL}};
const Segment SGI304112[] = {{&S2387[43], 7}, {NULL}};
const Segment SGI304113[] = {{&S2387[36], 14}, {NULL}};
const Segment SGI304114[] = {{&S2387[132], 15}, {NULL}};
const Segment SGI304115[] = {{&S2387[163], 9}, {NULL}};
const Segment SGI304116[] = {{&S2388[30], 4}, {NULL}};
const Segment SGI304117[] = {{&S2388[8], 3}, {NULL}};
const Segment SGI304118[] = {{&S2388[13], 10}, {NULL}};
const Segment SGI304119[] = {{&S2388[43], 7}, {&S2388[80], 5}, {NULL}};
const Segment SGI304120[] = {{&S2388[36], 14}, {NULL}};
const Segment SGI304121[] = {{&S2389[30], 4}, {&S2389[80], 3}, {NULL}};
const Segment SGI304122[] = {{&S2389[8], 3}, {NULL}};
const Segment SGI304123[] = {{&S2389[13], 10}, {NULL}};
const Segment SGI304124[] = {{&S2389[43], 7}, {NULL}};
const Segment SGI304125[] = {{&S2389[36], 14}, {NULL}};
const Segment SGI304126[] = {{&S2389[52], 6}, {NULL}};
const Segment SGI304127[] = {{&S2390[30], 4}, {NULL}};
const Segment SGI304128[] = {{&S2390[8], 3}, {NULL}};
const Segment SGI304129[] = {{&S2390[13], 10}, {NULL}};
const Segment SGI304130[] = {{&S2390[43], 7}, {NULL}};
const Segment SGI304131[] = {{&S2390[36], 14}, {NULL}};
const Segment SGI304132[] = {{&S2390[80], 3}, {NULL}};
const Segment SGI304133[] = {{&S2390[52], 6}, {NULL}};
const Segment SGI304134[] = {{&S2391[24], 4}, {NULL}};
const Segment SGI304135[] = {{&S2391[37], 7}, {NULL}};
const Segment SGI304136[] = {{&S2391[30], 14}, {NULL}};
const Segment SGI304137[] = {{&S2391[46], 6}, {NULL}};
const Segment SGI304138[] = {{&S2392[30], 4}, {NULL}};
const Segment SGI304139[] = {{&S2392[8], 3}, {NULL}};
const Segment SGI304140[] = {{&S2392[13], 10}, {NULL}};
const Segment SGI304141[] = {{&S2392[43], 7}, {NULL}};
const Segment SGI304142[] = {{&S2392[36], 14}, {NULL}};
const Segment SGI304143[] = {{&S2392[71], 5}, {NULL}};
const Segment SGI304144[] = {{&S2393[30], 4}, {NULL}};
const Segment SGI304145[] = {{&S2393[8], 3}, {NULL}};
const Segment SGI304146[] = {{&S2393[13], 10}, {NULL}};
const Segment SGI304147[] = {{&S2393[43], 7}, {&S2393[87], 5}, {NULL}};
const Segment SGI304148[] = {{&S2393[36], 14}, {NULL}};
const Segment SGI304149[] = {{&S2393[52], 6}, {NULL}};
const Segment SGI304150[] = {{&S2394[30], 4}, {&S2394[95], 3}, {NULL}};
const Segment SGI304151[] = {{&S2394[8], 3}, {&S2394[73], 3}, {NULL}};
const Segment SGI304152[] = {{&S2394[13], 10}, {&S2394[13], 10}, {NULL}};
const Segment SGI304153[] = {{&S2394[43], 7}, {&S2394[43], 7}, {NULL}};
const Segment SGI304154[] = {{&S2394[36], 14}, {&S2394[36], 14}, {NULL}};
const Segment SGI304155[] = {{&S2394[196], 15}, {NULL}};
const Segment SGI304156[] = {{&S2395[30], 4}, {NULL}};
const Segment SGI304157[] = {{&S2395[8], 3}, {NULL}};
const Segment SGI304158[] = {{&S2395[13], 10}, {NULL}};
const Segment SGI304159[] = {{&S2395[43], 7}, {NULL}};
const Segment SGI304160[] = {{&S2395[36], 14}, {NULL}};
const Segment SGI304161[] = {{&S2395[144], 15}, {NULL}};
const Segment SGI304162[] = {{&S2395[52], 5}, {NULL}};
const Segment SGI304163[] = {{&S2396[30], 4}, {&S2396[102], 3}, {NULL}};
const Segment SGI304164[] = {{&S2396[8], 3}, {&S2396[80], 3}, {NULL}};
const Segment SGI304165[] = {{&S2396[13], 10}, {&S2396[85], 10}, {NULL}};
const Segment SGI304166[] = {{&S2396[43], 7}, {&S2396[43], 3}, {NULL}};
const Segment SGI304167[] = {{&S2396[36], 14}, {NULL}};
const Segment SGI304168[] = {{&S2396[52], 6}, {NULL}};
const Segment SGI304169[] = {{&S2397[30], 4}, {&S2397[60], 3}, {NULL}};
const Segment SGI304170[] = {{&S2397[8], 3}, {&S2397[73], 3}, {NULL}};
const Segment SGI304171[] = {{&S2397[13], 10}, {&S2397[13], 10}, {NULL}};
const Segment SGI304172[] = {{&S2397[43], 7}, {NULL}};
const Segment SGI304173[] = {{&S2397[36], 14}, {NULL}};
const Segment SGI304174[] = {{&S2398[30], 4}, {NULL}};
const Segment SGI304175[] = {{&S2398[8], 3}, {NULL}};
const Segment SGI304176[] = {{&S2398[13], 10}, {NULL}};
const Segment SGI304177[] = {{&S2398[43], 7}, {NULL}};
const Segment SGI304178[] = {{&S2398[36], 14}, {NULL}};
const Segment SGI304179[] = {{&S2398[144], 15}, {NULL}};
const Segment SGI304180[] = {{&S2398[52], 5}, {NULL}};
const Segment SGI304181[] = {{&S2398[185], 9}, {NULL}};
const Segment SGI304182[] = {{&S2399[30], 4}, {NULL}};
const Segment SGI304183[] = {{&S2399[8], 3}, {NULL}};
const Segment SGI304184[] = {{&S2399[13], 10}, {NULL}};
const Segment SGI304185[] = {{&S2399[43], 7}, {NULL}};
const Segment SGI304186[] = {{&S2399[36], 14}, {NULL}};
const Segment SGI304187[] = {{&S2399[139], 15}, {NULL}};
const Segment SGI304188[] = {{&S2399[170], 4}, {NULL}};
const Segment SGI304189[] = {{&S2399[52], 6}, {NULL}};
const Segment SGI304190[] = {{&S2400[30], 4}, {&S2400[102], 3}, {NULL}};
const Segment SGI304191[] = {{&S2400[8], 3}, {&S2400[80], 3}, {NULL}};
const Segment SGI304192[] = {{&S2400[13], 10}, {&S2400[13], 10}, {NULL}};
const Segment SGI304193[] = {{&S2400[43], 7}, {&S2400[43], 7}, {NULL}};
const Segment SGI304194[] = {{&S2400[36], 14}, {&S2400[36], 14}, {NULL}};
const Segment SGI304195[] = {{&S2400[203], 15}, {NULL}};
const Segment SGI304196[] = {{&S2400[52], 6}, {NULL}};
const Segment SGI304197[] = {{&S2401[25], 4}, {NULL}};
const Segment SGI304198[] = {{&S2401[38], 7}, {NULL}};
const Segment SGI304199[] = {{&S2401[31], 14}, {NULL}};
const Segment SGI304200[] = {{&S2402[30], 4}, {NULL}};
const Segment SGI304201[] = {{&S2402[8], 3}, {NULL}};
const Segment SGI304202[] = {{&S2402[13], 10}, {NULL}};
const Segment SGI304203[] = {{&S2402[43], 7}, {NULL}};
const Segment SGI304204[] = {{&S2402[36], 14}, {NULL}};
const Segment SGI304205[] = {{&S2402[196], 15}, {NULL}};
const Segment SGI304206[] = {{&S2402[227], 4}, {NULL}};
const Segment SGI304207[] = {{&S2402[237], 9}, {NULL}};
const Segment SGI304208[] = {{&S2403[30], 4}, {&S2403[73], 3}, {&S2403[73], 3}, {NULL}};
const Segment SGI304209[] = {{&S2403[8], 3}, {NULL}};
const Segment SGI304210[] = {{&S2403[13], 10}, {NULL}};
const Segment SGI304211[] = {{&S2403[43], 7}, {NULL}};
const Segment SGI304212[] = {{&S2403[36], 14}, {NULL}};
const Segment SGI304213[] = {{&S2404[31], 4}, {NULL}};
const Segment SGI304214[] = {{&S2404[8], 3}, {NULL}};
const Segment SGI304215[] = {{&S2404[44], 7}, {NULL}};
const Segment SGI304216[] = {{&S2404[37], 14}, {NULL}};
const Segment SGI304217[] = {{&S2405[18], 4}, {NULL}};
const Segment SGI304218[] = {{&S2405[8], 3}, {NULL}};
const Segment SGI304219[] = {{&S2405[13], 9}, {NULL}};
const Segment SGI304220[] = {{&S2405[31], 7}, {NULL}};
const Segment SGI304221[] = {{&S2405[24], 14}, {NULL}};
const Segment SGI304222[] = {{&S2405[40], 6}, {NULL}};
const Segment SGI304223[] = {{&S2406[30], 4}, {&S2406[67], 3}, {&S2406[67], 3}, {&S2406[173], 3}, {&S2406[183], 3}, {NULL}};
const Segment SGI304224[] = {{&S2406[8], 3}, {NULL}};
const Segment SGI304225[] = {{&S2406[13], 10}, {NULL}};
const Segment SGI304226[] = {{&S2406[43], 7}, {&S2406[43], 3}, {&S2406[126], 2}, {&S2406[43], 3}, {&S2406[126], 2}, {NULL}};
const Segment SGI304227[] = {{&S2406[36], 14}, {NULL}};
const Segment SGI304228[] = {{&S2406[52], 6}, {NULL}};
const Segment SGI304229[] = {{&S2410[30], 3}, {NULL}};
const Segment SGI304230[] = {{&S2410[176], 9}, {NULL}};
const Segment SGI304231[] = {{&S2410[8], 3}, {NULL}};
const Segment SGI304232[] = {{&S2410[13], 10}, {NULL}};
const Segment SGI304233[] = {{&S2410[42], 7}, {NULL}};
const Segment SGI304234[] = {{&S2410[35], 14}, {NULL}};
const Segment SGI304235[] = {{&S2410[51], 6}, {NULL}};
const Segment SGI304236[] = {{&S2411[30], 3}, {NULL}};
const Segment SGI304237[] = {{&S2411[202], 9}, {NULL}};
const Segment SGI304238[] = {{&S2411[8], 3}, {NULL}};
const Segment SGI304239[] = {{&S2411[13], 10}, {NULL}};
const Segment SGI304240[] = {{&S2411[42], 7}, {NULL}};
const Segment SGI304241[] = {{&S2411[35], 14}, {NULL}};
const Segment SGI304242[] = {{&S2411[143], 15}, {NULL}};
const Segment SGI304243[] = {{&S2411[51], 5}, {NULL}};
const Segment SGI304244[] = {{&S2412[30], 3}, {NULL}};
const Segment SGI304245[] = {{&S2412[217], 9}, {NULL}};
const Segment SGI304246[] = {{&S2412[8], 3}, {NULL}};
const Segment SGI304247[] = {{&S2412[13], 10}, {NULL}};
const Segment SGI304248[] = {{&S2412[42], 7}, {NULL}};
const Segment SGI304249[] = {{&S2412[35], 14}, {NULL}};
const Segment SGI304250[] = {{&S2412[131], 15}, {NULL}};
const Segment SGI304251[] = {{&S2412[184], 9}, {NULL}};
const Segment SGI304252[] = {{&S2414[30], 3}, {NULL}};
const Segment SGI304253[] = {{&S2414[197], 9}, {NULL}};
const Segment SGI304254[] = {{&S2414[8], 3}, {NULL}};
const Segment SGI304255[] = {{&S2414[13], 10}, {NULL}};
const Segment SGI304256[] = {{&S2414[42], 7}, {NULL}};
const Segment SGI304257[] = {{&S2414[35], 14}, {NULL}};
const Segment SGI304258[] = {{&S2414[138], 15}, {NULL}};
const Segment SGI304259[] = {{&S2414[51], 6}, {NULL}};
const Segment SGI304260[] = {{&S2415[30], 3}, {NULL}};
const Segment SGI304261[] = {{&S2415[190], 9}, {NULL}};
const Segment SGI304262[] = {{&S2415[8], 3}, {NULL}};
const Segment SGI304263[] = {{&S2415[13], 10}, {NULL}};
const Segment SGI304264[] = {{&S2415[42], 7}, {NULL}};
const Segment SGI304265[] = {{&S2415[35], 14}, {NULL}};
const Segment SGI304266[] = {{&S2415[131], 15}, {NULL}};
const Segment SGI304267[] = {{&S2416[30], 3}, {NULL}};
const Segment SGI304268[] = {{&S2416[219], 9}, {NULL}};
const Segment SGI304269[] = {{&S2416[8], 3}, {NULL}};
const Segment SGI304270[] = {{&S2416[13], 10}, {NULL}};
const Segment SGI304271[] = {{&S2416[42], 7}, {NULL}};
const Segment SGI304272[] = {{&S2416[35], 14}, {NULL}};
const Segment SGI304273[] = {{&S2416[143], 15}, {NULL}};
const Segment SGI304274[] = {{&S2416[51], 5}, {NULL}};
const Segment SGI304275[] = {{&S2416[186], 9}, {NULL}};
const Segment SGI304276[] = {{&S2417[30], 3}, {NULL}};
const Segment SGI304277[] = {{&S2417[169], 9}, {NULL}};
const Segment SGI304278[] = {{&S2417[8], 3}, {NULL}};
const Segment SGI304279[] = {{&S2417[13], 10}, {NULL}};
const Segment SGI304280[] = {{&S2417[42], 7}, {NULL}};
const Segment SGI304281[] = {{&S2417[35], 14}, {NULL}};
const Segment SGI304282[] = {{&S2418[30], 3}, {NULL}};
const Segment SGI304283[] = {{&S2418[222], 9}, {NULL}};
const Segment SGI304284[] = {{&S2418[8], 3}, {NULL}};
const Segment SGI304285[] = {{&S2418[13], 10}, {NULL}};
const Segment SGI304286[] = {{&S2418[42], 7}, {NULL}};
const Segment SGI304287[] = {{&S2418[35], 14}, {NULL}};
const Segment SGI304288[] = {{&S2418[138], 15}, {NULL}};
const Segment SGI304289[] = {{&S2418[51], 6}, {NULL}};
const Segment SGI304290[] = {{&S2419[30], 3}, {NULL}};
const Segment SGI304291[] = {{&S2419[215], 9}, {NULL}};
const Segment SGI304292[] = {{&S2419[8], 3}, {NULL}};
const Segment SGI304293[] = {{&S2419[13], 10}, {NULL}};
const Segment SGI304294[] = {{&S2419[42], 7}, {NULL}};
const Segment SGI304295[] = {{&S2419[35], 14}, {NULL}};
const Segment SGI304296[] = {{&S2419[131], 15}, {NULL}};
const Segment SGI304297[] = {{&S2421[30], 3}, {NULL}};
const Segment SGI304298[] = {{&S2421[181], 9}, {NULL}};
const Segment SGI304299[] = {{&S2421[8], 3}, {NULL}};
const Segment SGI304300[] = {{&S2421[13], 10}, {NULL}};
const Segment SGI304301[] = {{&S2421[42], 7}, {NULL}};
const Segment SGI304302[] = {{&S2421[35], 14}, {NULL}};
const Segment SGI304303[] = {{&S2421[51], 5}, {NULL}};
const Segment SGI304304[] = {{&S2422[30], 3}, {NULL}};
const Segment SGI304305[] = {{&S2422[188], 9}, {NULL}};
const Segment SGI304306[] = {{&S2422[8], 3}, {NULL}};
const Segment SGI304307[] = {{&S2422[13], 10}, {NULL}};
const Segment SGI304308[] = {{&S2422[42], 7}, {NULL}};
const Segment SGI304309[] = {{&S2422[35], 14}, {NULL}};
const Segment SGI304310[] = {{&S2422[51], 6}, {NULL}};
const Segment SGI304311[] = {{&S2423[30], 3}, {NULL}};
const Segment SGI304312[] = {{&S2423[195], 9}, {NULL}};
const Segment SGI304313[] = {{&S2423[8], 3}, {NULL}};
const Segment SGI304314[] = {{&S2423[13], 10}, {NULL}};
const Segment SGI304315[] = {{&S2423[42], 7}, {NULL}};
const Segment SGI304316[] = {{&S2423[35], 14}, {NULL}};
const Segment SGI304317[] = {{&S2423[131], 15}, {NULL}};
const Segment SGI304318[] = {{&S2424[30], 3}, {NULL}};
const Segment SGI304319[] = {{&S2424[215], 9}, {NULL}};
const Segment SGI304320[] = {{&S2424[8], 3}, {NULL}};
const Segment SGI304321[] = {{&S2424[13], 10}, {NULL}};
const Segment SGI304322[] = {{&S2424[42], 7}, {NULL}};
const Segment SGI304323[] = {{&S2424[35], 14}, {NULL}};
const Segment SGI304324[] = {{&S2424[131], 15}, {NULL}};
const Segment SGI304325[] = {{&S2425[30], 3}, {NULL}};
const Segment SGI304326[] = {{&S2425[215], 9}, {NULL}};
const Segment SGI304327[] = {{&S2425[8], 3}, {NULL}};
const Segment SGI304328[] = {{&S2425[13], 10}, {NULL}};
const Segment SGI304329[] = {{&S2425[42], 7}, {NULL}};
const Segment SGI304330[] = {{&S2425[35], 14}, {NULL}};
const Segment SGI304331[] = {{&S2425[131], 15}, {NULL}};
const Segment SGI304332[] = {{&S2426[30], 3}, {NULL}};
const Segment SGI304333[] = {{&S2426[197], 9}, {NULL}};
const Segment SGI304334[] = {{&S2426[8], 3}, {NULL}};
const Segment SGI304335[] = {{&S2426[13], 10}, {NULL}};
const Segment SGI304336[] = {{&S2426[42], 7}, {NULL}};
const Segment SGI304337[] = {{&S2426[35], 14}, {NULL}};
const Segment SGI304338[] = {{&S2426[138], 15}, {NULL}};
const Segment SGI304339[] = {{&S2426[51], 6}, {NULL}};
const Segment SGI304340[] = {{&S2427[30], 3}, {NULL}};
const Segment SGI304341[] = {{&S2427[212], 9}, {NULL}};
const Segment SGI304342[] = {{&S2427[8], 3}, {NULL}};
const Segment SGI304343[] = {{&S2427[13], 10}, {NULL}};
const Segment SGI304344[] = {{&S2427[42], 7}, {NULL}};
const Segment SGI304345[] = {{&S2427[35], 14}, {NULL}};
const Segment SGI304346[] = {{&S2427[51], 6}, {NULL}};
const Segment SGI304347[] = {{&S2428[30], 3}, {NULL}};
const Segment SGI304348[] = {{&S2428[176], 9}, {NULL}};
const Segment SGI304349[] = {{&S2428[8], 3}, {NULL}};
const Segment SGI304350[] = {{&S2428[13], 10}, {NULL}};
const Segment SGI304351[] = {{&S2428[42], 7}, {NULL}};
const Segment SGI304352[] = {{&S2428[35], 14}, {NULL}};
const Segment SGI304353[] = {{&S2428[51], 6}, {NULL}};
const Segment SGI304354[] = {{&S2429[30], 3}, {NULL}};
const Segment SGI304355[] = {{&S2429[202], 9}, {NULL}};
const Segment SGI304356[] = {{&S2429[8], 3}, {NULL}};
const Segment SGI304357[] = {{&S2429[13], 10}, {NULL}};
const Segment SGI304358[] = {{&S2429[42], 7}, {NULL}};
const Segment SGI304359[] = {{&S2429[35], 14}, {NULL}};
const Segment SGI304360[] = {{&S2429[143], 15}, {NULL}};
const Segment SGI304361[] = {{&S2429[51], 5}, {NULL}};
const Segment SGI304362[] = {{&S2430[30], 3}, {NULL}};
const Segment SGI304363[] = {{&S2430[217], 9}, {NULL}};
const Segment SGI304364[] = {{&S2430[8], 3}, {NULL}};
const Segment SGI304365[] = {{&S2430[13], 10}, {NULL}};
const Segment SGI304366[] = {{&S2430[42], 7}, {NULL}};
const Segment SGI304367[] = {{&S2430[35], 14}, {NULL}};
const Segment SGI304368[] = {{&S2430[143], 15}, {NULL}};
const Segment SGI304369[] = {{&S2430[51], 5}, {NULL}};
const Segment SGI304370[] = {{&S2430[184], 9}, {NULL}};
const Segment SGI304371[] = {{&S2431[30], 3}, {NULL}};
const Segment SGI304372[] = {{&S2431[190], 9}, {NULL}};
const Segment SGI304373[] = {{&S2431[8], 3}, {NULL}};
const Segment SGI304374[] = {{&S2431[13], 10}, {NULL}};
const Segment SGI304375[] = {{&S2431[42], 7}, {NULL}};
const Segment SGI304376[] = {{&S2431[35], 14}, {NULL}};
const Segment SGI304377[] = {{&S2431[131], 15}, {NULL}};
const Segment SGI304378[] = {{&S2432[30], 3}, {NULL}};
const Segment SGI304379[] = {{&S2432[169], 9}, {NULL}};
const Segment SGI304380[] = {{&S2432[8], 3}, {NULL}};
const Segment SGI304381[] = {{&S2432[13], 10}, {NULL}};
const Segment SGI304382[] = {{&S2432[42], 7}, {NULL}};
const Segment SGI304383[] = {{&S2432[35], 14}, {NULL}};
const Segment SGI304384[] = {{&S2433[30], 3}, {NULL}};
const Segment SGI304385[] = {{&S2433[197], 9}, {NULL}};
const Segment SGI304386[] = {{&S2433[8], 3}, {NULL}};
const Segment SGI304387[] = {{&S2433[13], 10}, {NULL}};
const Segment SGI304388[] = {{&S2433[42], 7}, {NULL}};
const Segment SGI304389[] = {{&S2433[35], 14}, {NULL}};
const Segment SGI304390[] = {{&S2433[138], 15}, {NULL}};
const Segment SGI304391[] = {{&S2433[51], 6}, {NULL}};
const Segment SGI304392[] = {{&S2434[30], 3}, {NULL}};
const Segment SGI304393[] = {{&S2434[205], 9}, {NULL}};
const Segment SGI304394[] = {{&S2434[8], 3}, {NULL}};
const Segment SGI304395[] = {{&S2434[13], 10}, {NULL}};
const Segment SGI304396[] = {{&S2434[42], 7}, {NULL}};
const Segment SGI304397[] = {{&S2434[35], 14}, {NULL}};
const Segment SGI304398[] = {{&S2434[131], 15}, {NULL}};
const Segment SGI304399[] = {{&S2434[172], 9}, {NULL}};
const Segment SGI304400[] = {{&S2435[30], 3}, {NULL}};
const Segment SGI304401[] = {{&S2435[181], 9}, {NULL}};
const Segment SGI304402[] = {{&S2435[8], 3}, {NULL}};
const Segment SGI304403[] = {{&S2435[13], 10}, {NULL}};
const Segment SGI304404[] = {{&S2435[42], 7}, {NULL}};
const Segment SGI304405[] = {{&S2435[35], 14}, {NULL}};
const Segment SGI304406[] = {{&S2435[51], 5}, {NULL}};
const Segment SGI304407[] = {{&S2436[30], 3}, {NULL}};
const Segment SGI304408[] = {{&S2436[214], 9}, {NULL}};
const Segment SGI304409[] = {{&S2436[8], 3}, {NULL}};
const Segment SGI304410[] = {{&S2436[13], 10}, {NULL}};
const Segment SGI304411[] = {{&S2436[42], 7}, {NULL}};
const Segment SGI304412[] = {{&S2436[35], 14}, {NULL}};
const Segment SGI304413[] = {{&S2436[143], 15}, {NULL}};
const Segment SGI304414[] = {{&S2436[51], 5}, {NULL}};
const Segment SGI304415[] = {{&S2437[30], 3}, {NULL}};
const Segment SGI304416[] = {{&S2437[89], 9}, {NULL}};
const Segment SGI304417[] = {{&S2437[8], 3}, {NULL}};
const Segment SGI304418[] = {{&S2437[13], 10}, {NULL}};
const Segment SGI304419[] = {{&S2437[42], 7}, {NULL}};
const Segment SGI304420[] = {{&S2437[35], 14}, {NULL}};
const Segment SGI304421[] = {{&S2437[51], 6}, {NULL}};
const Segment SGI304422[] = {{&S2438[30], 3}, {NULL}};
const Segment SGI304423[] = {{&S2438[207], 9}, {NULL}};
const Segment SGI304424[] = {{&S2438[8], 3}, {NULL}};
const Segment SGI304425[] = {{&S2438[13], 10}, {NULL}};
const Segment SGI304426[] = {{&S2438[42], 7}, {NULL}};
const Segment SGI304427[] = {{&S2438[35], 14}, {NULL}};
const Segment SGI304428[] = {{&S2438[138], 15}, {NULL}};
const Segment SGI304429[] = {{&S2438[179], 4}, {NULL}};
const Segment SGI304430[] = {{&S2438[51], 6}, {NULL}};
const Segment SGI304431[] = {{&S2439[30], 3}, {NULL}};
const Segment SGI304432[] = {{&S2439[176], 9}, {NULL}};
const Segment SGI304433[] = {{&S2439[8], 3}, {NULL}};
const Segment SGI304434[] = {{&S2439[13], 10}, {NULL}};
const Segment SGI304435[] = {{&S2439[42], 7}, {NULL}};
const Segment SGI304436[] = {{&S2439[35], 14}, {NULL}};
const Segment SGI304437[] = {{&S2439[51], 6}, {NULL}};
const Segment SGI304438[] = {{&S2440[30], 3}, {NULL}};
const Segment SGI304439[] = {{&S2440[82], 9}, {NULL}};
const Segment SGI304440[] = {{&S2440[8], 3}, {NULL}};
const Segment SGI304441[] = {{&S2440[13], 10}, {NULL}};
const Segment SGI304442[] = {{&S2440[42], 7}, {NULL}};
const Segment SGI304443[] = {{&S2440[35], 14}, {NULL}};
const Segment SGI304444[] = {{&S2441[30], 3}, {NULL}};
const Segment SGI304445[] = {{&S2441[197], 9}, {NULL}};
const Segment SGI304446[] = {{&S2441[8], 3}, {NULL}};
const Segment SGI304447[] = {{&S2441[13], 10}, {NULL}};
const Segment SGI304448[] = {{&S2441[42], 7}, {NULL}};
const Segment SGI304449[] = {{&S2441[35], 14}, {NULL}};
const Segment SGI304450[] = {{&S2441[138], 15}, {NULL}};
const Segment SGI304451[] = {{&S2441[51], 6}, {NULL}};
const Segment SGI304452[] = {{&S2442[30], 3}, {NULL}};
const Segment SGI304453[] = {{&S2442[229], 9}, {NULL}};
const Segment SGI304454[] = {{&S2442[8], 3}, {NULL}};
const Segment SGI304455[] = {{&S2442[13], 10}, {NULL}};
const Segment SGI304456[] = {{&S2442[42], 7}, {NULL}};
const Segment SGI304457[] = {{&S2442[35], 14}, {NULL}};
const Segment SGI304458[] = {{&S2442[143], 15}, {NULL}};
const Segment SGI304459[] = {{&S2442[51], 5}, {NULL}};
const Segment SGI304460[] = {{&S2442[196], 9}, {NULL}};
const Segment SGI304461[] = {{&S2443[30], 3}, {NULL}};
const Segment SGI304462[] = {{&S2443[187], 9}, {NULL}};
const Segment SGI304463[] = {{&S2443[8], 3}, {NULL}};
const Segment SGI304464[] = {{&S2443[13], 10}, {NULL}};
const Segment SGI304465[] = {{&S2443[42], 7}, {NULL}};
const Segment SGI304466[] = {{&S2443[35], 14}, {NULL}};
const Segment SGI304467[] = {{&S2443[141], 15}, {NULL}};
const Segment SGI304468[] = {{&S2443[51], 6}, {NULL}};
const Segment SGI304469[] = {{&S2444[30], 3}, {NULL}};
const Segment SGI304470[] = {{&S2444[190], 9}, {NULL}};
const Segment SGI304471[] = {{&S2444[8], 3}, {NULL}};
const Segment SGI304472[] = {{&S2444[13], 10}, {NULL}};
const Segment SGI304473[] = {{&S2444[42], 7}, {NULL}};
const Segment SGI304474[] = {{&S2444[35], 14}, {NULL}};
const Segment SGI304475[] = {{&S2444[131], 15}, {NULL}};
const Segment SGI304476[] = {{&S2446[30], 3}, {NULL}};
const Segment SGI304477[] = {{&S2446[176], 9}, {NULL}};
const Segment SGI304478[] = {{&S2446[8], 3}, {NULL}};
const Segment SGI304479[] = {{&S2446[13], 10}, {NULL}};
const Segment SGI304480[] = {{&S2446[42], 7}, {NULL}};
const Segment SGI304481[] = {{&S2446[35], 14}, {NULL}};
const Segment SGI304482[] = {{&S2446[51], 6}, {NULL}};
const Segment SGI304483[] = {{&S2447[30], 3}, {NULL}};
const Segment SGI304484[] = {{&S2447[181], 9}, {NULL}};
const Segment SGI304485[] = {{&S2447[8], 3}, {NULL}};
const Segment SGI304486[] = {{&S2447[13], 10}, {NULL}};
const Segment SGI304487[] = {{&S2447[42], 7}, {NULL}};
const Segment SGI304488[] = {{&S2447[35], 14}, {NULL}};
const Segment SGI304489[] = {{&S2448[30], 3}, {NULL}};
const Segment SGI304490[] = {{&S2448[224], 9}, {NULL}};
const Segment SGI304491[] = {{&S2448[8], 3}, {NULL}};
const Segment SGI304492[] = {{&S2448[13], 10}, {NULL}};
const Segment SGI304493[] = {{&S2448[42], 7}, {NULL}};
const Segment SGI304494[] = {{&S2448[35], 14}, {NULL}};
const Segment SGI304495[] = {{&S2448[143], 15}, {NULL}};
const Segment SGI304496[] = {{&S2448[196], 4}, {NULL}};
const Segment SGI304497[] = {{&S2448[51], 5}, {NULL}};
const Segment SGI304498[] = {{&S2449[30], 3}, {NULL}};
const Segment SGI304499[] = {{&S2449[207], 9}, {NULL}};
const Segment SGI304500[] = {{&S2449[8], 3}, {NULL}};
const Segment SGI304501[] = {{&S2449[13], 10}, {NULL}};
const Segment SGI304502[] = {{&S2449[42], 7}, {NULL}};
const Segment SGI304503[] = {{&S2449[35], 14}, {NULL}};
const Segment SGI304504[] = {{&S2449[138], 15}, {NULL}};
const Segment SGI304505[] = {{&S2449[179], 4}, {NULL}};
const Segment SGI304506[] = {{&S2449[51], 6}, {NULL}};
const Segment SGI304507[] = {{&S2450[30], 3}, {NULL}};
const Segment SGI304508[] = {{&S2450[200], 9}, {NULL}};
const Segment SGI304509[] = {{&S2450[8], 3}, {NULL}};
const Segment SGI304510[] = {{&S2450[13], 10}, {NULL}};
const Segment SGI304511[] = {{&S2450[42], 7}, {NULL}};
const Segment SGI304512[] = {{&S2450[35], 14}, {NULL}};
const Segment SGI304513[] = {{&S2451[30], 3}, {NULL}};
const Segment SGI304514[] = {{&S2451[214], 9}, {NULL}};
const Segment SGI304515[] = {{&S2451[8], 3}, {NULL}};
const Segment SGI304516[] = {{&S2451[13], 10}, {NULL}};
const Segment SGI304517[] = {{&S2451[42], 7}, {NULL}};
const Segment SGI304518[] = {{&S2451[35], 14}, {NULL}};
const Segment SGI304519[] = {{&S2451[143], 15}, {NULL}};
const Segment SGI304520[] = {{&S2451[186], 4}, {NULL}};
const Segment SGI304521[] = {{&S2451[51], 5}, {NULL}};
const Segment SGI304522[] = {{&S2452[30], 3}, {NULL}};
const Segment SGI304523[] = {{&S2452[181], 9}, {NULL}};
const Segment SGI304524[] = {{&S2452[8], 3}, {NULL}};
const Segment SGI304525[] = {{&S2452[13], 10}, {NULL}};
const Segment SGI304526[] = {{&S2452[42], 7}, {NULL}};
const Segment SGI304527[] = {{&S2452[35], 14}, {NULL}};
const Segment SGI304528[] = {{&S2452[51], 5}, {NULL}};
const Segment SGI304529[] = {{&S2453[30], 3}, {NULL}};
const Segment SGI304530[] = {{&S2453[89], 9}, {NULL}};
const Segment SGI304531[] = {{&S2453[8], 3}, {NULL}};
const Segment SGI304532[] = {{&S2453[13], 10}, {NULL}};
const Segment SGI304533[] = {{&S2453[42], 7}, {NULL}};
const Segment SGI304534[] = {{&S2453[35], 14}, {NULL}};
const Segment SGI304535[] = {{&S2453[51], 6}, {NULL}};
const Segment SGI304536[] = {{&S2454[30], 3}, {NULL}};
const Segment SGI304537[] = {{&S2454[249], 7}, {NULL}};
const Segment SGI304538[] = {{&S2454[8], 3}, {NULL}};
const Segment SGI304539[] = {{&S2454[13], 10}, {NULL}};
const Segment SGI304540[] = {{&S2454[42], 7}, {NULL}};
const Segment SGI304541[] = {{&S2454[35], 14}, {NULL}};
const Segment SGI304542[] = {{&S2454[143], 15}, {NULL}};
const Segment SGI304543[] = {{&S2454[51], 5}, {NULL}};
const Segment SGI304544[] = {{&S2454[186], 9}, {NULL}};
const Segment SGI304545[] = {{&S2455[30], 3}, {NULL}};
const Segment SGI304546[] = {{&S2455[202], 9}, {NULL}};
const Segment SGI304547[] = {{&S2455[8], 3}, {NULL}};
const Segment SGI304548[] = {{&S2455[13], 10}, {NULL}};
const Segment SGI304549[] = {{&S2455[42], 7}, {NULL}};
const Segment SGI304550[] = {{&S2455[35], 14}, {NULL}};
const Segment SGI304551[] = {{&S2455[131], 15}, {NULL}};
const Segment SGI304552[] = {{&S2455[174], 4}, {NULL}};
const Segment SGI304553[] = {{&S2456[30], 3}, {NULL}};
const Segment SGI304554[] = {{&S2456[195], 9}, {NULL}};
const Segment SGI304555[] = {{&S2456[8], 3}, {NULL}};
const Segment SGI304556[] = {{&S2456[13], 10}, {NULL}};
const Segment SGI304557[] = {{&S2456[42], 7}, {NULL}};
const Segment SGI304558[] = {{&S2456[35], 14}, {NULL}};
const Segment SGI304559[] = {{&S2456[131], 15}, {NULL}};
const Segment SGI304560[] = {{&S2457[30], 3}, {NULL}};
const Segment SGI304561[] = {{&S2457[94], 9}, {NULL}};
const Segment SGI304562[] = {{&S2457[8], 3}, {NULL}};
const Segment SGI304563[] = {{&S2457[13], 10}, {NULL}};
const Segment SGI304564[] = {{&S2457[42], 7}, {NULL}};
const Segment SGI304565[] = {{&S2457[35], 14}, {NULL}};
const Segment SGI304566[] = {{&S2457[51], 5}, {NULL}};
const Segment SGI304567[] = {{&S2458[30], 3}, {NULL}};
const Segment SGI304568[] = {{&S2458[184], 9}, {NULL}};
const Segment SGI304569[] = {{&S2458[8], 3}, {NULL}};
const Segment SGI304570[] = {{&S2458[13], 10}, {NULL}};
const Segment SGI304571[] = {{&S2458[42], 7}, {NULL}};
const Segment SGI304572[] = {{&S2458[35], 14}, {NULL}};
const Segment SGI304573[] = {{&S2458[151], 9}, {NULL}};
const Segment SGI304574[] = {{&S2459[30], 3}, {NULL}};
const Segment SGI304575[] = {{&S2459[266], 9}, {NULL}};
const Segment SGI304576[] = {{&S2459[8], 3}, {NULL}};
const Segment SGI304577[] = {{&S2459[13], 10}, {NULL}};
const Segment SGI304578[] = {{&S2459[42], 7}, {NULL}};
const Segment SGI304579[] = {{&S2459[35], 14}, {NULL}};
const Segment SGI304580[] = {{&S2459[131], 15}, {&S2459[131], 15}, {NULL}};
const Segment SGI304581[] = {{&S2460[30], 3}, {NULL}};
const Segment SGI304582[] = {{&S2460[239], 9}, {NULL}};
const Segment SGI304583[] = {{&S2460[8], 3}, {NULL}};
const Segment SGI304584[] = {{&S2460[13], 10}, {NULL}};
const Segment SGI304585[] = {{&S2460[42], 7}, {NULL}};
const Segment SGI304586[] = {{&S2460[35], 14}, {NULL}};
const Segment SGI304587[] = {{&S2460[143], 15}, {NULL}};
const Segment SGI304588[] = {{&S2460[211], 4}, {NULL}};
const Segment SGI304589[] = {{&S2460[51], 5}, {NULL}};
const Segment SGI304590[] = {{&S2460[196], 9}, {NULL}};
const Segment SGI304591[] = {{&S2461[30], 3}, {NULL}};
const Segment SGI304592[] = {{&S2461[215], 9}, {NULL}};
const Segment SGI304593[] = {{&S2461[8], 3}, {NULL}};
const Segment SGI304594[] = {{&S2461[13], 10}, {NULL}};
const Segment SGI304595[] = {{&S2461[42], 7}, {NULL}};
const Segment SGI304596[] = {{&S2461[35], 14}, {NULL}};
const Segment SGI304597[] = {{&S2461[131], 15}, {NULL}};
const Segment SGI304598[] = {{&S2462[30], 3}, {NULL}};
const Segment SGI304599[] = {{&S2462[230], 9}, {NULL}};
const Segment SGI304600[] = {{&S2462[8], 3}, {NULL}};
const Segment SGI304601[] = {{&S2462[13], 10}, {NULL}};
const Segment SGI304602[] = {{&S2462[42], 7}, {NULL}};
const Segment SGI304603[] = {{&S2462[35], 14}, {NULL}};
const Segment SGI304604[] = {{&S2462[131], 15}, {NULL}};
const Segment SGI304605[] = {{&S2462[197], 9}, {NULL}};
const Segment SGI304606[] = {{&S2463[30], 3}, {NULL}};
const Segment SGI304607[] = {{&S2463[184], 9}, {NULL}};
const Segment SGI304608[] = {{&S2463[8], 3}, {NULL}};
const Segment SGI304609[] = {{&S2463[13], 10}, {NULL}};
const Segment SGI304610[] = {{&S2463[42], 7}, {NULL}};
const Segment SGI304611[] = {{&S2463[35], 14}, {NULL}};
const Segment SGI304612[] = {{&S2463[151], 9}, {NULL}};
const Segment SGI304613[] = {{&S2464[30], 3}, {NULL}};
const Segment SGI304614[] = {{&S2464[220], 9}, {NULL}};
const Segment SGI304615[] = {{&S2464[8], 3}, {NULL}};
const Segment SGI304616[] = {{&S2464[13], 10}, {NULL}};
const Segment SGI304617[] = {{&S2464[42], 7}, {NULL}};
const Segment SGI304618[] = {{&S2464[35], 14}, {NULL}};
const Segment SGI304619[] = {{&S2464[131], 15}, {&S2464[131], 15}, {NULL}};
const Segment SGI304620[] = {{&S2465[30], 3}, {NULL}};
const Segment SGI304621[] = {{&S2465[82], 9}, {NULL}};
const Segment SGI304622[] = {{&S2465[8], 3}, {NULL}};
const Segment SGI304623[] = {{&S2465[13], 10}, {NULL}};
const Segment SGI304624[] = {{&S2465[42], 7}, {NULL}};
const Segment SGI304625[] = {{&S2465[35], 14}, {NULL}};
const Segment SGI304626[] = {{&S2466[30], 3}, {NULL}};
const Segment SGI304627[] = {{&S2466[193], 9}, {NULL}};
const Segment SGI304628[] = {{&S2466[8], 3}, {NULL}};
const Segment SGI304629[] = {{&S2466[13], 10}, {NULL}};
const Segment SGI304630[] = {{&S2466[42], 7}, {NULL}};
const Segment SGI304631[] = {{&S2466[35], 14}, {NULL}};
const Segment SGI304632[] = {{&S2467[30], 3}, {NULL}};
const Segment SGI304633[] = {{&S2467[281], 9}, {NULL}};
const Segment SGI304634[] = {{&S2467[8], 3}, {NULL}};
const Segment SGI304635[] = {{&S2467[13], 10}, {NULL}};
const Segment SGI304636[] = {{&S2467[42], 7}, {NULL}};
const Segment SGI304637[] = {{&S2467[35], 14}, {NULL}};
const Segment SGI304638[] = {{&S2467[131], 15}, {&S2467[131], 15}, {NULL}};
const Segment SGI304639[] = {{&S2467[180], 9}, {NULL}};
const Segment SGI304640[] = {{&S2468[30], 3}, {NULL}};
const Segment SGI304641[] = {{&S2468[220], 9}, {NULL}};
const Segment SGI304642[] = {{&S2468[8], 3}, {NULL}};
const Segment SGI304643[] = {{&S2468[13], 10}, {NULL}};
const Segment SGI304644[] = {{&S2468[47], 7}, {NULL}};
const Segment SGI304645[] = {{&S2468[40], 14}, {NULL}};
const Segment SGI304646[] = {{&S2468[136], 15}, {NULL}};
const Segment SGI304647[] = {{&S2469[30], 3}, {NULL}};
const Segment SGI304648[] = {{&S2469[196], 9}, {NULL}};
const Segment SGI304649[] = {{&S2469[8], 3}, {NULL}};
const Segment SGI304650[] = {{&S2469[13], 10}, {NULL}};
const Segment SGI304651[] = {{&S2469[42], 7}, {NULL}};
const Segment SGI304652[] = {{&S2469[35], 14}, {NULL}};
const Segment SGI304653[] = {{&S2469[51], 5}, {NULL}};
const Segment SGI304654[] = {{&S2469[163], 9}, {NULL}};
const Segment SGI304655[] = {{&S2470[30], 3}, {NULL}};
const Segment SGI304656[] = {{&S2470[217], 9}, {NULL}};
const Segment SGI304657[] = {{&S2470[8], 3}, {NULL}};
const Segment SGI304658[] = {{&S2470[13], 10}, {NULL}};
const Segment SGI304659[] = {{&S2470[42], 7}, {NULL}};
const Segment SGI304660[] = {{&S2470[35], 14}, {NULL}};
const Segment SGI304661[] = {{&S2470[131], 15}, {NULL}};
const Segment SGI304662[] = {{&S2470[189], 4}, {NULL}};
const Segment SGI304663[] = {{&S2470[174], 9}, {NULL}};
const Segment SGI304664[] = {{&S2471[30], 3}, {NULL}};
const Segment SGI304665[] = {{&S2471[247], 9}, {NULL}};
const Segment SGI304666[] = {{&S2471[8], 3}, {NULL}};
const Segment SGI304667[] = {{&S2471[13], 10}, {NULL}};
const Segment SGI304668[] = {{&S2471[42], 7}, {NULL}};
const Segment SGI304669[] = {{&S2471[35], 14}, {NULL}};
const Segment SGI304670[] = {{&S2471[131], 15}, {NULL}};
const Segment SGI304671[] = {{&S2471[212], 4}, {NULL}};
const Segment SGI304672[] = {{&S2471[197], 9}, {NULL}};
const Segment SGI304673[] = {{&S2472[30], 3}, {NULL}};
const Segment SGI304674[] = {{&S2472[197], 9}, {NULL}};
const Segment SGI304675[] = {{&S2472[8], 3}, {NULL}};
const Segment SGI304676[] = {{&S2472[13], 10}, {NULL}};
const Segment SGI304677[] = {{&S2472[42], 7}, {NULL}};
const Segment SGI304678[] = {{&S2472[35], 14}, {NULL}};
const Segment SGI304679[] = {{&S2472[138], 15}, {NULL}};
const Segment SGI304680[] = {{&S2472[51], 6}, {NULL}};
const Segment SGI304681[] = {{&S2474[30], 3}, {NULL}};
const Segment SGI304682[] = {{&S2474[169], 9}, {NULL}};
const Segment SGI304683[] = {{&S2474[8], 3}, {NULL}};
const Segment SGI304684[] = {{&S2474[13], 10}, {NULL}};
const Segment SGI304685[] = {{&S2474[42], 7}, {NULL}};
const Segment SGI304686[] = {{&S2474[35], 14}, {NULL}};
const Segment SGI304687[] = {{&S2475[30], 3}, {NULL}};
const Segment SGI304688[] = {{&S2475[89], 9}, {NULL}};
const Segment SGI304689[] = {{&S2475[8], 3}, {NULL}};
const Segment SGI304690[] = {{&S2475[13], 10}, {NULL}};
const Segment SGI304691[] = {{&S2475[42], 7}, {NULL}};
const Segment SGI304692[] = {{&S2475[35], 14}, {NULL}};
const Segment SGI304693[] = {{&S2475[51], 6}, {NULL}};
const Segment SGI304694[] = {{&S2476[30], 3}, {NULL}};
const Segment SGI304695[] = {{&S2476[214], 9}, {NULL}};
const Segment SGI304696[] = {{&S2476[8], 3}, {NULL}};
const Segment SGI304697[] = {{&S2476[13], 10}, {NULL}};
const Segment SGI304698[] = {{&S2476[42], 7}, {NULL}};
const Segment SGI304699[] = {{&S2476[35], 14}, {NULL}};
const Segment SGI304700[] = {{&S2476[143], 15}, {NULL}};
const Segment SGI304701[] = {{&S2476[51], 5}, {NULL}};
const Segment SGI304702[] = {{&S2477[30], 3}, {NULL}};
const Segment SGI304703[] = {{&S2477[202], 9}, {NULL}};
const Segment SGI304704[] = {{&S2477[8], 3}, {NULL}};
const Segment SGI304705[] = {{&S2477[13], 10}, {NULL}};
const Segment SGI304706[] = {{&S2477[42], 7}, {NULL}};
const Segment SGI304707[] = {{&S2477[35], 14}, {NULL}};
const Segment SGI304708[] = {{&S2477[138], 15}, {NULL}};
const Segment SGI304709[] = {{&S2477[51], 6}, {NULL}};
const Segment SGI304710[] = {{&S2477[169], 13}, {NULL}};
const Segment SGI304711[] = {{&S2478[30], 3}, {NULL}};
const Segment SGI304712[] = {{&S2478[187], 9}, {NULL}};
const Segment SGI304713[] = {{&S2478[8], 3}, {NULL}};
const Segment SGI304714[] = {{&S2478[13], 10}, {NULL}};
const Segment SGI304715[] = {{&S2478[42], 7}, {NULL}};
const Segment SGI304716[] = {{&S2478[35], 14}, {NULL}};
const Segment SGI304717[] = {{&S2478[138], 15}, {NULL}};
const Segment SGI304718[] = {{&S2478[51], 6}, {NULL}};
const Segment SGI304719[] = {{&S2479[30], 3}, {NULL}};
const Segment SGI304720[] = {{&S2479[230], 9}, {NULL}};
const Segment SGI304721[] = {{&S2479[8], 3}, {NULL}};
const Segment SGI304722[] = {{&S2479[13], 10}, {NULL}};
const Segment SGI304723[] = {{&S2479[42], 7}, {NULL}};
const Segment SGI304724[] = {{&S2479[35], 14}, {NULL}};
const Segment SGI304725[] = {{&S2479[131], 15}, {NULL}};
const Segment SGI304726[] = {{&S2479[177], 9}, {NULL}};
const Segment SGI304727[] = {{&S2480[30], 3}, {NULL}};
const Segment SGI304728[] = {{&S2480[210], 9}, {NULL}};
const Segment SGI304729[] = {{&S2480[8], 3}, {NULL}};
const Segment SGI304730[] = {{&S2480[13], 10}, {NULL}};
const Segment SGI304731[] = {{&S2480[42], 7}, {NULL}};
const Segment SGI304732[] = {{&S2480[35], 14}, {NULL}};
const Segment SGI304733[] = {{&S2480[138], 15}, {NULL}};
const Segment SGI304734[] = {{&S2480[51], 6}, {NULL}};
const Segment SGI304735[] = {{&S2482[30], 3}, {NULL}};
const Segment SGI304736[] = {{&S2482[219], 9}, {NULL}};
const Segment SGI304737[] = {{&S2482[8], 3}, {NULL}};
const Segment SGI304738[] = {{&S2482[13], 10}, {NULL}};
const Segment SGI304739[] = {{&S2482[42], 7}, {NULL}};
const Segment SGI304740[] = {{&S2482[35], 14}, {NULL}};
const Segment SGI304741[] = {{&S2482[138], 15}, {NULL}};
const Segment SGI304742[] = {{&S2482[191], 4}, {NULL}};
const Segment SGI304743[] = {{&S2482[51], 6}, {NULL}};
const Segment SGI304744[] = {{&S2483[30], 3}, {NULL}};
const Segment SGI304745[] = {{&S2483[94], 9}, {NULL}};
const Segment SGI304746[] = {{&S2483[8], 3}, {NULL}};
const Segment SGI304747[] = {{&S2483[13], 10}, {NULL}};
const Segment SGI304748[] = {{&S2483[42], 7}, {NULL}};
const Segment SGI304749[] = {{&S2483[35], 14}, {NULL}};
const Segment SGI304750[] = {{&S2483[51], 5}, {NULL}};
const Segment SGI304751[] = {{&S2484[30], 3}, {NULL}};
const Segment SGI304752[] = {{&S2484[186], 9}, {NULL}};
const Segment SGI304753[] = {{&S2484[8], 3}, {NULL}};
const Segment SGI304754[] = {{&S2484[13], 10}, {NULL}};
const Segment SGI304755[] = {{&S2484[42], 7}, {NULL}};
const Segment SGI304756[] = {{&S2484[35], 14}, {NULL}};
const Segment SGI304757[] = {{&S2484[51], 5}, {NULL}};
const Segment SGI304758[] = {{&S2484[153], 9}, {NULL}};
const Segment SGI304759[] = {{&S2485[30], 3}, {NULL}};
const Segment SGI304760[] = {{&S2485[190], 9}, {NULL}};
const Segment SGI304761[] = {{&S2485[8], 3}, {NULL}};
const Segment SGI304762[] = {{&S2485[13], 10}, {NULL}};
const Segment SGI304763[] = {{&S2485[42], 7}, {NULL}};
const Segment SGI304764[] = {{&S2485[35], 14}, {NULL}};
const Segment SGI304765[] = {{&S2485[131], 15}, {NULL}};
const Segment SGI304766[] = {{&S2486[30], 3}, {NULL}};
const Segment SGI304767[] = {{&S2486[197], 9}, {NULL}};
const Segment SGI304768[] = {{&S2486[8], 3}, {NULL}};
const Segment SGI304769[] = {{&S2486[13], 10}, {NULL}};
const Segment SGI304770[] = {{&S2486[42], 7}, {NULL}};
const Segment SGI304771[] = {{&S2486[35], 14}, {NULL}};
const Segment SGI304772[] = {{&S2486[138], 15}, {NULL}};
const Segment SGI304773[] = {{&S2486[51], 6}, {NULL}};
const Segment SGI304774[] = {{&S2487[30], 3}, {NULL}};
const Segment SGI304775[] = {{&S2487[204], 9}, {NULL}};
const Segment SGI304776[] = {{&S2487[8], 3}, {NULL}};
const Segment SGI304777[] = {{&S2487[13], 10}, {NULL}};
const Segment SGI304778[] = {{&S2487[42], 7}, {NULL}};
const Segment SGI304779[] = {{&S2487[35], 14}, {NULL}};
const Segment SGI304780[] = {{&S2487[143], 15}, {NULL}};
const Segment SGI304781[] = {{&S2487[51], 5}, {NULL}};
const Segment SGI304782[] = {{&S2488[30], 3}, {NULL}};
const Segment SGI304783[] = {{&S2488[190], 9}, {NULL}};
const Segment SGI304784[] = {{&S2488[8], 3}, {NULL}};
const Segment SGI304785[] = {{&S2488[13], 10}, {NULL}};
const Segment SGI304786[] = {{&S2488[42], 7}, {NULL}};
const Segment SGI304787[] = {{&S2488[35], 14}, {NULL}};
const Segment SGI304788[] = {{&S2488[131], 15}, {NULL}};
const Segment SGI304789[] = {{&S2489[30], 3}, {NULL}};
const Segment SGI304790[] = {{&S2489[89], 9}, {NULL}};
const Segment SGI304791[] = {{&S2489[8], 3}, {NULL}};
const Segment SGI304792[] = {{&S2489[13], 10}, {NULL}};
const Segment SGI304793[] = {{&S2489[42], 7}, {NULL}};
const Segment SGI304794[] = {{&S2489[35], 14}, {NULL}};
const Segment SGI304795[] = {{&S2489[51], 6}, {NULL}};
const Segment SGI304796[] = {{&S2490[30], 3}, {NULL}};
const Segment SGI304797[] = {{&S2490[176], 9}, {NULL}};
const Segment SGI304798[] = {{&S2490[8], 3}, {NULL}};
const Segment SGI304799[] = {{&S2490[13], 10}, {NULL}};
const Segment SGI304800[] = {{&S2490[42], 7}, {NULL}};
const Segment SGI304801[] = {{&S2490[35], 14}, {NULL}};
const Segment SGI304802[] = {{&S2490[51], 6}, {NULL}};
const Segment SGI304803[] = {{&S2491[30], 3}, {NULL}};
const Segment SGI304804[] = {{&S2491[217], 9}, {NULL}};
const Segment SGI304805[] = {{&S2491[8], 3}, {NULL}};
const Segment SGI304806[] = {{&S2491[13], 10}, {NULL}};
const Segment SGI304807[] = {{&S2491[42], 7}, {NULL}};
const Segment SGI304808[] = {{&S2491[35], 14}, {NULL}};
const Segment SGI304809[] = {{&S2491[143], 15}, {NULL}};
const Segment SGI304810[] = {{&S2491[51], 5}, {NULL}};
const Segment SGI304811[] = {{&S2491[184], 9}, {NULL}};
const Segment SGI304812[] = {{&S2492[30], 3}, {NULL}};
const Segment SGI304813[] = {{&S2492[207], 9}, {NULL}};
const Segment SGI304814[] = {{&S2492[8], 3}, {NULL}};
const Segment SGI304815[] = {{&S2492[13], 10}, {NULL}};
const Segment SGI304816[] = {{&S2492[42], 7}, {NULL}};
const Segment SGI304817[] = {{&S2492[35], 14}, {NULL}};
const Segment SGI304818[] = {{&S2492[138], 15}, {NULL}};
const Segment SGI304819[] = {{&S2492[179], 4}, {NULL}};
const Segment SGI304820[] = {{&S2492[51], 6}, {NULL}};
const Segment SGI304821[] = {{&S2493[30], 3}, {NULL}};
const Segment SGI304822[] = {{&S2493[195], 9}, {NULL}};
const Segment SGI304823[] = {{&S2493[8], 3}, {NULL}};
const Segment SGI304824[] = {{&S2493[13], 10}, {NULL}};
const Segment SGI304825[] = {{&S2493[42], 7}, {NULL}};
const Segment SGI304826[] = {{&S2493[35], 14}, {NULL}};
const Segment SGI304827[] = {{&S2493[131], 15}, {NULL}};
const Segment SGI304828[] = {{&S2493[162], 9}, {NULL}};
const Segment SGI304829[] = {{&S2494[30], 3}, {NULL}};
const Segment SGI304830[] = {{&S2494[82], 9}, {NULL}};
const Segment SGI304831[] = {{&S2494[8], 3}, {NULL}};
const Segment SGI304832[] = {{&S2494[13], 10}, {NULL}};
const Segment SGI304833[] = {{&S2494[42], 7}, {NULL}};
const Segment SGI304834[] = {{&S2494[35], 14}, {NULL}};
const Segment SGI304835[] = {{&S2495[30], 3}, {NULL}};
const Segment SGI304836[] = {{&S2495[169], 9}, {NULL}};
const Segment SGI304837[] = {{&S2495[8], 3}, {NULL}};
const Segment SGI304838[] = {{&S2495[13], 10}, {NULL}};
const Segment SGI304839[] = {{&S2495[42], 7}, {NULL}};
const Segment SGI304840[] = {{&S2495[35], 14}, {NULL}};
const Segment SGI304841[] = {{&S2496[30], 3}, {NULL}};
const Segment SGI304842[] = {{&S2496[181], 9}, {NULL}};
const Segment SGI304843[] = {{&S2496[8], 3}, {NULL}};
const Segment SGI304844[] = {{&S2496[13], 10}, {NULL}};
const Segment SGI304845[] = {{&S2496[42], 7}, {NULL}};
const Segment SGI304846[] = {{&S2496[35], 14}, {NULL}};
const Segment SGI304847[] = {{&S2496[51], 5}, {NULL}};
const Segment SGI304848[] = {{&S2497[30], 3}, {NULL}};
const Segment SGI304849[] = {{&S2497[187], 9}, {NULL}};
const Segment SGI304850[] = {{&S2497[8], 3}, {NULL}};
const Segment SGI304851[] = {{&S2497[13], 10}, {NULL}};
const Segment SGI304852[] = {{&S2497[42], 7}, {NULL}};
const Segment SGI304853[] = {{&S2497[35], 14}, {NULL}};
const Segment SGI304854[] = {{&S2497[138], 15}, {NULL}};
const Segment SGI304855[] = {{&S2497[51], 6}, {NULL}};
const Segment SGI304856[] = {{&S2498[30], 3}, {NULL}};
const Segment SGI304857[] = {{&S2498[94], 9}, {NULL}};
const Segment SGI304858[] = {{&S2498[8], 3}, {NULL}};
const Segment SGI304859[] = {{&S2498[13], 10}, {NULL}};
const Segment SGI304860[] = {{&S2498[42], 7}, {NULL}};
const Segment SGI304861[] = {{&S2498[35], 14}, {NULL}};
const Segment SGI304862[] = {{&S2498[51], 5}, {NULL}};
const Segment SGI304863[] = {{&S2499[30], 3}, {NULL}};
const Segment SGI304864[] = {{&S2499[209], 9}, {NULL}};
const Segment SGI304865[] = {{&S2499[8], 3}, {NULL}};
const Segment SGI304866[] = {{&S2499[13], 10}, {NULL}};
const Segment SGI304867[] = {{&S2499[42], 7}, {NULL}};
const Segment SGI304868[] = {{&S2499[35], 14}, {NULL}};
const Segment SGI304869[] = {{&S2499[138], 15}, {NULL}};
const Segment SGI304870[] = {{&S2499[51], 6}, {NULL}};
const Segment SGI304871[] = {{&S2501[30], 3}, {NULL}};
const Segment SGI304872[] = {{&S2501[222], 9}, {NULL}};
const Segment SGI304873[] = {{&S2501[8], 3}, {NULL}};
const Segment SGI304874[] = {{&S2501[13], 10}, {NULL}};
const Segment SGI304875[] = {{&S2501[42], 7}, {NULL}};
const Segment SGI304876[] = {{&S2501[35], 14}, {NULL}};
const Segment SGI304877[] = {{&S2501[138], 15}, {NULL}};
const Segment SGI304878[] = {{&S2501[51], 6}, {NULL}};
const Segment SGI304879[] = {{&S2502[30], 3}, {NULL}};
const Segment SGI304880[] = {{&S2502[207], 9}, {NULL}};
const Segment SGI304881[] = {{&S2502[8], 3}, {NULL}};
const Segment SGI304882[] = {{&S2502[13], 10}, {NULL}};
const Segment SGI304883[] = {{&S2502[42], 7}, {NULL}};
const Segment SGI304884[] = {{&S2502[35], 14}, {NULL}};
const Segment SGI304885[] = {{&S2502[51], 6}, {NULL}};
const Segment SGI304886[] = {{&S2504[30], 3}, {NULL}};
const Segment SGI304887[] = {{&S2504[202], 9}, {NULL}};
const Segment SGI304888[] = {{&S2504[8], 3}, {NULL}};
const Segment SGI304889[] = {{&S2504[13], 10}, {NULL}};
const Segment SGI304890[] = {{&S2504[42], 7}, {NULL}};
const Segment SGI304891[] = {{&S2504[35], 14}, {NULL}};
const Segment SGI304892[] = {{&S2504[131], 15}, {NULL}};
const Segment SGI304893[] = {{&S2504[174], 4}, {NULL}};
const Segment SGI304894[] = {{&S2505[30], 3}, {NULL}};
const Segment SGI304895[] = {{&S2505[210], 9}, {NULL}};
const Segment SGI304896[] = {{&S2505[8], 3}, {NULL}};
const Segment SGI304897[] = {{&S2505[13], 10}, {NULL}};
const Segment SGI304898[] = {{&S2505[42], 7}, {NULL}};
const Segment SGI304899[] = {{&S2505[35], 14}, {NULL}};
const Segment SGI304900[] = {{&S2505[138], 15}, {NULL}};
const Segment SGI304901[] = {{&S2505[51], 6}, {NULL}};
const Segment SGI304902[] = {{&S2506[30], 3}, {NULL}};
const Segment SGI304903[] = {{&S2506[224], 9}, {NULL}};
const Segment SGI304904[] = {{&S2506[8], 3}, {NULL}};
const Segment SGI304905[] = {{&S2506[13], 10}, {NULL}};
const Segment SGI304906[] = {{&S2506[42], 7}, {NULL}};
const Segment SGI304907[] = {{&S2506[35], 14}, {NULL}};
const Segment SGI304908[] = {{&S2506[143], 15}, {NULL}};
const Segment SGI304909[] = {{&S2506[196], 4}, {NULL}};
const Segment SGI304910[] = {{&S2506[51], 5}, {NULL}};
const Segment SGI304911[] = {{&S2507[30], 3}, {NULL}};
const Segment SGI304912[] = {{&S2507[196], 9}, {NULL}};
const Segment SGI304913[] = {{&S2507[8], 3}, {NULL}};
const Segment SGI304914[] = {{&S2507[13], 10}, {NULL}};
const Segment SGI304915[] = {{&S2507[42], 7}, {NULL}};
const Segment SGI304916[] = {{&S2507[35], 14}, {NULL}};
const Segment SGI304917[] = {{&S2507[51], 5}, {NULL}};
const Segment SGI304918[] = {{&S2507[163], 9}, {NULL}};
const Segment SGI304919[] = {{&S2508[30], 3}, {NULL}};
const Segment SGI304920[] = {{&S2508[239], 9}, {NULL}};
const Segment SGI304921[] = {{&S2508[8], 3}, {NULL}};
const Segment SGI304922[] = {{&S2508[13], 10}, {NULL}};
const Segment SGI304923[] = {{&S2508[42], 7}, {NULL}};
const Segment SGI304924[] = {{&S2508[35], 14}, {NULL}};
const Segment SGI304925[] = {{&S2508[143], 15}, {NULL}};
const Segment SGI304926[] = {{&S2508[211], 4}, {NULL}};
const Segment SGI304927[] = {{&S2508[51], 5}, {NULL}};
const Segment SGI304928[] = {{&S2508[196], 9}, {NULL}};
const Segment SGI304929[] = {{&S2509[30], 3}, {NULL}};
const Segment SGI304930[] = {{&S2509[231], 9}, {NULL}};
const Segment SGI304931[] = {{&S2509[8], 3}, {NULL}};
const Segment SGI304932[] = {{&S2509[13], 10}, {NULL}};
const Segment SGI304933[] = {{&S2509[42], 7}, {NULL}};
const Segment SGI304934[] = {{&S2509[35], 14}, {NULL}};
const Segment SGI304935[] = {{&S2509[131], 15}, {NULL}};
const Segment SGI304936[] = {{&S2509[191], 4}, {NULL}};
const Segment SGI304937[] = {{&S2510[30], 3}, {NULL}};
const Segment SGI304938[] = {{&S2510[197], 9}, {NULL}};
const Segment SGI304939[] = {{&S2510[8], 3}, {NULL}};
const Segment SGI304940[] = {{&S2510[13], 10}, {NULL}};
const Segment SGI304941[] = {{&S2510[42], 7}, {NULL}};
const Segment SGI304942[] = {{&S2510[35], 14}, {NULL}};
const Segment SGI304943[] = {{&S2510[138], 15}, {NULL}};
const Segment SGI304944[] = {{&S2510[51], 6}, {NULL}};
const Segment SGI304945[] = {{&S2511[30], 3}, {NULL}};
const Segment SGI304946[] = {{&S2511[203], 9}, {NULL}};
const Segment SGI304947[] = {{&S2511[8], 3}, {NULL}};
const Segment SGI304948[] = {{&S2511[13], 10}, {NULL}};
const Segment SGI304949[] = {{&S2511[42], 7}, {NULL}};
const Segment SGI304950[] = {{&S2511[35], 14}, {NULL}};
const Segment SGI304951[] = {{&S2511[131], 15}, {NULL}};
const Segment SGI304952[] = {{&S2512[30], 3}, {NULL}};
const Segment SGI304953[] = {{&S2512[229], 9}, {NULL}};
const Segment SGI304954[] = {{&S2512[8], 3}, {NULL}};
const Segment SGI304955[] = {{&S2512[13], 10}, {NULL}};
const Segment SGI304956[] = {{&S2512[42], 7}, {NULL}};
const Segment SGI304957[] = {{&S2512[35], 14}, {NULL}};
const Segment SGI304958[] = {{&S2512[131], 15}, {NULL}};
const Segment SGI304959[] = {{&S2512[185], 9}, {NULL}};
const Segment SGI304960[] = {{&S2513[30], 3}, {NULL}};
const Segment SGI304961[] = {{&S2513[203], 9}, {NULL}};
const Segment SGI304962[] = {{&S2513[8], 3}, {NULL}};
const Segment SGI304963[] = {{&S2513[13], 10}, {NULL}};
const Segment SGI304964[] = {{&S2513[42], 7}, {NULL}};
const Segment SGI304965[] = {{&S2513[35], 14}, {NULL}};
const Segment SGI304966[] = {{&S2513[131], 15}, {NULL}};
const Segment SGI304967[] = {{&S2514[30], 3}, {NULL}};
const Segment SGI304968[] = {{&S2514[235], 9}, {NULL}};
const Segment SGI304969[] = {{&S2514[8], 3}, {NULL}};
const Segment SGI304970[] = {{&S2514[13], 10}, {NULL}};
const Segment SGI304971[] = {{&S2514[42], 7}, {NULL}};
const Segment SGI304972[] = {{&S2514[35], 14}, {NULL}};
const Segment SGI304973[] = {{&S2514[131], 15}, {&S2514[131], 15}, {NULL}};
const Segment SGI304974[] = {{&S2514[202], 9}, {NULL}};
const Segment SGI304975[] = {{&S2515[30], 3}, {NULL}};
const Segment SGI304976[] = {{&S2515[266], 9}, {NULL}};
const Segment SGI304977[] = {{&S2515[8], 3}, {NULL}};
const Segment SGI304978[] = {{&S2515[13], 10}, {NULL}};
const Segment SGI304979[] = {{&S2515[42], 7}, {NULL}};
const Segment SGI304980[] = {{&S2515[35], 14}, {NULL}};
const Segment SGI304981[] = {{&S2515[131], 15}, {&S2515[131], 15}, {NULL}};
const Segment SGI304982[] = {{&S2516[30], 3}, {NULL}};
const Segment SGI304983[] = {{&S2516[184], 9}, {NULL}};
const Segment SGI304984[] = {{&S2516[8], 3}, {NULL}};
const Segment SGI304985[] = {{&S2516[13], 10}, {NULL}};
const Segment SGI304986[] = {{&S2516[42], 7}, {NULL}};
const Segment SGI304987[] = {{&S2516[35], 14}, {NULL}};
const Segment SGI304988[] = {{&S2516[151], 9}, {NULL}};
const Segment SGI304989[] = {{&S2517[30], 3}, {NULL}};
const Segment SGI304990[] = {{&S2517[207], 9}, {NULL}};
const Segment SGI304991[] = {{&S2517[8], 3}, {NULL}};
const Segment SGI304992[] = {{&S2517[13], 10}, {NULL}};
const Segment SGI304993[] = {{&S2517[42], 7}, {NULL}};
const Segment SGI304994[] = {{&S2517[35], 14}, {NULL}};
const Segment SGI304995[] = {{&S2517[136], 15}, {NULL}};
const Segment SGI304996[] = {{&S2517[64], 3}, {NULL}};
const Segment SGI304997[] = {{&S2518[30], 3}, {NULL}};
const Segment SGI304998[] = {{&S2518[222], 9}, {NULL}};
const Segment SGI304999[] = {{&S2518[8], 3}, {NULL}};
const Segment SGI305000[] = {{&S2518[13], 10}, {NULL}};
const Segment SGI305001[] = {{&S2518[42], 7}, {NULL}};
const Segment SGI305002[] = {{&S2518[35], 14}, {NULL}};
const Segment SGI305003[] = {{&S2518[136], 15}, {NULL}};
const Segment SGI305004[] = {{&S2518[189], 9}, {NULL}};
const Segment SGI305005[] = {{&S2518[64], 3}, {NULL}};
const Segment SGI305006[] = {{&S2519[30], 3}, {NULL}};
const Segment SGI305007[] = {{&S2519[230], 9}, {NULL}};
const Segment SGI305008[] = {{&S2519[8], 3}, {NULL}};
const Segment SGI305009[] = {{&S2519[13], 10}, {NULL}};
const Segment SGI305010[] = {{&S2519[42], 7}, {NULL}};
const Segment SGI305011[] = {{&S2519[35], 14}, {NULL}};
const Segment SGI305012[] = {{&S2519[131], 15}, {NULL}};
const Segment SGI305013[] = {{&S2519[197], 9}, {NULL}};
const Segment SGI305014[] = {{&S2520[30], 3}, {NULL}};
const Segment SGI305015[] = {{&S2520[82], 9}, {NULL}};
const Segment SGI305016[] = {{&S2520[8], 3}, {NULL}};
const Segment SGI305017[] = {{&S2520[13], 10}, {NULL}};
const Segment SGI305018[] = {{&S2520[42], 7}, {NULL}};
const Segment SGI305019[] = {{&S2520[35], 14}, {NULL}};
const Segment SGI305020[] = {{&S2521[30], 3}, {NULL}};
const Segment SGI305021[] = {{&S2521[205], 9}, {NULL}};
const Segment SGI305022[] = {{&S2521[8], 3}, {NULL}};
const Segment SGI305023[] = {{&S2521[13], 10}, {NULL}};
const Segment SGI305024[] = {{&S2521[42], 7}, {NULL}};
const Segment SGI305025[] = {{&S2521[35], 14}, {NULL}};
const Segment SGI305026[] = {{&S2521[131], 15}, {NULL}};
const Segment SGI305027[] = {{&S2521[177], 4}, {NULL}};
const Segment SGI305028[] = {{&S2521[162], 9}, {NULL}};
const Segment SGI305029[] = {{&S2522[30], 3}, {NULL}};
const Segment SGI305030[] = {{&S2522[176], 9}, {NULL}};
const Segment SGI305031[] = {{&S2522[8], 3}, {NULL}};
const Segment SGI305032[] = {{&S2522[13], 10}, {NULL}};
const Segment SGI305033[] = {{&S2522[42], 7}, {NULL}};
const Segment SGI305034[] = {{&S2522[35], 14}, {NULL}};
const Segment SGI305035[] = {{&S2522[51], 6}, {NULL}};
const Segment SGI305036[] = {{&S2523[30], 3}, {NULL}};
const Segment SGI305037[] = {{&S2523[89], 9}, {NULL}};
const Segment SGI305038[] = {{&S2523[8], 3}, {NULL}};
const Segment SGI305039[] = {{&S2523[13], 10}, {NULL}};
const Segment SGI305040[] = {{&S2523[42], 7}, {NULL}};
const Segment SGI305041[] = {{&S2523[35], 14}, {NULL}};
const Segment SGI305042[] = {{&S2523[51], 6}, {NULL}};
const Segment SGI305043[] = {{&S2524[30], 3}, {NULL}};
const Segment SGI305044[] = {{&S2524[190], 9}, {NULL}};
const Segment SGI305045[] = {{&S2524[8], 3}, {NULL}};
const Segment SGI305046[] = {{&S2524[13], 10}, {NULL}};
const Segment SGI305047[] = {{&S2524[42], 7}, {NULL}};
const Segment SGI305048[] = {{&S2524[35], 14}, {NULL}};
const Segment SGI305049[] = {{&S2524[131], 15}, {NULL}};
const Segment SGI305050[] = {{&S2525[30], 3}, {NULL}};
const Segment SGI305051[] = {{&S2525[197], 9}, {NULL}};
const Segment SGI305052[] = {{&S2525[8], 3}, {NULL}};
const Segment SGI305053[] = {{&S2525[13], 10}, {NULL}};
const Segment SGI305054[] = {{&S2525[42], 7}, {NULL}};
const Segment SGI305055[] = {{&S2525[35], 14}, {NULL}};
const Segment SGI305056[] = {{&S2525[138], 15}, {NULL}};
const Segment SGI305057[] = {{&S2525[51], 6}, {NULL}};
const Segment SGI305058[] = {{&S2526[30], 3}, {NULL}};
const Segment SGI305059[] = {{&S2526[181], 9}, {NULL}};
const Segment SGI305060[] = {{&S2526[8], 3}, {NULL}};
const Segment SGI305061[] = {{&S2526[13], 10}, {NULL}};
const Segment SGI305062[] = {{&S2526[42], 7}, {NULL}};
const Segment SGI305063[] = {{&S2526[35], 14}, {NULL}};
const Segment SGI305064[] = {{&S2526[51], 5}, {NULL}};
const Segment SGI305065[] = {{&S2527[30], 3}, {NULL}};
const Segment SGI305066[] = {{&S2527[214], 9}, {NULL}};
const Segment SGI305067[] = {{&S2527[8], 3}, {NULL}};
const Segment SGI305068[] = {{&S2527[13], 10}, {NULL}};
const Segment SGI305069[] = {{&S2527[42], 7}, {NULL}};
const Segment SGI305070[] = {{&S2527[35], 14}, {NULL}};
const Segment SGI305071[] = {{&S2527[143], 15}, {NULL}};
const Segment SGI305072[] = {{&S2527[51], 5}, {NULL}};
const Segment SGI305073[] = {{&S2528[30], 3}, {NULL}};
const Segment SGI305074[] = {{&S2528[205], 9}, {NULL}};
const Segment SGI305075[] = {{&S2528[8], 3}, {NULL}};
const Segment SGI305076[] = {{&S2528[13], 10}, {NULL}};
const Segment SGI305077[] = {{&S2528[42], 7}, {NULL}};
const Segment SGI305078[] = {{&S2528[35], 14}, {NULL}};
const Segment SGI305079[] = {{&S2528[131], 15}, {NULL}};
const Segment SGI305080[] = {{&S2528[172], 9}, {NULL}};
const Segment SGI305081[] = {{&S2529[30], 3}, {NULL}};
const Segment SGI305082[] = {{&S2529[229], 9}, {NULL}};
const Segment SGI305083[] = {{&S2529[8], 3}, {NULL}};
const Segment SGI305084[] = {{&S2529[13], 10}, {NULL}};
const Segment SGI305085[] = {{&S2529[42], 7}, {NULL}};
const Segment SGI305086[] = {{&S2529[35], 14}, {NULL}};
const Segment SGI305087[] = {{&S2529[143], 15}, {NULL}};
const Segment SGI305088[] = {{&S2529[51], 5}, {NULL}};
const Segment SGI305089[] = {{&S2529[196], 9}, {NULL}};
const Segment SGI305090[] = {{&S2530[30], 3}, {NULL}};
const Segment SGI305091[] = {{&S2530[82], 9}, {NULL}};
const Segment SGI305092[] = {{&S2530[8], 3}, {NULL}};
const Segment SGI305093[] = {{&S2530[13], 10}, {NULL}};
const Segment SGI305094[] = {{&S2530[42], 7}, {NULL}};
const Segment SGI305095[] = {{&S2530[35], 14}, {NULL}};
const Segment SGI305096[] = {{&S2531[30], 3}, {NULL}};
const Segment SGI305097[] = {{&S2531[209], 9}, {NULL}};
const Segment SGI305098[] = {{&S2531[8], 3}, {NULL}};
const Segment SGI305099[] = {{&S2531[13], 10}, {NULL}};
const Segment SGI305100[] = {{&S2531[42], 7}, {NULL}};
const Segment SGI305101[] = {{&S2531[35], 14}, {NULL}};
const Segment SGI305102[] = {{&S2531[138], 15}, {NULL}};
const Segment SGI305103[] = {{&S2531[51], 6}, {NULL}};
const Segment SGI305104[] = {{&S2532[30], 3}, {NULL}};
const Segment SGI305105[] = {{&S2532[169], 9}, {NULL}};
const Segment SGI305106[] = {{&S2532[8], 3}, {NULL}};
const Segment SGI305107[] = {{&S2532[13], 10}, {NULL}};
const Segment SGI305108[] = {{&S2532[42], 7}, {NULL}};
const Segment SGI305109[] = {{&S2532[35], 14}, {NULL}};
const Segment SGI305110[] = {{&S2533[30], 3}, {NULL}};
const Segment SGI305111[] = {{&S2533[94], 9}, {NULL}};
const Segment SGI305112[] = {{&S2533[8], 3}, {NULL}};
const Segment SGI305113[] = {{&S2533[13], 10}, {NULL}};
const Segment SGI305114[] = {{&S2533[42], 7}, {NULL}};
const Segment SGI305115[] = {{&S2533[35], 14}, {NULL}};
const Segment SGI305116[] = {{&S2533[51], 5}, {NULL}};
const Segment SGI305117[] = {{&S2534[30], 3}, {NULL}};
const Segment SGI305118[] = {{&S2534[207], 9}, {NULL}};
const Segment SGI305119[] = {{&S2534[8], 3}, {NULL}};
const Segment SGI305120[] = {{&S2534[13], 10}, {NULL}};
const Segment SGI305121[] = {{&S2534[42], 7}, {NULL}};
const Segment SGI305122[] = {{&S2534[35], 14}, {NULL}};
const Segment SGI305123[] = {{&S2534[138], 15}, {NULL}};
const Segment SGI305124[] = {{&S2534[179], 4}, {NULL}};
const Segment SGI305125[] = {{&S2534[51], 6}, {NULL}};
const Segment SGI305126[] = {{&S2535[30], 3}, {NULL}};
const Segment SGI305127[] = {{&S2535[187], 9}, {NULL}};
const Segment SGI305128[] = {{&S2535[8], 3}, {NULL}};
const Segment SGI305129[] = {{&S2535[13], 10}, {NULL}};
const Segment SGI305130[] = {{&S2535[42], 7}, {NULL}};
const Segment SGI305131[] = {{&S2535[35], 14}, {NULL}};
const Segment SGI305132[] = {{&S2535[138], 15}, {NULL}};
const Segment SGI305133[] = {{&S2535[51], 6}, {NULL}};
const Segment SGI305134[] = {{&S2536[30], 3}, {NULL}};
const Segment SGI305135[] = {{&S2536[207], 9}, {NULL}};
const Segment SGI305136[] = {{&S2536[8], 3}, {NULL}};
const Segment SGI305137[] = {{&S2536[13], 10}, {NULL}};
const Segment SGI305138[] = {{&S2536[42], 7}, {NULL}};
const Segment SGI305139[] = {{&S2536[35], 14}, {NULL}};
const Segment SGI305140[] = {{&S2536[136], 15}, {NULL}};
const Segment SGI305141[] = {{&S2536[64], 3}, {NULL}};
const Segment SGI305142[] = {{&S2537[30], 3}, {NULL}};
const Segment SGI305143[] = {{&S2537[222], 9}, {NULL}};
const Segment SGI305144[] = {{&S2537[8], 3}, {NULL}};
const Segment SGI305145[] = {{&S2537[13], 10}, {NULL}};
const Segment SGI305146[] = {{&S2537[42], 7}, {NULL}};
const Segment SGI305147[] = {{&S2537[35], 14}, {NULL}};
const Segment SGI305148[] = {{&S2537[136], 15}, {NULL}};
const Segment SGI305149[] = {{&S2537[189], 9}, {NULL}};
const Segment SGI305150[] = {{&S2537[64], 3}, {NULL}};
const Segment SGI305151[] = {{&S2538[30], 3}, {NULL}};
const Segment SGI305152[] = {{&S2538[222], 9}, {NULL}};
const Segment SGI305153[] = {{&S2538[8], 3}, {NULL}};
const Segment SGI305154[] = {{&S2538[13], 10}, {NULL}};
const Segment SGI305155[] = {{&S2538[42], 7}, {NULL}};
const Segment SGI305156[] = {{&S2538[35], 14}, {NULL}};
const Segment SGI305157[] = {{&S2538[138], 15}, {NULL}};
const Segment SGI305158[] = {{&S2538[51], 6}, {NULL}};
const Segment SGI305159[] = {{&S2539[30], 3}, {NULL}};
const Segment SGI305160[] = {{&S2539[201], 9}, {NULL}};
const Segment SGI305161[] = {{&S2539[8], 3}, {NULL}};
const Segment SGI305162[] = {{&S2539[13], 10}, {NULL}};
const Segment SGI305163[] = {{&S2539[42], 7}, {NULL}};
const Segment SGI305164[] = {{&S2539[35], 14}, {NULL}};
const Segment SGI305165[] = {{&S2539[51], 6}, {NULL}};
const Segment SGI305166[] = {{&S2540[30], 3}, {NULL}};
const Segment SGI305167[] = {{&S2540[196], 9}, {NULL}};
const Segment SGI305168[] = {{&S2540[8], 3}, {NULL}};
const Segment SGI305169[] = {{&S2540[13], 10}, {NULL}};
const Segment SGI305170[] = {{&S2540[42], 7}, {NULL}};
const Segment SGI305171[] = {{&S2540[35], 14}, {NULL}};
const Segment SGI305172[] = {{&S2540[51], 5}, {NULL}};
const Segment SGI305173[] = {{&S2540[163], 9}, {NULL}};
const Segment SGI305174[] = {{&S2541[30], 4}, {NULL}};
const Segment SGI305175[] = {{&S2541[90], 9}, {NULL}};
const Segment SGI305176[] = {{&S2541[8], 3}, {NULL}};
const Segment SGI305177[] = {{&S2541[13], 10}, {NULL}};
const Segment SGI305178[] = {{&S2541[43], 7}, {NULL}};
const Segment SGI305179[] = {{&S2541[36], 14}, {NULL}};
const Segment SGI305180[] = {{&S2541[52], 6}, {NULL}};
const Segment SGI305181[] = {{&S2542[30], 3}, {NULL}};
const Segment SGI305182[] = {{&S2542[184], 9}, {NULL}};
const Segment SGI305183[] = {{&S2542[8], 3}, {NULL}};
const Segment SGI305184[] = {{&S2542[13], 10}, {NULL}};
const Segment SGI305185[] = {{&S2542[42], 7}, {NULL}};
const Segment SGI305186[] = {{&S2542[35], 14}, {NULL}};
const Segment SGI305187[] = {{&S2542[151], 9}, {NULL}};
const Segment SGI305188[] = {{&S2543[30], 3}, {NULL}};
const Segment SGI305189[] = {{&S2543[239], 9}, {NULL}};
const Segment SGI305190[] = {{&S2543[8], 3}, {NULL}};
const Segment SGI305191[] = {{&S2543[13], 10}, {NULL}};
const Segment SGI305192[] = {{&S2543[42], 7}, {NULL}};
const Segment SGI305193[] = {{&S2543[35], 14}, {NULL}};
const Segment SGI305194[] = {{&S2543[143], 15}, {NULL}};
const Segment SGI305195[] = {{&S2543[211], 4}, {NULL}};
const Segment SGI305196[] = {{&S2543[51], 5}, {NULL}};
const Segment SGI305197[] = {{&S2543[196], 9}, {NULL}};
const Segment SGI305198[] = {{&S2545[30], 3}, {NULL}};
const Segment SGI305199[] = {{&S2545[197], 9}, {NULL}};
const Segment SGI305200[] = {{&S2545[8], 3}, {NULL}};
const Segment SGI305201[] = {{&S2545[13], 10}, {NULL}};
const Segment SGI305202[] = {{&S2545[42], 7}, {NULL}};
const Segment SGI305203[] = {{&S2545[35], 14}, {NULL}};
const Segment SGI305204[] = {{&S2545[138], 15}, {NULL}};
const Segment SGI305205[] = {{&S2545[51], 6}, {NULL}};
const Segment SGI305206[] = {{&S2546[30], 4}, {NULL}};
const Segment SGI305207[] = {{&S2546[95], 9}, {NULL}};
const Segment SGI305208[] = {{&S2546[8], 3}, {NULL}};
const Segment SGI305209[] = {{&S2546[13], 10}, {NULL}};
const Segment SGI305210[] = {{&S2546[43], 7}, {NULL}};
const Segment SGI305211[] = {{&S2546[36], 14}, {NULL}};
const Segment SGI305212[] = {{&S2546[52], 5}, {NULL}};
const Segment SGI305213[] = {{&S2547[30], 4}, {NULL}};
const Segment SGI305214[] = {{&S2547[83], 9}, {NULL}};
const Segment SGI305215[] = {{&S2547[8], 3}, {NULL}};
const Segment SGI305216[] = {{&S2547[13], 10}, {NULL}};
const Segment SGI305217[] = {{&S2547[43], 7}, {NULL}};
const Segment SGI305218[] = {{&S2547[36], 14}, {NULL}};
const Segment SGI305219[] = {{&S2548[30], 3}, {NULL}};
const Segment SGI305220[] = {{&S2548[205], 9}, {NULL}};
const Segment SGI305221[] = {{&S2548[8], 3}, {NULL}};
const Segment SGI305222[] = {{&S2548[13], 10}, {NULL}};
const Segment SGI305223[] = {{&S2548[42], 7}, {NULL}};
const Segment SGI305224[] = {{&S2548[35], 14}, {NULL}};
const Segment SGI305225[] = {{&S2548[131], 15}, {NULL}};
const Segment SGI305226[] = {{&S2548[172], 9}, {NULL}};
const Segment SGI305227[] = {{&S2549[30], 3}, {NULL}};
const Segment SGI305228[] = {{&S2549[208], 9}, {NULL}};
const Segment SGI305229[] = {{&S2549[8], 3}, {NULL}};
const Segment SGI305230[] = {{&S2549[13], 10}, {NULL}};
const Segment SGI305231[] = {{&S2549[42], 7}, {NULL}};
const Segment SGI305232[] = {{&S2549[35], 14}, {NULL}};
const Segment SGI305233[] = {{&S2549[51], 6}, {NULL}};
const Segment SGI305234[] = {{&S2550[30], 3}, {NULL}};
const Segment SGI305235[] = {{&S2550[208], 9}, {NULL}};
const Segment SGI305236[] = {{&S2550[8], 3}, {NULL}};
const Segment SGI305237[] = {{&S2550[13], 10}, {NULL}};
const Segment SGI305238[] = {{&S2550[42], 7}, {NULL}};
const Segment SGI305239[] = {{&S2550[35], 14}, {NULL}};
const Segment SGI305240[] = {{&S2550[51], 6}, {NULL}};
const Segment SGI305241[] = {{&S2551[30], 3}, {NULL}};
const Segment SGI305242[] = {{&S2551[202], 9}, {NULL}};
const Segment SGI305243[] = {{&S2551[8], 3}, {NULL}};
const Segment SGI305244[] = {{&S2551[13], 10}, {NULL}};
const Segment SGI305245[] = {{&S2551[42], 7}, {NULL}};
const Segment SGI305246[] = {{&S2551[35], 14}, {NULL}};
const Segment SGI305247[] = {{&S2551[131], 15}, {NULL}};
const Segment SGI305248[] = {{&S2553[30], 3}, {NULL}};
const Segment SGI305249[] = {{&S2553[197], 9}, {NULL}};
const Segment SGI305250[] = {{&S2553[8], 3}, {NULL}};
const Segment SGI305251[] = {{&S2553[13], 10}, {NULL}};
const Segment SGI305252[] = {{&S2553[42], 7}, {NULL}};
const Segment SGI305253[] = {{&S2553[35], 14}, {NULL}};
const Segment SGI305254[] = {{&S2553[138], 15}, {NULL}};
const Segment SGI305255[] = {{&S2553[51], 6}, {NULL}};
const Segment SGI305256[] = {{&S2554[30], 3}, {NULL}};
const Segment SGI305257[] = {{&S2554[192], 9}, {NULL}};
const Segment SGI305258[] = {{&S2554[8], 3}, {NULL}};
const Segment SGI305259[] = {{&S2554[13], 10}, {NULL}};
const Segment SGI305260[] = {{&S2554[42], 7}, {NULL}};
const Segment SGI305261[] = {{&S2554[35], 14}, {NULL}};
const Segment SGI305262[] = {{&S2554[131], 15}, {NULL}};
const Segment SGI305263[] = {{&S2555[30], 3}, {NULL}};
const Segment SGI305264[] = {{&S2555[202], 9}, {NULL}};
const Segment SGI305265[] = {{&S2555[8], 3}, {NULL}};
const Segment SGI305266[] = {{&S2555[13], 10}, {NULL}};
const Segment SGI305267[] = {{&S2555[47], 7}, {NULL}};
const Segment SGI305268[] = {{&S2555[40], 14}, {NULL}};
const Segment SGI305269[] = {{&S2555[143], 15}, {NULL}};
const Segment SGI305270[] = {{&S2555[56], 6}, {NULL}};
const Segment SGI305271[] = {{&S2556[30], 3}, {NULL}};
const Segment SGI305272[] = {{&S2556[208], 9}, {NULL}};
const Segment SGI305273[] = {{&S2556[8], 3}, {NULL}};
const Segment SGI305274[] = {{&S2556[13], 10}, {NULL}};
const Segment SGI305275[] = {{&S2556[42], 7}, {NULL}};
const Segment SGI305276[] = {{&S2556[35], 14}, {NULL}};
const Segment SGI305277[] = {{&S2556[175], 9}, {NULL}};
const Segment SGI305278[] = {{&S2557[30], 3}, {NULL}};
const Segment SGI305279[] = {{&S2557[159], 9}, {NULL}};
const Segment SGI305280[] = {{&S2557[8], 3}, {NULL}};
const Segment SGI305281[] = {{&S2557[13], 10}, {NULL}};
const Segment SGI305282[] = {{&S2557[42], 7}, {NULL}};
const Segment SGI305283[] = {{&S2557[35], 14}, {NULL}};
const Segment SGI305284[] = {{&S2558[30], 3}, {NULL}};
const Segment SGI305285[] = {{&S2558[94], 9}, {NULL}};
const Segment SGI305286[] = {{&S2558[8], 3}, {NULL}};
const Segment SGI305287[] = {{&S2558[13], 10}, {NULL}};
const Segment SGI305288[] = {{&S2558[42], 7}, {NULL}};
const Segment SGI305289[] = {{&S2558[35], 14}, {NULL}};
const Segment SGI305290[] = {{&S2558[51], 5}, {NULL}};
const Segment SGI305291[] = {{&S2559[30], 3}, {NULL}};
const Segment SGI305292[] = {{&S2559[199], 9}, {NULL}};
const Segment SGI305293[] = {{&S2559[8], 3}, {NULL}};
const Segment SGI305294[] = {{&S2559[13], 10}, {NULL}};
const Segment SGI305295[] = {{&S2559[42], 7}, {NULL}};
const Segment SGI305296[] = {{&S2559[35], 14}, {NULL}};
const Segment SGI305297[] = {{&S2559[138], 15}, {NULL}};
const Segment SGI305298[] = {{&S2559[51], 6}, {NULL}};
const Segment SGI305299[] = {{&S2560[30], 3}, {NULL}};
const Segment SGI305300[] = {{&S2560[190], 9}, {NULL}};
const Segment SGI305301[] = {{&S2560[8], 3}, {NULL}};
const Segment SGI305302[] = {{&S2560[13], 10}, {NULL}};
const Segment SGI305303[] = {{&S2560[42], 7}, {NULL}};
const Segment SGI305304[] = {{&S2560[35], 14}, {NULL}};
const Segment SGI305305[] = {{&S2560[131], 15}, {NULL}};
const Segment SGI305306[] = {{&S2560[162], 4}, {NULL}};
const Segment SGI305307[] = {{&S2561[30], 3}, {NULL}};
const Segment SGI305308[] = {{&S2561[212], 9}, {NULL}};
const Segment SGI305309[] = {{&S2561[8], 3}, {NULL}};
const Segment SGI305310[] = {{&S2561[13], 10}, {NULL}};
const Segment SGI305311[] = {{&S2561[42], 7}, {NULL}};
const Segment SGI305312[] = {{&S2561[35], 14}, {NULL}};
const Segment SGI305313[] = {{&S2561[143], 15}, {NULL}};
const Segment SGI305314[] = {{&S2561[184], 4}, {NULL}};
const Segment SGI305315[] = {{&S2561[51], 5}, {NULL}};
const Segment SGI305316[] = {{&S2562[30], 3}, {NULL}};
const Segment SGI305317[] = {{&S2562[215], 9}, {NULL}};
const Segment SGI305318[] = {{&S2562[8], 3}, {NULL}};
const Segment SGI305319[] = {{&S2562[13], 10}, {NULL}};
const Segment SGI305320[] = {{&S2562[42], 7}, {NULL}};
const Segment SGI305321[] = {{&S2562[35], 14}, {NULL}};
const Segment SGI305322[] = {{&S2562[131], 15}, {NULL}};
const Segment SGI305323[] = {{&S2563[30], 4}, {NULL}};
const Segment SGI305324[] = {{&S2563[90], 9}, {NULL}};
const Segment SGI305325[] = {{&S2563[8], 3}, {NULL}};
const Segment SGI305326[] = {{&S2563[13], 10}, {NULL}};
const Segment SGI305327[] = {{&S2563[43], 7}, {NULL}};
const Segment SGI305328[] = {{&S2563[36], 14}, {NULL}};
const Segment SGI305329[] = {{&S2563[52], 6}, {NULL}};
const Segment SGI305330[] = {{&S2564[30], 3}, {NULL}};
const Segment SGI305331[] = {{&S2564[184], 9}, {NULL}};
const Segment SGI305332[] = {{&S2564[8], 3}, {NULL}};
const Segment SGI305333[] = {{&S2564[13], 10}, {NULL}};
const Segment SGI305334[] = {{&S2564[42], 7}, {NULL}};
const Segment SGI305335[] = {{&S2564[35], 14}, {NULL}};
const Segment SGI305336[] = {{&S2564[151], 9}, {NULL}};
const Segment SGI305337[] = {{&S2565[30], 3}, {NULL}};
const Segment SGI305338[] = {{&S2565[224], 9}, {NULL}};
const Segment SGI305339[] = {{&S2565[8], 3}, {NULL}};
const Segment SGI305340[] = {{&S2565[13], 10}, {NULL}};
const Segment SGI305341[] = {{&S2565[42], 7}, {NULL}};
const Segment SGI305342[] = {{&S2565[35], 14}, {NULL}};
const Segment SGI305343[] = {{&S2565[131], 15}, {NULL}};
const Segment SGI305344[] = {{&S2566[30], 3}, {NULL}};
const Segment SGI305345[] = {{&S2566[209], 9}, {NULL}};
const Segment SGI305346[] = {{&S2566[8], 3}, {NULL}};
const Segment SGI305347[] = {{&S2566[13], 10}, {NULL}};
const Segment SGI305348[] = {{&S2566[42], 7}, {NULL}};
const Segment SGI305349[] = {{&S2566[35], 14}, {NULL}};
const Segment SGI305350[] = {{&S2566[138], 15}, {NULL}};
const Segment SGI305351[] = {{&S2566[51], 6}, {NULL}};
const Segment SGI305352[] = {{&S2567[30], 3}, {NULL}};
const Segment SGI305353[] = {{&S2567[222], 9}, {NULL}};
const Segment SGI305354[] = {{&S2567[8], 3}, {NULL}};
const Segment SGI305355[] = {{&S2567[13], 10}, {NULL}};
const Segment SGI305356[] = {{&S2567[42], 7}, {NULL}};
const Segment SGI305357[] = {{&S2567[35], 14}, {NULL}};
const Segment SGI305358[] = {{&S2567[138], 15}, {NULL}};
const Segment SGI305359[] = {{&S2567[51], 6}, {NULL}};
const Segment SGI305360[] = {{&S2568[30], 4}, {NULL}};
const Segment SGI305361[] = {{&S2568[198], 9}, {NULL}};
const Segment SGI305362[] = {{&S2568[8], 3}, {NULL}};
const Segment SGI305363[] = {{&S2568[13], 10}, {NULL}};
const Segment SGI305364[] = {{&S2568[43], 7}, {NULL}};
const Segment SGI305365[] = {{&S2568[36], 14}, {NULL}};
const Segment SGI305366[] = {{&S2568[139], 15}, {NULL}};
const Segment SGI305367[] = {{&S2568[52], 6}, {NULL}};
const Segment SGI305368[] = {{&S2569[30], 3}, {NULL}};
const Segment SGI305369[] = {{&S2569[220], 9}, {NULL}};
const Segment SGI305370[] = {{&S2569[8], 3}, {NULL}};
const Segment SGI305371[] = {{&S2569[13], 10}, {NULL}};
const Segment SGI305372[] = {{&S2569[42], 7}, {NULL}};
const Segment SGI305373[] = {{&S2569[35], 14}, {NULL}};
const Segment SGI305374[] = {{&S2569[131], 15}, {&S2569[131], 15}, {NULL}};
const Segment SGI305375[] = {{&S2570[30], 3}, {NULL}};
const Segment SGI305376[] = {{&S2570[212], 9}, {NULL}};
const Segment SGI305377[] = {{&S2570[8], 3}, {NULL}};
const Segment SGI305378[] = {{&S2570[13], 10}, {NULL}};
const Segment SGI305379[] = {{&S2570[42], 7}, {NULL}};
const Segment SGI305380[] = {{&S2570[35], 14}, {NULL}};
const Segment SGI305381[] = {{&S2570[138], 15}, {NULL}};
const Segment SGI305382[] = {{&S2570[51], 6}, {NULL}};
const Segment SGI305383[] = {{&S2570[179], 13}, {NULL}};
const Segment SGI305384[] = {{&S2571[30], 3}, {NULL}};
const Segment SGI305385[] = {{&S2571[205], 9}, {NULL}};
const Segment SGI305386[] = {{&S2571[8], 3}, {NULL}};
const Segment SGI305387[] = {{&S2571[13], 10}, {NULL}};
const Segment SGI305388[] = {{&S2571[42], 7}, {NULL}};
const Segment SGI305389[] = {{&S2571[35], 14}, {NULL}};
const Segment SGI305390[] = {{&S2571[131], 15}, {NULL}};
const Segment SGI305391[] = {{&S2571[177], 4}, {NULL}};
const Segment SGI305392[] = {{&S2571[162], 9}, {NULL}};
const Segment SGI305393[] = {{&S2572[30], 3}, {NULL}};
const Segment SGI305394[] = {{&S2572[230], 9}, {NULL}};
const Segment SGI305395[] = {{&S2572[8], 3}, {NULL}};
const Segment SGI305396[] = {{&S2572[13], 10}, {NULL}};
const Segment SGI305397[] = {{&S2572[42], 7}, {NULL}};
const Segment SGI305398[] = {{&S2572[35], 14}, {NULL}};
const Segment SGI305399[] = {{&S2572[131], 15}, {NULL}};
const Segment SGI305400[] = {{&S2572[197], 9}, {NULL}};
const Segment SGI305401[] = {{&S2573[30], 3}, {NULL}};
const Segment SGI305402[] = {{&S2573[207], 9}, {NULL}};
const Segment SGI305403[] = {{&S2573[8], 3}, {NULL}};
const Segment SGI305404[] = {{&S2573[13], 10}, {NULL}};
const Segment SGI305405[] = {{&S2573[42], 7}, {NULL}};
const Segment SGI305406[] = {{&S2573[35], 14}, {NULL}};
const Segment SGI305407[] = {{&S2573[51], 6}, {NULL}};
const Segment SGI305408[] = {{&S2574[30], 3}, {NULL}};
const Segment SGI305409[] = {{&S2574[187], 9}, {NULL}};
const Segment SGI305410[] = {{&S2574[8], 3}, {NULL}};
const Segment SGI305411[] = {{&S2574[13], 10}, {NULL}};
const Segment SGI305412[] = {{&S2574[42], 7}, {NULL}};
const Segment SGI305413[] = {{&S2574[35], 14}, {NULL}};
const Segment SGI305414[] = {{&S2574[51], 6}, {NULL}};
const Segment SGI305415[] = {{&S2574[71], 3}, {NULL}};
const Segment SGI305416[] = {{&S2575[30], 3}, {NULL}};
const Segment SGI305417[] = {{&S2575[224], 9}, {NULL}};
const Segment SGI305418[] = {{&S2575[8], 3}, {NULL}};
const Segment SGI305419[] = {{&S2575[13], 10}, {NULL}};
const Segment SGI305420[] = {{&S2575[42], 7}, {NULL}};
const Segment SGI305421[] = {{&S2575[35], 14}, {NULL}};
const Segment SGI305422[] = {{&S2575[131], 15}, {NULL}};
const Segment SGI305423[] = {{&S2575[191], 4}, {NULL}};
const Segment SGI305424[] = {{&S2576[30], 3}, {NULL}};
const Segment SGI305425[] = {{&S2576[188], 9}, {NULL}};
const Segment SGI305426[] = {{&S2576[8], 3}, {NULL}};
const Segment SGI305427[] = {{&S2576[13], 10}, {NULL}};
const Segment SGI305428[] = {{&S2576[42], 7}, {NULL}};
const Segment SGI305429[] = {{&S2576[35], 14}, {NULL}};
const Segment SGI305430[] = {{&S2576[51], 6}, {NULL}};
const Segment SGI305431[] = {{&S2577[30], 3}, {NULL}};
const Segment SGI305432[] = {{&S2577[223], 9}, {NULL}};
const Segment SGI305433[] = {{&S2577[8], 3}, {NULL}};
const Segment SGI305434[] = {{&S2577[13], 10}, {NULL}};
const Segment SGI305435[] = {{&S2577[42], 7}, {NULL}};
const Segment SGI305436[] = {{&S2577[35], 14}, {NULL}};
const Segment SGI305437[] = {{&S2577[131], 15}, {&S2577[131], 15}, {NULL}};
const Segment SGI305438[] = {{&S2577[190], 9}, {NULL}};
const Segment SGI305439[] = {{&S2578[30], 3}, {NULL}};
const Segment SGI305440[] = {{&S2578[192], 9}, {NULL}};
const Segment SGI305441[] = {{&S2578[8], 3}, {NULL}};
const Segment SGI305442[] = {{&S2578[13], 10}, {NULL}};
const Segment SGI305443[] = {{&S2578[42], 7}, {NULL}};
const Segment SGI305444[] = {{&S2578[35], 14}, {NULL}};
const Segment SGI305445[] = {{&S2578[143], 15}, {NULL}};
const Segment SGI305446[] = {{&S2578[51], 5}, {NULL}};
const Segment SGI305447[] = {{&S2579[30], 3}, {NULL}};
const Segment SGI305448[] = {{&S2579[217], 9}, {NULL}};
const Segment SGI305449[] = {{&S2579[8], 3}, {NULL}};
const Segment SGI305450[] = {{&S2579[13], 10}, {NULL}};
const Segment SGI305451[] = {{&S2579[42], 7}, {NULL}};
const Segment SGI305452[] = {{&S2579[35], 14}, {NULL}};
const Segment SGI305453[] = {{&S2579[131], 15}, {NULL}};
const Segment SGI305454[] = {{&S2579[184], 9}, {NULL}};
const Segment SGI305455[] = {{&S2580[30], 4}, {NULL}};
const Segment SGI305456[] = {{&S2580[90], 9}, {NULL}};
const Segment SGI305457[] = {{&S2580[8], 3}, {NULL}};
const Segment SGI305458[] = {{&S2580[13], 10}, {NULL}};
const Segment SGI305459[] = {{&S2580[43], 7}, {NULL}};
const Segment SGI305460[] = {{&S2580[36], 14}, {NULL}};
const Segment SGI305461[] = {{&S2580[52], 6}, {NULL}};
const Segment SGI305462[] = {{&S2581[30], 3}, {NULL}};
const Segment SGI305463[] = {{&S2581[197], 9}, {NULL}};
const Segment SGI305464[] = {{&S2581[8], 3}, {NULL}};
const Segment SGI305465[] = {{&S2581[13], 10}, {NULL}};
const Segment SGI305466[] = {{&S2581[42], 7}, {NULL}};
const Segment SGI305467[] = {{&S2581[35], 14}, {NULL}};
const Segment SGI305468[] = {{&S2581[51], 6}, {NULL}};
const Segment SGI305469[] = {{&S2582[30], 3}, {NULL}};
const Segment SGI305470[] = {{&S2582[210], 9}, {NULL}};
const Segment SGI305471[] = {{&S2582[8], 3}, {NULL}};
const Segment SGI305472[] = {{&S2582[13], 10}, {NULL}};
const Segment SGI305473[] = {{&S2582[42], 7}, {NULL}};
const Segment SGI305474[] = {{&S2582[35], 14}, {NULL}};
const Segment SGI305475[] = {{&S2582[138], 15}, {NULL}};
const Segment SGI305476[] = {{&S2582[51], 6}, {NULL}};
const Segment SGI305477[] = {{&S2583[30], 3}, {NULL}};
const Segment SGI305478[] = {{&S2583[202], 9}, {NULL}};
const Segment SGI305479[] = {{&S2583[8], 3}, {NULL}};
const Segment SGI305480[] = {{&S2583[13], 10}, {NULL}};
const Segment SGI305481[] = {{&S2583[42], 7}, {NULL}};
const Segment SGI305482[] = {{&S2583[35], 14}, {NULL}};
const Segment SGI305483[] = {{&S2583[131], 15}, {NULL}};
const Segment SGI305484[] = {{&S2584[30], 4}, {NULL}};
const Segment SGI305485[] = {{&S2584[90], 9}, {NULL}};
const Segment SGI305486[] = {{&S2584[8], 3}, {NULL}};
const Segment SGI305487[] = {{&S2584[13], 10}, {NULL}};
const Segment SGI305488[] = {{&S2584[43], 7}, {NULL}};
const Segment SGI305489[] = {{&S2584[36], 14}, {NULL}};
const Segment SGI305490[] = {{&S2584[52], 6}, {NULL}};
const Segment SGI305491[] = {{&S2585[30], 3}, {NULL}};
const Segment SGI305492[] = {{&S2585[203], 9}, {NULL}};
const Segment SGI305493[] = {{&S2585[8], 3}, {NULL}};
const Segment SGI305494[] = {{&S2585[13], 10}, {NULL}};
const Segment SGI305495[] = {{&S2585[42], 7}, {NULL}};
const Segment SGI305496[] = {{&S2585[35], 14}, {NULL}};
const Segment SGI305497[] = {{&S2585[131], 15}, {NULL}};
const Segment SGI305498[] = {{&S2586[30], 4}, {NULL}};
const Segment SGI305499[] = {{&S2586[215], 9}, {NULL}};
const Segment SGI305500[] = {{&S2586[8], 3}, {NULL}};
const Segment SGI305501[] = {{&S2586[13], 10}, {NULL}};
const Segment SGI305502[] = {{&S2586[43], 7}, {NULL}};
const Segment SGI305503[] = {{&S2586[36], 14}, {NULL}};
const Segment SGI305504[] = {{&S2586[144], 15}, {NULL}};
const Segment SGI305505[] = {{&S2586[52], 5}, {NULL}};
const Segment SGI305506[] = {{&S2587[30], 3}, {NULL}};
const Segment SGI305507[] = {{&S2587[190], 9}, {NULL}};
const Segment SGI305508[] = {{&S2587[8], 3}, {NULL}};
const Segment SGI305509[] = {{&S2587[13], 10}, {NULL}};
const Segment SGI305510[] = {{&S2587[42], 7}, {NULL}};
const Segment SGI305511[] = {{&S2587[35], 14}, {NULL}};
const Segment SGI305512[] = {{&S2587[131], 15}, {NULL}};
const Segment SGI305513[] = {{&S2588[30], 3}, {NULL}};
const Segment SGI305514[] = {{&S2588[214], 9}, {NULL}};
const Segment SGI305515[] = {{&S2588[8], 3}, {NULL}};
const Segment SGI305516[] = {{&S2588[13], 10}, {NULL}};
const Segment SGI305517[] = {{&S2588[42], 7}, {NULL}};
const Segment SGI305518[] = {{&S2588[35], 14}, {NULL}};
const Segment SGI305519[] = {{&S2588[143], 15}, {NULL}};
const Segment SGI305520[] = {{&S2588[51], 5}, {NULL}};
const Segment SGI305521[] = {{&S2590[30], 3}, {NULL}};
const Segment SGI305522[] = {{&S2590[197], 9}, {NULL}};
const Segment SGI305523[] = {{&S2590[8], 3}, {NULL}};
const Segment SGI305524[] = {{&S2590[13], 10}, {NULL}};
const Segment SGI305525[] = {{&S2590[42], 7}, {NULL}};
const Segment SGI305526[] = {{&S2590[35], 14}, {NULL}};
const Segment SGI305527[] = {{&S2590[138], 15}, {NULL}};
const Segment SGI305528[] = {{&S2590[51], 6}, {NULL}};
const Segment SGI305529[] = {{&S2591[30], 3}, {NULL}};
const Segment SGI305530[] = {{&S2591[194], 9}, {NULL}};
const Segment SGI305531[] = {{&S2591[8], 3}, {NULL}};
const Segment SGI305532[] = {{&S2591[13], 10}, {NULL}};
const Segment SGI305533[] = {{&S2591[42], 7}, {NULL}};
const Segment SGI305534[] = {{&S2591[35], 14}, {NULL}};
const Segment SGI305535[] = {{&S2592[30], 4}, {NULL}};
const Segment SGI305536[] = {{&S2592[181], 9}, {NULL}};
const Segment SGI305537[] = {{&S2592[8], 3}, {NULL}};
const Segment SGI305538[] = {{&S2592[13], 10}, {NULL}};
const Segment SGI305539[] = {{&S2592[43], 7}, {NULL}};
const Segment SGI305540[] = {{&S2592[36], 14}, {NULL}};
const Segment SGI305541[] = {{&S2592[132], 15}, {NULL}};
const Segment SGI305542[] = {{&S2593[30], 3}, {NULL}};
const Segment SGI305543[] = {{&S2593[230], 9}, {NULL}};
const Segment SGI305544[] = {{&S2593[8], 3}, {NULL}};
const Segment SGI305545[] = {{&S2593[13], 10}, {NULL}};
const Segment SGI305546[] = {{&S2593[42], 7}, {NULL}};
const Segment SGI305547[] = {{&S2593[35], 14}, {NULL}};
const Segment SGI305548[] = {{&S2593[131], 15}, {NULL}};
const Segment SGI305549[] = {{&S2593[197], 9}, {NULL}};
const Segment SGI305550[] = {{&S2594[30], 3}, {NULL}};
const Segment SGI305551[] = {{&S2594[222], 9}, {NULL}};
const Segment SGI305552[] = {{&S2594[8], 3}, {NULL}};
const Segment SGI305553[] = {{&S2594[13], 10}, {NULL}};
const Segment SGI305554[] = {{&S2594[42], 7}, {NULL}};
const Segment SGI305555[] = {{&S2594[35], 14}, {NULL}};
const Segment SGI305556[] = {{&S2594[136], 15}, {NULL}};
const Segment SGI305557[] = {{&S2594[189], 9}, {NULL}};
const Segment SGI305558[] = {{&S2594[64], 3}, {NULL}};
const Segment SGI305559[] = {{&S2595[30], 3}, {NULL}};
const Segment SGI305560[] = {{&S2595[222], 9}, {NULL}};
const Segment SGI305561[] = {{&S2595[8], 3}, {NULL}};
const Segment SGI305562[] = {{&S2595[13], 10}, {NULL}};
const Segment SGI305563[] = {{&S2595[42], 7}, {NULL}};
const Segment SGI305564[] = {{&S2595[35], 14}, {NULL}};
const Segment SGI305565[] = {{&S2595[138], 15}, {NULL}};
const Segment SGI305566[] = {{&S2595[51], 6}, {NULL}};
const Segment SGI305567[] = {{&S2596[30], 3}, {NULL}};
const Segment SGI305568[] = {{&S2596[217], 9}, {NULL}};
const Segment SGI305569[] = {{&S2596[8], 3}, {NULL}};
const Segment SGI305570[] = {{&S2596[13], 10}, {NULL}};
const Segment SGI305571[] = {{&S2596[42], 7}, {NULL}};
const Segment SGI305572[] = {{&S2596[35], 14}, {NULL}};
const Segment SGI305573[] = {{&S2596[143], 15}, {NULL}};
const Segment SGI305574[] = {{&S2596[51], 5}, {NULL}};
const Segment SGI305575[] = {{&S2596[184], 9}, {NULL}};
const Segment SGI305576[] = {{&S2597[30], 3}, {NULL}};
const Segment SGI305577[] = {{&S2597[205], 9}, {NULL}};
const Segment SGI305578[] = {{&S2597[8], 3}, {NULL}};
const Segment SGI305579[] = {{&S2597[13], 10}, {NULL}};
const Segment SGI305580[] = {{&S2597[42], 7}, {NULL}};
const Segment SGI305581[] = {{&S2597[35], 14}, {NULL}};
const Segment SGI305582[] = {{&S2597[131], 15}, {NULL}};
const Segment SGI305583[] = {{&S2597[172], 9}, {NULL}};
const Segment SGI305584[] = {{&S2598[30], 3}, {NULL}};
const Segment SGI305585[] = {{&S2598[207], 9}, {NULL}};
const Segment SGI305586[] = {{&S2598[8], 3}, {NULL}};
const Segment SGI305587[] = {{&S2598[13], 10}, {NULL}};
const Segment SGI305588[] = {{&S2598[42], 7}, {NULL}};
const Segment SGI305589[] = {{&S2598[35], 14}, {NULL}};
const Segment SGI305590[] = {{&S2598[138], 15}, {NULL}};
const Segment SGI305591[] = {{&S2598[179], 4}, {NULL}};
const Segment SGI305592[] = {{&S2598[51], 6}, {NULL}};
const Segment SGI305593[] = {{&S2599[30], 3}, {NULL}};
const Segment SGI305594[] = {{&S2599[181], 9}, {NULL}};
const Segment SGI305595[] = {{&S2599[8], 3}, {NULL}};
const Segment SGI305596[] = {{&S2599[13], 10}, {NULL}};
const Segment SGI305597[] = {{&S2599[42], 7}, {NULL}};
const Segment SGI305598[] = {{&S2599[35], 14}, {NULL}};
const Segment SGI305599[] = {{&S2599[51], 5}, {NULL}};
const Segment SGI305600[] = {{&S2600[30], 3}, {NULL}};
const Segment SGI305601[] = {{&S2600[169], 9}, {NULL}};
const Segment SGI305602[] = {{&S2600[8], 3}, {NULL}};
const Segment SGI305603[] = {{&S2600[13], 10}, {NULL}};
const Segment SGI305604[] = {{&S2600[42], 7}, {NULL}};
const Segment SGI305605[] = {{&S2600[35], 14}, {NULL}};
const Segment SGI305606[] = {{&S2601[30], 4}, {NULL}};
const Segment SGI305607[] = {{&S2601[83], 9}, {NULL}};
const Segment SGI305608[] = {{&S2601[8], 3}, {NULL}};
const Segment SGI305609[] = {{&S2601[13], 10}, {NULL}};
const Segment SGI305610[] = {{&S2601[43], 7}, {NULL}};
const Segment SGI305611[] = {{&S2601[36], 14}, {NULL}};
const Segment SGI305612[] = {{&S2602[30], 3}, {NULL}};
const Segment SGI305613[] = {{&S2602[89], 9}, {NULL}};
const Segment SGI305614[] = {{&S2602[8], 3}, {NULL}};
const Segment SGI305615[] = {{&S2602[13], 10}, {NULL}};
const Segment SGI305616[] = {{&S2602[42], 7}, {NULL}};
const Segment SGI305617[] = {{&S2602[35], 14}, {NULL}};
const Segment SGI305618[] = {{&S2602[51], 6}, {NULL}};
const Segment SGI305619[] = {{&S2603[30], 3}, {NULL}};
const Segment SGI305620[] = {{&S2603[176], 9}, {NULL}};
const Segment SGI305621[] = {{&S2603[8], 3}, {NULL}};
const Segment SGI305622[] = {{&S2603[13], 10}, {NULL}};
const Segment SGI305623[] = {{&S2603[42], 7}, {NULL}};
const Segment SGI305624[] = {{&S2603[35], 14}, {NULL}};
const Segment SGI305625[] = {{&S2603[51], 6}, {NULL}};
const Segment SGI305626[] = {{&S2604[30], 4}, {NULL}};
const Segment SGI305627[] = {{&S2604[90], 9}, {NULL}};
const Segment SGI305628[] = {{&S2604[8], 3}, {NULL}};
const Segment SGI305629[] = {{&S2604[13], 10}, {NULL}};
const Segment SGI305630[] = {{&S2604[43], 7}, {NULL}};
const Segment SGI305631[] = {{&S2604[36], 14}, {NULL}};
const Segment SGI305632[] = {{&S2604[52], 6}, {NULL}};
const Segment SGI305633[] = {{&S2605[30], 3}, {NULL}};
const Segment SGI305634[] = {{&S2605[82], 9}, {NULL}};
const Segment SGI305635[] = {{&S2605[8], 3}, {NULL}};
const Segment SGI305636[] = {{&S2605[13], 10}, {NULL}};
const Segment SGI305637[] = {{&S2605[42], 7}, {NULL}};
const Segment SGI305638[] = {{&S2605[35], 14}, {NULL}};
const Segment SGI305639[] = {{&S2606[30], 4}, {NULL}};
const Segment SGI305640[] = {{&S2606[95], 9}, {NULL}};
const Segment SGI305641[] = {{&S2606[8], 3}, {NULL}};
const Segment SGI305642[] = {{&S2606[13], 10}, {NULL}};
const Segment SGI305643[] = {{&S2606[43], 7}, {NULL}};
const Segment SGI305644[] = {{&S2606[36], 14}, {NULL}};
const Segment SGI305645[] = {{&S2606[52], 5}, {NULL}};
const Segment SGI305646[] = {{&S2607[30], 3}, {NULL}};
const Segment SGI305647[] = {{&S2607[197], 9}, {NULL}};
const Segment SGI305648[] = {{&S2607[8], 3}, {NULL}};
const Segment SGI305649[] = {{&S2607[13], 10}, {NULL}};
const Segment SGI305650[] = {{&S2607[42], 7}, {NULL}};
const Segment SGI305651[] = {{&S2607[35], 14}, {NULL}};
const Segment SGI305652[] = {{&S2607[51], 6}, {NULL}};
const Segment SGI305653[] = {{&S2608[30], 3}, {NULL}};
const Segment SGI305654[] = {{&S2608[190], 9}, {NULL}};
const Segment SGI305655[] = {{&S2608[8], 3}, {NULL}};
const Segment SGI305656[] = {{&S2608[13], 10}, {NULL}};
const Segment SGI305657[] = {{&S2608[42], 7}, {NULL}};
const Segment SGI305658[] = {{&S2608[35], 14}, {NULL}};
const Segment SGI305659[] = {{&S2608[131], 15}, {NULL}};
const Segment SGI305660[] = {{&S2609[30], 4}, {NULL}};
const Segment SGI305661[] = {{&S2609[83], 9}, {NULL}};
const Segment SGI305662[] = {{&S2609[8], 3}, {NULL}};
const Segment SGI305663[] = {{&S2609[13], 10}, {NULL}};
const Segment SGI305664[] = {{&S2609[43], 7}, {NULL}};
const Segment SGI305665[] = {{&S2609[36], 14}, {NULL}};
const Segment SGI305666[] = {{&S2610[30], 3}, {NULL}};
const Segment SGI305667[] = {{&S2610[202], 9}, {NULL}};
const Segment SGI305668[] = {{&S2610[8], 3}, {NULL}};
const Segment SGI305669[] = {{&S2610[13], 10}, {NULL}};
const Segment SGI305670[] = {{&S2610[42], 7}, {NULL}};
const Segment SGI305671[] = {{&S2610[35], 14}, {NULL}};
const Segment SGI305672[] = {{&S2610[131], 15}, {NULL}};
const Segment SGI305673[] = {{&S2611[30], 3}, {NULL}};
const Segment SGI305674[] = {{&S2611[217], 9}, {NULL}};
const Segment SGI305675[] = {{&S2611[8], 3}, {NULL}};
const Segment SGI305676[] = {{&S2611[13], 10}, {NULL}};
const Segment SGI305677[] = {{&S2611[42], 7}, {NULL}};
const Segment SGI305678[] = {{&S2611[35], 14}, {NULL}};
const Segment SGI305679[] = {{&S2611[131], 15}, {NULL}};
const Segment SGI305680[] = {{&S2611[184], 9}, {NULL}};
const Segment SGI305681[] = {{&S2612[30], 3}, {NULL}};
const Segment SGI305682[] = {{&S2612[196], 9}, {NULL}};
const Segment SGI305683[] = {{&S2612[8], 3}, {NULL}};
const Segment SGI305684[] = {{&S2612[13], 10}, {NULL}};
const Segment SGI305685[] = {{&S2612[42], 7}, {NULL}};
const Segment SGI305686[] = {{&S2612[35], 14}, {NULL}};
const Segment SGI305687[] = {{&S2612[51], 5}, {NULL}};
const Segment SGI305688[] = {{&S2612[163], 9}, {NULL}};
const Segment SGI305689[] = {{&S2613[30], 3}, {NULL}};
const Segment SGI305690[] = {{&S2613[94], 9}, {NULL}};
const Segment SGI305691[] = {{&S2613[8], 3}, {NULL}};
const Segment SGI305692[] = {{&S2613[13], 10}, {NULL}};
const Segment SGI305693[] = {{&S2613[42], 7}, {NULL}};
const Segment SGI305694[] = {{&S2613[35], 14}, {NULL}};
const Segment SGI305695[] = {{&S2613[51], 5}, {NULL}};
const Segment SGI305696[] = {{&S2614[30], 3}, {NULL}};
const Segment SGI305697[] = {{&S2614[229], 9}, {NULL}};
const Segment SGI305698[] = {{&S2614[8], 3}, {NULL}};
const Segment SGI305699[] = {{&S2614[13], 10}, {NULL}};
const Segment SGI305700[] = {{&S2614[49], 7}, {NULL}};
const Segment SGI305701[] = {{&S2614[42], 14}, {NULL}};
const Segment SGI305702[] = {{&S2614[145], 15}, {NULL}};
const Segment SGI305703[] = {{&S2614[58], 6}, {NULL}};
const Segment SGI305704[] = {{&S2615[30], 4}, {NULL}};
const Segment SGI305705[] = {{&S2615[188], 9}, {NULL}};
const Segment SGI305706[] = {{&S2615[8], 3}, {NULL}};
const Segment SGI305707[] = {{&S2615[13], 10}, {NULL}};
const Segment SGI305708[] = {{&S2615[43], 7}, {NULL}};
const Segment SGI305709[] = {{&S2615[36], 14}, {NULL}};
const Segment SGI305710[] = {{&S2615[139], 15}, {NULL}};
const Segment SGI305711[] = {{&S2615[52], 6}, {NULL}};
const Segment SGI305712[] = {{&S2617[30], 3}, {NULL}};
const Segment SGI305713[] = {{&S2617[207], 9}, {NULL}};
const Segment SGI305714[] = {{&S2617[8], 3}, {NULL}};
const Segment SGI305715[] = {{&S2617[13], 10}, {NULL}};
const Segment SGI305716[] = {{&S2617[42], 7}, {NULL}};
const Segment SGI305717[] = {{&S2617[35], 14}, {NULL}};
const Segment SGI305718[] = {{&S2617[138], 15}, {NULL}};
const Segment SGI305719[] = {{&S2617[179], 4}, {NULL}};
const Segment SGI305720[] = {{&S2617[51], 6}, {NULL}};
const Segment SGI305721[] = {{&S2618[30], 3}, {NULL}};
const Segment SGI305722[] = {{&S2618[188], 9}, {NULL}};
const Segment SGI305723[] = {{&S2618[8], 3}, {NULL}};
const Segment SGI305724[] = {{&S2618[13], 10}, {NULL}};
const Segment SGI305725[] = {{&S2618[42], 7}, {NULL}};
const Segment SGI305726[] = {{&S2618[35], 14}, {NULL}};
const Segment SGI305727[] = {{&S2618[51], 6}, {NULL}};
const Segment SGI305728[] = {{&S2619[30], 3}, {NULL}};
const Segment SGI305729[] = {{&S2619[195], 9}, {NULL}};
const Segment SGI305730[] = {{&S2619[8], 3}, {NULL}};
const Segment SGI305731[] = {{&S2619[13], 10}, {NULL}};
const Segment SGI305732[] = {{&S2619[42], 7}, {NULL}};
const Segment SGI305733[] = {{&S2619[35], 14}, {NULL}};
const Segment SGI305734[] = {{&S2619[131], 15}, {NULL}};
const Segment SGI305735[] = {{&S2619[162], 9}, {NULL}};
const Segment SGI305736[] = {{&S2620[30], 3}, {NULL}};
const Segment SGI305737[] = {{&S2620[217], 9}, {NULL}};
const Segment SGI305738[] = {{&S2620[8], 3}, {NULL}};
const Segment SGI305739[] = {{&S2620[13], 10}, {NULL}};
const Segment SGI305740[] = {{&S2620[42], 7}, {NULL}};
const Segment SGI305741[] = {{&S2620[35], 14}, {NULL}};
const Segment SGI305742[] = {{&S2620[143], 15}, {NULL}};
const Segment SGI305743[] = {{&S2620[51], 5}, {NULL}};
const Segment SGI305744[] = {{&S2620[184], 9}, {NULL}};
const Segment SGI305745[] = {{&S2621[30], 3}, {NULL}};
const Segment SGI305746[] = {{&S2621[190], 9}, {NULL}};
const Segment SGI305747[] = {{&S2621[8], 3}, {NULL}};
const Segment SGI305748[] = {{&S2621[13], 10}, {NULL}};
const Segment SGI305749[] = {{&S2621[42], 7}, {NULL}};
const Segment SGI305750[] = {{&S2621[35], 14}, {NULL}};
const Segment SGI305751[] = {{&S2621[131], 15}, {NULL}};
const Segment SGI305752[] = {{&S2622[30], 3}, {NULL}};
const Segment SGI305753[] = {{&S2622[89], 9}, {NULL}};
const Segment SGI305754[] = {{&S2622[8], 3}, {NULL}};
const Segment SGI305755[] = {{&S2622[13], 10}, {NULL}};
const Segment SGI305756[] = {{&S2622[42], 7}, {NULL}};
const Segment SGI305757[] = {{&S2622[35], 14}, {NULL}};
const Segment SGI305758[] = {{&S2622[51], 6}, {NULL}};
const Segment SGI305759[] = {{&S2623[30], 3}, {NULL}};
const Segment SGI305760[] = {{&S2623[202], 9}, {NULL}};
const Segment SGI305761[] = {{&S2623[8], 3}, {NULL}};
const Segment SGI305762[] = {{&S2623[13], 10}, {NULL}};
const Segment SGI305763[] = {{&S2623[42], 7}, {NULL}};
const Segment SGI305764[] = {{&S2623[35], 14}, {NULL}};
const Segment SGI305765[] = {{&S2623[143], 15}, {NULL}};
const Segment SGI305766[] = {{&S2623[51], 5}, {NULL}};
const Segment SGI305767[] = {{&S2624[30], 4}, {NULL}};
const Segment SGI305768[] = {{&S2624[90], 9}, {NULL}};
const Segment SGI305769[] = {{&S2624[8], 3}, {NULL}};
const Segment SGI305770[] = {{&S2624[13], 10}, {NULL}};
const Segment SGI305771[] = {{&S2624[43], 7}, {NULL}};
const Segment SGI305772[] = {{&S2624[36], 14}, {NULL}};
const Segment SGI305773[] = {{&S2624[52], 6}, {NULL}};
const Segment SGI305774[] = {{&S2625[30], 4}, {NULL}};
const Segment SGI305775[] = {{&S2625[83], 9}, {NULL}};
const Segment SGI305776[] = {{&S2625[8], 3}, {NULL}};
const Segment SGI305777[] = {{&S2625[13], 10}, {NULL}};
const Segment SGI305778[] = {{&S2625[43], 7}, {NULL}};
const Segment SGI305779[] = {{&S2625[36], 14}, {NULL}};
const Segment SGI305780[] = {{&S2626[30], 3}, {NULL}};
const Segment SGI305781[] = {{&S2626[197], 9}, {NULL}};
const Segment SGI305782[] = {{&S2626[8], 3}, {NULL}};
const Segment SGI305783[] = {{&S2626[13], 10}, {NULL}};
const Segment SGI305784[] = {{&S2626[42], 7}, {NULL}};
const Segment SGI305785[] = {{&S2626[35], 14}, {NULL}};
const Segment SGI305786[] = {{&S2626[138], 15}, {NULL}};
const Segment SGI305787[] = {{&S2626[51], 6}, {NULL}};
const Segment SGI305788[] = {{&S2627[30], 4}, {NULL}};
const Segment SGI305789[] = {{&S2627[95], 9}, {NULL}};
const Segment SGI305790[] = {{&S2627[8], 3}, {NULL}};
const Segment SGI305791[] = {{&S2627[13], 10}, {NULL}};
const Segment SGI305792[] = {{&S2627[43], 7}, {NULL}};
const Segment SGI305793[] = {{&S2627[36], 14}, {NULL}};
const Segment SGI305794[] = {{&S2627[52], 5}, {NULL}};
const Segment SGI305795[] = {{&S2628[30], 3}, {NULL}};
const Segment SGI305796[] = {{&S2628[82], 9}, {NULL}};
const Segment SGI305797[] = {{&S2628[8], 3}, {NULL}};
const Segment SGI305798[] = {{&S2628[13], 10}, {NULL}};
const Segment SGI305799[] = {{&S2628[42], 7}, {NULL}};
const Segment SGI305800[] = {{&S2628[35], 14}, {NULL}};
const Segment SGI305801[] = {{&S2629[30], 3}, {NULL}};
const Segment SGI305802[] = {{&S2629[181], 9}, {NULL}};
const Segment SGI305803[] = {{&S2629[8], 3}, {NULL}};
const Segment SGI305804[] = {{&S2629[13], 10}, {NULL}};
const Segment SGI305805[] = {{&S2629[42], 7}, {NULL}};
const Segment SGI305806[] = {{&S2629[35], 14}, {NULL}};
const Segment SGI305807[] = {{&S2629[51], 5}, {NULL}};
const Segment SGI305808[] = {{&S2630[30], 4}, {NULL}};
const Segment SGI305809[] = {{&S2630[198], 9}, {NULL}};
const Segment SGI305810[] = {{&S2630[8], 3}, {NULL}};
const Segment SGI305811[] = {{&S2630[13], 10}, {NULL}};
const Segment SGI305812[] = {{&S2630[43], 7}, {NULL}};
const Segment SGI305813[] = {{&S2630[36], 14}, {NULL}};
const Segment SGI305814[] = {{&S2630[139], 15}, {NULL}};
const Segment SGI305815[] = {{&S2630[52], 6}, {NULL}};
const Segment SGI305816[] = {{&S2631[30], 3}, {NULL}};
const Segment SGI305817[] = {{&S2631[94], 9}, {NULL}};
const Segment SGI305818[] = {{&S2631[8], 3}, {NULL}};
const Segment SGI305819[] = {{&S2631[13], 10}, {NULL}};
const Segment SGI305820[] = {{&S2631[42], 7}, {NULL}};
const Segment SGI305821[] = {{&S2631[35], 14}, {NULL}};
const Segment SGI305822[] = {{&S2631[51], 5}, {NULL}};
const Segment SGI305823[] = {{&S2632[30], 3}, {NULL}};
const Segment SGI305824[] = {{&S2632[181], 9}, {NULL}};
const Segment SGI305825[] = {{&S2632[8], 3}, {NULL}};
const Segment SGI305826[] = {{&S2632[13], 10}, {NULL}};
const Segment SGI305827[] = {{&S2632[42], 7}, {NULL}};
const Segment SGI305828[] = {{&S2632[35], 14}, {NULL}};
const Segment SGI305829[] = {{&S2633[30], 3}, {NULL}};
const Segment SGI305830[] = {{&S2633[184], 9}, {NULL}};
const Segment SGI305831[] = {{&S2633[8], 3}, {NULL}};
const Segment SGI305832[] = {{&S2633[13], 10}, {NULL}};
const Segment SGI305833[] = {{&S2633[42], 7}, {NULL}};
const Segment SGI305834[] = {{&S2633[35], 14}, {NULL}};
const Segment SGI305835[] = {{&S2633[151], 9}, {NULL}};
const Segment SGI305836[] = {{&S2634[30], 3}, {NULL}};
const Segment SGI305837[] = {{&S2634[212], 9}, {NULL}};
const Segment SGI305838[] = {{&S2634[8], 3}, {NULL}};
const Segment SGI305839[] = {{&S2634[13], 10}, {NULL}};
const Segment SGI305840[] = {{&S2634[42], 7}, {NULL}};
const Segment SGI305841[] = {{&S2634[35], 14}, {NULL}};
const Segment SGI305842[] = {{&S2634[143], 15}, {NULL}};
const Segment SGI305843[] = {{&S2634[184], 4}, {NULL}};
const Segment SGI305844[] = {{&S2634[51], 5}, {NULL}};
const Segment SGI305845[] = {{&S2635[30], 3}, {NULL}};
const Segment SGI305846[] = {{&S2635[196], 9}, {NULL}};
const Segment SGI305847[] = {{&S2635[8], 3}, {NULL}};
const Segment SGI305848[] = {{&S2635[13], 10}, {NULL}};
const Segment SGI305849[] = {{&S2635[42], 7}, {NULL}};
const Segment SGI305850[] = {{&S2635[35], 14}, {NULL}};
const Segment SGI305851[] = {{&S2635[51], 5}, {NULL}};
const Segment SGI305852[] = {{&S2635[163], 9}, {NULL}};
const Segment SGI305853[] = {{&S2636[30], 3}, {NULL}};
const Segment SGI305854[] = {{&S2636[190], 9}, {NULL}};
const Segment SGI305855[] = {{&S2636[8], 3}, {NULL}};
const Segment SGI305856[] = {{&S2636[13], 10}, {NULL}};
const Segment SGI305857[] = {{&S2636[42], 7}, {NULL}};
const Segment SGI305858[] = {{&S2636[35], 14}, {NULL}};
const Segment SGI305859[] = {{&S2636[131], 15}, {NULL}};
const Segment SGI305860[] = {{&S2636[162], 4}, {NULL}};
const Segment SGI305861[] = {{&S2637[30], 3}, {NULL}};
const Segment SGI305862[] = {{&S2637[202], 9}, {NULL}};
const Segment SGI305863[] = {{&S2637[8], 3}, {NULL}};
const Segment SGI305864[] = {{&S2637[13], 10}, {NULL}};
const Segment SGI305865[] = {{&S2637[42], 7}, {NULL}};
const Segment SGI305866[] = {{&S2637[35], 14}, {NULL}};
const Segment SGI305867[] = {{&S2637[163], 15}, {NULL}};
const Segment SGI305868[] = {{&S2638[30], 3}, {NULL}};
const Segment SGI305869[] = {{&S2638[220], 9}, {NULL}};
const Segment SGI305870[] = {{&S2638[8], 3}, {NULL}};
const Segment SGI305871[] = {{&S2638[13], 10}, {NULL}};
const Segment SGI305872[] = {{&S2638[42], 7}, {NULL}};
const Segment SGI305873[] = {{&S2638[35], 14}, {NULL}};
const Segment SGI305874[] = {{&S2638[131], 15}, {&S2638[131], 15}, {NULL}};
const Segment SGI305875[] = {{&S2639[30], 3}, {NULL}};
const Segment SGI305876[] = {{&S2639[202], 9}, {NULL}};
const Segment SGI305877[] = {{&S2639[8], 3}, {NULL}};
const Segment SGI305878[] = {{&S2639[13], 10}, {NULL}};
const Segment SGI305879[] = {{&S2639[42], 7}, {NULL}};
const Segment SGI305880[] = {{&S2639[35], 14}, {NULL}};
const Segment SGI305881[] = {{&S2639[131], 15}, {NULL}};
const Segment SGI305882[] = {{&S2640[30], 3}, {NULL}};
const Segment SGI305883[] = {{&S2640[8], 3}, {NULL}};
const Segment SGI305884[] = {{&S2640[13], 10}, {NULL}};
const Segment SGI305885[] = {{&S2640[42], 7}, {NULL}};
const Segment SGI305886[] = {{&S2640[35], 14}, {NULL}};
const Segment SGI305887[] = {{&S2641[30], 3}, {NULL}};
const Segment SGI305888[] = {{&S2641[8], 3}, {NULL}};
const Segment SGI305889[] = {{&S2641[13], 10}, {NULL}};
const Segment SGI305890[] = {{&S2641[42], 7}, {NULL}};
const Segment SGI305891[] = {{&S2641[35], 14}, {NULL}};
const Segment SGI305892[] = {{&S2641[51], 6}, {NULL}};
const Segment SGI305893[] = {{&S2642[31], 3}, {NULL}};
const Segment SGI305894[] = {{&S2642[43], 7}, {NULL}};
const Segment SGI305895[] = {{&S2643[30], 3}, {NULL}};
const Segment SGI305896[] = {{&S2643[8], 3}, {NULL}};
const Segment SGI305897[] = {{&S2643[13], 10}, {NULL}};
const Segment SGI305898[] = {{&S2643[42], 7}, {NULL}};
const Segment SGI305899[] = {{&S2643[35], 14}, {NULL}};
const Segment SGI305900[] = {{&S2643[51], 5}, {NULL}};
const Segment SGI305901[] = {{&S2644[24], 3}, {NULL}};
const Segment SGI305902[] = {{&S2644[36], 7}, {NULL}};
const Segment SGI305903[] = {{&S2644[29], 14}, {NULL}};
const Segment SGI305904[] = {{&S2645[30], 3}, {&S2645[94], 3}, {NULL}};
const Segment SGI305905[] = {{&S2645[8], 3}, {&S2645[72], 3}, {NULL}};
const Segment SGI305906[] = {{&S2645[13], 10}, {&S2645[13], 10}, {NULL}};
const Segment SGI305907[] = {{&S2645[42], 7}, {NULL}};
const Segment SGI305908[] = {{&S2645[35], 14}, {NULL}};
const Segment SGI305909[] = {{&S2645[99], 5}, {NULL}};
const Segment SGI305910[] = {{&S2646[24], 3}, {NULL}};
const Segment SGI305911[] = {{&S2646[36], 7}, {NULL}};
const Segment SGI305912[] = {{&S2646[29], 14}, {NULL}};
const Segment SGI305913[] = {{&S2646[45], 6}, {NULL}};
const Segment SGI305914[] = {{&S2647[30], 3}, {NULL}};
const Segment SGI305915[] = {{&S2647[8], 3}, {NULL}};
const Segment SGI305916[] = {{&S2647[13], 10}, {NULL}};
const Segment SGI305917[] = {{&S2647[42], 7}, {&S2647[79], 5}, {NULL}};
const Segment SGI305918[] = {{&S2647[35], 14}, {NULL}};
const Segment SGI305919[] = {{&S2648[30], 3}, {NULL}};
const Segment SGI305920[] = {{&S2648[8], 3}, {NULL}};
const Segment SGI305921[] = {{&S2648[13], 10}, {NULL}};
const Segment SGI305922[] = {{&S2648[42], 7}, {NULL}};
const Segment SGI305923[] = {{&S2649[30], 3}, {NULL}};
const Segment SGI305924[] = {{&S2649[8], 3}, {NULL}};
const Segment SGI305925[] = {{&S2649[13], 10}, {NULL}};
const Segment SGI305926[] = {{&S2649[42], 7}, {&S2649[71], 9}, {NULL}};
const Segment SGI305927[] = {{&S2649[35], 14}, {NULL}};
const Segment SGI305928[] = {{&S2650[30], 3}, {&S2650[94], 3}, {NULL}};
const Segment SGI305929[] = {{&S2650[8], 3}, {&S2650[72], 3}, {NULL}};
const Segment SGI305930[] = {{&S2650[13], 10}, {&S2650[13], 10}, {NULL}};
const Segment SGI305931[] = {{&S2650[42], 7}, {&S2650[106], 7}, {NULL}};
const Segment SGI305932[] = {{&S2650[35], 14}, {&S2650[99], 14}, {NULL}};
const Segment SGI305933[] = {{&S2650[115], 6}, {NULL}};
const Segment SGI305934[] = {{&S2651[30], 3}, {NULL}};
const Segment SGI305935[] = {{&S2651[8], 3}, {NULL}};
const Segment SGI305936[] = {{&S2651[13], 10}, {NULL}};
const Segment SGI305937[] = {{&S2651[42], 7}, {&S2651[79], 5}, {&S2651[79], 5}, {NULL}};
const Segment SGI305938[] = {{&S2651[35], 14}, {NULL}};
const Segment SGI305939[] = {{&S2652[30], 3}, {NULL}};
const Segment SGI305940[] = {{&S2652[8], 3}, {NULL}};
const Segment SGI305941[] = {{&S2652[13], 10}, {NULL}};
const Segment SGI305942[] = {{&S2652[42], 7}, {NULL}};
const Segment SGI305943[] = {{&S2652[35], 14}, {NULL}};
const Segment SGI305944[] = {{&S2653[30], 14}, {NULL}};
const Segment SGI305945[] = {{&S2653[8], 3}, {NULL}};
const Segment SGI305946[] = {{&S2653[13], 10}, {NULL}};
const Segment SGI305947[] = {{&S2653[53], 7}, {NULL}};
const Segment SGI305948[] = {{&S2654[30], 3}, {NULL}};
const Segment SGI305949[] = {{&S2654[8], 3}, {NULL}};
const Segment SGI305950[] = {{&S2654[13], 10}, {NULL}};
const Segment SGI305951[] = {{&S2654[42], 7}, {NULL}};
const Segment SGI305952[] = {{&S2655[21], 3}, {NULL}};
const Segment SGI305953[] = {{&S2655[8], 3}, {NULL}};
const Segment SGI305954[] = {{&S2655[11], 1}, {NULL}};
const Segment SGI305955[] = {{&S2655[33], 7}, {NULL}};
const Segment SGI305956[] = {{&S2655[26], 14}, {NULL}};
const Segment SGI305957[] = {{&S2656[30], 3}, {NULL}};
const Segment SGI305958[] = {{&S2656[8], 3}, {NULL}};
const Segment SGI305959[] = {{&S2656[13], 10}, {NULL}};
const Segment SGI305960[] = {{&S2656[42], 7}, {NULL}};
const Segment SGI305961[] = {{&S2656[35], 14}, {NULL}};
const Segment SGI305962[] = {{&S2656[78], 15}, {NULL}};
const Segment SGI305963[] = {{&S2656[124], 4}, {NULL}};
const Segment SGI305964[] = {{&S2656[51], 6}, {NULL}};
const Segment SGI305965[] = {{&S2657[30], 3}, {NULL}};
const Segment SGI305966[] = {{&S2657[8], 3}, {NULL}};
const Segment SGI305967[] = {{&S2657[13], 10}, {NULL}};
const Segment SGI305968[] = {{&S2657[42], 7}, {&S2657[78], 9}, {NULL}};
const Segment SGI305969[] = {{&S2657[35], 14}, {NULL}};
const Segment SGI305970[] = {{&S2657[51], 6}, {NULL}};
const Segment SGI305971[] = {{&S2658[30], 3}, {&S2658[94], 3}, {&S2658[94], 3}, {NULL}};
const Segment SGI305972[] = {{&S2658[8], 3}, {&S2658[72], 3}, {&S2658[72], 3}, {NULL}};
const Segment SGI305973[] = {{&S2658[13], 10}, {&S2658[13], 10}, {&S2658[13], 10}, {NULL}};
const Segment SGI305974[] = {{&S2658[42], 7}, {NULL}};
const Segment SGI305975[] = {{&S2658[35], 14}, {NULL}};
const Segment SGI305976[] = {{&S2658[99], 5}, {&S2658[99], 5}, {NULL}};
const Segment SGI305977[] = {{&S2659[30], 3}, {&S2659[8], 3}, {NULL}};
const Segment SGI305978[] = {{&S2659[8], 3}, {&S2659[72], 3}, {NULL}};
const Segment SGI305979[] = {{&S2659[13], 10}, {&S2659[13], 10}, {NULL}};
const Segment SGI305980[] = {{&S2659[42], 7}, {&S2659[42], 3}, {NULL}};
const Segment SGI305981[] = {{&S2659[35], 14}, {NULL}};
const Segment SGI305982[] = {{&S2660[30], 3}, {NULL}};
const Segment SGI305983[] = {{&S2660[8], 3}, {NULL}};
const Segment SGI305984[] = {{&S2660[13], 10}, {NULL}};
const Segment SGI305985[] = {{&S2660[42], 7}, {NULL}};
const Segment SGI305986[] = {{&S2660[35], 14}, {NULL}};
const Segment SGI305987[] = {{&S2660[143], 15}, {NULL}};
const Segment SGI305988[] = {{&S2660[164], 4}, {NULL}};
const Segment SGI305989[] = {{&S2660[51], 5}, {NULL}};
const Segment SGI305990[] = {{&S2660[174], 9}, {NULL}};
const Segment SGI305991[] = {{&S2662[30], 3}, {NULL}};
const Segment SGI305992[] = {{&S2662[8], 3}, {NULL}};
const Segment SGI305993[] = {{&S2662[13], 10}, {NULL}};
const Segment SGI305994[] = {{&S2662[42], 7}, {NULL}};
const Segment SGI305995[] = {{&S2662[35], 14}, {NULL}};
const Segment SGI305996[] = {{&S2662[78], 15}, {NULL}};
const Segment SGI305997[] = {{&S2663[30], 3}, {NULL}};
const Segment SGI305998[] = {{&S2663[8], 3}, {NULL}};
const Segment SGI305999[] = {{&S2663[13], 10}, {NULL}};
const Segment SGI306000[] = {{&S2663[42], 7}, {NULL}};
const Segment SGI306001[] = {{&S2663[35], 14}, {NULL}};
const Segment SGI306002[] = {{&S2663[98], 15}, {NULL}};
const Segment SGI306003[] = {{&S2663[51], 6}, {NULL}};
const Segment SGI306004[] = {{&S2664[30], 3}, {NULL}};
const Segment SGI306005[] = {{&S2664[8], 3}, {NULL}};
const Segment SGI306006[] = {{&S2664[13], 10}, {NULL}};
const Segment SGI306007[] = {{&S2664[42], 6}, {NULL}};
const Segment SGI306008[] = {{&S2665[30], 3}, {NULL}};
const Segment SGI306009[] = {{&S2665[8], 3}, {NULL}};
const Segment SGI306010[] = {{&S2665[13], 10}, {NULL}};
const Segment SGI306011[] = {{&S2665[42], 7}, {&S2665[71], 8}, {NULL}};
const Segment SGI306012[] = {{&S2665[35], 14}, {NULL}};
const Segment SGI306013[] = {{&S2666[30], 3}, {&S2666[46], 3}, {NULL}};
const Segment SGI306014[] = {{&S2666[8], 3}, {&S2666[72], 3}, {NULL}};
const Segment SGI306015[] = {{&S2666[13], 10}, {&S2666[13], 10}, {NULL}};
const Segment SGI306016[] = {{&S2666[42], 7}, {NULL}};
const Segment SGI306017[] = {{&S2666[35], 14}, {NULL}};
const Segment SGI306018[] = {{&S2667[30], 3}, {NULL}};
const Segment SGI306019[] = {{&S2667[8], 3}, {NULL}};
const Segment SGI306020[] = {{&S2667[13], 10}, {NULL}};
const Segment SGI306021[] = {{&S2667[55], 7}, {NULL}};
const Segment SGI306022[] = {{&S2667[48], 14}, {NULL}};
const Segment SGI306023[] = {{&S2667[64], 3}, {NULL}};
const Segment SGI306024[] = {{&S2668[30], 3}, {NULL}};
const Segment SGI306025[] = {{&S2668[8], 3}, {NULL}};
const Segment SGI306026[] = {{&S2668[13], 10}, {NULL}};
const Segment SGI306027[] = {{&S2668[55], 7}, {NULL}};
const Segment SGI306028[] = {{&S2668[48], 14}, {NULL}};
const Segment SGI306029[] = {{&S2668[64], 3}, {NULL}};
const Segment SGI306030[] = {{&S2669[30], 3}, {NULL}};
const Segment SGI306031[] = {{&S2669[8], 3}, {NULL}};
const Segment SGI306032[] = {{&S2669[13], 10}, {NULL}};
const Segment SGI306033[] = {{&S2669[42], 7}, {NULL}};
const Segment SGI306034[] = {{&S2669[35], 14}, {NULL}};
const Segment SGI306035[] = {{&S2669[103], 15}, {NULL}};
const Segment SGI306036[] = {{&S2669[51], 5}, {NULL}};
const Segment SGI306037[] = {{&S2670[30], 3}, {NULL}};
const Segment SGI306038[] = {{&S2670[8], 3}, {NULL}};
const Segment SGI306039[] = {{&S2670[13], 10}, {NULL}};
const Segment SGI306040[] = {{&S2670[42], 7}, {&S2670[58], 16}, {NULL}};
const Segment SGI306041[] = {{&S2670[35], 14}, {NULL}};
const Segment SGI306042[] = {{&S2671[21], 3}, {NULL}};
const Segment SGI306043[] = {{&S2671[8], 3}, {NULL}};
const Segment SGI306044[] = {{&S2671[11], 1}, {NULL}};
const Segment SGI306045[] = {{&S2671[33], 7}, {NULL}};
const Segment SGI306046[] = {{&S2671[26], 14}, {NULL}};
const Segment SGI306047[] = {{&S2671[42], 6}, {NULL}};
const Segment SGI306048[] = {{&S2672[30], 3}, {NULL}};
const Segment SGI306049[] = {{&S2672[8], 3}, {NULL}};
const Segment SGI306050[] = {{&S2672[13], 10}, {NULL}};
const Segment SGI306051[] = {{&S2672[42], 7}, {NULL}};
const Segment SGI306052[] = {{&S2672[35], 14}, {NULL}};
const Segment SGI306053[] = {{&S2672[109], 6}, {NULL}};
const Segment SGI306054[] = {{&S2672[84], 3}, {NULL}};
const Segment SGI306055[] = {{&S2673[30], 3}, {NULL}};
const Segment SGI306056[] = {{&S2673[8], 3}, {NULL}};
const Segment SGI306057[] = {{&S2673[13], 10}, {NULL}};
const Segment SGI306058[] = {{&S2673[42], 7}, {NULL}};
const Segment SGI306059[] = {{&S2673[35], 14}, {NULL}};
const Segment SGI306060[] = {{&S2673[51], 6}, {NULL}};
const Segment SGI306061[] = {{&S2674[72], 14}, {NULL}};
const Segment SGI306062[] = {{&S2674[8], 3}, {NULL}};
const Segment SGI306063[] = {{&S2674[13], 10}, {NULL}};
const Segment SGI306064[] = {{&S2674[61], 4}, {NULL}};
const Segment SGI306065[] = {{&S2675[30], 3}, {NULL}};
const Segment SGI306066[] = {{&S2675[8], 3}, {NULL}};
const Segment SGI306067[] = {{&S2675[13], 10}, {NULL}};
const Segment SGI306068[] = {{&S2675[42], 7}, {NULL}};
const Segment SGI306069[] = {{&S2675[35], 14}, {NULL}};
const Segment SGI306070[] = {{&S2675[138], 15}, {NULL}};
const Segment SGI306071[] = {{&S2675[51], 6}, {NULL}};
const Segment SGI306072[] = {{&S2676[30], 3}, {NULL}};
const Segment SGI306073[] = {{&S2676[8], 3}, {NULL}};
const Segment SGI306074[] = {{&S2676[13], 10}, {NULL}};
const Segment SGI306075[] = {{&S2676[42], 7}, {NULL}};
const Segment SGI306076[] = {{&S2676[35], 14}, {NULL}};
const Segment SGI306077[] = {{&S2676[74], 3}, {NULL}};
const Segment SGI306078[] = {{&S2677[30], 3}, {NULL}};
const Segment SGI306079[] = {{&S2677[8], 3}, {NULL}};
const Segment SGI306080[] = {{&S2677[13], 10}, {NULL}};
const Segment SGI306081[] = {{&S2677[42], 7}, {NULL}};
const Segment SGI306082[] = {{&S2677[35], 14}, {NULL}};
const Segment SGI306083[] = {{&S2677[91], 15}, {NULL}};
const Segment SGI306084[] = {{&S2678[30], 3}, {NULL}};
const Segment SGI306085[] = {{&S2678[8], 3}, {NULL}};
const Segment SGI306086[] = {{&S2678[13], 10}, {NULL}};
const Segment SGI306087[] = {{&S2678[42], 7}, {NULL}};
const Segment SGI306088[] = {{&S2678[35], 14}, {NULL}};
const Segment SGI306089[] = {{&S2678[98], 15}, {&S2678[98], 15}, {NULL}};
const Segment SGI306090[] = {{&S2678[51], 6}, {NULL}};
const Segment SGI306091[] = {{&S2679[30], 3}, {NULL}};
const Segment SGI306092[] = {{&S2679[8], 3}, {NULL}};
const Segment SGI306093[] = {{&S2679[13], 10}, {NULL}};
const Segment SGI306094[] = {{&S2679[42], 7}, {NULL}};
const Segment SGI306095[] = {{&S2679[35], 14}, {NULL}};
const Segment SGI306096[] = {{&S2679[109], 6}, {NULL}};
const Segment SGI306097[] = {{&S2679[84], 3}, {NULL}};
const Segment SGI306098[] = {{&S2680[30], 3}, {NULL}};
const Segment SGI306099[] = {{&S2680[8], 3}, {NULL}};
const Segment SGI306100[] = {{&S2680[13], 10}, {NULL}};
const Segment SGI306101[] = {{&S2680[42], 7}, {NULL}};
const Segment SGI306102[] = {{&S2680[35], 14}, {NULL}};
const Segment SGI306103[] = {{&S2680[74], 3}, {NULL}};
const Segment SGI306104[] = {{&S2681[30], 3}, {NULL}};
const Segment SGI306105[] = {{&S2681[8], 3}, {NULL}};
const Segment SGI306106[] = {{&S2681[13], 10}, {NULL}};
const Segment SGI306107[] = {{&S2681[42], 7}, {NULL}};
const Segment SGI306108[] = {{&S2681[35], 14}, {NULL}};
const Segment SGI306109[] = {{&S2681[91], 15}, {NULL}};
const Segment SGI306110[] = {{&S2681[162], 4}, {NULL}};
const Segment SGI306111[] = {{&S2682[30], 3}, {NULL}};
const Segment SGI306112[] = {{&S2682[8], 3}, {NULL}};
const Segment SGI306113[] = {{&S2682[13], 10}, {NULL}};
const Segment SGI306114[] = {{&S2682[55], 7}, {NULL}};
const Segment SGI306115[] = {{&S2682[48], 14}, {NULL}};
const Segment SGI306116[] = {{&S2682[64], 3}, {&S2682[64], 3}, {NULL}};
const Segment SGI306117[] = {{&S2683[30], 3}, {NULL}};
const Segment SGI306118[] = {{&S2683[8], 3}, {NULL}};
const Segment SGI306119[] = {{&S2683[13], 7}, {NULL}};
const Segment SGI306120[] = {{&S2683[42], 7}, {NULL}};
const Segment SGI306121[] = {{&S2683[35], 14}, {NULL}};
const Segment SGI306122[] = {{&S2684[30], 3}, {NULL}};
const Segment SGI306123[] = {{&S2684[8], 3}, {NULL}};
const Segment SGI306124[] = {{&S2684[13], 10}, {NULL}};
const Segment SGI306125[] = {{&S2684[55], 7}, {NULL}};
const Segment SGI306126[] = {{&S2684[48], 14}, {NULL}};
const Segment SGI306127[] = {{&S2684[64], 3}, {&S2684[64], 3}, {NULL}};
const Segment SGI306128[] = {{&S2767[30], 4}, {NULL}};
const Segment SGI306129[] = {{&S2767[8], 3}, {NULL}};
const Segment SGI306130[] = {{&S2767[13], 10}, {NULL}};
const Segment SGI306131[] = {{&S2767[43], 7}, {NULL}};
const Segment SGI306132[] = {{&S2767[36], 14}, {NULL}};
const Segment SGI306133[] = {{&S2767[52], 6}, {NULL}};
const Segment SGI306134[] = {{&S2768[30], 4}, {NULL}};
const Segment SGI306135[] = {{&S2768[8], 3}, {NULL}};
const Segment SGI306136[] = {{&S2768[13], 10}, {NULL}};
const Segment SGI306137[] = {{&S2768[43], 7}, {NULL}};
const Segment SGI306138[] = {{&S2768[36], 14}, {NULL}};
const Segment SGI306139[] = {{&S2768[52], 5}, {NULL}};
const Segment SGI306140[] = {{&S2769[30], 4}, {NULL}};
const Segment SGI306141[] = {{&S2769[8], 3}, {NULL}};
const Segment SGI306142[] = {{&S2769[13], 10}, {NULL}};
const Segment SGI306143[] = {{&S2769[43], 7}, {NULL}};
const Segment SGI306144[] = {{&S2769[36], 14}, {NULL}};
const Segment SGI306145[] = {{&S2770[31], 4}, {NULL}};
const Segment SGI306146[] = {{&S2770[44], 7}, {NULL}};
const Segment SGI306147[] = {{&S2771[30], 4}, {NULL}};
const Segment SGI306148[] = {{&S2771[8], 3}, {NULL}};
const Segment SGI306149[] = {{&S2771[13], 10}, {NULL}};
const Segment SGI306150[] = {{&S2771[43], 7}, {NULL}};
const Segment SGI306151[] = {{&S2771[36], 14}, {NULL}};
const Segment SGI306152[] = {{&S2771[52], 3}, {NULL}};
const Segment SGI306153[] = {{&S2772[30], 4}, {NULL}};
const Segment SGI306154[] = {{&S2772[8], 3}, {NULL}};
const Segment SGI306155[] = {{&S2772[13], 10}, {NULL}};
const Segment SGI306156[] = {{&S2772[43], 7}, {NULL}};
const Segment SGI306157[] = {{&S2773[30], 4}, {NULL}};
const Segment SGI306158[] = {{&S2773[8], 3}, {NULL}};
const Segment SGI306159[] = {{&S2773[13], 10}, {NULL}};
const Segment SGI306160[] = {{&S2773[43], 7}, {NULL}};
const Segment SGI306161[] = {{&S2773[36], 14}, {NULL}};
const Segment SGI306162[] = {{&S2773[132], 15}, {NULL}};
const Segment SGI306163[] = {{&S2774[30], 4}, {NULL}};
const Segment SGI306164[] = {{&S2774[8], 3}, {NULL}};
const Segment SGI306165[] = {{&S2774[13], 10}, {NULL}};
const Segment SGI306166[] = {{&S2774[43], 7}, {NULL}};
const Segment SGI306167[] = {{&S2774[36], 14}, {NULL}};
const Segment SGI306168[] = {{&S2774[73], 4}, {NULL}};
const Segment SGI306169[] = {{&S2775[30], 4}, {NULL}};
const Segment SGI306170[] = {{&S2775[8], 3}, {NULL}};
const Segment SGI306171[] = {{&S2775[13], 10}, {NULL}};
const Segment SGI306172[] = {{&S2775[43], 7}, {NULL}};
const Segment SGI306173[] = {{&S2775[36], 14}, {NULL}};
const Segment SGI306174[] = {{&S2775[139], 15}, {NULL}};
const Segment SGI306175[] = {{&S2775[52], 6}, {NULL}};
const Segment SGI306176[] = {{&S2776[30], 4}, {NULL}};
const Segment SGI306177[] = {{&S2776[8], 3}, {NULL}};
const Segment SGI306178[] = {{&S2776[13], 10}, {NULL}};
const Segment SGI306179[] = {{&S2776[43], 7}, {NULL}};
const Segment SGI306180[] = {{&S2776[36], 14}, {NULL}};
const Segment SGI306181[] = {{&S2776[52], 6}, {NULL}};
const Segment SGI306182[] = {{&S2776[82], 3}, {NULL}};
const Segment SGI306183[] = {{&S2777[30], 4}, {NULL}};
const Segment SGI306184[] = {{&S2777[8], 3}, {NULL}};
const Segment SGI306185[] = {{&S2777[13], 10}, {NULL}};
const Segment SGI306186[] = {{&S2777[43], 7}, {NULL}};
const Segment SGI306187[] = {{&S2777[36], 14}, {NULL}};
const Segment SGI306188[] = {{&S2777[75], 3}, {NULL}};
const Segment SGI306189[] = {{&S2778[30], 4}, {NULL}};
const Segment SGI306190[] = {{&S2778[8], 3}, {NULL}};
const Segment SGI306191[] = {{&S2778[13], 10}, {NULL}};
const Segment SGI306192[] = {{&S2778[43], 7}, {NULL}};
const Segment SGI306193[] = {{&S2778[36], 14}, {NULL}};
const Segment SGI306194[] = {{&S2778[52], 6}, {NULL}};
const Segment SGI306195[] = {{&S2778[82], 3}, {NULL}};
const Segment SGI306196[] = {{&S2779[30], 4}, {NULL}};
const Segment SGI306197[] = {{&S2779[8], 3}, {NULL}};
const Segment SGI306198[] = {{&S2779[13], 10}, {NULL}};
const Segment SGI306199[] = {{&S2779[43], 7}, {NULL}};
const Segment SGI306200[] = {{&S2779[36], 14}, {NULL}};
const Segment SGI306201[] = {{&S2780[30], 4}, {NULL}};
const Segment SGI306202[] = {{&S2780[8], 3}, {NULL}};
const Segment SGI306203[] = {{&S2780[13], 10}, {NULL}};
const Segment SGI306204[] = {{&S2780[43], 7}, {NULL}};
const Segment SGI306205[] = {{&S2780[36], 14}, {NULL}};
const Segment SGI306206[] = {{&S2780[132], 15}, {NULL}};
const Segment SGI306207[] = {{&S2780[153], 9}, {NULL}};
const Segment SGI306208[] = {{&S2781[30], 4}, {NULL}};
const Segment SGI306209[] = {{&S2781[8], 3}, {NULL}};
const Segment SGI306210[] = {{&S2781[13], 10}, {NULL}};
const Segment SGI306211[] = {{&S2781[43], 7}, {NULL}};
const Segment SGI306212[] = {{&S2781[36], 14}, {NULL}};
const Segment SGI306213[] = {{&S2781[52], 6}, {NULL}};
const Segment SGI306214[] = {{&S2782[21], 4}, {NULL}};
const Segment SGI306215[] = {{&S2782[8], 3}, {NULL}};
const Segment SGI306216[] = {{&S2782[11], 1}, {NULL}};
const Segment SGI306217[] = {{&S2782[34], 7}, {NULL}};
const Segment SGI306218[] = {{&S2782[27], 14}, {NULL}};
const Segment SGI306219[] = {{&S2782[43], 6}, {NULL}};
const Segment SGI306220[] = {{&S2783[30], 4}, {NULL}};
const Segment SGI306221[] = {{&S2783[8], 3}, {NULL}};
const Segment SGI306222[] = {{&S2783[13], 10}, {NULL}};
const Segment SGI306223[] = {{&S2783[43], 7}, {NULL}};
const Segment SGI306224[] = {{&S2783[36], 14}, {NULL}};
const Segment SGI306225[] = {{&S2783[52], 6}, {NULL}};
const Segment SGI306226[] = {{&S2783[59], 3}, {NULL}};
const Segment SGI306227[] = {{&S2784[30], 4}, {NULL}};
const Segment SGI306228[] = {{&S2784[8], 3}, {NULL}};
const Segment SGI306229[] = {{&S2784[13], 10}, {NULL}};
const Segment SGI306230[] = {{&S2784[43], 7}, {NULL}};
const Segment SGI306231[] = {{&S2784[36], 14}, {NULL}};
const Segment SGI306232[] = {{&S2784[52], 6}, {NULL}};
const Segment SGI306233[] = {{&S2784[152], 9}, {NULL}};
const Segment SGI306234[] = {{&S2785[30], 4}, {NULL}};
const Segment SGI306235[] = {{&S2785[8], 3}, {NULL}};
const Segment SGI306236[] = {{&S2785[13], 10}, {NULL}};
const Segment SGI306237[] = {{&S2785[43], 2}, {NULL}};
const Segment SGI306238[] = {{&S2786[30], 4}, {NULL}};
const Segment SGI306239[] = {{&S2786[8], 3}, {NULL}};
const Segment SGI306240[] = {{&S2786[13], 10}, {NULL}};
const Segment SGI306241[] = {{&S2786[43], 7}, {NULL}};
const Segment SGI306242[] = {{&S2786[36], 14}, {NULL}};
const Segment SGI306243[] = {{&S2786[157], 15}, {NULL}};
const Segment SGI306244[] = {{&S2786[52], 5}, {NULL}};
const Segment SGI306245[] = {{&S2787[37], 4}, {NULL}};
const Segment SGI306246[] = {{&S2787[8], 3}, {NULL}};
const Segment SGI306247[] = {{&S2787[13], 10}, {NULL}};
const Segment SGI306248[] = {{&S2787[50], 7}, {NULL}};
const Segment SGI306249[] = {{&S2787[25], 6}, {NULL}};
const Segment SGI306250[] = {{&S2788[30], 4}, {NULL}};
const Segment SGI306251[] = {{&S2788[8], 3}, {NULL}};
const Segment SGI306252[] = {{&S2788[13], 10}, {NULL}};
const Segment SGI306253[] = {{&S2788[43], 7}, {NULL}};
const Segment SGI306254[] = {{&S2788[36], 14}, {NULL}};
const Segment SGI306255[] = {{&S2788[162], 4}, {NULL}};
const Segment SGI306256[] = {{&S2788[52], 6}, {NULL}};
const Segment SGI306257[] = {{&S2875[30], 3}, {NULL}};
const Segment SGI306258[] = {{&S2875[8], 3}, {NULL}};
const Segment SGI306259[] = {{&S2875[13], 10}, {NULL}};
const Segment SGI306260[] = {{&S2875[42], 7}, {NULL}};
const Segment SGI306261[] = {{&S2875[35], 14}, {NULL}};
const Segment SGI306262[] = {{&S2875[51], 6}, {NULL}};
const Segment SGI306263[] = {{&S2876[30], 3}, {NULL}};
const Segment SGI306264[] = {{&S2876[8], 3}, {NULL}};
const Segment SGI306265[] = {{&S2876[13], 10}, {NULL}};
const Segment SGI306266[] = {{&S2876[42], 7}, {NULL}};
const Segment SGI306267[] = {{&S2876[35], 14}, {NULL}};
const Segment SGI306268[] = {{&S2877[30], 3}, {NULL}};
const Segment SGI306269[] = {{&S2877[8], 3}, {NULL}};
const Segment SGI306270[] = {{&S2877[13], 10}, {NULL}};
const Segment SGI306271[] = {{&S2877[42], 7}, {NULL}};
const Segment SGI306272[] = {{&S2877[35], 14}, {NULL}};
const Segment SGI306273[] = {{&S2877[84], 15}, {NULL}};
const Segment SGI306274[] = {{&S2877[165], 4}, {NULL}};
const Segment SGI306275[] = {{&S2878[30], 3}, {NULL}};
const Segment SGI306276[] = {{&S2878[8], 3}, {NULL}};
const Segment SGI306277[] = {{&S2878[13], 10}, {NULL}};
const Segment SGI306278[] = {{&S2878[42], 7}, {NULL}};
const Segment SGI306279[] = {{&S2878[35], 14}, {NULL}};
const Segment SGI306280[] = {{&S2878[91], 15}, {NULL}};
const Segment SGI306281[] = {{&S2878[172], 4}, {NULL}};
const Segment SGI306282[] = {{&S2878[51], 6}, {NULL}};
const Segment SGI306283[] = {{&S2879[30], 3}, {NULL}};
const Segment SGI306284[] = {{&S2879[8], 3}, {NULL}};
const Segment SGI306285[] = {{&S2879[13], 10}, {NULL}};
const Segment SGI306286[] = {{&S2879[42], 7}, {NULL}};
const Segment SGI306287[] = {{&S2879[35], 14}, {NULL}};
const Segment SGI306288[] = {{&S2879[51], 5}, {NULL}};
const Segment SGI306289[] = {{&S2880[30], 3}, {NULL}};
const Segment SGI306290[] = {{&S2880[8], 3}, {NULL}};
const Segment SGI306291[] = {{&S2880[13], 10}, {NULL}};
const Segment SGI306292[] = {{&S2880[42], 7}, {NULL}};
const Segment SGI306293[] = {{&S2880[35], 14}, {NULL}};
const Segment SGI306294[] = {{&S2880[144], 4}, {NULL}};
const Segment SGI306295[] = {{&S2881[30], 3}, {NULL}};
const Segment SGI306296[] = {{&S2881[8], 3}, {NULL}};
const Segment SGI306297[] = {{&S2881[13], 10}, {NULL}};
const Segment SGI306298[] = {{&S2881[42], 7}, {NULL}};
const Segment SGI306299[] = {{&S2881[35], 14}, {NULL}};
const Segment SGI306300[] = {{&S2881[151], 4}, {NULL}};
const Segment SGI306301[] = {{&S2881[51], 6}, {NULL}};
const Segment SGI306302[] = {{&S2882[30], 3}, {NULL}};
const Segment SGI306303[] = {{&S2882[8], 3}, {NULL}};
const Segment SGI306304[] = {{&S2882[13], 10}, {NULL}};
const Segment SGI306305[] = {{&S2882[42], 7}, {NULL}};
const Segment SGI306306[] = {{&S2882[35], 14}, {NULL}};
const Segment SGI306307[] = {{&S2882[151], 15}, {NULL}};
const Segment SGI306308[] = {{&S2882[51], 6}, {NULL}};
const Segment SGI306309[] = {{&S2883[30], 3}, {NULL}};
const Segment SGI306310[] = {{&S2883[8], 3}, {NULL}};
const Segment SGI306311[] = {{&S2883[13], 10}, {NULL}};
const Segment SGI306312[] = {{&S2883[42], 7}, {NULL}};
const Segment SGI306313[] = {{&S2883[35], 14}, {NULL}};
const Segment SGI306314[] = {{&S2883[144], 15}, {NULL}};
const Segment SGI306315[] = {{&S2884[30], 3}, {NULL}};
const Segment SGI306316[] = {{&S2884[8], 3}, {NULL}};
const Segment SGI306317[] = {{&S2884[13], 10}, {NULL}};
const Segment SGI306318[] = {{&S2884[42], 7}, {NULL}};
const Segment SGI306319[] = {{&S2884[35], 14}, {NULL}};
const Segment SGI306320[] = {{&S2884[84], 9}, {NULL}};
const Segment SGI306321[] = {{&S2885[30], 3}, {NULL}};
const Segment SGI306322[] = {{&S2885[8], 3}, {NULL}};
const Segment SGI306323[] = {{&S2885[13], 10}, {NULL}};
const Segment SGI306324[] = {{&S2885[42], 7}, {NULL}};
const Segment SGI306325[] = {{&S2885[35], 14}, {NULL}};
const Segment SGI306326[] = {{&S2885[156], 4}, {NULL}};
const Segment SGI306327[] = {{&S2885[51], 5}, {NULL}};
const Segment SGI306328[] = {{&S2886[30], 3}, {NULL}};
const Segment SGI306329[] = {{&S2886[8], 3}, {NULL}};
const Segment SGI306330[] = {{&S2886[13], 10}, {NULL}};
const Segment SGI306331[] = {{&S2886[42], 7}, {&S2886[158], 30}, {NULL}};
const Segment SGI306332[] = {{&S2886[35], 14}, {NULL}};
const Segment SGI306333[] = {{&S2886[51], 6}, {NULL}};
const Segment SGI306334[] = {{&S2887[30], 3}, {NULL}};
const Segment SGI306335[] = {{&S2887[8], 3}, {NULL}};
const Segment SGI306336[] = {{&S2887[13], 10}, {NULL}};
const Segment SGI306337[] = {{&S2887[42], 7}, {NULL}};
const Segment SGI306338[] = {{&S2887[35], 14}, {NULL}};
const Segment SGI306339[] = {{&S2887[156], 15}, {NULL}};
const Segment SGI306340[] = {{&S2887[51], 5}, {NULL}};
const Segment SGI306341[] = {{&S2888[30], 3}, {NULL}};
const Segment SGI306342[] = {{&S2888[8], 3}, {NULL}};
const Segment SGI306343[] = {{&S2888[13], 10}, {NULL}};
const Segment SGI306344[] = {{&S2888[42], 7}, {&S2888[179], 30}, {NULL}};
const Segment SGI306345[] = {{&S2888[35], 14}, {NULL}};
const Segment SGI306346[] = {{&S2888[91], 15}, {NULL}};
const Segment SGI306347[] = {{&S2888[51], 6}, {NULL}};
const Segment SGI306348[] = {{&S2889[30], 3}, {NULL}};
const Segment SGI306349[] = {{&S2889[8], 3}, {NULL}};
const Segment SGI306350[] = {{&S2889[13], 10}, {NULL}};
const Segment SGI306351[] = {{&S2889[42], 7}, {NULL}};
const Segment SGI306352[] = {{&S2889[35], 14}, {NULL}};
const Segment SGI306353[] = {{&S2889[156], 15}, {NULL}};
const Segment SGI306354[] = {{&S2889[51], 5}, {NULL}};
const Segment SGI306355[] = {{&S2889[184], 9}, {NULL}};
const Segment SGI306356[] = {{&S2890[30], 3}, {NULL}};
const Segment SGI306357[] = {{&S2890[8], 3}, {NULL}};
const Segment SGI306358[] = {{&S2890[13], 10}, {NULL}};
const Segment SGI306359[] = {{&S2890[42], 7}, {&S2890[170], 30}, {NULL}};
const Segment SGI306360[] = {{&S2890[35], 14}, {NULL}};
const Segment SGI306361[] = {{&S2890[51], 6}, {NULL}};
const Segment SGI306362[] = {{&S2891[30], 3}, {NULL}};
const Segment SGI306363[] = {{&S2891[8], 3}, {NULL}};
const Segment SGI306364[] = {{&S2891[13], 10}, {NULL}};
const Segment SGI306365[] = {{&S2891[42], 7}, {&S2891[158], 30}, {NULL}};
const Segment SGI306366[] = {{&S2891[35], 14}, {NULL}};
const Segment SGI306367[] = {{&S2891[51], 6}, {NULL}};
const Segment SGI306368[] = {{&S2892[30], 3}, {NULL}};
const Segment SGI306369[] = {{&S2892[8], 3}, {NULL}};
const Segment SGI306370[] = {{&S2892[13], 10}, {NULL}};
const Segment SGI306371[] = {{&S2892[42], 7}, {NULL}};
const Segment SGI306372[] = {{&S2892[35], 14}, {NULL}};
const Segment SGI306373[] = {{&S2892[84], 15}, {NULL}};
const Segment SGI306374[] = {{&S2892[165], 9}, {NULL}};
const Segment SGI306375[] = {{&S2893[30], 3}, {NULL}};
const Segment SGI306376[] = {{&S2893[8], 3}, {NULL}};
const Segment SGI306377[] = {{&S2893[13], 10}, {NULL}};
const Segment SGI306378[] = {{&S2893[42], 7}, {NULL}};
const Segment SGI306379[] = {{&S2893[35], 14}, {NULL}};
const Segment SGI306380[] = {{&S2893[156], 15}, {NULL}};
const Segment SGI306381[] = {{&S2893[184], 4}, {NULL}};
const Segment SGI306382[] = {{&S2893[51], 5}, {NULL}};
const Segment SGI306383[] = {{&S2894[30], 3}, {NULL}};
const Segment SGI306384[] = {{&S2894[8], 3}, {NULL}};
const Segment SGI306385[] = {{&S2894[13], 10}, {NULL}};
const Segment SGI306386[] = {{&S2894[42], 7}, {NULL}};
const Segment SGI306387[] = {{&S2894[35], 14}, {NULL}};
const Segment SGI306388[] = {{&S2894[156], 15}, {NULL}};
const Segment SGI306389[] = {{&S2894[199], 4}, {NULL}};
const Segment SGI306390[] = {{&S2894[51], 5}, {NULL}};
const Segment SGI306391[] = {{&S2894[184], 9}, {NULL}};
const Segment SGI306392[] = {{&S2895[30], 3}, {NULL}};
const Segment SGI306393[] = {{&S2895[8], 3}, {NULL}};
const Segment SGI306394[] = {{&S2895[13], 10}, {NULL}};
const Segment SGI306395[] = {{&S2895[42], 7}, {NULL}};
const Segment SGI306396[] = {{&S2895[35], 14}, {NULL}};
const Segment SGI306397[] = {{&S2895[51], 6}, {NULL}};
const Segment SGI306398[] = {{&S2895[151], 9}, {NULL}};
const Segment SGI306399[] = {{&S2896[30], 3}, {NULL}};
const Segment SGI306400[] = {{&S2896[8], 3}, {NULL}};
const Segment SGI306401[] = {{&S2896[13], 10}, {NULL}};
const Segment SGI306402[] = {{&S2896[42], 7}, {NULL}};
const Segment SGI306403[] = {{&S2896[35], 14}, {NULL}};
const Segment SGI306404[] = {{&S2896[166], 4}, {NULL}};
const Segment SGI306405[] = {{&S2896[51], 6}, {NULL}};
const Segment SGI306406[] = {{&S2896[151], 9}, {NULL}};
const Segment SGI306407[] = {{&S2897[30], 3}, {NULL}};
const Segment SGI306408[] = {{&S2897[8], 3}, {NULL}};
const Segment SGI306409[] = {{&S2897[13], 10}, {NULL}};
const Segment SGI306410[] = {{&S2897[42], 7}, {NULL}};
const Segment SGI306411[] = {{&S2897[35], 14}, {NULL}};
const Segment SGI306412[] = {{&S2897[51], 5}, {NULL}};
const Segment SGI306413[] = {{&S2897[156], 9}, {NULL}};
const Segment SGI306414[] = {{&S2898[30], 3}, {NULL}};
const Segment SGI306415[] = {{&S2898[8], 3}, {NULL}};
const Segment SGI306416[] = {{&S2898[13], 10}, {NULL}};
const Segment SGI306417[] = {{&S2898[42], 7}, {NULL}};
const Segment SGI306418[] = {{&S2898[35], 14}, {NULL}};
const Segment SGI306419[] = {{&S2898[151], 15}, {NULL}};
const Segment SGI306420[] = {{&S2898[51], 6}, {NULL}};
const Segment SGI306421[] = {{&S2898[172], 9}, {NULL}};
const Segment SGI306422[] = {{&S2899[30], 3}, {NULL}};
const Segment SGI306423[] = {{&S2899[8], 3}, {NULL}};
const Segment SGI306424[] = {{&S2899[13], 10}, {NULL}};
const Segment SGI306425[] = {{&S2899[42], 7}, {&S2899[151], 30}, {NULL}};
const Segment SGI306426[] = {{&S2899[35], 14}, {NULL}};
const Segment SGI306427[] = {{&S2900[30], 3}, {NULL}};
const Segment SGI306428[] = {{&S2900[8], 3}, {NULL}};
const Segment SGI306429[] = {{&S2900[13], 10}, {NULL}};
const Segment SGI306430[] = {{&S2900[42], 7}, {&S2900[179], 30}, {NULL}};
const Segment SGI306431[] = {{&S2900[35], 14}, {NULL}};
const Segment SGI306432[] = {{&S2900[151], 15}, {NULL}};
const Segment SGI306433[] = {{&S2900[51], 6}, {NULL}};
const Segment SGI306434[] = {{&S2901[30], 3}, {NULL}};
const Segment SGI306435[] = {{&S2901[8], 3}, {NULL}};
const Segment SGI306436[] = {{&S2901[13], 10}, {NULL}};
const Segment SGI306437[] = {{&S2901[42], 7}, {NULL}};
const Segment SGI306438[] = {{&S2901[35], 14}, {NULL}};
const Segment SGI306439[] = {{&S2901[144], 15}, {&S2901[144], 15}, {NULL}};
const Segment SGI306440[] = {{&S2901[200], 9}, {NULL}};
const Segment SGI306441[] = {{&S2902[30], 3}, {NULL}};
const Segment SGI306442[] = {{&S2902[8], 3}, {NULL}};
const Segment SGI306443[] = {{&S2902[13], 10}, {NULL}};
const Segment SGI306444[] = {{&S2902[42], 7}, {NULL}};
const Segment SGI306445[] = {{&S2902[35], 14}, {NULL}};
const Segment SGI306446[] = {{&S2902[144], 15}, {&S2902[144], 15}, {NULL}};
const Segment SGI306447[] = {{&S2903[30], 3}, {NULL}};
const Segment SGI306448[] = {{&S2903[8], 3}, {NULL}};
const Segment SGI306449[] = {{&S2903[13], 10}, {NULL}};
const Segment SGI306450[] = {{&S2903[42], 7}, {NULL}};
const Segment SGI306451[] = {{&S2903[35], 14}, {NULL}};
const Segment SGI306452[] = {{&S2903[156], 15}, {&S2903[156], 15}, {NULL}};
const Segment SGI306453[] = {{&S2903[51], 5}, {NULL}};
const Segment SGI306454[] = {{&S2903[212], 9}, {NULL}};
const Segment SGI306455[] = {{&S2904[30], 3}, {NULL}};
const Segment SGI306456[] = {{&S2904[8], 3}, {NULL}};
const Segment SGI306457[] = {{&S2904[13], 10}, {NULL}};
const Segment SGI306458[] = {{&S2904[42], 7}, {NULL}};
const Segment SGI306459[] = {{&S2904[35], 14}, {NULL}};
const Segment SGI306460[] = {{&S2904[144], 15}, {NULL}};
const Segment SGI306461[] = {{&S2904[187], 4}, {NULL}};
const Segment SGI306462[] = {{&S2904[172], 9}, {NULL}};
const Segment SGI306463[] = {{&S2905[30], 3}, {NULL}};
const Segment SGI306464[] = {{&S2905[8], 3}, {NULL}};
const Segment SGI306465[] = {{&S2905[13], 10}, {NULL}};
const Segment SGI306466[] = {{&S2905[42], 7}, {NULL}};
const Segment SGI306467[] = {{&S2905[35], 14}, {NULL}};
const Segment SGI306468[] = {{&S2906[30], 3}, {NULL}};
const Segment SGI306469[] = {{&S2906[8], 3}, {NULL}};
const Segment SGI306470[] = {{&S2906[13], 10}, {NULL}};
const Segment SGI306471[] = {{&S2906[42], 7}, {NULL}};
const Segment SGI306472[] = {{&S2906[35], 14}, {NULL}};
const Segment SGI306473[] = {{&S2906[159], 4}, {NULL}};
const Segment SGI306474[] = {{&S2906[84], 9}, {NULL}};
const Segment SGI306475[] = {{&S2907[30], 3}, {NULL}};
const Segment SGI306476[] = {{&S2907[8], 3}, {NULL}};
const Segment SGI306477[] = {{&S2907[13], 10}, {NULL}};
const Segment SGI306478[] = {{&S2907[42], 7}, {NULL}};
const Segment SGI306479[] = {{&S2907[35], 14}, {NULL}};
const Segment SGI306480[] = {{&S2907[171], 4}, {NULL}};
const Segment SGI306481[] = {{&S2907[51], 5}, {NULL}};
const Segment SGI306482[] = {{&S2907[96], 9}, {NULL}};
const Segment SGI306483[] = {{&S2908[30], 3}, {NULL}};
const Segment SGI306484[] = {{&S2908[8], 3}, {NULL}};
const Segment SGI306485[] = {{&S2908[13], 10}, {NULL}};
const Segment SGI306486[] = {{&S2908[42], 7}, {NULL}};
const Segment SGI306487[] = {{&S2908[35], 14}, {NULL}};
const Segment SGI306488[] = {{&S2908[156], 15}, {&S2908[156], 15}, {NULL}};
const Segment SGI306489[] = {{&S2908[51], 5}, {NULL}};
const Segment SGI306490[] = {{&S2909[30], 3}, {NULL}};
const Segment SGI306491[] = {{&S2909[8], 3}, {NULL}};
const Segment SGI306492[] = {{&S2909[13], 10}, {NULL}};
const Segment SGI306493[] = {{&S2909[42], 7}, {NULL}};
const Segment SGI306494[] = {{&S2909[35], 14}, {NULL}};
const Segment SGI306495[] = {{&S2909[51], 6}, {NULL}};
const Segment SGI306496[] = {{&S3099[30], 3}, {NULL}};
const Segment SGI306497[] = {{&S3099[8], 3}, {NULL}};
const Segment SGI306498[] = {{&S3099[13], 10}, {NULL}};
const Segment SGI306499[] = {{&S3099[42], 7}, {NULL}};
const Segment SGI306500[] = {{&S3099[35], 14}, {NULL}};
const Segment SGI306501[] = {{&S3099[64], 11}, {NULL}};
const Segment SGI306502[] = {{&S3099[64], 16}, {NULL}};
const Segment SGI306503[] = {{&S3101[30], 3}, {NULL}};
const Segment SGI306504[] = {{&S3101[8], 3}, {NULL}};
const Segment SGI306505[] = {{&S3101[13], 10}, {NULL}};
const Segment SGI306506[] = {{&S3101[42], 7}, {NULL}};
const Segment SGI306507[] = {{&S3101[35], 14}, {NULL}};
const Segment SGI306508[] = {{&S3101[64], 11}, {NULL}};
const Segment SGI306509[] = {{&S3101[64], 16}, {NULL}};
const Segment SGI306510[] = {{&S3101[90], 9}, {NULL}};
const Segment SGI306511[] = {{&S3102[30], 3}, {NULL}};
const Segment SGI306512[] = {{&S3102[8], 3}, {NULL}};
const Segment SGI306513[] = {{&S3102[13], 10}, {NULL}};
const Segment SGI306514[] = {{&S3102[42], 7}, {NULL}};
const Segment SGI306515[] = {{&S3102[35], 14}, {NULL}};
const Segment SGI306516[] = {{&S3102[72], 11}, {NULL}};
const Segment SGI306517[] = {{&S3102[136], 15}, {NULL}};
const Segment SGI306518[] = {{&S3102[72], 16}, {NULL}};
const Segment SGI306519[] = {{&S3103[30], 3}, {NULL}};
const Segment SGI306520[] = {{&S3103[8], 3}, {NULL}};
const Segment SGI306521[] = {{&S3103[13], 10}, {NULL}};
const Segment SGI306522[] = {{&S3103[42], 7}, {NULL}};
const Segment SGI306523[] = {{&S3103[35], 14}, {NULL}};
const Segment SGI306524[] = {{&S3103[71], 11}, {NULL}};
const Segment SGI306525[] = {{&S3103[51], 6}, {NULL}};
const Segment SGI306526[] = {{&S3103[71], 16}, {NULL}};
const Segment SGI306527[] = {{&S3105[30], 3}, {NULL}};
const Segment SGI306528[] = {{&S3105[8], 3}, {NULL}};
const Segment SGI306529[] = {{&S3105[13], 10}, {NULL}};
const Segment SGI306530[] = {{&S3105[42], 7}, {NULL}};
const Segment SGI306531[] = {{&S3105[35], 14}, {NULL}};
const Segment SGI306532[] = {{&S3105[51], 11}, {NULL}};
const Segment SGI306533[] = {{&S3105[51], 16}, {NULL}};
const Segment SGI306534[] = {{&S3106[30], 3}, {&S3106[85], 3}, {NULL}};
const Segment SGI306535[] = {{&S3106[8], 3}, {&S3106[8], 3}, {NULL}};
const Segment SGI306536[] = {{&S3106[13], 10}, {&S3106[13], 10}, {NULL}};
const Segment SGI306537[] = {{&S3106[42], 7}, {&S3106[42], 7}, {NULL}};
const Segment SGI306538[] = {{&S3106[35], 14}, {&S3106[35], 14}, {NULL}};
const Segment SGI306539[] = {{&S3106[72], 11}, {NULL}};
const Segment SGI306540[] = {{&S3106[72], 16}, {NULL}};
const Segment SGI306541[] = {{&S3106[149], 3}, {NULL}};
const Segment SGI306542[] = {{&S3107[30], 3}, {NULL}};
const Segment SGI306543[] = {{&S3107[8], 3}, {NULL}};
const Segment SGI306544[] = {{&S3107[13], 10}, {NULL}};
const Segment SGI306545[] = {{&S3107[42], 7}, {NULL}};
const Segment SGI306546[] = {{&S3107[35], 14}, {NULL}};
const Segment SGI306547[] = {{&S3107[51], 11}, {NULL}};
const Segment SGI306548[] = {{&S3107[51], 16}, {NULL}};
const Segment SGI306549[] = {{&S3108[30], 3}, {NULL}};
const Segment SGI306550[] = {{&S3108[8], 3}, {NULL}};
const Segment SGI306551[] = {{&S3108[13], 10}, {NULL}};
const Segment SGI306552[] = {{&S3108[42], 7}, {NULL}};
const Segment SGI306553[] = {{&S3108[35], 14}, {NULL}};
const Segment SGI306554[] = {{&S3108[56], 11}, {NULL}};
const Segment SGI306555[] = {{&S3108[120], 15}, {NULL}};
const Segment SGI306556[] = {{&S3108[56], 16}, {NULL}};
const Segment SGI306557[] = {{&S3108[51], 3}, {NULL}};
const Segment SGI306558[] = {{&S3110[30], 3}, {NULL}};
const Segment SGI306559[] = {{&S3110[8], 3}, {NULL}};
const Segment SGI306560[] = {{&S3110[13], 10}, {NULL}};
const Segment SGI306561[] = {{&S3110[42], 7}, {NULL}};
const Segment SGI306562[] = {{&S3110[35], 14}, {NULL}};
const Segment SGI306563[] = {{&S3110[64], 11}, {NULL}};
const Segment SGI306564[] = {{&S3110[64], 16}, {NULL}};
const Segment SGI306565[] = {{&S3111[30], 3}, {NULL}};
const Segment SGI306566[] = {{&S3111[8], 3}, {NULL}};
const Segment SGI306567[] = {{&S3111[13], 10}, {NULL}};
const Segment SGI306568[] = {{&S3111[42], 7}, {NULL}};
const Segment SGI306569[] = {{&S3111[35], 14}, {NULL}};
const Segment SGI306570[] = {{&S3111[51], 11}, {NULL}};
const Segment SGI306571[] = {{&S3111[51], 16}, {NULL}};
const Segment SGI306572[] = {{&S3112[30], 3}, {NULL}};
const Segment SGI306573[] = {{&S3112[8], 3}, {NULL}};
const Segment SGI306574[] = {{&S3112[13], 10}, {NULL}};
const Segment SGI306575[] = {{&S3112[42], 7}, {NULL}};
const Segment SGI306576[] = {{&S3112[35], 14}, {NULL}};
const Segment SGI306577[] = {{&S3112[56], 11}, {NULL}};
const Segment SGI306578[] = {{&S3112[56], 16}, {NULL}};
const Segment SGI306579[] = {{&S3112[51], 3}, {NULL}};
const Segment SGI306580[] = {{&S3113[30], 3}, {NULL}};
const Segment SGI306581[] = {{&S3113[8], 3}, {NULL}};
const Segment SGI306582[] = {{&S3113[13], 10}, {NULL}};
const Segment SGI306583[] = {{&S3113[42], 7}, {NULL}};
const Segment SGI306584[] = {{&S3113[35], 14}, {NULL}};
const Segment SGI306585[] = {{&S3113[59], 11}, {NULL}};
const Segment SGI306586[] = {{&S3113[123], 15}, {NULL}};
const Segment SGI306587[] = {{&S3113[59], 16}, {NULL}};
const Segment SGI306588[] = {{&S3114[30], 3}, {NULL}};
const Segment SGI306589[] = {{&S3114[223], 9}, {NULL}};
const Segment SGI306590[] = {{&S3114[8], 3}, {NULL}};
const Segment SGI306591[] = {{&S3114[13], 10}, {NULL}};
const Segment SGI306592[] = {{&S3114[42], 7}, {NULL}};
const Segment SGI306593[] = {{&S3114[35], 14}, {NULL}};
const Segment SGI306594[] = {{&S3114[64], 11}, {NULL}};
const Segment SGI306595[] = {{&S3114[64], 16}, {NULL}};
const Segment SGI306596[] = {{&S3115[30], 3}, {NULL}};
const Segment SGI306597[] = {{&S3115[8], 3}, {NULL}};
const Segment SGI306598[] = {{&S3115[13], 10}, {NULL}};
const Segment SGI306599[] = {{&S3115[42], 7}, {NULL}};
const Segment SGI306600[] = {{&S3115[35], 14}, {NULL}};
const Segment SGI306601[] = {{&S3115[64], 11}, {NULL}};
const Segment SGI306602[] = {{&S3115[252], 4}, {NULL}};
const Segment SGI306603[] = {{&S3115[64], 16}, {NULL}};
const Segment SGI306604[] = {{&S3116[30], 3}, {&S3116[85], 3}, {NULL}};
const Segment SGI306605[] = {{&S3116[8], 3}, {&S3116[8], 3}, {NULL}};
const Segment SGI306606[] = {{&S3116[13], 10}, {&S3116[13], 10}, {NULL}};
const Segment SGI306607[] = {{&S3116[42], 7}, {&S3116[42], 7}, {NULL}};
const Segment SGI306608[] = {{&S3116[35], 14}, {&S3116[35], 14}, {NULL}};
const Segment SGI306609[] = {{&S3116[72], 11}, {NULL}};
const Segment SGI306610[] = {{&S3116[194], 15}, {NULL}};
const Segment SGI306611[] = {{&S3116[72], 16}, {NULL}};
const Segment SGI306612[] = {{&S3116[149], 3}, {NULL}};
const Segment SGI306613[] = {{&S3117[30], 3}, {NULL}};
const Segment SGI306614[] = {{&S3117[8], 3}, {NULL}};
const Segment SGI306615[] = {{&S3117[13], 10}, {NULL}};
const Segment SGI306616[] = {{&S3117[42], 7}, {NULL}};
const Segment SGI306617[] = {{&S3117[35], 14}, {NULL}};
const Segment SGI306618[] = {{&S3117[58], 11}, {NULL}};
const Segment SGI306619[] = {{&S3117[51], 6}, {NULL}};
const Segment SGI306620[] = {{&S3117[58], 16}, {NULL}};
const Segment SGI306621[] = {{&S3118[30], 3}, {NULL}};
const Segment SGI306622[] = {{&S3118[8], 3}, {NULL}};
const Segment SGI306623[] = {{&S3118[13], 10}, {NULL}};
const Segment SGI306624[] = {{&S3118[42], 7}, {NULL}};
const Segment SGI306625[] = {{&S3118[35], 14}, {NULL}};
const Segment SGI306626[] = {{&S3118[56], 11}, {NULL}};
const Segment SGI306627[] = {{&S3118[56], 16}, {NULL}};
const Segment SGI306628[] = {{&S3118[51], 3}, {NULL}};
const Segment SGI306629[] = {{&S3119[30], 3}, {&S3119[85], 3}, {NULL}};
const Segment SGI306630[] = {{&S3119[8], 3}, {&S3119[8], 3}, {NULL}};
const Segment SGI306631[] = {{&S3119[13], 10}, {&S3119[13], 10}, {NULL}};
const Segment SGI306632[] = {{&S3119[42], 7}, {&S3119[42], 7}, {NULL}};
const Segment SGI306633[] = {{&S3119[35], 14}, {&S3119[35], 14}, {NULL}};
const Segment SGI306634[] = {{&S3119[72], 11}, {NULL}};
const Segment SGI306635[] = {{&S3119[72], 16}, {NULL}};
const Segment SGI306636[] = {{&S3119[149], 3}, {NULL}};
const Segment SGI306637[] = {{&S3120[30], 3}, {NULL}};
const Segment SGI306638[] = {{&S3120[8], 3}, {NULL}};
const Segment SGI306639[] = {{&S3120[13], 10}, {NULL}};
const Segment SGI306640[] = {{&S3120[42], 7}, {NULL}};
const Segment SGI306641[] = {{&S3120[35], 14}, {NULL}};
const Segment SGI306642[] = {{&S3120[51], 11}, {NULL}};
const Segment SGI306643[] = {{&S3120[51], 16}, {NULL}};
const Segment SGI306644[] = {{&S3120[191], 9}, {NULL}};
const Segment SGI306645[] = {{&S3121[30], 3}, {NULL}};
const Segment SGI306646[] = {{&S3121[8], 3}, {NULL}};
const Segment SGI306647[] = {{&S3121[13], 10}, {NULL}};
const Segment SGI306648[] = {{&S3121[42], 7}, {NULL}};
const Segment SGI306649[] = {{&S3121[35], 14}, {NULL}};
const Segment SGI306650[] = {{&S3121[69], 11}, {NULL}};
const Segment SGI306651[] = {{&S3121[69], 16}, {NULL}};
const Segment SGI306652[] = {{&S3121[64], 3}, {NULL}};
const Segment SGI306653[] = {{&S3123[16], 3}, {NULL}};
const Segment SGI306654[] = {{&S3123[38], 7}, {NULL}};
const Segment SGI306655[] = {{&S3123[31], 14}, {NULL}};
const Segment SGI306656[] = {{&S3123[134], 15}, {NULL}};
const Segment SGI306657[] = {{&S3123[47], 6}, {NULL}};
const Segment SGI306658[] = {{&S3124[16], 3}, {NULL}};
const Segment SGI306659[] = {{&S3124[38], 7}, {NULL}};
const Segment SGI306660[] = {{&S3124[31], 14}, {NULL}};
const Segment SGI306661[] = {{&S3124[127], 15}, {NULL}};
const Segment SGI306662[] = {{&S3125[16], 3}, {&S3125[86], 3}, {NULL}};
const Segment SGI306663[] = {{&S3125[62], 3}, {NULL}};
const Segment SGI306664[] = {{&S3125[38], 7}, {&S3125[98], 7}, {NULL}};
const Segment SGI306665[] = {{&S3125[31], 14}, {&S3125[91], 14}, {NULL}};
const Segment SGI306666[] = {{&S3125[181], 15}, {NULL}};
const Segment SGI306667[] = {{&S3125[47], 6}, {NULL}};
const Segment SGI306668[] = {{&S3125[107], 3}, {NULL}};
const Segment SGI306669[] = {{&S3126[16], 3}, {NULL}};
const Segment SGI306670[] = {{&S3126[38], 7}, {NULL}};
const Segment SGI306671[] = {{&S3126[31], 14}, {NULL}};
const Segment SGI306672[] = {{&S3126[127], 15}, {NULL}};
const Segment SGI306673[] = {{&S3126[158], 9}, {NULL}};
const Segment SGI306674[] = {{&S3127[16], 3}, {NULL}};
const Segment SGI306675[] = {{&S3127[38], 7}, {NULL}};
const Segment SGI306676[] = {{&S3127[31], 14}, {NULL}};
const Segment SGI306677[] = {{&S3128[16], 3}, {NULL}};
const Segment SGI306678[] = {{&S3128[38], 7}, {NULL}};
const Segment SGI306679[] = {{&S3128[31], 14}, {NULL}};
const Segment SGI306680[] = {{&S3128[134], 15}, {NULL}};
const Segment SGI306681[] = {{&S3128[165], 4}, {NULL}};
const Segment SGI306682[] = {{&S3128[47], 6}, {NULL}};
const Segment SGI306683[] = {{&S3129[16], 3}, {NULL}};
const Segment SGI306684[] = {{&S3129[38], 7}, {NULL}};
const Segment SGI306685[] = {{&S3129[31], 14}, {NULL}};
const Segment SGI306686[] = {{&S3129[127], 15}, {&S3129[127], 15}, {NULL}};
const Segment SGI306687[] = {{&S3130[16], 3}, {NULL}};
const Segment SGI306688[] = {{&S3130[38], 7}, {NULL}};
const Segment SGI306689[] = {{&S3130[31], 14}, {NULL}};
const Segment SGI306690[] = {{&S3130[127], 15}, {&S3130[127], 15}, {NULL}};
const Segment SGI306691[] = {{&S3130[186], 9}, {NULL}};
const Segment SGI306692[] = {{&S3131[16], 3}, {NULL}};
const Segment SGI306693[] = {{&S3131[38], 7}, {NULL}};
const Segment SGI306694[] = {{&S3131[31], 14}, {NULL}};
const Segment SGI306695[] = {{&S3131[47], 6}, {NULL}};
const Segment SGI306696[] = {{&S3132[16], 3}, {NULL}};
const Segment SGI306697[] = {{&S3132[38], 7}, {NULL}};
const Segment SGI306698[] = {{&S3132[31], 14}, {NULL}};
const Segment SGI306699[] = {{&S3132[127], 15}, {NULL}};
const Segment SGI306700[] = {{&S3132[158], 4}, {NULL}};
const Segment SGI306701[] = {{&S3133[16], 3}, {&S3133[79], 3}, {NULL}};
const Segment SGI306702[] = {{&S3133[55], 3}, {NULL}};
const Segment SGI306703[] = {{&S3133[38], 7}, {&S3133[91], 7}, {NULL}};
const Segment SGI306704[] = {{&S3133[31], 14}, {&S3133[84], 14}, {NULL}};
const Segment SGI306705[] = {{&S3133[174], 15}, {NULL}};
const Segment SGI306706[] = {{&S3133[100], 3}, {NULL}};
const Segment SGI306707[] = {{&S3134[16], 3}, {&S3134[79], 3}, {NULL}};
const Segment SGI306708[] = {{&S3134[55], 3}, {NULL}};
const Segment SGI306709[] = {{&S3134[38], 7}, {&S3134[91], 7}, {NULL}};
const Segment SGI306710[] = {{&S3134[31], 14}, {&S3134[84], 14}, {NULL}};
const Segment SGI306711[] = {{&S3134[174], 15}, {NULL}};
const Segment SGI306712[] = {{&S3134[205], 9}, {NULL}};
const Segment SGI306713[] = {{&S3134[100], 3}, {NULL}};
const Segment SGI306714[] = {{&S3135[16], 3}, {NULL}};
const Segment SGI306715[] = {{&S3135[38], 7}, {NULL}};
const Segment SGI306716[] = {{&S3135[31], 14}, {NULL}};
const Segment SGI306717[] = {{&S3135[137], 9}, {NULL}};
const Segment SGI306718[] = {{&S3136[16], 3}, {NULL}};
const Segment SGI306719[] = {{&S3136[38], 7}, {NULL}};
const Segment SGI306720[] = {{&S3136[31], 14}, {NULL}};
const Segment SGI306721[] = {{&S3136[134], 15}, {NULL}};
const Segment SGI306722[] = {{&S3136[47], 6}, {NULL}};
const Segment SGI306723[] = {{&S3136[165], 13}, {NULL}};
const Segment SGI306724[] = {{&S3137[16], 3}, {NULL}};
const Segment SGI306725[] = {{&S3137[38], 7}, {NULL}};
const Segment SGI306726[] = {{&S3137[31], 14}, {NULL}};
const Segment SGI306727[] = {{&S3137[127], 15}, {NULL}};
const Segment SGI306728[] = {{&S3137[173], 4}, {NULL}};
const Segment SGI306729[] = {{&S3137[158], 9}, {NULL}};
const Segment SGI306730[] = {{&S3138[16], 3}, {NULL}};
const Segment SGI306731[] = {{&S3138[38], 7}, {NULL}};
const Segment SGI306732[] = {{&S3138[31], 14}, {NULL}};
const Segment SGI306733[] = {{&S3138[137], 4}, {NULL}};
const Segment SGI306734[] = {{&S3139[26], 3}, {NULL}};
const Segment SGI306735[] = {{&S3139[38], 7}, {NULL}};
const Segment SGI306736[] = {{&S3139[31], 14}, {NULL}};
const Segment SGI306737[] = {{&S3139[134], 15}, {NULL}};
const Segment SGI306738[] = {{&S3139[47], 6}, {NULL}};
const Segment SGI306739[] = {{&S3140[16], 3}, {NULL}};
const Segment SGI306740[] = {{&S3140[38], 7}, {NULL}};
const Segment SGI306741[] = {{&S3140[31], 14}, {NULL}};
const Segment SGI306742[] = {{&S3140[144], 4}, {NULL}};
const Segment SGI306743[] = {{&S3140[47], 6}, {NULL}};
const Segment SGI306744[] = {{&S3141[16], 3}, {NULL}};
const Segment SGI306745[] = {{&S3141[38], 7}, {NULL}};
const Segment SGI306746[] = {{&S3141[31], 14}, {NULL}};
const Segment SGI306747[] = {{&S3141[127], 15}, {NULL}};
const Segment SGI306748[] = {{&S3141[158], 13}, {NULL}};
const Segment SGI306749[] = {{&S3142[26], 3}, {NULL}};
const Segment SGI306750[] = {{&S3142[38], 7}, {NULL}};
const Segment SGI306751[] = {{&S3142[31], 14}, {NULL}};
const Segment SGI306752[] = {{&S3142[127], 15}, {NULL}};
const Segment SGI306753[] = {{&S3143[26], 4}, {NULL}};
const Segment SGI306754[] = {{&S3143[39], 7}, {NULL}};
const Segment SGI306755[] = {{&S3143[32], 14}, {NULL}};
const Segment SGI306756[] = {{&S3143[135], 15}, {NULL}};
const Segment SGI306757[] = {{&S3143[48], 6}, {NULL}};
const Segment SGI306758[] = {{&S3144[26], 4}, {NULL}};
const Segment SGI306759[] = {{&S3144[39], 7}, {NULL}};
const Segment SGI306760[] = {{&S3144[32], 14}, {NULL}};
const Segment SGI306761[] = {{&S3144[128], 15}, {NULL}};
const Segment SGI306762[] = {{&S3155[30], 3}, {NULL}};
const Segment SGI306763[] = {{&S3155[8], 3}, {NULL}};
const Segment SGI306764[] = {{&S3155[13], 10}, {NULL}};
const Segment SGI306765[] = {{&S3155[42], 7}, {NULL}};
const Segment SGI306766[] = {{&S3155[35], 14}, {NULL}};
const Segment SGI306767[] = {{&S3155[64], 11}, {NULL}};
const Segment SGI306768[] = {{&S3155[64], 16}, {NULL}};
const Segment SGI306769[] = {{&S3158[30], 3}, {NULL}};
const Segment SGI306770[] = {{&S3158[8], 3}, {NULL}};
const Segment SGI306771[] = {{&S3158[13], 10}, {NULL}};
const Segment SGI306772[] = {{&S3158[42], 7}, {NULL}};
const Segment SGI306773[] = {{&S3158[35], 14}, {NULL}};
const Segment SGI306774[] = {{&S3158[72], 11}, {NULL}};
const Segment SGI306775[] = {{&S3158[137], 15}, {NULL}};
const Segment SGI306776[] = {{&S3158[72], 16}, {NULL}};
const Segment SGI306777[] = {{&S3160[30], 3}, {NULL}};
const Segment SGI306778[] = {{&S3160[8], 3}, {NULL}};
const Segment SGI306779[] = {{&S3160[13], 10}, {NULL}};
const Segment SGI306780[] = {{&S3160[42], 7}, {NULL}};
const Segment SGI306781[] = {{&S3160[35], 14}, {NULL}};
const Segment SGI306782[] = {{&S3160[64], 11}, {NULL}};
const Segment SGI306783[] = {{&S3160[253], 4}, {NULL}};
const Segment SGI306784[] = {{&S3160[64], 16}, {NULL}};
const Segment SGI306785[] = {{&S3161[30], 3}, {NULL}};
const Segment SGI306786[] = {{&S3161[8], 3}, {NULL}};
const Segment SGI306787[] = {{&S3161[13], 10}, {NULL}};
const Segment SGI306788[] = {{&S3161[42], 7}, {NULL}};
const Segment SGI306789[] = {{&S3161[35], 14}, {NULL}};
const Segment SGI306790[] = {{&S3161[51], 11}, {NULL}};
const Segment SGI306791[] = {{&S3161[51], 16}, {NULL}};
const Segment SGI306792[] = {{&S3163[30], 3}, {NULL}};
const Segment SGI306793[] = {{&S3163[8], 3}, {NULL}};
const Segment SGI306794[] = {{&S3163[13], 10}, {NULL}};
const Segment SGI306795[] = {{&S3163[42], 7}, {NULL}};
const Segment SGI306796[] = {{&S3163[35], 14}, {NULL}};
const Segment SGI306797[] = {{&S3163[56], 11}, {NULL}};
const Segment SGI306798[] = {{&S3163[56], 16}, {NULL}};
const Segment SGI306799[] = {{&S3163[51], 3}, {NULL}};
const Segment SGI306800[] = {{&S3164[30], 3}, {NULL}};
const Segment SGI306801[] = {{&S3164[8], 3}, {NULL}};
const Segment SGI306802[] = {{&S3164[13], 10}, {NULL}};
const Segment SGI306803[] = {{&S3164[42], 7}, {NULL}};
const Segment SGI306804[] = {{&S3164[35], 14}, {NULL}};
const Segment SGI306805[] = {{&S3164[64], 11}, {NULL}};
const Segment SGI306806[] = {{&S3164[64], 16}, {NULL}};
const Segment SGI306807[] = {{&S3165[30], 3}, {NULL}};
const Segment SGI306808[] = {{&S3165[8], 3}, {NULL}};
const Segment SGI306809[] = {{&S3165[13], 10}, {NULL}};
const Segment SGI306810[] = {{&S3165[42], 7}, {NULL}};
const Segment SGI306811[] = {{&S3165[35], 14}, {NULL}};
const Segment SGI306812[] = {{&S3165[64], 11}, {NULL}};
const Segment SGI306813[] = {{&S3165[64], 16}, {NULL}};
const Segment SGI306814[] = {{&S3165[91], 9}, {NULL}};
const Segment SGI306815[] = {{&S3167[30], 3}, {&S3167[121], 3}, {NULL}};
const Segment SGI306816[] = {{&S3167[8], 3}, {&S3167[8], 3}, {NULL}};
const Segment SGI306817[] = {{&S3167[13], 10}, {&S3167[13], 10}, {NULL}};
const Segment SGI306818[] = {{&S3167[42], 7}, {&S3167[42], 7}, {NULL}};
const Segment SGI306819[] = {{&S3167[35], 14}, {&S3167[35], 14}, {NULL}};
const Segment SGI306820[] = {{&S3167[64], 11}, {NULL}};
const Segment SGI306821[] = {{&S3167[64], 16}, {NULL}};
const Segment SGI306822[] = {{&S3167[142], 3}, {NULL}};
const Segment SGI306823[] = {{&S3168[30], 3}, {NULL}};
const Segment SGI306824[] = {{&S3168[8], 3}, {NULL}};
const Segment SGI306825[] = {{&S3168[13], 10}, {NULL}};
const Segment SGI306826[] = {{&S3168[42], 7}, {NULL}};
const Segment SGI306827[] = {{&S3168[35], 14}, {NULL}};
const Segment SGI306828[] = {{&S3168[71], 11}, {NULL}};
const Segment SGI306829[] = {{&S3168[51], 6}, {NULL}};
const Segment SGI306830[] = {{&S3168[71], 16}, {NULL}};
const Segment SGI306831[] = {{&S3170[30], 3}, {NULL}};
const Segment SGI306832[] = {{&S3170[8], 3}, {NULL}};
const Segment SGI306833[] = {{&S3170[13], 10}, {NULL}};
const Segment SGI306834[] = {{&S3170[42], 7}, {NULL}};
const Segment SGI306835[] = {{&S3170[35], 14}, {NULL}};
const Segment SGI306836[] = {{&S3170[56], 11}, {NULL}};
const Segment SGI306837[] = {{&S3170[121], 15}, {NULL}};
const Segment SGI306838[] = {{&S3170[56], 16}, {NULL}};
const Segment SGI306839[] = {{&S3170[51], 3}, {NULL}};
const Segment SGI306840[] = {{&S3171[30], 3}, {NULL}};
const Segment SGI306841[] = {{&S3171[8], 3}, {NULL}};
const Segment SGI306842[] = {{&S3171[13], 10}, {NULL}};
const Segment SGI306843[] = {{&S3171[42], 7}, {NULL}};
const Segment SGI306844[] = {{&S3171[35], 14}, {NULL}};
const Segment SGI306845[] = {{&S3171[51], 11}, {NULL}};
const Segment SGI306846[] = {{&S3171[116], 15}, {NULL}};
const Segment SGI306847[] = {{&S3171[51], 16}, {NULL}};
const Segment SGI306848[] = {{&S3172[30], 3}, {NULL}};
const Segment SGI306849[] = {{&S3172[8], 3}, {NULL}};
const Segment SGI306850[] = {{&S3172[13], 10}, {NULL}};
const Segment SGI306851[] = {{&S3172[42], 7}, {NULL}};
const Segment SGI306852[] = {{&S3172[35], 14}, {NULL}};
const Segment SGI306853[] = {{&S3172[51], 11}, {NULL}};
const Segment SGI306854[] = {{&S3172[51], 16}, {NULL}};
const Segment SGI306855[] = {{&S3173[30], 3}, {NULL}};
const Segment SGI306856[] = {{&S3173[8], 3}, {NULL}};
const Segment SGI306857[] = {{&S3173[13], 10}, {NULL}};
const Segment SGI306858[] = {{&S3173[42], 7}, {NULL}};
const Segment SGI306859[] = {{&S3173[35], 14}, {NULL}};
const Segment SGI306860[] = {{&S3173[51], 11}, {NULL}};
const Segment SGI306861[] = {{&S3173[51], 16}, {NULL}};
const Segment SGI306862[] = {{&S3174[30], 3}, {NULL}};
const Segment SGI306863[] = {{&S3174[8], 3}, {NULL}};
const Segment SGI306864[] = {{&S3174[13], 10}, {NULL}};
const Segment SGI306865[] = {{&S3174[42], 7}, {NULL}};
const Segment SGI306866[] = {{&S3174[35], 14}, {NULL}};
const Segment SGI306867[] = {{&S3174[51], 11}, {NULL}};
const Segment SGI306868[] = {{&S3174[51], 16}, {NULL}};
const Segment SGI306869[] = {{&S3174[78], 9}, {NULL}};
const Segment SGI306870[] = {{&S3175[30], 3}, {NULL}};
const Segment SGI306871[] = {{&S3175[224], 9}, {NULL}};
const Segment SGI306872[] = {{&S3175[8], 3}, {NULL}};
const Segment SGI306873[] = {{&S3175[13], 10}, {NULL}};
const Segment SGI306874[] = {{&S3175[42], 7}, {NULL}};
const Segment SGI306875[] = {{&S3175[35], 14}, {NULL}};
const Segment SGI306876[] = {{&S3175[64], 11}, {NULL}};
const Segment SGI306877[] = {{&S3175[64], 16}, {NULL}};
const Segment SGI306878[] = {{&S4157[30], 3}, {NULL}};
const Segment SGI306879[] = {{&S4157[8], 3}, {NULL}};
const Segment SGI306880[] = {{&S4157[13], 10}, {NULL}};
const Segment SGI306881[] = {{&S4157[42], 7}, {NULL}};
const Segment SGI306882[] = {{&S4157[35], 14}, {NULL}};
const Segment SGI306883[] = {{&S4157[51], 11}, {NULL}};
const Segment SGI306884[] = {{&S4157[51], 16}, {NULL}};
const Segment SGI306885[] = {{&S4157[77], 9}, {NULL}};
const Segment SGI306886[] = {{&S4161[30], 3}, {NULL}};
const Segment SGI306887[] = {{&S4161[8], 3}, {NULL}};
const Segment SGI306888[] = {{&S4161[13], 10}, {NULL}};
const Segment SGI306889[] = {{&S4161[42], 7}, {NULL}};
const Segment SGI306890[] = {{&S4161[35], 14}, {NULL}};
const Segment SGI306891[] = {{&S4161[64], 11}, {NULL}};
const Segment SGI306892[] = {{&S4161[64], 16}, {NULL}};
const Segment SGI306893[] = {{&S4162[30], 3}, {NULL}};
const Segment SGI306894[] = {{&S4162[8], 3}, {NULL}};
const Segment SGI306895[] = {{&S4162[13], 10}, {NULL}};
const Segment SGI306896[] = {{&S4162[42], 7}, {NULL}};
const Segment SGI306897[] = {{&S4162[35], 14}, {NULL}};
const Segment SGI306898[] = {{&S4162[51], 11}, {NULL}};
const Segment SGI306899[] = {{&S4162[51], 16}, {NULL}};
const Segment SGI306900[] = {{&S4163[30], 3}, {NULL}};
const Segment SGI306901[] = {{&S4163[8], 3}, {NULL}};
const Segment SGI306902[] = {{&S4163[13], 10}, {NULL}};
const Segment SGI306903[] = {{&S4163[42], 7}, {NULL}};
const Segment SGI306904[] = {{&S4163[35], 14}, {NULL}};
const Segment SGI306905[] = {{&S4163[51], 11}, {NULL}};
const Segment SGI306906[] = {{&S4163[51], 16}, {NULL}};
const Segment SGI306907[] = {{&S4165[30], 3}, {NULL}};
const Segment SGI306908[] = {{&S4165[8], 3}, {NULL}};
const Segment SGI306909[] = {{&S4165[13], 10}, {NULL}};
const Segment SGI306910[] = {{&S4165[42], 7}, {NULL}};
const Segment SGI306911[] = {{&S4165[35], 14}, {NULL}};
const Segment SGI306912[] = {{&S4165[64], 11}, {NULL}};
const Segment SGI306913[] = {{&S4165[252], 4}, {NULL}};
const Segment SGI306914[] = {{&S4165[64], 16}, {NULL}};
const Segment SGI306915[] = {{&S4166[30], 3}, {NULL}};
const Segment SGI306916[] = {{&S4166[8], 3}, {NULL}};
const Segment SGI306917[] = {{&S4166[13], 10}, {NULL}};
const Segment SGI306918[] = {{&S4166[42], 7}, {NULL}};
const Segment SGI306919[] = {{&S4166[35], 14}, {NULL}};
const Segment SGI306920[] = {{&S4166[64], 11}, {NULL}};
const Segment SGI306921[] = {{&S4166[128], 15}, {NULL}};
const Segment SGI306922[] = {{&S4166[64], 16}, {NULL}};
const Segment SGI306923[] = {{&S4167[30], 3}, {NULL}};
const Segment SGI306924[] = {{&S4167[8], 3}, {NULL}};
const Segment SGI306925[] = {{&S4167[13], 10}, {NULL}};
const Segment SGI306926[] = {{&S4167[42], 7}, {NULL}};
const Segment SGI306927[] = {{&S4167[35], 14}, {NULL}};
const Segment SGI306928[] = {{&S4167[64], 11}, {NULL}};
const Segment SGI306929[] = {{&S4167[64], 16}, {NULL}};
const Segment SGI306930[] = {{&S4168[30], 3}, {NULL}};
const Segment SGI306931[] = {{&S4168[8], 3}, {NULL}};
const Segment SGI306932[] = {{&S4168[13], 10}, {NULL}};
const Segment SGI306933[] = {{&S4168[42], 7}, {NULL}};
const Segment SGI306934[] = {{&S4168[35], 14}, {NULL}};
const Segment SGI306935[] = {{&S4168[72], 11}, {NULL}};
const Segment SGI306936[] = {{&S4168[136], 15}, {NULL}};
const Segment SGI306937[] = {{&S4168[72], 16}, {NULL}};
const Segment SGI306938[] = {{&S4169[30], 3}, {NULL}};
const Segment SGI306939[] = {{&S4169[8], 3}, {NULL}};
const Segment SGI306940[] = {{&S4169[13], 10}, {NULL}};
const Segment SGI306941[] = {{&S4169[42], 7}, {NULL}};
const Segment SGI306942[] = {{&S4169[35], 14}, {NULL}};
const Segment SGI306943[] = {{&S4169[71], 11}, {NULL}};
const Segment SGI306944[] = {{&S4169[51], 6}, {NULL}};
const Segment SGI306945[] = {{&S4169[71], 16}, {NULL}};
const Segment SGI306946[] = {{&S4170[30], 3}, {NULL}};
const Segment SGI306947[] = {{&S4170[8], 3}, {NULL}};
const Segment SGI306948[] = {{&S4170[13], 10}, {NULL}};
const Segment SGI306949[] = {{&S4170[42], 7}, {NULL}};
const Segment SGI306950[] = {{&S4170[35], 14}, {NULL}};
const Segment SGI306951[] = {{&S4170[56], 11}, {NULL}};
const Segment SGI306952[] = {{&S4170[56], 16}, {NULL}};
const Segment SGI306953[] = {{&S4170[51], 3}, {NULL}};
const Segment SGI306954[] = {{&S4171[30], 3}, {NULL}};
const Segment SGI306955[] = {{&S4171[8], 3}, {NULL}};
const Segment SGI306956[] = {{&S4171[13], 10}, {NULL}};
const Segment SGI306957[] = {{&S4171[42], 7}, {NULL}};
const Segment SGI306958[] = {{&S4171[35], 14}, {NULL}};
const Segment SGI306959[] = {{&S4171[51], 11}, {NULL}};
const Segment SGI306960[] = {{&S4171[51], 16}, {NULL}};
const Segment SGI306961[] = {{&S4172[30], 3}, {NULL}};
const Segment SGI306962[] = {{&S4172[8], 3}, {NULL}};
const Segment SGI306963[] = {{&S4172[13], 10}, {NULL}};
const Segment SGI306964[] = {{&S4172[42], 7}, {NULL}};
const Segment SGI306965[] = {{&S4172[35], 14}, {NULL}};
const Segment SGI306966[] = {{&S4172[64], 11}, {NULL}};
const Segment SGI306967[] = {{&S4172[64], 16}, {NULL}};
const Segment SGI306968[] = {{&S4172[90], 9}, {NULL}};
const Segment SGI306969[] = {{&S4173[30], 3}, {&S4173[128], 3}, {NULL}};
const Segment SGI306970[] = {{&S4173[8], 3}, {&S4173[8], 3}, {NULL}};
const Segment SGI306971[] = {{&S4173[13], 10}, {&S4173[13], 10}, {NULL}};
const Segment SGI306972[] = {{&S4173[42], 7}, {&S4173[42], 7}, {NULL}};
const Segment SGI306973[] = {{&S4173[35], 14}, {&S4173[35], 14}, {NULL}};
const Segment SGI306974[] = {{&S4173[72], 11}, {NULL}};
const Segment SGI306975[] = {{&S4173[72], 16}, {NULL}};
const Segment SGI306976[] = {{&S4173[149], 3}, {NULL}};
const Segment SGI306977[] = {{&S4175[30], 3}, {NULL}};
const Segment SGI306978[] = {{&S4175[8], 3}, {NULL}};
const Segment SGI306979[] = {{&S4175[13], 10}, {NULL}};
const Segment SGI306980[] = {{&S4175[42], 7}, {NULL}};
const Segment SGI306981[] = {{&S4175[35], 14}, {NULL}};
const Segment SGI306982[] = {{&S4175[56], 11}, {NULL}};
const Segment SGI306983[] = {{&S4175[120], 15}, {NULL}};
const Segment SGI306984[] = {{&S4175[56], 16}, {NULL}};
const Segment SGI306985[] = {{&S4175[51], 3}, {NULL}};
const Segment SGI306986[] = {{&S4176[30], 3}, {NULL}};
const Segment SGI306987[] = {{&S4176[8], 3}, {NULL}};
const Segment SGI306988[] = {{&S4176[13], 10}, {NULL}};
const Segment SGI306989[] = {{&S4176[42], 7}, {NULL}};
const Segment SGI306990[] = {{&S4176[35], 14}, {NULL}};
const Segment SGI306991[] = {{&S4176[69], 11}, {NULL}};
const Segment SGI306992[] = {{&S4176[69], 16}, {NULL}};
const Segment SGI306993[] = {{&S4176[64], 3}, {NULL}};
const Segment SGI306994[] = {{&S4181[83], 3}, {NULL}};
const Segment SGI306995[] = {{&S4181[32], 7}, {NULL}};
const Segment SGI306996[] = {{&S4198[30], 3}, {NULL}};
const Segment SGI306997[] = {{&S4198[8], 3}, {NULL}};
const Segment SGI306998[] = {{&S4198[13], 10}, {NULL}};
const Segment SGI306999[] = {{&S4198[42], 7}, {NULL}};
const Segment SGI307000[] = {{&S4198[35], 14}, {NULL}};
const Segment SGI307001[] = {{&S4198[51], 11}, {NULL}};
const Segment SGI307002[] = {{&S4198[51], 16}, {NULL}};
const Segment SGI307003[] = {{&S4199[30], 3}, {NULL}};
const Segment SGI307004[] = {{&S4199[8], 3}, {NULL}};
const Segment SGI307005[] = {{&S4199[13], 10}, {NULL}};
const Segment SGI307006[] = {{&S4199[42], 7}, {NULL}};
const Segment SGI307007[] = {{&S4199[35], 14}, {NULL}};
const Segment SGI307008[] = {{&S4199[64], 11}, {NULL}};
const Segment SGI307009[] = {{&S4199[64], 16}, {NULL}};
const Segment SGI307010[] = {{&S4200[30], 3}, {NULL}};
const Segment SGI307011[] = {{&S4200[8], 3}, {NULL}};
const Segment SGI307012[] = {{&S4200[13], 10}, {NULL}};
const Segment SGI307013[] = {{&S4200[42], 7}, {NULL}};
const Segment SGI307014[] = {{&S4200[35], 14}, {NULL}};
const Segment SGI307015[] = {{&S4200[56], 11}, {NULL}};
const Segment SGI307016[] = {{&S4200[56], 16}, {NULL}};
const Segment SGI307017[] = {{&S4200[51], 3}, {NULL}};
const Segment SGI307018[] = {{&S4201[30], 3}, {NULL}};
const Segment SGI307019[] = {{&S4201[8], 3}, {NULL}};
const Segment SGI307020[] = {{&S4201[13], 10}, {NULL}};
const Segment SGI307021[] = {{&S4201[42], 7}, {NULL}};
const Segment SGI307022[] = {{&S4201[35], 14}, {NULL}};
const Segment SGI307023[] = {{&S4201[51], 11}, {NULL}};
const Segment SGI307024[] = {{&S4201[115], 15}, {NULL}};
const Segment SGI307025[] = {{&S4201[51], 16}, {NULL}};
const Segment SGI307026[] = {{&S4202[30], 3}, {NULL}};
const Segment SGI307027[] = {{&S4202[8], 3}, {NULL}};
const Segment SGI307028[] = {{&S4202[13], 10}, {NULL}};
const Segment SGI307029[] = {{&S4202[42], 7}, {NULL}};
const Segment SGI307030[] = {{&S4202[35], 14}, {NULL}};
const Segment SGI307031[] = {{&S4202[51], 11}, {NULL}};
const Segment SGI307032[] = {{&S4202[51], 16}, {NULL}};
const Segment SGI307033[] = {{&S4203[30], 3}, {NULL}};
const Segment SGI307034[] = {{&S4203[8], 3}, {NULL}};
const Segment SGI307035[] = {{&S4203[13], 10}, {NULL}};
const Segment SGI307036[] = {{&S4203[42], 7}, {NULL}};
const Segment SGI307037[] = {{&S4203[35], 14}, {NULL}};
const Segment SGI307038[] = {{&S4203[72], 11}, {NULL}};
const Segment SGI307039[] = {{&S4203[136], 15}, {NULL}};
const Segment SGI307040[] = {{&S4203[72], 16}, {NULL}};
const Segment SGI307041[] = {{&S4204[30], 3}, {NULL}};
const Segment SGI307042[] = {{&S4204[8], 3}, {NULL}};
const Segment SGI307043[] = {{&S4204[13], 10}, {NULL}};
const Segment SGI307044[] = {{&S4204[42], 7}, {NULL}};
const Segment SGI307045[] = {{&S4204[35], 14}, {NULL}};
const Segment SGI307046[] = {{&S4204[64], 11}, {NULL}};
const Segment SGI307047[] = {{&S4204[64], 16}, {NULL}};
const Segment SGI307048[] = {{&S4204[90], 9}, {NULL}};
const Segment SGI307049[] = {{&S4205[30], 3}, {NULL}};
const Segment SGI307050[] = {{&S4205[8], 3}, {NULL}};
const Segment SGI307051[] = {{&S4205[13], 10}, {NULL}};
const Segment SGI307052[] = {{&S4205[42], 7}, {NULL}};
const Segment SGI307053[] = {{&S4205[35], 14}, {NULL}};
const Segment SGI307054[] = {{&S4205[64], 11}, {NULL}};
const Segment SGI307055[] = {{&S4205[64], 16}, {NULL}};
const Segment SGI307056[] = {{&S4206[30], 3}, {NULL}};
const Segment SGI307057[] = {{&S4206[8], 3}, {NULL}};
const Segment SGI307058[] = {{&S4206[13], 10}, {NULL}};
const Segment SGI307059[] = {{&S4206[42], 7}, {NULL}};
const Segment SGI307060[] = {{&S4206[35], 14}, {NULL}};
const Segment SGI307061[] = {{&S4206[56], 11}, {NULL}};
const Segment SGI307062[] = {{&S4206[120], 15}, {NULL}};
const Segment SGI307063[] = {{&S4206[56], 16}, {NULL}};
const Segment SGI307064[] = {{&S4206[51], 3}, {NULL}};
const Segment SGI307065[] = {{&S4207[30], 3}, {NULL}};
const Segment SGI307066[] = {{&S4207[8], 3}, {NULL}};
const Segment SGI307067[] = {{&S4207[13], 10}, {NULL}};
const Segment SGI307068[] = {{&S4207[42], 7}, {NULL}};
const Segment SGI307069[] = {{&S4207[35], 14}, {NULL}};
const Segment SGI307070[] = {{&S4207[71], 11}, {NULL}};
const Segment SGI307071[] = {{&S4207[51], 6}, {NULL}};
const Segment SGI307072[] = {{&S4207[71], 16}, {NULL}};
const Segment SGI307073[] = {{&S4209[30], 3}, {NULL}};
const Segment SGI307074[] = {{&S4209[8], 3}, {NULL}};
const Segment SGI307075[] = {{&S4209[13], 10}, {NULL}};
const Segment SGI307076[] = {{&S4209[42], 7}, {NULL}};
const Segment SGI307077[] = {{&S4209[35], 14}, {NULL}};
const Segment SGI307078[] = {{&S4209[69], 11}, {NULL}};
const Segment SGI307079[] = {{&S4209[69], 16}, {NULL}};
const Segment SGI307080[] = {{&S4209[64], 3}, {NULL}};
const Segment SGI307081[] = {{&S4211[30], 3}, {&S4211[120], 3}, {NULL}};
const Segment SGI307082[] = {{&S4211[8], 3}, {&S4211[8], 3}, {NULL}};
const Segment SGI307083[] = {{&S4211[13], 10}, {&S4211[13], 10}, {NULL}};
const Segment SGI307084[] = {{&S4211[42], 7}, {&S4211[42], 7}, {NULL}};
const Segment SGI307085[] = {{&S4211[35], 14}, {&S4211[35], 14}, {NULL}};
const Segment SGI307086[] = {{&S4211[64], 11}, {NULL}};
const Segment SGI307087[] = {{&S4211[64], 16}, {NULL}};
const Segment SGI307088[] = {{&S4211[141], 3}, {NULL}};
const Segment SGI307089[] = {{&S4212[30], 3}, {NULL}};
const Segment SGI307090[] = {{&S4212[8], 3}, {NULL}};
const Segment SGI307091[] = {{&S4212[13], 10}, {NULL}};
const Segment SGI307092[] = {{&S4212[42], 7}, {NULL}};
const Segment SGI307093[] = {{&S4212[35], 14}, {NULL}};
const Segment SGI307094[] = {{&S4212[51], 11}, {NULL}};
const Segment SGI307095[] = {{&S4212[51], 16}, {NULL}};
const Segment SGI307096[] = {{&S4214[30], 3}, {&S4214[128], 3}, {NULL}};
const Segment SGI307097[] = {{&S4214[8], 3}, {&S4214[8], 3}, {NULL}};
const Segment SGI307098[] = {{&S4214[13], 10}, {&S4214[13], 10}, {NULL}};
const Segment SGI307099[] = {{&S4214[42], 7}, {&S4214[42], 7}, {NULL}};
const Segment SGI307100[] = {{&S4214[35], 14}, {&S4214[35], 14}, {NULL}};
const Segment SGI307101[] = {{&S4214[72], 11}, {NULL}};
const Segment SGI307102[] = {{&S4214[72], 16}, {NULL}};
const Segment SGI307103[] = {{&S4214[149], 3}, {NULL}};
const Segment SGI307104[] = {{&S4215[30], 3}, {NULL}};
const Segment SGI307105[] = {{&S4215[8], 3}, {NULL}};
const Segment SGI307106[] = {{&S4215[13], 10}, {NULL}};
const Segment SGI307107[] = {{&S4215[42], 7}, {NULL}};
const Segment SGI307108[] = {{&S4215[35], 14}, {NULL}};
const Segment SGI307109[] = {{&S4215[64], 11}, {NULL}};
const Segment SGI307110[] = {{&S4215[173], 4}, {NULL}};
const Segment SGI307111[] = {{&S4215[64], 16}, {NULL}};
const Segment SGI307112[] = {{&S4216[30], 3}, {&S4216[120], 3}, {NULL}};
const Segment SGI307113[] = {{&S4216[8], 3}, {&S4216[8], 3}, {NULL}};
const Segment SGI307114[] = {{&S4216[13], 10}, {&S4216[13], 10}, {NULL}};
const Segment SGI307115[] = {{&S4216[42], 7}, {&S4216[42], 7}, {NULL}};
const Segment SGI307116[] = {{&S4216[35], 14}, {&S4216[35], 14}, {NULL}};
const Segment SGI307117[] = {{&S4216[64], 11}, {NULL}};
const Segment SGI307118[] = {{&S4216[186], 15}, {NULL}};
const Segment SGI307119[] = {{&S4216[64], 16}, {NULL}};
const Segment SGI307120[] = {{&S4216[141], 3}, {NULL}};
const Segment SGI307121[] = {{&S4217[30], 3}, {NULL}};
const Segment SGI307122[] = {{&S4217[8], 3}, {NULL}};
const Segment SGI307123[] = {{&S4217[13], 10}, {NULL}};
const Segment SGI307124[] = {{&S4217[42], 7}, {NULL}};
const Segment SGI307125[] = {{&S4217[35], 14}, {NULL}};
const Segment SGI307126[] = {{&S4217[56], 11}, {NULL}};
const Segment SGI307127[] = {{&S4217[56], 16}, {NULL}};
const Segment SGI307128[] = {{&S4217[51], 3}, {NULL}};
const Segment SGI307129[] = {{&S4501[30], 3}, {NULL}};
const Segment SGI307130[] = {{&S4501[8], 3}, {NULL}};
const Segment SGI307131[] = {{&S4501[13], 10}, {NULL}};
const Segment SGI307132[] = {{&S4501[42], 7}, {NULL}};
const Segment SGI307133[] = {{&S4501[35], 14}, {NULL}};
const Segment SGI307134[] = {{&S4501[56], 11}, {NULL}};
const Segment SGI307135[] = {{&S4501[56], 16}, {NULL}};
const Segment SGI307136[] = {{&S4501[51], 3}, {NULL}};
const Segment SGI307137[] = {{&S4502[30], 3}, {NULL}};
const Segment SGI307138[] = {{&S4502[8], 3}, {NULL}};
const Segment SGI307139[] = {{&S4502[13], 10}, {NULL}};
const Segment SGI307140[] = {{&S4502[42], 7}, {NULL}};
const Segment SGI307141[] = {{&S4502[35], 14}, {NULL}};
const Segment SGI307142[] = {{&S4502[64], 11}, {NULL}};
const Segment SGI307143[] = {{&S4502[64], 16}, {NULL}};
const Segment SGI307144[] = {{&S4505[30], 3}, {NULL}};
const Segment SGI307145[] = {{&S4505[8], 3}, {NULL}};
const Segment SGI307146[] = {{&S4505[13], 10}, {NULL}};
const Segment SGI307147[] = {{&S4505[42], 7}, {NULL}};
const Segment SGI307148[] = {{&S4505[35], 14}, {NULL}};
const Segment SGI307149[] = {{&S4505[51], 11}, {NULL}};
const Segment SGI307150[] = {{&S4505[51], 16}, {NULL}};
const Segment SGI307151[] = {{&S4506[30], 3}, {NULL}};
const Segment SGI307152[] = {{&S4506[8], 3}, {NULL}};
const Segment SGI307153[] = {{&S4506[13], 10}, {NULL}};
const Segment SGI307154[] = {{&S4506[42], 7}, {NULL}};
const Segment SGI307155[] = {{&S4506[35], 14}, {NULL}};
const Segment SGI307156[] = {{&S4506[64], 11}, {NULL}};
const Segment SGI307157[] = {{&S4506[64], 16}, {NULL}};
const Segment SGI307158[] = {{&S4508[30], 3}, {NULL}};
const Segment SGI307159[] = {{&S4508[8], 3}, {NULL}};
const Segment SGI307160[] = {{&S4508[13], 10}, {NULL}};
const Segment SGI307161[] = {{&S4508[42], 7}, {NULL}};
const Segment SGI307162[] = {{&S4508[35], 14}, {NULL}};
const Segment SGI307163[] = {{&S4508[72], 11}, {NULL}};
const Segment SGI307164[] = {{&S4508[136], 15}, {NULL}};
const Segment SGI307165[] = {{&S4508[72], 16}, {NULL}};
const Segment SGI307166[] = {{&S4509[30], 3}, {NULL}};
const Segment SGI307167[] = {{&S4509[8], 3}, {NULL}};
const Segment SGI307168[] = {{&S4509[13], 10}, {NULL}};
const Segment SGI307169[] = {{&S4509[42], 7}, {NULL}};
const Segment SGI307170[] = {{&S4509[35], 14}, {NULL}};
const Segment SGI307171[] = {{&S4509[71], 11}, {NULL}};
const Segment SGI307172[] = {{&S4509[51], 6}, {NULL}};
const Segment SGI307173[] = {{&S4509[71], 16}, {NULL}};
const Segment SGI307174[] = {{&S4511[30], 3}, {NULL}};
const Segment SGI307175[] = {{&S4511[8], 3}, {NULL}};
const Segment SGI307176[] = {{&S4511[13], 10}, {NULL}};
const Segment SGI307177[] = {{&S4511[42], 7}, {NULL}};
const Segment SGI307178[] = {{&S4511[35], 14}, {NULL}};
const Segment SGI307179[] = {{&S4511[59], 11}, {NULL}};
const Segment SGI307180[] = {{&S4511[123], 15}, {NULL}};
const Segment SGI307181[] = {{&S4511[59], 16}, {NULL}};
const Segment SGI307182[] = {{&S4512[30], 3}, {&S4512[128], 3}, {NULL}};
const Segment SGI307183[] = {{&S4512[8], 3}, {&S4512[8], 3}, {NULL}};
const Segment SGI307184[] = {{&S4512[13], 10}, {&S4512[13], 10}, {NULL}};
const Segment SGI307185[] = {{&S4512[42], 7}, {&S4512[42], 7}, {NULL}};
const Segment SGI307186[] = {{&S4512[35], 14}, {&S4512[35], 14}, {NULL}};
const Segment SGI307187[] = {{&S4512[72], 11}, {NULL}};
const Segment SGI307188[] = {{&S4512[72], 16}, {NULL}};
const Segment SGI307189[] = {{&S4512[149], 3}, {NULL}};
const Segment SGI307190[] = {{&S4513[30], 3}, {NULL}};
const Segment SGI307191[] = {{&S4513[8], 3}, {NULL}};
const Segment SGI307192[] = {{&S4513[13], 10}, {NULL}};
const Segment SGI307193[] = {{&S4513[42], 7}, {NULL}};
const Segment SGI307194[] = {{&S4513[35], 14}, {NULL}};
const Segment SGI307195[] = {{&S4513[69], 11}, {NULL}};
const Segment SGI307196[] = {{&S4513[69], 16}, {NULL}};
const Segment SGI307197[] = {{&S4513[64], 3}, {NULL}};
const Segment SGI307198[] = {{&S4514[30], 3}, {NULL}};
const Segment SGI307199[] = {{&S4514[8], 3}, {NULL}};
const Segment SGI307200[] = {{&S4514[13], 10}, {NULL}};
const Segment SGI307201[] = {{&S4514[42], 7}, {NULL}};
const Segment SGI307202[] = {{&S4514[35], 14}, {NULL}};
const Segment SGI307203[] = {{&S4514[64], 11}, {NULL}};
const Segment SGI307204[] = {{&S4514[64], 16}, {NULL}};
const Segment SGI307205[] = {{&S4514[90], 9}, {NULL}};
const Segment SGI307206[] = {{&S4515[30], 3}, {NULL}};
const Segment SGI307207[] = {{&S4515[8], 3}, {NULL}};
const Segment SGI307208[] = {{&S4515[13], 10}, {NULL}};
const Segment SGI307209[] = {{&S4515[42], 7}, {NULL}};
const Segment SGI307210[] = {{&S4515[35], 14}, {NULL}};
const Segment SGI307211[] = {{&S4515[56], 11}, {NULL}};
const Segment SGI307212[] = {{&S4515[120], 15}, {NULL}};
const Segment SGI307213[] = {{&S4515[56], 16}, {NULL}};
const Segment SGI307214[] = {{&S4515[51], 3}, {NULL}};
const Segment SGI307215[] = {{&S4516[30], 3}, {NULL}};
const Segment SGI307216[] = {{&S4516[8], 3}, {NULL}};
const Segment SGI307217[] = {{&S4516[13], 10}, {NULL}};
const Segment SGI307218[] = {{&S4516[42], 7}, {NULL}};
const Segment SGI307219[] = {{&S4516[35], 14}, {NULL}};
const Segment SGI307220[] = {{&S4516[51], 11}, {NULL}};
const Segment SGI307221[] = {{&S4516[51], 16}, {NULL}};
const Segment SGI307222[] = {{&S4518[30], 3}, {NULL}};
const Segment SGI307223[] = {{&S4518[8], 3}, {NULL}};
const Segment SGI307224[] = {{&S4518[13], 10}, {NULL}};
const Segment SGI307225[] = {{&S4518[42], 7}, {NULL}};
const Segment SGI307226[] = {{&S4518[35], 14}, {NULL}};
const Segment SGI307227[] = {{&S4518[72], 11}, {NULL}};
const Segment SGI307228[] = {{&S4518[181], 4}, {NULL}};
const Segment SGI307229[] = {{&S4518[72], 16}, {NULL}};
const Segment SGI307230[] = {{&S4519[30], 3}, {&S4519[107], 3}, {NULL}};
const Segment SGI307231[] = {{&S4519[8], 3}, {&S4519[8], 3}, {NULL}};
const Segment SGI307232[] = {{&S4519[13], 10}, {&S4519[13], 10}, {NULL}};
const Segment SGI307233[] = {{&S4519[42], 7}, {&S4519[42], 7}, {NULL}};
const Segment SGI307234[] = {{&S4519[35], 14}, {&S4519[35], 14}, {NULL}};
const Segment SGI307235[] = {{&S4519[51], 11}, {NULL}};
const Segment SGI307236[] = {{&S4519[51], 16}, {NULL}};
const Segment SGI307237[] = {{&S4519[128], 3}, {NULL}};
const Segment SGI307238[] = {{&S4521[30], 3}, {NULL}};
const Segment SGI307239[] = {{&S4521[8], 3}, {NULL}};
const Segment SGI307240[] = {{&S4521[13], 10}, {NULL}};
const Segment SGI307241[] = {{&S4521[42], 7}, {NULL}};
const Segment SGI307242[] = {{&S4521[35], 14}, {NULL}};
const Segment SGI307243[] = {{&S4521[64], 11}, {NULL}};
const Segment SGI307244[] = {{&S4521[64], 16}, {NULL}};
const Segment SGI307245[] = {{&S4523[30], 3}, {NULL}};
const Segment SGI307246[] = {{&S4523[8], 3}, {NULL}};
const Segment SGI307247[] = {{&S4523[13], 10}, {NULL}};
const Segment SGI307248[] = {{&S4523[42], 7}, {NULL}};
const Segment SGI307249[] = {{&S4523[35], 14}, {NULL}};
const Segment SGI307250[] = {{&S4523[56], 11}, {NULL}};
const Segment SGI307251[] = {{&S4523[56], 16}, {NULL}};
const Segment SGI307252[] = {{&S4523[51], 3}, {NULL}};
const Segment SGI307253[] = {{&S4524[30], 3}, {&S4524[168], 3}, {NULL}};
const Segment SGI307254[] = {{&S4524[8], 3}, {&S4524[8], 3}, {NULL}};
const Segment SGI307255[] = {{&S4524[13], 10}, {&S4524[13], 10}, {NULL}};
const Segment SGI307256[] = {{&S4524[42], 7}, {&S4524[180], 7}, {NULL}};
const Segment SGI307257[] = {{&S4524[35], 14}, {&S4524[173], 14}, {NULL}};
const Segment SGI307258[] = {{&S4524[64], 11}, {NULL}};
const Segment SGI307259[] = {{&S4524[64], 16}, {NULL}};
const Segment SGI307260[] = {{&S4524[189], 3}, {NULL}};
const Segment SGI307261[] = {{&S4525[30], 3}, {NULL}};
const Segment SGI307262[] = {{&S4525[8], 3}, {NULL}};
const Segment SGI307263[] = {{&S4525[13], 10}, {NULL}};
const Segment SGI307264[] = {{&S4525[42], 7}, {NULL}};
const Segment SGI307265[] = {{&S4525[35], 14}, {NULL}};
const Segment SGI307266[] = {{&S4525[76], 11}, {NULL}};
const Segment SGI307267[] = {{&S4525[51], 6}, {NULL}};
const Segment SGI307268[] = {{&S4525[76], 16}, {NULL}};
const Segment SGI307269[] = {{&S4525[71], 3}, {NULL}};
const Segment SGI307270[] = {{&S4704[30], 4}, {NULL}};
const Segment SGI307271[] = {{&S4704[8], 3}, {NULL}};
const Segment SGI307272[] = {{&S4704[13], 10}, {NULL}};
const Segment SGI307273[] = {{&S4704[43], 7}, {NULL}};
const Segment SGI307274[] = {{&S4704[36], 14}, {NULL}};
const Segment SGI307275[] = {{&S4704[77], 5}, {NULL}};
const Segment SGI307276[] = {{&S4704[52], 5}, {NULL}};
const Segment SGI307277[] = {{&S4705[30], 4}, {NULL}};
const Segment SGI307278[] = {{&S4705[8], 3}, {NULL}};
const Segment SGI307279[] = {{&S4705[13], 10}, {NULL}};
const Segment SGI307280[] = {{&S4705[43], 7}, {NULL}};
const Segment SGI307281[] = {{&S4705[36], 14}, {NULL}};
const Segment SGI307282[] = {{&S4705[65], 5}, {NULL}};
const Segment SGI307283[] = {{&S4706[30], 4}, {NULL}};
const Segment SGI307284[] = {{&S4706[8], 3}, {NULL}};
const Segment SGI307285[] = {{&S4706[13], 10}, {NULL}};
const Segment SGI307286[] = {{&S4706[43], 7}, {NULL}};
const Segment SGI307287[] = {{&S4706[36], 14}, {NULL}};
const Segment SGI307288[] = {{&S4706[72], 5}, {NULL}};
const Segment SGI307289[] = {{&S4706[52], 6}, {NULL}};
const Segment SGI307290[] = {{&S4707[30], 4}, {NULL}};
const Segment SGI307291[] = {{&S4707[8], 3}, {NULL}};
const Segment SGI307292[] = {{&S4707[13], 10}, {NULL}};
const Segment SGI307293[] = {{&S4707[43], 7}, {NULL}};
const Segment SGI307294[] = {{&S4707[36], 14}, {NULL}};
const Segment SGI307295[] = {{&S4707[72], 5}, {NULL}};
const Segment SGI307296[] = {{&S4707[52], 6}, {NULL}};
const Segment SGI307297[] = {{&S4707[159], 9}, {NULL}};
const Segment SGI307298[] = {{&S4708[30], 4}, {NULL}};
const Segment SGI307299[] = {{&S4708[8], 3}, {NULL}};
const Segment SGI307300[] = {{&S4708[13], 10}, {NULL}};
const Segment SGI307301[] = {{&S4708[43], 7}, {NULL}};
const Segment SGI307302[] = {{&S4708[36], 14}, {NULL}};
const Segment SGI307303[] = {{&S4708[77], 5}, {NULL}};
const Segment SGI307304[] = {{&S4708[52], 5}, {NULL}};
const Segment SGI307305[] = {{&S4708[164], 9}, {NULL}};
const Segment SGI307306[] = {{&S4941[30], 3}, {NULL}};
const Segment SGI307307[] = {{&S4941[8], 3}, {NULL}};
const Segment SGI307308[] = {{&S4941[13], 10}, {NULL}};
const Segment SGI307309[] = {{&S4941[42], 7}, {NULL}};
const Segment SGI307310[] = {{&S4941[35], 14}, {NULL}};
const Segment SGI307311[] = {{&S4941[64], 11}, {NULL}};
const Segment SGI307312[] = {{&S4941[64], 16}, {NULL}};
const Segment SGI307313[] = {{&S4942[30], 3}, {NULL}};
const Segment SGI307314[] = {{&S4942[8], 3}, {NULL}};
const Segment SGI307315[] = {{&S4942[13], 10}, {NULL}};
const Segment SGI307316[] = {{&S4942[42], 7}, {NULL}};
const Segment SGI307317[] = {{&S4942[35], 14}, {NULL}};
const Segment SGI307318[] = {{&S4942[51], 11}, {NULL}};
const Segment SGI307319[] = {{&S4942[51], 16}, {NULL}};
const Segment SGI307320[] = {{&S4943[30], 3}, {NULL}};
const Segment SGI307321[] = {{&S4943[8], 3}, {NULL}};
const Segment SGI307322[] = {{&S4943[13], 10}, {NULL}};
const Segment SGI307323[] = {{&S4943[42], 7}, {NULL}};
const Segment SGI307324[] = {{&S4943[35], 14}, {NULL}};
const Segment SGI307325[] = {{&S4943[56], 11}, {NULL}};
const Segment SGI307326[] = {{&S4943[56], 16}, {NULL}};
const Segment SGI307327[] = {{&S4943[51], 3}, {NULL}};
const Segment SGI307328[] = {{&S4945[30], 3}, {NULL}};
const Segment SGI307329[] = {{&S4945[8], 3}, {NULL}};
const Segment SGI307330[] = {{&S4945[13], 10}, {NULL}};
const Segment SGI307331[] = {{&S4945[42], 7}, {NULL}};
const Segment SGI307332[] = {{&S4945[35], 14}, {NULL}};
const Segment SGI307333[] = {{&S4945[64], 11}, {NULL}};
const Segment SGI307334[] = {{&S4945[64], 16}, {NULL}};
const Segment SGI307335[] = {{&S4947[30], 3}, {NULL}};
const Segment SGI307336[] = {{&S4947[8], 3}, {NULL}};
const Segment SGI307337[] = {{&S4947[13], 10}, {NULL}};
const Segment SGI307338[] = {{&S4947[42], 7}, {NULL}};
const Segment SGI307339[] = {{&S4947[35], 14}, {NULL}};
const Segment SGI307340[] = {{&S4947[72], 11}, {NULL}};
const Segment SGI307341[] = {{&S4947[137], 15}, {NULL}};
const Segment SGI307342[] = {{&S4947[72], 16}, {NULL}};
const Segment SGI307343[] = {{&S4951[30], 3}, {NULL}};
const Segment SGI307344[] = {{&S4951[8], 3}, {NULL}};
const Segment SGI307345[] = {{&S4951[13], 10}, {NULL}};
const Segment SGI307346[] = {{&S4951[42], 7}, {NULL}};
const Segment SGI307347[] = {{&S4951[35], 14}, {NULL}};
const Segment SGI307348[] = {{&S4951[56], 11}, {NULL}};
const Segment SGI307349[] = {{&S4951[121], 15}, {NULL}};
const Segment SGI307350[] = {{&S4951[56], 16}, {NULL}};
const Segment SGI307351[] = {{&S4951[51], 3}, {NULL}};
const Segment SGI307352[] = {{&S4952[30], 3}, {NULL}};
const Segment SGI307353[] = {{&S4952[8], 3}, {NULL}};
const Segment SGI307354[] = {{&S4952[13], 10}, {NULL}};
const Segment SGI307355[] = {{&S4952[42], 7}, {NULL}};
const Segment SGI307356[] = {{&S4952[35], 14}, {NULL}};
const Segment SGI307357[] = {{&S4952[64], 11}, {NULL}};
const Segment SGI307358[] = {{&S4952[64], 16}, {NULL}};
const Segment SGI307359[] = {{&S4952[91], 9}, {NULL}};
const Segment SGI307360[] = {{&S4953[30], 3}, {NULL}};
const Segment SGI307361[] = {{&S4953[8], 3}, {NULL}};
const Segment SGI307362[] = {{&S4953[13], 10}, {NULL}};
const Segment SGI307363[] = {{&S4953[42], 7}, {NULL}};
const Segment SGI307364[] = {{&S4953[35], 14}, {NULL}};
const Segment SGI307365[] = {{&S4953[51], 11}, {NULL}};
const Segment SGI307366[] = {{&S4953[51], 16}, {NULL}};
const Segment SGI307367[] = {{&S4954[30], 3}, {&S4954[129], 3}, {NULL}};
const Segment SGI307368[] = {{&S4954[8], 3}, {&S4954[8], 3}, {NULL}};
const Segment SGI307369[] = {{&S4954[13], 10}, {&S4954[13], 10}, {NULL}};
const Segment SGI307370[] = {{&S4954[42], 7}, {&S4954[42], 7}, {NULL}};
const Segment SGI307371[] = {{&S4954[35], 14}, {&S4954[35], 14}, {NULL}};
const Segment SGI307372[] = {{&S4954[72], 11}, {NULL}};
const Segment SGI307373[] = {{&S4954[72], 16}, {NULL}};
const Segment SGI307374[] = {{&S4954[150], 3}, {NULL}};
const Segment SGI307375[] = {{&S4955[30], 3}, {NULL}};
const Segment SGI307376[] = {{&S4955[8], 3}, {NULL}};
const Segment SGI307377[] = {{&S4955[13], 10}, {NULL}};
const Segment SGI307378[] = {{&S4955[42], 7}, {NULL}};
const Segment SGI307379[] = {{&S4955[35], 14}, {NULL}};
const Segment SGI307380[] = {{&S4955[51], 11}, {NULL}};
const Segment SGI307381[] = {{&S4955[51], 16}, {NULL}};
const Segment SGI307382[] = {{&S4957[30], 3}, {NULL}};
const Segment SGI307383[] = {{&S4957[8], 3}, {NULL}};
const Segment SGI307384[] = {{&S4957[13], 10}, {NULL}};
const Segment SGI307385[] = {{&S4957[42], 7}, {NULL}};
const Segment SGI307386[] = {{&S4957[35], 14}, {NULL}};
const Segment SGI307387[] = {{&S4957[71], 11}, {NULL}};
const Segment SGI307388[] = {{&S4957[51], 6}, {NULL}};
const Segment SGI307389[] = {{&S4957[71], 16}, {NULL}};
const Segment SGI307390[] = {{&S4958[30], 3}, {NULL}};
const Segment SGI307391[] = {{&S4958[8], 3}, {NULL}};
const Segment SGI307392[] = {{&S4958[13], 10}, {NULL}};
const Segment SGI307393[] = {{&S4958[42], 7}, {NULL}};
const Segment SGI307394[] = {{&S4958[35], 14}, {NULL}};
const Segment SGI307395[] = {{&S4958[51], 11}, {NULL}};
const Segment SGI307396[] = {{&S4958[116], 15}, {NULL}};
const Segment SGI307397[] = {{&S4958[51], 16}, {NULL}};
const Segment SGI307398[] = {{&S4959[30], 3}, {NULL}};
const Segment SGI307399[] = {{&S4959[8], 3}, {NULL}};
const Segment SGI307400[] = {{&S4959[13], 10}, {NULL}};
const Segment SGI307401[] = {{&S4959[42], 7}, {NULL}};
const Segment SGI307402[] = {{&S4959[35], 14}, {NULL}};
const Segment SGI307403[] = {{&S4959[64], 11}, {NULL}};
const Segment SGI307404[] = {{&S4959[253], 4}, {NULL}};
const Segment SGI307405[] = {{&S4959[64], 16}, {NULL}};
const Segment SGI307406[] = {{&S4993[30], 3}, {NULL}};
const Segment SGI307407[] = {{&S4993[8], 3}, {NULL}};
const Segment SGI307408[] = {{&S4993[13], 10}, {NULL}};
const Segment SGI307409[] = {{&S4993[42], 7}, {NULL}};
const Segment SGI307410[] = {{&S4993[35], 14}, {NULL}};
const Segment SGI307411[] = {{&S4993[64], 11}, {NULL}};
const Segment SGI307412[] = {{&S4993[157], 15}, {NULL}};
const Segment SGI307413[] = {{&S4993[64], 16}, {NULL}};
const Segment SGI307414[] = {{&S4994[30], 3}, {NULL}};
const Segment SGI307415[] = {{&S4994[8], 3}, {NULL}};
const Segment SGI307416[] = {{&S4994[13], 10}, {NULL}};
const Segment SGI307417[] = {{&S4994[42], 7}, {NULL}};
const Segment SGI307418[] = {{&S4994[35], 14}, {NULL}};
const Segment SGI307419[] = {{&S4994[71], 11}, {NULL}};
const Segment SGI307420[] = {{&S4994[164], 15}, {NULL}};
const Segment SGI307421[] = {{&S4994[51], 6}, {NULL}};
const Segment SGI307422[] = {{&S4994[71], 16}, {NULL}};
const Segment SGI307423[] = {{&S4995[30], 3}, {NULL}};
const Segment SGI307424[] = {{&S4995[8], 3}, {NULL}};
const Segment SGI307425[] = {{&S4995[13], 10}, {NULL}};
const Segment SGI307426[] = {{&S4995[42], 7}, {NULL}};
const Segment SGI307427[] = {{&S4995[35], 14}, {NULL}};
const Segment SGI307428[] = {{&S4995[71], 11}, {NULL}};
const Segment SGI307429[] = {{&S4995[51], 6}, {NULL}};
const Segment SGI307430[] = {{&S4995[71], 16}, {NULL}};
const Segment SGI307431[] = {{&S4996[30], 3}, {NULL}};
const Segment SGI307432[] = {{&S4996[8], 3}, {NULL}};
const Segment SGI307433[] = {{&S4996[13], 10}, {NULL}};
const Segment SGI307434[] = {{&S4996[42], 7}, {NULL}};
const Segment SGI307435[] = {{&S4996[35], 14}, {NULL}};
const Segment SGI307436[] = {{&S4996[71], 11}, {NULL}};
const Segment SGI307437[] = {{&S4996[51], 6}, {NULL}};
const Segment SGI307438[] = {{&S4996[71], 16}, {NULL}};
const Segment SGI307439[] = {{&S4998[30], 3}, {NULL}};
const Segment SGI307440[] = {{&S4998[8], 3}, {NULL}};
const Segment SGI307441[] = {{&S4998[13], 10}, {NULL}};
const Segment SGI307442[] = {{&S4998[42], 7}, {NULL}};
const Segment SGI307443[] = {{&S4998[35], 14}, {NULL}};
const Segment SGI307444[] = {{&S4998[64], 11}, {NULL}};
const Segment SGI307445[] = {{&S4998[64], 16}, {NULL}};
const Segment SGI307446[] = {{&S5000[30], 4}, {NULL}};
const Segment SGI307447[] = {{&S5000[8], 3}, {NULL}};
const Segment SGI307448[] = {{&S5000[13], 10}, {NULL}};
const Segment SGI307449[] = {{&S5000[43], 7}, {NULL}};
const Segment SGI307450[] = {{&S5000[36], 14}, {NULL}};
const Segment SGI307451[] = {{&S5000[72], 11}, {NULL}};
const Segment SGI307452[] = {{&S5000[52], 6}, {NULL}};
const Segment SGI307453[] = {{&S5000[72], 16}, {NULL}};
const Segment SGI307454[] = {{&S5001[30], 4}, {NULL}};
const Segment SGI307455[] = {{&S5001[8], 3}, {NULL}};
const Segment SGI307456[] = {{&S5001[13], 10}, {NULL}};
const Segment SGI307457[] = {{&S5001[43], 7}, {NULL}};
const Segment SGI307458[] = {{&S5001[36], 14}, {NULL}};
const Segment SGI307459[] = {{&S5001[65], 11}, {NULL}};
const Segment SGI307460[] = {{&S5001[65], 16}, {NULL}};
const Segment SGI307461[] = {{&S5002[30], 3}, {NULL}};
const Segment SGI307462[] = {{&S5002[8], 3}, {NULL}};
const Segment SGI307463[] = {{&S5002[13], 10}, {NULL}};
const Segment SGI307464[] = {{&S5002[42], 7}, {NULL}};
const Segment SGI307465[] = {{&S5002[35], 14}, {NULL}};
const Segment SGI307466[] = {{&S5002[64], 11}, {NULL}};
const Segment SGI307467[] = {{&S5002[64], 16}, {NULL}};
const Segment SGI307468[] = {{&S5003[30], 3}, {NULL}};
const Segment SGI307469[] = {{&S5003[8], 3}, {NULL}};
const Segment SGI307470[] = {{&S5003[13], 10}, {NULL}};
const Segment SGI307471[] = {{&S5003[42], 7}, {NULL}};
const Segment SGI307472[] = {{&S5003[35], 14}, {NULL}};
const Segment SGI307473[] = {{&S5003[64], 11}, {NULL}};
const Segment SGI307474[] = {{&S5003[157], 15}, {NULL}};
const Segment SGI307475[] = {{&S5003[64], 16}, {NULL}};
const Segment SGI307476[] = {{&S5004[30], 3}, {NULL}};
const Segment SGI307477[] = {{&S5004[8], 3}, {NULL}};
const Segment SGI307478[] = {{&S5004[13], 10}, {NULL}};
const Segment SGI307479[] = {{&S5004[42], 7}, {NULL}};
const Segment SGI307480[] = {{&S5004[35], 14}, {NULL}};
const Segment SGI307481[] = {{&S5004[71], 11}, {NULL}};
const Segment SGI307482[] = {{&S5004[164], 15}, {NULL}};
const Segment SGI307483[] = {{&S5004[51], 6}, {NULL}};
const Segment SGI307484[] = {{&S5004[71], 16}, {NULL}};
const Segment SGI307485[] = {{&S5005[30], 3}, {NULL}};
const Segment SGI307486[] = {{&S5005[8], 3}, {NULL}};
const Segment SGI307487[] = {{&S5005[13], 10}, {NULL}};
const Segment SGI307488[] = {{&S5005[42], 7}, {NULL}};
const Segment SGI307489[] = {{&S5005[35], 14}, {NULL}};
const Segment SGI307490[] = {{&S5005[71], 11}, {NULL}};
const Segment SGI307491[] = {{&S5005[51], 6}, {NULL}};
const Segment SGI307492[] = {{&S5005[71], 16}, {NULL}};
const Segment SGI307493[] = {{&S5006[30], 3}, {NULL}};
const Segment SGI307494[] = {{&S5006[8], 3}, {NULL}};
const Segment SGI307495[] = {{&S5006[13], 10}, {NULL}};
const Segment SGI307496[] = {{&S5006[42], 7}, {NULL}};
const Segment SGI307497[] = {{&S5006[35], 14}, {NULL}};
const Segment SGI307498[] = {{&S5006[64], 11}, {NULL}};
const Segment SGI307499[] = {{&S5006[64], 16}, {NULL}};
const Segment SGI307500[] = {{&S5006[177], 9}, {NULL}};
const Segment SGI307501[] = {{&S5007[30], 3}, {NULL}};
const Segment SGI307502[] = {{&S5007[8], 3}, {NULL}};
const Segment SGI307503[] = {{&S5007[13], 10}, {NULL}};
const Segment SGI307504[] = {{&S5007[42], 7}, {NULL}};
const Segment SGI307505[] = {{&S5007[35], 14}, {NULL}};
const Segment SGI307506[] = {{&S5007[64], 11}, {NULL}};
const Segment SGI307507[] = {{&S5007[157], 15}, {NULL}};
const Segment SGI307508[] = {{&S5007[64], 16}, {NULL}};
const Segment SGI307509[] = {{&S5007[210], 9}, {NULL}};
const Segment SGI307510[] = {{&S5008[30], 3}, {NULL}};
const Segment SGI307511[] = {{&S5008[8], 3}, {NULL}};
const Segment SGI307512[] = {{&S5008[13], 10}, {NULL}};
const Segment SGI307513[] = {{&S5008[42], 7}, {NULL}};
const Segment SGI307514[] = {{&S5008[35], 14}, {NULL}};
const Segment SGI307515[] = {{&S5008[64], 11}, {NULL}};
const Segment SGI307516[] = {{&S5008[157], 15}, {NULL}};
const Segment SGI307517[] = {{&S5008[64], 16}, {NULL}};
const Segment SGI307518[] = {{&S5008[200], 9}, {NULL}};
const Segment SGI307519[] = {{&S5009[30], 3}, {NULL}};
const Segment SGI307520[] = {{&S5009[8], 3}, {NULL}};
const Segment SGI307521[] = {{&S5009[13], 10}, {NULL}};
const Segment SGI307522[] = {{&S5009[42], 7}, {NULL}};
const Segment SGI307523[] = {{&S5009[35], 14}, {NULL}};
const Segment SGI307524[] = {{&S5009[64], 11}, {NULL}};
const Segment SGI307525[] = {{&S5009[64], 16}, {NULL}};
const Segment SGI307526[] = {{&S5010[30], 3}, {NULL}};
const Segment SGI307527[] = {{&S5010[235], 9}, {NULL}};
const Segment SGI307528[] = {{&S5010[8], 3}, {NULL}};
const Segment SGI307529[] = {{&S5010[13], 10}, {NULL}};
const Segment SGI307530[] = {{&S5010[42], 7}, {NULL}};
const Segment SGI307531[] = {{&S5010[35], 14}, {NULL}};
const Segment SGI307532[] = {{&S5010[71], 11}, {NULL}};
const Segment SGI307533[] = {{&S5010[164], 15}, {NULL}};
const Segment SGI307534[] = {{&S5010[51], 6}, {NULL}};
const Segment SGI307535[] = {{&S5010[71], 16}, {NULL}};
const Segment SGI307536[] = {{&S5011[30], 3}, {NULL}};
const Segment SGI307537[] = {{&S5011[235], 9}, {NULL}};
const Segment SGI307538[] = {{&S5011[8], 3}, {NULL}};
const Segment SGI307539[] = {{&S5011[13], 10}, {NULL}};
const Segment SGI307540[] = {{&S5011[42], 7}, {NULL}};
const Segment SGI307541[] = {{&S5011[35], 14}, {NULL}};
const Segment SGI307542[] = {{&S5011[71], 11}, {NULL}};
const Segment SGI307543[] = {{&S5011[164], 15}, {NULL}};
const Segment SGI307544[] = {{&S5011[51], 6}, {NULL}};
const Segment SGI307545[] = {{&S5011[71], 16}, {NULL}};
const Segment SGI307546[] = {{&S5014[30], 3}, {NULL}};
const Segment SGI307547[] = {{&S5014[8], 3}, {NULL}};
const Segment SGI307548[] = {{&S5014[13], 10}, {NULL}};
const Segment SGI307549[] = {{&S5014[42], 7}, {NULL}};
const Segment SGI307550[] = {{&S5014[35], 14}, {NULL}};
const Segment SGI307551[] = {{&S5014[76], 11}, {NULL}};
const Segment SGI307552[] = {{&S5014[169], 15}, {NULL}};
const Segment SGI307553[] = {{&S5014[76], 16}, {NULL}};
const Segment SGI307554[] = {{&S5014[51], 5}, {NULL}};
const Segment SGI307555[] = {{&S5015[30], 3}, {NULL}};
const Segment SGI307556[] = {{&S5015[8], 3}, {NULL}};
const Segment SGI307557[] = {{&S5015[13], 10}, {NULL}};
const Segment SGI307558[] = {{&S5015[56], 7}, {NULL}};
const Segment SGI307559[] = {{&S5015[49], 14}, {NULL}};
const Segment SGI307560[] = {{&S5015[99], 11}, {NULL}};
const Segment SGI307561[] = {{&S5015[65], 6}, {NULL}};
const Segment SGI307562[] = {{&S5015[99], 16}, {NULL}};
const Segment SGI307563[] = {{&S5017[30], 3}, {NULL}};
const Segment SGI307564[] = {{&S5017[235], 9}, {NULL}};
const Segment SGI307565[] = {{&S5017[8], 3}, {NULL}};
const Segment SGI307566[] = {{&S5017[13], 10}, {NULL}};
const Segment SGI307567[] = {{&S5017[42], 7}, {NULL}};
const Segment SGI307568[] = {{&S5017[35], 14}, {NULL}};
const Segment SGI307569[] = {{&S5017[71], 11}, {NULL}};
const Segment SGI307570[] = {{&S5017[164], 15}, {NULL}};
const Segment SGI307571[] = {{&S5017[51], 6}, {NULL}};
const Segment SGI307572[] = {{&S5017[71], 16}, {NULL}};
const Segment SGI307573[] = {{&S5018[30], 3}, {&S5018[30], 3}, {NULL}};
const Segment SGI307574[] = {{&S5018[8], 3}, {NULL}};
const Segment SGI307575[] = {{&S5018[13], 10}, {NULL}};
const Segment SGI307576[] = {{&S5018[42], 7}, {NULL}};
const Segment SGI307577[] = {{&S5018[35], 14}, {NULL}};
const Segment SGI307578[] = {{&S5018[64], 11}, {NULL}};
const Segment SGI307579[] = {{&S5018[157], 15}, {NULL}};
const Segment SGI307580[] = {{&S5018[64], 16}, {NULL}};
const Segment SGI307581[] = {{&S5264[30], 3}, {NULL}};
const Segment SGI307582[] = {{&S5264[8], 3}, {NULL}};
const Segment SGI307583[] = {{&S5264[13], 10}, {NULL}};
const Segment SGI307584[] = {{&S5264[42], 7}, {NULL}};
const Segment SGI307585[] = {{&S5264[35], 14}, {NULL}};
const Segment SGI307586[] = {{&S5264[71], 11}, {NULL}};
const Segment SGI307587[] = {{&S5264[51], 6}, {NULL}};
const Segment SGI307588[] = {{&S5264[71], 16}, {NULL}};
const Segment SGI307589[] = {{&S5267[30], 4}, {NULL}};
const Segment SGI307590[] = {{&S5267[8], 3}, {NULL}};
const Segment SGI307591[] = {{&S5267[13], 10}, {NULL}};
const Segment SGI307592[] = {{&S5267[43], 7}, {NULL}};
const Segment SGI307593[] = {{&S5267[36], 14}, {NULL}};
const Segment SGI307594[] = {{&S5267[72], 11}, {NULL}};
const Segment SGI307595[] = {{&S5267[52], 6}, {NULL}};
const Segment SGI307596[] = {{&S5267[72], 16}, {NULL}};
const Segment SGI307597[] = {{&S5268[30], 3}, {NULL}};
const Segment SGI307598[] = {{&S5268[8], 3}, {NULL}};
const Segment SGI307599[] = {{&S5268[13], 10}, {NULL}};
const Segment SGI307600[] = {{&S5268[42], 7}, {NULL}};
const Segment SGI307601[] = {{&S5268[35], 14}, {NULL}};
const Segment SGI307602[] = {{&S5268[71], 11}, {NULL}};
const Segment SGI307603[] = {{&S5268[51], 6}, {NULL}};
const Segment SGI307604[] = {{&S5268[71], 16}, {NULL}};
const Segment SGI307605[] = {{&S5269[30], 3}, {NULL}};
const Segment SGI307606[] = {{&S5269[228], 9}, {NULL}};
const Segment SGI307607[] = {{&S5269[8], 3}, {NULL}};
const Segment SGI307608[] = {{&S5269[13], 10}, {NULL}};
const Segment SGI307609[] = {{&S5269[42], 7}, {NULL}};
const Segment SGI307610[] = {{&S5269[35], 14}, {NULL}};
const Segment SGI307611[] = {{&S5269[64], 11}, {NULL}};
const Segment SGI307612[] = {{&S5269[157], 15}, {NULL}};
const Segment SGI307613[] = {{&S5269[64], 16}, {NULL}};
const Segment SGI307614[] = {{&S5271[30], 4}, {NULL}};
const Segment SGI307615[] = {{&S5271[8], 3}, {NULL}};
const Segment SGI307616[] = {{&S5271[13], 10}, {NULL}};
const Segment SGI307617[] = {{&S5271[43], 7}, {NULL}};
const Segment SGI307618[] = {{&S5271[36], 14}, {NULL}};
const Segment SGI307619[] = {{&S5271[65], 11}, {NULL}};
const Segment SGI307620[] = {{&S5271[65], 16}, {NULL}};
const Segment SGI307621[] = {{&S5272[30], 3}, {NULL}};
const Segment SGI307622[] = {{&S5272[8], 3}, {NULL}};
const Segment SGI307623[] = {{&S5272[13], 10}, {NULL}};
const Segment SGI307624[] = {{&S5272[42], 7}, {NULL}};
const Segment SGI307625[] = {{&S5272[35], 14}, {NULL}};
const Segment SGI307626[] = {{&S5272[64], 11}, {NULL}};
const Segment SGI307627[] = {{&S5272[64], 16}, {NULL}};
const Segment SGI307628[] = {{&S5272[177], 9}, {NULL}};
const Segment SGI307629[] = {{&S5273[30], 3}, {NULL}};
const Segment SGI307630[] = {{&S5273[8], 3}, {NULL}};
const Segment SGI307631[] = {{&S5273[13], 10}, {NULL}};
const Segment SGI307632[] = {{&S5273[42], 7}, {NULL}};
const Segment SGI307633[] = {{&S5273[35], 14}, {NULL}};
const Segment SGI307634[] = {{&S5273[71], 11}, {NULL}};
const Segment SGI307635[] = {{&S5273[51], 6}, {NULL}};
const Segment SGI307636[] = {{&S5273[71], 16}, {NULL}};
const Segment SGI307637[] = {{&S5274[30], 3}, {NULL}};
const Segment SGI307638[] = {{&S5274[8], 3}, {NULL}};
const Segment SGI307639[] = {{&S5274[13], 10}, {NULL}};
const Segment SGI307640[] = {{&S5274[42], 7}, {NULL}};
const Segment SGI307641[] = {{&S5274[35], 14}, {NULL}};
const Segment SGI307642[] = {{&S5274[71], 11}, {NULL}};
const Segment SGI307643[] = {{&S5274[164], 15}, {NULL}};
const Segment SGI307644[] = {{&S5274[51], 6}, {NULL}};
const Segment SGI307645[] = {{&S5274[71], 16}, {NULL}};
const Segment SGI307646[] = {{&S5275[30], 3}, {NULL}};
const Segment SGI307647[] = {{&S5275[8], 3}, {NULL}};
const Segment SGI307648[] = {{&S5275[13], 10}, {NULL}};
const Segment SGI307649[] = {{&S5275[42], 7}, {NULL}};
const Segment SGI307650[] = {{&S5275[35], 14}, {NULL}};
const Segment SGI307651[] = {{&S5275[64], 11}, {NULL}};
const Segment SGI307652[] = {{&S5275[157], 15}, {NULL}};
const Segment SGI307653[] = {{&S5275[64], 16}, {NULL}};
const Segment SGI307654[] = {{&S5276[30], 3}, {NULL}};
const Segment SGI307655[] = {{&S5276[8], 3}, {NULL}};
const Segment SGI307656[] = {{&S5276[13], 10}, {NULL}};
const Segment SGI307657[] = {{&S5276[42], 7}, {NULL}};
const Segment SGI307658[] = {{&S5276[35], 14}, {NULL}};
const Segment SGI307659[] = {{&S5276[64], 11}, {NULL}};
const Segment SGI307660[] = {{&S5276[64], 16}, {NULL}};
const Segment SGI307661[] = {{&S5277[30], 3}, {NULL}};
const Segment SGI307662[] = {{&S5277[8], 3}, {NULL}};
const Segment SGI307663[] = {{&S5277[13], 10}, {NULL}};
const Segment SGI307664[] = {{&S5277[42], 7}, {NULL}};
const Segment SGI307665[] = {{&S5277[35], 14}, {NULL}};
const Segment SGI307666[] = {{&S5277[71], 11}, {NULL}};
const Segment SGI307667[] = {{&S5277[164], 15}, {NULL}};
const Segment SGI307668[] = {{&S5277[51], 6}, {NULL}};
const Segment SGI307669[] = {{&S5277[71], 16}, {NULL}};
const Segment SGI307670[] = {{&S5278[30], 3}, {NULL}};
const Segment SGI307671[] = {{&S5278[8], 3}, {NULL}};
const Segment SGI307672[] = {{&S5278[13], 10}, {NULL}};
const Segment SGI307673[] = {{&S5278[42], 7}, {NULL}};
const Segment SGI307674[] = {{&S5278[35], 14}, {NULL}};
const Segment SGI307675[] = {{&S5278[64], 11}, {NULL}};
const Segment SGI307676[] = {{&S5278[157], 15}, {NULL}};
const Segment SGI307677[] = {{&S5278[64], 16}, {NULL}};
const Segment SGI307678[] = {{&S5280[30], 3}, {NULL}};
const Segment SGI307679[] = {{&S5280[8], 3}, {NULL}};
const Segment SGI307680[] = {{&S5280[13], 10}, {NULL}};
const Segment SGI307681[] = {{&S5280[42], 7}, {NULL}};
const Segment SGI307682[] = {{&S5280[35], 14}, {NULL}};
const Segment SGI307683[] = {{&S5280[76], 11}, {NULL}};
const Segment SGI307684[] = {{&S5280[169], 15}, {NULL}};
const Segment SGI307685[] = {{&S5280[76], 16}, {NULL}};
const Segment SGI307686[] = {{&S5280[51], 5}, {NULL}};
const Segment SGI307687[] = {{&S5282[30], 3}, {NULL}};
const Segment SGI307688[] = {{&S5282[8], 3}, {NULL}};
const Segment SGI307689[] = {{&S5282[13], 10}, {NULL}};
const Segment SGI307690[] = {{&S5282[42], 7}, {NULL}};
const Segment SGI307691[] = {{&S5282[35], 14}, {NULL}};
const Segment SGI307692[] = {{&S5282[64], 11}, {NULL}};
const Segment SGI307693[] = {{&S5282[157], 15}, {NULL}};
const Segment SGI307694[] = {{&S5282[64], 16}, {NULL}};
const Segment SGI307695[] = {{&S5282[198], 9}, {NULL}};
const Segment SGI307696[] = {{&S5283[30], 3}, {NULL}};
const Segment SGI307697[] = {{&S5283[215], 9}, {NULL}};
const Segment SGI307698[] = {{&S5283[8], 3}, {NULL}};
const Segment SGI307699[] = {{&S5283[13], 10}, {NULL}};
const Segment SGI307700[] = {{&S5283[42], 7}, {NULL}};
const Segment SGI307701[] = {{&S5283[35], 14}, {NULL}};
const Segment SGI307702[] = {{&S5283[71], 11}, {NULL}};
const Segment SGI307703[] = {{&S5283[164], 15}, {NULL}};
const Segment SGI307704[] = {{&S5283[51], 6}, {NULL}};
const Segment SGI307705[] = {{&S5283[71], 16}, {NULL}};
const Segment SGI307706[] = {{&S5284[30], 3}, {NULL}};
const Segment SGI307707[] = {{&S5284[8], 3}, {NULL}};
const Segment SGI307708[] = {{&S5284[13], 10}, {NULL}};
const Segment SGI307709[] = {{&S5284[42], 7}, {NULL}};
const Segment SGI307710[] = {{&S5284[35], 14}, {NULL}};
const Segment SGI307711[] = {{&S5284[64], 11}, {NULL}};
const Segment SGI307712[] = {{&S5284[157], 15}, {NULL}};
const Segment SGI307713[] = {{&S5284[64], 16}, {NULL}};
const Segment SGI307714[] = {{&S5284[178], 9}, {NULL}};
const Segment SGI307715[] = {{&S5285[30], 3}, {&S5285[30], 3}, {NULL}};
const Segment SGI307716[] = {{&S5285[8], 3}, {NULL}};
const Segment SGI307717[] = {{&S5285[13], 10}, {NULL}};
const Segment SGI307718[] = {{&S5285[42], 7}, {NULL}};
const Segment SGI307719[] = {{&S5285[35], 14}, {NULL}};
const Segment SGI307720[] = {{&S5285[64], 11}, {NULL}};
const Segment SGI307721[] = {{&S5285[157], 15}, {NULL}};
const Segment SGI307722[] = {{&S5285[64], 16}, {NULL}};
const Segment SGI307723[] = {{&S5286[30], 3}, {NULL}};
const Segment SGI307724[] = {{&S5286[8], 3}, {NULL}};
const Segment SGI307725[] = {{&S5286[13], 10}, {NULL}};
const Segment SGI307726[] = {{&S5286[42], 7}, {NULL}};
const Segment SGI307727[] = {{&S5286[35], 14}, {NULL}};
const Segment SGI307728[] = {{&S5286[76], 11}, {NULL}};
const Segment SGI307729[] = {{&S5286[169], 15}, {NULL}};
const Segment SGI307730[] = {{&S5286[76], 16}, {NULL}};
const Segment SGI307731[] = {{&S5286[51], 5}, {NULL}};
const Segment SGI307732[] = {{&S5286[210], 9}, {NULL}};
const Segment SGI307733[] = {{&S5287[30], 3}, {NULL}};
const Segment SGI307734[] = {{&S5287[8], 3}, {NULL}};
const Segment SGI307735[] = {{&S5287[13], 10}, {NULL}};
const Segment SGI307736[] = {{&S5287[42], 7}, {NULL}};
const Segment SGI307737[] = {{&S5287[35], 14}, {NULL}};
const Segment SGI307738[] = {{&S5287[64], 11}, {NULL}};
const Segment SGI307739[] = {{&S5287[64], 16}, {NULL}};
const Segment SGI307740[] = {{&S5288[30], 3}, {NULL}};
const Segment SGI307741[] = {{&S5288[8], 3}, {NULL}};
const Segment SGI307742[] = {{&S5288[13], 10}, {NULL}};
const Segment SGI307743[] = {{&S5288[42], 7}, {NULL}};
const Segment SGI307744[] = {{&S5288[35], 14}, {NULL}};
const Segment SGI307745[] = {{&S5288[64], 11}, {NULL}};
const Segment SGI307746[] = {{&S5288[64], 16}, {NULL}};
const Segment SGI307747[] = {{&S5289[30], 4}, {NULL}};
const Segment SGI307748[] = {{&S5289[8], 3}, {NULL}};
const Segment SGI307749[] = {{&S5289[13], 10}, {NULL}};
const Segment SGI307750[] = {{&S5289[43], 7}, {NULL}};
const Segment SGI307751[] = {{&S5289[36], 14}, {NULL}};
const Segment SGI307752[] = {{&S5289[87], 11}, {NULL}};
const Segment SGI307753[] = {{&S5289[52], 6}, {NULL}};
const Segment SGI307754[] = {{&S5289[72], 13}, {NULL}};
const Segment SGI307755[] = {{&S5289[87], 16}, {NULL}};
const Segment SGI307756[] = {{&S5290[30], 3}, {NULL}};
const Segment SGI307757[] = {{&S5290[227], 9}, {NULL}};
const Segment SGI307758[] = {{&S5290[8], 3}, {NULL}};
const Segment SGI307759[] = {{&S5290[13], 10}, {NULL}};
const Segment SGI307760[] = {{&S5290[42], 7}, {NULL}};
const Segment SGI307761[] = {{&S5290[35], 14}, {NULL}};
const Segment SGI307762[] = {{&S5290[71], 11}, {NULL}};
const Segment SGI307763[] = {{&S5290[51], 6}, {NULL}};
const Segment SGI307764[] = {{&S5290[71], 16}, {NULL}};
const Segment SGI307765[] = {{&S5291[30], 3}, {NULL}};
const Segment SGI307766[] = {{&S5291[8], 3}, {NULL}};
const Segment SGI307767[] = {{&S5291[13], 10}, {NULL}};
const Segment SGI307768[] = {{&S5291[42], 7}, {NULL}};
const Segment SGI307769[] = {{&S5291[35], 14}, {NULL}};
const Segment SGI307770[] = {{&S5291[76], 11}, {NULL}};
const Segment SGI307771[] = {{&S5291[76], 16}, {NULL}};
const Segment SGI307772[] = {{&S5291[51], 5}, {NULL}};
const Segment SGI307773[] = {{&S5292[30], 3}, {NULL}};
const Segment SGI307774[] = {{&S5292[235], 9}, {NULL}};
const Segment SGI307775[] = {{&S5292[8], 3}, {NULL}};
const Segment SGI307776[] = {{&S5292[13], 10}, {NULL}};
const Segment SGI307777[] = {{&S5292[42], 7}, {NULL}};
const Segment SGI307778[] = {{&S5292[35], 14}, {NULL}};
const Segment SGI307779[] = {{&S5292[71], 11}, {NULL}};
const Segment SGI307780[] = {{&S5292[164], 15}, {NULL}};
const Segment SGI307781[] = {{&S5292[51], 6}, {NULL}};
const Segment SGI307782[] = {{&S5292[71], 16}, {NULL}};
const Segment SGI307783[] = {{&S5293[30], 3}, {NULL}};
const Segment SGI307784[] = {{&S5293[8], 3}, {NULL}};
const Segment SGI307785[] = {{&S5293[13], 10}, {NULL}};
const Segment SGI307786[] = {{&S5293[56], 7}, {NULL}};
const Segment SGI307787[] = {{&S5293[49], 14}, {NULL}};
const Segment SGI307788[] = {{&S5293[99], 11}, {NULL}};
const Segment SGI307789[] = {{&S5293[65], 6}, {NULL}};
const Segment SGI307790[] = {{&S5293[99], 16}, {NULL}};
const Segment SGI307791[] = {{&S5620[30], 3}, {&S5620[113], 3}, {NULL}};
const Segment SGI307792[] = {{&S5620[42], 7}, {&S5620[125], 7}, {&S5620[171], 7}, {NULL}};
const Segment SGI307793[] = {{&S5620[51], 6}, {NULL}};
const Segment SGI307794[] = {{&S5620[134], 3}, {NULL}};
const Segment SGI307795[] = {{&S5637[30], 3}, {NULL}};
const Segment SGI307796[] = {{&S5637[8], 3}, {NULL}};
const Segment SGI307797[] = {{&S5637[13], 10}, {NULL}};
const Segment SGI307798[] = {{&S5637[42], 7}, {NULL}};
const Segment SGI307799[] = {{&S5637[35], 14}, {NULL}};
const Segment SGI307800[] = {{&S5637[71], 11}, {NULL}};
const Segment SGI307801[] = {{&S5637[164], 15}, {NULL}};
const Segment SGI307802[] = {{&S5637[51], 6}, {NULL}};
const Segment SGI307803[] = {{&S5637[71], 16}, {NULL}};
const Segment SGI307804[] = {{&S5639[30], 3}, {NULL}};
const Segment SGI307805[] = {{&S5639[8], 3}, {NULL}};
const Segment SGI307806[] = {{&S5639[13], 10}, {NULL}};
const Segment SGI307807[] = {{&S5639[42], 7}, {NULL}};
const Segment SGI307808[] = {{&S5639[35], 14}, {NULL}};
const Segment SGI307809[] = {{&S5639[71], 11}, {NULL}};
const Segment SGI307810[] = {{&S5639[164], 15}, {NULL}};
const Segment SGI307811[] = {{&S5639[51], 6}, {NULL}};
const Segment SGI307812[] = {{&S5639[71], 16}, {NULL}};
const Segment SGI307813[] = {{&S5641[30], 3}, {NULL}};
const Segment SGI307814[] = {{&S5641[8], 3}, {NULL}};
const Segment SGI307815[] = {{&S5641[13], 10}, {NULL}};
const Segment SGI307816[] = {{&S5641[42], 7}, {NULL}};
const Segment SGI307817[] = {{&S5641[35], 14}, {NULL}};
const Segment SGI307818[] = {{&S5641[76], 11}, {NULL}};
const Segment SGI307819[] = {{&S5641[169], 15}, {NULL}};
const Segment SGI307820[] = {{&S5641[76], 16}, {NULL}};
const Segment SGI307821[] = {{&S5641[51], 5}, {NULL}};
const Segment SGI307822[] = {{&S5645[30], 3}, {NULL}};
const Segment SGI307823[] = {{&S5645[8], 3}, {NULL}};
const Segment SGI307824[] = {{&S5645[13], 10}, {NULL}};
const Segment SGI307825[] = {{&S5645[42], 7}, {NULL}};
const Segment SGI307826[] = {{&S5645[35], 14}, {NULL}};
const Segment SGI307827[] = {{&S5645[64], 11}, {NULL}};
const Segment SGI307828[] = {{&S5645[64], 16}, {NULL}};
const Segment SGI307829[] = {{&S5646[30], 3}, {NULL}};
const Segment SGI307830[] = {{&S5646[8], 3}, {NULL}};
const Segment SGI307831[] = {{&S5646[13], 10}, {NULL}};
const Segment SGI307832[] = {{&S5646[42], 7}, {NULL}};
const Segment SGI307833[] = {{&S5646[35], 14}, {NULL}};
const Segment SGI307834[] = {{&S5646[71], 11}, {NULL}};
const Segment SGI307835[] = {{&S5646[51], 6}, {NULL}};
const Segment SGI307836[] = {{&S5646[71], 16}, {NULL}};
const Segment SGI307837[] = {{&S5647[30], 4}, {NULL}};
const Segment SGI307838[] = {{&S5647[8], 3}, {NULL}};
const Segment SGI307839[] = {{&S5647[13], 10}, {NULL}};
const Segment SGI307840[] = {{&S5647[43], 7}, {NULL}};
const Segment SGI307841[] = {{&S5647[36], 14}, {NULL}};
const Segment SGI307842[] = {{&S5647[72], 11}, {NULL}};
const Segment SGI307843[] = {{&S5647[52], 6}, {NULL}};
const Segment SGI307844[] = {{&S5647[72], 16}, {NULL}};
const Segment SGI307845[] = {{&S5649[30], 4}, {NULL}};
const Segment SGI307846[] = {{&S5649[8], 3}, {NULL}};
const Segment SGI307847[] = {{&S5649[13], 10}, {NULL}};
const Segment SGI307848[] = {{&S5649[43], 7}, {NULL}};
const Segment SGI307849[] = {{&S5649[36], 14}, {NULL}};
const Segment SGI307850[] = {{&S5649[65], 11}, {NULL}};
const Segment SGI307851[] = {{&S5649[65], 16}, {NULL}};
const Segment SGI307852[] = {{&S5651[30], 3}, {NULL}};
const Segment SGI307853[] = {{&S5651[8], 3}, {NULL}};
const Segment SGI307854[] = {{&S5651[13], 10}, {NULL}};
const Segment SGI307855[] = {{&S5651[42], 7}, {NULL}};
const Segment SGI307856[] = {{&S5651[35], 14}, {NULL}};
const Segment SGI307857[] = {{&S5651[64], 11}, {NULL}};
const Segment SGI307858[] = {{&S5651[158], 15}, {NULL}};
const Segment SGI307859[] = {{&S5651[64], 16}, {NULL}};
const Segment SGI307860[] = {{&S5652[30], 3}, {NULL}};
const Segment SGI307861[] = {{&S5652[8], 3}, {NULL}};
const Segment SGI307862[] = {{&S5652[13], 10}, {NULL}};
const Segment SGI307863[] = {{&S5652[42], 7}, {NULL}};
const Segment SGI307864[] = {{&S5652[35], 14}, {NULL}};
const Segment SGI307865[] = {{&S5652[64], 11}, {NULL}};
const Segment SGI307866[] = {{&S5652[158], 15}, {NULL}};
const Segment SGI307867[] = {{&S5652[64], 16}, {NULL}};
const Segment SGI307868[] = {{&S5653[30], 3}, {NULL}};
const Segment SGI307869[] = {{&S5653[8], 3}, {NULL}};
const Segment SGI307870[] = {{&S5653[13], 10}, {NULL}};
const Segment SGI307871[] = {{&S5653[42], 7}, {NULL}};
const Segment SGI307872[] = {{&S5653[35], 14}, {NULL}};
const Segment SGI307873[] = {{&S5653[71], 11}, {NULL}};
const Segment SGI307874[] = {{&S5653[51], 6}, {NULL}};
const Segment SGI307875[] = {{&S5653[71], 16}, {NULL}};
const Segment SGI307876[] = {{&S5654[30], 3}, {NULL}};
const Segment SGI307877[] = {{&S5654[8], 3}, {NULL}};
const Segment SGI307878[] = {{&S5654[13], 10}, {NULL}};
const Segment SGI307879[] = {{&S5654[42], 7}, {NULL}};
const Segment SGI307880[] = {{&S5654[35], 14}, {NULL}};
const Segment SGI307881[] = {{&S5654[64], 11}, {NULL}};
const Segment SGI307882[] = {{&S5654[158], 15}, {NULL}};
const Segment SGI307883[] = {{&S5654[64], 16}, {NULL}};
const Segment SGI307884[] = {{&S5654[199], 9}, {NULL}};
const Segment SGI307885[] = {{&S5655[30], 3}, {NULL}};
const Segment SGI307886[] = {{&S5655[8], 3}, {NULL}};
const Segment SGI307887[] = {{&S5655[13], 10}, {NULL}};
const Segment SGI307888[] = {{&S5655[42], 7}, {NULL}};
const Segment SGI307889[] = {{&S5655[35], 14}, {NULL}};
const Segment SGI307890[] = {{&S5655[64], 11}, {NULL}};
const Segment SGI307891[] = {{&S5655[64], 16}, {NULL}};
const Segment SGI307892[] = {{&S5656[30], 3}, {NULL}};
const Segment SGI307893[] = {{&S5656[8], 3}, {NULL}};
const Segment SGI307894[] = {{&S5656[13], 10}, {NULL}};
const Segment SGI307895[] = {{&S5656[42], 7}, {NULL}};
const Segment SGI307896[] = {{&S5656[35], 14}, {NULL}};
const Segment SGI307897[] = {{&S5656[64], 11}, {NULL}};
const Segment SGI307898[] = {{&S5656[64], 16}, {NULL}};
const Segment SGI307899[] = {{&S5656[178], 9}, {NULL}};
const Segment SGI307900[] = {{&S5657[30], 3}, {NULL}};
const Segment SGI307901[] = {{&S5657[8], 3}, {NULL}};
const Segment SGI307902[] = {{&S5657[13], 10}, {NULL}};
const Segment SGI307903[] = {{&S5657[42], 7}, {NULL}};
const Segment SGI307904[] = {{&S5657[35], 14}, {NULL}};
const Segment SGI307905[] = {{&S5657[71], 11}, {NULL}};
const Segment SGI307906[] = {{&S5657[51], 6}, {NULL}};
const Segment SGI307907[] = {{&S5657[71], 16}, {NULL}};
const Segment SGI307908[] = {{&S5658[30], 3}, {NULL}};
const Segment SGI307909[] = {{&S5658[8], 3}, {NULL}};
const Segment SGI307910[] = {{&S5658[13], 10}, {NULL}};
const Segment SGI307911[] = {{&S5658[42], 7}, {NULL}};
const Segment SGI307912[] = {{&S5658[35], 14}, {NULL}};
const Segment SGI307913[] = {{&S5658[64], 11}, {NULL}};
const Segment SGI307914[] = {{&S5658[158], 15}, {NULL}};
const Segment SGI307915[] = {{&S5658[64], 16}, {NULL}};
const Segment SGI307916[] = {{&S5658[199], 9}, {NULL}};
const Segment SGI307917[] = {{&S5659[30], 3}, {NULL}};
const Segment SGI307918[] = {{&S5659[8], 3}, {NULL}};
const Segment SGI307919[] = {{&S5659[13], 10}, {NULL}};
const Segment SGI307920[] = {{&S5659[42], 7}, {NULL}};
const Segment SGI307921[] = {{&S5659[35], 14}, {NULL}};
const Segment SGI307922[] = {{&S5659[64], 11}, {NULL}};
const Segment SGI307923[] = {{&S5659[64], 16}, {NULL}};
const Segment SGI307924[] = {{&S5660[30], 3}, {NULL}};
const Segment SGI307925[] = {{&S5660[236], 9}, {NULL}};
const Segment SGI307926[] = {{&S5660[8], 3}, {NULL}};
const Segment SGI307927[] = {{&S5660[13], 10}, {NULL}};
const Segment SGI307928[] = {{&S5660[42], 7}, {NULL}};
const Segment SGI307929[] = {{&S5660[35], 14}, {NULL}};
const Segment SGI307930[] = {{&S5660[71], 11}, {NULL}};
const Segment SGI307931[] = {{&S5660[165], 15}, {NULL}};
const Segment SGI307932[] = {{&S5660[51], 6}, {NULL}};
const Segment SGI307933[] = {{&S5660[71], 16}, {NULL}};
const Segment SGI307934[] = {{&S5661[30], 3}, {NULL}};
const Segment SGI307935[] = {{&S5661[8], 3}, {NULL}};
const Segment SGI307936[] = {{&S5661[13], 10}, {NULL}};
const Segment SGI307937[] = {{&S5661[42], 7}, {NULL}};
const Segment SGI307938[] = {{&S5661[35], 14}, {NULL}};
const Segment SGI307939[] = {{&S5661[76], 11}, {NULL}};
const Segment SGI307940[] = {{&S5661[170], 15}, {NULL}};
const Segment SGI307941[] = {{&S5661[76], 16}, {NULL}};
const Segment SGI307942[] = {{&S5661[51], 5}, {NULL}};
const Segment SGI307943[] = {{&S5661[211], 9}, {NULL}};
const Segment SGI307944[] = {{&S5662[30], 3}, {NULL}};
const Segment SGI307945[] = {{&S5662[235], 9}, {NULL}};
const Segment SGI307946[] = {{&S5662[8], 3}, {NULL}};
const Segment SGI307947[] = {{&S5662[13], 10}, {NULL}};
const Segment SGI307948[] = {{&S5662[42], 7}, {NULL}};
const Segment SGI307949[] = {{&S5662[35], 14}, {NULL}};
const Segment SGI307950[] = {{&S5662[71], 11}, {NULL}};
const Segment SGI307951[] = {{&S5662[164], 15}, {NULL}};
const Segment SGI307952[] = {{&S5662[51], 6}, {NULL}};
const Segment SGI307953[] = {{&S5662[71], 16}, {NULL}};
const Segment SGI307954[] = {{&S5757[30], 3}, {NULL}};
const Segment SGI307955[] = {{&S5757[8], 3}, {NULL}};
const Segment SGI307956[] = {{&S5757[13], 10}, {NULL}};
const Segment SGI307957[] = {{&S5757[42], 7}, {NULL}};
const Segment SGI307958[] = {{&S5757[35], 14}, {NULL}};
const Segment SGI307959[] = {{&S5757[71], 11}, {NULL}};
const Segment SGI307960[] = {{&S5757[51], 6}, {NULL}};
const Segment SGI307961[] = {{&S5757[71], 16}, {NULL}};
const Segment SGI307962[] = {{&S5758[30], 3}, {NULL}};
const Segment SGI307963[] = {{&S5758[8], 3}, {NULL}};
const Segment SGI307964[] = {{&S5758[13], 10}, {NULL}};
const Segment SGI307965[] = {{&S5758[42], 7}, {NULL}};
const Segment SGI307966[] = {{&S5758[35], 14}, {NULL}};
const Segment SGI307967[] = {{&S5758[64], 11}, {NULL}};
const Segment SGI307968[] = {{&S5758[157], 15}, {NULL}};
const Segment SGI307969[] = {{&S5758[64], 16}, {NULL}};
const Segment SGI307970[] = {{&S5759[30], 3}, {NULL}};
const Segment SGI307971[] = {{&S5759[8], 3}, {NULL}};
const Segment SGI307972[] = {{&S5759[13], 10}, {NULL}};
const Segment SGI307973[] = {{&S5759[42], 7}, {NULL}};
const Segment SGI307974[] = {{&S5759[35], 14}, {NULL}};
const Segment SGI307975[] = {{&S5759[64], 11}, {NULL}};
const Segment SGI307976[] = {{&S5759[64], 16}, {NULL}};
const Segment SGI307977[] = {{&S5759[177], 9}, {NULL}};
const Segment SGI307978[] = {{&S5760[30], 3}, {NULL}};
const Segment SGI307979[] = {{&S5760[8], 3}, {NULL}};
const Segment SGI307980[] = {{&S5760[13], 10}, {NULL}};
const Segment SGI307981[] = {{&S5760[42], 7}, {NULL}};
const Segment SGI307982[] = {{&S5760[35], 14}, {NULL}};
const Segment SGI307983[] = {{&S5760[71], 11}, {NULL}};
const Segment SGI307984[] = {{&S5760[51], 6}, {NULL}};
const Segment SGI307985[] = {{&S5760[71], 16}, {NULL}};
const Segment SGI307986[] = {{&S5761[30], 3}, {NULL}};
const Segment SGI307987[] = {{&S5761[8], 3}, {NULL}};
const Segment SGI307988[] = {{&S5761[13], 10}, {NULL}};
const Segment SGI307989[] = {{&S5761[42], 7}, {NULL}};
const Segment SGI307990[] = {{&S5761[35], 14}, {NULL}};
const Segment SGI307991[] = {{&S5761[71], 11}, {NULL}};
const Segment SGI307992[] = {{&S5761[164], 15}, {NULL}};
const Segment SGI307993[] = {{&S5761[51], 6}, {NULL}};
const Segment SGI307994[] = {{&S5761[71], 16}, {NULL}};
const Segment SGI307995[] = {{&S5762[30], 3}, {NULL}};
const Segment SGI307996[] = {{&S5762[8], 3}, {NULL}};
const Segment SGI307997[] = {{&S5762[13], 10}, {NULL}};
const Segment SGI307998[] = {{&S5762[42], 7}, {NULL}};
const Segment SGI307999[] = {{&S5762[35], 14}, {NULL}};
const Segment SGI308000[] = {{&S5762[64], 11}, {NULL}};
const Segment SGI308001[] = {{&S5762[64], 16}, {NULL}};
const Segment SGI308002[] = {{&S5763[30], 3}, {NULL}};
const Segment SGI308003[] = {{&S5763[8], 3}, {NULL}};
const Segment SGI308004[] = {{&S5763[13], 10}, {NULL}};
const Segment SGI308005[] = {{&S5763[42], 7}, {NULL}};
const Segment SGI308006[] = {{&S5763[35], 14}, {NULL}};
const Segment SGI308007[] = {{&S5763[64], 11}, {NULL}};
const Segment SGI308008[] = {{&S5763[64], 16}, {NULL}};
const Segment SGI308009[] = {{&S5764[30], 3}, {NULL}};
const Segment SGI308010[] = {{&S5764[8], 3}, {NULL}};
const Segment SGI308011[] = {{&S5764[13], 10}, {NULL}};
const Segment SGI308012[] = {{&S5764[42], 7}, {NULL}};
const Segment SGI308013[] = {{&S5764[35], 14}, {NULL}};
const Segment SGI308014[] = {{&S5764[71], 11}, {NULL}};
const Segment SGI308015[] = {{&S5764[51], 6}, {NULL}};
const Segment SGI308016[] = {{&S5764[71], 16}, {NULL}};
const Segment SGI308017[] = {{&S5766[30], 3}, {NULL}};
const Segment SGI308018[] = {{&S5766[8], 3}, {NULL}};
const Segment SGI308019[] = {{&S5766[13], 10}, {NULL}};
const Segment SGI308020[] = {{&S5766[42], 7}, {NULL}};
const Segment SGI308021[] = {{&S5766[35], 14}, {NULL}};
const Segment SGI308022[] = {{&S5766[64], 11}, {NULL}};
const Segment SGI308023[] = {{&S5766[157], 15}, {NULL}};
const Segment SGI308024[] = {{&S5766[64], 16}, {NULL}};
const Segment SGI308025[] = {{&S5768[30], 4}, {NULL}};
const Segment SGI308026[] = {{&S5768[8], 3}, {NULL}};
const Segment SGI308027[] = {{&S5768[13], 10}, {NULL}};
const Segment SGI308028[] = {{&S5768[43], 7}, {NULL}};
const Segment SGI308029[] = {{&S5768[36], 14}, {NULL}};
const Segment SGI308030[] = {{&S5768[72], 11}, {NULL}};
const Segment SGI308031[] = {{&S5768[52], 6}, {NULL}};
const Segment SGI308032[] = {{&S5768[72], 16}, {NULL}};
const Segment SGI308033[] = {{&S5769[30], 4}, {NULL}};
const Segment SGI308034[] = {{&S5769[8], 3}, {NULL}};
const Segment SGI308035[] = {{&S5769[13], 10}, {NULL}};
const Segment SGI308036[] = {{&S5769[43], 7}, {NULL}};
const Segment SGI308037[] = {{&S5769[36], 14}, {NULL}};
const Segment SGI308038[] = {{&S5769[65], 11}, {NULL}};
const Segment SGI308039[] = {{&S5769[65], 16}, {NULL}};
const Segment SGI308040[] = {{&S5770[30], 3}, {NULL}};
const Segment SGI308041[] = {{&S5770[8], 3}, {NULL}};
const Segment SGI308042[] = {{&S5770[13], 10}, {NULL}};
const Segment SGI308043[] = {{&S5770[42], 7}, {NULL}};
const Segment SGI308044[] = {{&S5770[35], 14}, {NULL}};
const Segment SGI308045[] = {{&S5770[64], 11}, {NULL}};
const Segment SGI308046[] = {{&S5770[157], 15}, {NULL}};
const Segment SGI308047[] = {{&S5770[64], 16}, {NULL}};
const Segment SGI308048[] = {{&S5770[200], 9}, {NULL}};
const Segment SGI308049[] = {{&S5771[30], 3}, {NULL}};
const Segment SGI308050[] = {{&S5771[8], 3}, {NULL}};
const Segment SGI308051[] = {{&S5771[13], 10}, {NULL}};
const Segment SGI308052[] = {{&S5771[42], 7}, {NULL}};
const Segment SGI308053[] = {{&S5771[35], 14}, {NULL}};
const Segment SGI308054[] = {{&S5771[64], 11}, {NULL}};
const Segment SGI308055[] = {{&S5771[157], 15}, {NULL}};
const Segment SGI308056[] = {{&S5771[64], 16}, {NULL}};
const Segment SGI308057[] = {{&S5771[200], 9}, {NULL}};
const Segment SGI308058[] = {{&S5772[30], 3}, {NULL}};
const Segment SGI308059[] = {{&S5772[8], 3}, {NULL}};
const Segment SGI308060[] = {{&S5772[13], 10}, {NULL}};
const Segment SGI308061[] = {{&S5772[42], 7}, {NULL}};
const Segment SGI308062[] = {{&S5772[35], 14}, {NULL}};
const Segment SGI308063[] = {{&S5772[71], 11}, {NULL}};
const Segment SGI308064[] = {{&S5772[164], 15}, {NULL}};
const Segment SGI308065[] = {{&S5772[51], 6}, {NULL}};
const Segment SGI308066[] = {{&S5772[71], 16}, {NULL}};
const Segment SGI308067[] = {{&S5773[30], 3}, {NULL}};
const Segment SGI308068[] = {{&S5773[8], 3}, {NULL}};
const Segment SGI308069[] = {{&S5773[13], 10}, {NULL}};
const Segment SGI308070[] = {{&S5773[42], 7}, {NULL}};
const Segment SGI308071[] = {{&S5773[35], 14}, {NULL}};
const Segment SGI308072[] = {{&S5773[76], 11}, {NULL}};
const Segment SGI308073[] = {{&S5773[169], 15}, {NULL}};
const Segment SGI308074[] = {{&S5773[76], 16}, {NULL}};
const Segment SGI308075[] = {{&S5773[51], 5}, {NULL}};
const Segment SGI308076[] = {{&S5774[30], 3}, {NULL}};
const Segment SGI308077[] = {{&S5774[8], 3}, {NULL}};
const Segment SGI308078[] = {{&S5774[13], 10}, {NULL}};
const Segment SGI308079[] = {{&S5774[42], 7}, {NULL}};
const Segment SGI308080[] = {{&S5774[35], 14}, {NULL}};
const Segment SGI308081[] = {{&S5774[64], 11}, {NULL}};
const Segment SGI308082[] = {{&S5774[64], 16}, {NULL}};
const Segment SGI308083[] = {{&S5775[30], 4}, {NULL}};
const Segment SGI308084[] = {{&S5775[8], 3}, {NULL}};
const Segment SGI308085[] = {{&S5775[13], 10}, {NULL}};
const Segment SGI308086[] = {{&S5775[43], 7}, {NULL}};
const Segment SGI308087[] = {{&S5775[36], 14}, {NULL}};
const Segment SGI308088[] = {{&S5775[87], 11}, {NULL}};
const Segment SGI308089[] = {{&S5775[52], 6}, {NULL}};
const Segment SGI308090[] = {{&S5775[72], 13}, {NULL}};
const Segment SGI308091[] = {{&S5775[87], 16}, {NULL}};
const Segment SGI308092[] = {{&S5776[30], 3}, {NULL}};
const Segment SGI308093[] = {{&S5776[8], 3}, {NULL}};
const Segment SGI308094[] = {{&S5776[13], 10}, {NULL}};
const Segment SGI308095[] = {{&S5776[42], 7}, {NULL}};
const Segment SGI308096[] = {{&S5776[35], 14}, {NULL}};
const Segment SGI308097[] = {{&S5776[76], 11}, {NULL}};
const Segment SGI308098[] = {{&S5776[169], 15}, {NULL}};
const Segment SGI308099[] = {{&S5776[76], 16}, {NULL}};
const Segment SGI308100[] = {{&S5776[51], 5}, {NULL}};
const Segment SGI308101[] = {{&S5776[210], 9}, {NULL}};
const Segment SGI308102[] = {{&S5777[30], 3}, {NULL}};
const Segment SGI308103[] = {{&S5777[223], 9}, {NULL}};
const Segment SGI308104[] = {{&S5777[8], 3}, {NULL}};
const Segment SGI308105[] = {{&S5777[13], 10}, {NULL}};
const Segment SGI308106[] = {{&S5777[42], 7}, {NULL}};
const Segment SGI308107[] = {{&S5777[35], 14}, {NULL}};
const Segment SGI308108[] = {{&S5777[71], 11}, {NULL}};
const Segment SGI308109[] = {{&S5777[164], 15}, {NULL}};
const Segment SGI308110[] = {{&S5777[51], 6}, {NULL}};
const Segment SGI308111[] = {{&S5777[71], 16}, {NULL}};
const Segment SGI308112[] = {{&S5778[30], 3}, {NULL}};
const Segment SGI308113[] = {{&S5778[8], 3}, {NULL}};
const Segment SGI308114[] = {{&S5778[13], 10}, {NULL}};
const Segment SGI308115[] = {{&S5778[42], 7}, {NULL}};
const Segment SGI308116[] = {{&S5778[35], 14}, {NULL}};
const Segment SGI308117[] = {{&S5778[71], 11}, {NULL}};
const Segment SGI308118[] = {{&S5778[164], 15}, {NULL}};
const Segment SGI308119[] = {{&S5778[217], 4}, {NULL}};
const Segment SGI308120[] = {{&S5778[51], 6}, {NULL}};
const Segment SGI308121[] = {{&S5778[71], 16}, {NULL}};
const Segment SGI308122[] = {{&S5779[30], 3}, {NULL}};
const Segment SGI308123[] = {{&S5779[8], 3}, {NULL}};
const Segment SGI308124[] = {{&S5779[13], 10}, {NULL}};
const Segment SGI308125[] = {{&S5779[42], 7}, {NULL}};
const Segment SGI308126[] = {{&S5779[35], 14}, {NULL}};
const Segment SGI308127[] = {{&S5779[76], 11}, {NULL}};
const Segment SGI308128[] = {{&S5779[76], 16}, {NULL}};
const Segment SGI308129[] = {{&S5779[51], 5}, {NULL}};
const Segment SGI308130[] = {{&S5781[30], 3}, {NULL}};
const Segment SGI308131[] = {{&S5781[228], 9}, {NULL}};
const Segment SGI308132[] = {{&S5781[8], 3}, {NULL}};
const Segment SGI308133[] = {{&S5781[13], 10}, {NULL}};
const Segment SGI308134[] = {{&S5781[42], 7}, {NULL}};
const Segment SGI308135[] = {{&S5781[35], 14}, {NULL}};
const Segment SGI308136[] = {{&S5781[64], 11}, {NULL}};
const Segment SGI308137[] = {{&S5781[157], 15}, {NULL}};
const Segment SGI308138[] = {{&S5781[64], 16}, {NULL}};
const Segment SGI308139[] = {{&S5782[30], 3}, {NULL}};
const Segment SGI308140[] = {{&S5782[218], 9}, {NULL}};
const Segment SGI308141[] = {{&S5782[8], 3}, {NULL}};
const Segment SGI308142[] = {{&S5782[13], 10}, {NULL}};
const Segment SGI308143[] = {{&S5782[42], 7}, {NULL}};
const Segment SGI308144[] = {{&S5782[35], 14}, {NULL}};
const Segment SGI308145[] = {{&S5782[64], 11}, {NULL}};
const Segment SGI308146[] = {{&S5782[157], 15}, {NULL}};
const Segment SGI308147[] = {{&S5782[64], 16}, {NULL}};
const Segment SGI308148[] = {{&S5783[30], 3}, {NULL}};
const Segment SGI308149[] = {{&S5783[248], 9}, {NULL}};
const Segment SGI308150[] = {{&S5783[8], 3}, {NULL}};
const Segment SGI308151[] = {{&S5783[13], 10}, {NULL}};
const Segment SGI308152[] = {{&S5783[42], 7}, {NULL}};
const Segment SGI308153[] = {{&S5783[35], 14}, {NULL}};
const Segment SGI308154[] = {{&S5783[71], 11}, {NULL}};
const Segment SGI308155[] = {{&S5783[164], 15}, {NULL}};
const Segment SGI308156[] = {{&S5783[51], 6}, {NULL}};
const Segment SGI308157[] = {{&S5783[71], 16}, {NULL}};
const Segment SGI308158[] = {{&S5784[30], 3}, {NULL}};
const Segment SGI308159[] = {{&S5784[228], 9}, {NULL}};
const Segment SGI308160[] = {{&S5784[8], 3}, {NULL}};
const Segment SGI308161[] = {{&S5784[13], 10}, {NULL}};
const Segment SGI308162[] = {{&S5784[42], 7}, {NULL}};
const Segment SGI308163[] = {{&S5784[35], 14}, {NULL}};
const Segment SGI308164[] = {{&S5784[64], 11}, {NULL}};
const Segment SGI308165[] = {{&S5784[157], 15}, {NULL}};
const Segment SGI308166[] = {{&S5784[64], 16}, {NULL}};
const Segment SGI308167[] = {{&S5785[30], 3}, {NULL}};
const Segment SGI308168[] = {{&S5785[248], 9}, {NULL}};
const Segment SGI308169[] = {{&S5785[8], 3}, {NULL}};
const Segment SGI308170[] = {{&S5785[13], 10}, {NULL}};
const Segment SGI308171[] = {{&S5785[42], 7}, {NULL}};
const Segment SGI308172[] = {{&S5785[35], 14}, {NULL}};
const Segment SGI308173[] = {{&S5785[71], 11}, {NULL}};
const Segment SGI308174[] = {{&S5785[164], 15}, {NULL}};
const Segment SGI308175[] = {{&S5785[51], 6}, {NULL}};
const Segment SGI308176[] = {{&S5785[71], 16}, {NULL}};
const Segment SGI308177[] = {{&S5786[30], 3}, {NULL}};
const Segment SGI308178[] = {{&S5786[248], 9}, {NULL}};
const Segment SGI308179[] = {{&S5786[8], 3}, {NULL}};
const Segment SGI308180[] = {{&S5786[13], 10}, {NULL}};
const Segment SGI308181[] = {{&S5786[42], 7}, {NULL}};
const Segment SGI308182[] = {{&S5786[35], 14}, {NULL}};
const Segment SGI308183[] = {{&S5786[71], 11}, {NULL}};
const Segment SGI308184[] = {{&S5786[164], 15}, {NULL}};
const Segment SGI308185[] = {{&S5786[51], 6}, {NULL}};
const Segment SGI308186[] = {{&S5786[71], 16}, {NULL}};
const Segment SGI308187[] = {{&S5790[30], 3}, {NULL}};
const Segment SGI308188[] = {{&S5790[8], 3}, {NULL}};
const Segment SGI308189[] = {{&S5790[13], 10}, {NULL}};
const Segment SGI308190[] = {{&S5790[42], 7}, {NULL}};
const Segment SGI308191[] = {{&S5791[30], 3}, {NULL}};
const Segment SGI308192[] = {{&S5791[8], 3}, {NULL}};
const Segment SGI308193[] = {{&S5791[13], 10}, {NULL}};
const Segment SGI308194[] = {{&S5791[42], 7}, {NULL}};
const Segment SGI308195[] = {{&S5791[35], 14}, {NULL}};
const Segment SGI308196[] = {{&S5791[51], 3}, {NULL}};
const Segment SGI308197[] = {{&S5792[30], 3}, {NULL}};
const Segment SGI308198[] = {{&S5792[8], 3}, {NULL}};
const Segment SGI308199[] = {{&S5792[13], 10}, {NULL}};
const Segment SGI308200[] = {{&S5792[42], 7}, {NULL}};
const Segment SGI308201[] = {{&S5792[35], 14}, {NULL}};
const Segment SGI308202[] = {{&S5793[30], 3}, {NULL}};
const Segment SGI308203[] = {{&S5793[8], 3}, {NULL}};
const Segment SGI308204[] = {{&S5793[13], 10}, {NULL}};
const Segment SGI308205[] = {{&S5793[42], 7}, {&S5793[70], 5}, {NULL}};
const Segment SGI308206[] = {{&S5793[35], 14}, {NULL}};
const Segment SGI308207[] = {{&S5794[30], 3}, {NULL}};
const Segment SGI308208[] = {{&S5794[8], 3}, {NULL}};
const Segment SGI308209[] = {{&S5794[13], 10}, {NULL}};
const Segment SGI308210[] = {{&S5794[42], 7}, {NULL}};
const Segment SGI308211[] = {{&S5795[18], 3}, {NULL}};
const Segment SGI308212[] = {{&S5795[8], 3}, {NULL}};
const Segment SGI308213[] = {{&S5795[13], 8}, {NULL}};
const Segment SGI308214[] = {{&S5795[30], 7}, {NULL}};
const Segment SGI308215[] = {{&S5796[27], 3}, {NULL}};
const Segment SGI308216[] = {{&S5796[8], 3}, {NULL}};
const Segment SGI308217[] = {{&S5796[13], 7}, {NULL}};
const Segment SGI308218[] = {{&S5796[39], 7}, {NULL}};
const Segment SGI308219[] = {{&S5797[30], 3}, {NULL}};
const Segment SGI308220[] = {{&S5797[8], 3}, {NULL}};
const Segment SGI308221[] = {{&S5797[13], 36}, {NULL}};
const Segment SGI308222[] = {{&S5797[42], 7}, {NULL}};
const Segment SGI308223[] = {{&S5798[30], 4}, {NULL}};
const Segment SGI308224[] = {{&S5798[8], 3}, {NULL}};
const Segment SGI308225[] = {{&S5798[13], 10}, {NULL}};
const Segment SGI308226[] = {{&S5798[43], 7}, {NULL}};
const Segment SGI308227[] = {{&S5799[35], 3}, {NULL}};
const Segment SGI308228[] = {{&S5799[47], 7}, {NULL}};
const Segment SGI308229[] = {{&S5800[30], 3}, {NULL}};
const Segment SGI308230[] = {{&S5800[8], 3}, {NULL}};
const Segment SGI308231[] = {{&S5800[13], 10}, {NULL}};
const Segment SGI308232[] = {{&S5800[42], 7}, {NULL}};
const Segment SGI308233[] = {{&S5800[35], 14}, {NULL}};
const Segment SGI308234[] = {{&S5800[51], 6}, {NULL}};
const Segment SGI308235[] = {{&S5801[30], 3}, {NULL}};
const Segment SGI308236[] = {{&S5801[8], 3}, {NULL}};
const Segment SGI308237[] = {{&S5801[13], 10}, {NULL}};
const Segment SGI308238[] = {{&S5801[42], 7}, {NULL}};
const Segment SGI308239[] = {{&S5801[35], 14}, {NULL}};
const Segment SGI308240[] = {{&S5801[131], 15}, {NULL}};
const Segment SGI308241[] = {{&S5802[30], 3}, {NULL}};
const Segment SGI308242[] = {{&S5802[8], 3}, {NULL}};
const Segment SGI308243[] = {{&S5802[13], 10}, {NULL}};
const Segment SGI308244[] = {{&S5802[42], 7}, {NULL}};
const Segment SGI308245[] = {{&S5802[35], 14}, {NULL}};
const Segment SGI308246[] = {{&S5803[30], 3}, {NULL}};
const Segment SGI308247[] = {{&S5803[8], 3}, {NULL}};
const Segment SGI308248[] = {{&S5803[13], 10}, {NULL}};
const Segment SGI308249[] = {{&S5803[42], 7}, {NULL}};
const Segment SGI308250[] = {{&S5803[35], 14}, {NULL}};
const Segment SGI308251[] = {{&S5803[138], 15}, {NULL}};
const Segment SGI308252[] = {{&S5803[51], 6}, {NULL}};
const Segment SGI308253[] = {{&S16627[0], 74}, {NULL}};
const Segment SGI308254[] = {{&S17113[0], 70}, {NULL}};
const Segment SGI308255[] = {{&S17169[0], 72}, {NULL}};
const Segment SGI308256[] = {{&S9155[30], 3}, {NULL}};
const Segment SGI308257[] = {{&S9155[8], 3}, {NULL}};
const Segment SGI308258[] = {{&S9155[13], 10}, {NULL}};
const Segment SGI308259[] = {{&S9155[42], 7}, {NULL}};
const Segment SGI308260[] = {{&S9155[35], 14}, {NULL}};
const Segment SGI308261[] = {{&S9155[84], 11}, {NULL}};
const Segment SGI308262[] = {{&S9155[177], 15}, {NULL}};
const Segment SGI308263[] = {{&S9155[84], 16}, {NULL}};
const Segment SGI308264[] = {{&S9155[51], 5}, {NULL}};
const Segment SGI308265[] = {{&S9158[30], 3}, {NULL}};
const Segment SGI308266[] = {{&S9158[8], 3}, {NULL}};
const Segment SGI308267[] = {{&S9158[13], 10}, {NULL}};
const Segment SGI308268[] = {{&S9158[42], 7}, {NULL}};
const Segment SGI308269[] = {{&S9158[35], 14}, {NULL}};
const Segment SGI308270[] = {{&S9158[71], 11}, {NULL}};
const Segment SGI308271[] = {{&S9158[164], 15}, {NULL}};
const Segment SGI308272[] = {{&S9158[51], 6}, {NULL}};
const Segment SGI308273[] = {{&S9158[71], 16}, {NULL}};
const Segment SGI308274[] = {{&S9161[30], 3}, {NULL}};
const Segment SGI308275[] = {{&S9161[8], 3}, {NULL}};
const Segment SGI308276[] = {{&S9161[13], 10}, {NULL}};
const Segment SGI308277[] = {{&S9161[42], 7}, {NULL}};
const Segment SGI308278[] = {{&S9161[35], 14}, {NULL}};
const Segment SGI308279[] = {{&S9161[64], 11}, {NULL}};
const Segment SGI308280[] = {{&S9161[64], 16}, {NULL}};
const Segment SGI308281[] = {{&S9162[30], 3}, {NULL}};
const Segment SGI308282[] = {{&S9162[8], 3}, {NULL}};
const Segment SGI308283[] = {{&S9162[13], 10}, {NULL}};
const Segment SGI308284[] = {{&S9162[42], 7}, {NULL}};
const Segment SGI308285[] = {{&S9162[35], 14}, {NULL}};
const Segment SGI308286[] = {{&S9162[71], 11}, {NULL}};
const Segment SGI308287[] = {{&S9162[51], 6}, {NULL}};
const Segment SGI308288[] = {{&S9162[71], 16}, {NULL}};
const Segment SGI308289[] = {{&S9163[30], 3}, {NULL}};
const Segment SGI308290[] = {{&S9163[8], 3}, {NULL}};
const Segment SGI308291[] = {{&S9163[13], 10}, {NULL}};
const Segment SGI308292[] = {{&S9163[42], 7}, {NULL}};
const Segment SGI308293[] = {{&S9163[35], 14}, {NULL}};
const Segment SGI308294[] = {{&S9163[64], 11}, {NULL}};
const Segment SGI308295[] = {{&S9163[157], 15}, {NULL}};
const Segment SGI308296[] = {{&S9163[64], 16}, {NULL}};
const Segment SGI308297[] = {{&S9163[200], 9}, {NULL}};
const Segment SGI308298[] = {{&S9164[30], 4}, {NULL}};
const Segment SGI308299[] = {{&S9164[8], 3}, {NULL}};
const Segment SGI308300[] = {{&S9164[13], 10}, {NULL}};
const Segment SGI308301[] = {{&S9164[43], 7}, {NULL}};
const Segment SGI308302[] = {{&S9164[36], 14}, {NULL}};
const Segment SGI308303[] = {{&S9164[72], 11}, {NULL}};
const Segment SGI308304[] = {{&S9164[52], 6}, {NULL}};
const Segment SGI308305[] = {{&S9164[72], 16}, {NULL}};
const Segment SGI308306[] = {{&S9167[30], 3}, {NULL}};
const Segment SGI308307[] = {{&S9167[8], 3}, {NULL}};
const Segment SGI308308[] = {{&S9167[13], 10}, {NULL}};
const Segment SGI308309[] = {{&S9167[42], 7}, {NULL}};
const Segment SGI308310[] = {{&S9167[35], 14}, {NULL}};
const Segment SGI308311[] = {{&S9167[64], 11}, {NULL}};
const Segment SGI308312[] = {{&S9167[157], 15}, {NULL}};
const Segment SGI308313[] = {{&S9167[64], 16}, {NULL}};
const Segment SGI308314[] = {{&S9168[30], 3}, {NULL}};
const Segment SGI308315[] = {{&S9168[8], 3}, {NULL}};
const Segment SGI308316[] = {{&S9168[13], 10}, {NULL}};
const Segment SGI308317[] = {{&S9168[42], 7}, {NULL}};
const Segment SGI308318[] = {{&S9168[35], 14}, {NULL}};
const Segment SGI308319[] = {{&S9168[174], 11}, {NULL}};
const Segment SGI308320[] = {{&S9168[131], 15}, {NULL}};
const Segment SGI308321[] = {{&S9168[174], 16}, {NULL}};
const Segment SGI308322[] = {{&S9169[30], 3}, {NULL}};
const Segment SGI308323[] = {{&S9169[8], 3}, {NULL}};
const Segment SGI308324[] = {{&S9169[13], 10}, {NULL}};
const Segment SGI308325[] = {{&S9169[42], 7}, {NULL}};
const Segment SGI308326[] = {{&S9169[35], 14}, {NULL}};
const Segment SGI308327[] = {{&S9169[71], 11}, {NULL}};
const Segment SGI308328[] = {{&S9169[51], 6}, {NULL}};
const Segment SGI308329[] = {{&S9169[71], 16}, {NULL}};
const Segment SGI308330[] = {{&S9170[30], 3}, {NULL}};
const Segment SGI308331[] = {{&S9170[8], 3}, {NULL}};
const Segment SGI308332[] = {{&S9170[13], 10}, {NULL}};
const Segment SGI308333[] = {{&S9170[42], 7}, {NULL}};
const Segment SGI308334[] = {{&S9170[35], 14}, {NULL}};
const Segment SGI308335[] = {{&S9170[177], 11}, {NULL}};
const Segment SGI308336[] = {{&S9170[131], 15}, {NULL}};
const Segment SGI308337[] = {{&S9170[177], 16}, {NULL}};
const Segment SGI308338[] = {{&S9170[162], 9}, {NULL}};
const Segment SGI308339[] = {{&S9171[30], 3}, {NULL}};
const Segment SGI308340[] = {{&S9171[8], 3}, {NULL}};
const Segment SGI308341[] = {{&S9171[13], 10}, {NULL}};
const Segment SGI308342[] = {{&S9171[42], 7}, {NULL}};
const Segment SGI308343[] = {{&S9171[35], 14}, {NULL}};
const Segment SGI308344[] = {{&S9171[64], 11}, {NULL}};
const Segment SGI308345[] = {{&S9171[64], 16}, {NULL}};
const Segment SGI308346[] = {{&S9172[30], 3}, {NULL}};
const Segment SGI308347[] = {{&S9172[8], 3}, {NULL}};
const Segment SGI308348[] = {{&S9172[13], 10}, {NULL}};
const Segment SGI308349[] = {{&S9172[42], 7}, {NULL}};
const Segment SGI308350[] = {{&S9172[35], 14}, {NULL}};
const Segment SGI308351[] = {{&S9172[169], 11}, {NULL}};
const Segment SGI308352[] = {{&S9172[138], 15}, {NULL}};
const Segment SGI308353[] = {{&S9172[51], 6}, {NULL}};
const Segment SGI308354[] = {{&S9172[169], 16}, {NULL}};
const Segment SGI308355[] = {{&S9173[30], 3}, {NULL}};
const Segment SGI308356[] = {{&S9173[8], 3}, {NULL}};
const Segment SGI308357[] = {{&S9173[13], 10}, {NULL}};
const Segment SGI308358[] = {{&S9173[42], 7}, {NULL}};
const Segment SGI308359[] = {{&S9173[35], 14}, {NULL}};
const Segment SGI308360[] = {{&S9173[86], 11}, {NULL}};
const Segment SGI308361[] = {{&S9173[51], 6}, {NULL}};
const Segment SGI308362[] = {{&S9173[71], 13}, {NULL}};
const Segment SGI308363[] = {{&S9173[86], 16}, {NULL}};
const Segment SGI308364[] = {{&S9174[30], 3}, {NULL}};
const Segment SGI308365[] = {{&S9174[8], 3}, {NULL}};
const Segment SGI308366[] = {{&S9174[13], 10}, {NULL}};
const Segment SGI308367[] = {{&S9174[42], 7}, {NULL}};
const Segment SGI308368[] = {{&S9174[35], 14}, {NULL}};
const Segment SGI308369[] = {{&S9174[71], 11}, {NULL}};
const Segment SGI308370[] = {{&S9174[51], 6}, {NULL}};
const Segment SGI308371[] = {{&S9174[71], 16}, {NULL}};
const Segment SGI308372[] = {{&S9175[30], 3}, {NULL}};
const Segment SGI308373[] = {{&S9175[8], 3}, {NULL}};
const Segment SGI308374[] = {{&S9175[13], 10}, {NULL}};
const Segment SGI308375[] = {{&S9175[42], 7}, {NULL}};
const Segment SGI308376[] = {{&S9175[35], 14}, {NULL}};
const Segment SGI308377[] = {{&S9175[64], 11}, {NULL}};
const Segment SGI308378[] = {{&S9175[64], 16}, {NULL}};
const Segment SGI308379[] = {{&S9175[177], 9}, {NULL}};
const Segment SGI308380[] = {{&S9176[30], 4}, {NULL}};
const Segment SGI308381[] = {{&S9176[8], 3}, {NULL}};
const Segment SGI308382[] = {{&S9176[13], 10}, {NULL}};
const Segment SGI308383[] = {{&S9176[43], 7}, {NULL}};
const Segment SGI308384[] = {{&S9176[36], 14}, {NULL}};
const Segment SGI308385[] = {{&S9176[65], 11}, {NULL}};
const Segment SGI308386[] = {{&S9176[65], 16}, {NULL}};
const Segment SGI308387[] = {{&S9177[30], 3}, {NULL}};
const Segment SGI308388[] = {{&S9177[8], 3}, {NULL}};
const Segment SGI308389[] = {{&S9177[13], 10}, {NULL}};
const Segment SGI308390[] = {{&S9177[42], 7}, {NULL}};
const Segment SGI308391[] = {{&S9177[35], 14}, {NULL}};
const Segment SGI308392[] = {{&S9177[79], 11}, {NULL}};
const Segment SGI308393[] = {{&S9177[172], 15}, {NULL}};
const Segment SGI308394[] = {{&S9177[51], 6}, {NULL}};
const Segment SGI308395[] = {{&S9177[79], 16}, {NULL}};
const Segment SGI308396[] = {{&S9178[30], 3}, {NULL}};
const Segment SGI308397[] = {{&S9178[8], 3}, {NULL}};
const Segment SGI308398[] = {{&S9178[13], 10}, {NULL}};
const Segment SGI308399[] = {{&S9178[42], 7}, {NULL}};
const Segment SGI308400[] = {{&S9178[35], 14}, {NULL}};
const Segment SGI308401[] = {{&S9178[79], 11}, {NULL}};
const Segment SGI308402[] = {{&S9178[64], 13}, {NULL}};
const Segment SGI308403[] = {{&S9178[79], 16}, {NULL}};
const Segment SGI308404[] = {{&S9179[30], 3}, {NULL}};
const Segment SGI308405[] = {{&S9179[8], 3}, {NULL}};
const Segment SGI308406[] = {{&S9179[13], 10}, {NULL}};
const Segment SGI308407[] = {{&S9179[42], 7}, {NULL}};
const Segment SGI308408[] = {{&S9179[35], 14}, {NULL}};
const Segment SGI308409[] = {{&S9179[74], 11}, {NULL}};
const Segment SGI308410[] = {{&S9179[74], 16}, {NULL}};
const Segment SGI308411[] = {{&S9180[30], 3}, {NULL}};
const Segment SGI308412[] = {{&S9180[248], 9}, {NULL}};
const Segment SGI308413[] = {{&S9180[8], 3}, {NULL}};
const Segment SGI308414[] = {{&S9180[13], 10}, {NULL}};
const Segment SGI308415[] = {{&S9180[42], 7}, {NULL}};
const Segment SGI308416[] = {{&S9180[35], 14}, {NULL}};
const Segment SGI308417[] = {{&S9180[71], 11}, {NULL}};
const Segment SGI308418[] = {{&S9180[164], 15}, {NULL}};
const Segment SGI308419[] = {{&S9180[51], 6}, {NULL}};
const Segment SGI308420[] = {{&S9180[71], 16}, {NULL}};
const Segment SGI308421[] = {{&S9181[30], 3}, {NULL}};
const Segment SGI308422[] = {{&S9181[8], 3}, {NULL}};
const Segment SGI308423[] = {{&S9181[13], 10}, {NULL}};
const Segment SGI308424[] = {{&S9181[42], 7}, {NULL}};
const Segment SGI308425[] = {{&S9181[35], 14}, {NULL}};
const Segment SGI308426[] = {{&S9181[71], 11}, {NULL}};
const Segment SGI308427[] = {{&S9181[164], 15}, {NULL}};
const Segment SGI308428[] = {{&S9181[217], 4}, {NULL}};
const Segment SGI308429[] = {{&S9181[51], 6}, {NULL}};
const Segment SGI308430[] = {{&S9181[71], 16}, {NULL}};
const Segment SGI308431[] = {{&S9182[30], 3}, {NULL}};
const Segment SGI308432[] = {{&S9182[248], 9}, {NULL}};
const Segment SGI308433[] = {{&S9182[8], 3}, {NULL}};
const Segment SGI308434[] = {{&S9182[13], 10}, {NULL}};
const Segment SGI308435[] = {{&S9182[42], 7}, {NULL}};
const Segment SGI308436[] = {{&S9182[35], 14}, {NULL}};
const Segment SGI308437[] = {{&S9182[71], 11}, {NULL}};
const Segment SGI308438[] = {{&S9182[164], 15}, {NULL}};
const Segment SGI308439[] = {{&S9182[51], 6}, {NULL}};
const Segment SGI308440[] = {{&S9182[71], 16}, {NULL}};
const Segment SGI308441[] = {{&S20481[0], 82}, {NULL}};
const Segment SGI308442[] = {{&S9703[30], 3}, {NULL}};
const Segment SGI308443[] = {{&S9703[8], 3}, {NULL}};
const Segment SGI308444[] = {{&S9703[13], 10}, {NULL}};
const Segment SGI308445[] = {{&S9703[42], 7}, {NULL}};
const Segment SGI308446[] = {{&S9703[35], 14}, {NULL}};
const Segment SGI308447[] = {{&S9703[64], 11}, {NULL}};
const Segment SGI308448[] = {{&S9703[64], 16}, {NULL}};
const Segment SGI308449[] = {{&S9705[30], 3}, {NULL}};
const Segment SGI308450[] = {{&S9705[8], 3}, {NULL}};
const Segment SGI308451[] = {{&S9705[13], 10}, {NULL}};
const Segment SGI308452[] = {{&S9705[42], 7}, {NULL}};
const Segment SGI308453[] = {{&S9705[35], 14}, {NULL}};
const Segment SGI308454[] = {{&S9705[51], 11}, {NULL}};
const Segment SGI308455[] = {{&S9705[51], 16}, {NULL}};
const Segment SGI308456[] = {{&S9706[30], 3}, {NULL}};
const Segment SGI308457[] = {{&S9706[8], 3}, {NULL}};
const Segment SGI308458[] = {{&S9706[13], 10}, {NULL}};
const Segment SGI308459[] = {{&S9706[42], 7}, {NULL}};
const Segment SGI308460[] = {{&S9706[35], 14}, {NULL}};
const Segment SGI308461[] = {{&S9706[56], 11}, {NULL}};
const Segment SGI308462[] = {{&S9706[56], 16}, {NULL}};
const Segment SGI308463[] = {{&S9706[51], 3}, {NULL}};
const Segment SGI308464[] = {{&S9707[30], 3}, {NULL}};
const Segment SGI308465[] = {{&S9707[8], 3}, {NULL}};
const Segment SGI308466[] = {{&S9707[13], 10}, {NULL}};
const Segment SGI308467[] = {{&S9707[42], 7}, {NULL}};
const Segment SGI308468[] = {{&S9707[35], 14}, {NULL}};
const Segment SGI308469[] = {{&S9707[51], 11}, {NULL}};
const Segment SGI308470[] = {{&S9707[51], 16}, {NULL}};
const Segment SGI308471[] = {{&S9711[30], 3}, {NULL}};
const Segment SGI308472[] = {{&S9711[8], 3}, {NULL}};
const Segment SGI308473[] = {{&S9711[13], 10}, {NULL}};
const Segment SGI308474[] = {{&S9711[42], 7}, {NULL}};
const Segment SGI308475[] = {{&S9711[35], 14}, {NULL}};
const Segment SGI308476[] = {{&S9711[72], 11}, {NULL}};
const Segment SGI308477[] = {{&S9711[136], 15}, {NULL}};
const Segment SGI308478[] = {{&S9711[72], 16}, {NULL}};
const Segment SGI308479[] = {{&S9712[30], 3}, {NULL}};
const Segment SGI308480[] = {{&S9712[8], 3}, {NULL}};
const Segment SGI308481[] = {{&S9712[13], 10}, {NULL}};
const Segment SGI308482[] = {{&S9712[42], 7}, {NULL}};
const Segment SGI308483[] = {{&S9712[35], 14}, {NULL}};
const Segment SGI308484[] = {{&S9712[69], 11}, {NULL}};
const Segment SGI308485[] = {{&S9712[69], 16}, {NULL}};
const Segment SGI308486[] = {{&S9712[64], 3}, {NULL}};
const Segment SGI308487[] = {{&S9713[30], 3}, {NULL}};
const Segment SGI308488[] = {{&S9713[8], 3}, {NULL}};
const Segment SGI308489[] = {{&S9713[13], 10}, {NULL}};
const Segment SGI308490[] = {{&S9713[42], 7}, {NULL}};
const Segment SGI308491[] = {{&S9713[35], 14}, {NULL}};
const Segment SGI308492[] = {{&S9713[64], 11}, {NULL}};
const Segment SGI308493[] = {{&S9713[64], 16}, {NULL}};
const Segment SGI308494[] = {{&S9713[90], 9}, {NULL}};
const Segment SGI308495[] = {{&S9714[30], 3}, {&S9714[128], 3}, {NULL}};
const Segment SGI308496[] = {{&S9714[8], 3}, {&S9714[8], 3}, {NULL}};
const Segment SGI308497[] = {{&S9714[13], 10}, {&S9714[13], 10}, {NULL}};
const Segment SGI308498[] = {{&S9714[42], 7}, {&S9714[42], 7}, {NULL}};
const Segment SGI308499[] = {{&S9714[35], 14}, {&S9714[35], 14}, {NULL}};
const Segment SGI308500[] = {{&S9714[72], 11}, {NULL}};
const Segment SGI308501[] = {{&S9714[72], 16}, {NULL}};
const Segment SGI308502[] = {{&S9714[149], 3}, {NULL}};
const Segment SGI308503[] = {{&S9715[30], 3}, {NULL}};
const Segment SGI308504[] = {{&S9715[8], 3}, {NULL}};
const Segment SGI308505[] = {{&S9715[13], 10}, {NULL}};
const Segment SGI308506[] = {{&S9715[42], 7}, {NULL}};
const Segment SGI308507[] = {{&S9715[35], 14}, {NULL}};
const Segment SGI308508[] = {{&S9715[59], 11}, {NULL}};
const Segment SGI308509[] = {{&S9715[123], 15}, {NULL}};
const Segment SGI308510[] = {{&S9715[59], 16}, {NULL}};
const Segment SGI308511[] = {{&S9716[30], 3}, {NULL}};
const Segment SGI308512[] = {{&S9716[8], 3}, {NULL}};
const Segment SGI308513[] = {{&S9716[13], 10}, {NULL}};
const Segment SGI308514[] = {{&S9716[42], 7}, {NULL}};
const Segment SGI308515[] = {{&S9716[35], 14}, {NULL}};
const Segment SGI308516[] = {{&S9716[56], 11}, {NULL}};
const Segment SGI308517[] = {{&S9716[120], 15}, {NULL}};
const Segment SGI308518[] = {{&S9716[56], 16}, {NULL}};
const Segment SGI308519[] = {{&S9716[51], 3}, {NULL}};
const Segment SGI308520[] = {{&S9717[30], 3}, {NULL}};
const Segment SGI308521[] = {{&S9717[8], 3}, {NULL}};
const Segment SGI308522[] = {{&S9717[13], 10}, {NULL}};
const Segment SGI308523[] = {{&S9717[42], 7}, {NULL}};
const Segment SGI308524[] = {{&S9717[35], 14}, {NULL}};
const Segment SGI308525[] = {{&S9717[64], 11}, {NULL}};
const Segment SGI308526[] = {{&S9717[173], 4}, {NULL}};
const Segment SGI308527[] = {{&S9717[64], 16}, {NULL}};
const Segment SGI308528[] = {{&S9718[30], 3}, {NULL}};
const Segment SGI308529[] = {{&S9718[8], 3}, {NULL}};
const Segment SGI308530[] = {{&S9718[13], 10}, {NULL}};
const Segment SGI308531[] = {{&S9718[42], 7}, {NULL}};
const Segment SGI308532[] = {{&S9718[35], 14}, {NULL}};
const Segment SGI308533[] = {{&S9718[71], 11}, {NULL}};
const Segment SGI308534[] = {{&S9718[51], 6}, {NULL}};
const Segment SGI308535[] = {{&S9718[71], 16}, {NULL}};
const Segment SGI308536[] = {{&S9720[30], 3}, {&S9720[128], 3}, {NULL}};
const Segment SGI308537[] = {{&S9720[8], 3}, {&S9720[8], 3}, {NULL}};
const Segment SGI308538[] = {{&S9720[13], 10}, {&S9720[13], 10}, {NULL}};
const Segment SGI308539[] = {{&S9720[42], 7}, {&S9720[42], 7}, {NULL}};
const Segment SGI308540[] = {{&S9720[35], 14}, {&S9720[35], 14}, {NULL}};
const Segment SGI308541[] = {{&S9720[72], 11}, {NULL}};
const Segment SGI308542[] = {{&S9720[194], 15}, {NULL}};
const Segment SGI308543[] = {{&S9720[72], 16}, {NULL}};
const Segment SGI308544[] = {{&S9720[149], 3}, {NULL}};
const Segment SGI308545[] = {{&S9721[30], 3}, {NULL}};
const Segment SGI308546[] = {{&S9721[8], 3}, {NULL}};
const Segment SGI308547[] = {{&S9721[13], 10}, {NULL}};
const Segment SGI308548[] = {{&S9721[42], 7}, {NULL}};
const Segment SGI308549[] = {{&S9721[35], 14}, {NULL}};
const Segment SGI308550[] = {{&S9721[64], 11}, {NULL}};
const Segment SGI308551[] = {{&S9721[64], 16}, {NULL}};
const Segment SGI308552[] = {{&S9723[30], 3}, {NULL}};
const Segment SGI308553[] = {{&S9723[8], 3}, {NULL}};
const Segment SGI308554[] = {{&S9723[13], 10}, {NULL}};
const Segment SGI308555[] = {{&S9723[42], 7}, {NULL}};
const Segment SGI308556[] = {{&S9723[35], 14}, {NULL}};
const Segment SGI308557[] = {{&S9723[51], 11}, {NULL}};
const Segment SGI308558[] = {{&S9723[51], 16}, {NULL}};
const Segment SGI308559[] = {{&S9725[30], 3}, {NULL}};
const Segment SGI308560[] = {{&S9725[223], 9}, {NULL}};
const Segment SGI308561[] = {{&S9725[8], 3}, {NULL}};
const Segment SGI308562[] = {{&S9725[13], 10}, {NULL}};
const Segment SGI308563[] = {{&S9725[42], 7}, {NULL}};
const Segment SGI308564[] = {{&S9725[35], 14}, {NULL}};
const Segment SGI308565[] = {{&S9725[64], 11}, {NULL}};
const Segment SGI308566[] = {{&S9725[64], 16}, {NULL}};
const Segment SGI308567[] = {{&S9775[30], 3}, {NULL}};
const Segment SGI308568[] = {{&S9775[8], 3}, {NULL}};
const Segment SGI308569[] = {{&S9775[13], 10}, {NULL}};
const Segment SGI308570[] = {{&S9775[42], 7}, {NULL}};
const Segment SGI308571[] = {{&S9775[35], 14}, {NULL}};
const Segment SGI308572[] = {{&S9775[169], 11}, {NULL}};
const Segment SGI308573[] = {{&S9775[138], 15}, {NULL}};
const Segment SGI308574[] = {{&S9775[51], 6}, {NULL}};
const Segment SGI308575[] = {{&S9775[169], 16}, {NULL}};
const Segment SGI308576[] = {{&S9778[30], 4}, {NULL}};
const Segment SGI308577[] = {{&S9778[8], 3}, {NULL}};
const Segment SGI308578[] = {{&S9778[13], 10}, {NULL}};
const Segment SGI308579[] = {{&S9778[43], 7}, {NULL}};
const Segment SGI308580[] = {{&S9778[36], 14}, {NULL}};
const Segment SGI308581[] = {{&S9778[65], 11}, {NULL}};
const Segment SGI308582[] = {{&S9778[65], 16}, {NULL}};
const Segment SGI308583[] = {{&S9779[30], 3}, {NULL}};
const Segment SGI308584[] = {{&S9779[8], 3}, {NULL}};
const Segment SGI308585[] = {{&S9779[13], 10}, {NULL}};
const Segment SGI308586[] = {{&S9779[42], 7}, {NULL}};
const Segment SGI308587[] = {{&S9779[35], 14}, {NULL}};
const Segment SGI308588[] = {{&S9779[64], 11}, {NULL}};
const Segment SGI308589[] = {{&S9779[64], 16}, {NULL}};
const Segment SGI308590[] = {{&S9780[30], 3}, {NULL}};
const Segment SGI308591[] = {{&S9780[8], 3}, {NULL}};
const Segment SGI308592[] = {{&S9780[13], 10}, {NULL}};
const Segment SGI308593[] = {{&S9780[42], 7}, {NULL}};
const Segment SGI308594[] = {{&S9780[35], 14}, {NULL}};
const Segment SGI308595[] = {{&S9780[71], 11}, {NULL}};
const Segment SGI308596[] = {{&S9780[51], 6}, {NULL}};
const Segment SGI308597[] = {{&S9780[71], 16}, {NULL}};
const Segment SGI308598[] = {{&S9781[30], 3}, {NULL}};
const Segment SGI308599[] = {{&S9781[8], 3}, {NULL}};
const Segment SGI308600[] = {{&S9781[13], 10}, {NULL}};
const Segment SGI308601[] = {{&S9781[42], 7}, {NULL}};
const Segment SGI308602[] = {{&S9781[35], 14}, {NULL}};
const Segment SGI308603[] = {{&S9781[71], 11}, {NULL}};
const Segment SGI308604[] = {{&S9781[51], 6}, {NULL}};
const Segment SGI308605[] = {{&S9781[71], 16}, {NULL}};
const Segment SGI308606[] = {{&S9782[30], 4}, {NULL}};
const Segment SGI308607[] = {{&S9782[8], 3}, {NULL}};
const Segment SGI308608[] = {{&S9782[13], 10}, {NULL}};
const Segment SGI308609[] = {{&S9782[43], 7}, {NULL}};
const Segment SGI308610[] = {{&S9782[36], 14}, {NULL}};
const Segment SGI308611[] = {{&S9782[72], 11}, {NULL}};
const Segment SGI308612[] = {{&S9782[52], 6}, {NULL}};
const Segment SGI308613[] = {{&S9782[72], 16}, {NULL}};
const Segment SGI308614[] = {{&S9783[30], 3}, {NULL}};
const Segment SGI308615[] = {{&S9783[8], 3}, {NULL}};
const Segment SGI308616[] = {{&S9783[13], 10}, {NULL}};
const Segment SGI308617[] = {{&S9783[42], 7}, {NULL}};
const Segment SGI308618[] = {{&S9783[35], 14}, {NULL}};
const Segment SGI308619[] = {{&S9783[64], 11}, {NULL}};
const Segment SGI308620[] = {{&S9783[157], 15}, {NULL}};
const Segment SGI308621[] = {{&S9783[64], 16}, {NULL}};
const Segment SGI308622[] = {{&S9783[200], 9}, {NULL}};
const Segment SGI308623[] = {{&S9784[30], 3}, {NULL}};
const Segment SGI308624[] = {{&S9784[8], 3}, {NULL}};
const Segment SGI308625[] = {{&S9784[13], 10}, {NULL}};
const Segment SGI308626[] = {{&S9784[42], 7}, {NULL}};
const Segment SGI308627[] = {{&S9784[35], 14}, {NULL}};
const Segment SGI308628[] = {{&S9784[174], 11}, {NULL}};
const Segment SGI308629[] = {{&S9784[131], 15}, {NULL}};
const Segment SGI308630[] = {{&S9784[174], 16}, {NULL}};
const Segment SGI308631[] = {{&S9785[30], 3}, {NULL}};
const Segment SGI308632[] = {{&S9785[8], 3}, {NULL}};
const Segment SGI308633[] = {{&S9785[13], 10}, {NULL}};
const Segment SGI308634[] = {{&S9785[42], 7}, {NULL}};
const Segment SGI308635[] = {{&S9785[35], 14}, {NULL}};
const Segment SGI308636[] = {{&S9785[71], 11}, {NULL}};
const Segment SGI308637[] = {{&S9785[164], 15}, {NULL}};
const Segment SGI308638[] = {{&S9785[51], 6}, {NULL}};
const Segment SGI308639[] = {{&S9785[71], 16}, {NULL}};
const Segment SGI308640[] = {{&S9788[30], 3}, {NULL}};
const Segment SGI308641[] = {{&S9788[8], 3}, {NULL}};
const Segment SGI308642[] = {{&S9788[13], 10}, {NULL}};
const Segment SGI308643[] = {{&S9788[42], 7}, {NULL}};
const Segment SGI308644[] = {{&S9788[35], 14}, {NULL}};
const Segment SGI308645[] = {{&S9788[64], 11}, {NULL}};
const Segment SGI308646[] = {{&S9788[157], 15}, {NULL}};
const Segment SGI308647[] = {{&S9788[64], 16}, {NULL}};
const Segment SGI308648[] = {{&S9789[30], 3}, {NULL}};
const Segment SGI308649[] = {{&S9789[8], 3}, {NULL}};
const Segment SGI308650[] = {{&S9789[13], 10}, {NULL}};
const Segment SGI308651[] = {{&S9789[42], 7}, {NULL}};
const Segment SGI308652[] = {{&S9789[35], 14}, {NULL}};
const Segment SGI308653[] = {{&S9789[76], 11}, {NULL}};
const Segment SGI308654[] = {{&S9789[169], 15}, {NULL}};
const Segment SGI308655[] = {{&S9789[76], 16}, {NULL}};
const Segment SGI308656[] = {{&S9789[51], 5}, {NULL}};
const Segment SGI308657[] = {{&S9790[30], 3}, {NULL}};
const Segment SGI308658[] = {{&S9790[8], 3}, {NULL}};
const Segment SGI308659[] = {{&S9790[13], 10}, {NULL}};
const Segment SGI308660[] = {{&S9790[42], 7}, {NULL}};
const Segment SGI308661[] = {{&S9790[35], 14}, {NULL}};
const Segment SGI308662[] = {{&S9790[64], 11}, {NULL}};
const Segment SGI308663[] = {{&S9790[157], 15}, {NULL}};
const Segment SGI308664[] = {{&S9790[64], 16}, {NULL}};
const Segment SGI308665[] = {{&S9790[200], 9}, {NULL}};
const Segment SGI308666[] = {{&S9791[30], 3}, {NULL}};
const Segment SGI308667[] = {{&S9791[8], 3}, {NULL}};
const Segment SGI308668[] = {{&S9791[13], 10}, {NULL}};
const Segment SGI308669[] = {{&S9791[42], 7}, {NULL}};
const Segment SGI308670[] = {{&S9791[35], 14}, {NULL}};
const Segment SGI308671[] = {{&S9791[64], 11}, {NULL}};
const Segment SGI308672[] = {{&S9791[64], 16}, {NULL}};
const Segment SGI308673[] = {{&S9792[30], 3}, {NULL}};
const Segment SGI308674[] = {{&S9792[8], 3}, {NULL}};
const Segment SGI308675[] = {{&S9792[13], 10}, {NULL}};
const Segment SGI308676[] = {{&S9792[42], 7}, {NULL}};
const Segment SGI308677[] = {{&S9792[35], 14}, {NULL}};
const Segment SGI308678[] = {{&S9792[71], 11}, {NULL}};
const Segment SGI308679[] = {{&S9792[51], 6}, {NULL}};
const Segment SGI308680[] = {{&S9792[71], 16}, {NULL}};
const Segment SGI308681[] = {{&S9793[30], 3}, {NULL}};
const Segment SGI308682[] = {{&S9793[8], 3}, {NULL}};
const Segment SGI308683[] = {{&S9793[13], 10}, {NULL}};
const Segment SGI308684[] = {{&S9793[42], 7}, {NULL}};
const Segment SGI308685[] = {{&S9793[35], 14}, {NULL}};
const Segment SGI308686[] = {{&S9793[64], 11}, {NULL}};
const Segment SGI308687[] = {{&S9793[64], 16}, {NULL}};
const Segment SGI308688[] = {{&S9793[157], 9}, {NULL}};
const Segment SGI308689[] = {{&S9794[30], 3}, {NULL}};
const Segment SGI308690[] = {{&S9794[8], 3}, {NULL}};
const Segment SGI308691[] = {{&S9794[13], 10}, {NULL}};
const Segment SGI308692[] = {{&S9794[42], 7}, {NULL}};
const Segment SGI308693[] = {{&S9794[35], 14}, {NULL}};
const Segment SGI308694[] = {{&S9794[64], 11}, {NULL}};
const Segment SGI308695[] = {{&S9794[64], 16}, {NULL}};
const Segment SGI308696[] = {{&S9795[30], 3}, {NULL}};
const Segment SGI308697[] = {{&S9795[236], 9}, {NULL}};
const Segment SGI308698[] = {{&S9795[8], 3}, {NULL}};
const Segment SGI308699[] = {{&S9795[13], 10}, {NULL}};
const Segment SGI308700[] = {{&S9795[42], 7}, {NULL}};
const Segment SGI308701[] = {{&S9795[35], 14}, {NULL}};
const Segment SGI308702[] = {{&S9795[71], 11}, {NULL}};
const Segment SGI308703[] = {{&S9795[165], 15}, {NULL}};
const Segment SGI308704[] = {{&S9795[51], 6}, {NULL}};
const Segment SGI308705[] = {{&S9795[71], 16}, {NULL}};
const Segment SGI308706[] = {{&S9796[30], 3}, {NULL}};
const Segment SGI308707[] = {{&S9796[8], 3}, {NULL}};
const Segment SGI308708[] = {{&S9796[13], 10}, {NULL}};
const Segment SGI308709[] = {{&S9796[42], 7}, {NULL}};
const Segment SGI308710[] = {{&S9796[35], 14}, {NULL}};
const Segment SGI308711[] = {{&S9796[64], 11}, {NULL}};
const Segment SGI308712[] = {{&S9796[64], 16}, {NULL}};
const Segment SGI308713[] = {{&S9796[157], 9}, {NULL}};
const Segment SGI308714[] = {{&S9797[30], 3}, {NULL}};
const Segment SGI308715[] = {{&S9797[175], 9}, {NULL}};
const Segment SGI308716[] = {{&S9797[8], 3}, {NULL}};
const Segment SGI308717[] = {{&S9797[13], 10}, {NULL}};
const Segment SGI308718[] = {{&S9797[42], 7}, {NULL}};
const Segment SGI308719[] = {{&S9797[35], 14}, {NULL}};
const Segment SGI308720[] = {{&S9797[64], 11}, {NULL}};
const Segment SGI308721[] = {{&S9797[64], 16}, {NULL}};
const Segment SGI308722[] = {{&S9798[30], 3}, {NULL}};
const Segment SGI308723[] = {{&S9798[190], 9}, {NULL}};
const Segment SGI308724[] = {{&S9798[8], 3}, {NULL}};
const Segment SGI308725[] = {{&S9798[13], 10}, {NULL}};
const Segment SGI308726[] = {{&S9798[42], 7}, {NULL}};
const Segment SGI308727[] = {{&S9798[35], 14}, {NULL}};
const Segment SGI308728[] = {{&S9798[64], 11}, {NULL}};
const Segment SGI308729[] = {{&S9798[64], 16}, {NULL}};
const Segment SGI308730[] = {{&S9798[157], 9}, {NULL}};
const Segment SGI308731[] = {{&S10026[30], 3}, {NULL}};
const Segment SGI308732[] = {{&S10026[8], 3}, {NULL}};
const Segment SGI308733[] = {{&S10026[13], 10}, {NULL}};
const Segment SGI308734[] = {{&S10026[42], 7}, {NULL}};
const Segment SGI308735[] = {{&S10026[35], 14}, {NULL}};
const Segment SGI308736[] = {{&S10026[64], 11}, {NULL}};
const Segment SGI308737[] = {{&S10026[64], 16}, {NULL}};
const Segment SGI308738[] = {{&S10028[30], 3}, {NULL}};
const Segment SGI308739[] = {{&S10028[8], 3}, {NULL}};
const Segment SGI308740[] = {{&S10028[13], 10}, {NULL}};
const Segment SGI308741[] = {{&S10028[42], 7}, {NULL}};
const Segment SGI308742[] = {{&S10028[35], 14}, {NULL}};
const Segment SGI308743[] = {{&S10028[174], 11}, {NULL}};
const Segment SGI308744[] = {{&S10028[131], 15}, {NULL}};
const Segment SGI308745[] = {{&S10028[174], 16}, {NULL}};
const Segment SGI308746[] = {{&S10030[30], 3}, {NULL}};
const Segment SGI308747[] = {{&S10030[8], 3}, {NULL}};
const Segment SGI308748[] = {{&S10030[13], 10}, {NULL}};
const Segment SGI308749[] = {{&S10030[42], 7}, {NULL}};
const Segment SGI308750[] = {{&S10030[35], 14}, {NULL}};
const Segment SGI308751[] = {{&S10030[71], 11}, {NULL}};
const Segment SGI308752[] = {{&S10030[51], 6}, {NULL}};
const Segment SGI308753[] = {{&S10030[71], 16}, {NULL}};
const Segment SGI308754[] = {{&S10031[30], 3}, {NULL}};
const Segment SGI308755[] = {{&S10031[8], 3}, {NULL}};
const Segment SGI308756[] = {{&S10031[13], 10}, {NULL}};
const Segment SGI308757[] = {{&S10031[42], 7}, {NULL}};
const Segment SGI308758[] = {{&S10031[35], 14}, {NULL}};
const Segment SGI308759[] = {{&S10031[64], 11}, {NULL}};
const Segment SGI308760[] = {{&S10031[158], 15}, {NULL}};
const Segment SGI308761[] = {{&S10031[64], 16}, {NULL}};
const Segment SGI308762[] = {{&S10031[201], 9}, {NULL}};
const Segment SGI308763[] = {{&S10032[30], 3}, {NULL}};
const Segment SGI308764[] = {{&S10032[8], 3}, {NULL}};
const Segment SGI308765[] = {{&S10032[13], 10}, {NULL}};
const Segment SGI308766[] = {{&S10032[42], 7}, {NULL}};
const Segment SGI308767[] = {{&S10032[35], 14}, {NULL}};
const Segment SGI308768[] = {{&S10032[64], 11}, {NULL}};
const Segment SGI308769[] = {{&S10032[64], 16}, {NULL}};
const Segment SGI308770[] = {{&S10033[30], 4}, {NULL}};
const Segment SGI308771[] = {{&S10033[8], 3}, {NULL}};
const Segment SGI308772[] = {{&S10033[13], 10}, {NULL}};
const Segment SGI308773[] = {{&S10033[43], 7}, {NULL}};
const Segment SGI308774[] = {{&S10033[36], 14}, {NULL}};
const Segment SGI308775[] = {{&S10033[72], 11}, {NULL}};
const Segment SGI308776[] = {{&S10033[52], 6}, {NULL}};
const Segment SGI308777[] = {{&S10033[72], 16}, {NULL}};
const Segment SGI308778[] = {{&S10036[30], 3}, {NULL}};
const Segment SGI308779[] = {{&S10036[8], 3}, {NULL}};
const Segment SGI308780[] = {{&S10036[13], 10}, {NULL}};
const Segment SGI308781[] = {{&S10036[42], 7}, {NULL}};
const Segment SGI308782[] = {{&S10036[35], 14}, {NULL}};
const Segment SGI308783[] = {{&S10036[71], 11}, {NULL}};
const Segment SGI308784[] = {{&S10036[165], 15}, {NULL}};
const Segment SGI308785[] = {{&S10036[51], 6}, {NULL}};
const Segment SGI308786[] = {{&S10036[71], 16}, {NULL}};
const Segment SGI308787[] = {{&S10037[30], 4}, {NULL}};
const Segment SGI308788[] = {{&S10037[8], 3}, {NULL}};
const Segment SGI308789[] = {{&S10037[13], 10}, {NULL}};
const Segment SGI308790[] = {{&S10037[43], 7}, {NULL}};
const Segment SGI308791[] = {{&S10037[36], 14}, {NULL}};
const Segment SGI308792[] = {{&S10037[65], 11}, {NULL}};
const Segment SGI308793[] = {{&S10037[65], 16}, {NULL}};
const Segment SGI308794[] = {{&S10038[30], 3}, {NULL}};
const Segment SGI308795[] = {{&S10038[8], 3}, {NULL}};
const Segment SGI308796[] = {{&S10038[13], 10}, {NULL}};
const Segment SGI308797[] = {{&S10038[42], 7}, {NULL}};
const Segment SGI308798[] = {{&S10038[35], 14}, {NULL}};
const Segment SGI308799[] = {{&S10038[71], 11}, {NULL}};
const Segment SGI308800[] = {{&S10038[51], 6}, {NULL}};
const Segment SGI308801[] = {{&S10038[71], 16}, {NULL}};
const Segment SGI308802[] = {{&S10039[30], 3}, {NULL}};
const Segment SGI308803[] = {{&S10039[8], 3}, {NULL}};
const Segment SGI308804[] = {{&S10039[13], 10}, {NULL}};
const Segment SGI308805[] = {{&S10039[42], 7}, {NULL}};
const Segment SGI308806[] = {{&S10039[35], 14}, {NULL}};
const Segment SGI308807[] = {{&S10039[179], 11}, {NULL}};
const Segment SGI308808[] = {{&S10039[138], 15}, {NULL}};
const Segment SGI308809[] = {{&S10039[51], 6}, {NULL}};
const Segment SGI308810[] = {{&S10039[179], 16}, {NULL}};
const Segment SGI308811[] = {{&S10044[30], 3}, {NULL}};
const Segment SGI308812[] = {{&S10044[8], 3}, {NULL}};
const Segment SGI308813[] = {{&S10044[13], 10}, {NULL}};
const Segment SGI308814[] = {{&S10044[42], 7}, {NULL}};
const Segment SGI308815[] = {{&S10044[35], 14}, {NULL}};
const Segment SGI308816[] = {{&S10044[64], 11}, {NULL}};
const Segment SGI308817[] = {{&S10044[158], 15}, {NULL}};
const Segment SGI308818[] = {{&S10044[64], 16}, {NULL}};
const Segment SGI308819[] = {{&S10045[30], 3}, {NULL}};
const Segment SGI308820[] = {{&S10045[8], 3}, {NULL}};
const Segment SGI308821[] = {{&S10045[13], 10}, {NULL}};
const Segment SGI308822[] = {{&S10045[42], 7}, {NULL}};
const Segment SGI308823[] = {{&S10045[35], 14}, {NULL}};
const Segment SGI308824[] = {{&S10045[71], 11}, {NULL}};
const Segment SGI308825[] = {{&S10045[51], 6}, {NULL}};
const Segment SGI308826[] = {{&S10045[71], 16}, {NULL}};
const Segment SGI308827[] = {{&S10046[30], 3}, {NULL}};
const Segment SGI308828[] = {{&S10046[8], 3}, {NULL}};
const Segment SGI308829[] = {{&S10046[13], 10}, {NULL}};
const Segment SGI308830[] = {{&S10046[42], 7}, {NULL}};
const Segment SGI308831[] = {{&S10046[35], 14}, {NULL}};
const Segment SGI308832[] = {{&S10046[64], 11}, {NULL}};
const Segment SGI308833[] = {{&S10046[64], 16}, {NULL}};
const Segment SGI308834[] = {{&S10046[178], 9}, {NULL}};
const Segment SGI308835[] = {{&S10047[30], 3}, {NULL}};
const Segment SGI308836[] = {{&S10047[8], 3}, {NULL}};
const Segment SGI308837[] = {{&S10047[13], 10}, {NULL}};
const Segment SGI308838[] = {{&S10047[42], 7}, {NULL}};
const Segment SGI308839[] = {{&S10047[35], 14}, {NULL}};
const Segment SGI308840[] = {{&S10047[64], 11}, {NULL}};
const Segment SGI308841[] = {{&S10047[158], 15}, {NULL}};
const Segment SGI308842[] = {{&S10047[64], 16}, {NULL}};
const Segment SGI308843[] = {{&S10047[201], 9}, {NULL}};
const Segment SGI308844[] = {{&S10048[30], 3}, {NULL}};
const Segment SGI308845[] = {{&S10048[8], 3}, {NULL}};
const Segment SGI308846[] = {{&S10048[13], 10}, {NULL}};
const Segment SGI308847[] = {{&S10048[42], 7}, {NULL}};
const Segment SGI308848[] = {{&S10048[35], 14}, {NULL}};
const Segment SGI308849[] = {{&S10048[76], 11}, {NULL}};
const Segment SGI308850[] = {{&S10048[170], 15}, {NULL}};
const Segment SGI308851[] = {{&S10048[76], 16}, {NULL}};
const Segment SGI308852[] = {{&S10048[51], 5}, {NULL}};
const Segment SGI308853[] = {{&S10049[30], 3}, {NULL}};
const Segment SGI308854[] = {{&S10049[8], 3}, {NULL}};
const Segment SGI308855[] = {{&S10049[13], 10}, {NULL}};
const Segment SGI308856[] = {{&S10049[42], 7}, {NULL}};
const Segment SGI308857[] = {{&S10049[35], 14}, {NULL}};
const Segment SGI308858[] = {{&S10049[71], 11}, {NULL}};
const Segment SGI308859[] = {{&S10049[164], 15}, {NULL}};
const Segment SGI308860[] = {{&S10049[217], 4}, {NULL}};
const Segment SGI308861[] = {{&S10049[51], 6}, {NULL}};
const Segment SGI308862[] = {{&S10049[71], 16}, {NULL}};
const Segment SGI308863[] = {{&S10050[30], 3}, {NULL}};
const Segment SGI308864[] = {{&S10050[237], 9}, {NULL}};
const Segment SGI308865[] = {{&S10050[8], 3}, {NULL}};
const Segment SGI308866[] = {{&S10050[13], 10}, {NULL}};
const Segment SGI308867[] = {{&S10050[42], 7}, {NULL}};
const Segment SGI308868[] = {{&S10050[35], 14}, {NULL}};
const Segment SGI308869[] = {{&S10050[71], 11}, {NULL}};
const Segment SGI308870[] = {{&S10050[166], 15}, {NULL}};
const Segment SGI308871[] = {{&S10050[51], 6}, {NULL}};
const Segment SGI308872[] = {{&S10050[71], 16}, {NULL}};
const Segment SGI308873[] = {{&S10051[30], 3}, {NULL}};
const Segment SGI308874[] = {{&S10051[8], 3}, {NULL}};
const Segment SGI308875[] = {{&S10051[13], 10}, {NULL}};
const Segment SGI308876[] = {{&S10051[42], 7}, {NULL}};
const Segment SGI308877[] = {{&S10051[35], 14}, {NULL}};
const Segment SGI308878[] = {{&S10051[69], 11}, {NULL}};
const Segment SGI308879[] = {{&S10051[163], 15}, {NULL}};
const Segment SGI308880[] = {{&S10051[69], 16}, {NULL}};
const Segment SGI308881[] = {{&S10051[64], 3}, {NULL}};
const Segment SGI308882[] = {{&S10324[30], 3}, {NULL}};
const Segment SGI308883[] = {{&S10324[8], 3}, {NULL}};
const Segment SGI308884[] = {{&S10324[13], 10}, {NULL}};
const Segment SGI308885[] = {{&S10324[42], 7}, {NULL}};
const Segment SGI308886[] = {{&S10325[30], 3}, {NULL}};
const Segment SGI308887[] = {{&S10325[8], 3}, {NULL}};
const Segment SGI308888[] = {{&S10325[13], 10}, {NULL}};
const Segment SGI308889[] = {{&S10325[42], 7}, {NULL}};
const Segment SGI308890[] = {{&S10325[35], 14}, {NULL}};
const Segment SGI308891[] = {{&S10326[30], 3}, {NULL}};
const Segment SGI308892[] = {{&S10326[8], 3}, {NULL}};
const Segment SGI308893[] = {{&S10326[13], 10}, {NULL}};
const Segment SGI308894[] = {{&S10326[42], 7}, {NULL}};
const Segment SGI308895[] = {{&S10326[35], 14}, {NULL}};
const Segment SGI308896[] = {{&S10327[30], 3}, {NULL}};
const Segment SGI308897[] = {{&S10327[8], 3}, {NULL}};
const Segment SGI308898[] = {{&S10327[13], 10}, {NULL}};
const Segment SGI308899[] = {{&S10327[42], 7}, {NULL}};
const Segment SGI308900[] = {{&S10328[30], 3}, {NULL}};
const Segment SGI308901[] = {{&S10328[8], 3}, {NULL}};
const Segment SGI308902[] = {{&S10328[13], 10}, {NULL}};
const Segment SGI308903[] = {{&S10328[69], 7}, {NULL}};
const Segment SGI308904[] = {{&S10328[62], 14}, {NULL}};
const Segment SGI308905[] = {{&S10328[78], 6}, {NULL}};
const Segment SGI308906[] = {{&S10329[30], 3}, {NULL}};
const Segment SGI308907[] = {{&S10329[8], 3}, {NULL}};
const Segment SGI308908[] = {{&S10329[13], 10}, {NULL}};
const Segment SGI308909[] = {{&S10329[42], 7}, {NULL}};
const Segment SGI308910[] = {{&S10329[35], 14}, {NULL}};
const Segment SGI308911[] = {{&S10329[131], 15}, {NULL}};
const Segment SGI308912[] = {{&S10330[30], 3}, {NULL}};
const Segment SGI308913[] = {{&S10330[8], 3}, {NULL}};
const Segment SGI308914[] = {{&S10330[13], 10}, {NULL}};
const Segment SGI308915[] = {{&S10330[42], 7}, {NULL}};
const Segment SGI308916[] = {{&S10330[35], 14}, {NULL}};
const Segment SGI308917[] = {{&S10330[51], 6}, {NULL}};
const Segment SGI308918[] = {{&S10331[8], 3}, {&S10331[94], 3}, {NULL}};
const Segment SGI308919[] = {{&S10331[8], 3}, {&S10331[59], 3}, {NULL}};
const Segment SGI308920[] = {{&S10331[13], 10}, {&S10331[13], 10}, {NULL}};
const Segment SGI308921[] = {{&S10331[42], 7}, {&S10331[42], 7}, {NULL}};
const Segment SGI308922[] = {{&S10331[35], 14}, {&S10331[35], 14}, {NULL}};
const Segment SGI308923[] = {{&S10331[115], 3}, {NULL}};
const Segment SGI308924[] = {{&S10332[30], 3}, {NULL}};
const Segment SGI308925[] = {{&S10332[8], 3}, {NULL}};
const Segment SGI308926[] = {{&S10332[13], 10}, {NULL}};
const Segment SGI308927[] = {{&S10332[42], 7}, {NULL}};
const Segment SGI308928[] = {{&S10332[35], 14}, {NULL}};
const Segment SGI308929[] = {{&S10332[138], 15}, {NULL}};
const Segment SGI308930[] = {{&S10332[51], 6}, {NULL}};
const Segment SGI308931[] = {{&S10333[30], 3}, {NULL}};
const Segment SGI308932[] = {{&S10333[8], 3}, {NULL}};
const Segment SGI308933[] = {{&S10333[13], 10}, {NULL}};
const Segment SGI308934[] = {{&S10333[55], 7}, {NULL}};
const Segment SGI308935[] = {{&S10333[48], 14}, {NULL}};
const Segment SGI308936[] = {{&S10333[64], 3}, {NULL}};
const Segment SGI308937[] = {{&S10334[30], 3}, {NULL}};
const Segment SGI308938[] = {{&S10334[8], 3}, {NULL}};
const Segment SGI308939[] = {{&S10334[13], 10}, {NULL}};
const Segment SGI308940[] = {{&S10334[55], 7}, {NULL}};
const Segment SGI308941[] = {{&S10334[48], 14}, {NULL}};
const Segment SGI308942[] = {{&S10334[64], 3}, {NULL}};
const Segment SGI308943[] = {{&S10335[30], 3}, {NULL}};
const Segment SGI308944[] = {{&S10335[8], 3}, {NULL}};
const Segment SGI308945[] = {{&S10335[13], 10}, {NULL}};
const Segment SGI308946[] = {{&S10335[55], 7}, {NULL}};
const Segment SGI308947[] = {{&S10335[48], 14}, {NULL}};
const Segment SGI308948[] = {{&S10335[64], 3}, {NULL}};
const Segment SGI308949[] = {{&S10336[30], 3}, {NULL}};
const Segment SGI308950[] = {{&S10336[8], 3}, {NULL}};
const Segment SGI308951[] = {{&S10336[13], 10}, {NULL}};
const Segment SGI308952[] = {{&S10336[55], 7}, {NULL}};
const Segment SGI308953[] = {{&S10336[48], 14}, {NULL}};
const Segment SGI308954[] = {{&S10337[30], 3}, {&S10337[86], 3}, {NULL}};
const Segment SGI308955[] = {{&S10337[8], 3}, {&S10337[8], 3}, {NULL}};
const Segment SGI308956[] = {{&S10337[13], 10}, {&S10337[13], 10}, {NULL}};
const Segment SGI308957[] = {{&S10337[42], 7}, {&S10337[42], 7}, {NULL}};
const Segment SGI308958[] = {{&S10337[35], 14}, {&S10337[35], 14}, {NULL}};
const Segment SGI308959[] = {{&S10337[51], 3}, {&S10337[51], 3}, {NULL}};
const Segment SGI308960[] = {{&S10338[30], 3}, {NULL}};
const Segment SGI308961[] = {{&S10338[8], 3}, {NULL}};
const Segment SGI308962[] = {{&S10338[13], 10}, {NULL}};
const Segment SGI308963[] = {{&S10338[55], 7}, {NULL}};
const Segment SGI308964[] = {{&S10338[48], 14}, {NULL}};
const Segment SGI308965[] = {{&S10338[64], 3}, {NULL}};
const Segment SGI308966[] = {{&S10339[30], 3}, {NULL}};
const Segment SGI308967[] = {{&S10339[8], 3}, {NULL}};
const Segment SGI308968[] = {{&S10339[13], 10}, {NULL}};
const Segment SGI308969[] = {{&S10339[42], 7}, {NULL}};
const Segment SGI308970[] = {{&S10339[35], 14}, {NULL}};
const Segment SGI308971[] = {{&S10339[102], 15}, {NULL}};
const Segment SGI308972[] = {{&S10340[30], 3}, {NULL}};
const Segment SGI308973[] = {{&S10340[8], 3}, {NULL}};
const Segment SGI308974[] = {{&S10340[13], 7}, {NULL}};
const Segment SGI308975[] = {{&S10340[42], 7}, {NULL}};
const Segment SGI308976[] = {{&S10340[35], 14}, {NULL}};
const Segment SGI308977[] = {{&S10341[30], 3}, {NULL}};
const Segment SGI308978[] = {{&S10341[8], 3}, {NULL}};
const Segment SGI308979[] = {{&S10341[13], 10}, {NULL}};
const Segment SGI308980[] = {{&S10341[42], 7}, {NULL}};
const Segment SGI308981[] = {{&S10341[35], 14}, {NULL}};
const Segment SGI308982[] = {{&S10341[131], 15}, {NULL}};
const Segment SGI308983[] = {{&S10341[152], 9}, {NULL}};
const Segment SGI308984[] = {{&S10342[30], 3}, {NULL}};
const Segment SGI308985[] = {{&S10342[8], 3}, {NULL}};
const Segment SGI308986[] = {{&S10342[13], 10}, {NULL}};
const Segment SGI308987[] = {{&S10342[42], 7}, {&S10342[145], 20}, {NULL}};
const Segment SGI308988[] = {{&S10342[35], 14}, {NULL}};
const Segment SGI308989[] = {{&S10342[146], 15}, {NULL}};
const Segment SGI308990[] = {{&S10343[75], 14}, {NULL}};
const Segment SGI308991[] = {{&S10343[8], 3}, {NULL}};
const Segment SGI308992[] = {{&S10343[13], 10}, {NULL}};
const Segment SGI308993[] = {{&S10343[61], 7}, {NULL}};
const Segment SGI308994[] = {{&S10344[30], 3}, {NULL}};
const Segment SGI308995[] = {{&S10344[8], 3}, {NULL}};
const Segment SGI308996[] = {{&S10344[13], 10}, {NULL}};
const Segment SGI308997[] = {{&S10344[42], 7}, {NULL}};
const Segment SGI308998[] = {{&S10344[35], 14}, {NULL}};
const Segment SGI308999[] = {{&S10344[64], 9}, {NULL}};
const Segment SGI309000[] = {{&S10345[34], 3}, {NULL}};
const Segment SGI309001[] = {{&S10345[8], 3}, {NULL}};
const Segment SGI309002[] = {{&S10345[13], 14}, {NULL}};
const Segment SGI309003[] = {{&S10345[20], 7}, {NULL}};
const Segment SGI309004[] = {{&S10346[30], 3}, {NULL}};
const Segment SGI309005[] = {{&S10346[8], 3}, {NULL}};
const Segment SGI309006[] = {{&S10346[13], 10}, {NULL}};
const Segment SGI309007[] = {{&S10346[42], 7}, {NULL}};
const Segment SGI309008[] = {{&S10346[35], 14}, {NULL}};
const Segment SGI309009[] = {{&S10347[30], 3}, {NULL}};
const Segment SGI309010[] = {{&S10347[8], 3}, {NULL}};
const Segment SGI309011[] = {{&S10347[13], 10}, {NULL}};
const Segment SGI309012[] = {{&S10347[42], 7}, {NULL}};
const Segment SGI309013[] = {{&S10347[35], 14}, {NULL}};
const Segment SGI309014[] = {{&S10347[147], 4}, {NULL}};
const Segment SGI309015[] = {{&S10348[30], 3}, {&S10348[94], 3}, {NULL}};
const Segment SGI309016[] = {{&S10348[8], 3}, {&S10348[59], 3}, {NULL}};
const Segment SGI309017[] = {{&S10348[13], 10}, {&S10348[13], 10}, {NULL}};
const Segment SGI309018[] = {{&S10348[42], 7}, {&S10348[42], 7}, {NULL}};
const Segment SGI309019[] = {{&S10348[35], 14}, {&S10348[35], 14}, {NULL}};
const Segment SGI309020[] = {{&S10348[115], 3}, {NULL}};
const Segment SGI309021[] = {{&S10349[30], 3}, {NULL}};
const Segment SGI309022[] = {{&S10349[8], 3}, {NULL}};
const Segment SGI309023[] = {{&S10349[13], 10}, {NULL}};
const Segment SGI309024[] = {{&S10349[55], 7}, {NULL}};
const Segment SGI309025[] = {{&S10349[48], 14}, {NULL}};
const Segment SGI309026[] = {{&S10349[64], 3}, {&S10349[64], 3}, {NULL}};
const Segment SGI309027[] = {{&S10350[30], 3}, {NULL}};
const Segment SGI309028[] = {{&S10350[8], 3}, {NULL}};
const Segment SGI309029[] = {{&S10350[13], 10}, {NULL}};
const Segment SGI309030[] = {{&S10350[55], 7}, {NULL}};
const Segment SGI309031[] = {{&S10350[48], 14}, {NULL}};
const Segment SGI309032[] = {{&S10350[64], 3}, {&S10350[64], 3}, {NULL}};
const Segment SGI309033[] = {{&S10351[30], 3}, {NULL}};
const Segment SGI309034[] = {{&S10351[8], 3}, {NULL}};
const Segment SGI309035[] = {{&S10351[13], 10}, {NULL}};
const Segment SGI309036[] = {{&S10351[55], 7}, {NULL}};
const Segment SGI309037[] = {{&S10351[48], 14}, {NULL}};
const Segment SGI309038[] = {{&S10351[64], 3}, {&S10351[64], 3}, {NULL}};
const Segment SGI309039[] = {{&S10352[30], 3}, {NULL}};
const Segment SGI309040[] = {{&S10352[8], 3}, {NULL}};
const Segment SGI309041[] = {{&S10352[13], 10}, {NULL}};
const Segment SGI309042[] = {{&S10352[55], 7}, {NULL}};
const Segment SGI309043[] = {{&S10352[48], 14}, {NULL}};
const Segment SGI309044[] = {{&S10352[64], 3}, {&S10352[64], 3}, {NULL}};
const Segment SGI309045[] = {{&S10353[30], 3}, {NULL}};
const Segment SGI309046[] = {{&S10353[8], 3}, {NULL}};
const Segment SGI309047[] = {{&S10353[13], 10}, {NULL}};
const Segment SGI309048[] = {{&S10353[42], 7}, {NULL}};
const Segment SGI309049[] = {{&S10353[35], 14}, {NULL}};
const Segment SGI309050[] = {{&S10353[96], 9}, {NULL}};
const Segment SGI309051[] = {{&S12101[30], 3}, {NULL}};
const Segment SGI309052[] = {{&S12101[185], 9}, {NULL}};
const Segment SGI309053[] = {{&S12101[8], 3}, {NULL}};
const Segment SGI309054[] = {{&S12101[13], 10}, {NULL}};
const Segment SGI309055[] = {{&S12101[42], 7}, {NULL}};
const Segment SGI309056[] = {{&S12101[35], 14}, {NULL}};
const Segment SGI309057[] = {{&S12104[30], 3}, {NULL}};
const Segment SGI309058[] = {{&S12104[177], 9}, {NULL}};
const Segment SGI309059[] = {{&S12104[8], 3}, {NULL}};
const Segment SGI309060[] = {{&S12104[13], 10}, {NULL}};
const Segment SGI309061[] = {{&S12104[42], 7}, {NULL}};
const Segment SGI309062[] = {{&S12104[35], 14}, {NULL}};
const Segment SGI309063[] = {{&S12105[30], 3}, {NULL}};
const Segment SGI309064[] = {{&S12105[186], 9}, {NULL}};
const Segment SGI309065[] = {{&S12105[8], 3}, {NULL}};
const Segment SGI309066[] = {{&S12105[13], 10}, {NULL}};
const Segment SGI309067[] = {{&S12105[42], 7}, {NULL}};
const Segment SGI309068[] = {{&S12105[35], 14}, {NULL}};
const Segment SGI309069[] = {{&S12106[30], 3}, {NULL}};
const Segment SGI309070[] = {{&S12106[165], 9}, {NULL}};
const Segment SGI309071[] = {{&S12106[8], 3}, {NULL}};
const Segment SGI309072[] = {{&S12106[13], 10}, {NULL}};
const Segment SGI309073[] = {{&S12106[42], 7}, {NULL}};
const Segment SGI309074[] = {{&S12106[35], 14}, {NULL}};
const Segment SGI309075[] = {{&S12108[30], 3}, {NULL}};
const Segment SGI309076[] = {{&S12108[165], 9}, {NULL}};
const Segment SGI309077[] = {{&S12108[8], 3}, {NULL}};
const Segment SGI309078[] = {{&S12108[13], 10}, {NULL}};
const Segment SGI309079[] = {{&S12108[42], 7}, {NULL}};
const Segment SGI309080[] = {{&S12108[35], 14}, {NULL}};
const Segment SGI309081[] = {{&S12108[51], 6}, {NULL}};
const Segment SGI309082[] = {{&S12109[30], 3}, {NULL}};
const Segment SGI309083[] = {{&S12109[165], 9}, {NULL}};
const Segment SGI309084[] = {{&S12109[8], 3}, {NULL}};
const Segment SGI309085[] = {{&S12109[13], 10}, {NULL}};
const Segment SGI309086[] = {{&S12109[42], 7}, {NULL}};
const Segment SGI309087[] = {{&S12109[35], 14}, {NULL}};
const Segment SGI309088[] = {{&S12109[51], 6}, {NULL}};
const Segment SGI309089[] = {{&S12110[30], 3}, {NULL}};
const Segment SGI309090[] = {{&S12110[185], 9}, {NULL}};
const Segment SGI309091[] = {{&S12110[8], 3}, {NULL}};
const Segment SGI309092[] = {{&S12110[13], 10}, {NULL}};
const Segment SGI309093[] = {{&S12110[42], 7}, {NULL}};
const Segment SGI309094[] = {{&S12110[35], 14}, {NULL}};
const Segment SGI309095[] = {{&S12111[30], 3}, {NULL}};
const Segment SGI309096[] = {{&S12111[185], 9}, {NULL}};
const Segment SGI309097[] = {{&S12111[8], 3}, {NULL}};
const Segment SGI309098[] = {{&S12111[13], 10}, {NULL}};
const Segment SGI309099[] = {{&S12111[42], 7}, {NULL}};
const Segment SGI309100[] = {{&S12111[35], 14}, {NULL}};
const Segment SGI309101[] = {{&S12112[30], 3}, {NULL}};
const Segment SGI309102[] = {{&S12112[197], 9}, {NULL}};
const Segment SGI309103[] = {{&S12112[8], 3}, {NULL}};
const Segment SGI309104[] = {{&S12112[13], 10}, {NULL}};
const Segment SGI309105[] = {{&S12112[42], 7}, {NULL}};
const Segment SGI309106[] = {{&S12112[35], 14}, {NULL}};
const Segment SGI309107[] = {{&S12113[30], 3}, {NULL}};
const Segment SGI309108[] = {{&S12113[185], 9}, {NULL}};
const Segment SGI309109[] = {{&S12113[8], 3}, {NULL}};
const Segment SGI309110[] = {{&S12113[13], 10}, {NULL}};
const Segment SGI309111[] = {{&S12113[42], 7}, {NULL}};
const Segment SGI309112[] = {{&S12113[35], 14}, {NULL}};
const Segment SGI309113[] = {{&S12114[30], 3}, {NULL}};
const Segment SGI309114[] = {{&S12114[205], 9}, {NULL}};
const Segment SGI309115[] = {{&S12114[8], 3}, {NULL}};
const Segment SGI309116[] = {{&S12114[13], 10}, {NULL}};
const Segment SGI309117[] = {{&S12114[42], 7}, {NULL}};
const Segment SGI309118[] = {{&S12114[35], 14}, {NULL}};
const Segment SGI309119[] = {{&S12114[102], 15}, {NULL}};
const Segment SGI309120[] = {{&S12115[30], 3}, {NULL}};
const Segment SGI309121[] = {{&S12115[177], 9}, {NULL}};
const Segment SGI309122[] = {{&S12115[8], 3}, {NULL}};
const Segment SGI309123[] = {{&S12115[13], 10}, {NULL}};
const Segment SGI309124[] = {{&S12115[42], 7}, {NULL}};
const Segment SGI309125[] = {{&S12115[35], 14}, {NULL}};
const Segment SGI309126[] = {{&S12116[30], 3}, {NULL}};
const Segment SGI309127[] = {{&S12116[165], 9}, {NULL}};
const Segment SGI309128[] = {{&S12116[8], 3}, {NULL}};
const Segment SGI309129[] = {{&S12116[13], 10}, {NULL}};
const Segment SGI309130[] = {{&S12116[42], 7}, {NULL}};
const Segment SGI309131[] = {{&S12116[35], 14}, {NULL}};
const Segment SGI309132[] = {{&S12116[51], 6}, {NULL}};
const Segment SGI309133[] = {{&S12118[30], 3}, {NULL}};
const Segment SGI309134[] = {{&S12118[173], 9}, {NULL}};
const Segment SGI309135[] = {{&S12118[8], 3}, {NULL}};
const Segment SGI309136[] = {{&S12118[13], 10}, {NULL}};
const Segment SGI309137[] = {{&S12118[42], 7}, {NULL}};
const Segment SGI309138[] = {{&S12118[35], 14}, {NULL}};
const Segment SGI309139[] = {{&S12118[51], 6}, {NULL}};
const Segment SGI309140[] = {{&S12120[30], 3}, {NULL}};
const Segment SGI309141[] = {{&S12120[225], 9}, {NULL}};
const Segment SGI309142[] = {{&S12120[8], 3}, {NULL}};
const Segment SGI309143[] = {{&S12120[13], 10}, {NULL}};
const Segment SGI309144[] = {{&S12120[42], 7}, {NULL}};
const Segment SGI309145[] = {{&S12120[35], 14}, {NULL}};
const Segment SGI309146[] = {{&S12120[110], 15}, {NULL}};
const Segment SGI309147[] = {{&S12121[30], 3}, {NULL}};
const Segment SGI309148[] = {{&S12121[165], 9}, {NULL}};
const Segment SGI309149[] = {{&S12121[8], 3}, {NULL}};
const Segment SGI309150[] = {{&S12121[13], 10}, {NULL}};
const Segment SGI309151[] = {{&S12121[42], 7}, {NULL}};
const Segment SGI309152[] = {{&S12121[35], 14}, {NULL}};
const Segment SGI309153[] = {{&S12122[30], 3}, {NULL}};
const Segment SGI309154[] = {{&S12122[138], 9}, {NULL}};
const Segment SGI309155[] = {{&S12122[8], 3}, {NULL}};
const Segment SGI309156[] = {{&S12122[13], 10}, {NULL}};
const Segment SGI309157[] = {{&S12122[42], 7}, {NULL}};
const Segment SGI309158[] = {{&S12122[35], 14}, {NULL}};
const Segment SGI309159[] = {{&S12122[51], 3}, {NULL}};
const Segment SGI309160[] = {{&S12123[30], 3}, {NULL}};
const Segment SGI309161[] = {{&S12123[185], 9}, {NULL}};
const Segment SGI309162[] = {{&S12123[8], 3}, {NULL}};
const Segment SGI309163[] = {{&S12123[13], 10}, {NULL}};
const Segment SGI309164[] = {{&S12123[42], 7}, {NULL}};
const Segment SGI309165[] = {{&S12123[35], 14}, {NULL}};
const Segment SGI309166[] = {{&S12124[30], 3}, {NULL}};
const Segment SGI309167[] = {{&S12124[185], 9}, {NULL}};
const Segment SGI309168[] = {{&S12124[8], 3}, {NULL}};
const Segment SGI309169[] = {{&S12124[13], 10}, {NULL}};
const Segment SGI309170[] = {{&S12124[42], 7}, {NULL}};
const Segment SGI309171[] = {{&S12124[35], 14}, {NULL}};
const Segment SGI309172[] = {{&S12126[30], 3}, {NULL}};
const Segment SGI309173[] = {{&S12126[217], 9}, {NULL}};
const Segment SGI309174[] = {{&S12126[8], 3}, {NULL}};
const Segment SGI309175[] = {{&S12126[13], 10}, {NULL}};
const Segment SGI309176[] = {{&S12126[42], 7}, {NULL}};
const Segment SGI309177[] = {{&S12126[35], 14}, {NULL}};
const Segment SGI309178[] = {{&S12126[102], 15}, {NULL}};
const Segment SGI309179[] = {{&S12127[30], 3}, {NULL}};
const Segment SGI309180[] = {{&S12127[173], 9}, {NULL}};
const Segment SGI309181[] = {{&S12127[8], 3}, {NULL}};
const Segment SGI309182[] = {{&S12127[13], 10}, {NULL}};
const Segment SGI309183[] = {{&S12127[42], 7}, {NULL}};
const Segment SGI309184[] = {{&S12127[35], 14}, {NULL}};
const Segment SGI309185[] = {{&S12127[51], 6}, {NULL}};
const Segment SGI309186[] = {{&S12128[30], 3}, {NULL}};
const Segment SGI309187[] = {{&S12128[185], 9}, {NULL}};
const Segment SGI309188[] = {{&S12128[8], 3}, {NULL}};
const Segment SGI309189[] = {{&S12128[13], 10}, {NULL}};
const Segment SGI309190[] = {{&S12128[42], 7}, {NULL}};
const Segment SGI309191[] = {{&S12128[35], 14}, {NULL}};
const Segment SGI309192[] = {{&S12129[30], 3}, {NULL}};
const Segment SGI309193[] = {{&S12129[121], 9}, {NULL}};
const Segment SGI309194[] = {{&S12129[8], 3}, {NULL}};
const Segment SGI309195[] = {{&S12129[13], 10}, {NULL}};
const Segment SGI309196[] = {{&S12129[42], 7}, {NULL}};
const Segment SGI309197[] = {{&S12129[35], 14}, {NULL}};
const Segment SGI309198[] = {{&S12130[30], 3}, {NULL}};
const Segment SGI309199[] = {{&S12130[185], 9}, {NULL}};
const Segment SGI309200[] = {{&S12130[8], 3}, {NULL}};
const Segment SGI309201[] = {{&S12130[13], 10}, {NULL}};
const Segment SGI309202[] = {{&S12130[42], 7}, {NULL}};
const Segment SGI309203[] = {{&S12130[35], 14}, {NULL}};
const Segment SGI309204[] = {{&S12131[30], 3}, {NULL}};
const Segment SGI309205[] = {{&S12131[205], 9}, {NULL}};
const Segment SGI309206[] = {{&S12131[8], 3}, {NULL}};
const Segment SGI309207[] = {{&S12131[13], 10}, {NULL}};
const Segment SGI309208[] = {{&S12131[42], 7}, {NULL}};
const Segment SGI309209[] = {{&S12131[35], 14}, {NULL}};
const Segment SGI309210[] = {{&S12131[102], 15}, {NULL}};
const Segment SGI309211[] = {{&S12132[30], 3}, {NULL}};
const Segment SGI309212[] = {{&S12132[173], 9}, {NULL}};
const Segment SGI309213[] = {{&S12132[8], 3}, {NULL}};
const Segment SGI309214[] = {{&S12132[13], 10}, {NULL}};
const Segment SGI309215[] = {{&S12132[42], 7}, {NULL}};
const Segment SGI309216[] = {{&S12132[35], 14}, {NULL}};
const Segment SGI309217[] = {{&S12132[51], 6}, {NULL}};
const Segment SGI309218[] = {{&S12133[30], 3}, {NULL}};
const Segment SGI309219[] = {{&S12133[138], 9}, {NULL}};
const Segment SGI309220[] = {{&S12133[8], 3}, {NULL}};
const Segment SGI309221[] = {{&S12133[13], 10}, {NULL}};
const Segment SGI309222[] = {{&S12133[42], 7}, {NULL}};
const Segment SGI309223[] = {{&S12133[35], 14}, {NULL}};
const Segment SGI309224[] = {{&S12133[51], 3}, {NULL}};
const Segment SGI309225[] = {{&S12134[30], 3}, {NULL}};
const Segment SGI309226[] = {{&S12134[158], 9}, {NULL}};
const Segment SGI309227[] = {{&S12134[8], 3}, {NULL}};
const Segment SGI309228[] = {{&S12134[13], 10}, {NULL}};
const Segment SGI309229[] = {{&S12134[42], 7}, {NULL}};
const Segment SGI309230[] = {{&S12134[35], 14}, {NULL}};
const Segment SGI309231[] = {{&S12135[30], 3}, {&S12135[94], 3}, {NULL}};
const Segment SGI309232[] = {{&S12135[228], 9}, {NULL}};
const Segment SGI309233[] = {{&S12135[8], 3}, {&S12135[8], 3}, {NULL}};
const Segment SGI309234[] = {{&S12135[13], 10}, {&S12135[13], 10}, {NULL}};
const Segment SGI309235[] = {{&S12135[42], 7}, {&S12135[42], 7}, {NULL}};
const Segment SGI309236[] = {{&S12135[35], 14}, {&S12135[35], 14}, {NULL}};
const Segment SGI309237[] = {{&S12135[115], 3}, {NULL}};
const Segment SGI309238[] = {{&S12137[30], 3}, {NULL}};
const Segment SGI309239[] = {{&S12137[185], 9}, {NULL}};
const Segment SGI309240[] = {{&S12137[8], 3}, {NULL}};
const Segment SGI309241[] = {{&S12137[13], 10}, {NULL}};
const Segment SGI309242[] = {{&S12137[42], 7}, {NULL}};
const Segment SGI309243[] = {{&S12137[35], 14}, {NULL}};
const Segment SGI309244[] = {{&S12138[30], 3}, {NULL}};
const Segment SGI309245[] = {{&S12138[198], 9}, {NULL}};
const Segment SGI309246[] = {{&S12138[8], 3}, {NULL}};
const Segment SGI309247[] = {{&S12138[13], 10}, {NULL}};
const Segment SGI309248[] = {{&S12138[42], 7}, {NULL}};
const Segment SGI309249[] = {{&S12138[35], 14}, {NULL}};
const Segment SGI309250[] = {{&S12138[51], 6}, {NULL}};
const Segment SGI309251[] = {{&S12139[30], 3}, {NULL}};
const Segment SGI309252[] = {{&S12139[185], 9}, {NULL}};
const Segment SGI309253[] = {{&S12139[8], 3}, {NULL}};
const Segment SGI309254[] = {{&S12139[13], 10}, {NULL}};
const Segment SGI309255[] = {{&S12139[42], 7}, {NULL}};
const Segment SGI309256[] = {{&S12139[35], 14}, {NULL}};
const Segment SGI309257[] = {{&S12141[30], 3}, {NULL}};
const Segment SGI309258[] = {{&S12141[185], 9}, {NULL}};
const Segment SGI309259[] = {{&S12141[8], 3}, {NULL}};
const Segment SGI309260[] = {{&S12141[13], 10}, {NULL}};
const Segment SGI309261[] = {{&S12141[42], 7}, {NULL}};
const Segment SGI309262[] = {{&S12141[35], 14}, {NULL}};
const Segment SGI309263[] = {{&S12142[30], 3}, {NULL}};
const Segment SGI309264[] = {{&S12142[185], 9}, {NULL}};
const Segment SGI309265[] = {{&S12142[8], 3}, {NULL}};
const Segment SGI309266[] = {{&S12142[13], 10}, {NULL}};
const Segment SGI309267[] = {{&S12142[42], 7}, {NULL}};
const Segment SGI309268[] = {{&S12142[35], 14}, {NULL}};
const Segment SGI309269[] = {{&S12143[30], 3}, {NULL}};
const Segment SGI309270[] = {{&S12143[210], 9}, {NULL}};
const Segment SGI309271[] = {{&S12143[8], 3}, {NULL}};
const Segment SGI309272[] = {{&S12143[13], 10}, {NULL}};
const Segment SGI309273[] = {{&S12143[42], 7}, {NULL}};
const Segment SGI309274[] = {{&S12143[35], 14}, {NULL}};
const Segment SGI309275[] = {{&S12198[30], 4}, {NULL}};
const Segment SGI309276[] = {{&S12198[8], 3}, {NULL}};
const Segment SGI309277[] = {{&S12198[13], 10}, {NULL}};
const Segment SGI309278[] = {{&S12198[43], 3}, {NULL}};
const Segment SGI309279[] = {{&S12199[30], 3}, {NULL}};
const Segment SGI309280[] = {{&S12199[8], 3}, {NULL}};
const Segment SGI309281[] = {{&S12199[13], 10}, {NULL}};
const Segment SGI309282[] = {{&S12199[42], 3}, {NULL}};
const Segment SGI309283[] = {{&S12200[30], 3}, {NULL}};
const Segment SGI309284[] = {{&S12200[8], 3}, {NULL}};
const Segment SGI309285[] = {{&S12200[13], 10}, {NULL}};
const Segment SGI309286[] = {{&S12200[42], 3}, {NULL}};
const Segment SGI309287[] = {{&S12201[30], 3}, {NULL}};
const Segment SGI309288[] = {{&S12201[8], 3}, {NULL}};
const Segment SGI309289[] = {{&S12201[13], 10}, {NULL}};
const Segment SGI309290[] = {{&S12201[42], 3}, {NULL}};
const Segment SGI309291[] = {{&S12202[30], 3}, {NULL}};
const Segment SGI309292[] = {{&S12202[8], 3}, {NULL}};
const Segment SGI309293[] = {{&S12202[13], 10}, {NULL}};
const Segment SGI309294[] = {{&S12202[42], 3}, {NULL}};
const Segment SGI309295[] = {{&S12203[30], 4}, {NULL}};
const Segment SGI309296[] = {{&S12203[8], 3}, {NULL}};
const Segment SGI309297[] = {{&S12203[13], 10}, {NULL}};
const Segment SGI309298[] = {{&S12203[43], 3}, {NULL}};
const Segment SGI309299[] = {{&S12204[30], 3}, {NULL}};
const Segment SGI309300[] = {{&S12204[8], 3}, {NULL}};
const Segment SGI309301[] = {{&S12204[13], 10}, {NULL}};
const Segment SGI309302[] = {{&S12205[30], 3}, {NULL}};
const Segment SGI309303[] = {{&S12205[8], 3}, {NULL}};
const Segment SGI309304[] = {{&S12205[13], 10}, {NULL}};
const Segment SGI309305[] = {{&S12205[35], 5}, {NULL}};
const Segment SGI309306[] = {{&S14965[30], 3}, {NULL}};
const Segment SGI309307[] = {{&S14965[8], 3}, {NULL}};
const Segment SGI309308[] = {{&S14965[13], 10}, {NULL}};
const Segment SGI309309[] = {{&S14965[42], 7}, {NULL}};
const Segment SGI309310[] = {{&S14965[35], 14}, {NULL}};
const Segment SGI309311[] = {{&S14965[173], 4}, {NULL}};
const Segment SGI309312[] = {{&S14965[71], 5}, {NULL}};
const Segment SGI309313[] = {{&S14965[51], 6}, {NULL}};
const Segment SGI309314[] = {{&S14965[158], 9}, {NULL}};
const Segment SGI309315[] = {{&S14966[30], 3}, {NULL}};
const Segment SGI309316[] = {{&S14966[8], 3}, {NULL}};
const Segment SGI309317[] = {{&S14966[13], 10}, {NULL}};
const Segment SGI309318[] = {{&S14966[42], 7}, {NULL}};
const Segment SGI309319[] = {{&S14966[35], 14}, {NULL}};
const Segment SGI309320[] = {{&S14966[178], 15}, {NULL}};
const Segment SGI309321[] = {{&S14966[76], 5}, {NULL}};
const Segment SGI309322[] = {{&S14966[51], 5}, {NULL}};
const Segment SGI309323[] = {{&S14966[163], 9}, {NULL}};
const Segment SGI309324[] = {{&S14967[30], 3}, {NULL}};
const Segment SGI309325[] = {{&S14967[8], 3}, {NULL}};
const Segment SGI309326[] = {{&S14967[13], 10}, {NULL}};
const Segment SGI309327[] = {{&S14967[42], 7}, {NULL}};
const Segment SGI309328[] = {{&S14967[35], 14}, {NULL}};
const Segment SGI309329[] = {{&S14967[173], 15}, {NULL}};
const Segment SGI309330[] = {{&S14967[71], 5}, {NULL}};
const Segment SGI309331[] = {{&S14967[51], 6}, {NULL}};
const Segment SGI309332[] = {{&S14967[158], 9}, {NULL}};
const Segment SGI309333[] = {{&S14968[30], 3}, {NULL}};
const Segment SGI309334[] = {{&S14968[8], 3}, {NULL}};
const Segment SGI309335[] = {{&S14968[13], 10}, {NULL}};
const Segment SGI309336[] = {{&S14968[42], 7}, {NULL}};
const Segment SGI309337[] = {{&S14968[35], 14}, {NULL}};
const Segment SGI309338[] = {{&S14968[71], 5}, {NULL}};
const Segment SGI309339[] = {{&S14968[51], 6}, {NULL}};
const Segment SGI309340[] = {{&S14968[158], 9}, {NULL}};
const Segment SGI309341[] = {{&S14969[30], 3}, {NULL}};
const Segment SGI309342[] = {{&S14969[8], 3}, {NULL}};
const Segment SGI309343[] = {{&S14969[13], 10}, {NULL}};
const Segment SGI309344[] = {{&S14969[42], 7}, {NULL}};
const Segment SGI309345[] = {{&S14969[35], 14}, {NULL}};
const Segment SGI309346[] = {{&S14969[178], 4}, {NULL}};
const Segment SGI309347[] = {{&S14969[76], 5}, {NULL}};
const Segment SGI309348[] = {{&S14969[51], 5}, {NULL}};
const Segment SGI309349[] = {{&S14969[163], 9}, {NULL}};
const Segment SGI309350[] = {{&S14970[30], 3}, {NULL}};
const Segment SGI309351[] = {{&S14970[8], 3}, {NULL}};
const Segment SGI309352[] = {{&S14970[13], 10}, {NULL}};
const Segment SGI309353[] = {{&S14970[42], 7}, {NULL}};
const Segment SGI309354[] = {{&S14970[35], 14}, {NULL}};
const Segment SGI309355[] = {{&S14970[71], 5}, {NULL}};
const Segment SGI309356[] = {{&S14970[51], 6}, {NULL}};
const Segment SGI309357[] = {{&S14971[30], 3}, {NULL}};
const Segment SGI309358[] = {{&S14971[8], 3}, {NULL}};
const Segment SGI309359[] = {{&S14971[13], 10}, {NULL}};
const Segment SGI309360[] = {{&S14971[42], 7}, {NULL}};
const Segment SGI309361[] = {{&S14971[35], 14}, {NULL}};
const Segment SGI309362[] = {{&S14971[76], 5}, {NULL}};
const Segment SGI309363[] = {{&S14971[51], 5}, {NULL}};
const Segment SGI309364[] = {{&S14971[163], 9}, {NULL}};
const Segment SGI309365[] = {{&S14972[30], 3}, {NULL}};
const Segment SGI309366[] = {{&S14972[8], 3}, {NULL}};
const Segment SGI309367[] = {{&S14972[13], 10}, {NULL}};
const Segment SGI309368[] = {{&S14972[42], 7}, {NULL}};
const Segment SGI309369[] = {{&S14972[35], 14}, {NULL}};
const Segment SGI309370[] = {{&S14972[64], 5}, {NULL}};
const Segment SGI309371[] = {{&S14972[151], 9}, {NULL}};
const Segment SGI309372[] = {{&S14973[30], 3}, {NULL}};
const Segment SGI309373[] = {{&S14973[8], 3}, {NULL}};
const Segment SGI309374[] = {{&S14973[13], 10}, {NULL}};
const Segment SGI309375[] = {{&S14973[42], 7}, {NULL}};
const Segment SGI309376[] = {{&S14973[35], 14}, {NULL}};
const Segment SGI309377[] = {{&S14973[76], 5}, {NULL}};
const Segment SGI309378[] = {{&S14973[51], 5}, {NULL}};
const Segment SGI309379[] = {{&S14974[30], 3}, {NULL}};
const Segment SGI309380[] = {{&S14974[8], 3}, {NULL}};
const Segment SGI309381[] = {{&S14974[13], 10}, {NULL}};
const Segment SGI309382[] = {{&S14974[42], 7}, {NULL}};
const Segment SGI309383[] = {{&S14974[35], 14}, {NULL}};
const Segment SGI309384[] = {{&S14974[178], 15}, {NULL}};
const Segment SGI309385[] = {{&S14974[206], 4}, {NULL}};
const Segment SGI309386[] = {{&S14974[76], 5}, {NULL}};
const Segment SGI309387[] = {{&S14974[51], 5}, {NULL}};
const Segment SGI309388[] = {{&S14974[163], 9}, {NULL}};
const Segment SGI309389[] = {{&S14975[30], 3}, {NULL}};
const Segment SGI309390[] = {{&S14975[8], 3}, {NULL}};
const Segment SGI309391[] = {{&S14975[13], 10}, {NULL}};
const Segment SGI309392[] = {{&S14975[42], 7}, {NULL}};
const Segment SGI309393[] = {{&S14975[35], 14}, {NULL}};
const Segment SGI309394[] = {{&S14975[166], 4}, {NULL}};
const Segment SGI309395[] = {{&S14975[64], 5}, {NULL}};
const Segment SGI309396[] = {{&S14975[91], 9}, {NULL}};
const Segment SGI309397[] = {{&S14976[30], 3}, {NULL}};
const Segment SGI309398[] = {{&S14976[8], 3}, {NULL}};
const Segment SGI309399[] = {{&S14976[13], 10}, {NULL}};
const Segment SGI309400[] = {{&S14976[42], 7}, {NULL}};
const Segment SGI309401[] = {{&S14976[35], 14}, {NULL}};
const Segment SGI309402[] = {{&S14976[166], 15}, {NULL}};
const Segment SGI309403[] = {{&S14976[64], 5}, {NULL}};
const Segment SGI309404[] = {{&S14976[151], 9}, {NULL}};
const Segment SGI309405[] = {{&S14977[30], 3}, {NULL}};
const Segment SGI309406[] = {{&S14977[8], 3}, {NULL}};
const Segment SGI309407[] = {{&S14977[13], 10}, {NULL}};
const Segment SGI309408[] = {{&S14977[42], 7}, {NULL}};
const Segment SGI309409[] = {{&S14977[35], 14}, {NULL}};
const Segment SGI309410[] = {{&S14977[113], 15}, {NULL}};
const Segment SGI309411[] = {{&S14977[194], 4}, {NULL}};
const Segment SGI309412[] = {{&S14977[71], 5}, {NULL}};
const Segment SGI309413[] = {{&S14977[51], 6}, {NULL}};
const Segment SGI309414[] = {{&S14977[98], 9}, {NULL}};
const Segment SGI309415[] = {{&S15743[30], 3}, {NULL}};
const Segment SGI309416[] = {{&S15743[8], 3}, {NULL}};
const Segment SGI309417[] = {{&S15743[13], 10}, {NULL}};
const Segment SGI309418[] = {{&S15743[42], 7}, {NULL}};
const Segment SGI309419[] = {{&S15743[35], 14}, {NULL}};
const Segment SGI309420[] = {{&S15743[71], 11}, {NULL}};
const Segment SGI309421[] = {{&S15743[164], 15}, {NULL}};
const Segment SGI309422[] = {{&S15743[51], 6}, {NULL}};
const Segment SGI309423[] = {{&S15743[71], 16}, {NULL}};
const Segment SGI309424[] = {{&S15744[30], 3}, {NULL}};
const Segment SGI309425[] = {{&S15744[8], 3}, {NULL}};
const Segment SGI309426[] = {{&S15744[13], 10}, {NULL}};
const Segment SGI309427[] = {{&S15744[42], 7}, {NULL}};
const Segment SGI309428[] = {{&S15744[35], 14}, {NULL}};
const Segment SGI309429[] = {{&S15744[64], 11}, {NULL}};
const Segment SGI309430[] = {{&S15744[64], 16}, {NULL}};
const Segment SGI309431[] = {{&S15745[30], 3}, {NULL}};
const Segment SGI309432[] = {{&S15745[8], 3}, {NULL}};
const Segment SGI309433[] = {{&S15745[13], 10}, {NULL}};
const Segment SGI309434[] = {{&S15745[42], 7}, {NULL}};
const Segment SGI309435[] = {{&S15745[35], 14}, {NULL}};
const Segment SGI309436[] = {{&S15745[71], 11}, {NULL}};
const Segment SGI309437[] = {{&S15745[51], 6}, {NULL}};
const Segment SGI309438[] = {{&S15745[71], 16}, {NULL}};
const Segment SGI309439[] = {{&S15748[30], 3}, {NULL}};
const Segment SGI309440[] = {{&S15748[8], 3}, {NULL}};
const Segment SGI309441[] = {{&S15748[13], 10}, {NULL}};
const Segment SGI309442[] = {{&S15748[42], 7}, {NULL}};
const Segment SGI309443[] = {{&S15748[35], 14}, {NULL}};
const Segment SGI309444[] = {{&S15748[64], 11}, {NULL}};
const Segment SGI309445[] = {{&S15748[157], 15}, {NULL}};
const Segment SGI309446[] = {{&S15748[64], 16}, {NULL}};
const Segment SGI309447[] = {{&S15748[200], 9}, {NULL}};
const Segment SGI309448[] = {{&S15749[30], 3}, {NULL}};
const Segment SGI309449[] = {{&S15749[8], 3}, {NULL}};
const Segment SGI309450[] = {{&S15749[13], 10}, {NULL}};
const Segment SGI309451[] = {{&S15749[42], 7}, {NULL}};
const Segment SGI309452[] = {{&S15749[35], 14}, {NULL}};
const Segment SGI309453[] = {{&S15749[64], 11}, {NULL}};
const Segment SGI309454[] = {{&S15749[157], 15}, {NULL}};
const Segment SGI309455[] = {{&S15749[64], 16}, {NULL}};
const Segment SGI309456[] = {{&S15750[30], 3}, {NULL}};
const Segment SGI309457[] = {{&S15750[8], 3}, {NULL}};
const Segment SGI309458[] = {{&S15750[13], 10}, {NULL}};
const Segment SGI309459[] = {{&S15750[42], 7}, {NULL}};
const Segment SGI309460[] = {{&S15750[35], 14}, {NULL}};
const Segment SGI309461[] = {{&S15750[169], 11}, {NULL}};
const Segment SGI309462[] = {{&S15750[138], 15}, {NULL}};
const Segment SGI309463[] = {{&S15750[51], 6}, {NULL}};
const Segment SGI309464[] = {{&S15750[169], 16}, {NULL}};
const Segment SGI309465[] = {{&S15751[30], 3}, {NULL}};
const Segment SGI309466[] = {{&S15751[8], 3}, {NULL}};
const Segment SGI309467[] = {{&S15751[13], 10}, {NULL}};
const Segment SGI309468[] = {{&S15751[42], 7}, {NULL}};
const Segment SGI309469[] = {{&S15751[35], 14}, {NULL}};
const Segment SGI309470[] = {{&S15751[71], 11}, {NULL}};
const Segment SGI309471[] = {{&S15751[51], 6}, {NULL}};
const Segment SGI309472[] = {{&S15751[71], 16}, {NULL}};
const Segment SGI309473[] = {{&S15753[30], 3}, {NULL}};
const Segment SGI309474[] = {{&S15753[8], 3}, {NULL}};
const Segment SGI309475[] = {{&S15753[13], 10}, {NULL}};
const Segment SGI309476[] = {{&S15753[42], 7}, {NULL}};
const Segment SGI309477[] = {{&S15753[35], 14}, {NULL}};
const Segment SGI309478[] = {{&S15753[64], 11}, {NULL}};
const Segment SGI309479[] = {{&S15753[64], 16}, {NULL}};
const Segment SGI309480[] = {{&S15753[177], 9}, {NULL}};
const Segment SGI309481[] = {{&S15755[30], 3}, {NULL}};
const Segment SGI309482[] = {{&S15755[8], 3}, {NULL}};
const Segment SGI309483[] = {{&S15755[13], 10}, {NULL}};
const Segment SGI309484[] = {{&S15755[42], 7}, {NULL}};
const Segment SGI309485[] = {{&S15755[35], 14}, {NULL}};
const Segment SGI309486[] = {{&S15755[64], 11}, {NULL}};
const Segment SGI309487[] = {{&S15755[64], 16}, {NULL}};
const Segment SGI309488[] = {{&S15756[30], 3}, {NULL}};
const Segment SGI309489[] = {{&S15756[8], 3}, {NULL}};
const Segment SGI309490[] = {{&S15756[13], 10}, {NULL}};
const Segment SGI309491[] = {{&S15756[42], 7}, {NULL}};
const Segment SGI309492[] = {{&S15756[35], 14}, {NULL}};
const Segment SGI309493[] = {{&S15756[174], 11}, {NULL}};
const Segment SGI309494[] = {{&S15756[131], 15}, {NULL}};
const Segment SGI309495[] = {{&S15756[174], 16}, {NULL}};
const Segment SGI309496[] = {{&S15757[30], 3}, {NULL}};
const Segment SGI309497[] = {{&S15757[8], 3}, {NULL}};
const Segment SGI309498[] = {{&S15757[13], 10}, {NULL}};
const Segment SGI309499[] = {{&S15757[42], 7}, {NULL}};
const Segment SGI309500[] = {{&S15757[35], 14}, {NULL}};
const Segment SGI309501[] = {{&S15757[84], 11}, {NULL}};
const Segment SGI309502[] = {{&S15757[177], 15}, {NULL}};
const Segment SGI309503[] = {{&S15757[84], 16}, {NULL}};
const Segment SGI309504[] = {{&S15757[51], 5}, {NULL}};
const Segment SGI309505[] = {{&S15758[30], 3}, {NULL}};
const Segment SGI309506[] = {{&S15758[8], 3}, {NULL}};
const Segment SGI309507[] = {{&S15758[13], 10}, {NULL}};
const Segment SGI309508[] = {{&S15758[42], 7}, {NULL}};
const Segment SGI309509[] = {{&S15758[35], 14}, {NULL}};
const Segment SGI309510[] = {{&S15758[177], 11}, {NULL}};
const Segment SGI309511[] = {{&S15758[131], 15}, {NULL}};
const Segment SGI309512[] = {{&S15758[177], 16}, {NULL}};
const Segment SGI309513[] = {{&S15758[162], 9}, {NULL}};
const Segment SGI309514[] = {{&S15759[30], 3}, {NULL}};
const Segment SGI309515[] = {{&S15759[8], 3}, {NULL}};
const Segment SGI309516[] = {{&S15759[13], 10}, {NULL}};
const Segment SGI309517[] = {{&S15759[42], 7}, {NULL}};
const Segment SGI309518[] = {{&S15759[35], 14}, {NULL}};
const Segment SGI309519[] = {{&S15759[141], 11}, {NULL}};
const Segment SGI309520[] = {{&S15759[141], 16}, {NULL}};
const Segment SGI309521[] = {{&S15761[30], 3}, {NULL}};
const Segment SGI309522[] = {{&S15761[8], 3}, {NULL}};
const Segment SGI309523[] = {{&S15761[13], 10}, {NULL}};
const Segment SGI309524[] = {{&S15761[42], 7}, {NULL}};
const Segment SGI309525[] = {{&S15761[35], 14}, {NULL}};
const Segment SGI309526[] = {{&S15761[86], 11}, {NULL}};
const Segment SGI309527[] = {{&S15761[51], 6}, {NULL}};
const Segment SGI309528[] = {{&S15761[71], 13}, {NULL}};
const Segment SGI309529[] = {{&S15761[86], 16}, {NULL}};
const Segment SGI309530[] = {{&S15764[30], 3}, {NULL}};
const Segment SGI309531[] = {{&S15764[8], 3}, {NULL}};
const Segment SGI309532[] = {{&S15764[13], 10}, {NULL}};
const Segment SGI309533[] = {{&S15764[42], 7}, {NULL}};
const Segment SGI309534[] = {{&S15764[35], 14}, {NULL}};
const Segment SGI309535[] = {{&S15764[84], 11}, {NULL}};
const Segment SGI309536[] = {{&S15764[177], 15}, {NULL}};
const Segment SGI309537[] = {{&S15764[84], 16}, {NULL}};
const Segment SGI309538[] = {{&S15764[51], 5}, {NULL}};
const Segment SGI309539[] = {{&S15764[198], 9}, {NULL}};
const Segment SGI309540[] = {{&S15765[30], 3}, {NULL}};
const Segment SGI309541[] = {{&S15765[8], 3}, {NULL}};
const Segment SGI309542[] = {{&S15765[13], 10}, {NULL}};
const Segment SGI309543[] = {{&S15765[42], 7}, {NULL}};
const Segment SGI309544[] = {{&S15765[35], 14}, {NULL}};
const Segment SGI309545[] = {{&S15765[71], 11}, {NULL}};
const Segment SGI309546[] = {{&S15765[51], 6}, {NULL}};
const Segment SGI309547[] = {{&S15765[71], 16}, {NULL}};
const Segment SGI309548[] = {{&S15766[30], 3}, {NULL}};
const Segment SGI309549[] = {{&S15766[8], 3}, {NULL}};
const Segment SGI309550[] = {{&S15766[13], 10}, {NULL}};
const Segment SGI309551[] = {{&S15766[42], 7}, {NULL}};
const Segment SGI309552[] = {{&S15766[35], 14}, {NULL}};
const Segment SGI309553[] = {{&S15766[79], 11}, {NULL}};
const Segment SGI309554[] = {{&S15766[172], 15}, {NULL}};
const Segment SGI309555[] = {{&S15766[51], 6}, {NULL}};
const Segment SGI309556[] = {{&S15766[79], 16}, {NULL}};
const Segment SGI309557[] = {{&S15767[30], 3}, {NULL}};
const Segment SGI309558[] = {{&S15767[8], 3}, {NULL}};
const Segment SGI309559[] = {{&S15767[13], 10}, {NULL}};
const Segment SGI309560[] = {{&S15767[42], 7}, {NULL}};
const Segment SGI309561[] = {{&S15767[35], 14}, {NULL}};
const Segment SGI309562[] = {{&S15767[71], 11}, {NULL}};
const Segment SGI309563[] = {{&S15767[164], 15}, {NULL}};
const Segment SGI309564[] = {{&S15767[217], 4}, {NULL}};
const Segment SGI309565[] = {{&S15767[51], 6}, {NULL}};
const Segment SGI309566[] = {{&S15767[71], 16}, {NULL}};
const Segment SGI309567[] = {{&S15768[30], 4}, {NULL}};
const Segment SGI309568[] = {{&S15768[8], 3}, {NULL}};
const Segment SGI309569[] = {{&S15768[13], 10}, {NULL}};
const Segment SGI309570[] = {{&S15768[43], 7}, {NULL}};
const Segment SGI309571[] = {{&S15768[36], 14}, {NULL}};
const Segment SGI309572[] = {{&S15768[72], 11}, {NULL}};
const Segment SGI309573[] = {{&S15768[52], 6}, {NULL}};
const Segment SGI309574[] = {{&S15768[72], 16}, {NULL}};
const Segment SGI309575[] = {{&S15769[30], 4}, {NULL}};
const Segment SGI309576[] = {{&S15769[8], 3}, {NULL}};
const Segment SGI309577[] = {{&S15769[13], 10}, {NULL}};
const Segment SGI309578[] = {{&S15769[43], 7}, {NULL}};
const Segment SGI309579[] = {{&S15769[36], 14}, {NULL}};
const Segment SGI309580[] = {{&S15769[65], 11}, {NULL}};
const Segment SGI309581[] = {{&S15769[65], 16}, {NULL}};
const Segment SGI309582[] = {{&S17464[30], 3}, {NULL}};
const Segment SGI309583[] = {{&S17464[8], 3}, {NULL}};
const Segment SGI309584[] = {{&S17464[13], 10}, {NULL}};
const Segment SGI309585[] = {{&S17464[42], 7}, {NULL}};
const Segment SGI309586[] = {{&S17464[35], 14}, {NULL}};
const Segment SGI309587[] = {{&S17465[30], 3}, {NULL}};
const Segment SGI309588[] = {{&S17465[8], 3}, {NULL}};
const Segment SGI309589[] = {{&S17465[13], 10}, {NULL}};
const Segment SGI309590[] = {{&S17465[42], 7}, {NULL}};
const Segment SGI309591[] = {{&S17465[35], 14}, {NULL}};
const Segment SGI309592[] = {{&S17465[51], 5}, {NULL}};
const Segment SGI309593[] = {{&S17466[30], 3}, {NULL}};
const Segment SGI309594[] = {{&S17466[8], 3}, {NULL}};
const Segment SGI309595[] = {{&S17466[13], 10}, {NULL}};
const Segment SGI309596[] = {{&S17466[42], 7}, {NULL}};
const Segment SGI309597[] = {{&S17467[30], 3}, {NULL}};
const Segment SGI309598[] = {{&S17467[8], 3}, {NULL}};
const Segment SGI309599[] = {{&S17467[13], 10}, {NULL}};
const Segment SGI309600[] = {{&S17467[42], 7}, {NULL}};
const Segment SGI309601[] = {{&S17467[35], 14}, {NULL}};
const Segment SGI309602[] = {{&S17467[51], 6}, {NULL}};
const Segment SGI309603[] = {{&S17468[30], 4}, {NULL}};
const Segment SGI309604[] = {{&S17468[8], 3}, {NULL}};
const Segment SGI309605[] = {{&S17468[13], 10}, {NULL}};
const Segment SGI309606[] = {{&S17468[43], 7}, {NULL}};
const Segment SGI309607[] = {{&S17468[36], 14}, {NULL}};
const Segment SGI309608[] = {{&S17469[30], 4}, {NULL}};
const Segment SGI309609[] = {{&S17469[8], 3}, {NULL}};
const Segment SGI309610[] = {{&S17469[13], 10}, {NULL}};
const Segment SGI309611[] = {{&S17469[43], 7}, {&S17469[59], 16}, {NULL}};
const Segment SGI309612[] = {{&S17469[36], 14}, {NULL}};
const Segment SGI309613[] = {{&S17470[72], 11}, {NULL}};
const Segment SGI309614[] = {{&S17470[8], 3}, {NULL}};
const Segment SGI309615[] = {{&S17470[13], 10}, {NULL}};
const Segment SGI309616[] = {{&S17470[61], 4}, {NULL}};
const Segment SGI309617[] = {{&S17471[72], 15}, {NULL}};
const Segment SGI309618[] = {{&S17471[8], 3}, {NULL}};
const Segment SGI309619[] = {{&S17471[13], 10}, {NULL}};
const Segment SGI309620[] = {{&S17471[61], 4}, {NULL}};
const Segment SGI309621[] = {{&S17472[72], 15}, {NULL}};
const Segment SGI309622[] = {{&S17472[8], 3}, {NULL}};
const Segment SGI309623[] = {{&S17472[13], 10}, {NULL}};
const Segment SGI309624[] = {{&S17472[61], 4}, {NULL}};
const Segment SGI309625[] = {{&S17473[30], 3}, {NULL}};
const Segment SGI309626[] = {{&S17473[8], 3}, {NULL}};
const Segment SGI309627[] = {{&S17473[13], 10}, {NULL}};
const Segment SGI309628[] = {{&S17473[42], 7}, {NULL}};
const Segment SGI309629[] = {{&S17473[35], 14}, {NULL}};
const Segment SGI309630[] = {{&S17473[72], 4}, {NULL}};
const Segment SGI309631[] = {{&S17474[30], 3}, {NULL}};
const Segment SGI309632[] = {{&S17474[8], 3}, {NULL}};
const Segment SGI309633[] = {{&S17474[13], 10}, {NULL}};
const Segment SGI309634[] = {{&S17474[42], 7}, {NULL}};
const Segment SGI309635[] = {{&S17474[35], 14}, {NULL}};
const Segment SGI309636[] = {{&S17474[151], 15}, {NULL}};
const Segment SGI309637[] = {{&S17474[51], 6}, {NULL}};
const Segment SGI309638[] = {{&S17475[72], 15}, {NULL}};
const Segment SGI309639[] = {{&S17475[8], 3}, {NULL}};
const Segment SGI309640[] = {{&S17475[13], 10}, {NULL}};
const Segment SGI309641[] = {{&S17475[61], 4}, {NULL}};
const Segment SGI309642[] = {{&S17476[72], 15}, {NULL}};
const Segment SGI309643[] = {{&S17476[8], 3}, {NULL}};
const Segment SGI309644[] = {{&S17476[13], 10}, {NULL}};
const Segment SGI309645[] = {{&S17476[61], 4}, {NULL}};
const Segment SGI309646[] = {{&S17477[30], 3}, {NULL}};
const Segment SGI309647[] = {{&S17477[8], 3}, {NULL}};
const Segment SGI309648[] = {{&S17477[13], 10}, {NULL}};
const Segment SGI309649[] = {{&S17477[42], 7}, {NULL}};
const Segment SGI309650[] = {{&S17477[35], 14}, {NULL}};
const Segment SGI309651[] = {{&S17477[156], 15}, {NULL}};
const Segment SGI309652[] = {{&S17477[51], 5}, {NULL}};
const Segment SGI309653[] = {{&S17477[184], 9}, {NULL}};
const Segment SGI309654[] = {{&S17478[30], 3}, {NULL}};
const Segment SGI309655[] = {{&S17478[8], 3}, {NULL}};
const Segment SGI309656[] = {{&S17478[13], 10}, {NULL}};
const Segment SGI309657[] = {{&S17478[42], 7}, {NULL}};
const Segment SGI309658[] = {{&S17478[35], 14}, {NULL}};
const Segment SGI309659[] = {{&S17478[156], 15}, {NULL}};
const Segment SGI309660[] = {{&S17478[51], 5}, {NULL}};
const Segment SGI309661[] = {{&S17479[72], 15}, {NULL}};
const Segment SGI309662[] = {{&S17479[8], 3}, {NULL}};
const Segment SGI309663[] = {{&S17479[13], 10}, {NULL}};
const Segment SGI309664[] = {{&S17479[61], 4}, {NULL}};
const Segment SGI309665[] = {{&S17480[30], 3}, {NULL}};
const Segment SGI309666[] = {{&S17480[8], 3}, {NULL}};
const Segment SGI309667[] = {{&S17480[13], 10}, {NULL}};
const Segment SGI309668[] = {{&S17480[42], 7}, {NULL}};
const Segment SGI309669[] = {{&S17480[35], 14}, {NULL}};
const Segment SGI309670[] = {{&S17480[51], 6}, {NULL}};
const Segment SGI309671[] = {{&S17480[151], 9}, {NULL}};
const Segment SGI309672[] = {{&S17481[72], 15}, {NULL}};
const Segment SGI309673[] = {{&S17481[8], 3}, {NULL}};
const Segment SGI309674[] = {{&S17481[13], 10}, {NULL}};
const Segment SGI309675[] = {{&S17481[61], 4}, {NULL}};
const Segment SGI309676[] = {{&S17482[72], 15}, {NULL}};
const Segment SGI309677[] = {{&S17482[8], 3}, {NULL}};
const Segment SGI309678[] = {{&S17482[13], 10}, {NULL}};
const Segment SGI309679[] = {{&S17482[61], 4}, {NULL}};
const Segment SGI309680[] = {{&S17483[30], 3}, {NULL}};
const Segment SGI309681[] = {{&S17483[8], 3}, {NULL}};
const Segment SGI309682[] = {{&S17483[13], 10}, {NULL}};
const Segment SGI309683[] = {{&S17483[42], 7}, {NULL}};
const Segment SGI309684[] = {{&S17483[35], 14}, {NULL}};
const Segment SGI309685[] = {{&S17483[51], 6}, {NULL}};
const Segment SGI309686[] = {{&S17484[30], 4}, {NULL}};
const Segment SGI309687[] = {{&S17484[8], 3}, {NULL}};
const Segment SGI309688[] = {{&S17484[13], 10}, {NULL}};
const Segment SGI309689[] = {{&S17484[43], 7}, {NULL}};
const Segment SGI309690[] = {{&S17485[72], 15}, {NULL}};
const Segment SGI309691[] = {{&S17485[8], 3}, {NULL}};
const Segment SGI309692[] = {{&S17485[13], 10}, {NULL}};
const Segment SGI309693[] = {{&S17485[61], 4}, {NULL}};
const Segment SGI309694[] = {{&S17486[30], 3}, {NULL}};
const Segment SGI309695[] = {{&S17486[8], 3}, {NULL}};
const Segment SGI309696[] = {{&S17486[13], 10}, {NULL}};
const Segment SGI309697[] = {{&S17486[42], 7}, {NULL}};
const Segment SGI309698[] = {{&S17486[35], 14}, {NULL}};
const Segment SGI309699[] = {{&S17487[30], 3}, {NULL}};
const Segment SGI309700[] = {{&S17487[8], 3}, {NULL}};
const Segment SGI309701[] = {{&S17487[13], 10}, {NULL}};
const Segment SGI309702[] = {{&S17487[42], 7}, {NULL}};
const Segment SGI309703[] = {{&S17487[35], 14}, {NULL}};
const Segment SGI309704[] = {{&S17487[51], 5}, {NULL}};
const Segment SGI309705[] = {{&S17487[156], 9}, {NULL}};
const Segment SGI309706[] = {{&S17488[72], 15}, {NULL}};
const Segment SGI309707[] = {{&S17488[8], 3}, {NULL}};
const Segment SGI309708[] = {{&S17488[13], 10}, {NULL}};
const Segment SGI309709[] = {{&S17488[61], 4}, {NULL}};
const Segment SGI309710[] = {{&S17489[30], 4}, {NULL}};
const Segment SGI309711[] = {{&S17489[8], 3}, {NULL}};
const Segment SGI309712[] = {{&S17489[13], 10}, {NULL}};
const Segment SGI309713[] = {{&S17489[43], 7}, {NULL}};
const Segment SGI309714[] = {{&S17489[36], 14}, {NULL}};
const Segment SGI309715[] = {{&S17490[72], 15}, {NULL}};
const Segment SGI309716[] = {{&S17490[8], 3}, {NULL}};
const Segment SGI309717[] = {{&S17490[13], 10}, {NULL}};
const Segment SGI309718[] = {{&S17490[61], 4}, {NULL}};
const Segment SGI309719[] = {{&S17491[72], 15}, {NULL}};
const Segment SGI309720[] = {{&S17491[8], 3}, {NULL}};
const Segment SGI309721[] = {{&S17491[13], 10}, {NULL}};
const Segment SGI309722[] = {{&S17491[61], 4}, {NULL}};
const Segment SGI309723[] = {{&S17492[30], 4}, {NULL}};
const Segment SGI309724[] = {{&S17492[8], 3}, {NULL}};
const Segment SGI309725[] = {{&S17492[13], 10}, {NULL}};
const Segment SGI309726[] = {{&S17492[43], 7}, {NULL}};
const Segment SGI309727[] = {{&S17492[36], 14}, {NULL}};
const Segment SGI309728[] = {{&S17492[52], 6}, {NULL}};
const Segment SGI309729[] = {{&S17493[72], 15}, {NULL}};
const Segment SGI309730[] = {{&S17493[8], 3}, {NULL}};
const Segment SGI309731[] = {{&S17493[13], 10}, {NULL}};
const Segment SGI309732[] = {{&S17493[61], 4}, {NULL}};
const Segment SGI309733[] = {{&S17494[30], 3}, {NULL}};
const Segment SGI309734[] = {{&S17494[191], 9}, {&S17494[191], 9}, {NULL}};
const Segment SGI309735[] = {{&S17494[8], 3}, {NULL}};
const Segment SGI309736[] = {{&S17494[13], 10}, {NULL}};
const Segment SGI309737[] = {{&S17494[42], 7}, {NULL}};
const Segment SGI309738[] = {{&S17494[35], 14}, {NULL}};
const Segment SGI309739[] = {{&S17494[151], 4}, {NULL}};
const Segment SGI309740[] = {{&S17494[51], 6}, {NULL}};
const Segment SGI309741[] = {{&S17495[30], 3}, {NULL}};
const Segment SGI309742[] = {{&S17495[181], 9}, {&S17495[181], 9}, {NULL}};
const Segment SGI309743[] = {{&S17495[8], 3}, {NULL}};
const Segment SGI309744[] = {{&S17495[13], 10}, {NULL}};
const Segment SGI309745[] = {{&S17495[42], 7}, {NULL}};
const Segment SGI309746[] = {{&S17495[35], 14}, {NULL}};
const Segment SGI309747[] = {{&S17495[51], 6}, {NULL}};
const Segment SGI309748[] = {{&S17496[30], 3}, {NULL}};
const Segment SGI309749[] = {{&S17496[186], 9}, {NULL}};
const Segment SGI309750[] = {{&S17496[8], 3}, {NULL}};
const Segment SGI309751[] = {{&S17496[13], 10}, {NULL}};
const Segment SGI309752[] = {{&S17496[42], 7}, {NULL}};
const Segment SGI309753[] = {{&S17496[35], 14}, {NULL}};
const Segment SGI309754[] = {{&S17496[51], 5}, {NULL}};
const Segment SGI309755[] = {{&S17497[30], 3}, {NULL}};
const Segment SGI309756[] = {{&S17497[169], 9}, {NULL}};
const Segment SGI309757[] = {{&S17497[8], 3}, {NULL}};
const Segment SGI309758[] = {{&S17497[13], 10}, {NULL}};
const Segment SGI309759[] = {{&S17497[42], 7}, {NULL}};
const Segment SGI309760[] = {{&S17497[35], 14}, {NULL}};
const Segment SGI309761[] = {{&S17497[51], 6}, {NULL}};
const Segment SGI309762[] = {{&S17499[30], 3}, {NULL}};
const Segment SGI309763[] = {{&S17499[196], 9}, {&S17499[196], 9}, {NULL}};
const Segment SGI309764[] = {{&S17499[8], 3}, {NULL}};
const Segment SGI309765[] = {{&S17499[13], 10}, {NULL}};
const Segment SGI309766[] = {{&S17499[42], 7}, {NULL}};
const Segment SGI309767[] = {{&S17499[35], 14}, {NULL}};
const Segment SGI309768[] = {{&S17499[51], 6}, {NULL}};
const Segment SGI309769[] = {{&S17499[151], 9}, {NULL}};
const Segment SGI309770[] = {{&S17500[30], 3}, {NULL}};
const Segment SGI309771[] = {{&S17500[196], 9}, {&S17500[196], 9}, {NULL}};
const Segment SGI309772[] = {{&S17500[8], 3}, {NULL}};
const Segment SGI309773[] = {{&S17500[13], 10}, {NULL}};
const Segment SGI309774[] = {{&S17500[42], 7}, {NULL}};
const Segment SGI309775[] = {{&S17500[35], 14}, {NULL}};
const Segment SGI309776[] = {{&S17500[168], 4}, {NULL}};
const Segment SGI309777[] = {{&S17500[51], 5}, {NULL}};
const Segment SGI309778[] = {{&S17501[30], 3}, {NULL}};
const Segment SGI309779[] = {{&S17501[206], 9}, {&S17501[206], 9}, {NULL}};
const Segment SGI309780[] = {{&S17501[8], 3}, {NULL}};
const Segment SGI309781[] = {{&S17501[13], 10}, {NULL}};
const Segment SGI309782[] = {{&S17501[42], 7}, {NULL}};
const Segment SGI309783[] = {{&S17501[35], 14}, {NULL}};
const Segment SGI309784[] = {{&S17501[166], 4}, {NULL}};
const Segment SGI309785[] = {{&S17501[51], 6}, {NULL}};
const Segment SGI309786[] = {{&S17501[151], 9}, {NULL}};
const Segment SGI309787[] = {{&S17502[30], 3}, {NULL}};
const Segment SGI309788[] = {{&S17502[196], 9}, {NULL}};
const Segment SGI309789[] = {{&S17502[8], 3}, {NULL}};
const Segment SGI309790[] = {{&S17502[13], 10}, {NULL}};
const Segment SGI309791[] = {{&S17502[42], 7}, {NULL}};
const Segment SGI309792[] = {{&S17502[35], 14}, {NULL}};
const Segment SGI309793[] = {{&S17502[168], 4}, {NULL}};
const Segment SGI309794[] = {{&S17502[51], 5}, {NULL}};
const Segment SGI309795[] = {{&S17503[30], 3}, {NULL}};
const Segment SGI309796[] = {{&S17503[186], 9}, {&S17503[186], 9}, {NULL}};
const Segment SGI309797[] = {{&S17503[8], 3}, {NULL}};
const Segment SGI309798[] = {{&S17503[13], 10}, {NULL}};
const Segment SGI309799[] = {{&S17503[42], 7}, {NULL}};
const Segment SGI309800[] = {{&S17503[35], 14}, {NULL}};
const Segment SGI309801[] = {{&S17503[51], 5}, {NULL}};
const Segment SGI309802[] = {{&S17504[30], 3}, {NULL}};
const Segment SGI309803[] = {{&S17504[206], 9}, {NULL}};
const Segment SGI309804[] = {{&S17504[8], 3}, {NULL}};
const Segment SGI309805[] = {{&S17504[13], 10}, {NULL}};
const Segment SGI309806[] = {{&S17504[42], 7}, {NULL}};
const Segment SGI309807[] = {{&S17504[35], 14}, {NULL}};
const Segment SGI309808[] = {{&S17504[166], 4}, {NULL}};
const Segment SGI309809[] = {{&S17504[51], 6}, {NULL}};
const Segment SGI309810[] = {{&S17504[151], 9}, {NULL}};
const Segment SGI309811[] = {{&S17505[30], 3}, {NULL}};
const Segment SGI309812[] = {{&S17505[181], 9}, {&S17505[181], 9}, {NULL}};
const Segment SGI309813[] = {{&S17505[8], 3}, {NULL}};
const Segment SGI309814[] = {{&S17505[13], 10}, {NULL}};
const Segment SGI309815[] = {{&S17505[42], 7}, {NULL}};
const Segment SGI309816[] = {{&S17505[35], 14}, {NULL}};
const Segment SGI309817[] = {{&S17505[51], 6}, {NULL}};
const Segment SGI309818[] = {{&S17506[30], 3}, {NULL}};
const Segment SGI309819[] = {{&S17506[186], 9}, {NULL}};
const Segment SGI309820[] = {{&S17506[8], 3}, {NULL}};
const Segment SGI309821[] = {{&S17506[13], 10}, {NULL}};
const Segment SGI309822[] = {{&S17506[42], 7}, {NULL}};
const Segment SGI309823[] = {{&S17506[35], 14}, {NULL}};
const Segment SGI309824[] = {{&S17506[51], 5}, {NULL}};
const Segment SGI309825[] = {{&S17507[30], 3}, {NULL}};
const Segment SGI309826[] = {{&S17507[181], 9}, {NULL}};
const Segment SGI309827[] = {{&S17507[8], 3}, {NULL}};
const Segment SGI309828[] = {{&S17507[13], 10}, {NULL}};
const Segment SGI309829[] = {{&S17507[42], 7}, {NULL}};
const Segment SGI309830[] = {{&S17507[35], 14}, {NULL}};
const Segment SGI309831[] = {{&S17507[51], 6}, {NULL}};
const Segment SGI309832[] = {{&S17508[30], 3}, {NULL}};
const Segment SGI309833[] = {{&S17508[196], 9}, {NULL}};
const Segment SGI309834[] = {{&S17508[8], 3}, {NULL}};
const Segment SGI309835[] = {{&S17508[13], 10}, {NULL}};
const Segment SGI309836[] = {{&S17508[42], 7}, {NULL}};
const Segment SGI309837[] = {{&S17508[35], 14}, {NULL}};
const Segment SGI309838[] = {{&S17508[51], 6}, {NULL}};
const Segment SGI309839[] = {{&S17508[151], 9}, {NULL}};
const Segment SGI309840[] = {{&S17509[30], 3}, {NULL}};
const Segment SGI309841[] = {{&S17509[179], 9}, {NULL}};
const Segment SGI309842[] = {{&S17509[8], 3}, {NULL}};
const Segment SGI309843[] = {{&S17509[13], 10}, {NULL}};
const Segment SGI309844[] = {{&S17509[42], 7}, {NULL}};
const Segment SGI309845[] = {{&S17509[35], 14}, {NULL}};
const Segment SGI309846[] = {{&S17509[151], 4}, {NULL}};
const Segment SGI309847[] = {{&S17509[51], 6}, {NULL}};
const Segment SGI309848[] = {{&S17510[30], 3}, {NULL}};
const Segment SGI309849[] = {{&S17510[186], 9}, {&S17510[186], 9}, {NULL}};
const Segment SGI309850[] = {{&S17510[8], 3}, {NULL}};
const Segment SGI309851[] = {{&S17510[13], 10}, {NULL}};
const Segment SGI309852[] = {{&S17510[42], 7}, {NULL}};
const Segment SGI309853[] = {{&S17510[35], 14}, {NULL}};
const Segment SGI309854[] = {{&S17510[51], 5}, {NULL}};
const Segment SGI309855[] = {{&S17511[30], 3}, {NULL}};
const Segment SGI309856[] = {{&S17511[191], 9}, {NULL}};
const Segment SGI309857[] = {{&S17511[8], 3}, {NULL}};
const Segment SGI309858[] = {{&S17511[13], 10}, {NULL}};
const Segment SGI309859[] = {{&S17511[42], 7}, {NULL}};
const Segment SGI309860[] = {{&S17511[35], 14}, {NULL}};
const Segment SGI309861[] = {{&S17511[151], 4}, {NULL}};
const Segment SGI309862[] = {{&S17511[51], 6}, {NULL}};
const Segment SGI309863[] = {{&S17512[30], 3}, {NULL}};
const Segment SGI309864[] = {{&S17512[191], 9}, {&S17512[191], 9}, {NULL}};
const Segment SGI309865[] = {{&S17512[8], 3}, {NULL}};
const Segment SGI309866[] = {{&S17512[13], 10}, {NULL}};
const Segment SGI309867[] = {{&S17512[42], 7}, {NULL}};
const Segment SGI309868[] = {{&S17512[35], 14}, {NULL}};
const Segment SGI309869[] = {{&S17512[151], 4}, {NULL}};
const Segment SGI309870[] = {{&S17512[51], 6}, {NULL}};
const Segment SGI309871[] = {{&S17514[30], 3}, {NULL}};
const Segment SGI309872[] = {{&S17514[196], 9}, {NULL}};
const Segment SGI309873[] = {{&S17514[8], 3}, {NULL}};
const Segment SGI309874[] = {{&S17514[13], 10}, {NULL}};
const Segment SGI309875[] = {{&S17514[42], 7}, {NULL}};
const Segment SGI309876[] = {{&S17514[35], 14}, {NULL}};
const Segment SGI309877[] = {{&S17514[156], 4}, {NULL}};
const Segment SGI309878[] = {{&S17514[51], 5}, {NULL}};
const Segment SGI309879[] = {{&S17515[30], 3}, {NULL}};
const Segment SGI309880[] = {{&S17515[190], 9}, {NULL}};
const Segment SGI309881[] = {{&S17515[8], 3}, {NULL}};
const Segment SGI309882[] = {{&S17515[13], 10}, {NULL}};
const Segment SGI309883[] = {{&S17515[42], 7}, {NULL}};
const Segment SGI309884[] = {{&S17515[35], 14}, {NULL}};
const Segment SGI309885[] = {{&S17515[151], 15}, {NULL}};
const Segment SGI309886[] = {{&S17515[51], 6}, {NULL}};
const Segment SGI309887[] = {{&S17516[30], 3}, {NULL}};
const Segment SGI309888[] = {{&S17516[214], 9}, {NULL}};
const Segment SGI309889[] = {{&S17516[8], 3}, {NULL}};
const Segment SGI309890[] = {{&S17516[13], 10}, {NULL}};
const Segment SGI309891[] = {{&S17516[42], 7}, {NULL}};
const Segment SGI309892[] = {{&S17516[35], 14}, {NULL}};
const Segment SGI309893[] = {{&S17516[156], 15}, {NULL}};
const Segment SGI309894[] = {{&S17516[51], 5}, {NULL}};
const Segment SGI309895[] = {{&S17517[30], 3}, {NULL}};
const Segment SGI309896[] = {{&S17517[229], 9}, {NULL}};
const Segment SGI309897[] = {{&S17517[8], 3}, {NULL}};
const Segment SGI309898[] = {{&S17517[13], 10}, {NULL}};
const Segment SGI309899[] = {{&S17517[42], 7}, {NULL}};
const Segment SGI309900[] = {{&S17517[35], 14}, {NULL}};
const Segment SGI309901[] = {{&S17517[156], 15}, {NULL}};
const Segment SGI309902[] = {{&S17517[51], 5}, {NULL}};
const Segment SGI309903[] = {{&S17517[184], 9}, {NULL}};
const Segment SGI309904[] = {{&S17518[30], 3}, {NULL}};
const Segment SGI309905[] = {{&S17518[162], 9}, {NULL}};
const Segment SGI309906[] = {{&S17518[8], 3}, {NULL}};
const Segment SGI309907[] = {{&S17518[13], 10}, {NULL}};
const Segment SGI309908[] = {{&S17518[42], 7}, {NULL}};
const Segment SGI309909[] = {{&S17518[35], 14}, {NULL}};
const Segment SGI309910[] = {{&S17519[30], 3}, {NULL}};
const Segment SGI309911[] = {{&S17519[196], 9}, {NULL}};
const Segment SGI309912[] = {{&S17519[8], 3}, {NULL}};
const Segment SGI309913[] = {{&S17519[13], 10}, {NULL}};
const Segment SGI309914[] = {{&S17519[42], 7}, {NULL}};
const Segment SGI309915[] = {{&S17519[35], 14}, {NULL}};
const Segment SGI309916[] = {{&S17519[51], 6}, {NULL}};
const Segment SGI309917[] = {{&S17519[151], 9}, {NULL}};
const Segment SGI309918[] = {{&S17520[30], 3}, {NULL}};
const Segment SGI309919[] = {{&S17520[196], 9}, {&S17520[196], 9}, {NULL}};
const Segment SGI309920[] = {{&S17520[8], 3}, {NULL}};
const Segment SGI309921[] = {{&S17520[13], 10}, {NULL}};
const Segment SGI309922[] = {{&S17520[42], 7}, {NULL}};
const Segment SGI309923[] = {{&S17520[35], 14}, {NULL}};
const Segment SGI309924[] = {{&S17520[51], 6}, {NULL}};
const Segment SGI309925[] = {{&S17520[151], 9}, {NULL}};
const Segment SGI309926[] = {{&S17521[30], 3}, {NULL}};
const Segment SGI309927[] = {{&S17521[174], 9}, {NULL}};
const Segment SGI309928[] = {{&S17521[8], 3}, {NULL}};
const Segment SGI309929[] = {{&S17521[13], 10}, {NULL}};
const Segment SGI309930[] = {{&S17521[42], 7}, {NULL}};
const Segment SGI309931[] = {{&S17521[35], 14}, {NULL}};
const Segment SGI309932[] = {{&S17522[30], 3}, {NULL}};
const Segment SGI309933[] = {{&S17522[196], 9}, {&S17522[196], 9}, {NULL}};
const Segment SGI309934[] = {{&S17522[8], 3}, {NULL}};
const Segment SGI309935[] = {{&S17522[13], 10}, {NULL}};
const Segment SGI309936[] = {{&S17522[42], 7}, {NULL}};
const Segment SGI309937[] = {{&S17522[35], 14}, {NULL}};
const Segment SGI309938[] = {{&S17522[156], 4}, {NULL}};
const Segment SGI309939[] = {{&S17522[51], 5}, {NULL}};
const Segment SGI309940[] = {{&S17523[30], 3}, {NULL}};
const Segment SGI309941[] = {{&S17523[206], 9}, {&S17523[206], 9}, {NULL}};
const Segment SGI309942[] = {{&S17523[8], 3}, {NULL}};
const Segment SGI309943[] = {{&S17523[13], 10}, {NULL}};
const Segment SGI309944[] = {{&S17523[42], 7}, {NULL}};
const Segment SGI309945[] = {{&S17523[35], 14}, {NULL}};
const Segment SGI309946[] = {{&S17523[166], 4}, {NULL}};
const Segment SGI309947[] = {{&S17523[51], 6}, {NULL}};
const Segment SGI309948[] = {{&S17523[151], 9}, {NULL}};
const Segment SGI309949[] = {{&S17524[30], 3}, {NULL}};
const Segment SGI309950[] = {{&S17524[214], 9}, {&S17524[214], 9}, {NULL}};
const Segment SGI309951[] = {{&S17524[8], 3}, {NULL}};
const Segment SGI309952[] = {{&S17524[13], 10}, {NULL}};
const Segment SGI309953[] = {{&S17524[42], 7}, {NULL}};
const Segment SGI309954[] = {{&S17524[35], 14}, {NULL}};
const Segment SGI309955[] = {{&S17524[156], 15}, {NULL}};
const Segment SGI309956[] = {{&S17524[51], 5}, {NULL}};
const Segment SGI309957[] = {{&S17525[30], 3}, {NULL}};
const Segment SGI309958[] = {{&S17525[181], 9}, {&S17525[181], 9}, {&S17525[181], 9}, {NULL}};
const Segment SGI309959[] = {{&S17525[8], 3}, {NULL}};
const Segment SGI309960[] = {{&S17525[13], 10}, {NULL}};
const Segment SGI309961[] = {{&S17525[42], 7}, {NULL}};
const Segment SGI309962[] = {{&S17525[35], 14}, {NULL}};
const Segment SGI309963[] = {{&S17525[51], 6}, {NULL}};
const Segment SGI309964[] = {{&S17527[30], 3}, {NULL}};
const Segment SGI309965[] = {{&S17527[202], 9}, {NULL}};
const Segment SGI309966[] = {{&S17527[8], 3}, {NULL}};
const Segment SGI309967[] = {{&S17527[13], 10}, {NULL}};
const Segment SGI309968[] = {{&S17527[42], 7}, {NULL}};
const Segment SGI309969[] = {{&S17527[35], 14}, {NULL}};
const Segment SGI309970[] = {{&S17527[163], 15}, {NULL}};
const Segment SGI309971[] = {{&S17527[51], 6}, {NULL}};
const Segment SGI309972[] = {{&S17528[30], 3}, {NULL}};
const Segment SGI309973[] = {{&S17528[181], 9}, {&S17528[181], 9}, {NULL}};
const Segment SGI309974[] = {{&S17528[8], 3}, {NULL}};
const Segment SGI309975[] = {{&S17528[13], 10}, {NULL}};
const Segment SGI309976[] = {{&S17528[42], 7}, {NULL}};
const Segment SGI309977[] = {{&S17528[35], 14}, {NULL}};
const Segment SGI309978[] = {{&S17528[51], 6}, {NULL}};
const Segment SGI309979[] = {{&S17529[30], 3}, {NULL}};
const Segment SGI309980[] = {{&S17529[141], 9}, {NULL}};
const Segment SGI309981[] = {{&S17529[8], 3}, {NULL}};
const Segment SGI309982[] = {{&S17529[13], 10}, {NULL}};
const Segment SGI309983[] = {{&S17529[42], 7}, {NULL}};
const Segment SGI309984[] = {{&S17529[35], 14}, {NULL}};
const Segment SGI309985[] = {{&S17529[51], 5}, {NULL}};
const Segment SGI309986[] = {{&S17529[96], 9}, {NULL}};
const Segment SGI309987[] = {{&S17530[30], 3}, {NULL}};
const Segment SGI309988[] = {{&S17530[229], 9}, {&S17530[229], 9}, {NULL}};
const Segment SGI309989[] = {{&S17530[8], 3}, {NULL}};
const Segment SGI309990[] = {{&S17530[13], 10}, {NULL}};
const Segment SGI309991[] = {{&S17530[42], 7}, {NULL}};
const Segment SGI309992[] = {{&S17530[35], 14}, {NULL}};
const Segment SGI309993[] = {{&S17530[156], 15}, {NULL}};
const Segment SGI309994[] = {{&S17530[51], 5}, {NULL}};
const Segment SGI309995[] = {{&S17530[184], 9}, {NULL}};
const Segment SGI309996[] = {{&S17531[30], 3}, {NULL}};
const Segment SGI309997[] = {{&S17531[181], 9}, {NULL}};
const Segment SGI309998[] = {{&S17531[8], 3}, {NULL}};
const Segment SGI309999[] = {{&S17531[13], 10}, {NULL}};
const Segment SGI310000[] = {{&S17531[42], 7}, {NULL}};
const Segment SGI310001[] = {{&S17531[35], 14}, {NULL}};
const Segment SGI310002[] = {{&S17531[51], 6}, {NULL}};
const Segment SGI310003[] = {{&S17533[30], 3}, {NULL}};
const Segment SGI310004[] = {{&S17533[181], 9}, {&S17533[181], 9}, {NULL}};
const Segment SGI310005[] = {{&S17533[8], 3}, {NULL}};
const Segment SGI310006[] = {{&S17533[13], 10}, {NULL}};
const Segment SGI310007[] = {{&S17533[42], 7}, {NULL}};
const Segment SGI310008[] = {{&S17533[35], 14}, {NULL}};
const Segment SGI310009[] = {{&S17533[51], 6}, {NULL}};
const Segment SGI310010[] = {{&S17534[30], 3}, {NULL}};
const Segment SGI310011[] = {{&S17534[191], 9}, {&S17534[191], 9}, {NULL}};
const Segment SGI310012[] = {{&S17534[8], 3}, {NULL}};
const Segment SGI310013[] = {{&S17534[13], 10}, {NULL}};
const Segment SGI310014[] = {{&S17534[42], 7}, {NULL}};
const Segment SGI310015[] = {{&S17534[35], 14}, {NULL}};
const Segment SGI310016[] = {{&S17534[151], 4}, {NULL}};
const Segment SGI310017[] = {{&S17534[51], 6}, {NULL}};
const Segment SGI310018[] = {{&S17535[30], 3}, {NULL}};
const Segment SGI310019[] = {{&S17535[202], 9}, {&S17535[202], 9}, {NULL}};
const Segment SGI310020[] = {{&S17535[8], 3}, {NULL}};
const Segment SGI310021[] = {{&S17535[13], 10}, {NULL}};
const Segment SGI310022[] = {{&S17535[42], 7}, {NULL}};
const Segment SGI310023[] = {{&S17535[35], 14}, {NULL}};
const Segment SGI310024[] = {{&S17535[163], 15}, {NULL}};
const Segment SGI310025[] = {{&S17535[51], 6}, {NULL}};
const Segment SGI310026[] = {{&S17536[30], 3}, {NULL}};
const Segment SGI310027[] = {{&S17536[202], 9}, {NULL}};
const Segment SGI310028[] = {{&S17536[8], 3}, {NULL}};
const Segment SGI310029[] = {{&S17536[13], 10}, {NULL}};
const Segment SGI310030[] = {{&S17536[42], 7}, {NULL}};
const Segment SGI310031[] = {{&S17536[35], 14}, {NULL}};
const Segment SGI310032[] = {{&S17536[151], 15}, {NULL}};
const Segment SGI310033[] = {{&S17536[51], 6}, {NULL}};
const Segment SGI310034[] = {{&S17537[30], 3}, {NULL}};
const Segment SGI310035[] = {{&S17537[186], 9}, {NULL}};
const Segment SGI310036[] = {{&S17537[8], 3}, {NULL}};
const Segment SGI310037[] = {{&S17537[13], 10}, {NULL}};
const Segment SGI310038[] = {{&S17537[42], 7}, {NULL}};
const Segment SGI310039[] = {{&S17537[35], 14}, {NULL}};
const Segment SGI310040[] = {{&S17537[51], 5}, {NULL}};
const Segment SGI310041[] = {{&S17538[30], 3}, {NULL}};
const Segment SGI310042[] = {{&S17538[191], 9}, {&S17538[191], 9}, {NULL}};
const Segment SGI310043[] = {{&S17538[8], 3}, {NULL}};
const Segment SGI310044[] = {{&S17538[13], 10}, {NULL}};
const Segment SGI310045[] = {{&S17538[42], 7}, {NULL}};
const Segment SGI310046[] = {{&S17538[35], 14}, {NULL}};
const Segment SGI310047[] = {{&S17538[151], 4}, {NULL}};
const Segment SGI310048[] = {{&S17538[51], 6}, {NULL}};
const Segment SGI310049[] = {{&S17539[30], 3}, {NULL}};
const Segment SGI310050[] = {{&S17539[196], 9}, {NULL}};
const Segment SGI310051[] = {{&S17539[8], 3}, {NULL}};
const Segment SGI310052[] = {{&S17539[13], 10}, {NULL}};
const Segment SGI310053[] = {{&S17539[42], 7}, {NULL}};
const Segment SGI310054[] = {{&S17539[35], 14}, {NULL}};
const Segment SGI310055[] = {{&S17539[51], 6}, {NULL}};
const Segment SGI310056[] = {{&S17539[151], 9}, {NULL}};
const Segment SGI310057[] = {{&S17540[30], 3}, {NULL}};
const Segment SGI310058[] = {{&S17540[181], 9}, {&S17540[181], 9}, {&S17540[181], 9}, {&S17540[181], 9}, {NULL}};
const Segment SGI310059[] = {{&S17540[8], 3}, {NULL}};
const Segment SGI310060[] = {{&S17540[13], 10}, {NULL}};
const Segment SGI310061[] = {{&S17540[42], 7}, {NULL}};
const Segment SGI310062[] = {{&S17540[35], 14}, {NULL}};
const Segment SGI310063[] = {{&S17540[51], 6}, {NULL}};
const Segment SGI310064[] = {{&S17541[30], 3}, {NULL}};
const Segment SGI310065[] = {{&S17541[141], 9}, {&S17541[141], 9}, {NULL}};
const Segment SGI310066[] = {{&S17541[8], 3}, {NULL}};
const Segment SGI310067[] = {{&S17541[13], 10}, {NULL}};
const Segment SGI310068[] = {{&S17541[42], 7}, {NULL}};
const Segment SGI310069[] = {{&S17541[35], 14}, {NULL}};
const Segment SGI310070[] = {{&S17541[51], 5}, {NULL}};
const Segment SGI310071[] = {{&S17541[96], 9}, {NULL}};
const Segment SGI310072[] = {{&S17542[30], 3}, {NULL}};
const Segment SGI310073[] = {{&S17542[190], 9}, {&S17542[190], 9}, {NULL}};
const Segment SGI310074[] = {{&S17542[8], 3}, {NULL}};
const Segment SGI310075[] = {{&S17542[13], 10}, {NULL}};
const Segment SGI310076[] = {{&S17542[42], 7}, {NULL}};
const Segment SGI310077[] = {{&S17542[35], 14}, {NULL}};
const Segment SGI310078[] = {{&S17542[151], 15}, {NULL}};
const Segment SGI310079[] = {{&S17542[51], 6}, {NULL}};
const Segment SGI310080[] = {{&S17543[30], 3}, {NULL}};
const Segment SGI310081[] = {{&S17543[174], 9}, {&S17543[174], 9}, {NULL}};
const Segment SGI310082[] = {{&S17543[8], 3}, {NULL}};
const Segment SGI310083[] = {{&S17543[13], 10}, {NULL}};
const Segment SGI310084[] = {{&S17543[42], 7}, {NULL}};
const Segment SGI310085[] = {{&S17543[35], 14}, {NULL}};
const Segment SGI310086[] = {{&S17544[30], 3}, {NULL}};
const Segment SGI310087[] = {{&S17544[191], 9}, {NULL}};
const Segment SGI310088[] = {{&S17544[8], 3}, {NULL}};
const Segment SGI310089[] = {{&S17544[13], 10}, {NULL}};
const Segment SGI310090[] = {{&S17544[42], 7}, {NULL}};
const Segment SGI310091[] = {{&S17544[35], 14}, {NULL}};
const Segment SGI310092[] = {{&S17544[151], 4}, {NULL}};
const Segment SGI310093[] = {{&S17544[51], 6}, {NULL}};
const Segment SGI310094[] = {{&S17545[30], 3}, {NULL}};
const Segment SGI310095[] = {{&S17545[196], 9}, {&S17545[196], 9}, {NULL}};
const Segment SGI310096[] = {{&S17545[8], 3}, {NULL}};
const Segment SGI310097[] = {{&S17545[13], 10}, {NULL}};
const Segment SGI310098[] = {{&S17545[42], 7}, {NULL}};
const Segment SGI310099[] = {{&S17545[35], 14}, {NULL}};
const Segment SGI310100[] = {{&S17545[51], 6}, {NULL}};
const Segment SGI310101[] = {{&S17545[151], 9}, {NULL}};
const Segment SGI310102[] = {{&S17546[30], 3}, {NULL}};
const Segment SGI310103[] = {{&S17546[162], 9}, {NULL}};
const Segment SGI310104[] = {{&S17546[8], 3}, {NULL}};
const Segment SGI310105[] = {{&S17546[13], 10}, {NULL}};
const Segment SGI310106[] = {{&S17546[42], 7}, {NULL}};
const Segment SGI310107[] = {{&S17546[35], 14}, {NULL}};
const Segment SGI310108[] = {{&S17547[30], 3}, {NULL}};
const Segment SGI310109[] = {{&S17547[207], 9}, {NULL}};
const Segment SGI310110[] = {{&S17547[8], 3}, {NULL}};
const Segment SGI310111[] = {{&S17547[13], 10}, {NULL}};
const Segment SGI310112[] = {{&S17547[42], 7}, {NULL}};
const Segment SGI310113[] = {{&S17547[35], 14}, {NULL}};
const Segment SGI310114[] = {{&S17547[156], 15}, {NULL}};
const Segment SGI310115[] = {{&S17547[51], 5}, {NULL}};
const Segment SGI310116[] = {{&S17548[30], 3}, {NULL}};
const Segment SGI310117[] = {{&S17548[186], 9}, {NULL}};
const Segment SGI310118[] = {{&S17548[8], 3}, {NULL}};
const Segment SGI310119[] = {{&S17548[13], 10}, {NULL}};
const Segment SGI310120[] = {{&S17548[42], 7}, {NULL}};
const Segment SGI310121[] = {{&S17548[35], 14}, {NULL}};
const Segment SGI310122[] = {{&S17548[51], 5}, {NULL}};
const Segment SGI310123[] = {{&S17549[30], 3}, {NULL}};
const Segment SGI310124[] = {{&S17549[181], 9}, {NULL}};
const Segment SGI310125[] = {{&S17549[8], 3}, {NULL}};
const Segment SGI310126[] = {{&S17549[13], 10}, {NULL}};
const Segment SGI310127[] = {{&S17549[42], 7}, {NULL}};
const Segment SGI310128[] = {{&S17549[35], 14}, {NULL}};
const Segment SGI310129[] = {{&S17549[51], 6}, {NULL}};
const Segment SGI310130[] = {{&S17550[30], 3}, {NULL}};
const Segment SGI310131[] = {{&S17550[181], 9}, {&S17550[181], 9}, {NULL}};
const Segment SGI310132[] = {{&S17550[8], 3}, {NULL}};
const Segment SGI310133[] = {{&S17550[13], 10}, {NULL}};
const Segment SGI310134[] = {{&S17550[42], 7}, {NULL}};
const Segment SGI310135[] = {{&S17550[35], 14}, {NULL}};
const Segment SGI310136[] = {{&S17550[51], 6}, {NULL}};
const Segment SGI310137[] = {{&S17551[30], 3}, {NULL}};
const Segment SGI310138[] = {{&S17551[186], 9}, {&S17551[186], 9}, {NULL}};
const Segment SGI310139[] = {{&S17551[8], 3}, {NULL}};
const Segment SGI310140[] = {{&S17551[13], 10}, {NULL}};
const Segment SGI310141[] = {{&S17551[42], 7}, {NULL}};
const Segment SGI310142[] = {{&S17551[35], 14}, {NULL}};
const Segment SGI310143[] = {{&S17551[51], 5}, {NULL}};
const Segment SGI310144[] = {{&S17552[30], 3}, {NULL}};
const Segment SGI310145[] = {{&S17552[191], 9}, {NULL}};
const Segment SGI310146[] = {{&S17552[8], 3}, {NULL}};
const Segment SGI310147[] = {{&S17552[13], 10}, {NULL}};
const Segment SGI310148[] = {{&S17552[42], 7}, {NULL}};
const Segment SGI310149[] = {{&S17552[35], 14}, {NULL}};
const Segment SGI310150[] = {{&S17552[151], 4}, {NULL}};
const Segment SGI310151[] = {{&S17552[51], 6}, {NULL}};
const Segment SGI310152[] = {{&S17554[30], 3}, {NULL}};
const Segment SGI310153[] = {{&S17554[186], 9}, {&S17554[186], 9}, {NULL}};
const Segment SGI310154[] = {{&S17554[8], 3}, {NULL}};
const Segment SGI310155[] = {{&S17554[13], 10}, {NULL}};
const Segment SGI310156[] = {{&S17554[42], 7}, {NULL}};
const Segment SGI310157[] = {{&S17554[35], 14}, {NULL}};
const Segment SGI310158[] = {{&S17554[51], 5}, {NULL}};
const Segment SGI310159[] = {{&S17555[30], 3}, {NULL}};
const Segment SGI310160[] = {{&S17555[190], 9}, {&S17555[190], 9}, {NULL}};
const Segment SGI310161[] = {{&S17555[8], 3}, {NULL}};
const Segment SGI310162[] = {{&S17555[13], 10}, {NULL}};
const Segment SGI310163[] = {{&S17555[42], 7}, {NULL}};
const Segment SGI310164[] = {{&S17555[35], 14}, {NULL}};
const Segment SGI310165[] = {{&S17555[151], 15}, {NULL}};
const Segment SGI310166[] = {{&S17555[51], 6}, {NULL}};
const Segment SGI310167[] = {{&S17556[30], 3}, {NULL}};
const Segment SGI310168[] = {{&S17556[229], 9}, {NULL}};
const Segment SGI310169[] = {{&S17556[8], 3}, {NULL}};
const Segment SGI310170[] = {{&S17556[13], 10}, {NULL}};
const Segment SGI310171[] = {{&S17556[42], 7}, {NULL}};
const Segment SGI310172[] = {{&S17556[35], 14}, {NULL}};
const Segment SGI310173[] = {{&S17556[156], 15}, {NULL}};
const Segment SGI310174[] = {{&S17556[51], 5}, {NULL}};
const Segment SGI310175[] = {{&S17556[184], 9}, {NULL}};
const Segment SGI310176[] = {{&S17557[30], 3}, {NULL}};
const Segment SGI310177[] = {{&S17557[214], 9}, {NULL}};
const Segment SGI310178[] = {{&S17557[8], 3}, {NULL}};
const Segment SGI310179[] = {{&S17557[13], 10}, {NULL}};
const Segment SGI310180[] = {{&S17557[42], 7}, {NULL}};
const Segment SGI310181[] = {{&S17557[35], 14}, {NULL}};
const Segment SGI310182[] = {{&S17557[156], 15}, {NULL}};
const Segment SGI310183[] = {{&S17557[51], 5}, {NULL}};
const Segment SGI310184[] = {{&S17558[30], 3}, {NULL}};
const Segment SGI310185[] = {{&S17558[190], 9}, {NULL}};
const Segment SGI310186[] = {{&S17558[8], 3}, {NULL}};
const Segment SGI310187[] = {{&S17558[13], 10}, {NULL}};
const Segment SGI310188[] = {{&S17558[42], 7}, {NULL}};
const Segment SGI310189[] = {{&S17558[35], 14}, {NULL}};
const Segment SGI310190[] = {{&S17558[151], 15}, {NULL}};
const Segment SGI310191[] = {{&S17558[51], 6}, {NULL}};
const Segment SGI310192[] = {{&S17559[30], 3}, {NULL}};
const Segment SGI310193[] = {{&S17559[196], 9}, {NULL}};
const Segment SGI310194[] = {{&S17559[8], 3}, {NULL}};
const Segment SGI310195[] = {{&S17559[13], 10}, {NULL}};
const Segment SGI310196[] = {{&S17559[42], 7}, {NULL}};
const Segment SGI310197[] = {{&S17559[35], 14}, {NULL}};
const Segment SGI310198[] = {{&S17559[51], 6}, {NULL}};
const Segment SGI310199[] = {{&S17559[151], 9}, {NULL}};
const Segment SGI310200[] = {{&S17560[30], 3}, {NULL}};
const Segment SGI310201[] = {{&S17560[206], 9}, {NULL}};
const Segment SGI310202[] = {{&S17560[8], 3}, {NULL}};
const Segment SGI310203[] = {{&S17560[13], 10}, {NULL}};
const Segment SGI310204[] = {{&S17560[42], 7}, {NULL}};
const Segment SGI310205[] = {{&S17560[35], 14}, {NULL}};
const Segment SGI310206[] = {{&S17560[166], 4}, {NULL}};
const Segment SGI310207[] = {{&S17560[51], 6}, {NULL}};
const Segment SGI310208[] = {{&S17560[151], 9}, {NULL}};
const Segment SGI310209[] = {{&S17561[30], 3}, {NULL}};
const Segment SGI310210[] = {{&S17561[196], 9}, {&S17561[196], 9}, {NULL}};
const Segment SGI310211[] = {{&S17561[8], 3}, {NULL}};
const Segment SGI310212[] = {{&S17561[13], 10}, {NULL}};
const Segment SGI310213[] = {{&S17561[42], 7}, {NULL}};
const Segment SGI310214[] = {{&S17561[35], 14}, {NULL}};
const Segment SGI310215[] = {{&S17561[51], 6}, {NULL}};
const Segment SGI310216[] = {{&S17561[151], 9}, {NULL}};
const Segment SGI310217[] = {{&S17562[30], 3}, {NULL}};
const Segment SGI310218[] = {{&S17562[217], 9}, {NULL}};
const Segment SGI310219[] = {{&S17562[8], 3}, {NULL}};
const Segment SGI310220[] = {{&S17562[13], 10}, {NULL}};
const Segment SGI310221[] = {{&S17562[42], 7}, {NULL}};
const Segment SGI310222[] = {{&S17562[35], 14}, {NULL}};
const Segment SGI310223[] = {{&S17562[144], 15}, {NULL}};
const Segment SGI310224[] = {{&S17562[172], 9}, {NULL}};
const Segment SGI310225[] = {{&S17563[30], 3}, {NULL}};
const Segment SGI310226[] = {{&S17563[191], 9}, {NULL}};
const Segment SGI310227[] = {{&S17563[8], 3}, {NULL}};
const Segment SGI310228[] = {{&S17563[13], 10}, {NULL}};
const Segment SGI310229[] = {{&S17563[42], 7}, {NULL}};
const Segment SGI310230[] = {{&S17563[35], 14}, {NULL}};
const Segment SGI310231[] = {{&S17563[151], 4}, {NULL}};
const Segment SGI310232[] = {{&S17563[51], 6}, {NULL}};
const Segment SGI310233[] = {{&S17564[30], 3}, {NULL}};
const Segment SGI310234[] = {{&S17564[196], 9}, {NULL}};
const Segment SGI310235[] = {{&S17564[8], 3}, {NULL}};
const Segment SGI310236[] = {{&S17564[13], 10}, {NULL}};
const Segment SGI310237[] = {{&S17564[42], 7}, {NULL}};
const Segment SGI310238[] = {{&S17564[35], 14}, {NULL}};
const Segment SGI310239[] = {{&S17564[168], 4}, {NULL}};
const Segment SGI310240[] = {{&S17564[51], 5}, {NULL}};
const Segment SGI310241[] = {{&S17565[30], 3}, {NULL}};
const Segment SGI310242[] = {{&S17565[162], 9}, {NULL}};
const Segment SGI310243[] = {{&S17565[8], 3}, {NULL}};
const Segment SGI310244[] = {{&S17565[13], 10}, {NULL}};
const Segment SGI310245[] = {{&S17565[42], 7}, {NULL}};
const Segment SGI310246[] = {{&S17565[35], 14}, {NULL}};
const Segment SGI310247[] = {{&S17566[30], 3}, {NULL}};
const Segment SGI310248[] = {{&S17566[201], 9}, {&S17566[201], 9}, {NULL}};
const Segment SGI310249[] = {{&S17566[8], 3}, {NULL}};
const Segment SGI310250[] = {{&S17566[13], 10}, {NULL}};
const Segment SGI310251[] = {{&S17566[42], 7}, {NULL}};
const Segment SGI310252[] = {{&S17566[35], 14}, {NULL}};
const Segment SGI310253[] = {{&S17566[51], 5}, {NULL}};
const Segment SGI310254[] = {{&S17566[96], 9}, {NULL}};
const Segment SGI310255[] = {{&S17567[30], 3}, {NULL}};
const Segment SGI310256[] = {{&S17567[202], 9}, {NULL}};
const Segment SGI310257[] = {{&S17567[8], 3}, {NULL}};
const Segment SGI310258[] = {{&S17567[13], 10}, {NULL}};
const Segment SGI310259[] = {{&S17567[42], 7}, {NULL}};
const Segment SGI310260[] = {{&S17567[35], 14}, {NULL}};
const Segment SGI310261[] = {{&S17567[144], 15}, {NULL}};
const Segment SGI310262[] = {{&S17568[30], 3}, {NULL}};
const Segment SGI310263[] = {{&S17568[206], 9}, {&S17568[206], 9}, {NULL}};
const Segment SGI310264[] = {{&S17568[8], 3}, {NULL}};
const Segment SGI310265[] = {{&S17568[13], 10}, {NULL}};
const Segment SGI310266[] = {{&S17568[42], 7}, {NULL}};
const Segment SGI310267[] = {{&S17568[35], 14}, {NULL}};
const Segment SGI310268[] = {{&S17568[166], 4}, {NULL}};
const Segment SGI310269[] = {{&S17568[51], 6}, {NULL}};
const Segment SGI310270[] = {{&S17568[151], 9}, {NULL}};
const Segment SGI310271[] = {{&S17569[30], 3}, {NULL}};
const Segment SGI310272[] = {{&S17569[190], 9}, {&S17569[190], 9}, {NULL}};
const Segment SGI310273[] = {{&S17569[8], 3}, {NULL}};
const Segment SGI310274[] = {{&S17569[13], 10}, {NULL}};
const Segment SGI310275[] = {{&S17569[42], 7}, {NULL}};
const Segment SGI310276[] = {{&S17569[35], 14}, {NULL}};
const Segment SGI310277[] = {{&S17569[151], 15}, {NULL}};
const Segment SGI310278[] = {{&S17569[51], 6}, {NULL}};
const Segment SGI310279[] = {{&S17570[30], 3}, {NULL}};
const Segment SGI310280[] = {{&S17570[229], 9}, {&S17570[229], 9}, {NULL}};
const Segment SGI310281[] = {{&S17570[8], 3}, {NULL}};
const Segment SGI310282[] = {{&S17570[13], 10}, {NULL}};
const Segment SGI310283[] = {{&S17570[42], 7}, {NULL}};
const Segment SGI310284[] = {{&S17570[35], 14}, {NULL}};
const Segment SGI310285[] = {{&S17570[156], 15}, {NULL}};
const Segment SGI310286[] = {{&S17570[51], 5}, {NULL}};
const Segment SGI310287[] = {{&S17570[184], 9}, {NULL}};
const Segment SGI310288[] = {{&S17571[30], 3}, {NULL}};
const Segment SGI310289[] = {{&S17571[201], 9}, {NULL}};
const Segment SGI310290[] = {{&S17571[8], 3}, {NULL}};
const Segment SGI310291[] = {{&S17571[13], 10}, {NULL}};
const Segment SGI310292[] = {{&S17571[42], 7}, {NULL}};
const Segment SGI310293[] = {{&S17571[35], 14}, {NULL}};
const Segment SGI310294[] = {{&S17571[51], 5}, {NULL}};
const Segment SGI310295[] = {{&S17571[96], 9}, {NULL}};
const Segment SGI310296[] = {{&S17572[30], 3}, {NULL}};
const Segment SGI310297[] = {{&S17572[214], 9}, {&S17572[214], 9}, {NULL}};
const Segment SGI310298[] = {{&S17572[8], 3}, {NULL}};
const Segment SGI310299[] = {{&S17572[13], 10}, {NULL}};
const Segment SGI310300[] = {{&S17572[42], 7}, {NULL}};
const Segment SGI310301[] = {{&S17572[35], 14}, {NULL}};
const Segment SGI310302[] = {{&S17572[156], 15}, {NULL}};
const Segment SGI310303[] = {{&S17572[51], 5}, {NULL}};
const Segment SGI310304[] = {{&S17573[30], 3}, {NULL}};
const Segment SGI310305[] = {{&S17573[174], 9}, {&S17573[174], 9}, {NULL}};
const Segment SGI310306[] = {{&S17573[8], 3}, {NULL}};
const Segment SGI310307[] = {{&S17573[13], 10}, {NULL}};
const Segment SGI310308[] = {{&S17573[42], 7}, {NULL}};
const Segment SGI310309[] = {{&S17573[35], 14}, {NULL}};
const Segment SGI310310[] = {{&S17574[30], 3}, {NULL}};
const Segment SGI310311[] = {{&S17574[239], 9}, {NULL}};
const Segment SGI310312[] = {{&S17574[8], 3}, {NULL}};
const Segment SGI310313[] = {{&S17574[13], 10}, {NULL}};
const Segment SGI310314[] = {{&S17574[42], 7}, {NULL}};
const Segment SGI310315[] = {{&S17574[35], 14}, {NULL}};
const Segment SGI310316[] = {{&S17574[156], 15}, {NULL}};
const Segment SGI310317[] = {{&S17574[199], 4}, {NULL}};
const Segment SGI310318[] = {{&S17574[51], 5}, {NULL}};
const Segment SGI310319[] = {{&S17574[184], 9}, {NULL}};
const Segment SGI310320[] = {{&S17575[30], 3}, {NULL}};
const Segment SGI310321[] = {{&S17575[191], 9}, {&S17575[191], 9}, {NULL}};
const Segment SGI310322[] = {{&S17575[8], 3}, {NULL}};
const Segment SGI310323[] = {{&S17575[13], 10}, {NULL}};
const Segment SGI310324[] = {{&S17575[42], 7}, {NULL}};
const Segment SGI310325[] = {{&S17575[35], 14}, {NULL}};
const Segment SGI310326[] = {{&S17575[151], 4}, {NULL}};
const Segment SGI310327[] = {{&S17575[51], 6}, {NULL}};
const Segment SGI310328[] = {{&S17576[30], 3}, {NULL}};
const Segment SGI310329[] = {{&S17576[196], 9}, {&S17576[196], 9}, {NULL}};
const Segment SGI310330[] = {{&S17576[8], 3}, {NULL}};
const Segment SGI310331[] = {{&S17576[13], 10}, {NULL}};
const Segment SGI310332[] = {{&S17576[42], 7}, {NULL}};
const Segment SGI310333[] = {{&S17576[35], 14}, {NULL}};
const Segment SGI310334[] = {{&S17576[51], 6}, {NULL}};
const Segment SGI310335[] = {{&S17576[151], 9}, {NULL}};
const Segment SGI310336[] = {{&S17577[30], 3}, {NULL}};
const Segment SGI310337[] = {{&S17577[196], 9}, {&S17577[196], 9}, {NULL}};
const Segment SGI310338[] = {{&S17577[8], 3}, {NULL}};
const Segment SGI310339[] = {{&S17577[13], 10}, {NULL}};
const Segment SGI310340[] = {{&S17577[42], 7}, {NULL}};
const Segment SGI310341[] = {{&S17577[35], 14}, {NULL}};
const Segment SGI310342[] = {{&S17577[168], 4}, {NULL}};
const Segment SGI310343[] = {{&S17577[51], 5}, {NULL}};
const Segment SGI310344[] = {{&S17578[30], 3}, {NULL}};
const Segment SGI310345[] = {{&S17578[187], 9}, {NULL}};
const Segment SGI310346[] = {{&S17578[8], 3}, {NULL}};
const Segment SGI310347[] = {{&S17578[13], 10}, {NULL}};
const Segment SGI310348[] = {{&S17578[42], 7}, {NULL}};
const Segment SGI310349[] = {{&S17578[35], 14}, {NULL}};
const Segment SGI310350[] = {{&S17578[154], 9}, {NULL}};
const Segment SGI310351[] = {{&S17579[30], 3}, {NULL}};
const Segment SGI310352[] = {{&S17579[205], 9}, {NULL}};
const Segment SGI310353[] = {{&S17579[8], 3}, {NULL}};
const Segment SGI310354[] = {{&S17579[13], 10}, {NULL}};
const Segment SGI310355[] = {{&S17579[42], 7}, {NULL}};
const Segment SGI310356[] = {{&S17579[35], 14}, {NULL}};
const Segment SGI310357[] = {{&S17579[166], 15}, {NULL}};
const Segment SGI310358[] = {{&S17579[51], 6}, {NULL}};
const Segment SGI310359[] = {{&S17579[151], 9}, {NULL}};
const Segment SGI310360[] = {{&S17580[30], 3}, {NULL}};
const Segment SGI310361[] = {{&S17580[181], 9}, {&S17580[181], 9}, {&S17580[181], 9}, {NULL}};
const Segment SGI310362[] = {{&S17580[8], 3}, {NULL}};
const Segment SGI310363[] = {{&S17580[13], 10}, {NULL}};
const Segment SGI310364[] = {{&S17580[42], 7}, {NULL}};
const Segment SGI310365[] = {{&S17580[35], 14}, {NULL}};
const Segment SGI310366[] = {{&S17580[51], 6}, {NULL}};
const Segment SGI310367[] = {{&S17581[30], 3}, {NULL}};
const Segment SGI310368[] = {{&S17581[186], 9}, {&S17581[186], 9}, {&S17581[186], 9}, {NULL}};
const Segment SGI310369[] = {{&S17581[8], 3}, {NULL}};
const Segment SGI310370[] = {{&S17581[13], 10}, {NULL}};
const Segment SGI310371[] = {{&S17581[42], 7}, {NULL}};
const Segment SGI310372[] = {{&S17581[35], 14}, {NULL}};
const Segment SGI310373[] = {{&S17581[51], 5}, {NULL}};
const Segment SGI310374[] = {{&S17582[30], 3}, {NULL}};
const Segment SGI310375[] = {{&S17582[196], 9}, {&S17582[196], 9}, {&S17582[196], 9}, {NULL}};
const Segment SGI310376[] = {{&S17582[8], 3}, {NULL}};
const Segment SGI310377[] = {{&S17582[13], 10}, {NULL}};
const Segment SGI310378[] = {{&S17582[42], 7}, {NULL}};
const Segment SGI310379[] = {{&S17582[35], 14}, {NULL}};
const Segment SGI310380[] = {{&S17582[51], 6}, {NULL}};
const Segment SGI310381[] = {{&S17582[151], 9}, {NULL}};
const Segment SGI310382[] = {{&S17583[30], 3}, {NULL}};
const Segment SGI310383[] = {{&S17583[186], 9}, {&S17583[186], 9}, {&S17583[186], 9}, {NULL}};
const Segment SGI310384[] = {{&S17583[8], 3}, {NULL}};
const Segment SGI310385[] = {{&S17583[13], 10}, {NULL}};
const Segment SGI310386[] = {{&S17583[42], 7}, {NULL}};
const Segment SGI310387[] = {{&S17583[35], 14}, {NULL}};
const Segment SGI310388[] = {{&S17583[51], 5}, {NULL}};
const Segment SGI310389[] = {{&S17584[30], 3}, {NULL}};
const Segment SGI310390[] = {{&S17584[239], 9}, {NULL}};
const Segment SGI310391[] = {{&S17584[8], 3}, {NULL}};
const Segment SGI310392[] = {{&S17584[13], 10}, {NULL}};
const Segment SGI310393[] = {{&S17584[42], 7}, {NULL}};
const Segment SGI310394[] = {{&S17584[35], 14}, {NULL}};
const Segment SGI310395[] = {{&S17584[156], 15}, {NULL}};
const Segment SGI310396[] = {{&S17584[199], 4}, {NULL}};
const Segment SGI310397[] = {{&S17584[51], 5}, {NULL}};
const Segment SGI310398[] = {{&S17584[184], 9}, {NULL}};
const Segment SGI310399[] = {{&S17585[30], 3}, {NULL}};
const Segment SGI310400[] = {{&S17585[169], 9}, {NULL}};
const Segment SGI310401[] = {{&S17585[8], 3}, {NULL}};
const Segment SGI310402[] = {{&S17585[13], 10}, {NULL}};
const Segment SGI310403[] = {{&S17585[42], 7}, {NULL}};
const Segment SGI310404[] = {{&S17585[35], 14}, {NULL}};
const Segment SGI310405[] = {{&S17585[51], 6}, {NULL}};
const Segment SGI310406[] = {{&S17586[30], 3}, {NULL}};
const Segment SGI310407[] = {{&S17586[181], 9}, {&S17586[181], 9}, {NULL}};
const Segment SGI310408[] = {{&S17586[8], 3}, {NULL}};
const Segment SGI310409[] = {{&S17586[13], 10}, {NULL}};
const Segment SGI310410[] = {{&S17586[42], 7}, {NULL}};
const Segment SGI310411[] = {{&S17586[35], 14}, {NULL}};
const Segment SGI310412[] = {{&S17586[51], 6}, {NULL}};
const Segment SGI310413[] = {{&S17587[30], 3}, {NULL}};
const Segment SGI310414[] = {{&S17587[217], 9}, {NULL}};
const Segment SGI310415[] = {{&S17587[8], 3}, {NULL}};
const Segment SGI310416[] = {{&S17587[13], 10}, {NULL}};
const Segment SGI310417[] = {{&S17587[42], 7}, {NULL}};
const Segment SGI310418[] = {{&S17587[35], 14}, {NULL}};
const Segment SGI310419[] = {{&S17587[144], 15}, {NULL}};
const Segment SGI310420[] = {{&S17587[172], 9}, {NULL}};
const Segment SGI310421[] = {{&S17588[30], 3}, {NULL}};
const Segment SGI310422[] = {{&S17588[217], 9}, {NULL}};
const Segment SGI310423[] = {{&S17588[8], 3}, {NULL}};
const Segment SGI310424[] = {{&S17588[13], 10}, {NULL}};
const Segment SGI310425[] = {{&S17588[42], 7}, {NULL}};
const Segment SGI310426[] = {{&S17588[35], 14}, {NULL}};
const Segment SGI310427[] = {{&S17588[166], 15}, {NULL}};
const Segment SGI310428[] = {{&S17588[156], 4}, {NULL}};
const Segment SGI310429[] = {{&S17588[51], 5}, {NULL}};
const Segment SGI310430[] = {{&S17589[30], 3}, {NULL}};
const Segment SGI310431[] = {{&S17589[186], 9}, {NULL}};
const Segment SGI310432[] = {{&S17589[8], 3}, {NULL}};
const Segment SGI310433[] = {{&S17589[13], 10}, {NULL}};
const Segment SGI310434[] = {{&S17589[42], 7}, {NULL}};
const Segment SGI310435[] = {{&S17589[35], 14}, {NULL}};
const Segment SGI310436[] = {{&S17589[51], 5}, {NULL}};
const Segment SGI310437[] = {{&S17590[30], 3}, {NULL}};
const Segment SGI310438[] = {{&S17590[202], 9}, {NULL}};
const Segment SGI310439[] = {{&S17590[8], 3}, {NULL}};
const Segment SGI310440[] = {{&S17590[13], 10}, {NULL}};
const Segment SGI310441[] = {{&S17590[42], 7}, {NULL}};
const Segment SGI310442[] = {{&S17590[35], 14}, {NULL}};
const Segment SGI310443[] = {{&S17590[144], 15}, {NULL}};
const Segment SGI310444[] = {{&S17591[30], 3}, {NULL}};
const Segment SGI310445[] = {{&S17591[206], 9}, {&S17591[206], 9}, {NULL}};
const Segment SGI310446[] = {{&S17591[8], 3}, {NULL}};
const Segment SGI310447[] = {{&S17591[13], 10}, {NULL}};
const Segment SGI310448[] = {{&S17591[42], 7}, {NULL}};
const Segment SGI310449[] = {{&S17591[35], 14}, {NULL}};
const Segment SGI310450[] = {{&S17591[166], 4}, {NULL}};
const Segment SGI310451[] = {{&S17591[51], 6}, {NULL}};
const Segment SGI310452[] = {{&S17591[151], 9}, {NULL}};
const Segment SGI310453[] = {{&S17592[30], 3}, {NULL}};
const Segment SGI310454[] = {{&S17592[181], 9}, {NULL}};
const Segment SGI310455[] = {{&S17592[8], 3}, {NULL}};
const Segment SGI310456[] = {{&S17592[13], 10}, {NULL}};
const Segment SGI310457[] = {{&S17592[42], 7}, {NULL}};
const Segment SGI310458[] = {{&S17592[35], 14}, {NULL}};
const Segment SGI310459[] = {{&S17592[51], 6}, {NULL}};
const Segment SGI310460[] = {{&S17593[30], 3}, {NULL}};
const Segment SGI310461[] = {{&S17593[186], 9}, {&S17593[186], 9}, {NULL}};
const Segment SGI310462[] = {{&S17593[8], 3}, {NULL}};
const Segment SGI310463[] = {{&S17593[13], 10}, {NULL}};
const Segment SGI310464[] = {{&S17593[42], 7}, {NULL}};
const Segment SGI310465[] = {{&S17593[35], 14}, {NULL}};
const Segment SGI310466[] = {{&S17593[51], 5}, {NULL}};
const Segment SGI310467[] = {{&S17594[30], 3}, {NULL}};
const Segment SGI310468[] = {{&S17594[196], 9}, {NULL}};
const Segment SGI310469[] = {{&S17594[8], 3}, {NULL}};
const Segment SGI310470[] = {{&S17594[13], 10}, {NULL}};
const Segment SGI310471[] = {{&S17594[42], 7}, {NULL}};
const Segment SGI310472[] = {{&S17594[35], 14}, {NULL}};
const Segment SGI310473[] = {{&S17594[51], 6}, {NULL}};
const Segment SGI310474[] = {{&S17594[151], 9}, {NULL}};
const Segment SGI310475[] = {{&S17595[30], 3}, {NULL}};
const Segment SGI310476[] = {{&S17595[190], 9}, {NULL}};
const Segment SGI310477[] = {{&S17595[8], 3}, {NULL}};
const Segment SGI310478[] = {{&S17595[13], 10}, {NULL}};
const Segment SGI310479[] = {{&S17595[42], 7}, {NULL}};
const Segment SGI310480[] = {{&S17595[35], 14}, {NULL}};
const Segment SGI310481[] = {{&S17595[151], 15}, {NULL}};
const Segment SGI310482[] = {{&S17595[51], 6}, {NULL}};
const Segment SGI310483[] = {{&S17596[30], 3}, {NULL}};
const Segment SGI310484[] = {{&S17596[174], 9}, {NULL}};
const Segment SGI310485[] = {{&S17596[8], 3}, {NULL}};
const Segment SGI310486[] = {{&S17596[13], 10}, {NULL}};
const Segment SGI310487[] = {{&S17596[42], 7}, {NULL}};
const Segment SGI310488[] = {{&S17596[35], 14}, {NULL}};
const Segment SGI310489[] = {{&S17597[30], 3}, {NULL}};
const Segment SGI310490[] = {{&S17597[229], 9}, {NULL}};
const Segment SGI310491[] = {{&S17597[8], 3}, {NULL}};
const Segment SGI310492[] = {{&S17597[13], 10}, {NULL}};
const Segment SGI310493[] = {{&S17597[42], 7}, {NULL}};
const Segment SGI310494[] = {{&S17597[35], 14}, {NULL}};
const Segment SGI310495[] = {{&S17597[156], 15}, {NULL}};
const Segment SGI310496[] = {{&S17597[51], 5}, {NULL}};
const Segment SGI310497[] = {{&S17597[184], 9}, {NULL}};
const Segment SGI310498[] = {{&S17599[30], 3}, {NULL}};
const Segment SGI310499[] = {{&S17599[191], 9}, {NULL}};
const Segment SGI310500[] = {{&S17599[8], 3}, {NULL}};
const Segment SGI310501[] = {{&S17599[13], 10}, {NULL}};
const Segment SGI310502[] = {{&S17599[42], 7}, {NULL}};
const Segment SGI310503[] = {{&S17599[35], 14}, {NULL}};
const Segment SGI310504[] = {{&S17599[151], 4}, {NULL}};
const Segment SGI310505[] = {{&S17599[51], 6}, {NULL}};
const Segment SGI310506[] = {{&S17600[30], 3}, {NULL}};
const Segment SGI310507[] = {{&S17600[207], 9}, {NULL}};
const Segment SGI310508[] = {{&S17600[8], 3}, {NULL}};
const Segment SGI310509[] = {{&S17600[13], 10}, {NULL}};
const Segment SGI310510[] = {{&S17600[42], 7}, {NULL}};
const Segment SGI310511[] = {{&S17600[35], 14}, {NULL}};
const Segment SGI310512[] = {{&S17600[96], 15}, {NULL}};
const Segment SGI310513[] = {{&S17600[51], 5}, {NULL}};
const Segment SGI310514[] = {{&S17601[30], 3}, {NULL}};
const Segment SGI310515[] = {{&S17601[201], 9}, {NULL}};
const Segment SGI310516[] = {{&S17601[8], 3}, {NULL}};
const Segment SGI310517[] = {{&S17601[13], 10}, {NULL}};
const Segment SGI310518[] = {{&S17601[42], 7}, {NULL}};
const Segment SGI310519[] = {{&S17601[35], 14}, {NULL}};
const Segment SGI310520[] = {{&S17601[51], 5}, {NULL}};
const Segment SGI310521[] = {{&S17601[96], 9}, {NULL}};
const Segment SGI310522[] = {{&S17602[30], 3}, {NULL}};
const Segment SGI310523[] = {{&S17602[196], 9}, {NULL}};
const Segment SGI310524[] = {{&S17602[8], 3}, {NULL}};
const Segment SGI310525[] = {{&S17602[13], 10}, {NULL}};
const Segment SGI310526[] = {{&S17602[42], 7}, {NULL}};
const Segment SGI310527[] = {{&S17602[35], 14}, {NULL}};
const Segment SGI310528[] = {{&S17602[168], 4}, {NULL}};
const Segment SGI310529[] = {{&S17602[51], 5}, {NULL}};
const Segment SGI310530[] = {{&S17603[30], 3}, {NULL}};
const Segment SGI310531[] = {{&S17603[196], 9}, {&S17603[196], 9}, {NULL}};
const Segment SGI310532[] = {{&S17603[8], 3}, {NULL}};
const Segment SGI310533[] = {{&S17603[13], 10}, {NULL}};
const Segment SGI310534[] = {{&S17603[42], 7}, {NULL}};
const Segment SGI310535[] = {{&S17603[35], 14}, {NULL}};
const Segment SGI310536[] = {{&S17603[51], 6}, {NULL}};
const Segment SGI310537[] = {{&S17603[151], 9}, {NULL}};
const Segment SGI310538[] = {{&S17604[30], 3}, {NULL}};
const Segment SGI310539[] = {{&S17604[214], 9}, {&S17604[214], 9}, {NULL}};
const Segment SGI310540[] = {{&S17604[8], 3}, {NULL}};
const Segment SGI310541[] = {{&S17604[13], 10}, {NULL}};
const Segment SGI310542[] = {{&S17604[42], 7}, {NULL}};
const Segment SGI310543[] = {{&S17604[35], 14}, {NULL}};
const Segment SGI310544[] = {{&S17604[156], 15}, {NULL}};
const Segment SGI310545[] = {{&S17604[51], 5}, {NULL}};
const Segment SGI310546[] = {{&S17605[30], 3}, {NULL}};
const Segment SGI310547[] = {{&S17605[229], 9}, {&S17605[229], 9}, {NULL}};
const Segment SGI310548[] = {{&S17605[8], 3}, {NULL}};
const Segment SGI310549[] = {{&S17605[13], 10}, {NULL}};
const Segment SGI310550[] = {{&S17605[42], 7}, {NULL}};
const Segment SGI310551[] = {{&S17605[35], 14}, {NULL}};
const Segment SGI310552[] = {{&S17605[156], 15}, {NULL}};
const Segment SGI310553[] = {{&S17605[51], 5}, {NULL}};
const Segment SGI310554[] = {{&S17605[184], 9}, {NULL}};
const Segment SGI310555[] = {{&S17606[30], 3}, {NULL}};
const Segment SGI310556[] = {{&S17606[209], 9}, {&S17606[209], 9}, {NULL}};
const Segment SGI310557[] = {{&S17606[8], 3}, {NULL}};
const Segment SGI310558[] = {{&S17606[13], 10}, {NULL}};
const Segment SGI310559[] = {{&S17606[42], 7}, {NULL}};
const Segment SGI310560[] = {{&S17606[35], 14}, {NULL}};
const Segment SGI310561[] = {{&S17606[51], 5}, {NULL}};
const Segment SGI310562[] = {{&S17606[156], 9}, {NULL}};
const Segment SGI310563[] = {{&S17607[30], 3}, {NULL}};
const Segment SGI310564[] = {{&S17607[174], 9}, {&S17607[174], 9}, {NULL}};
const Segment SGI310565[] = {{&S17607[8], 3}, {NULL}};
const Segment SGI310566[] = {{&S17607[13], 10}, {NULL}};
const Segment SGI310567[] = {{&S17607[42], 7}, {NULL}};
const Segment SGI310568[] = {{&S17607[35], 14}, {NULL}};
const Segment SGI310569[] = {{&S17608[30], 3}, {NULL}};
const Segment SGI310570[] = {{&S17608[196], 9}, {&S17608[196], 9}, {NULL}};
const Segment SGI310571[] = {{&S17608[8], 3}, {NULL}};
const Segment SGI310572[] = {{&S17608[13], 10}, {NULL}};
const Segment SGI310573[] = {{&S17608[42], 7}, {NULL}};
const Segment SGI310574[] = {{&S17608[35], 14}, {NULL}};
const Segment SGI310575[] = {{&S17608[168], 4}, {NULL}};
const Segment SGI310576[] = {{&S17608[51], 5}, {NULL}};
const Segment SGI310577[] = {{&S17609[30], 3}, {NULL}};
const Segment SGI310578[] = {{&S17609[190], 9}, {&S17609[190], 9}, {NULL}};
const Segment SGI310579[] = {{&S17609[8], 3}, {NULL}};
const Segment SGI310580[] = {{&S17609[13], 10}, {NULL}};
const Segment SGI310581[] = {{&S17609[42], 7}, {NULL}};
const Segment SGI310582[] = {{&S17609[35], 14}, {NULL}};
const Segment SGI310583[] = {{&S17609[151], 15}, {NULL}};
const Segment SGI310584[] = {{&S17609[51], 6}, {NULL}};
const Segment SGI310585[] = {{&S17610[30], 3}, {NULL}};
const Segment SGI310586[] = {{&S17610[196], 9}, {&S17610[196], 9}, {NULL}};
const Segment SGI310587[] = {{&S17610[8], 3}, {NULL}};
const Segment SGI310588[] = {{&S17610[13], 10}, {NULL}};
const Segment SGI310589[] = {{&S17610[42], 7}, {NULL}};
const Segment SGI310590[] = {{&S17610[35], 14}, {NULL}};
const Segment SGI310591[] = {{&S17610[156], 4}, {NULL}};
const Segment SGI310592[] = {{&S17610[51], 5}, {NULL}};
const Segment SGI310593[] = {{&S17611[30], 3}, {NULL}};
const Segment SGI310594[] = {{&S17611[191], 9}, {&S17611[191], 9}, {NULL}};
const Segment SGI310595[] = {{&S17611[8], 3}, {NULL}};
const Segment SGI310596[] = {{&S17611[13], 10}, {NULL}};
const Segment SGI310597[] = {{&S17611[42], 7}, {NULL}};
const Segment SGI310598[] = {{&S17611[35], 14}, {NULL}};
const Segment SGI310599[] = {{&S17611[151], 4}, {NULL}};
const Segment SGI310600[] = {{&S17611[51], 6}, {NULL}};
const Segment SGI310601[] = {{&S17612[30], 3}, {NULL}};
const Segment SGI310602[] = {{&S17612[206], 9}, {&S17612[206], 9}, {NULL}};
const Segment SGI310603[] = {{&S17612[8], 3}, {NULL}};
const Segment SGI310604[] = {{&S17612[13], 10}, {NULL}};
const Segment SGI310605[] = {{&S17612[42], 7}, {NULL}};
const Segment SGI310606[] = {{&S17612[35], 14}, {NULL}};
const Segment SGI310607[] = {{&S17612[166], 4}, {NULL}};
const Segment SGI310608[] = {{&S17612[51], 6}, {NULL}};
const Segment SGI310609[] = {{&S17612[151], 9}, {NULL}};
const Segment SGI310610[] = {{&S17613[30], 3}, {NULL}};
const Segment SGI310611[] = {{&S17613[196], 9}, {&S17613[196], 9}, {&S17613[196], 9}, {NULL}};
const Segment SGI310612[] = {{&S17613[8], 3}, {NULL}};
const Segment SGI310613[] = {{&S17613[13], 10}, {NULL}};
const Segment SGI310614[] = {{&S17613[42], 7}, {NULL}};
const Segment SGI310615[] = {{&S17613[35], 14}, {NULL}};
const Segment SGI310616[] = {{&S17613[51], 6}, {NULL}};
const Segment SGI310617[] = {{&S17613[151], 9}, {NULL}};
const Segment SGI310618[] = {{&S17614[30], 3}, {NULL}};
const Segment SGI310619[] = {{&S17614[217], 9}, {NULL}};
const Segment SGI310620[] = {{&S17614[8], 3}, {NULL}};
const Segment SGI310621[] = {{&S17614[13], 10}, {NULL}};
const Segment SGI310622[] = {{&S17614[42], 7}, {NULL}};
const Segment SGI310623[] = {{&S17614[35], 14}, {NULL}};
const Segment SGI310624[] = {{&S17614[144], 15}, {NULL}};
const Segment SGI310625[] = {{&S17614[172], 9}, {NULL}};
const Segment SGI310626[] = {{&S17615[30], 3}, {NULL}};
const Segment SGI310627[] = {{&S17615[206], 9}, {NULL}};
const Segment SGI310628[] = {{&S17615[8], 3}, {NULL}};
const Segment SGI310629[] = {{&S17615[13], 10}, {NULL}};
const Segment SGI310630[] = {{&S17615[42], 7}, {NULL}};
const Segment SGI310631[] = {{&S17615[35], 14}, {NULL}};
const Segment SGI310632[] = {{&S17615[166], 4}, {NULL}};
const Segment SGI310633[] = {{&S17615[51], 6}, {NULL}};
const Segment SGI310634[] = {{&S17615[151], 9}, {NULL}};
const Segment SGI310635[] = {{&S17616[30], 3}, {NULL}};
const Segment SGI310636[] = {{&S17616[207], 9}, {NULL}};
const Segment SGI310637[] = {{&S17616[8], 3}, {NULL}};
const Segment SGI310638[] = {{&S17616[13], 10}, {NULL}};
const Segment SGI310639[] = {{&S17616[42], 7}, {NULL}};
const Segment SGI310640[] = {{&S17616[35], 14}, {NULL}};
const Segment SGI310641[] = {{&S17616[168], 15}, {NULL}};
const Segment SGI310642[] = {{&S17616[51], 5}, {NULL}};
const Segment SGI310643[] = {{&S17617[30], 3}, {NULL}};
const Segment SGI310644[] = {{&S17617[201], 9}, {NULL}};
const Segment SGI310645[] = {{&S17617[8], 3}, {NULL}};
const Segment SGI310646[] = {{&S17617[13], 10}, {NULL}};
const Segment SGI310647[] = {{&S17617[42], 7}, {NULL}};
const Segment SGI310648[] = {{&S17617[35], 14}, {NULL}};
const Segment SGI310649[] = {{&S17617[51], 5}, {NULL}};
const Segment SGI310650[] = {{&S17617[96], 9}, {NULL}};
const Segment SGI310651[] = {{&S17618[30], 3}, {NULL}};
const Segment SGI310652[] = {{&S17618[224], 9}, {NULL}};
const Segment SGI310653[] = {{&S17618[8], 3}, {NULL}};
const Segment SGI310654[] = {{&S17618[13], 10}, {NULL}};
const Segment SGI310655[] = {{&S17618[42], 7}, {NULL}};
const Segment SGI310656[] = {{&S17618[35], 14}, {NULL}};
const Segment SGI310657[] = {{&S17618[156], 15}, {NULL}};
const Segment SGI310658[] = {{&S17618[184], 4}, {NULL}};
const Segment SGI310659[] = {{&S17618[51], 5}, {NULL}};
const Segment SGI310660[] = {{&S17619[30], 3}, {NULL}};
const Segment SGI310661[] = {{&S17619[189], 9}, {NULL}};
const Segment SGI310662[] = {{&S17619[8], 3}, {NULL}};
const Segment SGI310663[] = {{&S17619[13], 10}, {NULL}};
const Segment SGI310664[] = {{&S17619[42], 7}, {NULL}};
const Segment SGI310665[] = {{&S17619[35], 14}, {NULL}};
const Segment SGI310666[] = {{&S17619[84], 9}, {NULL}};
const Segment SGI310667[] = {{&S17620[30], 3}, {NULL}};
const Segment SGI310668[] = {{&S17620[206], 9}, {NULL}};
const Segment SGI310669[] = {{&S17620[8], 3}, {NULL}};
const Segment SGI310670[] = {{&S17620[13], 10}, {NULL}};
const Segment SGI310671[] = {{&S17620[42], 7}, {NULL}};
const Segment SGI310672[] = {{&S17620[35], 14}, {NULL}};
const Segment SGI310673[] = {{&S17620[166], 4}, {NULL}};
const Segment SGI310674[] = {{&S17620[51], 6}, {NULL}};
const Segment SGI310675[] = {{&S17620[151], 9}, {NULL}};
const Segment SGI310676[] = {{&S17621[30], 3}, {NULL}};
const Segment SGI310677[] = {{&S17621[196], 9}, {NULL}};
const Segment SGI310678[] = {{&S17621[8], 3}, {NULL}};
const Segment SGI310679[] = {{&S17621[13], 10}, {NULL}};
const Segment SGI310680[] = {{&S17621[42], 7}, {NULL}};
const Segment SGI310681[] = {{&S17621[35], 14}, {NULL}};
const Segment SGI310682[] = {{&S17621[156], 4}, {NULL}};
const Segment SGI310683[] = {{&S17621[51], 5}, {NULL}};
const Segment SGI310684[] = {{&S17622[30], 3}, {NULL}};
const Segment SGI310685[] = {{&S17622[229], 9}, {NULL}};
const Segment SGI310686[] = {{&S17622[8], 3}, {NULL}};
const Segment SGI310687[] = {{&S17622[13], 10}, {NULL}};
const Segment SGI310688[] = {{&S17622[42], 7}, {NULL}};
const Segment SGI310689[] = {{&S17622[35], 14}, {NULL}};
const Segment SGI310690[] = {{&S17622[156], 15}, {NULL}};
const Segment SGI310691[] = {{&S17622[51], 5}, {NULL}};
const Segment SGI310692[] = {{&S17622[184], 9}, {NULL}};
const Segment SGI310693[] = {{&S17623[30], 3}, {NULL}};
const Segment SGI310694[] = {{&S17623[181], 9}, {&S17623[181], 9}, {&S17623[181], 9}, {NULL}};
const Segment SGI310695[] = {{&S17623[8], 3}, {NULL}};
const Segment SGI310696[] = {{&S17623[13], 10}, {NULL}};
const Segment SGI310697[] = {{&S17623[42], 7}, {NULL}};
const Segment SGI310698[] = {{&S17623[35], 14}, {NULL}};
const Segment SGI310699[] = {{&S17623[51], 6}, {NULL}};
const Segment SGI310700[] = {{&S17624[30], 3}, {NULL}};
const Segment SGI310701[] = {{&S17624[205], 9}, {&S17624[205], 9}, {NULL}};
const Segment SGI310702[] = {{&S17624[8], 3}, {NULL}};
const Segment SGI310703[] = {{&S17624[13], 10}, {NULL}};
const Segment SGI310704[] = {{&S17624[42], 7}, {NULL}};
const Segment SGI310705[] = {{&S17624[35], 14}, {NULL}};
const Segment SGI310706[] = {{&S17624[166], 15}, {NULL}};
const Segment SGI310707[] = {{&S17624[51], 6}, {NULL}};
const Segment SGI310708[] = {{&S17624[151], 9}, {NULL}};
const Segment SGI310709[] = {{&S17625[30], 3}, {NULL}};
const Segment SGI310710[] = {{&S17625[202], 9}, {NULL}};
const Segment SGI310711[] = {{&S17625[8], 3}, {NULL}};
const Segment SGI310712[] = {{&S17625[13], 10}, {NULL}};
const Segment SGI310713[] = {{&S17625[42], 7}, {NULL}};
const Segment SGI310714[] = {{&S17625[35], 14}, {NULL}};
const Segment SGI310715[] = {{&S17625[144], 15}, {NULL}};
const Segment SGI310716[] = {{&S17626[30], 3}, {NULL}};
const Segment SGI310717[] = {{&S17626[239], 9}, {NULL}};
const Segment SGI310718[] = {{&S17626[8], 3}, {NULL}};
const Segment SGI310719[] = {{&S17626[13], 10}, {NULL}};
const Segment SGI310720[] = {{&S17626[42], 7}, {NULL}};
const Segment SGI310721[] = {{&S17626[35], 14}, {NULL}};
const Segment SGI310722[] = {{&S17626[156], 15}, {NULL}};
const Segment SGI310723[] = {{&S17626[199], 4}, {NULL}};
const Segment SGI310724[] = {{&S17626[51], 5}, {NULL}};
const Segment SGI310725[] = {{&S17626[184], 9}, {NULL}};
const Segment SGI310726[] = {{&S17627[30], 3}, {NULL}};
const Segment SGI310727[] = {{&S17627[186], 9}, {&S17627[186], 9}, {&S17627[186], 9}, {NULL}};
const Segment SGI310728[] = {{&S17627[8], 3}, {NULL}};
const Segment SGI310729[] = {{&S17627[13], 10}, {NULL}};
const Segment SGI310730[] = {{&S17627[42], 7}, {NULL}};
const Segment SGI310731[] = {{&S17627[35], 14}, {NULL}};
const Segment SGI310732[] = {{&S17627[51], 5}, {NULL}};
const Segment SGI310733[] = {{&S17628[30], 3}, {NULL}};
const Segment SGI310734[] = {{&S17628[181], 9}, {&S17628[181], 9}, {&S17628[181], 9}, {NULL}};
const Segment SGI310735[] = {{&S17628[8], 3}, {NULL}};
const Segment SGI310736[] = {{&S17628[13], 10}, {NULL}};
const Segment SGI310737[] = {{&S17628[42], 7}, {NULL}};
const Segment SGI310738[] = {{&S17628[35], 14}, {NULL}};
const Segment SGI310739[] = {{&S17628[51], 6}, {NULL}};
const Segment SGI310740[] = {{&S17629[30], 3}, {NULL}};
const Segment SGI310741[] = {{&S17629[224], 9}, {NULL}};
const Segment SGI310742[] = {{&S17629[8], 3}, {NULL}};
const Segment SGI310743[] = {{&S17629[13], 10}, {NULL}};
const Segment SGI310744[] = {{&S17629[42], 7}, {NULL}};
const Segment SGI310745[] = {{&S17629[35], 14}, {NULL}};
const Segment SGI310746[] = {{&S17629[156], 15}, {NULL}};
const Segment SGI310747[] = {{&S17629[184], 4}, {NULL}};
const Segment SGI310748[] = {{&S17629[51], 5}, {NULL}};
const Segment SGI310749[] = {{&S17630[30], 3}, {NULL}};
const Segment SGI310750[] = {{&S17630[185], 9}, {NULL}};
const Segment SGI310751[] = {{&S17630[8], 3}, {NULL}};
const Segment SGI310752[] = {{&S17630[13], 10}, {NULL}};
const Segment SGI310753[] = {{&S17630[42], 7}, {NULL}};
const Segment SGI310754[] = {{&S17630[35], 14}, {NULL}};
const Segment SGI310755[] = {{&S17630[144], 9}, {NULL}};
const Segment SGI310756[] = {{&S17631[30], 3}, {NULL}};
const Segment SGI310757[] = {{&S17631[205], 9}, {NULL}};
const Segment SGI310758[] = {{&S17631[8], 3}, {NULL}};
const Segment SGI310759[] = {{&S17631[13], 10}, {NULL}};
const Segment SGI310760[] = {{&S17631[42], 7}, {NULL}};
const Segment SGI310761[] = {{&S17631[35], 14}, {NULL}};
const Segment SGI310762[] = {{&S17631[151], 15}, {NULL}};
const Segment SGI310763[] = {{&S17631[51], 6}, {NULL}};
const Segment SGI310764[] = {{&S17631[172], 9}, {NULL}};
const Segment SGI310765[] = {{&S17632[30], 4}, {NULL}};
const Segment SGI310766[] = {{&S17632[90], 9}, {NULL}};
const Segment SGI310767[] = {{&S17632[8], 3}, {NULL}};
const Segment SGI310768[] = {{&S17632[13], 10}, {NULL}};
const Segment SGI310769[] = {{&S17632[43], 7}, {NULL}};
const Segment SGI310770[] = {{&S17632[36], 14}, {NULL}};
const Segment SGI310771[] = {{&S17632[52], 6}, {NULL}};
const Segment SGI310772[] = {{&S17633[30], 4}, {NULL}};
const Segment SGI310773[] = {{&S17633[90], 9}, {&S17633[90], 9}, {NULL}};
const Segment SGI310774[] = {{&S17633[8], 3}, {NULL}};
const Segment SGI310775[] = {{&S17633[13], 10}, {NULL}};
const Segment SGI310776[] = {{&S17633[43], 7}, {NULL}};
const Segment SGI310777[] = {{&S17633[36], 14}, {NULL}};
const Segment SGI310778[] = {{&S17633[52], 6}, {NULL}};
const Segment SGI310779[] = {{&S17634[30], 3}, {NULL}};
const Segment SGI310780[] = {{&S17634[174], 9}, {NULL}};
const Segment SGI310781[] = {{&S17634[8], 3}, {NULL}};
const Segment SGI310782[] = {{&S17634[13], 10}, {NULL}};
const Segment SGI310783[] = {{&S17634[42], 7}, {NULL}};
const Segment SGI310784[] = {{&S17634[35], 14}, {NULL}};
const Segment SGI310785[] = {{&S17635[30], 3}, {NULL}};
const Segment SGI310786[] = {{&S17635[181], 9}, {&S17635[181], 9}, {NULL}};
const Segment SGI310787[] = {{&S17635[8], 3}, {NULL}};
const Segment SGI310788[] = {{&S17635[13], 10}, {NULL}};
const Segment SGI310789[] = {{&S17635[42], 7}, {NULL}};
const Segment SGI310790[] = {{&S17635[35], 14}, {NULL}};
const Segment SGI310791[] = {{&S17635[51], 6}, {NULL}};
const Segment SGI310792[] = {{&S17636[30], 3}, {NULL}};
const Segment SGI310793[] = {{&S17636[186], 9}, {NULL}};
const Segment SGI310794[] = {{&S17636[8], 3}, {NULL}};
const Segment SGI310795[] = {{&S17636[13], 10}, {NULL}};
const Segment SGI310796[] = {{&S17636[42], 7}, {NULL}};
const Segment SGI310797[] = {{&S17636[35], 14}, {NULL}};
const Segment SGI310798[] = {{&S17636[51], 5}, {NULL}};
const Segment SGI310799[] = {{&S17638[30], 3}, {NULL}};
const Segment SGI310800[] = {{&S17638[214], 9}, {NULL}};
const Segment SGI310801[] = {{&S17638[8], 3}, {NULL}};
const Segment SGI310802[] = {{&S17638[13], 10}, {NULL}};
const Segment SGI310803[] = {{&S17638[42], 7}, {NULL}};
const Segment SGI310804[] = {{&S17638[35], 14}, {NULL}};
const Segment SGI310805[] = {{&S17638[156], 15}, {NULL}};
const Segment SGI310806[] = {{&S17638[51], 5}, {NULL}};
const Segment SGI310807[] = {{&S17639[30], 3}, {NULL}};
const Segment SGI310808[] = {{&S17639[186], 9}, {&S17639[186], 9}, {NULL}};
const Segment SGI310809[] = {{&S17639[8], 3}, {NULL}};
const Segment SGI310810[] = {{&S17639[13], 10}, {NULL}};
const Segment SGI310811[] = {{&S17639[42], 7}, {NULL}};
const Segment SGI310812[] = {{&S17639[35], 14}, {NULL}};
const Segment SGI310813[] = {{&S17639[51], 5}, {NULL}};
const Segment SGI310814[] = {{&S17640[30], 3}, {NULL}};
const Segment SGI310815[] = {{&S17640[190], 9}, {NULL}};
const Segment SGI310816[] = {{&S17640[8], 3}, {NULL}};
const Segment SGI310817[] = {{&S17640[13], 10}, {NULL}};
const Segment SGI310818[] = {{&S17640[42], 7}, {NULL}};
const Segment SGI310819[] = {{&S17640[35], 14}, {NULL}};
const Segment SGI310820[] = {{&S17640[151], 15}, {NULL}};
const Segment SGI310821[] = {{&S17640[51], 6}, {NULL}};
const Segment SGI310822[] = {{&S17641[30], 3}, {NULL}};
const Segment SGI310823[] = {{&S17641[181], 9}, {NULL}};
const Segment SGI310824[] = {{&S17641[8], 3}, {NULL}};
const Segment SGI310825[] = {{&S17641[13], 10}, {NULL}};
const Segment SGI310826[] = {{&S17641[42], 7}, {NULL}};
const Segment SGI310827[] = {{&S17641[35], 14}, {NULL}};
const Segment SGI310828[] = {{&S17641[51], 6}, {NULL}};
const Segment SGI310829[] = {{&S17642[30], 3}, {NULL}};
const Segment SGI310830[] = {{&S17642[201], 9}, {NULL}};
const Segment SGI310831[] = {{&S17642[8], 3}, {NULL}};
const Segment SGI310832[] = {{&S17642[13], 10}, {NULL}};
const Segment SGI310833[] = {{&S17642[42], 7}, {NULL}};
const Segment SGI310834[] = {{&S17642[35], 14}, {NULL}};
const Segment SGI310835[] = {{&S17642[51], 5}, {NULL}};
const Segment SGI310836[] = {{&S17642[96], 9}, {NULL}};
const Segment SGI310837[] = {{&S17643[30], 3}, {NULL}};
const Segment SGI310838[] = {{&S17643[179], 9}, {NULL}};
const Segment SGI310839[] = {{&S17643[8], 3}, {NULL}};
const Segment SGI310840[] = {{&S17643[13], 10}, {NULL}};
const Segment SGI310841[] = {{&S17643[42], 7}, {NULL}};
const Segment SGI310842[] = {{&S17643[35], 14}, {NULL}};
const Segment SGI310843[] = {{&S17643[151], 4}, {NULL}};
const Segment SGI310844[] = {{&S17643[51], 6}, {NULL}};
const Segment SGI310845[] = {{&S17644[30], 3}, {NULL}};
const Segment SGI310846[] = {{&S17644[196], 9}, {NULL}};
const Segment SGI310847[] = {{&S17644[8], 3}, {NULL}};
const Segment SGI310848[] = {{&S17644[13], 10}, {NULL}};
const Segment SGI310849[] = {{&S17644[42], 7}, {NULL}};
const Segment SGI310850[] = {{&S17644[35], 14}, {NULL}};
const Segment SGI310851[] = {{&S17644[51], 6}, {NULL}};
const Segment SGI310852[] = {{&S17644[151], 9}, {NULL}};
const Segment SGI310853[] = {{&S17645[30], 3}, {NULL}};
const Segment SGI310854[] = {{&S17645[229], 9}, {NULL}};
const Segment SGI310855[] = {{&S17645[8], 3}, {NULL}};
const Segment SGI310856[] = {{&S17645[13], 10}, {NULL}};
const Segment SGI310857[] = {{&S17645[42], 7}, {NULL}};
const Segment SGI310858[] = {{&S17645[35], 14}, {NULL}};
const Segment SGI310859[] = {{&S17645[156], 15}, {NULL}};
const Segment SGI310860[] = {{&S17645[51], 5}, {NULL}};
const Segment SGI310861[] = {{&S17645[184], 9}, {NULL}};
const Segment SGI310862[] = {{&S17646[30], 3}, {NULL}};
const Segment SGI310863[] = {{&S17646[214], 9}, {&S17646[214], 9}, {NULL}};
const Segment SGI310864[] = {{&S17646[8], 3}, {NULL}};
const Segment SGI310865[] = {{&S17646[13], 10}, {NULL}};
const Segment SGI310866[] = {{&S17646[42], 7}, {NULL}};
const Segment SGI310867[] = {{&S17646[35], 14}, {NULL}};
const Segment SGI310868[] = {{&S17646[156], 15}, {NULL}};
const Segment SGI310869[] = {{&S17646[51], 5}, {NULL}};
const Segment SGI310870[] = {{&S17647[30], 3}, {NULL}};
const Segment SGI310871[] = {{&S17647[196], 9}, {NULL}};
const Segment SGI310872[] = {{&S17647[8], 3}, {NULL}};
const Segment SGI310873[] = {{&S17647[13], 10}, {NULL}};
const Segment SGI310874[] = {{&S17647[42], 7}, {NULL}};
const Segment SGI310875[] = {{&S17647[35], 14}, {NULL}};
const Segment SGI310876[] = {{&S17647[168], 4}, {NULL}};
const Segment SGI310877[] = {{&S17647[51], 5}, {NULL}};
const Segment SGI310878[] = {{&S17648[30], 3}, {NULL}};
const Segment SGI310879[] = {{&S17648[201], 9}, {&S17648[201], 9}, {NULL}};
const Segment SGI310880[] = {{&S17648[8], 3}, {NULL}};
const Segment SGI310881[] = {{&S17648[13], 10}, {NULL}};
const Segment SGI310882[] = {{&S17648[42], 7}, {NULL}};
const Segment SGI310883[] = {{&S17648[35], 14}, {NULL}};
const Segment SGI310884[] = {{&S17648[51], 5}, {NULL}};
const Segment SGI310885[] = {{&S17648[96], 9}, {NULL}};
const Segment SGI310886[] = {{&S17649[30], 3}, {NULL}};
const Segment SGI310887[] = {{&S17649[196], 9}, {&S17649[196], 9}, {NULL}};
const Segment SGI310888[] = {{&S17649[8], 3}, {NULL}};
const Segment SGI310889[] = {{&S17649[13], 10}, {NULL}};
const Segment SGI310890[] = {{&S17649[42], 7}, {NULL}};
const Segment SGI310891[] = {{&S17649[35], 14}, {NULL}};
const Segment SGI310892[] = {{&S17649[51], 6}, {NULL}};
const Segment SGI310893[] = {{&S17649[151], 9}, {NULL}};
const Segment SGI310894[] = {{&S17650[30], 3}, {NULL}};
const Segment SGI310895[] = {{&S17650[196], 9}, {&S17650[196], 9}, {&S17650[196], 9}, {&S17650[196], 9}, {NULL}};
const Segment SGI310896[] = {{&S17650[8], 3}, {NULL}};
const Segment SGI310897[] = {{&S17650[13], 10}, {NULL}};
const Segment SGI310898[] = {{&S17650[42], 7}, {NULL}};
const Segment SGI310899[] = {{&S17650[35], 14}, {NULL}};
const Segment SGI310900[] = {{&S17650[51], 6}, {NULL}};
const Segment SGI310901[] = {{&S17650[151], 9}, {NULL}};
const Segment SGI310902[] = {{&S17651[30], 3}, {NULL}};
const Segment SGI310903[] = {{&S17651[190], 9}, {&S17651[190], 9}, {NULL}};
const Segment SGI310904[] = {{&S17651[8], 3}, {NULL}};
const Segment SGI310905[] = {{&S17651[13], 10}, {NULL}};
const Segment SGI310906[] = {{&S17651[42], 7}, {NULL}};
const Segment SGI310907[] = {{&S17651[35], 14}, {NULL}};
const Segment SGI310908[] = {{&S17651[151], 15}, {NULL}};
const Segment SGI310909[] = {{&S17651[51], 6}, {NULL}};
const Segment SGI310910[] = {{&S17652[30], 3}, {NULL}};
const Segment SGI310911[] = {{&S17652[229], 9}, {&S17652[229], 9}, {NULL}};
const Segment SGI310912[] = {{&S17652[8], 3}, {NULL}};
const Segment SGI310913[] = {{&S17652[13], 10}, {NULL}};
const Segment SGI310914[] = {{&S17652[42], 7}, {NULL}};
const Segment SGI310915[] = {{&S17652[35], 14}, {NULL}};
const Segment SGI310916[] = {{&S17652[156], 15}, {NULL}};
const Segment SGI310917[] = {{&S17652[51], 5}, {NULL}};
const Segment SGI310918[] = {{&S17652[184], 9}, {NULL}};
const Segment SGI310919[] = {{&S17653[30], 3}, {NULL}};
const Segment SGI310920[] = {{&S17653[217], 9}, {NULL}};
const Segment SGI310921[] = {{&S17653[8], 3}, {NULL}};
const Segment SGI310922[] = {{&S17653[13], 10}, {NULL}};
const Segment SGI310923[] = {{&S17653[42], 7}, {NULL}};
const Segment SGI310924[] = {{&S17653[35], 14}, {NULL}};
const Segment SGI310925[] = {{&S17653[144], 15}, {NULL}};
const Segment SGI310926[] = {{&S17653[172], 9}, {NULL}};
const Segment SGI310927[] = {{&S17654[30], 3}, {NULL}};
const Segment SGI310928[] = {{&S17654[185], 9}, {NULL}};
const Segment SGI310929[] = {{&S17654[8], 3}, {NULL}};
const Segment SGI310930[] = {{&S17654[13], 10}, {NULL}};
const Segment SGI310931[] = {{&S17654[42], 7}, {NULL}};
const Segment SGI310932[] = {{&S17654[35], 14}, {NULL}};
const Segment SGI310933[] = {{&S17654[144], 9}, {NULL}};
const Segment SGI310934[] = {{&S17655[30], 3}, {NULL}};
const Segment SGI310935[] = {{&S17655[206], 9}, {NULL}};
const Segment SGI310936[] = {{&S17655[8], 3}, {NULL}};
const Segment SGI310937[] = {{&S17655[13], 10}, {NULL}};
const Segment SGI310938[] = {{&S17655[42], 7}, {NULL}};
const Segment SGI310939[] = {{&S17655[35], 14}, {NULL}};
const Segment SGI310940[] = {{&S17655[166], 4}, {NULL}};
const Segment SGI310941[] = {{&S17655[51], 6}, {NULL}};
const Segment SGI310942[] = {{&S17655[151], 9}, {NULL}};
const Segment SGI310943[] = {{&S17656[30], 3}, {NULL}};
const Segment SGI310944[] = {{&S17656[202], 9}, {NULL}};
const Segment SGI310945[] = {{&S17656[8], 3}, {NULL}};
const Segment SGI310946[] = {{&S17656[13], 10}, {NULL}};
const Segment SGI310947[] = {{&S17656[42], 7}, {NULL}};
const Segment SGI310948[] = {{&S17656[35], 14}, {NULL}};
const Segment SGI310949[] = {{&S17656[144], 15}, {NULL}};
const Segment SGI310950[] = {{&S17657[30], 3}, {NULL}};
const Segment SGI310951[] = {{&S17657[191], 9}, {&S17657[191], 9}, {NULL}};
const Segment SGI310952[] = {{&S17657[8], 3}, {NULL}};
const Segment SGI310953[] = {{&S17657[13], 10}, {NULL}};
const Segment SGI310954[] = {{&S17657[42], 7}, {NULL}};
const Segment SGI310955[] = {{&S17657[35], 14}, {NULL}};
const Segment SGI310956[] = {{&S17657[151], 4}, {NULL}};
const Segment SGI310957[] = {{&S17657[51], 6}, {NULL}};
const Segment SGI310958[] = {{&S17658[30], 3}, {NULL}};
const Segment SGI310959[] = {{&S17658[181], 9}, {&S17658[181], 9}, {&S17658[181], 9}, {NULL}};
const Segment SGI310960[] = {{&S17658[8], 3}, {NULL}};
const Segment SGI310961[] = {{&S17658[13], 10}, {NULL}};
const Segment SGI310962[] = {{&S17658[42], 7}, {NULL}};
const Segment SGI310963[] = {{&S17658[35], 14}, {NULL}};
const Segment SGI310964[] = {{&S17658[51], 6}, {NULL}};
const Segment SGI310965[] = {{&S17659[30], 3}, {NULL}};
const Segment SGI310966[] = {{&S17659[206], 9}, {&S17659[206], 9}, {NULL}};
const Segment SGI310967[] = {{&S17659[8], 3}, {NULL}};
const Segment SGI310968[] = {{&S17659[13], 10}, {NULL}};
const Segment SGI310969[] = {{&S17659[42], 7}, {NULL}};
const Segment SGI310970[] = {{&S17659[35], 14}, {NULL}};
const Segment SGI310971[] = {{&S17659[166], 4}, {NULL}};
const Segment SGI310972[] = {{&S17659[51], 6}, {NULL}};
const Segment SGI310973[] = {{&S17659[151], 9}, {NULL}};
const Segment SGI310974[] = {{&S17660[30], 3}, {NULL}};
const Segment SGI310975[] = {{&S17660[174], 9}, {&S17660[174], 9}, {NULL}};
const Segment SGI310976[] = {{&S17660[8], 3}, {NULL}};
const Segment SGI310977[] = {{&S17660[13], 10}, {NULL}};
const Segment SGI310978[] = {{&S17660[42], 7}, {NULL}};
const Segment SGI310979[] = {{&S17660[35], 14}, {NULL}};
const Segment SGI310980[] = {{&S17661[30], 3}, {NULL}};
const Segment SGI310981[] = {{&S17661[205], 9}, {NULL}};
const Segment SGI310982[] = {{&S17661[8], 3}, {NULL}};
const Segment SGI310983[] = {{&S17661[13], 10}, {NULL}};
const Segment SGI310984[] = {{&S17661[42], 7}, {NULL}};
const Segment SGI310985[] = {{&S17661[35], 14}, {NULL}};
const Segment SGI310986[] = {{&S17661[151], 15}, {NULL}};
const Segment SGI310987[] = {{&S17661[51], 6}, {NULL}};
const Segment SGI310988[] = {{&S17661[172], 9}, {NULL}};
const Segment SGI310989[] = {{&S17662[30], 3}, {NULL}};
const Segment SGI310990[] = {{&S17662[196], 9}, {&S17662[196], 9}, {NULL}};
const Segment SGI310991[] = {{&S17662[8], 3}, {NULL}};
const Segment SGI310992[] = {{&S17662[13], 10}, {NULL}};
const Segment SGI310993[] = {{&S17662[42], 7}, {NULL}};
const Segment SGI310994[] = {{&S17662[35], 14}, {NULL}};
const Segment SGI310995[] = {{&S17662[168], 4}, {NULL}};
const Segment SGI310996[] = {{&S17662[51], 5}, {NULL}};
const Segment SGI310997[] = {{&S17663[30], 3}, {NULL}};
const Segment SGI310998[] = {{&S17663[186], 9}, {&S17663[186], 9}, {&S17663[186], 9}, {NULL}};
const Segment SGI310999[] = {{&S17663[8], 3}, {NULL}};
const Segment SGI311000[] = {{&S17663[13], 10}, {NULL}};
const Segment SGI311001[] = {{&S17663[42], 7}, {NULL}};
const Segment SGI311002[] = {{&S17663[35], 14}, {NULL}};
const Segment SGI311003[] = {{&S17663[51], 5}, {NULL}};
const Segment SGI311004[] = {{&S17664[30], 3}, {NULL}};
const Segment SGI311005[] = {{&S17664[186], 9}, {&S17664[186], 9}, {&S17664[186], 9}, {&S17664[186], 9}, {NULL}};
const Segment SGI311006[] = {{&S17664[8], 3}, {NULL}};
const Segment SGI311007[] = {{&S17664[13], 10}, {NULL}};
const Segment SGI311008[] = {{&S17664[42], 7}, {NULL}};
const Segment SGI311009[] = {{&S17664[35], 14}, {NULL}};
const Segment SGI311010[] = {{&S17664[51], 5}, {NULL}};
const Segment SGI311011[] = {{&S17665[30], 3}, {NULL}};
const Segment SGI311012[] = {{&S17665[224], 9}, {NULL}};
const Segment SGI311013[] = {{&S17665[8], 3}, {NULL}};
const Segment SGI311014[] = {{&S17665[13], 10}, {NULL}};
const Segment SGI311015[] = {{&S17665[42], 7}, {NULL}};
const Segment SGI311016[] = {{&S17665[35], 14}, {NULL}};
const Segment SGI311017[] = {{&S17665[156], 15}, {NULL}};
const Segment SGI311018[] = {{&S17665[184], 4}, {NULL}};
const Segment SGI311019[] = {{&S17665[51], 5}, {NULL}};
const Segment SGI311020[] = {{&S17666[30], 4}, {NULL}};
const Segment SGI311021[] = {{&S17666[90], 9}, {NULL}};
const Segment SGI311022[] = {{&S17666[8], 3}, {NULL}};
const Segment SGI311023[] = {{&S17666[13], 10}, {NULL}};
const Segment SGI311024[] = {{&S17666[43], 7}, {NULL}};
const Segment SGI311025[] = {{&S17666[36], 14}, {NULL}};
const Segment SGI311026[] = {{&S17666[52], 6}, {NULL}};
const Segment SGI311027[] = {{&S17667[30], 4}, {NULL}};
const Segment SGI311028[] = {{&S17667[90], 9}, {&S17667[90], 9}, {NULL}};
const Segment SGI311029[] = {{&S17667[8], 3}, {NULL}};
const Segment SGI311030[] = {{&S17667[13], 10}, {NULL}};
const Segment SGI311031[] = {{&S17667[43], 7}, {NULL}};
const Segment SGI311032[] = {{&S17667[36], 14}, {NULL}};
const Segment SGI311033[] = {{&S17667[52], 6}, {NULL}};
const Segment SGI311034[] = {{&S17668[30], 3}, {NULL}};
const Segment SGI311035[] = {{&S17668[239], 9}, {NULL}};
const Segment SGI311036[] = {{&S17668[8], 3}, {NULL}};
const Segment SGI311037[] = {{&S17668[13], 10}, {NULL}};
const Segment SGI311038[] = {{&S17668[42], 7}, {NULL}};
const Segment SGI311039[] = {{&S17668[35], 14}, {NULL}};
const Segment SGI311040[] = {{&S17668[156], 15}, {NULL}};
const Segment SGI311041[] = {{&S17668[199], 4}, {NULL}};
const Segment SGI311042[] = {{&S17668[51], 5}, {NULL}};
const Segment SGI311043[] = {{&S17668[184], 9}, {NULL}};
const Segment SGI311044[] = {{&S17669[30], 3}, {NULL}};
const Segment SGI311045[] = {{&S17669[196], 9}, {&S17669[196], 9}, {&S17669[196], 9}, {NULL}};
const Segment SGI311046[] = {{&S17669[8], 3}, {NULL}};
const Segment SGI311047[] = {{&S17669[13], 10}, {NULL}};
const Segment SGI311048[] = {{&S17669[42], 7}, {NULL}};
const Segment SGI311049[] = {{&S17669[35], 14}, {NULL}};
const Segment SGI311050[] = {{&S17669[51], 6}, {NULL}};
const Segment SGI311051[] = {{&S17669[151], 9}, {NULL}};
const Segment SGI311052[] = {{&S17670[30], 3}, {NULL}};
const Segment SGI311053[] = {{&S17670[169], 9}, {NULL}};
const Segment SGI311054[] = {{&S17670[8], 3}, {NULL}};
const Segment SGI311055[] = {{&S17670[13], 10}, {NULL}};
const Segment SGI311056[] = {{&S17670[42], 7}, {NULL}};
const Segment SGI311057[] = {{&S17670[35], 14}, {NULL}};
const Segment SGI311058[] = {{&S17670[51], 6}, {NULL}};
const Segment SGI311059[] = {{&S17671[30], 3}, {NULL}};
const Segment SGI311060[] = {{&S17671[196], 9}, {&S17671[196], 9}, {&S17671[196], 9}, {NULL}};
const Segment SGI311061[] = {{&S17671[8], 3}, {NULL}};
const Segment SGI311062[] = {{&S17671[13], 10}, {NULL}};
const Segment SGI311063[] = {{&S17671[42], 7}, {NULL}};
const Segment SGI311064[] = {{&S17671[35], 14}, {NULL}};
const Segment SGI311065[] = {{&S17671[51], 6}, {NULL}};
const Segment SGI311066[] = {{&S17671[151], 9}, {NULL}};
const Segment SGI311067[] = {{&S17672[30], 3}, {NULL}};
const Segment SGI311068[] = {{&S17672[206], 9}, {NULL}};
const Segment SGI311069[] = {{&S17672[8], 3}, {NULL}};
const Segment SGI311070[] = {{&S17672[13], 10}, {NULL}};
const Segment SGI311071[] = {{&S17672[42], 7}, {NULL}};
const Segment SGI311072[] = {{&S17672[35], 14}, {NULL}};
const Segment SGI311073[] = {{&S17672[166], 4}, {NULL}};
const Segment SGI311074[] = {{&S17672[51], 6}, {NULL}};
const Segment SGI311075[] = {{&S17672[151], 9}, {NULL}};
const Segment SGI311076[] = {{&S17673[30], 3}, {NULL}};
const Segment SGI311077[] = {{&S17673[181], 9}, {&S17673[181], 9}, {&S17673[181], 9}, {&S17673[181], 9}, {NULL}};
const Segment SGI311078[] = {{&S17673[8], 3}, {NULL}};
const Segment SGI311079[] = {{&S17673[13], 10}, {NULL}};
const Segment SGI311080[] = {{&S17673[42], 7}, {NULL}};
const Segment SGI311081[] = {{&S17673[35], 14}, {NULL}};
const Segment SGI311082[] = {{&S17673[51], 6}, {NULL}};
const Segment SGI311083[] = {{&S17674[30], 3}, {NULL}};
const Segment SGI311084[] = {{&S17674[186], 9}, {NULL}};
const Segment SGI311085[] = {{&S17674[8], 3}, {NULL}};
const Segment SGI311086[] = {{&S17674[13], 10}, {NULL}};
const Segment SGI311087[] = {{&S17674[42], 7}, {NULL}};
const Segment SGI311088[] = {{&S17674[35], 14}, {NULL}};
const Segment SGI311089[] = {{&S17674[51], 5}, {NULL}};
const Segment SGI311090[] = {{&S17675[30], 3}, {NULL}};
const Segment SGI311091[] = {{&S17675[192], 9}, {NULL}};
const Segment SGI311092[] = {{&S17675[8], 3}, {NULL}};
const Segment SGI311093[] = {{&S17675[13], 10}, {NULL}};
const Segment SGI311094[] = {{&S17675[42], 7}, {NULL}};
const Segment SGI311095[] = {{&S17675[35], 14}, {NULL}};
const Segment SGI311096[] = {{&S17675[51], 6}, {NULL}};
const Segment SGI311097[] = {{&S17675[151], 9}, {NULL}};
const Segment SGI311098[] = {{&S17676[30], 3}, {NULL}};
const Segment SGI311099[] = {{&S17676[190], 9}, {NULL}};
const Segment SGI311100[] = {{&S17676[8], 3}, {NULL}};
const Segment SGI311101[] = {{&S17676[13], 10}, {NULL}};
const Segment SGI311102[] = {{&S17676[42], 7}, {NULL}};
const Segment SGI311103[] = {{&S17676[35], 14}, {NULL}};
const Segment SGI311104[] = {{&S17676[151], 15}, {NULL}};
const Segment SGI311105[] = {{&S17676[51], 6}, {NULL}};
const Segment SGI311106[] = {{&S17677[30], 3}, {NULL}};
const Segment SGI311107[] = {{&S17677[174], 9}, {NULL}};
const Segment SGI311108[] = {{&S17677[8], 3}, {NULL}};
const Segment SGI311109[] = {{&S17677[13], 10}, {NULL}};
const Segment SGI311110[] = {{&S17677[42], 7}, {NULL}};
const Segment SGI311111[] = {{&S17677[35], 14}, {NULL}};
const Segment SGI311112[] = {{&S17678[30], 3}, {NULL}};
const Segment SGI311113[] = {{&S17678[186], 9}, {&S17678[186], 9}, {&S17678[186], 9}, {&S17678[186], 9}, {NULL}};
const Segment SGI311114[] = {{&S17678[8], 3}, {NULL}};
const Segment SGI311115[] = {{&S17678[13], 10}, {NULL}};
const Segment SGI311116[] = {{&S17678[42], 7}, {NULL}};
const Segment SGI311117[] = {{&S17678[35], 14}, {NULL}};
const Segment SGI311118[] = {{&S17678[51], 5}, {NULL}};
const Segment SGI311119[] = {{&S17679[30], 4}, {NULL}};
const Segment SGI311120[] = {{&S17679[95], 9}, {&S17679[95], 9}, {NULL}};
const Segment SGI311121[] = {{&S17679[8], 3}, {NULL}};
const Segment SGI311122[] = {{&S17679[13], 10}, {NULL}};
const Segment SGI311123[] = {{&S17679[43], 7}, {NULL}};
const Segment SGI311124[] = {{&S17679[36], 14}, {NULL}};
const Segment SGI311125[] = {{&S17679[52], 5}, {NULL}};
const Segment SGI311126[] = {{&S17680[30], 3}, {NULL}};
const Segment SGI311127[] = {{&S17680[207], 9}, {NULL}};
const Segment SGI311128[] = {{&S17680[8], 3}, {NULL}};
const Segment SGI311129[] = {{&S17680[13], 10}, {NULL}};
const Segment SGI311130[] = {{&S17680[42], 7}, {NULL}};
const Segment SGI311131[] = {{&S17680[35], 14}, {NULL}};
const Segment SGI311132[] = {{&S17680[96], 15}, {NULL}};
const Segment SGI311133[] = {{&S17680[51], 5}, {NULL}};
const Segment SGI311134[] = {{&S17681[30], 3}, {NULL}};
const Segment SGI311135[] = {{&S17681[196], 9}, {&S17681[196], 9}, {NULL}};
const Segment SGI311136[] = {{&S17681[8], 3}, {NULL}};
const Segment SGI311137[] = {{&S17681[13], 10}, {NULL}};
const Segment SGI311138[] = {{&S17681[42], 7}, {NULL}};
const Segment SGI311139[] = {{&S17681[35], 14}, {NULL}};
const Segment SGI311140[] = {{&S17681[51], 6}, {NULL}};
const Segment SGI311141[] = {{&S17681[151], 9}, {NULL}};
const Segment SGI311142[] = {{&S17683[30], 3}, {NULL}};
const Segment SGI311143[] = {{&S17683[181], 9}, {&S17683[181], 9}, {NULL}};
const Segment SGI311144[] = {{&S17683[8], 3}, {NULL}};
const Segment SGI311145[] = {{&S17683[13], 10}, {NULL}};
const Segment SGI311146[] = {{&S17683[42], 7}, {NULL}};
const Segment SGI311147[] = {{&S17683[35], 14}, {NULL}};
const Segment SGI311148[] = {{&S17683[51], 6}, {NULL}};
const Segment SGI311149[] = {{&S17684[30], 3}, {NULL}};
const Segment SGI311150[] = {{&S17684[186], 9}, {&S17684[186], 9}, {NULL}};
const Segment SGI311151[] = {{&S17684[8], 3}, {NULL}};
const Segment SGI311152[] = {{&S17684[13], 10}, {NULL}};
const Segment SGI311153[] = {{&S17684[42], 7}, {NULL}};
const Segment SGI311154[] = {{&S17684[35], 14}, {NULL}};
const Segment SGI311155[] = {{&S17684[51], 5}, {NULL}};
const Segment SGI311156[] = {{&S17685[30], 3}, {NULL}};
const Segment SGI311157[] = {{&S17685[181], 9}, {&S17685[181], 9}, {&S17685[181], 9}, {&S17685[181], 9}, {NULL}};
const Segment SGI311158[] = {{&S17685[8], 3}, {NULL}};
const Segment SGI311159[] = {{&S17685[13], 10}, {NULL}};
const Segment SGI311160[] = {{&S17685[42], 7}, {NULL}};
const Segment SGI311161[] = {{&S17685[35], 14}, {NULL}};
const Segment SGI311162[] = {{&S17685[51], 6}, {NULL}};
const Segment SGI311163[] = {{&S17686[30], 3}, {NULL}};
const Segment SGI311164[] = {{&S17686[179], 9}, {NULL}};
const Segment SGI311165[] = {{&S17686[8], 3}, {NULL}};
const Segment SGI311166[] = {{&S17686[13], 10}, {NULL}};
const Segment SGI311167[] = {{&S17686[42], 7}, {NULL}};
const Segment SGI311168[] = {{&S17686[35], 14}, {NULL}};
const Segment SGI311169[] = {{&S17686[151], 4}, {NULL}};
const Segment SGI311170[] = {{&S17686[51], 6}, {NULL}};
const Segment SGI311171[] = {{&S17687[30], 3}, {NULL}};
const Segment SGI311172[] = {{&S17687[229], 9}, {NULL}};
const Segment SGI311173[] = {{&S17687[8], 3}, {NULL}};
const Segment SGI311174[] = {{&S17687[13], 10}, {NULL}};
const Segment SGI311175[] = {{&S17687[42], 7}, {NULL}};
const Segment SGI311176[] = {{&S17687[35], 14}, {NULL}};
const Segment SGI311177[] = {{&S17687[156], 15}, {NULL}};
const Segment SGI311178[] = {{&S17687[51], 5}, {NULL}};
const Segment SGI311179[] = {{&S17687[184], 9}, {NULL}};
const Segment SGI311180[] = {{&S17688[30], 3}, {NULL}};
const Segment SGI311181[] = {{&S17688[201], 9}, {NULL}};
const Segment SGI311182[] = {{&S17688[8], 3}, {NULL}};
const Segment SGI311183[] = {{&S17688[13], 10}, {NULL}};
const Segment SGI311184[] = {{&S17688[42], 7}, {NULL}};
const Segment SGI311185[] = {{&S17688[35], 14}, {NULL}};
const Segment SGI311186[] = {{&S17688[51], 5}, {NULL}};
const Segment SGI311187[] = {{&S17688[96], 9}, {NULL}};
const Segment SGI311188[] = {{&S17689[30], 3}, {NULL}};
const Segment SGI311189[] = {{&S17689[196], 9}, {NULL}};
const Segment SGI311190[] = {{&S17689[8], 3}, {NULL}};
const Segment SGI311191[] = {{&S17689[13], 10}, {NULL}};
const Segment SGI311192[] = {{&S17689[42], 7}, {NULL}};
const Segment SGI311193[] = {{&S17689[35], 14}, {NULL}};
const Segment SGI311194[] = {{&S17689[168], 4}, {NULL}};
const Segment SGI311195[] = {{&S17689[51], 5}, {NULL}};
const Segment SGI311196[] = {{&S17690[30], 3}, {NULL}};
const Segment SGI311197[] = {{&S17690[209], 9}, {&S17690[209], 9}, {NULL}};
const Segment SGI311198[] = {{&S17690[8], 3}, {NULL}};
const Segment SGI311199[] = {{&S17690[13], 10}, {NULL}};
const Segment SGI311200[] = {{&S17690[42], 7}, {NULL}};
const Segment SGI311201[] = {{&S17690[35], 14}, {NULL}};
const Segment SGI311202[] = {{&S17690[51], 5}, {NULL}};
const Segment SGI311203[] = {{&S17690[156], 9}, {NULL}};
const Segment SGI311204[] = {{&S18390[30], 3}, {NULL}};
const Segment SGI311205[] = {{&S18390[8], 3}, {NULL}};
const Segment SGI311206[] = {{&S18390[13], 10}, {NULL}};
const Segment SGI311207[] = {{&S18390[42], 7}, {NULL}};
const Segment SGI311208[] = {{&S18390[35], 14}, {NULL}};
const Segment SGI311209[] = {{&S18390[51], 7}, {NULL}};
const Segment SGI311210[] = {{&S18391[30], 3}, {NULL}};
const Segment SGI311211[] = {{&S18391[8], 3}, {NULL}};
const Segment SGI311212[] = {{&S18391[13], 10}, {NULL}};
const Segment SGI311213[] = {{&S18391[42], 7}, {NULL}};
const Segment SGI311214[] = {{&S18391[35], 14}, {NULL}};
const Segment SGI311215[] = {{&S18391[51], 7}, {NULL}};
const Segment SGI311216[] = {{&S18392[30], 3}, {NULL}};
const Segment SGI311217[] = {{&S18392[8], 3}, {NULL}};
const Segment SGI311218[] = {{&S18392[13], 10}, {NULL}};
const Segment SGI311219[] = {{&S18392[42], 7}, {NULL}};
const Segment SGI311220[] = {{&S18392[35], 14}, {NULL}};
const Segment SGI311221[] = {{&S18392[64], 7}, {NULL}};
const Segment SGI311222[] = {{&S18393[30], 3}, {NULL}};
const Segment SGI311223[] = {{&S18393[8], 3}, {NULL}};
const Segment SGI311224[] = {{&S18393[13], 10}, {NULL}};
const Segment SGI311225[] = {{&S18393[42], 7}, {NULL}};
const Segment SGI311226[] = {{&S18393[35], 14}, {NULL}};
const Segment SGI311227[] = {{&S18393[56], 7}, {NULL}};
const Segment SGI311228[] = {{&S18393[51], 3}, {NULL}};
const Segment SGI311229[] = {{&S18394[30], 3}, {NULL}};
const Segment SGI311230[] = {{&S18394[8], 3}, {NULL}};
const Segment SGI311231[] = {{&S18394[13], 10}, {NULL}};
const Segment SGI311232[] = {{&S18394[42], 7}, {NULL}};
const Segment SGI311233[] = {{&S18394[35], 14}, {NULL}};
const Segment SGI311234[] = {{&S18394[75], 7}, {NULL}};
const Segment SGI311235[] = {{&S18394[51], 3}, {NULL}};
const Segment SGI311236[] = {{&S18395[30], 3}, {&S18395[30], 3}, {NULL}};
const Segment SGI311237[] = {{&S18395[8], 3}, {&S18395[8], 3}, {NULL}};
const Segment SGI311238[] = {{&S18395[13], 10}, {&S18395[13], 10}, {NULL}};
const Segment SGI311239[] = {{&S18395[42], 7}, {&S18395[42], 7}, {NULL}};
const Segment SGI311240[] = {{&S18395[35], 14}, {&S18395[35], 14}, {NULL}};
const Segment SGI311241[] = {{&S18395[56], 7}, {NULL}};
const Segment SGI311242[] = {{&S18395[51], 3}, {&S18395[51], 3}, {NULL}};
const Segment SGI311243[] = {{&S18396[30], 3}, {NULL}};
const Segment SGI311244[] = {{&S18396[8], 3}, {NULL}};
const Segment SGI311245[] = {{&S18396[13], 10}, {NULL}};
const Segment SGI311246[] = {{&S18396[42], 7}, {NULL}};
const Segment SGI311247[] = {{&S18396[35], 14}, {NULL}};
const Segment SGI311248[] = {{&S18396[62], 7}, {NULL}};
const Segment SGI311249[] = {{&S18397[30], 3}, {NULL}};
const Segment SGI311250[] = {{&S18397[8], 3}, {NULL}};
const Segment SGI311251[] = {{&S18397[13], 10}, {NULL}};
const Segment SGI311252[] = {{&S18397[42], 7}, {NULL}};
const Segment SGI311253[] = {{&S18397[35], 14}, {NULL}};
const Segment SGI311254[] = {{&S18397[75], 7}, {NULL}};
const Segment SGI311255[] = {{&S18397[51], 3}, {NULL}};
const Segment SGI311256[] = {{&S18398[30], 3}, {NULL}};
const Segment SGI311257[] = {{&S18398[8], 3}, {NULL}};
const Segment SGI311258[] = {{&S18398[13], 10}, {NULL}};
const Segment SGI311259[] = {{&S18398[42], 7}, {NULL}};
const Segment SGI311260[] = {{&S18398[35], 14}, {NULL}};
const Segment SGI311261[] = {{&S18398[56], 7}, {NULL}};
const Segment SGI311262[] = {{&S18398[51], 3}, {NULL}};
const Segment SGI311263[] = {{&S18399[30], 3}, {NULL}};
const Segment SGI311264[] = {{&S18399[8], 3}, {NULL}};
const Segment SGI311265[] = {{&S18399[13], 10}, {NULL}};
const Segment SGI311266[] = {{&S18399[42], 7}, {NULL}};
const Segment SGI311267[] = {{&S18399[35], 14}, {NULL}};
const Segment SGI311268[] = {{&S18399[147], 7}, {NULL}};
const Segment SGI311269[] = {{&S18400[30], 3}, {NULL}};
const Segment SGI311270[] = {{&S18400[8], 3}, {NULL}};
const Segment SGI311271[] = {{&S18400[13], 10}, {NULL}};
const Segment SGI311272[] = {{&S18400[42], 7}, {NULL}};
const Segment SGI311273[] = {{&S18400[35], 14}, {NULL}};
const Segment SGI311274[] = {{&S18400[51], 7}, {NULL}};
const Segment SGI311275[] = {{&S18401[30], 3}, {NULL}};
const Segment SGI311276[] = {{&S18401[8], 3}, {NULL}};
const Segment SGI311277[] = {{&S18401[13], 10}, {NULL}};
const Segment SGI311278[] = {{&S18401[42], 7}, {NULL}};
const Segment SGI311279[] = {{&S18401[35], 14}, {NULL}};
const Segment SGI311280[] = {{&S18401[56], 7}, {NULL}};
const Segment SGI311281[] = {{&S18401[51], 3}, {NULL}};
const Segment SGI311282[] = {{&S18402[30], 3}, {NULL}};
const Segment SGI311283[] = {{&S18402[8], 3}, {NULL}};
const Segment SGI311284[] = {{&S18402[13], 10}, {NULL}};
const Segment SGI311285[] = {{&S18402[42], 7}, {NULL}};
const Segment SGI311286[] = {{&S18402[35], 14}, {NULL}};
const Segment SGI311287[] = {{&S18402[56], 7}, {NULL}};
const Segment SGI311288[] = {{&S18402[51], 3}, {NULL}};
const Segment SGI311289[] = {{&S18403[30], 3}, {&S18403[94], 3}, {NULL}};
const Segment SGI311290[] = {{&S18403[8], 3}, {&S18403[8], 3}, {NULL}};
const Segment SGI311291[] = {{&S18403[13], 10}, {&S18403[13], 10}, {NULL}};
const Segment SGI311292[] = {{&S18403[42], 7}, {&S18403[42], 7}, {NULL}};
const Segment SGI311293[] = {{&S18403[35], 14}, {&S18403[35], 14}, {NULL}};
const Segment SGI311294[] = {{&S18403[122], 7}, {NULL}};
const Segment SGI311295[] = {{&S18403[115], 3}, {NULL}};
const Segment SGI311296[] = {{&S18404[30], 3}, {NULL}};
const Segment SGI311297[] = {{&S18404[8], 3}, {NULL}};
const Segment SGI311298[] = {{&S18404[13], 10}, {NULL}};
const Segment SGI311299[] = {{&S18404[42], 7}, {NULL}};
const Segment SGI311300[] = {{&S18404[35], 14}, {NULL}};
const Segment SGI311301[] = {{&S18404[64], 7}, {NULL}};
const Segment SGI311302[] = {{&S18405[30], 3}, {NULL}};
const Segment SGI311303[] = {{&S18405[8], 3}, {NULL}};
const Segment SGI311304[] = {{&S18405[13], 10}, {NULL}};
const Segment SGI311305[] = {{&S18405[42], 7}, {NULL}};
const Segment SGI311306[] = {{&S18405[35], 14}, {NULL}};
const Segment SGI311307[] = {{&S18405[167], 7}, {NULL}};
const Segment SGI311308[] = {{&S18406[30], 3}, {NULL}};
const Segment SGI311309[] = {{&S18406[8], 3}, {NULL}};
const Segment SGI311310[] = {{&S18406[13], 10}, {NULL}};
const Segment SGI311311[] = {{&S18406[42], 7}, {NULL}};
const Segment SGI311312[] = {{&S18406[35], 14}, {NULL}};
const Segment SGI311313[] = {{&S18406[103], 15}, {NULL}};
const Segment SGI311314[] = {{&S18406[56], 7}, {NULL}};
const Segment SGI311315[] = {{&S18406[51], 3}, {NULL}};
const Segment SGI311316[] = {{&S18407[30], 3}, {NULL}};
const Segment SGI311317[] = {{&S18407[8], 3}, {NULL}};
const Segment SGI311318[] = {{&S18407[13], 10}, {NULL}};
const Segment SGI311319[] = {{&S18407[42], 7}, {NULL}};
const Segment SGI311320[] = {{&S18407[35], 14}, {NULL}};
const Segment SGI311321[] = {{&S18407[56], 7}, {&S18407[56], 7}, {NULL}};
const Segment SGI311322[] = {{&S18407[51], 3}, {NULL}};
const Segment SGI311323[] = {{&S18408[30], 3}, {NULL}};
const Segment SGI311324[] = {{&S18408[8], 3}, {NULL}};
const Segment SGI311325[] = {{&S18408[13], 10}, {NULL}};
const Segment SGI311326[] = {{&S18408[42], 7}, {NULL}};
const Segment SGI311327[] = {{&S18408[35], 14}, {NULL}};
const Segment SGI311328[] = {{&S18408[111], 15}, {NULL}};
const Segment SGI311329[] = {{&S18408[64], 7}, {NULL}};
const Segment SGI311330[] = {{&S18409[30], 3}, {NULL}};
const Segment SGI311331[] = {{&S18409[8], 3}, {NULL}};
const Segment SGI311332[] = {{&S18409[13], 10}, {NULL}};
const Segment SGI311333[] = {{&S18409[42], 7}, {NULL}};
const Segment SGI311334[] = {{&S18409[35], 14}, {NULL}};
const Segment SGI311335[] = {{&S18409[56], 7}, {NULL}};
const Segment SGI311336[] = {{&S18409[51], 3}, {NULL}};
const Segment SGI311337[] = {{&S18410[30], 3}, {NULL}};
const Segment SGI311338[] = {{&S18410[8], 3}, {NULL}};
const Segment SGI311339[] = {{&S18410[13], 10}, {NULL}};
const Segment SGI311340[] = {{&S18410[42], 7}, {NULL}};
const Segment SGI311341[] = {{&S18410[35], 14}, {NULL}};
const Segment SGI311342[] = {{&S18410[64], 7}, {NULL}};
const Segment SGI311343[] = {{&S18411[30], 3}, {NULL}};
const Segment SGI311344[] = {{&S18411[8], 3}, {NULL}};
const Segment SGI311345[] = {{&S18411[13], 10}, {NULL}};
const Segment SGI311346[] = {{&S18411[42], 7}, {NULL}};
const Segment SGI311347[] = {{&S18411[35], 14}, {NULL}};
const Segment SGI311348[] = {{&S18411[51], 6}, {NULL}};
const Segment SGI311349[] = {{&S18411[98], 7}, {NULL}};
const Segment SGI311350[] = {{&S18412[30], 3}, {NULL}};
const Segment SGI311351[] = {{&S18412[8], 3}, {NULL}};
const Segment SGI311352[] = {{&S18412[13], 10}, {NULL}};
const Segment SGI311353[] = {{&S18412[42], 7}, {NULL}};
const Segment SGI311354[] = {{&S18412[35], 14}, {NULL}};
const Segment SGI311355[] = {{&S18412[131], 15}, {NULL}};
const Segment SGI311356[] = {{&S18412[152], 7}, {NULL}};
const Segment SGI311357[] = {{&S18413[30], 3}, {NULL}};
const Segment SGI311358[] = {{&S18413[8], 3}, {NULL}};
const Segment SGI311359[] = {{&S18413[13], 10}, {NULL}};
const Segment SGI311360[] = {{&S18413[42], 7}, {NULL}};
const Segment SGI311361[] = {{&S18413[35], 14}, {NULL}};
const Segment SGI311362[] = {{&S18413[131], 15}, {NULL}};
const Segment SGI311363[] = {{&S18413[152], 7}, {NULL}};
const Segment SGI311364[] = {{&S18414[30], 3}, {NULL}};
const Segment SGI311365[] = {{&S18414[8], 3}, {NULL}};
const Segment SGI311366[] = {{&S18414[13], 10}, {NULL}};
const Segment SGI311367[] = {{&S18414[55], 7}, {NULL}};
const Segment SGI311368[] = {{&S18414[48], 14}, {NULL}};
const Segment SGI311369[] = {{&S18414[64], 7}, {NULL}};
const Segment SGI311370[] = {{&S18415[30], 3}, {NULL}};
const Segment SGI311371[] = {{&S18415[8], 3}, {NULL}};
const Segment SGI311372[] = {{&S18415[13], 10}, {NULL}};
const Segment SGI311373[] = {{&S18415[55], 7}, {NULL}};
const Segment SGI311374[] = {{&S18415[48], 14}, {NULL}};
const Segment SGI311375[] = {{&S18415[64], 7}, {NULL}};
const Segment SGI311376[] = {{&S18416[30], 3}, {NULL}};
const Segment SGI311377[] = {{&S18416[8], 3}, {NULL}};
const Segment SGI311378[] = {{&S18416[13], 10}, {NULL}};
const Segment SGI311379[] = {{&S18416[55], 7}, {NULL}};
const Segment SGI311380[] = {{&S18416[48], 14}, {NULL}};
const Segment SGI311381[] = {{&S18416[64], 7}, {NULL}};
const Segment SGI311382[] = {{&S18417[30], 3}, {&S18417[103], 3}, {NULL}};
const Segment SGI311383[] = {{&S18417[8], 3}, {&S18417[8], 3}, {NULL}};
const Segment SGI311384[] = {{&S18417[13], 10}, {&S18417[13], 10}, {NULL}};
const Segment SGI311385[] = {{&S18417[42], 7}, {&S18417[42], 7}, {NULL}};
const Segment SGI311386[] = {{&S18417[35], 14}, {&S18417[35], 14}, {NULL}};
const Segment SGI311387[] = {{&S18417[64], 7}, {NULL}};
const Segment SGI311388[] = {{&S18417[124], 3}, {NULL}};
const Segment SGI311389[] = {{&S18418[30], 3}, {NULL}};
const Segment SGI311390[] = {{&S18418[8], 3}, {NULL}};
const Segment SGI311391[] = {{&S18418[13], 10}, {NULL}};
const Segment SGI311392[] = {{&S18418[42], 7}, {NULL}};
const Segment SGI311393[] = {{&S18418[35], 14}, {NULL}};
const Segment SGI311394[] = {{&S18418[51], 6}, {NULL}};
const Segment SGI311395[] = {{&S18418[98], 7}, {NULL}};
const Segment SGI311396[] = {{&S18419[30], 3}, {NULL}};
const Segment SGI311397[] = {{&S18419[8], 3}, {NULL}};
const Segment SGI311398[] = {{&S18419[13], 10}, {NULL}};
const Segment SGI311399[] = {{&S18419[55], 7}, {NULL}};
const Segment SGI311400[] = {{&S18419[48], 14}, {NULL}};
const Segment SGI311401[] = {{&S18419[64], 7}, {NULL}};
const Segment SGI311402[] = {{&S18420[30], 3}, {NULL}};
const Segment SGI311403[] = {{&S18420[8], 3}, {NULL}};
const Segment SGI311404[] = {{&S18420[13], 10}, {NULL}};
const Segment SGI311405[] = {{&S18420[55], 7}, {NULL}};
const Segment SGI311406[] = {{&S18420[48], 14}, {NULL}};
const Segment SGI311407[] = {{&S18420[64], 7}, {NULL}};
const Segment SGI311408[] = {{&S18421[30], 3}, {NULL}};
const Segment SGI311409[] = {{&S18421[8], 3}, {NULL}};
const Segment SGI311410[] = {{&S18421[13], 10}, {NULL}};
const Segment SGI311411[] = {{&S18421[55], 7}, {NULL}};
const Segment SGI311412[] = {{&S18421[48], 14}, {NULL}};
const Segment SGI311413[] = {{&S18421[64], 7}, {NULL}};
const Segment SGI311414[] = {{&S18422[30], 3}, {NULL}};
const Segment SGI311415[] = {{&S18422[8], 3}, {NULL}};
const Segment SGI311416[] = {{&S18422[13], 10}, {NULL}};
const Segment SGI311417[] = {{&S18422[55], 7}, {NULL}};
const Segment SGI311418[] = {{&S18422[48], 14}, {NULL}};
const Segment SGI311419[] = {{&S18422[64], 7}, {NULL}};
const Segment SGI311420[] = {{&S18423[30], 3}, {NULL}};
const Segment SGI311421[] = {{&S18423[8], 3}, {NULL}};
const Segment SGI311422[] = {{&S18423[13], 10}, {NULL}};
const Segment SGI311423[] = {{&S18423[55], 7}, {NULL}};
const Segment SGI311424[] = {{&S18423[48], 14}, {NULL}};
const Segment SGI311425[] = {{&S18423[64], 7}, {NULL}};
const Segment SGI311426[] = {{&S18424[30], 3}, {NULL}};
const Segment SGI311427[] = {{&S18424[8], 3}, {NULL}};
const Segment SGI311428[] = {{&S18424[13], 10}, {NULL}};
const Segment SGI311429[] = {{&S18424[55], 7}, {NULL}};
const Segment SGI311430[] = {{&S18424[48], 14}, {NULL}};
const Segment SGI311431[] = {{&S18424[64], 7}, {NULL}};
const Segment SGI311432[] = {{&S18425[30], 3}, {NULL}};
const Segment SGI311433[] = {{&S18425[8], 3}, {NULL}};
const Segment SGI311434[] = {{&S18425[13], 10}, {NULL}};
const Segment SGI311435[] = {{&S18425[42], 7}, {NULL}};
const Segment SGI311436[] = {{&S18425[35], 14}, {NULL}};
const Segment SGI311437[] = {{&S18425[131], 15}, {NULL}};
const Segment SGI311438[] = {{&S18425[152], 7}, {NULL}};
const Segment SGI311439[] = {{&S18426[30], 3}, {NULL}};
const Segment SGI311440[] = {{&S18426[8], 3}, {NULL}};
const Segment SGI311441[] = {{&S18426[13], 10}, {NULL}};
const Segment SGI311442[] = {{&S18426[42], 7}, {NULL}};
const Segment SGI311443[] = {{&S18426[35], 14}, {NULL}};
const Segment SGI311444[] = {{&S18426[131], 15}, {NULL}};
const Segment SGI311445[] = {{&S18426[167], 7}, {NULL}};
const Segment SGI311446[] = {{&S18426[152], 9}, {NULL}};
const Segment SGI311447[] = {{&S18427[30], 3}, {NULL}};
const Segment SGI311448[] = {{&S18427[8], 3}, {NULL}};
const Segment SGI311449[] = {{&S18427[13], 10}, {NULL}};
const Segment SGI311450[] = {{&S18427[42], 7}, {NULL}};
const Segment SGI311451[] = {{&S18427[35], 14}, {NULL}};
const Segment SGI311452[] = {{&S18427[138], 15}, {NULL}};
const Segment SGI311453[] = {{&S18427[51], 6}, {NULL}};
const Segment SGI311454[] = {{&S18427[179], 7}, {NULL}};
const Segment SGI311455[] = {{&S18428[30], 3}, {NULL}};
const Segment SGI311456[] = {{&S18428[8], 3}, {NULL}};
const Segment SGI311457[] = {{&S18428[13], 10}, {NULL}};
const Segment SGI311458[] = {{&S18428[42], 7}, {NULL}};
const Segment SGI311459[] = {{&S18428[35], 14}, {NULL}};
const Segment SGI311460[] = {{&S18428[138], 15}, {NULL}};
const Segment SGI311461[] = {{&S18428[51], 6}, {NULL}};
const Segment SGI311462[] = {{&S18428[179], 7}, {NULL}};
const Segment SGI311463[] = {{&S18429[30], 3}, {NULL}};
const Segment SGI311464[] = {{&S18429[8], 3}, {NULL}};
const Segment SGI311465[] = {{&S18429[13], 10}, {NULL}};
const Segment SGI311466[] = {{&S18429[42], 7}, {NULL}};
const Segment SGI311467[] = {{&S18429[35], 14}, {NULL}};
const Segment SGI311468[] = {{&S18429[51], 6}, {NULL}};
const Segment SGI311469[] = {{&S18429[63], 7}, {NULL}};
const Segment SGI311470[] = {{&S18429[58], 3}, {NULL}};
const Segment SGI311471[] = {{&S18430[30], 3}, {NULL}};
const Segment SGI311472[] = {{&S18430[8], 3}, {NULL}};
const Segment SGI311473[] = {{&S18430[13], 10}, {NULL}};
const Segment SGI311474[] = {{&S18430[42], 7}, {NULL}};
const Segment SGI311475[] = {{&S18430[35], 14}, {NULL}};
const Segment SGI311476[] = {{&S18430[69], 7}, {NULL}};
const Segment SGI311477[] = {{&S18430[64], 3}, {NULL}};
const Segment SGI311478[] = {{&S18431[30], 3}, {NULL}};
const Segment SGI311479[] = {{&S18431[8], 3}, {NULL}};
const Segment SGI311480[] = {{&S18431[13], 10}, {NULL}};
const Segment SGI311481[] = {{&S18431[42], 7}, {NULL}};
const Segment SGI311482[] = {{&S18431[35], 14}, {NULL}};
const Segment SGI311483[] = {{&S18431[64], 7}, {&S18431[64], 7}, {NULL}};
const Segment SGI311484[] = {{&S18432[30], 3}, {NULL}};
const Segment SGI311485[] = {{&S18432[8], 3}, {NULL}};
const Segment SGI311486[] = {{&S18432[13], 10}, {NULL}};
const Segment SGI311487[] = {{&S18432[42], 7}, {NULL}};
const Segment SGI311488[] = {{&S18432[35], 14}, {NULL}};
const Segment SGI311489[] = {{&S18432[51], 6}, {NULL}};
const Segment SGI311490[] = {{&S18432[98], 7}, {NULL}};
const Segment SGI311491[] = {{&S18433[29], 3}, {NULL}};
const Segment SGI311492[] = {{&S18433[8], 3}, {NULL}};
const Segment SGI311493[] = {{&S18433[41], 7}, {NULL}};
const Segment SGI311494[] = {{&S18433[34], 14}, {NULL}};
const Segment SGI311495[] = {{&S18433[50], 7}, {NULL}};
const Segment SGI311496[] = {{&S18434[30], 3}, {NULL}};
const Segment SGI311497[] = {{&S18434[8], 3}, {NULL}};
const Segment SGI311498[] = {{&S18434[13], 10}, {NULL}};
const Segment SGI311499[] = {{&S18434[42], 7}, {NULL}};
const Segment SGI311500[] = {{&S18434[35], 14}, {NULL}};
const Segment SGI311501[] = {{&S18434[51], 7}, {NULL}};
const Segment SGI311502[] = {{&S18435[30], 3}, {NULL}};
const Segment SGI311503[] = {{&S18435[8], 3}, {NULL}};
const Segment SGI311504[] = {{&S18435[13], 10}, {NULL}};
const Segment SGI311505[] = {{&S18435[55], 7}, {NULL}};
const Segment SGI311506[] = {{&S18435[48], 14}, {NULL}};
const Segment SGI311507[] = {{&S18435[64], 7}, {NULL}};
const Segment SGI311508[] = {{&S18435[73], 13}, {NULL}};
const Segment SGI311509[] = {{&S18436[30], 3}, {NULL}};
const Segment SGI311510[] = {{&S18436[8], 3}, {NULL}};
const Segment SGI311511[] = {{&S18436[13], 10}, {NULL}};
const Segment SGI311512[] = {{&S18436[55], 7}, {NULL}};
const Segment SGI311513[] = {{&S18436[48], 14}, {NULL}};
const Segment SGI311514[] = {{&S18436[69], 7}, {NULL}};
const Segment SGI311515[] = {{&S18436[64], 3}, {NULL}};
const Segment SGI311516[] = {{&S18437[30], 3}, {NULL}};
const Segment SGI311517[] = {{&S18437[8], 3}, {NULL}};
const Segment SGI311518[] = {{&S18437[13], 10}, {NULL}};
const Segment SGI311519[] = {{&S18437[42], 7}, {NULL}};
const Segment SGI311520[] = {{&S18437[35], 14}, {NULL}};
const Segment SGI311521[] = {{&S18437[81], 7}, {NULL}};
const Segment SGI311522[] = {{&S18437[51], 13}, {&S18437[51], 13}, {NULL}};
const Segment SGI311523[] = {{&S18438[30], 3}, {NULL}};
const Segment SGI311524[] = {{&S18438[8], 3}, {NULL}};
const Segment SGI311525[] = {{&S18438[13], 10}, {NULL}};
const Segment SGI311526[] = {{&S18438[42], 7}, {NULL}};
const Segment SGI311527[] = {{&S18438[35], 14}, {NULL}};
const Segment SGI311528[] = {{&S18438[51], 6}, {NULL}};
const Segment SGI311529[] = {{&S18438[63], 7}, {NULL}};
const Segment SGI311530[] = {{&S18438[58], 3}, {NULL}};
const Segment SGI311531[] = {{&S18439[30], 3}, {NULL}};
const Segment SGI311532[] = {{&S18439[8], 3}, {NULL}};
const Segment SGI311533[] = {{&S18439[13], 10}, {NULL}};
const Segment SGI311534[] = {{&S18439[55], 7}, {NULL}};
const Segment SGI311535[] = {{&S18439[48], 14}, {NULL}};
const Segment SGI311536[] = {{&S18439[64], 7}, {NULL}};
const Segment SGI311537[] = {{&S18439[73], 3}, {NULL}};
const Segment SGI311538[] = {{&S18440[30], 3}, {NULL}};
const Segment SGI311539[] = {{&S18440[8], 3}, {NULL}};
const Segment SGI311540[] = {{&S18440[13], 10}, {NULL}};
const Segment SGI311541[] = {{&S18440[42], 7}, {NULL}};
const Segment SGI311542[] = {{&S18440[35], 14}, {NULL}};
const Segment SGI311543[] = {{&S18440[195], 15}, {NULL}};
const Segment SGI311544[] = {{&S18440[251], 7}, {NULL}};
const Segment SGI311545[] = {{&S18440[216], 9}, {NULL}};
const Segment SGI311546[] = {{&S18441[30], 3}, {NULL}};
const Segment SGI311547[] = {{&S18441[8], 3}, {NULL}};
const Segment SGI311548[] = {{&S18441[13], 10}, {NULL}};
const Segment SGI311549[] = {{&S18441[55], 7}, {NULL}};
const Segment SGI311550[] = {{&S18441[48], 14}, {NULL}};
const Segment SGI311551[] = {{&S18441[64], 7}, {&S18441[64], 7}, {NULL}};
const Segment SGI311552[] = {{&S18442[30], 3}, {NULL}};
const Segment SGI311553[] = {{&S18442[8], 3}, {NULL}};
const Segment SGI311554[] = {{&S18442[13], 10}, {NULL}};
const Segment SGI311555[] = {{&S18442[42], 7}, {NULL}};
const Segment SGI311556[] = {{&S18442[35], 14}, {NULL}};
const Segment SGI311557[] = {{&S18442[51], 6}, {NULL}};
const Segment SGI311558[] = {{&S18442[98], 7}, {NULL}};
const Segment SGI311559[] = {{&S18443[30], 3}, {NULL}};
const Segment SGI311560[] = {{&S18443[8], 3}, {NULL}};
const Segment SGI311561[] = {{&S18443[13], 10}, {NULL}};
const Segment SGI311562[] = {{&S18443[42], 7}, {NULL}};
const Segment SGI311563[] = {{&S18443[35], 14}, {NULL}};
const Segment SGI311564[] = {{&S18443[51], 6}, {NULL}};
const Segment SGI311565[] = {{&S18443[98], 7}, {NULL}};
const Segment SGI311566[] = {{&S18444[30], 3}, {NULL}};
const Segment SGI311567[] = {{&S18444[8], 3}, {NULL}};
const Segment SGI311568[] = {{&S18444[13], 10}, {NULL}};
const Segment SGI311569[] = {{&S18444[55], 7}, {NULL}};
const Segment SGI311570[] = {{&S18444[48], 14}, {NULL}};
const Segment SGI311571[] = {{&S18444[64], 7}, {NULL}};
const Segment SGI311572[] = {{&S18444[73], 3}, {NULL}};
const Segment SGI311573[] = {{&S18445[8], 3}, {NULL}};
const Segment SGI311574[] = {{&S18445[50], 11}, {NULL}};
const Segment SGI311575[] = {{&S18445[72], 7}, {NULL}};
const Segment SGI311576[] = {{&S18445[65], 3}, {NULL}};
const Segment SGI311577[] = {{&S18446[30], 3}, {NULL}};
const Segment SGI311578[] = {{&S18446[8], 3}, {NULL}};
const Segment SGI311579[] = {{&S18446[13], 10}, {NULL}};
const Segment SGI311580[] = {{&S18446[42], 7}, {NULL}};
const Segment SGI311581[] = {{&S18446[35], 14}, {NULL}};
const Segment SGI311582[] = {{&S18446[138], 15}, {NULL}};
const Segment SGI311583[] = {{&S18446[51], 6}, {NULL}};
const Segment SGI311584[] = {{&S18446[179], 7}, {NULL}};
const Segment SGI311585[] = {{&S18447[30], 3}, {NULL}};
const Segment SGI311586[] = {{&S18447[8], 3}, {NULL}};
const Segment SGI311587[] = {{&S18447[13], 10}, {NULL}};
const Segment SGI311588[] = {{&S18447[42], 7}, {NULL}};
const Segment SGI311589[] = {{&S18447[35], 14}, {NULL}};
const Segment SGI311590[] = {{&S18447[51], 6}, {NULL}};
const Segment SGI311591[] = {{&S18447[63], 7}, {NULL}};
const Segment SGI311592[] = {{&S18447[58], 3}, {NULL}};
const Segment SGI311593[] = {{&S18448[30], 3}, {NULL}};
const Segment SGI311594[] = {{&S18448[8], 3}, {NULL}};
const Segment SGI311595[] = {{&S18448[13], 10}, {NULL}};
const Segment SGI311596[] = {{&S18448[55], 7}, {NULL}};
const Segment SGI311597[] = {{&S18448[48], 14}, {NULL}};
const Segment SGI311598[] = {{&S18448[64], 7}, {&S18448[64], 7}, {NULL}};
const Segment SGI311599[] = {{&S18449[30], 3}, {NULL}};
const Segment SGI311600[] = {{&S18449[8], 3}, {NULL}};
const Segment SGI311601[] = {{&S18449[13], 10}, {NULL}};
const Segment SGI311602[] = {{&S18449[55], 7}, {NULL}};
const Segment SGI311603[] = {{&S18449[48], 14}, {NULL}};
const Segment SGI311604[] = {{&S18449[79], 7}, {NULL}};
const Segment SGI311605[] = {{&S18449[64], 13}, {NULL}};
const Segment SGI311606[] = {{&S18450[30], 3}, {&S18450[30], 3}, {NULL}};
const Segment SGI311607[] = {{&S18450[8], 3}, {&S18450[8], 3}, {NULL}};
const Segment SGI311608[] = {{&S18450[13], 10}, {&S18450[13], 10}, {NULL}};
const Segment SGI311609[] = {{&S18450[42], 7}, {&S18450[42], 7}, {NULL}};
const Segment SGI311610[] = {{&S18450[35], 14}, {&S18450[35], 14}, {NULL}};
const Segment SGI311611[] = {{&S18450[119], 7}, {NULL}};
const Segment SGI311612[] = {{&S18450[102], 3}, {NULL}};
const Segment SGI311613[] = {{&S18451[30], 3}, {&S18451[81], 3}, {NULL}};
const Segment SGI311614[] = {{&S18451[8], 3}, {&S18451[8], 3}, {NULL}};
const Segment SGI311615[] = {{&S18451[13], 10}, {&S18451[13], 10}, {NULL}};
const Segment SGI311616[] = {{&S18451[42], 7}, {&S18451[42], 7}, {NULL}};
const Segment SGI311617[] = {{&S18451[35], 14}, {&S18451[35], 14}, {NULL}};
const Segment SGI311618[] = {{&S18451[119], 7}, {NULL}};
const Segment SGI311619[] = {{&S18451[102], 3}, {NULL}};
const Segment SGI311620[] = {{&S18452[30], 3}, {NULL}};
const Segment SGI311621[] = {{&S18452[8], 3}, {NULL}};
const Segment SGI311622[] = {{&S18452[13], 10}, {NULL}};
const Segment SGI311623[] = {{&S18452[55], 7}, {NULL}};
const Segment SGI311624[] = {{&S18452[48], 14}, {NULL}};
const Segment SGI311625[] = {{&S18452[64], 7}, {NULL}};
const Segment SGI311626[] = {{&S18452[73], 13}, {NULL}};
const Segment SGI311627[] = {{&S18453[30], 3}, {NULL}};
const Segment SGI311628[] = {{&S18453[8], 3}, {NULL}};
const Segment SGI311629[] = {{&S18453[13], 10}, {NULL}};
const Segment SGI311630[] = {{&S18453[55], 7}, {NULL}};
const Segment SGI311631[] = {{&S18453[48], 14}, {NULL}};
const Segment SGI311632[] = {{&S18453[79], 7}, {NULL}};
const Segment SGI311633[] = {{&S18453[64], 13}, {NULL}};
const Segment SGI311634[] = {{&S18454[30], 3}, {NULL}};
const Segment SGI311635[] = {{&S18454[8], 3}, {NULL}};
const Segment SGI311636[] = {{&S18454[13], 10}, {NULL}};
const Segment SGI311637[] = {{&S18454[55], 7}, {NULL}};
const Segment SGI311638[] = {{&S18454[48], 14}, {NULL}};
const Segment SGI311639[] = {{&S18454[64], 7}, {NULL}};
const Segment SGI311640[] = {{&S18454[73], 3}, {NULL}};
const Segment SGI311641[] = {{&S18455[30], 3}, {NULL}};
const Segment SGI311642[] = {{&S18455[8], 3}, {NULL}};
const Segment SGI311643[] = {{&S18455[13], 10}, {NULL}};
const Segment SGI311644[] = {{&S18455[55], 7}, {NULL}};
const Segment SGI311645[] = {{&S18455[48], 14}, {NULL}};
const Segment SGI311646[] = {{&S18455[69], 7}, {NULL}};
const Segment SGI311647[] = {{&S18455[64], 3}, {NULL}};
const Segment SGI311648[] = {{&S18456[30], 3}, {NULL}};
const Segment SGI311649[] = {{&S18456[8], 3}, {NULL}};
const Segment SGI311650[] = {{&S18456[13], 10}, {NULL}};
const Segment SGI311651[] = {{&S18456[55], 7}, {NULL}};
const Segment SGI311652[] = {{&S18456[48], 14}, {NULL}};
const Segment SGI311653[] = {{&S18456[64], 7}, {NULL}};
const Segment SGI311654[] = {{&S18456[73], 13}, {NULL}};
const Segment SGI311655[] = {{&S18457[30], 3}, {NULL}};
const Segment SGI311656[] = {{&S18457[8], 3}, {NULL}};
const Segment SGI311657[] = {{&S18457[13], 10}, {NULL}};
const Segment SGI311658[] = {{&S18457[55], 7}, {NULL}};
const Segment SGI311659[] = {{&S18457[48], 14}, {NULL}};
const Segment SGI311660[] = {{&S18457[64], 7}, {&S18457[64], 7}, {NULL}};
const Segment SGI311661[] = {{&S18458[30], 3}, {NULL}};
const Segment SGI311662[] = {{&S18458[8], 3}, {NULL}};
const Segment SGI311663[] = {{&S18458[13], 10}, {NULL}};
const Segment SGI311664[] = {{&S18458[55], 7}, {NULL}};
const Segment SGI311665[] = {{&S18458[48], 14}, {NULL}};
const Segment SGI311666[] = {{&S18458[64], 7}, {NULL}};
const Segment SGI311667[] = {{&S18458[73], 13}, {NULL}};
const Segment SGI311668[] = {{&S18459[30], 3}, {NULL}};
const Segment SGI311669[] = {{&S18459[8], 3}, {NULL}};
const Segment SGI311670[] = {{&S18459[13], 10}, {NULL}};
const Segment SGI311671[] = {{&S18459[55], 7}, {NULL}};
const Segment SGI311672[] = {{&S18459[48], 14}, {NULL}};
const Segment SGI311673[] = {{&S18459[64], 7}, {NULL}};
const Segment SGI311674[] = {{&S18459[73], 13}, {NULL}};
const Segment SGI311675[] = {{&S18460[30], 3}, {NULL}};
const Segment SGI311676[] = {{&S18460[8], 3}, {NULL}};
const Segment SGI311677[] = {{&S18460[13], 10}, {NULL}};
const Segment SGI311678[] = {{&S18460[55], 7}, {NULL}};
const Segment SGI311679[] = {{&S18460[48], 14}, {NULL}};
const Segment SGI311680[] = {{&S18460[69], 7}, {NULL}};
const Segment SGI311681[] = {{&S18460[64], 3}, {NULL}};
const Segment SGI311682[] = {{&S18461[30], 3}, {NULL}};
const Segment SGI311683[] = {{&S18461[8], 3}, {NULL}};
const Segment SGI311684[] = {{&S18461[13], 10}, {NULL}};
const Segment SGI311685[] = {{&S18461[55], 7}, {NULL}};
const Segment SGI311686[] = {{&S18461[48], 14}, {NULL}};
const Segment SGI311687[] = {{&S18461[64], 7}, {&S18461[64], 7}, {NULL}};
const Segment SGI311688[] = {{&S18462[30], 3}, {NULL}};
const Segment SGI311689[] = {{&S18462[8], 3}, {NULL}};
const Segment SGI311690[] = {{&S18462[13], 10}, {NULL}};
const Segment SGI311691[] = {{&S18462[55], 7}, {NULL}};
const Segment SGI311692[] = {{&S18462[48], 14}, {NULL}};
const Segment SGI311693[] = {{&S18462[64], 7}, {&S18462[64], 7}, {NULL}};
const Segment SGI311694[] = {{&S18463[30], 3}, {NULL}};
const Segment SGI311695[] = {{&S18463[8], 3}, {NULL}};
const Segment SGI311696[] = {{&S18463[13], 10}, {NULL}};
const Segment SGI311697[] = {{&S18463[55], 7}, {NULL}};
const Segment SGI311698[] = {{&S18463[48], 14}, {NULL}};
const Segment SGI311699[] = {{&S18463[64], 7}, {&S18463[64], 7}, {NULL}};
const Segment SGI311700[] = {{&S18464[30], 3}, {NULL}};
const Segment SGI311701[] = {{&S18464[8], 3}, {NULL}};
const Segment SGI311702[] = {{&S18464[13], 10}, {NULL}};
const Segment SGI311703[] = {{&S18464[42], 7}, {NULL}};
const Segment SGI311704[] = {{&S18464[35], 14}, {NULL}};
const Segment SGI311705[] = {{&S18464[147], 7}, {NULL}};
const Segment SGI311706[] = {{&S18465[30], 3}, {NULL}};
const Segment SGI311707[] = {{&S18465[8], 3}, {NULL}};
const Segment SGI311708[] = {{&S18465[13], 10}, {NULL}};
const Segment SGI311709[] = {{&S18465[55], 7}, {NULL}};
const Segment SGI311710[] = {{&S18465[48], 14}, {NULL}};
const Segment SGI311711[] = {{&S18465[79], 7}, {NULL}};
const Segment SGI311712[] = {{&S18465[64], 13}, {NULL}};
const Segment SGI311713[] = {{&S18466[30], 3}, {NULL}};
const Segment SGI311714[] = {{&S18466[8], 3}, {NULL}};
const Segment SGI311715[] = {{&S18466[13], 10}, {NULL}};
const Segment SGI311716[] = {{&S18466[55], 7}, {NULL}};
const Segment SGI311717[] = {{&S18466[48], 14}, {NULL}};
const Segment SGI311718[] = {{&S18466[64], 7}, {&S18466[64], 7}, {NULL}};
const Segment SGI311719[] = {{&S18467[30], 3}, {NULL}};
const Segment SGI311720[] = {{&S18467[8], 3}, {NULL}};
const Segment SGI311721[] = {{&S18467[13], 10}, {NULL}};
const Segment SGI311722[] = {{&S18467[55], 7}, {NULL}};
const Segment SGI311723[] = {{&S18467[48], 14}, {NULL}};
const Segment SGI311724[] = {{&S18467[69], 7}, {NULL}};
const Segment SGI311725[] = {{&S18467[64], 3}, {NULL}};
const Segment SGI311726[] = {{&S18468[30], 3}, {NULL}};
const Segment SGI311727[] = {{&S18468[8], 3}, {NULL}};
const Segment SGI311728[] = {{&S18468[13], 10}, {NULL}};
const Segment SGI311729[] = {{&S18468[42], 7}, {NULL}};
const Segment SGI311730[] = {{&S18468[35], 14}, {NULL}};
const Segment SGI311731[] = {{&S18468[51], 7}, {NULL}};
const Segment SGI311732[] = {{&S18468[157], 13}, {NULL}};
const Segment SGI311733[] = {{&S18469[30], 3}, {NULL}};
const Segment SGI311734[] = {{&S18469[8], 3}, {NULL}};
const Segment SGI311735[] = {{&S18469[13], 10}, {NULL}};
const Segment SGI311736[] = {{&S18469[55], 7}, {NULL}};
const Segment SGI311737[] = {{&S18469[48], 14}, {NULL}};
const Segment SGI311738[] = {{&S18469[64], 7}, {&S18469[64], 7}, {NULL}};
const Segment SGI311739[] = {{&S50229[7], 3}, {NULL}};
const Segment SGI311740[] = {{&S50229[0], 112}, {NULL}};
const Segment SGI311741[] = {{&S18731[30], 3}, {NULL}};
const Segment SGI311742[] = {{&S18731[214], 9}, {NULL}};
const Segment SGI311743[] = {{&S18731[8], 3}, {NULL}};
const Segment SGI311744[] = {{&S18731[13], 10}, {NULL}};
const Segment SGI311745[] = {{&S18731[42], 7}, {NULL}};
const Segment SGI311746[] = {{&S18731[35], 14}, {NULL}};
const Segment SGI311747[] = {{&S18731[143], 15}, {NULL}};
const Segment SGI311748[] = {{&S18731[51], 5}, {NULL}};
const Segment SGI311749[] = {{&S18732[30], 3}, {NULL}};
const Segment SGI311750[] = {{&S18732[197], 9}, {NULL}};
const Segment SGI311751[] = {{&S18732[8], 3}, {NULL}};
const Segment SGI311752[] = {{&S18732[13], 10}, {NULL}};
const Segment SGI311753[] = {{&S18732[42], 7}, {NULL}};
const Segment SGI311754[] = {{&S18732[35], 14}, {NULL}};
const Segment SGI311755[] = {{&S18732[138], 15}, {NULL}};
const Segment SGI311756[] = {{&S18732[51], 6}, {NULL}};
const Segment SGI311757[] = {{&S18734[30], 3}, {NULL}};
const Segment SGI311758[] = {{&S18734[176], 9}, {NULL}};
const Segment SGI311759[] = {{&S18734[8], 3}, {NULL}};
const Segment SGI311760[] = {{&S18734[13], 10}, {NULL}};
const Segment SGI311761[] = {{&S18734[42], 7}, {NULL}};
const Segment SGI311762[] = {{&S18734[35], 14}, {NULL}};
const Segment SGI311763[] = {{&S18734[51], 6}, {NULL}};
const Segment SGI311764[] = {{&S18735[30], 3}, {NULL}};
const Segment SGI311765[] = {{&S18735[197], 9}, {&S18735[197], 9}, {NULL}};
const Segment SGI311766[] = {{&S18735[8], 3}, {NULL}};
const Segment SGI311767[] = {{&S18735[13], 10}, {NULL}};
const Segment SGI311768[] = {{&S18735[42], 7}, {NULL}};
const Segment SGI311769[] = {{&S18735[35], 14}, {NULL}};
const Segment SGI311770[] = {{&S18735[138], 15}, {NULL}};
const Segment SGI311771[] = {{&S18735[51], 6}, {NULL}};
const Segment SGI311772[] = {{&S18736[30], 3}, {NULL}};
const Segment SGI311773[] = {{&S18736[209], 9}, {NULL}};
const Segment SGI311774[] = {{&S18736[8], 3}, {NULL}};
const Segment SGI311775[] = {{&S18736[13], 10}, {NULL}};
const Segment SGI311776[] = {{&S18736[42], 7}, {NULL}};
const Segment SGI311777[] = {{&S18736[35], 14}, {NULL}};
const Segment SGI311778[] = {{&S18736[138], 15}, {NULL}};
const Segment SGI311779[] = {{&S18736[51], 6}, {NULL}};
const Segment SGI311780[] = {{&S18737[30], 3}, {NULL}};
const Segment SGI311781[] = {{&S18737[214], 9}, {NULL}};
const Segment SGI311782[] = {{&S18737[8], 3}, {NULL}};
const Segment SGI311783[] = {{&S18737[13], 10}, {NULL}};
const Segment SGI311784[] = {{&S18737[42], 7}, {NULL}};
const Segment SGI311785[] = {{&S18737[35], 14}, {NULL}};
const Segment SGI311786[] = {{&S18737[143], 15}, {NULL}};
const Segment SGI311787[] = {{&S18737[51], 5}, {NULL}};
const Segment SGI311788[] = {{&S18738[30], 3}, {NULL}};
const Segment SGI311789[] = {{&S18738[229], 9}, {NULL}};
const Segment SGI311790[] = {{&S18738[8], 3}, {NULL}};
const Segment SGI311791[] = {{&S18738[13], 10}, {NULL}};
const Segment SGI311792[] = {{&S18738[42], 7}, {NULL}};
const Segment SGI311793[] = {{&S18738[35], 14}, {NULL}};
const Segment SGI311794[] = {{&S18738[143], 15}, {NULL}};
const Segment SGI311795[] = {{&S18738[51], 5}, {NULL}};
const Segment SGI311796[] = {{&S18738[184], 9}, {NULL}};
const Segment SGI311797[] = {{&S18740[30], 3}, {NULL}};
const Segment SGI311798[] = {{&S18740[214], 9}, {&S18740[214], 9}, {NULL}};
const Segment SGI311799[] = {{&S18740[8], 3}, {NULL}};
const Segment SGI311800[] = {{&S18740[13], 10}, {NULL}};
const Segment SGI311801[] = {{&S18740[42], 7}, {NULL}};
const Segment SGI311802[] = {{&S18740[35], 14}, {NULL}};
const Segment SGI311803[] = {{&S18740[143], 15}, {NULL}};
const Segment SGI311804[] = {{&S18740[51], 5}, {NULL}};
const Segment SGI311805[] = {{&S18741[30], 3}, {NULL}};
const Segment SGI311806[] = {{&S18741[229], 9}, {&S18741[229], 9}, {NULL}};
const Segment SGI311807[] = {{&S18741[8], 3}, {NULL}};
const Segment SGI311808[] = {{&S18741[13], 10}, {NULL}};
const Segment SGI311809[] = {{&S18741[42], 7}, {NULL}};
const Segment SGI311810[] = {{&S18741[35], 14}, {NULL}};
const Segment SGI311811[] = {{&S18741[143], 15}, {NULL}};
const Segment SGI311812[] = {{&S18741[51], 5}, {NULL}};
const Segment SGI311813[] = {{&S18741[184], 9}, {NULL}};
const Segment SGI311814[] = {{&S18742[30], 3}, {NULL}};
const Segment SGI311815[] = {{&S18742[209], 9}, {&S18742[209], 9}, {NULL}};
const Segment SGI311816[] = {{&S18742[8], 3}, {NULL}};
const Segment SGI311817[] = {{&S18742[13], 10}, {NULL}};
const Segment SGI311818[] = {{&S18742[42], 7}, {NULL}};
const Segment SGI311819[] = {{&S18742[35], 14}, {NULL}};
const Segment SGI311820[] = {{&S18742[138], 15}, {NULL}};
const Segment SGI311821[] = {{&S18742[51], 6}, {NULL}};
const Segment SGI311822[] = {{&S18743[30], 3}, {NULL}};
const Segment SGI311823[] = {{&S18743[202], 9}, {NULL}};
const Segment SGI311824[] = {{&S18743[8], 3}, {NULL}};
const Segment SGI311825[] = {{&S18743[13], 10}, {NULL}};
const Segment SGI311826[] = {{&S18743[42], 7}, {NULL}};
const Segment SGI311827[] = {{&S18743[35], 14}, {NULL}};
const Segment SGI311828[] = {{&S18743[131], 15}, {NULL}};
const Segment SGI311829[] = {{&S18744[30], 3}, {NULL}};
const Segment SGI311830[] = {{&S18744[199], 9}, {NULL}};
const Segment SGI311831[] = {{&S18744[8], 3}, {NULL}};
const Segment SGI311832[] = {{&S18744[13], 10}, {NULL}};
const Segment SGI311833[] = {{&S18744[42], 7}, {NULL}};
const Segment SGI311834[] = {{&S18744[35], 14}, {NULL}};
const Segment SGI311835[] = {{&S18744[138], 15}, {NULL}};
const Segment SGI311836[] = {{&S18744[51], 6}, {NULL}};
const Segment SGI311837[] = {{&S18746[30], 3}, {NULL}};
const Segment SGI311838[] = {{&S18746[219], 9}, {NULL}};
const Segment SGI311839[] = {{&S18746[8], 3}, {NULL}};
const Segment SGI311840[] = {{&S18746[13], 10}, {NULL}};
const Segment SGI311841[] = {{&S18746[42], 7}, {NULL}};
const Segment SGI311842[] = {{&S18746[35], 14}, {NULL}};
const Segment SGI311843[] = {{&S18746[143], 15}, {NULL}};
const Segment SGI311844[] = {{&S18746[51], 5}, {NULL}};
const Segment SGI311845[] = {{&S18746[174], 9}, {NULL}};
const Segment SGI311846[] = {{&S18747[30], 3}, {NULL}};
const Segment SGI311847[] = {{&S18747[204], 9}, {NULL}};
const Segment SGI311848[] = {{&S18747[8], 3}, {NULL}};
const Segment SGI311849[] = {{&S18747[13], 10}, {NULL}};
const Segment SGI311850[] = {{&S18747[42], 7}, {NULL}};
const Segment SGI311851[] = {{&S18747[35], 14}, {NULL}};
const Segment SGI311852[] = {{&S18747[143], 15}, {NULL}};
const Segment SGI311853[] = {{&S18747[51], 5}, {NULL}};
const Segment SGI311854[] = {{&S18748[30], 3}, {NULL}};
const Segment SGI311855[] = {{&S18748[180], 9}, {NULL}};
const Segment SGI311856[] = {{&S18748[8], 3}, {NULL}};
const Segment SGI311857[] = {{&S18748[13], 10}, {NULL}};
const Segment SGI311858[] = {{&S18748[42], 7}, {NULL}};
const Segment SGI311859[] = {{&S18748[35], 14}, {NULL}};
const Segment SGI311860[] = {{&S18748[131], 15}, {NULL}};
const Segment SGI311861[] = {{&S18749[30], 3}, {NULL}};
const Segment SGI311862[] = {{&S18749[217], 9}, {NULL}};
const Segment SGI311863[] = {{&S18749[8], 3}, {NULL}};
const Segment SGI311864[] = {{&S18749[13], 10}, {NULL}};
const Segment SGI311865[] = {{&S18749[42], 7}, {NULL}};
const Segment SGI311866[] = {{&S18749[35], 14}, {NULL}};
const Segment SGI311867[] = {{&S18749[131], 15}, {NULL}};
const Segment SGI311868[] = {{&S18749[172], 9}, {NULL}};
const Segment SGI311869[] = {{&S18750[30], 3}, {NULL}};
const Segment SGI311870[] = {{&S18750[202], 9}, {&S18750[202], 9}, {NULL}};
const Segment SGI311871[] = {{&S18750[8], 3}, {NULL}};
const Segment SGI311872[] = {{&S18750[13], 10}, {NULL}};
const Segment SGI311873[] = {{&S18750[42], 7}, {NULL}};
const Segment SGI311874[] = {{&S18750[35], 14}, {NULL}};
const Segment SGI311875[] = {{&S18750[131], 15}, {NULL}};
const Segment SGI311876[] = {{&S18751[30], 3}, {NULL}};
const Segment SGI311877[] = {{&S18751[192], 9}, {NULL}};
const Segment SGI311878[] = {{&S18751[8], 3}, {NULL}};
const Segment SGI311879[] = {{&S18751[13], 10}, {NULL}};
const Segment SGI311880[] = {{&S18751[42], 7}, {NULL}};
const Segment SGI311881[] = {{&S18751[35], 14}, {NULL}};
const Segment SGI311882[] = {{&S18751[131], 15}, {NULL}};
const Segment SGI311883[] = {{&S18752[30], 3}, {NULL}};
const Segment SGI311884[] = {{&S18752[205], 9}, {&S18752[205], 9}, {NULL}};
const Segment SGI311885[] = {{&S18752[8], 3}, {NULL}};
const Segment SGI311886[] = {{&S18752[13], 10}, {NULL}};
const Segment SGI311887[] = {{&S18752[42], 7}, {NULL}};
const Segment SGI311888[] = {{&S18752[35], 14}, {NULL}};
const Segment SGI311889[] = {{&S18752[138], 15}, {NULL}};
const Segment SGI311890[] = {{&S18752[51], 6}, {NULL}};
const Segment SGI311891[] = {{&S18754[30], 3}, {NULL}};
const Segment SGI311892[] = {{&S18754[192], 9}, {NULL}};
const Segment SGI311893[] = {{&S18754[8], 3}, {NULL}};
const Segment SGI311894[] = {{&S18754[13], 10}, {NULL}};
const Segment SGI311895[] = {{&S18754[42], 7}, {NULL}};
const Segment SGI311896[] = {{&S18754[35], 14}, {NULL}};
const Segment SGI311897[] = {{&S18754[131], 15}, {NULL}};
const Segment SGI311898[] = {{&S18755[30], 3}, {NULL}};
const Segment SGI311899[] = {{&S18755[209], 9}, {&S18755[209], 9}, {NULL}};
const Segment SGI311900[] = {{&S18755[8], 3}, {NULL}};
const Segment SGI311901[] = {{&S18755[13], 10}, {NULL}};
const Segment SGI311902[] = {{&S18755[42], 7}, {NULL}};
const Segment SGI311903[] = {{&S18755[35], 14}, {NULL}};
const Segment SGI311904[] = {{&S18755[138], 15}, {NULL}};
const Segment SGI311905[] = {{&S18755[51], 6}, {NULL}};
const Segment SGI311906[] = {{&S18756[30], 3}, {NULL}};
const Segment SGI311907[] = {{&S18756[214], 9}, {&S18756[214], 9}, {NULL}};
const Segment SGI311908[] = {{&S18756[8], 3}, {NULL}};
const Segment SGI311909[] = {{&S18756[13], 10}, {NULL}};
const Segment SGI311910[] = {{&S18756[42], 7}, {NULL}};
const Segment SGI311911[] = {{&S18756[35], 14}, {NULL}};
const Segment SGI311912[] = {{&S18756[143], 15}, {NULL}};
const Segment SGI311913[] = {{&S18756[51], 5}, {NULL}};
const Segment SGI311914[] = {{&S18757[30], 3}, {NULL}};
const Segment SGI311915[] = {{&S18757[209], 9}, {NULL}};
const Segment SGI311916[] = {{&S18757[8], 3}, {NULL}};
const Segment SGI311917[] = {{&S18757[13], 10}, {NULL}};
const Segment SGI311918[] = {{&S18757[42], 7}, {NULL}};
const Segment SGI311919[] = {{&S18757[35], 14}, {NULL}};
const Segment SGI311920[] = {{&S18757[138], 15}, {NULL}};
const Segment SGI311921[] = {{&S18757[51], 6}, {NULL}};
const Segment SGI311922[] = {{&S18758[30], 3}, {NULL}};
const Segment SGI311923[] = {{&S18758[214], 9}, {NULL}};
const Segment SGI311924[] = {{&S18758[8], 3}, {NULL}};
const Segment SGI311925[] = {{&S18758[13], 10}, {NULL}};
const Segment SGI311926[] = {{&S18758[42], 7}, {NULL}};
const Segment SGI311927[] = {{&S18758[35], 14}, {NULL}};
const Segment SGI311928[] = {{&S18758[143], 15}, {NULL}};
const Segment SGI311929[] = {{&S18758[51], 5}, {NULL}};
const Segment SGI311930[] = {{&S18759[30], 3}, {NULL}};
const Segment SGI311931[] = {{&S18759[229], 9}, {NULL}};
const Segment SGI311932[] = {{&S18759[8], 3}, {NULL}};
const Segment SGI311933[] = {{&S18759[13], 10}, {NULL}};
const Segment SGI311934[] = {{&S18759[42], 7}, {NULL}};
const Segment SGI311935[] = {{&S18759[35], 14}, {NULL}};
const Segment SGI311936[] = {{&S18759[143], 15}, {NULL}};
const Segment SGI311937[] = {{&S18759[51], 5}, {NULL}};
const Segment SGI311938[] = {{&S18759[184], 9}, {NULL}};
const Segment SGI311939[] = {{&S18761[30], 3}, {NULL}};
const Segment SGI311940[] = {{&S18761[229], 9}, {NULL}};
const Segment SGI311941[] = {{&S18761[8], 3}, {NULL}};
const Segment SGI311942[] = {{&S18761[13], 10}, {NULL}};
const Segment SGI311943[] = {{&S18761[42], 7}, {NULL}};
const Segment SGI311944[] = {{&S18761[35], 14}, {NULL}};
const Segment SGI311945[] = {{&S18761[143], 15}, {NULL}};
const Segment SGI311946[] = {{&S18761[51], 5}, {NULL}};
const Segment SGI311947[] = {{&S18761[184], 9}, {NULL}};
const Segment SGI311948[] = {{&S18762[30], 3}, {NULL}};
const Segment SGI311949[] = {{&S18762[207], 9}, {NULL}};
const Segment SGI311950[] = {{&S18762[8], 3}, {NULL}};
const Segment SGI311951[] = {{&S18762[13], 10}, {NULL}};
const Segment SGI311952[] = {{&S18762[42], 7}, {NULL}};
const Segment SGI311953[] = {{&S18762[35], 14}, {NULL}};
const Segment SGI311954[] = {{&S18762[131], 15}, {NULL}};
const Segment SGI311955[] = {{&S18762[162], 9}, {NULL}};
const Segment SGI311956[] = {{&S18763[30], 3}, {NULL}};
const Segment SGI311957[] = {{&S18763[207], 9}, {NULL}};
const Segment SGI311958[] = {{&S18763[8], 3}, {NULL}};
const Segment SGI311959[] = {{&S18763[13], 10}, {NULL}};
const Segment SGI311960[] = {{&S18763[42], 7}, {NULL}};
const Segment SGI311961[] = {{&S18763[35], 14}, {NULL}};
const Segment SGI311962[] = {{&S18763[131], 15}, {NULL}};
const Segment SGI311963[] = {{&S18763[162], 9}, {NULL}};
const Segment SGI311964[] = {{&S18764[30], 3}, {NULL}};
const Segment SGI311965[] = {{&S18764[229], 9}, {&S18764[229], 9}, {NULL}};
const Segment SGI311966[] = {{&S18764[8], 3}, {NULL}};
const Segment SGI311967[] = {{&S18764[13], 10}, {NULL}};
const Segment SGI311968[] = {{&S18764[42], 7}, {NULL}};
const Segment SGI311969[] = {{&S18764[35], 14}, {NULL}};
const Segment SGI311970[] = {{&S18764[143], 15}, {NULL}};
const Segment SGI311971[] = {{&S18764[51], 5}, {NULL}};
const Segment SGI311972[] = {{&S18764[184], 9}, {NULL}};
const Segment SGI311973[] = {{&S18765[30], 3}, {NULL}};
const Segment SGI311974[] = {{&S18765[217], 9}, {&S18765[217], 9}, {NULL}};
const Segment SGI311975[] = {{&S18765[8], 3}, {NULL}};
const Segment SGI311976[] = {{&S18765[13], 10}, {NULL}};
const Segment SGI311977[] = {{&S18765[42], 7}, {NULL}};
const Segment SGI311978[] = {{&S18765[35], 14}, {NULL}};
const Segment SGI311979[] = {{&S18765[131], 15}, {NULL}};
const Segment SGI311980[] = {{&S18765[172], 9}, {NULL}};
const Segment SGI311981[] = {{&S18766[30], 3}, {NULL}};
const Segment SGI311982[] = {{&S18766[193], 9}, {NULL}};
const Segment SGI311983[] = {{&S18766[8], 3}, {NULL}};
const Segment SGI311984[] = {{&S18766[13], 10}, {NULL}};
const Segment SGI311985[] = {{&S18766[42], 7}, {NULL}};
const Segment SGI311986[] = {{&S18766[35], 14}, {NULL}};
const Segment SGI311987[] = {{&S18766[51], 5}, {NULL}};
const Segment SGI311988[] = {{&S18767[30], 3}, {NULL}};
const Segment SGI311989[] = {{&S18767[188], 9}, {&S18767[188], 9}, {NULL}};
const Segment SGI311990[] = {{&S18767[8], 3}, {NULL}};
const Segment SGI311991[] = {{&S18767[13], 10}, {NULL}};
const Segment SGI311992[] = {{&S18767[42], 7}, {NULL}};
const Segment SGI311993[] = {{&S18767[35], 14}, {NULL}};
const Segment SGI311994[] = {{&S18767[51], 6}, {NULL}};
const Segment SGI311995[] = {{&S18768[30], 3}, {NULL}};
const Segment SGI311996[] = {{&S18768[219], 9}, {&S18768[219], 9}, {NULL}};
const Segment SGI311997[] = {{&S18768[8], 3}, {NULL}};
const Segment SGI311998[] = {{&S18768[13], 10}, {NULL}};
const Segment SGI311999[] = {{&S18768[42], 7}, {NULL}};
const Segment SGI312000[] = {{&S18768[35], 14}, {NULL}};
const Segment SGI312001[] = {{&S18768[138], 15}, {NULL}};
const Segment SGI312002[] = {{&S18768[179], 4}, {NULL}};
const Segment SGI312003[] = {{&S18768[51], 6}, {NULL}};
const Segment SGI312004[] = {{&S18769[30], 3}, {NULL}};
const Segment SGI312005[] = {{&S18769[192], 9}, {&S18769[192], 9}, {NULL}};
const Segment SGI312006[] = {{&S18769[8], 3}, {NULL}};
const Segment SGI312007[] = {{&S18769[13], 10}, {NULL}};
const Segment SGI312008[] = {{&S18769[42], 7}, {NULL}};
const Segment SGI312009[] = {{&S18769[35], 14}, {NULL}};
const Segment SGI312010[] = {{&S18769[131], 15}, {NULL}};
const Segment SGI312011[] = {{&S18770[30], 3}, {NULL}};
const Segment SGI312012[] = {{&S18770[188], 9}, {NULL}};
const Segment SGI312013[] = {{&S18770[8], 3}, {NULL}};
const Segment SGI312014[] = {{&S18770[13], 10}, {NULL}};
const Segment SGI312015[] = {{&S18770[42], 7}, {NULL}};
const Segment SGI312016[] = {{&S18770[35], 14}, {NULL}};
const Segment SGI312017[] = {{&S18770[51], 6}, {NULL}};
const Segment SGI312018[] = {{&S18771[30], 3}, {NULL}};
const Segment SGI312019[] = {{&S18771[219], 9}, {NULL}};
const Segment SGI312020[] = {{&S18771[8], 3}, {NULL}};
const Segment SGI312021[] = {{&S18771[13], 10}, {NULL}};
const Segment SGI312022[] = {{&S18771[42], 7}, {NULL}};
const Segment SGI312023[] = {{&S18771[35], 14}, {NULL}};
const Segment SGI312024[] = {{&S18771[138], 15}, {NULL}};
const Segment SGI312025[] = {{&S18771[179], 4}, {NULL}};
const Segment SGI312026[] = {{&S18771[51], 6}, {NULL}};
const Segment SGI312027[] = {{&S18772[30], 3}, {NULL}};
const Segment SGI312028[] = {{&S18772[188], 9}, {&S18772[188], 9}, {NULL}};
const Segment SGI312029[] = {{&S18772[8], 3}, {NULL}};
const Segment SGI312030[] = {{&S18772[13], 10}, {NULL}};
const Segment SGI312031[] = {{&S18772[42], 7}, {NULL}};
const Segment SGI312032[] = {{&S18772[35], 14}, {NULL}};
const Segment SGI312033[] = {{&S18772[51], 6}, {NULL}};
const Segment SGI312034[] = {{&S18773[30], 3}, {NULL}};
const Segment SGI312035[] = {{&S18773[193], 9}, {NULL}};
const Segment SGI312036[] = {{&S18773[8], 3}, {NULL}};
const Segment SGI312037[] = {{&S18773[13], 10}, {NULL}};
const Segment SGI312038[] = {{&S18773[42], 7}, {NULL}};
const Segment SGI312039[] = {{&S18773[35], 14}, {NULL}};
const Segment SGI312040[] = {{&S18773[51], 5}, {NULL}};
const Segment SGI312041[] = {{&S18774[30], 3}, {NULL}};
const Segment SGI312042[] = {{&S18774[217], 9}, {&S18774[217], 9}, {NULL}};
const Segment SGI312043[] = {{&S18774[8], 3}, {NULL}};
const Segment SGI312044[] = {{&S18774[13], 10}, {NULL}};
const Segment SGI312045[] = {{&S18774[42], 7}, {NULL}};
const Segment SGI312046[] = {{&S18774[35], 14}, {NULL}};
const Segment SGI312047[] = {{&S18774[131], 15}, {NULL}};
const Segment SGI312048[] = {{&S18774[172], 9}, {NULL}};
const Segment SGI312049[] = {{&S18775[30], 3}, {NULL}};
const Segment SGI312050[] = {{&S18775[188], 9}, {NULL}};
const Segment SGI312051[] = {{&S18775[8], 3}, {NULL}};
const Segment SGI312052[] = {{&S18775[13], 10}, {NULL}};
const Segment SGI312053[] = {{&S18775[42], 7}, {NULL}};
const Segment SGI312054[] = {{&S18775[35], 14}, {NULL}};
const Segment SGI312055[] = {{&S18775[51], 6}, {NULL}};
const Segment SGI312056[] = {{&S18776[30], 4}, {NULL}};
const Segment SGI312057[] = {{&S18776[90], 9}, {NULL}};
const Segment SGI312058[] = {{&S18776[8], 3}, {NULL}};
const Segment SGI312059[] = {{&S18776[13], 10}, {NULL}};
const Segment SGI312060[] = {{&S18776[43], 7}, {NULL}};
const Segment SGI312061[] = {{&S18776[36], 14}, {NULL}};
const Segment SGI312062[] = {{&S18776[52], 6}, {NULL}};
const Segment SGI312063[] = {{&S18777[30], 3}, {NULL}};
const Segment SGI312064[] = {{&S18777[229], 9}, {NULL}};
const Segment SGI312065[] = {{&S18777[8], 3}, {NULL}};
const Segment SGI312066[] = {{&S18777[13], 10}, {NULL}};
const Segment SGI312067[] = {{&S18777[42], 7}, {NULL}};
const Segment SGI312068[] = {{&S18777[35], 14}, {NULL}};
const Segment SGI312069[] = {{&S18777[143], 15}, {NULL}};
const Segment SGI312070[] = {{&S18777[51], 5}, {NULL}};
const Segment SGI312071[] = {{&S18777[184], 9}, {NULL}};
const Segment SGI312072[] = {{&S18778[30], 3}, {NULL}};
const Segment SGI312073[] = {{&S18778[197], 9}, {&S18778[197], 9}, {NULL}};
const Segment SGI312074[] = {{&S18778[8], 3}, {NULL}};
const Segment SGI312075[] = {{&S18778[13], 10}, {NULL}};
const Segment SGI312076[] = {{&S18778[42], 7}, {NULL}};
const Segment SGI312077[] = {{&S18778[35], 14}, {NULL}};
const Segment SGI312078[] = {{&S18778[138], 15}, {NULL}};
const Segment SGI312079[] = {{&S18778[51], 6}, {NULL}};
const Segment SGI312080[] = {{&S18779[30], 3}, {NULL}};
const Segment SGI312081[] = {{&S18779[192], 9}, {NULL}};
const Segment SGI312082[] = {{&S18779[8], 3}, {NULL}};
const Segment SGI312083[] = {{&S18779[13], 10}, {NULL}};
const Segment SGI312084[] = {{&S18779[42], 7}, {NULL}};
const Segment SGI312085[] = {{&S18779[35], 14}, {NULL}};
const Segment SGI312086[] = {{&S18779[131], 15}, {NULL}};
const Segment SGI312087[] = {{&S18780[30], 3}, {NULL}};
const Segment SGI312088[] = {{&S18780[197], 9}, {NULL}};
const Segment SGI312089[] = {{&S18780[8], 3}, {NULL}};
const Segment SGI312090[] = {{&S18780[13], 10}, {NULL}};
const Segment SGI312091[] = {{&S18780[42], 7}, {NULL}};
const Segment SGI312092[] = {{&S18780[35], 14}, {NULL}};
const Segment SGI312093[] = {{&S18780[138], 15}, {NULL}};
const Segment SGI312094[] = {{&S18780[51], 6}, {NULL}};
const Segment SGI312095[] = {{&S18781[30], 3}, {NULL}};
const Segment SGI312096[] = {{&S18781[214], 9}, {NULL}};
const Segment SGI312097[] = {{&S18781[8], 3}, {NULL}};
const Segment SGI312098[] = {{&S18781[13], 10}, {NULL}};
const Segment SGI312099[] = {{&S18781[42], 7}, {NULL}};
const Segment SGI312100[] = {{&S18781[35], 14}, {NULL}};
const Segment SGI312101[] = {{&S18781[143], 15}, {NULL}};
const Segment SGI312102[] = {{&S18781[51], 5}, {NULL}};
const Segment SGI312103[] = {{&S18782[30], 3}, {NULL}};
const Segment SGI312104[] = {{&S18782[89], 9}, {NULL}};
const Segment SGI312105[] = {{&S18782[8], 3}, {NULL}};
const Segment SGI312106[] = {{&S18782[13], 10}, {NULL}};
const Segment SGI312107[] = {{&S18782[42], 7}, {NULL}};
const Segment SGI312108[] = {{&S18782[35], 14}, {NULL}};
const Segment SGI312109[] = {{&S18782[51], 6}, {NULL}};
const Segment SGI312110[] = {{&S18783[30], 3}, {NULL}};
const Segment SGI312111[] = {{&S18783[229], 9}, {&S18783[229], 9}, {NULL}};
const Segment SGI312112[] = {{&S18783[8], 3}, {NULL}};
const Segment SGI312113[] = {{&S18783[13], 10}, {NULL}};
const Segment SGI312114[] = {{&S18783[42], 7}, {NULL}};
const Segment SGI312115[] = {{&S18783[35], 14}, {NULL}};
const Segment SGI312116[] = {{&S18783[143], 15}, {NULL}};
const Segment SGI312117[] = {{&S18783[51], 5}, {NULL}};
const Segment SGI312118[] = {{&S18783[184], 9}, {NULL}};
const Segment SGI312119[] = {{&S18785[30], 3}, {NULL}};
const Segment SGI312120[] = {{&S18785[214], 9}, {&S18785[214], 9}, {NULL}};
const Segment SGI312121[] = {{&S18785[8], 3}, {NULL}};
const Segment SGI312122[] = {{&S18785[13], 10}, {NULL}};
const Segment SGI312123[] = {{&S18785[42], 7}, {NULL}};
const Segment SGI312124[] = {{&S18785[35], 14}, {NULL}};
const Segment SGI312125[] = {{&S18785[143], 15}, {NULL}};
const Segment SGI312126[] = {{&S18785[51], 5}, {NULL}};
const Segment SGI312127[] = {{&S18786[30], 3}, {NULL}};
const Segment SGI312128[] = {{&S18786[219], 9}, {NULL}};
const Segment SGI312129[] = {{&S18786[8], 3}, {NULL}};
const Segment SGI312130[] = {{&S18786[13], 10}, {NULL}};
const Segment SGI312131[] = {{&S18786[42], 7}, {NULL}};
const Segment SGI312132[] = {{&S18786[35], 14}, {NULL}};
const Segment SGI312133[] = {{&S18786[138], 15}, {NULL}};
const Segment SGI312134[] = {{&S18786[179], 4}, {NULL}};
const Segment SGI312135[] = {{&S18786[51], 6}, {NULL}};
const Segment SGI312136[] = {{&S18787[30], 3}, {NULL}};
const Segment SGI312137[] = {{&S18787[192], 9}, {&S18787[192], 9}, {NULL}};
const Segment SGI312138[] = {{&S18787[8], 3}, {NULL}};
const Segment SGI312139[] = {{&S18787[13], 10}, {NULL}};
const Segment SGI312140[] = {{&S18787[42], 7}, {NULL}};
const Segment SGI312141[] = {{&S18787[35], 14}, {NULL}};
const Segment SGI312142[] = {{&S18787[131], 15}, {NULL}};
const Segment SGI312143[] = {{&S18788[30], 4}, {NULL}};
const Segment SGI312144[] = {{&S18788[90], 9}, {NULL}};
const Segment SGI312145[] = {{&S18788[8], 3}, {NULL}};
const Segment SGI312146[] = {{&S18788[13], 10}, {NULL}};
const Segment SGI312147[] = {{&S18788[43], 7}, {NULL}};
const Segment SGI312148[] = {{&S18788[36], 14}, {NULL}};
const Segment SGI312149[] = {{&S18788[52], 6}, {NULL}};
const Segment SGI312150[] = {{&S18789[30], 3}, {NULL}};
const Segment SGI312151[] = {{&S18789[195], 9}, {NULL}};
const Segment SGI312152[] = {{&S18789[8], 3}, {NULL}};
const Segment SGI312153[] = {{&S18789[13], 10}, {NULL}};
const Segment SGI312154[] = {{&S18789[42], 7}, {NULL}};
const Segment SGI312155[] = {{&S18789[35], 14}, {NULL}};
const Segment SGI312156[] = {{&S18789[131], 15}, {NULL}};
const Segment SGI312157[] = {{&S18789[162], 9}, {NULL}};
const Segment SGI312158[] = {{&S18790[30], 3}, {NULL}};
const Segment SGI312159[] = {{&S18790[214], 9}, {&S18790[214], 9}, {NULL}};
const Segment SGI312160[] = {{&S18790[8], 3}, {NULL}};
const Segment SGI312161[] = {{&S18790[13], 10}, {NULL}};
const Segment SGI312162[] = {{&S18790[42], 7}, {NULL}};
const Segment SGI312163[] = {{&S18790[35], 14}, {NULL}};
const Segment SGI312164[] = {{&S18790[143], 15}, {NULL}};
const Segment SGI312165[] = {{&S18790[51], 5}, {NULL}};
const Segment SGI312166[] = {{&S18791[30], 3}, {NULL}};
const Segment SGI312167[] = {{&S18791[209], 9}, {&S18791[209], 9}, {&S18791[209], 9}, {NULL}};
const Segment SGI312168[] = {{&S18791[8], 3}, {NULL}};
const Segment SGI312169[] = {{&S18791[13], 10}, {NULL}};
const Segment SGI312170[] = {{&S18791[42], 7}, {NULL}};
const Segment SGI312171[] = {{&S18791[35], 14}, {NULL}};
const Segment SGI312172[] = {{&S18791[138], 15}, {NULL}};
const Segment SGI312173[] = {{&S18791[51], 6}, {NULL}};
const Segment SGI312174[] = {{&S18792[30], 3}, {NULL}};
const Segment SGI312175[] = {{&S18792[197], 9}, {NULL}};
const Segment SGI312176[] = {{&S18792[8], 3}, {NULL}};
const Segment SGI312177[] = {{&S18792[13], 10}, {NULL}};
const Segment SGI312178[] = {{&S18792[42], 7}, {NULL}};
const Segment SGI312179[] = {{&S18792[35], 14}, {NULL}};
const Segment SGI312180[] = {{&S18792[138], 15}, {NULL}};
const Segment SGI312181[] = {{&S18792[51], 6}, {NULL}};
const Segment SGI312182[] = {{&S18793[30], 3}, {NULL}};
const Segment SGI312183[] = {{&S18793[176], 9}, {NULL}};
const Segment SGI312184[] = {{&S18793[8], 3}, {NULL}};
const Segment SGI312185[] = {{&S18793[13], 10}, {NULL}};
const Segment SGI312186[] = {{&S18793[42], 7}, {NULL}};
const Segment SGI312187[] = {{&S18793[35], 14}, {NULL}};
const Segment SGI312188[] = {{&S18793[51], 6}, {NULL}};
const Segment SGI312189[] = {{&S18794[30], 3}, {NULL}};
const Segment SGI312190[] = {{&S18794[188], 9}, {&S18794[188], 9}, {NULL}};
const Segment SGI312191[] = {{&S18794[8], 3}, {NULL}};
const Segment SGI312192[] = {{&S18794[13], 10}, {NULL}};
const Segment SGI312193[] = {{&S18794[42], 7}, {NULL}};
const Segment SGI312194[] = {{&S18794[35], 14}, {NULL}};
const Segment SGI312195[] = {{&S18794[51], 6}, {NULL}};
const Segment SGI312196[] = {{&S18795[30], 3}, {NULL}};
const Segment SGI312197[] = {{&S18795[207], 9}, {&S18795[207], 9}, {NULL}};
const Segment SGI312198[] = {{&S18795[8], 3}, {NULL}};
const Segment SGI312199[] = {{&S18795[13], 10}, {NULL}};
const Segment SGI312200[] = {{&S18795[42], 7}, {NULL}};
const Segment SGI312201[] = {{&S18795[35], 14}, {NULL}};
const Segment SGI312202[] = {{&S18795[131], 15}, {NULL}};
const Segment SGI312203[] = {{&S18795[162], 9}, {NULL}};
const Segment SGI312204[] = {{&S18796[30], 3}, {NULL}};
const Segment SGI312205[] = {{&S18796[193], 9}, {NULL}};
const Segment SGI312206[] = {{&S18796[8], 3}, {NULL}};
const Segment SGI312207[] = {{&S18796[13], 10}, {NULL}};
const Segment SGI312208[] = {{&S18796[42], 7}, {NULL}};
const Segment SGI312209[] = {{&S18796[35], 14}, {NULL}};
const Segment SGI312210[] = {{&S18796[51], 5}, {NULL}};
const Segment SGI312211[] = {{&S18797[30], 4}, {NULL}};
const Segment SGI312212[] = {{&S18797[90], 9}, {&S18797[90], 9}, {NULL}};
const Segment SGI312213[] = {{&S18797[8], 3}, {NULL}};
const Segment SGI312214[] = {{&S18797[13], 10}, {NULL}};
const Segment SGI312215[] = {{&S18797[43], 7}, {NULL}};
const Segment SGI312216[] = {{&S18797[36], 14}, {NULL}};
const Segment SGI312217[] = {{&S18797[52], 6}, {NULL}};
const Segment SGI312218[] = {{&S18798[30], 3}, {NULL}};
const Segment SGI312219[] = {{&S18798[219], 9}, {&S18798[219], 9}, {NULL}};
const Segment SGI312220[] = {{&S18798[8], 3}, {NULL}};
const Segment SGI312221[] = {{&S18798[13], 10}, {NULL}};
const Segment SGI312222[] = {{&S18798[42], 7}, {NULL}};
const Segment SGI312223[] = {{&S18798[35], 14}, {NULL}};
const Segment SGI312224[] = {{&S18798[138], 15}, {NULL}};
const Segment SGI312225[] = {{&S18798[179], 4}, {NULL}};
const Segment SGI312226[] = {{&S18798[51], 6}, {NULL}};
const Segment SGI312227[] = {{&S18799[30], 3}, {NULL}};
const Segment SGI312228[] = {{&S18799[214], 9}, {NULL}};
const Segment SGI312229[] = {{&S18799[8], 3}, {NULL}};
const Segment SGI312230[] = {{&S18799[13], 10}, {NULL}};
const Segment SGI312231[] = {{&S18799[42], 7}, {NULL}};
const Segment SGI312232[] = {{&S18799[35], 14}, {NULL}};
const Segment SGI312233[] = {{&S18799[143], 15}, {NULL}};
const Segment SGI312234[] = {{&S18799[51], 5}, {NULL}};
const Segment SGI312235[] = {{&S18800[30], 3}, {NULL}};
const Segment SGI312236[] = {{&S18800[229], 9}, {NULL}};
const Segment SGI312237[] = {{&S18800[8], 3}, {NULL}};
const Segment SGI312238[] = {{&S18800[13], 10}, {NULL}};
const Segment SGI312239[] = {{&S18800[42], 7}, {NULL}};
const Segment SGI312240[] = {{&S18800[35], 14}, {NULL}};
const Segment SGI312241[] = {{&S18800[143], 15}, {NULL}};
const Segment SGI312242[] = {{&S18800[51], 5}, {NULL}};
const Segment SGI312243[] = {{&S18800[184], 9}, {NULL}};
const Segment SGI312244[] = {{&S18802[30], 3}, {NULL}};
const Segment SGI312245[] = {{&S18802[192], 9}, {NULL}};
const Segment SGI312246[] = {{&S18802[8], 3}, {NULL}};
const Segment SGI312247[] = {{&S18802[13], 10}, {NULL}};
const Segment SGI312248[] = {{&S18802[42], 7}, {NULL}};
const Segment SGI312249[] = {{&S18802[35], 14}, {NULL}};
const Segment SGI312250[] = {{&S18802[131], 15}, {NULL}};
const Segment SGI312251[] = {{&S18803[30], 3}, {NULL}};
const Segment SGI312252[] = {{&S18803[209], 9}, {&S18803[209], 9}, {NULL}};
const Segment SGI312253[] = {{&S18803[8], 3}, {NULL}};
const Segment SGI312254[] = {{&S18803[13], 10}, {NULL}};
const Segment SGI312255[] = {{&S18803[42], 7}, {NULL}};
const Segment SGI312256[] = {{&S18803[35], 14}, {NULL}};
const Segment SGI312257[] = {{&S18803[138], 15}, {NULL}};
const Segment SGI312258[] = {{&S18803[51], 6}, {NULL}};
const Segment SGI312259[] = {{&S18804[30], 3}, {NULL}};
const Segment SGI312260[] = {{&S18804[197], 9}, {NULL}};
const Segment SGI312261[] = {{&S18804[8], 3}, {NULL}};
const Segment SGI312262[] = {{&S18804[13], 10}, {NULL}};
const Segment SGI312263[] = {{&S18804[42], 7}, {NULL}};
const Segment SGI312264[] = {{&S18804[35], 14}, {NULL}};
const Segment SGI312265[] = {{&S18804[138], 15}, {NULL}};
const Segment SGI312266[] = {{&S18804[51], 6}, {NULL}};
const Segment SGI312267[] = {{&S18805[30], 3}, {NULL}};
const Segment SGI312268[] = {{&S18805[89], 9}, {&S18805[89], 9}, {NULL}};
const Segment SGI312269[] = {{&S18805[8], 3}, {NULL}};
const Segment SGI312270[] = {{&S18805[13], 10}, {NULL}};
const Segment SGI312271[] = {{&S18805[42], 7}, {NULL}};
const Segment SGI312272[] = {{&S18805[35], 14}, {NULL}};
const Segment SGI312273[] = {{&S18805[51], 6}, {NULL}};
const Segment SGI312274[] = {{&S18806[30], 3}, {NULL}};
const Segment SGI312275[] = {{&S18806[197], 9}, {&S18806[197], 9}, {&S18806[197], 9}, {NULL}};
const Segment SGI312276[] = {{&S18806[8], 3}, {NULL}};
const Segment SGI312277[] = {{&S18806[13], 10}, {NULL}};
const Segment SGI312278[] = {{&S18806[42], 7}, {NULL}};
const Segment SGI312279[] = {{&S18806[35], 14}, {NULL}};
const Segment SGI312280[] = {{&S18806[138], 15}, {NULL}};
const Segment SGI312281[] = {{&S18806[51], 6}, {NULL}};
const Segment SGI312282[] = {{&S18807[30], 3}, {NULL}};
const Segment SGI312283[] = {{&S18807[176], 9}, {NULL}};
const Segment SGI312284[] = {{&S18807[8], 3}, {NULL}};
const Segment SGI312285[] = {{&S18807[13], 10}, {NULL}};
const Segment SGI312286[] = {{&S18807[42], 7}, {NULL}};
const Segment SGI312287[] = {{&S18807[35], 14}, {NULL}};
const Segment SGI312288[] = {{&S18807[51], 6}, {NULL}};
const Segment SGI312289[] = {{&S18808[30], 3}, {NULL}};
const Segment SGI312290[] = {{&S18808[207], 9}, {NULL}};
const Segment SGI312291[] = {{&S18808[8], 3}, {NULL}};
const Segment SGI312292[] = {{&S18808[13], 10}, {NULL}};
const Segment SGI312293[] = {{&S18808[42], 7}, {NULL}};
const Segment SGI312294[] = {{&S18808[35], 14}, {NULL}};
const Segment SGI312295[] = {{&S18808[131], 15}, {NULL}};
const Segment SGI312296[] = {{&S18808[162], 9}, {NULL}};
const Segment SGI312297[] = {{&S18809[30], 4}, {NULL}};
const Segment SGI312298[] = {{&S18809[90], 9}, {NULL}};
const Segment SGI312299[] = {{&S18809[8], 3}, {NULL}};
const Segment SGI312300[] = {{&S18809[13], 10}, {NULL}};
const Segment SGI312301[] = {{&S18809[43], 7}, {NULL}};
const Segment SGI312302[] = {{&S18809[36], 14}, {NULL}};
const Segment SGI312303[] = {{&S18809[52], 6}, {NULL}};
const Segment SGI312304[] = {{&S18810[30], 3}, {NULL}};
const Segment SGI312305[] = {{&S18810[214], 9}, {&S18810[214], 9}, {NULL}};
const Segment SGI312306[] = {{&S18810[8], 3}, {NULL}};
const Segment SGI312307[] = {{&S18810[13], 10}, {NULL}};
const Segment SGI312308[] = {{&S18810[42], 7}, {NULL}};
const Segment SGI312309[] = {{&S18810[35], 14}, {NULL}};
const Segment SGI312310[] = {{&S18810[143], 15}, {NULL}};
const Segment SGI312311[] = {{&S18810[51], 5}, {NULL}};
const Segment SGI312312[] = {{&S18811[30], 3}, {NULL}};
const Segment SGI312313[] = {{&S18811[229], 9}, {&S18811[229], 9}, {NULL}};
const Segment SGI312314[] = {{&S18811[8], 3}, {NULL}};
const Segment SGI312315[] = {{&S18811[13], 10}, {NULL}};
const Segment SGI312316[] = {{&S18811[42], 7}, {NULL}};
const Segment SGI312317[] = {{&S18811[35], 14}, {NULL}};
const Segment SGI312318[] = {{&S18811[143], 15}, {NULL}};
const Segment SGI312319[] = {{&S18811[51], 5}, {NULL}};
const Segment SGI312320[] = {{&S18811[184], 9}, {NULL}};
const Segment SGI312321[] = {{&S18812[30], 3}, {NULL}};
const Segment SGI312322[] = {{&S18812[193], 9}, {&S18812[193], 9}, {NULL}};
const Segment SGI312323[] = {{&S18812[8], 3}, {NULL}};
const Segment SGI312324[] = {{&S18812[13], 10}, {NULL}};
const Segment SGI312325[] = {{&S18812[42], 7}, {NULL}};
const Segment SGI312326[] = {{&S18812[35], 14}, {NULL}};
const Segment SGI312327[] = {{&S18812[51], 5}, {NULL}};
const Segment SGI312328[] = {{&S18813[30], 3}, {NULL}};
const Segment SGI312329[] = {{&S18813[89], 9}, {&S18813[89], 9}, {NULL}};
const Segment SGI312330[] = {{&S18813[8], 3}, {NULL}};
const Segment SGI312331[] = {{&S18813[13], 10}, {NULL}};
const Segment SGI312332[] = {{&S18813[42], 7}, {NULL}};
const Segment SGI312333[] = {{&S18813[35], 14}, {NULL}};
const Segment SGI312334[] = {{&S18813[51], 6}, {NULL}};
const Segment SGI312335[] = {{&S18814[30], 3}, {NULL}};
const Segment SGI312336[] = {{&S18814[239], 9}, {NULL}};
const Segment SGI312337[] = {{&S18814[8], 3}, {NULL}};
const Segment SGI312338[] = {{&S18814[13], 10}, {NULL}};
const Segment SGI312339[] = {{&S18814[42], 7}, {NULL}};
const Segment SGI312340[] = {{&S18814[35], 14}, {NULL}};
const Segment SGI312341[] = {{&S18814[143], 15}, {NULL}};
const Segment SGI312342[] = {{&S18814[199], 4}, {NULL}};
const Segment SGI312343[] = {{&S18814[51], 5}, {NULL}};
const Segment SGI312344[] = {{&S18814[184], 9}, {NULL}};
const Segment SGI312345[] = {{&S18815[30], 3}, {NULL}};
const Segment SGI312346[] = {{&S18815[197], 9}, {&S18815[197], 9}, {NULL}};
const Segment SGI312347[] = {{&S18815[8], 3}, {NULL}};
const Segment SGI312348[] = {{&S18815[13], 10}, {NULL}};
const Segment SGI312349[] = {{&S18815[42], 7}, {NULL}};
const Segment SGI312350[] = {{&S18815[35], 14}, {NULL}};
const Segment SGI312351[] = {{&S18815[138], 15}, {NULL}};
const Segment SGI312352[] = {{&S18815[51], 6}, {NULL}};
const Segment SGI312353[] = {{&S18816[30], 3}, {NULL}};
const Segment SGI312354[] = {{&S18816[202], 9}, {&S18816[202], 9}, {NULL}};
const Segment SGI312355[] = {{&S18816[8], 3}, {NULL}};
const Segment SGI312356[] = {{&S18816[13], 10}, {NULL}};
const Segment SGI312357[] = {{&S18816[42], 7}, {NULL}};
const Segment SGI312358[] = {{&S18816[35], 14}, {NULL}};
const Segment SGI312359[] = {{&S18816[131], 15}, {NULL}};
const Segment SGI312360[] = {{&S18817[30], 3}, {NULL}};
const Segment SGI312361[] = {{&S18817[209], 9}, {&S18817[209], 9}, {&S18817[209], 9}, {NULL}};
const Segment SGI312362[] = {{&S18817[8], 3}, {NULL}};
const Segment SGI312363[] = {{&S18817[13], 10}, {NULL}};
const Segment SGI312364[] = {{&S18817[42], 7}, {NULL}};
const Segment SGI312365[] = {{&S18817[35], 14}, {NULL}};
const Segment SGI312366[] = {{&S18817[138], 15}, {NULL}};
const Segment SGI312367[] = {{&S18817[51], 6}, {NULL}};
const Segment SGI312368[] = {{&S18818[30], 3}, {NULL}};
const Segment SGI312369[] = {{&S18818[219], 9}, {NULL}};
const Segment SGI312370[] = {{&S18818[8], 3}, {NULL}};
const Segment SGI312371[] = {{&S18818[13], 10}, {NULL}};
const Segment SGI312372[] = {{&S18818[42], 7}, {NULL}};
const Segment SGI312373[] = {{&S18818[35], 14}, {NULL}};
const Segment SGI312374[] = {{&S18818[138], 15}, {NULL}};
const Segment SGI312375[] = {{&S18818[179], 4}, {NULL}};
const Segment SGI312376[] = {{&S18818[51], 6}, {NULL}};
const Segment SGI312377[] = {{&S18819[30], 3}, {NULL}};
const Segment SGI312378[] = {{&S18819[188], 9}, {&S18819[188], 9}, {NULL}};
const Segment SGI312379[] = {{&S18819[8], 3}, {NULL}};
const Segment SGI312380[] = {{&S18819[13], 10}, {NULL}};
const Segment SGI312381[] = {{&S18819[42], 7}, {NULL}};
const Segment SGI312382[] = {{&S18819[35], 14}, {NULL}};
const Segment SGI312383[] = {{&S18819[51], 6}, {NULL}};
const Segment SGI312384[] = {{&S18820[30], 3}, {NULL}};
const Segment SGI312385[] = {{&S18820[181], 9}, {NULL}};
const Segment SGI312386[] = {{&S18820[8], 3}, {NULL}};
const Segment SGI312387[] = {{&S18820[13], 10}, {NULL}};
const Segment SGI312388[] = {{&S18820[42], 7}, {NULL}};
const Segment SGI312389[] = {{&S18820[35], 14}, {NULL}};
const Segment SGI312390[] = {{&S18820[51], 5}, {NULL}};
const Segment SGI312391[] = {{&S18821[30], 4}, {NULL}};
const Segment SGI312392[] = {{&S18821[90], 9}, {&S18821[90], 9}, {NULL}};
const Segment SGI312393[] = {{&S18821[8], 3}, {NULL}};
const Segment SGI312394[] = {{&S18821[13], 10}, {NULL}};
const Segment SGI312395[] = {{&S18821[43], 7}, {NULL}};
const Segment SGI312396[] = {{&S18821[36], 14}, {NULL}};
const Segment SGI312397[] = {{&S18821[52], 6}, {NULL}};
const Segment SGI312398[] = {{&S18822[30], 3}, {NULL}};
const Segment SGI312399[] = {{&S18822[207], 9}, {&S18822[207], 9}, {NULL}};
const Segment SGI312400[] = {{&S18822[8], 3}, {NULL}};
const Segment SGI312401[] = {{&S18822[13], 10}, {NULL}};
const Segment SGI312402[] = {{&S18822[42], 7}, {NULL}};
const Segment SGI312403[] = {{&S18822[35], 14}, {NULL}};
const Segment SGI312404[] = {{&S18822[131], 15}, {NULL}};
const Segment SGI312405[] = {{&S18822[162], 9}, {NULL}};
const Segment SGI312406[] = {{&S18823[30], 4}, {NULL}};
const Segment SGI312407[] = {{&S18823[83], 9}, {NULL}};
const Segment SGI312408[] = {{&S18823[8], 3}, {NULL}};
const Segment SGI312409[] = {{&S18823[13], 10}, {NULL}};
const Segment SGI312410[] = {{&S18823[43], 7}, {NULL}};
const Segment SGI312411[] = {{&S18823[36], 14}, {NULL}};
const Segment SGI312412[] = {{&S18824[30], 3}, {NULL}};
const Segment SGI312413[] = {{&S18824[219], 9}, {&S18824[219], 9}, {NULL}};
const Segment SGI312414[] = {{&S18824[8], 3}, {NULL}};
const Segment SGI312415[] = {{&S18824[13], 10}, {NULL}};
const Segment SGI312416[] = {{&S18824[42], 7}, {NULL}};
const Segment SGI312417[] = {{&S18824[35], 14}, {NULL}};
const Segment SGI312418[] = {{&S18824[138], 15}, {NULL}};
const Segment SGI312419[] = {{&S18824[179], 4}, {NULL}};
const Segment SGI312420[] = {{&S18824[51], 6}, {NULL}};
const Segment SGI312421[] = {{&S18825[30], 4}, {NULL}};
const Segment SGI312422[] = {{&S18825[95], 9}, {&S18825[95], 9}, {NULL}};
const Segment SGI312423[] = {{&S18825[8], 3}, {NULL}};
const Segment SGI312424[] = {{&S18825[13], 10}, {NULL}};
const Segment SGI312425[] = {{&S18825[43], 7}, {NULL}};
const Segment SGI312426[] = {{&S18825[36], 14}, {NULL}};
const Segment SGI312427[] = {{&S18825[52], 5}, {NULL}};
const Segment SGI312428[] = {{&S18826[30], 4}, {NULL}};
const Segment SGI312429[] = {{&S18826[198], 9}, {NULL}};
const Segment SGI312430[] = {{&S18826[8], 3}, {NULL}};
const Segment SGI312431[] = {{&S18826[13], 10}, {NULL}};
const Segment SGI312432[] = {{&S18826[43], 7}, {NULL}};
const Segment SGI312433[] = {{&S18826[36], 14}, {NULL}};
const Segment SGI312434[] = {{&S18826[139], 15}, {NULL}};
const Segment SGI312435[] = {{&S18826[52], 6}, {NULL}};
const Segment SGI312436[] = {{&S18827[30], 3}, {NULL}};
const Segment SGI312437[] = {{&S18827[89], 9}, {&S18827[89], 9}, {NULL}};
const Segment SGI312438[] = {{&S18827[8], 3}, {NULL}};
const Segment SGI312439[] = {{&S18827[13], 10}, {NULL}};
const Segment SGI312440[] = {{&S18827[42], 7}, {NULL}};
const Segment SGI312441[] = {{&S18827[35], 14}, {NULL}};
const Segment SGI312442[] = {{&S18827[51], 6}, {NULL}};
const Segment SGI312443[] = {{&S18828[30], 4}, {NULL}};
const Segment SGI312444[] = {{&S18828[95], 9}, {NULL}};
const Segment SGI312445[] = {{&S18828[8], 3}, {NULL}};
const Segment SGI312446[] = {{&S18828[13], 10}, {NULL}};
const Segment SGI312447[] = {{&S18828[43], 7}, {NULL}};
const Segment SGI312448[] = {{&S18828[36], 14}, {NULL}};
const Segment SGI312449[] = {{&S18828[52], 5}, {NULL}};
const Segment SGI312450[] = {{&S18829[30], 3}, {NULL}};
const Segment SGI312451[] = {{&S18829[197], 9}, {NULL}};
const Segment SGI312452[] = {{&S18829[8], 3}, {NULL}};
const Segment SGI312453[] = {{&S18829[13], 10}, {NULL}};
const Segment SGI312454[] = {{&S18829[42], 7}, {NULL}};
const Segment SGI312455[] = {{&S18829[35], 14}, {NULL}};
const Segment SGI312456[] = {{&S18829[138], 15}, {NULL}};
const Segment SGI312457[] = {{&S18829[51], 6}, {NULL}};
const Segment SGI312458[] = {{&S18830[30], 3}, {NULL}};
const Segment SGI312459[] = {{&S18830[214], 9}, {&S18830[214], 9}, {&S18830[214], 9}, {NULL}};
const Segment SGI312460[] = {{&S18830[8], 3}, {NULL}};
const Segment SGI312461[] = {{&S18830[13], 10}, {NULL}};
const Segment SGI312462[] = {{&S18830[42], 7}, {NULL}};
const Segment SGI312463[] = {{&S18830[35], 14}, {NULL}};
const Segment SGI312464[] = {{&S18830[143], 15}, {NULL}};
const Segment SGI312465[] = {{&S18830[51], 5}, {NULL}};
const Segment SGI312466[] = {{&S18832[30], 3}, {NULL}};
const Segment SGI312467[] = {{&S18832[190], 9}, {NULL}};
const Segment SGI312468[] = {{&S18832[8], 3}, {NULL}};
const Segment SGI312469[] = {{&S18832[13], 10}, {NULL}};
const Segment SGI312470[] = {{&S18832[42], 7}, {NULL}};
const Segment SGI312471[] = {{&S18832[35], 14}, {NULL}};
const Segment SGI312472[] = {{&S18832[131], 15}, {NULL}};
const Segment SGI312473[] = {{&S18833[30], 3}, {NULL}};
const Segment SGI312474[] = {{&S18833[214], 9}, {NULL}};
const Segment SGI312475[] = {{&S18833[8], 3}, {NULL}};
const Segment SGI312476[] = {{&S18833[13], 10}, {NULL}};
const Segment SGI312477[] = {{&S18833[42], 7}, {NULL}};
const Segment SGI312478[] = {{&S18833[35], 14}, {NULL}};
const Segment SGI312479[] = {{&S18833[143], 15}, {NULL}};
const Segment SGI312480[] = {{&S18833[51], 5}, {NULL}};
const Segment SGI312481[] = {{&S18834[30], 3}, {NULL}};
const Segment SGI312482[] = {{&S18834[197], 9}, {&S18834[197], 9}, {NULL}};
const Segment SGI312483[] = {{&S18834[8], 3}, {NULL}};
const Segment SGI312484[] = {{&S18834[13], 10}, {NULL}};
const Segment SGI312485[] = {{&S18834[42], 7}, {NULL}};
const Segment SGI312486[] = {{&S18834[35], 14}, {NULL}};
const Segment SGI312487[] = {{&S18834[138], 15}, {NULL}};
const Segment SGI312488[] = {{&S18834[51], 6}, {NULL}};
const Segment SGI312489[] = {{&S18835[30], 3}, {NULL}};
const Segment SGI312490[] = {{&S18835[224], 9}, {NULL}};
const Segment SGI312491[] = {{&S18835[8], 3}, {NULL}};
const Segment SGI312492[] = {{&S18835[13], 10}, {NULL}};
const Segment SGI312493[] = {{&S18835[42], 7}, {NULL}};
const Segment SGI312494[] = {{&S18835[35], 14}, {NULL}};
const Segment SGI312495[] = {{&S18835[143], 15}, {NULL}};
const Segment SGI312496[] = {{&S18835[184], 4}, {NULL}};
const Segment SGI312497[] = {{&S18835[51], 5}, {NULL}};
const Segment SGI312498[] = {{&S18836[30], 3}, {NULL}};
const Segment SGI312499[] = {{&S18836[239], 9}, {NULL}};
const Segment SGI312500[] = {{&S18836[8], 3}, {NULL}};
const Segment SGI312501[] = {{&S18836[13], 10}, {NULL}};
const Segment SGI312502[] = {{&S18836[42], 7}, {NULL}};
const Segment SGI312503[] = {{&S18836[35], 14}, {NULL}};
const Segment SGI312504[] = {{&S18836[143], 15}, {NULL}};
const Segment SGI312505[] = {{&S18836[199], 4}, {NULL}};
const Segment SGI312506[] = {{&S18836[51], 5}, {NULL}};
const Segment SGI312507[] = {{&S18836[184], 9}, {NULL}};
const Segment SGI312508[] = {{&S18837[30], 4}, {NULL}};
const Segment SGI312509[] = {{&S18837[83], 9}, {&S18837[83], 9}, {NULL}};
const Segment SGI312510[] = {{&S18837[8], 3}, {NULL}};
const Segment SGI312511[] = {{&S18837[13], 10}, {NULL}};
const Segment SGI312512[] = {{&S18837[43], 7}, {NULL}};
const Segment SGI312513[] = {{&S18837[36], 14}, {NULL}};
const Segment SGI312514[] = {{&S18838[30], 3}, {NULL}};
const Segment SGI312515[] = {{&S18838[229], 9}, {NULL}};
const Segment SGI312516[] = {{&S18838[8], 3}, {NULL}};
const Segment SGI312517[] = {{&S18838[13], 10}, {NULL}};
const Segment SGI312518[] = {{&S18838[42], 7}, {NULL}};
const Segment SGI312519[] = {{&S18838[35], 14}, {NULL}};
const Segment SGI312520[] = {{&S18838[143], 15}, {NULL}};
const Segment SGI312521[] = {{&S18838[51], 5}, {NULL}};
const Segment SGI312522[] = {{&S18838[184], 9}, {NULL}};
const Segment SGI312523[] = {{&S18839[30], 3}, {NULL}};
const Segment SGI312524[] = {{&S18839[207], 9}, {NULL}};
const Segment SGI312525[] = {{&S18839[8], 3}, {NULL}};
const Segment SGI312526[] = {{&S18839[13], 10}, {NULL}};
const Segment SGI312527[] = {{&S18839[42], 7}, {NULL}};
const Segment SGI312528[] = {{&S18839[35], 14}, {NULL}};
const Segment SGI312529[] = {{&S18839[131], 15}, {NULL}};
const Segment SGI312530[] = {{&S18839[162], 9}, {NULL}};
const Segment SGI312531[] = {{&S18840[30], 3}, {NULL}};
const Segment SGI312532[] = {{&S18840[214], 9}, {&S18840[214], 9}, {NULL}};
const Segment SGI312533[] = {{&S18840[8], 3}, {NULL}};
const Segment SGI312534[] = {{&S18840[13], 10}, {NULL}};
const Segment SGI312535[] = {{&S18840[42], 7}, {NULL}};
const Segment SGI312536[] = {{&S18840[35], 14}, {NULL}};
const Segment SGI312537[] = {{&S18840[143], 15}, {NULL}};
const Segment SGI312538[] = {{&S18840[51], 5}, {NULL}};
const Segment SGI312539[] = {{&S18841[30], 3}, {NULL}};
const Segment SGI312540[] = {{&S18841[224], 9}, {NULL}};
const Segment SGI312541[] = {{&S18841[8], 3}, {NULL}};
const Segment SGI312542[] = {{&S18841[13], 10}, {NULL}};
const Segment SGI312543[] = {{&S18841[42], 7}, {NULL}};
const Segment SGI312544[] = {{&S18841[35], 14}, {NULL}};
const Segment SGI312545[] = {{&S18841[143], 15}, {NULL}};
const Segment SGI312546[] = {{&S18841[184], 4}, {NULL}};
const Segment SGI312547[] = {{&S18841[51], 5}, {NULL}};
const Segment SGI312548[] = {{&S18842[30], 3}, {NULL}};
const Segment SGI312549[] = {{&S18842[209], 9}, {&S18842[209], 9}, {&S18842[209], 9}, {&S18842[209], 9}, {NULL}};
const Segment SGI312550[] = {{&S18842[8], 3}, {NULL}};
const Segment SGI312551[] = {{&S18842[13], 10}, {NULL}};
const Segment SGI312552[] = {{&S18842[42], 7}, {NULL}};
const Segment SGI312553[] = {{&S18842[35], 14}, {NULL}};
const Segment SGI312554[] = {{&S18842[138], 15}, {NULL}};
const Segment SGI312555[] = {{&S18842[51], 6}, {NULL}};
const Segment SGI312556[] = {{&S18843[30], 4}, {NULL}};
const Segment SGI312557[] = {{&S18843[90], 9}, {NULL}};
const Segment SGI312558[] = {{&S18843[8], 3}, {NULL}};
const Segment SGI312559[] = {{&S18843[13], 10}, {NULL}};
const Segment SGI312560[] = {{&S18843[43], 7}, {NULL}};
const Segment SGI312561[] = {{&S18843[36], 14}, {NULL}};
const Segment SGI312562[] = {{&S18843[52], 6}, {NULL}};
const Segment SGI312563[] = {{&S18844[30], 3}, {NULL}};
const Segment SGI312564[] = {{&S18844[229], 9}, {&S18844[229], 9}, {NULL}};
const Segment SGI312565[] = {{&S18844[8], 3}, {NULL}};
const Segment SGI312566[] = {{&S18844[13], 10}, {NULL}};
const Segment SGI312567[] = {{&S18844[42], 7}, {NULL}};
const Segment SGI312568[] = {{&S18844[35], 14}, {NULL}};
const Segment SGI312569[] = {{&S18844[143], 15}, {NULL}};
const Segment SGI312570[] = {{&S18844[51], 5}, {NULL}};
const Segment SGI312571[] = {{&S18844[184], 9}, {NULL}};
const Segment SGI312572[] = {{&S18845[30], 3}, {NULL}};
const Segment SGI312573[] = {{&S18845[202], 9}, {&S18845[202], 9}, {NULL}};
const Segment SGI312574[] = {{&S18845[8], 3}, {NULL}};
const Segment SGI312575[] = {{&S18845[13], 10}, {NULL}};
const Segment SGI312576[] = {{&S18845[42], 7}, {NULL}};
const Segment SGI312577[] = {{&S18845[35], 14}, {NULL}};
const Segment SGI312578[] = {{&S18845[131], 15}, {NULL}};
const Segment SGI312579[] = {{&S19138[30], 3}, {NULL}};
const Segment SGI312580[] = {{&S19138[8], 3}, {NULL}};
const Segment SGI312581[] = {{&S19138[13], 10}, {NULL}};
const Segment SGI312582[] = {{&S19138[42], 3}, {NULL}};
const Segment SGI312583[] = {{&S19140[30], 3}, {NULL}};
const Segment SGI312584[] = {{&S19140[8], 3}, {NULL}};
const Segment SGI312585[] = {{&S19140[13], 10}, {NULL}};
const Segment SGI312586[] = {{&S19140[42], 7}, {NULL}};
const Segment SGI312587[] = {{&S19142[46], 1}, {NULL}};
const Segment SGI312588[] = {{&S19142[8], 3}, {NULL}};
const Segment SGI312589[] = {{&S19142[13], 10}, {NULL}};
const Segment SGI312590[] = {{&S19142[32], 7}, {NULL}};
const Segment SGI312591[] = {{&S19145[8], 3}, {NULL}};
const Segment SGI312592[] = {{&S19145[51], 3}, {NULL}};
const Segment SGI312593[] = {{&S19149[18], 1}, {NULL}};
const Segment SGI312594[] = {{&S19149[36], 3}, {NULL}};
const Segment SGI312595[] = {{&S19157[8], 3}, {NULL}};
const Segment SGI312596[] = {{&S19344[30], 3}, {NULL}};
const Segment SGI312597[] = {{&S19344[8], 3}, {NULL}};
const Segment SGI312598[] = {{&S19344[13], 10}, {NULL}};
const Segment SGI312599[] = {{&S19344[42], 7}, {NULL}};
const Segment SGI312600[] = {{&S19344[35], 14}, {NULL}};
const Segment SGI312601[] = {{&S19344[71], 11}, {NULL}};
const Segment SGI312602[] = {{&S19344[51], 6}, {NULL}};
const Segment SGI312603[] = {{&S19344[71], 16}, {NULL}};
const Segment SGI312604[] = {{&S19346[30], 3}, {NULL}};
const Segment SGI312605[] = {{&S19346[8], 3}, {NULL}};
const Segment SGI312606[] = {{&S19346[13], 10}, {NULL}};
const Segment SGI312607[] = {{&S19346[42], 7}, {NULL}};
const Segment SGI312608[] = {{&S19346[35], 14}, {NULL}};
const Segment SGI312609[] = {{&S19346[51], 11}, {NULL}};
const Segment SGI312610[] = {{&S19346[115], 15}, {NULL}};
const Segment SGI312611[] = {{&S19346[51], 16}, {NULL}};
const Segment SGI312612[] = {{&S19350[30], 3}, {NULL}};
const Segment SGI312613[] = {{&S19350[8], 3}, {NULL}};
const Segment SGI312614[] = {{&S19350[13], 10}, {NULL}};
const Segment SGI312615[] = {{&S19350[42], 7}, {NULL}};
const Segment SGI312616[] = {{&S19350[35], 14}, {NULL}};
const Segment SGI312617[] = {{&S19350[64], 11}, {NULL}};
const Segment SGI312618[] = {{&S19350[128], 15}, {NULL}};
const Segment SGI312619[] = {{&S19350[64], 16}, {NULL}};
const Segment SGI312620[] = {{&S19355[30], 3}, {NULL}};
const Segment SGI312621[] = {{&S19355[8], 3}, {NULL}};
const Segment SGI312622[] = {{&S19355[13], 10}, {NULL}};
const Segment SGI312623[] = {{&S19355[42], 7}, {NULL}};
const Segment SGI312624[] = {{&S19355[35], 14}, {NULL}};
const Segment SGI312625[] = {{&S19355[64], 11}, {NULL}};
const Segment SGI312626[] = {{&S19355[64], 16}, {NULL}};
const Segment SGI312627[] = {{&S19356[30], 3}, {NULL}};
const Segment SGI312628[] = {{&S19356[8], 3}, {NULL}};
const Segment SGI312629[] = {{&S19356[13], 10}, {NULL}};
const Segment SGI312630[] = {{&S19356[42], 7}, {NULL}};
const Segment SGI312631[] = {{&S19356[35], 14}, {NULL}};
const Segment SGI312632[] = {{&S19356[51], 11}, {NULL}};
const Segment SGI312633[] = {{&S19356[51], 16}, {NULL}};
const Segment SGI312634[] = {{&S19358[30], 3}, {NULL}};
const Segment SGI312635[] = {{&S19358[8], 3}, {NULL}};
const Segment SGI312636[] = {{&S19358[13], 10}, {NULL}};
const Segment SGI312637[] = {{&S19358[42], 7}, {NULL}};
const Segment SGI312638[] = {{&S19358[35], 14}, {NULL}};
const Segment SGI312639[] = {{&S19358[56], 11}, {NULL}};
const Segment SGI312640[] = {{&S19358[120], 15}, {NULL}};
const Segment SGI312641[] = {{&S19358[56], 16}, {NULL}};
const Segment SGI312642[] = {{&S19358[51], 3}, {NULL}};
const Segment SGI312643[] = {{&S19359[30], 3}, {NULL}};
const Segment SGI312644[] = {{&S19359[8], 3}, {NULL}};
const Segment SGI312645[] = {{&S19359[13], 10}, {NULL}};
const Segment SGI312646[] = {{&S19359[42], 7}, {NULL}};
const Segment SGI312647[] = {{&S19359[35], 14}, {NULL}};
const Segment SGI312648[] = {{&S19359[51], 11}, {NULL}};
const Segment SGI312649[] = {{&S19359[51], 16}, {NULL}};
const Segment SGI312650[] = {{&S19361[30], 3}, {NULL}};
const Segment SGI312651[] = {{&S19361[8], 3}, {NULL}};
const Segment SGI312652[] = {{&S19361[13], 10}, {NULL}};
const Segment SGI312653[] = {{&S19361[42], 7}, {NULL}};
const Segment SGI312654[] = {{&S19361[35], 14}, {NULL}};
const Segment SGI312655[] = {{&S19361[58], 11}, {NULL}};
const Segment SGI312656[] = {{&S19361[51], 6}, {NULL}};
const Segment SGI312657[] = {{&S19361[58], 16}, {NULL}};
const Segment SGI312658[] = {{&S19362[30], 3}, {&S19362[128], 3}, {NULL}};
const Segment SGI312659[] = {{&S19362[8], 3}, {&S19362[8], 3}, {NULL}};
const Segment SGI312660[] = {{&S19362[13], 10}, {&S19362[13], 10}, {NULL}};
const Segment SGI312661[] = {{&S19362[42], 7}, {&S19362[42], 7}, {NULL}};
const Segment SGI312662[] = {{&S19362[35], 14}, {&S19362[35], 14}, {NULL}};
const Segment SGI312663[] = {{&S19362[72], 11}, {NULL}};
const Segment SGI312664[] = {{&S19362[72], 16}, {NULL}};
const Segment SGI312665[] = {{&S19362[149], 3}, {NULL}};
const Segment SGI312666[] = {{&S19363[30], 3}, {NULL}};
const Segment SGI312667[] = {{&S19363[8], 3}, {NULL}};
const Segment SGI312668[] = {{&S19363[13], 10}, {NULL}};
const Segment SGI312669[] = {{&S19363[42], 7}, {NULL}};
const Segment SGI312670[] = {{&S19363[35], 14}, {NULL}};
const Segment SGI312671[] = {{&S19363[160], 11}, {NULL}};
const Segment SGI312672[] = {{&S19363[160], 16}, {NULL}};
const Segment SGI312673[] = {{&S19364[30], 3}, {NULL}};
const Segment SGI312674[] = {{&S19364[8], 3}, {NULL}};
const Segment SGI312675[] = {{&S19364[13], 10}, {NULL}};
const Segment SGI312676[] = {{&S19364[42], 7}, {NULL}};
const Segment SGI312677[] = {{&S19364[35], 14}, {NULL}};
const Segment SGI312678[] = {{&S19364[179], 11}, {NULL}};
const Segment SGI312679[] = {{&S19364[179], 16}, {NULL}};
const Segment SGI312680[] = {{&S19366[30], 3}, {NULL}};
const Segment SGI312681[] = {{&S19366[8], 3}, {NULL}};
const Segment SGI312682[] = {{&S19366[13], 10}, {NULL}};
const Segment SGI312683[] = {{&S19366[42], 7}, {NULL}};
const Segment SGI312684[] = {{&S19366[35], 14}, {NULL}};
const Segment SGI312685[] = {{&S19366[56], 11}, {NULL}};
const Segment SGI312686[] = {{&S19366[56], 16}, {NULL}};
const Segment SGI312687[] = {{&S19366[51], 3}, {NULL}};
const Segment SGI312688[] = {{&S19367[30], 3}, {NULL}};
const Segment SGI312689[] = {{&S19367[8], 3}, {NULL}};
const Segment SGI312690[] = {{&S19367[13], 10}, {NULL}};
const Segment SGI312691[] = {{&S19367[42], 7}, {NULL}};
const Segment SGI312692[] = {{&S19367[35], 14}, {NULL}};
const Segment SGI312693[] = {{&S19367[64], 11}, {NULL}};
const Segment SGI312694[] = {{&S19367[64], 16}, {NULL}};
const Segment SGI312695[] = {{&S19367[128], 9}, {NULL}};
const Segment SGI312696[] = {{&S19368[30], 3}, {NULL}};
const Segment SGI312697[] = {{&S19368[8], 3}, {NULL}};
const Segment SGI312698[] = {{&S19368[13], 10}, {NULL}};
const Segment SGI312699[] = {{&S19368[42], 7}, {NULL}};
const Segment SGI312700[] = {{&S19368[35], 14}, {NULL}};
const Segment SGI312701[] = {{&S19368[64], 11}, {NULL}};
const Segment SGI312702[] = {{&S19368[173], 4}, {NULL}};
const Segment SGI312703[] = {{&S19368[64], 16}, {NULL}};
const Segment SGI312704[] = {{&S19387[30], 3}, {NULL}};
const Segment SGI312705[] = {{&S19387[212], 9}, {NULL}};
const Segment SGI312706[] = {{&S19387[8], 3}, {NULL}};
const Segment SGI312707[] = {{&S19387[13], 10}, {NULL}};
const Segment SGI312708[] = {{&S19387[42], 7}, {NULL}};
const Segment SGI312709[] = {{&S19387[35], 14}, {NULL}};
const Segment SGI312710[] = {{&S19387[91], 15}, {NULL}};
const Segment SGI312711[] = {{&S19387[184], 4}, {NULL}};
const Segment SGI312712[] = {{&S19387[51], 6}, {NULL}};
const Segment SGI312713[] = {{&S19388[30], 3}, {NULL}};
const Segment SGI312714[] = {{&S19388[190], 9}, {NULL}};
const Segment SGI312715[] = {{&S19388[8], 3}, {NULL}};
const Segment SGI312716[] = {{&S19388[13], 10}, {NULL}};
const Segment SGI312717[] = {{&S19388[42], 7}, {NULL}};
const Segment SGI312718[] = {{&S19388[35], 14}, {NULL}};
const Segment SGI312719[] = {{&S19388[91], 15}, {NULL}};
const Segment SGI312720[] = {{&S19388[51], 6}, {NULL}};
const Segment SGI312721[] = {{&S19389[30], 3}, {NULL}};
const Segment SGI312722[] = {{&S19389[186], 9}, {NULL}};
const Segment SGI312723[] = {{&S19389[8], 3}, {NULL}};
const Segment SGI312724[] = {{&S19389[13], 10}, {NULL}};
const Segment SGI312725[] = {{&S19389[42], 7}, {NULL}};
const Segment SGI312726[] = {{&S19389[35], 14}, {NULL}};
const Segment SGI312727[] = {{&S19389[51], 5}, {NULL}};
const Segment SGI312728[] = {{&S19390[30], 3}, {NULL}};
const Segment SGI312729[] = {{&S19390[191], 9}, {NULL}};
const Segment SGI312730[] = {{&S19390[8], 3}, {NULL}};
const Segment SGI312731[] = {{&S19390[13], 10}, {NULL}};
const Segment SGI312732[] = {{&S19390[42], 7}, {NULL}};
const Segment SGI312733[] = {{&S19390[35], 14}, {NULL}};
const Segment SGI312734[] = {{&S19390[163], 4}, {NULL}};
const Segment SGI312735[] = {{&S19390[51], 6}, {NULL}};
const Segment SGI312736[] = {{&S19391[30], 3}, {NULL}};
const Segment SGI312737[] = {{&S19391[169], 9}, {NULL}};
const Segment SGI312738[] = {{&S19391[8], 3}, {NULL}};
const Segment SGI312739[] = {{&S19391[13], 10}, {NULL}};
const Segment SGI312740[] = {{&S19391[42], 7}, {NULL}};
const Segment SGI312741[] = {{&S19391[35], 14}, {NULL}};
const Segment SGI312742[] = {{&S19391[51], 6}, {NULL}};
const Segment SGI312743[] = {{&S19393[30], 3}, {NULL}};
const Segment SGI312744[] = {{&S19393[196], 9}, {NULL}};
const Segment SGI312745[] = {{&S19393[8], 3}, {NULL}};
const Segment SGI312746[] = {{&S19393[13], 10}, {NULL}};
const Segment SGI312747[] = {{&S19393[42], 7}, {NULL}};
const Segment SGI312748[] = {{&S19393[35], 14}, {NULL}};
const Segment SGI312749[] = {{&S19393[168], 4}, {NULL}};
const Segment SGI312750[] = {{&S19393[51], 5}, {NULL}};
const Segment SGI312751[] = {{&S19394[30], 3}, {NULL}};
const Segment SGI312752[] = {{&S19394[206], 9}, {NULL}};
const Segment SGI312753[] = {{&S19394[8], 3}, {NULL}};
const Segment SGI312754[] = {{&S19394[13], 10}, {NULL}};
const Segment SGI312755[] = {{&S19394[42], 7}, {NULL}};
const Segment SGI312756[] = {{&S19394[35], 14}, {NULL}};
const Segment SGI312757[] = {{&S19394[178], 4}, {NULL}};
const Segment SGI312758[] = {{&S19394[51], 6}, {NULL}};
const Segment SGI312759[] = {{&S19394[163], 9}, {NULL}};
const Segment SGI312760[] = {{&S19395[30], 3}, {NULL}};
const Segment SGI312761[] = {{&S19395[214], 9}, {NULL}};
const Segment SGI312762[] = {{&S19395[8], 3}, {NULL}};
const Segment SGI312763[] = {{&S19395[13], 10}, {NULL}};
const Segment SGI312764[] = {{&S19395[42], 7}, {NULL}};
const Segment SGI312765[] = {{&S19395[35], 14}, {NULL}};
const Segment SGI312766[] = {{&S19395[168], 15}, {NULL}};
const Segment SGI312767[] = {{&S19395[51], 5}, {NULL}};
const Segment SGI312768[] = {{&S19396[30], 3}, {NULL}};
const Segment SGI312769[] = {{&S19396[174], 9}, {NULL}};
const Segment SGI312770[] = {{&S19396[8], 3}, {NULL}};
const Segment SGI312771[] = {{&S19396[13], 10}, {NULL}};
const Segment SGI312772[] = {{&S19396[42], 7}, {NULL}};
const Segment SGI312773[] = {{&S19396[35], 14}, {NULL}};
const Segment SGI312774[] = {{&S19397[30], 3}, {NULL}};
const Segment SGI312775[] = {{&S19397[181], 9}, {NULL}};
const Segment SGI312776[] = {{&S19397[8], 3}, {NULL}};
const Segment SGI312777[] = {{&S19397[13], 10}, {NULL}};
const Segment SGI312778[] = {{&S19397[42], 7}, {NULL}};
const Segment SGI312779[] = {{&S19397[35], 14}, {NULL}};
const Segment SGI312780[] = {{&S19397[51], 6}, {NULL}};
const Segment SGI312781[] = {{&S19398[30], 3}, {NULL}};
const Segment SGI312782[] = {{&S19398[184], 9}, {NULL}};
const Segment SGI312783[] = {{&S19398[8], 3}, {NULL}};
const Segment SGI312784[] = {{&S19398[13], 10}, {NULL}};
const Segment SGI312785[] = {{&S19398[42], 7}, {NULL}};
const Segment SGI312786[] = {{&S19398[35], 14}, {NULL}};
const Segment SGI312787[] = {{&S19398[51], 6}, {NULL}};
const Segment SGI312788[] = {{&S19398[151], 9}, {NULL}};
const Segment SGI312789[] = {{&S19399[30], 3}, {NULL}};
const Segment SGI312790[] = {{&S19399[229], 9}, {NULL}};
const Segment SGI312791[] = {{&S19399[8], 3}, {NULL}};
const Segment SGI312792[] = {{&S19399[13], 10}, {NULL}};
const Segment SGI312793[] = {{&S19399[42], 7}, {NULL}};
const Segment SGI312794[] = {{&S19399[35], 14}, {NULL}};
const Segment SGI312795[] = {{&S19399[168], 15}, {NULL}};
const Segment SGI312796[] = {{&S19399[51], 5}, {NULL}};
const Segment SGI312797[] = {{&S19399[196], 9}, {NULL}};
const Segment SGI312798[] = {{&S19402[30], 3}, {NULL}};
const Segment SGI312799[] = {{&S19402[190], 9}, {NULL}};
const Segment SGI312800[] = {{&S19402[8], 3}, {NULL}};
const Segment SGI312801[] = {{&S19402[13], 10}, {NULL}};
const Segment SGI312802[] = {{&S19402[42], 7}, {NULL}};
const Segment SGI312803[] = {{&S19402[35], 14}, {NULL}};
const Segment SGI312804[] = {{&S19402[151], 15}, {NULL}};
const Segment SGI312805[] = {{&S19402[51], 6}, {NULL}};
const Segment SGI312806[] = {{&S19403[30], 3}, {NULL}};
const Segment SGI312807[] = {{&S19403[202], 9}, {NULL}};
const Segment SGI312808[] = {{&S19403[8], 3}, {NULL}};
const Segment SGI312809[] = {{&S19403[13], 10}, {NULL}};
const Segment SGI312810[] = {{&S19403[42], 7}, {NULL}};
const Segment SGI312811[] = {{&S19403[35], 14}, {NULL}};
const Segment SGI312812[] = {{&S19403[156], 15}, {NULL}};
const Segment SGI312813[] = {{&S19404[30], 3}, {NULL}};
const Segment SGI312814[] = {{&S19404[191], 9}, {NULL}};
const Segment SGI312815[] = {{&S19404[8], 3}, {NULL}};
const Segment SGI312816[] = {{&S19404[13], 10}, {NULL}};
const Segment SGI312817[] = {{&S19404[42], 7}, {NULL}};
const Segment SGI312818[] = {{&S19404[35], 14}, {NULL}};
const Segment SGI312819[] = {{&S19404[163], 4}, {NULL}};
const Segment SGI312820[] = {{&S19404[51], 6}, {NULL}};
const Segment SGI312821[] = {{&S19405[30], 3}, {NULL}};
const Segment SGI312822[] = {{&S19405[217], 9}, {NULL}};
const Segment SGI312823[] = {{&S19405[8], 3}, {NULL}};
const Segment SGI312824[] = {{&S19405[13], 10}, {NULL}};
const Segment SGI312825[] = {{&S19405[42], 7}, {NULL}};
const Segment SGI312826[] = {{&S19405[35], 14}, {NULL}};
const Segment SGI312827[] = {{&S19405[156], 15}, {NULL}};
const Segment SGI312828[] = {{&S19405[184], 9}, {NULL}};
const Segment SGI312829[] = {{&S19406[30], 3}, {NULL}};
const Segment SGI312830[] = {{&S19406[169], 9}, {NULL}};
const Segment SGI312831[] = {{&S19406[8], 3}, {NULL}};
const Segment SGI312832[] = {{&S19406[13], 10}, {NULL}};
const Segment SGI312833[] = {{&S19406[42], 7}, {NULL}};
const Segment SGI312834[] = {{&S19406[35], 14}, {NULL}};
const Segment SGI312835[] = {{&S19406[51], 6}, {NULL}};
const Segment SGI312836[] = {{&S19407[30], 3}, {NULL}};
const Segment SGI312837[] = {{&S19407[169], 9}, {NULL}};
const Segment SGI312838[] = {{&S19407[8], 3}, {NULL}};
const Segment SGI312839[] = {{&S19407[13], 10}, {NULL}};
const Segment SGI312840[] = {{&S19407[42], 7}, {NULL}};
const Segment SGI312841[] = {{&S19407[35], 14}, {NULL}};
const Segment SGI312842[] = {{&S19407[51], 6}, {NULL}};
const Segment SGI312843[] = {{&S19408[30], 3}, {NULL}};
const Segment SGI312844[] = {{&S19408[190], 9}, {NULL}};
const Segment SGI312845[] = {{&S19408[8], 3}, {NULL}};
const Segment SGI312846[] = {{&S19408[13], 10}, {NULL}};
const Segment SGI312847[] = {{&S19408[42], 7}, {NULL}};
const Segment SGI312848[] = {{&S19408[35], 14}, {NULL}};
const Segment SGI312849[] = {{&S19408[151], 15}, {NULL}};
const Segment SGI312850[] = {{&S19408[51], 6}, {NULL}};
const Segment SGI312851[] = {{&S19409[30], 3}, {NULL}};
const Segment SGI312852[] = {{&S19409[186], 9}, {NULL}};
const Segment SGI312853[] = {{&S19409[8], 3}, {NULL}};
const Segment SGI312854[] = {{&S19409[13], 10}, {NULL}};
const Segment SGI312855[] = {{&S19409[42], 7}, {NULL}};
const Segment SGI312856[] = {{&S19409[35], 14}, {NULL}};
const Segment SGI312857[] = {{&S19409[51], 5}, {NULL}};
const Segment SGI312858[] = {{&S19410[30], 3}, {NULL}};
const Segment SGI312859[] = {{&S19410[162], 9}, {NULL}};
const Segment SGI312860[] = {{&S19410[8], 3}, {NULL}};
const Segment SGI312861[] = {{&S19410[13], 10}, {NULL}};
const Segment SGI312862[] = {{&S19410[42], 7}, {NULL}};
const Segment SGI312863[] = {{&S19410[35], 14}, {NULL}};
const Segment SGI312864[] = {{&S19411[30], 3}, {NULL}};
const Segment SGI312865[] = {{&S19411[184], 9}, {NULL}};
const Segment SGI312866[] = {{&S19411[8], 3}, {NULL}};
const Segment SGI312867[] = {{&S19411[13], 10}, {NULL}};
const Segment SGI312868[] = {{&S19411[42], 7}, {NULL}};
const Segment SGI312869[] = {{&S19411[35], 14}, {NULL}};
const Segment SGI312870[] = {{&S19411[51], 6}, {NULL}};
const Segment SGI312871[] = {{&S19411[151], 9}, {NULL}};
const Segment SGI312872[] = {{&S19412[30], 3}, {NULL}};
const Segment SGI312873[] = {{&S19412[214], 9}, {NULL}};
const Segment SGI312874[] = {{&S19412[8], 3}, {NULL}};
const Segment SGI312875[] = {{&S19412[13], 10}, {NULL}};
const Segment SGI312876[] = {{&S19412[42], 7}, {NULL}};
const Segment SGI312877[] = {{&S19412[35], 14}, {NULL}};
const Segment SGI312878[] = {{&S19412[168], 15}, {NULL}};
const Segment SGI312879[] = {{&S19412[51], 5}, {NULL}};
const Segment SGI312880[] = {{&S19413[30], 3}, {NULL}};
const Segment SGI312881[] = {{&S19413[229], 9}, {NULL}};
const Segment SGI312882[] = {{&S19413[8], 3}, {NULL}};
const Segment SGI312883[] = {{&S19413[13], 10}, {NULL}};
const Segment SGI312884[] = {{&S19413[42], 7}, {NULL}};
const Segment SGI312885[] = {{&S19413[35], 14}, {NULL}};
const Segment SGI312886[] = {{&S19413[168], 15}, {NULL}};
const Segment SGI312887[] = {{&S19413[51], 5}, {NULL}};
const Segment SGI312888[] = {{&S19413[196], 9}, {NULL}};
const Segment SGI312889[] = {{&S19415[30], 3}, {NULL}};
const Segment SGI312890[] = {{&S19415[174], 9}, {NULL}};
const Segment SGI312891[] = {{&S19415[8], 3}, {NULL}};
const Segment SGI312892[] = {{&S19415[13], 10}, {NULL}};
const Segment SGI312893[] = {{&S19415[42], 7}, {NULL}};
const Segment SGI312894[] = {{&S19415[35], 14}, {NULL}};
const Segment SGI312895[] = {{&S19415[51], 5}, {NULL}};
const Segment SGI312896[] = {{&S19416[30], 3}, {NULL}};
const Segment SGI312897[] = {{&S19416[190], 9}, {NULL}};
const Segment SGI312898[] = {{&S19416[8], 3}, {NULL}};
const Segment SGI312899[] = {{&S19416[13], 10}, {NULL}};
const Segment SGI312900[] = {{&S19416[42], 7}, {NULL}};
const Segment SGI312901[] = {{&S19416[35], 14}, {NULL}};
const Segment SGI312902[] = {{&S19416[144], 15}, {NULL}};
const Segment SGI312903[] = {{&S19417[30], 3}, {NULL}};
const Segment SGI312904[] = {{&S19417[217], 9}, {NULL}};
const Segment SGI312905[] = {{&S19417[8], 3}, {NULL}};
const Segment SGI312906[] = {{&S19417[13], 10}, {NULL}};
const Segment SGI312907[] = {{&S19417[42], 7}, {NULL}};
const Segment SGI312908[] = {{&S19417[35], 14}, {NULL}};
const Segment SGI312909[] = {{&S19417[156], 15}, {NULL}};
const Segment SGI312910[] = {{&S19417[184], 9}, {NULL}};
const Segment SGI312911[] = {{&S19418[30], 3}, {NULL}};
const Segment SGI312912[] = {{&S19418[205], 9}, {NULL}};
const Segment SGI312913[] = {{&S19418[8], 3}, {NULL}};
const Segment SGI312914[] = {{&S19418[13], 10}, {NULL}};
const Segment SGI312915[] = {{&S19418[42], 7}, {NULL}};
const Segment SGI312916[] = {{&S19418[35], 14}, {NULL}};
const Segment SGI312917[] = {{&S19418[166], 15}, {NULL}};
const Segment SGI312918[] = {{&S19418[51], 6}, {NULL}};
const Segment SGI312919[] = {{&S19418[151], 9}, {NULL}};
const Segment SGI312920[] = {{&S19419[30], 3}, {NULL}};
const Segment SGI312921[] = {{&S19419[191], 9}, {NULL}};
const Segment SGI312922[] = {{&S19419[8], 3}, {NULL}};
const Segment SGI312923[] = {{&S19419[13], 10}, {NULL}};
const Segment SGI312924[] = {{&S19419[42], 7}, {NULL}};
const Segment SGI312925[] = {{&S19419[35], 14}, {NULL}};
const Segment SGI312926[] = {{&S19419[163], 4}, {NULL}};
const Segment SGI312927[] = {{&S19419[51], 6}, {NULL}};
const Segment SGI312928[] = {{&S19422[30], 3}, {NULL}};
const Segment SGI312929[] = {{&S19422[102], 9}, {NULL}};
const Segment SGI312930[] = {{&S19422[8], 3}, {NULL}};
const Segment SGI312931[] = {{&S19422[13], 10}, {NULL}};
const Segment SGI312932[] = {{&S19422[42], 7}, {NULL}};
const Segment SGI312933[] = {{&S19422[35], 14}, {NULL}};
const Segment SGI312934[] = {{&S19423[30], 3}, {NULL}};
const Segment SGI312935[] = {{&S19423[179], 9}, {NULL}};
const Segment SGI312936[] = {{&S19423[8], 3}, {NULL}};
const Segment SGI312937[] = {{&S19423[13], 10}, {NULL}};
const Segment SGI312938[] = {{&S19423[42], 7}, {NULL}};
const Segment SGI312939[] = {{&S19423[35], 14}, {NULL}};
const Segment SGI312940[] = {{&S19423[51], 6}, {NULL}};
const Segment SGI312941[] = {{&S19424[30], 4}, {NULL}};
const Segment SGI312942[] = {{&S19424[90], 9}, {NULL}};
const Segment SGI312943[] = {{&S19424[8], 3}, {NULL}};
const Segment SGI312944[] = {{&S19424[13], 10}, {NULL}};
const Segment SGI312945[] = {{&S19424[43], 7}, {NULL}};
const Segment SGI312946[] = {{&S19424[36], 14}, {NULL}};
const Segment SGI312947[] = {{&S19424[52], 6}, {NULL}};
const Segment SGI312948[] = {{&S19425[30], 3}, {NULL}};
const Segment SGI312949[] = {{&S19425[196], 9}, {NULL}};
const Segment SGI312950[] = {{&S19425[8], 3}, {NULL}};
const Segment SGI312951[] = {{&S19425[13], 10}, {NULL}};
const Segment SGI312952[] = {{&S19425[42], 7}, {NULL}};
const Segment SGI312953[] = {{&S19425[35], 14}, {NULL}};
const Segment SGI312954[] = {{&S19425[168], 4}, {NULL}};
const Segment SGI312955[] = {{&S19425[51], 5}, {NULL}};
const Segment SGI312956[] = {{&S19426[30], 3}, {NULL}};
const Segment SGI312957[] = {{&S19426[169], 9}, {NULL}};
const Segment SGI312958[] = {{&S19426[8], 3}, {NULL}};
const Segment SGI312959[] = {{&S19426[13], 10}, {NULL}};
const Segment SGI312960[] = {{&S19426[42], 7}, {NULL}};
const Segment SGI312961[] = {{&S19426[35], 14}, {NULL}};
const Segment SGI312962[] = {{&S19426[51], 6}, {NULL}};
const Segment SGI312963[] = {{&S19427[30], 3}, {NULL}};
const Segment SGI312964[] = {{&S19427[189], 9}, {NULL}};
const Segment SGI312965[] = {{&S19427[8], 3}, {NULL}};
const Segment SGI312966[] = {{&S19427[13], 10}, {NULL}};
const Segment SGI312967[] = {{&S19427[42], 7}, {NULL}};
const Segment SGI312968[] = {{&S19427[35], 14}, {NULL}};
const Segment SGI312969[] = {{&S19427[51], 5}, {NULL}};
const Segment SGI312970[] = {{&S19427[156], 9}, {NULL}};
const Segment SGI312971[] = {{&S19428[30], 3}, {NULL}};
const Segment SGI312972[] = {{&S19428[186], 9}, {NULL}};
const Segment SGI312973[] = {{&S19428[8], 3}, {NULL}};
const Segment SGI312974[] = {{&S19428[13], 10}, {NULL}};
const Segment SGI312975[] = {{&S19428[42], 7}, {NULL}};
const Segment SGI312976[] = {{&S19428[35], 14}, {NULL}};
const Segment SGI312977[] = {{&S19428[51], 5}, {NULL}};
const Segment SGI312978[] = {{&S19429[30], 3}, {NULL}};
const Segment SGI312979[] = {{&S19429[190], 9}, {NULL}};
const Segment SGI312980[] = {{&S19429[8], 3}, {NULL}};
const Segment SGI312981[] = {{&S19429[13], 10}, {NULL}};
const Segment SGI312982[] = {{&S19429[42], 7}, {NULL}};
const Segment SGI312983[] = {{&S19429[35], 14}, {NULL}};
const Segment SGI312984[] = {{&S19429[151], 15}, {NULL}};
const Segment SGI312985[] = {{&S19429[51], 6}, {NULL}};
const Segment SGI312986[] = {{&S19430[30], 3}, {NULL}};
const Segment SGI312987[] = {{&S19430[169], 9}, {NULL}};
const Segment SGI312988[] = {{&S19430[8], 3}, {NULL}};
const Segment SGI312989[] = {{&S19430[13], 10}, {NULL}};
const Segment SGI312990[] = {{&S19430[42], 7}, {NULL}};
const Segment SGI312991[] = {{&S19430[35], 14}, {NULL}};
const Segment SGI312992[] = {{&S19430[51], 6}, {NULL}};
const Segment SGI312993[] = {{&S19431[30], 3}, {NULL}};
const Segment SGI312994[] = {{&S19431[184], 9}, {NULL}};
const Segment SGI312995[] = {{&S19431[8], 3}, {NULL}};
const Segment SGI312996[] = {{&S19431[13], 10}, {NULL}};
const Segment SGI312997[] = {{&S19431[42], 7}, {NULL}};
const Segment SGI312998[] = {{&S19431[35], 14}, {NULL}};
const Segment SGI312999[] = {{&S19431[51], 6}, {NULL}};
const Segment SGI313000[] = {{&S19431[151], 9}, {NULL}};
const Segment SGI313001[] = {{&S19432[30], 3}, {NULL}};
const Segment SGI313002[] = {{&S19432[229], 9}, {NULL}};
const Segment SGI313003[] = {{&S19432[8], 3}, {NULL}};
const Segment SGI313004[] = {{&S19432[13], 10}, {NULL}};
const Segment SGI313005[] = {{&S19432[42], 7}, {NULL}};
const Segment SGI313006[] = {{&S19432[35], 14}, {NULL}};
const Segment SGI313007[] = {{&S19432[168], 15}, {NULL}};
const Segment SGI313008[] = {{&S19432[51], 5}, {NULL}};
const Segment SGI313009[] = {{&S19432[196], 9}, {NULL}};
const Segment SGI313010[] = {{&S19433[30], 3}, {NULL}};
const Segment SGI313011[] = {{&S19433[214], 9}, {NULL}};
const Segment SGI313012[] = {{&S19433[8], 3}, {NULL}};
const Segment SGI313013[] = {{&S19433[13], 10}, {NULL}};
const Segment SGI313014[] = {{&S19433[42], 7}, {NULL}};
const Segment SGI313015[] = {{&S19433[35], 14}, {NULL}};
const Segment SGI313016[] = {{&S19433[168], 15}, {NULL}};
const Segment SGI313017[] = {{&S19433[51], 5}, {NULL}};
const Segment SGI313018[] = {{&S19434[30], 3}, {NULL}};
const Segment SGI313019[] = {{&S19434[162], 9}, {NULL}};
const Segment SGI313020[] = {{&S19434[8], 3}, {NULL}};
const Segment SGI313021[] = {{&S19434[13], 10}, {NULL}};
const Segment SGI313022[] = {{&S19434[42], 7}, {NULL}};
const Segment SGI313023[] = {{&S19434[35], 14}, {NULL}};
const Segment SGI313024[] = {{&S19435[30], 3}, {NULL}};
const Segment SGI313025[] = {{&S19435[205], 9}, {NULL}};
const Segment SGI313026[] = {{&S19435[8], 3}, {NULL}};
const Segment SGI313027[] = {{&S19435[13], 10}, {NULL}};
const Segment SGI313028[] = {{&S19435[42], 7}, {NULL}};
const Segment SGI313029[] = {{&S19435[35], 14}, {NULL}};
const Segment SGI313030[] = {{&S19435[166], 15}, {NULL}};
const Segment SGI313031[] = {{&S19435[51], 6}, {NULL}};
const Segment SGI313032[] = {{&S19435[151], 9}, {NULL}};
const Segment SGI313033[] = {{&S19436[30], 4}, {NULL}};
const Segment SGI313034[] = {{&S19436[90], 9}, {NULL}};
const Segment SGI313035[] = {{&S19436[8], 3}, {NULL}};
const Segment SGI313036[] = {{&S19436[13], 10}, {NULL}};
const Segment SGI313037[] = {{&S19436[43], 7}, {NULL}};
const Segment SGI313038[] = {{&S19436[36], 14}, {NULL}};
const Segment SGI313039[] = {{&S19436[52], 6}, {NULL}};
const Segment SGI313040[] = {{&S19437[30], 3}, {NULL}};
const Segment SGI313041[] = {{&S19437[191], 9}, {NULL}};
const Segment SGI313042[] = {{&S19437[8], 3}, {NULL}};
const Segment SGI313043[] = {{&S19437[13], 10}, {NULL}};
const Segment SGI313044[] = {{&S19437[42], 7}, {NULL}};
const Segment SGI313045[] = {{&S19437[35], 14}, {NULL}};
const Segment SGI313046[] = {{&S19437[163], 4}, {NULL}};
const Segment SGI313047[] = {{&S19437[51], 6}, {NULL}};
const Segment SGI313048[] = {{&S19438[30], 3}, {NULL}};
const Segment SGI313049[] = {{&S19438[202], 9}, {NULL}};
const Segment SGI313050[] = {{&S19438[8], 3}, {NULL}};
const Segment SGI313051[] = {{&S19438[13], 10}, {NULL}};
const Segment SGI313052[] = {{&S19438[42], 7}, {NULL}};
const Segment SGI313053[] = {{&S19438[35], 14}, {NULL}};
const Segment SGI313054[] = {{&S19438[156], 15}, {NULL}};
const Segment SGI313055[] = {{&S19439[30], 3}, {NULL}};
const Segment SGI313056[] = {{&S19439[217], 9}, {NULL}};
const Segment SGI313057[] = {{&S19439[8], 3}, {NULL}};
const Segment SGI313058[] = {{&S19439[13], 10}, {NULL}};
const Segment SGI313059[] = {{&S19439[42], 7}, {NULL}};
const Segment SGI313060[] = {{&S19439[35], 14}, {NULL}};
const Segment SGI313061[] = {{&S19439[156], 15}, {NULL}};
const Segment SGI313062[] = {{&S19439[184], 9}, {NULL}};
const Segment SGI313063[] = {{&S19440[30], 3}, {NULL}};
const Segment SGI313064[] = {{&S19440[196], 9}, {NULL}};
const Segment SGI313065[] = {{&S19440[8], 3}, {NULL}};
const Segment SGI313066[] = {{&S19440[13], 10}, {NULL}};
const Segment SGI313067[] = {{&S19440[42], 7}, {NULL}};
const Segment SGI313068[] = {{&S19440[35], 14}, {NULL}};
const Segment SGI313069[] = {{&S19440[168], 4}, {NULL}};
const Segment SGI313070[] = {{&S19440[51], 5}, {NULL}};
const Segment SGI313071[] = {{&S19441[30], 3}, {NULL}};
const Segment SGI313072[] = {{&S19441[169], 9}, {NULL}};
const Segment SGI313073[] = {{&S19441[8], 3}, {NULL}};
const Segment SGI313074[] = {{&S19441[13], 10}, {NULL}};
const Segment SGI313075[] = {{&S19441[42], 7}, {NULL}};
const Segment SGI313076[] = {{&S19441[35], 14}, {NULL}};
const Segment SGI313077[] = {{&S19441[51], 6}, {NULL}};
const Segment SGI313078[] = {{&S19442[30], 3}, {NULL}};
const Segment SGI313079[] = {{&S19442[201], 9}, {NULL}};
const Segment SGI313080[] = {{&S19442[8], 3}, {NULL}};
const Segment SGI313081[] = {{&S19442[13], 10}, {NULL}};
const Segment SGI313082[] = {{&S19442[42], 7}, {NULL}};
const Segment SGI313083[] = {{&S19442[35], 14}, {NULL}};
const Segment SGI313084[] = {{&S19442[51], 5}, {NULL}};
const Segment SGI313085[] = {{&S19442[96], 9}, {NULL}};
const Segment SGI313086[] = {{&S19443[30], 3}, {NULL}};
const Segment SGI313087[] = {{&S19443[192], 9}, {NULL}};
const Segment SGI313088[] = {{&S19443[8], 3}, {NULL}};
const Segment SGI313089[] = {{&S19443[13], 10}, {NULL}};
const Segment SGI313090[] = {{&S19443[42], 7}, {NULL}};
const Segment SGI313091[] = {{&S19443[35], 14}, {NULL}};
const Segment SGI313092[] = {{&S19443[51], 6}, {NULL}};
const Segment SGI313093[] = {{&S19443[151], 9}, {NULL}};
const Segment SGI313094[] = {{&S19444[30], 4}, {NULL}};
const Segment SGI313095[] = {{&S19444[95], 9}, {NULL}};
const Segment SGI313096[] = {{&S19444[8], 3}, {NULL}};
const Segment SGI313097[] = {{&S19444[13], 10}, {NULL}};
const Segment SGI313098[] = {{&S19444[43], 7}, {NULL}};
const Segment SGI313099[] = {{&S19444[36], 14}, {NULL}};
const Segment SGI313100[] = {{&S19444[52], 5}, {NULL}};
const Segment SGI313101[] = {{&S19445[30], 3}, {NULL}};
const Segment SGI313102[] = {{&S19445[224], 9}, {NULL}};
const Segment SGI313103[] = {{&S19445[8], 3}, {NULL}};
const Segment SGI313104[] = {{&S19445[13], 10}, {NULL}};
const Segment SGI313105[] = {{&S19445[42], 7}, {NULL}};
const Segment SGI313106[] = {{&S19445[35], 14}, {NULL}};
const Segment SGI313107[] = {{&S19445[168], 15}, {NULL}};
const Segment SGI313108[] = {{&S19445[196], 4}, {NULL}};
const Segment SGI313109[] = {{&S19445[51], 5}, {NULL}};
const Segment SGI313110[] = {{&S19446[30], 3}, {NULL}};
const Segment SGI313111[] = {{&S19446[212], 9}, {NULL}};
const Segment SGI313112[] = {{&S19446[8], 3}, {NULL}};
const Segment SGI313113[] = {{&S19446[13], 10}, {NULL}};
const Segment SGI313114[] = {{&S19446[42], 7}, {NULL}};
const Segment SGI313115[] = {{&S19446[35], 14}, {NULL}};
const Segment SGI313116[] = {{&S19446[156], 15}, {NULL}};
const Segment SGI313117[] = {{&S19446[184], 4}, {NULL}};
const Segment SGI313118[] = {{&S19446[51], 5}, {NULL}};
const Segment SGI313119[] = {{&S19447[30], 3}, {NULL}};
const Segment SGI313120[] = {{&S19447[185], 9}, {NULL}};
const Segment SGI313121[] = {{&S19447[8], 3}, {NULL}};
const Segment SGI313122[] = {{&S19447[13], 10}, {NULL}};
const Segment SGI313123[] = {{&S19447[42], 7}, {NULL}};
const Segment SGI313124[] = {{&S19447[35], 14}, {NULL}};
const Segment SGI313125[] = {{&S19447[144], 9}, {NULL}};
const Segment SGI313126[] = {{&S19448[30], 3}, {NULL}};
const Segment SGI313127[] = {{&S19448[239], 9}, {NULL}};
const Segment SGI313128[] = {{&S19448[8], 3}, {NULL}};
const Segment SGI313129[] = {{&S19448[13], 10}, {NULL}};
const Segment SGI313130[] = {{&S19448[42], 7}, {NULL}};
const Segment SGI313131[] = {{&S19448[35], 14}, {NULL}};
const Segment SGI313132[] = {{&S19448[168], 15}, {NULL}};
const Segment SGI313133[] = {{&S19448[211], 4}, {NULL}};
const Segment SGI313134[] = {{&S19448[51], 5}, {NULL}};
const Segment SGI313135[] = {{&S19448[196], 9}, {NULL}};
const Segment SGI313136[] = {{&S19449[30], 3}, {NULL}};
const Segment SGI313137[] = {{&S19449[184], 9}, {NULL}};
const Segment SGI313138[] = {{&S19449[8], 3}, {NULL}};
const Segment SGI313139[] = {{&S19449[13], 10}, {NULL}};
const Segment SGI313140[] = {{&S19449[42], 7}, {NULL}};
const Segment SGI313141[] = {{&S19449[35], 14}, {NULL}};
const Segment SGI313142[] = {{&S19449[156], 4}, {NULL}};
const Segment SGI313143[] = {{&S19451[30], 3}, {NULL}};
const Segment SGI313144[] = {{&S19451[201], 9}, {NULL}};
const Segment SGI313145[] = {{&S19451[8], 3}, {NULL}};
const Segment SGI313146[] = {{&S19451[13], 10}, {NULL}};
const Segment SGI313147[] = {{&S19451[42], 7}, {NULL}};
const Segment SGI313148[] = {{&S19451[35], 14}, {NULL}};
const Segment SGI313149[] = {{&S19451[51], 5}, {NULL}};
const Segment SGI313150[] = {{&S19451[96], 9}, {NULL}};
const Segment SGI313151[] = {{&S19452[30], 3}, {NULL}};
const Segment SGI313152[] = {{&S19452[239], 9}, {NULL}};
const Segment SGI313153[] = {{&S19452[8], 3}, {NULL}};
const Segment SGI313154[] = {{&S19452[13], 10}, {NULL}};
const Segment SGI313155[] = {{&S19452[42], 7}, {NULL}};
const Segment SGI313156[] = {{&S19452[35], 14}, {NULL}};
const Segment SGI313157[] = {{&S19452[168], 15}, {NULL}};
const Segment SGI313158[] = {{&S19452[211], 4}, {NULL}};
const Segment SGI313159[] = {{&S19452[51], 5}, {NULL}};
const Segment SGI313160[] = {{&S19452[196], 9}, {NULL}};
const Segment SGI313161[] = {{&S19453[30], 3}, {NULL}};
const Segment SGI313162[] = {{&S19453[184], 9}, {NULL}};
const Segment SGI313163[] = {{&S19453[8], 3}, {NULL}};
const Segment SGI313164[] = {{&S19453[13], 10}, {NULL}};
const Segment SGI313165[] = {{&S19453[42], 7}, {NULL}};
const Segment SGI313166[] = {{&S19453[35], 14}, {NULL}};
const Segment SGI313167[] = {{&S19453[51], 6}, {NULL}};
const Segment SGI313168[] = {{&S19453[91], 9}, {NULL}};
const Segment SGI313169[] = {{&S19454[30], 3}, {NULL}};
const Segment SGI313170[] = {{&S19454[194], 9}, {NULL}};
const Segment SGI313171[] = {{&S19454[8], 3}, {NULL}};
const Segment SGI313172[] = {{&S19454[13], 10}, {NULL}};
const Segment SGI313173[] = {{&S19454[42], 7}, {NULL}};
const Segment SGI313174[] = {{&S19454[35], 14}, {NULL}};
const Segment SGI313175[] = {{&S19454[166], 4}, {NULL}};
const Segment SGI313176[] = {{&S19454[51], 6}, {NULL}};
const Segment SGI313177[] = {{&S19454[91], 9}, {NULL}};
const Segment SGI313178[] = {{&S19455[30], 3}, {NULL}};
const Segment SGI313179[] = {{&S19455[200], 9}, {NULL}};
const Segment SGI313180[] = {{&S19455[8], 3}, {NULL}};
const Segment SGI313181[] = {{&S19455[13], 10}, {NULL}};
const Segment SGI313182[] = {{&S19455[42], 7}, {NULL}};
const Segment SGI313183[] = {{&S19455[35], 14}, {NULL}};
const Segment SGI313184[] = {{&S19455[151], 15}, {NULL}};
const Segment SGI313185[] = {{&S19455[172], 4}, {NULL}};
const Segment SGI313186[] = {{&S19455[51], 6}, {NULL}};
const Segment SGI313187[] = {{&S19456[30], 3}, {NULL}};
const Segment SGI313188[] = {{&S19456[169], 9}, {NULL}};
const Segment SGI313189[] = {{&S19456[8], 3}, {NULL}};
const Segment SGI313190[] = {{&S19456[13], 10}, {NULL}};
const Segment SGI313191[] = {{&S19456[42], 7}, {NULL}};
const Segment SGI313192[] = {{&S19456[35], 14}, {NULL}};
const Segment SGI313193[] = {{&S19456[51], 6}, {NULL}};
const Segment SGI313194[] = {{&S19457[30], 3}, {NULL}};
const Segment SGI313195[] = {{&S19457[195], 9}, {NULL}};
const Segment SGI313196[] = {{&S19457[8], 3}, {NULL}};
const Segment SGI313197[] = {{&S19457[13], 10}, {NULL}};
const Segment SGI313198[] = {{&S19457[42], 7}, {NULL}};
const Segment SGI313199[] = {{&S19457[35], 14}, {NULL}};
const Segment SGI313200[] = {{&S19457[156], 15}, {NULL}};
const Segment SGI313201[] = {{&S19457[51], 5}, {NULL}};
const Segment SGI313202[] = {{&S19458[30], 3}, {NULL}};
const Segment SGI313203[] = {{&S19458[174], 9}, {NULL}};
const Segment SGI313204[] = {{&S19458[8], 3}, {NULL}};
const Segment SGI313205[] = {{&S19458[13], 10}, {NULL}};
const Segment SGI313206[] = {{&S19458[42], 7}, {NULL}};
const Segment SGI313207[] = {{&S19458[35], 14}, {NULL}};
const Segment SGI313208[] = {{&S19458[51], 5}, {NULL}};
const Segment SGI313209[] = {{&S19459[30], 3}, {NULL}};
const Segment SGI313210[] = {{&S19459[184], 9}, {NULL}};
const Segment SGI313211[] = {{&S19459[8], 3}, {NULL}};
const Segment SGI313212[] = {{&S19459[13], 10}, {NULL}};
const Segment SGI313213[] = {{&S19459[42], 7}, {NULL}};
const Segment SGI313214[] = {{&S19459[35], 14}, {NULL}};
const Segment SGI313215[] = {{&S19459[51], 6}, {NULL}};
const Segment SGI313216[] = {{&S19459[151], 9}, {NULL}};
const Segment SGI313217[] = {{&S19460[30], 3}, {NULL}};
const Segment SGI313218[] = {{&S19460[190], 9}, {NULL}};
const Segment SGI313219[] = {{&S19460[8], 3}, {NULL}};
const Segment SGI313220[] = {{&S19460[13], 10}, {NULL}};
const Segment SGI313221[] = {{&S19460[42], 7}, {NULL}};
const Segment SGI313222[] = {{&S19460[35], 14}, {NULL}};
const Segment SGI313223[] = {{&S19460[151], 15}, {NULL}};
const Segment SGI313224[] = {{&S19460[51], 6}, {NULL}};
const Segment SGI313225[] = {{&S19462[30], 3}, {NULL}};
const Segment SGI313226[] = {{&S19462[174], 9}, {NULL}};
const Segment SGI313227[] = {{&S19462[8], 3}, {NULL}};
const Segment SGI313228[] = {{&S19462[13], 10}, {NULL}};
const Segment SGI313229[] = {{&S19462[42], 7}, {NULL}};
const Segment SGI313230[] = {{&S19462[35], 14}, {NULL}};
const Segment SGI313231[] = {{&S19463[30], 3}, {NULL}};
const Segment SGI313232[] = {{&S19463[229], 9}, {NULL}};
const Segment SGI313233[] = {{&S19463[8], 3}, {NULL}};
const Segment SGI313234[] = {{&S19463[13], 10}, {NULL}};
const Segment SGI313235[] = {{&S19463[42], 7}, {NULL}};
const Segment SGI313236[] = {{&S19463[35], 14}, {NULL}};
const Segment SGI313237[] = {{&S19463[168], 15}, {NULL}};
const Segment SGI313238[] = {{&S19463[51], 5}, {NULL}};
const Segment SGI313239[] = {{&S19463[196], 9}, {NULL}};
const Segment SGI313240[] = {{&S19464[30], 3}, {NULL}};
const Segment SGI313241[] = {{&S19464[179], 9}, {NULL}};
const Segment SGI313242[] = {{&S19464[8], 3}, {NULL}};
const Segment SGI313243[] = {{&S19464[13], 10}, {NULL}};
const Segment SGI313244[] = {{&S19464[42], 7}, {NULL}};
const Segment SGI313245[] = {{&S19464[35], 14}, {NULL}};
const Segment SGI313246[] = {{&S19464[151], 4}, {NULL}};
const Segment SGI313247[] = {{&S19464[51], 6}, {NULL}};
const Segment SGI313248[] = {{&S19465[30], 3}, {NULL}};
const Segment SGI313249[] = {{&S19465[190], 9}, {NULL}};
const Segment SGI313250[] = {{&S19465[8], 3}, {NULL}};
const Segment SGI313251[] = {{&S19465[13], 10}, {NULL}};
const Segment SGI313252[] = {{&S19465[42], 7}, {NULL}};
const Segment SGI313253[] = {{&S19465[35], 14}, {NULL}};
const Segment SGI313254[] = {{&S19465[144], 15}, {NULL}};
const Segment SGI313255[] = {{&S19466[30], 3}, {NULL}};
const Segment SGI313256[] = {{&S19466[190], 9}, {NULL}};
const Segment SGI313257[] = {{&S19466[8], 3}, {NULL}};
const Segment SGI313258[] = {{&S19466[13], 10}, {NULL}};
const Segment SGI313259[] = {{&S19466[42], 7}, {NULL}};
const Segment SGI313260[] = {{&S19466[35], 14}, {NULL}};
const Segment SGI313261[] = {{&S19466[151], 15}, {NULL}};
const Segment SGI313262[] = {{&S19466[51], 6}, {NULL}};
const Segment SGI313263[] = {{&S19467[30], 3}, {NULL}};
const Segment SGI313264[] = {{&S19467[169], 9}, {NULL}};
const Segment SGI313265[] = {{&S19467[8], 3}, {NULL}};
const Segment SGI313266[] = {{&S19467[13], 10}, {NULL}};
const Segment SGI313267[] = {{&S19467[42], 7}, {NULL}};
const Segment SGI313268[] = {{&S19467[35], 14}, {NULL}};
const Segment SGI313269[] = {{&S19467[51], 6}, {NULL}};
const Segment SGI313270[] = {{&S19469[30], 3}, {NULL}};
const Segment SGI313271[] = {{&S19469[192], 9}, {NULL}};
const Segment SGI313272[] = {{&S19469[8], 3}, {NULL}};
const Segment SGI313273[] = {{&S19469[13], 10}, {NULL}};
const Segment SGI313274[] = {{&S19469[42], 7}, {NULL}};
const Segment SGI313275[] = {{&S19469[35], 14}, {NULL}};
const Segment SGI313276[] = {{&S19469[51], 6}, {NULL}};
const Segment SGI313277[] = {{&S19469[151], 9}, {NULL}};
const Segment SGI313278[] = {{&S19470[30], 3}, {NULL}};
const Segment SGI313279[] = {{&S19470[186], 9}, {NULL}};
const Segment SGI313280[] = {{&S19470[8], 3}, {NULL}};
const Segment SGI313281[] = {{&S19470[13], 10}, {NULL}};
const Segment SGI313282[] = {{&S19470[42], 7}, {NULL}};
const Segment SGI313283[] = {{&S19470[35], 14}, {NULL}};
const Segment SGI313284[] = {{&S19470[51], 5}, {NULL}};
const Segment SGI313285[] = {{&S19471[30], 3}, {NULL}};
const Segment SGI313286[] = {{&S19471[174], 9}, {NULL}};
const Segment SGI313287[] = {{&S19471[8], 3}, {NULL}};
const Segment SGI313288[] = {{&S19471[13], 10}, {NULL}};
const Segment SGI313289[] = {{&S19471[42], 7}, {NULL}};
const Segment SGI313290[] = {{&S19471[35], 14}, {NULL}};
const Segment SGI313291[] = {{&S19472[30], 3}, {NULL}};
const Segment SGI313292[] = {{&S19472[229], 9}, {NULL}};
const Segment SGI313293[] = {{&S19472[8], 3}, {NULL}};
const Segment SGI313294[] = {{&S19472[13], 10}, {NULL}};
const Segment SGI313295[] = {{&S19472[42], 7}, {NULL}};
const Segment SGI313296[] = {{&S19472[35], 14}, {NULL}};
const Segment SGI313297[] = {{&S19472[168], 15}, {NULL}};
const Segment SGI313298[] = {{&S19472[51], 5}, {NULL}};
const Segment SGI313299[] = {{&S19472[196], 9}, {NULL}};
const Segment SGI313300[] = {{&S19473[30], 3}, {NULL}};
const Segment SGI313301[] = {{&S19473[179], 9}, {NULL}};
const Segment SGI313302[] = {{&S19473[8], 3}, {NULL}};
const Segment SGI313303[] = {{&S19473[13], 10}, {NULL}};
const Segment SGI313304[] = {{&S19473[42], 7}, {NULL}};
const Segment SGI313305[] = {{&S19473[35], 14}, {NULL}};
const Segment SGI313306[] = {{&S19473[151], 4}, {NULL}};
const Segment SGI313307[] = {{&S19473[51], 6}, {NULL}};
const Segment SGI313308[] = {{&S19474[30], 4}, {NULL}};
const Segment SGI313309[] = {{&S19474[90], 9}, {NULL}};
const Segment SGI313310[] = {{&S19474[8], 3}, {NULL}};
const Segment SGI313311[] = {{&S19474[13], 10}, {NULL}};
const Segment SGI313312[] = {{&S19474[43], 7}, {NULL}};
const Segment SGI313313[] = {{&S19474[36], 14}, {NULL}};
const Segment SGI313314[] = {{&S19474[52], 6}, {NULL}};
const Segment SGI313315[] = {{&S19475[30], 3}, {NULL}};
const Segment SGI313316[] = {{&S19475[205], 9}, {NULL}};
const Segment SGI313317[] = {{&S19475[8], 3}, {NULL}};
const Segment SGI313318[] = {{&S19475[13], 10}, {NULL}};
const Segment SGI313319[] = {{&S19475[42], 7}, {NULL}};
const Segment SGI313320[] = {{&S19475[35], 14}, {NULL}};
const Segment SGI313321[] = {{&S19475[151], 15}, {NULL}};
const Segment SGI313322[] = {{&S19475[51], 6}, {NULL}};
const Segment SGI313323[] = {{&S19475[172], 9}, {NULL}};
const Segment SGI313324[] = {{&S19476[30], 3}, {NULL}};
const Segment SGI313325[] = {{&S19476[202], 9}, {NULL}};
const Segment SGI313326[] = {{&S19476[8], 3}, {NULL}};
const Segment SGI313327[] = {{&S19476[13], 10}, {NULL}};
const Segment SGI313328[] = {{&S19476[42], 7}, {NULL}};
const Segment SGI313329[] = {{&S19476[35], 14}, {NULL}};
const Segment SGI313330[] = {{&S19476[156], 15}, {NULL}};
const Segment SGI313331[] = {{&S19476[51], 5}, {NULL}};
const Segment SGI313332[] = {{&S19477[30], 3}, {NULL}};
const Segment SGI313333[] = {{&S19477[202], 9}, {NULL}};
const Segment SGI313334[] = {{&S19477[8], 3}, {NULL}};
const Segment SGI313335[] = {{&S19477[13], 10}, {NULL}};
const Segment SGI313336[] = {{&S19477[42], 7}, {NULL}};
const Segment SGI313337[] = {{&S19477[35], 14}, {NULL}};
const Segment SGI313338[] = {{&S19477[156], 15}, {NULL}};
const Segment SGI313339[] = {{&S19478[30], 3}, {NULL}};
const Segment SGI313340[] = {{&S19478[189], 9}, {NULL}};
const Segment SGI313341[] = {{&S19478[8], 3}, {NULL}};
const Segment SGI313342[] = {{&S19478[13], 10}, {NULL}};
const Segment SGI313343[] = {{&S19478[42], 7}, {NULL}};
const Segment SGI313344[] = {{&S19478[35], 14}, {NULL}};
const Segment SGI313345[] = {{&S19478[51], 5}, {NULL}};
const Segment SGI313346[] = {{&S19478[96], 9}, {NULL}};
const Segment SGI313347[] = {{&S19527[30], 3}, {NULL}};
const Segment SGI313348[] = {{&S19527[8], 3}, {NULL}};
const Segment SGI313349[] = {{&S19527[13], 10}, {NULL}};
const Segment SGI313350[] = {{&S19527[42], 7}, {NULL}};
const Segment SGI313351[] = {{&S19527[35], 14}, {NULL}};
const Segment SGI313352[] = {{&S19527[51], 11}, {NULL}};
const Segment SGI313353[] = {{&S19527[51], 16}, {NULL}};
const Segment SGI313354[] = {{&S19529[30], 3}, {NULL}};
const Segment SGI313355[] = {{&S19529[8], 3}, {NULL}};
const Segment SGI313356[] = {{&S19529[13], 10}, {NULL}};
const Segment SGI313357[] = {{&S19529[42], 7}, {NULL}};
const Segment SGI313358[] = {{&S19529[35], 14}, {NULL}};
const Segment SGI313359[] = {{&S19529[51], 11}, {NULL}};
const Segment SGI313360[] = {{&S19529[51], 16}, {NULL}};
const Segment SGI313361[] = {{&S19531[30], 3}, {NULL}};
const Segment SGI313362[] = {{&S19531[8], 3}, {NULL}};
const Segment SGI313363[] = {{&S19531[13], 10}, {NULL}};
const Segment SGI313364[] = {{&S19531[42], 7}, {NULL}};
const Segment SGI313365[] = {{&S19531[35], 14}, {NULL}};
const Segment SGI313366[] = {{&S19531[51], 11}, {NULL}};
const Segment SGI313367[] = {{&S19531[51], 16}, {NULL}};
const Segment SGI313368[] = {{&S19532[30], 3}, {NULL}};
const Segment SGI313369[] = {{&S19532[8], 3}, {NULL}};
const Segment SGI313370[] = {{&S19532[13], 10}, {NULL}};
const Segment SGI313371[] = {{&S19532[42], 7}, {NULL}};
const Segment SGI313372[] = {{&S19532[35], 14}, {NULL}};
const Segment SGI313373[] = {{&S19532[56], 11}, {NULL}};
const Segment SGI313374[] = {{&S19532[56], 16}, {NULL}};
const Segment SGI313375[] = {{&S19532[51], 3}, {NULL}};
const Segment SGI313376[] = {{&S19534[30], 3}, {NULL}};
const Segment SGI313377[] = {{&S19534[8], 3}, {NULL}};
const Segment SGI313378[] = {{&S19534[13], 10}, {NULL}};
const Segment SGI313379[] = {{&S19534[42], 7}, {NULL}};
const Segment SGI313380[] = {{&S19534[35], 14}, {NULL}};
const Segment SGI313381[] = {{&S19534[56], 11}, {NULL}};
const Segment SGI313382[] = {{&S19534[56], 16}, {NULL}};
const Segment SGI313383[] = {{&S19534[51], 3}, {NULL}};
const Segment SGI313384[] = {{&S19535[30], 3}, {NULL}};
const Segment SGI313385[] = {{&S19535[8], 3}, {NULL}};
const Segment SGI313386[] = {{&S19535[13], 10}, {NULL}};
const Segment SGI313387[] = {{&S19535[42], 7}, {NULL}};
const Segment SGI313388[] = {{&S19535[35], 14}, {NULL}};
const Segment SGI313389[] = {{&S19535[71], 11}, {NULL}};
const Segment SGI313390[] = {{&S19535[51], 6}, {NULL}};
const Segment SGI313391[] = {{&S19535[71], 16}, {NULL}};
const Segment SGI313392[] = {{&S19536[30], 3}, {NULL}};
const Segment SGI313393[] = {{&S19536[8], 3}, {NULL}};
const Segment SGI313394[] = {{&S19536[13], 10}, {NULL}};
const Segment SGI313395[] = {{&S19536[42], 7}, {NULL}};
const Segment SGI313396[] = {{&S19536[35], 14}, {NULL}};
const Segment SGI313397[] = {{&S19536[51], 11}, {NULL}};
const Segment SGI313398[] = {{&S19536[51], 16}, {NULL}};
const Segment SGI313399[] = {{&S19536[191], 9}, {NULL}};
const Segment SGI313400[] = {{&S19537[30], 3}, {NULL}};
const Segment SGI313401[] = {{&S19537[8], 3}, {NULL}};
const Segment SGI313402[] = {{&S19537[13], 10}, {NULL}};
const Segment SGI313403[] = {{&S19537[42], 7}, {NULL}};
const Segment SGI313404[] = {{&S19537[35], 14}, {NULL}};
const Segment SGI313405[] = {{&S19537[64], 11}, {NULL}};
const Segment SGI313406[] = {{&S19537[64], 16}, {NULL}};
const Segment SGI313407[] = {{&S19540[30], 3}, {NULL}};
const Segment SGI313408[] = {{&S19540[8], 3}, {NULL}};
const Segment SGI313409[] = {{&S19540[13], 10}, {NULL}};
const Segment SGI313410[] = {{&S19540[42], 7}, {NULL}};
const Segment SGI313411[] = {{&S19540[35], 14}, {NULL}};
const Segment SGI313412[] = {{&S19540[64], 11}, {NULL}};
const Segment SGI313413[] = {{&S19540[136], 15}, {NULL}};
const Segment SGI313414[] = {{&S19540[64], 16}, {NULL}};
const Segment SGI313415[] = {{&S19542[30], 3}, {NULL}};
const Segment SGI313416[] = {{&S19542[8], 3}, {NULL}};
const Segment SGI313417[] = {{&S19542[13], 10}, {NULL}};
const Segment SGI313418[] = {{&S19542[42], 7}, {NULL}};
const Segment SGI313419[] = {{&S19542[35], 14}, {NULL}};
const Segment SGI313420[] = {{&S19542[56], 11}, {NULL}};
const Segment SGI313421[] = {{&S19542[56], 16}, {NULL}};
const Segment SGI313422[] = {{&S19542[51], 3}, {NULL}};
const Segment SGI313423[] = {{&S19543[30], 3}, {NULL}};
const Segment SGI313424[] = {{&S19543[8], 3}, {NULL}};
const Segment SGI313425[] = {{&S19543[13], 10}, {NULL}};
const Segment SGI313426[] = {{&S19543[42], 7}, {NULL}};
const Segment SGI313427[] = {{&S19543[35], 14}, {NULL}};
const Segment SGI313428[] = {{&S19543[64], 11}, {NULL}};
const Segment SGI313429[] = {{&S19543[64], 16}, {NULL}};
const Segment SGI313430[] = {{&S19544[30], 3}, {&S19544[120], 3}, {NULL}};
const Segment SGI313431[] = {{&S19544[8], 3}, {&S19544[8], 3}, {NULL}};
const Segment SGI313432[] = {{&S19544[13], 10}, {&S19544[13], 10}, {NULL}};
const Segment SGI313433[] = {{&S19544[42], 7}, {&S19544[42], 7}, {NULL}};
const Segment SGI313434[] = {{&S19544[35], 14}, {&S19544[35], 14}, {NULL}};
const Segment SGI313435[] = {{&S19544[64], 11}, {NULL}};
const Segment SGI313436[] = {{&S19544[64], 16}, {NULL}};
const Segment SGI313437[] = {{&S19544[141], 3}, {NULL}};
const Segment SGI313438[] = {{&S19545[30], 3}, {NULL}};
const Segment SGI313439[] = {{&S19545[8], 3}, {NULL}};
const Segment SGI313440[] = {{&S19545[13], 10}, {NULL}};
const Segment SGI313441[] = {{&S19545[42], 7}, {NULL}};
const Segment SGI313442[] = {{&S19545[35], 14}, {NULL}};
const Segment SGI313443[] = {{&S19545[69], 11}, {NULL}};
const Segment SGI313444[] = {{&S19545[69], 16}, {NULL}};
const Segment SGI313445[] = {{&S19545[64], 3}, {NULL}};
const Segment SGI313446[] = {{&S53625[7], 8}, {NULL}};
const Segment SGI313447[] = {{&S53625[0], 190}, {NULL}};
const Segment SGI313448[] = {{&S20972[8], 3}, {NULL}};
const Segment SGI313449[] = {{&S20972[13], 7}, {NULL}};
const Segment SGI313450[] = {{&S20972[40], 7}, {NULL}};
const Segment SGI313451[] = {{&S20972[33], 14}, {NULL}};
const Segment SGI313452[] = {{&S20988[8], 3}, {NULL}};
const Segment SGI313453[] = {{&S20988[13], 10}, {NULL}};
const Segment SGI313454[] = {{&S20988[41], 3}, {NULL}};
const Segment SGI313455[] = {{&S20996[30], 3}, {NULL}};
const Segment SGI313456[] = {{&S20996[8], 3}, {NULL}};
const Segment SGI313457[] = {{&S20996[13], 10}, {NULL}};
const Segment SGI313458[] = {{&S20996[42], 3}, {NULL}};
const Segment SGI313459[] = {{&S54211[7], 5}, {NULL}};
const Segment SGI313460[] = {{&S54211[0], 103}, {NULL}};
const Segment SGI313461[] = {{&S22302[8], 3}, {NULL}};
const Segment SGI313462[] = {{&S22302[13], 10}, {NULL}};
const Segment SGI313463[] = {{&S22302[42], 7}, {NULL}};
const Segment SGI313464[] = {{&S22302[35], 14}, {NULL}};
const Segment SGI313465[] = {{&S22623[30], 3}, {NULL}};
const Segment SGI313466[] = {{&S22623[8], 3}, {NULL}};
const Segment SGI313467[] = {{&S22623[13], 10}, {NULL}};
const Segment SGI313468[] = {{&S22623[42], 7}, {NULL}};
const Segment SGI313469[] = {{&S22623[35], 14}, {NULL}};
const Segment SGI313470[] = {{&S22623[59], 11}, {NULL}};
const Segment SGI313471[] = {{&S22623[112], 15}, {NULL}};
const Segment SGI313472[] = {{&S22623[59], 16}, {NULL}};
const Segment SGI313473[] = {{&S22624[30], 3}, {NULL}};
const Segment SGI313474[] = {{&S22624[8], 3}, {NULL}};
const Segment SGI313475[] = {{&S22624[13], 10}, {NULL}};
const Segment SGI313476[] = {{&S22624[42], 7}, {NULL}};
const Segment SGI313477[] = {{&S22624[35], 14}, {NULL}};
const Segment SGI313478[] = {{&S22624[64], 11}, {NULL}};
const Segment SGI313479[] = {{&S22624[64], 16}, {NULL}};
const Segment SGI313480[] = {{&S22625[30], 3}, {NULL}};
const Segment SGI313481[] = {{&S22625[8], 3}, {NULL}};
const Segment SGI313482[] = {{&S22625[13], 10}, {NULL}};
const Segment SGI313483[] = {{&S22625[42], 7}, {NULL}};
const Segment SGI313484[] = {{&S22625[35], 14}, {NULL}};
const Segment SGI313485[] = {{&S22625[71], 11}, {NULL}};
const Segment SGI313486[] = {{&S22625[51], 6}, {NULL}};
const Segment SGI313487[] = {{&S22625[71], 16}, {NULL}};
const Segment SGI313488[] = {{&S22627[30], 3}, {NULL}};
const Segment SGI313489[] = {{&S22627[8], 3}, {NULL}};
const Segment SGI313490[] = {{&S22627[13], 10}, {NULL}};
const Segment SGI313491[] = {{&S22627[42], 7}, {NULL}};
const Segment SGI313492[] = {{&S22627[35], 14}, {NULL}};
const Segment SGI313493[] = {{&S22627[71], 11}, {NULL}};
const Segment SGI313494[] = {{&S22627[124], 15}, {NULL}};
const Segment SGI313495[] = {{&S22627[51], 6}, {NULL}};
const Segment SGI313496[] = {{&S22627[71], 16}, {NULL}};
const Segment SGI313497[] = {{&S22629[30], 3}, {NULL}};
const Segment SGI313498[] = {{&S22629[8], 3}, {NULL}};
const Segment SGI313499[] = {{&S22629[13], 10}, {NULL}};
const Segment SGI313500[] = {{&S22629[42], 7}, {NULL}};
const Segment SGI313501[] = {{&S22629[35], 14}, {NULL}};
const Segment SGI313502[] = {{&S22629[72], 11}, {NULL}};
const Segment SGI313503[] = {{&S22629[125], 15}, {NULL}};
const Segment SGI313504[] = {{&S22629[72], 16}, {NULL}};
const Segment SGI313505[] = {{&S22630[30], 3}, {NULL}};
const Segment SGI313506[] = {{&S22630[8], 3}, {NULL}};
const Segment SGI313507[] = {{&S22630[13], 10}, {NULL}};
const Segment SGI313508[] = {{&S22630[42], 7}, {NULL}};
const Segment SGI313509[] = {{&S22630[35], 14}, {NULL}};
const Segment SGI313510[] = {{&S22630[66], 11}, {NULL}};
const Segment SGI313511[] = {{&S22630[119], 15}, {NULL}};
const Segment SGI313512[] = {{&S22630[51], 6}, {NULL}};
const Segment SGI313513[] = {{&S22630[66], 16}, {NULL}};
const Segment SGI313514[] = {{&S22631[30], 3}, {NULL}};
const Segment SGI313515[] = {{&S22631[8], 3}, {NULL}};
const Segment SGI313516[] = {{&S22631[13], 10}, {NULL}};
const Segment SGI313517[] = {{&S22631[42], 7}, {NULL}};
const Segment SGI313518[] = {{&S22631[35], 14}, {NULL}};
const Segment SGI313519[] = {{&S22631[59], 11}, {NULL}};
const Segment SGI313520[] = {{&S22631[59], 16}, {NULL}};
const Segment SGI313521[] = {{&S22632[30], 3}, {NULL}};
const Segment SGI313522[] = {{&S22632[8], 3}, {NULL}};
const Segment SGI313523[] = {{&S22632[13], 10}, {NULL}};
const Segment SGI313524[] = {{&S22632[42], 7}, {NULL}};
const Segment SGI313525[] = {{&S22632[35], 14}, {NULL}};
const Segment SGI313526[] = {{&S22632[64], 11}, {NULL}};
const Segment SGI313527[] = {{&S22632[64], 16}, {NULL}};
const Segment SGI313528[] = {{&S22633[30], 3}, {NULL}};
const Segment SGI313529[] = {{&S22633[8], 3}, {NULL}};
const Segment SGI313530[] = {{&S22633[13], 10}, {NULL}};
const Segment SGI313531[] = {{&S22633[42], 7}, {NULL}};
const Segment SGI313532[] = {{&S22633[35], 14}, {NULL}};
const Segment SGI313533[] = {{&S22633[66], 11}, {NULL}};
const Segment SGI313534[] = {{&S22633[119], 15}, {&S22633[119], 15}, {NULL}};
const Segment SGI313535[] = {{&S22633[51], 6}, {NULL}};
const Segment SGI313536[] = {{&S22633[66], 16}, {NULL}};
const Segment SGI313537[] = {{&S23280[30], 3}, {NULL}};
const Segment SGI313538[] = {{&S23280[8], 3}, {NULL}};
const Segment SGI313539[] = {{&S23280[13], 7}, {NULL}};
const Segment SGI313540[] = {{&S23280[42], 7}, {NULL}};
const Segment SGI313541[] = {{&S23280[35], 14}, {NULL}};
const Segment SGI313542[] = {{&S23281[30], 3}, {NULL}};
const Segment SGI313543[] = {{&S23281[8], 3}, {NULL}};
const Segment SGI313544[] = {{&S23281[13], 10}, {NULL}};
const Segment SGI313545[] = {{&S23281[35], 5}, {NULL}};
const Segment SGI313546[] = {{&S23282[30], 3}, {NULL}};
const Segment SGI313547[] = {{&S23282[8], 3}, {NULL}};
const Segment SGI313548[] = {{&S23282[13], 10}, {NULL}};
const Segment SGI313549[] = {{&S23282[42], 7}, {NULL}};
const Segment SGI313550[] = {{&S23282[35], 14}, {NULL}};
const Segment SGI313551[] = {{&S23283[30], 3}, {NULL}};
const Segment SGI313552[] = {{&S23283[8], 3}, {NULL}};
const Segment SGI313553[] = {{&S23283[13], 10}, {NULL}};
const Segment SGI313554[] = {{&S23283[35], 5}, {NULL}};
const Segment SGI313555[] = {{&S23284[30], 3}, {NULL}};
const Segment SGI313556[] = {{&S23284[8], 3}, {NULL}};
const Segment SGI313557[] = {{&S23284[13], 7}, {NULL}};
const Segment SGI313558[] = {{&S23284[42], 7}, {NULL}};
const Segment SGI313559[] = {{&S23285[30], 3}, {NULL}};
const Segment SGI313560[] = {{&S23285[8], 3}, {NULL}};
const Segment SGI313561[] = {{&S23285[13], 10}, {NULL}};
const Segment SGI313562[] = {{&S23285[42], 7}, {NULL}};
const Segment SGI313563[] = {{&S23285[35], 14}, {NULL}};
const Segment SGI313564[] = {{&S23286[30], 3}, {&S23286[30], 3}, {NULL}};
const Segment SGI313565[] = {{&S23286[8], 3}, {&S23286[8], 3}, {NULL}};
const Segment SGI313566[] = {{&S23286[13], 7}, {&S23286[70], 10}, {NULL}};
const Segment SGI313567[] = {{&S23286[42], 7}, {&S23286[99], 7}, {NULL}};
const Segment SGI313568[] = {{&S23286[35], 14}, {&S23286[92], 14}, {NULL}};
const Segment SGI313569[] = {{&S23287[33], 3}, {NULL}};
const Segment SGI313570[] = {{&S23287[8], 3}, {NULL}};
const Segment SGI313571[] = {{&S23287[13], 10}, {NULL}};
const Segment SGI313572[] = {{&S23289[30], 3}, {NULL}};
const Segment SGI313573[] = {{&S23289[8], 3}, {NULL}};
const Segment SGI313574[] = {{&S23289[13], 10}, {NULL}};
const Segment SGI313575[] = {{&S23293[30], 3}, {NULL}};
const Segment SGI313576[] = {{&S23293[8], 3}, {NULL}};
const Segment SGI313577[] = {{&S23293[13], 10}, {NULL}};
const Segment SGI313578[] = {{&S23293[40], 6}, {NULL}};
const Segment SGI313579[] = {{&S23294[18], 3}, {NULL}};
const Segment SGI313580[] = {{&S23294[8], 3}, {NULL}};
const Segment SGI313581[] = {{&S23294[13], 8}, {NULL}};
const Segment SGI313582[] = {{&S23294[30], 7}, {NULL}};
const Segment SGI313583[] = {{&S23294[23], 14}, {NULL}};
const Segment SGI313584[] = {{&S23296[30], 3}, {NULL}};
const Segment SGI313585[] = {{&S23296[8], 3}, {NULL}};
const Segment SGI313586[] = {{&S23296[13], 10}, {NULL}};
const Segment SGI313587[] = {{&S23296[40], 5}, {NULL}};
const Segment SGI313588[] = {{&S23296[35], 3}, {NULL}};
const Segment SGI313589[] = {{&S23297[30], 3}, {NULL}};
const Segment SGI313590[] = {{&S23297[8], 3}, {NULL}};
const Segment SGI313591[] = {{&S23297[13], 10}, {NULL}};
const Segment SGI313592[] = {{&S23297[40], 3}, {NULL}};
const Segment SGI313593[] = {{&S23298[30], 3}, {NULL}};
const Segment SGI313594[] = {{&S23298[8], 3}, {NULL}};
const Segment SGI313595[] = {{&S23298[13], 10}, {NULL}};
const Segment SGI313596[] = {{&S23298[40], 6}, {NULL}};
const Segment SGI313597[] = {{&S23298[35], 3}, {NULL}};
const Segment SGI313598[] = {{&S23299[30], 3}, {NULL}};
const Segment SGI313599[] = {{&S23299[8], 3}, {NULL}};
const Segment SGI313600[] = {{&S23299[13], 10}, {NULL}};
const Segment SGI313601[] = {{&S23299[35], 3}, {NULL}};
const Segment SGI313602[] = {{&S23603[30], 3}, {NULL}};
const Segment SGI313603[] = {{&S23603[8], 3}, {NULL}};
const Segment SGI313604[] = {{&S23603[13], 10}, {NULL}};
const Segment SGI313605[] = {{&S23603[42], 7}, {NULL}};
const Segment SGI313606[] = {{&S23603[35], 14}, {NULL}};
const Segment SGI313607[] = {{&S23603[59], 11}, {NULL}};
const Segment SGI313608[] = {{&S23603[59], 16}, {NULL}};
const Segment SGI313609[] = {{&S23604[30], 3}, {NULL}};
const Segment SGI313610[] = {{&S23604[8], 3}, {NULL}};
const Segment SGI313611[] = {{&S23604[13], 10}, {NULL}};
const Segment SGI313612[] = {{&S23604[42], 7}, {NULL}};
const Segment SGI313613[] = {{&S23604[35], 14}, {NULL}};
const Segment SGI313614[] = {{&S23604[72], 11}, {NULL}};
const Segment SGI313615[] = {{&S23604[125], 15}, {NULL}};
const Segment SGI313616[] = {{&S23604[72], 16}, {NULL}};
const Segment SGI313617[] = {{&S23605[30], 3}, {NULL}};
const Segment SGI313618[] = {{&S23605[8], 3}, {NULL}};
const Segment SGI313619[] = {{&S23605[13], 10}, {NULL}};
const Segment SGI313620[] = {{&S23605[42], 7}, {NULL}};
const Segment SGI313621[] = {{&S23605[35], 14}, {NULL}};
const Segment SGI313622[] = {{&S23605[66], 11}, {NULL}};
const Segment SGI313623[] = {{&S23605[119], 15}, {NULL}};
const Segment SGI313624[] = {{&S23605[51], 6}, {NULL}};
const Segment SGI313625[] = {{&S23605[66], 16}, {NULL}};
const Segment SGI313626[] = {{&S23606[30], 3}, {NULL}};
const Segment SGI313627[] = {{&S23606[8], 3}, {NULL}};
const Segment SGI313628[] = {{&S23606[13], 10}, {NULL}};
const Segment SGI313629[] = {{&S23606[42], 7}, {NULL}};
const Segment SGI313630[] = {{&S23606[35], 14}, {NULL}};
const Segment SGI313631[] = {{&S23606[79], 11}, {NULL}};
const Segment SGI313632[] = {{&S23606[132], 15}, {NULL}};
const Segment SGI313633[] = {{&S23606[51], 6}, {NULL}};
const Segment SGI313634[] = {{&S23606[79], 16}, {NULL}};
const Segment SGI313635[] = {{&S23608[30], 3}, {NULL}};
const Segment SGI313636[] = {{&S23608[8], 3}, {NULL}};
const Segment SGI313637[] = {{&S23608[13], 10}, {NULL}};
const Segment SGI313638[] = {{&S23608[42], 7}, {NULL}};
const Segment SGI313639[] = {{&S23608[35], 14}, {NULL}};
const Segment SGI313640[] = {{&S23608[64], 11}, {NULL}};
const Segment SGI313641[] = {{&S23608[64], 16}, {NULL}};
const Segment SGI313642[] = {{&S23609[30], 3}, {NULL}};
const Segment SGI313643[] = {{&S23609[8], 3}, {NULL}};
const Segment SGI313644[] = {{&S23609[13], 10}, {NULL}};
const Segment SGI313645[] = {{&S23609[42], 7}, {NULL}};
const Segment SGI313646[] = {{&S23609[35], 14}, {NULL}};
const Segment SGI313647[] = {{&S23609[71], 11}, {NULL}};
const Segment SGI313648[] = {{&S23609[51], 6}, {NULL}};
const Segment SGI313649[] = {{&S23609[71], 16}, {NULL}};
const Segment SGI313650[] = {{&S23610[30], 3}, {NULL}};
const Segment SGI313651[] = {{&S23610[8], 3}, {NULL}};
const Segment SGI313652[] = {{&S23610[13], 10}, {NULL}};
const Segment SGI313653[] = {{&S23610[42], 7}, {NULL}};
const Segment SGI313654[] = {{&S23610[35], 14}, {NULL}};
const Segment SGI313655[] = {{&S23610[64], 11}, {NULL}};
const Segment SGI313656[] = {{&S23610[64], 16}, {NULL}};
const Segment SGI313657[] = {{&S23611[30], 3}, {NULL}};
const Segment SGI313658[] = {{&S23611[8], 3}, {NULL}};
const Segment SGI313659[] = {{&S23611[13], 10}, {NULL}};
const Segment SGI313660[] = {{&S23611[42], 7}, {NULL}};
const Segment SGI313661[] = {{&S23611[35], 14}, {NULL}};
const Segment SGI313662[] = {{&S23611[59], 11}, {NULL}};
const Segment SGI313663[] = {{&S23611[112], 15}, {NULL}};
const Segment SGI313664[] = {{&S23611[59], 16}, {NULL}};
const Segment SGI313665[] = {{&S23613[30], 3}, {&S23613[46], 3}, {NULL}};
const Segment SGI313666[] = {{&S23613[8], 3}, {&S23613[8], 3}, {NULL}};
const Segment SGI313667[] = {{&S23613[13], 10}, {&S23613[13], 10}, {NULL}};
const Segment SGI313668[] = {{&S23613[42], 7}, {&S23613[127], 7}, {NULL}};
const Segment SGI313669[] = {{&S23613[35], 14}, {&S23613[120], 14}, {NULL}};
const Segment SGI313670[] = {{&S23613[59], 11}, {NULL}};
const Segment SGI313671[] = {{&S23613[59], 16}, {NULL}};
const Segment SGI313672[] = {{&S23613[136], 3}, {NULL}};
const Segment SGI313673[] = {{&S23614[30], 3}, {NULL}};
const Segment SGI313674[] = {{&S23614[8], 3}, {NULL}};
const Segment SGI313675[] = {{&S23614[13], 10}, {NULL}};
const Segment SGI313676[] = {{&S23614[42], 7}, {NULL}};
const Segment SGI313677[] = {{&S23614[35], 14}, {NULL}};
const Segment SGI313678[] = {{&S23614[66], 11}, {NULL}};
const Segment SGI313679[] = {{&S23614[119], 15}, {&S23614[119], 15}, {NULL}};
const Segment SGI313680[] = {{&S23614[51], 6}, {NULL}};
const Segment SGI313681[] = {{&S23614[66], 16}, {NULL}};
const Segment SGI313682[] = {{&S23615[30], 3}, {&S23615[46], 3}, {NULL}};
const Segment SGI313683[] = {{&S23615[8], 3}, {&S23615[8], 3}, {NULL}};
const Segment SGI313684[] = {{&S23615[13], 10}, {&S23615[13], 10}, {NULL}};
const Segment SGI313685[] = {{&S23615[42], 7}, {&S23615[127], 7}, {NULL}};
const Segment SGI313686[] = {{&S23615[35], 14}, {&S23615[120], 14}, {NULL}};
const Segment SGI313687[] = {{&S23615[59], 11}, {NULL}};
const Segment SGI313688[] = {{&S23615[170], 15}, {NULL}};
const Segment SGI313689[] = {{&S23615[59], 16}, {NULL}};
const Segment SGI313690[] = {{&S23615[136], 3}, {NULL}};
const Segment SGI313691[] = {{&S23616[30], 3}, {&S23616[46], 3}, {NULL}};
const Segment SGI313692[] = {{&S23616[8], 3}, {&S23616[8], 3}, {NULL}};
const Segment SGI313693[] = {{&S23616[13], 10}, {&S23616[13], 10}, {NULL}};
const Segment SGI313694[] = {{&S23616[42], 7}, {&S23616[140], 7}, {NULL}};
const Segment SGI313695[] = {{&S23616[35], 14}, {&S23616[133], 14}, {NULL}};
const Segment SGI313696[] = {{&S23616[72], 11}, {NULL}};
const Segment SGI313697[] = {{&S23616[183], 15}, {NULL}};
const Segment SGI313698[] = {{&S23616[72], 16}, {NULL}};
const Segment SGI313699[] = {{&S23616[149], 3}, {NULL}};
const Segment SGI313700[] = {{&S23899[30], 4}, {NULL}};
const Segment SGI313701[] = {{&S23899[8], 3}, {NULL}};
const Segment SGI313702[] = {{&S23899[13], 10}, {NULL}};
const Segment SGI313703[] = {{&S23899[43], 7}, {NULL}};
const Segment SGI313704[] = {{&S23899[36], 14}, {NULL}};
const Segment SGI313705[] = {{&S23899[52], 3}, {NULL}};
const Segment SGI313706[] = {{&S23900[30], 4}, {NULL}};
const Segment SGI313707[] = {{&S23900[8], 3}, {NULL}};
const Segment SGI313708[] = {{&S23900[13], 10}, {NULL}};
const Segment SGI313709[] = {{&S23900[43], 7}, {NULL}};
const Segment SGI313710[] = {{&S23900[36], 14}, {NULL}};
const Segment SGI313711[] = {{&S23900[52], 3}, {NULL}};
const Segment SGI313712[] = {{&S23900[70], 5}, {NULL}};
const Segment SGI313713[] = {{&S23901[30], 4}, {NULL}};
const Segment SGI313714[] = {{&S23901[8], 3}, {NULL}};
const Segment SGI313715[] = {{&S23901[13], 10}, {NULL}};
const Segment SGI313716[] = {{&S23901[43], 7}, {NULL}};
const Segment SGI313717[] = {{&S23901[36], 14}, {NULL}};
const Segment SGI313718[] = {{&S23901[52], 3}, {&S23901[52], 3}, {NULL}};
const Segment SGI313719[] = {{&S23901[70], 5}, {NULL}};
const Segment SGI313720[] = {{&S23902[30], 4}, {NULL}};
const Segment SGI313721[] = {{&S23902[8], 3}, {NULL}};
const Segment SGI313722[] = {{&S23902[13], 10}, {NULL}};
const Segment SGI313723[] = {{&S23902[43], 7}, {NULL}};
const Segment SGI313724[] = {{&S23902[36], 14}, {NULL}};
const Segment SGI313725[] = {{&S23902[52], 3}, {&S23902[52], 3}, {NULL}};
const Segment SGI313726[] = {{&S23903[30], 4}, {NULL}};
const Segment SGI313727[] = {{&S23903[8], 3}, {NULL}};
const Segment SGI313728[] = {{&S23903[13], 10}, {NULL}};
const Segment SGI313729[] = {{&S23903[43], 7}, {NULL}};
const Segment SGI313730[] = {{&S23903[36], 14}, {NULL}};
const Segment SGI313731[] = {{&S23903[52], 3}, {NULL}};
const Segment SGI313732[] = {{&S23903[70], 5}, {NULL}};
const Segment SGI313733[] = {{&S23903[97], 9}, {NULL}};
const Segment SGI313734[] = {{&S24010[30], 3}, {NULL}};
const Segment SGI313735[] = {{&S24010[8], 3}, {NULL}};
const Segment SGI313736[] = {{&S24010[13], 10}, {NULL}};
const Segment SGI313737[] = {{&S24010[42], 3}, {NULL}};
const Segment SGI313738[] = {{&S24011[30], 4}, {NULL}};
const Segment SGI313739[] = {{&S24011[8], 3}, {NULL}};
const Segment SGI313740[] = {{&S24011[13], 10}, {NULL}};
const Segment SGI313741[] = {{&S24011[43], 3}, {NULL}};
const Segment SGI313742[] = {{&S24012[30], 4}, {NULL}};
const Segment SGI313743[] = {{&S24012[8], 3}, {NULL}};
const Segment SGI313744[] = {{&S24012[13], 10}, {NULL}};
const Segment SGI313745[] = {{&S24012[43], 3}, {NULL}};
const Segment SGI313746[] = {{&S24013[30], 3}, {NULL}};
const Segment SGI313747[] = {{&S24013[8], 3}, {NULL}};
const Segment SGI313748[] = {{&S24013[13], 10}, {NULL}};
const Segment SGI313749[] = {{&S24013[35], 5}, {NULL}};
const Segment SGI313750[] = {{&S24014[29], 3}, {NULL}};
const Segment SGI313751[] = {{&S24014[8], 3}, {NULL}};
const Segment SGI313752[] = {{&S24014[13], 10}, {NULL}};
const Segment SGI313753[] = {{&S24014[42], 7}, {NULL}};
const Segment SGI313754[] = {{&S24014[35], 14}, {NULL}};
const Segment SGI313755[] = {{&S24015[29], 3}, {NULL}};
const Segment SGI313756[] = {{&S24015[8], 3}, {NULL}};
const Segment SGI313757[] = {{&S24015[13], 10}, {NULL}};
const Segment SGI313758[] = {{&S24015[42], 7}, {NULL}};
const Segment SGI313759[] = {{&S24015[35], 14}, {NULL}};
const Segment SGI313760[] = {{&S24016[8], 3}, {NULL}};
const Segment SGI313761[] = {{&S24016[13], 10}, {NULL}};
const Segment SGI313762[] = {{&S24017[8], 3}, {NULL}};
const Segment SGI313763[] = {{&S24017[13], 10}, {NULL}};
const Segment SGI313764[] = {{&S24017[40], 6}, {NULL}};
const Segment SGI313765[] = {{&S24018[8], 3}, {NULL}};
const Segment SGI313766[] = {{&S24018[13], 10}, {NULL}};
const Segment SGI313767[] = {{&S24018[40], 5}, {NULL}};
const Segment SGI313768[] = {{&S24018[35], 3}, {NULL}};
const Segment SGI313769[] = {{&S24019[8], 3}, {NULL}};
const Segment SGI313770[] = {{&S24019[13], 10}, {NULL}};
const Segment SGI313771[] = {{&S24019[40], 3}, {NULL}};
const Segment SGI313772[] = {{&S24020[8], 3}, {NULL}};
const Segment SGI313773[] = {{&S24020[13], 10}, {NULL}};
const Segment SGI313774[] = {{&S24020[35], 3}, {NULL}};
const Segment SGI313775[] = {{&S24021[8], 3}, {NULL}};
const Segment SGI313776[] = {{&S24021[13], 10}, {NULL}};
const Segment SGI313777[] = {{&S24021[40], 6}, {NULL}};
const Segment SGI313778[] = {{&S24021[35], 3}, {NULL}};
const Segment SGI313779[] = {{&S24022[8], 3}, {NULL}};
const Segment SGI313780[] = {{&S24022[13], 10}, {NULL}};
const Segment SGI313781[] = {{&S24022[40], 5}, {NULL}};
const Segment SGI313782[] = {{&S51723[3], 5}, {NULL}};
const Segment SGI313783[] = {{&S24095[30], 3}, {NULL}};
const Segment SGI313784[] = {{&S24095[8], 3}, {NULL}};
const Segment SGI313785[] = {{&S24095[13], 10}, {NULL}};
const Segment SGI313786[] = {{&S24095[42], 7}, {NULL}};
const Segment SGI313787[] = {{&S24095[35], 14}, {NULL}};
const Segment SGI313788[] = {{&S24095[59], 11}, {NULL}};
const Segment SGI313789[] = {{&S24095[59], 16}, {NULL}};
const Segment SGI313790[] = {{&S24100[30], 3}, {NULL}};
const Segment SGI313791[] = {{&S24100[8], 3}, {NULL}};
const Segment SGI313792[] = {{&S24100[13], 10}, {NULL}};
const Segment SGI313793[] = {{&S24100[42], 7}, {NULL}};
const Segment SGI313794[] = {{&S24100[35], 14}, {NULL}};
const Segment SGI313795[] = {{&S24100[79], 11}, {NULL}};
const Segment SGI313796[] = {{&S24100[133], 15}, {NULL}};
const Segment SGI313797[] = {{&S24100[51], 6}, {NULL}};
const Segment SGI313798[] = {{&S24100[79], 16}, {NULL}};
const Segment SGI313799[] = {{&S24102[30], 3}, {NULL}};
const Segment SGI313800[] = {{&S24102[8], 3}, {NULL}};
const Segment SGI313801[] = {{&S24102[13], 10}, {NULL}};
const Segment SGI313802[] = {{&S24102[42], 7}, {NULL}};
const Segment SGI313803[] = {{&S24102[35], 14}, {NULL}};
const Segment SGI313804[] = {{&S24102[59], 11}, {NULL}};
const Segment SGI313805[] = {{&S24102[113], 15}, {NULL}};
const Segment SGI313806[] = {{&S24102[59], 16}, {NULL}};
const Segment SGI313807[] = {{&S24103[30], 3}, {NULL}};
const Segment SGI313808[] = {{&S24103[8], 3}, {NULL}};
const Segment SGI313809[] = {{&S24103[13], 10}, {NULL}};
const Segment SGI313810[] = {{&S24103[42], 7}, {NULL}};
const Segment SGI313811[] = {{&S24103[35], 14}, {NULL}};
const Segment SGI313812[] = {{&S24103[72], 11}, {NULL}};
const Segment SGI313813[] = {{&S24103[126], 15}, {NULL}};
const Segment SGI313814[] = {{&S24103[72], 16}, {NULL}};
const Segment SGI313815[] = {{&S24104[30], 3}, {NULL}};
const Segment SGI313816[] = {{&S24104[8], 3}, {NULL}};
const Segment SGI313817[] = {{&S24104[13], 10}, {NULL}};
const Segment SGI313818[] = {{&S24104[42], 7}, {NULL}};
const Segment SGI313819[] = {{&S24104[35], 14}, {NULL}};
const Segment SGI313820[] = {{&S24104[66], 11}, {NULL}};
const Segment SGI313821[] = {{&S24104[120], 15}, {NULL}};
const Segment SGI313822[] = {{&S24104[51], 6}, {NULL}};
const Segment SGI313823[] = {{&S24104[66], 16}, {NULL}};
const Segment SGI313824[] = {{&S24105[30], 3}, {&S24105[46], 3}, {NULL}};
const Segment SGI313825[] = {{&S24105[8], 3}, {&S24105[8], 3}, {NULL}};
const Segment SGI313826[] = {{&S24105[13], 10}, {&S24105[13], 10}, {NULL}};
const Segment SGI313827[] = {{&S24105[42], 7}, {&S24105[128], 7}, {NULL}};
const Segment SGI313828[] = {{&S24105[35], 14}, {&S24105[121], 14}, {NULL}};
const Segment SGI313829[] = {{&S24105[59], 11}, {NULL}};
const Segment SGI313830[] = {{&S24105[171], 15}, {NULL}};
const Segment SGI313831[] = {{&S24105[59], 16}, {NULL}};
const Segment SGI313832[] = {{&S24105[137], 3}, {NULL}};
const Segment SGI313833[] = {{&S24106[30], 3}, {NULL}};
const Segment SGI313834[] = {{&S24106[8], 3}, {NULL}};
const Segment SGI313835[] = {{&S24106[13], 10}, {NULL}};
const Segment SGI313836[] = {{&S24106[42], 7}, {NULL}};
const Segment SGI313837[] = {{&S24106[35], 14}, {NULL}};
const Segment SGI313838[] = {{&S24106[64], 11}, {NULL}};
const Segment SGI313839[] = {{&S24106[64], 16}, {NULL}};
const Segment SGI313840[] = {{&S24107[30], 3}, {NULL}};
const Segment SGI313841[] = {{&S24107[8], 3}, {NULL}};
const Segment SGI313842[] = {{&S24107[13], 10}, {NULL}};
const Segment SGI313843[] = {{&S24107[42], 7}, {NULL}};
const Segment SGI313844[] = {{&S24107[35], 14}, {NULL}};
const Segment SGI313845[] = {{&S24107[71], 11}, {NULL}};
const Segment SGI313846[] = {{&S24107[51], 6}, {NULL}};
const Segment SGI313847[] = {{&S24107[71], 16}, {NULL}};
const Segment SGI313848[] = {{&S24110[30], 3}, {NULL}};
const Segment SGI313849[] = {{&S24110[8], 3}, {NULL}};
const Segment SGI313850[] = {{&S24110[13], 10}, {NULL}};
const Segment SGI313851[] = {{&S24110[42], 7}, {NULL}};
const Segment SGI313852[] = {{&S24110[35], 14}, {NULL}};
const Segment SGI313853[] = {{&S24110[64], 11}, {NULL}};
const Segment SGI313854[] = {{&S24110[64], 16}, {NULL}};
const Segment SGI313855[] = {{&S24111[30], 3}, {NULL}};
const Segment SGI313856[] = {{&S24111[8], 3}, {NULL}};
const Segment SGI313857[] = {{&S24111[13], 10}, {NULL}};
const Segment SGI313858[] = {{&S24111[42], 7}, {NULL}};
const Segment SGI313859[] = {{&S24111[35], 14}, {NULL}};
const Segment SGI313860[] = {{&S24111[58], 11}, {NULL}};
const Segment SGI313861[] = {{&S24111[120], 15}, {&S24111[120], 15}, {NULL}};
const Segment SGI313862[] = {{&S24111[51], 6}, {NULL}};
const Segment SGI313863[] = {{&S24111[58], 16}, {NULL}};
const Segment SGI313864[] = {{&S25148[30], 3}, {NULL}};
const Segment SGI313865[] = {{&S25148[8], 3}, {NULL}};
const Segment SGI313866[] = {{&S25148[13], 10}, {NULL}};
const Segment SGI313867[] = {{&S25148[42], 7}, {NULL}};
const Segment SGI313868[] = {{&S25148[35], 14}, {NULL}};
const Segment SGI313869[] = {{&S25148[76], 7}, {NULL}};
const Segment SGI313870[] = {{&S25148[64], 4}, {NULL}};
const Segment SGI313871[] = {{&S25148[51], 3}, {NULL}};
const Segment SGI313872[] = {{&S25149[30], 3}, {&S25149[81], 3}, {NULL}};
const Segment SGI313873[] = {{&S25149[8], 3}, {&S25149[8], 3}, {NULL}};
const Segment SGI313874[] = {{&S25149[13], 10}, {&S25149[13], 10}, {NULL}};
const Segment SGI313875[] = {{&S25149[42], 7}, {&S25149[42], 7}, {NULL}};
const Segment SGI313876[] = {{&S25149[35], 14}, {&S25149[35], 14}, {NULL}};
const Segment SGI313877[] = {{&S25149[148], 7}, {NULL}};
const Segment SGI313878[] = {{&S25149[136], 4}, {NULL}};
const Segment SGI313879[] = {{&S25149[102], 3}, {NULL}};
const Segment SGI313880[] = {{&S25150[30], 3}, {NULL}};
const Segment SGI313881[] = {{&S25150[8], 3}, {NULL}};
const Segment SGI313882[] = {{&S25150[13], 10}, {NULL}};
const Segment SGI313883[] = {{&S25150[42], 7}, {NULL}};
const Segment SGI313884[] = {{&S25150[35], 14}, {NULL}};
const Segment SGI313885[] = {{&S25150[51], 7}, {NULL}};
const Segment SGI313886[] = {{&S25150[58], 4}, {NULL}};
const Segment SGI313887[] = {{&S25151[30], 3}, {NULL}};
const Segment SGI313888[] = {{&S25151[8], 3}, {NULL}};
const Segment SGI313889[] = {{&S25151[13], 10}, {NULL}};
const Segment SGI313890[] = {{&S25151[42], 7}, {NULL}};
const Segment SGI313891[] = {{&S25151[35], 14}, {NULL}};
const Segment SGI313892[] = {{&S25151[71], 7}, {NULL}};
const Segment SGI313893[] = {{&S25151[59], 4}, {NULL}};
const Segment SGI313894[] = {{&S25152[30], 3}, {NULL}};
const Segment SGI313895[] = {{&S25152[8], 3}, {NULL}};
const Segment SGI313896[] = {{&S25152[13], 10}, {NULL}};
const Segment SGI313897[] = {{&S25152[42], 7}, {NULL}};
const Segment SGI313898[] = {{&S25152[35], 14}, {NULL}};
const Segment SGI313899[] = {{&S25152[75], 7}, {NULL}};
const Segment SGI313900[] = {{&S25152[82], 4}, {NULL}};
const Segment SGI313901[] = {{&S25152[51], 3}, {NULL}};
const Segment SGI313902[] = {{&S25153[30], 3}, {&S25153[30], 3}, {NULL}};
const Segment SGI313903[] = {{&S25153[8], 3}, {&S25153[8], 3}, {NULL}};
const Segment SGI313904[] = {{&S25153[13], 10}, {&S25153[13], 10}, {NULL}};
const Segment SGI313905[] = {{&S25153[42], 7}, {&S25153[42], 7}, {NULL}};
const Segment SGI313906[] = {{&S25153[35], 14}, {&S25153[35], 14}, {NULL}};
const Segment SGI313907[] = {{&S25153[134], 7}, {NULL}};
const Segment SGI313908[] = {{&S25153[122], 4}, {NULL}};
const Segment SGI313909[] = {{&S25153[51], 3}, {&S25153[51], 3}, {NULL}};
const Segment SGI313910[] = {{&S25154[30], 3}, {&S25154[30], 3}, {NULL}};
const Segment SGI313911[] = {{&S25154[8], 3}, {&S25154[8], 3}, {NULL}};
const Segment SGI313912[] = {{&S25154[13], 10}, {&S25154[13], 10}, {NULL}};
const Segment SGI313913[] = {{&S25154[42], 7}, {&S25154[42], 7}, {NULL}};
const Segment SGI313914[] = {{&S25154[35], 14}, {&S25154[35], 14}, {NULL}};
const Segment SGI313915[] = {{&S25154[114], 7}, {NULL}};
const Segment SGI313916[] = {{&S25154[121], 4}, {NULL}};
const Segment SGI313917[] = {{&S25154[51], 3}, {&S25154[51], 3}, {NULL}};
const Segment SGI313918[] = {{&S25155[30], 3}, {&S25155[94], 3}, {NULL}};
const Segment SGI313919[] = {{&S25155[8], 3}, {&S25155[8], 3}, {NULL}};
const Segment SGI313920[] = {{&S25155[13], 10}, {&S25155[13], 10}, {NULL}};
const Segment SGI313921[] = {{&S25155[42], 7}, {&S25155[42], 7}, {NULL}};
const Segment SGI313922[] = {{&S25155[35], 14}, {&S25155[35], 14}, {NULL}};
const Segment SGI313923[] = {{&S25155[142], 7}, {NULL}};
const Segment SGI313924[] = {{&S25155[130], 4}, {NULL}};
const Segment SGI313925[] = {{&S25155[115], 3}, {NULL}};
const Segment SGI313926[] = {{&S25156[30], 3}, {NULL}};
const Segment SGI313927[] = {{&S25156[8], 3}, {NULL}};
const Segment SGI313928[] = {{&S25156[13], 10}, {NULL}};
const Segment SGI313929[] = {{&S25156[42], 7}, {NULL}};
const Segment SGI313930[] = {{&S25156[35], 14}, {NULL}};
const Segment SGI313931[] = {{&S25156[84], 7}, {NULL}};
const Segment SGI313932[] = {{&S25156[72], 4}, {NULL}};
const Segment SGI313933[] = {{&S25157[30], 3}, {NULL}};
const Segment SGI313934[] = {{&S25157[8], 3}, {NULL}};
const Segment SGI313935[] = {{&S25157[13], 10}, {NULL}};
const Segment SGI313936[] = {{&S25157[42], 7}, {NULL}};
const Segment SGI313937[] = {{&S25157[35], 14}, {NULL}};
const Segment SGI313938[] = {{&S25157[64], 7}, {NULL}};
const Segment SGI313939[] = {{&S25157[71], 4}, {NULL}};
const Segment SGI313940[] = {{&S25158[30], 3}, {NULL}};
const Segment SGI313941[] = {{&S25158[8], 3}, {NULL}};
const Segment SGI313942[] = {{&S25158[13], 10}, {NULL}};
const Segment SGI313943[] = {{&S25158[42], 7}, {NULL}};
const Segment SGI313944[] = {{&S25158[35], 14}, {NULL}};
const Segment SGI313945[] = {{&S25158[95], 7}, {NULL}};
const Segment SGI313946[] = {{&S25158[83], 4}, {NULL}};
const Segment SGI313947[] = {{&S25158[51], 3}, {NULL}};
const Segment SGI313948[] = {{&S25159[30], 3}, {NULL}};
const Segment SGI313949[] = {{&S25159[8], 3}, {NULL}};
const Segment SGI313950[] = {{&S25159[13], 10}, {NULL}};
const Segment SGI313951[] = {{&S25159[42], 7}, {NULL}};
const Segment SGI313952[] = {{&S25159[35], 14}, {NULL}};
const Segment SGI313953[] = {{&S25159[64], 7}, {NULL}};
const Segment SGI313954[] = {{&S25159[71], 4}, {NULL}};
const Segment SGI313955[] = {{&S25160[30], 3}, {NULL}};
const Segment SGI313956[] = {{&S25160[8], 3}, {NULL}};
const Segment SGI313957[] = {{&S25160[13], 10}, {NULL}};
const Segment SGI313958[] = {{&S25160[42], 7}, {NULL}};
const Segment SGI313959[] = {{&S25160[35], 14}, {NULL}};
const Segment SGI313960[] = {{&S25160[64], 7}, {&S25160[64], 7}, {NULL}};
const Segment SGI313961[] = {{&S25160[81], 4}, {NULL}};
const Segment SGI313962[] = {{&S25161[30], 3}, {NULL}};
const Segment SGI313963[] = {{&S25161[8], 3}, {NULL}};
const Segment SGI313964[] = {{&S25161[13], 10}, {NULL}};
const Segment SGI313965[] = {{&S25161[42], 7}, {NULL}};
const Segment SGI313966[] = {{&S25161[35], 14}, {NULL}};
const Segment SGI313967[] = {{&S25161[77], 15}, {NULL}};
const Segment SGI313968[] = {{&S25161[125], 7}, {NULL}};
const Segment SGI313969[] = {{&S25161[132], 4}, {NULL}};
const Segment SGI313970[] = {{&S25162[30], 3}, {NULL}};
const Segment SGI313971[] = {{&S25162[8], 3}, {NULL}};
const Segment SGI313972[] = {{&S25162[13], 10}, {NULL}};
const Segment SGI313973[] = {{&S25162[42], 7}, {NULL}};
const Segment SGI313974[] = {{&S25162[35], 14}, {NULL}};
const Segment SGI313975[] = {{&S25162[86], 7}, {NULL}};
const Segment SGI313976[] = {{&S25162[51], 13}, {NULL}};
const Segment SGI313977[] = {{&S25162[74], 4}, {NULL}};
const Segment SGI313978[] = {{&S25163[30], 3}, {NULL}};
const Segment SGI313979[] = {{&S25163[8], 3}, {NULL}};
const Segment SGI313980[] = {{&S25163[13], 10}, {NULL}};
const Segment SGI313981[] = {{&S25163[42], 7}, {NULL}};
const Segment SGI313982[] = {{&S25163[35], 14}, {NULL}};
const Segment SGI313983[] = {{&S25163[147], 7}, {NULL}};
const Segment SGI313984[] = {{&S25163[91], 4}, {NULL}};
const Segment SGI313985[] = {{&S25164[30], 3}, {&S25164[94], 3}, {NULL}};
const Segment SGI313986[] = {{&S25164[8], 3}, {&S25164[8], 3}, {NULL}};
const Segment SGI313987[] = {{&S25164[13], 10}, {&S25164[13], 10}, {NULL}};
const Segment SGI313988[] = {{&S25164[42], 7}, {&S25164[42], 7}, {NULL}};
const Segment SGI313989[] = {{&S25164[35], 14}, {&S25164[35], 14}, {NULL}};
const Segment SGI313990[] = {{&S25164[161], 7}, {NULL}};
const Segment SGI313991[] = {{&S25164[149], 4}, {NULL}};
const Segment SGI313992[] = {{&S25164[115], 3}, {NULL}};
const Segment SGI313993[] = {{&S25165[30], 3}, {NULL}};
const Segment SGI313994[] = {{&S25165[8], 3}, {NULL}};
const Segment SGI313995[] = {{&S25165[13], 10}, {NULL}};
const Segment SGI313996[] = {{&S25165[42], 7}, {NULL}};
const Segment SGI313997[] = {{&S25165[35], 14}, {NULL}};
const Segment SGI313998[] = {{&S25165[147], 7}, {NULL}};
const Segment SGI313999[] = {{&S25165[91], 4}, {NULL}};
const Segment SGI314000[] = {{&S25166[30], 3}, {NULL}};
const Segment SGI314001[] = {{&S25166[8], 3}, {NULL}};
const Segment SGI314002[] = {{&S25166[13], 10}, {NULL}};
const Segment SGI314003[] = {{&S25166[42], 7}, {NULL}};
const Segment SGI314004[] = {{&S25166[35], 14}, {NULL}};
const Segment SGI314005[] = {{&S25166[64], 7}, {&S25166[137], 7}, {NULL}};
const Segment SGI314006[] = {{&S25166[81], 4}, {NULL}};
const Segment SGI314007[] = {{&S25167[30], 3}, {NULL}};
const Segment SGI314008[] = {{&S25167[8], 3}, {NULL}};
const Segment SGI314009[] = {{&S25167[13], 10}, {NULL}};
const Segment SGI314010[] = {{&S25167[42], 7}, {NULL}};
const Segment SGI314011[] = {{&S25167[35], 14}, {NULL}};
const Segment SGI314012[] = {{&S25167[75], 7}, {NULL}};
const Segment SGI314013[] = {{&S25167[82], 4}, {NULL}};
const Segment SGI314014[] = {{&S25167[51], 3}, {NULL}};
const Segment SGI314015[] = {{&S25168[30], 3}, {NULL}};
const Segment SGI314016[] = {{&S25168[8], 3}, {NULL}};
const Segment SGI314017[] = {{&S25168[13], 10}, {NULL}};
const Segment SGI314018[] = {{&S25168[42], 7}, {NULL}};
const Segment SGI314019[] = {{&S25168[35], 14}, {NULL}};
const Segment SGI314020[] = {{&S25168[151], 7}, {&S25168[162], 7}, {NULL}};
const Segment SGI314021[] = {{&S25168[72], 4}, {NULL}};
const Segment SGI314022[] = {{&S25169[30], 3}, {&S25169[94], 3}, {NULL}};
const Segment SGI314023[] = {{&S25169[8], 3}, {&S25169[8], 3}, {NULL}};
const Segment SGI314024[] = {{&S25169[13], 10}, {&S25169[13], 10}, {NULL}};
const Segment SGI314025[] = {{&S25169[42], 7}, {&S25169[42], 7}, {NULL}};
const Segment SGI314026[] = {{&S25169[35], 14}, {&S25169[35], 14}, {NULL}};
const Segment SGI314027[] = {{&S25169[122], 7}, {NULL}};
const Segment SGI314028[] = {{&S25169[129], 4}, {NULL}};
const Segment SGI314029[] = {{&S25169[115], 3}, {NULL}};
const Segment SGI314030[] = {{&S25170[30], 3}, {NULL}};
const Segment SGI314031[] = {{&S25170[8], 3}, {NULL}};
const Segment SGI314032[] = {{&S25170[13], 10}, {NULL}};
const Segment SGI314033[] = {{&S25170[42], 7}, {NULL}};
const Segment SGI314034[] = {{&S25170[35], 14}, {NULL}};
const Segment SGI314035[] = {{&S25170[56], 7}, {&S25170[65], 7}, {NULL}};
const Segment SGI314036[] = {{&S25170[72], 4}, {NULL}};
const Segment SGI314037[] = {{&S25170[51], 3}, {NULL}};
const Segment SGI314038[] = {{&S25171[30], 3}, {NULL}};
const Segment SGI314039[] = {{&S25171[8], 3}, {NULL}};
const Segment SGI314040[] = {{&S25171[13], 10}, {NULL}};
const Segment SGI314041[] = {{&S25171[42], 7}, {NULL}};
const Segment SGI314042[] = {{&S25171[35], 14}, {NULL}};
const Segment SGI314043[] = {{&S25171[56], 7}, {&S25171[56], 7}, {NULL}};
const Segment SGI314044[] = {{&S25171[73], 4}, {NULL}};
const Segment SGI314045[] = {{&S25171[51], 3}, {NULL}};
const Segment SGI314046[] = {{&S25172[30], 3}, {NULL}};
const Segment SGI314047[] = {{&S25172[8], 3}, {NULL}};
const Segment SGI314048[] = {{&S25172[13], 10}, {NULL}};
const Segment SGI314049[] = {{&S25172[42], 7}, {NULL}};
const Segment SGI314050[] = {{&S25172[35], 14}, {NULL}};
const Segment SGI314051[] = {{&S25172[69], 7}, {NULL}};
const Segment SGI314052[] = {{&S25172[76], 4}, {NULL}};
const Segment SGI314053[] = {{&S25172[64], 3}, {NULL}};
const Segment SGI314054[] = {{&S25173[30], 3}, {NULL}};
const Segment SGI314055[] = {{&S25173[8], 3}, {NULL}};
const Segment SGI314056[] = {{&S25173[13], 10}, {NULL}};
const Segment SGI314057[] = {{&S25173[42], 7}, {NULL}};
const Segment SGI314058[] = {{&S25173[35], 14}, {NULL}};
const Segment SGI314059[] = {{&S25173[134], 7}, {NULL}};
const Segment SGI314060[] = {{&S25173[78], 4}, {NULL}};
const Segment SGI314061[] = {{&S25174[30], 3}, {NULL}};
const Segment SGI314062[] = {{&S25174[8], 3}, {NULL}};
const Segment SGI314063[] = {{&S25174[13], 10}, {NULL}};
const Segment SGI314064[] = {{&S25174[42], 7}, {NULL}};
const Segment SGI314065[] = {{&S25174[35], 14}, {NULL}};
const Segment SGI314066[] = {{&S25174[102], 15}, {NULL}};
const Segment SGI314067[] = {{&S25174[123], 7}, {NULL}};
const Segment SGI314068[] = {{&S25174[130], 4}, {NULL}};
const Segment SGI314069[] = {{&S25175[30], 3}, {NULL}};
const Segment SGI314070[] = {{&S25175[8], 3}, {NULL}};
const Segment SGI314071[] = {{&S25175[13], 10}, {NULL}};
const Segment SGI314072[] = {{&S25175[42], 7}, {NULL}};
const Segment SGI314073[] = {{&S25175[35], 14}, {NULL}};
const Segment SGI314074[] = {{&S25175[133], 7}, {NULL}};
const Segment SGI314075[] = {{&S25175[77], 4}, {NULL}};
const Segment SGI314076[] = {{&S25175[64], 3}, {NULL}};
const Segment SGI314077[] = {{&S25176[30], 3}, {NULL}};
const Segment SGI314078[] = {{&S25176[8], 3}, {NULL}};
const Segment SGI314079[] = {{&S25176[13], 10}, {NULL}};
const Segment SGI314080[] = {{&S25176[42], 7}, {NULL}};
const Segment SGI314081[] = {{&S25176[35], 14}, {NULL}};
const Segment SGI314082[] = {{&S25176[89], 15}, {NULL}};
const Segment SGI314083[] = {{&S25176[144], 7}, {NULL}};
const Segment SGI314084[] = {{&S25176[118], 4}, {NULL}};
const Segment SGI314085[] = {{&S25177[30], 3}, {NULL}};
const Segment SGI314086[] = {{&S25177[8], 3}, {NULL}};
const Segment SGI314087[] = {{&S25177[13], 10}, {NULL}};
const Segment SGI314088[] = {{&S25177[42], 7}, {NULL}};
const Segment SGI314089[] = {{&S25177[35], 14}, {NULL}};
const Segment SGI314090[] = {{&S25177[69], 7}, {NULL}};
const Segment SGI314091[] = {{&S25177[76], 4}, {NULL}};
const Segment SGI314092[] = {{&S25177[64], 3}, {NULL}};
const Segment SGI314093[] = {{&S25185[30], 3}, {NULL}};
const Segment SGI314094[] = {{&S25185[8], 3}, {NULL}};
const Segment SGI314095[] = {{&S25185[13], 10}, {NULL}};
const Segment SGI314096[] = {{&S25185[42], 7}, {NULL}};
const Segment SGI314097[] = {{&S25185[35], 14}, {NULL}};
const Segment SGI314098[] = {{&S25185[64], 11}, {NULL}};
const Segment SGI314099[] = {{&S25185[64], 16}, {NULL}};
const Segment SGI314100[] = {{&S25186[30], 3}, {NULL}};
const Segment SGI314101[] = {{&S25186[8], 3}, {NULL}};
const Segment SGI314102[] = {{&S25186[13], 10}, {NULL}};
const Segment SGI314103[] = {{&S25186[42], 7}, {NULL}};
const Segment SGI314104[] = {{&S25186[35], 14}, {NULL}};
const Segment SGI314105[] = {{&S25186[79], 11}, {NULL}};
const Segment SGI314106[] = {{&S25186[132], 15}, {NULL}};
const Segment SGI314107[] = {{&S25186[51], 6}, {NULL}};
const Segment SGI314108[] = {{&S25186[79], 16}, {NULL}};
const Segment SGI314109[] = {{&S25188[30], 3}, {NULL}};
const Segment SGI314110[] = {{&S25188[8], 3}, {NULL}};
const Segment SGI314111[] = {{&S25188[13], 10}, {NULL}};
const Segment SGI314112[] = {{&S25188[42], 7}, {NULL}};
const Segment SGI314113[] = {{&S25188[35], 14}, {NULL}};
const Segment SGI314114[] = {{&S25188[71], 11}, {NULL}};
const Segment SGI314115[] = {{&S25188[51], 6}, {NULL}};
const Segment SGI314116[] = {{&S25188[71], 16}, {NULL}};
const Segment SGI314117[] = {{&S25190[30], 3}, {NULL}};
const Segment SGI314118[] = {{&S25190[8], 3}, {NULL}};
const Segment SGI314119[] = {{&S25190[13], 10}, {NULL}};
const Segment SGI314120[] = {{&S25190[42], 7}, {NULL}};
const Segment SGI314121[] = {{&S25190[35], 14}, {NULL}};
const Segment SGI314122[] = {{&S25190[51], 11}, {NULL}};
const Segment SGI314123[] = {{&S25190[104], 15}, {NULL}};
const Segment SGI314124[] = {{&S25190[51], 16}, {NULL}};
const Segment SGI314125[] = {{&S25191[30], 3}, {NULL}};
const Segment SGI314126[] = {{&S25191[8], 3}, {NULL}};
const Segment SGI314127[] = {{&S25191[13], 10}, {NULL}};
const Segment SGI314128[] = {{&S25191[42], 7}, {NULL}};
const Segment SGI314129[] = {{&S25191[35], 14}, {NULL}};
const Segment SGI314130[] = {{&S25191[64], 11}, {NULL}};
const Segment SGI314131[] = {{&S25191[64], 16}, {NULL}};
const Segment SGI314132[] = {{&S25192[30], 3}, {NULL}};
const Segment SGI314133[] = {{&S25192[8], 3}, {NULL}};
const Segment SGI314134[] = {{&S25192[13], 10}, {NULL}};
const Segment SGI314135[] = {{&S25192[42], 7}, {NULL}};
const Segment SGI314136[] = {{&S25192[35], 14}, {NULL}};
const Segment SGI314137[] = {{&S25192[59], 11}, {NULL}};
const Segment SGI314138[] = {{&S25192[59], 16}, {NULL}};
const Segment SGI314139[] = {{&S25193[30], 3}, {NULL}};
const Segment SGI314140[] = {{&S25193[8], 3}, {NULL}};
const Segment SGI314141[] = {{&S25193[13], 10}, {NULL}};
const Segment SGI314142[] = {{&S25193[42], 7}, {NULL}};
const Segment SGI314143[] = {{&S25193[35], 14}, {NULL}};
const Segment SGI314144[] = {{&S25193[72], 11}, {NULL}};
const Segment SGI314145[] = {{&S25193[125], 15}, {NULL}};
const Segment SGI314146[] = {{&S25193[72], 16}, {NULL}};
const Segment SGI314147[] = {{&S25194[30], 3}, {NULL}};
const Segment SGI314148[] = {{&S25194[8], 3}, {NULL}};
const Segment SGI314149[] = {{&S25194[13], 10}, {NULL}};
const Segment SGI314150[] = {{&S25194[42], 7}, {NULL}};
const Segment SGI314151[] = {{&S25194[35], 14}, {NULL}};
const Segment SGI314152[] = {{&S25194[66], 11}, {NULL}};
const Segment SGI314153[] = {{&S25194[119], 15}, {NULL}};
const Segment SGI314154[] = {{&S25194[51], 6}, {NULL}};
const Segment SGI314155[] = {{&S25194[66], 16}, {NULL}};
const Segment SGI314156[] = {{&S25195[30], 3}, {&S25195[46], 3}, {NULL}};
const Segment SGI314157[] = {{&S25195[8], 3}, {&S25195[8], 3}, {NULL}};
const Segment SGI314158[] = {{&S25195[13], 10}, {&S25195[13], 10}, {NULL}};
const Segment SGI314159[] = {{&S25195[42], 7}, {&S25195[127], 7}, {NULL}};
const Segment SGI314160[] = {{&S25195[35], 14}, {&S25195[120], 14}, {NULL}};
const Segment SGI314161[] = {{&S25195[59], 11}, {NULL}};
const Segment SGI314162[] = {{&S25195[59], 16}, {NULL}};
const Segment SGI314163[] = {{&S25195[136], 3}, {NULL}};
const Segment SGI314164[] = {{&S25196[30], 3}, {NULL}};
const Segment SGI314165[] = {{&S25196[8], 3}, {NULL}};
const Segment SGI314166[] = {{&S25196[13], 10}, {NULL}};
const Segment SGI314167[] = {{&S25196[42], 7}, {NULL}};
const Segment SGI314168[] = {{&S25196[35], 14}, {NULL}};
const Segment SGI314169[] = {{&S25196[66], 11}, {NULL}};
const Segment SGI314170[] = {{&S25196[119], 15}, {&S25196[119], 15}, {NULL}};
const Segment SGI314171[] = {{&S25196[51], 6}, {NULL}};
const Segment SGI314172[] = {{&S25196[66], 16}, {NULL}};
const Segment SGI314173[] = {{&S25275[30], 3}, {NULL}};
const Segment SGI314174[] = {{&S25275[8], 3}, {NULL}};
const Segment SGI314175[] = {{&S25275[13], 10}, {NULL}};
const Segment SGI314176[] = {{&S25275[55], 3}, {NULL}};
const Segment SGI314177[] = {{&S25276[30], 3}, {NULL}};
const Segment SGI314178[] = {{&S25276[8], 3}, {NULL}};
const Segment SGI314179[] = {{&S25276[13], 10}, {NULL}};
const Segment SGI314180[] = {{&S25276[55], 3}, {NULL}};
const Segment SGI314181[] = {{&S25276[60], 3}, {NULL}};
const Segment SGI314182[] = {{&S25277[30], 3}, {NULL}};
const Segment SGI314183[] = {{&S25277[8], 3}, {NULL}};
const Segment SGI314184[] = {{&S25277[13], 10}, {NULL}};
const Segment SGI314185[] = {{&S25277[55], 3}, {NULL}};
const Segment SGI314186[] = {{&S25277[60], 3}, {NULL}};
const Segment SGI314187[] = {{&S25278[30], 3}, {NULL}};
const Segment SGI314188[] = {{&S25278[8], 3}, {NULL}};
const Segment SGI314189[] = {{&S25278[13], 10}, {NULL}};
const Segment SGI314190[] = {{&S25278[55], 3}, {NULL}};
const Segment SGI314191[] = {{&S25279[30], 3}, {NULL}};
const Segment SGI314192[] = {{&S25279[8], 3}, {NULL}};
const Segment SGI314193[] = {{&S25279[13], 10}, {NULL}};
const Segment SGI314194[] = {{&S25279[55], 3}, {NULL}};
const Segment SGI314195[] = {{&S25279[60], 3}, {&S25279[60], 3}, {NULL}};
const Segment SGI314196[] = {{&S25280[30], 3}, {NULL}};
const Segment SGI314197[] = {{&S25280[8], 3}, {NULL}};
const Segment SGI314198[] = {{&S25280[13], 10}, {NULL}};
const Segment SGI314199[] = {{&S25280[42], 3}, {NULL}};
const Segment SGI314200[] = {{&S25280[147], 15}, {NULL}};
const Segment SGI314201[] = {{&S25280[60], 6}, {NULL}};
const Segment SGI314202[] = {{&S25280[168], 9}, {NULL}};
const Segment SGI314203[] = {{&S25281[30], 3}, {NULL}};
const Segment SGI314204[] = {{&S25281[8], 3}, {NULL}};
const Segment SGI314205[] = {{&S25281[13], 10}, {NULL}};
const Segment SGI314206[] = {{&S25281[55], 3}, {NULL}};
const Segment SGI314207[] = {{&S25281[60], 3}, {&S25281[60], 3}, {NULL}};
const Segment SGI314208[] = {{&S25282[30], 3}, {NULL}};
const Segment SGI314209[] = {{&S25282[8], 3}, {NULL}};
const Segment SGI314210[] = {{&S25282[13], 10}, {NULL}};
const Segment SGI314211[] = {{&S25282[35], 5}, {NULL}};
const Segment SGI314212[] = {{&S25283[30], 3}, {NULL}};
const Segment SGI314213[] = {{&S25283[8], 3}, {NULL}};
const Segment SGI314214[] = {{&S25283[13], 10}, {NULL}};
const Segment SGI314215[] = {{&S25289[30], 3}, {NULL}};
const Segment SGI314216[] = {{&S25289[8], 3}, {NULL}};
const Segment SGI314217[] = {{&S25289[13], 10}, {NULL}};
const Segment SGI314218[] = {{&S25289[42], 7}, {NULL}};
const Segment SGI314219[] = {{&S25289[35], 14}, {NULL}};
const Segment SGI314220[] = {{&S25289[59], 11}, {NULL}};
const Segment SGI314221[] = {{&S25289[59], 16}, {NULL}};
const Segment SGI314222[] = {{&S25292[30], 3}, {NULL}};
const Segment SGI314223[] = {{&S25292[8], 3}, {NULL}};
const Segment SGI314224[] = {{&S25292[13], 10}, {NULL}};
const Segment SGI314225[] = {{&S25292[42], 7}, {NULL}};
const Segment SGI314226[] = {{&S25292[35], 14}, {NULL}};
const Segment SGI314227[] = {{&S25292[72], 11}, {NULL}};
const Segment SGI314228[] = {{&S25292[72], 16}, {NULL}};
const Segment SGI314229[] = {{&S25293[30], 3}, {NULL}};
const Segment SGI314230[] = {{&S25293[8], 3}, {NULL}};
const Segment SGI314231[] = {{&S25293[13], 10}, {NULL}};
const Segment SGI314232[] = {{&S25293[42], 7}, {NULL}};
const Segment SGI314233[] = {{&S25293[35], 14}, {NULL}};
const Segment SGI314234[] = {{&S25293[64], 11}, {NULL}};
const Segment SGI314235[] = {{&S25293[64], 16}, {NULL}};
const Segment SGI314236[] = {{&S25294[30], 3}, {NULL}};
const Segment SGI314237[] = {{&S25294[8], 3}, {NULL}};
const Segment SGI314238[] = {{&S25294[13], 10}, {NULL}};
const Segment SGI314239[] = {{&S25294[42], 7}, {NULL}};
const Segment SGI314240[] = {{&S25294[35], 14}, {NULL}};
const Segment SGI314241[] = {{&S25294[59], 11}, {NULL}};
const Segment SGI314242[] = {{&S25294[112], 15}, {NULL}};
const Segment SGI314243[] = {{&S25294[59], 16}, {NULL}};
const Segment SGI314244[] = {{&S25296[30], 3}, {NULL}};
const Segment SGI314245[] = {{&S25296[8], 3}, {NULL}};
const Segment SGI314246[] = {{&S25296[13], 10}, {NULL}};
const Segment SGI314247[] = {{&S25296[42], 7}, {NULL}};
const Segment SGI314248[] = {{&S25296[35], 14}, {NULL}};
const Segment SGI314249[] = {{&S25296[71], 11}, {NULL}};
const Segment SGI314250[] = {{&S25296[51], 6}, {NULL}};
const Segment SGI314251[] = {{&S25296[71], 16}, {NULL}};
const Segment SGI314252[] = {{&S25297[30], 3}, {NULL}};
const Segment SGI314253[] = {{&S25297[8], 3}, {NULL}};
const Segment SGI314254[] = {{&S25297[13], 10}, {NULL}};
const Segment SGI314255[] = {{&S25297[42], 7}, {NULL}};
const Segment SGI314256[] = {{&S25297[35], 14}, {NULL}};
const Segment SGI314257[] = {{&S25297[72], 11}, {NULL}};
const Segment SGI314258[] = {{&S25297[125], 15}, {NULL}};
const Segment SGI314259[] = {{&S25297[72], 16}, {NULL}};
const Segment SGI314260[] = {{&S25298[30], 3}, {NULL}};
const Segment SGI314261[] = {{&S25298[8], 3}, {NULL}};
const Segment SGI314262[] = {{&S25298[13], 10}, {NULL}};
const Segment SGI314263[] = {{&S25298[42], 7}, {NULL}};
const Segment SGI314264[] = {{&S25298[35], 14}, {NULL}};
const Segment SGI314265[] = {{&S25298[79], 11}, {NULL}};
const Segment SGI314266[] = {{&S25298[132], 15}, {NULL}};
const Segment SGI314267[] = {{&S25298[51], 6}, {NULL}};
const Segment SGI314268[] = {{&S25298[79], 16}, {NULL}};
const Segment SGI314269[] = {{&S25299[30], 3}, {NULL}};
const Segment SGI314270[] = {{&S25299[8], 3}, {NULL}};
const Segment SGI314271[] = {{&S25299[13], 10}, {NULL}};
const Segment SGI314272[] = {{&S25299[42], 7}, {NULL}};
const Segment SGI314273[] = {{&S25299[35], 14}, {NULL}};
const Segment SGI314274[] = {{&S25299[66], 11}, {NULL}};
const Segment SGI314275[] = {{&S25299[119], 15}, {NULL}};
const Segment SGI314276[] = {{&S25299[51], 6}, {NULL}};
const Segment SGI314277[] = {{&S25299[66], 16}, {NULL}};
const Segment SGI314278[] = {{&S25300[30], 3}, {NULL}};
const Segment SGI314279[] = {{&S25300[8], 3}, {NULL}};
const Segment SGI314280[] = {{&S25300[13], 10}, {NULL}};
const Segment SGI314281[] = {{&S25300[42], 7}, {NULL}};
const Segment SGI314282[] = {{&S25300[35], 14}, {NULL}};
const Segment SGI314283[] = {{&S25300[66], 11}, {NULL}};
const Segment SGI314284[] = {{&S25300[119], 15}, {&S25300[119], 15}, {NULL}};
const Segment SGI314285[] = {{&S25300[51], 6}, {NULL}};
const Segment SGI314286[] = {{&S25300[66], 16}, {NULL}};
const Segment SGI314287[] = {{&S25301[30], 3}, {&S25301[46], 3}, {NULL}};
const Segment SGI314288[] = {{&S25301[8], 3}, {&S25301[8], 3}, {NULL}};
const Segment SGI314289[] = {{&S25301[13], 10}, {&S25301[13], 10}, {NULL}};
const Segment SGI314290[] = {{&S25301[42], 7}, {&S25301[127], 7}, {NULL}};
const Segment SGI314291[] = {{&S25301[35], 14}, {&S25301[120], 14}, {NULL}};
const Segment SGI314292[] = {{&S25301[59], 11}, {NULL}};
const Segment SGI314293[] = {{&S25301[170], 15}, {NULL}};
const Segment SGI314294[] = {{&S25301[59], 16}, {NULL}};
const Segment SGI314295[] = {{&S25301[136], 3}, {NULL}};
const Segment SGI314296[] = {{&S25353[30], 3}, {NULL}};
const Segment SGI314297[] = {{&S25353[8], 3}, {NULL}};
const Segment SGI314298[] = {{&S25353[13], 10}, {NULL}};
const Segment SGI314299[] = {{&S25353[55], 3}, {NULL}};
const Segment SGI314300[] = {{&S25354[30], 3}, {NULL}};
const Segment SGI314301[] = {{&S25354[8], 3}, {NULL}};
const Segment SGI314302[] = {{&S25354[13], 10}, {NULL}};
const Segment SGI314303[] = {{&S25354[55], 3}, {NULL}};
const Segment SGI314304[] = {{&S25354[60], 3}, {NULL}};
const Segment SGI314305[] = {{&S25355[30], 3}, {NULL}};
const Segment SGI314306[] = {{&S25355[8], 3}, {NULL}};
const Segment SGI314307[] = {{&S25355[13], 10}, {NULL}};
const Segment SGI314308[] = {{&S25355[55], 3}, {NULL}};
const Segment SGI314309[] = {{&S25356[30], 3}, {NULL}};
const Segment SGI314310[] = {{&S25356[8], 3}, {NULL}};
const Segment SGI314311[] = {{&S25356[13], 10}, {NULL}};
const Segment SGI314312[] = {{&S25356[55], 3}, {NULL}};
const Segment SGI314313[] = {{&S25356[60], 3}, {NULL}};
const Segment SGI314314[] = {{&S25357[30], 3}, {NULL}};
const Segment SGI314315[] = {{&S25357[8], 3}, {NULL}};
const Segment SGI314316[] = {{&S25357[13], 10}, {NULL}};
const Segment SGI314317[] = {{&S25357[55], 3}, {NULL}};
const Segment SGI314318[] = {{&S25357[60], 3}, {&S25357[60], 3}, {NULL}};
const Segment SGI314319[] = {{&S25358[30], 3}, {NULL}};
const Segment SGI314320[] = {{&S25358[8], 3}, {NULL}};
const Segment SGI314321[] = {{&S25358[13], 10}, {NULL}};
const Segment SGI314322[] = {{&S25358[55], 3}, {NULL}};
const Segment SGI314323[] = {{&S25358[60], 3}, {&S25358[60], 3}, {NULL}};
const Segment SGI314324[] = {{&S25359[30], 3}, {NULL}};
const Segment SGI314325[] = {{&S25359[8], 3}, {NULL}};
const Segment SGI314326[] = {{&S25359[13], 10}, {NULL}};
const Segment SGI314327[] = {{&S25359[35], 5}, {NULL}};
const Segment SGI314328[] = {{&S25360[30], 3}, {NULL}};
const Segment SGI314329[] = {{&S25360[8], 3}, {NULL}};
const Segment SGI314330[] = {{&S25360[13], 10}, {NULL}};
const Segment SGI314331[] = {{&S25369[30], 3}, {NULL}};
const Segment SGI314332[] = {{&S25369[8], 3}, {NULL}};
const Segment SGI314333[] = {{&S25369[13], 10}, {NULL}};
const Segment SGI314334[] = {{&S25369[42], 3}, {NULL}};
const Segment SGI314335[] = {{&S25370[30], 4}, {NULL}};
const Segment SGI314336[] = {{&S25370[8], 3}, {NULL}};
const Segment SGI314337[] = {{&S25370[13], 10}, {NULL}};
const Segment SGI314338[] = {{&S25370[43], 3}, {NULL}};
const Segment SGI314339[] = {{&S25371[5], 3}, {&S25371[5], 3}, {NULL}};
const Segment SGI314340[] = {{&S25371[27], 3}, {NULL}};
const Segment SGI314341[] = {{&S25371[32], 10}, {NULL}};
const Segment SGI314342[] = {{&S25371[61], 3}, {NULL}};
const Segment SGI314343[] = {{&S25536[30], 4}, {NULL}};
const Segment SGI314344[] = {{&S25536[8], 3}, {NULL}};
const Segment SGI314345[] = {{&S25536[13], 10}, {NULL}};
const Segment SGI314346[] = {{&S25536[43], 3}, {NULL}};
const Segment SGI314347[] = {{&S25537[29], 3}, {NULL}};
const Segment SGI314348[] = {{&S25537[8], 3}, {NULL}};
const Segment SGI314349[] = {{&S25537[41], 3}, {NULL}};
const Segment SGI314350[] = {{&S25538[30], 3}, {NULL}};
const Segment SGI314351[] = {{&S25538[8], 3}, {NULL}};
const Segment SGI314352[] = {{&S25538[13], 10}, {NULL}};
const Segment SGI314353[] = {{&S25538[42], 3}, {NULL}};
const Segment SGI314354[] = {{&S25539[30], 4}, {NULL}};
const Segment SGI314355[] = {{&S25539[8], 3}, {NULL}};
const Segment SGI314356[] = {{&S25539[13], 10}, {NULL}};
const Segment SGI314357[] = {{&S25539[43], 3}, {NULL}};
const Segment SGI314358[] = {{&S25540[28], 4}, {NULL}};
const Segment SGI314359[] = {{&S25540[8], 3}, {NULL}};
const Segment SGI314360[] = {{&S25540[40], 3}, {NULL}};
const Segment SGI314361[] = {{&S25541[30], 4}, {NULL}};
const Segment SGI314362[] = {{&S25541[8], 3}, {NULL}};
const Segment SGI314363[] = {{&S25541[13], 10}, {NULL}};
const Segment SGI314364[] = {{&S25541[43], 3}, {NULL}};
const Segment SGI314365[] = {{&S25542[30], 4}, {NULL}};
const Segment SGI314366[] = {{&S25542[8], 3}, {NULL}};
const Segment SGI314367[] = {{&S25542[13], 10}, {NULL}};
const Segment SGI314368[] = {{&S25542[43], 3}, {NULL}};
const Segment SGI314369[] = {{&S25542[48], 5}, {NULL}};
const Segment SGI314370[] = {{&S25543[30], 4}, {NULL}};
const Segment SGI314371[] = {{&S25543[8], 3}, {NULL}};
const Segment SGI314372[] = {{&S25543[13], 10}, {NULL}};
const Segment SGI314373[] = {{&S25543[43], 3}, {NULL}};
const Segment SGI314374[] = {{&S25544[8], 3}, {NULL}};
const Segment SGI314375[] = {{&S25544[13], 10}, {NULL}};
const Segment SGI314376[] = {{&S25545[8], 3}, {NULL}};
const Segment SGI314377[] = {{&S25545[13], 10}, {NULL}};
const Segment SGI314378[] = {{&S25545[35], 5}, {NULL}};
const Segment SGI314379[] = {{&S25546[30], 3}, {NULL}};
const Segment SGI314380[] = {{&S25546[8], 3}, {NULL}};
const Segment SGI314381[] = {{&S25546[13], 10}, {NULL}};
const Segment SGI314382[] = {{&S25546[42], 3}, {NULL}};
const Segment SGI314383[] = {{&S25546[47], 13}, {&S25546[47], 13}, {NULL}};
const Segment SGI314384[] = {{&S59004[0], 83}, {NULL}};
const Segment SGI314385[] = {{&S26539[30], 3}, {NULL}};
const Segment SGI314386[] = {{&S26539[8], 3}, {NULL}};
const Segment SGI314387[] = {{&S26539[13], 10}, {NULL}};
const Segment SGI314388[] = {{&S26539[42], 7}, {NULL}};
const Segment SGI314389[] = {{&S26539[35], 14}, {NULL}};
const Segment SGI314390[] = {{&S26539[64], 11}, {NULL}};
const Segment SGI314391[] = {{&S26539[64], 16}, {NULL}};
const Segment SGI314392[] = {{&S26540[30], 3}, {NULL}};
const Segment SGI314393[] = {{&S26540[8], 3}, {NULL}};
const Segment SGI314394[] = {{&S26540[13], 10}, {NULL}};
const Segment SGI314395[] = {{&S26540[42], 7}, {NULL}};
const Segment SGI314396[] = {{&S26540[35], 14}, {NULL}};
const Segment SGI314397[] = {{&S26540[71], 11}, {NULL}};
const Segment SGI314398[] = {{&S26540[51], 6}, {NULL}};
const Segment SGI314399[] = {{&S26540[71], 16}, {NULL}};
const Segment SGI314400[] = {{&S26541[30], 3}, {NULL}};
const Segment SGI314401[] = {{&S26541[8], 3}, {NULL}};
const Segment SGI314402[] = {{&S26541[13], 10}, {NULL}};
const Segment SGI314403[] = {{&S26541[42], 7}, {NULL}};
const Segment SGI314404[] = {{&S26541[35], 14}, {NULL}};
const Segment SGI314405[] = {{&S26541[72], 11}, {NULL}};
const Segment SGI314406[] = {{&S26541[126], 15}, {NULL}};
const Segment SGI314407[] = {{&S26541[72], 16}, {NULL}};
const Segment SGI314408[] = {{&S26543[30], 3}, {NULL}};
const Segment SGI314409[] = {{&S26543[8], 3}, {NULL}};
const Segment SGI314410[] = {{&S26543[13], 10}, {NULL}};
const Segment SGI314411[] = {{&S26543[42], 7}, {NULL}};
const Segment SGI314412[] = {{&S26543[35], 14}, {NULL}};
const Segment SGI314413[] = {{&S26543[59], 11}, {NULL}};
const Segment SGI314414[] = {{&S26543[59], 16}, {NULL}};
const Segment SGI314415[] = {{&S26544[30], 3}, {NULL}};
const Segment SGI314416[] = {{&S26544[8], 3}, {NULL}};
const Segment SGI314417[] = {{&S26544[13], 10}, {NULL}};
const Segment SGI314418[] = {{&S26544[42], 7}, {NULL}};
const Segment SGI314419[] = {{&S26544[35], 14}, {NULL}};
const Segment SGI314420[] = {{&S26544[59], 11}, {NULL}};
const Segment SGI314421[] = {{&S26544[113], 15}, {NULL}};
const Segment SGI314422[] = {{&S26544[59], 16}, {NULL}};
const Segment SGI314423[] = {{&S26549[30], 3}, {NULL}};
const Segment SGI314424[] = {{&S26549[8], 3}, {NULL}};
const Segment SGI314425[] = {{&S26549[13], 10}, {NULL}};
const Segment SGI314426[] = {{&S26549[42], 7}, {NULL}};
const Segment SGI314427[] = {{&S26549[35], 14}, {NULL}};
const Segment SGI314428[] = {{&S26549[79], 11}, {NULL}};
const Segment SGI314429[] = {{&S26549[133], 15}, {NULL}};
const Segment SGI314430[] = {{&S26549[51], 6}, {NULL}};
const Segment SGI314431[] = {{&S26549[79], 16}, {NULL}};
const Segment SGI314432[] = {{&S26550[30], 3}, {NULL}};
const Segment SGI314433[] = {{&S26550[8], 3}, {NULL}};
const Segment SGI314434[] = {{&S26550[13], 10}, {NULL}};
const Segment SGI314435[] = {{&S26550[42], 7}, {NULL}};
const Segment SGI314436[] = {{&S26550[35], 14}, {NULL}};
const Segment SGI314437[] = {{&S26550[64], 11}, {NULL}};
const Segment SGI314438[] = {{&S26550[64], 16}, {NULL}};
const Segment SGI314439[] = {{&S26551[30], 3}, {NULL}};
const Segment SGI314440[] = {{&S26551[8], 3}, {NULL}};
const Segment SGI314441[] = {{&S26551[13], 10}, {NULL}};
const Segment SGI314442[] = {{&S26551[42], 7}, {NULL}};
const Segment SGI314443[] = {{&S26551[35], 14}, {NULL}};
const Segment SGI314444[] = {{&S26551[66], 11}, {NULL}};
const Segment SGI314445[] = {{&S26551[120], 15}, {NULL}};
const Segment SGI314446[] = {{&S26551[51], 6}, {NULL}};
const Segment SGI314447[] = {{&S26551[66], 16}, {NULL}};
const Segment SGI314448[] = {{&S26552[30], 3}, {NULL}};
const Segment SGI314449[] = {{&S26552[8], 3}, {NULL}};
const Segment SGI314450[] = {{&S26552[13], 10}, {NULL}};
const Segment SGI314451[] = {{&S26552[42], 7}, {NULL}};
const Segment SGI314452[] = {{&S26552[35], 14}, {NULL}};
const Segment SGI314453[] = {{&S26552[66], 11}, {NULL}};
const Segment SGI314454[] = {{&S26552[120], 15}, {&S26552[120], 15}, {NULL}};
const Segment SGI314455[] = {{&S26552[51], 6}, {NULL}};
const Segment SGI314456[] = {{&S26552[66], 16}, {NULL}};
const Segment SGI314457[] = {{&S26553[30], 3}, {&S26553[46], 3}, {NULL}};
const Segment SGI314458[] = {{&S26553[8], 3}, {&S26553[8], 3}, {NULL}};
const Segment SGI314459[] = {{&S26553[13], 10}, {&S26553[13], 10}, {NULL}};
const Segment SGI314460[] = {{&S26553[42], 7}, {&S26553[128], 7}, {NULL}};
const Segment SGI314461[] = {{&S26553[35], 14}, {&S26553[121], 14}, {NULL}};
const Segment SGI314462[] = {{&S26553[59], 11}, {NULL}};
const Segment SGI314463[] = {{&S26553[171], 15}, {NULL}};
const Segment SGI314464[] = {{&S26553[59], 16}, {NULL}};
const Segment SGI314465[] = {{&S26553[137], 3}, {NULL}};
const Segment SGI314466[] = {{&S26560[30], 3}, {NULL}};
const Segment SGI314467[] = {{&S26560[8], 3}, {NULL}};
const Segment SGI314468[] = {{&S26560[13], 10}, {NULL}};
const Segment SGI314469[] = {{&S26560[42], 7}, {NULL}};
const Segment SGI314470[] = {{&S26560[35], 14}, {NULL}};
const Segment SGI314471[] = {{&S26560[64], 11}, {NULL}};
const Segment SGI314472[] = {{&S26560[64], 16}, {NULL}};
const Segment SGI314473[] = {{&S26564[30], 3}, {NULL}};
const Segment SGI314474[] = {{&S26564[8], 3}, {NULL}};
const Segment SGI314475[] = {{&S26564[13], 10}, {NULL}};
const Segment SGI314476[] = {{&S26564[42], 7}, {NULL}};
const Segment SGI314477[] = {{&S26564[35], 14}, {NULL}};
const Segment SGI314478[] = {{&S26564[59], 11}, {NULL}};
const Segment SGI314479[] = {{&S26564[112], 15}, {NULL}};
const Segment SGI314480[] = {{&S26564[59], 16}, {NULL}};
const Segment SGI314481[] = {{&S26567[30], 3}, {NULL}};
const Segment SGI314482[] = {{&S26567[8], 3}, {NULL}};
const Segment SGI314483[] = {{&S26567[13], 10}, {NULL}};
const Segment SGI314484[] = {{&S26567[42], 7}, {NULL}};
const Segment SGI314485[] = {{&S26567[35], 14}, {NULL}};
const Segment SGI314486[] = {{&S26567[72], 11}, {NULL}};
const Segment SGI314487[] = {{&S26567[125], 15}, {NULL}};
const Segment SGI314488[] = {{&S26567[72], 16}, {NULL}};
const Segment SGI314489[] = {{&S26568[30], 3}, {NULL}};
const Segment SGI314490[] = {{&S26568[8], 3}, {NULL}};
const Segment SGI314491[] = {{&S26568[13], 10}, {NULL}};
const Segment SGI314492[] = {{&S26568[42], 7}, {NULL}};
const Segment SGI314493[] = {{&S26568[35], 14}, {NULL}};
const Segment SGI314494[] = {{&S26568[59], 11}, {NULL}};
const Segment SGI314495[] = {{&S26568[59], 16}, {NULL}};
const Segment SGI314496[] = {{&S26569[30], 3}, {NULL}};
const Segment SGI314497[] = {{&S26569[8], 3}, {NULL}};
const Segment SGI314498[] = {{&S26569[13], 10}, {NULL}};
const Segment SGI314499[] = {{&S26569[42], 7}, {NULL}};
const Segment SGI314500[] = {{&S26569[35], 14}, {NULL}};
const Segment SGI314501[] = {{&S26569[71], 11}, {NULL}};
const Segment SGI314502[] = {{&S26569[51], 6}, {NULL}};
const Segment SGI314503[] = {{&S26569[71], 16}, {NULL}};
const Segment SGI314504[] = {{&S26570[30], 3}, {NULL}};
const Segment SGI314505[] = {{&S26570[8], 3}, {NULL}};
const Segment SGI314506[] = {{&S26570[13], 10}, {NULL}};
const Segment SGI314507[] = {{&S26570[42], 7}, {NULL}};
const Segment SGI314508[] = {{&S26570[35], 14}, {NULL}};
const Segment SGI314509[] = {{&S26570[64], 11}, {NULL}};
const Segment SGI314510[] = {{&S26570[64], 16}, {NULL}};
const Segment SGI314511[] = {{&S26571[30], 3}, {NULL}};
const Segment SGI314512[] = {{&S26571[8], 3}, {NULL}};
const Segment SGI314513[] = {{&S26571[13], 10}, {NULL}};
const Segment SGI314514[] = {{&S26571[42], 7}, {NULL}};
const Segment SGI314515[] = {{&S26571[35], 14}, {NULL}};
const Segment SGI314516[] = {{&S26571[66], 11}, {NULL}};
const Segment SGI314517[] = {{&S26571[119], 15}, {NULL}};
const Segment SGI314518[] = {{&S26571[51], 6}, {NULL}};
const Segment SGI314519[] = {{&S26571[66], 16}, {NULL}};
const Segment SGI314520[] = {{&S26572[30], 3}, {NULL}};
const Segment SGI314521[] = {{&S26572[8], 3}, {NULL}};
const Segment SGI314522[] = {{&S26572[13], 10}, {NULL}};
const Segment SGI314523[] = {{&S26572[42], 7}, {NULL}};
const Segment SGI314524[] = {{&S26572[35], 14}, {NULL}};
const Segment SGI314525[] = {{&S26572[79], 11}, {NULL}};
const Segment SGI314526[] = {{&S26572[132], 15}, {NULL}};
const Segment SGI314527[] = {{&S26572[51], 6}, {NULL}};
const Segment SGI314528[] = {{&S26572[79], 16}, {NULL}};
const Segment SGI314529[] = {{&S26658[30], 3}, {NULL}};
const Segment SGI314530[] = {{&S26658[8], 3}, {NULL}};
const Segment SGI314531[] = {{&S26658[13], 10}, {NULL}};
const Segment SGI314532[] = {{&S26658[42], 7}, {NULL}};
const Segment SGI314533[] = {{&S26659[30], 3}, {NULL}};
const Segment SGI314534[] = {{&S26659[8], 3}, {NULL}};
const Segment SGI314535[] = {{&S26659[13], 10}, {NULL}};
const Segment SGI314536[] = {{&S26659[42], 7}, {NULL}};
const Segment SGI314537[] = {{&S26659[35], 14}, {NULL}};
const Segment SGI314538[] = {{&S26660[30], 3}, {NULL}};
const Segment SGI314539[] = {{&S26660[8], 3}, {NULL}};
const Segment SGI314540[] = {{&S26660[13], 10}, {NULL}};
const Segment SGI314541[] = {{&S26660[42], 8}, {NULL}};
const Segment SGI314542[] = {{&S26661[30], 3}, {NULL}};
const Segment SGI314543[] = {{&S26661[8], 3}, {NULL}};
const Segment SGI314544[] = {{&S26661[13], 10}, {NULL}};
const Segment SGI314545[] = {{&S26661[42], 6}, {NULL}};
const Segment SGI314546[] = {{&S26661[54], 3}, {NULL}};
const Segment SGI314547[] = {{&S26662[30], 3}, {NULL}};
const Segment SGI314548[] = {{&S26662[8], 3}, {NULL}};
const Segment SGI314549[] = {{&S26662[13], 10}, {NULL}};
const Segment SGI314550[] = {{&S26662[42], 7}, {NULL}};
const Segment SGI314551[] = {{&S26663[5], 3}, {&S26663[5], 3}, {NULL}};
const Segment SGI314552[] = {{&S26663[29], 3}, {NULL}};
const Segment SGI314553[] = {{&S26663[34], 10}, {NULL}};
const Segment SGI314554[] = {{&S26663[63], 8}, {NULL}};
const Segment SGI314555[] = {{&S41546[0], 96}, {NULL}};
const Segment SGI314556[] = {{&S41568[0], 74}, {NULL}};
const Segment SGI314557[] = {{&S41571[0], 74}, {NULL}};
const Segment SGI314558[] = {{&S55464[3], 5}, {NULL}};
const Segment SGI314559[] = {{&S55464[3], 12}, {NULL}};
const Segment SGI314560[] = {{&S55464[36], 8}, {NULL}};
const Segment SGI314561[] = {{&S55465[3], 5}, {NULL}};
const Segment SGI314562[] = {{&S55465[28], 8}, {NULL}};
const Segment SGI314563[] = {{&S27305[30], 3}, {NULL}};
const Segment SGI314564[] = {{&S27305[8], 3}, {NULL}};
const Segment SGI314565[] = {{&S27305[13], 10}, {NULL}};
const Segment SGI314566[] = {{&S27305[42], 7}, {NULL}};
const Segment SGI314567[] = {{&S27305[35], 14}, {NULL}};
const Segment SGI314568[] = {{&S27305[64], 11}, {NULL}};
const Segment SGI314569[] = {{&S27305[157], 15}, {NULL}};
const Segment SGI314570[] = {{&S27305[64], 16}, {NULL}};
const Segment SGI314571[] = {{&S27306[30], 3}, {NULL}};
const Segment SGI314572[] = {{&S27306[8], 3}, {NULL}};
const Segment SGI314573[] = {{&S27306[13], 10}, {NULL}};
const Segment SGI314574[] = {{&S27306[42], 7}, {NULL}};
const Segment SGI314575[] = {{&S27306[35], 14}, {NULL}};
const Segment SGI314576[] = {{&S27306[71], 11}, {NULL}};
const Segment SGI314577[] = {{&S27306[51], 6}, {NULL}};
const Segment SGI314578[] = {{&S27306[71], 16}, {NULL}};
const Segment SGI314579[] = {{&S27307[30], 3}, {NULL}};
const Segment SGI314580[] = {{&S27307[8], 3}, {NULL}};
const Segment SGI314581[] = {{&S27307[13], 10}, {NULL}};
const Segment SGI314582[] = {{&S27307[42], 7}, {NULL}};
const Segment SGI314583[] = {{&S27307[35], 14}, {NULL}};
const Segment SGI314584[] = {{&S27307[64], 11}, {NULL}};
const Segment SGI314585[] = {{&S27307[64], 16}, {NULL}};
const Segment SGI314586[] = {{&S27308[30], 3}, {NULL}};
const Segment SGI314587[] = {{&S27308[8], 3}, {NULL}};
const Segment SGI314588[] = {{&S27308[13], 10}, {NULL}};
const Segment SGI314589[] = {{&S27308[42], 7}, {NULL}};
const Segment SGI314590[] = {{&S27308[35], 14}, {NULL}};
const Segment SGI314591[] = {{&S27308[71], 11}, {NULL}};
const Segment SGI314592[] = {{&S27308[51], 6}, {NULL}};
const Segment SGI314593[] = {{&S27308[71], 16}, {NULL}};
const Segment SGI314594[] = {{&S27309[30], 3}, {NULL}};
const Segment SGI314595[] = {{&S27309[8], 3}, {NULL}};
const Segment SGI314596[] = {{&S27309[13], 10}, {NULL}};
const Segment SGI314597[] = {{&S27309[42], 7}, {NULL}};
const Segment SGI314598[] = {{&S27309[35], 14}, {NULL}};
const Segment SGI314599[] = {{&S27309[64], 11}, {NULL}};
const Segment SGI314600[] = {{&S27309[64], 16}, {NULL}};
const Segment SGI314601[] = {{&S27312[30], 3}, {NULL}};
const Segment SGI314602[] = {{&S27312[8], 3}, {NULL}};
const Segment SGI314603[] = {{&S27312[13], 10}, {NULL}};
const Segment SGI314604[] = {{&S27312[42], 7}, {NULL}};
const Segment SGI314605[] = {{&S27312[35], 14}, {NULL}};
const Segment SGI314606[] = {{&S27312[64], 11}, {NULL}};
const Segment SGI314607[] = {{&S27312[64], 16}, {NULL}};
const Segment SGI314608[] = {{&S27312[177], 9}, {NULL}};
const Segment SGI314609[] = {{&S27313[30], 3}, {NULL}};
const Segment SGI314610[] = {{&S27313[8], 3}, {NULL}};
const Segment SGI314611[] = {{&S27313[13], 10}, {NULL}};
const Segment SGI314612[] = {{&S27313[42], 7}, {NULL}};
const Segment SGI314613[] = {{&S27313[35], 14}, {NULL}};
const Segment SGI314614[] = {{&S27313[141], 11}, {NULL}};
const Segment SGI314615[] = {{&S27313[141], 16}, {NULL}};
const Segment SGI314616[] = {{&S27314[30], 3}, {NULL}};
const Segment SGI314617[] = {{&S27314[8], 3}, {NULL}};
const Segment SGI314618[] = {{&S27314[13], 10}, {NULL}};
const Segment SGI314619[] = {{&S27314[42], 7}, {NULL}};
const Segment SGI314620[] = {{&S27314[35], 14}, {NULL}};
const Segment SGI314621[] = {{&S27314[71], 11}, {NULL}};
const Segment SGI314622[] = {{&S27314[51], 6}, {NULL}};
const Segment SGI314623[] = {{&S27314[71], 16}, {NULL}};
const Segment SGI314624[] = {{&S27315[30], 3}, {NULL}};
const Segment SGI314625[] = {{&S27315[8], 3}, {NULL}};
const Segment SGI314626[] = {{&S27315[13], 10}, {NULL}};
const Segment SGI314627[] = {{&S27315[42], 7}, {NULL}};
const Segment SGI314628[] = {{&S27315[35], 14}, {NULL}};
const Segment SGI314629[] = {{&S27315[64], 11}, {NULL}};
const Segment SGI314630[] = {{&S27315[157], 15}, {NULL}};
const Segment SGI314631[] = {{&S27315[64], 16}, {NULL}};
const Segment SGI314632[] = {{&S27316[30], 3}, {NULL}};
const Segment SGI314633[] = {{&S27316[8], 3}, {NULL}};
const Segment SGI314634[] = {{&S27316[13], 10}, {NULL}};
const Segment SGI314635[] = {{&S27316[42], 7}, {NULL}};
const Segment SGI314636[] = {{&S27316[35], 14}, {NULL}};
const Segment SGI314637[] = {{&S27316[71], 11}, {NULL}};
const Segment SGI314638[] = {{&S27316[164], 15}, {NULL}};
const Segment SGI314639[] = {{&S27316[51], 6}, {NULL}};
const Segment SGI314640[] = {{&S27316[71], 16}, {NULL}};
const Segment SGI314641[] = {{&S27317[30], 3}, {&S27317[127], 3}, {NULL}};
const Segment SGI314642[] = {{&S27317[8], 3}, {&S27317[8], 3}, {NULL}};
const Segment SGI314643[] = {{&S27317[13], 10}, {&S27317[13], 10}, {NULL}};
const Segment SGI314644[] = {{&S27317[42], 7}, {&S27317[139], 7}, {NULL}};
const Segment SGI314645[] = {{&S27317[35], 14}, {&S27317[132], 14}, {NULL}};
const Segment SGI314646[] = {{&S27317[71], 11}, {NULL}};
const Segment SGI314647[] = {{&S27317[245], 15}, {NULL}};
const Segment SGI314648[] = {{&S27317[51], 6}, {NULL}};
const Segment SGI314649[] = {{&S27317[71], 16}, {NULL}};
const Segment SGI314650[] = {{&S27317[148], 3}, {NULL}};
const Segment SGI314651[] = {{&S27318[30], 3}, {NULL}};
const Segment SGI314652[] = {{&S27318[8], 3}, {NULL}};
const Segment SGI314653[] = {{&S27318[13], 10}, {NULL}};
const Segment SGI314654[] = {{&S27318[42], 7}, {NULL}};
const Segment SGI314655[] = {{&S27318[35], 14}, {NULL}};
const Segment SGI314656[] = {{&S27318[69], 11}, {NULL}};
const Segment SGI314657[] = {{&S27318[162], 15}, {NULL}};
const Segment SGI314658[] = {{&S27318[69], 16}, {NULL}};
const Segment SGI314659[] = {{&S27318[64], 3}, {NULL}};
const Segment SGI314660[] = {{&S27319[30], 3}, {NULL}};
const Segment SGI314661[] = {{&S27319[8], 3}, {NULL}};
const Segment SGI314662[] = {{&S27319[13], 10}, {NULL}};
const Segment SGI314663[] = {{&S27319[42], 7}, {NULL}};
const Segment SGI314664[] = {{&S27319[35], 14}, {NULL}};
const Segment SGI314665[] = {{&S27319[69], 11}, {NULL}};
const Segment SGI314666[] = {{&S27319[162], 15}, {NULL}};
const Segment SGI314667[] = {{&S27319[69], 16}, {NULL}};
const Segment SGI314668[] = {{&S27319[228], 9}, {NULL}};
const Segment SGI314669[] = {{&S27319[64], 3}, {NULL}};
const Segment SGI314670[] = {{&S27320[30], 3}, {NULL}};
const Segment SGI314671[] = {{&S27320[8], 3}, {NULL}};
const Segment SGI314672[] = {{&S27320[13], 10}, {NULL}};
const Segment SGI314673[] = {{&S27320[42], 7}, {NULL}};
const Segment SGI314674[] = {{&S27320[35], 14}, {NULL}};
const Segment SGI314675[] = {{&S27320[76], 11}, {NULL}};
const Segment SGI314676[] = {{&S27320[169], 15}, {NULL}};
const Segment SGI314677[] = {{&S27320[76], 16}, {NULL}};
const Segment SGI314678[] = {{&S27320[51], 5}, {NULL}};
const Segment SGI314679[] = {{&S27321[30], 4}, {NULL}};
const Segment SGI314680[] = {{&S27321[8], 3}, {NULL}};
const Segment SGI314681[] = {{&S27321[13], 10}, {NULL}};
const Segment SGI314682[] = {{&S27321[43], 7}, {NULL}};
const Segment SGI314683[] = {{&S27321[36], 14}, {NULL}};
const Segment SGI314684[] = {{&S27321[72], 11}, {NULL}};
const Segment SGI314685[] = {{&S27321[52], 6}, {NULL}};
const Segment SGI314686[] = {{&S27321[72], 16}, {NULL}};
const Segment SGI314687[] = {{&S27322[30], 3}, {NULL}};
const Segment SGI314688[] = {{&S27322[8], 3}, {NULL}};
const Segment SGI314689[] = {{&S27322[13], 10}, {NULL}};
const Segment SGI314690[] = {{&S27322[42], 7}, {NULL}};
const Segment SGI314691[] = {{&S27322[35], 14}, {NULL}};
const Segment SGI314692[] = {{&S27322[64], 11}, {NULL}};
const Segment SGI314693[] = {{&S27322[157], 15}, {NULL}};
const Segment SGI314694[] = {{&S27322[64], 16}, {NULL}};
const Segment SGI314695[] = {{&S27322[223], 9}, {NULL}};
const Segment SGI314696[] = {{&S27323[30], 4}, {NULL}};
const Segment SGI314697[] = {{&S27323[8], 3}, {NULL}};
const Segment SGI314698[] = {{&S27323[13], 10}, {NULL}};
const Segment SGI314699[] = {{&S27323[43], 7}, {NULL}};
const Segment SGI314700[] = {{&S27323[36], 14}, {NULL}};
const Segment SGI314701[] = {{&S27323[65], 11}, {NULL}};
const Segment SGI314702[] = {{&S27323[65], 16}, {NULL}};
const Segment SGI314703[] = {{&S27324[30], 3}, {NULL}};
const Segment SGI314704[] = {{&S27324[8], 3}, {NULL}};
const Segment SGI314705[] = {{&S27324[13], 10}, {NULL}};
const Segment SGI314706[] = {{&S27324[42], 7}, {NULL}};
const Segment SGI314707[] = {{&S27324[35], 14}, {NULL}};
const Segment SGI314708[] = {{&S27324[71], 11}, {NULL}};
const Segment SGI314709[] = {{&S27324[164], 15}, {NULL}};
const Segment SGI314710[] = {{&S27324[51], 6}, {NULL}};
const Segment SGI314711[] = {{&S27324[71], 16}, {NULL}};
const Segment SGI314712[] = {{&S27325[30], 3}, {NULL}};
const Segment SGI314713[] = {{&S27325[8], 3}, {NULL}};
const Segment SGI314714[] = {{&S27325[13], 10}, {NULL}};
const Segment SGI314715[] = {{&S27325[42], 7}, {NULL}};
const Segment SGI314716[] = {{&S27325[35], 14}, {NULL}};
const Segment SGI314717[] = {{&S27325[69], 11}, {NULL}};
const Segment SGI314718[] = {{&S27325[162], 15}, {NULL}};
const Segment SGI314719[] = {{&S27325[69], 16}, {NULL}};
const Segment SGI314720[] = {{&S27325[64], 3}, {NULL}};
const Segment SGI314721[] = {{&S27427[30], 3}, {NULL}};
const Segment SGI314722[] = {{&S27427[8], 3}, {NULL}};
const Segment SGI314723[] = {{&S27427[13], 10}, {NULL}};
const Segment SGI314724[] = {{&S27427[42], 7}, {NULL}};
const Segment SGI314725[] = {{&S27427[35], 14}, {NULL}};
const Segment SGI314726[] = {{&S27428[30], 3}, {NULL}};
const Segment SGI314727[] = {{&S27428[8], 3}, {NULL}};
const Segment SGI314728[] = {{&S27428[13], 10}, {NULL}};
const Segment SGI314729[] = {{&S27428[42], 7}, {NULL}};
const Segment SGI314730[] = {{&S27428[35], 14}, {NULL}};
const Segment SGI314731[] = {{&S27428[76], 3}, {NULL}};
const Segment SGI314732[] = {{&S27429[30], 3}, {NULL}};
const Segment SGI314733[] = {{&S27429[8], 3}, {NULL}};
const Segment SGI314734[] = {{&S27429[13], 10}, {NULL}};
const Segment SGI314735[] = {{&S27429[42], 7}, {NULL}};
const Segment SGI314736[] = {{&S27430[30], 3}, {NULL}};
const Segment SGI314737[] = {{&S27430[8], 3}, {NULL}};
const Segment SGI314738[] = {{&S27430[13], 10}, {NULL}};
const Segment SGI314739[] = {{&S27430[42], 7}, {NULL}};
const Segment SGI314740[] = {{&S27430[35], 14}, {NULL}};
const Segment SGI314741[] = {{&S27430[91], 15}, {NULL}};
const Segment SGI314742[] = {{&S27431[30], 3}, {NULL}};
const Segment SGI314743[] = {{&S27431[8], 3}, {NULL}};
const Segment SGI314744[] = {{&S27431[13], 10}, {NULL}};
const Segment SGI314745[] = {{&S27431[55], 7}, {NULL}};
const Segment SGI314746[] = {{&S27431[48], 14}, {NULL}};
const Segment SGI314747[] = {{&S27432[30], 3}, {NULL}};
const Segment SGI314748[] = {{&S27432[8], 3}, {NULL}};
const Segment SGI314749[] = {{&S27432[13], 10}, {NULL}};
const Segment SGI314750[] = {{&S27432[55], 7}, {NULL}};
const Segment SGI314751[] = {{&S27432[48], 14}, {NULL}};
const Segment SGI314752[] = {{&S27432[64], 3}, {NULL}};
const Segment SGI314753[] = {{&S27433[30], 3}, {NULL}};
const Segment SGI314754[] = {{&S27433[8], 3}, {NULL}};
const Segment SGI314755[] = {{&S27433[13], 10}, {NULL}};
const Segment SGI314756[] = {{&S27433[42], 7}, {NULL}};
const Segment SGI314757[] = {{&S27433[35], 14}, {NULL}};
const Segment SGI314758[] = {{&S27433[51], 6}, {NULL}};
const Segment SGI314759[] = {{&S27434[30], 3}, {NULL}};
const Segment SGI314760[] = {{&S27434[8], 3}, {NULL}};
const Segment SGI314761[] = {{&S27434[13], 10}, {NULL}};
const Segment SGI314762[] = {{&S27434[42], 7}, {NULL}};
const Segment SGI314763[] = {{&S27434[35], 14}, {NULL}};
const Segment SGI314764[] = {{&S27434[138], 15}, {NULL}};
const Segment SGI314765[] = {{&S27434[51], 6}, {NULL}};
const Segment SGI314766[] = {{&S27435[30], 4}, {NULL}};
const Segment SGI314767[] = {{&S27435[8], 3}, {NULL}};
const Segment SGI314768[] = {{&S27435[13], 10}, {NULL}};
const Segment SGI314769[] = {{&S27435[43], 7}, {NULL}};
const Segment SGI314770[] = {{&S27435[36], 14}, {NULL}};
const Segment SGI314771[] = {{&S27436[30], 3}, {NULL}};
const Segment SGI314772[] = {{&S27436[8], 3}, {NULL}};
const Segment SGI314773[] = {{&S27436[13], 10}, {NULL}};
const Segment SGI314774[] = {{&S27436[55], 7}, {NULL}};
const Segment SGI314775[] = {{&S27436[48], 14}, {NULL}};
const Segment SGI314776[] = {{&S27436[64], 3}, {&S27436[64], 3}, {NULL}};
const Segment SGI314777[] = {{&S27437[30], 3}, {NULL}};
const Segment SGI314778[] = {{&S27437[8], 3}, {NULL}};
const Segment SGI314779[] = {{&S27437[13], 10}, {NULL}};
const Segment SGI314780[] = {{&S27437[55], 7}, {NULL}};
const Segment SGI314781[] = {{&S27437[48], 14}, {NULL}};
const Segment SGI314782[] = {{&S27437[64], 3}, {&S27437[64], 3}, {NULL}};
const Segment SGI314783[] = {{&S44427[3], 5}, {NULL}};
const Segment SGI314784[] = {{&S44427[80], 13}, {NULL}};
const Segment SGI314785[] = {{&S28122[30], 3}, {NULL}};
const Segment SGI314786[] = {{&S28122[8], 3}, {NULL}};
const Segment SGI314787[] = {{&S28122[13], 10}, {NULL}};
const Segment SGI314788[] = {{&S28122[42], 7}, {NULL}};
const Segment SGI314789[] = {{&S28122[35], 14}, {NULL}};
const Segment SGI314790[] = {{&S28122[64], 13}, {NULL}};
const Segment SGI314791[] = {{&S28123[30], 3}, {NULL}};
const Segment SGI314792[] = {{&S28123[8], 3}, {NULL}};
const Segment SGI314793[] = {{&S28123[13], 10}, {NULL}};
const Segment SGI314794[] = {{&S28123[42], 7}, {NULL}};
const Segment SGI314795[] = {{&S28123[35], 14}, {NULL}};
const Segment SGI314796[] = {{&S28123[51], 6}, {NULL}};
const Segment SGI314797[] = {{&S28123[71], 13}, {NULL}};
const Segment SGI314798[] = {{&S28124[30], 3}, {NULL}};
const Segment SGI314799[] = {{&S28124[8], 3}, {NULL}};
const Segment SGI314800[] = {{&S28124[13], 10}, {NULL}};
const Segment SGI314801[] = {{&S28124[42], 7}, {NULL}};
const Segment SGI314802[] = {{&S28124[35], 14}, {NULL}};
const Segment SGI314803[] = {{&S28124[76], 13}, {NULL}};
const Segment SGI314804[] = {{&S28124[51], 5}, {NULL}};
const Segment SGI314805[] = {{&S28125[30], 3}, {NULL}};
const Segment SGI314806[] = {{&S28125[8], 3}, {NULL}};
const Segment SGI314807[] = {{&S28125[13], 10}, {NULL}};
const Segment SGI314808[] = {{&S28125[55], 7}, {NULL}};
const Segment SGI314809[] = {{&S28125[48], 14}, {NULL}};
const Segment SGI314810[] = {{&S28125[64], 13}, {NULL}};
const Segment SGI314811[] = {{&S28126[30], 3}, {NULL}};
const Segment SGI314812[] = {{&S28126[8], 3}, {NULL}};
const Segment SGI314813[] = {{&S28126[13], 10}, {NULL}};
const Segment SGI314814[] = {{&S28126[42], 7}, {NULL}};
const Segment SGI314815[] = {{&S28126[35], 14}, {NULL}};
const Segment SGI314816[] = {{&S28126[64], 12}, {NULL}};
const Segment SGI314817[] = {{&S28127[30], 3}, {NULL}};
const Segment SGI314818[] = {{&S28127[8], 3}, {NULL}};
const Segment SGI314819[] = {{&S28127[13], 10}, {NULL}};
const Segment SGI314820[] = {{&S28127[55], 7}, {NULL}};
const Segment SGI314821[] = {{&S28127[48], 14}, {NULL}};
const Segment SGI314822[] = {{&S28127[69], 13}, {NULL}};
const Segment SGI314823[] = {{&S28127[64], 3}, {NULL}};
const Segment SGI314824[] = {{&S28128[30], 3}, {NULL}};
const Segment SGI314825[] = {{&S28128[8], 3}, {NULL}};
const Segment SGI314826[] = {{&S28128[13], 10}, {NULL}};
const Segment SGI314827[] = {{&S28128[55], 7}, {NULL}};
const Segment SGI314828[] = {{&S28128[48], 14}, {NULL}};
const Segment SGI314829[] = {{&S28128[64], 13}, {&S28128[64], 13}, {NULL}};
const Segment SGI314830[] = {{&S28129[30], 3}, {NULL}};
const Segment SGI314831[] = {{&S28129[8], 3}, {NULL}};
const Segment SGI314832[] = {{&S28129[13], 10}, {NULL}};
const Segment SGI314833[] = {{&S28129[55], 7}, {NULL}};
const Segment SGI314834[] = {{&S28129[48], 14}, {NULL}};
const Segment SGI314835[] = {{&S28129[64], 13}, {&S28129[64], 13}, {NULL}};
const Segment SGI314836[] = {{&S28130[30], 3}, {NULL}};
const Segment SGI314837[] = {{&S28130[8], 3}, {NULL}};
const Segment SGI314838[] = {{&S28130[13], 10}, {NULL}};
const Segment SGI314839[] = {{&S28130[42], 7}, {NULL}};
const Segment SGI314840[] = {{&S28130[35], 14}, {NULL}};
const Segment SGI314841[] = {{&S28130[138], 15}, {NULL}};
const Segment SGI314842[] = {{&S28130[51], 6}, {NULL}};
const Segment SGI314843[] = {{&S28130[169], 13}, {NULL}};
const Segment SGI314844[] = {{&S28131[30], 3}, {NULL}};
const Segment SGI314845[] = {{&S28131[8], 3}, {NULL}};
const Segment SGI314846[] = {{&S28131[13], 10}, {NULL}};
const Segment SGI314847[] = {{&S28131[55], 7}, {NULL}};
const Segment SGI314848[] = {{&S28131[48], 14}, {NULL}};
const Segment SGI314849[] = {{&S28131[69], 13}, {NULL}};
const Segment SGI314850[] = {{&S28131[64], 3}, {NULL}};
const Segment SGI314851[] = {{&S28132[30], 3}, {NULL}};
const Segment SGI314852[] = {{&S28132[8], 3}, {NULL}};
const Segment SGI314853[] = {{&S28132[13], 10}, {NULL}};
const Segment SGI314854[] = {{&S28132[42], 7}, {NULL}};
const Segment SGI314855[] = {{&S28132[35], 14}, {NULL}};
const Segment SGI314856[] = {{&S28132[131], 15}, {NULL}};
const Segment SGI314857[] = {{&S28132[172], 13}, {NULL}};
const Segment SGI314858[] = {{&S28905[30], 4}, {NULL}};
const Segment SGI314859[] = {{&S28905[8], 3}, {NULL}};
const Segment SGI314860[] = {{&S28905[13], 10}, {NULL}};
const Segment SGI314861[] = {{&S28905[43], 7}, {NULL}};
const Segment SGI314862[] = {{&S28906[30], 3}, {NULL}};
const Segment SGI314863[] = {{&S28906[8], 3}, {NULL}};
const Segment SGI314864[] = {{&S28906[13], 10}, {NULL}};
const Segment SGI314865[] = {{&S28906[42], 7}, {NULL}};
const Segment SGI314866[] = {{&S28907[30], 4}, {NULL}};
const Segment SGI314867[] = {{&S28907[8], 3}, {NULL}};
const Segment SGI314868[] = {{&S28907[13], 10}, {NULL}};
const Segment SGI314869[] = {{&S28907[43], 7}, {NULL}};
const Segment SGI314870[] = {{&S28907[36], 14}, {NULL}};
const Segment SGI314871[] = {{&S28908[30], 3}, {NULL}};
const Segment SGI314872[] = {{&S28908[8], 3}, {NULL}};
const Segment SGI314873[] = {{&S28908[13], 10}, {NULL}};
const Segment SGI314874[] = {{&S28908[42], 7}, {NULL}};
const Segment SGI314875[] = {{&S28908[35], 14}, {NULL}};
const Segment SGI314876[] = {{&S28909[30], 4}, {NULL}};
const Segment SGI314877[] = {{&S28909[8], 3}, {NULL}};
const Segment SGI314878[] = {{&S28909[13], 10}, {NULL}};
const Segment SGI314879[] = {{&S28909[43], 7}, {NULL}};
const Segment SGI314880[] = {{&S28910[29], 3}, {NULL}};
const Segment SGI314881[] = {{&S28910[8], 3}, {NULL}};
const Segment SGI314882[] = {{&S28910[13], 10}, {NULL}};
const Segment SGI314883[] = {{&S28910[42], 7}, {NULL}};
const Segment SGI314884[] = {{&S28910[35], 14}, {NULL}};
const Segment SGI314885[] = {{&S28911[30], 4}, {NULL}};
const Segment SGI314886[] = {{&S28911[8], 3}, {NULL}};
const Segment SGI314887[] = {{&S28911[13], 10}, {NULL}};
const Segment SGI314888[] = {{&S28911[43], 7}, {NULL}};
const Segment SGI314889[] = {{&S28911[36], 14}, {NULL}};
const Segment SGI314890[] = {{&S28911[52], 3}, {NULL}};
const Segment SGI314891[] = {{&S28912[30], 4}, {NULL}};
const Segment SGI314892[] = {{&S28912[8], 3}, {NULL}};
const Segment SGI314893[] = {{&S28912[13], 10}, {NULL}};
const Segment SGI314894[] = {{&S28912[43], 7}, {NULL}};
const Segment SGI314895[] = {{&S28913[29], 3}, {NULL}};
const Segment SGI314896[] = {{&S28913[8], 3}, {NULL}};
const Segment SGI314897[] = {{&S28913[13], 10}, {NULL}};
const Segment SGI314898[] = {{&S28913[42], 7}, {NULL}};
const Segment SGI314899[] = {{&S28913[35], 14}, {NULL}};
const Segment SGI314900[] = {{&S28914[28], 4}, {NULL}};
const Segment SGI314901[] = {{&S28914[41], 7}, {NULL}};
const Segment SGI314902[] = {{&S28915[8], 3}, {NULL}};
const Segment SGI314903[] = {{&S28915[13], 10}, {NULL}};
const Segment SGI314904[] = {{&S28915[42], 7}, {NULL}};
const Segment SGI314905[] = {{&S28915[35], 14}, {NULL}};
const Segment SGI314906[] = {{&S62977[0], 131}, {NULL}};
const Segment SGI314907[] = {{&S29502[30], 3}, {NULL}};
const Segment SGI314908[] = {{&S29502[8], 3}, {NULL}};
const Segment SGI314909[] = {{&S29502[13], 10}, {NULL}};
const Segment SGI314910[] = {{&S29502[42], 7}, {NULL}};
const Segment SGI314911[] = {{&S29503[36], 3}, {NULL}};
const Segment SGI314912[] = {{&S29503[8], 3}, {NULL}};
const Segment SGI314913[] = {{&S29503[48], 7}, {NULL}};
const Segment SGI314914[] = {{&S29503[41], 14}, {NULL}};
const Segment SGI314915[] = {{&S29504[30], 4}, {NULL}};
const Segment SGI314916[] = {{&S29504[8], 3}, {NULL}};
const Segment SGI314917[] = {{&S29504[13], 10}, {NULL}};
const Segment SGI314918[] = {{&S29504[43], 7}, {NULL}};
const Segment SGI314919[] = {{&S29505[30], 3}, {NULL}};
const Segment SGI314920[] = {{&S29505[8], 3}, {NULL}};
const Segment SGI314921[] = {{&S29505[13], 10}, {NULL}};
const Segment SGI314922[] = {{&S29505[42], 7}, {NULL}};
const Segment SGI314923[] = {{&S29505[35], 14}, {NULL}};
const Segment SGI314924[] = {{&S29506[30], 3}, {NULL}};
const Segment SGI314925[] = {{&S29506[8], 3}, {NULL}};
const Segment SGI314926[] = {{&S29506[13], 10}, {NULL}};
const Segment SGI314927[] = {{&S29506[42], 7}, {NULL}};
const Segment SGI314928[] = {{&S29506[35], 14}, {NULL}};
const Segment SGI314929[] = {{&S29506[51], 3}, {NULL}};
const Segment SGI314930[] = {{&S29507[30], 3}, {NULL}};
const Segment SGI314931[] = {{&S29507[8], 3}, {NULL}};
const Segment SGI314932[] = {{&S29507[13], 10}, {NULL}};
const Segment SGI314933[] = {{&S29507[42], 7}, {NULL}};
const Segment SGI314934[] = {{&S29508[29], 3}, {NULL}};
const Segment SGI314935[] = {{&S29508[8], 3}, {NULL}};
const Segment SGI314936[] = {{&S29508[13], 10}, {NULL}};
const Segment SGI314937[] = {{&S29508[42], 7}, {NULL}};
const Segment SGI314938[] = {{&S29508[35], 14}, {NULL}};
const Segment SGI314939[] = {{&S29509[29], 3}, {NULL}};
const Segment SGI314940[] = {{&S29509[8], 3}, {NULL}};
const Segment SGI314941[] = {{&S29509[13], 10}, {NULL}};
const Segment SGI314942[] = {{&S29509[42], 7}, {NULL}};
const Segment SGI314943[] = {{&S29509[35], 14}, {NULL}};
const Segment SGI314944[] = {{&S29510[30], 3}, {NULL}};
const Segment SGI314945[] = {{&S29510[8], 3}, {NULL}};
const Segment SGI314946[] = {{&S29510[13], 10}, {NULL}};
const Segment SGI314947[] = {{&S29510[55], 7}, {NULL}};
const Segment SGI314948[] = {{&S29510[48], 14}, {NULL}};
const Segment SGI314949[] = {{&S29510[64], 3}, {NULL}};
const Segment SGI314950[] = {{&S29511[30], 3}, {&S29511[81], 3}, {NULL}};
const Segment SGI314951[] = {{&S29511[8], 3}, {&S29511[8], 3}, {NULL}};
const Segment SGI314952[] = {{&S29511[13], 10}, {&S29511[13], 10}, {NULL}};
const Segment SGI314953[] = {{&S29511[42], 7}, {NULL}};
const Segment SGI314954[] = {{&S29511[35], 14}, {NULL}};
const Segment SGI314955[] = {{&S29511[86], 5}, {NULL}};
const Segment SGI314956[] = {{&S29512[30], 3}, {NULL}};
const Segment SGI314957[] = {{&S29512[8], 3}, {NULL}};
const Segment SGI314958[] = {{&S29512[13], 10}, {NULL}};
const Segment SGI314959[] = {{&S29512[42], 7}, {NULL}};
const Segment SGI314960[] = {{&S29512[35], 14}, {NULL}};
const Segment SGI314961[] = {{&S29512[51], 6}, {NULL}};
const Segment SGI314962[] = {{&S29513[30], 3}, {NULL}};
const Segment SGI314963[] = {{&S29513[8], 3}, {NULL}};
const Segment SGI314964[] = {{&S29513[13], 10}, {NULL}};
const Segment SGI314965[] = {{&S29513[42], 7}, {NULL}};
const Segment SGI314966[] = {{&S29513[35], 14}, {NULL}};
const Segment SGI314967[] = {{&S29514[30], 3}, {NULL}};
const Segment SGI314968[] = {{&S29514[8], 3}, {NULL}};
const Segment SGI314969[] = {{&S29514[13], 10}, {NULL}};
const Segment SGI314970[] = {{&S29514[42], 7}, {NULL}};
const Segment SGI314971[] = {{&S29514[35], 14}, {NULL}};
const Segment SGI314972[] = {{&S29514[110], 15}, {NULL}};
const Segment SGI314973[] = {{&S29515[30], 4}, {NULL}};
const Segment SGI314974[] = {{&S29515[8], 3}, {NULL}};
const Segment SGI314975[] = {{&S29515[13], 10}, {NULL}};
const Segment SGI314976[] = {{&S29515[43], 7}, {NULL}};
const Segment SGI314977[] = {{&S44100[0], 17}, {NULL}};
const Segment SGI314978[] = {{&S44101[0], 46}, {NULL}};
const Segment SGI314979[] = {{&S44102[0], 47}, {NULL}};
const Segment SGI314980[] = {{&S44103[0], 46}, {NULL}};
const Segment SGI314981[] = {{&S44104[0], 68}, {NULL}};
const Segment SGI314982[] = {{&S44106[0], 205}, {NULL}};
const Segment SGI314983[] = {{&S44107[0], 186}, {NULL}};
const Segment SGI314984[] = {{&S44108[0], 194}, {NULL}};
const Segment SGI314985[] = {{&S44109[0], 203}, {NULL}};
const Segment SGI314986[] = {{&S44110[0], 208}, {NULL}};
const Segment SGI314987[] = {{&S44111[0], 203}, {NULL}};
const Segment SGI314988[] = {{&S44112[0], 205}, {NULL}};
const Segment SGI314989[] = {{&S44113[0], 198}, {NULL}};
const Segment SGI314990[] = {{&S44114[0], 211}, {NULL}};
const Segment SGI314991[] = {{&S44115[0], 198}, {NULL}};
const Segment SGI314992[] = {{&S44116[0], 368}, {NULL}};
const Segment SGI314993[] = {{&S44117[0], 213}, {NULL}};
const Segment SGI314994[] = {{&S44118[0], 208}, {NULL}};
const Segment SGI314995[] = {{&S44119[0], 212}, {NULL}};
const Segment SGI314996[] = {{&S44120[0], 205}, {NULL}};
const Segment SGI314997[] = {{&S44121[0], 246}, {NULL}};
const Segment SGI314998[] = {{&S44122[0], 205}, {NULL}};
const Segment SGI314999[] = {{&S44123[0], 195}, {NULL}};
const Segment SGI315000[] = {{&S44124[0], 196}, {NULL}};
const Segment SGI315001[] = {{&S44125[0], 205}, {NULL}};
const Segment SGI315002[] = {{&S44126[0], 203}, {NULL}};
const Segment SGI315003[] = {{&S44127[0], 205}, {NULL}};
const Segment SGI315004[] = {{&S44128[0], 186}, {NULL}};
const Segment SGI315005[] = {{&S44129[0], 196}, {NULL}};
const Segment SGI315006[] = {{&S44130[0], 212}, {NULL}};
const Segment SGI315007[] = {{&S44131[0], 212}, {NULL}};
const Segment SGI315008[] = {{&S44132[0], 212}, {NULL}};
const Segment SGI315009[] = {{&S44133[0], 203}, {NULL}};
const Segment SGI315010[] = {{&S44134[0], 369}, {NULL}};
const Segment SGI315011[] = {{&S44135[0], 246}, {NULL}};
const Segment SGI315012[] = {{&S44136[0], 212}, {NULL}};
const Segment SGI315013[] = {{&S44137[0], 205}, {NULL}};
const Segment SGI315014[] = {{&S44138[0], 206}, {NULL}};
const Segment SGI315015[] = {{&S44139[0], 211}, {NULL}};
const Segment SGI315016[] = {{&S44140[0], 203}, {NULL}};
const Segment SGI315017[] = {{&S44141[0], 203}, {NULL}};
const Segment SGI315018[] = {{&S44142[0], 246}, {NULL}};
const Segment SGI315019[] = {{&S44143[0], 205}, {NULL}};
const Segment SGI315020[] = {{&S44144[0], 209}, {NULL}};
const Segment SGI315021[] = {{&S44145[0], 211}, {NULL}};
const Segment SGI315022[] = {{&S44146[0], 205}, {NULL}};
const Segment SGI315023[] = {{&S44147[0], 246}, {NULL}};
const Segment SGI315024[] = {{&S44148[0], 194}, {NULL}};
const Segment SGI315025[] = {{&S44149[0], 245}, {NULL}};
const Segment SGI315026[] = {{&S44150[0], 199}, {NULL}};
const Segment SGI315027[] = {{&S44151[0], 245}, {NULL}};
const Segment SGI315028[] = {{&S44152[0], 195}, {NULL}};
const Segment SGI315029[] = {{&S44153[0], 203}, {NULL}};
const Segment SGI315030[] = {{&S44154[0], 246}, {NULL}};
const Segment SGI315031[] = {{&S44155[0], 205}, {NULL}};
const Segment SGI315032[] = {{&S44156[0], 209}, {NULL}};
const Segment SGI315033[] = {{&S44157[0], 212}, {NULL}};
const Segment SGI315034[] = {{&S44158[0], 203}, {NULL}};
const Segment SGI315035[] = {{&S44159[0], 203}, {NULL}};
const Segment SGI315036[] = {{&S44160[0], 210}, {NULL}};
const Segment SGI315037[] = {{&S44161[0], 203}, {NULL}};
const Segment SGI315038[] = {{&S44162[0], 208}, {NULL}};
const Segment SGI315039[] = {{&S44163[0], 203}, {NULL}};
const Segment SGI315040[] = {{&S44164[0], 211}, {NULL}};
const Segment SGI315041[] = {{&S44165[0], 194}, {NULL}};
const Segment SGI315042[] = {{&S44166[0], 205}, {NULL}};
const Segment SGI315043[] = {{&S44167[0], 212}, {NULL}};
const Segment SGI315044[] = {{&S44168[0], 369}, {NULL}};
const Segment SGI315045[] = {{&S44169[0], 211}, {NULL}};
const Segment SGI315046[] = {{&S44170[0], 212}, {NULL}};
const Segment SGI315047[] = {{&S44171[0], 211}, {NULL}};
const Segment SGI315048[] = {{&S44172[0], 357}, {NULL}};
const Segment SGI315049[] = {{&S44173[0], 203}, {NULL}};
const Segment SGI315050[] = {{&S44174[0], 210}, {NULL}};
const Segment SGI315051[] = {{&S44175[0], 203}, {NULL}};
const Segment SGI315052[] = {{&S44176[0], 371}, {NULL}};
const Segment SGI315053[] = {{&S44177[0], 371}, {NULL}};
const Segment SGI315054[] = {{&S44178[0], 374}, {NULL}};
const Segment SGI315055[] = {{&S44179[0], 206}, {NULL}};
const Segment SGI315056[] = {{&S44180[0], 203}, {NULL}};
const Segment SGI315057[] = {{&S44181[0], 206}, {NULL}};
const Segment SGI315058[] = {{&S44182[0], 203}, {NULL}};
const Segment SGI315059[] = {{&S44183[0], 199}, {NULL}};
const Segment SGI315060[] = {{&S29765[30], 3}, {NULL}};
const Segment SGI315061[] = {{&S29765[8], 3}, {NULL}};
const Segment SGI315062[] = {{&S29765[13], 10}, {NULL}};
const Segment SGI315063[] = {{&S29765[42], 7}, {NULL}};
const Segment SGI315064[] = {{&S29765[35], 14}, {NULL}};
const Segment SGI315065[] = {{&S29765[146], 15}, {NULL}};
const Segment SGI315066[] = {{&S29765[64], 13}, {NULL}};
const Segment SGI315067[] = {{&S29766[30], 3}, {NULL}};
const Segment SGI315068[] = {{&S29766[8], 3}, {NULL}};
const Segment SGI315069[] = {{&S29766[13], 10}, {NULL}};
const Segment SGI315070[] = {{&S29766[42], 7}, {NULL}};
const Segment SGI315071[] = {{&S29766[35], 14}, {NULL}};
const Segment SGI315072[] = {{&S29766[138], 15}, {NULL}};
const Segment SGI315073[] = {{&S29766[51], 6}, {NULL}};
const Segment SGI315074[] = {{&S29766[169], 13}, {NULL}};
const Segment SGI315075[] = {{&S29767[30], 3}, {NULL}};
const Segment SGI315076[] = {{&S29767[8], 3}, {NULL}};
const Segment SGI315077[] = {{&S29767[13], 10}, {NULL}};
const Segment SGI315078[] = {{&S29767[42], 7}, {NULL}};
const Segment SGI315079[] = {{&S29767[35], 14}, {NULL}};
const Segment SGI315080[] = {{&S29767[131], 15}, {NULL}};
const Segment SGI315081[] = {{&S29767[177], 13}, {NULL}};
const Segment SGI315082[] = {{&S29767[162], 9}, {NULL}};
const Segment SGI315083[] = {{&S29768[30], 3}, {NULL}};
const Segment SGI315084[] = {{&S29768[8], 3}, {NULL}};
const Segment SGI315085[] = {{&S29768[13], 10}, {NULL}};
const Segment SGI315086[] = {{&S29768[42], 7}, {NULL}};
const Segment SGI315087[] = {{&S29768[35], 14}, {NULL}};
const Segment SGI315088[] = {{&S29768[146], 15}, {NULL}};
const Segment SGI315089[] = {{&S29768[64], 13}, {&S29768[64], 13}, {NULL}};
const Segment SGI315090[] = {{&S29769[30], 3}, {NULL}};
const Segment SGI315091[] = {{&S29769[8], 3}, {NULL}};
const Segment SGI315092[] = {{&S29769[13], 10}, {NULL}};
const Segment SGI315093[] = {{&S29769[42], 7}, {NULL}};
const Segment SGI315094[] = {{&S29769[35], 14}, {NULL}};
const Segment SGI315095[] = {{&S29769[51], 6}, {NULL}};
const Segment SGI315096[] = {{&S29769[148], 13}, {NULL}};
const Segment SGI315097[] = {{&S29770[30], 3}, {NULL}};
const Segment SGI315098[] = {{&S29770[8], 3}, {NULL}};
const Segment SGI315099[] = {{&S29770[13], 10}, {NULL}};
const Segment SGI315100[] = {{&S29770[42], 7}, {NULL}};
const Segment SGI315101[] = {{&S29770[35], 14}, {NULL}};
const Segment SGI315102[] = {{&S29770[153], 15}, {NULL}};
const Segment SGI315103[] = {{&S29770[51], 6}, {NULL}};
const Segment SGI315104[] = {{&S29770[71], 13}, {&S29770[71], 13}, {NULL}};
const Segment SGI315105[] = {{&S29771[30], 3}, {NULL}};
const Segment SGI315106[] = {{&S29771[8], 3}, {NULL}};
const Segment SGI315107[] = {{&S29771[13], 10}, {NULL}};
const Segment SGI315108[] = {{&S29771[55], 7}, {NULL}};
const Segment SGI315109[] = {{&S29771[48], 14}, {NULL}};
const Segment SGI315110[] = {{&S29771[64], 13}, {NULL}};
const Segment SGI315111[] = {{&S29772[30], 3}, {NULL}};
const Segment SGI315112[] = {{&S29772[8], 3}, {NULL}};
const Segment SGI315113[] = {{&S29772[13], 10}, {NULL}};
const Segment SGI315114[] = {{&S29772[42], 7}, {NULL}};
const Segment SGI315115[] = {{&S29772[35], 14}, {NULL}};
const Segment SGI315116[] = {{&S29772[146], 15}, {NULL}};
const Segment SGI315117[] = {{&S29772[64], 13}, {&S29772[64], 13}, {NULL}};
const Segment SGI315118[] = {{&S29772[167], 9}, {NULL}};
const Segment SGI315119[] = {{&S29773[30], 3}, {NULL}};
const Segment SGI315120[] = {{&S29773[8], 3}, {NULL}};
const Segment SGI315121[] = {{&S29773[13], 10}, {NULL}};
const Segment SGI315122[] = {{&S29773[42], 7}, {NULL}};
const Segment SGI315123[] = {{&S29773[35], 14}, {NULL}};
const Segment SGI315124[] = {{&S29773[141], 13}, {NULL}};
const Segment SGI315125[] = {{&S29774[30], 3}, {NULL}};
const Segment SGI315126[] = {{&S29774[8], 3}, {NULL}};
const Segment SGI315127[] = {{&S29774[13], 10}, {NULL}};
const Segment SGI315128[] = {{&S29774[42], 7}, {NULL}};
const Segment SGI315129[] = {{&S29774[35], 14}, {NULL}};
const Segment SGI315130[] = {{&S29774[143], 15}, {NULL}};
const Segment SGI315131[] = {{&S29774[184], 13}, {NULL}};
const Segment SGI315132[] = {{&S29774[51], 5}, {NULL}};
const Segment SGI315133[] = {{&S29775[30], 3}, {NULL}};
const Segment SGI315134[] = {{&S29775[8], 3}, {NULL}};
const Segment SGI315135[] = {{&S29775[13], 10}, {NULL}};
const Segment SGI315136[] = {{&S29775[42], 7}, {NULL}};
const Segment SGI315137[] = {{&S29775[35], 14}, {NULL}};
const Segment SGI315138[] = {{&S29775[138], 15}, {NULL}};
const Segment SGI315139[] = {{&S29775[169], 4}, {NULL}};
const Segment SGI315140[] = {{&S29775[51], 6}, {NULL}};
const Segment SGI315141[] = {{&S29775[179], 13}, {NULL}};
const Segment SGI315142[] = {{&S29776[30], 3}, {NULL}};
const Segment SGI315143[] = {{&S29776[8], 3}, {NULL}};
const Segment SGI315144[] = {{&S29776[13], 10}, {NULL}};
const Segment SGI315145[] = {{&S29776[42], 7}, {NULL}};
const Segment SGI315146[] = {{&S29776[35], 14}, {NULL}};
const Segment SGI315147[] = {{&S29776[64], 13}, {&S29776[64], 13}, {NULL}};
const Segment SGI315148[] = {{&S29777[30], 3}, {NULL}};
const Segment SGI315149[] = {{&S29777[8], 3}, {NULL}};
const Segment SGI315150[] = {{&S29777[13], 10}, {NULL}};
const Segment SGI315151[] = {{&S29777[42], 7}, {NULL}};
const Segment SGI315152[] = {{&S29777[35], 14}, {NULL}};
const Segment SGI315153[] = {{&S29777[51], 6}, {NULL}};
const Segment SGI315154[] = {{&S29777[71], 13}, {&S29777[71], 13}, {NULL}};
const Segment SGI315155[] = {{&S29778[30], 3}, {NULL}};
const Segment SGI315156[] = {{&S29778[8], 3}, {NULL}};
const Segment SGI315157[] = {{&S29778[13], 10}, {NULL}};
const Segment SGI315158[] = {{&S29778[42], 7}, {NULL}};
const Segment SGI315159[] = {{&S29778[35], 14}, {NULL}};
const Segment SGI315160[] = {{&S29778[143], 15}, {NULL}};
const Segment SGI315161[] = {{&S29778[211], 13}, {NULL}};
const Segment SGI315162[] = {{&S29778[51], 5}, {NULL}};
const Segment SGI315163[] = {{&S29778[184], 9}, {NULL}};
const Segment SGI315164[] = {{&S29833[30], 3}, {NULL}};
const Segment SGI315165[] = {{&S29833[8], 3}, {NULL}};
const Segment SGI315166[] = {{&S29833[13], 10}, {NULL}};
const Segment SGI315167[] = {{&S29833[42], 7}, {NULL}};
const Segment SGI315168[] = {{&S29833[35], 14}, {NULL}};
const Segment SGI315169[] = {{&S29833[64], 13}, {NULL}};
const Segment SGI315170[] = {{&S29834[30], 3}, {NULL}};
const Segment SGI315171[] = {{&S29834[8], 3}, {NULL}};
const Segment SGI315172[] = {{&S29834[13], 10}, {NULL}};
const Segment SGI315173[] = {{&S29834[42], 7}, {NULL}};
const Segment SGI315174[] = {{&S29834[35], 14}, {NULL}};
const Segment SGI315175[] = {{&S29834[64], 13}, {&S29834[64], 13}, {NULL}};
const Segment SGI315176[] = {{&S29835[30], 3}, {&S29835[94], 3}, {NULL}};
const Segment SGI315177[] = {{&S29835[8], 3}, {&S29835[8], 3}, {NULL}};
const Segment SGI315178[] = {{&S29835[13], 10}, {&S29835[13], 10}, {NULL}};
const Segment SGI315179[] = {{&S29835[42], 7}, {&S29835[42], 7}, {NULL}};
const Segment SGI315180[] = {{&S29835[35], 14}, {&S29835[35], 14}, {NULL}};
const Segment SGI315181[] = {{&S29835[122], 13}, {NULL}};
const Segment SGI315182[] = {{&S29835[115], 3}, {NULL}};
const Segment SGI315183[] = {{&S29836[30], 3}, {NULL}};
const Segment SGI315184[] = {{&S29836[8], 3}, {NULL}};
const Segment SGI315185[] = {{&S29836[13], 10}, {NULL}};
const Segment SGI315186[] = {{&S29836[55], 7}, {NULL}};
const Segment SGI315187[] = {{&S29836[48], 14}, {NULL}};
const Segment SGI315188[] = {{&S29836[64], 13}, {NULL}};
const Segment SGI315189[] = {{&S29837[30], 3}, {NULL}};
const Segment SGI315190[] = {{&S29837[8], 3}, {NULL}};
const Segment SGI315191[] = {{&S29837[13], 10}, {NULL}};
const Segment SGI315192[] = {{&S29837[55], 7}, {NULL}};
const Segment SGI315193[] = {{&S29837[48], 14}, {NULL}};
const Segment SGI315194[] = {{&S29837[64], 13}, {NULL}};
const Segment SGI315195[] = {{&S29838[30], 3}, {NULL}};
const Segment SGI315196[] = {{&S29838[8], 3}, {NULL}};
const Segment SGI315197[] = {{&S29838[13], 10}, {NULL}};
const Segment SGI315198[] = {{&S29838[55], 7}, {NULL}};
const Segment SGI315199[] = {{&S29838[48], 14}, {NULL}};
const Segment SGI315200[] = {{&S29838[64], 13}, {NULL}};
const Segment SGI315201[] = {{&S29839[30], 3}, {&S29839[94], 3}, {NULL}};
const Segment SGI315202[] = {{&S29839[8], 3}, {&S29839[8], 3}, {NULL}};
const Segment SGI315203[] = {{&S29839[13], 10}, {&S29839[13], 10}, {NULL}};
const Segment SGI315204[] = {{&S29839[42], 7}, {&S29839[42], 7}, {NULL}};
const Segment SGI315205[] = {{&S29839[35], 14}, {&S29839[35], 14}, {NULL}};
const Segment SGI315206[] = {{&S29839[122], 13}, {&S29839[122], 13}, {NULL}};
const Segment SGI315207[] = {{&S29839[115], 3}, {NULL}};
const Segment SGI315208[] = {{&S29840[30], 3}, {NULL}};
const Segment SGI315209[] = {{&S29840[8], 3}, {NULL}};
const Segment SGI315210[] = {{&S29840[13], 10}, {NULL}};
const Segment SGI315211[] = {{&S29840[42], 7}, {NULL}};
const Segment SGI315212[] = {{&S29840[35], 14}, {NULL}};
const Segment SGI315213[] = {{&S29840[117], 15}, {NULL}};
const Segment SGI315214[] = {{&S29840[64], 13}, {&S29840[64], 13}, {NULL}};
const Segment SGI315215[] = {{&S29841[30], 3}, {&S29841[102], 3}, {NULL}};
const Segment SGI315216[] = {{&S29841[8], 3}, {&S29841[8], 3}, {NULL}};
const Segment SGI315217[] = {{&S29841[13], 10}, {&S29841[13], 10}, {NULL}};
const Segment SGI315218[] = {{&S29841[42], 7}, {&S29841[42], 7}, {NULL}};
const Segment SGI315219[] = {{&S29841[35], 14}, {&S29841[35], 14}, {NULL}};
const Segment SGI315220[] = {{&S29841[183], 15}, {NULL}};
const Segment SGI315221[] = {{&S29841[130], 13}, {NULL}};
const Segment SGI315222[] = {{&S29841[123], 3}, {NULL}};
const Segment SGI315223[] = {{&S29842[30], 3}, {NULL}};
const Segment SGI315224[] = {{&S29842[8], 3}, {NULL}};
const Segment SGI315225[] = {{&S29842[13], 10}, {NULL}};
const Segment SGI315226[] = {{&S29842[55], 7}, {NULL}};
const Segment SGI315227[] = {{&S29842[48], 14}, {NULL}};
const Segment SGI315228[] = {{&S29842[64], 13}, {&S29842[64], 13}, {NULL}};
const Segment SGI315229[] = {{&S29843[30], 3}, {NULL}};
const Segment SGI315230[] = {{&S29843[8], 3}, {NULL}};
const Segment SGI315231[] = {{&S29843[13], 10}, {NULL}};
const Segment SGI315232[] = {{&S29843[55], 7}, {NULL}};
const Segment SGI315233[] = {{&S29843[48], 14}, {NULL}};
const Segment SGI315234[] = {{&S29843[64], 13}, {&S29843[64], 13}, {NULL}};
const Segment SGI315235[] = {{&S29844[30], 3}, {NULL}};
const Segment SGI315236[] = {{&S29844[8], 3}, {NULL}};
const Segment SGI315237[] = {{&S29844[13], 10}, {NULL}};
const Segment SGI315238[] = {{&S29844[55], 7}, {NULL}};
const Segment SGI315239[] = {{&S29844[48], 14}, {NULL}};
const Segment SGI315240[] = {{&S29844[64], 13}, {NULL}};
const Segment SGI315241[] = {{&S29844[79], 3}, {NULL}};
const Segment SGI315242[] = {{&S29845[30], 3}, {NULL}};
const Segment SGI315243[] = {{&S29845[8], 3}, {NULL}};
const Segment SGI315244[] = {{&S29845[13], 10}, {NULL}};
const Segment SGI315245[] = {{&S29845[42], 7}, {NULL}};
const Segment SGI315246[] = {{&S29845[35], 14}, {NULL}};
const Segment SGI315247[] = {{&S29845[138], 15}, {NULL}};
const Segment SGI315248[] = {{&S29845[51], 6}, {NULL}};
const Segment SGI315249[] = {{&S29845[169], 13}, {NULL}};
const Segment SGI315250[] = {{&S29846[30], 3}, {NULL}};
const Segment SGI315251[] = {{&S29846[8], 3}, {NULL}};
const Segment SGI315252[] = {{&S29846[13], 10}, {NULL}};
const Segment SGI315253[] = {{&S29846[55], 7}, {NULL}};
const Segment SGI315254[] = {{&S29846[48], 14}, {NULL}};
const Segment SGI315255[] = {{&S29846[64], 13}, {&S29846[64], 13}, {NULL}};
const Segment SGI315256[] = {{&S29847[30], 3}, {NULL}};
const Segment SGI315257[] = {{&S29847[8], 3}, {NULL}};
const Segment SGI315258[] = {{&S29847[13], 10}, {NULL}};
const Segment SGI315259[] = {{&S29847[55], 7}, {NULL}};
const Segment SGI315260[] = {{&S29847[48], 14}, {NULL}};
const Segment SGI315261[] = {{&S29847[69], 13}, {NULL}};
const Segment SGI315262[] = {{&S29847[64], 3}, {NULL}};
const Segment SGI315263[] = {{&S29848[30], 3}, {NULL}};
const Segment SGI315264[] = {{&S29848[8], 3}, {NULL}};
const Segment SGI315265[] = {{&S29848[13], 10}, {NULL}};
const Segment SGI315266[] = {{&S29848[55], 7}, {NULL}};
const Segment SGI315267[] = {{&S29848[48], 14}, {NULL}};
const Segment SGI315268[] = {{&S29848[69], 13}, {NULL}};
const Segment SGI315269[] = {{&S29848[64], 3}, {NULL}};
const Segment SGI315270[] = {{&S29849[30], 3}, {NULL}};
const Segment SGI315271[] = {{&S29849[8], 3}, {NULL}};
const Segment SGI315272[] = {{&S29849[13], 10}, {NULL}};
const Segment SGI315273[] = {{&S29849[55], 7}, {NULL}};
const Segment SGI315274[] = {{&S29849[48], 14}, {NULL}};
const Segment SGI315275[] = {{&S29849[69], 13}, {NULL}};
const Segment SGI315276[] = {{&S29849[64], 3}, {NULL}};
const Segment SGI315277[] = {{&S29850[30], 3}, {NULL}};
const Segment SGI315278[] = {{&S29850[8], 3}, {NULL}};
const Segment SGI315279[] = {{&S29850[13], 10}, {NULL}};
const Segment SGI315280[] = {{&S29850[42], 7}, {NULL}};
const Segment SGI315281[] = {{&S29850[35], 14}, {NULL}};
const Segment SGI315282[] = {{&S29850[102], 15}, {NULL}};
const Segment SGI315283[] = {{&S29850[187], 13}, {NULL}};
const Segment SGI315284[] = {{&S29851[30], 3}, {NULL}};
const Segment SGI315285[] = {{&S29851[8], 3}, {NULL}};
const Segment SGI315286[] = {{&S29851[13], 10}, {NULL}};
const Segment SGI315287[] = {{&S29851[42], 7}, {NULL}};
const Segment SGI315288[] = {{&S29851[35], 14}, {NULL}};
const Segment SGI315289[] = {{&S29851[64], 13}, {&S29851[64], 13}, {&S29851[64], 13}, {NULL}};
const Segment SGI315290[] = {{&S63856[7], 3}, {NULL}};
const Segment SGI315291[] = {{&S63856[0], 110}, {NULL}};
const Segment SGI315292[] = {{&S30177[30], 3}, {NULL}};
const Segment SGI315293[] = {{&S30177[8], 3}, {NULL}};
const Segment SGI315294[] = {{&S30177[13], 10}, {NULL}};
const Segment SGI315295[] = {{&S30177[42], 7}, {NULL}};
const Segment SGI315296[] = {{&S30177[35], 14}, {NULL}};
const Segment SGI315297[] = {{&S30177[64], 11}, {NULL}};
const Segment SGI315298[] = {{&S30177[64], 16}, {NULL}};
const Segment SGI315299[] = {{&S30178[30], 3}, {NULL}};
const Segment SGI315300[] = {{&S30178[8], 3}, {NULL}};
const Segment SGI315301[] = {{&S30178[13], 10}, {NULL}};
const Segment SGI315302[] = {{&S30178[42], 7}, {NULL}};
const Segment SGI315303[] = {{&S30178[35], 14}, {NULL}};
const Segment SGI315304[] = {{&S30178[71], 11}, {NULL}};
const Segment SGI315305[] = {{&S30178[51], 6}, {NULL}};
const Segment SGI315306[] = {{&S30178[71], 16}, {NULL}};
const Segment SGI315307[] = {{&S30179[30], 3}, {NULL}};
const Segment SGI315308[] = {{&S30179[8], 3}, {NULL}};
const Segment SGI315309[] = {{&S30179[13], 10}, {NULL}};
const Segment SGI315310[] = {{&S30179[42], 7}, {NULL}};
const Segment SGI315311[] = {{&S30179[35], 14}, {NULL}};
const Segment SGI315312[] = {{&S30179[59], 11}, {NULL}};
const Segment SGI315313[] = {{&S30179[112], 15}, {NULL}};
const Segment SGI315314[] = {{&S30179[59], 16}, {NULL}};
const Segment SGI315315[] = {{&S30180[30], 3}, {NULL}};
const Segment SGI315316[] = {{&S30180[8], 3}, {NULL}};
const Segment SGI315317[] = {{&S30180[13], 10}, {NULL}};
const Segment SGI315318[] = {{&S30180[42], 7}, {NULL}};
const Segment SGI315319[] = {{&S30180[35], 14}, {NULL}};
const Segment SGI315320[] = {{&S30180[59], 11}, {NULL}};
const Segment SGI315321[] = {{&S30180[59], 16}, {NULL}};
const Segment SGI315322[] = {{&S30182[30], 3}, {NULL}};
const Segment SGI315323[] = {{&S30182[8], 3}, {NULL}};
const Segment SGI315324[] = {{&S30182[13], 10}, {NULL}};
const Segment SGI315325[] = {{&S30182[42], 7}, {NULL}};
const Segment SGI315326[] = {{&S30182[35], 14}, {NULL}};
const Segment SGI315327[] = {{&S30182[79], 11}, {NULL}};
const Segment SGI315328[] = {{&S30182[132], 15}, {NULL}};
const Segment SGI315329[] = {{&S30182[51], 6}, {NULL}};
const Segment SGI315330[] = {{&S30182[79], 16}, {NULL}};
const Segment SGI315331[] = {{&S30183[30], 3}, {NULL}};
const Segment SGI315332[] = {{&S30183[8], 3}, {NULL}};
const Segment SGI315333[] = {{&S30183[13], 10}, {NULL}};
const Segment SGI315334[] = {{&S30183[42], 7}, {NULL}};
const Segment SGI315335[] = {{&S30183[35], 14}, {NULL}};
const Segment SGI315336[] = {{&S30183[66], 11}, {NULL}};
const Segment SGI315337[] = {{&S30183[119], 15}, {NULL}};
const Segment SGI315338[] = {{&S30183[51], 6}, {NULL}};
const Segment SGI315339[] = {{&S30183[66], 16}, {NULL}};
const Segment SGI315340[] = {{&S30184[30], 3}, {NULL}};
const Segment SGI315341[] = {{&S30184[8], 3}, {NULL}};
const Segment SGI315342[] = {{&S30184[13], 10}, {NULL}};
const Segment SGI315343[] = {{&S30184[42], 7}, {NULL}};
const Segment SGI315344[] = {{&S30184[35], 14}, {NULL}};
const Segment SGI315345[] = {{&S30184[72], 11}, {NULL}};
const Segment SGI315346[] = {{&S30184[125], 15}, {NULL}};
const Segment SGI315347[] = {{&S30184[72], 16}, {NULL}};
const Segment SGI315348[] = {{&S30185[30], 3}, {NULL}};
const Segment SGI315349[] = {{&S30185[8], 3}, {NULL}};
const Segment SGI315350[] = {{&S30185[13], 10}, {NULL}};
const Segment SGI315351[] = {{&S30185[42], 7}, {NULL}};
const Segment SGI315352[] = {{&S30185[35], 14}, {NULL}};
const Segment SGI315353[] = {{&S30185[72], 11}, {NULL}};
const Segment SGI315354[] = {{&S30185[72], 16}, {NULL}};
const Segment SGI315355[] = {{&S30186[30], 3}, {&S30186[46], 3}, {NULL}};
const Segment SGI315356[] = {{&S30186[8], 3}, {&S30186[8], 3}, {NULL}};
const Segment SGI315357[] = {{&S30186[13], 10}, {&S30186[13], 10}, {NULL}};
const Segment SGI315358[] = {{&S30186[42], 7}, {&S30186[127], 7}, {NULL}};
const Segment SGI315359[] = {{&S30186[35], 14}, {&S30186[120], 14}, {NULL}};
const Segment SGI315360[] = {{&S30186[59], 11}, {NULL}};
const Segment SGI315361[] = {{&S30186[170], 15}, {NULL}};
const Segment SGI315362[] = {{&S30186[59], 16}, {NULL}};
const Segment SGI315363[] = {{&S30186[136], 3}, {NULL}};
const Segment SGI315364[] = {{&S44876[0], 78}, {NULL}};
const Segment SGI315365[] = {{&S44877[0], 77}, {NULL}};
const Segment SGI315366[] = {{&S44878[0], 79}, {NULL}};
const Segment SGI315367[] = {{&S31066[30], 3}, {NULL}};
const Segment SGI315368[] = {{&S31066[8], 3}, {NULL}};
const Segment SGI315369[] = {{&S31066[13], 10}, {NULL}};
const Segment SGI315370[] = {{&S31066[42], 7}, {NULL}};
const Segment SGI315371[] = {{&S31067[30], 3}, {NULL}};
const Segment SGI315372[] = {{&S31067[8], 3}, {NULL}};
const Segment SGI315373[] = {{&S31067[13], 10}, {NULL}};
const Segment SGI315374[] = {{&S31067[42], 7}, {NULL}};
const Segment SGI315375[] = {{&S31067[35], 14}, {NULL}};
const Segment SGI315376[] = {{&S31068[30], 3}, {&S31068[8], 3}, {NULL}};
const Segment SGI315377[] = {{&S31068[8], 3}, {&S31068[8], 3}, {NULL}};
const Segment SGI315378[] = {{&S31068[13], 10}, {&S31068[64], 10}, {NULL}};
const Segment SGI315379[] = {{&S31068[42], 7}, {&S31068[42], 3}, {&S31068[105], 3}, {NULL}};
const Segment SGI315380[] = {{&S31068[35], 14}, {NULL}};
const Segment SGI315381[] = {{&S31069[16], 3}, {NULL}};
const Segment SGI315382[] = {{&S31069[38], 7}, {NULL}};
const Segment SGI315383[] = {{&S31069[31], 14}, {NULL}};
const Segment SGI315384[] = {{&S31069[47], 6}, {NULL}};
const Segment SGI315385[] = {{&S31070[16], 3}, {NULL}};
const Segment SGI315386[] = {{&S31070[38], 7}, {NULL}};
const Segment SGI315387[] = {{&S31070[31], 14}, {NULL}};
const Segment SGI315388[] = {{&S31071[16], 3}, {NULL}};
const Segment SGI315389[] = {{&S31071[38], 7}, {NULL}};
const Segment SGI315390[] = {{&S31071[31], 14}, {NULL}};
const Segment SGI315391[] = {{&S31071[147], 4}, {NULL}};
const Segment SGI315392[] = {{&S31071[47], 6}, {NULL}};
const Segment SGI315393[] = {{&S31072[16], 3}, {NULL}};
const Segment SGI315394[] = {{&S31072[38], 7}, {NULL}};
const Segment SGI315395[] = {{&S31072[31], 14}, {NULL}};
const Segment SGI315396[] = {{&S31072[47], 6}, {NULL}};
const Segment SGI315397[] = {{&S31072[147], 9}, {NULL}};
const Segment SGI315398[] = {{&S31073[16], 3}, {NULL}};
const Segment SGI315399[] = {{&S31073[38], 7}, {NULL}};
const Segment SGI315400[] = {{&S31073[31], 14}, {NULL}};
const Segment SGI315401[] = {{&S31073[147], 15}, {NULL}};
const Segment SGI315402[] = {{&S31073[47], 6}, {NULL}};
const Segment SGI315403[] = {{&S31074[16], 3}, {NULL}};
const Segment SGI315404[] = {{&S31074[38], 7}, {NULL}};
const Segment SGI315405[] = {{&S31074[31], 14}, {NULL}};
const Segment SGI315406[] = {{&S31074[162], 15}, {NULL}};
const Segment SGI315407[] = {{&S31074[47], 6}, {NULL}};
const Segment SGI315408[] = {{&S31074[147], 9}, {NULL}};
const Segment SGI315409[] = {{&S31075[16], 3}, {NULL}};
const Segment SGI315410[] = {{&S31075[38], 7}, {NULL}};
const Segment SGI315411[] = {{&S31075[31], 14}, {NULL}};
const Segment SGI315412[] = {{&S31075[140], 15}, {NULL}};
const Segment SGI315413[] = {{&S31076[16], 3}, {NULL}};
const Segment SGI315414[] = {{&S31076[38], 7}, {NULL}};
const Segment SGI315415[] = {{&S31076[31], 14}, {NULL}};
const Segment SGI315416[] = {{&S31076[140], 4}, {NULL}};
const Segment SGI315417[] = {{&S31077[16], 3}, {NULL}};
const Segment SGI315418[] = {{&S31077[38], 7}, {NULL}};
const Segment SGI315419[] = {{&S31077[31], 14}, {NULL}};
const Segment SGI315420[] = {{&S31077[140], 15}, {NULL}};
const Segment SGI315421[] = {{&S31077[168], 9}, {NULL}};
const Segment SGI315422[] = {{&S31078[16], 3}, {NULL}};
const Segment SGI315423[] = {{&S31078[38], 7}, {NULL}};
const Segment SGI315424[] = {{&S31078[31], 14}, {NULL}};
const Segment SGI315425[] = {{&S31078[147], 15}, {NULL}};
const Segment SGI315426[] = {{&S31078[168], 4}, {NULL}};
const Segment SGI315427[] = {{&S31078[47], 6}, {NULL}};
const Segment SGI315428[] = {{&S31079[16], 3}, {NULL}};
const Segment SGI315429[] = {{&S31079[38], 7}, {NULL}};
const Segment SGI315430[] = {{&S31079[31], 14}, {NULL}};
const Segment SGI315431[] = {{&S31079[140], 15}, {&S31079[140], 15}, {NULL}};
const Segment SGI315432[] = {{&S31080[26], 3}, {NULL}};
const Segment SGI315433[] = {{&S31080[38], 7}, {NULL}};
const Segment SGI315434[] = {{&S31080[31], 14}, {NULL}};
const Segment SGI315435[] = {{&S31080[47], 6}, {NULL}};
const Segment SGI315436[] = {{&S31081[16], 3}, {NULL}};
const Segment SGI315437[] = {{&S31081[38], 7}, {NULL}};
const Segment SGI315438[] = {{&S31081[31], 14}, {NULL}};
const Segment SGI315439[] = {{&S31081[47], 6}, {NULL}};
const Segment SGI315440[] = {{&S31081[147], 13}, {NULL}};
const Segment SGI315441[] = {{&S31082[16], 3}, {NULL}};
const Segment SGI315442[] = {{&S31082[38], 7}, {NULL}};
const Segment SGI315443[] = {{&S31082[31], 14}, {NULL}};
const Segment SGI315444[] = {{&S31082[140], 15}, {&S31082[140], 15}, {NULL}};
const Segment SGI315445[] = {{&S31082[196], 9}, {NULL}};
const Segment SGI315446[] = {{&S31083[26], 3}, {NULL}};
const Segment SGI315447[] = {{&S31083[38], 7}, {NULL}};
const Segment SGI315448[] = {{&S31083[31], 14}, {NULL}};
const Segment SGI315449[] = {{&S31083[147], 15}, {NULL}};
const Segment SGI315450[] = {{&S31083[47], 6}, {NULL}};
const Segment SGI315451[] = {{&S31084[16], 3}, {NULL}};
const Segment SGI315452[] = {{&S31084[38], 7}, {NULL}};
const Segment SGI315453[] = {{&S31084[31], 14}, {NULL}};
const Segment SGI315454[] = {{&S31084[162], 4}, {NULL}};
const Segment SGI315455[] = {{&S31084[47], 6}, {NULL}};
const Segment SGI315456[] = {{&S31084[147], 9}, {NULL}};
const Segment SGI315457[] = {{&S31085[16], 3}, {NULL}};
const Segment SGI315458[] = {{&S31085[38], 7}, {NULL}};
const Segment SGI315459[] = {{&S31085[31], 14}, {NULL}};
const Segment SGI315460[] = {{&S31085[140], 9}, {NULL}};
const Segment SGI315461[] = {{&S31086[26], 4}, {NULL}};
const Segment SGI315462[] = {{&S31086[39], 7}, {NULL}};
const Segment SGI315463[] = {{&S31086[32], 14}, {NULL}};
const Segment SGI315464[] = {{&S31086[48], 6}, {NULL}};
const Segment SGI315465[] = {{&S31087[16], 3}, {NULL}};
const Segment SGI315466[] = {{&S31087[38], 7}, {NULL}};
const Segment SGI315467[] = {{&S31087[31], 14}, {NULL}};
const Segment SGI315468[] = {{&S31087[80], 15}, {NULL}};
const Segment SGI315469[] = {{&S31087[161], 4}, {NULL}};
const Segment SGI315470[] = {{&S31088[30], 3}, {NULL}};
const Segment SGI315471[] = {{&S31088[8], 3}, {NULL}};
const Segment SGI315472[] = {{&S31088[13], 10}, {NULL}};
const Segment SGI315473[] = {{&S31088[42], 7}, {NULL}};
const Segment SGI315474[] = {{&S31088[35], 14}, {NULL}};
const Segment SGI315475[] = {{&S31088[51], 6}, {NULL}};
const Segment SGI315476[] = {{&S31088[71], 15}, {NULL}};
const Segment SGI315477[] = {{&S31089[30], 3}, {NULL}};
const Segment SGI315478[] = {{&S31089[8], 3}, {NULL}};
const Segment SGI315479[] = {{&S31089[13], 10}, {NULL}};
const Segment SGI315480[] = {{&S31089[42], 7}, {NULL}};
const Segment SGI315481[] = {{&S31089[35], 14}, {NULL}};
const Segment SGI315482[] = {{&S31089[64], 15}, {NULL}};
const Segment SGI315483[] = {{&S31090[30], 3}, {NULL}};
const Segment SGI315484[] = {{&S31090[8], 3}, {NULL}};
const Segment SGI315485[] = {{&S31090[13], 10}, {NULL}};
const Segment SGI315486[] = {{&S31090[42], 7}, {NULL}};
const Segment SGI315487[] = {{&S31090[35], 14}, {NULL}};
const Segment SGI315488[] = {{&S31090[51], 6}, {NULL}};
const Segment SGI315489[] = {{&S31090[86], 15}, {NULL}};
const Segment SGI315490[] = {{&S31090[71], 13}, {NULL}};
const Segment SGI315491[] = {{&S31091[30], 3}, {NULL}};
const Segment SGI315492[] = {{&S31091[8], 3}, {NULL}};
const Segment SGI315493[] = {{&S31091[13], 10}, {NULL}};
const Segment SGI315494[] = {{&S31091[42], 7}, {NULL}};
const Segment SGI315495[] = {{&S31091[35], 14}, {NULL}};
const Segment SGI315496[] = {{&S31091[79], 15}, {NULL}};
const Segment SGI315497[] = {{&S31091[64], 13}, {NULL}};
const Segment SGI315498[] = {{&S31092[30], 3}, {NULL}};
const Segment SGI315499[] = {{&S31092[8], 3}, {NULL}};
const Segment SGI315500[] = {{&S31092[13], 10}, {NULL}};
const Segment SGI315501[] = {{&S31092[42], 7}, {NULL}};
const Segment SGI315502[] = {{&S31092[35], 14}, {NULL}};
const Segment SGI315503[] = {{&S31092[51], 6}, {NULL}};
const Segment SGI315504[] = {{&S31092[71], 15}, {NULL}};
const Segment SGI315505[] = {{&S31093[30], 4}, {NULL}};
const Segment SGI315506[] = {{&S31093[8], 3}, {NULL}};
const Segment SGI315507[] = {{&S31093[13], 10}, {NULL}};
const Segment SGI315508[] = {{&S31093[43], 7}, {NULL}};
const Segment SGI315509[] = {{&S31093[36], 14}, {NULL}};
const Segment SGI315510[] = {{&S31093[52], 6}, {NULL}};
const Segment SGI315511[] = {{&S31093[72], 15}, {NULL}};
const Segment SGI315512[] = {{&S31094[30], 4}, {NULL}};
const Segment SGI315513[] = {{&S31094[8], 3}, {NULL}};
const Segment SGI315514[] = {{&S31094[13], 10}, {NULL}};
const Segment SGI315515[] = {{&S31094[43], 7}, {NULL}};
const Segment SGI315516[] = {{&S31094[36], 14}, {NULL}};
const Segment SGI315517[] = {{&S31094[65], 15}, {NULL}};
const Segment SGI315518[] = {{&S31095[30], 4}, {NULL}};
const Segment SGI315519[] = {{&S31095[8], 3}, {NULL}};
const Segment SGI315520[] = {{&S31095[13], 10}, {NULL}};
const Segment SGI315521[] = {{&S31095[43], 7}, {NULL}};
const Segment SGI315522[] = {{&S31095[36], 14}, {NULL}};
const Segment SGI315523[] = {{&S31095[52], 6}, {NULL}};
const Segment SGI315524[] = {{&S31095[87], 15}, {NULL}};
const Segment SGI315525[] = {{&S31095[72], 13}, {NULL}};
const Segment SGI315526[] = {{&S31096[30], 4}, {NULL}};
const Segment SGI315527[] = {{&S31096[8], 3}, {NULL}};
const Segment SGI315528[] = {{&S31096[13], 10}, {NULL}};
const Segment SGI315529[] = {{&S31096[43], 7}, {NULL}};
const Segment SGI315530[] = {{&S31096[36], 14}, {NULL}};
const Segment SGI315531[] = {{&S31096[80], 15}, {NULL}};
const Segment SGI315532[] = {{&S31096[65], 13}, {NULL}};
const Segment SGI315533[] = {{&S31097[30], 3}, {NULL}};
const Segment SGI315534[] = {{&S31097[8], 3}, {NULL}};
const Segment SGI315535[] = {{&S31097[13], 10}, {NULL}};
const Segment SGI315536[] = {{&S31097[42], 7}, {NULL}};
const Segment SGI315537[] = {{&S31097[35], 14}, {NULL}};
const Segment SGI315538[] = {{&S31097[131], 15}, {NULL}};
const Segment SGI315539[] = {{&S31097[152], 15}, {NULL}};
const Segment SGI315540[] = {{&S31528[16], 3}, {NULL}};
const Segment SGI315541[] = {{&S31528[38], 7}, {NULL}};
const Segment SGI315542[] = {{&S31528[31], 14}, {NULL}};
const Segment SGI315543[] = {{&S31528[87], 15}, {NULL}};
const Segment SGI315544[] = {{&S31529[16], 3}, {NULL}};
const Segment SGI315545[] = {{&S31529[38], 7}, {NULL}};
const Segment SGI315546[] = {{&S31529[31], 14}, {NULL}};
const Segment SGI315547[] = {{&S31530[16], 3}, {NULL}};
const Segment SGI315548[] = {{&S31530[38], 7}, {NULL}};
const Segment SGI315549[] = {{&S31530[31], 14}, {NULL}};
const Segment SGI315550[] = {{&S31530[94], 15}, {NULL}};
const Segment SGI315551[] = {{&S31530[47], 6}, {NULL}};
const Segment SGI315552[] = {{&S31531[16], 3}, {&S31531[42], 3}, {NULL}};
const Segment SGI315553[] = {{&S31531[55], 3}, {NULL}};
const Segment SGI315554[] = {{&S31531[38], 7}, {&S31531[91], 7}, {NULL}};
const Segment SGI315555[] = {{&S31531[31], 14}, {&S31531[84], 14}, {NULL}};
const Segment SGI315556[] = {{&S31531[134], 15}, {NULL}};
const Segment SGI315557[] = {{&S31531[100], 3}, {NULL}};
const Segment SGI315558[] = {{&S31532[16], 3}, {NULL}};
const Segment SGI315559[] = {{&S31532[38], 7}, {NULL}};
const Segment SGI315560[] = {{&S31532[31], 14}, {NULL}};
const Segment SGI315561[] = {{&S31532[94], 15}, {&S31532[94], 15}, {NULL}};
const Segment SGI315562[] = {{&S31532[47], 6}, {NULL}};
const Segment SGI315563[] = {{&S31533[16], 3}, {NULL}};
const Segment SGI315564[] = {{&S31533[38], 7}, {NULL}};
const Segment SGI315565[] = {{&S31533[31], 14}, {NULL}};
const Segment SGI315566[] = {{&S31533[87], 15}, {NULL}};
const Segment SGI315567[] = {{&S31533[158], 4}, {NULL}};
const Segment SGI315568[] = {{&S31534[16], 3}, {NULL}};
const Segment SGI315569[] = {{&S31534[38], 7}, {NULL}};
const Segment SGI315570[] = {{&S31534[31], 14}, {NULL}};
const Segment SGI315571[] = {{&S31534[137], 4}, {NULL}};
const Segment SGI315572[] = {{&S31535[16], 3}, {NULL}};
const Segment SGI315573[] = {{&S31535[38], 7}, {NULL}};
const Segment SGI315574[] = {{&S31535[31], 14}, {NULL}};
const Segment SGI315575[] = {{&S31535[47], 6}, {NULL}};
const Segment SGI315576[] = {{&S31536[16], 3}, {NULL}};
const Segment SGI315577[] = {{&S31536[38], 7}, {NULL}};
const Segment SGI315578[] = {{&S31536[31], 14}, {NULL}};
const Segment SGI315579[] = {{&S31536[87], 15}, {NULL}};
const Segment SGI315580[] = {{&S31536[158], 9}, {NULL}};
const Segment SGI315581[] = {{&S31706[30], 3}, {NULL}};
const Segment SGI315582[] = {{&S31706[8], 3}, {NULL}};
const Segment SGI315583[] = {{&S31706[13], 10}, {NULL}};
const Segment SGI315584[] = {{&S31706[42], 3}, {NULL}};
const Segment SGI315585[] = {{&S31707[30], 4}, {NULL}};
const Segment SGI315586[] = {{&S31707[8], 3}, {NULL}};
const Segment SGI315587[] = {{&S31707[13], 10}, {NULL}};
const Segment SGI315588[] = {{&S31707[43], 3}, {NULL}};
const Segment SGI315589[] = {{&S65394[7], 4}, {NULL}};
const Segment SGI315590[] = {{&S65394[0], 111}, {NULL}};
const Segment SGI315591[] = {{&S65398[0], 111}, {NULL}};
const Segment SGI315592[] = {{&S32458[30], 3}, {NULL}};
const Segment SGI315593[] = {{&S32458[8], 3}, {NULL}};
const Segment SGI315594[] = {{&S32458[13], 10}, {NULL}};
const Segment SGI315595[] = {{&S32460[30], 3}, {NULL}};
const Segment SGI315596[] = {{&S32460[8], 3}, {NULL}};
const Segment SGI315597[] = {{&S32460[13], 10}, {NULL}};
const Segment SGI315598[] = {{&S32461[29], 3}, {NULL}};
const Segment SGI315599[] = {{&S32461[8], 3}, {NULL}};
const Segment SGI315600[] = {{&S32463[8], 3}, {NULL}};
const Segment SGI315601[] = {{&S32463[13], 10}, {NULL}};
const Segment SGI315602[] = {{&S32465[30], 4}, {NULL}};
const Segment SGI315603[] = {{&S32465[8], 3}, {NULL}};
const Segment SGI315604[] = {{&S32465[13], 10}, {NULL}};
const Segment SGI315605[] = {{&S32466[30], 3}, {NULL}};
const Segment SGI315606[] = {{&S32466[8], 3}, {NULL}};
const Segment SGI315607[] = {{&S32466[13], 20}, {NULL}};
const Segment SGI315608[] = {{&S32823[30], 4}, {NULL}};
const Segment SGI315609[] = {{&S32823[8], 3}, {NULL}};
const Segment SGI315610[] = {{&S32823[13], 10}, {NULL}};
const Segment SGI315611[] = {{&S32823[43], 7}, {NULL}};
const Segment SGI315612[] = {{&S32823[36], 14}, {NULL}};
const Segment SGI315613[] = {{&S32823[65], 13}, {NULL}};
const Segment SGI315614[] = {{&S32824[30], 4}, {NULL}};
const Segment SGI315615[] = {{&S32824[8], 3}, {NULL}};
const Segment SGI315616[] = {{&S32824[13], 10}, {NULL}};
const Segment SGI315617[] = {{&S32824[43], 7}, {NULL}};
const Segment SGI315618[] = {{&S32824[36], 14}, {NULL}};
const Segment SGI315619[] = {{&S32824[52], 6}, {NULL}};
const Segment SGI315620[] = {{&S32824[72], 13}, {NULL}};
const Segment SGI315621[] = {{&S32825[30], 4}, {NULL}};
const Segment SGI315622[] = {{&S32825[8], 3}, {NULL}};
const Segment SGI315623[] = {{&S32825[13], 10}, {NULL}};
const Segment SGI315624[] = {{&S32825[43], 7}, {NULL}};
const Segment SGI315625[] = {{&S32825[36], 14}, {NULL}};
const Segment SGI315626[] = {{&S32825[77], 13}, {NULL}};
const Segment SGI315627[] = {{&S32825[52], 5}, {NULL}};
const Segment SGI315628[] = {{&S61201[3], 6}, {NULL}};
const Segment SGI315629[] = {{&S61201[3], 13}, {NULL}};
const Segment SGI315630[] = {{&S61201[36], 8}, {NULL}};
const Segment SGI315631[] = {{&S61202[3], 6}, {NULL}};
const Segment SGI315632[] = {{&S61202[29], 8}, {NULL}};
const Segment SGI315633[] = {{&S61203[3], 6}, {NULL}};
const Segment SGI315634[] = {{&S61203[3], 15}, {NULL}};
const Segment SGI315635[] = {{&S61203[38], 8}, {NULL}};
const Segment SGI315636[] = {{&S33290[30], 3}, {NULL}};
const Segment SGI315637[] = {{&S33290[8], 3}, {NULL}};
const Segment SGI315638[] = {{&S33290[13], 10}, {NULL}};
const Segment SGI315639[] = {{&S33290[35], 5}, {NULL}};
const Segment SGI315640[] = {{&S33291[30], 3}, {NULL}};
const Segment SGI315641[] = {{&S33291[8], 3}, {NULL}};
const Segment SGI315642[] = {{&S33291[13], 10}, {NULL}};
const Segment SGI315643[] = {{&S33291[35], 5}, {NULL}};
const Segment SGI315644[] = {{&S33293[30], 3}, {NULL}};
const Segment SGI315645[] = {{&S33293[8], 3}, {NULL}};
const Segment SGI315646[] = {{&S33293[13], 10}, {NULL}};
const Segment SGI315647[] = {{&S33294[30], 3}, {NULL}};
const Segment SGI315648[] = {{&S33294[8], 3}, {NULL}};
const Segment SGI315649[] = {{&S33294[13], 10}, {NULL}};
const Segment SGI315650[] = {{&S33294[42], 3}, {NULL}};
const Segment SGI315651[] = {{&S33295[30], 3}, {NULL}};
const Segment SGI315652[] = {{&S33295[8], 3}, {NULL}};
const Segment SGI315653[] = {{&S33295[13], 10}, {NULL}};
const Segment SGI315654[] = {{&S33296[30], 3}, {NULL}};
const Segment SGI315655[] = {{&S33296[8], 3}, {NULL}};
const Segment SGI315656[] = {{&S33296[13], 10}, {NULL}};
const Segment SGI315657[] = {{&S33296[40], 5}, {NULL}};
const Segment SGI315658[] = {{&S33297[30], 3}, {NULL}};
const Segment SGI315659[] = {{&S33297[8], 3}, {NULL}};
const Segment SGI315660[] = {{&S33297[13], 10}, {NULL}};
const Segment SGI315661[] = {{&S33297[40], 3}, {NULL}};
const Segment SGI315662[] = {{&S33298[30], 3}, {NULL}};
const Segment SGI315663[] = {{&S33298[8], 3}, {NULL}};
const Segment SGI315664[] = {{&S33298[13], 10}, {NULL}};
const Segment SGI315665[] = {{&S33298[40], 6}, {NULL}};
const Segment SGI315666[] = {{&S33298[35], 3}, {NULL}};
const Segment SGI315667[] = {{&S33299[30], 3}, {NULL}};
const Segment SGI315668[] = {{&S33299[8], 3}, {NULL}};
const Segment SGI315669[] = {{&S33299[13], 10}, {NULL}};
const Segment SGI315670[] = {{&S33299[35], 3}, {NULL}};
const Segment SGI315671[] = {{&S33300[30], 3}, {NULL}};
const Segment SGI315672[] = {{&S33300[8], 3}, {NULL}};
const Segment SGI315673[] = {{&S33300[13], 10}, {NULL}};
const Segment SGI315674[] = {{&S33300[40], 6}, {NULL}};
const Segment SGI315675[] = {{&S33301[30], 3}, {NULL}};
const Segment SGI315676[] = {{&S33301[8], 3}, {NULL}};
const Segment SGI315677[] = {{&S33301[13], 10}, {NULL}};
const Segment SGI315678[] = {{&S33301[40], 5}, {NULL}};
const Segment SGI315679[] = {{&S33301[35], 3}, {NULL}};
const Segment SGI315680[] = {{&S33713[30], 3}, {NULL}};
const Segment SGI315681[] = {{&S33713[8], 3}, {NULL}};
const Segment SGI315682[] = {{&S33713[13], 10}, {NULL}};
const Segment SGI315683[] = {{&S33713[42], 7}, {NULL}};
const Segment SGI315684[] = {{&S33713[35], 14}, {NULL}};
const Segment SGI315685[] = {{&S33713[71], 5}, {NULL}};
const Segment SGI315686[] = {{&S33713[51], 6}, {NULL}};
const Segment SGI315687[] = {{&S33714[30], 3}, {NULL}};
const Segment SGI315688[] = {{&S33714[8], 3}, {NULL}};
const Segment SGI315689[] = {{&S33714[13], 10}, {NULL}};
const Segment SGI315690[] = {{&S33714[42], 7}, {NULL}};
const Segment SGI315691[] = {{&S33714[35], 14}, {NULL}};
const Segment SGI315692[] = {{&S33714[76], 5}, {NULL}};
const Segment SGI315693[] = {{&S33714[51], 5}, {NULL}};
const Segment SGI315694[] = {{&S33715[30], 3}, {NULL}};
const Segment SGI315695[] = {{&S33715[8], 3}, {NULL}};
const Segment SGI315696[] = {{&S33715[13], 10}, {NULL}};
const Segment SGI315697[] = {{&S33715[42], 7}, {NULL}};
const Segment SGI315698[] = {{&S33715[35], 14}, {NULL}};
const Segment SGI315699[] = {{&S33715[64], 5}, {NULL}};
const Segment SGI315700[] = {{&S33716[30], 3}, {NULL}};
const Segment SGI315701[] = {{&S33716[8], 3}, {NULL}};
const Segment SGI315702[] = {{&S33716[13], 10}, {NULL}};
const Segment SGI315703[] = {{&S33716[42], 7}, {NULL}};
const Segment SGI315704[] = {{&S33716[35], 14}, {NULL}};
const Segment SGI315705[] = {{&S33716[71], 5}, {NULL}};
const Segment SGI315706[] = {{&S33716[51], 6}, {NULL}};
const Segment SGI315707[] = {{&S33716[158], 9}, {NULL}};
const Segment SGI315708[] = {{&S33717[30], 3}, {NULL}};
const Segment SGI315709[] = {{&S33717[8], 3}, {NULL}};
const Segment SGI315710[] = {{&S33717[13], 10}, {NULL}};
const Segment SGI315711[] = {{&S33717[42], 7}, {NULL}};
const Segment SGI315712[] = {{&S33717[35], 14}, {NULL}};
const Segment SGI315713[] = {{&S33717[76], 5}, {NULL}};
const Segment SGI315714[] = {{&S33717[51], 5}, {NULL}};
const Segment SGI315715[] = {{&S33717[103], 9}, {NULL}};
const Segment SGI315716[] = {{&S33718[30], 3}, {NULL}};
const Segment SGI315717[] = {{&S33718[8], 3}, {NULL}};
const Segment SGI315718[] = {{&S33718[13], 10}, {NULL}};
const Segment SGI315719[] = {{&S33718[42], 7}, {NULL}};
const Segment SGI315720[] = {{&S33718[35], 14}, {NULL}};
const Segment SGI315721[] = {{&S33718[166], 11}, {NULL}};
const Segment SGI315722[] = {{&S33718[166], 15}, {NULL}};
const Segment SGI315723[] = {{&S33719[30], 3}, {NULL}};
const Segment SGI315724[] = {{&S33719[8], 3}, {NULL}};
const Segment SGI315725[] = {{&S33719[13], 10}, {NULL}};
const Segment SGI315726[] = {{&S33719[42], 7}, {NULL}};
const Segment SGI315727[] = {{&S33719[35], 14}, {NULL}};
const Segment SGI315728[] = {{&S33719[145], 11}, {NULL}};
const Segment SGI315729[] = {{&S33719[145], 15}, {NULL}};
const Segment SGI315730[] = {{&S33721[30], 3}, {NULL}};
const Segment SGI315731[] = {{&S33721[8], 3}, {NULL}};
const Segment SGI315732[] = {{&S33721[13], 10}, {NULL}};
const Segment SGI315733[] = {{&S33721[42], 7}, {NULL}};
const Segment SGI315734[] = {{&S33721[35], 14}, {NULL}};
const Segment SGI315735[] = {{&S33721[150], 11}, {NULL}};
const Segment SGI315736[] = {{&S33721[150], 15}, {NULL}};
const Segment SGI315737[] = {{&S33721[51], 3}, {NULL}};
const Segment SGI315738[] = {{&S33723[30], 3}, {NULL}};
const Segment SGI315739[] = {{&S33723[8], 3}, {NULL}};
const Segment SGI315740[] = {{&S33723[13], 10}, {NULL}};
const Segment SGI315741[] = {{&S33723[42], 7}, {NULL}};
const Segment SGI315742[] = {{&S33723[51], 11}, {NULL}};
const Segment SGI315743[] = {{&S33723[51], 15}, {NULL}};
const Segment SGI315744[] = {{&S33724[30], 3}, {NULL}};
const Segment SGI315745[] = {{&S33724[8], 3}, {NULL}};
const Segment SGI315746[] = {{&S33724[13], 10}, {NULL}};
const Segment SGI315747[] = {{&S33724[42], 7}, {NULL}};
const Segment SGI315748[] = {{&S33724[35], 14}, {NULL}};
const Segment SGI315749[] = {{&S33724[197], 11}, {NULL}};
const Segment SGI315750[] = {{&S33724[197], 15}, {NULL}};
const Segment SGI315751[] = {{&S33724[51], 3}, {&S33724[51], 3}, {NULL}};
const Segment SGI315752[] = {{&S33725[30], 3}, {NULL}};
const Segment SGI315753[] = {{&S33725[8], 3}, {NULL}};
const Segment SGI315754[] = {{&S33725[13], 10}, {NULL}};
const Segment SGI315755[] = {{&S33725[42], 7}, {NULL}};
const Segment SGI315756[] = {{&S33725[35], 14}, {NULL}};
const Segment SGI315757[] = {{&S33725[64], 11}, {NULL}};
const Segment SGI315758[] = {{&S33725[64], 15}, {NULL}};
const Segment SGI315759[] = {{&S33726[30], 3}, {NULL}};
const Segment SGI315760[] = {{&S33726[8], 3}, {NULL}};
const Segment SGI315761[] = {{&S33726[13], 10}, {NULL}};
const Segment SGI315762[] = {{&S33726[42], 7}, {NULL}};
const Segment SGI315763[] = {{&S33726[35], 14}, {NULL}};
const Segment SGI315764[] = {{&S33726[138], 11}, {NULL}};
const Segment SGI315765[] = {{&S33726[84], 4}, {NULL}};
const Segment SGI315766[] = {{&S33726[138], 15}, {NULL}};
const Segment SGI315767[] = {{&S33817[30], 4}, {NULL}};
const Segment SGI315768[] = {{&S33817[8], 3}, {NULL}};
const Segment SGI315769[] = {{&S33817[13], 10}, {NULL}};
const Segment SGI315770[] = {{&S33817[76], 3}, {NULL}};
const Segment SGI315771[] = {{&S33818[30], 3}, {NULL}};
const Segment SGI315772[] = {{&S33818[8], 3}, {NULL}};
const Segment SGI315773[] = {{&S33818[13], 10}, {NULL}};
const Segment SGI315774[] = {{&S33818[35], 5}, {NULL}};
const Segment SGI315775[] = {{&S33819[8], 3}, {NULL}};
const Segment SGI315776[] = {{&S33819[13], 10}, {NULL}};
const Segment SGI315777[] = {{&S33819[42], 3}, {NULL}};
const Segment SGI315778[] = {{&S33820[30], 4}, {NULL}};
const Segment SGI315779[] = {{&S33820[8], 3}, {NULL}};
const Segment SGI315780[] = {{&S33820[13], 10}, {NULL}};
const Segment SGI315781[] = {{&S33821[8], 3}, {NULL}};
const Segment SGI315782[] = {{&S33821[13], 10}, {NULL}};
const Segment SGI315783[] = {{&S33821[35], 5}, {NULL}};
const Segment SGI315784[] = {{&S33822[30], 4}, {NULL}};
const Segment SGI315785[] = {{&S33822[8], 3}, {NULL}};
const Segment SGI315786[] = {{&S33822[13], 10}, {NULL}};
const Segment SGI315787[] = {{&S33822[43], 3}, {NULL}};
const Segment SGI315788[] = {{&S33823[30], 3}, {NULL}};
const Segment SGI315789[] = {{&S33823[8], 3}, {NULL}};
const Segment SGI315790[] = {{&S33823[13], 10}, {NULL}};
const Segment SGI315791[] = {{&S33824[30], 3}, {NULL}};
const Segment SGI315792[] = {{&S33824[8], 3}, {NULL}};
const Segment SGI315793[] = {{&S33824[13], 10}, {NULL}};
const Segment SGI315794[] = {{&S33824[40], 6}, {NULL}};
const Segment SGI315795[] = {{&S33825[30], 3}, {NULL}};
const Segment SGI315796[] = {{&S33825[8], 3}, {NULL}};
const Segment SGI315797[] = {{&S33825[13], 10}, {NULL}};
const Segment SGI315798[] = {{&S33825[40], 5}, {NULL}};
const Segment SGI315799[] = {{&S33826[30], 3}, {NULL}};
const Segment SGI315800[] = {{&S33826[8], 3}, {NULL}};
const Segment SGI315801[] = {{&S33826[13], 10}, {NULL}};
const Segment SGI315802[] = {{&S33826[40], 3}, {NULL}};
const Segment SGI315803[] = {{&S33885[30], 3}, {NULL}};
const Segment SGI315804[] = {{&S33885[8], 3}, {NULL}};
const Segment SGI315805[] = {{&S33885[13], 10}, {NULL}};
const Segment SGI315806[] = {{&S33885[42], 3}, {NULL}};
const Segment SGI315807[] = {{&S33886[30], 3}, {NULL}};
const Segment SGI315808[] = {{&S33886[8], 3}, {NULL}};
const Segment SGI315809[] = {{&S33886[13], 10}, {NULL}};
const Segment SGI315810[] = {{&S33886[35], 5}, {NULL}};
const Segment SGI315811[] = {{&S33887[39], 3}, {NULL}};
const Segment SGI315812[] = {{&S33888[30], 4}, {NULL}};
const Segment SGI315813[] = {{&S33888[8], 3}, {NULL}};
const Segment SGI315814[] = {{&S33888[13], 10}, {NULL}};
const Segment SGI315815[] = {{&S33888[43], 3}, {NULL}};
const Segment SGI315816[] = {{&S33889[30], 3}, {NULL}};
const Segment SGI315817[] = {{&S33889[8], 3}, {NULL}};
const Segment SGI315818[] = {{&S33889[13], 10}, {NULL}};
const Segment SGI315819[] = {{&S33889[42], 3}, {NULL}};
const Segment SGI315820[] = {{&S33890[28], 3}, {NULL}};
const Segment SGI315821[] = {{&S33890[8], 3}, {NULL}};
const Segment SGI315822[] = {{&S33890[39], 3}, {NULL}};
const Segment SGI315823[] = {{&S34009[30], 3}, {NULL}};
const Segment SGI315824[] = {{&S34009[8], 3}, {NULL}};
const Segment SGI315825[] = {{&S34009[13], 10}, {NULL}};
const Segment SGI315826[] = {{&S34009[42], 7}, {NULL}};
const Segment SGI315827[] = {{&S34009[35], 14}, {NULL}};
const Segment SGI315828[] = {{&S34009[51], 6}, {NULL}};
const Segment SGI315829[] = {{&S34009[71], 15}, {NULL}};
const Segment SGI315830[] = {{&S34010[30], 3}, {NULL}};
const Segment SGI315831[] = {{&S34010[8], 3}, {NULL}};
const Segment SGI315832[] = {{&S34010[13], 10}, {NULL}};
const Segment SGI315833[] = {{&S34010[42], 7}, {NULL}};
const Segment SGI315834[] = {{&S34010[35], 14}, {NULL}};
const Segment SGI315835[] = {{&S34010[64], 15}, {NULL}};
const Segment SGI315836[] = {{&S34011[30], 4}, {NULL}};
const Segment SGI315837[] = {{&S34011[8], 3}, {NULL}};
const Segment SGI315838[] = {{&S34011[13], 10}, {NULL}};
const Segment SGI315839[] = {{&S34011[43], 7}, {NULL}};
const Segment SGI315840[] = {{&S34011[36], 14}, {NULL}};
const Segment SGI315841[] = {{&S34011[52], 6}, {NULL}};
const Segment SGI315842[] = {{&S34011[72], 15}, {NULL}};
const Segment SGI315843[] = {{&S34012[30], 4}, {NULL}};
const Segment SGI315844[] = {{&S34012[8], 3}, {NULL}};
const Segment SGI315845[] = {{&S34012[13], 10}, {NULL}};
const Segment SGI315846[] = {{&S34012[43], 7}, {NULL}};
const Segment SGI315847[] = {{&S34012[36], 14}, {NULL}};
const Segment SGI315848[] = {{&S34012[65], 15}, {NULL}};
const Segment SGI315849[] = {{&S34013[30], 3}, {NULL}};
const Segment SGI315850[] = {{&S34013[8], 3}, {NULL}};
const Segment SGI315851[] = {{&S34013[13], 10}, {NULL}};
const Segment SGI315852[] = {{&S34013[42], 7}, {NULL}};
const Segment SGI315853[] = {{&S34013[35], 14}, {NULL}};
const Segment SGI315854[] = {{&S34013[79], 15}, {NULL}};
const Segment SGI315855[] = {{&S34013[64], 13}, {NULL}};
const Segment SGI315856[] = {{&S34014[30], 4}, {NULL}};
const Segment SGI315857[] = {{&S34014[8], 3}, {NULL}};
const Segment SGI315858[] = {{&S34014[13], 10}, {NULL}};
const Segment SGI315859[] = {{&S34014[43], 7}, {NULL}};
const Segment SGI315860[] = {{&S34014[36], 14}, {NULL}};
const Segment SGI315861[] = {{&S34014[52], 6}, {NULL}};
const Segment SGI315862[] = {{&S34014[87], 15}, {NULL}};
const Segment SGI315863[] = {{&S34014[72], 13}, {NULL}};
const Segment SGI315864[] = {{&S34015[30], 4}, {NULL}};
const Segment SGI315865[] = {{&S34015[8], 3}, {NULL}};
const Segment SGI315866[] = {{&S34015[13], 10}, {NULL}};
const Segment SGI315867[] = {{&S34015[43], 7}, {NULL}};
const Segment SGI315868[] = {{&S34015[36], 14}, {NULL}};
const Segment SGI315869[] = {{&S34015[80], 15}, {NULL}};
const Segment SGI315870[] = {{&S34015[65], 13}, {NULL}};
const Segment SGI315871[] = {{&S48075[0], 85}, {NULL}};
const Segment SGI315872[] = {{&S34016[30], 3}, {NULL}};
const Segment SGI315873[] = {{&S34016[8], 3}, {NULL}};
const Segment SGI315874[] = {{&S34016[13], 10}, {NULL}};
const Segment SGI315875[] = {{&S34016[42], 7}, {NULL}};
const Segment SGI315876[] = {{&S34016[35], 14}, {NULL}};
const Segment SGI315877[] = {{&S34016[51], 6}, {NULL}};
const Segment SGI315878[] = {{&S34016[86], 15}, {NULL}};
const Segment SGI315879[] = {{&S34016[71], 13}, {NULL}};
const Segment SGI315880[] = {{&S48076[0], 211}, {NULL}};
const Segment SGI315881[] = {{&S34017[30], 3}, {NULL}};
const Segment SGI315882[] = {{&S34017[8], 3}, {NULL}};
const Segment SGI315883[] = {{&S34017[13], 10}, {NULL}};
const Segment SGI315884[] = {{&S34017[42], 7}, {NULL}};
const Segment SGI315885[] = {{&S34017[35], 14}, {NULL}};
const Segment SGI315886[] = {{&S34017[51], 6}, {NULL}};
const Segment SGI315887[] = {{&S34017[71], 15}, {NULL}};
const Segment SGI315888[] = {{&S34710[30], 3}, {NULL}};
const Segment SGI315889[] = {{&S34710[192], 9}, {NULL}};
const Segment SGI315890[] = {{&S34710[8], 3}, {NULL}};
const Segment SGI315891[] = {{&S34710[13], 10}, {NULL}};
const Segment SGI315892[] = {{&S34710[42], 7}, {NULL}};
const Segment SGI315893[] = {{&S34710[35], 14}, {NULL}};
const Segment SGI315894[] = {{&S34710[91], 15}, {NULL}};
const Segment SGI315895[] = {{&S34712[30], 3}, {NULL}};
const Segment SGI315896[] = {{&S34712[192], 9}, {NULL}};
const Segment SGI315897[] = {{&S34712[8], 3}, {NULL}};
const Segment SGI315898[] = {{&S34712[13], 10}, {NULL}};
const Segment SGI315899[] = {{&S34712[42], 7}, {NULL}};
const Segment SGI315900[] = {{&S34712[35], 14}, {NULL}};
const Segment SGI315901[] = {{&S34712[91], 15}, {NULL}};
const Segment SGI315902[] = {{&S34714[30], 3}, {NULL}};
const Segment SGI315903[] = {{&S34714[180], 9}, {NULL}};
const Segment SGI315904[] = {{&S34714[8], 3}, {NULL}};
const Segment SGI315905[] = {{&S34714[13], 10}, {NULL}};
const Segment SGI315906[] = {{&S34714[42], 7}, {NULL}};
const Segment SGI315907[] = {{&S34714[35], 14}, {NULL}};
const Segment SGI315908[] = {{&S34714[91], 15}, {NULL}};
const Segment SGI315909[] = {{&S34715[30], 3}, {NULL}};
const Segment SGI315910[] = {{&S34715[169], 9}, {NULL}};
const Segment SGI315911[] = {{&S34715[8], 3}, {NULL}};
const Segment SGI315912[] = {{&S34715[13], 10}, {NULL}};
const Segment SGI315913[] = {{&S34715[42], 7}, {NULL}};
const Segment SGI315914[] = {{&S34715[35], 14}, {NULL}};
const Segment SGI315915[] = {{&S34716[30], 3}, {NULL}};
const Segment SGI315916[] = {{&S34716[199], 9}, {NULL}};
const Segment SGI315917[] = {{&S34716[8], 3}, {NULL}};
const Segment SGI315918[] = {{&S34716[13], 10}, {NULL}};
const Segment SGI315919[] = {{&S34716[42], 7}, {NULL}};
const Segment SGI315920[] = {{&S34716[35], 14}, {NULL}};
const Segment SGI315921[] = {{&S34716[98], 15}, {NULL}};
const Segment SGI315922[] = {{&S34716[51], 6}, {NULL}};
const Segment SGI315923[] = {{&S34717[30], 3}, {NULL}};
const Segment SGI315924[] = {{&S34717[187], 9}, {NULL}};
const Segment SGI315925[] = {{&S34717[8], 3}, {NULL}};
const Segment SGI315926[] = {{&S34717[13], 10}, {NULL}};
const Segment SGI315927[] = {{&S34717[42], 7}, {NULL}};
const Segment SGI315928[] = {{&S34717[35], 14}, {NULL}};
const Segment SGI315929[] = {{&S34717[98], 15}, {NULL}};
const Segment SGI315930[] = {{&S34717[51], 6}, {NULL}};
const Segment SGI315931[] = {{&S34718[30], 3}, {NULL}};
const Segment SGI315932[] = {{&S34718[169], 9}, {NULL}};
const Segment SGI315933[] = {{&S34718[8], 3}, {NULL}};
const Segment SGI315934[] = {{&S34718[13], 10}, {NULL}};
const Segment SGI315935[] = {{&S34718[42], 7}, {NULL}};
const Segment SGI315936[] = {{&S34718[35], 14}, {NULL}};
const Segment SGI315937[] = {{&S34719[30], 3}, {NULL}};
const Segment SGI315938[] = {{&S34719[176], 9}, {NULL}};
const Segment SGI315939[] = {{&S34719[8], 3}, {NULL}};
const Segment SGI315940[] = {{&S34719[13], 10}, {NULL}};
const Segment SGI315941[] = {{&S34719[42], 7}, {NULL}};
const Segment SGI315942[] = {{&S34719[35], 14}, {NULL}};
const Segment SGI315943[] = {{&S34719[51], 6}, {NULL}};
const Segment SGI315944[] = {{&S34720[30], 3}, {NULL}};
const Segment SGI315945[] = {{&S34720[204], 9}, {NULL}};
const Segment SGI315946[] = {{&S34720[8], 3}, {NULL}};
const Segment SGI315947[] = {{&S34720[13], 10}, {NULL}};
const Segment SGI315948[] = {{&S34720[42], 7}, {NULL}};
const Segment SGI315949[] = {{&S34720[35], 14}, {NULL}};
const Segment SGI315950[] = {{&S34720[103], 15}, {NULL}};
const Segment SGI315951[] = {{&S34720[51], 5}, {NULL}};
const Segment SGI315952[] = {{&S34722[30], 3}, {NULL}};
const Segment SGI315953[] = {{&S34722[176], 9}, {NULL}};
const Segment SGI315954[] = {{&S34722[8], 3}, {NULL}};
const Segment SGI315955[] = {{&S34722[13], 10}, {NULL}};
const Segment SGI315956[] = {{&S34722[42], 7}, {NULL}};
const Segment SGI315957[] = {{&S34722[35], 14}, {NULL}};
const Segment SGI315958[] = {{&S34722[51], 6}, {NULL}};
const Segment SGI315959[] = {{&S34723[30], 3}, {NULL}};
const Segment SGI315960[] = {{&S34723[202], 9}, {NULL}};
const Segment SGI315961[] = {{&S34723[8], 3}, {NULL}};
const Segment SGI315962[] = {{&S34723[13], 10}, {NULL}};
const Segment SGI315963[] = {{&S34723[42], 7}, {NULL}};
const Segment SGI315964[] = {{&S34723[35], 14}, {NULL}};
const Segment SGI315965[] = {{&S34723[91], 15}, {NULL}};
const Segment SGI315966[] = {{&S34723[174], 4}, {NULL}};
const Segment SGI315967[] = {{&S34724[30], 3}, {NULL}};
const Segment SGI315968[] = {{&S34724[82], 9}, {NULL}};
const Segment SGI315969[] = {{&S34724[8], 3}, {NULL}};
const Segment SGI315970[] = {{&S34724[13], 10}, {NULL}};
const Segment SGI315971[] = {{&S34724[42], 7}, {NULL}};
const Segment SGI315972[] = {{&S34724[35], 14}, {NULL}};
const Segment SGI315973[] = {{&S34725[30], 3}, {NULL}};
const Segment SGI315974[] = {{&S34725[192], 9}, {NULL}};
const Segment SGI315975[] = {{&S34725[8], 3}, {NULL}};
const Segment SGI315976[] = {{&S34725[13], 10}, {NULL}};
const Segment SGI315977[] = {{&S34725[42], 7}, {NULL}};
const Segment SGI315978[] = {{&S34725[35], 14}, {NULL}};
const Segment SGI315979[] = {{&S34725[91], 15}, {NULL}};
const Segment SGI315980[] = {{&S34726[30], 3}, {NULL}};
const Segment SGI315981[] = {{&S34726[159], 9}, {NULL}};
const Segment SGI315982[] = {{&S34726[8], 3}, {NULL}};
const Segment SGI315983[] = {{&S34726[13], 10}, {NULL}};
const Segment SGI315984[] = {{&S34726[42], 7}, {NULL}};
const Segment SGI315985[] = {{&S34726[35], 14}, {NULL}};
const Segment SGI315986[] = {{&S34727[30], 3}, {NULL}};
const Segment SGI315987[] = {{&S34727[199], 9}, {NULL}};
const Segment SGI315988[] = {{&S34727[8], 3}, {NULL}};
const Segment SGI315989[] = {{&S34727[13], 10}, {NULL}};
const Segment SGI315990[] = {{&S34727[42], 7}, {NULL}};
const Segment SGI315991[] = {{&S34727[35], 14}, {NULL}};
const Segment SGI315992[] = {{&S34727[98], 15}, {NULL}};
const Segment SGI315993[] = {{&S34727[51], 6}, {NULL}};
const Segment SGI315994[] = {{&S34728[30], 3}, {NULL}};
const Segment SGI315995[] = {{&S34728[176], 9}, {NULL}};
const Segment SGI315996[] = {{&S34728[8], 3}, {NULL}};
const Segment SGI315997[] = {{&S34728[13], 10}, {NULL}};
const Segment SGI315998[] = {{&S34728[42], 7}, {NULL}};
const Segment SGI315999[] = {{&S34728[35], 14}, {NULL}};
const Segment SGI316000[] = {{&S34728[51], 6}, {NULL}};
const Segment SGI316001[] = {{&S34730[30], 3}, {NULL}};
const Segment SGI316002[] = {{&S34730[192], 9}, {NULL}};
const Segment SGI316003[] = {{&S34730[8], 3}, {NULL}};
const Segment SGI316004[] = {{&S34730[13], 10}, {NULL}};
const Segment SGI316005[] = {{&S34730[42], 7}, {NULL}};
const Segment SGI316006[] = {{&S34730[35], 14}, {NULL}};
const Segment SGI316007[] = {{&S34730[91], 15}, {NULL}};
const Segment SGI316008[] = {{&S34731[30], 3}, {NULL}};
const Segment SGI316009[] = {{&S34731[171], 9}, {NULL}};
const Segment SGI316010[] = {{&S34731[8], 3}, {NULL}};
const Segment SGI316011[] = {{&S34731[13], 10}, {NULL}};
const Segment SGI316012[] = {{&S34731[42], 7}, {NULL}};
const Segment SGI316013[] = {{&S34731[35], 14}, {NULL}};
const Segment SGI316014[] = {{&S34732[30], 3}, {NULL}};
const Segment SGI316015[] = {{&S34732[199], 9}, {NULL}};
const Segment SGI316016[] = {{&S34732[8], 3}, {NULL}};
const Segment SGI316017[] = {{&S34732[13], 10}, {NULL}};
const Segment SGI316018[] = {{&S34732[42], 7}, {NULL}};
const Segment SGI316019[] = {{&S34732[35], 14}, {NULL}};
const Segment SGI316020[] = {{&S34732[98], 15}, {NULL}};
const Segment SGI316021[] = {{&S34732[51], 6}, {NULL}};
const Segment SGI316022[] = {{&S34733[30], 3}, {NULL}};
const Segment SGI316023[] = {{&S34733[82], 9}, {NULL}};
const Segment SGI316024[] = {{&S34733[8], 3}, {NULL}};
const Segment SGI316025[] = {{&S34733[13], 10}, {NULL}};
const Segment SGI316026[] = {{&S34733[42], 7}, {NULL}};
const Segment SGI316027[] = {{&S34733[35], 14}, {NULL}};
const Segment SGI316028[] = {{&S34734[30], 3}, {NULL}};
const Segment SGI316029[] = {{&S34734[176], 9}, {NULL}};
const Segment SGI316030[] = {{&S34734[8], 3}, {NULL}};
const Segment SGI316031[] = {{&S34734[13], 10}, {NULL}};
const Segment SGI316032[] = {{&S34734[42], 7}, {NULL}};
const Segment SGI316033[] = {{&S34734[35], 14}, {NULL}};
const Segment SGI316034[] = {{&S34734[51], 6}, {NULL}};
const Segment SGI316035[] = {{&S34735[30], 3}, {NULL}};
const Segment SGI316036[] = {{&S34735[89], 9}, {NULL}};
const Segment SGI316037[] = {{&S34735[8], 3}, {NULL}};
const Segment SGI316038[] = {{&S34735[13], 10}, {NULL}};
const Segment SGI316039[] = {{&S34735[42], 7}, {NULL}};
const Segment SGI316040[] = {{&S34735[35], 14}, {NULL}};
const Segment SGI316041[] = {{&S34735[51], 6}, {NULL}};
const Segment SGI316042[] = {{&S34736[30], 3}, {NULL}};
const Segment SGI316043[] = {{&S34736[169], 9}, {NULL}};
const Segment SGI316044[] = {{&S34736[8], 3}, {NULL}};
const Segment SGI316045[] = {{&S34736[13], 10}, {NULL}};
const Segment SGI316046[] = {{&S34736[42], 7}, {NULL}};
const Segment SGI316047[] = {{&S34736[35], 14}, {NULL}};
const Segment SGI316048[] = {{&S34737[30], 3}, {NULL}};
const Segment SGI316049[] = {{&S34737[204], 9}, {NULL}};
const Segment SGI316050[] = {{&S34737[8], 3}, {NULL}};
const Segment SGI316051[] = {{&S34737[13], 10}, {NULL}};
const Segment SGI316052[] = {{&S34737[42], 7}, {NULL}};
const Segment SGI316053[] = {{&S34737[35], 14}, {NULL}};
const Segment SGI316054[] = {{&S34737[103], 15}, {NULL}};
const Segment SGI316055[] = {{&S34737[51], 5}, {NULL}};
const Segment SGI316056[] = {{&S34738[30], 3}, {NULL}};
const Segment SGI316057[] = {{&S34738[240], 9}, {NULL}};
const Segment SGI316058[] = {{&S34738[8], 3}, {NULL}};
const Segment SGI316059[] = {{&S34738[13], 10}, {NULL}};
const Segment SGI316060[] = {{&S34738[42], 7}, {NULL}};
const Segment SGI316061[] = {{&S34738[35], 14}, {NULL}};
const Segment SGI316062[] = {{&S34738[98], 15}, {&S34738[98], 15}, {NULL}};
const Segment SGI316063[] = {{&S34738[51], 6}, {NULL}};
const Segment SGI316064[] = {{&S34739[30], 3}, {NULL}};
const Segment SGI316065[] = {{&S34739[202], 9}, {NULL}};
const Segment SGI316066[] = {{&S34739[8], 3}, {NULL}};
const Segment SGI316067[] = {{&S34739[13], 10}, {NULL}};
const Segment SGI316068[] = {{&S34739[42], 7}, {NULL}};
const Segment SGI316069[] = {{&S34739[35], 14}, {NULL}};
const Segment SGI316070[] = {{&S34739[91], 15}, {NULL}};
const Segment SGI316071[] = {{&S34739[174], 4}, {NULL}};
const Segment SGI316072[] = {{&S34740[30], 3}, {NULL}};
const Segment SGI316073[] = {{&S34740[181], 9}, {NULL}};
const Segment SGI316074[] = {{&S34740[8], 3}, {NULL}};
const Segment SGI316075[] = {{&S34740[13], 10}, {NULL}};
const Segment SGI316076[] = {{&S34740[42], 7}, {NULL}};
const Segment SGI316077[] = {{&S34740[35], 14}, {NULL}};
const Segment SGI316078[] = {{&S34740[51], 5}, {NULL}};
const Segment SGI316079[] = {{&S34741[30], 3}, {NULL}};
const Segment SGI316080[] = {{&S34741[201], 9}, {NULL}};
const Segment SGI316081[] = {{&S34741[8], 3}, {NULL}};
const Segment SGI316082[] = {{&S34741[13], 10}, {NULL}};
const Segment SGI316083[] = {{&S34741[42], 7}, {NULL}};
const Segment SGI316084[] = {{&S34741[35], 14}, {NULL}};
const Segment SGI316085[] = {{&S34741[51], 6}, {NULL}};
const Segment SGI316086[] = {{&S34742[30], 3}, {NULL}};
const Segment SGI316087[] = {{&S34742[192], 9}, {NULL}};
const Segment SGI316088[] = {{&S34742[8], 3}, {NULL}};
const Segment SGI316089[] = {{&S34742[13], 10}, {NULL}};
const Segment SGI316090[] = {{&S34742[42], 7}, {NULL}};
const Segment SGI316091[] = {{&S34742[35], 14}, {NULL}};
const Segment SGI316092[] = {{&S34742[91], 15}, {NULL}};
const Segment SGI316093[] = {{&S34743[30], 3}, {NULL}};
const Segment SGI316094[] = {{&S34743[169], 9}, {NULL}};
const Segment SGI316095[] = {{&S34743[8], 3}, {NULL}};
const Segment SGI316096[] = {{&S34743[13], 10}, {NULL}};
const Segment SGI316097[] = {{&S34743[42], 7}, {NULL}};
const Segment SGI316098[] = {{&S34743[35], 14}, {NULL}};
const Segment SGI316099[] = {{&S34744[30], 3}, {NULL}};
const Segment SGI316100[] = {{&S34744[192], 9}, {NULL}};
const Segment SGI316101[] = {{&S34744[8], 3}, {NULL}};
const Segment SGI316102[] = {{&S34744[13], 10}, {NULL}};
const Segment SGI316103[] = {{&S34744[42], 7}, {NULL}};
const Segment SGI316104[] = {{&S34744[35], 14}, {NULL}};
const Segment SGI316105[] = {{&S34744[91], 15}, {NULL}};
const Segment SGI316106[] = {{&S34745[30], 3}, {NULL}};
const Segment SGI316107[] = {{&S34745[171], 9}, {NULL}};
const Segment SGI316108[] = {{&S34745[8], 3}, {NULL}};
const Segment SGI316109[] = {{&S34745[13], 10}, {NULL}};
const Segment SGI316110[] = {{&S34745[42], 7}, {NULL}};
const Segment SGI316111[] = {{&S34745[35], 14}, {NULL}};
const Segment SGI316112[] = {{&S34746[30], 3}, {NULL}};
const Segment SGI316113[] = {{&S34746[176], 9}, {NULL}};
const Segment SGI316114[] = {{&S34746[8], 3}, {NULL}};
const Segment SGI316115[] = {{&S34746[13], 10}, {NULL}};
const Segment SGI316116[] = {{&S34746[42], 7}, {NULL}};
const Segment SGI316117[] = {{&S34746[35], 14}, {NULL}};
const Segment SGI316118[] = {{&S34746[51], 6}, {NULL}};
const Segment SGI316119[] = {{&S34748[30], 3}, {NULL}};
const Segment SGI316120[] = {{&S34748[199], 9}, {NULL}};
const Segment SGI316121[] = {{&S34748[8], 3}, {NULL}};
const Segment SGI316122[] = {{&S34748[13], 10}, {NULL}};
const Segment SGI316123[] = {{&S34748[42], 7}, {NULL}};
const Segment SGI316124[] = {{&S34748[35], 14}, {NULL}};
const Segment SGI316125[] = {{&S34748[98], 15}, {NULL}};
const Segment SGI316126[] = {{&S34748[51], 6}, {NULL}};
const Segment SGI316127[] = {{&S34749[30], 3}, {NULL}};
const Segment SGI316128[] = {{&S34749[8], 3}, {NULL}};
const Segment SGI316129[] = {{&S34749[13], 10}, {NULL}};
const Segment SGI316130[] = {{&S34749[42], 7}, {NULL}};
const Segment SGI316131[] = {{&S34749[35], 14}, {NULL}};
const Segment SGI316132[] = {{&S34749[51], 13}, {NULL}};
const Segment SGI316133[] = {{&S34750[30], 3}, {NULL}};
const Segment SGI316134[] = {{&S34750[8], 3}, {NULL}};
const Segment SGI316135[] = {{&S34750[13], 10}, {NULL}};
const Segment SGI316136[] = {{&S34750[42], 7}, {NULL}};
const Segment SGI316137[] = {{&S34750[35], 14}, {NULL}};
const Segment SGI316138[] = {{&S34750[119], 15}, {NULL}};
const Segment SGI316139[] = {{&S34750[51], 13}, {&S34750[51], 13}, {NULL}};
const Segment SGI316140[] = {{&S34751[30], 3}, {NULL}};
const Segment SGI316141[] = {{&S34751[8], 3}, {NULL}};
const Segment SGI316142[] = {{&S34751[13], 10}, {NULL}};
const Segment SGI316143[] = {{&S34751[42], 7}, {NULL}};
const Segment SGI316144[] = {{&S34751[35], 14}, {NULL}};
const Segment SGI316145[] = {{&S34751[68], 13}, {NULL}};
const Segment SGI316146[] = {{&S34751[63], 3}, {NULL}};
const Segment SGI316147[] = {{&S34752[30], 3}, {NULL}};
const Segment SGI316148[] = {{&S34752[8], 3}, {NULL}};
const Segment SGI316149[] = {{&S34752[13], 10}, {NULL}};
const Segment SGI316150[] = {{&S34752[42], 7}, {NULL}};
const Segment SGI316151[] = {{&S34752[35], 14}, {NULL}};
const Segment SGI316152[] = {{&S34752[64], 13}, {&S34752[64], 13}, {NULL}};
const Segment SGI316153[] = {{&S34753[30], 3}, {&S34753[96], 3}, {NULL}};
const Segment SGI316154[] = {{&S34753[8], 3}, {&S34753[8], 3}, {NULL}};
const Segment SGI316155[] = {{&S34753[13], 10}, {&S34753[13], 10}, {NULL}};
const Segment SGI316156[] = {{&S34753[42], 7}, {&S34753[42], 7}, {NULL}};
const Segment SGI316157[] = {{&S34753[35], 14}, {&S34753[35], 14}, {NULL}};
const Segment SGI316158[] = {{&S34753[51], 13}, {NULL}};
const Segment SGI316159[] = {{&S34753[117], 3}, {NULL}};
const Segment SGI316160[] = {{&S34754[30], 3}, {NULL}};
const Segment SGI316161[] = {{&S34754[8], 3}, {NULL}};
const Segment SGI316162[] = {{&S34754[13], 10}, {NULL}};
const Segment SGI316163[] = {{&S34754[55], 7}, {NULL}};
const Segment SGI316164[] = {{&S34754[48], 14}, {NULL}};
const Segment SGI316165[] = {{&S34754[64], 13}, {NULL}};
const Segment SGI316166[] = {{&S34755[30], 3}, {NULL}};
const Segment SGI316167[] = {{&S34755[8], 3}, {NULL}};
const Segment SGI316168[] = {{&S34755[13], 10}, {NULL}};
const Segment SGI316169[] = {{&S34755[55], 7}, {NULL}};
const Segment SGI316170[] = {{&S34755[48], 14}, {NULL}};
const Segment SGI316171[] = {{&S34755[64], 13}, {NULL}};
const Segment SGI316172[] = {{&S34756[30], 3}, {&S34756[94], 3}, {NULL}};
const Segment SGI316173[] = {{&S34756[8], 3}, {&S34756[8], 3}, {NULL}};
const Segment SGI316174[] = {{&S34756[13], 10}, {&S34756[13], 10}, {NULL}};
const Segment SGI316175[] = {{&S34756[42], 7}, {&S34756[42], 7}, {NULL}};
const Segment SGI316176[] = {{&S34756[35], 14}, {&S34756[35], 14}, {NULL}};
const Segment SGI316177[] = {{&S34756[122], 13}, {&S34756[122], 13}, {NULL}};
const Segment SGI316178[] = {{&S34756[115], 3}, {NULL}};
const Segment SGI316179[] = {{&S34757[30], 3}, {NULL}};
const Segment SGI316180[] = {{&S34757[8], 3}, {NULL}};
const Segment SGI316181[] = {{&S34757[13], 10}, {NULL}};
const Segment SGI316182[] = {{&S34757[55], 7}, {NULL}};
const Segment SGI316183[] = {{&S34757[48], 14}, {NULL}};
const Segment SGI316184[] = {{&S34757[64], 13}, {NULL}};
const Segment SGI316185[] = {{&S34758[30], 3}, {NULL}};
const Segment SGI316186[] = {{&S34758[8], 3}, {NULL}};
const Segment SGI316187[] = {{&S34758[13], 10}, {NULL}};
const Segment SGI316188[] = {{&S34758[42], 7}, {NULL}};
const Segment SGI316189[] = {{&S34758[35], 14}, {NULL}};
const Segment SGI316190[] = {{&S34758[64], 13}, {&S34758[64], 13}, {&S34758[64], 13}, {NULL}};
const Segment SGI316191[] = {{&S34759[30], 3}, {NULL}};
const Segment SGI316192[] = {{&S34759[8], 3}, {NULL}};
const Segment SGI316193[] = {{&S34759[13], 10}, {NULL}};
const Segment SGI316194[] = {{&S34759[42], 7}, {NULL}};
const Segment SGI316195[] = {{&S34759[35], 14}, {NULL}};
const Segment SGI316196[] = {{&S34759[116], 15}, {NULL}};
const Segment SGI316197[] = {{&S34759[57], 13}, {NULL}};
const Segment SGI316198[] = {{&S34759[52], 3}, {NULL}};
const Segment SGI316199[] = {{&S34760[30], 3}, {NULL}};
const Segment SGI316200[] = {{&S34760[8], 3}, {NULL}};
const Segment SGI316201[] = {{&S34760[13], 10}, {NULL}};
const Segment SGI316202[] = {{&S34760[42], 7}, {NULL}};
const Segment SGI316203[] = {{&S34760[35], 14}, {NULL}};
const Segment SGI316204[] = {{&S34760[64], 13}, {&S34760[64], 13}, {NULL}};
const Segment SGI316205[] = {{&S34761[30], 3}, {NULL}};
const Segment SGI316206[] = {{&S34761[8], 3}, {NULL}};
const Segment SGI316207[] = {{&S34761[13], 10}, {NULL}};
const Segment SGI316208[] = {{&S34761[42], 7}, {NULL}};
const Segment SGI316209[] = {{&S34761[35], 14}, {NULL}};
const Segment SGI316210[] = {{&S34761[131], 15}, {NULL}};
const Segment SGI316211[] = {{&S34761[57], 13}, {NULL}};
const Segment SGI316212[] = {{&S34761[52], 3}, {NULL}};
const Segment SGI316213[] = {{&S34762[30], 3}, {NULL}};
const Segment SGI316214[] = {{&S34762[8], 3}, {NULL}};
const Segment SGI316215[] = {{&S34762[13], 10}, {NULL}};
const Segment SGI316216[] = {{&S34762[42], 7}, {NULL}};
const Segment SGI316217[] = {{&S34762[35], 14}, {NULL}};
const Segment SGI316218[] = {{&S34762[129], 15}, {NULL}};
const Segment SGI316219[] = {{&S34762[51], 13}, {NULL}};
const Segment SGI316220[] = {{&S68258[7], 5}, {NULL}};
const Segment SGI316221[] = {{&S68258[0], 102}, {NULL}};
const Segment SGI316222[] = {{&S34855[31], 3}, {NULL}};
const Segment SGI316223[] = {{&S34855[8], 3}, {NULL}};
const Segment SGI316224[] = {{&S34855[44], 3}, {NULL}};
const Segment SGI316225[] = {{&S34856[31], 3}, {NULL}};
const Segment SGI316226[] = {{&S34856[8], 3}, {NULL}};
const Segment SGI316227[] = {{&S34856[43], 3}, {NULL}};
const Segment SGI316228[] = {{&S34857[8], 3}, {NULL}};
const Segment SGI316229[] = {{&S34857[13], 10}, {NULL}};
const Segment SGI316230[] = {{&S34857[31], 5}, {NULL}};
const Segment SGI316231[] = {{&S34858[27], 3}, {NULL}};
const Segment SGI316232[] = {{&S34858[40], 3}, {NULL}};
const Segment SGI316233[] = {{&S34859[30], 3}, {NULL}};
const Segment SGI316234[] = {{&S34859[8], 3}, {NULL}};
const Segment SGI316235[] = {{&S34859[13], 10}, {NULL}};
const Segment SGI316236[] = {{&S34860[30], 3}, {NULL}};
const Segment SGI316237[] = {{&S34860[8], 3}, {NULL}};
const Segment SGI316238[] = {{&S34860[13], 10}, {NULL}};
const Segment SGI316239[] = {{&S34861[30], 3}, {NULL}};
const Segment SGI316240[] = {{&S34861[8], 3}, {NULL}};
const Segment SGI316241[] = {{&S34861[13], 10}, {NULL}};
const Segment SGI316242[] = {{&S34861[40], 5}, {NULL}};
const Segment SGI316243[] = {{&S34861[35], 3}, {NULL}};
const Segment SGI316244[] = {{&S34862[30], 3}, {NULL}};
const Segment SGI316245[] = {{&S34862[8], 3}, {NULL}};
const Segment SGI316246[] = {{&S34862[13], 10}, {NULL}};
const Segment SGI316247[] = {{&S34862[35], 3}, {NULL}};
const Segment SGI316248[] = {{&S34863[30], 3}, {NULL}};
const Segment SGI316249[] = {{&S34863[8], 3}, {NULL}};
const Segment SGI316250[] = {{&S34863[13], 10}, {NULL}};
const Segment SGI316251[] = {{&S34863[40], 5}, {NULL}};
const Segment SGI316252[] = {{&S34864[30], 3}, {NULL}};
const Segment SGI316253[] = {{&S34864[8], 3}, {NULL}};
const Segment SGI316254[] = {{&S34864[13], 10}, {NULL}};
const Segment SGI316255[] = {{&S34864[40], 6}, {NULL}};
const Segment SGI316256[] = {{&S34864[35], 3}, {NULL}};
const Segment SGI316257[] = {{&S34865[30], 3}, {NULL}};
const Segment SGI316258[] = {{&S34865[8], 3}, {NULL}};
const Segment SGI316259[] = {{&S34865[13], 10}, {NULL}};
const Segment SGI316260[] = {{&S34865[40], 5}, {NULL}};
const Segment SGI316261[] = {{&S34866[30], 3}, {NULL}};
const Segment SGI316262[] = {{&S34866[8], 3}, {NULL}};
const Segment SGI316263[] = {{&S34866[13], 10}, {NULL}};
const Segment SGI316264[] = {{&S34866[40], 6}, {NULL}};
const Segment SGI316265[] = {{&S34866[35], 3}, {NULL}};
const Segment SGI316266[] = {{&S34867[30], 3}, {NULL}};
const Segment SGI316267[] = {{&S34867[8], 3}, {NULL}};
const Segment SGI316268[] = {{&S34867[13], 10}, {NULL}};
const Segment SGI316269[] = {{&S34867[40], 3}, {NULL}};
const Segment SGI316270[] = {{&S34868[30], 3}, {NULL}};
const Segment SGI316271[] = {{&S34868[8], 3}, {NULL}};
const Segment SGI316272[] = {{&S34868[13], 10}, {NULL}};
const Segment SGI316273[] = {{&S34868[35], 3}, {NULL}};
const Segment SGI316274[] = {{&S34869[30], 3}, {NULL}};
const Segment SGI316275[] = {{&S34869[8], 3}, {NULL}};
const Segment SGI316276[] = {{&S34869[13], 10}, {NULL}};
const Segment SGI316277[] = {{&S34869[40], 6}, {NULL}};
const Segment SGI316278[] = {{&S34870[30], 3}, {NULL}};
const Segment SGI316279[] = {{&S34870[8], 3}, {NULL}};
const Segment SGI316280[] = {{&S34870[13], 10}, {NULL}};
const Segment SGI316281[] = {{&S34870[40], 3}, {NULL}};
const Segment SGI316282[] = {{&S34871[30], 3}, {NULL}};
const Segment SGI316283[] = {{&S34871[8], 3}, {NULL}};
const Segment SGI316284[] = {{&S34871[13], 10}, {NULL}};
const Segment SGI316285[] = {{&S34871[40], 6}, {NULL}};
const Segment SGI316286[] = {{&S34872[30], 3}, {NULL}};
const Segment SGI316287[] = {{&S34872[8], 3}, {NULL}};
const Segment SGI316288[] = {{&S34872[13], 10}, {NULL}};
const Segment SGI316289[] = {{&S34872[40], 5}, {NULL}};
const Segment SGI316290[] = {{&S34872[35], 3}, {NULL}};
const Segment SGI316291[] = {{&S34873[8], 3}, {NULL}};
const Segment SGI316292[] = {{&S34873[13], 10}, {NULL}};
const Segment SGI316293[] = {{&S34992[30], 3}, {&S34992[119], 3}, {NULL}};
const Segment SGI316294[] = {{&S34992[8], 3}, {&S34992[8], 3}, {NULL}};
const Segment SGI316295[] = {{&S34992[13], 10}, {&S34992[13], 10}, {NULL}};
const Segment SGI316296[] = {{&S34992[42], 7}, {&S34992[42], 7}, {NULL}};
const Segment SGI316297[] = {{&S34992[35], 14}, {&S34992[35], 14}, {NULL}};
const Segment SGI316298[] = {{&S34992[64], 11}, {NULL}};
const Segment SGI316299[] = {{&S34992[64], 16}, {NULL}};
const Segment SGI316300[] = {{&S34992[140], 3}, {NULL}};
const Segment SGI316301[] = {{&S34993[30], 3}, {NULL}};
const Segment SGI316302[] = {{&S34993[8], 3}, {NULL}};
const Segment SGI316303[] = {{&S34993[13], 10}, {NULL}};
const Segment SGI316304[] = {{&S34993[42], 7}, {NULL}};
const Segment SGI316305[] = {{&S34993[35], 14}, {NULL}};
const Segment SGI316306[] = {{&S34993[51], 11}, {NULL}};
const Segment SGI316307[] = {{&S34993[51], 16}, {NULL}};
const Segment SGI316308[] = {{&S34998[30], 3}, {NULL}};
const Segment SGI316309[] = {{&S34998[8], 3}, {NULL}};
const Segment SGI316310[] = {{&S34998[13], 10}, {NULL}};
const Segment SGI316311[] = {{&S34998[42], 7}, {NULL}};
const Segment SGI316312[] = {{&S34998[35], 14}, {NULL}};
const Segment SGI316313[] = {{&S34998[51], 11}, {NULL}};
const Segment SGI316314[] = {{&S34998[51], 16}, {NULL}};
const Segment SGI316315[] = {{&S35000[30], 3}, {NULL}};
const Segment SGI316316[] = {{&S35000[8], 3}, {NULL}};
const Segment SGI316317[] = {{&S35000[13], 10}, {NULL}};
const Segment SGI316318[] = {{&S35000[42], 7}, {NULL}};
const Segment SGI316319[] = {{&S35000[35], 14}, {NULL}};
const Segment SGI316320[] = {{&S35000[64], 11}, {NULL}};
const Segment SGI316321[] = {{&S35000[64], 16}, {NULL}};
const Segment SGI316322[] = {{&S35003[30], 3}, {NULL}};
const Segment SGI316323[] = {{&S35003[8], 3}, {NULL}};
const Segment SGI316324[] = {{&S35003[13], 10}, {NULL}};
const Segment SGI316325[] = {{&S35003[42], 7}, {NULL}};
const Segment SGI316326[] = {{&S35003[35], 14}, {NULL}};
const Segment SGI316327[] = {{&S35003[64], 11}, {NULL}};
const Segment SGI316328[] = {{&S35003[64], 16}, {NULL}};
const Segment SGI316329[] = {{&S35003[89], 9}, {NULL}};
const Segment SGI316330[] = {{&S35005[30], 3}, {NULL}};
const Segment SGI316331[] = {{&S35005[8], 3}, {NULL}};
const Segment SGI316332[] = {{&S35005[13], 10}, {NULL}};
const Segment SGI316333[] = {{&S35005[42], 7}, {NULL}};
const Segment SGI316334[] = {{&S35005[35], 14}, {NULL}};
const Segment SGI316335[] = {{&S35005[51], 11}, {NULL}};
const Segment SGI316336[] = {{&S35005[114], 15}, {NULL}};
const Segment SGI316337[] = {{&S35005[51], 16}, {NULL}};
const Segment SGI316338[] = {{&S35006[30], 3}, {NULL}};
const Segment SGI316339[] = {{&S35006[8], 3}, {NULL}};
const Segment SGI316340[] = {{&S35006[13], 10}, {NULL}};
const Segment SGI316341[] = {{&S35006[42], 7}, {NULL}};
const Segment SGI316342[] = {{&S35006[35], 14}, {NULL}};
const Segment SGI316343[] = {{&S35006[58], 11}, {NULL}};
const Segment SGI316344[] = {{&S35006[51], 6}, {NULL}};
const Segment SGI316345[] = {{&S35006[58], 16}, {NULL}};
const Segment SGI316346[] = {{&S35007[30], 3}, {NULL}};
const Segment SGI316347[] = {{&S35007[8], 3}, {NULL}};
const Segment SGI316348[] = {{&S35007[13], 10}, {NULL}};
const Segment SGI316349[] = {{&S35007[42], 7}, {NULL}};
const Segment SGI316350[] = {{&S35007[35], 14}, {NULL}};
const Segment SGI316351[] = {{&S35007[56], 11}, {NULL}};
const Segment SGI316352[] = {{&S35007[56], 16}, {NULL}};
const Segment SGI316353[] = {{&S35007[51], 3}, {NULL}};
const Segment SGI316354[] = {{&S35008[30], 3}, {NULL}};
const Segment SGI316355[] = {{&S35008[8], 3}, {NULL}};
const Segment SGI316356[] = {{&S35008[13], 10}, {NULL}};
const Segment SGI316357[] = {{&S35008[42], 7}, {NULL}};
const Segment SGI316358[] = {{&S35008[35], 14}, {NULL}};
const Segment SGI316359[] = {{&S35008[64], 11}, {NULL}};
const Segment SGI316360[] = {{&S35008[64], 16}, {NULL}};
const Segment SGI316361[] = {{&S35009[30], 3}, {NULL}};
const Segment SGI316362[] = {{&S35009[8], 3}, {NULL}};
const Segment SGI316363[] = {{&S35009[13], 10}, {NULL}};
const Segment SGI316364[] = {{&S35009[42], 7}, {NULL}};
const Segment SGI316365[] = {{&S35009[35], 14}, {NULL}};
const Segment SGI316366[] = {{&S35009[51], 11}, {NULL}};
const Segment SGI316367[] = {{&S35009[51], 16}, {NULL}};
const Segment SGI316368[] = {{&S35010[30], 3}, {NULL}};
const Segment SGI316369[] = {{&S35010[8], 3}, {NULL}};
const Segment SGI316370[] = {{&S35010[13], 10}, {NULL}};
const Segment SGI316371[] = {{&S35010[42], 7}, {NULL}};
const Segment SGI316372[] = {{&S35010[35], 14}, {NULL}};
const Segment SGI316373[] = {{&S35010[56], 11}, {NULL}};
const Segment SGI316374[] = {{&S35010[119], 15}, {NULL}};
const Segment SGI316375[] = {{&S35010[56], 16}, {NULL}};
const Segment SGI316376[] = {{&S35010[51], 3}, {NULL}};
const Segment SGI316377[] = {{&S35011[30], 3}, {NULL}};
const Segment SGI316378[] = {{&S35011[8], 3}, {NULL}};
const Segment SGI316379[] = {{&S35011[13], 10}, {NULL}};
const Segment SGI316380[] = {{&S35011[42], 7}, {NULL}};
const Segment SGI316381[] = {{&S35011[35], 14}, {NULL}};
const Segment SGI316382[] = {{&S35011[72], 11}, {NULL}};
const Segment SGI316383[] = {{&S35011[135], 15}, {NULL}};
const Segment SGI316384[] = {{&S35011[72], 16}, {NULL}};
const Segment SGI316385[] = {{&S51533[3], 4}, {NULL}};
const Segment SGI316386[] = {{&S35210[30], 3}, {NULL}};
const Segment SGI316387[] = {{&S35210[8], 3}, {NULL}};
const Segment SGI316388[] = {{&S35210[13], 10}, {NULL}};
const Segment SGI316389[] = {{&S35210[42], 7}, {NULL}};
const Segment SGI316390[] = {{&S35210[35], 14}, {NULL}};
const Segment SGI316391[] = {{&S35210[72], 11}, {NULL}};
const Segment SGI316392[] = {{&S35210[137], 15}, {NULL}};
const Segment SGI316393[] = {{&S35210[72], 16}, {NULL}};
const Segment SGI316394[] = {{&S35216[30], 3}, {NULL}};
const Segment SGI316395[] = {{&S35216[8], 3}, {NULL}};
const Segment SGI316396[] = {{&S35216[13], 10}, {NULL}};
const Segment SGI316397[] = {{&S35216[42], 7}, {NULL}};
const Segment SGI316398[] = {{&S35216[35], 14}, {NULL}};
const Segment SGI316399[] = {{&S35216[64], 11}, {NULL}};
const Segment SGI316400[] = {{&S35216[64], 16}, {NULL}};
const Segment SGI316401[] = {{&S35216[91], 9}, {NULL}};
const Segment SGI316402[] = {{&S35217[30], 3}, {NULL}};
const Segment SGI316403[] = {{&S35217[8], 3}, {NULL}};
const Segment SGI316404[] = {{&S35217[13], 10}, {NULL}};
const Segment SGI316405[] = {{&S35217[42], 7}, {NULL}};
const Segment SGI316406[] = {{&S35217[35], 14}, {NULL}};
const Segment SGI316407[] = {{&S35217[51], 11}, {NULL}};
const Segment SGI316408[] = {{&S35217[51], 16}, {NULL}};
const Segment SGI316409[] = {{&S35218[30], 3}, {NULL}};
const Segment SGI316410[] = {{&S35218[8], 3}, {NULL}};
const Segment SGI316411[] = {{&S35218[13], 10}, {NULL}};
const Segment SGI316412[] = {{&S35218[42], 7}, {NULL}};
const Segment SGI316413[] = {{&S35218[35], 14}, {NULL}};
const Segment SGI316414[] = {{&S35218[71], 11}, {NULL}};
const Segment SGI316415[] = {{&S35218[51], 6}, {NULL}};
const Segment SGI316416[] = {{&S35218[71], 16}, {NULL}};
const Segment SGI316417[] = {{&S35220[30], 3}, {&S35220[121], 3}, {NULL}};
const Segment SGI316418[] = {{&S35220[8], 3}, {&S35220[8], 3}, {NULL}};
const Segment SGI316419[] = {{&S35220[13], 10}, {&S35220[13], 10}, {NULL}};
const Segment SGI316420[] = {{&S35220[42], 7}, {&S35220[42], 7}, {NULL}};
const Segment SGI316421[] = {{&S35220[35], 14}, {&S35220[35], 14}, {NULL}};
const Segment SGI316422[] = {{&S35220[64], 11}, {NULL}};
const Segment SGI316423[] = {{&S35220[64], 16}, {NULL}};
const Segment SGI316424[] = {{&S35220[142], 3}, {NULL}};
const Segment SGI316425[] = {{&S35223[30], 3}, {NULL}};
const Segment SGI316426[] = {{&S35223[8], 3}, {NULL}};
const Segment SGI316427[] = {{&S35223[13], 10}, {NULL}};
const Segment SGI316428[] = {{&S35223[42], 7}, {NULL}};
const Segment SGI316429[] = {{&S35223[35], 14}, {NULL}};
const Segment SGI316430[] = {{&S35223[56], 11}, {NULL}};
const Segment SGI316431[] = {{&S35223[121], 15}, {NULL}};
const Segment SGI316432[] = {{&S35223[56], 16}, {NULL}};
const Segment SGI316433[] = {{&S35223[51], 3}, {NULL}};
const Segment SGI316434[] = {{&S35225[30], 3}, {NULL}};
const Segment SGI316435[] = {{&S35225[8], 3}, {NULL}};
const Segment SGI316436[] = {{&S35225[13], 10}, {NULL}};
const Segment SGI316437[] = {{&S35225[42], 7}, {NULL}};
const Segment SGI316438[] = {{&S35225[35], 14}, {NULL}};
const Segment SGI316439[] = {{&S35225[51], 11}, {NULL}};
const Segment SGI316440[] = {{&S35225[51], 16}, {NULL}};
const Segment SGI316441[] = {{&S35226[30], 3}, {NULL}};
const Segment SGI316442[] = {{&S35226[8], 3}, {NULL}};
const Segment SGI316443[] = {{&S35226[13], 10}, {NULL}};
const Segment SGI316444[] = {{&S35226[42], 7}, {NULL}};
const Segment SGI316445[] = {{&S35226[35], 14}, {NULL}};
const Segment SGI316446[] = {{&S35226[118], 11}, {NULL}};
const Segment SGI316447[] = {{&S35226[118], 16}, {NULL}};
const Segment SGI316448[] = {{&S35226[51], 3}, {NULL}};
const Segment SGI316449[] = {{&S35227[30], 3}, {NULL}};
const Segment SGI316450[] = {{&S35227[8], 3}, {NULL}};
const Segment SGI316451[] = {{&S35227[13], 10}, {NULL}};
const Segment SGI316452[] = {{&S35227[42], 7}, {NULL}};
const Segment SGI316453[] = {{&S35227[35], 14}, {NULL}};
const Segment SGI316454[] = {{&S35227[64], 11}, {NULL}};
const Segment SGI316455[] = {{&S35227[64], 16}, {NULL}};
const Segment SGI316456[] = {{&S35228[30], 3}, {NULL}};
const Segment SGI316457[] = {{&S35228[8], 3}, {NULL}};
const Segment SGI316458[] = {{&S35228[13], 10}, {NULL}};
const Segment SGI316459[] = {{&S35228[42], 7}, {NULL}};
const Segment SGI316460[] = {{&S35228[35], 14}, {NULL}};
const Segment SGI316461[] = {{&S35228[64], 11}, {NULL}};
const Segment SGI316462[] = {{&S35228[64], 16}, {NULL}};
const Segment SGI316463[] = {{&S35229[30], 3}, {NULL}};
const Segment SGI316464[] = {{&S35229[291], 9}, {NULL}};
const Segment SGI316465[] = {{&S35229[8], 3}, {NULL}};
const Segment SGI316466[] = {{&S35229[13], 10}, {NULL}};
const Segment SGI316467[] = {{&S35229[42], 7}, {NULL}};
const Segment SGI316468[] = {{&S35229[35], 14}, {NULL}};
const Segment SGI316469[] = {{&S35229[64], 11}, {NULL}};
const Segment SGI316470[] = {{&S35229[64], 16}, {NULL}};
const Segment SGI316471[] = {{&S35230[30], 3}, {NULL}};
const Segment SGI316472[] = {{&S35230[8], 3}, {NULL}};
const Segment SGI316473[] = {{&S35230[13], 10}, {NULL}};
const Segment SGI316474[] = {{&S35230[42], 7}, {NULL}};
const Segment SGI316475[] = {{&S35230[35], 14}, {NULL}};
const Segment SGI316476[] = {{&S35230[81], 11}, {NULL}};
const Segment SGI316477[] = {{&S35230[81], 16}, {NULL}};
const Segment SGI316478[] = {{&S35237[30], 3}, {NULL}};
const Segment SGI316479[] = {{&S35237[8], 3}, {NULL}};
const Segment SGI316480[] = {{&S35237[13], 10}, {NULL}};
const Segment SGI316481[] = {{&S35237[42], 7}, {NULL}};
const Segment SGI316482[] = {{&S35237[35], 14}, {NULL}};
const Segment SGI316483[] = {{&S35237[51], 11}, {NULL}};
const Segment SGI316484[] = {{&S35237[51], 16}, {NULL}};
const Segment SGI316485[] = {{&S35238[30], 3}, {NULL}};
const Segment SGI316486[] = {{&S35238[8], 3}, {NULL}};
const Segment SGI316487[] = {{&S35238[13], 10}, {NULL}};
const Segment SGI316488[] = {{&S35238[42], 7}, {NULL}};
const Segment SGI316489[] = {{&S35238[35], 14}, {NULL}};
const Segment SGI316490[] = {{&S35238[56], 11}, {NULL}};
const Segment SGI316491[] = {{&S35238[56], 16}, {NULL}};
const Segment SGI316492[] = {{&S35238[51], 3}, {NULL}};
const Segment SGI316493[] = {{&S35239[30], 3}, {NULL}};
const Segment SGI316494[] = {{&S35239[8], 3}, {NULL}};
const Segment SGI316495[] = {{&S35239[13], 10}, {NULL}};
const Segment SGI316496[] = {{&S35239[42], 7}, {NULL}};
const Segment SGI316497[] = {{&S35239[35], 14}, {NULL}};
const Segment SGI316498[] = {{&S35239[64], 11}, {NULL}};
const Segment SGI316499[] = {{&S35239[128], 15}, {NULL}};
const Segment SGI316500[] = {{&S35239[64], 16}, {NULL}};
const Segment SGI316501[] = {{&S35241[30], 3}, {&S35241[120], 3}, {NULL}};
const Segment SGI316502[] = {{&S35241[8], 3}, {&S35241[8], 3}, {NULL}};
const Segment SGI316503[] = {{&S35241[13], 10}, {&S35241[13], 10}, {NULL}};
const Segment SGI316504[] = {{&S35241[42], 7}, {&S35241[42], 7}, {NULL}};
const Segment SGI316505[] = {{&S35241[35], 14}, {&S35241[35], 14}, {NULL}};
const Segment SGI316506[] = {{&S35241[64], 11}, {NULL}};
const Segment SGI316507[] = {{&S35241[64], 16}, {NULL}};
const Segment SGI316508[] = {{&S35241[141], 3}, {NULL}};
const Segment SGI316509[] = {{&S35242[30], 3}, {NULL}};
const Segment SGI316510[] = {{&S35242[8], 3}, {NULL}};
const Segment SGI316511[] = {{&S35242[13], 10}, {NULL}};
const Segment SGI316512[] = {{&S35242[42], 7}, {NULL}};
const Segment SGI316513[] = {{&S35242[35], 14}, {NULL}};
const Segment SGI316514[] = {{&S35242[51], 11}, {NULL}};
const Segment SGI316515[] = {{&S35242[51], 16}, {NULL}};
const Segment SGI316516[] = {{&S35243[30], 3}, {NULL}};
const Segment SGI316517[] = {{&S35243[8], 3}, {NULL}};
const Segment SGI316518[] = {{&S35243[13], 10}, {NULL}};
const Segment SGI316519[] = {{&S35243[42], 7}, {NULL}};
const Segment SGI316520[] = {{&S35243[35], 14}, {NULL}};
const Segment SGI316521[] = {{&S35243[64], 11}, {NULL}};
const Segment SGI316522[] = {{&S35243[64], 16}, {NULL}};
const Segment SGI316523[] = {{&S35244[30], 3}, {NULL}};
const Segment SGI316524[] = {{&S35244[8], 3}, {NULL}};
const Segment SGI316525[] = {{&S35244[13], 10}, {NULL}};
const Segment SGI316526[] = {{&S35244[42], 7}, {NULL}};
const Segment SGI316527[] = {{&S35244[35], 14}, {NULL}};
const Segment SGI316528[] = {{&S35244[56], 11}, {NULL}};
const Segment SGI316529[] = {{&S35244[120], 15}, {NULL}};
const Segment SGI316530[] = {{&S35244[56], 16}, {NULL}};
const Segment SGI316531[] = {{&S35244[51], 3}, {NULL}};
const Segment SGI316532[] = {{&S35248[30], 3}, {NULL}};
const Segment SGI316533[] = {{&S35248[8], 3}, {NULL}};
const Segment SGI316534[] = {{&S35248[13], 10}, {NULL}};
const Segment SGI316535[] = {{&S35248[42], 7}, {NULL}};
const Segment SGI316536[] = {{&S35248[35], 14}, {NULL}};
const Segment SGI316537[] = {{&S35248[64], 11}, {NULL}};
const Segment SGI316538[] = {{&S35248[64], 16}, {NULL}};
const Segment SGI316539[] = {{&S35249[30], 3}, {NULL}};
const Segment SGI316540[] = {{&S35249[8], 3}, {NULL}};
const Segment SGI316541[] = {{&S35249[13], 10}, {NULL}};
const Segment SGI316542[] = {{&S35249[42], 7}, {NULL}};
const Segment SGI316543[] = {{&S35249[35], 14}, {NULL}};
const Segment SGI316544[] = {{&S35249[64], 11}, {NULL}};
const Segment SGI316545[] = {{&S35249[64], 16}, {NULL}};
const Segment SGI316546[] = {{&S35249[90], 9}, {NULL}};
const Segment SGI316547[] = {{&S35250[30], 3}, {NULL}};
const Segment SGI316548[] = {{&S35250[8], 3}, {NULL}};
const Segment SGI316549[] = {{&S35250[13], 10}, {NULL}};
const Segment SGI316550[] = {{&S35250[42], 7}, {NULL}};
const Segment SGI316551[] = {{&S35250[35], 14}, {NULL}};
const Segment SGI316552[] = {{&S35250[64], 11}, {NULL}};
const Segment SGI316553[] = {{&S35250[64], 16}, {NULL}};
const Segment SGI316554[] = {{&S35251[30], 3}, {NULL}};
const Segment SGI316555[] = {{&S35251[8], 3}, {NULL}};
const Segment SGI316556[] = {{&S35251[13], 10}, {NULL}};
const Segment SGI316557[] = {{&S35251[42], 7}, {NULL}};
const Segment SGI316558[] = {{&S35251[35], 14}, {NULL}};
const Segment SGI316559[] = {{&S35251[69], 11}, {NULL}};
const Segment SGI316560[] = {{&S35251[69], 16}, {NULL}};
const Segment SGI316561[] = {{&S35251[64], 3}, {NULL}};
const Segment SGI316562[] = {{&S35283[30], 3}, {NULL}};
const Segment SGI316563[] = {{&S35283[8], 3}, {NULL}};
const Segment SGI316564[] = {{&S35283[13], 10}, {NULL}};
const Segment SGI316565[] = {{&S35283[42], 7}, {NULL}};
const Segment SGI316566[] = {{&S35283[35], 14}, {NULL}};
const Segment SGI316567[] = {{&S35283[72], 11}, {NULL}};
const Segment SGI316568[] = {{&S35283[136], 15}, {NULL}};
const Segment SGI316569[] = {{&S35283[72], 16}, {NULL}};
const Segment SGI316570[] = {{&S35284[30], 3}, {NULL}};
const Segment SGI316571[] = {{&S35284[8], 3}, {NULL}};
const Segment SGI316572[] = {{&S35284[13], 10}, {NULL}};
const Segment SGI316573[] = {{&S35284[42], 7}, {NULL}};
const Segment SGI316574[] = {{&S35284[35], 14}, {NULL}};
const Segment SGI316575[] = {{&S35284[51], 11}, {NULL}};
const Segment SGI316576[] = {{&S35284[51], 16}, {NULL}};
const Segment SGI316577[] = {{&S35286[30], 3}, {NULL}};
const Segment SGI316578[] = {{&S35286[8], 3}, {NULL}};
const Segment SGI316579[] = {{&S35286[13], 10}, {NULL}};
const Segment SGI316580[] = {{&S35286[42], 7}, {NULL}};
const Segment SGI316581[] = {{&S35286[35], 14}, {NULL}};
const Segment SGI316582[] = {{&S35286[56], 11}, {NULL}};
const Segment SGI316583[] = {{&S35286[120], 15}, {NULL}};
const Segment SGI316584[] = {{&S35286[56], 16}, {NULL}};
const Segment SGI316585[] = {{&S35286[51], 3}, {NULL}};
const Segment SGI316586[] = {{&S35288[30], 3}, {NULL}};
const Segment SGI316587[] = {{&S35288[8], 3}, {NULL}};
const Segment SGI316588[] = {{&S35288[13], 10}, {NULL}};
const Segment SGI316589[] = {{&S35288[42], 7}, {NULL}};
const Segment SGI316590[] = {{&S35288[35], 14}, {NULL}};
const Segment SGI316591[] = {{&S35288[64], 11}, {NULL}};
const Segment SGI316592[] = {{&S35288[64], 16}, {NULL}};
const Segment SGI316593[] = {{&S35288[90], 9}, {NULL}};
const Segment SGI316594[] = {{&S35289[30], 3}, {NULL}};
const Segment SGI316595[] = {{&S35289[8], 3}, {NULL}};
const Segment SGI316596[] = {{&S35289[13], 10}, {NULL}};
const Segment SGI316597[] = {{&S35289[42], 7}, {NULL}};
const Segment SGI316598[] = {{&S35289[35], 14}, {NULL}};
const Segment SGI316599[] = {{&S35289[51], 11}, {NULL}};
const Segment SGI316600[] = {{&S35289[51], 16}, {NULL}};
const Segment SGI316601[] = {{&S35291[30], 3}, {&S35291[120], 3}, {NULL}};
const Segment SGI316602[] = {{&S35291[8], 3}, {&S35291[8], 3}, {NULL}};
const Segment SGI316603[] = {{&S35291[13], 10}, {&S35291[13], 10}, {NULL}};
const Segment SGI316604[] = {{&S35291[42], 7}, {&S35291[42], 7}, {NULL}};
const Segment SGI316605[] = {{&S35291[35], 14}, {&S35291[35], 14}, {NULL}};
const Segment SGI316606[] = {{&S35291[64], 11}, {NULL}};
const Segment SGI316607[] = {{&S35291[64], 16}, {NULL}};
const Segment SGI316608[] = {{&S35291[141], 3}, {NULL}};
const Segment SGI316609[] = {{&S35292[30], 3}, {NULL}};
const Segment SGI316610[] = {{&S35292[8], 3}, {NULL}};
const Segment SGI316611[] = {{&S35292[13], 10}, {NULL}};
const Segment SGI316612[] = {{&S35292[42], 7}, {NULL}};
const Segment SGI316613[] = {{&S35292[35], 14}, {NULL}};
const Segment SGI316614[] = {{&S35292[51], 11}, {NULL}};
const Segment SGI316615[] = {{&S35292[51], 16}, {NULL}};
const Segment SGI316616[] = {{&S35293[30], 3}, {NULL}};
const Segment SGI316617[] = {{&S35293[8], 3}, {NULL}};
const Segment SGI316618[] = {{&S35293[13], 10}, {NULL}};
const Segment SGI316619[] = {{&S35293[42], 7}, {NULL}};
const Segment SGI316620[] = {{&S35293[35], 14}, {NULL}};
const Segment SGI316621[] = {{&S35293[71], 11}, {NULL}};
const Segment SGI316622[] = {{&S35293[51], 6}, {NULL}};
const Segment SGI316623[] = {{&S35293[71], 16}, {NULL}};
const Segment SGI316624[] = {{&S35294[30], 3}, {NULL}};
const Segment SGI316625[] = {{&S35294[8], 3}, {NULL}};
const Segment SGI316626[] = {{&S35294[13], 10}, {NULL}};
const Segment SGI316627[] = {{&S35294[42], 7}, {NULL}};
const Segment SGI316628[] = {{&S35294[35], 14}, {NULL}};
const Segment SGI316629[] = {{&S35294[64], 11}, {NULL}};
const Segment SGI316630[] = {{&S35294[64], 16}, {NULL}};
const Segment SGI316631[] = {{&S35295[30], 3}, {NULL}};
const Segment SGI316632[] = {{&S35295[8], 3}, {NULL}};
const Segment SGI316633[] = {{&S35295[13], 10}, {NULL}};
const Segment SGI316634[] = {{&S35295[42], 7}, {NULL}};
const Segment SGI316635[] = {{&S35295[35], 14}, {NULL}};
const Segment SGI316636[] = {{&S35295[64], 11}, {NULL}};
const Segment SGI316637[] = {{&S35295[64], 16}, {NULL}};
const Segment SGI316638[] = {{&S35296[30], 3}, {NULL}};
const Segment SGI316639[] = {{&S35296[8], 3}, {NULL}};
const Segment SGI316640[] = {{&S35296[13], 10}, {NULL}};
const Segment SGI316641[] = {{&S35296[42], 7}, {NULL}};
const Segment SGI316642[] = {{&S35296[35], 14}, {NULL}};
const Segment SGI316643[] = {{&S35296[56], 11}, {NULL}};
const Segment SGI316644[] = {{&S35296[56], 16}, {NULL}};
const Segment SGI316645[] = {{&S35296[51], 3}, {NULL}};
const Segment SGI316646[] = {{&S35297[30], 3}, {NULL}};
const Segment SGI316647[] = {{&S35297[8], 3}, {NULL}};
const Segment SGI316648[] = {{&S35297[13], 10}, {NULL}};
const Segment SGI316649[] = {{&S35297[42], 7}, {NULL}};
const Segment SGI316650[] = {{&S35297[35], 14}, {NULL}};
const Segment SGI316651[] = {{&S35297[76], 11}, {NULL}};
const Segment SGI316652[] = {{&S35297[51], 6}, {NULL}};
const Segment SGI316653[] = {{&S35297[76], 16}, {NULL}};
const Segment SGI316654[] = {{&S68876[7], 4}, {NULL}};
const Segment SGI316655[] = {{&S68876[0], 123}, {NULL}};
const Segment SGI316656[] = {{&S35894[30], 3}, {NULL}};
const Segment SGI316657[] = {{&S35894[8], 3}, {NULL}};
const Segment SGI316658[] = {{&S35894[13], 10}, {NULL}};
const Segment SGI316659[] = {{&S35894[42], 7}, {NULL}};
const Segment SGI316660[] = {{&S35894[35], 14}, {NULL}};
const Segment SGI316661[] = {{&S35894[51], 11}, {NULL}};
const Segment SGI316662[] = {{&S35894[51], 16}, {NULL}};
const Segment SGI316663[] = {{&S35895[30], 3}, {NULL}};
const Segment SGI316664[] = {{&S35895[8], 3}, {NULL}};
const Segment SGI316665[] = {{&S35895[13], 10}, {NULL}};
const Segment SGI316666[] = {{&S35895[42], 7}, {NULL}};
const Segment SGI316667[] = {{&S35895[35], 14}, {NULL}};
const Segment SGI316668[] = {{&S35895[56], 11}, {NULL}};
const Segment SGI316669[] = {{&S35895[56], 16}, {NULL}};
const Segment SGI316670[] = {{&S35895[51], 3}, {NULL}};
const Segment SGI316671[] = {{&S35896[30], 3}, {NULL}};
const Segment SGI316672[] = {{&S35896[8], 3}, {NULL}};
const Segment SGI316673[] = {{&S35896[13], 10}, {NULL}};
const Segment SGI316674[] = {{&S35896[42], 7}, {NULL}};
const Segment SGI316675[] = {{&S35896[35], 14}, {NULL}};
const Segment SGI316676[] = {{&S35896[56], 11}, {NULL}};
const Segment SGI316677[] = {{&S35896[56], 16}, {NULL}};
const Segment SGI316678[] = {{&S35896[51], 3}, {NULL}};
const Segment SGI316679[] = {{&S35897[30], 3}, {NULL}};
const Segment SGI316680[] = {{&S35897[8], 3}, {NULL}};
const Segment SGI316681[] = {{&S35897[13], 10}, {NULL}};
const Segment SGI316682[] = {{&S35897[42], 7}, {NULL}};
const Segment SGI316683[] = {{&S35897[35], 14}, {NULL}};
const Segment SGI316684[] = {{&S35897[71], 11}, {NULL}};
const Segment SGI316685[] = {{&S35897[51], 6}, {NULL}};
const Segment SGI316686[] = {{&S35897[71], 16}, {NULL}};
const Segment SGI316687[] = {{&S35899[30], 3}, {NULL}};
const Segment SGI316688[] = {{&S35899[8], 3}, {NULL}};
const Segment SGI316689[] = {{&S35899[13], 10}, {NULL}};
const Segment SGI316690[] = {{&S35899[42], 7}, {NULL}};
const Segment SGI316691[] = {{&S35899[35], 14}, {NULL}};
const Segment SGI316692[] = {{&S35899[64], 11}, {NULL}};
const Segment SGI316693[] = {{&S35899[64], 16}, {NULL}};
const Segment SGI316694[] = {{&S35900[30], 3}, {NULL}};
const Segment SGI316695[] = {{&S35900[8], 3}, {NULL}};
const Segment SGI316696[] = {{&S35900[13], 10}, {NULL}};
const Segment SGI316697[] = {{&S35900[42], 7}, {NULL}};
const Segment SGI316698[] = {{&S35900[35], 14}, {NULL}};
const Segment SGI316699[] = {{&S35900[51], 11}, {NULL}};
const Segment SGI316700[] = {{&S35900[51], 16}, {NULL}};
const Segment SGI316701[] = {{&S35900[77], 3}, {NULL}};
const Segment SGI316702[] = {{&S35955[30], 3}, {NULL}};
const Segment SGI316703[] = {{&S35955[8], 3}, {NULL}};
const Segment SGI316704[] = {{&S35955[13], 10}, {NULL}};
const Segment SGI316705[] = {{&S35955[42], 7}, {NULL}};
const Segment SGI316706[] = {{&S35955[35], 14}, {NULL}};
const Segment SGI316707[] = {{&S35955[131], 15}, {NULL}};
const Segment SGI316708[] = {{&S35956[30], 3}, {NULL}};
const Segment SGI316709[] = {{&S35956[8], 3}, {NULL}};
const Segment SGI316710[] = {{&S35956[13], 10}, {NULL}};
const Segment SGI316711[] = {{&S35956[42], 7}, {NULL}};
const Segment SGI316712[] = {{&S35957[30], 3}, {NULL}};
const Segment SGI316713[] = {{&S35957[8], 3}, {NULL}};
const Segment SGI316714[] = {{&S35957[13], 10}, {NULL}};
const Segment SGI316715[] = {{&S35957[42], 7}, {NULL}};
const Segment SGI316716[] = {{&S35957[35], 14}, {NULL}};
const Segment SGI316717[] = {{&S35957[51], 6}, {NULL}};
const Segment SGI316718[] = {{&S35958[30], 3}, {NULL}};
const Segment SGI316719[] = {{&S35958[8], 3}, {NULL}};
const Segment SGI316720[] = {{&S35958[13], 10}, {NULL}};
const Segment SGI316721[] = {{&S35958[42], 7}, {NULL}};
const Segment SGI316722[] = {{&S35958[35], 14}, {NULL}};
const Segment SGI316723[] = {{&S35958[163], 4}, {NULL}};
const Segment SGI316724[] = {{&S35958[51], 6}, {NULL}};
const Segment SGI316725[] = {{&S35959[30], 3}, {NULL}};
const Segment SGI316726[] = {{&S35959[8], 3}, {NULL}};
const Segment SGI316727[] = {{&S35959[13], 10}, {NULL}};
const Segment SGI316728[] = {{&S35959[42], 7}, {NULL}};
const Segment SGI316729[] = {{&S35959[35], 14}, {NULL}};
const Segment SGI316730[] = {{&S35959[151], 15}, {NULL}};
const Segment SGI316731[] = {{&S35959[51], 6}, {NULL}};
const Segment SGI316732[] = {{&S35960[30], 3}, {NULL}};
const Segment SGI316733[] = {{&S35960[8], 3}, {NULL}};
const Segment SGI316734[] = {{&S35960[13], 10}, {NULL}};
const Segment SGI316735[] = {{&S35960[42], 7}, {NULL}};
const Segment SGI316736[] = {{&S35960[35], 14}, {NULL}};
const Segment SGI316737[] = {{&S35961[30], 3}, {NULL}};
const Segment SGI316738[] = {{&S35961[8], 3}, {NULL}};
const Segment SGI316739[] = {{&S35961[13], 10}, {NULL}};
const Segment SGI316740[] = {{&S35961[42], 7}, {NULL}};
const Segment SGI316741[] = {{&S35961[35], 14}, {NULL}};
const Segment SGI316742[] = {{&S35961[51], 6}, {NULL}};
const Segment SGI316743[] = {{&S35961[151], 9}, {NULL}};
const Segment SGI316744[] = {{&S35962[31], 3}, {NULL}};
const Segment SGI316745[] = {{&S35962[9], 3}, {NULL}};
const Segment SGI316746[] = {{&S35962[14], 10}, {NULL}};
const Segment SGI316747[] = {{&S35962[43], 7}, {NULL}};
const Segment SGI316748[] = {{&S35962[36], 14}, {NULL}};
const Segment SGI316749[] = {{&S35963[30], 3}, {NULL}};
const Segment SGI316750[] = {{&S35963[8], 3}, {NULL}};
const Segment SGI316751[] = {{&S35963[13], 10}, {NULL}};
const Segment SGI316752[] = {{&S35963[42], 7}, {NULL}};
const Segment SGI316753[] = {{&S35963[35], 14}, {NULL}};
const Segment SGI316754[] = {{&S35963[51], 5}, {NULL}};
const Segment SGI316755[] = {{&S36043[30], 3}, {NULL}};
const Segment SGI316756[] = {{&S36043[8], 3}, {NULL}};
const Segment SGI316757[] = {{&S36043[13], 10}, {NULL}};
const Segment SGI316758[] = {{&S36043[42], 3}, {NULL}};
const Segment SGI316759[] = {{&S36043[47], 7}, {&S36043[47], 7}, {NULL}};
const Segment SGI316760[] = {{&S36044[30], 3}, {NULL}};
const Segment SGI316761[] = {{&S36044[8], 3}, {NULL}};
const Segment SGI316762[] = {{&S36044[13], 10}, {NULL}};
const Segment SGI316763[] = {{&S36044[42], 3}, {NULL}};
const Segment SGI316764[] = {{&S36044[47], 7}, {NULL}};
const Segment SGI316765[] = {{&S36045[30], 3}, {NULL}};
const Segment SGI316766[] = {{&S36045[8], 3}, {NULL}};
const Segment SGI316767[] = {{&S36045[13], 10}, {NULL}};
const Segment SGI316768[] = {{&S36045[42], 3}, {NULL}};
const Segment SGI316769[] = {{&S36045[47], 7}, {NULL}};
const Segment SGI316770[] = {{&S36045[56], 13}, {NULL}};
const Segment SGI316771[] = {{&S36046[30], 3}, {NULL}};
const Segment SGI316772[] = {{&S36046[8], 3}, {NULL}};
const Segment SGI316773[] = {{&S36046[13], 10}, {NULL}};
const Segment SGI316774[] = {{&S36046[55], 3}, {NULL}};
const Segment SGI316775[] = {{&S36046[60], 7}, {NULL}};
const Segment SGI316776[] = {{&S36047[30], 3}, {NULL}};
const Segment SGI316777[] = {{&S36047[8], 3}, {NULL}};
const Segment SGI316778[] = {{&S36047[13], 10}, {NULL}};
const Segment SGI316779[] = {{&S36047[55], 3}, {NULL}};
const Segment SGI316780[] = {{&S36047[60], 7}, {NULL}};
const Segment SGI316781[] = {{&S36048[30], 3}, {NULL}};
const Segment SGI316782[] = {{&S36048[8], 3}, {NULL}};
const Segment SGI316783[] = {{&S36048[13], 10}, {NULL}};
const Segment SGI316784[] = {{&S36048[55], 3}, {NULL}};
const Segment SGI316785[] = {{&S36048[60], 7}, {NULL}};
const Segment SGI316786[] = {{&S36049[30], 3}, {NULL}};
const Segment SGI316787[] = {{&S36049[8], 3}, {NULL}};
const Segment SGI316788[] = {{&S36049[13], 10}, {NULL}};
const Segment SGI316789[] = {{&S36049[55], 3}, {NULL}};
const Segment SGI316790[] = {{&S36049[60], 7}, {NULL}};
const Segment SGI316791[] = {{&S36050[30], 3}, {NULL}};
const Segment SGI316792[] = {{&S36050[8], 3}, {NULL}};
const Segment SGI316793[] = {{&S36050[13], 10}, {NULL}};
const Segment SGI316794[] = {{&S36050[55], 3}, {NULL}};
const Segment SGI316795[] = {{&S36050[60], 7}, {NULL}};
const Segment SGI316796[] = {{&S36051[30], 3}, {NULL}};
const Segment SGI316797[] = {{&S36051[8], 3}, {NULL}};
const Segment SGI316798[] = {{&S36051[13], 10}, {NULL}};
const Segment SGI316799[] = {{&S36051[55], 3}, {NULL}};
const Segment SGI316800[] = {{&S36051[60], 7}, {NULL}};
const Segment SGI316801[] = {{&S36052[30], 3}, {NULL}};
const Segment SGI316802[] = {{&S36052[8], 3}, {NULL}};
const Segment SGI316803[] = {{&S36052[13], 10}, {NULL}};
const Segment SGI316804[] = {{&S36052[55], 3}, {NULL}};
const Segment SGI316805[] = {{&S36052[65], 7}, {NULL}};
const Segment SGI316806[] = {{&S36052[60], 3}, {NULL}};
const Segment SGI316807[] = {{&S36053[30], 3}, {NULL}};
const Segment SGI316808[] = {{&S36053[8], 3}, {NULL}};
const Segment SGI316809[] = {{&S36053[13], 10}, {NULL}};
const Segment SGI316810[] = {{&S36053[55], 3}, {NULL}};
const Segment SGI316811[] = {{&S36053[65], 7}, {NULL}};
const Segment SGI316812[] = {{&S36053[60], 3}, {NULL}};
const Segment SGI316813[] = {{&S36054[30], 3}, {NULL}};
const Segment SGI316814[] = {{&S36054[8], 3}, {NULL}};
const Segment SGI316815[] = {{&S36054[13], 10}, {NULL}};
const Segment SGI316816[] = {{&S36054[55], 3}, {NULL}};
const Segment SGI316817[] = {{&S36054[60], 7}, {&S36054[60], 7}, {NULL}};
const Segment SGI316818[] = {{&S36055[30], 3}, {NULL}};
const Segment SGI316819[] = {{&S36055[8], 3}, {NULL}};
const Segment SGI316820[] = {{&S36055[13], 10}, {NULL}};
const Segment SGI316821[] = {{&S36055[55], 3}, {NULL}};
const Segment SGI316822[] = {{&S36055[60], 7}, {NULL}};
const Segment SGI316823[] = {{&S36055[69], 13}, {NULL}};
const Segment SGI316824[] = {{&S36056[30], 3}, {NULL}};
const Segment SGI316825[] = {{&S36056[8], 3}, {NULL}};
const Segment SGI316826[] = {{&S36056[13], 10}, {NULL}};
const Segment SGI316827[] = {{&S36056[55], 3}, {NULL}};
const Segment SGI316828[] = {{&S36056[60], 7}, {NULL}};
const Segment SGI316829[] = {{&S36056[69], 3}, {NULL}};
const Segment SGI316830[] = {{&S36057[30], 3}, {NULL}};
const Segment SGI316831[] = {{&S36057[8], 3}, {NULL}};
const Segment SGI316832[] = {{&S36057[13], 10}, {NULL}};
const Segment SGI316833[] = {{&S36057[55], 3}, {NULL}};
const Segment SGI316834[] = {{&S36057[60], 7}, {&S36057[60], 7}, {NULL}};
const Segment SGI316835[] = {{&S36058[30], 3}, {NULL}};
const Segment SGI316836[] = {{&S36058[8], 3}, {NULL}};
const Segment SGI316837[] = {{&S36058[13], 10}, {NULL}};
const Segment SGI316838[] = {{&S36058[55], 3}, {NULL}};
const Segment SGI316839[] = {{&S36058[60], 7}, {&S36058[60], 7}, {NULL}};
const Segment SGI316840[] = {{&S36059[30], 3}, {NULL}};
const Segment SGI316841[] = {{&S36059[8], 3}, {NULL}};
const Segment SGI316842[] = {{&S36059[13], 10}, {NULL}};
const Segment SGI316843[] = {{&S36059[55], 3}, {NULL}};
const Segment SGI316844[] = {{&S36059[75], 7}, {NULL}};
const Segment SGI316845[] = {{&S36059[60], 13}, {NULL}};
const Segment SGI316846[] = {{&S36060[30], 3}, {NULL}};
const Segment SGI316847[] = {{&S36060[8], 3}, {NULL}};
const Segment SGI316848[] = {{&S36060[13], 10}, {NULL}};
const Segment SGI316849[] = {{&S36060[55], 3}, {NULL}};
const Segment SGI316850[] = {{&S36060[60], 7}, {NULL}};
const Segment SGI316851[] = {{&S36060[69], 3}, {NULL}};
const Segment SGI316852[] = {{&S36061[30], 3}, {NULL}};
const Segment SGI316853[] = {{&S36061[8], 3}, {NULL}};
const Segment SGI316854[] = {{&S36061[13], 10}, {NULL}};
const Segment SGI316855[] = {{&S36061[55], 3}, {NULL}};
const Segment SGI316856[] = {{&S36061[75], 7}, {NULL}};
const Segment SGI316857[] = {{&S36061[60], 13}, {NULL}};
const Segment SGI316858[] = {{&S36062[30], 3}, {NULL}};
const Segment SGI316859[] = {{&S36062[8], 3}, {NULL}};
const Segment SGI316860[] = {{&S36062[13], 10}, {NULL}};
const Segment SGI316861[] = {{&S36062[55], 3}, {NULL}};
const Segment SGI316862[] = {{&S36062[75], 7}, {NULL}};
const Segment SGI316863[] = {{&S36062[60], 13}, {NULL}};
const Segment SGI316864[] = {{&S36063[30], 3}, {NULL}};
const Segment SGI316865[] = {{&S36063[8], 3}, {NULL}};
const Segment SGI316866[] = {{&S36063[13], 10}, {NULL}};
const Segment SGI316867[] = {{&S36063[55], 3}, {NULL}};
const Segment SGI316868[] = {{&S36063[60], 7}, {NULL}};
const Segment SGI316869[] = {{&S36063[69], 13}, {NULL}};
const Segment SGI316870[] = {{&S36064[30], 3}, {NULL}};
const Segment SGI316871[] = {{&S36064[8], 3}, {NULL}};
const Segment SGI316872[] = {{&S36064[13], 10}, {NULL}};
const Segment SGI316873[] = {{&S36064[55], 3}, {NULL}};
const Segment SGI316874[] = {{&S36064[65], 7}, {NULL}};
const Segment SGI316875[] = {{&S36064[60], 3}, {NULL}};
const Segment SGI316876[] = {{&S36065[30], 3}, {NULL}};
const Segment SGI316877[] = {{&S36065[8], 3}, {NULL}};
const Segment SGI316878[] = {{&S36065[13], 10}, {NULL}};
const Segment SGI316879[] = {{&S36065[55], 3}, {NULL}};
const Segment SGI316880[] = {{&S36065[60], 7}, {NULL}};
const Segment SGI316881[] = {{&S36065[69], 3}, {NULL}};
const Segment SGI316882[] = {{&S36066[30], 3}, {NULL}};
const Segment SGI316883[] = {{&S36066[8], 3}, {NULL}};
const Segment SGI316884[] = {{&S36066[13], 10}, {NULL}};
const Segment SGI316885[] = {{&S36066[55], 3}, {NULL}};
const Segment SGI316886[] = {{&S36066[60], 7}, {&S36066[60], 7}, {NULL}};
const Segment SGI316887[] = {{&S36067[30], 3}, {NULL}};
const Segment SGI316888[] = {{&S36067[8], 3}, {NULL}};
const Segment SGI316889[] = {{&S36067[13], 10}, {NULL}};
const Segment SGI316890[] = {{&S36067[55], 3}, {NULL}};
const Segment SGI316891[] = {{&S36067[80], 7}, {NULL}};
const Segment SGI316892[] = {{&S36067[65], 13}, {NULL}};
const Segment SGI316893[] = {{&S36067[60], 3}, {NULL}};
const Segment SGI316894[] = {{&S36068[30], 3}, {NULL}};
const Segment SGI316895[] = {{&S36068[8], 3}, {NULL}};
const Segment SGI316896[] = {{&S36068[13], 10}, {NULL}};
const Segment SGI316897[] = {{&S36068[42], 7}, {NULL}};
const Segment SGI316898[] = {{&S36068[35], 14}, {NULL}};
const Segment SGI316899[] = {{&S36068[74], 3}, {NULL}};
const Segment SGI316900[] = {{&S36069[30], 3}, {NULL}};
const Segment SGI316901[] = {{&S36069[8], 3}, {NULL}};
const Segment SGI316902[] = {{&S36069[13], 10}, {NULL}};
const Segment SGI316903[] = {{&S36069[42], 7}, {NULL}};
const Segment SGI316904[] = {{&S36069[35], 14}, {NULL}};
const Segment SGI316905[] = {{&S36069[51], 6}, {NULL}};
const Segment SGI316906[] = {{&S36069[81], 3}, {NULL}};
const Segment SGI316907[] = {{&S36070[30], 4}, {NULL}};
const Segment SGI316908[] = {{&S36070[8], 3}, {NULL}};
const Segment SGI316909[] = {{&S36070[13], 10}, {NULL}};
const Segment SGI316910[] = {{&S36070[43], 7}, {NULL}};
const Segment SGI316911[] = {{&S36070[36], 14}, {NULL}};
const Segment SGI316912[] = {{&S36070[75], 3}, {NULL}};
const Segment SGI316913[] = {{&S36071[30], 4}, {NULL}};
const Segment SGI316914[] = {{&S36071[8], 3}, {NULL}};
const Segment SGI316915[] = {{&S36071[13], 10}, {NULL}};
const Segment SGI316916[] = {{&S36071[43], 7}, {NULL}};
const Segment SGI316917[] = {{&S36071[36], 14}, {NULL}};
const Segment SGI316918[] = {{&S36071[52], 6}, {NULL}};
const Segment SGI316919[] = {{&S36071[82], 3}, {NULL}};
const Segment SGI316920[] = {{&S36072[30], 3}, {NULL}};
const Segment SGI316921[] = {{&S36072[8], 3}, {NULL}};
const Segment SGI316922[] = {{&S36072[13], 10}, {NULL}};
const Segment SGI316923[] = {{&S36072[42], 7}, {NULL}};
const Segment SGI316924[] = {{&S36072[35], 14}, {NULL}};
const Segment SGI316925[] = {{&S36072[131], 15}, {NULL}};
const Segment SGI316926[] = {{&S36072[162], 3}, {NULL}};
const Segment SGI316927[] = {{&S36073[30], 3}, {NULL}};
const Segment SGI316928[] = {{&S36073[8], 3}, {NULL}};
const Segment SGI316929[] = {{&S36073[13], 10}, {NULL}};
const Segment SGI316930[] = {{&S36073[42], 7}, {NULL}};
const Segment SGI316931[] = {{&S36073[35], 14}, {NULL}};
const Segment SGI316932[] = {{&S36073[138], 15}, {NULL}};
const Segment SGI316933[] = {{&S36073[51], 6}, {NULL}};
const Segment SGI316934[] = {{&S36073[169], 3}, {NULL}};
const Segment SGI316935[] = {{&S36074[30], 3}, {NULL}};
const Segment SGI316936[] = {{&S36074[8], 3}, {NULL}};
const Segment SGI316937[] = {{&S36074[13], 10}, {NULL}};
const Segment SGI316938[] = {{&S36074[42], 7}, {NULL}};
const Segment SGI316939[] = {{&S36074[35], 14}, {NULL}};
const Segment SGI316940[] = {{&S36074[141], 3}, {NULL}};
const Segment SGI316941[] = {{&S36075[30], 3}, {NULL}};
const Segment SGI316942[] = {{&S36075[8], 3}, {NULL}};
const Segment SGI316943[] = {{&S36075[13], 10}, {NULL}};
const Segment SGI316944[] = {{&S36075[42], 7}, {NULL}};
const Segment SGI316945[] = {{&S36075[35], 14}, {NULL}};
const Segment SGI316946[] = {{&S36075[51], 6}, {NULL}};
const Segment SGI316947[] = {{&S36075[148], 3}, {NULL}};
const Segment SGI316948[] = {{&S36076[30], 3}, {NULL}};
const Segment SGI316949[] = {{&S36076[8], 3}, {NULL}};
const Segment SGI316950[] = {{&S36076[13], 10}, {NULL}};
const Segment SGI316951[] = {{&S36076[42], 7}, {NULL}};
const Segment SGI316952[] = {{&S36076[35], 14}, {NULL}};
const Segment SGI316953[] = {{&S36076[131], 15}, {NULL}};
const Segment SGI316954[] = {{&S36076[152], 9}, {NULL}};
const Segment SGI316955[] = {{&S36076[177], 3}, {NULL}};
const Segment SGI316956[] = {{&S36077[30], 3}, {NULL}};
const Segment SGI316957[] = {{&S36077[8], 3}, {NULL}};
const Segment SGI316958[] = {{&S36077[13], 10}, {NULL}};
const Segment SGI316959[] = {{&S36077[42], 7}, {NULL}};
const Segment SGI316960[] = {{&S36077[35], 14}, {NULL}};
const Segment SGI316961[] = {{&S36077[138], 15}, {NULL}};
const Segment SGI316962[] = {{&S36077[51], 6}, {NULL}};
const Segment SGI316963[] = {{&S36077[179], 3}, {NULL}};
const Segment SGI316964[] = {{&S36078[30], 3}, {NULL}};
const Segment SGI316965[] = {{&S36078[8], 3}, {NULL}};
const Segment SGI316966[] = {{&S36078[13], 10}, {NULL}};
const Segment SGI316967[] = {{&S36078[42], 7}, {NULL}};
const Segment SGI316968[] = {{&S36078[35], 14}, {NULL}};
const Segment SGI316969[] = {{&S36078[131], 15}, {NULL}};
const Segment SGI316970[] = {{&S36078[172], 3}, {NULL}};
const Segment SGI316971[] = {{&S36079[30], 3}, {NULL}};
const Segment SGI316972[] = {{&S36079[8], 3}, {NULL}};
const Segment SGI316973[] = {{&S36079[13], 10}, {NULL}};
const Segment SGI316974[] = {{&S36079[42], 7}, {NULL}};
const Segment SGI316975[] = {{&S36079[35], 14}, {NULL}};
const Segment SGI316976[] = {{&S36079[151], 3}, {NULL}};
const Segment SGI316977[] = {{&S69749[0], 109}, {NULL}};
const Segment SGI316978[] = {{&S64310[3], 5}, {NULL}};
const Segment SGI316979[] = {{&S64310[29], 4}, {NULL}};
const Segment SGI316980[] = {{&S64314[3], 6}, {NULL}};
const Segment SGI316981[] = {{&S64314[30], 4}, {NULL}};
const Segment SGI316982[] = {{&S36374[30], 3}, {NULL}};
const Segment SGI316983[] = {{&S36374[8], 3}, {NULL}};
const Segment SGI316984[] = {{&S36374[13], 10}, {NULL}};
const Segment SGI316985[] = {{&S36374[42], 7}, {NULL}};
const Segment SGI316986[] = {{&S36374[35], 14}, {NULL}};
const Segment SGI316987[] = {{&S36374[51], 6}, {NULL}};
const Segment SGI316988[] = {{&S36374[71], 7}, {NULL}};
const Segment SGI316989[] = {{&S36374[78], 4}, {NULL}};
const Segment SGI316990[] = {{&S36375[30], 3}, {NULL}};
const Segment SGI316991[] = {{&S36375[8], 3}, {NULL}};
const Segment SGI316992[] = {{&S36375[13], 10}, {NULL}};
const Segment SGI316993[] = {{&S36375[42], 7}, {NULL}};
const Segment SGI316994[] = {{&S36375[35], 14}, {NULL}};
const Segment SGI316995[] = {{&S36375[64], 7}, {NULL}};
const Segment SGI316996[] = {{&S36375[71], 4}, {NULL}};
const Segment SGI316997[] = {{&S36376[30], 3}, {NULL}};
const Segment SGI316998[] = {{&S36376[8], 3}, {NULL}};
const Segment SGI316999[] = {{&S36376[13], 10}, {NULL}};
const Segment SGI317000[] = {{&S36376[42], 7}, {NULL}};
const Segment SGI317001[] = {{&S36376[35], 14}, {NULL}};
const Segment SGI317002[] = {{&S36376[138], 15}, {NULL}};
const Segment SGI317003[] = {{&S36376[51], 6}, {NULL}};
const Segment SGI317004[] = {{&S36376[169], 7}, {NULL}};
const Segment SGI317005[] = {{&S36376[176], 4}, {NULL}};
const Segment SGI317006[] = {{&S36377[30], 3}, {NULL}};
const Segment SGI317007[] = {{&S36377[8], 3}, {NULL}};
const Segment SGI317008[] = {{&S36377[13], 10}, {NULL}};
const Segment SGI317009[] = {{&S36377[42], 7}, {NULL}};
const Segment SGI317010[] = {{&S36377[35], 14}, {NULL}};
const Segment SGI317011[] = {{&S36377[131], 15}, {NULL}};
const Segment SGI317012[] = {{&S36377[162], 7}, {NULL}};
const Segment SGI317013[] = {{&S36377[169], 4}, {NULL}};
const Segment SGI317014[] = {{&S36378[30], 3}, {NULL}};
const Segment SGI317015[] = {{&S36378[8], 3}, {NULL}};
const Segment SGI317016[] = {{&S36378[13], 10}, {NULL}};
const Segment SGI317017[] = {{&S36378[42], 7}, {NULL}};
const Segment SGI317018[] = {{&S36378[35], 14}, {NULL}};
const Segment SGI317019[] = {{&S36378[131], 15}, {NULL}};
const Segment SGI317020[] = {{&S36378[177], 7}, {NULL}};
const Segment SGI317021[] = {{&S36378[184], 4}, {NULL}};
const Segment SGI317022[] = {{&S36378[162], 9}, {NULL}};
const Segment SGI317023[] = {{&S36379[30], 3}, {NULL}};
const Segment SGI317024[] = {{&S36379[8], 3}, {NULL}};
const Segment SGI317025[] = {{&S36379[13], 10}, {NULL}};
const Segment SGI317026[] = {{&S36379[42], 7}, {NULL}};
const Segment SGI317027[] = {{&S36379[35], 14}, {NULL}};
const Segment SGI317028[] = {{&S36379[141], 7}, {NULL}};
const Segment SGI317029[] = {{&S36379[148], 4}, {NULL}};
const Segment SGI317030[] = {{&S36380[30], 3}, {NULL}};
const Segment SGI317031[] = {{&S36380[8], 3}, {NULL}};
const Segment SGI317032[] = {{&S36380[13], 10}, {NULL}};
const Segment SGI317033[] = {{&S36380[42], 7}, {NULL}};
const Segment SGI317034[] = {{&S36380[35], 14}, {NULL}};
const Segment SGI317035[] = {{&S36380[51], 6}, {NULL}};
const Segment SGI317036[] = {{&S36380[158], 7}, {NULL}};
const Segment SGI317037[] = {{&S36380[165], 4}, {NULL}};
const Segment SGI317038[] = {{&S36381[30], 3}, {NULL}};
const Segment SGI317039[] = {{&S36381[8], 3}, {NULL}};
const Segment SGI317040[] = {{&S36381[13], 10}, {NULL}};
const Segment SGI317041[] = {{&S36381[42], 7}, {NULL}};
const Segment SGI317042[] = {{&S36381[35], 14}, {NULL}};
const Segment SGI317043[] = {{&S36381[131], 15}, {NULL}};
const Segment SGI317044[] = {{&S36381[152], 7}, {NULL}};
const Segment SGI317045[] = {{&S36381[159], 4}, {NULL}};
const Segment SGI317046[] = {{&S36382[30], 3}, {NULL}};
const Segment SGI317047[] = {{&S36382[8], 3}, {NULL}};
const Segment SGI317048[] = {{&S36382[13], 10}, {NULL}};
const Segment SGI317049[] = {{&S36382[42], 7}, {NULL}};
const Segment SGI317050[] = {{&S36382[35], 14}, {NULL}};
const Segment SGI317051[] = {{&S36382[51], 6}, {NULL}};
const Segment SGI317052[] = {{&S36382[86], 7}, {NULL}};
const Segment SGI317053[] = {{&S36382[71], 13}, {NULL}};
const Segment SGI317054[] = {{&S36382[93], 4}, {NULL}};
const Segment SGI317055[] = {{&S36383[30], 3}, {NULL}};
const Segment SGI317056[] = {{&S36383[8], 3}, {NULL}};
const Segment SGI317057[] = {{&S36383[13], 10}, {NULL}};
const Segment SGI317058[] = {{&S36383[42], 7}, {NULL}};
const Segment SGI317059[] = {{&S36383[35], 14}, {NULL}};
const Segment SGI317060[] = {{&S36383[79], 7}, {NULL}};
const Segment SGI317061[] = {{&S36383[64], 13}, {NULL}};
const Segment SGI317062[] = {{&S36383[86], 4}, {NULL}};
const Segment SGI317063[] = {{&S36384[30], 3}, {NULL}};
const Segment SGI317064[] = {{&S36384[8], 3}, {NULL}};
const Segment SGI317065[] = {{&S36384[13], 10}, {NULL}};
const Segment SGI317066[] = {{&S36384[42], 7}, {NULL}};
const Segment SGI317067[] = {{&S36384[35], 14}, {NULL}};
const Segment SGI317068[] = {{&S36384[138], 15}, {NULL}};
const Segment SGI317069[] = {{&S36384[51], 6}, {NULL}};
const Segment SGI317070[] = {{&S36384[179], 7}, {NULL}};
const Segment SGI317071[] = {{&S36384[186], 4}, {NULL}};
const Segment SGI317072[] = {{&S36385[30], 4}, {NULL}};
const Segment SGI317073[] = {{&S36385[8], 3}, {NULL}};
const Segment SGI317074[] = {{&S36385[13], 10}, {NULL}};
const Segment SGI317075[] = {{&S36385[43], 7}, {NULL}};
const Segment SGI317076[] = {{&S36385[36], 14}, {NULL}};
const Segment SGI317077[] = {{&S36385[52], 6}, {NULL}};
const Segment SGI317078[] = {{&S36385[72], 7}, {NULL}};
const Segment SGI317079[] = {{&S36385[79], 4}, {NULL}};
const Segment SGI317080[] = {{&S36386[30], 3}, {NULL}};
const Segment SGI317081[] = {{&S36386[8], 3}, {NULL}};
const Segment SGI317082[] = {{&S36386[13], 10}, {NULL}};
const Segment SGI317083[] = {{&S36386[42], 7}, {NULL}};
const Segment SGI317084[] = {{&S36386[35], 14}, {NULL}};
const Segment SGI317085[] = {{&S36386[138], 15}, {NULL}};
const Segment SGI317086[] = {{&S36386[179], 4}, {NULL}};
const Segment SGI317087[] = {{&S36386[51], 6}, {NULL}};
const Segment SGI317088[] = {{&S36386[189], 7}, {NULL}};
const Segment SGI317089[] = {{&S36386[196], 4}, {NULL}};
const Segment SGI317090[] = {{&S36387[30], 4}, {NULL}};
const Segment SGI317091[] = {{&S36387[8], 3}, {NULL}};
const Segment SGI317092[] = {{&S36387[13], 10}, {NULL}};
const Segment SGI317093[] = {{&S36387[43], 7}, {NULL}};
const Segment SGI317094[] = {{&S36387[36], 14}, {NULL}};
const Segment SGI317095[] = {{&S36387[65], 7}, {NULL}};
const Segment SGI317096[] = {{&S36387[72], 4}, {NULL}};
const Segment SGI317097[] = {{&S36388[30], 4}, {NULL}};
const Segment SGI317098[] = {{&S36388[8], 3}, {NULL}};
const Segment SGI317099[] = {{&S36388[13], 10}, {NULL}};
const Segment SGI317100[] = {{&S36388[43], 7}, {NULL}};
const Segment SGI317101[] = {{&S36388[36], 14}, {NULL}};
const Segment SGI317102[] = {{&S36388[52], 6}, {NULL}};
const Segment SGI317103[] = {{&S36388[87], 7}, {NULL}};
const Segment SGI317104[] = {{&S36388[72], 13}, {NULL}};
const Segment SGI317105[] = {{&S36388[94], 4}, {NULL}};
const Segment SGI317106[] = {{&S36389[30], 3}, {NULL}};
const Segment SGI317107[] = {{&S36389[8], 3}, {NULL}};
const Segment SGI317108[] = {{&S36389[13], 10}, {NULL}};
const Segment SGI317109[] = {{&S36389[42], 7}, {NULL}};
const Segment SGI317110[] = {{&S36389[35], 14}, {NULL}};
const Segment SGI317111[] = {{&S36389[131], 15}, {NULL}};
const Segment SGI317112[] = {{&S36389[199], 7}, {NULL}};
const Segment SGI317113[] = {{&S36389[206], 4}, {NULL}};
const Segment SGI317114[] = {{&S36389[164], 9}, {NULL}};
const Segment SGI317115[] = {{&S64504[3], 5}, {NULL}};
const Segment SGI317116[] = {{&S64505[3], 6}, {NULL}};
const Segment SGI317117[] = {{&S53282[3], 5}, {NULL}};
const Segment SGI317118[] = {{&S53283[3], 5}, {NULL}};
const Segment SGI317119[] = {{&S53283[3], 12}, {NULL}};
const Segment SGI317120[] = {{&S53284[3], 5}, {NULL}};
const Segment SGI317121[] = {{&S53284[3], 8}, {NULL}};
const Segment SGI317122[] = {{&S36538[30], 3}, {NULL}};
const Segment SGI317123[] = {{&S36538[8], 3}, {NULL}};
const Segment SGI317124[] = {{&S36538[13], 10}, {NULL}};
const Segment SGI317125[] = {{&S36538[42], 7}, {NULL}};
const Segment SGI317126[] = {{&S36538[35], 14}, {NULL}};
const Segment SGI317127[] = {{&S36538[64], 11}, {NULL}};
const Segment SGI317128[] = {{&S36538[157], 15}, {NULL}};
const Segment SGI317129[] = {{&S36538[64], 16}, {NULL}};
const Segment SGI317130[] = {{&S50663[0], 14}, {NULL}};
const Segment SGI317131[] = {{&S36539[30], 3}, {NULL}};
const Segment SGI317132[] = {{&S36539[8], 3}, {NULL}};
const Segment SGI317133[] = {{&S36539[13], 10}, {NULL}};
const Segment SGI317134[] = {{&S36539[42], 7}, {NULL}};
const Segment SGI317135[] = {{&S36539[35], 14}, {NULL}};
const Segment SGI317136[] = {{&S36539[64], 11}, {NULL}};
const Segment SGI317137[] = {{&S36539[157], 15}, {NULL}};
const Segment SGI317138[] = {{&S36539[64], 16}, {NULL}};
const Segment SGI317139[] = {{&S36539[198], 9}, {NULL}};
const Segment SGI317140[] = {{&S36540[30], 3}, {NULL}};
const Segment SGI317141[] = {{&S36540[8], 3}, {NULL}};
const Segment SGI317142[] = {{&S36540[13], 10}, {NULL}};
const Segment SGI317143[] = {{&S36540[42], 7}, {NULL}};
const Segment SGI317144[] = {{&S36540[35], 14}, {NULL}};
const Segment SGI317145[] = {{&S36540[64], 11}, {NULL}};
const Segment SGI317146[] = {{&S36540[157], 15}, {NULL}};
const Segment SGI317147[] = {{&S36540[64], 16}, {NULL}};
const Segment SGI317148[] = {{&S36541[30], 3}, {NULL}};
const Segment SGI317149[] = {{&S36541[8], 3}, {NULL}};
const Segment SGI317150[] = {{&S36541[13], 10}, {NULL}};
const Segment SGI317151[] = {{&S36541[42], 7}, {NULL}};
const Segment SGI317152[] = {{&S36541[35], 14}, {NULL}};
const Segment SGI317153[] = {{&S36541[64], 11}, {NULL}};
const Segment SGI317154[] = {{&S36541[64], 16}, {NULL}};
const Segment SGI317155[] = {{&S36542[30], 3}, {NULL}};
const Segment SGI317156[] = {{&S36542[8], 3}, {NULL}};
const Segment SGI317157[] = {{&S36542[13], 10}, {NULL}};
const Segment SGI317158[] = {{&S36542[42], 7}, {NULL}};
const Segment SGI317159[] = {{&S36542[35], 14}, {NULL}};
const Segment SGI317160[] = {{&S36542[71], 11}, {NULL}};
const Segment SGI317161[] = {{&S36542[51], 6}, {NULL}};
const Segment SGI317162[] = {{&S36542[71], 16}, {NULL}};
const Segment SGI317163[] = {{&S36543[30], 3}, {NULL}};
const Segment SGI317164[] = {{&S36543[8], 3}, {NULL}};
const Segment SGI317165[] = {{&S36543[13], 10}, {NULL}};
const Segment SGI317166[] = {{&S36543[42], 7}, {NULL}};
const Segment SGI317167[] = {{&S36543[35], 14}, {NULL}};
const Segment SGI317168[] = {{&S36543[71], 11}, {NULL}};
const Segment SGI317169[] = {{&S36543[164], 15}, {NULL}};
const Segment SGI317170[] = {{&S36543[51], 6}, {NULL}};
const Segment SGI317171[] = {{&S36543[71], 16}, {NULL}};
const Segment SGI317172[] = {{&S36546[30], 3}, {NULL}};
const Segment SGI317173[] = {{&S36546[8], 3}, {NULL}};
const Segment SGI317174[] = {{&S36546[13], 10}, {NULL}};
const Segment SGI317175[] = {{&S36546[42], 7}, {NULL}};
const Segment SGI317176[] = {{&S36546[35], 14}, {NULL}};
const Segment SGI317177[] = {{&S36546[64], 11}, {NULL}};
const Segment SGI317178[] = {{&S36546[64], 16}, {NULL}};
const Segment SGI317179[] = {{&S36547[30], 3}, {NULL}};
const Segment SGI317180[] = {{&S36547[8], 3}, {NULL}};
const Segment SGI317181[] = {{&S36547[13], 10}, {NULL}};
const Segment SGI317182[] = {{&S36547[42], 7}, {NULL}};
const Segment SGI317183[] = {{&S36547[35], 14}, {NULL}};
const Segment SGI317184[] = {{&S36547[64], 11}, {NULL}};
const Segment SGI317185[] = {{&S36547[157], 15}, {NULL}};
const Segment SGI317186[] = {{&S36547[64], 16}, {NULL}};
const Segment SGI317187[] = {{&S36547[178], 9}, {NULL}};
const Segment SGI317188[] = {{&S36548[30], 3}, {NULL}};
const Segment SGI317189[] = {{&S36548[8], 3}, {NULL}};
const Segment SGI317190[] = {{&S36548[13], 10}, {NULL}};
const Segment SGI317191[] = {{&S36548[42], 7}, {NULL}};
const Segment SGI317192[] = {{&S36548[35], 14}, {NULL}};
const Segment SGI317193[] = {{&S36548[64], 11}, {NULL}};
const Segment SGI317194[] = {{&S36548[64], 16}, {NULL}};
const Segment SGI317195[] = {{&S36548[177], 9}, {NULL}};
const Segment SGI317196[] = {{&S36549[30], 3}, {NULL}};
const Segment SGI317197[] = {{&S36549[8], 3}, {NULL}};
const Segment SGI317198[] = {{&S36549[13], 10}, {NULL}};
const Segment SGI317199[] = {{&S36549[42], 7}, {NULL}};
const Segment SGI317200[] = {{&S36549[35], 14}, {NULL}};
const Segment SGI317201[] = {{&S36549[71], 11}, {NULL}};
const Segment SGI317202[] = {{&S36549[164], 15}, {NULL}};
const Segment SGI317203[] = {{&S36549[51], 6}, {NULL}};
const Segment SGI317204[] = {{&S36549[71], 16}, {NULL}};
const Segment SGI317205[] = {{&S36550[30], 3}, {NULL}};
const Segment SGI317206[] = {{&S36550[8], 3}, {NULL}};
const Segment SGI317207[] = {{&S36550[13], 10}, {NULL}};
const Segment SGI317208[] = {{&S36550[42], 7}, {NULL}};
const Segment SGI317209[] = {{&S36550[35], 14}, {NULL}};
const Segment SGI317210[] = {{&S36550[71], 11}, {NULL}};
const Segment SGI317211[] = {{&S36550[51], 6}, {NULL}};
const Segment SGI317212[] = {{&S36550[71], 16}, {NULL}};
const Segment SGI317213[] = {{&S36551[30], 3}, {NULL}};
const Segment SGI317214[] = {{&S36551[248], 9}, {NULL}};
const Segment SGI317215[] = {{&S36551[8], 3}, {NULL}};
const Segment SGI317216[] = {{&S36551[13], 10}, {NULL}};
const Segment SGI317217[] = {{&S36551[42], 7}, {NULL}};
const Segment SGI317218[] = {{&S36551[35], 14}, {NULL}};
const Segment SGI317219[] = {{&S36551[71], 11}, {NULL}};
const Segment SGI317220[] = {{&S36551[164], 15}, {NULL}};
const Segment SGI317221[] = {{&S36551[51], 6}, {NULL}};
const Segment SGI317222[] = {{&S36551[71], 16}, {NULL}};
const Segment SGI317223[] = {{&S36552[30], 4}, {NULL}};
const Segment SGI317224[] = {{&S36552[8], 3}, {NULL}};
const Segment SGI317225[] = {{&S36552[13], 10}, {NULL}};
const Segment SGI317226[] = {{&S36552[43], 7}, {NULL}};
const Segment SGI317227[] = {{&S36552[36], 14}, {NULL}};
const Segment SGI317228[] = {{&S36552[72], 11}, {NULL}};
const Segment SGI317229[] = {{&S36552[52], 6}, {NULL}};
const Segment SGI317230[] = {{&S36552[72], 16}, {NULL}};
const Segment SGI317231[] = {{&S70681[0], 122}, {NULL}};
const Segment SGI317232[] = {{&S36892[30], 3}, {NULL}};
const Segment SGI317233[] = {{&S36892[8], 3}, {NULL}};
const Segment SGI317234[] = {{&S36892[13], 10}, {NULL}};
const Segment SGI317235[] = {{&S36894[30], 4}, {NULL}};
const Segment SGI317236[] = {{&S36894[8], 3}, {NULL}};
const Segment SGI317237[] = {{&S36894[13], 10}, {NULL}};
const Segment SGI317238[] = {{&S36895[31], 3}, {NULL}};
const Segment SGI317239[] = {{&S36896[30], 3}, {NULL}};
const Segment SGI317240[] = {{&S36896[8], 3}, {NULL}};
const Segment SGI317241[] = {{&S36896[13], 10}, {NULL}};
const Segment SGI317242[] = {{&S36897[48], 3}, {NULL}};
const Segment SGI317243[] = {{&S36897[39], 3}, {NULL}};
const Segment SGI317244[] = {{&S36898[30], 3}, {NULL}};
const Segment SGI317245[] = {{&S36898[8], 3}, {NULL}};
const Segment SGI317246[] = {{&S36898[13], 10}, {NULL}};
const Segment SGI317247[] = {{&S36898[35], 3}, {NULL}};
const Segment SGI317248[] = {{&S36899[30], 3}, {NULL}};
const Segment SGI317249[] = {{&S36899[8], 3}, {NULL}};
const Segment SGI317250[] = {{&S36899[13], 10}, {NULL}};
const Segment SGI317251[] = {{&S36899[35], 3}, {NULL}};
const Segment SGI317252[] = {{&S36899[40], 3}, {NULL}};
const Segment SGI317253[] = {{&S36946[16], 3}, {NULL}};
const Segment SGI317254[] = {{&S36946[38], 7}, {NULL}};
const Segment SGI317255[] = {{&S36946[31], 14}, {NULL}};
const Segment SGI317256[] = {{&S36947[26], 3}, {NULL}};
const Segment SGI317257[] = {{&S36947[38], 7}, {NULL}};
const Segment SGI317258[] = {{&S36947[31], 14}, {NULL}};
const Segment SGI317259[] = {{&S36947[47], 3}, {NULL}};
const Segment SGI317260[] = {{&S36948[16], 3}, {NULL}};
const Segment SGI317261[] = {{&S36948[38], 7}, {NULL}};
const Segment SGI317262[] = {{&S36948[31], 14}, {NULL}};
const Segment SGI317263[] = {{&S36948[98], 15}, {NULL}};
const Segment SGI317264[] = {{&S36949[16], 3}, {&S36949[79], 3}, {NULL}};
const Segment SGI317265[] = {{&S36949[55], 3}, {NULL}};
const Segment SGI317266[] = {{&S36949[38], 7}, {&S36949[38], 7}, {NULL}};
const Segment SGI317267[] = {{&S36949[31], 14}, {&S36949[31], 14}, {NULL}};
const Segment SGI317268[] = {{&S36949[100], 3}, {NULL}};
const Segment SGI317269[] = {{&S36950[16], 3}, {NULL}};
const Segment SGI317270[] = {{&S36950[38], 7}, {NULL}};
const Segment SGI317271[] = {{&S36950[31], 14}, {NULL}};
const Segment SGI317272[] = {{&S36950[60], 9}, {NULL}};
const Segment SGI317273[] = {{&S36951[16], 3}, {NULL}};
const Segment SGI317274[] = {{&S36951[38], 7}, {NULL}};
const Segment SGI317275[] = {{&S36951[31], 14}, {NULL}};
const Segment SGI317276[] = {{&S36951[124], 4}, {NULL}};
const Segment SGI317277[] = {{&S36952[26], 3}, {NULL}};
const Segment SGI317278[] = {{&S36952[38], 7}, {NULL}};
const Segment SGI317279[] = {{&S36952[31], 14}, {NULL}};
const Segment SGI317280[] = {{&S36952[90], 15}, {NULL}};
const Segment SGI317281[] = {{&S36952[47], 3}, {NULL}};
const Segment SGI317282[] = {{&S37092[30], 3}, {NULL}};
const Segment SGI317283[] = {{&S37092[8], 3}, {NULL}};
const Segment SGI317284[] = {{&S37092[13], 10}, {NULL}};
const Segment SGI317285[] = {{&S37092[42], 7}, {NULL}};
const Segment SGI317286[] = {{&S37092[35], 14}, {NULL}};
const Segment SGI317287[] = {{&S37092[131], 15}, {NULL}};
const Segment SGI317288[] = {{&S37092[152], 7}, {NULL}};
const Segment SGI317289[] = {{&S37092[79], 4}, {NULL}};
const Segment SGI317290[] = {{&S37093[30], 3}, {NULL}};
const Segment SGI317291[] = {{&S37093[8], 3}, {NULL}};
const Segment SGI317292[] = {{&S37093[13], 10}, {NULL}};
const Segment SGI317293[] = {{&S37093[42], 7}, {NULL}};
const Segment SGI317294[] = {{&S37093[35], 14}, {NULL}};
const Segment SGI317295[] = {{&S37093[131], 15}, {NULL}};
const Segment SGI317296[] = {{&S37093[152], 7}, {NULL}};
const Segment SGI317297[] = {{&S37093[79], 4}, {NULL}};
const Segment SGI317298[] = {{&S37094[30], 3}, {&S37094[94], 3}, {NULL}};
const Segment SGI317299[] = {{&S37094[8], 3}, {&S37094[8], 3}, {NULL}};
const Segment SGI317300[] = {{&S37094[13], 10}, {&S37094[13], 10}, {NULL}};
const Segment SGI317301[] = {{&S37094[42], 7}, {&S37094[106], 7}, {NULL}};
const Segment SGI317302[] = {{&S37094[35], 14}, {&S37094[99], 14}, {NULL}};
const Segment SGI317303[] = {{&S37094[189], 15}, {NULL}};
const Segment SGI317304[] = {{&S37094[210], 7}, {NULL}};
const Segment SGI317305[] = {{&S37094[137], 4}, {NULL}};
const Segment SGI317306[] = {{&S37094[115], 3}, {NULL}};
const Segment SGI317307[] = {{&S37095[30], 3}, {NULL}};
const Segment SGI317308[] = {{&S37095[8], 3}, {NULL}};
const Segment SGI317309[] = {{&S37095[13], 10}, {NULL}};
const Segment SGI317310[] = {{&S37095[42], 7}, {NULL}};
const Segment SGI317311[] = {{&S37095[35], 14}, {NULL}};
const Segment SGI317312[] = {{&S37095[131], 15}, {NULL}};
const Segment SGI317313[] = {{&S37095[167], 7}, {NULL}};
const Segment SGI317314[] = {{&S37095[79], 4}, {NULL}};
const Segment SGI317315[] = {{&S37095[152], 9}, {NULL}};
const Segment SGI317316[] = {{&S37096[30], 3}, {NULL}};
const Segment SGI317317[] = {{&S37096[8], 3}, {NULL}};
const Segment SGI317318[] = {{&S37096[13], 10}, {NULL}};
const Segment SGI317319[] = {{&S37096[42], 7}, {NULL}};
const Segment SGI317320[] = {{&S37096[35], 14}, {NULL}};
const Segment SGI317321[] = {{&S37096[138], 15}, {NULL}};
const Segment SGI317322[] = {{&S37096[51], 6}, {NULL}};
const Segment SGI317323[] = {{&S37096[179], 7}, {NULL}};
const Segment SGI317324[] = {{&S37096[86], 4}, {NULL}};
const Segment SGI317325[] = {{&S37097[30], 3}, {NULL}};
const Segment SGI317326[] = {{&S37097[8], 3}, {NULL}};
const Segment SGI317327[] = {{&S37097[13], 10}, {NULL}};
const Segment SGI317328[] = {{&S37097[42], 7}, {NULL}};
const Segment SGI317329[] = {{&S37097[35], 14}, {NULL}};
const Segment SGI317330[] = {{&S37097[138], 15}, {NULL}};
const Segment SGI317331[] = {{&S37097[51], 6}, {NULL}};
const Segment SGI317332[] = {{&S37097[169], 7}, {NULL}};
const Segment SGI317333[] = {{&S37097[86], 4}, {NULL}};
const Segment SGI317334[] = {{&S37098[30], 3}, {NULL}};
const Segment SGI317335[] = {{&S37098[8], 3}, {NULL}};
const Segment SGI317336[] = {{&S37098[13], 10}, {NULL}};
const Segment SGI317337[] = {{&S37098[42], 7}, {NULL}};
const Segment SGI317338[] = {{&S37098[35], 14}, {NULL}};
const Segment SGI317339[] = {{&S37098[131], 15}, {NULL}};
const Segment SGI317340[] = {{&S37098[162], 7}, {NULL}};
const Segment SGI317341[] = {{&S37098[79], 4}, {NULL}};
const Segment SGI317342[] = {{&S37099[30], 3}, {NULL}};
const Segment SGI317343[] = {{&S37099[8], 3}, {NULL}};
const Segment SGI317344[] = {{&S37099[13], 10}, {NULL}};
const Segment SGI317345[] = {{&S37099[42], 7}, {NULL}};
const Segment SGI317346[] = {{&S37099[35], 14}, {NULL}};
const Segment SGI317347[] = {{&S37099[141], 7}, {NULL}};
const Segment SGI317348[] = {{&S37099[79], 4}, {NULL}};
const Segment SGI317349[] = {{&S37100[30], 3}, {NULL}};
const Segment SGI317350[] = {{&S37100[8], 3}, {NULL}};
const Segment SGI317351[] = {{&S37100[13], 10}, {NULL}};
const Segment SGI317352[] = {{&S37100[42], 7}, {NULL}};
const Segment SGI317353[] = {{&S37100[35], 14}, {NULL}};
const Segment SGI317354[] = {{&S37100[131], 15}, {NULL}};
const Segment SGI317355[] = {{&S37100[179], 7}, {NULL}};
const Segment SGI317356[] = {{&S37100[79], 4}, {NULL}};
const Segment SGI317357[] = {{&S37100[164], 9}, {NULL}};
const Segment SGI317358[] = {{&S37101[30], 3}, {NULL}};
const Segment SGI317359[] = {{&S37101[8], 3}, {NULL}};
const Segment SGI317360[] = {{&S37101[13], 10}, {NULL}};
const Segment SGI317361[] = {{&S37101[42], 7}, {NULL}};
const Segment SGI317362[] = {{&S37101[35], 14}, {NULL}};
const Segment SGI317363[] = {{&S37101[131], 15}, {NULL}};
const Segment SGI317364[] = {{&S37101[167], 7}, {NULL}};
const Segment SGI317365[] = {{&S37101[79], 4}, {NULL}};
const Segment SGI317366[] = {{&S37101[152], 9}, {NULL}};
const Segment SGI317367[] = {{&S37102[30], 3}, {NULL}};
const Segment SGI317368[] = {{&S37102[8], 3}, {NULL}};
const Segment SGI317369[] = {{&S37102[13], 10}, {NULL}};
const Segment SGI317370[] = {{&S37102[42], 7}, {NULL}};
const Segment SGI317371[] = {{&S37102[35], 14}, {NULL}};
const Segment SGI317372[] = {{&S37102[138], 15}, {NULL}};
const Segment SGI317373[] = {{&S37102[51], 6}, {NULL}};
const Segment SGI317374[] = {{&S37102[159], 7}, {NULL}};
const Segment SGI317375[] = {{&S37102[86], 4}, {NULL}};
const Segment SGI317376[] = {{&S37103[30], 3}, {NULL}};
const Segment SGI317377[] = {{&S37103[8], 3}, {NULL}};
const Segment SGI317378[] = {{&S37103[13], 10}, {NULL}};
const Segment SGI317379[] = {{&S37103[42], 7}, {NULL}};
const Segment SGI317380[] = {{&S37103[35], 14}, {NULL}};
const Segment SGI317381[] = {{&S37103[51], 6}, {NULL}};
const Segment SGI317382[] = {{&S37103[71], 7}, {NULL}};
const Segment SGI317383[] = {{&S37103[78], 4}, {NULL}};
const Segment SGI317384[] = {{&S37104[30], 3}, {NULL}};
const Segment SGI317385[] = {{&S37104[8], 3}, {NULL}};
const Segment SGI317386[] = {{&S37104[13], 10}, {NULL}};
const Segment SGI317387[] = {{&S37104[42], 7}, {NULL}};
const Segment SGI317388[] = {{&S37104[35], 14}, {NULL}};
const Segment SGI317389[] = {{&S37104[146], 7}, {NULL}};
const Segment SGI317390[] = {{&S37104[79], 4}, {NULL}};
const Segment SGI317391[] = {{&S37104[131], 9}, {NULL}};
const Segment SGI317392[] = {{&S37105[30], 3}, {&S37105[94], 3}, {NULL}};
const Segment SGI317393[] = {{&S37105[8], 3}, {&S37105[8], 3}, {NULL}};
const Segment SGI317394[] = {{&S37105[13], 10}, {&S37105[13], 10}, {NULL}};
const Segment SGI317395[] = {{&S37105[42], 7}, {&S37105[106], 7}, {NULL}};
const Segment SGI317396[] = {{&S37105[35], 14}, {&S37105[99], 14}, {NULL}};
const Segment SGI317397[] = {{&S37105[189], 15}, {NULL}};
const Segment SGI317398[] = {{&S37105[225], 7}, {NULL}};
const Segment SGI317399[] = {{&S37105[137], 4}, {NULL}};
const Segment SGI317400[] = {{&S37105[210], 9}, {NULL}};
const Segment SGI317401[] = {{&S37105[115], 3}, {NULL}};
const Segment SGI317402[] = {{&S37106[30], 3}, {NULL}};
const Segment SGI317403[] = {{&S37106[8], 3}, {NULL}};
const Segment SGI317404[] = {{&S37106[13], 10}, {NULL}};
const Segment SGI317405[] = {{&S37106[42], 7}, {NULL}};
const Segment SGI317406[] = {{&S37106[35], 14}, {NULL}};
const Segment SGI317407[] = {{&S37106[131], 15}, {NULL}};
const Segment SGI317408[] = {{&S37106[179], 7}, {NULL}};
const Segment SGI317409[] = {{&S37106[79], 4}, {NULL}};
const Segment SGI317410[] = {{&S37106[164], 9}, {NULL}};
const Segment SGI317411[] = {{&S37107[30], 3}, {NULL}};
const Segment SGI317412[] = {{&S37107[8], 3}, {NULL}};
const Segment SGI317413[] = {{&S37107[13], 10}, {NULL}};
const Segment SGI317414[] = {{&S37107[42], 7}, {NULL}};
const Segment SGI317415[] = {{&S37107[35], 14}, {NULL}};
const Segment SGI317416[] = {{&S37107[138], 15}, {NULL}};
const Segment SGI317417[] = {{&S37107[51], 6}, {NULL}};
const Segment SGI317418[] = {{&S37107[181], 7}, {NULL}};
const Segment SGI317419[] = {{&S37107[86], 4}, {NULL}};
const Segment SGI317420[] = {{&S37108[30], 3}, {NULL}};
const Segment SGI317421[] = {{&S37108[8], 3}, {NULL}};
const Segment SGI317422[] = {{&S37108[13], 10}, {NULL}};
const Segment SGI317423[] = {{&S37108[42], 7}, {NULL}};
const Segment SGI317424[] = {{&S37108[35], 14}, {NULL}};
const Segment SGI317425[] = {{&S37108[51], 6}, {NULL}};
const Segment SGI317426[] = {{&S37108[148], 7}, {NULL}};
const Segment SGI317427[] = {{&S37108[86], 4}, {NULL}};
const Segment SGI317428[] = {{&S37109[30], 3}, {NULL}};
const Segment SGI317429[] = {{&S37109[8], 3}, {NULL}};
const Segment SGI317430[] = {{&S37109[13], 10}, {NULL}};
const Segment SGI317431[] = {{&S37109[42], 7}, {NULL}};
const Segment SGI317432[] = {{&S37109[35], 14}, {NULL}};
const Segment SGI317433[] = {{&S37109[71], 11}, {NULL}};
const Segment SGI317434[] = {{&S37109[151], 15}, {NULL}};
const Segment SGI317435[] = {{&S37109[51], 6}, {NULL}};
const Segment SGI317436[] = {{&S37109[172], 7}, {NULL}};
const Segment SGI317437[] = {{&S37109[99], 4}, {NULL}};
const Segment SGI317438[] = {{&S37110[30], 3}, {NULL}};
const Segment SGI317439[] = {{&S37110[8], 3}, {NULL}};
const Segment SGI317440[] = {{&S37110[13], 10}, {NULL}};
const Segment SGI317441[] = {{&S37110[42], 7}, {NULL}};
const Segment SGI317442[] = {{&S37110[35], 14}, {NULL}};
const Segment SGI317443[] = {{&S37110[91], 15}, {NULL}};
const Segment SGI317444[] = {{&S37110[152], 7}, {NULL}};
const Segment SGI317445[] = {{&S37110[79], 4}, {NULL}};
const Segment SGI317446[] = {{&S37111[30], 3}, {NULL}};
const Segment SGI317447[] = {{&S37111[8], 3}, {NULL}};
const Segment SGI317448[] = {{&S37111[13], 10}, {NULL}};
const Segment SGI317449[] = {{&S37111[42], 7}, {NULL}};
const Segment SGI317450[] = {{&S37111[35], 14}, {NULL}};
const Segment SGI317451[] = {{&S37111[144], 15}, {NULL}};
const Segment SGI317452[] = {{&S37111[64], 6}, {NULL}};
const Segment SGI317453[] = {{&S37111[165], 7}, {NULL}};
const Segment SGI317454[] = {{&S37111[132], 4}, {NULL}};
const Segment SGI317455[] = {{&S37112[30], 3}, {NULL}};
const Segment SGI317456[] = {{&S37112[8], 3}, {NULL}};
const Segment SGI317457[] = {{&S37112[13], 10}, {NULL}};
const Segment SGI317458[] = {{&S37112[42], 7}, {NULL}};
const Segment SGI317459[] = {{&S37112[35], 14}, {NULL}};
const Segment SGI317460[] = {{&S37112[141], 7}, {NULL}};
const Segment SGI317461[] = {{&S37112[79], 4}, {NULL}};
const Segment SGI317462[] = {{&S37113[30], 3}, {NULL}};
const Segment SGI317463[] = {{&S37113[8], 3}, {NULL}};
const Segment SGI317464[] = {{&S37113[13], 10}, {NULL}};
const Segment SGI317465[] = {{&S37113[42], 7}, {NULL}};
const Segment SGI317466[] = {{&S37113[35], 14}, {NULL}};
const Segment SGI317467[] = {{&S37113[153], 7}, {NULL}};
const Segment SGI317468[] = {{&S37113[79], 4}, {NULL}};
const Segment SGI317469[] = {{&S37114[30], 3}, {NULL}};
const Segment SGI317470[] = {{&S37114[8], 3}, {NULL}};
const Segment SGI317471[] = {{&S37114[13], 10}, {NULL}};
const Segment SGI317472[] = {{&S37114[42], 7}, {NULL}};
const Segment SGI317473[] = {{&S37114[35], 14}, {NULL}};
const Segment SGI317474[] = {{&S37114[151], 15}, {NULL}};
const Segment SGI317475[] = {{&S37114[172], 4}, {NULL}};
const Segment SGI317476[] = {{&S37114[64], 6}, {NULL}};
const Segment SGI317477[] = {{&S37114[227], 7}, {NULL}};
const Segment SGI317478[] = {{&S37114[99], 4}, {NULL}};
const Segment SGI317479[] = {{&S37115[30], 3}, {NULL}};
const Segment SGI317480[] = {{&S37115[8], 3}, {NULL}};
const Segment SGI317481[] = {{&S37115[13], 10}, {NULL}};
const Segment SGI317482[] = {{&S37115[42], 7}, {NULL}};
const Segment SGI317483[] = {{&S37115[35], 14}, {NULL}};
const Segment SGI317484[] = {{&S37115[64], 7}, {NULL}};
const Segment SGI317485[] = {{&S37115[71], 4}, {NULL}};
const Segment SGI317486[] = {{&S37116[30], 3}, {NULL}};
const Segment SGI317487[] = {{&S37116[8], 3}, {NULL}};
const Segment SGI317488[] = {{&S37116[13], 10}, {NULL}};
const Segment SGI317489[] = {{&S37116[42], 7}, {NULL}};
const Segment SGI317490[] = {{&S37116[35], 14}, {NULL}};
const Segment SGI317491[] = {{&S37116[51], 6}, {NULL}};
const Segment SGI317492[] = {{&S37116[138], 7}, {NULL}};
const Segment SGI317493[] = {{&S37116[86], 4}, {NULL}};
const Segment SGI317494[] = {{&S37117[30], 3}, {NULL}};
const Segment SGI317495[] = {{&S37117[8], 3}, {NULL}};
const Segment SGI317496[] = {{&S37117[13], 10}, {NULL}};
const Segment SGI317497[] = {{&S37117[42], 7}, {NULL}};
const Segment SGI317498[] = {{&S37117[35], 14}, {NULL}};
const Segment SGI317499[] = {{&S37117[51], 6}, {NULL}};
const Segment SGI317500[] = {{&S37117[158], 7}, {NULL}};
const Segment SGI317501[] = {{&S37117[86], 4}, {NULL}};
const Segment SGI317502[] = {{&S37118[30], 3}, {NULL}};
const Segment SGI317503[] = {{&S37118[8], 3}, {NULL}};
const Segment SGI317504[] = {{&S37118[13], 10}, {NULL}};
const Segment SGI317505[] = {{&S37118[42], 7}, {NULL}};
const Segment SGI317506[] = {{&S37118[35], 14}, {NULL}};
const Segment SGI317507[] = {{&S37118[75], 7}, {NULL}};
const Segment SGI317508[] = {{&S37118[82], 4}, {NULL}};
const Segment SGI317509[] = {{&S37118[51], 3}, {NULL}};
const Segment SGI317510[] = {{&S37119[30], 3}, {&S37119[101], 3}, {NULL}};
const Segment SGI317511[] = {{&S37119[8], 3}, {&S37119[8], 3}, {NULL}};
const Segment SGI317512[] = {{&S37119[13], 10}, {&S37119[13], 10}, {NULL}};
const Segment SGI317513[] = {{&S37119[42], 7}, {&S37119[113], 7}, {NULL}};
const Segment SGI317514[] = {{&S37119[35], 14}, {&S37119[106], 14}, {NULL}};
const Segment SGI317515[] = {{&S37119[196], 15}, {NULL}};
const Segment SGI317516[] = {{&S37119[51], 6}, {NULL}};
const Segment SGI317517[] = {{&S37119[227], 7}, {NULL}};
const Segment SGI317518[] = {{&S37119[144], 4}, {NULL}};
const Segment SGI317519[] = {{&S37119[122], 3}, {NULL}};
const Segment SGI317520[] = {{&S37120[30], 3}, {NULL}};
const Segment SGI317521[] = {{&S37120[8], 3}, {NULL}};
const Segment SGI317522[] = {{&S37120[13], 10}, {NULL}};
const Segment SGI317523[] = {{&S37120[42], 7}, {NULL}};
const Segment SGI317524[] = {{&S37120[35], 14}, {NULL}};
const Segment SGI317525[] = {{&S37120[131], 15}, {&S37120[131], 15}, {NULL}};
const Segment SGI317526[] = {{&S37120[210], 7}, {NULL}};
const Segment SGI317527[] = {{&S37120[79], 4}, {NULL}};
const Segment SGI317528[] = {{&S37121[30], 4}, {NULL}};
const Segment SGI317529[] = {{&S37121[8], 3}, {NULL}};
const Segment SGI317530[] = {{&S37121[13], 10}, {NULL}};
const Segment SGI317531[] = {{&S37121[43], 7}, {NULL}};
const Segment SGI317532[] = {{&S37121[36], 14}, {NULL}};
const Segment SGI317533[] = {{&S37121[52], 6}, {NULL}};
const Segment SGI317534[] = {{&S37121[72], 7}, {NULL}};
const Segment SGI317535[] = {{&S37121[79], 4}, {NULL}};
const Segment SGI317536[] = {{&S37122[30], 4}, {NULL}};
const Segment SGI317537[] = {{&S37122[8], 3}, {NULL}};
const Segment SGI317538[] = {{&S37122[13], 10}, {NULL}};
const Segment SGI317539[] = {{&S37122[43], 7}, {NULL}};
const Segment SGI317540[] = {{&S37122[36], 14}, {NULL}};
const Segment SGI317541[] = {{&S37122[65], 7}, {NULL}};
const Segment SGI317542[] = {{&S37122[72], 4}, {NULL}};
const Segment SGI317543[] = {{&S37742[30], 3}, {NULL}};
const Segment SGI317544[] = {{&S37742[8], 3}, {NULL}};
const Segment SGI317545[] = {{&S37742[13], 10}, {NULL}};
const Segment SGI317546[] = {{&S37742[42], 7}, {NULL}};
const Segment SGI317547[] = {{&S37742[35], 14}, {NULL}};
const Segment SGI317548[] = {{&S37742[64], 11}, {NULL}};
const Segment SGI317549[] = {{&S37742[127], 15}, {NULL}};
const Segment SGI317550[] = {{&S37742[64], 16}, {NULL}};
const Segment SGI317551[] = {{&S37743[30], 3}, {&S37743[77], 3}, {NULL}};
const Segment SGI317552[] = {{&S37743[8], 3}, {&S37743[8], 3}, {NULL}};
const Segment SGI317553[] = {{&S37743[13], 10}, {&S37743[13], 10}, {NULL}};
const Segment SGI317554[] = {{&S37743[42], 7}, {&S37743[42], 7}, {NULL}};
const Segment SGI317555[] = {{&S37743[35], 14}, {&S37743[35], 14}, {NULL}};
const Segment SGI317556[] = {{&S37743[64], 11}, {NULL}};
const Segment SGI317557[] = {{&S37743[64], 16}, {NULL}};
const Segment SGI317558[] = {{&S37743[140], 3}, {NULL}};
const Segment SGI317559[] = {{&S37745[30], 3}, {NULL}};
const Segment SGI317560[] = {{&S37745[8], 3}, {NULL}};
const Segment SGI317561[] = {{&S37745[13], 10}, {NULL}};
const Segment SGI317562[] = {{&S37745[42], 7}, {NULL}};
const Segment SGI317563[] = {{&S37745[35], 14}, {NULL}};
const Segment SGI317564[] = {{&S37745[51], 11}, {NULL}};
const Segment SGI317565[] = {{&S37745[51], 16}, {NULL}};
const Segment SGI317566[] = {{&S37747[30], 3}, {NULL}};
const Segment SGI317567[] = {{&S37747[8], 3}, {NULL}};
const Segment SGI317568[] = {{&S37747[13], 10}, {NULL}};
const Segment SGI317569[] = {{&S37747[42], 7}, {NULL}};
const Segment SGI317570[] = {{&S37747[35], 14}, {NULL}};
const Segment SGI317571[] = {{&S37747[64], 11}, {NULL}};
const Segment SGI317572[] = {{&S37747[64], 16}, {NULL}};
const Segment SGI317573[] = {{&S37748[30], 3}, {NULL}};
const Segment SGI317574[] = {{&S37748[8], 3}, {NULL}};
const Segment SGI317575[] = {{&S37748[13], 10}, {NULL}};
const Segment SGI317576[] = {{&S37748[42], 7}, {NULL}};
const Segment SGI317577[] = {{&S37748[35], 14}, {NULL}};
const Segment SGI317578[] = {{&S37748[51], 11}, {NULL}};
const Segment SGI317579[] = {{&S37748[51], 16}, {NULL}};
const Segment SGI317580[] = {{&S37749[30], 3}, {NULL}};
const Segment SGI317581[] = {{&S37749[8], 3}, {NULL}};
const Segment SGI317582[] = {{&S37749[13], 10}, {NULL}};
const Segment SGI317583[] = {{&S37749[42], 7}, {NULL}};
const Segment SGI317584[] = {{&S37749[35], 14}, {NULL}};
const Segment SGI317585[] = {{&S37749[56], 11}, {NULL}};
const Segment SGI317586[] = {{&S37749[56], 16}, {NULL}};
const Segment SGI317587[] = {{&S37749[51], 3}, {NULL}};
const Segment SGI317588[] = {{&S37751[30], 3}, {NULL}};
const Segment SGI317589[] = {{&S37751[8], 3}, {NULL}};
const Segment SGI317590[] = {{&S37751[13], 10}, {NULL}};
const Segment SGI317591[] = {{&S37751[42], 7}, {NULL}};
const Segment SGI317592[] = {{&S37751[35], 14}, {NULL}};
const Segment SGI317593[] = {{&S37751[51], 11}, {NULL}};
const Segment SGI317594[] = {{&S37751[51], 16}, {NULL}};
const Segment SGI317595[] = {{&S37752[30], 3}, {NULL}};
const Segment SGI317596[] = {{&S37752[8], 3}, {NULL}};
const Segment SGI317597[] = {{&S37752[13], 10}, {NULL}};
const Segment SGI317598[] = {{&S37752[42], 7}, {NULL}};
const Segment SGI317599[] = {{&S37752[35], 14}, {NULL}};
const Segment SGI317600[] = {{&S37752[64], 11}, {NULL}};
const Segment SGI317601[] = {{&S37752[64], 16}, {NULL}};
const Segment SGI317602[] = {{&S37777[30], 3}, {NULL}};
const Segment SGI317603[] = {{&S37777[8], 3}, {NULL}};
const Segment SGI317604[] = {{&S37777[13], 10}, {NULL}};
const Segment SGI317605[] = {{&S37777[42], 7}, {NULL}};
const Segment SGI317606[] = {{&S37777[35], 14}, {NULL}};
const Segment SGI317607[] = {{&S37777[71], 11}, {NULL}};
const Segment SGI317608[] = {{&S37777[163], 15}, {NULL}};
const Segment SGI317609[] = {{&S37777[51], 6}, {NULL}};
const Segment SGI317610[] = {{&S37777[71], 16}, {NULL}};
const Segment SGI317611[] = {{&S37779[30], 3}, {NULL}};
const Segment SGI317612[] = {{&S37779[8], 3}, {NULL}};
const Segment SGI317613[] = {{&S37779[13], 10}, {NULL}};
const Segment SGI317614[] = {{&S37779[42], 7}, {NULL}};
const Segment SGI317615[] = {{&S37779[35], 14}, {NULL}};
const Segment SGI317616[] = {{&S37779[64], 11}, {NULL}};
const Segment SGI317617[] = {{&S37779[64], 16}, {NULL}};
const Segment SGI317618[] = {{&S37780[30], 3}, {NULL}};
const Segment SGI317619[] = {{&S37780[8], 3}, {NULL}};
const Segment SGI317620[] = {{&S37780[13], 10}, {NULL}};
const Segment SGI317621[] = {{&S37780[42], 7}, {NULL}};
const Segment SGI317622[] = {{&S37780[35], 14}, {NULL}};
const Segment SGI317623[] = {{&S37780[71], 11}, {NULL}};
const Segment SGI317624[] = {{&S37780[51], 6}, {NULL}};
const Segment SGI317625[] = {{&S37780[71], 16}, {NULL}};
const Segment SGI317626[] = {{&S37781[30], 3}, {NULL}};
const Segment SGI317627[] = {{&S37781[8], 3}, {NULL}};
const Segment SGI317628[] = {{&S37781[13], 10}, {NULL}};
const Segment SGI317629[] = {{&S37781[42], 7}, {NULL}};
const Segment SGI317630[] = {{&S37781[35], 14}, {NULL}};
const Segment SGI317631[] = {{&S37781[51], 11}, {NULL}};
const Segment SGI317632[] = {{&S37781[144], 15}, {NULL}};
const Segment SGI317633[] = {{&S37781[51], 16}, {NULL}};
const Segment SGI317634[] = {{&S37784[30], 3}, {NULL}};
const Segment SGI317635[] = {{&S37784[8], 3}, {NULL}};
const Segment SGI317636[] = {{&S37784[13], 10}, {NULL}};
const Segment SGI317637[] = {{&S37784[42], 7}, {NULL}};
const Segment SGI317638[] = {{&S37784[35], 14}, {NULL}};
const Segment SGI317639[] = {{&S37784[71], 11}, {NULL}};
const Segment SGI317640[] = {{&S37784[51], 6}, {NULL}};
const Segment SGI317641[] = {{&S37784[71], 16}, {NULL}};
const Segment SGI317642[] = {{&S37785[30], 3}, {NULL}};
const Segment SGI317643[] = {{&S37785[8], 3}, {NULL}};
const Segment SGI317644[] = {{&S37785[13], 10}, {NULL}};
const Segment SGI317645[] = {{&S37785[42], 7}, {NULL}};
const Segment SGI317646[] = {{&S37785[35], 14}, {NULL}};
const Segment SGI317647[] = {{&S37785[149], 11}, {NULL}};
const Segment SGI317648[] = {{&S37785[118], 15}, {NULL}};
const Segment SGI317649[] = {{&S37785[149], 16}, {NULL}};
const Segment SGI317650[] = {{&S37789[30], 3}, {NULL}};
const Segment SGI317651[] = {{&S37789[8], 3}, {NULL}};
const Segment SGI317652[] = {{&S37789[13], 10}, {NULL}};
const Segment SGI317653[] = {{&S37789[42], 7}, {NULL}};
const Segment SGI317654[] = {{&S37789[35], 14}, {NULL}};
const Segment SGI317655[] = {{&S37789[64], 11}, {NULL}};
const Segment SGI317656[] = {{&S37789[156], 15}, {NULL}};
const Segment SGI317657[] = {{&S37789[64], 16}, {NULL}};
const Segment SGI317658[] = {{&S37791[30], 3}, {NULL}};
const Segment SGI317659[] = {{&S37791[8], 3}, {NULL}};
const Segment SGI317660[] = {{&S37791[13], 10}, {NULL}};
const Segment SGI317661[] = {{&S37791[42], 7}, {NULL}};
const Segment SGI317662[] = {{&S37791[35], 14}, {NULL}};
const Segment SGI317663[] = {{&S37791[64], 11}, {NULL}};
const Segment SGI317664[] = {{&S37791[157], 15}, {NULL}};
const Segment SGI317665[] = {{&S37791[64], 16}, {NULL}};
const Segment SGI317666[] = {{&S37791[210], 9}, {NULL}};
const Segment SGI317667[] = {{&S37792[30], 3}, {NULL}};
const Segment SGI317668[] = {{&S37792[8], 3}, {NULL}};
const Segment SGI317669[] = {{&S37792[13], 10}, {NULL}};
const Segment SGI317670[] = {{&S37792[42], 7}, {NULL}};
const Segment SGI317671[] = {{&S37792[35], 14}, {NULL}};
const Segment SGI317672[] = {{&S37792[51], 11}, {NULL}};
const Segment SGI317673[] = {{&S37792[144], 15}, {NULL}};
const Segment SGI317674[] = {{&S37792[51], 16}, {NULL}};
const Segment SGI317675[] = {{&S38023[30], 3}, {NULL}};
const Segment SGI317676[] = {{&S38023[8], 3}, {NULL}};
const Segment SGI317677[] = {{&S38023[13], 10}, {NULL}};
const Segment SGI317678[] = {{&S38023[42], 7}, {NULL}};
const Segment SGI317679[] = {{&S38023[35], 14}, {NULL}};
const Segment SGI317680[] = {{&S38023[146], 15}, {NULL}};
const Segment SGI317681[] = {{&S38023[64], 13}, {NULL}};
const Segment SGI317682[] = {{&S38024[30], 3}, {NULL}};
const Segment SGI317683[] = {{&S38024[8], 3}, {NULL}};
const Segment SGI317684[] = {{&S38024[13], 10}, {NULL}};
const Segment SGI317685[] = {{&S38024[42], 7}, {NULL}};
const Segment SGI317686[] = {{&S38024[35], 14}, {NULL}};
const Segment SGI317687[] = {{&S38024[153], 15}, {NULL}};
const Segment SGI317688[] = {{&S38024[51], 6}, {NULL}};
const Segment SGI317689[] = {{&S38024[71], 13}, {&S38024[71], 13}, {NULL}};
const Segment SGI317690[] = {{&S38025[30], 3}, {NULL}};
const Segment SGI317691[] = {{&S38025[8], 3}, {NULL}};
const Segment SGI317692[] = {{&S38025[13], 10}, {NULL}};
const Segment SGI317693[] = {{&S38025[42], 7}, {NULL}};
const Segment SGI317694[] = {{&S38025[35], 14}, {NULL}};
const Segment SGI317695[] = {{&S38025[146], 15}, {NULL}};
const Segment SGI317696[] = {{&S38025[64], 13}, {&S38025[64], 13}, {NULL}};
const Segment SGI317697[] = {{&S38026[30], 3}, {NULL}};
const Segment SGI317698[] = {{&S38026[8], 3}, {NULL}};
const Segment SGI317699[] = {{&S38026[13], 10}, {NULL}};
const Segment SGI317700[] = {{&S38026[42], 7}, {NULL}};
const Segment SGI317701[] = {{&S38026[35], 14}, {NULL}};
const Segment SGI317702[] = {{&S38026[138], 15}, {NULL}};
const Segment SGI317703[] = {{&S38026[51], 6}, {NULL}};
const Segment SGI317704[] = {{&S38026[169], 13}, {NULL}};
const Segment SGI317705[] = {{&S38027[30], 3}, {NULL}};
const Segment SGI317706[] = {{&S38027[8], 3}, {NULL}};
const Segment SGI317707[] = {{&S38027[13], 10}, {NULL}};
const Segment SGI317708[] = {{&S38027[55], 7}, {NULL}};
const Segment SGI317709[] = {{&S38027[48], 14}, {NULL}};
const Segment SGI317710[] = {{&S38027[64], 13}, {NULL}};
const Segment SGI317711[] = {{&S38028[30], 3}, {NULL}};
const Segment SGI317712[] = {{&S38028[8], 3}, {NULL}};
const Segment SGI317713[] = {{&S38028[13], 10}, {NULL}};
const Segment SGI317714[] = {{&S38028[42], 7}, {NULL}};
const Segment SGI317715[] = {{&S38028[35], 14}, {NULL}};
const Segment SGI317716[] = {{&S38028[146], 15}, {NULL}};
const Segment SGI317717[] = {{&S38028[64], 13}, {&S38028[64], 13}, {NULL}};
const Segment SGI317718[] = {{&S38028[167], 9}, {NULL}};
const Segment SGI317719[] = {{&S38029[30], 3}, {NULL}};
const Segment SGI317720[] = {{&S38029[8], 3}, {NULL}};
const Segment SGI317721[] = {{&S38029[13], 10}, {NULL}};
const Segment SGI317722[] = {{&S38029[55], 7}, {NULL}};
const Segment SGI317723[] = {{&S38029[48], 14}, {NULL}};
const Segment SGI317724[] = {{&S38029[64], 13}, {NULL}};
const Segment SGI317725[] = {{&S38030[30], 3}, {NULL}};
const Segment SGI317726[] = {{&S38030[8], 3}, {NULL}};
const Segment SGI317727[] = {{&S38030[13], 10}, {NULL}};
const Segment SGI317728[] = {{&S38030[42], 7}, {NULL}};
const Segment SGI317729[] = {{&S38030[35], 14}, {NULL}};
const Segment SGI317730[] = {{&S38030[158], 15}, {NULL}};
const Segment SGI317731[] = {{&S38030[76], 13}, {&S38030[76], 13}, {NULL}};
const Segment SGI317732[] = {{&S38030[51], 5}, {NULL}};
const Segment SGI317733[] = {{&S38031[30], 3}, {NULL}};
const Segment SGI317734[] = {{&S38031[8], 3}, {NULL}};
const Segment SGI317735[] = {{&S38031[13], 10}, {NULL}};
const Segment SGI317736[] = {{&S38031[42], 7}, {NULL}};
const Segment SGI317737[] = {{&S38031[35], 14}, {NULL}};
const Segment SGI317738[] = {{&S38031[158], 15}, {NULL}};
const Segment SGI317739[] = {{&S38031[76], 13}, {&S38031[76], 13}, {NULL}};
const Segment SGI317740[] = {{&S38031[51], 5}, {NULL}};
const Segment SGI317741[] = {{&S38031[189], 9}, {NULL}};
const Segment SGI317742[] = {{&S38032[30], 3}, {NULL}};
const Segment SGI317743[] = {{&S38032[8], 3}, {NULL}};
const Segment SGI317744[] = {{&S38032[13], 10}, {NULL}};
const Segment SGI317745[] = {{&S38032[42], 7}, {NULL}};
const Segment SGI317746[] = {{&S38032[35], 14}, {NULL}};
const Segment SGI317747[] = {{&S38032[64], 13}, {&S38032[64], 13}, {NULL}};
const Segment SGI317748[] = {{&S38033[30], 3}, {NULL}};
const Segment SGI317749[] = {{&S38033[8], 3}, {NULL}};
const Segment SGI317750[] = {{&S38033[13], 10}, {NULL}};
const Segment SGI317751[] = {{&S38033[42], 7}, {NULL}};
const Segment SGI317752[] = {{&S38033[35], 14}, {NULL}};
const Segment SGI317753[] = {{&S38033[143], 15}, {NULL}};
const Segment SGI317754[] = {{&S38033[184], 13}, {NULL}};
const Segment SGI317755[] = {{&S38033[51], 5}, {NULL}};
const Segment SGI317756[] = {{&S38034[30], 3}, {NULL}};
const Segment SGI317757[] = {{&S38034[8], 3}, {NULL}};
const Segment SGI317758[] = {{&S38034[13], 10}, {NULL}};
const Segment SGI317759[] = {{&S38034[42], 7}, {NULL}};
const Segment SGI317760[] = {{&S38034[35], 14}, {NULL}};
const Segment SGI317761[] = {{&S38034[51], 6}, {NULL}};
const Segment SGI317762[] = {{&S38034[71], 13}, {&S38034[71], 13}, {NULL}};
const Segment SGI317763[] = {{&S38035[30], 3}, {NULL}};
const Segment SGI317764[] = {{&S38035[8], 3}, {NULL}};
const Segment SGI317765[] = {{&S38035[13], 10}, {NULL}};
const Segment SGI317766[] = {{&S38035[55], 7}, {NULL}};
const Segment SGI317767[] = {{&S38035[48], 14}, {NULL}};
const Segment SGI317768[] = {{&S38035[64], 13}, {NULL}};
const Segment SGI317769[] = {{&S38035[79], 3}, {NULL}};
const Segment SGI317770[] = {{&S38036[30], 3}, {NULL}};
const Segment SGI317771[] = {{&S38036[8], 3}, {NULL}};
const Segment SGI317772[] = {{&S38036[13], 10}, {NULL}};
const Segment SGI317773[] = {{&S38036[55], 7}, {NULL}};
const Segment SGI317774[] = {{&S38036[48], 14}, {NULL}};
const Segment SGI317775[] = {{&S38036[69], 13}, {NULL}};
const Segment SGI317776[] = {{&S38036[64], 3}, {NULL}};
const Segment SGI317777[] = {{&S38037[30], 3}, {NULL}};
const Segment SGI317778[] = {{&S38037[8], 3}, {NULL}};
const Segment SGI317779[] = {{&S38037[13], 10}, {NULL}};
const Segment SGI317780[] = {{&S38037[55], 7}, {NULL}};
const Segment SGI317781[] = {{&S38037[48], 14}, {NULL}};
const Segment SGI317782[] = {{&S38037[64], 13}, {&S38037[64], 13}, {NULL}};
const Segment SGI317783[] = {{&S38038[30], 3}, {NULL}};
const Segment SGI317784[] = {{&S38038[8], 3}, {NULL}};
const Segment SGI317785[] = {{&S38038[13], 10}, {NULL}};
const Segment SGI317786[] = {{&S38038[42], 7}, {NULL}};
const Segment SGI317787[] = {{&S38038[35], 14}, {NULL}};
const Segment SGI317788[] = {{&S38038[131], 15}, {NULL}};
const Segment SGI317789[] = {{&S38038[187], 13}, {NULL}};
const Segment SGI317790[] = {{&S38038[152], 9}, {NULL}};
const Segment SGI317791[] = {{&S38039[30], 3}, {NULL}};
const Segment SGI317792[] = {{&S38039[8], 3}, {NULL}};
const Segment SGI317793[] = {{&S38039[13], 10}, {NULL}};
const Segment SGI317794[] = {{&S38039[42], 7}, {NULL}};
const Segment SGI317795[] = {{&S38039[35], 14}, {NULL}};
const Segment SGI317796[] = {{&S38039[169], 15}, {NULL}};
const Segment SGI317797[] = {{&S38039[159], 4}, {NULL}};
const Segment SGI317798[] = {{&S38039[202], 13}, {NULL}};
const Segment SGI317799[] = {{&S38039[51], 5}, {NULL}};
const Segment SGI317800[] = {{&S38061[30], 3}, {NULL}};
const Segment SGI317801[] = {{&S38061[8], 3}, {NULL}};
const Segment SGI317802[] = {{&S38061[13], 10}, {NULL}};
const Segment SGI317803[] = {{&S38061[42], 8}, {NULL}};
const Segment SGI317804[] = {{&S38061[52], 3}, {NULL}};
const Segment SGI317805[] = {{&S38062[30], 3}, {NULL}};
const Segment SGI317806[] = {{&S38062[8], 3}, {NULL}};
const Segment SGI317807[] = {{&S38062[13], 10}, {NULL}};
const Segment SGI317808[] = {{&S38062[42], 8}, {NULL}};
const Segment SGI317809[] = {{&S38063[30], 3}, {NULL}};
const Segment SGI317810[] = {{&S38063[8], 3}, {NULL}};
const Segment SGI317811[] = {{&S38063[13], 10}, {NULL}};
const Segment SGI317812[] = {{&S38063[55], 7}, {NULL}};
const Segment SGI317813[] = {{&S38063[48], 14}, {NULL}};
const Segment SGI317814[] = {{&S38064[30], 3}, {NULL}};
const Segment SGI317815[] = {{&S38064[8], 3}, {NULL}};
const Segment SGI317816[] = {{&S38064[13], 10}, {NULL}};
const Segment SGI317817[] = {{&S38064[55], 7}, {NULL}};
const Segment SGI317818[] = {{&S38064[48], 14}, {NULL}};
const Segment SGI317819[] = {{&S38064[64], 3}, {NULL}};
const Segment SGI317820[] = {{&S38065[30], 3}, {NULL}};
const Segment SGI317821[] = {{&S38065[8], 3}, {NULL}};
const Segment SGI317822[] = {{&S38065[13], 10}, {NULL}};
const Segment SGI317823[] = {{&S38065[55], 7}, {NULL}};
const Segment SGI317824[] = {{&S38065[48], 14}, {NULL}};
const Segment SGI317825[] = {{&S38066[30], 3}, {NULL}};
const Segment SGI317826[] = {{&S38066[8], 3}, {NULL}};
const Segment SGI317827[] = {{&S38066[13], 10}, {NULL}};
const Segment SGI317828[] = {{&S38066[55], 7}, {NULL}};
const Segment SGI317829[] = {{&S38066[48], 14}, {NULL}};
const Segment SGI317830[] = {{&S38066[64], 3}, {NULL}};
const Segment SGI317831[] = {{&S38067[30], 3}, {NULL}};
const Segment SGI317832[] = {{&S38067[8], 3}, {NULL}};
const Segment SGI317833[] = {{&S38067[13], 10}, {NULL}};
const Segment SGI317834[] = {{&S38067[42], 8}, {NULL}};
const Segment SGI317835[] = {{&S38067[52], 6}, {NULL}};
const Segment SGI317836[] = {{&S38068[30], 3}, {NULL}};
const Segment SGI317837[] = {{&S38068[8], 3}, {NULL}};
const Segment SGI317838[] = {{&S38068[13], 10}, {NULL}};
const Segment SGI317839[] = {{&S38068[42], 8}, {NULL}};
const Segment SGI317840[] = {{&S38068[132], 15}, {NULL}};
const Segment SGI317841[] = {{&S38069[30], 3}, {NULL}};
const Segment SGI317842[] = {{&S38069[8], 3}, {NULL}};
const Segment SGI317843[] = {{&S38069[13], 10}, {NULL}};
const Segment SGI317844[] = {{&S38069[55], 7}, {NULL}};
const Segment SGI317845[] = {{&S38069[48], 14}, {NULL}};
const Segment SGI317846[] = {{&S38069[64], 3}, {&S38069[64], 3}, {NULL}};
const Segment SGI317847[] = {{&S38070[30], 3}, {NULL}};
const Segment SGI317848[] = {{&S38070[8], 3}, {NULL}};
const Segment SGI317849[] = {{&S38070[13], 10}, {NULL}};
const Segment SGI317850[] = {{&S38070[42], 8}, {NULL}};
const Segment SGI317851[] = {{&S38070[139], 15}, {NULL}};
const Segment SGI317852[] = {{&S38070[52], 6}, {NULL}};
const Segment SGI317853[] = {{&S38071[30], 3}, {NULL}};
const Segment SGI317854[] = {{&S38071[8], 3}, {NULL}};
const Segment SGI317855[] = {{&S38071[13], 10}, {NULL}};
const Segment SGI317856[] = {{&S38071[55], 7}, {NULL}};
const Segment SGI317857[] = {{&S38071[48], 14}, {NULL}};
const Segment SGI317858[] = {{&S38071[64], 3}, {&S38071[64], 3}, {NULL}};
const Segment SGI317859[] = {{&S38092[30], 3}, {NULL}};
const Segment SGI317860[] = {{&S38092[8], 3}, {NULL}};
const Segment SGI317861[] = {{&S38092[13], 10}, {NULL}};
const Segment SGI317862[] = {{&S38092[42], 7}, {NULL}};
const Segment SGI317863[] = {{&S38092[35], 14}, {NULL}};
const Segment SGI317864[] = {{&S38092[140], 15}, {NULL}};
const Segment SGI317865[] = {{&S38092[64], 7}, {NULL}};
const Segment SGI317866[] = {{&S38093[30], 3}, {NULL}};
const Segment SGI317867[] = {{&S38093[8], 3}, {NULL}};
const Segment SGI317868[] = {{&S38093[13], 10}, {NULL}};
const Segment SGI317869[] = {{&S38093[42], 7}, {NULL}};
const Segment SGI317870[] = {{&S38093[35], 14}, {NULL}};
const Segment SGI317871[] = {{&S38093[64], 7}, {NULL}};
const Segment SGI317872[] = {{&S38094[30], 3}, {NULL}};
const Segment SGI317873[] = {{&S38094[8], 3}, {NULL}};
const Segment SGI317874[] = {{&S38094[13], 10}, {NULL}};
const Segment SGI317875[] = {{&S38094[42], 7}, {NULL}};
const Segment SGI317876[] = {{&S38094[35], 14}, {NULL}};
const Segment SGI317877[] = {{&S38094[140], 15}, {NULL}};
const Segment SGI317878[] = {{&S38094[64], 7}, {NULL}};
const Segment SGI317879[] = {{&S38094[161], 9}, {NULL}};
const Segment SGI317880[] = {{&S38095[30], 3}, {NULL}};
const Segment SGI317881[] = {{&S38095[8], 3}, {NULL}};
const Segment SGI317882[] = {{&S38095[13], 10}, {NULL}};
const Segment SGI317883[] = {{&S38095[42], 7}, {NULL}};
const Segment SGI317884[] = {{&S38095[35], 14}, {NULL}};
const Segment SGI317885[] = {{&S38095[75], 7}, {NULL}};
const Segment SGI317886[] = {{&S38095[51], 3}, {NULL}};
const Segment SGI317887[] = {{&S38096[30], 3}, {NULL}};
const Segment SGI317888[] = {{&S38096[8], 3}, {NULL}};
const Segment SGI317889[] = {{&S38096[13], 10}, {NULL}};
const Segment SGI317890[] = {{&S38096[42], 7}, {NULL}};
const Segment SGI317891[] = {{&S38096[35], 14}, {NULL}};
const Segment SGI317892[] = {{&S38096[51], 7}, {NULL}};
const Segment SGI317893[] = {{&S38096[60], 5}, {NULL}};
const Segment SGI317894[] = {{&S38097[30], 3}, {NULL}};
const Segment SGI317895[] = {{&S38097[8], 3}, {NULL}};
const Segment SGI317896[] = {{&S38097[13], 10}, {NULL}};
const Segment SGI317897[] = {{&S38097[42], 7}, {NULL}};
const Segment SGI317898[] = {{&S38097[35], 14}, {NULL}};
const Segment SGI317899[] = {{&S38097[56], 7}, {NULL}};
const Segment SGI317900[] = {{&S38097[51], 3}, {NULL}};
const Segment SGI317901[] = {{&S38098[30], 3}, {NULL}};
const Segment SGI317902[] = {{&S38098[8], 3}, {NULL}};
const Segment SGI317903[] = {{&S38098[13], 10}, {NULL}};
const Segment SGI317904[] = {{&S38098[42], 7}, {NULL}};
const Segment SGI317905[] = {{&S38098[35], 14}, {NULL}};
const Segment SGI317906[] = {{&S38098[140], 15}, {NULL}};
const Segment SGI317907[] = {{&S38098[64], 7}, {NULL}};
const Segment SGI317908[] = {{&S38099[30], 3}, {NULL}};
const Segment SGI317909[] = {{&S38099[8], 3}, {NULL}};
const Segment SGI317910[] = {{&S38099[13], 10}, {NULL}};
const Segment SGI317911[] = {{&S38099[42], 7}, {NULL}};
const Segment SGI317912[] = {{&S38099[35], 14}, {NULL}};
const Segment SGI317913[] = {{&S38099[56], 7}, {NULL}};
const Segment SGI317914[] = {{&S38099[51], 3}, {NULL}};
const Segment SGI317915[] = {{&S38100[30], 3}, {NULL}};
const Segment SGI317916[] = {{&S38100[8], 3}, {NULL}};
const Segment SGI317917[] = {{&S38100[13], 10}, {NULL}};
const Segment SGI317918[] = {{&S38100[42], 7}, {NULL}};
const Segment SGI317919[] = {{&S38100[35], 14}, {NULL}};
const Segment SGI317920[] = {{&S38100[51], 6}, {NULL}};
const Segment SGI317921[] = {{&S38100[71], 7}, {NULL}};
const Segment SGI317922[] = {{&S38101[30], 3}, {NULL}};
const Segment SGI317923[] = {{&S38101[8], 3}, {NULL}};
const Segment SGI317924[] = {{&S38101[13], 10}, {NULL}};
const Segment SGI317925[] = {{&S38101[42], 7}, {NULL}};
const Segment SGI317926[] = {{&S38101[35], 14}, {NULL}};
const Segment SGI317927[] = {{&S38101[75], 7}, {NULL}};
const Segment SGI317928[] = {{&S38101[51], 3}, {NULL}};
const Segment SGI317929[] = {{&S38102[30], 3}, {NULL}};
const Segment SGI317930[] = {{&S38102[8], 3}, {NULL}};
const Segment SGI317931[] = {{&S38102[13], 10}, {NULL}};
const Segment SGI317932[] = {{&S38102[42], 7}, {NULL}};
const Segment SGI317933[] = {{&S38102[35], 14}, {NULL}};
const Segment SGI317934[] = {{&S38102[64], 7}, {NULL}};
const Segment SGI317935[] = {{&S38103[30], 3}, {NULL}};
const Segment SGI317936[] = {{&S38103[8], 3}, {NULL}};
const Segment SGI317937[] = {{&S38103[13], 10}, {NULL}};
const Segment SGI317938[] = {{&S38103[42], 7}, {NULL}};
const Segment SGI317939[] = {{&S38103[35], 14}, {NULL}};
const Segment SGI317940[] = {{&S38103[56], 7}, {NULL}};
const Segment SGI317941[] = {{&S38103[51], 3}, {NULL}};
const Segment SGI317942[] = {{&S38104[30], 3}, {NULL}};
const Segment SGI317943[] = {{&S38104[8], 3}, {NULL}};
const Segment SGI317944[] = {{&S38104[13], 10}, {NULL}};
const Segment SGI317945[] = {{&S38104[55], 7}, {NULL}};
const Segment SGI317946[] = {{&S38104[48], 14}, {NULL}};
const Segment SGI317947[] = {{&S38104[64], 7}, {NULL}};
const Segment SGI317948[] = {{&S38105[30], 3}, {NULL}};
const Segment SGI317949[] = {{&S38105[8], 3}, {NULL}};
const Segment SGI317950[] = {{&S38105[13], 10}, {NULL}};
const Segment SGI317951[] = {{&S38105[55], 7}, {NULL}};
const Segment SGI317952[] = {{&S38105[48], 14}, {NULL}};
const Segment SGI317953[] = {{&S38105[64], 7}, {NULL}};
const Segment SGI317954[] = {{&S38106[30], 3}, {NULL}};
const Segment SGI317955[] = {{&S38106[8], 3}, {NULL}};
const Segment SGI317956[] = {{&S38106[13], 10}, {NULL}};
const Segment SGI317957[] = {{&S38106[42], 7}, {NULL}};
const Segment SGI317958[] = {{&S38106[35], 14}, {NULL}};
const Segment SGI317959[] = {{&S38106[131], 15}, {NULL}};
const Segment SGI317960[] = {{&S38106[152], 7}, {NULL}};
const Segment SGI317961[] = {{&S38107[30], 3}, {NULL}};
const Segment SGI317962[] = {{&S38107[8], 3}, {NULL}};
const Segment SGI317963[] = {{&S38107[13], 10}, {NULL}};
const Segment SGI317964[] = {{&S38107[42], 7}, {NULL}};
const Segment SGI317965[] = {{&S38107[35], 14}, {NULL}};
const Segment SGI317966[] = {{&S38107[84], 7}, {NULL}};
const Segment SGI317967[] = {{&S38108[30], 3}, {NULL}};
const Segment SGI317968[] = {{&S38108[8], 3}, {NULL}};
const Segment SGI317969[] = {{&S38108[13], 10}, {NULL}};
const Segment SGI317970[] = {{&S38108[55], 7}, {NULL}};
const Segment SGI317971[] = {{&S38108[48], 14}, {NULL}};
const Segment SGI317972[] = {{&S38108[64], 7}, {NULL}};
const Segment SGI317973[] = {{&S38109[30], 3}, {NULL}};
const Segment SGI317974[] = {{&S38109[8], 3}, {NULL}};
const Segment SGI317975[] = {{&S38109[13], 10}, {NULL}};
const Segment SGI317976[] = {{&S38109[55], 7}, {NULL}};
const Segment SGI317977[] = {{&S38109[48], 14}, {NULL}};
const Segment SGI317978[] = {{&S38109[64], 7}, {NULL}};
const Segment SGI317979[] = {{&S38110[30], 3}, {NULL}};
const Segment SGI317980[] = {{&S38110[8], 3}, {NULL}};
const Segment SGI317981[] = {{&S38110[13], 10}, {NULL}};
const Segment SGI317982[] = {{&S38110[55], 7}, {NULL}};
const Segment SGI317983[] = {{&S38110[48], 14}, {NULL}};
const Segment SGI317984[] = {{&S38110[64], 7}, {NULL}};
const Segment SGI317985[] = {{&S38111[30], 3}, {NULL}};
const Segment SGI317986[] = {{&S38111[8], 3}, {NULL}};
const Segment SGI317987[] = {{&S38111[13], 10}, {NULL}};
const Segment SGI317988[] = {{&S38111[42], 7}, {NULL}};
const Segment SGI317989[] = {{&S38111[35], 14}, {NULL}};
const Segment SGI317990[] = {{&S38111[51], 6}, {NULL}};
const Segment SGI317991[] = {{&S38111[98], 7}, {NULL}};
const Segment SGI317992[] = {{&S38112[30], 3}, {NULL}};
const Segment SGI317993[] = {{&S38112[8], 3}, {NULL}};
const Segment SGI317994[] = {{&S38112[13], 10}, {NULL}};
const Segment SGI317995[] = {{&S38112[42], 7}, {NULL}};
const Segment SGI317996[] = {{&S38112[35], 14}, {NULL}};
const Segment SGI317997[] = {{&S38112[131], 15}, {NULL}};
const Segment SGI317998[] = {{&S38112[152], 7}, {NULL}};
const Segment SGI317999[] = {{&S38113[30], 3}, {NULL}};
const Segment SGI318000[] = {{&S38113[8], 3}, {NULL}};
const Segment SGI318001[] = {{&S38113[13], 10}, {NULL}};
const Segment SGI318002[] = {{&S38113[42], 7}, {NULL}};
const Segment SGI318003[] = {{&S38113[35], 14}, {NULL}};
const Segment SGI318004[] = {{&S38113[64], 7}, {NULL}};
const Segment SGI318005[] = {{&S38114[30], 3}, {NULL}};
const Segment SGI318006[] = {{&S38114[8], 3}, {NULL}};
const Segment SGI318007[] = {{&S38114[13], 10}, {NULL}};
const Segment SGI318008[] = {{&S38114[42], 7}, {NULL}};
const Segment SGI318009[] = {{&S38114[35], 14}, {NULL}};
const Segment SGI318010[] = {{&S38114[138], 15}, {NULL}};
const Segment SGI318011[] = {{&S38114[51], 6}, {NULL}};
const Segment SGI318012[] = {{&S38114[179], 7}, {NULL}};
const Segment SGI318013[] = {{&S38115[30], 3}, {NULL}};
const Segment SGI318014[] = {{&S38115[8], 3}, {NULL}};
const Segment SGI318015[] = {{&S38115[13], 10}, {NULL}};
const Segment SGI318016[] = {{&S38115[42], 7}, {NULL}};
const Segment SGI318017[] = {{&S38115[35], 14}, {NULL}};
const Segment SGI318018[] = {{&S38115[83], 7}, {NULL}};
const Segment SGI318019[] = {{&S38116[30], 3}, {NULL}};
const Segment SGI318020[] = {{&S38116[8], 3}, {NULL}};
const Segment SGI318021[] = {{&S38116[13], 10}, {NULL}};
const Segment SGI318022[] = {{&S38116[42], 7}, {NULL}};
const Segment SGI318023[] = {{&S38116[35], 14}, {NULL}};
const Segment SGI318024[] = {{&S38116[138], 15}, {NULL}};
const Segment SGI318025[] = {{&S38116[51], 6}, {NULL}};
const Segment SGI318026[] = {{&S38116[159], 7}, {NULL}};
const Segment SGI318027[] = {{&S38117[30], 3}, {NULL}};
const Segment SGI318028[] = {{&S38117[8], 3}, {NULL}};
const Segment SGI318029[] = {{&S38117[13], 10}, {NULL}};
const Segment SGI318030[] = {{&S38117[42], 7}, {NULL}};
const Segment SGI318031[] = {{&S38117[35], 14}, {NULL}};
const Segment SGI318032[] = {{&S38117[51], 6}, {NULL}};
const Segment SGI318033[] = {{&S38117[98], 7}, {NULL}};
const Segment SGI318034[] = {{&S38118[30], 4}, {NULL}};
const Segment SGI318035[] = {{&S38118[8], 3}, {NULL}};
const Segment SGI318036[] = {{&S38118[13], 10}, {NULL}};
const Segment SGI318037[] = {{&S38118[43], 7}, {NULL}};
const Segment SGI318038[] = {{&S38118[36], 14}, {NULL}};
const Segment SGI318039[] = {{&S38118[52], 6}, {NULL}};
const Segment SGI318040[] = {{&S38118[99], 7}, {NULL}};
const Segment SGI318041[] = {{&S38119[30], 3}, {NULL}};
const Segment SGI318042[] = {{&S38119[8], 3}, {NULL}};
const Segment SGI318043[] = {{&S38119[13], 10}, {NULL}};
const Segment SGI318044[] = {{&S38119[42], 7}, {NULL}};
const Segment SGI318045[] = {{&S38119[35], 14}, {NULL}};
const Segment SGI318046[] = {{&S38119[64], 7}, {NULL}};
const Segment SGI318047[] = {{&S38120[30], 3}, {NULL}};
const Segment SGI318048[] = {{&S38120[8], 3}, {NULL}};
const Segment SGI318049[] = {{&S38120[13], 10}, {NULL}};
const Segment SGI318050[] = {{&S38120[55], 7}, {NULL}};
const Segment SGI318051[] = {{&S38120[48], 14}, {NULL}};
const Segment SGI318052[] = {{&S38120[79], 7}, {NULL}};
const Segment SGI318053[] = {{&S38120[64], 13}, {NULL}};
const Segment SGI318054[] = {{&S38121[30], 3}, {NULL}};
const Segment SGI318055[] = {{&S38121[8], 3}, {NULL}};
const Segment SGI318056[] = {{&S38121[13], 10}, {NULL}};
const Segment SGI318057[] = {{&S38121[55], 7}, {NULL}};
const Segment SGI318058[] = {{&S38121[48], 14}, {NULL}};
const Segment SGI318059[] = {{&S38121[69], 7}, {NULL}};
const Segment SGI318060[] = {{&S38121[64], 3}, {NULL}};
const Segment SGI318061[] = {{&S38123[30], 3}, {NULL}};
const Segment SGI318062[] = {{&S38123[8], 3}, {NULL}};
const Segment SGI318063[] = {{&S38123[13], 10}, {NULL}};
const Segment SGI318064[] = {{&S38123[55], 7}, {NULL}};
const Segment SGI318065[] = {{&S38123[48], 14}, {NULL}};
const Segment SGI318066[] = {{&S38123[79], 7}, {NULL}};
const Segment SGI318067[] = {{&S38123[64], 13}, {NULL}};
const Segment SGI318068[] = {{&S38124[30], 4}, {NULL}};
const Segment SGI318069[] = {{&S38124[8], 3}, {NULL}};
const Segment SGI318070[] = {{&S38124[13], 10}, {NULL}};
const Segment SGI318071[] = {{&S38124[43], 7}, {NULL}};
const Segment SGI318072[] = {{&S38124[36], 14}, {NULL}};
const Segment SGI318073[] = {{&S38124[57], 7}, {NULL}};
const Segment SGI318074[] = {{&S38124[52], 3}, {NULL}};
const Segment SGI318075[] = {{&S38125[30], 3}, {NULL}};
const Segment SGI318076[] = {{&S38125[8], 3}, {NULL}};
const Segment SGI318077[] = {{&S38125[13], 10}, {NULL}};
const Segment SGI318078[] = {{&S38125[42], 7}, {NULL}};
const Segment SGI318079[] = {{&S38125[35], 14}, {NULL}};
const Segment SGI318080[] = {{&S38125[77], 7}, {NULL}};
const Segment SGI318081[] = {{&S38125[51], 3}, {NULL}};
const Segment SGI318082[] = {{&S38126[30], 3}, {NULL}};
const Segment SGI318083[] = {{&S38126[8], 3}, {NULL}};
const Segment SGI318084[] = {{&S38126[13], 10}, {NULL}};
const Segment SGI318085[] = {{&S38126[55], 7}, {NULL}};
const Segment SGI318086[] = {{&S38126[48], 14}, {NULL}};
const Segment SGI318087[] = {{&S38126[64], 7}, {&S38126[64], 7}, {NULL}};
const Segment SGI318088[] = {{&S38127[30], 3}, {NULL}};
const Segment SGI318089[] = {{&S38127[8], 3}, {NULL}};
const Segment SGI318090[] = {{&S38127[13], 10}, {NULL}};
const Segment SGI318091[] = {{&S38127[42], 7}, {NULL}};
const Segment SGI318092[] = {{&S38127[35], 14}, {NULL}};
const Segment SGI318093[] = {{&S38127[131], 15}, {NULL}};
const Segment SGI318094[] = {{&S38127[152], 7}, {NULL}};
const Segment SGI318095[] = {{&S38128[30], 3}, {&S38128[103], 3}, {NULL}};
const Segment SGI318096[] = {{&S38128[8], 3}, {&S38128[8], 3}, {NULL}};
const Segment SGI318097[] = {{&S38128[13], 10}, {&S38128[13], 10}, {NULL}};
const Segment SGI318098[] = {{&S38128[42], 7}, {&S38128[115], 7}, {NULL}};
const Segment SGI318099[] = {{&S38128[35], 14}, {&S38128[108], 14}, {NULL}};
const Segment SGI318100[] = {{&S38128[64], 7}, {NULL}};
const Segment SGI318101[] = {{&S38128[124], 3}, {NULL}};
const Segment SGI318102[] = {{&S38129[30], 3}, {NULL}};
const Segment SGI318103[] = {{&S38129[8], 3}, {NULL}};
const Segment SGI318104[] = {{&S38129[13], 10}, {NULL}};
const Segment SGI318105[] = {{&S38129[55], 7}, {NULL}};
const Segment SGI318106[] = {{&S38129[48], 14}, {NULL}};
const Segment SGI318107[] = {{&S38129[64], 7}, {NULL}};
const Segment SGI318108[] = {{&S38129[73], 3}, {NULL}};
const Segment SGI318109[] = {{&S38130[30], 4}, {NULL}};
const Segment SGI318110[] = {{&S38130[8], 3}, {NULL}};
const Segment SGI318111[] = {{&S38130[13], 10}, {NULL}};
const Segment SGI318112[] = {{&S38130[43], 7}, {NULL}};
const Segment SGI318113[] = {{&S38130[36], 14}, {NULL}};
const Segment SGI318114[] = {{&S38130[65], 7}, {NULL}};
const Segment SGI318115[] = {{&S38131[30], 3}, {NULL}};
const Segment SGI318116[] = {{&S38131[8], 3}, {NULL}};
const Segment SGI318117[] = {{&S38131[13], 10}, {NULL}};
const Segment SGI318118[] = {{&S38131[42], 7}, {NULL}};
const Segment SGI318119[] = {{&S38131[35], 14}, {NULL}};
const Segment SGI318120[] = {{&S38131[138], 15}, {NULL}};
const Segment SGI318121[] = {{&S38131[51], 6}, {NULL}};
const Segment SGI318122[] = {{&S38131[179], 7}, {NULL}};
const Segment SGI318123[] = {{&S38132[30], 3}, {NULL}};
const Segment SGI318124[] = {{&S38132[8], 3}, {NULL}};
const Segment SGI318125[] = {{&S38132[13], 10}, {NULL}};
const Segment SGI318126[] = {{&S38132[55], 7}, {NULL}};
const Segment SGI318127[] = {{&S38132[48], 14}, {NULL}};
const Segment SGI318128[] = {{&S38132[64], 7}, {&S38132[64], 7}, {NULL}};
const Segment SGI318129[] = {{&S38133[30], 3}, {NULL}};
const Segment SGI318130[] = {{&S38133[8], 3}, {NULL}};
const Segment SGI318131[] = {{&S38133[13], 10}, {NULL}};
const Segment SGI318132[] = {{&S38133[55], 7}, {NULL}};
const Segment SGI318133[] = {{&S38133[48], 14}, {NULL}};
const Segment SGI318134[] = {{&S38133[64], 7}, {&S38133[64], 7}, {NULL}};
const Segment SGI318135[] = {{&S38134[30], 3}, {NULL}};
const Segment SGI318136[] = {{&S38134[8], 3}, {NULL}};
const Segment SGI318137[] = {{&S38134[13], 10}, {NULL}};
const Segment SGI318138[] = {{&S38134[42], 7}, {NULL}};
const Segment SGI318139[] = {{&S38134[35], 14}, {NULL}};
const Segment SGI318140[] = {{&S38134[155], 15}, {NULL}};
const Segment SGI318141[] = {{&S38134[51], 6}, {NULL}};
const Segment SGI318142[] = {{&S38134[79], 7}, {NULL}};
const Segment SGI318143[] = {{&S38135[30], 3}, {NULL}};
const Segment SGI318144[] = {{&S38135[8], 3}, {NULL}};
const Segment SGI318145[] = {{&S38135[13], 10}, {NULL}};
const Segment SGI318146[] = {{&S38135[55], 7}, {NULL}};
const Segment SGI318147[] = {{&S38135[48], 14}, {NULL}};
const Segment SGI318148[] = {{&S38135[79], 7}, {NULL}};
const Segment SGI318149[] = {{&S38135[64], 13}, {NULL}};
const Segment SGI318150[] = {{&S38136[30], 3}, {NULL}};
const Segment SGI318151[] = {{&S38136[8], 3}, {NULL}};
const Segment SGI318152[] = {{&S38136[13], 10}, {NULL}};
const Segment SGI318153[] = {{&S38136[55], 7}, {NULL}};
const Segment SGI318154[] = {{&S38136[48], 14}, {NULL}};
const Segment SGI318155[] = {{&S38136[69], 7}, {NULL}};
const Segment SGI318156[] = {{&S38136[64], 3}, {NULL}};
const Segment SGI318157[] = {{&S38137[30], 3}, {NULL}};
const Segment SGI318158[] = {{&S38137[8], 3}, {NULL}};
const Segment SGI318159[] = {{&S38137[13], 10}, {NULL}};
const Segment SGI318160[] = {{&S38137[55], 7}, {NULL}};
const Segment SGI318161[] = {{&S38137[48], 14}, {NULL}};
const Segment SGI318162[] = {{&S38137[79], 7}, {NULL}};
const Segment SGI318163[] = {{&S38137[64], 13}, {NULL}};
const Segment SGI318164[] = {{&S38138[30], 3}, {NULL}};
const Segment SGI318165[] = {{&S38138[8], 3}, {NULL}};
const Segment SGI318166[] = {{&S38138[13], 10}, {NULL}};
const Segment SGI318167[] = {{&S38138[55], 7}, {NULL}};
const Segment SGI318168[] = {{&S38138[48], 14}, {NULL}};
const Segment SGI318169[] = {{&S38138[64], 7}, {&S38138[64], 7}, {NULL}};
const Segment SGI318170[] = {{&S38139[30], 3}, {NULL}};
const Segment SGI318171[] = {{&S38139[8], 3}, {NULL}};
const Segment SGI318172[] = {{&S38139[13], 10}, {NULL}};
const Segment SGI318173[] = {{&S38139[42], 7}, {NULL}};
const Segment SGI318174[] = {{&S38139[35], 14}, {NULL}};
const Segment SGI318175[] = {{&S38139[147], 15}, {NULL}};
const Segment SGI318176[] = {{&S38139[51], 6}, {NULL}};
const Segment SGI318177[] = {{&S38139[71], 7}, {NULL}};
const Segment SGI318178[] = {{&S38140[30], 3}, {NULL}};
const Segment SGI318179[] = {{&S38140[8], 3}, {NULL}};
const Segment SGI318180[] = {{&S38140[13], 10}, {NULL}};
const Segment SGI318181[] = {{&S38140[42], 7}, {NULL}};
const Segment SGI318182[] = {{&S38140[35], 14}, {NULL}};
const Segment SGI318183[] = {{&S38140[51], 6}, {NULL}};
const Segment SGI318184[] = {{&S38140[144], 7}, {NULL}};
const Segment SGI318185[] = {{&S38141[30], 4}, {NULL}};
const Segment SGI318186[] = {{&S38141[8], 3}, {NULL}};
const Segment SGI318187[] = {{&S38141[13], 10}, {NULL}};
const Segment SGI318188[] = {{&S38141[43], 7}, {NULL}};
const Segment SGI318189[] = {{&S38141[36], 14}, {NULL}};
const Segment SGI318190[] = {{&S38141[139], 15}, {NULL}};
const Segment SGI318191[] = {{&S38141[52], 6}, {NULL}};
const Segment SGI318192[] = {{&S38141[180], 7}, {NULL}};
const Segment SGI318193[] = {{&S55466[3], 5}, {NULL}};
const Segment SGI318194[] = {{&S55466[28], 8}, {NULL}};
const Segment SGI318195[] = {{&S38240[31], 3}, {NULL}};
const Segment SGI318196[] = {{&S38240[8], 3}, {NULL}};
const Segment SGI318197[] = {{&S38240[43], 3}, {NULL}};
const Segment SGI318198[] = {{&S38241[30], 3}, {NULL}};
const Segment SGI318199[] = {{&S38241[8], 3}, {NULL}};
const Segment SGI318200[] = {{&S38241[13], 10}, {NULL}};
const Segment SGI318201[] = {{&S38241[35], 5}, {NULL}};
const Segment SGI318202[] = {{&S38242[30], 3}, {NULL}};
const Segment SGI318203[] = {{&S38242[8], 3}, {NULL}};
const Segment SGI318204[] = {{&S38242[13], 10}, {NULL}};
const Segment SGI318205[] = {{&S38242[35], 5}, {NULL}};
const Segment SGI318206[] = {{&S38448[30], 3}, {NULL}};
const Segment SGI318207[] = {{&S38448[8], 3}, {NULL}};
const Segment SGI318208[] = {{&S38448[13], 10}, {NULL}};
const Segment SGI318209[] = {{&S38448[43], 7}, {NULL}};
const Segment SGI318210[] = {{&S38449[30], 3}, {NULL}};
const Segment SGI318211[] = {{&S38449[8], 3}, {NULL}};
const Segment SGI318212[] = {{&S38449[13], 10}, {NULL}};
const Segment SGI318213[] = {{&S38449[43], 7}, {NULL}};
const Segment SGI318214[] = {{&S38449[36], 14}, {NULL}};
const Segment SGI318215[] = {{&S38450[30], 3}, {NULL}};
const Segment SGI318216[] = {{&S38450[8], 3}, {NULL}};
const Segment SGI318217[] = {{&S38450[13], 10}, {NULL}};
const Segment SGI318218[] = {{&S38450[42], 7}, {NULL}};
const Segment SGI318219[] = {{&S38450[35], 14}, {NULL}};
const Segment SGI318220[] = {{&S38450[51], 3}, {NULL}};
const Segment SGI318221[] = {{&S38451[30], 3}, {NULL}};
const Segment SGI318222[] = {{&S38451[8], 3}, {NULL}};
const Segment SGI318223[] = {{&S38451[13], 10}, {NULL}};
const Segment SGI318224[] = {{&S38451[42], 7}, {NULL}};
const Segment SGI318225[] = {{&S38451[35], 14}, {NULL}};
const Segment SGI318226[] = {{&S38451[51], 6}, {NULL}};
const Segment SGI318227[] = {{&S38452[30], 3}, {NULL}};
const Segment SGI318228[] = {{&S38452[8], 3}, {NULL}};
const Segment SGI318229[] = {{&S38452[13], 10}, {NULL}};
const Segment SGI318230[] = {{&S38452[35], 8}, {NULL}};
const Segment SGI318231[] = {{&S38453[30], 3}, {NULL}};
const Segment SGI318232[] = {{&S38453[8], 3}, {NULL}};
const Segment SGI318233[] = {{&S38453[13], 10}, {NULL}};
const Segment SGI318234[] = {{&S38453[42], 7}, {NULL}};
const Segment SGI318235[] = {{&S38453[35], 14}, {NULL}};
const Segment SGI318236[] = {{&S38453[131], 15}, {NULL}};
const Segment SGI318237[] = {{&S38454[30], 3}, {NULL}};
const Segment SGI318238[] = {{&S38454[8], 3}, {NULL}};
const Segment SGI318239[] = {{&S38454[13], 10}, {NULL}};
const Segment SGI318240[] = {{&S38454[42], 7}, {NULL}};
const Segment SGI318241[] = {{&S38454[35], 14}, {NULL}};
const Segment SGI318242[] = {{&S38454[138], 15}, {NULL}};
const Segment SGI318243[] = {{&S38454[51], 6}, {NULL}};
const Segment SGI318244[] = {{&S38554[30], 3}, {NULL}};
const Segment SGI318245[] = {{&S38554[8], 3}, {NULL}};
const Segment SGI318246[] = {{&S38554[13], 10}, {NULL}};
const Segment SGI318247[] = {{&S38554[42], 7}, {NULL}};
const Segment SGI318248[] = {{&S38554[35], 14}, {NULL}};
const Segment SGI318249[] = {{&S38554[51], 13}, {NULL}};
const Segment SGI318250[] = {{&S38555[30], 3}, {NULL}};
const Segment SGI318251[] = {{&S38555[8], 3}, {NULL}};
const Segment SGI318252[] = {{&S38555[13], 10}, {NULL}};
const Segment SGI318253[] = {{&S38555[42], 7}, {NULL}};
const Segment SGI318254[] = {{&S38555[35], 14}, {NULL}};
const Segment SGI318255[] = {{&S38555[51], 13}, {&S38555[51], 13}, {NULL}};
const Segment SGI318256[] = {{&S38568[30], 3}, {NULL}};
const Segment SGI318257[] = {{&S38568[8], 3}, {NULL}};
const Segment SGI318258[] = {{&S38568[13], 10}, {NULL}};
const Segment SGI318259[] = {{&S38568[42], 7}, {NULL}};
const Segment SGI318260[] = {{&S38569[30], 4}, {NULL}};
const Segment SGI318261[] = {{&S38569[8], 3}, {NULL}};
const Segment SGI318262[] = {{&S38569[13], 10}, {NULL}};
const Segment SGI318263[] = {{&S38569[43], 7}, {NULL}};
const Segment SGI318264[] = {{&S38570[30], 3}, {NULL}};
const Segment SGI318265[] = {{&S38570[8], 3}, {NULL}};
const Segment SGI318266[] = {{&S38570[13], 10}, {NULL}};
const Segment SGI318267[] = {{&S38570[42], 7}, {NULL}};
const Segment SGI318268[] = {{&S38570[35], 14}, {NULL}};
const Segment SGI318269[] = {{&S38571[8], 3}, {NULL}};
const Segment SGI318270[] = {{&S38571[13], 10}, {NULL}};
const Segment SGI318271[] = {{&S38571[42], 7}, {NULL}};
const Segment SGI318272[] = {{&S38571[35], 14}, {NULL}};
const Segment SGI318273[] = {{&S38572[30], 4}, {NULL}};
const Segment SGI318274[] = {{&S38572[8], 3}, {NULL}};
const Segment SGI318275[] = {{&S38572[13], 10}, {NULL}};
const Segment SGI318276[] = {{&S38572[43], 7}, {NULL}};
const Segment SGI318277[] = {{&S38573[29], 3}, {NULL}};
const Segment SGI318278[] = {{&S38573[8], 3}, {NULL}};
const Segment SGI318279[] = {{&S38573[13], 10}, {NULL}};
const Segment SGI318280[] = {{&S38573[42], 7}, {NULL}};
const Segment SGI318281[] = {{&S38573[35], 14}, {NULL}};
const Segment SGI318282[] = {{&S38574[30], 4}, {NULL}};
const Segment SGI318283[] = {{&S38574[8], 3}, {NULL}};
const Segment SGI318284[] = {{&S38574[13], 10}, {NULL}};
const Segment SGI318285[] = {{&S38574[43], 7}, {NULL}};
const Segment SGI318286[] = {{&S38575[8], 3}, {NULL}};
const Segment SGI318287[] = {{&S38575[13], 10}, {NULL}};
const Segment SGI318288[] = {{&S38575[35], 8}, {NULL}};
const Segment SGI318289[] = {{&S38809[27], 3}, {NULL}};
const Segment SGI318290[] = {{&S38809[8], 3}, {NULL}};
const Segment SGI318291[] = {{&S38809[13], 7}, {NULL}};
const Segment SGI318292[] = {{&S38809[39], 7}, {NULL}};
const Segment SGI318293[] = {{&S38810[30], 3}, {NULL}};
const Segment SGI318294[] = {{&S38810[8], 3}, {NULL}};
const Segment SGI318295[] = {{&S38810[13], 10}, {NULL}};
const Segment SGI318296[] = {{&S38810[54], 7}, {NULL}};
const Segment SGI318297[] = {{&S38810[47], 14}, {NULL}};
const Segment SGI318298[] = {{&S38811[30], 4}, {NULL}};
const Segment SGI318299[] = {{&S38811[8], 3}, {NULL}};
const Segment SGI318300[] = {{&S38811[13], 10}, {NULL}};
const Segment SGI318301[] = {{&S38811[43], 7}, {NULL}};
const Segment SGI318302[] = {{&S38812[30], 3}, {NULL}};
const Segment SGI318303[] = {{&S38812[8], 3}, {NULL}};
const Segment SGI318304[] = {{&S38812[13], 10}, {NULL}};
const Segment SGI318305[] = {{&S38812[42], 7}, {NULL}};
const Segment SGI318306[] = {{&S38813[30], 3}, {NULL}};
const Segment SGI318307[] = {{&S38813[8], 3}, {NULL}};
const Segment SGI318308[] = {{&S38813[13], 10}, {NULL}};
const Segment SGI318309[] = {{&S38813[42], 7}, {NULL}};
const Segment SGI318310[] = {{&S38813[35], 14}, {NULL}};
const Segment SGI318311[] = {{&S38814[30], 3}, {NULL}};
const Segment SGI318312[] = {{&S38814[8], 3}, {NULL}};
const Segment SGI318313[] = {{&S38814[13], 7}, {NULL}};
const Segment SGI318314[] = {{&S38814[42], 7}, {NULL}};
const Segment SGI318315[] = {{&S38814[35], 14}, {NULL}};
const Segment SGI318316[] = {{&S38815[30], 3}, {NULL}};
const Segment SGI318317[] = {{&S38815[8], 3}, {NULL}};
const Segment SGI318318[] = {{&S38815[13], 10}, {NULL}};
const Segment SGI318319[] = {{&S38815[42], 7}, {NULL}};
const Segment SGI318320[] = {{&S38815[35], 14}, {NULL}};
const Segment SGI318321[] = {{&S38815[51], 3}, {NULL}};
const Segment SGI318322[] = {{&S38816[30], 3}, {NULL}};
const Segment SGI318323[] = {{&S38816[8], 3}, {NULL}};
const Segment SGI318324[] = {{&S38816[13], 10}, {NULL}};
const Segment SGI318325[] = {{&S38816[42], 7}, {NULL}};
const Segment SGI318326[] = {{&S38816[35], 14}, {NULL}};
const Segment SGI318327[] = {{&S38816[85], 15}, {&S38816[85], 15}, {NULL}};
const Segment SGI318328[] = {{&S38816[51], 6}, {NULL}};
const Segment SGI318329[] = {{&S38960[30], 3}, {NULL}};
const Segment SGI318330[] = {{&S38960[8], 3}, {NULL}};
const Segment SGI318331[] = {{&S38960[13], 10}, {NULL}};
const Segment SGI318332[] = {{&S38960[42], 7}, {NULL}};
const Segment SGI318333[] = {{&S38961[30], 3}, {NULL}};
const Segment SGI318334[] = {{&S38961[8], 3}, {NULL}};
const Segment SGI318335[] = {{&S38961[13], 10}, {NULL}};
const Segment SGI318336[] = {{&S38961[55], 7}, {NULL}};
const Segment SGI318337[] = {{&S38961[48], 14}, {NULL}};
const Segment SGI318338[] = {{&S38962[30], 3}, {NULL}};
const Segment SGI318339[] = {{&S38962[8], 3}, {NULL}};
const Segment SGI318340[] = {{&S38962[13], 10}, {NULL}};
const Segment SGI318341[] = {{&S38962[55], 7}, {NULL}};
const Segment SGI318342[] = {{&S38962[48], 14}, {NULL}};
const Segment SGI318343[] = {{&S38963[30], 3}, {NULL}};
const Segment SGI318344[] = {{&S38963[8], 3}, {NULL}};
const Segment SGI318345[] = {{&S38963[13], 10}, {NULL}};
const Segment SGI318346[] = {{&S38963[55], 7}, {NULL}};
const Segment SGI318347[] = {{&S38963[48], 14}, {NULL}};
const Segment SGI318348[] = {{&S38963[64], 3}, {NULL}};
const Segment SGI318349[] = {{&S38964[30], 3}, {NULL}};
const Segment SGI318350[] = {{&S38964[8], 3}, {NULL}};
const Segment SGI318351[] = {{&S38964[13], 10}, {NULL}};
const Segment SGI318352[] = {{&S38964[55], 7}, {NULL}};
const Segment SGI318353[] = {{&S38964[48], 14}, {NULL}};
const Segment SGI318354[] = {{&S38964[64], 3}, {NULL}};
const Segment SGI318355[] = {{&S38965[30], 3}, {NULL}};
const Segment SGI318356[] = {{&S38965[8], 3}, {NULL}};
const Segment SGI318357[] = {{&S38965[13], 10}, {NULL}};
const Segment SGI318358[] = {{&S38965[55], 7}, {NULL}};
const Segment SGI318359[] = {{&S38965[48], 14}, {NULL}};
const Segment SGI318360[] = {{&S38965[64], 3}, {&S38965[64], 3}, {NULL}};
const Segment SGI318361[] = {{&S38966[30], 3}, {NULL}};
const Segment SGI318362[] = {{&S38966[8], 3}, {NULL}};
const Segment SGI318363[] = {{&S38966[13], 10}, {NULL}};
const Segment SGI318364[] = {{&S38966[55], 7}, {NULL}};
const Segment SGI318365[] = {{&S38966[48], 14}, {NULL}};
const Segment SGI318366[] = {{&S38966[64], 3}, {&S38966[64], 3}, {NULL}};
const Segment SGI318367[] = {{&S38969[30], 3}, {NULL}};
const Segment SGI318368[] = {{&S38969[8], 3}, {NULL}};
const Segment SGI318369[] = {{&S38969[13], 10}, {NULL}};
const Segment SGI318370[] = {{&S38969[42], 3}, {NULL}};
const Segment SGI318371[] = {{&S38970[30], 3}, {NULL}};
const Segment SGI318372[] = {{&S38970[8], 3}, {NULL}};
const Segment SGI318373[] = {{&S38970[13], 10}, {NULL}};
const Segment SGI318374[] = {{&S38971[30], 3}, {NULL}};
const Segment SGI318375[] = {{&S38971[8], 3}, {NULL}};
const Segment SGI318376[] = {{&S38971[13], 10}, {NULL}};
const Segment SGI318377[] = {{&S38971[35], 5}, {NULL}};
const Segment SGI318378[] = {{&S38972[30], 3}, {NULL}};
const Segment SGI318379[] = {{&S38972[8], 3}, {NULL}};
const Segment SGI318380[] = {{&S38972[13], 10}, {NULL}};
const Segment SGI318381[] = {{&S38973[30], 3}, {NULL}};
const Segment SGI318382[] = {{&S38973[8], 3}, {NULL}};
const Segment SGI318383[] = {{&S38973[13], 10}, {NULL}};
const Segment SGI318384[] = {{&S38973[40], 5}, {NULL}};
const Segment SGI318385[] = {{&S38974[30], 3}, {NULL}};
const Segment SGI318386[] = {{&S38974[8], 3}, {NULL}};
const Segment SGI318387[] = {{&S38974[13], 10}, {NULL}};
const Segment SGI318388[] = {{&S38974[40], 6}, {NULL}};
const Segment SGI318389[] = {{&S38975[30], 3}, {NULL}};
const Segment SGI318390[] = {{&S38975[8], 3}, {NULL}};
const Segment SGI318391[] = {{&S38975[13], 10}, {NULL}};
const Segment SGI318392[] = {{&S38975[40], 3}, {NULL}};
const Segment SGI318393[] = {{&S38976[30], 3}, {NULL}};
const Segment SGI318394[] = {{&S38976[8], 3}, {NULL}};
const Segment SGI318395[] = {{&S38976[13], 10}, {NULL}};
const Segment SGI318396[] = {{&S38976[42], 3}, {NULL}};
const Segment SGI318397[] = {{&S38988[30], 3}, {NULL}};
const Segment SGI318398[] = {{&S38988[8], 3}, {NULL}};
const Segment SGI318399[] = {{&S38988[13], 10}, {NULL}};
const Segment SGI318400[] = {{&S38988[42], 7}, {NULL}};
const Segment SGI318401[] = {{&S38988[35], 14}, {NULL}};
const Segment SGI318402[] = {{&S38988[80], 5}, {NULL}};
const Segment SGI318403[] = {{&S38988[51], 6}, {NULL}};
const Segment SGI318404[] = {{&S38989[30], 3}, {NULL}};
const Segment SGI318405[] = {{&S38989[8], 3}, {NULL}};
const Segment SGI318406[] = {{&S38989[13], 10}, {NULL}};
const Segment SGI318407[] = {{&S38989[42], 7}, {NULL}};
const Segment SGI318408[] = {{&S38989[35], 14}, {NULL}};
const Segment SGI318409[] = {{&S38989[80], 5}, {NULL}};
const Segment SGI318410[] = {{&S38989[51], 6}, {NULL}};
const Segment SGI318411[] = {{&S38990[30], 3}, {NULL}};
const Segment SGI318412[] = {{&S38990[8], 3}, {NULL}};
const Segment SGI318413[] = {{&S38990[13], 10}, {NULL}};
const Segment SGI318414[] = {{&S38990[42], 7}, {NULL}};
const Segment SGI318415[] = {{&S38990[35], 14}, {NULL}};
const Segment SGI318416[] = {{&S38990[80], 5}, {NULL}};
const Segment SGI318417[] = {{&S38990[51], 6}, {NULL}};
const Segment SGI318418[] = {{&S38991[30], 3}, {NULL}};
const Segment SGI318419[] = {{&S38991[8], 3}, {NULL}};
const Segment SGI318420[] = {{&S38991[13], 10}, {NULL}};
const Segment SGI318421[] = {{&S38991[42], 7}, {NULL}};
const Segment SGI318422[] = {{&S38991[35], 14}, {NULL}};
const Segment SGI318423[] = {{&S38991[73], 5}, {NULL}};
const Segment SGI318424[] = {{&S67460[3], 5}, {NULL}};
const Segment SGI318425[] = {{&S38992[30], 3}, {NULL}};
const Segment SGI318426[] = {{&S38992[8], 3}, {NULL}};
const Segment SGI318427[] = {{&S38992[13], 10}, {NULL}};
const Segment SGI318428[] = {{&S38992[42], 7}, {NULL}};
const Segment SGI318429[] = {{&S38992[35], 14}, {NULL}};
const Segment SGI318430[] = {{&S38992[131], 15}, {NULL}};
const Segment SGI318431[] = {{&S38992[171], 5}, {NULL}};
const Segment SGI318432[] = {{&S38993[30], 3}, {NULL}};
const Segment SGI318433[] = {{&S38993[8], 3}, {NULL}};
const Segment SGI318434[] = {{&S38993[13], 10}, {NULL}};
const Segment SGI318435[] = {{&S38993[42], 7}, {NULL}};
const Segment SGI318436[] = {{&S38993[35], 14}, {NULL}};
const Segment SGI318437[] = {{&S38993[73], 5}, {NULL}};
const Segment SGI318438[] = {{&S67465[3], 4}, {NULL}};
const Segment SGI318439[] = {{&S38994[30], 3}, {NULL}};
const Segment SGI318440[] = {{&S38994[8], 3}, {NULL}};
const Segment SGI318441[] = {{&S38994[13], 10}, {NULL}};
const Segment SGI318442[] = {{&S38994[42], 7}, {NULL}};
const Segment SGI318443[] = {{&S38994[35], 14}, {NULL}};
const Segment SGI318444[] = {{&S38994[80], 5}, {NULL}};
const Segment SGI318445[] = {{&S38994[51], 6}, {NULL}};
const Segment SGI318446[] = {{&S38995[30], 3}, {NULL}};
const Segment SGI318447[] = {{&S38995[8], 3}, {NULL}};
const Segment SGI318448[] = {{&S38995[13], 10}, {NULL}};
const Segment SGI318449[] = {{&S38995[42], 7}, {NULL}};
const Segment SGI318450[] = {{&S38995[35], 14}, {NULL}};
const Segment SGI318451[] = {{&S38995[73], 5}, {NULL}};
const Segment SGI318452[] = {{&S38996[30], 3}, {NULL}};
const Segment SGI318453[] = {{&S38996[8], 3}, {NULL}};
const Segment SGI318454[] = {{&S38996[13], 10}, {NULL}};
const Segment SGI318455[] = {{&S38996[42], 7}, {NULL}};
const Segment SGI318456[] = {{&S38996[35], 14}, {NULL}};
const Segment SGI318457[] = {{&S38996[131], 15}, {NULL}};
const Segment SGI318458[] = {{&S38996[186], 5}, {NULL}};
const Segment SGI318459[] = {{&S38996[152], 9}, {NULL}};
const Segment SGI318460[] = {{&S38997[30], 3}, {NULL}};
const Segment SGI318461[] = {{&S38997[8], 3}, {NULL}};
const Segment SGI318462[] = {{&S38997[13], 10}, {NULL}};
const Segment SGI318463[] = {{&S38997[42], 7}, {NULL}};
const Segment SGI318464[] = {{&S38997[35], 14}, {NULL}};
const Segment SGI318465[] = {{&S38997[73], 5}, {NULL}};
const Segment SGI318466[] = {{&S38998[30], 3}, {NULL}};
const Segment SGI318467[] = {{&S38998[8], 3}, {NULL}};
const Segment SGI318468[] = {{&S38998[13], 10}, {NULL}};
const Segment SGI318469[] = {{&S38998[42], 7}, {NULL}};
const Segment SGI318470[] = {{&S38998[35], 14}, {NULL}};
const Segment SGI318471[] = {{&S38998[146], 15}, {NULL}};
const Segment SGI318472[] = {{&S38998[186], 5}, {NULL}};
const Segment SGI318473[] = {{&S38998[51], 6}, {NULL}};
const Segment SGI318474[] = {{&S38999[30], 3}, {NULL}};
const Segment SGI318475[] = {{&S38999[8], 3}, {NULL}};
const Segment SGI318476[] = {{&S38999[13], 10}, {NULL}};
const Segment SGI318477[] = {{&S38999[42], 7}, {NULL}};
const Segment SGI318478[] = {{&S38999[35], 14}, {NULL}};
const Segment SGI318479[] = {{&S38999[131], 15}, {NULL}};
const Segment SGI318480[] = {{&S38999[171], 5}, {NULL}};
const Segment SGI318481[] = {{&S39000[30], 3}, {NULL}};
const Segment SGI318482[] = {{&S39000[8], 3}, {NULL}};
const Segment SGI318483[] = {{&S39000[13], 10}, {NULL}};
const Segment SGI318484[] = {{&S39000[42], 7}, {NULL}};
const Segment SGI318485[] = {{&S39000[35], 14}, {NULL}};
const Segment SGI318486[] = {{&S39000[80], 6}, {NULL}};
const Segment SGI318487[] = {{&S39000[51], 6}, {NULL}};
const Segment SGI318488[] = {{&S39001[30], 3}, {NULL}};
const Segment SGI318489[] = {{&S39001[8], 3}, {NULL}};
const Segment SGI318490[] = {{&S39001[13], 10}, {NULL}};
const Segment SGI318491[] = {{&S39001[42], 7}, {NULL}};
const Segment SGI318492[] = {{&S39001[35], 14}, {NULL}};
const Segment SGI318493[] = {{&S39001[73], 6}, {NULL}};
const Segment SGI318494[] = {{&S39002[30], 3}, {NULL}};
const Segment SGI318495[] = {{&S39002[8], 3}, {NULL}};
const Segment SGI318496[] = {{&S39002[13], 10}, {NULL}};
const Segment SGI318497[] = {{&S39002[42], 7}, {NULL}};
const Segment SGI318498[] = {{&S39002[35], 14}, {NULL}};
const Segment SGI318499[] = {{&S39002[73], 5}, {NULL}};
const Segment SGI318500[] = {{&S39003[30], 3}, {NULL}};
const Segment SGI318501[] = {{&S39003[8], 3}, {NULL}};
const Segment SGI318502[] = {{&S39003[13], 10}, {NULL}};
const Segment SGI318503[] = {{&S39003[42], 7}, {NULL}};
const Segment SGI318504[] = {{&S39003[35], 14}, {NULL}};
const Segment SGI318505[] = {{&S39003[138], 15}, {NULL}};
const Segment SGI318506[] = {{&S39003[178], 6}, {NULL}};
const Segment SGI318507[] = {{&S39003[51], 6}, {NULL}};
const Segment SGI318508[] = {{&S39004[30], 3}, {NULL}};
const Segment SGI318509[] = {{&S39004[8], 3}, {NULL}};
const Segment SGI318510[] = {{&S39004[13], 10}, {NULL}};
const Segment SGI318511[] = {{&S39004[42], 7}, {NULL}};
const Segment SGI318512[] = {{&S39004[35], 14}, {NULL}};
const Segment SGI318513[] = {{&S39004[80], 5}, {NULL}};
const Segment SGI318514[] = {{&S39004[51], 6}, {NULL}};
const Segment SGI318515[] = {{&S39005[30], 3}, {NULL}};
const Segment SGI318516[] = {{&S39005[8], 3}, {NULL}};
const Segment SGI318517[] = {{&S39005[13], 10}, {NULL}};
const Segment SGI318518[] = {{&S39005[42], 7}, {NULL}};
const Segment SGI318519[] = {{&S39005[35], 14}, {NULL}};
const Segment SGI318520[] = {{&S39005[131], 15}, {NULL}};
const Segment SGI318521[] = {{&S39005[171], 5}, {NULL}};
const Segment SGI318522[] = {{&S39006[30], 3}, {NULL}};
const Segment SGI318523[] = {{&S39006[8], 3}, {NULL}};
const Segment SGI318524[] = {{&S39006[13], 10}, {NULL}};
const Segment SGI318525[] = {{&S39006[42], 7}, {NULL}};
const Segment SGI318526[] = {{&S39006[35], 14}, {NULL}};
const Segment SGI318527[] = {{&S39006[80], 5}, {NULL}};
const Segment SGI318528[] = {{&S39006[51], 6}, {NULL}};
const Segment SGI318529[] = {{&S39007[30], 3}, {NULL}};
const Segment SGI318530[] = {{&S39007[8], 3}, {NULL}};
const Segment SGI318531[] = {{&S39007[13], 10}, {NULL}};
const Segment SGI318532[] = {{&S39007[42], 7}, {NULL}};
const Segment SGI318533[] = {{&S39007[35], 14}, {NULL}};
const Segment SGI318534[] = {{&S39007[138], 15}, {NULL}};
const Segment SGI318535[] = {{&S39007[188], 5}, {NULL}};
const Segment SGI318536[] = {{&S39007[51], 6}, {NULL}};
const Segment SGI318537[] = {{&S39008[30], 3}, {NULL}};
const Segment SGI318538[] = {{&S39008[8], 3}, {NULL}};
const Segment SGI318539[] = {{&S39008[13], 10}, {NULL}};
const Segment SGI318540[] = {{&S39008[42], 7}, {NULL}};
const Segment SGI318541[] = {{&S39008[35], 14}, {NULL}};
const Segment SGI318542[] = {{&S39008[80], 6}, {NULL}};
const Segment SGI318543[] = {{&S39008[51], 6}, {NULL}};
const Segment SGI318544[] = {{&S39009[30], 3}, {NULL}};
const Segment SGI318545[] = {{&S39009[8], 3}, {NULL}};
const Segment SGI318546[] = {{&S39009[13], 10}, {NULL}};
const Segment SGI318547[] = {{&S39009[42], 7}, {NULL}};
const Segment SGI318548[] = {{&S39009[35], 14}, {NULL}};
const Segment SGI318549[] = {{&S39009[73], 6}, {NULL}};
const Segment SGI318550[] = {{&S39010[30], 3}, {NULL}};
const Segment SGI318551[] = {{&S39010[8], 3}, {NULL}};
const Segment SGI318552[] = {{&S39010[13], 10}, {NULL}};
const Segment SGI318553[] = {{&S39010[42], 7}, {NULL}};
const Segment SGI318554[] = {{&S39010[35], 14}, {NULL}};
const Segment SGI318555[] = {{&S39010[73], 5}, {NULL}};
const Segment SGI318556[] = {{&S39011[30], 3}, {NULL}};
const Segment SGI318557[] = {{&S39011[8], 3}, {NULL}};
const Segment SGI318558[] = {{&S39011[13], 10}, {NULL}};
const Segment SGI318559[] = {{&S39011[42], 7}, {NULL}};
const Segment SGI318560[] = {{&S39011[35], 14}, {NULL}};
const Segment SGI318561[] = {{&S39011[131], 15}, {NULL}};
const Segment SGI318562[] = {{&S39011[171], 5}, {NULL}};
const Segment SGI318563[] = {{&S39012[30], 3}, {NULL}};
const Segment SGI318564[] = {{&S39012[8], 3}, {NULL}};
const Segment SGI318565[] = {{&S39012[13], 10}, {NULL}};
const Segment SGI318566[] = {{&S39012[42], 7}, {NULL}};
const Segment SGI318567[] = {{&S39012[35], 14}, {NULL}};
const Segment SGI318568[] = {{&S39012[138], 15}, {NULL}};
const Segment SGI318569[] = {{&S39012[178], 6}, {NULL}};
const Segment SGI318570[] = {{&S39012[51], 6}, {NULL}};
const Segment SGI318571[] = {{&S39013[30], 3}, {NULL}};
const Segment SGI318572[] = {{&S39013[8], 3}, {NULL}};
const Segment SGI318573[] = {{&S39013[13], 10}, {NULL}};
const Segment SGI318574[] = {{&S39013[42], 7}, {NULL}};
const Segment SGI318575[] = {{&S39013[35], 14}, {NULL}};
const Segment SGI318576[] = {{&S39013[80], 6}, {NULL}};
const Segment SGI318577[] = {{&S39013[51], 6}, {NULL}};
const Segment SGI318578[] = {{&S39014[30], 3}, {NULL}};
const Segment SGI318579[] = {{&S39014[8], 3}, {NULL}};
const Segment SGI318580[] = {{&S39014[13], 10}, {NULL}};
const Segment SGI318581[] = {{&S39014[42], 7}, {NULL}};
const Segment SGI318582[] = {{&S39014[35], 14}, {NULL}};
const Segment SGI318583[] = {{&S39014[73], 6}, {NULL}};
const Segment SGI318584[] = {{&S39015[30], 3}, {NULL}};
const Segment SGI318585[] = {{&S39015[8], 3}, {NULL}};
const Segment SGI318586[] = {{&S39015[13], 10}, {NULL}};
const Segment SGI318587[] = {{&S39015[42], 7}, {NULL}};
const Segment SGI318588[] = {{&S39015[35], 14}, {NULL}};
const Segment SGI318589[] = {{&S39015[80], 5}, {NULL}};
const Segment SGI318590[] = {{&S39015[51], 6}, {NULL}};
const Segment SGI318591[] = {{&S39016[30], 3}, {NULL}};
const Segment SGI318592[] = {{&S39016[8], 3}, {NULL}};
const Segment SGI318593[] = {{&S39016[13], 10}, {NULL}};
const Segment SGI318594[] = {{&S39016[42], 7}, {NULL}};
const Segment SGI318595[] = {{&S39016[35], 14}, {NULL}};
const Segment SGI318596[] = {{&S39016[73], 6}, {NULL}};
const Segment SGI318597[] = {{&S39017[30], 3}, {NULL}};
const Segment SGI318598[] = {{&S39017[8], 3}, {NULL}};
const Segment SGI318599[] = {{&S39017[13], 10}, {NULL}};
const Segment SGI318600[] = {{&S39017[42], 7}, {NULL}};
const Segment SGI318601[] = {{&S39017[35], 14}, {NULL}};
const Segment SGI318602[] = {{&S39017[80], 6}, {NULL}};
const Segment SGI318603[] = {{&S39017[51], 6}, {NULL}};
const Segment SGI318604[] = {{&S39018[30], 3}, {NULL}};
const Segment SGI318605[] = {{&S39018[8], 3}, {NULL}};
const Segment SGI318606[] = {{&S39018[13], 10}, {NULL}};
const Segment SGI318607[] = {{&S39018[42], 7}, {NULL}};
const Segment SGI318608[] = {{&S39018[35], 14}, {NULL}};
const Segment SGI318609[] = {{&S39018[138], 15}, {NULL}};
const Segment SGI318610[] = {{&S39018[178], 6}, {NULL}};
const Segment SGI318611[] = {{&S39018[51], 6}, {NULL}};
const Segment SGI318612[] = {{&S39019[30], 3}, {NULL}};
const Segment SGI318613[] = {{&S39019[8], 3}, {NULL}};
const Segment SGI318614[] = {{&S39019[13], 10}, {NULL}};
const Segment SGI318615[] = {{&S39019[42], 7}, {NULL}};
const Segment SGI318616[] = {{&S39019[35], 14}, {NULL}};
const Segment SGI318617[] = {{&S39019[138], 15}, {NULL}};
const Segment SGI318618[] = {{&S39019[178], 6}, {NULL}};
const Segment SGI318619[] = {{&S39019[51], 6}, {NULL}};
const Segment SGI318620[] = {{&S39020[30], 3}, {NULL}};
const Segment SGI318621[] = {{&S39020[8], 3}, {NULL}};
const Segment SGI318622[] = {{&S39020[13], 10}, {NULL}};
const Segment SGI318623[] = {{&S39020[42], 7}, {NULL}};
const Segment SGI318624[] = {{&S39020[35], 14}, {NULL}};
const Segment SGI318625[] = {{&S39020[131], 15}, {NULL}};
const Segment SGI318626[] = {{&S39020[161], 6}, {NULL}};
const Segment SGI318627[] = {{&S39021[30], 3}, {NULL}};
const Segment SGI318628[] = {{&S39021[8], 3}, {NULL}};
const Segment SGI318629[] = {{&S39021[13], 10}, {NULL}};
const Segment SGI318630[] = {{&S39021[42], 7}, {NULL}};
const Segment SGI318631[] = {{&S39021[35], 14}, {NULL}};
const Segment SGI318632[] = {{&S39021[131], 15}, {NULL}};
const Segment SGI318633[] = {{&S39021[171], 6}, {NULL}};
const Segment SGI318634[] = {{&S39022[30], 3}, {NULL}};
const Segment SGI318635[] = {{&S39022[8], 3}, {NULL}};
const Segment SGI318636[] = {{&S39022[13], 10}, {NULL}};
const Segment SGI318637[] = {{&S39022[42], 7}, {NULL}};
const Segment SGI318638[] = {{&S39022[35], 14}, {NULL}};
const Segment SGI318639[] = {{&S39022[138], 15}, {NULL}};
const Segment SGI318640[] = {{&S39022[185], 5}, {NULL}};
const Segment SGI318641[] = {{&S39022[51], 6}, {NULL}};
const Segment SGI318642[] = {{&S39023[30], 3}, {NULL}};
const Segment SGI318643[] = {{&S39023[8], 3}, {NULL}};
const Segment SGI318644[] = {{&S39023[13], 10}, {NULL}};
const Segment SGI318645[] = {{&S39023[42], 7}, {NULL}};
const Segment SGI318646[] = {{&S39023[35], 14}, {NULL}};
const Segment SGI318647[] = {{&S39023[131], 15}, {NULL}};
const Segment SGI318648[] = {{&S39023[161], 6}, {NULL}};
const Segment SGI318649[] = {{&S39024[30], 3}, {NULL}};
const Segment SGI318650[] = {{&S39024[8], 3}, {NULL}};
const Segment SGI318651[] = {{&S39024[13], 10}, {NULL}};
const Segment SGI318652[] = {{&S39024[42], 7}, {NULL}};
const Segment SGI318653[] = {{&S39024[35], 14}, {NULL}};
const Segment SGI318654[] = {{&S39024[73], 6}, {NULL}};
const Segment SGI318655[] = {{&S39025[30], 3}, {NULL}};
const Segment SGI318656[] = {{&S39025[8], 3}, {NULL}};
const Segment SGI318657[] = {{&S39025[13], 10}, {NULL}};
const Segment SGI318658[] = {{&S39025[42], 7}, {NULL}};
const Segment SGI318659[] = {{&S39025[35], 14}, {NULL}};
const Segment SGI318660[] = {{&S39025[80], 6}, {NULL}};
const Segment SGI318661[] = {{&S39025[51], 6}, {NULL}};
const Segment SGI318662[] = {{&S39026[30], 3}, {NULL}};
const Segment SGI318663[] = {{&S39026[8], 3}, {NULL}};
const Segment SGI318664[] = {{&S39026[13], 10}, {NULL}};
const Segment SGI318665[] = {{&S39026[42], 7}, {NULL}};
const Segment SGI318666[] = {{&S39026[35], 14}, {NULL}};
const Segment SGI318667[] = {{&S39026[138], 15}, {NULL}};
const Segment SGI318668[] = {{&S39026[178], 6}, {NULL}};
const Segment SGI318669[] = {{&S39026[51], 6}, {NULL}};
const Segment SGI318670[] = {{&S39027[30], 3}, {NULL}};
const Segment SGI318671[] = {{&S39027[8], 3}, {NULL}};
const Segment SGI318672[] = {{&S39027[13], 10}, {NULL}};
const Segment SGI318673[] = {{&S39027[42], 7}, {NULL}};
const Segment SGI318674[] = {{&S39027[35], 14}, {NULL}};
const Segment SGI318675[] = {{&S39027[138], 15}, {NULL}};
const Segment SGI318676[] = {{&S39027[180], 5}, {NULL}};
const Segment SGI318677[] = {{&S39027[51], 6}, {NULL}};
const Segment SGI318678[] = {{&S39028[30], 3}, {NULL}};
const Segment SGI318679[] = {{&S39028[8], 3}, {NULL}};
const Segment SGI318680[] = {{&S39028[13], 10}, {NULL}};
const Segment SGI318681[] = {{&S39028[42], 7}, {NULL}};
const Segment SGI318682[] = {{&S39028[35], 14}, {NULL}};
const Segment SGI318683[] = {{&S39028[131], 15}, {NULL}};
const Segment SGI318684[] = {{&S39028[171], 6}, {NULL}};
const Segment SGI318685[] = {{&S39029[30], 4}, {NULL}};
const Segment SGI318686[] = {{&S39029[8], 3}, {NULL}};
const Segment SGI318687[] = {{&S39029[13], 10}, {NULL}};
const Segment SGI318688[] = {{&S39029[43], 7}, {NULL}};
const Segment SGI318689[] = {{&S39029[36], 14}, {NULL}};
const Segment SGI318690[] = {{&S39029[81], 6}, {NULL}};
const Segment SGI318691[] = {{&S39029[52], 6}, {NULL}};
const Segment SGI318692[] = {{&S39030[30], 4}, {NULL}};
const Segment SGI318693[] = {{&S39030[8], 3}, {NULL}};
const Segment SGI318694[] = {{&S39030[13], 10}, {NULL}};
const Segment SGI318695[] = {{&S39030[43], 7}, {NULL}};
const Segment SGI318696[] = {{&S39030[36], 14}, {NULL}};
const Segment SGI318697[] = {{&S39030[74], 6}, {NULL}};
const Segment SGI318698[] = {{&S39031[30], 4}, {NULL}};
const Segment SGI318699[] = {{&S39031[8], 3}, {NULL}};
const Segment SGI318700[] = {{&S39031[13], 10}, {NULL}};
const Segment SGI318701[] = {{&S39031[43], 7}, {NULL}};
const Segment SGI318702[] = {{&S39031[36], 14}, {NULL}};
const Segment SGI318703[] = {{&S39031[81], 6}, {NULL}};
const Segment SGI318704[] = {{&S39031[52], 6}, {NULL}};
const Segment SGI318705[] = {{&S39032[30], 3}, {NULL}};
const Segment SGI318706[] = {{&S39032[8], 3}, {NULL}};
const Segment SGI318707[] = {{&S39032[13], 10}, {NULL}};
const Segment SGI318708[] = {{&S39032[42], 7}, {NULL}};
const Segment SGI318709[] = {{&S39032[35], 14}, {NULL}};
const Segment SGI318710[] = {{&S39032[80], 6}, {NULL}};
const Segment SGI318711[] = {{&S39032[51], 6}, {NULL}};
const Segment SGI318712[] = {{&S39033[30], 3}, {NULL}};
const Segment SGI318713[] = {{&S39033[8], 3}, {NULL}};
const Segment SGI318714[] = {{&S39033[13], 10}, {NULL}};
const Segment SGI318715[] = {{&S39033[42], 7}, {NULL}};
const Segment SGI318716[] = {{&S39033[35], 14}, {NULL}};
const Segment SGI318717[] = {{&S39033[73], 6}, {NULL}};
const Segment SGI318718[] = {{&S39034[30], 4}, {NULL}};
const Segment SGI318719[] = {{&S39034[8], 3}, {NULL}};
const Segment SGI318720[] = {{&S39034[13], 10}, {NULL}};
const Segment SGI318721[] = {{&S39034[43], 7}, {NULL}};
const Segment SGI318722[] = {{&S39034[36], 14}, {NULL}};
const Segment SGI318723[] = {{&S39034[81], 6}, {NULL}};
const Segment SGI318724[] = {{&S39034[52], 6}, {NULL}};
const Segment SGI318725[] = {{&S39035[30], 4}, {NULL}};
const Segment SGI318726[] = {{&S39035[8], 3}, {NULL}};
const Segment SGI318727[] = {{&S39035[13], 10}, {NULL}};
const Segment SGI318728[] = {{&S39035[43], 7}, {NULL}};
const Segment SGI318729[] = {{&S39035[36], 14}, {NULL}};
const Segment SGI318730[] = {{&S39035[74], 6}, {NULL}};
const Segment SGI318731[] = {{&S39036[30], 3}, {NULL}};
const Segment SGI318732[] = {{&S39036[8], 3}, {NULL}};
const Segment SGI318733[] = {{&S39036[13], 10}, {NULL}};
const Segment SGI318734[] = {{&S39036[42], 7}, {NULL}};
const Segment SGI318735[] = {{&S39036[35], 14}, {NULL}};
const Segment SGI318736[] = {{&S39036[138], 15}, {NULL}};
const Segment SGI318737[] = {{&S39036[188], 6}, {NULL}};
const Segment SGI318738[] = {{&S39036[51], 6}, {NULL}};
const Segment SGI318739[] = {{&S39037[30], 4}, {NULL}};
const Segment SGI318740[] = {{&S39037[8], 3}, {NULL}};
const Segment SGI318741[] = {{&S39037[13], 10}, {NULL}};
const Segment SGI318742[] = {{&S39037[43], 7}, {NULL}};
const Segment SGI318743[] = {{&S39037[36], 14}, {NULL}};
const Segment SGI318744[] = {{&S39037[81], 6}, {NULL}};
const Segment SGI318745[] = {{&S39037[52], 6}, {NULL}};
const Segment SGI318746[] = {{&S39038[30], 3}, {NULL}};
const Segment SGI318747[] = {{&S39038[8], 3}, {NULL}};
const Segment SGI318748[] = {{&S39038[13], 10}, {NULL}};
const Segment SGI318749[] = {{&S39038[42], 7}, {NULL}};
const Segment SGI318750[] = {{&S39038[35], 14}, {NULL}};
const Segment SGI318751[] = {{&S39038[80], 6}, {NULL}};
const Segment SGI318752[] = {{&S39038[51], 6}, {NULL}};
const Segment SGI318753[] = {{&S39039[30], 4}, {NULL}};
const Segment SGI318754[] = {{&S39039[8], 3}, {NULL}};
const Segment SGI318755[] = {{&S39039[13], 10}, {NULL}};
const Segment SGI318756[] = {{&S39039[43], 7}, {NULL}};
const Segment SGI318757[] = {{&S39039[36], 14}, {NULL}};
const Segment SGI318758[] = {{&S39039[74], 6}, {NULL}};
const Segment SGI318759[] = {{&S39040[30], 3}, {NULL}};
const Segment SGI318760[] = {{&S39040[8], 3}, {NULL}};
const Segment SGI318761[] = {{&S39040[13], 10}, {NULL}};
const Segment SGI318762[] = {{&S39040[42], 7}, {NULL}};
const Segment SGI318763[] = {{&S39040[35], 14}, {NULL}};
const Segment SGI318764[] = {{&S39040[138], 15}, {NULL}};
const Segment SGI318765[] = {{&S39040[178], 6}, {NULL}};
const Segment SGI318766[] = {{&S39040[51], 6}, {NULL}};
const Segment SGI318767[] = {{&S39041[30], 4}, {NULL}};
const Segment SGI318768[] = {{&S39041[8], 3}, {NULL}};
const Segment SGI318769[] = {{&S39041[13], 10}, {NULL}};
const Segment SGI318770[] = {{&S39041[43], 7}, {NULL}};
const Segment SGI318771[] = {{&S39041[36], 14}, {NULL}};
const Segment SGI318772[] = {{&S39041[74], 6}, {NULL}};
const Segment SGI318773[] = {{&S39042[30], 3}, {NULL}};
const Segment SGI318774[] = {{&S39042[8], 3}, {NULL}};
const Segment SGI318775[] = {{&S39042[13], 10}, {NULL}};
const Segment SGI318776[] = {{&S39042[42], 7}, {NULL}};
const Segment SGI318777[] = {{&S39042[35], 14}, {NULL}};
const Segment SGI318778[] = {{&S39042[73], 6}, {NULL}};
const Segment SGI318779[] = {{&S39043[30], 4}, {NULL}};
const Segment SGI318780[] = {{&S39043[8], 3}, {NULL}};
const Segment SGI318781[] = {{&S39043[13], 10}, {NULL}};
const Segment SGI318782[] = {{&S39043[43], 7}, {NULL}};
const Segment SGI318783[] = {{&S39043[36], 14}, {NULL}};
const Segment SGI318784[] = {{&S39043[81], 6}, {NULL}};
const Segment SGI318785[] = {{&S39043[52], 6}, {NULL}};
const Segment SGI318786[] = {{&S39044[30], 4}, {NULL}};
const Segment SGI318787[] = {{&S39044[8], 3}, {NULL}};
const Segment SGI318788[] = {{&S39044[13], 10}, {NULL}};
const Segment SGI318789[] = {{&S39044[43], 7}, {NULL}};
const Segment SGI318790[] = {{&S39044[36], 14}, {NULL}};
const Segment SGI318791[] = {{&S39044[81], 5}, {NULL}};
const Segment SGI318792[] = {{&S39044[52], 6}, {NULL}};
const Segment SGI318793[] = {{&S39045[30], 4}, {NULL}};
const Segment SGI318794[] = {{&S39045[8], 3}, {NULL}};
const Segment SGI318795[] = {{&S39045[13], 10}, {NULL}};
const Segment SGI318796[] = {{&S39045[43], 7}, {NULL}};
const Segment SGI318797[] = {{&S39045[36], 14}, {NULL}};
const Segment SGI318798[] = {{&S39045[81], 5}, {NULL}};
const Segment SGI318799[] = {{&S39045[52], 6}, {NULL}};
const Segment SGI318800[] = {{&S39046[30], 4}, {NULL}};
const Segment SGI318801[] = {{&S39046[8], 3}, {NULL}};
const Segment SGI318802[] = {{&S39046[13], 10}, {NULL}};
const Segment SGI318803[] = {{&S39046[43], 7}, {NULL}};
const Segment SGI318804[] = {{&S39046[36], 14}, {NULL}};
const Segment SGI318805[] = {{&S39046[81], 6}, {NULL}};
const Segment SGI318806[] = {{&S39046[52], 6}, {NULL}};
const Segment SGI318807[] = {{&S39047[30], 4}, {NULL}};
const Segment SGI318808[] = {{&S39047[8], 3}, {NULL}};
const Segment SGI318809[] = {{&S39047[13], 10}, {NULL}};
const Segment SGI318810[] = {{&S39047[43], 7}, {NULL}};
const Segment SGI318811[] = {{&S39047[36], 14}, {NULL}};
const Segment SGI318812[] = {{&S39047[74], 6}, {NULL}};
const Segment SGI318813[] = {{&S39048[30], 4}, {NULL}};
const Segment SGI318814[] = {{&S39048[8], 3}, {NULL}};
const Segment SGI318815[] = {{&S39048[13], 10}, {NULL}};
const Segment SGI318816[] = {{&S39048[43], 7}, {NULL}};
const Segment SGI318817[] = {{&S39048[36], 14}, {NULL}};
const Segment SGI318818[] = {{&S39048[81], 6}, {NULL}};
const Segment SGI318819[] = {{&S39048[52], 6}, {NULL}};
const Segment SGI318820[] = {{&S39049[30], 3}, {NULL}};
const Segment SGI318821[] = {{&S39049[8], 3}, {NULL}};
const Segment SGI318822[] = {{&S39049[13], 10}, {NULL}};
const Segment SGI318823[] = {{&S39049[42], 7}, {NULL}};
const Segment SGI318824[] = {{&S39049[35], 14}, {NULL}};
const Segment SGI318825[] = {{&S39049[80], 6}, {NULL}};
const Segment SGI318826[] = {{&S39049[51], 6}, {NULL}};
const Segment SGI318827[] = {{&S39050[30], 3}, {NULL}};
const Segment SGI318828[] = {{&S39050[8], 3}, {NULL}};
const Segment SGI318829[] = {{&S39050[13], 10}, {NULL}};
const Segment SGI318830[] = {{&S39050[42], 7}, {NULL}};
const Segment SGI318831[] = {{&S39050[35], 14}, {NULL}};
const Segment SGI318832[] = {{&S39050[73], 6}, {NULL}};
const Segment SGI318833[] = {{&S39051[30], 4}, {NULL}};
const Segment SGI318834[] = {{&S39051[8], 3}, {NULL}};
const Segment SGI318835[] = {{&S39051[13], 10}, {NULL}};
const Segment SGI318836[] = {{&S39051[43], 7}, {NULL}};
const Segment SGI318837[] = {{&S39051[36], 14}, {NULL}};
const Segment SGI318838[] = {{&S39051[74], 5}, {NULL}};
const Segment SGI318839[] = {{&S39052[30], 4}, {NULL}};
const Segment SGI318840[] = {{&S39052[8], 3}, {NULL}};
const Segment SGI318841[] = {{&S39052[13], 10}, {NULL}};
const Segment SGI318842[] = {{&S39052[43], 7}, {NULL}};
const Segment SGI318843[] = {{&S39052[36], 14}, {NULL}};
const Segment SGI318844[] = {{&S39052[81], 5}, {NULL}};
const Segment SGI318845[] = {{&S39052[52], 6}, {NULL}};
const Segment SGI318846[] = {{&S39053[30], 4}, {NULL}};
const Segment SGI318847[] = {{&S39053[8], 3}, {NULL}};
const Segment SGI318848[] = {{&S39053[13], 10}, {NULL}};
const Segment SGI318849[] = {{&S39053[43], 7}, {NULL}};
const Segment SGI318850[] = {{&S39053[36], 14}, {NULL}};
const Segment SGI318851[] = {{&S39053[74], 6}, {NULL}};
const Segment SGI318852[] = {{&S56196[3], 6}, {NULL}};
const Segment SGI318853[] = {{&S56197[3], 6}, {NULL}};
const Segment SGI318854[] = {{&S56198[3], 5}, {NULL}};
const Segment SGI318855[] = {{&S39319[30], 3}, {NULL}};
const Segment SGI318856[] = {{&S39319[8], 3}, {NULL}};
const Segment SGI318857[] = {{&S39319[13], 10}, {NULL}};
const Segment SGI318858[] = {{&S39319[42], 4}, {NULL}};
const Segment SGI318859[] = {{&S39320[30], 3}, {NULL}};
const Segment SGI318860[] = {{&S39320[8], 3}, {NULL}};
const Segment SGI318861[] = {{&S39320[13], 10}, {NULL}};
const Segment SGI318862[] = {{&S39320[35], 6}, {NULL}};
const Segment SGI318863[] = {{&S39321[8], 3}, {NULL}};
const Segment SGI318864[] = {{&S39321[13], 10}, {NULL}};
const Segment SGI318865[] = {{&S39321[35], 6}, {NULL}};
const Segment SGI318866[] = {{&S39341[30], 3}, {NULL}};
const Segment SGI318867[] = {{&S39341[8], 3}, {NULL}};
const Segment SGI318868[] = {{&S39341[13], 10}, {NULL}};
const Segment SGI318869[] = {{&S39341[42], 7}, {NULL}};
const Segment SGI318870[] = {{&S39341[35], 14}, {NULL}};
const Segment SGI318871[] = {{&S39341[64], 11}, {NULL}};
const Segment SGI318872[] = {{&S39342[30], 3}, {&S39342[94], 3}, {NULL}};
const Segment SGI318873[] = {{&S39342[8], 3}, {&S39342[8], 3}, {NULL}};
const Segment SGI318874[] = {{&S39342[13], 10}, {&S39342[13], 10}, {NULL}};
const Segment SGI318875[] = {{&S39342[42], 7}, {&S39342[42], 7}, {NULL}};
const Segment SGI318876[] = {{&S39342[35], 14}, {&S39342[35], 14}, {NULL}};
const Segment SGI318877[] = {{&S39342[122], 11}, {NULL}};
const Segment SGI318878[] = {{&S39342[115], 3}, {NULL}};
const Segment SGI318879[] = {{&S39343[30], 3}, {NULL}};
const Segment SGI318880[] = {{&S39343[8], 3}, {NULL}};
const Segment SGI318881[] = {{&S39343[13], 10}, {NULL}};
const Segment SGI318882[] = {{&S39343[42], 7}, {NULL}};
const Segment SGI318883[] = {{&S39343[35], 14}, {NULL}};
const Segment SGI318884[] = {{&S39343[64], 11}, {NULL}};
const Segment SGI318885[] = {{&S39343[115], 15}, {NULL}};
const Segment SGI318886[] = {{&S39344[30], 3}, {&S39344[94], 3}, {NULL}};
const Segment SGI318887[] = {{&S39344[8], 3}, {&S39344[8], 3}, {NULL}};
const Segment SGI318888[] = {{&S39344[13], 10}, {&S39344[13], 10}, {NULL}};
const Segment SGI318889[] = {{&S39344[42], 7}, {&S39344[42], 7}, {NULL}};
const Segment SGI318890[] = {{&S39344[35], 14}, {&S39344[35], 14}, {NULL}};
const Segment SGI318891[] = {{&S39344[122], 11}, {NULL}};
const Segment SGI318892[] = {{&S39344[115], 3}, {NULL}};
const Segment SGI318893[] = {{&S39345[30], 3}, {NULL}};
const Segment SGI318894[] = {{&S39345[8], 3}, {NULL}};
const Segment SGI318895[] = {{&S39345[13], 10}, {NULL}};
const Segment SGI318896[] = {{&S39345[42], 7}, {NULL}};
const Segment SGI318897[] = {{&S39345[35], 14}, {NULL}};
const Segment SGI318898[] = {{&S39345[64], 11}, {NULL}};
const Segment SGI318899[] = {{&S39364[30], 3}, {NULL}};
const Segment SGI318900[] = {{&S39364[8], 3}, {NULL}};
const Segment SGI318901[] = {{&S39364[13], 10}, {NULL}};
const Segment SGI318902[] = {{&S39364[35], 5}, {NULL}};
const Segment SGI318903[] = {{&S39365[30], 3}, {NULL}};
const Segment SGI318904[] = {{&S39365[8], 3}, {NULL}};
const Segment SGI318905[] = {{&S39365[13], 10}, {NULL}};
const Segment SGI318906[] = {{&S39365[42], 3}, {NULL}};
const Segment SGI318907[] = {{&S39366[30], 4}, {NULL}};
const Segment SGI318908[] = {{&S39366[8], 3}, {NULL}};
const Segment SGI318909[] = {{&S39366[13], 10}, {NULL}};
const Segment SGI318910[] = {{&S39366[43], 3}, {NULL}};
const Segment SGI318911[] = {{&S39367[30], 3}, {NULL}};
const Segment SGI318912[] = {{&S39367[8], 3}, {NULL}};
const Segment SGI318913[] = {{&S39367[13], 10}, {NULL}};
const Segment SGI318914[] = {{&S39367[42], 7}, {NULL}};
const Segment SGI318915[] = {{&S39368[30], 3}, {NULL}};
const Segment SGI318916[] = {{&S39368[8], 3}, {NULL}};
const Segment SGI318917[] = {{&S39368[13], 10}, {NULL}};
const Segment SGI318918[] = {{&S39368[34], 5}, {NULL}};
const Segment SGI318919[] = {{&S39369[30], 3}, {NULL}};
const Segment SGI318920[] = {{&S39369[8], 3}, {NULL}};
const Segment SGI318921[] = {{&S39369[13], 10}, {NULL}};
const Segment SGI318922[] = {{&S39369[42], 7}, {NULL}};
const Segment SGI318923[] = {{&S39369[35], 14}, {NULL}};
const Segment SGI318924[] = {{&S39369[131], 15}, {NULL}};
const Segment SGI318925[] = {{&S39370[29], 3}, {NULL}};
const Segment SGI318926[] = {{&S39370[8], 3}, {NULL}};
const Segment SGI318927[] = {{&S39370[13], 10}, {NULL}};
const Segment SGI318928[] = {{&S39370[42], 7}, {NULL}};
const Segment SGI318929[] = {{&S39370[35], 14}, {NULL}};
const Segment SGI318930[] = {{&S39371[29], 3}, {NULL}};
const Segment SGI318931[] = {{&S39371[8], 3}, {NULL}};
const Segment SGI318932[] = {{&S39371[13], 10}, {NULL}};
const Segment SGI318933[] = {{&S39371[42], 7}, {NULL}};
const Segment SGI318934[] = {{&S39371[35], 14}, {NULL}};
const Segment SGI318935[] = {{&S39372[29], 3}, {NULL}};
const Segment SGI318936[] = {{&S39372[8], 3}, {NULL}};
const Segment SGI318937[] = {{&S39372[13], 10}, {NULL}};
const Segment SGI318938[] = {{&S39372[42], 7}, {NULL}};
const Segment SGI318939[] = {{&S39372[35], 14}, {NULL}};
const Segment SGI318940[] = {{&S39373[29], 3}, {NULL}};
const Segment SGI318941[] = {{&S39373[8], 3}, {NULL}};
const Segment SGI318942[] = {{&S39373[13], 10}, {NULL}};
const Segment SGI318943[] = {{&S39373[42], 7}, {NULL}};
const Segment SGI318944[] = {{&S39373[35], 14}, {NULL}};
const Segment SGI318945[] = {{&S39374[30], 3}, {NULL}};
const Segment SGI318946[] = {{&S39374[8], 3}, {NULL}};
const Segment SGI318947[] = {{&S39374[13], 10}, {NULL}};
const Segment SGI318948[] = {{&S39375[30], 3}, {NULL}};
const Segment SGI318949[] = {{&S39375[8], 3}, {NULL}};
const Segment SGI318950[] = {{&S39375[13], 10}, {NULL}};
const Segment SGI318951[] = {{&S39376[30], 3}, {NULL}};
const Segment SGI318952[] = {{&S39376[8], 3}, {NULL}};
const Segment SGI318953[] = {{&S39376[13], 10}, {NULL}};
const Segment SGI318954[] = {{&S39376[40], 3}, {NULL}};
const Segment SGI318955[] = {{&S39377[30], 3}, {NULL}};
const Segment SGI318956[] = {{&S39377[8], 3}, {NULL}};
const Segment SGI318957[] = {{&S39377[13], 10}, {NULL}};
const Segment SGI318958[] = {{&S39377[40], 5}, {NULL}};
const Segment SGI318959[] = {{&S39378[30], 3}, {NULL}};
const Segment SGI318960[] = {{&S39378[8], 3}, {NULL}};
const Segment SGI318961[] = {{&S39378[13], 10}, {NULL}};
const Segment SGI318962[] = {{&S39378[40], 6}, {NULL}};
const Segment SGI318963[] = {{&S39378[35], 3}, {NULL}};
const Segment SGI318964[] = {{&S39379[30], 3}, {NULL}};
const Segment SGI318965[] = {{&S39379[8], 3}, {NULL}};
const Segment SGI318966[] = {{&S39379[13], 10}, {NULL}};
const Segment SGI318967[] = {{&S39379[40], 6}, {NULL}};
const Segment SGI318968[] = {{&S39380[30], 3}, {NULL}};
const Segment SGI318969[] = {{&S39380[8], 3}, {NULL}};
const Segment SGI318970[] = {{&S39380[13], 10}, {NULL}};
const Segment SGI318971[] = {{&S39380[35], 3}, {NULL}};
const Segment SGI318972[] = {{&S39381[30], 3}, {NULL}};
const Segment SGI318973[] = {{&S39381[8], 3}, {NULL}};
const Segment SGI318974[] = {{&S39381[13], 10}, {NULL}};
const Segment SGI318975[] = {{&S39381[40], 5}, {NULL}};
const Segment SGI318976[] = {{&S39381[35], 3}, {NULL}};
const Segment SGI318977[] = {{&S39403[30], 3}, {NULL}};
const Segment SGI318978[] = {{&S39403[8], 3}, {NULL}};
const Segment SGI318979[] = {{&S39403[13], 10}, {NULL}};
const Segment SGI318980[] = {{&S39403[42], 7}, {NULL}};
const Segment SGI318981[] = {{&S39403[35], 14}, {NULL}};
const Segment SGI318982[] = {{&S39403[71], 11}, {NULL}};
const Segment SGI318983[] = {{&S39403[51], 6}, {NULL}};
const Segment SGI318984[] = {{&S39403[71], 16}, {NULL}};
const Segment SGI318985[] = {{&S39404[30], 3}, {NULL}};
const Segment SGI318986[] = {{&S39404[8], 3}, {NULL}};
const Segment SGI318987[] = {{&S39404[13], 10}, {NULL}};
const Segment SGI318988[] = {{&S39404[42], 7}, {NULL}};
const Segment SGI318989[] = {{&S39404[35], 14}, {NULL}};
const Segment SGI318990[] = {{&S39404[64], 11}, {NULL}};
const Segment SGI318991[] = {{&S39404[156], 15}, {NULL}};
const Segment SGI318992[] = {{&S39404[64], 16}, {NULL}};
const Segment SGI318993[] = {{&S39404[189], 9}, {NULL}};
const Segment SGI318994[] = {{&S39407[30], 3}, {NULL}};
const Segment SGI318995[] = {{&S39407[8], 3}, {NULL}};
const Segment SGI318996[] = {{&S39407[13], 10}, {NULL}};
const Segment SGI318997[] = {{&S39407[42], 7}, {NULL}};
const Segment SGI318998[] = {{&S39407[35], 14}, {NULL}};
const Segment SGI318999[] = {{&S39407[64], 11}, {NULL}};
const Segment SGI319000[] = {{&S39407[64], 16}, {NULL}};
const Segment SGI319001[] = {{&S39410[30], 3}, {NULL}};
const Segment SGI319002[] = {{&S39410[8], 3}, {NULL}};
const Segment SGI319003[] = {{&S39410[13], 10}, {NULL}};
const Segment SGI319004[] = {{&S39410[42], 7}, {NULL}};
const Segment SGI319005[] = {{&S39410[35], 14}, {NULL}};
const Segment SGI319006[] = {{&S39410[71], 11}, {NULL}};
const Segment SGI319007[] = {{&S39410[51], 6}, {NULL}};
const Segment SGI319008[] = {{&S39410[71], 16}, {NULL}};
const Segment SGI319009[] = {{&S39411[30], 3}, {NULL}};
const Segment SGI319010[] = {{&S39411[8], 3}, {NULL}};
const Segment SGI319011[] = {{&S39411[13], 10}, {NULL}};
const Segment SGI319012[] = {{&S39411[42], 7}, {NULL}};
const Segment SGI319013[] = {{&S39411[35], 14}, {NULL}};
const Segment SGI319014[] = {{&S39411[64], 11}, {NULL}};
const Segment SGI319015[] = {{&S39411[156], 15}, {NULL}};
const Segment SGI319016[] = {{&S39411[64], 16}, {NULL}};
const Segment SGI319017[] = {{&S39412[30], 3}, {NULL}};
const Segment SGI319018[] = {{&S39412[8], 3}, {NULL}};
const Segment SGI319019[] = {{&S39412[13], 10}, {NULL}};
const Segment SGI319020[] = {{&S39412[42], 7}, {NULL}};
const Segment SGI319021[] = {{&S39412[35], 14}, {NULL}};
const Segment SGI319022[] = {{&S39412[64], 11}, {NULL}};
const Segment SGI319023[] = {{&S39412[64], 16}, {NULL}};
const Segment SGI319024[] = {{&S39413[30], 3}, {NULL}};
const Segment SGI319025[] = {{&S39413[8], 3}, {NULL}};
const Segment SGI319026[] = {{&S39413[13], 10}, {NULL}};
const Segment SGI319027[] = {{&S39413[42], 7}, {NULL}};
const Segment SGI319028[] = {{&S39413[35], 14}, {NULL}};
const Segment SGI319029[] = {{&S39413[71], 11}, {NULL}};
const Segment SGI319030[] = {{&S39413[163], 15}, {NULL}};
const Segment SGI319031[] = {{&S39413[51], 6}, {NULL}};
const Segment SGI319032[] = {{&S39413[71], 16}, {NULL}};
const Segment SGI319033[] = {{&S39414[30], 3}, {NULL}};
const Segment SGI319034[] = {{&S39414[8], 3}, {NULL}};
const Segment SGI319035[] = {{&S39414[13], 10}, {NULL}};
const Segment SGI319036[] = {{&S39414[42], 7}, {NULL}};
const Segment SGI319037[] = {{&S39414[35], 14}, {NULL}};
const Segment SGI319038[] = {{&S39414[172], 11}, {NULL}};
const Segment SGI319039[] = {{&S39414[131], 15}, {NULL}};
const Segment SGI319040[] = {{&S39414[172], 16}, {NULL}};
const Segment SGI319041[] = {{&S39416[30], 3}, {NULL}};
const Segment SGI319042[] = {{&S39416[8], 3}, {NULL}};
const Segment SGI319043[] = {{&S39416[13], 10}, {NULL}};
const Segment SGI319044[] = {{&S39416[42], 7}, {NULL}};
const Segment SGI319045[] = {{&S39416[35], 14}, {NULL}};
const Segment SGI319046[] = {{&S39416[64], 11}, {NULL}};
const Segment SGI319047[] = {{&S39416[64], 16}, {NULL}};
const Segment SGI319048[] = {{&S39416[156], 9}, {NULL}};
const Segment SGI319049[] = {{&S39417[30], 3}, {NULL}};
const Segment SGI319050[] = {{&S39417[8], 3}, {NULL}};
const Segment SGI319051[] = {{&S39417[13], 10}, {NULL}};
const Segment SGI319052[] = {{&S39417[42], 7}, {NULL}};
const Segment SGI319053[] = {{&S39417[35], 14}, {NULL}};
const Segment SGI319054[] = {{&S39417[71], 11}, {NULL}};
const Segment SGI319055[] = {{&S39417[164], 15}, {NULL}};
const Segment SGI319056[] = {{&S39417[51], 6}, {NULL}};
const Segment SGI319057[] = {{&S39417[71], 16}, {NULL}};
const Segment SGI319058[] = {{&S39418[30], 3}, {NULL}};
const Segment SGI319059[] = {{&S39418[8], 3}, {NULL}};
const Segment SGI319060[] = {{&S39418[13], 10}, {NULL}};
const Segment SGI319061[] = {{&S39418[42], 7}, {NULL}};
const Segment SGI319062[] = {{&S39418[35], 14}, {NULL}};
const Segment SGI319063[] = {{&S39418[64], 11}, {NULL}};
const Segment SGI319064[] = {{&S39418[64], 16}, {NULL}};
const Segment SGI319065[] = {{&S39419[30], 3}, {NULL}};
const Segment SGI319066[] = {{&S39419[174], 9}, {NULL}};
const Segment SGI319067[] = {{&S39419[8], 3}, {NULL}};
const Segment SGI319068[] = {{&S39419[13], 10}, {NULL}};
const Segment SGI319069[] = {{&S39419[42], 7}, {NULL}};
const Segment SGI319070[] = {{&S39419[35], 14}, {NULL}};
const Segment SGI319071[] = {{&S39419[64], 11}, {NULL}};
const Segment SGI319072[] = {{&S39419[64], 16}, {NULL}};
const Segment SGI319073[] = {{&S39420[30], 3}, {NULL}};
const Segment SGI319074[] = {{&S39420[8], 3}, {NULL}};
const Segment SGI319075[] = {{&S39420[13], 10}, {NULL}};
const Segment SGI319076[] = {{&S39420[42], 7}, {NULL}};
const Segment SGI319077[] = {{&S39420[35], 14}, {NULL}};
const Segment SGI319078[] = {{&S39420[64], 11}, {NULL}};
const Segment SGI319079[] = {{&S39420[64], 16}, {NULL}};
const Segment SGI319080[] = {{&S39420[156], 9}, {NULL}};
const Segment SGI319081[] = {{&S39421[30], 3}, {NULL}};
const Segment SGI319082[] = {{&S39421[235], 9}, {NULL}};
const Segment SGI319083[] = {{&S39421[8], 3}, {NULL}};
const Segment SGI319084[] = {{&S39421[13], 10}, {NULL}};
const Segment SGI319085[] = {{&S39421[42], 7}, {NULL}};
const Segment SGI319086[] = {{&S39421[35], 14}, {NULL}};
const Segment SGI319087[] = {{&S39421[71], 11}, {NULL}};
const Segment SGI319088[] = {{&S39421[164], 15}, {NULL}};
const Segment SGI319089[] = {{&S39421[51], 6}, {NULL}};
const Segment SGI319090[] = {{&S39421[71], 16}, {NULL}};
const Segment SGI319091[] = {{&S39422[30], 4}, {NULL}};
const Segment SGI319092[] = {{&S39422[8], 3}, {NULL}};
const Segment SGI319093[] = {{&S39422[13], 10}, {NULL}};
const Segment SGI319094[] = {{&S39422[43], 7}, {NULL}};
const Segment SGI319095[] = {{&S39422[36], 14}, {NULL}};
const Segment SGI319096[] = {{&S39422[72], 11}, {NULL}};
const Segment SGI319097[] = {{&S39422[52], 6}, {NULL}};
const Segment SGI319098[] = {{&S39422[72], 16}, {NULL}};
const Segment SGI319099[] = {{&S39423[30], 3}, {NULL}};
const Segment SGI319100[] = {{&S39423[235], 9}, {NULL}};
const Segment SGI319101[] = {{&S39423[8], 3}, {NULL}};
const Segment SGI319102[] = {{&S39423[13], 10}, {NULL}};
const Segment SGI319103[] = {{&S39423[42], 7}, {NULL}};
const Segment SGI319104[] = {{&S39423[35], 14}, {NULL}};
const Segment SGI319105[] = {{&S39423[71], 11}, {NULL}};
const Segment SGI319106[] = {{&S39423[164], 15}, {NULL}};
const Segment SGI319107[] = {{&S39423[51], 6}, {NULL}};
const Segment SGI319108[] = {{&S39423[71], 16}, {NULL}};
const Segment SGI319109[] = {{&S39424[30], 4}, {NULL}};
const Segment SGI319110[] = {{&S39424[8], 3}, {NULL}};
const Segment SGI319111[] = {{&S39424[13], 10}, {NULL}};
const Segment SGI319112[] = {{&S39424[43], 7}, {NULL}};
const Segment SGI319113[] = {{&S39424[36], 14}, {NULL}};
const Segment SGI319114[] = {{&S39424[65], 11}, {NULL}};
const Segment SGI319115[] = {{&S39424[65], 16}, {NULL}};
const Segment SGI319116[] = {{&S39493[30], 3}, {NULL}};
const Segment SGI319117[] = {{&S39493[8], 3}, {NULL}};
const Segment SGI319118[] = {{&S39493[13], 10}, {NULL}};
const Segment SGI319119[] = {{&S39493[42], 7}, {NULL}};
const Segment SGI319120[] = {{&S39493[35], 14}, {NULL}};
const Segment SGI319121[] = {{&S39493[51], 11}, {NULL}};
const Segment SGI319122[] = {{&S39493[51], 16}, {NULL}};
const Segment SGI319123[] = {{&S39494[30], 3}, {NULL}};
const Segment SGI319124[] = {{&S39494[8], 3}, {NULL}};
const Segment SGI319125[] = {{&S39494[13], 10}, {NULL}};
const Segment SGI319126[] = {{&S39494[42], 7}, {NULL}};
const Segment SGI319127[] = {{&S39494[35], 14}, {NULL}};
const Segment SGI319128[] = {{&S39494[64], 11}, {&S39494[64], 11}, {NULL}};
const Segment SGI319129[] = {{&S39494[64], 16}, {NULL}};
const Segment SGI319130[] = {{&S39495[30], 3}, {NULL}};
const Segment SGI319131[] = {{&S39495[8], 3}, {NULL}};
const Segment SGI319132[] = {{&S39495[13], 10}, {NULL}};
const Segment SGI319133[] = {{&S39495[42], 7}, {NULL}};
const Segment SGI319134[] = {{&S39495[35], 14}, {NULL}};
const Segment SGI319135[] = {{&S39495[56], 11}, {NULL}};
const Segment SGI319136[] = {{&S39495[56], 16}, {NULL}};
const Segment SGI319137[] = {{&S39495[51], 3}, {NULL}};
const Segment SGI319138[] = {{&S39500[30], 3}, {NULL}};
const Segment SGI319139[] = {{&S39500[8], 3}, {NULL}};
const Segment SGI319140[] = {{&S39500[13], 10}, {NULL}};
const Segment SGI319141[] = {{&S39500[42], 7}, {NULL}};
const Segment SGI319142[] = {{&S39500[35], 14}, {NULL}};
const Segment SGI319143[] = {{&S39500[64], 11}, {NULL}};
const Segment SGI319144[] = {{&S39500[64], 16}, {NULL}};
const Segment SGI319145[] = {{&S39501[30], 3}, {NULL}};
const Segment SGI319146[] = {{&S39501[8], 3}, {NULL}};
const Segment SGI319147[] = {{&S39501[13], 10}, {NULL}};
const Segment SGI319148[] = {{&S39501[42], 7}, {NULL}};
const Segment SGI319149[] = {{&S39501[35], 14}, {NULL}};
const Segment SGI319150[] = {{&S39501[64], 11}, {NULL}};
const Segment SGI319151[] = {{&S39501[64], 16}, {NULL}};
const Segment SGI319152[] = {{&S39502[30], 3}, {NULL}};
const Segment SGI319153[] = {{&S39502[8], 3}, {NULL}};
const Segment SGI319154[] = {{&S39502[13], 10}, {NULL}};
const Segment SGI319155[] = {{&S39502[42], 7}, {NULL}};
const Segment SGI319156[] = {{&S39502[35], 14}, {NULL}};
const Segment SGI319157[] = {{&S39502[64], 11}, {NULL}};
const Segment SGI319158[] = {{&S39502[64], 16}, {NULL}};
const Segment SGI319159[] = {{&S39503[30], 3}, {NULL}};
const Segment SGI319160[] = {{&S39503[8], 3}, {NULL}};
const Segment SGI319161[] = {{&S39503[13], 10}, {NULL}};
const Segment SGI319162[] = {{&S39503[42], 7}, {NULL}};
const Segment SGI319163[] = {{&S39503[35], 14}, {NULL}};
const Segment SGI319164[] = {{&S39503[58], 11}, {NULL}};
const Segment SGI319165[] = {{&S39503[51], 6}, {NULL}};
const Segment SGI319166[] = {{&S39503[58], 16}, {NULL}};
const Segment SGI319167[] = {{&S39505[30], 3}, {NULL}};
const Segment SGI319168[] = {{&S39505[8], 3}, {NULL}};
const Segment SGI319169[] = {{&S39505[13], 10}, {NULL}};
const Segment SGI319170[] = {{&S39505[42], 7}, {NULL}};
const Segment SGI319171[] = {{&S39505[35], 14}, {NULL}};
const Segment SGI319172[] = {{&S39505[64], 11}, {NULL}};
const Segment SGI319173[] = {{&S39505[128], 15}, {NULL}};
const Segment SGI319174[] = {{&S39505[64], 16}, {NULL}};
const Segment SGI319175[] = {{&S39506[30], 3}, {NULL}};
const Segment SGI319176[] = {{&S39506[8], 3}, {NULL}};
const Segment SGI319177[] = {{&S39506[13], 10}, {NULL}};
const Segment SGI319178[] = {{&S39506[42], 7}, {NULL}};
const Segment SGI319179[] = {{&S39506[35], 14}, {NULL}};
const Segment SGI319180[] = {{&S39506[51], 11}, {NULL}};
const Segment SGI319181[] = {{&S39506[51], 16}, {NULL}};
const Segment SGI319182[] = {{&S39509[30], 3}, {&S39509[147], 3}, {NULL}};
const Segment SGI319183[] = {{&S39509[8], 3}, {&S39509[8], 3}, {NULL}};
const Segment SGI319184[] = {{&S39509[13], 10}, {&S39509[13], 10}, {NULL}};
const Segment SGI319185[] = {{&S39509[42], 7}, {&S39509[42], 7}, {NULL}};
const Segment SGI319186[] = {{&S39509[35], 14}, {&S39509[35], 14}, {NULL}};
const Segment SGI319187[] = {{&S39509[83], 11}, {NULL}};
const Segment SGI319188[] = {{&S39509[83], 16}, {NULL}};
const Segment SGI319189[] = {{&S39509[168], 3}, {NULL}};
const Segment SGI319190[] = {{&S39510[30], 3}, {NULL}};
const Segment SGI319191[] = {{&S39510[8], 3}, {NULL}};
const Segment SGI319192[] = {{&S39510[13], 10}, {NULL}};
const Segment SGI319193[] = {{&S39510[42], 7}, {NULL}};
const Segment SGI319194[] = {{&S39510[35], 14}, {NULL}};
const Segment SGI319195[] = {{&S39510[64], 11}, {NULL}};
const Segment SGI319196[] = {{&S39510[64], 16}, {NULL}};
const Segment SGI319197[] = {{&S39510[128], 9}, {NULL}};
const Segment SGI319198[] = {{&S68165[3], 5}, {NULL}};
const Segment SGI319199[] = {{&S39887[30], 3}, {NULL}};
const Segment SGI319200[] = {{&S39887[8], 3}, {NULL}};
const Segment SGI319201[] = {{&S39887[13], 10}, {NULL}};
const Segment SGI319202[] = {{&S39887[55], 7}, {NULL}};
const Segment SGI319203[] = {{&S39887[48], 14}, {NULL}};
const Segment SGI319204[] = {{&S39887[64], 3}, {NULL}};
const Segment SGI319205[] = {{&S39888[30], 3}, {NULL}};
const Segment SGI319206[] = {{&S39888[8], 3}, {NULL}};
const Segment SGI319207[] = {{&S39888[13], 10}, {NULL}};
const Segment SGI319208[] = {{&S39888[55], 7}, {NULL}};
const Segment SGI319209[] = {{&S39888[48], 14}, {NULL}};
const Segment SGI319210[] = {{&S39889[30], 3}, {NULL}};
const Segment SGI319211[] = {{&S39889[8], 3}, {NULL}};
const Segment SGI319212[] = {{&S39889[13], 10}, {NULL}};
const Segment SGI319213[] = {{&S39889[55], 7}, {NULL}};
const Segment SGI319214[] = {{&S39889[48], 14}, {NULL}};
const Segment SGI319215[] = {{&S39889[64], 3}, {NULL}};
const Segment SGI319216[] = {{&S39890[30], 3}, {NULL}};
const Segment SGI319217[] = {{&S39890[8], 3}, {NULL}};
const Segment SGI319218[] = {{&S39890[13], 10}, {NULL}};
const Segment SGI319219[] = {{&S39890[55], 7}, {NULL}};
const Segment SGI319220[] = {{&S39890[48], 14}, {NULL}};
const Segment SGI319221[] = {{&S39891[30], 3}, {NULL}};
const Segment SGI319222[] = {{&S39891[8], 3}, {NULL}};
const Segment SGI319223[] = {{&S39891[13], 10}, {NULL}};
const Segment SGI319224[] = {{&S39891[55], 7}, {NULL}};
const Segment SGI319225[] = {{&S39891[48], 14}, {NULL}};
const Segment SGI319226[] = {{&S39891[64], 3}, {&S39891[64], 3}, {NULL}};
const Segment SGI319227[] = {{&S39892[30], 3}, {NULL}};
const Segment SGI319228[] = {{&S39892[8], 3}, {NULL}};
const Segment SGI319229[] = {{&S39892[13], 10}, {NULL}};
const Segment SGI319230[] = {{&S39892[55], 7}, {NULL}};
const Segment SGI319231[] = {{&S39892[48], 14}, {NULL}};
const Segment SGI319232[] = {{&S39892[64], 3}, {&S39892[64], 3}, {NULL}};
const Segment SGI319233[] = {{&S39933[30], 4}, {NULL}};
const Segment SGI319234[] = {{&S39933[8], 3}, {NULL}};
const Segment SGI319235[] = {{&S39933[13], 10}, {NULL}};
const Segment SGI319236[] = {{&S39933[43], 7}, {NULL}};
const Segment SGI319237[] = {{&S39933[36], 14}, {NULL}};
const Segment SGI319238[] = {{&S39933[52], 6}, {NULL}};
const Segment SGI319239[] = {{&S39933[72], 13}, {NULL}};
const Segment SGI319240[] = {{&S39934[30], 4}, {NULL}};
const Segment SGI319241[] = {{&S39934[8], 3}, {NULL}};
const Segment SGI319242[] = {{&S39934[13], 10}, {NULL}};
const Segment SGI319243[] = {{&S39934[43], 7}, {NULL}};
const Segment SGI319244[] = {{&S39934[36], 14}, {NULL}};
const Segment SGI319245[] = {{&S39934[65], 13}, {NULL}};
const Segment SGI319246[] = {{&S39935[30], 4}, {NULL}};
const Segment SGI319247[] = {{&S39935[8], 3}, {NULL}};
const Segment SGI319248[] = {{&S39935[13], 10}, {NULL}};
const Segment SGI319249[] = {{&S39935[43], 7}, {NULL}};
const Segment SGI319250[] = {{&S39935[36], 14}, {NULL}};
const Segment SGI319251[] = {{&S39935[77], 13}, {NULL}};
const Segment SGI319252[] = {{&S39935[52], 5}, {NULL}};
const Segment SGI319253[] = {{&S39947[8], 3}, {NULL}};
const Segment SGI319254[] = {{&S39947[13], 10}, {NULL}};
const Segment SGI319255[] = {{&S39947[63], 7}, {NULL}};
const Segment SGI319256[] = {{&S57126[3], 6}, {NULL}};
const Segment SGI319257[] = {{&S57127[3], 5}, {NULL}};
const Segment SGI319258[] = {{&S40225[30], 3}, {NULL}};
const Segment SGI319259[] = {{&S40225[8], 3}, {NULL}};
const Segment SGI319260[] = {{&S40225[13], 10}, {NULL}};
const Segment SGI319261[] = {{&S40225[42], 7}, {NULL}};
const Segment SGI319262[] = {{&S40225[35], 14}, {NULL}};
const Segment SGI319263[] = {{&S40225[64], 13}, {NULL}};
const Segment SGI319264[] = {{&S40226[30], 3}, {NULL}};
const Segment SGI319265[] = {{&S40226[8], 3}, {NULL}};
const Segment SGI319266[] = {{&S40226[13], 10}, {NULL}};
const Segment SGI319267[] = {{&S40226[42], 7}, {NULL}};
const Segment SGI319268[] = {{&S40226[35], 14}, {NULL}};
const Segment SGI319269[] = {{&S40226[51], 6}, {NULL}};
const Segment SGI319270[] = {{&S40226[71], 13}, {NULL}};
const Segment SGI319271[] = {{&S40227[30], 3}, {NULL}};
const Segment SGI319272[] = {{&S40227[8], 3}, {NULL}};
const Segment SGI319273[] = {{&S40227[13], 10}, {NULL}};
const Segment SGI319274[] = {{&S40227[42], 7}, {NULL}};
const Segment SGI319275[] = {{&S40227[35], 14}, {NULL}};
const Segment SGI319276[] = {{&S40227[76], 13}, {NULL}};
const Segment SGI319277[] = {{&S40227[51], 5}, {NULL}};
const Segment SGI319278[] = {{&S40228[30], 3}, {NULL}};
const Segment SGI319279[] = {{&S40228[8], 3}, {NULL}};
const Segment SGI319280[] = {{&S40228[13], 10}, {NULL}};
const Segment SGI319281[] = {{&S40228[55], 7}, {NULL}};
const Segment SGI319282[] = {{&S40228[48], 14}, {NULL}};
const Segment SGI319283[] = {{&S40228[64], 13}, {NULL}};
const Segment SGI319284[] = {{&S40229[30], 3}, {NULL}};
const Segment SGI319285[] = {{&S40229[8], 3}, {NULL}};
const Segment SGI319286[] = {{&S40229[13], 10}, {NULL}};
const Segment SGI319287[] = {{&S40229[55], 7}, {NULL}};
const Segment SGI319288[] = {{&S40229[48], 14}, {NULL}};
const Segment SGI319289[] = {{&S40229[64], 13}, {&S40229[64], 13}, {NULL}};
const Segment SGI319290[] = {{&S40230[30], 3}, {NULL}};
const Segment SGI319291[] = {{&S40230[8], 3}, {NULL}};
const Segment SGI319292[] = {{&S40230[13], 10}, {NULL}};
const Segment SGI319293[] = {{&S40230[55], 7}, {NULL}};
const Segment SGI319294[] = {{&S40230[48], 14}, {NULL}};
const Segment SGI319295[] = {{&S40230[64], 13}, {NULL}};
const Segment SGI319296[] = {{&S40230[79], 3}, {NULL}};
const Segment SGI319297[] = {{&S40231[30], 3}, {NULL}};
const Segment SGI319298[] = {{&S40231[8], 3}, {NULL}};
const Segment SGI319299[] = {{&S40231[13], 10}, {NULL}};
const Segment SGI319300[] = {{&S40231[55], 7}, {NULL}};
const Segment SGI319301[] = {{&S40231[48], 14}, {NULL}};
const Segment SGI319302[] = {{&S40231[69], 13}, {NULL}};
const Segment SGI319303[] = {{&S40231[64], 3}, {NULL}};
const Segment SGI319304[] = {{&S40232[30], 3}, {NULL}};
const Segment SGI319305[] = {{&S40232[8], 3}, {NULL}};
const Segment SGI319306[] = {{&S40232[13], 10}, {NULL}};
const Segment SGI319307[] = {{&S40232[55], 7}, {NULL}};
const Segment SGI319308[] = {{&S40232[48], 14}, {NULL}};
const Segment SGI319309[] = {{&S40232[64], 13}, {&S40232[64], 13}, {NULL}};
const Segment SGI319310[] = {{&S40329[30], 3}, {NULL}};
const Segment SGI319311[] = {{&S40329[8], 3}, {NULL}};
const Segment SGI319312[] = {{&S40329[13], 10}, {NULL}};
const Segment SGI319313[] = {{&S40329[42], 7}, {NULL}};
const Segment SGI319314[] = {{&S40329[35], 14}, {NULL}};
const Segment SGI319315[] = {{&S40329[51], 11}, {NULL}};
const Segment SGI319316[] = {{&S40329[51], 16}, {NULL}};
const Segment SGI319317[] = {{&S40332[30], 3}, {NULL}};
const Segment SGI319318[] = {{&S40332[8], 3}, {NULL}};
const Segment SGI319319[] = {{&S40332[13], 10}, {NULL}};
const Segment SGI319320[] = {{&S40332[42], 7}, {NULL}};
const Segment SGI319321[] = {{&S40332[35], 14}, {NULL}};
const Segment SGI319322[] = {{&S40332[64], 11}, {NULL}};
const Segment SGI319323[] = {{&S40332[64], 16}, {NULL}};
const Segment SGI319324[] = {{&S40334[30], 3}, {NULL}};
const Segment SGI319325[] = {{&S40334[8], 3}, {NULL}};
const Segment SGI319326[] = {{&S40334[13], 10}, {NULL}};
const Segment SGI319327[] = {{&S40334[42], 7}, {NULL}};
const Segment SGI319328[] = {{&S40334[35], 14}, {NULL}};
const Segment SGI319329[] = {{&S40334[51], 11}, {NULL}};
const Segment SGI319330[] = {{&S40334[51], 16}, {NULL}};
const Segment SGI319331[] = {{&S40336[30], 3}, {NULL}};
const Segment SGI319332[] = {{&S40336[8], 3}, {NULL}};
const Segment SGI319333[] = {{&S40336[13], 10}, {NULL}};
const Segment SGI319334[] = {{&S40336[42], 7}, {NULL}};
const Segment SGI319335[] = {{&S40336[35], 14}, {NULL}};
const Segment SGI319336[] = {{&S40336[56], 11}, {NULL}};
const Segment SGI319337[] = {{&S40336[56], 16}, {NULL}};
const Segment SGI319338[] = {{&S40336[51], 3}, {NULL}};
const Segment SGI319339[] = {{&S40338[30], 3}, {NULL}};
const Segment SGI319340[] = {{&S40338[8], 3}, {NULL}};
const Segment SGI319341[] = {{&S40338[13], 10}, {NULL}};
const Segment SGI319342[] = {{&S40338[42], 7}, {NULL}};
const Segment SGI319343[] = {{&S40338[35], 14}, {NULL}};
const Segment SGI319344[] = {{&S40338[56], 11}, {NULL}};
const Segment SGI319345[] = {{&S40338[120], 15}, {NULL}};
const Segment SGI319346[] = {{&S40338[56], 16}, {NULL}};
const Segment SGI319347[] = {{&S40338[51], 3}, {NULL}};
const Segment SGI319348[] = {{&S57410[3], 5}, {NULL}};
const Segment SGI319349[] = {{&S57410[3], 14}, {NULL}};
const Segment SGI319350[] = {{&S57411[3], 7}, {NULL}};
const Segment SGI319351[] = {{&S57412[3], 5}, {NULL}};
const Segment SGI319352[] = {{&S57412[3], 12}, {NULL}};
const Segment SGI319353[] = {{&S57413[3], 5}, {NULL}};
const Segment SGI319354[] = {{&S57413[3], 12}, {NULL}};
const Segment SGI319355[] = {{&S57414[3], 5}, {NULL}};
const Segment SGI319356[] = {{&S40705[159], 3}, {NULL}};
const Segment SGI319357[] = {{&S40705[8], 3}, {&S40705[8], 3}, {NULL}};
const Segment SGI319358[] = {{&S40705[13], 9}, {&S40705[142], 10}, {NULL}};
const Segment SGI319359[] = {{&S40705[171], 7}, {NULL}};
const Segment SGI319360[] = {{&S40705[33], 15}, {&S40705[164], 14}, {NULL}};
const Segment SGI319361[] = {{&S40782[30], 3}, {NULL}};
const Segment SGI319362[] = {{&S40782[8], 3}, {NULL}};
const Segment SGI319363[] = {{&S40782[13], 10}, {NULL}};
const Segment SGI319364[] = {{&S40782[42], 7}, {NULL}};
const Segment SGI319365[] = {{&S40782[35], 14}, {NULL}};
const Segment SGI319366[] = {{&S40782[74], 4}, {NULL}};
const Segment SGI319367[] = {{&S40783[30], 3}, {NULL}};
const Segment SGI319368[] = {{&S40783[8], 3}, {NULL}};
const Segment SGI319369[] = {{&S40783[13], 10}, {NULL}};
const Segment SGI319370[] = {{&S40783[42], 7}, {NULL}};
const Segment SGI319371[] = {{&S40783[35], 14}, {NULL}};
const Segment SGI319372[] = {{&S40783[74], 4}, {NULL}};
const Segment SGI319373[] = {{&S40784[30], 3}, {NULL}};
const Segment SGI319374[] = {{&S40784[8], 3}, {NULL}};
const Segment SGI319375[] = {{&S40784[13], 10}, {NULL}};
const Segment SGI319376[] = {{&S40784[42], 7}, {NULL}};
const Segment SGI319377[] = {{&S40784[35], 14}, {NULL}};
const Segment SGI319378[] = {{&S40784[51], 3}, {NULL}};
const Segment SGI319379[] = {{&S40784[66], 4}, {NULL}};
const Segment SGI319380[] = {{&S40785[30], 3}, {NULL}};
const Segment SGI319381[] = {{&S40785[8], 3}, {NULL}};
const Segment SGI319382[] = {{&S40785[13], 10}, {NULL}};
const Segment SGI319383[] = {{&S40785[42], 7}, {NULL}};
const Segment SGI319384[] = {{&S40785[35], 14}, {NULL}};
const Segment SGI319385[] = {{&S40785[51], 3}, {NULL}};
const Segment SGI319386[] = {{&S40785[66], 4}, {NULL}};
const Segment SGI319387[] = {{&S40786[30], 3}, {NULL}};
const Segment SGI319388[] = {{&S40786[8], 3}, {NULL}};
const Segment SGI319389[] = {{&S40786[13], 10}, {NULL}};
const Segment SGI319390[] = {{&S40786[42], 7}, {NULL}};
const Segment SGI319391[] = {{&S40786[35], 14}, {NULL}};
const Segment SGI319392[] = {{&S40786[74], 4}, {NULL}};
const Segment SGI319393[] = {{&S40787[30], 3}, {NULL}};
const Segment SGI319394[] = {{&S40787[8], 3}, {NULL}};
const Segment SGI319395[] = {{&S40787[13], 10}, {NULL}};
const Segment SGI319396[] = {{&S40787[42], 7}, {NULL}};
const Segment SGI319397[] = {{&S40787[35], 14}, {NULL}};
const Segment SGI319398[] = {{&S40787[131], 15}, {NULL}};
const Segment SGI319399[] = {{&S40787[162], 4}, {NULL}};
const Segment SGI319400[] = {{&S40788[30], 3}, {NULL}};
const Segment SGI319401[] = {{&S40788[8], 3}, {NULL}};
const Segment SGI319402[] = {{&S40788[13], 10}, {NULL}};
const Segment SGI319403[] = {{&S40788[42], 7}, {NULL}};
const Segment SGI319404[] = {{&S40788[35], 14}, {NULL}};
const Segment SGI319405[] = {{&S40788[131], 15}, {NULL}};
const Segment SGI319406[] = {{&S40788[162], 4}, {NULL}};
const Segment SGI319407[] = {{&S40789[30], 3}, {NULL}};
const Segment SGI319408[] = {{&S40789[8], 3}, {NULL}};
const Segment SGI319409[] = {{&S40789[13], 10}, {NULL}};
const Segment SGI319410[] = {{&S40789[42], 7}, {NULL}};
const Segment SGI319411[] = {{&S40789[35], 14}, {NULL}};
const Segment SGI319412[] = {{&S40789[51], 3}, {NULL}};
const Segment SGI319413[] = {{&S40789[66], 4}, {NULL}};
const Segment SGI319414[] = {{&S40790[30], 3}, {NULL}};
const Segment SGI319415[] = {{&S40790[8], 3}, {NULL}};
const Segment SGI319416[] = {{&S40790[13], 10}, {NULL}};
const Segment SGI319417[] = {{&S40790[42], 7}, {NULL}};
const Segment SGI319418[] = {{&S40790[35], 14}, {NULL}};
const Segment SGI319419[] = {{&S40790[51], 6}, {NULL}};
const Segment SGI319420[] = {{&S40790[108], 4}, {NULL}};
const Segment SGI319421[] = {{&S40791[30], 3}, {NULL}};
const Segment SGI319422[] = {{&S40791[8], 3}, {NULL}};
const Segment SGI319423[] = {{&S40791[13], 10}, {NULL}};
const Segment SGI319424[] = {{&S40791[42], 7}, {NULL}};
const Segment SGI319425[] = {{&S40791[35], 14}, {NULL}};
const Segment SGI319426[] = {{&S40791[51], 6}, {NULL}};
const Segment SGI319427[] = {{&S40791[108], 4}, {NULL}};
const Segment SGI319428[] = {{&S40792[30], 3}, {NULL}};
const Segment SGI319429[] = {{&S40792[8], 3}, {NULL}};
const Segment SGI319430[] = {{&S40792[13], 10}, {NULL}};
const Segment SGI319431[] = {{&S40792[42], 7}, {NULL}};
const Segment SGI319432[] = {{&S40792[35], 14}, {NULL}};
const Segment SGI319433[] = {{&S40792[138], 15}, {NULL}};
const Segment SGI319434[] = {{&S40792[51], 6}, {NULL}};
const Segment SGI319435[] = {{&S40792[179], 4}, {NULL}};
const Segment SGI319436[] = {{&S40794[30], 3}, {NULL}};
const Segment SGI319437[] = {{&S40794[8], 3}, {NULL}};
const Segment SGI319438[] = {{&S40794[13], 10}, {NULL}};
const Segment SGI319439[] = {{&S40794[42], 7}, {NULL}};
const Segment SGI319440[] = {{&S40794[35], 14}, {NULL}};
const Segment SGI319441[] = {{&S40794[138], 15}, {NULL}};
const Segment SGI319442[] = {{&S40794[51], 6}, {NULL}};
const Segment SGI319443[] = {{&S40794[179], 4}, {NULL}};
const Segment SGI319444[] = {{&S40795[30], 3}, {NULL}};
const Segment SGI319445[] = {{&S40795[8], 3}, {NULL}};
const Segment SGI319446[] = {{&S40795[13], 10}, {NULL}};
const Segment SGI319447[] = {{&S40795[42], 7}, {NULL}};
const Segment SGI319448[] = {{&S40795[35], 14}, {NULL}};
const Segment SGI319449[] = {{&S40795[51], 6}, {NULL}};
const Segment SGI319450[] = {{&S40795[108], 4}, {NULL}};
const Segment SGI319451[] = {{&S40796[30], 3}, {NULL}};
const Segment SGI319452[] = {{&S40796[8], 3}, {NULL}};
const Segment SGI319453[] = {{&S40796[13], 10}, {NULL}};
const Segment SGI319454[] = {{&S40796[42], 7}, {NULL}};
const Segment SGI319455[] = {{&S40796[35], 14}, {NULL}};
const Segment SGI319456[] = {{&S40796[131], 15}, {NULL}};
const Segment SGI319457[] = {{&S40796[152], 9}, {NULL}};
const Segment SGI319458[] = {{&S40796[187], 4}, {NULL}};
const Segment SGI319459[] = {{&S40797[30], 3}, {NULL}};
const Segment SGI319460[] = {{&S40797[8], 3}, {NULL}};
const Segment SGI319461[] = {{&S40797[13], 10}, {NULL}};
const Segment SGI319462[] = {{&S40797[42], 7}, {NULL}};
const Segment SGI319463[] = {{&S40797[35], 14}, {NULL}};
const Segment SGI319464[] = {{&S40797[138], 15}, {NULL}};
const Segment SGI319465[] = {{&S40797[51], 6}, {NULL}};
const Segment SGI319466[] = {{&S40797[169], 4}, {NULL}};
const Segment SGI319467[] = {{&S40798[30], 3}, {NULL}};
const Segment SGI319468[] = {{&S40798[8], 3}, {NULL}};
const Segment SGI319469[] = {{&S40798[13], 10}, {NULL}};
const Segment SGI319470[] = {{&S40798[42], 7}, {NULL}};
const Segment SGI319471[] = {{&S40798[35], 14}, {NULL}};
const Segment SGI319472[] = {{&S40798[51], 6}, {NULL}};
const Segment SGI319473[] = {{&S40798[108], 4}, {NULL}};
const Segment SGI319474[] = {{&S40799[30], 3}, {NULL}};
const Segment SGI319475[] = {{&S40799[8], 3}, {NULL}};
const Segment SGI319476[] = {{&S40799[13], 10}, {NULL}};
const Segment SGI319477[] = {{&S40799[42], 7}, {NULL}};
const Segment SGI319478[] = {{&S40799[35], 14}, {NULL}};
const Segment SGI319479[] = {{&S40799[51], 6}, {NULL}};
const Segment SGI319480[] = {{&S40799[108], 4}, {NULL}};
const Segment SGI319481[] = {{&S40836[30], 3}, {NULL}};
const Segment SGI319482[] = {{&S40836[8], 3}, {NULL}};
const Segment SGI319483[] = {{&S40836[13], 10}, {NULL}};
const Segment SGI319484[] = {{&S40836[42], 7}, {NULL}};
const Segment SGI319485[] = {{&S40836[35], 14}, {NULL}};
const Segment SGI319486[] = {{&S40836[131], 15}, {NULL}};
const Segment SGI319487[] = {{&S40836[152], 9}, {NULL}};
const Segment SGI319488[] = {{&S40836[187], 4}, {NULL}};
const Segment SGI319489[] = {{&S40837[30], 3}, {NULL}};
const Segment SGI319490[] = {{&S40837[8], 3}, {NULL}};
const Segment SGI319491[] = {{&S40837[13], 10}, {NULL}};
const Segment SGI319492[] = {{&S40837[42], 7}, {NULL}};
const Segment SGI319493[] = {{&S40837[35], 14}, {NULL}};
const Segment SGI319494[] = {{&S40837[131], 15}, {NULL}};
const Segment SGI319495[] = {{&S40837[172], 4}, {NULL}};
const Segment SGI319496[] = {{&S40838[30], 3}, {NULL}};
const Segment SGI319497[] = {{&S40838[8], 3}, {NULL}};
const Segment SGI319498[] = {{&S40838[13], 10}, {NULL}};
const Segment SGI319499[] = {{&S40838[42], 7}, {NULL}};
const Segment SGI319500[] = {{&S40838[35], 14}, {NULL}};
const Segment SGI319501[] = {{&S40838[138], 15}, {NULL}};
const Segment SGI319502[] = {{&S40838[51], 6}, {NULL}};
const Segment SGI319503[] = {{&S40838[179], 4}, {NULL}};
const Segment SGI319504[] = {{&S40839[30], 3}, {NULL}};
const Segment SGI319505[] = {{&S40839[8], 3}, {NULL}};
const Segment SGI319506[] = {{&S40839[13], 10}, {NULL}};
const Segment SGI319507[] = {{&S40839[42], 7}, {NULL}};
const Segment SGI319508[] = {{&S40839[35], 14}, {NULL}};
const Segment SGI319509[] = {{&S40839[151], 4}, {NULL}};
const Segment SGI319510[] = {{&S40840[30], 3}, {NULL}};
const Segment SGI319511[] = {{&S40840[8], 3}, {NULL}};
const Segment SGI319512[] = {{&S40840[13], 10}, {NULL}};
const Segment SGI319513[] = {{&S40840[42], 7}, {NULL}};
const Segment SGI319514[] = {{&S40840[35], 14}, {NULL}};
const Segment SGI319515[] = {{&S40840[51], 6}, {NULL}};
const Segment SGI319516[] = {{&S40840[81], 4}, {NULL}};
const Segment SGI319517[] = {{&S40841[30], 3}, {NULL}};
const Segment SGI319518[] = {{&S40841[8], 3}, {NULL}};
const Segment SGI319519[] = {{&S40841[13], 10}, {NULL}};
const Segment SGI319520[] = {{&S40841[42], 7}, {NULL}};
const Segment SGI319521[] = {{&S40841[35], 14}, {NULL}};
const Segment SGI319522[] = {{&S40841[51], 6}, {NULL}};
const Segment SGI319523[] = {{&S40841[148], 4}, {NULL}};
const Segment SGI319524[] = {{&S40842[30], 3}, {NULL}};
const Segment SGI319525[] = {{&S40842[8], 3}, {NULL}};
const Segment SGI319526[] = {{&S40842[13], 10}, {NULL}};
const Segment SGI319527[] = {{&S40842[42], 7}, {NULL}};
const Segment SGI319528[] = {{&S40842[35], 14}, {NULL}};
const Segment SGI319529[] = {{&S40842[138], 15}, {NULL}};
const Segment SGI319530[] = {{&S40842[51], 6}, {NULL}};
const Segment SGI319531[] = {{&S40842[179], 4}, {NULL}};
const Segment SGI319532[] = {{&S40844[30], 3}, {NULL}};
const Segment SGI319533[] = {{&S40844[8], 3}, {NULL}};
const Segment SGI319534[] = {{&S40844[13], 10}, {NULL}};
const Segment SGI319535[] = {{&S40844[42], 7}, {NULL}};
const Segment SGI319536[] = {{&S40844[35], 14}, {NULL}};
const Segment SGI319537[] = {{&S40844[131], 15}, {NULL}};
const Segment SGI319538[] = {{&S40844[172], 4}, {NULL}};
const Segment SGI319539[] = {{&S40845[30], 3}, {NULL}};
const Segment SGI319540[] = {{&S40845[8], 3}, {NULL}};
const Segment SGI319541[] = {{&S40845[13], 10}, {NULL}};
const Segment SGI319542[] = {{&S40845[42], 7}, {NULL}};
const Segment SGI319543[] = {{&S40845[35], 14}, {NULL}};
const Segment SGI319544[] = {{&S40845[74], 4}, {NULL}};
const Segment SGI319545[] = {{&S40846[30], 3}, {&S40846[94], 3}, {NULL}};
const Segment SGI319546[] = {{&S40846[8], 3}, {&S40846[8], 3}, {NULL}};
const Segment SGI319547[] = {{&S40846[13], 10}, {&S40846[13], 10}, {NULL}};
const Segment SGI319548[] = {{&S40846[42], 7}, {&S40846[106], 7}, {NULL}};
const Segment SGI319549[] = {{&S40846[35], 14}, {&S40846[99], 14}, {NULL}};
const Segment SGI319550[] = {{&S40846[189], 15}, {NULL}};
const Segment SGI319551[] = {{&S40846[115], 3}, {NULL}};
const Segment SGI319552[] = {{&S40846[235], 4}, {NULL}};
const Segment SGI319553[] = {{&S40847[30], 3}, {NULL}};
const Segment SGI319554[] = {{&S40847[8], 3}, {NULL}};
const Segment SGI319555[] = {{&S40847[13], 10}, {NULL}};
const Segment SGI319556[] = {{&S40847[62], 7}, {NULL}};
const Segment SGI319557[] = {{&S40847[55], 14}, {NULL}};
const Segment SGI319558[] = {{&S40847[168], 4}, {NULL}};
const Segment SGI319559[] = {{&S41132[30], 3}, {NULL}};
const Segment SGI319560[] = {{&S41132[8], 3}, {NULL}};
const Segment SGI319561[] = {{&S41132[13], 10}, {NULL}};
const Segment SGI319562[] = {{&S41132[35], 5}, {NULL}};
const Segment SGI319563[] = {{&S41133[30], 3}, {NULL}};
const Segment SGI319564[] = {{&S41133[8], 3}, {NULL}};
const Segment SGI319565[] = {{&S41133[13], 10}, {NULL}};
const Segment SGI319566[] = {{&S41133[35], 5}, {NULL}};
const Segment SGI319567[] = {{&S41134[30], 3}, {NULL}};
const Segment SGI319568[] = {{&S41134[8], 3}, {NULL}};
const Segment SGI319569[] = {{&S41134[13], 10}, {NULL}};
const Segment SGI319570[] = {{&S41134[35], 5}, {NULL}};
const Segment SGI319571[] = {{&S41134[47], 5}, {NULL}};
const Segment SGI319572[] = {{&S41135[30], 3}, {NULL}};
const Segment SGI319573[] = {{&S41135[8], 3}, {NULL}};
const Segment SGI319574[] = {{&S41135[13], 10}, {NULL}};
const Segment SGI319575[] = {{&S41135[35], 5}, {NULL}};
const Segment SGI319576[] = {{&S41135[47], 9}, {NULL}};
const Segment SGI319577[] = {{&S41136[30], 3}, {NULL}};
const Segment SGI319578[] = {{&S41136[8], 3}, {NULL}};
const Segment SGI319579[] = {{&S41136[13], 10}, {NULL}};
const Segment SGI319580[] = {{&S41136[35], 5}, {NULL}};
const Segment SGI319581[] = {{&S41136[47], 3}, {NULL}};
const Segment SGI319582[] = {{&S41137[30], 3}, {NULL}};
const Segment SGI319583[] = {{&S41137[8], 3}, {NULL}};
const Segment SGI319584[] = {{&S41137[13], 10}, {NULL}};
const Segment SGI319585[] = {{&S41137[35], 5}, {NULL}};
const Segment SGI319586[] = {{&S41137[47], 6}, {NULL}};
const Segment SGI319587[] = {{&S41138[30], 3}, {NULL}};
const Segment SGI319588[] = {{&S41138[8], 3}, {NULL}};
const Segment SGI319589[] = {{&S41138[13], 10}, {NULL}};
const Segment SGI319590[] = {{&S41138[35], 5}, {NULL}};
const Segment SGI319591[] = {{&S41138[49], 9}, {NULL}};
const Segment SGI319592[] = {{&S41139[30], 3}, {NULL}};
const Segment SGI319593[] = {{&S41139[8], 3}, {NULL}};
const Segment SGI319594[] = {{&S41139[13], 10}, {NULL}};
const Segment SGI319595[] = {{&S41139[35], 5}, {NULL}};
const Segment SGI319596[] = {{&S41139[49], 5}, {NULL}};
const Segment SGI319597[] = {{&S41140[30], 3}, {NULL}};
const Segment SGI319598[] = {{&S41140[8], 3}, {NULL}};
const Segment SGI319599[] = {{&S41140[13], 10}, {NULL}};
const Segment SGI319600[] = {{&S41140[35], 5}, {NULL}};
const Segment SGI319601[] = {{&S41140[49], 3}, {NULL}};
const Segment SGI319602[] = {{&S41141[30], 3}, {NULL}};
const Segment SGI319603[] = {{&S41141[8], 3}, {NULL}};
const Segment SGI319604[] = {{&S41141[13], 10}, {NULL}};
const Segment SGI319605[] = {{&S41141[35], 5}, {NULL}};
const Segment SGI319606[] = {{&S41141[49], 6}, {NULL}};
const Segment SGI319607[] = {{&S41254[30], 4}, {NULL}};
const Segment SGI319608[] = {{&S41254[8], 3}, {NULL}};
const Segment SGI319609[] = {{&S41254[13], 10}, {NULL}};
const Segment SGI319610[] = {{&S41254[70], 7}, {NULL}};
const Segment SGI319611[] = {{&S41254[63], 14}, {NULL}};
const Segment SGI319612[] = {{&S41255[30], 4}, {NULL}};
const Segment SGI319613[] = {{&S41255[8], 3}, {NULL}};
const Segment SGI319614[] = {{&S41255[13], 10}, {NULL}};
const Segment SGI319615[] = {{&S41255[43], 7}, {NULL}};
const Segment SGI319616[] = {{&S41255[36], 14}, {NULL}};
const Segment SGI319617[] = {{&S41256[30], 4}, {NULL}};
const Segment SGI319618[] = {{&S41256[8], 3}, {NULL}};
const Segment SGI319619[] = {{&S41256[13], 10}, {NULL}};
const Segment SGI319620[] = {{&S41256[72], 7}, {NULL}};
const Segment SGI319621[] = {{&S41256[65], 14}, {NULL}};
const Segment SGI319622[] = {{&S41256[81], 6}, {NULL}};
const Segment SGI319623[] = {{&S41328[30], 3}, {NULL}};
const Segment SGI319624[] = {{&S41328[8], 3}, {NULL}};
const Segment SGI319625[] = {{&S41328[13], 10}, {NULL}};
const Segment SGI319626[] = {{&S41328[42], 7}, {NULL}};
const Segment SGI319627[] = {{&S41328[35], 14}, {NULL}};
const Segment SGI319628[] = {{&S41328[74], 4}, {NULL}};
const Segment SGI319629[] = {{&S41329[30], 3}, {NULL}};
const Segment SGI319630[] = {{&S41329[8], 3}, {NULL}};
const Segment SGI319631[] = {{&S41329[13], 10}, {NULL}};
const Segment SGI319632[] = {{&S41329[42], 7}, {NULL}};
const Segment SGI319633[] = {{&S41329[35], 14}, {NULL}};
const Segment SGI319634[] = {{&S41329[51], 3}, {NULL}};
const Segment SGI319635[] = {{&S41329[66], 4}, {NULL}};
const Segment SGI319636[] = {{&S41331[30], 3}, {NULL}};
const Segment SGI319637[] = {{&S41331[8], 3}, {NULL}};
const Segment SGI319638[] = {{&S41331[13], 10}, {NULL}};
const Segment SGI319639[] = {{&S41331[42], 7}, {NULL}};
const Segment SGI319640[] = {{&S41331[61], 4}, {NULL}};
const Segment SGI319641[] = {{&S41333[30], 3}, {&S41333[30], 3}, {NULL}};
const Segment SGI319642[] = {{&S41333[8], 3}, {&S41333[8], 3}, {NULL}};
const Segment SGI319643[] = {{&S41333[13], 10}, {&S41333[13], 10}, {NULL}};
const Segment SGI319644[] = {{&S41333[42], 7}, {&S41333[42], 7}, {NULL}};
const Segment SGI319645[] = {{&S41333[35], 14}, {&S41333[35], 14}, {NULL}};
const Segment SGI319646[] = {{&S41333[51], 3}, {&S41333[51], 3}, {NULL}};
const Segment SGI319647[] = {{&S41333[140], 4}, {NULL}};
const Segment SGI319648[] = {{&S41334[30], 3}, {NULL}};
const Segment SGI319649[] = {{&S41334[8], 3}, {NULL}};
const Segment SGI319650[] = {{&S41334[13], 10}, {NULL}};
const Segment SGI319651[] = {{&S41334[42], 7}, {&S41334[145], 20}, {NULL}};
const Segment SGI319652[] = {{&S41334[35], 14}, {NULL}};
const Segment SGI319653[] = {{&S41334[146], 15}, {NULL}};
const Segment SGI319654[] = {{&S41334[177], 4}, {NULL}};
const Segment SGI319655[] = {{&S41335[30], 3}, {NULL}};
const Segment SGI319656[] = {{&S41335[8], 3}, {NULL}};
const Segment SGI319657[] = {{&S41335[13], 10}, {NULL}};
const Segment SGI319658[] = {{&S41335[42], 7}, {NULL}};
const Segment SGI319659[] = {{&S41335[35], 14}, {NULL}};
const Segment SGI319660[] = {{&S41335[64], 3}, {NULL}};
const Segment SGI319661[] = {{&S41335[175], 4}, {NULL}};
const Segment SGI319662[] = {{&S69872[3], 4}, {NULL}};
const Segment SGI319663[] = {{&S41939[30], 3}, {NULL}};
const Segment SGI319664[] = {{&S41939[8], 3}, {NULL}};
const Segment SGI319665[] = {{&S41939[13], 10}, {NULL}};
const Segment SGI319666[] = {{&S41939[42], 7}, {NULL}};
const Segment SGI319667[] = {{&S41939[35], 14}, {NULL}};
const Segment SGI319668[] = {{&S41939[64], 4}, {NULL}};
const Segment SGI319669[] = {{&S55756[0], 83}, {NULL}};
const Segment SGI319670[] = {{&S42001[30], 3}, {NULL}};
const Segment SGI319671[] = {{&S42001[8], 3}, {NULL}};
const Segment SGI319672[] = {{&S42001[13], 10}, {NULL}};
const Segment SGI319673[] = {{&S42001[42], 7}, {NULL}};
const Segment SGI319674[] = {{&S42001[35], 14}, {NULL}};
const Segment SGI319675[] = {{&S42001[56], 13}, {NULL}};
const Segment SGI319676[] = {{&S42001[51], 3}, {NULL}};
const Segment SGI319677[] = {{&S42002[30], 3}, {NULL}};
const Segment SGI319678[] = {{&S42002[8], 3}, {NULL}};
const Segment SGI319679[] = {{&S42002[13], 10}, {NULL}};
const Segment SGI319680[] = {{&S42002[42], 7}, {NULL}};
const Segment SGI319681[] = {{&S42002[35], 14}, {NULL}};
const Segment SGI319682[] = {{&S42002[124], 15}, {NULL}};
const Segment SGI319683[] = {{&S42002[51], 13}, {&S42002[51], 13}, {NULL}};
const Segment SGI319684[] = {{&S42002[66], 3}, {NULL}};
const Segment SGI319685[] = {{&S42003[30], 3}, {NULL}};
const Segment SGI319686[] = {{&S42003[8], 3}, {NULL}};
const Segment SGI319687[] = {{&S42003[13], 10}, {NULL}};
const Segment SGI319688[] = {{&S42003[42], 7}, {NULL}};
const Segment SGI319689[] = {{&S42003[35], 14}, {NULL}};
const Segment SGI319690[] = {{&S42003[51], 13}, {&S42003[51], 13}, {NULL}};
const Segment SGI319691[] = {{&S42004[30], 3}, {NULL}};
const Segment SGI319692[] = {{&S42004[8], 3}, {NULL}};
const Segment SGI319693[] = {{&S42004[13], 10}, {NULL}};
const Segment SGI319694[] = {{&S42004[42], 7}, {NULL}};
const Segment SGI319695[] = {{&S42004[35], 14}, {NULL}};
const Segment SGI319696[] = {{&S42004[51], 13}, {NULL}};
const Segment SGI319697[] = {{&S42005[30], 3}, {&S42005[30], 3}, {NULL}};
const Segment SGI319698[] = {{&S42005[8], 3}, {&S42005[8], 3}, {NULL}};
const Segment SGI319699[] = {{&S42005[13], 10}, {&S42005[13], 10}, {NULL}};
const Segment SGI319700[] = {{&S42005[42], 7}, {&S42005[42], 7}, {NULL}};
const Segment SGI319701[] = {{&S42005[35], 14}, {&S42005[35], 14}, {NULL}};
const Segment SGI319702[] = {{&S42005[114], 13}, {NULL}};
const Segment SGI319703[] = {{&S42005[51], 3}, {&S42005[51], 3}, {NULL}};
const Segment SGI319704[] = {{&S42006[30], 3}, {NULL}};
const Segment SGI319705[] = {{&S42006[8], 3}, {NULL}};
const Segment SGI319706[] = {{&S42006[13], 10}, {NULL}};
const Segment SGI319707[] = {{&S42006[42], 7}, {NULL}};
const Segment SGI319708[] = {{&S42006[35], 14}, {NULL}};
const Segment SGI319709[] = {{&S42006[56], 13}, {&S42006[56], 13}, {NULL}};
const Segment SGI319710[] = {{&S42006[51], 3}, {NULL}};
const Segment SGI319711[] = {{&S42007[30], 3}, {NULL}};
const Segment SGI319712[] = {{&S42007[8], 3}, {NULL}};
const Segment SGI319713[] = {{&S42007[13], 10}, {NULL}};
const Segment SGI319714[] = {{&S42007[42], 7}, {NULL}};
const Segment SGI319715[] = {{&S42007[35], 14}, {NULL}};
const Segment SGI319716[] = {{&S42007[109], 15}, {NULL}};
const Segment SGI319717[] = {{&S42007[56], 13}, {NULL}};
const Segment SGI319718[] = {{&S42007[51], 3}, {NULL}};
const Segment SGI319719[] = {{&S42008[48], 3}, {NULL}};
const Segment SGI319720[] = {{&S42008[60], 7}, {NULL}};
const Segment SGI319721[] = {{&S42008[53], 14}, {NULL}};
const Segment SGI319722[] = {{&S42008[74], 13}, {NULL}};
const Segment SGI319723[] = {{&S42008[69], 3}, {NULL}};
const Segment SGI319724[] = {{&S42009[30], 3}, {NULL}};
const Segment SGI319725[] = {{&S42009[8], 3}, {NULL}};
const Segment SGI319726[] = {{&S42009[13], 10}, {NULL}};
const Segment SGI319727[] = {{&S42009[42], 7}, {NULL}};
const Segment SGI319728[] = {{&S42009[35], 14}, {NULL}};
const Segment SGI319729[] = {{&S42009[138], 15}, {NULL}};
const Segment SGI319730[] = {{&S42009[51], 6}, {NULL}};
const Segment SGI319731[] = {{&S42009[169], 13}, {NULL}};
const Segment SGI319732[] = {{&S42010[30], 3}, {&S42010[30], 3}, {NULL}};
const Segment SGI319733[] = {{&S42010[8], 3}, {&S42010[8], 3}, {NULL}};
const Segment SGI319734[] = {{&S42010[13], 10}, {&S42010[13], 10}, {NULL}};
const Segment SGI319735[] = {{&S42010[42], 7}, {&S42010[42], 7}, {NULL}};
const Segment SGI319736[] = {{&S42010[35], 14}, {&S42010[35], 14}, {NULL}};
const Segment SGI319737[] = {{&S42010[51], 13}, {NULL}};
const Segment SGI319738[] = {{&S42010[117], 3}, {NULL}};
const Segment SGI319739[] = {{&S42011[30], 3}, {NULL}};
const Segment SGI319740[] = {{&S42011[8], 3}, {NULL}};
const Segment SGI319741[] = {{&S42011[13], 10}, {NULL}};
const Segment SGI319742[] = {{&S42011[42], 7}, {NULL}};
const Segment SGI319743[] = {{&S42011[35], 14}, {NULL}};
const Segment SGI319744[] = {{&S42011[131], 15}, {NULL}};
const Segment SGI319745[] = {{&S42011[172], 13}, {NULL}};
const Segment SGI319746[] = {{&S58822[3], 5}, {NULL}};
const Segment SGI319747[] = {{&S58822[80], 13}, {NULL}};
const Segment SGI319748[] = {{&S59100[3], 4}, {NULL}};
const Segment SGI319749[] = {{&S59100[79], 13}, {NULL}};
const Segment SGI319750[] = {{&S42459[30], 3}, {&S42459[46], 3}, {NULL}};
const Segment SGI319751[] = {{&S42459[8], 3}, {&S42459[8], 3}, {NULL}};
const Segment SGI319752[] = {{&S42459[13], 10}, {&S42459[13], 10}, {NULL}};
const Segment SGI319753[] = {{&S42459[42], 7}, {&S42459[113], 7}, {NULL}};
const Segment SGI319754[] = {{&S42459[35], 14}, {&S42459[106], 14}, {NULL}};
const Segment SGI319755[] = {{&S42459[129], 7}, {NULL}};
const Segment SGI319756[] = {{&S42459[122], 3}, {NULL}};
const Segment SGI319757[] = {{&S42460[30], 3}, {NULL}};
const Segment SGI319758[] = {{&S42460[8], 3}, {NULL}};
const Segment SGI319759[] = {{&S42460[13], 10}, {NULL}};
const Segment SGI319760[] = {{&S42460[42], 7}, {NULL}};
const Segment SGI319761[] = {{&S42460[35], 14}, {NULL}};
const Segment SGI319762[] = {{&S42460[114], 15}, {NULL}};
const Segment SGI319763[] = {{&S42460[51], 6}, {NULL}};
const Segment SGI319764[] = {{&S42460[58], 7}, {NULL}};
const Segment SGI319765[] = {{&S42461[30], 3}, {NULL}};
const Segment SGI319766[] = {{&S42461[8], 3}, {NULL}};
const Segment SGI319767[] = {{&S42461[13], 10}, {NULL}};
const Segment SGI319768[] = {{&S42461[42], 7}, {NULL}};
const Segment SGI319769[] = {{&S42461[35], 14}, {NULL}};
const Segment SGI319770[] = {{&S42461[75], 7}, {NULL}};
const Segment SGI319771[] = {{&S42461[51], 3}, {NULL}};
const Segment SGI319772[] = {{&S42462[30], 3}, {&S42462[46], 3}, {NULL}};
const Segment SGI319773[] = {{&S42462[8], 3}, {&S42462[8], 3}, {NULL}};
const Segment SGI319774[] = {{&S42462[13], 10}, {&S42462[13], 10}, {NULL}};
const Segment SGI319775[] = {{&S42462[42], 7}, {&S42462[93], 7}, {NULL}};
const Segment SGI319776[] = {{&S42462[35], 14}, {&S42462[86], 14}, {NULL}};
const Segment SGI319777[] = {{&S42462[145], 15}, {NULL}};
const Segment SGI319778[] = {{&S42462[109], 7}, {NULL}};
const Segment SGI319779[] = {{&S42462[102], 3}, {NULL}};
const Segment SGI319780[] = {{&S42463[30], 3}, {NULL}};
const Segment SGI319781[] = {{&S42463[8], 3}, {NULL}};
const Segment SGI319782[] = {{&S42463[13], 10}, {NULL}};
const Segment SGI319783[] = {{&S42463[42], 7}, {NULL}};
const Segment SGI319784[] = {{&S42463[35], 14}, {NULL}};
const Segment SGI319785[] = {{&S42463[51], 7}, {NULL}};
const Segment SGI319786[] = {{&S42464[30], 3}, {NULL}};
const Segment SGI319787[] = {{&S42464[8], 3}, {NULL}};
const Segment SGI319788[] = {{&S42464[13], 10}, {NULL}};
const Segment SGI319789[] = {{&S42464[55], 7}, {NULL}};
const Segment SGI319790[] = {{&S42464[48], 14}, {NULL}};
const Segment SGI319791[] = {{&S42464[64], 7}, {NULL}};
const Segment SGI319792[] = {{&S42465[30], 3}, {NULL}};
const Segment SGI319793[] = {{&S42465[8], 3}, {NULL}};
const Segment SGI319794[] = {{&S42465[13], 10}, {NULL}};
const Segment SGI319795[] = {{&S42465[55], 7}, {NULL}};
const Segment SGI319796[] = {{&S42465[48], 14}, {NULL}};
const Segment SGI319797[] = {{&S42465[64], 7}, {NULL}};
const Segment SGI319798[] = {{&S42466[30], 3}, {NULL}};
const Segment SGI319799[] = {{&S42466[8], 3}, {NULL}};
const Segment SGI319800[] = {{&S42466[13], 10}, {NULL}};
const Segment SGI319801[] = {{&S42466[55], 7}, {NULL}};
const Segment SGI319802[] = {{&S42466[48], 14}, {NULL}};
const Segment SGI319803[] = {{&S42466[64], 7}, {NULL}};
const Segment SGI319804[] = {{&S42467[30], 3}, {NULL}};
const Segment SGI319805[] = {{&S42467[8], 3}, {NULL}};
const Segment SGI319806[] = {{&S42467[13], 10}, {NULL}};
const Segment SGI319807[] = {{&S42467[55], 7}, {NULL}};
const Segment SGI319808[] = {{&S42467[48], 14}, {NULL}};
const Segment SGI319809[] = {{&S42467[64], 7}, {NULL}};
const Segment SGI319810[] = {{&S42468[30], 3}, {NULL}};
const Segment SGI319811[] = {{&S42468[8], 3}, {NULL}};
const Segment SGI319812[] = {{&S42468[13], 10}, {NULL}};
const Segment SGI319813[] = {{&S42468[55], 7}, {NULL}};
const Segment SGI319814[] = {{&S42468[48], 14}, {NULL}};
const Segment SGI319815[] = {{&S42468[64], 7}, {NULL}};
const Segment SGI319816[] = {{&S42469[30], 3}, {NULL}};
const Segment SGI319817[] = {{&S42469[8], 3}, {NULL}};
const Segment SGI319818[] = {{&S42469[13], 10}, {NULL}};
const Segment SGI319819[] = {{&S42469[42], 7}, {NULL}};
const Segment SGI319820[] = {{&S42469[35], 14}, {NULL}};
const Segment SGI319821[] = {{&S42469[75], 7}, {NULL}};
const Segment SGI319822[] = {{&S42469[51], 3}, {NULL}};
const Segment SGI319823[] = {{&S42470[30], 3}, {NULL}};
const Segment SGI319824[] = {{&S42470[8], 3}, {NULL}};
const Segment SGI319825[] = {{&S42470[13], 10}, {NULL}};
const Segment SGI319826[] = {{&S42470[42], 7}, {NULL}};
const Segment SGI319827[] = {{&S42470[35], 14}, {NULL}};
const Segment SGI319828[] = {{&S42470[114], 15}, {NULL}};
const Segment SGI319829[] = {{&S42470[51], 6}, {NULL}};
const Segment SGI319830[] = {{&S42470[58], 7}, {NULL}};
const Segment SGI319831[] = {{&S42471[30], 3}, {NULL}};
const Segment SGI319832[] = {{&S42471[8], 3}, {NULL}};
const Segment SGI319833[] = {{&S42471[13], 10}, {NULL}};
const Segment SGI319834[] = {{&S42471[42], 7}, {NULL}};
const Segment SGI319835[] = {{&S42471[35], 14}, {NULL}};
const Segment SGI319836[] = {{&S42471[87], 15}, {NULL}};
const Segment SGI319837[] = {{&S42471[51], 7}, {NULL}};
const Segment SGI319838[] = {{&S42472[30], 3}, {NULL}};
const Segment SGI319839[] = {{&S42472[8], 3}, {NULL}};
const Segment SGI319840[] = {{&S42472[13], 10}, {NULL}};
const Segment SGI319841[] = {{&S42472[42], 7}, {NULL}};
const Segment SGI319842[] = {{&S42472[35], 14}, {NULL}};
const Segment SGI319843[] = {{&S42472[131], 15}, {NULL}};
const Segment SGI319844[] = {{&S42472[152], 7}, {NULL}};
const Segment SGI319845[] = {{&S42473[30], 3}, {NULL}};
const Segment SGI319846[] = {{&S42473[8], 3}, {NULL}};
const Segment SGI319847[] = {{&S42473[13], 10}, {NULL}};
const Segment SGI319848[] = {{&S42473[42], 7}, {NULL}};
const Segment SGI319849[] = {{&S42473[35], 14}, {NULL}};
const Segment SGI319850[] = {{&S42473[64], 7}, {NULL}};
const Segment SGI319851[] = {{&S42474[30], 3}, {NULL}};
const Segment SGI319852[] = {{&S42474[8], 3}, {NULL}};
const Segment SGI319853[] = {{&S42474[13], 10}, {NULL}};
const Segment SGI319854[] = {{&S42474[42], 7}, {NULL}};
const Segment SGI319855[] = {{&S42474[35], 14}, {NULL}};
const Segment SGI319856[] = {{&S42474[51], 6}, {NULL}};
const Segment SGI319857[] = {{&S42474[98], 7}, {NULL}};
const Segment SGI319858[] = {{&S42475[30], 3}, {NULL}};
const Segment SGI319859[] = {{&S42475[8], 3}, {NULL}};
const Segment SGI319860[] = {{&S42475[13], 10}, {NULL}};
const Segment SGI319861[] = {{&S42475[55], 7}, {NULL}};
const Segment SGI319862[] = {{&S42475[48], 14}, {NULL}};
const Segment SGI319863[] = {{&S42475[79], 7}, {NULL}};
const Segment SGI319864[] = {{&S42475[64], 13}, {NULL}};
const Segment SGI319865[] = {{&S42476[30], 3}, {NULL}};
const Segment SGI319866[] = {{&S42476[8], 3}, {NULL}};
const Segment SGI319867[] = {{&S42476[13], 10}, {NULL}};
const Segment SGI319868[] = {{&S42476[55], 7}, {NULL}};
const Segment SGI319869[] = {{&S42476[48], 14}, {NULL}};
const Segment SGI319870[] = {{&S42476[69], 7}, {NULL}};
const Segment SGI319871[] = {{&S42476[64], 3}, {NULL}};
const Segment SGI319872[] = {{&S42477[30], 3}, {NULL}};
const Segment SGI319873[] = {{&S42477[8], 3}, {NULL}};
const Segment SGI319874[] = {{&S42477[13], 10}, {NULL}};
const Segment SGI319875[] = {{&S42477[55], 7}, {NULL}};
const Segment SGI319876[] = {{&S42477[48], 14}, {NULL}};
const Segment SGI319877[] = {{&S42477[64], 7}, {&S42477[64], 7}, {NULL}};
const Segment SGI319878[] = {{&S42478[30], 3}, {NULL}};
const Segment SGI319879[] = {{&S42478[8], 3}, {NULL}};
const Segment SGI319880[] = {{&S42478[13], 10}, {NULL}};
const Segment SGI319881[] = {{&S42478[42], 7}, {NULL}};
const Segment SGI319882[] = {{&S42478[35], 14}, {NULL}};
const Segment SGI319883[] = {{&S42478[51], 6}, {NULL}};
const Segment SGI319884[] = {{&S42478[98], 7}, {NULL}};
const Segment SGI319885[] = {{&S42479[30], 3}, {NULL}};
const Segment SGI319886[] = {{&S42479[8], 3}, {NULL}};
const Segment SGI319887[] = {{&S42479[13], 10}, {NULL}};
const Segment SGI319888[] = {{&S42479[55], 7}, {NULL}};
const Segment SGI319889[] = {{&S42479[48], 14}, {NULL}};
const Segment SGI319890[] = {{&S42479[64], 7}, {NULL}};
const Segment SGI319891[] = {{&S42479[73], 3}, {NULL}};
const Segment SGI319892[] = {{&S42480[30], 3}, {NULL}};
const Segment SGI319893[] = {{&S42480[8], 3}, {NULL}};
const Segment SGI319894[] = {{&S42480[13], 10}, {NULL}};
const Segment SGI319895[] = {{&S42480[55], 7}, {NULL}};
const Segment SGI319896[] = {{&S42480[48], 14}, {NULL}};
const Segment SGI319897[] = {{&S42480[64], 7}, {&S42480[64], 7}, {NULL}};
const Segment SGI319898[] = {{&S42481[30], 3}, {NULL}};
const Segment SGI319899[] = {{&S42481[8], 3}, {NULL}};
const Segment SGI319900[] = {{&S42481[13], 10}, {NULL}};
const Segment SGI319901[] = {{&S42481[55], 7}, {NULL}};
const Segment SGI319902[] = {{&S42481[48], 14}, {NULL}};
const Segment SGI319903[] = {{&S42481[69], 7}, {NULL}};
const Segment SGI319904[] = {{&S42481[64], 3}, {NULL}};
const Segment SGI319905[] = {{&S42482[30], 3}, {NULL}};
const Segment SGI319906[] = {{&S42482[8], 3}, {NULL}};
const Segment SGI319907[] = {{&S42482[13], 10}, {NULL}};
const Segment SGI319908[] = {{&S42482[55], 7}, {NULL}};
const Segment SGI319909[] = {{&S42482[48], 14}, {NULL}};
const Segment SGI319910[] = {{&S42482[79], 7}, {NULL}};
const Segment SGI319911[] = {{&S42482[64], 13}, {NULL}};
const Segment SGI319912[] = {{&S42483[30], 3}, {NULL}};
const Segment SGI319913[] = {{&S42483[8], 3}, {NULL}};
const Segment SGI319914[] = {{&S42483[13], 10}, {NULL}};
const Segment SGI319915[] = {{&S42483[42], 7}, {NULL}};
const Segment SGI319916[] = {{&S42483[35], 14}, {NULL}};
const Segment SGI319917[] = {{&S42483[64], 7}, {NULL}};
const Segment SGI319918[] = {{&S42484[30], 3}, {NULL}};
const Segment SGI319919[] = {{&S42484[8], 3}, {NULL}};
const Segment SGI319920[] = {{&S42484[13], 10}, {NULL}};
const Segment SGI319921[] = {{&S42484[55], 7}, {NULL}};
const Segment SGI319922[] = {{&S42484[48], 14}, {NULL}};
const Segment SGI319923[] = {{&S42484[64], 7}, {NULL}};
const Segment SGI319924[] = {{&S42484[73], 13}, {NULL}};
const Segment SGI319925[] = {{&S42485[30], 3}, {NULL}};
const Segment SGI319926[] = {{&S42485[8], 3}, {NULL}};
const Segment SGI319927[] = {{&S42485[13], 10}, {NULL}};
const Segment SGI319928[] = {{&S42485[55], 7}, {NULL}};
const Segment SGI319929[] = {{&S42485[48], 14}, {NULL}};
const Segment SGI319930[] = {{&S42485[64], 7}, {&S42485[64], 7}, {NULL}};
const Segment SGI319931[] = {{&S42486[30], 3}, {NULL}};
const Segment SGI319932[] = {{&S42486[8], 3}, {NULL}};
const Segment SGI319933[] = {{&S42486[13], 10}, {NULL}};
const Segment SGI319934[] = {{&S42486[55], 7}, {NULL}};
const Segment SGI319935[] = {{&S42486[48], 14}, {NULL}};
const Segment SGI319936[] = {{&S42486[69], 7}, {NULL}};
const Segment SGI319937[] = {{&S42486[64], 3}, {NULL}};
const Segment SGI319938[] = {{&S42487[30], 3}, {NULL}};
const Segment SGI319939[] = {{&S42487[8], 3}, {NULL}};
const Segment SGI319940[] = {{&S42487[13], 10}, {NULL}};
const Segment SGI319941[] = {{&S42487[55], 7}, {NULL}};
const Segment SGI319942[] = {{&S42487[48], 14}, {NULL}};
const Segment SGI319943[] = {{&S42487[79], 7}, {NULL}};
const Segment SGI319944[] = {{&S42487[64], 13}, {NULL}};
const Segment SGI319945[] = {{&S42488[30], 3}, {NULL}};
const Segment SGI319946[] = {{&S42488[8], 3}, {NULL}};
const Segment SGI319947[] = {{&S42488[13], 10}, {NULL}};
const Segment SGI319948[] = {{&S42488[55], 7}, {NULL}};
const Segment SGI319949[] = {{&S42488[48], 14}, {NULL}};
const Segment SGI319950[] = {{&S42488[64], 7}, {&S42488[64], 7}, {NULL}};
const Segment SGI319951[] = {{&S42489[30], 4}, {NULL}};
const Segment SGI319952[] = {{&S42489[8], 3}, {NULL}};
const Segment SGI319953[] = {{&S42489[13], 10}, {NULL}};
const Segment SGI319954[] = {{&S42489[43], 7}, {NULL}};
const Segment SGI319955[] = {{&S42489[36], 14}, {NULL}};
const Segment SGI319956[] = {{&S42489[52], 6}, {NULL}};
const Segment SGI319957[] = {{&S42489[99], 7}, {NULL}};
const Segment SGI319958[] = {{&S42498[30], 3}, {NULL}};
const Segment SGI319959[] = {{&S42498[8], 3}, {NULL}};
const Segment SGI319960[] = {{&S42498[13], 10}, {NULL}};
const Segment SGI319961[] = {{&S42498[42], 7}, {NULL}};
const Segment SGI319962[] = {{&S42498[35], 14}, {NULL}};
const Segment SGI319963[] = {{&S42498[64], 11}, {NULL}};
const Segment SGI319964[] = {{&S42498[64], 16}, {NULL}};
const Segment SGI319965[] = {{&S42499[30], 3}, {NULL}};
const Segment SGI319966[] = {{&S42499[8], 3}, {NULL}};
const Segment SGI319967[] = {{&S42499[13], 10}, {NULL}};
const Segment SGI319968[] = {{&S42499[42], 7}, {NULL}};
const Segment SGI319969[] = {{&S42499[35], 14}, {NULL}};
const Segment SGI319970[] = {{&S42499[71], 11}, {NULL}};
const Segment SGI319971[] = {{&S42499[51], 6}, {NULL}};
const Segment SGI319972[] = {{&S42499[71], 16}, {NULL}};
const Segment SGI319973[] = {{&S42508[30], 3}, {NULL}};
const Segment SGI319974[] = {{&S42508[8], 3}, {NULL}};
const Segment SGI319975[] = {{&S42508[13], 10}, {NULL}};
const Segment SGI319976[] = {{&S42508[42], 7}, {NULL}};
const Segment SGI319977[] = {{&S42508[35], 14}, {NULL}};
const Segment SGI319978[] = {{&S42508[64], 11}, {NULL}};
const Segment SGI319979[] = {{&S42508[156], 15}, {NULL}};
const Segment SGI319980[] = {{&S42508[64], 16}, {NULL}};
const Segment SGI319981[] = {{&S42510[30], 3}, {NULL}};
const Segment SGI319982[] = {{&S42510[8], 3}, {NULL}};
const Segment SGI319983[] = {{&S42510[13], 10}, {NULL}};
const Segment SGI319984[] = {{&S42510[42], 7}, {NULL}};
const Segment SGI319985[] = {{&S42510[35], 14}, {NULL}};
const Segment SGI319986[] = {{&S42510[64], 11}, {NULL}};
const Segment SGI319987[] = {{&S42510[64], 16}, {NULL}};
const Segment SGI319988[] = {{&S42511[30], 3}, {NULL}};
const Segment SGI319989[] = {{&S42511[8], 3}, {NULL}};
const Segment SGI319990[] = {{&S42511[13], 10}, {NULL}};
const Segment SGI319991[] = {{&S42511[42], 7}, {NULL}};
const Segment SGI319992[] = {{&S42511[35], 14}, {NULL}};
const Segment SGI319993[] = {{&S42511[71], 11}, {NULL}};
const Segment SGI319994[] = {{&S42511[163], 15}, {NULL}};
const Segment SGI319995[] = {{&S42511[51], 6}, {NULL}};
const Segment SGI319996[] = {{&S42511[71], 16}, {NULL}};
const Segment SGI319997[] = {{&S42513[30], 3}, {NULL}};
const Segment SGI319998[] = {{&S42513[247], 9}, {NULL}};
const Segment SGI319999[] = {{&S42513[8], 3}, {NULL}};
const Segment SGI320000[] = {{&S42513[13], 10}, {NULL}};
const Segment SGI320001[] = {{&S42513[42], 7}, {NULL}};
const Segment SGI320002[] = {{&S42513[35], 14}, {NULL}};
const Segment SGI320003[] = {{&S42513[71], 11}, {NULL}};
const Segment SGI320004[] = {{&S42513[163], 15}, {NULL}};
const Segment SGI320005[] = {{&S42513[51], 6}, {NULL}};
const Segment SGI320006[] = {{&S42513[71], 16}, {NULL}};
const Segment SGI320007[] = {{&S42515[30], 3}, {NULL}};
const Segment SGI320008[] = {{&S42515[8], 3}, {NULL}};
const Segment SGI320009[] = {{&S42515[13], 10}, {NULL}};
const Segment SGI320010[] = {{&S42515[42], 7}, {NULL}};
const Segment SGI320011[] = {{&S42515[35], 14}, {NULL}};
const Segment SGI320012[] = {{&S42515[64], 11}, {NULL}};
const Segment SGI320013[] = {{&S42515[156], 15}, {NULL}};
const Segment SGI320014[] = {{&S42515[64], 16}, {NULL}};
const Segment SGI320015[] = {{&S42515[228], 9}, {NULL}};
const Segment SGI320016[] = {{&S42516[30], 3}, {NULL}};
const Segment SGI320017[] = {{&S42516[8], 3}, {NULL}};
const Segment SGI320018[] = {{&S42516[13], 10}, {NULL}};
const Segment SGI320019[] = {{&S42516[42], 7}, {NULL}};
const Segment SGI320020[] = {{&S42516[35], 14}, {NULL}};
const Segment SGI320021[] = {{&S42516[71], 11}, {NULL}};
const Segment SGI320022[] = {{&S42516[51], 6}, {NULL}};
const Segment SGI320023[] = {{&S42516[71], 16}, {NULL}};
const Segment SGI320024[] = {{&S42517[30], 3}, {NULL}};
const Segment SGI320025[] = {{&S42517[8], 3}, {NULL}};
const Segment SGI320026[] = {{&S42517[13], 10}, {NULL}};
const Segment SGI320027[] = {{&S42517[42], 7}, {NULL}};
const Segment SGI320028[] = {{&S42517[35], 14}, {NULL}};
const Segment SGI320029[] = {{&S42517[188], 11}, {NULL}};
const Segment SGI320030[] = {{&S42517[131], 15}, {NULL}};
const Segment SGI320031[] = {{&S42517[188], 16}, {NULL}};
const Segment SGI320032[] = {{&S42518[30], 3}, {NULL}};
const Segment SGI320033[] = {{&S42518[8], 3}, {NULL}};
const Segment SGI320034[] = {{&S42518[13], 10}, {NULL}};
const Segment SGI320035[] = {{&S42518[42], 7}, {NULL}};
const Segment SGI320036[] = {{&S42518[35], 14}, {NULL}};
const Segment SGI320037[] = {{&S42518[71], 11}, {NULL}};
const Segment SGI320038[] = {{&S42518[163], 15}, {NULL}};
const Segment SGI320039[] = {{&S42518[51], 6}, {NULL}};
const Segment SGI320040[] = {{&S42518[71], 16}, {NULL}};
const Segment SGI320041[] = {{&S42519[30], 3}, {NULL}};
const Segment SGI320042[] = {{&S42519[8], 3}, {NULL}};
const Segment SGI320043[] = {{&S42519[13], 10}, {NULL}};
const Segment SGI320044[] = {{&S42519[42], 7}, {NULL}};
const Segment SGI320045[] = {{&S42519[35], 14}, {NULL}};
const Segment SGI320046[] = {{&S42519[71], 11}, {NULL}};
const Segment SGI320047[] = {{&S42519[51], 6}, {NULL}};
const Segment SGI320048[] = {{&S42519[71], 16}, {NULL}};
const Segment SGI320049[] = {{&S42520[30], 3}, {NULL}};
const Segment SGI320050[] = {{&S42520[226], 9}, {NULL}};
const Segment SGI320051[] = {{&S42520[8], 3}, {NULL}};
const Segment SGI320052[] = {{&S42520[13], 10}, {NULL}};
const Segment SGI320053[] = {{&S42520[42], 7}, {NULL}};
const Segment SGI320054[] = {{&S42520[35], 14}, {NULL}};
const Segment SGI320055[] = {{&S42520[71], 11}, {NULL}};
const Segment SGI320056[] = {{&S42520[51], 6}, {NULL}};
const Segment SGI320057[] = {{&S42520[71], 16}, {NULL}};
const Segment SGI320058[] = {{&S42521[30], 4}, {NULL}};
const Segment SGI320059[] = {{&S42521[8], 3}, {NULL}};
const Segment SGI320060[] = {{&S42521[13], 10}, {NULL}};
const Segment SGI320061[] = {{&S42521[43], 7}, {NULL}};
const Segment SGI320062[] = {{&S42521[36], 14}, {NULL}};
const Segment SGI320063[] = {{&S42521[72], 11}, {NULL}};
const Segment SGI320064[] = {{&S42521[52], 6}, {NULL}};
const Segment SGI320065[] = {{&S42521[72], 16}, {NULL}};
const Segment SGI320066[] = {{&S42584[30], 3}, {NULL}};
const Segment SGI320067[] = {{&S42584[8], 3}, {NULL}};
const Segment SGI320068[] = {{&S42584[13], 10}, {NULL}};
const Segment SGI320069[] = {{&S42584[42], 7}, {NULL}};
const Segment SGI320070[] = {{&S42584[35], 14}, {NULL}};
const Segment SGI320071[] = {{&S42584[71], 11}, {NULL}};
const Segment SGI320072[] = {{&S42584[51], 6}, {NULL}};
const Segment SGI320073[] = {{&S42584[71], 16}, {NULL}};
const Segment SGI320074[] = {{&S42585[30], 3}, {NULL}};
const Segment SGI320075[] = {{&S42585[8], 3}, {NULL}};
const Segment SGI320076[] = {{&S42585[13], 10}, {NULL}};
const Segment SGI320077[] = {{&S42585[42], 7}, {NULL}};
const Segment SGI320078[] = {{&S42585[35], 14}, {NULL}};
const Segment SGI320079[] = {{&S42585[64], 11}, {NULL}};
const Segment SGI320080[] = {{&S42585[64], 16}, {NULL}};
const Segment SGI320081[] = {{&S42588[30], 3}, {NULL}};
const Segment SGI320082[] = {{&S42588[8], 3}, {NULL}};
const Segment SGI320083[] = {{&S42588[13], 10}, {NULL}};
const Segment SGI320084[] = {{&S42588[42], 7}, {NULL}};
const Segment SGI320085[] = {{&S42588[35], 14}, {NULL}};
const Segment SGI320086[] = {{&S42588[64], 11}, {NULL}};
const Segment SGI320087[] = {{&S42588[64], 16}, {NULL}};
const Segment SGI320088[] = {{&S42589[30], 3}, {NULL}};
const Segment SGI320089[] = {{&S42589[8], 3}, {NULL}};
const Segment SGI320090[] = {{&S42589[13], 10}, {NULL}};
const Segment SGI320091[] = {{&S42589[42], 7}, {NULL}};
const Segment SGI320092[] = {{&S42589[35], 14}, {NULL}};
const Segment SGI320093[] = {{&S42589[71], 11}, {NULL}};
const Segment SGI320094[] = {{&S42589[164], 15}, {NULL}};
const Segment SGI320095[] = {{&S42589[51], 6}, {NULL}};
const Segment SGI320096[] = {{&S42589[71], 16}, {NULL}};
const Segment SGI320097[] = {{&S42592[30], 3}, {NULL}};
const Segment SGI320098[] = {{&S42592[8], 3}, {NULL}};
const Segment SGI320099[] = {{&S42592[13], 10}, {NULL}};
const Segment SGI320100[] = {{&S42592[42], 7}, {NULL}};
const Segment SGI320101[] = {{&S42592[35], 14}, {NULL}};
const Segment SGI320102[] = {{&S42592[64], 11}, {NULL}};
const Segment SGI320103[] = {{&S42592[157], 15}, {NULL}};
const Segment SGI320104[] = {{&S42592[64], 16}, {NULL}};
const Segment SGI320105[] = {{&S42595[30], 3}, {NULL}};
const Segment SGI320106[] = {{&S42595[8], 3}, {NULL}};
const Segment SGI320107[] = {{&S42595[13], 10}, {NULL}};
const Segment SGI320108[] = {{&S42595[42], 7}, {NULL}};
const Segment SGI320109[] = {{&S42595[35], 14}, {NULL}};
const Segment SGI320110[] = {{&S42595[71], 11}, {NULL}};
const Segment SGI320111[] = {{&S42595[51], 6}, {NULL}};
const Segment SGI320112[] = {{&S42595[71], 16}, {NULL}};
const Segment SGI320113[] = {{&S42596[30], 3}, {NULL}};
const Segment SGI320114[] = {{&S42596[8], 3}, {NULL}};
const Segment SGI320115[] = {{&S42596[13], 10}, {NULL}};
const Segment SGI320116[] = {{&S42596[42], 7}, {NULL}};
const Segment SGI320117[] = {{&S42596[35], 14}, {NULL}};
const Segment SGI320118[] = {{&S42596[64], 11}, {NULL}};
const Segment SGI320119[] = {{&S42596[157], 15}, {NULL}};
const Segment SGI320120[] = {{&S42596[64], 16}, {NULL}};
const Segment SGI320121[] = {{&S42597[30], 3}, {NULL}};
const Segment SGI320122[] = {{&S42597[8], 3}, {NULL}};
const Segment SGI320123[] = {{&S42597[13], 10}, {NULL}};
const Segment SGI320124[] = {{&S42597[42], 7}, {NULL}};
const Segment SGI320125[] = {{&S42597[35], 14}, {NULL}};
const Segment SGI320126[] = {{&S42597[71], 11}, {NULL}};
const Segment SGI320127[] = {{&S42597[164], 15}, {NULL}};
const Segment SGI320128[] = {{&S42597[51], 6}, {NULL}};
const Segment SGI320129[] = {{&S42597[71], 16}, {NULL}};
const Segment SGI320130[] = {{&S42600[30], 3}, {NULL}};
const Segment SGI320131[] = {{&S42600[8], 3}, {NULL}};
const Segment SGI320132[] = {{&S42600[13], 10}, {NULL}};
const Segment SGI320133[] = {{&S42600[42], 7}, {NULL}};
const Segment SGI320134[] = {{&S42600[35], 14}, {NULL}};
const Segment SGI320135[] = {{&S42600[64], 11}, {NULL}};
const Segment SGI320136[] = {{&S42600[157], 15}, {NULL}};
const Segment SGI320137[] = {{&S42600[210], 4}, {NULL}};
const Segment SGI320138[] = {{&S42600[64], 16}, {NULL}};
const Segment SGI320139[] = {{&S42601[30], 3}, {NULL}};
const Segment SGI320140[] = {{&S42601[8], 3}, {NULL}};
const Segment SGI320141[] = {{&S42601[13], 10}, {NULL}};
const Segment SGI320142[] = {{&S42601[42], 7}, {NULL}};
const Segment SGI320143[] = {{&S42601[35], 14}, {NULL}};
const Segment SGI320144[] = {{&S42601[64], 11}, {NULL}};
const Segment SGI320145[] = {{&S42601[157], 15}, {NULL}};
const Segment SGI320146[] = {{&S42601[64], 16}, {NULL}};
const Segment SGI320147[] = {{&S42601[229], 9}, {NULL}};
const Segment SGI320148[] = {{&S76831[7], 4}, {NULL}};
const Segment SGI320149[] = {{&S76831[0], 122}, {NULL}};
const Segment SGI320150[] = {{&S42646[30], 3}, {NULL}};
const Segment SGI320151[] = {{&S42646[8], 3}, {NULL}};
const Segment SGI320152[] = {{&S42646[13], 10}, {NULL}};
const Segment SGI320153[] = {{&S42646[42], 7}, {NULL}};
const Segment SGI320154[] = {{&S42646[35], 14}, {NULL}};
const Segment SGI320155[] = {{&S42646[58], 7}, {NULL}};
const Segment SGI320156[] = {{&S42647[30], 3}, {NULL}};
const Segment SGI320157[] = {{&S42647[8], 3}, {NULL}};
const Segment SGI320158[] = {{&S42647[13], 10}, {NULL}};
const Segment SGI320159[] = {{&S42647[55], 7}, {NULL}};
const Segment SGI320160[] = {{&S42647[48], 14}, {NULL}};
const Segment SGI320161[] = {{&S42647[64], 7}, {NULL}};
const Segment SGI320162[] = {{&S42648[30], 3}, {NULL}};
const Segment SGI320163[] = {{&S42648[8], 3}, {NULL}};
const Segment SGI320164[] = {{&S42648[13], 10}, {NULL}};
const Segment SGI320165[] = {{&S42648[55], 7}, {NULL}};
const Segment SGI320166[] = {{&S42648[48], 14}, {NULL}};
const Segment SGI320167[] = {{&S42648[64], 7}, {NULL}};
const Segment SGI320168[] = {{&S42649[30], 3}, {NULL}};
const Segment SGI320169[] = {{&S42649[8], 3}, {NULL}};
const Segment SGI320170[] = {{&S42649[13], 10}, {NULL}};
const Segment SGI320171[] = {{&S42649[55], 7}, {NULL}};
const Segment SGI320172[] = {{&S42649[48], 14}, {NULL}};
const Segment SGI320173[] = {{&S42649[64], 7}, {NULL}};
const Segment SGI320174[] = {{&S42650[30], 3}, {NULL}};
const Segment SGI320175[] = {{&S42650[8], 3}, {NULL}};
const Segment SGI320176[] = {{&S42650[13], 10}, {NULL}};
const Segment SGI320177[] = {{&S42650[55], 7}, {NULL}};
const Segment SGI320178[] = {{&S42650[48], 14}, {NULL}};
const Segment SGI320179[] = {{&S42650[64], 7}, {NULL}};
const Segment SGI320180[] = {{&S42651[30], 3}, {NULL}};
const Segment SGI320181[] = {{&S42651[8], 3}, {NULL}};
const Segment SGI320182[] = {{&S42651[13], 10}, {NULL}};
const Segment SGI320183[] = {{&S42651[55], 7}, {NULL}};
const Segment SGI320184[] = {{&S42651[48], 14}, {NULL}};
const Segment SGI320185[] = {{&S42651[64], 7}, {NULL}};
const Segment SGI320186[] = {{&S42652[30], 3}, {NULL}};
const Segment SGI320187[] = {{&S42652[8], 3}, {NULL}};
const Segment SGI320188[] = {{&S42652[13], 10}, {NULL}};
const Segment SGI320189[] = {{&S42652[55], 7}, {NULL}};
const Segment SGI320190[] = {{&S42652[48], 14}, {NULL}};
const Segment SGI320191[] = {{&S42652[64], 7}, {NULL}};
const Segment SGI320192[] = {{&S42653[30], 4}, {NULL}};
const Segment SGI320193[] = {{&S42653[8], 3}, {NULL}};
const Segment SGI320194[] = {{&S42653[13], 10}, {NULL}};
const Segment SGI320195[] = {{&S42653[43], 7}, {NULL}};
const Segment SGI320196[] = {{&S42653[36], 14}, {NULL}};
const Segment SGI320197[] = {{&S42653[52], 7}, {NULL}};
const Segment SGI320198[] = {{&S42654[30], 3}, {NULL}};
const Segment SGI320199[] = {{&S42654[8], 3}, {NULL}};
const Segment SGI320200[] = {{&S42654[13], 10}, {NULL}};
const Segment SGI320201[] = {{&S42654[55], 7}, {NULL}};
const Segment SGI320202[] = {{&S42654[48], 14}, {NULL}};
const Segment SGI320203[] = {{&S42654[79], 7}, {NULL}};
const Segment SGI320204[] = {{&S42654[64], 13}, {NULL}};
const Segment SGI320205[] = {{&S42655[30], 3}, {NULL}};
const Segment SGI320206[] = {{&S42655[8], 3}, {NULL}};
const Segment SGI320207[] = {{&S42655[13], 10}, {NULL}};
const Segment SGI320208[] = {{&S42655[55], 7}, {NULL}};
const Segment SGI320209[] = {{&S42655[48], 14}, {NULL}};
const Segment SGI320210[] = {{&S42655[64], 7}, {&S42655[64], 7}, {NULL}};
const Segment SGI320211[] = {{&S42656[30], 3}, {NULL}};
const Segment SGI320212[] = {{&S42656[8], 3}, {NULL}};
const Segment SGI320213[] = {{&S42656[13], 10}, {NULL}};
const Segment SGI320214[] = {{&S42656[55], 7}, {NULL}};
const Segment SGI320215[] = {{&S42656[48], 14}, {NULL}};
const Segment SGI320216[] = {{&S42656[64], 7}, {NULL}};
const Segment SGI320217[] = {{&S42656[73], 3}, {NULL}};
const Segment SGI320218[] = {{&S42657[30], 3}, {NULL}};
const Segment SGI320219[] = {{&S42657[8], 3}, {NULL}};
const Segment SGI320220[] = {{&S42657[13], 10}, {NULL}};
const Segment SGI320221[] = {{&S42657[55], 7}, {NULL}};
const Segment SGI320222[] = {{&S42657[48], 14}, {NULL}};
const Segment SGI320223[] = {{&S42657[79], 7}, {NULL}};
const Segment SGI320224[] = {{&S42657[64], 13}, {NULL}};
const Segment SGI320225[] = {{&S42658[30], 3}, {NULL}};
const Segment SGI320226[] = {{&S42658[8], 3}, {NULL}};
const Segment SGI320227[] = {{&S42658[13], 10}, {NULL}};
const Segment SGI320228[] = {{&S42658[42], 7}, {NULL}};
const Segment SGI320229[] = {{&S42658[35], 14}, {NULL}};
const Segment SGI320230[] = {{&S42658[51], 6}, {NULL}};
const Segment SGI320231[] = {{&S42658[98], 7}, {NULL}};
const Segment SGI320232[] = {{&S42659[30], 3}, {NULL}};
const Segment SGI320233[] = {{&S42659[8], 3}, {NULL}};
const Segment SGI320234[] = {{&S42659[13], 10}, {NULL}};
const Segment SGI320235[] = {{&S42659[55], 7}, {NULL}};
const Segment SGI320236[] = {{&S42659[48], 14}, {NULL}};
const Segment SGI320237[] = {{&S42659[79], 7}, {NULL}};
const Segment SGI320238[] = {{&S42659[64], 13}, {NULL}};
const Segment SGI320239[] = {{&S42660[30], 3}, {NULL}};
const Segment SGI320240[] = {{&S42660[8], 3}, {NULL}};
const Segment SGI320241[] = {{&S42660[13], 10}, {NULL}};
const Segment SGI320242[] = {{&S42660[55], 7}, {NULL}};
const Segment SGI320243[] = {{&S42660[48], 14}, {NULL}};
const Segment SGI320244[] = {{&S42660[69], 7}, {NULL}};
const Segment SGI320245[] = {{&S42660[64], 3}, {NULL}};
const Segment SGI320246[] = {{&S42661[30], 3}, {NULL}};
const Segment SGI320247[] = {{&S42661[8], 3}, {NULL}};
const Segment SGI320248[] = {{&S42661[13], 10}, {NULL}};
const Segment SGI320249[] = {{&S42661[55], 7}, {NULL}};
const Segment SGI320250[] = {{&S42661[48], 14}, {NULL}};
const Segment SGI320251[] = {{&S42661[69], 7}, {NULL}};
const Segment SGI320252[] = {{&S42661[64], 3}, {NULL}};
const Segment SGI320253[] = {{&S42662[30], 3}, {NULL}};
const Segment SGI320254[] = {{&S42662[8], 3}, {NULL}};
const Segment SGI320255[] = {{&S42662[13], 10}, {NULL}};
const Segment SGI320256[] = {{&S42662[55], 7}, {NULL}};
const Segment SGI320257[] = {{&S42662[48], 14}, {NULL}};
const Segment SGI320258[] = {{&S42662[64], 7}, {NULL}};
const Segment SGI320259[] = {{&S42662[73], 13}, {NULL}};
const Segment SGI320260[] = {{&S42663[30], 3}, {NULL}};
const Segment SGI320261[] = {{&S42663[8], 3}, {NULL}};
const Segment SGI320262[] = {{&S42663[13], 10}, {NULL}};
const Segment SGI320263[] = {{&S42663[55], 7}, {NULL}};
const Segment SGI320264[] = {{&S42663[48], 14}, {NULL}};
const Segment SGI320265[] = {{&S42663[64], 7}, {&S42663[64], 7}, {NULL}};
const Segment SGI320266[] = {{&S42664[30], 3}, {NULL}};
const Segment SGI320267[] = {{&S42664[8], 3}, {NULL}};
const Segment SGI320268[] = {{&S42664[13], 10}, {NULL}};
const Segment SGI320269[] = {{&S42664[55], 7}, {NULL}};
const Segment SGI320270[] = {{&S42664[48], 14}, {NULL}};
const Segment SGI320271[] = {{&S42664[64], 7}, {NULL}};
const Segment SGI320272[] = {{&S42664[73], 3}, {NULL}};
const Segment SGI320273[] = {{&S42665[30], 3}, {NULL}};
const Segment SGI320274[] = {{&S42665[8], 3}, {NULL}};
const Segment SGI320275[] = {{&S42665[13], 10}, {NULL}};
const Segment SGI320276[] = {{&S42665[55], 7}, {NULL}};
const Segment SGI320277[] = {{&S42665[48], 14}, {NULL}};
const Segment SGI320278[] = {{&S42665[64], 7}, {&S42665[64], 7}, {NULL}};
const Segment SGI320279[] = {{&S42666[30], 3}, {NULL}};
const Segment SGI320280[] = {{&S42666[8], 3}, {NULL}};
const Segment SGI320281[] = {{&S42666[13], 10}, {NULL}};
const Segment SGI320282[] = {{&S42666[42], 7}, {NULL}};
const Segment SGI320283[] = {{&S42666[35], 14}, {NULL}};
const Segment SGI320284[] = {{&S42666[51], 6}, {NULL}};
const Segment SGI320285[] = {{&S42666[98], 7}, {NULL}};
const Segment SGI320286[] = {{&S42667[30], 3}, {NULL}};
const Segment SGI320287[] = {{&S42667[8], 3}, {NULL}};
const Segment SGI320288[] = {{&S42667[13], 10}, {NULL}};
const Segment SGI320289[] = {{&S42667[55], 7}, {NULL}};
const Segment SGI320290[] = {{&S42667[48], 14}, {NULL}};
const Segment SGI320291[] = {{&S42667[64], 7}, {&S42667[64], 7}, {NULL}};
const Segment SGI320292[] = {{&S42668[30], 3}, {NULL}};
const Segment SGI320293[] = {{&S42668[8], 3}, {NULL}};
const Segment SGI320294[] = {{&S42668[13], 10}, {NULL}};
const Segment SGI320295[] = {{&S42668[42], 7}, {NULL}};
const Segment SGI320296[] = {{&S42668[35], 14}, {NULL}};
const Segment SGI320297[] = {{&S42668[74], 3}, {NULL}};
const Segment SGI320298[] = {{&S42669[30], 3}, {NULL}};
const Segment SGI320299[] = {{&S42669[8], 3}, {NULL}};
const Segment SGI320300[] = {{&S42669[13], 10}, {NULL}};
const Segment SGI320301[] = {{&S42669[42], 7}, {NULL}};
const Segment SGI320302[] = {{&S42669[35], 14}, {NULL}};
const Segment SGI320303[] = {{&S42669[64], 3}, {NULL}};
const Segment SGI320304[] = {{&S42669[79], 3}, {NULL}};
const Segment SGI320305[] = {{&S42670[30], 3}, {NULL}};
const Segment SGI320306[] = {{&S42670[8], 3}, {NULL}};
const Segment SGI320307[] = {{&S42670[13], 10}, {NULL}};
const Segment SGI320308[] = {{&S42670[42], 7}, {NULL}};
const Segment SGI320309[] = {{&S42670[35], 14}, {NULL}};
const Segment SGI320310[] = {{&S42670[51], 3}, {NULL}};
const Segment SGI320311[] = {{&S42670[66], 3}, {NULL}};
const Segment SGI320312[] = {{&S42671[30], 3}, {NULL}};
const Segment SGI320313[] = {{&S42671[8], 3}, {NULL}};
const Segment SGI320314[] = {{&S42671[13], 10}, {NULL}};
const Segment SGI320315[] = {{&S42671[42], 7}, {NULL}};
const Segment SGI320316[] = {{&S42671[35], 14}, {NULL}};
const Segment SGI320317[] = {{&S42671[74], 3}, {NULL}};
const Segment SGI320318[] = {{&S42711[30], 3}, {NULL}};
const Segment SGI320319[] = {{&S42711[8], 3}, {NULL}};
const Segment SGI320320[] = {{&S42711[13], 10}, {NULL}};
const Segment SGI320321[] = {{&S42711[42], 7}, {NULL}};
const Segment SGI320322[] = {{&S42711[35], 14}, {NULL}};
const Segment SGI320323[] = {{&S42711[204], 11}, {NULL}};
const Segment SGI320324[] = {{&S42711[138], 15}, {NULL}};
const Segment SGI320325[] = {{&S42711[51], 6}, {NULL}};
const Segment SGI320326[] = {{&S42711[204], 15}, {NULL}};
const Segment SGI320327[] = {{&S42713[30], 3}, {NULL}};
const Segment SGI320328[] = {{&S42713[8], 3}, {NULL}};
const Segment SGI320329[] = {{&S42713[13], 10}, {NULL}};
const Segment SGI320330[] = {{&S42713[42], 7}, {NULL}};
const Segment SGI320331[] = {{&S42713[35], 14}, {NULL}};
const Segment SGI320332[] = {{&S42713[206], 11}, {NULL}};
const Segment SGI320333[] = {{&S42713[131], 15}, {NULL}};
const Segment SGI320334[] = {{&S42713[206], 15}, {NULL}};
const Segment SGI320335[] = {{&S42713[191], 9}, {NULL}};
const Segment SGI320336[] = {{&S42714[30], 3}, {NULL}};
const Segment SGI320337[] = {{&S42714[8], 3}, {NULL}};
const Segment SGI320338[] = {{&S42714[13], 10}, {NULL}};
const Segment SGI320339[] = {{&S42714[42], 7}, {NULL}};
const Segment SGI320340[] = {{&S42714[35], 14}, {NULL}};
const Segment SGI320341[] = {{&S42714[71], 11}, {NULL}};
const Segment SGI320342[] = {{&S42714[51], 6}, {NULL}};
const Segment SGI320343[] = {{&S42714[71], 15}, {NULL}};
const Segment SGI320344[] = {{&S42715[30], 3}, {NULL}};
const Segment SGI320345[] = {{&S42715[8], 3}, {NULL}};
const Segment SGI320346[] = {{&S42715[13], 10}, {NULL}};
const Segment SGI320347[] = {{&S42715[42], 7}, {NULL}};
const Segment SGI320348[] = {{&S42715[35], 14}, {NULL}};
const Segment SGI320349[] = {{&S42715[64], 11}, {NULL}};
const Segment SGI320350[] = {{&S42715[64], 15}, {NULL}};
const Segment SGI320351[] = {{&S42716[30], 3}, {NULL}};
const Segment SGI320352[] = {{&S42716[8], 3}, {NULL}};
const Segment SGI320353[] = {{&S42716[13], 10}, {NULL}};
const Segment SGI320354[] = {{&S42716[42], 7}, {NULL}};
const Segment SGI320355[] = {{&S42716[35], 14}, {NULL}};
const Segment SGI320356[] = {{&S42716[162], 11}, {NULL}};
const Segment SGI320357[] = {{&S42716[131], 15}, {NULL}};
const Segment SGI320358[] = {{&S42716[162], 15}, {NULL}};
const Segment SGI320359[] = {{&S42717[30], 3}, {NULL}};
const Segment SGI320360[] = {{&S42717[8], 3}, {NULL}};
const Segment SGI320361[] = {{&S42717[13], 10}, {NULL}};
const Segment SGI320362[] = {{&S42717[42], 7}, {NULL}};
const Segment SGI320363[] = {{&S42717[35], 14}, {NULL}};
const Segment SGI320364[] = {{&S42717[191], 11}, {NULL}};
const Segment SGI320365[] = {{&S42717[138], 15}, {NULL}};
const Segment SGI320366[] = {{&S42717[51], 6}, {NULL}};
const Segment SGI320367[] = {{&S42717[191], 15}, {NULL}};
const Segment SGI320368[] = {{&S42718[31], 3}, {NULL}};
const Segment SGI320369[] = {{&S42718[9], 3}, {NULL}};
const Segment SGI320370[] = {{&S42718[14], 10}, {NULL}};
const Segment SGI320371[] = {{&S42718[43], 7}, {NULL}};
const Segment SGI320372[] = {{&S42718[36], 14}, {NULL}};
const Segment SGI320373[] = {{&S42718[204], 11}, {NULL}};
const Segment SGI320374[] = {{&S42718[52], 6}, {NULL}};
const Segment SGI320375[] = {{&S42718[204], 15}, {NULL}};
const Segment SGI320376[] = {{&S42945[30], 3}, {NULL}};
const Segment SGI320377[] = {{&S42945[8], 3}, {NULL}};
const Segment SGI320378[] = {{&S42945[13], 10}, {NULL}};
const Segment SGI320379[] = {{&S42945[55], 3}, {NULL}};
const Segment SGI320380[] = {{&S42945[60], 13}, {NULL}};
const Segment SGI320381[] = {{&S42946[30], 3}, {NULL}};
const Segment SGI320382[] = {{&S42946[8], 3}, {NULL}};
const Segment SGI320383[] = {{&S42946[13], 10}, {NULL}};
const Segment SGI320384[] = {{&S42946[55], 3}, {NULL}};
const Segment SGI320385[] = {{&S42946[60], 13}, {NULL}};
const Segment SGI320386[] = {{&S42947[30], 3}, {NULL}};
const Segment SGI320387[] = {{&S42947[8], 3}, {NULL}};
const Segment SGI320388[] = {{&S42947[13], 10}, {NULL}};
const Segment SGI320389[] = {{&S42947[55], 3}, {NULL}};
const Segment SGI320390[] = {{&S42947[60], 13}, {NULL}};
const Segment SGI320391[] = {{&S42947[75], 3}, {NULL}};
const Segment SGI320392[] = {{&S42948[30], 3}, {NULL}};
const Segment SGI320393[] = {{&S42948[8], 3}, {NULL}};
const Segment SGI320394[] = {{&S42948[13], 10}, {NULL}};
const Segment SGI320395[] = {{&S42948[55], 3}, {NULL}};
const Segment SGI320396[] = {{&S42948[60], 13}, {&S42948[60], 13}, {NULL}};
const Segment SGI320397[] = {{&S42949[30], 3}, {NULL}};
const Segment SGI320398[] = {{&S42949[8], 3}, {NULL}};
const Segment SGI320399[] = {{&S42949[13], 10}, {NULL}};
const Segment SGI320400[] = {{&S42949[55], 3}, {NULL}};
const Segment SGI320401[] = {{&S42949[60], 13}, {&S42949[60], 13}, {NULL}};
const Segment SGI320402[] = {{&S42950[30], 3}, {NULL}};
const Segment SGI320403[] = {{&S42950[8], 3}, {NULL}};
const Segment SGI320404[] = {{&S42950[13], 10}, {NULL}};
const Segment SGI320405[] = {{&S42950[55], 3}, {NULL}};
const Segment SGI320406[] = {{&S42950[65], 13}, {NULL}};
const Segment SGI320407[] = {{&S42950[60], 3}, {NULL}};
const Segment SGI320408[] = {{&S59900[3], 5}, {NULL}};
const Segment SGI320409[] = {{&S43133[30], 3}, {NULL}};
const Segment SGI320410[] = {{&S43133[8], 3}, {NULL}};
const Segment SGI320411[] = {{&S43133[13], 10}, {NULL}};
const Segment SGI320412[] = {{&S43133[42], 7}, {NULL}};
const Segment SGI320413[] = {{&S43133[35], 14}, {NULL}};
const Segment SGI320414[] = {{&S43133[64], 11}, {NULL}};
const Segment SGI320415[] = {{&S43133[64], 16}, {NULL}};
const Segment SGI320416[] = {{&S43134[30], 3}, {NULL}};
const Segment SGI320417[] = {{&S43134[8], 3}, {NULL}};
const Segment SGI320418[] = {{&S43134[13], 10}, {NULL}};
const Segment SGI320419[] = {{&S43134[42], 7}, {NULL}};
const Segment SGI320420[] = {{&S43134[35], 14}, {NULL}};
const Segment SGI320421[] = {{&S43134[71], 11}, {NULL}};
const Segment SGI320422[] = {{&S43134[51], 6}, {NULL}};
const Segment SGI320423[] = {{&S43134[71], 16}, {NULL}};
const Segment SGI320424[] = {{&S43135[30], 3}, {NULL}};
const Segment SGI320425[] = {{&S43135[8], 3}, {NULL}};
const Segment SGI320426[] = {{&S43135[13], 10}, {NULL}};
const Segment SGI320427[] = {{&S43135[42], 7}, {NULL}};
const Segment SGI320428[] = {{&S43135[35], 14}, {NULL}};
const Segment SGI320429[] = {{&S43135[71], 11}, {NULL}};
const Segment SGI320430[] = {{&S43135[164], 15}, {NULL}};
const Segment SGI320431[] = {{&S43135[51], 6}, {NULL}};
const Segment SGI320432[] = {{&S43135[71], 16}, {NULL}};
const Segment SGI320433[] = {{&S43137[30], 3}, {NULL}};
const Segment SGI320434[] = {{&S43137[8], 3}, {NULL}};
const Segment SGI320435[] = {{&S43137[13], 10}, {NULL}};
const Segment SGI320436[] = {{&S43137[42], 7}, {NULL}};
const Segment SGI320437[] = {{&S43137[35], 14}, {NULL}};
const Segment SGI320438[] = {{&S43137[64], 11}, {NULL}};
const Segment SGI320439[] = {{&S43137[157], 15}, {NULL}};
const Segment SGI320440[] = {{&S43137[64], 16}, {NULL}};
const Segment SGI320441[] = {{&S43138[30], 3}, {NULL}};
const Segment SGI320442[] = {{&S43138[8], 3}, {NULL}};
const Segment SGI320443[] = {{&S43138[13], 10}, {NULL}};
const Segment SGI320444[] = {{&S43138[42], 7}, {NULL}};
const Segment SGI320445[] = {{&S43138[35], 14}, {NULL}};
const Segment SGI320446[] = {{&S43138[64], 11}, {NULL}};
const Segment SGI320447[] = {{&S43138[64], 16}, {NULL}};
const Segment SGI320448[] = {{&S43139[30], 3}, {NULL}};
const Segment SGI320449[] = {{&S43139[8], 3}, {NULL}};
const Segment SGI320450[] = {{&S43139[13], 10}, {NULL}};
const Segment SGI320451[] = {{&S43139[42], 7}, {NULL}};
const Segment SGI320452[] = {{&S43139[35], 14}, {NULL}};
const Segment SGI320453[] = {{&S43139[71], 11}, {NULL}};
const Segment SGI320454[] = {{&S43139[51], 6}, {NULL}};
const Segment SGI320455[] = {{&S43139[71], 16}, {NULL}};
const Segment SGI320456[] = {{&S43268[30], 3}, {NULL}};
const Segment SGI320457[] = {{&S43268[8], 3}, {NULL}};
const Segment SGI320458[] = {{&S43268[13], 10}, {NULL}};
const Segment SGI320459[] = {{&S43268[42], 7}, {NULL}};
const Segment SGI320460[] = {{&S43268[35], 14}, {NULL}};
const Segment SGI320461[] = {{&S43269[30], 3}, {NULL}};
const Segment SGI320462[] = {{&S43269[8], 3}, {NULL}};
const Segment SGI320463[] = {{&S43269[13], 10}, {NULL}};
const Segment SGI320464[] = {{&S43269[42], 7}, {NULL}};
const Segment SGI320465[] = {{&S43270[30], 3}, {&S43270[209], 3}, {NULL}};
const Segment SGI320466[] = {{&S43270[8], 3}, {&S43270[8], 3}, {NULL}};
const Segment SGI320467[] = {{&S43270[13], 10}, {&S43270[13], 10}, {NULL}};
const Segment SGI320468[] = {{&S43270[42], 7}, {&S43270[42], 3}, {NULL}};
const Segment SGI320469[] = {{&S43270[35], 14}, {NULL}};
const Segment SGI320470[] = {{&S43271[30], 3}, {NULL}};
const Segment SGI320471[] = {{&S43271[8], 3}, {NULL}};
const Segment SGI320472[] = {{&S43271[13], 10}, {NULL}};
const Segment SGI320473[] = {{&S43271[42], 3}, {NULL}};
const Segment SGI320474[] = {{&S43272[30], 3}, {NULL}};
const Segment SGI320475[] = {{&S43272[8], 3}, {NULL}};
const Segment SGI320476[] = {{&S43272[13], 10}, {NULL}};
const Segment SGI320477[] = {{&S43272[42], 7}, {NULL}};
const Segment SGI320478[] = {{&S43272[35], 14}, {NULL}};
const Segment SGI320479[] = {{&S43273[29], 3}, {NULL}};
const Segment SGI320480[] = {{&S43273[8], 3}, {NULL}};
const Segment SGI320481[] = {{&S43273[13], 10}, {NULL}};
const Segment SGI320482[] = {{&S43273[42], 7}, {NULL}};
const Segment SGI320483[] = {{&S43273[35], 14}, {NULL}};
const Segment SGI320484[] = {{&S43274[30], 4}, {NULL}};
const Segment SGI320485[] = {{&S43274[8], 3}, {NULL}};
const Segment SGI320486[] = {{&S43274[13], 10}, {NULL}};
const Segment SGI320487[] = {{&S43274[43], 7}, {NULL}};
const Segment SGI320488[] = {{&S43275[29], 3}, {NULL}};
const Segment SGI320489[] = {{&S43275[8], 3}, {NULL}};
const Segment SGI320490[] = {{&S43275[13], 10}, {NULL}};
const Segment SGI320491[] = {{&S43275[42], 7}, {NULL}};
const Segment SGI320492[] = {{&S43275[35], 14}, {NULL}};
const Segment SGI320493[] = {{&S43276[32], 3}, {NULL}};
const Segment SGI320494[] = {{&S43276[10], 3}, {NULL}};
const Segment SGI320495[] = {{&S43276[15], 10}, {NULL}};
const Segment SGI320496[] = {{&S43276[44], 7}, {&S43276[103], 3}, {NULL}};
const Segment SGI320497[] = {{&S43276[37], 14}, {NULL}};
const Segment SGI320498[] = {{&S43277[8], 3}, {NULL}};
const Segment SGI320499[] = {{&S43277[13], 10}, {NULL}};
const Segment SGI320500[] = {{&S43277[42], 7}, {NULL}};
const Segment SGI320501[] = {{&S43277[35], 14}, {NULL}};
const Segment SGI320502[] = {{&S43278[8], 3}, {NULL}};
const Segment SGI320503[] = {{&S43278[13], 10}, {NULL}};
const Segment SGI320504[] = {{&S43278[42], 7}, {NULL}};
const Segment SGI320505[] = {{&S43278[35], 14}, {NULL}};
const Segment SGI320506[] = {{&S43279[30], 3}, {NULL}};
const Segment SGI320507[] = {{&S43279[8], 3}, {NULL}};
const Segment SGI320508[] = {{&S43279[13], 10}, {NULL}};
const Segment SGI320509[] = {{&S43279[42], 7}, {NULL}};
const Segment SGI320510[] = {{&S43279[35], 14}, {NULL}};
const Segment SGI320511[] = {{&S43387[30], 3}, {NULL}};
const Segment SGI320512[] = {{&S43387[8], 3}, {NULL}};
const Segment SGI320513[] = {{&S43387[13], 10}, {NULL}};
const Segment SGI320514[] = {{&S43387[55], 3}, {NULL}};
const Segment SGI320515[] = {{&S43387[60], 13}, {NULL}};
const Segment SGI320516[] = {{&S43388[30], 3}, {NULL}};
const Segment SGI320517[] = {{&S43388[8], 3}, {NULL}};
const Segment SGI320518[] = {{&S43388[13], 10}, {NULL}};
const Segment SGI320519[] = {{&S43388[55], 3}, {NULL}};
const Segment SGI320520[] = {{&S43388[60], 13}, {NULL}};
const Segment SGI320521[] = {{&S43389[30], 3}, {NULL}};
const Segment SGI320522[] = {{&S43389[8], 3}, {NULL}};
const Segment SGI320523[] = {{&S43389[13], 10}, {NULL}};
const Segment SGI320524[] = {{&S43389[55], 3}, {NULL}};
const Segment SGI320525[] = {{&S43389[60], 13}, {NULL}};
const Segment SGI320526[] = {{&S43389[75], 3}, {NULL}};
const Segment SGI320527[] = {{&S43390[30], 3}, {NULL}};
const Segment SGI320528[] = {{&S43390[8], 3}, {NULL}};
const Segment SGI320529[] = {{&S43390[13], 10}, {NULL}};
const Segment SGI320530[] = {{&S43390[55], 3}, {NULL}};
const Segment SGI320531[] = {{&S43390[60], 13}, {&S43390[60], 13}, {NULL}};
const Segment SGI320532[] = {{&S43391[30], 3}, {NULL}};
const Segment SGI320533[] = {{&S43391[8], 3}, {NULL}};
const Segment SGI320534[] = {{&S43391[13], 10}, {NULL}};
const Segment SGI320535[] = {{&S43391[55], 3}, {NULL}};
const Segment SGI320536[] = {{&S43391[60], 13}, {&S43391[60], 13}, {NULL}};
const Segment SGI320537[] = {{&S43392[30], 3}, {NULL}};
const Segment SGI320538[] = {{&S43392[8], 3}, {NULL}};
const Segment SGI320539[] = {{&S43392[13], 10}, {NULL}};
const Segment SGI320540[] = {{&S43392[55], 3}, {NULL}};
const Segment SGI320541[] = {{&S43392[65], 13}, {NULL}};
const Segment SGI320542[] = {{&S43392[60], 3}, {NULL}};
const Segment SGI320543[] = {{&S43816[30], 3}, {NULL}};
const Segment SGI320544[] = {{&S43816[8], 3}, {NULL}};
const Segment SGI320545[] = {{&S43816[13], 10}, {NULL}};
const Segment SGI320546[] = {{&S43816[42], 7}, {NULL}};
const Segment SGI320547[] = {{&S43816[35], 14}, {NULL}};
const Segment SGI320548[] = {{&S43816[64], 11}, {NULL}};
const Segment SGI320549[] = {{&S43816[158], 15}, {NULL}};
const Segment SGI320550[] = {{&S43816[64], 16}, {NULL}};
const Segment SGI320551[] = {{&S43817[30], 3}, {NULL}};
const Segment SGI320552[] = {{&S43817[8], 3}, {NULL}};
const Segment SGI320553[] = {{&S43817[13], 10}, {NULL}};
const Segment SGI320554[] = {{&S43817[42], 7}, {NULL}};
const Segment SGI320555[] = {{&S43817[35], 14}, {NULL}};
const Segment SGI320556[] = {{&S43817[64], 11}, {NULL}};
const Segment SGI320557[] = {{&S43817[158], 15}, {NULL}};
const Segment SGI320558[] = {{&S43817[64], 16}, {NULL}};
const Segment SGI320559[] = {{&S43822[30], 3}, {NULL}};
const Segment SGI320560[] = {{&S43822[8], 3}, {NULL}};
const Segment SGI320561[] = {{&S43822[13], 10}, {NULL}};
const Segment SGI320562[] = {{&S43822[42], 7}, {NULL}};
const Segment SGI320563[] = {{&S43822[35], 14}, {NULL}};
const Segment SGI320564[] = {{&S43822[71], 11}, {NULL}};
const Segment SGI320565[] = {{&S43822[51], 6}, {NULL}};
const Segment SGI320566[] = {{&S43822[71], 16}, {NULL}};
const Segment SGI320567[] = {{&S43824[30], 3}, {NULL}};
const Segment SGI320568[] = {{&S43824[8], 3}, {NULL}};
const Segment SGI320569[] = {{&S43824[13], 10}, {NULL}};
const Segment SGI320570[] = {{&S43824[42], 7}, {NULL}};
const Segment SGI320571[] = {{&S43824[35], 14}, {NULL}};
const Segment SGI320572[] = {{&S43824[64], 11}, {NULL}};
const Segment SGI320573[] = {{&S43824[64], 16}, {NULL}};
const Segment SGI320574[] = {{&S43826[30], 3}, {NULL}};
const Segment SGI320575[] = {{&S43826[8], 3}, {NULL}};
const Segment SGI320576[] = {{&S43826[13], 10}, {NULL}};
const Segment SGI320577[] = {{&S43826[42], 7}, {NULL}};
const Segment SGI320578[] = {{&S43826[35], 14}, {NULL}};
const Segment SGI320579[] = {{&S43826[71], 11}, {NULL}};
const Segment SGI320580[] = {{&S43826[165], 15}, {NULL}};
const Segment SGI320581[] = {{&S43826[51], 6}, {NULL}};
const Segment SGI320582[] = {{&S43826[71], 16}, {NULL}};
const Segment SGI320583[] = {{&S43827[30], 3}, {NULL}};
const Segment SGI320584[] = {{&S43827[8], 3}, {NULL}};
const Segment SGI320585[] = {{&S43827[13], 10}, {NULL}};
const Segment SGI320586[] = {{&S43827[42], 7}, {NULL}};
const Segment SGI320587[] = {{&S43827[35], 14}, {NULL}};
const Segment SGI320588[] = {{&S43827[64], 11}, {NULL}};
const Segment SGI320589[] = {{&S43827[64], 16}, {NULL}};
const Segment SGI320590[] = {{&S43830[30], 3}, {NULL}};
const Segment SGI320591[] = {{&S43830[8], 3}, {NULL}};
const Segment SGI320592[] = {{&S43830[13], 10}, {NULL}};
const Segment SGI320593[] = {{&S43830[42], 7}, {NULL}};
const Segment SGI320594[] = {{&S43830[35], 14}, {NULL}};
const Segment SGI320595[] = {{&S43830[64], 11}, {NULL}};
const Segment SGI320596[] = {{&S43830[158], 15}, {NULL}};
const Segment SGI320597[] = {{&S43830[64], 16}, {NULL}};
const Segment SGI320598[] = {{&S43830[179], 9}, {NULL}};
const Segment SGI320599[] = {{&S43831[30], 3}, {NULL}};
const Segment SGI320600[] = {{&S43831[8], 3}, {NULL}};
const Segment SGI320601[] = {{&S43831[13], 10}, {NULL}};
const Segment SGI320602[] = {{&S43831[42], 7}, {NULL}};
const Segment SGI320603[] = {{&S43831[35], 14}, {NULL}};
const Segment SGI320604[] = {{&S43831[71], 11}, {NULL}};
const Segment SGI320605[] = {{&S43831[51], 6}, {NULL}};
const Segment SGI320606[] = {{&S43831[71], 16}, {NULL}};
const Segment SGI320607[] = {{&S43832[30], 3}, {NULL}};
const Segment SGI320608[] = {{&S43832[8], 3}, {NULL}};
const Segment SGI320609[] = {{&S43832[13], 10}, {NULL}};
const Segment SGI320610[] = {{&S43832[42], 7}, {NULL}};
const Segment SGI320611[] = {{&S43832[35], 14}, {NULL}};
const Segment SGI320612[] = {{&S43832[72], 11}, {NULL}};
const Segment SGI320613[] = {{&S43832[72], 16}, {NULL}};
const Segment SGI320614[] = {{&S43833[30], 3}, {NULL}};
const Segment SGI320615[] = {{&S43833[8], 3}, {NULL}};
const Segment SGI320616[] = {{&S43833[13], 10}, {NULL}};
const Segment SGI320617[] = {{&S43833[42], 7}, {NULL}};
const Segment SGI320618[] = {{&S43833[35], 14}, {NULL}};
const Segment SGI320619[] = {{&S43833[71], 11}, {NULL}};
const Segment SGI320620[] = {{&S43833[165], 15}, {NULL}};
const Segment SGI320621[] = {{&S43833[51], 6}, {NULL}};
const Segment SGI320622[] = {{&S43833[71], 16}, {NULL}};
const Segment SGI320623[] = {{&S43834[30], 4}, {NULL}};
const Segment SGI320624[] = {{&S43834[8], 3}, {NULL}};
const Segment SGI320625[] = {{&S43834[13], 10}, {NULL}};
const Segment SGI320626[] = {{&S43834[43], 7}, {NULL}};
const Segment SGI320627[] = {{&S43834[36], 14}, {NULL}};
const Segment SGI320628[] = {{&S43834[65], 11}, {NULL}};
const Segment SGI320629[] = {{&S43834[65], 16}, {NULL}};
const Segment SGI320630[] = {{&S43835[30], 4}, {NULL}};
const Segment SGI320631[] = {{&S43835[8], 3}, {NULL}};
const Segment SGI320632[] = {{&S43835[13], 10}, {NULL}};
const Segment SGI320633[] = {{&S43835[43], 7}, {NULL}};
const Segment SGI320634[] = {{&S43835[36], 14}, {NULL}};
const Segment SGI320635[] = {{&S43835[72], 11}, {NULL}};
const Segment SGI320636[] = {{&S43835[52], 6}, {NULL}};
const Segment SGI320637[] = {{&S43835[72], 16}, {NULL}};
const Segment SGI320638[] = {{&S43936[30], 4}, {NULL}};
const Segment SGI320639[] = {{&S43936[8], 3}, {NULL}};
const Segment SGI320640[] = {{&S43936[13], 10}, {NULL}};
const Segment SGI320641[] = {{&S43936[52], 3}, {NULL}};
const Segment SGI320642[] = {{&S43937[30], 3}, {NULL}};
const Segment SGI320643[] = {{&S43937[8], 3}, {NULL}};
const Segment SGI320644[] = {{&S43937[13], 10}, {NULL}};
const Segment SGI320645[] = {{&S43937[42], 3}, {NULL}};
const Segment SGI320646[] = {{&S43938[30], 3}, {NULL}};
const Segment SGI320647[] = {{&S43938[8], 3}, {NULL}};
const Segment SGI320648[] = {{&S43938[13], 10}, {NULL}};
const Segment SGI320649[] = {{&S43938[42], 3}, {NULL}};
const Segment SGI320650[] = {{&S43938[47], 3}, {&S43938[47], 3}, {NULL}};
const Segment SGI320651[] = {{&S43939[30], 3}, {NULL}};
const Segment SGI320652[] = {{&S43939[8], 3}, {NULL}};
const Segment SGI320653[] = {{&S43939[13], 10}, {NULL}};
const Segment SGI320654[] = {{&S43939[55], 3}, {NULL}};
const Segment SGI320655[] = {{&S43939[60], 3}, {NULL}};
const Segment SGI320656[] = {{&S43940[30], 3}, {NULL}};
const Segment SGI320657[] = {{&S43940[8], 3}, {NULL}};
const Segment SGI320658[] = {{&S43940[13], 10}, {NULL}};
const Segment SGI320659[] = {{&S43940[55], 3}, {NULL}};
const Segment SGI320660[] = {{&S43941[30], 3}, {NULL}};
const Segment SGI320661[] = {{&S43941[8], 3}, {NULL}};
const Segment SGI320662[] = {{&S43941[13], 10}, {NULL}};
const Segment SGI320663[] = {{&S43941[55], 3}, {NULL}};
const Segment SGI320664[] = {{&S43941[60], 3}, {NULL}};
const Segment SGI320665[] = {{&S43942[30], 3}, {NULL}};
const Segment SGI320666[] = {{&S43942[8], 3}, {NULL}};
const Segment SGI320667[] = {{&S43942[13], 10}, {NULL}};
const Segment SGI320668[] = {{&S43942[55], 3}, {NULL}};
const Segment SGI320669[] = {{&S43942[60], 3}, {&S43942[60], 3}, {NULL}};
const Segment SGI320670[] = {{&S43943[30], 3}, {NULL}};
const Segment SGI320671[] = {{&S43943[8], 3}, {NULL}};
const Segment SGI320672[] = {{&S43943[13], 10}, {NULL}};
const Segment SGI320673[] = {{&S43943[35], 5}, {NULL}};
const Segment SGI320674[] = {{&S43944[30], 3}, {NULL}};
const Segment SGI320675[] = {{&S43944[8], 3}, {NULL}};
const Segment SGI320676[] = {{&S43944[13], 10}, {NULL}};
const Segment SGI320677[] = {{&S44031[30], 3}, {NULL}};
const Segment SGI320678[] = {{&S44031[8], 3}, {NULL}};
const Segment SGI320679[] = {{&S44031[13], 10}, {NULL}};
const Segment SGI320680[] = {{&S44031[42], 3}, {NULL}};
const Segment SGI320681[] = {{&S44032[8], 3}, {NULL}};
const Segment SGI320682[] = {{&S44032[13], 10}, {NULL}};
const Segment SGI320683[] = {{&S44032[35], 5}, {NULL}};
const Segment SGI320684[] = {{&S44033[30], 4}, {NULL}};
const Segment SGI320685[] = {{&S44033[8], 3}, {NULL}};
const Segment SGI320686[] = {{&S44033[13], 10}, {NULL}};
const Segment SGI320687[] = {{&S44033[42], 3}, {NULL}};
const Segment SGI320688[] = {{&S52061[4], 5}, {NULL}};
const Segment SGI320689[] = {{&S44296[30], 3}, {NULL}};
const Segment SGI320690[] = {{&S44296[8], 3}, {NULL}};
const Segment SGI320691[] = {{&S44296[13], 10}, {NULL}};
const Segment SGI320692[] = {{&S44296[42], 7}, {NULL}};
const Segment SGI320693[] = {{&S44296[35], 14}, {NULL}};
const Segment SGI320694[] = {{&S44296[71], 11}, {NULL}};
const Segment SGI320695[] = {{&S44296[164], 15}, {NULL}};
const Segment SGI320696[] = {{&S44296[51], 6}, {NULL}};
const Segment SGI320697[] = {{&S44296[71], 16}, {NULL}};
const Segment SGI320698[] = {{&S44297[30], 3}, {NULL}};
const Segment SGI320699[] = {{&S44297[8], 3}, {NULL}};
const Segment SGI320700[] = {{&S44297[13], 10}, {NULL}};
const Segment SGI320701[] = {{&S44297[42], 7}, {NULL}};
const Segment SGI320702[] = {{&S44297[35], 14}, {NULL}};
const Segment SGI320703[] = {{&S44297[71], 11}, {NULL}};
const Segment SGI320704[] = {{&S44297[51], 6}, {NULL}};
const Segment SGI320705[] = {{&S44297[71], 16}, {NULL}};
const Segment SGI320706[] = {{&S44298[30], 3}, {NULL}};
const Segment SGI320707[] = {{&S44298[8], 3}, {NULL}};
const Segment SGI320708[] = {{&S44298[13], 10}, {NULL}};
const Segment SGI320709[] = {{&S44298[42], 7}, {NULL}};
const Segment SGI320710[] = {{&S44298[35], 14}, {NULL}};
const Segment SGI320711[] = {{&S44298[64], 11}, {NULL}};
const Segment SGI320712[] = {{&S44298[64], 16}, {NULL}};
const Segment SGI320713[] = {{&S44306[30], 3}, {NULL}};
const Segment SGI320714[] = {{&S44306[8], 3}, {NULL}};
const Segment SGI320715[] = {{&S44306[13], 10}, {NULL}};
const Segment SGI320716[] = {{&S44306[42], 7}, {NULL}};
const Segment SGI320717[] = {{&S44306[35], 14}, {NULL}};
const Segment SGI320718[] = {{&S44306[64], 11}, {NULL}};
const Segment SGI320719[] = {{&S44306[157], 15}, {NULL}};
const Segment SGI320720[] = {{&S44306[64], 16}, {NULL}};
const Segment SGI320721[] = {{&S44308[30], 3}, {NULL}};
const Segment SGI320722[] = {{&S44308[8], 3}, {NULL}};
const Segment SGI320723[] = {{&S44308[13], 10}, {NULL}};
const Segment SGI320724[] = {{&S44308[42], 7}, {NULL}};
const Segment SGI320725[] = {{&S44308[35], 14}, {NULL}};
const Segment SGI320726[] = {{&S44308[64], 11}, {NULL}};
const Segment SGI320727[] = {{&S44308[157], 15}, {NULL}};
const Segment SGI320728[] = {{&S44308[64], 16}, {NULL}};
const Segment SGI320729[] = {{&S44308[188], 9}, {NULL}};
const Segment SGI320730[] = {{&S44309[30], 3}, {NULL}};
const Segment SGI320731[] = {{&S44309[8], 3}, {NULL}};
const Segment SGI320732[] = {{&S44309[13], 10}, {NULL}};
const Segment SGI320733[] = {{&S44309[42], 7}, {NULL}};
const Segment SGI320734[] = {{&S44309[35], 14}, {NULL}};
const Segment SGI320735[] = {{&S44309[71], 11}, {NULL}};
const Segment SGI320736[] = {{&S44309[51], 6}, {NULL}};
const Segment SGI320737[] = {{&S44309[71], 16}, {NULL}};
const Segment SGI320738[] = {{&S44310[30], 3}, {NULL}};
const Segment SGI320739[] = {{&S44310[8], 3}, {NULL}};
const Segment SGI320740[] = {{&S44310[13], 10}, {NULL}};
const Segment SGI320741[] = {{&S44310[42], 7}, {NULL}};
const Segment SGI320742[] = {{&S44310[35], 14}, {NULL}};
const Segment SGI320743[] = {{&S44310[64], 11}, {NULL}};
const Segment SGI320744[] = {{&S44310[64], 16}, {NULL}};
const Segment SGI320745[] = {{&S44310[189], 9}, {NULL}};
const Segment SGI320746[] = {{&S44311[30], 3}, {NULL}};
const Segment SGI320747[] = {{&S44311[8], 3}, {NULL}};
const Segment SGI320748[] = {{&S44311[13], 10}, {NULL}};
const Segment SGI320749[] = {{&S44311[42], 7}, {NULL}};
const Segment SGI320750[] = {{&S44311[35], 14}, {NULL}};
const Segment SGI320751[] = {{&S44311[64], 11}, {NULL}};
const Segment SGI320752[] = {{&S44311[64], 16}, {NULL}};
const Segment SGI320753[] = {{&S44327[30], 3}, {NULL}};
const Segment SGI320754[] = {{&S44327[8], 3}, {NULL}};
const Segment SGI320755[] = {{&S44327[13], 10}, {NULL}};
const Segment SGI320756[] = {{&S44327[42], 7}, {NULL}};
const Segment SGI320757[] = {{&S44327[35], 14}, {NULL}};
const Segment SGI320758[] = {{&S44327[125], 15}, {NULL}};
const Segment SGI320759[] = {{&S44327[51], 6}, {NULL}};
const Segment SGI320760[] = {{&S44328[30], 3}, {NULL}};
const Segment SGI320761[] = {{&S44328[8], 3}, {NULL}};
const Segment SGI320762[] = {{&S44328[13], 10}, {NULL}};
const Segment SGI320763[] = {{&S44328[42], 7}, {NULL}};
const Segment SGI320764[] = {{&S44328[35], 14}, {NULL}};
const Segment SGI320765[] = {{&S44328[51], 3}, {NULL}};
const Segment SGI320766[] = {{&S44329[29], 3}, {NULL}};
const Segment SGI320767[] = {{&S44329[8], 3}, {NULL}};
const Segment SGI320768[] = {{&S44329[13], 10}, {NULL}};
const Segment SGI320769[] = {{&S44329[40], 7}, {NULL}};
const Segment SGI320770[] = {{&S44330[30], 3}, {NULL}};
const Segment SGI320771[] = {{&S44330[8], 3}, {NULL}};
const Segment SGI320772[] = {{&S44330[13], 10}, {NULL}};
const Segment SGI320773[] = {{&S44330[42], 7}, {NULL}};
const Segment SGI320774[] = {{&S44330[35], 14}, {NULL}};
const Segment SGI320775[] = {{&S44331[30], 3}, {NULL}};
const Segment SGI320776[] = {{&S44331[8], 3}, {NULL}};
const Segment SGI320777[] = {{&S44331[13], 10}, {NULL}};
const Segment SGI320778[] = {{&S44331[42], 7}, {NULL}};
const Segment SGI320779[] = {{&S44331[35], 14}, {NULL}};
const Segment SGI320780[] = {{&S44332[30], 3}, {NULL}};
const Segment SGI320781[] = {{&S44332[8], 3}, {NULL}};
const Segment SGI320782[] = {{&S44332[13], 10}, {NULL}};
const Segment SGI320783[] = {{&S44332[42], 7}, {NULL}};
const Segment SGI320784[] = {{&S44332[35], 14}, {NULL}};
const Segment SGI320785[] = {{&S44332[51], 6}, {NULL}};
const Segment SGI320786[] = {{&S44333[30], 3}, {NULL}};
const Segment SGI320787[] = {{&S44333[8], 3}, {NULL}};
const Segment SGI320788[] = {{&S44333[13], 10}, {NULL}};
const Segment SGI320789[] = {{&S44333[42], 7}, {NULL}};
const Segment SGI320790[] = {{&S44333[35], 14}, {NULL}};
const Segment SGI320791[] = {{&S44333[118], 15}, {NULL}};
const Segment SGI320792[] = {{&S44334[30], 3}, {NULL}};
const Segment SGI320793[] = {{&S44334[8], 3}, {NULL}};
const Segment SGI320794[] = {{&S44334[13], 10}, {NULL}};
const Segment SGI320795[] = {{&S44334[42], 7}, {NULL}};
const Segment SGI320796[] = {{&S44334[35], 14}, {NULL}};
const Segment SGI320797[] = {{&S44334[131], 15}, {NULL}};
const Segment SGI320798[] = {{&S44334[152], 9}, {NULL}};
const Segment SGI320799[] = {{&S44335[30], 3}, {&S44335[30], 3}, {NULL}};
const Segment SGI320800[] = {{&S44335[8], 3}, {&S44335[8], 3}, {NULL}};
const Segment SGI320801[] = {{&S44335[13], 10}, {&S44335[13], 10}, {NULL}};
const Segment SGI320802[] = {{&S44335[42], 7}, {&S44335[98], 7}, {NULL}};
const Segment SGI320803[] = {{&S44335[35], 14}, {&S44335[91], 14}, {NULL}};
const Segment SGI320804[] = {{&S44335[51], 3}, {&S44335[51], 3}, {NULL}};
const Segment SGI320805[] = {{&S44336[30], 3}, {NULL}};
const Segment SGI320806[] = {{&S44336[8], 3}, {NULL}};
const Segment SGI320807[] = {{&S44336[13], 10}, {NULL}};
const Segment SGI320808[] = {{&S44336[42], 7}, {NULL}};
const Segment SGI320809[] = {{&S44336[35], 14}, {NULL}};
const Segment SGI320810[] = {{&S44336[51], 5}, {NULL}};
const Segment SGI320811[] = {{&S44337[30], 3}, {NULL}};
const Segment SGI320812[] = {{&S44337[8], 3}, {NULL}};
const Segment SGI320813[] = {{&S44337[13], 10}, {NULL}};
const Segment SGI320814[] = {{&S44337[42], 7}, {NULL}};
const Segment SGI320815[] = {{&S44337[35], 14}, {NULL}};
const Segment SGI320816[] = {{&S44337[123], 15}, {NULL}};
const Segment SGI320817[] = {{&S44337[51], 3}, {NULL}};
const Segment SGI320818[] = {{&S44338[30], 3}, {NULL}};
const Segment SGI320819[] = {{&S44338[8], 3}, {NULL}};
const Segment SGI320820[] = {{&S44338[13], 10}, {NULL}};
const Segment SGI320821[] = {{&S44338[42], 7}, {NULL}};
const Segment SGI320822[] = {{&S44338[35], 14}, {NULL}};
const Segment SGI320823[] = {{&S44338[51], 6}, {NULL}};
const Segment SGI320824[] = {{&S44339[30], 3}, {NULL}};
const Segment SGI320825[] = {{&S44339[8], 3}, {NULL}};
const Segment SGI320826[] = {{&S44339[13], 10}, {NULL}};
const Segment SGI320827[] = {{&S44339[42], 7}, {&S44339[58], 7}, {NULL}};
const Segment SGI320828[] = {{&S44339[35], 14}, {&S44339[51], 14}, {NULL}};
const Segment SGI320829[] = {{&S60956[3], 5}, {NULL}};
const Segment SGI320830[] = {{&S44495[30], 4}, {NULL}};
const Segment SGI320831[] = {{&S44495[8], 3}, {NULL}};
const Segment SGI320832[] = {{&S44495[13], 10}, {NULL}};
const Segment SGI320833[] = {{&S44495[43], 7}, {NULL}};
const Segment SGI320834[] = {{&S44495[36], 14}, {NULL}};
const Segment SGI320835[] = {{&S44495[65], 11}, {NULL}};
const Segment SGI320836[] = {{&S44495[65], 16}, {NULL}};
const Segment SGI320837[] = {{&S44496[30], 4}, {NULL}};
const Segment SGI320838[] = {{&S44496[8], 3}, {NULL}};
const Segment SGI320839[] = {{&S44496[13], 10}, {NULL}};
const Segment SGI320840[] = {{&S44496[43], 7}, {NULL}};
const Segment SGI320841[] = {{&S44496[36], 14}, {NULL}};
const Segment SGI320842[] = {{&S44496[72], 11}, {NULL}};
const Segment SGI320843[] = {{&S44496[52], 6}, {NULL}};
const Segment SGI320844[] = {{&S44496[72], 16}, {NULL}};
const Segment SGI320845[] = {{&S44497[30], 3}, {NULL}};
const Segment SGI320846[] = {{&S44497[8], 3}, {NULL}};
const Segment SGI320847[] = {{&S44497[13], 10}, {NULL}};
const Segment SGI320848[] = {{&S44497[42], 7}, {NULL}};
const Segment SGI320849[] = {{&S44497[35], 14}, {NULL}};
const Segment SGI320850[] = {{&S44497[71], 11}, {NULL}};
const Segment SGI320851[] = {{&S44497[51], 6}, {NULL}};
const Segment SGI320852[] = {{&S44497[71], 16}, {NULL}};
const Segment SGI320853[] = {{&S44499[30], 3}, {NULL}};
const Segment SGI320854[] = {{&S44499[8], 3}, {NULL}};
const Segment SGI320855[] = {{&S44499[13], 10}, {NULL}};
const Segment SGI320856[] = {{&S44499[42], 7}, {NULL}};
const Segment SGI320857[] = {{&S44499[35], 14}, {NULL}};
const Segment SGI320858[] = {{&S44499[161], 11}, {NULL}};
const Segment SGI320859[] = {{&S44499[51], 6}, {NULL}};
const Segment SGI320860[] = {{&S44499[161], 16}, {NULL}};
const Segment SGI320861[] = {{&S44517[30], 3}, {NULL}};
const Segment SGI320862[] = {{&S44517[8], 3}, {NULL}};
const Segment SGI320863[] = {{&S44517[13], 10}, {NULL}};
const Segment SGI320864[] = {{&S44517[42], 7}, {NULL}};
const Segment SGI320865[] = {{&S44517[35], 14}, {NULL}};
const Segment SGI320866[] = {{&S44517[169], 11}, {NULL}};
const Segment SGI320867[] = {{&S44517[138], 15}, {NULL}};
const Segment SGI320868[] = {{&S44517[51], 6}, {NULL}};
const Segment SGI320869[] = {{&S44517[169], 16}, {NULL}};
const Segment SGI320870[] = {{&S44518[30], 3}, {NULL}};
const Segment SGI320871[] = {{&S44518[8], 3}, {NULL}};
const Segment SGI320872[] = {{&S44518[13], 10}, {NULL}};
const Segment SGI320873[] = {{&S44518[42], 7}, {NULL}};
const Segment SGI320874[] = {{&S44518[35], 14}, {NULL}};
const Segment SGI320875[] = {{&S44518[71], 11}, {NULL}};
const Segment SGI320876[] = {{&S44518[51], 6}, {NULL}};
const Segment SGI320877[] = {{&S44518[71], 16}, {NULL}};
const Segment SGI320878[] = {{&S44519[30], 3}, {NULL}};
const Segment SGI320879[] = {{&S44519[8], 3}, {NULL}};
const Segment SGI320880[] = {{&S44519[13], 10}, {NULL}};
const Segment SGI320881[] = {{&S44519[42], 7}, {NULL}};
const Segment SGI320882[] = {{&S44519[35], 14}, {NULL}};
const Segment SGI320883[] = {{&S44519[64], 11}, {NULL}};
const Segment SGI320884[] = {{&S44519[64], 16}, {NULL}};
const Segment SGI320885[] = {{&S44522[30], 3}, {NULL}};
const Segment SGI320886[] = {{&S44522[8], 3}, {NULL}};
const Segment SGI320887[] = {{&S44522[13], 10}, {NULL}};
const Segment SGI320888[] = {{&S44522[42], 7}, {NULL}};
const Segment SGI320889[] = {{&S44522[35], 14}, {NULL}};
const Segment SGI320890[] = {{&S44522[71], 11}, {NULL}};
const Segment SGI320891[] = {{&S44522[164], 15}, {NULL}};
const Segment SGI320892[] = {{&S44522[51], 6}, {NULL}};
const Segment SGI320893[] = {{&S44522[71], 16}, {NULL}};
const Segment SGI320894[] = {{&S44523[30], 3}, {NULL}};
const Segment SGI320895[] = {{&S44523[8], 3}, {NULL}};
const Segment SGI320896[] = {{&S44523[13], 10}, {NULL}};
const Segment SGI320897[] = {{&S44523[42], 7}, {NULL}};
const Segment SGI320898[] = {{&S44523[35], 14}, {NULL}};
const Segment SGI320899[] = {{&S44523[64], 11}, {NULL}};
const Segment SGI320900[] = {{&S44523[157], 15}, {NULL}};
const Segment SGI320901[] = {{&S44523[64], 16}, {NULL}};
const Segment SGI320902[] = {{&S44524[30], 3}, {NULL}};
const Segment SGI320903[] = {{&S44524[8], 3}, {NULL}};
const Segment SGI320904[] = {{&S44524[13], 10}, {NULL}};
const Segment SGI320905[] = {{&S44524[42], 7}, {NULL}};
const Segment SGI320906[] = {{&S44524[35], 14}, {NULL}};
const Segment SGI320907[] = {{&S44524[64], 11}, {NULL}};
const Segment SGI320908[] = {{&S44524[157], 15}, {NULL}};
const Segment SGI320909[] = {{&S44524[64], 16}, {NULL}};
const Segment SGI320910[] = {{&S44524[217], 9}, {NULL}};
const Segment SGI320911[] = {{&S44525[30], 3}, {NULL}};
const Segment SGI320912[] = {{&S44525[8], 3}, {NULL}};
const Segment SGI320913[] = {{&S44525[13], 10}, {NULL}};
const Segment SGI320914[] = {{&S44525[42], 7}, {NULL}};
const Segment SGI320915[] = {{&S44525[35], 14}, {NULL}};
const Segment SGI320916[] = {{&S44525[71], 11}, {NULL}};
const Segment SGI320917[] = {{&S44525[51], 6}, {NULL}};
const Segment SGI320918[] = {{&S44525[71], 16}, {NULL}};
const Segment SGI320919[] = {{&S44526[30], 3}, {NULL}};
const Segment SGI320920[] = {{&S44526[8], 3}, {NULL}};
const Segment SGI320921[] = {{&S44526[13], 10}, {NULL}};
const Segment SGI320922[] = {{&S44526[42], 7}, {NULL}};
const Segment SGI320923[] = {{&S44526[35], 14}, {NULL}};
const Segment SGI320924[] = {{&S44526[64], 11}, {NULL}};
const Segment SGI320925[] = {{&S44526[64], 16}, {NULL}};
const Segment SGI320926[] = {{&S44527[30], 3}, {NULL}};
const Segment SGI320927[] = {{&S44527[8], 3}, {NULL}};
const Segment SGI320928[] = {{&S44527[13], 10}, {NULL}};
const Segment SGI320929[] = {{&S44527[42], 7}, {NULL}};
const Segment SGI320930[] = {{&S44527[35], 14}, {NULL}};
const Segment SGI320931[] = {{&S44527[141], 11}, {NULL}};
const Segment SGI320932[] = {{&S44527[141], 16}, {NULL}};
const Segment SGI320933[] = {{&S44528[30], 3}, {NULL}};
const Segment SGI320934[] = {{&S44528[8], 3}, {NULL}};
const Segment SGI320935[] = {{&S44528[13], 10}, {NULL}};
const Segment SGI320936[] = {{&S44528[42], 7}, {NULL}};
const Segment SGI320937[] = {{&S44528[35], 14}, {NULL}};
const Segment SGI320938[] = {{&S44528[64], 11}, {NULL}};
const Segment SGI320939[] = {{&S44528[64], 16}, {NULL}};
const Segment SGI320940[] = {{&S44528[189], 9}, {NULL}};
const Segment SGI320941[] = {{&S44529[30], 3}, {NULL}};
const Segment SGI320942[] = {{&S44529[8], 3}, {NULL}};
const Segment SGI320943[] = {{&S44529[13], 10}, {NULL}};
const Segment SGI320944[] = {{&S44529[42], 7}, {NULL}};
const Segment SGI320945[] = {{&S44529[35], 14}, {NULL}};
const Segment SGI320946[] = {{&S44529[76], 11}, {NULL}};
const Segment SGI320947[] = {{&S44529[169], 15}, {NULL}};
const Segment SGI320948[] = {{&S44529[76], 16}, {NULL}};
const Segment SGI320949[] = {{&S44529[51], 5}, {NULL}};
const Segment SGI320950[] = {{&S44530[30], 3}, {NULL}};
const Segment SGI320951[] = {{&S44530[8], 3}, {NULL}};
const Segment SGI320952[] = {{&S44530[13], 10}, {NULL}};
const Segment SGI320953[] = {{&S44530[42], 7}, {NULL}};
const Segment SGI320954[] = {{&S44530[35], 14}, {NULL}};
const Segment SGI320955[] = {{&S44530[64], 11}, {NULL}};
const Segment SGI320956[] = {{&S44530[157], 15}, {NULL}};
const Segment SGI320957[] = {{&S44530[64], 16}, {NULL}};
const Segment SGI320958[] = {{&S44531[30], 4}, {NULL}};
const Segment SGI320959[] = {{&S44531[8], 3}, {NULL}};
const Segment SGI320960[] = {{&S44531[13], 10}, {NULL}};
const Segment SGI320961[] = {{&S44531[43], 7}, {NULL}};
const Segment SGI320962[] = {{&S44531[36], 14}, {NULL}};
const Segment SGI320963[] = {{&S44531[72], 11}, {NULL}};
const Segment SGI320964[] = {{&S44531[52], 6}, {NULL}};
const Segment SGI320965[] = {{&S44531[72], 16}, {NULL}};
const Segment SGI320966[] = {{&S44532[30], 4}, {NULL}};
const Segment SGI320967[] = {{&S44532[8], 3}, {NULL}};
const Segment SGI320968[] = {{&S44532[13], 10}, {NULL}};
const Segment SGI320969[] = {{&S44532[43], 7}, {NULL}};
const Segment SGI320970[] = {{&S44532[36], 14}, {NULL}};
const Segment SGI320971[] = {{&S44532[65], 11}, {NULL}};
const Segment SGI320972[] = {{&S44532[65], 16}, {NULL}};
const Segment SGI320973[] = {{&S61195[3], 5}, {NULL}};
const Segment SGI320974[] = {{&S61196[3], 5}, {NULL}};
const Segment SGI320975[] = {{&S61196[3], 14}, {NULL}};
const Segment SGI320976[] = {{&S61197[3], 5}, {NULL}};
const Segment SGI320977[] = {{&S61197[3], 12}, {NULL}};
const Segment SGI320978[] = {{&S58508[0], 126}, {NULL}};
const Segment SGI320979[] = {{&S58510[0], 126}, {NULL}};
const Segment SGI320980[] = {{&S58511[0], 133}, {NULL}};
const Segment SGI320981[] = {{&S58512[0], 133}, {NULL}};
const Segment SGI320982[] = {{&S58513[0], 139}, {NULL}};
const Segment SGI320983[] = {{&S58514[0], 126}, {NULL}};
const Segment SGI320984[] = {{&S58515[0], 139}, {NULL}};
const Segment SGI320985[] = {{&S58516[0], 139}, {NULL}};
const Segment SGI320986[] = {{&S58517[0], 121}, {NULL}};
const Segment SGI320987[] = {{&S44848[30], 4}, {NULL}};
const Segment SGI320988[] = {{&S44848[8], 3}, {NULL}};
const Segment SGI320989[] = {{&S44848[13], 10}, {NULL}};
const Segment SGI320990[] = {{&S44848[43], 7}, {NULL}};
const Segment SGI320991[] = {{&S44848[36], 14}, {NULL}};
const Segment SGI320992[] = {{&S44848[72], 11}, {NULL}};
const Segment SGI320993[] = {{&S44848[52], 6}, {NULL}};
const Segment SGI320994[] = {{&S44848[72], 16}, {NULL}};
const Segment SGI320995[] = {{&S44850[30], 4}, {NULL}};
const Segment SGI320996[] = {{&S44850[8], 3}, {NULL}};
const Segment SGI320997[] = {{&S44850[13], 10}, {NULL}};
const Segment SGI320998[] = {{&S44850[43], 7}, {NULL}};
const Segment SGI320999[] = {{&S44850[36], 14}, {NULL}};
const Segment SGI321000[] = {{&S44850[65], 11}, {NULL}};
const Segment SGI321001[] = {{&S44850[65], 16}, {NULL}};
const Segment SGI321002[] = {{&S44851[30], 3}, {NULL}};
const Segment SGI321003[] = {{&S44851[8], 3}, {NULL}};
const Segment SGI321004[] = {{&S44851[13], 10}, {NULL}};
const Segment SGI321005[] = {{&S44851[42], 7}, {NULL}};
const Segment SGI321006[] = {{&S44851[35], 14}, {NULL}};
const Segment SGI321007[] = {{&S44851[151], 11}, {NULL}};
const Segment SGI321008[] = {{&S44851[51], 6}, {NULL}};
const Segment SGI321009[] = {{&S44851[151], 16}, {NULL}};
const Segment SGI321010[] = {{&S45042[30], 3}, {NULL}};
const Segment SGI321011[] = {{&S45042[8], 3}, {NULL}};
const Segment SGI321012[] = {{&S45042[13], 10}, {NULL}};
const Segment SGI321013[] = {{&S45042[42], 7}, {NULL}};
const Segment SGI321014[] = {{&S45042[35], 14}, {NULL}};
const Segment SGI321015[] = {{&S45042[64], 11}, {NULL}};
const Segment SGI321016[] = {{&S45042[64], 16}, {NULL}};
const Segment SGI321017[] = {{&S45043[30], 3}, {NULL}};
const Segment SGI321018[] = {{&S45043[8], 3}, {NULL}};
const Segment SGI321019[] = {{&S45043[13], 10}, {NULL}};
const Segment SGI321020[] = {{&S45043[42], 7}, {NULL}};
const Segment SGI321021[] = {{&S45043[35], 14}, {NULL}};
const Segment SGI321022[] = {{&S45043[71], 11}, {NULL}};
const Segment SGI321023[] = {{&S45043[51], 6}, {NULL}};
const Segment SGI321024[] = {{&S45043[71], 16}, {NULL}};
const Segment SGI321025[] = {{&S45044[30], 3}, {NULL}};
const Segment SGI321026[] = {{&S45044[8], 3}, {NULL}};
const Segment SGI321027[] = {{&S45044[13], 10}, {NULL}};
const Segment SGI321028[] = {{&S45044[42], 7}, {NULL}};
const Segment SGI321029[] = {{&S45044[35], 14}, {NULL}};
const Segment SGI321030[] = {{&S45044[64], 11}, {NULL}};
const Segment SGI321031[] = {{&S45044[156], 15}, {NULL}};
const Segment SGI321032[] = {{&S45044[64], 16}, {NULL}};
const Segment SGI321033[] = {{&S45179[30], 3}, {NULL}};
const Segment SGI321034[] = {{&S45179[8], 3}, {NULL}};
const Segment SGI321035[] = {{&S45179[13], 10}, {NULL}};
const Segment SGI321036[] = {{&S45179[42], 7}, {NULL}};
const Segment SGI321037[] = {{&S45179[35], 14}, {NULL}};
const Segment SGI321038[] = {{&S45179[51], 3}, {NULL}};
const Segment SGI321039[] = {{&S45179[69], 5}, {NULL}};
const Segment SGI321040[] = {{&S45179[96], 9}, {NULL}};
const Segment SGI321041[] = {{&S45180[30], 3}, {NULL}};
const Segment SGI321042[] = {{&S45180[8], 3}, {NULL}};
const Segment SGI321043[] = {{&S45180[13], 10}, {NULL}};
const Segment SGI321044[] = {{&S45180[42], 7}, {NULL}};
const Segment SGI321045[] = {{&S45180[35], 14}, {NULL}};
const Segment SGI321046[] = {{&S45180[51], 3}, {&S45180[51], 3}, {NULL}};
const Segment SGI321047[] = {{&S45180[69], 5}, {NULL}};
const Segment SGI321048[] = {{&S45180[96], 9}, {NULL}};
const Segment SGI321049[] = {{&S45181[30], 3}, {NULL}};
const Segment SGI321050[] = {{&S45181[8], 3}, {NULL}};
const Segment SGI321051[] = {{&S45181[13], 10}, {NULL}};
const Segment SGI321052[] = {{&S45181[42], 7}, {NULL}};
const Segment SGI321053[] = {{&S45181[35], 14}, {NULL}};
const Segment SGI321054[] = {{&S45181[51], 3}, {NULL}};
const Segment SGI321055[] = {{&S45181[89], 9}, {NULL}};
const Segment SGI321056[] = {{&S45182[30], 3}, {NULL}};
const Segment SGI321057[] = {{&S45182[8], 3}, {NULL}};
const Segment SGI321058[] = {{&S45182[13], 10}, {NULL}};
const Segment SGI321059[] = {{&S45182[42], 7}, {NULL}};
const Segment SGI321060[] = {{&S45182[35], 14}, {NULL}};
const Segment SGI321061[] = {{&S45182[51], 3}, {&S45182[51], 3}, {NULL}};
const Segment SGI321062[] = {{&S45182[89], 9}, {NULL}};
const Segment SGI321063[] = {{&S58988[0], 50}, {NULL}};
const Segment SGI321064[] = {{&S58989[0], 50}, {NULL}};
const Segment SGI321065[] = {{&S58990[0], 50}, {NULL}};
const Segment SGI321066[] = {{&S58991[0], 18}, {NULL}};
const Segment SGI321067[] = {{&S61797[3], 5}, {NULL}};
const Segment SGI321068[] = {{&S61797[3], 12}, {NULL}};
const Segment SGI321069[] = {{&S61798[3], 5}, {NULL}};
const Segment SGI321070[] = {{&S59061[0], 39}, {NULL}};
const Segment SGI321071[] = {{&S45423[30], 3}, {NULL}};
const Segment SGI321072[] = {{&S45423[8], 3}, {NULL}};
const Segment SGI321073[] = {{&S45423[13], 10}, {NULL}};
const Segment SGI321074[] = {{&S45423[42], 7}, {NULL}};
const Segment SGI321075[] = {{&S45423[35], 14}, {NULL}};
const Segment SGI321076[] = {{&S45423[64], 11}, {NULL}};
const Segment SGI321077[] = {{&S45423[64], 16}, {NULL}};
const Segment SGI321078[] = {{&S45424[30], 3}, {NULL}};
const Segment SGI321079[] = {{&S45424[8], 3}, {NULL}};
const Segment SGI321080[] = {{&S45424[13], 10}, {NULL}};
const Segment SGI321081[] = {{&S45424[42], 7}, {NULL}};
const Segment SGI321082[] = {{&S45424[35], 14}, {NULL}};
const Segment SGI321083[] = {{&S45424[71], 11}, {NULL}};
const Segment SGI321084[] = {{&S45424[51], 6}, {NULL}};
const Segment SGI321085[] = {{&S45424[71], 16}, {NULL}};
const Segment SGI321086[] = {{&S45426[30], 3}, {NULL}};
const Segment SGI321087[] = {{&S45426[8], 3}, {NULL}};
const Segment SGI321088[] = {{&S45426[13], 10}, {NULL}};
const Segment SGI321089[] = {{&S45426[42], 7}, {NULL}};
const Segment SGI321090[] = {{&S45426[35], 14}, {NULL}};
const Segment SGI321091[] = {{&S45426[51], 11}, {NULL}};
const Segment SGI321092[] = {{&S45426[51], 16}, {NULL}};
const Segment SGI321093[] = {{&S45427[30], 3}, {NULL}};
const Segment SGI321094[] = {{&S45427[8], 3}, {NULL}};
const Segment SGI321095[] = {{&S45427[13], 10}, {NULL}};
const Segment SGI321096[] = {{&S45427[42], 7}, {NULL}};
const Segment SGI321097[] = {{&S45427[35], 14}, {NULL}};
const Segment SGI321098[] = {{&S45427[72], 11}, {NULL}};
const Segment SGI321099[] = {{&S45427[72], 16}, {NULL}};
const Segment SGI321100[] = {{&S45429[30], 3}, {NULL}};
const Segment SGI321101[] = {{&S45429[8], 3}, {NULL}};
const Segment SGI321102[] = {{&S45429[13], 10}, {NULL}};
const Segment SGI321103[] = {{&S45429[42], 7}, {NULL}};
const Segment SGI321104[] = {{&S45429[35], 14}, {NULL}};
const Segment SGI321105[] = {{&S45429[177], 11}, {NULL}};
const Segment SGI321106[] = {{&S45429[91], 15}, {NULL}};
const Segment SGI321107[] = {{&S45429[177], 16}, {NULL}};
const Segment SGI321108[] = {{&S45430[30], 3}, {NULL}};
const Segment SGI321109[] = {{&S45430[8], 3}, {NULL}};
const Segment SGI321110[] = {{&S45430[13], 10}, {NULL}};
const Segment SGI321111[] = {{&S45430[42], 7}, {NULL}};
const Segment SGI321112[] = {{&S45430[35], 14}, {NULL}};
const Segment SGI321113[] = {{&S45430[177], 11}, {NULL}};
const Segment SGI321114[] = {{&S45430[91], 15}, {NULL}};
const Segment SGI321115[] = {{&S45430[177], 16}, {NULL}};
const Segment SGI321116[] = {{&S59187[0], 83}, {NULL}};
const Segment SGI321117[] = {{&S62259[3], 5}, {NULL}};
const Segment SGI321118[] = {{&S62316[3], 5}, {NULL}};
const Segment SGI321119[] = {{&S62316[80], 13}, {NULL}};
const Segment SGI321120[] = {{&S62493[3], 5}, {NULL}};
const Segment SGI321121[] = {{&S59789[0], 74}, {NULL}};
const Segment SGI321122[] = {{&S62652[3], 5}, {NULL}};
const Segment SGI321123[] = {{&S80670[0], 83}, {NULL}};
const Segment SGI321124[] = {{&S46105[30], 3}, {NULL}};
const Segment SGI321125[] = {{&S46105[8], 3}, {NULL}};
const Segment SGI321126[] = {{&S46105[13], 10}, {NULL}};
const Segment SGI321127[] = {{&S46107[30], 3}, {NULL}};
const Segment SGI321128[] = {{&S46107[8], 3}, {NULL}};
const Segment SGI321129[] = {{&S46107[13], 10}, {NULL}};
const Segment SGI321130[] = {{&S46107[122], 15}, {NULL}};
const Segment SGI321131[] = {{&S46107[50], 3}, {NULL}};
const Segment SGI321132[] = {{&S46108[30], 3}, {NULL}};
const Segment SGI321133[] = {{&S46108[8], 3}, {NULL}};
const Segment SGI321134[] = {{&S46108[13], 10}, {NULL}};
const Segment SGI321135[] = {{&S46108[35], 3}, {NULL}};
const Segment SGI321136[] = {{&S62735[3], 5}, {NULL}};
const Segment SGI321137[] = {{&S46109[30], 3}, {NULL}};
const Segment SGI321138[] = {{&S46109[8], 3}, {NULL}};
const Segment SGI321139[] = {{&S46109[13], 10}, {NULL}};
const Segment SGI321140[] = {{&S46110[30], 3}, {NULL}};
const Segment SGI321141[] = {{&S46110[8], 3}, {NULL}};
const Segment SGI321142[] = {{&S46110[13], 10}, {NULL}};
const Segment SGI321143[] = {{&S46110[35], 3}, {NULL}};
const Segment SGI321144[] = {{&S46110[40], 3}, {NULL}};
const Segment SGI321145[] = {{&S62912[3], 4}, {NULL}};
const Segment SGI321146[] = {{&S62912[3], 11}, {NULL}};
const Segment SGI321147[] = {{&S62913[3], 4}, {NULL}};
const Segment SGI321148[] = {{&S74829[3], 5}, {NULL}};
const Segment SGI321149[] = {{&S74829[28], 9}, {NULL}};
const Segment SGI321150[] = {{&S46663[30], 3}, {NULL}};
const Segment SGI321151[] = {{&S46663[8], 3}, {NULL}};
const Segment SGI321152[] = {{&S46663[13], 10}, {NULL}};
const Segment SGI321153[] = {{&S46663[35], 5}, {NULL}};
const Segment SGI321154[] = {{&S46664[30], 3}, {NULL}};
const Segment SGI321155[] = {{&S46664[8], 3}, {NULL}};
const Segment SGI321156[] = {{&S46664[13], 10}, {NULL}};
const Segment SGI321157[] = {{&S46664[35], 5}, {NULL}};
const Segment SGI321158[] = {{&S46664[49], 5}, {NULL}};
const Segment SGI321159[] = {{&S46665[30], 3}, {NULL}};
const Segment SGI321160[] = {{&S46665[8], 3}, {NULL}};
const Segment SGI321161[] = {{&S46665[13], 10}, {NULL}};
const Segment SGI321162[] = {{&S46665[35], 5}, {NULL}};
const Segment SGI321163[] = {{&S46665[47], 6}, {NULL}};
const Segment SGI321164[] = {{&S46666[30], 3}, {NULL}};
const Segment SGI321165[] = {{&S46666[8], 3}, {NULL}};
const Segment SGI321166[] = {{&S46666[13], 10}, {NULL}};
const Segment SGI321167[] = {{&S46666[35], 5}, {NULL}};
const Segment SGI321168[] = {{&S46666[47], 3}, {NULL}};
const Segment SGI321169[] = {{&S46667[30], 3}, {NULL}};
const Segment SGI321170[] = {{&S46667[8], 3}, {NULL}};
const Segment SGI321171[] = {{&S46667[13], 10}, {NULL}};
const Segment SGI321172[] = {{&S46667[35], 5}, {NULL}};
const Segment SGI321173[] = {{&S46667[47], 9}, {NULL}};
const Segment SGI321174[] = {{&S46689[30], 3}, {NULL}};
const Segment SGI321175[] = {{&S46689[8], 3}, {NULL}};
const Segment SGI321176[] = {{&S46689[13], 10}, {NULL}};
const Segment SGI321177[] = {{&S46689[42], 7}, {NULL}};
const Segment SGI321178[] = {{&S46689[35], 14}, {NULL}};
const Segment SGI321179[] = {{&S46689[115], 11}, {NULL}};
const Segment SGI321180[] = {{&S46689[115], 15}, {NULL}};
const Segment SGI321181[] = {{&S46690[30], 3}, {NULL}};
const Segment SGI321182[] = {{&S46690[8], 3}, {NULL}};
const Segment SGI321183[] = {{&S46690[13], 10}, {NULL}};
const Segment SGI321184[] = {{&S46690[42], 7}, {NULL}};
const Segment SGI321185[] = {{&S46690[35], 14}, {NULL}};
const Segment SGI321186[] = {{&S46690[148], 11}, {NULL}};
const Segment SGI321187[] = {{&S46690[148], 15}, {NULL}};
const Segment SGI321188[] = {{&S46691[30], 3}, {NULL}};
const Segment SGI321189[] = {{&S46691[8], 3}, {NULL}};
const Segment SGI321190[] = {{&S46691[13], 10}, {NULL}};
const Segment SGI321191[] = {{&S46691[42], 7}, {NULL}};
const Segment SGI321192[] = {{&S46691[35], 14}, {NULL}};
const Segment SGI321193[] = {{&S46691[120], 11}, {NULL}};
const Segment SGI321194[] = {{&S46691[120], 15}, {NULL}};
const Segment SGI321195[] = {{&S46691[51], 3}, {NULL}};
const Segment SGI321196[] = {{&S46698[30], 4}, {NULL}};
const Segment SGI321197[] = {{&S46698[8], 3}, {NULL}};
const Segment SGI321198[] = {{&S46698[13], 10}, {NULL}};
const Segment SGI321199[] = {{&S46698[36], 5}, {NULL}};
const Segment SGI321200[] = {{&S46699[30], 4}, {NULL}};
const Segment SGI321201[] = {{&S46699[8], 3}, {NULL}};
const Segment SGI321202[] = {{&S46699[13], 10}, {NULL}};
const Segment SGI321203[] = {{&S46699[36], 5}, {NULL}};
const Segment SGI321204[] = {{&S46699[50], 6}, {NULL}};
const Segment SGI321205[] = {{&S46700[30], 4}, {NULL}};
const Segment SGI321206[] = {{&S46700[8], 3}, {NULL}};
const Segment SGI321207[] = {{&S46700[13], 10}, {NULL}};
const Segment SGI321208[] = {{&S46700[36], 5}, {NULL}};
const Segment SGI321209[] = {{&S46700[48], 3}, {NULL}};
const Segment SGI321210[] = {{&S46701[30], 4}, {NULL}};
const Segment SGI321211[] = {{&S46701[8], 3}, {NULL}};
const Segment SGI321212[] = {{&S46701[13], 10}, {NULL}};
const Segment SGI321213[] = {{&S46701[36], 3}, {NULL}};
const Segment SGI321214[] = {{&S46701[44], 5}, {NULL}};
const Segment SGI321215[] = {{&S46702[30], 4}, {NULL}};
const Segment SGI321216[] = {{&S46702[8], 3}, {NULL}};
const Segment SGI321217[] = {{&S46702[13], 10}, {NULL}};
const Segment SGI321218[] = {{&S46702[36], 5}, {NULL}};
const Segment SGI321219[] = {{&S46702[50], 5}, {NULL}};
const Segment SGI321220[] = {{&S46703[30], 4}, {NULL}};
const Segment SGI321221[] = {{&S46703[8], 3}, {NULL}};
const Segment SGI321222[] = {{&S46703[13], 10}, {NULL}};
const Segment SGI321223[] = {{&S46703[36], 5}, {NULL}};
const Segment SGI321224[] = {{&S46703[48], 9}, {NULL}};
const Segment SGI321225[] = {{&S46781[30], 3}, {NULL}};
const Segment SGI321226[] = {{&S46781[8], 3}, {NULL}};
const Segment SGI321227[] = {{&S46781[13], 10}, {NULL}};
const Segment SGI321228[] = {{&S46781[35], 5}, {NULL}};
const Segment SGI321229[] = {{&S46782[30], 3}, {NULL}};
const Segment SGI321230[] = {{&S46782[8], 3}, {NULL}};
const Segment SGI321231[] = {{&S46782[13], 10}, {NULL}};
const Segment SGI321232[] = {{&S46782[35], 5}, {NULL}};
const Segment SGI321233[] = {{&S46782[49], 3}, {NULL}};
const Segment SGI321234[] = {{&S46783[30], 3}, {NULL}};
const Segment SGI321235[] = {{&S46783[8], 3}, {NULL}};
const Segment SGI321236[] = {{&S46783[13], 10}, {NULL}};
const Segment SGI321237[] = {{&S46783[35], 5}, {NULL}};
const Segment SGI321238[] = {{&S46783[49], 6}, {NULL}};
const Segment SGI321239[] = {{&S46784[30], 3}, {NULL}};
const Segment SGI321240[] = {{&S46784[8], 3}, {NULL}};
const Segment SGI321241[] = {{&S46784[13], 10}, {NULL}};
const Segment SGI321242[] = {{&S46784[35], 5}, {NULL}};
const Segment SGI321243[] = {{&S46784[49], 5}, {NULL}};
const Segment SGI321244[] = {{&S46785[30], 3}, {NULL}};
const Segment SGI321245[] = {{&S46785[8], 3}, {NULL}};
const Segment SGI321246[] = {{&S46785[13], 10}, {NULL}};
const Segment SGI321247[] = {{&S46785[35], 5}, {NULL}};
const Segment SGI321248[] = {{&S46785[49], 9}, {NULL}};
const Segment SGI321249[] = {{&S46998[8], 3}, {NULL}};
const Segment SGI321250[] = {{&S46998[13], 10}, {NULL}};
const Segment SGI321251[] = {{&S46998[35], 5}, {NULL}};
const Segment SGI321252[] = {{&S46999[8], 3}, {NULL}};
const Segment SGI321253[] = {{&S46999[13], 10}, {NULL}};
const Segment SGI321254[] = {{&S46999[35], 5}, {NULL}};
const Segment SGI321255[] = {{&S46999[49], 3}, {NULL}};
const Segment SGI321256[] = {{&S47000[8], 3}, {NULL}};
const Segment SGI321257[] = {{&S47000[13], 10}, {NULL}};
const Segment SGI321258[] = {{&S47000[35], 5}, {NULL}};
const Segment SGI321259[] = {{&S47000[49], 6}, {NULL}};
const Segment SGI321260[] = {{&S47001[8], 3}, {NULL}};
const Segment SGI321261[] = {{&S47001[13], 10}, {NULL}};
const Segment SGI321262[] = {{&S47001[35], 5}, {NULL}};
const Segment SGI321263[] = {{&S47001[49], 5}, {NULL}};
const Segment SGI321264[] = {{&S47002[8], 3}, {NULL}};
const Segment SGI321265[] = {{&S47002[13], 10}, {NULL}};
const Segment SGI321266[] = {{&S47002[35], 5}, {NULL}};
const Segment SGI321267[] = {{&S47002[49], 9}, {NULL}};
const Segment SGI321268[] = {{&S47101[30], 3}, {NULL}};
const Segment SGI321269[] = {{&S47101[8], 3}, {NULL}};
const Segment SGI321270[] = {{&S47101[13], 10}, {NULL}};
const Segment SGI321271[] = {{&S47101[35], 5}, {NULL}};
const Segment SGI321272[] = {{&S47102[30], 3}, {NULL}};
const Segment SGI321273[] = {{&S47102[8], 3}, {NULL}};
const Segment SGI321274[] = {{&S47102[13], 10}, {NULL}};
const Segment SGI321275[] = {{&S47102[35], 5}, {NULL}};
const Segment SGI321276[] = {{&S47102[49], 5}, {NULL}};
const Segment SGI321277[] = {{&S47103[30], 3}, {NULL}};
const Segment SGI321278[] = {{&S47103[8], 3}, {NULL}};
const Segment SGI321279[] = {{&S47103[13], 10}, {NULL}};
const Segment SGI321280[] = {{&S47103[35], 5}, {NULL}};
const Segment SGI321281[] = {{&S47103[47], 6}, {NULL}};
const Segment SGI321282[] = {{&S47104[30], 3}, {NULL}};
const Segment SGI321283[] = {{&S47104[8], 3}, {NULL}};
const Segment SGI321284[] = {{&S47104[13], 10}, {NULL}};
const Segment SGI321285[] = {{&S47104[35], 5}, {NULL}};
const Segment SGI321286[] = {{&S47104[47], 3}, {NULL}};
const Segment SGI321287[] = {{&S47105[30], 3}, {NULL}};
const Segment SGI321288[] = {{&S47105[8], 3}, {NULL}};
const Segment SGI321289[] = {{&S47105[13], 10}, {NULL}};
const Segment SGI321290[] = {{&S47105[35], 5}, {NULL}};
const Segment SGI321291[] = {{&S47105[47], 9}, {NULL}};
const Segment SGI321292[] = {{&S47151[31], 4}, {NULL}};
const Segment SGI321293[] = {{&S47151[8], 3}, {NULL}};
const Segment SGI321294[] = {{&S47151[44], 4}, {NULL}};
const Segment SGI321295[] = {{&S47152[30], 4}, {NULL}};
const Segment SGI321296[] = {{&S47152[8], 3}, {NULL}};
const Segment SGI321297[] = {{&S47152[13], 10}, {NULL}};
const Segment SGI321298[] = {{&S47152[36], 6}, {NULL}};
const Segment SGI321299[] = {{&S63689[3], 5}, {NULL}};
const Segment SGI321300[] = {{&S63689[80], 13}, {NULL}};
const Segment SGI321301[] = {{&S47214[30], 3}, {NULL}};
const Segment SGI321302[] = {{&S47214[8], 3}, {NULL}};
const Segment SGI321303[] = {{&S47214[13], 10}, {NULL}};
const Segment SGI321304[] = {{&S47214[42], 7}, {NULL}};
const Segment SGI321305[] = {{&S47214[35], 14}, {NULL}};
const Segment SGI321306[] = {{&S47214[138], 15}, {NULL}};
const Segment SGI321307[] = {{&S47214[51], 6}, {NULL}};
const Segment SGI321308[] = {{&S47214[179], 4}, {NULL}};
const Segment SGI321309[] = {{&S47215[30], 3}, {NULL}};
const Segment SGI321310[] = {{&S47215[8], 3}, {NULL}};
const Segment SGI321311[] = {{&S47215[13], 10}, {NULL}};
const Segment SGI321312[] = {{&S47215[42], 7}, {NULL}};
const Segment SGI321313[] = {{&S47215[35], 14}, {NULL}};
const Segment SGI321314[] = {{&S47215[131], 15}, {NULL}};
const Segment SGI321315[] = {{&S47215[172], 4}, {NULL}};
const Segment SGI321316[] = {{&S47217[30], 3}, {NULL}};
const Segment SGI321317[] = {{&S47217[8], 3}, {NULL}};
const Segment SGI321318[] = {{&S47217[13], 10}, {NULL}};
const Segment SGI321319[] = {{&S47217[42], 7}, {NULL}};
const Segment SGI321320[] = {{&S47217[35], 14}, {NULL}};
const Segment SGI321321[] = {{&S47217[131], 15}, {NULL}};
const Segment SGI321322[] = {{&S47217[152], 9}, {NULL}};
const Segment SGI321323[] = {{&S47217[177], 4}, {NULL}};
const Segment SGI321324[] = {{&S47218[30], 3}, {NULL}};
const Segment SGI321325[] = {{&S47218[8], 3}, {NULL}};
const Segment SGI321326[] = {{&S47218[13], 10}, {NULL}};
const Segment SGI321327[] = {{&S47218[42], 7}, {NULL}};
const Segment SGI321328[] = {{&S47218[35], 14}, {NULL}};
const Segment SGI321329[] = {{&S47218[151], 4}, {NULL}};
const Segment SGI321330[] = {{&S47220[30], 3}, {NULL}};
const Segment SGI321331[] = {{&S47220[8], 3}, {NULL}};
const Segment SGI321332[] = {{&S47220[13], 10}, {NULL}};
const Segment SGI321333[] = {{&S47220[42], 7}, {NULL}};
const Segment SGI321334[] = {{&S47220[35], 14}, {NULL}};
const Segment SGI321335[] = {{&S47220[74], 4}, {NULL}};
const Segment SGI321336[] = {{&S47221[30], 3}, {NULL}};
const Segment SGI321337[] = {{&S47221[8], 3}, {NULL}};
const Segment SGI321338[] = {{&S47221[13], 10}, {NULL}};
const Segment SGI321339[] = {{&S47221[42], 7}, {NULL}};
const Segment SGI321340[] = {{&S47221[35], 14}, {NULL}};
const Segment SGI321341[] = {{&S47221[51], 3}, {NULL}};
const Segment SGI321342[] = {{&S47221[66], 4}, {NULL}};
const Segment SGI321343[] = {{&S47222[30], 3}, {NULL}};
const Segment SGI321344[] = {{&S47222[8], 3}, {NULL}};
const Segment SGI321345[] = {{&S47222[13], 10}, {NULL}};
const Segment SGI321346[] = {{&S47222[42], 7}, {NULL}};
const Segment SGI321347[] = {{&S47222[35], 14}, {NULL}};
const Segment SGI321348[] = {{&S47222[51], 6}, {NULL}};
const Segment SGI321349[] = {{&S47222[197], 4}, {NULL}};
const Segment SGI321350[] = {{&S47223[30], 3}, {NULL}};
const Segment SGI321351[] = {{&S47223[8], 3}, {NULL}};
const Segment SGI321352[] = {{&S47223[13], 10}, {NULL}};
const Segment SGI321353[] = {{&S47223[42], 7}, {NULL}};
const Segment SGI321354[] = {{&S47223[35], 14}, {NULL}};
const Segment SGI321355[] = {{&S47223[51], 3}, {NULL}};
const Segment SGI321356[] = {{&S47223[66], 4}, {NULL}};
const Segment SGI321357[] = {{&S47224[30], 3}, {NULL}};
const Segment SGI321358[] = {{&S47224[8], 3}, {NULL}};
const Segment SGI321359[] = {{&S47224[13], 10}, {NULL}};
const Segment SGI321360[] = {{&S47224[42], 7}, {NULL}};
const Segment SGI321361[] = {{&S47224[35], 14}, {NULL}};
const Segment SGI321362[] = {{&S47224[74], 4}, {NULL}};
const Segment SGI321363[] = {{&S47225[30], 3}, {NULL}};
const Segment SGI321364[] = {{&S47225[8], 3}, {NULL}};
const Segment SGI321365[] = {{&S47225[13], 10}, {NULL}};
const Segment SGI321366[] = {{&S47225[42], 7}, {NULL}};
const Segment SGI321367[] = {{&S47225[35], 14}, {NULL}};
const Segment SGI321368[] = {{&S47225[51], 6}, {NULL}};
const Segment SGI321369[] = {{&S47225[108], 4}, {NULL}};
const Segment SGI321370[] = {{&S47226[30], 3}, {NULL}};
const Segment SGI321371[] = {{&S47226[8], 3}, {NULL}};
const Segment SGI321372[] = {{&S47226[13], 10}, {NULL}};
const Segment SGI321373[] = {{&S47226[42], 7}, {NULL}};
const Segment SGI321374[] = {{&S47226[35], 14}, {NULL}};
const Segment SGI321375[] = {{&S47226[131], 15}, {NULL}};
const Segment SGI321376[] = {{&S47226[162], 4}, {NULL}};
const Segment SGI321377[] = {{&S47227[30], 3}, {NULL}};
const Segment SGI321378[] = {{&S47227[8], 3}, {NULL}};
const Segment SGI321379[] = {{&S47227[13], 10}, {NULL}};
const Segment SGI321380[] = {{&S47227[42], 7}, {NULL}};
const Segment SGI321381[] = {{&S47227[35], 14}, {NULL}};
const Segment SGI321382[] = {{&S47227[51], 6}, {NULL}};
const Segment SGI321383[] = {{&S47227[108], 4}, {NULL}};
const Segment SGI321384[] = {{&S47228[30], 3}, {NULL}};
const Segment SGI321385[] = {{&S47228[8], 3}, {NULL}};
const Segment SGI321386[] = {{&S47228[13], 10}, {NULL}};
const Segment SGI321387[] = {{&S47228[42], 7}, {NULL}};
const Segment SGI321388[] = {{&S47228[35], 14}, {NULL}};
const Segment SGI321389[] = {{&S47228[74], 4}, {NULL}};
const Segment SGI321390[] = {{&S47229[30], 4}, {NULL}};
const Segment SGI321391[] = {{&S47229[8], 3}, {NULL}};
const Segment SGI321392[] = {{&S47229[13], 10}, {NULL}};
const Segment SGI321393[] = {{&S47229[43], 7}, {NULL}};
const Segment SGI321394[] = {{&S47229[36], 14}, {NULL}};
const Segment SGI321395[] = {{&S47229[52], 6}, {NULL}};
const Segment SGI321396[] = {{&S47229[109], 4}, {NULL}};
const Segment SGI321397[] = {{&S47230[30], 3}, {NULL}};
const Segment SGI321398[] = {{&S47230[8], 3}, {NULL}};
const Segment SGI321399[] = {{&S47230[13], 10}, {NULL}};
const Segment SGI321400[] = {{&S47230[42], 7}, {NULL}};
const Segment SGI321401[] = {{&S47230[35], 14}, {NULL}};
const Segment SGI321402[] = {{&S47230[138], 15}, {NULL}};
const Segment SGI321403[] = {{&S47230[51], 6}, {NULL}};
const Segment SGI321404[] = {{&S47230[189], 4}, {NULL}};
const Segment SGI321405[] = {{&S47231[30], 3}, {NULL}};
const Segment SGI321406[] = {{&S47231[8], 3}, {NULL}};
const Segment SGI321407[] = {{&S47231[13], 10}, {NULL}};
const Segment SGI321408[] = {{&S47231[42], 7}, {NULL}};
const Segment SGI321409[] = {{&S47231[35], 14}, {NULL}};
const Segment SGI321410[] = {{&S47231[131], 15}, {NULL}};
const Segment SGI321411[] = {{&S47231[182], 4}, {NULL}};
const Segment SGI321412[] = {{&S47232[30], 4}, {NULL}};
const Segment SGI321413[] = {{&S47232[8], 3}, {NULL}};
const Segment SGI321414[] = {{&S47232[13], 10}, {NULL}};
const Segment SGI321415[] = {{&S47232[43], 7}, {NULL}};
const Segment SGI321416[] = {{&S47232[36], 14}, {NULL}};
const Segment SGI321417[] = {{&S47232[52], 3}, {NULL}};
const Segment SGI321418[] = {{&S47232[67], 4}, {NULL}};
const Segment SGI321419[] = {{&S47233[30], 3}, {NULL}};
const Segment SGI321420[] = {{&S47233[8], 3}, {NULL}};
const Segment SGI321421[] = {{&S47233[13], 10}, {NULL}};
const Segment SGI321422[] = {{&S47233[42], 7}, {NULL}};
const Segment SGI321423[] = {{&S47233[35], 14}, {NULL}};
const Segment SGI321424[] = {{&S47233[138], 15}, {NULL}};
const Segment SGI321425[] = {{&S47233[51], 6}, {NULL}};
const Segment SGI321426[] = {{&S47233[189], 4}, {NULL}};
const Segment SGI321427[] = {{&S47234[30], 4}, {NULL}};
const Segment SGI321428[] = {{&S47234[8], 3}, {NULL}};
const Segment SGI321429[] = {{&S47234[13], 10}, {NULL}};
const Segment SGI321430[] = {{&S47234[43], 7}, {NULL}};
const Segment SGI321431[] = {{&S47234[36], 14}, {NULL}};
const Segment SGI321432[] = {{&S47234[75], 4}, {NULL}};
const Segment SGI321433[] = {{&S47235[30], 3}, {NULL}};
const Segment SGI321434[] = {{&S47235[8], 3}, {NULL}};
const Segment SGI321435[] = {{&S47235[13], 10}, {NULL}};
const Segment SGI321436[] = {{&S47235[42], 7}, {NULL}};
const Segment SGI321437[] = {{&S47235[35], 14}, {NULL}};
const Segment SGI321438[] = {{&S47235[131], 15}, {NULL}};
const Segment SGI321439[] = {{&S47235[162], 4}, {NULL}};
const Segment SGI321440[] = {{&S47236[30], 3}, {NULL}};
const Segment SGI321441[] = {{&S47236[8], 3}, {NULL}};
const Segment SGI321442[] = {{&S47236[13], 10}, {NULL}};
const Segment SGI321443[] = {{&S47236[35], 5}, {NULL}};
const Segment SGI321444[] = {{&S47237[30], 3}, {NULL}};
const Segment SGI321445[] = {{&S47237[8], 3}, {NULL}};
const Segment SGI321446[] = {{&S47237[13], 10}, {NULL}};
const Segment SGI321447[] = {{&S47237[35], 5}, {NULL}};
const Segment SGI321448[] = {{&S47238[30], 3}, {NULL}};
const Segment SGI321449[] = {{&S47238[8], 3}, {NULL}};
const Segment SGI321450[] = {{&S47238[13], 10}, {NULL}};
const Segment SGI321451[] = {{&S47238[35], 5}, {NULL}};
const Segment SGI321452[] = {{&S47238[49], 6}, {NULL}};
const Segment SGI321453[] = {{&S47239[30], 3}, {NULL}};
const Segment SGI321454[] = {{&S47239[8], 3}, {NULL}};
const Segment SGI321455[] = {{&S47239[13], 10}, {NULL}};
const Segment SGI321456[] = {{&S47239[35], 5}, {NULL}};
const Segment SGI321457[] = {{&S47239[47], 3}, {NULL}};
const Segment SGI321458[] = {{&S47240[30], 3}, {NULL}};
const Segment SGI321459[] = {{&S47240[8], 3}, {NULL}};
const Segment SGI321460[] = {{&S47240[13], 10}, {NULL}};
const Segment SGI321461[] = {{&S47240[35], 5}, {NULL}};
const Segment SGI321462[] = {{&S47240[49], 5}, {NULL}};
const Segment SGI321463[] = {{&S47241[30], 3}, {NULL}};
const Segment SGI321464[] = {{&S47241[8], 3}, {NULL}};
const Segment SGI321465[] = {{&S47241[13], 10}, {NULL}};
const Segment SGI321466[] = {{&S47241[35], 5}, {NULL}};
const Segment SGI321467[] = {{&S47241[47], 9}, {NULL}};
const Segment SGI321468[] = {{&S47307[30], 4}, {NULL}};
const Segment SGI321469[] = {{&S47307[8], 3}, {NULL}};
const Segment SGI321470[] = {{&S47307[13], 10}, {NULL}};
const Segment SGI321471[] = {{&S47307[43], 7}, {NULL}};
const Segment SGI321472[] = {{&S47307[36], 14}, {NULL}};
const Segment SGI321473[] = {{&S47307[72], 11}, {NULL}};
const Segment SGI321474[] = {{&S47307[52], 6}, {NULL}};
const Segment SGI321475[] = {{&S47307[72], 16}, {NULL}};
const Segment SGI321476[] = {{&S47309[30], 4}, {NULL}};
const Segment SGI321477[] = {{&S47309[8], 3}, {NULL}};
const Segment SGI321478[] = {{&S47309[13], 10}, {NULL}};
const Segment SGI321479[] = {{&S47309[43], 7}, {NULL}};
const Segment SGI321480[] = {{&S47309[36], 14}, {NULL}};
const Segment SGI321481[] = {{&S47309[65], 11}, {NULL}};
const Segment SGI321482[] = {{&S47309[65], 16}, {NULL}};
const Segment SGI321483[] = {{&S47310[30], 3}, {NULL}};
const Segment SGI321484[] = {{&S47310[8], 3}, {NULL}};
const Segment SGI321485[] = {{&S47310[13], 10}, {NULL}};
const Segment SGI321486[] = {{&S47310[42], 7}, {NULL}};
const Segment SGI321487[] = {{&S47310[35], 14}, {NULL}};
const Segment SGI321488[] = {{&S47310[71], 11}, {NULL}};
const Segment SGI321489[] = {{&S47310[51], 6}, {NULL}};
const Segment SGI321490[] = {{&S47310[71], 16}, {NULL}};
const Segment SGI321491[] = {{&S47311[30], 3}, {NULL}};
const Segment SGI321492[] = {{&S47311[8], 3}, {NULL}};
const Segment SGI321493[] = {{&S47311[13], 10}, {NULL}};
const Segment SGI321494[] = {{&S47311[42], 7}, {NULL}};
const Segment SGI321495[] = {{&S47311[35], 14}, {NULL}};
const Segment SGI321496[] = {{&S47311[151], 11}, {NULL}};
const Segment SGI321497[] = {{&S47311[51], 6}, {NULL}};
const Segment SGI321498[] = {{&S47311[151], 16}, {NULL}};
const Segment SGI321499[] = {{&S81792[7], 4}, {NULL}};
const Segment SGI321500[] = {{&S81792[0], 81}, {NULL}};
const Segment SGI321501[] = {{&S81794[7], 4}, {NULL}};
const Segment SGI321502[] = {{&S81794[0], 81}, {NULL}};
const Segment SGI321503[] = {{&S81796[7], 4}, {NULL}};
const Segment SGI321504[] = {{&S81796[0], 80}, {NULL}};
const Segment SGI321505[] = {{&S81798[7], 4}, {NULL}};
const Segment SGI321506[] = {{&S81798[0], 81}, {NULL}};
const Segment SGI321507[] = {{&S47356[30], 3}, {NULL}};
const Segment SGI321508[] = {{&S47356[8], 3}, {NULL}};
const Segment SGI321509[] = {{&S47356[13], 10}, {NULL}};
const Segment SGI321510[] = {{&S47356[35], 5}, {NULL}};
const Segment SGI321511[] = {{&S63893[3], 5}, {NULL}};
const Segment SGI321512[] = {{&S47357[30], 3}, {NULL}};
const Segment SGI321513[] = {{&S47357[8], 3}, {NULL}};
const Segment SGI321514[] = {{&S47357[13], 10}, {NULL}};
const Segment SGI321515[] = {{&S47357[35], 5}, {NULL}};
const Segment SGI321516[] = {{&S47357[49], 6}, {NULL}};
const Segment SGI321517[] = {{&S47358[30], 3}, {NULL}};
const Segment SGI321518[] = {{&S47358[8], 3}, {NULL}};
const Segment SGI321519[] = {{&S47358[13], 10}, {NULL}};
const Segment SGI321520[] = {{&S47358[35], 5}, {NULL}};
const Segment SGI321521[] = {{&S47358[47], 5}, {NULL}};
const Segment SGI321522[] = {{&S47359[30], 3}, {NULL}};
const Segment SGI321523[] = {{&S47359[8], 3}, {NULL}};
const Segment SGI321524[] = {{&S47359[13], 10}, {NULL}};
const Segment SGI321525[] = {{&S47359[35], 5}, {NULL}};
const Segment SGI321526[] = {{&S47359[49], 9}, {NULL}};
const Segment SGI321527[] = {{&S47360[30], 3}, {NULL}};
const Segment SGI321528[] = {{&S47360[8], 3}, {NULL}};
const Segment SGI321529[] = {{&S47360[13], 10}, {NULL}};
const Segment SGI321530[] = {{&S47360[35], 5}, {NULL}};
const Segment SGI321531[] = {{&S47360[49], 3}, {NULL}};
const Segment SGI321532[] = {{&S47381[8], 3}, {NULL}};
const Segment SGI321533[] = {{&S47382[30], 4}, {NULL}};
const Segment SGI321534[] = {{&S47382[8], 3}, {NULL}};
const Segment SGI321535[] = {{&S47382[13], 10}, {NULL}};
const Segment SGI321536[] = {{&S47383[8], 3}, {NULL}};
const Segment SGI321537[] = {{&S47383[13], 10}, {NULL}};
const Segment SGI321538[] = {{&S47416[30], 3}, {NULL}};
const Segment SGI321539[] = {{&S47416[8], 3}, {NULL}};
const Segment SGI321540[] = {{&S47416[13], 10}, {NULL}};
const Segment SGI321541[] = {{&S47416[42], 7}, {NULL}};
const Segment SGI321542[] = {{&S47416[35], 14}, {NULL}};
const Segment SGI321543[] = {{&S47416[51], 11}, {NULL}};
const Segment SGI321544[] = {{&S47416[51], 16}, {NULL}};
const Segment SGI321545[] = {{&S47418[30], 3}, {NULL}};
const Segment SGI321546[] = {{&S47418[8], 3}, {NULL}};
const Segment SGI321547[] = {{&S47418[13], 10}, {NULL}};
const Segment SGI321548[] = {{&S47418[42], 7}, {NULL}};
const Segment SGI321549[] = {{&S47418[35], 14}, {NULL}};
const Segment SGI321550[] = {{&S47418[193], 11}, {NULL}};
const Segment SGI321551[] = {{&S47418[193], 16}, {NULL}};
const Segment SGI321552[] = {{&S47418[64], 9}, {NULL}};
const Segment SGI321553[] = {{&S47420[30], 3}, {&S47420[138], 3}, {NULL}};
const Segment SGI321554[] = {{&S47420[8], 3}, {&S47420[8], 3}, {NULL}};
const Segment SGI321555[] = {{&S47420[13], 10}, {&S47420[13], 10}, {NULL}};
const Segment SGI321556[] = {{&S47420[42], 7}, {&S47420[42], 7}, {NULL}};
const Segment SGI321557[] = {{&S47420[35], 14}, {&S47420[35], 14}, {NULL}};
const Segment SGI321558[] = {{&S47420[83], 11}, {NULL}};
const Segment SGI321559[] = {{&S47420[83], 16}, {NULL}};
const Segment SGI321560[] = {{&S47420[159], 3}, {NULL}};
const Segment SGI321561[] = {{&S47421[30], 3}, {NULL}};
const Segment SGI321562[] = {{&S47421[8], 3}, {NULL}};
const Segment SGI321563[] = {{&S47421[13], 10}, {NULL}};
const Segment SGI321564[] = {{&S47421[42], 7}, {NULL}};
const Segment SGI321565[] = {{&S47421[35], 14}, {NULL}};
const Segment SGI321566[] = {{&S47421[179], 11}, {NULL}};
const Segment SGI321567[] = {{&S47421[179], 16}, {NULL}};
const Segment SGI321568[] = {{&S47425[30], 3}, {NULL}};
const Segment SGI321569[] = {{&S47425[8], 3}, {NULL}};
const Segment SGI321570[] = {{&S47425[13], 10}, {NULL}};
const Segment SGI321571[] = {{&S47425[42], 7}, {NULL}};
const Segment SGI321572[] = {{&S47425[35], 14}, {NULL}};
const Segment SGI321573[] = {{&S47425[56], 11}, {NULL}};
const Segment SGI321574[] = {{&S47425[56], 16}, {NULL}};
const Segment SGI321575[] = {{&S47425[51], 3}, {NULL}};
const Segment SGI321576[] = {{&S47433[18], 4}, {NULL}};
const Segment SGI321577[] = {{&S47433[49], 3}, {NULL}};
const Segment SGI321578[] = {{&S47434[30], 4}, {NULL}};
const Segment SGI321579[] = {{&S47434[8], 3}, {NULL}};
const Segment SGI321580[] = {{&S47434[13], 10}, {NULL}};
const Segment SGI321581[] = {{&S47434[43], 3}, {NULL}};
const Segment SGI321582[] = {{&S47435[29], 4}, {NULL}};
const Segment SGI321583[] = {{&S47435[8], 3}, {NULL}};
const Segment SGI321584[] = {{&S47435[60], 3}, {NULL}};
const Segment SGI321585[] = {{&S47436[8], 3}, {NULL}};
const Segment SGI321586[] = {{&S47436[13], 10}, {NULL}};
const Segment SGI321587[] = {{&S47436[42], 3}, {NULL}};
const Segment SGI321588[] = {{&S47437[8], 3}, {NULL}};
const Segment SGI321589[] = {{&S47437[13], 10}, {NULL}};
const Segment SGI321590[] = {{&S47437[42], 4}, {NULL}};
const Segment SGI321591[] = {{&S47438[30], 3}, {NULL}};
const Segment SGI321592[] = {{&S47438[8], 3}, {NULL}};
const Segment SGI321593[] = {{&S47438[13], 10}, {NULL}};
const Segment SGI321594[] = {{&S47438[42], 3}, {NULL}};
const Segment SGI321595[] = {{&S47439[30], 4}, {NULL}};
const Segment SGI321596[] = {{&S47439[8], 3}, {NULL}};
const Segment SGI321597[] = {{&S47439[13], 10}, {NULL}};
const Segment SGI321598[] = {{&S47439[43], 3}, {NULL}};
const Segment SGI321599[] = {{&S47440[30], 3}, {NULL}};
const Segment SGI321600[] = {{&S47440[8], 3}, {NULL}};
const Segment SGI321601[] = {{&S47440[13], 10}, {NULL}};
const Segment SGI321602[] = {{&S47441[30], 3}, {NULL}};
const Segment SGI321603[] = {{&S47441[8], 3}, {NULL}};
const Segment SGI321604[] = {{&S47441[13], 10}, {NULL}};
const Segment SGI321605[] = {{&S47441[35], 5}, {NULL}};
const Segment SGI321606[] = {{&S64075[3], 4}, {NULL}};
const Segment SGI321607[] = {{&S64075[3], 11}, {NULL}};
const Segment SGI321608[] = {{&S64076[3], 4}, {NULL}};
const Segment SGI321609[] = {{&S64081[3], 5}, {NULL}};
const Segment SGI321610[] = {{&S64081[3], 14}, {NULL}};
const Segment SGI321611[] = {{&S47515[30], 3}, {NULL}};
const Segment SGI321612[] = {{&S47515[8], 3}, {NULL}};
const Segment SGI321613[] = {{&S47515[13], 10}, {NULL}};
const Segment SGI321614[] = {{&S47515[35], 5}, {NULL}};
const Segment SGI321615[] = {{&S64082[3], 5}, {NULL}};
const Segment SGI321616[] = {{&S64082[3], 12}, {NULL}};
const Segment SGI321617[] = {{&S47516[30], 3}, {NULL}};
const Segment SGI321618[] = {{&S64083[3], 6}, {NULL}};
const Segment SGI321619[] = {{&S47516[8], 3}, {NULL}};
const Segment SGI321620[] = {{&S47516[13], 10}, {NULL}};
const Segment SGI321621[] = {{&S47516[35], 5}, {NULL}};
const Segment SGI321622[] = {{&S47516[49], 5}, {NULL}};
const Segment SGI321623[] = {{&S64084[3], 5}, {NULL}};
const Segment SGI321624[] = {{&S47517[30], 3}, {NULL}};
const Segment SGI321625[] = {{&S47517[8], 3}, {NULL}};
const Segment SGI321626[] = {{&S47517[13], 10}, {NULL}};
const Segment SGI321627[] = {{&S47517[35], 5}, {NULL}};
const Segment SGI321628[] = {{&S47517[47], 6}, {NULL}};
const Segment SGI321629[] = {{&S47518[30], 3}, {NULL}};
const Segment SGI321630[] = {{&S47518[8], 3}, {NULL}};
const Segment SGI321631[] = {{&S47518[13], 10}, {NULL}};
const Segment SGI321632[] = {{&S47518[35], 5}, {NULL}};
const Segment SGI321633[] = {{&S47518[49], 9}, {NULL}};
const Segment SGI321634[] = {{&S47519[30], 3}, {NULL}};
const Segment SGI321635[] = {{&S47519[8], 3}, {NULL}};
const Segment SGI321636[] = {{&S47519[13], 10}, {NULL}};
const Segment SGI321637[] = {{&S47519[35], 5}, {NULL}};
const Segment SGI321638[] = {{&S47519[49], 3}, {NULL}};
const Segment SGI321639[] = {{&S64311[3], 5}, {NULL}};
const Segment SGI321640[] = {{&S64311[3], 12}, {NULL}};
const Segment SGI321641[] = {{&S64311[36], 4}, {NULL}};
const Segment SGI321642[] = {{&S64315[3], 5}, {NULL}};
const Segment SGI321643[] = {{&S64315[29], 4}, {NULL}};
const Segment SGI321644[] = {{&S64316[3], 5}, {NULL}};
const Segment SGI321645[] = {{&S64316[29], 4}, {NULL}};
const Segment SGI321646[] = {{&S64316[212], 13}, {NULL}};
const Segment SGI321647[] = {{&S64317[3], 5}, {NULL}};
const Segment SGI321648[] = {{&S64317[29], 4}, {NULL}};
const Segment SGI321649[] = {{&S64317[186], 13}, {NULL}};
const Segment SGI321650[] = {{&S64318[3], 5}, {NULL}};
const Segment SGI321651[] = {{&S64318[29], 4}, {NULL}};
const Segment SGI321652[] = {{&S64318[190], 13}, {NULL}};
const Segment SGI321653[] = {{&S64343[3], 5}, {NULL}};
const Segment SGI321654[] = {{&S64344[3], 6}, {NULL}};
const Segment SGI321655[] = {{&S64345[3], 6}, {NULL}};
const Segment SGI321656[] = {{&S64346[3], 5}, {NULL}};
const Segment SGI321657[] = {{&S64346[3], 12}, {NULL}};
const Segment SGI321658[] = {{&S47790[30], 3}, {NULL}};
const Segment SGI321659[] = {{&S47790[8], 3}, {NULL}};
const Segment SGI321660[] = {{&S47790[13], 10}, {NULL}};
const Segment SGI321661[] = {{&S47790[55], 7}, {NULL}};
const Segment SGI321662[] = {{&S47790[48], 14}, {NULL}};
const Segment SGI321663[] = {{&S47790[64], 7}, {NULL}};
const Segment SGI321664[] = {{&S47791[30], 3}, {NULL}};
const Segment SGI321665[] = {{&S47791[8], 3}, {NULL}};
const Segment SGI321666[] = {{&S47791[13], 10}, {NULL}};
const Segment SGI321667[] = {{&S47791[55], 7}, {NULL}};
const Segment SGI321668[] = {{&S47791[48], 14}, {NULL}};
const Segment SGI321669[] = {{&S47791[64], 7}, {NULL}};
const Segment SGI321670[] = {{&S47792[30], 3}, {NULL}};
const Segment SGI321671[] = {{&S47792[8], 3}, {NULL}};
const Segment SGI321672[] = {{&S47792[13], 10}, {NULL}};
const Segment SGI321673[] = {{&S47792[55], 7}, {NULL}};
const Segment SGI321674[] = {{&S47792[48], 14}, {NULL}};
const Segment SGI321675[] = {{&S47792[64], 7}, {NULL}};
const Segment SGI321676[] = {{&S47793[30], 3}, {NULL}};
const Segment SGI321677[] = {{&S47793[8], 3}, {NULL}};
const Segment SGI321678[] = {{&S47793[13], 10}, {NULL}};
const Segment SGI321679[] = {{&S47793[55], 7}, {NULL}};
const Segment SGI321680[] = {{&S47793[48], 14}, {NULL}};
const Segment SGI321681[] = {{&S47793[64], 7}, {NULL}};
const Segment SGI321682[] = {{&S47794[30], 3}, {NULL}};
const Segment SGI321683[] = {{&S47794[8], 3}, {NULL}};
const Segment SGI321684[] = {{&S47794[13], 10}, {NULL}};
const Segment SGI321685[] = {{&S47794[55], 7}, {NULL}};
const Segment SGI321686[] = {{&S47794[48], 14}, {NULL}};
const Segment SGI321687[] = {{&S47794[64], 7}, {NULL}};
const Segment SGI321688[] = {{&S47795[30], 3}, {NULL}};
const Segment SGI321689[] = {{&S47795[8], 3}, {NULL}};
const Segment SGI321690[] = {{&S47795[13], 10}, {NULL}};
const Segment SGI321691[] = {{&S47795[55], 7}, {NULL}};
const Segment SGI321692[] = {{&S47795[48], 14}, {NULL}};
const Segment SGI321693[] = {{&S47795[64], 7}, {NULL}};
const Segment SGI321694[] = {{&S47796[30], 3}, {NULL}};
const Segment SGI321695[] = {{&S47796[8], 3}, {NULL}};
const Segment SGI321696[] = {{&S47796[13], 10}, {NULL}};
const Segment SGI321697[] = {{&S47796[55], 7}, {NULL}};
const Segment SGI321698[] = {{&S47796[48], 14}, {NULL}};
const Segment SGI321699[] = {{&S47796[79], 7}, {NULL}};
const Segment SGI321700[] = {{&S47796[64], 13}, {NULL}};
const Segment SGI321701[] = {{&S47797[30], 3}, {NULL}};
const Segment SGI321702[] = {{&S47797[8], 3}, {NULL}};
const Segment SGI321703[] = {{&S47797[13], 10}, {NULL}};
const Segment SGI321704[] = {{&S47797[55], 7}, {NULL}};
const Segment SGI321705[] = {{&S47797[48], 14}, {NULL}};
const Segment SGI321706[] = {{&S47797[64], 7}, {&S47797[64], 7}, {NULL}};
const Segment SGI321707[] = {{&S47798[30], 3}, {NULL}};
const Segment SGI321708[] = {{&S47798[8], 3}, {NULL}};
const Segment SGI321709[] = {{&S47798[13], 10}, {NULL}};
const Segment SGI321710[] = {{&S47798[55], 7}, {NULL}};
const Segment SGI321711[] = {{&S47798[48], 14}, {NULL}};
const Segment SGI321712[] = {{&S47798[64], 7}, {NULL}};
const Segment SGI321713[] = {{&S47798[73], 3}, {NULL}};
const Segment SGI321714[] = {{&S47799[30], 3}, {NULL}};
const Segment SGI321715[] = {{&S47799[8], 3}, {NULL}};
const Segment SGI321716[] = {{&S47799[13], 10}, {NULL}};
const Segment SGI321717[] = {{&S47799[55], 7}, {NULL}};
const Segment SGI321718[] = {{&S47799[48], 14}, {NULL}};
const Segment SGI321719[] = {{&S47799[64], 7}, {NULL}};
const Segment SGI321720[] = {{&S47799[73], 13}, {NULL}};
const Segment SGI321721[] = {{&S47800[30], 3}, {NULL}};
const Segment SGI321722[] = {{&S47800[8], 3}, {NULL}};
const Segment SGI321723[] = {{&S47800[13], 10}, {NULL}};
const Segment SGI321724[] = {{&S47800[55], 7}, {NULL}};
const Segment SGI321725[] = {{&S47800[48], 14}, {NULL}};
const Segment SGI321726[] = {{&S47800[64], 7}, {&S47800[64], 7}, {NULL}};
const Segment SGI321727[] = {{&S47801[30], 3}, {NULL}};
const Segment SGI321728[] = {{&S47801[8], 3}, {NULL}};
const Segment SGI321729[] = {{&S47801[13], 10}, {NULL}};
const Segment SGI321730[] = {{&S47801[55], 7}, {NULL}};
const Segment SGI321731[] = {{&S47801[48], 14}, {NULL}};
const Segment SGI321732[] = {{&S47801[69], 7}, {NULL}};
const Segment SGI321733[] = {{&S47801[64], 3}, {NULL}};
const Segment SGI321734[] = {{&S47802[30], 3}, {NULL}};
const Segment SGI321735[] = {{&S47802[8], 3}, {NULL}};
const Segment SGI321736[] = {{&S47802[13], 10}, {NULL}};
const Segment SGI321737[] = {{&S47802[55], 7}, {NULL}};
const Segment SGI321738[] = {{&S47802[48], 14}, {NULL}};
const Segment SGI321739[] = {{&S47802[69], 7}, {NULL}};
const Segment SGI321740[] = {{&S47802[64], 3}, {NULL}};
const Segment SGI321741[] = {{&S47803[30], 3}, {NULL}};
const Segment SGI321742[] = {{&S47803[8], 3}, {NULL}};
const Segment SGI321743[] = {{&S47803[13], 10}, {NULL}};
const Segment SGI321744[] = {{&S47803[55], 7}, {NULL}};
const Segment SGI321745[] = {{&S47803[48], 14}, {NULL}};
const Segment SGI321746[] = {{&S47803[64], 7}, {&S47803[64], 7}, {NULL}};
const Segment SGI321747[] = {{&S47804[30], 3}, {NULL}};
const Segment SGI321748[] = {{&S47804[8], 3}, {NULL}};
const Segment SGI321749[] = {{&S47804[13], 10}, {NULL}};
const Segment SGI321750[] = {{&S47804[55], 7}, {NULL}};
const Segment SGI321751[] = {{&S47804[48], 14}, {NULL}};
const Segment SGI321752[] = {{&S47804[79], 7}, {NULL}};
const Segment SGI321753[] = {{&S47804[64], 13}, {NULL}};
const Segment SGI321754[] = {{&S47805[30], 3}, {NULL}};
const Segment SGI321755[] = {{&S47805[8], 3}, {NULL}};
const Segment SGI321756[] = {{&S47805[13], 10}, {NULL}};
const Segment SGI321757[] = {{&S47805[42], 8}, {NULL}};
const Segment SGI321758[] = {{&S47805[52], 6}, {NULL}};
const Segment SGI321759[] = {{&S47805[99], 7}, {NULL}};
const Segment SGI321760[] = {{&S47806[30], 3}, {NULL}};
const Segment SGI321761[] = {{&S47806[8], 3}, {NULL}};
const Segment SGI321762[] = {{&S47806[13], 10}, {NULL}};
const Segment SGI321763[] = {{&S47806[55], 7}, {NULL}};
const Segment SGI321764[] = {{&S47806[48], 14}, {NULL}};
const Segment SGI321765[] = {{&S47806[64], 7}, {NULL}};
const Segment SGI321766[] = {{&S47806[73], 13}, {NULL}};
const Segment SGI321767[] = {{&S47807[30], 3}, {NULL}};
const Segment SGI321768[] = {{&S47807[8], 3}, {NULL}};
const Segment SGI321769[] = {{&S47807[13], 10}, {NULL}};
const Segment SGI321770[] = {{&S47807[55], 7}, {NULL}};
const Segment SGI321771[] = {{&S47807[48], 14}, {NULL}};
const Segment SGI321772[] = {{&S47807[64], 7}, {NULL}};
const Segment SGI321773[] = {{&S47807[73], 3}, {NULL}};
const Segment SGI321774[] = {{&S47808[30], 3}, {NULL}};
const Segment SGI321775[] = {{&S55180[4], 5}, {NULL}};
const Segment SGI321776[] = {{&S47808[8], 3}, {NULL}};
const Segment SGI321777[] = {{&S47808[13], 10}, {NULL}};
const Segment SGI321778[] = {{&S55180[25], 3}, {NULL}};
const Segment SGI321779[] = {{&S47808[55], 7}, {NULL}};
const Segment SGI321780[] = {{&S47808[48], 14}, {NULL}};
const Segment SGI321781[] = {{&S47808[64], 7}, {&S47808[64], 7}, {NULL}};
const Segment SGI321782[] = {{&S47809[30], 3}, {NULL}};
const Segment SGI321783[] = {{&S47809[8], 3}, {NULL}};
const Segment SGI321784[] = {{&S47809[13], 10}, {NULL}};
const Segment SGI321785[] = {{&S47809[42], 8}, {NULL}};
const Segment SGI321786[] = {{&S47809[52], 6}, {NULL}};
const Segment SGI321787[] = {{&S47809[99], 7}, {NULL}};
const Segment SGI321788[] = {{&S64496[3], 5}, {NULL}};
const Segment SGI321789[] = {{&S64496[3], 10}, {NULL}};
const Segment SGI321790[] = {{&S64496[29], 4}, {NULL}};
const Segment SGI321791[] = {{&S64497[3], 5}, {NULL}};
const Segment SGI321792[] = {{&S64497[3], 10}, {NULL}};
const Segment SGI321793[] = {{&S64497[29], 4}, {NULL}};
const Segment SGI321794[] = {{&S64498[3], 5}, {NULL}};
const Segment SGI321795[] = {{&S64498[3], 10}, {NULL}};
const Segment SGI321796[] = {{&S64498[29], 4}, {NULL}};
const Segment SGI321797[] = {{&S64499[3], 6}, {NULL}};
const Segment SGI321798[] = {{&S64499[25], 4}, {NULL}};
const Segment SGI321799[] = {{&S64500[3], 7}, {NULL}};
const Segment SGI321800[] = {{&S64500[26], 4}, {NULL}};
const Segment SGI321801[] = {{&S64501[3], 5}, {NULL}};
const Segment SGI321802[] = {{&S64501[3], 12}, {NULL}};
const Segment SGI321803[] = {{&S64501[36], 4}, {NULL}};
const Segment SGI321804[] = {{&S64502[3], 5}, {NULL}};
const Segment SGI321805[] = {{&S64502[3], 10}, {NULL}};
const Segment SGI321806[] = {{&S64502[29], 4}, {NULL}};
const Segment SGI321807[] = {{&S64503[3], 5}, {NULL}};
const Segment SGI321808[] = {{&S76107[3], 5}, {NULL}};
const Segment SGI321809[] = {{&S61826[0], 53}, {NULL}};
const Segment SGI321810[] = {{&S61827[0], 57}, {NULL}};
const Segment SGI321811[] = {{&S61828[0], 55}, {NULL}};
const Segment SGI321812[] = {{&S61829[0], 58}, {NULL}};
const Segment SGI321813[] = {{&S61830[0], 55}, {NULL}};
const Segment SGI321814[] = {{&S61831[0], 55}, {NULL}};
const Segment SGI321815[] = {{&S61832[0], 54}, {NULL}};
const Segment SGI321816[] = {{&S61833[0], 54}, {NULL}};
const Segment SGI321817[] = {{&S61838[0], 60}, {NULL}};
const Segment SGI321818[] = {{&S64645[3], 7}, {NULL}};
const Segment SGI321819[] = {{&S64646[3], 5}, {NULL}};
const Segment SGI321820[] = {{&S61847[0], 59}, {NULL}};
const Segment SGI321821[] = {{&S64647[3], 12}, {NULL}};
const Segment SGI321822[] = {{&S61848[0], 57}, {NULL}};
const Segment SGI321823[] = {{&S48111[30], 3}, {NULL}};
const Segment SGI321824[] = {{&S48111[8], 3}, {NULL}};
const Segment SGI321825[] = {{&S48111[13], 10}, {NULL}};
const Segment SGI321826[] = {{&S48111[42], 7}, {NULL}};
const Segment SGI321827[] = {{&S48111[35], 14}, {NULL}};
const Segment SGI321828[] = {{&S48111[56], 11}, {NULL}};
const Segment SGI321829[] = {{&S48111[56], 16}, {NULL}};
const Segment SGI321830[] = {{&S48111[51], 3}, {NULL}};
const Segment SGI321831[] = {{&S48113[30], 3}, {NULL}};
const Segment SGI321832[] = {{&S48113[8], 3}, {NULL}};
const Segment SGI321833[] = {{&S48113[13], 10}, {NULL}};
const Segment SGI321834[] = {{&S48113[42], 7}, {NULL}};
const Segment SGI321835[] = {{&S48113[35], 14}, {NULL}};
const Segment SGI321836[] = {{&S48113[51], 11}, {NULL}};
const Segment SGI321837[] = {{&S48113[51], 16}, {NULL}};
const Segment SGI321838[] = {{&S48114[30], 3}, {NULL}};
const Segment SGI321839[] = {{&S48114[8], 3}, {NULL}};
const Segment SGI321840[] = {{&S48114[13], 10}, {NULL}};
const Segment SGI321841[] = {{&S48114[42], 7}, {NULL}};
const Segment SGI321842[] = {{&S48114[35], 14}, {NULL}};
const Segment SGI321843[] = {{&S48114[51], 11}, {NULL}};
const Segment SGI321844[] = {{&S48114[51], 16}, {NULL}};
const Segment SGI321845[] = {{&S48116[30], 3}, {NULL}};
const Segment SGI321846[] = {{&S48116[8], 3}, {NULL}};
const Segment SGI321847[] = {{&S48116[13], 10}, {NULL}};
const Segment SGI321848[] = {{&S48116[42], 7}, {NULL}};
const Segment SGI321849[] = {{&S48116[35], 14}, {NULL}};
const Segment SGI321850[] = {{&S48116[64], 11}, {NULL}};
const Segment SGI321851[] = {{&S48116[64], 16}, {NULL}};
const Segment SGI321852[] = {{&S48118[30], 3}, {NULL}};
const Segment SGI321853[] = {{&S48118[8], 3}, {NULL}};
const Segment SGI321854[] = {{&S48118[13], 10}, {NULL}};
const Segment SGI321855[] = {{&S48118[42], 7}, {NULL}};
const Segment SGI321856[] = {{&S48118[35], 14}, {NULL}};
const Segment SGI321857[] = {{&S48118[51], 11}, {NULL}};
const Segment SGI321858[] = {{&S48118[51], 16}, {NULL}};
const Segment SGI321859[] = {{&S48122[30], 3}, {NULL}};
const Segment SGI321860[] = {{&S48122[8], 3}, {NULL}};
const Segment SGI321861[] = {{&S48122[13], 10}, {NULL}};
const Segment SGI321862[] = {{&S48122[42], 7}, {NULL}};
const Segment SGI321863[] = {{&S48122[35], 14}, {NULL}};
const Segment SGI321864[] = {{&S48122[64], 11}, {NULL}};
const Segment SGI321865[] = {{&S48122[128], 15}, {NULL}};
const Segment SGI321866[] = {{&S48122[64], 16}, {NULL}};
const Segment SGI321867[] = {{&S48124[30], 3}, {NULL}};
const Segment SGI321868[] = {{&S48124[8], 3}, {NULL}};
const Segment SGI321869[] = {{&S48124[13], 10}, {NULL}};
const Segment SGI321870[] = {{&S48124[42], 7}, {NULL}};
const Segment SGI321871[] = {{&S48124[35], 14}, {NULL}};
const Segment SGI321872[] = {{&S48124[64], 11}, {NULL}};
const Segment SGI321873[] = {{&S48124[64], 16}, {NULL}};
const Segment SGI321874[] = {{&S48215[30], 3}, {NULL}};
const Segment SGI321875[] = {{&S48215[8], 3}, {NULL}};
const Segment SGI321876[] = {{&S48215[13], 10}, {NULL}};
const Segment SGI321877[] = {{&S48215[36], 5}, {NULL}};
const Segment SGI321878[] = {{&S48216[30], 3}, {NULL}};
const Segment SGI321879[] = {{&S48216[8], 3}, {NULL}};
const Segment SGI321880[] = {{&S48216[13], 10}, {NULL}};
const Segment SGI321881[] = {{&S48216[43], 3}, {NULL}};
const Segment SGI321882[] = {{&S48217[30], 4}, {NULL}};
const Segment SGI321883[] = {{&S48217[8], 3}, {NULL}};
const Segment SGI321884[] = {{&S48217[13], 10}, {NULL}};
const Segment SGI321885[] = {{&S48217[57], 3}, {NULL}};
const Segment SGI321886[] = {{&S48218[30], 4}, {NULL}};
const Segment SGI321887[] = {{&S48218[8], 3}, {NULL}};
const Segment SGI321888[] = {{&S48218[13], 10}, {NULL}};
const Segment SGI321889[] = {{&S48218[43], 3}, {NULL}};
const Segment SGI321890[] = {{&S48219[30], 3}, {NULL}};
const Segment SGI321891[] = {{&S48219[8], 3}, {NULL}};
const Segment SGI321892[] = {{&S48219[13], 10}, {NULL}};
const Segment SGI321893[] = {{&S48220[30], 3}, {NULL}};
const Segment SGI321894[] = {{&S48220[8], 3}, {NULL}};
const Segment SGI321895[] = {{&S48220[13], 10}, {NULL}};
const Segment SGI321896[] = {{&S48220[40], 6}, {NULL}};
const Segment SGI321897[] = {{&S48221[30], 3}, {NULL}};
const Segment SGI321898[] = {{&S48221[8], 3}, {NULL}};
const Segment SGI321899[] = {{&S48221[13], 10}, {NULL}};
const Segment SGI321900[] = {{&S48221[40], 5}, {NULL}};
const Segment SGI321901[] = {{&S48222[30], 3}, {NULL}};
const Segment SGI321902[] = {{&S48222[8], 3}, {NULL}};
const Segment SGI321903[] = {{&S48222[13], 10}, {NULL}};
const Segment SGI321904[] = {{&S48222[40], 3}, {NULL}};
const Segment SGI321905[] = {{&S48276[30], 3}, {NULL}};
const Segment SGI321906[] = {{&S48276[8], 3}, {NULL}};
const Segment SGI321907[] = {{&S48276[13], 10}, {NULL}};
const Segment SGI321908[] = {{&S48276[42], 7}, {NULL}};
const Segment SGI321909[] = {{&S48276[35], 14}, {NULL}};
const Segment SGI321910[] = {{&S48276[91], 15}, {NULL}};
const Segment SGI321911[] = {{&S48276[162], 7}, {NULL}};
const Segment SGI321912[] = {{&S48276[79], 4}, {NULL}};
const Segment SGI321913[] = {{&S48277[30], 3}, {&S48277[46], 3}, {NULL}};
const Segment SGI321914[] = {{&S48277[8], 3}, {&S48277[8], 3}, {NULL}};
const Segment SGI321915[] = {{&S48277[13], 10}, {&S48277[13], 10}, {NULL}};
const Segment SGI321916[] = {{&S48277[42], 7}, {&S48277[93], 7}, {NULL}};
const Segment SGI321917[] = {{&S48277[35], 14}, {&S48277[86], 14}, {NULL}};
const Segment SGI321918[] = {{&S48277[136], 15}, {NULL}};
const Segment SGI321919[] = {{&S48277[209], 7}, {NULL}};
const Segment SGI321920[] = {{&S48277[124], 4}, {NULL}};
const Segment SGI321921[] = {{&S48277[102], 3}, {NULL}};
const Segment SGI321922[] = {{&S48278[30], 3}, {NULL}};
const Segment SGI321923[] = {{&S48278[8], 3}, {NULL}};
const Segment SGI321924[] = {{&S48278[13], 10}, {NULL}};
const Segment SGI321925[] = {{&S48278[42], 7}, {NULL}};
const Segment SGI321926[] = {{&S48278[35], 14}, {NULL}};
const Segment SGI321927[] = {{&S48278[141], 7}, {NULL}};
const Segment SGI321928[] = {{&S48278[79], 4}, {NULL}};
const Segment SGI321929[] = {{&S48279[30], 3}, {NULL}};
const Segment SGI321930[] = {{&S48279[8], 3}, {NULL}};
const Segment SGI321931[] = {{&S48279[13], 10}, {NULL}};
const Segment SGI321932[] = {{&S48279[42], 7}, {NULL}};
const Segment SGI321933[] = {{&S48279[35], 14}, {NULL}};
const Segment SGI321934[] = {{&S48279[91], 15}, {NULL}};
const Segment SGI321935[] = {{&S48279[152], 7}, {NULL}};
const Segment SGI321936[] = {{&S48279[79], 4}, {NULL}};
const Segment SGI321937[] = {{&S48280[30], 3}, {NULL}};
const Segment SGI321938[] = {{&S48280[8], 3}, {NULL}};
const Segment SGI321939[] = {{&S48280[13], 10}, {NULL}};
const Segment SGI321940[] = {{&S48280[42], 7}, {NULL}};
const Segment SGI321941[] = {{&S48280[35], 14}, {NULL}};
const Segment SGI321942[] = {{&S48280[116], 15}, {NULL}};
const Segment SGI321943[] = {{&S48280[177], 7}, {NULL}};
const Segment SGI321944[] = {{&S48280[97], 4}, {NULL}};
const Segment SGI321945[] = {{&S48280[64], 5}, {NULL}};
const Segment SGI321946[] = {{&S48281[30], 3}, {NULL}};
const Segment SGI321947[] = {{&S48281[8], 3}, {NULL}};
const Segment SGI321948[] = {{&S48281[13], 10}, {NULL}};
const Segment SGI321949[] = {{&S48281[42], 7}, {NULL}};
const Segment SGI321950[] = {{&S48281[35], 14}, {NULL}};
const Segment SGI321951[] = {{&S48281[79], 4}, {NULL}};
const Segment SGI321952[] = {{&S48281[58], 7}, {NULL}};
const Segment SGI321953[] = {{&S48281[65], 4}, {NULL}};
const Segment SGI321954[] = {{&S48282[30], 3}, {NULL}};
const Segment SGI321955[] = {{&S48282[8], 3}, {NULL}};
const Segment SGI321956[] = {{&S48282[13], 10}, {NULL}};
const Segment SGI321957[] = {{&S48282[42], 7}, {NULL}};
const Segment SGI321958[] = {{&S48282[35], 14}, {NULL}};
const Segment SGI321959[] = {{&S48282[75], 7}, {NULL}};
const Segment SGI321960[] = {{&S48282[82], 4}, {NULL}};
const Segment SGI321961[] = {{&S48282[51], 3}, {NULL}};
const Segment SGI321962[] = {{&S48283[30], 3}, {NULL}};
const Segment SGI321963[] = {{&S48283[8], 3}, {NULL}};
const Segment SGI321964[] = {{&S48283[13], 10}, {NULL}};
const Segment SGI321965[] = {{&S48283[42], 7}, {NULL}};
const Segment SGI321966[] = {{&S48283[35], 14}, {NULL}};
const Segment SGI321967[] = {{&S48283[105], 15}, {NULL}};
const Segment SGI321968[] = {{&S48283[51], 6}, {NULL}};
const Segment SGI321969[] = {{&S48283[165], 7}, {NULL}};
const Segment SGI321970[] = {{&S48283[73], 4}, {NULL}};
const Segment SGI321971[] = {{&S48284[30], 3}, {NULL}};
const Segment SGI321972[] = {{&S48284[8], 3}, {NULL}};
const Segment SGI321973[] = {{&S48284[13], 10}, {NULL}};
const Segment SGI321974[] = {{&S48284[42], 7}, {NULL}};
const Segment SGI321975[] = {{&S48284[35], 14}, {NULL}};
const Segment SGI321976[] = {{&S48284[78], 15}, {NULL}};
const Segment SGI321977[] = {{&S48284[145], 7}, {NULL}};
const Segment SGI321978[] = {{&S48284[66], 4}, {NULL}};
const Segment SGI321979[] = {{&S48285[30], 3}, {NULL}};
const Segment SGI321980[] = {{&S48285[8], 3}, {NULL}};
const Segment SGI321981[] = {{&S48285[13], 10}, {NULL}};
const Segment SGI321982[] = {{&S48285[42], 7}, {NULL}};
const Segment SGI321983[] = {{&S48285[35], 14}, {NULL}};
const Segment SGI321984[] = {{&S48285[98], 15}, {NULL}};
const Segment SGI321985[] = {{&S48285[51], 6}, {NULL}};
const Segment SGI321986[] = {{&S48285[169], 7}, {NULL}};
const Segment SGI321987[] = {{&S48285[86], 4}, {NULL}};
const Segment SGI321988[] = {{&S48286[30], 3}, {NULL}};
const Segment SGI321989[] = {{&S48286[8], 3}, {NULL}};
const Segment SGI321990[] = {{&S48286[13], 10}, {NULL}};
const Segment SGI321991[] = {{&S48286[42], 7}, {NULL}};
const Segment SGI321992[] = {{&S48286[35], 14}, {NULL}};
const Segment SGI321993[] = {{&S48286[126], 7}, {NULL}};
const Segment SGI321994[] = {{&S48286[74], 4}, {NULL}};
const Segment SGI321995[] = {{&S65019[3], 12}, {NULL}};
const Segment SGI321996[] = {{&S65020[3], 5}, {NULL}};
const Segment SGI321997[] = {{&S65206[3], 6}, {NULL}};
const Segment SGI321998[] = {{&S65207[3], 4}, {NULL}};
const Segment SGI321999[] = {{&S65207[3], 11}, {NULL}};
const Segment SGI322000[] = {{&S65208[3], 4}, {NULL}};
const Segment SGI322001[] = {{&S48745[30], 3}, {NULL}};
const Segment SGI322002[] = {{&S48745[8], 3}, {NULL}};
const Segment SGI322003[] = {{&S48745[13], 10}, {NULL}};
const Segment SGI322004[] = {{&S48745[55], 7}, {NULL}};
const Segment SGI322005[] = {{&S48745[48], 14}, {NULL}};
const Segment SGI322006[] = {{&S48745[64], 13}, {NULL}};
const Segment SGI322007[] = {{&S48746[30], 3}, {NULL}};
const Segment SGI322008[] = {{&S48746[8], 3}, {NULL}};
const Segment SGI322009[] = {{&S48746[13], 10}, {NULL}};
const Segment SGI322010[] = {{&S48746[55], 7}, {NULL}};
const Segment SGI322011[] = {{&S48746[48], 14}, {NULL}};
const Segment SGI322012[] = {{&S48746[64], 13}, {NULL}};
const Segment SGI322013[] = {{&S48747[30], 3}, {NULL}};
const Segment SGI322014[] = {{&S48747[8], 3}, {NULL}};
const Segment SGI322015[] = {{&S48747[13], 10}, {NULL}};
const Segment SGI322016[] = {{&S48747[55], 7}, {NULL}};
const Segment SGI322017[] = {{&S48747[48], 14}, {NULL}};
const Segment SGI322018[] = {{&S48747[64], 13}, {NULL}};
const Segment SGI322019[] = {{&S48748[30], 3}, {NULL}};
const Segment SGI322020[] = {{&S48748[8], 3}, {NULL}};
const Segment SGI322021[] = {{&S48748[13], 10}, {NULL}};
const Segment SGI322022[] = {{&S48748[55], 7}, {NULL}};
const Segment SGI322023[] = {{&S48748[48], 14}, {NULL}};
const Segment SGI322024[] = {{&S48748[64], 13}, {NULL}};
const Segment SGI322025[] = {{&S48749[30], 3}, {NULL}};
const Segment SGI322026[] = {{&S48749[8], 3}, {NULL}};
const Segment SGI322027[] = {{&S48749[13], 10}, {NULL}};
const Segment SGI322028[] = {{&S48749[55], 7}, {NULL}};
const Segment SGI322029[] = {{&S48749[48], 14}, {NULL}};
const Segment SGI322030[] = {{&S48749[64], 13}, {&S48749[64], 13}, {NULL}};
const Segment SGI322031[] = {{&S48750[30], 3}, {NULL}};
const Segment SGI322032[] = {{&S48750[8], 3}, {NULL}};
const Segment SGI322033[] = {{&S48750[13], 10}, {NULL}};
const Segment SGI322034[] = {{&S48750[55], 7}, {NULL}};
const Segment SGI322035[] = {{&S48750[48], 14}, {NULL}};
const Segment SGI322036[] = {{&S48750[64], 13}, {&S48750[64], 13}, {NULL}};
const Segment SGI322037[] = {{&S48751[30], 3}, {NULL}};
const Segment SGI322038[] = {{&S48751[8], 3}, {NULL}};
const Segment SGI322039[] = {{&S48751[13], 10}, {NULL}};
const Segment SGI322040[] = {{&S48751[55], 7}, {NULL}};
const Segment SGI322041[] = {{&S48751[48], 14}, {NULL}};
const Segment SGI322042[] = {{&S48751[64], 13}, {NULL}};
const Segment SGI322043[] = {{&S48751[79], 3}, {NULL}};
const Segment SGI322044[] = {{&S48752[30], 3}, {NULL}};
const Segment SGI322045[] = {{&S48752[8], 3}, {NULL}};
const Segment SGI322046[] = {{&S48752[13], 10}, {NULL}};
const Segment SGI322047[] = {{&S48752[55], 7}, {NULL}};
const Segment SGI322048[] = {{&S48752[48], 14}, {NULL}};
const Segment SGI322049[] = {{&S48752[69], 13}, {NULL}};
const Segment SGI322050[] = {{&S48752[64], 3}, {NULL}};
const Segment SGI322051[] = {{&S48753[30], 3}, {NULL}};
const Segment SGI322052[] = {{&S48753[8], 3}, {NULL}};
const Segment SGI322053[] = {{&S48753[13], 10}, {NULL}};
const Segment SGI322054[] = {{&S48753[55], 7}, {NULL}};
const Segment SGI322055[] = {{&S48753[48], 14}, {NULL}};
const Segment SGI322056[] = {{&S48753[64], 13}, {NULL}};
const Segment SGI322057[] = {{&S48753[79], 3}, {NULL}};
const Segment SGI322058[] = {{&S48754[30], 3}, {NULL}};
const Segment SGI322059[] = {{&S48754[8], 3}, {NULL}};
const Segment SGI322060[] = {{&S48754[13], 10}, {NULL}};
const Segment SGI322061[] = {{&S48754[55], 7}, {NULL}};
const Segment SGI322062[] = {{&S48754[48], 14}, {NULL}};
const Segment SGI322063[] = {{&S48754[64], 13}, {&S48754[64], 13}, {NULL}};
const Segment SGI322064[] = {{&S48755[30], 3}, {NULL}};
const Segment SGI322065[] = {{&S48755[8], 3}, {NULL}};
const Segment SGI322066[] = {{&S48755[13], 10}, {NULL}};
const Segment SGI322067[] = {{&S48755[55], 7}, {NULL}};
const Segment SGI322068[] = {{&S48755[48], 14}, {NULL}};
const Segment SGI322069[] = {{&S48755[64], 13}, {&S48755[64], 13}, {NULL}};
const Segment SGI322070[] = {{&S48756[30], 3}, {NULL}};
const Segment SGI322071[] = {{&S48756[8], 3}, {NULL}};
const Segment SGI322072[] = {{&S48756[13], 10}, {NULL}};
const Segment SGI322073[] = {{&S48756[55], 7}, {NULL}};
const Segment SGI322074[] = {{&S48756[48], 14}, {NULL}};
const Segment SGI322075[] = {{&S48756[69], 13}, {NULL}};
const Segment SGI322076[] = {{&S48756[64], 3}, {NULL}};
const Segment SGI322077[] = {{&S49176[30], 3}, {NULL}};
const Segment SGI322078[] = {{&S49176[8], 3}, {NULL}};
const Segment SGI322079[] = {{&S49176[13], 10}, {NULL}};
const Segment SGI322080[] = {{&S49176[42], 5}, {NULL}};
const Segment SGI322081[] = {{&S49177[30], 3}, {NULL}};
const Segment SGI322082[] = {{&S49177[8], 3}, {NULL}};
const Segment SGI322083[] = {{&S49177[13], 10}, {NULL}};
const Segment SGI322084[] = {{&S49177[42], 5}, {NULL}};
const Segment SGI322085[] = {{&S77261[3], 5}, {NULL}};
const Segment SGI322086[] = {{&S77261[29], 4}, {NULL}};
const Segment SGI322087[] = {{&S77262[3], 5}, {NULL}};
const Segment SGI322088[] = {{&S77262[3], 14}, {NULL}};
const Segment SGI322089[] = {{&S77262[38], 4}, {NULL}};
const Segment SGI322090[] = {{&S49737[30], 3}, {NULL}};
const Segment SGI322091[] = {{&S49737[8], 3}, {NULL}};
const Segment SGI322092[] = {{&S49737[13], 10}, {NULL}};
const Segment SGI322093[] = {{&S49737[42], 7}, {NULL}};
const Segment SGI322094[] = {{&S49737[35], 14}, {NULL}};
const Segment SGI322095[] = {{&S49737[64], 11}, {NULL}};
const Segment SGI322096[] = {{&S49737[64], 15}, {NULL}};
const Segment SGI322097[] = {{&S49738[30], 3}, {NULL}};
const Segment SGI322098[] = {{&S49738[8], 3}, {NULL}};
const Segment SGI322099[] = {{&S49738[13], 10}, {NULL}};
const Segment SGI322100[] = {{&S49738[42], 7}, {NULL}};
const Segment SGI322101[] = {{&S49738[35], 14}, {NULL}};
const Segment SGI322102[] = {{&S49738[203], 11}, {NULL}};
const Segment SGI322103[] = {{&S49738[131], 15}, {NULL}};
const Segment SGI322104[] = {{&S49738[203], 15}, {NULL}};
const Segment SGI322105[] = {{&S49739[30], 3}, {NULL}};
const Segment SGI322106[] = {{&S49739[8], 3}, {NULL}};
const Segment SGI322107[] = {{&S49739[13], 10}, {NULL}};
const Segment SGI322108[] = {{&S49739[42], 7}, {NULL}};
const Segment SGI322109[] = {{&S49739[35], 14}, {NULL}};
const Segment SGI322110[] = {{&S49739[71], 11}, {NULL}};
const Segment SGI322111[] = {{&S49739[51], 6}, {NULL}};
const Segment SGI322112[] = {{&S49739[71], 15}, {NULL}};
const Segment SGI322113[] = {{&S49740[30], 3}, {NULL}};
const Segment SGI322114[] = {{&S49740[8], 3}, {NULL}};
const Segment SGI322115[] = {{&S49740[13], 10}, {NULL}};
const Segment SGI322116[] = {{&S49740[42], 7}, {NULL}};
const Segment SGI322117[] = {{&S49740[35], 14}, {NULL}};
const Segment SGI322118[] = {{&S49740[218], 11}, {NULL}};
const Segment SGI322119[] = {{&S49740[131], 15}, {NULL}};
const Segment SGI322120[] = {{&S49740[218], 15}, {NULL}};
const Segment SGI322121[] = {{&S49740[203], 9}, {NULL}};
const Segment SGI322122[] = {{&S49741[30], 3}, {NULL}};
const Segment SGI322123[] = {{&S49741[8], 3}, {NULL}};
const Segment SGI322124[] = {{&S49741[13], 10}, {NULL}};
const Segment SGI322125[] = {{&S49741[42], 7}, {NULL}};
const Segment SGI322126[] = {{&S49741[35], 14}, {NULL}};
const Segment SGI322127[] = {{&S49741[223], 11}, {NULL}};
const Segment SGI322128[] = {{&S49741[138], 15}, {NULL}};
const Segment SGI322129[] = {{&S49741[51], 6}, {NULL}};
const Segment SGI322130[] = {{&S49741[223], 15}, {NULL}};
const Segment SGI322131[] = {{&S49742[30], 3}, {NULL}};
const Segment SGI322132[] = {{&S49742[8], 3}, {NULL}};
const Segment SGI322133[] = {{&S49742[13], 10}, {NULL}};
const Segment SGI322134[] = {{&S49742[42], 7}, {NULL}};
const Segment SGI322135[] = {{&S49742[35], 14}, {NULL}};
const Segment SGI322136[] = {{&S49742[151], 11}, {NULL}};
const Segment SGI322137[] = {{&S49742[151], 15}, {NULL}};
const Segment SGI322138[] = {{&S49743[30], 4}, {NULL}};
const Segment SGI322139[] = {{&S49743[8], 3}, {NULL}};
const Segment SGI322140[] = {{&S49743[13], 10}, {NULL}};
const Segment SGI322141[] = {{&S49743[43], 7}, {NULL}};
const Segment SGI322142[] = {{&S49743[36], 14}, {NULL}};
const Segment SGI322143[] = {{&S49743[65], 11}, {NULL}};
const Segment SGI322144[] = {{&S49743[65], 15}, {NULL}};
const Segment SGI322145[] = {{&S49744[30], 4}, {NULL}};
const Segment SGI322146[] = {{&S49744[8], 3}, {NULL}};
const Segment SGI322147[] = {{&S49744[13], 10}, {NULL}};
const Segment SGI322148[] = {{&S49744[43], 7}, {NULL}};
const Segment SGI322149[] = {{&S49744[36], 14}, {NULL}};
const Segment SGI322150[] = {{&S49744[72], 11}, {NULL}};
const Segment SGI322151[] = {{&S49744[52], 6}, {NULL}};
const Segment SGI322152[] = {{&S49744[72], 15}, {NULL}};
const Segment SGI322153[] = {{&S77863[3], 4}, {NULL}};
const Segment SGI322154[] = {{&S63425[0], 32}, {NULL}};
const Segment SGI322155[] = {{&S63426[0], 33}, {NULL}};
const Segment SGI322156[] = {{&S49848[30], 3}, {NULL}};
const Segment SGI322157[] = {{&S49848[8], 3}, {NULL}};
const Segment SGI322158[] = {{&S78065[3], 5}, {NULL}};
const Segment SGI322159[] = {{&S49848[13], 10}, {NULL}};
const Segment SGI322160[] = {{&S49848[42], 7}, {NULL}};
const Segment SGI322161[] = {{&S49848[35], 14}, {NULL}};
const Segment SGI322162[] = {{&S49848[51], 6}, {NULL}};
const Segment SGI322163[] = {{&S49848[71], 15}, {NULL}};
const Segment SGI322164[] = {{&S49849[30], 3}, {NULL}};
const Segment SGI322165[] = {{&S49849[8], 3}, {NULL}};
const Segment SGI322166[] = {{&S49849[13], 10}, {NULL}};
const Segment SGI322167[] = {{&S49849[42], 7}, {NULL}};
const Segment SGI322168[] = {{&S49849[35], 14}, {NULL}};
const Segment SGI322169[] = {{&S49849[64], 15}, {NULL}};
const Segment SGI322170[] = {{&S49850[30], 3}, {NULL}};
const Segment SGI322171[] = {{&S49850[8], 3}, {NULL}};
const Segment SGI322172[] = {{&S49850[13], 10}, {NULL}};
const Segment SGI322173[] = {{&S49850[42], 7}, {NULL}};
const Segment SGI322174[] = {{&S49850[35], 14}, {NULL}};
const Segment SGI322175[] = {{&S49850[79], 15}, {NULL}};
const Segment SGI322176[] = {{&S49850[64], 13}, {NULL}};
const Segment SGI322177[] = {{&S66550[3], 5}, {NULL}};
const Segment SGI322178[] = {{&S66550[3], 14}, {NULL}};
const Segment SGI322179[] = {{&S66550[38], 4}, {NULL}};
const Segment SGI322180[] = {{&S66551[3], 5}, {NULL}};
const Segment SGI322181[] = {{&S66551[3], 12}, {NULL}};
const Segment SGI322182[] = {{&S66551[36], 4}, {NULL}};
const Segment SGI322183[] = {{&S66552[3], 5}, {NULL}};
const Segment SGI322184[] = {{&S66552[29], 4}, {NULL}};
const Segment SGI322185[] = {{&S49974[30], 3}, {&S49974[46], 3}, {NULL}};
const Segment SGI322186[] = {{&S49974[8], 3}, {&S49974[8], 3}, {NULL}};
const Segment SGI322187[] = {{&S49974[13], 10}, {&S49974[13], 10}, {NULL}};
const Segment SGI322188[] = {{&S49974[42], 7}, {&S49974[106], 7}, {NULL}};
const Segment SGI322189[] = {{&S49974[35], 14}, {&S49974[99], 14}, {NULL}};
const Segment SGI322190[] = {{&S49974[164], 15}, {NULL}};
const Segment SGI322191[] = {{&S49974[122], 13}, {NULL}};
const Segment SGI322192[] = {{&S49974[115], 3}, {NULL}};
const Segment SGI322193[] = {{&S49975[30], 3}, {NULL}};
const Segment SGI322194[] = {{&S49975[8], 3}, {NULL}};
const Segment SGI322195[] = {{&S49975[13], 10}, {NULL}};
const Segment SGI322196[] = {{&S49975[42], 7}, {NULL}};
const Segment SGI322197[] = {{&S49975[35], 14}, {NULL}};
const Segment SGI322198[] = {{&S49975[91], 15}, {NULL}};
const Segment SGI322199[] = {{&S49975[162], 13}, {NULL}};
const Segment SGI322200[] = {{&S49976[30], 3}, {NULL}};
const Segment SGI322201[] = {{&S49976[8], 3}, {NULL}};
const Segment SGI322202[] = {{&S49976[13], 10}, {NULL}};
const Segment SGI322203[] = {{&S49976[42], 7}, {NULL}};
const Segment SGI322204[] = {{&S49976[35], 14}, {NULL}};
const Segment SGI322205[] = {{&S49976[141], 13}, {NULL}};
const Segment SGI322206[] = {{&S49977[30], 3}, {NULL}};
const Segment SGI322207[] = {{&S49977[8], 3}, {NULL}};
const Segment SGI322208[] = {{&S49977[13], 10}, {NULL}};
const Segment SGI322209[] = {{&S49977[42], 7}, {NULL}};
const Segment SGI322210[] = {{&S49977[35], 14}, {NULL}};
const Segment SGI322211[] = {{&S49977[93], 15}, {NULL}};
const Segment SGI322212[] = {{&S49977[51], 13}, {&S49977[51], 13}, {NULL}};
const Segment SGI322213[] = {{&S49978[30], 3}, {NULL}};
const Segment SGI322214[] = {{&S49978[8], 3}, {NULL}};
const Segment SGI322215[] = {{&S49978[13], 10}, {NULL}};
const Segment SGI322216[] = {{&S49978[55], 7}, {NULL}};
const Segment SGI322217[] = {{&S49978[48], 14}, {NULL}};
const Segment SGI322218[] = {{&S49978[64], 13}, {NULL}};
const Segment SGI322219[] = {{&S49979[30], 3}, {NULL}};
const Segment SGI322220[] = {{&S49979[8], 3}, {NULL}};
const Segment SGI322221[] = {{&S49979[13], 10}, {NULL}};
const Segment SGI322222[] = {{&S49979[42], 7}, {NULL}};
const Segment SGI322223[] = {{&S49979[35], 14}, {NULL}};
const Segment SGI322224[] = {{&S49979[98], 15}, {NULL}};
const Segment SGI322225[] = {{&S49979[51], 6}, {NULL}};
const Segment SGI322226[] = {{&S49979[169], 13}, {NULL}};
const Segment SGI322227[] = {{&S49980[30], 3}, {NULL}};
const Segment SGI322228[] = {{&S49980[8], 3}, {NULL}};
const Segment SGI322229[] = {{&S49980[13], 10}, {NULL}};
const Segment SGI322230[] = {{&S49980[42], 7}, {NULL}};
const Segment SGI322231[] = {{&S49980[35], 14}, {NULL}};
const Segment SGI322232[] = {{&S49980[51], 13}, {&S49980[51], 13}, {NULL}};
const Segment SGI322233[] = {{&S49981[30], 3}, {NULL}};
const Segment SGI322234[] = {{&S49981[8], 3}, {NULL}};
const Segment SGI322235[] = {{&S49981[13], 10}, {NULL}};
const Segment SGI322236[] = {{&S49981[42], 7}, {NULL}};
const Segment SGI322237[] = {{&S49981[35], 14}, {NULL}};
const Segment SGI322238[] = {{&S49981[108], 15}, {NULL}};
const Segment SGI322239[] = {{&S49981[51], 13}, {&S49981[51], 13}, {NULL}};
const Segment SGI322240[] = {{&S49981[149], 9}, {NULL}};
const Segment SGI322241[] = {{&S50029[30], 4}, {NULL}};
const Segment SGI322242[] = {{&S50029[8], 3}, {NULL}};
const Segment SGI322243[] = {{&S50029[13], 10}, {NULL}};
const Segment SGI322244[] = {{&S50029[43], 7}, {NULL}};
const Segment SGI322245[] = {{&S50029[36], 14}, {NULL}};
const Segment SGI322246[] = {{&S50029[72], 11}, {NULL}};
const Segment SGI322247[] = {{&S50029[52], 6}, {NULL}};
const Segment SGI322248[] = {{&S50029[72], 16}, {NULL}};
const Segment SGI322249[] = {{&S50030[30], 4}, {NULL}};
const Segment SGI322250[] = {{&S50030[8], 3}, {NULL}};
const Segment SGI322251[] = {{&S50030[13], 10}, {NULL}};
const Segment SGI322252[] = {{&S50030[43], 7}, {NULL}};
const Segment SGI322253[] = {{&S50030[36], 14}, {NULL}};
const Segment SGI322254[] = {{&S50030[65], 11}, {NULL}};
const Segment SGI322255[] = {{&S50030[65], 16}, {NULL}};
const Segment SGI322256[] = {{&S50033[30], 3}, {NULL}};
const Segment SGI322257[] = {{&S50033[8], 3}, {NULL}};
const Segment SGI322258[] = {{&S50033[13], 10}, {NULL}};
const Segment SGI322259[] = {{&S50033[42], 7}, {NULL}};
const Segment SGI322260[] = {{&S50033[35], 14}, {NULL}};
const Segment SGI322261[] = {{&S50033[151], 11}, {NULL}};
const Segment SGI322262[] = {{&S50033[51], 6}, {NULL}};
const Segment SGI322263[] = {{&S50033[151], 16}, {NULL}};
const Segment SGI322264[] = {{&S50034[30], 3}, {NULL}};
const Segment SGI322265[] = {{&S50034[8], 3}, {NULL}};
const Segment SGI322266[] = {{&S50034[13], 10}, {NULL}};
const Segment SGI322267[] = {{&S50034[42], 7}, {NULL}};
const Segment SGI322268[] = {{&S50034[35], 14}, {NULL}};
const Segment SGI322269[] = {{&S50034[71], 11}, {NULL}};
const Segment SGI322270[] = {{&S50034[51], 6}, {NULL}};
const Segment SGI322271[] = {{&S50034[71], 16}, {NULL}};
const Segment SGI322272[] = {{&S57030[4], 5}, {NULL}};
const Segment SGI322273[] = {{&S57030[25], 3}, {NULL}};
const Segment SGI322274[] = {{&S66854[3], 4}, {NULL}};
const Segment SGI322275[] = {{&S66887[3], 5}, {NULL}};
const Segment SGI322276[] = {{&S50207[30], 3}, {NULL}};
const Segment SGI322277[] = {{&S50207[8], 3}, {NULL}};
const Segment SGI322278[] = {{&S50207[13], 10}, {NULL}};
const Segment SGI322279[] = {{&S50208[30], 3}, {NULL}};
const Segment SGI322280[] = {{&S50208[8], 3}, {NULL}};
const Segment SGI322281[] = {{&S50208[13], 10}, {NULL}};
const Segment SGI322282[] = {{&S50208[35], 3}, {NULL}};
const Segment SGI322283[] = {{&S50209[30], 3}, {NULL}};
const Segment SGI322284[] = {{&S50209[8], 3}, {NULL}};
const Segment SGI322285[] = {{&S50209[13], 10}, {NULL}};
const Segment SGI322286[] = {{&S50209[35], 3}, {NULL}};
const Segment SGI322287[] = {{&S50210[30], 3}, {NULL}};
const Segment SGI322288[] = {{&S50210[8], 3}, {NULL}};
const Segment SGI322289[] = {{&S50210[13], 10}, {NULL}};
const Segment SGI322290[] = {{&S50210[35], 3}, {NULL}};
const Segment SGI322291[] = {{&S50210[40], 3}, {NULL}};
const Segment SGI322292[] = {{&S50211[30], 3}, {NULL}};
const Segment SGI322293[] = {{&S50211[8], 3}, {NULL}};
const Segment SGI322294[] = {{&S50211[13], 10}, {NULL}};
const Segment SGI322295[] = {{&S50211[35], 3}, {NULL}};
const Segment SGI322296[] = {{&S50211[40], 3}, {NULL}};
const Segment SGI322297[] = {{&S67159[3], 5}, {NULL}};
const Segment SGI322298[] = {{&S67160[3], 4}, {NULL}};
const Segment SGI322299[] = {{&S64192[0], 27}, {NULL}};
const Segment SGI322300[] = {{&S64308[0], 119}, {NULL}};
const Segment SGI322301[] = {{&S64309[0], 119}, {NULL}};
const Segment SGI322302[] = {{&S64312[0], 119}, {NULL}};
const Segment SGI322303[] = {{&S64313[0], 119}, {NULL}};
const Segment SGI322304[] = {{&S67364[3], 5}, {NULL}};
const Segment SGI322305[] = {{&S64490[0], 119}, {NULL}};
const Segment SGI322306[] = {{&S64491[0], 119}, {NULL}};
const Segment SGI322307[] = {{&S64492[0], 119}, {NULL}};
const Segment SGI322308[] = {{&S64493[0], 119}, {NULL}};
const Segment SGI322309[] = {{&S64494[0], 93}, {NULL}};
const Segment SGI322310[] = {{&S64495[0], 119}, {NULL}};
const Segment SGI322311[] = {{&S50706[30], 4}, {NULL}};
const Segment SGI322312[] = {{&S50706[8], 3}, {NULL}};
const Segment SGI322313[] = {{&S50706[13], 10}, {NULL}};
const Segment SGI322314[] = {{&S50706[36], 5}, {NULL}};
const Segment SGI322315[] = {{&S50707[30], 4}, {NULL}};
const Segment SGI322316[] = {{&S50707[8], 3}, {NULL}};
const Segment SGI322317[] = {{&S50707[13], 10}, {NULL}};
const Segment SGI322318[] = {{&S50707[36], 5}, {NULL}};
const Segment SGI322319[] = {{&S50708[30], 4}, {NULL}};
const Segment SGI322320[] = {{&S50708[8], 3}, {NULL}};
const Segment SGI322321[] = {{&S50708[13], 10}, {NULL}};
const Segment SGI322322[] = {{&S50709[30], 4}, {NULL}};
const Segment SGI322323[] = {{&S50709[8], 3}, {NULL}};
const Segment SGI322324[] = {{&S50709[13], 10}, {NULL}};
const Segment SGI322325[] = {{&S50709[41], 5}, {NULL}};
const Segment SGI322326[] = {{&S50709[36], 3}, {NULL}};
const Segment SGI322327[] = {{&S50710[30], 4}, {NULL}};
const Segment SGI322328[] = {{&S50710[8], 3}, {NULL}};
const Segment SGI322329[] = {{&S50710[13], 10}, {NULL}};
const Segment SGI322330[] = {{&S50710[41], 6}, {NULL}};
const Segment SGI322331[] = {{&S50710[36], 3}, {NULL}};
const Segment SGI322332[] = {{&S50711[30], 4}, {NULL}};
const Segment SGI322333[] = {{&S50711[8], 3}, {NULL}};
const Segment SGI322334[] = {{&S50711[13], 10}, {NULL}};
const Segment SGI322335[] = {{&S50711[41], 3}, {NULL}};
const Segment SGI322336[] = {{&S50712[30], 4}, {NULL}};
const Segment SGI322337[] = {{&S50712[8], 3}, {NULL}};
const Segment SGI322338[] = {{&S50712[13], 10}, {NULL}};
const Segment SGI322339[] = {{&S50712[36], 3}, {NULL}};
const Segment SGI322340[] = {{&S50713[30], 4}, {NULL}};
const Segment SGI322341[] = {{&S50713[8], 3}, {NULL}};
const Segment SGI322342[] = {{&S50713[13], 10}, {NULL}};
const Segment SGI322343[] = {{&S50713[41], 6}, {NULL}};
const Segment SGI322344[] = {{&S67459[3], 4}, {NULL}};
const Segment SGI322345[] = {{&S67459[3], 11}, {NULL}};
const Segment SGI322346[] = {{&S67461[3], 5}, {NULL}};
const Segment SGI322347[] = {{&S67463[3], 4}, {NULL}};
const Segment SGI322348[] = {{&S67464[3], 4}, {NULL}};
const Segment SGI322349[] = {{&S67464[3], 9}, {NULL}};
const Segment SGI322350[] = {{&S50742[30], 4}, {NULL}};
const Segment SGI322351[] = {{&S50742[8], 3}, {NULL}};
const Segment SGI322352[] = {{&S50742[13], 10}, {NULL}};
const Segment SGI322353[] = {{&S50742[43], 7}, {NULL}};
const Segment SGI322354[] = {{&S50742[36], 14}, {NULL}};
const Segment SGI322355[] = {{&S50742[65], 11}, {NULL}};
const Segment SGI322356[] = {{&S50742[65], 16}, {NULL}};
const Segment SGI322357[] = {{&S50744[30], 4}, {NULL}};
const Segment SGI322358[] = {{&S50744[8], 3}, {NULL}};
const Segment SGI322359[] = {{&S50744[13], 10}, {NULL}};
const Segment SGI322360[] = {{&S50744[43], 7}, {NULL}};
const Segment SGI322361[] = {{&S50744[36], 14}, {NULL}};
const Segment SGI322362[] = {{&S50744[72], 11}, {NULL}};
const Segment SGI322363[] = {{&S50744[52], 6}, {NULL}};
const Segment SGI322364[] = {{&S50744[72], 16}, {NULL}};
const Segment SGI322365[] = {{&S50745[30], 3}, {NULL}};
const Segment SGI322366[] = {{&S50745[8], 3}, {NULL}};
const Segment SGI322367[] = {{&S50745[13], 10}, {NULL}};
const Segment SGI322368[] = {{&S50745[42], 7}, {NULL}};
const Segment SGI322369[] = {{&S50745[35], 14}, {NULL}};
const Segment SGI322370[] = {{&S50745[71], 11}, {NULL}};
const Segment SGI322371[] = {{&S50745[51], 6}, {NULL}};
const Segment SGI322372[] = {{&S50745[71], 16}, {NULL}};
const Segment SGI322373[] = {{&S50747[30], 3}, {NULL}};
const Segment SGI322374[] = {{&S50747[8], 3}, {NULL}};
const Segment SGI322375[] = {{&S50747[13], 10}, {NULL}};
const Segment SGI322376[] = {{&S50747[42], 7}, {NULL}};
const Segment SGI322377[] = {{&S50747[35], 14}, {NULL}};
const Segment SGI322378[] = {{&S50747[151], 11}, {NULL}};
const Segment SGI322379[] = {{&S50747[51], 6}, {NULL}};
const Segment SGI322380[] = {{&S50747[151], 16}, {NULL}};
const Segment SGI322381[] = {{&S50976[30], 4}, {NULL}};
const Segment SGI322382[] = {{&S50976[8], 3}, {NULL}};
const Segment SGI322383[] = {{&S50976[13], 10}, {NULL}};
const Segment SGI322384[] = {{&S50976[43], 7}, {NULL}};
const Segment SGI322385[] = {{&S50976[36], 14}, {NULL}};
const Segment SGI322386[] = {{&S50976[52], 6}, {NULL}};
const Segment SGI322387[] = {{&S50976[72], 15}, {NULL}};
const Segment SGI322388[] = {{&S50977[30], 4}, {NULL}};
const Segment SGI322389[] = {{&S50977[8], 3}, {NULL}};
const Segment SGI322390[] = {{&S50977[13], 10}, {NULL}};
const Segment SGI322391[] = {{&S50977[43], 7}, {NULL}};
const Segment SGI322392[] = {{&S50977[36], 14}, {NULL}};
const Segment SGI322393[] = {{&S50977[65], 15}, {NULL}};
const Segment SGI322394[] = {{&S50978[30], 4}, {NULL}};
const Segment SGI322395[] = {{&S50978[8], 3}, {NULL}};
const Segment SGI322396[] = {{&S50978[13], 10}, {NULL}};
const Segment SGI322397[] = {{&S50978[43], 7}, {NULL}};
const Segment SGI322398[] = {{&S50978[36], 14}, {NULL}};
const Segment SGI322399[] = {{&S50978[52], 6}, {NULL}};
const Segment SGI322400[] = {{&S50978[87], 15}, {NULL}};
const Segment SGI322401[] = {{&S50978[72], 13}, {NULL}};
const Segment SGI322402[] = {{&S51033[30], 3}, {NULL}};
const Segment SGI322403[] = {{&S51033[8], 3}, {NULL}};
const Segment SGI322404[] = {{&S51033[13], 10}, {NULL}};
const Segment SGI322405[] = {{&S51033[42], 7}, {NULL}};
const Segment SGI322406[] = {{&S51033[35], 14}, {NULL}};
const Segment SGI322407[] = {{&S51033[137], 5}, {NULL}};
const Segment SGI322408[] = {{&S51034[30], 3}, {NULL}};
const Segment SGI322409[] = {{&S51034[8], 3}, {NULL}};
const Segment SGI322410[] = {{&S51034[13], 10}, {NULL}};
const Segment SGI322411[] = {{&S51034[42], 7}, {NULL}};
const Segment SGI322412[] = {{&S51034[35], 14}, {NULL}};
const Segment SGI322413[] = {{&S51034[143], 5}, {NULL}};
const Segment SGI322414[] = {{&S51035[30], 3}, {NULL}};
const Segment SGI322415[] = {{&S51035[8], 3}, {NULL}};
const Segment SGI322416[] = {{&S51035[13], 10}, {NULL}};
const Segment SGI322417[] = {{&S51035[42], 7}, {NULL}};
const Segment SGI322418[] = {{&S51035[35], 14}, {NULL}};
const Segment SGI322419[] = {{&S51035[143], 5}, {NULL}};
const Segment SGI322420[] = {{&S51036[30], 3}, {NULL}};
const Segment SGI322421[] = {{&S51036[8], 3}, {NULL}};
const Segment SGI322422[] = {{&S51036[13], 10}, {NULL}};
const Segment SGI322423[] = {{&S51036[42], 7}, {NULL}};
const Segment SGI322424[] = {{&S51036[35], 14}, {NULL}};
const Segment SGI322425[] = {{&S51036[148], 5}, {NULL}};
const Segment SGI322426[] = {{&S51036[51], 3}, {NULL}};
const Segment SGI322427[] = {{&S51037[30], 3}, {NULL}};
const Segment SGI322428[] = {{&S51037[8], 3}, {NULL}};
const Segment SGI322429[] = {{&S51037[13], 10}, {NULL}};
const Segment SGI322430[] = {{&S51037[42], 7}, {NULL}};
const Segment SGI322431[] = {{&S51037[35], 14}, {NULL}};
const Segment SGI322432[] = {{&S51037[137], 5}, {NULL}};
const Segment SGI322433[] = {{&S51038[30], 3}, {NULL}};
const Segment SGI322434[] = {{&S51038[8], 3}, {NULL}};
const Segment SGI322435[] = {{&S51038[13], 10}, {NULL}};
const Segment SGI322436[] = {{&S51038[42], 7}, {NULL}};
const Segment SGI322437[] = {{&S51038[35], 14}, {NULL}};
const Segment SGI322438[] = {{&S51038[137], 6}, {NULL}};
const Segment SGI322439[] = {{&S51039[30], 3}, {NULL}};
const Segment SGI322440[] = {{&S51039[8], 3}, {NULL}};
const Segment SGI322441[] = {{&S51039[13], 10}, {NULL}};
const Segment SGI322442[] = {{&S51039[42], 7}, {NULL}};
const Segment SGI322443[] = {{&S51039[35], 14}, {NULL}};
const Segment SGI322444[] = {{&S51039[148], 5}, {NULL}};
const Segment SGI322445[] = {{&S51039[51], 3}, {NULL}};
const Segment SGI322446[] = {{&S51040[30], 3}, {NULL}};
const Segment SGI322447[] = {{&S51040[8], 3}, {NULL}};
const Segment SGI322448[] = {{&S51040[13], 10}, {NULL}};
const Segment SGI322449[] = {{&S51040[42], 7}, {NULL}};
const Segment SGI322450[] = {{&S51040[35], 14}, {NULL}};
const Segment SGI322451[] = {{&S51040[143], 5}, {NULL}};
const Segment SGI322452[] = {{&S51041[30], 3}, {NULL}};
const Segment SGI322453[] = {{&S51041[8], 3}, {NULL}};
const Segment SGI322454[] = {{&S51041[13], 10}, {NULL}};
const Segment SGI322455[] = {{&S51041[42], 7}, {NULL}};
const Segment SGI322456[] = {{&S51041[35], 14}, {NULL}};
const Segment SGI322457[] = {{&S51041[143], 5}, {NULL}};
const Segment SGI322458[] = {{&S51042[30], 3}, {NULL}};
const Segment SGI322459[] = {{&S51042[8], 3}, {NULL}};
const Segment SGI322460[] = {{&S51042[13], 10}, {NULL}};
const Segment SGI322461[] = {{&S51042[42], 7}, {NULL}};
const Segment SGI322462[] = {{&S51042[35], 14}, {NULL}};
const Segment SGI322463[] = {{&S51042[137], 6}, {NULL}};
const Segment SGI322464[] = {{&S51043[30], 3}, {NULL}};
const Segment SGI322465[] = {{&S51043[8], 3}, {NULL}};
const Segment SGI322466[] = {{&S51043[13], 10}, {NULL}};
const Segment SGI322467[] = {{&S51043[42], 7}, {NULL}};
const Segment SGI322468[] = {{&S51043[35], 14}, {NULL}};
const Segment SGI322469[] = {{&S51043[164], 6}, {NULL}};
const Segment SGI322470[] = {{&S51044[30], 3}, {NULL}};
const Segment SGI322471[] = {{&S51044[8], 3}, {NULL}};
const Segment SGI322472[] = {{&S51044[13], 10}, {NULL}};
const Segment SGI322473[] = {{&S51044[42], 7}, {NULL}};
const Segment SGI322474[] = {{&S51044[35], 14}, {NULL}};
const Segment SGI322475[] = {{&S51044[156], 6}, {NULL}};
const Segment SGI322476[] = {{&S51045[30], 3}, {NULL}};
const Segment SGI322477[] = {{&S51045[8], 3}, {NULL}};
const Segment SGI322478[] = {{&S51045[13], 10}, {NULL}};
const Segment SGI322479[] = {{&S51045[42], 7}, {NULL}};
const Segment SGI322480[] = {{&S51045[35], 14}, {NULL}};
const Segment SGI322481[] = {{&S51045[156], 5}, {NULL}};
const Segment SGI322482[] = {{&S51046[30], 3}, {NULL}};
const Segment SGI322483[] = {{&S51046[8], 3}, {NULL}};
const Segment SGI322484[] = {{&S51046[13], 10}, {NULL}};
const Segment SGI322485[] = {{&S51046[42], 7}, {NULL}};
const Segment SGI322486[] = {{&S51046[35], 14}, {NULL}};
const Segment SGI322487[] = {{&S51046[166], 5}, {NULL}};
const Segment SGI322488[] = {{&S51047[30], 3}, {NULL}};
const Segment SGI322489[] = {{&S51047[8], 3}, {NULL}};
const Segment SGI322490[] = {{&S51047[13], 10}, {NULL}};
const Segment SGI322491[] = {{&S51047[42], 7}, {NULL}};
const Segment SGI322492[] = {{&S51047[35], 14}, {NULL}};
const Segment SGI322493[] = {{&S51047[178], 5}, {NULL}};
const Segment SGI322494[] = {{&S51048[30], 3}, {NULL}};
const Segment SGI322495[] = {{&S51048[8], 3}, {NULL}};
const Segment SGI322496[] = {{&S51048[13], 10}, {NULL}};
const Segment SGI322497[] = {{&S51048[42], 7}, {NULL}};
const Segment SGI322498[] = {{&S51048[35], 14}, {NULL}};
const Segment SGI322499[] = {{&S51048[143], 5}, {NULL}};
const Segment SGI322500[] = {{&S51049[30], 3}, {NULL}};
const Segment SGI322501[] = {{&S51049[8], 3}, {NULL}};
const Segment SGI322502[] = {{&S51049[13], 10}, {NULL}};
const Segment SGI322503[] = {{&S51049[42], 7}, {NULL}};
const Segment SGI322504[] = {{&S51049[35], 14}, {NULL}};
const Segment SGI322505[] = {{&S51049[156], 6}, {NULL}};
const Segment SGI322506[] = {{&S51050[30], 3}, {NULL}};
const Segment SGI322507[] = {{&S51050[8], 3}, {NULL}};
const Segment SGI322508[] = {{&S51050[13], 10}, {NULL}};
const Segment SGI322509[] = {{&S51050[42], 7}, {NULL}};
const Segment SGI322510[] = {{&S51050[35], 14}, {NULL}};
const Segment SGI322511[] = {{&S51050[85], 5}, {NULL}};
const Segment SGI322512[] = {{&S51051[30], 3}, {NULL}};
const Segment SGI322513[] = {{&S51051[8], 3}, {NULL}};
const Segment SGI322514[] = {{&S51051[13], 10}, {NULL}};
const Segment SGI322515[] = {{&S51051[42], 7}, {NULL}};
const Segment SGI322516[] = {{&S51051[35], 14}, {NULL}};
const Segment SGI322517[] = {{&S51051[168], 5}, {NULL}};
const Segment SGI322518[] = {{&S51052[30], 3}, {NULL}};
const Segment SGI322519[] = {{&S51052[8], 3}, {NULL}};
const Segment SGI322520[] = {{&S51052[13], 10}, {NULL}};
const Segment SGI322521[] = {{&S51052[42], 7}, {NULL}};
const Segment SGI322522[] = {{&S51052[35], 14}, {NULL}};
const Segment SGI322523[] = {{&S51052[233], 5}, {NULL}};
const Segment SGI322524[] = {{&S51053[30], 3}, {NULL}};
const Segment SGI322525[] = {{&S51053[8], 3}, {NULL}};
const Segment SGI322526[] = {{&S51053[13], 10}, {NULL}};
const Segment SGI322527[] = {{&S51053[42], 7}, {NULL}};
const Segment SGI322528[] = {{&S51053[35], 14}, {NULL}};
const Segment SGI322529[] = {{&S51053[156], 6}, {NULL}};
const Segment SGI322530[] = {{&S51054[30], 3}, {NULL}};
const Segment SGI322531[] = {{&S51054[8], 3}, {NULL}};
const Segment SGI322532[] = {{&S51054[13], 10}, {NULL}};
const Segment SGI322533[] = {{&S51054[42], 7}, {NULL}};
const Segment SGI322534[] = {{&S51054[35], 14}, {NULL}};
const Segment SGI322535[] = {{&S51054[137], 6}, {NULL}};
const Segment SGI322536[] = {{&S51055[30], 3}, {NULL}};
const Segment SGI322537[] = {{&S51055[8], 3}, {NULL}};
const Segment SGI322538[] = {{&S51055[13], 10}, {NULL}};
const Segment SGI322539[] = {{&S51055[42], 7}, {NULL}};
const Segment SGI322540[] = {{&S51055[35], 14}, {NULL}};
const Segment SGI322541[] = {{&S51055[137], 6}, {NULL}};
const Segment SGI322542[] = {{&S65174[0], 12}, {NULL}};
const Segment SGI322543[] = {{&S65175[0], 51}, {NULL}};
const Segment SGI322544[] = {{&S65177[0], 55}, {NULL}};
const Segment SGI322545[] = {{&S65178[0], 55}, {NULL}};
const Segment SGI322546[] = {{&S65179[0], 41}, {NULL}};
const Segment SGI322547[] = {{&S65180[0], 54}, {NULL}};
const Segment SGI322548[] = {{&S65181[0], 68}, {NULL}};
const Segment SGI322549[] = {{&S65182[0], 53}, {NULL}};
const Segment SGI322550[] = {{&S65183[0], 53}, {NULL}};
const Segment SGI322551[] = {{&S65186[0], 54}, {NULL}};
const Segment SGI322552[] = {{&S65187[0], 55}, {NULL}};
const Segment SGI322553[] = {{&S51157[30], 4}, {NULL}};
const Segment SGI322554[] = {{&S51157[8], 3}, {NULL}};
const Segment SGI322555[] = {{&S51157[13], 10}, {NULL}};
const Segment SGI322556[] = {{&S51157[43], 3}, {NULL}};
const Segment SGI322557[] = {{&S51158[8], 3}, {NULL}};
const Segment SGI322558[] = {{&S51158[13], 10}, {NULL}};
const Segment SGI322559[] = {{&S51158[42], 3}, {NULL}};
const Segment SGI322560[] = {{&S51159[30], 3}, {NULL}};
const Segment SGI322561[] = {{&S51159[8], 3}, {NULL}};
const Segment SGI322562[] = {{&S51159[13], 10}, {NULL}};
const Segment SGI322563[] = {{&S51159[35], 5}, {NULL}};
const Segment SGI322564[] = {{&S68160[3], 5}, {NULL}};
const Segment SGI322565[] = {{&S68160[3], 14}, {NULL}};
const Segment SGI322566[] = {{&S68161[3], 5}, {NULL}};
const Segment SGI322567[] = {{&S68161[3], 9}, {NULL}};
const Segment SGI322568[] = {{&S68162[3], 5}, {NULL}};
const Segment SGI322569[] = {{&S68163[3], 5}, {NULL}};
const Segment SGI322570[] = {{&S68163[3], 12}, {NULL}};
const Segment SGI322571[] = {{&S51357[30], 3}, {NULL}};
const Segment SGI322572[] = {{&S51357[8], 3}, {NULL}};
const Segment SGI322573[] = {{&S51357[13], 10}, {NULL}};
const Segment SGI322574[] = {{&S51357[42], 7}, {NULL}};
const Segment SGI322575[] = {{&S51357[35], 14}, {NULL}};
const Segment SGI322576[] = {{&S51357[71], 11}, {NULL}};
const Segment SGI322577[] = {{&S51357[151], 15}, {NULL}};
const Segment SGI322578[] = {{&S51357[51], 6}, {NULL}};
const Segment SGI322579[] = {{&S51358[30], 3}, {NULL}};
const Segment SGI322580[] = {{&S51358[8], 3}, {NULL}};
const Segment SGI322581[] = {{&S51358[13], 10}, {NULL}};
const Segment SGI322582[] = {{&S51358[42], 7}, {NULL}};
const Segment SGI322583[] = {{&S51358[35], 14}, {NULL}};
const Segment SGI322584[] = {{&S51358[64], 11}, {NULL}};
const Segment SGI322585[] = {{&S51358[144], 15}, {NULL}};
const Segment SGI322586[] = {{&S51359[30], 3}, {NULL}};
const Segment SGI322587[] = {{&S51359[8], 3}, {NULL}};
const Segment SGI322588[] = {{&S51359[13], 10}, {NULL}};
const Segment SGI322589[] = {{&S51359[42], 7}, {NULL}};
const Segment SGI322590[] = {{&S51359[35], 14}, {NULL}};
const Segment SGI322591[] = {{&S51359[71], 11}, {NULL}};
const Segment SGI322592[] = {{&S51359[151], 15}, {NULL}};
const Segment SGI322593[] = {{&S51359[184], 4}, {NULL}};
const Segment SGI322594[] = {{&S51359[51], 6}, {NULL}};
const Segment SGI322595[] = {{&S51360[30], 3}, {NULL}};
const Segment SGI322596[] = {{&S51360[8], 3}, {NULL}};
const Segment SGI322597[] = {{&S51360[13], 10}, {NULL}};
const Segment SGI322598[] = {{&S51360[42], 7}, {NULL}};
const Segment SGI322599[] = {{&S51360[35], 14}, {NULL}};
const Segment SGI322600[] = {{&S51360[64], 11}, {NULL}};
const Segment SGI322601[] = {{&S51360[144], 15}, {NULL}};
const Segment SGI322602[] = {{&S51360[165], 9}, {NULL}};
const Segment SGI322603[] = {{&S51361[30], 3}, {NULL}};
const Segment SGI322604[] = {{&S51361[8], 3}, {NULL}};
const Segment SGI322605[] = {{&S51361[13], 10}, {NULL}};
const Segment SGI322606[] = {{&S51361[42], 7}, {NULL}};
const Segment SGI322607[] = {{&S51361[35], 14}, {NULL}};
const Segment SGI322608[] = {{&S51361[71], 11}, {NULL}};
const Segment SGI322609[] = {{&S51361[151], 15}, {NULL}};
const Segment SGI322610[] = {{&S51361[51], 6}, {NULL}};
const Segment SGI322611[] = {{&S51385[30], 4}, {NULL}};
const Segment SGI322612[] = {{&S51385[8], 3}, {NULL}};
const Segment SGI322613[] = {{&S51385[13], 10}, {NULL}};
const Segment SGI322614[] = {{&S51385[43], 7}, {NULL}};
const Segment SGI322615[] = {{&S51385[36], 14}, {NULL}};
const Segment SGI322616[] = {{&S51385[72], 11}, {NULL}};
const Segment SGI322617[] = {{&S51385[52], 6}, {NULL}};
const Segment SGI322618[] = {{&S51385[72], 16}, {NULL}};
const Segment SGI322619[] = {{&S51386[30], 4}, {NULL}};
const Segment SGI322620[] = {{&S51386[8], 3}, {NULL}};
const Segment SGI322621[] = {{&S51386[13], 10}, {NULL}};
const Segment SGI322622[] = {{&S51386[43], 7}, {NULL}};
const Segment SGI322623[] = {{&S51386[36], 14}, {NULL}};
const Segment SGI322624[] = {{&S51386[65], 11}, {NULL}};
const Segment SGI322625[] = {{&S51386[65], 16}, {NULL}};
const Segment SGI322626[] = {{&S51387[30], 3}, {NULL}};
const Segment SGI322627[] = {{&S51387[8], 3}, {NULL}};
const Segment SGI322628[] = {{&S51387[13], 10}, {NULL}};
const Segment SGI322629[] = {{&S51387[42], 7}, {NULL}};
const Segment SGI322630[] = {{&S51387[35], 14}, {NULL}};
const Segment SGI322631[] = {{&S51387[151], 11}, {NULL}};
const Segment SGI322632[] = {{&S51387[51], 6}, {NULL}};
const Segment SGI322633[] = {{&S51387[151], 16}, {NULL}};
const Segment SGI322634[] = {{&S51389[30], 3}, {NULL}};
const Segment SGI322635[] = {{&S51389[8], 3}, {NULL}};
const Segment SGI322636[] = {{&S51389[13], 10}, {NULL}};
const Segment SGI322637[] = {{&S51389[42], 7}, {NULL}};
const Segment SGI322638[] = {{&S51389[35], 14}, {NULL}};
const Segment SGI322639[] = {{&S51389[71], 11}, {NULL}};
const Segment SGI322640[] = {{&S51389[51], 6}, {NULL}};
const Segment SGI322641[] = {{&S51389[71], 16}, {NULL}};
const Segment SGI322642[] = {{&S68384[3], 4}, {NULL}};
const Segment SGI322643[] = {{&S51451[30], 3}, {NULL}};
const Segment SGI322644[] = {{&S51451[8], 3}, {NULL}};
const Segment SGI322645[] = {{&S51451[13], 10}, {NULL}};
const Segment SGI322646[] = {{&S51451[42], 7}, {NULL}};
const Segment SGI322647[] = {{&S51451[35], 14}, {NULL}};
const Segment SGI322648[] = {{&S51451[80], 5}, {NULL}};
const Segment SGI322649[] = {{&S51451[51], 6}, {NULL}};
const Segment SGI322650[] = {{&S51452[30], 4}, {NULL}};
const Segment SGI322651[] = {{&S51452[8], 3}, {NULL}};
const Segment SGI322652[] = {{&S51452[13], 10}, {NULL}};
const Segment SGI322653[] = {{&S51452[43], 7}, {NULL}};
const Segment SGI322654[] = {{&S51452[36], 14}, {NULL}};
const Segment SGI322655[] = {{&S51452[81], 5}, {NULL}};
const Segment SGI322656[] = {{&S51452[52], 6}, {NULL}};
const Segment SGI322657[] = {{&S51453[30], 3}, {NULL}};
const Segment SGI322658[] = {{&S51453[8], 3}, {NULL}};
const Segment SGI322659[] = {{&S51453[13], 10}, {NULL}};
const Segment SGI322660[] = {{&S51453[42], 7}, {NULL}};
const Segment SGI322661[] = {{&S51453[35], 14}, {NULL}};
const Segment SGI322662[] = {{&S51453[80], 5}, {NULL}};
const Segment SGI322663[] = {{&S51453[51], 6}, {NULL}};
const Segment SGI322664[] = {{&S51454[30], 4}, {NULL}};
const Segment SGI322665[] = {{&S51454[8], 3}, {NULL}};
const Segment SGI322666[] = {{&S51454[13], 10}, {NULL}};
const Segment SGI322667[] = {{&S51454[43], 7}, {NULL}};
const Segment SGI322668[] = {{&S51454[36], 14}, {NULL}};
const Segment SGI322669[] = {{&S51454[81], 5}, {NULL}};
const Segment SGI322670[] = {{&S51454[52], 6}, {NULL}};
const Segment SGI322671[] = {{&S51457[30], 4}, {NULL}};
const Segment SGI322672[] = {{&S51457[8], 3}, {NULL}};
const Segment SGI322673[] = {{&S51457[13], 10}, {NULL}};
const Segment SGI322674[] = {{&S51457[43], 7}, {NULL}};
const Segment SGI322675[] = {{&S51457[36], 14}, {NULL}};
const Segment SGI322676[] = {{&S51457[81], 5}, {NULL}};
const Segment SGI322677[] = {{&S51457[52], 6}, {NULL}};
const Segment SGI322678[] = {{&S51458[30], 3}, {NULL}};
const Segment SGI322679[] = {{&S51458[8], 3}, {NULL}};
const Segment SGI322680[] = {{&S51458[13], 10}, {NULL}};
const Segment SGI322681[] = {{&S51458[42], 7}, {NULL}};
const Segment SGI322682[] = {{&S51458[35], 14}, {NULL}};
const Segment SGI322683[] = {{&S51458[80], 5}, {NULL}};
const Segment SGI322684[] = {{&S51458[51], 6}, {NULL}};
const Segment SGI322685[] = {{&S51461[30], 4}, {NULL}};
const Segment SGI322686[] = {{&S51461[8], 3}, {NULL}};
const Segment SGI322687[] = {{&S51461[13], 10}, {NULL}};
const Segment SGI322688[] = {{&S51461[43], 7}, {NULL}};
const Segment SGI322689[] = {{&S51461[36], 14}, {NULL}};
const Segment SGI322690[] = {{&S51461[81], 5}, {NULL}};
const Segment SGI322691[] = {{&S51461[52], 6}, {NULL}};
const Segment SGI322692[] = {{&S51462[30], 3}, {NULL}};
const Segment SGI322693[] = {{&S51462[8], 3}, {NULL}};
const Segment SGI322694[] = {{&S51462[13], 10}, {NULL}};
const Segment SGI322695[] = {{&S51462[42], 7}, {NULL}};
const Segment SGI322696[] = {{&S51462[35], 14}, {NULL}};
const Segment SGI322697[] = {{&S51462[80], 5}, {NULL}};
const Segment SGI322698[] = {{&S51462[51], 6}, {NULL}};
const Segment SGI322699[] = {{&S80029[3], 5}, {NULL}};
const Segment SGI322700[] = {{&S80029[3], 8}, {NULL}};
const Segment SGI322701[] = {{&S80029[31], 9}, {NULL}};
const Segment SGI322702[] = {{&S80031[3], 5}, {NULL}};
const Segment SGI322703[] = {{&S80031[28], 9}, {NULL}};
const Segment SGI322704[] = {{&S58544[4], 6}, {NULL}};
const Segment SGI322705[] = {{&S58544[26], 3}, {NULL}};
const Segment SGI322706[] = {{&S58545[4], 6}, {NULL}};
const Segment SGI322707[] = {{&S58545[26], 3}, {NULL}};
const Segment SGI322708[] = {{&S80263[3], 6}, {NULL}};
const Segment SGI322709[] = {{&S80263[3], 15}, {NULL}};
const Segment SGI322710[] = {{&S80265[3], 6}, {NULL}};
const Segment SGI322711[] = {{&S55713[4], 5}, {NULL}};
const Segment SGI322712[] = {{&S55713[27], 11}, {NULL}};
const Segment SGI322713[] = {{&S55713[39], 5}, {NULL}};
const Segment SGI322714[] = {{&S80342[3], 6}, {NULL}};
const Segment SGI322715[] = {{&S69079[3], 5}, {NULL}};
const Segment SGI322716[] = {{&S52242[30], 3}, {NULL}};
const Segment SGI322717[] = {{&S52242[8], 3}, {NULL}};
const Segment SGI322718[] = {{&S52242[13], 10}, {NULL}};
const Segment SGI322719[] = {{&S52242[42], 7}, {NULL}};
const Segment SGI322720[] = {{&S52242[35], 14}, {NULL}};
const Segment SGI322721[] = {{&S52242[64], 11}, {NULL}};
const Segment SGI322722[] = {{&S52242[64], 16}, {NULL}};
const Segment SGI322723[] = {{&S52243[30], 3}, {NULL}};
const Segment SGI322724[] = {{&S52243[8], 3}, {NULL}};
const Segment SGI322725[] = {{&S52243[13], 10}, {NULL}};
const Segment SGI322726[] = {{&S52243[42], 7}, {NULL}};
const Segment SGI322727[] = {{&S52243[35], 14}, {NULL}};
const Segment SGI322728[] = {{&S52243[71], 11}, {NULL}};
const Segment SGI322729[] = {{&S52243[51], 6}, {NULL}};
const Segment SGI322730[] = {{&S52243[71], 16}, {NULL}};
const Segment SGI322731[] = {{&S52245[30], 3}, {NULL}};
const Segment SGI322732[] = {{&S52245[8], 3}, {NULL}};
const Segment SGI322733[] = {{&S52245[13], 10}, {NULL}};
const Segment SGI322734[] = {{&S52245[42], 7}, {NULL}};
const Segment SGI322735[] = {{&S52245[35], 14}, {NULL}};
const Segment SGI322736[] = {{&S52245[66], 11}, {NULL}};
const Segment SGI322737[] = {{&S52245[119], 15}, {NULL}};
const Segment SGI322738[] = {{&S52245[51], 6}, {NULL}};
const Segment SGI322739[] = {{&S52245[66], 16}, {NULL}};
const Segment SGI322740[] = {{&S52246[30], 3}, {NULL}};
const Segment SGI322741[] = {{&S52246[8], 3}, {NULL}};
const Segment SGI322742[] = {{&S52246[13], 10}, {NULL}};
const Segment SGI322743[] = {{&S52246[42], 7}, {NULL}};
const Segment SGI322744[] = {{&S52246[35], 14}, {NULL}};
const Segment SGI322745[] = {{&S52246[51], 11}, {NULL}};
const Segment SGI322746[] = {{&S52246[112], 15}, {NULL}};
const Segment SGI322747[] = {{&S52246[51], 16}, {NULL}};
const Segment SGI322748[] = {{&S52248[30], 3}, {NULL}};
const Segment SGI322749[] = {{&S52248[8], 3}, {NULL}};
const Segment SGI322750[] = {{&S52248[13], 10}, {NULL}};
const Segment SGI322751[] = {{&S52248[42], 7}, {NULL}};
const Segment SGI322752[] = {{&S52248[35], 14}, {NULL}};
const Segment SGI322753[] = {{&S52248[71], 11}, {NULL}};
const Segment SGI322754[] = {{&S52248[51], 6}, {NULL}};
const Segment SGI322755[] = {{&S52248[71], 16}, {NULL}};
const Segment SGI322756[] = {{&S52249[30], 3}, {NULL}};
const Segment SGI322757[] = {{&S52249[8], 3}, {NULL}};
const Segment SGI322758[] = {{&S52249[13], 10}, {NULL}};
const Segment SGI322759[] = {{&S52249[42], 7}, {NULL}};
const Segment SGI322760[] = {{&S52249[35], 14}, {NULL}};
const Segment SGI322761[] = {{&S52249[71], 11}, {NULL}};
const Segment SGI322762[] = {{&S52249[124], 15}, {NULL}};
const Segment SGI322763[] = {{&S52249[51], 6}, {NULL}};
const Segment SGI322764[] = {{&S52249[71], 16}, {NULL}};
const Segment SGI322765[] = {{&S52250[30], 3}, {NULL}};
const Segment SGI322766[] = {{&S52250[8], 3}, {NULL}};
const Segment SGI322767[] = {{&S52250[13], 10}, {NULL}};
const Segment SGI322768[] = {{&S52250[42], 7}, {NULL}};
const Segment SGI322769[] = {{&S52250[35], 14}, {NULL}};
const Segment SGI322770[] = {{&S52250[51], 11}, {NULL}};
const Segment SGI322771[] = {{&S52250[51], 16}, {NULL}};
const Segment SGI322772[] = {{&S52251[30], 3}, {NULL}};
const Segment SGI322773[] = {{&S52251[8], 3}, {NULL}};
const Segment SGI322774[] = {{&S52251[13], 10}, {NULL}};
const Segment SGI322775[] = {{&S52251[42], 7}, {NULL}};
const Segment SGI322776[] = {{&S52251[35], 14}, {NULL}};
const Segment SGI322777[] = {{&S52251[64], 11}, {NULL}};
const Segment SGI322778[] = {{&S52251[125], 15}, {NULL}};
const Segment SGI322779[] = {{&S52251[64], 16}, {NULL}};
const Segment SGI322780[] = {{&S52252[30], 3}, {NULL}};
const Segment SGI322781[] = {{&S52252[8], 3}, {NULL}};
const Segment SGI322782[] = {{&S52252[13], 10}, {NULL}};
const Segment SGI322783[] = {{&S52252[42], 7}, {NULL}};
const Segment SGI322784[] = {{&S52252[35], 14}, {NULL}};
const Segment SGI322785[] = {{&S52252[64], 11}, {NULL}};
const Segment SGI322786[] = {{&S52252[64], 16}, {NULL}};
const Segment SGI322787[] = {{&S52563[30], 4}, {NULL}};
const Segment SGI322788[] = {{&S52563[8], 3}, {NULL}};
const Segment SGI322789[] = {{&S52563[13], 10}, {NULL}};
const Segment SGI322790[] = {{&S52563[43], 7}, {NULL}};
const Segment SGI322791[] = {{&S52563[36], 14}, {NULL}};
const Segment SGI322792[] = {{&S52563[52], 6}, {NULL}};
const Segment SGI322793[] = {{&S52563[72], 15}, {NULL}};
const Segment SGI322794[] = {{&S52564[30], 4}, {NULL}};
const Segment SGI322795[] = {{&S52564[8], 3}, {NULL}};
const Segment SGI322796[] = {{&S52564[13], 10}, {NULL}};
const Segment SGI322797[] = {{&S52564[43], 7}, {NULL}};
const Segment SGI322798[] = {{&S52564[36], 14}, {NULL}};
const Segment SGI322799[] = {{&S52564[65], 15}, {NULL}};
const Segment SGI322800[] = {{&S52565[30], 4}, {NULL}};
const Segment SGI322801[] = {{&S52565[8], 3}, {NULL}};
const Segment SGI322802[] = {{&S52565[13], 10}, {NULL}};
const Segment SGI322803[] = {{&S52565[43], 7}, {NULL}};
const Segment SGI322804[] = {{&S52565[36], 14}, {NULL}};
const Segment SGI322805[] = {{&S52565[52], 6}, {NULL}};
const Segment SGI322806[] = {{&S52565[87], 15}, {NULL}};
const Segment SGI322807[] = {{&S52565[72], 13}, {NULL}};
const Segment SGI322808[] = {{&S52635[30], 3}, {NULL}};
const Segment SGI322809[] = {{&S52635[8], 3}, {NULL}};
const Segment SGI322810[] = {{&S52635[13], 10}, {NULL}};
const Segment SGI322811[] = {{&S52635[42], 7}, {NULL}};
const Segment SGI322812[] = {{&S52635[35], 14}, {NULL}};
const Segment SGI322813[] = {{&S52635[64], 7}, {NULL}};
const Segment SGI322814[] = {{&S52635[71], 4}, {NULL}};
const Segment SGI322815[] = {{&S52636[30], 3}, {NULL}};
const Segment SGI322816[] = {{&S52636[8], 3}, {NULL}};
const Segment SGI322817[] = {{&S52636[13], 10}, {NULL}};
const Segment SGI322818[] = {{&S52636[42], 7}, {NULL}};
const Segment SGI322819[] = {{&S52636[35], 14}, {NULL}};
const Segment SGI322820[] = {{&S52636[64], 7}, {NULL}};
const Segment SGI322821[] = {{&S52636[71], 4}, {NULL}};
const Segment SGI322822[] = {{&S52637[30], 3}, {&S52637[94], 3}, {NULL}};
const Segment SGI322823[] = {{&S52637[8], 3}, {&S52637[8], 3}, {NULL}};
const Segment SGI322824[] = {{&S52637[13], 10}, {&S52637[13], 10}, {NULL}};
const Segment SGI322825[] = {{&S52637[42], 7}, {&S52637[42], 7}, {NULL}};
const Segment SGI322826[] = {{&S52637[35], 14}, {&S52637[35], 14}, {NULL}};
const Segment SGI322827[] = {{&S52637[122], 7}, {NULL}};
const Segment SGI322828[] = {{&S52637[129], 4}, {NULL}};
const Segment SGI322829[] = {{&S52637[115], 3}, {NULL}};
const Segment SGI322830[] = {{&S59489[4], 5}, {NULL}};
const Segment SGI322831[] = {{&S59489[25], 3}, {NULL}};
const Segment SGI322832[] = {{&S59490[4], 5}, {NULL}};
const Segment SGI322833[] = {{&S59490[25], 3}, {NULL}};
const Segment SGI322834[] = {{&S52696[30], 3}, {NULL}};
const Segment SGI322835[] = {{&S52696[8], 3}, {NULL}};
const Segment SGI322836[] = {{&S52696[13], 10}, {NULL}};
const Segment SGI322837[] = {{&S52696[35], 3}, {NULL}};
const Segment SGI322838[] = {{&S52696[40], 9}, {NULL}};
const Segment SGI322839[] = {{&S66548[0], 119}, {NULL}};
const Segment SGI322840[] = {{&S66549[0], 119}, {NULL}};
const Segment SGI322841[] = {{&S66553[0], 119}, {NULL}};
const Segment SGI322842[] = {{&S66554[0], 119}, {NULL}};
const Segment SGI322843[] = {{&S66606[0], 34}, {NULL}};
const Segment SGI322844[] = {{&S59625[4], 5}, {NULL}};
const Segment SGI322845[] = {{&S59625[25], 3}, {NULL}};
const Segment SGI322846[] = {{&S69873[3], 4}, {NULL}};
const Segment SGI322847[] = {{&S69873[3], 11}, {NULL}};
const Segment SGI322848[] = {{&S69875[3], 4}, {NULL}};
const Segment SGI322849[] = {{&S52991[30], 4}, {NULL}};
const Segment SGI322850[] = {{&S52991[8], 3}, {NULL}};
const Segment SGI322851[] = {{&S52991[13], 10}, {NULL}};
const Segment SGI322852[] = {{&S52991[43], 7}, {NULL}};
const Segment SGI322853[] = {{&S52991[36], 14}, {NULL}};
const Segment SGI322854[] = {{&S52991[75], 3}, {NULL}};
const Segment SGI322855[] = {{&S52992[30], 4}, {NULL}};
const Segment SGI322856[] = {{&S52992[8], 3}, {NULL}};
const Segment SGI322857[] = {{&S52992[13], 10}, {NULL}};
const Segment SGI322858[] = {{&S52992[43], 7}, {NULL}};
const Segment SGI322859[] = {{&S52992[36], 14}, {NULL}};
const Segment SGI322860[] = {{&S52992[52], 6}, {NULL}};
const Segment SGI322861[] = {{&S52992[82], 3}, {NULL}};
const Segment SGI322862[] = {{&S52993[30], 3}, {NULL}};
const Segment SGI322863[] = {{&S52993[8], 3}, {NULL}};
const Segment SGI322864[] = {{&S52993[13], 10}, {NULL}};
const Segment SGI322865[] = {{&S52993[62], 7}, {NULL}};
const Segment SGI322866[] = {{&S52993[55], 14}, {NULL}};
const Segment SGI322867[] = {{&S52993[71], 6}, {NULL}};
const Segment SGI322868[] = {{&S52993[101], 3}, {NULL}};
const Segment SGI322869[] = {{&S69920[3], 5}, {NULL}};
const Segment SGI322870[] = {{&S53111[30], 4}, {NULL}};
const Segment SGI322871[] = {{&S53111[8], 3}, {NULL}};
const Segment SGI322872[] = {{&S53111[13], 10}, {NULL}};
const Segment SGI322873[] = {{&S53111[43], 7}, {NULL}};
const Segment SGI322874[] = {{&S53111[36], 14}, {NULL}};
const Segment SGI322875[] = {{&S53111[52], 6}, {NULL}};
const Segment SGI322876[] = {{&S53112[30], 4}, {NULL}};
const Segment SGI322877[] = {{&S53112[8], 3}, {NULL}};
const Segment SGI322878[] = {{&S53112[13], 10}, {NULL}};
const Segment SGI322879[] = {{&S53112[43], 7}, {NULL}};
const Segment SGI322880[] = {{&S53112[36], 14}, {NULL}};
const Segment SGI322881[] = {{&S53113[30], 4}, {NULL}};
const Segment SGI322882[] = {{&S53113[8], 3}, {NULL}};
const Segment SGI322883[] = {{&S53113[13], 10}, {NULL}};
const Segment SGI322884[] = {{&S53113[43], 7}, {NULL}};
const Segment SGI322885[] = {{&S53113[36], 14}, {NULL}};
const Segment SGI322886[] = {{&S53113[52], 3}, {NULL}};
const Segment SGI322887[] = {{&S53114[30], 4}, {NULL}};
const Segment SGI322888[] = {{&S53114[8], 3}, {NULL}};
const Segment SGI322889[] = {{&S53114[13], 10}, {NULL}};
const Segment SGI322890[] = {{&S53114[43], 7}, {NULL}};
const Segment SGI322891[] = {{&S53114[36], 14}, {NULL}};
const Segment SGI322892[] = {{&S53114[132], 15}, {NULL}};
const Segment SGI322893[] = {{&S53115[30], 4}, {NULL}};
const Segment SGI322894[] = {{&S53115[8], 3}, {NULL}};
const Segment SGI322895[] = {{&S53115[13], 10}, {NULL}};
const Segment SGI322896[] = {{&S53115[43], 7}, {NULL}};
const Segment SGI322897[] = {{&S53115[36], 14}, {NULL}};
const Segment SGI322898[] = {{&S53115[139], 15}, {NULL}};
const Segment SGI322899[] = {{&S53115[52], 6}, {NULL}};
const Segment SGI322900[] = {{&S53179[30], 4}, {NULL}};
const Segment SGI322901[] = {{&S53179[8], 3}, {NULL}};
const Segment SGI322902[] = {{&S53179[13], 10}, {NULL}};
const Segment SGI322903[] = {{&S53179[43], 7}, {NULL}};
const Segment SGI322904[] = {{&S53179[36], 14}, {NULL}};
const Segment SGI322905[] = {{&S53179[52], 6}, {NULL}};
const Segment SGI322906[] = {{&S53179[72], 7}, {NULL}};
const Segment SGI322907[] = {{&S53179[79], 4}, {NULL}};
const Segment SGI322908[] = {{&S53180[30], 3}, {NULL}};
const Segment SGI322909[] = {{&S53180[8], 3}, {NULL}};
const Segment SGI322910[] = {{&S53180[13], 10}, {NULL}};
const Segment SGI322911[] = {{&S53180[42], 7}, {NULL}};
const Segment SGI322912[] = {{&S53180[35], 14}, {NULL}};
const Segment SGI322913[] = {{&S53180[51], 6}, {NULL}};
const Segment SGI322914[] = {{&S53180[71], 7}, {NULL}};
const Segment SGI322915[] = {{&S53180[78], 4}, {NULL}};
const Segment SGI322916[] = {{&S53181[30], 3}, {NULL}};
const Segment SGI322917[] = {{&S53181[8], 3}, {NULL}};
const Segment SGI322918[] = {{&S53181[13], 10}, {NULL}};
const Segment SGI322919[] = {{&S53181[42], 7}, {NULL}};
const Segment SGI322920[] = {{&S53181[35], 14}, {NULL}};
const Segment SGI322921[] = {{&S53181[64], 7}, {NULL}};
const Segment SGI322922[] = {{&S53181[71], 4}, {NULL}};
const Segment SGI322923[] = {{&S53182[30], 4}, {NULL}};
const Segment SGI322924[] = {{&S53182[8], 3}, {NULL}};
const Segment SGI322925[] = {{&S53182[13], 10}, {NULL}};
const Segment SGI322926[] = {{&S53182[43], 7}, {NULL}};
const Segment SGI322927[] = {{&S53182[36], 14}, {NULL}};
const Segment SGI322928[] = {{&S53182[65], 7}, {NULL}};
const Segment SGI322929[] = {{&S53182[72], 4}, {NULL}};
const Segment SGI322930[] = {{&S53183[30], 3}, {NULL}};
const Segment SGI322931[] = {{&S53183[8], 3}, {NULL}};
const Segment SGI322932[] = {{&S53183[13], 10}, {NULL}};
const Segment SGI322933[] = {{&S53183[42], 7}, {NULL}};
const Segment SGI322934[] = {{&S53183[35], 14}, {NULL}};
const Segment SGI322935[] = {{&S53183[138], 15}, {NULL}};
const Segment SGI322936[] = {{&S53183[51], 6}, {NULL}};
const Segment SGI322937[] = {{&S53183[179], 7}, {NULL}};
const Segment SGI322938[] = {{&S53183[7], 4}, {NULL}};
const Segment SGI322939[] = {{&S53184[30], 3}, {NULL}};
const Segment SGI322940[] = {{&S53184[8], 3}, {NULL}};
const Segment SGI322941[] = {{&S53184[13], 10}, {NULL}};
const Segment SGI322942[] = {{&S53184[42], 7}, {NULL}};
const Segment SGI322943[] = {{&S53184[35], 14}, {NULL}};
const Segment SGI322944[] = {{&S53184[131], 15}, {NULL}};
const Segment SGI322945[] = {{&S53184[162], 7}, {NULL}};
const Segment SGI322946[] = {{&S53184[7], 4}, {NULL}};
const Segment SGI322947[] = {{&S53314[30], 3}, {NULL}};
const Segment SGI322948[] = {{&S53314[8], 3}, {NULL}};
const Segment SGI322949[] = {{&S53314[13], 10}, {NULL}};
const Segment SGI322950[] = {{&S53314[42], 7}, {NULL}};
const Segment SGI322951[] = {{&S53314[35], 14}, {NULL}};
const Segment SGI322952[] = {{&S53314[80], 5}, {NULL}};
const Segment SGI322953[] = {{&S53314[51], 6}, {NULL}};
const Segment SGI322954[] = {{&S53315[30], 3}, {NULL}};
const Segment SGI322955[] = {{&S53315[8], 3}, {NULL}};
const Segment SGI322956[] = {{&S53315[13], 10}, {NULL}};
const Segment SGI322957[] = {{&S53315[42], 7}, {NULL}};
const Segment SGI322958[] = {{&S53315[35], 14}, {NULL}};
const Segment SGI322959[] = {{&S53315[73], 5}, {NULL}};
const Segment SGI322960[] = {{&S53316[30], 3}, {NULL}};
const Segment SGI322961[] = {{&S53316[8], 3}, {NULL}};
const Segment SGI322962[] = {{&S53316[13], 10}, {NULL}};
const Segment SGI322963[] = {{&S53316[42], 7}, {NULL}};
const Segment SGI322964[] = {{&S53316[35], 14}, {NULL}};
const Segment SGI322965[] = {{&S53316[80], 5}, {NULL}};
const Segment SGI322966[] = {{&S53316[51], 6}, {NULL}};
const Segment SGI322967[] = {{&S53317[30], 3}, {NULL}};
const Segment SGI322968[] = {{&S53317[8], 3}, {NULL}};
const Segment SGI322969[] = {{&S53317[13], 10}, {NULL}};
const Segment SGI322970[] = {{&S53317[42], 7}, {NULL}};
const Segment SGI322971[] = {{&S53317[35], 14}, {NULL}};
const Segment SGI322972[] = {{&S53317[80], 5}, {NULL}};
const Segment SGI322973[] = {{&S53317[51], 6}, {NULL}};
const Segment SGI322974[] = {{&S53318[30], 3}, {NULL}};
const Segment SGI322975[] = {{&S53318[8], 3}, {NULL}};
const Segment SGI322976[] = {{&S53318[13], 10}, {NULL}};
const Segment SGI322977[] = {{&S53318[42], 7}, {NULL}};
const Segment SGI322978[] = {{&S53318[35], 14}, {NULL}};
const Segment SGI322979[] = {{&S53318[73], 5}, {NULL}};
const Segment SGI322980[] = {{&S53319[30], 3}, {NULL}};
const Segment SGI322981[] = {{&S53319[8], 3}, {NULL}};
const Segment SGI322982[] = {{&S53319[13], 10}, {NULL}};
const Segment SGI322983[] = {{&S53319[42], 7}, {NULL}};
const Segment SGI322984[] = {{&S53319[35], 14}, {NULL}};
const Segment SGI322985[] = {{&S53319[73], 5}, {NULL}};
const Segment SGI322986[] = {{&S53320[30], 3}, {NULL}};
const Segment SGI322987[] = {{&S53320[8], 3}, {NULL}};
const Segment SGI322988[] = {{&S53320[13], 10}, {NULL}};
const Segment SGI322989[] = {{&S53320[42], 7}, {NULL}};
const Segment SGI322990[] = {{&S53320[35], 14}, {NULL}};
const Segment SGI322991[] = {{&S53320[80], 5}, {NULL}};
const Segment SGI322992[] = {{&S53320[51], 6}, {NULL}};
const Segment SGI322993[] = {{&S53321[30], 3}, {NULL}};
const Segment SGI322994[] = {{&S53321[8], 3}, {NULL}};
const Segment SGI322995[] = {{&S53321[13], 10}, {NULL}};
const Segment SGI322996[] = {{&S53321[42], 7}, {NULL}};
const Segment SGI322997[] = {{&S53321[35], 14}, {NULL}};
const Segment SGI322998[] = {{&S53321[80], 5}, {NULL}};
const Segment SGI322999[] = {{&S53321[51], 6}, {NULL}};
const Segment SGI323000[] = {{&S53324[30], 3}, {NULL}};
const Segment SGI323001[] = {{&S53324[8], 3}, {NULL}};
const Segment SGI323002[] = {{&S53324[13], 10}, {NULL}};
const Segment SGI323003[] = {{&S53324[42], 7}, {NULL}};
const Segment SGI323004[] = {{&S53324[35], 14}, {NULL}};
const Segment SGI323005[] = {{&S53324[73], 5}, {NULL}};
const Segment SGI323006[] = {{&S53325[30], 3}, {NULL}};
const Segment SGI323007[] = {{&S53325[8], 3}, {NULL}};
const Segment SGI323008[] = {{&S53325[13], 10}, {NULL}};
const Segment SGI323009[] = {{&S53325[42], 7}, {NULL}};
const Segment SGI323010[] = {{&S53325[35], 14}, {NULL}};
const Segment SGI323011[] = {{&S53325[73], 5}, {NULL}};
const Segment SGI323012[] = {{&S53361[30], 4}, {NULL}};
const Segment SGI323013[] = {{&S53361[8], 3}, {NULL}};
const Segment SGI323014[] = {{&S53361[13], 10}, {NULL}};
const Segment SGI323015[] = {{&S53361[53], 15}, {NULL}};
const Segment SGI323016[] = {{&S53568[30], 3}, {NULL}};
const Segment SGI323017[] = {{&S53568[8], 3}, {NULL}};
const Segment SGI323018[] = {{&S53568[13], 10}, {NULL}};
const Segment SGI323019[] = {{&S53568[42], 7}, {NULL}};
const Segment SGI323020[] = {{&S53568[35], 14}, {NULL}};
const Segment SGI323021[] = {{&S53568[64], 11}, {NULL}};
const Segment SGI323022[] = {{&S53568[64], 16}, {NULL}};
const Segment SGI323023[] = {{&S53569[30], 3}, {NULL}};
const Segment SGI323024[] = {{&S53569[8], 3}, {NULL}};
const Segment SGI323025[] = {{&S53569[13], 10}, {NULL}};
const Segment SGI323026[] = {{&S53569[42], 7}, {NULL}};
const Segment SGI323027[] = {{&S53569[35], 14}, {NULL}};
const Segment SGI323028[] = {{&S53569[71], 11}, {NULL}};
const Segment SGI323029[] = {{&S53569[51], 6}, {NULL}};
const Segment SGI323030[] = {{&S53569[71], 16}, {NULL}};
const Segment SGI323031[] = {{&S53572[30], 3}, {NULL}};
const Segment SGI323032[] = {{&S53572[8], 3}, {NULL}};
const Segment SGI323033[] = {{&S53572[13], 10}, {NULL}};
const Segment SGI323034[] = {{&S53572[42], 7}, {NULL}};
const Segment SGI323035[] = {{&S53572[35], 14}, {NULL}};
const Segment SGI323036[] = {{&S53572[51], 11}, {NULL}};
const Segment SGI323037[] = {{&S53572[104], 15}, {NULL}};
const Segment SGI323038[] = {{&S53572[51], 16}, {NULL}};
const Segment SGI323039[] = {{&S53573[30], 3}, {NULL}};
const Segment SGI323040[] = {{&S53573[8], 3}, {NULL}};
const Segment SGI323041[] = {{&S53573[13], 10}, {NULL}};
const Segment SGI323042[] = {{&S53573[42], 7}, {NULL}};
const Segment SGI323043[] = {{&S53573[35], 14}, {NULL}};
const Segment SGI323044[] = {{&S53573[58], 11}, {NULL}};
const Segment SGI323045[] = {{&S53573[111], 15}, {NULL}};
const Segment SGI323046[] = {{&S53573[51], 6}, {NULL}};
const Segment SGI323047[] = {{&S53573[58], 16}, {NULL}};
const Segment SGI323048[] = {{&S53575[30], 3}, {NULL}};
const Segment SGI323049[] = {{&S53575[8], 3}, {NULL}};
const Segment SGI323050[] = {{&S53575[13], 10}, {NULL}};
const Segment SGI323051[] = {{&S53575[42], 7}, {NULL}};
const Segment SGI323052[] = {{&S53575[35], 14}, {NULL}};
const Segment SGI323053[] = {{&S53575[72], 11}, {NULL}};
const Segment SGI323054[] = {{&S53575[72], 16}, {NULL}};
const Segment SGI323055[] = {{&S53576[30], 3}, {NULL}};
const Segment SGI323056[] = {{&S53576[8], 3}, {NULL}};
const Segment SGI323057[] = {{&S53576[13], 10}, {NULL}};
const Segment SGI323058[] = {{&S53576[42], 7}, {NULL}};
const Segment SGI323059[] = {{&S53576[35], 14}, {NULL}};
const Segment SGI323060[] = {{&S53576[51], 11}, {NULL}};
const Segment SGI323061[] = {{&S53576[51], 16}, {NULL}};
const Segment SGI323062[] = {{&S53577[30], 3}, {NULL}};
const Segment SGI323063[] = {{&S53577[8], 3}, {NULL}};
const Segment SGI323064[] = {{&S53577[13], 10}, {NULL}};
const Segment SGI323065[] = {{&S53577[42], 7}, {NULL}};
const Segment SGI323066[] = {{&S53577[35], 14}, {NULL}};
const Segment SGI323067[] = {{&S53577[58], 11}, {NULL}};
const Segment SGI323068[] = {{&S53577[51], 6}, {NULL}};
const Segment SGI323069[] = {{&S53577[58], 16}, {NULL}};
const Segment SGI323070[] = {{&S70583[3], 5}, {NULL}};
const Segment SGI323071[] = {{&S70583[3], 12}, {NULL}};
const Segment SGI323072[] = {{&S70584[3], 5}, {NULL}};
const Segment SGI323073[] = {{&S70588[3], 5}, {NULL}};
const Segment SGI323074[] = {{&S70588[104], 13}, {NULL}};
const Segment SGI323075[] = {{&S53763[30], 3}, {NULL}};
const Segment SGI323076[] = {{&S53763[8], 3}, {NULL}};
const Segment SGI323077[] = {{&S53763[13], 10}, {NULL}};
const Segment SGI323078[] = {{&S53763[42], 7}, {NULL}};
const Segment SGI323079[] = {{&S53763[35], 14}, {NULL}};
const Segment SGI323080[] = {{&S53763[106], 15}, {NULL}};
const Segment SGI323081[] = {{&S53763[64], 13}, {NULL}};
const Segment SGI323082[] = {{&S53764[30], 3}, {NULL}};
const Segment SGI323083[] = {{&S53764[8], 3}, {NULL}};
const Segment SGI323084[] = {{&S53764[13], 10}, {NULL}};
const Segment SGI323085[] = {{&S53764[55], 7}, {NULL}};
const Segment SGI323086[] = {{&S53764[48], 14}, {NULL}};
const Segment SGI323087[] = {{&S53764[64], 13}, {NULL}};
const Segment SGI323088[] = {{&S53765[30], 3}, {NULL}};
const Segment SGI323089[] = {{&S53765[8], 3}, {NULL}};
const Segment SGI323090[] = {{&S53765[13], 10}, {NULL}};
const Segment SGI323091[] = {{&S53765[55], 7}, {NULL}};
const Segment SGI323092[] = {{&S53765[48], 14}, {NULL}};
const Segment SGI323093[] = {{&S53765[64], 13}, {NULL}};
const Segment SGI323094[] = {{&S53766[30], 3}, {NULL}};
const Segment SGI323095[] = {{&S53766[8], 3}, {NULL}};
const Segment SGI323096[] = {{&S53766[13], 10}, {NULL}};
const Segment SGI323097[] = {{&S53766[55], 7}, {NULL}};
const Segment SGI323098[] = {{&S53766[48], 14}, {NULL}};
const Segment SGI323099[] = {{&S53766[64], 13}, {NULL}};
const Segment SGI323100[] = {{&S53766[79], 3}, {NULL}};
const Segment SGI323101[] = {{&S53767[30], 3}, {NULL}};
const Segment SGI323102[] = {{&S53767[8], 3}, {NULL}};
const Segment SGI323103[] = {{&S53767[13], 10}, {NULL}};
const Segment SGI323104[] = {{&S53767[55], 7}, {NULL}};
const Segment SGI323105[] = {{&S53767[48], 14}, {NULL}};
const Segment SGI323106[] = {{&S53767[64], 13}, {&S53767[64], 13}, {NULL}};
const Segment SGI323107[] = {{&S53768[30], 3}, {NULL}};
const Segment SGI323108[] = {{&S53768[8], 3}, {NULL}};
const Segment SGI323109[] = {{&S53768[13], 10}, {NULL}};
const Segment SGI323110[] = {{&S53768[55], 7}, {NULL}};
const Segment SGI323111[] = {{&S53768[48], 14}, {NULL}};
const Segment SGI323112[] = {{&S53768[69], 13}, {NULL}};
const Segment SGI323113[] = {{&S53768[64], 3}, {NULL}};
const Segment SGI323114[] = {{&S53769[30], 3}, {NULL}};
const Segment SGI323115[] = {{&S53769[8], 3}, {NULL}};
const Segment SGI323116[] = {{&S53769[13], 10}, {NULL}};
const Segment SGI323117[] = {{&S53769[55], 7}, {NULL}};
const Segment SGI323118[] = {{&S53769[48], 14}, {NULL}};
const Segment SGI323119[] = {{&S53769[64], 13}, {&S53769[64], 13}, {NULL}};
const Segment SGI323120[] = {{&S70712[3], 5}, {NULL}};
const Segment SGI323121[] = {{&S70712[3], 12}, {NULL}};
const Segment SGI323122[] = {{&S70713[3], 5}, {NULL}};
const Segment SGI323123[] = {{&S70714[3], 6}, {NULL}};
const Segment SGI323124[] = {{&S57310[4], 14}, {NULL}};
const Segment SGI323125[] = {{&S57310[131], 8}, {NULL}};
const Segment SGI323126[] = {{&S57311[4], 14}, {NULL}};
const Segment SGI323127[] = {{&S57311[131], 8}, {NULL}};
const Segment SGI323128[] = {{&S57312[4], 14}, {NULL}};
const Segment SGI323129[] = {{&S57312[131], 15}, {NULL}};
const Segment SGI323130[] = {{&S57313[4], 14}, {NULL}};
const Segment SGI323131[] = {{&S57313[131], 15}, {NULL}};
const Segment SGI323132[] = {{&S53987[30], 3}, {NULL}};
const Segment SGI323133[] = {{&S53987[8], 3}, {NULL}};
const Segment SGI323134[] = {{&S53987[13], 10}, {NULL}};
const Segment SGI323135[] = {{&S53987[42], 7}, {NULL}};
const Segment SGI323136[] = {{&S53988[30], 3}, {NULL}};
const Segment SGI323137[] = {{&S53988[8], 3}, {NULL}};
const Segment SGI323138[] = {{&S53988[13], 10}, {NULL}};
const Segment SGI323139[] = {{&S53988[42], 7}, {NULL}};
const Segment SGI323140[] = {{&S53988[35], 14}, {NULL}};
const Segment SGI323141[] = {{&S53988[51], 3}, {NULL}};
const Segment SGI323142[] = {{&S53989[30], 3}, {NULL}};
const Segment SGI323143[] = {{&S53989[8], 3}, {NULL}};
const Segment SGI323144[] = {{&S53989[13], 10}, {NULL}};
const Segment SGI323145[] = {{&S53989[42], 9}, {NULL}};
const Segment SGI323146[] = {{&S53990[30], 3}, {NULL}};
const Segment SGI323147[] = {{&S53990[8], 3}, {NULL}};
const Segment SGI323148[] = {{&S53990[13], 10}, {NULL}};
const Segment SGI323149[] = {{&S53990[42], 7}, {NULL}};
const Segment SGI323150[] = {{&S53990[35], 14}, {NULL}};
const Segment SGI323151[] = {{&S53991[30], 3}, {NULL}};
const Segment SGI323152[] = {{&S53991[8], 3}, {NULL}};
const Segment SGI323153[] = {{&S53991[13], 10}, {NULL}};
const Segment SGI323154[] = {{&S53991[42], 7}, {NULL}};
const Segment SGI323155[] = {{&S53991[35], 14}, {NULL}};
const Segment SGI323156[] = {{&S53992[30], 3}, {NULL}};
const Segment SGI323157[] = {{&S53992[8], 3}, {NULL}};
const Segment SGI323158[] = {{&S53992[13], 10}, {NULL}};
const Segment SGI323159[] = {{&S53992[42], 7}, {NULL}};
const Segment SGI323160[] = {{&S53992[35], 14}, {NULL}};
const Segment SGI323161[] = {{&S53992[51], 3}, {NULL}};
const Segment SGI323162[] = {{&S53993[30], 3}, {NULL}};
const Segment SGI323163[] = {{&S53993[8], 3}, {NULL}};
const Segment SGI323164[] = {{&S53993[13], 10}, {NULL}};
const Segment SGI323165[] = {{&S53993[42], 7}, {NULL}};
const Segment SGI323166[] = {{&S53993[35], 14}, {NULL}};
const Segment SGI323167[] = {{&S53993[100], 6}, {NULL}};
const Segment SGI323168[] = {{&S53994[30], 3}, {&S53994[69], 3}, {NULL}};
const Segment SGI323169[] = {{&S53994[8], 3}, {NULL}};
const Segment SGI323170[] = {{&S53994[13], 10}, {NULL}};
const Segment SGI323171[] = {{&S53994[42], 7}, {&S53994[81], 7}, {NULL}};
const Segment SGI323172[] = {{&S53994[35], 14}, {&S53994[74], 14}, {NULL}};
const Segment SGI323173[] = {{&S67458[0], 117}, {NULL}};
const Segment SGI323174[] = {{&S67462[0], 117}, {NULL}};
const Segment SGI323175[] = {{&S71171[3], 5}, {NULL}};
const Segment SGI323176[] = {{&S71171[3], 14}, {NULL}};
const Segment SGI323177[] = {{&S71172[3], 5}, {NULL}};
const Segment SGI323178[] = {{&S71173[3], 5}, {NULL}};
const Segment SGI323179[] = {{&S71173[3], 12}, {NULL}};
const Segment SGI323180[] = {{&S54377[30], 3}, {NULL}};
const Segment SGI323181[] = {{&S54377[8], 3}, {NULL}};
const Segment SGI323182[] = {{&S54377[13], 10}, {NULL}};
const Segment SGI323183[] = {{&S54377[42], 7}, {NULL}};
const Segment SGI323184[] = {{&S54377[35], 14}, {NULL}};
const Segment SGI323185[] = {{&S54377[51], 6}, {NULL}};
const Segment SGI323186[] = {{&S54377[151], 13}, {NULL}};
const Segment SGI323187[] = {{&S54378[30], 3}, {NULL}};
const Segment SGI323188[] = {{&S54378[8], 3}, {NULL}};
const Segment SGI323189[] = {{&S54378[13], 10}, {NULL}};
const Segment SGI323190[] = {{&S54378[42], 7}, {NULL}};
const Segment SGI323191[] = {{&S54378[35], 14}, {NULL}};
const Segment SGI323192[] = {{&S54378[144], 13}, {NULL}};
const Segment SGI323193[] = {{&S54379[30], 3}, {NULL}};
const Segment SGI323194[] = {{&S54379[8], 3}, {NULL}};
const Segment SGI323195[] = {{&S54379[13], 10}, {NULL}};
const Segment SGI323196[] = {{&S54379[42], 7}, {NULL}};
const Segment SGI323197[] = {{&S54379[35], 14}, {NULL}};
const Segment SGI323198[] = {{&S54379[151], 15}, {NULL}};
const Segment SGI323199[] = {{&S54379[51], 6}, {NULL}};
const Segment SGI323200[] = {{&S54379[172], 13}, {NULL}};
const Segment SGI323201[] = {{&S54380[30], 3}, {NULL}};
const Segment SGI323202[] = {{&S54380[8], 3}, {NULL}};
const Segment SGI323203[] = {{&S54380[13], 10}, {NULL}};
const Segment SGI323204[] = {{&S54380[42], 7}, {NULL}};
const Segment SGI323205[] = {{&S54380[35], 14}, {NULL}};
const Segment SGI323206[] = {{&S54380[84], 15}, {NULL}};
const Segment SGI323207[] = {{&S54380[165], 13}, {NULL}};
const Segment SGI323208[] = {{&S54381[30], 3}, {NULL}};
const Segment SGI323209[] = {{&S54381[8], 3}, {NULL}};
const Segment SGI323210[] = {{&S54381[13], 10}, {NULL}};
const Segment SGI323211[] = {{&S54381[42], 7}, {NULL}};
const Segment SGI323212[] = {{&S54381[35], 14}, {NULL}};
const Segment SGI323213[] = {{&S54381[51], 6}, {NULL}};
const Segment SGI323214[] = {{&S54381[166], 13}, {NULL}};
const Segment SGI323215[] = {{&S54381[151], 9}, {NULL}};
const Segment SGI323216[] = {{&S71221[3], 5}, {NULL}};
const Segment SGI323217[] = {{&S71222[3], 6}, {NULL}};
const Segment SGI323218[] = {{&S54426[30], 3}, {NULL}};
const Segment SGI323219[] = {{&S54426[8], 3}, {NULL}};
const Segment SGI323220[] = {{&S54426[13], 10}, {NULL}};
const Segment SGI323221[] = {{&S54426[42], 7}, {NULL}};
const Segment SGI323222[] = {{&S54426[35], 14}, {NULL}};
const Segment SGI323223[] = {{&S54426[74], 4}, {NULL}};
const Segment SGI323224[] = {{&S54427[30], 3}, {NULL}};
const Segment SGI323225[] = {{&S54427[8], 3}, {NULL}};
const Segment SGI323226[] = {{&S54427[13], 10}, {NULL}};
const Segment SGI323227[] = {{&S54427[42], 7}, {NULL}};
const Segment SGI323228[] = {{&S54427[35], 14}, {NULL}};
const Segment SGI323229[] = {{&S54427[51], 3}, {NULL}};
const Segment SGI323230[] = {{&S54427[66], 4}, {NULL}};
const Segment SGI323231[] = {{&S71405[3], 4}, {NULL}};
const Segment SGI323232[] = {{&S54780[30], 3}, {NULL}};
const Segment SGI323233[] = {{&S54780[8], 3}, {NULL}};
const Segment SGI323234[] = {{&S54780[13], 10}, {NULL}};
const Segment SGI323235[] = {{&S54780[55], 7}, {NULL}};
const Segment SGI323236[] = {{&S54780[48], 14}, {NULL}};
const Segment SGI323237[] = {{&S54780[64], 13}, {NULL}};
const Segment SGI323238[] = {{&S54781[30], 3}, {NULL}};
const Segment SGI323239[] = {{&S54781[8], 3}, {NULL}};
const Segment SGI323240[] = {{&S54781[13], 10}, {NULL}};
const Segment SGI323241[] = {{&S54781[55], 7}, {NULL}};
const Segment SGI323242[] = {{&S54781[48], 14}, {NULL}};
const Segment SGI323243[] = {{&S54781[64], 13}, {NULL}};
const Segment SGI323244[] = {{&S54782[30], 3}, {NULL}};
const Segment SGI323245[] = {{&S54782[8], 3}, {NULL}};
const Segment SGI323246[] = {{&S54782[13], 10}, {NULL}};
const Segment SGI323247[] = {{&S54782[55], 7}, {NULL}};
const Segment SGI323248[] = {{&S54782[48], 14}, {NULL}};
const Segment SGI323249[] = {{&S54782[64], 13}, {NULL}};
const Segment SGI323250[] = {{&S54782[79], 3}, {NULL}};
const Segment SGI323251[] = {{&S54783[30], 3}, {NULL}};
const Segment SGI323252[] = {{&S54783[8], 3}, {NULL}};
const Segment SGI323253[] = {{&S54783[13], 10}, {NULL}};
const Segment SGI323254[] = {{&S54783[55], 7}, {NULL}};
const Segment SGI323255[] = {{&S54783[48], 14}, {NULL}};
const Segment SGI323256[] = {{&S54783[64], 13}, {NULL}};
const Segment SGI323257[] = {{&S54783[79], 3}, {NULL}};
const Segment SGI323258[] = {{&S54784[30], 3}, {NULL}};
const Segment SGI323259[] = {{&S54784[8], 3}, {NULL}};
const Segment SGI323260[] = {{&S54784[13], 10}, {NULL}};
const Segment SGI323261[] = {{&S54784[55], 7}, {NULL}};
const Segment SGI323262[] = {{&S54784[48], 14}, {NULL}};
const Segment SGI323263[] = {{&S54784[64], 13}, {&S54784[64], 13}, {NULL}};
const Segment SGI323264[] = {{&S54785[30], 3}, {NULL}};
const Segment SGI323265[] = {{&S54785[8], 3}, {NULL}};
const Segment SGI323266[] = {{&S54785[13], 10}, {NULL}};
const Segment SGI323267[] = {{&S54785[55], 7}, {NULL}};
const Segment SGI323268[] = {{&S54785[48], 14}, {NULL}};
const Segment SGI323269[] = {{&S54785[64], 13}, {&S54785[64], 13}, {NULL}};
const Segment SGI323270[] = {{&S82518[3], 5}, {NULL}};
const Segment SGI323271[] = {{&S82518[28], 8}, {NULL}};
const Segment SGI323272[] = {{&S54811[33], 3}, {NULL}};
const Segment SGI323273[] = {{&S54811[45], 7}, {NULL}};
const Segment SGI323274[] = {{&S54811[38], 14}, {NULL}};
const Segment SGI323275[] = {{&S54816[30], 3}, {NULL}};
const Segment SGI323276[] = {{&S54816[8], 3}, {NULL}};
const Segment SGI323277[] = {{&S54816[13], 10}, {NULL}};
const Segment SGI323278[] = {{&S54817[8], 3}, {NULL}};
const Segment SGI323279[] = {{&S54817[13], 10}, {NULL}};
const Segment SGI323280[] = {{&S54818[8], 3}, {NULL}};
const Segment SGI323281[] = {{&S54818[13], 10}, {NULL}};
const Segment SGI323282[] = {{&S54818[35], 3}, {NULL}};
const Segment SGI323283[] = {{&S54819[8], 3}, {NULL}};
const Segment SGI323284[] = {{&S54819[13], 10}, {NULL}};
const Segment SGI323285[] = {{&S54819[35], 3}, {NULL}};
const Segment SGI323286[] = {{&S54819[40], 3}, {NULL}};
const Segment SGI323287[] = {{&S71895[3], 4}, {NULL}};
const Segment SGI323288[] = {{&S71933[3], 5}, {NULL}};
const Segment SGI323289[] = {{&S55147[30], 3}, {NULL}};
const Segment SGI323290[] = {{&S55147[8], 3}, {NULL}};
const Segment SGI323291[] = {{&S55147[13], 10}, {NULL}};
const Segment SGI323292[] = {{&S55147[42], 7}, {NULL}};
const Segment SGI323293[] = {{&S55147[35], 14}, {NULL}};
const Segment SGI323294[] = {{&S55147[73], 5}, {NULL}};
const Segment SGI323295[] = {{&S55148[30], 3}, {NULL}};
const Segment SGI323296[] = {{&S55148[8], 3}, {NULL}};
const Segment SGI323297[] = {{&S55148[13], 10}, {NULL}};
const Segment SGI323298[] = {{&S55148[42], 7}, {NULL}};
const Segment SGI323299[] = {{&S55148[35], 14}, {NULL}};
const Segment SGI323300[] = {{&S55148[73], 5}, {NULL}};
const Segment SGI323301[] = {{&S55149[30], 3}, {NULL}};
const Segment SGI323302[] = {{&S55149[8], 3}, {NULL}};
const Segment SGI323303[] = {{&S55149[13], 10}, {NULL}};
const Segment SGI323304[] = {{&S55149[42], 7}, {NULL}};
const Segment SGI323305[] = {{&S55149[35], 14}, {NULL}};
const Segment SGI323306[] = {{&S55149[73], 5}, {NULL}};
const Segment SGI323307[] = {{&S55150[30], 3}, {NULL}};
const Segment SGI323308[] = {{&S55150[8], 3}, {NULL}};
const Segment SGI323309[] = {{&S55150[13], 10}, {NULL}};
const Segment SGI323310[] = {{&S55150[42], 7}, {NULL}};
const Segment SGI323311[] = {{&S55150[35], 14}, {NULL}};
const Segment SGI323312[] = {{&S55150[73], 5}, {NULL}};
const Segment SGI323313[] = {{&S55151[30], 3}, {NULL}};
const Segment SGI323314[] = {{&S55151[8], 3}, {NULL}};
const Segment SGI323315[] = {{&S55151[13], 10}, {NULL}};
const Segment SGI323316[] = {{&S55151[42], 7}, {NULL}};
const Segment SGI323317[] = {{&S55151[35], 14}, {NULL}};
const Segment SGI323318[] = {{&S55151[91], 15}, {NULL}};
const Segment SGI323319[] = {{&S55151[171], 5}, {NULL}};
const Segment SGI323320[] = {{&S55152[30], 3}, {NULL}};
const Segment SGI323321[] = {{&S55152[8], 3}, {NULL}};
const Segment SGI323322[] = {{&S55152[13], 10}, {NULL}};
const Segment SGI323323[] = {{&S55152[42], 7}, {NULL}};
const Segment SGI323324[] = {{&S55152[35], 14}, {NULL}};
const Segment SGI323325[] = {{&S55152[73], 6}, {NULL}};
const Segment SGI323326[] = {{&S55153[30], 3}, {NULL}};
const Segment SGI323327[] = {{&S55153[8], 3}, {NULL}};
const Segment SGI323328[] = {{&S55153[13], 10}, {NULL}};
const Segment SGI323329[] = {{&S55153[42], 7}, {NULL}};
const Segment SGI323330[] = {{&S55153[35], 14}, {NULL}};
const Segment SGI323331[] = {{&S55153[73], 5}, {NULL}};
const Segment SGI323332[] = {{&S55154[30], 3}, {NULL}};
const Segment SGI323333[] = {{&S55154[8], 3}, {NULL}};
const Segment SGI323334[] = {{&S55154[13], 10}, {NULL}};
const Segment SGI323335[] = {{&S55154[42], 7}, {NULL}};
const Segment SGI323336[] = {{&S55154[35], 14}, {NULL}};
const Segment SGI323337[] = {{&S55154[80], 5}, {NULL}};
const Segment SGI323338[] = {{&S55154[51], 6}, {NULL}};
const Segment SGI323339[] = {{&S55155[30], 3}, {NULL}};
const Segment SGI323340[] = {{&S55155[8], 3}, {NULL}};
const Segment SGI323341[] = {{&S55155[13], 10}, {NULL}};
const Segment SGI323342[] = {{&S55155[42], 7}, {NULL}};
const Segment SGI323343[] = {{&S55155[35], 14}, {NULL}};
const Segment SGI323344[] = {{&S55155[91], 15}, {NULL}};
const Segment SGI323345[] = {{&S55155[171], 5}, {NULL}};
const Segment SGI323346[] = {{&S55156[30], 3}, {NULL}};
const Segment SGI323347[] = {{&S55156[8], 3}, {NULL}};
const Segment SGI323348[] = {{&S55156[13], 10}, {NULL}};
const Segment SGI323349[] = {{&S55156[42], 7}, {NULL}};
const Segment SGI323350[] = {{&S55156[35], 14}, {NULL}};
const Segment SGI323351[] = {{&S55156[80], 6}, {NULL}};
const Segment SGI323352[] = {{&S55156[51], 6}, {NULL}};
const Segment SGI323353[] = {{&S55157[30], 3}, {NULL}};
const Segment SGI323354[] = {{&S55157[8], 3}, {NULL}};
const Segment SGI323355[] = {{&S55157[13], 10}, {NULL}};
const Segment SGI323356[] = {{&S55157[42], 7}, {NULL}};
const Segment SGI323357[] = {{&S55157[35], 14}, {NULL}};
const Segment SGI323358[] = {{&S55157[80], 5}, {NULL}};
const Segment SGI323359[] = {{&S55157[51], 6}, {NULL}};
const Segment SGI323360[] = {{&S55158[30], 3}, {NULL}};
const Segment SGI323361[] = {{&S55158[8], 3}, {NULL}};
const Segment SGI323362[] = {{&S55158[13], 10}, {NULL}};
const Segment SGI323363[] = {{&S55158[42], 7}, {NULL}};
const Segment SGI323364[] = {{&S55158[35], 14}, {NULL}};
const Segment SGI323365[] = {{&S55158[80], 5}, {NULL}};
const Segment SGI323366[] = {{&S55158[51], 6}, {NULL}};
const Segment SGI323367[] = {{&S55159[30], 3}, {NULL}};
const Segment SGI323368[] = {{&S55159[8], 3}, {NULL}};
const Segment SGI323369[] = {{&S55159[13], 10}, {NULL}};
const Segment SGI323370[] = {{&S55159[42], 7}, {NULL}};
const Segment SGI323371[] = {{&S55159[35], 14}, {NULL}};
const Segment SGI323372[] = {{&S55159[73], 6}, {NULL}};
const Segment SGI323373[] = {{&S55160[30], 3}, {NULL}};
const Segment SGI323374[] = {{&S55160[8], 3}, {NULL}};
const Segment SGI323375[] = {{&S55160[13], 10}, {NULL}};
const Segment SGI323376[] = {{&S55160[42], 7}, {NULL}};
const Segment SGI323377[] = {{&S55160[35], 14}, {NULL}};
const Segment SGI323378[] = {{&S55160[73], 6}, {NULL}};
const Segment SGI323379[] = {{&S55161[30], 3}, {NULL}};
const Segment SGI323380[] = {{&S55161[8], 3}, {NULL}};
const Segment SGI323381[] = {{&S55161[13], 10}, {NULL}};
const Segment SGI323382[] = {{&S55161[42], 7}, {NULL}};
const Segment SGI323383[] = {{&S55161[35], 14}, {NULL}};
const Segment SGI323384[] = {{&S55161[73], 5}, {NULL}};
const Segment SGI323385[] = {{&S55162[30], 3}, {NULL}};
const Segment SGI323386[] = {{&S55162[8], 3}, {NULL}};
const Segment SGI323387[] = {{&S55162[13], 10}, {NULL}};
const Segment SGI323388[] = {{&S55162[42], 7}, {NULL}};
const Segment SGI323389[] = {{&S55162[35], 14}, {NULL}};
const Segment SGI323390[] = {{&S55162[80], 6}, {NULL}};
const Segment SGI323391[] = {{&S55162[51], 6}, {NULL}};
const Segment SGI323392[] = {{&S55163[30], 3}, {NULL}};
const Segment SGI323393[] = {{&S55163[8], 3}, {NULL}};
const Segment SGI323394[] = {{&S55163[13], 10}, {NULL}};
const Segment SGI323395[] = {{&S55163[42], 7}, {NULL}};
const Segment SGI323396[] = {{&S55163[35], 14}, {NULL}};
const Segment SGI323397[] = {{&S55163[73], 6}, {NULL}};
const Segment SGI323398[] = {{&S55164[30], 3}, {NULL}};
const Segment SGI323399[] = {{&S55164[8], 3}, {NULL}};
const Segment SGI323400[] = {{&S55164[13], 10}, {NULL}};
const Segment SGI323401[] = {{&S55164[42], 7}, {NULL}};
const Segment SGI323402[] = {{&S55164[35], 14}, {NULL}};
const Segment SGI323403[] = {{&S55164[80], 6}, {NULL}};
const Segment SGI323404[] = {{&S55164[51], 6}, {NULL}};
const Segment SGI323405[] = {{&S55165[30], 3}, {NULL}};
const Segment SGI323406[] = {{&S55165[8], 3}, {NULL}};
const Segment SGI323407[] = {{&S55165[13], 10}, {NULL}};
const Segment SGI323408[] = {{&S55165[42], 7}, {NULL}};
const Segment SGI323409[] = {{&S55165[35], 14}, {NULL}};
const Segment SGI323410[] = {{&S55165[80], 6}, {NULL}};
const Segment SGI323411[] = {{&S55165[51], 6}, {NULL}};
const Segment SGI323412[] = {{&S55166[30], 3}, {NULL}};
const Segment SGI323413[] = {{&S55166[8], 3}, {NULL}};
const Segment SGI323414[] = {{&S55166[13], 10}, {NULL}};
const Segment SGI323415[] = {{&S55166[42], 7}, {NULL}};
const Segment SGI323416[] = {{&S55166[35], 14}, {NULL}};
const Segment SGI323417[] = {{&S55166[150], 6}, {NULL}};
const Segment SGI323418[] = {{&S55167[30], 3}, {NULL}};
const Segment SGI323419[] = {{&S55167[8], 3}, {NULL}};
const Segment SGI323420[] = {{&S55167[13], 10}, {NULL}};
const Segment SGI323421[] = {{&S55167[42], 7}, {NULL}};
const Segment SGI323422[] = {{&S55167[35], 14}, {NULL}};
const Segment SGI323423[] = {{&S55167[80], 6}, {NULL}};
const Segment SGI323424[] = {{&S55167[51], 6}, {NULL}};
const Segment SGI323425[] = {{&S55168[30], 3}, {NULL}};
const Segment SGI323426[] = {{&S55168[8], 3}, {NULL}};
const Segment SGI323427[] = {{&S55168[13], 10}, {NULL}};
const Segment SGI323428[] = {{&S55168[42], 7}, {NULL}};
const Segment SGI323429[] = {{&S55168[35], 14}, {NULL}};
const Segment SGI323430[] = {{&S55168[73], 6}, {NULL}};
const Segment SGI323431[] = {{&S55169[30], 3}, {NULL}};
const Segment SGI323432[] = {{&S55169[8], 3}, {NULL}};
const Segment SGI323433[] = {{&S55169[13], 10}, {NULL}};
const Segment SGI323434[] = {{&S55169[42], 7}, {NULL}};
const Segment SGI323435[] = {{&S55169[35], 14}, {NULL}};
const Segment SGI323436[] = {{&S55169[147], 5}, {NULL}};
const Segment SGI323437[] = {{&S55169[51], 6}, {NULL}};
const Segment SGI323438[] = {{&S89701[7], 3}, {NULL}};
const Segment SGI323439[] = {{&S89701[0], 80}, {NULL}};
const Segment SGI323440[] = {{&S55170[30], 3}, {NULL}};
const Segment SGI323441[] = {{&S55170[8], 3}, {NULL}};
const Segment SGI323442[] = {{&S55170[13], 10}, {NULL}};
const Segment SGI323443[] = {{&S55170[42], 7}, {NULL}};
const Segment SGI323444[] = {{&S55170[35], 14}, {NULL}};
const Segment SGI323445[] = {{&S55170[73], 6}, {NULL}};
const Segment SGI323446[] = {{&S89702[7], 3}, {NULL}};
const Segment SGI323447[] = {{&S89702[0], 80}, {NULL}};
const Segment SGI323448[] = {{&S89703[7], 3}, {NULL}};
const Segment SGI323449[] = {{&S89703[0], 83}, {NULL}};
const Segment SGI323450[] = {{&S55171[30], 3}, {NULL}};
const Segment SGI323451[] = {{&S55171[8], 3}, {NULL}};
const Segment SGI323452[] = {{&S55171[13], 10}, {NULL}};
const Segment SGI323453[] = {{&S55171[42], 7}, {NULL}};
const Segment SGI323454[] = {{&S55171[35], 14}, {NULL}};
const Segment SGI323455[] = {{&S55171[80], 6}, {NULL}};
const Segment SGI323456[] = {{&S55171[51], 6}, {NULL}};
const Segment SGI323457[] = {{&S89704[7], 3}, {NULL}};
const Segment SGI323458[] = {{&S89704[0], 83}, {NULL}};
const Segment SGI323459[] = {{&S55172[30], 3}, {NULL}};
const Segment SGI323460[] = {{&S55172[8], 3}, {NULL}};
const Segment SGI323461[] = {{&S55172[13], 10}, {NULL}};
const Segment SGI323462[] = {{&S55172[42], 7}, {NULL}};
const Segment SGI323463[] = {{&S55172[35], 14}, {NULL}};
const Segment SGI323464[] = {{&S55172[73], 6}, {NULL}};
const Segment SGI323465[] = {{&S55173[30], 3}, {NULL}};
const Segment SGI323466[] = {{&S55173[8], 3}, {NULL}};
const Segment SGI323467[] = {{&S55173[13], 10}, {NULL}};
const Segment SGI323468[] = {{&S55173[42], 7}, {NULL}};
const Segment SGI323469[] = {{&S55173[35], 14}, {NULL}};
const Segment SGI323470[] = {{&S55173[91], 15}, {NULL}};
const Segment SGI323471[] = {{&S55173[171], 6}, {NULL}};
const Segment SGI323472[] = {{&S55174[30], 3}, {NULL}};
const Segment SGI323473[] = {{&S55174[8], 3}, {NULL}};
const Segment SGI323474[] = {{&S55174[13], 10}, {NULL}};
const Segment SGI323475[] = {{&S55174[42], 7}, {NULL}};
const Segment SGI323476[] = {{&S55174[35], 14}, {NULL}};
const Segment SGI323477[] = {{&S55174[80], 6}, {NULL}};
const Segment SGI323478[] = {{&S55174[51], 6}, {NULL}};
const Segment SGI323479[] = {{&S55175[30], 3}, {NULL}};
const Segment SGI323480[] = {{&S55175[8], 3}, {NULL}};
const Segment SGI323481[] = {{&S55175[13], 10}, {NULL}};
const Segment SGI323482[] = {{&S55175[42], 7}, {NULL}};
const Segment SGI323483[] = {{&S55175[35], 14}, {NULL}};
const Segment SGI323484[] = {{&S55175[73], 6}, {NULL}};
const Segment SGI323485[] = {{&S82885[3], 5}, {NULL}};
const Segment SGI323486[] = {{&S72051[3], 5}, {NULL}};
const Segment SGI323487[] = {{&S61697[4], 5}, {NULL}};
const Segment SGI323488[] = {{&S61697[25], 3}, {NULL}};
const Segment SGI323489[] = {{&S61737[4], 6}, {NULL}};
const Segment SGI323490[] = {{&S61737[26], 3}, {NULL}};
const Segment SGI323491[] = {{&S61738[4], 6}, {NULL}};
const Segment SGI323492[] = {{&S61738[26], 3}, {NULL}};
const Segment SGI323493[] = {{&S61739[4], 5}, {NULL}};
const Segment SGI323494[] = {{&S61739[25], 3}, {NULL}};
const Segment SGI323495[] = {{&S55413[30], 3}, {NULL}};
const Segment SGI323496[] = {{&S55413[8], 3}, {NULL}};
const Segment SGI323497[] = {{&S55413[13], 10}, {NULL}};
const Segment SGI323498[] = {{&S55413[55], 7}, {NULL}};
const Segment SGI323499[] = {{&S55413[48], 14}, {NULL}};
const Segment SGI323500[] = {{&S55413[64], 13}, {NULL}};
const Segment SGI323501[] = {{&S55414[30], 3}, {NULL}};
const Segment SGI323502[] = {{&S55414[8], 3}, {NULL}};
const Segment SGI323503[] = {{&S55414[13], 10}, {NULL}};
const Segment SGI323504[] = {{&S55414[55], 7}, {NULL}};
const Segment SGI323505[] = {{&S55414[48], 14}, {NULL}};
const Segment SGI323506[] = {{&S55414[64], 13}, {NULL}};
const Segment SGI323507[] = {{&S55415[30], 3}, {NULL}};
const Segment SGI323508[] = {{&S55415[8], 3}, {NULL}};
const Segment SGI323509[] = {{&S55415[13], 10}, {NULL}};
const Segment SGI323510[] = {{&S55415[55], 7}, {NULL}};
const Segment SGI323511[] = {{&S55415[48], 14}, {NULL}};
const Segment SGI323512[] = {{&S55415[69], 13}, {NULL}};
const Segment SGI323513[] = {{&S55415[64], 3}, {NULL}};
const Segment SGI323514[] = {{&S55416[30], 3}, {NULL}};
const Segment SGI323515[] = {{&S55416[8], 3}, {NULL}};
const Segment SGI323516[] = {{&S55416[13], 10}, {NULL}};
const Segment SGI323517[] = {{&S55416[55], 7}, {NULL}};
const Segment SGI323518[] = {{&S55416[48], 14}, {NULL}};
const Segment SGI323519[] = {{&S55416[69], 13}, {NULL}};
const Segment SGI323520[] = {{&S55416[64], 3}, {NULL}};
const Segment SGI323521[] = {{&S55417[30], 3}, {NULL}};
const Segment SGI323522[] = {{&S55417[8], 3}, {NULL}};
const Segment SGI323523[] = {{&S55417[13], 10}, {NULL}};
const Segment SGI323524[] = {{&S55417[55], 7}, {NULL}};
const Segment SGI323525[] = {{&S55417[48], 14}, {NULL}};
const Segment SGI323526[] = {{&S55417[64], 13}, {&S55417[64], 13}, {NULL}};
const Segment SGI323527[] = {{&S55418[30], 3}, {NULL}};
const Segment SGI323528[] = {{&S55418[8], 3}, {NULL}};
const Segment SGI323529[] = {{&S55418[13], 10}, {NULL}};
const Segment SGI323530[] = {{&S55418[55], 7}, {NULL}};
const Segment SGI323531[] = {{&S55418[48], 14}, {NULL}};
const Segment SGI323532[] = {{&S55418[64], 13}, {&S55418[64], 13}, {NULL}};
const Segment SGI323533[] = {{&S55475[30], 3}, {NULL}};
const Segment SGI323534[] = {{&S55475[8], 3}, {NULL}};
const Segment SGI323535[] = {{&S55475[13], 10}, {NULL}};
const Segment SGI323536[] = {{&S55475[55], 7}, {NULL}};
const Segment SGI323537[] = {{&S55475[48], 14}, {NULL}};
const Segment SGI323538[] = {{&S55475[64], 13}, {NULL}};
const Segment SGI323539[] = {{&S55476[30], 3}, {NULL}};
const Segment SGI323540[] = {{&S55476[8], 3}, {NULL}};
const Segment SGI323541[] = {{&S55476[13], 10}, {NULL}};
const Segment SGI323542[] = {{&S55476[55], 7}, {NULL}};
const Segment SGI323543[] = {{&S55476[48], 14}, {NULL}};
const Segment SGI323544[] = {{&S55476[64], 13}, {NULL}};
const Segment SGI323545[] = {{&S55477[30], 3}, {NULL}};
const Segment SGI323546[] = {{&S55477[8], 3}, {NULL}};
const Segment SGI323547[] = {{&S55477[13], 10}, {NULL}};
const Segment SGI323548[] = {{&S55477[55], 7}, {NULL}};
const Segment SGI323549[] = {{&S55477[48], 14}, {NULL}};
const Segment SGI323550[] = {{&S55477[64], 13}, {NULL}};
const Segment SGI323551[] = {{&S55477[79], 3}, {NULL}};
const Segment SGI323552[] = {{&S55478[30], 3}, {NULL}};
const Segment SGI323553[] = {{&S55478[8], 3}, {NULL}};
const Segment SGI323554[] = {{&S55478[13], 10}, {NULL}};
const Segment SGI323555[] = {{&S55478[55], 7}, {NULL}};
const Segment SGI323556[] = {{&S55478[48], 14}, {NULL}};
const Segment SGI323557[] = {{&S55478[64], 13}, {NULL}};
const Segment SGI323558[] = {{&S55478[79], 3}, {NULL}};
const Segment SGI323559[] = {{&S55479[30], 3}, {NULL}};
const Segment SGI323560[] = {{&S55479[8], 3}, {NULL}};
const Segment SGI323561[] = {{&S55479[13], 10}, {NULL}};
const Segment SGI323562[] = {{&S55479[55], 7}, {NULL}};
const Segment SGI323563[] = {{&S55479[48], 14}, {NULL}};
const Segment SGI323564[] = {{&S55479[64], 13}, {&S55479[64], 13}, {NULL}};
const Segment SGI323565[] = {{&S55480[30], 3}, {NULL}};
const Segment SGI323566[] = {{&S55480[8], 3}, {NULL}};
const Segment SGI323567[] = {{&S55480[13], 10}, {NULL}};
const Segment SGI323568[] = {{&S55480[55], 7}, {NULL}};
const Segment SGI323569[] = {{&S55480[48], 14}, {NULL}};
const Segment SGI323570[] = {{&S55480[64], 13}, {&S55480[64], 13}, {NULL}};
const Segment SGI323571[] = {{&S72402[3], 5}, {NULL}};
const Segment SGI323572[] = {{&S72402[20], 3}, {NULL}};
const Segment SGI323573[] = {{&S68637[0], 118}, {NULL}};
const Segment SGI323574[] = {{&S68638[0], 118}, {NULL}};
const Segment SGI323575[] = {{&S68639[0], 118}, {NULL}};
const Segment SGI323576[] = {{&S68640[0], 118}, {NULL}};
const Segment SGI323577[] = {{&S68641[0], 118}, {NULL}};
const Segment SGI323578[] = {{&S68649[0], 32}, {NULL}};
const Segment SGI323579[] = {{&S68654[0], 57}, {NULL}};
const Segment SGI323580[] = {{&S68659[0], 59}, {NULL}};
const Segment SGI323581[] = {{&S55679[30], 3}, {NULL}};
const Segment SGI323582[] = {{&S55679[8], 3}, {NULL}};
const Segment SGI323583[] = {{&S55679[13], 10}, {NULL}};
const Segment SGI323584[] = {{&S55679[42], 7}, {NULL}};
const Segment SGI323585[] = {{&S55679[35], 14}, {NULL}};
const Segment SGI323586[] = {{&S55679[64], 15}, {NULL}};
const Segment SGI323587[] = {{&S55680[30], 3}, {NULL}};
const Segment SGI323588[] = {{&S55680[8], 3}, {NULL}};
const Segment SGI323589[] = {{&S55680[13], 10}, {NULL}};
const Segment SGI323590[] = {{&S55680[42], 7}, {NULL}};
const Segment SGI323591[] = {{&S55680[35], 14}, {NULL}};
const Segment SGI323592[] = {{&S55680[51], 6}, {NULL}};
const Segment SGI323593[] = {{&S55680[71], 15}, {NULL}};
const Segment SGI323594[] = {{&S55688[30], 3}, {NULL}};
const Segment SGI323595[] = {{&S55688[8], 3}, {NULL}};
const Segment SGI323596[] = {{&S55688[13], 10}, {NULL}};
const Segment SGI323597[] = {{&S55688[55], 7}, {NULL}};
const Segment SGI323598[] = {{&S55688[48], 14}, {NULL}};
const Segment SGI323599[] = {{&S55688[64], 13}, {NULL}};
const Segment SGI323600[] = {{&S55689[30], 3}, {NULL}};
const Segment SGI323601[] = {{&S55689[8], 3}, {NULL}};
const Segment SGI323602[] = {{&S55689[13], 10}, {NULL}};
const Segment SGI323603[] = {{&S55689[55], 7}, {NULL}};
const Segment SGI323604[] = {{&S55689[48], 14}, {NULL}};
const Segment SGI323605[] = {{&S55689[64], 13}, {NULL}};
const Segment SGI323606[] = {{&S55690[30], 3}, {NULL}};
const Segment SGI323607[] = {{&S55690[8], 3}, {NULL}};
const Segment SGI323608[] = {{&S55690[13], 10}, {NULL}};
const Segment SGI323609[] = {{&S55690[55], 7}, {NULL}};
const Segment SGI323610[] = {{&S55690[48], 14}, {NULL}};
const Segment SGI323611[] = {{&S55690[64], 13}, {&S55690[64], 13}, {NULL}};
const Segment SGI323612[] = {{&S55691[30], 3}, {NULL}};
const Segment SGI323613[] = {{&S55691[8], 3}, {NULL}};
const Segment SGI323614[] = {{&S55691[13], 10}, {NULL}};
const Segment SGI323615[] = {{&S55691[55], 7}, {NULL}};
const Segment SGI323616[] = {{&S55691[48], 14}, {NULL}};
const Segment SGI323617[] = {{&S55691[69], 13}, {NULL}};
const Segment SGI323618[] = {{&S55691[64], 3}, {NULL}};
const Segment SGI323619[] = {{&S55692[30], 3}, {NULL}};
const Segment SGI323620[] = {{&S55692[8], 3}, {NULL}};
const Segment SGI323621[] = {{&S55692[13], 10}, {NULL}};
const Segment SGI323622[] = {{&S55692[55], 7}, {NULL}};
const Segment SGI323623[] = {{&S55692[48], 14}, {NULL}};
const Segment SGI323624[] = {{&S55692[69], 13}, {NULL}};
const Segment SGI323625[] = {{&S55692[64], 3}, {NULL}};
const Segment SGI323626[] = {{&S55693[30], 3}, {NULL}};
const Segment SGI323627[] = {{&S55693[8], 3}, {NULL}};
const Segment SGI323628[] = {{&S55693[13], 10}, {NULL}};
const Segment SGI323629[] = {{&S55693[55], 7}, {NULL}};
const Segment SGI323630[] = {{&S55693[48], 14}, {NULL}};
const Segment SGI323631[] = {{&S55693[64], 13}, {&S55693[64], 13}, {NULL}};
const Segment SGI323632[] = {{&S72531[3], 5}, {NULL}};
const Segment SGI323633[] = {{&S72546[3], 5}, {NULL}};
const Segment SGI323634[] = {{&S72546[105], 13}, {NULL}};
const Segment SGI323635[] = {{&S72560[3], 4}, {NULL}};
const Segment SGI323636[] = {{&S69008[0], 82}, {NULL}};
const Segment SGI323637[] = {{&S69009[0], 83}, {NULL}};
const Segment SGI323638[] = {{&S62650[4], 5}, {NULL}};
const Segment SGI323639[] = {{&S62650[25], 3}, {NULL}};
const Segment SGI323640[] = {{&S72984[3], 4}, {NULL}};
const Segment SGI323641[] = {{&S56059[30], 4}, {NULL}};
const Segment SGI323642[] = {{&S56059[8], 3}, {NULL}};
const Segment SGI323643[] = {{&S56059[13], 10}, {NULL}};
const Segment SGI323644[] = {{&S56059[43], 7}, {NULL}};
const Segment SGI323645[] = {{&S56059[36], 14}, {NULL}};
const Segment SGI323646[] = {{&S56059[52], 6}, {NULL}};
const Segment SGI323647[] = {{&S56059[72], 7}, {NULL}};
const Segment SGI323648[] = {{&S56059[79], 4}, {NULL}};
const Segment SGI323649[] = {{&S56060[30], 4}, {NULL}};
const Segment SGI323650[] = {{&S56060[8], 3}, {NULL}};
const Segment SGI323651[] = {{&S56060[13], 10}, {NULL}};
const Segment SGI323652[] = {{&S56060[43], 7}, {NULL}};
const Segment SGI323653[] = {{&S56060[36], 14}, {NULL}};
const Segment SGI323654[] = {{&S56060[65], 7}, {NULL}};
const Segment SGI323655[] = {{&S56060[72], 4}, {NULL}};
const Segment SGI323656[] = {{&S56061[30], 3}, {NULL}};
const Segment SGI323657[] = {{&S56061[8], 3}, {NULL}};
const Segment SGI323658[] = {{&S56061[13], 10}, {NULL}};
const Segment SGI323659[] = {{&S56061[42], 7}, {NULL}};
const Segment SGI323660[] = {{&S56061[35], 14}, {NULL}};
const Segment SGI323661[] = {{&S56061[64], 7}, {NULL}};
const Segment SGI323662[] = {{&S56061[71], 4}, {NULL}};
const Segment SGI323663[] = {{&S56062[30], 3}, {NULL}};
const Segment SGI323664[] = {{&S56062[8], 3}, {NULL}};
const Segment SGI323665[] = {{&S56062[13], 10}, {NULL}};
const Segment SGI323666[] = {{&S56062[42], 7}, {NULL}};
const Segment SGI323667[] = {{&S56062[35], 14}, {NULL}};
const Segment SGI323668[] = {{&S56062[51], 6}, {NULL}};
const Segment SGI323669[] = {{&S56062[71], 7}, {NULL}};
const Segment SGI323670[] = {{&S56062[78], 4}, {NULL}};
const Segment SGI323671[] = {{&S73023[3], 4}, {NULL}};
const Segment SGI323672[] = {{&S73024[3], 5}, {NULL}};
const Segment SGI323673[] = {{&S69382[0], 42}, {NULL}};
const Segment SGI323674[] = {{&S56216[30], 3}, {NULL}};
const Segment SGI323675[] = {{&S56216[8], 3}, {NULL}};
const Segment SGI323676[] = {{&S56216[13], 10}, {NULL}};
const Segment SGI323677[] = {{&S56217[30], 3}, {NULL}};
const Segment SGI323678[] = {{&S56217[8], 3}, {NULL}};
const Segment SGI323679[] = {{&S56217[13], 10}, {NULL}};
const Segment SGI323680[] = {{&S56218[30], 3}, {NULL}};
const Segment SGI323681[] = {{&S56218[8], 3}, {NULL}};
const Segment SGI323682[] = {{&S56218[13], 10}, {NULL}};
const Segment SGI323683[] = {{&S56218[35], 3}, {NULL}};
const Segment SGI323684[] = {{&S56219[30], 3}, {NULL}};
const Segment SGI323685[] = {{&S56219[8], 3}, {NULL}};
const Segment SGI323686[] = {{&S56219[13], 10}, {NULL}};
const Segment SGI323687[] = {{&S56219[35], 3}, {NULL}};
const Segment SGI323688[] = {{&S56219[40], 3}, {NULL}};
const Segment SGI323689[] = {{&S73428[3], 4}, {NULL}};
const Segment SGI323690[] = {{&S69871[0], 117}, {NULL}};
const Segment SGI323691[] = {{&S69874[0], 117}, {NULL}};
const Segment SGI323692[] = {{&S73779[3], 5}, {NULL}};
const Segment SGI323693[] = {{&S73816[3], 5}, {NULL}};
const Segment SGI323694[] = {{&S73816[127], 13}, {NULL}};
const Segment SGI323695[] = {{&S56838[30], 3}, {NULL}};
const Segment SGI323696[] = {{&S56838[8], 3}, {NULL}};
const Segment SGI323697[] = {{&S56838[13], 10}, {NULL}};
const Segment SGI323698[] = {{&S56838[42], 8}, {NULL}};
const Segment SGI323699[] = {{&S56838[52], 3}, {NULL}};
const Segment SGI323700[] = {{&S56839[30], 3}, {NULL}};
const Segment SGI323701[] = {{&S56839[8], 3}, {NULL}};
const Segment SGI323702[] = {{&S56839[13], 10}, {NULL}};
const Segment SGI323703[] = {{&S56839[42], 8}, {NULL}};
const Segment SGI323704[] = {{&S56840[30], 3}, {NULL}};
const Segment SGI323705[] = {{&S56840[8], 3}, {NULL}};
const Segment SGI323706[] = {{&S56840[13], 10}, {NULL}};
const Segment SGI323707[] = {{&S56840[42], 8}, {NULL}};
const Segment SGI323708[] = {{&S56840[132], 15}, {NULL}};
const Segment SGI323709[] = {{&S56841[30], 3}, {NULL}};
const Segment SGI323710[] = {{&S56841[8], 3}, {NULL}};
const Segment SGI323711[] = {{&S56841[13], 10}, {NULL}};
const Segment SGI323712[] = {{&S56841[42], 8}, {NULL}};
const Segment SGI323713[] = {{&S56841[52], 6}, {NULL}};
const Segment SGI323714[] = {{&S56842[30], 3}, {NULL}};
const Segment SGI323715[] = {{&S56842[8], 3}, {NULL}};
const Segment SGI323716[] = {{&S56842[13], 10}, {NULL}};
const Segment SGI323717[] = {{&S56842[42], 8}, {NULL}};
const Segment SGI323718[] = {{&S56842[139], 15}, {NULL}};
const Segment SGI323719[] = {{&S56842[52], 6}, {NULL}};
const Segment SGI323720[] = {{&S63640[4], 6}, {NULL}};
const Segment SGI323721[] = {{&S63640[26], 3}, {NULL}};
const Segment SGI323722[] = {{&S63641[4], 6}, {NULL}};
const Segment SGI323723[] = {{&S63641[26], 3}, {NULL}};
const Segment SGI323724[] = {{&S63675[4], 5}, {NULL}};
const Segment SGI323725[] = {{&S63675[4], 9}, {NULL}};
const Segment SGI323726[] = {{&S70294[0], 41}, {NULL}};
const Segment SGI323727[] = {{&S74314[3], 5}, {NULL}};
const Segment SGI323728[] = {{&S60407[4], 14}, {NULL}};
const Segment SGI323729[] = {{&S60407[131], 15}, {NULL}};
const Segment SGI323730[] = {{&S74826[3], 5}, {NULL}};
const Segment SGI323731[] = {{&S74826[3], 10}, {NULL}};
const Segment SGI323732[] = {{&S74826[28], 9}, {NULL}};
const Segment SGI323733[] = {{&S74827[3], 5}, {NULL}};
const Segment SGI323734[] = {{&S74827[3], 12}, {NULL}};
const Segment SGI323735[] = {{&S74827[35], 9}, {NULL}};
const Segment SGI323736[] = {{&S74828[3], 5}, {NULL}};
const Segment SGI323737[] = {{&S74828[3], 10}, {NULL}};
const Segment SGI323738[] = {{&S74828[28], 9}, {NULL}};
const Segment SGI323739[] = {{&S74830[3], 5}, {NULL}};
const Segment SGI323740[] = {{&S74830[28], 9}, {NULL}};
const Segment SGI323741[] = {{&S74833[3], 5}, {NULL}};
const Segment SGI323742[] = {{&S74833[3], 10}, {NULL}};
const Segment SGI323743[] = {{&S74833[28], 9}, {NULL}};
const Segment SGI323744[] = {{&S57583[30], 3}, {NULL}};
const Segment SGI323745[] = {{&S57583[8], 3}, {NULL}};
const Segment SGI323746[] = {{&S57583[13], 10}, {NULL}};
const Segment SGI323747[] = {{&S57583[46], 3}, {NULL}};
const Segment SGI323748[] = {{&S57584[30], 4}, {NULL}};
const Segment SGI323749[] = {{&S57584[8], 3}, {NULL}};
const Segment SGI323750[] = {{&S57584[13], 10}, {NULL}};
const Segment SGI323751[] = {{&S57584[62], 3}, {NULL}};
const Segment SGI323752[] = {{&S57585[30], 4}, {NULL}};
const Segment SGI323753[] = {{&S57585[8], 3}, {NULL}};
const Segment SGI323754[] = {{&S57585[13], 10}, {NULL}};
const Segment SGI323755[] = {{&S57585[43], 3}, {NULL}};
const Segment SGI323756[] = {{&S57586[30], 3}, {NULL}};
const Segment SGI323757[] = {{&S64616[4], 5}, {NULL}};
const Segment SGI323758[] = {{&S57586[8], 3}, {NULL}};
const Segment SGI323759[] = {{&S57586[13], 10}, {NULL}};
const Segment SGI323760[] = {{&S64616[25], 3}, {NULL}};
const Segment SGI323761[] = {{&S57586[35], 5}, {NULL}};
const Segment SGI323762[] = {{&S75130[3], 5}, {NULL}};
const Segment SGI323763[] = {{&S75130[20], 3}, {NULL}};
const Segment SGI323764[] = {{&S70913[0], 40}, {NULL}};
const Segment SGI323765[] = {{&S57781[30], 3}, {NULL}};
const Segment SGI323766[] = {{&S57781[8], 3}, {NULL}};
const Segment SGI323767[] = {{&S57781[13], 10}, {NULL}};
const Segment SGI323768[] = {{&S57781[42], 7}, {NULL}};
const Segment SGI323769[] = {{&S57781[35], 14}, {NULL}};
const Segment SGI323770[] = {{&S70914[0], 40}, {NULL}};
const Segment SGI323771[] = {{&S57781[74], 3}, {NULL}};
const Segment SGI323772[] = {{&S70915[0], 40}, {NULL}};
const Segment SGI323773[] = {{&S57782[30], 3}, {NULL}};
const Segment SGI323774[] = {{&S57782[8], 3}, {NULL}};
const Segment SGI323775[] = {{&S57782[13], 10}, {NULL}};
const Segment SGI323776[] = {{&S57782[42], 7}, {NULL}};
const Segment SGI323777[] = {{&S57782[35], 14}, {NULL}};
const Segment SGI323778[] = {{&S57782[51], 6}, {NULL}};
const Segment SGI323779[] = {{&S57782[81], 3}, {NULL}};
const Segment SGI323780[] = {{&S70916[0], 40}, {NULL}};
const Segment SGI323781[] = {{&S70917[0], 40}, {NULL}};
const Segment SGI323782[] = {{&S57783[30], 3}, {NULL}};
const Segment SGI323783[] = {{&S57783[8], 3}, {NULL}};
const Segment SGI323784[] = {{&S57783[13], 10}, {NULL}};
const Segment SGI323785[] = {{&S57783[42], 7}, {NULL}};
const Segment SGI323786[] = {{&S57783[35], 14}, {NULL}};
const Segment SGI323787[] = {{&S57783[131], 15}, {NULL}};
const Segment SGI323788[] = {{&S70918[0], 40}, {NULL}};
const Segment SGI323789[] = {{&S57783[162], 3}, {NULL}};
const Segment SGI323790[] = {{&S57784[30], 4}, {NULL}};
const Segment SGI323791[] = {{&S57784[8], 3}, {NULL}};
const Segment SGI323792[] = {{&S57784[13], 10}, {NULL}};
const Segment SGI323793[] = {{&S57784[43], 7}, {NULL}};
const Segment SGI323794[] = {{&S57784[36], 14}, {NULL}};
const Segment SGI323795[] = {{&S57784[52], 6}, {NULL}};
const Segment SGI323796[] = {{&S57784[82], 3}, {NULL}};
const Segment SGI323797[] = {{&S57785[30], 3}, {NULL}};
const Segment SGI323798[] = {{&S57785[8], 3}, {NULL}};
const Segment SGI323799[] = {{&S57785[13], 10}, {NULL}};
const Segment SGI323800[] = {{&S57785[42], 7}, {NULL}};
const Segment SGI323801[] = {{&S57785[35], 14}, {NULL}};
const Segment SGI323802[] = {{&S57785[138], 15}, {NULL}};
const Segment SGI323803[] = {{&S57785[51], 6}, {NULL}};
const Segment SGI323804[] = {{&S57785[169], 3}, {NULL}};
const Segment SGI323805[] = {{&S57786[30], 3}, {NULL}};
const Segment SGI323806[] = {{&S57786[8], 3}, {NULL}};
const Segment SGI323807[] = {{&S57786[13], 10}, {NULL}};
const Segment SGI323808[] = {{&S57786[42], 7}, {NULL}};
const Segment SGI323809[] = {{&S57786[35], 14}, {NULL}};
const Segment SGI323810[] = {{&S57786[131], 15}, {NULL}};
const Segment SGI323811[] = {{&S57789[30], 3}, {NULL}};
const Segment SGI323812[] = {{&S57789[8], 3}, {NULL}};
const Segment SGI323813[] = {{&S57789[13], 10}, {NULL}};
const Segment SGI323814[] = {{&S57789[42], 7}, {NULL}};
const Segment SGI323815[] = {{&S57789[35], 14}, {NULL}};
const Segment SGI323816[] = {{&S57789[51], 6}, {NULL}};
const Segment SGI323817[] = {{&S57790[30], 3}, {NULL}};
const Segment SGI323818[] = {{&S57790[8], 3}, {NULL}};
const Segment SGI323819[] = {{&S57790[13], 10}, {NULL}};
const Segment SGI323820[] = {{&S57790[42], 7}, {NULL}};
const Segment SGI323821[] = {{&S57790[35], 14}, {NULL}};
const Segment SGI323822[] = {{&S57791[30], 3}, {NULL}};
const Segment SGI323823[] = {{&S57791[8], 3}, {NULL}};
const Segment SGI323824[] = {{&S57791[13], 10}, {NULL}};
const Segment SGI323825[] = {{&S57791[42], 7}, {NULL}};
const Segment SGI323826[] = {{&S57791[35], 14}, {NULL}};
const Segment SGI323827[] = {{&S57791[91], 15}, {NULL}};
const Segment SGI323828[] = {{&S57791[51], 6}, {NULL}};
const Segment SGI323829[] = {{&S57792[30], 3}, {NULL}};
const Segment SGI323830[] = {{&S57792[8], 3}, {NULL}};
const Segment SGI323831[] = {{&S64931[4], 5}, {NULL}};
const Segment SGI323832[] = {{&S64931[4], 7}, {NULL}};
const Segment SGI323833[] = {{&S64931[27], 3}, {NULL}};
const Segment SGI323834[] = {{&S64932[4], 5}, {NULL}};
const Segment SGI323835[] = {{&S64932[25], 3}, {NULL}};
const Segment SGI323836[] = {{&S57792[13], 10}, {NULL}};
const Segment SGI323837[] = {{&S57792[42], 7}, {NULL}};
const Segment SGI323838[] = {{&S57792[35], 14}, {NULL}};
const Segment SGI323839[] = {{&S57792[51], 5}, {NULL}};
const Segment SGI323840[] = {{&S57793[30], 3}, {NULL}};
const Segment SGI323841[] = {{&S57793[8], 3}, {NULL}};
const Segment SGI323842[] = {{&S57793[13], 10}, {NULL}};
const Segment SGI323843[] = {{&S57793[42], 7}, {NULL}};
const Segment SGI323844[] = {{&S57793[35], 14}, {NULL}};
const Segment SGI323845[] = {{&S57793[51], 6}, {NULL}};
const Segment SGI323846[] = {{&S57793[151], 9}, {NULL}};
const Segment SGI323847[] = {{&S86210[3], 5}, {NULL}};
const Segment SGI323848[] = {{&S75445[3], 4}, {NULL}};
const Segment SGI323849[] = {{&S75507[3], 5}, {NULL}};
const Segment SGI323850[] = {{&S75544[3], 5}, {NULL}};
const Segment SGI323851[] = {{&S75544[3], 10}, {NULL}};
const Segment SGI323852[] = {{&S75545[3], 5}, {NULL}};
const Segment SGI323853[] = {{&S75618[3], 3}, {NULL}};
const Segment SGI323854[] = {{&S71452[0], 32}, {NULL}};
const Segment SGI323855[] = {{&S86528[3], 5}, {NULL}};
const Segment SGI323856[] = {{&S58253[30], 4}, {NULL}};
const Segment SGI323857[] = {{&S58253[8], 3}, {NULL}};
const Segment SGI323858[] = {{&S58253[13], 10}, {NULL}};
const Segment SGI323859[] = {{&S58254[30], 4}, {NULL}};
const Segment SGI323860[] = {{&S58254[8], 3}, {NULL}};
const Segment SGI323861[] = {{&S58254[13], 10}, {NULL}};
const Segment SGI323862[] = {{&S58254[36], 3}, {NULL}};
const Segment SGI323863[] = {{&S58255[30], 4}, {NULL}};
const Segment SGI323864[] = {{&S58255[8], 3}, {NULL}};
const Segment SGI323865[] = {{&S58255[13], 10}, {NULL}};
const Segment SGI323866[] = {{&S58256[30], 4}, {NULL}};
const Segment SGI323867[] = {{&S58256[8], 3}, {NULL}};
const Segment SGI323868[] = {{&S58256[13], 10}, {NULL}};
const Segment SGI323869[] = {{&S58256[36], 3}, {NULL}};
const Segment SGI323870[] = {{&S58256[41], 3}, {NULL}};
const Segment SGI323871[] = {{&S76108[3], 5}, {NULL}};
const Segment SGI323872[] = {{&S58551[30], 3}, {NULL}};
const Segment SGI323873[] = {{&S58551[8], 3}, {NULL}};
const Segment SGI323874[] = {{&S58551[13], 10}, {NULL}};
const Segment SGI323875[] = {{&S58551[35], 3}, {NULL}};
const Segment SGI323876[] = {{&S58552[30], 3}, {NULL}};
const Segment SGI323877[] = {{&S58552[8], 3}, {NULL}};
const Segment SGI323878[] = {{&S58552[13], 10}, {NULL}};
const Segment SGI323879[] = {{&S58552[35], 3}, {NULL}};
const Segment SGI323880[] = {{&S58552[40], 3}, {NULL}};
const Segment SGI323881[] = {{&S76263[3], 5}, {NULL}};
const Segment SGI323882[] = {{&S71843[0], 11}, {NULL}};
const Segment SGI323883[] = {{&S71844[0], 39}, {NULL}};
const Segment SGI323884[] = {{&S58758[30], 3}, {NULL}};
const Segment SGI323885[] = {{&S58758[8], 3}, {NULL}};
const Segment SGI323886[] = {{&S58758[13], 10}, {NULL}};
const Segment SGI323887[] = {{&S58758[42], 7}, {NULL}};
const Segment SGI323888[] = {{&S58758[35], 14}, {NULL}};
const Segment SGI323889[] = {{&S58758[131], 15}, {NULL}};
const Segment SGI323890[] = {{&S58758[147], 3}, {NULL}};
const Segment SGI323891[] = {{&S58761[30], 3}, {NULL}};
const Segment SGI323892[] = {{&S58761[8], 3}, {NULL}};
const Segment SGI323893[] = {{&S58761[13], 10}, {NULL}};
const Segment SGI323894[] = {{&S58761[42], 7}, {NULL}};
const Segment SGI323895[] = {{&S58761[61], 3}, {NULL}};
const Segment SGI323896[] = {{&S71896[0], 36}, {NULL}};
const Segment SGI323897[] = {{&S58839[30], 3}, {NULL}};
const Segment SGI323898[] = {{&S58839[8], 3}, {NULL}};
const Segment SGI323899[] = {{&S58839[13], 10}, {NULL}};
const Segment SGI323900[] = {{&S58839[35], 3}, {NULL}};
const Segment SGI323901[] = {{&S58840[30], 3}, {NULL}};
const Segment SGI323902[] = {{&S58840[8], 3}, {NULL}};
const Segment SGI323903[] = {{&S58840[13], 10}, {NULL}};
const Segment SGI323904[] = {{&S58840[35], 3}, {NULL}};
const Segment SGI323905[] = {{&S58840[40], 3}, {NULL}};
const Segment SGI323906[] = {{&S58960[30], 3}, {NULL}};
const Segment SGI323907[] = {{&S58960[8], 3}, {NULL}};
const Segment SGI323908[] = {{&S58960[13], 10}, {NULL}};
const Segment SGI323909[] = {{&S58960[42], 7}, {NULL}};
const Segment SGI323910[] = {{&S58960[35], 14}, {NULL}};
const Segment SGI323911[] = {{&S58960[51], 6}, {NULL}};
const Segment SGI323912[] = {{&S58960[151], 7}, {NULL}};
const Segment SGI323913[] = {{&S58960[158], 4}, {NULL}};
const Segment SGI323914[] = {{&S58961[30], 4}, {NULL}};
const Segment SGI323915[] = {{&S58961[8], 3}, {NULL}};
const Segment SGI323916[] = {{&S58961[13], 10}, {NULL}};
const Segment SGI323917[] = {{&S58961[43], 7}, {NULL}};
const Segment SGI323918[] = {{&S58961[36], 14}, {NULL}};
const Segment SGI323919[] = {{&S58961[52], 6}, {NULL}};
const Segment SGI323920[] = {{&S58961[72], 7}, {NULL}};
const Segment SGI323921[] = {{&S58961[79], 4}, {NULL}};
const Segment SGI323922[] = {{&S58962[30], 3}, {NULL}};
const Segment SGI323923[] = {{&S58962[8], 3}, {NULL}};
const Segment SGI323924[] = {{&S58962[13], 10}, {NULL}};
const Segment SGI323925[] = {{&S58962[42], 7}, {NULL}};
const Segment SGI323926[] = {{&S58962[35], 14}, {NULL}};
const Segment SGI323927[] = {{&S58962[144], 7}, {NULL}};
const Segment SGI323928[] = {{&S58962[151], 4}, {NULL}};
const Segment SGI323929[] = {{&S58963[30], 3}, {NULL}};
const Segment SGI323930[] = {{&S58963[8], 3}, {NULL}};
const Segment SGI323931[] = {{&S58963[13], 10}, {NULL}};
const Segment SGI323932[] = {{&S58963[42], 7}, {NULL}};
const Segment SGI323933[] = {{&S58963[35], 14}, {NULL}};
const Segment SGI323934[] = {{&S58963[91], 15}, {NULL}};
const Segment SGI323935[] = {{&S58963[51], 6}, {NULL}};
const Segment SGI323936[] = {{&S58963[172], 7}, {NULL}};
const Segment SGI323937[] = {{&S58963[179], 4}, {NULL}};
const Segment SGI323938[] = {{&S58964[30], 4}, {NULL}};
const Segment SGI323939[] = {{&S58964[8], 3}, {NULL}};
const Segment SGI323940[] = {{&S58964[13], 10}, {NULL}};
const Segment SGI323941[] = {{&S58964[43], 7}, {NULL}};
const Segment SGI323942[] = {{&S58964[36], 14}, {NULL}};
const Segment SGI323943[] = {{&S58964[65], 7}, {NULL}};
const Segment SGI323944[] = {{&S58964[72], 4}, {NULL}};
const Segment SGI323945[] = {{&S58965[30], 3}, {NULL}};
const Segment SGI323946[] = {{&S58965[8], 3}, {NULL}};
const Segment SGI323947[] = {{&S58965[13], 10}, {NULL}};
const Segment SGI323948[] = {{&S58965[42], 7}, {NULL}};
const Segment SGI323949[] = {{&S58965[35], 14}, {NULL}};
const Segment SGI323950[] = {{&S58965[84], 15}, {NULL}};
const Segment SGI323951[] = {{&S58965[165], 7}, {NULL}};
const Segment SGI323952[] = {{&S58965[172], 4}, {NULL}};
const Segment SGI323953[] = {{&S58966[30], 3}, {NULL}};
const Segment SGI323954[] = {{&S58966[8], 3}, {NULL}};
const Segment SGI323955[] = {{&S58966[13], 10}, {NULL}};
const Segment SGI323956[] = {{&S58966[42], 7}, {NULL}};
const Segment SGI323957[] = {{&S58966[35], 14}, {NULL}};
const Segment SGI323958[] = {{&S58966[51], 6}, {NULL}};
const Segment SGI323959[] = {{&S58966[166], 7}, {NULL}};
const Segment SGI323960[] = {{&S58966[173], 4}, {NULL}};
const Segment SGI323961[] = {{&S58966[151], 9}, {NULL}};
const Segment SGI323962[] = {{&S76595[3], 5}, {NULL}};
const Segment SGI323963[] = {{&S62548[4], 2}, {NULL}};
const Segment SGI323964[] = {{&S62548[36], 48}, {NULL}};
const Segment SGI323965[] = {{&S62549[4], 2}, {NULL}};
const Segment SGI323966[] = {{&S62549[37], 45}, {NULL}};
const Segment SGI323967[] = {{&S62550[4], 2}, {NULL}};
const Segment SGI323968[] = {{&S62550[36], 48}, {NULL}};
const Segment SGI323969[] = {{&S62551[4], 15}, {NULL}};
const Segment SGI323970[] = {{&S62552[4], 13}, {NULL}};
const Segment SGI323971[] = {{&S62553[4], 2}, {NULL}};
const Segment SGI323972[] = {{&S62553[36], 48}, {NULL}};
const Segment SGI323973[] = {{&S62555[4], 18}, {NULL}};
const Segment SGI323974[] = {{&S76639[3], 5}, {NULL}};
const Segment SGI323975[] = {{&S76639[80], 13}, {NULL}};
const Segment SGI323976[] = {{&S76640[3], 4}, {NULL}};
const Segment SGI323977[] = {{&S72231[0], 32}, {NULL}};
const Segment SGI323978[] = {{&S72232[0], 33}, {NULL}};
const Segment SGI323979[] = {{&S59188[30], 4}, {NULL}};
const Segment SGI323980[] = {{&S59188[8], 3}, {NULL}};
const Segment SGI323981[] = {{&S59188[13], 10}, {NULL}};
const Segment SGI323982[] = {{&S59188[43], 7}, {NULL}};
const Segment SGI323983[] = {{&S59188[36], 14}, {NULL}};
const Segment SGI323984[] = {{&S59188[72], 5}, {NULL}};
const Segment SGI323985[] = {{&S59188[52], 6}, {NULL}};
const Segment SGI323986[] = {{&S59188[79], 13}, {NULL}};
const Segment SGI323987[] = {{&S59189[30], 4}, {NULL}};
const Segment SGI323988[] = {{&S59189[8], 3}, {NULL}};
const Segment SGI323989[] = {{&S59189[13], 10}, {NULL}};
const Segment SGI323990[] = {{&S59189[43], 7}, {NULL}};
const Segment SGI323991[] = {{&S59189[36], 14}, {NULL}};
const Segment SGI323992[] = {{&S59189[77], 5}, {NULL}};
const Segment SGI323993[] = {{&S59189[84], 13}, {NULL}};
const Segment SGI323994[] = {{&S59189[52], 5}, {NULL}};
const Segment SGI323995[] = {{&S59190[30], 4}, {NULL}};
const Segment SGI323996[] = {{&S59190[8], 3}, {NULL}};
const Segment SGI323997[] = {{&S59190[13], 10}, {NULL}};
const Segment SGI323998[] = {{&S59190[43], 7}, {NULL}};
const Segment SGI323999[] = {{&S59190[36], 14}, {NULL}};
const Segment SGI324000[] = {{&S59190[65], 5}, {NULL}};
const Segment SGI324001[] = {{&S59190[72], 13}, {NULL}};
const Segment SGI324002[] = {{&S94415[7], 4}, {NULL}};
const Segment SGI324003[] = {{&S94415[0], 81}, {NULL}};
const Segment SGI324004[] = {{&S59364[30], 3}, {NULL}};
const Segment SGI324005[] = {{&S59364[8], 3}, {NULL}};
const Segment SGI324006[] = {{&S59364[13], 10}, {NULL}};
const Segment SGI324007[] = {{&S59364[35], 3}, {NULL}};
const Segment SGI324008[] = {{&S59365[30], 3}, {NULL}};
const Segment SGI324009[] = {{&S59365[8], 3}, {NULL}};
const Segment SGI324010[] = {{&S59365[13], 10}, {NULL}};
const Segment SGI324011[] = {{&S59365[35], 3}, {NULL}};
const Segment SGI324012[] = {{&S59365[40], 3}, {NULL}};
const Segment SGI324013[] = {{&S59371[30], 3}, {NULL}};
const Segment SGI324014[] = {{&S59371[8], 3}, {NULL}};
const Segment SGI324015[] = {{&S59371[13], 10}, {NULL}};
const Segment SGI324016[] = {{&S59371[42], 7}, {NULL}};
const Segment SGI324017[] = {{&S59371[35], 14}, {NULL}};
const Segment SGI324018[] = {{&S59371[131], 15}, {NULL}};
const Segment SGI324019[] = {{&S59371[172], 4}, {NULL}};
const Segment SGI324020[] = {{&S59372[30], 3}, {NULL}};
const Segment SGI324021[] = {{&S59372[8], 3}, {NULL}};
const Segment SGI324022[] = {{&S59372[13], 10}, {NULL}};
const Segment SGI324023[] = {{&S59372[42], 7}, {NULL}};
const Segment SGI324024[] = {{&S59372[35], 14}, {NULL}};
const Segment SGI324025[] = {{&S59372[138], 15}, {NULL}};
const Segment SGI324026[] = {{&S59372[51], 6}, {NULL}};
const Segment SGI324027[] = {{&S59372[179], 4}, {NULL}};
const Segment SGI324028[] = {{&S59373[30], 3}, {NULL}};
const Segment SGI324029[] = {{&S59373[8], 3}, {NULL}};
const Segment SGI324030[] = {{&S59373[13], 10}, {NULL}};
const Segment SGI324031[] = {{&S59373[42], 7}, {NULL}};
const Segment SGI324032[] = {{&S59373[35], 14}, {NULL}};
const Segment SGI324033[] = {{&S59373[151], 4}, {NULL}};
const Segment SGI324034[] = {{&S59374[30], 3}, {NULL}};
const Segment SGI324035[] = {{&S59374[8], 3}, {NULL}};
const Segment SGI324036[] = {{&S59374[13], 10}, {NULL}};
const Segment SGI324037[] = {{&S59374[42], 7}, {NULL}};
const Segment SGI324038[] = {{&S59374[35], 14}, {NULL}};
const Segment SGI324039[] = {{&S59374[51], 6}, {NULL}};
const Segment SGI324040[] = {{&S59374[164], 4}, {NULL}};
const Segment SGI324041[] = {{&S59375[30], 3}, {NULL}};
const Segment SGI324042[] = {{&S59375[8], 3}, {NULL}};
const Segment SGI324043[] = {{&S59375[13], 10}, {NULL}};
const Segment SGI324044[] = {{&S59375[62], 7}, {NULL}};
const Segment SGI324045[] = {{&S59375[55], 14}, {NULL}};
const Segment SGI324046[] = {{&S59375[151], 15}, {NULL}};
const Segment SGI324047[] = {{&S59375[192], 4}, {NULL}};
const Segment SGI324048[] = {{&S59376[30], 3}, {NULL}};
const Segment SGI324049[] = {{&S59376[8], 3}, {NULL}};
const Segment SGI324050[] = {{&S59376[13], 10}, {NULL}};
const Segment SGI324051[] = {{&S59376[42], 7}, {NULL}};
const Segment SGI324052[] = {{&S59376[35], 14}, {NULL}};
const Segment SGI324053[] = {{&S59376[131], 15}, {NULL}};
const Segment SGI324054[] = {{&S59376[152], 9}, {NULL}};
const Segment SGI324055[] = {{&S59376[183], 4}, {NULL}};
const Segment SGI324056[] = {{&S59377[30], 3}, {NULL}};
const Segment SGI324057[] = {{&S59377[8], 3}, {&S59377[8], 3}, {NULL}};
const Segment SGI324058[] = {{&S59377[13], 10}, {NULL}};
const Segment SGI324059[] = {{&S59377[42], 7}, {NULL}};
const Segment SGI324060[] = {{&S59377[35], 14}, {NULL}};
const Segment SGI324061[] = {{&S59377[208], 15}, {NULL}};
const Segment SGI324062[] = {{&S59377[239], 4}, {NULL}};
const Segment SGI324063[] = {{&S59590[30], 4}, {NULL}};
const Segment SGI324064[] = {{&S59590[8], 3}, {NULL}};
const Segment SGI324065[] = {{&S59590[13], 10}, {NULL}};
const Segment SGI324066[] = {{&S59590[43], 3}, {NULL}};
const Segment SGI324067[] = {{&S59591[30], 3}, {NULL}};
const Segment SGI324068[] = {{&S59591[8], 3}, {NULL}};
const Segment SGI324069[] = {{&S59591[13], 10}, {NULL}};
const Segment SGI324070[] = {{&S59592[30], 3}, {NULL}};
const Segment SGI324071[] = {{&S59592[8], 3}, {NULL}};
const Segment SGI324072[] = {{&S59592[13], 10}, {NULL}};
const Segment SGI324073[] = {{&S59592[35], 5}, {NULL}};
const Segment SGI324074[] = {{&S77256[3], 5}, {NULL}};
const Segment SGI324075[] = {{&S77256[3], 10}, {NULL}};
const Segment SGI324076[] = {{&S77256[29], 4}, {NULL}};
const Segment SGI324077[] = {{&S77257[3], 5}, {NULL}};
const Segment SGI324078[] = {{&S77257[3], 10}, {NULL}};
const Segment SGI324079[] = {{&S77257[29], 4}, {NULL}};
const Segment SGI324080[] = {{&S77258[3], 5}, {NULL}};
const Segment SGI324081[] = {{&S77258[3], 10}, {NULL}};
const Segment SGI324082[] = {{&S77258[29], 4}, {NULL}};
const Segment SGI324083[] = {{&S77259[3], 5}, {NULL}};
const Segment SGI324084[] = {{&S77259[3], 10}, {NULL}};
const Segment SGI324085[] = {{&S77259[29], 4}, {NULL}};
const Segment SGI324086[] = {{&S77260[3], 5}, {NULL}};
const Segment SGI324087[] = {{&S77260[3], 10}, {NULL}};
const Segment SGI324088[] = {{&S77260[29], 4}, {NULL}};
const Segment SGI324089[] = {{&S77263[3], 5}, {NULL}};
const Segment SGI324090[] = {{&S77263[29], 4}, {NULL}};
const Segment SGI324091[] = {{&S77264[3], 5}, {NULL}};
const Segment SGI324092[] = {{&S77264[3], 10}, {NULL}};
const Segment SGI324093[] = {{&S77264[29], 4}, {NULL}};
const Segment SGI324094[] = {{&S77265[3], 5}, {NULL}};
const Segment SGI324095[] = {{&S77265[3], 10}, {NULL}};
const Segment SGI324096[] = {{&S77265[29], 4}, {NULL}};
const Segment SGI324097[] = {{&S77266[3], 5}, {NULL}};
const Segment SGI324098[] = {{&S77266[3], 10}, {NULL}};
const Segment SGI324099[] = {{&S77266[29], 4}, {NULL}};
const Segment SGI324100[] = {{&S77267[3], 5}, {NULL}};
const Segment SGI324101[] = {{&S77267[3], 10}, {NULL}};
const Segment SGI324102[] = {{&S77267[29], 4}, {NULL}};
const Segment SGI324103[] = {{&S94995[0], 91}, {NULL}};
const Segment SGI324104[] = {{&S63341[4], 6}, {NULL}};
const Segment SGI324105[] = {{&S63341[29], 56}, {NULL}};
const Segment SGI324106[] = {{&S63342[4], 6}, {NULL}};
const Segment SGI324107[] = {{&S63438[4], 10}, {NULL}};
const Segment SGI324108[] = {{&S63438[46], 32}, {NULL}};
const Segment SGI324109[] = {{&S63439[4], 10}, {NULL}};
const Segment SGI324110[] = {{&S63439[44], 32}, {NULL}};
const Segment SGI324111[] = {{&S63440[4], 11}, {NULL}};
const Segment SGI324112[] = {{&S63440[45], 32}, {NULL}};
const Segment SGI324113[] = {{&S63441[4], 11}, {NULL}};
const Segment SGI324114[] = {{&S63441[45], 32}, {NULL}};
const Segment SGI324115[] = {{&S63442[4], 10}, {NULL}};
const Segment SGI324116[] = {{&S63442[45], 32}, {NULL}};
const Segment SGI324117[] = {{&S63443[4], 10}, {NULL}};
const Segment SGI324118[] = {{&S63443[44], 32}, {NULL}};
const Segment SGI324119[] = {{&S63444[4], 10}, {NULL}};
const Segment SGI324120[] = {{&S63444[44], 32}, {NULL}};
const Segment SGI324121[] = {{&S63445[4], 10}, {NULL}};
const Segment SGI324122[] = {{&S63445[45], 32}, {NULL}};
const Segment SGI324123[] = {{&S77522[3], 5}, {NULL}};
const Segment SGI324124[] = {{&S63446[4], 11}, {NULL}};
const Segment SGI324125[] = {{&S63446[45], 25}, {NULL}};
const Segment SGI324126[] = {{&S63447[4], 10}, {NULL}};
const Segment SGI324127[] = {{&S63447[45], 25}, {NULL}};
const Segment SGI324128[] = {{&S95125[7], 7}, {NULL}};
const Segment SGI324129[] = {{&S95125[0], 84}, {NULL}};
const Segment SGI324130[] = {{&S95126[7], 7}, {NULL}};
const Segment SGI324131[] = {{&S95126[0], 84}, {NULL}};
const Segment SGI324132[] = {{&S95127[7], 4}, {NULL}};
const Segment SGI324133[] = {{&S95127[0], 81}, {NULL}};
const Segment SGI324134[] = {{&S87911[3], 5}, {NULL}};
const Segment SGI324135[] = {{&S77727[3], 5}, {NULL}};
const Segment SGI324136[] = {{&S77727[29], 4}, {NULL}};
const Segment SGI324137[] = {{&S77728[3], 5}, {NULL}};
const Segment SGI324138[] = {{&S77728[3], 12}, {NULL}};
const Segment SGI324139[] = {{&S77728[36], 4}, {NULL}};
const Segment SGI324140[] = {{&S60048[30], 3}, {NULL}};
const Segment SGI324141[] = {{&S60048[8], 3}, {NULL}};
const Segment SGI324142[] = {{&S60048[13], 10}, {NULL}};
const Segment SGI324143[] = {{&S60048[42], 7}, {NULL}};
const Segment SGI324144[] = {{&S60048[35], 14}, {NULL}};
const Segment SGI324145[] = {{&S60048[56], 7}, {NULL}};
const Segment SGI324146[] = {{&S60048[63], 4}, {NULL}};
const Segment SGI324147[] = {{&S60048[51], 3}, {NULL}};
const Segment SGI324148[] = {{&S60049[30], 3}, {NULL}};
const Segment SGI324149[] = {{&S60049[8], 3}, {NULL}};
const Segment SGI324150[] = {{&S60049[13], 10}, {NULL}};
const Segment SGI324151[] = {{&S60049[42], 7}, {NULL}};
const Segment SGI324152[] = {{&S60049[35], 14}, {NULL}};
const Segment SGI324153[] = {{&S60049[51], 7}, {NULL}};
const Segment SGI324154[] = {{&S60049[58], 4}, {NULL}};
const Segment SGI324155[] = {{&S60050[30], 3}, {&S60050[30], 3}, {NULL}};
const Segment SGI324156[] = {{&S60050[8], 3}, {&S60050[8], 3}, {NULL}};
const Segment SGI324157[] = {{&S60050[13], 10}, {&S60050[13], 10}, {NULL}};
const Segment SGI324158[] = {{&S60050[42], 7}, {&S60050[42], 7}, {NULL}};
const Segment SGI324159[] = {{&S60050[35], 14}, {&S60050[35], 14}, {NULL}};
const Segment SGI324160[] = {{&S60050[114], 7}, {NULL}};
const Segment SGI324161[] = {{&S60050[121], 4}, {NULL}};
const Segment SGI324162[] = {{&S60050[51], 3}, {&S60050[51], 3}, {NULL}};
const Segment SGI324163[] = {{&S77877[3], 5}, {NULL}};
const Segment SGI324164[] = {{&S77877[88], 12}, {NULL}};
const Segment SGI324165[] = {{&S73212[0], 82}, {NULL}};
const Segment SGI324166[] = {{&S73213[0], 77}, {NULL}};
const Segment SGI324167[] = {{&S73214[0], 76}, {NULL}};
const Segment SGI324168[] = {{&S77927[3], 5}, {NULL}};
const Segment SGI324169[] = {{&S77927[3], 8}, {NULL}};
const Segment SGI324170[] = {{&S77928[3], 5}, {NULL}};
const Segment SGI324171[] = {{&S77973[3], 5}, {NULL}};
const Segment SGI324172[] = {{&S77973[123], 13}, {NULL}};
const Segment SGI324173[] = {{&S78024[3], 5}, {NULL}};
const Segment SGI324174[] = {{&S78024[124], 13}, {NULL}};
const Segment SGI324175[] = {{&S73506[0], 83}, {NULL}};
const Segment SGI324176[] = {{&S73511[0], 72}, {NULL}};
const Segment SGI324177[] = {{&S67610[4], 6}, {NULL}};
const Segment SGI324178[] = {{&S67610[26], 3}, {NULL}};
const Segment SGI324179[] = {{&S78119[3], 5}, {NULL}};
const Segment SGI324180[] = {{&S78156[3], 6}, {NULL}};
const Segment SGI324181[] = {{&S78157[3], 6}, {NULL}};
const Segment SGI324182[] = {{&S78158[3], 5}, {NULL}};
const Segment SGI324183[] = {{&S73739[0], 40}, {NULL}};
const Segment SGI324184[] = {{&S73740[0], 46}, {NULL}};
const Segment SGI324185[] = {{&S73742[0], 25}, {NULL}};
const Segment SGI324186[] = {{&S73743[0], 91}, {NULL}};
const Segment SGI324187[] = {{&S73745[0], 34}, {NULL}};
const Segment SGI324188[] = {{&S67838[4], 5}, {NULL}};
const Segment SGI324189[] = {{&S67838[25], 3}, {NULL}};
const Segment SGI324190[] = {{&S73775[0], 118}, {NULL}};
const Segment SGI324191[] = {{&S73776[0], 118}, {NULL}};
const Segment SGI324192[] = {{&S73777[0], 118}, {NULL}};
const Segment SGI324193[] = {{&S73778[0], 118}, {NULL}};
const Segment SGI324194[] = {{&S73780[0], 118}, {NULL}};
const Segment SGI324195[] = {{&S73781[0], 118}, {NULL}};
const Segment SGI324196[] = {{&S73782[0], 118}, {NULL}};
const Segment SGI324197[] = {{&S88571[3], 6}, {NULL}};
const Segment SGI324198[] = {{&S88571[29], 9}, {NULL}};
const Segment SGI324199[] = {{&S64254[4], 9}, {NULL}};
const Segment SGI324200[] = {{&S64254[44], 32}, {NULL}};
const Segment SGI324201[] = {{&S64255[4], 8}, {NULL}};
const Segment SGI324202[] = {{&S64255[43], 32}, {NULL}};
const Segment SGI324203[] = {{&S64256[4], 8}, {NULL}};
const Segment SGI324204[] = {{&S64256[43], 32}, {NULL}};
const Segment SGI324205[] = {{&S64257[4], 8}, {NULL}};
const Segment SGI324206[] = {{&S64257[42], 32}, {NULL}};
const Segment SGI324207[] = {{&S64258[4], 8}, {NULL}};
const Segment SGI324208[] = {{&S64258[42], 32}, {NULL}};
const Segment SGI324209[] = {{&S64259[4], 8}, {NULL}};
const Segment SGI324210[] = {{&S64259[42], 32}, {NULL}};
const Segment SGI324211[] = {{&S64260[4], 8}, {NULL}};
const Segment SGI324212[] = {{&S64260[42], 32}, {NULL}};
const Segment SGI324213[] = {{&S64261[4], 8}, {NULL}};
const Segment SGI324214[] = {{&S64261[42], 32}, {NULL}};
const Segment SGI324215[] = {{&S64262[4], 8}, {NULL}};
const Segment SGI324216[] = {{&S64262[42], 32}, {NULL}};
const Segment SGI324217[] = {{&S64263[4], 8}, {NULL}};
const Segment SGI324218[] = {{&S64263[42], 32}, {NULL}};
const Segment SGI324219[] = {{&S64264[4], 8}, {NULL}};
const Segment SGI324220[] = {{&S64264[44], 32}, {NULL}};
const Segment SGI324221[] = {{&S64265[4], 9}, {NULL}};
const Segment SGI324222[] = {{&S64265[43], 32}, {NULL}};
const Segment SGI324223[] = {{&S64266[4], 8}, {NULL}};
const Segment SGI324224[] = {{&S64266[43], 32}, {NULL}};
const Segment SGI324225[] = {{&S64267[4], 8}, {NULL}};
const Segment SGI324226[] = {{&S64267[42], 32}, {NULL}};
const Segment SGI324227[] = {{&S64268[4], 9}, {NULL}};
const Segment SGI324228[] = {{&S64268[44], 32}, {NULL}};
const Segment SGI324229[] = {{&S64269[4], 9}, {NULL}};
const Segment SGI324230[] = {{&S64269[43], 25}, {NULL}};
const Segment SGI324231[] = {{&S64270[4], 8}, {NULL}};
const Segment SGI324232[] = {{&S64270[42], 25}, {NULL}};
const Segment SGI324233[] = {{&S60838[30], 3}, {NULL}};
const Segment SGI324234[] = {{&S60838[8], 3}, {NULL}};
const Segment SGI324235[] = {{&S60838[13], 10}, {NULL}};
const Segment SGI324236[] = {{&S60838[52], 3}, {NULL}};
const Segment SGI324237[] = {{&S60842[30], 3}, {NULL}};
const Segment SGI324238[] = {{&S60842[8], 3}, {NULL}};
const Segment SGI324239[] = {{&S60842[13], 10}, {NULL}};
const Segment SGI324240[] = {{&S60842[40], 9}, {NULL}};
const Segment SGI324241[] = {{&S60843[30], 3}, {NULL}};
const Segment SGI324242[] = {{&S60843[8], 3}, {NULL}};
const Segment SGI324243[] = {{&S60843[13], 10}, {NULL}};
const Segment SGI324244[] = {{&S60843[35], 3}, {NULL}};
const Segment SGI324245[] = {{&S60843[40], 9}, {NULL}};
const Segment SGI324246[] = {{&S60844[30], 3}, {NULL}};
const Segment SGI324247[] = {{&S60844[8], 3}, {NULL}};
const Segment SGI324248[] = {{&S60844[13], 10}, {NULL}};
const Segment SGI324249[] = {{&S60844[42], 2}, {NULL}};
const Segment SGI324250[] = {{&S60845[30], 3}, {NULL}};
const Segment SGI324251[] = {{&S60845[8], 3}, {NULL}};
const Segment SGI324252[] = {{&S60845[13], 10}, {NULL}};
const Segment SGI324253[] = {{&S60845[40], 9}, {NULL}};
const Segment SGI324254[] = {{&S60846[30], 3}, {NULL}};
const Segment SGI324255[] = {{&S60846[8], 3}, {NULL}};
const Segment SGI324256[] = {{&S60846[13], 10}, {NULL}};
const Segment SGI324257[] = {{&S60846[42], 2}, {NULL}};
const Segment SGI324258[] = {{&S60847[30], 3}, {NULL}};
const Segment SGI324259[] = {{&S60847[8], 3}, {NULL}};
const Segment SGI324260[] = {{&S60847[13], 10}, {NULL}};
const Segment SGI324261[] = {{&S60847[35], 3}, {NULL}};
const Segment SGI324262[] = {{&S60847[40], 9}, {NULL}};
const Segment SGI324263[] = {{&S64436[4], 14}, {NULL}};
const Segment SGI324264[] = {{&S64436[131], 8}, {NULL}};
const Segment SGI324265[] = {{&S78564[3], 5}, {NULL}};
const Segment SGI324266[] = {{&S78564[3], 14}, {NULL}};
const Segment SGI324267[] = {{&S78565[3], 5}, {NULL}};
const Segment SGI324268[] = {{&S78565[3], 12}, {NULL}};
const Segment SGI324269[] = {{&S78566[3], 5}, {NULL}};
const Segment SGI324270[] = {{&S78568[3], 5}, {NULL}};
const Segment SGI324271[] = {{&S78569[3], 5}, {NULL}};
const Segment SGI324272[] = {{&S60918[30], 3}, {NULL}};
const Segment SGI324273[] = {{&S60918[8], 3}, {NULL}};
const Segment SGI324274[] = {{&S60918[13], 10}, {NULL}};
const Segment SGI324275[] = {{&S60918[42], 7}, {NULL}};
const Segment SGI324276[] = {{&S60918[35], 14}, {NULL}};
const Segment SGI324277[] = {{&S60918[131], 15}, {NULL}};
const Segment SGI324278[] = {{&S60918[147], 3}, {NULL}};
const Segment SGI324279[] = {{&S60919[30], 3}, {NULL}};
const Segment SGI324280[] = {{&S60919[8], 3}, {NULL}};
const Segment SGI324281[] = {{&S60919[13], 10}, {NULL}};
const Segment SGI324282[] = {{&S60919[42], 7}, {NULL}};
const Segment SGI324283[] = {{&S60919[35], 14}, {NULL}};
const Segment SGI324284[] = {{&S60919[51], 3}, {NULL}};
const Segment SGI324285[] = {{&S60923[30], 3}, {NULL}};
const Segment SGI324286[] = {{&S60923[8], 3}, {NULL}};
const Segment SGI324287[] = {{&S60923[13], 10}, {NULL}};
const Segment SGI324288[] = {{&S60923[42], 7}, {NULL}};
const Segment SGI324289[] = {{&S60923[61], 3}, {NULL}};
const Segment SGI324290[] = {{&S60924[30], 3}, {NULL}};
const Segment SGI324291[] = {{&S60924[8], 3}, {NULL}};
const Segment SGI324292[] = {{&S60924[13], 10}, {NULL}};
const Segment SGI324293[] = {{&S60924[42], 7}, {NULL}};
const Segment SGI324294[] = {{&S60924[61], 3}, {NULL}};
const Segment SGI324295[] = {{&S74069[0], 32}, {NULL}};
const Segment SGI324296[] = {{&S78725[3], 5}, {NULL}};
const Segment SGI324297[] = {{&S74252[0], 32}, {NULL}};
const Segment SGI324298[] = {{&S61261[30], 3}, {NULL}};
const Segment SGI324299[] = {{&S61261[8], 3}, {NULL}};
const Segment SGI324300[] = {{&S61261[13], 10}, {NULL}};
const Segment SGI324301[] = {{&S61261[35], 3}, {NULL}};
const Segment SGI324302[] = {{&S61261[40], 9}, {NULL}};
const Segment SGI324303[] = {{&S79040[3], 5}, {NULL}};
const Segment SGI324304[] = {{&S79040[124], 13}, {NULL}};
const Segment SGI324305[] = {{&S79124[3], 5}, {NULL}};
const Segment SGI324306[] = {{&S68829[4], 5}, {NULL}};
const Segment SGI324307[] = {{&S68829[25], 3}, {NULL}};
const Segment SGI324308[] = {{&S61628[30], 3}, {NULL}};
const Segment SGI324309[] = {{&S61628[8], 3}, {NULL}};
const Segment SGI324310[] = {{&S61628[13], 10}, {NULL}};
const Segment SGI324311[] = {{&S61628[42], 7}, {NULL}};
const Segment SGI324312[] = {{&S61628[35], 14}, {NULL}};
const Segment SGI324313[] = {{&S61628[80], 5}, {NULL}};
const Segment SGI324314[] = {{&S61628[51], 6}, {NULL}};
const Segment SGI324315[] = {{&S61629[30], 4}, {NULL}};
const Segment SGI324316[] = {{&S61629[8], 3}, {NULL}};
const Segment SGI324317[] = {{&S61629[13], 10}, {NULL}};
const Segment SGI324318[] = {{&S61629[43], 7}, {NULL}};
const Segment SGI324319[] = {{&S61629[36], 14}, {NULL}};
const Segment SGI324320[] = {{&S61629[81], 5}, {NULL}};
const Segment SGI324321[] = {{&S61629[52], 6}, {NULL}};
const Segment SGI324322[] = {{&S61631[30], 3}, {NULL}};
const Segment SGI324323[] = {{&S61631[8], 3}, {NULL}};
const Segment SGI324324[] = {{&S61631[13], 10}, {NULL}};
const Segment SGI324325[] = {{&S61631[42], 7}, {NULL}};
const Segment SGI324326[] = {{&S61631[35], 14}, {NULL}};
const Segment SGI324327[] = {{&S61631[80], 5}, {NULL}};
const Segment SGI324328[] = {{&S61631[51], 6}, {NULL}};
const Segment SGI324329[] = {{&S61632[30], 4}, {NULL}};
const Segment SGI324330[] = {{&S61632[8], 3}, {NULL}};
const Segment SGI324331[] = {{&S61632[13], 10}, {NULL}};
const Segment SGI324332[] = {{&S61632[43], 7}, {NULL}};
const Segment SGI324333[] = {{&S61632[36], 14}, {NULL}};
const Segment SGI324334[] = {{&S61632[81], 5}, {NULL}};
const Segment SGI324335[] = {{&S61632[52], 6}, {NULL}};
const Segment SGI324336[] = {{&S61634[30], 3}, {NULL}};
const Segment SGI324337[] = {{&S61634[8], 3}, {NULL}};
const Segment SGI324338[] = {{&S61634[13], 10}, {NULL}};
const Segment SGI324339[] = {{&S61634[42], 7}, {NULL}};
const Segment SGI324340[] = {{&S61634[35], 14}, {NULL}};
const Segment SGI324341[] = {{&S61634[73], 5}, {NULL}};
const Segment SGI324342[] = {{&S61654[30], 3}, {NULL}};
const Segment SGI324343[] = {{&S61654[8], 3}, {NULL}};
const Segment SGI324344[] = {{&S61654[13], 10}, {NULL}};
const Segment SGI324345[] = {{&S74831[0], 98}, {NULL}};
const Segment SGI324346[] = {{&S74832[0], 98}, {NULL}};
const Segment SGI324347[] = {{&S61780[18], 3}, {NULL}};
const Segment SGI324348[] = {{&S61780[8], 3}, {NULL}};
const Segment SGI324349[] = {{&S61780[13], 8}, {NULL}};
const Segment SGI324350[] = {{&S61780[30], 7}, {NULL}};
const Segment SGI324351[] = {{&S61780[23], 14}, {NULL}};
const Segment SGI324352[] = {{&S61780[39], 6}, {NULL}};
const Segment SGI324353[] = {{&S61781[18], 3}, {NULL}};
const Segment SGI324354[] = {{&S61781[8], 3}, {NULL}};
const Segment SGI324355[] = {{&S61781[13], 8}, {NULL}};
const Segment SGI324356[] = {{&S61781[30], 7}, {NULL}};
const Segment SGI324357[] = {{&S61781[23], 14}, {NULL}};
const Segment SGI324358[] = {{&S61781[39], 5}, {NULL}};
const Segment SGI324359[] = {{&S61782[18], 3}, {NULL}};
const Segment SGI324360[] = {{&S61782[8], 3}, {NULL}};
const Segment SGI324361[] = {{&S61782[13], 8}, {NULL}};
const Segment SGI324362[] = {{&S61782[30], 7}, {NULL}};
const Segment SGI324363[] = {{&S61782[23], 14}, {NULL}};
const Segment SGI324364[] = {{&S61783[30], 3}, {NULL}};
const Segment SGI324365[] = {{&S61783[8], 3}, {NULL}};
const Segment SGI324366[] = {{&S61783[13], 10}, {NULL}};
const Segment SGI324367[] = {{&S61783[42], 7}, {NULL}};
const Segment SGI324368[] = {{&S61783[35], 14}, {NULL}};
const Segment SGI324369[] = {{&S61783[51], 6}, {NULL}};
const Segment SGI324370[] = {{&S79741[3], 5}, {NULL}};
const Segment SGI324371[] = {{&S79741[105], 13}, {NULL}};
const Segment SGI324372[] = {{&S79748[3], 5}, {NULL}};
const Segment SGI324373[] = {{&S79878[3], 5}, {NULL}};
const Segment SGI324374[] = {{&S69576[4], 5}, {NULL}};
const Segment SGI324375[] = {{&S69576[25], 3}, {NULL}};
const Segment SGI324376[] = {{&S62264[51], 3}, {NULL}};
const Segment SGI324377[] = {{&S75429[0], 38}, {NULL}};
const Segment SGI324378[] = {{&S80027[3], 5}, {NULL}};
const Segment SGI324379[] = {{&S80027[28], 9}, {NULL}};
const Segment SGI324380[] = {{&S80028[3], 5}, {NULL}};
const Segment SGI324381[] = {{&S80028[3], 12}, {NULL}};
const Segment SGI324382[] = {{&S80028[35], 9}, {NULL}};
const Segment SGI324383[] = {{&S80030[3], 6}, {NULL}};
const Segment SGI324384[] = {{&S80030[24], 9}, {NULL}};
const Segment SGI324385[] = {{&S80059[3], 5}, {NULL}};
const Segment SGI324386[] = {{&S80059[88], 12}, {NULL}};
const Segment SGI324387[] = {{&S62520[30], 3}, {NULL}};
const Segment SGI324388[] = {{&S62520[8], 3}, {NULL}};
const Segment SGI324389[] = {{&S62520[13], 10}, {NULL}};
const Segment SGI324390[] = {{&S62520[42], 7}, {NULL}};
const Segment SGI324391[] = {{&S62520[35], 14}, {NULL}};
const Segment SGI324392[] = {{&S62520[124], 5}, {NULL}};
const Segment SGI324393[] = {{&S62521[30], 3}, {NULL}};
const Segment SGI324394[] = {{&S62521[8], 3}, {NULL}};
const Segment SGI324395[] = {{&S62521[13], 10}, {NULL}};
const Segment SGI324396[] = {{&S62521[42], 7}, {NULL}};
const Segment SGI324397[] = {{&S62521[60], 5}, {NULL}};
const Segment SGI324398[] = {{&S62522[30], 3}, {NULL}};
const Segment SGI324399[] = {{&S62522[8], 3}, {NULL}};
const Segment SGI324400[] = {{&S62522[13], 10}, {NULL}};
const Segment SGI324401[] = {{&S62522[42], 7}, {NULL}};
const Segment SGI324402[] = {{&S62522[35], 14}, {NULL}};
const Segment SGI324403[] = {{&S62522[111], 5}, {NULL}};
const Segment SGI324404[] = {{&S62523[30], 3}, {NULL}};
const Segment SGI324405[] = {{&S62523[8], 3}, {NULL}};
const Segment SGI324406[] = {{&S62523[13], 10}, {NULL}};
const Segment SGI324407[] = {{&S62523[42], 7}, {NULL}};
const Segment SGI324408[] = {{&S62523[35], 14}, {NULL}};
const Segment SGI324409[] = {{&S62523[179], 5}, {NULL}};
const Segment SGI324410[] = {{&S62524[30], 3}, {NULL}};
const Segment SGI324411[] = {{&S62524[8], 3}, {NULL}};
const Segment SGI324412[] = {{&S62524[13], 10}, {NULL}};
const Segment SGI324413[] = {{&S62524[42], 7}, {NULL}};
const Segment SGI324414[] = {{&S62524[35], 14}, {NULL}};
const Segment SGI324415[] = {{&S62524[131], 5}, {NULL}};
const Segment SGI324416[] = {{&S62524[51], 6}, {NULL}};
const Segment SGI324417[] = {{&S62525[30], 3}, {NULL}};
const Segment SGI324418[] = {{&S62525[8], 3}, {NULL}};
const Segment SGI324419[] = {{&S62525[13], 10}, {NULL}};
const Segment SGI324420[] = {{&S62525[42], 7}, {NULL}};
const Segment SGI324421[] = {{&S62525[35], 14}, {NULL}};
const Segment SGI324422[] = {{&S62525[146], 5}, {NULL}};
const Segment SGI324423[] = {{&S62526[30], 3}, {NULL}};
const Segment SGI324424[] = {{&S62526[8], 3}, {NULL}};
const Segment SGI324425[] = {{&S62526[13], 10}, {NULL}};
const Segment SGI324426[] = {{&S62526[42], 7}, {NULL}};
const Segment SGI324427[] = {{&S62526[35], 14}, {NULL}};
const Segment SGI324428[] = {{&S62526[148], 5}, {NULL}};
const Segment SGI324429[] = {{&S62526[51], 3}, {NULL}};
const Segment SGI324430[] = {{&S62610[30], 3}, {NULL}};
const Segment SGI324431[] = {{&S62610[8], 3}, {NULL}};
const Segment SGI324432[] = {{&S62610[13], 10}, {NULL}};
const Segment SGI324433[] = {{&S62610[42], 7}, {NULL}};
const Segment SGI324434[] = {{&S62611[30], 3}, {NULL}};
const Segment SGI324435[] = {{&S62611[8], 3}, {NULL}};
const Segment SGI324436[] = {{&S62611[13], 10}, {NULL}};
const Segment SGI324437[] = {{&S62611[42], 7}, {NULL}};
const Segment SGI324438[] = {{&S62611[35], 14}, {NULL}};
const Segment SGI324439[] = {{&S62612[30], 3}, {NULL}};
const Segment SGI324440[] = {{&S62612[8], 3}, {NULL}};
const Segment SGI324441[] = {{&S62612[13], 10}, {NULL}};
const Segment SGI324442[] = {{&S62612[42], 7}, {NULL}};
const Segment SGI324443[] = {{&S62612[35], 14}, {NULL}};
const Segment SGI324444[] = {{&S62612[51], 3}, {NULL}};
const Segment SGI324445[] = {{&S62613[30], 3}, {NULL}};
const Segment SGI324446[] = {{&S62613[8], 3}, {NULL}};
const Segment SGI324447[] = {{&S62613[13], 10}, {NULL}};
const Segment SGI324448[] = {{&S62613[42], 7}, {NULL}};
const Segment SGI324449[] = {{&S62613[35], 14}, {NULL}};
const Segment SGI324450[] = {{&S62613[131], 15}, {NULL}};
const Segment SGI324451[] = {{&S62614[30], 3}, {NULL}};
const Segment SGI324452[] = {{&S62614[8], 3}, {NULL}};
const Segment SGI324453[] = {{&S62614[13], 10}, {NULL}};
const Segment SGI324454[] = {{&S62614[42], 7}, {NULL}};
const Segment SGI324455[] = {{&S62614[35], 14}, {NULL}};
const Segment SGI324456[] = {{&S62614[51], 6}, {NULL}};
const Segment SGI324457[] = {{&S62615[30], 3}, {NULL}};
const Segment SGI324458[] = {{&S62615[8], 3}, {NULL}};
const Segment SGI324459[] = {{&S62615[13], 10}, {NULL}};
const Segment SGI324460[] = {{&S62615[35], 8}, {NULL}};
const Segment SGI324461[] = {{&S62616[30], 3}, {NULL}};
const Segment SGI324462[] = {{&S62616[8], 3}, {NULL}};
const Segment SGI324463[] = {{&S62616[13], 10}, {NULL}};
const Segment SGI324464[] = {{&S62616[42], 7}, {NULL}};
const Segment SGI324465[] = {{&S62616[35], 14}, {NULL}};
const Segment SGI324466[] = {{&S62616[138], 15}, {NULL}};
const Segment SGI324467[] = {{&S62616[51], 6}, {NULL}};
const Segment SGI324468[] = {{&S80266[3], 6}, {NULL}};
const Segment SGI324469[] = {{&S80266[3], 11}, {NULL}};
const Segment SGI324470[] = {{&S80266[30], 4}, {NULL}};
const Segment SGI324471[] = {{&S80268[3], 6}, {NULL}};
const Segment SGI324472[] = {{&S80268[3], 13}, {NULL}};
const Segment SGI324473[] = {{&S80268[37], 4}, {NULL}};
const Segment SGI324474[] = {{&S80269[3], 6}, {NULL}};
const Segment SGI324475[] = {{&S90538[3], 5}, {NULL}};
const Segment SGI324476[] = {{&S80368[3], 5}, {NULL}};
const Segment SGI324477[] = {{&S80368[125], 13}, {NULL}};
const Segment SGI324478[] = {{&S66324[4], 5}, {NULL}};
const Segment SGI324479[] = {{&S66324[49], 39}, {NULL}};
const Segment SGI324480[] = {{&S80389[3], 5}, {NULL}};
const Segment SGI324481[] = {{&S80415[3], 5}, {NULL}};
const Segment SGI324482[] = {{&S80415[146], 13}, {NULL}};
const Segment SGI324483[] = {{&S62858[30], 3}, {NULL}};
const Segment SGI324484[] = {{&S62858[8], 3}, {NULL}};
const Segment SGI324485[] = {{&S62858[13], 10}, {NULL}};
const Segment SGI324486[] = {{&S62858[42], 7}, {NULL}};
const Segment SGI324487[] = {{&S62858[35], 14}, {NULL}};
const Segment SGI324488[] = {{&S62858[64], 11}, {NULL}};
const Segment SGI324489[] = {{&S62858[64], 16}, {NULL}};
const Segment SGI324490[] = {{&S62861[30], 3}, {NULL}};
const Segment SGI324491[] = {{&S62861[8], 3}, {NULL}};
const Segment SGI324492[] = {{&S62861[13], 10}, {NULL}};
const Segment SGI324493[] = {{&S62861[42], 7}, {NULL}};
const Segment SGI324494[] = {{&S62861[35], 14}, {NULL}};
const Segment SGI324495[] = {{&S62861[71], 11}, {NULL}};
const Segment SGI324496[] = {{&S62861[51], 6}, {NULL}};
const Segment SGI324497[] = {{&S62861[71], 16}, {NULL}};
const Segment SGI324498[] = {{&S62863[30], 3}, {NULL}};
const Segment SGI324499[] = {{&S62863[8], 3}, {NULL}};
const Segment SGI324500[] = {{&S62863[13], 10}, {NULL}};
const Segment SGI324501[] = {{&S62863[42], 7}, {NULL}};
const Segment SGI324502[] = {{&S62863[35], 14}, {NULL}};
const Segment SGI324503[] = {{&S62863[64], 11}, {NULL}};
const Segment SGI324504[] = {{&S62863[64], 16}, {NULL}};
const Segment SGI324505[] = {{&S62864[30], 3}, {NULL}};
const Segment SGI324506[] = {{&S62864[8], 3}, {NULL}};
const Segment SGI324507[] = {{&S62864[13], 10}, {NULL}};
const Segment SGI324508[] = {{&S62864[42], 7}, {NULL}};
const Segment SGI324509[] = {{&S62864[35], 14}, {NULL}};
const Segment SGI324510[] = {{&S62864[51], 11}, {NULL}};
const Segment SGI324511[] = {{&S62864[51], 16}, {NULL}};
const Segment SGI324512[] = {{&S62865[30], 3}, {NULL}};
const Segment SGI324513[] = {{&S62865[8], 3}, {NULL}};
const Segment SGI324514[] = {{&S62865[13], 10}, {NULL}};
const Segment SGI324515[] = {{&S62865[42], 7}, {NULL}};
const Segment SGI324516[] = {{&S62865[35], 14}, {NULL}};
const Segment SGI324517[] = {{&S62865[64], 11}, {NULL}};
const Segment SGI324518[] = {{&S62865[116], 15}, {NULL}};
const Segment SGI324519[] = {{&S62865[64], 16}, {NULL}};
const Segment SGI324520[] = {{&S62865[51], 3}, {NULL}};
const Segment SGI324521[] = {{&S62866[30], 3}, {NULL}};
const Segment SGI324522[] = {{&S62866[8], 3}, {NULL}};
const Segment SGI324523[] = {{&S62866[13], 10}, {NULL}};
const Segment SGI324524[] = {{&S62866[42], 7}, {NULL}};
const Segment SGI324525[] = {{&S62866[35], 14}, {NULL}};
const Segment SGI324526[] = {{&S62866[72], 11}, {NULL}};
const Segment SGI324527[] = {{&S62866[124], 15}, {NULL}};
const Segment SGI324528[] = {{&S62866[72], 16}, {NULL}};
const Segment SGI324529[] = {{&S62867[30], 3}, {NULL}};
const Segment SGI324530[] = {{&S62867[8], 3}, {NULL}};
const Segment SGI324531[] = {{&S62867[13], 10}, {NULL}};
const Segment SGI324532[] = {{&S62867[42], 7}, {NULL}};
const Segment SGI324533[] = {{&S62867[35], 14}, {NULL}};
const Segment SGI324534[] = {{&S62867[72], 11}, {NULL}};
const Segment SGI324535[] = {{&S62867[124], 15}, {NULL}};
const Segment SGI324536[] = {{&S62867[72], 16}, {NULL}};
const Segment SGI324537[] = {{&S80576[3], 4}, {NULL}};
const Segment SGI324538[] = {{&S76048[0], 32}, {NULL}};
const Segment SGI324539[] = {{&S63003[30], 4}, {NULL}};
const Segment SGI324540[] = {{&S63003[8], 3}, {NULL}};
const Segment SGI324541[] = {{&S63003[13], 10}, {NULL}};
const Segment SGI324542[] = {{&S63003[43], 7}, {NULL}};
const Segment SGI324543[] = {{&S63003[36], 14}, {NULL}};
const Segment SGI324544[] = {{&S63003[79], 11}, {NULL}};
const Segment SGI324545[] = {{&S63003[72], 5}, {NULL}};
const Segment SGI324546[] = {{&S63003[52], 6}, {NULL}};
const Segment SGI324547[] = {{&S63003[79], 16}, {NULL}};
const Segment SGI324548[] = {{&S76079[0], 38}, {NULL}};
const Segment SGI324549[] = {{&S80727[3], 5}, {NULL}};
const Segment SGI324550[] = {{&S80728[3], 4}, {NULL}};
const Segment SGI324551[] = {{&S80806[3], 4}, {NULL}};
const Segment SGI324552[] = {{&S80866[3], 5}, {NULL}};
const Segment SGI324553[] = {{&S80933[3], 5}, {NULL}};
const Segment SGI324554[] = {{&S80935[3], 5}, {NULL}};
const Segment SGI324555[] = {{&S63305[30], 3}, {NULL}};
const Segment SGI324556[] = {{&S63305[8], 3}, {NULL}};
const Segment SGI324557[] = {{&S63305[13], 10}, {NULL}};
const Segment SGI324558[] = {{&S63305[42], 4}, {NULL}};
const Segment SGI324559[] = {{&S63306[30], 4}, {NULL}};
const Segment SGI324560[] = {{&S63306[8], 3}, {NULL}};
const Segment SGI324561[] = {{&S63306[13], 10}, {NULL}};
const Segment SGI324562[] = {{&S63306[43], 4}, {NULL}};
const Segment SGI324563[] = {{&S63307[30], 4}, {NULL}};
const Segment SGI324564[] = {{&S63307[8], 3}, {NULL}};
const Segment SGI324565[] = {{&S63307[13], 10}, {NULL}};
const Segment SGI324566[] = {{&S63307[43], 4}, {NULL}};
const Segment SGI324567[] = {{&S63308[30], 3}, {NULL}};
const Segment SGI324568[] = {{&S63308[8], 3}, {NULL}};
const Segment SGI324569[] = {{&S63308[13], 10}, {NULL}};
const Segment SGI324570[] = {{&S63308[35], 6}, {NULL}};
const Segment SGI324571[] = {{&S66671[4], 4}, {NULL}};
const Segment SGI324572[] = {{&S66672[4], 3}, {NULL}};
const Segment SGI324573[] = {{&S66673[4], 3}, {NULL}};
const Segment SGI324574[] = {{&S66674[4], 2}, {NULL}};
const Segment SGI324575[] = {{&S66675[4], 3}, {NULL}};
const Segment SGI324576[] = {{&S66676[4], 3}, {NULL}};
const Segment SGI324577[] = {{&S81050[3], 4}, {NULL}};
const Segment SGI324578[] = {{&S76341[0], 32}, {NULL}};
const Segment SGI324579[] = {{&S81156[3], 5}, {NULL}};
const Segment SGI324580[] = {{&S81156[105], 13}, {NULL}};
const Segment SGI324581[] = {{&S63518[30], 4}, {NULL}};
const Segment SGI324582[] = {{&S63518[8], 3}, {NULL}};
const Segment SGI324583[] = {{&S63518[13], 10}, {NULL}};
const Segment SGI324584[] = {{&S63518[43], 7}, {NULL}};
const Segment SGI324585[] = {{&S63518[36], 14}, {NULL}};
const Segment SGI324586[] = {{&S63519[30], 4}, {NULL}};
const Segment SGI324587[] = {{&S63519[8], 3}, {NULL}};
const Segment SGI324588[] = {{&S63519[13], 10}, {NULL}};
const Segment SGI324589[] = {{&S63519[43], 7}, {NULL}};
const Segment SGI324590[] = {{&S63519[36], 14}, {NULL}};
const Segment SGI324591[] = {{&S63519[52], 6}, {NULL}};
const Segment SGI324592[] = {{&S63520[18], 4}, {NULL}};
const Segment SGI324593[] = {{&S63520[8], 3}, {NULL}};
const Segment SGI324594[] = {{&S63520[13], 9}, {NULL}};
const Segment SGI324595[] = {{&S63520[31], 7}, {NULL}};
const Segment SGI324596[] = {{&S63520[24], 14}, {NULL}};
const Segment SGI324597[] = {{&S63520[40], 6}, {NULL}};
const Segment SGI324598[] = {{&S81195[3], 5}, {NULL}};
const Segment SGI324599[] = {{&S81195[104], 13}, {NULL}};
const Segment SGI324600[] = {{&S63521[18], 4}, {NULL}};
const Segment SGI324601[] = {{&S63521[8], 3}, {NULL}};
const Segment SGI324602[] = {{&S63521[13], 9}, {NULL}};
const Segment SGI324603[] = {{&S63521[31], 7}, {NULL}};
const Segment SGI324604[] = {{&S63521[24], 14}, {NULL}};
const Segment SGI324605[] = {{&S63521[40], 5}, {NULL}};
const Segment SGI324606[] = {{&S63522[30], 4}, {NULL}};
const Segment SGI324607[] = {{&S63522[8], 3}, {NULL}};
const Segment SGI324608[] = {{&S63522[13], 10}, {NULL}};
const Segment SGI324609[] = {{&S63522[43], 7}, {NULL}};
const Segment SGI324610[] = {{&S63522[36], 14}, {NULL}};
const Segment SGI324611[] = {{&S63522[52], 5}, {NULL}};
const Segment SGI324612[] = {{&S81314[3], 4}, {NULL}};
const Segment SGI324613[] = {{&S67014[4], 14}, {NULL}};
const Segment SGI324614[] = {{&S67014[131], 41}, {NULL}};
const Segment SGI324615[] = {{&S81379[3], 5}, {NULL}};
const Segment SGI324616[] = {{&S81379[3], 12}, {NULL}};
const Segment SGI324617[] = {{&S81379[35], 9}, {NULL}};
const Segment SGI324618[] = {{&S81380[3], 5}, {NULL}};
const Segment SGI324619[] = {{&S81380[23], 9}, {NULL}};
const Segment SGI324620[] = {{&S81447[3], 4}, {NULL}};
const Segment SGI324621[] = {{&S91673[3], 5}, {NULL}};
const Segment SGI324622[] = {{&S91673[28], 9}, {NULL}};
const Segment SGI324623[] = {{&S81520[3], 4}, {NULL}};
const Segment SGI324624[] = {{&S81602[3], 5}, {NULL}};
const Segment SGI324625[] = {{&S81602[107], 12}, {NULL}};
const Segment SGI324626[] = {{&S71126[4], 6}, {NULL}};
const Segment SGI324627[] = {{&S71126[26], 3}, {NULL}};
const Segment SGI324628[] = {{&S76861[0], 76}, {NULL}};
const Segment SGI324629[] = {{&S81733[3], 5}, {NULL}};
const Segment SGI324630[] = {{&S64135[30], 3}, {NULL}};
const Segment SGI324631[] = {{&S64135[8], 3}, {NULL}};
const Segment SGI324632[] = {{&S64135[13], 10}, {NULL}};
const Segment SGI324633[] = {{&S64135[42], 7}, {NULL}};
const Segment SGI324634[] = {{&S64135[35], 14}, {NULL}};
const Segment SGI324635[] = {{&S64135[51], 3}, {NULL}};
const Segment SGI324636[] = {{&S64135[69], 5}, {NULL}};
const Segment SGI324637[] = {{&S64136[30], 3}, {NULL}};
const Segment SGI324638[] = {{&S64136[8], 3}, {NULL}};
const Segment SGI324639[] = {{&S64136[13], 10}, {NULL}};
const Segment SGI324640[] = {{&S64136[42], 7}, {NULL}};
const Segment SGI324641[] = {{&S64136[35], 14}, {NULL}};
const Segment SGI324642[] = {{&S64136[51], 3}, {&S64136[51], 3}, {NULL}};
const Segment SGI324643[] = {{&S64136[69], 5}, {NULL}};
const Segment SGI324644[] = {{&S81781[3], 5}, {NULL}};
const Segment SGI324645[] = {{&S81848[3], 5}, {NULL}};
const Segment SGI324646[] = {{&S92076[3], 4}, {NULL}};
const Segment SGI324647[] = {{&S92076[28], 4}, {NULL}};
const Segment SGI324648[] = {{&S77160[0], 85}, {NULL}};
const Segment SGI324649[] = {{&S77161[0], 87}, {NULL}};
const Segment SGI324650[] = {{&S77162[0], 85}, {NULL}};
const Segment SGI324651[] = {{&S77163[0], 85}, {NULL}};
const Segment SGI324652[] = {{&S77164[0], 85}, {NULL}};
const Segment SGI324653[] = {{&S77165[0], 86}, {NULL}};
const Segment SGI324654[] = {{&S77166[0], 90}, {NULL}};
const Segment SGI324655[] = {{&S77167[0], 85}, {NULL}};
const Segment SGI324656[] = {{&S77168[0], 83}, {NULL}};
const Segment SGI324657[] = {{&S77169[0], 85}, {NULL}};
const Segment SGI324658[] = {{&S77170[0], 85}, {NULL}};
const Segment SGI324659[] = {{&S77171[0], 85}, {NULL}};
const Segment SGI324660[] = {{&S77172[0], 83}, {NULL}};
const Segment SGI324661[] = {{&S77173[0], 85}, {NULL}};
const Segment SGI324662[] = {{&S77174[0], 87}, {NULL}};
const Segment SGI324663[] = {{&S77175[0], 85}, {NULL}};
const Segment SGI324664[] = {{&S77176[0], 85}, {NULL}};
const Segment SGI324665[] = {{&S77177[0], 85}, {NULL}};
const Segment SGI324666[] = {{&S77178[0], 92}, {NULL}};
const Segment SGI324667[] = {{&S82100[3], 5}, {NULL}};
const Segment SGI324668[] = {{&S82101[3], 5}, {NULL}};
const Segment SGI324669[] = {{&S64469[30], 4}, {NULL}};
const Segment SGI324670[] = {{&S64469[8], 3}, {NULL}};
const Segment SGI324671[] = {{&S64469[13], 10}, {NULL}};
const Segment SGI324672[] = {{&S64469[43], 7}, {NULL}};
const Segment SGI324673[] = {{&S64469[36], 14}, {NULL}};
const Segment SGI324674[] = {{&S64469[65], 11}, {NULL}};
const Segment SGI324675[] = {{&S64469[65], 16}, {NULL}};
const Segment SGI324676[] = {{&S64470[30], 4}, {NULL}};
const Segment SGI324677[] = {{&S64470[8], 3}, {NULL}};
const Segment SGI324678[] = {{&S64470[13], 10}, {NULL}};
const Segment SGI324679[] = {{&S64470[43], 7}, {NULL}};
const Segment SGI324680[] = {{&S64470[36], 14}, {NULL}};
const Segment SGI324681[] = {{&S64470[72], 11}, {NULL}};
const Segment SGI324682[] = {{&S64470[52], 6}, {NULL}};
const Segment SGI324683[] = {{&S64470[72], 16}, {NULL}};
const Segment SGI324684[] = {{&S64472[30], 3}, {NULL}};
const Segment SGI324685[] = {{&S64472[8], 3}, {NULL}};
const Segment SGI324686[] = {{&S64472[13], 10}, {NULL}};
const Segment SGI324687[] = {{&S64472[42], 7}, {NULL}};
const Segment SGI324688[] = {{&S64472[35], 14}, {NULL}};
const Segment SGI324689[] = {{&S64472[151], 11}, {NULL}};
const Segment SGI324690[] = {{&S64472[51], 6}, {NULL}};
const Segment SGI324691[] = {{&S64472[151], 16}, {NULL}};
const Segment SGI324692[] = {{&S92296[3], 4}, {NULL}};
const Segment SGI324693[] = {{&S82294[3], 5}, {NULL}};
const Segment SGI324694[] = {{&S82294[124], 13}, {NULL}};
const Segment SGI324695[] = {{&S71856[4], 10}, {NULL}};
const Segment SGI324696[] = {{&S77418[0], 34}, {NULL}};
const Segment SGI324697[] = {{&S68248[4], 5}, {NULL}};
const Segment SGI324698[] = {{&S68248[27], 8}, {NULL}};
const Segment SGI324699[] = {{&S68248[43], 5}, {NULL}};
const Segment SGI324700[] = {{&S68249[4], 5}, {NULL}};
const Segment SGI324701[] = {{&S68249[27], 8}, {NULL}};
const Segment SGI324702[] = {{&S68249[43], 5}, {NULL}};
const Segment SGI324703[] = {{&S68250[4], 5}, {NULL}};
const Segment SGI324704[] = {{&S68250[27], 8}, {NULL}};
const Segment SGI324705[] = {{&S68250[43], 5}, {NULL}};
const Segment SGI324706[] = {{&S68251[4], 5}, {NULL}};
const Segment SGI324707[] = {{&S68251[27], 8}, {NULL}};
const Segment SGI324708[] = {{&S68251[43], 5}, {NULL}};
const Segment SGI324709[] = {{&S68252[4], 4}, {NULL}};
const Segment SGI324710[] = {{&S68252[26], 8}, {NULL}};
const Segment SGI324711[] = {{&S68252[42], 5}, {NULL}};
const Segment SGI324712[] = {{&S68253[4], 5}, {NULL}};
const Segment SGI324713[] = {{&S68253[27], 8}, {NULL}};
const Segment SGI324714[] = {{&S68253[43], 5}, {NULL}};
const Segment SGI324715[] = {{&S68254[4], 5}, {NULL}};
const Segment SGI324716[] = {{&S68254[27], 8}, {NULL}};
const Segment SGI324717[] = {{&S68254[43], 5}, {NULL}};
const Segment SGI324718[] = {{&S68255[4], 5}, {NULL}};
const Segment SGI324719[] = {{&S68255[27], 8}, {NULL}};
const Segment SGI324720[] = {{&S68255[43], 5}, {NULL}};
const Segment SGI324721[] = {{&S82517[3], 5}, {NULL}};
const Segment SGI324722[] = {{&S82517[28], 8}, {NULL}};
const Segment SGI324723[] = {{&S82519[3], 5}, {NULL}};
const Segment SGI324724[] = {{&S64826[30], 3}, {NULL}};
const Segment SGI324725[] = {{&S64826[8], 3}, {NULL}};
const Segment SGI324726[] = {{&S64826[13], 10}, {NULL}};
const Segment SGI324727[] = {{&S64826[42], 7}, {NULL}};
const Segment SGI324728[] = {{&S64826[35], 14}, {NULL}};
const Segment SGI324729[] = {{&S64826[118], 15}, {NULL}};
const Segment SGI324730[] = {{&S64827[30], 3}, {NULL}};
const Segment SGI324731[] = {{&S64827[8], 3}, {NULL}};
const Segment SGI324732[] = {{&S64827[13], 10}, {NULL}};
const Segment SGI324733[] = {{&S64827[42], 7}, {NULL}};
const Segment SGI324734[] = {{&S64827[35], 14}, {NULL}};
const Segment SGI324735[] = {{&S64876[30], 3}, {NULL}};
const Segment SGI324736[] = {{&S64876[8], 3}, {NULL}};
const Segment SGI324737[] = {{&S64876[13], 10}, {NULL}};
const Segment SGI324738[] = {{&S64876[40], 5}, {NULL}};
const Segment SGI324739[] = {{&S64876[35], 3}, {NULL}};
const Segment SGI324740[] = {{&S64877[30], 3}, {NULL}};
const Segment SGI324741[] = {{&S64877[8], 3}, {NULL}};
const Segment SGI324742[] = {{&S64877[13], 10}, {NULL}};
const Segment SGI324743[] = {{&S64877[40], 6}, {NULL}};
const Segment SGI324744[] = {{&S64877[35], 3}, {NULL}};
const Segment SGI324745[] = {{&S64878[30], 3}, {NULL}};
const Segment SGI324746[] = {{&S64878[8], 3}, {NULL}};
const Segment SGI324747[] = {{&S64878[13], 10}, {NULL}};
const Segment SGI324748[] = {{&S64878[35], 3}, {NULL}};
const Segment SGI324749[] = {{&S82610[3], 6}, {NULL}};
const Segment SGI324750[] = {{&S77624[0], 73}, {NULL}};
const Segment SGI324751[] = {{&S64978[30], 3}, {NULL}};
const Segment SGI324752[] = {{&S64978[8], 3}, {NULL}};
const Segment SGI324753[] = {{&S64978[13], 10}, {NULL}};
const Segment SGI324754[] = {{&S64978[42], 7}, {NULL}};
const Segment SGI324755[] = {{&S64978[35], 14}, {NULL}};
const Segment SGI324756[] = {{&S64978[179], 11}, {NULL}};
const Segment SGI324757[] = {{&S64978[179], 15}, {NULL}};
const Segment SGI324758[] = {{&S64979[30], 3}, {NULL}};
const Segment SGI324759[] = {{&S64979[8], 3}, {NULL}};
const Segment SGI324760[] = {{&S64979[13], 10}, {NULL}};
const Segment SGI324761[] = {{&S64979[42], 7}, {NULL}};
const Segment SGI324762[] = {{&S64979[35], 14}, {NULL}};
const Segment SGI324763[] = {{&S64979[147], 11}, {NULL}};
const Segment SGI324764[] = {{&S64979[147], 15}, {NULL}};
const Segment SGI324765[] = {{&S64980[30], 3}, {NULL}};
const Segment SGI324766[] = {{&S64980[8], 3}, {NULL}};
const Segment SGI324767[] = {{&S64980[13], 10}, {NULL}};
const Segment SGI324768[] = {{&S64980[42], 7}, {NULL}};
const Segment SGI324769[] = {{&S64980[35], 14}, {NULL}};
const Segment SGI324770[] = {{&S64980[107], 11}, {NULL}};
const Segment SGI324771[] = {{&S64980[107], 15}, {NULL}};
const Segment SGI324772[] = {{&S64980[51], 3}, {NULL}};
const Segment SGI324773[] = {{&S77742[0], 32}, {NULL}};
const Segment SGI324774[] = {{&S92836[3], 4}, {NULL}};
const Segment SGI324775[] = {{&S77839[0], 32}, {NULL}};
const Segment SGI324776[] = {{&S82880[3], 5}, {NULL}};
const Segment SGI324777[] = {{&S82933[3], 5}, {NULL}};
const Segment SGI324778[] = {{&S82933[105], 13}, {NULL}};
const Segment SGI324779[] = {{&S82982[3], 5}, {NULL}};
const Segment SGI324780[] = {{&S82982[88], 12}, {NULL}};
const Segment SGI324781[] = {{&S93063[3], 4}, {NULL}};
const Segment SGI324782[] = {{&S83055[3], 5}, {NULL}};
const Segment SGI324783[] = {{&S83055[127], 13}, {NULL}};
const Segment SGI324784[] = {{&S83072[3], 5}, {NULL}};
const Segment SGI324785[] = {{&S83072[123], 13}, {NULL}};
const Segment SGI324786[] = {{&S65375[30], 3}, {NULL}};
const Segment SGI324787[] = {{&S65375[8], 3}, {NULL}};
const Segment SGI324788[] = {{&S65375[13], 10}, {NULL}};
const Segment SGI324789[] = {{&S65375[35], 3}, {NULL}};
const Segment SGI324790[] = {{&S65376[30], 3}, {NULL}};
const Segment SGI324791[] = {{&S65376[8], 3}, {NULL}};
const Segment SGI324792[] = {{&S65376[13], 10}, {NULL}};
const Segment SGI324793[] = {{&S65376[40], 5}, {NULL}};
const Segment SGI324794[] = {{&S65376[35], 3}, {NULL}};
const Segment SGI324795[] = {{&S65377[30], 3}, {NULL}};
const Segment SGI324796[] = {{&S65377[8], 3}, {NULL}};
const Segment SGI324797[] = {{&S65377[13], 10}, {NULL}};
const Segment SGI324798[] = {{&S65377[40], 6}, {NULL}};
const Segment SGI324799[] = {{&S65377[35], 3}, {NULL}};
const Segment SGI324800[] = {{&S83078[3], 5}, {NULL}};
const Segment SGI324801[] = {{&S83078[124], 13}, {NULL}};
const Segment SGI324802[] = {{&S65440[30], 3}, {NULL}};
const Segment SGI324803[] = {{&S65440[8], 3}, {NULL}};
const Segment SGI324804[] = {{&S65440[13], 10}, {NULL}};
const Segment SGI324805[] = {{&S65440[42], 7}, {NULL}};
const Segment SGI324806[] = {{&S65440[35], 14}, {NULL}};
const Segment SGI324807[] = {{&S65440[80], 5}, {NULL}};
const Segment SGI324808[] = {{&S65440[51], 6}, {NULL}};
const Segment SGI324809[] = {{&S65441[30], 3}, {NULL}};
const Segment SGI324810[] = {{&S65441[8], 3}, {NULL}};
const Segment SGI324811[] = {{&S65441[13], 10}, {NULL}};
const Segment SGI324812[] = {{&S65441[42], 7}, {NULL}};
const Segment SGI324813[] = {{&S65441[35], 14}, {NULL}};
const Segment SGI324814[] = {{&S65441[73], 5}, {NULL}};
const Segment SGI324815[] = {{&S65442[30], 4}, {NULL}};
const Segment SGI324816[] = {{&S65442[8], 3}, {NULL}};
const Segment SGI324817[] = {{&S65442[13], 10}, {NULL}};
const Segment SGI324818[] = {{&S65442[43], 7}, {NULL}};
const Segment SGI324819[] = {{&S65442[36], 14}, {NULL}};
const Segment SGI324820[] = {{&S65442[81], 5}, {NULL}};
const Segment SGI324821[] = {{&S65442[52], 6}, {NULL}};
const Segment SGI324822[] = {{&S83127[3], 5}, {NULL}};
const Segment SGI324823[] = {{&S65480[30], 3}, {NULL}};
const Segment SGI324824[] = {{&S65480[8], 3}, {NULL}};
const Segment SGI324825[] = {{&S65480[13], 10}, {NULL}};
const Segment SGI324826[] = {{&S65481[30], 3}, {NULL}};
const Segment SGI324827[] = {{&S65481[8], 3}, {NULL}};
const Segment SGI324828[] = {{&S65481[13], 10}, {NULL}};
const Segment SGI324829[] = {{&S65482[30], 3}, {NULL}};
const Segment SGI324830[] = {{&S65482[8], 3}, {NULL}};
const Segment SGI324831[] = {{&S65482[13], 10}, {NULL}};
const Segment SGI324832[] = {{&S65482[35], 5}, {NULL}};
const Segment SGI324833[] = {{&S65483[30], 3}, {NULL}};
const Segment SGI324834[] = {{&S65483[8], 3}, {NULL}};
const Segment SGI324835[] = {{&S65483[13], 10}, {NULL}};
const Segment SGI324836[] = {{&S65483[35], 5}, {NULL}};
const Segment SGI324837[] = {{&S68995[4], 8}, {NULL}};
const Segment SGI324838[] = {{&S68996[4], 8}, {NULL}};
const Segment SGI324839[] = {{&S68997[4], 9}, {NULL}};
const Segment SGI324840[] = {{&S68998[4], 7}, {NULL}};
const Segment SGI324841[] = {{&S78224[0], 77}, {NULL}};
const Segment SGI324842[] = {{&S78273[0], 40}, {NULL}};
const Segment SGI324843[] = {{&S78274[0], 40}, {NULL}};
const Segment SGI324844[] = {{&S78275[0], 40}, {NULL}};
const Segment SGI324845[] = {{&S78276[0], 40}, {NULL}};
const Segment SGI324846[] = {{&S78277[0], 40}, {NULL}};
const Segment SGI324847[] = {{&S78278[0], 40}, {NULL}};
const Segment SGI324848[] = {{&S83296[3], 4}, {NULL}};
const Segment SGI324849[] = {{&S83296[70], 3}, {NULL}};
const Segment SGI324850[] = {{&S93516[3], 4}, {NULL}};
const Segment SGI324851[] = {{&S93524[3], 4}, {NULL}};
const Segment SGI324852[] = {{&S93525[3], 5}, {NULL}};
const Segment SGI324853[] = {{&S69380[4], 5}, {NULL}};
const Segment SGI324854[] = {{&S72925[4], 5}, {NULL}};
const Segment SGI324855[] = {{&S72925[25], 3}, {NULL}};
const Segment SGI324856[] = {{&S101170[7], 4}, {NULL}};
const Segment SGI324857[] = {{&S101170[0], 65}, {NULL}};
const Segment SGI324858[] = {{&S101171[7], 4}, {NULL}};
const Segment SGI324859[] = {{&S101171[0], 67}, {NULL}};
const Segment SGI324860[] = {{&S93735[3], 5}, {NULL}};
const Segment SGI324861[] = {{&S83655[3], 5}, {NULL}};
const Segment SGI324862[] = {{&S83655[124], 13}, {NULL}};
const Segment SGI324863[] = {{&S65975[8], 3}, {NULL}};
const Segment SGI324864[] = {{&S65975[13], 10}, {NULL}};
const Segment SGI324865[] = {{&S65975[42], 7}, {NULL}};
const Segment SGI324866[] = {{&S65975[35], 14}, {NULL}};
const Segment SGI324867[] = {{&S69660[4], 10}, {NULL}};
const Segment SGI324868[] = {{&S69660[19], 65}, {NULL}};
const Segment SGI324869[] = {{&S69661[4], 2}, {NULL}};
const Segment SGI324870[] = {{&S65976[30], 3}, {NULL}};
const Segment SGI324871[] = {{&S69661[29], 55}, {NULL}};
const Segment SGI324872[] = {{&S65976[8], 3}, {NULL}};
const Segment SGI324873[] = {{&S65976[13], 10}, {NULL}};
const Segment SGI324874[] = {{&S65976[42], 7}, {NULL}};
const Segment SGI324875[] = {{&S65976[35], 14}, {NULL}};
const Segment SGI324876[] = {{&S69662[4], 2}, {NULL}};
const Segment SGI324877[] = {{&S69662[29], 55}, {NULL}};
const Segment SGI324878[] = {{&S66052[30], 4}, {NULL}};
const Segment SGI324879[] = {{&S66052[8], 3}, {NULL}};
const Segment SGI324880[] = {{&S66052[13], 10}, {NULL}};
const Segment SGI324881[] = {{&S66052[43], 7}, {NULL}};
const Segment SGI324882[] = {{&S66052[36], 14}, {NULL}};
const Segment SGI324883[] = {{&S66052[79], 11}, {NULL}};
const Segment SGI324884[] = {{&S66052[72], 5}, {NULL}};
const Segment SGI324885[] = {{&S66052[52], 6}, {NULL}};
const Segment SGI324886[] = {{&S66052[79], 16}, {NULL}};
const Segment SGI324887[] = {{&S66053[30], 4}, {NULL}};
const Segment SGI324888[] = {{&S66053[8], 3}, {NULL}};
const Segment SGI324889[] = {{&S66053[13], 10}, {NULL}};
const Segment SGI324890[] = {{&S66053[43], 7}, {NULL}};
const Segment SGI324891[] = {{&S66053[36], 14}, {NULL}};
const Segment SGI324892[] = {{&S66053[72], 11}, {NULL}};
const Segment SGI324893[] = {{&S66053[65], 5}, {NULL}};
const Segment SGI324894[] = {{&S66053[72], 16}, {NULL}};
const Segment SGI324895[] = {{&S83918[3], 5}, {NULL}};
const Segment SGI324896[] = {{&S73312[4], 5}, {NULL}};
const Segment SGI324897[] = {{&S73312[25], 3}, {NULL}};
const Segment SGI324898[] = {{&S66196[30], 3}, {NULL}};
const Segment SGI324899[] = {{&S66196[8], 3}, {NULL}};
const Segment SGI324900[] = {{&S66196[13], 10}, {NULL}};
const Segment SGI324901[] = {{&S66196[40], 5}, {NULL}};
const Segment SGI324902[] = {{&S66196[35], 3}, {NULL}};
const Segment SGI324903[] = {{&S66197[30], 3}, {NULL}};
const Segment SGI324904[] = {{&S66197[8], 3}, {NULL}};
const Segment SGI324905[] = {{&S66197[13], 10}, {NULL}};
const Segment SGI324906[] = {{&S66197[35], 3}, {NULL}};
const Segment SGI324907[] = {{&S66198[30], 3}, {NULL}};
const Segment SGI324908[] = {{&S66198[8], 3}, {NULL}};
const Segment SGI324909[] = {{&S66198[13], 10}, {NULL}};
const Segment SGI324910[] = {{&S66198[40], 6}, {NULL}};
const Segment SGI324911[] = {{&S66198[35], 3}, {NULL}};
const Segment SGI324912[] = {{&S79057[0], 38}, {NULL}};
const Segment SGI324913[] = {{&S84168[3], 5}, {NULL}};
const Segment SGI324914[] = {{&S66315[30], 3}, {NULL}};
const Segment SGI324915[] = {{&S66315[8], 3}, {NULL}};
const Segment SGI324916[] = {{&S66315[13], 10}, {NULL}};
const Segment SGI324917[] = {{&S66315[42], 7}, {NULL}};
const Segment SGI324918[] = {{&S66315[35], 14}, {NULL}};
const Segment SGI324919[] = {{&S66315[128], 15}, {NULL}};
const Segment SGI324920[] = {{&S66398[30], 3}, {NULL}};
const Segment SGI324921[] = {{&S66398[8], 3}, {NULL}};
const Segment SGI324922[] = {{&S66398[13], 10}, {NULL}};
const Segment SGI324923[] = {{&S66398[39], 15}, {NULL}};
const Segment SGI324924[] = {{&S66398[35], 3}, {NULL}};
const Segment SGI324925[] = {{&S66399[30], 3}, {NULL}};
const Segment SGI324926[] = {{&S66399[8], 3}, {NULL}};
const Segment SGI324927[] = {{&S66399[13], 10}, {NULL}};
const Segment SGI324928[] = {{&S66399[39], 15}, {NULL}};
const Segment SGI324929[] = {{&S66399[35], 3}, {NULL}};
const Segment SGI324930[] = {{&S66400[30], 3}, {NULL}};
const Segment SGI324931[] = {{&S66400[8], 3}, {NULL}};
const Segment SGI324932[] = {{&S66400[13], 10}, {NULL}};
const Segment SGI324933[] = {{&S66400[39], 15}, {NULL}};
const Segment SGI324934[] = {{&S66400[35], 3}, {NULL}};
const Segment SGI324935[] = {{&S66401[30], 3}, {NULL}};
const Segment SGI324936[] = {{&S66401[8], 3}, {NULL}};
const Segment SGI324937[] = {{&S66401[13], 10}, {NULL}};
const Segment SGI324938[] = {{&S66401[39], 15}, {NULL}};
const Segment SGI324939[] = {{&S66401[35], 3}, {NULL}};
const Segment SGI324940[] = {{&S66402[30], 3}, {NULL}};
const Segment SGI324941[] = {{&S66402[8], 3}, {NULL}};
const Segment SGI324942[] = {{&S66402[13], 10}, {NULL}};
const Segment SGI324943[] = {{&S66402[39], 15}, {NULL}};
const Segment SGI324944[] = {{&S66402[35], 3}, {NULL}};
const Segment SGI324945[] = {{&S84278[3], 5}, {NULL}};
const Segment SGI324946[] = {{&S66403[30], 3}, {NULL}};
const Segment SGI324947[] = {{&S66403[8], 3}, {NULL}};
const Segment SGI324948[] = {{&S66403[13], 10}, {NULL}};
const Segment SGI324949[] = {{&S84278[107], 12}, {NULL}};
const Segment SGI324950[] = {{&S66403[39], 15}, {NULL}};
const Segment SGI324951[] = {{&S66403[35], 3}, {NULL}};
const Segment SGI324952[] = {{&S66404[30], 3}, {NULL}};
const Segment SGI324953[] = {{&S66404[8], 3}, {NULL}};
const Segment SGI324954[] = {{&S66404[13], 10}, {NULL}};
const Segment SGI324955[] = {{&S66404[39], 15}, {NULL}};
const Segment SGI324956[] = {{&S66404[35], 3}, {NULL}};
const Segment SGI324957[] = {{&S84289[3], 4}, {NULL}};
const Segment SGI324958[] = {{&S79224[0], 39}, {NULL}};
const Segment SGI324959[] = {{&S79225[0], 38}, {NULL}};
const Segment SGI324960[] = {{&S79226[0], 39}, {NULL}};
const Segment SGI324961[] = {{&S84362[3], 10}, {NULL}};
const Segment SGI324962[] = {{&S84363[3], 5}, {NULL}};
const Segment SGI324963[] = {{&S84410[3], 4}, {NULL}};
const Segment SGI324964[] = {{&S84411[3], 4}, {NULL}};
const Segment SGI324965[] = {{&S84411[3], 11}, {NULL}};
const Segment SGI324966[] = {{&S94401[3], 4}, {NULL}};
const Segment SGI324967[] = {{&S66521[8], 3}, {NULL}};
const Segment SGI324968[] = {{&S66521[13], 10}, {NULL}};
const Segment SGI324969[] = {{&S66521[42], 2}, {NULL}};
const Segment SGI324970[] = {{&S94410[3], 4}, {NULL}};
const Segment SGI324971[] = {{&S66522[8], 3}, {NULL}};
const Segment SGI324972[] = {{&S66522[13], 10}, {NULL}};
const Segment SGI324973[] = {{&S66522[40], 9}, {NULL}};
const Segment SGI324974[] = {{&S66523[8], 3}, {NULL}};
const Segment SGI324975[] = {{&S66523[13], 10}, {NULL}};
const Segment SGI324976[] = {{&S66523[35], 3}, {NULL}};
const Segment SGI324977[] = {{&S66523[40], 9}, {NULL}};
const Segment SGI324978[] = {{&S79412[0], 28}, {NULL}};
const Segment SGI324979[] = {{&S66634[29], 3}, {NULL}};
const Segment SGI324980[] = {{&S66634[8], 3}, {NULL}};
const Segment SGI324981[] = {{&S66634[13], 10}, {NULL}};
const Segment SGI324982[] = {{&S66634[42], 7}, {NULL}};
const Segment SGI324983[] = {{&S66634[35], 14}, {NULL}};
const Segment SGI324984[] = {{&S66635[29], 3}, {NULL}};
const Segment SGI324985[] = {{&S66635[8], 3}, {NULL}};
const Segment SGI324986[] = {{&S66635[13], 10}, {NULL}};
const Segment SGI324987[] = {{&S66635[42], 7}, {NULL}};
const Segment SGI324988[] = {{&S66635[35], 14}, {NULL}};
const Segment SGI324989[] = {{&S66636[8], 3}, {NULL}};
const Segment SGI324990[] = {{&S66636[13], 10}, {NULL}};
const Segment SGI324991[] = {{&S66636[42], 7}, {NULL}};
const Segment SGI324992[] = {{&S66636[35], 14}, {NULL}};
const Segment SGI324993[] = {{&S66637[30], 3}, {NULL}};
const Segment SGI324994[] = {{&S66637[8], 3}, {NULL}};
const Segment SGI324995[] = {{&S66637[13], 10}, {NULL}};
const Segment SGI324996[] = {{&S66637[42], 7}, {NULL}};
const Segment SGI324997[] = {{&S66637[35], 14}, {NULL}};
const Segment SGI324998[] = {{&S84550[3], 5}, {NULL}};
const Segment SGI324999[] = {{&S84550[124], 13}, {NULL}};
const Segment SGI325000[] = {{&S102020[7], 4}, {NULL}};
const Segment SGI325001[] = {{&S102020[0], 106}, {NULL}};
const Segment SGI325002[] = {{&S70500[4], 4}, {NULL}};
const Segment SGI325003[] = {{&S70500[26], 11}, {NULL}};
const Segment SGI325004[] = {{&S70500[38], 5}, {NULL}};
const Segment SGI325005[] = {{&S66855[30], 3}, {NULL}};
const Segment SGI325006[] = {{&S66855[8], 3}, {NULL}};
const Segment SGI325007[] = {{&S66855[13], 10}, {NULL}};
const Segment SGI325008[] = {{&S66855[42], 7}, {NULL}};
const Segment SGI325009[] = {{&S66855[35], 14}, {NULL}};
const Segment SGI325010[] = {{&S66855[64], 11}, {NULL}};
const Segment SGI325011[] = {{&S66855[64], 16}, {NULL}};
const Segment SGI325012[] = {{&S66859[30], 3}, {NULL}};
const Segment SGI325013[] = {{&S66859[8], 3}, {NULL}};
const Segment SGI325014[] = {{&S66859[13], 10}, {NULL}};
const Segment SGI325015[] = {{&S66859[42], 7}, {NULL}};
const Segment SGI325016[] = {{&S66859[35], 14}, {NULL}};
const Segment SGI325017[] = {{&S66859[71], 11}, {NULL}};
const Segment SGI325018[] = {{&S66859[51], 6}, {NULL}};
const Segment SGI325019[] = {{&S66859[71], 16}, {NULL}};
const Segment SGI325020[] = {{&S66860[30], 3}, {NULL}};
const Segment SGI325021[] = {{&S66860[8], 3}, {NULL}};
const Segment SGI325022[] = {{&S66860[13], 10}, {NULL}};
const Segment SGI325023[] = {{&S66860[42], 7}, {NULL}};
const Segment SGI325024[] = {{&S66860[35], 14}, {NULL}};
const Segment SGI325025[] = {{&S66860[71], 11}, {NULL}};
const Segment SGI325026[] = {{&S66860[51], 6}, {NULL}};
const Segment SGI325027[] = {{&S66860[71], 16}, {NULL}};
const Segment SGI325028[] = {{&S66861[30], 3}, {NULL}};
const Segment SGI325029[] = {{&S66861[8], 3}, {NULL}};
const Segment SGI325030[] = {{&S66861[13], 10}, {NULL}};
const Segment SGI325031[] = {{&S66861[42], 7}, {NULL}};
const Segment SGI325032[] = {{&S66861[35], 14}, {NULL}};
const Segment SGI325033[] = {{&S66861[64], 11}, {NULL}};
const Segment SGI325034[] = {{&S66861[64], 16}, {NULL}};
const Segment SGI325035[] = {{&S66864[30], 3}, {NULL}};
const Segment SGI325036[] = {{&S66864[8], 3}, {NULL}};
const Segment SGI325037[] = {{&S66864[13], 10}, {NULL}};
const Segment SGI325038[] = {{&S66864[42], 7}, {NULL}};
const Segment SGI325039[] = {{&S66864[35], 14}, {NULL}};
const Segment SGI325040[] = {{&S66864[71], 11}, {NULL}};
const Segment SGI325041[] = {{&S66864[164], 15}, {NULL}};
const Segment SGI325042[] = {{&S66864[51], 6}, {NULL}};
const Segment SGI325043[] = {{&S66864[71], 16}, {NULL}};
const Segment SGI325044[] = {{&S66866[30], 3}, {NULL}};
const Segment SGI325045[] = {{&S66866[8], 3}, {NULL}};
const Segment SGI325046[] = {{&S66866[13], 10}, {NULL}};
const Segment SGI325047[] = {{&S66866[42], 7}, {NULL}};
const Segment SGI325048[] = {{&S66866[35], 14}, {NULL}};
const Segment SGI325049[] = {{&S66866[64], 11}, {NULL}};
const Segment SGI325050[] = {{&S66866[157], 15}, {NULL}};
const Segment SGI325051[] = {{&S66866[64], 16}, {NULL}};
const Segment SGI325052[] = {{&S66868[30], 3}, {NULL}};
const Segment SGI325053[] = {{&S66868[8], 3}, {NULL}};
const Segment SGI325054[] = {{&S66868[13], 10}, {NULL}};
const Segment SGI325055[] = {{&S66868[42], 7}, {NULL}};
const Segment SGI325056[] = {{&S66868[35], 14}, {NULL}};
const Segment SGI325057[] = {{&S66868[64], 11}, {NULL}};
const Segment SGI325058[] = {{&S66868[157], 15}, {NULL}};
const Segment SGI325059[] = {{&S66868[64], 16}, {NULL}};
const Segment SGI325060[] = {{&S66868[178], 9}, {NULL}};
const Segment SGI325061[] = {{&S84709[3], 5}, {NULL}};
const Segment SGI325062[] = {{&S84709[147], 13}, {NULL}};
const Segment SGI325063[] = {{&S74101[4], 6}, {NULL}};
const Segment SGI325064[] = {{&S74101[26], 3}, {NULL}};
const Segment SGI325065[] = {{&S102155[7], 4}, {NULL}};
const Segment SGI325066[] = {{&S102155[0], 86}, {NULL}};
const Segment SGI325067[] = {{&S102156[7], 4}, {NULL}};
const Segment SGI325068[] = {{&S102156[0], 81}, {NULL}};
const Segment SGI325069[] = {{&S70640[4], 16}, {NULL}};
const Segment SGI325070[] = {{&S70641[4], 11}, {NULL}};
const Segment SGI325071[] = {{&S66898[30], 3}, {NULL}};
const Segment SGI325072[] = {{&S66898[8], 3}, {NULL}};
const Segment SGI325073[] = {{&S66898[13], 10}, {NULL}};
const Segment SGI325074[] = {{&S66898[42], 2}, {NULL}};
const Segment SGI325075[] = {{&S66899[30], 3}, {NULL}};
const Segment SGI325076[] = {{&S66899[8], 3}, {NULL}};
const Segment SGI325077[] = {{&S66899[13], 10}, {NULL}};
const Segment SGI325078[] = {{&S66899[35], 3}, {NULL}};
const Segment SGI325079[] = {{&S66899[40], 9}, {NULL}};
const Segment SGI325080[] = {{&S66900[30], 3}, {NULL}};
const Segment SGI325081[] = {{&S66900[8], 3}, {NULL}};
const Segment SGI325082[] = {{&S66900[13], 10}, {NULL}};
const Segment SGI325083[] = {{&S66900[40], 9}, {NULL}};
const Segment SGI325084[] = {{&S79749[0], 75}, {NULL}};
const Segment SGI325085[] = {{&S84842[3], 5}, {NULL}};
const Segment SGI325086[] = {{&S84843[3], 10}, {NULL}};
const Segment SGI325087[] = {{&S74249[4], 5}, {NULL}};
const Segment SGI325088[] = {{&S74249[4], 11}, {NULL}};
const Segment SGI325089[] = {{&S66977[30], 3}, {NULL}};
const Segment SGI325090[] = {{&S66977[8], 3}, {NULL}};
const Segment SGI325091[] = {{&S66977[13], 10}, {NULL}};
const Segment SGI325092[] = {{&S66977[55], 7}, {NULL}};
const Segment SGI325093[] = {{&S66977[48], 14}, {NULL}};
const Segment SGI325094[] = {{&S66978[30], 3}, {NULL}};
const Segment SGI325095[] = {{&S66978[8], 3}, {NULL}};
const Segment SGI325096[] = {{&S66978[13], 10}, {NULL}};
const Segment SGI325097[] = {{&S66978[55], 7}, {NULL}};
const Segment SGI325098[] = {{&S66978[48], 14}, {NULL}};
const Segment SGI325099[] = {{&S66979[30], 3}, {NULL}};
const Segment SGI325100[] = {{&S66979[8], 3}, {NULL}};
const Segment SGI325101[] = {{&S66979[13], 10}, {NULL}};
const Segment SGI325102[] = {{&S66979[55], 7}, {NULL}};
const Segment SGI325103[] = {{&S66979[48], 14}, {NULL}};
const Segment SGI325104[] = {{&S66979[64], 3}, {NULL}};
const Segment SGI325105[] = {{&S66980[30], 3}, {NULL}};
const Segment SGI325106[] = {{&S66980[8], 3}, {NULL}};
const Segment SGI325107[] = {{&S66980[13], 10}, {NULL}};
const Segment SGI325108[] = {{&S66980[55], 7}, {NULL}};
const Segment SGI325109[] = {{&S66980[48], 14}, {NULL}};
const Segment SGI325110[] = {{&S66980[64], 3}, {NULL}};
const Segment SGI325111[] = {{&S66981[30], 3}, {NULL}};
const Segment SGI325112[] = {{&S66981[8], 3}, {NULL}};
const Segment SGI325113[] = {{&S66981[13], 10}, {NULL}};
const Segment SGI325114[] = {{&S66981[42], 8}, {NULL}};
const Segment SGI325115[] = {{&S66982[30], 3}, {NULL}};
const Segment SGI325116[] = {{&S66982[8], 3}, {NULL}};
const Segment SGI325117[] = {{&S66982[13], 10}, {NULL}};
const Segment SGI325118[] = {{&S66982[42], 8}, {NULL}};
const Segment SGI325119[] = {{&S66982[52], 6}, {NULL}};
const Segment SGI325120[] = {{&S66983[30], 3}, {NULL}};
const Segment SGI325121[] = {{&S66983[8], 3}, {NULL}};
const Segment SGI325122[] = {{&S66983[13], 10}, {NULL}};
const Segment SGI325123[] = {{&S66983[42], 8}, {NULL}};
const Segment SGI325124[] = {{&S66983[52], 3}, {NULL}};
const Segment SGI325125[] = {{&S70839[4], 4}, {NULL}};
const Segment SGI325126[] = {{&S70839[26], 11}, {NULL}};
const Segment SGI325127[] = {{&S70839[38], 5}, {NULL}};
const Segment SGI325128[] = {{&S70840[4], 4}, {NULL}};
const Segment SGI325129[] = {{&S70840[26], 11}, {NULL}};
const Segment SGI325130[] = {{&S70840[38], 5}, {NULL}};
const Segment SGI325131[] = {{&S70841[4], 4}, {NULL}};
const Segment SGI325132[] = {{&S70841[26], 11}, {NULL}};
const Segment SGI325133[] = {{&S70841[38], 5}, {NULL}};
const Segment SGI325134[] = {{&S70842[4], 4}, {NULL}};
const Segment SGI325135[] = {{&S70842[26], 11}, {NULL}};
const Segment SGI325136[] = {{&S70842[38], 5}, {NULL}};
const Segment SGI325137[] = {{&S70843[4], 4}, {NULL}};
const Segment SGI325138[] = {{&S70843[26], 11}, {NULL}};
const Segment SGI325139[] = {{&S70843[38], 5}, {NULL}};
const Segment SGI325140[] = {{&S70844[4], 4}, {NULL}};
const Segment SGI325141[] = {{&S70844[26], 11}, {NULL}};
const Segment SGI325142[] = {{&S70844[38], 5}, {NULL}};
const Segment SGI325143[] = {{&S95131[3], 5}, {NULL}};
const Segment SGI325144[] = {{&S85141[3], 5}, {NULL}};
const Segment SGI325145[] = {{&S67127[30], 3}, {NULL}};
const Segment SGI325146[] = {{&S67127[8], 3}, {NULL}};
const Segment SGI325147[] = {{&S67127[13], 10}, {NULL}};
const Segment SGI325148[] = {{&S67127[42], 3}, {NULL}};
const Segment SGI325149[] = {{&S67128[30], 4}, {NULL}};
const Segment SGI325150[] = {{&S67128[8], 3}, {NULL}};
const Segment SGI325151[] = {{&S67128[13], 10}, {NULL}};
const Segment SGI325152[] = {{&S67128[43], 3}, {NULL}};
const Segment SGI325153[] = {{&S67129[30], 3}, {NULL}};
const Segment SGI325154[] = {{&S67129[8], 3}, {NULL}};
const Segment SGI325155[] = {{&S67129[13], 10}, {NULL}};
const Segment SGI325156[] = {{&S67129[35], 5}, {NULL}};
const Segment SGI325157[] = {{&S85239[3], 5}, {NULL}};
const Segment SGI325158[] = {{&S85239[3], 12}, {NULL}};
const Segment SGI325159[] = {{&S85240[3], 5}, {NULL}};
const Segment SGI325160[] = {{&S85242[3], 5}, {NULL}};
const Segment SGI325161[] = {{&S95376[3], 4}, {NULL}};
const Segment SGI325162[] = {{&S67207[30], 3}, {NULL}};
const Segment SGI325163[] = {{&S67207[8], 3}, {NULL}};
const Segment SGI325164[] = {{&S67207[13], 10}, {NULL}};
const Segment SGI325165[] = {{&S67207[42], 7}, {NULL}};
const Segment SGI325166[] = {{&S67207[35], 14}, {NULL}};
const Segment SGI325167[] = {{&S67207[71], 11}, {NULL}};
const Segment SGI325168[] = {{&S67207[51], 6}, {NULL}};
const Segment SGI325169[] = {{&S67207[71], 16}, {NULL}};
const Segment SGI325170[] = {{&S67208[30], 3}, {NULL}};
const Segment SGI325171[] = {{&S67208[8], 3}, {NULL}};
const Segment SGI325172[] = {{&S67208[13], 10}, {NULL}};
const Segment SGI325173[] = {{&S67208[42], 7}, {NULL}};
const Segment SGI325174[] = {{&S67208[35], 14}, {NULL}};
const Segment SGI325175[] = {{&S67208[64], 11}, {NULL}};
const Segment SGI325176[] = {{&S67208[64], 16}, {NULL}};
const Segment SGI325177[] = {{&S67209[30], 3}, {NULL}};
const Segment SGI325178[] = {{&S67209[8], 3}, {NULL}};
const Segment SGI325179[] = {{&S67209[13], 10}, {NULL}};
const Segment SGI325180[] = {{&S67209[42], 7}, {NULL}};
const Segment SGI325181[] = {{&S67209[35], 14}, {NULL}};
const Segment SGI325182[] = {{&S67209[51], 11}, {NULL}};
const Segment SGI325183[] = {{&S67209[51], 16}, {NULL}};
const Segment SGI325184[] = {{&S102603[7], 3}, {NULL}};
const Segment SGI325185[] = {{&S102603[0], 84}, {NULL}};
const Segment SGI325186[] = {{&S80264[0], 120}, {NULL}};
const Segment SGI325187[] = {{&S80267[0], 120}, {NULL}};
const Segment SGI325188[] = {{&S85334[3], 8}, {NULL}};
const Segment SGI325189[] = {{&S85335[3], 5}, {NULL}};
const Segment SGI325190[] = {{&S67260[30], 3}, {NULL}};
const Segment SGI325191[] = {{&S67260[8], 3}, {NULL}};
const Segment SGI325192[] = {{&S67260[13], 10}, {NULL}};
const Segment SGI325193[] = {{&S67260[42], 7}, {NULL}};
const Segment SGI325194[] = {{&S67260[35], 14}, {NULL}};
const Segment SGI325195[] = {{&S67260[74], 3}, {NULL}};
const Segment SGI325196[] = {{&S67261[30], 3}, {NULL}};
const Segment SGI325197[] = {{&S67261[8], 3}, {NULL}};
const Segment SGI325198[] = {{&S67261[13], 10}, {NULL}};
const Segment SGI325199[] = {{&S67261[42], 7}, {NULL}};
const Segment SGI325200[] = {{&S67261[35], 14}, {NULL}};
const Segment SGI325201[] = {{&S67262[30], 3}, {NULL}};
const Segment SGI325202[] = {{&S67262[8], 3}, {NULL}};
const Segment SGI325203[] = {{&S67262[13], 10}, {NULL}};
const Segment SGI325204[] = {{&S67262[42], 7}, {NULL}};
const Segment SGI325205[] = {{&S67262[35], 14}, {NULL}};
const Segment SGI325206[] = {{&S67262[64], 3}, {NULL}};
const Segment SGI325207[] = {{&S67262[79], 3}, {NULL}};
const Segment SGI325208[] = {{&S67263[30], 3}, {NULL}};
const Segment SGI325209[] = {{&S67263[8], 3}, {NULL}};
const Segment SGI325210[] = {{&S67263[13], 10}, {NULL}};
const Segment SGI325211[] = {{&S67263[42], 7}, {NULL}};
const Segment SGI325212[] = {{&S67263[35], 14}, {NULL}};
const Segment SGI325213[] = {{&S67263[51], 3}, {NULL}};
const Segment SGI325214[] = {{&S67263[66], 3}, {NULL}};
const Segment SGI325215[] = {{&S80392[0], 103}, {NULL}};
const Segment SGI325216[] = {{&S80394[0], 11}, {NULL}};
const Segment SGI325217[] = {{&S102748[7], 5}, {NULL}};
const Segment SGI325218[] = {{&S102748[0], 82}, {NULL}};
const Segment SGI325219[] = {{&S85430[3], 5}, {NULL}};
const Segment SGI325220[] = {{&S102786[7], 6}, {NULL}};
const Segment SGI325221[] = {{&S102786[0], 83}, {NULL}};
const Segment SGI325222[] = {{&S102787[7], 6}, {NULL}};
const Segment SGI325223[] = {{&S102787[0], 83}, {NULL}};
const Segment SGI325224[] = {{&S80432[0], 32}, {NULL}};
const Segment SGI325225[] = {{&S102805[7], 6}, {NULL}};
const Segment SGI325226[] = {{&S102805[0], 83}, {NULL}};
const Segment SGI325227[] = {{&S80479[0], 32}, {NULL}};
const Segment SGI325228[] = {{&S85555[3], 7}, {NULL}};
const Segment SGI325229[] = {{&S85644[3], 5}, {NULL}};
const Segment SGI325230[] = {{&S75129[4], 5}, {NULL}};
const Segment SGI325231[] = {{&S75129[21], 3}, {NULL}};
const Segment SGI325232[] = {{&S71504[4], 3}, {NULL}};
const Segment SGI325233[] = {{&S71505[4], 11}, {NULL}};
const Segment SGI325234[] = {{&S71506[4], 2}, {NULL}};
const Segment SGI325235[] = {{&S71507[4], 7}, {NULL}};
const Segment SGI325236[] = {{&S71508[4], 3}, {NULL}};
const Segment SGI325237[] = {{&S71509[4], 7}, {NULL}};
const Segment SGI325238[] = {{&S71510[4], 9}, {NULL}};
const Segment SGI325239[] = {{&S71511[4], 3}, {NULL}};
const Segment SGI325240[] = {{&S67557[30], 3}, {NULL}};
const Segment SGI325241[] = {{&S67557[8], 3}, {NULL}};
const Segment SGI325242[] = {{&S67557[13], 10}, {NULL}};
const Segment SGI325243[] = {{&S67557[42], 7}, {NULL}};
const Segment SGI325244[] = {{&S67557[35], 14}, {NULL}};
const Segment SGI325245[] = {{&S67557[71], 5}, {NULL}};
const Segment SGI325246[] = {{&S67557[51], 6}, {NULL}};
const Segment SGI325247[] = {{&S67557[158], 9}, {NULL}};
const Segment SGI325248[] = {{&S67558[30], 3}, {NULL}};
const Segment SGI325249[] = {{&S67558[8], 3}, {NULL}};
const Segment SGI325250[] = {{&S67558[13], 10}, {NULL}};
const Segment SGI325251[] = {{&S67558[42], 7}, {NULL}};
const Segment SGI325252[] = {{&S67558[35], 14}, {NULL}};
const Segment SGI325253[] = {{&S67558[76], 5}, {NULL}};
const Segment SGI325254[] = {{&S67558[51], 5}, {NULL}};
const Segment SGI325255[] = {{&S67558[163], 9}, {NULL}};
const Segment SGI325256[] = {{&S103288[7], 4}, {NULL}};
const Segment SGI325257[] = {{&S103288[0], 73}, {NULL}};
const Segment SGI325258[] = {{&S103289[7], 4}, {NULL}};
const Segment SGI325259[] = {{&S103289[0], 73}, {NULL}};
const Segment SGI325260[] = {{&S103297[7], 3}, {NULL}};
const Segment SGI325261[] = {{&S103297[0], 80}, {NULL}};
const Segment SGI325262[] = {{&S85828[3], 5}, {NULL}};
const Segment SGI325263[] = {{&S85828[123], 13}, {NULL}};
const Segment SGI325264[] = {{&S96300[3], 6}, {NULL}};
const Segment SGI325265[] = {{&S96300[29], 8}, {NULL}};
const Segment SGI325266[] = {{&S81184[0], 32}, {NULL}};
const Segment SGI325267[] = {{&S86020[3], 6}, {NULL}};
const Segment SGI325268[] = {{&S81245[0], 77}, {NULL}};
const Segment SGI325269[] = {{&S68009[30], 4}, {NULL}};
const Segment SGI325270[] = {{&S68009[8], 3}, {NULL}};
const Segment SGI325271[] = {{&S68009[13], 10}, {NULL}};
const Segment SGI325272[] = {{&S68009[36], 3}, {NULL}};
const Segment SGI325273[] = {{&S68009[41], 9}, {NULL}};
const Segment SGI325274[] = {{&S68010[30], 4}, {NULL}};
const Segment SGI325275[] = {{&S68010[8], 3}, {NULL}};
const Segment SGI325276[] = {{&S68010[13], 10}, {NULL}};
const Segment SGI325277[] = {{&S68010[43], 2}, {NULL}};
const Segment SGI325278[] = {{&S68011[30], 4}, {NULL}};
const Segment SGI325279[] = {{&S68011[8], 3}, {NULL}};
const Segment SGI325280[] = {{&S68011[13], 10}, {NULL}};
const Segment SGI325281[] = {{&S68011[41], 9}, {NULL}};
const Segment SGI325282[] = {{&S86108[3], 5}, {NULL}};
const Segment SGI325283[] = {{&S81295[0], 32}, {NULL}};
const Segment SGI325284[] = {{&S86211[3], 5}, {NULL}};
const Segment SGI325285[] = {{&S72047[4], 10}, {NULL}};
const Segment SGI325286[] = {{&S72047[140], 15}, {NULL}};
const Segment SGI325287[] = {{&S86213[3], 6}, {NULL}};
const Segment SGI325288[] = {{&S72048[4], 14}, {NULL}};
const Segment SGI325289[] = {{&S72048[131], 8}, {NULL}};
const Segment SGI325290[] = {{&S86214[3], 5}, {NULL}};
const Segment SGI325291[] = {{&S96610[3], 5}, {NULL}};
const Segment SGI325292[] = {{&S72049[4], 10}, {NULL}};
const Segment SGI325293[] = {{&S72049[140], 15}, {NULL}};
const Segment SGI325294[] = {{&S72050[4], 10}, {NULL}};
const Segment SGI325295[] = {{&S72050[140], 15}, {NULL}};
const Segment SGI325296[] = {{&S86216[3], 5}, {NULL}};
const Segment SGI325297[] = {{&S86216[3], 12}, {NULL}};
const Segment SGI325298[] = {{&S96613[3], 4}, {NULL}};
const Segment SGI325299[] = {{&S86225[3], 5}, {NULL}};
const Segment SGI325300[] = {{&S86243[3], 5}, {NULL}};
const Segment SGI325301[] = {{&S86243[108], 12}, {NULL}};
const Segment SGI325302[] = {{&S81381[0], 123}, {NULL}};
const Segment SGI325303[] = {{&S86255[3], 5}, {NULL}};
const Segment SGI325304[] = {{&S86255[123], 13}, {NULL}};
const Segment SGI325305[] = {{&S81382[0], 123}, {NULL}};
const Segment SGI325306[] = {{&S103897[7], 4}, {NULL}};
const Segment SGI325307[] = {{&S103897[0], 81}, {NULL}};
const Segment SGI325308[] = {{&S86311[3], 5}, {NULL}};
const Segment SGI325309[] = {{&S86312[3], 6}, {NULL}};
const Segment SGI325310[] = {{&S86342[3], 5}, {NULL}};
const Segment SGI325311[] = {{&S86342[107], 12}, {NULL}};
const Segment SGI325312[] = {{&S68274[30], 3}, {NULL}};
const Segment SGI325313[] = {{&S68274[8], 3}, {NULL}};
const Segment SGI325314[] = {{&S68274[13], 10}, {NULL}};
const Segment SGI325315[] = {{&S68274[42], 2}, {NULL}};
const Segment SGI325316[] = {{&S68275[30], 3}, {NULL}};
const Segment SGI325317[] = {{&S68275[8], 3}, {NULL}};
const Segment SGI325318[] = {{&S68275[13], 10}, {NULL}};
const Segment SGI325319[] = {{&S68275[35], 3}, {NULL}};
const Segment SGI325320[] = {{&S68275[40], 9}, {NULL}};
const Segment SGI325321[] = {{&S68276[30], 3}, {NULL}};
const Segment SGI325322[] = {{&S68276[8], 3}, {NULL}};
const Segment SGI325323[] = {{&S68276[13], 10}, {NULL}};
const Segment SGI325324[] = {{&S68276[40], 9}, {NULL}};
const Segment SGI325325[] = {{&S72245[4], 5}, {NULL}};
const Segment SGI325326[] = {{&S72245[27], 11}, {NULL}};
const Segment SGI325327[] = {{&S72245[39], 5}, {NULL}};
const Segment SGI325328[] = {{&S81643[0], 213}, {NULL}};
const Segment SGI325329[] = {{&S81644[0], 214}, {NULL}};
const Segment SGI325330[] = {{&S81645[0], 356}, {NULL}};
const Segment SGI325331[] = {{&S86532[3], 5}, {NULL}};
const Segment SGI325332[] = {{&S86533[3], 5}, {NULL}};
const Segment SGI325333[] = {{&S86533[3], 12}, {NULL}};
const Segment SGI325334[] = {{&S86534[3], 6}, {NULL}};
const Segment SGI325335[] = {{&S81672[0], 20}, {NULL}};
const Segment SGI325336[] = {{&S104180[7], 4}, {NULL}};
const Segment SGI325337[] = {{&S104180[0], 48}, {NULL}};
const Segment SGI325338[] = {{&S86590[3], 5}, {NULL}};
const Segment SGI325339[] = {{&S86590[123], 13}, {NULL}};
const Segment SGI325340[] = {{&S104254[7], 5}, {NULL}};
const Segment SGI325341[] = {{&S104254[0], 82}, {NULL}};
const Segment SGI325342[] = {{&S86716[3], 5}, {NULL}};
const Segment SGI325343[] = {{&S86716[124], 13}, {NULL}};
const Segment SGI325344[] = {{&S86769[3], 4}, {NULL}};
const Segment SGI325345[] = {{&S81931[0], 89}, {NULL}};
const Segment SGI325346[] = {{&S81932[0], 87}, {NULL}};
const Segment SGI325347[] = {{&S81933[0], 89}, {NULL}};
const Segment SGI325348[] = {{&S81934[0], 87}, {NULL}};
const Segment SGI325349[] = {{&S81935[0], 91}, {NULL}};
const Segment SGI325350[] = {{&S81936[0], 87}, {NULL}};
const Segment SGI325351[] = {{&S81937[0], 88}, {NULL}};
const Segment SGI325352[] = {{&S81938[0], 87}, {NULL}};
const Segment SGI325353[] = {{&S81939[0], 89}, {NULL}};
const Segment SGI325354[] = {{&S81940[0], 87}, {NULL}};
const Segment SGI325355[] = {{&S81941[0], 89}, {NULL}};
const Segment SGI325356[] = {{&S68819[30], 3}, {NULL}};
const Segment SGI325357[] = {{&S68819[8], 3}, {NULL}};
const Segment SGI325358[] = {{&S68819[13], 10}, {NULL}};
const Segment SGI325359[] = {{&S68819[42], 7}, {NULL}};
const Segment SGI325360[] = {{&S68819[35], 14}, {NULL}};
const Segment SGI325361[] = {{&S68819[80], 5}, {NULL}};
const Segment SGI325362[] = {{&S68819[51], 6}, {NULL}};
const Segment SGI325363[] = {{&S68822[30], 3}, {NULL}};
const Segment SGI325364[] = {{&S68822[8], 3}, {NULL}};
const Segment SGI325365[] = {{&S68822[13], 10}, {NULL}};
const Segment SGI325366[] = {{&S68822[42], 7}, {NULL}};
const Segment SGI325367[] = {{&S68822[35], 14}, {NULL}};
const Segment SGI325368[] = {{&S68822[80], 5}, {NULL}};
const Segment SGI325369[] = {{&S68822[51], 6}, {NULL}};
const Segment SGI325370[] = {{&S68823[30], 3}, {NULL}};
const Segment SGI325371[] = {{&S68823[8], 3}, {NULL}};
const Segment SGI325372[] = {{&S68823[13], 10}, {NULL}};
const Segment SGI325373[] = {{&S68823[42], 7}, {NULL}};
const Segment SGI325374[] = {{&S68823[35], 14}, {NULL}};
const Segment SGI325375[] = {{&S68823[80], 5}, {NULL}};
const Segment SGI325376[] = {{&S68823[51], 6}, {NULL}};
const Segment SGI325377[] = {{&S68826[30], 3}, {NULL}};
const Segment SGI325378[] = {{&S68826[8], 3}, {NULL}};
const Segment SGI325379[] = {{&S68826[13], 10}, {NULL}};
const Segment SGI325380[] = {{&S68826[42], 7}, {NULL}};
const Segment SGI325381[] = {{&S68826[35], 14}, {NULL}};
const Segment SGI325382[] = {{&S68826[80], 5}, {NULL}};
const Segment SGI325383[] = {{&S68826[51], 6}, {NULL}};
const Segment SGI325384[] = {{&S82067[0], 32}, {NULL}};
const Segment SGI325385[] = {{&S87032[3], 5}, {NULL}};
const Segment SGI325386[] = {{&S104702[7], 4}, {NULL}};
const Segment SGI325387[] = {{&S104702[0], 86}, {NULL}};
const Segment SGI325388[] = {{&S87186[3], 5}, {NULL}};
const Segment SGI325389[] = {{&S69105[30], 3}, {NULL}};
const Segment SGI325390[] = {{&S69105[8], 3}, {NULL}};
const Segment SGI325391[] = {{&S69105[13], 10}, {NULL}};
const Segment SGI325392[] = {{&S69105[42], 7}, {NULL}};
const Segment SGI325393[] = {{&S69105[35], 14}, {NULL}};
const Segment SGI325394[] = {{&S69105[64], 7}, {NULL}};
const Segment SGI325395[] = {{&S69105[71], 4}, {NULL}};
const Segment SGI325396[] = {{&S69106[30], 3}, {NULL}};
const Segment SGI325397[] = {{&S69106[8], 3}, {NULL}};
const Segment SGI325398[] = {{&S69106[13], 10}, {NULL}};
const Segment SGI325399[] = {{&S69106[42], 7}, {NULL}};
const Segment SGI325400[] = {{&S69106[35], 14}, {NULL}};
const Segment SGI325401[] = {{&S69106[91], 15}, {NULL}};
const Segment SGI325402[] = {{&S69106[162], 7}, {NULL}};
const Segment SGI325403[] = {{&S69106[169], 4}, {NULL}};
const Segment SGI325404[] = {{&S69107[30], 3}, {NULL}};
const Segment SGI325405[] = {{&S69107[8], 3}, {NULL}};
const Segment SGI325406[] = {{&S69107[13], 10}, {NULL}};
const Segment SGI325407[] = {{&S69107[42], 7}, {NULL}};
const Segment SGI325408[] = {{&S69107[35], 14}, {NULL}};
const Segment SGI325409[] = {{&S69107[51], 6}, {NULL}};
const Segment SGI325410[] = {{&S69107[71], 7}, {NULL}};
const Segment SGI325411[] = {{&S69107[78], 4}, {NULL}};
const Segment SGI325412[] = {{&S69108[30], 3}, {NULL}};
const Segment SGI325413[] = {{&S69108[8], 3}, {NULL}};
const Segment SGI325414[] = {{&S69108[13], 10}, {NULL}};
const Segment SGI325415[] = {{&S69108[42], 7}, {NULL}};
const Segment SGI325416[] = {{&S69108[35], 14}, {NULL}};
const Segment SGI325417[] = {{&S69108[141], 7}, {NULL}};
const Segment SGI325418[] = {{&S69108[148], 4}, {NULL}};
const Segment SGI325419[] = {{&S69139[30], 3}, {NULL}};
const Segment SGI325420[] = {{&S69139[8], 3}, {NULL}};
const Segment SGI325421[] = {{&S69139[13], 10}, {NULL}};
const Segment SGI325422[] = {{&S69139[42], 7}, {NULL}};
const Segment SGI325423[] = {{&S69139[35], 14}, {NULL}};
const Segment SGI325424[] = {{&S69139[91], 15}, {NULL}};
const Segment SGI325425[] = {{&S69139[172], 4}, {NULL}};
const Segment SGI325426[] = {{&S87237[3], 5}, {NULL}};
const Segment SGI325427[] = {{&S87237[124], 13}, {NULL}};
const Segment SGI325428[] = {{&S87407[3], 5}, {NULL}};
const Segment SGI325429[] = {{&S87408[3], 5}, {NULL}};
const Segment SGI325430[] = {{&S82520[0], 34}, {NULL}};
const Segment SGI325431[] = {{&S87543[3], 5}, {NULL}};
const Segment SGI325432[] = {{&S87544[3], 5}, {NULL}};
const Segment SGI325433[] = {{&S87544[3], 12}, {NULL}};
const Segment SGI325434[] = {{&S87587[3], 5}, {NULL}};
const Segment SGI325435[] = {{&S87587[124], 13}, {NULL}};
const Segment SGI325436[] = {{&S105176[7], 4}, {NULL}};
const Segment SGI325437[] = {{&S105176[0], 61}, {NULL}};
const Segment SGI325438[] = {{&S105188[7], 3}, {NULL}};
const Segment SGI325439[] = {{&S105188[0], 80}, {NULL}};
const Segment SGI325440[] = {{&S69424[30], 3}, {NULL}};
const Segment SGI325441[] = {{&S69424[8], 3}, {NULL}};
const Segment SGI325442[] = {{&S69424[13], 10}, {NULL}};
const Segment SGI325443[] = {{&S69424[40], 9}, {NULL}};
const Segment SGI325444[] = {{&S69425[30], 3}, {NULL}};
const Segment SGI325445[] = {{&S69425[8], 3}, {NULL}};
const Segment SGI325446[] = {{&S69425[13], 10}, {NULL}};
const Segment SGI325447[] = {{&S69425[42], 2}, {NULL}};
const Segment SGI325448[] = {{&S69426[30], 3}, {NULL}};
const Segment SGI325449[] = {{&S69426[8], 3}, {NULL}};
const Segment SGI325450[] = {{&S69426[13], 10}, {NULL}};
const Segment SGI325451[] = {{&S69426[35], 3}, {NULL}};
const Segment SGI325452[] = {{&S69426[40], 9}, {NULL}};
const Segment SGI325453[] = {{&S76857[4], 5}, {NULL}};
const Segment SGI325454[] = {{&S76857[25], 3}, {NULL}};
const Segment SGI325455[] = {{&S76858[4], 5}, {NULL}};
const Segment SGI325456[] = {{&S76858[25], 3}, {NULL}};
const Segment SGI325457[] = {{&S98062[3], 5}, {NULL}};
const Segment SGI325458[] = {{&S98062[28], 10}, {NULL}};
const Segment SGI325459[] = {{&S105257[7], 4}, {NULL}};
const Segment SGI325460[] = {{&S105257[0], 92}, {NULL}};
const Segment SGI325461[] = {{&S76999[4], 5}, {NULL}};
const Segment SGI325462[] = {{&S76999[25], 3}, {NULL}};
const Segment SGI325463[] = {{&S82929[0], 32}, {NULL}};
const Segment SGI325464[] = {{&S82930[0], 32}, {NULL}};
const Segment SGI325465[] = {{&S77132[4], 5}, {NULL}};
const Segment SGI325466[] = {{&S77132[25], 3}, {NULL}};
const Segment SGI325467[] = {{&S69755[30], 3}, {NULL}};
const Segment SGI325468[] = {{&S69755[8], 3}, {NULL}};
const Segment SGI325469[] = {{&S69755[13], 10}, {NULL}};
const Segment SGI325470[] = {{&S69755[42], 7}, {NULL}};
const Segment SGI325471[] = {{&S69755[35], 14}, {NULL}};
const Segment SGI325472[] = {{&S69756[30], 3}, {NULL}};
const Segment SGI325473[] = {{&S69756[8], 3}, {NULL}};
const Segment SGI325474[] = {{&S69756[13], 10}, {NULL}};
const Segment SGI325475[] = {{&S69756[42], 7}, {NULL}};
const Segment SGI325476[] = {{&S69756[35], 14}, {NULL}};
const Segment SGI325477[] = {{&S69757[30], 3}, {NULL}};
const Segment SGI325478[] = {{&S69757[8], 3}, {NULL}};
const Segment SGI325479[] = {{&S69757[13], 10}, {NULL}};
const Segment SGI325480[] = {{&S69757[42], 7}, {NULL}};
const Segment SGI325481[] = {{&S69757[35], 14}, {NULL}};
const Segment SGI325482[] = {{&S69758[30], 3}, {NULL}};
const Segment SGI325483[] = {{&S69758[8], 3}, {NULL}};
const Segment SGI325484[] = {{&S69758[13], 10}, {NULL}};
const Segment SGI325485[] = {{&S69758[42], 7}, {NULL}};
const Segment SGI325486[] = {{&S69758[35], 14}, {NULL}};
const Segment SGI325487[] = {{&S87910[3], 5}, {NULL}};
const Segment SGI325488[] = {{&S87912[3], 5}, {NULL}};
const Segment SGI325489[] = {{&S73559[4], 5}, {NULL}};
const Segment SGI325490[] = {{&S73559[27], 11}, {NULL}};
const Segment SGI325491[] = {{&S73559[39], 5}, {NULL}};
const Segment SGI325492[] = {{&S73560[4], 12}, {NULL}};
const Segment SGI325493[] = {{&S73560[32], 11}, {NULL}};
const Segment SGI325494[] = {{&S73560[44], 5}, {NULL}};
const Segment SGI325495[] = {{&S88181[3], 5}, {NULL}};
const Segment SGI325496[] = {{&S88288[3], 5}, {NULL}};
const Segment SGI325497[] = {{&S83412[0], 31}, {NULL}};
const Segment SGI325498[] = {{&S73899[4], 4}, {NULL}};
const Segment SGI325499[] = {{&S73900[4], 3}, {NULL}};
const Segment SGI325500[] = {{&S73901[4], 5}, {NULL}};
const Segment SGI325501[] = {{&S73902[4], 4}, {NULL}};
const Segment SGI325502[] = {{&S70042[30], 3}, {NULL}};
const Segment SGI325503[] = {{&S70042[8], 3}, {NULL}};
const Segment SGI325504[] = {{&S70042[13], 10}, {NULL}};
const Segment SGI325505[] = {{&S70042[42], 7}, {NULL}};
const Segment SGI325506[] = {{&S70043[29], 3}, {NULL}};
const Segment SGI325507[] = {{&S70043[8], 3}, {NULL}};
const Segment SGI325508[] = {{&S70043[13], 10}, {NULL}};
const Segment SGI325509[] = {{&S70043[42], 7}, {NULL}};
const Segment SGI325510[] = {{&S70043[35], 14}, {NULL}};
const Segment SGI325511[] = {{&S70044[29], 3}, {NULL}};
const Segment SGI325512[] = {{&S70044[8], 3}, {NULL}};
const Segment SGI325513[] = {{&S70044[13], 10}, {NULL}};
const Segment SGI325514[] = {{&S70044[42], 7}, {NULL}};
const Segment SGI325515[] = {{&S70044[35], 14}, {NULL}};
const Segment SGI325516[] = {{&S70045[29], 3}, {NULL}};
const Segment SGI325517[] = {{&S70045[8], 3}, {NULL}};
const Segment SGI325518[] = {{&S70045[13], 10}, {NULL}};
const Segment SGI325519[] = {{&S70045[42], 7}, {NULL}};
const Segment SGI325520[] = {{&S70045[35], 14}, {NULL}};
const Segment SGI325521[] = {{&S70046[29], 3}, {NULL}};
const Segment SGI325522[] = {{&S70046[8], 3}, {NULL}};
const Segment SGI325523[] = {{&S70046[13], 10}, {NULL}};
const Segment SGI325524[] = {{&S70046[42], 7}, {NULL}};
const Segment SGI325525[] = {{&S70046[35], 14}, {NULL}};
const Segment SGI325526[] = {{&S70047[30], 3}, {NULL}};
const Segment SGI325527[] = {{&S70047[8], 3}, {NULL}};
const Segment SGI325528[] = {{&S70047[13], 10}, {NULL}};
const Segment SGI325529[] = {{&S70047[42], 7}, {NULL}};
const Segment SGI325530[] = {{&S70047[35], 14}, {NULL}};
const Segment SGI325531[] = {{&S70047[51], 3}, {NULL}};
const Segment SGI325532[] = {{&S70048[30], 3}, {NULL}};
const Segment SGI325533[] = {{&S70048[8], 3}, {NULL}};
const Segment SGI325534[] = {{&S70048[13], 10}, {NULL}};
const Segment SGI325535[] = {{&S70048[35], 8}, {NULL}};
const Segment SGI325536[] = {{&S70049[30], 3}, {NULL}};
const Segment SGI325537[] = {{&S70049[8], 3}, {NULL}};
const Segment SGI325538[] = {{&S70049[13], 10}, {NULL}};
const Segment SGI325539[] = {{&S70049[42], 7}, {NULL}};
const Segment SGI325540[] = {{&S70049[35], 14}, {NULL}};
const Segment SGI325541[] = {{&S70049[51], 6}, {NULL}};
const Segment SGI325542[] = {{&S88327[3], 5}, {NULL}};
const Segment SGI325543[] = {{&S88327[108], 12}, {NULL}};
const Segment SGI325544[] = {{&S88396[3], 5}, {NULL}};
const Segment SGI325545[] = {{&S88396[125], 13}, {NULL}};
const Segment SGI325546[] = {{&S70282[30], 3}, {NULL}};
const Segment SGI325547[] = {{&S70282[8], 3}, {NULL}};
const Segment SGI325548[] = {{&S70282[13], 10}, {NULL}};
const Segment SGI325549[] = {{&S70282[42], 7}, {NULL}};
const Segment SGI325550[] = {{&S70282[35], 14}, {NULL}};
const Segment SGI325551[] = {{&S70283[30], 3}, {NULL}};
const Segment SGI325552[] = {{&S70283[8], 3}, {NULL}};
const Segment SGI325553[] = {{&S70283[13], 10}, {NULL}};
const Segment SGI325554[] = {{&S70283[42], 7}, {NULL}};
const Segment SGI325555[] = {{&S70283[35], 14}, {NULL}};
const Segment SGI325556[] = {{&S77734[4], 6}, {NULL}};
const Segment SGI325557[] = {{&S77734[26], 3}, {NULL}};
const Segment SGI325558[] = {{&S99043[3], 5}, {NULL}};
const Segment SGI325559[] = {{&S88572[3], 6}, {NULL}};
const Segment SGI325560[] = {{&S88572[29], 9}, {NULL}};
const Segment SGI325561[] = {{&S88577[3], 4}, {NULL}};
const Segment SGI325562[] = {{&S83851[0], 12}, {NULL}};
const Segment SGI325563[] = {{&S74354[4], 14}, {NULL}};
const Segment SGI325564[] = {{&S74354[131], 41}, {NULL}};
const Segment SGI325565[] = {{&S88691[3], 5}, {NULL}};
const Segment SGI325566[] = {{&S88827[3], 5}, {NULL}};
const Segment SGI325567[] = {{&S88867[3], 4}, {NULL}};
const Segment SGI325568[] = {{&S88871[3], 5}, {NULL}};
const Segment SGI325569[] = {{&S88871[146], 13}, {NULL}};
const Segment SGI325570[] = {{&S70790[30], 4}, {NULL}};
const Segment SGI325571[] = {{&S70790[8], 3}, {NULL}};
const Segment SGI325572[] = {{&S70790[13], 10}, {NULL}};
const Segment SGI325573[] = {{&S70790[43], 7}, {NULL}};
const Segment SGI325574[] = {{&S84209[0], 71}, {NULL}};
const Segment SGI325575[] = {{&S70791[8], 3}, {NULL}};
const Segment SGI325576[] = {{&S70791[13], 10}, {NULL}};
const Segment SGI325577[] = {{&S70791[42], 7}, {NULL}};
const Segment SGI325578[] = {{&S70792[8], 3}, {NULL}};
const Segment SGI325579[] = {{&S70792[13], 10}, {NULL}};
const Segment SGI325580[] = {{&S70792[42], 6}, {NULL}};
const Segment SGI325581[] = {{&S70793[30], 3}, {NULL}};
const Segment SGI325582[] = {{&S70793[8], 3}, {NULL}};
const Segment SGI325583[] = {{&S70793[13], 10}, {NULL}};
const Segment SGI325584[] = {{&S70793[42], 7}, {NULL}};
const Segment SGI325585[] = {{&S70793[35], 14}, {NULL}};
const Segment SGI325586[] = {{&S74719[4], 11}, {NULL}};
const Segment SGI325587[] = {{&S70847[30], 4}, {NULL}};
const Segment SGI325588[] = {{&S70847[8], 3}, {NULL}};
const Segment SGI325589[] = {{&S70847[13], 10}, {NULL}};
const Segment SGI325590[] = {{&S70847[43], 3}, {NULL}};
const Segment SGI325591[] = {{&S70848[30], 3}, {NULL}};
const Segment SGI325592[] = {{&S70848[8], 3}, {NULL}};
const Segment SGI325593[] = {{&S70848[13], 10}, {NULL}};
const Segment SGI325594[] = {{&S70848[42], 3}, {NULL}};
const Segment SGI325595[] = {{&S70849[30], 3}, {NULL}};
const Segment SGI325596[] = {{&S70849[8], 3}, {NULL}};
const Segment SGI325597[] = {{&S70849[13], 10}, {NULL}};
const Segment SGI325598[] = {{&S70849[35], 5}, {NULL}};
const Segment SGI325599[] = {{&S74759[4], 8}, {NULL}};
const Segment SGI325600[] = {{&S74759[48], 44}, {NULL}};
const Segment SGI325601[] = {{&S74760[4], 6}, {NULL}};
const Segment SGI325602[] = {{&S74760[46], 42}, {NULL}};
const Segment SGI325603[] = {{&S74761[4], 6}, {NULL}};
const Segment SGI325604[] = {{&S74761[42], 50}, {NULL}};
const Segment SGI325605[] = {{&S74762[4], 6}, {NULL}};
const Segment SGI325606[] = {{&S74763[4], 8}, {NULL}};
const Segment SGI325607[] = {{&S74764[4], 6}, {NULL}};
const Segment SGI325608[] = {{&S74764[40], 40}, {NULL}};
const Segment SGI325609[] = {{&S74765[4], 5}, {NULL}};
const Segment SGI325610[] = {{&S74766[4], 6}, {NULL}};
const Segment SGI325611[] = {{&S74767[4], 5}, {NULL}};
const Segment SGI325612[] = {{&S74767[39], 51}, {NULL}};
const Segment SGI325613[] = {{&S74768[4], 5}, {NULL}};
const Segment SGI325614[] = {{&S74768[41], 50}, {NULL}};
const Segment SGI325615[] = {{&S74769[4], 5}, {NULL}};
const Segment SGI325616[] = {{&S74769[39], 51}, {NULL}};
const Segment SGI325617[] = {{&S89059[3], 6}, {NULL}};
const Segment SGI325618[] = {{&S71012[30], 3}, {NULL}};
const Segment SGI325619[] = {{&S71012[8], 3}, {NULL}};
const Segment SGI325620[] = {{&S71012[13], 10}, {NULL}};
const Segment SGI325621[] = {{&S71012[39], 15}, {NULL}};
const Segment SGI325622[] = {{&S71012[35], 3}, {NULL}};
const Segment SGI325623[] = {{&S89141[3], 5}, {NULL}};
const Segment SGI325624[] = {{&S89141[108], 12}, {NULL}};
const Segment SGI325625[] = {{&S71148[30], 3}, {NULL}};
const Segment SGI325626[] = {{&S71148[8], 3}, {NULL}};
const Segment SGI325627[] = {{&S71148[13], 10}, {NULL}};
const Segment SGI325628[] = {{&S71149[30], 3}, {NULL}};
const Segment SGI325629[] = {{&S71149[8], 3}, {NULL}};
const Segment SGI325630[] = {{&S71149[13], 10}, {NULL}};
const Segment SGI325631[] = {{&S71150[30], 4}, {NULL}};
const Segment SGI325632[] = {{&S71150[8], 3}, {NULL}};
const Segment SGI325633[] = {{&S71150[13], 10}, {NULL}};
const Segment SGI325634[] = {{&S89230[3], 5}, {NULL}};
const Segment SGI325635[] = {{&S89231[3], 5}, {NULL}};
const Segment SGI325636[] = {{&S71223[30], 3}, {NULL}};
const Segment SGI325637[] = {{&S71223[8], 3}, {NULL}};
const Segment SGI325638[] = {{&S71223[13], 10}, {NULL}};
const Segment SGI325639[] = {{&S71223[39], 15}, {NULL}};
const Segment SGI325640[] = {{&S71223[35], 3}, {NULL}};
const Segment SGI325641[] = {{&S71224[30], 3}, {NULL}};
const Segment SGI325642[] = {{&S71224[8], 3}, {NULL}};
const Segment SGI325643[] = {{&S71224[13], 10}, {NULL}};
const Segment SGI325644[] = {{&S71224[39], 15}, {NULL}};
const Segment SGI325645[] = {{&S71224[35], 3}, {NULL}};
const Segment SGI325646[] = {{&S75120[4], 10}, {NULL}};
const Segment SGI325647[] = {{&S75120[44], 46}, {NULL}};
const Segment SGI325648[] = {{&S75121[4], 10}, {NULL}};
const Segment SGI325649[] = {{&S84365[0], 11}, {NULL}};
const Segment SGI325650[] = {{&S75122[4], 11}, {NULL}};
const Segment SGI325651[] = {{&S75123[4], 11}, {NULL}};
const Segment SGI325652[] = {{&S75123[49], 47}, {NULL}};
const Segment SGI325653[] = {{&S75238[4], 2}, {NULL}};
const Segment SGI325654[] = {{&S75239[4], 3}, {NULL}};
const Segment SGI325655[] = {{&S71380[30], 3}, {NULL}};
const Segment SGI325656[] = {{&S71380[8], 3}, {NULL}};
const Segment SGI325657[] = {{&S71380[13], 10}, {NULL}};
const Segment SGI325658[] = {{&S71386[30], 3}, {NULL}};
const Segment SGI325659[] = {{&S71386[8], 3}, {NULL}};
const Segment SGI325660[] = {{&S71386[13], 10}, {NULL}};
const Segment SGI325661[] = {{&S71386[42], 7}, {NULL}};
const Segment SGI325662[] = {{&S71386[35], 14}, {NULL}};
const Segment SGI325663[] = {{&S71386[73], 5}, {NULL}};
const Segment SGI325664[] = {{&S84502[0], 70}, {NULL}};
const Segment SGI325665[] = {{&S71418[30], 3}, {NULL}};
const Segment SGI325666[] = {{&S71418[8], 3}, {NULL}};
const Segment SGI325667[] = {{&S71418[13], 10}, {NULL}};
const Segment SGI325668[] = {{&S71418[42], 7}, {NULL}};
const Segment SGI325669[] = {{&S71418[35], 14}, {NULL}};
const Segment SGI325670[] = {{&S71419[30], 3}, {NULL}};
const Segment SGI325671[] = {{&S75314[4], 11}, {NULL}};
const Segment SGI325672[] = {{&S75314[45], 32}, {NULL}};
const Segment SGI325673[] = {{&S71419[8], 3}, {NULL}};
const Segment SGI325674[] = {{&S71419[13], 10}, {NULL}};
const Segment SGI325675[] = {{&S71419[42], 7}, {NULL}};
const Segment SGI325676[] = {{&S71419[35], 14}, {NULL}};
const Segment SGI325677[] = {{&S71419[51], 5}, {NULL}};
const Segment SGI325678[] = {{&S75315[4], 11}, {NULL}};
const Segment SGI325679[] = {{&S75315[45], 25}, {NULL}};
const Segment SGI325680[] = {{&S71420[30], 3}, {NULL}};
const Segment SGI325681[] = {{&S71420[8], 3}, {NULL}};
const Segment SGI325682[] = {{&S71420[13], 10}, {NULL}};
const Segment SGI325683[] = {{&S71420[42], 7}, {NULL}};
const Segment SGI325684[] = {{&S71420[35], 14}, {NULL}};
const Segment SGI325685[] = {{&S71420[51], 6}, {NULL}};
const Segment SGI325686[] = {{&S71421[30], 3}, {NULL}};
const Segment SGI325687[] = {{&S71421[8], 3}, {NULL}};
const Segment SGI325688[] = {{&S71421[13], 10}, {NULL}};
const Segment SGI325689[] = {{&S71421[42], 7}, {NULL}};
const Segment SGI325690[] = {{&S71421[35], 14}, {NULL}};
const Segment SGI325691[] = {{&S89556[3], 5}, {NULL}};
const Segment SGI325692[] = {{&S89562[3], 5}, {NULL}};
const Segment SGI325693[] = {{&S84708[0], 9}, {NULL}};
const Segment SGI325694[] = {{&S71610[30], 3}, {NULL}};
const Segment SGI325695[] = {{&S71610[8], 3}, {NULL}};
const Segment SGI325696[] = {{&S71610[13], 10}, {NULL}};
const Segment SGI325697[] = {{&S78966[4], 6}, {NULL}};
const Segment SGI325698[] = {{&S78966[26], 3}, {NULL}};
const Segment SGI325699[] = {{&S78967[4], 5}, {NULL}};
const Segment SGI325700[] = {{&S78967[25], 3}, {NULL}};
const Segment SGI325701[] = {{&S78968[4], 5}, {NULL}};
const Segment SGI325702[] = {{&S78968[25], 3}, {&S78968[25], 3}, {NULL}};
const Segment SGI325703[] = {{&S78969[4], 5}, {NULL}};
const Segment SGI325704[] = {{&S78969[25], 3}, {NULL}};
const Segment SGI325705[] = {{&S78970[4], 5}, {NULL}};
const Segment SGI325706[] = {{&S78970[25], 3}, {NULL}};
const Segment SGI325707[] = {{&S78972[4], 6}, {NULL}};
const Segment SGI325708[] = {{&S78972[26], 3}, {NULL}};
const Segment SGI325709[] = {{&S78973[4], 5}, {NULL}};
const Segment SGI325710[] = {{&S78973[25], 3}, {NULL}};
const Segment SGI325711[] = {{&S89694[3], 5}, {NULL}};
const Segment SGI325712[] = {{&S89801[3], 5}, {NULL}};
const Segment SGI325713[] = {{&S89801[124], 13}, {NULL}};
const Segment SGI325714[] = {{&S84852[0], 20}, {NULL}};
const Segment SGI325715[] = {{&S89851[3], 5}, {NULL}};
const Segment SGI325716[] = {{&S89852[3], 5}, {NULL}};
const Segment SGI325717[] = {{&S71750[30], 3}, {NULL}};
const Segment SGI325718[] = {{&S71750[8], 3}, {NULL}};
const Segment SGI325719[] = {{&S71750[13], 10}, {NULL}};
const Segment SGI325720[] = {{&S71750[42], 2}, {NULL}};
const Segment SGI325721[] = {{&S71751[30], 3}, {NULL}};
const Segment SGI325722[] = {{&S71751[8], 3}, {NULL}};
const Segment SGI325723[] = {{&S71751[13], 10}, {NULL}};
const Segment SGI325724[] = {{&S71751[40], 9}, {NULL}};
const Segment SGI325725[] = {{&S84980[0], 31}, {NULL}};
const Segment SGI325726[] = {{&S90179[3], 5}, {NULL}};
const Segment SGI325727[] = {{&S90179[24], 3}, {NULL}};
const Segment SGI325728[] = {{&S85241[0], 90}, {NULL}};
const Segment SGI325729[] = {{&S85244[0], 24}, {NULL}};
const Segment SGI325730[] = {{&S71937[30], 3}, {NULL}};
const Segment SGI325731[] = {{&S71937[8], 3}, {NULL}};
const Segment SGI325732[] = {{&S71937[13], 10}, {NULL}};
const Segment SGI325733[] = {{&S71937[42], 7}, {NULL}};
const Segment SGI325734[] = {{&S71937[35], 14}, {NULL}};
const Segment SGI325735[] = {{&S71937[64], 11}, {NULL}};
const Segment SGI325736[] = {{&S71937[64], 16}, {NULL}};
const Segment SGI325737[] = {{&S71938[30], 3}, {NULL}};
const Segment SGI325738[] = {{&S71938[8], 3}, {NULL}};
const Segment SGI325739[] = {{&S71938[13], 10}, {NULL}};
const Segment SGI325740[] = {{&S71938[42], 7}, {NULL}};
const Segment SGI325741[] = {{&S71938[35], 14}, {NULL}};
const Segment SGI325742[] = {{&S71938[71], 11}, {NULL}};
const Segment SGI325743[] = {{&S71938[51], 6}, {NULL}};
const Segment SGI325744[] = {{&S71938[71], 16}, {NULL}};
const Segment SGI325745[] = {{&S71940[30], 3}, {NULL}};
const Segment SGI325746[] = {{&S71940[8], 3}, {NULL}};
const Segment SGI325747[] = {{&S71940[13], 10}, {NULL}};
const Segment SGI325748[] = {{&S71940[42], 7}, {NULL}};
const Segment SGI325749[] = {{&S71940[35], 14}, {NULL}};
const Segment SGI325750[] = {{&S71940[51], 11}, {NULL}};
const Segment SGI325751[] = {{&S71940[104], 15}, {NULL}};
const Segment SGI325752[] = {{&S71940[51], 16}, {NULL}};
const Segment SGI325753[] = {{&S71942[30], 3}, {NULL}};
const Segment SGI325754[] = {{&S71942[8], 3}, {NULL}};
const Segment SGI325755[] = {{&S71942[13], 10}, {NULL}};
const Segment SGI325756[] = {{&S71942[42], 7}, {NULL}};
const Segment SGI325757[] = {{&S71942[35], 14}, {NULL}};
const Segment SGI325758[] = {{&S71942[64], 11}, {NULL}};
const Segment SGI325759[] = {{&S71942[117], 15}, {NULL}};
const Segment SGI325760[] = {{&S71942[64], 16}, {NULL}};
const Segment SGI325761[] = {{&S71943[30], 3}, {NULL}};
const Segment SGI325762[] = {{&S71943[8], 3}, {NULL}};
const Segment SGI325763[] = {{&S71943[13], 10}, {NULL}};
const Segment SGI325764[] = {{&S71943[42], 7}, {NULL}};
const Segment SGI325765[] = {{&S71943[35], 14}, {NULL}};
const Segment SGI325766[] = {{&S71943[64], 11}, {NULL}};
const Segment SGI325767[] = {{&S71943[64], 16}, {NULL}};
const Segment SGI325768[] = {{&S71944[30], 3}, {NULL}};
const Segment SGI325769[] = {{&S71944[8], 3}, {NULL}};
const Segment SGI325770[] = {{&S71944[13], 10}, {NULL}};
const Segment SGI325771[] = {{&S71944[42], 7}, {NULL}};
const Segment SGI325772[] = {{&S71944[35], 14}, {NULL}};
const Segment SGI325773[] = {{&S71944[79], 11}, {NULL}};
const Segment SGI325774[] = {{&S71944[132], 15}, {NULL}};
const Segment SGI325775[] = {{&S71944[51], 6}, {NULL}};
const Segment SGI325776[] = {{&S71944[79], 16}, {NULL}};
const Segment SGI325777[] = {{&S71945[30], 3}, {NULL}};
const Segment SGI325778[] = {{&S71945[8], 3}, {NULL}};
const Segment SGI325779[] = {{&S71945[13], 10}, {NULL}};
const Segment SGI325780[] = {{&S71945[42], 7}, {NULL}};
const Segment SGI325781[] = {{&S71945[35], 14}, {NULL}};
const Segment SGI325782[] = {{&S71945[51], 11}, {NULL}};
const Segment SGI325783[] = {{&S71945[51], 16}, {NULL}};
const Segment SGI325784[] = {{&S71946[30], 3}, {NULL}};
const Segment SGI325785[] = {{&S71946[8], 3}, {NULL}};
const Segment SGI325786[] = {{&S71946[13], 10}, {NULL}};
const Segment SGI325787[] = {{&S71946[42], 7}, {NULL}};
const Segment SGI325788[] = {{&S71946[35], 14}, {NULL}};
const Segment SGI325789[] = {{&S71946[71], 11}, {NULL}};
const Segment SGI325790[] = {{&S71946[51], 6}, {NULL}};
const Segment SGI325791[] = {{&S71946[71], 16}, {NULL}};
const Segment SGI325792[] = {{&S75961[4], 5}, {NULL}};
const Segment SGI325793[] = {{&S75962[4], 4}, {NULL}};
const Segment SGI325794[] = {{&S100730[3], 5}, {NULL}};
const Segment SGI325795[] = {{&S72088[30], 4}, {NULL}};
const Segment SGI325796[] = {{&S72088[8], 3}, {NULL}};
const Segment SGI325797[] = {{&S72088[13], 10}, {NULL}};
const Segment SGI325798[] = {{&S72088[43], 7}, {NULL}};
const Segment SGI325799[] = {{&S72088[36], 14}, {NULL}};
const Segment SGI325800[] = {{&S72088[81], 6}, {NULL}};
const Segment SGI325801[] = {{&S72088[52], 6}, {NULL}};
const Segment SGI325802[] = {{&S72089[30], 4}, {NULL}};
const Segment SGI325803[] = {{&S72089[8], 3}, {NULL}};
const Segment SGI325804[] = {{&S72089[13], 10}, {NULL}};
const Segment SGI325805[] = {{&S72089[43], 7}, {NULL}};
const Segment SGI325806[] = {{&S72089[36], 14}, {NULL}};
const Segment SGI325807[] = {{&S72089[74], 6}, {NULL}};
const Segment SGI325808[] = {{&S72090[30], 4}, {NULL}};
const Segment SGI325809[] = {{&S72090[8], 3}, {NULL}};
const Segment SGI325810[] = {{&S72090[13], 10}, {NULL}};
const Segment SGI325811[] = {{&S72090[43], 7}, {NULL}};
const Segment SGI325812[] = {{&S72090[36], 14}, {NULL}};
const Segment SGI325813[] = {{&S72090[81], 6}, {NULL}};
const Segment SGI325814[] = {{&S72090[52], 6}, {NULL}};
const Segment SGI325815[] = {{&S72091[30], 4}, {NULL}};
const Segment SGI325816[] = {{&S72091[8], 3}, {NULL}};
const Segment SGI325817[] = {{&S72091[13], 10}, {NULL}};
const Segment SGI325818[] = {{&S72091[43], 7}, {NULL}};
const Segment SGI325819[] = {{&S72091[36], 14}, {NULL}};
const Segment SGI325820[] = {{&S72091[74], 6}, {NULL}};
const Segment SGI325821[] = {{&S72092[30], 4}, {NULL}};
const Segment SGI325822[] = {{&S72092[8], 3}, {NULL}};
const Segment SGI325823[] = {{&S72092[13], 10}, {NULL}};
const Segment SGI325824[] = {{&S72092[43], 7}, {NULL}};
const Segment SGI325825[] = {{&S72092[36], 14}, {NULL}};
const Segment SGI325826[] = {{&S72092[81], 6}, {NULL}};
const Segment SGI325827[] = {{&S72092[52], 6}, {NULL}};
const Segment SGI325828[] = {{&S72093[30], 4}, {NULL}};
const Segment SGI325829[] = {{&S72093[8], 3}, {NULL}};
const Segment SGI325830[] = {{&S72093[13], 10}, {NULL}};
const Segment SGI325831[] = {{&S72093[43], 7}, {NULL}};
const Segment SGI325832[] = {{&S72093[36], 14}, {NULL}};
const Segment SGI325833[] = {{&S72093[81], 6}, {NULL}};
const Segment SGI325834[] = {{&S72093[52], 6}, {NULL}};
const Segment SGI325835[] = {{&S72094[30], 4}, {NULL}};
const Segment SGI325836[] = {{&S72094[8], 3}, {NULL}};
const Segment SGI325837[] = {{&S72094[13], 10}, {NULL}};
const Segment SGI325838[] = {{&S72094[43], 7}, {NULL}};
const Segment SGI325839[] = {{&S72094[36], 14}, {NULL}};
const Segment SGI325840[] = {{&S72094[81], 6}, {NULL}};
const Segment SGI325841[] = {{&S72094[52], 6}, {NULL}};
const Segment SGI325842[] = {{&S85421[0], 71}, {NULL}};
const Segment SGI325843[] = {{&S85422[0], 60}, {NULL}};
const Segment SGI325844[] = {{&S85423[0], 60}, {NULL}};
const Segment SGI325845[] = {{&S85424[0], 51}, {NULL}};
const Segment SGI325846[] = {{&S76099[4], 3}, {NULL}};
const Segment SGI325847[] = {{&S76099[42], 43}, {NULL}};
const Segment SGI325848[] = {{&S76100[4], 3}, {NULL}};
const Segment SGI325849[] = {{&S76100[38], 41}, {NULL}};
const Segment SGI325850[] = {{&S85697[0], 59}, {NULL}};
const Segment SGI325851[] = {{&S72375[30], 3}, {NULL}};
const Segment SGI325852[] = {{&S72375[8], 3}, {NULL}};
const Segment SGI325853[] = {{&S72375[13], 10}, {NULL}};
const Segment SGI325854[] = {{&S72375[42], 7}, {NULL}};
const Segment SGI325855[] = {{&S72375[35], 14}, {NULL}};
const Segment SGI325856[] = {{&S72375[56], 11}, {NULL}};
const Segment SGI325857[] = {{&S72375[56], 15}, {NULL}};
const Segment SGI325858[] = {{&S72375[51], 3}, {NULL}};
const Segment SGI325859[] = {{&S72377[30], 3}, {NULL}};
const Segment SGI325860[] = {{&S72377[8], 3}, {NULL}};
const Segment SGI325861[] = {{&S72377[13], 10}, {NULL}};
const Segment SGI325862[] = {{&S72377[42], 7}, {NULL}};
const Segment SGI325863[] = {{&S72377[35], 14}, {NULL}};
const Segment SGI325864[] = {{&S72377[179], 11}, {NULL}};
const Segment SGI325865[] = {{&S72377[179], 15}, {NULL}};
const Segment SGI325866[] = {{&S72379[30], 3}, {NULL}};
const Segment SGI325867[] = {{&S72379[8], 3}, {NULL}};
const Segment SGI325868[] = {{&S90496[3], 5}, {NULL}};
const Segment SGI325869[] = {{&S72379[13], 10}, {NULL}};
const Segment SGI325870[] = {{&S72379[42], 7}, {NULL}};
const Segment SGI325871[] = {{&S72379[35], 14}, {NULL}};
const Segment SGI325872[] = {{&S72379[178], 11}, {NULL}};
const Segment SGI325873[] = {{&S72379[51], 6}, {NULL}};
const Segment SGI325874[] = {{&S90496[124], 13}, {NULL}};
const Segment SGI325875[] = {{&S72379[178], 15}, {NULL}};
const Segment SGI325876[] = {{&S90537[3], 5}, {NULL}};
const Segment SGI325877[] = {{&S76255[4], 2}, {NULL}};
const Segment SGI325878[] = {{&S76255[35], 45}, {NULL}};
const Segment SGI325879[] = {{&S72463[30], 4}, {NULL}};
const Segment SGI325880[] = {{&S72463[8], 3}, {NULL}};
const Segment SGI325881[] = {{&S72463[13], 10}, {NULL}};
const Segment SGI325882[] = {{&S72463[43], 7}, {NULL}};
const Segment SGI325883[] = {{&S72463[36], 14}, {NULL}};
const Segment SGI325884[] = {{&S76258[4], 15}, {NULL}};
const Segment SGI325885[] = {{&S72463[52], 6}, {NULL}};
const Segment SGI325886[] = {{&S72463[99], 7}, {NULL}};
const Segment SGI325887[] = {{&S72464[30], 4}, {NULL}};
const Segment SGI325888[] = {{&S72464[8], 3}, {NULL}};
const Segment SGI325889[] = {{&S72464[13], 10}, {NULL}};
const Segment SGI325890[] = {{&S72464[43], 7}, {NULL}};
const Segment SGI325891[] = {{&S72464[36], 14}, {NULL}};
const Segment SGI325892[] = {{&S72464[57], 7}, {NULL}};
const Segment SGI325893[] = {{&S72464[52], 3}, {NULL}};
const Segment SGI325894[] = {{&S72465[30], 4}, {NULL}};
const Segment SGI325895[] = {{&S72465[8], 3}, {NULL}};
const Segment SGI325896[] = {{&S72465[13], 10}, {NULL}};
const Segment SGI325897[] = {{&S72465[43], 7}, {NULL}};
const Segment SGI325898[] = {{&S72465[36], 14}, {NULL}};
const Segment SGI325899[] = {{&S72465[65], 7}, {NULL}};
const Segment SGI325900[] = {{&S72466[30], 4}, {NULL}};
const Segment SGI325901[] = {{&S72466[8], 3}, {NULL}};
const Segment SGI325902[] = {{&S72466[13], 10}, {NULL}};
const Segment SGI325903[] = {{&S72466[43], 7}, {NULL}};
const Segment SGI325904[] = {{&S72466[36], 14}, {NULL}};
const Segment SGI325905[] = {{&S72466[132], 15}, {NULL}};
const Segment SGI325906[] = {{&S72466[153], 7}, {NULL}};
const Segment SGI325907[] = {{&S101137[3], 5}, {NULL}};
const Segment SGI325908[] = {{&S101137[29], 4}, {NULL}};
const Segment SGI325909[] = {{&S72500[30], 3}, {NULL}};
const Segment SGI325910[] = {{&S72500[8], 3}, {NULL}};
const Segment SGI325911[] = {{&S72500[13], 10}, {NULL}};
const Segment SGI325912[] = {{&S72500[42], 3}, {NULL}};
const Segment SGI325913[] = {{&S72500[52], 13}, {NULL}};
const Segment SGI325914[] = {{&S72500[47], 3}, {NULL}};
const Segment SGI325915[] = {{&S72501[30], 3}, {NULL}};
const Segment SGI325916[] = {{&S72501[8], 3}, {NULL}};
const Segment SGI325917[] = {{&S72501[13], 10}, {NULL}};
const Segment SGI325918[] = {{&S72501[42], 3}, {NULL}};
const Segment SGI325919[] = {{&S72501[47], 13}, {&S72501[47], 13}, {NULL}};
const Segment SGI325920[] = {{&S72502[30], 3}, {NULL}};
const Segment SGI325921[] = {{&S72502[8], 3}, {NULL}};
const Segment SGI325922[] = {{&S72502[13], 10}, {NULL}};
const Segment SGI325923[] = {{&S72502[55], 3}, {NULL}};
const Segment SGI325924[] = {{&S72502[60], 13}, {NULL}};
const Segment SGI325925[] = {{&S72503[30], 3}, {NULL}};
const Segment SGI325926[] = {{&S72503[8], 3}, {NULL}};
const Segment SGI325927[] = {{&S72503[13], 10}, {NULL}};
const Segment SGI325928[] = {{&S72503[55], 3}, {NULL}};
const Segment SGI325929[] = {{&S72503[60], 13}, {NULL}};
const Segment SGI325930[] = {{&S72504[30], 3}, {NULL}};
const Segment SGI325931[] = {{&S72504[8], 3}, {NULL}};
const Segment SGI325932[] = {{&S72504[13], 10}, {NULL}};
const Segment SGI325933[] = {{&S72504[55], 3}, {NULL}};
const Segment SGI325934[] = {{&S72504[60], 13}, {NULL}};
const Segment SGI325935[] = {{&S72504[75], 3}, {NULL}};
const Segment SGI325936[] = {{&S72505[30], 3}, {NULL}};
const Segment SGI325937[] = {{&S72505[8], 3}, {NULL}};
const Segment SGI325938[] = {{&S72505[13], 10}, {NULL}};
const Segment SGI325939[] = {{&S72505[55], 3}, {NULL}};
const Segment SGI325940[] = {{&S72505[60], 13}, {&S72505[60], 13}, {NULL}};
const Segment SGI325941[] = {{&S101183[3], 5}, {NULL}};
const Segment SGI325942[] = {{&S72509[30], 3}, {NULL}};
const Segment SGI325943[] = {{&S72509[8], 3}, {NULL}};
const Segment SGI325944[] = {{&S72509[13], 10}, {NULL}};
const Segment SGI325945[] = {{&S72509[42], 7}, {NULL}};
const Segment SGI325946[] = {{&S72509[35], 14}, {NULL}};
const Segment SGI325947[] = {{&S72509[51], 6}, {NULL}};
const Segment SGI325948[] = {{&S72509[71], 13}, {NULL}};
const Segment SGI325949[] = {{&S72510[30], 3}, {NULL}};
const Segment SGI325950[] = {{&S72510[8], 3}, {NULL}};
const Segment SGI325951[] = {{&S72510[13], 10}, {NULL}};
const Segment SGI325952[] = {{&S72510[42], 7}, {NULL}};
const Segment SGI325953[] = {{&S72510[35], 14}, {NULL}};
const Segment SGI325954[] = {{&S72510[64], 13}, {NULL}};
const Segment SGI325955[] = {{&S72511[30], 3}, {NULL}};
const Segment SGI325956[] = {{&S72511[8], 3}, {NULL}};
const Segment SGI325957[] = {{&S72511[13], 10}, {NULL}};
const Segment SGI325958[] = {{&S72511[42], 7}, {NULL}};
const Segment SGI325959[] = {{&S72511[35], 14}, {NULL}};
const Segment SGI325960[] = {{&S72511[76], 13}, {NULL}};
const Segment SGI325961[] = {{&S72511[51], 5}, {NULL}};
const Segment SGI325962[] = {{&S76370[4], 4}, {NULL}};
const Segment SGI325963[] = {{&S76370[26], 11}, {NULL}};
const Segment SGI325964[] = {{&S76370[38], 5}, {NULL}};
const Segment SGI325965[] = {{&S76371[4], 4}, {NULL}};
const Segment SGI325966[] = {{&S76371[26], 11}, {NULL}};
const Segment SGI325967[] = {{&S76371[38], 5}, {NULL}};
const Segment SGI325968[] = {{&S85937[0], 32}, {NULL}};
const Segment SGI325969[] = {{&S85940[0], 32}, {NULL}};
const Segment SGI325970[] = {{&S101291[3], 4}, {NULL}};
const Segment SGI325971[] = {{&S101310[3], 5}, {NULL}};
const Segment SGI325972[] = {{&S72714[8], 3}, {NULL}};
const Segment SGI325973[] = {{&S72714[13], 10}, {NULL}};
const Segment SGI325974[] = {{&S72714[38], 7}, {NULL}};
const Segment SGI325975[] = {{&S72714[31], 14}, {NULL}};
const Segment SGI325976[] = {{&S72715[30], 3}, {NULL}};
const Segment SGI325977[] = {{&S72715[8], 3}, {NULL}};
const Segment SGI325978[] = {{&S72715[13], 10}, {NULL}};
const Segment SGI325979[] = {{&S72715[42], 7}, {NULL}};
const Segment SGI325980[] = {{&S72715[35], 14}, {NULL}};
const Segment SGI325981[] = {{&S72716[30], 3}, {NULL}};
const Segment SGI325982[] = {{&S72716[8], 3}, {NULL}};
const Segment SGI325983[] = {{&S72716[13], 10}, {NULL}};
const Segment SGI325984[] = {{&S72716[42], 7}, {NULL}};
const Segment SGI325985[] = {{&S72716[35], 14}, {NULL}};
const Segment SGI325986[] = {{&S90715[3], 5}, {NULL}};
const Segment SGI325987[] = {{&S90715[124], 13}, {NULL}};
const Segment SGI325988[] = {{&S90744[3], 5}, {NULL}};
const Segment SGI325989[] = {{&S90745[3], 5}, {NULL}};
const Segment SGI325990[] = {{&S101463[3], 5}, {NULL}};
const Segment SGI325991[] = {{&S86146[0], 12}, {NULL}};
const Segment SGI325992[] = {{&S72804[30], 4}, {NULL}};
const Segment SGI325993[] = {{&S72804[8], 3}, {NULL}};
const Segment SGI325994[] = {{&S72804[13], 10}, {NULL}};
const Segment SGI325995[] = {{&S72804[43], 7}, {NULL}};
const Segment SGI325996[] = {{&S72804[36], 14}, {NULL}};
const Segment SGI325997[] = {{&S72804[52], 6}, {NULL}};
const Segment SGI325998[] = {{&S72804[72], 7}, {NULL}};
const Segment SGI325999[] = {{&S72804[79], 4}, {NULL}};
const Segment SGI326000[] = {{&S72805[30], 4}, {NULL}};
const Segment SGI326001[] = {{&S72805[8], 3}, {NULL}};
const Segment SGI326002[] = {{&S72805[13], 10}, {NULL}};
const Segment SGI326003[] = {{&S72805[43], 7}, {NULL}};
const Segment SGI326004[] = {{&S72805[36], 14}, {NULL}};
const Segment SGI326005[] = {{&S90898[3], 5}, {NULL}};
const Segment SGI326006[] = {{&S72805[65], 7}, {NULL}};
const Segment SGI326007[] = {{&S72805[72], 4}, {NULL}};
const Segment SGI326008[] = {{&S90898[125], 13}, {NULL}};
const Segment SGI326009[] = {{&S86209[0], 13}, {NULL}};
const Segment SGI326010[] = {{&S72817[30], 3}, {NULL}};
const Segment SGI326011[] = {{&S72817[8], 3}, {NULL}};
const Segment SGI326012[] = {{&S72817[13], 10}, {NULL}};
const Segment SGI326013[] = {{&S72817[42], 7}, {NULL}};
const Segment SGI326014[] = {{&S72817[35], 14}, {NULL}};
const Segment SGI326015[] = {{&S72817[64], 11}, {NULL}};
const Segment SGI326016[] = {{&S86212[0], 93}, {NULL}};
const Segment SGI326017[] = {{&S72818[30], 3}, {NULL}};
const Segment SGI326018[] = {{&S72818[8], 3}, {NULL}};
const Segment SGI326019[] = {{&S72818[13], 10}, {NULL}};
const Segment SGI326020[] = {{&S72818[42], 7}, {NULL}};
const Segment SGI326021[] = {{&S72818[35], 14}, {NULL}};
const Segment SGI326022[] = {{&S72818[64], 11}, {NULL}};
const Segment SGI326023[] = {{&S72818[104], 15}, {NULL}};
const Segment SGI326024[] = {{&S86215[0], 93}, {NULL}};
const Segment SGI326025[] = {{&S90963[3], 4}, {NULL}};
const Segment SGI326026[] = {{&S72949[43], 3}, {NULL}};
const Segment SGI326027[] = {{&S72949[63], 5}, {NULL}};
const Segment SGI326028[] = {{&S72994[30], 3}, {NULL}};
const Segment SGI326029[] = {{&S72994[8], 3}, {NULL}};
const Segment SGI326030[] = {{&S72994[13], 10}, {NULL}};
const Segment SGI326031[] = {{&S72994[42], 7}, {NULL}};
const Segment SGI326032[] = {{&S72995[8], 3}, {NULL}};
const Segment SGI326033[] = {{&S72995[13], 10}, {NULL}};
const Segment SGI326034[] = {{&S72995[42], 7}, {NULL}};
const Segment SGI326035[] = {{&S72996[30], 3}, {NULL}};
const Segment SGI326036[] = {{&S72996[8], 3}, {NULL}};
const Segment SGI326037[] = {{&S72996[13], 10}, {NULL}};
const Segment SGI326038[] = {{&S72996[42], 7}, {NULL}};
const Segment SGI326039[] = {{&S72996[35], 14}, {NULL}};
const Segment SGI326040[] = {{&S101802[3], 5}, {NULL}};
const Segment SGI326041[] = {{&S101811[3], 4}, {NULL}};
const Segment SGI326042[] = {{&S77087[4], 5}, {NULL}};
const Segment SGI326043[] = {{&S91282[3], 5}, {NULL}};
const Segment SGI326044[] = {{&S77159[4], 2}, {NULL}};
const Segment SGI326045[] = {{&S77159[37], 39}, {NULL}};
const Segment SGI326046[] = {{&S86621[0], 32}, {NULL}};
const Segment SGI326047[] = {{&S101999[3], 5}, {NULL}};
const Segment SGI326048[] = {{&S73257[30], 4}, {NULL}};
const Segment SGI326049[] = {{&S73257[8], 3}, {NULL}};
const Segment SGI326050[] = {{&S73257[13], 10}, {NULL}};
const Segment SGI326051[] = {{&S73257[43], 7}, {NULL}};
const Segment SGI326052[] = {{&S73257[36], 14}, {NULL}};
const Segment SGI326053[] = {{&S73257[52], 6}, {NULL}};
const Segment SGI326054[] = {{&S73257[72], 7}, {NULL}};
const Segment SGI326055[] = {{&S73257[79], 4}, {NULL}};
const Segment SGI326056[] = {{&S73258[30], 3}, {NULL}};
const Segment SGI326057[] = {{&S73258[8], 3}, {NULL}};
const Segment SGI326058[] = {{&S73258[13], 10}, {NULL}};
const Segment SGI326059[] = {{&S73258[42], 7}, {NULL}};
const Segment SGI326060[] = {{&S73258[35], 14}, {NULL}};
const Segment SGI326061[] = {{&S73258[51], 6}, {NULL}};
const Segment SGI326062[] = {{&S73258[151], 7}, {NULL}};
const Segment SGI326063[] = {{&S73258[7], 4}, {NULL}};
const Segment SGI326064[] = {{&S73259[30], 4}, {NULL}};
const Segment SGI326065[] = {{&S73259[8], 3}, {NULL}};
const Segment SGI326066[] = {{&S73259[13], 10}, {NULL}};
const Segment SGI326067[] = {{&S73259[43], 7}, {NULL}};
const Segment SGI326068[] = {{&S73259[36], 14}, {NULL}};
const Segment SGI326069[] = {{&S73259[65], 7}, {NULL}};
const Segment SGI326070[] = {{&S73259[72], 4}, {NULL}};
const Segment SGI326071[] = {{&S86797[0], 32}, {NULL}};
const Segment SGI326072[] = {{&S73324[30], 3}, {NULL}};
const Segment SGI326073[] = {{&S73324[8], 3}, {NULL}};
const Segment SGI326074[] = {{&S73324[13], 10}, {NULL}};
const Segment SGI326075[] = {{&S73324[42], 7}, {NULL}};
const Segment SGI326076[] = {{&S73324[35], 14}, {NULL}};
const Segment SGI326077[] = {{&S73324[74], 3}, {NULL}};
const Segment SGI326078[] = {{&S91672[3], 5}, {NULL}};
const Segment SGI326079[] = {{&S91672[28], 9}, {NULL}};
const Segment SGI326080[] = {{&S73360[30], 3}, {NULL}};
const Segment SGI326081[] = {{&S73360[8], 3}, {NULL}};
const Segment SGI326082[] = {{&S73360[13], 10}, {NULL}};
const Segment SGI326083[] = {{&S73360[42], 2}, {NULL}};
const Segment SGI326084[] = {{&S73361[30], 3}, {NULL}};
const Segment SGI326085[] = {{&S73361[8], 3}, {NULL}};
const Segment SGI326086[] = {{&S73361[13], 10}, {NULL}};
const Segment SGI326087[] = {{&S73361[40], 9}, {NULL}};
const Segment SGI326088[] = {{&S73408[30], 4}, {NULL}};
const Segment SGI326089[] = {{&S73408[8], 3}, {NULL}};
const Segment SGI326090[] = {{&S73408[13], 10}, {NULL}};
const Segment SGI326091[] = {{&S73408[43], 7}, {NULL}};
const Segment SGI326092[] = {{&S73408[36], 14}, {NULL}};
const Segment SGI326093[] = {{&S73408[79], 11}, {NULL}};
const Segment SGI326094[] = {{&S73408[72], 5}, {NULL}};
const Segment SGI326095[] = {{&S73408[52], 6}, {NULL}};
const Segment SGI326096[] = {{&S73408[79], 16}, {NULL}};
const Segment SGI326097[] = {{&S86966[0], 71}, {NULL}};
const Segment SGI326098[] = {{&S102368[3], 5}, {NULL}};
const Segment SGI326099[] = {{&S77731[4], 5}, {NULL}};
const Segment SGI326100[] = {{&S77731[27], 11}, {NULL}};
const Segment SGI326101[] = {{&S77731[39], 5}, {NULL}};
const Segment SGI326102[] = {{&S77732[4], 5}, {NULL}};
const Segment SGI326103[] = {{&S77732[27], 11}, {NULL}};
const Segment SGI326104[] = {{&S77732[39], 5}, {NULL}};
const Segment SGI326105[] = {{&S77733[4], 5}, {NULL}};
const Segment SGI326106[] = {{&S77733[27], 11}, {NULL}};
const Segment SGI326107[] = {{&S77733[39], 5}, {NULL}};
const Segment SGI326108[] = {{&S77798[4], 5}, {NULL}};
const Segment SGI326109[] = {{&S77798[27], 11}, {NULL}};
const Segment SGI326110[] = {{&S77798[39], 5}, {NULL}};
const Segment SGI326111[] = {{&S77799[4], 5}, {NULL}};
const Segment SGI326112[] = {{&S77799[27], 11}, {NULL}};
const Segment SGI326113[] = {{&S77799[39], 5}, {NULL}};
const Segment SGI326114[] = {{&S77800[4], 5}, {NULL}};
const Segment SGI326115[] = {{&S77800[27], 11}, {NULL}};
const Segment SGI326116[] = {{&S77800[39], 5}, {NULL}};
const Segment SGI326117[] = {{&S77801[4], 5}, {NULL}};
const Segment SGI326118[] = {{&S77801[27], 11}, {NULL}};
const Segment SGI326119[] = {{&S77801[39], 5}, {NULL}};
const Segment SGI326120[] = {{&S77802[4], 5}, {NULL}};
const Segment SGI326121[] = {{&S77802[27], 11}, {NULL}};
const Segment SGI326122[] = {{&S77802[39], 5}, {NULL}};
const Segment SGI326123[] = {{&S77803[4], 4}, {NULL}};
const Segment SGI326124[] = {{&S77803[26], 11}, {NULL}};
const Segment SGI326125[] = {{&S77803[39], 5}, {NULL}};
const Segment SGI326126[] = {{&S77804[4], 5}, {NULL}};
const Segment SGI326127[] = {{&S77804[27], 11}, {NULL}};
const Segment SGI326128[] = {{&S77804[39], 5}, {NULL}};
const Segment SGI326129[] = {{&S73708[30], 3}, {NULL}};
const Segment SGI326130[] = {{&S73708[8], 3}, {NULL}};
const Segment SGI326131[] = {{&S73708[13], 10}, {NULL}};
const Segment SGI326132[] = {{&S73708[42], 7}, {NULL}};
const Segment SGI326133[] = {{&S73708[35], 14}, {NULL}};
const Segment SGI326134[] = {{&S73708[51], 6}, {NULL}};
const Segment SGI326135[] = {{&S73708[108], 4}, {NULL}};
const Segment SGI326136[] = {{&S73709[30], 3}, {NULL}};
const Segment SGI326137[] = {{&S73709[8], 3}, {NULL}};
const Segment SGI326138[] = {{&S73709[13], 10}, {NULL}};
const Segment SGI326139[] = {{&S73709[42], 7}, {NULL}};
const Segment SGI326140[] = {{&S73709[35], 14}, {NULL}};
const Segment SGI326141[] = {{&S73709[51], 3}, {NULL}};
const Segment SGI326142[] = {{&S73709[66], 4}, {NULL}};
const Segment SGI326143[] = {{&S73710[30], 3}, {NULL}};
const Segment SGI326144[] = {{&S73710[8], 3}, {NULL}};
const Segment SGI326145[] = {{&S73710[13], 10}, {NULL}};
const Segment SGI326146[] = {{&S73710[42], 7}, {NULL}};
const Segment SGI326147[] = {{&S73710[35], 14}, {NULL}};
const Segment SGI326148[] = {{&S73710[74], 4}, {NULL}};
const Segment SGI326149[] = {{&S73711[30], 3}, {NULL}};
const Segment SGI326150[] = {{&S73711[8], 3}, {NULL}};
const Segment SGI326151[] = {{&S73711[13], 10}, {NULL}};
const Segment SGI326152[] = {{&S87035[0], 33}, {NULL}};
const Segment SGI326153[] = {{&S73711[42], 7}, {NULL}};
const Segment SGI326154[] = {{&S73711[35], 14}, {NULL}};
const Segment SGI326155[] = {{&S73711[51], 6}, {NULL}};
const Segment SGI326156[] = {{&S73711[108], 4}, {NULL}};
const Segment SGI326157[] = {{&S73712[30], 3}, {NULL}};
const Segment SGI326158[] = {{&S73712[8], 3}, {NULL}};
const Segment SGI326159[] = {{&S73712[13], 10}, {NULL}};
const Segment SGI326160[] = {{&S73712[42], 7}, {NULL}};
const Segment SGI326161[] = {{&S73712[35], 14}, {NULL}};
const Segment SGI326162[] = {{&S73712[147], 15}, {NULL}};
const Segment SGI326163[] = {{&S73712[168], 9}, {NULL}};
const Segment SGI326164[] = {{&S73712[193], 4}, {NULL}};
const Segment SGI326165[] = {{&S92077[3], 5}, {NULL}};
const Segment SGI326166[] = {{&S102568[3], 4}, {NULL}};
const Segment SGI326167[] = {{&S73766[30], 3}, {NULL}};
const Segment SGI326168[] = {{&S73766[8], 3}, {NULL}};
const Segment SGI326169[] = {{&S73766[13], 10}, {NULL}};
const Segment SGI326170[] = {{&S73766[35], 8}, {NULL}};
const Segment SGI326171[] = {{&S73767[30], 3}, {NULL}};
const Segment SGI326172[] = {{&S73767[8], 3}, {NULL}};
const Segment SGI326173[] = {{&S73767[13], 10}, {NULL}};
const Segment SGI326174[] = {{&S73767[35], 8}, {NULL}};
const Segment SGI326175[] = {{&S78061[4], 9}, {NULL}};
const Segment SGI326176[] = {{&S78061[43], 32}, {NULL}};
const Segment SGI326177[] = {{&S78062[4], 9}, {NULL}};
const Segment SGI326178[] = {{&S78062[43], 25}, {NULL}};
const Segment SGI326179[] = {{&S102776[3], 5}, {NULL}};
const Segment SGI326180[] = {{&S73956[30], 3}, {NULL}};
const Segment SGI326181[] = {{&S73956[8], 3}, {NULL}};
const Segment SGI326182[] = {{&S73956[13], 10}, {NULL}};
const Segment SGI326183[] = {{&S73956[42], 2}, {NULL}};
const Segment SGI326184[] = {{&S73957[30], 3}, {NULL}};
const Segment SGI326185[] = {{&S73957[8], 3}, {NULL}};
const Segment SGI326186[] = {{&S73957[13], 10}, {NULL}};
const Segment SGI326187[] = {{&S73957[40], 9}, {NULL}};
const Segment SGI326188[] = {{&S78112[4], 4}, {NULL}};
const Segment SGI326189[] = {{&S78112[26], 11}, {NULL}};
const Segment SGI326190[] = {{&S78112[38], 5}, {NULL}};
const Segment SGI326191[] = {{&S78113[4], 4}, {NULL}};
const Segment SGI326192[] = {{&S78113[26], 11}, {NULL}};
const Segment SGI326193[] = {{&S78113[38], 5}, {NULL}};
const Segment SGI326194[] = {{&S92212[3], 4}, {NULL}};
const Segment SGI326195[] = {{&S92274[3], 5}, {NULL}};
const Segment SGI326196[] = {{&S92278[3], 5}, {NULL}};
const Segment SGI326197[] = {{&S92280[3], 5}, {NULL}};
const Segment SGI326198[] = {{&S81584[4], 6}, {NULL}};
const Segment SGI326199[] = {{&S102984[3], 5}, {NULL}};
const Segment SGI326200[] = {{&S87528[0], 85}, {NULL}};
const Segment SGI326201[] = {{&S87529[0], 85}, {NULL}};
const Segment SGI326202[] = {{&S87530[0], 85}, {NULL}};
const Segment SGI326203[] = {{&S87531[0], 87}, {NULL}};
const Segment SGI326204[] = {{&S87532[0], 85}, {NULL}};
const Segment SGI326205[] = {{&S87533[0], 85}, {NULL}};
const Segment SGI326206[] = {{&S87534[0], 85}, {NULL}};
const Segment SGI326207[] = {{&S87535[0], 85}, {NULL}};
const Segment SGI326208[] = {{&S87536[0], 85}, {NULL}};
const Segment SGI326209[] = {{&S87537[0], 85}, {NULL}};
const Segment SGI326210[] = {{&S92484[3], 4}, {NULL}};
const Segment SGI326211[] = {{&S87726[0], 32}, {NULL}};
const Segment SGI326212[] = {{&S81859[4], 5}, {NULL}};
const Segment SGI326213[] = {{&S74140[30], 3}, {NULL}};
const Segment SGI326214[] = {{&S74140[8], 3}, {NULL}};
const Segment SGI326215[] = {{&S74140[13], 10}, {NULL}};
const Segment SGI326216[] = {{&S74140[42], 7}, {NULL}};
const Segment SGI326217[] = {{&S74140[35], 14}, {NULL}};
const Segment SGI326218[] = {{&S74140[56], 11}, {NULL}};
const Segment SGI326219[] = {{&S74140[56], 15}, {NULL}};
const Segment SGI326220[] = {{&S74140[51], 3}, {NULL}};
const Segment SGI326221[] = {{&S74141[30], 3}, {NULL}};
const Segment SGI326222[] = {{&S74141[8], 3}, {NULL}};
const Segment SGI326223[] = {{&S74141[13], 10}, {NULL}};
const Segment SGI326224[] = {{&S74141[42], 7}, {NULL}};
const Segment SGI326225[] = {{&S74141[35], 14}, {NULL}};
const Segment SGI326226[] = {{&S74141[202], 11}, {NULL}};
const Segment SGI326227[] = {{&S74141[202], 15}, {NULL}};
const Segment SGI326228[] = {{&S103260[3], 5}, {NULL}};
const Segment SGI326229[] = {{&S103263[3], 5}, {NULL}};
const Segment SGI326230[] = {{&S87931[0], 70}, {NULL}};
const Segment SGI326231[] = {{&S87955[0], 71}, {NULL}};
const Segment SGI326232[] = {{&S103515[3], 5}, {NULL}};
const Segment SGI326233[] = {{&S103540[3], 6}, {NULL}};
const Segment SGI326234[] = {{&S103540[29], 8}, {NULL}};
const Segment SGI326235[] = {{&S103546[3], 5}, {NULL}};
const Segment SGI326236[] = {{&S103573[3], 5}, {NULL}};
const Segment SGI326237[] = {{&S103608[3], 6}, {NULL}};
const Segment SGI326238[] = {{&S82263[4], 6}, {NULL}};
const Segment SGI326239[] = {{&S82263[26], 3}, {NULL}};
const Segment SGI326240[] = {{&S93016[3], 5}, {NULL}};
const Segment SGI326241[] = {{&S93016[123], 13}, {NULL}};
const Segment SGI326242[] = {{&S74532[30], 3}, {NULL}};
const Segment SGI326243[] = {{&S74532[8], 3}, {NULL}};
const Segment SGI326244[] = {{&S74532[13], 10}, {NULL}};
const Segment SGI326245[] = {{&S74532[42], 7}, {NULL}};
const Segment SGI326246[] = {{&S74532[35], 14}, {NULL}};
const Segment SGI326247[] = {{&S74532[131], 15}, {NULL}};
const Segment SGI326248[] = {{&S74533[30], 3}, {NULL}};
const Segment SGI326249[] = {{&S74533[8], 3}, {NULL}};
const Segment SGI326250[] = {{&S74533[13], 10}, {NULL}};
const Segment SGI326251[] = {{&S74533[42], 7}, {NULL}};
const Segment SGI326252[] = {{&S74533[35], 14}, {NULL}};
const Segment SGI326253[] = {{&S74534[30], 3}, {NULL}};
const Segment SGI326254[] = {{&S74534[8], 3}, {NULL}};
const Segment SGI326255[] = {{&S74534[13], 10}, {NULL}};
const Segment SGI326256[] = {{&S74534[42], 7}, {NULL}};
const Segment SGI326257[] = {{&S74534[35], 14}, {NULL}};
const Segment SGI326258[] = {{&S74689[8], 3}, {NULL}};
const Segment SGI326259[] = {{&S74689[13], 10}, {NULL}};
const Segment SGI326260[] = {{&S74689[63], 7}, {NULL}};
const Segment SGI326261[] = {{&S74697[30], 3}, {NULL}};
const Segment SGI326262[] = {{&S74697[8], 3}, {NULL}};
const Segment SGI326263[] = {{&S74697[13], 10}, {NULL}};
const Segment SGI326264[] = {{&S74697[42], 7}, {NULL}};
const Segment SGI326265[] = {{&S74697[35], 14}, {NULL}};
const Segment SGI326266[] = {{&S74698[30], 3}, {NULL}};
const Segment SGI326267[] = {{&S74698[8], 3}, {NULL}};
const Segment SGI326268[] = {{&S74698[13], 10}, {NULL}};
const Segment SGI326269[] = {{&S74698[42], 7}, {NULL}};
const Segment SGI326270[] = {{&S74698[35], 14}, {NULL}};
const Segment SGI326271[] = {{&S74844[30], 3}, {NULL}};
const Segment SGI326272[] = {{&S74844[8], 3}, {NULL}};
const Segment SGI326273[] = {{&S74844[13], 10}, {NULL}};
const Segment SGI326274[] = {{&S74844[42], 7}, {NULL}};
const Segment SGI326275[] = {{&S74844[35], 14}, {NULL}};
const Segment SGI326276[] = {{&S74844[64], 7}, {NULL}};
const Segment SGI326277[] = {{&S74844[7], 4}, {NULL}};
const Segment SGI326278[] = {{&S88610[0], 37}, {NULL}};
const Segment SGI326279[] = {{&S79302[4], 11}, {NULL}};
const Segment SGI326280[] = {{&S74885[30], 3}, {NULL}};
const Segment SGI326281[] = {{&S74885[8], 3}, {NULL}};
const Segment SGI326282[] = {{&S74885[13], 10}, {NULL}};
const Segment SGI326283[] = {{&S74885[35], 6}, {NULL}};
const Segment SGI326284[] = {{&S74886[30], 3}, {NULL}};
const Segment SGI326285[] = {{&S74886[8], 3}, {NULL}};
const Segment SGI326286[] = {{&S74886[13], 10}, {NULL}};
const Segment SGI326287[] = {{&S74886[35], 6}, {NULL}};
const Segment SGI326288[] = {{&S93277[3], 5}, {NULL}};
const Segment SGI326289[] = {{&S79411[4], 4}, {NULL}};
const Segment SGI326290[] = {{&S79411[27], 8}, {NULL}};
const Segment SGI326291[] = {{&S79411[40], 5}, {NULL}};
const Segment SGI326292[] = {{&S93336[3], 5}, {NULL}};
const Segment SGI326293[] = {{&S93336[107], 12}, {NULL}};
const Segment SGI326294[] = {{&S93354[3], 5}, {NULL}};
const Segment SGI326295[] = {{&S93354[147], 13}, {NULL}};
const Segment SGI326296[] = {{&S93378[3], 5}, {NULL}};
const Segment SGI326297[] = {{&S93378[146], 13}, {NULL}};
const Segment SGI326298[] = {{&S93514[3], 6}, {NULL}};
const Segment SGI326299[] = {{&S93515[3], 5}, {NULL}};
const Segment SGI326300[] = {{&S83128[4], 6}, {NULL}};
const Segment SGI326301[] = {{&S83128[26], 3}, {NULL}};
const Segment SGI326302[] = {{&S93724[3], 6}, {NULL}};
const Segment SGI326303[] = {{&S93725[3], 5}, {NULL}};
const Segment SGI326304[] = {{&S93726[3], 5}, {NULL}};
const Segment SGI326305[] = {{&S93726[3], 10}, {NULL}};
const Segment SGI326306[] = {{&S93736[3], 5}, {NULL}};
const Segment SGI326307[] = {{&S93967[3], 5}, {NULL}};
const Segment SGI326308[] = {{&S93973[3], 5}, {NULL}};
const Segment SGI326309[] = {{&S93974[3], 5}, {NULL}};
const Segment SGI326310[] = {{&S80116[4], 6}, {NULL}};
const Segment SGI326311[] = {{&S80116[40], 32}, {NULL}};
const Segment SGI326312[] = {{&S80117[4], 6}, {NULL}};
const Segment SGI326313[] = {{&S80117[40], 32}, {NULL}};
const Segment SGI326314[] = {{&S80118[4], 6}, {NULL}};
const Segment SGI326315[] = {{&S80118[40], 25}, {NULL}};
const Segment SGI326316[] = {{&S75582[30], 3}, {NULL}};
const Segment SGI326317[] = {{&S75582[8], 3}, {NULL}};
const Segment SGI326318[] = {{&S75582[13], 10}, {NULL}};
const Segment SGI326319[] = {{&S75582[42], 7}, {NULL}};
const Segment SGI326320[] = {{&S75582[35], 14}, {NULL}};
const Segment SGI326321[] = {{&S75582[131], 15}, {NULL}};
const Segment SGI326322[] = {{&S75582[46], 3}, {NULL}};
const Segment SGI326323[] = {{&S75584[30], 3}, {NULL}};
const Segment SGI326324[] = {{&S75584[8], 3}, {NULL}};
const Segment SGI326325[] = {{&S75584[13], 10}, {NULL}};
const Segment SGI326326[] = {{&S75584[42], 7}, {NULL}};
const Segment SGI326327[] = {{&S75584[46], 3}, {NULL}};
const Segment SGI326328[] = {{&S94051[3], 5}, {NULL}};
const Segment SGI326329[] = {{&S94051[123], 13}, {NULL}};
const Segment SGI326330[] = {{&S94148[3], 5}, {NULL}};
const Segment SGI326331[] = {{&S94148[124], 13}, {NULL}};
const Segment SGI326332[] = {{&S89713[0], 55}, {NULL}};
const Segment SGI326333[] = {{&S89714[0], 56}, {NULL}};
const Segment SGI326334[] = {{&S89715[0], 56}, {NULL}};
const Segment SGI326335[] = {{&S89716[0], 56}, {NULL}};
const Segment SGI326336[] = {{&S89717[0], 55}, {NULL}};
const Segment SGI326337[] = {{&S75662[30], 4}, {NULL}};
const Segment SGI326338[] = {{&S75662[8], 3}, {NULL}};
const Segment SGI326339[] = {{&S75662[13], 10}, {NULL}};
const Segment SGI326340[] = {{&S75662[43], 7}, {NULL}};
const Segment SGI326341[] = {{&S75662[36], 14}, {NULL}};
const Segment SGI326342[] = {{&S75662[81], 5}, {NULL}};
const Segment SGI326343[] = {{&S75662[52], 6}, {NULL}};
const Segment SGI326344[] = {{&S75744[30], 3}, {NULL}};
const Segment SGI326345[] = {{&S75744[8], 3}, {NULL}};
const Segment SGI326346[] = {{&S75744[13], 10}, {NULL}};
const Segment SGI326347[] = {{&S75744[42], 7}, {NULL}};
const Segment SGI326348[] = {{&S75745[30], 3}, {NULL}};
const Segment SGI326349[] = {{&S75745[8], 3}, {NULL}};
const Segment SGI326350[] = {{&S75745[13], 10}, {NULL}};
const Segment SGI326351[] = {{&S75745[35], 8}, {NULL}};
const Segment SGI326352[] = {{&S75746[30], 3}, {NULL}};
const Segment SGI326353[] = {{&S75746[8], 3}, {NULL}};
const Segment SGI326354[] = {{&S75746[13], 10}, {NULL}};
const Segment SGI326355[] = {{&S75746[69], 7}, {NULL}};
const Segment SGI326356[] = {{&S75746[62], 14}, {NULL}};
const Segment SGI326357[] = {{&S94202[3], 4}, {NULL}};
const Segment SGI326358[] = {{&S89822[0], 59}, {NULL}};
const Segment SGI326359[] = {{&S94302[3], 5}, {NULL}};
const Segment SGI326360[] = {{&S94302[125], 13}, {NULL}};
const Segment SGI326361[] = {{&S94388[3], 5}, {NULL}};
const Segment SGI326362[] = {{&S94400[3], 5}, {NULL}};
const Segment SGI326363[] = {{&S76040[30], 3}, {NULL}};
const Segment SGI326364[] = {{&S76040[8], 3}, {NULL}};
const Segment SGI326365[] = {{&S76040[13], 10}, {NULL}};
const Segment SGI326366[] = {{&S76040[42], 7}, {NULL}};
const Segment SGI326367[] = {{&S76040[35], 14}, {NULL}};
const Segment SGI326368[] = {{&S76040[91], 15}, {NULL}};
const Segment SGI326369[] = {{&S76040[187], 5}, {NULL}};
const Segment SGI326370[] = {{&S76041[30], 3}, {NULL}};
const Segment SGI326371[] = {{&S76041[8], 3}, {NULL}};
const Segment SGI326372[] = {{&S76041[13], 10}, {NULL}};
const Segment SGI326373[] = {{&S76041[42], 7}, {NULL}};
const Segment SGI326374[] = {{&S76041[35], 14}, {NULL}};
const Segment SGI326375[] = {{&S76041[97], 15}, {NULL}};
const Segment SGI326376[] = {{&S76041[167], 5}, {NULL}};
const Segment SGI326377[] = {{&S76042[30], 3}, {NULL}};
const Segment SGI326378[] = {{&S76042[8], 3}, {NULL}};
const Segment SGI326379[] = {{&S76042[13], 10}, {NULL}};
const Segment SGI326380[] = {{&S76042[42], 7}, {NULL}};
const Segment SGI326381[] = {{&S76042[35], 14}, {NULL}};
const Segment SGI326382[] = {{&S76042[145], 5}, {NULL}};
const Segment SGI326383[] = {{&S80502[4], 2}, {NULL}};
const Segment SGI326384[] = {{&S76111[30], 3}, {NULL}};
const Segment SGI326385[] = {{&S76111[8], 3}, {NULL}};
const Segment SGI326386[] = {{&S76111[13], 10}, {NULL}};
const Segment SGI326387[] = {{&S76111[42], 7}, {NULL}};
const Segment SGI326388[] = {{&S76111[35], 14}, {NULL}};
const Segment SGI326389[] = {{&S76111[80], 5}, {NULL}};
const Segment SGI326390[] = {{&S76111[51], 6}, {NULL}};
const Segment SGI326391[] = {{&S76112[30], 3}, {NULL}};
const Segment SGI326392[] = {{&S76112[8], 3}, {NULL}};
const Segment SGI326393[] = {{&S76112[13], 10}, {NULL}};
const Segment SGI326394[] = {{&S76112[42], 7}, {NULL}};
const Segment SGI326395[] = {{&S76112[35], 14}, {NULL}};
const Segment SGI326396[] = {{&S76112[80], 5}, {NULL}};
const Segment SGI326397[] = {{&S76112[51], 6}, {NULL}};
const Segment SGI326398[] = {{&S76113[30], 3}, {NULL}};
const Segment SGI326399[] = {{&S76113[8], 3}, {NULL}};
const Segment SGI326400[] = {{&S76113[13], 10}, {NULL}};
const Segment SGI326401[] = {{&S76113[42], 7}, {NULL}};
const Segment SGI326402[] = {{&S76113[35], 14}, {NULL}};
const Segment SGI326403[] = {{&S76113[73], 5}, {NULL}};
const Segment SGI326404[] = {{&S76114[30], 3}, {NULL}};
const Segment SGI326405[] = {{&S76114[8], 3}, {NULL}};
const Segment SGI326406[] = {{&S76114[13], 10}, {NULL}};
const Segment SGI326407[] = {{&S76114[42], 7}, {NULL}};
const Segment SGI326408[] = {{&S76114[35], 14}, {NULL}};
const Segment SGI326409[] = {{&S76114[80], 5}, {NULL}};
const Segment SGI326410[] = {{&S76114[51], 6}, {NULL}};
const Segment SGI326411[] = {{&S76115[30], 3}, {NULL}};
const Segment SGI326412[] = {{&S76115[8], 3}, {NULL}};
const Segment SGI326413[] = {{&S76115[13], 10}, {NULL}};
const Segment SGI326414[] = {{&S76115[42], 7}, {NULL}};
const Segment SGI326415[] = {{&S76115[35], 14}, {NULL}};
const Segment SGI326416[] = {{&S76115[80], 5}, {NULL}};
const Segment SGI326417[] = {{&S76115[51], 6}, {NULL}};
const Segment SGI326418[] = {{&S76116[30], 3}, {NULL}};
const Segment SGI326419[] = {{&S76116[8], 3}, {NULL}};
const Segment SGI326420[] = {{&S76116[13], 10}, {NULL}};
const Segment SGI326421[] = {{&S76116[42], 7}, {NULL}};
const Segment SGI326422[] = {{&S76116[35], 14}, {NULL}};
const Segment SGI326423[] = {{&S76116[80], 6}, {NULL}};
const Segment SGI326424[] = {{&S76116[51], 6}, {NULL}};
const Segment SGI326425[] = {{&S76117[30], 3}, {NULL}};
const Segment SGI326426[] = {{&S76117[8], 3}, {NULL}};
const Segment SGI326427[] = {{&S76117[13], 10}, {NULL}};
const Segment SGI326428[] = {{&S76117[42], 7}, {NULL}};
const Segment SGI326429[] = {{&S76117[35], 14}, {NULL}};
const Segment SGI326430[] = {{&S76117[80], 5}, {NULL}};
const Segment SGI326431[] = {{&S76117[51], 6}, {NULL}};
const Segment SGI326432[] = {{&S76118[30], 3}, {NULL}};
const Segment SGI326433[] = {{&S76118[8], 3}, {NULL}};
const Segment SGI326434[] = {{&S76118[13], 10}, {NULL}};
const Segment SGI326435[] = {{&S76118[42], 7}, {NULL}};
const Segment SGI326436[] = {{&S76118[35], 14}, {NULL}};
const Segment SGI326437[] = {{&S76118[73], 5}, {NULL}};
const Segment SGI326438[] = {{&S76119[30], 3}, {NULL}};
const Segment SGI326439[] = {{&S76119[8], 3}, {NULL}};
const Segment SGI326440[] = {{&S76119[13], 10}, {NULL}};
const Segment SGI326441[] = {{&S76119[42], 7}, {NULL}};
const Segment SGI326442[] = {{&S76119[35], 14}, {NULL}};
const Segment SGI326443[] = {{&S76119[80], 5}, {NULL}};
const Segment SGI326444[] = {{&S76119[51], 6}, {NULL}};
const Segment SGI326445[] = {{&S76120[30], 3}, {NULL}};
const Segment SGI326446[] = {{&S76120[8], 3}, {NULL}};
const Segment SGI326447[] = {{&S76120[13], 10}, {NULL}};
const Segment SGI326448[] = {{&S76120[42], 7}, {NULL}};
const Segment SGI326449[] = {{&S76120[35], 14}, {NULL}};
const Segment SGI326450[] = {{&S76120[80], 5}, {NULL}};
const Segment SGI326451[] = {{&S76120[51], 6}, {NULL}};
const Segment SGI326452[] = {{&S76121[30], 3}, {NULL}};
const Segment SGI326453[] = {{&S76121[8], 3}, {NULL}};
const Segment SGI326454[] = {{&S76121[13], 10}, {NULL}};
const Segment SGI326455[] = {{&S76121[42], 7}, {NULL}};
const Segment SGI326456[] = {{&S76121[35], 14}, {NULL}};
const Segment SGI326457[] = {{&S76121[73], 5}, {NULL}};
const Segment SGI326458[] = {{&S76122[30], 3}, {NULL}};
const Segment SGI326459[] = {{&S76122[8], 3}, {NULL}};
const Segment SGI326460[] = {{&S76122[13], 10}, {NULL}};
const Segment SGI326461[] = {{&S76122[42], 7}, {NULL}};
const Segment SGI326462[] = {{&S76122[35], 14}, {NULL}};
const Segment SGI326463[] = {{&S76122[80], 5}, {NULL}};
const Segment SGI326464[] = {{&S76122[51], 6}, {NULL}};
const Segment SGI326465[] = {{&S76123[30], 3}, {NULL}};
const Segment SGI326466[] = {{&S76123[8], 3}, {NULL}};
const Segment SGI326467[] = {{&S76123[13], 10}, {NULL}};
const Segment SGI326468[] = {{&S76123[42], 7}, {NULL}};
const Segment SGI326469[] = {{&S76123[35], 14}, {NULL}};
const Segment SGI326470[] = {{&S76123[73], 6}, {NULL}};
const Segment SGI326471[] = {{&S76124[30], 3}, {NULL}};
const Segment SGI326472[] = {{&S76124[8], 3}, {NULL}};
const Segment SGI326473[] = {{&S76124[13], 10}, {NULL}};
const Segment SGI326474[] = {{&S76124[42], 7}, {NULL}};
const Segment SGI326475[] = {{&S76124[35], 14}, {NULL}};
const Segment SGI326476[] = {{&S76124[80], 6}, {NULL}};
const Segment SGI326477[] = {{&S76124[51], 6}, {NULL}};
const Segment SGI326478[] = {{&S76125[30], 3}, {NULL}};
const Segment SGI326479[] = {{&S76125[8], 3}, {NULL}};
const Segment SGI326480[] = {{&S76125[13], 10}, {NULL}};
const Segment SGI326481[] = {{&S76125[42], 7}, {NULL}};
const Segment SGI326482[] = {{&S76125[35], 14}, {NULL}};
const Segment SGI326483[] = {{&S76125[73], 6}, {NULL}};
const Segment SGI326484[] = {{&S76126[30], 3}, {NULL}};
const Segment SGI326485[] = {{&S76126[8], 3}, {NULL}};
const Segment SGI326486[] = {{&S76126[13], 10}, {NULL}};
const Segment SGI326487[] = {{&S76126[42], 7}, {NULL}};
const Segment SGI326488[] = {{&S76126[35], 14}, {NULL}};
const Segment SGI326489[] = {{&S76126[80], 6}, {NULL}};
const Segment SGI326490[] = {{&S76126[51], 6}, {NULL}};
const Segment SGI326491[] = {{&S76127[30], 3}, {NULL}};
const Segment SGI326492[] = {{&S76127[8], 3}, {NULL}};
const Segment SGI326493[] = {{&S76127[13], 10}, {NULL}};
const Segment SGI326494[] = {{&S76127[42], 7}, {NULL}};
const Segment SGI326495[] = {{&S76127[35], 14}, {NULL}};
const Segment SGI326496[] = {{&S76127[80], 5}, {NULL}};
const Segment SGI326497[] = {{&S76127[51], 6}, {NULL}};
const Segment SGI326498[] = {{&S76128[30], 3}, {NULL}};
const Segment SGI326499[] = {{&S76128[8], 3}, {NULL}};
const Segment SGI326500[] = {{&S76128[13], 10}, {NULL}};
const Segment SGI326501[] = {{&S76128[42], 7}, {NULL}};
const Segment SGI326502[] = {{&S76128[35], 14}, {NULL}};
const Segment SGI326503[] = {{&S76128[80], 6}, {NULL}};
const Segment SGI326504[] = {{&S76128[51], 6}, {NULL}};
const Segment SGI326505[] = {{&S76129[30], 3}, {NULL}};
const Segment SGI326506[] = {{&S76129[8], 3}, {NULL}};
const Segment SGI326507[] = {{&S76129[13], 10}, {NULL}};
const Segment SGI326508[] = {{&S76129[42], 7}, {NULL}};
const Segment SGI326509[] = {{&S76129[35], 14}, {NULL}};
const Segment SGI326510[] = {{&S76129[73], 6}, {NULL}};
const Segment SGI326511[] = {{&S76130[30], 3}, {NULL}};
const Segment SGI326512[] = {{&S76130[8], 3}, {NULL}};
const Segment SGI326513[] = {{&S76130[13], 10}, {NULL}};
const Segment SGI326514[] = {{&S76130[42], 7}, {NULL}};
const Segment SGI326515[] = {{&S76130[35], 14}, {NULL}};
const Segment SGI326516[] = {{&S76130[80], 6}, {NULL}};
const Segment SGI326517[] = {{&S76130[51], 6}, {NULL}};
const Segment SGI326518[] = {{&S80575[4], 5}, {NULL}};
const Segment SGI326519[] = {{&S76131[30], 3}, {NULL}};
const Segment SGI326520[] = {{&S76131[8], 3}, {NULL}};
const Segment SGI326521[] = {{&S76131[13], 10}, {NULL}};
const Segment SGI326522[] = {{&S76131[42], 7}, {NULL}};
const Segment SGI326523[] = {{&S76131[35], 14}, {NULL}};
const Segment SGI326524[] = {{&S76131[80], 6}, {NULL}};
const Segment SGI326525[] = {{&S76131[51], 6}, {NULL}};
const Segment SGI326526[] = {{&S76132[30], 3}, {NULL}};
const Segment SGI326527[] = {{&S76132[8], 3}, {NULL}};
const Segment SGI326528[] = {{&S76132[13], 10}, {NULL}};
const Segment SGI326529[] = {{&S76132[42], 7}, {NULL}};
const Segment SGI326530[] = {{&S76132[35], 14}, {NULL}};
const Segment SGI326531[] = {{&S76132[73], 6}, {NULL}};
const Segment SGI326532[] = {{&S76133[30], 3}, {NULL}};
const Segment SGI326533[] = {{&S76133[8], 3}, {NULL}};
const Segment SGI326534[] = {{&S76133[13], 10}, {NULL}};
const Segment SGI326535[] = {{&S76133[42], 7}, {NULL}};
const Segment SGI326536[] = {{&S76133[35], 14}, {NULL}};
const Segment SGI326537[] = {{&S76133[80], 6}, {NULL}};
const Segment SGI326538[] = {{&S76133[51], 6}, {NULL}};
const Segment SGI326539[] = {{&S76134[30], 3}, {NULL}};
const Segment SGI326540[] = {{&S76134[8], 3}, {NULL}};
const Segment SGI326541[] = {{&S76134[13], 10}, {NULL}};
const Segment SGI326542[] = {{&S76134[42], 7}, {NULL}};
const Segment SGI326543[] = {{&S76134[35], 14}, {NULL}};
const Segment SGI326544[] = {{&S76134[80], 6}, {NULL}};
const Segment SGI326545[] = {{&S76134[51], 6}, {NULL}};
const Segment SGI326546[] = {{&S76135[30], 3}, {NULL}};
const Segment SGI326547[] = {{&S76135[8], 3}, {NULL}};
const Segment SGI326548[] = {{&S76135[13], 10}, {NULL}};
const Segment SGI326549[] = {{&S76135[42], 7}, {NULL}};
const Segment SGI326550[] = {{&S76135[35], 14}, {NULL}};
const Segment SGI326551[] = {{&S76135[80], 5}, {NULL}};
const Segment SGI326552[] = {{&S76135[51], 6}, {NULL}};
const Segment SGI326553[] = {{&S76136[30], 3}, {NULL}};
const Segment SGI326554[] = {{&S76136[8], 3}, {NULL}};
const Segment SGI326555[] = {{&S76136[13], 10}, {NULL}};
const Segment SGI326556[] = {{&S76136[42], 7}, {NULL}};
const Segment SGI326557[] = {{&S76136[35], 14}, {NULL}};
const Segment SGI326558[] = {{&S76136[73], 6}, {NULL}};
const Segment SGI326559[] = {{&S76144[30], 4}, {NULL}};
const Segment SGI326560[] = {{&S76144[8], 3}, {NULL}};
const Segment SGI326561[] = {{&S76144[13], 10}, {NULL}};
const Segment SGI326562[] = {{&S76144[43], 7}, {NULL}};
const Segment SGI326563[] = {{&S76144[36], 14}, {NULL}};
const Segment SGI326564[] = {{&S76144[79], 11}, {NULL}};
const Segment SGI326565[] = {{&S76144[72], 5}, {NULL}};
const Segment SGI326566[] = {{&S76144[52], 6}, {NULL}};
const Segment SGI326567[] = {{&S76144[79], 16}, {NULL}};
const Segment SGI326568[] = {{&S105130[3], 5}, {NULL}};
const Segment SGI326569[] = {{&S105130[28], 9}, {NULL}};
const Segment SGI326570[] = {{&S94607[3], 5}, {NULL}};
const Segment SGI326571[] = {{&S94607[124], 13}, {NULL}};
const Segment SGI326572[] = {{&S84171[4], 6}, {NULL}};
const Segment SGI326573[] = {{&S84171[26], 3}, {NULL}};
const Segment SGI326574[] = {{&S84172[4], 6}, {NULL}};
const Segment SGI326575[] = {{&S84172[26], 3}, {NULL}};
const Segment SGI326576[] = {{&S94627[3], 5}, {NULL}};
const Segment SGI326577[] = {{&S94638[3], 5}, {NULL}};
const Segment SGI326578[] = {{&S94639[3], 5}, {NULL}};
const Segment SGI326579[] = {{&S94639[3], 12}, {NULL}};
const Segment SGI326580[] = {{&S94649[3], 5}, {NULL}};
const Segment SGI326581[] = {{&S94649[20], 3}, {NULL}};
const Segment SGI326582[] = {{&S80647[4], 10}, {NULL}};
const Segment SGI326583[] = {{&S80647[44], 32}, {NULL}};
const Segment SGI326584[] = {{&S76273[30], 3}, {NULL}};
const Segment SGI326585[] = {{&S76273[8], 3}, {NULL}};
const Segment SGI326586[] = {{&S76273[13], 10}, {NULL}};
const Segment SGI326587[] = {{&S76273[42], 7}, {NULL}};
const Segment SGI326588[] = {{&S76273[35], 14}, {NULL}};
const Segment SGI326589[] = {{&S80648[4], 10}, {NULL}};
const Segment SGI326590[] = {{&S80648[45], 32}, {NULL}};
const Segment SGI326591[] = {{&S80649[4], 10}, {NULL}};
const Segment SGI326592[] = {{&S80649[44], 32}, {NULL}};
const Segment SGI326593[] = {{&S80650[4], 10}, {NULL}};
const Segment SGI326594[] = {{&S80650[45], 32}, {NULL}};
const Segment SGI326595[] = {{&S80651[4], 10}, {NULL}};
const Segment SGI326596[] = {{&S80651[45], 25}, {NULL}};
const Segment SGI326597[] = {{&S76281[30], 4}, {NULL}};
const Segment SGI326598[] = {{&S76281[8], 3}, {NULL}};
const Segment SGI326599[] = {{&S76281[13], 10}, {NULL}};
const Segment SGI326600[] = {{&S76281[43], 7}, {NULL}};
const Segment SGI326601[] = {{&S76281[36], 14}, {NULL}};
const Segment SGI326602[] = {{&S76281[79], 11}, {NULL}};
const Segment SGI326603[] = {{&S76281[72], 5}, {NULL}};
const Segment SGI326604[] = {{&S76281[52], 6}, {NULL}};
const Segment SGI326605[] = {{&S76281[79], 16}, {NULL}};
const Segment SGI326606[] = {{&S84259[4], 6}, {NULL}};
const Segment SGI326607[] = {{&S84259[22], 3}, {NULL}};
const Segment SGI326608[] = {{&S94772[3], 5}, {NULL}};
const Segment SGI326609[] = {{&S94772[105], 13}, {NULL}};
const Segment SGI326610[] = {{&S94882[3], 5}, {NULL}};
const Segment SGI326611[] = {{&S94883[3], 5}, {NULL}};
const Segment SGI326612[] = {{&S94885[3], 5}, {NULL}};
const Segment SGI326613[] = {{&S94888[3], 5}, {NULL}};
const Segment SGI326614[] = {{&S80793[4], 4}, {NULL}};
const Segment SGI326615[] = {{&S80793[27], 8}, {NULL}};
const Segment SGI326616[] = {{&S80793[39], 5}, {NULL}};
const Segment SGI326617[] = {{&S94987[3], 5}, {NULL}};
const Segment SGI326618[] = {{&S94987[124], 13}, {NULL}};
const Segment SGI326619[] = {{&S95130[3], 5}, {NULL}};
const Segment SGI326620[] = {{&S95132[3], 6}, {NULL}};
const Segment SGI326621[] = {{&S95132[25], 4}, {NULL}};
const Segment SGI326622[] = {{&S90253[0], 6}, {NULL}};
const Segment SGI326623[] = {{&S95139[3], 5}, {NULL}};
const Segment SGI326624[] = {{&S95139[29], 9}, {NULL}};
const Segment SGI326625[] = {{&S95141[3], 5}, {NULL}};
const Segment SGI326626[] = {{&S95144[3], 4}, {NULL}};
const Segment SGI326627[] = {{&S76663[30], 3}, {NULL}};
const Segment SGI326628[] = {{&S76663[8], 3}, {NULL}};
const Segment SGI326629[] = {{&S76663[13], 10}, {NULL}};
const Segment SGI326630[] = {{&S76663[35], 6}, {NULL}};
const Segment SGI326631[] = {{&S90366[0], 80}, {NULL}};
const Segment SGI326632[] = {{&S76887[30], 4}, {NULL}};
const Segment SGI326633[] = {{&S76887[8], 3}, {NULL}};
const Segment SGI326634[] = {{&S76887[13], 10}, {NULL}};
const Segment SGI326635[] = {{&S76888[30], 4}, {NULL}};
const Segment SGI326636[] = {{&S76888[8], 3}, {NULL}};
const Segment SGI326637[] = {{&S76888[13], 10}, {NULL}};
const Segment SGI326638[] = {{&S76888[36], 5}, {NULL}};
const Segment SGI326639[] = {{&S90596[0], 70}, {NULL}};
const Segment SGI326640[] = {{&S77300[30], 3}, {NULL}};
const Segment SGI326641[] = {{&S77300[8], 3}, {NULL}};
const Segment SGI326642[] = {{&S77300[13], 10}, {NULL}};
const Segment SGI326643[] = {{&S77301[30], 3}, {NULL}};
const Segment SGI326644[] = {{&S77301[8], 3}, {NULL}};
const Segment SGI326645[] = {{&S77301[13], 10}, {NULL}};
const Segment SGI326646[] = {{&S95839[3], 5}, {NULL}};
const Segment SGI326647[] = {{&S95839[124], 13}, {NULL}};
const Segment SGI326648[] = {{&S90806[0], 19}, {NULL}};
const Segment SGI326649[] = {{&S95949[3], 5}, {NULL}};
const Segment SGI326650[] = {{&S95970[3], 5}, {NULL}};
const Segment SGI326651[] = {{&S95980[3], 5}, {NULL}};
const Segment SGI326652[] = {{&S81230[4], 5}, {NULL}};
const Segment SGI326653[] = {{&S81230[27], 11}, {NULL}};
const Segment SGI326654[] = {{&S81230[39], 5}, {NULL}};
const Segment SGI326655[] = {{&S81231[4], 5}, {NULL}};
const Segment SGI326656[] = {{&S81231[27], 11}, {NULL}};
const Segment SGI326657[] = {{&S81231[39], 5}, {NULL}};
const Segment SGI326658[] = {{&S81232[4], 5}, {NULL}};
const Segment SGI326659[] = {{&S81232[27], 11}, {NULL}};
const Segment SGI326660[] = {{&S81232[39], 5}, {NULL}};
const Segment SGI326661[] = {{&S81233[4], 5}, {NULL}};
const Segment SGI326662[] = {{&S81233[25], 11}, {NULL}};
const Segment SGI326663[] = {{&S81233[37], 5}, {NULL}};
const Segment SGI326664[] = {{&S81234[4], 5}, {NULL}};
const Segment SGI326665[] = {{&S81234[27], 11}, {NULL}};
const Segment SGI326666[] = {{&S81234[39], 5}, {NULL}};
const Segment SGI326667[] = {{&S81235[4], 12}, {NULL}};
const Segment SGI326668[] = {{&S81235[32], 11}, {NULL}};
const Segment SGI326669[] = {{&S81235[44], 5}, {NULL}};
const Segment SGI326670[] = {{&S81236[4], 5}, {NULL}};
const Segment SGI326671[] = {{&S81236[27], 11}, {NULL}};
const Segment SGI326672[] = {{&S81236[39], 5}, {NULL}};
const Segment SGI326673[] = {{&S96028[3], 5}, {NULL}};
const Segment SGI326674[] = {{&S96028[115], 13}, {NULL}};
const Segment SGI326675[] = {{&S96077[3], 5}, {NULL}};
const Segment SGI326676[] = {{&S96077[123], 13}, {NULL}};
const Segment SGI326677[] = {{&S77655[30], 4}, {NULL}};
const Segment SGI326678[] = {{&S77655[8], 3}, {NULL}};
const Segment SGI326679[] = {{&S77655[13], 10}, {NULL}};
const Segment SGI326680[] = {{&S77655[36], 8}, {NULL}};
const Segment SGI326681[] = {{&S77748[30], 3}, {NULL}};
const Segment SGI326682[] = {{&S77748[8], 3}, {NULL}};
const Segment SGI326683[] = {{&S77748[13], 10}, {NULL}};
const Segment SGI326684[] = {{&S77748[35], 3}, {NULL}};
const Segment SGI326685[] = {{&S77748[40], 9}, {NULL}};
const Segment SGI326686[] = {{&S96292[3], 5}, {NULL}};
const Segment SGI326687[] = {{&S96383[3], 5}, {NULL}};
const Segment SGI326688[] = {{&S96383[123], 13}, {NULL}};
const Segment SGI326689[] = {{&S81585[4], 5}, {NULL}};
const Segment SGI326690[] = {{&S81585[27], 11}, {NULL}};
const Segment SGI326691[] = {{&S81585[39], 5}, {NULL}};
const Segment SGI326692[] = {{&S78036[30], 4}, {NULL}};
const Segment SGI326693[] = {{&S78036[8], 3}, {NULL}};
const Segment SGI326694[] = {{&S78036[13], 10}, {NULL}};
const Segment SGI326695[] = {{&S78036[43], 7}, {NULL}};
const Segment SGI326696[] = {{&S78036[36], 14}, {NULL}};
const Segment SGI326697[] = {{&S78036[52], 6}, {NULL}};
const Segment SGI326698[] = {{&S78036[109], 4}, {NULL}};
const Segment SGI326699[] = {{&S78037[30], 4}, {NULL}};
const Segment SGI326700[] = {{&S78037[8], 3}, {NULL}};
const Segment SGI326701[] = {{&S78037[13], 10}, {NULL}};
const Segment SGI326702[] = {{&S78037[43], 7}, {NULL}};
const Segment SGI326703[] = {{&S78037[36], 14}, {NULL}};
const Segment SGI326704[] = {{&S78037[75], 4}, {NULL}};
const Segment SGI326705[] = {{&S78038[30], 4}, {NULL}};
const Segment SGI326706[] = {{&S78038[8], 3}, {NULL}};
const Segment SGI326707[] = {{&S78038[13], 10}, {NULL}};
const Segment SGI326708[] = {{&S78038[43], 7}, {NULL}};
const Segment SGI326709[] = {{&S78038[36], 14}, {NULL}};
const Segment SGI326710[] = {{&S78038[52], 3}, {NULL}};
const Segment SGI326711[] = {{&S78038[67], 4}, {NULL}};
const Segment SGI326712[] = {{&S78039[30], 4}, {NULL}};
const Segment SGI326713[] = {{&S78039[8], 3}, {NULL}};
const Segment SGI326714[] = {{&S78039[13], 10}, {NULL}};
const Segment SGI326715[] = {{&S78039[43], 7}, {NULL}};
const Segment SGI326716[] = {{&S78039[36], 14}, {NULL}};
const Segment SGI326717[] = {{&S78039[139], 15}, {NULL}};
const Segment SGI326718[] = {{&S78039[52], 6}, {NULL}};
const Segment SGI326719[] = {{&S78039[190], 4}, {NULL}};
const Segment SGI326720[] = {{&S91106[0], 33}, {NULL}};
const Segment SGI326721[] = {{&S91107[0], 37}, {NULL}};
const Segment SGI326722[] = {{&S96596[3], 6}, {NULL}};
const Segment SGI326723[] = {{&S96597[3], 6}, {NULL}};
const Segment SGI326724[] = {{&S96607[3], 5}, {NULL}};
const Segment SGI326725[] = {{&S78137[30], 3}, {NULL}};
const Segment SGI326726[] = {{&S78137[8], 3}, {NULL}};
const Segment SGI326727[] = {{&S78137[13], 10}, {NULL}};
const Segment SGI326728[] = {{&S78137[42], 7}, {NULL}};
const Segment SGI326729[] = {{&S78137[35], 14}, {NULL}};
const Segment SGI326730[] = {{&S96678[3], 5}, {NULL}};
const Segment SGI326731[] = {{&S96678[123], 13}, {NULL}};
const Segment SGI326732[] = {{&S78325[30], 3}, {NULL}};
const Segment SGI326733[] = {{&S78325[8], 3}, {NULL}};
const Segment SGI326734[] = {{&S78325[13], 10}, {NULL}};
const Segment SGI326735[] = {{&S78325[42], 7}, {NULL}};
const Segment SGI326736[] = {{&S78325[35], 14}, {NULL}};
const Segment SGI326737[] = {{&S78325[74], 4}, {NULL}};
const Segment SGI326738[] = {{&S78328[30], 3}, {NULL}};
const Segment SGI326739[] = {{&S78328[8], 3}, {NULL}};
const Segment SGI326740[] = {{&S78328[13], 10}, {NULL}};
const Segment SGI326741[] = {{&S78328[42], 7}, {NULL}};
const Segment SGI326742[] = {{&S78328[35], 14}, {NULL}};
const Segment SGI326743[] = {{&S78328[51], 3}, {NULL}};
const Segment SGI326744[] = {{&S78328[86], 4}, {NULL}};
const Segment SGI326745[] = {{&S78332[8], 3}, {NULL}};
const Segment SGI326746[] = {{&S78332[13], 10}, {NULL}};
const Segment SGI326747[] = {{&S78332[42], 7}, {NULL}};
const Segment SGI326748[] = {{&S78333[30], 3}, {NULL}};
const Segment SGI326749[] = {{&S78333[8], 3}, {NULL}};
const Segment SGI326750[] = {{&S78333[13], 10}, {NULL}};
const Segment SGI326751[] = {{&S78333[35], 8}, {NULL}};
const Segment SGI326752[] = {{&S96878[3], 4}, {NULL}};
const Segment SGI326753[] = {{&S96922[3], 5}, {NULL}};
const Segment SGI326754[] = {{&S91482[0], 13}, {NULL}};
const Segment SGI326755[] = {{&S96938[3], 5}, {NULL}};
const Segment SGI326756[] = {{&S78432[30], 4}, {NULL}};
const Segment SGI326757[] = {{&S78432[8], 3}, {NULL}};
const Segment SGI326758[] = {{&S78432[13], 10}, {NULL}};
const Segment SGI326759[] = {{&S78432[43], 8}, {NULL}};
const Segment SGI326760[] = {{&S78433[8], 3}, {NULL}};
const Segment SGI326761[] = {{&S78433[13], 10}, {NULL}};
const Segment SGI326762[] = {{&S97116[3], 5}, {NULL}};
const Segment SGI326763[] = {{&S97116[115], 13}, {NULL}};
const Segment SGI326764[] = {{&S78550[30], 3}, {NULL}};
const Segment SGI326765[] = {{&S78550[8], 3}, {NULL}};
const Segment SGI326766[] = {{&S78550[13], 10}, {NULL}};
const Segment SGI326767[] = {{&S97239[3], 5}, {NULL}};
const Segment SGI326768[] = {{&S82181[4], 6}, {NULL}};
const Segment SGI326769[] = {{&S82181[40], 32}, {NULL}};
const Segment SGI326770[] = {{&S82182[4], 6}, {NULL}};
const Segment SGI326771[] = {{&S82182[40], 32}, {NULL}};
const Segment SGI326772[] = {{&S82183[4], 6}, {NULL}};
const Segment SGI326773[] = {{&S82183[40], 32}, {NULL}};
const Segment SGI326774[] = {{&S82184[4], 6}, {NULL}};
const Segment SGI326775[] = {{&S82184[41], 32}, {NULL}};
const Segment SGI326776[] = {{&S82185[4], 6}, {NULL}};
const Segment SGI326777[] = {{&S82185[40], 32}, {NULL}};
const Segment SGI326778[] = {{&S82186[4], 7}, {NULL}};
const Segment SGI326779[] = {{&S82186[42], 32}, {NULL}};
const Segment SGI326780[] = {{&S82187[4], 6}, {NULL}};
const Segment SGI326781[] = {{&S82187[40], 32}, {NULL}};
const Segment SGI326782[] = {{&S82188[4], 6}, {NULL}};
const Segment SGI326783[] = {{&S82188[40], 32}, {NULL}};
const Segment SGI326784[] = {{&S82189[4], 6}, {NULL}};
const Segment SGI326785[] = {{&S82189[41], 32}, {NULL}};
const Segment SGI326786[] = {{&S82190[4], 6}, {NULL}};
const Segment SGI326787[] = {{&S82190[40], 32}, {NULL}};
const Segment SGI326788[] = {{&S82191[4], 6}, {NULL}};
const Segment SGI326789[] = {{&S82191[41], 32}, {NULL}};
const Segment SGI326790[] = {{&S82192[4], 8}, {NULL}};
const Segment SGI326791[] = {{&S82192[43], 32}, {NULL}};
const Segment SGI326792[] = {{&S82193[4], 7}, {NULL}};
const Segment SGI326793[] = {{&S82193[41], 32}, {NULL}};
const Segment SGI326794[] = {{&S82194[4], 6}, {NULL}};
const Segment SGI326795[] = {{&S82194[40], 32}, {NULL}};
const Segment SGI326796[] = {{&S91947[0], 71}, {NULL}};
const Segment SGI326797[] = {{&S91973[0], 209}, {NULL}};
const Segment SGI326798[] = {{&S97366[3], 5}, {NULL}};
const Segment SGI326799[] = {{&S97366[81], 13}, {NULL}};
const Segment SGI326800[] = {{&S78914[30], 4}, {NULL}};
const Segment SGI326801[] = {{&S78914[8], 3}, {NULL}};
const Segment SGI326802[] = {{&S78914[13], 10}, {NULL}};
const Segment SGI326803[] = {{&S78914[36], 5}, {NULL}};
const Segment SGI326804[] = {{&S82509[4], 4}, {NULL}};
const Segment SGI326805[] = {{&S82509[27], 8}, {NULL}};
const Segment SGI326806[] = {{&S79019[30], 3}, {NULL}};
const Segment SGI326807[] = {{&S79019[8], 3}, {NULL}};
const Segment SGI326808[] = {{&S79019[13], 10}, {NULL}};
const Segment SGI326809[] = {{&S79019[42], 5}, {NULL}};
const Segment SGI326810[] = {{&S79020[30], 3}, {NULL}};
const Segment SGI326811[] = {{&S79020[8], 3}, {NULL}};
const Segment SGI326812[] = {{&S79020[13], 10}, {NULL}};
const Segment SGI326813[] = {{&S79020[42], 5}, {NULL}};
const Segment SGI326814[] = {{&S79113[30], 3}, {NULL}};
const Segment SGI326815[] = {{&S79113[8], 3}, {NULL}};
const Segment SGI326816[] = {{&S79113[13], 10}, {NULL}};
const Segment SGI326817[] = {{&S79113[42], 7}, {NULL}};
const Segment SGI326818[] = {{&S79113[35], 14}, {NULL}};
const Segment SGI326819[] = {{&S79113[51], 3}, {NULL}};
const Segment SGI326820[] = {{&S79113[69], 5}, {NULL}};
const Segment SGI326821[] = {{&S79113[96], 9}, {NULL}};
const Segment SGI326822[] = {{&S79125[30], 3}, {NULL}};
const Segment SGI326823[] = {{&S79125[8], 3}, {NULL}};
const Segment SGI326824[] = {{&S79125[13], 10}, {NULL}};
const Segment SGI326825[] = {{&S79125[42], 7}, {NULL}};
const Segment SGI326826[] = {{&S79125[35], 14}, {NULL}};
const Segment SGI326827[] = {{&S79125[64], 11}, {NULL}};
const Segment SGI326828[] = {{&S79125[64], 16}, {NULL}};
const Segment SGI326829[] = {{&S79126[30], 3}, {NULL}};
const Segment SGI326830[] = {{&S79126[8], 3}, {NULL}};
const Segment SGI326831[] = {{&S79126[13], 10}, {NULL}};
const Segment SGI326832[] = {{&S79126[42], 7}, {NULL}};
const Segment SGI326833[] = {{&S79126[35], 14}, {NULL}};
const Segment SGI326834[] = {{&S79126[59], 11}, {NULL}};
const Segment SGI326835[] = {{&S79126[59], 16}, {NULL}};
const Segment SGI326836[] = {{&S79128[30], 3}, {NULL}};
const Segment SGI326837[] = {{&S79128[8], 3}, {NULL}};
const Segment SGI326838[] = {{&S79128[13], 10}, {NULL}};
const Segment SGI326839[] = {{&S79128[42], 7}, {NULL}};
const Segment SGI326840[] = {{&S79128[35], 14}, {NULL}};
const Segment SGI326841[] = {{&S79128[64], 11}, {NULL}};
const Segment SGI326842[] = {{&S79128[64], 16}, {NULL}};
const Segment SGI326843[] = {{&S79130[30], 3}, {NULL}};
const Segment SGI326844[] = {{&S79130[8], 3}, {NULL}};
const Segment SGI326845[] = {{&S79130[13], 10}, {NULL}};
const Segment SGI326846[] = {{&S79130[42], 7}, {NULL}};
const Segment SGI326847[] = {{&S79130[35], 14}, {NULL}};
const Segment SGI326848[] = {{&S79130[72], 11}, {NULL}};
const Segment SGI326849[] = {{&S79130[124], 15}, {NULL}};
const Segment SGI326850[] = {{&S79130[72], 16}, {NULL}};
const Segment SGI326851[] = {{&S79131[30], 3}, {NULL}};
const Segment SGI326852[] = {{&S79131[8], 3}, {NULL}};
const Segment SGI326853[] = {{&S79131[13], 10}, {NULL}};
const Segment SGI326854[] = {{&S79131[42], 7}, {NULL}};
const Segment SGI326855[] = {{&S79131[35], 14}, {NULL}};
const Segment SGI326856[] = {{&S79131[83], 11}, {NULL}};
const Segment SGI326857[] = {{&S79131[51], 6}, {NULL}};
const Segment SGI326858[] = {{&S79131[83], 16}, {NULL}};
const Segment SGI326859[] = {{&S97630[3], 5}, {NULL}};
const Segment SGI326860[] = {{&S97632[3], 5}, {NULL}};
const Segment SGI326861[] = {{&S79133[30], 3}, {NULL}};
const Segment SGI326862[] = {{&S79133[8], 3}, {NULL}};
const Segment SGI326863[] = {{&S79133[13], 10}, {NULL}};
const Segment SGI326864[] = {{&S79133[42], 7}, {NULL}};
const Segment SGI326865[] = {{&S79133[35], 14}, {NULL}};
const Segment SGI326866[] = {{&S79133[51], 11}, {NULL}};
const Segment SGI326867[] = {{&S79133[103], 15}, {NULL}};
const Segment SGI326868[] = {{&S79133[51], 16}, {NULL}};
const Segment SGI326869[] = {{&S79134[30], 3}, {NULL}};
const Segment SGI326870[] = {{&S79134[8], 3}, {NULL}};
const Segment SGI326871[] = {{&S79134[13], 10}, {NULL}};
const Segment SGI326872[] = {{&S79134[42], 7}, {NULL}};
const Segment SGI326873[] = {{&S79134[35], 14}, {NULL}};
const Segment SGI326874[] = {{&S79134[58], 11}, {NULL}};
const Segment SGI326875[] = {{&S79134[110], 15}, {NULL}};
const Segment SGI326876[] = {{&S79134[51], 6}, {NULL}};
const Segment SGI326877[] = {{&S79134[58], 16}, {NULL}};
const Segment SGI326878[] = {{&S92352[0], 38}, {NULL}};
const Segment SGI326879[] = {{&S92463[0], 118}, {NULL}};
const Segment SGI326880[] = {{&S97805[3], 5}, {NULL}};
const Segment SGI326881[] = {{&S97805[121], 13}, {NULL}};
const Segment SGI326882[] = {{&S92477[0], 34}, {NULL}};
const Segment SGI326883[] = {{&S92478[0], 109}, {NULL}};
const Segment SGI326884[] = {{&S83138[4], 4}, {NULL}};
const Segment SGI326885[] = {{&S83138[26], 8}, {NULL}};
const Segment SGI326886[] = {{&S83138[42], 5}, {NULL}};
const Segment SGI326887[] = {{&S83139[4], 4}, {NULL}};
const Segment SGI326888[] = {{&S83139[26], 8}, {NULL}};
const Segment SGI326889[] = {{&S83139[42], 5}, {NULL}};
const Segment SGI326890[] = {{&S83140[4], 4}, {NULL}};
const Segment SGI326891[] = {{&S83140[26], 8}, {NULL}};
const Segment SGI326892[] = {{&S83140[42], 5}, {NULL}};
const Segment SGI326893[] = {{&S79550[30], 3}, {NULL}};
const Segment SGI326894[] = {{&S79550[8], 3}, {NULL}};
const Segment SGI326895[] = {{&S79550[13], 10}, {NULL}};
const Segment SGI326896[] = {{&S79550[35], 6}, {NULL}};
const Segment SGI326897[] = {{&S79551[30], 3}, {NULL}};
const Segment SGI326898[] = {{&S79551[8], 3}, {NULL}};
const Segment SGI326899[] = {{&S79551[13], 10}, {NULL}};
const Segment SGI326900[] = {{&S79551[42], 7}, {NULL}};
const Segment SGI326901[] = {{&S79551[35], 14}, {NULL}};
const Segment SGI326902[] = {{&S79552[30], 3}, {NULL}};
const Segment SGI326903[] = {{&S79552[8], 3}, {NULL}};
const Segment SGI326904[] = {{&S79552[13], 10}, {NULL}};
const Segment SGI326905[] = {{&S79552[42], 7}, {NULL}};
const Segment SGI326906[] = {{&S79552[35], 14}, {NULL}};
const Segment SGI326907[] = {{&S98049[3], 5}, {NULL}};
const Segment SGI326908[] = {{&S98059[3], 5}, {NULL}};
const Segment SGI326909[] = {{&S98059[3], 12}, {NULL}};
const Segment SGI326910[] = {{&S98059[35], 10}, {NULL}};
const Segment SGI326911[] = {{&S98060[3], 5}, {NULL}};
const Segment SGI326912[] = {{&S98060[23], 10}, {NULL}};
const Segment SGI326913[] = {{&S98080[3], 5}, {NULL}};
const Segment SGI326914[] = {{&S98080[28], 9}, {NULL}};
const Segment SGI326915[] = {{&S98209[3], 5}, {NULL}};
const Segment SGI326916[] = {{&S98209[123], 13}, {NULL}};
const Segment SGI326917[] = {{&S98282[3], 5}, {NULL}};
const Segment SGI326918[] = {{&S98282[124], 13}, {NULL}};
const Segment SGI326919[] = {{&S83478[4], 4}, {NULL}};
const Segment SGI326920[] = {{&S83478[24], 11}, {NULL}};
const Segment SGI326921[] = {{&S83478[36], 5}, {NULL}};
const Segment SGI326922[] = {{&S83479[4], 4}, {NULL}};
const Segment SGI326923[] = {{&S83479[26], 11}, {NULL}};
const Segment SGI326924[] = {{&S83479[38], 5}, {NULL}};
const Segment SGI326925[] = {{&S88180[4], 5}, {NULL}};
const Segment SGI326926[] = {{&S98576[3], 5}, {NULL}};
const Segment SGI326927[] = {{&S98576[78], 13}, {NULL}};
const Segment SGI326928[] = {{&S80166[30], 3}, {NULL}};
const Segment SGI326929[] = {{&S80166[8], 3}, {NULL}};
const Segment SGI326930[] = {{&S80166[13], 10}, {NULL}};
const Segment SGI326931[] = {{&S80166[42], 7}, {NULL}};
const Segment SGI326932[] = {{&S80166[35], 14}, {NULL}};
const Segment SGI326933[] = {{&S88289[4], 5}, {NULL}};
const Segment SGI326934[] = {{&S88289[25], 3}, {NULL}};
const Segment SGI326935[] = {{&S93529[0], 43}, {NULL}};
const Segment SGI326936[] = {{&S83989[4], 5}, {NULL}};
const Segment SGI326937[] = {{&S83989[27], 8}, {NULL}};
const Segment SGI326938[] = {{&S83989[43], 5}, {NULL}};
const Segment SGI326939[] = {{&S80376[30], 3}, {NULL}};
const Segment SGI326940[] = {{&S80376[8], 3}, {NULL}};
const Segment SGI326941[] = {{&S80376[13], 10}, {NULL}};
const Segment SGI326942[] = {{&S80376[42], 7}, {NULL}};
const Segment SGI326943[] = {{&S80376[35], 14}, {NULL}};
const Segment SGI326944[] = {{&S84073[4], 5}, {NULL}};
const Segment SGI326945[] = {{&S84073[27], 8}, {NULL}};
const Segment SGI326946[] = {{&S84073[43], 5}, {NULL}};
const Segment SGI326947[] = {{&S84074[4], 5}, {NULL}};
const Segment SGI326948[] = {{&S84074[27], 8}, {NULL}};
const Segment SGI326949[] = {{&S84074[43], 5}, {NULL}};
const Segment SGI326950[] = {{&S84075[4], 4}, {NULL}};
const Segment SGI326951[] = {{&S84075[24], 8}, {NULL}};
const Segment SGI326952[] = {{&S84075[40], 5}, {NULL}};
const Segment SGI326953[] = {{&S80484[30], 3}, {NULL}};
const Segment SGI326954[] = {{&S80484[8], 3}, {NULL}};
const Segment SGI326955[] = {{&S80484[13], 10}, {NULL}};
const Segment SGI326956[] = {{&S80484[42], 7}, {NULL}};
const Segment SGI326957[] = {{&S80484[35], 14}, {NULL}};
const Segment SGI326958[] = {{&S80485[30], 3}, {NULL}};
const Segment SGI326959[] = {{&S80485[8], 3}, {NULL}};
const Segment SGI326960[] = {{&S80485[13], 10}, {NULL}};
const Segment SGI326961[] = {{&S80485[42], 7}, {NULL}};
const Segment SGI326962[] = {{&S80485[35], 14}, {NULL}};
const Segment SGI326963[] = {{&S84164[4], 3}, {NULL}};
const Segment SGI326964[] = {{&S84164[48], 49}, {NULL}};
const Segment SGI326965[] = {{&S88576[4], 5}, {NULL}};
const Segment SGI326966[] = {{&S88576[25], 3}, {NULL}};
const Segment SGI326967[] = {{&S93970[0], 44}, {NULL}};
const Segment SGI326968[] = {{&S99008[3], 5}, {NULL}};
const Segment SGI326969[] = {{&S99025[3], 6}, {NULL}};
const Segment SGI326970[] = {{&S99048[3], 5}, {NULL}};
const Segment SGI326971[] = {{&S80904[30], 3}, {NULL}};
const Segment SGI326972[] = {{&S80904[8], 3}, {NULL}};
const Segment SGI326973[] = {{&S80904[13], 10}, {NULL}};
const Segment SGI326974[] = {{&S80960[30], 3}, {NULL}};
const Segment SGI326975[] = {{&S80960[8], 3}, {NULL}};
const Segment SGI326976[] = {{&S80960[13], 10}, {NULL}};
const Segment SGI326977[] = {{&S80960[35], 6}, {NULL}};
const Segment SGI326978[] = {{&S80961[30], 4}, {NULL}};
const Segment SGI326979[] = {{&S80961[8], 3}, {NULL}};
const Segment SGI326980[] = {{&S80961[13], 10}, {NULL}};
const Segment SGI326981[] = {{&S80961[43], 5}, {NULL}};
const Segment SGI326982[] = {{&S84397[4], 5}, {NULL}};
const Segment SGI326983[] = {{&S99191[3], 5}, {NULL}};
const Segment SGI326984[] = {{&S99191[123], 13}, {NULL}};
const Segment SGI326985[] = {{&S84419[4], 5}, {NULL}};
const Segment SGI326986[] = {{&S81031[30], 4}, {NULL}};
const Segment SGI326987[] = {{&S81031[8], 3}, {NULL}};
const Segment SGI326988[] = {{&S81031[13], 10}, {NULL}};
const Segment SGI326989[] = {{&S88975[4], 6}, {NULL}};
const Segment SGI326990[] = {{&S88975[26], 3}, {NULL}};
const Segment SGI326991[] = {{&S99296[3], 5}, {NULL}};
const Segment SGI326992[] = {{&S99296[162], 13}, {NULL}};
const Segment SGI326993[] = {{&S84564[4], 7}, {NULL}};
const Segment SGI326994[] = {{&S84564[42], 32}, {NULL}};
const Segment SGI326995[] = {{&S84565[4], 7}, {NULL}};
const Segment SGI326996[] = {{&S84565[42], 25}, {NULL}};
const Segment SGI326997[] = {{&S84566[4], 11}, {NULL}};
const Segment SGI326998[] = {{&S84566[20], 52}, {NULL}};
const Segment SGI326999[] = {{&S84567[4], 11}, {NULL}};
const Segment SGI327000[] = {{&S84567[20], 49}, {NULL}};
const Segment SGI327001[] = {{&S84568[4], 11}, {NULL}};
const Segment SGI327002[] = {{&S84568[20], 49}, {NULL}};
const Segment SGI327003[] = {{&S84569[4], 11}, {NULL}};
const Segment SGI327004[] = {{&S84569[20], 49}, {NULL}};
const Segment SGI327005[] = {{&S84570[4], 5}, {NULL}};
const Segment SGI327006[] = {{&S84570[21], 52}, {NULL}};
const Segment SGI327007[] = {{&S84571[4], 5}, {NULL}};
const Segment SGI327008[] = {{&S84571[21], 52}, {NULL}};
const Segment SGI327009[] = {{&S84572[4], 5}, {NULL}};
const Segment SGI327010[] = {{&S84572[21], 52}, {NULL}};
const Segment SGI327011[] = {{&S84573[4], 11}, {NULL}};
const Segment SGI327012[] = {{&S84573[20], 52}, {NULL}};
const Segment SGI327013[] = {{&S84574[4], 11}, {NULL}};
const Segment SGI327014[] = {{&S84574[20], 52}, {NULL}};
const Segment SGI327015[] = {{&S84575[4], 11}, {NULL}};
const Segment SGI327016[] = {{&S84575[20], 49}, {NULL}};
const Segment SGI327017[] = {{&S99478[3], 5}, {NULL}};
const Segment SGI327018[] = {{&S99478[105], 13}, {NULL}};
const Segment SGI327019[] = {{&S99524[3], 5}, {NULL}};
const Segment SGI327020[] = {{&S99544[3], 5}, {NULL}};
const Segment SGI327021[] = {{&S81369[18], 3}, {NULL}};
const Segment SGI327022[] = {{&S81369[8], 3}, {NULL}};
const Segment SGI327023[] = {{&S81369[13], 8}, {NULL}};
const Segment SGI327024[] = {{&S81369[30], 7}, {NULL}};
const Segment SGI327025[] = {{&S81369[23], 14}, {NULL}};
const Segment SGI327026[] = {{&S81369[59], 5}, {NULL}};
const Segment SGI327027[] = {{&S81369[39], 6}, {NULL}};
const Segment SGI327028[] = {{&S81370[18], 3}, {NULL}};
const Segment SGI327029[] = {{&S81370[8], 3}, {NULL}};
const Segment SGI327030[] = {{&S81370[13], 8}, {NULL}};
const Segment SGI327031[] = {{&S81370[30], 7}, {NULL}};
const Segment SGI327032[] = {{&S81370[23], 14}, {NULL}};
const Segment SGI327033[] = {{&S81370[64], 5}, {NULL}};
const Segment SGI327034[] = {{&S81370[39], 5}, {NULL}};
const Segment SGI327035[] = {{&S99598[3], 5}, {NULL}};
const Segment SGI327036[] = {{&S99598[78], 13}, {NULL}};
const Segment SGI327037[] = {{&S84771[4], 11}, {NULL}};
const Segment SGI327038[] = {{&S99653[3], 5}, {NULL}};
const Segment SGI327039[] = {{&S99653[77], 13}, {NULL}};
const Segment SGI327040[] = {{&S81503[30], 3}, {NULL}};
const Segment SGI327041[] = {{&S81503[8], 3}, {NULL}};
const Segment SGI327042[] = {{&S81503[13], 10}, {NULL}};
const Segment SGI327043[] = {{&S81591[30], 3}, {NULL}};
const Segment SGI327044[] = {{&S81591[8], 3}, {NULL}};
const Segment SGI327045[] = {{&S81591[13], 10}, {NULL}};
const Segment SGI327046[] = {{&S81591[42], 7}, {NULL}};
const Segment SGI327047[] = {{&S81591[35], 14}, {NULL}};
const Segment SGI327048[] = {{&S81591[151], 4}, {NULL}};
const Segment SGI327049[] = {{&S81592[30], 3}, {NULL}};
const Segment SGI327050[] = {{&S81592[8], 3}, {NULL}};
const Segment SGI327051[] = {{&S81592[13], 10}, {NULL}};
const Segment SGI327052[] = {{&S81592[42], 7}, {NULL}};
const Segment SGI327053[] = {{&S81592[35], 14}, {NULL}};
const Segment SGI327054[] = {{&S81592[131], 15}, {NULL}};
const Segment SGI327055[] = {{&S81592[172], 4}, {NULL}};
const Segment SGI327056[] = {{&S81593[30], 3}, {NULL}};
const Segment SGI327057[] = {{&S81593[8], 3}, {NULL}};
const Segment SGI327058[] = {{&S81593[13], 10}, {NULL}};
const Segment SGI327059[] = {{&S81593[42], 7}, {NULL}};
const Segment SGI327060[] = {{&S81593[35], 14}, {NULL}};
const Segment SGI327061[] = {{&S81593[138], 15}, {NULL}};
const Segment SGI327062[] = {{&S81593[51], 6}, {NULL}};
const Segment SGI327063[] = {{&S81593[189], 4}, {NULL}};
const Segment SGI327064[] = {{&S81594[30], 3}, {NULL}};
const Segment SGI327065[] = {{&S81594[8], 3}, {NULL}};
const Segment SGI327066[] = {{&S81594[13], 10}, {NULL}};
const Segment SGI327067[] = {{&S81594[42], 7}, {NULL}};
const Segment SGI327068[] = {{&S81594[35], 14}, {NULL}};
const Segment SGI327069[] = {{&S81594[131], 15}, {NULL}};
const Segment SGI327070[] = {{&S81594[162], 9}, {NULL}};
const Segment SGI327071[] = {{&S81594[187], 4}, {NULL}};
const Segment SGI327072[] = {{&S81595[30], 3}, {NULL}};
const Segment SGI327073[] = {{&S81595[8], 3}, {NULL}};
const Segment SGI327074[] = {{&S81595[13], 10}, {NULL}};
const Segment SGI327075[] = {{&S81595[42], 7}, {NULL}};
const Segment SGI327076[] = {{&S81595[35], 14}, {NULL}};
const Segment SGI327077[] = {{&S81595[51], 6}, {NULL}};
const Segment SGI327078[] = {{&S81595[81], 4}, {NULL}};
const Segment SGI327079[] = {{&S81596[30], 3}, {NULL}};
const Segment SGI327080[] = {{&S81596[8], 3}, {NULL}};
const Segment SGI327081[] = {{&S81596[13], 10}, {NULL}};
const Segment SGI327082[] = {{&S81596[42], 7}, {NULL}};
const Segment SGI327083[] = {{&S81596[35], 14}, {NULL}};
const Segment SGI327084[] = {{&S81596[51], 6}, {NULL}};
const Segment SGI327085[] = {{&S81596[174], 4}, {NULL}};
const Segment SGI327086[] = {{&S81737[30], 3}, {NULL}};
const Segment SGI327087[] = {{&S81737[8], 3}, {NULL}};
const Segment SGI327088[] = {{&S81737[13], 10}, {NULL}};
const Segment SGI327089[] = {{&S81737[42], 7}, {NULL}};
const Segment SGI327090[] = {{&S81737[35], 14}, {NULL}};
const Segment SGI327091[] = {{&S81737[74], 4}, {NULL}};
const Segment SGI327092[] = {{&S81738[30], 3}, {NULL}};
const Segment SGI327093[] = {{&S81738[8], 3}, {NULL}};
const Segment SGI327094[] = {{&S81738[13], 10}, {NULL}};
const Segment SGI327095[] = {{&S81738[42], 7}, {NULL}};
const Segment SGI327096[] = {{&S81738[35], 14}, {NULL}};
const Segment SGI327097[] = {{&S81738[51], 3}, {NULL}};
const Segment SGI327098[] = {{&S81738[66], 4}, {NULL}};
const Segment SGI327099[] = {{&S81739[30], 3}, {NULL}};
const Segment SGI327100[] = {{&S81739[8], 3}, {NULL}};
const Segment SGI327101[] = {{&S81739[13], 10}, {NULL}};
const Segment SGI327102[] = {{&S81739[42], 7}, {NULL}};
const Segment SGI327103[] = {{&S81739[61], 4}, {NULL}};
const Segment SGI327104[] = {{&S81867[30], 3}, {NULL}};
const Segment SGI327105[] = {{&S81867[8], 3}, {NULL}};
const Segment SGI327106[] = {{&S81867[13], 10}, {NULL}};
const Segment SGI327107[] = {{&S81867[42], 7}, {NULL}};
const Segment SGI327108[] = {{&S81867[35], 14}, {NULL}};
const Segment SGI327109[] = {{&S81867[64], 11}, {NULL}};
const Segment SGI327110[] = {{&S81867[64], 16}, {NULL}};
const Segment SGI327111[] = {{&S81870[30], 3}, {NULL}};
const Segment SGI327112[] = {{&S81870[8], 3}, {NULL}};
const Segment SGI327113[] = {{&S81870[13], 10}, {NULL}};
const Segment SGI327114[] = {{&S81870[42], 7}, {NULL}};
const Segment SGI327115[] = {{&S81870[35], 14}, {NULL}};
const Segment SGI327116[] = {{&S81870[51], 11}, {NULL}};
const Segment SGI327117[] = {{&S81870[105], 15}, {NULL}};
const Segment SGI327118[] = {{&S81870[51], 16}, {NULL}};
const Segment SGI327119[] = {{&S81871[30], 3}, {NULL}};
const Segment SGI327120[] = {{&S81871[8], 3}, {NULL}};
const Segment SGI327121[] = {{&S81871[13], 10}, {NULL}};
const Segment SGI327122[] = {{&S81871[42], 7}, {NULL}};
const Segment SGI327123[] = {{&S81871[35], 14}, {NULL}};
const Segment SGI327124[] = {{&S81871[64], 11}, {NULL}};
const Segment SGI327125[] = {{&S81871[64], 16}, {NULL}};
const Segment SGI327126[] = {{&S81872[30], 3}, {NULL}};
const Segment SGI327127[] = {{&S81872[8], 3}, {NULL}};
const Segment SGI327128[] = {{&S81872[13], 10}, {NULL}};
const Segment SGI327129[] = {{&S81872[42], 7}, {NULL}};
const Segment SGI327130[] = {{&S81872[35], 14}, {NULL}};
const Segment SGI327131[] = {{&S81872[71], 11}, {NULL}};
const Segment SGI327132[] = {{&S81872[51], 6}, {NULL}};
const Segment SGI327133[] = {{&S81872[71], 16}, {NULL}};
const Segment SGI327134[] = {{&S81874[30], 3}, {NULL}};
const Segment SGI327135[] = {{&S81874[8], 3}, {NULL}};
const Segment SGI327136[] = {{&S81874[13], 10}, {NULL}};
const Segment SGI327137[] = {{&S81874[42], 7}, {NULL}};
const Segment SGI327138[] = {{&S81874[35], 14}, {NULL}};
const Segment SGI327139[] = {{&S81874[59], 11}, {NULL}};
const Segment SGI327140[] = {{&S81874[59], 16}, {NULL}};
const Segment SGI327141[] = {{&S81875[30], 3}, {NULL}};
const Segment SGI327142[] = {{&S81875[8], 3}, {NULL}};
const Segment SGI327143[] = {{&S81875[13], 10}, {NULL}};
const Segment SGI327144[] = {{&S81875[42], 7}, {NULL}};
const Segment SGI327145[] = {{&S81875[35], 14}, {NULL}};
const Segment SGI327146[] = {{&S81875[64], 11}, {NULL}};
const Segment SGI327147[] = {{&S81875[118], 15}, {NULL}};
const Segment SGI327148[] = {{&S81875[64], 16}, {NULL}};
const Segment SGI327149[] = {{&S100110[3], 5}, {NULL}};
const Segment SGI327150[] = {{&S100122[3], 6}, {NULL}};
const Segment SGI327151[] = {{&S100125[3], 5}, {NULL}};
const Segment SGI327152[] = {{&S100143[3], 5}, {NULL}};
const Segment SGI327153[] = {{&S81964[30], 3}, {NULL}};
const Segment SGI327154[] = {{&S81964[8], 3}, {NULL}};
const Segment SGI327155[] = {{&S81964[13], 10}, {NULL}};
const Segment SGI327156[] = {{&S81964[42], 7}, {NULL}};
const Segment SGI327157[] = {{&S81964[35], 14}, {NULL}};
const Segment SGI327158[] = {{&S81964[80], 5}, {NULL}};
const Segment SGI327159[] = {{&S81964[51], 6}, {NULL}};
const Segment SGI327160[] = {{&S81967[30], 3}, {NULL}};
const Segment SGI327161[] = {{&S81967[8], 3}, {NULL}};
const Segment SGI327162[] = {{&S81967[13], 10}, {NULL}};
const Segment SGI327163[] = {{&S81967[42], 7}, {NULL}};
const Segment SGI327164[] = {{&S81967[35], 14}, {NULL}};
const Segment SGI327165[] = {{&S81967[80], 5}, {NULL}};
const Segment SGI327166[] = {{&S81967[51], 6}, {NULL}};
const Segment SGI327167[] = {{&S82000[30], 3}, {NULL}};
const Segment SGI327168[] = {{&S82000[8], 3}, {NULL}};
const Segment SGI327169[] = {{&S82000[13], 10}, {NULL}};
const Segment SGI327170[] = {{&S82000[35], 5}, {NULL}};
const Segment SGI327171[] = {{&S100211[3], 5}, {NULL}};
const Segment SGI327172[] = {{&S100211[77], 13}, {NULL}};
const Segment SGI327173[] = {{&S100236[3], 5}, {NULL}};
const Segment SGI327174[] = {{&S100236[58], 13}, {NULL}};
const Segment SGI327175[] = {{&S95140[0], 99}, {NULL}};
const Segment SGI327176[] = {{&S82241[30], 3}, {NULL}};
const Segment SGI327177[] = {{&S82241[122], 9}, {NULL}};
const Segment SGI327178[] = {{&S82241[8], 3}, {NULL}};
const Segment SGI327179[] = {{&S82241[13], 10}, {NULL}};
const Segment SGI327180[] = {{&S82241[42], 7}, {NULL}};
const Segment SGI327181[] = {{&S82241[35], 14}, {NULL}};
const Segment SGI327182[] = {{&S82241[51], 3}, {NULL}};
const Segment SGI327183[] = {{&S82241[89], 9}, {NULL}};
const Segment SGI327184[] = {{&S82242[30], 3}, {NULL}};
const Segment SGI327185[] = {{&S82242[122], 9}, {&S82242[122], 9}, {NULL}};
const Segment SGI327186[] = {{&S82242[8], 3}, {NULL}};
const Segment SGI327187[] = {{&S82242[13], 10}, {NULL}};
const Segment SGI327188[] = {{&S82242[42], 7}, {NULL}};
const Segment SGI327189[] = {{&S82242[35], 14}, {NULL}};
const Segment SGI327190[] = {{&S82242[51], 3}, {NULL}};
const Segment SGI327191[] = {{&S82242[89], 9}, {NULL}};
const Segment SGI327192[] = {{&S100322[3], 5}, {NULL}};
const Segment SGI327193[] = {{&S100322[146], 13}, {NULL}};
const Segment SGI327194[] = {{&S85764[4], 11}, {NULL}};
const Segment SGI327195[] = {{&S100464[3], 5}, {NULL}};
const Segment SGI327196[] = {{&S100464[123], 13}, {NULL}};
const Segment SGI327197[] = {{&S100582[3], 5}, {NULL}};
const Segment SGI327198[] = {{&S100582[124], 13}, {NULL}};
const Segment SGI327199[] = {{&S82489[30], 3}, {NULL}};
const Segment SGI327200[] = {{&S82489[8], 3}, {NULL}};
const Segment SGI327201[] = {{&S82489[13], 10}, {NULL}};
const Segment SGI327202[] = {{&S82489[42], 7}, {NULL}};
const Segment SGI327203[] = {{&S82489[35], 14}, {NULL}};
const Segment SGI327204[] = {{&S82489[71], 5}, {NULL}};
const Segment SGI327205[] = {{&S82489[51], 6}, {NULL}};
const Segment SGI327206[] = {{&S82489[98], 9}, {NULL}};
const Segment SGI327207[] = {{&S100591[3], 5}, {NULL}};
const Segment SGI327208[] = {{&S100591[124], 13}, {NULL}};
const Segment SGI327209[] = {{&S82556[30], 3}, {NULL}};
const Segment SGI327210[] = {{&S82556[8], 3}, {NULL}};
const Segment SGI327211[] = {{&S82556[13], 10}, {NULL}};
const Segment SGI327212[] = {{&S82556[42], 7}, {NULL}};
const Segment SGI327213[] = {{&S82556[35], 14}, {NULL}};
const Segment SGI327214[] = {{&S82556[58], 10}, {NULL}};
const Segment SGI327215[] = {{&S95646[0], 43}, {NULL}};
const Segment SGI327216[] = {{&S82660[30], 4}, {NULL}};
const Segment SGI327217[] = {{&S82660[8], 3}, {NULL}};
const Segment SGI327218[] = {{&S82660[13], 10}, {NULL}};
const Segment SGI327219[] = {{&S82660[43], 7}, {NULL}};
const Segment SGI327220[] = {{&S82660[36], 14}, {NULL}};
const Segment SGI327221[] = {{&S82660[72], 5}, {NULL}};
const Segment SGI327222[] = {{&S82660[52], 6}, {NULL}};
const Segment SGI327223[] = {{&S82661[30], 4}, {NULL}};
const Segment SGI327224[] = {{&S82661[8], 3}, {NULL}};
const Segment SGI327225[] = {{&S82661[13], 10}, {NULL}};
const Segment SGI327226[] = {{&S82661[43], 7}, {NULL}};
const Segment SGI327227[] = {{&S82661[36], 14}, {NULL}};
const Segment SGI327228[] = {{&S82661[77], 5}, {NULL}};
const Segment SGI327229[] = {{&S82661[52], 5}, {NULL}};
const Segment SGI327230[] = {{&S95782[0], 42}, {NULL}};
const Segment SGI327231[] = {{&S86206[4], 2}, {NULL}};
const Segment SGI327232[] = {{&S86207[4], 17}, {NULL}};
const Segment SGI327233[] = {{&S86208[4], 12}, {NULL}};
const Segment SGI327234[] = {{&S100828[3], 5}, {NULL}};
const Segment SGI327235[] = {{&S100828[58], 13}, {NULL}};
const Segment SGI327236[] = {{&S100832[3], 5}, {NULL}};
const Segment SGI327237[] = {{&S100832[77], 13}, {NULL}};
const Segment SGI327238[] = {{&S100882[3], 5}, {NULL}};
const Segment SGI327239[] = {{&S100882[107], 12}, {NULL}};
const Segment SGI327240[] = {{&S95958[0], 80}, {NULL}};
const Segment SGI327241[] = {{&S95966[0], 34}, {NULL}};
const Segment SGI327242[] = {{&S100948[3], 5}, {NULL}};
const Segment SGI327243[] = {{&S100949[3], 6}, {NULL}};
const Segment SGI327244[] = {{&S100950[3], 5}, {NULL}};
const Segment SGI327245[] = {{&S100953[3], 5}, {NULL}};
const Segment SGI327246[] = {{&S100953[3], 12}, {NULL}};
const Segment SGI327247[] = {{&S100954[3], 5}, {NULL}};
const Segment SGI327248[] = {{&S100956[3], 5}, {NULL}};
const Segment SGI327249[] = {{&S100956[3], 10}, {NULL}};
const Segment SGI327250[] = {{&S100957[3], 5}, {NULL}};
const Segment SGI327251[] = {{&S100957[3], 10}, {NULL}};
const Segment SGI327252[] = {{&S100958[3], 5}, {NULL}};
const Segment SGI327253[] = {{&S100961[3], 6}, {NULL}};
const Segment SGI327254[] = {{&S100961[25], 3}, {NULL}};
const Segment SGI327255[] = {{&S100975[3], 5}, {NULL}};
const Segment SGI327256[] = {{&S82958[30], 3}, {NULL}};
const Segment SGI327257[] = {{&S82958[8], 3}, {NULL}};
const Segment SGI327258[] = {{&S82958[13], 10}, {NULL}};
const Segment SGI327259[] = {{&S82958[42], 7}, {NULL}};
const Segment SGI327260[] = {{&S82958[35], 14}, {NULL}};
const Segment SGI327261[] = {{&S82958[71], 11}, {NULL}};
const Segment SGI327262[] = {{&S82958[51], 6}, {NULL}};
const Segment SGI327263[] = {{&S82958[71], 16}, {NULL}};
const Segment SGI327264[] = {{&S82960[30], 3}, {NULL}};
const Segment SGI327265[] = {{&S82960[8], 3}, {NULL}};
const Segment SGI327266[] = {{&S82960[13], 10}, {NULL}};
const Segment SGI327267[] = {{&S82960[42], 7}, {NULL}};
const Segment SGI327268[] = {{&S82960[35], 14}, {NULL}};
const Segment SGI327269[] = {{&S82960[64], 11}, {NULL}};
const Segment SGI327270[] = {{&S82960[64], 16}, {NULL}};
const Segment SGI327271[] = {{&S82964[30], 3}, {NULL}};
const Segment SGI327272[] = {{&S82964[8], 3}, {NULL}};
const Segment SGI327273[] = {{&S82964[13], 10}, {NULL}};
const Segment SGI327274[] = {{&S82964[42], 7}, {NULL}};
const Segment SGI327275[] = {{&S82964[35], 14}, {NULL}};
const Segment SGI327276[] = {{&S82964[64], 11}, {NULL}};
const Segment SGI327277[] = {{&S82964[64], 16}, {NULL}};
const Segment SGI327278[] = {{&S83002[30], 3}, {NULL}};
const Segment SGI327279[] = {{&S83002[8], 3}, {NULL}};
const Segment SGI327280[] = {{&S83002[13], 10}, {NULL}};
const Segment SGI327281[] = {{&S83002[35], 5}, {NULL}};
const Segment SGI327282[] = {{&S101139[3], 5}, {NULL}};
const Segment SGI327283[] = {{&S101139[29], 4}, {NULL}};
const Segment SGI327284[] = {{&S101180[3], 4}, {NULL}};
const Segment SGI327285[] = {{&S83066[30], 3}, {NULL}};
const Segment SGI327286[] = {{&S83066[8], 3}, {NULL}};
const Segment SGI327287[] = {{&S83066[13], 10}, {NULL}};
const Segment SGI327288[] = {{&S83066[42], 7}, {NULL}};
const Segment SGI327289[] = {{&S83066[60], 5}, {NULL}};
const Segment SGI327290[] = {{&S83067[30], 3}, {NULL}};
const Segment SGI327291[] = {{&S83067[8], 3}, {NULL}};
const Segment SGI327292[] = {{&S83067[13], 10}, {NULL}};
const Segment SGI327293[] = {{&S83067[42], 7}, {NULL}};
const Segment SGI327294[] = {{&S83067[35], 14}, {NULL}};
const Segment SGI327295[] = {{&S83067[79], 5}, {NULL}};
const Segment SGI327296[] = {{&S83068[30], 3}, {NULL}};
const Segment SGI327297[] = {{&S83068[8], 3}, {NULL}};
const Segment SGI327298[] = {{&S83068[13], 10}, {NULL}};
const Segment SGI327299[] = {{&S83068[42], 7}, {NULL}};
const Segment SGI327300[] = {{&S83068[35], 14}, {NULL}};
const Segment SGI327301[] = {{&S83068[104], 5}, {NULL}};
const Segment SGI327302[] = {{&S83069[30], 3}, {NULL}};
const Segment SGI327303[] = {{&S83069[8], 3}, {NULL}};
const Segment SGI327304[] = {{&S83069[13], 10}, {NULL}};
const Segment SGI327305[] = {{&S83069[42], 7}, {NULL}};
const Segment SGI327306[] = {{&S83069[35], 14}, {NULL}};
const Segment SGI327307[] = {{&S83069[79], 5}, {NULL}};
const Segment SGI327308[] = {{&S83101[30], 3}, {NULL}};
const Segment SGI327309[] = {{&S83101[8], 3}, {NULL}};
const Segment SGI327310[] = {{&S83101[13], 10}, {NULL}};
const Segment SGI327311[] = {{&S83101[35], 6}, {NULL}};
const Segment SGI327312[] = {{&S83131[30], 3}, {NULL}};
const Segment SGI327313[] = {{&S83131[8], 3}, {NULL}};
const Segment SGI327314[] = {{&S83131[13], 10}, {NULL}};
const Segment SGI327315[] = {{&S83131[42], 7}, {NULL}};
const Segment SGI327316[] = {{&S83131[35], 14}, {NULL}};
const Segment SGI327317[] = {{&S83131[131], 15}, {NULL}};
const Segment SGI327318[] = {{&S83131[152], 9}, {NULL}};
const Segment SGI327319[] = {{&S83131[177], 4}, {NULL}};
const Segment SGI327320[] = {{&S83132[30], 3}, {NULL}};
const Segment SGI327321[] = {{&S83132[8], 3}, {NULL}};
const Segment SGI327322[] = {{&S83132[13], 10}, {NULL}};
const Segment SGI327323[] = {{&S83132[42], 7}, {NULL}};
const Segment SGI327324[] = {{&S83132[35], 14}, {NULL}};
const Segment SGI327325[] = {{&S83132[138], 15}, {NULL}};
const Segment SGI327326[] = {{&S83132[51], 6}, {NULL}};
const Segment SGI327327[] = {{&S83132[179], 4}, {NULL}};
const Segment SGI327328[] = {{&S83133[30], 3}, {NULL}};
const Segment SGI327329[] = {{&S83133[8], 3}, {NULL}};
const Segment SGI327330[] = {{&S83133[13], 10}, {NULL}};
const Segment SGI327331[] = {{&S83133[42], 7}, {NULL}};
const Segment SGI327332[] = {{&S83133[35], 14}, {NULL}};
const Segment SGI327333[] = {{&S83133[131], 15}, {NULL}};
const Segment SGI327334[] = {{&S83133[162], 4}, {NULL}};
const Segment SGI327335[] = {{&S83134[30], 3}, {NULL}};
const Segment SGI327336[] = {{&S83134[8], 3}, {NULL}};
const Segment SGI327337[] = {{&S83134[13], 10}, {NULL}};
const Segment SGI327338[] = {{&S83134[42], 7}, {NULL}};
const Segment SGI327339[] = {{&S83134[35], 14}, {NULL}};
const Segment SGI327340[] = {{&S83134[167], 4}, {NULL}};
const Segment SGI327341[] = {{&S83143[30], 3}, {NULL}};
const Segment SGI327342[] = {{&S83143[8], 3}, {NULL}};
const Segment SGI327343[] = {{&S83143[13], 10}, {NULL}};
const Segment SGI327344[] = {{&S83143[42], 3}, {NULL}};
const Segment SGI327345[] = {{&S101297[3], 5}, {NULL}};
const Segment SGI327346[] = {{&S101299[3], 5}, {NULL}};
const Segment SGI327347[] = {{&S83182[30], 3}, {NULL}};
const Segment SGI327348[] = {{&S83182[8], 3}, {NULL}};
const Segment SGI327349[] = {{&S83182[13], 10}, {NULL}};
const Segment SGI327350[] = {{&S83182[42], 7}, {NULL}};
const Segment SGI327351[] = {{&S83182[35], 14}, {NULL}};
const Segment SGI327352[] = {{&S101323[3], 10}, {NULL}};
const Segment SGI327353[] = {{&S101323[47], 4}, {NULL}};
const Segment SGI327354[] = {{&S101325[3], 5}, {NULL}};
const Segment SGI327355[] = {{&S86776[4], 5}, {NULL}};
const Segment SGI327356[] = {{&S86776[27], 11}, {NULL}};
const Segment SGI327357[] = {{&S86776[39], 5}, {NULL}};
const Segment SGI327358[] = {{&S101377[3], 4}, {NULL}};
const Segment SGI327359[] = {{&S86777[4], 5}, {NULL}};
const Segment SGI327360[] = {{&S86777[27], 11}, {NULL}};
const Segment SGI327361[] = {{&S86777[39], 5}, {NULL}};
const Segment SGI327362[] = {{&S86778[4], 5}, {NULL}};
const Segment SGI327363[] = {{&S86778[27], 11}, {NULL}};
const Segment SGI327364[] = {{&S86778[39], 5}, {NULL}};
const Segment SGI327365[] = {{&S101439[3], 5}, {NULL}};
const Segment SGI327366[] = {{&S101442[3], 5}, {NULL}};
const Segment SGI327367[] = {{&S101448[3], 5}, {NULL}};
const Segment SGI327368[] = {{&S101451[3], 5}, {NULL}};
const Segment SGI327369[] = {{&S101453[3], 5}, {NULL}};
const Segment SGI327370[] = {{&S101471[3], 5}, {NULL}};
const Segment SGI327371[] = {{&S101472[3], 5}, {NULL}};
const Segment SGI327372[] = {{&S96504[0], 60}, {NULL}};
const Segment SGI327373[] = {{&S96505[0], 30}, {NULL}};
const Segment SGI327374[] = {{&S101639[3], 5}, {NULL}};
const Segment SGI327375[] = {{&S101639[3], 12}, {NULL}};
const Segment SGI327376[] = {{&S101639[35], 9}, {NULL}};
const Segment SGI327377[] = {{&S101646[3], 5}, {NULL}};
const Segment SGI327378[] = {{&S101653[3], 5}, {NULL}};
const Segment SGI327379[] = {{&S101653[24], 4}, {NULL}};
const Segment SGI327380[] = {{&S83534[30], 3}, {NULL}};
const Segment SGI327381[] = {{&S83534[8], 3}, {NULL}};
const Segment SGI327382[] = {{&S83534[13], 10}, {NULL}};
const Segment SGI327383[] = {{&S83534[42], 7}, {NULL}};
const Segment SGI327384[] = {{&S83534[35], 14}, {NULL}};
const Segment SGI327385[] = {{&S83669[30], 3}, {NULL}};
const Segment SGI327386[] = {{&S83669[8], 3}, {NULL}};
const Segment SGI327387[] = {{&S83669[13], 10}, {NULL}};
const Segment SGI327388[] = {{&S83669[55], 7}, {NULL}};
const Segment SGI327389[] = {{&S83669[48], 14}, {NULL}};
const Segment SGI327390[] = {{&S83669[64], 13}, {NULL}};
const Segment SGI327391[] = {{&S83670[30], 3}, {NULL}};
const Segment SGI327392[] = {{&S83670[8], 3}, {NULL}};
const Segment SGI327393[] = {{&S83670[13], 10}, {NULL}};
const Segment SGI327394[] = {{&S83670[55], 7}, {NULL}};
const Segment SGI327395[] = {{&S83670[48], 14}, {NULL}};
const Segment SGI327396[] = {{&S83670[64], 13}, {NULL}};
const Segment SGI327397[] = {{&S83671[30], 3}, {NULL}};
const Segment SGI327398[] = {{&S83671[8], 3}, {NULL}};
const Segment SGI327399[] = {{&S83671[13], 10}, {NULL}};
const Segment SGI327400[] = {{&S83671[42], 7}, {NULL}};
const Segment SGI327401[] = {{&S83671[35], 14}, {NULL}};
const Segment SGI327402[] = {{&S83671[128], 15}, {NULL}};
const Segment SGI327403[] = {{&S83671[51], 13}, {NULL}};
const Segment SGI327404[] = {{&S101801[3], 6}, {NULL}};
const Segment SGI327405[] = {{&S101801[24], 9}, {NULL}};
const Segment SGI327406[] = {{&S101807[3], 5}, {NULL}};
const Segment SGI327407[] = {{&S101822[3], 5}, {NULL}};
const Segment SGI327408[] = {{&S101870[3], 4}, {NULL}};
const Segment SGI327409[] = {{&S101991[3], 5}, {NULL}};
const Segment SGI327410[] = {{&S102014[3], 6}, {NULL}};
const Segment SGI327411[] = {{&S102015[3], 5}, {NULL}};
const Segment SGI327412[] = {{&S102015[3], 14}, {NULL}};
const Segment SGI327413[] = {{&S87401[4], 5}, {NULL}};
const Segment SGI327414[] = {{&S87402[4], 4}, {NULL}};
const Segment SGI327415[] = {{&S102170[3], 5}, {NULL}};
const Segment SGI327416[] = {{&S102170[28], 9}, {NULL}};
const Segment SGI327417[] = {{&S102180[3], 5}, {NULL}};
const Segment SGI327418[] = {{&S102188[3], 6}, {NULL}};
const Segment SGI327419[] = {{&S102189[3], 6}, {NULL}};
const Segment SGI327420[] = {{&S102190[3], 5}, {NULL}};
const Segment SGI327421[] = {{&S87527[4], 2}, {NULL}};
const Segment SGI327422[] = {{&S87527[37], 39}, {NULL}};
const Segment SGI327423[] = {{&S102340[3], 5}, {NULL}};
const Segment SGI327424[] = {{&S102340[20], 3}, {NULL}};
const Segment SGI327425[] = {{&S102347[3], 5}, {NULL}};
const Segment SGI327426[] = {{&S102348[3], 5}, {NULL}};
const Segment SGI327427[] = {{&S102348[26], 3}, {NULL}};
const Segment SGI327428[] = {{&S102356[3], 5}, {NULL}};
const Segment SGI327429[] = {{&S102378[3], 5}, {NULL}};
const Segment SGI327430[] = {{&S102383[3], 5}, {NULL}};
const Segment SGI327431[] = {{&S102389[3], 5}, {NULL}};
const Segment SGI327432[] = {{&S102391[3], 5}, {NULL}};
const Segment SGI327433[] = {{&S102407[3], 5}, {NULL}};
const Segment SGI327434[] = {{&S84219[30], 3}, {NULL}};
const Segment SGI327435[] = {{&S84219[8], 3}, {NULL}};
const Segment SGI327436[] = {{&S84219[13], 10}, {NULL}};
const Segment SGI327437[] = {{&S84219[42], 7}, {NULL}};
const Segment SGI327438[] = {{&S84219[35], 14}, {NULL}};
const Segment SGI327439[] = {{&S87785[4], 11}, {NULL}};
const Segment SGI327440[] = {{&S91865[4], 6}, {NULL}};
const Segment SGI327441[] = {{&S91865[26], 3}, {NULL}};
const Segment SGI327442[] = {{&S102575[3], 5}, {NULL}};
const Segment SGI327443[] = {{&S102575[28], 9}, {NULL}};
const Segment SGI327444[] = {{&S102579[3], 6}, {NULL}};
const Segment SGI327445[] = {{&S102579[31], 3}, {NULL}};
const Segment SGI327446[] = {{&S92074[4], 5}, {NULL}};
const Segment SGI327447[] = {{&S92074[25], 3}, {NULL}};
const Segment SGI327448[] = {{&S102749[3], 5}, {NULL}};
const Segment SGI327449[] = {{&S102751[3], 5}, {NULL}};
const Segment SGI327450[] = {{&S102755[3], 5}, {NULL}};
const Segment SGI327451[] = {{&S102764[3], 5}, {NULL}};
const Segment SGI327452[] = {{&S102765[3], 6}, {NULL}};
const Segment SGI327453[] = {{&S84508[30], 3}, {NULL}};
const Segment SGI327454[] = {{&S102775[3], 4}, {NULL}};
const Segment SGI327455[] = {{&S84508[8], 3}, {NULL}};
const Segment SGI327456[] = {{&S84508[13], 10}, {NULL}};
const Segment SGI327457[] = {{&S84508[42], 7}, {NULL}};
const Segment SGI327458[] = {{&S84508[35], 14}, {NULL}};
const Segment SGI327459[] = {{&S84508[79], 5}, {NULL}};
const Segment SGI327460[] = {{&S88176[4], 2}, {NULL}};
const Segment SGI327461[] = {{&S88176[45], 19}, {&S88176[85], 39}, {NULL}};
const Segment SGI327462[] = {{&S84509[30], 3}, {NULL}};
const Segment SGI327463[] = {{&S84509[8], 3}, {NULL}};
const Segment SGI327464[] = {{&S84509[13], 10}, {NULL}};
const Segment SGI327465[] = {{&S84509[42], 7}, {NULL}};
const Segment SGI327466[] = {{&S84509[35], 14}, {NULL}};
const Segment SGI327467[] = {{&S84509[89], 15}, {NULL}};
const Segment SGI327468[] = {{&S84509[162], 5}, {NULL}};
const Segment SGI327469[] = {{&S84510[30], 3}, {NULL}};
const Segment SGI327470[] = {{&S84510[8], 3}, {NULL}};
const Segment SGI327471[] = {{&S84510[13], 10}, {NULL}};
const Segment SGI327472[] = {{&S84510[42], 7}, {NULL}};
const Segment SGI327473[] = {{&S84510[35], 14}, {NULL}};
const Segment SGI327474[] = {{&S84510[99], 5}, {NULL}};
const Segment SGI327475[] = {{&S102812[3], 6}, {NULL}};
const Segment SGI327476[] = {{&S84543[8], 3}, {NULL}};
const Segment SGI327477[] = {{&S84543[13], 10}, {NULL}};
const Segment SGI327478[] = {{&S84543[42], 7}, {NULL}};
const Segment SGI327479[] = {{&S84543[35], 14}, {NULL}};
const Segment SGI327480[] = {{&S97624[0], 86}, {NULL}};
const Segment SGI327481[] = {{&S97625[0], 86}, {NULL}};
const Segment SGI327482[] = {{&S102966[3], 5}, {NULL}};
const Segment SGI327483[] = {{&S102980[3], 5}, {NULL}};
const Segment SGI327484[] = {{&S88404[4], 4}, {NULL}};
const Segment SGI327485[] = {{&S88404[27], 8}, {NULL}};
const Segment SGI327486[] = {{&S88405[4], 5}, {NULL}};
const Segment SGI327487[] = {{&S88405[27], 11}, {NULL}};
const Segment SGI327488[] = {{&S88405[39], 5}, {NULL}};
const Segment SGI327489[] = {{&S103019[3], 5}, {NULL}};
const Segment SGI327490[] = {{&S103028[3], 5}, {NULL}};
const Segment SGI327491[] = {{&S103039[3], 5}, {NULL}};
const Segment SGI327492[] = {{&S103054[3], 5}, {NULL}};
const Segment SGI327493[] = {{&S103054[24], 3}, {NULL}};
const Segment SGI327494[] = {{&S84586[30], 3}, {NULL}};
const Segment SGI327495[] = {{&S84586[8], 3}, {NULL}};
const Segment SGI327496[] = {{&S84586[13], 10}, {NULL}};
const Segment SGI327497[] = {{&S84586[42], 7}, {NULL}};
const Segment SGI327498[] = {{&S84586[35], 14}, {NULL}};
const Segment SGI327499[] = {{&S84586[64], 11}, {NULL}};
const Segment SGI327500[] = {{&S84586[64], 15}, {NULL}};
const Segment SGI327501[] = {{&S84587[30], 3}, {NULL}};
const Segment SGI327502[] = {{&S84587[8], 3}, {NULL}};
const Segment SGI327503[] = {{&S84587[13], 10}, {NULL}};
const Segment SGI327504[] = {{&S84587[42], 7}, {NULL}};
const Segment SGI327505[] = {{&S84587[35], 14}, {NULL}};
const Segment SGI327506[] = {{&S84587[165], 11}, {NULL}};
const Segment SGI327507[] = {{&S84587[131], 15}, {NULL}};
const Segment SGI327508[] = {{&S84587[165], 15}, {NULL}};
const Segment SGI327509[] = {{&S84588[30], 3}, {NULL}};
const Segment SGI327510[] = {{&S84588[8], 3}, {NULL}};
const Segment SGI327511[] = {{&S84588[13], 10}, {NULL}};
const Segment SGI327512[] = {{&S84588[42], 7}, {NULL}};
const Segment SGI327513[] = {{&S84588[35], 14}, {NULL}};
const Segment SGI327514[] = {{&S84588[165], 11}, {NULL}};
const Segment SGI327515[] = {{&S84588[131], 15}, {NULL}};
const Segment SGI327516[] = {{&S84588[165], 15}, {NULL}};
const Segment SGI327517[] = {{&S97874[0], 85}, {NULL}};
const Segment SGI327518[] = {{&S103249[3], 6}, {NULL}};
const Segment SGI327519[] = {{&S103266[3], 5}, {NULL}};
const Segment SGI327520[] = {{&S103272[3], 5}, {NULL}};
const Segment SGI327521[] = {{&S103274[3], 5}, {NULL}};
const Segment SGI327522[] = {{&S103286[3], 6}, {NULL}};
const Segment SGI327523[] = {{&S84704[30], 3}, {NULL}};
const Segment SGI327524[] = {{&S84704[8], 3}, {NULL}};
const Segment SGI327525[] = {{&S84704[13], 10}, {NULL}};
const Segment SGI327526[] = {{&S84704[42], 7}, {NULL}};
const Segment SGI327527[] = {{&S84704[35], 14}, {NULL}};
const Segment SGI327528[] = {{&S103287[3], 5}, {NULL}};
const Segment SGI327529[] = {{&S84704[64], 11}, {NULL}};
const Segment SGI327530[] = {{&S84704[64], 15}, {NULL}};
const Segment SGI327531[] = {{&S84705[30], 3}, {NULL}};
const Segment SGI327532[] = {{&S84705[8], 3}, {NULL}};
const Segment SGI327533[] = {{&S84705[13], 10}, {NULL}};
const Segment SGI327534[] = {{&S84705[42], 7}, {NULL}};
const Segment SGI327535[] = {{&S84705[35], 14}, {NULL}};
const Segment SGI327536[] = {{&S84705[203], 11}, {NULL}};
const Segment SGI327537[] = {{&S84705[91], 15}, {NULL}};
const Segment SGI327538[] = {{&S84705[203], 15}, {NULL}};
const Segment SGI327539[] = {{&S103306[3], 5}, {NULL}};
const Segment SGI327540[] = {{&S103535[3], 5}, {NULL}};
const Segment SGI327541[] = {{&S103536[3], 5}, {NULL}};
const Segment SGI327542[] = {{&S103539[3], 5}, {NULL}};
const Segment SGI327543[] = {{&S103552[3], 5}, {NULL}};
const Segment SGI327544[] = {{&S103554[3], 5}, {NULL}};
const Segment SGI327545[] = {{&S103561[3], 5}, {NULL}};
const Segment SGI327546[] = {{&S103597[3], 5}, {NULL}};
const Segment SGI327547[] = {{&S103614[3], 5}, {NULL}};
const Segment SGI327548[] = {{&S103647[3], 6}, {NULL}};
const Segment SGI327549[] = {{&S85110[30], 4}, {NULL}};
const Segment SGI327550[] = {{&S85110[8], 3}, {NULL}};
const Segment SGI327551[] = {{&S85110[13], 10}, {NULL}};
const Segment SGI327552[] = {{&S85110[43], 7}, {NULL}};
const Segment SGI327553[] = {{&S85110[36], 14}, {NULL}};
const Segment SGI327554[] = {{&S103835[3], 5}, {NULL}};
const Segment SGI327555[] = {{&S88957[4], 2}, {NULL}};
const Segment SGI327556[] = {{&S88957[36], 32}, {NULL}};
const Segment SGI327557[] = {{&S88958[4], 3}, {NULL}};
const Segment SGI327558[] = {{&S88958[38], 32}, {NULL}};
const Segment SGI327559[] = {{&S88959[4], 2}, {NULL}};
const Segment SGI327560[] = {{&S88959[36], 32}, {NULL}};
const Segment SGI327561[] = {{&S88960[4], 2}, {NULL}};
const Segment SGI327562[] = {{&S88960[36], 32}, {NULL}};
const Segment SGI327563[] = {{&S88961[4], 2}, {NULL}};
const Segment SGI327564[] = {{&S88961[36], 32}, {NULL}};
const Segment SGI327565[] = {{&S88962[4], 2}, {NULL}};
const Segment SGI327566[] = {{&S88962[36], 32}, {NULL}};
const Segment SGI327567[] = {{&S103842[3], 5}, {NULL}};
const Segment SGI327568[] = {{&S88963[4], 3}, {NULL}};
const Segment SGI327569[] = {{&S88963[38], 32}, {NULL}};
const Segment SGI327570[] = {{&S88964[4], 2}, {NULL}};
const Segment SGI327571[] = {{&S88964[36], 25}, {NULL}};
const Segment SGI327572[] = {{&S103848[3], 4}, {NULL}};
const Segment SGI327573[] = {{&S103850[3], 5}, {NULL}};
const Segment SGI327574[] = {{&S88971[4], 3}, {NULL}};
const Segment SGI327575[] = {{&S88971[25], 11}, {NULL}};
const Segment SGI327576[] = {{&S88971[37], 5}, {NULL}};
const Segment SGI327577[] = {{&S88972[4], 3}, {NULL}};
const Segment SGI327578[] = {{&S88972[25], 11}, {NULL}};
const Segment SGI327579[] = {{&S88972[37], 5}, {NULL}};
const Segment SGI327580[] = {{&S103854[3], 5}, {NULL}};
const Segment SGI327581[] = {{&S85249[30], 3}, {NULL}};
const Segment SGI327582[] = {{&S85249[8], 3}, {NULL}};
const Segment SGI327583[] = {{&S85249[13], 10}, {NULL}};
const Segment SGI327584[] = {{&S85249[42], 7}, {NULL}};
const Segment SGI327585[] = {{&S85249[35], 14}, {NULL}};
const Segment SGI327586[] = {{&S85249[127], 11}, {NULL}};
const Segment SGI327587[] = {{&S85249[127], 15}, {NULL}};
const Segment SGI327588[] = {{&S85250[30], 3}, {NULL}};
const Segment SGI327589[] = {{&S85250[8], 3}, {NULL}};
const Segment SGI327590[] = {{&S85250[13], 10}, {NULL}};
const Segment SGI327591[] = {{&S85250[42], 7}, {NULL}};
const Segment SGI327592[] = {{&S85250[35], 14}, {NULL}};
const Segment SGI327593[] = {{&S85250[167], 11}, {NULL}};
const Segment SGI327594[] = {{&S85250[167], 15}, {NULL}};
const Segment SGI327595[] = {{&S103874[3], 5}, {NULL}};
const Segment SGI327596[] = {{&S103909[3], 5}, {NULL}};
const Segment SGI327597[] = {{&S103909[3], 10}, {NULL}};
const Segment SGI327598[] = {{&S103931[3], 6}, {NULL}};
const Segment SGI327599[] = {{&S103935[3], 5}, {NULL}};
const Segment SGI327600[] = {{&S103947[3], 5}, {NULL}};
const Segment SGI327601[] = {{&S89107[4], 4}, {NULL}};
const Segment SGI327602[] = {{&S89107[27], 8}, {NULL}};
const Segment SGI327603[] = {{&S89107[40], 5}, {NULL}};
const Segment SGI327604[] = {{&S93271[4], 5}, {NULL}};
const Segment SGI327605[] = {{&S93271[25], 3}, {NULL}};
const Segment SGI327606[] = {{&S104190[3], 5}, {NULL}};
const Segment SGI327607[] = {{&S104211[3], 5}, {NULL}};
const Segment SGI327608[] = {{&S104211[20], 3}, {NULL}};
const Segment SGI327609[] = {{&S104214[3], 5}, {NULL}};
const Segment SGI327610[] = {{&S104241[3], 5}, {NULL}};
const Segment SGI327611[] = {{&S104247[3], 5}, {NULL}};
const Segment SGI327612[] = {{&S104283[3], 5}, {NULL}};
const Segment SGI327613[] = {{&S104284[3], 5}, {NULL}};
const Segment SGI327614[] = {{&S93530[4], 6}, {NULL}};
const Segment SGI327615[] = {{&S93530[26], 3}, {NULL}};
const Segment SGI327616[] = {{&S104298[3], 5}, {NULL}};
const Segment SGI327617[] = {{&S104298[20], 3}, {NULL}};
const Segment SGI327618[] = {{&S104299[3], 5}, {NULL}};
const Segment SGI327619[] = {{&S104300[3], 5}, {NULL}};
const Segment SGI327620[] = {{&S104309[3], 5}, {NULL}};
const Segment SGI327621[] = {{&S104317[3], 5}, {NULL}};
const Segment SGI327622[] = {{&S104317[20], 3}, {NULL}};
const Segment SGI327623[] = {{&S99009[0], 118}, {NULL}};
const Segment SGI327624[] = {{&S99010[0], 118}, {NULL}};
const Segment SGI327625[] = {{&S89383[4], 3}, {NULL}};
const Segment SGI327626[] = {{&S89384[4], 2}, {NULL}};
const Segment SGI327627[] = {{&S85772[30], 3}, {NULL}};
const Segment SGI327628[] = {{&S85772[8], 3}, {NULL}};
const Segment SGI327629[] = {{&S85772[13], 10}, {NULL}};
const Segment SGI327630[] = {{&S85772[42], 7}, {NULL}};
const Segment SGI327631[] = {{&S85772[35], 14}, {NULL}};
const Segment SGI327632[] = {{&S85772[51], 11}, {NULL}};
const Segment SGI327633[] = {{&S85772[104], 15}, {NULL}};
const Segment SGI327634[] = {{&S85772[51], 16}, {NULL}};
const Segment SGI327635[] = {{&S85773[30], 3}, {NULL}};
const Segment SGI327636[] = {{&S85773[8], 3}, {NULL}};
const Segment SGI327637[] = {{&S85773[13], 10}, {NULL}};
const Segment SGI327638[] = {{&S85773[42], 7}, {NULL}};
const Segment SGI327639[] = {{&S85773[35], 14}, {NULL}};
const Segment SGI327640[] = {{&S85773[64], 11}, {NULL}};
const Segment SGI327641[] = {{&S85773[64], 16}, {NULL}};
const Segment SGI327642[] = {{&S85774[30], 3}, {NULL}};
const Segment SGI327643[] = {{&S85774[8], 3}, {NULL}};
const Segment SGI327644[] = {{&S85774[13], 10}, {NULL}};
const Segment SGI327645[] = {{&S85774[42], 7}, {NULL}};
const Segment SGI327646[] = {{&S85774[35], 14}, {NULL}};
const Segment SGI327647[] = {{&S85774[51], 11}, {NULL}};
const Segment SGI327648[] = {{&S85774[51], 16}, {NULL}};
const Segment SGI327649[] = {{&S85775[30], 3}, {NULL}};
const Segment SGI327650[] = {{&S85775[8], 3}, {NULL}};
const Segment SGI327651[] = {{&S85775[13], 10}, {NULL}};
const Segment SGI327652[] = {{&S85775[42], 7}, {NULL}};
const Segment SGI327653[] = {{&S85775[35], 14}, {NULL}};
const Segment SGI327654[] = {{&S85775[64], 11}, {NULL}};
const Segment SGI327655[] = {{&S85775[125], 15}, {NULL}};
const Segment SGI327656[] = {{&S85775[64], 16}, {NULL}};
const Segment SGI327657[] = {{&S85776[30], 3}, {NULL}};
const Segment SGI327658[] = {{&S85776[8], 3}, {NULL}};
const Segment SGI327659[] = {{&S85776[13], 10}, {NULL}};
const Segment SGI327660[] = {{&S85776[42], 7}, {NULL}};
const Segment SGI327661[] = {{&S85776[35], 14}, {NULL}};
const Segment SGI327662[] = {{&S85776[64], 11}, {NULL}};
const Segment SGI327663[] = {{&S85776[64], 16}, {NULL}};
const Segment SGI327664[] = {{&S85777[30], 3}, {NULL}};
const Segment SGI327665[] = {{&S85777[8], 3}, {NULL}};
const Segment SGI327666[] = {{&S85777[13], 10}, {NULL}};
const Segment SGI327667[] = {{&S85777[42], 7}, {NULL}};
const Segment SGI327668[] = {{&S85777[35], 14}, {NULL}};
const Segment SGI327669[] = {{&S85777[71], 11}, {NULL}};
const Segment SGI327670[] = {{&S85777[51], 6}, {NULL}};
const Segment SGI327671[] = {{&S85777[71], 16}, {NULL}};
const Segment SGI327672[] = {{&S89555[4], 4}, {NULL}};
const Segment SGI327673[] = {{&S89555[41], 39}, {&S89555[99], 19}, {NULL}};
const Segment SGI327674[] = {{&S89561[4], 4}, {NULL}};
const Segment SGI327675[] = {{&S104614[3], 5}, {NULL}};
const Segment SGI327676[] = {{&S104623[3], 5}, {NULL}};
const Segment SGI327677[] = {{&S104624[3], 5}, {NULL}};
const Segment SGI327678[] = {{&S104624[24], 3}, {NULL}};
const Segment SGI327679[] = {{&S104627[3], 5}, {NULL}};
const Segment SGI327680[] = {{&S104628[3], 6}, {NULL}};
const Segment SGI327681[] = {{&S86051[30], 3}, {NULL}};
const Segment SGI327682[] = {{&S86051[8], 3}, {NULL}};
const Segment SGI327683[] = {{&S86051[13], 10}, {NULL}};
const Segment SGI327684[] = {{&S86051[42], 7}, {NULL}};
const Segment SGI327685[] = {{&S86051[35], 14}, {NULL}};
const Segment SGI327686[] = {{&S86051[64], 4}, {NULL}};
const Segment SGI327687[] = {{&S89860[4], 12}, {NULL}};
const Segment SGI327688[] = {{&S104667[3], 5}, {NULL}};
const Segment SGI327689[] = {{&S104682[3], 5}, {NULL}};
const Segment SGI327690[] = {{&S104682[3], 9}, {NULL}};
const Segment SGI327691[] = {{&S99560[0], 80}, {NULL}};
const Segment SGI327692[] = {{&S104736[3], 5}, {NULL}};
const Segment SGI327693[] = {{&S104736[20], 3}, {NULL}};
const Segment SGI327694[] = {{&S104737[3], 5}, {NULL}};
const Segment SGI327695[] = {{&S104738[3], 5}, {NULL}};
const Segment SGI327696[] = {{&S104758[3], 5}, {NULL}};
const Segment SGI327697[] = {{&S86276[30], 3}, {NULL}};
const Segment SGI327698[] = {{&S86276[8], 3}, {NULL}};
const Segment SGI327699[] = {{&S86276[13], 10}, {NULL}};
const Segment SGI327700[] = {{&S90170[4], 4}, {NULL}};
const Segment SGI327701[] = {{&S90170[27], 8}, {NULL}};
const Segment SGI327702[] = {{&S90170[40], 5}, {NULL}};
const Segment SGI327703[] = {{&S99985[0], 43}, {NULL}};
const Segment SGI327704[] = {{&S105141[3], 6}, {NULL}};
const Segment SGI327705[] = {{&S105143[3], 5}, {NULL}};
const Segment SGI327706[] = {{&S105143[20], 3}, {NULL}};
const Segment SGI327707[] = {{&S105149[3], 5}, {NULL}};
const Segment SGI327708[] = {{&S105185[3], 5}, {NULL}};
const Segment SGI327709[] = {{&S94636[4], 5}, {NULL}};
const Segment SGI327710[] = {{&S94636[25], 3}, {NULL}};
const Segment SGI327711[] = {{&S90359[4], 4}, {NULL}};
const Segment SGI327712[] = {{&S90359[27], 8}, {NULL}};
const Segment SGI327713[] = {{&S90359[39], 5}, {NULL}};
const Segment SGI327714[] = {{&S105190[3], 5}, {NULL}};
const Segment SGI327715[] = {{&S105192[3], 5}, {NULL}};
const Segment SGI327716[] = {{&S105230[3], 9}, {NULL}};
const Segment SGI327717[] = {{&S105231[3], 5}, {NULL}};
const Segment SGI327718[] = {{&S105240[3], 5}, {NULL}};
const Segment SGI327719[] = {{&S105240[24], 3}, {NULL}};
const Segment SGI327720[] = {{&S105245[3], 5}, {NULL}};
const Segment SGI327721[] = {{&S105245[20], 3}, {NULL}};
const Segment SGI327722[] = {{&S105251[3], 11}, {NULL}};
const Segment SGI327723[] = {{&S100123[0], 84}, {NULL}};
const Segment SGI327724[] = {{&S105273[3], 5}, {NULL}};
const Segment SGI327725[] = {{&S105286[3], 5}, {NULL}};
const Segment SGI327726[] = {{&S100157[0], 30}, {NULL}};
const Segment SGI327727[] = {{&S105293[3], 5}, {NULL}};
const Segment SGI327728[] = {{&S105293[20], 3}, {NULL}};
const Segment SGI327729[] = {{&S105294[3], 5}, {NULL}};
const Segment SGI327730[] = {{&S105294[20], 3}, {NULL}};
const Segment SGI327731[] = {{&S105304[3], 5}, {NULL}};
const Segment SGI327732[] = {{&S105311[3], 5}, {NULL}};
const Segment SGI327733[] = {{&S105323[3], 5}, {NULL}};
const Segment SGI327734[] = {{&S105339[3], 4}, {NULL}};
const Segment SGI327735[] = {{&S90532[4], 5}, {NULL}};
const Segment SGI327736[] = {{&S90532[73], 7}, {&S90532[73], 7}, {NULL}};
const Segment SGI327737[] = {{&S90532[109], 5}, {NULL}};
const Segment SGI327738[] = {{&S90533[4], 5}, {NULL}};
const Segment SGI327739[] = {{&S90533[73], 7}, {&S90533[73], 7}, {&S90533[73], 7}, {NULL}};
const Segment SGI327740[] = {{&S90533[137], 5}, {NULL}};
const Segment SGI327741[] = {{&S90534[4], 5}, {NULL}};
const Segment SGI327742[] = {{&S90534[73], 7}, {&S90534[73], 7}, {&S90534[73], 7}, {&S90534[73], 7}, {NULL}};
const Segment SGI327743[] = {{&S90534[165], 5}, {NULL}};
const Segment SGI327744[] = {{&S90535[4], 5}, {NULL}};
const Segment SGI327745[] = {{&S90535[73], 7}, {&S90535[73], 7}, {&S90535[73], 7}, {&S90535[73], 7}, {&S90535[73], 7}, {NULL}};
const Segment SGI327746[] = {{&S90535[193], 5}, {NULL}};
const Segment SGI327747[] = {{&S90536[4], 5}, {NULL}};
const Segment SGI327748[] = {{&S90536[73], 7}, {NULL}};
const Segment SGI327749[] = {{&S90536[81], 5}, {NULL}};
const Segment SGI327750[] = {{&S105380[3], 5}, {NULL}};
const Segment SGI327751[] = {{&S105380[20], 3}, {NULL}};
const Segment SGI327752[] = {{&S105381[3], 5}, {NULL}};
const Segment SGI327753[] = {{&S105382[3], 5}, {NULL}};
const Segment SGI327754[] = {{&S105383[3], 5}, {NULL}};
const Segment SGI327755[] = {{&S105384[3], 5}, {NULL}};
const Segment SGI327756[] = {{&S105418[3], 5}, {NULL}};
const Segment SGI327757[] = {{&S86898[30], 3}, {NULL}};
const Segment SGI327758[] = {{&S86898[8], 3}, {NULL}};
const Segment SGI327759[] = {{&S86898[13], 10}, {NULL}};
const Segment SGI327760[] = {{&S86898[39], 15}, {NULL}};
const Segment SGI327761[] = {{&S86898[35], 3}, {NULL}};
const Segment SGI327762[] = {{&S90933[4], 5}, {NULL}};
const Segment SGI327763[] = {{&S90933[11], 49}, {NULL}};
const Segment SGI327764[] = {{&S90934[4], 55}, {NULL}};
const Segment SGI327765[] = {{&S90934[10], 49}, {NULL}};
const Segment SGI327766[] = {{&S100732[0], 76}, {NULL}};
const Segment SGI327767[] = {{&S95371[4], 6}, {NULL}};
const Segment SGI327768[] = {{&S95371[26], 3}, {NULL}};
const Segment SGI327769[] = {{&S87340[30], 3}, {NULL}};
const Segment SGI327770[] = {{&S87340[8], 3}, {NULL}};
const Segment SGI327771[] = {{&S87340[13], 10}, {NULL}};
const Segment SGI327772[] = {{&S87340[35], 8}, {NULL}};
const Segment SGI327773[] = {{&S100964[0], 44}, {NULL}};
const Segment SGI327774[] = {{&S100965[0], 38}, {NULL}};
const Segment SGI327775[] = {{&S87427[30], 3}, {NULL}};
const Segment SGI327776[] = {{&S87427[8], 3}, {NULL}};
const Segment SGI327777[] = {{&S87427[13], 10}, {NULL}};
const Segment SGI327778[] = {{&S87427[42], 7}, {NULL}};
const Segment SGI327779[] = {{&S87427[35], 14}, {NULL}};
const Segment SGI327780[] = {{&S87427[51], 3}, {NULL}};
const Segment SGI327781[] = {{&S87427[66], 3}, {NULL}};
const Segment SGI327782[] = {{&S87428[30], 3}, {NULL}};
const Segment SGI327783[] = {{&S87428[8], 3}, {NULL}};
const Segment SGI327784[] = {{&S87428[13], 10}, {NULL}};
const Segment SGI327785[] = {{&S87428[42], 7}, {NULL}};
const Segment SGI327786[] = {{&S87428[35], 14}, {NULL}};
const Segment SGI327787[] = {{&S87428[74], 3}, {NULL}};
const Segment SGI327788[] = {{&S95681[4], 5}, {NULL}};
const Segment SGI327789[] = {{&S95681[25], 3}, {NULL}};
const Segment SGI327790[] = {{&S101135[0], 118}, {NULL}};
const Segment SGI327791[] = {{&S101136[0], 118}, {NULL}};
const Segment SGI327792[] = {{&S101138[0], 94}, {NULL}};
const Segment SGI327793[] = {{&S101164[0], 39}, {NULL}};
const Segment SGI327794[] = {{&S101165[0], 33}, {NULL}};
const Segment SGI327795[] = {{&S87680[30], 3}, {NULL}};
const Segment SGI327796[] = {{&S87680[8], 3}, {NULL}};
const Segment SGI327797[] = {{&S87680[13], 10}, {NULL}};
const Segment SGI327798[] = {{&S87680[42], 7}, {NULL}};
const Segment SGI327799[] = {{&S87680[35], 14}, {NULL}};
const Segment SGI327800[] = {{&S87680[80], 5}, {NULL}};
const Segment SGI327801[] = {{&S87680[51], 6}, {NULL}};
const Segment SGI327802[] = {{&S87682[30], 3}, {NULL}};
const Segment SGI327803[] = {{&S87682[8], 3}, {NULL}};
const Segment SGI327804[] = {{&S87682[13], 10}, {NULL}};
const Segment SGI327805[] = {{&S87682[42], 7}, {NULL}};
const Segment SGI327806[] = {{&S87682[35], 14}, {NULL}};
const Segment SGI327807[] = {{&S87682[80], 5}, {NULL}};
const Segment SGI327808[] = {{&S87682[51], 6}, {NULL}};
const Segment SGI327809[] = {{&S95982[4], 6}, {NULL}};
const Segment SGI327810[] = {{&S91665[4], 5}, {NULL}};
const Segment SGI327811[] = {{&S91665[39], 32}, {NULL}};
const Segment SGI327812[] = {{&S91666[4], 5}, {NULL}};
const Segment SGI327813[] = {{&S91666[39], 32}, {NULL}};
const Segment SGI327814[] = {{&S101502[0], 20}, {NULL}};
const Segment SGI327815[] = {{&S87802[30], 3}, {NULL}};
const Segment SGI327816[] = {{&S87802[8], 3}, {NULL}};
const Segment SGI327817[] = {{&S87802[13], 10}, {NULL}};
const Segment SGI327818[] = {{&S87802[42], 7}, {NULL}};
const Segment SGI327819[] = {{&S87802[35], 14}, {NULL}};
const Segment SGI327820[] = {{&S87802[138], 15}, {NULL}};
const Segment SGI327821[] = {{&S87802[51], 6}, {NULL}};
const Segment SGI327822[] = {{&S87802[179], 3}, {NULL}};
const Segment SGI327823[] = {{&S87804[30], 3}, {NULL}};
const Segment SGI327824[] = {{&S87804[8], 3}, {NULL}};
const Segment SGI327825[] = {{&S87804[13], 10}, {NULL}};
const Segment SGI327826[] = {{&S87804[42], 7}, {NULL}};
const Segment SGI327827[] = {{&S87804[35], 14}, {NULL}};
const Segment SGI327828[] = {{&S87804[131], 15}, {NULL}};
const Segment SGI327829[] = {{&S87804[172], 3}, {NULL}};
const Segment SGI327830[] = {{&S101606[0], 81}, {NULL}};
const Segment SGI327831[] = {{&S101607[0], 80}, {NULL}};
const Segment SGI327832[] = {{&S101619[0], 89}, {NULL}};
const Segment SGI327833[] = {{&S101620[0], 87}, {NULL}};
const Segment SGI327834[] = {{&S101621[0], 87}, {NULL}};
const Segment SGI327835[] = {{&S101638[0], 73}, {NULL}};
const Segment SGI327836[] = {{&S101645[0], 83}, {NULL}};
const Segment SGI327837[] = {{&S101656[0], 31}, {NULL}};
const Segment SGI327838[] = {{&S91877[4], 4}, {NULL}};
const Segment SGI327839[] = {{&S91877[26], 11}, {NULL}};
const Segment SGI327840[] = {{&S91877[38], 5}, {NULL}};
const Segment SGI327841[] = {{&S91878[4], 4}, {NULL}};
const Segment SGI327842[] = {{&S91878[26], 11}, {NULL}};
const Segment SGI327843[] = {{&S91878[38], 5}, {NULL}};
const Segment SGI327844[] = {{&S92064[4], 4}, {NULL}};
const Segment SGI327845[] = {{&S92064[41], 39}, {&S92064[99], 19}, {NULL}};
const Segment SGI327846[] = {{&S92082[4], 5}, {NULL}};
const Segment SGI327847[] = {{&S92082[27], 8}, {NULL}};
const Segment SGI327848[] = {{&S92082[43], 5}, {NULL}};
const Segment SGI327849[] = {{&S101975[0], 80}, {NULL}};
const Segment SGI327850[] = {{&S88325[30], 3}, {NULL}};
const Segment SGI327851[] = {{&S88325[8], 3}, {NULL}};
const Segment SGI327852[] = {{&S88325[13], 10}, {NULL}};
const Segment SGI327853[] = {{&S88325[42], 7}, {NULL}};
const Segment SGI327854[] = {{&S88325[35], 14}, {NULL}};
const Segment SGI327855[] = {{&S88325[78], 15}, {NULL}};
const Segment SGI327856[] = {{&S88325[139], 7}, {NULL}};
const Segment SGI327857[] = {{&S88325[146], 4}, {NULL}};
const Segment SGI327858[] = {{&S88326[30], 3}, {NULL}};
const Segment SGI327859[] = {{&S88326[8], 3}, {NULL}};
const Segment SGI327860[] = {{&S88326[13], 10}, {NULL}};
const Segment SGI327861[] = {{&S88326[42], 7}, {NULL}};
const Segment SGI327862[] = {{&S88326[35], 14}, {NULL}};
const Segment SGI327863[] = {{&S88326[98], 7}, {NULL}};
const Segment SGI327864[] = {{&S88326[105], 4}, {NULL}};
const Segment SGI327865[] = {{&S102157[0], 80}, {NULL}};
const Segment SGI327866[] = {{&S102166[0], 35}, {NULL}};
const Segment SGI327867[] = {{&S88463[30], 3}, {NULL}};
const Segment SGI327868[] = {{&S88463[8], 3}, {NULL}};
const Segment SGI327869[] = {{&S88463[13], 10}, {NULL}};
const Segment SGI327870[] = {{&S92279[4], 5}, {NULL}};
const Segment SGI327871[] = {{&S92279[27], 8}, {NULL}};
const Segment SGI327872[] = {{&S92279[43], 5}, {NULL}};
const Segment SGI327873[] = {{&S88519[30], 3}, {NULL}};
const Segment SGI327874[] = {{&S88519[8], 3}, {NULL}};
const Segment SGI327875[] = {{&S88519[13], 10}, {NULL}};
const Segment SGI327876[] = {{&S88519[42], 7}, {NULL}};
const Segment SGI327877[] = {{&S88519[35], 14}, {NULL}};
const Segment SGI327878[] = {{&S88519[128], 15}, {NULL}};
const Segment SGI327879[] = {{&S102364[0], 31}, {NULL}};
const Segment SGI327880[] = {{&S88649[30], 3}, {NULL}};
const Segment SGI327881[] = {{&S88649[8], 3}, {NULL}};
const Segment SGI327882[] = {{&S88649[13], 10}, {NULL}};
const Segment SGI327883[] = {{&S88649[42], 7}, {NULL}};
const Segment SGI327884[] = {{&S88649[35], 14}, {NULL}};
const Segment SGI327885[] = {{&S88649[56], 7}, {NULL}};
const Segment SGI327886[] = {{&S88649[7], 4}, {NULL}};
const Segment SGI327887[] = {{&S88649[51], 3}, {NULL}};
const Segment SGI327888[] = {{&S102409[0], 22}, {NULL}};
const Segment SGI327889[] = {{&S96925[4], 5}, {NULL}};
const Segment SGI327890[] = {{&S96925[25], 3}, {NULL}};
const Segment SGI327891[] = {{&S92660[4], 3}, {NULL}};
const Segment SGI327892[] = {{&S92660[25], 8}, {NULL}};
const Segment SGI327893[] = {{&S92660[41], 5}, {NULL}};
const Segment SGI327894[] = {{&S88926[30], 3}, {NULL}};
const Segment SGI327895[] = {{&S88926[8], 3}, {NULL}};
const Segment SGI327896[] = {{&S88926[13], 10}, {NULL}};
const Segment SGI327897[] = {{&S88982[30], 3}, {NULL}};
const Segment SGI327898[] = {{&S88982[8], 3}, {NULL}};
const Segment SGI327899[] = {{&S88982[13], 10}, {NULL}};
const Segment SGI327900[] = {{&S88982[42], 7}, {NULL}};
const Segment SGI327901[] = {{&S88982[35], 14}, {NULL}};
const Segment SGI327902[] = {{&S88982[130], 15}, {NULL}};
const Segment SGI327903[] = {{&S88982[168], 3}, {NULL}};
const Segment SGI327904[] = {{&S89006[30], 3}, {NULL}};
const Segment SGI327905[] = {{&S89006[8], 3}, {NULL}};
const Segment SGI327906[] = {{&S89006[13], 10}, {NULL}};
const Segment SGI327907[] = {{&S89006[42], 7}, {NULL}};
const Segment SGI327908[] = {{&S89006[35], 14}, {NULL}};
const Segment SGI327909[] = {{&S89006[123], 6}, {NULL}};
const Segment SGI327910[] = {{&S89007[30], 3}, {NULL}};
const Segment SGI327911[] = {{&S89007[8], 3}, {NULL}};
const Segment SGI327912[] = {{&S89007[13], 10}, {NULL}};
const Segment SGI327913[] = {{&S89007[42], 7}, {NULL}};
const Segment SGI327914[] = {{&S89007[35], 14}, {NULL}};
const Segment SGI327915[] = {{&S89007[168], 6}, {NULL}};
const Segment SGI327916[] = {{&S102602[0], 85}, {NULL}};
const Segment SGI327917[] = {{&S92829[4], 4}, {NULL}};
const Segment SGI327918[] = {{&S102732[0], 27}, {NULL}};
const Segment SGI327919[] = {{&S102733[0], 27}, {NULL}};
const Segment SGI327920[] = {{&S102734[0], 27}, {NULL}};
const Segment SGI327921[] = {{&S102735[0], 27}, {NULL}};
const Segment SGI327922[] = {{&S102736[0], 66}, {NULL}};
const Segment SGI327923[] = {{&S102801[0], 37}, {NULL}};
const Segment SGI327924[] = {{&S102827[0], 54}, {NULL}};
const Segment SGI327925[] = {{&S102828[0], 58}, {NULL}};
const Segment SGI327926[] = {{&S102959[0], 44}, {NULL}};
const Segment SGI327927[] = {{&S103007[0], 27}, {NULL}};
const Segment SGI327928[] = {{&S103064[0], 70}, {NULL}};
const Segment SGI327929[] = {{&S89587[30], 3}, {NULL}};
const Segment SGI327930[] = {{&S89587[8], 3}, {NULL}};
const Segment SGI327931[] = {{&S89587[13], 10}, {NULL}};
const Segment SGI327932[] = {{&S89587[42], 7}, {NULL}};
const Segment SGI327933[] = {{&S89587[35], 14}, {NULL}};
const Segment SGI327934[] = {{&S89587[60], 5}, {NULL}};
const Segment SGI327935[] = {{&S103253[0], 68}, {NULL}};
const Segment SGI327936[] = {{&S103267[0], 35}, {NULL}};
const Segment SGI327937[] = {{&S103284[0], 38}, {NULL}};
const Segment SGI327938[] = {{&S103299[0], 49}, {NULL}};
const Segment SGI327939[] = {{&S103300[0], 49}, {NULL}};
const Segment SGI327940[] = {{&S103301[0], 26}, {NULL}};
const Segment SGI327941[] = {{&S93504[4], 5}, {NULL}};
const Segment SGI327942[] = {{&S93505[4], 4}, {NULL}};
const Segment SGI327943[] = {{&S103556[0], 33}, {NULL}};
const Segment SGI327944[] = {{&S103611[0], 38}, {NULL}};
const Segment SGI327945[] = {{&S93741[4], 4}, {NULL}};
const Segment SGI327946[] = {{&S90193[30], 3}, {NULL}};
const Segment SGI327947[] = {{&S90193[8], 3}, {NULL}};
const Segment SGI327948[] = {{&S90193[13], 10}, {NULL}};
const Segment SGI327949[] = {{&S90193[42], 7}, {NULL}};
const Segment SGI327950[] = {{&S90193[35], 14}, {NULL}};
const Segment SGI327951[] = {{&S90193[51], 3}, {NULL}};
const Segment SGI327952[] = {{&S90193[86], 3}, {NULL}};
const Segment SGI327953[] = {{&S90194[30], 3}, {NULL}};
const Segment SGI327954[] = {{&S90194[8], 3}, {NULL}};
const Segment SGI327955[] = {{&S90194[13], 10}, {NULL}};
const Segment SGI327956[] = {{&S90194[42], 7}, {NULL}};
const Segment SGI327957[] = {{&S90194[35], 14}, {NULL}};
const Segment SGI327958[] = {{&S90194[74], 3}, {NULL}};
const Segment SGI327959[] = {{&S103882[0], 28}, {NULL}};
const Segment SGI327960[] = {{&S90385[30], 3}, {NULL}};
const Segment SGI327961[] = {{&S90385[8], 3}, {NULL}};
const Segment SGI327962[] = {{&S90385[13], 10}, {NULL}};
const Segment SGI327963[] = {{&S90385[42], 3}, {NULL}};
const Segment SGI327964[] = {{&S90385[47], 13}, {&S90385[47], 13}, {NULL}};
const Segment SGI327965[] = {{&S90412[30], 4}, {NULL}};
const Segment SGI327966[] = {{&S90412[8], 3}, {NULL}};
const Segment SGI327967[] = {{&S90412[13], 10}, {NULL}};
const Segment SGI327968[] = {{&S90412[43], 7}, {NULL}};
const Segment SGI327969[] = {{&S90412[36], 14}, {NULL}};
const Segment SGI327970[] = {{&S90412[52], 6}, {NULL}};
const Segment SGI327971[] = {{&S90412[82], 3}, {NULL}};
const Segment SGI327972[] = {{&S90413[30], 4}, {NULL}};
const Segment SGI327973[] = {{&S90413[8], 3}, {NULL}};
const Segment SGI327974[] = {{&S90413[13], 10}, {NULL}};
const Segment SGI327975[] = {{&S90413[43], 7}, {NULL}};
const Segment SGI327976[] = {{&S90413[36], 14}, {NULL}};
const Segment SGI327977[] = {{&S90413[75], 3}, {NULL}};
const Segment SGI327978[] = {{&S104163[0], 57}, {NULL}};
const Segment SGI327979[] = {{&S104170[0], 29}, {NULL}};
const Segment SGI327980[] = {{&S104171[0], 29}, {NULL}};
const Segment SGI327981[] = {{&S104232[0], 37}, {NULL}};
const Segment SGI327982[] = {{&S104235[0], 14}, {NULL}};
const Segment SGI327983[] = {{&S104248[0], 33}, {NULL}};
const Segment SGI327984[] = {{&S104269[0], 83}, {NULL}};
const Segment SGI327985[] = {{&S98523[4], 5}, {NULL}};
const Segment SGI327986[] = {{&S98523[25], 3}, {NULL}};
const Segment SGI327987[] = {{&S94399[4], 4}, {NULL}};
const Segment SGI327988[] = {{&S94409[4], 4}, {NULL}};
const Segment SGI327989[] = {{&S104610[0], 14}, {NULL}};
const Segment SGI327990[] = {{&S104611[0], 39}, {NULL}};
const Segment SGI327991[] = {{&S90976[30], 3}, {NULL}};
const Segment SGI327992[] = {{&S90976[8], 3}, {NULL}};
const Segment SGI327993[] = {{&S90976[13], 10}, {NULL}};
const Segment SGI327994[] = {{&S90976[42], 7}, {NULL}};
const Segment SGI327995[] = {{&S90976[35], 14}, {NULL}};
const Segment SGI327996[] = {{&S90976[105], 15}, {NULL}};
const Segment SGI327997[] = {{&S90976[135], 5}, {NULL}};
const Segment SGI327998[] = {{&S90976[51], 6}, {NULL}};
const Segment SGI327999[] = {{&S104617[0], 21}, {NULL}};
const Segment SGI328000[] = {{&S104619[0], 81}, {NULL}};
const Segment SGI328001[] = {{&S104646[0], 32}, {NULL}};
const Segment SGI328002[] = {{&S104660[0], 68}, {NULL}};
const Segment SGI328003[] = {{&S104751[0], 64}, {NULL}};
const Segment SGI328004[] = {{&S91126[30], 3}, {NULL}};
const Segment SGI328005[] = {{&S91126[8], 3}, {NULL}};
const Segment SGI328006[] = {{&S91126[13], 10}, {NULL}};
const Segment SGI328007[] = {{&S91126[42], 7}, {NULL}};
const Segment SGI328008[] = {{&S91126[35], 14}, {NULL}};
const Segment SGI328009[] = {{&S91126[139], 11}, {NULL}};
const Segment SGI328010[] = {{&S91126[78], 15}, {NULL}};
const Segment SGI328011[] = {{&S91126[139], 15}, {NULL}};
const Segment SGI328012[] = {{&S91127[30], 3}, {NULL}};
const Segment SGI328013[] = {{&S91127[8], 3}, {NULL}};
const Segment SGI328014[] = {{&S91127[13], 10}, {NULL}};
const Segment SGI328015[] = {{&S91127[42], 7}, {NULL}};
const Segment SGI328016[] = {{&S91127[35], 14}, {NULL}};
const Segment SGI328017[] = {{&S91127[226], 11}, {NULL}};
const Segment SGI328018[] = {{&S91127[97], 15}, {NULL}};
const Segment SGI328019[] = {{&S91127[226], 15}, {NULL}};
const Segment SGI328020[] = {{&S91152[30], 3}, {NULL}};
const Segment SGI328021[] = {{&S91152[8], 3}, {NULL}};
const Segment SGI328022[] = {{&S91152[13], 10}, {NULL}};
const Segment SGI328023[] = {{&S91152[42], 7}, {NULL}};
const Segment SGI328024[] = {{&S91152[61], 3}, {NULL}};
const Segment SGI328025[] = {{&S94879[4], 7}, {NULL}};
const Segment SGI328026[] = {{&S91486[30], 3}, {NULL}};
const Segment SGI328027[] = {{&S91486[8], 3}, {NULL}};
const Segment SGI328028[] = {{&S91486[13], 10}, {NULL}};
const Segment SGI328029[] = {{&S91486[42], 3}, {NULL}};
const Segment SGI328030[] = {{&S91486[47], 13}, {NULL}};
const Segment SGI328031[] = {{&S91487[30], 3}, {NULL}};
const Segment SGI328032[] = {{&S91487[8], 3}, {NULL}};
const Segment SGI328033[] = {{&S91487[13], 10}, {NULL}};
const Segment SGI328034[] = {{&S91487[42], 3}, {NULL}};
const Segment SGI328035[] = {{&S91487[47], 13}, {&S91487[47], 13}, {NULL}};
const Segment SGI328036[] = {{&S91492[30], 3}, {NULL}};
const Segment SGI328037[] = {{&S91492[8], 3}, {NULL}};
const Segment SGI328038[] = {{&S91492[13], 10}, {NULL}};
const Segment SGI328039[] = {{&S91492[42], 7}, {NULL}};
const Segment SGI328040[] = {{&S91492[35], 14}, {NULL}};
const Segment SGI328041[] = {{&S91492[51], 3}, {NULL}};
const Segment SGI328042[] = {{&S91492[66], 3}, {NULL}};
const Segment SGI328043[] = {{&S91493[30], 3}, {NULL}};
const Segment SGI328044[] = {{&S91493[8], 3}, {NULL}};
const Segment SGI328045[] = {{&S91493[13], 10}, {NULL}};
const Segment SGI328046[] = {{&S91493[42], 7}, {NULL}};
const Segment SGI328047[] = {{&S91493[35], 14}, {NULL}};
const Segment SGI328048[] = {{&S91493[74], 3}, {NULL}};
const Segment SGI328049[] = {{&S91525[30], 4}, {NULL}};
const Segment SGI328050[] = {{&S91525[8], 3}, {NULL}};
const Segment SGI328051[] = {{&S91525[13], 10}, {NULL}};
const Segment SGI328052[] = {{&S91525[43], 7}, {NULL}};
const Segment SGI328053[] = {{&S91525[36], 14}, {NULL}};
const Segment SGI328054[] = {{&S91525[79], 11}, {NULL}};
const Segment SGI328055[] = {{&S91525[72], 5}, {NULL}};
const Segment SGI328056[] = {{&S91525[52], 6}, {NULL}};
const Segment SGI328057[] = {{&S91525[79], 16}, {NULL}};
const Segment SGI328058[] = {{&S105164[0], 31}, {NULL}};
const Segment SGI328059[] = {{&S105187[0], 44}, {NULL}};
const Segment SGI328060[] = {{&S105272[0], 24}, {NULL}};
const Segment SGI328061[] = {{&S105353[0], 75}, {NULL}};
const Segment SGI328062[] = {{&S91790[30], 3}, {NULL}};
const Segment SGI328063[] = {{&S91790[8], 3}, {NULL}};
const Segment SGI328064[] = {{&S91790[13], 10}, {NULL}};
const Segment SGI328065[] = {{&S91790[42], 7}, {NULL}};
const Segment SGI328066[] = {{&S91790[35], 14}, {NULL}};
const Segment SGI328067[] = {{&S91790[51], 7}, {NULL}};
const Segment SGI328068[] = {{&S91791[30], 3}, {NULL}};
const Segment SGI328069[] = {{&S91791[8], 3}, {NULL}};
const Segment SGI328070[] = {{&S91791[13], 10}, {NULL}};
const Segment SGI328071[] = {{&S91791[42], 7}, {NULL}};
const Segment SGI328072[] = {{&S91791[35], 14}, {NULL}};
const Segment SGI328073[] = {{&S91791[51], 6}, {NULL}};
const Segment SGI328074[] = {{&S91791[98], 7}, {NULL}};
const Segment SGI328075[] = {{&S105450[0], 76}, {NULL}};
const Segment SGI328076[] = {{&S99561[4], 5}, {NULL}};
const Segment SGI328077[] = {{&S99561[25], 3}, {NULL}};
const Segment SGI328078[] = {{&S95676[4], 2}, {NULL}};
const Segment SGI328079[] = {{&S92159[34], 5}, {NULL}};
const Segment SGI328080[] = {{&S92159[48], 7}, {NULL}};
const Segment SGI328081[] = {{&S92159[41], 14}, {NULL}};
const Segment SGI328082[] = {{&S106005[0], 36}, {NULL}};
const Segment SGI328083[] = {{&S100129[4], 6}, {NULL}};
const Segment SGI328084[] = {{&S100129[26], 3}, {NULL}};
const Segment SGI328085[] = {{&S93064[30], 3}, {NULL}};
const Segment SGI328086[] = {{&S93064[8], 3}, {NULL}};
const Segment SGI328087[] = {{&S93064[13], 10}, {NULL}};
const Segment SGI328088[] = {{&S93064[42], 7}, {NULL}};
const Segment SGI328089[] = {{&S93064[35], 14}, {NULL}};
const Segment SGI328090[] = {{&S93064[81], 4}, {NULL}};
const Segment SGI328091[] = {{&S93064[51], 3}, {NULL}};
const Segment SGI328092[] = {{&S93065[30], 3}, {NULL}};
const Segment SGI328093[] = {{&S93065[8], 3}, {NULL}};
const Segment SGI328094[] = {{&S93065[13], 10}, {NULL}};
const Segment SGI328095[] = {{&S93065[42], 7}, {NULL}};
const Segment SGI328096[] = {{&S93065[35], 14}, {NULL}};
const Segment SGI328097[] = {{&S93065[105], 4}, {NULL}};
const Segment SGI328098[] = {{&S100744[4], 6}, {NULL}};
const Segment SGI328099[] = {{&S100744[26], 3}, {NULL}};
const Segment SGI328100[] = {{&S93222[30], 3}, {NULL}};
const Segment SGI328101[] = {{&S93222[8], 3}, {NULL}};
const Segment SGI328102[] = {{&S93222[13], 10}, {NULL}};
const Segment SGI328103[] = {{&S93222[42], 8}, {NULL}};
const Segment SGI328104[] = {{&S93222[52], 6}, {NULL}};
const Segment SGI328105[] = {{&S93222[109], 4}, {NULL}};
const Segment SGI328106[] = {{&S93223[30], 3}, {NULL}};
const Segment SGI328107[] = {{&S93223[8], 3}, {NULL}};
const Segment SGI328108[] = {{&S93223[13], 10}, {NULL}};
const Segment SGI328109[] = {{&S93223[42], 8}, {NULL}};
const Segment SGI328110[] = {{&S93223[52], 6}, {NULL}};
const Segment SGI328111[] = {{&S93223[109], 4}, {NULL}};
const Segment SGI328112[] = {{&S93297[30], 3}, {NULL}};
const Segment SGI328113[] = {{&S93297[8], 3}, {NULL}};
const Segment SGI328114[] = {{&S93297[13], 10}, {NULL}};
const Segment SGI328115[] = {{&S93297[42], 7}, {NULL}};
const Segment SGI328116[] = {{&S93297[35], 14}, {NULL}};
const Segment SGI328117[] = {{&S93297[71], 11}, {NULL}};
const Segment SGI328118[] = {{&S93297[51], 6}, {NULL}};
const Segment SGI328119[] = {{&S93297[71], 15}, {NULL}};
const Segment SGI328120[] = {{&S93298[30], 3}, {NULL}};
const Segment SGI328121[] = {{&S93298[8], 3}, {NULL}};
const Segment SGI328122[] = {{&S93298[13], 10}, {NULL}};
const Segment SGI328123[] = {{&S93298[42], 7}, {NULL}};
const Segment SGI328124[] = {{&S93298[35], 14}, {NULL}};
const Segment SGI328125[] = {{&S93298[64], 11}, {NULL}};
const Segment SGI328126[] = {{&S93298[64], 15}, {NULL}};
const Segment SGI328127[] = {{&S93345[30], 3}, {NULL}};
const Segment SGI328128[] = {{&S93345[8], 3}, {NULL}};
const Segment SGI328129[] = {{&S93345[13], 10}, {NULL}};
const Segment SGI328130[] = {{&S93345[42], 7}, {NULL}};
const Segment SGI328131[] = {{&S93345[35], 14}, {NULL}};
const Segment SGI328132[] = {{&S93345[78], 15}, {NULL}};
const Segment SGI328133[] = {{&S93345[140], 6}, {NULL}};
const Segment SGI328134[] = {{&S96905[4], 4}, {NULL}};
const Segment SGI328135[] = {{&S96906[4], 9}, {NULL}};
const Segment SGI328136[] = {{&S93400[30], 3}, {NULL}};
const Segment SGI328137[] = {{&S93400[8], 3}, {NULL}};
const Segment SGI328138[] = {{&S93400[13], 10}, {NULL}};
const Segment SGI328139[] = {{&S93400[35], 6}, {NULL}};
const Segment SGI328140[] = {{&S96907[4], 9}, {NULL}};
const Segment SGI328141[] = {{&S96908[4], 14}, {NULL}};
const Segment SGI328142[] = {{&S96909[4], 16}, {NULL}};
const Segment SGI328143[] = {{&S96910[4], 7}, {NULL}};
const Segment SGI328144[] = {{&S97255[4], 2}, {NULL}};
const Segment SGI328145[] = {{&S97255[70], 7}, {NULL}};
const Segment SGI328146[] = {{&S97255[78], 5}, {NULL}};
const Segment SGI328147[] = {{&S97268[4], 5}, {NULL}};
const Segment SGI328148[] = {{&S97268[27], 11}, {NULL}};
const Segment SGI328149[] = {{&S97268[39], 5}, {NULL}};
const Segment SGI328150[] = {{&S93762[30], 3}, {NULL}};
const Segment SGI328151[] = {{&S93762[8], 3}, {NULL}};
const Segment SGI328152[] = {{&S93762[13], 10}, {NULL}};
const Segment SGI328153[] = {{&S93762[42], 7}, {NULL}};
const Segment SGI328154[] = {{&S93762[35], 14}, {NULL}};
const Segment SGI328155[] = {{&S93762[64], 11}, {NULL}};
const Segment SGI328156[] = {{&S93762[64], 16}, {NULL}};
const Segment SGI328157[] = {{&S93763[30], 3}, {NULL}};
const Segment SGI328158[] = {{&S93763[8], 3}, {NULL}};
const Segment SGI328159[] = {{&S93763[13], 10}, {NULL}};
const Segment SGI328160[] = {{&S93763[42], 7}, {NULL}};
const Segment SGI328161[] = {{&S93763[35], 14}, {NULL}};
const Segment SGI328162[] = {{&S93763[71], 11}, {NULL}};
const Segment SGI328163[] = {{&S93763[51], 6}, {NULL}};
const Segment SGI328164[] = {{&S93763[71], 16}, {NULL}};
const Segment SGI328165[] = {{&S101470[4], 6}, {NULL}};
const Segment SGI328166[] = {{&S101470[26], 3}, {NULL}};
const Segment SGI328167[] = {{&S97626[4], 12}, {NULL}};
const Segment SGI328168[] = {{&S97626[29], 40}, {NULL}};
const Segment SGI328169[] = {{&S97627[4], 3}, {NULL}};
const Segment SGI328170[] = {{&S97627[38], 43}, {NULL}};
const Segment SGI328171[] = {{&S97629[4], 5}, {NULL}};
const Segment SGI328172[] = {{&S97634[4], 4}, {NULL}};
const Segment SGI328173[] = {{&S94254[30], 3}, {NULL}};
const Segment SGI328174[] = {{&S94254[8], 3}, {NULL}};
const Segment SGI328175[] = {{&S94254[13], 10}, {NULL}};
const Segment SGI328176[] = {{&S94254[42], 7}, {NULL}};
const Segment SGI328177[] = {{&S94254[60], 4}, {NULL}};
const Segment SGI328178[] = {{&S101630[4], 6}, {NULL}};
const Segment SGI328179[] = {{&S101630[26], 3}, {NULL}};
const Segment SGI328180[] = {{&S101660[4], 6}, {NULL}};
const Segment SGI328181[] = {{&S101660[26], 3}, {NULL}};
const Segment SGI328182[] = {{&S101661[4], 6}, {NULL}};
const Segment SGI328183[] = {{&S101661[26], 3}, {NULL}};
const Segment SGI328184[] = {{&S94335[30], 3}, {NULL}};
const Segment SGI328185[] = {{&S94335[8], 3}, {NULL}};
const Segment SGI328186[] = {{&S94335[13], 10}, {NULL}};
const Segment SGI328187[] = {{&S94335[42], 7}, {NULL}};
const Segment SGI328188[] = {{&S94335[35], 14}, {NULL}};
const Segment SGI328189[] = {{&S94335[64], 7}, {NULL}};
const Segment SGI328190[] = {{&S94335[71], 4}, {NULL}};
const Segment SGI328191[] = {{&S101830[4], 5}, {NULL}};
const Segment SGI328192[] = {{&S98081[4], 5}, {NULL}};
const Segment SGI328193[] = {{&S98081[27], 8}, {NULL}};
const Segment SGI328194[] = {{&S98081[43], 5}, {NULL}};
const Segment SGI328195[] = {{&S98096[4], 4}, {NULL}};
const Segment SGI328196[] = {{&S98096[26], 8}, {NULL}};
const Segment SGI328197[] = {{&S98096[42], 5}, {NULL}};
const Segment SGI328198[] = {{&S101978[4], 5}, {NULL}};
const Segment SGI328199[] = {{&S101978[21], 3}, {NULL}};
const Segment SGI328200[] = {{&S101981[4], 6}, {NULL}};
const Segment SGI328201[] = {{&S101981[26], 3}, {NULL}};
const Segment SGI328202[] = {{&S102003[4], 5}, {NULL}};
const Segment SGI328203[] = {{&S102003[25], 3}, {NULL}};
const Segment SGI328204[] = {{&S94716[30], 3}, {NULL}};
const Segment SGI328205[] = {{&S94716[8], 3}, {NULL}};
const Segment SGI328206[] = {{&S94716[13], 10}, {NULL}};
const Segment SGI328207[] = {{&S94716[42], 7}, {NULL}};
const Segment SGI328208[] = {{&S94716[35], 14}, {NULL}};
const Segment SGI328209[] = {{&S94716[78], 15}, {NULL}};
const Segment SGI328210[] = {{&S94716[128], 5}, {NULL}};
const Segment SGI328211[] = {{&S94725[30], 3}, {NULL}};
const Segment SGI328212[] = {{&S94725[8], 3}, {NULL}};
const Segment SGI328213[] = {{&S94725[13], 10}, {NULL}};
const Segment SGI328214[] = {{&S94725[42], 7}, {NULL}};
const Segment SGI328215[] = {{&S94725[35], 14}, {NULL}};
const Segment SGI328216[] = {{&S94725[99], 5}, {NULL}};
const Segment SGI328217[] = {{&S102168[4], 5}, {NULL}};
const Segment SGI328218[] = {{&S102168[25], 3}, {NULL}};
const Segment SGI328219[] = {{&S102169[4], 5}, {NULL}};
const Segment SGI328220[] = {{&S102169[25], 3}, {NULL}};
const Segment SGI328221[] = {{&S94947[30], 3}, {NULL}};
const Segment SGI328222[] = {{&S94947[8], 3}, {NULL}};
const Segment SGI328223[] = {{&S94947[13], 10}, {NULL}};
const Segment SGI328224[] = {{&S94947[42], 7}, {NULL}};
const Segment SGI328225[] = {{&S94947[35], 14}, {NULL}};
const Segment SGI328226[] = {{&S94947[122], 5}, {NULL}};
const Segment SGI328227[] = {{&S98522[4], 5}, {NULL}};
const Segment SGI328228[] = {{&S98522[27], 11}, {NULL}};
const Segment SGI328229[] = {{&S98522[39], 5}, {NULL}};
const Segment SGI328230[] = {{&S98526[4], 5}, {NULL}};
const Segment SGI328231[] = {{&S98526[40], 38}, {NULL}};
const Segment SGI328232[] = {{&S98527[4], 5}, {NULL}};
const Segment SGI328233[] = {{&S98527[40], 51}, {NULL}};
const Segment SGI328234[] = {{&S98528[4], 5}, {NULL}};
const Segment SGI328235[] = {{&S98528[40], 51}, {NULL}};
const Segment SGI328236[] = {{&S98529[4], 5}, {NULL}};
const Segment SGI328237[] = {{&S98529[40], 51}, {NULL}};
const Segment SGI328238[] = {{&S98530[4], 5}, {NULL}};
const Segment SGI328239[] = {{&S98530[40], 51}, {NULL}};
const Segment SGI328240[] = {{&S98531[4], 5}, {NULL}};
const Segment SGI328241[] = {{&S98531[40], 38}, {NULL}};
const Segment SGI328242[] = {{&S102350[4], 5}, {NULL}};
const Segment SGI328243[] = {{&S102350[25], 3}, {NULL}};
const Segment SGI328244[] = {{&S102400[4], 6}, {NULL}};
const Segment SGI328245[] = {{&S102400[26], 3}, {NULL}};
const Segment SGI328246[] = {{&S102601[4], 6}, {NULL}};
const Segment SGI328247[] = {{&S102601[26], 3}, {NULL}};
const Segment SGI328248[] = {{&S102611[4], 5}, {NULL}};
const Segment SGI328249[] = {{&S95313[30], 3}, {NULL}};
const Segment SGI328250[] = {{&S95313[8], 3}, {NULL}};
const Segment SGI328251[] = {{&S95313[13], 10}, {NULL}};
const Segment SGI328252[] = {{&S95313[42], 7}, {NULL}};
const Segment SGI328253[] = {{&S95313[35], 14}, {NULL}};
const Segment SGI328254[] = {{&S95313[71], 11}, {NULL}};
const Segment SGI328255[] = {{&S95313[51], 6}, {NULL}};
const Segment SGI328256[] = {{&S95313[71], 16}, {NULL}};
const Segment SGI328257[] = {{&S95383[30], 3}, {NULL}};
const Segment SGI328258[] = {{&S95383[8], 3}, {NULL}};
const Segment SGI328259[] = {{&S95383[13], 10}, {NULL}};
const Segment SGI328260[] = {{&S95383[42], 7}, {NULL}};
const Segment SGI328261[] = {{&S95383[35], 14}, {NULL}};
const Segment SGI328262[] = {{&S95383[64], 11}, {NULL}};
const Segment SGI328263[] = {{&S95383[64], 16}, {NULL}};
const Segment SGI328264[] = {{&S95384[30], 3}, {NULL}};
const Segment SGI328265[] = {{&S95384[8], 3}, {NULL}};
const Segment SGI328266[] = {{&S95384[13], 10}, {NULL}};
const Segment SGI328267[] = {{&S95384[42], 7}, {NULL}};
const Segment SGI328268[] = {{&S95384[35], 14}, {NULL}};
const Segment SGI328269[] = {{&S95384[71], 11}, {NULL}};
const Segment SGI328270[] = {{&S95384[51], 6}, {NULL}};
const Segment SGI328271[] = {{&S95384[71], 16}, {NULL}};
const Segment SGI328272[] = {{&S95387[30], 3}, {NULL}};
const Segment SGI328273[] = {{&S95387[8], 3}, {NULL}};
const Segment SGI328274[] = {{&S95387[13], 10}, {NULL}};
const Segment SGI328275[] = {{&S95387[42], 7}, {NULL}};
const Segment SGI328276[] = {{&S95387[35], 14}, {NULL}};
const Segment SGI328277[] = {{&S95387[59], 11}, {NULL}};
const Segment SGI328278[] = {{&S95387[59], 16}, {NULL}};
const Segment SGI328279[] = {{&S95389[30], 3}, {NULL}};
const Segment SGI328280[] = {{&S95389[8], 3}, {NULL}};
const Segment SGI328281[] = {{&S95389[13], 10}, {NULL}};
const Segment SGI328282[] = {{&S95389[55], 7}, {NULL}};
const Segment SGI328283[] = {{&S95389[48], 14}, {NULL}};
const Segment SGI328284[] = {{&S95389[64], 13}, {NULL}};
const Segment SGI328285[] = {{&S95390[30], 3}, {NULL}};
const Segment SGI328286[] = {{&S95390[8], 3}, {NULL}};
const Segment SGI328287[] = {{&S95390[13], 10}, {NULL}};
const Segment SGI328288[] = {{&S95390[55], 7}, {NULL}};
const Segment SGI328289[] = {{&S95390[48], 14}, {NULL}};
const Segment SGI328290[] = {{&S95390[64], 13}, {NULL}};
const Segment SGI328291[] = {{&S95421[30], 3}, {NULL}};
const Segment SGI328292[] = {{&S95421[8], 3}, {NULL}};
const Segment SGI328293[] = {{&S95421[13], 10}, {NULL}};
const Segment SGI328294[] = {{&S95421[42], 3}, {NULL}};
const Segment SGI328295[] = {{&S102813[4], 6}, {NULL}};
const Segment SGI328296[] = {{&S102813[26], 3}, {NULL}};
const Segment SGI328297[] = {{&S99027[4], 4}, {NULL}};
const Segment SGI328298[] = {{&S99028[4], 5}, {NULL}};
const Segment SGI328299[] = {{&S99037[4], 7}, {NULL}};
const Segment SGI328300[] = {{&S99038[4], 4}, {NULL}};
const Segment SGI328301[] = {{&S95908[30], 4}, {NULL}};
const Segment SGI328302[] = {{&S95908[8], 3}, {NULL}};
const Segment SGI328303[] = {{&S95908[13], 10}, {NULL}};
const Segment SGI328304[] = {{&S95908[43], 7}, {NULL}};
const Segment SGI328305[] = {{&S95908[36], 14}, {NULL}};
const Segment SGI328306[] = {{&S95908[52], 3}, {NULL}};
const Segment SGI328307[] = {{&S95908[70], 5}, {NULL}};
const Segment SGI328308[] = {{&S95990[30], 3}, {NULL}};
const Segment SGI328309[] = {{&S95990[8], 3}, {NULL}};
const Segment SGI328310[] = {{&S95990[13], 10}, {NULL}};
const Segment SGI328311[] = {{&S95990[42], 7}, {NULL}};
const Segment SGI328312[] = {{&S95990[35], 14}, {NULL}};
const Segment SGI328313[] = {{&S95990[131], 15}, {NULL}};
const Segment SGI328314[] = {{&S95990[162], 3}, {NULL}};
const Segment SGI328315[] = {{&S95991[30], 3}, {NULL}};
const Segment SGI328316[] = {{&S95991[8], 3}, {NULL}};
const Segment SGI328317[] = {{&S95991[13], 10}, {NULL}};
const Segment SGI328318[] = {{&S95991[42], 7}, {NULL}};
const Segment SGI328319[] = {{&S95991[35], 14}, {NULL}};
const Segment SGI328320[] = {{&S95991[138], 15}, {NULL}};
const Segment SGI328321[] = {{&S95991[51], 6}, {NULL}};
const Segment SGI328322[] = {{&S95991[179], 3}, {NULL}};
const Segment SGI328323[] = {{&S99535[4], 14}, {NULL}};
const Segment SGI328324[] = {{&S99535[48], 32}, {NULL}};
const Segment SGI328325[] = {{&S99536[4], 14}, {NULL}};
const Segment SGI328326[] = {{&S99536[48], 32}, {NULL}};
const Segment SGI328327[] = {{&S99537[4], 14}, {NULL}};
const Segment SGI328328[] = {{&S99537[49], 32}, {NULL}};
const Segment SGI328329[] = {{&S99538[4], 14}, {NULL}};
const Segment SGI328330[] = {{&S99538[48], 32}, {NULL}};
const Segment SGI328331[] = {{&S99539[4], 14}, {NULL}};
const Segment SGI328332[] = {{&S99539[48], 32}, {NULL}};
const Segment SGI328333[] = {{&S99540[4], 14}, {NULL}};
const Segment SGI328334[] = {{&S99540[48], 32}, {NULL}};
const Segment SGI328335[] = {{&S103545[4], 5}, {NULL}};
const Segment SGI328336[] = {{&S103545[25], 3}, {NULL}};
const Segment SGI328337[] = {{&S103547[4], 5}, {NULL}};
const Segment SGI328338[] = {{&S100117[4], 4}, {NULL}};
const Segment SGI328339[] = {{&S103612[4], 5}, {NULL}};
const Segment SGI328340[] = {{&S103613[4], 6}, {NULL}};
const Segment SGI328341[] = {{&S96672[30], 4}, {NULL}};
const Segment SGI328342[] = {{&S96672[8], 3}, {NULL}};
const Segment SGI328343[] = {{&S96672[13], 10}, {NULL}};
const Segment SGI328344[] = {{&S96672[43], 5}, {NULL}};
const Segment SGI328345[] = {{&S96737[30], 3}, {NULL}};
const Segment SGI328346[] = {{&S96737[8], 3}, {NULL}};
const Segment SGI328347[] = {{&S96737[13], 10}, {NULL}};
const Segment SGI328348[] = {{&S96737[42], 7}, {NULL}};
const Segment SGI328349[] = {{&S96737[35], 14}, {NULL}};
const Segment SGI328350[] = {{&S96737[51], 3}, {NULL}};
const Segment SGI328351[] = {{&S96737[69], 5}, {NULL}};
const Segment SGI328352[] = {{&S96737[96], 9}, {NULL}};
const Segment SGI328353[] = {{&S96773[30], 3}, {NULL}};
const Segment SGI328354[] = {{&S96773[8], 3}, {NULL}};
const Segment SGI328355[] = {{&S96773[13], 10}, {NULL}};
const Segment SGI328356[] = {{&S96773[42], 7}, {NULL}};
const Segment SGI328357[] = {{&S96773[35], 14}, {NULL}};
const Segment SGI328358[] = {{&S96773[203], 11}, {NULL}};
const Segment SGI328359[] = {{&S96773[91], 15}, {NULL}};
const Segment SGI328360[] = {{&S96773[203], 15}, {NULL}};
const Segment SGI328361[] = {{&S103885[4], 6}, {NULL}};
const Segment SGI328362[] = {{&S103885[26], 3}, {NULL}};
const Segment SGI328363[] = {{&S103912[4], 6}, {NULL}};
const Segment SGI328364[] = {{&S103912[26], 3}, {NULL}};
const Segment SGI328365[] = {{&S104189[4], 6}, {NULL}};
const Segment SGI328366[] = {{&S104189[26], 3}, {NULL}};
const Segment SGI328367[] = {{&S104196[4], 5}, {NULL}};
const Segment SGI328368[] = {{&S100723[4], 4}, {NULL}};
const Segment SGI328369[] = {{&S100723[27], 8}, {NULL}};
const Segment SGI328370[] = {{&S100723[40], 5}, {NULL}};
const Segment SGI328371[] = {{&S104200[4], 5}, {NULL}};
const Segment SGI328372[] = {{&S104200[25], 3}, {NULL}};
const Segment SGI328373[] = {{&S100740[4], 5}, {NULL}};
const Segment SGI328374[] = {{&S101150[4], 13}, {NULL}};
const Segment SGI328375[] = {{&S101151[4], 4}, {NULL}};
const Segment SGI328376[] = {{&S101154[4], 7}, {NULL}};
const Segment SGI328377[] = {{&S101154[41], 52}, {NULL}};
const Segment SGI328378[] = {{&S101281[4], 4}, {NULL}};
const Segment SGI328379[] = {{&S101282[4], 5}, {NULL}};
const Segment SGI328380[] = {{&S101288[4], 3}, {NULL}};
const Segment SGI328381[] = {{&S101288[25], 8}, {NULL}};
const Segment SGI328382[] = {{&S101288[41], 5}, {NULL}};
const Segment SGI328383[] = {{&S101302[4], 4}, {NULL}};
const Segment SGI328384[] = {{&S101306[4], 4}, {NULL}};
const Segment SGI328385[] = {{&S101306[27], 8}, {NULL}};
const Segment SGI328386[] = {{&S101306[40], 5}, {NULL}};
const Segment SGI328387[] = {{&S97684[30], 3}, {NULL}};
const Segment SGI328388[] = {{&S97684[8], 3}, {NULL}};
const Segment SGI328389[] = {{&S97684[13], 10}, {NULL}};
const Segment SGI328390[] = {{&S97684[42], 7}, {NULL}};
const Segment SGI328391[] = {{&S97684[35], 14}, {NULL}};
const Segment SGI328392[] = {{&S97684[266], 4}, {NULL}};
const Segment SGI328393[] = {{&S101443[4], 4}, {NULL}};
const Segment SGI328394[] = {{&S101443[26], 8}, {NULL}};
const Segment SGI328395[] = {{&S101443[42], 5}, {NULL}};
const Segment SGI328396[] = {{&S101602[4], 5}, {NULL}};
const Segment SGI328397[] = {{&S101636[4], 4}, {NULL}};
const Segment SGI328398[] = {{&S101643[4], 4}, {NULL}};
const Segment SGI328399[] = {{&S101775[4], 5}, {NULL}};
const Segment SGI328400[] = {{&S101793[4], 4}, {NULL}};
const Segment SGI328401[] = {{&S101794[4], 7}, {NULL}};
const Segment SGI328402[] = {{&S101795[4], 4}, {NULL}};
const Segment SGI328403[] = {{&S101795[27], 8}, {NULL}};
const Segment SGI328404[] = {{&S98147[30], 3}, {NULL}};
const Segment SGI328405[] = {{&S98147[8], 3}, {NULL}};
const Segment SGI328406[] = {{&S98147[13], 10}, {NULL}};
const Segment SGI328407[] = {{&S98147[42], 7}, {NULL}};
const Segment SGI328408[] = {{&S98147[35], 14}, {NULL}};
const Segment SGI328409[] = {{&S98147[74], 3}, {NULL}};
const Segment SGI328410[] = {{&S105252[4], 5}, {NULL}};
const Segment SGI328411[] = {{&S105252[25], 3}, {NULL}};
const Segment SGI328412[] = {{&S105412[4], 5}, {NULL}};
const Segment SGI328413[] = {{&S105419[4], 5}, {NULL}};
const Segment SGI328414[] = {{&S105419[25], 3}, {NULL}};
const Segment SGI328415[] = {{&S105424[4], 5}, {NULL}};
const Segment SGI328416[] = {{&S105424[25], 3}, {NULL}};
const Segment SGI328417[] = {{&S102016[4], 4}, {NULL}};
const Segment SGI328418[] = {{&S102016[24], 8}, {NULL}};
const Segment SGI328419[] = {{&S102016[40], 5}, {NULL}};
const Segment SGI328420[] = {{&S102021[4], 4}, {NULL}};
const Segment SGI328421[] = {{&S102021[37], 49}, {NULL}};
const Segment SGI328422[] = {{&S102022[4], 4}, {NULL}};
const Segment SGI328423[] = {{&S102022[37], 50}, {NULL}};
const Segment SGI328424[] = {{&S102023[4], 4}, {NULL}};
const Segment SGI328425[] = {{&S102023[37], 37}, {NULL}};
const Segment SGI328426[] = {{&S102148[4], 3}, {NULL}};
const Segment SGI328427[] = {{&S102149[4], 2}, {NULL}};
const Segment SGI328428[] = {{&S102187[4], 5}, {NULL}};
const Segment SGI328429[] = {{&S102331[4], 14}, {NULL}};
const Segment SGI328430[] = {{&S102331[48], 32}, {NULL}};
const Segment SGI328431[] = {{&S102332[4], 5}, {NULL}};
const Segment SGI328432[] = {{&S102332[55], 51}, {NULL}};
const Segment SGI328433[] = {{&S102337[4], 4}, {NULL}};
const Segment SGI328434[] = {{&S102371[4], 4}, {NULL}};
const Segment SGI328435[] = {{&S102371[27], 8}, {NULL}};
const Segment SGI328436[] = {{&S102371[39], 5}, {NULL}};
const Segment SGI328437[] = {{&S98939[30], 4}, {NULL}};
const Segment SGI328438[] = {{&S98939[8], 3}, {NULL}};
const Segment SGI328439[] = {{&S98939[13], 10}, {NULL}};
const Segment SGI328440[] = {{&S98939[43], 7}, {NULL}};
const Segment SGI328441[] = {{&S98939[36], 14}, {NULL}};
const Segment SGI328442[] = {{&S98939[65], 4}, {&S98939[65], 4}, {NULL}};
const Segment SGI328443[] = {{&S102563[4], 4}, {NULL}};
const Segment SGI328444[] = {{&S102563[24], 11}, {NULL}};
const Segment SGI328445[] = {{&S102563[36], 5}, {NULL}};
const Segment SGI328446[] = {{&S102571[4], 5}, {NULL}};
const Segment SGI328447[] = {{&S102571[27], 8}, {NULL}};
const Segment SGI328448[] = {{&S102571[43], 5}, {NULL}};
const Segment SGI328449[] = {{&S99060[30], 3}, {NULL}};
const Segment SGI328450[] = {{&S99060[8], 3}, {NULL}};
const Segment SGI328451[] = {{&S99060[13], 10}, {NULL}};
const Segment SGI328452[] = {{&S99060[42], 7}, {NULL}};
const Segment SGI328453[] = {{&S99060[35], 14}, {NULL}};
const Segment SGI328454[] = {{&S99060[84], 4}, {NULL}};
const Segment SGI328455[] = {{&S99060[51], 3}, {NULL}};
const Segment SGI328456[] = {{&S102582[4], 4}, {NULL}};
const Segment SGI328457[] = {{&S99072[30], 3}, {NULL}};
const Segment SGI328458[] = {{&S99072[8], 3}, {NULL}};
const Segment SGI328459[] = {{&S99072[13], 10}, {NULL}};
const Segment SGI328460[] = {{&S99072[42], 7}, {NULL}};
const Segment SGI328461[] = {{&S99072[35], 14}, {NULL}};
const Segment SGI328462[] = {{&S99072[131], 15}, {NULL}};
const Segment SGI328463[] = {{&S99072[172], 3}, {NULL}};
const Segment SGI328464[] = {{&S99088[30], 4}, {NULL}};
const Segment SGI328465[] = {{&S99088[8], 3}, {NULL}};
const Segment SGI328466[] = {{&S99088[13], 10}, {NULL}};
const Segment SGI328467[] = {{&S99088[43], 7}, {NULL}};
const Segment SGI328468[] = {{&S99088[36], 14}, {NULL}};
const Segment SGI328469[] = {{&S99088[72], 11}, {NULL}};
const Segment SGI328470[] = {{&S99088[52], 6}, {NULL}};
const Segment SGI328471[] = {{&S99088[72], 16}, {NULL}};
const Segment SGI328472[] = {{&S99089[30], 4}, {NULL}};
const Segment SGI328473[] = {{&S99089[8], 3}, {NULL}};
const Segment SGI328474[] = {{&S99089[13], 10}, {NULL}};
const Segment SGI328475[] = {{&S99089[43], 7}, {NULL}};
const Segment SGI328476[] = {{&S99089[36], 14}, {NULL}};
const Segment SGI328477[] = {{&S99089[65], 11}, {NULL}};
const Segment SGI328478[] = {{&S99089[65], 16}, {NULL}};
const Segment SGI328479[] = {{&S102744[4], 4}, {NULL}};
const Segment SGI328480[] = {{&S102760[4], 5}, {NULL}};
const Segment SGI328481[] = {{&S102760[27], 8}, {NULL}};
const Segment SGI328482[] = {{&S102760[43], 5}, {NULL}};
const Segment SGI328483[] = {{&S102797[4], 5}, {NULL}};
const Segment SGI328484[] = {{&S102960[4], 3}, {NULL}};
const Segment SGI328485[] = {{&S102963[4], 5}, {NULL}};
const Segment SGI328486[] = {{&S102963[40], 32}, {NULL}};
const Segment SGI328487[] = {{&S102970[4], 3}, {NULL}};
const Segment SGI328488[] = {{&S102973[4], 4}, {NULL}};
const Segment SGI328489[] = {{&S102979[4], 4}, {NULL}};
const Segment SGI328490[] = {{&S103001[4], 5}, {NULL}};
const Segment SGI328491[] = {{&S103001[27], 8}, {NULL}};
const Segment SGI328492[] = {{&S103001[43], 5}, {NULL}};
const Segment SGI328493[] = {{&S103020[4], 4}, {NULL}};
const Segment SGI328494[] = {{&S103020[27], 8}, {NULL}};
const Segment SGI328495[] = {{&S103020[40], 5}, {NULL}};
const Segment SGI328496[] = {{&S103037[4], 4}, {NULL}};
const Segment SGI328497[] = {{&S103053[4], 5}, {NULL}};
const Segment SGI328498[] = {{&S103053[27], 8}, {NULL}};
const Segment SGI328499[] = {{&S103053[43], 5}, {NULL}};
const Segment SGI328500[] = {{&S99581[30], 3}, {NULL}};
const Segment SGI328501[] = {{&S99581[8], 3}, {NULL}};
const Segment SGI328502[] = {{&S99581[13], 10}, {NULL}};
const Segment SGI328503[] = {{&S99581[42], 7}, {NULL}};
const Segment SGI328504[] = {{&S99581[35], 14}, {NULL}};
const Segment SGI328505[] = {{&S99581[51], 3}, {NULL}};
const Segment SGI328506[] = {{&S99581[66], 3}, {NULL}};
const Segment SGI328507[] = {{&S99583[30], 3}, {NULL}};
const Segment SGI328508[] = {{&S99583[8], 3}, {NULL}};
const Segment SGI328509[] = {{&S99583[13], 10}, {NULL}};
const Segment SGI328510[] = {{&S99583[42], 7}, {NULL}};
const Segment SGI328511[] = {{&S99583[35], 14}, {NULL}};
const Segment SGI328512[] = {{&S99583[74], 3}, {NULL}};
const Segment SGI328513[] = {{&S103229[4], 15}, {NULL}};
const Segment SGI328514[] = {{&S103229[20], 8}, {NULL}};
const Segment SGI328515[] = {{&S103229[32], 5}, {NULL}};
const Segment SGI328516[] = {{&S103230[4], 4}, {NULL}};
const Segment SGI328517[] = {{&S103230[27], 8}, {NULL}};
const Segment SGI328518[] = {{&S103230[39], 5}, {NULL}};
const Segment SGI328519[] = {{&S99640[30], 3}, {NULL}};
const Segment SGI328520[] = {{&S99640[8], 3}, {NULL}};
const Segment SGI328521[] = {{&S99640[13], 10}, {NULL}};
const Segment SGI328522[] = {{&S99640[42], 7}, {NULL}};
const Segment SGI328523[] = {{&S99640[35], 14}, {NULL}};
const Segment SGI328524[] = {{&S103246[4], 8}, {NULL}};
const Segment SGI328525[] = {{&S103246[42], 28}, {NULL}};
const Segment SGI328526[] = {{&S103247[4], 8}, {NULL}};
const Segment SGI328527[] = {{&S103247[42], 28}, {NULL}};
const Segment SGI328528[] = {{&S103261[4], 5}, {NULL}};
const Segment SGI328529[] = {{&S103261[27], 8}, {NULL}};
const Segment SGI328530[] = {{&S103261[43], 5}, {NULL}};
const Segment SGI328531[] = {{&S103265[4], 4}, {NULL}};
const Segment SGI328532[] = {{&S103265[27], 8}, {NULL}};
const Segment SGI328533[] = {{&S103304[4], 5}, {NULL}};
const Segment SGI328534[] = {{&S103304[27], 8}, {NULL}};
const Segment SGI328535[] = {{&S103304[43], 5}, {NULL}};
const Segment SGI328536[] = {{&S103521[4], 4}, {NULL}};
const Segment SGI328537[] = {{&S103527[4], 4}, {NULL}};
const Segment SGI328538[] = {{&S103568[4], 10}, {NULL}};
const Segment SGI328539[] = {{&S103568[93], 49}, {NULL}};
const Segment SGI328540[] = {{&S103589[4], 4}, {NULL}};
const Segment SGI328541[] = {{&S103606[4], 4}, {NULL}};
const Segment SGI328542[] = {{&S103618[4], 10}, {NULL}};
const Segment SGI328543[] = {{&S103619[4], 5}, {NULL}};
const Segment SGI328544[] = {{&S103621[4], 15}, {NULL}};
const Segment SGI328545[] = {{&S103621[20], 7}, {NULL}};
const Segment SGI328546[] = {{&S103621[28], 5}, {NULL}};
const Segment SGI328547[] = {{&S103840[4], 4}, {NULL}};
const Segment SGI328548[] = {{&S103873[4], 4}, {NULL}};
const Segment SGI328549[] = {{&S103883[4], 11}, {NULL}};
const Segment SGI328550[] = {{&S103883[28], 39}, {NULL}};
const Segment SGI328551[] = {{&S103915[4], 5}, {NULL}};
const Segment SGI328552[] = {{&S104167[4], 4}, {NULL}};
const Segment SGI328553[] = {{&S104252[4], 4}, {NULL}};
const Segment SGI328554[] = {{&S104252[26], 11}, {NULL}};
const Segment SGI328555[] = {{&S104252[38], 5}, {NULL}};
const Segment SGI328556[] = {{&S104262[4], 5}, {NULL}};
const Segment SGI328557[] = {{&S100611[30], 3}, {NULL}};
const Segment SGI328558[] = {{&S100611[8], 3}, {NULL}};
const Segment SGI328559[] = {{&S100611[13], 10}, {NULL}};
const Segment SGI328560[] = {{&S100611[42], 7}, {NULL}};
const Segment SGI328561[] = {{&S100611[35], 14}, {NULL}};
const Segment SGI328562[] = {{&S100611[51], 6}, {NULL}};
const Segment SGI328563[] = {{&S100611[108], 4}, {NULL}};
const Segment SGI328564[] = {{&S104576[4], 13}, {NULL}};
const Segment SGI328565[] = {{&S104585[4], 15}, {NULL}};
const Segment SGI328566[] = {{&S104599[4], 4}, {NULL}};
const Segment SGI328567[] = {{&S104599[26], 8}, {NULL}};
const Segment SGI328568[] = {{&S104599[42], 5}, {NULL}};
const Segment SGI328569[] = {{&S104603[4], 4}, {NULL}};
const Segment SGI328570[] = {{&S104658[4], 5}, {NULL}};
const Segment SGI328571[] = {{&S104658[27], 8}, {NULL}};
const Segment SGI328572[] = {{&S104658[43], 5}, {NULL}};
const Segment SGI328573[] = {{&S104681[4], 4}, {NULL}};
const Segment SGI328574[] = {{&S104683[4], 4}, {NULL}};
const Segment SGI328575[] = {{&S104683[24], 11}, {NULL}};
const Segment SGI328576[] = {{&S104683[36], 5}, {NULL}};
const Segment SGI328577[] = {{&S104709[4], 15}, {NULL}};
const Segment SGI328578[] = {{&S104709[20], 7}, {NULL}};
const Segment SGI328579[] = {{&S104709[28], 5}, {NULL}};
const Segment SGI328580[] = {{&S105095[4], 10}, {NULL}};
const Segment SGI328581[] = {{&S105095[103], 49}, {NULL}};
const Segment SGI328582[] = {{&S105101[4], 9}, {NULL}};
const Segment SGI328583[] = {{&S105101[8], 5}, {NULL}};
const Segment SGI328584[] = {{&S105111[4], 2}, {NULL}};
const Segment SGI328585[] = {{&S105128[4], 3}, {NULL}};
const Segment SGI328586[] = {{&S105128[30], 49}, {NULL}};
const Segment SGI328587[] = {{&S105166[4], 9}, {NULL}};
const Segment SGI328588[] = {{&S105167[4], 3}, {NULL}};
const Segment SGI328589[] = {{&S105167[41], 41}, {NULL}};
const Segment SGI328590[] = {{&S105201[4], 6}, {NULL}};
const Segment SGI328591[] = {{&S105201[43], 32}, {NULL}};
const Segment SGI328592[] = {{&S105206[4], 4}, {NULL}};
const Segment SGI328593[] = {{&S105218[4], 4}, {NULL}};
const Segment SGI328594[] = {{&S105302[4], 4}, {NULL}};
const Segment SGI328595[] = {{&S101840[30], 3}, {NULL}};
const Segment SGI328596[] = {{&S101840[8], 3}, {NULL}};
const Segment SGI328597[] = {{&S101840[13], 10}, {NULL}};
const Segment SGI328598[] = {{&S101840[42], 7}, {NULL}};
const Segment SGI328599[] = {{&S101840[35], 14}, {NULL}};
const Segment SGI328600[] = {{&S101840[51], 3}, {NULL}};
const Segment SGI328601[] = {{&S101840[66], 3}, {NULL}};
const Segment SGI328602[] = {{&S105372[4], 4}, {NULL}};
const Segment SGI328603[] = {{&S105413[4], 7}, {NULL}};
const Segment SGI328604[] = {{&S105413[90], 48}, {NULL}};
const Segment SGI328605[] = {{&S102040[30], 3}, {NULL}};
const Segment SGI328606[] = {{&S102040[8], 3}, {NULL}};
const Segment SGI328607[] = {{&S102040[13], 10}, {NULL}};
const Segment SGI328608[] = {{&S102040[42], 7}, {NULL}};
const Segment SGI328609[] = {{&S102040[35], 14}, {NULL}};
const Segment SGI328610[] = {{&S102040[73], 6}, {NULL}};
const Segment SGI328611[] = {{&S102041[30], 3}, {NULL}};
const Segment SGI328612[] = {{&S102041[8], 3}, {NULL}};
const Segment SGI328613[] = {{&S102041[13], 10}, {NULL}};
const Segment SGI328614[] = {{&S102041[42], 7}, {NULL}};
const Segment SGI328615[] = {{&S102041[35], 14}, {NULL}};
const Segment SGI328616[] = {{&S102041[73], 6}, {NULL}};
const Segment SGI328617[] = {{&S102042[30], 3}, {NULL}};
const Segment SGI328618[] = {{&S102042[8], 3}, {NULL}};
const Segment SGI328619[] = {{&S102042[13], 10}, {NULL}};
const Segment SGI328620[] = {{&S102042[42], 7}, {NULL}};
const Segment SGI328621[] = {{&S102042[35], 14}, {NULL}};
const Segment SGI328622[] = {{&S102042[80], 6}, {NULL}};
const Segment SGI328623[] = {{&S102042[51], 6}, {NULL}};
const Segment SGI328624[] = {{&S102431[30], 4}, {NULL}};
const Segment SGI328625[] = {{&S102431[8], 3}, {NULL}};
const Segment SGI328626[] = {{&S102431[13], 10}, {NULL}};
const Segment SGI328627[] = {{&S102431[43], 7}, {NULL}};
const Segment SGI328628[] = {{&S102431[36], 14}, {NULL}};
const Segment SGI328629[] = {{&S102431[72], 11}, {NULL}};
const Segment SGI328630[] = {{&S102431[52], 6}, {NULL}};
const Segment SGI328631[] = {{&S102431[72], 16}, {NULL}};
const Segment SGI328632[] = {{&S102462[30], 3}, {NULL}};
const Segment SGI328633[] = {{&S102462[8], 3}, {NULL}};
const Segment SGI328634[] = {{&S102462[13], 10}, {NULL}};
const Segment SGI328635[] = {{&S102462[42], 7}, {NULL}};
const Segment SGI328636[] = {{&S102462[35], 14}, {NULL}};
const Segment SGI328637[] = {{&S102462[64], 11}, {NULL}};
const Segment SGI328638[] = {{&S102462[64], 15}, {NULL}};
const Segment SGI328639[] = {{&S102674[30], 3}, {NULL}};
const Segment SGI328640[] = {{&S102674[8], 3}, {NULL}};
const Segment SGI328641[] = {{&S102674[13], 10}, {NULL}};
const Segment SGI328642[] = {{&S102674[42], 7}, {NULL}};
const Segment SGI328643[] = {{&S102674[35], 14}, {NULL}};
const Segment SGI328644[] = {{&S102674[51], 3}, {NULL}};
const Segment SGI328645[] = {{&S102674[66], 3}, {NULL}};
const Segment SGI328646[] = {{&S102834[30], 3}, {NULL}};
const Segment SGI328647[] = {{&S102834[8], 3}, {NULL}};
const Segment SGI328648[] = {{&S102834[13], 10}, {NULL}};
const Segment SGI328649[] = {{&S102834[42], 7}, {NULL}};
const Segment SGI328650[] = {{&S102834[35], 14}, {NULL}};
const Segment SGI328651[] = {{&S102834[138], 11}, {NULL}};
const Segment SGI328652[] = {{&S102834[51], 6}, {NULL}};
const Segment SGI328653[] = {{&S102834[138], 15}, {NULL}};
const Segment SGI328654[] = {{&S102835[30], 3}, {NULL}};
const Segment SGI328655[] = {{&S102835[8], 3}, {NULL}};
const Segment SGI328656[] = {{&S102835[13], 10}, {NULL}};
const Segment SGI328657[] = {{&S102835[42], 7}, {NULL}};
const Segment SGI328658[] = {{&S102835[35], 14}, {NULL}};
const Segment SGI328659[] = {{&S102835[64], 11}, {NULL}};
const Segment SGI328660[] = {{&S102835[64], 15}, {NULL}};
const Segment SGI328661[] = {{&S102840[30], 4}, {NULL}};
const Segment SGI328662[] = {{&S102840[8], 3}, {NULL}};
const Segment SGI328663[] = {{&S102840[13], 10}, {NULL}};
const Segment SGI328664[] = {{&S102840[43], 5}, {NULL}};
const Segment SGI328665[] = {{&S102841[30], 4}, {NULL}};
const Segment SGI328666[] = {{&S102841[8], 3}, {NULL}};
const Segment SGI328667[] = {{&S102841[13], 10}, {NULL}};
const Segment SGI328668[] = {{&S102841[43], 5}, {NULL}};
const Segment SGI328669[] = {{&S102853[30], 4}, {NULL}};
const Segment SGI328670[] = {{&S102853[8], 3}, {NULL}};
const Segment SGI328671[] = {{&S102853[13], 10}, {NULL}};
const Segment SGI328672[] = {{&S102853[43], 7}, {NULL}};
const Segment SGI328673[] = {{&S102853[36], 14}, {NULL}};
const Segment SGI328674[] = {{&S102853[72], 11}, {NULL}};
const Segment SGI328675[] = {{&S102853[52], 6}, {NULL}};
const Segment SGI328676[] = {{&S102853[72], 16}, {NULL}};
const Segment SGI328677[] = {{&S102854[30], 4}, {NULL}};
const Segment SGI328678[] = {{&S102854[8], 3}, {NULL}};
const Segment SGI328679[] = {{&S102854[13], 10}, {NULL}};
const Segment SGI328680[] = {{&S102854[43], 7}, {NULL}};
const Segment SGI328681[] = {{&S102854[36], 14}, {NULL}};
const Segment SGI328682[] = {{&S102854[65], 11}, {NULL}};
const Segment SGI328683[] = {{&S102854[65], 16}, {NULL}};
const Segment SGI328684[] = {{&S103978[30], 4}, {NULL}};
const Segment SGI328685[] = {{&S103978[8], 3}, {NULL}};
const Segment SGI328686[] = {{&S103978[13], 10}, {NULL}};
const Segment SGI328687[] = {{&S103978[43], 7}, {NULL}};
const Segment SGI328688[] = {{&S103978[36], 14}, {NULL}};
const Segment SGI328689[] = {{&S103978[72], 11}, {NULL}};
const Segment SGI328690[] = {{&S103978[52], 6}, {NULL}};
const Segment SGI328691[] = {{&S103978[72], 16}, {NULL}};
const Segment SGI328692[] = {{&S103979[30], 4}, {NULL}};
const Segment SGI328693[] = {{&S103979[8], 3}, {NULL}};
const Segment SGI328694[] = {{&S103979[13], 10}, {NULL}};
const Segment SGI328695[] = {{&S103979[43], 7}, {NULL}};
const Segment SGI328696[] = {{&S103979[36], 14}, {NULL}};
const Segment SGI328697[] = {{&S103979[65], 11}, {NULL}};
const Segment SGI328698[] = {{&S103979[65], 16}, {NULL}};
const Segment SGI328699[] = {{&S26843[5], 5}, {NULL}};
const Segment SGI328700[] = {{&S26846[5], 7}, {NULL}};
const Segment SGI328701[] = {{&S26849[39], 34}, {NULL}};
const Segment SGI328702[] = {{&S26850[101], 5}, {NULL}};
const Segment SGI328703[] = {{&S28021[5], 13}, {&S28021[44], 5}, {NULL}};
const Segment SGI328704[] = {{&S28029[5], 3}, {NULL}};
const Segment SGI328705[] = {{&S28030[30], 34}, {NULL}};
const Segment SGI328706[] = {{&S28031[5], 5}, {NULL}};
const Segment SGI328707[] = {{&S28579[5], 5}, {NULL}};
const Segment SGI328708[] = {{&S28580[5], 7}, {NULL}};
const Segment SGI328709[] = {{&S28581[5], 5}, {&S28581[68], 5}, {NULL}};
const Segment SGI328710[] = {{&S28582[5], 5}, {&S28582[5], 5}, {NULL}};
const Segment SGI328711[] = {{&S2758[75], 3}, {NULL}};
const Segment SGI328712[] = {{&S2758[0], 4}, {NULL}};
const Segment SGI328713[] = {{&S2759[0], 21}, {NULL}};
const Segment SGI328714[] = {{&S2759[34], 3}, {NULL}};
const Segment SGI328715[] = {{&S2760[75], 3}, {NULL}};
const Segment SGI328716[] = {{&S2760[0], 4}, {NULL}};
const Segment SGI328717[] = {{&S2761[0], 21}, {NULL}};
const Segment SGI328718[] = {{&S2761[34], 3}, {NULL}};
const Segment SGI328719[] = {{&S2761[73], 13}, {NULL}};
const Segment SGI328720[] = {{&S2762[0], 21}, {NULL}};
const Segment SGI328721[] = {{&S2762[34], 3}, {NULL}};
const Segment SGI328722[] = {{&S2763[0], 21}, {NULL}};
const Segment SGI328723[] = {{&S2763[34], 3}, {NULL}};
const Segment SGI328724[] = {{&S2764[0], 23}, {NULL}};
const Segment SGI328725[] = {{&S2764[36], 3}, {NULL}};
const Segment SGI328726[] = {{&S2764[77], 13}, {NULL}};
const Segment SGI328727[] = {{&S2765[0], 21}, {NULL}};
const Segment SGI328728[] = {{&S2765[34], 3}, {NULL}};
const Segment SGI328729[] = {{&S2765[75], 13}, {NULL}};
const Segment SGI328730[] = {{&S2766[0], 15}, {NULL}};
const Segment SGI328731[] = {{&S2766[28], 3}, {NULL}};
const Segment SGI328732[] = {{&S2766[68], 13}, {NULL}};
const Segment SGI328733[] = {{&S30170[5], 3}, {NULL}};
const Segment SGI328734[] = {{&S30170[18], 3}, {NULL}};
const Segment SGI328735[] = {{&S30171[5], 3}, {NULL}};
const Segment SGI328736[] = {{&S31170[100], 11}, {NULL}};
const Segment SGI328737[] = {{&S31171[100], 19}, {NULL}};
const Segment SGI328738[] = {{&S31172[100], 21}, {NULL}};
const Segment SGI328739[] = {{&S31173[100], 19}, {NULL}};
const Segment SGI328740[] = {{&S31174[100], 11}, {NULL}};
const Segment SGI328741[] = {{&S31175[100], 12}, {NULL}};
const Segment SGI328742[] = {{&S31176[100], 19}, {NULL}};
const Segment SGI328743[] = {{&S4699[0], 21}, {NULL}};
const Segment SGI328744[] = {{&S4699[34], 3}, {NULL}};
const Segment SGI328745[] = {{&S4700[0], 21}, {NULL}};
const Segment SGI328746[] = {{&S4700[34], 3}, {NULL}};
const Segment SGI328747[] = {{&S4701[0], 21}, {NULL}};
const Segment SGI328748[] = {{&S4701[34], 3}, {NULL}};
const Segment SGI328749[] = {{&S4702[0], 21}, {NULL}};
const Segment SGI328750[] = {{&S4702[34], 3}, {NULL}};
const Segment SGI328751[] = {{&S4702[73], 13}, {NULL}};
const Segment SGI328752[] = {{&S4703[0], 21}, {NULL}};
const Segment SGI328753[] = {{&S4703[34], 3}, {NULL}};
const Segment SGI328754[] = {{&S4703[75], 13}, {NULL}};
const Segment SGI328755[] = {{&S4866[0], 21}, {NULL}};
const Segment SGI328756[] = {{&S4866[34], 3}, {NULL}};
const Segment SGI328757[] = {{&S4866[75], 13}, {NULL}};
const Segment SGI328758[] = {{&S4867[0], 21}, {NULL}};
const Segment SGI328759[] = {{&S4867[34], 3}, {NULL}};
const Segment SGI328760[] = {{&S4867[75], 13}, {NULL}};
const Segment SGI328761[] = {{&S4868[0], 21}, {NULL}};
const Segment SGI328762[] = {{&S4868[34], 3}, {NULL}};
const Segment SGI328763[] = {{&S4869[0], 30}, {NULL}};
const Segment SGI328764[] = {{&S4869[43], 3}, {NULL}};
const Segment SGI328765[] = {{&S4869[84], 13}, {NULL}};
const Segment SGI328766[] = {{&S4929[0], 21}, {NULL}};
const Segment SGI328767[] = {{&S4929[34], 3}, {NULL}};
const Segment SGI328768[] = {{&S4929[75], 13}, {NULL}};
const Segment SGI328769[] = {{&S5212[0], 21}, {NULL}};
const Segment SGI328770[] = {{&S5212[34], 3}, {NULL}};
const Segment SGI328771[] = {{&S5212[75], 13}, {NULL}};
const Segment SGI328772[] = {{&S5213[0], 21}, {NULL}};
const Segment SGI328773[] = {{&S5213[34], 3}, {NULL}};
const Segment SGI328774[] = {{&S5532[0], 21}, {NULL}};
const Segment SGI328775[] = {{&S5532[34], 3}, {NULL}};
const Segment SGI328776[] = {{&S5532[75], 13}, {NULL}};
const Segment SGI328777[] = {{&S32145[5], 4}, {NULL}};
const Segment SGI328778[] = {{&S32145[19], 3}, {NULL}};
const Segment SGI328779[] = {{&S32146[5], 4}, {NULL}};
const Segment SGI328780[] = {{&S5787[0], 21}, {NULL}};
const Segment SGI328781[] = {{&S5787[34], 3}, {NULL}};
const Segment SGI328782[] = {{&S5787[75], 13}, {NULL}};
const Segment SGI328783[] = {{&S5788[0], 21}, {NULL}};
const Segment SGI328784[] = {{&S5788[34], 3}, {NULL}};
const Segment SGI328785[] = {{&S5789[0], 15}, {NULL}};
const Segment SGI328786[] = {{&S5789[28], 3}, {NULL}};
const Segment SGI328787[] = {{&S5789[69], 13}, {NULL}};
const Segment SGI328788[] = {{&S8698[61], 5}, {NULL}};
const Segment SGI328789[] = {{&S8698[13], 10}, {NULL}};
const Segment SGI328790[] = {{&S8650[6], 4}, {NULL}};
const Segment SGI328791[] = {{&S8650[72], 5}, {NULL}};
const Segment SGI328792[] = {{&S8650[12], 14}, {NULL}};
const Segment SGI328793[] = {{&S8651[6], 4}, {NULL}};
const Segment SGI328794[] = {{&S8651[81], 5}, {NULL}};
const Segment SGI328795[] = {{&S8651[12], 14}, {&S8651[40], 11}, {NULL}};
const Segment SGI328796[] = {{&S8651[12], 14}, {NULL}};
const Segment SGI328797[] = {{&S8652[6], 4}, {NULL}};
const Segment SGI328798[] = {{&S8652[59], 5}, {NULL}};
const Segment SGI328799[] = {{&S8652[12], 14}, {&S8652[70], 31}, {NULL}};
const Segment SGI328800[] = {{&S8652[12], 14}, {NULL}};
const Segment SGI328801[] = {{&S8653[6], 4}, {NULL}};
const Segment SGI328802[] = {{&S8653[89], 5}, {NULL}};
const Segment SGI328803[] = {{&S8653[12], 14}, {&S8653[48], 11}, {NULL}};
const Segment SGI328804[] = {{&S8653[12], 14}, {NULL}};
const Segment SGI328805[] = {{&S8654[6], 4}, {NULL}};
const Segment SGI328806[] = {{&S8654[82], 5}, {NULL}};
const Segment SGI328807[] = {{&S8654[12], 14}, {&S8654[41], 11}, {NULL}};
const Segment SGI328808[] = {{&S8654[12], 14}, {NULL}};
const Segment SGI328809[] = {{&S8655[6], 4}, {NULL}};
const Segment SGI328810[] = {{&S8655[81], 5}, {NULL}};
const Segment SGI328811[] = {{&S8655[12], 14}, {&S8655[40], 11}, {NULL}};
const Segment SGI328812[] = {{&S8655[12], 14}, {NULL}};
const Segment SGI328813[] = {{&S8656[6], 4}, {NULL}};
const Segment SGI328814[] = {{&S8656[89], 5}, {NULL}};
const Segment SGI328815[] = {{&S8656[12], 14}, {&S8656[48], 11}, {NULL}};
const Segment SGI328816[] = {{&S8656[12], 14}, {NULL}};
const Segment SGI328817[] = {{&S8657[6], 4}, {NULL}};
const Segment SGI328818[] = {{&S8657[88], 5}, {NULL}};
const Segment SGI328819[] = {{&S8657[12], 14}, {&S8657[47], 11}, {NULL}};
const Segment SGI328820[] = {{&S8657[12], 14}, {NULL}};
const Segment SGI328821[] = {{&S8658[6], 4}, {NULL}};
const Segment SGI328822[] = {{&S8658[88], 5}, {NULL}};
const Segment SGI328823[] = {{&S8658[12], 14}, {&S8658[47], 11}, {NULL}};
const Segment SGI328824[] = {{&S8658[12], 14}, {NULL}};
const Segment SGI328825[] = {{&S8659[6], 4}, {NULL}};
const Segment SGI328826[] = {{&S8659[81], 5}, {NULL}};
const Segment SGI328827[] = {{&S8659[12], 14}, {&S8659[40], 11}, {NULL}};
const Segment SGI328828[] = {{&S8659[12], 14}, {NULL}};
const Segment SGI328829[] = {{&S8660[6], 4}, {NULL}};
const Segment SGI328830[] = {{&S8660[81], 5}, {NULL}};
const Segment SGI328831[] = {{&S8660[12], 14}, {&S8660[40], 11}, {NULL}};
const Segment SGI328832[] = {{&S8660[12], 14}, {NULL}};
const Segment SGI328833[] = {{&S8661[6], 4}, {NULL}};
const Segment SGI328834[] = {{&S8661[82], 5}, {NULL}};
const Segment SGI328835[] = {{&S8661[12], 14}, {&S8661[40], 12}, {NULL}};
const Segment SGI328836[] = {{&S8661[12], 14}, {NULL}};
const Segment SGI328837[] = {{&S8662[6], 4}, {NULL}};
const Segment SGI328838[] = {{&S8662[88], 5}, {NULL}};
const Segment SGI328839[] = {{&S8662[12], 14}, {&S8662[47], 11}, {NULL}};
const Segment SGI328840[] = {{&S8662[12], 14}, {NULL}};
const Segment SGI328841[] = {{&S8663[6], 4}, {NULL}};
const Segment SGI328842[] = {{&S8663[81], 5}, {NULL}};
const Segment SGI328843[] = {{&S8663[12], 14}, {&S8663[40], 11}, {NULL}};
const Segment SGI328844[] = {{&S8663[12], 14}, {NULL}};
const Segment SGI328845[] = {{&S8664[6], 4}, {NULL}};
const Segment SGI328846[] = {{&S8664[82], 5}, {NULL}};
const Segment SGI328847[] = {{&S8664[12], 14}, {&S8664[40], 12}, {NULL}};
const Segment SGI328848[] = {{&S8664[12], 14}, {NULL}};
const Segment SGI328849[] = {{&S8665[6], 4}, {NULL}};
const Segment SGI328850[] = {{&S8665[88], 5}, {NULL}};
const Segment SGI328851[] = {{&S8665[12], 14}, {&S8665[47], 11}, {NULL}};
const Segment SGI328852[] = {{&S8665[12], 14}, {NULL}};
const Segment SGI328853[] = {{&S8666[6], 4}, {NULL}};
const Segment SGI328854[] = {{&S8666[81], 5}, {NULL}};
const Segment SGI328855[] = {{&S8666[12], 14}, {&S8666[40], 11}, {NULL}};
const Segment SGI328856[] = {{&S8666[12], 14}, {NULL}};
const Segment SGI328857[] = {{&S8667[6], 4}, {NULL}};
const Segment SGI328858[] = {{&S8667[81], 5}, {NULL}};
const Segment SGI328859[] = {{&S8667[12], 14}, {&S8667[40], 11}, {NULL}};
const Segment SGI328860[] = {{&S8667[12], 14}, {NULL}};
const Segment SGI328861[] = {{&S8668[6], 4}, {NULL}};
const Segment SGI328862[] = {{&S8668[82], 5}, {NULL}};
const Segment SGI328863[] = {{&S8668[12], 14}, {&S8668[41], 11}, {NULL}};
const Segment SGI328864[] = {{&S8668[12], 14}, {NULL}};
const Segment SGI328865[] = {{&S8669[6], 4}, {NULL}};
const Segment SGI328866[] = {{&S8669[82], 5}, {NULL}};
const Segment SGI328867[] = {{&S8669[12], 14}, {&S8669[40], 12}, {NULL}};
const Segment SGI328868[] = {{&S8669[12], 14}, {NULL}};
const Segment SGI328869[] = {{&S8670[6], 4}, {NULL}};
const Segment SGI328870[] = {{&S8670[89], 5}, {NULL}};
const Segment SGI328871[] = {{&S8670[12], 14}, {&S8670[48], 11}, {NULL}};
const Segment SGI328872[] = {{&S8670[12], 14}, {NULL}};
const Segment SGI328873[] = {{&S8671[6], 4}, {NULL}};
const Segment SGI328874[] = {{&S8671[82], 5}, {NULL}};
const Segment SGI328875[] = {{&S8671[12], 14}, {&S8671[41], 11}, {NULL}};
const Segment SGI328876[] = {{&S8671[12], 14}, {NULL}};
const Segment SGI328877[] = {{&S8672[6], 4}, {NULL}};
const Segment SGI328878[] = {{&S8672[82], 5}, {NULL}};
const Segment SGI328879[] = {{&S8672[12], 14}, {&S8672[41], 11}, {NULL}};
const Segment SGI328880[] = {{&S8672[12], 14}, {NULL}};
const Segment SGI328881[] = {{&S8673[6], 4}, {NULL}};
const Segment SGI328882[] = {{&S8673[81], 5}, {NULL}};
const Segment SGI328883[] = {{&S8673[12], 14}, {&S8673[40], 11}, {NULL}};
const Segment SGI328884[] = {{&S8673[12], 14}, {NULL}};
const Segment SGI328885[] = {{&S8674[6], 4}, {NULL}};
const Segment SGI328886[] = {{&S8674[82], 5}, {NULL}};
const Segment SGI328887[] = {{&S8674[12], 14}, {&S8674[41], 11}, {NULL}};
const Segment SGI328888[] = {{&S8674[12], 14}, {NULL}};
const Segment SGI328889[] = {{&S8675[6], 4}, {NULL}};
const Segment SGI328890[] = {{&S8675[82], 5}, {NULL}};
const Segment SGI328891[] = {{&S8675[12], 14}, {&S8675[40], 12}, {NULL}};
const Segment SGI328892[] = {{&S8675[12], 14}, {NULL}};
const Segment SGI328893[] = {{&S8676[6], 4}, {NULL}};
const Segment SGI328894[] = {{&S8676[82], 5}, {NULL}};
const Segment SGI328895[] = {{&S8676[12], 14}, {&S8676[41], 11}, {NULL}};
const Segment SGI328896[] = {{&S8676[12], 14}, {NULL}};
const Segment SGI328897[] = {{&S8677[6], 4}, {NULL}};
const Segment SGI328898[] = {{&S8677[88], 5}, {NULL}};
const Segment SGI328899[] = {{&S8677[12], 14}, {&S8677[47], 11}, {NULL}};
const Segment SGI328900[] = {{&S8677[12], 14}, {NULL}};
const Segment SGI328901[] = {{&S8678[6], 4}, {NULL}};
const Segment SGI328902[] = {{&S8678[81], 5}, {NULL}};
const Segment SGI328903[] = {{&S8678[12], 14}, {&S8678[40], 11}, {NULL}};
const Segment SGI328904[] = {{&S8678[12], 14}, {NULL}};
const Segment SGI328905[] = {{&S8679[6], 4}, {NULL}};
const Segment SGI328906[] = {{&S8679[82], 5}, {NULL}};
const Segment SGI328907[] = {{&S8679[12], 14}, {&S8679[41], 11}, {NULL}};
const Segment SGI328908[] = {{&S8679[12], 14}, {NULL}};
const Segment SGI328909[] = {{&S8680[6], 4}, {NULL}};
const Segment SGI328910[] = {{&S8680[82], 5}, {NULL}};
const Segment SGI328911[] = {{&S8680[12], 14}, {&S8680[40], 12}, {NULL}};
const Segment SGI328912[] = {{&S8680[12], 14}, {NULL}};
const Segment SGI328913[] = {{&S8681[6], 4}, {NULL}};
const Segment SGI328914[] = {{&S8681[81], 5}, {NULL}};
const Segment SGI328915[] = {{&S8681[12], 14}, {&S8681[40], 11}, {NULL}};
const Segment SGI328916[] = {{&S8681[12], 14}, {NULL}};
const Segment SGI328917[] = {{&S8682[6], 4}, {NULL}};
const Segment SGI328918[] = {{&S8682[81], 5}, {NULL}};
const Segment SGI328919[] = {{&S8682[12], 14}, {&S8682[40], 11}, {NULL}};
const Segment SGI328920[] = {{&S8682[12], 14}, {NULL}};
const Segment SGI328921[] = {{&S8683[6], 4}, {NULL}};
const Segment SGI328922[] = {{&S8683[81], 5}, {NULL}};
const Segment SGI328923[] = {{&S8683[12], 14}, {&S8683[40], 11}, {NULL}};
const Segment SGI328924[] = {{&S8683[12], 14}, {NULL}};
const Segment SGI328925[] = {{&S8684[6], 4}, {NULL}};
const Segment SGI328926[] = {{&S8684[89], 5}, {NULL}};
const Segment SGI328927[] = {{&S8684[12], 14}, {&S8684[48], 11}, {NULL}};
const Segment SGI328928[] = {{&S8684[12], 14}, {NULL}};
const Segment SGI328929[] = {{&S8685[6], 4}, {NULL}};
const Segment SGI328930[] = {{&S8685[81], 5}, {NULL}};
const Segment SGI328931[] = {{&S8685[12], 14}, {&S8685[40], 11}, {NULL}};
const Segment SGI328932[] = {{&S8685[12], 14}, {NULL}};
const Segment SGI328933[] = {{&S8686[6], 4}, {NULL}};
const Segment SGI328934[] = {{&S8686[81], 5}, {NULL}};
const Segment SGI328935[] = {{&S8686[12], 14}, {&S8686[40], 11}, {NULL}};
const Segment SGI328936[] = {{&S8686[12], 14}, {NULL}};
const Segment SGI328937[] = {{&S8687[6], 4}, {NULL}};
const Segment SGI328938[] = {{&S8687[82], 5}, {NULL}};
const Segment SGI328939[] = {{&S8687[12], 14}, {&S8687[41], 11}, {NULL}};
const Segment SGI328940[] = {{&S8687[12], 14}, {NULL}};
const Segment SGI328941[] = {{&S8689[6], 4}, {NULL}};
const Segment SGI328942[] = {{&S8689[81], 5}, {NULL}};
const Segment SGI328943[] = {{&S8689[12], 14}, {&S8689[40], 11}, {NULL}};
const Segment SGI328944[] = {{&S8689[12], 14}, {NULL}};
const Segment SGI328945[] = {{&S8690[6], 4}, {NULL}};
const Segment SGI328946[] = {{&S8690[82], 5}, {NULL}};
const Segment SGI328947[] = {{&S8690[12], 14}, {&S8690[40], 12}, {NULL}};
const Segment SGI328948[] = {{&S8690[12], 14}, {NULL}};
const Segment SGI328949[] = {{&S8691[6], 4}, {NULL}};
const Segment SGI328950[] = {{&S8691[82], 5}, {NULL}};
const Segment SGI328951[] = {{&S8691[12], 14}, {&S8691[41], 11}, {NULL}};
const Segment SGI328952[] = {{&S8691[12], 14}, {NULL}};
const Segment SGI328953[] = {{&S8692[6], 4}, {NULL}};
const Segment SGI328954[] = {{&S8692[81], 5}, {NULL}};
const Segment SGI328955[] = {{&S8692[12], 14}, {&S8692[40], 11}, {NULL}};
const Segment SGI328956[] = {{&S8692[12], 14}, {NULL}};
const Segment SGI328957[] = {{&S8693[6], 4}, {NULL}};
const Segment SGI328958[] = {{&S8693[82], 5}, {NULL}};
const Segment SGI328959[] = {{&S8693[12], 14}, {&S8693[41], 11}, {NULL}};
const Segment SGI328960[] = {{&S8693[12], 14}, {NULL}};
const Segment SGI328961[] = {{&S8694[6], 4}, {NULL}};
const Segment SGI328962[] = {{&S8694[89], 5}, {NULL}};
const Segment SGI328963[] = {{&S8694[12], 14}, {&S8694[48], 11}, {NULL}};
const Segment SGI328964[] = {{&S8694[12], 14}, {NULL}};
const Segment SGI328965[] = {{&S8695[6], 4}, {NULL}};
const Segment SGI328966[] = {{&S8695[89], 5}, {NULL}};
const Segment SGI328967[] = {{&S8695[12], 14}, {&S8695[48], 11}, {NULL}};
const Segment SGI328968[] = {{&S8695[12], 14}, {NULL}};
const Segment SGI328969[] = {{&S8696[6], 4}, {NULL}};
const Segment SGI328970[] = {{&S8696[81], 5}, {NULL}};
const Segment SGI328971[] = {{&S8696[12], 14}, {&S8696[40], 11}, {NULL}};
const Segment SGI328972[] = {{&S8696[12], 14}, {NULL}};
const Segment SGI328973[] = {{&S8697[6], 4}, {NULL}};
const Segment SGI328974[] = {{&S8697[81], 5}, {NULL}};
const Segment SGI328975[] = {{&S8697[12], 14}, {&S8697[40], 11}, {NULL}};
const Segment SGI328976[] = {{&S8697[12], 14}, {NULL}};
const Segment SGI328977[] = {{&S8699[6], 4}, {NULL}};
const Segment SGI328978[] = {{&S8699[81], 5}, {NULL}};
const Segment SGI328979[] = {{&S8699[12], 14}, {&S8699[40], 11}, {NULL}};
const Segment SGI328980[] = {{&S8699[12], 14}, {NULL}};
const Segment SGI328981[] = {{&S8700[6], 4}, {NULL}};
const Segment SGI328982[] = {{&S8700[82], 5}, {NULL}};
const Segment SGI328983[] = {{&S8700[12], 14}, {&S8700[40], 12}, {NULL}};
const Segment SGI328984[] = {{&S8700[12], 14}, {NULL}};
const Segment SGI328985[] = {{&S8701[6], 4}, {NULL}};
const Segment SGI328986[] = {{&S8701[66], 5}, {NULL}};
const Segment SGI328987[] = {{&S8701[12], 14}, {NULL}};
const Segment SGI328988[] = {{&S8702[6], 4}, {NULL}};
const Segment SGI328989[] = {{&S8702[82], 5}, {NULL}};
const Segment SGI328990[] = {{&S8702[12], 14}, {&S8702[40], 12}, {NULL}};
const Segment SGI328991[] = {{&S8702[12], 14}, {NULL}};
const Segment SGI328992[] = {{&S8703[6], 4}, {NULL}};
const Segment SGI328993[] = {{&S8703[82], 5}, {NULL}};
const Segment SGI328994[] = {{&S8703[12], 14}, {&S8703[41], 11}, {NULL}};
const Segment SGI328995[] = {{&S8703[12], 14}, {NULL}};
const Segment SGI328996[] = {{&S8704[6], 4}, {NULL}};
const Segment SGI328997[] = {{&S8704[82], 5}, {NULL}};
const Segment SGI328998[] = {{&S8704[12], 14}, {&S8704[41], 11}, {NULL}};
const Segment SGI328999[] = {{&S8704[12], 14}, {NULL}};
const Segment SGI329000[] = {{&S8705[6], 4}, {NULL}};
const Segment SGI329001[] = {{&S8705[82], 5}, {NULL}};
const Segment SGI329002[] = {{&S8705[12], 14}, {&S8705[40], 12}, {NULL}};
const Segment SGI329003[] = {{&S8705[12], 14}, {NULL}};
const Segment SGI329004[] = {{&S8706[6], 4}, {NULL}};
const Segment SGI329005[] = {{&S8706[82], 5}, {NULL}};
const Segment SGI329006[] = {{&S8706[12], 14}, {&S8706[41], 11}, {NULL}};
const Segment SGI329007[] = {{&S8706[12], 14}, {NULL}};
const Segment SGI329008[] = {{&S8707[6], 4}, {NULL}};
const Segment SGI329009[] = {{&S8707[66], 5}, {NULL}};
const Segment SGI329010[] = {{&S8707[12], 14}, {&S8707[77], 31}, {NULL}};
const Segment SGI329011[] = {{&S8707[12], 14}, {NULL}};
const Segment SGI329012[] = {{&S8708[6], 4}, {NULL}};
const Segment SGI329013[] = {{&S8708[88], 5}, {NULL}};
const Segment SGI329014[] = {{&S8708[12], 14}, {&S8708[47], 11}, {NULL}};
const Segment SGI329015[] = {{&S8708[12], 14}, {NULL}};
const Segment SGI329016[] = {{&S8709[6], 4}, {NULL}};
const Segment SGI329017[] = {{&S8709[88], 5}, {NULL}};
const Segment SGI329018[] = {{&S8709[12], 14}, {&S8709[47], 11}, {NULL}};
const Segment SGI329019[] = {{&S8709[12], 14}, {NULL}};
const Segment SGI329020[] = {{&S8710[6], 4}, {NULL}};
const Segment SGI329021[] = {{&S8710[88], 5}, {NULL}};
const Segment SGI329022[] = {{&S8710[12], 14}, {&S8710[47], 11}, {NULL}};
const Segment SGI329023[] = {{&S8710[12], 14}, {NULL}};
const Segment SGI329024[] = {{&S8711[6], 4}, {NULL}};
const Segment SGI329025[] = {{&S8711[88], 5}, {NULL}};
const Segment SGI329026[] = {{&S8711[12], 14}, {&S8711[47], 11}, {NULL}};
const Segment SGI329027[] = {{&S8711[12], 14}, {NULL}};
const Segment SGI329028[] = {{&S8712[6], 4}, {NULL}};
const Segment SGI329029[] = {{&S8712[81], 5}, {NULL}};
const Segment SGI329030[] = {{&S8712[12], 14}, {&S8712[40], 11}, {NULL}};
const Segment SGI329031[] = {{&S8712[12], 14}, {NULL}};
const Segment SGI329032[] = {{&S8713[6], 4}, {NULL}};
const Segment SGI329033[] = {{&S8713[82], 5}, {NULL}};
const Segment SGI329034[] = {{&S8713[12], 14}, {&S8713[41], 11}, {NULL}};
const Segment SGI329035[] = {{&S8713[12], 14}, {NULL}};
const Segment SGI329036[] = {{&S8714[6], 4}, {NULL}};
const Segment SGI329037[] = {{&S8714[81], 5}, {NULL}};
const Segment SGI329038[] = {{&S8714[12], 14}, {&S8714[40], 11}, {NULL}};
const Segment SGI329039[] = {{&S8714[12], 14}, {NULL}};
const Segment SGI329040[] = {{&S8715[6], 4}, {NULL}};
const Segment SGI329041[] = {{&S8715[82], 5}, {NULL}};
const Segment SGI329042[] = {{&S8715[12], 14}, {&S8715[41], 11}, {NULL}};
const Segment SGI329043[] = {{&S8715[12], 14}, {NULL}};
const Segment SGI329044[] = {{&S8716[6], 4}, {NULL}};
const Segment SGI329045[] = {{&S8716[89], 5}, {NULL}};
const Segment SGI329046[] = {{&S8716[12], 14}, {&S8716[47], 12}, {NULL}};
const Segment SGI329047[] = {{&S8716[12], 14}, {NULL}};
const Segment SGI329048[] = {{&S8717[6], 4}, {NULL}};
const Segment SGI329049[] = {{&S8717[88], 5}, {NULL}};
const Segment SGI329050[] = {{&S8717[12], 14}, {&S8717[47], 11}, {NULL}};
const Segment SGI329051[] = {{&S8717[12], 14}, {NULL}};
const Segment SGI329052[] = {{&S8718[6], 4}, {NULL}};
const Segment SGI329053[] = {{&S8718[82], 5}, {NULL}};
const Segment SGI329054[] = {{&S8718[12], 14}, {&S8718[41], 11}, {NULL}};
const Segment SGI329055[] = {{&S8718[12], 14}, {NULL}};
const Segment SGI329056[] = {{&S8719[6], 4}, {NULL}};
const Segment SGI329057[] = {{&S8719[81], 5}, {NULL}};
const Segment SGI329058[] = {{&S8719[12], 14}, {&S8719[40], 11}, {NULL}};
const Segment SGI329059[] = {{&S8719[12], 14}, {NULL}};
const Segment SGI329060[] = {{&S8720[6], 4}, {NULL}};
const Segment SGI329061[] = {{&S8720[88], 5}, {NULL}};
const Segment SGI329062[] = {{&S8720[12], 14}, {&S8720[47], 11}, {NULL}};
const Segment SGI329063[] = {{&S8720[12], 14}, {NULL}};
const Segment SGI329064[] = {{&S8721[6], 4}, {NULL}};
const Segment SGI329065[] = {{&S8721[81], 5}, {NULL}};
const Segment SGI329066[] = {{&S8721[12], 14}, {&S8721[40], 11}, {NULL}};
const Segment SGI329067[] = {{&S8721[12], 14}, {NULL}};
const Segment SGI329068[] = {{&S8722[6], 4}, {NULL}};
const Segment SGI329069[] = {{&S8722[88], 5}, {NULL}};
const Segment SGI329070[] = {{&S8722[12], 14}, {&S8722[47], 11}, {NULL}};
const Segment SGI329071[] = {{&S8722[12], 14}, {NULL}};
const Segment SGI329072[] = {{&S8723[6], 4}, {NULL}};
const Segment SGI329073[] = {{&S8723[81], 5}, {NULL}};
const Segment SGI329074[] = {{&S8723[12], 14}, {&S8723[40], 11}, {NULL}};
const Segment SGI329075[] = {{&S8723[12], 14}, {NULL}};
const Segment SGI329076[] = {{&S8724[6], 4}, {NULL}};
const Segment SGI329077[] = {{&S8724[82], 5}, {NULL}};
const Segment SGI329078[] = {{&S8724[12], 14}, {&S8724[41], 11}, {NULL}};
const Segment SGI329079[] = {{&S8724[12], 14}, {NULL}};
const Segment SGI329080[] = {{&S8725[6], 4}, {NULL}};
const Segment SGI329081[] = {{&S8725[82], 5}, {NULL}};
const Segment SGI329082[] = {{&S8725[12], 14}, {&S8725[41], 11}, {NULL}};
const Segment SGI329083[] = {{&S8725[12], 14}, {NULL}};
const Segment SGI329084[] = {{&S8726[6], 4}, {NULL}};
const Segment SGI329085[] = {{&S8726[89], 5}, {NULL}};
const Segment SGI329086[] = {{&S8726[12], 14}, {&S8726[48], 11}, {NULL}};
const Segment SGI329087[] = {{&S8726[12], 14}, {NULL}};
const Segment SGI329088[] = {{&S8727[6], 4}, {NULL}};
const Segment SGI329089[] = {{&S8727[81], 5}, {NULL}};
const Segment SGI329090[] = {{&S8727[12], 14}, {&S8727[40], 11}, {NULL}};
const Segment SGI329091[] = {{&S8727[12], 14}, {NULL}};
const Segment SGI329092[] = {{&S8728[6], 4}, {NULL}};
const Segment SGI329093[] = {{&S8728[82], 5}, {NULL}};
const Segment SGI329094[] = {{&S8728[12], 14}, {&S8728[41], 11}, {NULL}};
const Segment SGI329095[] = {{&S8728[12], 14}, {NULL}};
const Segment SGI329096[] = {{&S8729[6], 4}, {NULL}};
const Segment SGI329097[] = {{&S8729[89], 5}, {NULL}};
const Segment SGI329098[] = {{&S8729[12], 14}, {&S8729[48], 11}, {NULL}};
const Segment SGI329099[] = {{&S8729[12], 14}, {NULL}};
const Segment SGI329100[] = {{&S8730[6], 4}, {NULL}};
const Segment SGI329101[] = {{&S8730[82], 5}, {NULL}};
const Segment SGI329102[] = {{&S8730[12], 14}, {&S8730[40], 12}, {NULL}};
const Segment SGI329103[] = {{&S8730[12], 14}, {NULL}};
const Segment SGI329104[] = {{&S8731[6], 4}, {NULL}};
const Segment SGI329105[] = {{&S8731[82], 5}, {NULL}};
const Segment SGI329106[] = {{&S8731[12], 14}, {&S8731[41], 11}, {NULL}};
const Segment SGI329107[] = {{&S8731[12], 14}, {NULL}};
const Segment SGI329108[] = {{&S8732[6], 4}, {NULL}};
const Segment SGI329109[] = {{&S8732[82], 5}, {NULL}};
const Segment SGI329110[] = {{&S8732[12], 14}, {&S8732[41], 11}, {NULL}};
const Segment SGI329111[] = {{&S8732[12], 14}, {NULL}};
const Segment SGI329112[] = {{&S8733[6], 4}, {NULL}};
const Segment SGI329113[] = {{&S8733[82], 5}, {NULL}};
const Segment SGI329114[] = {{&S8733[12], 14}, {&S8733[40], 12}, {NULL}};
const Segment SGI329115[] = {{&S8733[12], 14}, {NULL}};
const Segment SGI329116[] = {{&S8734[6], 4}, {NULL}};
const Segment SGI329117[] = {{&S8734[81], 5}, {NULL}};
const Segment SGI329118[] = {{&S8734[12], 14}, {&S8734[40], 11}, {NULL}};
const Segment SGI329119[] = {{&S8734[12], 14}, {NULL}};
const Segment SGI329120[] = {{&S8735[6], 4}, {NULL}};
const Segment SGI329121[] = {{&S8735[82], 5}, {NULL}};
const Segment SGI329122[] = {{&S8735[12], 14}, {&S8735[40], 12}, {NULL}};
const Segment SGI329123[] = {{&S8735[12], 14}, {NULL}};
const Segment SGI329124[] = {{&S8736[6], 4}, {NULL}};
const Segment SGI329125[] = {{&S8736[88], 5}, {NULL}};
const Segment SGI329126[] = {{&S8736[12], 14}, {&S8736[47], 11}, {NULL}};
const Segment SGI329127[] = {{&S8736[12], 14}, {NULL}};
const Segment SGI329128[] = {{&S8737[6], 4}, {NULL}};
const Segment SGI329129[] = {{&S8737[88], 5}, {NULL}};
const Segment SGI329130[] = {{&S8737[12], 14}, {&S8737[47], 11}, {NULL}};
const Segment SGI329131[] = {{&S8737[12], 14}, {NULL}};
const Segment SGI329132[] = {{&S8738[6], 4}, {NULL}};
const Segment SGI329133[] = {{&S8738[81], 5}, {NULL}};
const Segment SGI329134[] = {{&S8738[12], 14}, {&S8738[40], 11}, {NULL}};
const Segment SGI329135[] = {{&S8738[12], 14}, {NULL}};
const Segment SGI329136[] = {{&S8739[6], 4}, {NULL}};
const Segment SGI329137[] = {{&S8739[81], 5}, {NULL}};
const Segment SGI329138[] = {{&S8739[12], 14}, {&S8739[40], 11}, {NULL}};
const Segment SGI329139[] = {{&S8739[12], 14}, {NULL}};
const Segment SGI329140[] = {{&S8740[6], 4}, {NULL}};
const Segment SGI329141[] = {{&S8740[81], 5}, {NULL}};
const Segment SGI329142[] = {{&S8740[12], 14}, {&S8740[40], 11}, {NULL}};
const Segment SGI329143[] = {{&S8740[12], 14}, {NULL}};
const Segment SGI329144[] = {{&S8741[6], 4}, {NULL}};
const Segment SGI329145[] = {{&S8741[88], 5}, {NULL}};
const Segment SGI329146[] = {{&S8741[12], 14}, {&S8741[47], 11}, {NULL}};
const Segment SGI329147[] = {{&S8741[12], 14}, {NULL}};
const Segment SGI329148[] = {{&S8742[6], 4}, {NULL}};
const Segment SGI329149[] = {{&S8742[82], 5}, {NULL}};
const Segment SGI329150[] = {{&S8742[12], 14}, {&S8742[41], 11}, {NULL}};
const Segment SGI329151[] = {{&S8742[12], 14}, {NULL}};
const Segment SGI329152[] = {{&S8743[6], 4}, {NULL}};
const Segment SGI329153[] = {{&S8743[81], 5}, {NULL}};
const Segment SGI329154[] = {{&S8743[12], 14}, {&S8743[40], 11}, {NULL}};
const Segment SGI329155[] = {{&S8743[12], 14}, {NULL}};
const Segment SGI329156[] = {{&S8744[6], 4}, {NULL}};
const Segment SGI329157[] = {{&S8744[81], 5}, {NULL}};
const Segment SGI329158[] = {{&S8744[12], 14}, {&S8744[40], 11}, {NULL}};
const Segment SGI329159[] = {{&S8744[12], 14}, {NULL}};
const Segment SGI329160[] = {{&S8745[6], 4}, {NULL}};
const Segment SGI329161[] = {{&S8745[88], 5}, {NULL}};
const Segment SGI329162[] = {{&S8745[12], 14}, {&S8745[47], 11}, {NULL}};
const Segment SGI329163[] = {{&S8745[12], 14}, {NULL}};
const Segment SGI329164[] = {{&S8746[6], 4}, {NULL}};
const Segment SGI329165[] = {{&S8746[81], 5}, {NULL}};
const Segment SGI329166[] = {{&S8746[12], 14}, {&S8746[40], 11}, {NULL}};
const Segment SGI329167[] = {{&S8746[12], 14}, {NULL}};
const Segment SGI329168[] = {{&S8747[6], 4}, {NULL}};
const Segment SGI329169[] = {{&S8747[82], 5}, {NULL}};
const Segment SGI329170[] = {{&S8747[12], 14}, {&S8747[40], 12}, {NULL}};
const Segment SGI329171[] = {{&S8747[12], 14}, {NULL}};
const Segment SGI329172[] = {{&S8748[6], 4}, {NULL}};
const Segment SGI329173[] = {{&S8748[81], 5}, {NULL}};
const Segment SGI329174[] = {{&S8748[12], 14}, {&S8748[40], 11}, {NULL}};
const Segment SGI329175[] = {{&S8748[12], 14}, {NULL}};
const Segment SGI329176[] = {{&S8749[6], 4}, {NULL}};
const Segment SGI329177[] = {{&S8749[88], 5}, {NULL}};
const Segment SGI329178[] = {{&S8749[12], 14}, {&S8749[47], 11}, {NULL}};
const Segment SGI329179[] = {{&S8749[12], 14}, {NULL}};
const Segment SGI329180[] = {{&S8750[6], 4}, {NULL}};
const Segment SGI329181[] = {{&S8750[82], 5}, {NULL}};
const Segment SGI329182[] = {{&S8750[12], 14}, {&S8750[40], 12}, {NULL}};
const Segment SGI329183[] = {{&S8750[12], 14}, {NULL}};
const Segment SGI329184[] = {{&S8751[6], 4}, {NULL}};
const Segment SGI329185[] = {{&S8751[82], 5}, {NULL}};
const Segment SGI329186[] = {{&S8751[12], 14}, {&S8751[40], 12}, {NULL}};
const Segment SGI329187[] = {{&S8751[12], 14}, {NULL}};
const Segment SGI329188[] = {{&S8752[6], 4}, {NULL}};
const Segment SGI329189[] = {{&S8752[81], 5}, {NULL}};
const Segment SGI329190[] = {{&S8752[12], 14}, {&S8752[40], 11}, {NULL}};
const Segment SGI329191[] = {{&S8752[12], 14}, {NULL}};
const Segment SGI329192[] = {{&S8753[6], 4}, {NULL}};
const Segment SGI329193[] = {{&S8753[89], 5}, {NULL}};
const Segment SGI329194[] = {{&S8753[12], 14}, {&S8753[48], 11}, {NULL}};
const Segment SGI329195[] = {{&S8753[12], 14}, {NULL}};
const Segment SGI329196[] = {{&S8754[6], 4}, {NULL}};
const Segment SGI329197[] = {{&S8754[89], 5}, {NULL}};
const Segment SGI329198[] = {{&S8754[12], 14}, {&S8754[47], 12}, {NULL}};
const Segment SGI329199[] = {{&S8754[12], 14}, {NULL}};
const Segment SGI329200[] = {{&S8755[6], 4}, {NULL}};
const Segment SGI329201[] = {{&S8755[82], 5}, {NULL}};
const Segment SGI329202[] = {{&S8755[12], 14}, {&S8755[40], 12}, {NULL}};
const Segment SGI329203[] = {{&S8755[12], 14}, {NULL}};
const Segment SGI329204[] = {{&S8756[6], 4}, {NULL}};
const Segment SGI329205[] = {{&S8756[81], 5}, {NULL}};
const Segment SGI329206[] = {{&S8756[12], 14}, {&S8756[40], 11}, {NULL}};
const Segment SGI329207[] = {{&S8756[12], 14}, {NULL}};
const Segment SGI329208[] = {{&S8757[6], 4}, {NULL}};
const Segment SGI329209[] = {{&S8757[81], 5}, {NULL}};
const Segment SGI329210[] = {{&S8757[12], 14}, {&S8757[40], 11}, {NULL}};
const Segment SGI329211[] = {{&S8757[12], 14}, {NULL}};
const Segment SGI329212[] = {{&S8758[6], 4}, {NULL}};
const Segment SGI329213[] = {{&S8758[88], 5}, {NULL}};
const Segment SGI329214[] = {{&S8758[12], 14}, {&S8758[47], 11}, {NULL}};
const Segment SGI329215[] = {{&S8758[12], 14}, {NULL}};
const Segment SGI329216[] = {{&S8759[6], 4}, {NULL}};
const Segment SGI329217[] = {{&S8759[81], 5}, {NULL}};
const Segment SGI329218[] = {{&S8759[12], 14}, {&S8759[40], 11}, {NULL}};
const Segment SGI329219[] = {{&S8759[12], 14}, {NULL}};
const Segment SGI329220[] = {{&S8760[6], 4}, {NULL}};
const Segment SGI329221[] = {{&S8760[88], 5}, {NULL}};
const Segment SGI329222[] = {{&S8760[12], 14}, {&S8760[47], 11}, {NULL}};
const Segment SGI329223[] = {{&S8760[12], 14}, {NULL}};
const Segment SGI329224[] = {{&S8761[6], 4}, {NULL}};
const Segment SGI329225[] = {{&S8761[88], 5}, {NULL}};
const Segment SGI329226[] = {{&S8761[12], 14}, {&S8761[47], 11}, {NULL}};
const Segment SGI329227[] = {{&S8761[12], 14}, {NULL}};
const Segment SGI329228[] = {{&S8762[6], 4}, {NULL}};
const Segment SGI329229[] = {{&S8762[82], 5}, {NULL}};
const Segment SGI329230[] = {{&S8762[12], 14}, {&S8762[40], 12}, {NULL}};
const Segment SGI329231[] = {{&S8762[12], 14}, {NULL}};
const Segment SGI329232[] = {{&S8763[6], 4}, {NULL}};
const Segment SGI329233[] = {{&S8763[59], 5}, {NULL}};
const Segment SGI329234[] = {{&S8763[12], 14}, {&S8763[70], 4}, {NULL}};
const Segment SGI329235[] = {{&S8763[12], 14}, {NULL}};
const Segment SGI329236[] = {{&S8764[6], 4}, {NULL}};
const Segment SGI329237[] = {{&S8764[82], 5}, {NULL}};
const Segment SGI329238[] = {{&S8764[12], 14}, {&S8764[40], 12}, {NULL}};
const Segment SGI329239[] = {{&S8764[12], 14}, {NULL}};
const Segment SGI329240[] = {{&S8765[6], 4}, {NULL}};
const Segment SGI329241[] = {{&S8765[88], 5}, {NULL}};
const Segment SGI329242[] = {{&S8765[12], 14}, {&S8765[47], 11}, {NULL}};
const Segment SGI329243[] = {{&S8765[12], 14}, {NULL}};
const Segment SGI329244[] = {{&S8766[6], 4}, {NULL}};
const Segment SGI329245[] = {{&S8766[82], 5}, {NULL}};
const Segment SGI329246[] = {{&S8766[12], 14}, {&S8766[40], 12}, {NULL}};
const Segment SGI329247[] = {{&S8766[12], 14}, {NULL}};
const Segment SGI329248[] = {{&S8767[6], 4}, {NULL}};
const Segment SGI329249[] = {{&S8767[89], 5}, {NULL}};
const Segment SGI329250[] = {{&S8767[12], 14}, {&S8767[48], 11}, {NULL}};
const Segment SGI329251[] = {{&S8767[12], 14}, {NULL}};
const Segment SGI329252[] = {{&S8768[6], 4}, {NULL}};
const Segment SGI329253[] = {{&S8768[89], 5}, {NULL}};
const Segment SGI329254[] = {{&S8768[12], 14}, {&S8768[47], 12}, {NULL}};
const Segment SGI329255[] = {{&S8768[12], 14}, {NULL}};
const Segment SGI329256[] = {{&S8769[6], 4}, {NULL}};
const Segment SGI329257[] = {{&S8769[82], 5}, {NULL}};
const Segment SGI329258[] = {{&S8769[12], 14}, {&S8769[40], 12}, {NULL}};
const Segment SGI329259[] = {{&S8769[12], 14}, {NULL}};
const Segment SGI329260[] = {{&S8770[6], 4}, {NULL}};
const Segment SGI329261[] = {{&S8770[89], 5}, {NULL}};
const Segment SGI329262[] = {{&S8770[12], 14}, {&S8770[48], 11}, {NULL}};
const Segment SGI329263[] = {{&S8770[12], 14}, {NULL}};
const Segment SGI329264[] = {{&S8771[6], 4}, {NULL}};
const Segment SGI329265[] = {{&S8771[82], 5}, {NULL}};
const Segment SGI329266[] = {{&S8771[12], 14}, {&S8771[41], 11}, {NULL}};
const Segment SGI329267[] = {{&S8771[12], 14}, {NULL}};
const Segment SGI329268[] = {{&S8772[6], 4}, {NULL}};
const Segment SGI329269[] = {{&S8772[82], 5}, {NULL}};
const Segment SGI329270[] = {{&S8772[12], 14}, {&S8772[41], 11}, {NULL}};
const Segment SGI329271[] = {{&S8772[12], 14}, {NULL}};
const Segment SGI329272[] = {{&S8773[6], 4}, {NULL}};
const Segment SGI329273[] = {{&S8773[81], 5}, {NULL}};
const Segment SGI329274[] = {{&S8773[12], 14}, {&S8773[40], 11}, {NULL}};
const Segment SGI329275[] = {{&S8773[12], 14}, {NULL}};
const Segment SGI329276[] = {{&S8774[6], 4}, {NULL}};
const Segment SGI329277[] = {{&S8774[81], 5}, {NULL}};
const Segment SGI329278[] = {{&S8774[12], 14}, {&S8774[40], 11}, {NULL}};
const Segment SGI329279[] = {{&S8774[12], 14}, {NULL}};
const Segment SGI329280[] = {{&S8775[6], 4}, {NULL}};
const Segment SGI329281[] = {{&S8775[58], 5}, {NULL}};
const Segment SGI329282[] = {{&S8775[12], 14}, {NULL}};
const Segment SGI329283[] = {{&S8776[6], 4}, {NULL}};
const Segment SGI329284[] = {{&S8776[89], 5}, {NULL}};
const Segment SGI329285[] = {{&S8776[12], 14}, {&S8776[47], 12}, {NULL}};
const Segment SGI329286[] = {{&S8776[12], 14}, {NULL}};
const Segment SGI329287[] = {{&S8777[6], 4}, {NULL}};
const Segment SGI329288[] = {{&S8777[59], 5}, {NULL}};
const Segment SGI329289[] = {{&S8777[12], 14}, {&S8777[70], 4}, {NULL}};
const Segment SGI329290[] = {{&S8777[12], 14}, {NULL}};
const Segment SGI329291[] = {{&S8778[6], 4}, {NULL}};
const Segment SGI329292[] = {{&S8778[81], 5}, {NULL}};
const Segment SGI329293[] = {{&S8778[12], 14}, {&S8778[40], 11}, {NULL}};
const Segment SGI329294[] = {{&S8778[12], 14}, {NULL}};
const Segment SGI329295[] = {{&S8779[6], 4}, {NULL}};
const Segment SGI329296[] = {{&S8779[88], 5}, {NULL}};
const Segment SGI329297[] = {{&S8779[12], 14}, {&S8779[47], 11}, {NULL}};
const Segment SGI329298[] = {{&S8779[12], 14}, {NULL}};
const Segment SGI329299[] = {{&S8780[6], 4}, {NULL}};
const Segment SGI329300[] = {{&S8780[82], 5}, {NULL}};
const Segment SGI329301[] = {{&S8780[12], 14}, {&S8780[41], 11}, {NULL}};
const Segment SGI329302[] = {{&S8780[12], 14}, {NULL}};
const Segment SGI329303[] = {{&S8781[6], 4}, {NULL}};
const Segment SGI329304[] = {{&S8781[88], 5}, {NULL}};
const Segment SGI329305[] = {{&S8781[12], 14}, {&S8781[47], 11}, {NULL}};
const Segment SGI329306[] = {{&S8781[12], 14}, {NULL}};
const Segment SGI329307[] = {{&S8782[6], 4}, {NULL}};
const Segment SGI329308[] = {{&S8782[81], 5}, {NULL}};
const Segment SGI329309[] = {{&S8782[12], 14}, {&S8782[40], 11}, {NULL}};
const Segment SGI329310[] = {{&S8782[12], 14}, {NULL}};
const Segment SGI329311[] = {{&S8783[6], 4}, {NULL}};
const Segment SGI329312[] = {{&S8783[81], 5}, {NULL}};
const Segment SGI329313[] = {{&S8783[12], 14}, {&S8783[40], 11}, {NULL}};
const Segment SGI329314[] = {{&S8783[12], 14}, {NULL}};
const Segment SGI329315[] = {{&S8784[6], 4}, {NULL}};
const Segment SGI329316[] = {{&S8784[81], 5}, {NULL}};
const Segment SGI329317[] = {{&S8784[12], 14}, {&S8784[40], 11}, {NULL}};
const Segment SGI329318[] = {{&S8784[12], 14}, {NULL}};
const Segment SGI329319[] = {{&S8785[6], 4}, {NULL}};
const Segment SGI329320[] = {{&S8785[88], 5}, {NULL}};
const Segment SGI329321[] = {{&S8785[12], 14}, {&S8785[47], 11}, {NULL}};
const Segment SGI329322[] = {{&S8785[12], 14}, {NULL}};
const Segment SGI329323[] = {{&S8786[6], 4}, {NULL}};
const Segment SGI329324[] = {{&S8786[81], 5}, {NULL}};
const Segment SGI329325[] = {{&S8786[12], 14}, {&S8786[40], 11}, {NULL}};
const Segment SGI329326[] = {{&S8786[12], 14}, {NULL}};
const Segment SGI329327[] = {{&S8787[6], 4}, {NULL}};
const Segment SGI329328[] = {{&S8787[81], 5}, {NULL}};
const Segment SGI329329[] = {{&S8787[12], 14}, {&S8787[40], 11}, {NULL}};
const Segment SGI329330[] = {{&S8787[12], 14}, {NULL}};
const Segment SGI329331[] = {{&S8788[6], 4}, {NULL}};
const Segment SGI329332[] = {{&S8788[81], 5}, {NULL}};
const Segment SGI329333[] = {{&S8788[12], 14}, {&S8788[40], 11}, {NULL}};
const Segment SGI329334[] = {{&S8788[12], 14}, {NULL}};
const Segment SGI329335[] = {{&S8789[6], 4}, {NULL}};
const Segment SGI329336[] = {{&S8789[81], 5}, {NULL}};
const Segment SGI329337[] = {{&S8789[12], 14}, {&S8789[40], 11}, {NULL}};
const Segment SGI329338[] = {{&S8789[12], 14}, {NULL}};
const Segment SGI329339[] = {{&S8790[6], 4}, {NULL}};
const Segment SGI329340[] = {{&S8790[81], 5}, {NULL}};
const Segment SGI329341[] = {{&S8790[12], 14}, {&S8790[40], 11}, {NULL}};
const Segment SGI329342[] = {{&S8790[12], 14}, {NULL}};
const Segment SGI329343[] = {{&S8791[6], 4}, {NULL}};
const Segment SGI329344[] = {{&S8791[88], 5}, {NULL}};
const Segment SGI329345[] = {{&S8791[12], 14}, {&S8791[47], 11}, {NULL}};
const Segment SGI329346[] = {{&S8791[12], 14}, {NULL}};
const Segment SGI329347[] = {{&S8792[6], 4}, {NULL}};
const Segment SGI329348[] = {{&S8792[81], 5}, {NULL}};
const Segment SGI329349[] = {{&S8792[12], 14}, {&S8792[40], 11}, {NULL}};
const Segment SGI329350[] = {{&S8792[12], 14}, {NULL}};
const Segment SGI329351[] = {{&S8793[6], 4}, {NULL}};
const Segment SGI329352[] = {{&S8793[81], 5}, {NULL}};
const Segment SGI329353[] = {{&S8793[12], 14}, {&S8793[40], 11}, {NULL}};
const Segment SGI329354[] = {{&S8793[12], 14}, {NULL}};
const Segment SGI329355[] = {{&S8794[6], 4}, {NULL}};
const Segment SGI329356[] = {{&S8794[81], 5}, {NULL}};
const Segment SGI329357[] = {{&S8794[12], 14}, {&S8794[40], 11}, {NULL}};
const Segment SGI329358[] = {{&S8794[12], 14}, {NULL}};
const Segment SGI329359[] = {{&S8795[6], 4}, {NULL}};
const Segment SGI329360[] = {{&S8795[81], 5}, {NULL}};
const Segment SGI329361[] = {{&S8795[12], 14}, {&S8795[40], 11}, {NULL}};
const Segment SGI329362[] = {{&S8795[12], 14}, {NULL}};
const Segment SGI329363[] = {{&S8796[6], 4}, {NULL}};
const Segment SGI329364[] = {{&S8796[81], 5}, {NULL}};
const Segment SGI329365[] = {{&S8796[12], 14}, {&S8796[40], 11}, {NULL}};
const Segment SGI329366[] = {{&S8796[12], 14}, {NULL}};
const Segment SGI329367[] = {{&S8797[6], 4}, {NULL}};
const Segment SGI329368[] = {{&S8797[88], 5}, {NULL}};
const Segment SGI329369[] = {{&S8797[12], 14}, {&S8797[47], 11}, {NULL}};
const Segment SGI329370[] = {{&S8797[12], 14}, {NULL}};
const Segment SGI329371[] = {{&S8798[6], 4}, {NULL}};
const Segment SGI329372[] = {{&S8798[81], 5}, {NULL}};
const Segment SGI329373[] = {{&S8798[12], 14}, {&S8798[40], 11}, {NULL}};
const Segment SGI329374[] = {{&S8798[12], 14}, {NULL}};
const Segment SGI329375[] = {{&S8799[6], 4}, {NULL}};
const Segment SGI329376[] = {{&S8799[81], 5}, {NULL}};
const Segment SGI329377[] = {{&S8799[12], 14}, {&S8799[40], 11}, {NULL}};
const Segment SGI329378[] = {{&S8799[12], 14}, {NULL}};
const Segment SGI329379[] = {{&S8800[6], 4}, {NULL}};
const Segment SGI329380[] = {{&S8800[82], 5}, {NULL}};
const Segment SGI329381[] = {{&S8800[12], 14}, {&S8800[41], 11}, {NULL}};
const Segment SGI329382[] = {{&S8800[12], 14}, {NULL}};
const Segment SGI329383[] = {{&S8801[6], 4}, {NULL}};
const Segment SGI329384[] = {{&S8801[81], 5}, {NULL}};
const Segment SGI329385[] = {{&S8801[12], 14}, {&S8801[40], 11}, {NULL}};
const Segment SGI329386[] = {{&S8801[12], 14}, {NULL}};
const Segment SGI329387[] = {{&S8802[6], 4}, {NULL}};
const Segment SGI329388[] = {{&S8802[81], 5}, {NULL}};
const Segment SGI329389[] = {{&S8802[12], 14}, {&S8802[40], 11}, {NULL}};
const Segment SGI329390[] = {{&S8802[12], 14}, {NULL}};
const Segment SGI329391[] = {{&S8803[6], 4}, {NULL}};
const Segment SGI329392[] = {{&S8803[81], 5}, {NULL}};
const Segment SGI329393[] = {{&S8803[12], 14}, {&S8803[40], 11}, {NULL}};
const Segment SGI329394[] = {{&S8803[12], 14}, {NULL}};
const Segment SGI329395[] = {{&S8804[6], 4}, {NULL}};
const Segment SGI329396[] = {{&S8804[88], 5}, {NULL}};
const Segment SGI329397[] = {{&S8804[12], 14}, {&S8804[47], 11}, {NULL}};
const Segment SGI329398[] = {{&S8804[12], 14}, {NULL}};
const Segment SGI329399[] = {{&S8805[6], 4}, {NULL}};
const Segment SGI329400[] = {{&S8805[81], 5}, {NULL}};
const Segment SGI329401[] = {{&S8805[12], 14}, {&S8805[40], 11}, {NULL}};
const Segment SGI329402[] = {{&S8805[12], 14}, {NULL}};
const Segment SGI329403[] = {{&S8806[6], 4}, {NULL}};
const Segment SGI329404[] = {{&S8806[81], 5}, {NULL}};
const Segment SGI329405[] = {{&S8806[12], 14}, {&S8806[40], 11}, {NULL}};
const Segment SGI329406[] = {{&S8806[12], 14}, {NULL}};
const Segment SGI329407[] = {{&S8807[6], 4}, {NULL}};
const Segment SGI329408[] = {{&S8807[82], 5}, {NULL}};
const Segment SGI329409[] = {{&S8807[12], 14}, {&S8807[40], 12}, {NULL}};
const Segment SGI329410[] = {{&S8807[12], 14}, {NULL}};
const Segment SGI329411[] = {{&S8808[6], 4}, {NULL}};
const Segment SGI329412[] = {{&S8808[81], 5}, {NULL}};
const Segment SGI329413[] = {{&S8808[12], 14}, {&S8808[40], 11}, {NULL}};
const Segment SGI329414[] = {{&S8808[12], 14}, {NULL}};
const Segment SGI329415[] = {{&S8809[6], 4}, {NULL}};
const Segment SGI329416[] = {{&S8809[81], 5}, {NULL}};
const Segment SGI329417[] = {{&S8809[12], 14}, {&S8809[40], 11}, {NULL}};
const Segment SGI329418[] = {{&S8809[12], 14}, {NULL}};
const Segment SGI329419[] = {{&S8810[6], 4}, {NULL}};
const Segment SGI329420[] = {{&S8810[81], 5}, {NULL}};
const Segment SGI329421[] = {{&S8810[12], 14}, {&S8810[40], 11}, {NULL}};
const Segment SGI329422[] = {{&S8810[12], 14}, {NULL}};
const Segment SGI329423[] = {{&S8811[6], 4}, {NULL}};
const Segment SGI329424[] = {{&S8811[81], 5}, {NULL}};
const Segment SGI329425[] = {{&S8811[12], 14}, {&S8811[40], 11}, {NULL}};
const Segment SGI329426[] = {{&S8811[12], 14}, {NULL}};
const Segment SGI329427[] = {{&S8812[6], 4}, {NULL}};
const Segment SGI329428[] = {{&S8812[82], 5}, {NULL}};
const Segment SGI329429[] = {{&S8812[12], 14}, {&S8812[40], 12}, {NULL}};
const Segment SGI329430[] = {{&S8812[12], 14}, {NULL}};
const Segment SGI329431[] = {{&S8813[6], 4}, {NULL}};
const Segment SGI329432[] = {{&S8813[89], 5}, {NULL}};
const Segment SGI329433[] = {{&S8813[12], 14}, {&S8813[48], 11}, {NULL}};
const Segment SGI329434[] = {{&S8813[12], 14}, {NULL}};
const Segment SGI329435[] = {{&S8814[6], 4}, {NULL}};
const Segment SGI329436[] = {{&S8814[81], 5}, {NULL}};
const Segment SGI329437[] = {{&S8814[12], 14}, {&S8814[40], 11}, {NULL}};
const Segment SGI329438[] = {{&S8814[12], 14}, {NULL}};
const Segment SGI329439[] = {{&S8815[6], 4}, {NULL}};
const Segment SGI329440[] = {{&S8815[81], 5}, {NULL}};
const Segment SGI329441[] = {{&S8815[12], 14}, {&S8815[40], 11}, {NULL}};
const Segment SGI329442[] = {{&S8815[12], 14}, {NULL}};
const Segment SGI329443[] = {{&S8816[6], 4}, {NULL}};
const Segment SGI329444[] = {{&S8816[81], 5}, {NULL}};
const Segment SGI329445[] = {{&S8816[12], 14}, {&S8816[40], 11}, {NULL}};
const Segment SGI329446[] = {{&S8816[12], 14}, {NULL}};
const Segment SGI329447[] = {{&S8817[6], 4}, {NULL}};
const Segment SGI329448[] = {{&S8817[89], 5}, {NULL}};
const Segment SGI329449[] = {{&S8817[12], 14}, {&S8817[47], 12}, {NULL}};
const Segment SGI329450[] = {{&S8817[12], 14}, {NULL}};
const Segment SGI329451[] = {{&S8818[6], 4}, {NULL}};
const Segment SGI329452[] = {{&S8818[89], 5}, {NULL}};
const Segment SGI329453[] = {{&S8818[12], 14}, {&S8818[48], 11}, {NULL}};
const Segment SGI329454[] = {{&S8818[12], 14}, {NULL}};
const Segment SGI329455[] = {{&S8819[6], 4}, {NULL}};
const Segment SGI329456[] = {{&S8819[82], 5}, {NULL}};
const Segment SGI329457[] = {{&S8819[12], 14}, {&S8819[41], 11}, {NULL}};
const Segment SGI329458[] = {{&S8819[12], 14}, {NULL}};
const Segment SGI329459[] = {{&S8820[6], 4}, {NULL}};
const Segment SGI329460[] = {{&S8820[81], 5}, {NULL}};
const Segment SGI329461[] = {{&S8820[12], 14}, {&S8820[40], 11}, {NULL}};
const Segment SGI329462[] = {{&S8820[12], 14}, {NULL}};
const Segment SGI329463[] = {{&S8821[6], 4}, {NULL}};
const Segment SGI329464[] = {{&S8821[88], 5}, {NULL}};
const Segment SGI329465[] = {{&S8821[12], 14}, {&S8821[47], 11}, {NULL}};
const Segment SGI329466[] = {{&S8821[12], 14}, {NULL}};
const Segment SGI329467[] = {{&S8822[6], 4}, {NULL}};
const Segment SGI329468[] = {{&S8822[81], 5}, {NULL}};
const Segment SGI329469[] = {{&S8822[12], 14}, {&S8822[40], 11}, {NULL}};
const Segment SGI329470[] = {{&S8822[12], 14}, {NULL}};
const Segment SGI329471[] = {{&S8823[6], 4}, {NULL}};
const Segment SGI329472[] = {{&S8823[88], 5}, {NULL}};
const Segment SGI329473[] = {{&S8823[12], 14}, {&S8823[47], 11}, {NULL}};
const Segment SGI329474[] = {{&S8823[12], 14}, {NULL}};
const Segment SGI329475[] = {{&S8824[6], 4}, {NULL}};
const Segment SGI329476[] = {{&S8824[81], 5}, {NULL}};
const Segment SGI329477[] = {{&S8824[12], 14}, {&S8824[40], 11}, {NULL}};
const Segment SGI329478[] = {{&S8824[12], 14}, {NULL}};
const Segment SGI329479[] = {{&S8825[6], 4}, {NULL}};
const Segment SGI329480[] = {{&S8825[81], 5}, {NULL}};
const Segment SGI329481[] = {{&S8825[12], 14}, {&S8825[40], 11}, {NULL}};
const Segment SGI329482[] = {{&S8825[12], 14}, {NULL}};
const Segment SGI329483[] = {{&S8826[6], 4}, {NULL}};
const Segment SGI329484[] = {{&S8826[81], 5}, {NULL}};
const Segment SGI329485[] = {{&S8826[12], 14}, {&S8826[40], 11}, {NULL}};
const Segment SGI329486[] = {{&S8826[12], 14}, {NULL}};
const Segment SGI329487[] = {{&S8827[6], 4}, {NULL}};
const Segment SGI329488[] = {{&S8827[81], 5}, {NULL}};
const Segment SGI329489[] = {{&S8827[12], 14}, {&S8827[40], 11}, {NULL}};
const Segment SGI329490[] = {{&S8827[12], 14}, {NULL}};
const Segment SGI329491[] = {{&S8828[6], 4}, {NULL}};
const Segment SGI329492[] = {{&S8828[81], 5}, {NULL}};
const Segment SGI329493[] = {{&S8828[12], 14}, {&S8828[40], 11}, {NULL}};
const Segment SGI329494[] = {{&S8828[12], 14}, {NULL}};
const Segment SGI329495[] = {{&S8829[6], 4}, {NULL}};
const Segment SGI329496[] = {{&S8829[81], 5}, {NULL}};
const Segment SGI329497[] = {{&S8829[12], 14}, {&S8829[40], 11}, {NULL}};
const Segment SGI329498[] = {{&S8829[12], 14}, {NULL}};
const Segment SGI329499[] = {{&S8830[6], 4}, {NULL}};
const Segment SGI329500[] = {{&S8830[88], 5}, {NULL}};
const Segment SGI329501[] = {{&S8830[12], 14}, {&S8830[47], 11}, {NULL}};
const Segment SGI329502[] = {{&S8830[12], 14}, {NULL}};
const Segment SGI329503[] = {{&S8831[6], 4}, {NULL}};
const Segment SGI329504[] = {{&S8831[88], 5}, {NULL}};
const Segment SGI329505[] = {{&S8831[12], 14}, {&S8831[47], 11}, {NULL}};
const Segment SGI329506[] = {{&S8831[12], 14}, {NULL}};
const Segment SGI329507[] = {{&S8832[6], 4}, {NULL}};
const Segment SGI329508[] = {{&S8832[81], 5}, {NULL}};
const Segment SGI329509[] = {{&S8832[12], 14}, {&S8832[40], 11}, {NULL}};
const Segment SGI329510[] = {{&S8832[12], 14}, {NULL}};
const Segment SGI329511[] = {{&S9052[68], 5}, {NULL}};
const Segment SGI329512[] = {{&S9052[13], 10}, {NULL}};
const Segment SGI329513[] = {{&S9025[6], 4}, {NULL}};
const Segment SGI329514[] = {{&S9025[66], 5}, {NULL}};
const Segment SGI329515[] = {{&S9025[12], 14}, {NULL}};
const Segment SGI329516[] = {{&S9026[6], 4}, {NULL}};
const Segment SGI329517[] = {{&S9026[89], 5}, {NULL}};
const Segment SGI329518[] = {{&S9026[12], 14}, {&S9026[48], 11}, {NULL}};
const Segment SGI329519[] = {{&S9026[12], 14}, {NULL}};
const Segment SGI329520[] = {{&S9028[6], 4}, {NULL}};
const Segment SGI329521[] = {{&S9028[82], 5}, {NULL}};
const Segment SGI329522[] = {{&S9028[12], 14}, {&S9028[41], 11}, {NULL}};
const Segment SGI329523[] = {{&S9028[12], 14}, {NULL}};
const Segment SGI329524[] = {{&S9029[6], 4}, {NULL}};
const Segment SGI329525[] = {{&S9029[81], 5}, {NULL}};
const Segment SGI329526[] = {{&S9029[12], 14}, {&S9029[40], 11}, {NULL}};
const Segment SGI329527[] = {{&S9029[12], 14}, {NULL}};
const Segment SGI329528[] = {{&S9030[6], 4}, {NULL}};
const Segment SGI329529[] = {{&S9030[88], 5}, {NULL}};
const Segment SGI329530[] = {{&S9030[12], 14}, {&S9030[47], 11}, {NULL}};
const Segment SGI329531[] = {{&S9030[12], 14}, {NULL}};
const Segment SGI329532[] = {{&S9031[6], 4}, {NULL}};
const Segment SGI329533[] = {{&S9031[89], 5}, {NULL}};
const Segment SGI329534[] = {{&S9031[12], 14}, {&S9031[47], 12}, {NULL}};
const Segment SGI329535[] = {{&S9031[12], 14}, {NULL}};
const Segment SGI329536[] = {{&S9032[6], 4}, {NULL}};
const Segment SGI329537[] = {{&S9032[81], 5}, {NULL}};
const Segment SGI329538[] = {{&S9032[12], 14}, {&S9032[40], 11}, {NULL}};
const Segment SGI329539[] = {{&S9032[12], 14}, {NULL}};
const Segment SGI329540[] = {{&S9033[6], 4}, {NULL}};
const Segment SGI329541[] = {{&S9033[81], 5}, {NULL}};
const Segment SGI329542[] = {{&S9033[12], 14}, {&S9033[40], 11}, {NULL}};
const Segment SGI329543[] = {{&S9033[12], 14}, {NULL}};
const Segment SGI329544[] = {{&S9034[6], 4}, {NULL}};
const Segment SGI329545[] = {{&S9034[88], 5}, {NULL}};
const Segment SGI329546[] = {{&S9034[12], 14}, {&S9034[47], 11}, {NULL}};
const Segment SGI329547[] = {{&S9034[12], 14}, {NULL}};
const Segment SGI329548[] = {{&S9035[6], 4}, {NULL}};
const Segment SGI329549[] = {{&S9035[59], 5}, {NULL}};
const Segment SGI329550[] = {{&S9035[12], 14}, {&S9035[70], 31}, {NULL}};
const Segment SGI329551[] = {{&S9035[12], 14}, {NULL}};
const Segment SGI329552[] = {{&S9036[6], 4}, {NULL}};
const Segment SGI329553[] = {{&S9036[82], 5}, {NULL}};
const Segment SGI329554[] = {{&S9036[12], 14}, {&S9036[41], 11}, {NULL}};
const Segment SGI329555[] = {{&S9036[12], 14}, {NULL}};
const Segment SGI329556[] = {{&S9037[6], 4}, {NULL}};
const Segment SGI329557[] = {{&S9037[81], 5}, {NULL}};
const Segment SGI329558[] = {{&S9037[12], 14}, {&S9037[40], 11}, {NULL}};
const Segment SGI329559[] = {{&S9037[12], 14}, {NULL}};
const Segment SGI329560[] = {{&S9038[6], 4}, {NULL}};
const Segment SGI329561[] = {{&S33197[5], 5}, {NULL}};
const Segment SGI329562[] = {{&S9038[82], 5}, {NULL}};
const Segment SGI329563[] = {{&S9038[12], 14}, {&S9038[40], 12}, {NULL}};
const Segment SGI329564[] = {{&S9038[12], 14}, {NULL}};
const Segment SGI329565[] = {{&S9039[6], 4}, {NULL}};
const Segment SGI329566[] = {{&S9039[88], 5}, {NULL}};
const Segment SGI329567[] = {{&S9039[12], 14}, {&S9039[47], 11}, {NULL}};
const Segment SGI329568[] = {{&S9039[12], 14}, {NULL}};
const Segment SGI329569[] = {{&S9040[6], 4}, {NULL}};
const Segment SGI329570[] = {{&S9040[81], 5}, {NULL}};
const Segment SGI329571[] = {{&S9040[12], 14}, {&S9040[40], 11}, {NULL}};
const Segment SGI329572[] = {{&S9040[12], 14}, {NULL}};
const Segment SGI329573[] = {{&S9041[6], 4}, {NULL}};
const Segment SGI329574[] = {{&S9041[88], 5}, {NULL}};
const Segment SGI329575[] = {{&S9041[12], 14}, {&S9041[47], 11}, {NULL}};
const Segment SGI329576[] = {{&S9041[12], 14}, {NULL}};
const Segment SGI329577[] = {{&S9042[6], 4}, {NULL}};
const Segment SGI329578[] = {{&S9042[88], 5}, {NULL}};
const Segment SGI329579[] = {{&S9042[12], 14}, {&S9042[47], 11}, {NULL}};
const Segment SGI329580[] = {{&S9042[12], 14}, {NULL}};
const Segment SGI329581[] = {{&S9043[6], 4}, {NULL}};
const Segment SGI329582[] = {{&S9043[89], 5}, {NULL}};
const Segment SGI329583[] = {{&S9043[12], 14}, {&S9043[48], 11}, {NULL}};
const Segment SGI329584[] = {{&S9043[12], 14}, {NULL}};
const Segment SGI329585[] = {{&S9044[6], 4}, {NULL}};
const Segment SGI329586[] = {{&S9044[81], 5}, {NULL}};
const Segment SGI329587[] = {{&S9044[12], 14}, {&S9044[40], 11}, {NULL}};
const Segment SGI329588[] = {{&S9044[12], 14}, {NULL}};
const Segment SGI329589[] = {{&S9045[6], 4}, {NULL}};
const Segment SGI329590[] = {{&S9045[82], 5}, {NULL}};
const Segment SGI329591[] = {{&S9045[12], 14}, {&S9045[40], 12}, {NULL}};
const Segment SGI329592[] = {{&S9045[12], 14}, {NULL}};
const Segment SGI329593[] = {{&S9046[6], 4}, {NULL}};
const Segment SGI329594[] = {{&S9046[88], 5}, {NULL}};
const Segment SGI329595[] = {{&S9046[12], 14}, {&S9046[47], 11}, {NULL}};
const Segment SGI329596[] = {{&S9046[12], 14}, {NULL}};
const Segment SGI329597[] = {{&S9047[6], 4}, {NULL}};
const Segment SGI329598[] = {{&S9047[81], 5}, {NULL}};
const Segment SGI329599[] = {{&S9047[12], 14}, {&S9047[40], 11}, {NULL}};
const Segment SGI329600[] = {{&S9047[12], 14}, {NULL}};
const Segment SGI329601[] = {{&S9048[6], 4}, {NULL}};
const Segment SGI329602[] = {{&S9048[82], 5}, {NULL}};
const Segment SGI329603[] = {{&S9048[12], 14}, {&S9048[40], 12}, {NULL}};
const Segment SGI329604[] = {{&S9048[12], 14}, {NULL}};
const Segment SGI329605[] = {{&S9049[6], 4}, {NULL}};
const Segment SGI329606[] = {{&S9049[89], 5}, {NULL}};
const Segment SGI329607[] = {{&S9049[12], 14}, {&S9049[48], 11}, {NULL}};
const Segment SGI329608[] = {{&S9049[12], 14}, {NULL}};
const Segment SGI329609[] = {{&S9050[6], 4}, {NULL}};
const Segment SGI329610[] = {{&S9050[82], 5}, {NULL}};
const Segment SGI329611[] = {{&S9050[12], 14}, {&S9050[41], 11}, {NULL}};
const Segment SGI329612[] = {{&S9050[12], 14}, {NULL}};
const Segment SGI329613[] = {{&S9051[6], 4}, {NULL}};
const Segment SGI329614[] = {{&S9051[82], 5}, {NULL}};
const Segment SGI329615[] = {{&S9051[12], 14}, {&S9051[41], 11}, {NULL}};
const Segment SGI329616[] = {{&S9051[12], 14}, {NULL}};
const Segment SGI329617[] = {{&S9053[6], 4}, {NULL}};
const Segment SGI329618[] = {{&S9053[89], 5}, {NULL}};
const Segment SGI329619[] = {{&S9053[12], 14}, {&S9053[48], 11}, {NULL}};
const Segment SGI329620[] = {{&S9053[12], 14}, {NULL}};
const Segment SGI329621[] = {{&S9054[6], 4}, {NULL}};
const Segment SGI329622[] = {{&S9054[81], 5}, {NULL}};
const Segment SGI329623[] = {{&S9054[12], 14}, {&S9054[40], 11}, {NULL}};
const Segment SGI329624[] = {{&S9054[12], 14}, {NULL}};
const Segment SGI329625[] = {{&S9055[6], 4}, {NULL}};
const Segment SGI329626[] = {{&S9055[82], 5}, {NULL}};
const Segment SGI329627[] = {{&S9055[12], 14}, {&S9055[40], 12}, {NULL}};
const Segment SGI329628[] = {{&S9055[12], 14}, {NULL}};
const Segment SGI329629[] = {{&S9056[6], 4}, {NULL}};
const Segment SGI329630[] = {{&S9056[81], 5}, {NULL}};
const Segment SGI329631[] = {{&S9056[12], 14}, {&S9056[40], 11}, {NULL}};
const Segment SGI329632[] = {{&S9056[12], 14}, {NULL}};
const Segment SGI329633[] = {{&S9057[6], 4}, {NULL}};
const Segment SGI329634[] = {{&S9057[82], 5}, {NULL}};
const Segment SGI329635[] = {{&S9057[12], 14}, {&S9057[41], 11}, {NULL}};
const Segment SGI329636[] = {{&S9057[12], 14}, {NULL}};
const Segment SGI329637[] = {{&S9058[6], 4}, {NULL}};
const Segment SGI329638[] = {{&S9058[88], 5}, {NULL}};
const Segment SGI329639[] = {{&S9058[12], 14}, {&S9058[47], 11}, {NULL}};
const Segment SGI329640[] = {{&S9058[12], 14}, {NULL}};
const Segment SGI329641[] = {{&S9059[6], 4}, {NULL}};
const Segment SGI329642[] = {{&S9059[81], 5}, {NULL}};
const Segment SGI329643[] = {{&S9059[12], 14}, {&S9059[40], 11}, {NULL}};
const Segment SGI329644[] = {{&S9059[12], 14}, {NULL}};
const Segment SGI329645[] = {{&S9060[6], 4}, {NULL}};
const Segment SGI329646[] = {{&S9060[89], 5}, {NULL}};
const Segment SGI329647[] = {{&S9060[12], 14}, {&S9060[48], 11}, {NULL}};
const Segment SGI329648[] = {{&S9060[12], 14}, {NULL}};
const Segment SGI329649[] = {{&S9061[6], 4}, {NULL}};
const Segment SGI329650[] = {{&S9061[89], 5}, {NULL}};
const Segment SGI329651[] = {{&S9061[12], 14}, {&S9061[48], 11}, {NULL}};
const Segment SGI329652[] = {{&S9061[12], 14}, {NULL}};
const Segment SGI329653[] = {{&S9062[6], 4}, {NULL}};
const Segment SGI329654[] = {{&S9062[89], 5}, {NULL}};
const Segment SGI329655[] = {{&S9062[12], 14}, {&S9062[48], 11}, {NULL}};
const Segment SGI329656[] = {{&S9062[12], 14}, {NULL}};
const Segment SGI329657[] = {{&S9063[6], 4}, {NULL}};
const Segment SGI329658[] = {{&S9063[81], 5}, {NULL}};
const Segment SGI329659[] = {{&S9063[12], 14}, {&S9063[40], 11}, {NULL}};
const Segment SGI329660[] = {{&S9063[12], 14}, {NULL}};
const Segment SGI329661[] = {{&S9064[6], 4}, {NULL}};
const Segment SGI329662[] = {{&S9064[82], 5}, {NULL}};
const Segment SGI329663[] = {{&S9064[12], 14}, {&S9064[40], 12}, {NULL}};
const Segment SGI329664[] = {{&S9064[12], 14}, {NULL}};
const Segment SGI329665[] = {{&S9065[6], 4}, {NULL}};
const Segment SGI329666[] = {{&S9065[81], 5}, {NULL}};
const Segment SGI329667[] = {{&S9065[12], 14}, {&S9065[40], 11}, {NULL}};
const Segment SGI329668[] = {{&S9065[12], 14}, {NULL}};
const Segment SGI329669[] = {{&S9066[6], 4}, {NULL}};
const Segment SGI329670[] = {{&S9066[88], 5}, {NULL}};
const Segment SGI329671[] = {{&S9066[12], 14}, {&S9066[47], 11}, {NULL}};
const Segment SGI329672[] = {{&S9066[12], 14}, {NULL}};
const Segment SGI329673[] = {{&S9067[6], 4}, {NULL}};
const Segment SGI329674[] = {{&S9067[88], 5}, {NULL}};
const Segment SGI329675[] = {{&S9067[12], 14}, {&S9067[47], 11}, {NULL}};
const Segment SGI329676[] = {{&S9067[12], 14}, {NULL}};
const Segment SGI329677[] = {{&S9068[6], 4}, {NULL}};
const Segment SGI329678[] = {{&S9068[88], 5}, {NULL}};
const Segment SGI329679[] = {{&S9068[12], 14}, {&S9068[47], 11}, {NULL}};
const Segment SGI329680[] = {{&S9068[12], 14}, {NULL}};
const Segment SGI329681[] = {{&S9069[6], 4}, {NULL}};
const Segment SGI329682[] = {{&S9069[89], 5}, {NULL}};
const Segment SGI329683[] = {{&S9069[12], 14}, {&S9069[48], 11}, {NULL}};
const Segment SGI329684[] = {{&S9069[12], 14}, {NULL}};
const Segment SGI329685[] = {{&S9070[6], 4}, {NULL}};
const Segment SGI329686[] = {{&S9070[81], 5}, {NULL}};
const Segment SGI329687[] = {{&S9070[12], 14}, {&S9070[40], 11}, {NULL}};
const Segment SGI329688[] = {{&S9070[12], 14}, {NULL}};
const Segment SGI329689[] = {{&S9071[6], 4}, {NULL}};
const Segment SGI329690[] = {{&S9071[82], 5}, {NULL}};
const Segment SGI329691[] = {{&S9071[12], 14}, {&S9071[41], 11}, {NULL}};
const Segment SGI329692[] = {{&S9071[12], 14}, {NULL}};
const Segment SGI329693[] = {{&S9072[6], 4}, {NULL}};
const Segment SGI329694[] = {{&S9072[82], 5}, {NULL}};
const Segment SGI329695[] = {{&S9072[12], 14}, {&S9072[41], 11}, {NULL}};
const Segment SGI329696[] = {{&S9072[12], 14}, {NULL}};
const Segment SGI329697[] = {{&S9073[6], 4}, {NULL}};
const Segment SGI329698[] = {{&S9073[81], 5}, {NULL}};
const Segment SGI329699[] = {{&S9073[12], 14}, {&S9073[40], 11}, {NULL}};
const Segment SGI329700[] = {{&S9073[12], 14}, {NULL}};
const Segment SGI329701[] = {{&S9074[6], 4}, {NULL}};
const Segment SGI329702[] = {{&S9074[81], 5}, {NULL}};
const Segment SGI329703[] = {{&S9074[12], 14}, {&S9074[40], 11}, {NULL}};
const Segment SGI329704[] = {{&S9074[12], 14}, {NULL}};
const Segment SGI329705[] = {{&S9075[6], 4}, {NULL}};
const Segment SGI329706[] = {{&S9075[82], 5}, {NULL}};
const Segment SGI329707[] = {{&S9075[12], 14}, {&S9075[40], 12}, {NULL}};
const Segment SGI329708[] = {{&S9075[12], 14}, {NULL}};
const Segment SGI329709[] = {{&S9076[6], 4}, {NULL}};
const Segment SGI329710[] = {{&S9076[81], 5}, {NULL}};
const Segment SGI329711[] = {{&S9076[12], 14}, {&S9076[40], 11}, {NULL}};
const Segment SGI329712[] = {{&S9076[12], 14}, {NULL}};
const Segment SGI329713[] = {{&S9077[6], 4}, {NULL}};
const Segment SGI329714[] = {{&S9077[82], 5}, {NULL}};
const Segment SGI329715[] = {{&S9077[12], 14}, {&S9077[40], 12}, {NULL}};
const Segment SGI329716[] = {{&S9077[12], 14}, {NULL}};
const Segment SGI329717[] = {{&S9078[6], 4}, {NULL}};
const Segment SGI329718[] = {{&S9078[82], 5}, {NULL}};
const Segment SGI329719[] = {{&S9078[12], 14}, {&S9078[41], 11}, {NULL}};
const Segment SGI329720[] = {{&S9078[12], 14}, {NULL}};
const Segment SGI329721[] = {{&S9079[6], 4}, {NULL}};
const Segment SGI329722[] = {{&S9079[89], 5}, {NULL}};
const Segment SGI329723[] = {{&S9079[12], 14}, {&S9079[47], 12}, {NULL}};
const Segment SGI329724[] = {{&S9079[12], 14}, {NULL}};
const Segment SGI329725[] = {{&S9080[6], 4}, {NULL}};
const Segment SGI329726[] = {{&S9080[88], 5}, {NULL}};
const Segment SGI329727[] = {{&S9080[12], 14}, {&S9080[47], 11}, {NULL}};
const Segment SGI329728[] = {{&S9080[12], 14}, {NULL}};
const Segment SGI329729[] = {{&S9081[6], 4}, {NULL}};
const Segment SGI329730[] = {{&S9081[81], 5}, {NULL}};
const Segment SGI329731[] = {{&S9081[12], 14}, {&S9081[40], 11}, {NULL}};
const Segment SGI329732[] = {{&S9081[12], 14}, {NULL}};
const Segment SGI329733[] = {{&S9082[6], 4}, {NULL}};
const Segment SGI329734[] = {{&S9082[88], 5}, {NULL}};
const Segment SGI329735[] = {{&S9082[12], 14}, {&S9082[47], 11}, {NULL}};
const Segment SGI329736[] = {{&S9082[12], 14}, {NULL}};
const Segment SGI329737[] = {{&S9083[6], 4}, {NULL}};
const Segment SGI329738[] = {{&S9083[89], 5}, {NULL}};
const Segment SGI329739[] = {{&S9083[12], 14}, {&S9083[47], 12}, {NULL}};
const Segment SGI329740[] = {{&S9083[12], 14}, {NULL}};
const Segment SGI329741[] = {{&S9084[6], 4}, {NULL}};
const Segment SGI329742[] = {{&S9084[89], 5}, {NULL}};
const Segment SGI329743[] = {{&S9084[12], 14}, {&S9084[48], 11}, {NULL}};
const Segment SGI329744[] = {{&S9084[12], 14}, {NULL}};
const Segment SGI329745[] = {{&S9085[6], 4}, {NULL}};
const Segment SGI329746[] = {{&S9085[82], 5}, {NULL}};
const Segment SGI329747[] = {{&S9085[12], 14}, {&S9085[40], 12}, {NULL}};
const Segment SGI329748[] = {{&S9085[12], 14}, {NULL}};
const Segment SGI329749[] = {{&S9086[6], 4}, {NULL}};
const Segment SGI329750[] = {{&S9086[82], 5}, {NULL}};
const Segment SGI329751[] = {{&S9086[12], 14}, {&S9086[40], 12}, {NULL}};
const Segment SGI329752[] = {{&S9086[12], 14}, {NULL}};
const Segment SGI329753[] = {{&S9087[6], 4}, {NULL}};
const Segment SGI329754[] = {{&S9087[88], 5}, {NULL}};
const Segment SGI329755[] = {{&S9087[12], 14}, {&S9087[47], 11}, {NULL}};
const Segment SGI329756[] = {{&S9087[12], 14}, {NULL}};
const Segment SGI329757[] = {{&S9088[6], 4}, {NULL}};
const Segment SGI329758[] = {{&S9088[89], 5}, {NULL}};
const Segment SGI329759[] = {{&S9088[12], 14}, {&S9088[47], 12}, {NULL}};
const Segment SGI329760[] = {{&S9088[12], 14}, {NULL}};
const Segment SGI329761[] = {{&S9089[6], 4}, {NULL}};
const Segment SGI329762[] = {{&S9089[89], 5}, {NULL}};
const Segment SGI329763[] = {{&S9089[12], 14}, {&S9089[47], 12}, {NULL}};
const Segment SGI329764[] = {{&S9089[12], 14}, {NULL}};
const Segment SGI329765[] = {{&S9090[6], 4}, {NULL}};
const Segment SGI329766[] = {{&S9090[88], 5}, {NULL}};
const Segment SGI329767[] = {{&S9090[12], 14}, {&S9090[47], 11}, {NULL}};
const Segment SGI329768[] = {{&S9090[12], 14}, {NULL}};
const Segment SGI329769[] = {{&S9091[6], 4}, {NULL}};
const Segment SGI329770[] = {{&S9091[59], 5}, {NULL}};
const Segment SGI329771[] = {{&S9091[12], 14}, {&S9091[70], 4}, {NULL}};
const Segment SGI329772[] = {{&S9091[12], 14}, {NULL}};
const Segment SGI329773[] = {{&S9092[6], 4}, {NULL}};
const Segment SGI329774[] = {{&S9092[81], 5}, {NULL}};
const Segment SGI329775[] = {{&S9092[12], 14}, {&S9092[40], 11}, {NULL}};
const Segment SGI329776[] = {{&S9092[12], 14}, {NULL}};
const Segment SGI329777[] = {{&S9093[6], 4}, {NULL}};
const Segment SGI329778[] = {{&S9093[82], 5}, {NULL}};
const Segment SGI329779[] = {{&S9093[12], 14}, {&S9093[40], 12}, {NULL}};
const Segment SGI329780[] = {{&S9093[12], 14}, {NULL}};
const Segment SGI329781[] = {{&S9094[6], 4}, {NULL}};
const Segment SGI329782[] = {{&S9094[82], 5}, {NULL}};
const Segment SGI329783[] = {{&S9094[12], 14}, {&S9094[41], 11}, {NULL}};
const Segment SGI329784[] = {{&S9094[12], 14}, {NULL}};
const Segment SGI329785[] = {{&S9095[6], 4}, {NULL}};
const Segment SGI329786[] = {{&S9095[82], 5}, {NULL}};
const Segment SGI329787[] = {{&S9095[12], 14}, {&S9095[41], 11}, {NULL}};
const Segment SGI329788[] = {{&S9095[12], 14}, {NULL}};
const Segment SGI329789[] = {{&S9096[6], 4}, {NULL}};
const Segment SGI329790[] = {{&S9096[81], 5}, {NULL}};
const Segment SGI329791[] = {{&S9096[12], 14}, {&S9096[40], 11}, {NULL}};
const Segment SGI329792[] = {{&S9096[12], 14}, {NULL}};
const Segment SGI329793[] = {{&S9097[6], 4}, {NULL}};
const Segment SGI329794[] = {{&S9097[81], 5}, {NULL}};
const Segment SGI329795[] = {{&S9097[12], 14}, {&S9097[40], 11}, {NULL}};
const Segment SGI329796[] = {{&S9097[12], 14}, {NULL}};
const Segment SGI329797[] = {{&S9098[6], 4}, {NULL}};
const Segment SGI329798[] = {{&S9098[88], 5}, {NULL}};
const Segment SGI329799[] = {{&S9098[12], 14}, {&S9098[47], 11}, {NULL}};
const Segment SGI329800[] = {{&S9098[12], 14}, {NULL}};
const Segment SGI329801[] = {{&S9099[6], 4}, {NULL}};
const Segment SGI329802[] = {{&S9099[88], 5}, {NULL}};
const Segment SGI329803[] = {{&S9099[12], 14}, {&S9099[47], 11}, {NULL}};
const Segment SGI329804[] = {{&S9099[12], 14}, {NULL}};
const Segment SGI329805[] = {{&S9100[6], 4}, {NULL}};
const Segment SGI329806[] = {{&S9100[88], 5}, {NULL}};
const Segment SGI329807[] = {{&S9100[12], 14}, {&S9100[47], 11}, {NULL}};
const Segment SGI329808[] = {{&S9100[12], 14}, {NULL}};
const Segment SGI329809[] = {{&S9101[6], 4}, {NULL}};
const Segment SGI329810[] = {{&S9101[88], 5}, {NULL}};
const Segment SGI329811[] = {{&S9101[12], 14}, {&S9101[47], 11}, {NULL}};
const Segment SGI329812[] = {{&S9101[12], 14}, {NULL}};
const Segment SGI329813[] = {{&S9102[6], 4}, {NULL}};
const Segment SGI329814[] = {{&S9102[82], 5}, {NULL}};
const Segment SGI329815[] = {{&S9102[12], 14}, {&S9102[40], 12}, {NULL}};
const Segment SGI329816[] = {{&S9102[12], 14}, {NULL}};
const Segment SGI329817[] = {{&S9103[6], 4}, {NULL}};
const Segment SGI329818[] = {{&S9103[81], 5}, {NULL}};
const Segment SGI329819[] = {{&S9103[12], 14}, {&S9103[40], 11}, {NULL}};
const Segment SGI329820[] = {{&S9103[12], 14}, {NULL}};
const Segment SGI329821[] = {{&S9105[6], 4}, {NULL}};
const Segment SGI329822[] = {{&S9105[88], 5}, {NULL}};
const Segment SGI329823[] = {{&S9105[12], 14}, {&S9105[47], 11}, {NULL}};
const Segment SGI329824[] = {{&S9105[12], 14}, {NULL}};
const Segment SGI329825[] = {{&S9106[6], 4}, {NULL}};
const Segment SGI329826[] = {{&S9106[88], 5}, {NULL}};
const Segment SGI329827[] = {{&S9106[12], 14}, {&S9106[47], 11}, {NULL}};
const Segment SGI329828[] = {{&S9106[12], 14}, {NULL}};
const Segment SGI329829[] = {{&S9107[6], 4}, {NULL}};
const Segment SGI329830[] = {{&S9107[81], 5}, {NULL}};
const Segment SGI329831[] = {{&S9107[12], 14}, {&S9107[40], 11}, {NULL}};
const Segment SGI329832[] = {{&S9107[12], 14}, {NULL}};
const Segment SGI329833[] = {{&S9108[6], 4}, {NULL}};
const Segment SGI329834[] = {{&S9108[81], 5}, {NULL}};
const Segment SGI329835[] = {{&S9108[12], 14}, {&S9108[40], 11}, {NULL}};
const Segment SGI329836[] = {{&S9108[12], 14}, {NULL}};
const Segment SGI329837[] = {{&S9109[6], 4}, {NULL}};
const Segment SGI329838[] = {{&S9109[81], 5}, {NULL}};
const Segment SGI329839[] = {{&S9109[12], 14}, {&S9109[40], 11}, {NULL}};
const Segment SGI329840[] = {{&S9109[12], 14}, {NULL}};
const Segment SGI329841[] = {{&S9110[6], 4}, {NULL}};
const Segment SGI329842[] = {{&S9110[82], 5}, {NULL}};
const Segment SGI329843[] = {{&S9110[12], 14}, {&S9110[40], 12}, {NULL}};
const Segment SGI329844[] = {{&S9110[12], 14}, {NULL}};
const Segment SGI329845[] = {{&S9111[6], 4}, {NULL}};
const Segment SGI329846[] = {{&S9111[89], 5}, {NULL}};
const Segment SGI329847[] = {{&S9111[12], 14}, {&S9111[48], 11}, {NULL}};
const Segment SGI329848[] = {{&S9111[12], 14}, {NULL}};
const Segment SGI329849[] = {{&S9112[6], 4}, {NULL}};
const Segment SGI329850[] = {{&S9112[88], 5}, {NULL}};
const Segment SGI329851[] = {{&S9112[12], 14}, {&S9112[47], 11}, {NULL}};
const Segment SGI329852[] = {{&S9112[12], 14}, {NULL}};
const Segment SGI329853[] = {{&S9113[6], 4}, {NULL}};
const Segment SGI329854[] = {{&S9113[88], 5}, {NULL}};
const Segment SGI329855[] = {{&S9113[12], 14}, {&S9113[47], 11}, {NULL}};
const Segment SGI329856[] = {{&S9113[12], 14}, {NULL}};
const Segment SGI329857[] = {{&S9114[6], 5}, {NULL}};
const Segment SGI329858[] = {{&S9114[61], 5}, {NULL}};
const Segment SGI329859[] = {{&S9114[13], 14}, {NULL}};
const Segment SGI329860[] = {{&S9115[6], 4}, {NULL}};
const Segment SGI329861[] = {{&S9115[82], 5}, {NULL}};
const Segment SGI329862[] = {{&S9115[12], 14}, {&S9115[41], 11}, {NULL}};
const Segment SGI329863[] = {{&S9115[12], 14}, {NULL}};
const Segment SGI329864[] = {{&S9116[6], 4}, {NULL}};
const Segment SGI329865[] = {{&S9116[81], 5}, {NULL}};
const Segment SGI329866[] = {{&S9116[12], 14}, {&S9116[40], 11}, {NULL}};
const Segment SGI329867[] = {{&S9116[12], 14}, {NULL}};
const Segment SGI329868[] = {{&S9117[6], 4}, {NULL}};
const Segment SGI329869[] = {{&S9117[81], 5}, {NULL}};
const Segment SGI329870[] = {{&S9117[12], 14}, {&S9117[40], 11}, {NULL}};
const Segment SGI329871[] = {{&S9117[12], 14}, {NULL}};
const Segment SGI329872[] = {{&S9118[6], 4}, {NULL}};
const Segment SGI329873[] = {{&S9118[81], 5}, {NULL}};
const Segment SGI329874[] = {{&S9118[12], 14}, {&S9118[40], 11}, {NULL}};
const Segment SGI329875[] = {{&S9118[12], 14}, {NULL}};
const Segment SGI329876[] = {{&S9119[6], 4}, {NULL}};
const Segment SGI329877[] = {{&S9119[88], 5}, {NULL}};
const Segment SGI329878[] = {{&S9119[12], 14}, {&S9119[47], 11}, {NULL}};
const Segment SGI329879[] = {{&S9119[12], 14}, {NULL}};
const Segment SGI329880[] = {{&S9120[6], 4}, {NULL}};
const Segment SGI329881[] = {{&S9120[81], 5}, {NULL}};
const Segment SGI329882[] = {{&S9120[12], 14}, {&S9120[40], 11}, {NULL}};
const Segment SGI329883[] = {{&S9120[12], 14}, {NULL}};
const Segment SGI329884[] = {{&S9121[6], 4}, {NULL}};
const Segment SGI329885[] = {{&S9121[88], 5}, {NULL}};
const Segment SGI329886[] = {{&S9121[12], 14}, {&S9121[47], 11}, {NULL}};
const Segment SGI329887[] = {{&S9121[12], 14}, {NULL}};
const Segment SGI329888[] = {{&S9122[6], 4}, {NULL}};
const Segment SGI329889[] = {{&S9122[88], 5}, {NULL}};
const Segment SGI329890[] = {{&S9122[12], 14}, {&S9122[47], 11}, {NULL}};
const Segment SGI329891[] = {{&S9122[12], 14}, {NULL}};
const Segment SGI329892[] = {{&S9123[6], 4}, {NULL}};
const Segment SGI329893[] = {{&S9123[81], 5}, {NULL}};
const Segment SGI329894[] = {{&S9123[12], 14}, {&S9123[40], 11}, {NULL}};
const Segment SGI329895[] = {{&S9123[12], 14}, {NULL}};
const Segment SGI329896[] = {{&S9124[6], 4}, {NULL}};
const Segment SGI329897[] = {{&S9124[81], 5}, {NULL}};
const Segment SGI329898[] = {{&S9124[12], 14}, {&S9124[40], 11}, {NULL}};
const Segment SGI329899[] = {{&S9124[12], 14}, {NULL}};
const Segment SGI329900[] = {{&S9125[6], 4}, {NULL}};
const Segment SGI329901[] = {{&S9125[81], 5}, {NULL}};
const Segment SGI329902[] = {{&S9125[12], 14}, {&S9125[40], 11}, {NULL}};
const Segment SGI329903[] = {{&S9125[12], 14}, {NULL}};
const Segment SGI329904[] = {{&S9126[6], 4}, {NULL}};
const Segment SGI329905[] = {{&S9126[81], 5}, {NULL}};
const Segment SGI329906[] = {{&S9126[12], 14}, {&S9126[40], 11}, {NULL}};
const Segment SGI329907[] = {{&S9126[12], 14}, {NULL}};
const Segment SGI329908[] = {{&S9127[6], 4}, {NULL}};
const Segment SGI329909[] = {{&S9127[81], 5}, {NULL}};
const Segment SGI329910[] = {{&S9127[12], 14}, {&S9127[40], 11}, {NULL}};
const Segment SGI329911[] = {{&S9127[12], 14}, {NULL}};
const Segment SGI329912[] = {{&S9128[6], 4}, {NULL}};
const Segment SGI329913[] = {{&S9128[81], 5}, {NULL}};
const Segment SGI329914[] = {{&S9128[12], 14}, {&S9128[40], 11}, {NULL}};
const Segment SGI329915[] = {{&S9128[12], 14}, {NULL}};
const Segment SGI329916[] = {{&S9129[6], 4}, {NULL}};
const Segment SGI329917[] = {{&S9129[81], 5}, {NULL}};
const Segment SGI329918[] = {{&S9129[12], 14}, {&S9129[40], 11}, {NULL}};
const Segment SGI329919[] = {{&S9129[12], 14}, {NULL}};
const Segment SGI329920[] = {{&S9130[6], 4}, {NULL}};
const Segment SGI329921[] = {{&S9130[82], 5}, {NULL}};
const Segment SGI329922[] = {{&S9130[12], 14}, {&S9130[40], 12}, {NULL}};
const Segment SGI329923[] = {{&S9130[12], 14}, {NULL}};
const Segment SGI329924[] = {{&S9131[6], 4}, {NULL}};
const Segment SGI329925[] = {{&S9131[81], 5}, {NULL}};
const Segment SGI329926[] = {{&S9131[12], 14}, {&S9131[40], 11}, {NULL}};
const Segment SGI329927[] = {{&S9131[12], 14}, {NULL}};
const Segment SGI329928[] = {{&S9132[6], 4}, {NULL}};
const Segment SGI329929[] = {{&S9132[88], 5}, {NULL}};
const Segment SGI329930[] = {{&S9132[12], 14}, {&S9132[47], 11}, {NULL}};
const Segment SGI329931[] = {{&S9132[12], 14}, {NULL}};
const Segment SGI329932[] = {{&S9133[6], 4}, {NULL}};
const Segment SGI329933[] = {{&S9133[82], 5}, {NULL}};
const Segment SGI329934[] = {{&S9133[12], 14}, {&S9133[40], 12}, {NULL}};
const Segment SGI329935[] = {{&S9133[12], 14}, {NULL}};
const Segment SGI329936[] = {{&S9134[6], 4}, {NULL}};
const Segment SGI329937[] = {{&S9134[81], 5}, {NULL}};
const Segment SGI329938[] = {{&S9134[12], 14}, {&S9134[40], 11}, {NULL}};
const Segment SGI329939[] = {{&S9134[12], 14}, {NULL}};
const Segment SGI329940[] = {{&S9135[6], 4}, {NULL}};
const Segment SGI329941[] = {{&S9135[88], 5}, {NULL}};
const Segment SGI329942[] = {{&S9135[12], 14}, {&S9135[47], 11}, {NULL}};
const Segment SGI329943[] = {{&S9135[12], 14}, {NULL}};
const Segment SGI329944[] = {{&S9136[6], 4}, {NULL}};
const Segment SGI329945[] = {{&S9136[81], 5}, {NULL}};
const Segment SGI329946[] = {{&S9136[12], 14}, {&S9136[40], 11}, {NULL}};
const Segment SGI329947[] = {{&S9136[12], 14}, {NULL}};
const Segment SGI329948[] = {{&S9137[6], 4}, {NULL}};
const Segment SGI329949[] = {{&S9137[81], 5}, {NULL}};
const Segment SGI329950[] = {{&S9137[12], 14}, {&S9137[40], 11}, {NULL}};
const Segment SGI329951[] = {{&S9137[12], 14}, {NULL}};
const Segment SGI329952[] = {{&S9138[6], 4}, {NULL}};
const Segment SGI329953[] = {{&S9138[59], 5}, {NULL}};
const Segment SGI329954[] = {{&S9138[12], 14}, {&S9138[70], 33}, {NULL}};
const Segment SGI329955[] = {{&S9138[12], 14}, {NULL}};
const Segment SGI329956[] = {{&S9139[6], 4}, {NULL}};
const Segment SGI329957[] = {{&S9139[88], 5}, {NULL}};
const Segment SGI329958[] = {{&S9139[12], 14}, {&S9139[47], 11}, {NULL}};
const Segment SGI329959[] = {{&S9139[12], 14}, {NULL}};
const Segment SGI329960[] = {{&S9140[6], 4}, {NULL}};
const Segment SGI329961[] = {{&S9140[81], 5}, {NULL}};
const Segment SGI329962[] = {{&S9140[12], 14}, {&S9140[40], 11}, {NULL}};
const Segment SGI329963[] = {{&S9140[12], 14}, {NULL}};
const Segment SGI329964[] = {{&S9141[6], 4}, {NULL}};
const Segment SGI329965[] = {{&S9141[88], 5}, {NULL}};
const Segment SGI329966[] = {{&S9141[12], 14}, {&S9141[47], 11}, {NULL}};
const Segment SGI329967[] = {{&S9141[12], 14}, {NULL}};
const Segment SGI329968[] = {{&S9142[6], 4}, {NULL}};
const Segment SGI329969[] = {{&S9142[81], 5}, {NULL}};
const Segment SGI329970[] = {{&S9142[12], 14}, {&S9142[40], 11}, {NULL}};
const Segment SGI329971[] = {{&S9142[12], 14}, {NULL}};
const Segment SGI329972[] = {{&S9143[6], 4}, {NULL}};
const Segment SGI329973[] = {{&S9143[81], 5}, {NULL}};
const Segment SGI329974[] = {{&S9143[12], 14}, {&S9143[40], 11}, {NULL}};
const Segment SGI329975[] = {{&S9143[12], 14}, {NULL}};
const Segment SGI329976[] = {{&S9144[6], 4}, {NULL}};
const Segment SGI329977[] = {{&S9144[81], 5}, {NULL}};
const Segment SGI329978[] = {{&S9144[12], 14}, {&S9144[40], 11}, {NULL}};
const Segment SGI329979[] = {{&S9144[12], 14}, {NULL}};
const Segment SGI329980[] = {{&S9145[6], 4}, {NULL}};
const Segment SGI329981[] = {{&S9145[81], 5}, {NULL}};
const Segment SGI329982[] = {{&S9145[12], 14}, {&S9145[40], 11}, {NULL}};
const Segment SGI329983[] = {{&S9145[12], 14}, {NULL}};
const Segment SGI329984[] = {{&S33877[30], 34}, {NULL}};
const Segment SGI329985[] = {{&S33882[5], 3}, {NULL}};
const Segment SGI329986[] = {{&S34700[5], 7}, {NULL}};
const Segment SGI329987[] = {{&S34701[5], 5}, {NULL}};
const Segment SGI329988[] = {{&S34702[5], 5}, {&S34702[68], 5}, {NULL}};
const Segment SGI329989[] = {{&S34703[5], 5}, {&S34703[5], 5}, {NULL}};
const Segment SGI329990[] = {{&S10925[64], 5}, {NULL}};
const Segment SGI329991[] = {{&S10925[13], 10}, {NULL}};
const Segment SGI329992[] = {{&S11182[64], 5}, {NULL}};
const Segment SGI329993[] = {{&S11182[13], 10}, {NULL}};
const Segment SGI329994[] = {{&S10917[6], 4}, {NULL}};
const Segment SGI329995[] = {{&S10917[59], 5}, {NULL}};
const Segment SGI329996[] = {{&S10917[12], 14}, {NULL}};
const Segment SGI329997[] = {{&S10918[6], 4}, {NULL}};
const Segment SGI329998[] = {{&S10918[52], 5}, {NULL}};
const Segment SGI329999[] = {{&S10918[12], 45}, {NULL}};
const Segment SGI330000[] = {{&S10918[12], 14}, {NULL}};
const Segment SGI330001[] = {{&S10919[6], 4}, {NULL}};
const Segment SGI330002[] = {{&S10919[75], 5}, {NULL}};
const Segment SGI330003[] = {{&S10919[12], 14}, {&S10919[37], 43}, {NULL}};
const Segment SGI330004[] = {{&S10919[12], 14}, {NULL}};
const Segment SGI330005[] = {{&S10920[6], 4}, {NULL}};
const Segment SGI330006[] = {{&S10920[81], 5}, {NULL}};
const Segment SGI330007[] = {{&S10920[12], 14}, {&S10920[44], 42}, {NULL}};
const Segment SGI330008[] = {{&S10920[12], 14}, {NULL}};
const Segment SGI330009[] = {{&S10922[6], 4}, {NULL}};
const Segment SGI330010[] = {{&S10922[81], 5}, {NULL}};
const Segment SGI330011[] = {{&S10922[12], 14}, {&S10922[44], 42}, {NULL}};
const Segment SGI330012[] = {{&S10922[12], 14}, {NULL}};
const Segment SGI330013[] = {{&S10923[6], 4}, {NULL}};
const Segment SGI330014[] = {{&S10923[81], 5}, {NULL}};
const Segment SGI330015[] = {{&S10923[12], 14}, {&S10923[44], 42}, {NULL}};
const Segment SGI330016[] = {{&S10923[12], 14}, {NULL}};
const Segment SGI330017[] = {{&S10924[6], 4}, {NULL}};
const Segment SGI330018[] = {{&S10924[74], 5}, {NULL}};
const Segment SGI330019[] = {{&S10924[12], 14}, {&S10924[37], 42}, {NULL}};
const Segment SGI330020[] = {{&S10924[12], 14}, {NULL}};
const Segment SGI330021[] = {{&S10926[6], 4}, {NULL}};
const Segment SGI330022[] = {{&S10926[81], 5}, {NULL}};
const Segment SGI330023[] = {{&S10926[12], 14}, {&S10926[44], 42}, {NULL}};
const Segment SGI330024[] = {{&S10926[12], 14}, {NULL}};
const Segment SGI330025[] = {{&S10927[6], 4}, {NULL}};
const Segment SGI330026[] = {{&S10927[81], 5}, {NULL}};
const Segment SGI330027[] = {{&S10927[12], 14}, {&S10927[44], 42}, {NULL}};
const Segment SGI330028[] = {{&S10927[12], 14}, {NULL}};
const Segment SGI330029[] = {{&S10928[6], 4}, {NULL}};
const Segment SGI330030[] = {{&S10928[81], 5}, {NULL}};
const Segment SGI330031[] = {{&S10928[12], 14}, {&S10928[44], 42}, {NULL}};
const Segment SGI330032[] = {{&S10928[12], 14}, {NULL}};
const Segment SGI330033[] = {{&S10929[6], 4}, {NULL}};
const Segment SGI330034[] = {{&S10929[75], 5}, {NULL}};
const Segment SGI330035[] = {{&S10929[12], 14}, {&S10929[38], 42}, {NULL}};
const Segment SGI330036[] = {{&S10929[12], 14}, {NULL}};
const Segment SGI330037[] = {{&S10930[6], 4}, {NULL}};
const Segment SGI330038[] = {{&S10930[82], 5}, {NULL}};
const Segment SGI330039[] = {{&S10930[12], 14}, {&S10930[45], 42}, {NULL}};
const Segment SGI330040[] = {{&S10930[12], 14}, {NULL}};
const Segment SGI330041[] = {{&S10931[6], 4}, {NULL}};
const Segment SGI330042[] = {{&S10931[81], 5}, {NULL}};
const Segment SGI330043[] = {{&S10931[12], 14}, {&S10931[44], 42}, {NULL}};
const Segment SGI330044[] = {{&S10931[12], 14}, {NULL}};
const Segment SGI330045[] = {{&S10932[6], 4}, {NULL}};
const Segment SGI330046[] = {{&S10932[75], 5}, {NULL}};
const Segment SGI330047[] = {{&S10932[12], 14}, {&S10932[38], 42}, {NULL}};
const Segment SGI330048[] = {{&S10932[12], 14}, {NULL}};
const Segment SGI330049[] = {{&S10933[6], 4}, {NULL}};
const Segment SGI330050[] = {{&S10933[81], 5}, {NULL}};
const Segment SGI330051[] = {{&S10933[12], 14}, {&S10933[44], 42}, {NULL}};
const Segment SGI330052[] = {{&S10933[12], 14}, {NULL}};
const Segment SGI330053[] = {{&S10934[6], 4}, {NULL}};
const Segment SGI330054[] = {{&S10934[74], 5}, {NULL}};
const Segment SGI330055[] = {{&S10934[12], 14}, {&S10934[37], 42}, {NULL}};
const Segment SGI330056[] = {{&S10934[12], 14}, {NULL}};
const Segment SGI330057[] = {{&S10935[6], 4}, {NULL}};
const Segment SGI330058[] = {{&S10935[74], 5}, {NULL}};
const Segment SGI330059[] = {{&S10935[12], 14}, {&S10935[37], 42}, {NULL}};
const Segment SGI330060[] = {{&S10935[12], 14}, {NULL}};
const Segment SGI330061[] = {{&S10936[6], 4}, {NULL}};
const Segment SGI330062[] = {{&S10936[81], 5}, {NULL}};
const Segment SGI330063[] = {{&S10936[12], 14}, {&S10936[44], 42}, {NULL}};
const Segment SGI330064[] = {{&S10936[12], 14}, {NULL}};
const Segment SGI330065[] = {{&S10937[6], 4}, {NULL}};
const Segment SGI330066[] = {{&S10937[82], 5}, {NULL}};
const Segment SGI330067[] = {{&S10937[12], 14}, {&S10937[45], 42}, {NULL}};
const Segment SGI330068[] = {{&S10937[12], 14}, {NULL}};
const Segment SGI330069[] = {{&S10939[6], 4}, {NULL}};
const Segment SGI330070[] = {{&S10939[81], 5}, {NULL}};
const Segment SGI330071[] = {{&S10939[12], 14}, {&S10939[44], 42}, {NULL}};
const Segment SGI330072[] = {{&S10939[12], 14}, {NULL}};
const Segment SGI330073[] = {{&S10940[6], 4}, {NULL}};
const Segment SGI330074[] = {{&S10940[81], 5}, {NULL}};
const Segment SGI330075[] = {{&S10940[12], 14}, {&S10940[44], 42}, {NULL}};
const Segment SGI330076[] = {{&S10940[12], 14}, {NULL}};
const Segment SGI330077[] = {{&S10941[6], 4}, {NULL}};
const Segment SGI330078[] = {{&S10941[81], 5}, {NULL}};
const Segment SGI330079[] = {{&S10941[12], 14}, {&S10941[44], 42}, {NULL}};
const Segment SGI330080[] = {{&S10941[12], 14}, {NULL}};
const Segment SGI330081[] = {{&S10942[6], 4}, {NULL}};
const Segment SGI330082[] = {{&S10942[81], 5}, {NULL}};
const Segment SGI330083[] = {{&S10942[12], 14}, {&S10942[44], 42}, {NULL}};
const Segment SGI330084[] = {{&S10942[12], 14}, {NULL}};
const Segment SGI330085[] = {{&S10943[6], 4}, {NULL}};
const Segment SGI330086[] = {{&S10943[82], 5}, {NULL}};
const Segment SGI330087[] = {{&S10943[12], 14}, {&S10943[45], 42}, {NULL}};
const Segment SGI330088[] = {{&S10943[12], 14}, {NULL}};
const Segment SGI330089[] = {{&S10944[6], 4}, {NULL}};
const Segment SGI330090[] = {{&S10944[75], 5}, {NULL}};
const Segment SGI330091[] = {{&S10944[12], 14}, {&S10944[38], 42}, {NULL}};
const Segment SGI330092[] = {{&S10944[12], 14}, {NULL}};
const Segment SGI330093[] = {{&S10945[6], 4}, {NULL}};
const Segment SGI330094[] = {{&S10945[81], 5}, {NULL}};
const Segment SGI330095[] = {{&S10945[12], 14}, {&S10945[44], 42}, {NULL}};
const Segment SGI330096[] = {{&S10945[12], 14}, {NULL}};
const Segment SGI330097[] = {{&S10946[6], 4}, {NULL}};
const Segment SGI330098[] = {{&S10946[81], 5}, {NULL}};
const Segment SGI330099[] = {{&S10946[12], 14}, {&S10946[44], 42}, {NULL}};
const Segment SGI330100[] = {{&S10946[12], 14}, {NULL}};
const Segment SGI330101[] = {{&S10947[6], 4}, {NULL}};
const Segment SGI330102[] = {{&S10947[81], 5}, {NULL}};
const Segment SGI330103[] = {{&S10947[12], 14}, {&S10947[44], 42}, {NULL}};
const Segment SGI330104[] = {{&S10947[12], 14}, {NULL}};
const Segment SGI330105[] = {{&S10948[6], 4}, {NULL}};
const Segment SGI330106[] = {{&S10948[81], 5}, {NULL}};
const Segment SGI330107[] = {{&S10948[12], 14}, {&S10948[44], 42}, {NULL}};
const Segment SGI330108[] = {{&S10948[12], 14}, {NULL}};
const Segment SGI330109[] = {{&S10949[6], 4}, {NULL}};
const Segment SGI330110[] = {{&S10949[81], 5}, {NULL}};
const Segment SGI330111[] = {{&S10949[12], 14}, {&S10949[44], 42}, {NULL}};
const Segment SGI330112[] = {{&S10949[12], 14}, {NULL}};
const Segment SGI330113[] = {{&S10950[6], 4}, {NULL}};
const Segment SGI330114[] = {{&S10950[81], 5}, {NULL}};
const Segment SGI330115[] = {{&S10950[12], 14}, {&S10950[44], 42}, {NULL}};
const Segment SGI330116[] = {{&S10950[12], 14}, {NULL}};
const Segment SGI330117[] = {{&S10951[6], 4}, {NULL}};
const Segment SGI330118[] = {{&S10951[81], 5}, {NULL}};
const Segment SGI330119[] = {{&S10951[12], 14}, {&S10951[44], 42}, {NULL}};
const Segment SGI330120[] = {{&S10951[12], 14}, {NULL}};
const Segment SGI330121[] = {{&S10952[6], 4}, {NULL}};
const Segment SGI330122[] = {{&S10952[74], 5}, {NULL}};
const Segment SGI330123[] = {{&S10952[12], 14}, {&S10952[37], 42}, {NULL}};
const Segment SGI330124[] = {{&S10952[12], 14}, {NULL}};
const Segment SGI330125[] = {{&S10953[6], 4}, {NULL}};
const Segment SGI330126[] = {{&S10953[52], 5}, {NULL}};
const Segment SGI330127[] = {{&S10953[12], 55}, {NULL}};
const Segment SGI330128[] = {{&S10953[12], 14}, {NULL}};
const Segment SGI330129[] = {{&S10954[6], 4}, {NULL}};
const Segment SGI330130[] = {{&S10954[81], 5}, {NULL}};
const Segment SGI330131[] = {{&S10954[12], 14}, {&S10954[44], 42}, {NULL}};
const Segment SGI330132[] = {{&S10954[12], 14}, {NULL}};
const Segment SGI330133[] = {{&S10955[6], 4}, {NULL}};
const Segment SGI330134[] = {{&S10955[81], 5}, {NULL}};
const Segment SGI330135[] = {{&S10955[12], 14}, {&S10955[44], 42}, {NULL}};
const Segment SGI330136[] = {{&S10955[12], 14}, {NULL}};
const Segment SGI330137[] = {{&S10956[6], 4}, {NULL}};
const Segment SGI330138[] = {{&S10956[52], 5}, {NULL}};
const Segment SGI330139[] = {{&S10956[12], 62}, {NULL}};
const Segment SGI330140[] = {{&S10956[12], 14}, {NULL}};
const Segment SGI330141[] = {{&S10957[6], 4}, {NULL}};
const Segment SGI330142[] = {{&S10957[81], 5}, {NULL}};
const Segment SGI330143[] = {{&S10957[12], 14}, {&S10957[44], 42}, {NULL}};
const Segment SGI330144[] = {{&S10957[12], 14}, {NULL}};
const Segment SGI330145[] = {{&S10958[6], 4}, {NULL}};
const Segment SGI330146[] = {{&S10958[81], 5}, {NULL}};
const Segment SGI330147[] = {{&S10958[12], 14}, {&S10958[44], 42}, {NULL}};
const Segment SGI330148[] = {{&S10958[12], 14}, {NULL}};
const Segment SGI330149[] = {{&S10959[6], 4}, {NULL}};
const Segment SGI330150[] = {{&S10959[81], 5}, {NULL}};
const Segment SGI330151[] = {{&S10959[12], 14}, {&S10959[44], 42}, {NULL}};
const Segment SGI330152[] = {{&S10959[12], 14}, {NULL}};
const Segment SGI330153[] = {{&S10960[6], 4}, {NULL}};
const Segment SGI330154[] = {{&S10960[81], 5}, {NULL}};
const Segment SGI330155[] = {{&S10960[12], 14}, {&S10960[44], 42}, {NULL}};
const Segment SGI330156[] = {{&S10960[12], 14}, {NULL}};
const Segment SGI330157[] = {{&S10961[6], 4}, {NULL}};
const Segment SGI330158[] = {{&S10961[81], 5}, {NULL}};
const Segment SGI330159[] = {{&S10961[12], 14}, {&S10961[44], 42}, {NULL}};
const Segment SGI330160[] = {{&S10961[12], 14}, {NULL}};
const Segment SGI330161[] = {{&S10962[6], 4}, {NULL}};
const Segment SGI330162[] = {{&S10962[81], 5}, {NULL}};
const Segment SGI330163[] = {{&S10962[12], 14}, {&S10962[44], 42}, {NULL}};
const Segment SGI330164[] = {{&S10962[12], 14}, {NULL}};
const Segment SGI330165[] = {{&S10963[6], 4}, {NULL}};
const Segment SGI330166[] = {{&S10963[81], 5}, {NULL}};
const Segment SGI330167[] = {{&S10963[12], 14}, {&S10963[44], 42}, {NULL}};
const Segment SGI330168[] = {{&S10963[12], 14}, {NULL}};
const Segment SGI330169[] = {{&S10964[6], 4}, {NULL}};
const Segment SGI330170[] = {{&S10964[81], 5}, {NULL}};
const Segment SGI330171[] = {{&S10964[12], 14}, {&S10964[44], 42}, {NULL}};
const Segment SGI330172[] = {{&S10964[12], 14}, {NULL}};
const Segment SGI330173[] = {{&S10965[6], 4}, {NULL}};
const Segment SGI330174[] = {{&S10965[75], 5}, {NULL}};
const Segment SGI330175[] = {{&S10965[12], 14}, {&S10965[37], 43}, {NULL}};
const Segment SGI330176[] = {{&S10965[12], 14}, {NULL}};
const Segment SGI330177[] = {{&S10966[6], 4}, {NULL}};
const Segment SGI330178[] = {{&S10966[82], 5}, {NULL}};
const Segment SGI330179[] = {{&S10966[12], 14}, {&S10966[45], 42}, {NULL}};
const Segment SGI330180[] = {{&S10966[12], 14}, {NULL}};
const Segment SGI330181[] = {{&S10967[6], 4}, {NULL}};
const Segment SGI330182[] = {{&S10967[81], 5}, {NULL}};
const Segment SGI330183[] = {{&S10967[12], 14}, {&S10967[44], 42}, {NULL}};
const Segment SGI330184[] = {{&S10967[12], 14}, {NULL}};
const Segment SGI330185[] = {{&S10968[6], 4}, {NULL}};
const Segment SGI330186[] = {{&S10968[81], 5}, {NULL}};
const Segment SGI330187[] = {{&S10968[12], 14}, {&S10968[44], 42}, {NULL}};
const Segment SGI330188[] = {{&S10968[12], 14}, {NULL}};
const Segment SGI330189[] = {{&S10969[6], 4}, {NULL}};
const Segment SGI330190[] = {{&S10969[81], 5}, {NULL}};
const Segment SGI330191[] = {{&S10969[12], 14}, {&S10969[44], 42}, {NULL}};
const Segment SGI330192[] = {{&S10969[12], 14}, {NULL}};
const Segment SGI330193[] = {{&S10970[6], 4}, {NULL}};
const Segment SGI330194[] = {{&S10970[82], 5}, {NULL}};
const Segment SGI330195[] = {{&S10970[12], 14}, {&S10970[45], 42}, {NULL}};
const Segment SGI330196[] = {{&S10970[12], 14}, {NULL}};
const Segment SGI330197[] = {{&S10971[6], 4}, {NULL}};
const Segment SGI330198[] = {{&S10971[75], 5}, {NULL}};
const Segment SGI330199[] = {{&S10971[12], 14}, {&S10971[37], 43}, {NULL}};
const Segment SGI330200[] = {{&S10971[12], 14}, {NULL}};
const Segment SGI330201[] = {{&S10972[6], 4}, {NULL}};
const Segment SGI330202[] = {{&S10972[59], 5}, {NULL}};
const Segment SGI330203[] = {{&S10972[12], 14}, {&S10972[70], 4}, {NULL}};
const Segment SGI330204[] = {{&S10972[12], 14}, {NULL}};
const Segment SGI330205[] = {{&S10973[6], 4}, {NULL}};
const Segment SGI330206[] = {{&S10973[81], 5}, {NULL}};
const Segment SGI330207[] = {{&S10973[12], 14}, {&S10973[44], 42}, {NULL}};
const Segment SGI330208[] = {{&S10973[12], 14}, {NULL}};
const Segment SGI330209[] = {{&S10974[6], 4}, {NULL}};
const Segment SGI330210[] = {{&S10974[81], 5}, {NULL}};
const Segment SGI330211[] = {{&S10974[12], 14}, {&S10974[44], 42}, {NULL}};
const Segment SGI330212[] = {{&S10974[12], 14}, {NULL}};
const Segment SGI330213[] = {{&S10975[6], 4}, {NULL}};
const Segment SGI330214[] = {{&S10975[81], 5}, {NULL}};
const Segment SGI330215[] = {{&S10975[12], 14}, {&S10975[44], 42}, {NULL}};
const Segment SGI330216[] = {{&S10975[12], 14}, {NULL}};
const Segment SGI330217[] = {{&S10976[6], 4}, {NULL}};
const Segment SGI330218[] = {{&S10976[81], 5}, {NULL}};
const Segment SGI330219[] = {{&S10976[12], 14}, {&S10976[44], 42}, {NULL}};
const Segment SGI330220[] = {{&S10976[12], 14}, {NULL}};
const Segment SGI330221[] = {{&S10977[6], 4}, {NULL}};
const Segment SGI330222[] = {{&S10977[81], 5}, {NULL}};
const Segment SGI330223[] = {{&S10977[12], 14}, {&S10977[44], 42}, {NULL}};
const Segment SGI330224[] = {{&S10977[12], 14}, {NULL}};
const Segment SGI330225[] = {{&S10978[6], 4}, {NULL}};
const Segment SGI330226[] = {{&S10978[81], 5}, {NULL}};
const Segment SGI330227[] = {{&S10978[12], 14}, {&S10978[44], 42}, {NULL}};
const Segment SGI330228[] = {{&S10978[12], 14}, {NULL}};
const Segment SGI330229[] = {{&S10979[6], 4}, {NULL}};
const Segment SGI330230[] = {{&S10979[74], 5}, {NULL}};
const Segment SGI330231[] = {{&S10979[12], 14}, {&S10979[37], 42}, {NULL}};
const Segment SGI330232[] = {{&S10979[12], 14}, {NULL}};
const Segment SGI330233[] = {{&S10980[6], 4}, {NULL}};
const Segment SGI330234[] = {{&S10980[74], 5}, {NULL}};
const Segment SGI330235[] = {{&S10980[12], 14}, {&S10980[37], 42}, {NULL}};
const Segment SGI330236[] = {{&S10980[12], 14}, {NULL}};
const Segment SGI330237[] = {{&S10981[6], 4}, {NULL}};
const Segment SGI330238[] = {{&S10981[75], 5}, {NULL}};
const Segment SGI330239[] = {{&S10981[12], 14}, {&S10981[37], 43}, {NULL}};
const Segment SGI330240[] = {{&S10981[12], 14}, {NULL}};
const Segment SGI330241[] = {{&S10982[6], 4}, {NULL}};
const Segment SGI330242[] = {{&S10982[74], 5}, {NULL}};
const Segment SGI330243[] = {{&S10982[12], 14}, {&S10982[37], 42}, {NULL}};
const Segment SGI330244[] = {{&S10982[12], 14}, {NULL}};
const Segment SGI330245[] = {{&S10983[6], 4}, {NULL}};
const Segment SGI330246[] = {{&S10983[82], 5}, {NULL}};
const Segment SGI330247[] = {{&S10983[12], 14}, {&S10983[44], 43}, {NULL}};
const Segment SGI330248[] = {{&S10983[12], 14}, {NULL}};
const Segment SGI330249[] = {{&S10984[6], 4}, {NULL}};
const Segment SGI330250[] = {{&S10984[81], 5}, {NULL}};
const Segment SGI330251[] = {{&S10984[12], 14}, {&S10984[44], 42}, {NULL}};
const Segment SGI330252[] = {{&S10984[12], 14}, {NULL}};
const Segment SGI330253[] = {{&S10985[6], 4}, {NULL}};
const Segment SGI330254[] = {{&S10985[75], 5}, {NULL}};
const Segment SGI330255[] = {{&S10985[12], 14}, {&S10985[38], 42}, {NULL}};
const Segment SGI330256[] = {{&S10985[12], 14}, {NULL}};
const Segment SGI330257[] = {{&S10986[6], 4}, {NULL}};
const Segment SGI330258[] = {{&S10986[74], 5}, {NULL}};
const Segment SGI330259[] = {{&S10986[12], 14}, {&S10986[37], 42}, {NULL}};
const Segment SGI330260[] = {{&S10986[12], 14}, {NULL}};
const Segment SGI330261[] = {{&S10987[6], 4}, {NULL}};
const Segment SGI330262[] = {{&S10987[75], 5}, {NULL}};
const Segment SGI330263[] = {{&S10987[12], 14}, {&S10987[38], 42}, {NULL}};
const Segment SGI330264[] = {{&S10987[12], 14}, {NULL}};
const Segment SGI330265[] = {{&S10988[6], 4}, {NULL}};
const Segment SGI330266[] = {{&S10988[81], 5}, {NULL}};
const Segment SGI330267[] = {{&S10988[12], 14}, {&S10988[44], 42}, {NULL}};
const Segment SGI330268[] = {{&S10988[12], 14}, {NULL}};
const Segment SGI330269[] = {{&S10989[6], 4}, {NULL}};
const Segment SGI330270[] = {{&S10989[74], 5}, {NULL}};
const Segment SGI330271[] = {{&S10989[12], 14}, {&S10989[37], 42}, {NULL}};
const Segment SGI330272[] = {{&S10989[12], 14}, {NULL}};
const Segment SGI330273[] = {{&S10990[6], 4}, {NULL}};
const Segment SGI330274[] = {{&S10990[75], 5}, {NULL}};
const Segment SGI330275[] = {{&S10990[12], 14}, {&S10990[38], 42}, {NULL}};
const Segment SGI330276[] = {{&S10990[12], 14}, {NULL}};
const Segment SGI330277[] = {{&S10991[6], 4}, {NULL}};
const Segment SGI330278[] = {{&S10991[82], 5}, {NULL}};
const Segment SGI330279[] = {{&S10991[12], 14}, {&S10991[45], 42}, {NULL}};
const Segment SGI330280[] = {{&S10991[12], 14}, {NULL}};
const Segment SGI330281[] = {{&S10992[6], 4}, {NULL}};
const Segment SGI330282[] = {{&S10992[75], 5}, {NULL}};
const Segment SGI330283[] = {{&S10992[12], 14}, {&S10992[38], 42}, {NULL}};
const Segment SGI330284[] = {{&S10992[12], 14}, {NULL}};
const Segment SGI330285[] = {{&S10993[6], 4}, {NULL}};
const Segment SGI330286[] = {{&S10993[75], 5}, {NULL}};
const Segment SGI330287[] = {{&S10993[12], 14}, {&S10993[38], 42}, {NULL}};
const Segment SGI330288[] = {{&S10993[12], 14}, {NULL}};
const Segment SGI330289[] = {{&S10994[6], 4}, {NULL}};
const Segment SGI330290[] = {{&S10994[75], 5}, {NULL}};
const Segment SGI330291[] = {{&S10994[12], 14}, {&S10994[37], 43}, {NULL}};
const Segment SGI330292[] = {{&S10994[12], 14}, {NULL}};
const Segment SGI330293[] = {{&S10995[6], 4}, {NULL}};
const Segment SGI330294[] = {{&S10995[74], 5}, {NULL}};
const Segment SGI330295[] = {{&S10995[12], 14}, {&S10995[37], 42}, {NULL}};
const Segment SGI330296[] = {{&S10995[12], 14}, {NULL}};
const Segment SGI330297[] = {{&S10996[6], 4}, {NULL}};
const Segment SGI330298[] = {{&S10996[81], 5}, {NULL}};
const Segment SGI330299[] = {{&S10996[12], 14}, {&S10996[44], 42}, {NULL}};
const Segment SGI330300[] = {{&S10996[12], 14}, {NULL}};
const Segment SGI330301[] = {{&S10997[6], 4}, {NULL}};
const Segment SGI330302[] = {{&S10997[81], 5}, {NULL}};
const Segment SGI330303[] = {{&S10997[12], 14}, {&S10997[44], 42}, {NULL}};
const Segment SGI330304[] = {{&S10997[12], 14}, {NULL}};
const Segment SGI330305[] = {{&S10998[6], 4}, {NULL}};
const Segment SGI330306[] = {{&S10998[75], 5}, {NULL}};
const Segment SGI330307[] = {{&S10998[12], 14}, {&S10998[37], 43}, {NULL}};
const Segment SGI330308[] = {{&S10998[12], 14}, {NULL}};
const Segment SGI330309[] = {{&S10999[6], 4}, {NULL}};
const Segment SGI330310[] = {{&S10999[74], 5}, {NULL}};
const Segment SGI330311[] = {{&S10999[12], 14}, {&S10999[37], 42}, {NULL}};
const Segment SGI330312[] = {{&S10999[12], 14}, {NULL}};
const Segment SGI330313[] = {{&S11000[6], 4}, {NULL}};
const Segment SGI330314[] = {{&S11000[81], 5}, {NULL}};
const Segment SGI330315[] = {{&S11000[12], 14}, {&S11000[44], 42}, {NULL}};
const Segment SGI330316[] = {{&S11000[12], 14}, {NULL}};
const Segment SGI330317[] = {{&S11001[6], 4}, {NULL}};
const Segment SGI330318[] = {{&S11001[81], 5}, {NULL}};
const Segment SGI330319[] = {{&S11001[12], 14}, {&S11001[44], 42}, {NULL}};
const Segment SGI330320[] = {{&S11001[12], 14}, {NULL}};
const Segment SGI330321[] = {{&S11002[6], 4}, {NULL}};
const Segment SGI330322[] = {{&S11002[74], 5}, {NULL}};
const Segment SGI330323[] = {{&S11002[12], 14}, {&S11002[37], 42}, {NULL}};
const Segment SGI330324[] = {{&S11002[12], 14}, {NULL}};
const Segment SGI330325[] = {{&S11003[6], 4}, {NULL}};
const Segment SGI330326[] = {{&S11003[59], 5}, {NULL}};
const Segment SGI330327[] = {{&S11003[12], 14}, {&S11003[70], 33}, {NULL}};
const Segment SGI330328[] = {{&S11003[12], 14}, {NULL}};
const Segment SGI330329[] = {{&S11004[6], 4}, {NULL}};
const Segment SGI330330[] = {{&S11004[75], 5}, {NULL}};
const Segment SGI330331[] = {{&S11004[12], 14}, {&S11004[37], 43}, {NULL}};
const Segment SGI330332[] = {{&S11004[12], 14}, {NULL}};
const Segment SGI330333[] = {{&S11005[6], 4}, {NULL}};
const Segment SGI330334[] = {{&S11005[81], 5}, {NULL}};
const Segment SGI330335[] = {{&S11005[12], 14}, {&S11005[44], 42}, {NULL}};
const Segment SGI330336[] = {{&S11005[12], 14}, {NULL}};
const Segment SGI330337[] = {{&S11006[6], 4}, {NULL}};
const Segment SGI330338[] = {{&S11006[74], 5}, {NULL}};
const Segment SGI330339[] = {{&S11006[12], 14}, {&S11006[37], 42}, {NULL}};
const Segment SGI330340[] = {{&S11006[12], 14}, {NULL}};
const Segment SGI330341[] = {{&S11007[6], 4}, {NULL}};
const Segment SGI330342[] = {{&S11007[74], 5}, {NULL}};
const Segment SGI330343[] = {{&S11007[12], 14}, {&S11007[37], 42}, {NULL}};
const Segment SGI330344[] = {{&S11007[12], 14}, {NULL}};
const Segment SGI330345[] = {{&S11008[6], 4}, {NULL}};
const Segment SGI330346[] = {{&S11008[82], 5}, {NULL}};
const Segment SGI330347[] = {{&S11008[12], 14}, {&S11008[45], 42}, {NULL}};
const Segment SGI330348[] = {{&S11008[12], 14}, {NULL}};
const Segment SGI330349[] = {{&S11009[6], 4}, {NULL}};
const Segment SGI330350[] = {{&S11009[74], 5}, {NULL}};
const Segment SGI330351[] = {{&S11009[12], 14}, {&S11009[37], 42}, {NULL}};
const Segment SGI330352[] = {{&S11009[12], 14}, {NULL}};
const Segment SGI330353[] = {{&S11010[6], 4}, {NULL}};
const Segment SGI330354[] = {{&S11010[75], 5}, {NULL}};
const Segment SGI330355[] = {{&S11010[12], 14}, {&S11010[37], 43}, {NULL}};
const Segment SGI330356[] = {{&S11010[12], 14}, {NULL}};
const Segment SGI330357[] = {{&S11011[6], 4}, {NULL}};
const Segment SGI330358[] = {{&S11011[74], 5}, {NULL}};
const Segment SGI330359[] = {{&S11011[12], 14}, {&S11011[37], 42}, {NULL}};
const Segment SGI330360[] = {{&S11011[12], 14}, {NULL}};
const Segment SGI330361[] = {{&S11012[6], 4}, {NULL}};
const Segment SGI330362[] = {{&S11012[74], 5}, {NULL}};
const Segment SGI330363[] = {{&S11012[12], 14}, {&S11012[37], 42}, {NULL}};
const Segment SGI330364[] = {{&S11012[12], 14}, {NULL}};
const Segment SGI330365[] = {{&S11013[6], 4}, {NULL}};
const Segment SGI330366[] = {{&S11013[74], 5}, {NULL}};
const Segment SGI330367[] = {{&S11013[12], 14}, {&S11013[37], 42}, {NULL}};
const Segment SGI330368[] = {{&S11013[12], 14}, {NULL}};
const Segment SGI330369[] = {{&S11014[6], 4}, {NULL}};
const Segment SGI330370[] = {{&S11014[81], 5}, {NULL}};
const Segment SGI330371[] = {{&S11014[12], 14}, {&S11014[44], 42}, {NULL}};
const Segment SGI330372[] = {{&S11014[12], 14}, {NULL}};
const Segment SGI330373[] = {{&S11015[6], 4}, {NULL}};
const Segment SGI330374[] = {{&S11015[52], 5}, {NULL}};
const Segment SGI330375[] = {{&S11015[12], 84}, {NULL}};
const Segment SGI330376[] = {{&S11015[12], 14}, {NULL}};
const Segment SGI330377[] = {{&S11016[6], 4}, {NULL}};
const Segment SGI330378[] = {{&S11016[75], 5}, {NULL}};
const Segment SGI330379[] = {{&S11016[12], 14}, {&S11016[38], 42}, {NULL}};
const Segment SGI330380[] = {{&S11016[12], 14}, {NULL}};
const Segment SGI330381[] = {{&S11017[6], 4}, {NULL}};
const Segment SGI330382[] = {{&S11017[81], 5}, {NULL}};
const Segment SGI330383[] = {{&S11017[12], 14}, {&S11017[44], 42}, {NULL}};
const Segment SGI330384[] = {{&S11017[12], 14}, {NULL}};
const Segment SGI330385[] = {{&S11018[6], 4}, {NULL}};
const Segment SGI330386[] = {{&S11018[81], 5}, {NULL}};
const Segment SGI330387[] = {{&S11018[12], 14}, {&S11018[44], 42}, {NULL}};
const Segment SGI330388[] = {{&S11018[12], 14}, {NULL}};
const Segment SGI330389[] = {{&S11019[6], 4}, {NULL}};
const Segment SGI330390[] = {{&S11019[74], 5}, {NULL}};
const Segment SGI330391[] = {{&S11019[12], 14}, {&S11019[37], 42}, {NULL}};
const Segment SGI330392[] = {{&S11019[12], 14}, {NULL}};
const Segment SGI330393[] = {{&S11020[6], 4}, {NULL}};
const Segment SGI330394[] = {{&S11020[74], 5}, {NULL}};
const Segment SGI330395[] = {{&S11020[12], 14}, {&S11020[37], 42}, {NULL}};
const Segment SGI330396[] = {{&S11020[12], 14}, {NULL}};
const Segment SGI330397[] = {{&S11021[6], 4}, {NULL}};
const Segment SGI330398[] = {{&S11021[81], 5}, {NULL}};
const Segment SGI330399[] = {{&S11021[12], 14}, {&S11021[44], 42}, {NULL}};
const Segment SGI330400[] = {{&S11021[12], 14}, {NULL}};
const Segment SGI330401[] = {{&S11022[6], 4}, {NULL}};
const Segment SGI330402[] = {{&S11022[74], 5}, {NULL}};
const Segment SGI330403[] = {{&S11022[12], 14}, {&S11022[37], 42}, {NULL}};
const Segment SGI330404[] = {{&S11022[12], 14}, {NULL}};
const Segment SGI330405[] = {{&S11023[6], 4}, {NULL}};
const Segment SGI330406[] = {{&S11023[82], 5}, {NULL}};
const Segment SGI330407[] = {{&S11023[12], 14}, {&S11023[45], 42}, {NULL}};
const Segment SGI330408[] = {{&S11023[12], 14}, {NULL}};
const Segment SGI330409[] = {{&S11024[6], 4}, {NULL}};
const Segment SGI330410[] = {{&S11024[82], 5}, {NULL}};
const Segment SGI330411[] = {{&S11024[12], 14}, {&S11024[44], 43}, {NULL}};
const Segment SGI330412[] = {{&S11024[12], 14}, {NULL}};
const Segment SGI330413[] = {{&S11025[6], 4}, {NULL}};
const Segment SGI330414[] = {{&S11025[81], 5}, {NULL}};
const Segment SGI330415[] = {{&S11025[12], 14}, {&S11025[44], 42}, {NULL}};
const Segment SGI330416[] = {{&S11025[12], 14}, {NULL}};
const Segment SGI330417[] = {{&S11026[6], 4}, {NULL}};
const Segment SGI330418[] = {{&S11026[81], 5}, {NULL}};
const Segment SGI330419[] = {{&S11026[12], 14}, {&S11026[44], 42}, {NULL}};
const Segment SGI330420[] = {{&S11026[12], 14}, {NULL}};
const Segment SGI330421[] = {{&S11027[6], 4}, {NULL}};
const Segment SGI330422[] = {{&S11027[81], 5}, {NULL}};
const Segment SGI330423[] = {{&S11027[12], 14}, {&S11027[44], 42}, {NULL}};
const Segment SGI330424[] = {{&S11027[12], 14}, {NULL}};
const Segment SGI330425[] = {{&S11028[6], 4}, {NULL}};
const Segment SGI330426[] = {{&S11028[81], 5}, {NULL}};
const Segment SGI330427[] = {{&S11028[12], 14}, {&S11028[44], 42}, {NULL}};
const Segment SGI330428[] = {{&S11028[12], 14}, {NULL}};
const Segment SGI330429[] = {{&S11029[6], 4}, {NULL}};
const Segment SGI330430[] = {{&S11029[75], 5}, {NULL}};
const Segment SGI330431[] = {{&S11029[12], 14}, {&S11029[37], 43}, {NULL}};
const Segment SGI330432[] = {{&S11029[12], 14}, {NULL}};
const Segment SGI330433[] = {{&S11030[6], 4}, {NULL}};
const Segment SGI330434[] = {{&S11030[75], 5}, {NULL}};
const Segment SGI330435[] = {{&S11030[12], 14}, {&S11030[37], 43}, {NULL}};
const Segment SGI330436[] = {{&S11030[12], 14}, {NULL}};
const Segment SGI330437[] = {{&S11031[6], 4}, {NULL}};
const Segment SGI330438[] = {{&S11031[75], 5}, {NULL}};
const Segment SGI330439[] = {{&S11031[12], 14}, {&S11031[37], 43}, {NULL}};
const Segment SGI330440[] = {{&S11031[12], 14}, {NULL}};
const Segment SGI330441[] = {{&S11032[6], 4}, {NULL}};
const Segment SGI330442[] = {{&S11032[81], 5}, {NULL}};
const Segment SGI330443[] = {{&S11032[12], 14}, {&S11032[44], 42}, {NULL}};
const Segment SGI330444[] = {{&S11032[12], 14}, {NULL}};
const Segment SGI330445[] = {{&S11033[6], 4}, {NULL}};
const Segment SGI330446[] = {{&S11033[82], 5}, {NULL}};
const Segment SGI330447[] = {{&S11033[12], 14}, {&S11033[44], 43}, {NULL}};
const Segment SGI330448[] = {{&S11033[12], 14}, {NULL}};
const Segment SGI330449[] = {{&S11034[6], 4}, {NULL}};
const Segment SGI330450[] = {{&S11034[75], 5}, {NULL}};
const Segment SGI330451[] = {{&S11034[12], 14}, {&S11034[37], 43}, {NULL}};
const Segment SGI330452[] = {{&S11034[12], 14}, {NULL}};
const Segment SGI330453[] = {{&S11035[6], 4}, {NULL}};
const Segment SGI330454[] = {{&S11035[81], 5}, {NULL}};
const Segment SGI330455[] = {{&S11035[12], 14}, {&S11035[44], 42}, {NULL}};
const Segment SGI330456[] = {{&S11035[12], 14}, {NULL}};
const Segment SGI330457[] = {{&S11036[6], 4}, {NULL}};
const Segment SGI330458[] = {{&S11036[82], 5}, {NULL}};
const Segment SGI330459[] = {{&S11036[12], 14}, {&S11036[45], 42}, {NULL}};
const Segment SGI330460[] = {{&S11036[12], 14}, {NULL}};
const Segment SGI330461[] = {{&S11037[6], 4}, {NULL}};
const Segment SGI330462[] = {{&S11037[102], 5}, {NULL}};
const Segment SGI330463[] = {{&S11037[12], 14}, {&S11037[37], 12}, {NULL}};
const Segment SGI330464[] = {{&S11037[12], 14}, {NULL}};
const Segment SGI330465[] = {{&S11038[6], 4}, {NULL}};
const Segment SGI330466[] = {{&S11038[81], 5}, {NULL}};
const Segment SGI330467[] = {{&S11038[12], 14}, {&S11038[44], 42}, {NULL}};
const Segment SGI330468[] = {{&S11038[12], 14}, {NULL}};
const Segment SGI330469[] = {{&S11039[6], 4}, {NULL}};
const Segment SGI330470[] = {{&S11039[74], 5}, {NULL}};
const Segment SGI330471[] = {{&S11039[12], 14}, {&S11039[37], 42}, {NULL}};
const Segment SGI330472[] = {{&S11039[12], 14}, {NULL}};
const Segment SGI330473[] = {{&S11040[6], 4}, {NULL}};
const Segment SGI330474[] = {{&S11040[82], 5}, {NULL}};
const Segment SGI330475[] = {{&S11040[12], 14}, {&S11040[45], 42}, {NULL}};
const Segment SGI330476[] = {{&S11040[12], 14}, {NULL}};
const Segment SGI330477[] = {{&S11041[6], 4}, {NULL}};
const Segment SGI330478[] = {{&S11041[75], 5}, {NULL}};
const Segment SGI330479[] = {{&S11041[12], 14}, {&S11041[37], 43}, {NULL}};
const Segment SGI330480[] = {{&S11041[12], 14}, {NULL}};
const Segment SGI330481[] = {{&S11042[6], 4}, {NULL}};
const Segment SGI330482[] = {{&S11042[75], 5}, {NULL}};
const Segment SGI330483[] = {{&S11042[12], 14}, {&S11042[38], 42}, {NULL}};
const Segment SGI330484[] = {{&S11042[12], 14}, {NULL}};
const Segment SGI330485[] = {{&S11043[6], 4}, {NULL}};
const Segment SGI330486[] = {{&S11043[82], 5}, {NULL}};
const Segment SGI330487[] = {{&S11043[12], 14}, {&S11043[44], 43}, {NULL}};
const Segment SGI330488[] = {{&S11043[12], 14}, {NULL}};
const Segment SGI330489[] = {{&S11044[6], 4}, {NULL}};
const Segment SGI330490[] = {{&S11044[81], 5}, {NULL}};
const Segment SGI330491[] = {{&S11044[12], 14}, {&S11044[44], 42}, {NULL}};
const Segment SGI330492[] = {{&S11044[12], 14}, {NULL}};
const Segment SGI330493[] = {{&S11045[6], 4}, {NULL}};
const Segment SGI330494[] = {{&S11045[81], 5}, {NULL}};
const Segment SGI330495[] = {{&S11045[12], 14}, {&S11045[44], 42}, {NULL}};
const Segment SGI330496[] = {{&S11045[12], 14}, {NULL}};
const Segment SGI330497[] = {{&S11046[6], 4}, {NULL}};
const Segment SGI330498[] = {{&S11046[81], 5}, {NULL}};
const Segment SGI330499[] = {{&S11046[12], 14}, {&S11046[44], 42}, {NULL}};
const Segment SGI330500[] = {{&S11046[12], 14}, {NULL}};
const Segment SGI330501[] = {{&S11047[6], 4}, {NULL}};
const Segment SGI330502[] = {{&S11047[81], 5}, {NULL}};
const Segment SGI330503[] = {{&S11047[12], 14}, {&S11047[44], 42}, {NULL}};
const Segment SGI330504[] = {{&S11047[12], 14}, {NULL}};
const Segment SGI330505[] = {{&S11048[6], 4}, {NULL}};
const Segment SGI330506[] = {{&S11048[81], 5}, {NULL}};
const Segment SGI330507[] = {{&S11048[12], 14}, {&S11048[44], 42}, {NULL}};
const Segment SGI330508[] = {{&S11048[12], 14}, {NULL}};
const Segment SGI330509[] = {{&S11049[6], 4}, {NULL}};
const Segment SGI330510[] = {{&S11049[81], 5}, {NULL}};
const Segment SGI330511[] = {{&S11049[12], 14}, {&S11049[44], 42}, {NULL}};
const Segment SGI330512[] = {{&S11049[12], 14}, {NULL}};
const Segment SGI330513[] = {{&S11050[6], 4}, {NULL}};
const Segment SGI330514[] = {{&S11050[81], 5}, {NULL}};
const Segment SGI330515[] = {{&S11050[12], 14}, {&S11050[44], 42}, {NULL}};
const Segment SGI330516[] = {{&S11050[12], 14}, {NULL}};
const Segment SGI330517[] = {{&S11051[6], 4}, {NULL}};
const Segment SGI330518[] = {{&S11051[74], 5}, {NULL}};
const Segment SGI330519[] = {{&S11051[12], 14}, {&S11051[37], 42}, {NULL}};
const Segment SGI330520[] = {{&S11051[12], 14}, {NULL}};
const Segment SGI330521[] = {{&S8937[0], 22}, {NULL}};
const Segment SGI330522[] = {{&S11052[6], 4}, {NULL}};
const Segment SGI330523[] = {{&S8937[35], 3}, {NULL}};
const Segment SGI330524[] = {{&S11052[81], 5}, {NULL}};
const Segment SGI330525[] = {{&S11052[12], 14}, {&S11052[44], 42}, {NULL}};
const Segment SGI330526[] = {{&S11052[12], 14}, {NULL}};
const Segment SGI330527[] = {{&S8938[0], 21}, {NULL}};
const Segment SGI330528[] = {{&S8938[34], 3}, {NULL}};
const Segment SGI330529[] = {{&S11053[6], 4}, {NULL}};
const Segment SGI330530[] = {{&S11053[81], 5}, {NULL}};
const Segment SGI330531[] = {{&S8938[73], 13}, {NULL}};
const Segment SGI330532[] = {{&S11053[12], 14}, {&S11053[44], 42}, {NULL}};
const Segment SGI330533[] = {{&S11053[12], 14}, {NULL}};
const Segment SGI330534[] = {{&S8939[0], 21}, {NULL}};
const Segment SGI330535[] = {{&S8939[34], 3}, {NULL}};
const Segment SGI330536[] = {{&S11054[6], 4}, {NULL}};
const Segment SGI330537[] = {{&S11054[81], 5}, {NULL}};
const Segment SGI330538[] = {{&S11054[12], 14}, {&S11054[44], 42}, {NULL}};
const Segment SGI330539[] = {{&S11054[12], 14}, {NULL}};
const Segment SGI330540[] = {{&S8940[0], 21}, {NULL}};
const Segment SGI330541[] = {{&S8940[34], 3}, {NULL}};
const Segment SGI330542[] = {{&S11055[6], 4}, {NULL}};
const Segment SGI330543[] = {{&S11055[102], 5}, {NULL}};
const Segment SGI330544[] = {{&S11055[12], 14}, {&S11055[37], 12}, {NULL}};
const Segment SGI330545[] = {{&S11055[12], 14}, {NULL}};
const Segment SGI330546[] = {{&S8941[0], 21}, {NULL}};
const Segment SGI330547[] = {{&S8941[34], 3}, {NULL}};
const Segment SGI330548[] = {{&S8941[75], 13}, {NULL}};
const Segment SGI330549[] = {{&S11056[6], 4}, {NULL}};
const Segment SGI330550[] = {{&S8942[0], 15}, {NULL}};
const Segment SGI330551[] = {{&S11056[101], 5}, {NULL}};
const Segment SGI330552[] = {{&S8942[28], 3}, {NULL}};
const Segment SGI330553[] = {{&S11056[12], 14}, {&S11056[37], 12}, {NULL}};
const Segment SGI330554[] = {{&S8942[68], 13}, {NULL}};
const Segment SGI330555[] = {{&S11056[12], 14}, {NULL}};
const Segment SGI330556[] = {{&S11057[6], 4}, {NULL}};
const Segment SGI330557[] = {{&S11057[81], 5}, {NULL}};
const Segment SGI330558[] = {{&S11057[12], 14}, {&S11057[44], 42}, {NULL}};
const Segment SGI330559[] = {{&S11057[12], 14}, {NULL}};
const Segment SGI330560[] = {{&S11058[6], 4}, {NULL}};
const Segment SGI330561[] = {{&S11058[81], 5}, {NULL}};
const Segment SGI330562[] = {{&S11058[12], 14}, {&S11058[44], 42}, {NULL}};
const Segment SGI330563[] = {{&S11058[12], 14}, {NULL}};
const Segment SGI330564[] = {{&S11059[6], 4}, {NULL}};
const Segment SGI330565[] = {{&S11059[81], 5}, {NULL}};
const Segment SGI330566[] = {{&S11059[12], 14}, {&S11059[44], 42}, {NULL}};
const Segment SGI330567[] = {{&S11059[12], 14}, {NULL}};
const Segment SGI330568[] = {{&S11060[6], 4}, {NULL}};
const Segment SGI330569[] = {{&S11060[74], 5}, {NULL}};
const Segment SGI330570[] = {{&S11060[12], 14}, {&S11060[37], 42}, {NULL}};
const Segment SGI330571[] = {{&S11060[12], 14}, {NULL}};
const Segment SGI330572[] = {{&S11061[6], 4}, {NULL}};
const Segment SGI330573[] = {{&S11061[81], 5}, {NULL}};
const Segment SGI330574[] = {{&S11061[12], 14}, {&S11061[44], 42}, {NULL}};
const Segment SGI330575[] = {{&S11061[12], 14}, {NULL}};
const Segment SGI330576[] = {{&S11062[6], 4}, {NULL}};
const Segment SGI330577[] = {{&S11062[74], 5}, {NULL}};
const Segment SGI330578[] = {{&S11062[12], 14}, {&S11062[37], 42}, {NULL}};
const Segment SGI330579[] = {{&S11062[12], 14}, {NULL}};
const Segment SGI330580[] = {{&S11063[6], 4}, {NULL}};
const Segment SGI330581[] = {{&S11063[81], 5}, {NULL}};
const Segment SGI330582[] = {{&S11063[12], 14}, {&S11063[44], 42}, {NULL}};
const Segment SGI330583[] = {{&S11063[12], 14}, {NULL}};
const Segment SGI330584[] = {{&S11064[6], 4}, {NULL}};
const Segment SGI330585[] = {{&S11064[74], 5}, {NULL}};
const Segment SGI330586[] = {{&S11064[12], 14}, {&S11064[37], 42}, {NULL}};
const Segment SGI330587[] = {{&S11064[12], 14}, {NULL}};
const Segment SGI330588[] = {{&S11065[6], 4}, {NULL}};
const Segment SGI330589[] = {{&S11065[81], 5}, {NULL}};
const Segment SGI330590[] = {{&S11065[12], 14}, {&S11065[44], 42}, {NULL}};
const Segment SGI330591[] = {{&S11065[12], 14}, {NULL}};
const Segment SGI330592[] = {{&S11066[6], 4}, {NULL}};
const Segment SGI330593[] = {{&S11066[59], 5}, {NULL}};
const Segment SGI330594[] = {{&S11066[12], 14}, {&S11066[70], 14}, {NULL}};
const Segment SGI330595[] = {{&S11066[12], 14}, {NULL}};
const Segment SGI330596[] = {{&S11067[6], 4}, {NULL}};
const Segment SGI330597[] = {{&S11067[52], 5}, {NULL}};
const Segment SGI330598[] = {{&S11067[12], 65}, {NULL}};
const Segment SGI330599[] = {{&S11067[12], 14}, {NULL}};
const Segment SGI330600[] = {{&S11068[6], 4}, {NULL}};
const Segment SGI330601[] = {{&S11068[81], 5}, {NULL}};
const Segment SGI330602[] = {{&S11068[12], 14}, {&S11068[44], 42}, {NULL}};
const Segment SGI330603[] = {{&S11068[12], 14}, {NULL}};
const Segment SGI330604[] = {{&S11069[6], 4}, {NULL}};
const Segment SGI330605[] = {{&S11069[74], 5}, {NULL}};
const Segment SGI330606[] = {{&S11069[12], 14}, {&S11069[37], 42}, {NULL}};
const Segment SGI330607[] = {{&S11069[12], 14}, {NULL}};
const Segment SGI330608[] = {{&S11070[6], 4}, {NULL}};
const Segment SGI330609[] = {{&S11070[74], 5}, {NULL}};
const Segment SGI330610[] = {{&S11070[12], 14}, {&S11070[37], 42}, {NULL}};
const Segment SGI330611[] = {{&S11070[12], 14}, {NULL}};
const Segment SGI330612[] = {{&S11071[6], 4}, {NULL}};
const Segment SGI330613[] = {{&S11071[74], 5}, {NULL}};
const Segment SGI330614[] = {{&S11071[12], 14}, {&S11071[37], 42}, {NULL}};
const Segment SGI330615[] = {{&S11071[12], 14}, {NULL}};
const Segment SGI330616[] = {{&S11072[6], 4}, {NULL}};
const Segment SGI330617[] = {{&S11072[81], 5}, {NULL}};
const Segment SGI330618[] = {{&S11072[12], 14}, {&S11072[44], 42}, {NULL}};
const Segment SGI330619[] = {{&S11072[12], 14}, {NULL}};
const Segment SGI330620[] = {{&S11073[6], 4}, {NULL}};
const Segment SGI330621[] = {{&S11073[74], 5}, {NULL}};
const Segment SGI330622[] = {{&S11073[12], 14}, {&S11073[37], 42}, {NULL}};
const Segment SGI330623[] = {{&S11073[12], 14}, {NULL}};
const Segment SGI330624[] = {{&S11074[6], 4}, {NULL}};
const Segment SGI330625[] = {{&S11074[74], 5}, {NULL}};
const Segment SGI330626[] = {{&S11074[12], 14}, {&S11074[37], 42}, {NULL}};
const Segment SGI330627[] = {{&S11074[12], 14}, {NULL}};
const Segment SGI330628[] = {{&S11075[6], 4}, {NULL}};
const Segment SGI330629[] = {{&S11075[81], 5}, {NULL}};
const Segment SGI330630[] = {{&S11075[12], 14}, {&S11075[44], 42}, {NULL}};
const Segment SGI330631[] = {{&S11075[12], 14}, {NULL}};
const Segment SGI330632[] = {{&S11076[6], 4}, {NULL}};
const Segment SGI330633[] = {{&S11076[74], 5}, {NULL}};
const Segment SGI330634[] = {{&S11076[12], 14}, {&S11076[37], 42}, {NULL}};
const Segment SGI330635[] = {{&S11076[12], 14}, {NULL}};
const Segment SGI330636[] = {{&S11173[6], 4}, {NULL}};
const Segment SGI330637[] = {{&S11173[59], 5}, {NULL}};
const Segment SGI330638[] = {{&S11173[12], 14}, {NULL}};
const Segment SGI330639[] = {{&S11174[6], 4}, {NULL}};
const Segment SGI330640[] = {{&S11174[52], 5}, {NULL}};
const Segment SGI330641[] = {{&S11174[12], 45}, {NULL}};
const Segment SGI330642[] = {{&S11174[12], 14}, {NULL}};
const Segment SGI330643[] = {{&S11175[6], 4}, {NULL}};
const Segment SGI330644[] = {{&S11175[81], 5}, {NULL}};
const Segment SGI330645[] = {{&S11175[12], 14}, {&S11175[44], 42}, {NULL}};
const Segment SGI330646[] = {{&S11175[12], 14}, {NULL}};
const Segment SGI330647[] = {{&S11176[6], 4}, {NULL}};
const Segment SGI330648[] = {{&S11176[81], 5}, {NULL}};
const Segment SGI330649[] = {{&S11176[12], 14}, {&S11176[44], 42}, {NULL}};
const Segment SGI330650[] = {{&S11176[12], 14}, {NULL}};
const Segment SGI330651[] = {{&S11177[6], 4}, {NULL}};
const Segment SGI330652[] = {{&S11177[81], 5}, {NULL}};
const Segment SGI330653[] = {{&S35924[100], 11}, {NULL}};
const Segment SGI330654[] = {{&S35925[100], 21}, {NULL}};
const Segment SGI330655[] = {{&S35926[100], 19}, {NULL}};
const Segment SGI330656[] = {{&S35927[100], 19}, {NULL}};
const Segment SGI330657[] = {{&S11177[12], 14}, {&S11177[44], 42}, {NULL}};
const Segment SGI330658[] = {{&S11177[12], 14}, {NULL}};
const Segment SGI330659[] = {{&S35928[100], 11}, {NULL}};
const Segment SGI330660[] = {{&S11178[6], 4}, {NULL}};
const Segment SGI330661[] = {{&S11178[81], 5}, {NULL}};
const Segment SGI330662[] = {{&S11178[12], 14}, {&S11178[44], 42}, {NULL}};
const Segment SGI330663[] = {{&S11178[12], 14}, {NULL}};
const Segment SGI330664[] = {{&S35929[100], 19}, {NULL}};
const Segment SGI330665[] = {{&S11179[6], 4}, {NULL}};
const Segment SGI330666[] = {{&S11179[81], 5}, {NULL}};
const Segment SGI330667[] = {{&S11179[12], 14}, {&S11179[44], 42}, {NULL}};
const Segment SGI330668[] = {{&S11179[12], 14}, {NULL}};
const Segment SGI330669[] = {{&S11180[6], 4}, {NULL}};
const Segment SGI330670[] = {{&S11180[81], 5}, {NULL}};
const Segment SGI330671[] = {{&S11180[12], 14}, {&S11180[44], 42}, {NULL}};
const Segment SGI330672[] = {{&S11180[12], 14}, {NULL}};
const Segment SGI330673[] = {{&S11181[6], 4}, {NULL}};
const Segment SGI330674[] = {{&S11181[75], 5}, {NULL}};
const Segment SGI330675[] = {{&S11181[12], 14}, {&S11181[38], 42}, {NULL}};
const Segment SGI330676[] = {{&S11181[12], 14}, {NULL}};
const Segment SGI330677[] = {{&S11183[6], 4}, {NULL}};
const Segment SGI330678[] = {{&S11183[74], 5}, {NULL}};
const Segment SGI330679[] = {{&S11183[12], 14}, {&S11183[37], 42}, {NULL}};
const Segment SGI330680[] = {{&S11183[12], 14}, {NULL}};
const Segment SGI330681[] = {{&S11184[6], 4}, {NULL}};
const Segment SGI330682[] = {{&S11184[74], 5}, {NULL}};
const Segment SGI330683[] = {{&S11184[12], 14}, {&S11184[37], 42}, {NULL}};
const Segment SGI330684[] = {{&S11184[12], 14}, {NULL}};
const Segment SGI330685[] = {{&S11185[6], 4}, {NULL}};
const Segment SGI330686[] = {{&S11185[75], 5}, {NULL}};
const Segment SGI330687[] = {{&S11185[12], 14}, {&S11185[37], 43}, {NULL}};
const Segment SGI330688[] = {{&S11185[12], 14}, {NULL}};
const Segment SGI330689[] = {{&S11186[6], 4}, {NULL}};
const Segment SGI330690[] = {{&S11186[59], 5}, {NULL}};
const Segment SGI330691[] = {{&S11186[12], 14}, {&S11186[70], 4}, {NULL}};
const Segment SGI330692[] = {{&S11186[12], 14}, {NULL}};
const Segment SGI330693[] = {{&S11187[6], 4}, {NULL}};
const Segment SGI330694[] = {{&S11187[74], 5}, {NULL}};
const Segment SGI330695[] = {{&S11187[12], 14}, {&S11187[37], 42}, {NULL}};
const Segment SGI330696[] = {{&S11187[12], 14}, {NULL}};
const Segment SGI330697[] = {{&S11188[6], 4}, {NULL}};
const Segment SGI330698[] = {{&S11188[74], 5}, {NULL}};
const Segment SGI330699[] = {{&S11188[12], 14}, {&S11188[37], 42}, {NULL}};
const Segment SGI330700[] = {{&S11188[12], 14}, {NULL}};
const Segment SGI330701[] = {{&S11190[6], 4}, {NULL}};
const Segment SGI330702[] = {{&S11190[74], 5}, {NULL}};
const Segment SGI330703[] = {{&S11190[12], 14}, {&S11190[37], 42}, {NULL}};
const Segment SGI330704[] = {{&S11190[12], 14}, {NULL}};
const Segment SGI330705[] = {{&S11191[6], 4}, {NULL}};
const Segment SGI330706[] = {{&S11191[82], 5}, {NULL}};
const Segment SGI330707[] = {{&S11191[12], 14}, {&S11191[45], 42}, {NULL}};
const Segment SGI330708[] = {{&S11191[12], 14}, {NULL}};
const Segment SGI330709[] = {{&S11192[6], 4}, {NULL}};
const Segment SGI330710[] = {{&S11192[52], 5}, {NULL}};
const Segment SGI330711[] = {{&S11192[12], 55}, {NULL}};
const Segment SGI330712[] = {{&S11192[12], 14}, {NULL}};
const Segment SGI330713[] = {{&S11193[6], 4}, {NULL}};
const Segment SGI330714[] = {{&S11193[81], 5}, {NULL}};
const Segment SGI330715[] = {{&S11193[12], 14}, {&S11193[44], 42}, {NULL}};
const Segment SGI330716[] = {{&S11193[12], 14}, {NULL}};
const Segment SGI330717[] = {{&S11194[6], 4}, {NULL}};
const Segment SGI330718[] = {{&S11194[74], 5}, {NULL}};
const Segment SGI330719[] = {{&S11194[12], 14}, {&S11194[37], 42}, {NULL}};
const Segment SGI330720[] = {{&S11194[12], 14}, {NULL}};
const Segment SGI330721[] = {{&S11195[6], 4}, {NULL}};
const Segment SGI330722[] = {{&S11195[74], 5}, {NULL}};
const Segment SGI330723[] = {{&S11195[12], 14}, {&S11195[37], 42}, {NULL}};
const Segment SGI330724[] = {{&S11195[12], 14}, {NULL}};
const Segment SGI330725[] = {{&S11196[6], 4}, {NULL}};
const Segment SGI330726[] = {{&S11196[75], 5}, {NULL}};
const Segment SGI330727[] = {{&S11196[12], 14}, {&S11196[38], 42}, {NULL}};
const Segment SGI330728[] = {{&S11196[12], 14}, {NULL}};
const Segment SGI330729[] = {{&S11197[6], 4}, {NULL}};
const Segment SGI330730[] = {{&S11197[81], 5}, {NULL}};
const Segment SGI330731[] = {{&S11197[12], 14}, {&S11197[44], 42}, {NULL}};
const Segment SGI330732[] = {{&S11197[12], 14}, {NULL}};
const Segment SGI330733[] = {{&S11198[6], 4}, {NULL}};
const Segment SGI330734[] = {{&S11198[75], 5}, {NULL}};
const Segment SGI330735[] = {{&S11198[12], 14}, {&S11198[37], 43}, {NULL}};
const Segment SGI330736[] = {{&S11198[12], 14}, {NULL}};
const Segment SGI330737[] = {{&S11199[6], 4}, {NULL}};
const Segment SGI330738[] = {{&S11199[75], 5}, {NULL}};
const Segment SGI330739[] = {{&S11199[12], 14}, {&S11199[38], 42}, {NULL}};
const Segment SGI330740[] = {{&S11199[12], 14}, {NULL}};
const Segment SGI330741[] = {{&S11200[6], 4}, {NULL}};
const Segment SGI330742[] = {{&S11200[75], 5}, {NULL}};
const Segment SGI330743[] = {{&S11200[12], 14}, {&S11200[38], 42}, {NULL}};
const Segment SGI330744[] = {{&S11200[12], 14}, {NULL}};
const Segment SGI330745[] = {{&S11201[6], 4}, {NULL}};
const Segment SGI330746[] = {{&S11201[82], 5}, {NULL}};
const Segment SGI330747[] = {{&S11201[12], 14}, {&S11201[45], 42}, {NULL}};
const Segment SGI330748[] = {{&S11201[12], 14}, {NULL}};
const Segment SGI330749[] = {{&S11202[6], 4}, {NULL}};
const Segment SGI330750[] = {{&S11202[82], 5}, {NULL}};
const Segment SGI330751[] = {{&S11202[12], 14}, {&S11202[45], 42}, {NULL}};
const Segment SGI330752[] = {{&S11202[12], 14}, {NULL}};
const Segment SGI330753[] = {{&S11203[6], 4}, {NULL}};
const Segment SGI330754[] = {{&S11203[74], 5}, {NULL}};
const Segment SGI330755[] = {{&S11203[12], 14}, {&S11203[37], 42}, {NULL}};
const Segment SGI330756[] = {{&S11203[12], 14}, {NULL}};
const Segment SGI330757[] = {{&S11204[6], 4}, {NULL}};
const Segment SGI330758[] = {{&S11204[74], 5}, {NULL}};
const Segment SGI330759[] = {{&S11204[12], 14}, {&S11204[37], 42}, {NULL}};
const Segment SGI330760[] = {{&S11204[12], 14}, {NULL}};
const Segment SGI330761[] = {{&S11205[6], 4}, {NULL}};
const Segment SGI330762[] = {{&S11205[75], 5}, {NULL}};
const Segment SGI330763[] = {{&S11205[12], 14}, {&S11205[37], 43}, {NULL}};
const Segment SGI330764[] = {{&S11205[12], 14}, {NULL}};
const Segment SGI330765[] = {{&S11206[6], 4}, {NULL}};
const Segment SGI330766[] = {{&S11206[74], 5}, {NULL}};
const Segment SGI330767[] = {{&S11206[12], 14}, {&S11206[37], 42}, {NULL}};
const Segment SGI330768[] = {{&S11206[12], 14}, {NULL}};
const Segment SGI330769[] = {{&S11207[6], 4}, {NULL}};
const Segment SGI330770[] = {{&S11207[75], 5}, {NULL}};
const Segment SGI330771[] = {{&S11207[12], 14}, {&S11207[38], 42}, {NULL}};
const Segment SGI330772[] = {{&S11207[12], 14}, {NULL}};
const Segment SGI330773[] = {{&S11208[6], 4}, {NULL}};
const Segment SGI330774[] = {{&S11208[81], 5}, {NULL}};
const Segment SGI330775[] = {{&S11208[12], 14}, {&S11208[44], 42}, {NULL}};
const Segment SGI330776[] = {{&S11208[12], 14}, {NULL}};
const Segment SGI330777[] = {{&S11209[6], 4}, {NULL}};
const Segment SGI330778[] = {{&S11209[81], 5}, {NULL}};
const Segment SGI330779[] = {{&S11209[12], 14}, {&S11209[44], 42}, {NULL}};
const Segment SGI330780[] = {{&S11209[12], 14}, {NULL}};
const Segment SGI330781[] = {{&S11210[6], 4}, {NULL}};
const Segment SGI330782[] = {{&S11210[74], 5}, {NULL}};
const Segment SGI330783[] = {{&S11210[12], 14}, {&S11210[37], 42}, {NULL}};
const Segment SGI330784[] = {{&S11210[12], 14}, {NULL}};
const Segment SGI330785[] = {{&S11211[6], 4}, {NULL}};
const Segment SGI330786[] = {{&S11211[81], 5}, {NULL}};
const Segment SGI330787[] = {{&S11211[12], 14}, {&S11211[44], 42}, {NULL}};
const Segment SGI330788[] = {{&S11211[12], 14}, {NULL}};
const Segment SGI330789[] = {{&S11212[6], 4}, {NULL}};
const Segment SGI330790[] = {{&S11212[82], 5}, {NULL}};
const Segment SGI330791[] = {{&S11212[12], 14}, {&S11212[45], 42}, {NULL}};
const Segment SGI330792[] = {{&S11212[12], 14}, {NULL}};
const Segment SGI330793[] = {{&S11213[6], 4}, {NULL}};
const Segment SGI330794[] = {{&S11213[74], 5}, {NULL}};
const Segment SGI330795[] = {{&S11213[12], 14}, {&S11213[37], 42}, {NULL}};
const Segment SGI330796[] = {{&S11213[12], 14}, {NULL}};
const Segment SGI330797[] = {{&S11214[6], 4}, {NULL}};
const Segment SGI330798[] = {{&S11214[75], 5}, {NULL}};
const Segment SGI330799[] = {{&S11214[12], 14}, {&S11214[37], 43}, {NULL}};
const Segment SGI330800[] = {{&S11214[12], 14}, {NULL}};
const Segment SGI330801[] = {{&S11215[6], 4}, {NULL}};
const Segment SGI330802[] = {{&S11215[74], 5}, {NULL}};
const Segment SGI330803[] = {{&S11215[12], 14}, {&S11215[37], 42}, {NULL}};
const Segment SGI330804[] = {{&S11215[12], 14}, {NULL}};
const Segment SGI330805[] = {{&S11216[6], 4}, {NULL}};
const Segment SGI330806[] = {{&S11216[74], 5}, {NULL}};
const Segment SGI330807[] = {{&S11216[12], 14}, {&S11216[37], 42}, {NULL}};
const Segment SGI330808[] = {{&S11216[12], 14}, {NULL}};
const Segment SGI330809[] = {{&S11217[6], 4}, {NULL}};
const Segment SGI330810[] = {{&S11217[74], 5}, {NULL}};
const Segment SGI330811[] = {{&S11217[12], 14}, {&S11217[37], 42}, {NULL}};
const Segment SGI330812[] = {{&S11217[12], 14}, {NULL}};
const Segment SGI330813[] = {{&S11218[6], 4}, {NULL}};
const Segment SGI330814[] = {{&S11218[75], 5}, {NULL}};
const Segment SGI330815[] = {{&S11218[12], 14}, {&S11218[38], 42}, {NULL}};
const Segment SGI330816[] = {{&S11218[12], 14}, {NULL}};
const Segment SGI330817[] = {{&S11219[6], 4}, {NULL}};
const Segment SGI330818[] = {{&S11219[74], 5}, {NULL}};
const Segment SGI330819[] = {{&S11219[12], 14}, {&S11219[37], 42}, {NULL}};
const Segment SGI330820[] = {{&S11219[12], 14}, {NULL}};
const Segment SGI330821[] = {{&S11220[6], 4}, {NULL}};
const Segment SGI330822[] = {{&S11220[75], 5}, {NULL}};
const Segment SGI330823[] = {{&S11220[12], 14}, {&S11220[37], 43}, {NULL}};
const Segment SGI330824[] = {{&S11220[12], 14}, {NULL}};
const Segment SGI330825[] = {{&S11221[6], 4}, {NULL}};
const Segment SGI330826[] = {{&S11221[81], 5}, {NULL}};
const Segment SGI330827[] = {{&S11221[12], 14}, {&S11221[44], 42}, {NULL}};
const Segment SGI330828[] = {{&S11221[12], 14}, {NULL}};
const Segment SGI330829[] = {{&S11223[6], 4}, {NULL}};
const Segment SGI330830[] = {{&S11223[74], 5}, {NULL}};
const Segment SGI330831[] = {{&S11223[12], 14}, {&S11223[37], 42}, {NULL}};
const Segment SGI330832[] = {{&S11223[12], 14}, {NULL}};
const Segment SGI330833[] = {{&S11224[6], 4}, {NULL}};
const Segment SGI330834[] = {{&S11224[81], 5}, {NULL}};
const Segment SGI330835[] = {{&S11224[12], 14}, {&S11224[44], 42}, {NULL}};
const Segment SGI330836[] = {{&S11224[12], 14}, {NULL}};
const Segment SGI330837[] = {{&S11225[6], 4}, {NULL}};
const Segment SGI330838[] = {{&S11225[74], 5}, {NULL}};
const Segment SGI330839[] = {{&S11225[12], 14}, {&S11225[37], 42}, {NULL}};
const Segment SGI330840[] = {{&S11225[12], 14}, {NULL}};
const Segment SGI330841[] = {{&S11226[6], 4}, {NULL}};
const Segment SGI330842[] = {{&S11226[81], 5}, {NULL}};
const Segment SGI330843[] = {{&S11226[12], 14}, {&S11226[44], 42}, {NULL}};
const Segment SGI330844[] = {{&S11226[12], 14}, {NULL}};
const Segment SGI330845[] = {{&S11227[6], 4}, {NULL}};
const Segment SGI330846[] = {{&S11227[74], 5}, {NULL}};
const Segment SGI330847[] = {{&S11227[12], 14}, {&S11227[37], 42}, {NULL}};
const Segment SGI330848[] = {{&S11227[12], 14}, {NULL}};
const Segment SGI330849[] = {{&S11228[6], 4}, {NULL}};
const Segment SGI330850[] = {{&S11228[75], 5}, {NULL}};
const Segment SGI330851[] = {{&S11228[12], 14}, {&S11228[37], 43}, {NULL}};
const Segment SGI330852[] = {{&S11228[12], 14}, {NULL}};
const Segment SGI330853[] = {{&S11229[6], 4}, {NULL}};
const Segment SGI330854[] = {{&S11229[74], 5}, {NULL}};
const Segment SGI330855[] = {{&S11229[12], 14}, {&S11229[37], 42}, {NULL}};
const Segment SGI330856[] = {{&S11229[12], 14}, {NULL}};
const Segment SGI330857[] = {{&S11230[6], 4}, {NULL}};
const Segment SGI330858[] = {{&S11230[82], 5}, {NULL}};
const Segment SGI330859[] = {{&S11230[12], 14}, {&S11230[44], 43}, {NULL}};
const Segment SGI330860[] = {{&S11230[12], 14}, {NULL}};
const Segment SGI330861[] = {{&S11231[6], 4}, {NULL}};
const Segment SGI330862[] = {{&S11231[74], 5}, {NULL}};
const Segment SGI330863[] = {{&S11231[12], 14}, {&S11231[37], 42}, {NULL}};
const Segment SGI330864[] = {{&S11231[12], 14}, {NULL}};
const Segment SGI330865[] = {{&S11232[6], 4}, {NULL}};
const Segment SGI330866[] = {{&S11232[82], 5}, {NULL}};
const Segment SGI330867[] = {{&S11232[12], 14}, {&S11232[45], 42}, {NULL}};
const Segment SGI330868[] = {{&S11232[12], 14}, {NULL}};
const Segment SGI330869[] = {{&S11233[6], 4}, {NULL}};
const Segment SGI330870[] = {{&S11233[52], 5}, {NULL}};
const Segment SGI330871[] = {{&S11233[12], 62}, {NULL}};
const Segment SGI330872[] = {{&S11233[12], 14}, {NULL}};
const Segment SGI330873[] = {{&S11234[6], 4}, {NULL}};
const Segment SGI330874[] = {{&S11234[75], 5}, {NULL}};
const Segment SGI330875[] = {{&S11234[12], 14}, {&S11234[37], 43}, {NULL}};
const Segment SGI330876[] = {{&S11234[12], 14}, {NULL}};
const Segment SGI330877[] = {{&S11235[6], 4}, {NULL}};
const Segment SGI330878[] = {{&S11235[82], 5}, {NULL}};
const Segment SGI330879[] = {{&S11235[12], 14}, {&S11235[45], 42}, {NULL}};
const Segment SGI330880[] = {{&S11235[12], 14}, {NULL}};
const Segment SGI330881[] = {{&S11236[6], 4}, {NULL}};
const Segment SGI330882[] = {{&S11236[74], 5}, {NULL}};
const Segment SGI330883[] = {{&S11236[12], 14}, {&S11236[37], 42}, {NULL}};
const Segment SGI330884[] = {{&S11236[12], 14}, {NULL}};
const Segment SGI330885[] = {{&S11237[6], 4}, {NULL}};
const Segment SGI330886[] = {{&S11237[82], 5}, {NULL}};
const Segment SGI330887[] = {{&S11237[12], 14}, {&S11237[45], 42}, {NULL}};
const Segment SGI330888[] = {{&S11237[12], 14}, {NULL}};
const Segment SGI330889[] = {{&S11238[6], 4}, {NULL}};
const Segment SGI330890[] = {{&S11238[82], 5}, {NULL}};
const Segment SGI330891[] = {{&S11238[12], 14}, {&S11238[45], 42}, {NULL}};
const Segment SGI330892[] = {{&S11238[12], 14}, {NULL}};
const Segment SGI330893[] = {{&S11239[6], 4}, {NULL}};
const Segment SGI330894[] = {{&S11239[81], 5}, {NULL}};
const Segment SGI330895[] = {{&S11239[12], 14}, {&S11239[44], 42}, {NULL}};
const Segment SGI330896[] = {{&S11239[12], 14}, {NULL}};
const Segment SGI330897[] = {{&S11240[6], 4}, {NULL}};
const Segment SGI330898[] = {{&S11240[75], 5}, {NULL}};
const Segment SGI330899[] = {{&S11240[12], 14}, {&S11240[37], 43}, {NULL}};
const Segment SGI330900[] = {{&S11240[12], 14}, {NULL}};
const Segment SGI330901[] = {{&S11241[6], 4}, {NULL}};
const Segment SGI330902[] = {{&S11241[75], 5}, {NULL}};
const Segment SGI330903[] = {{&S11241[12], 14}, {&S11241[38], 42}, {NULL}};
const Segment SGI330904[] = {{&S11241[12], 14}, {NULL}};
const Segment SGI330905[] = {{&S11242[6], 4}, {NULL}};
const Segment SGI330906[] = {{&S11242[75], 5}, {NULL}};
const Segment SGI330907[] = {{&S11242[12], 14}, {&S11242[37], 43}, {NULL}};
const Segment SGI330908[] = {{&S11242[12], 14}, {NULL}};
const Segment SGI330909[] = {{&S11243[6], 4}, {NULL}};
const Segment SGI330910[] = {{&S11243[75], 5}, {NULL}};
const Segment SGI330911[] = {{&S11243[12], 14}, {&S11243[38], 42}, {NULL}};
const Segment SGI330912[] = {{&S11243[12], 14}, {NULL}};
const Segment SGI330913[] = {{&S11244[6], 4}, {NULL}};
const Segment SGI330914[] = {{&S11244[81], 5}, {NULL}};
const Segment SGI330915[] = {{&S11244[12], 14}, {&S11244[44], 42}, {NULL}};
const Segment SGI330916[] = {{&S11244[12], 14}, {NULL}};
const Segment SGI330917[] = {{&S11245[6], 4}, {NULL}};
const Segment SGI330918[] = {{&S11245[81], 5}, {NULL}};
const Segment SGI330919[] = {{&S11245[12], 14}, {&S11245[44], 42}, {NULL}};
const Segment SGI330920[] = {{&S11245[12], 14}, {NULL}};
const Segment SGI330921[] = {{&S11246[6], 4}, {NULL}};
const Segment SGI330922[] = {{&S11246[74], 5}, {NULL}};
const Segment SGI330923[] = {{&S11246[12], 14}, {&S11246[37], 42}, {NULL}};
const Segment SGI330924[] = {{&S11246[12], 14}, {NULL}};
const Segment SGI330925[] = {{&S11247[6], 4}, {NULL}};
const Segment SGI330926[] = {{&S11247[74], 5}, {NULL}};
const Segment SGI330927[] = {{&S11247[12], 14}, {&S11247[37], 42}, {NULL}};
const Segment SGI330928[] = {{&S11247[12], 14}, {NULL}};
const Segment SGI330929[] = {{&S11248[6], 4}, {NULL}};
const Segment SGI330930[] = {{&S11248[81], 5}, {NULL}};
const Segment SGI330931[] = {{&S11248[12], 14}, {&S11248[44], 42}, {NULL}};
const Segment SGI330932[] = {{&S11248[12], 14}, {NULL}};
const Segment SGI330933[] = {{&S11249[6], 4}, {NULL}};
const Segment SGI330934[] = {{&S11249[74], 5}, {NULL}};
const Segment SGI330935[] = {{&S11249[12], 14}, {&S11249[37], 42}, {NULL}};
const Segment SGI330936[] = {{&S11249[12], 14}, {NULL}};
const Segment SGI330937[] = {{&S11250[6], 4}, {NULL}};
const Segment SGI330938[] = {{&S11250[81], 5}, {NULL}};
const Segment SGI330939[] = {{&S11250[12], 14}, {&S11250[44], 42}, {NULL}};
const Segment SGI330940[] = {{&S11250[12], 14}, {NULL}};
const Segment SGI330941[] = {{&S11251[6], 4}, {NULL}};
const Segment SGI330942[] = {{&S11251[75], 5}, {NULL}};
const Segment SGI330943[] = {{&S11251[12], 14}, {&S11251[38], 42}, {NULL}};
const Segment SGI330944[] = {{&S11251[12], 14}, {NULL}};
const Segment SGI330945[] = {{&S11252[6], 4}, {NULL}};
const Segment SGI330946[] = {{&S11252[74], 5}, {NULL}};
const Segment SGI330947[] = {{&S11252[12], 14}, {&S11252[37], 42}, {NULL}};
const Segment SGI330948[] = {{&S11252[12], 14}, {NULL}};
const Segment SGI330949[] = {{&S11253[6], 4}, {NULL}};
const Segment SGI330950[] = {{&S11253[82], 5}, {NULL}};
const Segment SGI330951[] = {{&S11253[12], 14}, {&S11253[44], 43}, {NULL}};
const Segment SGI330952[] = {{&S11253[12], 14}, {NULL}};
const Segment SGI330953[] = {{&S11254[6], 4}, {NULL}};
const Segment SGI330954[] = {{&S11254[75], 5}, {NULL}};
const Segment SGI330955[] = {{&S11254[12], 14}, {&S11254[37], 43}, {NULL}};
const Segment SGI330956[] = {{&S11254[12], 14}, {NULL}};
const Segment SGI330957[] = {{&S11255[6], 4}, {NULL}};
const Segment SGI330958[] = {{&S11255[75], 5}, {NULL}};
const Segment SGI330959[] = {{&S11255[12], 14}, {&S11255[37], 43}, {NULL}};
const Segment SGI330960[] = {{&S11255[12], 14}, {NULL}};
const Segment SGI330961[] = {{&S11256[6], 4}, {NULL}};
const Segment SGI330962[] = {{&S11256[74], 5}, {NULL}};
const Segment SGI330963[] = {{&S11256[12], 14}, {&S11256[37], 42}, {NULL}};
const Segment SGI330964[] = {{&S11256[12], 14}, {NULL}};
const Segment SGI330965[] = {{&S11257[6], 4}, {NULL}};
const Segment SGI330966[] = {{&S11257[74], 5}, {NULL}};
const Segment SGI330967[] = {{&S11257[12], 14}, {&S11257[37], 42}, {NULL}};
const Segment SGI330968[] = {{&S11257[12], 14}, {NULL}};
const Segment SGI330969[] = {{&S11258[6], 4}, {NULL}};
const Segment SGI330970[] = {{&S11258[74], 5}, {NULL}};
const Segment SGI330971[] = {{&S11258[12], 14}, {&S11258[37], 42}, {NULL}};
const Segment SGI330972[] = {{&S11258[12], 14}, {NULL}};
const Segment SGI330973[] = {{&S11259[6], 4}, {NULL}};
const Segment SGI330974[] = {{&S11259[74], 5}, {NULL}};
const Segment SGI330975[] = {{&S11259[12], 14}, {&S11259[37], 42}, {NULL}};
const Segment SGI330976[] = {{&S11259[12], 14}, {NULL}};
const Segment SGI330977[] = {{&S11260[6], 4}, {NULL}};
const Segment SGI330978[] = {{&S11260[74], 5}, {NULL}};
const Segment SGI330979[] = {{&S11260[12], 14}, {&S11260[37], 42}, {NULL}};
const Segment SGI330980[] = {{&S11260[12], 14}, {NULL}};
const Segment SGI330981[] = {{&S11261[6], 4}, {NULL}};
const Segment SGI330982[] = {{&S11261[81], 5}, {NULL}};
const Segment SGI330983[] = {{&S11261[12], 14}, {&S11261[44], 42}, {NULL}};
const Segment SGI330984[] = {{&S11261[12], 14}, {NULL}};
const Segment SGI330985[] = {{&S11262[6], 4}, {NULL}};
const Segment SGI330986[] = {{&S11262[81], 5}, {NULL}};
const Segment SGI330987[] = {{&S11262[12], 14}, {&S11262[44], 42}, {NULL}};
const Segment SGI330988[] = {{&S11262[12], 14}, {NULL}};
const Segment SGI330989[] = {{&S11263[6], 4}, {NULL}};
const Segment SGI330990[] = {{&S11263[74], 5}, {NULL}};
const Segment SGI330991[] = {{&S11263[12], 14}, {&S11263[37], 42}, {NULL}};
const Segment SGI330992[] = {{&S11263[12], 14}, {NULL}};
const Segment SGI330993[] = {{&S11264[6], 4}, {NULL}};
const Segment SGI330994[] = {{&S11264[82], 5}, {NULL}};
const Segment SGI330995[] = {{&S11264[12], 14}, {&S11264[45], 42}, {NULL}};
const Segment SGI330996[] = {{&S11264[12], 14}, {NULL}};
const Segment SGI330997[] = {{&S11265[6], 4}, {NULL}};
const Segment SGI330998[] = {{&S11265[74], 5}, {NULL}};
const Segment SGI330999[] = {{&S11265[12], 14}, {&S11265[37], 42}, {NULL}};
const Segment SGI331000[] = {{&S11265[12], 14}, {NULL}};
const Segment SGI331001[] = {{&S11266[6], 4}, {NULL}};
const Segment SGI331002[] = {{&S11266[81], 5}, {NULL}};
const Segment SGI331003[] = {{&S11266[12], 14}, {&S11266[44], 42}, {NULL}};
const Segment SGI331004[] = {{&S11266[12], 14}, {NULL}};
const Segment SGI331005[] = {{&S11267[6], 4}, {NULL}};
const Segment SGI331006[] = {{&S11267[74], 5}, {NULL}};
const Segment SGI331007[] = {{&S11267[12], 14}, {&S11267[37], 42}, {NULL}};
const Segment SGI331008[] = {{&S11267[12], 14}, {NULL}};
const Segment SGI331009[] = {{&S11268[6], 4}, {NULL}};
const Segment SGI331010[] = {{&S11268[74], 5}, {NULL}};
const Segment SGI331011[] = {{&S11268[12], 14}, {&S11268[37], 42}, {NULL}};
const Segment SGI331012[] = {{&S11268[12], 14}, {NULL}};
const Segment SGI331013[] = {{&S11269[6], 4}, {NULL}};
const Segment SGI331014[] = {{&S11269[81], 5}, {NULL}};
const Segment SGI331015[] = {{&S11269[12], 14}, {&S11269[44], 42}, {NULL}};
const Segment SGI331016[] = {{&S11269[12], 14}, {NULL}};
const Segment SGI331017[] = {{&S11270[6], 4}, {NULL}};
const Segment SGI331018[] = {{&S11270[81], 5}, {NULL}};
const Segment SGI331019[] = {{&S11270[12], 14}, {&S11270[44], 42}, {NULL}};
const Segment SGI331020[] = {{&S11270[12], 14}, {NULL}};
const Segment SGI331021[] = {{&S11271[6], 4}, {NULL}};
const Segment SGI331022[] = {{&S11271[109], 5}, {NULL}};
const Segment SGI331023[] = {{&S11271[12], 14}, {&S11271[44], 12}, {NULL}};
const Segment SGI331024[] = {{&S11271[12], 14}, {NULL}};
const Segment SGI331025[] = {{&S11272[6], 4}, {NULL}};
const Segment SGI331026[] = {{&S11272[74], 5}, {NULL}};
const Segment SGI331027[] = {{&S11272[12], 14}, {&S11272[37], 42}, {NULL}};
const Segment SGI331028[] = {{&S11272[12], 14}, {NULL}};
const Segment SGI331029[] = {{&S11273[6], 4}, {NULL}};
const Segment SGI331030[] = {{&S11273[81], 5}, {NULL}};
const Segment SGI331031[] = {{&S11273[12], 14}, {&S11273[44], 42}, {NULL}};
const Segment SGI331032[] = {{&S11273[12], 14}, {NULL}};
const Segment SGI331033[] = {{&S11274[6], 4}, {NULL}};
const Segment SGI331034[] = {{&S11274[75], 5}, {NULL}};
const Segment SGI331035[] = {{&S11274[12], 14}, {&S11274[37], 43}, {NULL}};
const Segment SGI331036[] = {{&S11274[12], 14}, {NULL}};
const Segment SGI331037[] = {{&S11275[6], 4}, {NULL}};
const Segment SGI331038[] = {{&S11275[75], 5}, {NULL}};
const Segment SGI331039[] = {{&S11275[12], 14}, {&S11275[37], 43}, {NULL}};
const Segment SGI331040[] = {{&S11275[12], 14}, {NULL}};
const Segment SGI331041[] = {{&S11276[6], 4}, {NULL}};
const Segment SGI331042[] = {{&S11276[75], 5}, {NULL}};
const Segment SGI331043[] = {{&S11276[12], 14}, {&S11276[37], 43}, {NULL}};
const Segment SGI331044[] = {{&S11276[12], 14}, {NULL}};
const Segment SGI331045[] = {{&S11277[6], 4}, {NULL}};
const Segment SGI331046[] = {{&S11277[73], 5}, {NULL}};
const Segment SGI331047[] = {{&S11277[12], 14}, {&S11277[37], 41}, {NULL}};
const Segment SGI331048[] = {{&S11277[12], 14}, {NULL}};
const Segment SGI331049[] = {{&S11278[6], 4}, {NULL}};
const Segment SGI331050[] = {{&S11278[74], 5}, {NULL}};
const Segment SGI331051[] = {{&S11278[12], 14}, {&S11278[37], 42}, {NULL}};
const Segment SGI331052[] = {{&S11278[12], 14}, {NULL}};
const Segment SGI331053[] = {{&S11279[6], 4}, {NULL}};
const Segment SGI331054[] = {{&S11279[74], 5}, {NULL}};
const Segment SGI331055[] = {{&S11279[12], 14}, {&S11279[37], 42}, {NULL}};
const Segment SGI331056[] = {{&S11279[12], 14}, {NULL}};
const Segment SGI331057[] = {{&S11280[6], 4}, {NULL}};
const Segment SGI331058[] = {{&S11280[74], 5}, {NULL}};
const Segment SGI331059[] = {{&S11280[12], 14}, {&S11280[37], 42}, {NULL}};
const Segment SGI331060[] = {{&S11280[12], 14}, {NULL}};
const Segment SGI331061[] = {{&S11281[6], 4}, {NULL}};
const Segment SGI331062[] = {{&S11281[74], 5}, {NULL}};
const Segment SGI331063[] = {{&S11281[12], 14}, {&S11281[37], 42}, {NULL}};
const Segment SGI331064[] = {{&S11281[12], 14}, {NULL}};
const Segment SGI331065[] = {{&S11282[6], 4}, {NULL}};
const Segment SGI331066[] = {{&S11282[74], 5}, {NULL}};
const Segment SGI331067[] = {{&S11282[12], 14}, {&S11282[37], 42}, {NULL}};
const Segment SGI331068[] = {{&S11282[12], 14}, {NULL}};
const Segment SGI331069[] = {{&S11283[6], 4}, {NULL}};
const Segment SGI331070[] = {{&S11283[81], 5}, {NULL}};
const Segment SGI331071[] = {{&S11283[12], 14}, {&S11283[44], 42}, {NULL}};
const Segment SGI331072[] = {{&S11283[12], 14}, {NULL}};
const Segment SGI331073[] = {{&S11284[6], 4}, {NULL}};
const Segment SGI331074[] = {{&S11284[74], 5}, {NULL}};
const Segment SGI331075[] = {{&S11284[12], 14}, {&S11284[37], 42}, {NULL}};
const Segment SGI331076[] = {{&S11284[12], 14}, {NULL}};
const Segment SGI331077[] = {{&S11285[6], 4}, {NULL}};
const Segment SGI331078[] = {{&S11285[81], 5}, {NULL}};
const Segment SGI331079[] = {{&S11285[12], 14}, {&S11285[44], 42}, {NULL}};
const Segment SGI331080[] = {{&S11285[12], 14}, {NULL}};
const Segment SGI331081[] = {{&S11286[6], 4}, {NULL}};
const Segment SGI331082[] = {{&S11286[81], 5}, {NULL}};
const Segment SGI331083[] = {{&S11286[12], 14}, {&S11286[44], 42}, {NULL}};
const Segment SGI331084[] = {{&S11286[12], 14}, {NULL}};
const Segment SGI331085[] = {{&S11287[6], 4}, {NULL}};
const Segment SGI331086[] = {{&S11287[81], 5}, {NULL}};
const Segment SGI331087[] = {{&S11287[12], 14}, {&S11287[44], 42}, {NULL}};
const Segment SGI331088[] = {{&S11287[12], 14}, {NULL}};
const Segment SGI331089[] = {{&S11288[6], 4}, {NULL}};
const Segment SGI331090[] = {{&S11288[74], 5}, {NULL}};
const Segment SGI331091[] = {{&S11288[12], 14}, {&S11288[37], 42}, {NULL}};
const Segment SGI331092[] = {{&S11288[12], 14}, {NULL}};
const Segment SGI331093[] = {{&S11289[6], 4}, {NULL}};
const Segment SGI331094[] = {{&S11289[81], 5}, {NULL}};
const Segment SGI331095[] = {{&S11289[12], 14}, {&S11289[44], 42}, {NULL}};
const Segment SGI331096[] = {{&S11289[12], 14}, {NULL}};
const Segment SGI331097[] = {{&S12232[64], 5}, {NULL}};
const Segment SGI331098[] = {{&S12232[13], 10}, {NULL}};
const Segment SGI331099[] = {{&S12496[64], 5}, {NULL}};
const Segment SGI331100[] = {{&S12496[13], 10}, {NULL}};
const Segment SGI331101[] = {{&S11719[6], 5}, {NULL}};
const Segment SGI331102[] = {{&S11719[76], 5}, {NULL}};
const Segment SGI331103[] = {{&S11719[12], 6}, {NULL}};
const Segment SGI331104[] = {{&S36528[5], 5}, {&S36528[68], 5}, {NULL}};
const Segment SGI331105[] = {{&S36529[5], 5}, {&S36529[5], 5}, {NULL}};
const Segment SGI331106[] = {{&S36530[5], 5}, {NULL}};
const Segment SGI331107[] = {{&S36531[5], 7}, {NULL}};
const Segment SGI331108[] = {{&S12813[61], 5}, {NULL}};
const Segment SGI331109[] = {{&S12813[13], 10}, {NULL}};
const Segment SGI331110[] = {{&S12211[6], 4}, {NULL}};
const Segment SGI331111[] = {{&S12211[59], 5}, {NULL}};
const Segment SGI331112[] = {{&S12211[12], 14}, {NULL}};
const Segment SGI331113[] = {{&S12212[6], 4}, {NULL}};
const Segment SGI331114[] = {{&S12212[52], 5}, {NULL}};
const Segment SGI331115[] = {{&S12212[12], 45}, {NULL}};
const Segment SGI331116[] = {{&S12212[12], 14}, {NULL}};
const Segment SGI331117[] = {{&S12214[6], 4}, {NULL}};
const Segment SGI331118[] = {{&S12214[74], 5}, {NULL}};
const Segment SGI331119[] = {{&S12214[12], 14}, {&S12214[37], 42}, {NULL}};
const Segment SGI331120[] = {{&S12214[12], 14}, {NULL}};
const Segment SGI331121[] = {{&S12215[6], 4}, {NULL}};
const Segment SGI331122[] = {{&S12215[74], 5}, {NULL}};
const Segment SGI331123[] = {{&S12215[12], 14}, {&S12215[37], 42}, {NULL}};
const Segment SGI331124[] = {{&S12215[12], 14}, {NULL}};
const Segment SGI331125[] = {{&S12216[6], 4}, {NULL}};
const Segment SGI331126[] = {{&S12216[74], 5}, {NULL}};
const Segment SGI331127[] = {{&S12216[12], 14}, {&S12216[37], 42}, {NULL}};
const Segment SGI331128[] = {{&S12216[12], 14}, {NULL}};
const Segment SGI331129[] = {{&S12217[6], 4}, {NULL}};
const Segment SGI331130[] = {{&S12217[59], 5}, {NULL}};
const Segment SGI331131[] = {{&S12217[12], 14}, {&S12217[70], 4}, {NULL}};
const Segment SGI331132[] = {{&S12217[12], 14}, {NULL}};
const Segment SGI331133[] = {{&S12218[6], 4}, {NULL}};
const Segment SGI331134[] = {{&S12218[74], 5}, {NULL}};
const Segment SGI331135[] = {{&S12218[12], 14}, {&S12218[37], 42}, {NULL}};
const Segment SGI331136[] = {{&S12218[12], 14}, {NULL}};
const Segment SGI331137[] = {{&S12219[6], 4}, {NULL}};
const Segment SGI331138[] = {{&S12219[52], 5}, {NULL}};
const Segment SGI331139[] = {{&S12219[12], 55}, {NULL}};
const Segment SGI331140[] = {{&S12219[12], 14}, {NULL}};
const Segment SGI331141[] = {{&S12220[6], 4}, {NULL}};
const Segment SGI331142[] = {{&S12220[74], 5}, {NULL}};
const Segment SGI331143[] = {{&S12220[12], 14}, {&S12220[37], 42}, {NULL}};
const Segment SGI331144[] = {{&S12220[12], 14}, {NULL}};
const Segment SGI331145[] = {{&S12221[6], 4}, {NULL}};
const Segment SGI331146[] = {{&S12221[81], 5}, {NULL}};
const Segment SGI331147[] = {{&S12221[12], 14}, {&S12221[44], 42}, {NULL}};
const Segment SGI331148[] = {{&S12221[12], 14}, {NULL}};
const Segment SGI331149[] = {{&S12222[6], 4}, {NULL}};
const Segment SGI331150[] = {{&S12222[81], 5}, {NULL}};
const Segment SGI331151[] = {{&S12222[12], 14}, {&S12222[44], 42}, {NULL}};
const Segment SGI331152[] = {{&S12222[12], 14}, {NULL}};
const Segment SGI331153[] = {{&S12223[6], 4}, {NULL}};
const Segment SGI331154[] = {{&S12223[81], 5}, {NULL}};
const Segment SGI331155[] = {{&S12223[12], 14}, {&S12223[44], 42}, {NULL}};
const Segment SGI331156[] = {{&S12223[12], 14}, {NULL}};
const Segment SGI331157[] = {{&S12224[6], 4}, {NULL}};
const Segment SGI331158[] = {{&S12224[74], 5}, {NULL}};
const Segment SGI331159[] = {{&S12224[12], 14}, {&S12224[37], 42}, {NULL}};
const Segment SGI331160[] = {{&S12224[12], 14}, {NULL}};
const Segment SGI331161[] = {{&S12225[6], 4}, {NULL}};
const Segment SGI331162[] = {{&S12225[81], 5}, {NULL}};
const Segment SGI331163[] = {{&S12225[12], 14}, {&S12225[44], 42}, {NULL}};
const Segment SGI331164[] = {{&S12225[12], 14}, {NULL}};
const Segment SGI331165[] = {{&S12226[6], 4}, {NULL}};
const Segment SGI331166[] = {{&S12226[81], 5}, {NULL}};
const Segment SGI331167[] = {{&S12226[12], 14}, {&S12226[44], 42}, {NULL}};
const Segment SGI331168[] = {{&S12226[12], 14}, {NULL}};
const Segment SGI331169[] = {{&S12227[6], 4}, {NULL}};
const Segment SGI331170[] = {{&S12227[75], 5}, {NULL}};
const Segment SGI331171[] = {{&S12227[12], 14}, {&S12227[38], 42}, {NULL}};
const Segment SGI331172[] = {{&S12227[12], 14}, {NULL}};
const Segment SGI331173[] = {{&S12228[6], 4}, {NULL}};
const Segment SGI331174[] = {{&S12228[75], 5}, {NULL}};
const Segment SGI331175[] = {{&S12228[12], 14}, {&S12228[37], 43}, {NULL}};
const Segment SGI331176[] = {{&S12228[12], 14}, {NULL}};
const Segment SGI331177[] = {{&S12229[6], 4}, {NULL}};
const Segment SGI331178[] = {{&S12229[81], 5}, {NULL}};
const Segment SGI331179[] = {{&S12229[12], 14}, {&S12229[44], 42}, {NULL}};
const Segment SGI331180[] = {{&S12229[12], 14}, {NULL}};
const Segment SGI331181[] = {{&S12230[6], 4}, {NULL}};
const Segment SGI331182[] = {{&S12230[74], 5}, {NULL}};
const Segment SGI331183[] = {{&S12230[12], 14}, {&S12230[37], 42}, {NULL}};
const Segment SGI331184[] = {{&S12230[12], 14}, {NULL}};
const Segment SGI331185[] = {{&S12231[6], 4}, {NULL}};
const Segment SGI331186[] = {{&S12231[75], 5}, {NULL}};
const Segment SGI331187[] = {{&S12231[12], 14}, {&S12231[37], 43}, {NULL}};
const Segment SGI331188[] = {{&S12231[12], 14}, {NULL}};
const Segment SGI331189[] = {{&S12233[6], 4}, {NULL}};
const Segment SGI331190[] = {{&S12233[75], 5}, {NULL}};
const Segment SGI331191[] = {{&S12233[12], 14}, {&S12233[38], 42}, {NULL}};
const Segment SGI331192[] = {{&S12233[12], 14}, {NULL}};
const Segment SGI331193[] = {{&S12234[6], 4}, {NULL}};
const Segment SGI331194[] = {{&S12234[74], 5}, {NULL}};
const Segment SGI331195[] = {{&S12234[12], 14}, {&S12234[37], 42}, {NULL}};
const Segment SGI331196[] = {{&S12234[12], 14}, {NULL}};
const Segment SGI331197[] = {{&S12235[6], 4}, {NULL}};
const Segment SGI331198[] = {{&S12235[74], 5}, {NULL}};
const Segment SGI331199[] = {{&S12235[12], 14}, {&S12235[37], 42}, {NULL}};
const Segment SGI331200[] = {{&S12235[12], 14}, {NULL}};
const Segment SGI331201[] = {{&S12236[6], 4}, {NULL}};
const Segment SGI331202[] = {{&S12236[82], 5}, {NULL}};
const Segment SGI331203[] = {{&S12236[12], 14}, {&S12236[45], 42}, {NULL}};
const Segment SGI331204[] = {{&S12236[12], 14}, {NULL}};
const Segment SGI331205[] = {{&S12237[6], 4}, {NULL}};
const Segment SGI331206[] = {{&S12237[74], 5}, {NULL}};
const Segment SGI331207[] = {{&S12237[12], 14}, {&S12237[37], 42}, {NULL}};
const Segment SGI331208[] = {{&S12237[12], 14}, {NULL}};
const Segment SGI331209[] = {{&S12238[6], 4}, {NULL}};
const Segment SGI331210[] = {{&S12238[74], 5}, {NULL}};
const Segment SGI331211[] = {{&S12238[12], 14}, {&S12238[37], 42}, {NULL}};
const Segment SGI331212[] = {{&S12238[12], 14}, {NULL}};
const Segment SGI331213[] = {{&S12239[6], 4}, {NULL}};
const Segment SGI331214[] = {{&S12239[81], 5}, {NULL}};
const Segment SGI331215[] = {{&S12239[12], 14}, {&S12239[44], 42}, {NULL}};
const Segment SGI331216[] = {{&S12239[12], 14}, {NULL}};
const Segment SGI331217[] = {{&S12240[6], 4}, {NULL}};
const Segment SGI331218[] = {{&S12240[75], 5}, {NULL}};
const Segment SGI331219[] = {{&S12240[12], 14}, {&S12240[37], 43}, {NULL}};
const Segment SGI331220[] = {{&S12240[12], 14}, {NULL}};
const Segment SGI331221[] = {{&S12241[6], 4}, {NULL}};
const Segment SGI331222[] = {{&S12241[81], 5}, {NULL}};
const Segment SGI331223[] = {{&S12241[12], 14}, {&S12241[44], 42}, {NULL}};
const Segment SGI331224[] = {{&S12241[12], 14}, {NULL}};
const Segment SGI331225[] = {{&S12243[6], 4}, {NULL}};
const Segment SGI331226[] = {{&S12243[75], 5}, {NULL}};
const Segment SGI331227[] = {{&S12243[12], 14}, {&S12243[37], 43}, {NULL}};
const Segment SGI331228[] = {{&S12243[12], 14}, {NULL}};
const Segment SGI331229[] = {{&S12244[6], 4}, {NULL}};
const Segment SGI331230[] = {{&S12244[74], 5}, {NULL}};
const Segment SGI331231[] = {{&S12244[12], 14}, {&S12244[37], 42}, {NULL}};
const Segment SGI331232[] = {{&S12244[12], 14}, {NULL}};
const Segment SGI331233[] = {{&S12245[6], 4}, {NULL}};
const Segment SGI331234[] = {{&S12245[81], 5}, {NULL}};
const Segment SGI331235[] = {{&S12245[12], 14}, {&S12245[44], 42}, {NULL}};
const Segment SGI331236[] = {{&S12245[12], 14}, {NULL}};
const Segment SGI331237[] = {{&S12246[6], 4}, {NULL}};
const Segment SGI331238[] = {{&S12246[74], 5}, {NULL}};
const Segment SGI331239[] = {{&S12246[12], 14}, {&S12246[37], 42}, {NULL}};
const Segment SGI331240[] = {{&S12246[12], 14}, {NULL}};
const Segment SGI331241[] = {{&S12247[6], 4}, {NULL}};
const Segment SGI331242[] = {{&S12247[75], 5}, {NULL}};
const Segment SGI331243[] = {{&S12247[12], 14}, {&S12247[38], 42}, {NULL}};
const Segment SGI331244[] = {{&S12247[12], 14}, {NULL}};
const Segment SGI331245[] = {{&S12248[6], 4}, {NULL}};
const Segment SGI331246[] = {{&S12248[81], 5}, {NULL}};
const Segment SGI331247[] = {{&S12248[12], 14}, {&S12248[44], 42}, {NULL}};
const Segment SGI331248[] = {{&S12248[12], 14}, {NULL}};
const Segment SGI331249[] = {{&S12249[6], 4}, {NULL}};
const Segment SGI331250[] = {{&S12249[75], 5}, {NULL}};
const Segment SGI331251[] = {{&S12249[12], 14}, {&S12249[38], 42}, {NULL}};
const Segment SGI331252[] = {{&S12249[12], 14}, {NULL}};
const Segment SGI331253[] = {{&S12250[6], 4}, {NULL}};
const Segment SGI331254[] = {{&S12250[82], 5}, {NULL}};
const Segment SGI331255[] = {{&S12250[12], 14}, {&S12250[45], 42}, {NULL}};
const Segment SGI331256[] = {{&S12250[12], 14}, {NULL}};
const Segment SGI331257[] = {{&S12251[6], 4}, {NULL}};
const Segment SGI331258[] = {{&S12251[81], 5}, {NULL}};
const Segment SGI331259[] = {{&S12251[12], 14}, {&S12251[44], 42}, {NULL}};
const Segment SGI331260[] = {{&S12251[12], 14}, {NULL}};
const Segment SGI331261[] = {{&S12252[6], 4}, {NULL}};
const Segment SGI331262[] = {{&S12252[81], 5}, {NULL}};
const Segment SGI331263[] = {{&S12252[12], 14}, {&S12252[44], 42}, {NULL}};
const Segment SGI331264[] = {{&S12252[12], 14}, {NULL}};
const Segment SGI331265[] = {{&S12253[6], 4}, {NULL}};
const Segment SGI331266[] = {{&S12253[74], 5}, {NULL}};
const Segment SGI331267[] = {{&S12253[12], 14}, {&S12253[37], 42}, {NULL}};
const Segment SGI331268[] = {{&S12253[12], 14}, {NULL}};
const Segment SGI331269[] = {{&S12254[6], 4}, {NULL}};
const Segment SGI331270[] = {{&S12254[81], 5}, {NULL}};
const Segment SGI331271[] = {{&S12254[12], 14}, {&S12254[44], 42}, {NULL}};
const Segment SGI331272[] = {{&S12254[12], 14}, {NULL}};
const Segment SGI331273[] = {{&S12255[6], 4}, {NULL}};
const Segment SGI331274[] = {{&S12255[75], 5}, {NULL}};
const Segment SGI331275[] = {{&S12255[12], 14}, {&S12255[38], 42}, {NULL}};
const Segment SGI331276[] = {{&S12255[12], 14}, {NULL}};
const Segment SGI331277[] = {{&S12256[6], 4}, {NULL}};
const Segment SGI331278[] = {{&S12256[75], 5}, {NULL}};
const Segment SGI331279[] = {{&S12256[12], 14}, {&S12256[38], 42}, {NULL}};
const Segment SGI331280[] = {{&S12256[12], 14}, {NULL}};
const Segment SGI331281[] = {{&S12257[6], 4}, {NULL}};
const Segment SGI331282[] = {{&S12257[75], 5}, {NULL}};
const Segment SGI331283[] = {{&S12257[12], 14}, {&S12257[38], 42}, {NULL}};
const Segment SGI331284[] = {{&S12257[12], 14}, {NULL}};
const Segment SGI331285[] = {{&S12258[6], 4}, {NULL}};
const Segment SGI331286[] = {{&S12258[75], 5}, {NULL}};
const Segment SGI331287[] = {{&S12258[12], 14}, {&S12258[38], 42}, {NULL}};
const Segment SGI331288[] = {{&S12258[12], 14}, {NULL}};
const Segment SGI331289[] = {{&S12259[6], 4}, {NULL}};
const Segment SGI331290[] = {{&S12259[82], 5}, {NULL}};
const Segment SGI331291[] = {{&S12259[12], 14}, {&S12259[45], 42}, {NULL}};
const Segment SGI331292[] = {{&S12259[12], 14}, {NULL}};
const Segment SGI331293[] = {{&S12260[6], 4}, {NULL}};
const Segment SGI331294[] = {{&S12260[52], 5}, {NULL}};
const Segment SGI331295[] = {{&S12260[12], 62}, {NULL}};
const Segment SGI331296[] = {{&S12260[12], 14}, {NULL}};
const Segment SGI331297[] = {{&S12261[6], 4}, {NULL}};
const Segment SGI331298[] = {{&S12261[74], 5}, {NULL}};
const Segment SGI331299[] = {{&S12261[12], 14}, {&S12261[37], 42}, {NULL}};
const Segment SGI331300[] = {{&S12261[12], 14}, {NULL}};
const Segment SGI331301[] = {{&S12262[6], 4}, {NULL}};
const Segment SGI331302[] = {{&S12262[81], 5}, {NULL}};
const Segment SGI331303[] = {{&S12262[12], 14}, {&S12262[44], 42}, {NULL}};
const Segment SGI331304[] = {{&S12262[12], 14}, {NULL}};
const Segment SGI331305[] = {{&S12263[6], 4}, {NULL}};
const Segment SGI331306[] = {{&S12263[81], 5}, {NULL}};
const Segment SGI331307[] = {{&S12263[12], 14}, {&S12263[44], 42}, {NULL}};
const Segment SGI331308[] = {{&S12263[12], 14}, {NULL}};
const Segment SGI331309[] = {{&S12264[6], 4}, {NULL}};
const Segment SGI331310[] = {{&S12264[81], 5}, {NULL}};
const Segment SGI331311[] = {{&S12264[12], 14}, {&S12264[44], 42}, {NULL}};
const Segment SGI331312[] = {{&S12264[12], 14}, {NULL}};
const Segment SGI331313[] = {{&S12265[6], 4}, {NULL}};
const Segment SGI331314[] = {{&S12265[75], 5}, {NULL}};
const Segment SGI331315[] = {{&S12265[12], 14}, {&S12265[37], 43}, {NULL}};
const Segment SGI331316[] = {{&S12265[12], 14}, {NULL}};
const Segment SGI331317[] = {{&S12266[6], 4}, {NULL}};
const Segment SGI331318[] = {{&S12266[74], 5}, {NULL}};
const Segment SGI331319[] = {{&S12266[12], 14}, {&S12266[37], 42}, {NULL}};
const Segment SGI331320[] = {{&S12266[12], 14}, {NULL}};
const Segment SGI331321[] = {{&S12267[6], 4}, {NULL}};
const Segment SGI331322[] = {{&S12267[81], 5}, {NULL}};
const Segment SGI331323[] = {{&S12267[12], 14}, {&S12267[44], 42}, {NULL}};
const Segment SGI331324[] = {{&S12267[12], 14}, {NULL}};
const Segment SGI331325[] = {{&S12268[6], 4}, {NULL}};
const Segment SGI331326[] = {{&S12268[82], 5}, {NULL}};
const Segment SGI331327[] = {{&S12268[12], 14}, {&S12268[45], 42}, {NULL}};
const Segment SGI331328[] = {{&S12268[12], 14}, {NULL}};
const Segment SGI331329[] = {{&S12269[6], 4}, {NULL}};
const Segment SGI331330[] = {{&S12269[75], 5}, {NULL}};
const Segment SGI331331[] = {{&S12269[12], 14}, {&S12269[38], 42}, {NULL}};
const Segment SGI331332[] = {{&S12269[12], 14}, {NULL}};
const Segment SGI331333[] = {{&S12270[6], 4}, {NULL}};
const Segment SGI331334[] = {{&S12270[75], 5}, {NULL}};
const Segment SGI331335[] = {{&S12270[12], 14}, {&S12270[37], 43}, {NULL}};
const Segment SGI331336[] = {{&S12270[12], 14}, {NULL}};
const Segment SGI331337[] = {{&S12271[6], 4}, {NULL}};
const Segment SGI331338[] = {{&S12271[81], 5}, {NULL}};
const Segment SGI331339[] = {{&S12271[12], 14}, {&S12271[44], 42}, {NULL}};
const Segment SGI331340[] = {{&S12271[12], 14}, {NULL}};
const Segment SGI331341[] = {{&S12272[6], 4}, {NULL}};
const Segment SGI331342[] = {{&S12272[75], 5}, {NULL}};
const Segment SGI331343[] = {{&S12272[12], 14}, {&S12272[37], 43}, {NULL}};
const Segment SGI331344[] = {{&S12272[12], 14}, {NULL}};
const Segment SGI331345[] = {{&S12273[6], 4}, {NULL}};
const Segment SGI331346[] = {{&S12273[75], 5}, {NULL}};
const Segment SGI331347[] = {{&S12273[12], 14}, {&S12273[37], 43}, {NULL}};
const Segment SGI331348[] = {{&S12273[12], 14}, {NULL}};
const Segment SGI331349[] = {{&S12274[6], 4}, {NULL}};
const Segment SGI331350[] = {{&S12274[74], 5}, {NULL}};
const Segment SGI331351[] = {{&S12274[12], 14}, {&S12274[37], 42}, {NULL}};
const Segment SGI331352[] = {{&S12274[12], 14}, {NULL}};
const Segment SGI331353[] = {{&S12275[6], 4}, {NULL}};
const Segment SGI331354[] = {{&S12275[82], 5}, {NULL}};
const Segment SGI331355[] = {{&S12275[12], 14}, {&S12275[44], 43}, {NULL}};
const Segment SGI331356[] = {{&S12275[12], 14}, {NULL}};
const Segment SGI331357[] = {{&S12276[6], 4}, {NULL}};
const Segment SGI331358[] = {{&S12276[74], 5}, {NULL}};
const Segment SGI331359[] = {{&S12276[12], 14}, {&S12276[37], 42}, {NULL}};
const Segment SGI331360[] = {{&S12276[12], 14}, {NULL}};
const Segment SGI331361[] = {{&S12277[6], 4}, {NULL}};
const Segment SGI331362[] = {{&S12277[81], 5}, {NULL}};
const Segment SGI331363[] = {{&S12277[12], 14}, {&S12277[44], 42}, {NULL}};
const Segment SGI331364[] = {{&S12277[12], 14}, {NULL}};
const Segment SGI331365[] = {{&S12278[6], 4}, {NULL}};
const Segment SGI331366[] = {{&S12278[74], 5}, {NULL}};
const Segment SGI331367[] = {{&S12278[12], 14}, {&S12278[37], 42}, {NULL}};
const Segment SGI331368[] = {{&S12278[12], 14}, {NULL}};
const Segment SGI331369[] = {{&S12279[6], 4}, {NULL}};
const Segment SGI331370[] = {{&S12279[81], 5}, {NULL}};
const Segment SGI331371[] = {{&S12279[12], 14}, {&S12279[44], 42}, {NULL}};
const Segment SGI331372[] = {{&S12279[12], 14}, {NULL}};
const Segment SGI331373[] = {{&S12280[6], 4}, {NULL}};
const Segment SGI331374[] = {{&S12280[74], 5}, {NULL}};
const Segment SGI331375[] = {{&S12280[12], 14}, {&S12280[37], 42}, {NULL}};
const Segment SGI331376[] = {{&S12280[12], 14}, {NULL}};
const Segment SGI331377[] = {{&S12281[6], 4}, {NULL}};
const Segment SGI331378[] = {{&S12281[81], 5}, {NULL}};
const Segment SGI331379[] = {{&S12281[12], 14}, {&S12281[44], 42}, {NULL}};
const Segment SGI331380[] = {{&S12281[12], 14}, {NULL}};
const Segment SGI331381[] = {{&S12282[6], 4}, {NULL}};
const Segment SGI331382[] = {{&S12282[74], 5}, {NULL}};
const Segment SGI331383[] = {{&S12282[12], 14}, {&S12282[37], 42}, {NULL}};
const Segment SGI331384[] = {{&S12282[12], 14}, {NULL}};
const Segment SGI331385[] = {{&S12283[6], 4}, {NULL}};
const Segment SGI331386[] = {{&S12283[74], 5}, {NULL}};
const Segment SGI331387[] = {{&S12283[12], 14}, {&S12283[37], 42}, {NULL}};
const Segment SGI331388[] = {{&S12283[12], 14}, {NULL}};
const Segment SGI331389[] = {{&S12284[6], 4}, {NULL}};
const Segment SGI331390[] = {{&S12284[81], 5}, {NULL}};
const Segment SGI331391[] = {{&S12284[12], 14}, {&S12284[44], 42}, {NULL}};
const Segment SGI331392[] = {{&S12284[12], 14}, {NULL}};
const Segment SGI331393[] = {{&S12285[6], 4}, {NULL}};
const Segment SGI331394[] = {{&S12285[81], 5}, {NULL}};
const Segment SGI331395[] = {{&S12285[12], 14}, {&S12285[44], 42}, {NULL}};
const Segment SGI331396[] = {{&S12285[12], 14}, {NULL}};
const Segment SGI331397[] = {{&S12286[6], 4}, {NULL}};
const Segment SGI331398[] = {{&S12286[81], 5}, {NULL}};
const Segment SGI331399[] = {{&S12286[12], 14}, {&S12286[44], 42}, {NULL}};
const Segment SGI331400[] = {{&S12286[12], 14}, {NULL}};
const Segment SGI331401[] = {{&S12287[6], 4}, {NULL}};
const Segment SGI331402[] = {{&S12287[82], 5}, {NULL}};
const Segment SGI331403[] = {{&S12287[12], 14}, {&S12287[45], 42}, {NULL}};
const Segment SGI331404[] = {{&S12287[12], 14}, {NULL}};
const Segment SGI331405[] = {{&S12288[6], 4}, {NULL}};
const Segment SGI331406[] = {{&S12288[81], 5}, {NULL}};
const Segment SGI331407[] = {{&S12288[12], 14}, {&S12288[44], 42}, {NULL}};
const Segment SGI331408[] = {{&S12288[12], 14}, {NULL}};
const Segment SGI331409[] = {{&S12289[6], 4}, {NULL}};
const Segment SGI331410[] = {{&S12289[82], 5}, {NULL}};
const Segment SGI331411[] = {{&S12289[12], 14}, {&S12289[44], 43}, {NULL}};
const Segment SGI331412[] = {{&S12289[12], 14}, {NULL}};
const Segment SGI331413[] = {{&S12290[6], 4}, {NULL}};
const Segment SGI331414[] = {{&S12290[82], 5}, {NULL}};
const Segment SGI331415[] = {{&S12290[12], 14}, {&S12290[45], 42}, {NULL}};
const Segment SGI331416[] = {{&S12290[12], 14}, {NULL}};
const Segment SGI331417[] = {{&S12291[6], 4}, {NULL}};
const Segment SGI331418[] = {{&S12291[81], 5}, {NULL}};
const Segment SGI331419[] = {{&S12291[12], 14}, {&S12291[44], 42}, {NULL}};
const Segment SGI331420[] = {{&S12291[12], 14}, {NULL}};
const Segment SGI331421[] = {{&S12292[6], 4}, {NULL}};
const Segment SGI331422[] = {{&S12292[81], 5}, {NULL}};
const Segment SGI331423[] = {{&S12292[12], 14}, {&S12292[44], 42}, {NULL}};
const Segment SGI331424[] = {{&S12292[12], 14}, {NULL}};
const Segment SGI331425[] = {{&S12293[6], 4}, {NULL}};
const Segment SGI331426[] = {{&S12293[82], 5}, {NULL}};
const Segment SGI331427[] = {{&S12293[12], 14}, {&S12293[45], 42}, {NULL}};
const Segment SGI331428[] = {{&S12293[12], 14}, {NULL}};
const Segment SGI331429[] = {{&S12294[6], 4}, {NULL}};
const Segment SGI331430[] = {{&S12294[81], 5}, {NULL}};
const Segment SGI331431[] = {{&S12294[12], 14}, {&S12294[44], 42}, {NULL}};
const Segment SGI331432[] = {{&S12294[12], 14}, {NULL}};
const Segment SGI331433[] = {{&S12295[6], 4}, {NULL}};
const Segment SGI331434[] = {{&S12295[74], 5}, {NULL}};
const Segment SGI331435[] = {{&S12295[12], 14}, {&S12295[37], 42}, {NULL}};
const Segment SGI331436[] = {{&S12295[12], 14}, {NULL}};
const Segment SGI331437[] = {{&S12296[6], 4}, {NULL}};
const Segment SGI331438[] = {{&S12296[74], 5}, {NULL}};
const Segment SGI331439[] = {{&S12296[12], 14}, {&S12296[37], 42}, {NULL}};
const Segment SGI331440[] = {{&S12296[12], 14}, {NULL}};
const Segment SGI331441[] = {{&S12297[6], 4}, {NULL}};
const Segment SGI331442[] = {{&S12297[74], 5}, {NULL}};
const Segment SGI331443[] = {{&S12297[12], 14}, {&S12297[37], 42}, {NULL}};
const Segment SGI331444[] = {{&S12297[12], 14}, {NULL}};
const Segment SGI331445[] = {{&S12298[6], 4}, {NULL}};
const Segment SGI331446[] = {{&S12298[81], 5}, {NULL}};
const Segment SGI331447[] = {{&S12298[12], 14}, {&S12298[44], 42}, {NULL}};
const Segment SGI331448[] = {{&S12298[12], 14}, {NULL}};
const Segment SGI331449[] = {{&S12299[6], 4}, {NULL}};
const Segment SGI331450[] = {{&S12299[81], 5}, {NULL}};
const Segment SGI331451[] = {{&S12299[12], 14}, {&S12299[44], 42}, {NULL}};
const Segment SGI331452[] = {{&S12299[12], 14}, {NULL}};
const Segment SGI331453[] = {{&S12300[6], 4}, {NULL}};
const Segment SGI331454[] = {{&S12300[81], 5}, {NULL}};
const Segment SGI331455[] = {{&S12300[12], 14}, {&S12300[44], 42}, {NULL}};
const Segment SGI331456[] = {{&S12300[12], 14}, {NULL}};
const Segment SGI331457[] = {{&S12301[6], 4}, {NULL}};
const Segment SGI331458[] = {{&S12301[81], 5}, {NULL}};
const Segment SGI331459[] = {{&S12301[12], 14}, {&S12301[44], 42}, {NULL}};
const Segment SGI331460[] = {{&S12301[12], 14}, {NULL}};
const Segment SGI331461[] = {{&S12463[6], 4}, {NULL}};
const Segment SGI331462[] = {{&S12463[74], 5}, {NULL}};
const Segment SGI331463[] = {{&S12463[12], 14}, {&S12463[37], 42}, {NULL}};
const Segment SGI331464[] = {{&S12463[12], 14}, {NULL}};
const Segment SGI331465[] = {{&S12464[6], 4}, {NULL}};
const Segment SGI331466[] = {{&S12464[81], 5}, {NULL}};
const Segment SGI331467[] = {{&S12464[12], 14}, {&S12464[44], 42}, {NULL}};
const Segment SGI331468[] = {{&S12464[12], 14}, {NULL}};
const Segment SGI331469[] = {{&S12465[6], 4}, {NULL}};
const Segment SGI331470[] = {{&S12465[81], 5}, {NULL}};
const Segment SGI331471[] = {{&S12465[12], 14}, {&S12465[44], 42}, {NULL}};
const Segment SGI331472[] = {{&S12465[12], 14}, {NULL}};
const Segment SGI331473[] = {{&S12466[6], 4}, {NULL}};
const Segment SGI331474[] = {{&S12466[74], 5}, {NULL}};
const Segment SGI331475[] = {{&S12466[12], 14}, {&S12466[37], 42}, {NULL}};
const Segment SGI331476[] = {{&S12466[12], 14}, {NULL}};
const Segment SGI331477[] = {{&S12467[6], 4}, {NULL}};
const Segment SGI331478[] = {{&S12467[81], 5}, {NULL}};
const Segment SGI331479[] = {{&S12467[12], 14}, {&S12467[44], 42}, {NULL}};
const Segment SGI331480[] = {{&S12467[12], 14}, {NULL}};
const Segment SGI331481[] = {{&S12468[6], 4}, {NULL}};
const Segment SGI331482[] = {{&S12468[81], 5}, {NULL}};
const Segment SGI331483[] = {{&S12468[12], 14}, {&S12468[44], 42}, {NULL}};
const Segment SGI331484[] = {{&S12468[12], 14}, {NULL}};
const Segment SGI331485[] = {{&S12469[6], 4}, {NULL}};
const Segment SGI331486[] = {{&S12469[74], 5}, {NULL}};
const Segment SGI331487[] = {{&S12469[12], 14}, {&S12469[37], 42}, {NULL}};
const Segment SGI331488[] = {{&S12469[12], 14}, {NULL}};
const Segment SGI331489[] = {{&S12470[6], 4}, {NULL}};
const Segment SGI331490[] = {{&S12470[81], 5}, {NULL}};
const Segment SGI331491[] = {{&S12470[12], 14}, {&S12470[44], 42}, {NULL}};
const Segment SGI331492[] = {{&S12470[12], 14}, {NULL}};
const Segment SGI331493[] = {{&S12471[6], 4}, {NULL}};
const Segment SGI331494[] = {{&S12471[59], 5}, {NULL}};
const Segment SGI331495[] = {{&S12471[12], 14}, {&S12471[70], 4}, {NULL}};
const Segment SGI331496[] = {{&S12471[12], 14}, {NULL}};
const Segment SGI331497[] = {{&S12472[6], 4}, {NULL}};
const Segment SGI331498[] = {{&S12472[74], 5}, {NULL}};
const Segment SGI331499[] = {{&S12472[12], 14}, {&S12472[37], 42}, {NULL}};
const Segment SGI331500[] = {{&S12472[12], 14}, {NULL}};
const Segment SGI331501[] = {{&S12473[6], 4}, {NULL}};
const Segment SGI331502[] = {{&S12473[81], 5}, {NULL}};
const Segment SGI331503[] = {{&S12473[12], 14}, {&S12473[44], 42}, {NULL}};
const Segment SGI331504[] = {{&S12473[12], 14}, {NULL}};
const Segment SGI331505[] = {{&S12474[6], 4}, {NULL}};
const Segment SGI331506[] = {{&S12474[52], 5}, {NULL}};
const Segment SGI331507[] = {{&S12474[12], 55}, {NULL}};
const Segment SGI331508[] = {{&S12474[12], 14}, {NULL}};
const Segment SGI331509[] = {{&S12475[6], 4}, {NULL}};
const Segment SGI331510[] = {{&S37083[5], 4}, {NULL}};
const Segment SGI331511[] = {{&S12475[81], 5}, {NULL}};
const Segment SGI331512[] = {{&S12475[12], 14}, {&S12475[44], 42}, {NULL}};
const Segment SGI331513[] = {{&S12475[12], 14}, {NULL}};
const Segment SGI331514[] = {{&S37083[19], 3}, {NULL}};
const Segment SGI331515[] = {{&S37084[5], 4}, {NULL}};
const Segment SGI331516[] = {{&S12476[6], 4}, {NULL}};
const Segment SGI331517[] = {{&S12476[81], 5}, {NULL}};
const Segment SGI331518[] = {{&S12476[12], 14}, {&S12476[44], 42}, {NULL}};
const Segment SGI331519[] = {{&S12476[12], 14}, {NULL}};
const Segment SGI331520[] = {{&S12477[6], 4}, {NULL}};
const Segment SGI331521[] = {{&S12477[75], 5}, {NULL}};
const Segment SGI331522[] = {{&S12477[12], 14}, {&S12477[37], 43}, {NULL}};
const Segment SGI331523[] = {{&S37086[5], 5}, {NULL}};
const Segment SGI331524[] = {{&S12477[12], 14}, {NULL}};
const Segment SGI331525[] = {{&S12478[6], 4}, {NULL}};
const Segment SGI331526[] = {{&S12478[75], 5}, {NULL}};
const Segment SGI331527[] = {{&S12478[12], 14}, {&S12478[38], 42}, {NULL}};
const Segment SGI331528[] = {{&S12478[12], 14}, {NULL}};
const Segment SGI331529[] = {{&S12479[6], 4}, {NULL}};
const Segment SGI331530[] = {{&S12479[75], 5}, {NULL}};
const Segment SGI331531[] = {{&S12479[12], 14}, {&S12479[37], 43}, {NULL}};
const Segment SGI331532[] = {{&S12479[12], 14}, {NULL}};
const Segment SGI331533[] = {{&S12480[6], 4}, {NULL}};
const Segment SGI331534[] = {{&S12480[75], 5}, {NULL}};
const Segment SGI331535[] = {{&S12480[12], 14}, {&S12480[38], 42}, {NULL}};
const Segment SGI331536[] = {{&S12480[12], 14}, {NULL}};
const Segment SGI331537[] = {{&S12481[6], 4}, {NULL}};
const Segment SGI331538[] = {{&S12481[82], 5}, {NULL}};
const Segment SGI331539[] = {{&S12481[12], 14}, {&S12481[45], 42}, {NULL}};
const Segment SGI331540[] = {{&S12481[12], 14}, {NULL}};
const Segment SGI331541[] = {{&S12482[6], 4}, {NULL}};
const Segment SGI331542[] = {{&S12482[81], 5}, {NULL}};
const Segment SGI331543[] = {{&S12482[12], 14}, {&S12482[44], 42}, {NULL}};
const Segment SGI331544[] = {{&S12482[12], 14}, {NULL}};
const Segment SGI331545[] = {{&S12483[6], 4}, {NULL}};
const Segment SGI331546[] = {{&S12483[81], 5}, {NULL}};
const Segment SGI331547[] = {{&S12483[12], 14}, {&S12483[44], 42}, {NULL}};
const Segment SGI331548[] = {{&S12483[12], 14}, {NULL}};
const Segment SGI331549[] = {{&S12484[6], 4}, {NULL}};
const Segment SGI331550[] = {{&S12484[81], 5}, {NULL}};
const Segment SGI331551[] = {{&S12484[12], 14}, {&S12484[44], 42}, {NULL}};
const Segment SGI331552[] = {{&S12484[12], 14}, {NULL}};
const Segment SGI331553[] = {{&S12485[6], 4}, {NULL}};
const Segment SGI331554[] = {{&S12485[81], 5}, {NULL}};
const Segment SGI331555[] = {{&S12485[12], 14}, {&S12485[44], 42}, {NULL}};
const Segment SGI331556[] = {{&S12485[12], 14}, {NULL}};
const Segment SGI331557[] = {{&S12486[6], 4}, {NULL}};
const Segment SGI331558[] = {{&S12486[81], 5}, {NULL}};
const Segment SGI331559[] = {{&S12486[12], 14}, {&S12486[44], 42}, {NULL}};
const Segment SGI331560[] = {{&S12486[12], 14}, {NULL}};
const Segment SGI331561[] = {{&S12487[6], 4}, {NULL}};
const Segment SGI331562[] = {{&S12487[81], 5}, {NULL}};
const Segment SGI331563[] = {{&S12487[12], 14}, {&S12487[44], 42}, {NULL}};
const Segment SGI331564[] = {{&S12487[12], 14}, {NULL}};
const Segment SGI331565[] = {{&S12488[6], 4}, {NULL}};
const Segment SGI331566[] = {{&S12488[75], 5}, {NULL}};
const Segment SGI331567[] = {{&S12488[12], 14}, {&S12488[37], 43}, {NULL}};
const Segment SGI331568[] = {{&S12488[12], 14}, {NULL}};
const Segment SGI331569[] = {{&S12489[6], 4}, {NULL}};
const Segment SGI331570[] = {{&S12489[81], 5}, {NULL}};
const Segment SGI331571[] = {{&S12489[12], 14}, {&S12489[44], 42}, {NULL}};
const Segment SGI331572[] = {{&S12489[12], 14}, {NULL}};
const Segment SGI331573[] = {{&S12490[6], 4}, {NULL}};
const Segment SGI331574[] = {{&S12490[81], 5}, {NULL}};
const Segment SGI331575[] = {{&S12490[12], 14}, {&S12490[44], 42}, {NULL}};
const Segment SGI331576[] = {{&S12490[12], 14}, {NULL}};
const Segment SGI331577[] = {{&S12491[6], 4}, {NULL}};
const Segment SGI331578[] = {{&S12491[81], 5}, {NULL}};
const Segment SGI331579[] = {{&S12491[12], 14}, {&S12491[44], 42}, {NULL}};
const Segment SGI331580[] = {{&S12491[12], 14}, {NULL}};
const Segment SGI331581[] = {{&S12492[6], 4}, {NULL}};
const Segment SGI331582[] = {{&S12492[81], 5}, {NULL}};
const Segment SGI331583[] = {{&S12492[12], 14}, {&S12492[44], 42}, {NULL}};
const Segment SGI331584[] = {{&S12492[12], 14}, {NULL}};
const Segment SGI331585[] = {{&S12494[6], 4}, {NULL}};
const Segment SGI331586[] = {{&S12494[82], 5}, {NULL}};
const Segment SGI331587[] = {{&S12494[12], 14}, {&S12494[45], 42}, {NULL}};
const Segment SGI331588[] = {{&S12494[12], 14}, {NULL}};
const Segment SGI331589[] = {{&S12495[6], 4}, {NULL}};
const Segment SGI331590[] = {{&S12495[81], 5}, {NULL}};
const Segment SGI331591[] = {{&S12495[12], 14}, {&S12495[44], 42}, {NULL}};
const Segment SGI331592[] = {{&S12495[12], 14}, {NULL}};
const Segment SGI331593[] = {{&S12497[6], 4}, {NULL}};
const Segment SGI331594[] = {{&S12497[81], 5}, {NULL}};
const Segment SGI331595[] = {{&S12497[12], 14}, {&S12497[44], 42}, {NULL}};
const Segment SGI331596[] = {{&S12497[12], 14}, {NULL}};
const Segment SGI331597[] = {{&S12498[6], 4}, {NULL}};
const Segment SGI331598[] = {{&S12498[81], 5}, {NULL}};
const Segment SGI331599[] = {{&S12498[12], 14}, {&S12498[44], 42}, {NULL}};
const Segment SGI331600[] = {{&S12498[12], 14}, {NULL}};
const Segment SGI331601[] = {{&S12499[6], 4}, {NULL}};
const Segment SGI331602[] = {{&S12499[81], 5}, {NULL}};
const Segment SGI331603[] = {{&S12499[12], 14}, {&S12499[44], 42}, {NULL}};
const Segment SGI331604[] = {{&S12499[12], 14}, {NULL}};
const Segment SGI331605[] = {{&S12500[6], 4}, {NULL}};
const Segment SGI331606[] = {{&S12500[81], 5}, {NULL}};
const Segment SGI331607[] = {{&S12500[12], 14}, {&S12500[44], 42}, {NULL}};
const Segment SGI331608[] = {{&S12500[12], 14}, {NULL}};
const Segment SGI331609[] = {{&S12501[6], 4}, {NULL}};
const Segment SGI331610[] = {{&S12501[82], 5}, {NULL}};
const Segment SGI331611[] = {{&S12501[12], 14}, {&S12501[45], 42}, {NULL}};
const Segment SGI331612[] = {{&S12501[12], 14}, {NULL}};
const Segment SGI331613[] = {{&S12502[6], 4}, {NULL}};
const Segment SGI331614[] = {{&S12502[81], 5}, {NULL}};
const Segment SGI331615[] = {{&S12502[12], 14}, {&S12502[44], 42}, {NULL}};
const Segment SGI331616[] = {{&S12502[12], 14}, {NULL}};
const Segment SGI331617[] = {{&S12503[6], 4}, {NULL}};
const Segment SGI331618[] = {{&S12503[75], 5}, {NULL}};
const Segment SGI331619[] = {{&S12503[12], 14}, {&S12503[38], 42}, {NULL}};
const Segment SGI331620[] = {{&S12503[12], 14}, {NULL}};
const Segment SGI331621[] = {{&S12504[6], 4}, {NULL}};
const Segment SGI331622[] = {{&S12504[81], 5}, {NULL}};
const Segment SGI331623[] = {{&S12504[12], 14}, {&S12504[44], 42}, {NULL}};
const Segment SGI331624[] = {{&S12504[12], 14}, {NULL}};
const Segment SGI331625[] = {{&S12505[6], 4}, {NULL}};
const Segment SGI331626[] = {{&S12505[74], 5}, {NULL}};
const Segment SGI331627[] = {{&S12505[12], 14}, {&S12505[37], 42}, {NULL}};
const Segment SGI331628[] = {{&S12505[12], 14}, {NULL}};
const Segment SGI331629[] = {{&S12506[6], 4}, {NULL}};
const Segment SGI331630[] = {{&S12506[81], 5}, {NULL}};
const Segment SGI331631[] = {{&S12506[12], 14}, {&S12506[44], 42}, {NULL}};
const Segment SGI331632[] = {{&S12506[12], 14}, {NULL}};
const Segment SGI331633[] = {{&S12507[6], 4}, {NULL}};
const Segment SGI331634[] = {{&S12507[52], 5}, {NULL}};
const Segment SGI331635[] = {{&S12507[12], 45}, {NULL}};
const Segment SGI331636[] = {{&S12507[12], 14}, {NULL}};
const Segment SGI331637[] = {{&S12508[6], 4}, {NULL}};
const Segment SGI331638[] = {{&S12508[75], 5}, {NULL}};
const Segment SGI331639[] = {{&S12508[12], 14}, {&S12508[37], 43}, {NULL}};
const Segment SGI331640[] = {{&S12508[12], 14}, {NULL}};
const Segment SGI331641[] = {{&S12509[6], 4}, {NULL}};
const Segment SGI331642[] = {{&S12509[59], 5}, {NULL}};
const Segment SGI331643[] = {{&S12509[12], 14}, {NULL}};
const Segment SGI331644[] = {{&S12510[6], 4}, {NULL}};
const Segment SGI331645[] = {{&S12510[81], 5}, {NULL}};
const Segment SGI331646[] = {{&S12510[12], 14}, {&S12510[44], 42}, {NULL}};
const Segment SGI331647[] = {{&S12510[12], 14}, {NULL}};
const Segment SGI331648[] = {{&S12511[6], 4}, {NULL}};
const Segment SGI331649[] = {{&S12511[81], 5}, {NULL}};
const Segment SGI331650[] = {{&S12511[12], 14}, {&S12511[44], 42}, {NULL}};
const Segment SGI331651[] = {{&S12511[12], 14}, {NULL}};
const Segment SGI331652[] = {{&S12512[6], 4}, {NULL}};
const Segment SGI331653[] = {{&S12512[74], 5}, {NULL}};
const Segment SGI331654[] = {{&S12512[12], 14}, {&S12512[37], 42}, {NULL}};
const Segment SGI331655[] = {{&S12512[12], 14}, {NULL}};
const Segment SGI331656[] = {{&S12513[6], 4}, {NULL}};
const Segment SGI331657[] = {{&S12513[81], 5}, {NULL}};
const Segment SGI331658[] = {{&S12513[12], 14}, {&S12513[44], 42}, {NULL}};
const Segment SGI331659[] = {{&S12513[12], 14}, {NULL}};
const Segment SGI331660[] = {{&S12514[6], 4}, {NULL}};
const Segment SGI331661[] = {{&S12514[74], 5}, {NULL}};
const Segment SGI331662[] = {{&S12514[12], 14}, {&S12514[37], 42}, {NULL}};
const Segment SGI331663[] = {{&S12514[12], 14}, {NULL}};
const Segment SGI331664[] = {{&S12515[6], 4}, {NULL}};
const Segment SGI331665[] = {{&S12515[81], 5}, {NULL}};
const Segment SGI331666[] = {{&S12515[12], 14}, {&S12515[44], 42}, {NULL}};
const Segment SGI331667[] = {{&S12515[12], 14}, {NULL}};
const Segment SGI331668[] = {{&S12516[6], 4}, {NULL}};
const Segment SGI331669[] = {{&S12516[75], 5}, {NULL}};
const Segment SGI331670[] = {{&S12516[12], 14}, {&S12516[38], 42}, {NULL}};
const Segment SGI331671[] = {{&S12516[12], 14}, {NULL}};
const Segment SGI331672[] = {{&S12517[6], 4}, {NULL}};
const Segment SGI331673[] = {{&S12517[74], 5}, {NULL}};
const Segment SGI331674[] = {{&S12517[12], 14}, {&S12517[37], 42}, {NULL}};
const Segment SGI331675[] = {{&S12517[12], 14}, {NULL}};
const Segment SGI331676[] = {{&S12518[6], 4}, {NULL}};
const Segment SGI331677[] = {{&S12518[81], 5}, {NULL}};
const Segment SGI331678[] = {{&S12518[12], 14}, {&S12518[44], 42}, {NULL}};
const Segment SGI331679[] = {{&S12518[12], 14}, {NULL}};
const Segment SGI331680[] = {{&S12519[6], 4}, {NULL}};
const Segment SGI331681[] = {{&S12519[81], 5}, {NULL}};
const Segment SGI331682[] = {{&S12519[12], 14}, {&S12519[44], 42}, {NULL}};
const Segment SGI331683[] = {{&S12519[12], 14}, {NULL}};
const Segment SGI331684[] = {{&S12520[6], 4}, {NULL}};
const Segment SGI331685[] = {{&S12520[75], 5}, {NULL}};
const Segment SGI331686[] = {{&S12520[12], 14}, {&S12520[37], 43}, {NULL}};
const Segment SGI331687[] = {{&S12520[12], 14}, {NULL}};
const Segment SGI331688[] = {{&S12521[6], 4}, {NULL}};
const Segment SGI331689[] = {{&S12521[82], 5}, {NULL}};
const Segment SGI331690[] = {{&S12521[12], 14}, {&S12521[45], 42}, {NULL}};
const Segment SGI331691[] = {{&S12521[12], 14}, {NULL}};
const Segment SGI331692[] = {{&S12522[6], 4}, {NULL}};
const Segment SGI331693[] = {{&S12522[82], 5}, {NULL}};
const Segment SGI331694[] = {{&S12522[12], 14}, {&S12522[45], 42}, {NULL}};
const Segment SGI331695[] = {{&S12522[12], 14}, {NULL}};
const Segment SGI331696[] = {{&S12523[6], 4}, {NULL}};
const Segment SGI331697[] = {{&S12523[75], 5}, {NULL}};
const Segment SGI331698[] = {{&S12523[12], 14}, {&S12523[38], 42}, {NULL}};
const Segment SGI331699[] = {{&S12523[12], 14}, {NULL}};
const Segment SGI331700[] = {{&S12524[6], 4}, {NULL}};
const Segment SGI331701[] = {{&S12524[81], 5}, {NULL}};
const Segment SGI331702[] = {{&S12524[12], 14}, {&S12524[44], 42}, {NULL}};
const Segment SGI331703[] = {{&S12524[12], 14}, {NULL}};
const Segment SGI331704[] = {{&S12526[6], 4}, {NULL}};
const Segment SGI331705[] = {{&S12526[52], 5}, {NULL}};
const Segment SGI331706[] = {{&S12526[12], 62}, {NULL}};
const Segment SGI331707[] = {{&S12526[12], 14}, {NULL}};
const Segment SGI331708[] = {{&S12527[6], 4}, {NULL}};
const Segment SGI331709[] = {{&S12527[81], 5}, {NULL}};
const Segment SGI331710[] = {{&S12527[12], 14}, {&S12527[44], 42}, {NULL}};
const Segment SGI331711[] = {{&S12527[12], 14}, {NULL}};
const Segment SGI331712[] = {{&S12528[6], 4}, {NULL}};
const Segment SGI331713[] = {{&S12528[74], 5}, {NULL}};
const Segment SGI331714[] = {{&S12528[12], 14}, {&S12528[37], 42}, {NULL}};
const Segment SGI331715[] = {{&S12528[12], 14}, {NULL}};
const Segment SGI331716[] = {{&S12529[6], 4}, {NULL}};
const Segment SGI331717[] = {{&S12529[74], 5}, {NULL}};
const Segment SGI331718[] = {{&S12529[12], 14}, {&S12529[37], 42}, {NULL}};
const Segment SGI331719[] = {{&S12529[12], 14}, {NULL}};
const Segment SGI331720[] = {{&S12530[6], 4}, {NULL}};
const Segment SGI331721[] = {{&S12530[81], 5}, {NULL}};
const Segment SGI331722[] = {{&S12530[12], 14}, {&S12530[44], 42}, {NULL}};
const Segment SGI331723[] = {{&S12530[12], 14}, {NULL}};
const Segment SGI331724[] = {{&S12531[6], 4}, {NULL}};
const Segment SGI331725[] = {{&S12531[81], 5}, {NULL}};
const Segment SGI331726[] = {{&S12531[12], 14}, {&S12531[44], 42}, {NULL}};
const Segment SGI331727[] = {{&S12531[12], 14}, {NULL}};
const Segment SGI331728[] = {{&S12532[6], 4}, {NULL}};
const Segment SGI331729[] = {{&S12532[74], 5}, {NULL}};
const Segment SGI331730[] = {{&S12532[12], 14}, {&S12532[37], 42}, {NULL}};
const Segment SGI331731[] = {{&S12532[12], 14}, {NULL}};
const Segment SGI331732[] = {{&S12533[6], 4}, {NULL}};
const Segment SGI331733[] = {{&S12533[81], 5}, {NULL}};
const Segment SGI331734[] = {{&S12533[12], 14}, {&S12533[44], 42}, {NULL}};
const Segment SGI331735[] = {{&S12533[12], 14}, {NULL}};
const Segment SGI331736[] = {{&S12534[6], 4}, {NULL}};
const Segment SGI331737[] = {{&S12534[82], 5}, {NULL}};
const Segment SGI331738[] = {{&S12534[12], 14}, {&S12534[45], 42}, {NULL}};
const Segment SGI331739[] = {{&S12534[12], 14}, {NULL}};
const Segment SGI331740[] = {{&S12535[6], 4}, {NULL}};
const Segment SGI331741[] = {{&S12535[59], 5}, {NULL}};
const Segment SGI331742[] = {{&S12535[12], 14}, {&S12535[70], 33}, {NULL}};
const Segment SGI331743[] = {{&S12535[12], 14}, {NULL}};
const Segment SGI331744[] = {{&S12536[6], 4}, {NULL}};
const Segment SGI331745[] = {{&S12536[75], 5}, {NULL}};
const Segment SGI331746[] = {{&S12536[12], 14}, {&S12536[37], 43}, {NULL}};
const Segment SGI331747[] = {{&S12536[12], 14}, {NULL}};
const Segment SGI331748[] = {{&S12537[6], 4}, {NULL}};
const Segment SGI331749[] = {{&S12537[82], 5}, {NULL}};
const Segment SGI331750[] = {{&S12537[12], 14}, {&S12537[44], 43}, {NULL}};
const Segment SGI331751[] = {{&S12537[12], 14}, {NULL}};
const Segment SGI331752[] = {{&S12538[6], 4}, {NULL}};
const Segment SGI331753[] = {{&S12538[82], 5}, {NULL}};
const Segment SGI331754[] = {{&S12538[12], 14}, {&S12538[44], 43}, {NULL}};
const Segment SGI331755[] = {{&S12538[12], 14}, {NULL}};
const Segment SGI331756[] = {{&S12539[6], 4}, {NULL}};
const Segment SGI331757[] = {{&S12539[75], 5}, {NULL}};
const Segment SGI331758[] = {{&S12539[12], 14}, {&S12539[38], 42}, {NULL}};
const Segment SGI331759[] = {{&S12539[12], 14}, {NULL}};
const Segment SGI331760[] = {{&S37163[5], 5}, {NULL}};
const Segment SGI331761[] = {{&S12540[6], 4}, {NULL}};
const Segment SGI331762[] = {{&S12540[81], 5}, {NULL}};
const Segment SGI331763[] = {{&S12540[12], 14}, {&S12540[44], 42}, {NULL}};
const Segment SGI331764[] = {{&S12540[12], 14}, {NULL}};
const Segment SGI331765[] = {{&S12541[6], 4}, {NULL}};
const Segment SGI331766[] = {{&S12541[82], 5}, {NULL}};
const Segment SGI331767[] = {{&S12541[12], 14}, {&S12541[45], 42}, {NULL}};
const Segment SGI331768[] = {{&S12541[12], 14}, {NULL}};
const Segment SGI331769[] = {{&S37166[5], 6}, {NULL}};
const Segment SGI331770[] = {{&S12542[6], 4}, {NULL}};
const Segment SGI331771[] = {{&S12542[74], 5}, {NULL}};
const Segment SGI331772[] = {{&S12542[12], 14}, {&S12542[37], 42}, {NULL}};
const Segment SGI331773[] = {{&S12542[12], 14}, {NULL}};
const Segment SGI331774[] = {{&S12543[6], 4}, {NULL}};
const Segment SGI331775[] = {{&S12543[75], 5}, {NULL}};
const Segment SGI331776[] = {{&S12543[12], 14}, {&S12543[37], 43}, {NULL}};
const Segment SGI331777[] = {{&S12543[12], 14}, {NULL}};
const Segment SGI331778[] = {{&S12544[6], 4}, {NULL}};
const Segment SGI331779[] = {{&S12544[74], 5}, {NULL}};
const Segment SGI331780[] = {{&S12544[12], 14}, {&S12544[37], 42}, {NULL}};
const Segment SGI331781[] = {{&S12544[12], 14}, {NULL}};
const Segment SGI331782[] = {{&S37169[5], 3}, {NULL}};
const Segment SGI331783[] = {{&S12545[6], 4}, {NULL}};
const Segment SGI331784[] = {{&S12545[75], 5}, {NULL}};
const Segment SGI331785[] = {{&S12545[12], 14}, {&S12545[38], 42}, {NULL}};
const Segment SGI331786[] = {{&S12545[12], 14}, {NULL}};
const Segment SGI331787[] = {{&S12546[6], 4}, {NULL}};
const Segment SGI331788[] = {{&S12546[74], 5}, {NULL}};
const Segment SGI331789[] = {{&S12546[12], 14}, {&S12546[37], 42}, {NULL}};
const Segment SGI331790[] = {{&S12546[12], 14}, {NULL}};
const Segment SGI331791[] = {{&S12547[6], 4}, {NULL}};
const Segment SGI331792[] = {{&S12547[74], 5}, {NULL}};
const Segment SGI331793[] = {{&S12547[12], 14}, {&S12547[37], 42}, {NULL}};
const Segment SGI331794[] = {{&S12547[12], 14}, {NULL}};
const Segment SGI331795[] = {{&S12548[6], 4}, {NULL}};
const Segment SGI331796[] = {{&S12548[52], 5}, {NULL}};
const Segment SGI331797[] = {{&S12548[12], 84}, {NULL}};
const Segment SGI331798[] = {{&S12548[12], 14}, {NULL}};
const Segment SGI331799[] = {{&S12549[6], 4}, {NULL}};
const Segment SGI331800[] = {{&S12549[81], 5}, {NULL}};
const Segment SGI331801[] = {{&S12549[12], 14}, {&S12549[44], 42}, {NULL}};
const Segment SGI331802[] = {{&S12549[12], 14}, {NULL}};
const Segment SGI331803[] = {{&S12550[6], 4}, {NULL}};
const Segment SGI331804[] = {{&S12550[74], 5}, {NULL}};
const Segment SGI331805[] = {{&S12550[12], 14}, {&S12550[37], 42}, {NULL}};
const Segment SGI331806[] = {{&S12550[12], 14}, {NULL}};
const Segment SGI331807[] = {{&S12551[6], 4}, {NULL}};
const Segment SGI331808[] = {{&S12551[74], 5}, {NULL}};
const Segment SGI331809[] = {{&S12551[12], 14}, {&S12551[37], 42}, {NULL}};
const Segment SGI331810[] = {{&S12551[12], 14}, {NULL}};
const Segment SGI331811[] = {{&S12552[6], 4}, {NULL}};
const Segment SGI331812[] = {{&S12552[81], 5}, {NULL}};
const Segment SGI331813[] = {{&S12552[12], 14}, {&S12552[44], 42}, {NULL}};
const Segment SGI331814[] = {{&S12552[12], 14}, {NULL}};
const Segment SGI331815[] = {{&S12553[6], 4}, {NULL}};
const Segment SGI331816[] = {{&S12553[81], 5}, {NULL}};
const Segment SGI331817[] = {{&S12553[12], 14}, {&S12553[44], 42}, {NULL}};
const Segment SGI331818[] = {{&S12553[12], 14}, {NULL}};
const Segment SGI331819[] = {{&S12554[6], 4}, {NULL}};
const Segment SGI331820[] = {{&S12554[74], 5}, {NULL}};
const Segment SGI331821[] = {{&S12554[12], 14}, {&S12554[37], 42}, {NULL}};
const Segment SGI331822[] = {{&S12554[12], 14}, {NULL}};
const Segment SGI331823[] = {{&S12555[6], 4}, {NULL}};
const Segment SGI331824[] = {{&S12555[82], 5}, {NULL}};
const Segment SGI331825[] = {{&S12555[12], 14}, {&S12555[45], 42}, {NULL}};
const Segment SGI331826[] = {{&S12555[12], 14}, {NULL}};
const Segment SGI331827[] = {{&S12556[6], 4}, {NULL}};
const Segment SGI331828[] = {{&S12556[74], 5}, {NULL}};
const Segment SGI331829[] = {{&S12556[12], 14}, {&S12556[37], 42}, {NULL}};
const Segment SGI331830[] = {{&S12556[12], 14}, {NULL}};
const Segment SGI331831[] = {{&S12557[6], 4}, {NULL}};
const Segment SGI331832[] = {{&S12557[82], 5}, {NULL}};
const Segment SGI331833[] = {{&S12557[12], 14}, {&S12557[45], 42}, {NULL}};
const Segment SGI331834[] = {{&S12557[12], 14}, {NULL}};
const Segment SGI331835[] = {{&S12558[6], 4}, {NULL}};
const Segment SGI331836[] = {{&S12558[75], 5}, {NULL}};
const Segment SGI331837[] = {{&S12558[12], 14}, {&S12558[37], 43}, {NULL}};
const Segment SGI331838[] = {{&S12558[12], 14}, {NULL}};
const Segment SGI331839[] = {{&S12559[6], 4}, {NULL}};
const Segment SGI331840[] = {{&S12559[74], 5}, {NULL}};
const Segment SGI331841[] = {{&S12559[12], 14}, {&S12559[37], 42}, {NULL}};
const Segment SGI331842[] = {{&S12559[12], 14}, {NULL}};
const Segment SGI331843[] = {{&S12560[6], 4}, {NULL}};
const Segment SGI331844[] = {{&S12560[81], 5}, {NULL}};
const Segment SGI331845[] = {{&S12560[12], 14}, {&S12560[44], 42}, {NULL}};
const Segment SGI331846[] = {{&S12560[12], 14}, {NULL}};
const Segment SGI331847[] = {{&S12561[6], 4}, {NULL}};
const Segment SGI331848[] = {{&S12561[74], 5}, {NULL}};
const Segment SGI331849[] = {{&S12561[12], 14}, {&S12561[37], 42}, {NULL}};
const Segment SGI331850[] = {{&S12561[12], 14}, {NULL}};
const Segment SGI331851[] = {{&S12562[6], 4}, {NULL}};
const Segment SGI331852[] = {{&S12562[75], 5}, {NULL}};
const Segment SGI331853[] = {{&S12562[12], 14}, {&S12562[38], 42}, {NULL}};
const Segment SGI331854[] = {{&S12562[12], 14}, {NULL}};
const Segment SGI331855[] = {{&S12563[6], 4}, {NULL}};
const Segment SGI331856[] = {{&S12563[81], 5}, {NULL}};
const Segment SGI331857[] = {{&S12563[12], 14}, {&S12563[44], 42}, {NULL}};
const Segment SGI331858[] = {{&S12563[12], 14}, {NULL}};
const Segment SGI331859[] = {{&S12564[6], 4}, {NULL}};
const Segment SGI331860[] = {{&S12564[75], 5}, {NULL}};
const Segment SGI331861[] = {{&S12564[12], 14}, {&S12564[37], 43}, {NULL}};
const Segment SGI331862[] = {{&S12564[12], 14}, {NULL}};
const Segment SGI331863[] = {{&S12565[6], 4}, {NULL}};
const Segment SGI331864[] = {{&S12565[82], 5}, {NULL}};
const Segment SGI331865[] = {{&S12565[12], 14}, {&S12565[44], 43}, {NULL}};
const Segment SGI331866[] = {{&S12565[12], 14}, {NULL}};
const Segment SGI331867[] = {{&S12566[6], 4}, {NULL}};
const Segment SGI331868[] = {{&S12566[82], 5}, {NULL}};
const Segment SGI331869[] = {{&S12566[12], 14}, {&S12566[45], 42}, {NULL}};
const Segment SGI331870[] = {{&S12566[12], 14}, {NULL}};
const Segment SGI331871[] = {{&S12567[6], 4}, {NULL}};
const Segment SGI331872[] = {{&S12567[81], 5}, {NULL}};
const Segment SGI331873[] = {{&S12567[12], 14}, {&S12567[44], 42}, {NULL}};
const Segment SGI331874[] = {{&S12567[12], 14}, {NULL}};
const Segment SGI331875[] = {{&S12568[6], 4}, {NULL}};
const Segment SGI331876[] = {{&S12568[75], 5}, {NULL}};
const Segment SGI331877[] = {{&S12568[12], 14}, {&S12568[38], 42}, {NULL}};
const Segment SGI331878[] = {{&S12568[12], 14}, {NULL}};
const Segment SGI331879[] = {{&S12569[6], 4}, {NULL}};
const Segment SGI331880[] = {{&S12569[81], 5}, {NULL}};
const Segment SGI331881[] = {{&S12569[12], 14}, {&S12569[44], 42}, {NULL}};
const Segment SGI331882[] = {{&S12569[12], 14}, {NULL}};
const Segment SGI331883[] = {{&S12570[6], 4}, {NULL}};
const Segment SGI331884[] = {{&S12570[75], 5}, {NULL}};
const Segment SGI331885[] = {{&S12570[12], 14}, {&S12570[37], 43}, {NULL}};
const Segment SGI331886[] = {{&S12570[12], 14}, {NULL}};
const Segment SGI331887[] = {{&S12571[6], 4}, {NULL}};
const Segment SGI331888[] = {{&S12571[75], 5}, {NULL}};
const Segment SGI331889[] = {{&S12571[12], 14}, {&S12571[37], 43}, {NULL}};
const Segment SGI331890[] = {{&S12571[12], 14}, {NULL}};
const Segment SGI331891[] = {{&S12572[6], 4}, {NULL}};
const Segment SGI331892[] = {{&S12572[102], 5}, {NULL}};
const Segment SGI331893[] = {{&S12572[12], 14}, {&S12572[37], 12}, {NULL}};
const Segment SGI331894[] = {{&S12572[12], 14}, {NULL}};
const Segment SGI331895[] = {{&S12573[6], 4}, {NULL}};
const Segment SGI331896[] = {{&S12573[74], 5}, {NULL}};
const Segment SGI331897[] = {{&S12573[12], 14}, {&S12573[37], 42}, {NULL}};
const Segment SGI331898[] = {{&S12573[12], 14}, {NULL}};
const Segment SGI331899[] = {{&S12574[6], 4}, {NULL}};
const Segment SGI331900[] = {{&S12574[75], 5}, {NULL}};
const Segment SGI331901[] = {{&S12574[12], 14}, {&S12574[37], 43}, {NULL}};
const Segment SGI331902[] = {{&S12574[12], 14}, {NULL}};
const Segment SGI331903[] = {{&S12575[6], 4}, {NULL}};
const Segment SGI331904[] = {{&S12575[75], 5}, {NULL}};
const Segment SGI331905[] = {{&S12575[12], 14}, {&S12575[37], 43}, {NULL}};
const Segment SGI331906[] = {{&S12575[12], 14}, {NULL}};
const Segment SGI331907[] = {{&S12576[6], 4}, {NULL}};
const Segment SGI331908[] = {{&S12576[75], 5}, {NULL}};
const Segment SGI331909[] = {{&S12576[12], 14}, {&S12576[37], 43}, {NULL}};
const Segment SGI331910[] = {{&S12576[12], 14}, {NULL}};
const Segment SGI331911[] = {{&S12577[6], 4}, {NULL}};
const Segment SGI331912[] = {{&S12577[75], 5}, {NULL}};
const Segment SGI331913[] = {{&S12577[12], 14}, {&S12577[37], 43}, {NULL}};
const Segment SGI331914[] = {{&S12577[12], 14}, {NULL}};
const Segment SGI331915[] = {{&S12578[6], 4}, {NULL}};
const Segment SGI331916[] = {{&S12578[81], 5}, {NULL}};
const Segment SGI331917[] = {{&S12578[12], 14}, {&S12578[44], 42}, {NULL}};
const Segment SGI331918[] = {{&S12578[12], 14}, {NULL}};
const Segment SGI331919[] = {{&S12579[6], 4}, {NULL}};
const Segment SGI331920[] = {{&S12579[81], 5}, {NULL}};
const Segment SGI331921[] = {{&S12579[12], 14}, {&S12579[44], 42}, {NULL}};
const Segment SGI331922[] = {{&S12579[12], 14}, {NULL}};
const Segment SGI331923[] = {{&S12580[6], 4}, {NULL}};
const Segment SGI331924[] = {{&S12580[81], 5}, {NULL}};
const Segment SGI331925[] = {{&S12580[12], 14}, {&S12580[44], 52}, {NULL}};
const Segment SGI331926[] = {{&S12580[12], 14}, {NULL}};
const Segment SGI331927[] = {{&S12581[6], 4}, {NULL}};
const Segment SGI331928[] = {{&S12581[74], 5}, {NULL}};
const Segment SGI331929[] = {{&S12581[12], 14}, {&S12581[37], 42}, {NULL}};
const Segment SGI331930[] = {{&S12581[12], 14}, {NULL}};
const Segment SGI331931[] = {{&S12582[6], 4}, {NULL}};
const Segment SGI331932[] = {{&S12582[74], 5}, {NULL}};
const Segment SGI331933[] = {{&S12582[12], 14}, {&S12582[37], 42}, {NULL}};
const Segment SGI331934[] = {{&S12582[12], 14}, {NULL}};
const Segment SGI331935[] = {{&S12583[6], 4}, {NULL}};
const Segment SGI331936[] = {{&S12583[108], 5}, {NULL}};
const Segment SGI331937[] = {{&S12583[12], 14}, {&S12583[44], 12}, {NULL}};
const Segment SGI331938[] = {{&S12583[12], 14}, {NULL}};
const Segment SGI331939[] = {{&S12584[6], 4}, {NULL}};
const Segment SGI331940[] = {{&S12584[82], 5}, {NULL}};
const Segment SGI331941[] = {{&S12584[12], 14}, {&S12584[45], 42}, {NULL}};
const Segment SGI331942[] = {{&S12584[12], 14}, {NULL}};
const Segment SGI331943[] = {{&S12585[6], 4}, {NULL}};
const Segment SGI331944[] = {{&S12585[74], 5}, {NULL}};
const Segment SGI331945[] = {{&S12585[12], 14}, {&S12585[37], 42}, {NULL}};
const Segment SGI331946[] = {{&S12585[12], 14}, {NULL}};
const Segment SGI331947[] = {{&S12586[6], 4}, {NULL}};
const Segment SGI331948[] = {{&S12586[74], 5}, {NULL}};
const Segment SGI331949[] = {{&S12586[12], 14}, {&S12586[37], 42}, {NULL}};
const Segment SGI331950[] = {{&S12586[12], 14}, {NULL}};
const Segment SGI331951[] = {{&S12587[6], 4}, {NULL}};
const Segment SGI331952[] = {{&S12587[81], 5}, {NULL}};
const Segment SGI331953[] = {{&S12587[12], 14}, {&S12587[44], 42}, {NULL}};
const Segment SGI331954[] = {{&S12587[12], 14}, {NULL}};
const Segment SGI331955[] = {{&S12588[6], 4}, {NULL}};
const Segment SGI331956[] = {{&S12588[74], 5}, {NULL}};
const Segment SGI331957[] = {{&S12588[12], 14}, {&S12588[37], 42}, {NULL}};
const Segment SGI331958[] = {{&S12588[12], 14}, {NULL}};
const Segment SGI331959[] = {{&S12589[6], 4}, {NULL}};
const Segment SGI331960[] = {{&S12589[74], 5}, {NULL}};
const Segment SGI331961[] = {{&S12589[12], 14}, {&S12589[37], 42}, {NULL}};
const Segment SGI331962[] = {{&S12589[12], 14}, {NULL}};
const Segment SGI331963[] = {{&S12590[6], 4}, {NULL}};
const Segment SGI331964[] = {{&S12590[81], 5}, {NULL}};
const Segment SGI331965[] = {{&S12590[12], 14}, {&S12590[44], 42}, {NULL}};
const Segment SGI331966[] = {{&S12590[12], 14}, {NULL}};
const Segment SGI331967[] = {{&S12591[6], 4}, {NULL}};
const Segment SGI331968[] = {{&S12591[74], 5}, {NULL}};
const Segment SGI331969[] = {{&S12591[12], 14}, {&S12591[37], 42}, {NULL}};
const Segment SGI331970[] = {{&S12591[12], 14}, {NULL}};
const Segment SGI331971[] = {{&S12592[6], 4}, {NULL}};
const Segment SGI331972[] = {{&S12592[74], 5}, {NULL}};
const Segment SGI331973[] = {{&S12592[12], 14}, {&S12592[37], 42}, {NULL}};
const Segment SGI331974[] = {{&S12592[12], 14}, {NULL}};
const Segment SGI331975[] = {{&S12593[6], 4}, {NULL}};
const Segment SGI331976[] = {{&S12593[74], 5}, {NULL}};
const Segment SGI331977[] = {{&S12593[12], 14}, {&S12593[37], 42}, {NULL}};
const Segment SGI331978[] = {{&S12593[12], 14}, {NULL}};
const Segment SGI331979[] = {{&S12594[6], 4}, {NULL}};
const Segment SGI331980[] = {{&S12594[74], 5}, {NULL}};
const Segment SGI331981[] = {{&S12594[12], 14}, {&S12594[37], 42}, {NULL}};
const Segment SGI331982[] = {{&S12594[12], 14}, {NULL}};
const Segment SGI331983[] = {{&S12595[6], 4}, {NULL}};
const Segment SGI331984[] = {{&S12595[74], 5}, {NULL}};
const Segment SGI331985[] = {{&S12595[12], 14}, {&S12595[37], 42}, {NULL}};
const Segment SGI331986[] = {{&S12595[12], 14}, {NULL}};
const Segment SGI331987[] = {{&S12596[6], 4}, {NULL}};
const Segment SGI331988[] = {{&S12596[74], 5}, {NULL}};
const Segment SGI331989[] = {{&S12596[12], 14}, {&S12596[37], 42}, {NULL}};
const Segment SGI331990[] = {{&S12596[12], 14}, {NULL}};
const Segment SGI331991[] = {{&S12597[6], 4}, {NULL}};
const Segment SGI331992[] = {{&S12597[74], 5}, {NULL}};
const Segment SGI331993[] = {{&S12597[12], 14}, {&S12597[37], 42}, {NULL}};
const Segment SGI331994[] = {{&S12597[12], 14}, {NULL}};
const Segment SGI331995[] = {{&S12598[6], 4}, {NULL}};
const Segment SGI331996[] = {{&S12598[74], 5}, {NULL}};
const Segment SGI331997[] = {{&S12598[12], 14}, {&S12598[37], 42}, {NULL}};
const Segment SGI331998[] = {{&S12598[12], 14}, {NULL}};
const Segment SGI331999[] = {{&S12599[6], 4}, {NULL}};
const Segment SGI332000[] = {{&S12599[74], 5}, {NULL}};
const Segment SGI332001[] = {{&S12599[12], 14}, {&S12599[37], 42}, {NULL}};
const Segment SGI332002[] = {{&S12599[12], 14}, {NULL}};
const Segment SGI332003[] = {{&S12600[6], 4}, {NULL}};
const Segment SGI332004[] = {{&S12600[59], 5}, {NULL}};
const Segment SGI332005[] = {{&S12600[12], 14}, {&S12600[70], 14}, {NULL}};
const Segment SGI332006[] = {{&S12600[12], 14}, {NULL}};
const Segment SGI332007[] = {{&S12601[6], 4}, {NULL}};
const Segment SGI332008[] = {{&S12601[81], 5}, {NULL}};
const Segment SGI332009[] = {{&S12601[12], 14}, {&S12601[44], 42}, {NULL}};
const Segment SGI332010[] = {{&S12601[12], 14}, {NULL}};
const Segment SGI332011[] = {{&S12602[6], 4}, {NULL}};
const Segment SGI332012[] = {{&S12602[81], 5}, {NULL}};
const Segment SGI332013[] = {{&S12602[12], 14}, {&S12602[44], 42}, {NULL}};
const Segment SGI332014[] = {{&S12602[12], 14}, {NULL}};
const Segment SGI332015[] = {{&S12603[6], 4}, {NULL}};
const Segment SGI332016[] = {{&S12603[81], 5}, {NULL}};
const Segment SGI332017[] = {{&S12603[12], 14}, {&S12603[44], 42}, {NULL}};
const Segment SGI332018[] = {{&S12603[12], 14}, {NULL}};
const Segment SGI332019[] = {{&S12604[6], 4}, {NULL}};
const Segment SGI332020[] = {{&S12604[52], 5}, {NULL}};
const Segment SGI332021[] = {{&S12604[12], 65}, {NULL}};
const Segment SGI332022[] = {{&S12604[12], 14}, {NULL}};
const Segment SGI332023[] = {{&S12605[6], 4}, {NULL}};
const Segment SGI332024[] = {{&S12605[74], 5}, {NULL}};
const Segment SGI332025[] = {{&S12605[12], 14}, {&S12605[37], 42}, {NULL}};
const Segment SGI332026[] = {{&S12605[12], 14}, {NULL}};
const Segment SGI332027[] = {{&S12606[6], 4}, {NULL}};
const Segment SGI332028[] = {{&S12606[74], 5}, {NULL}};
const Segment SGI332029[] = {{&S12606[12], 14}, {&S12606[37], 42}, {NULL}};
const Segment SGI332030[] = {{&S12606[12], 14}, {NULL}};
const Segment SGI332031[] = {{&S12607[6], 4}, {NULL}};
const Segment SGI332032[] = {{&S12607[74], 5}, {NULL}};
const Segment SGI332033[] = {{&S12607[12], 14}, {&S12607[37], 42}, {NULL}};
const Segment SGI332034[] = {{&S12607[12], 14}, {NULL}};
const Segment SGI332035[] = {{&S12608[6], 4}, {NULL}};
const Segment SGI332036[] = {{&S12608[74], 5}, {NULL}};
const Segment SGI332037[] = {{&S12608[12], 14}, {&S12608[37], 42}, {NULL}};
const Segment SGI332038[] = {{&S12608[12], 14}, {NULL}};
const Segment SGI332039[] = {{&S12609[6], 4}, {NULL}};
const Segment SGI332040[] = {{&S12609[74], 5}, {NULL}};
const Segment SGI332041[] = {{&S12609[12], 14}, {&S12609[37], 42}, {NULL}};
const Segment SGI332042[] = {{&S12609[12], 14}, {NULL}};
const Segment SGI332043[] = {{&S37498[5], 5}, {NULL}};
const Segment SGI332044[] = {{&S37499[5], 5}, {&S37499[68], 5}, {NULL}};
const Segment SGI332045[] = {{&S37500[5], 7}, {NULL}};
const Segment SGI332046[] = {{&S12808[6], 4}, {NULL}};
const Segment SGI332047[] = {{&S12808[59], 5}, {NULL}};
const Segment SGI332048[] = {{&S12808[12], 14}, {NULL}};
const Segment SGI332049[] = {{&S12810[6], 4}, {NULL}};
const Segment SGI332050[] = {{&S12810[81], 5}, {NULL}};
const Segment SGI332051[] = {{&S12810[12], 14}, {&S12810[40], 11}, {NULL}};
const Segment SGI332052[] = {{&S12810[12], 14}, {NULL}};
const Segment SGI332053[] = {{&S12811[6], 4}, {NULL}};
const Segment SGI332054[] = {{&S12811[82], 5}, {NULL}};
const Segment SGI332055[] = {{&S12811[12], 14}, {&S12811[40], 12}, {NULL}};
const Segment SGI332056[] = {{&S12811[12], 14}, {NULL}};
const Segment SGI332057[] = {{&S12812[6], 4}, {NULL}};
const Segment SGI332058[] = {{&S12812[82], 5}, {NULL}};
const Segment SGI332059[] = {{&S12812[12], 14}, {&S12812[41], 11}, {NULL}};
const Segment SGI332060[] = {{&S12812[12], 14}, {NULL}};
const Segment SGI332061[] = {{&S12814[6], 4}, {NULL}};
const Segment SGI332062[] = {{&S12814[59], 5}, {NULL}};
const Segment SGI332063[] = {{&S12814[12], 14}, {NULL}};
const Segment SGI332064[] = {{&S12815[6], 4}, {NULL}};
const Segment SGI332065[] = {{&S12815[82], 5}, {NULL}};
const Segment SGI332066[] = {{&S12815[12], 14}, {&S12815[41], 11}, {NULL}};
const Segment SGI332067[] = {{&S12815[12], 14}, {NULL}};
const Segment SGI332068[] = {{&S12816[6], 4}, {NULL}};
const Segment SGI332069[] = {{&S12816[81], 5}, {NULL}};
const Segment SGI332070[] = {{&S12816[12], 14}, {&S12816[40], 11}, {NULL}};
const Segment SGI332071[] = {{&S12816[12], 14}, {NULL}};
const Segment SGI332072[] = {{&S12817[6], 4}, {NULL}};
const Segment SGI332073[] = {{&S12817[82], 5}, {NULL}};
const Segment SGI332074[] = {{&S12817[12], 14}, {&S12817[41], 11}, {NULL}};
const Segment SGI332075[] = {{&S12817[12], 14}, {NULL}};
const Segment SGI332076[] = {{&S12818[6], 4}, {NULL}};
const Segment SGI332077[] = {{&S12818[82], 5}, {NULL}};
const Segment SGI332078[] = {{&S12818[12], 14}, {&S12818[40], 12}, {NULL}};
const Segment SGI332079[] = {{&S12818[12], 14}, {NULL}};
const Segment SGI332080[] = {{&S12819[6], 4}, {NULL}};
const Segment SGI332081[] = {{&S12819[81], 5}, {NULL}};
const Segment SGI332082[] = {{&S12819[12], 14}, {&S12819[40], 11}, {NULL}};
const Segment SGI332083[] = {{&S12819[12], 14}, {NULL}};
const Segment SGI332084[] = {{&S12820[6], 4}, {NULL}};
const Segment SGI332085[] = {{&S12820[82], 5}, {NULL}};
const Segment SGI332086[] = {{&S12820[12], 14}, {&S12820[40], 12}, {NULL}};
const Segment SGI332087[] = {{&S12820[12], 14}, {NULL}};
const Segment SGI332088[] = {{&S12821[6], 4}, {NULL}};
const Segment SGI332089[] = {{&S12821[82], 5}, {NULL}};
const Segment SGI332090[] = {{&S12821[12], 14}, {&S12821[41], 11}, {NULL}};
const Segment SGI332091[] = {{&S12821[12], 14}, {NULL}};
const Segment SGI332092[] = {{&S12822[6], 4}, {NULL}};
const Segment SGI332093[] = {{&S12822[82], 5}, {NULL}};
const Segment SGI332094[] = {{&S12822[12], 14}, {&S12822[41], 11}, {NULL}};
const Segment SGI332095[] = {{&S12822[12], 14}, {NULL}};
const Segment SGI332096[] = {{&S12823[6], 4}, {NULL}};
const Segment SGI332097[] = {{&S12823[81], 5}, {NULL}};
const Segment SGI332098[] = {{&S12823[12], 14}, {&S12823[40], 11}, {NULL}};
const Segment SGI332099[] = {{&S12823[12], 14}, {NULL}};
const Segment SGI332100[] = {{&S12824[6], 4}, {NULL}};
const Segment SGI332101[] = {{&S12824[82], 5}, {NULL}};
const Segment SGI332102[] = {{&S12824[12], 14}, {&S12824[40], 12}, {NULL}};
const Segment SGI332103[] = {{&S12824[12], 14}, {NULL}};
const Segment SGI332104[] = {{&S12825[6], 4}, {NULL}};
const Segment SGI332105[] = {{&S12825[81], 5}, {NULL}};
const Segment SGI332106[] = {{&S12825[12], 14}, {&S12825[40], 11}, {NULL}};
const Segment SGI332107[] = {{&S12825[12], 14}, {NULL}};
const Segment SGI332108[] = {{&S12826[6], 4}, {NULL}};
const Segment SGI332109[] = {{&S12826[59], 5}, {NULL}};
const Segment SGI332110[] = {{&S12826[12], 14}, {&S12826[70], 31}, {NULL}};
const Segment SGI332111[] = {{&S12826[12], 14}, {NULL}};
const Segment SGI332112[] = {{&S12827[6], 4}, {NULL}};
const Segment SGI332113[] = {{&S12827[82], 5}, {NULL}};
const Segment SGI332114[] = {{&S12827[12], 14}, {&S12827[40], 12}, {NULL}};
const Segment SGI332115[] = {{&S12827[12], 14}, {NULL}};
const Segment SGI332116[] = {{&S12828[6], 4}, {NULL}};
const Segment SGI332117[] = {{&S12828[81], 5}, {NULL}};
const Segment SGI332118[] = {{&S12828[12], 14}, {&S12828[40], 11}, {NULL}};
const Segment SGI332119[] = {{&S12828[12], 14}, {NULL}};
const Segment SGI332120[] = {{&S12829[6], 4}, {NULL}};
const Segment SGI332121[] = {{&S12829[81], 5}, {NULL}};
const Segment SGI332122[] = {{&S12829[12], 14}, {&S12829[40], 11}, {NULL}};
const Segment SGI332123[] = {{&S12829[12], 14}, {NULL}};
const Segment SGI332124[] = {{&S12830[6], 4}, {NULL}};
const Segment SGI332125[] = {{&S12830[81], 5}, {NULL}};
const Segment SGI332126[] = {{&S12830[12], 14}, {&S12830[40], 11}, {NULL}};
const Segment SGI332127[] = {{&S12830[12], 14}, {NULL}};
const Segment SGI332128[] = {{&S12831[6], 4}, {NULL}};
const Segment SGI332129[] = {{&S12831[82], 5}, {NULL}};
const Segment SGI332130[] = {{&S12831[12], 14}, {&S12831[41], 11}, {NULL}};
const Segment SGI332131[] = {{&S12831[12], 14}, {NULL}};
const Segment SGI332132[] = {{&S12832[6], 4}, {NULL}};
const Segment SGI332133[] = {{&S12832[81], 5}, {NULL}};
const Segment SGI332134[] = {{&S12832[12], 14}, {&S12832[40], 11}, {NULL}};
const Segment SGI332135[] = {{&S12832[12], 14}, {NULL}};
const Segment SGI332136[] = {{&S12833[6], 4}, {NULL}};
const Segment SGI332137[] = {{&S12833[81], 5}, {NULL}};
const Segment SGI332138[] = {{&S12833[12], 14}, {&S12833[40], 11}, {NULL}};
const Segment SGI332139[] = {{&S12833[12], 14}, {NULL}};
const Segment SGI332140[] = {{&S12834[6], 4}, {NULL}};
const Segment SGI332141[] = {{&S12834[81], 5}, {NULL}};
const Segment SGI332142[] = {{&S12834[12], 14}, {&S12834[40], 11}, {NULL}};
const Segment SGI332143[] = {{&S12834[12], 14}, {NULL}};
const Segment SGI332144[] = {{&S12835[6], 4}, {NULL}};
const Segment SGI332145[] = {{&S12835[82], 5}, {NULL}};
const Segment SGI332146[] = {{&S12835[12], 14}, {&S12835[40], 12}, {NULL}};
const Segment SGI332147[] = {{&S12835[12], 14}, {NULL}};
const Segment SGI332148[] = {{&S12836[6], 4}, {NULL}};
const Segment SGI332149[] = {{&S12836[81], 5}, {NULL}};
const Segment SGI332150[] = {{&S12836[12], 14}, {&S12836[40], 11}, {NULL}};
const Segment SGI332151[] = {{&S12836[12], 14}, {NULL}};
const Segment SGI332152[] = {{&S12837[6], 4}, {NULL}};
const Segment SGI332153[] = {{&S12837[81], 5}, {NULL}};
const Segment SGI332154[] = {{&S12837[12], 14}, {&S12837[40], 11}, {NULL}};
const Segment SGI332155[] = {{&S12837[12], 14}, {NULL}};
const Segment SGI332156[] = {{&S12838[6], 4}, {NULL}};
const Segment SGI332157[] = {{&S12838[81], 5}, {NULL}};
const Segment SGI332158[] = {{&S12838[12], 14}, {&S12838[40], 11}, {NULL}};
const Segment SGI332159[] = {{&S12838[12], 14}, {NULL}};
const Segment SGI332160[] = {{&S12839[6], 4}, {NULL}};
const Segment SGI332161[] = {{&S12839[82], 5}, {NULL}};
const Segment SGI332162[] = {{&S12839[12], 14}, {&S12839[40], 12}, {NULL}};
const Segment SGI332163[] = {{&S12839[12], 14}, {NULL}};
const Segment SGI332164[] = {{&S12840[6], 4}, {NULL}};
const Segment SGI332165[] = {{&S12840[82], 5}, {NULL}};
const Segment SGI332166[] = {{&S12840[12], 14}, {&S12840[40], 12}, {NULL}};
const Segment SGI332167[] = {{&S12840[12], 14}, {NULL}};
const Segment SGI332168[] = {{&S12841[6], 4}, {NULL}};
const Segment SGI332169[] = {{&S12841[82], 5}, {NULL}};
const Segment SGI332170[] = {{&S12841[12], 14}, {&S12841[41], 11}, {NULL}};
const Segment SGI332171[] = {{&S12841[12], 14}, {NULL}};
const Segment SGI332172[] = {{&S12842[6], 4}, {NULL}};
const Segment SGI332173[] = {{&S12842[82], 5}, {NULL}};
const Segment SGI332174[] = {{&S12842[12], 14}, {&S12842[41], 11}, {NULL}};
const Segment SGI332175[] = {{&S12842[12], 14}, {NULL}};
const Segment SGI332176[] = {{&S12843[6], 4}, {NULL}};
const Segment SGI332177[] = {{&S12843[81], 5}, {NULL}};
const Segment SGI332178[] = {{&S12843[12], 14}, {&S12843[40], 11}, {NULL}};
const Segment SGI332179[] = {{&S12843[12], 14}, {NULL}};
const Segment SGI332180[] = {{&S12844[6], 4}, {NULL}};
const Segment SGI332181[] = {{&S12844[81], 5}, {NULL}};
const Segment SGI332182[] = {{&S12844[12], 14}, {&S12844[40], 11}, {NULL}};
const Segment SGI332183[] = {{&S12844[12], 14}, {NULL}};
const Segment SGI332184[] = {{&S12845[6], 4}, {NULL}};
const Segment SGI332185[] = {{&S12845[82], 5}, {NULL}};
const Segment SGI332186[] = {{&S12845[12], 14}, {&S12845[40], 12}, {NULL}};
const Segment SGI332187[] = {{&S12845[12], 14}, {NULL}};
const Segment SGI332188[] = {{&S12846[6], 4}, {NULL}};
const Segment SGI332189[] = {{&S12846[82], 5}, {NULL}};
const Segment SGI332190[] = {{&S12846[12], 14}, {&S12846[41], 11}, {NULL}};
const Segment SGI332191[] = {{&S12846[12], 14}, {NULL}};
const Segment SGI332192[] = {{&S12847[6], 4}, {NULL}};
const Segment SGI332193[] = {{&S12847[82], 5}, {NULL}};
const Segment SGI332194[] = {{&S12847[12], 14}, {&S12847[41], 11}, {NULL}};
const Segment SGI332195[] = {{&S12847[12], 14}, {NULL}};
const Segment SGI332196[] = {{&S12848[6], 4}, {NULL}};
const Segment SGI332197[] = {{&S12848[82], 5}, {NULL}};
const Segment SGI332198[] = {{&S12848[12], 14}, {&S12848[41], 11}, {NULL}};
const Segment SGI332199[] = {{&S12848[12], 14}, {NULL}};
const Segment SGI332200[] = {{&S12849[6], 4}, {NULL}};
const Segment SGI332201[] = {{&S12849[82], 5}, {NULL}};
const Segment SGI332202[] = {{&S12849[12], 14}, {&S12849[40], 12}, {NULL}};
const Segment SGI332203[] = {{&S12849[12], 14}, {NULL}};
const Segment SGI332204[] = {{&S12850[6], 4}, {NULL}};
const Segment SGI332205[] = {{&S12850[82], 5}, {NULL}};
const Segment SGI332206[] = {{&S12850[12], 14}, {&S12850[40], 12}, {NULL}};
const Segment SGI332207[] = {{&S12850[12], 14}, {NULL}};
const Segment SGI332208[] = {{&S12851[6], 4}, {NULL}};
const Segment SGI332209[] = {{&S12851[82], 5}, {NULL}};
const Segment SGI332210[] = {{&S12851[12], 14}, {&S12851[41], 11}, {NULL}};
const Segment SGI332211[] = {{&S12851[12], 14}, {NULL}};
const Segment SGI332212[] = {{&S12852[6], 4}, {NULL}};
const Segment SGI332213[] = {{&S12852[82], 5}, {NULL}};
const Segment SGI332214[] = {{&S12852[12], 14}, {&S12852[41], 11}, {NULL}};
const Segment SGI332215[] = {{&S12852[12], 14}, {NULL}};
const Segment SGI332216[] = {{&S12853[6], 4}, {NULL}};
const Segment SGI332217[] = {{&S12853[81], 5}, {NULL}};
const Segment SGI332218[] = {{&S12853[12], 14}, {&S12853[40], 11}, {NULL}};
const Segment SGI332219[] = {{&S12853[12], 14}, {NULL}};
const Segment SGI332220[] = {{&S12854[6], 4}, {NULL}};
const Segment SGI332221[] = {{&S12854[82], 5}, {NULL}};
const Segment SGI332222[] = {{&S12854[12], 14}, {&S12854[40], 12}, {NULL}};
const Segment SGI332223[] = {{&S12854[12], 14}, {NULL}};
const Segment SGI332224[] = {{&S12855[6], 4}, {NULL}};
const Segment SGI332225[] = {{&S12855[82], 5}, {NULL}};
const Segment SGI332226[] = {{&S12855[12], 14}, {&S12855[41], 11}, {NULL}};
const Segment SGI332227[] = {{&S12855[12], 14}, {NULL}};
const Segment SGI332228[] = {{&S12856[6], 4}, {NULL}};
const Segment SGI332229[] = {{&S12856[81], 5}, {NULL}};
const Segment SGI332230[] = {{&S12856[12], 14}, {&S12856[40], 11}, {NULL}};
const Segment SGI332231[] = {{&S12856[12], 14}, {NULL}};
const Segment SGI332232[] = {{&S12857[6], 4}, {NULL}};
const Segment SGI332233[] = {{&S12857[81], 5}, {NULL}};
const Segment SGI332234[] = {{&S12857[12], 14}, {&S12857[40], 11}, {NULL}};
const Segment SGI332235[] = {{&S12857[12], 14}, {NULL}};
const Segment SGI332236[] = {{&S12858[6], 4}, {NULL}};
const Segment SGI332237[] = {{&S12858[82], 5}, {NULL}};
const Segment SGI332238[] = {{&S12858[12], 14}, {&S12858[41], 11}, {NULL}};
const Segment SGI332239[] = {{&S12858[12], 14}, {NULL}};
const Segment SGI332240[] = {{&S12859[6], 4}, {NULL}};
const Segment SGI332241[] = {{&S12859[81], 5}, {NULL}};
const Segment SGI332242[] = {{&S12859[12], 14}, {&S12859[40], 11}, {NULL}};
const Segment SGI332243[] = {{&S12859[12], 14}, {NULL}};
const Segment SGI332244[] = {{&S12860[6], 4}, {NULL}};
const Segment SGI332245[] = {{&S12860[81], 5}, {NULL}};
const Segment SGI332246[] = {{&S12860[12], 14}, {&S12860[40], 11}, {NULL}};
const Segment SGI332247[] = {{&S12860[12], 14}, {NULL}};
const Segment SGI332248[] = {{&S12861[6], 4}, {NULL}};
const Segment SGI332249[] = {{&S12861[82], 5}, {NULL}};
const Segment SGI332250[] = {{&S12861[12], 14}, {&S12861[41], 11}, {NULL}};
const Segment SGI332251[] = {{&S12861[12], 14}, {NULL}};
const Segment SGI332252[] = {{&S12862[6], 4}, {NULL}};
const Segment SGI332253[] = {{&S12862[82], 5}, {NULL}};
const Segment SGI332254[] = {{&S12862[12], 14}, {&S12862[41], 11}, {NULL}};
const Segment SGI332255[] = {{&S12862[12], 14}, {NULL}};
const Segment SGI332256[] = {{&S12863[6], 4}, {NULL}};
const Segment SGI332257[] = {{&S12863[81], 5}, {NULL}};
const Segment SGI332258[] = {{&S12863[12], 14}, {&S12863[40], 11}, {NULL}};
const Segment SGI332259[] = {{&S12863[12], 14}, {NULL}};
const Segment SGI332260[] = {{&S12864[6], 4}, {NULL}};
const Segment SGI332261[] = {{&S12864[81], 5}, {NULL}};
const Segment SGI332262[] = {{&S12864[12], 14}, {&S12864[40], 11}, {NULL}};
const Segment SGI332263[] = {{&S12864[12], 14}, {NULL}};
const Segment SGI332264[] = {{&S12865[6], 4}, {NULL}};
const Segment SGI332265[] = {{&S12865[81], 5}, {NULL}};
const Segment SGI332266[] = {{&S12865[12], 14}, {&S12865[40], 11}, {NULL}};
const Segment SGI332267[] = {{&S12865[12], 14}, {NULL}};
const Segment SGI332268[] = {{&S12866[6], 4}, {NULL}};
const Segment SGI332269[] = {{&S12866[82], 5}, {NULL}};
const Segment SGI332270[] = {{&S12866[12], 14}, {&S12866[41], 11}, {NULL}};
const Segment SGI332271[] = {{&S12866[12], 14}, {NULL}};
const Segment SGI332272[] = {{&S12867[6], 4}, {NULL}};
const Segment SGI332273[] = {{&S12867[82], 5}, {NULL}};
const Segment SGI332274[] = {{&S12867[12], 14}, {&S12867[40], 12}, {NULL}};
const Segment SGI332275[] = {{&S12867[12], 14}, {NULL}};
const Segment SGI332276[] = {{&S12868[6], 4}, {NULL}};
const Segment SGI332277[] = {{&S12868[81], 5}, {NULL}};
const Segment SGI332278[] = {{&S12868[12], 14}, {&S12868[40], 11}, {NULL}};
const Segment SGI332279[] = {{&S12868[12], 14}, {NULL}};
const Segment SGI332280[] = {{&S12869[6], 4}, {NULL}};
const Segment SGI332281[] = {{&S12869[82], 5}, {NULL}};
const Segment SGI332282[] = {{&S12869[12], 14}, {&S12869[40], 12}, {NULL}};
const Segment SGI332283[] = {{&S12869[12], 14}, {NULL}};
const Segment SGI332284[] = {{&S12870[6], 4}, {NULL}};
const Segment SGI332285[] = {{&S12870[81], 5}, {NULL}};
const Segment SGI332286[] = {{&S12870[12], 14}, {&S12870[40], 11}, {NULL}};
const Segment SGI332287[] = {{&S12870[12], 14}, {NULL}};
const Segment SGI332288[] = {{&S12871[6], 4}, {NULL}};
const Segment SGI332289[] = {{&S12871[81], 5}, {NULL}};
const Segment SGI332290[] = {{&S12871[12], 14}, {&S12871[40], 11}, {NULL}};
const Segment SGI332291[] = {{&S12871[12], 14}, {NULL}};
const Segment SGI332292[] = {{&S12872[6], 4}, {NULL}};
const Segment SGI332293[] = {{&S12872[82], 5}, {NULL}};
const Segment SGI332294[] = {{&S12872[12], 14}, {&S12872[40], 12}, {NULL}};
const Segment SGI332295[] = {{&S12872[12], 14}, {NULL}};
const Segment SGI332296[] = {{&S12873[6], 4}, {NULL}};
const Segment SGI332297[] = {{&S12873[82], 5}, {NULL}};
const Segment SGI332298[] = {{&S12873[12], 14}, {&S12873[41], 11}, {&S12873[93], 31}, {NULL}};
const Segment SGI332299[] = {{&S12873[12], 14}, {NULL}};
const Segment SGI332300[] = {{&S12874[6], 4}, {NULL}};
const Segment SGI332301[] = {{&S12874[59], 5}, {NULL}};
const Segment SGI332302[] = {{&S12874[12], 14}, {NULL}};
const Segment SGI332303[] = {{&S12875[6], 4}, {NULL}};
const Segment SGI332304[] = {{&S12875[82], 5}, {NULL}};
const Segment SGI332305[] = {{&S12875[12], 14}, {&S12875[41], 11}, {NULL}};
const Segment SGI332306[] = {{&S12875[12], 14}, {NULL}};
const Segment SGI332307[] = {{&S12876[6], 4}, {NULL}};
const Segment SGI332308[] = {{&S12876[82], 5}, {NULL}};
const Segment SGI332309[] = {{&S12876[12], 14}, {&S12876[40], 12}, {NULL}};
const Segment SGI332310[] = {{&S12876[12], 14}, {NULL}};
const Segment SGI332311[] = {{&S12877[6], 4}, {NULL}};
const Segment SGI332312[] = {{&S12877[82], 5}, {NULL}};
const Segment SGI332313[] = {{&S12877[12], 14}, {&S12877[41], 11}, {NULL}};
const Segment SGI332314[] = {{&S12877[12], 14}, {NULL}};
const Segment SGI332315[] = {{&S12878[6], 4}, {NULL}};
const Segment SGI332316[] = {{&S12878[81], 5}, {NULL}};
const Segment SGI332317[] = {{&S12878[12], 14}, {&S12878[40], 11}, {NULL}};
const Segment SGI332318[] = {{&S12878[12], 14}, {NULL}};
const Segment SGI332319[] = {{&S12879[6], 4}, {NULL}};
const Segment SGI332320[] = {{&S12879[81], 5}, {NULL}};
const Segment SGI332321[] = {{&S12879[12], 14}, {&S12879[40], 11}, {NULL}};
const Segment SGI332322[] = {{&S12879[12], 14}, {NULL}};
const Segment SGI332323[] = {{&S12880[6], 4}, {NULL}};
const Segment SGI332324[] = {{&S12880[81], 5}, {NULL}};
const Segment SGI332325[] = {{&S12880[12], 14}, {&S12880[40], 11}, {NULL}};
const Segment SGI332326[] = {{&S12880[12], 14}, {NULL}};
const Segment SGI332327[] = {{&S12881[6], 4}, {NULL}};
const Segment SGI332328[] = {{&S12881[81], 5}, {NULL}};
const Segment SGI332329[] = {{&S12881[12], 14}, {&S12881[40], 11}, {NULL}};
const Segment SGI332330[] = {{&S12881[12], 14}, {NULL}};
const Segment SGI332331[] = {{&S12882[6], 4}, {NULL}};
const Segment SGI332332[] = {{&S12882[81], 5}, {NULL}};
const Segment SGI332333[] = {{&S12882[12], 14}, {&S12882[40], 11}, {NULL}};
const Segment SGI332334[] = {{&S12882[12], 14}, {NULL}};
const Segment SGI332335[] = {{&S12883[6], 4}, {NULL}};
const Segment SGI332336[] = {{&S12883[82], 5}, {NULL}};
const Segment SGI332337[] = {{&S12883[12], 14}, {&S12883[40], 12}, {NULL}};
const Segment SGI332338[] = {{&S12883[12], 14}, {NULL}};
const Segment SGI332339[] = {{&S12884[6], 4}, {NULL}};
const Segment SGI332340[] = {{&S12884[82], 5}, {NULL}};
const Segment SGI332341[] = {{&S12884[12], 14}, {&S12884[40], 12}, {NULL}};
const Segment SGI332342[] = {{&S12884[12], 14}, {NULL}};
const Segment SGI332343[] = {{&S12885[6], 4}, {NULL}};
const Segment SGI332344[] = {{&S12885[81], 5}, {NULL}};
const Segment SGI332345[] = {{&S12885[12], 14}, {&S12885[40], 11}, {NULL}};
const Segment SGI332346[] = {{&S12885[12], 14}, {NULL}};
const Segment SGI332347[] = {{&S12886[6], 4}, {NULL}};
const Segment SGI332348[] = {{&S12886[82], 5}, {NULL}};
const Segment SGI332349[] = {{&S12886[12], 14}, {&S12886[41], 11}, {NULL}};
const Segment SGI332350[] = {{&S12886[12], 14}, {NULL}};
const Segment SGI332351[] = {{&S12887[6], 4}, {NULL}};
const Segment SGI332352[] = {{&S12887[81], 5}, {NULL}};
const Segment SGI332353[] = {{&S12887[12], 14}, {&S12887[40], 11}, {NULL}};
const Segment SGI332354[] = {{&S12887[12], 14}, {NULL}};
const Segment SGI332355[] = {{&S12888[6], 4}, {NULL}};
const Segment SGI332356[] = {{&S12888[82], 5}, {NULL}};
const Segment SGI332357[] = {{&S12888[12], 14}, {&S12888[40], 12}, {NULL}};
const Segment SGI332358[] = {{&S12888[12], 14}, {NULL}};
const Segment SGI332359[] = {{&S12889[6], 4}, {NULL}};
const Segment SGI332360[] = {{&S12889[81], 5}, {NULL}};
const Segment SGI332361[] = {{&S12889[12], 14}, {&S12889[40], 11}, {NULL}};
const Segment SGI332362[] = {{&S12889[12], 14}, {NULL}};
const Segment SGI332363[] = {{&S12890[6], 4}, {NULL}};
const Segment SGI332364[] = {{&S12890[82], 5}, {NULL}};
const Segment SGI332365[] = {{&S12890[12], 14}, {&S12890[41], 11}, {NULL}};
const Segment SGI332366[] = {{&S12890[12], 14}, {NULL}};
const Segment SGI332367[] = {{&S12891[6], 4}, {NULL}};
const Segment SGI332368[] = {{&S12891[82], 5}, {NULL}};
const Segment SGI332369[] = {{&S12891[12], 14}, {&S12891[40], 12}, {NULL}};
const Segment SGI332370[] = {{&S12891[12], 14}, {NULL}};
const Segment SGI332371[] = {{&S12892[6], 4}, {NULL}};
const Segment SGI332372[] = {{&S12892[82], 5}, {NULL}};
const Segment SGI332373[] = {{&S12892[12], 14}, {&S12892[40], 12}, {NULL}};
const Segment SGI332374[] = {{&S12892[12], 14}, {NULL}};
const Segment SGI332375[] = {{&S12893[6], 4}, {NULL}};
const Segment SGI332376[] = {{&S12893[59], 5}, {NULL}};
const Segment SGI332377[] = {{&S12893[12], 14}, {NULL}};
const Segment SGI332378[] = {{&S12894[6], 4}, {NULL}};
const Segment SGI332379[] = {{&S12894[81], 5}, {NULL}};
const Segment SGI332380[] = {{&S12894[12], 14}, {&S12894[40], 11}, {NULL}};
const Segment SGI332381[] = {{&S12894[12], 14}, {NULL}};
const Segment SGI332382[] = {{&S12895[6], 4}, {NULL}};
const Segment SGI332383[] = {{&S12895[81], 5}, {NULL}};
const Segment SGI332384[] = {{&S12895[12], 14}, {&S12895[40], 11}, {NULL}};
const Segment SGI332385[] = {{&S12895[12], 14}, {NULL}};
const Segment SGI332386[] = {{&S12896[6], 4}, {NULL}};
const Segment SGI332387[] = {{&S12896[82], 5}, {NULL}};
const Segment SGI332388[] = {{&S12896[12], 14}, {&S12896[40], 12}, {NULL}};
const Segment SGI332389[] = {{&S12896[12], 14}, {NULL}};
const Segment SGI332390[] = {{&S12897[6], 4}, {NULL}};
const Segment SGI332391[] = {{&S12897[82], 5}, {NULL}};
const Segment SGI332392[] = {{&S12897[12], 14}, {&S12897[41], 11}, {NULL}};
const Segment SGI332393[] = {{&S12897[12], 14}, {NULL}};
const Segment SGI332394[] = {{&S12898[6], 4}, {NULL}};
const Segment SGI332395[] = {{&S12898[82], 5}, {NULL}};
const Segment SGI332396[] = {{&S12898[12], 14}, {&S12898[40], 12}, {NULL}};
const Segment SGI332397[] = {{&S12898[12], 14}, {NULL}};
const Segment SGI332398[] = {{&S12899[6], 4}, {NULL}};
const Segment SGI332399[] = {{&S12899[81], 5}, {NULL}};
const Segment SGI332400[] = {{&S12899[12], 14}, {&S12899[40], 11}, {NULL}};
const Segment SGI332401[] = {{&S12899[12], 14}, {NULL}};
const Segment SGI332402[] = {{&S12900[6], 4}, {NULL}};
const Segment SGI332403[] = {{&S12900[59], 5}, {NULL}};
const Segment SGI332404[] = {{&S12900[12], 14}, {&S12900[70], 31}, {NULL}};
const Segment SGI332405[] = {{&S12900[12], 14}, {NULL}};
const Segment SGI332406[] = {{&S12901[6], 4}, {NULL}};
const Segment SGI332407[] = {{&S12901[82], 5}, {NULL}};
const Segment SGI332408[] = {{&S12901[12], 14}, {&S12901[41], 11}, {NULL}};
const Segment SGI332409[] = {{&S12901[12], 14}, {NULL}};
const Segment SGI332410[] = {{&S12902[6], 4}, {NULL}};
const Segment SGI332411[] = {{&S12902[81], 5}, {NULL}};
const Segment SGI332412[] = {{&S12902[12], 14}, {&S12902[40], 11}, {NULL}};
const Segment SGI332413[] = {{&S12902[12], 14}, {NULL}};
const Segment SGI332414[] = {{&S12903[6], 4}, {NULL}};
const Segment SGI332415[] = {{&S12903[81], 5}, {NULL}};
const Segment SGI332416[] = {{&S12903[12], 14}, {&S12903[40], 11}, {NULL}};
const Segment SGI332417[] = {{&S12903[12], 14}, {NULL}};
const Segment SGI332418[] = {{&S12904[6], 4}, {NULL}};
const Segment SGI332419[] = {{&S12904[81], 5}, {NULL}};
const Segment SGI332420[] = {{&S12904[12], 14}, {&S12904[40], 11}, {NULL}};
const Segment SGI332421[] = {{&S12904[12], 14}, {NULL}};
const Segment SGI332422[] = {{&S12905[6], 4}, {NULL}};
const Segment SGI332423[] = {{&S12905[82], 5}, {NULL}};
const Segment SGI332424[] = {{&S12905[12], 14}, {&S12905[41], 11}, {NULL}};
const Segment SGI332425[] = {{&S12905[12], 14}, {NULL}};
const Segment SGI332426[] = {{&S12906[6], 4}, {NULL}};
const Segment SGI332427[] = {{&S12906[82], 5}, {NULL}};
const Segment SGI332428[] = {{&S12906[12], 14}, {&S12906[40], 12}, {NULL}};
const Segment SGI332429[] = {{&S12906[12], 14}, {NULL}};
const Segment SGI332430[] = {{&S12907[6], 4}, {NULL}};
const Segment SGI332431[] = {{&S12907[81], 5}, {NULL}};
const Segment SGI332432[] = {{&S12907[12], 14}, {&S12907[40], 11}, {NULL}};
const Segment SGI332433[] = {{&S12907[12], 14}, {NULL}};
const Segment SGI332434[] = {{&S12908[6], 4}, {NULL}};
const Segment SGI332435[] = {{&S12908[81], 5}, {NULL}};
const Segment SGI332436[] = {{&S12908[12], 14}, {&S12908[40], 11}, {NULL}};
const Segment SGI332437[] = {{&S12908[12], 14}, {NULL}};
const Segment SGI332438[] = {{&S12909[6], 4}, {NULL}};
const Segment SGI332439[] = {{&S12909[82], 5}, {NULL}};
const Segment SGI332440[] = {{&S12909[12], 14}, {&S12909[41], 11}, {NULL}};
const Segment SGI332441[] = {{&S12909[12], 14}, {NULL}};
const Segment SGI332442[] = {{&S12910[6], 4}, {NULL}};
const Segment SGI332443[] = {{&S12910[81], 5}, {NULL}};
const Segment SGI332444[] = {{&S12910[12], 14}, {&S12910[40], 11}, {NULL}};
const Segment SGI332445[] = {{&S12910[12], 14}, {NULL}};
const Segment SGI332446[] = {{&S12911[6], 4}, {NULL}};
const Segment SGI332447[] = {{&S12911[82], 5}, {NULL}};
const Segment SGI332448[] = {{&S12911[12], 14}, {&S12911[41], 11}, {NULL}};
const Segment SGI332449[] = {{&S12911[12], 14}, {NULL}};
const Segment SGI332450[] = {{&S12912[6], 4}, {NULL}};
const Segment SGI332451[] = {{&S12912[81], 5}, {NULL}};
const Segment SGI332452[] = {{&S12912[12], 14}, {&S12912[40], 11}, {NULL}};
const Segment SGI332453[] = {{&S12912[12], 14}, {NULL}};
const Segment SGI332454[] = {{&S12913[6], 4}, {NULL}};
const Segment SGI332455[] = {{&S12913[82], 5}, {NULL}};
const Segment SGI332456[] = {{&S12913[12], 14}, {&S12913[41], 11}, {NULL}};
const Segment SGI332457[] = {{&S12913[12], 14}, {NULL}};
const Segment SGI332458[] = {{&S12914[6], 4}, {NULL}};
const Segment SGI332459[] = {{&S12914[82], 5}, {NULL}};
const Segment SGI332460[] = {{&S12914[12], 14}, {&S12914[41], 11}, {NULL}};
const Segment SGI332461[] = {{&S12914[12], 14}, {NULL}};
const Segment SGI332462[] = {{&S12915[6], 4}, {NULL}};
const Segment SGI332463[] = {{&S12915[81], 5}, {NULL}};
const Segment SGI332464[] = {{&S12915[12], 14}, {&S12915[40], 11}, {NULL}};
const Segment SGI332465[] = {{&S12915[12], 14}, {NULL}};
const Segment SGI332466[] = {{&S12916[6], 4}, {NULL}};
const Segment SGI332467[] = {{&S12916[81], 5}, {NULL}};
const Segment SGI332468[] = {{&S12916[12], 14}, {&S12916[40], 11}, {NULL}};
const Segment SGI332469[] = {{&S12916[12], 14}, {NULL}};
const Segment SGI332470[] = {{&S12917[6], 4}, {NULL}};
const Segment SGI332471[] = {{&S12917[81], 5}, {NULL}};
const Segment SGI332472[] = {{&S12917[12], 14}, {&S12917[40], 11}, {NULL}};
const Segment SGI332473[] = {{&S12917[12], 14}, {NULL}};
const Segment SGI332474[] = {{&S12918[6], 4}, {NULL}};
const Segment SGI332475[] = {{&S12918[81], 5}, {NULL}};
const Segment SGI332476[] = {{&S12918[12], 14}, {&S12918[40], 11}, {NULL}};
const Segment SGI332477[] = {{&S12918[12], 14}, {NULL}};
const Segment SGI332478[] = {{&S12919[6], 4}, {NULL}};
const Segment SGI332479[] = {{&S12919[82], 5}, {NULL}};
const Segment SGI332480[] = {{&S12919[12], 14}, {&S12919[40], 12}, {NULL}};
const Segment SGI332481[] = {{&S12919[12], 14}, {NULL}};
const Segment SGI332482[] = {{&S12920[6], 4}, {NULL}};
const Segment SGI332483[] = {{&S12920[81], 5}, {NULL}};
const Segment SGI332484[] = {{&S12920[12], 14}, {&S12920[40], 11}, {NULL}};
const Segment SGI332485[] = {{&S12920[12], 14}, {NULL}};
const Segment SGI332486[] = {{&S12921[6], 4}, {NULL}};
const Segment SGI332487[] = {{&S12921[81], 5}, {NULL}};
const Segment SGI332488[] = {{&S12921[12], 14}, {&S12921[40], 11}, {NULL}};
const Segment SGI332489[] = {{&S12921[12], 14}, {NULL}};
const Segment SGI332490[] = {{&S12922[6], 4}, {NULL}};
const Segment SGI332491[] = {{&S12922[82], 5}, {NULL}};
const Segment SGI332492[] = {{&S12922[12], 14}, {&S12922[40], 12}, {NULL}};
const Segment SGI332493[] = {{&S12922[12], 14}, {NULL}};
const Segment SGI332494[] = {{&S12923[6], 4}, {NULL}};
const Segment SGI332495[] = {{&S12923[82], 5}, {NULL}};
const Segment SGI332496[] = {{&S12923[12], 14}, {&S12923[41], 11}, {NULL}};
const Segment SGI332497[] = {{&S12923[12], 14}, {NULL}};
const Segment SGI332498[] = {{&S12924[6], 4}, {NULL}};
const Segment SGI332499[] = {{&S12924[82], 5}, {NULL}};
const Segment SGI332500[] = {{&S12924[12], 14}, {&S12924[41], 11}, {NULL}};
const Segment SGI332501[] = {{&S12924[12], 14}, {NULL}};
const Segment SGI332502[] = {{&S12925[6], 4}, {NULL}};
const Segment SGI332503[] = {{&S12925[81], 5}, {NULL}};
const Segment SGI332504[] = {{&S12925[12], 14}, {&S12925[40], 11}, {NULL}};
const Segment SGI332505[] = {{&S12925[12], 14}, {NULL}};
const Segment SGI332506[] = {{&S12926[6], 4}, {NULL}};
const Segment SGI332507[] = {{&S12926[81], 5}, {NULL}};
const Segment SGI332508[] = {{&S12926[12], 14}, {&S12926[40], 11}, {NULL}};
const Segment SGI332509[] = {{&S12926[12], 14}, {NULL}};
const Segment SGI332510[] = {{&S12927[6], 4}, {NULL}};
const Segment SGI332511[] = {{&S12927[81], 5}, {NULL}};
const Segment SGI332512[] = {{&S12927[12], 14}, {&S12927[40], 11}, {NULL}};
const Segment SGI332513[] = {{&S12927[12], 14}, {NULL}};
const Segment SGI332514[] = {{&S12928[6], 4}, {NULL}};
const Segment SGI332515[] = {{&S12928[82], 5}, {NULL}};
const Segment SGI332516[] = {{&S12928[12], 14}, {&S12928[40], 12}, {NULL}};
const Segment SGI332517[] = {{&S12928[12], 14}, {NULL}};
const Segment SGI332518[] = {{&S12929[6], 4}, {NULL}};
const Segment SGI332519[] = {{&S12929[82], 5}, {NULL}};
const Segment SGI332520[] = {{&S12929[12], 14}, {&S12929[40], 12}, {NULL}};
const Segment SGI332521[] = {{&S12929[12], 14}, {NULL}};
const Segment SGI332522[] = {{&S12930[6], 4}, {NULL}};
const Segment SGI332523[] = {{&S12930[81], 5}, {NULL}};
const Segment SGI332524[] = {{&S12930[12], 14}, {&S12930[40], 11}, {NULL}};
const Segment SGI332525[] = {{&S12930[12], 14}, {NULL}};
const Segment SGI332526[] = {{&S12931[6], 4}, {NULL}};
const Segment SGI332527[] = {{&S12931[82], 5}, {NULL}};
const Segment SGI332528[] = {{&S12931[12], 14}, {&S12931[41], 11}, {NULL}};
const Segment SGI332529[] = {{&S12931[12], 14}, {NULL}};
const Segment SGI332530[] = {{&S12932[6], 4}, {NULL}};
const Segment SGI332531[] = {{&S12932[82], 5}, {NULL}};
const Segment SGI332532[] = {{&S12932[12], 14}, {&S12932[41], 11}, {NULL}};
const Segment SGI332533[] = {{&S12932[12], 14}, {NULL}};
const Segment SGI332534[] = {{&S12933[6], 4}, {NULL}};
const Segment SGI332535[] = {{&S12933[81], 5}, {NULL}};
const Segment SGI332536[] = {{&S12933[12], 14}, {&S12933[40], 11}, {NULL}};
const Segment SGI332537[] = {{&S12933[12], 14}, {NULL}};
const Segment SGI332538[] = {{&S12934[6], 4}, {NULL}};
const Segment SGI332539[] = {{&S12934[81], 5}, {NULL}};
const Segment SGI332540[] = {{&S12934[12], 14}, {&S12934[40], 11}, {NULL}};
const Segment SGI332541[] = {{&S12934[12], 14}, {NULL}};
const Segment SGI332542[] = {{&S12935[6], 4}, {NULL}};
const Segment SGI332543[] = {{&S12935[82], 5}, {NULL}};
const Segment SGI332544[] = {{&S12935[12], 14}, {&S12935[41], 11}, {NULL}};
const Segment SGI332545[] = {{&S12935[12], 14}, {NULL}};
const Segment SGI332546[] = {{&S12936[6], 4}, {NULL}};
const Segment SGI332547[] = {{&S12936[82], 5}, {NULL}};
const Segment SGI332548[] = {{&S12936[12], 14}, {&S12936[40], 12}, {NULL}};
const Segment SGI332549[] = {{&S12936[12], 14}, {NULL}};
const Segment SGI332550[] = {{&S12937[6], 4}, {NULL}};
const Segment SGI332551[] = {{&S12937[82], 5}, {NULL}};
const Segment SGI332552[] = {{&S12937[12], 14}, {&S12937[40], 12}, {NULL}};
const Segment SGI332553[] = {{&S12937[12], 14}, {NULL}};
const Segment SGI332554[] = {{&S12938[6], 4}, {NULL}};
const Segment SGI332555[] = {{&S12938[82], 5}, {NULL}};
const Segment SGI332556[] = {{&S12938[12], 14}, {&S12938[41], 11}, {NULL}};
const Segment SGI332557[] = {{&S12938[12], 14}, {NULL}};
const Segment SGI332558[] = {{&S12939[6], 4}, {NULL}};
const Segment SGI332559[] = {{&S12939[81], 5}, {NULL}};
const Segment SGI332560[] = {{&S12939[12], 14}, {&S12939[40], 11}, {NULL}};
const Segment SGI332561[] = {{&S12939[12], 14}, {NULL}};
const Segment SGI332562[] = {{&S12940[6], 4}, {NULL}};
const Segment SGI332563[] = {{&S12940[81], 5}, {NULL}};
const Segment SGI332564[] = {{&S12940[12], 14}, {&S12940[40], 11}, {NULL}};
const Segment SGI332565[] = {{&S12940[12], 14}, {NULL}};
const Segment SGI332566[] = {{&S12941[6], 4}, {NULL}};
const Segment SGI332567[] = {{&S12941[81], 5}, {NULL}};
const Segment SGI332568[] = {{&S12941[12], 14}, {&S12941[40], 11}, {NULL}};
const Segment SGI332569[] = {{&S12941[12], 14}, {NULL}};
const Segment SGI332570[] = {{&S12942[6], 4}, {NULL}};
const Segment SGI332571[] = {{&S12942[82], 5}, {NULL}};
const Segment SGI332572[] = {{&S12942[12], 14}, {&S12942[41], 11}, {NULL}};
const Segment SGI332573[] = {{&S12942[12], 14}, {NULL}};
const Segment SGI332574[] = {{&S12943[6], 4}, {NULL}};
const Segment SGI332575[] = {{&S12943[82], 5}, {NULL}};
const Segment SGI332576[] = {{&S12943[12], 14}, {&S12943[41], 11}, {NULL}};
const Segment SGI332577[] = {{&S12943[12], 14}, {NULL}};
const Segment SGI332578[] = {{&S12944[6], 4}, {NULL}};
const Segment SGI332579[] = {{&S12944[82], 5}, {NULL}};
const Segment SGI332580[] = {{&S12944[12], 14}, {&S12944[40], 12}, {NULL}};
const Segment SGI332581[] = {{&S12944[12], 14}, {NULL}};
const Segment SGI332582[] = {{&S12945[6], 4}, {NULL}};
const Segment SGI332583[] = {{&S12945[82], 5}, {NULL}};
const Segment SGI332584[] = {{&S12945[12], 14}, {&S12945[41], 11}, {NULL}};
const Segment SGI332585[] = {{&S12945[12], 14}, {NULL}};
const Segment SGI332586[] = {{&S12946[6], 4}, {NULL}};
const Segment SGI332587[] = {{&S12946[59], 5}, {NULL}};
const Segment SGI332588[] = {{&S12946[12], 14}, {&S12946[84], 33}, {NULL}};
const Segment SGI332589[] = {{&S12946[12], 14}, {NULL}};
const Segment SGI332590[] = {{&S12947[6], 4}, {NULL}};
const Segment SGI332591[] = {{&S12947[59], 5}, {NULL}};
const Segment SGI332592[] = {{&S12947[12], 14}, {&S12947[70], 4}, {NULL}};
const Segment SGI332593[] = {{&S12947[12], 14}, {NULL}};
const Segment SGI332594[] = {{&S12948[6], 4}, {NULL}};
const Segment SGI332595[] = {{&S12948[82], 5}, {NULL}};
const Segment SGI332596[] = {{&S12948[12], 14}, {&S12948[40], 12}, {NULL}};
const Segment SGI332597[] = {{&S12948[12], 14}, {NULL}};
const Segment SGI332598[] = {{&S12949[6], 4}, {NULL}};
const Segment SGI332599[] = {{&S12949[97], 5}, {NULL}};
const Segment SGI332600[] = {{&S12949[12], 14}, {NULL}};
const Segment SGI332601[] = {{&S12949[12], 14}, {&S12949[53], 14}, {NULL}};
const Segment SGI332602[] = {{&S12950[6], 4}, {NULL}};
const Segment SGI332603[] = {{&S12950[82], 5}, {NULL}};
const Segment SGI332604[] = {{&S12950[12], 14}, {&S12950[41], 11}, {&S12950[93], 31}, {NULL}};
const Segment SGI332605[] = {{&S12950[12], 14}, {NULL}};
const Segment SGI332606[] = {{&S12951[6], 4}, {NULL}};
const Segment SGI332607[] = {{&S12951[82], 5}, {NULL}};
const Segment SGI332608[] = {{&S12951[12], 14}, {&S12951[41], 11}, {NULL}};
const Segment SGI332609[] = {{&S12951[12], 14}, {NULL}};
const Segment SGI332610[] = {{&S12952[6], 4}, {NULL}};
const Segment SGI332611[] = {{&S12952[82], 5}, {NULL}};
const Segment SGI332612[] = {{&S12952[12], 14}, {&S12952[40], 12}, {NULL}};
const Segment SGI332613[] = {{&S12952[12], 14}, {NULL}};
const Segment SGI332614[] = {{&S12953[6], 4}, {NULL}};
const Segment SGI332615[] = {{&S12953[59], 5}, {NULL}};
const Segment SGI332616[] = {{&S12953[12], 14}, {&S12953[70], 4}, {NULL}};
const Segment SGI332617[] = {{&S12953[12], 14}, {NULL}};
const Segment SGI332618[] = {{&S12954[6], 4}, {NULL}};
const Segment SGI332619[] = {{&S12954[81], 5}, {NULL}};
const Segment SGI332620[] = {{&S12954[12], 14}, {&S12954[40], 11}, {NULL}};
const Segment SGI332621[] = {{&S12954[12], 14}, {NULL}};
const Segment SGI332622[] = {{&S12955[6], 4}, {NULL}};
const Segment SGI332623[] = {{&S12955[81], 5}, {NULL}};
const Segment SGI332624[] = {{&S12955[12], 14}, {&S12955[40], 11}, {NULL}};
const Segment SGI332625[] = {{&S12955[12], 14}, {NULL}};
const Segment SGI332626[] = {{&S12956[6], 4}, {NULL}};
const Segment SGI332627[] = {{&S12956[81], 5}, {NULL}};
const Segment SGI332628[] = {{&S12956[12], 14}, {&S12956[40], 11}, {NULL}};
const Segment SGI332629[] = {{&S12956[12], 14}, {NULL}};
const Segment SGI332630[] = {{&S12957[6], 4}, {NULL}};
const Segment SGI332631[] = {{&S12957[81], 5}, {NULL}};
const Segment SGI332632[] = {{&S12957[12], 14}, {&S12957[40], 11}, {NULL}};
const Segment SGI332633[] = {{&S12957[12], 14}, {NULL}};
const Segment SGI332634[] = {{&S12958[6], 4}, {NULL}};
const Segment SGI332635[] = {{&S12958[81], 5}, {NULL}};
const Segment SGI332636[] = {{&S12958[12], 14}, {&S12958[40], 11}, {NULL}};
const Segment SGI332637[] = {{&S12958[12], 14}, {NULL}};
const Segment SGI332638[] = {{&S12959[6], 4}, {NULL}};
const Segment SGI332639[] = {{&S12959[81], 5}, {NULL}};
const Segment SGI332640[] = {{&S12959[12], 14}, {&S12959[40], 11}, {NULL}};
const Segment SGI332641[] = {{&S12959[12], 14}, {NULL}};
const Segment SGI332642[] = {{&S12960[6], 4}, {NULL}};
const Segment SGI332643[] = {{&S12960[81], 5}, {NULL}};
const Segment SGI332644[] = {{&S12960[12], 14}, {&S12960[40], 11}, {NULL}};
const Segment SGI332645[] = {{&S12960[12], 14}, {NULL}};
const Segment SGI332646[] = {{&S12961[6], 4}, {NULL}};
const Segment SGI332647[] = {{&S12961[81], 5}, {NULL}};
const Segment SGI332648[] = {{&S12961[12], 14}, {&S12961[40], 11}, {NULL}};
const Segment SGI332649[] = {{&S12961[12], 14}, {NULL}};
const Segment SGI332650[] = {{&S12962[6], 4}, {NULL}};
const Segment SGI332651[] = {{&S12962[81], 5}, {NULL}};
const Segment SGI332652[] = {{&S12962[12], 14}, {&S12962[40], 11}, {NULL}};
const Segment SGI332653[] = {{&S12962[12], 14}, {NULL}};
const Segment SGI332654[] = {{&S12963[6], 4}, {NULL}};
const Segment SGI332655[] = {{&S12963[81], 5}, {NULL}};
const Segment SGI332656[] = {{&S12963[12], 14}, {&S12963[40], 11}, {NULL}};
const Segment SGI332657[] = {{&S12963[12], 14}, {NULL}};
const Segment SGI332658[] = {{&S12964[6], 4}, {NULL}};
const Segment SGI332659[] = {{&S12964[81], 5}, {NULL}};
const Segment SGI332660[] = {{&S12964[12], 14}, {&S12964[40], 11}, {NULL}};
const Segment SGI332661[] = {{&S12964[12], 14}, {NULL}};
const Segment SGI332662[] = {{&S12965[6], 4}, {NULL}};
const Segment SGI332663[] = {{&S12965[81], 5}, {NULL}};
const Segment SGI332664[] = {{&S12965[12], 14}, {&S12965[40], 11}, {NULL}};
const Segment SGI332665[] = {{&S12965[12], 14}, {NULL}};
const Segment SGI332666[] = {{&S12966[6], 4}, {NULL}};
const Segment SGI332667[] = {{&S12966[81], 5}, {NULL}};
const Segment SGI332668[] = {{&S12966[12], 14}, {&S12966[40], 11}, {NULL}};
const Segment SGI332669[] = {{&S12966[12], 14}, {NULL}};
const Segment SGI332670[] = {{&S12967[6], 4}, {NULL}};
const Segment SGI332671[] = {{&S12967[81], 5}, {NULL}};
const Segment SGI332672[] = {{&S12967[12], 14}, {&S12967[40], 11}, {NULL}};
const Segment SGI332673[] = {{&S12967[12], 14}, {NULL}};
const Segment SGI332674[] = {{&S12968[6], 4}, {NULL}};
const Segment SGI332675[] = {{&S12968[81], 5}, {NULL}};
const Segment SGI332676[] = {{&S12968[12], 14}, {&S12968[40], 11}, {NULL}};
const Segment SGI332677[] = {{&S12968[12], 14}, {NULL}};
const Segment SGI332678[] = {{&S12969[6], 4}, {NULL}};
const Segment SGI332679[] = {{&S12969[81], 5}, {NULL}};
const Segment SGI332680[] = {{&S12969[12], 14}, {&S12969[40], 11}, {NULL}};
const Segment SGI332681[] = {{&S12969[12], 14}, {NULL}};
const Segment SGI332682[] = {{&S12970[6], 4}, {NULL}};
const Segment SGI332683[] = {{&S12970[81], 5}, {NULL}};
const Segment SGI332684[] = {{&S12970[12], 14}, {&S12970[40], 11}, {NULL}};
const Segment SGI332685[] = {{&S12970[12], 14}, {NULL}};
const Segment SGI332686[] = {{&S12971[6], 4}, {NULL}};
const Segment SGI332687[] = {{&S12971[55], 5}, {NULL}};
const Segment SGI332688[] = {{&S12971[11], 14}, {NULL}};
const Segment SGI332689[] = {{&S12972[6], 4}, {NULL}};
const Segment SGI332690[] = {{&S12972[55], 5}, {NULL}};
const Segment SGI332691[] = {{&S12972[11], 14}, {NULL}};
const Segment SGI332692[] = {{&S12973[6], 4}, {NULL}};
const Segment SGI332693[] = {{&S12973[81], 5}, {NULL}};
const Segment SGI332694[] = {{&S12973[12], 14}, {&S12973[40], 11}, {NULL}};
const Segment SGI332695[] = {{&S12973[12], 14}, {NULL}};
const Segment SGI332696[] = {{&S12974[6], 4}, {NULL}};
const Segment SGI332697[] = {{&S12974[81], 5}, {NULL}};
const Segment SGI332698[] = {{&S12974[12], 14}, {&S12974[40], 11}, {NULL}};
const Segment SGI332699[] = {{&S12974[12], 14}, {NULL}};
const Segment SGI332700[] = {{&S12975[6], 4}, {NULL}};
const Segment SGI332701[] = {{&S12975[81], 5}, {NULL}};
const Segment SGI332702[] = {{&S12975[12], 14}, {&S12975[40], 11}, {NULL}};
const Segment SGI332703[] = {{&S12975[12], 14}, {NULL}};
const Segment SGI332704[] = {{&S12976[6], 4}, {NULL}};
const Segment SGI332705[] = {{&S12976[81], 5}, {NULL}};
const Segment SGI332706[] = {{&S12976[12], 14}, {&S12976[40], 11}, {NULL}};
const Segment SGI332707[] = {{&S12976[12], 14}, {NULL}};
const Segment SGI332708[] = {{&S12977[6], 4}, {NULL}};
const Segment SGI332709[] = {{&S12977[82], 5}, {NULL}};
const Segment SGI332710[] = {{&S12977[12], 14}, {&S12977[40], 12}, {NULL}};
const Segment SGI332711[] = {{&S12977[12], 14}, {NULL}};
const Segment SGI332712[] = {{&S12978[6], 4}, {NULL}};
const Segment SGI332713[] = {{&S12978[81], 5}, {NULL}};
const Segment SGI332714[] = {{&S12978[12], 14}, {&S12978[40], 11}, {NULL}};
const Segment SGI332715[] = {{&S12978[12], 14}, {NULL}};
const Segment SGI332716[] = {{&S12979[6], 4}, {NULL}};
const Segment SGI332717[] = {{&S12979[82], 5}, {NULL}};
const Segment SGI332718[] = {{&S12979[12], 14}, {&S12979[40], 12}, {NULL}};
const Segment SGI332719[] = {{&S12979[12], 14}, {NULL}};
const Segment SGI332720[] = {{&S12980[6], 4}, {NULL}};
const Segment SGI332721[] = {{&S12980[82], 5}, {NULL}};
const Segment SGI332722[] = {{&S12980[12], 14}, {&S12980[40], 12}, {NULL}};
const Segment SGI332723[] = {{&S12980[12], 14}, {NULL}};
const Segment SGI332724[] = {{&S12981[6], 4}, {NULL}};
const Segment SGI332725[] = {{&S12981[81], 5}, {NULL}};
const Segment SGI332726[] = {{&S12981[12], 14}, {&S12981[40], 11}, {NULL}};
const Segment SGI332727[] = {{&S12981[12], 14}, {NULL}};
const Segment SGI332728[] = {{&S12982[6], 4}, {NULL}};
const Segment SGI332729[] = {{&S12982[81], 5}, {NULL}};
const Segment SGI332730[] = {{&S12982[12], 14}, {&S12982[40], 11}, {NULL}};
const Segment SGI332731[] = {{&S12982[12], 14}, {NULL}};
const Segment SGI332732[] = {{&S12983[6], 4}, {NULL}};
const Segment SGI332733[] = {{&S12983[81], 5}, {NULL}};
const Segment SGI332734[] = {{&S12983[12], 14}, {&S12983[40], 11}, {NULL}};
const Segment SGI332735[] = {{&S12983[12], 14}, {NULL}};
const Segment SGI332736[] = {{&S12984[6], 4}, {NULL}};
const Segment SGI332737[] = {{&S12984[81], 5}, {NULL}};
const Segment SGI332738[] = {{&S12984[12], 14}, {&S12984[40], 11}, {NULL}};
const Segment SGI332739[] = {{&S12984[12], 14}, {NULL}};
const Segment SGI332740[] = {{&S12985[6], 4}, {NULL}};
const Segment SGI332741[] = {{&S12985[81], 5}, {NULL}};
const Segment SGI332742[] = {{&S12985[12], 14}, {&S12985[40], 11}, {NULL}};
const Segment SGI332743[] = {{&S12985[12], 14}, {NULL}};
const Segment SGI332744[] = {{&S12986[6], 4}, {NULL}};
const Segment SGI332745[] = {{&S12986[81], 5}, {NULL}};
const Segment SGI332746[] = {{&S12986[12], 14}, {&S12986[40], 11}, {NULL}};
const Segment SGI332747[] = {{&S12986[12], 14}, {NULL}};
const Segment SGI332748[] = {{&S12987[6], 4}, {NULL}};
const Segment SGI332749[] = {{&S12987[81], 5}, {NULL}};
const Segment SGI332750[] = {{&S12987[12], 14}, {&S12987[40], 11}, {NULL}};
const Segment SGI332751[] = {{&S12987[12], 14}, {NULL}};
const Segment SGI332752[] = {{&S12988[6], 4}, {NULL}};
const Segment SGI332753[] = {{&S12988[81], 5}, {NULL}};
const Segment SGI332754[] = {{&S12988[12], 14}, {&S12988[40], 11}, {NULL}};
const Segment SGI332755[] = {{&S12988[12], 14}, {NULL}};
const Segment SGI332756[] = {{&S12989[6], 4}, {NULL}};
const Segment SGI332757[] = {{&S12989[81], 5}, {NULL}};
const Segment SGI332758[] = {{&S12989[12], 14}, {&S12989[40], 11}, {NULL}};
const Segment SGI332759[] = {{&S12989[12], 14}, {NULL}};
const Segment SGI332760[] = {{&S12990[6], 4}, {NULL}};
const Segment SGI332761[] = {{&S12990[81], 5}, {NULL}};
const Segment SGI332762[] = {{&S12990[12], 14}, {&S12990[40], 11}, {NULL}};
const Segment SGI332763[] = {{&S12990[12], 14}, {NULL}};
const Segment SGI332764[] = {{&S12991[6], 4}, {NULL}};
const Segment SGI332765[] = {{&S12991[82], 5}, {NULL}};
const Segment SGI332766[] = {{&S12991[12], 14}, {&S12991[40], 12}, {NULL}};
const Segment SGI332767[] = {{&S12991[12], 14}, {NULL}};
const Segment SGI332768[] = {{&S12992[6], 4}, {NULL}};
const Segment SGI332769[] = {{&S12992[82], 5}, {NULL}};
const Segment SGI332770[] = {{&S12992[12], 14}, {&S12992[40], 12}, {NULL}};
const Segment SGI332771[] = {{&S12992[12], 14}, {NULL}};
const Segment SGI332772[] = {{&S12993[6], 4}, {NULL}};
const Segment SGI332773[] = {{&S12993[82], 5}, {NULL}};
const Segment SGI332774[] = {{&S12993[12], 14}, {&S12993[40], 12}, {NULL}};
const Segment SGI332775[] = {{&S12993[12], 14}, {NULL}};
const Segment SGI332776[] = {{&S12994[6], 4}, {NULL}};
const Segment SGI332777[] = {{&S12994[81], 5}, {NULL}};
const Segment SGI332778[] = {{&S12994[12], 14}, {&S12994[40], 11}, {NULL}};
const Segment SGI332779[] = {{&S12994[12], 14}, {NULL}};
const Segment SGI332780[] = {{&S12995[6], 4}, {NULL}};
const Segment SGI332781[] = {{&S12995[81], 5}, {NULL}};
const Segment SGI332782[] = {{&S12995[12], 14}, {&S12995[40], 11}, {NULL}};
const Segment SGI332783[] = {{&S12995[12], 14}, {NULL}};
const Segment SGI332784[] = {{&S12996[6], 4}, {NULL}};
const Segment SGI332785[] = {{&S12996[81], 5}, {NULL}};
const Segment SGI332786[] = {{&S12996[12], 14}, {&S12996[40], 11}, {NULL}};
const Segment SGI332787[] = {{&S12996[12], 14}, {NULL}};
const Segment SGI332788[] = {{&S12997[6], 4}, {NULL}};
const Segment SGI332789[] = {{&S12997[81], 5}, {NULL}};
const Segment SGI332790[] = {{&S12997[12], 14}, {&S12997[40], 11}, {NULL}};
const Segment SGI332791[] = {{&S12997[12], 14}, {NULL}};
const Segment SGI332792[] = {{&S12998[6], 4}, {NULL}};
const Segment SGI332793[] = {{&S12998[81], 5}, {NULL}};
const Segment SGI332794[] = {{&S12998[12], 14}, {&S12998[40], 11}, {NULL}};
const Segment SGI332795[] = {{&S12998[12], 14}, {NULL}};
const Segment SGI332796[] = {{&S12999[6], 4}, {NULL}};
const Segment SGI332797[] = {{&S12999[81], 5}, {NULL}};
const Segment SGI332798[] = {{&S12999[12], 14}, {&S12999[40], 11}, {NULL}};
const Segment SGI332799[] = {{&S12999[12], 14}, {NULL}};
const Segment SGI332800[] = {{&S13000[6], 4}, {NULL}};
const Segment SGI332801[] = {{&S13000[81], 5}, {NULL}};
const Segment SGI332802[] = {{&S13000[12], 14}, {&S13000[40], 11}, {NULL}};
const Segment SGI332803[] = {{&S13000[12], 14}, {NULL}};
const Segment SGI332804[] = {{&S13001[6], 4}, {NULL}};
const Segment SGI332805[] = {{&S13001[82], 5}, {NULL}};
const Segment SGI332806[] = {{&S13001[12], 14}, {&S13001[40], 12}, {NULL}};
const Segment SGI332807[] = {{&S13001[12], 14}, {NULL}};
const Segment SGI332808[] = {{&S13002[6], 4}, {NULL}};
const Segment SGI332809[] = {{&S13002[81], 5}, {NULL}};
const Segment SGI332810[] = {{&S13002[12], 14}, {&S13002[40], 11}, {NULL}};
const Segment SGI332811[] = {{&S13002[12], 14}, {NULL}};
const Segment SGI332812[] = {{&S13003[6], 4}, {NULL}};
const Segment SGI332813[] = {{&S13003[81], 5}, {NULL}};
const Segment SGI332814[] = {{&S13003[12], 14}, {&S13003[40], 11}, {NULL}};
const Segment SGI332815[] = {{&S13003[12], 14}, {NULL}};
const Segment SGI332816[] = {{&S13004[6], 4}, {NULL}};
const Segment SGI332817[] = {{&S13004[81], 5}, {NULL}};
const Segment SGI332818[] = {{&S13004[12], 14}, {&S13004[40], 11}, {NULL}};
const Segment SGI332819[] = {{&S13004[12], 14}, {NULL}};
const Segment SGI332820[] = {{&S13005[6], 4}, {NULL}};
const Segment SGI332821[] = {{&S13005[81], 5}, {NULL}};
const Segment SGI332822[] = {{&S13005[12], 14}, {&S13005[40], 11}, {NULL}};
const Segment SGI332823[] = {{&S13005[12], 14}, {NULL}};
const Segment SGI332824[] = {{&S13006[6], 4}, {NULL}};
const Segment SGI332825[] = {{&S13006[81], 5}, {NULL}};
const Segment SGI332826[] = {{&S13006[12], 14}, {&S13006[40], 11}, {NULL}};
const Segment SGI332827[] = {{&S13006[12], 14}, {NULL}};
const Segment SGI332828[] = {{&S38337[5], 5}, {NULL}};
const Segment SGI332829[] = {{&S38338[5], 5}, {&S38338[68], 5}, {NULL}};
const Segment SGI332830[] = {{&S38339[5], 5}, {&S38339[5], 5}, {NULL}};
const Segment SGI332831[] = {{&S14633[64], 5}, {NULL}};
const Segment SGI332832[] = {{&S14633[13], 10}, {NULL}};
const Segment SGI332833[] = {{&S14816[61], 5}, {NULL}};
const Segment SGI332834[] = {{&S14816[13], 10}, {NULL}};
const Segment SGI332835[] = {{&S12206[0], 21}, {NULL}};
const Segment SGI332836[] = {{&S12206[34], 3}, {NULL}};
const Segment SGI332837[] = {{&S12206[73], 13}, {NULL}};
const Segment SGI332838[] = {{&S12207[0], 21}, {NULL}};
const Segment SGI332839[] = {{&S12207[34], 3}, {NULL}};
const Segment SGI332840[] = {{&S12208[0], 21}, {NULL}};
const Segment SGI332841[] = {{&S12208[34], 3}, {NULL}};
const Segment SGI332842[] = {{&S12209[0], 21}, {NULL}};
const Segment SGI332843[] = {{&S12209[34], 3}, {NULL}};
const Segment SGI332844[] = {{&S12209[69], 13}, {NULL}};
const Segment SGI332845[] = {{&S12210[0], 21}, {NULL}};
const Segment SGI332846[] = {{&S12210[34], 3}, {NULL}};
const Segment SGI332847[] = {{&S12210[67], 13}, {NULL}};
const Segment SGI332848[] = {{&S14620[6], 4}, {NULL}};
const Segment SGI332849[] = {{&S14620[52], 5}, {NULL}};
const Segment SGI332850[] = {{&S14620[12], 45}, {NULL}};
const Segment SGI332851[] = {{&S14620[12], 14}, {NULL}};
const Segment SGI332852[] = {{&S14621[6], 4}, {NULL}};
const Segment SGI332853[] = {{&S14621[59], 5}, {NULL}};
const Segment SGI332854[] = {{&S14621[12], 14}, {NULL}};
const Segment SGI332855[] = {{&S14622[6], 4}, {NULL}};
const Segment SGI332856[] = {{&S14622[81], 5}, {NULL}};
const Segment SGI332857[] = {{&S14622[12], 14}, {&S14622[44], 42}, {NULL}};
const Segment SGI332858[] = {{&S14622[12], 14}, {NULL}};
const Segment SGI332859[] = {{&S14623[6], 4}, {NULL}};
const Segment SGI332860[] = {{&S14623[59], 5}, {NULL}};
const Segment SGI332861[] = {{&S14623[12], 14}, {&S14623[70], 4}, {NULL}};
const Segment SGI332862[] = {{&S14623[12], 14}, {NULL}};
const Segment SGI332863[] = {{&S14624[6], 4}, {NULL}};
const Segment SGI332864[] = {{&S14624[81], 5}, {NULL}};
const Segment SGI332865[] = {{&S14624[12], 14}, {&S14624[44], 42}, {NULL}};
const Segment SGI332866[] = {{&S14624[12], 14}, {NULL}};
const Segment SGI332867[] = {{&S14625[6], 4}, {NULL}};
const Segment SGI332868[] = {{&S14625[81], 5}, {NULL}};
const Segment SGI332869[] = {{&S14625[12], 14}, {&S14625[44], 42}, {NULL}};
const Segment SGI332870[] = {{&S14625[12], 14}, {NULL}};
const Segment SGI332871[] = {{&S14626[6], 4}, {NULL}};
const Segment SGI332872[] = {{&S14626[81], 5}, {NULL}};
const Segment SGI332873[] = {{&S14626[12], 14}, {&S14626[44], 42}, {NULL}};
const Segment SGI332874[] = {{&S14626[12], 14}, {NULL}};
const Segment SGI332875[] = {{&S14627[6], 4}, {NULL}};
const Segment SGI332876[] = {{&S14627[81], 5}, {NULL}};
const Segment SGI332877[] = {{&S14627[12], 14}, {&S14627[44], 42}, {NULL}};
const Segment SGI332878[] = {{&S14627[12], 14}, {NULL}};
const Segment SGI332879[] = {{&S14628[6], 4}, {NULL}};
const Segment SGI332880[] = {{&S14628[75], 5}, {NULL}};
const Segment SGI332881[] = {{&S14628[12], 14}, {&S14628[38], 42}, {NULL}};
const Segment SGI332882[] = {{&S14628[12], 14}, {NULL}};
const Segment SGI332883[] = {{&S14629[6], 4}, {NULL}};
const Segment SGI332884[] = {{&S14629[81], 5}, {NULL}};
const Segment SGI332885[] = {{&S14629[12], 14}, {&S14629[44], 42}, {NULL}};
const Segment SGI332886[] = {{&S14629[12], 14}, {NULL}};
const Segment SGI332887[] = {{&S14630[6], 4}, {NULL}};
const Segment SGI332888[] = {{&S14630[52], 5}, {NULL}};
const Segment SGI332889[] = {{&S14630[12], 55}, {NULL}};
const Segment SGI332890[] = {{&S14630[12], 14}, {NULL}};
const Segment SGI332891[] = {{&S14631[6], 4}, {NULL}};
const Segment SGI332892[] = {{&S14631[81], 5}, {NULL}};
const Segment SGI332893[] = {{&S14631[12], 14}, {&S14631[44], 42}, {NULL}};
const Segment SGI332894[] = {{&S14631[12], 14}, {NULL}};
const Segment SGI332895[] = {{&S14632[6], 4}, {NULL}};
const Segment SGI332896[] = {{&S14632[81], 5}, {NULL}};
const Segment SGI332897[] = {{&S14632[12], 14}, {&S14632[44], 42}, {NULL}};
const Segment SGI332898[] = {{&S14632[12], 14}, {NULL}};
const Segment SGI332899[] = {{&S14634[6], 4}, {NULL}};
const Segment SGI332900[] = {{&S14634[81], 5}, {NULL}};
const Segment SGI332901[] = {{&S14634[12], 14}, {&S14634[44], 42}, {NULL}};
const Segment SGI332902[] = {{&S14634[12], 14}, {NULL}};
const Segment SGI332903[] = {{&S14636[6], 4}, {NULL}};
const Segment SGI332904[] = {{&S14636[81], 5}, {NULL}};
const Segment SGI332905[] = {{&S14636[12], 14}, {&S14636[44], 42}, {NULL}};
const Segment SGI332906[] = {{&S14636[12], 14}, {NULL}};
const Segment SGI332907[] = {{&S14637[6], 4}, {NULL}};
const Segment SGI332908[] = {{&S14637[81], 5}, {NULL}};
const Segment SGI332909[] = {{&S14637[12], 14}, {&S14637[44], 42}, {NULL}};
const Segment SGI332910[] = {{&S14637[12], 14}, {NULL}};
const Segment SGI332911[] = {{&S14638[6], 4}, {NULL}};
const Segment SGI332912[] = {{&S14638[81], 5}, {NULL}};
const Segment SGI332913[] = {{&S14638[12], 14}, {&S14638[44], 42}, {NULL}};
const Segment SGI332914[] = {{&S14638[12], 14}, {NULL}};
const Segment SGI332915[] = {{&S14639[6], 4}, {NULL}};
const Segment SGI332916[] = {{&S14639[81], 5}, {NULL}};
const Segment SGI332917[] = {{&S14639[12], 14}, {&S14639[44], 42}, {NULL}};
const Segment SGI332918[] = {{&S14639[12], 14}, {NULL}};
const Segment SGI332919[] = {{&S14640[6], 4}, {NULL}};
const Segment SGI332920[] = {{&S14640[81], 5}, {NULL}};
const Segment SGI332921[] = {{&S14640[12], 14}, {&S14640[44], 42}, {NULL}};
const Segment SGI332922[] = {{&S14640[12], 14}, {NULL}};
const Segment SGI332923[] = {{&S14641[6], 4}, {NULL}};
const Segment SGI332924[] = {{&S14641[81], 5}, {NULL}};
const Segment SGI332925[] = {{&S14641[12], 14}, {&S14641[44], 42}, {NULL}};
const Segment SGI332926[] = {{&S14641[12], 14}, {NULL}};
const Segment SGI332927[] = {{&S14642[6], 4}, {NULL}};
const Segment SGI332928[] = {{&S14642[75], 5}, {NULL}};
const Segment SGI332929[] = {{&S14642[12], 14}, {&S14642[38], 42}, {NULL}};
const Segment SGI332930[] = {{&S14642[12], 14}, {NULL}};
const Segment SGI332931[] = {{&S14643[6], 4}, {NULL}};
const Segment SGI332932[] = {{&S14643[81], 5}, {NULL}};
const Segment SGI332933[] = {{&S14643[12], 14}, {&S14643[44], 42}, {NULL}};
const Segment SGI332934[] = {{&S14643[12], 14}, {NULL}};
const Segment SGI332935[] = {{&S14644[6], 4}, {NULL}};
const Segment SGI332936[] = {{&S14644[81], 5}, {NULL}};
const Segment SGI332937[] = {{&S14644[12], 14}, {&S14644[44], 42}, {NULL}};
const Segment SGI332938[] = {{&S14644[12], 14}, {NULL}};
const Segment SGI332939[] = {{&S14645[6], 4}, {NULL}};
const Segment SGI332940[] = {{&S14645[81], 5}, {NULL}};
const Segment SGI332941[] = {{&S14645[12], 14}, {&S14645[44], 42}, {NULL}};
const Segment SGI332942[] = {{&S14645[12], 14}, {NULL}};
const Segment SGI332943[] = {{&S14646[6], 4}, {NULL}};
const Segment SGI332944[] = {{&S14646[81], 5}, {NULL}};
const Segment SGI332945[] = {{&S14646[12], 14}, {&S14646[44], 42}, {NULL}};
const Segment SGI332946[] = {{&S14646[12], 14}, {NULL}};
const Segment SGI332947[] = {{&S14647[6], 4}, {NULL}};
const Segment SGI332948[] = {{&S14647[81], 5}, {NULL}};
const Segment SGI332949[] = {{&S14647[12], 14}, {&S14647[44], 42}, {NULL}};
const Segment SGI332950[] = {{&S14647[12], 14}, {NULL}};
const Segment SGI332951[] = {{&S14648[6], 4}, {NULL}};
const Segment SGI332952[] = {{&S14648[81], 5}, {NULL}};
const Segment SGI332953[] = {{&S14648[12], 14}, {&S14648[44], 42}, {NULL}};
const Segment SGI332954[] = {{&S14648[12], 14}, {NULL}};
const Segment SGI332955[] = {{&S14649[6], 4}, {NULL}};
const Segment SGI332956[] = {{&S14649[81], 5}, {NULL}};
const Segment SGI332957[] = {{&S14649[12], 14}, {&S14649[44], 42}, {NULL}};
const Segment SGI332958[] = {{&S14649[12], 14}, {NULL}};
const Segment SGI332959[] = {{&S14650[6], 4}, {NULL}};
const Segment SGI332960[] = {{&S14650[81], 5}, {NULL}};
const Segment SGI332961[] = {{&S14650[12], 14}, {&S14650[44], 42}, {NULL}};
const Segment SGI332962[] = {{&S14650[12], 14}, {NULL}};
const Segment SGI332963[] = {{&S14651[6], 4}, {NULL}};
const Segment SGI332964[] = {{&S14651[81], 5}, {NULL}};
const Segment SGI332965[] = {{&S14651[12], 14}, {&S14651[44], 42}, {NULL}};
const Segment SGI332966[] = {{&S14651[12], 14}, {NULL}};
const Segment SGI332967[] = {{&S14652[6], 4}, {NULL}};
const Segment SGI332968[] = {{&S14652[81], 5}, {NULL}};
const Segment SGI332969[] = {{&S14652[12], 14}, {&S14652[44], 42}, {NULL}};
const Segment SGI332970[] = {{&S14652[12], 14}, {NULL}};
const Segment SGI332971[] = {{&S14653[6], 4}, {NULL}};
const Segment SGI332972[] = {{&S14653[81], 5}, {NULL}};
const Segment SGI332973[] = {{&S14653[12], 14}, {&S14653[44], 42}, {NULL}};
const Segment SGI332974[] = {{&S14653[12], 14}, {NULL}};
const Segment SGI332975[] = {{&S14654[6], 4}, {NULL}};
const Segment SGI332976[] = {{&S14654[82], 5}, {NULL}};
const Segment SGI332977[] = {{&S14654[12], 14}, {&S14654[45], 42}, {NULL}};
const Segment SGI332978[] = {{&S14654[12], 14}, {NULL}};
const Segment SGI332979[] = {{&S14655[6], 4}, {NULL}};
const Segment SGI332980[] = {{&S14655[81], 5}, {NULL}};
const Segment SGI332981[] = {{&S14655[12], 14}, {&S14655[44], 42}, {NULL}};
const Segment SGI332982[] = {{&S14655[12], 14}, {NULL}};
const Segment SGI332983[] = {{&S14656[6], 4}, {NULL}};
const Segment SGI332984[] = {{&S14656[81], 5}, {NULL}};
const Segment SGI332985[] = {{&S14656[12], 14}, {&S14656[44], 42}, {NULL}};
const Segment SGI332986[] = {{&S14656[12], 14}, {NULL}};
const Segment SGI332987[] = {{&S14657[6], 4}, {NULL}};
const Segment SGI332988[] = {{&S14657[81], 5}, {NULL}};
const Segment SGI332989[] = {{&S14657[12], 14}, {&S14657[44], 42}, {NULL}};
const Segment SGI332990[] = {{&S14657[12], 14}, {NULL}};
const Segment SGI332991[] = {{&S14658[6], 4}, {NULL}};
const Segment SGI332992[] = {{&S14658[81], 5}, {NULL}};
const Segment SGI332993[] = {{&S14658[12], 14}, {&S14658[44], 42}, {NULL}};
const Segment SGI332994[] = {{&S14658[12], 14}, {NULL}};
const Segment SGI332995[] = {{&S14659[6], 4}, {NULL}};
const Segment SGI332996[] = {{&S14659[81], 5}, {NULL}};
const Segment SGI332997[] = {{&S14659[12], 14}, {&S14659[44], 42}, {NULL}};
const Segment SGI332998[] = {{&S14659[12], 14}, {NULL}};
const Segment SGI332999[] = {{&S14660[6], 4}, {NULL}};
const Segment SGI333000[] = {{&S14660[81], 5}, {NULL}};
const Segment SGI333001[] = {{&S14660[12], 14}, {&S14660[44], 42}, {NULL}};
const Segment SGI333002[] = {{&S14660[12], 14}, {NULL}};
const Segment SGI333003[] = {{&S14661[6], 4}, {NULL}};
const Segment SGI333004[] = {{&S14661[81], 5}, {NULL}};
const Segment SGI333005[] = {{&S14661[12], 14}, {&S14661[44], 42}, {NULL}};
const Segment SGI333006[] = {{&S14661[12], 14}, {NULL}};
const Segment SGI333007[] = {{&S14662[6], 4}, {NULL}};
const Segment SGI333008[] = {{&S14662[74], 5}, {NULL}};
const Segment SGI333009[] = {{&S14662[12], 14}, {&S14662[37], 42}, {NULL}};
const Segment SGI333010[] = {{&S14662[12], 14}, {NULL}};
const Segment SGI333011[] = {{&S14663[6], 4}, {NULL}};
const Segment SGI333012[] = {{&S14663[81], 5}, {NULL}};
const Segment SGI333013[] = {{&S14663[12], 14}, {&S14663[44], 42}, {NULL}};
const Segment SGI333014[] = {{&S14663[12], 14}, {NULL}};
const Segment SGI333015[] = {{&S14664[6], 4}, {NULL}};
const Segment SGI333016[] = {{&S14664[81], 5}, {NULL}};
const Segment SGI333017[] = {{&S14664[12], 14}, {&S14664[44], 42}, {NULL}};
const Segment SGI333018[] = {{&S14664[12], 14}, {NULL}};
const Segment SGI333019[] = {{&S14665[6], 4}, {NULL}};
const Segment SGI333020[] = {{&S14665[81], 5}, {NULL}};
const Segment SGI333021[] = {{&S14665[12], 14}, {&S14665[44], 42}, {NULL}};
const Segment SGI333022[] = {{&S14665[12], 14}, {NULL}};
const Segment SGI333023[] = {{&S14666[6], 4}, {NULL}};
const Segment SGI333024[] = {{&S14666[81], 5}, {NULL}};
const Segment SGI333025[] = {{&S14666[12], 14}, {&S14666[44], 42}, {NULL}};
const Segment SGI333026[] = {{&S14666[12], 14}, {NULL}};
const Segment SGI333027[] = {{&S14667[6], 4}, {NULL}};
const Segment SGI333028[] = {{&S14667[81], 5}, {NULL}};
const Segment SGI333029[] = {{&S14667[12], 14}, {&S14667[44], 42}, {NULL}};
const Segment SGI333030[] = {{&S14667[12], 14}, {NULL}};
const Segment SGI333031[] = {{&S14668[6], 4}, {NULL}};
const Segment SGI333032[] = {{&S14668[82], 5}, {NULL}};
const Segment SGI333033[] = {{&S14668[12], 14}, {&S14668[44], 43}, {NULL}};
const Segment SGI333034[] = {{&S14668[12], 14}, {NULL}};
const Segment SGI333035[] = {{&S14669[6], 4}, {NULL}};
const Segment SGI333036[] = {{&S14669[81], 5}, {NULL}};
const Segment SGI333037[] = {{&S14669[12], 14}, {&S14669[44], 42}, {NULL}};
const Segment SGI333038[] = {{&S14669[12], 14}, {NULL}};
const Segment SGI333039[] = {{&S14670[6], 4}, {NULL}};
const Segment SGI333040[] = {{&S14670[81], 5}, {NULL}};
const Segment SGI333041[] = {{&S14670[12], 14}, {&S14670[44], 42}, {NULL}};
const Segment SGI333042[] = {{&S14670[12], 14}, {NULL}};
const Segment SGI333043[] = {{&S14671[6], 4}, {NULL}};
const Segment SGI333044[] = {{&S14671[75], 5}, {NULL}};
const Segment SGI333045[] = {{&S14671[12], 14}, {&S14671[37], 43}, {NULL}};
const Segment SGI333046[] = {{&S14671[12], 14}, {NULL}};
const Segment SGI333047[] = {{&S14672[6], 4}, {NULL}};
const Segment SGI333048[] = {{&S14672[81], 5}, {NULL}};
const Segment SGI333049[] = {{&S14672[12], 14}, {&S14672[44], 42}, {NULL}};
const Segment SGI333050[] = {{&S14672[12], 14}, {NULL}};
const Segment SGI333051[] = {{&S14673[6], 4}, {NULL}};
const Segment SGI333052[] = {{&S14673[81], 5}, {NULL}};
const Segment SGI333053[] = {{&S14673[12], 14}, {&S14673[44], 42}, {NULL}};
const Segment SGI333054[] = {{&S14673[12], 14}, {NULL}};
const Segment SGI333055[] = {{&S14674[6], 4}, {NULL}};
const Segment SGI333056[] = {{&S14674[74], 5}, {NULL}};
const Segment SGI333057[] = {{&S14674[12], 14}, {&S14674[37], 42}, {NULL}};
const Segment SGI333058[] = {{&S14674[12], 14}, {NULL}};
const Segment SGI333059[] = {{&S14675[6], 4}, {NULL}};
const Segment SGI333060[] = {{&S14675[81], 5}, {NULL}};
const Segment SGI333061[] = {{&S14675[12], 14}, {&S14675[44], 42}, {NULL}};
const Segment SGI333062[] = {{&S14675[12], 14}, {NULL}};
const Segment SGI333063[] = {{&S14676[6], 4}, {NULL}};
const Segment SGI333064[] = {{&S14676[75], 5}, {NULL}};
const Segment SGI333065[] = {{&S14676[12], 14}, {&S14676[37], 43}, {NULL}};
const Segment SGI333066[] = {{&S14676[12], 14}, {NULL}};
const Segment SGI333067[] = {{&S14677[6], 4}, {NULL}};
const Segment SGI333068[] = {{&S14677[81], 5}, {NULL}};
const Segment SGI333069[] = {{&S14677[12], 14}, {&S14677[44], 42}, {NULL}};
const Segment SGI333070[] = {{&S14677[12], 14}, {NULL}};
const Segment SGI333071[] = {{&S14678[6], 4}, {NULL}};
const Segment SGI333072[] = {{&S14678[75], 5}, {NULL}};
const Segment SGI333073[] = {{&S14678[12], 14}, {&S14678[37], 43}, {NULL}};
const Segment SGI333074[] = {{&S14678[12], 14}, {NULL}};
const Segment SGI333075[] = {{&S14679[6], 4}, {NULL}};
const Segment SGI333076[] = {{&S14679[82], 5}, {NULL}};
const Segment SGI333077[] = {{&S14679[12], 14}, {&S14679[45], 42}, {NULL}};
const Segment SGI333078[] = {{&S14679[12], 14}, {NULL}};
const Segment SGI333079[] = {{&S14680[6], 4}, {NULL}};
const Segment SGI333080[] = {{&S14680[81], 5}, {NULL}};
const Segment SGI333081[] = {{&S14680[12], 14}, {&S14680[44], 42}, {NULL}};
const Segment SGI333082[] = {{&S14680[12], 14}, {NULL}};
const Segment SGI333083[] = {{&S14681[6], 4}, {NULL}};
const Segment SGI333084[] = {{&S14681[81], 5}, {NULL}};
const Segment SGI333085[] = {{&S14681[12], 14}, {&S14681[44], 42}, {NULL}};
const Segment SGI333086[] = {{&S14681[12], 14}, {NULL}};
const Segment SGI333087[] = {{&S14682[6], 4}, {NULL}};
const Segment SGI333088[] = {{&S14682[81], 5}, {NULL}};
const Segment SGI333089[] = {{&S14682[12], 14}, {&S14682[44], 42}, {NULL}};
const Segment SGI333090[] = {{&S14682[12], 14}, {NULL}};
const Segment SGI333091[] = {{&S14683[6], 4}, {NULL}};
const Segment SGI333092[] = {{&S14683[81], 5}, {NULL}};
const Segment SGI333093[] = {{&S14683[12], 14}, {&S14683[44], 42}, {NULL}};
const Segment SGI333094[] = {{&S14683[12], 14}, {NULL}};
const Segment SGI333095[] = {{&S14684[6], 4}, {NULL}};
const Segment SGI333096[] = {{&S14684[74], 5}, {NULL}};
const Segment SGI333097[] = {{&S14684[12], 14}, {&S14684[37], 42}, {NULL}};
const Segment SGI333098[] = {{&S14684[12], 14}, {NULL}};
const Segment SGI333099[] = {{&S14685[6], 4}, {NULL}};
const Segment SGI333100[] = {{&S14685[81], 5}, {NULL}};
const Segment SGI333101[] = {{&S14685[12], 14}, {&S14685[44], 42}, {NULL}};
const Segment SGI333102[] = {{&S14685[12], 14}, {NULL}};
const Segment SGI333103[] = {{&S14686[6], 4}, {NULL}};
const Segment SGI333104[] = {{&S14686[81], 5}, {NULL}};
const Segment SGI333105[] = {{&S14686[12], 14}, {&S14686[44], 42}, {NULL}};
const Segment SGI333106[] = {{&S14686[12], 14}, {NULL}};
const Segment SGI333107[] = {{&S14687[6], 4}, {NULL}};
const Segment SGI333108[] = {{&S14687[81], 5}, {NULL}};
const Segment SGI333109[] = {{&S14687[12], 14}, {&S14687[44], 42}, {NULL}};
const Segment SGI333110[] = {{&S14687[12], 14}, {NULL}};
const Segment SGI333111[] = {{&S14688[6], 4}, {NULL}};
const Segment SGI333112[] = {{&S14688[81], 5}, {NULL}};
const Segment SGI333113[] = {{&S14688[12], 14}, {&S14688[44], 42}, {NULL}};
const Segment SGI333114[] = {{&S14688[12], 14}, {NULL}};
const Segment SGI333115[] = {{&S14689[6], 4}, {NULL}};
const Segment SGI333116[] = {{&S14689[81], 5}, {NULL}};
const Segment SGI333117[] = {{&S14689[12], 14}, {&S14689[44], 42}, {NULL}};
const Segment SGI333118[] = {{&S14689[12], 14}, {NULL}};
const Segment SGI333119[] = {{&S14690[6], 4}, {NULL}};
const Segment SGI333120[] = {{&S14690[75], 5}, {NULL}};
const Segment SGI333121[] = {{&S14690[12], 14}, {&S14690[37], 43}, {NULL}};
const Segment SGI333122[] = {{&S14690[12], 14}, {NULL}};
const Segment SGI333123[] = {{&S14692[6], 4}, {NULL}};
const Segment SGI333124[] = {{&S14692[81], 5}, {NULL}};
const Segment SGI333125[] = {{&S14692[12], 14}, {&S14692[44], 42}, {NULL}};
const Segment SGI333126[] = {{&S14692[12], 14}, {NULL}};
const Segment SGI333127[] = {{&S14693[6], 4}, {NULL}};
const Segment SGI333128[] = {{&S14693[74], 5}, {NULL}};
const Segment SGI333129[] = {{&S14693[12], 14}, {&S14693[37], 42}, {NULL}};
const Segment SGI333130[] = {{&S14693[12], 14}, {NULL}};
const Segment SGI333131[] = {{&S14694[6], 4}, {NULL}};
const Segment SGI333132[] = {{&S14694[74], 5}, {NULL}};
const Segment SGI333133[] = {{&S14694[12], 14}, {&S14694[37], 42}, {NULL}};
const Segment SGI333134[] = {{&S14694[12], 14}, {NULL}};
const Segment SGI333135[] = {{&S14695[6], 4}, {NULL}};
const Segment SGI333136[] = {{&S14695[81], 5}, {NULL}};
const Segment SGI333137[] = {{&S14695[12], 14}, {&S14695[44], 42}, {NULL}};
const Segment SGI333138[] = {{&S14695[12], 14}, {NULL}};
const Segment SGI333139[] = {{&S14696[6], 4}, {NULL}};
const Segment SGI333140[] = {{&S14696[75], 5}, {NULL}};
const Segment SGI333141[] = {{&S14696[12], 14}, {&S14696[38], 42}, {NULL}};
const Segment SGI333142[] = {{&S14696[12], 14}, {NULL}};
const Segment SGI333143[] = {{&S14697[6], 4}, {NULL}};
const Segment SGI333144[] = {{&S14697[81], 5}, {NULL}};
const Segment SGI333145[] = {{&S14697[12], 14}, {&S14697[44], 42}, {NULL}};
const Segment SGI333146[] = {{&S14697[12], 14}, {NULL}};
const Segment SGI333147[] = {{&S14698[6], 4}, {NULL}};
const Segment SGI333148[] = {{&S14698[75], 5}, {NULL}};
const Segment SGI333149[] = {{&S14698[12], 14}, {&S14698[38], 42}, {NULL}};
const Segment SGI333150[] = {{&S14698[12], 14}, {NULL}};
const Segment SGI333151[] = {{&S14699[6], 4}, {NULL}};
const Segment SGI333152[] = {{&S14699[74], 5}, {NULL}};
const Segment SGI333153[] = {{&S14699[12], 14}, {&S14699[37], 42}, {NULL}};
const Segment SGI333154[] = {{&S14699[12], 14}, {NULL}};
const Segment SGI333155[] = {{&S14700[6], 4}, {NULL}};
const Segment SGI333156[] = {{&S14700[81], 5}, {NULL}};
const Segment SGI333157[] = {{&S14700[12], 14}, {&S14700[44], 42}, {NULL}};
const Segment SGI333158[] = {{&S14700[12], 14}, {NULL}};
const Segment SGI333159[] = {{&S14701[6], 4}, {NULL}};
const Segment SGI333160[] = {{&S14701[81], 5}, {NULL}};
const Segment SGI333161[] = {{&S14701[12], 14}, {&S14701[44], 42}, {NULL}};
const Segment SGI333162[] = {{&S14701[12], 14}, {NULL}};
const Segment SGI333163[] = {{&S14702[6], 4}, {NULL}};
const Segment SGI333164[] = {{&S14702[74], 5}, {NULL}};
const Segment SGI333165[] = {{&S14702[12], 14}, {&S14702[37], 42}, {NULL}};
const Segment SGI333166[] = {{&S14702[12], 14}, {NULL}};
const Segment SGI333167[] = {{&S14703[6], 4}, {NULL}};
const Segment SGI333168[] = {{&S14703[82], 5}, {NULL}};
const Segment SGI333169[] = {{&S14703[12], 14}, {&S14703[45], 42}, {NULL}};
const Segment SGI333170[] = {{&S14703[12], 14}, {NULL}};
const Segment SGI333171[] = {{&S14704[6], 4}, {NULL}};
const Segment SGI333172[] = {{&S14704[82], 5}, {NULL}};
const Segment SGI333173[] = {{&S14704[12], 14}, {&S14704[45], 42}, {NULL}};
const Segment SGI333174[] = {{&S14704[12], 14}, {NULL}};
const Segment SGI333175[] = {{&S14705[6], 4}, {NULL}};
const Segment SGI333176[] = {{&S14705[81], 5}, {NULL}};
const Segment SGI333177[] = {{&S14705[12], 14}, {&S14705[44], 42}, {NULL}};
const Segment SGI333178[] = {{&S14705[12], 14}, {NULL}};
const Segment SGI333179[] = {{&S14706[6], 4}, {NULL}};
const Segment SGI333180[] = {{&S14706[75], 5}, {NULL}};
const Segment SGI333181[] = {{&S14706[12], 14}, {&S14706[38], 42}, {NULL}};
const Segment SGI333182[] = {{&S14706[12], 14}, {NULL}};
const Segment SGI333183[] = {{&S14707[6], 4}, {NULL}};
const Segment SGI333184[] = {{&S14707[82], 5}, {NULL}};
const Segment SGI333185[] = {{&S14707[12], 14}, {&S14707[45], 42}, {NULL}};
const Segment SGI333186[] = {{&S14707[12], 14}, {NULL}};
const Segment SGI333187[] = {{&S14708[6], 4}, {NULL}};
const Segment SGI333188[] = {{&S14708[81], 5}, {NULL}};
const Segment SGI333189[] = {{&S14708[12], 14}, {&S14708[44], 42}, {NULL}};
const Segment SGI333190[] = {{&S14708[12], 14}, {NULL}};
const Segment SGI333191[] = {{&S14709[6], 4}, {NULL}};
const Segment SGI333192[] = {{&S14709[81], 5}, {NULL}};
const Segment SGI333193[] = {{&S14709[12], 14}, {&S14709[44], 42}, {NULL}};
const Segment SGI333194[] = {{&S14709[12], 14}, {NULL}};
const Segment SGI333195[] = {{&S14710[6], 4}, {NULL}};
const Segment SGI333196[] = {{&S14710[74], 5}, {NULL}};
const Segment SGI333197[] = {{&S14710[12], 14}, {&S14710[37], 42}, {NULL}};
const Segment SGI333198[] = {{&S14710[12], 14}, {NULL}};
const Segment SGI333199[] = {{&S14711[6], 4}, {NULL}};
const Segment SGI333200[] = {{&S14711[81], 5}, {NULL}};
const Segment SGI333201[] = {{&S14711[12], 14}, {&S14711[44], 42}, {NULL}};
const Segment SGI333202[] = {{&S14711[12], 14}, {NULL}};
const Segment SGI333203[] = {{&S14712[6], 4}, {NULL}};
const Segment SGI333204[] = {{&S14712[74], 5}, {NULL}};
const Segment SGI333205[] = {{&S14712[12], 14}, {&S14712[37], 42}, {NULL}};
const Segment SGI333206[] = {{&S14712[12], 14}, {NULL}};
const Segment SGI333207[] = {{&S14713[6], 4}, {NULL}};
const Segment SGI333208[] = {{&S14713[81], 5}, {NULL}};
const Segment SGI333209[] = {{&S14713[12], 14}, {&S14713[44], 42}, {NULL}};
const Segment SGI333210[] = {{&S14713[12], 14}, {NULL}};
const Segment SGI333211[] = {{&S14714[6], 4}, {NULL}};
const Segment SGI333212[] = {{&S14714[59], 5}, {NULL}};
const Segment SGI333213[] = {{&S14714[12], 14}, {&S14714[70], 14}, {NULL}};
const Segment SGI333214[] = {{&S14714[12], 14}, {NULL}};
const Segment SGI333215[] = {{&S14715[6], 4}, {NULL}};
const Segment SGI333216[] = {{&S14715[74], 5}, {NULL}};
const Segment SGI333217[] = {{&S14715[12], 14}, {&S14715[37], 42}, {NULL}};
const Segment SGI333218[] = {{&S14715[12], 14}, {NULL}};
const Segment SGI333219[] = {{&S14716[6], 4}, {NULL}};
const Segment SGI333220[] = {{&S14716[81], 5}, {NULL}};
const Segment SGI333221[] = {{&S14716[12], 14}, {&S14716[44], 42}, {NULL}};
const Segment SGI333222[] = {{&S14716[12], 14}, {NULL}};
const Segment SGI333223[] = {{&S14717[6], 4}, {NULL}};
const Segment SGI333224[] = {{&S14717[52], 5}, {NULL}};
const Segment SGI333225[] = {{&S14717[12], 84}, {NULL}};
const Segment SGI333226[] = {{&S14717[12], 14}, {NULL}};
const Segment SGI333227[] = {{&S14718[6], 4}, {NULL}};
const Segment SGI333228[] = {{&S14718[81], 5}, {NULL}};
const Segment SGI333229[] = {{&S14718[12], 14}, {&S14718[44], 42}, {NULL}};
const Segment SGI333230[] = {{&S14718[12], 14}, {NULL}};
const Segment SGI333231[] = {{&S14719[6], 4}, {NULL}};
const Segment SGI333232[] = {{&S14719[75], 5}, {NULL}};
const Segment SGI333233[] = {{&S14719[12], 14}, {&S14719[37], 43}, {NULL}};
const Segment SGI333234[] = {{&S14719[12], 14}, {NULL}};
const Segment SGI333235[] = {{&S14720[6], 4}, {NULL}};
const Segment SGI333236[] = {{&S14720[74], 5}, {NULL}};
const Segment SGI333237[] = {{&S14720[12], 14}, {&S14720[37], 42}, {NULL}};
const Segment SGI333238[] = {{&S14720[12], 14}, {NULL}};
const Segment SGI333239[] = {{&S14721[6], 4}, {NULL}};
const Segment SGI333240[] = {{&S14721[75], 5}, {NULL}};
const Segment SGI333241[] = {{&S14721[12], 14}, {&S14721[37], 43}, {NULL}};
const Segment SGI333242[] = {{&S14721[12], 14}, {NULL}};
const Segment SGI333243[] = {{&S14722[6], 4}, {NULL}};
const Segment SGI333244[] = {{&S14722[82], 5}, {NULL}};
const Segment SGI333245[] = {{&S14722[12], 14}, {&S14722[45], 42}, {NULL}};
const Segment SGI333246[] = {{&S14722[12], 14}, {NULL}};
const Segment SGI333247[] = {{&S14723[6], 4}, {NULL}};
const Segment SGI333248[] = {{&S14723[81], 5}, {NULL}};
const Segment SGI333249[] = {{&S14723[12], 14}, {&S14723[44], 42}, {NULL}};
const Segment SGI333250[] = {{&S14723[12], 14}, {NULL}};
const Segment SGI333251[] = {{&S14724[6], 4}, {NULL}};
const Segment SGI333252[] = {{&S14724[74], 5}, {NULL}};
const Segment SGI333253[] = {{&S14724[12], 14}, {&S14724[37], 42}, {NULL}};
const Segment SGI333254[] = {{&S14724[12], 14}, {NULL}};
const Segment SGI333255[] = {{&S14725[6], 4}, {NULL}};
const Segment SGI333256[] = {{&S14725[81], 5}, {NULL}};
const Segment SGI333257[] = {{&S14725[12], 14}, {&S14725[44], 42}, {NULL}};
const Segment SGI333258[] = {{&S14725[12], 14}, {NULL}};
const Segment SGI333259[] = {{&S14726[6], 4}, {NULL}};
const Segment SGI333260[] = {{&S14726[74], 5}, {NULL}};
const Segment SGI333261[] = {{&S14726[12], 14}, {&S14726[37], 42}, {NULL}};
const Segment SGI333262[] = {{&S14726[12], 14}, {NULL}};
const Segment SGI333263[] = {{&S14727[6], 4}, {NULL}};
const Segment SGI333264[] = {{&S14727[74], 5}, {NULL}};
const Segment SGI333265[] = {{&S14727[12], 14}, {&S14727[37], 42}, {NULL}};
const Segment SGI333266[] = {{&S14727[12], 14}, {NULL}};
const Segment SGI333267[] = {{&S14728[6], 4}, {NULL}};
const Segment SGI333268[] = {{&S14728[82], 5}, {NULL}};
const Segment SGI333269[] = {{&S14728[12], 14}, {&S14728[45], 42}, {NULL}};
const Segment SGI333270[] = {{&S14728[12], 14}, {NULL}};
const Segment SGI333271[] = {{&S14729[6], 4}, {NULL}};
const Segment SGI333272[] = {{&S14729[81], 5}, {NULL}};
const Segment SGI333273[] = {{&S14729[12], 14}, {&S14729[44], 42}, {NULL}};
const Segment SGI333274[] = {{&S14729[12], 14}, {NULL}};
const Segment SGI333275[] = {{&S14730[6], 4}, {NULL}};
const Segment SGI333276[] = {{&S14730[81], 5}, {NULL}};
const Segment SGI333277[] = {{&S14730[12], 14}, {&S14730[44], 42}, {NULL}};
const Segment SGI333278[] = {{&S14730[12], 14}, {NULL}};
const Segment SGI333279[] = {{&S14731[6], 4}, {NULL}};
const Segment SGI333280[] = {{&S14731[74], 5}, {NULL}};
const Segment SGI333281[] = {{&S14731[12], 14}, {&S14731[37], 42}, {NULL}};
const Segment SGI333282[] = {{&S14731[12], 14}, {NULL}};
const Segment SGI333283[] = {{&S14732[6], 4}, {NULL}};
const Segment SGI333284[] = {{&S14732[81], 5}, {NULL}};
const Segment SGI333285[] = {{&S14732[12], 14}, {&S14732[44], 42}, {NULL}};
const Segment SGI333286[] = {{&S14732[12], 14}, {NULL}};
const Segment SGI333287[] = {{&S14733[6], 4}, {NULL}};
const Segment SGI333288[] = {{&S14733[75], 5}, {NULL}};
const Segment SGI333289[] = {{&S14733[12], 14}, {&S14733[37], 43}, {NULL}};
const Segment SGI333290[] = {{&S14733[12], 14}, {NULL}};
const Segment SGI333291[] = {{&S14734[6], 4}, {NULL}};
const Segment SGI333292[] = {{&S14734[82], 5}, {NULL}};
const Segment SGI333293[] = {{&S14734[12], 14}, {&S14734[44], 43}, {NULL}};
const Segment SGI333294[] = {{&S14734[12], 14}, {NULL}};
const Segment SGI333295[] = {{&S14735[6], 4}, {NULL}};
const Segment SGI333296[] = {{&S14735[82], 5}, {NULL}};
const Segment SGI333297[] = {{&S14735[12], 14}, {&S14735[44], 43}, {NULL}};
const Segment SGI333298[] = {{&S14735[12], 14}, {NULL}};
const Segment SGI333299[] = {{&S14736[6], 4}, {NULL}};
const Segment SGI333300[] = {{&S14736[81], 5}, {NULL}};
const Segment SGI333301[] = {{&S14736[12], 14}, {&S14736[44], 42}, {NULL}};
const Segment SGI333302[] = {{&S14736[12], 14}, {NULL}};
const Segment SGI333303[] = {{&S14737[6], 4}, {NULL}};
const Segment SGI333304[] = {{&S14737[82], 5}, {NULL}};
const Segment SGI333305[] = {{&S14737[12], 14}, {&S14737[45], 42}, {NULL}};
const Segment SGI333306[] = {{&S14737[12], 14}, {NULL}};
const Segment SGI333307[] = {{&S14738[6], 4}, {NULL}};
const Segment SGI333308[] = {{&S14738[82], 5}, {NULL}};
const Segment SGI333309[] = {{&S14738[12], 14}, {&S14738[44], 43}, {NULL}};
const Segment SGI333310[] = {{&S14738[12], 14}, {NULL}};
const Segment SGI333311[] = {{&S14739[6], 4}, {NULL}};
const Segment SGI333312[] = {{&S14739[52], 5}, {NULL}};
const Segment SGI333313[] = {{&S14739[12], 62}, {NULL}};
const Segment SGI333314[] = {{&S14739[12], 14}, {NULL}};
const Segment SGI333315[] = {{&S14740[6], 4}, {NULL}};
const Segment SGI333316[] = {{&S14740[74], 5}, {NULL}};
const Segment SGI333317[] = {{&S14740[12], 14}, {&S14740[37], 42}, {NULL}};
const Segment SGI333318[] = {{&S14740[12], 14}, {NULL}};
const Segment SGI333319[] = {{&S14741[6], 4}, {NULL}};
const Segment SGI333320[] = {{&S14741[82], 5}, {NULL}};
const Segment SGI333321[] = {{&S14741[12], 14}, {&S14741[44], 43}, {NULL}};
const Segment SGI333322[] = {{&S14741[12], 14}, {NULL}};
const Segment SGI333323[] = {{&S14742[6], 4}, {NULL}};
const Segment SGI333324[] = {{&S14742[82], 5}, {NULL}};
const Segment SGI333325[] = {{&S14742[12], 14}, {&S14742[45], 42}, {NULL}};
const Segment SGI333326[] = {{&S14742[12], 14}, {NULL}};
const Segment SGI333327[] = {{&S14743[6], 4}, {NULL}};
const Segment SGI333328[] = {{&S14743[59], 5}, {NULL}};
const Segment SGI333329[] = {{&S14743[12], 14}, {&S14743[70], 33}, {NULL}};
const Segment SGI333330[] = {{&S14743[12], 14}, {NULL}};
const Segment SGI333331[] = {{&S14744[6], 4}, {NULL}};
const Segment SGI333332[] = {{&S14744[74], 5}, {NULL}};
const Segment SGI333333[] = {{&S14744[12], 14}, {&S14744[37], 42}, {NULL}};
const Segment SGI333334[] = {{&S14744[12], 14}, {NULL}};
const Segment SGI333335[] = {{&S14745[6], 4}, {NULL}};
const Segment SGI333336[] = {{&S14745[75], 5}, {NULL}};
const Segment SGI333337[] = {{&S14745[12], 14}, {&S14745[38], 42}, {NULL}};
const Segment SGI333338[] = {{&S14745[12], 14}, {NULL}};
const Segment SGI333339[] = {{&S14746[6], 4}, {NULL}};
const Segment SGI333340[] = {{&S14746[81], 5}, {NULL}};
const Segment SGI333341[] = {{&S14746[12], 14}, {&S14746[44], 42}, {NULL}};
const Segment SGI333342[] = {{&S14746[12], 14}, {NULL}};
const Segment SGI333343[] = {{&S14747[6], 4}, {NULL}};
const Segment SGI333344[] = {{&S14747[81], 5}, {NULL}};
const Segment SGI333345[] = {{&S14747[12], 14}, {&S14747[44], 42}, {NULL}};
const Segment SGI333346[] = {{&S14747[12], 14}, {NULL}};
const Segment SGI333347[] = {{&S14748[6], 4}, {NULL}};
const Segment SGI333348[] = {{&S14748[81], 5}, {NULL}};
const Segment SGI333349[] = {{&S14748[12], 14}, {&S14748[44], 42}, {NULL}};
const Segment SGI333350[] = {{&S14748[12], 14}, {NULL}};
const Segment SGI333351[] = {{&S14749[6], 4}, {NULL}};
const Segment SGI333352[] = {{&S14749[75], 5}, {NULL}};
const Segment SGI333353[] = {{&S14749[12], 14}, {&S14749[38], 42}, {NULL}};
const Segment SGI333354[] = {{&S14749[12], 14}, {NULL}};
const Segment SGI333355[] = {{&S14750[6], 4}, {NULL}};
const Segment SGI333356[] = {{&S14750[52], 5}, {NULL}};
const Segment SGI333357[] = {{&S14750[12], 65}, {NULL}};
const Segment SGI333358[] = {{&S14750[12], 14}, {NULL}};
const Segment SGI333359[] = {{&S14751[6], 4}, {NULL}};
const Segment SGI333360[] = {{&S14751[81], 5}, {NULL}};
const Segment SGI333361[] = {{&S14751[12], 14}, {&S14751[44], 42}, {NULL}};
const Segment SGI333362[] = {{&S14751[12], 14}, {NULL}};
const Segment SGI333363[] = {{&S14752[6], 4}, {NULL}};
const Segment SGI333364[] = {{&S14752[74], 5}, {NULL}};
const Segment SGI333365[] = {{&S14752[12], 14}, {&S14752[37], 42}, {NULL}};
const Segment SGI333366[] = {{&S14752[12], 14}, {NULL}};
const Segment SGI333367[] = {{&S14753[6], 4}, {NULL}};
const Segment SGI333368[] = {{&S14753[74], 5}, {NULL}};
const Segment SGI333369[] = {{&S14753[12], 14}, {&S14753[37], 42}, {NULL}};
const Segment SGI333370[] = {{&S14753[12], 14}, {NULL}};
const Segment SGI333371[] = {{&S14754[6], 4}, {NULL}};
const Segment SGI333372[] = {{&S14754[81], 5}, {NULL}};
const Segment SGI333373[] = {{&S14754[12], 14}, {&S14754[44], 42}, {NULL}};
const Segment SGI333374[] = {{&S14754[12], 14}, {NULL}};
const Segment SGI333375[] = {{&S14755[6], 4}, {NULL}};
const Segment SGI333376[] = {{&S14755[81], 5}, {NULL}};
const Segment SGI333377[] = {{&S14755[12], 14}, {&S14755[44], 42}, {NULL}};
const Segment SGI333378[] = {{&S14755[12], 14}, {NULL}};
const Segment SGI333379[] = {{&S14756[6], 4}, {NULL}};
const Segment SGI333380[] = {{&S14756[74], 5}, {NULL}};
const Segment SGI333381[] = {{&S14756[12], 14}, {&S14756[37], 42}, {NULL}};
const Segment SGI333382[] = {{&S14756[12], 14}, {NULL}};
const Segment SGI333383[] = {{&S14757[6], 4}, {NULL}};
const Segment SGI333384[] = {{&S14757[81], 5}, {NULL}};
const Segment SGI333385[] = {{&S14757[12], 14}, {&S14757[44], 42}, {NULL}};
const Segment SGI333386[] = {{&S14757[12], 14}, {NULL}};
const Segment SGI333387[] = {{&S14758[6], 4}, {NULL}};
const Segment SGI333388[] = {{&S14758[81], 5}, {NULL}};
const Segment SGI333389[] = {{&S14758[12], 14}, {&S14758[44], 42}, {NULL}};
const Segment SGI333390[] = {{&S14758[12], 14}, {NULL}};
const Segment SGI333391[] = {{&S14759[6], 4}, {NULL}};
const Segment SGI333392[] = {{&S14759[81], 5}, {NULL}};
const Segment SGI333393[] = {{&S14759[12], 14}, {&S14759[44], 42}, {NULL}};
const Segment SGI333394[] = {{&S14759[12], 14}, {NULL}};
const Segment SGI333395[] = {{&S14760[6], 4}, {NULL}};
const Segment SGI333396[] = {{&S14760[74], 5}, {NULL}};
const Segment SGI333397[] = {{&S14760[12], 14}, {&S14760[37], 42}, {NULL}};
const Segment SGI333398[] = {{&S14760[12], 14}, {NULL}};
const Segment SGI333399[] = {{&S14761[6], 4}, {NULL}};
const Segment SGI333400[] = {{&S14761[82], 5}, {NULL}};
const Segment SGI333401[] = {{&S14761[12], 14}, {&S14761[45], 42}, {NULL}};
const Segment SGI333402[] = {{&S14761[12], 14}, {NULL}};
const Segment SGI333403[] = {{&S14762[6], 4}, {NULL}};
const Segment SGI333404[] = {{&S14762[81], 5}, {NULL}};
const Segment SGI333405[] = {{&S14762[12], 14}, {&S14762[44], 42}, {NULL}};
const Segment SGI333406[] = {{&S14762[12], 14}, {NULL}};
const Segment SGI333407[] = {{&S14763[6], 4}, {NULL}};
const Segment SGI333408[] = {{&S14763[81], 5}, {NULL}};
const Segment SGI333409[] = {{&S14763[12], 14}, {&S14763[44], 42}, {NULL}};
const Segment SGI333410[] = {{&S14763[12], 14}, {NULL}};
const Segment SGI333411[] = {{&S14764[6], 4}, {NULL}};
const Segment SGI333412[] = {{&S14764[75], 5}, {NULL}};
const Segment SGI333413[] = {{&S14764[12], 14}, {&S14764[38], 42}, {NULL}};
const Segment SGI333414[] = {{&S14764[12], 14}, {NULL}};
const Segment SGI333415[] = {{&S14765[6], 4}, {NULL}};
const Segment SGI333416[] = {{&S14765[81], 5}, {NULL}};
const Segment SGI333417[] = {{&S14765[12], 14}, {&S14765[44], 42}, {NULL}};
const Segment SGI333418[] = {{&S14765[12], 14}, {NULL}};
const Segment SGI333419[] = {{&S14766[6], 4}, {NULL}};
const Segment SGI333420[] = {{&S14766[75], 5}, {NULL}};
const Segment SGI333421[] = {{&S14766[12], 14}, {&S14766[37], 43}, {NULL}};
const Segment SGI333422[] = {{&S14766[12], 14}, {NULL}};
const Segment SGI333423[] = {{&S14767[6], 4}, {NULL}};
const Segment SGI333424[] = {{&S14767[81], 5}, {NULL}};
const Segment SGI333425[] = {{&S14767[12], 14}, {&S14767[44], 42}, {NULL}};
const Segment SGI333426[] = {{&S14767[12], 14}, {NULL}};
const Segment SGI333427[] = {{&S14768[6], 4}, {NULL}};
const Segment SGI333428[] = {{&S14768[81], 5}, {NULL}};
const Segment SGI333429[] = {{&S14768[12], 14}, {&S14768[44], 42}, {NULL}};
const Segment SGI333430[] = {{&S14768[12], 14}, {NULL}};
const Segment SGI333431[] = {{&S14769[6], 4}, {NULL}};
const Segment SGI333432[] = {{&S14769[81], 5}, {NULL}};
const Segment SGI333433[] = {{&S14769[12], 14}, {&S14769[44], 42}, {NULL}};
const Segment SGI333434[] = {{&S14769[12], 14}, {NULL}};
const Segment SGI333435[] = {{&S14770[6], 4}, {NULL}};
const Segment SGI333436[] = {{&S14770[81], 5}, {NULL}};
const Segment SGI333437[] = {{&S14770[12], 14}, {&S14770[44], 42}, {NULL}};
const Segment SGI333438[] = {{&S14770[12], 14}, {NULL}};
const Segment SGI333439[] = {{&S14771[6], 4}, {NULL}};
const Segment SGI333440[] = {{&S14771[81], 5}, {NULL}};
const Segment SGI333441[] = {{&S14771[12], 14}, {&S14771[44], 42}, {NULL}};
const Segment SGI333442[] = {{&S14771[12], 14}, {NULL}};
const Segment SGI333443[] = {{&S14772[6], 4}, {NULL}};
const Segment SGI333444[] = {{&S14772[75], 5}, {NULL}};
const Segment SGI333445[] = {{&S14772[12], 14}, {&S14772[37], 43}, {NULL}};
const Segment SGI333446[] = {{&S14772[12], 14}, {NULL}};
const Segment SGI333447[] = {{&S14773[6], 4}, {NULL}};
const Segment SGI333448[] = {{&S14773[74], 5}, {NULL}};
const Segment SGI333449[] = {{&S14773[12], 14}, {&S14773[37], 42}, {NULL}};
const Segment SGI333450[] = {{&S14773[12], 14}, {NULL}};
const Segment SGI333451[] = {{&S14774[6], 4}, {NULL}};
const Segment SGI333452[] = {{&S14774[74], 5}, {NULL}};
const Segment SGI333453[] = {{&S14774[12], 14}, {&S14774[37], 42}, {NULL}};
const Segment SGI333454[] = {{&S14774[12], 14}, {NULL}};
const Segment SGI333455[] = {{&S14775[6], 4}, {NULL}};
const Segment SGI333456[] = {{&S14775[74], 5}, {NULL}};
const Segment SGI333457[] = {{&S14775[12], 14}, {&S14775[37], 42}, {NULL}};
const Segment SGI333458[] = {{&S14775[12], 14}, {NULL}};
const Segment SGI333459[] = {{&S14776[6], 4}, {NULL}};
const Segment SGI333460[] = {{&S14776[81], 5}, {NULL}};
const Segment SGI333461[] = {{&S14776[12], 14}, {&S14776[44], 42}, {NULL}};
const Segment SGI333462[] = {{&S14776[12], 14}, {NULL}};
const Segment SGI333463[] = {{&S14777[6], 4}, {NULL}};
const Segment SGI333464[] = {{&S14777[59], 5}, {NULL}};
const Segment SGI333465[] = {{&S14777[12], 14}, {NULL}};
const Segment SGI333466[] = {{&S14778[6], 4}, {NULL}};
const Segment SGI333467[] = {{&S14778[81], 5}, {NULL}};
const Segment SGI333468[] = {{&S14778[12], 14}, {&S14778[40], 11}, {NULL}};
const Segment SGI333469[] = {{&S14778[12], 14}, {NULL}};
const Segment SGI333470[] = {{&S14779[6], 4}, {NULL}};
const Segment SGI333471[] = {{&S14779[81], 5}, {NULL}};
const Segment SGI333472[] = {{&S14779[12], 14}, {&S14779[40], 11}, {NULL}};
const Segment SGI333473[] = {{&S14779[12], 14}, {NULL}};
const Segment SGI333474[] = {{&S14780[6], 4}, {NULL}};
const Segment SGI333475[] = {{&S14780[82], 5}, {NULL}};
const Segment SGI333476[] = {{&S15842[64], 5}, {NULL}};
const Segment SGI333477[] = {{&S14780[12], 14}, {&S14780[41], 11}, {NULL}};
const Segment SGI333478[] = {{&S15842[13], 10}, {NULL}};
const Segment SGI333479[] = {{&S14780[12], 14}, {NULL}};
const Segment SGI333480[] = {{&S14781[6], 4}, {NULL}};
const Segment SGI333481[] = {{&S14781[82], 5}, {NULL}};
const Segment SGI333482[] = {{&S14781[12], 14}, {&S14781[40], 12}, {NULL}};
const Segment SGI333483[] = {{&S14781[12], 14}, {NULL}};
const Segment SGI333484[] = {{&S14782[6], 4}, {NULL}};
const Segment SGI333485[] = {{&S14782[81], 5}, {NULL}};
const Segment SGI333486[] = {{&S14782[12], 14}, {&S14782[40], 11}, {NULL}};
const Segment SGI333487[] = {{&S14782[12], 14}, {NULL}};
const Segment SGI333488[] = {{&S14783[6], 4}, {NULL}};
const Segment SGI333489[] = {{&S14783[82], 5}, {NULL}};
const Segment SGI333490[] = {{&S14783[12], 14}, {&S14783[41], 11}, {NULL}};
const Segment SGI333491[] = {{&S14783[12], 14}, {NULL}};
const Segment SGI333492[] = {{&S14784[6], 4}, {NULL}};
const Segment SGI333493[] = {{&S14784[81], 5}, {NULL}};
const Segment SGI333494[] = {{&S14784[12], 14}, {&S14784[40], 11}, {NULL}};
const Segment SGI333495[] = {{&S14784[12], 14}, {NULL}};
const Segment SGI333496[] = {{&S14785[6], 4}, {NULL}};
const Segment SGI333497[] = {{&S14785[82], 5}, {NULL}};
const Segment SGI333498[] = {{&S14785[12], 14}, {&S14785[40], 12}, {NULL}};
const Segment SGI333499[] = {{&S14785[12], 14}, {NULL}};
const Segment SGI333500[] = {{&S14786[6], 4}, {NULL}};
const Segment SGI333501[] = {{&S14786[81], 5}, {NULL}};
const Segment SGI333502[] = {{&S14786[12], 14}, {&S14786[40], 11}, {NULL}};
const Segment SGI333503[] = {{&S14786[12], 14}, {NULL}};
const Segment SGI333504[] = {{&S14787[6], 4}, {NULL}};
const Segment SGI333505[] = {{&S14787[82], 5}, {NULL}};
const Segment SGI333506[] = {{&S14787[12], 14}, {&S14787[41], 11}, {NULL}};
const Segment SGI333507[] = {{&S14787[12], 14}, {NULL}};
const Segment SGI333508[] = {{&S14788[6], 4}, {NULL}};
const Segment SGI333509[] = {{&S14788[59], 5}, {NULL}};
const Segment SGI333510[] = {{&S14788[12], 14}, {&S14788[70], 31}, {NULL}};
const Segment SGI333511[] = {{&S14788[12], 14}, {NULL}};
const Segment SGI333512[] = {{&S14789[6], 4}, {NULL}};
const Segment SGI333513[] = {{&S14789[82], 5}, {NULL}};
const Segment SGI333514[] = {{&S14789[12], 14}, {&S14789[40], 12}, {NULL}};
const Segment SGI333515[] = {{&S14789[12], 14}, {NULL}};
const Segment SGI333516[] = {{&S14790[6], 4}, {NULL}};
const Segment SGI333517[] = {{&S14790[81], 5}, {NULL}};
const Segment SGI333518[] = {{&S14790[12], 14}, {&S14790[40], 11}, {NULL}};
const Segment SGI333519[] = {{&S14790[12], 14}, {NULL}};
const Segment SGI333520[] = {{&S14791[6], 4}, {NULL}};
const Segment SGI333521[] = {{&S14791[66], 5}, {NULL}};
const Segment SGI333522[] = {{&S14791[12], 14}, {NULL}};
const Segment SGI333523[] = {{&S14792[6], 4}, {NULL}};
const Segment SGI333524[] = {{&S14792[81], 5}, {NULL}};
const Segment SGI333525[] = {{&S14792[12], 14}, {&S14792[40], 11}, {NULL}};
const Segment SGI333526[] = {{&S14792[12], 14}, {NULL}};
const Segment SGI333527[] = {{&S14793[6], 4}, {NULL}};
const Segment SGI333528[] = {{&S14793[81], 5}, {NULL}};
const Segment SGI333529[] = {{&S14793[12], 14}, {&S14793[40], 11}, {NULL}};
const Segment SGI333530[] = {{&S14793[12], 14}, {NULL}};
const Segment SGI333531[] = {{&S14794[6], 4}, {NULL}};
const Segment SGI333532[] = {{&S14794[81], 5}, {NULL}};
const Segment SGI333533[] = {{&S14794[12], 14}, {&S14794[40], 11}, {NULL}};
const Segment SGI333534[] = {{&S14794[12], 14}, {NULL}};
const Segment SGI333535[] = {{&S14795[6], 4}, {NULL}};
const Segment SGI333536[] = {{&S14795[82], 5}, {NULL}};
const Segment SGI333537[] = {{&S14795[12], 14}, {&S14795[41], 11}, {NULL}};
const Segment SGI333538[] = {{&S14795[12], 14}, {NULL}};
const Segment SGI333539[] = {{&S14796[6], 4}, {NULL}};
const Segment SGI333540[] = {{&S14796[81], 5}, {NULL}};
const Segment SGI333541[] = {{&S14796[12], 14}, {&S14796[40], 11}, {NULL}};
const Segment SGI333542[] = {{&S14796[12], 14}, {NULL}};
const Segment SGI333543[] = {{&S14797[6], 4}, {NULL}};
const Segment SGI333544[] = {{&S14797[81], 5}, {NULL}};
const Segment SGI333545[] = {{&S14797[12], 14}, {&S14797[40], 11}, {NULL}};
const Segment SGI333546[] = {{&S14797[12], 14}, {NULL}};
const Segment SGI333547[] = {{&S14798[6], 4}, {NULL}};
const Segment SGI333548[] = {{&S14798[81], 5}, {NULL}};
const Segment SGI333549[] = {{&S14798[12], 14}, {&S14798[40], 11}, {NULL}};
const Segment SGI333550[] = {{&S14798[12], 14}, {NULL}};
const Segment SGI333551[] = {{&S14799[6], 4}, {NULL}};
const Segment SGI333552[] = {{&S14799[82], 5}, {NULL}};
const Segment SGI333553[] = {{&S14799[12], 14}, {&S14799[40], 12}, {NULL}};
const Segment SGI333554[] = {{&S14799[12], 14}, {NULL}};
const Segment SGI333555[] = {{&S14800[6], 4}, {NULL}};
const Segment SGI333556[] = {{&S14800[81], 5}, {NULL}};
const Segment SGI333557[] = {{&S14800[12], 14}, {&S14800[40], 11}, {NULL}};
const Segment SGI333558[] = {{&S14800[12], 14}, {NULL}};
const Segment SGI333559[] = {{&S14801[6], 4}, {NULL}};
const Segment SGI333560[] = {{&S14801[82], 5}, {NULL}};
const Segment SGI333561[] = {{&S14801[12], 14}, {&S14801[41], 11}, {NULL}};
const Segment SGI333562[] = {{&S14801[12], 14}, {NULL}};
const Segment SGI333563[] = {{&S14802[6], 4}, {NULL}};
const Segment SGI333564[] = {{&S14802[82], 5}, {NULL}};
const Segment SGI333565[] = {{&S14802[12], 14}, {&S14802[41], 11}, {NULL}};
const Segment SGI333566[] = {{&S14802[12], 14}, {NULL}};
const Segment SGI333567[] = {{&S14804[6], 4}, {NULL}};
const Segment SGI333568[] = {{&S14804[82], 5}, {NULL}};
const Segment SGI333569[] = {{&S14804[12], 14}, {&S14804[40], 12}, {NULL}};
const Segment SGI333570[] = {{&S14804[12], 14}, {NULL}};
const Segment SGI333571[] = {{&S14805[6], 4}, {NULL}};
const Segment SGI333572[] = {{&S14805[82], 5}, {NULL}};
const Segment SGI333573[] = {{&S14805[12], 14}, {&S14805[40], 12}, {NULL}};
const Segment SGI333574[] = {{&S14805[12], 14}, {NULL}};
const Segment SGI333575[] = {{&S14806[6], 4}, {NULL}};
const Segment SGI333576[] = {{&S14806[82], 5}, {NULL}};
const Segment SGI333577[] = {{&S14806[12], 14}, {&S14806[41], 11}, {NULL}};
const Segment SGI333578[] = {{&S14806[12], 14}, {NULL}};
const Segment SGI333579[] = {{&S14807[6], 4}, {NULL}};
const Segment SGI333580[] = {{&S14807[82], 5}, {NULL}};
const Segment SGI333581[] = {{&S14807[12], 14}, {&S14807[41], 11}, {NULL}};
const Segment SGI333582[] = {{&S14807[12], 14}, {NULL}};
const Segment SGI333583[] = {{&S14808[6], 4}, {NULL}};
const Segment SGI333584[] = {{&S14808[81], 5}, {NULL}};
const Segment SGI333585[] = {{&S14808[12], 14}, {&S14808[40], 11}, {NULL}};
const Segment SGI333586[] = {{&S14808[12], 14}, {NULL}};
const Segment SGI333587[] = {{&S14809[6], 4}, {NULL}};
const Segment SGI333588[] = {{&S14809[81], 5}, {NULL}};
const Segment SGI333589[] = {{&S14809[12], 14}, {&S14809[40], 11}, {NULL}};
const Segment SGI333590[] = {{&S14809[12], 14}, {NULL}};
const Segment SGI333591[] = {{&S14810[6], 4}, {NULL}};
const Segment SGI333592[] = {{&S14810[81], 5}, {NULL}};
const Segment SGI333593[] = {{&S14810[12], 14}, {&S14810[40], 11}, {NULL}};
const Segment SGI333594[] = {{&S14810[12], 14}, {NULL}};
const Segment SGI333595[] = {{&S14811[6], 4}, {NULL}};
const Segment SGI333596[] = {{&S14811[82], 5}, {NULL}};
const Segment SGI333597[] = {{&S14811[12], 14}, {&S14811[41], 11}, {NULL}};
const Segment SGI333598[] = {{&S14811[12], 14}, {NULL}};
const Segment SGI333599[] = {{&S14812[6], 4}, {NULL}};
const Segment SGI333600[] = {{&S14812[82], 5}, {NULL}};
const Segment SGI333601[] = {{&S14812[12], 14}, {&S14812[40], 12}, {NULL}};
const Segment SGI333602[] = {{&S14812[12], 14}, {NULL}};
const Segment SGI333603[] = {{&S14813[6], 4}, {NULL}};
const Segment SGI333604[] = {{&S14813[81], 5}, {NULL}};
const Segment SGI333605[] = {{&S14813[12], 14}, {&S14813[40], 11}, {NULL}};
const Segment SGI333606[] = {{&S14813[12], 14}, {NULL}};
const Segment SGI333607[] = {{&S14814[6], 4}, {NULL}};
const Segment SGI333608[] = {{&S14814[81], 5}, {NULL}};
const Segment SGI333609[] = {{&S14814[12], 14}, {&S14814[40], 11}, {NULL}};
const Segment SGI333610[] = {{&S14814[12], 14}, {NULL}};
const Segment SGI333611[] = {{&S14815[6], 4}, {NULL}};
const Segment SGI333612[] = {{&S14815[82], 5}, {NULL}};
const Segment SGI333613[] = {{&S14815[12], 14}, {&S14815[40], 12}, {NULL}};
const Segment SGI333614[] = {{&S14815[12], 14}, {NULL}};
const Segment SGI333615[] = {{&S14817[6], 4}, {NULL}};
const Segment SGI333616[] = {{&S14817[82], 5}, {NULL}};
const Segment SGI333617[] = {{&S14817[12], 14}, {&S14817[40], 12}, {NULL}};
const Segment SGI333618[] = {{&S14817[12], 14}, {NULL}};
const Segment SGI333619[] = {{&S14818[6], 4}, {NULL}};
const Segment SGI333620[] = {{&S14818[82], 5}, {NULL}};
const Segment SGI333621[] = {{&S14818[12], 14}, {&S14818[40], 12}, {NULL}};
const Segment SGI333622[] = {{&S14818[12], 14}, {NULL}};
const Segment SGI333623[] = {{&S14819[6], 4}, {NULL}};
const Segment SGI333624[] = {{&S14819[81], 5}, {NULL}};
const Segment SGI333625[] = {{&S14819[12], 14}, {&S14819[40], 11}, {NULL}};
const Segment SGI333626[] = {{&S14819[12], 14}, {NULL}};
const Segment SGI333627[] = {{&S14820[6], 4}, {NULL}};
const Segment SGI333628[] = {{&S14820[82], 5}, {NULL}};
const Segment SGI333629[] = {{&S14820[12], 14}, {&S14820[40], 12}, {NULL}};
const Segment SGI333630[] = {{&S14820[12], 14}, {NULL}};
const Segment SGI333631[] = {{&S14821[6], 4}, {NULL}};
const Segment SGI333632[] = {{&S14821[82], 5}, {NULL}};
const Segment SGI333633[] = {{&S14821[12], 14}, {&S14821[41], 11}, {NULL}};
const Segment SGI333634[] = {{&S14821[12], 14}, {NULL}};
const Segment SGI333635[] = {{&S14822[6], 4}, {NULL}};
const Segment SGI333636[] = {{&S14822[81], 5}, {NULL}};
const Segment SGI333637[] = {{&S14822[12], 14}, {&S14822[40], 11}, {NULL}};
const Segment SGI333638[] = {{&S14822[12], 14}, {NULL}};
const Segment SGI333639[] = {{&S14823[6], 4}, {NULL}};
const Segment SGI333640[] = {{&S14823[82], 5}, {NULL}};
const Segment SGI333641[] = {{&S14823[12], 14}, {&S14823[41], 11}, {NULL}};
const Segment SGI333642[] = {{&S14823[12], 14}, {NULL}};
const Segment SGI333643[] = {{&S14824[6], 4}, {NULL}};
const Segment SGI333644[] = {{&S14824[82], 5}, {NULL}};
const Segment SGI333645[] = {{&S14824[12], 14}, {&S14824[41], 11}, {NULL}};
const Segment SGI333646[] = {{&S14824[12], 14}, {NULL}};
const Segment SGI333647[] = {{&S14825[6], 4}, {NULL}};
const Segment SGI333648[] = {{&S14825[81], 5}, {NULL}};
const Segment SGI333649[] = {{&S14825[12], 14}, {&S14825[40], 11}, {NULL}};
const Segment SGI333650[] = {{&S14825[12], 14}, {NULL}};
const Segment SGI333651[] = {{&S14826[6], 4}, {NULL}};
const Segment SGI333652[] = {{&S14826[81], 5}, {NULL}};
const Segment SGI333653[] = {{&S14826[12], 14}, {&S14826[40], 11}, {NULL}};
const Segment SGI333654[] = {{&S14826[12], 14}, {NULL}};
const Segment SGI333655[] = {{&S14827[6], 4}, {NULL}};
const Segment SGI333656[] = {{&S14827[82], 5}, {NULL}};
const Segment SGI333657[] = {{&S14827[12], 14}, {&S14827[40], 12}, {NULL}};
const Segment SGI333658[] = {{&S14827[12], 14}, {NULL}};
const Segment SGI333659[] = {{&S14828[6], 4}, {NULL}};
const Segment SGI333660[] = {{&S14828[81], 5}, {NULL}};
const Segment SGI333661[] = {{&S14828[12], 14}, {&S14828[40], 11}, {NULL}};
const Segment SGI333662[] = {{&S14828[12], 14}, {NULL}};
const Segment SGI333663[] = {{&S14829[6], 4}, {NULL}};
const Segment SGI333664[] = {{&S14829[82], 5}, {NULL}};
const Segment SGI333665[] = {{&S14829[12], 14}, {&S14829[41], 11}, {NULL}};
const Segment SGI333666[] = {{&S14829[12], 14}, {NULL}};
const Segment SGI333667[] = {{&S14830[6], 4}, {NULL}};
const Segment SGI333668[] = {{&S14830[82], 5}, {NULL}};
const Segment SGI333669[] = {{&S14830[12], 14}, {&S14830[41], 11}, {NULL}};
const Segment SGI333670[] = {{&S14830[12], 14}, {NULL}};
const Segment SGI333671[] = {{&S14831[6], 4}, {NULL}};
const Segment SGI333672[] = {{&S14831[81], 5}, {NULL}};
const Segment SGI333673[] = {{&S14831[12], 14}, {&S14831[40], 11}, {NULL}};
const Segment SGI333674[] = {{&S14831[12], 14}, {NULL}};
const Segment SGI333675[] = {{&S14832[6], 4}, {NULL}};
const Segment SGI333676[] = {{&S14832[82], 5}, {NULL}};
const Segment SGI333677[] = {{&S14832[12], 14}, {&S14832[41], 11}, {NULL}};
const Segment SGI333678[] = {{&S14832[12], 14}, {NULL}};
const Segment SGI333679[] = {{&S14833[6], 4}, {NULL}};
const Segment SGI333680[] = {{&S14833[81], 5}, {NULL}};
const Segment SGI333681[] = {{&S14833[12], 14}, {&S14833[40], 11}, {NULL}};
const Segment SGI333682[] = {{&S14833[12], 14}, {NULL}};
const Segment SGI333683[] = {{&S14834[6], 4}, {NULL}};
const Segment SGI333684[] = {{&S14834[81], 5}, {NULL}};
const Segment SGI333685[] = {{&S14834[12], 14}, {&S14834[40], 11}, {NULL}};
const Segment SGI333686[] = {{&S14834[12], 14}, {NULL}};
const Segment SGI333687[] = {{&S14835[6], 4}, {NULL}};
const Segment SGI333688[] = {{&S14835[81], 5}, {NULL}};
const Segment SGI333689[] = {{&S14835[12], 14}, {&S14835[40], 11}, {NULL}};
const Segment SGI333690[] = {{&S14835[12], 14}, {NULL}};
const Segment SGI333691[] = {{&S14836[6], 4}, {NULL}};
const Segment SGI333692[] = {{&S14836[97], 5}, {NULL}};
const Segment SGI333693[] = {{&S14836[12], 14}, {NULL}};
const Segment SGI333694[] = {{&S14836[12], 14}, {&S14836[53], 14}, {NULL}};
const Segment SGI333695[] = {{&S14837[6], 4}, {NULL}};
const Segment SGI333696[] = {{&S14837[81], 5}, {NULL}};
const Segment SGI333697[] = {{&S14837[12], 14}, {&S14837[40], 11}, {NULL}};
const Segment SGI333698[] = {{&S14837[12], 14}, {NULL}};
const Segment SGI333699[] = {{&S14838[6], 4}, {NULL}};
const Segment SGI333700[] = {{&S14838[82], 5}, {NULL}};
const Segment SGI333701[] = {{&S14838[12], 14}, {&S14838[40], 12}, {NULL}};
const Segment SGI333702[] = {{&S14838[12], 14}, {NULL}};
const Segment SGI333703[] = {{&S14839[6], 4}, {NULL}};
const Segment SGI333704[] = {{&S14839[82], 5}, {NULL}};
const Segment SGI333705[] = {{&S14839[12], 14}, {&S14839[40], 12}, {NULL}};
const Segment SGI333706[] = {{&S14839[12], 14}, {NULL}};
const Segment SGI333707[] = {{&S14840[6], 4}, {NULL}};
const Segment SGI333708[] = {{&S14840[81], 5}, {NULL}};
const Segment SGI333709[] = {{&S14840[12], 14}, {&S14840[40], 11}, {NULL}};
const Segment SGI333710[] = {{&S14840[12], 14}, {NULL}};
const Segment SGI333711[] = {{&S14841[6], 4}, {NULL}};
const Segment SGI333712[] = {{&S14841[82], 5}, {NULL}};
const Segment SGI333713[] = {{&S14841[12], 14}, {&S14841[41], 11}, {NULL}};
const Segment SGI333714[] = {{&S14841[12], 14}, {NULL}};
const Segment SGI333715[] = {{&S14842[6], 4}, {NULL}};
const Segment SGI333716[] = {{&S14842[82], 5}, {NULL}};
const Segment SGI333717[] = {{&S14842[12], 14}, {&S14842[41], 11}, {NULL}};
const Segment SGI333718[] = {{&S14842[12], 14}, {NULL}};
const Segment SGI333719[] = {{&S14843[6], 4}, {NULL}};
const Segment SGI333720[] = {{&S14843[81], 5}, {NULL}};
const Segment SGI333721[] = {{&S14843[12], 14}, {&S14843[40], 11}, {NULL}};
const Segment SGI333722[] = {{&S14843[12], 14}, {NULL}};
const Segment SGI333723[] = {{&S14844[6], 4}, {NULL}};
const Segment SGI333724[] = {{&S14844[82], 5}, {NULL}};
const Segment SGI333725[] = {{&S14844[12], 14}, {&S14844[41], 11}, {NULL}};
const Segment SGI333726[] = {{&S14844[12], 14}, {NULL}};
const Segment SGI333727[] = {{&S14845[6], 4}, {NULL}};
const Segment SGI333728[] = {{&S14845[81], 5}, {NULL}};
const Segment SGI333729[] = {{&S14845[12], 14}, {&S14845[40], 11}, {NULL}};
const Segment SGI333730[] = {{&S14845[12], 14}, {NULL}};
const Segment SGI333731[] = {{&S14846[6], 4}, {NULL}};
const Segment SGI333732[] = {{&S14846[82], 5}, {NULL}};
const Segment SGI333733[] = {{&S14846[12], 14}, {&S14846[40], 12}, {NULL}};
const Segment SGI333734[] = {{&S14846[12], 14}, {NULL}};
const Segment SGI333735[] = {{&S14847[6], 4}, {NULL}};
const Segment SGI333736[] = {{&S14847[82], 5}, {NULL}};
const Segment SGI333737[] = {{&S14847[12], 14}, {&S14847[40], 12}, {NULL}};
const Segment SGI333738[] = {{&S14847[12], 14}, {NULL}};
const Segment SGI333739[] = {{&S14848[6], 4}, {NULL}};
const Segment SGI333740[] = {{&S14848[82], 5}, {NULL}};
const Segment SGI333741[] = {{&S14848[12], 14}, {&S14848[41], 11}, {NULL}};
const Segment SGI333742[] = {{&S14848[12], 14}, {NULL}};
const Segment SGI333743[] = {{&S14849[6], 4}, {NULL}};
const Segment SGI333744[] = {{&S14849[82], 5}, {NULL}};
const Segment SGI333745[] = {{&S14849[12], 14}, {&S14849[40], 12}, {NULL}};
const Segment SGI333746[] = {{&S14849[12], 14}, {NULL}};
const Segment SGI333747[] = {{&S14850[6], 4}, {NULL}};
const Segment SGI333748[] = {{&S14850[82], 5}, {NULL}};
const Segment SGI333749[] = {{&S14850[12], 14}, {&S14850[40], 12}, {NULL}};
const Segment SGI333750[] = {{&S14850[12], 14}, {NULL}};
const Segment SGI333751[] = {{&S14851[6], 4}, {NULL}};
const Segment SGI333752[] = {{&S14851[59], 5}, {NULL}};
const Segment SGI333753[] = {{&S14851[12], 14}, {&S14851[70], 4}, {NULL}};
const Segment SGI333754[] = {{&S14851[12], 14}, {NULL}};
const Segment SGI333755[] = {{&S14852[6], 4}, {NULL}};
const Segment SGI333756[] = {{&S14852[81], 5}, {NULL}};
const Segment SGI333757[] = {{&S14852[12], 14}, {&S14852[40], 11}, {NULL}};
const Segment SGI333758[] = {{&S14852[12], 14}, {NULL}};
const Segment SGI333759[] = {{&S14853[6], 4}, {NULL}};
const Segment SGI333760[] = {{&S14853[82], 5}, {NULL}};
const Segment SGI333761[] = {{&S14853[12], 14}, {&S14853[40], 12}, {NULL}};
const Segment SGI333762[] = {{&S14853[12], 14}, {NULL}};
const Segment SGI333763[] = {{&S14854[6], 4}, {NULL}};
const Segment SGI333764[] = {{&S14854[82], 5}, {NULL}};
const Segment SGI333765[] = {{&S14854[12], 14}, {&S14854[40], 12}, {NULL}};
const Segment SGI333766[] = {{&S14854[12], 14}, {NULL}};
const Segment SGI333767[] = {{&S14855[6], 4}, {NULL}};
const Segment SGI333768[] = {{&S14855[81], 5}, {NULL}};
const Segment SGI333769[] = {{&S14855[12], 14}, {&S14855[40], 11}, {NULL}};
const Segment SGI333770[] = {{&S14855[12], 14}, {NULL}};
const Segment SGI333771[] = {{&S14856[6], 4}, {NULL}};
const Segment SGI333772[] = {{&S14856[81], 5}, {NULL}};
const Segment SGI333773[] = {{&S14856[12], 14}, {&S14856[40], 11}, {NULL}};
const Segment SGI333774[] = {{&S14856[12], 14}, {NULL}};
const Segment SGI333775[] = {{&S14857[6], 4}, {NULL}};
const Segment SGI333776[] = {{&S14857[82], 5}, {NULL}};
const Segment SGI333777[] = {{&S14857[12], 14}, {&S14857[40], 12}, {NULL}};
const Segment SGI333778[] = {{&S14857[12], 14}, {NULL}};
const Segment SGI333779[] = {{&S14858[6], 4}, {NULL}};
const Segment SGI333780[] = {{&S14858[81], 5}, {NULL}};
const Segment SGI333781[] = {{&S14858[12], 14}, {&S14858[40], 11}, {NULL}};
const Segment SGI333782[] = {{&S14858[12], 14}, {NULL}};
const Segment SGI333783[] = {{&S14859[6], 4}, {NULL}};
const Segment SGI333784[] = {{&S14859[81], 5}, {NULL}};
const Segment SGI333785[] = {{&S14859[12], 14}, {&S14859[40], 11}, {NULL}};
const Segment SGI333786[] = {{&S14859[12], 14}, {NULL}};
const Segment SGI333787[] = {{&S14860[6], 4}, {NULL}};
const Segment SGI333788[] = {{&S14860[81], 5}, {NULL}};
const Segment SGI333789[] = {{&S14860[12], 14}, {&S14860[40], 11}, {NULL}};
const Segment SGI333790[] = {{&S14860[12], 14}, {NULL}};
const Segment SGI333791[] = {{&S14861[6], 4}, {NULL}};
const Segment SGI333792[] = {{&S14861[81], 5}, {NULL}};
const Segment SGI333793[] = {{&S14861[12], 14}, {&S14861[40], 11}, {NULL}};
const Segment SGI333794[] = {{&S14861[12], 14}, {NULL}};
const Segment SGI333795[] = {{&S14862[6], 4}, {NULL}};
const Segment SGI333796[] = {{&S14862[82], 5}, {NULL}};
const Segment SGI333797[] = {{&S14862[12], 14}, {&S14862[40], 12}, {NULL}};
const Segment SGI333798[] = {{&S14862[12], 14}, {NULL}};
const Segment SGI333799[] = {{&S14863[6], 4}, {NULL}};
const Segment SGI333800[] = {{&S14863[81], 5}, {NULL}};
const Segment SGI333801[] = {{&S14863[12], 14}, {&S14863[40], 11}, {NULL}};
const Segment SGI333802[] = {{&S14863[12], 14}, {NULL}};
const Segment SGI333803[] = {{&S14864[6], 4}, {NULL}};
const Segment SGI333804[] = {{&S14864[82], 5}, {NULL}};
const Segment SGI333805[] = {{&S14864[12], 14}, {&S14864[41], 11}, {NULL}};
const Segment SGI333806[] = {{&S14864[12], 14}, {NULL}};
const Segment SGI333807[] = {{&S14865[6], 4}, {NULL}};
const Segment SGI333808[] = {{&S14865[81], 5}, {NULL}};
const Segment SGI333809[] = {{&S14865[12], 14}, {&S14865[40], 11}, {NULL}};
const Segment SGI333810[] = {{&S14865[12], 14}, {NULL}};
const Segment SGI333811[] = {{&S14866[6], 4}, {NULL}};
const Segment SGI333812[] = {{&S14866[81], 5}, {NULL}};
const Segment SGI333813[] = {{&S14866[12], 14}, {&S14866[40], 11}, {NULL}};
const Segment SGI333814[] = {{&S14866[12], 14}, {NULL}};
const Segment SGI333815[] = {{&S14867[6], 5}, {NULL}};
const Segment SGI333816[] = {{&S14867[60], 5}, {NULL}};
const Segment SGI333817[] = {{&S14867[13], 14}, {NULL}};
const Segment SGI333818[] = {{&S14868[6], 4}, {NULL}};
const Segment SGI333819[] = {{&S14868[81], 5}, {NULL}};
const Segment SGI333820[] = {{&S14868[12], 14}, {&S14868[40], 11}, {NULL}};
const Segment SGI333821[] = {{&S14868[12], 14}, {NULL}};
const Segment SGI333822[] = {{&S14869[6], 4}, {NULL}};
const Segment SGI333823[] = {{&S14869[81], 5}, {NULL}};
const Segment SGI333824[] = {{&S14869[12], 14}, {&S14869[40], 11}, {NULL}};
const Segment SGI333825[] = {{&S14869[12], 14}, {NULL}};
const Segment SGI333826[] = {{&S14870[6], 4}, {NULL}};
const Segment SGI333827[] = {{&S14870[109], 5}, {NULL}};
const Segment SGI333828[] = {{&S14870[12], 14}, {&S14870[40], 12}, {NULL}};
const Segment SGI333829[] = {{&S14870[12], 14}, {NULL}};
const Segment SGI333830[] = {{&S14871[6], 4}, {NULL}};
const Segment SGI333831[] = {{&S14871[82], 5}, {NULL}};
const Segment SGI333832[] = {{&S14871[12], 14}, {&S14871[41], 11}, {NULL}};
const Segment SGI333833[] = {{&S14871[12], 14}, {NULL}};
const Segment SGI333834[] = {{&S14872[6], 4}, {NULL}};
const Segment SGI333835[] = {{&S14872[82], 5}, {NULL}};
const Segment SGI333836[] = {{&S14872[12], 14}, {&S14872[40], 12}, {NULL}};
const Segment SGI333837[] = {{&S14872[12], 14}, {NULL}};
const Segment SGI333838[] = {{&S14873[6], 4}, {NULL}};
const Segment SGI333839[] = {{&S14873[81], 5}, {NULL}};
const Segment SGI333840[] = {{&S14873[12], 14}, {&S14873[40], 11}, {NULL}};
const Segment SGI333841[] = {{&S14873[12], 14}, {NULL}};
const Segment SGI333842[] = {{&S14874[6], 4}, {NULL}};
const Segment SGI333843[] = {{&S14874[81], 5}, {NULL}};
const Segment SGI333844[] = {{&S14874[12], 14}, {&S14874[40], 11}, {NULL}};
const Segment SGI333845[] = {{&S14874[12], 14}, {NULL}};
const Segment SGI333846[] = {{&S14875[6], 4}, {NULL}};
const Segment SGI333847[] = {{&S14875[81], 5}, {NULL}};
const Segment SGI333848[] = {{&S14875[12], 14}, {&S14875[40], 11}, {NULL}};
const Segment SGI333849[] = {{&S14875[12], 14}, {NULL}};
const Segment SGI333850[] = {{&S14876[6], 4}, {NULL}};
const Segment SGI333851[] = {{&S14876[81], 5}, {NULL}};
const Segment SGI333852[] = {{&S14876[12], 14}, {&S14876[40], 11}, {NULL}};
const Segment SGI333853[] = {{&S14876[12], 14}, {NULL}};
const Segment SGI333854[] = {{&S14877[6], 4}, {NULL}};
const Segment SGI333855[] = {{&S14877[81], 5}, {NULL}};
const Segment SGI333856[] = {{&S14877[12], 14}, {&S14877[40], 11}, {NULL}};
const Segment SGI333857[] = {{&S14877[12], 14}, {NULL}};
const Segment SGI333858[] = {{&S14878[6], 4}, {NULL}};
const Segment SGI333859[] = {{&S14878[81], 5}, {NULL}};
const Segment SGI333860[] = {{&S14878[12], 14}, {&S14878[40], 11}, {NULL}};
const Segment SGI333861[] = {{&S14878[12], 14}, {NULL}};
const Segment SGI333862[] = {{&S14879[6], 4}, {NULL}};
const Segment SGI333863[] = {{&S14879[81], 5}, {NULL}};
const Segment SGI333864[] = {{&S14879[12], 14}, {&S14879[40], 11}, {NULL}};
const Segment SGI333865[] = {{&S14879[12], 14}, {NULL}};
const Segment SGI333866[] = {{&S14880[6], 4}, {NULL}};
const Segment SGI333867[] = {{&S14880[81], 5}, {NULL}};
const Segment SGI333868[] = {{&S14880[12], 14}, {&S14880[40], 11}, {NULL}};
const Segment SGI333869[] = {{&S14880[12], 14}, {NULL}};
const Segment SGI333870[] = {{&S14881[6], 4}, {NULL}};
const Segment SGI333871[] = {{&S14881[81], 5}, {NULL}};
const Segment SGI333872[] = {{&S14881[12], 14}, {&S14881[40], 11}, {NULL}};
const Segment SGI333873[] = {{&S14881[12], 14}, {NULL}};
const Segment SGI333874[] = {{&S14882[6], 4}, {NULL}};
const Segment SGI333875[] = {{&S14882[109], 5}, {NULL}};
const Segment SGI333876[] = {{&S14882[12], 14}, {&S14882[40], 12}, {NULL}};
const Segment SGI333877[] = {{&S14882[12], 14}, {NULL}};
const Segment SGI333878[] = {{&S14883[6], 4}, {NULL}};
const Segment SGI333879[] = {{&S14883[81], 5}, {NULL}};
const Segment SGI333880[] = {{&S14883[12], 14}, {&S14883[40], 11}, {NULL}};
const Segment SGI333881[] = {{&S14883[12], 14}, {NULL}};
const Segment SGI333882[] = {{&S14884[6], 4}, {NULL}};
const Segment SGI333883[] = {{&S14884[81], 5}, {NULL}};
const Segment SGI333884[] = {{&S14884[12], 14}, {&S14884[40], 11}, {NULL}};
const Segment SGI333885[] = {{&S14884[12], 14}, {NULL}};
const Segment SGI333886[] = {{&S14885[6], 4}, {NULL}};
const Segment SGI333887[] = {{&S14885[81], 5}, {NULL}};
const Segment SGI333888[] = {{&S14885[12], 14}, {&S14885[40], 11}, {NULL}};
const Segment SGI333889[] = {{&S14885[12], 14}, {NULL}};
const Segment SGI333890[] = {{&S14886[6], 4}, {NULL}};
const Segment SGI333891[] = {{&S14886[81], 5}, {NULL}};
const Segment SGI333892[] = {{&S14886[12], 14}, {&S14886[40], 11}, {NULL}};
const Segment SGI333893[] = {{&S14886[12], 14}, {NULL}};
const Segment SGI333894[] = {{&S14887[6], 4}, {NULL}};
const Segment SGI333895[] = {{&S14887[81], 5}, {NULL}};
const Segment SGI333896[] = {{&S14887[12], 14}, {&S14887[40], 11}, {NULL}};
const Segment SGI333897[] = {{&S14887[12], 14}, {NULL}};
const Segment SGI333898[] = {{&S14888[6], 4}, {NULL}};
const Segment SGI333899[] = {{&S14888[81], 5}, {NULL}};
const Segment SGI333900[] = {{&S14888[12], 14}, {&S14888[40], 11}, {NULL}};
const Segment SGI333901[] = {{&S14888[12], 14}, {NULL}};
const Segment SGI333902[] = {{&S14889[6], 4}, {NULL}};
const Segment SGI333903[] = {{&S14889[81], 5}, {NULL}};
const Segment SGI333904[] = {{&S14889[12], 14}, {&S14889[40], 11}, {NULL}};
const Segment SGI333905[] = {{&S14889[12], 14}, {NULL}};
const Segment SGI333906[] = {{&S14890[6], 4}, {NULL}};
const Segment SGI333907[] = {{&S14890[81], 5}, {NULL}};
const Segment SGI333908[] = {{&S14890[12], 14}, {&S14890[40], 11}, {NULL}};
const Segment SGI333909[] = {{&S14890[12], 14}, {NULL}};
const Segment SGI333910[] = {{&S14891[6], 4}, {NULL}};
const Segment SGI333911[] = {{&S14891[81], 5}, {NULL}};
const Segment SGI333912[] = {{&S14891[12], 14}, {&S14891[40], 11}, {NULL}};
const Segment SGI333913[] = {{&S14891[12], 14}, {NULL}};
const Segment SGI333914[] = {{&S14892[6], 4}, {NULL}};
const Segment SGI333915[] = {{&S14892[81], 5}, {NULL}};
const Segment SGI333916[] = {{&S14892[12], 14}, {&S14892[40], 11}, {NULL}};
const Segment SGI333917[] = {{&S14892[12], 14}, {NULL}};
const Segment SGI333918[] = {{&S14893[6], 4}, {NULL}};
const Segment SGI333919[] = {{&S14893[59], 5}, {NULL}};
const Segment SGI333920[] = {{&S14893[12], 14}, {&S14893[70], 33}, {NULL}};
const Segment SGI333921[] = {{&S14893[12], 14}, {NULL}};
const Segment SGI333922[] = {{&S14894[6], 4}, {NULL}};
const Segment SGI333923[] = {{&S14894[82], 5}, {NULL}};
const Segment SGI333924[] = {{&S14894[12], 14}, {&S14894[40], 12}, {NULL}};
const Segment SGI333925[] = {{&S14894[12], 14}, {NULL}};
const Segment SGI333926[] = {{&S14895[6], 4}, {NULL}};
const Segment SGI333927[] = {{&S14895[82], 5}, {NULL}};
const Segment SGI333928[] = {{&S14895[12], 14}, {&S14895[40], 12}, {NULL}};
const Segment SGI333929[] = {{&S14895[12], 14}, {NULL}};
const Segment SGI333930[] = {{&S14896[6], 4}, {NULL}};
const Segment SGI333931[] = {{&S14896[81], 5}, {NULL}};
const Segment SGI333932[] = {{&S14896[12], 14}, {&S14896[40], 11}, {NULL}};
const Segment SGI333933[] = {{&S14896[12], 14}, {NULL}};
const Segment SGI333934[] = {{&S14897[6], 4}, {NULL}};
const Segment SGI333935[] = {{&S14897[81], 5}, {NULL}};
const Segment SGI333936[] = {{&S14897[12], 14}, {&S14897[40], 11}, {NULL}};
const Segment SGI333937[] = {{&S14897[12], 14}, {NULL}};
const Segment SGI333938[] = {{&S14898[6], 4}, {NULL}};
const Segment SGI333939[] = {{&S14898[81], 5}, {NULL}};
const Segment SGI333940[] = {{&S14898[12], 14}, {&S14898[40], 11}, {NULL}};
const Segment SGI333941[] = {{&S14898[12], 14}, {NULL}};
const Segment SGI333942[] = {{&S14899[6], 4}, {NULL}};
const Segment SGI333943[] = {{&S14899[81], 5}, {NULL}};
const Segment SGI333944[] = {{&S14899[12], 14}, {&S14899[40], 11}, {NULL}};
const Segment SGI333945[] = {{&S14899[12], 14}, {NULL}};
const Segment SGI333946[] = {{&S14900[6], 4}, {NULL}};
const Segment SGI333947[] = {{&S14900[81], 5}, {NULL}};
const Segment SGI333948[] = {{&S14900[12], 14}, {&S14900[40], 11}, {NULL}};
const Segment SGI333949[] = {{&S14900[12], 14}, {NULL}};
const Segment SGI333950[] = {{&S14901[6], 4}, {NULL}};
const Segment SGI333951[] = {{&S14901[81], 5}, {NULL}};
const Segment SGI333952[] = {{&S14901[12], 14}, {&S14901[40], 11}, {NULL}};
const Segment SGI333953[] = {{&S14901[12], 14}, {NULL}};
const Segment SGI333954[] = {{&S14902[6], 4}, {NULL}};
const Segment SGI333955[] = {{&S14902[59], 5}, {NULL}};
const Segment SGI333956[] = {{&S14902[12], 14}, {&S14902[70], 14}, {NULL}};
const Segment SGI333957[] = {{&S14902[12], 14}, {NULL}};
const Segment SGI333958[] = {{&S14903[6], 4}, {NULL}};
const Segment SGI333959[] = {{&S14903[81], 5}, {NULL}};
const Segment SGI333960[] = {{&S14903[12], 14}, {&S14903[40], 11}, {NULL}};
const Segment SGI333961[] = {{&S14903[12], 14}, {NULL}};
const Segment SGI333962[] = {{&S39764[100], 11}, {NULL}};
const Segment SGI333963[] = {{&S39765[100], 21}, {NULL}};
const Segment SGI333964[] = {{&S39766[100], 11}, {NULL}};
const Segment SGI333965[] = {{&S39767[100], 19}, {NULL}};
const Segment SGI333966[] = {{&S39768[100], 19}, {NULL}};
const Segment SGI333967[] = {{&S26839[46], 6}, {NULL}};
const Segment SGI333968[] = {{&S26839[37], 3}, {NULL}};
const Segment SGI333969[] = {{&S26840[46], 7}, {&S26840[148], 7}, {NULL}};
const Segment SGI333970[] = {{&S26840[37], 3}, {NULL}};
const Segment SGI333971[] = {{&S26841[46], 7}, {NULL}};
const Segment SGI333972[] = {{&S26841[37], 3}, {NULL}};
const Segment SGI333973[] = {{&S26842[46], 5}, {NULL}};
const Segment SGI333974[] = {{&S26842[37], 3}, {NULL}};
const Segment SGI333975[] = {{&S39963[5], 3}, {NULL}};
const Segment SGI333976[] = {{&S39964[5], 7}, {NULL}};
const Segment SGI333977[] = {{&S26844[46], 6}, {&S26844[147], 7}, {NULL}};
const Segment SGI333978[] = {{&S26844[37], 3}, {NULL}};
const Segment SGI333979[] = {{&S26845[45], 5}, {&S26845[145], 7}, {NULL}};
const Segment SGI333980[] = {{&S26845[37], 10}, {NULL}};
const Segment SGI333981[] = {{&S26847[46], 5}, {&S26847[146], 7}, {NULL}};
const Segment SGI333982[] = {{&S26847[37], 3}, {NULL}};
const Segment SGI333983[] = {{&S26848[8], 5}, {NULL}};
const Segment SGI333984[] = {{&S26848[37], 3}, {NULL}};
const Segment SGI333985[] = {{&S40048[5], 4}, {NULL}};
const Segment SGI333986[] = {{&S40469[100], 7}, {NULL}};
const Segment SGI333987[] = {{&S40470[100], 11}, {NULL}};
const Segment SGI333988[] = {{&S40744[5], 5}, {NULL}};
const Segment SGI333989[] = {{&S15833[6], 4}, {NULL}};
const Segment SGI333990[] = {{&S15833[59], 5}, {NULL}};
const Segment SGI333991[] = {{&S15833[12], 14}, {NULL}};
const Segment SGI333992[] = {{&S15834[6], 4}, {NULL}};
const Segment SGI333993[] = {{&S15834[52], 5}, {NULL}};
const Segment SGI333994[] = {{&S15834[12], 45}, {NULL}};
const Segment SGI333995[] = {{&S15834[12], 14}, {NULL}};
const Segment SGI333996[] = {{&S15835[6], 4}, {NULL}};
const Segment SGI333997[] = {{&S15835[52], 5}, {NULL}};
const Segment SGI333998[] = {{&S15835[12], 55}, {NULL}};
const Segment SGI333999[] = {{&S15835[12], 14}, {NULL}};
const Segment SGI334000[] = {{&S15836[6], 4}, {NULL}};
const Segment SGI334001[] = {{&S15836[82], 5}, {NULL}};
const Segment SGI334002[] = {{&S15836[12], 14}, {&S15836[45], 42}, {NULL}};
const Segment SGI334003[] = {{&S15836[12], 14}, {NULL}};
const Segment SGI334004[] = {{&S15837[6], 4}, {NULL}};
const Segment SGI334005[] = {{&S15837[74], 5}, {NULL}};
const Segment SGI334006[] = {{&S15837[12], 14}, {&S15837[37], 42}, {NULL}};
const Segment SGI334007[] = {{&S15837[12], 14}, {NULL}};
const Segment SGI334008[] = {{&S15838[6], 4}, {NULL}};
const Segment SGI334009[] = {{&S15838[74], 5}, {NULL}};
const Segment SGI334010[] = {{&S15838[12], 14}, {&S15838[37], 42}, {NULL}};
const Segment SGI334011[] = {{&S15838[12], 14}, {NULL}};
const Segment SGI334012[] = {{&S15839[6], 4}, {NULL}};
const Segment SGI334013[] = {{&S15839[74], 5}, {NULL}};
const Segment SGI334014[] = {{&S15839[12], 14}, {&S15839[37], 42}, {NULL}};
const Segment SGI334015[] = {{&S15839[12], 14}, {NULL}};
const Segment SGI334016[] = {{&S15840[6], 4}, {NULL}};
const Segment SGI334017[] = {{&S15840[74], 5}, {NULL}};
const Segment SGI334018[] = {{&S15840[12], 14}, {&S15840[37], 42}, {NULL}};
const Segment SGI334019[] = {{&S15840[12], 14}, {NULL}};
const Segment SGI334020[] = {{&S15841[6], 4}, {NULL}};
const Segment SGI334021[] = {{&S15841[74], 5}, {NULL}};
const Segment SGI334022[] = {{&S15841[12], 14}, {&S15841[37], 42}, {NULL}};
const Segment SGI334023[] = {{&S15841[12], 14}, {NULL}};
const Segment SGI334024[] = {{&S15843[6], 4}, {NULL}};
const Segment SGI334025[] = {{&S15843[74], 5}, {NULL}};
const Segment SGI334026[] = {{&S15843[12], 14}, {&S15843[37], 42}, {NULL}};
const Segment SGI334027[] = {{&S15843[12], 14}, {NULL}};
const Segment SGI334028[] = {{&S15844[6], 4}, {NULL}};
const Segment SGI334029[] = {{&S15844[74], 5}, {NULL}};
const Segment SGI334030[] = {{&S15844[12], 14}, {&S15844[37], 42}, {NULL}};
const Segment SGI334031[] = {{&S15844[12], 14}, {NULL}};
const Segment SGI334032[] = {{&S15845[6], 4}, {NULL}};
const Segment SGI334033[] = {{&S15845[75], 5}, {NULL}};
const Segment SGI334034[] = {{&S15845[12], 14}, {&S15845[37], 43}, {NULL}};
const Segment SGI334035[] = {{&S15845[12], 14}, {NULL}};
const Segment SGI334036[] = {{&S15846[6], 4}, {NULL}};
const Segment SGI334037[] = {{&S15846[74], 5}, {NULL}};
const Segment SGI334038[] = {{&S15846[12], 14}, {&S15846[37], 42}, {NULL}};
const Segment SGI334039[] = {{&S15846[12], 14}, {NULL}};
const Segment SGI334040[] = {{&S15847[6], 4}, {NULL}};
const Segment SGI334041[] = {{&S15847[81], 5}, {NULL}};
const Segment SGI334042[] = {{&S15847[12], 14}, {&S15847[44], 42}, {NULL}};
const Segment SGI334043[] = {{&S15847[12], 14}, {NULL}};
const Segment SGI334044[] = {{&S15848[6], 4}, {NULL}};
const Segment SGI334045[] = {{&S15848[74], 5}, {NULL}};
const Segment SGI334046[] = {{&S15848[12], 14}, {&S15848[37], 42}, {NULL}};
const Segment SGI334047[] = {{&S15848[12], 14}, {NULL}};
const Segment SGI334048[] = {{&S15849[6], 4}, {NULL}};
const Segment SGI334049[] = {{&S15849[81], 5}, {NULL}};
const Segment SGI334050[] = {{&S15849[12], 14}, {&S15849[44], 42}, {NULL}};
const Segment SGI334051[] = {{&S15849[12], 14}, {NULL}};
const Segment SGI334052[] = {{&S15850[6], 4}, {NULL}};
const Segment SGI334053[] = {{&S15850[82], 5}, {NULL}};
const Segment SGI334054[] = {{&S15850[12], 14}, {&S15850[45], 42}, {NULL}};
const Segment SGI334055[] = {{&S15850[12], 14}, {NULL}};
const Segment SGI334056[] = {{&S15851[6], 4}, {NULL}};
const Segment SGI334057[] = {{&S15851[74], 5}, {NULL}};
const Segment SGI334058[] = {{&S15851[12], 14}, {&S15851[37], 42}, {NULL}};
const Segment SGI334059[] = {{&S15851[12], 14}, {NULL}};
const Segment SGI334060[] = {{&S15853[6], 4}, {NULL}};
const Segment SGI334061[] = {{&S15853[82], 5}, {NULL}};
const Segment SGI334062[] = {{&S15853[12], 14}, {&S15853[45], 42}, {NULL}};
const Segment SGI334063[] = {{&S15853[12], 14}, {NULL}};
const Segment SGI334064[] = {{&S15854[6], 4}, {NULL}};
const Segment SGI334065[] = {{&S15854[74], 5}, {NULL}};
const Segment SGI334066[] = {{&S15854[12], 14}, {&S15854[37], 42}, {NULL}};
const Segment SGI334067[] = {{&S15854[12], 14}, {NULL}};
const Segment SGI334068[] = {{&S15855[6], 4}, {NULL}};
const Segment SGI334069[] = {{&S15855[81], 5}, {NULL}};
const Segment SGI334070[] = {{&S15855[12], 14}, {&S15855[44], 42}, {NULL}};
const Segment SGI334071[] = {{&S15855[12], 14}, {NULL}};
const Segment SGI334072[] = {{&S15856[6], 4}, {NULL}};
const Segment SGI334073[] = {{&S15856[81], 5}, {NULL}};
const Segment SGI334074[] = {{&S15856[12], 14}, {&S15856[44], 42}, {NULL}};
const Segment SGI334075[] = {{&S15856[12], 14}, {NULL}};
const Segment SGI334076[] = {{&S15857[6], 4}, {NULL}};
const Segment SGI334077[] = {{&S15857[81], 5}, {NULL}};
const Segment SGI334078[] = {{&S15857[12], 14}, {&S15857[44], 42}, {NULL}};
const Segment SGI334079[] = {{&S15857[12], 14}, {NULL}};
const Segment SGI334080[] = {{&S15858[6], 4}, {NULL}};
const Segment SGI334081[] = {{&S15858[81], 5}, {NULL}};
const Segment SGI334082[] = {{&S15858[12], 14}, {&S15858[44], 42}, {NULL}};
const Segment SGI334083[] = {{&S15858[12], 14}, {NULL}};
const Segment SGI334084[] = {{&S15859[6], 4}, {NULL}};
const Segment SGI334085[] = {{&S15859[81], 5}, {NULL}};
const Segment SGI334086[] = {{&S15859[12], 14}, {&S15859[44], 42}, {NULL}};
const Segment SGI334087[] = {{&S15859[12], 14}, {NULL}};
const Segment SGI334088[] = {{&S15860[6], 4}, {NULL}};
const Segment SGI334089[] = {{&S15860[81], 5}, {NULL}};
const Segment SGI334090[] = {{&S15860[12], 14}, {&S15860[44], 42}, {NULL}};
const Segment SGI334091[] = {{&S15860[12], 14}, {NULL}};
const Segment SGI334092[] = {{&S15861[6], 4}, {NULL}};
const Segment SGI334093[] = {{&S15861[75], 5}, {NULL}};
const Segment SGI334094[] = {{&S15861[12], 14}, {&S15861[37], 43}, {NULL}};
const Segment SGI334095[] = {{&S15861[12], 14}, {NULL}};
const Segment SGI334096[] = {{&S15862[6], 4}, {NULL}};
const Segment SGI334097[] = {{&S15862[81], 5}, {NULL}};
const Segment SGI334098[] = {{&S15862[12], 14}, {&S15862[44], 42}, {NULL}};
const Segment SGI334099[] = {{&S15862[12], 14}, {NULL}};
const Segment SGI334100[] = {{&S15863[6], 4}, {NULL}};
const Segment SGI334101[] = {{&S15863[74], 5}, {NULL}};
const Segment SGI334102[] = {{&S15863[12], 14}, {&S15863[37], 42}, {NULL}};
const Segment SGI334103[] = {{&S15863[12], 14}, {NULL}};
const Segment SGI334104[] = {{&S15864[6], 4}, {NULL}};
const Segment SGI334105[] = {{&S15864[75], 5}, {NULL}};
const Segment SGI334106[] = {{&S15864[12], 14}, {&S15864[38], 42}, {NULL}};
const Segment SGI334107[] = {{&S15864[12], 14}, {NULL}};
const Segment SGI334108[] = {{&S15865[6], 4}, {NULL}};
const Segment SGI334109[] = {{&S15865[81], 5}, {NULL}};
const Segment SGI334110[] = {{&S15865[12], 14}, {&S15865[44], 42}, {NULL}};
const Segment SGI334111[] = {{&S15865[12], 14}, {NULL}};
const Segment SGI334112[] = {{&S15866[6], 4}, {NULL}};
const Segment SGI334113[] = {{&S15866[81], 5}, {NULL}};
const Segment SGI334114[] = {{&S15866[12], 14}, {&S15866[44], 42}, {NULL}};
const Segment SGI334115[] = {{&S15866[12], 14}, {NULL}};
const Segment SGI334116[] = {{&S15867[6], 4}, {NULL}};
const Segment SGI334117[] = {{&S15867[82], 5}, {NULL}};
const Segment SGI334118[] = {{&S15867[12], 14}, {&S15867[45], 42}, {NULL}};
const Segment SGI334119[] = {{&S15867[12], 14}, {NULL}};
const Segment SGI334120[] = {{&S15868[6], 4}, {NULL}};
const Segment SGI334121[] = {{&S40800[5], 5}, {&S40800[68], 5}, {NULL}};
const Segment SGI334122[] = {{&S15868[59], 5}, {NULL}};
const Segment SGI334123[] = {{&S15868[12], 14}, {&S15868[70], 4}, {NULL}};
const Segment SGI334124[] = {{&S15868[12], 14}, {NULL}};
const Segment SGI334125[] = {{&S15869[6], 4}, {NULL}};
const Segment SGI334126[] = {{&S40801[5], 5}, {NULL}};
const Segment SGI334127[] = {{&S15869[81], 5}, {NULL}};
const Segment SGI334128[] = {{&S15869[12], 14}, {&S15869[44], 42}, {NULL}};
const Segment SGI334129[] = {{&S15869[12], 14}, {NULL}};
const Segment SGI334130[] = {{&S15870[6], 4}, {NULL}};
const Segment SGI334131[] = {{&S40802[5], 10}, {NULL}};
const Segment SGI334132[] = {{&S15870[81], 5}, {NULL}};
const Segment SGI334133[] = {{&S15870[12], 14}, {&S15870[44], 42}, {NULL}};
const Segment SGI334134[] = {{&S15870[12], 14}, {NULL}};
const Segment SGI334135[] = {{&S40803[5], 7}, {NULL}};
const Segment SGI334136[] = {{&S15871[6], 4}, {NULL}};
const Segment SGI334137[] = {{&S15871[81], 5}, {NULL}};
const Segment SGI334138[] = {{&S15871[12], 14}, {&S15871[44], 42}, {NULL}};
const Segment SGI334139[] = {{&S15871[12], 14}, {NULL}};
const Segment SGI334140[] = {{&S40804[5], 5}, {&S40804[5], 5}, {NULL}};
const Segment SGI334141[] = {{&S15872[6], 4}, {NULL}};
const Segment SGI334142[] = {{&S15872[81], 5}, {NULL}};
const Segment SGI334143[] = {{&S15872[12], 14}, {&S15872[44], 42}, {NULL}};
const Segment SGI334144[] = {{&S15872[12], 14}, {NULL}};
const Segment SGI334145[] = {{&S15873[6], 4}, {NULL}};
const Segment SGI334146[] = {{&S15873[74], 5}, {NULL}};
const Segment SGI334147[] = {{&S15873[12], 14}, {&S15873[37], 42}, {NULL}};
const Segment SGI334148[] = {{&S15873[12], 14}, {NULL}};
const Segment SGI334149[] = {{&S15874[6], 4}, {NULL}};
const Segment SGI334150[] = {{&S15874[74], 5}, {NULL}};
const Segment SGI334151[] = {{&S15874[12], 14}, {&S15874[37], 42}, {NULL}};
const Segment SGI334152[] = {{&S15874[12], 14}, {NULL}};
const Segment SGI334153[] = {{&S15875[6], 4}, {NULL}};
const Segment SGI334154[] = {{&S15875[75], 5}, {NULL}};
const Segment SGI334155[] = {{&S15875[12], 14}, {&S15875[37], 43}, {NULL}};
const Segment SGI334156[] = {{&S15875[12], 14}, {NULL}};
const Segment SGI334157[] = {{&S15876[6], 4}, {NULL}};
const Segment SGI334158[] = {{&S15876[74], 5}, {NULL}};
const Segment SGI334159[] = {{&S15876[12], 14}, {&S15876[37], 42}, {NULL}};
const Segment SGI334160[] = {{&S15876[12], 14}, {NULL}};
const Segment SGI334161[] = {{&S15877[6], 4}, {NULL}};
const Segment SGI334162[] = {{&S15877[81], 5}, {NULL}};
const Segment SGI334163[] = {{&S15877[12], 14}, {&S15877[44], 42}, {NULL}};
const Segment SGI334164[] = {{&S15877[12], 14}, {NULL}};
const Segment SGI334165[] = {{&S15878[6], 4}, {NULL}};
const Segment SGI334166[] = {{&S15878[81], 5}, {NULL}};
const Segment SGI334167[] = {{&S15878[12], 14}, {&S15878[44], 42}, {NULL}};
const Segment SGI334168[] = {{&S15878[12], 14}, {NULL}};
const Segment SGI334169[] = {{&S15879[6], 4}, {NULL}};
const Segment SGI334170[] = {{&S15879[75], 5}, {NULL}};
const Segment SGI334171[] = {{&S15879[12], 14}, {&S15879[38], 42}, {NULL}};
const Segment SGI334172[] = {{&S15879[12], 14}, {NULL}};
const Segment SGI334173[] = {{&S15880[6], 4}, {NULL}};
const Segment SGI334174[] = {{&S15880[82], 5}, {NULL}};
const Segment SGI334175[] = {{&S15880[12], 14}, {&S15880[45], 42}, {NULL}};
const Segment SGI334176[] = {{&S15880[12], 14}, {NULL}};
const Segment SGI334177[] = {{&S15881[6], 4}, {NULL}};
const Segment SGI334178[] = {{&S15881[74], 5}, {NULL}};
const Segment SGI334179[] = {{&S15881[12], 14}, {&S15881[37], 42}, {NULL}};
const Segment SGI334180[] = {{&S15881[12], 14}, {NULL}};
const Segment SGI334181[] = {{&S15882[6], 4}, {NULL}};
const Segment SGI334182[] = {{&S15882[75], 5}, {NULL}};
const Segment SGI334183[] = {{&S15882[12], 14}, {&S15882[37], 43}, {NULL}};
const Segment SGI334184[] = {{&S15882[12], 14}, {NULL}};
const Segment SGI334185[] = {{&S15883[6], 4}, {NULL}};
const Segment SGI334186[] = {{&S15883[75], 5}, {NULL}};
const Segment SGI334187[] = {{&S15883[12], 14}, {&S15883[38], 42}, {NULL}};
const Segment SGI334188[] = {{&S15883[12], 14}, {NULL}};
const Segment SGI334189[] = {{&S15884[6], 4}, {NULL}};
const Segment SGI334190[] = {{&S15884[75], 5}, {NULL}};
const Segment SGI334191[] = {{&S15884[12], 14}, {&S15884[37], 43}, {NULL}};
const Segment SGI334192[] = {{&S15884[12], 14}, {NULL}};
const Segment SGI334193[] = {{&S15885[6], 4}, {NULL}};
const Segment SGI334194[] = {{&S15885[74], 5}, {NULL}};
const Segment SGI334195[] = {{&S15885[12], 14}, {&S15885[37], 42}, {NULL}};
const Segment SGI334196[] = {{&S15885[12], 14}, {NULL}};
const Segment SGI334197[] = {{&S15886[6], 4}, {NULL}};
const Segment SGI334198[] = {{&S15886[74], 5}, {NULL}};
const Segment SGI334199[] = {{&S15886[12], 14}, {&S15886[37], 42}, {NULL}};
const Segment SGI334200[] = {{&S15886[12], 14}, {NULL}};
const Segment SGI334201[] = {{&S15887[6], 4}, {NULL}};
const Segment SGI334202[] = {{&S15887[74], 5}, {NULL}};
const Segment SGI334203[] = {{&S15887[12], 14}, {&S15887[37], 42}, {NULL}};
const Segment SGI334204[] = {{&S15887[12], 14}, {NULL}};
const Segment SGI334205[] = {{&S15888[6], 4}, {NULL}};
const Segment SGI334206[] = {{&S15888[75], 5}, {NULL}};
const Segment SGI334207[] = {{&S15888[12], 14}, {&S15888[38], 42}, {NULL}};
const Segment SGI334208[] = {{&S15888[12], 14}, {NULL}};
const Segment SGI334209[] = {{&S15889[6], 4}, {NULL}};
const Segment SGI334210[] = {{&S15889[52], 5}, {NULL}};
const Segment SGI334211[] = {{&S15889[12], 62}, {NULL}};
const Segment SGI334212[] = {{&S15889[12], 14}, {NULL}};
const Segment SGI334213[] = {{&S15890[6], 4}, {NULL}};
const Segment SGI334214[] = {{&S15890[75], 5}, {NULL}};
const Segment SGI334215[] = {{&S15890[12], 14}, {&S15890[37], 43}, {NULL}};
const Segment SGI334216[] = {{&S15890[12], 14}, {NULL}};
const Segment SGI334217[] = {{&S15891[6], 4}, {NULL}};
const Segment SGI334218[] = {{&S15891[75], 5}, {NULL}};
const Segment SGI334219[] = {{&S15891[12], 14}, {&S15891[37], 43}, {NULL}};
const Segment SGI334220[] = {{&S15891[12], 14}, {NULL}};
const Segment SGI334221[] = {{&S15892[6], 4}, {NULL}};
const Segment SGI334222[] = {{&S15892[81], 5}, {NULL}};
const Segment SGI334223[] = {{&S15892[12], 14}, {&S15892[44], 42}, {NULL}};
const Segment SGI334224[] = {{&S15892[12], 14}, {NULL}};
const Segment SGI334225[] = {{&S15893[6], 4}, {NULL}};
const Segment SGI334226[] = {{&S15893[82], 5}, {NULL}};
const Segment SGI334227[] = {{&S15893[12], 14}, {&S15893[44], 43}, {NULL}};
const Segment SGI334228[] = {{&S15893[12], 14}, {NULL}};
const Segment SGI334229[] = {{&S15894[6], 4}, {NULL}};
const Segment SGI334230[] = {{&S15894[82], 5}, {NULL}};
const Segment SGI334231[] = {{&S15894[12], 14}, {&S15894[45], 42}, {NULL}};
const Segment SGI334232[] = {{&S15894[12], 14}, {NULL}};
const Segment SGI334233[] = {{&S15896[6], 4}, {NULL}};
const Segment SGI334234[] = {{&S15896[74], 5}, {NULL}};
const Segment SGI334235[] = {{&S15896[12], 14}, {&S15896[37], 42}, {NULL}};
const Segment SGI334236[] = {{&S15896[12], 14}, {NULL}};
const Segment SGI334237[] = {{&S15897[6], 4}, {NULL}};
const Segment SGI334238[] = {{&S15897[82], 5}, {NULL}};
const Segment SGI334239[] = {{&S15897[12], 14}, {&S15897[45], 42}, {NULL}};
const Segment SGI334240[] = {{&S15897[12], 14}, {NULL}};
const Segment SGI334241[] = {{&S15898[6], 4}, {NULL}};
const Segment SGI334242[] = {{&S15898[74], 5}, {NULL}};
const Segment SGI334243[] = {{&S15898[12], 14}, {&S15898[37], 42}, {NULL}};
const Segment SGI334244[] = {{&S15898[12], 14}, {NULL}};
const Segment SGI334245[] = {{&S15899[6], 4}, {NULL}};
const Segment SGI334246[] = {{&S15899[82], 5}, {NULL}};
const Segment SGI334247[] = {{&S15899[12], 14}, {&S15899[44], 43}, {NULL}};
const Segment SGI334248[] = {{&S15899[12], 14}, {NULL}};
const Segment SGI334249[] = {{&S15900[6], 4}, {NULL}};
const Segment SGI334250[] = {{&S15900[75], 5}, {NULL}};
const Segment SGI334251[] = {{&S15900[12], 14}, {&S15900[37], 43}, {NULL}};
const Segment SGI334252[] = {{&S15900[12], 14}, {NULL}};
const Segment SGI334253[] = {{&S15901[6], 4}, {NULL}};
const Segment SGI334254[] = {{&S15901[74], 5}, {NULL}};
const Segment SGI334255[] = {{&S15901[12], 14}, {&S15901[37], 42}, {NULL}};
const Segment SGI334256[] = {{&S15901[12], 14}, {NULL}};
const Segment SGI334257[] = {{&S15902[6], 4}, {NULL}};
const Segment SGI334258[] = {{&S15902[74], 5}, {NULL}};
const Segment SGI334259[] = {{&S15902[12], 14}, {&S15902[37], 42}, {NULL}};
const Segment SGI334260[] = {{&S15902[12], 14}, {NULL}};
const Segment SGI334261[] = {{&S15903[6], 4}, {NULL}};
const Segment SGI334262[] = {{&S15903[81], 5}, {NULL}};
const Segment SGI334263[] = {{&S15903[12], 14}, {&S15903[44], 42}, {NULL}};
const Segment SGI334264[] = {{&S15903[12], 14}, {NULL}};
const Segment SGI334265[] = {{&S15904[6], 4}, {NULL}};
const Segment SGI334266[] = {{&S15904[75], 5}, {NULL}};
const Segment SGI334267[] = {{&S15904[12], 14}, {&S15904[38], 42}, {NULL}};
const Segment SGI334268[] = {{&S15904[12], 14}, {NULL}};
const Segment SGI334269[] = {{&S15905[6], 4}, {NULL}};
const Segment SGI334270[] = {{&S15905[82], 5}, {NULL}};
const Segment SGI334271[] = {{&S15905[12], 14}, {&S15905[45], 42}, {NULL}};
const Segment SGI334272[] = {{&S15905[12], 14}, {NULL}};
const Segment SGI334273[] = {{&S15906[6], 4}, {NULL}};
const Segment SGI334274[] = {{&S15906[75], 5}, {NULL}};
const Segment SGI334275[] = {{&S15906[12], 14}, {&S15906[37], 43}, {NULL}};
const Segment SGI334276[] = {{&S15906[12], 14}, {NULL}};
const Segment SGI334277[] = {{&S15907[6], 4}, {NULL}};
const Segment SGI334278[] = {{&S15907[82], 5}, {NULL}};
const Segment SGI334279[] = {{&S15907[12], 14}, {&S15907[45], 42}, {NULL}};
const Segment SGI334280[] = {{&S15907[12], 14}, {NULL}};
const Segment SGI334281[] = {{&S15908[6], 4}, {NULL}};
const Segment SGI334282[] = {{&S15908[74], 5}, {NULL}};
const Segment SGI334283[] = {{&S15908[12], 14}, {&S15908[37], 42}, {NULL}};
const Segment SGI334284[] = {{&S15908[12], 14}, {NULL}};
const Segment SGI334285[] = {{&S15909[6], 4}, {NULL}};
const Segment SGI334286[] = {{&S15909[74], 5}, {NULL}};
const Segment SGI334287[] = {{&S15909[12], 14}, {&S15909[37], 42}, {NULL}};
const Segment SGI334288[] = {{&S15909[12], 14}, {NULL}};
const Segment SGI334289[] = {{&S15910[6], 4}, {NULL}};
const Segment SGI334290[] = {{&S15910[75], 5}, {NULL}};
const Segment SGI334291[] = {{&S15910[12], 14}, {&S15910[38], 42}, {NULL}};
const Segment SGI334292[] = {{&S15910[12], 14}, {NULL}};
const Segment SGI334293[] = {{&S15911[6], 4}, {NULL}};
const Segment SGI334294[] = {{&S15911[82], 5}, {NULL}};
const Segment SGI334295[] = {{&S15911[12], 14}, {&S15911[45], 42}, {NULL}};
const Segment SGI334296[] = {{&S15911[12], 14}, {NULL}};
const Segment SGI334297[] = {{&S15912[6], 4}, {NULL}};
const Segment SGI334298[] = {{&S15912[59], 5}, {NULL}};
const Segment SGI334299[] = {{&S15912[12], 14}, {&S15912[70], 33}, {NULL}};
const Segment SGI334300[] = {{&S15912[12], 14}, {NULL}};
const Segment SGI334301[] = {{&S15913[6], 4}, {NULL}};
const Segment SGI334302[] = {{&S15913[75], 5}, {NULL}};
const Segment SGI334303[] = {{&S15913[12], 14}, {&S15913[38], 42}, {NULL}};
const Segment SGI334304[] = {{&S15913[12], 14}, {NULL}};
const Segment SGI334305[] = {{&S15914[6], 4}, {NULL}};
const Segment SGI334306[] = {{&S15914[82], 5}, {NULL}};
const Segment SGI334307[] = {{&S15914[12], 14}, {&S15914[45], 42}, {NULL}};
const Segment SGI334308[] = {{&S15914[12], 14}, {NULL}};
const Segment SGI334309[] = {{&S15915[6], 4}, {NULL}};
const Segment SGI334310[] = {{&S15915[81], 5}, {NULL}};
const Segment SGI334311[] = {{&S15915[12], 14}, {&S15915[44], 42}, {NULL}};
const Segment SGI334312[] = {{&S15915[12], 14}, {NULL}};
const Segment SGI334313[] = {{&S15916[6], 4}, {NULL}};
const Segment SGI334314[] = {{&S15916[82], 5}, {NULL}};
const Segment SGI334315[] = {{&S15916[12], 14}, {&S15916[45], 42}, {NULL}};
const Segment SGI334316[] = {{&S15916[12], 14}, {NULL}};
const Segment SGI334317[] = {{&S15917[6], 4}, {NULL}};
const Segment SGI334318[] = {{&S15917[81], 5}, {NULL}};
const Segment SGI334319[] = {{&S15917[12], 14}, {&S15917[44], 42}, {NULL}};
const Segment SGI334320[] = {{&S15917[12], 14}, {NULL}};
const Segment SGI334321[] = {{&S15918[6], 4}, {NULL}};
const Segment SGI334322[] = {{&S15918[82], 5}, {NULL}};
const Segment SGI334323[] = {{&S15918[12], 14}, {&S15918[44], 43}, {NULL}};
const Segment SGI334324[] = {{&S15918[12], 14}, {NULL}};
const Segment SGI334325[] = {{&S15919[6], 4}, {NULL}};
const Segment SGI334326[] = {{&S15919[74], 5}, {NULL}};
const Segment SGI334327[] = {{&S15919[12], 14}, {&S15919[37], 42}, {NULL}};
const Segment SGI334328[] = {{&S15919[12], 14}, {NULL}};
const Segment SGI334329[] = {{&S15920[6], 4}, {NULL}};
const Segment SGI334330[] = {{&S15920[81], 5}, {NULL}};
const Segment SGI334331[] = {{&S15920[12], 14}, {&S15920[44], 42}, {NULL}};
const Segment SGI334332[] = {{&S15920[12], 14}, {NULL}};
const Segment SGI334333[] = {{&S15921[6], 4}, {NULL}};
const Segment SGI334334[] = {{&S15921[81], 5}, {NULL}};
const Segment SGI334335[] = {{&S15921[12], 14}, {&S15921[44], 42}, {NULL}};
const Segment SGI334336[] = {{&S15921[12], 14}, {NULL}};
const Segment SGI334337[] = {{&S15922[6], 4}, {NULL}};
const Segment SGI334338[] = {{&S15922[81], 5}, {NULL}};
const Segment SGI334339[] = {{&S15922[12], 14}, {&S15922[44], 42}, {NULL}};
const Segment SGI334340[] = {{&S15922[12], 14}, {NULL}};
const Segment SGI334341[] = {{&S15923[6], 4}, {NULL}};
const Segment SGI334342[] = {{&S15923[74], 5}, {NULL}};
const Segment SGI334343[] = {{&S15923[12], 14}, {&S15923[37], 42}, {NULL}};
const Segment SGI334344[] = {{&S15923[12], 14}, {NULL}};
const Segment SGI334345[] = {{&S15924[6], 4}, {NULL}};
const Segment SGI334346[] = {{&S15924[81], 5}, {NULL}};
const Segment SGI334347[] = {{&S15924[12], 14}, {&S15924[44], 42}, {NULL}};
const Segment SGI334348[] = {{&S15924[12], 14}, {NULL}};
const Segment SGI334349[] = {{&S15925[6], 4}, {NULL}};
const Segment SGI334350[] = {{&S15925[81], 5}, {NULL}};
const Segment SGI334351[] = {{&S15925[12], 14}, {&S15925[44], 42}, {NULL}};
const Segment SGI334352[] = {{&S15925[12], 14}, {NULL}};
const Segment SGI334353[] = {{&S15926[6], 4}, {NULL}};
const Segment SGI334354[] = {{&S15926[74], 5}, {NULL}};
const Segment SGI334355[] = {{&S15926[12], 14}, {&S15926[37], 42}, {NULL}};
const Segment SGI334356[] = {{&S15926[12], 14}, {NULL}};
const Segment SGI334357[] = {{&S15927[6], 4}, {NULL}};
const Segment SGI334358[] = {{&S15927[74], 5}, {NULL}};
const Segment SGI334359[] = {{&S15927[12], 14}, {&S15927[37], 42}, {NULL}};
const Segment SGI334360[] = {{&S15927[12], 14}, {NULL}};
const Segment SGI334361[] = {{&S15928[6], 4}, {NULL}};
const Segment SGI334362[] = {{&S15928[81], 5}, {NULL}};
const Segment SGI334363[] = {{&S15928[12], 14}, {&S15928[44], 42}, {NULL}};
const Segment SGI334364[] = {{&S15928[12], 14}, {NULL}};
const Segment SGI334365[] = {{&S15929[6], 4}, {NULL}};
const Segment SGI334366[] = {{&S15929[74], 5}, {NULL}};
const Segment SGI334367[] = {{&S15929[12], 14}, {&S15929[37], 42}, {NULL}};
const Segment SGI334368[] = {{&S15929[12], 14}, {NULL}};
const Segment SGI334369[] = {{&S15930[6], 4}, {NULL}};
const Segment SGI334370[] = {{&S15930[74], 5}, {NULL}};
const Segment SGI334371[] = {{&S15930[12], 14}, {&S15930[37], 42}, {NULL}};
const Segment SGI334372[] = {{&S15930[12], 14}, {NULL}};
const Segment SGI334373[] = {{&S15931[6], 4}, {NULL}};
const Segment SGI334374[] = {{&S15931[74], 5}, {NULL}};
const Segment SGI334375[] = {{&S15931[12], 14}, {&S15931[37], 42}, {NULL}};
const Segment SGI334376[] = {{&S15931[12], 14}, {NULL}};
const Segment SGI334377[] = {{&S15932[6], 4}, {NULL}};
const Segment SGI334378[] = {{&S15932[81], 5}, {NULL}};
const Segment SGI334379[] = {{&S15932[12], 14}, {&S15932[44], 42}, {NULL}};
const Segment SGI334380[] = {{&S15932[12], 14}, {NULL}};
const Segment SGI334381[] = {{&S15933[6], 4}, {NULL}};
const Segment SGI334382[] = {{&S15933[74], 5}, {NULL}};
const Segment SGI334383[] = {{&S15933[12], 14}, {&S15933[37], 42}, {NULL}};
const Segment SGI334384[] = {{&S15933[12], 14}, {NULL}};
const Segment SGI334385[] = {{&S15934[6], 4}, {NULL}};
const Segment SGI334386[] = {{&S15934[81], 5}, {NULL}};
const Segment SGI334387[] = {{&S15934[12], 14}, {&S15934[44], 42}, {NULL}};
const Segment SGI334388[] = {{&S15934[12], 14}, {NULL}};
const Segment SGI334389[] = {{&S15935[6], 4}, {NULL}};
const Segment SGI334390[] = {{&S15935[81], 5}, {NULL}};
const Segment SGI334391[] = {{&S15935[12], 14}, {&S15935[44], 42}, {NULL}};
const Segment SGI334392[] = {{&S15935[12], 14}, {NULL}};
const Segment SGI334393[] = {{&S15936[6], 4}, {NULL}};
const Segment SGI334394[] = {{&S15936[74], 5}, {NULL}};
const Segment SGI334395[] = {{&S15936[12], 14}, {&S15936[37], 42}, {NULL}};
const Segment SGI334396[] = {{&S15936[12], 14}, {NULL}};
const Segment SGI334397[] = {{&S15937[6], 4}, {NULL}};
const Segment SGI334398[] = {{&S15937[74], 5}, {NULL}};
const Segment SGI334399[] = {{&S15937[12], 14}, {&S15937[37], 42}, {NULL}};
const Segment SGI334400[] = {{&S15937[12], 14}, {NULL}};
const Segment SGI334401[] = {{&S15938[6], 4}, {NULL}};
const Segment SGI334402[] = {{&S15938[75], 5}, {NULL}};
const Segment SGI334403[] = {{&S15938[12], 14}, {&S15938[37], 43}, {NULL}};
const Segment SGI334404[] = {{&S15938[12], 14}, {NULL}};
const Segment SGI334405[] = {{&S15939[6], 4}, {NULL}};
const Segment SGI334406[] = {{&S15939[81], 5}, {NULL}};
const Segment SGI334407[] = {{&S15939[12], 14}, {&S15939[44], 42}, {NULL}};
const Segment SGI334408[] = {{&S15939[12], 14}, {NULL}};
const Segment SGI334409[] = {{&S15940[6], 4}, {NULL}};
const Segment SGI334410[] = {{&S15940[81], 5}, {NULL}};
const Segment SGI334411[] = {{&S15940[12], 14}, {&S15940[44], 42}, {NULL}};
const Segment SGI334412[] = {{&S15940[12], 14}, {NULL}};
const Segment SGI334413[] = {{&S15941[6], 4}, {NULL}};
const Segment SGI334414[] = {{&S15941[52], 5}, {NULL}};
const Segment SGI334415[] = {{&S15941[12], 84}, {NULL}};
const Segment SGI334416[] = {{&S15941[12], 14}, {NULL}};
const Segment SGI334417[] = {{&S15942[6], 4}, {NULL}};
const Segment SGI334418[] = {{&S15942[101], 5}, {NULL}};
const Segment SGI334419[] = {{&S15942[12], 14}, {&S15942[37], 12}, {NULL}};
const Segment SGI334420[] = {{&S15942[12], 14}, {NULL}};
const Segment SGI334421[] = {{&S15943[6], 4}, {NULL}};
const Segment SGI334422[] = {{&S15943[81], 5}, {NULL}};
const Segment SGI334423[] = {{&S15943[12], 14}, {&S15943[44], 42}, {NULL}};
const Segment SGI334424[] = {{&S15943[12], 14}, {NULL}};
const Segment SGI334425[] = {{&S15944[6], 4}, {NULL}};
const Segment SGI334426[] = {{&S15944[75], 5}, {NULL}};
const Segment SGI334427[] = {{&S15944[12], 14}, {&S15944[37], 43}, {NULL}};
const Segment SGI334428[] = {{&S15944[12], 14}, {NULL}};
const Segment SGI334429[] = {{&S15945[6], 4}, {NULL}};
const Segment SGI334430[] = {{&S15945[75], 5}, {NULL}};
const Segment SGI334431[] = {{&S15945[12], 14}, {&S15945[37], 43}, {NULL}};
const Segment SGI334432[] = {{&S15945[12], 14}, {NULL}};
const Segment SGI334433[] = {{&S15946[6], 4}, {NULL}};
const Segment SGI334434[] = {{&S15946[74], 5}, {NULL}};
const Segment SGI334435[] = {{&S15946[12], 14}, {&S15946[37], 42}, {NULL}};
const Segment SGI334436[] = {{&S15946[12], 14}, {NULL}};
const Segment SGI334437[] = {{&S15947[6], 4}, {NULL}};
const Segment SGI334438[] = {{&S15947[74], 5}, {NULL}};
const Segment SGI334439[] = {{&S15947[12], 14}, {&S15947[37], 42}, {NULL}};
const Segment SGI334440[] = {{&S15947[12], 14}, {NULL}};
const Segment SGI334441[] = {{&S15948[6], 4}, {NULL}};
const Segment SGI334442[] = {{&S15948[81], 5}, {NULL}};
const Segment SGI334443[] = {{&S15948[12], 14}, {&S15948[44], 42}, {NULL}};
const Segment SGI334444[] = {{&S15948[12], 14}, {NULL}};
const Segment SGI334445[] = {{&S15949[6], 4}, {NULL}};
const Segment SGI334446[] = {{&S15949[74], 5}, {NULL}};
const Segment SGI334447[] = {{&S15949[12], 14}, {&S15949[37], 42}, {NULL}};
const Segment SGI334448[] = {{&S15949[12], 14}, {NULL}};
const Segment SGI334449[] = {{&S15950[6], 4}, {NULL}};
const Segment SGI334450[] = {{&S15950[81], 5}, {NULL}};
const Segment SGI334451[] = {{&S15950[12], 14}, {&S15950[44], 42}, {NULL}};
const Segment SGI334452[] = {{&S15950[12], 14}, {NULL}};
const Segment SGI334453[] = {{&S15951[6], 4}, {NULL}};
const Segment SGI334454[] = {{&S15951[74], 5}, {NULL}};
const Segment SGI334455[] = {{&S15951[12], 14}, {&S15951[37], 42}, {NULL}};
const Segment SGI334456[] = {{&S15951[12], 14}, {NULL}};
const Segment SGI334457[] = {{&S15952[6], 4}, {NULL}};
const Segment SGI334458[] = {{&S15952[81], 5}, {NULL}};
const Segment SGI334459[] = {{&S15952[12], 14}, {&S15952[44], 42}, {NULL}};
const Segment SGI334460[] = {{&S15952[12], 14}, {NULL}};
const Segment SGI334461[] = {{&S15953[6], 4}, {NULL}};
const Segment SGI334462[] = {{&S15953[81], 5}, {NULL}};
const Segment SGI334463[] = {{&S15953[12], 14}, {&S15953[44], 42}, {NULL}};
const Segment SGI334464[] = {{&S15953[12], 14}, {NULL}};
const Segment SGI334465[] = {{&S41353[5], 3}, {NULL}};
const Segment SGI334466[] = {{&S28022[5], 4}, {NULL}};
const Segment SGI334467[] = {{&S28022[15], 3}, {NULL}};
const Segment SGI334468[] = {{&S28023[48], 12}, {NULL}};
const Segment SGI334469[] = {{&S28023[39], 3}, {NULL}};
const Segment SGI334470[] = {{&S28024[49], 5}, {NULL}};
const Segment SGI334471[] = {{&S28024[40], 3}, {NULL}};
const Segment SGI334472[] = {{&S28025[5], 3}, {NULL}};
const Segment SGI334473[] = {{&S28025[49], 3}, {NULL}};
const Segment SGI334474[] = {{&S28026[49], 5}, {&S28026[49], 5}, {NULL}};
const Segment SGI334475[] = {{&S28026[40], 3}, {NULL}};
const Segment SGI334476[] = {{&S28027[49], 5}, {NULL}};
const Segment SGI334477[] = {{&S28027[40], 3}, {NULL}};
const Segment SGI334478[] = {{&S28028[49], 5}, {NULL}};
const Segment SGI334479[] = {{&S28028[40], 3}, {NULL}};
const Segment SGI334480[] = {{&S41736[5], 3}, {NULL}};
const Segment SGI334481[] = {{&S14998[0], 21}, {NULL}};
const Segment SGI334482[] = {{&S14998[34], 3}, {NULL}};
const Segment SGI334483[] = {{&S14998[75], 13}, {NULL}};
const Segment SGI334484[] = {{&S14999[0], 21}, {NULL}};
const Segment SGI334485[] = {{&S14999[34], 3}, {NULL}};
const Segment SGI334486[] = {{&S14999[75], 13}, {NULL}};
const Segment SGI334487[] = {{&S41996[5], 4}, {NULL}};
const Segment SGI334488[] = {{&S41996[15], 3}, {NULL}};
const Segment SGI334489[] = {{&S41997[35], 4}, {NULL}};
const Segment SGI334490[] = {{&S41997[90], 3}, {NULL}};
const Segment SGI334491[] = {{&S41998[5], 13}, {NULL}};
const Segment SGI334492[] = {{&S41998[19], 3}, {NULL}};
const Segment SGI334493[] = {{&S41999[35], 5}, {NULL}};
const Segment SGI334494[] = {{&S41999[91], 3}, {NULL}};
const Segment SGI334495[] = {{&S42000[41], 4}, {NULL}};
const Segment SGI334496[] = {{&S42000[6], 3}, {NULL}};
const Segment SGI334497[] = {{&S42000[66], 3}, {NULL}};
const Segment SGI334498[] = {{&S18358[61], 4}, {NULL}};
const Segment SGI334499[] = {{&S18358[13], 10}, {NULL}};
const Segment SGI334500[] = {{&S18361[61], 4}, {NULL}};
const Segment SGI334501[] = {{&S18361[13], 10}, {NULL}};
const Segment SGI334502[] = {{&S18362[42], 4}, {NULL}};
const Segment SGI334503[] = {{&S18362[13], 14}, {NULL}};
const Segment SGI334504[] = {{&S18363[42], 4}, {NULL}};
const Segment SGI334505[] = {{&S18363[13], 14}, {NULL}};
const Segment SGI334506[] = {{&S18364[61], 3}, {NULL}};
const Segment SGI334507[] = {{&S18364[13], 10}, {NULL}};
const Segment SGI334508[] = {{&S18366[61], 4}, {NULL}};
const Segment SGI334509[] = {{&S18366[13], 10}, {NULL}};
const Segment SGI334510[] = {{&S18375[61], 3}, {&S18375[61], 3}, {NULL}};
const Segment SGI334511[] = {{&S18375[13], 10}, {NULL}};
const Segment SGI334512[] = {{&S18377[42], 4}, {NULL}};
const Segment SGI334513[] = {{&S18377[13], 14}, {NULL}};
const Segment SGI334514[] = {{&S18379[42], 3}, {NULL}};
const Segment SGI334515[] = {{&S18379[13], 14}, {NULL}};
const Segment SGI334516[] = {{&S18388[63], 3}, {&S18388[63], 3}, {NULL}};
const Segment SGI334517[] = {{&S18388[1], 24}, {NULL}};
const Segment SGI334518[] = {{&S33484[28], 3}, {NULL}};
const Segment SGI334519[] = {{&S33484[102], 5}, {NULL}};
const Segment SGI334520[] = {{&S42166[5], 3}, {NULL}};
const Segment SGI334521[] = {{&S42443[5], 7}, {NULL}};
const Segment SGI334522[] = {{&S42444[5], 4}, {&S42444[5], 4}, {NULL}};
const Segment SGI334523[] = {{&S42445[5], 4}, {NULL}};
const Segment SGI334524[] = {{&S42493[5], 5}, {&S42493[68], 5}, {NULL}};
const Segment SGI334525[] = {{&S42494[5], 5}, {NULL}};
const Segment SGI334526[] = {{&S15475[0], 21}, {NULL}};
const Segment SGI334527[] = {{&S15475[34], 3}, {NULL}};
const Segment SGI334528[] = {{&S15476[0], 21}, {NULL}};
const Segment SGI334529[] = {{&S15476[34], 3}, {NULL}};
const Segment SGI334530[] = {{&S15476[75], 13}, {NULL}};
const Segment SGI334531[] = {{&S15477[0], 21}, {NULL}};
const Segment SGI334532[] = {{&S15477[34], 3}, {NULL}};
const Segment SGI334533[] = {{&S15478[0], 21}, {NULL}};
const Segment SGI334534[] = {{&S15478[34], 3}, {NULL}};
const Segment SGI334535[] = {{&S15478[67], 13}, {NULL}};
const Segment SGI334536[] = {{&S15502[0], 21}, {NULL}};
const Segment SGI334537[] = {{&S15502[34], 3}, {NULL}};
const Segment SGI334538[] = {{&S15502[73], 13}, {NULL}};
const Segment SGI334539[] = {{&S15503[0], 21}, {NULL}};
const Segment SGI334540[] = {{&S15503[34], 3}, {NULL}};
const Segment SGI334541[] = {{&S15503[75], 13}, {NULL}};
const Segment SGI334542[] = {{&S15504[0], 21}, {NULL}};
const Segment SGI334543[] = {{&S15504[34], 3}, {NULL}};
const Segment SGI334544[] = {{&S15506[0], 21}, {NULL}};
const Segment SGI334545[] = {{&S15506[34], 3}, {NULL}};
const Segment SGI334546[] = {{&S15506[75], 13}, {NULL}};
const Segment SGI334547[] = {{&S15507[0], 21}, {NULL}};
const Segment SGI334548[] = {{&S15507[34], 3}, {NULL}};
const Segment SGI334549[] = {{&S15508[0], 21}, {NULL}};
const Segment SGI334550[] = {{&S15508[34], 3}, {NULL}};
const Segment SGI334551[] = {{&S15508[75], 13}, {NULL}};
const Segment SGI334552[] = {{&S15509[0], 23}, {NULL}};
const Segment SGI334553[] = {{&S15509[36], 3}, {NULL}};
const Segment SGI334554[] = {{&S15509[77], 13}, {NULL}};
const Segment SGI334555[] = {{&S42635[5], 5}, {&S42635[68], 5}, {NULL}};
const Segment SGI334556[] = {{&S42636[5], 5}, {NULL}};
const Segment SGI334557[] = {{&S42637[5], 5}, {&S42637[5], 5}, {NULL}};
const Segment SGI334558[] = {{&S19374[61], 3}, {NULL}};
const Segment SGI334559[] = {{&S19374[13], 10}, {NULL}};
const Segment SGI334560[] = {{&S19383[14], 4}, {NULL}};
const Segment SGI334561[] = {{&S19383[20], 14}, {NULL}};
const Segment SGI334562[] = {{&S19384[8], 4}, {&S19384[8], 3}, {NULL}};
const Segment SGI334563[] = {{&S19384[1], 27}, {NULL}};
const Segment SGI334564[] = {{&S29673[46], 5}, {NULL}};
const Segment SGI334565[] = {{&S29673[37], 3}, {NULL}};
const Segment SGI334566[] = {{&S29674[5], 3}, {NULL}};
const Segment SGI334567[] = {{&S29674[45], 3}, {NULL}};
const Segment SGI334568[] = {{&S29675[46], 5}, {&S29675[146], 7}, {NULL}};
const Segment SGI334569[] = {{&S29675[37], 3}, {NULL}};
const Segment SGI334570[] = {{&S29676[46], 5}, {NULL}};
const Segment SGI334571[] = {{&S29676[37], 3}, {NULL}};
const Segment SGI334572[] = {{&S29677[5], 5}, {NULL}};
const Segment SGI334573[] = {{&S29677[34], 3}, {NULL}};
const Segment SGI334574[] = {{&S29678[46], 5}, {&S29678[146], 7}, {NULL}};
const Segment SGI334575[] = {{&S29678[37], 3}, {NULL}};
const Segment SGI334576[] = {{&S19604[61], 5}, {NULL}};
const Segment SGI334577[] = {{&S19604[13], 10}, {NULL}};
const Segment SGI334578[] = {{&S18359[6], 4}, {NULL}};
const Segment SGI334579[] = {{&S18359[12], 14}, {NULL}};
const Segment SGI334580[] = {{&S18360[6], 4}, {NULL}};
const Segment SGI334581[] = {{&S18360[12], 15}, {NULL}};
const Segment SGI334582[] = {{&S18360[12], 14}, {NULL}};
const Segment SGI334583[] = {{&S18365[6], 4}, {NULL}};
const Segment SGI334584[] = {{&S18365[12], 14}, {NULL}};
const Segment SGI334585[] = {{&S18367[6], 2}, {NULL}};
const Segment SGI334586[] = {{&S18367[12], 14}, {NULL}};
const Segment SGI334587[] = {{&S18368[6], 4}, {NULL}};
const Segment SGI334588[] = {{&S18368[12], 14}, {NULL}};
const Segment SGI334589[] = {{&S18369[6], 4}, {NULL}};
const Segment SGI334590[] = {{&S18369[12], 14}, {NULL}};
const Segment SGI334591[] = {{&S18370[6], 4}, {NULL}};
const Segment SGI334592[] = {{&S18370[12], 14}, {NULL}};
const Segment SGI334593[] = {{&S18371[6], 4}, {NULL}};
const Segment SGI334594[] = {{&S18371[12], 14}, {NULL}};
const Segment SGI334595[] = {{&S18372[6], 4}, {NULL}};
const Segment SGI334596[] = {{&S18372[12], 14}, {NULL}};
const Segment SGI334597[] = {{&S18373[6], 4}, {NULL}};
const Segment SGI334598[] = {{&S18373[12], 14}, {NULL}};
const Segment SGI334599[] = {{&S18374[6], 4}, {NULL}};
const Segment SGI334600[] = {{&S18374[12], 14}, {NULL}};
const Segment SGI334601[] = {{&S18376[6], 3}, {NULL}};
const Segment SGI334602[] = {{&S18376[11], 14}, {NULL}};
const Segment SGI334603[] = {{&S18378[6], 4}, {NULL}};
const Segment SGI334604[] = {{&S18378[12], 14}, {NULL}};
const Segment SGI334605[] = {{&S18380[6], 4}, {NULL}};
const Segment SGI334606[] = {{&S18380[43], 4}, {NULL}};
const Segment SGI334607[] = {{&S18380[12], 14}, {NULL}};
const Segment SGI334608[] = {{&S18381[6], 4}, {NULL}};
const Segment SGI334609[] = {{&S18381[43], 4}, {NULL}};
const Segment SGI334610[] = {{&S18381[12], 14}, {NULL}};
const Segment SGI334611[] = {{&S18382[6], 4}, {NULL}};
const Segment SGI334612[] = {{&S18382[57], 5}, {NULL}};
const Segment SGI334613[] = {{&S18382[12], 14}, {NULL}};
const Segment SGI334614[] = {{&S18383[6], 4}, {NULL}};
const Segment SGI334615[] = {{&S18383[57], 5}, {NULL}};
const Segment SGI334616[] = {{&S18383[12], 14}, {NULL}};
const Segment SGI334617[] = {{&S18384[6], 4}, {NULL}};
const Segment SGI334618[] = {{&S18384[43], 4}, {NULL}};
const Segment SGI334619[] = {{&S18384[12], 14}, {NULL}};
const Segment SGI334620[] = {{&S18385[6], 4}, {NULL}};
const Segment SGI334621[] = {{&S18385[43], 4}, {NULL}};
const Segment SGI334622[] = {{&S18385[12], 14}, {NULL}};
const Segment SGI334623[] = {{&S18386[6], 4}, {NULL}};
const Segment SGI334624[] = {{&S18386[57], 5}, {NULL}};
const Segment SGI334625[] = {{&S18386[12], 14}, {NULL}};
const Segment SGI334626[] = {{&S18387[6], 4}, {NULL}};
const Segment SGI334627[] = {{&S18387[57], 5}, {NULL}};
const Segment SGI334628[] = {{&S18387[12], 14}, {NULL}};
const Segment SGI334629[] = {{&S18389[6], 4}, {NULL}};
const Segment SGI334630[] = {{&S18389[12], 14}, {NULL}};
const Segment SGI334631[] = {{&S35045[28], 5}, {NULL}};
const Segment SGI334632[] = {{&S35045[104], 5}, {NULL}};
const Segment SGI334633[] = {{&S19975[57], 5}, {NULL}};
const Segment SGI334634[] = {{&S19975[13], 10}, {NULL}};
const Segment SGI334635[] = {{&S43403[5], 4}, {NULL}};
const Segment SGI334636[] = {{&S43404[5], 4}, {&S43404[5], 4}, {NULL}};
const Segment SGI334637[] = {{&S30164[49], 5}, {NULL}};
const Segment SGI334638[] = {{&S30164[40], 3}, {NULL}};
const Segment SGI334639[] = {{&S30165[49], 5}, {NULL}};
const Segment SGI334640[] = {{&S30165[40], 3}, {NULL}};
const Segment SGI334641[] = {{&S30166[49], 5}, {&S30166[49], 5}, {NULL}};
const Segment SGI334642[] = {{&S30166[40], 3}, {&S30166[40], 3}, {NULL}};
const Segment SGI334643[] = {{&S30167[19], 3}, {&S30167[79], 5}, {NULL}};
const Segment SGI334644[] = {{&S30167[40], 3}, {&S30167[40], 3}, {NULL}};
const Segment SGI334645[] = {{&S30168[49], 5}, {NULL}};
const Segment SGI334646[] = {{&S30168[40], 3}, {NULL}};
const Segment SGI334647[] = {{&S30169[5], 3}, {NULL}};
const Segment SGI334648[] = {{&S30169[48], 3}, {NULL}};
const Segment SGI334649[] = {{&S20284[61], 5}, {NULL}};
const Segment SGI334650[] = {{&S20284[13], 10}, {NULL}};
const Segment SGI334651[] = {{&S20295[61], 5}, {NULL}};
const Segment SGI334652[] = {{&S20295[13], 10}, {NULL}};
const Segment SGI334653[] = {{&S43753[5], 12}, {NULL}};
const Segment SGI334654[] = {{&S43754[5], 12}, {&S43754[5], 12}, {NULL}};
const Segment SGI334655[] = {{&S44189[100], 11}, {NULL}};
const Segment SGI334656[] = {{&S44190[100], 21}, {NULL}};
const Segment SGI334657[] = {{&S44191[100], 19}, {NULL}};
const Segment SGI334658[] = {{&S44192[100], 19}, {NULL}};
const Segment SGI334659[] = {{&S44193[100], 29}, {NULL}};
const Segment SGI334660[] = {{&S44194[100], 19}, {NULL}};
const Segment SGI334661[] = {{&S44219[100], 11}, {NULL}};
const Segment SGI334662[] = {{&S21107[57], 4}, {NULL}};
const Segment SGI334663[] = {{&S21107[13], 10}, {NULL}};
const Segment SGI334664[] = {{&S21109[67], 4}, {NULL}};
const Segment SGI334665[] = {{&S21109[13], 10}, {NULL}};
const Segment SGI334666[] = {{&S21111[61], 4}, {NULL}};
const Segment SGI334667[] = {{&S21111[13], 10}, {NULL}};
const Segment SGI334668[] = {{&S21116[67], 4}, {NULL}};
const Segment SGI334669[] = {{&S21116[13], 10}, {NULL}};
const Segment SGI334670[] = {{&S21120[57], 4}, {NULL}};
const Segment SGI334671[] = {{&S21120[13], 10}, {NULL}};
const Segment SGI334672[] = {{&S21121[57], 4}, {NULL}};
const Segment SGI334673[] = {{&S21121[13], 10}, {NULL}};
const Segment SGI334674[] = {{&S19373[6], 4}, {NULL}};
const Segment SGI334675[] = {{&S21122[57], 4}, {NULL}};
const Segment SGI334676[] = {{&S19373[12], 14}, {NULL}};
const Segment SGI334677[] = {{&S21122[13], 10}, {NULL}};
const Segment SGI334678[] = {{&S21123[63], 3}, {NULL}};
const Segment SGI334679[] = {{&S21123[13], 10}, {NULL}};
const Segment SGI334680[] = {{&S19375[6], 3}, {NULL}};
const Segment SGI334681[] = {{&S19375[11], 14}, {NULL}};
const Segment SGI334682[] = {{&S21124[67], 4}, {NULL}};
const Segment SGI334683[] = {{&S21124[13], 10}, {NULL}};
const Segment SGI334684[] = {{&S19376[6], 4}, {NULL}};
const Segment SGI334685[] = {{&S19376[12], 14}, {NULL}};
const Segment SGI334686[] = {{&S19377[6], 4}, {NULL}};
const Segment SGI334687[] = {{&S19377[12], 14}, {NULL}};
const Segment SGI334688[] = {{&S19378[6], 4}, {NULL}};
const Segment SGI334689[] = {{&S19378[12], 14}, {NULL}};
const Segment SGI334690[] = {{&S19379[6], 4}, {NULL}};
const Segment SGI334691[] = {{&S19379[12], 14}, {NULL}};
const Segment SGI334692[] = {{&S19380[6], 4}, {NULL}};
const Segment SGI334693[] = {{&S19380[12], 10}, {NULL}};
const Segment SGI334694[] = {{&S19381[6], 4}, {NULL}};
const Segment SGI334695[] = {{&S21132[57], 4}, {NULL}};
const Segment SGI334696[] = {{&S19381[12], 14}, {NULL}};
const Segment SGI334697[] = {{&S21132[13], 10}, {NULL}};
const Segment SGI334698[] = {{&S19382[6], 4}, {NULL}};
const Segment SGI334699[] = {{&S19382[12], 15}, {NULL}};
const Segment SGI334700[] = {{&S19382[12], 14}, {NULL}};
const Segment SGI334701[] = {{&S21135[57], 4}, {NULL}};
const Segment SGI334702[] = {{&S21135[13], 10}, {NULL}};
const Segment SGI334703[] = {{&S21137[57], 4}, {NULL}};
const Segment SGI334704[] = {{&S21137[13], 10}, {NULL}};
const Segment SGI334705[] = {{&S21139[67], 4}, {NULL}};
const Segment SGI334706[] = {{&S21139[13], 10}, {NULL}};
const Segment SGI334707[] = {{&S21140[6], 3}, {&S21140[26], 4}, {NULL}};
const Segment SGI334708[] = {{&S21140[11], 31}, {NULL}};
const Segment SGI334709[] = {{&S21141[69], 4}, {&S21141[69], 4}, {NULL}};
const Segment SGI334710[] = {{&S21141[1], 24}, {NULL}};
const Segment SGI334711[] = {{&S21142[6], 4}, {&S21142[6], 4}, {NULL}};
const Segment SGI334712[] = {{&S21142[12], 35}, {NULL}};
const Segment SGI334713[] = {{&S21143[57], 4}, {NULL}};
const Segment SGI334714[] = {{&S21143[13], 10}, {&S21143[68], 24}, {NULL}};
const Segment SGI334715[] = {{&S21144[57], 4}, {NULL}};
const Segment SGI334716[] = {{&S21144[13], 10}, {NULL}};
const Segment SGI334717[] = {{&S21145[57], 4}, {NULL}};
const Segment SGI334718[] = {{&S21145[13], 10}, {NULL}};
const Segment SGI334719[] = {{&S21147[38], 4}, {NULL}};
const Segment SGI334720[] = {{&S21147[13], 14}, {NULL}};
const Segment SGI334721[] = {{&S21149[38], 4}, {NULL}};
const Segment SGI334722[] = {{&S21149[13], 14}, {NULL}};
const Segment SGI334723[] = {{&S31177[49], 9}, {NULL}};
const Segment SGI334724[] = {{&S31177[8], 3}, {NULL}};
const Segment SGI334725[] = {{&S31178[5], 4}, {NULL}};
const Segment SGI334726[] = {{&S31178[49], 3}, {NULL}};
const Segment SGI334727[] = {{&S31179[49], 20}, {&S31179[49], 7}, {NULL}};
const Segment SGI334728[] = {{&S31179[8], 3}, {&S31179[8], 3}, {NULL}};
const Segment SGI334729[] = {{&S31180[5], 5}, {NULL}};
const Segment SGI334730[] = {{&S31180[48], 3}, {NULL}};
const Segment SGI334731[] = {{&S31181[46], 8}, {NULL}};
const Segment SGI334732[] = {{&S31181[8], 3}, {NULL}};
const Segment SGI334733[] = {{&S31182[46], 7}, {NULL}};
const Segment SGI334734[] = {{&S31182[8], 3}, {NULL}};
const Segment SGI334735[] = {{&S31183[46], 7}, {&S31183[149], 7}, {NULL}};
const Segment SGI334736[] = {{&S31183[8], 3}, {NULL}};
const Segment SGI334737[] = {{&S31184[5], 5}, {NULL}};
const Segment SGI334738[] = {{&S31184[48], 3}, {NULL}};
const Segment SGI334739[] = {{&S31185[46], 4}, {&S31185[145], 7}, {NULL}};
const Segment SGI334740[] = {{&S31185[8], 3}, {NULL}};
const Segment SGI334741[] = {{&S21337[61], 5}, {NULL}};
const Segment SGI334742[] = {{&S21337[13], 10}, {NULL}};
const Segment SGI334743[] = {{&S19546[6], 4}, {NULL}};
const Segment SGI334744[] = {{&S19546[59], 5}, {NULL}};
const Segment SGI334745[] = {{&S19546[12], 14}, {NULL}};
const Segment SGI334746[] = {{&S19547[6], 4}, {NULL}};
const Segment SGI334747[] = {{&S19547[81], 5}, {NULL}};
const Segment SGI334748[] = {{&S19547[12], 14}, {&S19547[40], 11}, {NULL}};
const Segment SGI334749[] = {{&S19547[12], 14}, {NULL}};
const Segment SGI334750[] = {{&S19548[6], 4}, {NULL}};
const Segment SGI334751[] = {{&S19548[82], 5}, {NULL}};
const Segment SGI334752[] = {{&S19548[12], 14}, {&S19548[41], 11}, {NULL}};
const Segment SGI334753[] = {{&S19548[12], 14}, {NULL}};
const Segment SGI334754[] = {{&S19549[6], 4}, {NULL}};
const Segment SGI334755[] = {{&S19549[81], 5}, {NULL}};
const Segment SGI334756[] = {{&S19549[12], 14}, {&S19549[40], 11}, {NULL}};
const Segment SGI334757[] = {{&S19549[12], 14}, {NULL}};
const Segment SGI334758[] = {{&S19550[6], 4}, {NULL}};
const Segment SGI334759[] = {{&S19550[82], 5}, {NULL}};
const Segment SGI334760[] = {{&S19550[12], 14}, {&S19550[40], 12}, {NULL}};
const Segment SGI334761[] = {{&S19550[12], 14}, {NULL}};
const Segment SGI334762[] = {{&S19551[6], 4}, {NULL}};
const Segment SGI334763[] = {{&S19551[89], 5}, {NULL}};
const Segment SGI334764[] = {{&S19551[12], 14}, {&S19551[48], 11}, {NULL}};
const Segment SGI334765[] = {{&S19551[12], 14}, {NULL}};
const Segment SGI334766[] = {{&S19552[6], 4}, {NULL}};
const Segment SGI334767[] = {{&S19552[82], 5}, {NULL}};
const Segment SGI334768[] = {{&S19552[12], 14}, {&S19552[41], 11}, {NULL}};
const Segment SGI334769[] = {{&S19552[12], 14}, {NULL}};
const Segment SGI334770[] = {{&S19553[6], 4}, {NULL}};
const Segment SGI334771[] = {{&S19553[89], 5}, {NULL}};
const Segment SGI334772[] = {{&S19553[12], 14}, {&S19553[48], 11}, {NULL}};
const Segment SGI334773[] = {{&S19553[12], 14}, {NULL}};
const Segment SGI334774[] = {{&S19554[6], 4}, {NULL}};
const Segment SGI334775[] = {{&S19554[82], 5}, {NULL}};
const Segment SGI334776[] = {{&S19554[12], 14}, {&S19554[40], 12}, {NULL}};
const Segment SGI334777[] = {{&S19554[12], 14}, {NULL}};
const Segment SGI334778[] = {{&S19555[6], 4}, {NULL}};
const Segment SGI334779[] = {{&S19555[89], 5}, {NULL}};
const Segment SGI334780[] = {{&S19555[12], 14}, {&S19555[48], 11}, {NULL}};
const Segment SGI334781[] = {{&S19555[12], 14}, {NULL}};
const Segment SGI334782[] = {{&S19556[6], 4}, {NULL}};
const Segment SGI334783[] = {{&S19556[89], 5}, {NULL}};
const Segment SGI334784[] = {{&S19556[12], 14}, {&S19556[47], 12}, {NULL}};
const Segment SGI334785[] = {{&S19556[12], 14}, {NULL}};
const Segment SGI334786[] = {{&S19557[6], 4}, {NULL}};
const Segment SGI334787[] = {{&S19557[81], 5}, {NULL}};
const Segment SGI334788[] = {{&S19557[12], 14}, {&S19557[40], 11}, {NULL}};
const Segment SGI334789[] = {{&S19557[12], 14}, {NULL}};
const Segment SGI334790[] = {{&S19558[6], 4}, {NULL}};
const Segment SGI334791[] = {{&S19558[82], 5}, {NULL}};
const Segment SGI334792[] = {{&S19558[12], 14}, {&S19558[41], 11}, {NULL}};
const Segment SGI334793[] = {{&S19558[12], 14}, {NULL}};
const Segment SGI334794[] = {{&S19559[6], 4}, {NULL}};
const Segment SGI334795[] = {{&S19559[82], 5}, {NULL}};
const Segment SGI334796[] = {{&S19559[12], 14}, {&S19559[41], 11}, {NULL}};
const Segment SGI334797[] = {{&S19559[12], 14}, {NULL}};
const Segment SGI334798[] = {{&S19560[6], 4}, {NULL}};
const Segment SGI334799[] = {{&S19560[82], 5}, {NULL}};
const Segment SGI334800[] = {{&S19560[12], 14}, {&S19560[41], 11}, {NULL}};
const Segment SGI334801[] = {{&S19560[12], 14}, {NULL}};
const Segment SGI334802[] = {{&S19561[6], 4}, {NULL}};
const Segment SGI334803[] = {{&S19561[82], 5}, {NULL}};
const Segment SGI334804[] = {{&S19561[12], 14}, {&S19561[41], 11}, {NULL}};
const Segment SGI334805[] = {{&S19561[12], 14}, {NULL}};
const Segment SGI334806[] = {{&S19562[6], 4}, {NULL}};
const Segment SGI334807[] = {{&S19562[89], 5}, {NULL}};
const Segment SGI334808[] = {{&S19562[12], 14}, {&S19562[48], 11}, {NULL}};
const Segment SGI334809[] = {{&S19562[12], 14}, {NULL}};
const Segment SGI334810[] = {{&S19563[6], 4}, {NULL}};
const Segment SGI334811[] = {{&S19563[82], 5}, {NULL}};
const Segment SGI334812[] = {{&S19563[12], 14}, {&S19563[41], 11}, {NULL}};
const Segment SGI334813[] = {{&S19563[12], 14}, {NULL}};
const Segment SGI334814[] = {{&S19564[6], 4}, {NULL}};
const Segment SGI334815[] = {{&S19564[88], 5}, {NULL}};
const Segment SGI334816[] = {{&S19564[12], 14}, {&S19564[47], 11}, {NULL}};
const Segment SGI334817[] = {{&S19564[12], 14}, {NULL}};
const Segment SGI334818[] = {{&S19565[6], 4}, {NULL}};
const Segment SGI334819[] = {{&S19565[82], 5}, {NULL}};
const Segment SGI334820[] = {{&S19565[12], 14}, {&S19565[40], 12}, {NULL}};
const Segment SGI334821[] = {{&S19565[12], 14}, {NULL}};
const Segment SGI334822[] = {{&S19566[6], 4}, {NULL}};
const Segment SGI334823[] = {{&S19566[81], 5}, {NULL}};
const Segment SGI334824[] = {{&S19566[12], 14}, {&S19566[40], 11}, {NULL}};
const Segment SGI334825[] = {{&S19566[12], 14}, {NULL}};
const Segment SGI334826[] = {{&S19567[6], 4}, {NULL}};
const Segment SGI334827[] = {{&S19567[82], 5}, {NULL}};
const Segment SGI334828[] = {{&S19567[12], 14}, {&S19567[41], 11}, {NULL}};
const Segment SGI334829[] = {{&S19567[12], 14}, {NULL}};
const Segment SGI334830[] = {{&S19568[6], 4}, {NULL}};
const Segment SGI334831[] = {{&S19568[59], 5}, {NULL}};
const Segment SGI334832[] = {{&S19568[12], 14}, {&S19568[70], 31}, {NULL}};
const Segment SGI334833[] = {{&S19568[12], 14}, {NULL}};
const Segment SGI334834[] = {{&S19569[6], 4}, {NULL}};
const Segment SGI334835[] = {{&S19569[81], 5}, {NULL}};
const Segment SGI334836[] = {{&S19569[12], 14}, {&S19569[40], 11}, {NULL}};
const Segment SGI334837[] = {{&S19569[12], 14}, {NULL}};
const Segment SGI334838[] = {{&S19570[6], 4}, {NULL}};
const Segment SGI334839[] = {{&S19570[82], 5}, {NULL}};
const Segment SGI334840[] = {{&S19570[12], 14}, {&S19570[40], 12}, {NULL}};
const Segment SGI334841[] = {{&S19570[12], 14}, {NULL}};
const Segment SGI334842[] = {{&S19571[6], 4}, {NULL}};
const Segment SGI334843[] = {{&S19571[82], 5}, {NULL}};
const Segment SGI334844[] = {{&S19571[12], 14}, {&S19571[41], 11}, {NULL}};
const Segment SGI334845[] = {{&S19571[12], 14}, {NULL}};
const Segment SGI334846[] = {{&S19572[6], 4}, {NULL}};
const Segment SGI334847[] = {{&S19572[82], 5}, {NULL}};
const Segment SGI334848[] = {{&S19572[12], 14}, {&S19572[41], 11}, {NULL}};
const Segment SGI334849[] = {{&S19572[12], 14}, {NULL}};
const Segment SGI334850[] = {{&S19573[6], 4}, {NULL}};
const Segment SGI334851[] = {{&S19573[88], 5}, {NULL}};
const Segment SGI334852[] = {{&S19573[12], 14}, {&S19573[47], 11}, {NULL}};
const Segment SGI334853[] = {{&S19573[12], 14}, {NULL}};
const Segment SGI334854[] = {{&S19574[6], 4}, {NULL}};
const Segment SGI334855[] = {{&S19574[82], 5}, {NULL}};
const Segment SGI334856[] = {{&S19574[12], 14}, {&S19574[40], 12}, {NULL}};
const Segment SGI334857[] = {{&S19574[12], 14}, {NULL}};
const Segment SGI334858[] = {{&S19575[6], 4}, {NULL}};
const Segment SGI334859[] = {{&S19575[81], 5}, {NULL}};
const Segment SGI334860[] = {{&S19575[12], 14}, {&S19575[40], 11}, {NULL}};
const Segment SGI334861[] = {{&S19575[12], 14}, {NULL}};
const Segment SGI334862[] = {{&S19576[6], 4}, {NULL}};
const Segment SGI334863[] = {{&S19576[82], 5}, {NULL}};
const Segment SGI334864[] = {{&S19576[12], 14}, {&S19576[41], 11}, {NULL}};
const Segment SGI334865[] = {{&S19576[12], 14}, {NULL}};
const Segment SGI334866[] = {{&S19577[6], 4}, {NULL}};
const Segment SGI334867[] = {{&S19577[88], 5}, {NULL}};
const Segment SGI334868[] = {{&S19577[12], 14}, {&S19577[47], 11}, {NULL}};
const Segment SGI334869[] = {{&S19577[12], 14}, {NULL}};
const Segment SGI334870[] = {{&S19578[6], 4}, {NULL}};
const Segment SGI334871[] = {{&S19578[89], 5}, {NULL}};
const Segment SGI334872[] = {{&S19578[12], 14}, {&S19578[48], 11}, {NULL}};
const Segment SGI334873[] = {{&S19578[12], 14}, {NULL}};
const Segment SGI334874[] = {{&S19580[6], 4}, {NULL}};
const Segment SGI334875[] = {{&S19580[82], 5}, {NULL}};
const Segment SGI334876[] = {{&S19580[12], 14}, {&S19580[41], 11}, {NULL}};
const Segment SGI334877[] = {{&S19580[12], 14}, {NULL}};
const Segment SGI334878[] = {{&S19581[6], 4}, {NULL}};
const Segment SGI334879[] = {{&S19581[82], 5}, {NULL}};
const Segment SGI334880[] = {{&S19581[12], 14}, {&S19581[40], 12}, {NULL}};
const Segment SGI334881[] = {{&S19581[12], 14}, {NULL}};
const Segment SGI334882[] = {{&S19582[6], 4}, {NULL}};
const Segment SGI334883[] = {{&S19582[82], 5}, {NULL}};
const Segment SGI334884[] = {{&S19582[12], 14}, {&S19582[40], 12}, {NULL}};
const Segment SGI334885[] = {{&S19582[12], 14}, {NULL}};
const Segment SGI334886[] = {{&S19583[6], 4}, {NULL}};
const Segment SGI334887[] = {{&S19583[88], 5}, {NULL}};
const Segment SGI334888[] = {{&S19583[12], 14}, {&S19583[47], 11}, {NULL}};
const Segment SGI334889[] = {{&S19583[12], 14}, {NULL}};
const Segment SGI334890[] = {{&S19584[6], 4}, {NULL}};
const Segment SGI334891[] = {{&S19584[81], 5}, {NULL}};
const Segment SGI334892[] = {{&S19584[12], 14}, {&S19584[40], 11}, {NULL}};
const Segment SGI334893[] = {{&S19584[12], 14}, {NULL}};
const Segment SGI334894[] = {{&S19585[6], 4}, {NULL}};
const Segment SGI334895[] = {{&S19585[81], 5}, {NULL}};
const Segment SGI334896[] = {{&S19585[12], 14}, {&S19585[40], 11}, {NULL}};
const Segment SGI334897[] = {{&S19585[12], 14}, {NULL}};
const Segment SGI334898[] = {{&S19586[6], 4}, {NULL}};
const Segment SGI334899[] = {{&S19586[88], 5}, {NULL}};
const Segment SGI334900[] = {{&S19586[12], 14}, {&S19586[47], 11}, {NULL}};
const Segment SGI334901[] = {{&S19586[12], 14}, {NULL}};
const Segment SGI334902[] = {{&S19587[6], 4}, {NULL}};
const Segment SGI334903[] = {{&S19587[82], 5}, {NULL}};
const Segment SGI334904[] = {{&S19587[12], 14}, {&S19587[40], 12}, {NULL}};
const Segment SGI334905[] = {{&S19587[12], 14}, {NULL}};
const Segment SGI334906[] = {{&S19588[6], 4}, {NULL}};
const Segment SGI334907[] = {{&S19588[82], 5}, {NULL}};
const Segment SGI334908[] = {{&S19588[12], 14}, {&S19588[40], 12}, {NULL}};
const Segment SGI334909[] = {{&S19588[12], 14}, {NULL}};
const Segment SGI334910[] = {{&S19589[6], 4}, {NULL}};
const Segment SGI334911[] = {{&S19589[81], 5}, {NULL}};
const Segment SGI334912[] = {{&S19589[12], 14}, {&S19589[40], 11}, {NULL}};
const Segment SGI334913[] = {{&S19589[12], 14}, {NULL}};
const Segment SGI334914[] = {{&S19590[6], 4}, {NULL}};
const Segment SGI334915[] = {{&S19590[82], 5}, {NULL}};
const Segment SGI334916[] = {{&S19590[12], 14}, {&S19590[41], 11}, {NULL}};
const Segment SGI334917[] = {{&S19590[12], 14}, {NULL}};
const Segment SGI334918[] = {{&S19591[6], 4}, {NULL}};
const Segment SGI334919[] = {{&S19591[88], 5}, {NULL}};
const Segment SGI334920[] = {{&S19591[12], 14}, {&S19591[47], 11}, {NULL}};
const Segment SGI334921[] = {{&S19591[12], 14}, {NULL}};
const Segment SGI334922[] = {{&S19592[6], 4}, {NULL}};
const Segment SGI334923[] = {{&S19592[88], 5}, {NULL}};
const Segment SGI334924[] = {{&S19592[12], 14}, {&S19592[47], 11}, {NULL}};
const Segment SGI334925[] = {{&S19592[12], 14}, {NULL}};
const Segment SGI334926[] = {{&S19593[6], 4}, {NULL}};
const Segment SGI334927[] = {{&S19593[88], 5}, {NULL}};
const Segment SGI334928[] = {{&S19593[12], 14}, {&S19593[47], 11}, {NULL}};
const Segment SGI334929[] = {{&S19593[12], 14}, {NULL}};
const Segment SGI334930[] = {{&S19594[6], 4}, {NULL}};
const Segment SGI334931[] = {{&S19594[81], 5}, {NULL}};
const Segment SGI334932[] = {{&S19594[12], 14}, {&S19594[40], 11}, {NULL}};
const Segment SGI334933[] = {{&S19594[12], 14}, {NULL}};
const Segment SGI334934[] = {{&S19595[6], 4}, {NULL}};
const Segment SGI334935[] = {{&S19595[81], 5}, {NULL}};
const Segment SGI334936[] = {{&S19595[12], 14}, {&S19595[40], 11}, {NULL}};
const Segment SGI334937[] = {{&S19595[12], 14}, {NULL}};
const Segment SGI334938[] = {{&S19596[6], 4}, {NULL}};
const Segment SGI334939[] = {{&S19596[81], 5}, {NULL}};
const Segment SGI334940[] = {{&S19596[12], 14}, {&S19596[40], 11}, {NULL}};
const Segment SGI334941[] = {{&S19596[12], 14}, {NULL}};
const Segment SGI334942[] = {{&S19597[6], 4}, {NULL}};
const Segment SGI334943[] = {{&S19597[81], 5}, {NULL}};
const Segment SGI334944[] = {{&S19597[12], 14}, {&S19597[40], 11}, {NULL}};
const Segment SGI334945[] = {{&S19597[12], 14}, {NULL}};
const Segment SGI334946[] = {{&S19598[6], 4}, {NULL}};
const Segment SGI334947[] = {{&S19598[81], 5}, {NULL}};
const Segment SGI334948[] = {{&S19598[12], 14}, {&S19598[40], 11}, {NULL}};
const Segment SGI334949[] = {{&S19598[12], 14}, {NULL}};
const Segment SGI334950[] = {{&S19599[6], 4}, {NULL}};
const Segment SGI334951[] = {{&S19599[81], 5}, {NULL}};
const Segment SGI334952[] = {{&S19599[12], 14}, {&S19599[40], 11}, {NULL}};
const Segment SGI334953[] = {{&S19599[12], 14}, {NULL}};
const Segment SGI334954[] = {{&S19600[6], 4}, {NULL}};
const Segment SGI334955[] = {{&S19600[81], 5}, {NULL}};
const Segment SGI334956[] = {{&S19600[12], 14}, {&S19600[40], 11}, {NULL}};
const Segment SGI334957[] = {{&S19600[12], 14}, {NULL}};
const Segment SGI334958[] = {{&S19601[6], 4}, {NULL}};
const Segment SGI334959[] = {{&S19601[81], 5}, {NULL}};
const Segment SGI334960[] = {{&S19601[12], 14}, {&S19601[40], 11}, {NULL}};
const Segment SGI334961[] = {{&S19601[12], 14}, {NULL}};
const Segment SGI334962[] = {{&S19602[6], 4}, {NULL}};
const Segment SGI334963[] = {{&S19602[82], 5}, {NULL}};
const Segment SGI334964[] = {{&S19602[12], 14}, {&S19602[40], 12}, {NULL}};
const Segment SGI334965[] = {{&S19602[12], 14}, {NULL}};
const Segment SGI334966[] = {{&S19603[6], 4}, {NULL}};
const Segment SGI334967[] = {{&S19603[82], 5}, {NULL}};
const Segment SGI334968[] = {{&S19603[12], 14}, {&S19603[40], 12}, {NULL}};
const Segment SGI334969[] = {{&S19603[12], 14}, {NULL}};
const Segment SGI334970[] = {{&S19605[6], 4}, {NULL}};
const Segment SGI334971[] = {{&S19605[88], 5}, {NULL}};
const Segment SGI334972[] = {{&S19605[12], 14}, {&S19605[47], 11}, {NULL}};
const Segment SGI334973[] = {{&S19605[12], 14}, {NULL}};
const Segment SGI334974[] = {{&S19606[6], 4}, {NULL}};
const Segment SGI334975[] = {{&S19606[81], 5}, {NULL}};
const Segment SGI334976[] = {{&S19606[12], 14}, {&S19606[40], 11}, {NULL}};
const Segment SGI334977[] = {{&S19606[12], 14}, {NULL}};
const Segment SGI334978[] = {{&S19607[6], 4}, {NULL}};
const Segment SGI334979[] = {{&S19607[81], 5}, {NULL}};
const Segment SGI334980[] = {{&S19607[12], 14}, {&S19607[40], 11}, {NULL}};
const Segment SGI334981[] = {{&S19607[12], 14}, {NULL}};
const Segment SGI334982[] = {{&S19608[6], 4}, {NULL}};
const Segment SGI334983[] = {{&S19608[82], 5}, {NULL}};
const Segment SGI334984[] = {{&S19608[12], 14}, {&S19608[41], 11}, {NULL}};
const Segment SGI334985[] = {{&S19608[12], 14}, {NULL}};
const Segment SGI334986[] = {{&S19609[6], 4}, {NULL}};
const Segment SGI334987[] = {{&S19609[81], 5}, {NULL}};
const Segment SGI334988[] = {{&S19609[12], 14}, {&S19609[40], 11}, {NULL}};
const Segment SGI334989[] = {{&S19609[12], 14}, {NULL}};
const Segment SGI334990[] = {{&S19610[6], 4}, {NULL}};
const Segment SGI334991[] = {{&S19610[82], 5}, {NULL}};
const Segment SGI334992[] = {{&S19610[12], 14}, {&S19610[41], 11}, {NULL}};
const Segment SGI334993[] = {{&S19610[12], 14}, {NULL}};
const Segment SGI334994[] = {{&S19611[6], 4}, {NULL}};
const Segment SGI334995[] = {{&S19611[59], 5}, {NULL}};
const Segment SGI334996[] = {{&S19611[12], 14}, {&S19611[70], 4}, {NULL}};
const Segment SGI334997[] = {{&S19611[12], 14}, {NULL}};
const Segment SGI334998[] = {{&S19612[6], 4}, {NULL}};
const Segment SGI334999[] = {{&S19612[81], 5}, {NULL}};
const Segment SGI335000[] = {{&S19612[12], 14}, {&S19612[40], 11}, {NULL}};
const Segment SGI335001[] = {{&S19612[12], 14}, {NULL}};
const Segment SGI335002[] = {{&S19613[6], 4}, {NULL}};
const Segment SGI335003[] = {{&S19613[81], 5}, {NULL}};
const Segment SGI335004[] = {{&S19613[12], 14}, {&S19613[40], 11}, {NULL}};
const Segment SGI335005[] = {{&S19613[12], 14}, {NULL}};
const Segment SGI335006[] = {{&S19614[6], 4}, {NULL}};
const Segment SGI335007[] = {{&S19614[88], 5}, {NULL}};
const Segment SGI335008[] = {{&S19614[12], 14}, {&S19614[47], 11}, {NULL}};
const Segment SGI335009[] = {{&S19614[12], 14}, {NULL}};
const Segment SGI335010[] = {{&S19615[6], 4}, {NULL}};
const Segment SGI335011[] = {{&S19615[81], 5}, {NULL}};
const Segment SGI335012[] = {{&S19615[12], 14}, {&S19615[40], 11}, {NULL}};
const Segment SGI335013[] = {{&S19615[12], 14}, {NULL}};
const Segment SGI335014[] = {{&S19616[6], 4}, {NULL}};
const Segment SGI335015[] = {{&S19616[81], 5}, {NULL}};
const Segment SGI335016[] = {{&S19616[12], 14}, {&S19616[40], 11}, {NULL}};
const Segment SGI335017[] = {{&S19616[12], 14}, {NULL}};
const Segment SGI335018[] = {{&S19617[6], 4}, {NULL}};
const Segment SGI335019[] = {{&S19617[88], 5}, {NULL}};
const Segment SGI335020[] = {{&S19617[12], 14}, {&S19617[47], 11}, {NULL}};
const Segment SGI335021[] = {{&S19617[12], 14}, {NULL}};
const Segment SGI335022[] = {{&S19618[6], 4}, {NULL}};
const Segment SGI335023[] = {{&S19618[88], 5}, {NULL}};
const Segment SGI335024[] = {{&S19618[12], 14}, {&S19618[47], 11}, {NULL}};
const Segment SGI335025[] = {{&S19618[12], 14}, {NULL}};
const Segment SGI335026[] = {{&S19619[6], 4}, {NULL}};
const Segment SGI335027[] = {{&S19619[88], 5}, {NULL}};
const Segment SGI335028[] = {{&S19619[12], 14}, {&S19619[47], 11}, {NULL}};
const Segment SGI335029[] = {{&S19619[12], 14}, {NULL}};
const Segment SGI335030[] = {{&S19620[6], 4}, {NULL}};
const Segment SGI335031[] = {{&S19620[81], 5}, {NULL}};
const Segment SGI335032[] = {{&S19620[12], 14}, {&S19620[40], 11}, {NULL}};
const Segment SGI335033[] = {{&S19620[12], 14}, {NULL}};
const Segment SGI335034[] = {{&S19621[6], 4}, {NULL}};
const Segment SGI335035[] = {{&S19621[81], 5}, {NULL}};
const Segment SGI335036[] = {{&S19621[12], 14}, {&S19621[40], 11}, {NULL}};
const Segment SGI335037[] = {{&S19621[12], 14}, {NULL}};
const Segment SGI335038[] = {{&S19622[6], 4}, {NULL}};
const Segment SGI335039[] = {{&S19622[81], 5}, {NULL}};
const Segment SGI335040[] = {{&S19622[12], 14}, {&S19622[40], 11}, {NULL}};
const Segment SGI335041[] = {{&S19622[12], 14}, {NULL}};
const Segment SGI335042[] = {{&S19623[6], 4}, {NULL}};
const Segment SGI335043[] = {{&S19623[88], 5}, {NULL}};
const Segment SGI335044[] = {{&S19623[12], 14}, {&S19623[47], 11}, {NULL}};
const Segment SGI335045[] = {{&S19623[12], 14}, {NULL}};
const Segment SGI335046[] = {{&S19624[6], 4}, {NULL}};
const Segment SGI335047[] = {{&S19624[81], 5}, {NULL}};
const Segment SGI335048[] = {{&S19624[12], 14}, {&S19624[40], 11}, {NULL}};
const Segment SGI335049[] = {{&S19624[12], 14}, {NULL}};
const Segment SGI335050[] = {{&S19625[6], 4}, {NULL}};
const Segment SGI335051[] = {{&S19625[88], 5}, {NULL}};
const Segment SGI335052[] = {{&S19625[12], 14}, {&S19625[47], 11}, {NULL}};
const Segment SGI335053[] = {{&S19625[12], 14}, {NULL}};
const Segment SGI335054[] = {{&S19626[6], 4}, {NULL}};
const Segment SGI335055[] = {{&S19626[81], 5}, {NULL}};
const Segment SGI335056[] = {{&S19626[12], 14}, {&S19626[40], 11}, {NULL}};
const Segment SGI335057[] = {{&S19626[12], 14}, {NULL}};
const Segment SGI335058[] = {{&S44950[100], 21}, {NULL}};
const Segment SGI335059[] = {{&S44951[100], 19}, {NULL}};
const Segment SGI335060[] = {{&S44952[100], 11}, {NULL}};
const Segment SGI335061[] = {{&S44953[100], 19}, {NULL}};
const Segment SGI335062[] = {{&S44954[100], 19}, {NULL}};
const Segment SGI335063[] = {{&S44955[100], 18}, {NULL}};
const Segment SGI335064[] = {{&S44956[100], 19}, {NULL}};
const Segment SGI335065[] = {{&S44957[100], 11}, {NULL}};
const Segment SGI335066[] = {{&S45006[25], 2}, {&S45006[73], 7}, {NULL}};
const Segment SGI335067[] = {{&S45076[5], 3}, {NULL}};
const Segment SGI335068[] = {{&S45077[5], 3}, {&S45077[5], 3}, {NULL}};
const Segment SGI335069[] = {{&S36664[28], 5}, {NULL}};
const Segment SGI335070[] = {{&S36664[104], 5}, {NULL}};
const Segment SGI335071[] = {{&S21833[57], 5}, {NULL}};
const Segment SGI335072[] = {{&S21833[13], 10}, {NULL}};
const Segment SGI335073[] = {{&S19970[6], 4}, {NULL}};
const Segment SGI335074[] = {{&S19970[52], 5}, {NULL}};
const Segment SGI335075[] = {{&S19970[12], 45}, {NULL}};
const Segment SGI335076[] = {{&S19970[12], 14}, {NULL}};
const Segment SGI335077[] = {{&S19971[6], 4}, {NULL}};
const Segment SGI335078[] = {{&S19971[74], 5}, {NULL}};
const Segment SGI335079[] = {{&S19971[12], 14}, {&S19971[37], 42}, {NULL}};
const Segment SGI335080[] = {{&S19971[12], 14}, {NULL}};
const Segment SGI335081[] = {{&S19972[6], 4}, {NULL}};
const Segment SGI335082[] = {{&S19972[75], 5}, {NULL}};
const Segment SGI335083[] = {{&S19972[12], 14}, {&S19972[38], 42}, {NULL}};
const Segment SGI335084[] = {{&S19972[12], 14}, {NULL}};
const Segment SGI335085[] = {{&S19973[6], 4}, {NULL}};
const Segment SGI335086[] = {{&S19973[52], 5}, {NULL}};
const Segment SGI335087[] = {{&S19973[12], 55}, {NULL}};
const Segment SGI335088[] = {{&S19973[12], 14}, {NULL}};
const Segment SGI335089[] = {{&S19974[6], 4}, {NULL}};
const Segment SGI335090[] = {{&S19974[52], 5}, {NULL}};
const Segment SGI335091[] = {{&S19974[12], 45}, {NULL}};
const Segment SGI335092[] = {{&S19974[12], 14}, {NULL}};
const Segment SGI335093[] = {{&S19976[6], 4}, {NULL}};
const Segment SGI335094[] = {{&S19976[75], 5}, {NULL}};
const Segment SGI335095[] = {{&S19976[12], 14}, {&S19976[37], 43}, {NULL}};
const Segment SGI335096[] = {{&S19976[12], 14}, {NULL}};
const Segment SGI335097[] = {{&S19977[6], 4}, {NULL}};
const Segment SGI335098[] = {{&S19977[73], 5}, {NULL}};
const Segment SGI335099[] = {{&S19977[12], 14}, {NULL}};
const Segment SGI335100[] = {{&S19978[6], 4}, {NULL}};
const Segment SGI335101[] = {{&S19978[74], 5}, {NULL}};
const Segment SGI335102[] = {{&S19978[12], 14}, {&S19978[37], 42}, {NULL}};
const Segment SGI335103[] = {{&S19978[12], 14}, {NULL}};
const Segment SGI335104[] = {{&S19979[6], 4}, {NULL}};
const Segment SGI335105[] = {{&S19979[75], 5}, {NULL}};
const Segment SGI335106[] = {{&S19979[12], 14}, {&S19979[37], 43}, {NULL}};
const Segment SGI335107[] = {{&S19979[12], 14}, {NULL}};
const Segment SGI335108[] = {{&S19980[6], 4}, {NULL}};
const Segment SGI335109[] = {{&S19980[74], 5}, {NULL}};
const Segment SGI335110[] = {{&S19980[12], 14}, {&S19980[37], 42}, {NULL}};
const Segment SGI335111[] = {{&S19980[12], 14}, {NULL}};
const Segment SGI335112[] = {{&S19981[6], 4}, {NULL}};
const Segment SGI335113[] = {{&S19981[74], 5}, {NULL}};
const Segment SGI335114[] = {{&S19981[12], 14}, {&S19981[37], 42}, {NULL}};
const Segment SGI335115[] = {{&S19981[12], 14}, {NULL}};
const Segment SGI335116[] = {{&S19982[6], 4}, {NULL}};
const Segment SGI335117[] = {{&S19982[74], 5}, {NULL}};
const Segment SGI335118[] = {{&S19982[12], 14}, {&S19982[37], 42}, {NULL}};
const Segment SGI335119[] = {{&S19982[12], 14}, {NULL}};
const Segment SGI335120[] = {{&S19983[6], 4}, {NULL}};
const Segment SGI335121[] = {{&S19983[74], 5}, {NULL}};
const Segment SGI335122[] = {{&S19983[12], 14}, {&S19983[37], 42}, {NULL}};
const Segment SGI335123[] = {{&S19983[12], 14}, {NULL}};
const Segment SGI335124[] = {{&S19984[6], 4}, {NULL}};
const Segment SGI335125[] = {{&S19984[75], 5}, {NULL}};
const Segment SGI335126[] = {{&S19984[12], 14}, {&S19984[38], 42}, {NULL}};
const Segment SGI335127[] = {{&S19984[12], 14}, {NULL}};
const Segment SGI335128[] = {{&S19985[6], 4}, {NULL}};
const Segment SGI335129[] = {{&S19985[74], 5}, {NULL}};
const Segment SGI335130[] = {{&S19985[12], 14}, {&S19985[37], 42}, {NULL}};
const Segment SGI335131[] = {{&S19985[12], 14}, {NULL}};
const Segment SGI335132[] = {{&S19986[6], 4}, {NULL}};
const Segment SGI335133[] = {{&S19986[74], 5}, {NULL}};
const Segment SGI335134[] = {{&S19986[12], 14}, {&S19986[37], 42}, {NULL}};
const Segment SGI335135[] = {{&S19986[12], 14}, {NULL}};
const Segment SGI335136[] = {{&S19987[6], 4}, {NULL}};
const Segment SGI335137[] = {{&S19987[74], 5}, {NULL}};
const Segment SGI335138[] = {{&S19987[12], 14}, {&S19987[37], 42}, {NULL}};
const Segment SGI335139[] = {{&S19987[12], 14}, {NULL}};
const Segment SGI335140[] = {{&S19988[6], 4}, {NULL}};
const Segment SGI335141[] = {{&S19988[74], 5}, {NULL}};
const Segment SGI335142[] = {{&S19988[12], 14}, {&S19988[37], 42}, {NULL}};
const Segment SGI335143[] = {{&S19988[12], 14}, {NULL}};
const Segment SGI335144[] = {{&S19989[6], 4}, {NULL}};
const Segment SGI335145[] = {{&S19989[74], 5}, {NULL}};
const Segment SGI335146[] = {{&S19989[12], 14}, {&S19989[37], 42}, {NULL}};
const Segment SGI335147[] = {{&S19989[12], 14}, {NULL}};
const Segment SGI335148[] = {{&S19990[6], 4}, {NULL}};
const Segment SGI335149[] = {{&S19990[74], 5}, {NULL}};
const Segment SGI335150[] = {{&S19990[12], 14}, {&S19990[37], 42}, {NULL}};
const Segment SGI335151[] = {{&S19990[12], 14}, {NULL}};
const Segment SGI335152[] = {{&S19991[6], 4}, {NULL}};
const Segment SGI335153[] = {{&S19991[75], 5}, {NULL}};
const Segment SGI335154[] = {{&S19991[12], 14}, {&S19991[38], 42}, {NULL}};
const Segment SGI335155[] = {{&S19991[12], 14}, {NULL}};
const Segment SGI335156[] = {{&S19992[6], 4}, {NULL}};
const Segment SGI335157[] = {{&S19992[75], 5}, {NULL}};
const Segment SGI335158[] = {{&S19992[12], 14}, {&S19992[38], 42}, {NULL}};
const Segment SGI335159[] = {{&S19992[12], 14}, {NULL}};
const Segment SGI335160[] = {{&S19993[6], 4}, {NULL}};
const Segment SGI335161[] = {{&S19993[75], 5}, {NULL}};
const Segment SGI335162[] = {{&S19993[12], 14}, {&S19993[37], 43}, {NULL}};
const Segment SGI335163[] = {{&S19993[12], 14}, {NULL}};
const Segment SGI335164[] = {{&S19994[6], 4}, {NULL}};
const Segment SGI335165[] = {{&S19994[74], 5}, {NULL}};
const Segment SGI335166[] = {{&S19994[12], 14}, {&S19994[37], 42}, {NULL}};
const Segment SGI335167[] = {{&S19994[12], 14}, {NULL}};
const Segment SGI335168[] = {{&S19995[6], 4}, {NULL}};
const Segment SGI335169[] = {{&S19995[74], 5}, {NULL}};
const Segment SGI335170[] = {{&S19995[12], 14}, {&S19995[37], 42}, {NULL}};
const Segment SGI335171[] = {{&S19995[12], 14}, {NULL}};
const Segment SGI335172[] = {{&S19996[6], 4}, {NULL}};
const Segment SGI335173[] = {{&S19996[74], 5}, {NULL}};
const Segment SGI335174[] = {{&S19996[12], 14}, {&S19996[37], 42}, {NULL}};
const Segment SGI335175[] = {{&S19996[12], 14}, {NULL}};
const Segment SGI335176[] = {{&S19997[6], 4}, {NULL}};
const Segment SGI335177[] = {{&S19997[74], 5}, {NULL}};
const Segment SGI335178[] = {{&S19997[12], 14}, {&S19997[37], 42}, {NULL}};
const Segment SGI335179[] = {{&S19997[12], 14}, {NULL}};
const Segment SGI335180[] = {{&S19998[6], 4}, {NULL}};
const Segment SGI335181[] = {{&S19998[74], 5}, {NULL}};
const Segment SGI335182[] = {{&S19998[12], 14}, {&S19998[37], 42}, {NULL}};
const Segment SGI335183[] = {{&S19998[12], 14}, {NULL}};
const Segment SGI335184[] = {{&S19999[6], 4}, {NULL}};
const Segment SGI335185[] = {{&S19999[75], 5}, {NULL}};
const Segment SGI335186[] = {{&S19999[12], 14}, {&S19999[38], 42}, {NULL}};
const Segment SGI335187[] = {{&S19999[12], 14}, {NULL}};
const Segment SGI335188[] = {{&S20000[6], 4}, {NULL}};
const Segment SGI335189[] = {{&S20000[52], 5}, {NULL}};
const Segment SGI335190[] = {{&S20000[12], 62}, {NULL}};
const Segment SGI335191[] = {{&S20000[12], 14}, {NULL}};
const Segment SGI335192[] = {{&S20001[6], 4}, {NULL}};
const Segment SGI335193[] = {{&S20001[74], 5}, {NULL}};
const Segment SGI335194[] = {{&S20001[12], 14}, {&S20001[37], 42}, {NULL}};
const Segment SGI335195[] = {{&S20001[12], 14}, {NULL}};
const Segment SGI335196[] = {{&S20002[6], 4}, {NULL}};
const Segment SGI335197[] = {{&S20002[74], 5}, {NULL}};
const Segment SGI335198[] = {{&S20002[12], 14}, {&S20002[37], 42}, {NULL}};
const Segment SGI335199[] = {{&S20002[12], 14}, {NULL}};
const Segment SGI335200[] = {{&S20003[6], 4}, {NULL}};
const Segment SGI335201[] = {{&S20003[74], 5}, {NULL}};
const Segment SGI335202[] = {{&S20003[12], 14}, {&S20003[37], 42}, {NULL}};
const Segment SGI335203[] = {{&S20003[12], 14}, {NULL}};
const Segment SGI335204[] = {{&S20005[6], 4}, {NULL}};
const Segment SGI335205[] = {{&S20005[75], 5}, {NULL}};
const Segment SGI335206[] = {{&S20005[12], 14}, {&S20005[38], 42}, {NULL}};
const Segment SGI335207[] = {{&S20005[12], 14}, {NULL}};
const Segment SGI335208[] = {{&S20006[6], 4}, {NULL}};
const Segment SGI335209[] = {{&S20006[74], 5}, {NULL}};
const Segment SGI335210[] = {{&S20006[12], 14}, {&S20006[37], 42}, {NULL}};
const Segment SGI335211[] = {{&S20006[12], 14}, {NULL}};
const Segment SGI335212[] = {{&S20007[6], 4}, {NULL}};
const Segment SGI335213[] = {{&S20007[74], 5}, {NULL}};
const Segment SGI335214[] = {{&S20007[12], 14}, {&S20007[37], 42}, {NULL}};
const Segment SGI335215[] = {{&S20007[12], 14}, {NULL}};
const Segment SGI335216[] = {{&S20008[6], 4}, {NULL}};
const Segment SGI335217[] = {{&S20008[74], 5}, {NULL}};
const Segment SGI335218[] = {{&S20008[12], 14}, {&S20008[37], 42}, {NULL}};
const Segment SGI335219[] = {{&S20008[12], 14}, {NULL}};
const Segment SGI335220[] = {{&S20009[6], 4}, {NULL}};
const Segment SGI335221[] = {{&S20009[74], 5}, {NULL}};
const Segment SGI335222[] = {{&S20009[12], 14}, {&S20009[37], 42}, {NULL}};
const Segment SGI335223[] = {{&S20009[12], 14}, {NULL}};
const Segment SGI335224[] = {{&S20010[6], 4}, {NULL}};
const Segment SGI335225[] = {{&S20010[75], 5}, {NULL}};
const Segment SGI335226[] = {{&S20010[12], 14}, {&S20010[37], 43}, {NULL}};
const Segment SGI335227[] = {{&S20010[12], 14}, {NULL}};
const Segment SGI335228[] = {{&S20011[6], 4}, {NULL}};
const Segment SGI335229[] = {{&S20011[74], 5}, {NULL}};
const Segment SGI335230[] = {{&S20011[12], 14}, {&S20011[37], 42}, {NULL}};
const Segment SGI335231[] = {{&S20011[12], 14}, {NULL}};
const Segment SGI335232[] = {{&S20012[6], 4}, {NULL}};
const Segment SGI335233[] = {{&S20012[74], 5}, {NULL}};
const Segment SGI335234[] = {{&S20012[12], 14}, {&S20012[37], 42}, {NULL}};
const Segment SGI335235[] = {{&S20012[12], 14}, {NULL}};
const Segment SGI335236[] = {{&S20013[6], 4}, {NULL}};
const Segment SGI335237[] = {{&S20013[74], 5}, {NULL}};
const Segment SGI335238[] = {{&S20013[12], 14}, {&S20013[37], 42}, {NULL}};
const Segment SGI335239[] = {{&S20013[12], 14}, {NULL}};
const Segment SGI335240[] = {{&S20014[6], 4}, {NULL}};
const Segment SGI335241[] = {{&S20014[74], 5}, {NULL}};
const Segment SGI335242[] = {{&S20014[12], 14}, {&S20014[37], 42}, {NULL}};
const Segment SGI335243[] = {{&S20014[12], 14}, {NULL}};
const Segment SGI335244[] = {{&S20015[6], 4}, {NULL}};
const Segment SGI335245[] = {{&S20015[74], 5}, {NULL}};
const Segment SGI335246[] = {{&S20015[12], 14}, {&S20015[37], 42}, {NULL}};
const Segment SGI335247[] = {{&S20015[12], 14}, {NULL}};
const Segment SGI335248[] = {{&S20016[6], 4}, {NULL}};
const Segment SGI335249[] = {{&S20016[75], 5}, {NULL}};
const Segment SGI335250[] = {{&S20016[12], 14}, {&S20016[37], 43}, {NULL}};
const Segment SGI335251[] = {{&S20016[12], 14}, {NULL}};
const Segment SGI335252[] = {{&S20017[6], 4}, {NULL}};
const Segment SGI335253[] = {{&S20017[74], 5}, {NULL}};
const Segment SGI335254[] = {{&S20017[12], 14}, {&S20017[37], 42}, {NULL}};
const Segment SGI335255[] = {{&S20017[12], 14}, {NULL}};
const Segment SGI335256[] = {{&S20018[6], 4}, {NULL}};
const Segment SGI335257[] = {{&S20018[74], 5}, {NULL}};
const Segment SGI335258[] = {{&S20018[12], 14}, {&S20018[37], 42}, {NULL}};
const Segment SGI335259[] = {{&S20018[12], 14}, {NULL}};
const Segment SGI335260[] = {{&S20019[6], 4}, {NULL}};
const Segment SGI335261[] = {{&S20019[75], 5}, {NULL}};
const Segment SGI335262[] = {{&S20019[12], 14}, {&S20019[37], 43}, {NULL}};
const Segment SGI335263[] = {{&S20019[12], 14}, {NULL}};
const Segment SGI335264[] = {{&S20020[6], 4}, {NULL}};
const Segment SGI335265[] = {{&S20020[74], 5}, {NULL}};
const Segment SGI335266[] = {{&S20020[12], 14}, {&S20020[37], 42}, {NULL}};
const Segment SGI335267[] = {{&S20020[12], 14}, {NULL}};
const Segment SGI335268[] = {{&S20021[6], 4}, {NULL}};
const Segment SGI335269[] = {{&S20021[74], 5}, {NULL}};
const Segment SGI335270[] = {{&S20021[12], 14}, {&S20021[37], 42}, {NULL}};
const Segment SGI335271[] = {{&S20021[12], 14}, {NULL}};
const Segment SGI335272[] = {{&S20022[6], 4}, {NULL}};
const Segment SGI335273[] = {{&S20022[59], 5}, {NULL}};
const Segment SGI335274[] = {{&S20022[12], 14}, {NULL}};
const Segment SGI335275[] = {{&S20023[6], 4}, {NULL}};
const Segment SGI335276[] = {{&S20023[75], 5}, {NULL}};
const Segment SGI335277[] = {{&S20023[12], 14}, {&S20023[38], 42}, {NULL}};
const Segment SGI335278[] = {{&S20023[12], 14}, {NULL}};
const Segment SGI335279[] = {{&S20024[6], 4}, {NULL}};
const Segment SGI335280[] = {{&S20024[74], 5}, {NULL}};
const Segment SGI335281[] = {{&S20024[12], 14}, {&S20024[37], 42}, {NULL}};
const Segment SGI335282[] = {{&S20024[12], 14}, {NULL}};
const Segment SGI335283[] = {{&S20025[6], 4}, {NULL}};
const Segment SGI335284[] = {{&S20025[101], 5}, {NULL}};
const Segment SGI335285[] = {{&S20025[12], 14}, {&S20025[37], 12}, {NULL}};
const Segment SGI335286[] = {{&S20025[12], 14}, {NULL}};
const Segment SGI335287[] = {{&S20026[6], 4}, {NULL}};
const Segment SGI335288[] = {{&S20026[74], 5}, {NULL}};
const Segment SGI335289[] = {{&S20026[12], 14}, {&S20026[37], 42}, {NULL}};
const Segment SGI335290[] = {{&S20026[12], 14}, {NULL}};
const Segment SGI335291[] = {{&S45421[5], 5}, {NULL}};
const Segment SGI335292[] = {{&S20027[6], 4}, {NULL}};
const Segment SGI335293[] = {{&S20027[74], 5}, {NULL}};
const Segment SGI335294[] = {{&S20027[12], 14}, {&S20027[37], 42}, {NULL}};
const Segment SGI335295[] = {{&S20027[12], 14}, {NULL}};
const Segment SGI335296[] = {{&S45422[5], 7}, {NULL}};
const Segment SGI335297[] = {{&S20028[6], 4}, {NULL}};
const Segment SGI335298[] = {{&S20028[75], 5}, {NULL}};
const Segment SGI335299[] = {{&S20028[12], 14}, {&S20028[37], 43}, {NULL}};
const Segment SGI335300[] = {{&S20028[12], 14}, {NULL}};
const Segment SGI335301[] = {{&S20029[6], 4}, {NULL}};
const Segment SGI335302[] = {{&S20029[74], 5}, {NULL}};
const Segment SGI335303[] = {{&S20029[12], 14}, {&S20029[37], 42}, {NULL}};
const Segment SGI335304[] = {{&S20029[12], 14}, {NULL}};
const Segment SGI335305[] = {{&S20030[6], 4}, {NULL}};
const Segment SGI335306[] = {{&S20030[74], 5}, {NULL}};
const Segment SGI335307[] = {{&S20030[12], 14}, {&S20030[37], 42}, {NULL}};
const Segment SGI335308[] = {{&S20030[12], 14}, {NULL}};
const Segment SGI335309[] = {{&S20031[6], 4}, {NULL}};
const Segment SGI335310[] = {{&S20031[74], 5}, {NULL}};
const Segment SGI335311[] = {{&S20031[12], 14}, {&S20031[37], 42}, {NULL}};
const Segment SGI335312[] = {{&S20031[12], 14}, {NULL}};
const Segment SGI335313[] = {{&S20032[6], 4}, {NULL}};
const Segment SGI335314[] = {{&S20032[74], 5}, {NULL}};
const Segment SGI335315[] = {{&S20032[12], 14}, {&S20032[37], 42}, {NULL}};
const Segment SGI335316[] = {{&S20032[12], 14}, {NULL}};
const Segment SGI335317[] = {{&S20033[6], 4}, {NULL}};
const Segment SGI335318[] = {{&S20033[74], 5}, {NULL}};
const Segment SGI335319[] = {{&S20033[12], 14}, {&S20033[37], 42}, {NULL}};
const Segment SGI335320[] = {{&S20033[12], 14}, {NULL}};
const Segment SGI335321[] = {{&S20034[6], 4}, {NULL}};
const Segment SGI335322[] = {{&S20034[75], 5}, {NULL}};
const Segment SGI335323[] = {{&S20034[12], 14}, {&S20034[38], 42}, {NULL}};
const Segment SGI335324[] = {{&S20034[12], 14}, {NULL}};
const Segment SGI335325[] = {{&S20035[6], 4}, {NULL}};
const Segment SGI335326[] = {{&S20035[74], 5}, {NULL}};
const Segment SGI335327[] = {{&S20035[12], 14}, {&S20035[37], 42}, {NULL}};
const Segment SGI335328[] = {{&S20035[12], 14}, {NULL}};
const Segment SGI335329[] = {{&S20036[6], 4}, {NULL}};
const Segment SGI335330[] = {{&S20036[75], 5}, {NULL}};
const Segment SGI335331[] = {{&S20036[12], 14}, {&S20036[38], 42}, {NULL}};
const Segment SGI335332[] = {{&S20036[12], 14}, {NULL}};
const Segment SGI335333[] = {{&S20037[6], 4}, {NULL}};
const Segment SGI335334[] = {{&S20037[75], 5}, {NULL}};
const Segment SGI335335[] = {{&S20037[12], 14}, {&S20037[37], 43}, {NULL}};
const Segment SGI335336[] = {{&S20037[12], 14}, {NULL}};
const Segment SGI335337[] = {{&S20038[6], 4}, {NULL}};
const Segment SGI335338[] = {{&S20038[75], 5}, {NULL}};
const Segment SGI335339[] = {{&S20038[12], 14}, {&S20038[38], 42}, {NULL}};
const Segment SGI335340[] = {{&S20038[12], 14}, {NULL}};
const Segment SGI335341[] = {{&S20039[6], 4}, {NULL}};
const Segment SGI335342[] = {{&S20039[74], 5}, {NULL}};
const Segment SGI335343[] = {{&S20039[12], 14}, {&S20039[37], 42}, {NULL}};
const Segment SGI335344[] = {{&S20039[12], 14}, {NULL}};
const Segment SGI335345[] = {{&S45436[5], 5}, {NULL}};
const Segment SGI335346[] = {{&S20040[6], 4}, {NULL}};
const Segment SGI335347[] = {{&S20040[74], 5}, {NULL}};
const Segment SGI335348[] = {{&S20040[12], 14}, {&S20040[37], 42}, {NULL}};
const Segment SGI335349[] = {{&S45437[5], 5}, {&S45437[68], 5}, {NULL}};
const Segment SGI335350[] = {{&S20040[12], 14}, {NULL}};
const Segment SGI335351[] = {{&S20041[6], 4}, {NULL}};
const Segment SGI335352[] = {{&S45438[5], 5}, {&S45438[5], 5}, {NULL}};
const Segment SGI335353[] = {{&S20041[74], 5}, {NULL}};
const Segment SGI335354[] = {{&S20041[12], 14}, {&S20041[37], 42}, {NULL}};
const Segment SGI335355[] = {{&S20041[12], 14}, {NULL}};
const Segment SGI335356[] = {{&S20042[6], 4}, {NULL}};
const Segment SGI335357[] = {{&S20042[74], 5}, {NULL}};
const Segment SGI335358[] = {{&S20042[12], 14}, {&S20042[37], 42}, {NULL}};
const Segment SGI335359[] = {{&S20042[12], 14}, {NULL}};
const Segment SGI335360[] = {{&S20043[6], 4}, {NULL}};
const Segment SGI335361[] = {{&S20043[74], 5}, {NULL}};
const Segment SGI335362[] = {{&S20043[12], 14}, {&S20043[37], 42}, {NULL}};
const Segment SGI335363[] = {{&S20043[12], 14}, {NULL}};
const Segment SGI335364[] = {{&S20044[6], 4}, {NULL}};
const Segment SGI335365[] = {{&S20044[74], 5}, {NULL}};
const Segment SGI335366[] = {{&S20044[12], 14}, {&S20044[37], 42}, {NULL}};
const Segment SGI335367[] = {{&S20044[12], 14}, {NULL}};
const Segment SGI335368[] = {{&S20045[6], 4}, {NULL}};
const Segment SGI335369[] = {{&S20045[74], 5}, {NULL}};
const Segment SGI335370[] = {{&S20045[12], 14}, {&S20045[37], 42}, {NULL}};
const Segment SGI335371[] = {{&S20045[12], 14}, {NULL}};
const Segment SGI335372[] = {{&S20046[6], 4}, {NULL}};
const Segment SGI335373[] = {{&S20046[102], 5}, {NULL}};
const Segment SGI335374[] = {{&S20046[12], 14}, {&S20046[37], 12}, {NULL}};
const Segment SGI335375[] = {{&S20046[12], 14}, {NULL}};
const Segment SGI335376[] = {{&S20047[6], 4}, {NULL}};
const Segment SGI335377[] = {{&S20047[75], 5}, {NULL}};
const Segment SGI335378[] = {{&S20047[12], 14}, {&S20047[37], 43}, {NULL}};
const Segment SGI335379[] = {{&S20047[12], 14}, {NULL}};
const Segment SGI335380[] = {{&S20048[6], 4}, {NULL}};
const Segment SGI335381[] = {{&S20048[74], 5}, {NULL}};
const Segment SGI335382[] = {{&S20048[12], 14}, {&S20048[37], 42}, {NULL}};
const Segment SGI335383[] = {{&S20048[12], 14}, {NULL}};
const Segment SGI335384[] = {{&S20049[6], 4}, {NULL}};
const Segment SGI335385[] = {{&S20049[74], 5}, {NULL}};
const Segment SGI335386[] = {{&S20049[12], 14}, {&S20049[37], 42}, {NULL}};
const Segment SGI335387[] = {{&S20049[12], 14}, {NULL}};
const Segment SGI335388[] = {{&S20050[6], 4}, {NULL}};
const Segment SGI335389[] = {{&S20050[75], 5}, {NULL}};
const Segment SGI335390[] = {{&S20050[12], 14}, {&S20050[37], 43}, {NULL}};
const Segment SGI335391[] = {{&S20050[12], 14}, {NULL}};
const Segment SGI335392[] = {{&S20051[6], 4}, {NULL}};
const Segment SGI335393[] = {{&S20051[74], 5}, {NULL}};
const Segment SGI335394[] = {{&S20051[12], 14}, {&S20051[37], 42}, {NULL}};
const Segment SGI335395[] = {{&S20051[12], 14}, {NULL}};
const Segment SGI335396[] = {{&S20052[6], 4}, {NULL}};
const Segment SGI335397[] = {{&S20052[75], 5}, {NULL}};
const Segment SGI335398[] = {{&S20052[12], 14}, {&S20052[38], 42}, {NULL}};
const Segment SGI335399[] = {{&S20052[12], 14}, {NULL}};
const Segment SGI335400[] = {{&S20053[6], 4}, {NULL}};
const Segment SGI335401[] = {{&S20053[74], 5}, {NULL}};
const Segment SGI335402[] = {{&S20053[12], 14}, {&S20053[37], 42}, {NULL}};
const Segment SGI335403[] = {{&S20053[12], 14}, {NULL}};
const Segment SGI335404[] = {{&S20054[6], 4}, {NULL}};
const Segment SGI335405[] = {{&S20054[74], 5}, {NULL}};
const Segment SGI335406[] = {{&S20054[12], 14}, {&S20054[37], 42}, {NULL}};
const Segment SGI335407[] = {{&S20054[12], 14}, {NULL}};
const Segment SGI335408[] = {{&S20055[6], 4}, {NULL}};
const Segment SGI335409[] = {{&S20055[74], 5}, {NULL}};
const Segment SGI335410[] = {{&S20055[12], 14}, {&S20055[37], 42}, {NULL}};
const Segment SGI335411[] = {{&S20055[12], 14}, {NULL}};
const Segment SGI335412[] = {{&S20056[6], 4}, {NULL}};
const Segment SGI335413[] = {{&S20056[74], 5}, {NULL}};
const Segment SGI335414[] = {{&S20056[12], 14}, {&S20056[37], 42}, {NULL}};
const Segment SGI335415[] = {{&S20056[12], 14}, {NULL}};
const Segment SGI335416[] = {{&S20057[6], 4}, {NULL}};
const Segment SGI335417[] = {{&S20057[74], 5}, {NULL}};
const Segment SGI335418[] = {{&S20057[12], 14}, {&S20057[37], 42}, {NULL}};
const Segment SGI335419[] = {{&S20057[12], 14}, {NULL}};
const Segment SGI335420[] = {{&S20058[6], 4}, {NULL}};
const Segment SGI335421[] = {{&S20058[64], 5}, {NULL}};
const Segment SGI335422[] = {{&S20058[12], 14}, {&S20058[75], 4}, {NULL}};
const Segment SGI335423[] = {{&S20058[12], 14}, {NULL}};
const Segment SGI335424[] = {{&S20059[6], 4}, {NULL}};
const Segment SGI335425[] = {{&S20059[74], 5}, {NULL}};
const Segment SGI335426[] = {{&S20059[12], 14}, {&S20059[37], 42}, {NULL}};
const Segment SGI335427[] = {{&S20059[12], 14}, {NULL}};
const Segment SGI335428[] = {{&S20060[6], 4}, {NULL}};
const Segment SGI335429[] = {{&S20060[75], 5}, {NULL}};
const Segment SGI335430[] = {{&S20060[12], 14}, {&S20060[38], 42}, {NULL}};
const Segment SGI335431[] = {{&S20060[12], 14}, {NULL}};
const Segment SGI335432[] = {{&S20061[6], 4}, {NULL}};
const Segment SGI335433[] = {{&S20061[75], 5}, {NULL}};
const Segment SGI335434[] = {{&S20061[12], 14}, {&S20061[37], 43}, {NULL}};
const Segment SGI335435[] = {{&S20061[12], 14}, {NULL}};
const Segment SGI335436[] = {{&S20062[6], 4}, {NULL}};
const Segment SGI335437[] = {{&S20062[74], 5}, {NULL}};
const Segment SGI335438[] = {{&S20062[12], 14}, {&S20062[37], 42}, {NULL}};
const Segment SGI335439[] = {{&S20062[12], 14}, {NULL}};
const Segment SGI335440[] = {{&S20063[6], 4}, {NULL}};
const Segment SGI335441[] = {{&S20063[75], 5}, {NULL}};
const Segment SGI335442[] = {{&S20063[12], 14}, {&S20063[37], 43}, {NULL}};
const Segment SGI335443[] = {{&S20063[12], 14}, {NULL}};
const Segment SGI335444[] = {{&S20064[6], 4}, {NULL}};
const Segment SGI335445[] = {{&S20064[75], 5}, {NULL}};
const Segment SGI335446[] = {{&S20064[12], 14}, {&S20064[38], 42}, {NULL}};
const Segment SGI335447[] = {{&S20064[12], 14}, {NULL}};
const Segment SGI335448[] = {{&S20065[6], 4}, {NULL}};
const Segment SGI335449[] = {{&S20065[75], 5}, {NULL}};
const Segment SGI335450[] = {{&S20065[12], 14}, {&S20065[37], 43}, {NULL}};
const Segment SGI335451[] = {{&S20065[12], 14}, {NULL}};
const Segment SGI335452[] = {{&S20066[6], 4}, {NULL}};
const Segment SGI335453[] = {{&S20066[75], 5}, {NULL}};
const Segment SGI335454[] = {{&S20066[12], 14}, {&S20066[37], 43}, {NULL}};
const Segment SGI335455[] = {{&S20066[12], 14}, {NULL}};
const Segment SGI335456[] = {{&S20067[6], 4}, {NULL}};
const Segment SGI335457[] = {{&S20067[74], 5}, {NULL}};
const Segment SGI335458[] = {{&S20067[12], 14}, {&S20067[37], 42}, {NULL}};
const Segment SGI335459[] = {{&S20067[12], 14}, {NULL}};
const Segment SGI335460[] = {{&S20068[6], 4}, {NULL}};
const Segment SGI335461[] = {{&S20068[74], 5}, {NULL}};
const Segment SGI335462[] = {{&S20068[12], 14}, {&S20068[37], 42}, {NULL}};
const Segment SGI335463[] = {{&S20068[12], 14}, {NULL}};
const Segment SGI335464[] = {{&S20069[6], 4}, {NULL}};
const Segment SGI335465[] = {{&S20069[74], 5}, {NULL}};
const Segment SGI335466[] = {{&S20069[12], 14}, {&S20069[37], 42}, {NULL}};
const Segment SGI335467[] = {{&S20069[12], 14}, {NULL}};
const Segment SGI335468[] = {{&S20070[6], 4}, {NULL}};
const Segment SGI335469[] = {{&S20070[102], 5}, {NULL}};
const Segment SGI335470[] = {{&S20070[12], 14}, {&S20070[37], 12}, {NULL}};
const Segment SGI335471[] = {{&S20070[12], 14}, {NULL}};
const Segment SGI335472[] = {{&S20071[6], 4}, {NULL}};
const Segment SGI335473[] = {{&S20071[75], 5}, {NULL}};
const Segment SGI335474[] = {{&S20071[12], 14}, {&S20071[37], 43}, {NULL}};
const Segment SGI335475[] = {{&S20071[12], 14}, {NULL}};
const Segment SGI335476[] = {{&S20072[6], 4}, {NULL}};
const Segment SGI335477[] = {{&S20072[75], 5}, {NULL}};
const Segment SGI335478[] = {{&S20072[12], 14}, {&S20072[37], 43}, {NULL}};
const Segment SGI335479[] = {{&S20072[12], 14}, {NULL}};
const Segment SGI335480[] = {{&S20073[6], 4}, {NULL}};
const Segment SGI335481[] = {{&S20073[74], 5}, {NULL}};
const Segment SGI335482[] = {{&S20073[12], 14}, {&S20073[37], 42}, {NULL}};
const Segment SGI335483[] = {{&S20073[12], 14}, {NULL}};
const Segment SGI335484[] = {{&S20074[6], 4}, {NULL}};
const Segment SGI335485[] = {{&S20074[75], 5}, {NULL}};
const Segment SGI335486[] = {{&S20074[12], 14}, {&S20074[37], 43}, {NULL}};
const Segment SGI335487[] = {{&S20074[12], 14}, {NULL}};
const Segment SGI335488[] = {{&S20075[6], 4}, {NULL}};
const Segment SGI335489[] = {{&S20075[74], 5}, {NULL}};
const Segment SGI335490[] = {{&S20075[12], 14}, {&S20075[37], 42}, {NULL}};
const Segment SGI335491[] = {{&S20075[12], 14}, {NULL}};
const Segment SGI335492[] = {{&S20076[6], 4}, {NULL}};
const Segment SGI335493[] = {{&S20076[102], 5}, {NULL}};
const Segment SGI335494[] = {{&S20076[12], 14}, {&S20076[37], 12}, {NULL}};
const Segment SGI335495[] = {{&S20076[12], 14}, {NULL}};
const Segment SGI335496[] = {{&S20077[6], 4}, {NULL}};
const Segment SGI335497[] = {{&S20077[75], 5}, {NULL}};
const Segment SGI335498[] = {{&S20077[12], 14}, {&S20077[38], 42}, {NULL}};
const Segment SGI335499[] = {{&S20077[12], 14}, {NULL}};
const Segment SGI335500[] = {{&S20078[6], 4}, {NULL}};
const Segment SGI335501[] = {{&S20078[75], 5}, {NULL}};
const Segment SGI335502[] = {{&S20078[12], 14}, {&S20078[38], 42}, {NULL}};
const Segment SGI335503[] = {{&S20078[12], 14}, {NULL}};
const Segment SGI335504[] = {{&S20079[6], 4}, {NULL}};
const Segment SGI335505[] = {{&S20079[75], 5}, {NULL}};
const Segment SGI335506[] = {{&S20079[12], 14}, {&S20079[38], 42}, {NULL}};
const Segment SGI335507[] = {{&S20079[12], 14}, {NULL}};
const Segment SGI335508[] = {{&S20080[6], 4}, {NULL}};
const Segment SGI335509[] = {{&S20080[74], 5}, {NULL}};
const Segment SGI335510[] = {{&S20080[12], 14}, {&S20080[37], 42}, {NULL}};
const Segment SGI335511[] = {{&S20080[12], 14}, {NULL}};
const Segment SGI335512[] = {{&S20081[6], 4}, {NULL}};
const Segment SGI335513[] = {{&S20081[52], 5}, {NULL}};
const Segment SGI335514[] = {{&S20081[12], 84}, {NULL}};
const Segment SGI335515[] = {{&S20081[12], 14}, {NULL}};
const Segment SGI335516[] = {{&S20082[6], 4}, {NULL}};
const Segment SGI335517[] = {{&S20082[74], 5}, {NULL}};
const Segment SGI335518[] = {{&S20082[12], 14}, {&S20082[37], 42}, {NULL}};
const Segment SGI335519[] = {{&S20082[12], 14}, {NULL}};
const Segment SGI335520[] = {{&S20083[6], 4}, {NULL}};
const Segment SGI335521[] = {{&S20083[74], 5}, {NULL}};
const Segment SGI335522[] = {{&S20083[12], 14}, {&S20083[37], 42}, {NULL}};
const Segment SGI335523[] = {{&S20083[12], 14}, {NULL}};
const Segment SGI335524[] = {{&S20084[6], 4}, {NULL}};
const Segment SGI335525[] = {{&S20084[74], 5}, {NULL}};
const Segment SGI335526[] = {{&S20084[12], 14}, {&S20084[37], 42}, {NULL}};
const Segment SGI335527[] = {{&S20084[12], 14}, {NULL}};
const Segment SGI335528[] = {{&S20085[6], 4}, {NULL}};
const Segment SGI335529[] = {{&S20085[74], 5}, {NULL}};
const Segment SGI335530[] = {{&S20085[12], 14}, {&S20085[37], 42}, {NULL}};
const Segment SGI335531[] = {{&S20085[12], 14}, {NULL}};
const Segment SGI335532[] = {{&S20086[6], 4}, {NULL}};
const Segment SGI335533[] = {{&S20086[74], 5}, {NULL}};
const Segment SGI335534[] = {{&S20086[12], 14}, {&S20086[37], 42}, {NULL}};
const Segment SGI335535[] = {{&S20086[12], 14}, {NULL}};
const Segment SGI335536[] = {{&S20087[6], 4}, {NULL}};
const Segment SGI335537[] = {{&S20087[75], 5}, {NULL}};
const Segment SGI335538[] = {{&S20087[12], 14}, {&S20087[37], 43}, {NULL}};
const Segment SGI335539[] = {{&S20087[12], 14}, {NULL}};
const Segment SGI335540[] = {{&S20088[6], 4}, {NULL}};
const Segment SGI335541[] = {{&S20088[75], 5}, {NULL}};
const Segment SGI335542[] = {{&S20088[12], 14}, {&S20088[37], 43}, {NULL}};
const Segment SGI335543[] = {{&S20088[12], 14}, {NULL}};
const Segment SGI335544[] = {{&S20089[6], 4}, {NULL}};
const Segment SGI335545[] = {{&S20089[75], 5}, {NULL}};
const Segment SGI335546[] = {{&S20089[12], 14}, {&S20089[38], 42}, {NULL}};
const Segment SGI335547[] = {{&S20089[12], 14}, {NULL}};
const Segment SGI335548[] = {{&S20090[6], 4}, {NULL}};
const Segment SGI335549[] = {{&S20090[74], 5}, {NULL}};
const Segment SGI335550[] = {{&S20090[12], 14}, {&S20090[37], 42}, {NULL}};
const Segment SGI335551[] = {{&S20090[12], 14}, {NULL}};
const Segment SGI335552[] = {{&S37071[28], 3}, {NULL}};
const Segment SGI335553[] = {{&S37071[102], 5}, {NULL}};
const Segment SGI335554[] = {{&S20091[6], 4}, {NULL}};
const Segment SGI335555[] = {{&S20091[75], 5}, {NULL}};
const Segment SGI335556[] = {{&S20091[12], 14}, {&S20091[38], 42}, {NULL}};
const Segment SGI335557[] = {{&S20091[12], 14}, {NULL}};
const Segment SGI335558[] = {{&S37072[28], 3}, {NULL}};
const Segment SGI335559[] = {{&S20092[6], 4}, {NULL}};
const Segment SGI335560[] = {{&S37072[102], 5}, {NULL}};
const Segment SGI335561[] = {{&S20092[74], 5}, {NULL}};
const Segment SGI335562[] = {{&S20092[12], 14}, {&S20092[37], 42}, {NULL}};
const Segment SGI335563[] = {{&S20092[12], 14}, {NULL}};
const Segment SGI335564[] = {{&S20093[6], 4}, {NULL}};
const Segment SGI335565[] = {{&S20093[74], 5}, {NULL}};
const Segment SGI335566[] = {{&S20093[12], 14}, {&S20093[37], 42}, {NULL}};
const Segment SGI335567[] = {{&S20093[12], 14}, {NULL}};
const Segment SGI335568[] = {{&S20094[6], 4}, {NULL}};
const Segment SGI335569[] = {{&S20094[75], 5}, {NULL}};
const Segment SGI335570[] = {{&S20094[12], 14}, {&S20094[37], 43}, {NULL}};
const Segment SGI335571[] = {{&S20094[12], 14}, {NULL}};
const Segment SGI335572[] = {{&S20095[6], 4}, {NULL}};
const Segment SGI335573[] = {{&S20095[74], 5}, {NULL}};
const Segment SGI335574[] = {{&S20095[12], 14}, {&S20095[37], 42}, {NULL}};
const Segment SGI335575[] = {{&S20095[12], 14}, {NULL}};
const Segment SGI335576[] = {{&S20096[6], 4}, {NULL}};
const Segment SGI335577[] = {{&S20096[52], 5}, {NULL}};
const Segment SGI335578[] = {{&S20096[12], 59}, {NULL}};
const Segment SGI335579[] = {{&S20096[12], 14}, {NULL}};
const Segment SGI335580[] = {{&S20097[6], 4}, {NULL}};
const Segment SGI335581[] = {{&S20097[75], 5}, {NULL}};
const Segment SGI335582[] = {{&S20097[12], 14}, {&S20097[37], 43}, {NULL}};
const Segment SGI335583[] = {{&S20097[12], 14}, {NULL}};
const Segment SGI335584[] = {{&S20098[6], 4}, {NULL}};
const Segment SGI335585[] = {{&S20098[75], 5}, {NULL}};
const Segment SGI335586[] = {{&S20098[12], 14}, {&S20098[38], 42}, {NULL}};
const Segment SGI335587[] = {{&S20098[12], 14}, {NULL}};
const Segment SGI335588[] = {{&S20099[6], 4}, {NULL}};
const Segment SGI335589[] = {{&S20099[74], 5}, {NULL}};
const Segment SGI335590[] = {{&S20099[12], 14}, {&S20099[37], 42}, {NULL}};
const Segment SGI335591[] = {{&S20099[12], 14}, {NULL}};
const Segment SGI335592[] = {{&S20100[6], 4}, {NULL}};
const Segment SGI335593[] = {{&S20100[75], 5}, {NULL}};
const Segment SGI335594[] = {{&S20100[12], 14}, {&S20100[37], 43}, {NULL}};
const Segment SGI335595[] = {{&S20100[12], 14}, {NULL}};
const Segment SGI335596[] = {{&S20101[6], 4}, {NULL}};
const Segment SGI335597[] = {{&S20101[74], 5}, {NULL}};
const Segment SGI335598[] = {{&S20101[12], 14}, {&S20101[37], 42}, {NULL}};
const Segment SGI335599[] = {{&S20101[12], 14}, {NULL}};
const Segment SGI335600[] = {{&S20102[6], 4}, {NULL}};
const Segment SGI335601[] = {{&S20102[74], 5}, {NULL}};
const Segment SGI335602[] = {{&S20102[12], 14}, {&S20102[37], 42}, {NULL}};
const Segment SGI335603[] = {{&S20102[12], 14}, {NULL}};
const Segment SGI335604[] = {{&S20103[6], 4}, {NULL}};
const Segment SGI335605[] = {{&S20103[74], 5}, {NULL}};
const Segment SGI335606[] = {{&S20103[12], 14}, {&S20103[37], 42}, {NULL}};
const Segment SGI335607[] = {{&S20103[12], 14}, {NULL}};
const Segment SGI335608[] = {{&S20104[6], 4}, {NULL}};
const Segment SGI335609[] = {{&S20104[74], 5}, {NULL}};
const Segment SGI335610[] = {{&S20104[12], 14}, {&S20104[37], 42}, {NULL}};
const Segment SGI335611[] = {{&S20104[12], 14}, {NULL}};
const Segment SGI335612[] = {{&S20105[6], 4}, {NULL}};
const Segment SGI335613[] = {{&S20105[74], 5}, {NULL}};
const Segment SGI335614[] = {{&S20105[12], 14}, {&S20105[37], 42}, {NULL}};
const Segment SGI335615[] = {{&S20105[12], 14}, {NULL}};
const Segment SGI335616[] = {{&S20106[6], 4}, {NULL}};
const Segment SGI335617[] = {{&S20106[74], 5}, {NULL}};
const Segment SGI335618[] = {{&S20106[12], 14}, {&S20106[37], 42}, {NULL}};
const Segment SGI335619[] = {{&S20106[12], 14}, {NULL}};
const Segment SGI335620[] = {{&S20107[6], 4}, {NULL}};
const Segment SGI335621[] = {{&S20107[74], 5}, {NULL}};
const Segment SGI335622[] = {{&S20107[12], 14}, {&S20107[37], 42}, {NULL}};
const Segment SGI335623[] = {{&S20107[12], 14}, {NULL}};
const Segment SGI335624[] = {{&S20108[6], 4}, {NULL}};
const Segment SGI335625[] = {{&S20108[74], 5}, {NULL}};
const Segment SGI335626[] = {{&S20108[12], 14}, {&S20108[37], 42}, {NULL}};
const Segment SGI335627[] = {{&S20108[12], 14}, {NULL}};
const Segment SGI335628[] = {{&S20109[6], 4}, {NULL}};
const Segment SGI335629[] = {{&S20109[74], 5}, {NULL}};
const Segment SGI335630[] = {{&S20109[12], 14}, {&S20109[37], 42}, {NULL}};
const Segment SGI335631[] = {{&S20109[12], 14}, {NULL}};
const Segment SGI335632[] = {{&S20110[6], 4}, {NULL}};
const Segment SGI335633[] = {{&S20110[75], 5}, {NULL}};
const Segment SGI335634[] = {{&S20110[12], 14}, {&S20110[38], 42}, {NULL}};
const Segment SGI335635[] = {{&S20110[12], 14}, {NULL}};
const Segment SGI335636[] = {{&S20111[6], 4}, {NULL}};
const Segment SGI335637[] = {{&S20111[75], 5}, {NULL}};
const Segment SGI335638[] = {{&S20111[12], 14}, {&S20111[37], 43}, {NULL}};
const Segment SGI335639[] = {{&S20111[12], 14}, {NULL}};
const Segment SGI335640[] = {{&S20112[6], 4}, {NULL}};
const Segment SGI335641[] = {{&S20112[75], 5}, {NULL}};
const Segment SGI335642[] = {{&S20112[12], 14}, {&S20112[37], 43}, {NULL}};
const Segment SGI335643[] = {{&S20112[12], 14}, {NULL}};
const Segment SGI335644[] = {{&S20113[6], 4}, {NULL}};
const Segment SGI335645[] = {{&S20113[74], 5}, {NULL}};
const Segment SGI335646[] = {{&S20113[12], 14}, {&S20113[37], 42}, {NULL}};
const Segment SGI335647[] = {{&S20113[12], 14}, {NULL}};
const Segment SGI335648[] = {{&S20114[6], 4}, {NULL}};
const Segment SGI335649[] = {{&S20114[74], 5}, {NULL}};
const Segment SGI335650[] = {{&S20114[12], 14}, {&S20114[37], 42}, {NULL}};
const Segment SGI335651[] = {{&S20114[12], 14}, {NULL}};
const Segment SGI335652[] = {{&S20115[6], 4}, {NULL}};
const Segment SGI335653[] = {{&S20115[74], 5}, {NULL}};
const Segment SGI335654[] = {{&S20115[12], 14}, {&S20115[37], 42}, {NULL}};
const Segment SGI335655[] = {{&S20115[12], 14}, {NULL}};
const Segment SGI335656[] = {{&S20116[6], 4}, {NULL}};
const Segment SGI335657[] = {{&S20116[74], 5}, {NULL}};
const Segment SGI335658[] = {{&S20116[12], 14}, {&S20116[37], 42}, {NULL}};
const Segment SGI335659[] = {{&S20116[12], 14}, {NULL}};
const Segment SGI335660[] = {{&S20117[6], 4}, {NULL}};
const Segment SGI335661[] = {{&S20117[74], 5}, {NULL}};
const Segment SGI335662[] = {{&S20117[12], 14}, {&S20117[37], 42}, {NULL}};
const Segment SGI335663[] = {{&S20117[12], 14}, {NULL}};
const Segment SGI335664[] = {{&S20118[6], 4}, {NULL}};
const Segment SGI335665[] = {{&S20118[75], 5}, {NULL}};
const Segment SGI335666[] = {{&S20118[12], 14}, {&S20118[37], 43}, {NULL}};
const Segment SGI335667[] = {{&S20118[12], 14}, {NULL}};
const Segment SGI335668[] = {{&S20119[6], 4}, {NULL}};
const Segment SGI335669[] = {{&S20119[102], 5}, {NULL}};
const Segment SGI335670[] = {{&S20119[12], 14}, {&S20119[37], 12}, {NULL}};
const Segment SGI335671[] = {{&S20119[12], 14}, {NULL}};
const Segment SGI335672[] = {{&S20120[6], 4}, {NULL}};
const Segment SGI335673[] = {{&S20120[75], 5}, {NULL}};
const Segment SGI335674[] = {{&S20120[12], 14}, {&S20120[37], 43}, {NULL}};
const Segment SGI335675[] = {{&S20120[12], 14}, {NULL}};
const Segment SGI335676[] = {{&S20121[6], 4}, {NULL}};
const Segment SGI335677[] = {{&S20121[74], 5}, {NULL}};
const Segment SGI335678[] = {{&S20121[12], 14}, {&S20121[37], 42}, {NULL}};
const Segment SGI335679[] = {{&S20121[12], 14}, {NULL}};
const Segment SGI335680[] = {{&S20122[6], 4}, {NULL}};
const Segment SGI335681[] = {{&S20122[74], 5}, {NULL}};
const Segment SGI335682[] = {{&S20122[12], 14}, {&S20122[37], 42}, {NULL}};
const Segment SGI335683[] = {{&S20122[12], 14}, {NULL}};
const Segment SGI335684[] = {{&S20123[6], 5}, {NULL}};
const Segment SGI335685[] = {{&S20123[82], 5}, {NULL}};
const Segment SGI335686[] = {{&S20123[13], 14}, {NULL}};
const Segment SGI335687[] = {{&S20124[6], 4}, {NULL}};
const Segment SGI335688[] = {{&S20124[74], 5}, {NULL}};
const Segment SGI335689[] = {{&S20124[12], 14}, {&S20124[37], 42}, {NULL}};
const Segment SGI335690[] = {{&S20124[12], 14}, {NULL}};
const Segment SGI335691[] = {{&S20125[6], 4}, {NULL}};
const Segment SGI335692[] = {{&S20125[52], 5}, {NULL}};
const Segment SGI335693[] = {{&S20125[12], 65}, {NULL}};
const Segment SGI335694[] = {{&S20125[12], 14}, {NULL}};
const Segment SGI335695[] = {{&S20126[6], 4}, {NULL}};
const Segment SGI335696[] = {{&S20126[74], 5}, {NULL}};
const Segment SGI335697[] = {{&S20126[12], 14}, {&S20126[37], 42}, {NULL}};
const Segment SGI335698[] = {{&S20126[12], 14}, {NULL}};
const Segment SGI335699[] = {{&S20127[6], 4}, {NULL}};
const Segment SGI335700[] = {{&S20127[74], 5}, {NULL}};
const Segment SGI335701[] = {{&S20127[12], 14}, {&S20127[37], 42}, {NULL}};
const Segment SGI335702[] = {{&S20127[12], 14}, {NULL}};
const Segment SGI335703[] = {{&S20128[6], 4}, {NULL}};
const Segment SGI335704[] = {{&S20128[74], 5}, {NULL}};
const Segment SGI335705[] = {{&S20128[12], 14}, {&S20128[37], 42}, {NULL}};
const Segment SGI335706[] = {{&S20128[12], 14}, {NULL}};
const Segment SGI335707[] = {{&S20129[6], 4}, {NULL}};
const Segment SGI335708[] = {{&S20129[74], 5}, {NULL}};
const Segment SGI335709[] = {{&S20129[12], 14}, {&S20129[37], 42}, {NULL}};
const Segment SGI335710[] = {{&S20129[12], 14}, {NULL}};
const Segment SGI335711[] = {{&S20130[6], 4}, {NULL}};
const Segment SGI335712[] = {{&S20130[74], 5}, {NULL}};
const Segment SGI335713[] = {{&S20130[12], 14}, {&S20130[37], 42}, {NULL}};
const Segment SGI335714[] = {{&S20130[12], 14}, {NULL}};
const Segment SGI335715[] = {{&S20131[6], 4}, {NULL}};
const Segment SGI335716[] = {{&S20131[101], 5}, {NULL}};
const Segment SGI335717[] = {{&S20131[12], 14}, {&S20131[37], 12}, {NULL}};
const Segment SGI335718[] = {{&S20131[12], 14}, {NULL}};
const Segment SGI335719[] = {{&S20132[6], 4}, {NULL}};
const Segment SGI335720[] = {{&S20132[74], 5}, {NULL}};
const Segment SGI335721[] = {{&S20132[12], 14}, {&S20132[37], 42}, {NULL}};
const Segment SGI335722[] = {{&S20132[12], 14}, {NULL}};
const Segment SGI335723[] = {{&S20133[6], 4}, {NULL}};
const Segment SGI335724[] = {{&S20133[74], 5}, {NULL}};
const Segment SGI335725[] = {{&S20133[12], 14}, {&S20133[37], 42}, {NULL}};
const Segment SGI335726[] = {{&S20133[12], 14}, {NULL}};
const Segment SGI335727[] = {{&S20134[6], 4}, {NULL}};
const Segment SGI335728[] = {{&S20134[74], 5}, {NULL}};
const Segment SGI335729[] = {{&S20134[12], 14}, {&S20134[37], 42}, {NULL}};
const Segment SGI335730[] = {{&S20134[12], 14}, {NULL}};
const Segment SGI335731[] = {{&S37130[28], 3}, {NULL}};
const Segment SGI335732[] = {{&S37130[102], 5}, {NULL}};
const Segment SGI335733[] = {{&S32137[45], 6}, {NULL}};
const Segment SGI335734[] = {{&S32137[37], 12}, {NULL}};
const Segment SGI335735[] = {{&S32138[46], 7}, {NULL}};
const Segment SGI335736[] = {{&S32138[8], 3}, {NULL}};
const Segment SGI335737[] = {{&S32139[46], 7}, {NULL}};
const Segment SGI335738[] = {{&S32139[8], 3}, {NULL}};
const Segment SGI335739[] = {{&S32140[45], 4}, {&S32140[145], 7}, {NULL}};
const Segment SGI335740[] = {{&S32140[37], 12}, {NULL}};
const Segment SGI335741[] = {{&S32141[46], 7}, {&S32141[149], 7}, {NULL}};
const Segment SGI335742[] = {{&S32141[8], 3}, {NULL}};
const Segment SGI335743[] = {{&S32142[46], 6}, {NULL}};
const Segment SGI335744[] = {{&S32142[8], 3}, {NULL}};
const Segment SGI335745[] = {{&S32143[46], 4}, {NULL}};
const Segment SGI335746[] = {{&S32143[8], 3}, {NULL}};
const Segment SGI335747[] = {{&S22089[57], 5}, {NULL}};
const Segment SGI335748[] = {{&S22089[13], 10}, {NULL}};
const Segment SGI335749[] = {{&S32144[46], 7}, {NULL}};
const Segment SGI335750[] = {{&S32144[8], 3}, {NULL}};
const Segment SGI335751[] = {{&S32147[46], 4}, {&S32147[146], 7}, {NULL}};
const Segment SGI335752[] = {{&S32147[8], 3}, {NULL}};
const Segment SGI335753[] = {{&S32148[46], 7}, {&S32148[149], 7}, {NULL}};
const Segment SGI335754[] = {{&S32148[8], 3}, {NULL}};
const Segment SGI335755[] = {{&S45618[5], 10}, {NULL}};
const Segment SGI335756[] = {{&S45619[5], 5}, {NULL}};
const Segment SGI335757[] = {{&S45620[5], 11}, {NULL}};
const Segment SGI335758[] = {{&S45621[5], 11}, {&S45621[5], 11}, {NULL}};
const Segment SGI335759[] = {{&S20261[6], 4}, {NULL}};
const Segment SGI335760[] = {{&S20261[80], 5}, {NULL}};
const Segment SGI335761[] = {{&S20261[12], 14}, {&S20261[40], 12}, {NULL}};
const Segment SGI335762[] = {{&S20261[12], 14}, {NULL}};
const Segment SGI335763[] = {{&S20262[6], 4}, {NULL}};
const Segment SGI335764[] = {{&S20262[80], 5}, {NULL}};
const Segment SGI335765[] = {{&S20262[12], 14}, {&S20262[40], 12}, {NULL}};
const Segment SGI335766[] = {{&S20262[12], 14}, {NULL}};
const Segment SGI335767[] = {{&S20263[6], 4}, {NULL}};
const Segment SGI335768[] = {{&S20263[80], 5}, {NULL}};
const Segment SGI335769[] = {{&S20263[12], 14}, {&S20263[40], 12}, {NULL}};
const Segment SGI335770[] = {{&S20263[12], 14}, {NULL}};
const Segment SGI335771[] = {{&S20264[6], 4}, {NULL}};
const Segment SGI335772[] = {{&S20264[80], 5}, {NULL}};
const Segment SGI335773[] = {{&S20264[12], 14}, {&S20264[40], 12}, {NULL}};
const Segment SGI335774[] = {{&S20264[12], 14}, {NULL}};
const Segment SGI335775[] = {{&S20265[6], 4}, {NULL}};
const Segment SGI335776[] = {{&S20265[60], 5}, {NULL}};
const Segment SGI335777[] = {{&S20265[12], 14}, {NULL}};
const Segment SGI335778[] = {{&S20266[6], 4}, {NULL}};
const Segment SGI335779[] = {{&S20266[57], 5}, {NULL}};
const Segment SGI335780[] = {{&S20266[12], 14}, {&S20266[68], 31}, {NULL}};
const Segment SGI335781[] = {{&S20266[12], 14}, {NULL}};
const Segment SGI335782[] = {{&S20267[6], 4}, {NULL}};
const Segment SGI335783[] = {{&S20267[80], 5}, {NULL}};
const Segment SGI335784[] = {{&S20267[12], 14}, {&S20267[40], 12}, {NULL}};
const Segment SGI335785[] = {{&S20267[12], 14}, {NULL}};
const Segment SGI335786[] = {{&S20268[6], 4}, {NULL}};
const Segment SGI335787[] = {{&S20268[80], 5}, {NULL}};
const Segment SGI335788[] = {{&S20268[12], 14}, {&S20268[40], 12}, {NULL}};
const Segment SGI335789[] = {{&S20268[12], 14}, {NULL}};
const Segment SGI335790[] = {{&S20269[6], 4}, {NULL}};
const Segment SGI335791[] = {{&S20269[80], 5}, {NULL}};
const Segment SGI335792[] = {{&S20269[12], 14}, {&S20269[40], 12}, {NULL}};
const Segment SGI335793[] = {{&S20269[12], 14}, {NULL}};
const Segment SGI335794[] = {{&S20270[6], 4}, {NULL}};
const Segment SGI335795[] = {{&S20270[80], 5}, {NULL}};
const Segment SGI335796[] = {{&S20270[12], 14}, {&S20270[40], 12}, {NULL}};
const Segment SGI335797[] = {{&S20270[12], 14}, {NULL}};
const Segment SGI335798[] = {{&S20271[6], 4}, {NULL}};
const Segment SGI335799[] = {{&S20271[57], 5}, {NULL}};
const Segment SGI335800[] = {{&S20271[12], 14}, {&S20271[68], 31}, {NULL}};
const Segment SGI335801[] = {{&S20271[12], 14}, {NULL}};
const Segment SGI335802[] = {{&S20272[6], 4}, {NULL}};
const Segment SGI335803[] = {{&S20272[80], 5}, {NULL}};
const Segment SGI335804[] = {{&S20272[12], 14}, {&S20272[40], 12}, {NULL}};
const Segment SGI335805[] = {{&S20272[12], 14}, {NULL}};
const Segment SGI335806[] = {{&S20273[6], 4}, {NULL}};
const Segment SGI335807[] = {{&S20273[79], 5}, {NULL}};
const Segment SGI335808[] = {{&S20273[12], 14}, {&S20273[40], 11}, {NULL}};
const Segment SGI335809[] = {{&S20273[12], 14}, {NULL}};
const Segment SGI335810[] = {{&S20274[6], 4}, {NULL}};
const Segment SGI335811[] = {{&S20274[79], 5}, {NULL}};
const Segment SGI335812[] = {{&S20274[12], 14}, {&S20274[40], 11}, {NULL}};
const Segment SGI335813[] = {{&S20274[12], 14}, {NULL}};
const Segment SGI335814[] = {{&S20275[6], 4}, {NULL}};
const Segment SGI335815[] = {{&S20275[80], 5}, {NULL}};
const Segment SGI335816[] = {{&S20275[12], 14}, {&S20275[40], 12}, {NULL}};
const Segment SGI335817[] = {{&S20275[12], 14}, {NULL}};
const Segment SGI335818[] = {{&S20276[6], 4}, {NULL}};
const Segment SGI335819[] = {{&S20276[57], 5}, {NULL}};
const Segment SGI335820[] = {{&S20276[12], 14}, {NULL}};
const Segment SGI335821[] = {{&S20278[6], 4}, {NULL}};
const Segment SGI335822[] = {{&S20278[80], 5}, {NULL}};
const Segment SGI335823[] = {{&S20278[12], 14}, {&S20278[40], 12}, {NULL}};
const Segment SGI335824[] = {{&S20278[12], 14}, {NULL}};
const Segment SGI335825[] = {{&S20279[6], 4}, {NULL}};
const Segment SGI335826[] = {{&S20279[79], 5}, {NULL}};
const Segment SGI335827[] = {{&S20279[12], 14}, {&S20279[40], 11}, {NULL}};
const Segment SGI335828[] = {{&S20279[12], 14}, {NULL}};
const Segment SGI335829[] = {{&S20280[6], 4}, {NULL}};
const Segment SGI335830[] = {{&S20280[80], 5}, {NULL}};
const Segment SGI335831[] = {{&S20280[12], 14}, {&S20280[40], 12}, {NULL}};
const Segment SGI335832[] = {{&S20280[12], 14}, {NULL}};
const Segment SGI335833[] = {{&S20281[6], 4}, {NULL}};
const Segment SGI335834[] = {{&S20281[80], 5}, {NULL}};
const Segment SGI335835[] = {{&S20281[12], 14}, {&S20281[40], 12}, {NULL}};
const Segment SGI335836[] = {{&S20281[12], 14}, {NULL}};
const Segment SGI335837[] = {{&S20282[6], 4}, {NULL}};
const Segment SGI335838[] = {{&S20282[80], 5}, {NULL}};
const Segment SGI335839[] = {{&S20282[12], 14}, {&S20282[40], 12}, {NULL}};
const Segment SGI335840[] = {{&S20282[12], 14}, {NULL}};
const Segment SGI335841[] = {{&S20283[6], 4}, {NULL}};
const Segment SGI335842[] = {{&S20283[79], 5}, {NULL}};
const Segment SGI335843[] = {{&S20283[12], 14}, {&S20283[40], 11}, {NULL}};
const Segment SGI335844[] = {{&S20283[12], 14}, {NULL}};
const Segment SGI335845[] = {{&S20286[6], 4}, {NULL}};
const Segment SGI335846[] = {{&S20286[79], 5}, {NULL}};
const Segment SGI335847[] = {{&S20286[12], 14}, {&S20286[40], 11}, {NULL}};
const Segment SGI335848[] = {{&S20286[12], 14}, {NULL}};
const Segment SGI335849[] = {{&S20287[6], 4}, {NULL}};
const Segment SGI335850[] = {{&S20287[79], 5}, {NULL}};
const Segment SGI335851[] = {{&S20287[12], 14}, {&S20287[40], 11}, {NULL}};
const Segment SGI335852[] = {{&S20287[12], 14}, {NULL}};
const Segment SGI335853[] = {{&S20288[6], 4}, {NULL}};
const Segment SGI335854[] = {{&S20288[79], 5}, {NULL}};
const Segment SGI335855[] = {{&S20288[12], 14}, {&S20288[40], 11}, {NULL}};
const Segment SGI335856[] = {{&S20288[12], 14}, {NULL}};
const Segment SGI335857[] = {{&S20289[6], 4}, {NULL}};
const Segment SGI335858[] = {{&S20289[61], 5}, {NULL}};
const Segment SGI335859[] = {{&S20289[12], 18}, {NULL}};
const Segment SGI335860[] = {{&S20289[12], 14}, {NULL}};
const Segment SGI335861[] = {{&S20290[6], 4}, {NULL}};
const Segment SGI335862[] = {{&S20290[79], 5}, {NULL}};
const Segment SGI335863[] = {{&S20290[12], 14}, {&S20290[40], 11}, {NULL}};
const Segment SGI335864[] = {{&S20290[12], 14}, {NULL}};
const Segment SGI335865[] = {{&S20291[6], 4}, {NULL}};
const Segment SGI335866[] = {{&S20291[80], 5}, {NULL}};
const Segment SGI335867[] = {{&S20291[12], 14}, {&S20291[41], 11}, {NULL}};
const Segment SGI335868[] = {{&S20291[12], 14}, {NULL}};
const Segment SGI335869[] = {{&S20292[6], 4}, {NULL}};
const Segment SGI335870[] = {{&S20292[80], 5}, {NULL}};
const Segment SGI335871[] = {{&S20292[12], 14}, {&S20292[41], 11}, {NULL}};
const Segment SGI335872[] = {{&S20292[12], 14}, {NULL}};
const Segment SGI335873[] = {{&S20293[6], 4}, {NULL}};
const Segment SGI335874[] = {{&S20293[79], 5}, {NULL}};
const Segment SGI335875[] = {{&S20293[12], 14}, {&S20293[40], 11}, {NULL}};
const Segment SGI335876[] = {{&S20293[12], 14}, {NULL}};
const Segment SGI335877[] = {{&S20294[6], 4}, {NULL}};
const Segment SGI335878[] = {{&S20294[79], 5}, {NULL}};
const Segment SGI335879[] = {{&S20294[12], 14}, {&S20294[40], 11}, {NULL}};
const Segment SGI335880[] = {{&S20294[12], 14}, {NULL}};
const Segment SGI335881[] = {{&S20296[6], 4}, {NULL}};
const Segment SGI335882[] = {{&S20296[79], 5}, {NULL}};
const Segment SGI335883[] = {{&S20296[12], 14}, {&S20296[40], 11}, {NULL}};
const Segment SGI335884[] = {{&S20296[12], 14}, {NULL}};
const Segment SGI335885[] = {{&S20297[6], 4}, {NULL}};
const Segment SGI335886[] = {{&S20297[79], 5}, {NULL}};
const Segment SGI335887[] = {{&S20297[12], 14}, {&S20297[40], 11}, {NULL}};
const Segment SGI335888[] = {{&S20297[12], 14}, {NULL}};
const Segment SGI335889[] = {{&S20298[6], 4}, {NULL}};
const Segment SGI335890[] = {{&S20298[79], 5}, {NULL}};
const Segment SGI335891[] = {{&S20298[12], 14}, {&S20298[40], 11}, {NULL}};
const Segment SGI335892[] = {{&S20298[12], 14}, {NULL}};
const Segment SGI335893[] = {{&S20299[6], 4}, {NULL}};
const Segment SGI335894[] = {{&S20299[79], 5}, {NULL}};
const Segment SGI335895[] = {{&S20299[12], 14}, {&S20299[40], 11}, {NULL}};
const Segment SGI335896[] = {{&S20299[12], 14}, {NULL}};
const Segment SGI335897[] = {{&S20300[6], 4}, {NULL}};
const Segment SGI335898[] = {{&S20300[80], 5}, {NULL}};
const Segment SGI335899[] = {{&S20300[12], 14}, {&S20300[40], 12}, {NULL}};
const Segment SGI335900[] = {{&S20300[12], 14}, {NULL}};
const Segment SGI335901[] = {{&S20301[6], 4}, {NULL}};
const Segment SGI335902[] = {{&S20301[80], 5}, {NULL}};
const Segment SGI335903[] = {{&S20301[12], 14}, {&S20301[40], 12}, {NULL}};
const Segment SGI335904[] = {{&S20301[12], 14}, {NULL}};
const Segment SGI335905[] = {{&S20302[6], 4}, {NULL}};
const Segment SGI335906[] = {{&S20302[79], 5}, {NULL}};
const Segment SGI335907[] = {{&S20302[12], 14}, {&S20302[40], 11}, {NULL}};
const Segment SGI335908[] = {{&S20302[12], 14}, {NULL}};
const Segment SGI335909[] = {{&S20303[6], 4}, {NULL}};
const Segment SGI335910[] = {{&S20303[80], 5}, {NULL}};
const Segment SGI335911[] = {{&S20303[12], 14}, {&S20303[40], 12}, {NULL}};
const Segment SGI335912[] = {{&S20303[12], 14}, {NULL}};
const Segment SGI335913[] = {{&S20304[6], 4}, {NULL}};
const Segment SGI335914[] = {{&S20304[80], 5}, {NULL}};
const Segment SGI335915[] = {{&S20304[12], 14}, {&S20304[41], 11}, {NULL}};
const Segment SGI335916[] = {{&S20304[12], 14}, {NULL}};
const Segment SGI335917[] = {{&S20305[6], 4}, {NULL}};
const Segment SGI335918[] = {{&S20305[79], 5}, {NULL}};
const Segment SGI335919[] = {{&S20305[12], 14}, {&S20305[40], 11}, {NULL}};
const Segment SGI335920[] = {{&S20305[12], 14}, {NULL}};
const Segment SGI335921[] = {{&S20307[6], 4}, {NULL}};
const Segment SGI335922[] = {{&S20307[79], 5}, {NULL}};
const Segment SGI335923[] = {{&S20307[12], 14}, {&S20307[40], 11}, {NULL}};
const Segment SGI335924[] = {{&S20307[12], 14}, {NULL}};
const Segment SGI335925[] = {{&S20308[6], 4}, {NULL}};
const Segment SGI335926[] = {{&S20308[79], 5}, {NULL}};
const Segment SGI335927[] = {{&S20308[12], 14}, {&S20308[40], 11}, {NULL}};
const Segment SGI335928[] = {{&S20308[12], 14}, {NULL}};
const Segment SGI335929[] = {{&S20309[6], 4}, {NULL}};
const Segment SGI335930[] = {{&S20309[79], 5}, {NULL}};
const Segment SGI335931[] = {{&S20309[12], 14}, {&S20309[40], 11}, {NULL}};
const Segment SGI335932[] = {{&S20309[12], 14}, {NULL}};
const Segment SGI335933[] = {{&S20310[6], 4}, {NULL}};
const Segment SGI335934[] = {{&S20310[80], 5}, {NULL}};
const Segment SGI335935[] = {{&S20310[12], 14}, {&S20310[41], 11}, {NULL}};
const Segment SGI335936[] = {{&S20310[12], 14}, {NULL}};
const Segment SGI335937[] = {{&S20311[6], 4}, {NULL}};
const Segment SGI335938[] = {{&S20311[80], 5}, {NULL}};
const Segment SGI335939[] = {{&S20311[12], 14}, {&S20311[41], 11}, {NULL}};
const Segment SGI335940[] = {{&S20311[12], 14}, {NULL}};
const Segment SGI335941[] = {{&S20312[6], 4}, {NULL}};
const Segment SGI335942[] = {{&S20312[80], 5}, {NULL}};
const Segment SGI335943[] = {{&S20312[12], 14}, {&S20312[41], 11}, {NULL}};
const Segment SGI335944[] = {{&S20312[12], 14}, {NULL}};
const Segment SGI335945[] = {{&S20313[6], 4}, {NULL}};
const Segment SGI335946[] = {{&S20313[80], 5}, {NULL}};
const Segment SGI335947[] = {{&S20313[12], 14}, {&S20313[41], 11}, {NULL}};
const Segment SGI335948[] = {{&S20313[12], 14}, {NULL}};
const Segment SGI335949[] = {{&S20314[6], 4}, {NULL}};
const Segment SGI335950[] = {{&S20314[80], 5}, {NULL}};
const Segment SGI335951[] = {{&S20314[12], 14}, {&S20314[41], 11}, {NULL}};
const Segment SGI335952[] = {{&S20314[12], 14}, {NULL}};
const Segment SGI335953[] = {{&S20315[6], 4}, {NULL}};
const Segment SGI335954[] = {{&S20315[79], 5}, {NULL}};
const Segment SGI335955[] = {{&S20315[12], 14}, {&S20315[40], 11}, {NULL}};
const Segment SGI335956[] = {{&S20315[12], 14}, {NULL}};
const Segment SGI335957[] = {{&S20316[6], 4}, {NULL}};
const Segment SGI335958[] = {{&S20316[79], 5}, {NULL}};
const Segment SGI335959[] = {{&S20316[12], 14}, {&S20316[40], 11}, {NULL}};
const Segment SGI335960[] = {{&S20316[12], 14}, {NULL}};
const Segment SGI335961[] = {{&S20317[6], 4}, {NULL}};
const Segment SGI335962[] = {{&S20317[80], 5}, {NULL}};
const Segment SGI335963[] = {{&S20317[12], 14}, {&S20317[41], 11}, {NULL}};
const Segment SGI335964[] = {{&S20317[12], 14}, {NULL}};
const Segment SGI335965[] = {{&S20318[6], 4}, {NULL}};
const Segment SGI335966[] = {{&S20318[79], 5}, {NULL}};
const Segment SGI335967[] = {{&S20318[12], 14}, {&S20318[40], 11}, {NULL}};
const Segment SGI335968[] = {{&S20318[12], 14}, {NULL}};
const Segment SGI335969[] = {{&S20319[6], 4}, {NULL}};
const Segment SGI335970[] = {{&S20319[80], 5}, {NULL}};
const Segment SGI335971[] = {{&S20319[12], 14}, {&S20319[41], 11}, {NULL}};
const Segment SGI335972[] = {{&S20319[12], 14}, {NULL}};
const Segment SGI335973[] = {{&S20320[6], 4}, {NULL}};
const Segment SGI335974[] = {{&S20320[79], 5}, {NULL}};
const Segment SGI335975[] = {{&S20320[12], 14}, {&S20320[40], 11}, {NULL}};
const Segment SGI335976[] = {{&S20320[12], 14}, {NULL}};
const Segment SGI335977[] = {{&S20321[6], 4}, {NULL}};
const Segment SGI335978[] = {{&S20321[79], 5}, {NULL}};
const Segment SGI335979[] = {{&S20321[12], 14}, {&S20321[40], 11}, {NULL}};
const Segment SGI335980[] = {{&S20321[12], 14}, {NULL}};
const Segment SGI335981[] = {{&S20322[6], 4}, {NULL}};
const Segment SGI335982[] = {{&S20322[79], 5}, {NULL}};
const Segment SGI335983[] = {{&S20322[12], 14}, {&S20322[40], 11}, {NULL}};
const Segment SGI335984[] = {{&S20322[12], 14}, {NULL}};
const Segment SGI335985[] = {{&S20323[6], 4}, {NULL}};
const Segment SGI335986[] = {{&S20323[79], 5}, {NULL}};
const Segment SGI335987[] = {{&S20323[12], 14}, {&S20323[40], 11}, {NULL}};
const Segment SGI335988[] = {{&S20323[12], 14}, {NULL}};
const Segment SGI335989[] = {{&S20324[6], 4}, {NULL}};
const Segment SGI335990[] = {{&S20324[79], 5}, {NULL}};
const Segment SGI335991[] = {{&S20324[12], 14}, {&S20324[40], 11}, {NULL}};
const Segment SGI335992[] = {{&S20324[12], 14}, {NULL}};
const Segment SGI335993[] = {{&S20325[6], 4}, {NULL}};
const Segment SGI335994[] = {{&S20325[80], 5}, {NULL}};
const Segment SGI335995[] = {{&S20325[12], 14}, {&S20325[40], 12}, {NULL}};
const Segment SGI335996[] = {{&S20325[12], 14}, {NULL}};
const Segment SGI335997[] = {{&S20326[6], 4}, {NULL}};
const Segment SGI335998[] = {{&S20326[79], 5}, {NULL}};
const Segment SGI335999[] = {{&S20326[12], 14}, {&S20326[40], 11}, {NULL}};
const Segment SGI336000[] = {{&S20326[12], 14}, {NULL}};
const Segment SGI336001[] = {{&S20327[6], 4}, {NULL}};
const Segment SGI336002[] = {{&S20327[80], 5}, {NULL}};
const Segment SGI336003[] = {{&S20327[12], 14}, {&S20327[41], 11}, {NULL}};
const Segment SGI336004[] = {{&S20327[12], 14}, {NULL}};
const Segment SGI336005[] = {{&S20328[6], 4}, {NULL}};
const Segment SGI336006[] = {{&S20328[79], 5}, {NULL}};
const Segment SGI336007[] = {{&S20328[12], 14}, {&S20328[40], 11}, {NULL}};
const Segment SGI336008[] = {{&S20328[12], 14}, {NULL}};
const Segment SGI336009[] = {{&S20329[6], 4}, {NULL}};
const Segment SGI336010[] = {{&S20329[79], 5}, {NULL}};
const Segment SGI336011[] = {{&S20329[12], 14}, {&S20329[40], 11}, {NULL}};
const Segment SGI336012[] = {{&S20329[12], 14}, {NULL}};
const Segment SGI336013[] = {{&S20330[6], 4}, {NULL}};
const Segment SGI336014[] = {{&S20330[79], 5}, {NULL}};
const Segment SGI336015[] = {{&S20330[12], 14}, {&S20330[40], 11}, {NULL}};
const Segment SGI336016[] = {{&S20330[12], 14}, {NULL}};
const Segment SGI336017[] = {{&S20331[6], 4}, {NULL}};
const Segment SGI336018[] = {{&S20331[79], 5}, {NULL}};
const Segment SGI336019[] = {{&S20331[12], 14}, {&S20331[40], 11}, {NULL}};
const Segment SGI336020[] = {{&S20331[12], 14}, {NULL}};
const Segment SGI336021[] = {{&S20332[6], 4}, {NULL}};
const Segment SGI336022[] = {{&S20332[79], 5}, {NULL}};
const Segment SGI336023[] = {{&S20332[12], 14}, {&S20332[40], 11}, {NULL}};
const Segment SGI336024[] = {{&S20332[12], 14}, {NULL}};
const Segment SGI336025[] = {{&S20333[6], 4}, {NULL}};
const Segment SGI336026[] = {{&S20333[57], 5}, {NULL}};
const Segment SGI336027[] = {{&S20333[12], 14}, {&S20333[68], 4}, {NULL}};
const Segment SGI336028[] = {{&S20333[12], 14}, {NULL}};
const Segment SGI336029[] = {{&S20334[6], 4}, {NULL}};
const Segment SGI336030[] = {{&S20334[80], 5}, {NULL}};
const Segment SGI336031[] = {{&S20334[12], 14}, {&S20334[41], 11}, {NULL}};
const Segment SGI336032[] = {{&S20334[12], 14}, {NULL}};
const Segment SGI336033[] = {{&S20335[6], 4}, {NULL}};
const Segment SGI336034[] = {{&S20335[57], 5}, {NULL}};
const Segment SGI336035[] = {{&S20335[12], 14}, {&S20335[68], 4}, {NULL}};
const Segment SGI336036[] = {{&S20335[12], 14}, {NULL}};
const Segment SGI336037[] = {{&S20336[6], 4}, {NULL}};
const Segment SGI336038[] = {{&S20336[79], 5}, {NULL}};
const Segment SGI336039[] = {{&S20336[12], 14}, {&S20336[40], 11}, {NULL}};
const Segment SGI336040[] = {{&S20336[12], 14}, {NULL}};
const Segment SGI336041[] = {{&S20337[6], 4}, {NULL}};
const Segment SGI336042[] = {{&S20337[79], 5}, {NULL}};
const Segment SGI336043[] = {{&S20337[12], 14}, {&S20337[40], 11}, {NULL}};
const Segment SGI336044[] = {{&S20337[12], 14}, {NULL}};
const Segment SGI336045[] = {{&S20338[6], 4}, {NULL}};
const Segment SGI336046[] = {{&S20338[79], 5}, {NULL}};
const Segment SGI336047[] = {{&S20338[12], 14}, {&S20338[40], 11}, {NULL}};
const Segment SGI336048[] = {{&S20338[12], 14}, {NULL}};
const Segment SGI336049[] = {{&S20339[6], 4}, {NULL}};
const Segment SGI336050[] = {{&S20339[79], 5}, {NULL}};
const Segment SGI336051[] = {{&S20339[12], 14}, {&S20339[40], 11}, {NULL}};
const Segment SGI336052[] = {{&S20339[12], 14}, {NULL}};
const Segment SGI336053[] = {{&S20340[6], 4}, {NULL}};
const Segment SGI336054[] = {{&S20340[79], 5}, {NULL}};
const Segment SGI336055[] = {{&S20340[12], 14}, {&S20340[40], 11}, {NULL}};
const Segment SGI336056[] = {{&S20340[12], 14}, {NULL}};
const Segment SGI336057[] = {{&S20341[6], 4}, {NULL}};
const Segment SGI336058[] = {{&S20341[79], 5}, {NULL}};
const Segment SGI336059[] = {{&S20341[12], 14}, {&S20341[40], 11}, {NULL}};
const Segment SGI336060[] = {{&S20341[12], 14}, {NULL}};
const Segment SGI336061[] = {{&S20342[6], 4}, {NULL}};
const Segment SGI336062[] = {{&S20342[79], 5}, {NULL}};
const Segment SGI336063[] = {{&S20342[12], 14}, {&S20342[40], 11}, {NULL}};
const Segment SGI336064[] = {{&S20342[12], 14}, {NULL}};
const Segment SGI336065[] = {{&S20343[6], 4}, {NULL}};
const Segment SGI336066[] = {{&S20343[79], 5}, {NULL}};
const Segment SGI336067[] = {{&S20343[12], 14}, {&S20343[40], 11}, {NULL}};
const Segment SGI336068[] = {{&S20343[12], 14}, {NULL}};
const Segment SGI336069[] = {{&S20344[6], 4}, {NULL}};
const Segment SGI336070[] = {{&S20344[79], 5}, {NULL}};
const Segment SGI336071[] = {{&S20344[12], 14}, {&S20344[40], 11}, {NULL}};
const Segment SGI336072[] = {{&S20344[12], 14}, {NULL}};
const Segment SGI336073[] = {{&S20345[6], 4}, {NULL}};
const Segment SGI336074[] = {{&S20345[79], 5}, {NULL}};
const Segment SGI336075[] = {{&S20345[12], 14}, {&S20345[40], 11}, {NULL}};
const Segment SGI336076[] = {{&S20345[12], 14}, {NULL}};
const Segment SGI336077[] = {{&S20346[6], 4}, {NULL}};
const Segment SGI336078[] = {{&S20346[79], 5}, {NULL}};
const Segment SGI336079[] = {{&S20346[12], 14}, {&S20346[40], 11}, {NULL}};
const Segment SGI336080[] = {{&S20346[12], 14}, {NULL}};
const Segment SGI336081[] = {{&S20347[6], 4}, {NULL}};
const Segment SGI336082[] = {{&S20347[79], 5}, {NULL}};
const Segment SGI336083[] = {{&S20347[12], 14}, {&S20347[40], 11}, {NULL}};
const Segment SGI336084[] = {{&S20347[12], 14}, {NULL}};
const Segment SGI336085[] = {{&S20348[6], 4}, {NULL}};
const Segment SGI336086[] = {{&S20348[79], 5}, {NULL}};
const Segment SGI336087[] = {{&S20348[12], 14}, {&S20348[40], 11}, {NULL}};
const Segment SGI336088[] = {{&S20348[12], 14}, {NULL}};
const Segment SGI336089[] = {{&S20349[6], 4}, {NULL}};
const Segment SGI336090[] = {{&S20349[79], 5}, {NULL}};
const Segment SGI336091[] = {{&S20349[12], 14}, {&S20349[40], 11}, {NULL}};
const Segment SGI336092[] = {{&S20349[12], 14}, {NULL}};
const Segment SGI336093[] = {{&S20350[6], 4}, {NULL}};
const Segment SGI336094[] = {{&S20350[79], 5}, {NULL}};
const Segment SGI336095[] = {{&S20350[12], 14}, {&S20350[40], 11}, {NULL}};
const Segment SGI336096[] = {{&S20350[12], 14}, {NULL}};
const Segment SGI336097[] = {{&S20351[6], 4}, {NULL}};
const Segment SGI336098[] = {{&S20351[79], 5}, {NULL}};
const Segment SGI336099[] = {{&S20351[12], 14}, {&S20351[40], 11}, {NULL}};
const Segment SGI336100[] = {{&S20351[12], 14}, {NULL}};
const Segment SGI336101[] = {{&S45870[5], 3}, {&S45870[5], 3}, {NULL}};
const Segment SGI336102[] = {{&S45870[109], 3}, {NULL}};
const Segment SGI336103[] = {{&S37414[28], 3}, {NULL}};
const Segment SGI336104[] = {{&S37414[102], 5}, {NULL}};
const Segment SGI336105[] = {{&S37444[28], 3}, {NULL}};
const Segment SGI336106[] = {{&S37444[102], 5}, {NULL}};
const Segment SGI336107[] = {{&S46003[100], 11}, {NULL}};
const Segment SGI336108[] = {{&S46061[5], 5}, {NULL}};
const Segment SGI336109[] = {{&S46062[5], 5}, {&S46062[5], 5}, {NULL}};
const Segment SGI336110[] = {{&S46063[5], 10}, {NULL}};
const Segment SGI336111[] = {{&S46064[5], 4}, {NULL}};
const Segment SGI336112[] = {{&S46074[100], 11}, {NULL}};
const Segment SGI336113[] = {{&S46075[5], 5}, {&S46075[97], 5}, {NULL}};
const Segment SGI336114[] = {{&S46075[113], 3}, {NULL}};
const Segment SGI336115[] = {{&S22525[61], 4}, {NULL}};
const Segment SGI336116[] = {{&S22525[13], 10}, {NULL}};
const Segment SGI336117[] = {{&S22534[61], 4}, {NULL}};
const Segment SGI336118[] = {{&S22534[13], 10}, {NULL}};
const Segment SGI336119[] = {{&S37711[28], 3}, {NULL}};
const Segment SGI336120[] = {{&S37711[102], 5}, {NULL}};
const Segment SGI336121[] = {{&S37712[28], 3}, {NULL}};
const Segment SGI336122[] = {{&S37712[102], 5}, {NULL}};
const Segment SGI336123[] = {{&S46342[5], 5}, {&S46342[5], 5}, {NULL}};
const Segment SGI336124[] = {{&S46343[5], 5}, {NULL}};
const Segment SGI336125[] = {{&S46493[5], 5}, {NULL}};
const Segment SGI336126[] = {{&S46494[5], 5}, {&S46494[68], 5}, {NULL}};
const Segment SGI336127[] = {{&S46495[5], 5}, {&S46495[5], 5}, {NULL}};
const Segment SGI336128[] = {{&S33193[41], 5}, {NULL}};
const Segment SGI336129[] = {{&S33193[33], 10}, {NULL}};
const Segment SGI336130[] = {{&S33194[42], 5}, {NULL}};
const Segment SGI336131[] = {{&S33194[33], 3}, {NULL}};
const Segment SGI336132[] = {{&S33195[42], 5}, {&S33195[142], 7}, {NULL}};
const Segment SGI336133[] = {{&S33195[33], 3}, {NULL}};
const Segment SGI336134[] = {{&S33196[42], 6}, {NULL}};
const Segment SGI336135[] = {{&S33196[33], 3}, {NULL}};
const Segment SGI336136[] = {{&S33260[42], 5}, {NULL}};
const Segment SGI336137[] = {{&S33260[33], 3}, {NULL}};
const Segment SGI336138[] = {{&S33261[42], 6}, {&S33261[143], 7}, {NULL}};
const Segment SGI336139[] = {{&S33261[33], 3}, {NULL}};
const Segment SGI336140[] = {{&S33262[42], 5}, {&S33262[142], 7}, {NULL}};
const Segment SGI336141[] = {{&S33262[33], 3}, {NULL}};
const Segment SGI336142[] = {{&S33263[42], 7}, {&S33263[144], 7}, {NULL}};
const Segment SGI336143[] = {{&S33263[33], 3}, {NULL}};
const Segment SGI336144[] = {{&S33264[41], 7}, {&S33264[143], 7}, {NULL}};
const Segment SGI336145[] = {{&S33264[33], 10}, {NULL}};
const Segment SGI336146[] = {{&S33286[46], 7}, {&S33286[149], 7}, {NULL}};
const Segment SGI336147[] = {{&S33286[8], 3}, {NULL}};
const Segment SGI336148[] = {{&S33287[46], 7}, {NULL}};
const Segment SGI336149[] = {{&S33287[8], 3}, {NULL}};
const Segment SGI336150[] = {{&S33288[46], 5}, {NULL}};
const Segment SGI336151[] = {{&S33288[8], 3}, {NULL}};
const Segment SGI336152[] = {{&S33289[46], 5}, {&S33289[147], 7}, {NULL}};
const Segment SGI336153[] = {{&S33289[8], 3}, {NULL}};
const Segment SGI336154[] = {{&S33302[46], 5}, {&S33302[146], 7}, {NULL}};
const Segment SGI336155[] = {{&S33302[37], 3}, {NULL}};
const Segment SGI336156[] = {{&S33303[46], 7}, {&S33303[148], 7}, {NULL}};
const Segment SGI336157[] = {{&S33303[37], 3}, {NULL}};
const Segment SGI336158[] = {{&S33304[46], 7}, {&S33304[148], 7}, {NULL}};
const Segment SGI336159[] = {{&S33304[37], 3}, {NULL}};
const Segment SGI336160[] = {{&S33305[46], 7}, {NULL}};
const Segment SGI336161[] = {{&S33305[37], 3}, {NULL}};
const Segment SGI336162[] = {{&S33306[46], 5}, {NULL}};
const Segment SGI336163[] = {{&S33306[37], 3}, {NULL}};
const Segment SGI336164[] = {{&S33307[5], 6}, {NULL}};
const Segment SGI336165[] = {{&S33307[48], 3}, {NULL}};
const Segment SGI336166[] = {{&S33308[46], 7}, {&S33308[148], 7}, {NULL}};
const Segment SGI336167[] = {{&S33308[37], 3}, {NULL}};
const Segment SGI336168[] = {{&S21106[6], 4}, {NULL}};
const Segment SGI336169[] = {{&S21106[12], 14}, {NULL}};
const Segment SGI336170[] = {{&S21108[6], 4}, {NULL}};
const Segment SGI336171[] = {{&S21108[12], 14}, {NULL}};
const Segment SGI336172[] = {{&S21110[6], 4}, {NULL}};
const Segment SGI336173[] = {{&S21110[12], 14}, {&S21110[38], 24}, {NULL}};
const Segment SGI336174[] = {{&S21110[12], 14}, {NULL}};
const Segment SGI336175[] = {{&S21112[6], 2}, {NULL}};
const Segment SGI336176[] = {{&S21112[11], 14}, {NULL}};
const Segment SGI336177[] = {{&S21113[6], 4}, {NULL}};
const Segment SGI336178[] = {{&S21113[12], 10}, {NULL}};
const Segment SGI336179[] = {{&S21114[6], 4}, {NULL}};
const Segment SGI336180[] = {{&S21114[12], 14}, {NULL}};
const Segment SGI336181[] = {{&S21115[6], 4}, {NULL}};
const Segment SGI336182[] = {{&S21115[12], 14}, {NULL}};
const Segment SGI336183[] = {{&S21117[6], 4}, {NULL}};
const Segment SGI336184[] = {{&S21117[12], 14}, {NULL}};
const Segment SGI336185[] = {{&S21118[6], 4}, {NULL}};
const Segment SGI336186[] = {{&S21118[12], 14}, {NULL}};
const Segment SGI336187[] = {{&S21119[6], 4}, {NULL}};
const Segment SGI336188[] = {{&S21119[12], 14}, {NULL}};
const Segment SGI336189[] = {{&S21125[6], 4}, {NULL}};
const Segment SGI336190[] = {{&S21125[12], 14}, {NULL}};
const Segment SGI336191[] = {{&S21126[6], 4}, {NULL}};
const Segment SGI336192[] = {{&S21126[43], 4}, {NULL}};
const Segment SGI336193[] = {{&S21126[12], 14}, {NULL}};
const Segment SGI336194[] = {{&S21127[6], 4}, {NULL}};
const Segment SGI336195[] = {{&S21127[43], 4}, {NULL}};
const Segment SGI336196[] = {{&S21127[12], 14}, {NULL}};
const Segment SGI336197[] = {{&S21128[6], 4}, {NULL}};
const Segment SGI336198[] = {{&S21128[57], 5}, {NULL}};
const Segment SGI336199[] = {{&S21128[12], 14}, {NULL}};
const Segment SGI336200[] = {{&S21129[6], 4}, {NULL}};
const Segment SGI336201[] = {{&S21129[43], 4}, {NULL}};
const Segment SGI336202[] = {{&S21129[12], 14}, {NULL}};
const Segment SGI336203[] = {{&S21130[6], 4}, {NULL}};
const Segment SGI336204[] = {{&S21130[43], 4}, {NULL}};
const Segment SGI336205[] = {{&S21130[12], 14}, {NULL}};
const Segment SGI336206[] = {{&S21131[6], 4}, {NULL}};
const Segment SGI336207[] = {{&S21131[57], 5}, {NULL}};
const Segment SGI336208[] = {{&S21131[12], 14}, {NULL}};
const Segment SGI336209[] = {{&S21133[6], 4}, {NULL}};
const Segment SGI336210[] = {{&S21133[57], 5}, {NULL}};
const Segment SGI336211[] = {{&S21133[12], 14}, {NULL}};
const Segment SGI336212[] = {{&S21134[6], 4}, {NULL}};
const Segment SGI336213[] = {{&S21134[57], 5}, {NULL}};
const Segment SGI336214[] = {{&S21134[12], 14}, {NULL}};
const Segment SGI336215[] = {{&S21136[6], 4}, {NULL}};
const Segment SGI336216[] = {{&S21136[12], 14}, {NULL}};
const Segment SGI336217[] = {{&S21138[6], 4}, {NULL}};
const Segment SGI336218[] = {{&S21138[12], 14}, {NULL}};
const Segment SGI336219[] = {{&S21146[6], 4}, {NULL}};
const Segment SGI336220[] = {{&S21146[12], 14}, {NULL}};
const Segment SGI336221[] = {{&S21148[6], 3}, {NULL}};
const Segment SGI336222[] = {{&S21148[11], 14}, {NULL}};
const Segment SGI336223[] = {{&S21150[6], 4}, {NULL}};
const Segment SGI336224[] = {{&S21150[12], 14}, {NULL}};
const Segment SGI336225[] = {{&S23192[57], 5}, {NULL}};
const Segment SGI336226[] = {{&S23192[13], 10}, {NULL}};
const Segment SGI336227[] = {{&S21288[6], 4}, {NULL}};
const Segment SGI336228[] = {{&S21288[59], 5}, {NULL}};
const Segment SGI336229[] = {{&S21288[12], 14}, {NULL}};
const Segment SGI336230[] = {{&S21289[6], 4}, {NULL}};
const Segment SGI336231[] = {{&S21289[81], 5}, {NULL}};
const Segment SGI336232[] = {{&S21289[12], 14}, {&S21289[40], 11}, {NULL}};
const Segment SGI336233[] = {{&S21289[12], 14}, {NULL}};
const Segment SGI336234[] = {{&S21290[6], 4}, {NULL}};
const Segment SGI336235[] = {{&S21290[82], 5}, {NULL}};
const Segment SGI336236[] = {{&S21290[12], 14}, {&S21290[41], 11}, {NULL}};
const Segment SGI336237[] = {{&S21290[12], 14}, {NULL}};
const Segment SGI336238[] = {{&S21291[6], 4}, {NULL}};
const Segment SGI336239[] = {{&S21291[82], 5}, {NULL}};
const Segment SGI336240[] = {{&S21291[12], 14}, {&S21291[41], 11}, {NULL}};
const Segment SGI336241[] = {{&S21291[12], 14}, {NULL}};
const Segment SGI336242[] = {{&S21292[6], 4}, {NULL}};
const Segment SGI336243[] = {{&S21292[82], 5}, {NULL}};
const Segment SGI336244[] = {{&S21292[12], 14}, {&S21292[40], 12}, {NULL}};
const Segment SGI336245[] = {{&S21292[12], 14}, {NULL}};
const Segment SGI336246[] = {{&S21293[6], 4}, {NULL}};
const Segment SGI336247[] = {{&S21293[81], 5}, {NULL}};
const Segment SGI336248[] = {{&S21293[12], 14}, {&S21293[40], 11}, {NULL}};
const Segment SGI336249[] = {{&S21293[12], 14}, {NULL}};
const Segment SGI336250[] = {{&S21294[6], 4}, {NULL}};
const Segment SGI336251[] = {{&S21294[82], 5}, {NULL}};
const Segment SGI336252[] = {{&S21294[12], 14}, {&S21294[41], 11}, {NULL}};
const Segment SGI336253[] = {{&S21294[12], 14}, {NULL}};
const Segment SGI336254[] = {{&S21295[6], 4}, {NULL}};
const Segment SGI336255[] = {{&S21295[82], 5}, {NULL}};
const Segment SGI336256[] = {{&S21295[12], 14}, {&S21295[40], 12}, {NULL}};
const Segment SGI336257[] = {{&S21295[12], 14}, {NULL}};
const Segment SGI336258[] = {{&S21296[6], 4}, {NULL}};
const Segment SGI336259[] = {{&S21296[82], 5}, {NULL}};
const Segment SGI336260[] = {{&S21296[12], 14}, {&S21296[41], 11}, {NULL}};
const Segment SGI336261[] = {{&S21296[12], 14}, {NULL}};
const Segment SGI336262[] = {{&S21297[6], 4}, {NULL}};
const Segment SGI336263[] = {{&S21297[82], 5}, {NULL}};
const Segment SGI336264[] = {{&S21297[12], 14}, {&S21297[41], 11}, {NULL}};
const Segment SGI336265[] = {{&S21297[12], 14}, {NULL}};
const Segment SGI336266[] = {{&S21298[6], 4}, {NULL}};
const Segment SGI336267[] = {{&S21298[81], 5}, {NULL}};
const Segment SGI336268[] = {{&S21298[12], 14}, {&S21298[40], 11}, {NULL}};
const Segment SGI336269[] = {{&S21298[12], 14}, {NULL}};
const Segment SGI336270[] = {{&S21300[6], 4}, {NULL}};
const Segment SGI336271[] = {{&S21300[81], 5}, {NULL}};
const Segment SGI336272[] = {{&S21300[12], 14}, {&S21300[40], 11}, {NULL}};
const Segment SGI336273[] = {{&S21300[12], 14}, {NULL}};
const Segment SGI336274[] = {{&S21301[6], 4}, {NULL}};
const Segment SGI336275[] = {{&S21301[82], 5}, {NULL}};
const Segment SGI336276[] = {{&S21301[12], 14}, {&S21301[40], 12}, {NULL}};
const Segment SGI336277[] = {{&S21301[12], 14}, {NULL}};
const Segment SGI336278[] = {{&S21302[6], 4}, {NULL}};
const Segment SGI336279[] = {{&S21302[82], 5}, {NULL}};
const Segment SGI336280[] = {{&S21302[12], 14}, {&S21302[41], 11}, {NULL}};
const Segment SGI336281[] = {{&S21302[12], 14}, {NULL}};
const Segment SGI336282[] = {{&S21303[6], 4}, {NULL}};
const Segment SGI336283[] = {{&S21303[82], 5}, {NULL}};
const Segment SGI336284[] = {{&S21303[12], 14}, {&S21303[41], 11}, {NULL}};
const Segment SGI336285[] = {{&S21303[12], 14}, {NULL}};
const Segment SGI336286[] = {{&S21304[6], 4}, {NULL}};
const Segment SGI336287[] = {{&S21304[81], 5}, {NULL}};
const Segment SGI336288[] = {{&S21304[12], 14}, {&S21304[40], 11}, {NULL}};
const Segment SGI336289[] = {{&S21304[12], 14}, {NULL}};
const Segment SGI336290[] = {{&S21305[6], 4}, {NULL}};
const Segment SGI336291[] = {{&S21305[82], 5}, {NULL}};
const Segment SGI336292[] = {{&S21305[12], 14}, {&S21305[41], 11}, {NULL}};
const Segment SGI336293[] = {{&S21305[12], 14}, {NULL}};
const Segment SGI336294[] = {{&S21306[6], 4}, {NULL}};
const Segment SGI336295[] = {{&S21306[82], 5}, {NULL}};
const Segment SGI336296[] = {{&S21306[12], 14}, {&S21306[41], 11}, {NULL}};
const Segment SGI336297[] = {{&S21306[12], 14}, {NULL}};
const Segment SGI336298[] = {{&S21307[6], 4}, {NULL}};
const Segment SGI336299[] = {{&S21307[82], 5}, {NULL}};
const Segment SGI336300[] = {{&S21307[12], 14}, {&S21307[41], 11}, {NULL}};
const Segment SGI336301[] = {{&S21307[12], 14}, {NULL}};
const Segment SGI336302[] = {{&S21308[6], 4}, {NULL}};
const Segment SGI336303[] = {{&S21308[82], 5}, {NULL}};
const Segment SGI336304[] = {{&S21308[12], 14}, {&S21308[40], 12}, {NULL}};
const Segment SGI336305[] = {{&S21308[12], 14}, {NULL}};
const Segment SGI336306[] = {{&S21309[6], 4}, {NULL}};
const Segment SGI336307[] = {{&S21309[82], 5}, {NULL}};
const Segment SGI336308[] = {{&S21309[12], 14}, {&S21309[41], 11}, {NULL}};
const Segment SGI336309[] = {{&S21309[12], 14}, {NULL}};
const Segment SGI336310[] = {{&S21310[6], 4}, {NULL}};
const Segment SGI336311[] = {{&S21310[82], 5}, {NULL}};
const Segment SGI336312[] = {{&S21310[12], 14}, {&S21310[41], 11}, {NULL}};
const Segment SGI336313[] = {{&S21310[12], 14}, {NULL}};
const Segment SGI336314[] = {{&S21311[6], 4}, {NULL}};
const Segment SGI336315[] = {{&S21311[81], 5}, {NULL}};
const Segment SGI336316[] = {{&S21311[12], 14}, {&S21311[40], 11}, {NULL}};
const Segment SGI336317[] = {{&S21311[12], 14}, {NULL}};
const Segment SGI336318[] = {{&S21312[6], 4}, {NULL}};
const Segment SGI336319[] = {{&S21312[81], 5}, {NULL}};
const Segment SGI336320[] = {{&S21312[12], 14}, {&S21312[40], 11}, {NULL}};
const Segment SGI336321[] = {{&S21312[12], 14}, {NULL}};
const Segment SGI336322[] = {{&S21313[6], 4}, {NULL}};
const Segment SGI336323[] = {{&S21313[82], 5}, {NULL}};
const Segment SGI336324[] = {{&S21313[12], 14}, {&S21313[40], 12}, {NULL}};
const Segment SGI336325[] = {{&S21313[12], 14}, {NULL}};
const Segment SGI336326[] = {{&S21314[6], 4}, {NULL}};
const Segment SGI336327[] = {{&S21314[82], 5}, {NULL}};
const Segment SGI336328[] = {{&S21314[12], 14}, {&S21314[40], 12}, {NULL}};
const Segment SGI336329[] = {{&S21314[12], 14}, {NULL}};
const Segment SGI336330[] = {{&S21315[6], 4}, {NULL}};
const Segment SGI336331[] = {{&S21315[82], 5}, {NULL}};
const Segment SGI336332[] = {{&S21315[12], 14}, {&S21315[40], 12}, {NULL}};
const Segment SGI336333[] = {{&S21315[12], 14}, {NULL}};
const Segment SGI336334[] = {{&S21316[6], 4}, {NULL}};
const Segment SGI336335[] = {{&S21316[81], 5}, {NULL}};
const Segment SGI336336[] = {{&S21316[12], 14}, {&S21316[40], 11}, {NULL}};
const Segment SGI336337[] = {{&S21316[12], 14}, {NULL}};
const Segment SGI336338[] = {{&S21317[6], 4}, {NULL}};
const Segment SGI336339[] = {{&S21317[82], 5}, {NULL}};
const Segment SGI336340[] = {{&S21317[12], 14}, {&S21317[41], 11}, {NULL}};
const Segment SGI336341[] = {{&S21317[12], 14}, {NULL}};
const Segment SGI336342[] = {{&S21318[6], 4}, {NULL}};
const Segment SGI336343[] = {{&S21318[59], 5}, {NULL}};
const Segment SGI336344[] = {{&S21318[12], 14}, {&S21318[70], 31}, {NULL}};
const Segment SGI336345[] = {{&S21318[12], 14}, {NULL}};
const Segment SGI336346[] = {{&S21319[6], 4}, {NULL}};
const Segment SGI336347[] = {{&S21319[82], 5}, {NULL}};
const Segment SGI336348[] = {{&S21319[12], 14}, {&S21319[41], 11}, {NULL}};
const Segment SGI336349[] = {{&S21319[12], 14}, {NULL}};
const Segment SGI336350[] = {{&S21320[6], 4}, {NULL}};
const Segment SGI336351[] = {{&S21320[81], 5}, {NULL}};
const Segment SGI336352[] = {{&S21320[12], 14}, {&S21320[40], 11}, {NULL}};
const Segment SGI336353[] = {{&S21320[12], 14}, {NULL}};
const Segment SGI336354[] = {{&S21321[6], 4}, {NULL}};
const Segment SGI336355[] = {{&S21321[81], 5}, {NULL}};
const Segment SGI336356[] = {{&S21321[12], 14}, {&S21321[40], 11}, {NULL}};
const Segment SGI336357[] = {{&S21321[12], 14}, {NULL}};
const Segment SGI336358[] = {{&S21322[6], 4}, {NULL}};
const Segment SGI336359[] = {{&S21322[82], 5}, {NULL}};
const Segment SGI336360[] = {{&S21322[12], 14}, {&S21322[41], 11}, {NULL}};
const Segment SGI336361[] = {{&S21322[12], 14}, {NULL}};
const Segment SGI336362[] = {{&S21323[6], 4}, {NULL}};
const Segment SGI336363[] = {{&S21323[81], 5}, {NULL}};
const Segment SGI336364[] = {{&S21323[12], 14}, {&S21323[40], 11}, {NULL}};
const Segment SGI336365[] = {{&S21323[12], 14}, {NULL}};
const Segment SGI336366[] = {{&S21324[6], 4}, {NULL}};
const Segment SGI336367[] = {{&S21324[82], 5}, {NULL}};
const Segment SGI336368[] = {{&S21324[12], 14}, {&S21324[40], 12}, {NULL}};
const Segment SGI336369[] = {{&S21324[12], 14}, {NULL}};
const Segment SGI336370[] = {{&S21325[6], 4}, {NULL}};
const Segment SGI336371[] = {{&S21325[81], 5}, {NULL}};
const Segment SGI336372[] = {{&S21325[12], 14}, {&S21325[40], 11}, {NULL}};
const Segment SGI336373[] = {{&S21325[12], 14}, {NULL}};
const Segment SGI336374[] = {{&S21326[6], 4}, {NULL}};
const Segment SGI336375[] = {{&S21326[82], 5}, {NULL}};
const Segment SGI336376[] = {{&S21326[12], 14}, {&S21326[40], 12}, {NULL}};
const Segment SGI336377[] = {{&S21326[12], 14}, {NULL}};
const Segment SGI336378[] = {{&S21327[6], 4}, {NULL}};
const Segment SGI336379[] = {{&S21327[82], 5}, {NULL}};
const Segment SGI336380[] = {{&S21327[12], 14}, {&S21327[41], 11}, {&S21327[93], 31}, {NULL}};
const Segment SGI336381[] = {{&S21327[12], 14}, {NULL}};
const Segment SGI336382[] = {{&S21328[6], 4}, {NULL}};
const Segment SGI336383[] = {{&S21328[82], 5}, {NULL}};
const Segment SGI336384[] = {{&S21328[12], 14}, {&S21328[40], 12}, {NULL}};
const Segment SGI336385[] = {{&S21328[12], 14}, {NULL}};
const Segment SGI336386[] = {{&S21329[6], 4}, {NULL}};
const Segment SGI336387[] = {{&S21329[82], 5}, {NULL}};
const Segment SGI336388[] = {{&S21329[12], 14}, {&S21329[41], 11}, {NULL}};
const Segment SGI336389[] = {{&S21329[12], 14}, {NULL}};
const Segment SGI336390[] = {{&S21330[6], 4}, {NULL}};
const Segment SGI336391[] = {{&S21330[59], 5}, {NULL}};
const Segment SGI336392[] = {{&S21330[12], 14}, {NULL}};
const Segment SGI336393[] = {{&S21331[6], 4}, {NULL}};
const Segment SGI336394[] = {{&S21331[82], 5}, {NULL}};
const Segment SGI336395[] = {{&S21331[12], 14}, {&S21331[41], 11}, {NULL}};
const Segment SGI336396[] = {{&S21331[12], 14}, {NULL}};
const Segment SGI336397[] = {{&S21332[6], 4}, {NULL}};
const Segment SGI336398[] = {{&S21332[81], 5}, {NULL}};
const Segment SGI336399[] = {{&S21332[12], 14}, {&S21332[40], 11}, {NULL}};
const Segment SGI336400[] = {{&S21332[12], 14}, {NULL}};
const Segment SGI336401[] = {{&S21333[6], 4}, {NULL}};
const Segment SGI336402[] = {{&S21333[82], 5}, {NULL}};
const Segment SGI336403[] = {{&S21333[12], 14}, {&S21333[40], 12}, {NULL}};
const Segment SGI336404[] = {{&S21333[12], 14}, {NULL}};
const Segment SGI336405[] = {{&S21334[6], 4}, {NULL}};
const Segment SGI336406[] = {{&S21334[82], 5}, {NULL}};
const Segment SGI336407[] = {{&S21334[12], 14}, {&S21334[40], 12}, {NULL}};
const Segment SGI336408[] = {{&S21334[12], 14}, {NULL}};
const Segment SGI336409[] = {{&S21335[6], 4}, {NULL}};
const Segment SGI336410[] = {{&S21335[82], 5}, {NULL}};
const Segment SGI336411[] = {{&S21335[12], 14}, {&S21335[40], 12}, {NULL}};
const Segment SGI336412[] = {{&S21335[12], 14}, {NULL}};
const Segment SGI336413[] = {{&S21336[6], 4}, {NULL}};
const Segment SGI336414[] = {{&S21336[82], 5}, {NULL}};
const Segment SGI336415[] = {{&S21336[12], 14}, {&S21336[41], 11}, {NULL}};
const Segment SGI336416[] = {{&S21336[12], 14}, {NULL}};
const Segment SGI336417[] = {{&S21338[6], 4}, {NULL}};
const Segment SGI336418[] = {{&S21338[81], 5}, {NULL}};
const Segment SGI336419[] = {{&S21338[12], 14}, {&S21338[40], 11}, {NULL}};
const Segment SGI336420[] = {{&S21338[12], 14}, {NULL}};
const Segment SGI336421[] = {{&S21339[6], 4}, {NULL}};
const Segment SGI336422[] = {{&S21339[82], 5}, {NULL}};
const Segment SGI336423[] = {{&S21339[12], 14}, {&S21339[40], 12}, {NULL}};
const Segment SGI336424[] = {{&S21339[12], 14}, {NULL}};
const Segment SGI336425[] = {{&S21340[6], 4}, {NULL}};
const Segment SGI336426[] = {{&S21340[82], 5}, {NULL}};
const Segment SGI336427[] = {{&S21340[12], 14}, {&S21340[41], 11}, {NULL}};
const Segment SGI336428[] = {{&S21340[12], 14}, {NULL}};
const Segment SGI336429[] = {{&S21341[6], 4}, {NULL}};
const Segment SGI336430[] = {{&S21341[82], 5}, {NULL}};
const Segment SGI336431[] = {{&S21341[12], 14}, {&S21341[40], 12}, {NULL}};
const Segment SGI336432[] = {{&S21341[12], 14}, {NULL}};
const Segment SGI336433[] = {{&S21342[6], 4}, {NULL}};
const Segment SGI336434[] = {{&S21342[81], 5}, {NULL}};
const Segment SGI336435[] = {{&S21342[12], 14}, {&S21342[40], 11}, {NULL}};
const Segment SGI336436[] = {{&S21342[12], 14}, {NULL}};
const Segment SGI336437[] = {{&S21343[6], 4}, {NULL}};
const Segment SGI336438[] = {{&S21343[81], 5}, {NULL}};
const Segment SGI336439[] = {{&S21343[12], 14}, {&S21343[40], 11}, {NULL}};
const Segment SGI336440[] = {{&S21343[12], 14}, {NULL}};
const Segment SGI336441[] = {{&S21344[6], 4}, {NULL}};
const Segment SGI336442[] = {{&S21344[82], 5}, {NULL}};
const Segment SGI336443[] = {{&S21344[12], 14}, {&S21344[40], 12}, {NULL}};
const Segment SGI336444[] = {{&S21344[12], 14}, {NULL}};
const Segment SGI336445[] = {{&S21345[6], 4}, {NULL}};
const Segment SGI336446[] = {{&S21345[82], 5}, {NULL}};
const Segment SGI336447[] = {{&S21345[12], 14}, {&S21345[40], 12}, {NULL}};
const Segment SGI336448[] = {{&S21345[12], 14}, {NULL}};
const Segment SGI336449[] = {{&S21346[6], 4}, {NULL}};
const Segment SGI336450[] = {{&S21346[81], 5}, {NULL}};
const Segment SGI336451[] = {{&S21346[12], 14}, {&S21346[40], 11}, {NULL}};
const Segment SGI336452[] = {{&S21346[12], 14}, {NULL}};
const Segment SGI336453[] = {{&S21347[6], 4}, {NULL}};
const Segment SGI336454[] = {{&S21347[82], 5}, {NULL}};
const Segment SGI336455[] = {{&S21347[12], 14}, {&S21347[41], 11}, {NULL}};
const Segment SGI336456[] = {{&S21347[12], 14}, {NULL}};
const Segment SGI336457[] = {{&S21348[6], 4}, {NULL}};
const Segment SGI336458[] = {{&S21348[81], 5}, {NULL}};
const Segment SGI336459[] = {{&S21348[12], 14}, {&S21348[40], 11}, {NULL}};
const Segment SGI336460[] = {{&S21348[12], 14}, {NULL}};
const Segment SGI336461[] = {{&S21349[6], 4}, {NULL}};
const Segment SGI336462[] = {{&S21349[81], 5}, {NULL}};
const Segment SGI336463[] = {{&S21349[12], 14}, {&S21349[40], 11}, {NULL}};
const Segment SGI336464[] = {{&S21349[12], 14}, {NULL}};
const Segment SGI336465[] = {{&S21350[6], 4}, {NULL}};
const Segment SGI336466[] = {{&S21350[81], 5}, {NULL}};
const Segment SGI336467[] = {{&S21350[12], 14}, {&S21350[40], 11}, {NULL}};
const Segment SGI336468[] = {{&S21350[12], 14}, {NULL}};
const Segment SGI336469[] = {{&S21351[6], 4}, {NULL}};
const Segment SGI336470[] = {{&S21351[81], 5}, {NULL}};
const Segment SGI336471[] = {{&S21351[12], 14}, {&S21351[40], 11}, {NULL}};
const Segment SGI336472[] = {{&S21351[12], 14}, {NULL}};
const Segment SGI336473[] = {{&S21352[6], 4}, {NULL}};
const Segment SGI336474[] = {{&S21352[81], 5}, {NULL}};
const Segment SGI336475[] = {{&S21352[12], 14}, {&S21352[40], 11}, {NULL}};
const Segment SGI336476[] = {{&S21352[12], 14}, {NULL}};
const Segment SGI336477[] = {{&S21353[6], 4}, {NULL}};
const Segment SGI336478[] = {{&S21353[81], 5}, {NULL}};
const Segment SGI336479[] = {{&S21353[12], 14}, {&S21353[40], 11}, {NULL}};
const Segment SGI336480[] = {{&S21353[12], 14}, {NULL}};
const Segment SGI336481[] = {{&S21354[6], 4}, {NULL}};
const Segment SGI336482[] = {{&S21354[81], 5}, {NULL}};
const Segment SGI336483[] = {{&S21354[12], 14}, {&S21354[40], 11}, {NULL}};
const Segment SGI336484[] = {{&S21354[12], 14}, {NULL}};
const Segment SGI336485[] = {{&S21355[6], 4}, {NULL}};
const Segment SGI336486[] = {{&S21355[82], 5}, {NULL}};
const Segment SGI336487[] = {{&S21355[12], 14}, {&S21355[41], 11}, {NULL}};
const Segment SGI336488[] = {{&S21355[12], 14}, {NULL}};
const Segment SGI336489[] = {{&S21356[6], 4}, {NULL}};
const Segment SGI336490[] = {{&S21356[81], 5}, {NULL}};
const Segment SGI336491[] = {{&S21356[12], 14}, {&S21356[40], 11}, {NULL}};
const Segment SGI336492[] = {{&S21356[12], 14}, {NULL}};
const Segment SGI336493[] = {{&S21357[6], 4}, {NULL}};
const Segment SGI336494[] = {{&S21357[82], 5}, {NULL}};
const Segment SGI336495[] = {{&S21357[12], 14}, {&S21357[40], 12}, {NULL}};
const Segment SGI336496[] = {{&S21357[12], 14}, {NULL}};
const Segment SGI336497[] = {{&S21358[6], 4}, {NULL}};
const Segment SGI336498[] = {{&S21358[81], 5}, {NULL}};
const Segment SGI336499[] = {{&S21358[12], 14}, {&S21358[40], 11}, {NULL}};
const Segment SGI336500[] = {{&S21358[12], 14}, {NULL}};
const Segment SGI336501[] = {{&S21359[6], 4}, {NULL}};
const Segment SGI336502[] = {{&S21359[81], 5}, {NULL}};
const Segment SGI336503[] = {{&S21359[12], 14}, {&S21359[40], 11}, {NULL}};
const Segment SGI336504[] = {{&S21359[12], 14}, {NULL}};
const Segment SGI336505[] = {{&S21360[6], 4}, {NULL}};
const Segment SGI336506[] = {{&S21360[81], 5}, {NULL}};
const Segment SGI336507[] = {{&S21360[12], 14}, {&S21360[40], 11}, {NULL}};
const Segment SGI336508[] = {{&S21360[12], 14}, {NULL}};
const Segment SGI336509[] = {{&S21361[6], 4}, {NULL}};
const Segment SGI336510[] = {{&S21361[81], 5}, {NULL}};
const Segment SGI336511[] = {{&S21361[12], 14}, {&S21361[40], 11}, {NULL}};
const Segment SGI336512[] = {{&S21361[12], 14}, {NULL}};
const Segment SGI336513[] = {{&S21362[6], 4}, {NULL}};
const Segment SGI336514[] = {{&S21362[81], 5}, {NULL}};
const Segment SGI336515[] = {{&S21362[12], 14}, {&S21362[40], 11}, {NULL}};
const Segment SGI336516[] = {{&S21362[12], 14}, {NULL}};
const Segment SGI336517[] = {{&S21363[6], 4}, {NULL}};
const Segment SGI336518[] = {{&S21363[82], 5}, {NULL}};
const Segment SGI336519[] = {{&S21363[12], 14}, {&S21363[40], 12}, {NULL}};
const Segment SGI336520[] = {{&S21363[12], 14}, {NULL}};
const Segment SGI336521[] = {{&S21364[6], 4}, {NULL}};
const Segment SGI336522[] = {{&S21364[59], 5}, {NULL}};
const Segment SGI336523[] = {{&S21364[12], 14}, {&S21364[70], 4}, {NULL}};
const Segment SGI336524[] = {{&S21364[12], 14}, {NULL}};
const Segment SGI336525[] = {{&S21365[6], 4}, {NULL}};
const Segment SGI336526[] = {{&S21365[81], 5}, {NULL}};
const Segment SGI336527[] = {{&S21365[12], 14}, {&S21365[40], 11}, {NULL}};
const Segment SGI336528[] = {{&S21365[12], 14}, {NULL}};
const Segment SGI336529[] = {{&S21366[6], 4}, {NULL}};
const Segment SGI336530[] = {{&S21366[81], 5}, {NULL}};
const Segment SGI336531[] = {{&S21366[12], 14}, {&S21366[40], 11}, {NULL}};
const Segment SGI336532[] = {{&S21366[12], 14}, {NULL}};
const Segment SGI336533[] = {{&S21367[6], 4}, {NULL}};
const Segment SGI336534[] = {{&S21367[81], 5}, {NULL}};
const Segment SGI336535[] = {{&S21367[12], 14}, {&S21367[40], 11}, {NULL}};
const Segment SGI336536[] = {{&S21367[12], 14}, {NULL}};
const Segment SGI336537[] = {{&S21368[6], 4}, {NULL}};
const Segment SGI336538[] = {{&S21368[81], 5}, {NULL}};
const Segment SGI336539[] = {{&S21368[12], 14}, {&S21368[40], 11}, {NULL}};
const Segment SGI336540[] = {{&S21368[12], 14}, {NULL}};
const Segment SGI336541[] = {{&S21369[6], 4}, {NULL}};
const Segment SGI336542[] = {{&S21369[81], 5}, {NULL}};
const Segment SGI336543[] = {{&S21369[12], 14}, {&S21369[40], 11}, {NULL}};
const Segment SGI336544[] = {{&S21369[12], 14}, {NULL}};
const Segment SGI336545[] = {{&S21370[6], 4}, {NULL}};
const Segment SGI336546[] = {{&S21370[81], 5}, {NULL}};
const Segment SGI336547[] = {{&S21370[12], 14}, {&S21370[40], 11}, {NULL}};
const Segment SGI336548[] = {{&S21370[12], 14}, {NULL}};
const Segment SGI336549[] = {{&S21371[6], 4}, {NULL}};
const Segment SGI336550[] = {{&S21371[81], 5}, {NULL}};
const Segment SGI336551[] = {{&S47144[5], 3}, {&S47144[5], 3}, {NULL}};
const Segment SGI336552[] = {{&S47144[9], 3}, {NULL}};
const Segment SGI336553[] = {{&S47144[127], 3}, {NULL}};
const Segment SGI336554[] = {{&S38471[28], 3}, {NULL}};
const Segment SGI336555[] = {{&S38471[102], 5}, {NULL}};
const Segment SGI336556[] = {{&S38472[5], 3}, {&S38472[5], 3}, {NULL}};
const Segment SGI336557[] = {{&S38472[163], 5}, {NULL}};
const Segment SGI336558[] = {{&S21371[12], 14}, {&S21371[40], 11}, {NULL}};
const Segment SGI336559[] = {{&S21371[12], 14}, {NULL}};
const Segment SGI336560[] = {{&S21372[6], 4}, {NULL}};
const Segment SGI336561[] = {{&S21372[81], 5}, {NULL}};
const Segment SGI336562[] = {{&S21372[12], 14}, {&S21372[40], 11}, {NULL}};
const Segment SGI336563[] = {{&S21372[12], 14}, {NULL}};
const Segment SGI336564[] = {{&S21373[6], 4}, {NULL}};
const Segment SGI336565[] = {{&S21373[81], 5}, {NULL}};
const Segment SGI336566[] = {{&S21373[12], 14}, {&S21373[40], 11}, {NULL}};
const Segment SGI336567[] = {{&S21373[12], 14}, {NULL}};
const Segment SGI336568[] = {{&S21374[6], 4}, {NULL}};
const Segment SGI336569[] = {{&S21374[81], 5}, {NULL}};
const Segment SGI336570[] = {{&S21374[12], 14}, {&S21374[40], 11}, {NULL}};
const Segment SGI336571[] = {{&S21374[12], 14}, {NULL}};
const Segment SGI336572[] = {{&S21375[6], 4}, {NULL}};
const Segment SGI336573[] = {{&S21375[81], 5}, {NULL}};
const Segment SGI336574[] = {{&S21375[12], 14}, {&S21375[40], 11}, {NULL}};
const Segment SGI336575[] = {{&S21375[12], 14}, {NULL}};
const Segment SGI336576[] = {{&S21376[6], 4}, {NULL}};
const Segment SGI336577[] = {{&S21376[81], 5}, {NULL}};
const Segment SGI336578[] = {{&S21376[12], 14}, {&S21376[40], 11}, {NULL}};
const Segment SGI336579[] = {{&S21376[12], 14}, {NULL}};
const Segment SGI336580[] = {{&S21377[6], 4}, {NULL}};
const Segment SGI336581[] = {{&S21377[81], 5}, {NULL}};
const Segment SGI336582[] = {{&S21377[12], 14}, {&S21377[40], 11}, {NULL}};
const Segment SGI336583[] = {{&S21377[12], 14}, {NULL}};
const Segment SGI336584[] = {{&S21378[6], 4}, {NULL}};
const Segment SGI336585[] = {{&S21378[81], 5}, {NULL}};
const Segment SGI336586[] = {{&S21378[12], 14}, {&S21378[40], 11}, {NULL}};
const Segment SGI336587[] = {{&S21378[12], 14}, {NULL}};
const Segment SGI336588[] = {{&S21379[6], 4}, {NULL}};
const Segment SGI336589[] = {{&S21379[81], 5}, {NULL}};
const Segment SGI336590[] = {{&S21379[12], 14}, {&S21379[40], 11}, {NULL}};
const Segment SGI336591[] = {{&S21379[12], 14}, {NULL}};
const Segment SGI336592[] = {{&S21380[6], 4}, {NULL}};
const Segment SGI336593[] = {{&S21380[81], 5}, {NULL}};
const Segment SGI336594[] = {{&S21380[12], 14}, {&S21380[40], 11}, {NULL}};
const Segment SGI336595[] = {{&S21380[12], 14}, {NULL}};
const Segment SGI336596[] = {{&S47462[5], 5}, {&S47462[68], 5}, {NULL}};
const Segment SGI336597[] = {{&S47463[5], 5}, {NULL}};
const Segment SGI336598[] = {{&S33872[46], 5}, {NULL}};
const Segment SGI336599[] = {{&S33872[8], 3}, {NULL}};
const Segment SGI336600[] = {{&S33873[5], 3}, {NULL}};
const Segment SGI336601[] = {{&S33873[43], 3}, {NULL}};
const Segment SGI336602[] = {{&S33874[46], 5}, {&S33874[146], 7}, {NULL}};
const Segment SGI336603[] = {{&S33874[8], 3}, {NULL}};
const Segment SGI336604[] = {{&S23713[57], 5}, {NULL}};
const Segment SGI336605[] = {{&S23713[13], 10}, {NULL}};
const Segment SGI336606[] = {{&S33875[46], 5}, {&S33875[146], 7}, {NULL}};
const Segment SGI336607[] = {{&S33875[8], 3}, {NULL}};
const Segment SGI336608[] = {{&S33876[5], 4}, {NULL}};
const Segment SGI336609[] = {{&S33876[44], 3}, {NULL}};
const Segment SGI336610[] = {{&S33878[46], 5}, {&S33878[144], 7}, {NULL}};
const Segment SGI336611[] = {{&S33878[8], 3}, {NULL}};
const Segment SGI336612[] = {{&S33879[46], 5}, {NULL}};
const Segment SGI336613[] = {{&S33879[8], 3}, {NULL}};
const Segment SGI336614[] = {{&S33880[46], 5}, {NULL}};
const Segment SGI336615[] = {{&S33880[8], 3}, {NULL}};
const Segment SGI336616[] = {{&S33881[46], 5}, {NULL}};
const Segment SGI336617[] = {{&S33881[8], 3}, {NULL}};
const Segment SGI336618[] = {{&S20183[0], 21}, {NULL}};
const Segment SGI336619[] = {{&S20183[34], 3}, {NULL}};
const Segment SGI336620[] = {{&S20183[73], 13}, {NULL}};
const Segment SGI336621[] = {{&S20184[0], 21}, {NULL}};
const Segment SGI336622[] = {{&S20184[34], 3}, {NULL}};
const Segment SGI336623[] = {{&S20184[75], 13}, {NULL}};
const Segment SGI336624[] = {{&S21797[6], 4}, {NULL}};
const Segment SGI336625[] = {{&S21797[74], 5}, {NULL}};
const Segment SGI336626[] = {{&S21797[12], 14}, {&S21797[37], 42}, {NULL}};
const Segment SGI336627[] = {{&S21797[12], 14}, {NULL}};
const Segment SGI336628[] = {{&S21798[6], 4}, {NULL}};
const Segment SGI336629[] = {{&S21798[74], 5}, {NULL}};
const Segment SGI336630[] = {{&S21798[12], 14}, {&S21798[37], 42}, {NULL}};
const Segment SGI336631[] = {{&S21798[12], 14}, {NULL}};
const Segment SGI336632[] = {{&S21799[6], 4}, {NULL}};
const Segment SGI336633[] = {{&S21799[74], 5}, {NULL}};
const Segment SGI336634[] = {{&S21799[12], 14}, {&S21799[37], 42}, {NULL}};
const Segment SGI336635[] = {{&S21799[12], 14}, {NULL}};
const Segment SGI336636[] = {{&S47721[5], 5}, {NULL}};
const Segment SGI336637[] = {{&S21800[6], 4}, {NULL}};
const Segment SGI336638[] = {{&S21800[74], 5}, {NULL}};
const Segment SGI336639[] = {{&S21800[12], 14}, {&S21800[37], 42}, {NULL}};
const Segment SGI336640[] = {{&S21800[12], 14}, {NULL}};
const Segment SGI336641[] = {{&S47722[5], 7}, {NULL}};
const Segment SGI336642[] = {{&S21801[6], 4}, {NULL}};
const Segment SGI336643[] = {{&S21801[74], 5}, {NULL}};
const Segment SGI336644[] = {{&S21801[12], 14}, {&S21801[37], 42}, {NULL}};
const Segment SGI336645[] = {{&S21801[12], 14}, {NULL}};
const Segment SGI336646[] = {{&S47723[5], 7}, {&S47723[5], 7}, {NULL}};
const Segment SGI336647[] = {{&S21802[6], 4}, {NULL}};
const Segment SGI336648[] = {{&S21802[75], 5}, {NULL}};
const Segment SGI336649[] = {{&S21802[12], 14}, {&S21802[37], 43}, {NULL}};
const Segment SGI336650[] = {{&S21802[12], 14}, {NULL}};
const Segment SGI336651[] = {{&S21803[6], 4}, {NULL}};
const Segment SGI336652[] = {{&S21803[75], 5}, {NULL}};
const Segment SGI336653[] = {{&S21803[12], 14}, {&S21803[38], 42}, {NULL}};
const Segment SGI336654[] = {{&S21803[12], 14}, {NULL}};
const Segment SGI336655[] = {{&S21804[6], 4}, {NULL}};
const Segment SGI336656[] = {{&S21804[75], 5}, {NULL}};
const Segment SGI336657[] = {{&S21804[12], 14}, {&S21804[37], 43}, {NULL}};
const Segment SGI336658[] = {{&S21804[12], 14}, {NULL}};
const Segment SGI336659[] = {{&S21805[6], 4}, {NULL}};
const Segment SGI336660[] = {{&S21805[74], 5}, {NULL}};
const Segment SGI336661[] = {{&S21805[12], 14}, {&S21805[37], 42}, {NULL}};
const Segment SGI336662[] = {{&S21805[12], 14}, {NULL}};
const Segment SGI336663[] = {{&S21806[6], 4}, {NULL}};
const Segment SGI336664[] = {{&S21806[74], 5}, {NULL}};
const Segment SGI336665[] = {{&S21806[12], 14}, {&S21806[37], 42}, {NULL}};
const Segment SGI336666[] = {{&S21806[12], 14}, {NULL}};
const Segment SGI336667[] = {{&S21807[6], 4}, {NULL}};
const Segment SGI336668[] = {{&S21807[74], 5}, {NULL}};
const Segment SGI336669[] = {{&S21807[12], 14}, {&S21807[37], 42}, {NULL}};
const Segment SGI336670[] = {{&S21807[12], 14}, {NULL}};
const Segment SGI336671[] = {{&S21808[6], 4}, {NULL}};
const Segment SGI336672[] = {{&S21808[75], 5}, {NULL}};
const Segment SGI336673[] = {{&S21808[12], 14}, {&S21808[37], 43}, {NULL}};
const Segment SGI336674[] = {{&S21808[12], 14}, {NULL}};
const Segment SGI336675[] = {{&S21809[6], 4}, {NULL}};
const Segment SGI336676[] = {{&S21809[74], 5}, {NULL}};
const Segment SGI336677[] = {{&S21809[12], 14}, {&S21809[37], 42}, {NULL}};
const Segment SGI336678[] = {{&S21809[12], 14}, {NULL}};
const Segment SGI336679[] = {{&S21810[6], 4}, {NULL}};
const Segment SGI336680[] = {{&S21810[74], 5}, {NULL}};
const Segment SGI336681[] = {{&S21810[12], 14}, {&S21810[37], 42}, {NULL}};
const Segment SGI336682[] = {{&S21810[12], 14}, {NULL}};
const Segment SGI336683[] = {{&S21811[6], 4}, {NULL}};
const Segment SGI336684[] = {{&S21811[75], 5}, {NULL}};
const Segment SGI336685[] = {{&S21811[12], 14}, {&S21811[38], 42}, {NULL}};
const Segment SGI336686[] = {{&S21811[12], 14}, {NULL}};
const Segment SGI336687[] = {{&S21812[6], 4}, {NULL}};
const Segment SGI336688[] = {{&S21812[74], 5}, {NULL}};
const Segment SGI336689[] = {{&S21812[12], 14}, {&S21812[37], 42}, {NULL}};
const Segment SGI336690[] = {{&S21812[12], 14}, {NULL}};
const Segment SGI336691[] = {{&S21813[6], 4}, {NULL}};
const Segment SGI336692[] = {{&S21813[59], 5}, {NULL}};
const Segment SGI336693[] = {{&S21813[12], 14}, {NULL}};
const Segment SGI336694[] = {{&S21814[6], 4}, {NULL}};
const Segment SGI336695[] = {{&S21814[52], 5}, {NULL}};
const Segment SGI336696[] = {{&S21814[12], 45}, {NULL}};
const Segment SGI336697[] = {{&S21814[12], 14}, {NULL}};
const Segment SGI336698[] = {{&S21815[6], 4}, {NULL}};
const Segment SGI336699[] = {{&S21815[75], 5}, {NULL}};
const Segment SGI336700[] = {{&S21815[12], 14}, {&S21815[38], 42}, {NULL}};
const Segment SGI336701[] = {{&S21815[12], 14}, {NULL}};
const Segment SGI336702[] = {{&S21816[6], 4}, {NULL}};
const Segment SGI336703[] = {{&S21816[74], 5}, {NULL}};
const Segment SGI336704[] = {{&S21816[12], 14}, {&S21816[37], 42}, {NULL}};
const Segment SGI336705[] = {{&S21816[12], 14}, {NULL}};
const Segment SGI336706[] = {{&S21817[6], 4}, {NULL}};
const Segment SGI336707[] = {{&S21817[74], 5}, {NULL}};
const Segment SGI336708[] = {{&S21817[12], 14}, {&S21817[37], 42}, {NULL}};
const Segment SGI336709[] = {{&S21817[12], 14}, {NULL}};
const Segment SGI336710[] = {{&S21818[6], 4}, {NULL}};
const Segment SGI336711[] = {{&S21818[74], 5}, {NULL}};
const Segment SGI336712[] = {{&S21818[12], 14}, {&S21818[37], 42}, {NULL}};
const Segment SGI336713[] = {{&S21818[12], 14}, {NULL}};
const Segment SGI336714[] = {{&S21819[6], 4}, {NULL}};
const Segment SGI336715[] = {{&S21819[74], 5}, {NULL}};
const Segment SGI336716[] = {{&S21819[12], 14}, {&S21819[37], 42}, {NULL}};
const Segment SGI336717[] = {{&S21819[12], 14}, {NULL}};
const Segment SGI336718[] = {{&S21820[6], 4}, {NULL}};
const Segment SGI336719[] = {{&S21820[74], 5}, {NULL}};
const Segment SGI336720[] = {{&S21820[12], 14}, {&S21820[37], 42}, {NULL}};
const Segment SGI336721[] = {{&S21820[12], 14}, {NULL}};
const Segment SGI336722[] = {{&S21821[6], 4}, {NULL}};
const Segment SGI336723[] = {{&S21821[73], 5}, {NULL}};
const Segment SGI336724[] = {{&S21821[12], 14}, {NULL}};
const Segment SGI336725[] = {{&S21822[6], 4}, {NULL}};
const Segment SGI336726[] = {{&S21822[74], 5}, {NULL}};
const Segment SGI336727[] = {{&S21822[12], 14}, {&S21822[37], 42}, {NULL}};
const Segment SGI336728[] = {{&S21822[12], 14}, {NULL}};
const Segment SGI336729[] = {{&S21823[6], 4}, {NULL}};
const Segment SGI336730[] = {{&S21823[74], 5}, {NULL}};
const Segment SGI336731[] = {{&S21823[12], 14}, {&S21823[37], 42}, {NULL}};
const Segment SGI336732[] = {{&S21823[12], 14}, {NULL}};
const Segment SGI336733[] = {{&S21824[6], 4}, {NULL}};
const Segment SGI336734[] = {{&S21824[74], 5}, {NULL}};
const Segment SGI336735[] = {{&S21824[12], 14}, {&S21824[37], 42}, {NULL}};
const Segment SGI336736[] = {{&S21824[12], 14}, {NULL}};
const Segment SGI336737[] = {{&S21825[6], 4}, {NULL}};
const Segment SGI336738[] = {{&S21825[74], 5}, {NULL}};
const Segment SGI336739[] = {{&S21825[12], 14}, {&S21825[37], 42}, {NULL}};
const Segment SGI336740[] = {{&S21825[12], 14}, {NULL}};
const Segment SGI336741[] = {{&S21826[6], 4}, {NULL}};
const Segment SGI336742[] = {{&S21826[75], 5}, {NULL}};
const Segment SGI336743[] = {{&S21826[12], 14}, {&S21826[38], 42}, {NULL}};
const Segment SGI336744[] = {{&S21826[12], 14}, {NULL}};
const Segment SGI336745[] = {{&S21827[6], 4}, {NULL}};
const Segment SGI336746[] = {{&S21827[74], 5}, {NULL}};
const Segment SGI336747[] = {{&S21827[12], 14}, {&S21827[37], 42}, {NULL}};
const Segment SGI336748[] = {{&S21827[12], 14}, {NULL}};
const Segment SGI336749[] = {{&S21828[6], 4}, {NULL}};
const Segment SGI336750[] = {{&S21828[75], 5}, {NULL}};
const Segment SGI336751[] = {{&S21828[12], 14}, {&S21828[37], 43}, {NULL}};
const Segment SGI336752[] = {{&S21828[12], 14}, {NULL}};
const Segment SGI336753[] = {{&S21829[6], 4}, {NULL}};
const Segment SGI336754[] = {{&S21829[74], 5}, {NULL}};
const Segment SGI336755[] = {{&S21829[12], 14}, {&S21829[37], 42}, {NULL}};
const Segment SGI336756[] = {{&S21829[12], 14}, {NULL}};
const Segment SGI336757[] = {{&S21830[6], 4}, {NULL}};
const Segment SGI336758[] = {{&S21830[52], 5}, {NULL}};
const Segment SGI336759[] = {{&S21830[12], 55}, {NULL}};
const Segment SGI336760[] = {{&S21830[12], 14}, {NULL}};
const Segment SGI336761[] = {{&S21831[6], 4}, {NULL}};
const Segment SGI336762[] = {{&S21831[74], 5}, {NULL}};
const Segment SGI336763[] = {{&S21831[12], 14}, {&S21831[37], 42}, {NULL}};
const Segment SGI336764[] = {{&S21831[12], 14}, {NULL}};
const Segment SGI336765[] = {{&S21832[6], 4}, {NULL}};
const Segment SGI336766[] = {{&S21832[74], 5}, {NULL}};
const Segment SGI336767[] = {{&S21832[12], 14}, {&S21832[37], 42}, {NULL}};
const Segment SGI336768[] = {{&S21832[12], 14}, {NULL}};
const Segment SGI336769[] = {{&S21834[6], 4}, {NULL}};
const Segment SGI336770[] = {{&S21834[75], 5}, {NULL}};
const Segment SGI336771[] = {{&S21834[12], 14}, {&S21834[38], 42}, {NULL}};
const Segment SGI336772[] = {{&S21834[12], 14}, {NULL}};
const Segment SGI336773[] = {{&S21836[6], 4}, {NULL}};
const Segment SGI336774[] = {{&S21836[74], 5}, {NULL}};
const Segment SGI336775[] = {{&S21836[12], 14}, {&S21836[37], 42}, {NULL}};
const Segment SGI336776[] = {{&S21836[12], 14}, {NULL}};
const Segment SGI336777[] = {{&S21837[6], 4}, {NULL}};
const Segment SGI336778[] = {{&S21837[75], 5}, {NULL}};
const Segment SGI336779[] = {{&S21837[12], 14}, {&S21837[38], 42}, {NULL}};
const Segment SGI336780[] = {{&S21837[12], 14}, {NULL}};
const Segment SGI336781[] = {{&S21838[6], 4}, {NULL}};
const Segment SGI336782[] = {{&S21838[74], 5}, {NULL}};
const Segment SGI336783[] = {{&S21838[12], 14}, {&S21838[37], 42}, {NULL}};
const Segment SGI336784[] = {{&S21838[12], 14}, {NULL}};
const Segment SGI336785[] = {{&S21839[6], 4}, {NULL}};
const Segment SGI336786[] = {{&S21839[74], 5}, {NULL}};
const Segment SGI336787[] = {{&S21839[12], 14}, {&S21839[37], 42}, {NULL}};
const Segment SGI336788[] = {{&S21839[12], 14}, {NULL}};
const Segment SGI336789[] = {{&S21840[6], 4}, {NULL}};
const Segment SGI336790[] = {{&S21840[74], 5}, {NULL}};
const Segment SGI336791[] = {{&S21840[12], 14}, {&S21840[37], 42}, {NULL}};
const Segment SGI336792[] = {{&S21840[12], 14}, {NULL}};
const Segment SGI336793[] = {{&S21841[6], 4}, {NULL}};
const Segment SGI336794[] = {{&S21841[52], 5}, {NULL}};
const Segment SGI336795[] = {{&S21841[12], 45}, {NULL}};
const Segment SGI336796[] = {{&S21841[12], 14}, {NULL}};
const Segment SGI336797[] = {{&S21842[6], 4}, {NULL}};
const Segment SGI336798[] = {{&S21842[74], 5}, {NULL}};
const Segment SGI336799[] = {{&S21842[12], 14}, {&S21842[37], 42}, {NULL}};
const Segment SGI336800[] = {{&S21842[12], 14}, {NULL}};
const Segment SGI336801[] = {{&S21843[6], 4}, {NULL}};
const Segment SGI336802[] = {{&S21843[75], 5}, {NULL}};
const Segment SGI336803[] = {{&S21843[12], 14}, {&S21843[37], 43}, {NULL}};
const Segment SGI336804[] = {{&S21843[12], 14}, {NULL}};
const Segment SGI336805[] = {{&S21844[6], 4}, {NULL}};
const Segment SGI336806[] = {{&S21844[75], 5}, {NULL}};
const Segment SGI336807[] = {{&S21844[12], 14}, {&S21844[37], 43}, {NULL}};
const Segment SGI336808[] = {{&S21844[12], 14}, {NULL}};
const Segment SGI336809[] = {{&S21845[6], 4}, {NULL}};
const Segment SGI336810[] = {{&S21845[74], 5}, {NULL}};
const Segment SGI336811[] = {{&S21845[12], 14}, {&S21845[37], 42}, {NULL}};
const Segment SGI336812[] = {{&S21845[12], 14}, {NULL}};
const Segment SGI336813[] = {{&S21846[6], 4}, {NULL}};
const Segment SGI336814[] = {{&S21846[74], 5}, {NULL}};
const Segment SGI336815[] = {{&S21846[12], 14}, {&S21846[37], 42}, {NULL}};
const Segment SGI336816[] = {{&S21846[12], 14}, {NULL}};
const Segment SGI336817[] = {{&S21847[6], 4}, {NULL}};
const Segment SGI336818[] = {{&S21847[52], 5}, {NULL}};
const Segment SGI336819[] = {{&S21847[12], 62}, {NULL}};
const Segment SGI336820[] = {{&S21847[12], 14}, {NULL}};
const Segment SGI336821[] = {{&S21848[6], 4}, {NULL}};
const Segment SGI336822[] = {{&S21848[74], 5}, {NULL}};
const Segment SGI336823[] = {{&S21848[12], 14}, {&S21848[37], 42}, {NULL}};
const Segment SGI336824[] = {{&S21848[12], 14}, {NULL}};
const Segment SGI336825[] = {{&S21849[6], 4}, {NULL}};
const Segment SGI336826[] = {{&S21849[74], 5}, {NULL}};
const Segment SGI336827[] = {{&S21849[12], 14}, {&S21849[37], 42}, {NULL}};
const Segment SGI336828[] = {{&S21849[12], 14}, {NULL}};
const Segment SGI336829[] = {{&S21850[6], 4}, {NULL}};
const Segment SGI336830[] = {{&S21850[75], 5}, {NULL}};
const Segment SGI336831[] = {{&S21850[12], 14}, {&S21850[37], 43}, {NULL}};
const Segment SGI336832[] = {{&S21850[12], 14}, {NULL}};
const Segment SGI336833[] = {{&S21851[6], 4}, {NULL}};
const Segment SGI336834[] = {{&S21851[74], 5}, {NULL}};
const Segment SGI336835[] = {{&S21851[12], 14}, {&S21851[37], 42}, {NULL}};
const Segment SGI336836[] = {{&S21851[12], 14}, {NULL}};
const Segment SGI336837[] = {{&S21852[6], 4}, {NULL}};
const Segment SGI336838[] = {{&S21852[74], 5}, {NULL}};
const Segment SGI336839[] = {{&S21852[12], 14}, {&S21852[37], 42}, {NULL}};
const Segment SGI336840[] = {{&S21852[12], 14}, {NULL}};
const Segment SGI336841[] = {{&S21853[6], 4}, {NULL}};
const Segment SGI336842[] = {{&S21853[75], 5}, {NULL}};
const Segment SGI336843[] = {{&S21853[12], 14}, {&S21853[38], 42}, {NULL}};
const Segment SGI336844[] = {{&S21853[12], 14}, {NULL}};
const Segment SGI336845[] = {{&S21854[6], 4}, {NULL}};
const Segment SGI336846[] = {{&S21854[74], 5}, {NULL}};
const Segment SGI336847[] = {{&S21854[12], 14}, {&S21854[37], 42}, {NULL}};
const Segment SGI336848[] = {{&S21854[12], 14}, {NULL}};
const Segment SGI336849[] = {{&S21855[6], 4}, {NULL}};
const Segment SGI336850[] = {{&S21855[74], 5}, {NULL}};
const Segment SGI336851[] = {{&S21855[12], 14}, {&S21855[37], 42}, {NULL}};
const Segment SGI336852[] = {{&S21855[12], 14}, {NULL}};
const Segment SGI336853[] = {{&S21856[6], 4}, {NULL}};
const Segment SGI336854[] = {{&S21856[75], 5}, {NULL}};
const Segment SGI336855[] = {{&S21856[12], 14}, {&S21856[37], 43}, {NULL}};
const Segment SGI336856[] = {{&S21856[12], 14}, {NULL}};
const Segment SGI336857[] = {{&S21857[6], 4}, {NULL}};
const Segment SGI336858[] = {{&S21857[75], 5}, {NULL}};
const Segment SGI336859[] = {{&S21857[12], 14}, {&S21857[37], 43}, {NULL}};
const Segment SGI336860[] = {{&S21857[12], 14}, {NULL}};
const Segment SGI336861[] = {{&S21858[6], 4}, {NULL}};
const Segment SGI336862[] = {{&S21858[74], 5}, {NULL}};
const Segment SGI336863[] = {{&S21858[12], 14}, {&S21858[37], 42}, {NULL}};
const Segment SGI336864[] = {{&S21858[12], 14}, {NULL}};
const Segment SGI336865[] = {{&S21859[6], 4}, {NULL}};
const Segment SGI336866[] = {{&S21859[75], 5}, {NULL}};
const Segment SGI336867[] = {{&S21859[12], 14}, {&S21859[37], 43}, {NULL}};
const Segment SGI336868[] = {{&S21859[12], 14}, {NULL}};
const Segment SGI336869[] = {{&S21860[6], 4}, {NULL}};
const Segment SGI336870[] = {{&S21860[74], 5}, {NULL}};
const Segment SGI336871[] = {{&S21860[12], 14}, {&S21860[37], 42}, {NULL}};
const Segment SGI336872[] = {{&S21860[12], 14}, {NULL}};
const Segment SGI336873[] = {{&S21861[6], 4}, {NULL}};
const Segment SGI336874[] = {{&S21861[75], 5}, {NULL}};
const Segment SGI336875[] = {{&S21861[12], 14}, {&S21861[38], 42}, {NULL}};
const Segment SGI336876[] = {{&S21861[12], 14}, {NULL}};
const Segment SGI336877[] = {{&S21862[6], 4}, {NULL}};
const Segment SGI336878[] = {{&S21862[75], 5}, {NULL}};
const Segment SGI336879[] = {{&S21862[12], 14}, {&S21862[38], 42}, {NULL}};
const Segment SGI336880[] = {{&S21862[12], 14}, {NULL}};
const Segment SGI336881[] = {{&S21863[6], 4}, {NULL}};
const Segment SGI336882[] = {{&S21863[74], 5}, {NULL}};
const Segment SGI336883[] = {{&S21863[12], 14}, {&S21863[37], 42}, {NULL}};
const Segment SGI336884[] = {{&S21863[12], 14}, {NULL}};
const Segment SGI336885[] = {{&S21864[6], 4}, {NULL}};
const Segment SGI336886[] = {{&S21864[74], 5}, {NULL}};
const Segment SGI336887[] = {{&S21864[12], 14}, {&S21864[37], 42}, {NULL}};
const Segment SGI336888[] = {{&S47826[5], 7}, {NULL}};
const Segment SGI336889[] = {{&S47827[5], 7}, {&S47827[5], 7}, {NULL}};
const Segment SGI336890[] = {{&S47828[5], 4}, {&S47828[5], 4}, {NULL}};
const Segment SGI336891[] = {{&S47829[5], 4}, {NULL}};
const Segment SGI336892[] = {{&S47864[42], 4}, {NULL}};
const Segment SGI336893[] = {{&S47864[6], 3}, {NULL}};
const Segment SGI336894[] = {{&S47864[67], 3}, {NULL}};
const Segment SGI336895[] = {{&S47865[5], 4}, {NULL}};
const Segment SGI336896[] = {{&S47865[15], 3}, {NULL}};
const Segment SGI336897[] = {{&S47866[42], 5}, {NULL}};
const Segment SGI336898[] = {{&S47866[6], 3}, {NULL}};
const Segment SGI336899[] = {{&S47866[68], 3}, {NULL}};
const Segment SGI336900[] = {{&S21864[12], 14}, {NULL}};
const Segment SGI336901[] = {{&S21865[6], 4}, {NULL}};
const Segment SGI336902[] = {{&S21865[74], 5}, {NULL}};
const Segment SGI336903[] = {{&S21865[12], 14}, {&S21865[37], 42}, {NULL}};
const Segment SGI336904[] = {{&S21865[12], 14}, {NULL}};
const Segment SGI336905[] = {{&S21866[6], 4}, {NULL}};
const Segment SGI336906[] = {{&S21866[74], 5}, {NULL}};
const Segment SGI336907[] = {{&S21866[12], 14}, {&S21866[37], 42}, {NULL}};
const Segment SGI336908[] = {{&S21866[12], 14}, {NULL}};
const Segment SGI336909[] = {{&S21867[6], 4}, {NULL}};
const Segment SGI336910[] = {{&S21867[101], 5}, {NULL}};
const Segment SGI336911[] = {{&S21867[12], 14}, {&S21867[37], 12}, {NULL}};
const Segment SGI336912[] = {{&S21867[12], 14}, {NULL}};
const Segment SGI336913[] = {{&S21868[6], 4}, {NULL}};
const Segment SGI336914[] = {{&S21868[74], 5}, {NULL}};
const Segment SGI336915[] = {{&S21868[12], 14}, {&S21868[37], 42}, {NULL}};
const Segment SGI336916[] = {{&S21868[12], 14}, {NULL}};
const Segment SGI336917[] = {{&S21869[6], 4}, {NULL}};
const Segment SGI336918[] = {{&S21869[74], 5}, {NULL}};
const Segment SGI336919[] = {{&S21869[12], 14}, {&S21869[37], 42}, {NULL}};
const Segment SGI336920[] = {{&S21869[12], 14}, {NULL}};
const Segment SGI336921[] = {{&S21870[6], 4}, {NULL}};
const Segment SGI336922[] = {{&S21870[74], 5}, {NULL}};
const Segment SGI336923[] = {{&S21870[12], 14}, {&S21870[37], 42}, {NULL}};
const Segment SGI336924[] = {{&S21870[12], 14}, {NULL}};
const Segment SGI336925[] = {{&S21871[6], 4}, {NULL}};
const Segment SGI336926[] = {{&S21871[102], 5}, {NULL}};
const Segment SGI336927[] = {{&S21871[12], 14}, {&S21871[37], 12}, {NULL}};
const Segment SGI336928[] = {{&S21871[12], 14}, {NULL}};
const Segment SGI336929[] = {{&S21872[6], 4}, {NULL}};
const Segment SGI336930[] = {{&S21872[74], 5}, {NULL}};
const Segment SGI336931[] = {{&S21872[12], 14}, {&S21872[37], 42}, {NULL}};
const Segment SGI336932[] = {{&S21872[12], 14}, {NULL}};
const Segment SGI336933[] = {{&S21873[6], 4}, {NULL}};
const Segment SGI336934[] = {{&S21873[74], 5}, {NULL}};
const Segment SGI336935[] = {{&S21873[12], 14}, {&S21873[37], 42}, {NULL}};
const Segment SGI336936[] = {{&S21873[12], 14}, {NULL}};
const Segment SGI336937[] = {{&S21874[6], 4}, {NULL}};
const Segment SGI336938[] = {{&S21874[75], 5}, {NULL}};
const Segment SGI336939[] = {{&S21874[12], 14}, {&S21874[38], 42}, {NULL}};
const Segment SGI336940[] = {{&S21874[12], 14}, {NULL}};
const Segment SGI336941[] = {{&S21875[6], 4}, {NULL}};
const Segment SGI336942[] = {{&S21875[102], 5}, {NULL}};
const Segment SGI336943[] = {{&S21875[12], 14}, {&S21875[37], 12}, {NULL}};
const Segment SGI336944[] = {{&S21875[12], 14}, {NULL}};
const Segment SGI336945[] = {{&S21876[6], 4}, {NULL}};
const Segment SGI336946[] = {{&S21876[75], 5}, {NULL}};
const Segment SGI336947[] = {{&S21876[12], 14}, {&S21876[38], 42}, {NULL}};
const Segment SGI336948[] = {{&S21876[12], 14}, {NULL}};
const Segment SGI336949[] = {{&S21877[6], 4}, {NULL}};
const Segment SGI336950[] = {{&S21877[74], 5}, {NULL}};
const Segment SGI336951[] = {{&S21877[12], 14}, {&S21877[37], 42}, {NULL}};
const Segment SGI336952[] = {{&S21877[12], 14}, {NULL}};
const Segment SGI336953[] = {{&S21878[6], 4}, {NULL}};
const Segment SGI336954[] = {{&S21878[75], 5}, {NULL}};
const Segment SGI336955[] = {{&S21878[12], 14}, {&S21878[38], 42}, {NULL}};
const Segment SGI336956[] = {{&S21878[12], 14}, {NULL}};
const Segment SGI336957[] = {{&S21879[6], 4}, {NULL}};
const Segment SGI336958[] = {{&S21879[74], 5}, {NULL}};
const Segment SGI336959[] = {{&S21879[12], 14}, {&S21879[37], 42}, {NULL}};
const Segment SGI336960[] = {{&S21879[12], 14}, {NULL}};
const Segment SGI336961[] = {{&S21880[6], 4}, {NULL}};
const Segment SGI336962[] = {{&S21880[75], 5}, {NULL}};
const Segment SGI336963[] = {{&S21880[12], 14}, {&S21880[37], 43}, {NULL}};
const Segment SGI336964[] = {{&S21880[12], 14}, {NULL}};
const Segment SGI336965[] = {{&S21881[6], 4}, {NULL}};
const Segment SGI336966[] = {{&S21881[75], 5}, {NULL}};
const Segment SGI336967[] = {{&S21881[12], 14}, {&S21881[38], 42}, {NULL}};
const Segment SGI336968[] = {{&S21881[12], 14}, {NULL}};
const Segment SGI336969[] = {{&S21882[6], 4}, {NULL}};
const Segment SGI336970[] = {{&S21882[75], 5}, {NULL}};
const Segment SGI336971[] = {{&S21882[12], 14}, {&S21882[37], 43}, {NULL}};
const Segment SGI336972[] = {{&S21882[12], 14}, {NULL}};
const Segment SGI336973[] = {{&S21883[6], 4}, {NULL}};
const Segment SGI336974[] = {{&S21883[75], 5}, {NULL}};
const Segment SGI336975[] = {{&S21883[12], 14}, {&S21883[38], 42}, {NULL}};
const Segment SGI336976[] = {{&S21883[12], 14}, {NULL}};
const Segment SGI336977[] = {{&S21884[6], 4}, {NULL}};
const Segment SGI336978[] = {{&S21884[64], 5}, {NULL}};
const Segment SGI336979[] = {{&S21884[12], 14}, {&S21884[75], 4}, {NULL}};
const Segment SGI336980[] = {{&S21884[12], 14}, {NULL}};
const Segment SGI336981[] = {{&S21885[6], 4}, {NULL}};
const Segment SGI336982[] = {{&S21885[74], 5}, {NULL}};
const Segment SGI336983[] = {{&S21885[12], 14}, {&S21885[37], 42}, {NULL}};
const Segment SGI336984[] = {{&S21885[12], 14}, {NULL}};
const Segment SGI336985[] = {{&S21886[6], 4}, {NULL}};
const Segment SGI336986[] = {{&S21886[74], 5}, {NULL}};
const Segment SGI336987[] = {{&S21886[12], 14}, {&S21886[37], 42}, {NULL}};
const Segment SGI336988[] = {{&S21886[12], 14}, {NULL}};
const Segment SGI336989[] = {{&S21887[6], 4}, {NULL}};
const Segment SGI336990[] = {{&S21887[74], 5}, {NULL}};
const Segment SGI336991[] = {{&S21887[12], 14}, {&S21887[37], 42}, {NULL}};
const Segment SGI336992[] = {{&S21887[12], 14}, {NULL}};
const Segment SGI336993[] = {{&S21888[6], 4}, {NULL}};
const Segment SGI336994[] = {{&S21888[74], 5}, {NULL}};
const Segment SGI336995[] = {{&S21888[12], 14}, {&S21888[37], 42}, {NULL}};
const Segment SGI336996[] = {{&S21888[12], 14}, {NULL}};
const Segment SGI336997[] = {{&S21889[6], 4}, {NULL}};
const Segment SGI336998[] = {{&S21889[74], 5}, {NULL}};
const Segment SGI336999[] = {{&S21889[12], 14}, {&S21889[37], 42}, {NULL}};
const Segment SGI337000[] = {{&S21889[12], 14}, {NULL}};
const Segment SGI337001[] = {{&S21890[6], 4}, {NULL}};
const Segment SGI337002[] = {{&S21890[75], 5}, {NULL}};
const Segment SGI337003[] = {{&S21890[12], 14}, {&S21890[37], 43}, {NULL}};
const Segment SGI337004[] = {{&S21890[12], 14}, {NULL}};
const Segment SGI337005[] = {{&S21891[6], 4}, {NULL}};
const Segment SGI337006[] = {{&S21891[75], 5}, {NULL}};
const Segment SGI337007[] = {{&S21891[12], 14}, {&S21891[37], 43}, {NULL}};
const Segment SGI337008[] = {{&S21891[12], 14}, {NULL}};
const Segment SGI337009[] = {{&S21892[6], 4}, {NULL}};
const Segment SGI337010[] = {{&S21892[75], 5}, {NULL}};
const Segment SGI337011[] = {{&S21892[12], 14}, {&S21892[38], 42}, {NULL}};
const Segment SGI337012[] = {{&S21892[12], 14}, {NULL}};
const Segment SGI337013[] = {{&S21893[6], 4}, {NULL}};
const Segment SGI337014[] = {{&S21893[74], 5}, {NULL}};
const Segment SGI337015[] = {{&S21893[12], 14}, {&S21893[37], 42}, {NULL}};
const Segment SGI337016[] = {{&S21893[12], 14}, {NULL}};
const Segment SGI337017[] = {{&S21894[6], 4}, {NULL}};
const Segment SGI337018[] = {{&S21894[75], 5}, {NULL}};
const Segment SGI337019[] = {{&S21894[12], 14}, {&S21894[38], 42}, {NULL}};
const Segment SGI337020[] = {{&S21894[12], 14}, {NULL}};
const Segment SGI337021[] = {{&S21895[6], 4}, {NULL}};
const Segment SGI337022[] = {{&S21895[74], 5}, {NULL}};
const Segment SGI337023[] = {{&S21895[12], 14}, {&S21895[37], 42}, {NULL}};
const Segment SGI337024[] = {{&S21895[12], 14}, {NULL}};
const Segment SGI337025[] = {{&S21896[6], 4}, {NULL}};
const Segment SGI337026[] = {{&S21896[74], 5}, {NULL}};
const Segment SGI337027[] = {{&S21896[12], 14}, {&S21896[37], 42}, {NULL}};
const Segment SGI337028[] = {{&S21896[12], 14}, {NULL}};
const Segment SGI337029[] = {{&S21897[6], 4}, {NULL}};
const Segment SGI337030[] = {{&S21897[74], 5}, {NULL}};
const Segment SGI337031[] = {{&S21897[12], 14}, {&S21897[37], 42}, {NULL}};
const Segment SGI337032[] = {{&S21897[12], 14}, {NULL}};
const Segment SGI337033[] = {{&S21898[6], 4}, {NULL}};
const Segment SGI337034[] = {{&S21898[74], 5}, {NULL}};
const Segment SGI337035[] = {{&S21898[12], 14}, {&S21898[37], 42}, {NULL}};
const Segment SGI337036[] = {{&S21898[12], 14}, {NULL}};
const Segment SGI337037[] = {{&S21899[6], 4}, {NULL}};
const Segment SGI337038[] = {{&S21899[74], 5}, {NULL}};
const Segment SGI337039[] = {{&S21899[12], 14}, {&S21899[37], 42}, {NULL}};
const Segment SGI337040[] = {{&S21899[12], 14}, {NULL}};
const Segment SGI337041[] = {{&S21900[6], 4}, {NULL}};
const Segment SGI337042[] = {{&S21900[74], 5}, {NULL}};
const Segment SGI337043[] = {{&S21900[12], 14}, {&S21900[37], 42}, {NULL}};
const Segment SGI337044[] = {{&S21900[12], 14}, {NULL}};
const Segment SGI337045[] = {{&S21901[6], 4}, {NULL}};
const Segment SGI337046[] = {{&S21901[75], 5}, {NULL}};
const Segment SGI337047[] = {{&S21901[12], 14}, {&S21901[37], 43}, {NULL}};
const Segment SGI337048[] = {{&S21901[12], 14}, {NULL}};
const Segment SGI337049[] = {{&S21902[6], 4}, {NULL}};
const Segment SGI337050[] = {{&S21902[102], 5}, {NULL}};
const Segment SGI337051[] = {{&S21902[12], 14}, {&S21902[37], 12}, {NULL}};
const Segment SGI337052[] = {{&S21902[12], 14}, {NULL}};
const Segment SGI337053[] = {{&S21903[6], 4}, {NULL}};
const Segment SGI337054[] = {{&S21903[52], 5}, {NULL}};
const Segment SGI337055[] = {{&S21903[12], 84}, {NULL}};
const Segment SGI337056[] = {{&S21903[12], 14}, {NULL}};
const Segment SGI337057[] = {{&S21904[6], 4}, {NULL}};
const Segment SGI337058[] = {{&S21904[74], 5}, {NULL}};
const Segment SGI337059[] = {{&S21904[12], 14}, {&S21904[37], 42}, {NULL}};
const Segment SGI337060[] = {{&S21904[12], 14}, {NULL}};
const Segment SGI337061[] = {{&S21905[6], 4}, {NULL}};
const Segment SGI337062[] = {{&S21905[75], 5}, {NULL}};
const Segment SGI337063[] = {{&S21905[12], 14}, {&S21905[38], 42}, {NULL}};
const Segment SGI337064[] = {{&S21905[12], 14}, {NULL}};
const Segment SGI337065[] = {{&S21906[6], 4}, {NULL}};
const Segment SGI337066[] = {{&S21906[75], 5}, {NULL}};
const Segment SGI337067[] = {{&S21906[12], 14}, {&S21906[37], 43}, {NULL}};
const Segment SGI337068[] = {{&S21906[12], 14}, {NULL}};
const Segment SGI337069[] = {{&S21907[6], 4}, {NULL}};
const Segment SGI337070[] = {{&S21907[75], 5}, {NULL}};
const Segment SGI337071[] = {{&S21907[12], 14}, {&S21907[37], 43}, {NULL}};
const Segment SGI337072[] = {{&S21907[12], 14}, {NULL}};
const Segment SGI337073[] = {{&S21908[6], 4}, {NULL}};
const Segment SGI337074[] = {{&S21908[74], 5}, {NULL}};
const Segment SGI337075[] = {{&S21908[12], 14}, {&S21908[37], 42}, {NULL}};
const Segment SGI337076[] = {{&S21908[12], 14}, {NULL}};
const Segment SGI337077[] = {{&S21909[6], 4}, {NULL}};
const Segment SGI337078[] = {{&S21909[75], 5}, {NULL}};
const Segment SGI337079[] = {{&S21909[12], 14}, {&S21909[38], 42}, {NULL}};
const Segment SGI337080[] = {{&S21909[12], 14}, {NULL}};
const Segment SGI337081[] = {{&S21910[6], 4}, {NULL}};
const Segment SGI337082[] = {{&S21910[75], 5}, {NULL}};
const Segment SGI337083[] = {{&S21910[12], 14}, {&S21910[38], 42}, {NULL}};
const Segment SGI337084[] = {{&S21910[12], 14}, {NULL}};
const Segment SGI337085[] = {{&S21911[6], 4}, {NULL}};
const Segment SGI337086[] = {{&S21911[75], 5}, {NULL}};
const Segment SGI337087[] = {{&S21911[12], 14}, {&S21911[37], 43}, {NULL}};
const Segment SGI337088[] = {{&S21911[12], 14}, {NULL}};
const Segment SGI337089[] = {{&S21912[6], 4}, {NULL}};
const Segment SGI337090[] = {{&S21912[75], 5}, {NULL}};
const Segment SGI337091[] = {{&S21912[12], 14}, {&S21912[37], 43}, {NULL}};
const Segment SGI337092[] = {{&S21912[12], 14}, {NULL}};
const Segment SGI337093[] = {{&S21913[6], 4}, {NULL}};
const Segment SGI337094[] = {{&S21913[74], 5}, {NULL}};
const Segment SGI337095[] = {{&S21913[12], 14}, {&S21913[37], 42}, {NULL}};
const Segment SGI337096[] = {{&S21913[12], 14}, {NULL}};
const Segment SGI337097[] = {{&S21914[6], 4}, {NULL}};
const Segment SGI337098[] = {{&S21914[74], 5}, {NULL}};
const Segment SGI337099[] = {{&S21914[12], 14}, {&S21914[37], 42}, {NULL}};
const Segment SGI337100[] = {{&S21914[12], 14}, {NULL}};
const Segment SGI337101[] = {{&S21915[6], 4}, {NULL}};
const Segment SGI337102[] = {{&S21915[75], 5}, {NULL}};
const Segment SGI337103[] = {{&S21915[12], 14}, {&S21915[38], 42}, {NULL}};
const Segment SGI337104[] = {{&S21915[12], 14}, {NULL}};
const Segment SGI337105[] = {{&S21916[6], 4}, {NULL}};
const Segment SGI337106[] = {{&S21916[74], 5}, {NULL}};
const Segment SGI337107[] = {{&S21916[12], 14}, {&S21916[37], 42}, {NULL}};
const Segment SGI337108[] = {{&S21916[12], 14}, {NULL}};
const Segment SGI337109[] = {{&S21917[6], 4}, {NULL}};
const Segment SGI337110[] = {{&S21917[102], 5}, {NULL}};
const Segment SGI337111[] = {{&S21917[12], 14}, {&S21917[37], 12}, {NULL}};
const Segment SGI337112[] = {{&S21917[12], 14}, {NULL}};
const Segment SGI337113[] = {{&S21918[6], 4}, {NULL}};
const Segment SGI337114[] = {{&S21918[74], 5}, {NULL}};
const Segment SGI337115[] = {{&S21918[12], 14}, {&S21918[37], 42}, {NULL}};
const Segment SGI337116[] = {{&S21918[12], 14}, {NULL}};
const Segment SGI337117[] = {{&S21919[6], 4}, {NULL}};
const Segment SGI337118[] = {{&S21919[75], 5}, {NULL}};
const Segment SGI337119[] = {{&S21919[12], 14}, {&S21919[37], 43}, {NULL}};
const Segment SGI337120[] = {{&S21919[12], 14}, {NULL}};
const Segment SGI337121[] = {{&S21920[6], 4}, {NULL}};
const Segment SGI337122[] = {{&S21920[75], 5}, {NULL}};
const Segment SGI337123[] = {{&S21920[12], 14}, {&S21920[37], 43}, {NULL}};
const Segment SGI337124[] = {{&S21920[12], 14}, {NULL}};
const Segment SGI337125[] = {{&S21921[6], 4}, {NULL}};
const Segment SGI337126[] = {{&S21921[74], 5}, {NULL}};
const Segment SGI337127[] = {{&S21921[12], 14}, {&S21921[37], 42}, {NULL}};
const Segment SGI337128[] = {{&S21921[12], 14}, {NULL}};
const Segment SGI337129[] = {{&S21922[6], 4}, {NULL}};
const Segment SGI337130[] = {{&S21922[74], 5}, {NULL}};
const Segment SGI337131[] = {{&S21922[12], 14}, {&S21922[37], 42}, {NULL}};
const Segment SGI337132[] = {{&S21922[12], 14}, {NULL}};
const Segment SGI337133[] = {{&S21923[6], 4}, {NULL}};
const Segment SGI337134[] = {{&S21923[75], 5}, {NULL}};
const Segment SGI337135[] = {{&S21923[12], 14}, {&S21923[37], 43}, {NULL}};
const Segment SGI337136[] = {{&S21923[12], 14}, {NULL}};
const Segment SGI337137[] = {{&S21924[6], 4}, {NULL}};
const Segment SGI337138[] = {{&S21924[75], 5}, {NULL}};
const Segment SGI337139[] = {{&S21924[12], 14}, {&S21924[37], 43}, {NULL}};
const Segment SGI337140[] = {{&S21924[12], 14}, {NULL}};
const Segment SGI337141[] = {{&S21925[6], 4}, {NULL}};
const Segment SGI337142[] = {{&S21925[75], 5}, {NULL}};
const Segment SGI337143[] = {{&S21925[12], 14}, {&S21925[37], 43}, {NULL}};
const Segment SGI337144[] = {{&S21925[12], 14}, {NULL}};
const Segment SGI337145[] = {{&S21926[6], 4}, {NULL}};
const Segment SGI337146[] = {{&S21926[102], 5}, {NULL}};
const Segment SGI337147[] = {{&S21926[12], 14}, {&S21926[37], 12}, {NULL}};
const Segment SGI337148[] = {{&S21926[12], 14}, {NULL}};
const Segment SGI337149[] = {{&S21927[6], 4}, {NULL}};
const Segment SGI337150[] = {{&S21927[75], 5}, {NULL}};
const Segment SGI337151[] = {{&S21927[12], 14}, {&S21927[37], 43}, {NULL}};
const Segment SGI337152[] = {{&S21927[12], 14}, {NULL}};
const Segment SGI337153[] = {{&S21928[6], 4}, {NULL}};
const Segment SGI337154[] = {{&S21928[74], 5}, {NULL}};
const Segment SGI337155[] = {{&S21928[12], 14}, {&S21928[37], 42}, {NULL}};
const Segment SGI337156[] = {{&S21928[12], 14}, {NULL}};
const Segment SGI337157[] = {{&S21929[6], 4}, {NULL}};
const Segment SGI337158[] = {{&S21929[74], 5}, {NULL}};
const Segment SGI337159[] = {{&S21929[12], 14}, {&S21929[37], 42}, {NULL}};
const Segment SGI337160[] = {{&S21929[12], 14}, {NULL}};
const Segment SGI337161[] = {{&S21930[6], 4}, {NULL}};
const Segment SGI337162[] = {{&S21930[74], 5}, {NULL}};
const Segment SGI337163[] = {{&S21930[12], 14}, {&S21930[37], 42}, {NULL}};
const Segment SGI337164[] = {{&S21930[12], 14}, {NULL}};
const Segment SGI337165[] = {{&S21931[6], 4}, {NULL}};
const Segment SGI337166[] = {{&S21931[74], 5}, {NULL}};
const Segment SGI337167[] = {{&S21931[12], 14}, {&S21931[37], 42}, {NULL}};
const Segment SGI337168[] = {{&S21931[12], 14}, {NULL}};
const Segment SGI337169[] = {{&S21932[6], 4}, {NULL}};
const Segment SGI337170[] = {{&S21932[74], 5}, {NULL}};
const Segment SGI337171[] = {{&S21932[12], 14}, {&S21932[37], 42}, {NULL}};
const Segment SGI337172[] = {{&S21932[12], 14}, {NULL}};
const Segment SGI337173[] = {{&S21933[6], 4}, {NULL}};
const Segment SGI337174[] = {{&S21933[74], 5}, {NULL}};
const Segment SGI337175[] = {{&S21933[12], 14}, {&S21933[37], 42}, {NULL}};
const Segment SGI337176[] = {{&S21933[12], 14}, {NULL}};
const Segment SGI337177[] = {{&S21934[6], 4}, {NULL}};
const Segment SGI337178[] = {{&S21934[74], 5}, {NULL}};
const Segment SGI337179[] = {{&S21934[12], 14}, {&S21934[37], 42}, {NULL}};
const Segment SGI337180[] = {{&S21934[12], 14}, {NULL}};
const Segment SGI337181[] = {{&S21935[6], 4}, {NULL}};
const Segment SGI337182[] = {{&S21935[74], 5}, {NULL}};
const Segment SGI337183[] = {{&S21935[12], 14}, {&S21935[37], 42}, {NULL}};
const Segment SGI337184[] = {{&S21935[12], 14}, {NULL}};
const Segment SGI337185[] = {{&S21936[6], 4}, {NULL}};
const Segment SGI337186[] = {{&S21936[52], 5}, {NULL}};
const Segment SGI337187[] = {{&S21936[12], 65}, {NULL}};
const Segment SGI337188[] = {{&S21936[12], 14}, {NULL}};
const Segment SGI337189[] = {{&S21937[6], 4}, {NULL}};
const Segment SGI337190[] = {{&S21937[74], 5}, {NULL}};
const Segment SGI337191[] = {{&S21937[12], 14}, {&S21937[37], 42}, {NULL}};
const Segment SGI337192[] = {{&S21937[12], 14}, {NULL}};
const Segment SGI337193[] = {{&S21938[6], 4}, {NULL}};
const Segment SGI337194[] = {{&S21938[74], 5}, {NULL}};
const Segment SGI337195[] = {{&S21938[12], 14}, {&S21938[37], 42}, {NULL}};
const Segment SGI337196[] = {{&S21938[12], 14}, {NULL}};
const Segment SGI337197[] = {{&S21939[6], 4}, {NULL}};
const Segment SGI337198[] = {{&S21939[74], 5}, {NULL}};
const Segment SGI337199[] = {{&S21939[12], 14}, {&S21939[37], 42}, {NULL}};
const Segment SGI337200[] = {{&S21939[12], 14}, {NULL}};
const Segment SGI337201[] = {{&S21940[6], 4}, {NULL}};
const Segment SGI337202[] = {{&S21940[74], 5}, {NULL}};
const Segment SGI337203[] = {{&S21940[12], 14}, {&S21940[37], 42}, {NULL}};
const Segment SGI337204[] = {{&S21940[12], 14}, {NULL}};
const Segment SGI337205[] = {{&S21941[6], 4}, {NULL}};
const Segment SGI337206[] = {{&S21941[102], 5}, {NULL}};
const Segment SGI337207[] = {{&S21941[12], 14}, {&S21941[37], 12}, {NULL}};
const Segment SGI337208[] = {{&S21941[12], 14}, {NULL}};
const Segment SGI337209[] = {{&S21942[6], 4}, {NULL}};
const Segment SGI337210[] = {{&S21942[74], 5}, {NULL}};
const Segment SGI337211[] = {{&S21942[12], 14}, {&S21942[37], 42}, {NULL}};
const Segment SGI337212[] = {{&S21942[12], 14}, {NULL}};
const Segment SGI337213[] = {{&S21943[6], 4}, {NULL}};
const Segment SGI337214[] = {{&S21943[74], 5}, {NULL}};
const Segment SGI337215[] = {{&S21943[12], 14}, {&S21943[37], 42}, {NULL}};
const Segment SGI337216[] = {{&S21943[12], 14}, {NULL}};
const Segment SGI337217[] = {{&S21944[6], 4}, {NULL}};
const Segment SGI337218[] = {{&S21944[74], 5}, {NULL}};
const Segment SGI337219[] = {{&S21944[12], 14}, {&S21944[37], 42}, {NULL}};
const Segment SGI337220[] = {{&S21944[12], 14}, {NULL}};
const Segment SGI337221[] = {{&S21945[6], 4}, {NULL}};
const Segment SGI337222[] = {{&S21945[74], 5}, {NULL}};
const Segment SGI337223[] = {{&S21945[12], 14}, {&S21945[37], 42}, {NULL}};
const Segment SGI337224[] = {{&S21945[12], 14}, {NULL}};
const Segment SGI337225[] = {{&S21946[6], 4}, {NULL}};
const Segment SGI337226[] = {{&S21946[74], 5}, {NULL}};
const Segment SGI337227[] = {{&S21946[12], 14}, {&S21946[37], 42}, {NULL}};
const Segment SGI337228[] = {{&S21946[12], 14}, {NULL}};
const Segment SGI337229[] = {{&S22002[6], 4}, {NULL}};
const Segment SGI337230[] = {{&S22002[57], 5}, {NULL}};
const Segment SGI337231[] = {{&S22002[12], 14}, {NULL}};
const Segment SGI337232[] = {{&S22003[6], 4}, {NULL}};
const Segment SGI337233[] = {{&S22003[57], 5}, {NULL}};
const Segment SGI337234[] = {{&S22003[12], 14}, {NULL}};
const Segment SGI337235[] = {{&S22004[6], 4}, {NULL}};
const Segment SGI337236[] = {{&S22004[57], 5}, {NULL}};
const Segment SGI337237[] = {{&S22004[12], 14}, {NULL}};
const Segment SGI337238[] = {{&S22005[6], 4}, {NULL}};
const Segment SGI337239[] = {{&S22005[80], 5}, {NULL}};
const Segment SGI337240[] = {{&S22005[12], 14}, {&S22005[40], 12}, {NULL}};
const Segment SGI337241[] = {{&S22005[12], 14}, {NULL}};
const Segment SGI337242[] = {{&S22006[6], 4}, {NULL}};
const Segment SGI337243[] = {{&S22006[57], 5}, {NULL}};
const Segment SGI337244[] = {{&S22006[12], 14}, {NULL}};
const Segment SGI337245[] = {{&S22007[6], 4}, {NULL}};
const Segment SGI337246[] = {{&S22007[80], 5}, {NULL}};
const Segment SGI337247[] = {{&S22007[12], 14}, {&S22007[40], 12}, {NULL}};
const Segment SGI337248[] = {{&S22007[12], 14}, {NULL}};
const Segment SGI337249[] = {{&S22008[6], 4}, {NULL}};
const Segment SGI337250[] = {{&S22008[80], 5}, {NULL}};
const Segment SGI337251[] = {{&S22008[12], 14}, {&S22008[40], 12}, {NULL}};
const Segment SGI337252[] = {{&S22008[12], 14}, {NULL}};
const Segment SGI337253[] = {{&S22009[6], 4}, {NULL}};
const Segment SGI337254[] = {{&S22009[80], 5}, {NULL}};
const Segment SGI337255[] = {{&S22009[12], 14}, {&S22009[40], 12}, {NULL}};
const Segment SGI337256[] = {{&S22009[12], 14}, {NULL}};
const Segment SGI337257[] = {{&S22010[6], 4}, {NULL}};
const Segment SGI337258[] = {{&S22010[80], 5}, {NULL}};
const Segment SGI337259[] = {{&S22010[12], 14}, {&S22010[40], 12}, {NULL}};
const Segment SGI337260[] = {{&S22010[12], 14}, {NULL}};
const Segment SGI337261[] = {{&S22011[6], 4}, {NULL}};
const Segment SGI337262[] = {{&S22011[80], 5}, {NULL}};
const Segment SGI337263[] = {{&S22011[12], 14}, {&S22011[40], 12}, {NULL}};
const Segment SGI337264[] = {{&S22011[12], 14}, {NULL}};
const Segment SGI337265[] = {{&S22012[6], 4}, {NULL}};
const Segment SGI337266[] = {{&S22012[80], 5}, {NULL}};
const Segment SGI337267[] = {{&S22012[12], 14}, {&S22012[40], 12}, {NULL}};
const Segment SGI337268[] = {{&S22012[12], 14}, {NULL}};
const Segment SGI337269[] = {{&S22013[6], 4}, {NULL}};
const Segment SGI337270[] = {{&S22013[80], 5}, {NULL}};
const Segment SGI337271[] = {{&S22013[12], 14}, {&S22013[40], 12}, {NULL}};
const Segment SGI337272[] = {{&S22013[12], 14}, {NULL}};
const Segment SGI337273[] = {{&S22014[6], 4}, {NULL}};
const Segment SGI337274[] = {{&S22014[57], 5}, {NULL}};
const Segment SGI337275[] = {{&S22014[12], 14}, {NULL}};
const Segment SGI337276[] = {{&S22015[6], 4}, {NULL}};
const Segment SGI337277[] = {{&S22015[57], 5}, {NULL}};
const Segment SGI337278[] = {{&S22015[12], 14}, {NULL}};
const Segment SGI337279[] = {{&S22016[6], 4}, {NULL}};
const Segment SGI337280[] = {{&S22016[80], 5}, {NULL}};
const Segment SGI337281[] = {{&S22016[12], 14}, {&S22016[40], 12}, {NULL}};
const Segment SGI337282[] = {{&S22016[12], 14}, {NULL}};
const Segment SGI337283[] = {{&S22017[6], 4}, {NULL}};
const Segment SGI337284[] = {{&S22017[80], 5}, {NULL}};
const Segment SGI337285[] = {{&S22017[12], 14}, {&S22017[40], 12}, {NULL}};
const Segment SGI337286[] = {{&S22017[12], 14}, {NULL}};
const Segment SGI337287[] = {{&S22018[6], 4}, {NULL}};
const Segment SGI337288[] = {{&S22018[80], 5}, {NULL}};
const Segment SGI337289[] = {{&S22018[12], 14}, {&S22018[40], 12}, {NULL}};
const Segment SGI337290[] = {{&S22018[12], 14}, {NULL}};
const Segment SGI337291[] = {{&S22019[6], 4}, {NULL}};
const Segment SGI337292[] = {{&S22019[80], 5}, {NULL}};
const Segment SGI337293[] = {{&S22019[12], 14}, {&S22019[40], 12}, {NULL}};
const Segment SGI337294[] = {{&S22019[12], 14}, {NULL}};
const Segment SGI337295[] = {{&S22020[6], 4}, {NULL}};
const Segment SGI337296[] = {{&S22020[80], 5}, {NULL}};
const Segment SGI337297[] = {{&S22020[12], 14}, {&S22020[40], 12}, {NULL}};
const Segment SGI337298[] = {{&S22020[12], 14}, {NULL}};
const Segment SGI337299[] = {{&S22021[6], 4}, {NULL}};
const Segment SGI337300[] = {{&S22021[80], 5}, {NULL}};
const Segment SGI337301[] = {{&S22021[12], 14}, {&S22021[40], 12}, {NULL}};
const Segment SGI337302[] = {{&S22021[12], 14}, {NULL}};
const Segment SGI337303[] = {{&S22022[6], 4}, {NULL}};
const Segment SGI337304[] = {{&S22022[80], 5}, {NULL}};
const Segment SGI337305[] = {{&S22022[12], 14}, {&S22022[40], 12}, {NULL}};
const Segment SGI337306[] = {{&S22022[12], 14}, {NULL}};
const Segment SGI337307[] = {{&S22023[6], 4}, {NULL}};
const Segment SGI337308[] = {{&S22023[107], 5}, {NULL}};
const Segment SGI337309[] = {{&S22023[12], 14}, {&S22023[40], 12}, {NULL}};
const Segment SGI337310[] = {{&S22023[12], 14}, {NULL}};
const Segment SGI337311[] = {{&S22024[6], 4}, {NULL}};
const Segment SGI337312[] = {{&S22024[57], 5}, {NULL}};
const Segment SGI337313[] = {{&S22024[12], 14}, {NULL}};
const Segment SGI337314[] = {{&S22025[6], 4}, {NULL}};
const Segment SGI337315[] = {{&S22025[57], 5}, {NULL}};
const Segment SGI337316[] = {{&S22025[12], 14}, {NULL}};
const Segment SGI337317[] = {{&S22026[6], 4}, {NULL}};
const Segment SGI337318[] = {{&S22026[80], 5}, {NULL}};
const Segment SGI337319[] = {{&S22026[12], 14}, {&S22026[40], 12}, {NULL}};
const Segment SGI337320[] = {{&S22026[12], 14}, {NULL}};
const Segment SGI337321[] = {{&S22027[6], 4}, {NULL}};
const Segment SGI337322[] = {{&S22027[80], 5}, {NULL}};
const Segment SGI337323[] = {{&S22027[12], 14}, {&S22027[40], 12}, {NULL}};
const Segment SGI337324[] = {{&S22027[12], 14}, {NULL}};
const Segment SGI337325[] = {{&S22028[6], 4}, {NULL}};
const Segment SGI337326[] = {{&S22028[57], 5}, {NULL}};
const Segment SGI337327[] = {{&S22028[12], 14}, {&S22028[68], 31}, {NULL}};
const Segment SGI337328[] = {{&S22028[12], 14}, {NULL}};
const Segment SGI337329[] = {{&S22029[6], 4}, {NULL}};
const Segment SGI337330[] = {{&S22029[80], 5}, {NULL}};
const Segment SGI337331[] = {{&S22029[12], 14}, {&S22029[40], 12}, {NULL}};
const Segment SGI337332[] = {{&S22029[12], 14}, {NULL}};
const Segment SGI337333[] = {{&S22030[6], 4}, {NULL}};
const Segment SGI337334[] = {{&S22030[80], 5}, {NULL}};
const Segment SGI337335[] = {{&S22030[12], 14}, {&S22030[40], 12}, {NULL}};
const Segment SGI337336[] = {{&S22030[12], 14}, {NULL}};
const Segment SGI337337[] = {{&S22031[6], 4}, {NULL}};
const Segment SGI337338[] = {{&S22031[80], 5}, {NULL}};
const Segment SGI337339[] = {{&S22031[12], 14}, {&S22031[40], 12}, {NULL}};
const Segment SGI337340[] = {{&S22031[12], 14}, {NULL}};
const Segment SGI337341[] = {{&S22032[6], 4}, {NULL}};
const Segment SGI337342[] = {{&S22032[80], 5}, {NULL}};
const Segment SGI337343[] = {{&S22032[12], 14}, {&S22032[40], 12}, {NULL}};
const Segment SGI337344[] = {{&S22032[12], 14}, {NULL}};
const Segment SGI337345[] = {{&S22033[6], 4}, {NULL}};
const Segment SGI337346[] = {{&S22033[80], 5}, {NULL}};
const Segment SGI337347[] = {{&S22033[12], 14}, {&S22033[40], 12}, {NULL}};
const Segment SGI337348[] = {{&S22033[12], 14}, {NULL}};
const Segment SGI337349[] = {{&S22034[6], 4}, {NULL}};
const Segment SGI337350[] = {{&S22034[80], 5}, {NULL}};
const Segment SGI337351[] = {{&S22034[12], 14}, {&S22034[40], 12}, {NULL}};
const Segment SGI337352[] = {{&S22034[12], 14}, {NULL}};
const Segment SGI337353[] = {{&S39216[28], 3}, {NULL}};
const Segment SGI337354[] = {{&S22035[6], 4}, {NULL}};
const Segment SGI337355[] = {{&S22035[80], 5}, {NULL}};
const Segment SGI337356[] = {{&S39216[102], 5}, {NULL}};
const Segment SGI337357[] = {{&S22035[12], 14}, {&S22035[40], 12}, {NULL}};
const Segment SGI337358[] = {{&S22035[12], 14}, {NULL}};
const Segment SGI337359[] = {{&S22036[6], 4}, {NULL}};
const Segment SGI337360[] = {{&S22036[80], 5}, {NULL}};
const Segment SGI337361[] = {{&S22036[12], 14}, {&S22036[40], 12}, {NULL}};
const Segment SGI337362[] = {{&S22036[12], 14}, {NULL}};
const Segment SGI337363[] = {{&S22037[6], 4}, {NULL}};
const Segment SGI337364[] = {{&S22037[80], 5}, {NULL}};
const Segment SGI337365[] = {{&S22037[12], 14}, {&S22037[40], 12}, {NULL}};
const Segment SGI337366[] = {{&S22037[12], 14}, {NULL}};
const Segment SGI337367[] = {{&S22038[6], 4}, {NULL}};
const Segment SGI337368[] = {{&S22038[79], 5}, {NULL}};
const Segment SGI337369[] = {{&S22038[12], 14}, {&S22038[40], 11}, {NULL}};
const Segment SGI337370[] = {{&S22038[12], 14}, {NULL}};
const Segment SGI337371[] = {{&S39220[28], 3}, {NULL}};
const Segment SGI337372[] = {{&S39220[102], 5}, {NULL}};
const Segment SGI337373[] = {{&S22039[6], 4}, {NULL}};
const Segment SGI337374[] = {{&S22039[79], 5}, {NULL}};
const Segment SGI337375[] = {{&S22039[12], 14}, {&S22039[40], 11}, {NULL}};
const Segment SGI337376[] = {{&S22039[12], 14}, {NULL}};
const Segment SGI337377[] = {{&S39221[28], 3}, {NULL}};
const Segment SGI337378[] = {{&S39221[102], 5}, {NULL}};
const Segment SGI337379[] = {{&S22040[6], 4}, {NULL}};
const Segment SGI337380[] = {{&S22040[79], 5}, {NULL}};
const Segment SGI337381[] = {{&S22040[12], 14}, {&S22040[40], 11}, {NULL}};
const Segment SGI337382[] = {{&S22040[12], 14}, {NULL}};
const Segment SGI337383[] = {{&S22041[6], 4}, {NULL}};
const Segment SGI337384[] = {{&S22041[79], 5}, {NULL}};
const Segment SGI337385[] = {{&S22041[12], 14}, {&S22041[40], 11}, {NULL}};
const Segment SGI337386[] = {{&S22041[12], 14}, {NULL}};
const Segment SGI337387[] = {{&S22042[6], 4}, {NULL}};
const Segment SGI337388[] = {{&S22042[80], 5}, {NULL}};
const Segment SGI337389[] = {{&S22042[12], 14}, {&S22042[40], 12}, {NULL}};
const Segment SGI337390[] = {{&S22042[12], 14}, {NULL}};
const Segment SGI337391[] = {{&S22043[6], 5}, {NULL}};
const Segment SGI337392[] = {{&S22043[13], 14}, {NULL}};
const Segment SGI337393[] = {{&S22044[6], 4}, {NULL}};
const Segment SGI337394[] = {{&S22044[80], 5}, {NULL}};
const Segment SGI337395[] = {{&S22044[12], 14}, {&S22044[41], 11}, {NULL}};
const Segment SGI337396[] = {{&S22044[12], 14}, {NULL}};
const Segment SGI337397[] = {{&S22045[6], 4}, {NULL}};
const Segment SGI337398[] = {{&S22045[80], 5}, {NULL}};
const Segment SGI337399[] = {{&S22045[12], 14}, {&S22045[41], 11}, {NULL}};
const Segment SGI337400[] = {{&S22045[12], 14}, {NULL}};
const Segment SGI337401[] = {{&S22046[6], 4}, {NULL}};
const Segment SGI337402[] = {{&S22046[80], 5}, {NULL}};
const Segment SGI337403[] = {{&S22046[12], 14}, {&S22046[40], 12}, {NULL}};
const Segment SGI337404[] = {{&S22046[12], 14}, {NULL}};
const Segment SGI337405[] = {{&S22047[6], 4}, {NULL}};
const Segment SGI337406[] = {{&S22047[80], 5}, {NULL}};
const Segment SGI337407[] = {{&S22047[12], 14}, {&S22047[40], 12}, {NULL}};
const Segment SGI337408[] = {{&S22047[12], 14}, {NULL}};
const Segment SGI337409[] = {{&S22048[6], 4}, {NULL}};
const Segment SGI337410[] = {{&S22048[57], 5}, {NULL}};
const Segment SGI337411[] = {{&S22048[12], 14}, {&S22048[68], 31}, {NULL}};
const Segment SGI337412[] = {{&S22048[12], 14}, {NULL}};
const Segment SGI337413[] = {{&S22049[6], 4}, {NULL}};
const Segment SGI337414[] = {{&S22049[43], 4}, {NULL}};
const Segment SGI337415[] = {{&S22049[12], 14}, {NULL}};
const Segment SGI337416[] = {{&S22050[6], 4}, {NULL}};
const Segment SGI337417[] = {{&S22050[57], 5}, {NULL}};
const Segment SGI337418[] = {{&S22050[12], 14}, {&S22050[68], 31}, {NULL}};
const Segment SGI337419[] = {{&S22050[12], 14}, {NULL}};
const Segment SGI337420[] = {{&S22051[6], 4}, {NULL}};
const Segment SGI337421[] = {{&S22051[79], 5}, {NULL}};
const Segment SGI337422[] = {{&S22051[12], 14}, {&S22051[40], 11}, {NULL}};
const Segment SGI337423[] = {{&S22051[12], 14}, {NULL}};
const Segment SGI337424[] = {{&S22052[6], 4}, {NULL}};
const Segment SGI337425[] = {{&S22052[43], 4}, {NULL}};
const Segment SGI337426[] = {{&S22052[12], 14}, {NULL}};
const Segment SGI337427[] = {{&S22053[6], 4}, {NULL}};
const Segment SGI337428[] = {{&S22053[79], 5}, {NULL}};
const Segment SGI337429[] = {{&S22053[12], 14}, {&S22053[40], 11}, {NULL}};
const Segment SGI337430[] = {{&S22053[12], 14}, {NULL}};
const Segment SGI337431[] = {{&S22054[6], 4}, {NULL}};
const Segment SGI337432[] = {{&S22054[79], 5}, {NULL}};
const Segment SGI337433[] = {{&S22054[12], 14}, {&S22054[40], 11}, {NULL}};
const Segment SGI337434[] = {{&S22054[12], 14}, {NULL}};
const Segment SGI337435[] = {{&S22055[6], 4}, {NULL}};
const Segment SGI337436[] = {{&S22055[79], 5}, {NULL}};
const Segment SGI337437[] = {{&S22055[12], 14}, {&S22055[40], 11}, {NULL}};
const Segment SGI337438[] = {{&S22055[12], 14}, {NULL}};
const Segment SGI337439[] = {{&S22056[6], 4}, {NULL}};
const Segment SGI337440[] = {{&S22056[43], 4}, {NULL}};
const Segment SGI337441[] = {{&S22056[12], 14}, {NULL}};
const Segment SGI337442[] = {{&S22057[6], 4}, {NULL}};
const Segment SGI337443[] = {{&S22057[79], 5}, {NULL}};
const Segment SGI337444[] = {{&S22057[12], 14}, {&S22057[40], 11}, {NULL}};
const Segment SGI337445[] = {{&S22057[12], 14}, {NULL}};
const Segment SGI337446[] = {{&S22058[6], 4}, {NULL}};
const Segment SGI337447[] = {{&S22058[80], 5}, {NULL}};
const Segment SGI337448[] = {{&S22058[12], 14}, {&S22058[40], 12}, {NULL}};
const Segment SGI337449[] = {{&S22058[12], 14}, {NULL}};
const Segment SGI337450[] = {{&S22059[6], 4}, {NULL}};
const Segment SGI337451[] = {{&S22059[43], 4}, {NULL}};
const Segment SGI337452[] = {{&S22059[12], 14}, {NULL}};
const Segment SGI337453[] = {{&S22060[6], 4}, {NULL}};
const Segment SGI337454[] = {{&S22060[80], 5}, {NULL}};
const Segment SGI337455[] = {{&S22060[12], 14}, {&S22060[41], 11}, {NULL}};
const Segment SGI337456[] = {{&S22060[12], 14}, {NULL}};
const Segment SGI337457[] = {{&S22061[6], 4}, {NULL}};
const Segment SGI337458[] = {{&S22061[79], 5}, {NULL}};
const Segment SGI337459[] = {{&S22061[12], 14}, {&S22061[40], 11}, {NULL}};
const Segment SGI337460[] = {{&S22061[12], 14}, {NULL}};
const Segment SGI337461[] = {{&S22062[6], 4}, {NULL}};
const Segment SGI337462[] = {{&S22062[79], 5}, {NULL}};
const Segment SGI337463[] = {{&S22062[12], 14}, {&S22062[40], 11}, {NULL}};
const Segment SGI337464[] = {{&S22062[12], 14}, {NULL}};
const Segment SGI337465[] = {{&S22063[6], 4}, {NULL}};
const Segment SGI337466[] = {{&S22063[79], 5}, {NULL}};
const Segment SGI337467[] = {{&S22063[12], 14}, {&S22063[40], 11}, {NULL}};
const Segment SGI337468[] = {{&S22063[12], 14}, {NULL}};
const Segment SGI337469[] = {{&S22064[6], 4}, {NULL}};
const Segment SGI337470[] = {{&S22064[79], 5}, {NULL}};
const Segment SGI337471[] = {{&S22064[12], 14}, {&S22064[40], 11}, {NULL}};
const Segment SGI337472[] = {{&S22064[12], 14}, {NULL}};
const Segment SGI337473[] = {{&S22065[6], 4}, {NULL}};
const Segment SGI337474[] = {{&S22065[57], 5}, {NULL}};
const Segment SGI337475[] = {{&S22065[12], 14}, {&S22065[68], 4}, {NULL}};
const Segment SGI337476[] = {{&S22065[12], 14}, {NULL}};
const Segment SGI337477[] = {{&S22066[6], 4}, {NULL}};
const Segment SGI337478[] = {{&S22066[80], 5}, {NULL}};
const Segment SGI337479[] = {{&S22066[12], 14}, {&S22066[41], 11}, {NULL}};
const Segment SGI337480[] = {{&S22066[12], 14}, {NULL}};
const Segment SGI337481[] = {{&S22067[6], 4}, {NULL}};
const Segment SGI337482[] = {{&S22067[79], 5}, {NULL}};
const Segment SGI337483[] = {{&S22067[12], 14}, {&S22067[40], 11}, {NULL}};
const Segment SGI337484[] = {{&S22067[12], 14}, {NULL}};
const Segment SGI337485[] = {{&S22068[6], 4}, {NULL}};
const Segment SGI337486[] = {{&S22068[79], 5}, {NULL}};
const Segment SGI337487[] = {{&S22068[12], 14}, {&S22068[40], 11}, {NULL}};
const Segment SGI337488[] = {{&S22068[12], 14}, {NULL}};
const Segment SGI337489[] = {{&S22069[6], 4}, {NULL}};
const Segment SGI337490[] = {{&S22069[79], 5}, {NULL}};
const Segment SGI337491[] = {{&S22069[12], 14}, {&S22069[40], 11}, {NULL}};
const Segment SGI337492[] = {{&S22069[12], 14}, {NULL}};
const Segment SGI337493[] = {{&S22070[6], 4}, {NULL}};
const Segment SGI337494[] = {{&S22070[57], 5}, {NULL}};
const Segment SGI337495[] = {{&S22070[12], 14}, {&S22070[68], 4}, {NULL}};
const Segment SGI337496[] = {{&S22070[12], 14}, {NULL}};
const Segment SGI337497[] = {{&S22071[6], 4}, {NULL}};
const Segment SGI337498[] = {{&S22071[79], 5}, {NULL}};
const Segment SGI337499[] = {{&S22071[12], 14}, {&S22071[40], 11}, {NULL}};
const Segment SGI337500[] = {{&S22071[12], 14}, {NULL}};
const Segment SGI337501[] = {{&S22079[6], 4}, {NULL}};
const Segment SGI337502[] = {{&S22079[52], 5}, {NULL}};
const Segment SGI337503[] = {{&S22079[12], 45}, {NULL}};
const Segment SGI337504[] = {{&S22079[12], 14}, {NULL}};
const Segment SGI337505[] = {{&S22080[6], 4}, {NULL}};
const Segment SGI337506[] = {{&S22080[59], 5}, {NULL}};
const Segment SGI337507[] = {{&S22080[12], 14}, {NULL}};
const Segment SGI337508[] = {{&S22081[6], 4}, {NULL}};
const Segment SGI337509[] = {{&S22081[74], 5}, {NULL}};
const Segment SGI337510[] = {{&S22081[12], 14}, {&S22081[37], 42}, {NULL}};
const Segment SGI337511[] = {{&S22081[12], 14}, {NULL}};
const Segment SGI337512[] = {{&S22082[6], 4}, {NULL}};
const Segment SGI337513[] = {{&S22082[64], 5}, {NULL}};
const Segment SGI337514[] = {{&S22082[12], 14}, {NULL}};
const Segment SGI337515[] = {{&S22083[6], 4}, {NULL}};
const Segment SGI337516[] = {{&S22083[74], 5}, {NULL}};
const Segment SGI337517[] = {{&S22083[12], 14}, {&S22083[37], 42}, {NULL}};
const Segment SGI337518[] = {{&S22083[12], 14}, {NULL}};
const Segment SGI337519[] = {{&S22084[6], 4}, {NULL}};
const Segment SGI337520[] = {{&S22084[74], 5}, {NULL}};
const Segment SGI337521[] = {{&S22084[12], 14}, {&S22084[37], 42}, {NULL}};
const Segment SGI337522[] = {{&S22084[12], 14}, {NULL}};
const Segment SGI337523[] = {{&S22085[6], 4}, {NULL}};
const Segment SGI337524[] = {{&S22085[74], 5}, {NULL}};
const Segment SGI337525[] = {{&S22085[12], 14}, {&S22085[37], 42}, {NULL}};
const Segment SGI337526[] = {{&S22085[12], 14}, {NULL}};
const Segment SGI337527[] = {{&S22086[6], 4}, {NULL}};
const Segment SGI337528[] = {{&S22086[74], 5}, {NULL}};
const Segment SGI337529[] = {{&S22086[12], 14}, {&S22086[37], 42}, {NULL}};
const Segment SGI337530[] = {{&S22086[12], 14}, {NULL}};
const Segment SGI337531[] = {{&S22087[6], 4}, {NULL}};
const Segment SGI337532[] = {{&S22087[74], 5}, {NULL}};
const Segment SGI337533[] = {{&S22087[12], 14}, {&S22087[37], 42}, {NULL}};
const Segment SGI337534[] = {{&S22087[12], 14}, {NULL}};
const Segment SGI337535[] = {{&S22088[6], 4}, {NULL}};
const Segment SGI337536[] = {{&S22088[75], 5}, {NULL}};
const Segment SGI337537[] = {{&S22088[12], 14}, {&S22088[38], 42}, {NULL}};
const Segment SGI337538[] = {{&S22088[12], 14}, {NULL}};
const Segment SGI337539[] = {{&S22090[6], 4}, {NULL}};
const Segment SGI337540[] = {{&S22090[74], 5}, {NULL}};
const Segment SGI337541[] = {{&S22090[12], 14}, {&S22090[37], 42}, {NULL}};
const Segment SGI337542[] = {{&S22090[12], 14}, {NULL}};
const Segment SGI337543[] = {{&S22091[6], 4}, {NULL}};
const Segment SGI337544[] = {{&S22091[75], 5}, {NULL}};
const Segment SGI337545[] = {{&S22091[12], 14}, {&S22091[37], 43}, {NULL}};
const Segment SGI337546[] = {{&S22091[12], 14}, {NULL}};
const Segment SGI337547[] = {{&S22092[6], 4}, {NULL}};
const Segment SGI337548[] = {{&S22092[75], 5}, {NULL}};
const Segment SGI337549[] = {{&S22092[12], 14}, {&S22092[37], 43}, {NULL}};
const Segment SGI337550[] = {{&S22092[12], 14}, {NULL}};
const Segment SGI337551[] = {{&S22093[6], 4}, {NULL}};
const Segment SGI337552[] = {{&S22093[74], 5}, {NULL}};
const Segment SGI337553[] = {{&S22093[12], 14}, {&S22093[37], 42}, {NULL}};
const Segment SGI337554[] = {{&S22093[12], 14}, {NULL}};
const Segment SGI337555[] = {{&S22094[6], 4}, {NULL}};
const Segment SGI337556[] = {{&S22094[74], 5}, {NULL}};
const Segment SGI337557[] = {{&S22094[12], 14}, {&S22094[37], 42}, {NULL}};
const Segment SGI337558[] = {{&S22094[12], 14}, {NULL}};
const Segment SGI337559[] = {{&S22095[6], 4}, {NULL}};
const Segment SGI337560[] = {{&S22095[52], 5}, {NULL}};
const Segment SGI337561[] = {{&S22095[12], 45}, {NULL}};
const Segment SGI337562[] = {{&S22095[12], 14}, {NULL}};
const Segment SGI337563[] = {{&S22097[6], 4}, {NULL}};
const Segment SGI337564[] = {{&S22097[52], 5}, {NULL}};
const Segment SGI337565[] = {{&S22097[12], 55}, {NULL}};
const Segment SGI337566[] = {{&S22097[12], 14}, {NULL}};
const Segment SGI337567[] = {{&S22098[6], 4}, {NULL}};
const Segment SGI337568[] = {{&S22098[75], 5}, {NULL}};
const Segment SGI337569[] = {{&S22098[12], 14}, {&S22098[38], 42}, {NULL}};
const Segment SGI337570[] = {{&S22098[12], 14}, {NULL}};
const Segment SGI337571[] = {{&S22099[6], 4}, {NULL}};
const Segment SGI337572[] = {{&S22099[74], 5}, {NULL}};
const Segment SGI337573[] = {{&S22099[12], 14}, {&S22099[37], 42}, {NULL}};
const Segment SGI337574[] = {{&S22099[12], 14}, {NULL}};
const Segment SGI337575[] = {{&S22100[6], 4}, {NULL}};
const Segment SGI337576[] = {{&S22100[74], 5}, {NULL}};
const Segment SGI337577[] = {{&S22100[12], 14}, {&S22100[37], 42}, {NULL}};
const Segment SGI337578[] = {{&S22100[12], 14}, {NULL}};
const Segment SGI337579[] = {{&S22101[6], 4}, {NULL}};
const Segment SGI337580[] = {{&S22101[74], 5}, {NULL}};
const Segment SGI337581[] = {{&S22101[12], 14}, {&S22101[37], 42}, {NULL}};
const Segment SGI337582[] = {{&S22101[12], 14}, {NULL}};
const Segment SGI337583[] = {{&S22102[6], 4}, {NULL}};
const Segment SGI337584[] = {{&S22102[74], 5}, {NULL}};
const Segment SGI337585[] = {{&S22102[12], 14}, {&S22102[37], 42}, {NULL}};
const Segment SGI337586[] = {{&S22102[12], 14}, {NULL}};
const Segment SGI337587[] = {{&S22103[6], 4}, {NULL}};
const Segment SGI337588[] = {{&S22103[74], 5}, {NULL}};
const Segment SGI337589[] = {{&S22103[12], 14}, {&S22103[37], 42}, {NULL}};
const Segment SGI337590[] = {{&S22103[12], 14}, {NULL}};
const Segment SGI337591[] = {{&S22104[6], 4}, {NULL}};
const Segment SGI337592[] = {{&S22104[74], 5}, {NULL}};
const Segment SGI337593[] = {{&S22104[12], 14}, {&S22104[37], 42}, {NULL}};
const Segment SGI337594[] = {{&S22104[12], 14}, {NULL}};
const Segment SGI337595[] = {{&S22105[6], 4}, {NULL}};
const Segment SGI337596[] = {{&S22105[74], 5}, {NULL}};
const Segment SGI337597[] = {{&S22105[12], 14}, {&S22105[37], 42}, {NULL}};
const Segment SGI337598[] = {{&S22105[12], 14}, {NULL}};
const Segment SGI337599[] = {{&S22106[6], 4}, {NULL}};
const Segment SGI337600[] = {{&S22106[74], 5}, {NULL}};
const Segment SGI337601[] = {{&S22106[12], 14}, {&S22106[37], 42}, {NULL}};
const Segment SGI337602[] = {{&S22106[12], 14}, {NULL}};
const Segment SGI337603[] = {{&S22107[6], 4}, {NULL}};
const Segment SGI337604[] = {{&S22107[74], 5}, {NULL}};
const Segment SGI337605[] = {{&S22107[12], 14}, {&S22107[37], 42}, {NULL}};
const Segment SGI337606[] = {{&S22107[12], 14}, {NULL}};
const Segment SGI337607[] = {{&S22108[6], 4}, {NULL}};
const Segment SGI337608[] = {{&S22108[75], 5}, {NULL}};
const Segment SGI337609[] = {{&S22108[12], 14}, {&S22108[38], 42}, {NULL}};
const Segment SGI337610[] = {{&S22108[12], 14}, {NULL}};
const Segment SGI337611[] = {{&S22109[6], 4}, {NULL}};
const Segment SGI337612[] = {{&S22109[75], 5}, {NULL}};
const Segment SGI337613[] = {{&S22109[12], 14}, {&S22109[37], 43}, {NULL}};
const Segment SGI337614[] = {{&S22109[12], 14}, {NULL}};
const Segment SGI337615[] = {{&S22110[6], 4}, {NULL}};
const Segment SGI337616[] = {{&S22110[75], 5}, {NULL}};
const Segment SGI337617[] = {{&S22110[12], 14}, {&S22110[38], 42}, {NULL}};
const Segment SGI337618[] = {{&S22110[12], 14}, {NULL}};
const Segment SGI337619[] = {{&S22111[6], 4}, {NULL}};
const Segment SGI337620[] = {{&S22111[75], 5}, {NULL}};
const Segment SGI337621[] = {{&S22111[12], 14}, {&S22111[38], 42}, {NULL}};
const Segment SGI337622[] = {{&S22111[12], 14}, {NULL}};
const Segment SGI337623[] = {{&S22112[6], 4}, {NULL}};
const Segment SGI337624[] = {{&S22112[74], 5}, {NULL}};
const Segment SGI337625[] = {{&S22112[12], 14}, {&S22112[37], 42}, {NULL}};
const Segment SGI337626[] = {{&S22112[12], 14}, {NULL}};
const Segment SGI337627[] = {{&S22113[6], 4}, {NULL}};
const Segment SGI337628[] = {{&S22113[75], 5}, {NULL}};
const Segment SGI337629[] = {{&S22113[12], 14}, {&S22113[38], 42}, {NULL}};
const Segment SGI337630[] = {{&S22113[12], 14}, {NULL}};
const Segment SGI337631[] = {{&S22114[6], 4}, {NULL}};
const Segment SGI337632[] = {{&S22114[74], 5}, {NULL}};
const Segment SGI337633[] = {{&S22114[12], 14}, {&S22114[37], 42}, {NULL}};
const Segment SGI337634[] = {{&S22114[12], 14}, {NULL}};
const Segment SGI337635[] = {{&S22115[6], 4}, {NULL}};
const Segment SGI337636[] = {{&S22115[75], 5}, {NULL}};
const Segment SGI337637[] = {{&S22115[12], 14}, {&S22115[38], 42}, {NULL}};
const Segment SGI337638[] = {{&S22115[12], 14}, {NULL}};
const Segment SGI337639[] = {{&S22116[6], 4}, {NULL}};
const Segment SGI337640[] = {{&S22116[75], 5}, {NULL}};
const Segment SGI337641[] = {{&S22116[12], 14}, {&S22116[37], 43}, {NULL}};
const Segment SGI337642[] = {{&S22116[12], 14}, {NULL}};
const Segment SGI337643[] = {{&S22117[6], 4}, {NULL}};
const Segment SGI337644[] = {{&S22117[74], 5}, {NULL}};
const Segment SGI337645[] = {{&S22117[12], 14}, {&S22117[37], 42}, {NULL}};
const Segment SGI337646[] = {{&S22117[12], 14}, {NULL}};
const Segment SGI337647[] = {{&S22118[6], 4}, {NULL}};
const Segment SGI337648[] = {{&S22118[74], 5}, {NULL}};
const Segment SGI337649[] = {{&S22118[12], 14}, {&S22118[37], 42}, {NULL}};
const Segment SGI337650[] = {{&S22118[12], 14}, {NULL}};
const Segment SGI337651[] = {{&S22119[6], 4}, {NULL}};
const Segment SGI337652[] = {{&S22119[74], 5}, {NULL}};
const Segment SGI337653[] = {{&S22119[12], 14}, {&S22119[37], 42}, {NULL}};
const Segment SGI337654[] = {{&S22119[12], 14}, {NULL}};
const Segment SGI337655[] = {{&S22120[6], 4}, {NULL}};
const Segment SGI337656[] = {{&S22120[74], 5}, {NULL}};
const Segment SGI337657[] = {{&S22120[12], 14}, {&S22120[37], 42}, {NULL}};
const Segment SGI337658[] = {{&S22120[12], 14}, {NULL}};
const Segment SGI337659[] = {{&S22121[6], 4}, {NULL}};
const Segment SGI337660[] = {{&S22121[74], 5}, {NULL}};
const Segment SGI337661[] = {{&S22121[12], 14}, {&S22121[37], 42}, {NULL}};
const Segment SGI337662[] = {{&S22121[12], 14}, {NULL}};
const Segment SGI337663[] = {{&S22122[6], 4}, {NULL}};
const Segment SGI337664[] = {{&S22122[74], 5}, {NULL}};
const Segment SGI337665[] = {{&S22122[12], 14}, {&S22122[37], 42}, {NULL}};
const Segment SGI337666[] = {{&S22122[12], 14}, {NULL}};
const Segment SGI337667[] = {{&S22123[6], 4}, {NULL}};
const Segment SGI337668[] = {{&S22123[75], 5}, {NULL}};
const Segment SGI337669[] = {{&S22123[12], 14}, {&S22123[37], 43}, {NULL}};
const Segment SGI337670[] = {{&S22123[12], 14}, {NULL}};
const Segment SGI337671[] = {{&S22124[6], 4}, {NULL}};
const Segment SGI337672[] = {{&S22124[75], 5}, {NULL}};
const Segment SGI337673[] = {{&S22124[12], 14}, {&S22124[37], 43}, {NULL}};
const Segment SGI337674[] = {{&S22124[12], 14}, {NULL}};
const Segment SGI337675[] = {{&S22125[6], 4}, {NULL}};
const Segment SGI337676[] = {{&S22125[75], 5}, {NULL}};
const Segment SGI337677[] = {{&S22125[12], 14}, {&S22125[38], 42}, {NULL}};
const Segment SGI337678[] = {{&S22125[12], 14}, {NULL}};
const Segment SGI337679[] = {{&S22126[6], 4}, {NULL}};
const Segment SGI337680[] = {{&S22126[52], 5}, {NULL}};
const Segment SGI337681[] = {{&S22126[12], 62}, {NULL}};
const Segment SGI337682[] = {{&S22126[12], 14}, {NULL}};
const Segment SGI337683[] = {{&S22127[6], 4}, {NULL}};
const Segment SGI337684[] = {{&S22127[74], 5}, {NULL}};
const Segment SGI337685[] = {{&S22127[12], 14}, {&S22127[37], 42}, {NULL}};
const Segment SGI337686[] = {{&S22127[12], 14}, {NULL}};
const Segment SGI337687[] = {{&S22128[6], 4}, {NULL}};
const Segment SGI337688[] = {{&S22128[75], 5}, {NULL}};
const Segment SGI337689[] = {{&S22128[12], 14}, {&S22128[38], 42}, {NULL}};
const Segment SGI337690[] = {{&S22128[12], 14}, {NULL}};
const Segment SGI337691[] = {{&S22129[6], 4}, {NULL}};
const Segment SGI337692[] = {{&S22129[52], 5}, {NULL}};
const Segment SGI337693[] = {{&S22129[12], 59}, {NULL}};
const Segment SGI337694[] = {{&S22129[12], 14}, {NULL}};
const Segment SGI337695[] = {{&S22130[6], 4}, {NULL}};
const Segment SGI337696[] = {{&S22130[74], 5}, {NULL}};
const Segment SGI337697[] = {{&S22130[12], 14}, {&S22130[37], 42}, {NULL}};
const Segment SGI337698[] = {{&S22130[12], 14}, {NULL}};
const Segment SGI337699[] = {{&S22131[6], 4}, {NULL}};
const Segment SGI337700[] = {{&S22131[74], 5}, {NULL}};
const Segment SGI337701[] = {{&S22131[12], 14}, {&S22131[37], 42}, {NULL}};
const Segment SGI337702[] = {{&S22131[12], 14}, {NULL}};
const Segment SGI337703[] = {{&S22132[6], 4}, {NULL}};
const Segment SGI337704[] = {{&S22132[75], 5}, {NULL}};
const Segment SGI337705[] = {{&S22132[12], 14}, {&S22132[37], 43}, {NULL}};
const Segment SGI337706[] = {{&S22132[12], 14}, {NULL}};
const Segment SGI337707[] = {{&S22133[6], 4}, {NULL}};
const Segment SGI337708[] = {{&S22133[75], 5}, {NULL}};
const Segment SGI337709[] = {{&S22133[12], 14}, {&S22133[38], 42}, {NULL}};
const Segment SGI337710[] = {{&S22133[12], 14}, {NULL}};
const Segment SGI337711[] = {{&S22134[6], 4}, {NULL}};
const Segment SGI337712[] = {{&S22134[75], 5}, {NULL}};
const Segment SGI337713[] = {{&S22134[12], 14}, {&S22134[37], 43}, {NULL}};
const Segment SGI337714[] = {{&S22134[12], 14}, {NULL}};
const Segment SGI337715[] = {{&S22135[6], 4}, {NULL}};
const Segment SGI337716[] = {{&S22135[74], 5}, {NULL}};
const Segment SGI337717[] = {{&S22135[12], 14}, {&S22135[37], 42}, {NULL}};
const Segment SGI337718[] = {{&S22135[12], 14}, {NULL}};
const Segment SGI337719[] = {{&S22136[6], 4}, {NULL}};
const Segment SGI337720[] = {{&S22136[101], 5}, {NULL}};
const Segment SGI337721[] = {{&S22136[12], 14}, {&S22136[37], 12}, {NULL}};
const Segment SGI337722[] = {{&S22136[12], 14}, {NULL}};
const Segment SGI337723[] = {{&S22137[6], 4}, {NULL}};
const Segment SGI337724[] = {{&S22137[75], 5}, {NULL}};
const Segment SGI337725[] = {{&S22137[12], 14}, {&S22137[38], 42}, {NULL}};
const Segment SGI337726[] = {{&S22137[12], 14}, {NULL}};
const Segment SGI337727[] = {{&S22138[6], 4}, {NULL}};
const Segment SGI337728[] = {{&S22138[74], 5}, {NULL}};
const Segment SGI337729[] = {{&S22138[12], 14}, {&S22138[37], 42}, {NULL}};
const Segment SGI337730[] = {{&S22138[12], 14}, {NULL}};
const Segment SGI337731[] = {{&S22139[6], 4}, {NULL}};
const Segment SGI337732[] = {{&S22139[74], 5}, {NULL}};
const Segment SGI337733[] = {{&S22139[12], 14}, {&S22139[37], 42}, {NULL}};
const Segment SGI337734[] = {{&S22139[12], 14}, {NULL}};
const Segment SGI337735[] = {{&S22140[6], 4}, {NULL}};
const Segment SGI337736[] = {{&S22140[75], 5}, {NULL}};
const Segment SGI337737[] = {{&S22140[12], 14}, {&S22140[38], 42}, {NULL}};
const Segment SGI337738[] = {{&S22140[12], 14}, {NULL}};
const Segment SGI337739[] = {{&S22141[6], 4}, {NULL}};
const Segment SGI337740[] = {{&S22141[75], 5}, {NULL}};
const Segment SGI337741[] = {{&S22141[12], 14}, {&S22141[37], 43}, {NULL}};
const Segment SGI337742[] = {{&S22141[12], 14}, {NULL}};
const Segment SGI337743[] = {{&S22142[6], 4}, {NULL}};
const Segment SGI337744[] = {{&S22142[75], 5}, {NULL}};
const Segment SGI337745[] = {{&S22142[12], 14}, {&S22142[37], 43}, {NULL}};
const Segment SGI337746[] = {{&S22142[12], 14}, {NULL}};
const Segment SGI337747[] = {{&S22143[6], 4}, {NULL}};
const Segment SGI337748[] = {{&S22143[74], 5}, {NULL}};
const Segment SGI337749[] = {{&S22143[12], 14}, {&S22143[37], 42}, {NULL}};
const Segment SGI337750[] = {{&S22143[12], 14}, {NULL}};
const Segment SGI337751[] = {{&S22144[6], 4}, {NULL}};
const Segment SGI337752[] = {{&S22144[64], 5}, {NULL}};
const Segment SGI337753[] = {{&S22144[12], 14}, {&S22144[75], 4}, {NULL}};
const Segment SGI337754[] = {{&S22144[12], 14}, {NULL}};
const Segment SGI337755[] = {{&S22145[6], 4}, {NULL}};
const Segment SGI337756[] = {{&S22145[75], 5}, {NULL}};
const Segment SGI337757[] = {{&S22145[12], 14}, {&S22145[37], 43}, {NULL}};
const Segment SGI337758[] = {{&S22145[12], 14}, {NULL}};
const Segment SGI337759[] = {{&S22146[6], 4}, {NULL}};
const Segment SGI337760[] = {{&S22146[75], 5}, {NULL}};
const Segment SGI337761[] = {{&S22146[12], 14}, {&S22146[37], 43}, {NULL}};
const Segment SGI337762[] = {{&S22146[12], 14}, {NULL}};
const Segment SGI337763[] = {{&S22147[6], 4}, {NULL}};
const Segment SGI337764[] = {{&S22147[75], 5}, {NULL}};
const Segment SGI337765[] = {{&S22147[12], 14}, {&S22147[38], 42}, {NULL}};
const Segment SGI337766[] = {{&S22147[12], 14}, {NULL}};
const Segment SGI337767[] = {{&S22148[6], 4}, {NULL}};
const Segment SGI337768[] = {{&S22148[75], 5}, {NULL}};
const Segment SGI337769[] = {{&S22148[12], 14}, {&S22148[37], 43}, {NULL}};
const Segment SGI337770[] = {{&S22148[12], 14}, {NULL}};
const Segment SGI337771[] = {{&S22149[6], 4}, {NULL}};
const Segment SGI337772[] = {{&S22149[74], 5}, {NULL}};
const Segment SGI337773[] = {{&S22149[12], 14}, {&S22149[37], 42}, {NULL}};
const Segment SGI337774[] = {{&S22149[12], 14}, {NULL}};
const Segment SGI337775[] = {{&S22150[6], 4}, {NULL}};
const Segment SGI337776[] = {{&S22150[75], 5}, {NULL}};
const Segment SGI337777[] = {{&S22150[12], 14}, {&S22150[38], 42}, {NULL}};
const Segment SGI337778[] = {{&S22150[12], 14}, {NULL}};
const Segment SGI337779[] = {{&S22151[6], 4}, {NULL}};
const Segment SGI337780[] = {{&S22151[74], 5}, {NULL}};
const Segment SGI337781[] = {{&S22151[12], 14}, {&S22151[37], 42}, {NULL}};
const Segment SGI337782[] = {{&S22151[12], 14}, {NULL}};
const Segment SGI337783[] = {{&S22152[6], 4}, {NULL}};
const Segment SGI337784[] = {{&S22152[102], 5}, {NULL}};
const Segment SGI337785[] = {{&S22152[12], 14}, {&S22152[37], 12}, {NULL}};
const Segment SGI337786[] = {{&S22152[12], 14}, {NULL}};
const Segment SGI337787[] = {{&S22153[6], 4}, {NULL}};
const Segment SGI337788[] = {{&S22153[74], 5}, {NULL}};
const Segment SGI337789[] = {{&S22153[12], 14}, {&S22153[37], 42}, {NULL}};
const Segment SGI337790[] = {{&S22153[12], 14}, {NULL}};
const Segment SGI337791[] = {{&S22154[6], 4}, {NULL}};
const Segment SGI337792[] = {{&S22154[102], 5}, {NULL}};
const Segment SGI337793[] = {{&S22154[12], 14}, {&S22154[37], 12}, {NULL}};
const Segment SGI337794[] = {{&S22154[12], 14}, {NULL}};
const Segment SGI337795[] = {{&S22155[6], 4}, {NULL}};
const Segment SGI337796[] = {{&S22155[74], 5}, {NULL}};
const Segment SGI337797[] = {{&S22155[12], 14}, {&S22155[37], 42}, {NULL}};
const Segment SGI337798[] = {{&S22155[12], 14}, {NULL}};
const Segment SGI337799[] = {{&S22156[6], 4}, {NULL}};
const Segment SGI337800[] = {{&S22156[74], 5}, {NULL}};
const Segment SGI337801[] = {{&S22156[12], 14}, {&S22156[37], 42}, {NULL}};
const Segment SGI337802[] = {{&S22156[12], 14}, {NULL}};
const Segment SGI337803[] = {{&S22157[6], 4}, {NULL}};
const Segment SGI337804[] = {{&S22157[75], 5}, {NULL}};
const Segment SGI337805[] = {{&S22157[12], 14}, {&S22157[37], 43}, {NULL}};
const Segment SGI337806[] = {{&S22157[12], 14}, {NULL}};
const Segment SGI337807[] = {{&S22158[6], 4}, {NULL}};
const Segment SGI337808[] = {{&S22158[74], 5}, {NULL}};
const Segment SGI337809[] = {{&S22158[12], 14}, {&S22158[37], 42}, {NULL}};
const Segment SGI337810[] = {{&S22158[12], 14}, {NULL}};
const Segment SGI337811[] = {{&S22159[6], 4}, {NULL}};
const Segment SGI337812[] = {{&S22159[75], 5}, {NULL}};
const Segment SGI337813[] = {{&S22159[12], 14}, {&S22159[38], 42}, {NULL}};
const Segment SGI337814[] = {{&S22159[12], 14}, {NULL}};
const Segment SGI337815[] = {{&S22160[6], 4}, {NULL}};
const Segment SGI337816[] = {{&S22160[74], 5}, {NULL}};
const Segment SGI337817[] = {{&S22160[12], 14}, {&S22160[37], 42}, {NULL}};
const Segment SGI337818[] = {{&S22160[12], 14}, {NULL}};
const Segment SGI337819[] = {{&S22161[6], 4}, {NULL}};
const Segment SGI337820[] = {{&S22161[74], 5}, {NULL}};
const Segment SGI337821[] = {{&S22161[12], 14}, {&S22161[37], 42}, {NULL}};
const Segment SGI337822[] = {{&S22161[12], 14}, {NULL}};
const Segment SGI337823[] = {{&S22162[6], 4}, {NULL}};
const Segment SGI337824[] = {{&S22162[75], 5}, {NULL}};
const Segment SGI337825[] = {{&S22162[12], 14}, {&S22162[38], 42}, {NULL}};
const Segment SGI337826[] = {{&S22162[12], 14}, {NULL}};
const Segment SGI337827[] = {{&S22163[6], 4}, {NULL}};
const Segment SGI337828[] = {{&S22163[75], 5}, {NULL}};
const Segment SGI337829[] = {{&S22163[12], 14}, {&S22163[38], 42}, {NULL}};
const Segment SGI337830[] = {{&S22163[12], 14}, {NULL}};
const Segment SGI337831[] = {{&S22164[6], 4}, {NULL}};
const Segment SGI337832[] = {{&S22164[74], 5}, {NULL}};
const Segment SGI337833[] = {{&S22164[12], 14}, {&S22164[37], 42}, {NULL}};
const Segment SGI337834[] = {{&S22164[12], 14}, {NULL}};
const Segment SGI337835[] = {{&S22165[6], 4}, {NULL}};
const Segment SGI337836[] = {{&S22165[74], 5}, {NULL}};
const Segment SGI337837[] = {{&S22165[12], 14}, {&S22165[37], 42}, {NULL}};
const Segment SGI337838[] = {{&S22165[12], 14}, {NULL}};
const Segment SGI337839[] = {{&S22166[6], 4}, {NULL}};
const Segment SGI337840[] = {{&S22166[74], 5}, {NULL}};
const Segment SGI337841[] = {{&S22166[12], 14}, {&S22166[37], 42}, {NULL}};
const Segment SGI337842[] = {{&S22166[12], 14}, {NULL}};
const Segment SGI337843[] = {{&S22167[6], 4}, {NULL}};
const Segment SGI337844[] = {{&S22167[75], 5}, {NULL}};
const Segment SGI337845[] = {{&S22167[12], 14}, {&S22167[38], 42}, {NULL}};
const Segment SGI337846[] = {{&S22167[12], 14}, {NULL}};
const Segment SGI337847[] = {{&S22168[6], 4}, {NULL}};
const Segment SGI337848[] = {{&S22168[75], 5}, {NULL}};
const Segment SGI337849[] = {{&S22168[12], 14}, {&S22168[37], 43}, {NULL}};
const Segment SGI337850[] = {{&S22168[12], 14}, {NULL}};
const Segment SGI337851[] = {{&S22169[6], 4}, {NULL}};
const Segment SGI337852[] = {{&S22169[74], 5}, {NULL}};
const Segment SGI337853[] = {{&S22169[12], 14}, {&S22169[37], 42}, {NULL}};
const Segment SGI337854[] = {{&S22169[12], 14}, {NULL}};
const Segment SGI337855[] = {{&S22170[6], 4}, {NULL}};
const Segment SGI337856[] = {{&S22170[74], 5}, {NULL}};
const Segment SGI337857[] = {{&S22170[12], 14}, {&S22170[37], 42}, {NULL}};
const Segment SGI337858[] = {{&S22170[12], 14}, {NULL}};
const Segment SGI337859[] = {{&S22171[6], 4}, {NULL}};
const Segment SGI337860[] = {{&S22171[74], 5}, {NULL}};
const Segment SGI337861[] = {{&S22171[12], 14}, {&S22171[37], 42}, {NULL}};
const Segment SGI337862[] = {{&S22171[12], 14}, {NULL}};
const Segment SGI337863[] = {{&S22172[6], 4}, {NULL}};
const Segment SGI337864[] = {{&S22172[74], 5}, {NULL}};
const Segment SGI337865[] = {{&S22172[12], 14}, {&S22172[37], 42}, {NULL}};
const Segment SGI337866[] = {{&S22172[12], 14}, {NULL}};
const Segment SGI337867[] = {{&S22173[6], 4}, {NULL}};
const Segment SGI337868[] = {{&S22173[74], 5}, {NULL}};
const Segment SGI337869[] = {{&S22173[12], 14}, {&S22173[37], 42}, {NULL}};
const Segment SGI337870[] = {{&S22173[12], 14}, {NULL}};
const Segment SGI337871[] = {{&S22174[6], 4}, {NULL}};
const Segment SGI337872[] = {{&S22174[75], 5}, {NULL}};
const Segment SGI337873[] = {{&S22174[12], 14}, {&S22174[38], 42}, {NULL}};
const Segment SGI337874[] = {{&S22174[12], 14}, {NULL}};
const Segment SGI337875[] = {{&S22175[6], 4}, {NULL}};
const Segment SGI337876[] = {{&S22175[74], 5}, {NULL}};
const Segment SGI337877[] = {{&S22175[12], 14}, {&S22175[37], 42}, {NULL}};
const Segment SGI337878[] = {{&S22175[12], 14}, {NULL}};
const Segment SGI337879[] = {{&S22176[6], 4}, {NULL}};
const Segment SGI337880[] = {{&S22176[74], 5}, {NULL}};
const Segment SGI337881[] = {{&S22176[12], 14}, {&S22176[37], 42}, {NULL}};
const Segment SGI337882[] = {{&S22176[12], 14}, {NULL}};
const Segment SGI337883[] = {{&S22177[6], 4}, {NULL}};
const Segment SGI337884[] = {{&S22177[74], 5}, {NULL}};
const Segment SGI337885[] = {{&S22177[12], 14}, {&S22177[37], 42}, {NULL}};
const Segment SGI337886[] = {{&S22177[12], 14}, {NULL}};
const Segment SGI337887[] = {{&S22178[6], 4}, {NULL}};
const Segment SGI337888[] = {{&S22178[75], 5}, {NULL}};
const Segment SGI337889[] = {{&S22178[12], 14}, {&S22178[37], 43}, {NULL}};
const Segment SGI337890[] = {{&S22178[12], 14}, {NULL}};
const Segment SGI337891[] = {{&S22179[6], 4}, {NULL}};
const Segment SGI337892[] = {{&S22179[75], 5}, {NULL}};
const Segment SGI337893[] = {{&S22179[12], 14}, {&S22179[37], 43}, {NULL}};
const Segment SGI337894[] = {{&S22179[12], 14}, {NULL}};
const Segment SGI337895[] = {{&S22180[6], 4}, {NULL}};
const Segment SGI337896[] = {{&S22180[52], 5}, {NULL}};
const Segment SGI337897[] = {{&S22180[12], 84}, {NULL}};
const Segment SGI337898[] = {{&S22180[12], 14}, {NULL}};
const Segment SGI337899[] = {{&S22181[6], 4}, {NULL}};
const Segment SGI337900[] = {{&S22181[75], 5}, {NULL}};
const Segment SGI337901[] = {{&S22181[12], 14}, {&S22181[37], 43}, {NULL}};
const Segment SGI337902[] = {{&S22181[12], 14}, {NULL}};
const Segment SGI337903[] = {{&S22182[6], 4}, {NULL}};
const Segment SGI337904[] = {{&S22182[74], 5}, {NULL}};
const Segment SGI337905[] = {{&S22182[12], 14}, {&S22182[37], 42}, {NULL}};
const Segment SGI337906[] = {{&S22182[12], 14}, {NULL}};
const Segment SGI337907[] = {{&S22183[6], 4}, {NULL}};
const Segment SGI337908[] = {{&S22183[74], 5}, {NULL}};
const Segment SGI337909[] = {{&S22183[12], 14}, {&S22183[37], 42}, {NULL}};
const Segment SGI337910[] = {{&S22183[12], 14}, {NULL}};
const Segment SGI337911[] = {{&S22184[6], 4}, {NULL}};
const Segment SGI337912[] = {{&S22184[74], 5}, {NULL}};
const Segment SGI337913[] = {{&S22184[12], 14}, {&S22184[37], 42}, {NULL}};
const Segment SGI337914[] = {{&S22184[12], 14}, {NULL}};
const Segment SGI337915[] = {{&S22185[6], 4}, {NULL}};
const Segment SGI337916[] = {{&S22185[74], 5}, {NULL}};
const Segment SGI337917[] = {{&S22185[12], 14}, {&S22185[37], 42}, {NULL}};
const Segment SGI337918[] = {{&S22185[12], 14}, {NULL}};
const Segment SGI337919[] = {{&S22186[6], 4}, {NULL}};
const Segment SGI337920[] = {{&S22186[74], 5}, {NULL}};
const Segment SGI337921[] = {{&S22186[12], 14}, {&S22186[37], 42}, {NULL}};
const Segment SGI337922[] = {{&S22186[12], 14}, {NULL}};
const Segment SGI337923[] = {{&S22187[6], 4}, {NULL}};
const Segment SGI337924[] = {{&S22187[74], 5}, {NULL}};
const Segment SGI337925[] = {{&S22187[12], 14}, {&S22187[37], 42}, {NULL}};
const Segment SGI337926[] = {{&S22187[12], 14}, {NULL}};
const Segment SGI337927[] = {{&S22188[6], 4}, {NULL}};
const Segment SGI337928[] = {{&S22188[74], 5}, {NULL}};
const Segment SGI337929[] = {{&S22188[12], 14}, {&S22188[37], 42}, {NULL}};
const Segment SGI337930[] = {{&S22188[12], 14}, {NULL}};
const Segment SGI337931[] = {{&S22189[6], 4}, {NULL}};
const Segment SGI337932[] = {{&S22189[74], 5}, {NULL}};
const Segment SGI337933[] = {{&S22189[12], 14}, {&S22189[37], 42}, {NULL}};
const Segment SGI337934[] = {{&S22189[12], 14}, {NULL}};
const Segment SGI337935[] = {{&S22190[6], 4}, {NULL}};
const Segment SGI337936[] = {{&S22190[75], 5}, {NULL}};
const Segment SGI337937[] = {{&S22190[12], 14}, {&S22190[37], 43}, {NULL}};
const Segment SGI337938[] = {{&S22190[12], 14}, {NULL}};
const Segment SGI337939[] = {{&S48272[5], 16}, {NULL}};
const Segment SGI337940[] = {{&S48272[27], 3}, {NULL}};
const Segment SGI337941[] = {{&S22191[6], 4}, {NULL}};
const Segment SGI337942[] = {{&S22191[75], 5}, {NULL}};
const Segment SGI337943[] = {{&S22191[12], 14}, {&S22191[37], 43}, {NULL}};
const Segment SGI337944[] = {{&S22191[12], 14}, {NULL}};
const Segment SGI337945[] = {{&S48273[35], 16}, {NULL}};
const Segment SGI337946[] = {{&S48273[102], 3}, {NULL}};
const Segment SGI337947[] = {{&S22192[6], 4}, {NULL}};
const Segment SGI337948[] = {{&S22192[75], 5}, {NULL}};
const Segment SGI337949[] = {{&S22192[12], 14}, {&S22192[37], 43}, {NULL}};
const Segment SGI337950[] = {{&S22192[12], 14}, {NULL}};
const Segment SGI337951[] = {{&S48274[5], 25}, {NULL}};
const Segment SGI337952[] = {{&S48274[31], 3}, {NULL}};
const Segment SGI337953[] = {{&S22193[6], 4}, {NULL}};
const Segment SGI337954[] = {{&S22193[74], 5}, {NULL}};
const Segment SGI337955[] = {{&S22193[12], 14}, {&S22193[37], 42}, {NULL}};
const Segment SGI337956[] = {{&S22193[12], 14}, {NULL}};
const Segment SGI337957[] = {{&S48275[41], 7}, {NULL}};
const Segment SGI337958[] = {{&S48275[6], 3}, {NULL}};
const Segment SGI337959[] = {{&S48275[69], 3}, {NULL}};
const Segment SGI337960[] = {{&S22194[6], 4}, {NULL}};
const Segment SGI337961[] = {{&S22194[74], 5}, {NULL}};
const Segment SGI337962[] = {{&S22194[12], 14}, {&S22194[37], 42}, {NULL}};
const Segment SGI337963[] = {{&S22194[12], 14}, {NULL}};
const Segment SGI337964[] = {{&S22195[6], 4}, {NULL}};
const Segment SGI337965[] = {{&S22195[74], 5}, {NULL}};
const Segment SGI337966[] = {{&S22195[12], 14}, {&S22195[37], 42}, {NULL}};
const Segment SGI337967[] = {{&S22195[12], 14}, {NULL}};
const Segment SGI337968[] = {{&S22196[6], 4}, {NULL}};
const Segment SGI337969[] = {{&S22196[74], 5}, {NULL}};
const Segment SGI337970[] = {{&S22196[12], 14}, {&S22196[37], 42}, {NULL}};
const Segment SGI337971[] = {{&S22196[12], 14}, {NULL}};
const Segment SGI337972[] = {{&S22197[6], 4}, {NULL}};
const Segment SGI337973[] = {{&S22197[74], 5}, {NULL}};
const Segment SGI337974[] = {{&S22197[12], 14}, {&S22197[37], 42}, {NULL}};
const Segment SGI337975[] = {{&S22197[12], 14}, {NULL}};
const Segment SGI337976[] = {{&S22198[6], 4}, {NULL}};
const Segment SGI337977[] = {{&S22198[74], 5}, {NULL}};
const Segment SGI337978[] = {{&S22198[12], 14}, {&S22198[37], 42}, {NULL}};
const Segment SGI337979[] = {{&S22198[12], 14}, {NULL}};
const Segment SGI337980[] = {{&S22199[6], 4}, {NULL}};
const Segment SGI337981[] = {{&S22199[74], 5}, {NULL}};
const Segment SGI337982[] = {{&S22199[12], 14}, {&S22199[37], 42}, {NULL}};
const Segment SGI337983[] = {{&S22199[12], 14}, {NULL}};
const Segment SGI337984[] = {{&S22200[6], 4}, {NULL}};
const Segment SGI337985[] = {{&S22200[74], 5}, {NULL}};
const Segment SGI337986[] = {{&S22200[12], 14}, {&S22200[37], 42}, {NULL}};
const Segment SGI337987[] = {{&S22200[12], 14}, {NULL}};
const Segment SGI337988[] = {{&S22201[6], 4}, {NULL}};
const Segment SGI337989[] = {{&S22201[74], 5}, {NULL}};
const Segment SGI337990[] = {{&S22201[12], 14}, {&S22201[37], 42}, {NULL}};
const Segment SGI337991[] = {{&S22201[12], 14}, {NULL}};
const Segment SGI337992[] = {{&S22202[6], 4}, {NULL}};
const Segment SGI337993[] = {{&S22202[74], 5}, {NULL}};
const Segment SGI337994[] = {{&S22202[12], 14}, {&S22202[37], 42}, {NULL}};
const Segment SGI337995[] = {{&S22202[12], 14}, {NULL}};
const Segment SGI337996[] = {{&S22203[6], 4}, {NULL}};
const Segment SGI337997[] = {{&S22203[57], 5}, {NULL}};
const Segment SGI337998[] = {{&S22203[12], 14}, {NULL}};
const Segment SGI337999[] = {{&S22204[6], 4}, {NULL}};
const Segment SGI338000[] = {{&S22204[57], 5}, {NULL}};
const Segment SGI338001[] = {{&S22204[12], 14}, {NULL}};
const Segment SGI338002[] = {{&S22205[6], 4}, {NULL}};
const Segment SGI338003[] = {{&S22205[57], 5}, {NULL}};
const Segment SGI338004[] = {{&S22205[12], 14}, {NULL}};
const Segment SGI338005[] = {{&S22206[6], 4}, {NULL}};
const Segment SGI338006[] = {{&S22206[80], 5}, {NULL}};
const Segment SGI338007[] = {{&S22206[12], 14}, {&S22206[40], 12}, {NULL}};
const Segment SGI338008[] = {{&S22206[12], 14}, {NULL}};
const Segment SGI338009[] = {{&S22207[6], 4}, {NULL}};
const Segment SGI338010[] = {{&S22207[80], 5}, {NULL}};
const Segment SGI338011[] = {{&S22207[12], 14}, {&S22207[40], 12}, {NULL}};
const Segment SGI338012[] = {{&S22207[12], 14}, {NULL}};
const Segment SGI338013[] = {{&S22208[6], 4}, {NULL}};
const Segment SGI338014[] = {{&S22208[80], 5}, {NULL}};
const Segment SGI338015[] = {{&S22208[12], 14}, {&S22208[40], 12}, {NULL}};
const Segment SGI338016[] = {{&S22208[12], 14}, {NULL}};
const Segment SGI338017[] = {{&S22209[6], 4}, {NULL}};
const Segment SGI338018[] = {{&S22209[80], 5}, {NULL}};
const Segment SGI338019[] = {{&S22209[12], 14}, {&S22209[40], 12}, {NULL}};
const Segment SGI338020[] = {{&S22209[12], 14}, {NULL}};
const Segment SGI338021[] = {{&S22210[6], 4}, {NULL}};
const Segment SGI338022[] = {{&S22210[80], 5}, {NULL}};
const Segment SGI338023[] = {{&S22210[12], 14}, {&S22210[40], 12}, {NULL}};
const Segment SGI338024[] = {{&S22210[12], 14}, {NULL}};
const Segment SGI338025[] = {{&S22211[6], 4}, {NULL}};
const Segment SGI338026[] = {{&S22211[67], 5}, {NULL}};
const Segment SGI338027[] = {{&S22211[12], 14}, {NULL}};
const Segment SGI338028[] = {{&S22212[6], 4}, {NULL}};
const Segment SGI338029[] = {{&S22212[80], 5}, {NULL}};
const Segment SGI338030[] = {{&S22212[12], 14}, {&S22212[40], 12}, {NULL}};
const Segment SGI338031[] = {{&S22212[12], 14}, {NULL}};
const Segment SGI338032[] = {{&S22213[6], 4}, {NULL}};
const Segment SGI338033[] = {{&S22213[80], 5}, {NULL}};
const Segment SGI338034[] = {{&S22213[12], 14}, {&S22213[40], 12}, {NULL}};
const Segment SGI338035[] = {{&S22213[12], 14}, {NULL}};
const Segment SGI338036[] = {{&S22214[6], 4}, {NULL}};
const Segment SGI338037[] = {{&S22214[80], 5}, {NULL}};
const Segment SGI338038[] = {{&S22214[12], 14}, {&S22214[40], 12}, {NULL}};
const Segment SGI338039[] = {{&S22214[12], 14}, {NULL}};
const Segment SGI338040[] = {{&S22215[6], 4}, {NULL}};
const Segment SGI338041[] = {{&S22215[80], 5}, {NULL}};
const Segment SGI338042[] = {{&S22215[12], 14}, {&S22215[40], 12}, {NULL}};
const Segment SGI338043[] = {{&S22215[12], 14}, {NULL}};
const Segment SGI338044[] = {{&S22216[6], 4}, {NULL}};
const Segment SGI338045[] = {{&S22216[80], 5}, {NULL}};
const Segment SGI338046[] = {{&S22216[12], 14}, {&S22216[40], 12}, {NULL}};
const Segment SGI338047[] = {{&S22216[12], 14}, {NULL}};
const Segment SGI338048[] = {{&S22217[6], 4}, {NULL}};
const Segment SGI338049[] = {{&S22217[80], 5}, {NULL}};
const Segment SGI338050[] = {{&S22217[12], 14}, {&S22217[40], 12}, {NULL}};
const Segment SGI338051[] = {{&S22217[12], 14}, {NULL}};
const Segment SGI338052[] = {{&S22218[6], 4}, {NULL}};
const Segment SGI338053[] = {{&S22218[57], 5}, {NULL}};
const Segment SGI338054[] = {{&S22218[12], 14}, {&S22218[68], 31}, {NULL}};
const Segment SGI338055[] = {{&S22218[12], 14}, {NULL}};
const Segment SGI338056[] = {{&S22219[6], 4}, {NULL}};
const Segment SGI338057[] = {{&S22219[57], 5}, {NULL}};
const Segment SGI338058[] = {{&S22219[12], 14}, {&S22219[68], 31}, {NULL}};
const Segment SGI338059[] = {{&S22219[12], 14}, {NULL}};
const Segment SGI338060[] = {{&S22220[6], 4}, {NULL}};
const Segment SGI338061[] = {{&S22220[80], 5}, {NULL}};
const Segment SGI338062[] = {{&S22220[12], 14}, {&S22220[40], 12}, {NULL}};
const Segment SGI338063[] = {{&S22220[12], 14}, {NULL}};
const Segment SGI338064[] = {{&S22221[6], 4}, {NULL}};
const Segment SGI338065[] = {{&S22221[80], 5}, {NULL}};
const Segment SGI338066[] = {{&S22221[12], 14}, {&S22221[40], 12}, {NULL}};
const Segment SGI338067[] = {{&S22221[12], 14}, {NULL}};
const Segment SGI338068[] = {{&S22222[6], 4}, {NULL}};
const Segment SGI338069[] = {{&S22222[80], 5}, {NULL}};
const Segment SGI338070[] = {{&S22222[12], 14}, {&S22222[40], 12}, {NULL}};
const Segment SGI338071[] = {{&S22222[12], 14}, {NULL}};
const Segment SGI338072[] = {{&S22223[6], 4}, {NULL}};
const Segment SGI338073[] = {{&S22223[80], 5}, {NULL}};
const Segment SGI338074[] = {{&S22223[12], 14}, {&S22223[40], 12}, {NULL}};
const Segment SGI338075[] = {{&S22223[12], 14}, {NULL}};
const Segment SGI338076[] = {{&S22224[6], 4}, {NULL}};
const Segment SGI338077[] = {{&S22224[80], 5}, {NULL}};
const Segment SGI338078[] = {{&S22224[12], 14}, {&S22224[40], 12}, {NULL}};
const Segment SGI338079[] = {{&S22224[12], 14}, {NULL}};
const Segment SGI338080[] = {{&S22225[6], 4}, {NULL}};
const Segment SGI338081[] = {{&S22225[79], 5}, {NULL}};
const Segment SGI338082[] = {{&S22225[12], 14}, {&S22225[40], 11}, {NULL}};
const Segment SGI338083[] = {{&S22225[12], 14}, {NULL}};
const Segment SGI338084[] = {{&S22226[6], 4}, {NULL}};
const Segment SGI338085[] = {{&S22226[80], 5}, {NULL}};
const Segment SGI338086[] = {{&S22226[12], 14}, {&S22226[41], 11}, {NULL}};
const Segment SGI338087[] = {{&S22226[12], 14}, {NULL}};
const Segment SGI338088[] = {{&S22227[6], 4}, {NULL}};
const Segment SGI338089[] = {{&S22227[57], 5}, {NULL}};
const Segment SGI338090[] = {{&S22227[12], 14}, {&S22227[68], 31}, {NULL}};
const Segment SGI338091[] = {{&S22227[12], 14}, {NULL}};
const Segment SGI338092[] = {{&S22228[6], 4}, {NULL}};
const Segment SGI338093[] = {{&S22228[80], 5}, {NULL}};
const Segment SGI338094[] = {{&S22228[12], 14}, {&S22228[40], 12}, {NULL}};
const Segment SGI338095[] = {{&S22228[12], 14}, {NULL}};
const Segment SGI338096[] = {{&S22229[6], 4}, {NULL}};
const Segment SGI338097[] = {{&S22229[79], 5}, {NULL}};
const Segment SGI338098[] = {{&S22229[12], 14}, {&S22229[40], 11}, {NULL}};
const Segment SGI338099[] = {{&S22229[12], 14}, {NULL}};
const Segment SGI338100[] = {{&S22230[6], 4}, {NULL}};
const Segment SGI338101[] = {{&S22230[80], 5}, {NULL}};
const Segment SGI338102[] = {{&S22230[12], 14}, {&S22230[40], 12}, {NULL}};
const Segment SGI338103[] = {{&S22230[12], 14}, {NULL}};
const Segment SGI338104[] = {{&S24192[61], 4}, {NULL}};
const Segment SGI338105[] = {{&S24192[13], 10}, {NULL}};
const Segment SGI338106[] = {{&S22231[6], 4}, {NULL}};
const Segment SGI338107[] = {{&S22231[80], 5}, {NULL}};
const Segment SGI338108[] = {{&S22231[12], 14}, {&S22231[40], 12}, {NULL}};
const Segment SGI338109[] = {{&S22231[12], 14}, {NULL}};
const Segment SGI338110[] = {{&S22232[6], 4}, {NULL}};
const Segment SGI338111[] = {{&S22232[79], 5}, {NULL}};
const Segment SGI338112[] = {{&S22232[12], 14}, {&S22232[40], 11}, {NULL}};
const Segment SGI338113[] = {{&S22232[12], 14}, {NULL}};
const Segment SGI338114[] = {{&S22233[6], 4}, {NULL}};
const Segment SGI338115[] = {{&S22233[79], 5}, {NULL}};
const Segment SGI338116[] = {{&S22233[12], 14}, {&S22233[40], 11}, {NULL}};
const Segment SGI338117[] = {{&S22233[12], 14}, {NULL}};
const Segment SGI338118[] = {{&S22234[6], 4}, {NULL}};
const Segment SGI338119[] = {{&S22234[79], 5}, {NULL}};
const Segment SGI338120[] = {{&S22234[12], 14}, {&S22234[40], 11}, {NULL}};
const Segment SGI338121[] = {{&S22234[12], 14}, {NULL}};
const Segment SGI338122[] = {{&S22235[6], 4}, {NULL}};
const Segment SGI338123[] = {{&S22235[79], 5}, {NULL}};
const Segment SGI338124[] = {{&S22235[12], 14}, {&S22235[40], 11}, {NULL}};
const Segment SGI338125[] = {{&S22235[12], 14}, {NULL}};
const Segment SGI338126[] = {{&S22236[6], 4}, {NULL}};
const Segment SGI338127[] = {{&S22236[79], 5}, {NULL}};
const Segment SGI338128[] = {{&S22236[12], 14}, {&S22236[40], 11}, {NULL}};
const Segment SGI338129[] = {{&S22236[12], 14}, {NULL}};
const Segment SGI338130[] = {{&S22237[6], 4}, {NULL}};
const Segment SGI338131[] = {{&S22237[79], 5}, {NULL}};
const Segment SGI338132[] = {{&S22237[12], 14}, {&S22237[40], 11}, {NULL}};
const Segment SGI338133[] = {{&S22237[12], 14}, {NULL}};
const Segment SGI338134[] = {{&S22238[6], 5}, {NULL}};
const Segment SGI338135[] = {{&S22238[13], 14}, {NULL}};
const Segment SGI338136[] = {{&S24206[61], 4}, {NULL}};
const Segment SGI338137[] = {{&S24206[13], 10}, {NULL}};
const Segment SGI338138[] = {{&S22239[6], 4}, {NULL}};
const Segment SGI338139[] = {{&S22239[79], 5}, {NULL}};
const Segment SGI338140[] = {{&S22239[12], 14}, {&S22239[40], 11}, {NULL}};
const Segment SGI338141[] = {{&S22239[12], 14}, {NULL}};
const Segment SGI338142[] = {{&S22240[6], 4}, {NULL}};
const Segment SGI338143[] = {{&S22240[79], 5}, {NULL}};
const Segment SGI338144[] = {{&S22240[12], 14}, {&S22240[40], 11}, {NULL}};
const Segment SGI338145[] = {{&S22240[12], 14}, {NULL}};
const Segment SGI338146[] = {{&S24208[61], 4}, {NULL}};
const Segment SGI338147[] = {{&S24208[13], 10}, {NULL}};
const Segment SGI338148[] = {{&S22241[6], 4}, {NULL}};
const Segment SGI338149[] = {{&S22241[57], 5}, {NULL}};
const Segment SGI338150[] = {{&S22241[12], 14}, {&S22241[68], 31}, {NULL}};
const Segment SGI338151[] = {{&S22241[12], 14}, {NULL}};
const Segment SGI338152[] = {{&S24210[61], 4}, {NULL}};
const Segment SGI338153[] = {{&S24210[13], 10}, {NULL}};
const Segment SGI338154[] = {{&S22242[6], 4}, {NULL}};
const Segment SGI338155[] = {{&S22242[80], 5}, {NULL}};
const Segment SGI338156[] = {{&S22242[12], 14}, {&S22242[41], 11}, {NULL}};
const Segment SGI338157[] = {{&S22242[12], 14}, {NULL}};
const Segment SGI338158[] = {{&S22243[6], 4}, {NULL}};
const Segment SGI338159[] = {{&S22243[80], 5}, {NULL}};
const Segment SGI338160[] = {{&S22243[12], 14}, {&S22243[40], 12}, {NULL}};
const Segment SGI338161[] = {{&S22243[12], 14}, {NULL}};
const Segment SGI338162[] = {{&S22244[6], 4}, {NULL}};
const Segment SGI338163[] = {{&S22244[79], 5}, {NULL}};
const Segment SGI338164[] = {{&S22244[12], 14}, {&S22244[40], 11}, {NULL}};
const Segment SGI338165[] = {{&S22244[12], 14}, {NULL}};
const Segment SGI338166[] = {{&S22245[6], 4}, {NULL}};
const Segment SGI338167[] = {{&S22245[79], 5}, {NULL}};
const Segment SGI338168[] = {{&S22245[12], 14}, {&S22245[40], 11}, {NULL}};
const Segment SGI338169[] = {{&S22245[12], 14}, {NULL}};
const Segment SGI338170[] = {{&S22246[6], 4}, {NULL}};
const Segment SGI338171[] = {{&S22246[79], 5}, {NULL}};
const Segment SGI338172[] = {{&S22246[12], 14}, {&S22246[40], 11}, {NULL}};
const Segment SGI338173[] = {{&S22246[12], 14}, {NULL}};
const Segment SGI338174[] = {{&S24217[61], 4}, {NULL}};
const Segment SGI338175[] = {{&S24217[13], 10}, {NULL}};
const Segment SGI338176[] = {{&S22247[6], 4}, {NULL}};
const Segment SGI338177[] = {{&S22247[79], 5}, {NULL}};
const Segment SGI338178[] = {{&S22247[12], 14}, {&S22247[40], 11}, {NULL}};
const Segment SGI338179[] = {{&S22247[12], 14}, {NULL}};
const Segment SGI338180[] = {{&S22248[6], 4}, {NULL}};
const Segment SGI338181[] = {{&S24219[82], 4}, {NULL}};
const Segment SGI338182[] = {{&S22248[80], 5}, {NULL}};
const Segment SGI338183[] = {{&S24219[13], 10}, {NULL}};
const Segment SGI338184[] = {{&S22248[12], 14}, {&S22248[41], 11}, {NULL}};
const Segment SGI338185[] = {{&S22248[12], 14}, {NULL}};
const Segment SGI338186[] = {{&S22249[6], 4}, {NULL}};
const Segment SGI338187[] = {{&S22249[57], 5}, {NULL}};
const Segment SGI338188[] = {{&S22249[12], 14}, {&S22249[89], 4}, {NULL}};
const Segment SGI338189[] = {{&S22249[12], 14}, {NULL}};
const Segment SGI338190[] = {{&S22250[6], 4}, {NULL}};
const Segment SGI338191[] = {{&S22250[79], 5}, {NULL}};
const Segment SGI338192[] = {{&S22250[12], 14}, {&S22250[40], 11}, {NULL}};
const Segment SGI338193[] = {{&S22250[12], 14}, {NULL}};
const Segment SGI338194[] = {{&S22251[6], 4}, {NULL}};
const Segment SGI338195[] = {{&S22251[79], 5}, {NULL}};
const Segment SGI338196[] = {{&S22251[12], 14}, {&S22251[40], 11}, {NULL}};
const Segment SGI338197[] = {{&S22251[12], 14}, {NULL}};
const Segment SGI338198[] = {{&S22252[6], 4}, {NULL}};
const Segment SGI338199[] = {{&S22252[79], 5}, {NULL}};
const Segment SGI338200[] = {{&S22252[12], 14}, {&S22252[40], 11}, {NULL}};
const Segment SGI338201[] = {{&S22252[12], 14}, {NULL}};
const Segment SGI338202[] = {{&S22253[6], 4}, {NULL}};
const Segment SGI338203[] = {{&S22253[57], 5}, {NULL}};
const Segment SGI338204[] = {{&S22253[12], 14}, {&S22253[68], 4}, {NULL}};
const Segment SGI338205[] = {{&S22253[12], 14}, {NULL}};
const Segment SGI338206[] = {{&S22254[6], 4}, {NULL}};
const Segment SGI338207[] = {{&S22254[80], 5}, {NULL}};
const Segment SGI338208[] = {{&S22254[12], 14}, {&S22254[41], 11}, {NULL}};
const Segment SGI338209[] = {{&S22254[12], 14}, {NULL}};
const Segment SGI338210[] = {{&S22255[6], 4}, {NULL}};
const Segment SGI338211[] = {{&S22255[79], 5}, {NULL}};
const Segment SGI338212[] = {{&S22255[12], 14}, {&S22255[40], 11}, {NULL}};
const Segment SGI338213[] = {{&S22255[12], 14}, {NULL}};
const Segment SGI338214[] = {{&S22256[6], 4}, {NULL}};
const Segment SGI338215[] = {{&S22256[79], 5}, {NULL}};
const Segment SGI338216[] = {{&S22256[12], 14}, {&S22256[40], 11}, {NULL}};
const Segment SGI338217[] = {{&S22256[12], 14}, {NULL}};
const Segment SGI338218[] = {{&S22257[6], 4}, {NULL}};
const Segment SGI338219[] = {{&S22257[79], 5}, {NULL}};
const Segment SGI338220[] = {{&S22257[12], 14}, {&S22257[40], 11}, {NULL}};
const Segment SGI338221[] = {{&S22257[12], 14}, {NULL}};
const Segment SGI338222[] = {{&S22258[6], 4}, {NULL}};
const Segment SGI338223[] = {{&S22258[79], 5}, {NULL}};
const Segment SGI338224[] = {{&S22258[12], 14}, {&S22258[40], 11}, {NULL}};
const Segment SGI338225[] = {{&S22258[12], 14}, {NULL}};
const Segment SGI338226[] = {{&S22259[6], 4}, {NULL}};
const Segment SGI338227[] = {{&S22259[79], 5}, {NULL}};
const Segment SGI338228[] = {{&S22259[12], 14}, {&S22259[40], 11}, {NULL}};
const Segment SGI338229[] = {{&S22259[12], 14}, {NULL}};
const Segment SGI338230[] = {{&S22260[6], 4}, {NULL}};
const Segment SGI338231[] = {{&S22260[79], 5}, {NULL}};
const Segment SGI338232[] = {{&S22260[12], 14}, {&S22260[40], 11}, {NULL}};
const Segment SGI338233[] = {{&S22260[12], 14}, {NULL}};
const Segment SGI338234[] = {{&S22261[6], 4}, {NULL}};
const Segment SGI338235[] = {{&S22261[79], 5}, {NULL}};
const Segment SGI338236[] = {{&S22261[12], 14}, {&S22261[40], 11}, {NULL}};
const Segment SGI338237[] = {{&S22261[12], 14}, {NULL}};
const Segment SGI338238[] = {{&S48331[5], 7}, {&S48331[5], 7}, {NULL}};
const Segment SGI338239[] = {{&S48332[5], 7}, {NULL}};
const Segment SGI338240[] = {{&S48333[5], 5}, {NULL}};
const Segment SGI338241[] = {{&S48334[5], 8}, {NULL}};
const Segment SGI338242[] = {{&S39586[28], 5}, {NULL}};
const Segment SGI338243[] = {{&S39586[104], 5}, {NULL}};
const Segment SGI338244[] = {{&S39587[28], 5}, {NULL}};
const Segment SGI338245[] = {{&S39587[104], 5}, {NULL}};
const Segment SGI338246[] = {{&S48542[5], 3}, {NULL}};
const Segment SGI338247[] = {{&S48549[35], 7}, {NULL}};
const Segment SGI338248[] = {{&S48549[93], 3}, {NULL}};
const Segment SGI338249[] = {{&S48550[35], 5}, {NULL}};
const Segment SGI338250[] = {{&S48550[91], 3}, {NULL}};
const Segment SGI338251[] = {{&S48551[41], 5}, {NULL}};
const Segment SGI338252[] = {{&S48551[6], 3}, {NULL}};
const Segment SGI338253[] = {{&S48551[67], 3}, {NULL}};
const Segment SGI338254[] = {{&S48552[41], 7}, {NULL}};
const Segment SGI338255[] = {{&S48552[6], 3}, {NULL}};
const Segment SGI338256[] = {{&S48552[69], 3}, {NULL}};
const Segment SGI338257[] = {{&S22498[6], 4}, {NULL}};
const Segment SGI338258[] = {{&S22498[11], 14}, {NULL}};
const Segment SGI338259[] = {{&S22499[6], 4}, {NULL}};
const Segment SGI338260[] = {{&S22499[12], 14}, {NULL}};
const Segment SGI338261[] = {{&S22500[6], 4}, {NULL}};
const Segment SGI338262[] = {{&S22500[12], 14}, {NULL}};
const Segment SGI338263[] = {{&S22501[6], 4}, {NULL}};
const Segment SGI338264[] = {{&S22501[12], 14}, {NULL}};
const Segment SGI338265[] = {{&S22502[6], 4}, {NULL}};
const Segment SGI338266[] = {{&S22502[12], 14}, {&S22502[40], 12}, {NULL}};
const Segment SGI338267[] = {{&S22502[12], 14}, {NULL}};
const Segment SGI338268[] = {{&S22503[6], 4}, {NULL}};
const Segment SGI338269[] = {{&S22503[12], 14}, {NULL}};
const Segment SGI338270[] = {{&S22504[6], 4}, {NULL}};
const Segment SGI338271[] = {{&S22504[12], 14}, {NULL}};
const Segment SGI338272[] = {{&S22505[6], 4}, {NULL}};
const Segment SGI338273[] = {{&S22505[12], 14}, {NULL}};
const Segment SGI338274[] = {{&S48582[35], 5}, {NULL}};
const Segment SGI338275[] = {{&S22506[6], 4}, {NULL}};
const Segment SGI338276[] = {{&S22506[12], 14}, {&S22506[40], 12}, {NULL}};
const Segment SGI338277[] = {{&S48582[91], 3}, {NULL}};
const Segment SGI338278[] = {{&S22506[12], 14}, {NULL}};
const Segment SGI338279[] = {{&S48583[41], 5}, {NULL}};
const Segment SGI338280[] = {{&S22507[6], 4}, {NULL}};
const Segment SGI338281[] = {{&S22507[12], 14}, {&S22507[40], 12}, {NULL}};
const Segment SGI338282[] = {{&S48583[6], 3}, {NULL}};
const Segment SGI338283[] = {{&S22507[12], 14}, {NULL}};
const Segment SGI338284[] = {{&S48583[67], 3}, {NULL}};
const Segment SGI338285[] = {{&S22508[6], 4}, {NULL}};
const Segment SGI338286[] = {{&S22508[12], 14}, {NULL}};
const Segment SGI338287[] = {{&S48584[5], 5}, {NULL}};
const Segment SGI338288[] = {{&S48584[16], 3}, {NULL}};
const Segment SGI338289[] = {{&S22509[6], 4}, {NULL}};
const Segment SGI338290[] = {{&S22509[12], 14}, {&S22509[40], 12}, {NULL}};
const Segment SGI338291[] = {{&S48585[5], 14}, {NULL}};
const Segment SGI338292[] = {{&S22509[12], 14}, {NULL}};
const Segment SGI338293[] = {{&S48585[20], 3}, {NULL}};
const Segment SGI338294[] = {{&S22510[6], 4}, {NULL}};
const Segment SGI338295[] = {{&S48586[35], 7}, {NULL}};
const Segment SGI338296[] = {{&S22510[12], 14}, {&S22510[40], 12}, {NULL}};
const Segment SGI338297[] = {{&S22510[12], 14}, {NULL}};
const Segment SGI338298[] = {{&S48586[93], 3}, {NULL}};
const Segment SGI338299[] = {{&S22511[6], 4}, {NULL}};
const Segment SGI338300[] = {{&S22511[12], 14}, {&S22511[40], 12}, {NULL}};
const Segment SGI338301[] = {{&S22511[12], 14}, {NULL}};
const Segment SGI338302[] = {{&S22512[6], 4}, {NULL}};
const Segment SGI338303[] = {{&S22512[12], 14}, {NULL}};
const Segment SGI338304[] = {{&S22513[6], 4}, {NULL}};
const Segment SGI338305[] = {{&S22513[12], 14}, {&S22513[40], 12}, {NULL}};
const Segment SGI338306[] = {{&S22513[12], 14}, {NULL}};
const Segment SGI338307[] = {{&S22514[6], 4}, {NULL}};
const Segment SGI338308[] = {{&S22514[12], 14}, {&S22514[40], 12}, {NULL}};
const Segment SGI338309[] = {{&S22514[12], 14}, {NULL}};
const Segment SGI338310[] = {{&S22515[6], 4}, {NULL}};
const Segment SGI338311[] = {{&S22515[12], 14}, {NULL}};
const Segment SGI338312[] = {{&S22516[6], 4}, {NULL}};
const Segment SGI338313[] = {{&S22516[12], 14}, {&S22516[40], 12}, {NULL}};
const Segment SGI338314[] = {{&S22516[12], 14}, {NULL}};
const Segment SGI338315[] = {{&S22517[6], 4}, {NULL}};
const Segment SGI338316[] = {{&S22517[12], 14}, {&S22517[40], 12}, {NULL}};
const Segment SGI338317[] = {{&S22517[12], 14}, {NULL}};
const Segment SGI338318[] = {{&S22518[6], 4}, {NULL}};
const Segment SGI338319[] = {{&S22518[12], 14}, {&S22518[40], 12}, {NULL}};
const Segment SGI338320[] = {{&S22518[12], 14}, {NULL}};
const Segment SGI338321[] = {{&S22519[6], 4}, {NULL}};
const Segment SGI338322[] = {{&S22519[12], 14}, {&S22519[40], 12}, {NULL}};
const Segment SGI338323[] = {{&S22519[12], 14}, {NULL}};
const Segment SGI338324[] = {{&S22520[6], 4}, {NULL}};
const Segment SGI338325[] = {{&S22520[12], 14}, {&S22520[40], 12}, {NULL}};
const Segment SGI338326[] = {{&S22520[12], 14}, {NULL}};
const Segment SGI338327[] = {{&S22521[6], 4}, {NULL}};
const Segment SGI338328[] = {{&S22521[12], 14}, {&S22521[40], 11}, {NULL}};
const Segment SGI338329[] = {{&S22521[12], 14}, {NULL}};
const Segment SGI338330[] = {{&S22522[6], 4}, {NULL}};
const Segment SGI338331[] = {{&S22522[12], 14}, {&S22522[40], 11}, {NULL}};
const Segment SGI338332[] = {{&S22522[12], 14}, {NULL}};
const Segment SGI338333[] = {{&S22523[6], 4}, {NULL}};
const Segment SGI338334[] = {{&S22523[12], 14}, {&S22523[40], 11}, {NULL}};
const Segment SGI338335[] = {{&S22523[12], 14}, {NULL}};
const Segment SGI338336[] = {{&S22526[6], 4}, {NULL}};
const Segment SGI338337[] = {{&S22526[12], 14}, {&S22526[40], 11}, {NULL}};
const Segment SGI338338[] = {{&S22526[12], 14}, {NULL}};
const Segment SGI338339[] = {{&S22527[6], 4}, {NULL}};
const Segment SGI338340[] = {{&S22527[12], 14}, {&S22527[40], 12}, {NULL}};
const Segment SGI338341[] = {{&S22527[12], 14}, {NULL}};
const Segment SGI338342[] = {{&S22529[6], 4}, {NULL}};
const Segment SGI338343[] = {{&S22529[12], 14}, {&S22529[40], 12}, {NULL}};
const Segment SGI338344[] = {{&S22529[12], 14}, {NULL}};
const Segment SGI338345[] = {{&S22530[6], 4}, {NULL}};
const Segment SGI338346[] = {{&S22530[12], 14}, {&S22530[40], 12}, {NULL}};
const Segment SGI338347[] = {{&S22530[12], 14}, {NULL}};
const Segment SGI338348[] = {{&S22531[6], 4}, {NULL}};
const Segment SGI338349[] = {{&S22531[12], 14}, {&S22531[40], 11}, {NULL}};
const Segment SGI338350[] = {{&S22531[12], 14}, {NULL}};
const Segment SGI338351[] = {{&S22532[6], 4}, {NULL}};
const Segment SGI338352[] = {{&S22532[12], 14}, {&S22532[40], 11}, {NULL}};
const Segment SGI338353[] = {{&S22532[12], 14}, {NULL}};
const Segment SGI338354[] = {{&S22533[6], 4}, {NULL}};
const Segment SGI338355[] = {{&S22533[12], 14}, {&S22533[41], 11}, {NULL}};
const Segment SGI338356[] = {{&S22533[12], 14}, {NULL}};
const Segment SGI338357[] = {{&S22535[6], 4}, {NULL}};
const Segment SGI338358[] = {{&S22535[12], 14}, {&S22535[40], 12}, {NULL}};
const Segment SGI338359[] = {{&S22535[12], 14}, {NULL}};
const Segment SGI338360[] = {{&S22536[6], 4}, {NULL}};
const Segment SGI338361[] = {{&S22536[12], 14}, {&S22536[40], 11}, {NULL}};
const Segment SGI338362[] = {{&S22536[12], 14}, {NULL}};
const Segment SGI338363[] = {{&S22537[6], 4}, {NULL}};
const Segment SGI338364[] = {{&S22537[12], 14}, {&S22537[41], 11}, {NULL}};
const Segment SGI338365[] = {{&S22537[12], 14}, {NULL}};
const Segment SGI338366[] = {{&S22538[6], 4}, {NULL}};
const Segment SGI338367[] = {{&S22538[12], 14}, {&S22538[40], 11}, {NULL}};
const Segment SGI338368[] = {{&S22538[12], 14}, {NULL}};
const Segment SGI338369[] = {{&S22539[6], 4}, {NULL}};
const Segment SGI338370[] = {{&S22539[12], 14}, {&S22539[40], 11}, {NULL}};
const Segment SGI338371[] = {{&S22539[12], 14}, {NULL}};
const Segment SGI338372[] = {{&S22540[6], 4}, {NULL}};
const Segment SGI338373[] = {{&S22540[12], 14}, {&S22540[41], 11}, {NULL}};
const Segment SGI338374[] = {{&S22540[12], 14}, {NULL}};
const Segment SGI338375[] = {{&S22542[6], 4}, {NULL}};
const Segment SGI338376[] = {{&S22542[12], 14}, {&S22542[40], 12}, {NULL}};
const Segment SGI338377[] = {{&S22542[12], 14}, {NULL}};
const Segment SGI338378[] = {{&S22543[6], 4}, {NULL}};
const Segment SGI338379[] = {{&S22543[12], 14}, {&S22543[40], 12}, {NULL}};
const Segment SGI338380[] = {{&S22543[12], 14}, {NULL}};
const Segment SGI338381[] = {{&S22544[6], 4}, {NULL}};
const Segment SGI338382[] = {{&S22544[12], 14}, {&S22544[40], 12}, {NULL}};
const Segment SGI338383[] = {{&S22544[12], 14}, {NULL}};
const Segment SGI338384[] = {{&S22545[6], 4}, {NULL}};
const Segment SGI338385[] = {{&S22545[12], 14}, {&S22545[40], 11}, {NULL}};
const Segment SGI338386[] = {{&S22545[12], 14}, {NULL}};
const Segment SGI338387[] = {{&S22546[6], 4}, {NULL}};
const Segment SGI338388[] = {{&S22546[12], 14}, {&S22546[40], 11}, {NULL}};
const Segment SGI338389[] = {{&S22546[12], 14}, {NULL}};
const Segment SGI338390[] = {{&S22547[6], 4}, {NULL}};
const Segment SGI338391[] = {{&S22547[12], 14}, {&S22547[41], 11}, {NULL}};
const Segment SGI338392[] = {{&S22547[12], 14}, {NULL}};
const Segment SGI338393[] = {{&S22548[6], 4}, {NULL}};
const Segment SGI338394[] = {{&S22548[12], 14}, {&S22548[40], 11}, {NULL}};
const Segment SGI338395[] = {{&S22548[12], 14}, {NULL}};
const Segment SGI338396[] = {{&S22549[6], 4}, {NULL}};
const Segment SGI338397[] = {{&S22549[12], 14}, {&S22549[40], 11}, {NULL}};
const Segment SGI338398[] = {{&S22549[12], 14}, {NULL}};
const Segment SGI338399[] = {{&S22550[6], 4}, {NULL}};
const Segment SGI338400[] = {{&S22550[12], 14}, {&S22550[40], 11}, {NULL}};
const Segment SGI338401[] = {{&S22550[12], 14}, {NULL}};
const Segment SGI338402[] = {{&S22552[6], 4}, {NULL}};
const Segment SGI338403[] = {{&S22552[12], 14}, {&S22552[41], 11}, {NULL}};
const Segment SGI338404[] = {{&S22552[12], 14}, {NULL}};
const Segment SGI338405[] = {{&S22553[6], 4}, {NULL}};
const Segment SGI338406[] = {{&S22553[12], 14}, {&S22553[40], 11}, {NULL}};
const Segment SGI338407[] = {{&S22553[12], 14}, {NULL}};
const Segment SGI338408[] = {{&S22554[6], 4}, {NULL}};
const Segment SGI338409[] = {{&S22554[12], 14}, {&S22554[40], 11}, {NULL}};
const Segment SGI338410[] = {{&S22554[12], 14}, {NULL}};
const Segment SGI338411[] = {{&S22555[6], 4}, {NULL}};
const Segment SGI338412[] = {{&S22555[12], 14}, {&S22555[40], 11}, {NULL}};
const Segment SGI338413[] = {{&S22555[12], 14}, {NULL}};
const Segment SGI338414[] = {{&S22556[6], 4}, {NULL}};
const Segment SGI338415[] = {{&S22556[12], 14}, {&S22556[40], 11}, {NULL}};
const Segment SGI338416[] = {{&S22556[12], 14}, {NULL}};
const Segment SGI338417[] = {{&S22557[6], 4}, {NULL}};
const Segment SGI338418[] = {{&S22557[12], 14}, {&S22557[40], 11}, {NULL}};
const Segment SGI338419[] = {{&S22557[12], 14}, {NULL}};
const Segment SGI338420[] = {{&S22558[6], 4}, {NULL}};
const Segment SGI338421[] = {{&S22558[12], 14}, {&S22558[40], 11}, {NULL}};
const Segment SGI338422[] = {{&S22558[12], 14}, {NULL}};
const Segment SGI338423[] = {{&S22559[6], 4}, {NULL}};
const Segment SGI338424[] = {{&S22559[12], 14}, {&S22559[40], 11}, {NULL}};
const Segment SGI338425[] = {{&S22559[12], 14}, {NULL}};
const Segment SGI338426[] = {{&S22560[6], 4}, {NULL}};
const Segment SGI338427[] = {{&S22560[12], 14}, {&S22560[40], 11}, {NULL}};
const Segment SGI338428[] = {{&S22560[12], 14}, {NULL}};
const Segment SGI338429[] = {{&S22561[6], 4}, {NULL}};
const Segment SGI338430[] = {{&S22561[12], 14}, {&S22561[40], 11}, {NULL}};
const Segment SGI338431[] = {{&S22561[12], 14}, {NULL}};
const Segment SGI338432[] = {{&S22562[6], 4}, {NULL}};
const Segment SGI338433[] = {{&S22562[12], 14}, {&S22562[40], 11}, {NULL}};
const Segment SGI338434[] = {{&S22562[12], 14}, {NULL}};
const Segment SGI338435[] = {{&S22563[6], 4}, {NULL}};
const Segment SGI338436[] = {{&S22563[12], 14}, {&S22563[40], 11}, {NULL}};
const Segment SGI338437[] = {{&S22563[12], 14}, {NULL}};
const Segment SGI338438[] = {{&S39966[28], 5}, {NULL}};
const Segment SGI338439[] = {{&S39966[105], 5}, {NULL}};
const Segment SGI338440[] = {{&S48909[5], 3}, {NULL}};
const Segment SGI338441[] = {{&S48910[5], 3}, {&S48910[5], 3}, {NULL}};
const Segment SGI338442[] = {{&S49092[5], 4}, {NULL}};
const Segment SGI338443[] = {{&S49092[15], 3}, {NULL}};
const Segment SGI338444[] = {{&S49093[35], 4}, {NULL}};
const Segment SGI338445[] = {{&S49093[90], 3}, {NULL}};
const Segment SGI338446[] = {{&S49094[35], 5}, {NULL}};
const Segment SGI338447[] = {{&S49094[91], 3}, {NULL}};
const Segment SGI338448[] = {{&S23187[6], 4}, {NULL}};
const Segment SGI338449[] = {{&S23187[52], 5}, {NULL}};
const Segment SGI338450[] = {{&S23187[12], 45}, {NULL}};
const Segment SGI338451[] = {{&S23187[12], 14}, {NULL}};
const Segment SGI338452[] = {{&S23189[6], 4}, {NULL}};
const Segment SGI338453[] = {{&S23189[52], 5}, {NULL}};
const Segment SGI338454[] = {{&S23189[12], 45}, {NULL}};
const Segment SGI338455[] = {{&S23189[12], 14}, {NULL}};
const Segment SGI338456[] = {{&S23190[6], 4}, {NULL}};
const Segment SGI338457[] = {{&S23190[64], 5}, {NULL}};
const Segment SGI338458[] = {{&S23190[12], 14}, {NULL}};
const Segment SGI338459[] = {{&S23191[6], 4}, {NULL}};
const Segment SGI338460[] = {{&S23191[59], 5}, {NULL}};
const Segment SGI338461[] = {{&S23191[12], 14}, {NULL}};
const Segment SGI338462[] = {{&S23193[6], 4}, {NULL}};
const Segment SGI338463[] = {{&S23193[74], 5}, {NULL}};
const Segment SGI338464[] = {{&S23193[12], 14}, {&S23193[37], 42}, {NULL}};
const Segment SGI338465[] = {{&S23193[12], 14}, {NULL}};
const Segment SGI338466[] = {{&S23194[6], 4}, {NULL}};
const Segment SGI338467[] = {{&S23194[74], 5}, {NULL}};
const Segment SGI338468[] = {{&S23194[12], 14}, {&S23194[37], 42}, {NULL}};
const Segment SGI338469[] = {{&S23194[12], 14}, {NULL}};
const Segment SGI338470[] = {{&S23195[6], 4}, {NULL}};
const Segment SGI338471[] = {{&S23195[74], 5}, {NULL}};
const Segment SGI338472[] = {{&S23195[12], 14}, {&S23195[37], 42}, {NULL}};
const Segment SGI338473[] = {{&S23195[12], 14}, {NULL}};
const Segment SGI338474[] = {{&S23196[6], 4}, {NULL}};
const Segment SGI338475[] = {{&S23196[74], 5}, {NULL}};
const Segment SGI338476[] = {{&S23196[12], 14}, {&S23196[37], 42}, {NULL}};
const Segment SGI338477[] = {{&S23196[12], 14}, {NULL}};
const Segment SGI338478[] = {{&S23197[6], 4}, {NULL}};
const Segment SGI338479[] = {{&S23197[74], 5}, {NULL}};
const Segment SGI338480[] = {{&S23197[12], 14}, {&S23197[37], 42}, {NULL}};
const Segment SGI338481[] = {{&S23197[12], 14}, {NULL}};
const Segment SGI338482[] = {{&S23198[6], 4}, {NULL}};
const Segment SGI338483[] = {{&S23198[74], 5}, {NULL}};
const Segment SGI338484[] = {{&S23198[12], 14}, {&S23198[37], 42}, {NULL}};
const Segment SGI338485[] = {{&S23198[12], 14}, {NULL}};
const Segment SGI338486[] = {{&S23199[6], 4}, {NULL}};
const Segment SGI338487[] = {{&S23199[75], 5}, {NULL}};
const Segment SGI338488[] = {{&S23199[12], 14}, {&S23199[37], 43}, {NULL}};
const Segment SGI338489[] = {{&S23199[12], 14}, {NULL}};
const Segment SGI338490[] = {{&S23200[6], 4}, {NULL}};
const Segment SGI338491[] = {{&S23200[75], 5}, {NULL}};
const Segment SGI338492[] = {{&S23200[12], 14}, {&S23200[37], 43}, {NULL}};
const Segment SGI338493[] = {{&S23200[12], 14}, {NULL}};
const Segment SGI338494[] = {{&S23201[6], 4}, {NULL}};
const Segment SGI338495[] = {{&S23201[75], 5}, {NULL}};
const Segment SGI338496[] = {{&S23201[12], 14}, {&S23201[38], 42}, {NULL}};
const Segment SGI338497[] = {{&S23201[12], 14}, {NULL}};
const Segment SGI338498[] = {{&S23202[6], 4}, {NULL}};
const Segment SGI338499[] = {{&S23202[74], 5}, {NULL}};
const Segment SGI338500[] = {{&S23202[12], 14}, {&S23202[37], 42}, {NULL}};
const Segment SGI338501[] = {{&S23202[12], 14}, {NULL}};
const Segment SGI338502[] = {{&S23203[6], 4}, {NULL}};
const Segment SGI338503[] = {{&S23203[74], 5}, {NULL}};
const Segment SGI338504[] = {{&S23203[12], 14}, {&S23203[37], 42}, {NULL}};
const Segment SGI338505[] = {{&S23203[12], 14}, {NULL}};
const Segment SGI338506[] = {{&S23204[6], 4}, {NULL}};
const Segment SGI338507[] = {{&S23204[74], 5}, {NULL}};
const Segment SGI338508[] = {{&S23204[12], 14}, {&S23204[37], 42}, {NULL}};
const Segment SGI338509[] = {{&S23204[12], 14}, {NULL}};
const Segment SGI338510[] = {{&S23205[6], 4}, {NULL}};
const Segment SGI338511[] = {{&S23205[75], 5}, {NULL}};
const Segment SGI338512[] = {{&S23205[12], 14}, {&S23205[38], 42}, {NULL}};
const Segment SGI338513[] = {{&S23205[12], 14}, {NULL}};
const Segment SGI338514[] = {{&S23206[6], 4}, {NULL}};
const Segment SGI338515[] = {{&S23206[75], 5}, {NULL}};
const Segment SGI338516[] = {{&S23206[12], 14}, {&S23206[38], 42}, {NULL}};
const Segment SGI338517[] = {{&S23206[12], 14}, {NULL}};
const Segment SGI338518[] = {{&S23207[6], 4}, {NULL}};
const Segment SGI338519[] = {{&S23207[52], 5}, {NULL}};
const Segment SGI338520[] = {{&S23207[12], 55}, {NULL}};
const Segment SGI338521[] = {{&S23207[12], 14}, {NULL}};
const Segment SGI338522[] = {{&S23208[6], 4}, {NULL}};
const Segment SGI338523[] = {{&S23208[74], 5}, {NULL}};
const Segment SGI338524[] = {{&S23208[12], 14}, {&S23208[37], 42}, {NULL}};
const Segment SGI338525[] = {{&S23208[12], 14}, {NULL}};
const Segment SGI338526[] = {{&S23209[6], 4}, {NULL}};
const Segment SGI338527[] = {{&S23209[74], 5}, {NULL}};
const Segment SGI338528[] = {{&S23209[12], 14}, {&S23209[37], 42}, {NULL}};
const Segment SGI338529[] = {{&S23209[12], 14}, {NULL}};
const Segment SGI338530[] = {{&S23210[6], 4}, {NULL}};
const Segment SGI338531[] = {{&S23210[74], 5}, {NULL}};
const Segment SGI338532[] = {{&S23210[12], 14}, {&S23210[37], 42}, {NULL}};
const Segment SGI338533[] = {{&S23210[12], 14}, {NULL}};
const Segment SGI338534[] = {{&S23211[6], 4}, {NULL}};
const Segment SGI338535[] = {{&S23211[74], 5}, {NULL}};
const Segment SGI338536[] = {{&S23211[12], 14}, {&S23211[37], 42}, {NULL}};
const Segment SGI338537[] = {{&S23211[12], 14}, {NULL}};
const Segment SGI338538[] = {{&S23212[6], 4}, {NULL}};
const Segment SGI338539[] = {{&S23212[74], 5}, {NULL}};
const Segment SGI338540[] = {{&S23212[12], 14}, {&S23212[37], 42}, {NULL}};
const Segment SGI338541[] = {{&S23212[12], 14}, {NULL}};
const Segment SGI338542[] = {{&S23213[6], 4}, {NULL}};
const Segment SGI338543[] = {{&S23213[74], 5}, {NULL}};
const Segment SGI338544[] = {{&S23213[12], 14}, {&S23213[37], 42}, {NULL}};
const Segment SGI338545[] = {{&S23213[12], 14}, {NULL}};
const Segment SGI338546[] = {{&S23214[6], 4}, {NULL}};
const Segment SGI338547[] = {{&S23214[74], 5}, {NULL}};
const Segment SGI338548[] = {{&S23214[12], 14}, {&S23214[37], 42}, {NULL}};
const Segment SGI338549[] = {{&S23214[12], 14}, {NULL}};
const Segment SGI338550[] = {{&S23215[6], 4}, {NULL}};
const Segment SGI338551[] = {{&S23215[75], 5}, {NULL}};
const Segment SGI338552[] = {{&S23215[12], 14}, {&S23215[37], 43}, {NULL}};
const Segment SGI338553[] = {{&S23215[12], 14}, {NULL}};
const Segment SGI338554[] = {{&S23216[6], 4}, {NULL}};
const Segment SGI338555[] = {{&S23216[74], 5}, {NULL}};
const Segment SGI338556[] = {{&S23216[12], 14}, {&S23216[37], 42}, {NULL}};
const Segment SGI338557[] = {{&S23216[12], 14}, {NULL}};
const Segment SGI338558[] = {{&S23217[6], 4}, {NULL}};
const Segment SGI338559[] = {{&S23217[74], 5}, {NULL}};
const Segment SGI338560[] = {{&S23217[12], 14}, {&S23217[37], 42}, {NULL}};
const Segment SGI338561[] = {{&S23217[12], 14}, {NULL}};
const Segment SGI338562[] = {{&S23218[6], 4}, {NULL}};
const Segment SGI338563[] = {{&S23218[74], 5}, {NULL}};
const Segment SGI338564[] = {{&S23218[12], 14}, {&S23218[37], 42}, {NULL}};
const Segment SGI338565[] = {{&S23218[12], 14}, {NULL}};
const Segment SGI338566[] = {{&S23219[6], 4}, {NULL}};
const Segment SGI338567[] = {{&S23219[74], 5}, {NULL}};
const Segment SGI338568[] = {{&S23219[12], 14}, {&S23219[37], 42}, {NULL}};
const Segment SGI338569[] = {{&S23219[12], 14}, {NULL}};
const Segment SGI338570[] = {{&S23220[6], 4}, {NULL}};
const Segment SGI338571[] = {{&S23220[74], 5}, {NULL}};
const Segment SGI338572[] = {{&S23220[12], 14}, {&S23220[37], 42}, {NULL}};
const Segment SGI338573[] = {{&S23220[12], 14}, {NULL}};
const Segment SGI338574[] = {{&S23221[6], 4}, {NULL}};
const Segment SGI338575[] = {{&S23221[74], 5}, {NULL}};
const Segment SGI338576[] = {{&S23221[12], 14}, {&S23221[37], 42}, {NULL}};
const Segment SGI338577[] = {{&S23221[12], 14}, {NULL}};
const Segment SGI338578[] = {{&S23222[6], 4}, {NULL}};
const Segment SGI338579[] = {{&S23222[74], 5}, {NULL}};
const Segment SGI338580[] = {{&S23222[12], 14}, {&S23222[37], 42}, {NULL}};
const Segment SGI338581[] = {{&S23222[12], 14}, {NULL}};
const Segment SGI338582[] = {{&S23223[6], 4}, {NULL}};
const Segment SGI338583[] = {{&S23223[75], 5}, {NULL}};
const Segment SGI338584[] = {{&S23223[12], 14}, {&S23223[37], 43}, {NULL}};
const Segment SGI338585[] = {{&S23223[12], 14}, {NULL}};
const Segment SGI338586[] = {{&S23224[6], 4}, {NULL}};
const Segment SGI338587[] = {{&S23224[75], 5}, {NULL}};
const Segment SGI338588[] = {{&S23224[12], 14}, {&S23224[38], 42}, {NULL}};
const Segment SGI338589[] = {{&S23224[12], 14}, {NULL}};
const Segment SGI338590[] = {{&S23225[6], 4}, {NULL}};
const Segment SGI338591[] = {{&S23225[52], 5}, {NULL}};
const Segment SGI338592[] = {{&S23225[12], 62}, {NULL}};
const Segment SGI338593[] = {{&S23225[12], 14}, {NULL}};
const Segment SGI338594[] = {{&S23226[6], 4}, {NULL}};
const Segment SGI338595[] = {{&S23226[75], 5}, {NULL}};
const Segment SGI338596[] = {{&S23226[12], 14}, {&S23226[37], 43}, {NULL}};
const Segment SGI338597[] = {{&S23226[12], 14}, {NULL}};
const Segment SGI338598[] = {{&S23227[6], 4}, {NULL}};
const Segment SGI338599[] = {{&S23227[74], 5}, {NULL}};
const Segment SGI338600[] = {{&S23227[12], 14}, {&S23227[37], 42}, {NULL}};
const Segment SGI338601[] = {{&S23227[12], 14}, {NULL}};
const Segment SGI338602[] = {{&S23228[6], 4}, {NULL}};
const Segment SGI338603[] = {{&S23228[75], 5}, {NULL}};
const Segment SGI338604[] = {{&S23228[12], 14}, {&S23228[38], 42}, {NULL}};
const Segment SGI338605[] = {{&S23228[12], 14}, {NULL}};
const Segment SGI338606[] = {{&S23229[6], 4}, {NULL}};
const Segment SGI338607[] = {{&S23229[75], 5}, {NULL}};
const Segment SGI338608[] = {{&S23229[12], 14}, {&S23229[38], 42}, {NULL}};
const Segment SGI338609[] = {{&S23229[12], 14}, {NULL}};
const Segment SGI338610[] = {{&S23230[6], 4}, {NULL}};
const Segment SGI338611[] = {{&S23230[75], 5}, {NULL}};
const Segment SGI338612[] = {{&S23230[12], 14}, {&S23230[38], 42}, {NULL}};
const Segment SGI338613[] = {{&S23230[12], 14}, {NULL}};
const Segment SGI338614[] = {{&S23231[6], 4}, {NULL}};
const Segment SGI338615[] = {{&S23231[75], 5}, {NULL}};
const Segment SGI338616[] = {{&S23231[12], 14}, {&S23231[37], 43}, {NULL}};
const Segment SGI338617[] = {{&S23231[12], 14}, {NULL}};
const Segment SGI338618[] = {{&S23232[6], 4}, {NULL}};
const Segment SGI338619[] = {{&S23232[75], 5}, {NULL}};
const Segment SGI338620[] = {{&S23232[12], 14}, {&S23232[37], 43}, {NULL}};
const Segment SGI338621[] = {{&S23232[12], 14}, {NULL}};
const Segment SGI338622[] = {{&S23233[6], 4}, {NULL}};
const Segment SGI338623[] = {{&S23233[75], 5}, {NULL}};
const Segment SGI338624[] = {{&S23233[12], 14}, {&S23233[38], 42}, {NULL}};
const Segment SGI338625[] = {{&S23233[12], 14}, {NULL}};
const Segment SGI338626[] = {{&S23234[6], 4}, {NULL}};
const Segment SGI338627[] = {{&S23234[75], 5}, {NULL}};
const Segment SGI338628[] = {{&S23234[12], 14}, {&S23234[38], 42}, {NULL}};
const Segment SGI338629[] = {{&S23234[12], 14}, {NULL}};
const Segment SGI338630[] = {{&S23235[6], 4}, {NULL}};
const Segment SGI338631[] = {{&S23235[75], 5}, {NULL}};
const Segment SGI338632[] = {{&S23235[12], 14}, {&S23235[37], 43}, {NULL}};
const Segment SGI338633[] = {{&S23235[12], 14}, {NULL}};
const Segment SGI338634[] = {{&S23236[6], 4}, {NULL}};
const Segment SGI338635[] = {{&S23236[74], 5}, {NULL}};
const Segment SGI338636[] = {{&S23236[12], 14}, {&S23236[37], 42}, {NULL}};
const Segment SGI338637[] = {{&S23236[12], 14}, {NULL}};
const Segment SGI338638[] = {{&S23237[6], 4}, {NULL}};
const Segment SGI338639[] = {{&S23237[74], 5}, {NULL}};
const Segment SGI338640[] = {{&S23237[12], 14}, {&S23237[37], 42}, {NULL}};
const Segment SGI338641[] = {{&S23237[12], 14}, {NULL}};
const Segment SGI338642[] = {{&S23238[6], 4}, {NULL}};
const Segment SGI338643[] = {{&S23238[74], 5}, {NULL}};
const Segment SGI338644[] = {{&S23238[12], 14}, {&S23238[37], 42}, {NULL}};
const Segment SGI338645[] = {{&S23238[12], 14}, {NULL}};
const Segment SGI338646[] = {{&S23239[6], 4}, {NULL}};
const Segment SGI338647[] = {{&S23239[75], 5}, {NULL}};
const Segment SGI338648[] = {{&S23239[12], 14}, {&S23239[38], 42}, {NULL}};
const Segment SGI338649[] = {{&S23239[12], 14}, {NULL}};
const Segment SGI338650[] = {{&S23240[6], 4}, {NULL}};
const Segment SGI338651[] = {{&S23240[74], 5}, {NULL}};
const Segment SGI338652[] = {{&S23240[12], 14}, {&S23240[37], 42}, {NULL}};
const Segment SGI338653[] = {{&S23240[12], 14}, {NULL}};
const Segment SGI338654[] = {{&S23241[6], 4}, {NULL}};
const Segment SGI338655[] = {{&S23241[74], 5}, {NULL}};
const Segment SGI338656[] = {{&S23241[12], 14}, {&S23241[37], 42}, {NULL}};
const Segment SGI338657[] = {{&S23241[12], 14}, {NULL}};
const Segment SGI338658[] = {{&S23242[6], 4}, {NULL}};
const Segment SGI338659[] = {{&S23242[74], 5}, {NULL}};
const Segment SGI338660[] = {{&S23242[12], 14}, {&S23242[37], 42}, {NULL}};
const Segment SGI338661[] = {{&S23242[12], 14}, {NULL}};
const Segment SGI338662[] = {{&S23243[6], 4}, {NULL}};
const Segment SGI338663[] = {{&S23243[75], 5}, {NULL}};
const Segment SGI338664[] = {{&S23243[12], 14}, {&S23243[38], 42}, {NULL}};
const Segment SGI338665[] = {{&S23243[12], 14}, {NULL}};
const Segment SGI338666[] = {{&S23244[6], 4}, {NULL}};
const Segment SGI338667[] = {{&S23244[74], 5}, {NULL}};
const Segment SGI338668[] = {{&S23244[12], 14}, {&S23244[37], 42}, {NULL}};
const Segment SGI338669[] = {{&S23244[12], 14}, {NULL}};
const Segment SGI338670[] = {{&S23245[6], 4}, {NULL}};
const Segment SGI338671[] = {{&S23245[74], 5}, {NULL}};
const Segment SGI338672[] = {{&S23245[12], 14}, {&S23245[37], 42}, {NULL}};
const Segment SGI338673[] = {{&S23245[12], 14}, {NULL}};
const Segment SGI338674[] = {{&S23246[6], 4}, {NULL}};
const Segment SGI338675[] = {{&S23246[75], 5}, {NULL}};
const Segment SGI338676[] = {{&S23246[12], 14}, {&S23246[37], 43}, {NULL}};
const Segment SGI338677[] = {{&S23246[12], 14}, {NULL}};
const Segment SGI338678[] = {{&S23247[6], 4}, {NULL}};
const Segment SGI338679[] = {{&S23247[73], 5}, {NULL}};
const Segment SGI338680[] = {{&S23247[12], 14}, {&S23247[84], 4}, {NULL}};
const Segment SGI338681[] = {{&S23247[12], 14}, {NULL}};
const Segment SGI338682[] = {{&S23248[6], 4}, {NULL}};
const Segment SGI338683[] = {{&S23248[75], 5}, {NULL}};
const Segment SGI338684[] = {{&S23248[12], 14}, {&S23248[38], 42}, {NULL}};
const Segment SGI338685[] = {{&S23248[12], 14}, {NULL}};
const Segment SGI338686[] = {{&S23249[6], 4}, {NULL}};
const Segment SGI338687[] = {{&S23249[75], 5}, {NULL}};
const Segment SGI338688[] = {{&S23249[12], 14}, {&S23249[37], 43}, {NULL}};
const Segment SGI338689[] = {{&S23249[12], 14}, {NULL}};
const Segment SGI338690[] = {{&S23250[6], 4}, {NULL}};
const Segment SGI338691[] = {{&S23250[101], 5}, {NULL}};
const Segment SGI338692[] = {{&S23250[12], 14}, {&S23250[37], 12}, {NULL}};
const Segment SGI338693[] = {{&S23250[12], 14}, {NULL}};
const Segment SGI338694[] = {{&S23251[6], 4}, {NULL}};
const Segment SGI338695[] = {{&S23251[75], 5}, {NULL}};
const Segment SGI338696[] = {{&S23251[12], 14}, {&S23251[37], 43}, {NULL}};
const Segment SGI338697[] = {{&S23251[12], 14}, {NULL}};
const Segment SGI338698[] = {{&S23252[6], 4}, {NULL}};
const Segment SGI338699[] = {{&S23252[75], 5}, {NULL}};
const Segment SGI338700[] = {{&S23252[12], 14}, {&S23252[38], 42}, {NULL}};
const Segment SGI338701[] = {{&S23252[12], 14}, {NULL}};
const Segment SGI338702[] = {{&S49276[5], 3}, {NULL}};
const Segment SGI338703[] = {{&S23253[6], 4}, {NULL}};
const Segment SGI338704[] = {{&S23253[74], 5}, {NULL}};
const Segment SGI338705[] = {{&S23253[12], 14}, {&S23253[37], 42}, {NULL}};
const Segment SGI338706[] = {{&S23253[12], 14}, {NULL}};
const Segment SGI338707[] = {{&S49277[5], 3}, {NULL}};
const Segment SGI338708[] = {{&S23254[6], 4}, {NULL}};
const Segment SGI338709[] = {{&S23254[74], 5}, {NULL}};
const Segment SGI338710[] = {{&S23254[12], 14}, {&S23254[37], 42}, {NULL}};
const Segment SGI338711[] = {{&S23254[12], 14}, {NULL}};
const Segment SGI338712[] = {{&S49278[5], 3}, {&S49278[5], 3}, {NULL}};
const Segment SGI338713[] = {{&S23255[6], 4}, {NULL}};
const Segment SGI338714[] = {{&S23255[74], 5}, {NULL}};
const Segment SGI338715[] = {{&S23255[12], 14}, {&S23255[37], 42}, {NULL}};
const Segment SGI338716[] = {{&S23255[12], 14}, {NULL}};
const Segment SGI338717[] = {{&S23256[6], 4}, {NULL}};
const Segment SGI338718[] = {{&S23256[74], 5}, {NULL}};
const Segment SGI338719[] = {{&S23256[12], 14}, {&S23256[37], 42}, {NULL}};
const Segment SGI338720[] = {{&S23256[12], 14}, {NULL}};
const Segment SGI338721[] = {{&S23257[6], 4}, {NULL}};
const Segment SGI338722[] = {{&S23257[74], 5}, {NULL}};
const Segment SGI338723[] = {{&S23257[12], 14}, {&S23257[37], 42}, {NULL}};
const Segment SGI338724[] = {{&S23257[12], 14}, {NULL}};
const Segment SGI338725[] = {{&S23258[6], 4}, {NULL}};
const Segment SGI338726[] = {{&S23258[74], 5}, {NULL}};
const Segment SGI338727[] = {{&S23258[12], 14}, {&S23258[37], 42}, {NULL}};
const Segment SGI338728[] = {{&S23258[12], 14}, {NULL}};
const Segment SGI338729[] = {{&S23259[6], 4}, {NULL}};
const Segment SGI338730[] = {{&S23259[74], 5}, {NULL}};
const Segment SGI338731[] = {{&S23259[12], 14}, {&S23259[37], 42}, {NULL}};
const Segment SGI338732[] = {{&S23259[12], 14}, {NULL}};
const Segment SGI338733[] = {{&S23260[6], 4}, {NULL}};
const Segment SGI338734[] = {{&S23260[74], 5}, {NULL}};
const Segment SGI338735[] = {{&S23260[12], 14}, {&S23260[37], 42}, {NULL}};
const Segment SGI338736[] = {{&S23260[12], 14}, {NULL}};
const Segment SGI338737[] = {{&S23261[6], 4}, {NULL}};
const Segment SGI338738[] = {{&S23261[74], 5}, {NULL}};
const Segment SGI338739[] = {{&S23261[12], 14}, {&S23261[37], 42}, {NULL}};
const Segment SGI338740[] = {{&S23261[12], 14}, {NULL}};
const Segment SGI338741[] = {{&S23262[6], 4}, {NULL}};
const Segment SGI338742[] = {{&S23262[74], 5}, {NULL}};
const Segment SGI338743[] = {{&S23262[12], 14}, {&S23262[37], 42}, {NULL}};
const Segment SGI338744[] = {{&S23262[12], 14}, {NULL}};
const Segment SGI338745[] = {{&S23263[6], 4}, {NULL}};
const Segment SGI338746[] = {{&S23263[74], 5}, {NULL}};
const Segment SGI338747[] = {{&S23263[12], 14}, {&S23263[37], 42}, {NULL}};
const Segment SGI338748[] = {{&S23263[12], 14}, {NULL}};
const Segment SGI338749[] = {{&S23264[6], 4}, {NULL}};
const Segment SGI338750[] = {{&S23264[75], 5}, {NULL}};
const Segment SGI338751[] = {{&S23264[12], 14}, {&S23264[38], 42}, {NULL}};
const Segment SGI338752[] = {{&S23264[12], 14}, {NULL}};
const Segment SGI338753[] = {{&S23265[6], 4}, {NULL}};
const Segment SGI338754[] = {{&S23265[74], 5}, {NULL}};
const Segment SGI338755[] = {{&S23265[12], 14}, {&S23265[37], 42}, {NULL}};
const Segment SGI338756[] = {{&S23265[12], 14}, {NULL}};
const Segment SGI338757[] = {{&S23266[6], 4}, {NULL}};
const Segment SGI338758[] = {{&S23266[74], 5}, {NULL}};
const Segment SGI338759[] = {{&S23266[12], 14}, {&S23266[37], 42}, {NULL}};
const Segment SGI338760[] = {{&S23266[12], 14}, {NULL}};
const Segment SGI338761[] = {{&S23267[6], 4}, {NULL}};
const Segment SGI338762[] = {{&S23267[75], 5}, {NULL}};
const Segment SGI338763[] = {{&S23267[12], 14}, {&S23267[37], 43}, {NULL}};
const Segment SGI338764[] = {{&S23267[12], 14}, {NULL}};
const Segment SGI338765[] = {{&S23268[6], 4}, {NULL}};
const Segment SGI338766[] = {{&S23268[74], 5}, {NULL}};
const Segment SGI338767[] = {{&S23268[12], 14}, {&S23268[37], 42}, {NULL}};
const Segment SGI338768[] = {{&S23268[12], 14}, {NULL}};
const Segment SGI338769[] = {{&S23269[6], 4}, {NULL}};
const Segment SGI338770[] = {{&S23269[75], 5}, {NULL}};
const Segment SGI338771[] = {{&S23269[12], 14}, {&S23269[37], 43}, {NULL}};
const Segment SGI338772[] = {{&S23269[12], 14}, {NULL}};
const Segment SGI338773[] = {{&S23270[6], 4}, {NULL}};
const Segment SGI338774[] = {{&S23270[74], 5}, {NULL}};
const Segment SGI338775[] = {{&S23270[12], 14}, {&S23270[37], 42}, {NULL}};
const Segment SGI338776[] = {{&S23270[12], 14}, {NULL}};
const Segment SGI338777[] = {{&S23271[6], 4}, {NULL}};
const Segment SGI338778[] = {{&S23271[74], 5}, {NULL}};
const Segment SGI338779[] = {{&S23271[12], 14}, {&S23271[37], 42}, {NULL}};
const Segment SGI338780[] = {{&S23271[12], 14}, {NULL}};
const Segment SGI338781[] = {{&S23272[6], 4}, {NULL}};
const Segment SGI338782[] = {{&S23272[74], 5}, {NULL}};
const Segment SGI338783[] = {{&S23272[12], 14}, {&S23272[37], 42}, {NULL}};
const Segment SGI338784[] = {{&S23272[12], 14}, {NULL}};
const Segment SGI338785[] = {{&S23273[6], 4}, {NULL}};
const Segment SGI338786[] = {{&S23273[74], 5}, {NULL}};
const Segment SGI338787[] = {{&S23273[12], 14}, {&S23273[37], 42}, {NULL}};
const Segment SGI338788[] = {{&S23273[12], 14}, {NULL}};
const Segment SGI338789[] = {{&S23274[6], 4}, {NULL}};
const Segment SGI338790[] = {{&S23274[102], 5}, {NULL}};
const Segment SGI338791[] = {{&S23274[12], 14}, {&S23274[37], 12}, {NULL}};
const Segment SGI338792[] = {{&S23274[12], 14}, {NULL}};
const Segment SGI338793[] = {{&S23275[6], 4}, {NULL}};
const Segment SGI338794[] = {{&S23275[74], 5}, {NULL}};
const Segment SGI338795[] = {{&S23275[12], 14}, {&S23275[37], 42}, {NULL}};
const Segment SGI338796[] = {{&S23275[12], 14}, {NULL}};
const Segment SGI338797[] = {{&S23276[6], 4}, {NULL}};
const Segment SGI338798[] = {{&S23276[74], 5}, {NULL}};
const Segment SGI338799[] = {{&S23276[12], 14}, {&S23276[37], 42}, {NULL}};
const Segment SGI338800[] = {{&S23276[12], 14}, {NULL}};
const Segment SGI338801[] = {{&S23277[6], 4}, {NULL}};
const Segment SGI338802[] = {{&S23277[74], 5}, {NULL}};
const Segment SGI338803[] = {{&S23277[12], 14}, {&S23277[37], 42}, {NULL}};
const Segment SGI338804[] = {{&S23277[12], 14}, {NULL}};
const Segment SGI338805[] = {{&S23278[6], 4}, {NULL}};
const Segment SGI338806[] = {{&S23278[74], 5}, {NULL}};
const Segment SGI338807[] = {{&S23278[12], 14}, {&S23278[37], 42}, {NULL}};
const Segment SGI338808[] = {{&S23278[12], 14}, {NULL}};
const Segment SGI338809[] = {{&S49327[5], 5}, {&S49327[97], 5}, {NULL}};
const Segment SGI338810[] = {{&S49327[17], 3}, {NULL}};
const Segment SGI338811[] = {{&S21962[0], 21}, {NULL}};
const Segment SGI338812[] = {{&S21962[34], 3}, {NULL}};
const Segment SGI338813[] = {{&S21963[0], 21}, {NULL}};
const Segment SGI338814[] = {{&S21963[34], 3}, {NULL}};
const Segment SGI338815[] = {{&S21963[75], 13}, {NULL}};
const Segment SGI338816[] = {{&S49370[5], 3}, {&S49370[5], 3}, {NULL}};
const Segment SGI338817[] = {{&S49370[9], 3}, {NULL}};
const Segment SGI338818[] = {{&S49370[126], 3}, {NULL}};
const Segment SGI338819[] = {{&S40471[28], 5}, {NULL}};
const Segment SGI338820[] = {{&S40471[104], 5}, {NULL}};
const Segment SGI338821[] = {{&S40472[28], 5}, {NULL}};
const Segment SGI338822[] = {{&S40472[104], 5}, {NULL}};
const Segment SGI338823[] = {{&S35978[46], 7}, {NULL}};
const Segment SGI338824[] = {{&S35978[8], 3}, {NULL}};
const Segment SGI338825[] = {{&S35979[46], 5}, {NULL}};
const Segment SGI338826[] = {{&S35979[8], 3}, {NULL}};
const Segment SGI338827[] = {{&S35980[46], 5}, {&S35980[146], 7}, {NULL}};
const Segment SGI338828[] = {{&S35980[8], 3}, {NULL}};
const Segment SGI338829[] = {{&S35981[46], 7}, {&S35981[148], 7}, {NULL}};
const Segment SGI338830[] = {{&S35981[8], 3}, {NULL}};
const Segment SGI338831[] = {{&S36139[42], 14}, {&S36139[134], 7}, {NULL}};
const Segment SGI338832[] = {{&S36139[33], 3}, {NULL}};
const Segment SGI338833[] = {{&S36140[42], 5}, {&S36140[147], 7}, {NULL}};
const Segment SGI338834[] = {{&S36140[33], 3}, {NULL}};
const Segment SGI338835[] = {{&S49701[5], 5}, {&S49701[68], 5}, {NULL}};
const Segment SGI338836[] = {{&S49702[5], 5}, {NULL}};
const Segment SGI338837[] = {{&S49703[5], 7}, {NULL}};
const Segment SGI338838[] = {{&S49713[5], 7}, {&S49713[5], 7}, {NULL}};
const Segment SGI338839[] = {{&S49713[117], 3}, {NULL}};
const Segment SGI338840[] = {{&S23694[6], 4}, {NULL}};
const Segment SGI338841[] = {{&S23694[52], 5}, {NULL}};
const Segment SGI338842[] = {{&S23694[12], 45}, {NULL}};
const Segment SGI338843[] = {{&S23694[12], 14}, {NULL}};
const Segment SGI338844[] = {{&S23695[6], 4}, {NULL}};
const Segment SGI338845[] = {{&S23695[59], 5}, {NULL}};
const Segment SGI338846[] = {{&S23695[12], 14}, {NULL}};
const Segment SGI338847[] = {{&S23696[6], 4}, {NULL}};
const Segment SGI338848[] = {{&S23696[74], 5}, {NULL}};
const Segment SGI338849[] = {{&S23696[12], 14}, {&S23696[37], 42}, {NULL}};
const Segment SGI338850[] = {{&S23696[12], 14}, {NULL}};
const Segment SGI338851[] = {{&S23697[6], 4}, {NULL}};
const Segment SGI338852[] = {{&S23697[74], 5}, {NULL}};
const Segment SGI338853[] = {{&S23697[12], 14}, {&S23697[37], 42}, {NULL}};
const Segment SGI338854[] = {{&S23697[12], 14}, {NULL}};
const Segment SGI338855[] = {{&S23698[6], 4}, {NULL}};
const Segment SGI338856[] = {{&S23698[74], 5}, {NULL}};
const Segment SGI338857[] = {{&S23698[12], 14}, {&S23698[37], 42}, {NULL}};
const Segment SGI338858[] = {{&S23698[12], 14}, {NULL}};
const Segment SGI338859[] = {{&S49720[5], 5}, {&S49720[105], 5}, {NULL}};
const Segment SGI338860[] = {{&S49720[69], 3}, {NULL}};
const Segment SGI338861[] = {{&S49720[131], 3}, {NULL}};
const Segment SGI338862[] = {{&S23699[6], 4}, {NULL}};
const Segment SGI338863[] = {{&S23699[75], 5}, {NULL}};
const Segment SGI338864[] = {{&S23699[12], 14}, {&S23699[37], 43}, {NULL}};
const Segment SGI338865[] = {{&S23699[12], 14}, {NULL}};
const Segment SGI338866[] = {{&S49721[5], 5}, {&S49721[97], 5}, {NULL}};
const Segment SGI338867[] = {{&S23700[6], 4}, {NULL}};
const Segment SGI338868[] = {{&S49721[113], 3}, {NULL}};
const Segment SGI338869[] = {{&S23700[75], 5}, {NULL}};
const Segment SGI338870[] = {{&S23700[12], 14}, {&S23700[37], 43}, {NULL}};
const Segment SGI338871[] = {{&S23700[12], 14}, {NULL}};
const Segment SGI338872[] = {{&S23701[6], 4}, {NULL}};
const Segment SGI338873[] = {{&S23701[75], 5}, {NULL}};
const Segment SGI338874[] = {{&S23701[12], 14}, {&S23701[38], 42}, {NULL}};
const Segment SGI338875[] = {{&S23701[12], 14}, {NULL}};
const Segment SGI338876[] = {{&S23702[6], 4}, {NULL}};
const Segment SGI338877[] = {{&S23702[75], 5}, {NULL}};
const Segment SGI338878[] = {{&S23702[12], 14}, {&S23702[38], 42}, {NULL}};
const Segment SGI338879[] = {{&S23702[12], 14}, {NULL}};
const Segment SGI338880[] = {{&S23703[6], 4}, {NULL}};
const Segment SGI338881[] = {{&S23703[74], 5}, {NULL}};
const Segment SGI338882[] = {{&S23703[12], 14}, {&S23703[37], 42}, {NULL}};
const Segment SGI338883[] = {{&S23703[12], 14}, {NULL}};
const Segment SGI338884[] = {{&S23704[6], 4}, {NULL}};
const Segment SGI338885[] = {{&S23704[74], 5}, {NULL}};
const Segment SGI338886[] = {{&S23704[12], 14}, {&S23704[37], 42}, {NULL}};
const Segment SGI338887[] = {{&S23704[12], 14}, {NULL}};
const Segment SGI338888[] = {{&S23705[6], 4}, {NULL}};
const Segment SGI338889[] = {{&S23705[75], 5}, {NULL}};
const Segment SGI338890[] = {{&S23705[12], 14}, {&S23705[38], 42}, {NULL}};
const Segment SGI338891[] = {{&S23705[12], 14}, {NULL}};
const Segment SGI338892[] = {{&S23706[6], 4}, {NULL}};
const Segment SGI338893[] = {{&S23706[74], 5}, {NULL}};
const Segment SGI338894[] = {{&S23706[12], 14}, {&S23706[37], 42}, {NULL}};
const Segment SGI338895[] = {{&S23706[12], 14}, {NULL}};
const Segment SGI338896[] = {{&S23707[6], 4}, {NULL}};
const Segment SGI338897[] = {{&S23707[75], 5}, {NULL}};
const Segment SGI338898[] = {{&S23707[12], 14}, {&S23707[38], 42}, {NULL}};
const Segment SGI338899[] = {{&S23707[12], 14}, {NULL}};
const Segment SGI338900[] = {{&S23708[6], 4}, {NULL}};
const Segment SGI338901[] = {{&S23708[74], 5}, {NULL}};
const Segment SGI338902[] = {{&S23708[12], 14}, {&S23708[37], 42}, {NULL}};
const Segment SGI338903[] = {{&S23708[12], 14}, {NULL}};
const Segment SGI338904[] = {{&S23709[6], 4}, {NULL}};
const Segment SGI338905[] = {{&S23709[74], 5}, {NULL}};
const Segment SGI338906[] = {{&S23709[12], 14}, {&S23709[37], 42}, {NULL}};
const Segment SGI338907[] = {{&S23709[12], 14}, {NULL}};
const Segment SGI338908[] = {{&S23710[6], 4}, {NULL}};
const Segment SGI338909[] = {{&S23710[74], 5}, {NULL}};
const Segment SGI338910[] = {{&S23710[12], 14}, {&S23710[37], 42}, {NULL}};
const Segment SGI338911[] = {{&S23710[12], 14}, {NULL}};
const Segment SGI338912[] = {{&S23711[6], 4}, {NULL}};
const Segment SGI338913[] = {{&S23711[74], 5}, {NULL}};
const Segment SGI338914[] = {{&S23711[12], 14}, {&S23711[37], 42}, {NULL}};
const Segment SGI338915[] = {{&S23711[12], 14}, {NULL}};
const Segment SGI338916[] = {{&S23714[6], 4}, {NULL}};
const Segment SGI338917[] = {{&S23714[75], 5}, {NULL}};
const Segment SGI338918[] = {{&S23714[12], 14}, {&S23714[37], 43}, {NULL}};
const Segment SGI338919[] = {{&S23714[12], 14}, {NULL}};
const Segment SGI338920[] = {{&S23715[6], 4}, {NULL}};
const Segment SGI338921[] = {{&S23715[64], 5}, {NULL}};
const Segment SGI338922[] = {{&S23715[12], 14}, {NULL}};
const Segment SGI338923[] = {{&S23716[6], 4}, {NULL}};
const Segment SGI338924[] = {{&S23716[74], 5}, {NULL}};
const Segment SGI338925[] = {{&S23716[12], 14}, {&S23716[37], 42}, {NULL}};
const Segment SGI338926[] = {{&S23716[12], 14}, {NULL}};
const Segment SGI338927[] = {{&S23717[6], 4}, {NULL}};
const Segment SGI338928[] = {{&S23717[75], 5}, {NULL}};
const Segment SGI338929[] = {{&S23717[12], 14}, {&S23717[38], 42}, {NULL}};
const Segment SGI338930[] = {{&S23717[12], 14}, {NULL}};
const Segment SGI338931[] = {{&S23718[6], 4}, {NULL}};
const Segment SGI338932[] = {{&S23718[75], 5}, {NULL}};
const Segment SGI338933[] = {{&S23718[12], 14}, {&S23718[38], 42}, {NULL}};
const Segment SGI338934[] = {{&S23718[12], 14}, {NULL}};
const Segment SGI338935[] = {{&S23719[6], 4}, {NULL}};
const Segment SGI338936[] = {{&S23719[74], 5}, {NULL}};
const Segment SGI338937[] = {{&S23719[12], 14}, {&S23719[37], 42}, {NULL}};
const Segment SGI338938[] = {{&S23719[12], 14}, {NULL}};
const Segment SGI338939[] = {{&S23720[6], 4}, {NULL}};
const Segment SGI338940[] = {{&S23720[52], 5}, {NULL}};
const Segment SGI338941[] = {{&S23720[12], 45}, {NULL}};
const Segment SGI338942[] = {{&S23720[12], 14}, {NULL}};
const Segment SGI338943[] = {{&S23721[6], 4}, {NULL}};
const Segment SGI338944[] = {{&S23721[74], 5}, {NULL}};
const Segment SGI338945[] = {{&S23721[12], 14}, {&S23721[37], 42}, {NULL}};
const Segment SGI338946[] = {{&S23721[12], 14}, {NULL}};
const Segment SGI338947[] = {{&S23722[6], 4}, {NULL}};
const Segment SGI338948[] = {{&S23722[74], 5}, {NULL}};
const Segment SGI338949[] = {{&S23722[12], 14}, {&S23722[37], 42}, {NULL}};
const Segment SGI338950[] = {{&S23722[12], 14}, {NULL}};
const Segment SGI338951[] = {{&S23723[6], 4}, {NULL}};
const Segment SGI338952[] = {{&S23723[52], 5}, {NULL}};
const Segment SGI338953[] = {{&S23723[12], 55}, {NULL}};
const Segment SGI338954[] = {{&S23723[12], 14}, {NULL}};
const Segment SGI338955[] = {{&S23724[6], 4}, {NULL}};
const Segment SGI338956[] = {{&S23724[74], 5}, {NULL}};
const Segment SGI338957[] = {{&S23724[12], 14}, {&S23724[37], 42}, {NULL}};
const Segment SGI338958[] = {{&S23724[12], 14}, {NULL}};
const Segment SGI338959[] = {{&S23725[6], 4}, {NULL}};
const Segment SGI338960[] = {{&S23725[74], 5}, {NULL}};
const Segment SGI338961[] = {{&S23725[12], 14}, {&S23725[37], 42}, {NULL}};
const Segment SGI338962[] = {{&S23725[12], 14}, {NULL}};
const Segment SGI338963[] = {{&S23726[6], 4}, {NULL}};
const Segment SGI338964[] = {{&S23726[75], 5}, {NULL}};
const Segment SGI338965[] = {{&S23726[12], 14}, {&S23726[38], 42}, {NULL}};
const Segment SGI338966[] = {{&S23726[12], 14}, {NULL}};
const Segment SGI338967[] = {{&S23727[6], 4}, {NULL}};
const Segment SGI338968[] = {{&S23727[74], 5}, {NULL}};
const Segment SGI338969[] = {{&S23727[12], 14}, {&S23727[37], 42}, {NULL}};
const Segment SGI338970[] = {{&S23727[12], 14}, {NULL}};
const Segment SGI338971[] = {{&S23728[6], 4}, {NULL}};
const Segment SGI338972[] = {{&S23728[75], 5}, {NULL}};
const Segment SGI338973[] = {{&S23728[12], 14}, {&S23728[37], 43}, {NULL}};
const Segment SGI338974[] = {{&S23728[12], 14}, {NULL}};
const Segment SGI338975[] = {{&S23729[6], 4}, {NULL}};
const Segment SGI338976[] = {{&S23729[74], 5}, {NULL}};
const Segment SGI338977[] = {{&S23729[12], 14}, {&S23729[37], 42}, {NULL}};
const Segment SGI338978[] = {{&S23729[12], 14}, {NULL}};
const Segment SGI338979[] = {{&S23730[6], 4}, {NULL}};
const Segment SGI338980[] = {{&S23730[74], 5}, {NULL}};
const Segment SGI338981[] = {{&S23730[12], 14}, {&S23730[37], 42}, {NULL}};
const Segment SGI338982[] = {{&S23730[12], 14}, {NULL}};
const Segment SGI338983[] = {{&S23731[6], 4}, {NULL}};
const Segment SGI338984[] = {{&S23731[74], 5}, {NULL}};
const Segment SGI338985[] = {{&S23731[12], 14}, {&S23731[37], 42}, {NULL}};
const Segment SGI338986[] = {{&S23731[12], 14}, {NULL}};
const Segment SGI338987[] = {{&S23732[6], 4}, {NULL}};
const Segment SGI338988[] = {{&S23732[75], 5}, {NULL}};
const Segment SGI338989[] = {{&S23732[12], 14}, {&S23732[37], 43}, {NULL}};
const Segment SGI338990[] = {{&S23732[12], 14}, {NULL}};
const Segment SGI338991[] = {{&S23733[6], 4}, {NULL}};
const Segment SGI338992[] = {{&S23733[74], 5}, {NULL}};
const Segment SGI338993[] = {{&S23733[12], 14}, {&S23733[37], 42}, {NULL}};
const Segment SGI338994[] = {{&S23733[12], 14}, {NULL}};
const Segment SGI338995[] = {{&S23734[6], 4}, {NULL}};
const Segment SGI338996[] = {{&S23734[74], 5}, {NULL}};
const Segment SGI338997[] = {{&S23734[12], 14}, {&S23734[37], 42}, {NULL}};
const Segment SGI338998[] = {{&S23734[12], 14}, {NULL}};
const Segment SGI338999[] = {{&S23735[6], 4}, {NULL}};
const Segment SGI339000[] = {{&S23735[74], 5}, {NULL}};
const Segment SGI339001[] = {{&S23735[12], 14}, {&S23735[37], 42}, {NULL}};
const Segment SGI339002[] = {{&S23735[12], 14}, {NULL}};
const Segment SGI339003[] = {{&S23736[6], 4}, {NULL}};
const Segment SGI339004[] = {{&S23736[74], 5}, {NULL}};
const Segment SGI339005[] = {{&S23736[12], 14}, {&S23736[37], 42}, {NULL}};
const Segment SGI339006[] = {{&S23736[12], 14}, {NULL}};
const Segment SGI339007[] = {{&S23737[6], 4}, {NULL}};
const Segment SGI339008[] = {{&S23737[74], 5}, {NULL}};
const Segment SGI339009[] = {{&S23737[12], 14}, {&S23737[37], 42}, {NULL}};
const Segment SGI339010[] = {{&S23737[12], 14}, {NULL}};
const Segment SGI339011[] = {{&S23738[6], 4}, {NULL}};
const Segment SGI339012[] = {{&S23738[74], 5}, {NULL}};
const Segment SGI339013[] = {{&S23738[12], 14}, {&S23738[37], 42}, {NULL}};
const Segment SGI339014[] = {{&S23738[12], 14}, {NULL}};
const Segment SGI339015[] = {{&S23739[6], 4}, {NULL}};
const Segment SGI339016[] = {{&S23739[75], 5}, {NULL}};
const Segment SGI339017[] = {{&S23739[12], 14}, {&S23739[38], 42}, {NULL}};
const Segment SGI339018[] = {{&S23739[12], 14}, {NULL}};
const Segment SGI339019[] = {{&S23740[6], 4}, {NULL}};
const Segment SGI339020[] = {{&S23740[74], 5}, {NULL}};
const Segment SGI339021[] = {{&S23740[12], 14}, {&S23740[37], 42}, {NULL}};
const Segment SGI339022[] = {{&S23740[12], 14}, {NULL}};
const Segment SGI339023[] = {{&S49771[41], 5}, {NULL}};
const Segment SGI339024[] = {{&S49771[6], 3}, {NULL}};
const Segment SGI339025[] = {{&S49771[67], 3}, {NULL}};
const Segment SGI339026[] = {{&S23741[6], 4}, {NULL}};
const Segment SGI339027[] = {{&S23741[74], 5}, {NULL}};
const Segment SGI339028[] = {{&S23741[12], 14}, {&S23741[37], 42}, {NULL}};
const Segment SGI339029[] = {{&S23741[12], 14}, {NULL}};
const Segment SGI339030[] = {{&S49772[5], 5}, {NULL}};
const Segment SGI339031[] = {{&S49772[16], 3}, {NULL}};
const Segment SGI339032[] = {{&S23742[6], 4}, {NULL}};
const Segment SGI339033[] = {{&S23742[75], 5}, {NULL}};
const Segment SGI339034[] = {{&S23742[12], 14}, {&S23742[38], 42}, {NULL}};
const Segment SGI339035[] = {{&S23742[12], 14}, {NULL}};
const Segment SGI339036[] = {{&S49773[38], 5}, {NULL}};
const Segment SGI339037[] = {{&S49773[94], 3}, {NULL}};
const Segment SGI339038[] = {{&S23743[6], 4}, {NULL}};
const Segment SGI339039[] = {{&S23743[75], 5}, {NULL}};
const Segment SGI339040[] = {{&S23743[12], 14}, {&S23743[38], 42}, {NULL}};
const Segment SGI339041[] = {{&S23743[12], 14}, {NULL}};
const Segment SGI339042[] = {{&S49774[5], 14}, {NULL}};
const Segment SGI339043[] = {{&S49774[20], 3}, {NULL}};
const Segment SGI339044[] = {{&S23744[6], 4}, {NULL}};
const Segment SGI339045[] = {{&S23744[74], 5}, {NULL}};
const Segment SGI339046[] = {{&S23744[12], 14}, {&S23744[37], 42}, {NULL}};
const Segment SGI339047[] = {{&S23744[12], 14}, {NULL}};
const Segment SGI339048[] = {{&S23745[6], 4}, {NULL}};
const Segment SGI339049[] = {{&S23745[74], 5}, {NULL}};
const Segment SGI339050[] = {{&S23745[12], 14}, {&S23745[37], 42}, {NULL}};
const Segment SGI339051[] = {{&S23745[12], 14}, {NULL}};
const Segment SGI339052[] = {{&S23746[6], 4}, {NULL}};
const Segment SGI339053[] = {{&S23746[75], 5}, {NULL}};
const Segment SGI339054[] = {{&S23746[12], 14}, {&S23746[37], 43}, {NULL}};
const Segment SGI339055[] = {{&S23746[12], 14}, {NULL}};
const Segment SGI339056[] = {{&S23747[6], 4}, {NULL}};
const Segment SGI339057[] = {{&S23747[75], 5}, {NULL}};
const Segment SGI339058[] = {{&S23747[12], 14}, {&S23747[37], 43}, {NULL}};
const Segment SGI339059[] = {{&S23747[12], 14}, {NULL}};
const Segment SGI339060[] = {{&S23748[6], 4}, {NULL}};
const Segment SGI339061[] = {{&S23748[74], 5}, {NULL}};
const Segment SGI339062[] = {{&S23748[12], 14}, {&S23748[37], 42}, {NULL}};
const Segment SGI339063[] = {{&S23748[12], 14}, {NULL}};
const Segment SGI339064[] = {{&S23749[6], 4}, {NULL}};
const Segment SGI339065[] = {{&S23749[74], 5}, {NULL}};
const Segment SGI339066[] = {{&S23749[12], 14}, {&S23749[37], 42}, {NULL}};
const Segment SGI339067[] = {{&S23749[12], 14}, {NULL}};
const Segment SGI339068[] = {{&S23750[6], 4}, {NULL}};
const Segment SGI339069[] = {{&S23750[75], 5}, {NULL}};
const Segment SGI339070[] = {{&S23750[12], 14}, {&S23750[37], 43}, {NULL}};
const Segment SGI339071[] = {{&S23750[12], 14}, {NULL}};
const Segment SGI339072[] = {{&S23751[6], 4}, {NULL}};
const Segment SGI339073[] = {{&S23751[74], 5}, {NULL}};
const Segment SGI339074[] = {{&S23751[12], 14}, {&S23751[37], 42}, {NULL}};
const Segment SGI339075[] = {{&S23751[12], 14}, {NULL}};
const Segment SGI339076[] = {{&S23752[6], 4}, {NULL}};
const Segment SGI339077[] = {{&S23752[75], 5}, {NULL}};
const Segment SGI339078[] = {{&S23752[12], 14}, {&S23752[37], 43}, {NULL}};
const Segment SGI339079[] = {{&S23752[12], 14}, {NULL}};
const Segment SGI339080[] = {{&S23753[6], 4}, {NULL}};
const Segment SGI339081[] = {{&S23753[74], 5}, {NULL}};
const Segment SGI339082[] = {{&S23753[12], 14}, {&S23753[37], 42}, {NULL}};
const Segment SGI339083[] = {{&S23753[12], 14}, {NULL}};
const Segment SGI339084[] = {{&S23754[6], 4}, {NULL}};
const Segment SGI339085[] = {{&S23754[74], 5}, {NULL}};
const Segment SGI339086[] = {{&S23754[12], 14}, {&S23754[37], 42}, {NULL}};
const Segment SGI339087[] = {{&S23754[12], 14}, {NULL}};
const Segment SGI339088[] = {{&S23755[6], 4}, {NULL}};
const Segment SGI339089[] = {{&S23755[74], 5}, {NULL}};
const Segment SGI339090[] = {{&S23755[12], 14}, {&S23755[37], 42}, {NULL}};
const Segment SGI339091[] = {{&S23755[12], 14}, {NULL}};
const Segment SGI339092[] = {{&S23756[6], 4}, {NULL}};
const Segment SGI339093[] = {{&S23756[74], 5}, {NULL}};
const Segment SGI339094[] = {{&S23756[12], 14}, {&S23756[37], 42}, {NULL}};
const Segment SGI339095[] = {{&S23756[12], 14}, {NULL}};
const Segment SGI339096[] = {{&S23757[6], 4}, {NULL}};
const Segment SGI339097[] = {{&S23757[101], 5}, {NULL}};
const Segment SGI339098[] = {{&S23757[12], 14}, {&S23757[37], 12}, {NULL}};
const Segment SGI339099[] = {{&S23757[12], 14}, {NULL}};
const Segment SGI339100[] = {{&S23758[6], 4}, {NULL}};
const Segment SGI339101[] = {{&S23758[75], 5}, {NULL}};
const Segment SGI339102[] = {{&S23758[12], 14}, {&S23758[37], 43}, {NULL}};
const Segment SGI339103[] = {{&S23758[12], 14}, {NULL}};
const Segment SGI339104[] = {{&S49794[5], 3}, {NULL}};
const Segment SGI339105[] = {{&S23759[6], 4}, {NULL}};
const Segment SGI339106[] = {{&S23759[52], 5}, {NULL}};
const Segment SGI339107[] = {{&S23759[12], 62}, {NULL}};
const Segment SGI339108[] = {{&S23759[12], 14}, {NULL}};
const Segment SGI339109[] = {{&S49795[5], 3}, {&S49795[5], 3}, {NULL}};
const Segment SGI339110[] = {{&S23760[6], 4}, {NULL}};
const Segment SGI339111[] = {{&S23760[75], 5}, {NULL}};
const Segment SGI339112[] = {{&S23760[12], 14}, {&S23760[38], 42}, {NULL}};
const Segment SGI339113[] = {{&S23760[12], 14}, {NULL}};
const Segment SGI339114[] = {{&S23761[6], 4}, {NULL}};
const Segment SGI339115[] = {{&S23761[73], 5}, {NULL}};
const Segment SGI339116[] = {{&S23761[12], 14}, {&S23761[84], 4}, {NULL}};
const Segment SGI339117[] = {{&S23761[12], 14}, {NULL}};
const Segment SGI339118[] = {{&S23762[6], 4}, {NULL}};
const Segment SGI339119[] = {{&S23762[75], 5}, {NULL}};
const Segment SGI339120[] = {{&S23762[12], 14}, {&S23762[37], 43}, {NULL}};
const Segment SGI339121[] = {{&S23762[12], 14}, {NULL}};
const Segment SGI339122[] = {{&S23763[6], 4}, {NULL}};
const Segment SGI339123[] = {{&S23763[74], 5}, {NULL}};
const Segment SGI339124[] = {{&S23763[12], 14}, {&S23763[37], 42}, {NULL}};
const Segment SGI339125[] = {{&S23763[12], 14}, {NULL}};
const Segment SGI339126[] = {{&S23764[6], 4}, {NULL}};
const Segment SGI339127[] = {{&S23764[74], 5}, {NULL}};
const Segment SGI339128[] = {{&S23764[12], 14}, {&S23764[37], 42}, {NULL}};
const Segment SGI339129[] = {{&S23764[12], 14}, {NULL}};
const Segment SGI339130[] = {{&S23765[6], 4}, {NULL}};
const Segment SGI339131[] = {{&S23765[74], 5}, {NULL}};
const Segment SGI339132[] = {{&S23765[12], 14}, {&S23765[37], 42}, {NULL}};
const Segment SGI339133[] = {{&S23765[12], 14}, {NULL}};
const Segment SGI339134[] = {{&S23766[6], 4}, {NULL}};
const Segment SGI339135[] = {{&S23766[74], 5}, {NULL}};
const Segment SGI339136[] = {{&S23766[12], 14}, {&S23766[37], 42}, {NULL}};
const Segment SGI339137[] = {{&S23766[12], 14}, {NULL}};
const Segment SGI339138[] = {{&S23767[6], 4}, {NULL}};
const Segment SGI339139[] = {{&S23767[75], 5}, {NULL}};
const Segment SGI339140[] = {{&S23767[12], 14}, {&S23767[37], 43}, {NULL}};
const Segment SGI339141[] = {{&S23767[12], 14}, {NULL}};
const Segment SGI339142[] = {{&S23768[6], 4}, {NULL}};
const Segment SGI339143[] = {{&S23768[75], 5}, {NULL}};
const Segment SGI339144[] = {{&S23768[12], 14}, {&S23768[37], 43}, {NULL}};
const Segment SGI339145[] = {{&S23768[12], 14}, {NULL}};
const Segment SGI339146[] = {{&S23769[6], 4}, {NULL}};
const Segment SGI339147[] = {{&S23769[75], 5}, {NULL}};
const Segment SGI339148[] = {{&S23769[12], 14}, {&S23769[38], 42}, {NULL}};
const Segment SGI339149[] = {{&S23769[12], 14}, {NULL}};
const Segment SGI339150[] = {{&S23770[6], 4}, {NULL}};
const Segment SGI339151[] = {{&S23770[74], 5}, {NULL}};
const Segment SGI339152[] = {{&S23770[12], 14}, {&S23770[37], 42}, {NULL}};
const Segment SGI339153[] = {{&S23770[12], 14}, {NULL}};
const Segment SGI339154[] = {{&S23771[6], 4}, {NULL}};
const Segment SGI339155[] = {{&S23771[74], 5}, {NULL}};
const Segment SGI339156[] = {{&S23771[12], 14}, {&S23771[37], 42}, {NULL}};
const Segment SGI339157[] = {{&S23771[12], 14}, {NULL}};
const Segment SGI339158[] = {{&S23772[6], 4}, {NULL}};
const Segment SGI339159[] = {{&S23772[75], 5}, {NULL}};
const Segment SGI339160[] = {{&S23772[12], 14}, {&S23772[38], 42}, {NULL}};
const Segment SGI339161[] = {{&S23772[12], 14}, {NULL}};
const Segment SGI339162[] = {{&S23773[6], 4}, {NULL}};
const Segment SGI339163[] = {{&S23773[75], 5}, {NULL}};
const Segment SGI339164[] = {{&S23773[12], 14}, {&S23773[38], 42}, {NULL}};
const Segment SGI339165[] = {{&S23773[12], 14}, {NULL}};
const Segment SGI339166[] = {{&S23774[6], 4}, {NULL}};
const Segment SGI339167[] = {{&S23774[74], 5}, {NULL}};
const Segment SGI339168[] = {{&S23774[12], 14}, {&S23774[37], 42}, {NULL}};
const Segment SGI339169[] = {{&S23774[12], 14}, {NULL}};
const Segment SGI339170[] = {{&S23775[6], 4}, {NULL}};
const Segment SGI339171[] = {{&S23775[74], 5}, {NULL}};
const Segment SGI339172[] = {{&S23775[12], 14}, {&S23775[37], 42}, {NULL}};
const Segment SGI339173[] = {{&S23775[12], 14}, {NULL}};
const Segment SGI339174[] = {{&S23776[6], 4}, {NULL}};
const Segment SGI339175[] = {{&S23776[75], 5}, {NULL}};
const Segment SGI339176[] = {{&S23776[12], 14}, {&S23776[37], 43}, {NULL}};
const Segment SGI339177[] = {{&S23776[12], 14}, {NULL}};
const Segment SGI339178[] = {{&S23777[6], 4}, {NULL}};
const Segment SGI339179[] = {{&S23777[75], 5}, {NULL}};
const Segment SGI339180[] = {{&S23777[12], 14}, {&S23777[38], 42}, {NULL}};
const Segment SGI339181[] = {{&S23777[12], 14}, {NULL}};
const Segment SGI339182[] = {{&S23778[6], 4}, {NULL}};
const Segment SGI339183[] = {{&S23778[102], 5}, {NULL}};
const Segment SGI339184[] = {{&S23778[12], 14}, {&S23778[37], 12}, {NULL}};
const Segment SGI339185[] = {{&S23778[12], 14}, {NULL}};
const Segment SGI339186[] = {{&S23779[6], 4}, {NULL}};
const Segment SGI339187[] = {{&S23779[74], 5}, {NULL}};
const Segment SGI339188[] = {{&S23779[12], 14}, {&S23779[37], 42}, {NULL}};
const Segment SGI339189[] = {{&S23779[12], 14}, {NULL}};
const Segment SGI339190[] = {{&S40961[28], 3}, {NULL}};
const Segment SGI339191[] = {{&S40961[102], 5}, {NULL}};
const Segment SGI339192[] = {{&S40962[28], 3}, {NULL}};
const Segment SGI339193[] = {{&S40962[102], 5}, {NULL}};
const Segment SGI339194[] = {{&S49881[5], 3}, {NULL}};
const Segment SGI339195[] = {{&S25716[61], 5}, {NULL}};
const Segment SGI339196[] = {{&S25716[13], 10}, {NULL}};
const Segment SGI339197[] = {{&S23780[6], 4}, {NULL}};
const Segment SGI339198[] = {{&S23780[75], 5}, {NULL}};
const Segment SGI339199[] = {{&S23780[12], 14}, {&S23780[37], 43}, {NULL}};
const Segment SGI339200[] = {{&S23780[12], 14}, {NULL}};
const Segment SGI339201[] = {{&S23781[6], 4}, {NULL}};
const Segment SGI339202[] = {{&S23781[75], 5}, {NULL}};
const Segment SGI339203[] = {{&S23781[12], 14}, {&S23781[38], 42}, {NULL}};
const Segment SGI339204[] = {{&S23781[12], 14}, {NULL}};
const Segment SGI339205[] = {{&S23782[6], 4}, {NULL}};
const Segment SGI339206[] = {{&S23782[75], 5}, {NULL}};
const Segment SGI339207[] = {{&S23782[12], 14}, {&S23782[38], 42}, {NULL}};
const Segment SGI339208[] = {{&S23782[12], 14}, {NULL}};
const Segment SGI339209[] = {{&S23783[6], 4}, {NULL}};
const Segment SGI339210[] = {{&S23783[52], 5}, {NULL}};
const Segment SGI339211[] = {{&S23783[12], 84}, {NULL}};
const Segment SGI339212[] = {{&S23783[12], 14}, {NULL}};
const Segment SGI339213[] = {{&S23784[6], 4}, {NULL}};
const Segment SGI339214[] = {{&S23784[75], 5}, {NULL}};
const Segment SGI339215[] = {{&S23784[12], 14}, {&S23784[38], 42}, {NULL}};
const Segment SGI339216[] = {{&S23784[12], 14}, {NULL}};
const Segment SGI339217[] = {{&S23785[6], 4}, {NULL}};
const Segment SGI339218[] = {{&S23785[74], 5}, {NULL}};
const Segment SGI339219[] = {{&S23785[12], 14}, {&S23785[37], 42}, {NULL}};
const Segment SGI339220[] = {{&S23785[12], 14}, {NULL}};
const Segment SGI339221[] = {{&S23786[6], 4}, {NULL}};
const Segment SGI339222[] = {{&S23786[74], 5}, {NULL}};
const Segment SGI339223[] = {{&S23786[12], 14}, {&S23786[37], 42}, {NULL}};
const Segment SGI339224[] = {{&S23786[12], 14}, {NULL}};
const Segment SGI339225[] = {{&S23787[6], 4}, {NULL}};
const Segment SGI339226[] = {{&S23787[75], 5}, {NULL}};
const Segment SGI339227[] = {{&S23787[12], 14}, {&S23787[37], 43}, {NULL}};
const Segment SGI339228[] = {{&S23787[12], 14}, {NULL}};
const Segment SGI339229[] = {{&S23788[6], 4}, {NULL}};
const Segment SGI339230[] = {{&S23788[102], 5}, {NULL}};
const Segment SGI339231[] = {{&S23788[12], 14}, {&S23788[37], 12}, {NULL}};
const Segment SGI339232[] = {{&S23788[12], 14}, {NULL}};
const Segment SGI339233[] = {{&S23789[6], 4}, {NULL}};
const Segment SGI339234[] = {{&S23789[74], 5}, {NULL}};
const Segment SGI339235[] = {{&S23789[12], 14}, {&S23789[37], 42}, {NULL}};
const Segment SGI339236[] = {{&S23789[12], 14}, {NULL}};
const Segment SGI339237[] = {{&S23790[6], 4}, {NULL}};
const Segment SGI339238[] = {{&S23790[74], 5}, {NULL}};
const Segment SGI339239[] = {{&S23790[12], 14}, {&S23790[37], 42}, {NULL}};
const Segment SGI339240[] = {{&S23790[12], 14}, {NULL}};
const Segment SGI339241[] = {{&S23791[6], 4}, {NULL}};
const Segment SGI339242[] = {{&S23791[74], 5}, {NULL}};
const Segment SGI339243[] = {{&S23791[12], 14}, {&S23791[37], 42}, {NULL}};
const Segment SGI339244[] = {{&S23791[12], 14}, {NULL}};
const Segment SGI339245[] = {{&S23792[6], 4}, {NULL}};
const Segment SGI339246[] = {{&S23792[74], 5}, {NULL}};
const Segment SGI339247[] = {{&S23792[12], 14}, {&S23792[37], 42}, {NULL}};
const Segment SGI339248[] = {{&S23792[12], 14}, {NULL}};
const Segment SGI339249[] = {{&S23793[6], 4}, {NULL}};
const Segment SGI339250[] = {{&S23793[74], 5}, {NULL}};
const Segment SGI339251[] = {{&S23793[12], 14}, {&S23793[37], 42}, {NULL}};
const Segment SGI339252[] = {{&S23793[12], 14}, {NULL}};
const Segment SGI339253[] = {{&S23794[6], 4}, {NULL}};
const Segment SGI339254[] = {{&S23794[74], 5}, {NULL}};
const Segment SGI339255[] = {{&S23794[12], 14}, {&S23794[37], 42}, {NULL}};
const Segment SGI339256[] = {{&S23794[12], 14}, {NULL}};
const Segment SGI339257[] = {{&S23795[6], 4}, {NULL}};
const Segment SGI339258[] = {{&S23795[75], 5}, {NULL}};
const Segment SGI339259[] = {{&S23795[12], 14}, {&S23795[37], 43}, {NULL}};
const Segment SGI339260[] = {{&S23795[12], 14}, {NULL}};
const Segment SGI339261[] = {{&S23796[6], 4}, {NULL}};
const Segment SGI339262[] = {{&S23796[74], 5}, {NULL}};
const Segment SGI339263[] = {{&S23796[12], 14}, {&S23796[37], 42}, {NULL}};
const Segment SGI339264[] = {{&S23796[12], 14}, {NULL}};
const Segment SGI339265[] = {{&S23797[6], 4}, {NULL}};
const Segment SGI339266[] = {{&S23797[74], 5}, {NULL}};
const Segment SGI339267[] = {{&S23797[12], 14}, {&S23797[37], 42}, {NULL}};
const Segment SGI339268[] = {{&S23797[12], 14}, {NULL}};
const Segment SGI339269[] = {{&S23798[6], 4}, {NULL}};
const Segment SGI339270[] = {{&S23798[74], 5}, {NULL}};
const Segment SGI339271[] = {{&S23798[12], 14}, {&S23798[37], 42}, {NULL}};
const Segment SGI339272[] = {{&S23798[12], 14}, {NULL}};
const Segment SGI339273[] = {{&S23799[6], 4}, {NULL}};
const Segment SGI339274[] = {{&S23799[74], 5}, {NULL}};
const Segment SGI339275[] = {{&S23799[12], 14}, {&S23799[37], 42}, {NULL}};
const Segment SGI339276[] = {{&S23799[12], 14}, {NULL}};
const Segment SGI339277[] = {{&S23800[6], 4}, {NULL}};
const Segment SGI339278[] = {{&S23800[52], 5}, {NULL}};
const Segment SGI339279[] = {{&S23800[12], 59}, {NULL}};
const Segment SGI339280[] = {{&S23800[12], 14}, {NULL}};
const Segment SGI339281[] = {{&S23801[6], 4}, {NULL}};
const Segment SGI339282[] = {{&S23801[74], 5}, {NULL}};
const Segment SGI339283[] = {{&S23801[12], 14}, {&S23801[37], 42}, {NULL}};
const Segment SGI339284[] = {{&S23801[12], 14}, {NULL}};
const Segment SGI339285[] = {{&S23802[6], 4}, {NULL}};
const Segment SGI339286[] = {{&S23802[75], 5}, {NULL}};
const Segment SGI339287[] = {{&S23802[12], 14}, {&S23802[38], 42}, {NULL}};
const Segment SGI339288[] = {{&S23802[12], 14}, {NULL}};
const Segment SGI339289[] = {{&S23803[6], 4}, {NULL}};
const Segment SGI339290[] = {{&S23803[75], 5}, {NULL}};
const Segment SGI339291[] = {{&S23803[12], 14}, {&S23803[38], 42}, {NULL}};
const Segment SGI339292[] = {{&S23803[12], 14}, {NULL}};
const Segment SGI339293[] = {{&S23804[6], 4}, {NULL}};
const Segment SGI339294[] = {{&S23804[74], 5}, {NULL}};
const Segment SGI339295[] = {{&S23804[12], 14}, {&S23804[37], 42}, {NULL}};
const Segment SGI339296[] = {{&S23804[12], 14}, {NULL}};
const Segment SGI339297[] = {{&S23805[6], 4}, {NULL}};
const Segment SGI339298[] = {{&S23805[74], 5}, {NULL}};
const Segment SGI339299[] = {{&S23805[12], 14}, {&S23805[37], 42}, {NULL}};
const Segment SGI339300[] = {{&S23805[12], 14}, {NULL}};
const Segment SGI339301[] = {{&S23806[6], 4}, {NULL}};
const Segment SGI339302[] = {{&S23806[74], 5}, {NULL}};
const Segment SGI339303[] = {{&S23806[12], 14}, {&S23806[37], 42}, {NULL}};
const Segment SGI339304[] = {{&S23806[12], 14}, {NULL}};
const Segment SGI339305[] = {{&S23807[6], 4}, {NULL}};
const Segment SGI339306[] = {{&S23807[74], 5}, {NULL}};
const Segment SGI339307[] = {{&S23807[12], 14}, {&S23807[37], 42}, {NULL}};
const Segment SGI339308[] = {{&S23807[12], 14}, {NULL}};
const Segment SGI339309[] = {{&S23808[6], 4}, {NULL}};
const Segment SGI339310[] = {{&S23808[74], 5}, {NULL}};
const Segment SGI339311[] = {{&S23808[12], 14}, {&S23808[37], 42}, {NULL}};
const Segment SGI339312[] = {{&S23808[12], 14}, {NULL}};
const Segment SGI339313[] = {{&S23809[6], 4}, {NULL}};
const Segment SGI339314[] = {{&S23809[75], 5}, {NULL}};
const Segment SGI339315[] = {{&S23809[12], 14}, {&S23809[37], 43}, {NULL}};
const Segment SGI339316[] = {{&S23809[12], 14}, {NULL}};
const Segment SGI339317[] = {{&S23810[6], 4}, {NULL}};
const Segment SGI339318[] = {{&S23810[74], 5}, {NULL}};
const Segment SGI339319[] = {{&S23810[12], 14}, {&S23810[37], 42}, {NULL}};
const Segment SGI339320[] = {{&S23810[12], 14}, {NULL}};
const Segment SGI339321[] = {{&S23811[6], 4}, {NULL}};
const Segment SGI339322[] = {{&S23811[102], 5}, {NULL}};
const Segment SGI339323[] = {{&S23811[12], 14}, {&S23811[37], 12}, {NULL}};
const Segment SGI339324[] = {{&S23811[12], 14}, {NULL}};
const Segment SGI339325[] = {{&S23812[6], 4}, {NULL}};
const Segment SGI339326[] = {{&S23812[74], 5}, {NULL}};
const Segment SGI339327[] = {{&S23812[12], 14}, {&S23812[37], 42}, {NULL}};
const Segment SGI339328[] = {{&S23812[12], 14}, {NULL}};
const Segment SGI339329[] = {{&S23813[6], 4}, {NULL}};
const Segment SGI339330[] = {{&S23813[74], 5}, {NULL}};
const Segment SGI339331[] = {{&S23813[12], 14}, {&S23813[37], 42}, {NULL}};
const Segment SGI339332[] = {{&S23813[12], 14}, {NULL}};
const Segment SGI339333[] = {{&S23814[6], 4}, {NULL}};
const Segment SGI339334[] = {{&S23814[75], 5}, {NULL}};
const Segment SGI339335[] = {{&S23814[12], 14}, {&S23814[37], 43}, {NULL}};
const Segment SGI339336[] = {{&S23814[12], 14}, {NULL}};
const Segment SGI339337[] = {{&S23815[6], 4}, {NULL}};
const Segment SGI339338[] = {{&S23815[75], 5}, {NULL}};
const Segment SGI339339[] = {{&S23815[12], 14}, {&S23815[37], 43}, {NULL}};
const Segment SGI339340[] = {{&S23815[12], 14}, {NULL}};
const Segment SGI339341[] = {{&S23816[6], 4}, {NULL}};
const Segment SGI339342[] = {{&S23816[75], 5}, {NULL}};
const Segment SGI339343[] = {{&S23816[12], 14}, {&S23816[37], 43}, {NULL}};
const Segment SGI339344[] = {{&S23816[12], 14}, {NULL}};
const Segment SGI339345[] = {{&S23817[6], 4}, {NULL}};
const Segment SGI339346[] = {{&S23817[75], 5}, {NULL}};
const Segment SGI339347[] = {{&S23817[12], 14}, {&S23817[37], 43}, {NULL}};
const Segment SGI339348[] = {{&S23817[12], 14}, {NULL}};
const Segment SGI339349[] = {{&S23818[6], 4}, {NULL}};
const Segment SGI339350[] = {{&S23818[74], 5}, {NULL}};
const Segment SGI339351[] = {{&S23818[12], 14}, {&S23818[37], 42}, {NULL}};
const Segment SGI339352[] = {{&S23818[12], 14}, {NULL}};
const Segment SGI339353[] = {{&S23819[6], 4}, {NULL}};
const Segment SGI339354[] = {{&S23819[74], 5}, {NULL}};
const Segment SGI339355[] = {{&S23819[12], 14}, {&S23819[37], 42}, {NULL}};
const Segment SGI339356[] = {{&S23819[12], 14}, {NULL}};
const Segment SGI339357[] = {{&S23820[6], 4}, {NULL}};
const Segment SGI339358[] = {{&S23820[74], 5}, {NULL}};
const Segment SGI339359[] = {{&S23820[12], 14}, {&S23820[37], 42}, {NULL}};
const Segment SGI339360[] = {{&S23820[12], 14}, {NULL}};
const Segment SGI339361[] = {{&S23821[6], 4}, {NULL}};
const Segment SGI339362[] = {{&S23821[75], 5}, {NULL}};
const Segment SGI339363[] = {{&S23821[12], 14}, {&S23821[37], 43}, {NULL}};
const Segment SGI339364[] = {{&S23821[12], 14}, {NULL}};
const Segment SGI339365[] = {{&S23822[6], 4}, {NULL}};
const Segment SGI339366[] = {{&S23822[74], 5}, {NULL}};
const Segment SGI339367[] = {{&S23822[12], 14}, {&S23822[37], 42}, {NULL}};
const Segment SGI339368[] = {{&S23822[12], 14}, {NULL}};
const Segment SGI339369[] = {{&S23823[6], 4}, {NULL}};
const Segment SGI339370[] = {{&S23823[52], 5}, {NULL}};
const Segment SGI339371[] = {{&S23823[12], 65}, {NULL}};
const Segment SGI339372[] = {{&S23823[12], 14}, {NULL}};
const Segment SGI339373[] = {{&S23824[6], 4}, {NULL}};
const Segment SGI339374[] = {{&S23824[74], 5}, {NULL}};
const Segment SGI339375[] = {{&S23824[12], 14}, {&S23824[37], 42}, {NULL}};
const Segment SGI339376[] = {{&S23824[12], 14}, {NULL}};
const Segment SGI339377[] = {{&S23825[6], 4}, {NULL}};
const Segment SGI339378[] = {{&S23825[74], 5}, {NULL}};
const Segment SGI339379[] = {{&S23825[12], 14}, {&S23825[37], 42}, {NULL}};
const Segment SGI339380[] = {{&S23825[12], 14}, {NULL}};
const Segment SGI339381[] = {{&S23826[6], 4}, {NULL}};
const Segment SGI339382[] = {{&S23826[74], 5}, {NULL}};
const Segment SGI339383[] = {{&S23826[12], 14}, {&S23826[37], 42}, {NULL}};
const Segment SGI339384[] = {{&S23826[12], 14}, {NULL}};
const Segment SGI339385[] = {{&S50178[5], 7}, {&S50178[5], 7}, {NULL}};
const Segment SGI339386[] = {{&S50179[5], 7}, {NULL}};
const Segment SGI339387[] = {{&S50180[5], 5}, {NULL}};
const Segment SGI339388[] = {{&S50185[5], 3}, {&S50185[5], 3}, {NULL}};
const Segment SGI339389[] = {{&S50185[149], 3}, {NULL}};
const Segment SGI339390[] = {{&S41286[28], 5}, {NULL}};
const Segment SGI339391[] = {{&S41286[104], 5}, {NULL}};
const Segment SGI339392[] = {{&S41287[28], 7}, {NULL}};
const Segment SGI339393[] = {{&S41287[106], 5}, {NULL}};
const Segment SGI339394[] = {{&S50225[35], 3}, {NULL}};
const Segment SGI339395[] = {{&S50225[89], 3}, {NULL}};
const Segment SGI339396[] = {{&S50226[5], 12}, {NULL}};
const Segment SGI339397[] = {{&S50226[18], 3}, {NULL}};
const Segment SGI339398[] = {{&S50227[41], 5}, {NULL}};
const Segment SGI339399[] = {{&S50227[6], 3}, {NULL}};
const Segment SGI339400[] = {{&S50227[67], 3}, {NULL}};
const Segment SGI339401[] = {{&S50228[5], 3}, {NULL}};
const Segment SGI339402[] = {{&S50228[14], 3}, {NULL}};
const Segment SGI339403[] = {{&S50308[5], 5}, {&S50308[105], 5}, {NULL}};
const Segment SGI339404[] = {{&S50308[11], 3}, {NULL}};
const Segment SGI339405[] = {{&S50308[131], 3}, {NULL}};
const Segment SGI339406[] = {{&S50318[5], 4}, {NULL}};
const Segment SGI339407[] = {{&S25935[61], 5}, {NULL}};
const Segment SGI339408[] = {{&S25935[13], 10}, {NULL}};
const Segment SGI339409[] = {{&S50369[5], 5}, {&S50369[105], 5}, {NULL}};
const Segment SGI339410[] = {{&S50369[11], 3}, {NULL}};
const Segment SGI339411[] = {{&S50369[131], 3}, {NULL}};
const Segment SGI339412[] = {{&S41488[28], 5}, {NULL}};
const Segment SGI339413[] = {{&S41488[104], 5}, {NULL}};
const Segment SGI339414[] = {{&S41489[28], 5}, {NULL}};
const Segment SGI339415[] = {{&S41489[104], 5}, {NULL}};
const Segment SGI339416[] = {{&S50400[5], 5}, {&S50400[97], 5}, {NULL}};
const Segment SGI339417[] = {{&S50400[113], 3}, {NULL}};
const Segment SGI339418[] = {{&S50401[5], 4}, {NULL}};
const Segment SGI339419[] = {{&S50401[15], 3}, {NULL}};
const Segment SGI339420[] = {{&S50402[35], 4}, {NULL}};
const Segment SGI339421[] = {{&S50402[90], 3}, {NULL}};
const Segment SGI339422[] = {{&S50403[5], 13}, {NULL}};
const Segment SGI339423[] = {{&S50403[19], 3}, {NULL}};
const Segment SGI339424[] = {{&S50404[41], 4}, {NULL}};
const Segment SGI339425[] = {{&S50404[6], 3}, {NULL}};
const Segment SGI339426[] = {{&S50404[66], 3}, {NULL}};
const Segment SGI339427[] = {{&S50430[5], 5}, {&S50430[97], 5}, {NULL}};
const Segment SGI339428[] = {{&S50430[113], 3}, {NULL}};
const Segment SGI339429[] = {{&S50513[5], 5}, {&S50513[97], 5}, {NULL}};
const Segment SGI339430[] = {{&S50513[153], 3}, {NULL}};
const Segment SGI339431[] = {{&S26148[61], 5}, {NULL}};
const Segment SGI339432[] = {{&S26148[13], 10}, {NULL}};
const Segment SGI339433[] = {{&S37082[46], 4}, {NULL}};
const Segment SGI339434[] = {{&S37082[8], 3}, {NULL}};
const Segment SGI339435[] = {{&S37085[46], 4}, {&S37085[146], 7}, {NULL}};
const Segment SGI339436[] = {{&S37085[8], 3}, {NULL}};
const Segment SGI339437[] = {{&S37158[5], 5}, {&S37158[5], 5}, {NULL}};
const Segment SGI339438[] = {{&S37158[44], 3}, {NULL}};
const Segment SGI339439[] = {{&S37159[49], 6}, {NULL}};
const Segment SGI339440[] = {{&S37159[40], 3}, {NULL}};
const Segment SGI339441[] = {{&S37160[5], 6}, {&S37160[5], 6}, {NULL}};
const Segment SGI339442[] = {{&S37160[45], 3}, {NULL}};
const Segment SGI339443[] = {{&S50688[5], 5}, {&S50688[105], 5}, {NULL}};
const Segment SGI339444[] = {{&S37161[49], 5}, {NULL}};
const Segment SGI339445[] = {{&S50688[11], 3}, {NULL}};
const Segment SGI339446[] = {{&S37161[40], 3}, {NULL}};
const Segment SGI339447[] = {{&S50688[131], 3}, {NULL}};
const Segment SGI339448[] = {{&S37162[5], 12}, {NULL}};
const Segment SGI339449[] = {{&S37162[41], 3}, {NULL}};
const Segment SGI339450[] = {{&S37164[5], 13}, {NULL}};
const Segment SGI339451[] = {{&S37164[42], 3}, {NULL}};
const Segment SGI339452[] = {{&S37165[49], 12}, {NULL}};
const Segment SGI339453[] = {{&S37165[40], 3}, {NULL}};
const Segment SGI339454[] = {{&S50703[5], 14}, {NULL}};
const Segment SGI339455[] = {{&S26337[61], 5}, {NULL}};
const Segment SGI339456[] = {{&S26337[13], 10}, {NULL}};
const Segment SGI339457[] = {{&S26360[61], 5}, {NULL}};
const Segment SGI339458[] = {{&S26360[13], 10}, {NULL}};
const Segment SGI339459[] = {{&S24193[6], 4}, {NULL}};
const Segment SGI339460[] = {{&S24193[12], 14}, {NULL}};
const Segment SGI339461[] = {{&S24194[6], 4}, {NULL}};
const Segment SGI339462[] = {{&S24194[12], 14}, {NULL}};
const Segment SGI339463[] = {{&S24195[6], 4}, {NULL}};
const Segment SGI339464[] = {{&S24195[12], 14}, {NULL}};
const Segment SGI339465[] = {{&S24196[6], 4}, {NULL}};
const Segment SGI339466[] = {{&S24196[12], 14}, {NULL}};
const Segment SGI339467[] = {{&S24197[6], 4}, {NULL}};
const Segment SGI339468[] = {{&S24197[12], 14}, {NULL}};
const Segment SGI339469[] = {{&S24198[6], 4}, {NULL}};
const Segment SGI339470[] = {{&S24198[12], 14}, {NULL}};
const Segment SGI339471[] = {{&S24199[6], 4}, {NULL}};
const Segment SGI339472[] = {{&S24199[12], 14}, {NULL}};
const Segment SGI339473[] = {{&S24200[6], 4}, {NULL}};
const Segment SGI339474[] = {{&S24200[12], 14}, {NULL}};
const Segment SGI339475[] = {{&S24201[6], 4}, {NULL}};
const Segment SGI339476[] = {{&S24201[12], 14}, {NULL}};
const Segment SGI339477[] = {{&S24203[6], 4}, {NULL}};
const Segment SGI339478[] = {{&S24203[12], 14}, {NULL}};
const Segment SGI339479[] = {{&S24204[6], 4}, {NULL}};
const Segment SGI339480[] = {{&S24204[12], 14}, {NULL}};
const Segment SGI339481[] = {{&S24205[6], 4}, {NULL}};
const Segment SGI339482[] = {{&S24205[12], 14}, {NULL}};
const Segment SGI339483[] = {{&S24207[6], 4}, {NULL}};
const Segment SGI339484[] = {{&S24207[12], 14}, {&S24207[40], 12}, {NULL}};
const Segment SGI339485[] = {{&S24207[12], 14}, {NULL}};
const Segment SGI339486[] = {{&S24212[6], 4}, {NULL}};
const Segment SGI339487[] = {{&S24212[12], 14}, {NULL}};
const Segment SGI339488[] = {{&S24216[6], 4}, {NULL}};
const Segment SGI339489[] = {{&S24216[43], 4}, {NULL}};
const Segment SGI339490[] = {{&S24216[12], 14}, {NULL}};
const Segment SGI339491[] = {{&S24218[6], 4}, {NULL}};
const Segment SGI339492[] = {{&S24218[57], 5}, {NULL}};
const Segment SGI339493[] = {{&S24218[12], 14}, {NULL}};
const Segment SGI339494[] = {{&S41888[28], 3}, {NULL}};
const Segment SGI339495[] = {{&S41888[102], 5}, {NULL}};
const Segment SGI339496[] = {{&S24220[6], 4}, {NULL}};
const Segment SGI339497[] = {{&S24220[12], 14}, {&S24220[40], 12}, {NULL}};
const Segment SGI339498[] = {{&S24220[12], 14}, {NULL}};
const Segment SGI339499[] = {{&S24221[6], 4}, {NULL}};
const Segment SGI339500[] = {{&S24221[12], 14}, {&S24221[40], 11}, {NULL}};
const Segment SGI339501[] = {{&S24221[12], 14}, {NULL}};
const Segment SGI339502[] = {{&S50805[5], 15}, {NULL}};
const Segment SGI339503[] = {{&S50806[5], 15}, {&S50806[78], 15}, {NULL}};
const Segment SGI339504[] = {{&S50807[5], 4}, {NULL}};
const Segment SGI339505[] = {{&S50808[5], 9}, {NULL}};
const Segment SGI339506[] = {{&S50809[5], 20}, {NULL}};
const Segment SGI339507[] = {{&S42043[28], 5}, {NULL}};
const Segment SGI339508[] = {{&S42043[104], 5}, {NULL}};
const Segment SGI339509[] = {{&S42044[28], 5}, {NULL}};
const Segment SGI339510[] = {{&S42044[104], 5}, {NULL}};
const Segment SGI339511[] = {{&S23623[0], 21}, {NULL}};
const Segment SGI339512[] = {{&S23623[34], 3}, {NULL}};
const Segment SGI339513[] = {{&S23623[75], 13}, {NULL}};
const Segment SGI339514[] = {{&S23624[0], 21}, {NULL}};
const Segment SGI339515[] = {{&S23624[34], 3}, {NULL}};
const Segment SGI339516[] = {{&S23625[0], 21}, {NULL}};
const Segment SGI339517[] = {{&S23625[34], 3}, {NULL}};
const Segment SGI339518[] = {{&S23625[75], 13}, {NULL}};
const Segment SGI339519[] = {{&S42117[28], 3}, {NULL}};
const Segment SGI339520[] = {{&S42117[102], 5}, {NULL}};
const Segment SGI339521[] = {{&S42118[28], 3}, {NULL}};
const Segment SGI339522[] = {{&S42118[102], 5}, {NULL}};
const Segment SGI339523[] = {{&S26685[64], 5}, {NULL}};
const Segment SGI339524[] = {{&S26685[13], 10}, {NULL}};
const Segment SGI339525[] = {{&S51022[42], 7}, {NULL}};
const Segment SGI339526[] = {{&S51022[6], 3}, {NULL}};
const Segment SGI339527[] = {{&S51022[70], 3}, {NULL}};
const Segment SGI339528[] = {{&S51023[41], 5}, {NULL}};
const Segment SGI339529[] = {{&S51023[6], 3}, {NULL}};
const Segment SGI339530[] = {{&S51023[67], 3}, {NULL}};
const Segment SGI339531[] = {{&S51024[42], 5}, {NULL}};
const Segment SGI339532[] = {{&S51024[6], 3}, {NULL}};
const Segment SGI339533[] = {{&S51024[68], 3}, {NULL}};
const Segment SGI339534[] = {{&S51025[41], 7}, {NULL}};
const Segment SGI339535[] = {{&S51025[6], 3}, {NULL}};
const Segment SGI339536[] = {{&S51025[69], 3}, {NULL}};
const Segment SGI339537[] = {{&S24756[6], 4}, {NULL}};
const Segment SGI339538[] = {{&S24756[58], 5}, {NULL}};
const Segment SGI339539[] = {{&S24756[12], 3}, {&S24756[76], 9}, {&S24756[140], 52}, {NULL}};
const Segment SGI339540[] = {{&S24756[96], 13}, {NULL}};
const Segment SGI339541[] = {{&S51303[5], 5}, {NULL}};
const Segment SGI339542[] = {{&S51303[16], 3}, {NULL}};
const Segment SGI339543[] = {{&S51304[35], 5}, {NULL}};
const Segment SGI339544[] = {{&S51304[91], 3}, {NULL}};
const Segment SGI339545[] = {{&S51305[5], 14}, {NULL}};
const Segment SGI339546[] = {{&S51305[20], 3}, {NULL}};
const Segment SGI339547[] = {{&S42561[28], 3}, {NULL}};
const Segment SGI339548[] = {{&S42561[102], 5}, {NULL}};
const Segment SGI339549[] = {{&S37878[46], 7}, {NULL}};
const Segment SGI339550[] = {{&S37878[8], 3}, {NULL}};
const Segment SGI339551[] = {{&S37879[5], 6}, {NULL}};
const Segment SGI339552[] = {{&S37879[49], 3}, {NULL}};
const Segment SGI339553[] = {{&S37880[46], 5}, {NULL}};
const Segment SGI339554[] = {{&S37880[8], 3}, {NULL}};
const Segment SGI339555[] = {{&S37881[46], 5}, {&S37881[147], 7}, {NULL}};
const Segment SGI339556[] = {{&S37881[8], 3}, {NULL}};
const Segment SGI339557[] = {{&S37882[46], 7}, {&S37882[149], 7}, {NULL}};
const Segment SGI339558[] = {{&S37882[8], 3}, {NULL}};
const Segment SGI339559[] = {{&S51518[100], 5}, {NULL}};
const Segment SGI339560[] = {{&S51532[5], 5}, {&S51532[104], 5}, {NULL}};
const Segment SGI339561[] = {{&S51532[11], 3}, {NULL}};
const Segment SGI339562[] = {{&S51532[130], 3}, {NULL}};
const Segment SGI339563[] = {{&S51638[35], 4}, {NULL}};
const Segment SGI339564[] = {{&S51638[51], 3}, {NULL}};
const Segment SGI339565[] = {{&S51639[5], 4}, {NULL}};
const Segment SGI339566[] = {{&S51639[15], 3}, {NULL}};
const Segment SGI339567[] = {{&S51640[34], 4}, {NULL}};
const Segment SGI339568[] = {{&S51640[49], 3}, {NULL}};
const Segment SGI339569[] = {{&S42858[28], 3}, {NULL}};
const Segment SGI339570[] = {{&S42858[102], 5}, {NULL}};
const Segment SGI339571[] = {{&S42859[28], 3}, {NULL}};
const Segment SGI339572[] = {{&S42859[102], 5}, {NULL}};
const Segment SGI339573[] = {{&S51895[5], 5}, {&S51895[105], 5}, {NULL}};
const Segment SGI339574[] = {{&S51895[69], 3}, {NULL}};
const Segment SGI339575[] = {{&S51895[131], 3}, {NULL}};
const Segment SGI339576[] = {{&S51896[5], 7}, {&S51896[107], 7}, {NULL}};
const Segment SGI339577[] = {{&S51896[71], 3}, {NULL}};
const Segment SGI339578[] = {{&S51896[135], 3}, {NULL}};
const Segment SGI339579[] = {{&S51941[5], 5}, {&S51941[104], 5}, {NULL}};
const Segment SGI339580[] = {{&S51941[69], 3}, {NULL}};
const Segment SGI339581[] = {{&S51941[130], 3}, {NULL}};
const Segment SGI339582[] = {{&S51960[5], 5}, {&S51960[104], 5}, {NULL}};
const Segment SGI339583[] = {{&S51960[11], 3}, {NULL}};
const Segment SGI339584[] = {{&S51960[130], 3}, {NULL}};
const Segment SGI339585[] = {{&S25703[6], 4}, {NULL}};
const Segment SGI339586[] = {{&S25703[66], 5}, {NULL}};
const Segment SGI339587[] = {{&S25703[12], 14}, {NULL}};
const Segment SGI339588[] = {{&S25704[6], 4}, {NULL}};
const Segment SGI339589[] = {{&S25704[82], 5}, {NULL}};
const Segment SGI339590[] = {{&S25704[12], 14}, {&S25704[40], 12}, {NULL}};
const Segment SGI339591[] = {{&S25704[12], 14}, {NULL}};
const Segment SGI339592[] = {{&S25705[6], 4}, {NULL}};
const Segment SGI339593[] = {{&S25705[89], 5}, {NULL}};
const Segment SGI339594[] = {{&S25705[12], 14}, {&S25705[48], 11}, {NULL}};
const Segment SGI339595[] = {{&S25705[12], 14}, {NULL}};
const Segment SGI339596[] = {{&S25706[6], 4}, {NULL}};
const Segment SGI339597[] = {{&S25706[81], 5}, {NULL}};
const Segment SGI339598[] = {{&S25706[12], 14}, {&S25706[40], 11}, {NULL}};
const Segment SGI339599[] = {{&S25706[12], 14}, {NULL}};
const Segment SGI339600[] = {{&S25707[6], 4}, {NULL}};
const Segment SGI339601[] = {{&S25707[81], 5}, {NULL}};
const Segment SGI339602[] = {{&S25707[12], 14}, {&S25707[40], 11}, {NULL}};
const Segment SGI339603[] = {{&S25707[12], 14}, {NULL}};
const Segment SGI339604[] = {{&S25708[6], 4}, {NULL}};
const Segment SGI339605[] = {{&S25708[81], 5}, {NULL}};
const Segment SGI339606[] = {{&S25708[12], 14}, {&S25708[40], 11}, {NULL}};
const Segment SGI339607[] = {{&S25708[12], 14}, {NULL}};
const Segment SGI339608[] = {{&S25709[6], 4}, {NULL}};
const Segment SGI339609[] = {{&S25709[82], 5}, {NULL}};
const Segment SGI339610[] = {{&S25709[12], 14}, {&S25709[41], 11}, {NULL}};
const Segment SGI339611[] = {{&S25709[12], 14}, {NULL}};
const Segment SGI339612[] = {{&S25710[6], 4}, {NULL}};
const Segment SGI339613[] = {{&S25710[81], 5}, {NULL}};
const Segment SGI339614[] = {{&S25710[12], 14}, {&S25710[40], 11}, {NULL}};
const Segment SGI339615[] = {{&S25710[12], 14}, {NULL}};
const Segment SGI339616[] = {{&S25711[6], 4}, {NULL}};
const Segment SGI339617[] = {{&S25711[89], 5}, {NULL}};
const Segment SGI339618[] = {{&S25711[12], 14}, {&S25711[48], 11}, {NULL}};
const Segment SGI339619[] = {{&S25711[12], 14}, {NULL}};
const Segment SGI339620[] = {{&S25713[6], 4}, {NULL}};
const Segment SGI339621[] = {{&S25713[82], 5}, {NULL}};
const Segment SGI339622[] = {{&S25713[12], 14}, {&S25713[40], 12}, {NULL}};
const Segment SGI339623[] = {{&S25713[12], 14}, {NULL}};
const Segment SGI339624[] = {{&S25714[6], 4}, {NULL}};
const Segment SGI339625[] = {{&S25714[89], 5}, {NULL}};
const Segment SGI339626[] = {{&S25714[12], 14}, {&S25714[48], 11}, {NULL}};
const Segment SGI339627[] = {{&S25714[12], 14}, {NULL}};
const Segment SGI339628[] = {{&S25715[6], 4}, {NULL}};
const Segment SGI339629[] = {{&S25715[81], 5}, {NULL}};
const Segment SGI339630[] = {{&S25715[12], 14}, {&S25715[40], 11}, {NULL}};
const Segment SGI339631[] = {{&S25715[12], 14}, {NULL}};
const Segment SGI339632[] = {{&S25717[6], 4}, {NULL}};
const Segment SGI339633[] = {{&S25717[81], 5}, {NULL}};
const Segment SGI339634[] = {{&S25717[12], 14}, {&S25717[40], 11}, {NULL}};
const Segment SGI339635[] = {{&S25717[12], 14}, {NULL}};
const Segment SGI339636[] = {{&S25718[6], 4}, {NULL}};
const Segment SGI339637[] = {{&S25718[88], 5}, {NULL}};
const Segment SGI339638[] = {{&S25718[12], 14}, {&S25718[47], 11}, {NULL}};
const Segment SGI339639[] = {{&S25718[12], 14}, {NULL}};
const Segment SGI339640[] = {{&S25719[6], 4}, {NULL}};
const Segment SGI339641[] = {{&S25719[89], 5}, {NULL}};
const Segment SGI339642[] = {{&S25719[12], 14}, {&S25719[48], 11}, {NULL}};
const Segment SGI339643[] = {{&S25719[12], 14}, {NULL}};
const Segment SGI339644[] = {{&S25720[6], 4}, {NULL}};
const Segment SGI339645[] = {{&S25720[89], 5}, {NULL}};
const Segment SGI339646[] = {{&S25720[12], 14}, {&S25720[48], 11}, {NULL}};
const Segment SGI339647[] = {{&S25720[12], 14}, {NULL}};
const Segment SGI339648[] = {{&S52107[5], 3}, {&S52107[5], 3}, {NULL}};
const Segment SGI339649[] = {{&S52107[9], 3}, {NULL}};
const Segment SGI339650[] = {{&S52107[127], 3}, {NULL}};
const Segment SGI339651[] = {{&S25721[6], 4}, {NULL}};
const Segment SGI339652[] = {{&S25721[88], 5}, {NULL}};
const Segment SGI339653[] = {{&S25721[12], 14}, {&S25721[47], 11}, {NULL}};
const Segment SGI339654[] = {{&S25721[12], 14}, {NULL}};
const Segment SGI339655[] = {{&S25722[6], 4}, {NULL}};
const Segment SGI339656[] = {{&S25722[82], 5}, {NULL}};
const Segment SGI339657[] = {{&S25722[12], 14}, {&S25722[41], 11}, {NULL}};
const Segment SGI339658[] = {{&S25722[12], 14}, {NULL}};
const Segment SGI339659[] = {{&S25723[6], 4}, {NULL}};
const Segment SGI339660[] = {{&S25723[89], 5}, {NULL}};
const Segment SGI339661[] = {{&S25723[12], 14}, {&S25723[48], 11}, {NULL}};
const Segment SGI339662[] = {{&S25723[12], 14}, {NULL}};
const Segment SGI339663[] = {{&S25724[6], 4}, {NULL}};
const Segment SGI339664[] = {{&S25724[82], 5}, {NULL}};
const Segment SGI339665[] = {{&S25724[12], 14}, {&S25724[41], 11}, {NULL}};
const Segment SGI339666[] = {{&S25724[12], 14}, {NULL}};
const Segment SGI339667[] = {{&S25725[6], 4}, {NULL}};
const Segment SGI339668[] = {{&S25725[82], 5}, {NULL}};
const Segment SGI339669[] = {{&S25725[12], 14}, {&S25725[41], 11}, {NULL}};
const Segment SGI339670[] = {{&S25725[12], 14}, {NULL}};
const Segment SGI339671[] = {{&S25726[6], 4}, {NULL}};
const Segment SGI339672[] = {{&S25726[81], 5}, {NULL}};
const Segment SGI339673[] = {{&S25726[12], 14}, {&S25726[40], 11}, {NULL}};
const Segment SGI339674[] = {{&S25726[12], 14}, {NULL}};
const Segment SGI339675[] = {{&S25727[6], 4}, {NULL}};
const Segment SGI339676[] = {{&S25727[81], 5}, {NULL}};
const Segment SGI339677[] = {{&S25727[12], 14}, {&S25727[40], 11}, {NULL}};
const Segment SGI339678[] = {{&S25727[12], 14}, {NULL}};
const Segment SGI339679[] = {{&S25728[6], 4}, {NULL}};
const Segment SGI339680[] = {{&S25728[89], 5}, {NULL}};
const Segment SGI339681[] = {{&S25728[12], 14}, {&S25728[47], 12}, {NULL}};
const Segment SGI339682[] = {{&S25728[12], 14}, {NULL}};
const Segment SGI339683[] = {{&S25729[6], 4}, {NULL}};
const Segment SGI339684[] = {{&S25729[81], 5}, {NULL}};
const Segment SGI339685[] = {{&S25729[12], 14}, {&S25729[40], 11}, {NULL}};
const Segment SGI339686[] = {{&S25729[12], 14}, {NULL}};
const Segment SGI339687[] = {{&S25730[6], 4}, {NULL}};
const Segment SGI339688[] = {{&S25730[82], 5}, {NULL}};
const Segment SGI339689[] = {{&S25730[12], 14}, {&S25730[41], 11}, {NULL}};
const Segment SGI339690[] = {{&S25730[12], 14}, {NULL}};
const Segment SGI339691[] = {{&S25731[6], 4}, {NULL}};
const Segment SGI339692[] = {{&S25731[82], 5}, {NULL}};
const Segment SGI339693[] = {{&S25731[12], 14}, {&S25731[40], 12}, {NULL}};
const Segment SGI339694[] = {{&S25731[12], 14}, {NULL}};
const Segment SGI339695[] = {{&S25732[6], 4}, {NULL}};
const Segment SGI339696[] = {{&S25732[82], 5}, {NULL}};
const Segment SGI339697[] = {{&S25732[12], 14}, {&S25732[41], 11}, {NULL}};
const Segment SGI339698[] = {{&S25732[12], 14}, {NULL}};
const Segment SGI339699[] = {{&S25733[6], 4}, {NULL}};
const Segment SGI339700[] = {{&S25733[81], 5}, {NULL}};
const Segment SGI339701[] = {{&S25733[12], 14}, {&S25733[40], 11}, {NULL}};
const Segment SGI339702[] = {{&S25733[12], 14}, {NULL}};
const Segment SGI339703[] = {{&S25734[6], 4}, {NULL}};
const Segment SGI339704[] = {{&S25734[82], 5}, {NULL}};
const Segment SGI339705[] = {{&S25734[12], 14}, {&S25734[40], 12}, {NULL}};
const Segment SGI339706[] = {{&S25734[12], 14}, {NULL}};
const Segment SGI339707[] = {{&S25735[6], 4}, {NULL}};
const Segment SGI339708[] = {{&S25735[82], 5}, {NULL}};
const Segment SGI339709[] = {{&S25735[12], 14}, {&S25735[41], 11}, {NULL}};
const Segment SGI339710[] = {{&S25735[12], 14}, {NULL}};
const Segment SGI339711[] = {{&S25736[6], 4}, {NULL}};
const Segment SGI339712[] = {{&S25736[88], 5}, {NULL}};
const Segment SGI339713[] = {{&S25736[12], 14}, {&S25736[47], 11}, {NULL}};
const Segment SGI339714[] = {{&S25736[12], 14}, {NULL}};
const Segment SGI339715[] = {{&S25737[6], 4}, {NULL}};
const Segment SGI339716[] = {{&S25737[82], 5}, {NULL}};
const Segment SGI339717[] = {{&S25737[12], 14}, {&S25737[41], 11}, {NULL}};
const Segment SGI339718[] = {{&S25737[12], 14}, {NULL}};
const Segment SGI339719[] = {{&S25738[6], 4}, {NULL}};
const Segment SGI339720[] = {{&S25738[82], 5}, {NULL}};
const Segment SGI339721[] = {{&S25738[12], 14}, {&S25738[41], 11}, {NULL}};
const Segment SGI339722[] = {{&S25738[12], 14}, {NULL}};
const Segment SGI339723[] = {{&S25739[6], 4}, {NULL}};
const Segment SGI339724[] = {{&S25739[82], 5}, {NULL}};
const Segment SGI339725[] = {{&S25739[12], 14}, {&S25739[41], 11}, {NULL}};
const Segment SGI339726[] = {{&S25739[12], 14}, {NULL}};
const Segment SGI339727[] = {{&S25740[6], 4}, {NULL}};
const Segment SGI339728[] = {{&S25740[89], 5}, {NULL}};
const Segment SGI339729[] = {{&S25740[12], 14}, {&S25740[48], 11}, {NULL}};
const Segment SGI339730[] = {{&S25740[12], 14}, {NULL}};
const Segment SGI339731[] = {{&S25741[6], 4}, {NULL}};
const Segment SGI339732[] = {{&S25741[66], 5}, {NULL}};
const Segment SGI339733[] = {{&S25741[12], 14}, {&S25741[77], 31}, {NULL}};
const Segment SGI339734[] = {{&S25741[12], 14}, {NULL}};
const Segment SGI339735[] = {{&S25742[6], 4}, {NULL}};
const Segment SGI339736[] = {{&S25742[89], 5}, {NULL}};
const Segment SGI339737[] = {{&S25742[12], 14}, {&S25742[47], 12}, {NULL}};
const Segment SGI339738[] = {{&S25742[12], 14}, {NULL}};
const Segment SGI339739[] = {{&S25743[6], 4}, {NULL}};
const Segment SGI339740[] = {{&S25743[81], 5}, {NULL}};
const Segment SGI339741[] = {{&S25743[12], 14}, {&S25743[40], 11}, {NULL}};
const Segment SGI339742[] = {{&S25743[12], 14}, {NULL}};
const Segment SGI339743[] = {{&S25744[6], 4}, {NULL}};
const Segment SGI339744[] = {{&S25744[81], 5}, {NULL}};
const Segment SGI339745[] = {{&S25744[12], 14}, {&S25744[40], 11}, {NULL}};
const Segment SGI339746[] = {{&S25744[12], 14}, {NULL}};
const Segment SGI339747[] = {{&S25745[6], 4}, {NULL}};
const Segment SGI339748[] = {{&S25745[82], 5}, {NULL}};
const Segment SGI339749[] = {{&S25745[12], 14}, {&S25745[41], 11}, {NULL}};
const Segment SGI339750[] = {{&S25745[12], 14}, {NULL}};
const Segment SGI339751[] = {{&S25746[6], 4}, {NULL}};
const Segment SGI339752[] = {{&S25746[81], 5}, {NULL}};
const Segment SGI339753[] = {{&S25746[12], 14}, {&S25746[40], 11}, {NULL}};
const Segment SGI339754[] = {{&S25746[12], 14}, {NULL}};
const Segment SGI339755[] = {{&S25747[6], 4}, {NULL}};
const Segment SGI339756[] = {{&S25747[82], 5}, {NULL}};
const Segment SGI339757[] = {{&S25747[12], 14}, {&S25747[40], 12}, {NULL}};
const Segment SGI339758[] = {{&S25747[12], 14}, {NULL}};
const Segment SGI339759[] = {{&S25748[6], 4}, {NULL}};
const Segment SGI339760[] = {{&S25748[88], 5}, {NULL}};
const Segment SGI339761[] = {{&S25748[12], 14}, {&S25748[47], 11}, {NULL}};
const Segment SGI339762[] = {{&S25748[12], 14}, {NULL}};
const Segment SGI339763[] = {{&S25749[6], 4}, {NULL}};
const Segment SGI339764[] = {{&S25749[82], 5}, {NULL}};
const Segment SGI339765[] = {{&S25749[12], 14}, {&S25749[40], 12}, {NULL}};
const Segment SGI339766[] = {{&S25749[12], 14}, {NULL}};
const Segment SGI339767[] = {{&S25750[6], 4}, {NULL}};
const Segment SGI339768[] = {{&S25750[82], 5}, {NULL}};
const Segment SGI339769[] = {{&S25750[12], 14}, {&S25750[41], 11}, {NULL}};
const Segment SGI339770[] = {{&S25750[12], 14}, {NULL}};
const Segment SGI339771[] = {{&S25751[6], 4}, {NULL}};
const Segment SGI339772[] = {{&S25751[81], 5}, {NULL}};
const Segment SGI339773[] = {{&S25751[12], 14}, {&S25751[40], 11}, {NULL}};
const Segment SGI339774[] = {{&S25751[12], 14}, {NULL}};
const Segment SGI339775[] = {{&S25752[6], 4}, {NULL}};
const Segment SGI339776[] = {{&S25752[81], 5}, {NULL}};
const Segment SGI339777[] = {{&S25752[12], 14}, {&S25752[40], 11}, {NULL}};
const Segment SGI339778[] = {{&S25752[12], 14}, {NULL}};
const Segment SGI339779[] = {{&S52151[5], 4}, {NULL}};
const Segment SGI339780[] = {{&S52151[20], 3}, {NULL}};
const Segment SGI339781[] = {{&S25753[6], 4}, {NULL}};
const Segment SGI339782[] = {{&S25753[82], 5}, {NULL}};
const Segment SGI339783[] = {{&S25753[12], 14}, {&S25753[40], 12}, {NULL}};
const Segment SGI339784[] = {{&S25753[12], 14}, {NULL}};
const Segment SGI339785[] = {{&S52152[5], 4}, {NULL}};
const Segment SGI339786[] = {{&S52152[15], 3}, {NULL}};
const Segment SGI339787[] = {{&S25754[6], 4}, {NULL}};
const Segment SGI339788[] = {{&S25754[88], 5}, {NULL}};
const Segment SGI339789[] = {{&S25754[12], 14}, {&S25754[47], 11}, {NULL}};
const Segment SGI339790[] = {{&S25754[12], 14}, {NULL}};
const Segment SGI339791[] = {{&S52153[5], 5}, {NULL}};
const Segment SGI339792[] = {{&S52153[21], 3}, {NULL}};
const Segment SGI339793[] = {{&S25755[6], 4}, {NULL}};
const Segment SGI339794[] = {{&S25755[81], 5}, {NULL}};
const Segment SGI339795[] = {{&S25755[12], 14}, {&S25755[40], 11}, {NULL}};
const Segment SGI339796[] = {{&S25755[12], 14}, {NULL}};
const Segment SGI339797[] = {{&S25756[6], 4}, {NULL}};
const Segment SGI339798[] = {{&S25756[88], 5}, {NULL}};
const Segment SGI339799[] = {{&S25756[12], 14}, {&S25756[47], 11}, {NULL}};
const Segment SGI339800[] = {{&S25756[12], 14}, {NULL}};
const Segment SGI339801[] = {{&S25757[6], 4}, {NULL}};
const Segment SGI339802[] = {{&S25757[82], 5}, {NULL}};
const Segment SGI339803[] = {{&S25757[12], 14}, {&S25757[40], 12}, {NULL}};
const Segment SGI339804[] = {{&S25757[12], 14}, {NULL}};
const Segment SGI339805[] = {{&S25758[6], 4}, {NULL}};
const Segment SGI339806[] = {{&S25758[89], 5}, {NULL}};
const Segment SGI339807[] = {{&S25758[12], 14}, {&S25758[47], 12}, {NULL}};
const Segment SGI339808[] = {{&S25758[12], 14}, {NULL}};
const Segment SGI339809[] = {{&S25759[6], 4}, {NULL}};
const Segment SGI339810[] = {{&S25759[81], 5}, {NULL}};
const Segment SGI339811[] = {{&S25759[12], 14}, {&S25759[40], 11}, {NULL}};
const Segment SGI339812[] = {{&S25759[12], 14}, {NULL}};
const Segment SGI339813[] = {{&S43419[28], 3}, {NULL}};
const Segment SGI339814[] = {{&S25760[6], 4}, {NULL}};
const Segment SGI339815[] = {{&S43419[102], 5}, {NULL}};
const Segment SGI339816[] = {{&S25760[82], 5}, {NULL}};
const Segment SGI339817[] = {{&S25760[12], 14}, {&S25760[40], 12}, {NULL}};
const Segment SGI339818[] = {{&S25760[12], 14}, {NULL}};
const Segment SGI339819[] = {{&S25761[6], 4}, {NULL}};
const Segment SGI339820[] = {{&S25761[88], 5}, {NULL}};
const Segment SGI339821[] = {{&S25761[12], 14}, {&S25761[47], 11}, {NULL}};
const Segment SGI339822[] = {{&S25761[12], 14}, {NULL}};
const Segment SGI339823[] = {{&S25762[6], 4}, {NULL}};
const Segment SGI339824[] = {{&S25762[88], 5}, {NULL}};
const Segment SGI339825[] = {{&S25762[12], 14}, {&S25762[47], 11}, {NULL}};
const Segment SGI339826[] = {{&S25762[12], 14}, {NULL}};
const Segment SGI339827[] = {{&S25763[6], 4}, {NULL}};
const Segment SGI339828[] = {{&S25763[88], 5}, {NULL}};
const Segment SGI339829[] = {{&S25763[12], 14}, {&S25763[47], 11}, {NULL}};
const Segment SGI339830[] = {{&S25763[12], 14}, {NULL}};
const Segment SGI339831[] = {{&S25764[6], 4}, {NULL}};
const Segment SGI339832[] = {{&S25764[89], 5}, {NULL}};
const Segment SGI339833[] = {{&S25764[12], 14}, {&S25764[47], 12}, {NULL}};
const Segment SGI339834[] = {{&S25764[12], 14}, {NULL}};
const Segment SGI339835[] = {{&S25765[6], 4}, {NULL}};
const Segment SGI339836[] = {{&S25765[89], 5}, {NULL}};
const Segment SGI339837[] = {{&S25765[12], 14}, {&S25765[48], 11}, {NULL}};
const Segment SGI339838[] = {{&S25765[12], 14}, {NULL}};
const Segment SGI339839[] = {{&S25766[6], 5}, {NULL}};
const Segment SGI339840[] = {{&S25766[13], 14}, {NULL}};
const Segment SGI339841[] = {{&S25767[6], 4}, {NULL}};
const Segment SGI339842[] = {{&S25767[81], 5}, {NULL}};
const Segment SGI339843[] = {{&S25767[12], 14}, {&S25767[40], 11}, {NULL}};
const Segment SGI339844[] = {{&S25767[12], 14}, {NULL}};
const Segment SGI339845[] = {{&S25768[6], 4}, {NULL}};
const Segment SGI339846[] = {{&S25768[59], 5}, {NULL}};
const Segment SGI339847[] = {{&S25768[12], 14}, {&S25768[70], 4}, {NULL}};
const Segment SGI339848[] = {{&S25768[12], 14}, {NULL}};
const Segment SGI339849[] = {{&S25769[6], 4}, {NULL}};
const Segment SGI339850[] = {{&S25769[88], 5}, {NULL}};
const Segment SGI339851[] = {{&S25769[12], 14}, {&S25769[47], 11}, {NULL}};
const Segment SGI339852[] = {{&S25769[12], 14}, {NULL}};
const Segment SGI339853[] = {{&S25770[6], 4}, {NULL}};
const Segment SGI339854[] = {{&S25770[81], 5}, {NULL}};
const Segment SGI339855[] = {{&S25770[12], 14}, {&S25770[40], 11}, {NULL}};
const Segment SGI339856[] = {{&S25770[12], 14}, {NULL}};
const Segment SGI339857[] = {{&S25771[6], 4}, {NULL}};
const Segment SGI339858[] = {{&S25771[81], 5}, {NULL}};
const Segment SGI339859[] = {{&S25771[12], 14}, {&S25771[40], 11}, {NULL}};
const Segment SGI339860[] = {{&S25771[12], 14}, {NULL}};
const Segment SGI339861[] = {{&S25772[6], 4}, {NULL}};
const Segment SGI339862[] = {{&S25772[81], 5}, {NULL}};
const Segment SGI339863[] = {{&S25772[12], 14}, {&S25772[40], 11}, {NULL}};
const Segment SGI339864[] = {{&S25772[12], 14}, {NULL}};
const Segment SGI339865[] = {{&S25773[6], 4}, {NULL}};
const Segment SGI339866[] = {{&S25773[52], 5}, {NULL}};
const Segment SGI339867[] = {{&S25773[12], 45}, {NULL}};
const Segment SGI339868[] = {{&S25773[12], 14}, {NULL}};
const Segment SGI339869[] = {{&S25774[6], 4}, {NULL}};
const Segment SGI339870[] = {{&S25774[82], 5}, {NULL}};
const Segment SGI339871[] = {{&S25774[12], 14}, {&S25774[41], 11}, {NULL}};
const Segment SGI339872[] = {{&S25774[12], 14}, {NULL}};
const Segment SGI339873[] = {{&S25775[6], 4}, {NULL}};
const Segment SGI339874[] = {{&S25775[88], 5}, {NULL}};
const Segment SGI339875[] = {{&S25775[12], 14}, {&S25775[47], 11}, {NULL}};
const Segment SGI339876[] = {{&S25775[12], 14}, {NULL}};
const Segment SGI339877[] = {{&S25776[6], 4}, {NULL}};
const Segment SGI339878[] = {{&S25776[81], 5}, {NULL}};
const Segment SGI339879[] = {{&S25776[12], 14}, {&S25776[40], 11}, {NULL}};
const Segment SGI339880[] = {{&S25776[12], 14}, {NULL}};
const Segment SGI339881[] = {{&S25777[6], 4}, {NULL}};
const Segment SGI339882[] = {{&S25777[88], 5}, {NULL}};
const Segment SGI339883[] = {{&S25777[12], 14}, {&S25777[47], 11}, {NULL}};
const Segment SGI339884[] = {{&S25777[12], 14}, {NULL}};
const Segment SGI339885[] = {{&S25778[6], 4}, {NULL}};
const Segment SGI339886[] = {{&S25778[88], 5}, {NULL}};
const Segment SGI339887[] = {{&S25778[12], 14}, {&S25778[47], 11}, {NULL}};
const Segment SGI339888[] = {{&S25778[12], 14}, {NULL}};
const Segment SGI339889[] = {{&S25779[6], 4}, {NULL}};
const Segment SGI339890[] = {{&S25779[81], 5}, {NULL}};
const Segment SGI339891[] = {{&S25779[12], 14}, {&S25779[40], 11}, {NULL}};
const Segment SGI339892[] = {{&S25779[12], 14}, {NULL}};
const Segment SGI339893[] = {{&S43445[28], 3}, {NULL}};
const Segment SGI339894[] = {{&S43445[102], 5}, {NULL}};
const Segment SGI339895[] = {{&S43527[28], 3}, {NULL}};
const Segment SGI339896[] = {{&S43527[102], 5}, {NULL}};
const Segment SGI339897[] = {{&S43528[28], 3}, {NULL}};
const Segment SGI339898[] = {{&S43528[102], 5}, {NULL}};
const Segment SGI339899[] = {{&S43529[28], 3}, {NULL}};
const Segment SGI339900[] = {{&S43529[102], 5}, {NULL}};
const Segment SGI339901[] = {{&S43530[28], 3}, {NULL}};
const Segment SGI339902[] = {{&S43530[102], 5}, {NULL}};
const Segment SGI339903[] = {{&S27830[57], 5}, {NULL}};
const Segment SGI339904[] = {{&S27830[13], 10}, {NULL}};
const Segment SGI339905[] = {{&S25921[6], 4}, {NULL}};
const Segment SGI339906[] = {{&S25921[59], 5}, {NULL}};
const Segment SGI339907[] = {{&S25921[12], 14}, {NULL}};
const Segment SGI339908[] = {{&S25922[6], 4}, {NULL}};
const Segment SGI339909[] = {{&S25922[82], 5}, {NULL}};
const Segment SGI339910[] = {{&S25922[12], 14}, {&S25922[40], 12}, {NULL}};
const Segment SGI339911[] = {{&S25922[12], 14}, {NULL}};
const Segment SGI339912[] = {{&S25923[6], 4}, {NULL}};
const Segment SGI339913[] = {{&S25923[81], 5}, {NULL}};
const Segment SGI339914[] = {{&S25923[12], 14}, {&S25923[40], 11}, {NULL}};
const Segment SGI339915[] = {{&S25923[12], 14}, {NULL}};
const Segment SGI339916[] = {{&S25924[6], 4}, {NULL}};
const Segment SGI339917[] = {{&S25924[81], 5}, {NULL}};
const Segment SGI339918[] = {{&S25924[12], 14}, {&S25924[40], 11}, {NULL}};
const Segment SGI339919[] = {{&S25924[12], 14}, {NULL}};
const Segment SGI339920[] = {{&S25925[6], 4}, {NULL}};
const Segment SGI339921[] = {{&S25925[81], 5}, {NULL}};
const Segment SGI339922[] = {{&S25925[12], 14}, {&S25925[40], 11}, {NULL}};
const Segment SGI339923[] = {{&S25925[12], 14}, {NULL}};
const Segment SGI339924[] = {{&S25926[6], 4}, {NULL}};
const Segment SGI339925[] = {{&S25926[81], 5}, {NULL}};
const Segment SGI339926[] = {{&S25926[12], 14}, {&S25926[40], 11}, {NULL}};
const Segment SGI339927[] = {{&S25926[12], 14}, {NULL}};
const Segment SGI339928[] = {{&S25927[6], 4}, {NULL}};
const Segment SGI339929[] = {{&S25927[82], 5}, {NULL}};
const Segment SGI339930[] = {{&S25927[12], 14}, {&S25927[41], 11}, {NULL}};
const Segment SGI339931[] = {{&S25927[12], 14}, {NULL}};
const Segment SGI339932[] = {{&S25928[6], 4}, {NULL}};
const Segment SGI339933[] = {{&S25928[82], 5}, {NULL}};
const Segment SGI339934[] = {{&S25928[12], 14}, {&S25928[41], 11}, {NULL}};
const Segment SGI339935[] = {{&S25928[12], 14}, {NULL}};
const Segment SGI339936[] = {{&S25929[6], 4}, {NULL}};
const Segment SGI339937[] = {{&S25929[82], 5}, {NULL}};
const Segment SGI339938[] = {{&S25929[12], 14}, {&S25929[41], 11}, {NULL}};
const Segment SGI339939[] = {{&S25929[12], 14}, {NULL}};
const Segment SGI339940[] = {{&S25930[6], 4}, {NULL}};
const Segment SGI339941[] = {{&S25930[82], 5}, {NULL}};
const Segment SGI339942[] = {{&S25930[12], 14}, {&S25930[41], 11}, {NULL}};
const Segment SGI339943[] = {{&S25930[12], 14}, {NULL}};
const Segment SGI339944[] = {{&S25931[6], 4}, {NULL}};
const Segment SGI339945[] = {{&S25931[82], 5}, {NULL}};
const Segment SGI339946[] = {{&S25931[12], 14}, {&S25931[41], 11}, {NULL}};
const Segment SGI339947[] = {{&S25931[12], 14}, {NULL}};
const Segment SGI339948[] = {{&S25932[6], 4}, {NULL}};
const Segment SGI339949[] = {{&S25932[82], 5}, {NULL}};
const Segment SGI339950[] = {{&S25932[12], 14}, {&S25932[41], 11}, {NULL}};
const Segment SGI339951[] = {{&S25932[12], 14}, {NULL}};
const Segment SGI339952[] = {{&S25933[6], 4}, {NULL}};
const Segment SGI339953[] = {{&S25933[59], 5}, {NULL}};
const Segment SGI339954[] = {{&S25933[12], 14}, {&S25933[70], 31}, {NULL}};
const Segment SGI339955[] = {{&S25933[12], 14}, {NULL}};
const Segment SGI339956[] = {{&S25934[6], 4}, {NULL}};
const Segment SGI339957[] = {{&S25934[81], 5}, {NULL}};
const Segment SGI339958[] = {{&S25934[12], 14}, {&S25934[40], 11}, {NULL}};
const Segment SGI339959[] = {{&S25934[12], 14}, {NULL}};
const Segment SGI339960[] = {{&S25936[6], 4}, {NULL}};
const Segment SGI339961[] = {{&S25936[82], 5}, {NULL}};
const Segment SGI339962[] = {{&S25936[12], 14}, {&S25936[41], 11}, {NULL}};
const Segment SGI339963[] = {{&S25936[12], 14}, {NULL}};
const Segment SGI339964[] = {{&S25937[6], 4}, {NULL}};
const Segment SGI339965[] = {{&S25937[82], 5}, {NULL}};
const Segment SGI339966[] = {{&S25937[12], 14}, {&S25937[40], 12}, {NULL}};
const Segment SGI339967[] = {{&S25937[12], 14}, {NULL}};
const Segment SGI339968[] = {{&S25938[6], 4}, {NULL}};
const Segment SGI339969[] = {{&S25938[82], 5}, {NULL}};
const Segment SGI339970[] = {{&S25938[12], 14}, {&S25938[41], 11}, {NULL}};
const Segment SGI339971[] = {{&S25938[12], 14}, {NULL}};
const Segment SGI339972[] = {{&S25939[6], 4}, {NULL}};
const Segment SGI339973[] = {{&S25939[82], 5}, {NULL}};
const Segment SGI339974[] = {{&S25939[12], 14}, {&S25939[40], 12}, {NULL}};
const Segment SGI339975[] = {{&S25939[12], 14}, {NULL}};
const Segment SGI339976[] = {{&S25940[6], 4}, {NULL}};
const Segment SGI339977[] = {{&S25940[81], 5}, {NULL}};
const Segment SGI339978[] = {{&S25940[12], 14}, {&S25940[40], 11}, {NULL}};
const Segment SGI339979[] = {{&S25940[12], 14}, {NULL}};
const Segment SGI339980[] = {{&S25941[6], 4}, {NULL}};
const Segment SGI339981[] = {{&S25941[82], 5}, {NULL}};
const Segment SGI339982[] = {{&S25941[12], 14}, {&S25941[40], 12}, {NULL}};
const Segment SGI339983[] = {{&S25941[12], 14}, {NULL}};
const Segment SGI339984[] = {{&S25942[6], 4}, {NULL}};
const Segment SGI339985[] = {{&S25942[82], 5}, {NULL}};
const Segment SGI339986[] = {{&S25942[12], 14}, {&S25942[40], 12}, {NULL}};
const Segment SGI339987[] = {{&S25942[12], 14}, {NULL}};
const Segment SGI339988[] = {{&S43560[28], 3}, {NULL}};
const Segment SGI339989[] = {{&S43560[102], 5}, {NULL}};
const Segment SGI339990[] = {{&S25943[6], 4}, {NULL}};
const Segment SGI339991[] = {{&S25943[81], 5}, {NULL}};
const Segment SGI339992[] = {{&S25943[12], 14}, {&S25943[40], 11}, {NULL}};
const Segment SGI339993[] = {{&S25943[12], 14}, {NULL}};
const Segment SGI339994[] = {{&S43561[28], 3}, {NULL}};
const Segment SGI339995[] = {{&S43561[102], 5}, {NULL}};
const Segment SGI339996[] = {{&S25944[6], 4}, {NULL}};
const Segment SGI339997[] = {{&S25944[82], 5}, {NULL}};
const Segment SGI339998[] = {{&S25944[12], 14}, {&S25944[41], 11}, {NULL}};
const Segment SGI339999[] = {{&S25944[12], 14}, {NULL}};
const Segment SGI340000[] = {{&S25945[6], 4}, {NULL}};
const Segment SGI340001[] = {{&S25945[82], 5}, {NULL}};
const Segment SGI340002[] = {{&S25945[12], 14}, {&S25945[41], 11}, {NULL}};
const Segment SGI340003[] = {{&S25945[12], 14}, {NULL}};
const Segment SGI340004[] = {{&S25946[6], 4}, {NULL}};
const Segment SGI340005[] = {{&S25946[81], 5}, {NULL}};
const Segment SGI340006[] = {{&S25946[12], 14}, {&S25946[40], 11}, {NULL}};
const Segment SGI340007[] = {{&S25946[12], 14}, {NULL}};
const Segment SGI340008[] = {{&S25947[6], 4}, {NULL}};
const Segment SGI340009[] = {{&S25947[81], 5}, {NULL}};
const Segment SGI340010[] = {{&S25947[12], 14}, {&S25947[40], 11}, {NULL}};
const Segment SGI340011[] = {{&S25947[12], 14}, {NULL}};
const Segment SGI340012[] = {{&S25948[6], 4}, {NULL}};
const Segment SGI340013[] = {{&S25948[81], 5}, {NULL}};
const Segment SGI340014[] = {{&S25948[12], 14}, {&S25948[40], 11}, {NULL}};
const Segment SGI340015[] = {{&S25948[12], 14}, {NULL}};
const Segment SGI340016[] = {{&S25949[6], 4}, {NULL}};
const Segment SGI340017[] = {{&S25949[82], 5}, {NULL}};
const Segment SGI340018[] = {{&S25949[12], 14}, {&S25949[40], 12}, {NULL}};
const Segment SGI340019[] = {{&S25949[12], 14}, {NULL}};
const Segment SGI340020[] = {{&S25950[6], 4}, {NULL}};
const Segment SGI340021[] = {{&S25950[81], 5}, {NULL}};
const Segment SGI340022[] = {{&S25950[12], 14}, {&S25950[40], 11}, {NULL}};
const Segment SGI340023[] = {{&S25950[12], 14}, {NULL}};
const Segment SGI340024[] = {{&S25952[6], 4}, {NULL}};
const Segment SGI340025[] = {{&S25952[82], 5}, {NULL}};
const Segment SGI340026[] = {{&S25952[12], 14}, {&S25952[40], 12}, {NULL}};
const Segment SGI340027[] = {{&S25952[12], 14}, {NULL}};
const Segment SGI340028[] = {{&S25953[6], 4}, {NULL}};
const Segment SGI340029[] = {{&S25953[81], 5}, {NULL}};
const Segment SGI340030[] = {{&S25953[12], 14}, {&S25953[40], 11}, {NULL}};
const Segment SGI340031[] = {{&S25953[12], 14}, {NULL}};
const Segment SGI340032[] = {{&S25954[6], 4}, {NULL}};
const Segment SGI340033[] = {{&S25954[81], 5}, {NULL}};
const Segment SGI340034[] = {{&S25954[12], 14}, {&S25954[40], 11}, {NULL}};
const Segment SGI340035[] = {{&S25954[12], 14}, {NULL}};
const Segment SGI340036[] = {{&S25955[6], 4}, {NULL}};
const Segment SGI340037[] = {{&S25955[82], 5}, {NULL}};
const Segment SGI340038[] = {{&S25955[12], 14}, {&S25955[41], 11}, {NULL}};
const Segment SGI340039[] = {{&S25955[12], 14}, {NULL}};
const Segment SGI340040[] = {{&S25956[6], 4}, {NULL}};
const Segment SGI340041[] = {{&S25956[81], 5}, {NULL}};
const Segment SGI340042[] = {{&S25956[12], 14}, {&S25956[40], 11}, {NULL}};
const Segment SGI340043[] = {{&S25956[12], 14}, {NULL}};
const Segment SGI340044[] = {{&S25957[6], 4}, {NULL}};
const Segment SGI340045[] = {{&S25957[82], 5}, {NULL}};
const Segment SGI340046[] = {{&S25957[12], 14}, {&S25957[40], 12}, {NULL}};
const Segment SGI340047[] = {{&S25957[12], 14}, {NULL}};
const Segment SGI340048[] = {{&S25958[6], 4}, {NULL}};
const Segment SGI340049[] = {{&S25958[82], 5}, {NULL}};
const Segment SGI340050[] = {{&S25958[12], 14}, {&S25958[41], 11}, {NULL}};
const Segment SGI340051[] = {{&S25958[12], 14}, {NULL}};
const Segment SGI340052[] = {{&S25959[6], 4}, {NULL}};
const Segment SGI340053[] = {{&S25959[81], 5}, {NULL}};
const Segment SGI340054[] = {{&S25959[12], 14}, {&S25959[40], 11}, {NULL}};
const Segment SGI340055[] = {{&S25959[12], 14}, {NULL}};
const Segment SGI340056[] = {{&S25960[6], 4}, {NULL}};
const Segment SGI340057[] = {{&S25960[81], 5}, {NULL}};
const Segment SGI340058[] = {{&S25960[12], 14}, {&S25960[40], 11}, {NULL}};
const Segment SGI340059[] = {{&S25960[12], 14}, {NULL}};
const Segment SGI340060[] = {{&S25961[6], 4}, {NULL}};
const Segment SGI340061[] = {{&S25961[81], 5}, {NULL}};
const Segment SGI340062[] = {{&S25961[12], 14}, {&S25961[40], 11}, {NULL}};
const Segment SGI340063[] = {{&S25961[12], 14}, {NULL}};
const Segment SGI340064[] = {{&S25962[6], 4}, {NULL}};
const Segment SGI340065[] = {{&S25962[82], 5}, {NULL}};
const Segment SGI340066[] = {{&S25962[12], 14}, {&S25962[40], 12}, {NULL}};
const Segment SGI340067[] = {{&S25962[12], 14}, {NULL}};
const Segment SGI340068[] = {{&S25963[6], 4}, {NULL}};
const Segment SGI340069[] = {{&S25963[82], 5}, {NULL}};
const Segment SGI340070[] = {{&S25963[12], 14}, {&S25963[40], 12}, {NULL}};
const Segment SGI340071[] = {{&S25963[12], 14}, {NULL}};
const Segment SGI340072[] = {{&S25964[6], 4}, {NULL}};
const Segment SGI340073[] = {{&S25964[81], 5}, {NULL}};
const Segment SGI340074[] = {{&S25964[12], 14}, {&S25964[40], 11}, {NULL}};
const Segment SGI340075[] = {{&S25964[12], 14}, {NULL}};
const Segment SGI340076[] = {{&S25965[6], 4}, {NULL}};
const Segment SGI340077[] = {{&S25965[82], 5}, {NULL}};
const Segment SGI340078[] = {{&S25965[12], 14}, {&S25965[40], 12}, {NULL}};
const Segment SGI340079[] = {{&S25965[12], 14}, {NULL}};
const Segment SGI340080[] = {{&S25966[6], 4}, {NULL}};
const Segment SGI340081[] = {{&S25966[82], 5}, {NULL}};
const Segment SGI340082[] = {{&S25966[12], 14}, {&S25966[40], 12}, {NULL}};
const Segment SGI340083[] = {{&S25966[12], 14}, {NULL}};
const Segment SGI340084[] = {{&S25967[6], 4}, {NULL}};
const Segment SGI340085[] = {{&S25967[82], 5}, {NULL}};
const Segment SGI340086[] = {{&S25967[12], 14}, {&S25967[41], 11}, {NULL}};
const Segment SGI340087[] = {{&S25967[12], 14}, {NULL}};
const Segment SGI340088[] = {{&S25968[6], 4}, {NULL}};
const Segment SGI340089[] = {{&S25968[82], 5}, {NULL}};
const Segment SGI340090[] = {{&S25968[12], 14}, {&S25968[41], 11}, {NULL}};
const Segment SGI340091[] = {{&S25968[12], 14}, {NULL}};
const Segment SGI340092[] = {{&S25969[6], 4}, {NULL}};
const Segment SGI340093[] = {{&S25969[82], 5}, {NULL}};
const Segment SGI340094[] = {{&S25969[12], 14}, {&S25969[41], 11}, {NULL}};
const Segment SGI340095[] = {{&S25969[12], 14}, {NULL}};
const Segment SGI340096[] = {{&S25970[6], 4}, {NULL}};
const Segment SGI340097[] = {{&S25970[81], 5}, {NULL}};
const Segment SGI340098[] = {{&S25970[12], 14}, {&S25970[40], 11}, {NULL}};
const Segment SGI340099[] = {{&S25970[12], 14}, {NULL}};
const Segment SGI340100[] = {{&S25971[6], 4}, {NULL}};
const Segment SGI340101[] = {{&S25971[81], 5}, {NULL}};
const Segment SGI340102[] = {{&S25971[12], 14}, {&S25971[40], 11}, {NULL}};
const Segment SGI340103[] = {{&S25971[12], 14}, {NULL}};
const Segment SGI340104[] = {{&S25972[6], 4}, {NULL}};
const Segment SGI340105[] = {{&S25972[82], 5}, {NULL}};
const Segment SGI340106[] = {{&S25972[12], 14}, {&S25972[40], 12}, {NULL}};
const Segment SGI340107[] = {{&S25972[12], 14}, {NULL}};
const Segment SGI340108[] = {{&S25973[6], 4}, {NULL}};
const Segment SGI340109[] = {{&S25973[82], 5}, {NULL}};
const Segment SGI340110[] = {{&S25973[12], 14}, {&S25973[40], 12}, {NULL}};
const Segment SGI340111[] = {{&S25973[12], 14}, {NULL}};
const Segment SGI340112[] = {{&S25974[6], 4}, {NULL}};
const Segment SGI340113[] = {{&S25974[82], 5}, {NULL}};
const Segment SGI340114[] = {{&S25974[12], 14}, {&S25974[41], 11}, {NULL}};
const Segment SGI340115[] = {{&S25974[12], 14}, {NULL}};
const Segment SGI340116[] = {{&S25975[6], 4}, {NULL}};
const Segment SGI340117[] = {{&S25975[81], 5}, {NULL}};
const Segment SGI340118[] = {{&S25975[12], 14}, {&S25975[40], 11}, {NULL}};
const Segment SGI340119[] = {{&S25975[12], 14}, {NULL}};
const Segment SGI340120[] = {{&S25976[6], 4}, {NULL}};
const Segment SGI340121[] = {{&S25976[82], 5}, {NULL}};
const Segment SGI340122[] = {{&S25976[12], 14}, {&S25976[41], 11}, {NULL}};
const Segment SGI340123[] = {{&S25976[12], 14}, {NULL}};
const Segment SGI340124[] = {{&S25977[6], 4}, {NULL}};
const Segment SGI340125[] = {{&S25977[81], 5}, {NULL}};
const Segment SGI340126[] = {{&S25977[12], 14}, {&S25977[40], 11}, {NULL}};
const Segment SGI340127[] = {{&S25977[12], 14}, {NULL}};
const Segment SGI340128[] = {{&S25978[6], 4}, {NULL}};
const Segment SGI340129[] = {{&S25978[81], 5}, {NULL}};
const Segment SGI340130[] = {{&S25978[12], 14}, {&S25978[40], 11}, {NULL}};
const Segment SGI340131[] = {{&S25978[12], 14}, {NULL}};
const Segment SGI340132[] = {{&S25979[6], 4}, {NULL}};
const Segment SGI340133[] = {{&S25979[81], 5}, {NULL}};
const Segment SGI340134[] = {{&S25979[12], 14}, {&S25979[40], 11}, {NULL}};
const Segment SGI340135[] = {{&S25979[12], 14}, {NULL}};
const Segment SGI340136[] = {{&S25980[6], 4}, {NULL}};
const Segment SGI340137[] = {{&S25980[81], 5}, {NULL}};
const Segment SGI340138[] = {{&S25980[12], 14}, {&S25980[40], 11}, {NULL}};
const Segment SGI340139[] = {{&S25980[12], 14}, {NULL}};
const Segment SGI340140[] = {{&S25981[6], 4}, {NULL}};
const Segment SGI340141[] = {{&S25981[81], 5}, {NULL}};
const Segment SGI340142[] = {{&S25981[12], 14}, {&S25981[40], 11}, {NULL}};
const Segment SGI340143[] = {{&S25981[12], 14}, {NULL}};
const Segment SGI340144[] = {{&S25982[6], 4}, {NULL}};
const Segment SGI340145[] = {{&S25982[81], 5}, {NULL}};
const Segment SGI340146[] = {{&S25982[12], 14}, {&S25982[40], 11}, {NULL}};
const Segment SGI340147[] = {{&S25982[12], 14}, {NULL}};
const Segment SGI340148[] = {{&S25983[6], 4}, {NULL}};
const Segment SGI340149[] = {{&S25983[82], 5}, {NULL}};
const Segment SGI340150[] = {{&S25983[12], 14}, {&S25983[40], 12}, {NULL}};
const Segment SGI340151[] = {{&S25983[12], 14}, {NULL}};
const Segment SGI340152[] = {{&S25984[6], 4}, {NULL}};
const Segment SGI340153[] = {{&S25984[81], 5}, {NULL}};
const Segment SGI340154[] = {{&S25984[12], 14}, {&S25984[40], 11}, {NULL}};
const Segment SGI340155[] = {{&S25984[12], 14}, {NULL}};
const Segment SGI340156[] = {{&S25985[6], 4}, {NULL}};
const Segment SGI340157[] = {{&S25985[59], 5}, {NULL}};
const Segment SGI340158[] = {{&S25985[12], 14}, {&S25985[70], 4}, {NULL}};
const Segment SGI340159[] = {{&S25985[12], 14}, {NULL}};
const Segment SGI340160[] = {{&S25986[6], 4}, {NULL}};
const Segment SGI340161[] = {{&S25986[81], 5}, {NULL}};
const Segment SGI340162[] = {{&S25986[12], 14}, {&S25986[40], 11}, {NULL}};
const Segment SGI340163[] = {{&S25986[12], 14}, {NULL}};
const Segment SGI340164[] = {{&S25987[6], 4}, {NULL}};
const Segment SGI340165[] = {{&S25987[81], 5}, {NULL}};
const Segment SGI340166[] = {{&S25987[12], 14}, {&S25987[40], 11}, {NULL}};
const Segment SGI340167[] = {{&S25987[12], 14}, {NULL}};
const Segment SGI340168[] = {{&S25988[6], 4}, {NULL}};
const Segment SGI340169[] = {{&S25988[82], 5}, {NULL}};
const Segment SGI340170[] = {{&S25988[12], 14}, {&S25988[40], 12}, {NULL}};
const Segment SGI340171[] = {{&S25988[12], 14}, {NULL}};
const Segment SGI340172[] = {{&S25989[6], 4}, {NULL}};
const Segment SGI340173[] = {{&S25989[81], 5}, {NULL}};
const Segment SGI340174[] = {{&S25989[12], 14}, {&S25989[40], 11}, {NULL}};
const Segment SGI340175[] = {{&S25989[12], 14}, {NULL}};
const Segment SGI340176[] = {{&S25990[6], 4}, {NULL}};
const Segment SGI340177[] = {{&S25990[81], 5}, {NULL}};
const Segment SGI340178[] = {{&S25990[12], 14}, {&S25990[40], 11}, {NULL}};
const Segment SGI340179[] = {{&S25990[12], 14}, {NULL}};
const Segment SGI340180[] = {{&S25991[6], 4}, {NULL}};
const Segment SGI340181[] = {{&S25991[81], 5}, {NULL}};
const Segment SGI340182[] = {{&S25991[12], 14}, {&S25991[40], 11}, {NULL}};
const Segment SGI340183[] = {{&S25991[12], 14}, {NULL}};
const Segment SGI340184[] = {{&S25992[6], 4}, {NULL}};
const Segment SGI340185[] = {{&S25992[81], 5}, {NULL}};
const Segment SGI340186[] = {{&S25992[12], 14}, {&S25992[40], 11}, {NULL}};
const Segment SGI340187[] = {{&S25992[12], 14}, {NULL}};
const Segment SGI340188[] = {{&S25993[6], 4}, {NULL}};
const Segment SGI340189[] = {{&S25993[81], 5}, {NULL}};
const Segment SGI340190[] = {{&S25993[12], 14}, {&S25993[40], 11}, {NULL}};
const Segment SGI340191[] = {{&S25993[12], 14}, {NULL}};
const Segment SGI340192[] = {{&S25994[6], 4}, {NULL}};
const Segment SGI340193[] = {{&S25994[81], 5}, {NULL}};
const Segment SGI340194[] = {{&S25994[12], 14}, {&S25994[40], 11}, {NULL}};
const Segment SGI340195[] = {{&S25994[12], 14}, {NULL}};
const Segment SGI340196[] = {{&S25995[6], 4}, {NULL}};
const Segment SGI340197[] = {{&S25995[81], 5}, {NULL}};
const Segment SGI340198[] = {{&S25995[12], 14}, {&S25995[40], 11}, {NULL}};
const Segment SGI340199[] = {{&S25995[12], 14}, {NULL}};
const Segment SGI340200[] = {{&S25996[6], 4}, {NULL}};
const Segment SGI340201[] = {{&S25996[81], 5}, {NULL}};
const Segment SGI340202[] = {{&S25996[12], 14}, {&S25996[40], 11}, {NULL}};
const Segment SGI340203[] = {{&S25996[12], 14}, {NULL}};
const Segment SGI340204[] = {{&S25997[6], 4}, {NULL}};
const Segment SGI340205[] = {{&S25997[81], 5}, {NULL}};
const Segment SGI340206[] = {{&S25997[12], 14}, {&S25997[40], 11}, {NULL}};
const Segment SGI340207[] = {{&S25997[12], 14}, {NULL}};
const Segment SGI340208[] = {{&S25998[6], 4}, {NULL}};
const Segment SGI340209[] = {{&S25998[81], 5}, {NULL}};
const Segment SGI340210[] = {{&S25998[12], 14}, {&S25998[40], 11}, {NULL}};
const Segment SGI340211[] = {{&S25998[12], 14}, {NULL}};
const Segment SGI340212[] = {{&S25999[6], 4}, {NULL}};
const Segment SGI340213[] = {{&S25999[81], 5}, {NULL}};
const Segment SGI340214[] = {{&S25999[12], 14}, {&S25999[40], 11}, {NULL}};
const Segment SGI340215[] = {{&S25999[12], 14}, {NULL}};
const Segment SGI340216[] = {{&S26000[6], 4}, {NULL}};
const Segment SGI340217[] = {{&S26000[81], 5}, {NULL}};
const Segment SGI340218[] = {{&S26000[12], 14}, {&S26000[40], 11}, {NULL}};
const Segment SGI340219[] = {{&S26000[12], 14}, {NULL}};
const Segment SGI340220[] = {{&S38925[5], 3}, {NULL}};
const Segment SGI340221[] = {{&S38925[45], 3}, {NULL}};
const Segment SGI340222[] = {{&S26001[6], 4}, {NULL}};
const Segment SGI340223[] = {{&S26001[81], 5}, {NULL}};
const Segment SGI340224[] = {{&S26001[12], 14}, {&S26001[40], 11}, {NULL}};
const Segment SGI340225[] = {{&S26001[12], 14}, {NULL}};
const Segment SGI340226[] = {{&S38926[46], 5}, {NULL}};
const Segment SGI340227[] = {{&S26002[6], 4}, {NULL}};
const Segment SGI340228[] = {{&S38926[37], 3}, {NULL}};
const Segment SGI340229[] = {{&S26002[81], 5}, {NULL}};
const Segment SGI340230[] = {{&S26002[12], 14}, {&S26002[40], 11}, {NULL}};
const Segment SGI340231[] = {{&S26002[12], 14}, {NULL}};
const Segment SGI340232[] = {{&S26003[6], 4}, {NULL}};
const Segment SGI340233[] = {{&S38927[46], 5}, {NULL}};
const Segment SGI340234[] = {{&S26003[81], 5}, {NULL}};
const Segment SGI340235[] = {{&S26003[12], 14}, {&S26003[40], 11}, {NULL}};
const Segment SGI340236[] = {{&S38927[37], 3}, {NULL}};
const Segment SGI340237[] = {{&S26003[12], 14}, {NULL}};
const Segment SGI340238[] = {{&S26004[6], 4}, {NULL}};
const Segment SGI340239[] = {{&S26004[81], 5}, {NULL}};
const Segment SGI340240[] = {{&S26004[12], 14}, {&S26004[40], 11}, {NULL}};
const Segment SGI340241[] = {{&S26004[12], 14}, {NULL}};
const Segment SGI340242[] = {{&S38928[46], 5}, {&S38928[146], 7}, {NULL}};
const Segment SGI340243[] = {{&S38928[37], 3}, {NULL}};
const Segment SGI340244[] = {{&S26005[6], 4}, {NULL}};
const Segment SGI340245[] = {{&S26005[81], 5}, {NULL}};
const Segment SGI340246[] = {{&S26005[12], 14}, {&S26005[40], 11}, {NULL}};
const Segment SGI340247[] = {{&S26005[12], 14}, {NULL}};
const Segment SGI340248[] = {{&S38929[46], 5}, {&S38929[146], 7}, {NULL}};
const Segment SGI340249[] = {{&S38929[37], 3}, {NULL}};
const Segment SGI340250[] = {{&S26006[6], 4}, {NULL}};
const Segment SGI340251[] = {{&S26006[81], 5}, {NULL}};
const Segment SGI340252[] = {{&S26006[12], 14}, {&S26006[40], 11}, {NULL}};
const Segment SGI340253[] = {{&S26006[12], 14}, {NULL}};
const Segment SGI340254[] = {{&S26007[6], 4}, {NULL}};
const Segment SGI340255[] = {{&S26007[81], 5}, {NULL}};
const Segment SGI340256[] = {{&S26007[12], 14}, {&S26007[40], 11}, {NULL}};
const Segment SGI340257[] = {{&S26007[12], 14}, {NULL}};
const Segment SGI340258[] = {{&S26008[6], 4}, {NULL}};
const Segment SGI340259[] = {{&S26008[81], 5}, {NULL}};
const Segment SGI340260[] = {{&S26008[12], 14}, {&S26008[40], 11}, {NULL}};
const Segment SGI340261[] = {{&S26008[12], 14}, {NULL}};
const Segment SGI340262[] = {{&S26009[6], 4}, {NULL}};
const Segment SGI340263[] = {{&S26009[81], 5}, {NULL}};
const Segment SGI340264[] = {{&S26009[12], 14}, {&S26009[40], 11}, {NULL}};
const Segment SGI340265[] = {{&S26009[12], 14}, {NULL}};
const Segment SGI340266[] = {{&S26010[6], 4}, {NULL}};
const Segment SGI340267[] = {{&S26010[81], 5}, {NULL}};
const Segment SGI340268[] = {{&S26010[12], 14}, {&S26010[40], 11}, {NULL}};
const Segment SGI340269[] = {{&S26010[12], 14}, {NULL}};
const Segment SGI340270[] = {{&S26011[6], 4}, {NULL}};
const Segment SGI340271[] = {{&S26011[81], 5}, {NULL}};
const Segment SGI340272[] = {{&S26011[12], 14}, {&S26011[40], 11}, {NULL}};
const Segment SGI340273[] = {{&S26011[12], 14}, {NULL}};
const Segment SGI340274[] = {{&S26012[6], 4}, {NULL}};
const Segment SGI340275[] = {{&S26012[81], 5}, {NULL}};
const Segment SGI340276[] = {{&S26012[12], 14}, {&S26012[40], 11}, {NULL}};
const Segment SGI340277[] = {{&S26012[12], 14}, {NULL}};
const Segment SGI340278[] = {{&S52402[5], 7}, {&S52402[5], 7}, {NULL}};
const Segment SGI340279[] = {{&S52403[5], 7}, {NULL}};
const Segment SGI340280[] = {{&S52404[5], 5}, {NULL}};
const Segment SGI340281[] = {{&S52405[5], 5}, {&S52405[5], 5}, {NULL}};
const Segment SGI340282[] = {{&S52406[5], 8}, {NULL}};
const Segment SGI340283[] = {{&S52407[5], 12}, {NULL}};
const Segment SGI340284[] = {{&S52408[5], 10}, {NULL}};
const Segment SGI340285[] = {{&S26131[6], 4}, {NULL}};
const Segment SGI340286[] = {{&S26131[59], 5}, {NULL}};
const Segment SGI340287[] = {{&S26131[12], 14}, {NULL}};
const Segment SGI340288[] = {{&S26132[6], 4}, {NULL}};
const Segment SGI340289[] = {{&S26132[82], 5}, {NULL}};
const Segment SGI340290[] = {{&S26132[12], 14}, {&S26132[40], 12}, {NULL}};
const Segment SGI340291[] = {{&S26132[12], 14}, {NULL}};
const Segment SGI340292[] = {{&S26133[6], 4}, {NULL}};
const Segment SGI340293[] = {{&S26133[82], 5}, {NULL}};
const Segment SGI340294[] = {{&S26133[12], 14}, {&S26133[40], 12}, {NULL}};
const Segment SGI340295[] = {{&S26133[12], 14}, {NULL}};
const Segment SGI340296[] = {{&S26135[6], 4}, {NULL}};
const Segment SGI340297[] = {{&S26135[81], 5}, {NULL}};
const Segment SGI340298[] = {{&S26135[12], 14}, {&S26135[40], 11}, {NULL}};
const Segment SGI340299[] = {{&S26135[12], 14}, {NULL}};
const Segment SGI340300[] = {{&S26136[6], 4}, {NULL}};
const Segment SGI340301[] = {{&S26136[81], 5}, {NULL}};
const Segment SGI340302[] = {{&S26136[12], 14}, {&S26136[40], 11}, {NULL}};
const Segment SGI340303[] = {{&S26136[12], 14}, {NULL}};
const Segment SGI340304[] = {{&S26137[6], 4}, {NULL}};
const Segment SGI340305[] = {{&S26137[111], 5}, {NULL}};
const Segment SGI340306[] = {{&S26137[12], 14}, {&S26137[44], 10}, {NULL}};
const Segment SGI340307[] = {{&S26137[12], 14}, {&S26137[66], 14}, {NULL}};
const Segment SGI340308[] = {{&S26138[6], 4}, {NULL}};
const Segment SGI340309[] = {{&S26138[81], 5}, {NULL}};
const Segment SGI340310[] = {{&S26138[12], 14}, {&S26138[40], 11}, {NULL}};
const Segment SGI340311[] = {{&S26138[12], 14}, {NULL}};
const Segment SGI340312[] = {{&S26139[6], 4}, {NULL}};
const Segment SGI340313[] = {{&S26139[82], 5}, {NULL}};
const Segment SGI340314[] = {{&S26139[12], 14}, {&S26139[41], 11}, {NULL}};
const Segment SGI340315[] = {{&S26139[12], 14}, {NULL}};
const Segment SGI340316[] = {{&S26140[6], 4}, {NULL}};
const Segment SGI340317[] = {{&S26140[82], 5}, {NULL}};
const Segment SGI340318[] = {{&S26140[12], 14}, {&S26140[40], 12}, {NULL}};
const Segment SGI340319[] = {{&S26140[12], 14}, {NULL}};
const Segment SGI340320[] = {{&S26141[6], 4}, {NULL}};
const Segment SGI340321[] = {{&S26141[82], 5}, {NULL}};
const Segment SGI340322[] = {{&S26141[12], 14}, {&S26141[41], 11}, {NULL}};
const Segment SGI340323[] = {{&S26141[12], 14}, {NULL}};
const Segment SGI340324[] = {{&S26142[6], 4}, {NULL}};
const Segment SGI340325[] = {{&S26142[81], 5}, {NULL}};
const Segment SGI340326[] = {{&S26142[12], 14}, {&S26142[40], 11}, {NULL}};
const Segment SGI340327[] = {{&S26142[12], 14}, {NULL}};
const Segment SGI340328[] = {{&S26143[6], 4}, {NULL}};
const Segment SGI340329[] = {{&S26143[81], 5}, {NULL}};
const Segment SGI340330[] = {{&S26143[12], 14}, {&S26143[40], 11}, {NULL}};
const Segment SGI340331[] = {{&S26143[12], 14}, {NULL}};
const Segment SGI340332[] = {{&S26144[6], 4}, {NULL}};
const Segment SGI340333[] = {{&S26144[59], 5}, {NULL}};
const Segment SGI340334[] = {{&S26144[12], 14}, {&S26144[70], 31}, {NULL}};
const Segment SGI340335[] = {{&S26144[12], 14}, {NULL}};
const Segment SGI340336[] = {{&S26145[6], 4}, {NULL}};
const Segment SGI340337[] = {{&S26145[82], 5}, {NULL}};
const Segment SGI340338[] = {{&S26145[12], 14}, {&S26145[40], 12}, {NULL}};
const Segment SGI340339[] = {{&S26145[12], 14}, {NULL}};
const Segment SGI340340[] = {{&S26146[6], 4}, {NULL}};
const Segment SGI340341[] = {{&S26146[81], 5}, {NULL}};
const Segment SGI340342[] = {{&S26146[12], 14}, {&S26146[40], 11}, {NULL}};
const Segment SGI340343[] = {{&S26146[12], 14}, {NULL}};
const Segment SGI340344[] = {{&S26147[6], 4}, {NULL}};
const Segment SGI340345[] = {{&S26147[81], 5}, {NULL}};
const Segment SGI340346[] = {{&S26147[12], 14}, {&S26147[40], 11}, {NULL}};
const Segment SGI340347[] = {{&S26147[12], 14}, {NULL}};
const Segment SGI340348[] = {{&S26149[6], 4}, {NULL}};
const Segment SGI340349[] = {{&S26149[81], 5}, {NULL}};
const Segment SGI340350[] = {{&S26149[12], 14}, {&S26149[40], 11}, {NULL}};
const Segment SGI340351[] = {{&S26149[12], 14}, {NULL}};
const Segment SGI340352[] = {{&S26151[6], 4}, {NULL}};
const Segment SGI340353[] = {{&S26151[82], 5}, {NULL}};
const Segment SGI340354[] = {{&S26151[12], 14}, {&S26151[40], 12}, {NULL}};
const Segment SGI340355[] = {{&S26151[12], 14}, {NULL}};
const Segment SGI340356[] = {{&S26152[6], 4}, {NULL}};
const Segment SGI340357[] = {{&S26152[81], 5}, {NULL}};
const Segment SGI340358[] = {{&S26152[12], 14}, {&S26152[40], 11}, {NULL}};
const Segment SGI340359[] = {{&S26152[12], 14}, {NULL}};
const Segment SGI340360[] = {{&S26153[6], 4}, {NULL}};
const Segment SGI340361[] = {{&S26153[82], 5}, {NULL}};
const Segment SGI340362[] = {{&S26153[12], 14}, {&S26153[40], 12}, {NULL}};
const Segment SGI340363[] = {{&S26153[12], 14}, {NULL}};
const Segment SGI340364[] = {{&S26155[6], 4}, {NULL}};
const Segment SGI340365[] = {{&S26155[82], 5}, {NULL}};
const Segment SGI340366[] = {{&S26155[12], 14}, {&S26155[40], 12}, {NULL}};
const Segment SGI340367[] = {{&S26155[12], 14}, {NULL}};
const Segment SGI340368[] = {{&S26156[6], 4}, {NULL}};
const Segment SGI340369[] = {{&S26156[81], 5}, {NULL}};
const Segment SGI340370[] = {{&S26156[12], 14}, {&S26156[40], 11}, {NULL}};
const Segment SGI340371[] = {{&S26156[12], 14}, {NULL}};
const Segment SGI340372[] = {{&S26157[6], 4}, {NULL}};
const Segment SGI340373[] = {{&S26157[59], 5}, {NULL}};
const Segment SGI340374[] = {{&S26157[12], 14}, {&S26157[70], 4}, {NULL}};
const Segment SGI340375[] = {{&S26157[12], 14}, {NULL}};
const Segment SGI340376[] = {{&S26158[6], 4}, {NULL}};
const Segment SGI340377[] = {{&S26158[81], 5}, {NULL}};
const Segment SGI340378[] = {{&S26158[12], 14}, {&S26158[40], 11}, {NULL}};
const Segment SGI340379[] = {{&S26158[12], 14}, {NULL}};
const Segment SGI340380[] = {{&S26159[6], 5}, {NULL}};
const Segment SGI340381[] = {{&S26159[13], 14}, {NULL}};
const Segment SGI340382[] = {{&S26160[6], 4}, {NULL}};
const Segment SGI340383[] = {{&S26160[81], 5}, {NULL}};
const Segment SGI340384[] = {{&S26160[12], 14}, {&S26160[40], 11}, {NULL}};
const Segment SGI340385[] = {{&S26160[12], 14}, {NULL}};
const Segment SGI340386[] = {{&S26161[6], 4}, {NULL}};
const Segment SGI340387[] = {{&S26161[81], 5}, {NULL}};
const Segment SGI340388[] = {{&S26161[12], 14}, {&S26161[40], 11}, {NULL}};
const Segment SGI340389[] = {{&S26161[12], 14}, {NULL}};
const Segment SGI340390[] = {{&S26162[6], 4}, {NULL}};
const Segment SGI340391[] = {{&S26162[81], 5}, {NULL}};
const Segment SGI340392[] = {{&S26162[12], 14}, {&S26162[40], 11}, {NULL}};
const Segment SGI340393[] = {{&S26162[12], 14}, {NULL}};
const Segment SGI340394[] = {{&S26163[6], 4}, {NULL}};
const Segment SGI340395[] = {{&S26163[81], 5}, {NULL}};
const Segment SGI340396[] = {{&S26163[12], 14}, {&S26163[40], 11}, {NULL}};
const Segment SGI340397[] = {{&S26163[12], 14}, {NULL}};
const Segment SGI340398[] = {{&S26164[6], 4}, {NULL}};
const Segment SGI340399[] = {{&S26164[82], 5}, {NULL}};
const Segment SGI340400[] = {{&S26164[12], 14}, {&S26164[41], 11}, {NULL}};
const Segment SGI340401[] = {{&S26164[12], 14}, {NULL}};
const Segment SGI340402[] = {{&S26165[6], 4}, {NULL}};
const Segment SGI340403[] = {{&S26165[81], 5}, {NULL}};
const Segment SGI340404[] = {{&S26165[12], 14}, {&S26165[40], 11}, {NULL}};
const Segment SGI340405[] = {{&S26165[12], 14}, {NULL}};
const Segment SGI340406[] = {{&S26166[6], 4}, {NULL}};
const Segment SGI340407[] = {{&S26166[81], 5}, {NULL}};
const Segment SGI340408[] = {{&S26166[12], 14}, {&S26166[40], 11}, {NULL}};
const Segment SGI340409[] = {{&S26166[12], 14}, {NULL}};
const Segment SGI340410[] = {{&S26167[6], 4}, {NULL}};
const Segment SGI340411[] = {{&S26167[81], 5}, {NULL}};
const Segment SGI340412[] = {{&S26167[12], 14}, {&S26167[40], 11}, {NULL}};
const Segment SGI340413[] = {{&S26167[12], 14}, {NULL}};
const Segment SGI340414[] = {{&S26168[6], 4}, {NULL}};
const Segment SGI340415[] = {{&S26168[81], 5}, {NULL}};
const Segment SGI340416[] = {{&S26168[12], 14}, {&S26168[40], 11}, {NULL}};
const Segment SGI340417[] = {{&S26168[12], 14}, {NULL}};
const Segment SGI340418[] = {{&S26169[6], 4}, {NULL}};
const Segment SGI340419[] = {{&S26169[82], 5}, {NULL}};
const Segment SGI340420[] = {{&S26169[12], 14}, {&S26169[41], 11}, {NULL}};
const Segment SGI340421[] = {{&S26169[12], 14}, {NULL}};
const Segment SGI340422[] = {{&S26170[6], 4}, {NULL}};
const Segment SGI340423[] = {{&S26170[82], 5}, {NULL}};
const Segment SGI340424[] = {{&S26170[12], 14}, {&S26170[40], 12}, {NULL}};
const Segment SGI340425[] = {{&S26170[12], 14}, {NULL}};
const Segment SGI340426[] = {{&S26171[6], 4}, {NULL}};
const Segment SGI340427[] = {{&S26171[82], 5}, {NULL}};
const Segment SGI340428[] = {{&S26171[12], 14}, {&S26171[41], 11}, {NULL}};
const Segment SGI340429[] = {{&S26171[12], 14}, {NULL}};
const Segment SGI340430[] = {{&S26172[6], 4}, {NULL}};
const Segment SGI340431[] = {{&S26172[82], 5}, {NULL}};
const Segment SGI340432[] = {{&S26172[12], 14}, {&S26172[40], 12}, {NULL}};
const Segment SGI340433[] = {{&S26172[12], 14}, {NULL}};
const Segment SGI340434[] = {{&S26173[6], 4}, {NULL}};
const Segment SGI340435[] = {{&S26173[82], 5}, {NULL}};
const Segment SGI340436[] = {{&S26173[12], 14}, {&S26173[40], 12}, {NULL}};
const Segment SGI340437[] = {{&S26173[12], 14}, {NULL}};
const Segment SGI340438[] = {{&S26174[6], 4}, {NULL}};
const Segment SGI340439[] = {{&S26174[81], 5}, {NULL}};
const Segment SGI340440[] = {{&S26174[12], 14}, {&S26174[40], 11}, {NULL}};
const Segment SGI340441[] = {{&S26174[12], 14}, {NULL}};
const Segment SGI340442[] = {{&S52517[42], 16}, {NULL}};
const Segment SGI340443[] = {{&S52517[6], 3}, {NULL}};
const Segment SGI340444[] = {{&S52517[79], 3}, {NULL}};
const Segment SGI340445[] = {{&S52518[5], 16}, {NULL}};
const Segment SGI340446[] = {{&S52518[27], 3}, {NULL}};
const Segment SGI340447[] = {{&S52519[41], 7}, {NULL}};
const Segment SGI340448[] = {{&S52519[6], 3}, {NULL}};
const Segment SGI340449[] = {{&S52519[69], 3}, {NULL}};
const Segment SGI340450[] = {{&S52520[42], 17}, {NULL}};
const Segment SGI340451[] = {{&S52520[6], 3}, {NULL}};
const Segment SGI340452[] = {{&S52520[80], 3}, {NULL}};
const Segment SGI340453[] = {{&S52521[5], 17}, {NULL}};
const Segment SGI340454[] = {{&S52521[28], 3}, {NULL}};
const Segment SGI340455[] = {{&S52543[42], 5}, {NULL}};
const Segment SGI340456[] = {{&S52543[6], 3}, {NULL}};
const Segment SGI340457[] = {{&S52543[68], 3}, {NULL}};
const Segment SGI340458[] = {{&S52544[41], 5}, {NULL}};
const Segment SGI340459[] = {{&S52544[6], 3}, {NULL}};
const Segment SGI340460[] = {{&S52544[67], 3}, {NULL}};
const Segment SGI340461[] = {{&S52545[42], 7}, {NULL}};
const Segment SGI340462[] = {{&S52545[6], 3}, {NULL}};
const Segment SGI340463[] = {{&S52545[70], 3}, {NULL}};
const Segment SGI340464[] = {{&S52546[42], 6}, {NULL}};
const Segment SGI340465[] = {{&S52546[6], 3}, {NULL}};
const Segment SGI340466[] = {{&S52546[69], 3}, {NULL}};
const Segment SGI340467[] = {{&S52547[5], 5}, {NULL}};
const Segment SGI340468[] = {{&S52547[16], 3}, {NULL}};
const Segment SGI340469[] = {{&S52548[5], 4}, {NULL}};
const Segment SGI340470[] = {{&S52548[15], 3}, {NULL}};
const Segment SGI340471[] = {{&S52549[43], 4}, {NULL}};
const Segment SGI340472[] = {{&S52549[6], 3}, {NULL}};
const Segment SGI340473[] = {{&S52549[68], 3}, {NULL}};
const Segment SGI340474[] = {{&S52550[43], 5}, {NULL}};
const Segment SGI340475[] = {{&S52550[6], 3}, {NULL}};
const Segment SGI340476[] = {{&S52550[69], 3}, {NULL}};
const Segment SGI340477[] = {{&S52629[5], 4}, {NULL}};
const Segment SGI340478[] = {{&S52630[5], 4}, {&S52630[5], 4}, {NULL}};
const Segment SGI340479[] = {{&S52631[5], 9}, {NULL}};
const Segment SGI340480[] = {{&S26314[6], 4}, {NULL}};
const Segment SGI340481[] = {{&S26314[57], 5}, {NULL}};
const Segment SGI340482[] = {{&S26314[12], 14}, {NULL}};
const Segment SGI340483[] = {{&S26315[6], 4}, {NULL}};
const Segment SGI340484[] = {{&S26315[57], 5}, {NULL}};
const Segment SGI340485[] = {{&S26315[12], 14}, {&S26315[68], 31}, {NULL}};
const Segment SGI340486[] = {{&S26315[12], 14}, {NULL}};
const Segment SGI340487[] = {{&S26316[6], 4}, {NULL}};
const Segment SGI340488[] = {{&S26316[80], 5}, {NULL}};
const Segment SGI340489[] = {{&S26316[12], 14}, {&S26316[40], 12}, {NULL}};
const Segment SGI340490[] = {{&S26316[12], 14}, {NULL}};
const Segment SGI340491[] = {{&S26317[6], 4}, {NULL}};
const Segment SGI340492[] = {{&S26317[80], 5}, {NULL}};
const Segment SGI340493[] = {{&S26317[12], 14}, {&S26317[40], 12}, {NULL}};
const Segment SGI340494[] = {{&S26317[12], 14}, {NULL}};
const Segment SGI340495[] = {{&S26318[6], 4}, {NULL}};
const Segment SGI340496[] = {{&S26318[80], 5}, {NULL}};
const Segment SGI340497[] = {{&S26318[12], 14}, {&S26318[40], 12}, {NULL}};
const Segment SGI340498[] = {{&S26318[12], 14}, {NULL}};
const Segment SGI340499[] = {{&S26319[6], 4}, {NULL}};
const Segment SGI340500[] = {{&S26319[57], 5}, {NULL}};
const Segment SGI340501[] = {{&S26319[12], 14}, {NULL}};
const Segment SGI340502[] = {{&S26320[6], 4}, {NULL}};
const Segment SGI340503[] = {{&S26320[80], 5}, {NULL}};
const Segment SGI340504[] = {{&S26320[12], 14}, {&S26320[40], 12}, {NULL}};
const Segment SGI340505[] = {{&S26320[12], 14}, {NULL}};
const Segment SGI340506[] = {{&S26321[6], 4}, {NULL}};
const Segment SGI340507[] = {{&S26321[57], 5}, {NULL}};
const Segment SGI340508[] = {{&S26321[12], 14}, {&S26321[68], 31}, {NULL}};
const Segment SGI340509[] = {{&S26321[12], 14}, {NULL}};
const Segment SGI340510[] = {{&S26322[6], 4}, {NULL}};
const Segment SGI340511[] = {{&S26322[80], 5}, {NULL}};
const Segment SGI340512[] = {{&S26322[12], 14}, {&S26322[40], 12}, {NULL}};
const Segment SGI340513[] = {{&S26322[12], 14}, {NULL}};
const Segment SGI340514[] = {{&S26323[6], 4}, {NULL}};
const Segment SGI340515[] = {{&S26323[79], 5}, {NULL}};
const Segment SGI340516[] = {{&S26323[12], 14}, {&S26323[40], 11}, {NULL}};
const Segment SGI340517[] = {{&S26323[12], 14}, {NULL}};
const Segment SGI340518[] = {{&S26324[6], 4}, {NULL}};
const Segment SGI340519[] = {{&S26324[80], 5}, {NULL}};
const Segment SGI340520[] = {{&S26324[12], 14}, {&S26324[40], 12}, {NULL}};
const Segment SGI340521[] = {{&S26324[12], 14}, {NULL}};
const Segment SGI340522[] = {{&S26325[6], 4}, {NULL}};
const Segment SGI340523[] = {{&S26325[80], 5}, {NULL}};
const Segment SGI340524[] = {{&S26325[12], 14}, {&S26325[40], 12}, {NULL}};
const Segment SGI340525[] = {{&S26325[12], 14}, {NULL}};
const Segment SGI340526[] = {{&S26326[6], 4}, {NULL}};
const Segment SGI340527[] = {{&S26326[80], 5}, {NULL}};
const Segment SGI340528[] = {{&S26326[12], 14}, {&S26326[40], 12}, {NULL}};
const Segment SGI340529[] = {{&S26326[12], 14}, {NULL}};
const Segment SGI340530[] = {{&S26327[6], 4}, {NULL}};
const Segment SGI340531[] = {{&S26327[80], 5}, {NULL}};
const Segment SGI340532[] = {{&S26327[12], 14}, {&S26327[40], 12}, {NULL}};
const Segment SGI340533[] = {{&S26327[12], 14}, {NULL}};
const Segment SGI340534[] = {{&S26328[6], 4}, {NULL}};
const Segment SGI340535[] = {{&S26328[57], 5}, {NULL}};
const Segment SGI340536[] = {{&S26328[12], 14}, {NULL}};
const Segment SGI340537[] = {{&S26329[6], 4}, {NULL}};
const Segment SGI340538[] = {{&S26329[80], 5}, {NULL}};
const Segment SGI340539[] = {{&S26329[12], 14}, {&S26329[40], 12}, {NULL}};
const Segment SGI340540[] = {{&S26329[12], 14}, {NULL}};
const Segment SGI340541[] = {{&S26330[6], 4}, {NULL}};
const Segment SGI340542[] = {{&S26330[80], 5}, {NULL}};
const Segment SGI340543[] = {{&S26330[12], 14}, {&S26330[40], 12}, {NULL}};
const Segment SGI340544[] = {{&S26330[12], 14}, {NULL}};
const Segment SGI340545[] = {{&S26331[6], 4}, {NULL}};
const Segment SGI340546[] = {{&S26331[80], 5}, {NULL}};
const Segment SGI340547[] = {{&S26331[12], 14}, {&S26331[40], 12}, {NULL}};
const Segment SGI340548[] = {{&S26331[12], 14}, {NULL}};
const Segment SGI340549[] = {{&S26332[6], 4}, {NULL}};
const Segment SGI340550[] = {{&S26332[80], 5}, {NULL}};
const Segment SGI340551[] = {{&S26332[12], 14}, {&S26332[40], 12}, {NULL}};
const Segment SGI340552[] = {{&S26332[12], 14}, {NULL}};
const Segment SGI340553[] = {{&S26333[6], 4}, {NULL}};
const Segment SGI340554[] = {{&S26333[79], 5}, {NULL}};
const Segment SGI340555[] = {{&S26333[12], 14}, {&S26333[40], 11}, {NULL}};
const Segment SGI340556[] = {{&S26333[12], 14}, {NULL}};
const Segment SGI340557[] = {{&S26334[6], 4}, {NULL}};
const Segment SGI340558[] = {{&S26334[79], 5}, {NULL}};
const Segment SGI340559[] = {{&S26334[12], 14}, {&S26334[40], 11}, {NULL}};
const Segment SGI340560[] = {{&S26334[12], 14}, {NULL}};
const Segment SGI340561[] = {{&S26335[6], 4}, {NULL}};
const Segment SGI340562[] = {{&S26335[57], 5}, {NULL}};
const Segment SGI340563[] = {{&S26335[12], 14}, {NULL}};
const Segment SGI340564[] = {{&S26336[6], 4}, {NULL}};
const Segment SGI340565[] = {{&S26336[79], 5}, {NULL}};
const Segment SGI340566[] = {{&S26336[12], 14}, {&S26336[40], 11}, {NULL}};
const Segment SGI340567[] = {{&S26336[12], 14}, {NULL}};
const Segment SGI340568[] = {{&S26338[6], 4}, {NULL}};
const Segment SGI340569[] = {{&S26338[79], 5}, {NULL}};
const Segment SGI340570[] = {{&S26338[12], 14}, {&S26338[40], 11}, {NULL}};
const Segment SGI340571[] = {{&S26338[12], 14}, {NULL}};
const Segment SGI340572[] = {{&S26339[6], 4}, {NULL}};
const Segment SGI340573[] = {{&S26339[79], 5}, {NULL}};
const Segment SGI340574[] = {{&S26339[12], 14}, {&S26339[40], 11}, {NULL}};
const Segment SGI340575[] = {{&S26339[12], 14}, {NULL}};
const Segment SGI340576[] = {{&S26340[6], 4}, {NULL}};
const Segment SGI340577[] = {{&S26340[79], 5}, {NULL}};
const Segment SGI340578[] = {{&S26340[12], 14}, {&S26340[40], 11}, {NULL}};
const Segment SGI340579[] = {{&S26340[12], 14}, {NULL}};
const Segment SGI340580[] = {{&S26341[6], 4}, {NULL}};
const Segment SGI340581[] = {{&S26341[79], 5}, {NULL}};
const Segment SGI340582[] = {{&S26341[12], 14}, {&S26341[40], 11}, {NULL}};
const Segment SGI340583[] = {{&S26341[12], 14}, {NULL}};
const Segment SGI340584[] = {{&S26342[6], 4}, {NULL}};
const Segment SGI340585[] = {{&S26342[80], 5}, {NULL}};
const Segment SGI340586[] = {{&S26342[12], 14}, {&S26342[40], 12}, {NULL}};
const Segment SGI340587[] = {{&S26342[12], 14}, {NULL}};
const Segment SGI340588[] = {{&S26343[6], 4}, {NULL}};
const Segment SGI340589[] = {{&S26343[79], 5}, {NULL}};
const Segment SGI340590[] = {{&S26343[12], 14}, {&S26343[40], 11}, {NULL}};
const Segment SGI340591[] = {{&S26343[12], 14}, {NULL}};
const Segment SGI340592[] = {{&S26344[6], 4}, {NULL}};
const Segment SGI340593[] = {{&S26344[80], 5}, {NULL}};
const Segment SGI340594[] = {{&S26344[12], 14}, {&S26344[40], 12}, {NULL}};
const Segment SGI340595[] = {{&S26344[12], 14}, {NULL}};
const Segment SGI340596[] = {{&S26345[6], 4}, {NULL}};
const Segment SGI340597[] = {{&S26345[80], 5}, {NULL}};
const Segment SGI340598[] = {{&S26345[12], 14}, {&S26345[40], 12}, {NULL}};
const Segment SGI340599[] = {{&S26345[12], 14}, {NULL}};
const Segment SGI340600[] = {{&S26346[6], 4}, {NULL}};
const Segment SGI340601[] = {{&S26346[80], 5}, {NULL}};
const Segment SGI340602[] = {{&S26346[12], 14}, {&S26346[40], 12}, {NULL}};
const Segment SGI340603[] = {{&S26346[12], 14}, {NULL}};
const Segment SGI340604[] = {{&S26347[6], 4}, {NULL}};
const Segment SGI340605[] = {{&S26347[106], 5}, {NULL}};
const Segment SGI340606[] = {{&S26347[12], 14}, {&S26347[40], 12}, {NULL}};
const Segment SGI340607[] = {{&S26347[12], 14}, {NULL}};
const Segment SGI340608[] = {{&S26348[6], 4}, {NULL}};
const Segment SGI340609[] = {{&S26348[79], 5}, {NULL}};
const Segment SGI340610[] = {{&S26348[12], 14}, {&S26348[40], 11}, {NULL}};
const Segment SGI340611[] = {{&S26348[12], 14}, {NULL}};
const Segment SGI340612[] = {{&S26349[6], 4}, {NULL}};
const Segment SGI340613[] = {{&S26349[79], 5}, {NULL}};
const Segment SGI340614[] = {{&S26349[12], 14}, {&S26349[40], 11}, {NULL}};
const Segment SGI340615[] = {{&S26349[12], 14}, {NULL}};
const Segment SGI340616[] = {{&S26350[6], 4}, {NULL}};
const Segment SGI340617[] = {{&S26350[79], 5}, {NULL}};
const Segment SGI340618[] = {{&S26350[12], 14}, {&S26350[40], 11}, {NULL}};
const Segment SGI340619[] = {{&S26350[12], 14}, {NULL}};
const Segment SGI340620[] = {{&S26351[6], 4}, {NULL}};
const Segment SGI340621[] = {{&S26351[79], 5}, {NULL}};
const Segment SGI340622[] = {{&S26351[12], 14}, {&S26351[40], 11}, {NULL}};
const Segment SGI340623[] = {{&S26351[12], 14}, {NULL}};
const Segment SGI340624[] = {{&S26352[6], 4}, {NULL}};
const Segment SGI340625[] = {{&S26352[79], 5}, {NULL}};
const Segment SGI340626[] = {{&S26352[12], 14}, {&S26352[40], 11}, {NULL}};
const Segment SGI340627[] = {{&S26352[12], 14}, {NULL}};
const Segment SGI340628[] = {{&S26353[6], 4}, {NULL}};
const Segment SGI340629[] = {{&S26353[79], 5}, {NULL}};
const Segment SGI340630[] = {{&S26353[12], 14}, {&S26353[40], 11}, {NULL}};
const Segment SGI340631[] = {{&S26353[12], 14}, {NULL}};
const Segment SGI340632[] = {{&S26354[6], 4}, {NULL}};
const Segment SGI340633[] = {{&S26354[80], 5}, {NULL}};
const Segment SGI340634[] = {{&S26354[12], 14}, {&S26354[41], 11}, {NULL}};
const Segment SGI340635[] = {{&S26354[12], 14}, {NULL}};
const Segment SGI340636[] = {{&S26355[6], 4}, {NULL}};
const Segment SGI340637[] = {{&S26355[80], 5}, {NULL}};
const Segment SGI340638[] = {{&S26355[12], 14}, {&S26355[41], 11}, {NULL}};
const Segment SGI340639[] = {{&S26355[12], 14}, {NULL}};
const Segment SGI340640[] = {{&S26356[6], 4}, {NULL}};
const Segment SGI340641[] = {{&S26356[79], 5}, {NULL}};
const Segment SGI340642[] = {{&S26356[12], 14}, {&S26356[40], 11}, {NULL}};
const Segment SGI340643[] = {{&S26356[12], 14}, {NULL}};
const Segment SGI340644[] = {{&S26357[6], 4}, {NULL}};
const Segment SGI340645[] = {{&S26357[79], 5}, {NULL}};
const Segment SGI340646[] = {{&S26357[12], 14}, {&S26357[40], 11}, {NULL}};
const Segment SGI340647[] = {{&S26357[12], 14}, {NULL}};
const Segment SGI340648[] = {{&S26359[6], 4}, {NULL}};
const Segment SGI340649[] = {{&S26359[79], 5}, {NULL}};
const Segment SGI340650[] = {{&S26359[12], 14}, {&S26359[40], 11}, {NULL}};
const Segment SGI340651[] = {{&S26359[12], 14}, {NULL}};
const Segment SGI340652[] = {{&S26361[6], 4}, {NULL}};
const Segment SGI340653[] = {{&S26361[80], 5}, {NULL}};
const Segment SGI340654[] = {{&S26361[12], 14}, {&S26361[40], 12}, {NULL}};
const Segment SGI340655[] = {{&S26361[12], 14}, {NULL}};
const Segment SGI340656[] = {{&S26362[6], 4}, {NULL}};
const Segment SGI340657[] = {{&S26362[80], 5}, {NULL}};
const Segment SGI340658[] = {{&S26362[12], 14}, {&S26362[40], 12}, {NULL}};
const Segment SGI340659[] = {{&S26362[12], 14}, {NULL}};
const Segment SGI340660[] = {{&S26363[6], 4}, {NULL}};
const Segment SGI340661[] = {{&S26363[79], 5}, {NULL}};
const Segment SGI340662[] = {{&S26363[12], 14}, {&S26363[40], 11}, {NULL}};
const Segment SGI340663[] = {{&S26363[12], 14}, {NULL}};
const Segment SGI340664[] = {{&S26364[6], 4}, {NULL}};
const Segment SGI340665[] = {{&S26364[80], 5}, {NULL}};
const Segment SGI340666[] = {{&S26364[12], 14}, {&S26364[41], 11}, {NULL}};
const Segment SGI340667[] = {{&S26364[12], 14}, {NULL}};
const Segment SGI340668[] = {{&S26365[6], 4}, {NULL}};
const Segment SGI340669[] = {{&S26365[79], 5}, {NULL}};
const Segment SGI340670[] = {{&S26365[12], 14}, {&S26365[40], 11}, {NULL}};
const Segment SGI340671[] = {{&S26365[12], 14}, {NULL}};
const Segment SGI340672[] = {{&S26367[6], 4}, {NULL}};
const Segment SGI340673[] = {{&S26367[79], 5}, {NULL}};
const Segment SGI340674[] = {{&S26367[12], 14}, {&S26367[40], 11}, {NULL}};
const Segment SGI340675[] = {{&S26367[12], 14}, {NULL}};
const Segment SGI340676[] = {{&S26368[6], 4}, {NULL}};
const Segment SGI340677[] = {{&S26368[80], 5}, {NULL}};
const Segment SGI340678[] = {{&S26368[12], 14}, {&S26368[41], 11}, {NULL}};
const Segment SGI340679[] = {{&S26368[12], 14}, {NULL}};
const Segment SGI340680[] = {{&S26369[6], 4}, {NULL}};
const Segment SGI340681[] = {{&S26369[80], 5}, {NULL}};
const Segment SGI340682[] = {{&S26369[12], 14}, {&S26369[41], 11}, {NULL}};
const Segment SGI340683[] = {{&S26369[12], 14}, {NULL}};
const Segment SGI340684[] = {{&S26370[6], 4}, {NULL}};
const Segment SGI340685[] = {{&S26370[80], 5}, {NULL}};
const Segment SGI340686[] = {{&S26370[12], 14}, {&S26370[41], 11}, {NULL}};
const Segment SGI340687[] = {{&S26370[12], 14}, {NULL}};
const Segment SGI340688[] = {{&S26371[6], 4}, {NULL}};
const Segment SGI340689[] = {{&S26371[80], 5}, {NULL}};
const Segment SGI340690[] = {{&S26371[12], 14}, {&S26371[41], 11}, {NULL}};
const Segment SGI340691[] = {{&S26371[12], 14}, {NULL}};
const Segment SGI340692[] = {{&S26372[6], 4}, {NULL}};
const Segment SGI340693[] = {{&S26372[80], 5}, {NULL}};
const Segment SGI340694[] = {{&S26372[12], 14}, {&S26372[40], 12}, {NULL}};
const Segment SGI340695[] = {{&S26372[12], 14}, {NULL}};
const Segment SGI340696[] = {{&S26373[6], 4}, {NULL}};
const Segment SGI340697[] = {{&S26373[80], 5}, {NULL}};
const Segment SGI340698[] = {{&S26373[12], 14}, {&S26373[40], 12}, {NULL}};
const Segment SGI340699[] = {{&S26373[12], 14}, {NULL}};
const Segment SGI340700[] = {{&S26374[6], 4}, {NULL}};
const Segment SGI340701[] = {{&S26374[80], 5}, {NULL}};
const Segment SGI340702[] = {{&S26374[12], 14}, {&S26374[40], 12}, {NULL}};
const Segment SGI340703[] = {{&S26374[12], 14}, {NULL}};
const Segment SGI340704[] = {{&S26375[6], 4}, {NULL}};
const Segment SGI340705[] = {{&S26375[79], 5}, {NULL}};
const Segment SGI340706[] = {{&S26375[12], 14}, {&S26375[40], 11}, {NULL}};
const Segment SGI340707[] = {{&S26375[12], 14}, {NULL}};
const Segment SGI340708[] = {{&S26376[6], 4}, {NULL}};
const Segment SGI340709[] = {{&S26376[80], 5}, {NULL}};
const Segment SGI340710[] = {{&S26376[12], 14}, {&S26376[41], 11}, {NULL}};
const Segment SGI340711[] = {{&S26376[12], 14}, {NULL}};
const Segment SGI340712[] = {{&S26377[6], 4}, {NULL}};
const Segment SGI340713[] = {{&S26377[80], 5}, {NULL}};
const Segment SGI340714[] = {{&S26377[12], 14}, {&S26377[41], 11}, {NULL}};
const Segment SGI340715[] = {{&S26377[12], 14}, {NULL}};
const Segment SGI340716[] = {{&S26378[6], 4}, {NULL}};
const Segment SGI340717[] = {{&S26378[79], 5}, {NULL}};
const Segment SGI340718[] = {{&S26378[12], 14}, {&S26378[40], 11}, {NULL}};
const Segment SGI340719[] = {{&S26378[12], 14}, {NULL}};
const Segment SGI340720[] = {{&S26379[6], 4}, {NULL}};
const Segment SGI340721[] = {{&S26379[80], 5}, {NULL}};
const Segment SGI340722[] = {{&S26379[12], 14}, {&S26379[41], 11}, {NULL}};
const Segment SGI340723[] = {{&S26379[12], 14}, {NULL}};
const Segment SGI340724[] = {{&S26380[6], 4}, {NULL}};
const Segment SGI340725[] = {{&S26380[79], 5}, {NULL}};
const Segment SGI340726[] = {{&S26380[12], 14}, {&S26380[40], 11}, {NULL}};
const Segment SGI340727[] = {{&S26380[12], 14}, {NULL}};
const Segment SGI340728[] = {{&S26381[6], 4}, {NULL}};
const Segment SGI340729[] = {{&S26381[79], 5}, {NULL}};
const Segment SGI340730[] = {{&S26381[12], 14}, {&S26381[40], 11}, {NULL}};
const Segment SGI340731[] = {{&S26381[12], 14}, {NULL}};
const Segment SGI340732[] = {{&S26382[6], 4}, {NULL}};
const Segment SGI340733[] = {{&S26382[79], 5}, {NULL}};
const Segment SGI340734[] = {{&S26382[12], 14}, {&S26382[40], 11}, {NULL}};
const Segment SGI340735[] = {{&S26382[12], 14}, {NULL}};
const Segment SGI340736[] = {{&S26383[6], 4}, {NULL}};
const Segment SGI340737[] = {{&S26383[80], 5}, {NULL}};
const Segment SGI340738[] = {{&S26383[12], 14}, {&S26383[41], 11}, {NULL}};
const Segment SGI340739[] = {{&S26383[12], 14}, {NULL}};
const Segment SGI340740[] = {{&S26384[6], 4}, {NULL}};
const Segment SGI340741[] = {{&S26384[79], 5}, {NULL}};
const Segment SGI340742[] = {{&S26384[12], 14}, {&S26384[40], 11}, {NULL}};
const Segment SGI340743[] = {{&S26384[12], 14}, {NULL}};
const Segment SGI340744[] = {{&S26385[6], 4}, {NULL}};
const Segment SGI340745[] = {{&S26385[79], 5}, {NULL}};
const Segment SGI340746[] = {{&S26385[12], 14}, {&S26385[40], 11}, {NULL}};
const Segment SGI340747[] = {{&S26385[12], 14}, {NULL}};
const Segment SGI340748[] = {{&S26386[6], 4}, {NULL}};
const Segment SGI340749[] = {{&S26386[79], 5}, {NULL}};
const Segment SGI340750[] = {{&S26386[12], 14}, {&S26386[40], 11}, {NULL}};
const Segment SGI340751[] = {{&S26386[12], 14}, {NULL}};
const Segment SGI340752[] = {{&S26387[6], 4}, {NULL}};
const Segment SGI340753[] = {{&S26387[79], 5}, {NULL}};
const Segment SGI340754[] = {{&S26387[12], 14}, {&S26387[40], 11}, {NULL}};
const Segment SGI340755[] = {{&S26387[12], 14}, {NULL}};
const Segment SGI340756[] = {{&S26388[6], 4}, {NULL}};
const Segment SGI340757[] = {{&S26388[80], 5}, {NULL}};
const Segment SGI340758[] = {{&S26388[12], 14}, {&S26388[41], 11}, {NULL}};
const Segment SGI340759[] = {{&S26388[12], 14}, {NULL}};
const Segment SGI340760[] = {{&S52733[100], 7}, {NULL}};
const Segment SGI340761[] = {{&S26389[6], 4}, {NULL}};
const Segment SGI340762[] = {{&S26389[79], 5}, {NULL}};
const Segment SGI340763[] = {{&S26389[12], 14}, {&S26389[40], 11}, {NULL}};
const Segment SGI340764[] = {{&S26389[12], 14}, {NULL}};
const Segment SGI340765[] = {{&S26390[6], 4}, {NULL}};
const Segment SGI340766[] = {{&S52735[100], 7}, {NULL}};
const Segment SGI340767[] = {{&S26390[79], 5}, {NULL}};
const Segment SGI340768[] = {{&S26390[12], 14}, {&S26390[40], 11}, {NULL}};
const Segment SGI340769[] = {{&S26390[12], 14}, {NULL}};
const Segment SGI340770[] = {{&S26391[6], 4}, {NULL}};
const Segment SGI340771[] = {{&S26391[79], 5}, {NULL}};
const Segment SGI340772[] = {{&S26391[12], 14}, {&S26391[40], 11}, {NULL}};
const Segment SGI340773[] = {{&S52736[100], 22}, {NULL}};
const Segment SGI340774[] = {{&S26391[12], 14}, {NULL}};
const Segment SGI340775[] = {{&S26392[6], 4}, {NULL}};
const Segment SGI340776[] = {{&S26392[79], 5}, {NULL}};
const Segment SGI340777[] = {{&S26392[12], 14}, {&S26392[40], 11}, {NULL}};
const Segment SGI340778[] = {{&S26392[12], 14}, {NULL}};
const Segment SGI340779[] = {{&S26393[6], 4}, {NULL}};
const Segment SGI340780[] = {{&S26393[80], 5}, {NULL}};
const Segment SGI340781[] = {{&S26393[12], 14}, {&S26393[40], 12}, {NULL}};
const Segment SGI340782[] = {{&S26393[12], 14}, {NULL}};
const Segment SGI340783[] = {{&S52739[100], 11}, {NULL}};
const Segment SGI340784[] = {{&S26394[6], 4}, {NULL}};
const Segment SGI340785[] = {{&S26394[80], 5}, {NULL}};
const Segment SGI340786[] = {{&S26394[12], 14}, {&S26394[41], 11}, {NULL}};
const Segment SGI340787[] = {{&S26394[12], 14}, {NULL}};
const Segment SGI340788[] = {{&S26395[6], 4}, {NULL}};
const Segment SGI340789[] = {{&S26395[80], 5}, {NULL}};
const Segment SGI340790[] = {{&S26395[12], 14}, {&S26395[41], 11}, {NULL}};
const Segment SGI340791[] = {{&S26395[12], 14}, {NULL}};
const Segment SGI340792[] = {{&S52741[100], 7}, {NULL}};
const Segment SGI340793[] = {{&S26396[6], 4}, {NULL}};
const Segment SGI340794[] = {{&S26396[79], 5}, {NULL}};
const Segment SGI340795[] = {{&S26396[12], 14}, {&S26396[40], 11}, {NULL}};
const Segment SGI340796[] = {{&S26396[12], 14}, {NULL}};
const Segment SGI340797[] = {{&S26397[6], 4}, {NULL}};
const Segment SGI340798[] = {{&S26397[57], 5}, {NULL}};
const Segment SGI340799[] = {{&S26397[12], 14}, {&S26397[68], 4}, {NULL}};
const Segment SGI340800[] = {{&S26397[12], 14}, {NULL}};
const Segment SGI340801[] = {{&S52743[100], 12}, {NULL}};
const Segment SGI340802[] = {{&S26398[6], 4}, {NULL}};
const Segment SGI340803[] = {{&S26398[79], 5}, {NULL}};
const Segment SGI340804[] = {{&S26398[12], 14}, {&S26398[40], 11}, {NULL}};
const Segment SGI340805[] = {{&S26398[12], 14}, {NULL}};
const Segment SGI340806[] = {{&S52744[100], 13}, {NULL}};
const Segment SGI340807[] = {{&S26399[6], 4}, {NULL}};
const Segment SGI340808[] = {{&S26399[79], 5}, {NULL}};
const Segment SGI340809[] = {{&S26399[12], 14}, {&S26399[40], 11}, {NULL}};
const Segment SGI340810[] = {{&S26399[12], 14}, {NULL}};
const Segment SGI340811[] = {{&S52745[100], 7}, {NULL}};
const Segment SGI340812[] = {{&S26400[6], 4}, {NULL}};
const Segment SGI340813[] = {{&S26400[80], 5}, {NULL}};
const Segment SGI340814[] = {{&S26400[12], 14}, {&S26400[41], 11}, {NULL}};
const Segment SGI340815[] = {{&S26400[12], 14}, {NULL}};
const Segment SGI340816[] = {{&S52746[100], 12}, {NULL}};
const Segment SGI340817[] = {{&S26401[6], 4}, {NULL}};
const Segment SGI340818[] = {{&S26401[79], 5}, {NULL}};
const Segment SGI340819[] = {{&S26401[12], 14}, {&S26401[40], 11}, {NULL}};
const Segment SGI340820[] = {{&S26401[12], 14}, {NULL}};
const Segment SGI340821[] = {{&S52747[100], 15}, {NULL}};
const Segment SGI340822[] = {{&S26402[6], 4}, {NULL}};
const Segment SGI340823[] = {{&S26402[79], 5}, {NULL}};
const Segment SGI340824[] = {{&S26402[12], 14}, {&S26402[40], 11}, {NULL}};
const Segment SGI340825[] = {{&S26402[12], 14}, {NULL}};
const Segment SGI340826[] = {{&S52748[100], 12}, {NULL}};
const Segment SGI340827[] = {{&S26403[6], 4}, {NULL}};
const Segment SGI340828[] = {{&S26403[57], 5}, {NULL}};
const Segment SGI340829[] = {{&S26403[12], 14}, {&S26403[68], 4}, {NULL}};
const Segment SGI340830[] = {{&S26403[12], 14}, {NULL}};
const Segment SGI340831[] = {{&S52749[100], 11}, {NULL}};
const Segment SGI340832[] = {{&S26404[6], 4}, {NULL}};
const Segment SGI340833[] = {{&S26404[80], 5}, {NULL}};
const Segment SGI340834[] = {{&S26404[12], 14}, {&S26404[41], 11}, {NULL}};
const Segment SGI340835[] = {{&S26404[12], 14}, {NULL}};
const Segment SGI340836[] = {{&S26405[6], 4}, {NULL}};
const Segment SGI340837[] = {{&S26405[79], 5}, {NULL}};
const Segment SGI340838[] = {{&S26405[12], 14}, {&S26405[40], 11}, {NULL}};
const Segment SGI340839[] = {{&S52750[100], 13}, {NULL}};
const Segment SGI340840[] = {{&S26405[12], 14}, {NULL}};
const Segment SGI340841[] = {{&S26406[6], 4}, {NULL}};
const Segment SGI340842[] = {{&S26406[79], 5}, {NULL}};
const Segment SGI340843[] = {{&S26406[12], 14}, {&S26406[40], 11}, {NULL}};
const Segment SGI340844[] = {{&S26406[12], 14}, {NULL}};
const Segment SGI340845[] = {{&S26407[6], 4}, {NULL}};
const Segment SGI340846[] = {{&S26407[79], 5}, {NULL}};
const Segment SGI340847[] = {{&S26407[12], 14}, {&S26407[40], 11}, {NULL}};
const Segment SGI340848[] = {{&S26407[12], 14}, {NULL}};
const Segment SGI340849[] = {{&S26408[6], 4}, {NULL}};
const Segment SGI340850[] = {{&S26408[80], 5}, {NULL}};
const Segment SGI340851[] = {{&S26408[12], 14}, {&S26408[41], 11}, {NULL}};
const Segment SGI340852[] = {{&S26408[12], 14}, {NULL}};
const Segment SGI340853[] = {{&S26409[6], 4}, {NULL}};
const Segment SGI340854[] = {{&S26409[79], 5}, {NULL}};
const Segment SGI340855[] = {{&S26409[12], 14}, {&S26409[40], 11}, {NULL}};
const Segment SGI340856[] = {{&S26409[12], 14}, {NULL}};
const Segment SGI340857[] = {{&S26410[6], 4}, {NULL}};
const Segment SGI340858[] = {{&S26410[79], 5}, {NULL}};
const Segment SGI340859[] = {{&S26410[12], 14}, {&S26410[40], 11}, {NULL}};
const Segment SGI340860[] = {{&S26410[12], 14}, {NULL}};
const Segment SGI340861[] = {{&S26411[6], 4}, {NULL}};
const Segment SGI340862[] = {{&S26411[79], 5}, {NULL}};
const Segment SGI340863[] = {{&S26411[12], 14}, {&S26411[40], 11}, {NULL}};
const Segment SGI340864[] = {{&S26411[12], 14}, {NULL}};
const Segment SGI340865[] = {{&S26412[6], 4}, {NULL}};
const Segment SGI340866[] = {{&S26412[79], 5}, {NULL}};
const Segment SGI340867[] = {{&S26412[12], 14}, {&S26412[40], 11}, {NULL}};
const Segment SGI340868[] = {{&S26412[12], 14}, {NULL}};
const Segment SGI340869[] = {{&S26413[6], 4}, {NULL}};
const Segment SGI340870[] = {{&S26413[79], 5}, {NULL}};
const Segment SGI340871[] = {{&S26413[12], 14}, {&S26413[40], 11}, {NULL}};
const Segment SGI340872[] = {{&S26413[12], 14}, {NULL}};
const Segment SGI340873[] = {{&S26414[6], 4}, {NULL}};
const Segment SGI340874[] = {{&S26414[79], 5}, {NULL}};
const Segment SGI340875[] = {{&S26414[12], 14}, {&S26414[40], 11}, {NULL}};
const Segment SGI340876[] = {{&S26414[12], 14}, {NULL}};
const Segment SGI340877[] = {{&S26415[6], 4}, {NULL}};
const Segment SGI340878[] = {{&S26415[79], 5}, {NULL}};
const Segment SGI340879[] = {{&S26415[12], 14}, {&S26415[40], 11}, {NULL}};
const Segment SGI340880[] = {{&S26415[12], 14}, {NULL}};
const Segment SGI340881[] = {{&S26416[6], 4}, {NULL}};
const Segment SGI340882[] = {{&S26416[79], 5}, {NULL}};
const Segment SGI340883[] = {{&S26416[12], 14}, {&S26416[40], 11}, {NULL}};
const Segment SGI340884[] = {{&S26416[12], 14}, {NULL}};
const Segment SGI340885[] = {{&S52839[42], 3}, {NULL}};
const Segment SGI340886[] = {{&S52839[6], 3}, {NULL}};
const Segment SGI340887[] = {{&S52839[66], 3}, {NULL}};
const Segment SGI340888[] = {{&S52840[5], 3}, {NULL}};
const Segment SGI340889[] = {{&S52840[14], 3}, {NULL}};
const Segment SGI340890[] = {{&S52841[41], 5}, {NULL}};
const Segment SGI340891[] = {{&S52841[6], 3}, {NULL}};
const Segment SGI340892[] = {{&S52841[67], 3}, {NULL}};
const Segment SGI340893[] = {{&S44090[28], 3}, {NULL}};
const Segment SGI340894[] = {{&S44090[102], 5}, {NULL}};
const Segment SGI340895[] = {{&S26664[6], 4}, {NULL}};
const Segment SGI340896[] = {{&S26664[64], 5}, {NULL}};
const Segment SGI340897[] = {{&S26664[12], 14}, {NULL}};
const Segment SGI340898[] = {{&S26665[6], 4}, {NULL}};
const Segment SGI340899[] = {{&S26665[52], 5}, {NULL}};
const Segment SGI340900[] = {{&S53014[5], 7}, {&S53014[5], 7}, {NULL}};
const Segment SGI340901[] = {{&S26665[12], 45}, {NULL}};
const Segment SGI340902[] = {{&S26665[12], 14}, {NULL}};
const Segment SGI340903[] = {{&S53014[157], 3}, {NULL}};
const Segment SGI340904[] = {{&S26666[6], 4}, {NULL}};
const Segment SGI340905[] = {{&S26666[81], 5}, {NULL}};
const Segment SGI340906[] = {{&S26666[12], 14}, {&S26666[44], 42}, {NULL}};
const Segment SGI340907[] = {{&S26666[12], 14}, {NULL}};
const Segment SGI340908[] = {{&S26667[6], 4}, {NULL}};
const Segment SGI340909[] = {{&S26667[59], 5}, {NULL}};
const Segment SGI340910[] = {{&S26667[12], 14}, {&S26667[70], 4}, {NULL}};
const Segment SGI340911[] = {{&S26667[12], 14}, {NULL}};
const Segment SGI340912[] = {{&S26668[6], 4}, {NULL}};
const Segment SGI340913[] = {{&S26668[81], 5}, {NULL}};
const Segment SGI340914[] = {{&S26668[12], 14}, {&S26668[44], 42}, {NULL}};
const Segment SGI340915[] = {{&S26668[12], 14}, {NULL}};
const Segment SGI340916[] = {{&S26669[6], 4}, {NULL}};
const Segment SGI340917[] = {{&S26669[81], 5}, {NULL}};
const Segment SGI340918[] = {{&S26669[12], 14}, {&S26669[44], 42}, {NULL}};
const Segment SGI340919[] = {{&S26669[12], 14}, {NULL}};
const Segment SGI340920[] = {{&S26670[6], 4}, {NULL}};
const Segment SGI340921[] = {{&S26670[81], 5}, {NULL}};
const Segment SGI340922[] = {{&S26670[12], 14}, {&S26670[44], 42}, {NULL}};
const Segment SGI340923[] = {{&S26670[12], 14}, {NULL}};
const Segment SGI340924[] = {{&S26671[6], 4}, {NULL}};
const Segment SGI340925[] = {{&S26671[81], 5}, {NULL}};
const Segment SGI340926[] = {{&S26671[12], 14}, {&S26671[44], 42}, {NULL}};
const Segment SGI340927[] = {{&S26671[12], 14}, {NULL}};
const Segment SGI340928[] = {{&S26672[6], 4}, {NULL}};
const Segment SGI340929[] = {{&S26672[81], 5}, {NULL}};
const Segment SGI340930[] = {{&S26672[12], 14}, {&S26672[44], 42}, {NULL}};
const Segment SGI340931[] = {{&S26672[12], 14}, {NULL}};
const Segment SGI340932[] = {{&S26674[6], 4}, {NULL}};
const Segment SGI340933[] = {{&S26674[81], 5}, {NULL}};
const Segment SGI340934[] = {{&S26674[12], 14}, {&S26674[44], 42}, {NULL}};
const Segment SGI340935[] = {{&S26674[12], 14}, {NULL}};
const Segment SGI340936[] = {{&S26675[6], 4}, {NULL}};
const Segment SGI340937[] = {{&S26675[81], 5}, {NULL}};
const Segment SGI340938[] = {{&S26675[12], 14}, {&S26675[44], 42}, {NULL}};
const Segment SGI340939[] = {{&S26675[12], 14}, {NULL}};
const Segment SGI340940[] = {{&S26676[6], 4}, {NULL}};
const Segment SGI340941[] = {{&S26676[81], 5}, {NULL}};
const Segment SGI340942[] = {{&S26676[12], 14}, {&S26676[44], 42}, {NULL}};
const Segment SGI340943[] = {{&S26676[12], 14}, {NULL}};
const Segment SGI340944[] = {{&S26677[6], 4}, {NULL}};
const Segment SGI340945[] = {{&S26677[81], 5}, {NULL}};
const Segment SGI340946[] = {{&S26677[12], 14}, {&S26677[44], 42}, {NULL}};
const Segment SGI340947[] = {{&S26677[12], 14}, {NULL}};
const Segment SGI340948[] = {{&S26678[6], 4}, {NULL}};
const Segment SGI340949[] = {{&S26678[81], 5}, {NULL}};
const Segment SGI340950[] = {{&S26678[12], 14}, {&S26678[44], 42}, {NULL}};
const Segment SGI340951[] = {{&S26678[12], 14}, {NULL}};
const Segment SGI340952[] = {{&S26679[6], 4}, {NULL}};
const Segment SGI340953[] = {{&S26679[81], 5}, {NULL}};
const Segment SGI340954[] = {{&S26679[12], 14}, {&S26679[44], 42}, {NULL}};
const Segment SGI340955[] = {{&S26679[12], 14}, {NULL}};
const Segment SGI340956[] = {{&S26680[6], 4}, {NULL}};
const Segment SGI340957[] = {{&S26680[81], 5}, {NULL}};
const Segment SGI340958[] = {{&S26680[12], 14}, {&S26680[44], 42}, {NULL}};
const Segment SGI340959[] = {{&S26680[12], 14}, {NULL}};
const Segment SGI340960[] = {{&S26681[6], 4}, {NULL}};
const Segment SGI340961[] = {{&S26681[81], 5}, {NULL}};
const Segment SGI340962[] = {{&S26681[12], 14}, {&S26681[44], 42}, {NULL}};
const Segment SGI340963[] = {{&S26681[12], 14}, {NULL}};
const Segment SGI340964[] = {{&S26682[6], 4}, {NULL}};
const Segment SGI340965[] = {{&S26682[81], 5}, {NULL}};
const Segment SGI340966[] = {{&S26682[12], 14}, {&S26682[44], 42}, {NULL}};
const Segment SGI340967[] = {{&S26682[12], 14}, {NULL}};
const Segment SGI340968[] = {{&S26683[6], 4}, {NULL}};
const Segment SGI340969[] = {{&S26683[52], 5}, {NULL}};
const Segment SGI340970[] = {{&S26683[12], 55}, {NULL}};
const Segment SGI340971[] = {{&S26683[12], 14}, {NULL}};
const Segment SGI340972[] = {{&S26684[6], 4}, {NULL}};
const Segment SGI340973[] = {{&S26684[81], 5}, {NULL}};
const Segment SGI340974[] = {{&S26684[12], 14}, {&S26684[44], 42}, {NULL}};
const Segment SGI340975[] = {{&S26684[12], 14}, {NULL}};
const Segment SGI340976[] = {{&S26686[6], 4}, {NULL}};
const Segment SGI340977[] = {{&S26686[81], 5}, {NULL}};
const Segment SGI340978[] = {{&S26686[12], 14}, {&S26686[44], 42}, {NULL}};
const Segment SGI340979[] = {{&S26686[12], 14}, {NULL}};
const Segment SGI340980[] = {{&S26687[6], 4}, {NULL}};
const Segment SGI340981[] = {{&S26687[81], 5}, {NULL}};
const Segment SGI340982[] = {{&S26687[12], 14}, {&S26687[44], 42}, {NULL}};
const Segment SGI340983[] = {{&S26687[12], 14}, {NULL}};
const Segment SGI340984[] = {{&S53046[38], 3}, {NULL}};
const Segment SGI340985[] = {{&S53046[92], 3}, {NULL}};
const Segment SGI340986[] = {{&S26688[6], 4}, {NULL}};
const Segment SGI340987[] = {{&S26688[81], 5}, {NULL}};
const Segment SGI340988[] = {{&S26688[12], 14}, {&S26688[44], 42}, {NULL}};
const Segment SGI340989[] = {{&S26688[12], 14}, {NULL}};
const Segment SGI340990[] = {{&S53047[5], 12}, {NULL}};
const Segment SGI340991[] = {{&S53047[18], 3}, {NULL}};
const Segment SGI340992[] = {{&S26689[6], 4}, {NULL}};
const Segment SGI340993[] = {{&S26689[81], 5}, {NULL}};
const Segment SGI340994[] = {{&S26689[12], 14}, {&S26689[44], 42}, {NULL}};
const Segment SGI340995[] = {{&S26689[12], 14}, {NULL}};
const Segment SGI340996[] = {{&S53048[41], 3}, {NULL}};
const Segment SGI340997[] = {{&S53048[6], 3}, {NULL}};
const Segment SGI340998[] = {{&S53048[65], 3}, {NULL}};
const Segment SGI340999[] = {{&S26690[6], 4}, {NULL}};
const Segment SGI341000[] = {{&S26690[81], 5}, {NULL}};
const Segment SGI341001[] = {{&S26690[12], 14}, {&S26690[44], 42}, {NULL}};
const Segment SGI341002[] = {{&S26690[12], 14}, {NULL}};
const Segment SGI341003[] = {{&S26691[6], 4}, {NULL}};
const Segment SGI341004[] = {{&S26691[81], 5}, {NULL}};
const Segment SGI341005[] = {{&S26691[12], 14}, {&S26691[44], 42}, {NULL}};
const Segment SGI341006[] = {{&S26691[12], 14}, {NULL}};
const Segment SGI341007[] = {{&S26692[6], 4}, {NULL}};
const Segment SGI341008[] = {{&S26692[81], 5}, {NULL}};
const Segment SGI341009[] = {{&S26692[12], 14}, {&S26692[44], 42}, {NULL}};
const Segment SGI341010[] = {{&S26692[12], 14}, {NULL}};
const Segment SGI341011[] = {{&S26693[6], 4}, {NULL}};
const Segment SGI341012[] = {{&S26693[81], 5}, {NULL}};
const Segment SGI341013[] = {{&S26693[12], 14}, {&S26693[44], 42}, {NULL}};
const Segment SGI341014[] = {{&S26693[12], 14}, {NULL}};
const Segment SGI341015[] = {{&S26694[6], 4}, {NULL}};
const Segment SGI341016[] = {{&S26694[81], 5}, {NULL}};
const Segment SGI341017[] = {{&S26694[12], 14}, {&S26694[44], 42}, {NULL}};
const Segment SGI341018[] = {{&S26694[12], 14}, {NULL}};
const Segment SGI341019[] = {{&S26695[6], 4}, {NULL}};
const Segment SGI341020[] = {{&S26695[81], 5}, {NULL}};
const Segment SGI341021[] = {{&S26695[12], 14}, {&S26695[44], 42}, {NULL}};
const Segment SGI341022[] = {{&S26695[12], 14}, {NULL}};
const Segment SGI341023[] = {{&S26696[6], 4}, {NULL}};
const Segment SGI341024[] = {{&S26696[81], 5}, {NULL}};
const Segment SGI341025[] = {{&S26696[12], 14}, {&S26696[44], 42}, {NULL}};
const Segment SGI341026[] = {{&S26696[12], 14}, {NULL}};
const Segment SGI341027[] = {{&S26697[6], 4}, {NULL}};
const Segment SGI341028[] = {{&S26697[81], 5}, {NULL}};
const Segment SGI341029[] = {{&S26697[12], 14}, {&S26697[44], 42}, {NULL}};
const Segment SGI341030[] = {{&S26697[12], 14}, {NULL}};
const Segment SGI341031[] = {{&S26698[6], 4}, {NULL}};
const Segment SGI341032[] = {{&S26698[82], 5}, {NULL}};
const Segment SGI341033[] = {{&S26698[12], 14}, {&S26698[45], 42}, {NULL}};
const Segment SGI341034[] = {{&S26698[12], 14}, {NULL}};
const Segment SGI341035[] = {{&S26699[6], 4}, {NULL}};
const Segment SGI341036[] = {{&S26699[64], 5}, {NULL}};
const Segment SGI341037[] = {{&S26699[12], 14}, {&S26699[89], 33}, {NULL}};
const Segment SGI341038[] = {{&S26699[12], 14}, {NULL}};
const Segment SGI341039[] = {{&S26700[6], 4}, {NULL}};
const Segment SGI341040[] = {{&S26700[81], 5}, {NULL}};
const Segment SGI341041[] = {{&S26700[12], 14}, {&S26700[44], 42}, {NULL}};
const Segment SGI341042[] = {{&S26700[12], 14}, {NULL}};
const Segment SGI341043[] = {{&S26702[6], 4}, {NULL}};
const Segment SGI341044[] = {{&S26702[81], 5}, {NULL}};
const Segment SGI341045[] = {{&S26702[12], 14}, {&S26702[44], 42}, {NULL}};
const Segment SGI341046[] = {{&S26702[12], 14}, {NULL}};
const Segment SGI341047[] = {{&S26703[6], 4}, {NULL}};
const Segment SGI341048[] = {{&S26703[81], 5}, {NULL}};
const Segment SGI341049[] = {{&S26703[12], 14}, {&S26703[44], 42}, {NULL}};
const Segment SGI341050[] = {{&S26703[12], 14}, {NULL}};
const Segment SGI341051[] = {{&S26704[6], 4}, {NULL}};
const Segment SGI341052[] = {{&S26704[81], 5}, {NULL}};
const Segment SGI341053[] = {{&S26704[12], 14}, {&S26704[44], 42}, {NULL}};
const Segment SGI341054[] = {{&S26704[12], 14}, {NULL}};
const Segment SGI341055[] = {{&S26705[6], 4}, {NULL}};
const Segment SGI341056[] = {{&S26705[81], 5}, {NULL}};
const Segment SGI341057[] = {{&S26705[12], 14}, {&S26705[44], 42}, {NULL}};
const Segment SGI341058[] = {{&S26705[12], 14}, {NULL}};
const Segment SGI341059[] = {{&S26706[6], 4}, {NULL}};
const Segment SGI341060[] = {{&S26706[81], 5}, {NULL}};
const Segment SGI341061[] = {{&S26706[12], 14}, {&S26706[44], 42}, {NULL}};
const Segment SGI341062[] = {{&S26706[12], 14}, {NULL}};
const Segment SGI341063[] = {{&S26707[6], 4}, {NULL}};
const Segment SGI341064[] = {{&S26707[81], 5}, {NULL}};
const Segment SGI341065[] = {{&S26707[12], 14}, {&S26707[44], 42}, {NULL}};
const Segment SGI341066[] = {{&S26707[12], 14}, {NULL}};
const Segment SGI341067[] = {{&S26708[6], 4}, {NULL}};
const Segment SGI341068[] = {{&S26708[81], 5}, {NULL}};
const Segment SGI341069[] = {{&S26708[12], 14}, {&S26708[44], 42}, {NULL}};
const Segment SGI341070[] = {{&S26708[12], 14}, {NULL}};
const Segment SGI341071[] = {{&S26709[6], 4}, {NULL}};
const Segment SGI341072[] = {{&S26709[81], 5}, {NULL}};
const Segment SGI341073[] = {{&S26709[12], 14}, {&S26709[44], 42}, {NULL}};
const Segment SGI341074[] = {{&S26709[12], 14}, {NULL}};
const Segment SGI341075[] = {{&S26710[6], 4}, {NULL}};
const Segment SGI341076[] = {{&S26710[81], 5}, {NULL}};
const Segment SGI341077[] = {{&S26710[12], 14}, {&S26710[44], 42}, {NULL}};
const Segment SGI341078[] = {{&S26710[12], 14}, {NULL}};
const Segment SGI341079[] = {{&S26711[6], 4}, {NULL}};
const Segment SGI341080[] = {{&S26711[81], 5}, {NULL}};
const Segment SGI341081[] = {{&S26711[12], 14}, {&S26711[44], 42}, {NULL}};
const Segment SGI341082[] = {{&S26711[12], 14}, {NULL}};
const Segment SGI341083[] = {{&S26712[6], 4}, {NULL}};
const Segment SGI341084[] = {{&S26712[81], 5}, {NULL}};
const Segment SGI341085[] = {{&S26712[12], 14}, {&S26712[44], 42}, {NULL}};
const Segment SGI341086[] = {{&S26712[12], 14}, {NULL}};
const Segment SGI341087[] = {{&S26713[6], 4}, {NULL}};
const Segment SGI341088[] = {{&S26713[82], 5}, {NULL}};
const Segment SGI341089[] = {{&S26713[12], 14}, {&S26713[45], 42}, {NULL}};
const Segment SGI341090[] = {{&S26713[12], 14}, {NULL}};
const Segment SGI341091[] = {{&S26714[6], 4}, {NULL}};
const Segment SGI341092[] = {{&S26714[81], 5}, {NULL}};
const Segment SGI341093[] = {{&S26714[12], 14}, {&S26714[44], 42}, {NULL}};
const Segment SGI341094[] = {{&S26714[12], 14}, {NULL}};
const Segment SGI341095[] = {{&S26715[6], 4}, {NULL}};
const Segment SGI341096[] = {{&S26715[81], 5}, {NULL}};
const Segment SGI341097[] = {{&S26715[12], 14}, {&S26715[44], 42}, {NULL}};
const Segment SGI341098[] = {{&S26715[12], 14}, {NULL}};
const Segment SGI341099[] = {{&S26716[6], 4}, {NULL}};
const Segment SGI341100[] = {{&S26716[81], 5}, {NULL}};
const Segment SGI341101[] = {{&S26716[12], 14}, {&S26716[44], 42}, {NULL}};
const Segment SGI341102[] = {{&S26716[12], 14}, {NULL}};
const Segment SGI341103[] = {{&S26717[6], 4}, {NULL}};
const Segment SGI341104[] = {{&S26717[81], 5}, {NULL}};
const Segment SGI341105[] = {{&S26717[12], 14}, {&S26717[44], 42}, {NULL}};
const Segment SGI341106[] = {{&S26717[12], 14}, {NULL}};
const Segment SGI341107[] = {{&S26718[6], 4}, {NULL}};
const Segment SGI341108[] = {{&S26718[81], 5}, {NULL}};
const Segment SGI341109[] = {{&S26718[12], 14}, {&S26718[44], 42}, {NULL}};
const Segment SGI341110[] = {{&S26718[12], 14}, {NULL}};
const Segment SGI341111[] = {{&S26719[6], 4}, {NULL}};
const Segment SGI341112[] = {{&S26719[81], 5}, {NULL}};
const Segment SGI341113[] = {{&S26719[12], 14}, {&S26719[44], 42}, {NULL}};
const Segment SGI341114[] = {{&S26719[12], 14}, {NULL}};
const Segment SGI341115[] = {{&S26720[6], 4}, {NULL}};
const Segment SGI341116[] = {{&S26720[81], 5}, {NULL}};
const Segment SGI341117[] = {{&S26720[12], 14}, {&S26720[44], 42}, {NULL}};
const Segment SGI341118[] = {{&S26720[12], 14}, {NULL}};
const Segment SGI341119[] = {{&S26721[6], 4}, {NULL}};
const Segment SGI341120[] = {{&S26721[81], 5}, {NULL}};
const Segment SGI341121[] = {{&S26721[12], 14}, {&S26721[44], 42}, {NULL}};
const Segment SGI341122[] = {{&S26721[12], 14}, {NULL}};
const Segment SGI341123[] = {{&S26722[6], 4}, {NULL}};
const Segment SGI341124[] = {{&S26722[81], 5}, {NULL}};
const Segment SGI341125[] = {{&S26722[12], 14}, {&S26722[44], 42}, {NULL}};
const Segment SGI341126[] = {{&S26722[12], 14}, {NULL}};
const Segment SGI341127[] = {{&S26723[6], 4}, {NULL}};
const Segment SGI341128[] = {{&S26723[81], 5}, {NULL}};
const Segment SGI341129[] = {{&S26723[12], 14}, {&S26723[44], 42}, {NULL}};
const Segment SGI341130[] = {{&S26723[12], 14}, {NULL}};
const Segment SGI341131[] = {{&S26724[6], 4}, {NULL}};
const Segment SGI341132[] = {{&S26724[81], 5}, {NULL}};
const Segment SGI341133[] = {{&S26724[12], 14}, {&S26724[44], 42}, {NULL}};
const Segment SGI341134[] = {{&S26724[12], 14}, {NULL}};
const Segment SGI341135[] = {{&S26725[6], 4}, {NULL}};
const Segment SGI341136[] = {{&S26725[81], 5}, {NULL}};
const Segment SGI341137[] = {{&S26725[12], 14}, {&S26725[44], 42}, {NULL}};
const Segment SGI341138[] = {{&S26725[12], 14}, {NULL}};
const Segment SGI341139[] = {{&S26726[6], 4}, {NULL}};
const Segment SGI341140[] = {{&S26726[81], 5}, {NULL}};
const Segment SGI341141[] = {{&S26726[12], 14}, {&S26726[44], 42}, {NULL}};
const Segment SGI341142[] = {{&S26726[12], 14}, {NULL}};
const Segment SGI341143[] = {{&S26727[6], 4}, {NULL}};
const Segment SGI341144[] = {{&S26727[81], 5}, {NULL}};
const Segment SGI341145[] = {{&S26727[12], 14}, {&S26727[44], 42}, {NULL}};
const Segment SGI341146[] = {{&S26727[12], 14}, {NULL}};
const Segment SGI341147[] = {{&S26728[6], 4}, {NULL}};
const Segment SGI341148[] = {{&S26728[82], 5}, {NULL}};
const Segment SGI341149[] = {{&S26728[12], 14}, {&S26728[44], 43}, {NULL}};
const Segment SGI341150[] = {{&S26728[12], 14}, {NULL}};
const Segment SGI341151[] = {{&S53148[5], 5}, {&S53148[97], 5}, {NULL}};
const Segment SGI341152[] = {{&S53148[153], 3}, {NULL}};
const Segment SGI341153[] = {{&S53152[5], 5}, {&S53152[104], 5}, {NULL}};
const Segment SGI341154[] = {{&S53152[69], 3}, {NULL}};
const Segment SGI341155[] = {{&S53152[130], 3}, {NULL}};
const Segment SGI341156[] = {{&S53157[5], 5}, {&S53157[97], 5}, {NULL}};
const Segment SGI341157[] = {{&S53157[17], 3}, {NULL}};
const Segment SGI341158[] = {{&S53169[5], 5}, {&S53169[104], 5}, {NULL}};
const Segment SGI341159[] = {{&S53169[11], 3}, {NULL}};
const Segment SGI341160[] = {{&S53169[130], 3}, {NULL}};
const Segment SGI341161[] = {{&S53194[5], 7}, {NULL}};
const Segment SGI341162[] = {{&S53202[35], 5}, {NULL}};
const Segment SGI341163[] = {{&S53202[91], 3}, {NULL}};
const Segment SGI341164[] = {{&S53203[5], 14}, {NULL}};
const Segment SGI341165[] = {{&S53203[20], 3}, {NULL}};
const Segment SGI341166[] = {{&S53204[41], 5}, {NULL}};
const Segment SGI341167[] = {{&S53204[6], 3}, {NULL}};
const Segment SGI341168[] = {{&S53204[67], 3}, {NULL}};
const Segment SGI341169[] = {{&S44379[28], 3}, {NULL}};
const Segment SGI341170[] = {{&S44379[102], 5}, {NULL}};
const Segment SGI341171[] = {{&S28557[6], 4}, {NULL}};
const Segment SGI341172[] = {{&S28557[12], 14}, {NULL}};
const Segment SGI341173[] = {{&S28564[61], 4}, {NULL}};
const Segment SGI341174[] = {{&S28564[13], 10}, {NULL}};
const Segment SGI341175[] = {{&S28569[61], 4}, {NULL}};
const Segment SGI341176[] = {{&S28569[13], 10}, {NULL}};
const Segment SGI341177[] = {{&S28570[61], 4}, {NULL}};
const Segment SGI341178[] = {{&S28570[13], 10}, {NULL}};
const Segment SGI341179[] = {{&S53327[5], 5}, {&S53327[97], 5}, {NULL}};
const Segment SGI341180[] = {{&S53327[153], 3}, {NULL}};
const Segment SGI341181[] = {{&S53381[35], 4}, {NULL}};
const Segment SGI341182[] = {{&S53381[51], 3}, {NULL}};
const Segment SGI341183[] = {{&S53382[5], 4}, {NULL}};
const Segment SGI341184[] = {{&S53382[15], 3}, {NULL}};
const Segment SGI341185[] = {{&S53383[34], 4}, {NULL}};
const Segment SGI341186[] = {{&S53383[89], 3}, {NULL}};
const Segment SGI341187[] = {{&S53409[5], 3}, {&S53409[5], 3}, {NULL}};
const Segment SGI341188[] = {{&S53409[109], 3}, {NULL}};
const Segment SGI341189[] = {{&S53410[5], 3}, {&S53410[5], 3}, {&S53410[5], 3}, {NULL}};
const Segment SGI341190[] = {{&S53410[170], 3}, {NULL}};
const Segment SGI341191[] = {{&S53445[35], 3}, {NULL}};
const Segment SGI341192[] = {{&S53445[89], 3}, {NULL}};
const Segment SGI341193[] = {{&S53446[5], 12}, {NULL}};
const Segment SGI341194[] = {{&S53446[18], 3}, {NULL}};
const Segment SGI341195[] = {{&S53515[5], 3}, {&S53515[5], 3}, {NULL}};
const Segment SGI341196[] = {{&S53515[67], 3}, {NULL}};
const Segment SGI341197[] = {{&S53515[127], 3}, {NULL}};
const Segment SGI341198[] = {{&S53529[5], 3}, {NULL}};
const Segment SGI341199[] = {{&S53530[5], 4}, {NULL}};
const Segment SGI341200[] = {{&S53531[5], 3}, {&S53531[5], 3}, {NULL}};
const Segment SGI341201[] = {{&S53590[5], 7}, {&S53590[5], 7}, {NULL}};
const Segment SGI341202[] = {{&S53590[13], 3}, {NULL}};
const Segment SGI341203[] = {{&S53590[135], 3}, {NULL}};
const Segment SGI341204[] = {{&S53591[5], 5}, {&S53591[5], 5}, {NULL}};
const Segment SGI341205[] = {{&S53591[11], 3}, {NULL}};
const Segment SGI341206[] = {{&S53591[131], 3}, {NULL}};
const Segment SGI341207[] = {{&S53610[35], 3}, {NULL}};
const Segment SGI341208[] = {{&S53610[89], 3}, {NULL}};
const Segment SGI341209[] = {{&S53611[5], 12}, {NULL}};
const Segment SGI341210[] = {{&S53611[18], 3}, {NULL}};
const Segment SGI341211[] = {{&S53612[5], 3}, {NULL}};
const Segment SGI341212[] = {{&S53612[14], 3}, {NULL}};
const Segment SGI341213[] = {{&S53613[41], 5}, {NULL}};
const Segment SGI341214[] = {{&S53613[6], 3}, {NULL}};
const Segment SGI341215[] = {{&S53613[67], 3}, {NULL}};
const Segment SGI341216[] = {{&S53614[35], 4}, {NULL}};
const Segment SGI341217[] = {{&S53614[90], 3}, {NULL}};
const Segment SGI341218[] = {{&S53615[41], 5}, {NULL}};
const Segment SGI341219[] = {{&S53615[6], 3}, {NULL}};
const Segment SGI341220[] = {{&S53615[67], 3}, {NULL}};
const Segment SGI341221[] = {{&S53616[5], 13}, {&S53616[76], 13}, {NULL}};
const Segment SGI341222[] = {{&S53617[5], 13}, {NULL}};
const Segment SGI341223[] = {{&S53618[5], 10}, {NULL}};
const Segment SGI341224[] = {{&S53619[5], 5}, {NULL}};
const Segment SGI341225[] = {{&S39948[5], 17}, {&S39948[5], 9}, {NULL}};
const Segment SGI341226[] = {{&S39948[53], 3}, {NULL}};
const Segment SGI341227[] = {{&S39949[49], 9}, {NULL}};
const Segment SGI341228[] = {{&S39949[40], 3}, {NULL}};
const Segment SGI341229[] = {{&S39950[49], 7}, {NULL}};
const Segment SGI341230[] = {{&S39950[40], 3}, {NULL}};
const Segment SGI341231[] = {{&S39960[49], 5}, {NULL}};
const Segment SGI341232[] = {{&S39960[40], 3}, {NULL}};
const Segment SGI341233[] = {{&S39961[49], 5}, {NULL}};
const Segment SGI341234[] = {{&S39961[40], 3}, {NULL}};
const Segment SGI341235[] = {{&S39962[49], 3}, {NULL}};
const Segment SGI341236[] = {{&S39962[40], 3}, {NULL}};
const Segment SGI341237[] = {{&S39965[5], 3}, {NULL}};
const Segment SGI341238[] = {{&S39965[45], 4}, {NULL}};
const Segment SGI341239[] = {{&S44757[28], 3}, {NULL}};
const Segment SGI341240[] = {{&S44757[102], 5}, {NULL}};
const Segment SGI341241[] = {{&S53665[5], 4}, {NULL}};
const Segment SGI341242[] = {{&S53665[20], 3}, {NULL}};
const Segment SGI341243[] = {{&S53666[5], 6}, {NULL}};
const Segment SGI341244[] = {{&S53666[17], 3}, {NULL}};
const Segment SGI341245[] = {{&S53681[5], 11}, {NULL}};
const Segment SGI341246[] = {{&S53682[5], 11}, {&S53682[5], 11}, {NULL}};
const Segment SGI341247[] = {{&S28980[61], 5}, {NULL}};
const Segment SGI341248[] = {{&S28980[13], 10}, {NULL}};
const Segment SGI341249[] = {{&S40046[5], 4}, {NULL}};
const Segment SGI341250[] = {{&S40046[47], 3}, {NULL}};
const Segment SGI341251[] = {{&S40047[46], 7}, {NULL}};
const Segment SGI341252[] = {{&S40047[8], 3}, {NULL}};
const Segment SGI341253[] = {{&S40049[46], 7}, {&S40049[149], 7}, {NULL}};
const Segment SGI341254[] = {{&S40049[8], 3}, {NULL}};
const Segment SGI341255[] = {{&S26128[0], 21}, {NULL}};
const Segment SGI341256[] = {{&S26128[34], 3}, {NULL}};
const Segment SGI341257[] = {{&S26128[75], 13}, {NULL}};
const Segment SGI341258[] = {{&S26129[0], 21}, {NULL}};
const Segment SGI341259[] = {{&S26129[34], 3}, {NULL}};
const Segment SGI341260[] = {{&S26129[75], 13}, {NULL}};
const Segment SGI341261[] = {{&S53820[5], 7}, {&S53820[5], 7}, {NULL}};
const Segment SGI341262[] = {{&S53820[117], 3}, {NULL}};
const Segment SGI341263[] = {{&S53821[5], 4}, {&S53821[5], 4}, {NULL}};
const Segment SGI341264[] = {{&S53821[111], 3}, {NULL}};
const Segment SGI341265[] = {{&S53851[5], 7}, {NULL}};
const Segment SGI341266[] = {{&S53852[5], 7}, {&S53852[5], 7}, {NULL}};
const Segment SGI341267[] = {{&S53853[5], 5}, {NULL}};
const Segment SGI341268[] = {{&S53854[5], 8}, {NULL}};
const Segment SGI341269[] = {{&S53889[5], 4}, {NULL}};
const Segment SGI341270[] = {{&S53906[42], 4}, {NULL}};
const Segment SGI341271[] = {{&S53906[6], 3}, {NULL}};
const Segment SGI341272[] = {{&S53906[67], 3}, {NULL}};
const Segment SGI341273[] = {{&S53907[5], 4}, {NULL}};
const Segment SGI341274[] = {{&S53907[15], 3}, {NULL}};
const Segment SGI341275[] = {{&S53908[42], 6}, {NULL}};
const Segment SGI341276[] = {{&S53908[6], 3}, {NULL}};
const Segment SGI341277[] = {{&S53908[69], 3}, {NULL}};
const Segment SGI341278[] = {{&S53909[42], 7}, {NULL}};
const Segment SGI341279[] = {{&S53909[6], 3}, {NULL}};
const Segment SGI341280[] = {{&S53909[70], 3}, {NULL}};
const Segment SGI341281[] = {{&S53910[42], 5}, {NULL}};
const Segment SGI341282[] = {{&S53910[6], 3}, {NULL}};
const Segment SGI341283[] = {{&S53910[68], 3}, {NULL}};
const Segment SGI341284[] = {{&S53933[38], 5}, {NULL}};
const Segment SGI341285[] = {{&S53933[94], 3}, {NULL}};
const Segment SGI341286[] = {{&S53934[41], 5}, {NULL}};
const Segment SGI341287[] = {{&S53934[6], 3}, {NULL}};
const Segment SGI341288[] = {{&S53934[67], 3}, {NULL}};
const Segment SGI341289[] = {{&S53935[5], 14}, {NULL}};
const Segment SGI341290[] = {{&S53935[20], 3}, {NULL}};
const Segment SGI341291[] = {{&S53962[5], 7}, {&S53962[5], 7}, {NULL}};
const Segment SGI341292[] = {{&S53963[5], 7}, {NULL}};
const Segment SGI341293[] = {{&S53964[5], 8}, {NULL}};
const Segment SGI341294[] = {{&S53965[5], 13}, {NULL}};
const Segment SGI341295[] = {{&S54001[42], 7}, {NULL}};
const Segment SGI341296[] = {{&S54001[6], 3}, {NULL}};
const Segment SGI341297[] = {{&S54001[70], 3}, {NULL}};
const Segment SGI341298[] = {{&S54002[42], 5}, {NULL}};
const Segment SGI341299[] = {{&S54002[6], 3}, {NULL}};
const Segment SGI341300[] = {{&S54002[68], 3}, {NULL}};
const Segment SGI341301[] = {{&S54003[43], 5}, {NULL}};
const Segment SGI341302[] = {{&S54003[6], 3}, {NULL}};
const Segment SGI341303[] = {{&S54003[69], 3}, {NULL}};
const Segment SGI341304[] = {{&S54004[43], 7}, {NULL}};
const Segment SGI341305[] = {{&S54004[6], 3}, {NULL}};
const Segment SGI341306[] = {{&S54004[71], 3}, {NULL}};
const Segment SGI341307[] = {{&S54180[5], 4}, {&S54180[5], 4}, {NULL}};
const Segment SGI341308[] = {{&S54181[5], 4}, {NULL}};
const Segment SGI341309[] = {{&S45350[28], 5}, {NULL}};
const Segment SGI341310[] = {{&S45350[104], 5}, {NULL}};
const Segment SGI341311[] = {{&S45351[28], 7}, {NULL}};
const Segment SGI341312[] = {{&S45351[106], 5}, {NULL}};
const Segment SGI341313[] = {{&S45418[28], 3}, {NULL}};
const Segment SGI341314[] = {{&S45418[102], 5}, {NULL}};
const Segment SGI341315[] = {{&S45419[28], 3}, {NULL}};
const Segment SGI341316[] = {{&S45419[102], 5}, {NULL}};
const Segment SGI341317[] = {{&S45420[28], 3}, {NULL}};
const Segment SGI341318[] = {{&S45420[102], 5}, {NULL}};
const Segment SGI341319[] = {{&S45439[28], 3}, {NULL}};
const Segment SGI341320[] = {{&S45439[102], 5}, {NULL}};
const Segment SGI341321[] = {{&S45452[28], 5}, {NULL}};
const Segment SGI341322[] = {{&S45452[104], 5}, {NULL}};
const Segment SGI341323[] = {{&S54365[35], 5}, {NULL}};
const Segment SGI341324[] = {{&S54365[91], 3}, {NULL}};
const Segment SGI341325[] = {{&S54366[5], 5}, {NULL}};
const Segment SGI341326[] = {{&S54366[16], 3}, {NULL}};
const Segment SGI341327[] = {{&S54367[41], 5}, {NULL}};
const Segment SGI341328[] = {{&S54367[6], 3}, {NULL}};
const Segment SGI341329[] = {{&S54367[67], 3}, {NULL}};
const Segment SGI341330[] = {{&S54368[5], 14}, {NULL}};
const Segment SGI341331[] = {{&S54368[20], 3}, {NULL}};
const Segment SGI341332[] = {{&S45517[28], 3}, {NULL}};
const Segment SGI341333[] = {{&S45517[102], 5}, {NULL}};
const Segment SGI341334[] = {{&S29597[61], 5}, {NULL}};
const Segment SGI341335[] = {{&S29597[13], 10}, {NULL}};
const Segment SGI341336[] = {{&S45522[28], 3}, {NULL}};
const Segment SGI341337[] = {{&S45522[102], 5}, {NULL}};
const Segment SGI341338[] = {{&S45523[28], 5}, {NULL}};
const Segment SGI341339[] = {{&S45523[104], 5}, {NULL}};
const Segment SGI341340[] = {{&S45524[28], 4}, {NULL}};
const Segment SGI341341[] = {{&S45524[103], 5}, {NULL}};
const Segment SGI341342[] = {{&S45525[28], 5}, {NULL}};
const Segment SGI341343[] = {{&S27809[6], 4}, {NULL}};
const Segment SGI341344[] = {{&S27809[52], 5}, {NULL}};
const Segment SGI341345[] = {{&S27809[12], 45}, {NULL}};
const Segment SGI341346[] = {{&S27809[12], 14}, {NULL}};
const Segment SGI341347[] = {{&S54398[5], 4}, {NULL}};
const Segment SGI341348[] = {{&S27811[6], 4}, {NULL}};
const Segment SGI341349[] = {{&S27811[59], 5}, {NULL}};
const Segment SGI341350[] = {{&S27811[12], 14}, {NULL}};
const Segment SGI341351[] = {{&S54400[5], 5}, {&S54400[105], 5}, {NULL}};
const Segment SGI341352[] = {{&S54400[69], 3}, {NULL}};
const Segment SGI341353[] = {{&S27812[6], 4}, {NULL}};
const Segment SGI341354[] = {{&S27812[81], 5}, {NULL}};
const Segment SGI341355[] = {{&S27812[12], 14}, {&S27812[44], 42}, {NULL}};
const Segment SGI341356[] = {{&S54400[131], 3}, {NULL}};
const Segment SGI341357[] = {{&S27812[12], 14}, {NULL}};
const Segment SGI341358[] = {{&S27813[6], 4}, {NULL}};
const Segment SGI341359[] = {{&S27813[81], 5}, {NULL}};
const Segment SGI341360[] = {{&S27813[12], 14}, {&S27813[44], 42}, {NULL}};
const Segment SGI341361[] = {{&S27813[12], 14}, {NULL}};
const Segment SGI341362[] = {{&S27814[6], 4}, {NULL}};
const Segment SGI341363[] = {{&S27814[81], 5}, {NULL}};
const Segment SGI341364[] = {{&S27814[12], 14}, {&S27814[44], 42}, {NULL}};
const Segment SGI341365[] = {{&S27814[12], 14}, {NULL}};
const Segment SGI341366[] = {{&S27815[6], 4}, {NULL}};
const Segment SGI341367[] = {{&S27815[81], 5}, {NULL}};
const Segment SGI341368[] = {{&S27815[12], 14}, {&S27815[44], 42}, {NULL}};
const Segment SGI341369[] = {{&S27815[12], 14}, {NULL}};
const Segment SGI341370[] = {{&S27816[6], 4}, {NULL}};
const Segment SGI341371[] = {{&S27816[81], 5}, {NULL}};
const Segment SGI341372[] = {{&S27816[12], 14}, {&S27816[44], 42}, {NULL}};
const Segment SGI341373[] = {{&S27816[12], 14}, {NULL}};
const Segment SGI341374[] = {{&S27817[6], 4}, {NULL}};
const Segment SGI341375[] = {{&S27817[81], 5}, {NULL}};
const Segment SGI341376[] = {{&S27817[12], 14}, {&S27817[44], 42}, {NULL}};
const Segment SGI341377[] = {{&S27817[12], 14}, {NULL}};
const Segment SGI341378[] = {{&S27818[6], 4}, {NULL}};
const Segment SGI341379[] = {{&S27818[74], 5}, {NULL}};
const Segment SGI341380[] = {{&S27818[12], 14}, {&S27818[37], 42}, {NULL}};
const Segment SGI341381[] = {{&S27818[12], 14}, {NULL}};
const Segment SGI341382[] = {{&S27819[6], 4}, {NULL}};
const Segment SGI341383[] = {{&S27819[81], 5}, {NULL}};
const Segment SGI341384[] = {{&S27819[12], 14}, {&S27819[44], 42}, {NULL}};
const Segment SGI341385[] = {{&S27819[12], 14}, {NULL}};
const Segment SGI341386[] = {{&S27820[6], 4}, {NULL}};
const Segment SGI341387[] = {{&S27820[59], 5}, {NULL}};
const Segment SGI341388[] = {{&S27820[12], 14}, {&S27820[70], 4}, {NULL}};
const Segment SGI341389[] = {{&S27820[12], 14}, {NULL}};
const Segment SGI341390[] = {{&S27821[6], 4}, {NULL}};
const Segment SGI341391[] = {{&S27821[74], 5}, {NULL}};
const Segment SGI341392[] = {{&S27821[12], 14}, {&S27821[37], 42}, {NULL}};
const Segment SGI341393[] = {{&S27821[12], 14}, {NULL}};
const Segment SGI341394[] = {{&S27822[6], 4}, {NULL}};
const Segment SGI341395[] = {{&S27822[81], 5}, {NULL}};
const Segment SGI341396[] = {{&S27822[12], 14}, {&S27822[44], 42}, {NULL}};
const Segment SGI341397[] = {{&S27822[12], 14}, {NULL}};
const Segment SGI341398[] = {{&S27823[6], 4}, {NULL}};
const Segment SGI341399[] = {{&S27823[52], 5}, {NULL}};
const Segment SGI341400[] = {{&S27823[12], 55}, {NULL}};
const Segment SGI341401[] = {{&S27823[12], 14}, {NULL}};
const Segment SGI341402[] = {{&S27824[6], 4}, {NULL}};
const Segment SGI341403[] = {{&S27824[75], 5}, {NULL}};
const Segment SGI341404[] = {{&S27824[12], 14}, {&S27824[38], 42}, {NULL}};
const Segment SGI341405[] = {{&S27824[12], 14}, {NULL}};
const Segment SGI341406[] = {{&S27825[6], 4}, {NULL}};
const Segment SGI341407[] = {{&S27825[81], 5}, {NULL}};
const Segment SGI341408[] = {{&S27825[12], 14}, {&S27825[44], 42}, {NULL}};
const Segment SGI341409[] = {{&S27825[12], 14}, {NULL}};
const Segment SGI341410[] = {{&S27826[6], 4}, {NULL}};
const Segment SGI341411[] = {{&S27826[81], 5}, {NULL}};
const Segment SGI341412[] = {{&S27826[12], 14}, {&S27826[44], 42}, {NULL}};
const Segment SGI341413[] = {{&S27826[12], 14}, {NULL}};
const Segment SGI341414[] = {{&S27827[6], 4}, {NULL}};
const Segment SGI341415[] = {{&S27827[81], 5}, {NULL}};
const Segment SGI341416[] = {{&S27827[12], 14}, {&S27827[44], 42}, {NULL}};
const Segment SGI341417[] = {{&S27827[12], 14}, {NULL}};
const Segment SGI341418[] = {{&S27828[6], 4}, {NULL}};
const Segment SGI341419[] = {{&S27828[75], 5}, {NULL}};
const Segment SGI341420[] = {{&S27828[12], 14}, {&S27828[38], 42}, {NULL}};
const Segment SGI341421[] = {{&S27828[12], 14}, {NULL}};
const Segment SGI341422[] = {{&S27829[6], 4}, {NULL}};
const Segment SGI341423[] = {{&S27829[81], 5}, {NULL}};
const Segment SGI341424[] = {{&S27829[12], 14}, {&S27829[44], 42}, {NULL}};
const Segment SGI341425[] = {{&S27829[12], 14}, {NULL}};
const Segment SGI341426[] = {{&S27831[6], 4}, {NULL}};
const Segment SGI341427[] = {{&S27831[74], 5}, {NULL}};
const Segment SGI341428[] = {{&S27831[12], 14}, {&S27831[37], 42}, {NULL}};
const Segment SGI341429[] = {{&S27831[12], 14}, {NULL}};
const Segment SGI341430[] = {{&S27832[6], 4}, {NULL}};
const Segment SGI341431[] = {{&S27832[74], 5}, {NULL}};
const Segment SGI341432[] = {{&S27832[12], 14}, {&S27832[37], 42}, {NULL}};
const Segment SGI341433[] = {{&S27832[12], 14}, {NULL}};
const Segment SGI341434[] = {{&S27833[6], 4}, {NULL}};
const Segment SGI341435[] = {{&S27833[74], 5}, {NULL}};
const Segment SGI341436[] = {{&S27833[12], 14}, {&S27833[37], 42}, {NULL}};
const Segment SGI341437[] = {{&S27833[12], 14}, {NULL}};
const Segment SGI341438[] = {{&S27834[6], 4}, {NULL}};
const Segment SGI341439[] = {{&S27834[82], 5}, {NULL}};
const Segment SGI341440[] = {{&S27834[12], 14}, {&S27834[44], 43}, {NULL}};
const Segment SGI341441[] = {{&S27834[12], 14}, {NULL}};
const Segment SGI341442[] = {{&S27835[6], 4}, {NULL}};
const Segment SGI341443[] = {{&S27835[75], 5}, {NULL}};
const Segment SGI341444[] = {{&S27835[12], 14}, {&S27835[37], 43}, {NULL}};
const Segment SGI341445[] = {{&S27835[12], 14}, {NULL}};
const Segment SGI341446[] = {{&S27836[6], 4}, {NULL}};
const Segment SGI341447[] = {{&S27836[74], 5}, {NULL}};
const Segment SGI341448[] = {{&S27836[12], 14}, {&S27836[37], 42}, {NULL}};
const Segment SGI341449[] = {{&S27836[12], 14}, {NULL}};
const Segment SGI341450[] = {{&S27837[6], 4}, {NULL}};
const Segment SGI341451[] = {{&S27837[81], 5}, {NULL}};
const Segment SGI341452[] = {{&S27837[12], 14}, {&S27837[44], 42}, {NULL}};
const Segment SGI341453[] = {{&S27837[12], 14}, {NULL}};
const Segment SGI341454[] = {{&S27838[6], 4}, {NULL}};
const Segment SGI341455[] = {{&S27838[74], 5}, {NULL}};
const Segment SGI341456[] = {{&S27838[12], 14}, {&S27838[37], 42}, {NULL}};
const Segment SGI341457[] = {{&S27838[12], 14}, {NULL}};
const Segment SGI341458[] = {{&S27839[6], 4}, {NULL}};
const Segment SGI341459[] = {{&S27839[74], 5}, {NULL}};
const Segment SGI341460[] = {{&S27839[12], 14}, {&S27839[37], 42}, {NULL}};
const Segment SGI341461[] = {{&S27839[12], 14}, {NULL}};
const Segment SGI341462[] = {{&S27840[6], 4}, {NULL}};
const Segment SGI341463[] = {{&S27840[74], 5}, {NULL}};
const Segment SGI341464[] = {{&S27840[12], 14}, {&S27840[37], 42}, {NULL}};
const Segment SGI341465[] = {{&S27840[12], 14}, {NULL}};
const Segment SGI341466[] = {{&S27841[6], 4}, {NULL}};
const Segment SGI341467[] = {{&S27841[81], 5}, {NULL}};
const Segment SGI341468[] = {{&S27841[12], 14}, {&S27841[44], 42}, {NULL}};
const Segment SGI341469[] = {{&S27841[12], 14}, {NULL}};
const Segment SGI341470[] = {{&S27842[6], 4}, {NULL}};
const Segment SGI341471[] = {{&S27842[74], 5}, {NULL}};
const Segment SGI341472[] = {{&S27842[12], 14}, {&S27842[37], 42}, {NULL}};
const Segment SGI341473[] = {{&S27842[12], 14}, {NULL}};
const Segment SGI341474[] = {{&S27843[6], 4}, {NULL}};
const Segment SGI341475[] = {{&S27843[74], 5}, {NULL}};
const Segment SGI341476[] = {{&S27843[12], 14}, {&S27843[37], 42}, {NULL}};
const Segment SGI341477[] = {{&S27843[12], 14}, {NULL}};
const Segment SGI341478[] = {{&S27844[6], 4}, {NULL}};
const Segment SGI341479[] = {{&S27844[75], 5}, {NULL}};
const Segment SGI341480[] = {{&S27844[12], 14}, {&S27844[37], 43}, {NULL}};
const Segment SGI341481[] = {{&S27844[12], 14}, {NULL}};
const Segment SGI341482[] = {{&S27845[6], 4}, {NULL}};
const Segment SGI341483[] = {{&S27845[74], 5}, {NULL}};
const Segment SGI341484[] = {{&S27845[12], 14}, {&S27845[37], 42}, {NULL}};
const Segment SGI341485[] = {{&S27845[12], 14}, {NULL}};
const Segment SGI341486[] = {{&S27846[6], 4}, {NULL}};
const Segment SGI341487[] = {{&S27846[82], 5}, {NULL}};
const Segment SGI341488[] = {{&S27846[12], 14}, {&S27846[45], 42}, {NULL}};
const Segment SGI341489[] = {{&S27846[12], 14}, {NULL}};
const Segment SGI341490[] = {{&S27847[6], 4}, {NULL}};
const Segment SGI341491[] = {{&S27847[74], 5}, {NULL}};
const Segment SGI341492[] = {{&S27847[12], 14}, {&S27847[37], 42}, {NULL}};
const Segment SGI341493[] = {{&S27847[12], 14}, {NULL}};
const Segment SGI341494[] = {{&S27848[6], 4}, {NULL}};
const Segment SGI341495[] = {{&S27848[82], 5}, {NULL}};
const Segment SGI341496[] = {{&S27848[12], 14}, {&S27848[45], 42}, {NULL}};
const Segment SGI341497[] = {{&S27848[12], 14}, {NULL}};
const Segment SGI341498[] = {{&S27849[6], 4}, {NULL}};
const Segment SGI341499[] = {{&S27849[81], 5}, {NULL}};
const Segment SGI341500[] = {{&S27849[12], 14}, {&S27849[44], 42}, {NULL}};
const Segment SGI341501[] = {{&S27849[12], 14}, {NULL}};
const Segment SGI341502[] = {{&S27850[6], 4}, {NULL}};
const Segment SGI341503[] = {{&S27850[82], 5}, {NULL}};
const Segment SGI341504[] = {{&S27850[12], 14}, {&S27850[45], 42}, {NULL}};
const Segment SGI341505[] = {{&S27850[12], 14}, {NULL}};
const Segment SGI341506[] = {{&S27851[6], 4}, {NULL}};
const Segment SGI341507[] = {{&S27851[74], 5}, {NULL}};
const Segment SGI341508[] = {{&S27851[12], 14}, {&S27851[37], 42}, {NULL}};
const Segment SGI341509[] = {{&S27851[12], 14}, {NULL}};
const Segment SGI341510[] = {{&S27852[6], 4}, {NULL}};
const Segment SGI341511[] = {{&S27852[75], 5}, {NULL}};
const Segment SGI341512[] = {{&S27852[12], 14}, {&S27852[38], 42}, {NULL}};
const Segment SGI341513[] = {{&S27852[12], 14}, {NULL}};
const Segment SGI341514[] = {{&S27853[6], 4}, {NULL}};
const Segment SGI341515[] = {{&S27853[81], 5}, {NULL}};
const Segment SGI341516[] = {{&S27853[12], 14}, {&S27853[44], 42}, {NULL}};
const Segment SGI341517[] = {{&S27853[12], 14}, {NULL}};
const Segment SGI341518[] = {{&S27854[6], 4}, {NULL}};
const Segment SGI341519[] = {{&S27854[75], 5}, {NULL}};
const Segment SGI341520[] = {{&S27854[12], 14}, {&S27854[37], 43}, {NULL}};
const Segment SGI341521[] = {{&S27854[12], 14}, {NULL}};
const Segment SGI341522[] = {{&S27855[6], 4}, {NULL}};
const Segment SGI341523[] = {{&S27855[74], 5}, {NULL}};
const Segment SGI341524[] = {{&S27855[12], 14}, {&S27855[37], 42}, {NULL}};
const Segment SGI341525[] = {{&S27855[12], 14}, {NULL}};
const Segment SGI341526[] = {{&S27856[6], 4}, {NULL}};
const Segment SGI341527[] = {{&S27856[82], 5}, {NULL}};
const Segment SGI341528[] = {{&S27856[12], 14}, {&S27856[45], 42}, {NULL}};
const Segment SGI341529[] = {{&S27856[12], 14}, {NULL}};
const Segment SGI341530[] = {{&S27857[6], 4}, {NULL}};
const Segment SGI341531[] = {{&S27857[75], 5}, {NULL}};
const Segment SGI341532[] = {{&S27857[12], 14}, {&S27857[38], 42}, {NULL}};
const Segment SGI341533[] = {{&S27857[12], 14}, {NULL}};
const Segment SGI341534[] = {{&S27858[6], 4}, {NULL}};
const Segment SGI341535[] = {{&S27858[74], 5}, {NULL}};
const Segment SGI341536[] = {{&S27858[12], 14}, {&S27858[37], 42}, {NULL}};
const Segment SGI341537[] = {{&S27858[12], 14}, {NULL}};
const Segment SGI341538[] = {{&S27859[6], 4}, {NULL}};
const Segment SGI341539[] = {{&S27859[74], 5}, {NULL}};
const Segment SGI341540[] = {{&S27859[12], 14}, {&S27859[37], 42}, {NULL}};
const Segment SGI341541[] = {{&S27859[12], 14}, {NULL}};
const Segment SGI341542[] = {{&S27860[6], 4}, {NULL}};
const Segment SGI341543[] = {{&S27860[81], 5}, {NULL}};
const Segment SGI341544[] = {{&S27860[12], 14}, {&S27860[44], 42}, {NULL}};
const Segment SGI341545[] = {{&S27860[12], 14}, {NULL}};
const Segment SGI341546[] = {{&S27861[6], 4}, {NULL}};
const Segment SGI341547[] = {{&S27861[82], 5}, {NULL}};
const Segment SGI341548[] = {{&S27861[12], 14}, {&S27861[45], 42}, {NULL}};
const Segment SGI341549[] = {{&S27861[12], 14}, {NULL}};
const Segment SGI341550[] = {{&S27862[6], 4}, {NULL}};
const Segment SGI341551[] = {{&S27862[81], 5}, {NULL}};
const Segment SGI341552[] = {{&S27862[12], 14}, {&S27862[44], 42}, {NULL}};
const Segment SGI341553[] = {{&S27862[12], 14}, {NULL}};
const Segment SGI341554[] = {{&S27863[6], 4}, {NULL}};
const Segment SGI341555[] = {{&S27863[81], 5}, {NULL}};
const Segment SGI341556[] = {{&S27863[12], 14}, {&S27863[44], 42}, {NULL}};
const Segment SGI341557[] = {{&S27863[12], 14}, {NULL}};
const Segment SGI341558[] = {{&S27864[6], 4}, {NULL}};
const Segment SGI341559[] = {{&S27864[81], 5}, {NULL}};
const Segment SGI341560[] = {{&S27864[12], 14}, {&S27864[44], 42}, {NULL}};
const Segment SGI341561[] = {{&S27864[12], 14}, {NULL}};
const Segment SGI341562[] = {{&S27865[6], 4}, {NULL}};
const Segment SGI341563[] = {{&S27865[81], 5}, {NULL}};
const Segment SGI341564[] = {{&S27865[12], 14}, {&S27865[44], 42}, {NULL}};
const Segment SGI341565[] = {{&S27865[12], 14}, {NULL}};
const Segment SGI341566[] = {{&S27866[6], 4}, {NULL}};
const Segment SGI341567[] = {{&S27866[75], 5}, {NULL}};
const Segment SGI341568[] = {{&S27866[12], 14}, {&S27866[38], 42}, {NULL}};
const Segment SGI341569[] = {{&S27866[12], 14}, {NULL}};
const Segment SGI341570[] = {{&S27867[6], 4}, {NULL}};
const Segment SGI341571[] = {{&S27867[81], 5}, {NULL}};
const Segment SGI341572[] = {{&S27867[12], 14}, {&S27867[44], 42}, {NULL}};
const Segment SGI341573[] = {{&S27867[12], 14}, {NULL}};
const Segment SGI341574[] = {{&S27868[6], 4}, {NULL}};
const Segment SGI341575[] = {{&S27868[74], 5}, {NULL}};
const Segment SGI341576[] = {{&S27868[12], 14}, {&S27868[37], 42}, {NULL}};
const Segment SGI341577[] = {{&S27868[12], 14}, {NULL}};
const Segment SGI341578[] = {{&S27869[6], 4}, {NULL}};
const Segment SGI341579[] = {{&S27869[82], 5}, {NULL}};
const Segment SGI341580[] = {{&S27869[12], 14}, {&S27869[45], 42}, {NULL}};
const Segment SGI341581[] = {{&S27869[12], 14}, {NULL}};
const Segment SGI341582[] = {{&S27870[6], 4}, {NULL}};
const Segment SGI341583[] = {{&S27870[74], 5}, {NULL}};
const Segment SGI341584[] = {{&S27870[12], 14}, {&S27870[37], 42}, {NULL}};
const Segment SGI341585[] = {{&S27870[12], 14}, {NULL}};
const Segment SGI341586[] = {{&S27871[6], 4}, {NULL}};
const Segment SGI341587[] = {{&S27871[81], 5}, {NULL}};
const Segment SGI341588[] = {{&S27871[12], 14}, {&S27871[44], 42}, {NULL}};
const Segment SGI341589[] = {{&S27871[12], 14}, {NULL}};
const Segment SGI341590[] = {{&S27872[6], 4}, {NULL}};
const Segment SGI341591[] = {{&S27872[74], 5}, {NULL}};
const Segment SGI341592[] = {{&S27872[12], 14}, {&S27872[37], 42}, {NULL}};
const Segment SGI341593[] = {{&S27872[12], 14}, {NULL}};
const Segment SGI341594[] = {{&S27873[6], 4}, {NULL}};
const Segment SGI341595[] = {{&S27873[82], 5}, {NULL}};
const Segment SGI341596[] = {{&S27873[12], 14}, {&S27873[44], 43}, {NULL}};
const Segment SGI341597[] = {{&S27873[12], 14}, {NULL}};
const Segment SGI341598[] = {{&S27874[6], 4}, {NULL}};
const Segment SGI341599[] = {{&S27874[74], 5}, {NULL}};
const Segment SGI341600[] = {{&S27874[12], 14}, {&S27874[37], 42}, {NULL}};
const Segment SGI341601[] = {{&S27874[12], 14}, {NULL}};
const Segment SGI341602[] = {{&S27875[6], 4}, {NULL}};
const Segment SGI341603[] = {{&S27875[74], 5}, {NULL}};
const Segment SGI341604[] = {{&S27875[12], 14}, {&S27875[37], 42}, {NULL}};
const Segment SGI341605[] = {{&S27875[12], 14}, {NULL}};
const Segment SGI341606[] = {{&S54477[5], 7}, {&S54477[5], 7}, {NULL}};
const Segment SGI341607[] = {{&S27876[6], 4}, {NULL}};
const Segment SGI341608[] = {{&S54477[13], 3}, {NULL}};
const Segment SGI341609[] = {{&S27876[74], 5}, {NULL}};
const Segment SGI341610[] = {{&S27876[12], 14}, {&S27876[37], 42}, {NULL}};
const Segment SGI341611[] = {{&S27876[12], 14}, {NULL}};
const Segment SGI341612[] = {{&S54477[134], 3}, {NULL}};
const Segment SGI341613[] = {{&S27877[6], 4}, {NULL}};
const Segment SGI341614[] = {{&S27877[81], 5}, {NULL}};
const Segment SGI341615[] = {{&S27877[12], 14}, {&S27877[44], 42}, {NULL}};
const Segment SGI341616[] = {{&S27877[12], 14}, {NULL}};
const Segment SGI341617[] = {{&S54478[5], 5}, {&S54478[5], 5}, {NULL}};
const Segment SGI341618[] = {{&S54478[11], 3}, {NULL}};
const Segment SGI341619[] = {{&S54478[130], 3}, {NULL}};
const Segment SGI341620[] = {{&S27878[6], 4}, {NULL}};
const Segment SGI341621[] = {{&S27878[74], 5}, {NULL}};
const Segment SGI341622[] = {{&S27878[12], 14}, {&S27878[37], 42}, {NULL}};
const Segment SGI341623[] = {{&S27878[12], 14}, {NULL}};
const Segment SGI341624[] = {{&S27880[6], 4}, {NULL}};
const Segment SGI341625[] = {{&S27880[74], 5}, {NULL}};
const Segment SGI341626[] = {{&S27880[12], 14}, {&S27880[37], 42}, {NULL}};
const Segment SGI341627[] = {{&S27880[12], 14}, {NULL}};
const Segment SGI341628[] = {{&S27881[6], 4}, {NULL}};
const Segment SGI341629[] = {{&S27881[74], 5}, {NULL}};
const Segment SGI341630[] = {{&S27881[12], 14}, {&S27881[37], 42}, {NULL}};
const Segment SGI341631[] = {{&S27881[12], 14}, {NULL}};
const Segment SGI341632[] = {{&S27882[6], 4}, {NULL}};
const Segment SGI341633[] = {{&S27882[74], 5}, {NULL}};
const Segment SGI341634[] = {{&S27882[12], 14}, {&S27882[37], 42}, {NULL}};
const Segment SGI341635[] = {{&S27882[12], 14}, {NULL}};
const Segment SGI341636[] = {{&S27883[6], 4}, {NULL}};
const Segment SGI341637[] = {{&S27883[81], 5}, {NULL}};
const Segment SGI341638[] = {{&S27883[12], 14}, {&S27883[44], 42}, {NULL}};
const Segment SGI341639[] = {{&S27883[12], 14}, {NULL}};
const Segment SGI341640[] = {{&S27884[6], 4}, {NULL}};
const Segment SGI341641[] = {{&S27884[74], 5}, {NULL}};
const Segment SGI341642[] = {{&S27884[12], 14}, {&S27884[37], 42}, {NULL}};
const Segment SGI341643[] = {{&S27884[12], 14}, {NULL}};
const Segment SGI341644[] = {{&S54503[43], 4}, {NULL}};
const Segment SGI341645[] = {{&S54503[6], 3}, {NULL}};
const Segment SGI341646[] = {{&S54503[68], 3}, {NULL}};
const Segment SGI341647[] = {{&S54504[43], 5}, {NULL}};
const Segment SGI341648[] = {{&S54504[6], 3}, {NULL}};
const Segment SGI341649[] = {{&S54504[69], 3}, {NULL}};
const Segment SGI341650[] = {{&S54508[5], 5}, {&S54508[97], 5}, {NULL}};
const Segment SGI341651[] = {{&S54508[113], 3}, {NULL}};
const Segment SGI341652[] = {{&S45525[104], 5}, {NULL}};
const Segment SGI341653[] = {{&S45526[28], 4}, {NULL}};
const Segment SGI341654[] = {{&S45526[103], 5}, {NULL}};
const Segment SGI341655[] = {{&S45527[28], 4}, {NULL}};
const Segment SGI341656[] = {{&S45527[103], 5}, {NULL}};
const Segment SGI341657[] = {{&S40740[42], 5}, {NULL}};
const Segment SGI341658[] = {{&S40740[33], 3}, {NULL}};
const Segment SGI341659[] = {{&S40741[42], 7}, {NULL}};
const Segment SGI341660[] = {{&S40741[33], 3}, {NULL}};
const Segment SGI341661[] = {{&S40742[42], 7}, {&S40742[144], 7}, {NULL}};
const Segment SGI341662[] = {{&S40742[33], 3}, {NULL}};
const Segment SGI341663[] = {{&S40743[42], 5}, {&S40743[142], 7}, {NULL}};
const Segment SGI341664[] = {{&S40743[33], 3}, {NULL}};
const Segment SGI341665[] = {{&S45681[28], 5}, {NULL}};
const Segment SGI341666[] = {{&S45681[104], 5}, {NULL}};
const Segment SGI341667[] = {{&S28133[6], 4}, {NULL}};
const Segment SGI341668[] = {{&S28133[80], 5}, {NULL}};
const Segment SGI341669[] = {{&S28133[12], 14}, {&S28133[40], 11}, {NULL}};
const Segment SGI341670[] = {{&S28133[12], 14}, {NULL}};
const Segment SGI341671[] = {{&S28134[6], 4}, {NULL}};
const Segment SGI341672[] = {{&S28134[58], 5}, {NULL}};
const Segment SGI341673[] = {{&S28134[12], 14}, {NULL}};
const Segment SGI341674[] = {{&S28135[6], 4}, {NULL}};
const Segment SGI341675[] = {{&S28135[80], 5}, {NULL}};
const Segment SGI341676[] = {{&S28135[12], 14}, {&S28135[40], 11}, {NULL}};
const Segment SGI341677[] = {{&S28135[12], 14}, {NULL}};
const Segment SGI341678[] = {{&S28136[6], 4}, {NULL}};
const Segment SGI341679[] = {{&S28136[80], 5}, {NULL}};
const Segment SGI341680[] = {{&S28136[12], 14}, {&S28136[40], 11}, {NULL}};
const Segment SGI341681[] = {{&S28136[12], 14}, {NULL}};
const Segment SGI341682[] = {{&S28137[6], 4}, {NULL}};
const Segment SGI341683[] = {{&S28137[58], 5}, {NULL}};
const Segment SGI341684[] = {{&S28137[12], 14}, {NULL}};
const Segment SGI341685[] = {{&S28138[6], 4}, {NULL}};
const Segment SGI341686[] = {{&S28138[81], 5}, {NULL}};
const Segment SGI341687[] = {{&S28138[12], 14}, {&S28138[40], 12}, {NULL}};
const Segment SGI341688[] = {{&S28138[12], 14}, {NULL}};
const Segment SGI341689[] = {{&S28139[6], 4}, {NULL}};
const Segment SGI341690[] = {{&S28139[81], 5}, {NULL}};
const Segment SGI341691[] = {{&S28139[12], 14}, {&S28139[40], 12}, {NULL}};
const Segment SGI341692[] = {{&S28139[12], 14}, {NULL}};
const Segment SGI341693[] = {{&S28140[6], 4}, {NULL}};
const Segment SGI341694[] = {{&S28140[81], 5}, {NULL}};
const Segment SGI341695[] = {{&S28140[12], 14}, {&S28140[40], 12}, {NULL}};
const Segment SGI341696[] = {{&S28140[12], 14}, {NULL}};
const Segment SGI341697[] = {{&S28141[6], 4}, {NULL}};
const Segment SGI341698[] = {{&S28141[81], 5}, {NULL}};
const Segment SGI341699[] = {{&S28141[12], 14}, {&S28141[40], 12}, {NULL}};
const Segment SGI341700[] = {{&S28141[12], 14}, {NULL}};
const Segment SGI341701[] = {{&S28142[6], 4}, {NULL}};
const Segment SGI341702[] = {{&S28142[58], 5}, {NULL}};
const Segment SGI341703[] = {{&S28142[12], 14}, {&S28142[69], 31}, {NULL}};
const Segment SGI341704[] = {{&S28142[12], 14}, {NULL}};
const Segment SGI341705[] = {{&S28143[6], 4}, {NULL}};
const Segment SGI341706[] = {{&S28143[81], 5}, {NULL}};
const Segment SGI341707[] = {{&S28143[12], 14}, {&S28143[40], 12}, {NULL}};
const Segment SGI341708[] = {{&S28143[12], 14}, {NULL}};
const Segment SGI341709[] = {{&S54613[35], 16}, {NULL}};
const Segment SGI341710[] = {{&S54613[102], 3}, {NULL}};
const Segment SGI341711[] = {{&S28144[6], 4}, {NULL}};
const Segment SGI341712[] = {{&S28144[81], 5}, {NULL}};
const Segment SGI341713[] = {{&S28144[12], 14}, {&S28144[40], 12}, {NULL}};
const Segment SGI341714[] = {{&S28144[12], 14}, {NULL}};
const Segment SGI341715[] = {{&S54614[5], 16}, {NULL}};
const Segment SGI341716[] = {{&S54614[27], 3}, {NULL}};
const Segment SGI341717[] = {{&S28145[6], 4}, {NULL}};
const Segment SGI341718[] = {{&S28145[81], 5}, {NULL}};
const Segment SGI341719[] = {{&S28145[12], 14}, {&S28145[40], 12}, {NULL}};
const Segment SGI341720[] = {{&S28145[12], 14}, {NULL}};
const Segment SGI341721[] = {{&S54615[34], 7}, {NULL}};
const Segment SGI341722[] = {{&S28146[6], 4}, {NULL}};
const Segment SGI341723[] = {{&S54615[92], 3}, {NULL}};
const Segment SGI341724[] = {{&S28146[80], 5}, {NULL}};
const Segment SGI341725[] = {{&S28146[12], 14}, {&S28146[40], 11}, {NULL}};
const Segment SGI341726[] = {{&S28146[12], 14}, {NULL}};
const Segment SGI341727[] = {{&S54616[5], 17}, {NULL}};
const Segment SGI341728[] = {{&S54616[28], 3}, {NULL}};
const Segment SGI341729[] = {{&S28147[6], 4}, {NULL}};
const Segment SGI341730[] = {{&S28147[80], 5}, {NULL}};
const Segment SGI341731[] = {{&S28147[12], 14}, {&S28147[40], 11}, {NULL}};
const Segment SGI341732[] = {{&S28147[12], 14}, {NULL}};
const Segment SGI341733[] = {{&S28148[6], 4}, {NULL}};
const Segment SGI341734[] = {{&S54617[35], 5}, {NULL}};
const Segment SGI341735[] = {{&S28148[80], 5}, {NULL}};
const Segment SGI341736[] = {{&S28148[12], 14}, {&S28148[40], 11}, {NULL}};
const Segment SGI341737[] = {{&S28148[12], 14}, {NULL}};
const Segment SGI341738[] = {{&S54617[91], 3}, {NULL}};
const Segment SGI341739[] = {{&S28149[6], 4}, {NULL}};
const Segment SGI341740[] = {{&S28149[81], 5}, {NULL}};
const Segment SGI341741[] = {{&S28149[12], 14}, {&S28149[40], 12}, {NULL}};
const Segment SGI341742[] = {{&S28149[12], 14}, {NULL}};
const Segment SGI341743[] = {{&S54618[35], 7}, {NULL}};
const Segment SGI341744[] = {{&S54618[93], 3}, {NULL}};
const Segment SGI341745[] = {{&S28150[6], 4}, {NULL}};
const Segment SGI341746[] = {{&S28150[80], 5}, {NULL}};
const Segment SGI341747[] = {{&S28150[12], 14}, {&S28150[40], 11}, {NULL}};
const Segment SGI341748[] = {{&S28150[12], 14}, {NULL}};
const Segment SGI341749[] = {{&S54619[34], 7}, {NULL}};
const Segment SGI341750[] = {{&S54619[92], 3}, {NULL}};
const Segment SGI341751[] = {{&S28151[6], 4}, {NULL}};
const Segment SGI341752[] = {{&S28151[81], 5}, {NULL}};
const Segment SGI341753[] = {{&S28151[12], 14}, {&S28151[40], 12}, {NULL}};
const Segment SGI341754[] = {{&S28151[12], 14}, {NULL}};
const Segment SGI341755[] = {{&S54620[34], 5}, {NULL}};
const Segment SGI341756[] = {{&S54620[90], 3}, {NULL}};
const Segment SGI341757[] = {{&S28152[6], 4}, {NULL}};
const Segment SGI341758[] = {{&S28152[81], 5}, {NULL}};
const Segment SGI341759[] = {{&S28152[12], 14}, {&S28152[40], 12}, {NULL}};
const Segment SGI341760[] = {{&S28152[12], 14}, {NULL}};
const Segment SGI341761[] = {{&S28153[6], 4}, {NULL}};
const Segment SGI341762[] = {{&S28153[81], 5}, {NULL}};
const Segment SGI341763[] = {{&S28153[12], 14}, {&S28153[40], 12}, {NULL}};
const Segment SGI341764[] = {{&S28153[12], 14}, {NULL}};
const Segment SGI341765[] = {{&S28154[6], 4}, {NULL}};
const Segment SGI341766[] = {{&S28154[80], 5}, {NULL}};
const Segment SGI341767[] = {{&S28154[12], 14}, {&S28154[40], 11}, {NULL}};
const Segment SGI341768[] = {{&S28154[12], 14}, {NULL}};
const Segment SGI341769[] = {{&S28155[6], 4}, {NULL}};
const Segment SGI341770[] = {{&S28155[80], 5}, {NULL}};
const Segment SGI341771[] = {{&S28155[12], 14}, {&S28155[40], 11}, {NULL}};
const Segment SGI341772[] = {{&S28155[12], 14}, {NULL}};
const Segment SGI341773[] = {{&S28156[6], 4}, {NULL}};
const Segment SGI341774[] = {{&S28156[12], 22}, {NULL}};
const Segment SGI341775[] = {{&S28156[12], 14}, {NULL}};
const Segment SGI341776[] = {{&S28157[6], 4}, {NULL}};
const Segment SGI341777[] = {{&S28157[80], 5}, {NULL}};
const Segment SGI341778[] = {{&S28157[12], 14}, {&S28157[40], 11}, {NULL}};
const Segment SGI341779[] = {{&S28157[12], 14}, {NULL}};
const Segment SGI341780[] = {{&S28158[6], 4}, {NULL}};
const Segment SGI341781[] = {{&S28158[80], 5}, {NULL}};
const Segment SGI341782[] = {{&S28158[12], 14}, {&S28158[40], 11}, {NULL}};
const Segment SGI341783[] = {{&S28158[12], 14}, {NULL}};
const Segment SGI341784[] = {{&S28159[6], 4}, {NULL}};
const Segment SGI341785[] = {{&S28159[81], 5}, {NULL}};
const Segment SGI341786[] = {{&S28159[12], 14}, {&S28159[40], 12}, {NULL}};
const Segment SGI341787[] = {{&S28159[12], 14}, {NULL}};
const Segment SGI341788[] = {{&S28160[6], 4}, {NULL}};
const Segment SGI341789[] = {{&S28160[80], 5}, {NULL}};
const Segment SGI341790[] = {{&S28160[12], 14}, {&S28160[40], 11}, {NULL}};
const Segment SGI341791[] = {{&S28160[12], 14}, {NULL}};
const Segment SGI341792[] = {{&S28161[6], 4}, {NULL}};
const Segment SGI341793[] = {{&S28161[80], 5}, {NULL}};
const Segment SGI341794[] = {{&S28161[12], 14}, {&S28161[40], 11}, {NULL}};
const Segment SGI341795[] = {{&S28161[12], 14}, {NULL}};
const Segment SGI341796[] = {{&S28162[6], 4}, {NULL}};
const Segment SGI341797[] = {{&S28162[80], 5}, {NULL}};
const Segment SGI341798[] = {{&S28162[12], 14}, {&S28162[40], 11}, {NULL}};
const Segment SGI341799[] = {{&S28162[12], 14}, {NULL}};
const Segment SGI341800[] = {{&S28163[6], 4}, {NULL}};
const Segment SGI341801[] = {{&S28163[80], 5}, {NULL}};
const Segment SGI341802[] = {{&S28163[12], 14}, {&S28163[40], 11}, {NULL}};
const Segment SGI341803[] = {{&S28163[12], 14}, {NULL}};
const Segment SGI341804[] = {{&S28164[6], 4}, {NULL}};
const Segment SGI341805[] = {{&S28164[80], 5}, {NULL}};
const Segment SGI341806[] = {{&S28164[12], 14}, {&S28164[40], 11}, {NULL}};
const Segment SGI341807[] = {{&S28164[12], 14}, {NULL}};
const Segment SGI341808[] = {{&S28165[6], 4}, {NULL}};
const Segment SGI341809[] = {{&S28165[81], 5}, {NULL}};
const Segment SGI341810[] = {{&S28165[12], 14}, {&S28165[41], 11}, {NULL}};
const Segment SGI341811[] = {{&S28165[12], 14}, {NULL}};
const Segment SGI341812[] = {{&S28166[6], 4}, {NULL}};
const Segment SGI341813[] = {{&S28166[80], 5}, {NULL}};
const Segment SGI341814[] = {{&S28166[12], 14}, {&S28166[40], 11}, {NULL}};
const Segment SGI341815[] = {{&S28166[12], 14}, {NULL}};
const Segment SGI341816[] = {{&S28167[6], 4}, {NULL}};
const Segment SGI341817[] = {{&S28167[80], 5}, {NULL}};
const Segment SGI341818[] = {{&S28167[12], 14}, {&S28167[40], 11}, {NULL}};
const Segment SGI341819[] = {{&S28167[12], 14}, {NULL}};
const Segment SGI341820[] = {{&S28168[6], 4}, {NULL}};
const Segment SGI341821[] = {{&S28168[58], 5}, {NULL}};
const Segment SGI341822[] = {{&S28168[12], 14}, {&S28168[69], 31}, {NULL}};
const Segment SGI341823[] = {{&S28168[12], 14}, {NULL}};
const Segment SGI341824[] = {{&S28169[6], 4}, {NULL}};
const Segment SGI341825[] = {{&S28169[81], 5}, {NULL}};
const Segment SGI341826[] = {{&S28169[12], 14}, {&S28169[40], 12}, {NULL}};
const Segment SGI341827[] = {{&S28169[12], 14}, {NULL}};
const Segment SGI341828[] = {{&S28170[6], 4}, {NULL}};
const Segment SGI341829[] = {{&S28170[80], 5}, {NULL}};
const Segment SGI341830[] = {{&S28170[12], 14}, {&S28170[40], 11}, {NULL}};
const Segment SGI341831[] = {{&S28170[12], 14}, {NULL}};
const Segment SGI341832[] = {{&S28171[6], 4}, {NULL}};
const Segment SGI341833[] = {{&S28171[80], 5}, {NULL}};
const Segment SGI341834[] = {{&S28171[12], 14}, {&S28171[40], 11}, {NULL}};
const Segment SGI341835[] = {{&S28171[12], 14}, {NULL}};
const Segment SGI341836[] = {{&S28172[6], 4}, {NULL}};
const Segment SGI341837[] = {{&S28172[81], 5}, {NULL}};
const Segment SGI341838[] = {{&S28172[12], 14}, {&S28172[41], 11}, {NULL}};
const Segment SGI341839[] = {{&S28172[12], 14}, {NULL}};
const Segment SGI341840[] = {{&S28173[6], 4}, {NULL}};
const Segment SGI341841[] = {{&S28173[81], 5}, {NULL}};
const Segment SGI341842[] = {{&S28173[12], 14}, {&S28173[41], 11}, {NULL}};
const Segment SGI341843[] = {{&S28173[12], 14}, {NULL}};
const Segment SGI341844[] = {{&S28174[6], 4}, {NULL}};
const Segment SGI341845[] = {{&S28174[81], 5}, {NULL}};
const Segment SGI341846[] = {{&S28174[12], 14}, {&S28174[40], 12}, {NULL}};
const Segment SGI341847[] = {{&S28174[12], 14}, {NULL}};
const Segment SGI341848[] = {{&S28175[6], 4}, {NULL}};
const Segment SGI341849[] = {{&S28175[81], 5}, {NULL}};
const Segment SGI341850[] = {{&S28175[12], 14}, {&S28175[41], 11}, {NULL}};
const Segment SGI341851[] = {{&S28175[12], 14}, {NULL}};
const Segment SGI341852[] = {{&S28176[6], 4}, {NULL}};
const Segment SGI341853[] = {{&S28176[81], 5}, {NULL}};
const Segment SGI341854[] = {{&S28176[12], 14}, {&S28176[41], 11}, {NULL}};
const Segment SGI341855[] = {{&S28176[12], 14}, {NULL}};
const Segment SGI341856[] = {{&S28177[6], 4}, {NULL}};
const Segment SGI341857[] = {{&S28177[81], 5}, {NULL}};
const Segment SGI341858[] = {{&S28177[12], 14}, {&S28177[41], 11}, {NULL}};
const Segment SGI341859[] = {{&S28177[12], 14}, {NULL}};
const Segment SGI341860[] = {{&S28178[6], 4}, {NULL}};
const Segment SGI341861[] = {{&S28178[81], 5}, {NULL}};
const Segment SGI341862[] = {{&S28178[12], 14}, {&S28178[40], 12}, {NULL}};
const Segment SGI341863[] = {{&S28178[12], 14}, {NULL}};
const Segment SGI341864[] = {{&S54654[5], 4}, {NULL}};
const Segment SGI341865[] = {{&S28179[6], 4}, {NULL}};
const Segment SGI341866[] = {{&S28179[80], 5}, {NULL}};
const Segment SGI341867[] = {{&S28179[12], 14}, {&S28179[40], 11}, {NULL}};
const Segment SGI341868[] = {{&S28179[12], 14}, {NULL}};
const Segment SGI341869[] = {{&S54655[5], 5}, {NULL}};
const Segment SGI341870[] = {{&S28180[6], 4}, {NULL}};
const Segment SGI341871[] = {{&S28180[80], 5}, {NULL}};
const Segment SGI341872[] = {{&S28180[12], 14}, {&S28180[40], 11}, {NULL}};
const Segment SGI341873[] = {{&S28180[12], 14}, {NULL}};
const Segment SGI341874[] = {{&S54656[5], 4}, {&S54656[5], 4}, {NULL}};
const Segment SGI341875[] = {{&S28181[6], 4}, {NULL}};
const Segment SGI341876[] = {{&S28181[81], 5}, {NULL}};
const Segment SGI341877[] = {{&S28181[12], 14}, {&S28181[40], 12}, {NULL}};
const Segment SGI341878[] = {{&S28181[12], 14}, {NULL}};
const Segment SGI341879[] = {{&S28182[6], 4}, {NULL}};
const Segment SGI341880[] = {{&S28182[81], 5}, {NULL}};
const Segment SGI341881[] = {{&S28182[12], 14}, {&S28182[41], 11}, {NULL}};
const Segment SGI341882[] = {{&S28182[12], 14}, {NULL}};
const Segment SGI341883[] = {{&S28183[6], 4}, {NULL}};
const Segment SGI341884[] = {{&S28183[80], 5}, {NULL}};
const Segment SGI341885[] = {{&S28183[12], 14}, {&S28183[40], 11}, {NULL}};
const Segment SGI341886[] = {{&S28183[12], 14}, {NULL}};
const Segment SGI341887[] = {{&S28184[6], 4}, {NULL}};
const Segment SGI341888[] = {{&S28184[81], 5}, {NULL}};
const Segment SGI341889[] = {{&S28184[12], 14}, {&S28184[41], 11}, {NULL}};
const Segment SGI341890[] = {{&S28184[12], 14}, {NULL}};
const Segment SGI341891[] = {{&S28185[6], 4}, {NULL}};
const Segment SGI341892[] = {{&S28185[80], 5}, {NULL}};
const Segment SGI341893[] = {{&S28185[12], 14}, {&S28185[40], 11}, {NULL}};
const Segment SGI341894[] = {{&S28185[12], 14}, {NULL}};
const Segment SGI341895[] = {{&S28186[6], 4}, {NULL}};
const Segment SGI341896[] = {{&S28186[80], 5}, {NULL}};
const Segment SGI341897[] = {{&S28186[12], 14}, {&S28186[40], 11}, {NULL}};
const Segment SGI341898[] = {{&S28186[12], 14}, {NULL}};
const Segment SGI341899[] = {{&S28187[6], 4}, {NULL}};
const Segment SGI341900[] = {{&S28187[12], 14}, {NULL}};
const Segment SGI341901[] = {{&S28188[6], 4}, {NULL}};
const Segment SGI341902[] = {{&S28188[80], 5}, {NULL}};
const Segment SGI341903[] = {{&S28188[12], 14}, {&S28188[40], 11}, {NULL}};
const Segment SGI341904[] = {{&S28188[12], 14}, {NULL}};
const Segment SGI341905[] = {{&S28189[6], 4}, {NULL}};
const Segment SGI341906[] = {{&S28189[80], 5}, {NULL}};
const Segment SGI341907[] = {{&S28189[12], 14}, {&S28189[40], 11}, {NULL}};
const Segment SGI341908[] = {{&S28189[12], 14}, {NULL}};
const Segment SGI341909[] = {{&S28190[6], 4}, {NULL}};
const Segment SGI341910[] = {{&S28190[80], 5}, {NULL}};
const Segment SGI341911[] = {{&S28190[12], 14}, {&S28190[40], 11}, {NULL}};
const Segment SGI341912[] = {{&S28190[12], 14}, {NULL}};
const Segment SGI341913[] = {{&S28191[6], 4}, {NULL}};
const Segment SGI341914[] = {{&S28191[80], 5}, {NULL}};
const Segment SGI341915[] = {{&S28191[12], 14}, {&S28191[40], 11}, {NULL}};
const Segment SGI341916[] = {{&S28191[12], 14}, {NULL}};
const Segment SGI341917[] = {{&S28192[6], 4}, {NULL}};
const Segment SGI341918[] = {{&S28192[80], 5}, {NULL}};
const Segment SGI341919[] = {{&S28192[12], 14}, {&S28192[40], 11}, {NULL}};
const Segment SGI341920[] = {{&S28192[12], 14}, {NULL}};
const Segment SGI341921[] = {{&S28193[6], 4}, {NULL}};
const Segment SGI341922[] = {{&S28193[80], 5}, {NULL}};
const Segment SGI341923[] = {{&S28193[12], 14}, {&S28193[40], 11}, {NULL}};
const Segment SGI341924[] = {{&S28193[12], 14}, {NULL}};
const Segment SGI341925[] = {{&S28194[6], 4}, {NULL}};
const Segment SGI341926[] = {{&S28194[80], 5}, {NULL}};
const Segment SGI341927[] = {{&S28194[12], 14}, {&S28194[40], 11}, {NULL}};
const Segment SGI341928[] = {{&S28194[12], 14}, {NULL}};
const Segment SGI341929[] = {{&S28195[6], 4}, {NULL}};
const Segment SGI341930[] = {{&S28195[80], 5}, {NULL}};
const Segment SGI341931[] = {{&S28195[12], 14}, {&S28195[40], 11}, {NULL}};
const Segment SGI341932[] = {{&S28195[12], 14}, {NULL}};
const Segment SGI341933[] = {{&S28196[6], 4}, {NULL}};
const Segment SGI341934[] = {{&S28196[80], 5}, {NULL}};
const Segment SGI341935[] = {{&S28196[12], 14}, {&S28196[40], 11}, {NULL}};
const Segment SGI341936[] = {{&S28196[12], 14}, {NULL}};
const Segment SGI341937[] = {{&S28197[6], 4}, {NULL}};
const Segment SGI341938[] = {{&S28197[58], 5}, {NULL}};
const Segment SGI341939[] = {{&S28197[12], 14}, {&S28197[69], 4}, {NULL}};
const Segment SGI341940[] = {{&S28197[12], 14}, {NULL}};
const Segment SGI341941[] = {{&S28198[6], 4}, {NULL}};
const Segment SGI341942[] = {{&S28198[58], 5}, {NULL}};
const Segment SGI341943[] = {{&S28198[12], 14}, {&S28198[69], 4}, {NULL}};
const Segment SGI341944[] = {{&S28198[12], 14}, {NULL}};
const Segment SGI341945[] = {{&S28199[6], 4}, {NULL}};
const Segment SGI341946[] = {{&S28199[80], 5}, {NULL}};
const Segment SGI341947[] = {{&S28199[12], 14}, {&S28199[40], 11}, {NULL}};
const Segment SGI341948[] = {{&S28199[12], 14}, {NULL}};
const Segment SGI341949[] = {{&S54678[5], 5}, {&S54678[97], 5}, {NULL}};
const Segment SGI341950[] = {{&S28200[6], 4}, {NULL}};
const Segment SGI341951[] = {{&S54678[17], 3}, {NULL}};
const Segment SGI341952[] = {{&S28200[80], 5}, {NULL}};
const Segment SGI341953[] = {{&S28200[12], 14}, {&S28200[40], 11}, {NULL}};
const Segment SGI341954[] = {{&S28200[12], 14}, {NULL}};
const Segment SGI341955[] = {{&S28201[6], 4}, {NULL}};
const Segment SGI341956[] = {{&S28201[81], 5}, {NULL}};
const Segment SGI341957[] = {{&S28201[12], 14}, {&S28201[41], 11}, {NULL}};
const Segment SGI341958[] = {{&S28201[12], 14}, {NULL}};
const Segment SGI341959[] = {{&S28202[6], 4}, {NULL}};
const Segment SGI341960[] = {{&S28202[81], 5}, {NULL}};
const Segment SGI341961[] = {{&S28202[12], 14}, {&S28202[41], 11}, {NULL}};
const Segment SGI341962[] = {{&S28202[12], 14}, {NULL}};
const Segment SGI341963[] = {{&S28203[6], 4}, {NULL}};
const Segment SGI341964[] = {{&S28203[80], 5}, {NULL}};
const Segment SGI341965[] = {{&S28203[12], 14}, {&S28203[40], 11}, {NULL}};
const Segment SGI341966[] = {{&S28203[12], 14}, {NULL}};
const Segment SGI341967[] = {{&S54682[5], 5}, {&S54682[97], 5}, {NULL}};
const Segment SGI341968[] = {{&S28204[6], 4}, {NULL}};
const Segment SGI341969[] = {{&S28204[80], 5}, {NULL}};
const Segment SGI341970[] = {{&S54682[153], 3}, {NULL}};
const Segment SGI341971[] = {{&S28204[12], 14}, {&S28204[40], 11}, {NULL}};
const Segment SGI341972[] = {{&S28204[12], 14}, {NULL}};
const Segment SGI341973[] = {{&S28205[6], 4}, {NULL}};
const Segment SGI341974[] = {{&S28205[80], 5}, {NULL}};
const Segment SGI341975[] = {{&S28205[12], 14}, {&S28205[40], 11}, {NULL}};
const Segment SGI341976[] = {{&S28205[12], 14}, {NULL}};
const Segment SGI341977[] = {{&S28206[6], 4}, {NULL}};
const Segment SGI341978[] = {{&S28206[81], 5}, {NULL}};
const Segment SGI341979[] = {{&S28206[12], 14}, {&S28206[41], 11}, {NULL}};
const Segment SGI341980[] = {{&S28206[12], 14}, {NULL}};
const Segment SGI341981[] = {{&S45829[28], 3}, {NULL}};
const Segment SGI341982[] = {{&S28207[6], 4}, {NULL}};
const Segment SGI341983[] = {{&S28207[80], 5}, {NULL}};
const Segment SGI341984[] = {{&S28207[12], 14}, {&S28207[40], 11}, {NULL}};
const Segment SGI341985[] = {{&S28207[12], 14}, {NULL}};
const Segment SGI341986[] = {{&S45829[102], 5}, {NULL}};
const Segment SGI341987[] = {{&S45830[28], 3}, {NULL}};
const Segment SGI341988[] = {{&S28208[6], 4}, {NULL}};
const Segment SGI341989[] = {{&S28208[80], 5}, {NULL}};
const Segment SGI341990[] = {{&S28208[12], 14}, {&S28208[40], 11}, {NULL}};
const Segment SGI341991[] = {{&S45830[102], 5}, {NULL}};
const Segment SGI341992[] = {{&S28208[12], 14}, {NULL}};
const Segment SGI341993[] = {{&S27250[0], 26}, {NULL}};
const Segment SGI341994[] = {{&S27250[39], 3}, {NULL}};
const Segment SGI341995[] = {{&S27250[78], 13}, {NULL}};
const Segment SGI341996[] = {{&S41029[42], 3}, {&S41029[145], 7}, {NULL}};
const Segment SGI341997[] = {{&S41029[33], 3}, {NULL}};
const Segment SGI341998[] = {{&S27251[0], 26}, {NULL}};
const Segment SGI341999[] = {{&S27251[39], 3}, {NULL}};
const Segment SGI342000[] = {{&S27251[80], 13}, {NULL}};
const Segment SGI342001[] = {{&S41030[42], 3}, {&S41030[145], 7}, {NULL}};
const Segment SGI342002[] = {{&S41030[33], 3}, {NULL}};
const Segment SGI342003[] = {{&S41031[42], 12}, {&S41031[132], 7}, {NULL}};
const Segment SGI342004[] = {{&S41031[33], 3}, {NULL}};
const Segment SGI342005[] = {{&S41032[42], 3}, {&S41032[145], 7}, {NULL}};
const Segment SGI342006[] = {{&S41032[33], 3}, {NULL}};
const Segment SGI342007[] = {{&S54814[5], 4}, {NULL}};
const Segment SGI342008[] = {{&S54815[5], 4}, {&S54815[5], 4}, {NULL}};
const Segment SGI342009[] = {{&S45957[28], 5}, {NULL}};
const Segment SGI342010[] = {{&S45957[104], 5}, {NULL}};
const Segment SGI342011[] = {{&S30124[61], 5}, {NULL}};
const Segment SGI342012[] = {{&S30124[13], 10}, {NULL}};
const Segment SGI342013[] = {{&S54988[5], 3}, {&S54988[5], 3}, {NULL}};
const Segment SGI342014[] = {{&S54988[9], 3}, {NULL}};
const Segment SGI342015[] = {{&S54988[127], 3}, {NULL}};
const Segment SGI342016[] = {{&S28556[6], 3}, {NULL}};
const Segment SGI342017[] = {{&S28556[12], 14}, {NULL}};
const Segment SGI342018[] = {{&S28558[6], 4}, {NULL}};
const Segment SGI342019[] = {{&S28558[12], 14}, {NULL}};
const Segment SGI342020[] = {{&S28559[6], 4}, {NULL}};
const Segment SGI342021[] = {{&S28559[12], 14}, {NULL}};
const Segment SGI342022[] = {{&S28560[6], 4}, {NULL}};
const Segment SGI342023[] = {{&S28560[12], 14}, {NULL}};
const Segment SGI342024[] = {{&S28561[6], 4}, {NULL}};
const Segment SGI342025[] = {{&S28561[12], 14}, {NULL}};
const Segment SGI342026[] = {{&S28562[6], 3}, {NULL}};
const Segment SGI342027[] = {{&S28562[11], 14}, {NULL}};
const Segment SGI342028[] = {{&S28563[6], 4}, {NULL}};
const Segment SGI342029[] = {{&S28563[12], 14}, {NULL}};
const Segment SGI342030[] = {{&S28565[6], 4}, {NULL}};
const Segment SGI342031[] = {{&S28565[12], 14}, {&S28565[40], 12}, {NULL}};
const Segment SGI342032[] = {{&S28565[12], 14}, {NULL}};
const Segment SGI342033[] = {{&S28566[6], 4}, {NULL}};
const Segment SGI342034[] = {{&S28566[12], 14}, {&S28566[40], 12}, {NULL}};
const Segment SGI342035[] = {{&S28566[12], 14}, {NULL}};
const Segment SGI342036[] = {{&S28567[6], 4}, {NULL}};
const Segment SGI342037[] = {{&S28567[12], 14}, {&S28567[40], 12}, {NULL}};
const Segment SGI342038[] = {{&S28567[12], 14}, {NULL}};
const Segment SGI342039[] = {{&S28568[6], 4}, {NULL}};
const Segment SGI342040[] = {{&S28568[12], 14}, {NULL}};
const Segment SGI342041[] = {{&S28571[6], 4}, {NULL}};
const Segment SGI342042[] = {{&S28571[12], 14}, {NULL}};
const Segment SGI342043[] = {{&S28572[6], 4}, {NULL}};
const Segment SGI342044[] = {{&S28572[12], 14}, {&S28572[40], 12}, {NULL}};
const Segment SGI342045[] = {{&S28572[12], 14}, {NULL}};
const Segment SGI342046[] = {{&S28574[6], 4}, {NULL}};
const Segment SGI342047[] = {{&S28574[50], 5}, {NULL}};
const Segment SGI342048[] = {{&S28574[12], 5}, {NULL}};
const Segment SGI342049[] = {{&S28575[6], 4}, {NULL}};
const Segment SGI342050[] = {{&S28575[60], 5}, {NULL}};
const Segment SGI342051[] = {{&S28575[12], 3}, {NULL}};
const Segment SGI342052[] = {{&S55050[5], 4}, {&S55050[5], 4}, {NULL}};
const Segment SGI342053[] = {{&S28576[6], 4}, {NULL}};
const Segment SGI342054[] = {{&S28576[55], 5}, {NULL}};
const Segment SGI342055[] = {{&S28576[12], 3}, {NULL}};
const Segment SGI342056[] = {{&S55051[5], 4}, {NULL}};
const Segment SGI342057[] = {{&S28577[6], 4}, {NULL}};
const Segment SGI342058[] = {{&S28577[55], 5}, {NULL}};
const Segment SGI342059[] = {{&S28577[12], 10}, {NULL}};
const Segment SGI342060[] = {{&S28578[6], 4}, {NULL}};
const Segment SGI342061[] = {{&S28578[83], 5}, {NULL}};
const Segment SGI342062[] = {{&S28578[12], 12}, {NULL}};
const Segment SGI342063[] = {{&S55094[42], 5}, {NULL}};
const Segment SGI342064[] = {{&S55094[6], 3}, {NULL}};
const Segment SGI342065[] = {{&S55094[68], 3}, {NULL}};
const Segment SGI342066[] = {{&S55095[43], 5}, {NULL}};
const Segment SGI342067[] = {{&S55095[6], 3}, {NULL}};
const Segment SGI342068[] = {{&S55095[69], 3}, {NULL}};
const Segment SGI342069[] = {{&S55096[43], 7}, {NULL}};
const Segment SGI342070[] = {{&S55096[6], 3}, {NULL}};
const Segment SGI342071[] = {{&S55096[71], 3}, {NULL}};
const Segment SGI342072[] = {{&S55097[42], 7}, {NULL}};
const Segment SGI342073[] = {{&S55097[6], 3}, {NULL}};
const Segment SGI342074[] = {{&S55097[70], 3}, {NULL}};
const Segment SGI342075[] = {{&S55098[43], 6}, {NULL}};
const Segment SGI342076[] = {{&S55098[6], 3}, {NULL}};
const Segment SGI342077[] = {{&S55098[70], 3}, {NULL}};
const Segment SGI342078[] = {{&S55121[5], 3}, {&S55121[5], 3}, {NULL}};
const Segment SGI342079[] = {{&S55121[9], 3}, {NULL}};
const Segment SGI342080[] = {{&S55121[127], 3}, {NULL}};
const Segment SGI342081[] = {{&S55214[5], 5}, {&S55214[104], 5}, {NULL}};
const Segment SGI342082[] = {{&S55214[69], 3}, {NULL}};
const Segment SGI342083[] = {{&S55214[130], 3}, {NULL}};
const Segment SGI342084[] = {{&S55295[5], 3}, {NULL}};
const Segment SGI342085[] = {{&S30536[61], 5}, {NULL}};
const Segment SGI342086[] = {{&S30536[13], 10}, {NULL}};
const Segment SGI342087[] = {{&S55310[42], 3}, {NULL}};
const Segment SGI342088[] = {{&S55310[6], 3}, {NULL}};
const Segment SGI342089[] = {{&S55310[66], 3}, {NULL}};
const Segment SGI342090[] = {{&S55311[42], 4}, {NULL}};
const Segment SGI342091[] = {{&S55311[6], 3}, {NULL}};
const Segment SGI342092[] = {{&S55311[67], 3}, {NULL}};
const Segment SGI342093[] = {{&S55312[41], 5}, {NULL}};
const Segment SGI342094[] = {{&S55312[6], 3}, {NULL}};
const Segment SGI342095[] = {{&S55312[67], 3}, {NULL}};
const Segment SGI342096[] = {{&S55313[5], 3}, {NULL}};
const Segment SGI342097[] = {{&S55313[14], 3}, {NULL}};
const Segment SGI342098[] = {{&S41460[42], 5}, {&S41460[147], 7}, {NULL}};
const Segment SGI342099[] = {{&S41460[33], 3}, {NULL}};
const Segment SGI342100[] = {{&S41461[42], 6}, {&S41461[148], 7}, {NULL}};
const Segment SGI342101[] = {{&S41461[33], 3}, {NULL}};
const Segment SGI342102[] = {{&S55323[5], 4}, {NULL}};
const Segment SGI342103[] = {{&S55324[5], 4}, {&S55324[5], 4}, {NULL}};
const Segment SGI342104[] = {{&S30588[8], 4}, {NULL}};
const Segment SGI342105[] = {{&S30588[14], 3}, {NULL}};
const Segment SGI342106[] = {{&S30588[45], 3}, {NULL}};
const Segment SGI342107[] = {{&S30588[61], 5}, {NULL}};
const Segment SGI342108[] = {{&S28020[0], 24}, {NULL}};
const Segment SGI342109[] = {{&S28020[37], 3}, {NULL}};
const Segment SGI342110[] = {{&S28020[78], 13}, {NULL}};
const Segment SGI342111[] = {{&S28943[6], 4}, {NULL}};
const Segment SGI342112[] = {{&S28943[80], 5}, {NULL}};
const Segment SGI342113[] = {{&S28943[12], 14}, {&S28943[40], 11}, {NULL}};
const Segment SGI342114[] = {{&S28943[12], 14}, {NULL}};
const Segment SGI342115[] = {{&S28944[6], 4}, {NULL}};
const Segment SGI342116[] = {{&S28944[80], 5}, {NULL}};
const Segment SGI342117[] = {{&S28944[12], 14}, {&S28944[40], 11}, {NULL}};
const Segment SGI342118[] = {{&S28944[12], 14}, {NULL}};
const Segment SGI342119[] = {{&S28945[6], 4}, {NULL}};
const Segment SGI342120[] = {{&S28945[80], 5}, {NULL}};
const Segment SGI342121[] = {{&S28945[12], 14}, {&S28945[40], 11}, {NULL}};
const Segment SGI342122[] = {{&S28945[12], 14}, {NULL}};
const Segment SGI342123[] = {{&S28946[6], 4}, {NULL}};
const Segment SGI342124[] = {{&S28946[81], 5}, {NULL}};
const Segment SGI342125[] = {{&S28946[12], 14}, {&S28946[40], 12}, {NULL}};
const Segment SGI342126[] = {{&S28946[12], 14}, {NULL}};
const Segment SGI342127[] = {{&S28947[6], 4}, {NULL}};
const Segment SGI342128[] = {{&S28947[80], 5}, {NULL}};
const Segment SGI342129[] = {{&S28947[12], 14}, {&S28947[40], 11}, {NULL}};
const Segment SGI342130[] = {{&S28947[12], 14}, {NULL}};
const Segment SGI342131[] = {{&S28948[6], 4}, {NULL}};
const Segment SGI342132[] = {{&S28948[80], 5}, {NULL}};
const Segment SGI342133[] = {{&S28948[12], 14}, {&S28948[40], 11}, {NULL}};
const Segment SGI342134[] = {{&S28948[12], 14}, {NULL}};
const Segment SGI342135[] = {{&S28949[6], 4}, {NULL}};
const Segment SGI342136[] = {{&S28949[81], 5}, {NULL}};
const Segment SGI342137[] = {{&S28949[12], 14}, {&S28949[40], 12}, {NULL}};
const Segment SGI342138[] = {{&S28949[12], 14}, {NULL}};
const Segment SGI342139[] = {{&S28950[6], 4}, {NULL}};
const Segment SGI342140[] = {{&S28950[81], 5}, {NULL}};
const Segment SGI342141[] = {{&S28950[12], 14}, {&S28950[40], 12}, {NULL}};
const Segment SGI342142[] = {{&S28950[12], 14}, {NULL}};
const Segment SGI342143[] = {{&S28951[6], 4}, {NULL}};
const Segment SGI342144[] = {{&S28951[80], 5}, {NULL}};
const Segment SGI342145[] = {{&S28951[12], 14}, {&S28951[40], 11}, {NULL}};
const Segment SGI342146[] = {{&S28951[12], 14}, {NULL}};
const Segment SGI342147[] = {{&S28952[6], 4}, {NULL}};
const Segment SGI342148[] = {{&S28952[80], 5}, {NULL}};
const Segment SGI342149[] = {{&S28952[12], 14}, {&S28952[40], 11}, {NULL}};
const Segment SGI342150[] = {{&S28952[12], 14}, {NULL}};
const Segment SGI342151[] = {{&S28953[6], 4}, {NULL}};
const Segment SGI342152[] = {{&S28953[80], 5}, {NULL}};
const Segment SGI342153[] = {{&S28953[12], 14}, {&S28953[40], 11}, {NULL}};
const Segment SGI342154[] = {{&S28953[12], 14}, {NULL}};
const Segment SGI342155[] = {{&S28954[6], 4}, {NULL}};
const Segment SGI342156[] = {{&S28954[80], 5}, {NULL}};
const Segment SGI342157[] = {{&S28954[12], 14}, {&S28954[40], 11}, {NULL}};
const Segment SGI342158[] = {{&S28954[12], 14}, {NULL}};
const Segment SGI342159[] = {{&S28955[6], 4}, {NULL}};
const Segment SGI342160[] = {{&S28955[81], 5}, {NULL}};
const Segment SGI342161[] = {{&S28955[12], 14}, {&S28955[40], 12}, {NULL}};
const Segment SGI342162[] = {{&S28955[12], 14}, {NULL}};
const Segment SGI342163[] = {{&S28956[6], 4}, {NULL}};
const Segment SGI342164[] = {{&S28956[80], 5}, {NULL}};
const Segment SGI342165[] = {{&S28956[12], 14}, {&S28956[40], 11}, {NULL}};
const Segment SGI342166[] = {{&S28956[12], 14}, {NULL}};
const Segment SGI342167[] = {{&S28957[6], 4}, {NULL}};
const Segment SGI342168[] = {{&S28957[81], 5}, {NULL}};
const Segment SGI342169[] = {{&S28957[12], 14}, {&S28957[40], 12}, {NULL}};
const Segment SGI342170[] = {{&S28957[12], 14}, {NULL}};
const Segment SGI342171[] = {{&S28958[6], 4}, {NULL}};
const Segment SGI342172[] = {{&S28958[81], 5}, {NULL}};
const Segment SGI342173[] = {{&S28958[12], 14}, {&S28958[40], 12}, {NULL}};
const Segment SGI342174[] = {{&S28958[12], 14}, {NULL}};
const Segment SGI342175[] = {{&S28959[6], 4}, {NULL}};
const Segment SGI342176[] = {{&S28959[80], 5}, {NULL}};
const Segment SGI342177[] = {{&S28959[12], 14}, {&S28959[40], 11}, {NULL}};
const Segment SGI342178[] = {{&S28959[12], 14}, {NULL}};
const Segment SGI342179[] = {{&S28960[6], 4}, {NULL}};
const Segment SGI342180[] = {{&S28960[80], 5}, {NULL}};
const Segment SGI342181[] = {{&S28960[12], 14}, {&S28960[40], 11}, {NULL}};
const Segment SGI342182[] = {{&S28960[12], 14}, {NULL}};
const Segment SGI342183[] = {{&S28961[6], 4}, {NULL}};
const Segment SGI342184[] = {{&S28961[80], 5}, {NULL}};
const Segment SGI342185[] = {{&S28961[12], 14}, {&S28961[40], 11}, {NULL}};
const Segment SGI342186[] = {{&S28961[12], 14}, {NULL}};
const Segment SGI342187[] = {{&S41734[5], 3}, {NULL}};
const Segment SGI342188[] = {{&S41734[49], 3}, {NULL}};
const Segment SGI342189[] = {{&S28962[6], 4}, {NULL}};
const Segment SGI342190[] = {{&S28962[80], 5}, {NULL}};
const Segment SGI342191[] = {{&S28962[12], 14}, {&S28962[40], 11}, {NULL}};
const Segment SGI342192[] = {{&S28962[12], 14}, {NULL}};
const Segment SGI342193[] = {{&S41735[49], 5}, {NULL}};
const Segment SGI342194[] = {{&S41735[40], 3}, {NULL}};
const Segment SGI342195[] = {{&S28963[6], 4}, {NULL}};
const Segment SGI342196[] = {{&S28963[81], 5}, {NULL}};
const Segment SGI342197[] = {{&S28963[12], 14}, {&S28963[40], 12}, {NULL}};
const Segment SGI342198[] = {{&S28963[12], 14}, {NULL}};
const Segment SGI342199[] = {{&S28964[6], 4}, {NULL}};
const Segment SGI342200[] = {{&S28964[81], 5}, {NULL}};
const Segment SGI342201[] = {{&S28964[12], 14}, {&S28964[92], 31}, {NULL}};
const Segment SGI342202[] = {{&S28964[12], 14}, {NULL}};
const Segment SGI342203[] = {{&S28965[6], 4}, {NULL}};
const Segment SGI342204[] = {{&S28965[81], 5}, {NULL}};
const Segment SGI342205[] = {{&S28965[12], 14}, {&S28965[40], 12}, {NULL}};
const Segment SGI342206[] = {{&S28965[12], 14}, {NULL}};
const Segment SGI342207[] = {{&S28966[6], 4}, {NULL}};
const Segment SGI342208[] = {{&S28966[81], 5}, {NULL}};
const Segment SGI342209[] = {{&S28966[12], 14}, {&S28966[41], 11}, {NULL}};
const Segment SGI342210[] = {{&S28966[12], 14}, {NULL}};
const Segment SGI342211[] = {{&S28967[6], 4}, {NULL}};
const Segment SGI342212[] = {{&S28967[58], 5}, {NULL}};
const Segment SGI342213[] = {{&S28967[12], 14}, {NULL}};
const Segment SGI342214[] = {{&S28968[6], 4}, {NULL}};
const Segment SGI342215[] = {{&S28968[81], 5}, {NULL}};
const Segment SGI342216[] = {{&S28968[12], 14}, {&S28968[41], 11}, {NULL}};
const Segment SGI342217[] = {{&S28968[12], 14}, {NULL}};
const Segment SGI342218[] = {{&S28969[6], 4}, {NULL}};
const Segment SGI342219[] = {{&S28969[80], 5}, {NULL}};
const Segment SGI342220[] = {{&S28969[12], 14}, {&S28969[40], 11}, {NULL}};
const Segment SGI342221[] = {{&S28969[12], 14}, {NULL}};
const Segment SGI342222[] = {{&S28970[6], 4}, {NULL}};
const Segment SGI342223[] = {{&S28970[81], 5}, {NULL}};
const Segment SGI342224[] = {{&S28970[12], 14}, {&S28970[40], 12}, {NULL}};
const Segment SGI342225[] = {{&S28970[12], 14}, {NULL}};
const Segment SGI342226[] = {{&S28971[6], 4}, {NULL}};
const Segment SGI342227[] = {{&S28971[80], 5}, {NULL}};
const Segment SGI342228[] = {{&S28971[12], 14}, {&S28971[40], 11}, {NULL}};
const Segment SGI342229[] = {{&S28971[12], 14}, {NULL}};
const Segment SGI342230[] = {{&S28972[6], 4}, {NULL}};
const Segment SGI342231[] = {{&S28972[81], 5}, {NULL}};
const Segment SGI342232[] = {{&S28972[12], 14}, {&S28972[40], 12}, {NULL}};
const Segment SGI342233[] = {{&S28972[12], 14}, {NULL}};
const Segment SGI342234[] = {{&S28973[6], 4}, {NULL}};
const Segment SGI342235[] = {{&S28973[81], 5}, {NULL}};
const Segment SGI342236[] = {{&S28973[12], 14}, {&S28973[40], 12}, {NULL}};
const Segment SGI342237[] = {{&S28973[12], 14}, {NULL}};
const Segment SGI342238[] = {{&S28974[6], 4}, {NULL}};
const Segment SGI342239[] = {{&S28974[81], 5}, {NULL}};
const Segment SGI342240[] = {{&S28974[12], 14}, {&S28974[41], 11}, {NULL}};
const Segment SGI342241[] = {{&S28974[12], 14}, {NULL}};
const Segment SGI342242[] = {{&S28976[6], 4}, {NULL}};
const Segment SGI342243[] = {{&S28976[81], 5}, {NULL}};
const Segment SGI342244[] = {{&S28976[12], 14}, {&S28976[40], 12}, {NULL}};
const Segment SGI342245[] = {{&S28976[12], 14}, {NULL}};
const Segment SGI342246[] = {{&S28977[6], 4}, {NULL}};
const Segment SGI342247[] = {{&S28977[81], 5}, {NULL}};
const Segment SGI342248[] = {{&S28977[12], 14}, {&S28977[41], 11}, {NULL}};
const Segment SGI342249[] = {{&S28977[12], 14}, {NULL}};
const Segment SGI342250[] = {{&S28978[6], 4}, {NULL}};
const Segment SGI342251[] = {{&S28978[80], 5}, {NULL}};
const Segment SGI342252[] = {{&S28978[12], 14}, {&S28978[40], 11}, {NULL}};
const Segment SGI342253[] = {{&S28978[12], 14}, {NULL}};
const Segment SGI342254[] = {{&S28979[6], 4}, {NULL}};
const Segment SGI342255[] = {{&S28979[81], 5}, {NULL}};
const Segment SGI342256[] = {{&S28979[12], 14}, {&S28979[40], 12}, {NULL}};
const Segment SGI342257[] = {{&S28979[12], 14}, {NULL}};
const Segment SGI342258[] = {{&S28981[6], 4}, {NULL}};
const Segment SGI342259[] = {{&S28981[81], 5}, {NULL}};
const Segment SGI342260[] = {{&S28981[12], 14}, {&S28981[41], 11}, {NULL}};
const Segment SGI342261[] = {{&S28981[12], 14}, {NULL}};
const Segment SGI342262[] = {{&S28982[6], 4}, {NULL}};
const Segment SGI342263[] = {{&S28982[81], 5}, {NULL}};
const Segment SGI342264[] = {{&S28982[12], 14}, {&S28982[41], 11}, {NULL}};
const Segment SGI342265[] = {{&S28982[12], 14}, {NULL}};
const Segment SGI342266[] = {{&S28983[6], 4}, {NULL}};
const Segment SGI342267[] = {{&S28983[80], 5}, {NULL}};
const Segment SGI342268[] = {{&S28983[12], 14}, {&S28983[40], 11}, {NULL}};
const Segment SGI342269[] = {{&S28983[12], 14}, {NULL}};
const Segment SGI342270[] = {{&S28984[6], 4}, {NULL}};
const Segment SGI342271[] = {{&S28984[81], 5}, {NULL}};
const Segment SGI342272[] = {{&S28984[12], 14}, {&S28984[41], 11}, {NULL}};
const Segment SGI342273[] = {{&S28984[12], 14}, {NULL}};
const Segment SGI342274[] = {{&S28985[6], 4}, {NULL}};
const Segment SGI342275[] = {{&S28985[80], 5}, {NULL}};
const Segment SGI342276[] = {{&S28985[12], 14}, {&S28985[40], 11}, {NULL}};
const Segment SGI342277[] = {{&S28985[12], 14}, {NULL}};
const Segment SGI342278[] = {{&S28986[6], 4}, {NULL}};
const Segment SGI342279[] = {{&S28986[80], 5}, {NULL}};
const Segment SGI342280[] = {{&S28986[12], 14}, {&S28986[40], 11}, {NULL}};
const Segment SGI342281[] = {{&S28986[12], 14}, {NULL}};
const Segment SGI342282[] = {{&S28987[6], 4}, {NULL}};
const Segment SGI342283[] = {{&S28987[80], 5}, {NULL}};
const Segment SGI342284[] = {{&S28987[12], 14}, {&S28987[40], 11}, {NULL}};
const Segment SGI342285[] = {{&S28987[12], 14}, {NULL}};
const Segment SGI342286[] = {{&S28988[6], 4}, {NULL}};
const Segment SGI342287[] = {{&S28988[80], 5}, {NULL}};
const Segment SGI342288[] = {{&S28988[12], 14}, {&S28988[40], 11}, {NULL}};
const Segment SGI342289[] = {{&S28988[12], 14}, {NULL}};
const Segment SGI342290[] = {{&S28989[6], 4}, {NULL}};
const Segment SGI342291[] = {{&S28989[81], 5}, {NULL}};
const Segment SGI342292[] = {{&S28989[12], 14}, {&S28989[40], 12}, {NULL}};
const Segment SGI342293[] = {{&S28989[12], 14}, {NULL}};
const Segment SGI342294[] = {{&S28990[6], 4}, {NULL}};
const Segment SGI342295[] = {{&S28990[80], 5}, {NULL}};
const Segment SGI342296[] = {{&S28990[12], 14}, {&S28990[40], 11}, {NULL}};
const Segment SGI342297[] = {{&S28990[12], 14}, {NULL}};
const Segment SGI342298[] = {{&S28991[6], 4}, {NULL}};
const Segment SGI342299[] = {{&S28991[81], 5}, {NULL}};
const Segment SGI342300[] = {{&S28991[12], 14}, {&S28991[41], 11}, {NULL}};
const Segment SGI342301[] = {{&S28991[12], 14}, {NULL}};
const Segment SGI342302[] = {{&S28992[6], 4}, {NULL}};
const Segment SGI342303[] = {{&S28992[81], 5}, {NULL}};
const Segment SGI342304[] = {{&S28992[12], 14}, {&S28992[41], 11}, {NULL}};
const Segment SGI342305[] = {{&S28992[12], 14}, {NULL}};
const Segment SGI342306[] = {{&S28993[6], 4}, {NULL}};
const Segment SGI342307[] = {{&S28993[80], 5}, {NULL}};
const Segment SGI342308[] = {{&S28993[12], 14}, {&S28993[40], 11}, {NULL}};
const Segment SGI342309[] = {{&S28993[12], 14}, {NULL}};
const Segment SGI342310[] = {{&S28994[6], 4}, {NULL}};
const Segment SGI342311[] = {{&S28994[80], 5}, {NULL}};
const Segment SGI342312[] = {{&S28994[12], 14}, {&S28994[40], 11}, {NULL}};
const Segment SGI342313[] = {{&S28994[12], 14}, {NULL}};
const Segment SGI342314[] = {{&S28995[6], 4}, {NULL}};
const Segment SGI342315[] = {{&S28995[58], 5}, {NULL}};
const Segment SGI342316[] = {{&S28995[12], 14}, {&S28995[69], 4}, {NULL}};
const Segment SGI342317[] = {{&S28995[12], 14}, {NULL}};
const Segment SGI342318[] = {{&S28996[6], 4}, {NULL}};
const Segment SGI342319[] = {{&S28996[80], 5}, {NULL}};
const Segment SGI342320[] = {{&S28996[12], 14}, {&S28996[40], 11}, {NULL}};
const Segment SGI342321[] = {{&S28996[12], 14}, {NULL}};
const Segment SGI342322[] = {{&S28997[6], 4}, {NULL}};
const Segment SGI342323[] = {{&S28997[80], 5}, {NULL}};
const Segment SGI342324[] = {{&S28997[12], 14}, {&S28997[40], 11}, {NULL}};
const Segment SGI342325[] = {{&S28997[12], 14}, {NULL}};
const Segment SGI342326[] = {{&S28998[6], 4}, {NULL}};
const Segment SGI342327[] = {{&S28998[80], 5}, {NULL}};
const Segment SGI342328[] = {{&S28998[12], 14}, {&S28998[40], 11}, {NULL}};
const Segment SGI342329[] = {{&S28998[12], 14}, {NULL}};
const Segment SGI342330[] = {{&S28999[6], 4}, {NULL}};
const Segment SGI342331[] = {{&S28999[80], 5}, {NULL}};
const Segment SGI342332[] = {{&S28999[12], 14}, {&S28999[40], 11}, {NULL}};
const Segment SGI342333[] = {{&S28999[12], 14}, {NULL}};
const Segment SGI342334[] = {{&S29000[6], 4}, {NULL}};
const Segment SGI342335[] = {{&S29000[80], 5}, {NULL}};
const Segment SGI342336[] = {{&S29000[12], 14}, {&S29000[40], 11}, {NULL}};
const Segment SGI342337[] = {{&S29000[12], 14}, {NULL}};
const Segment SGI342338[] = {{&S29001[6], 4}, {NULL}};
const Segment SGI342339[] = {{&S29001[80], 5}, {NULL}};
const Segment SGI342340[] = {{&S29001[12], 14}, {&S29001[40], 11}, {NULL}};
const Segment SGI342341[] = {{&S29001[12], 14}, {NULL}};
const Segment SGI342342[] = {{&S29002[6], 4}, {NULL}};
const Segment SGI342343[] = {{&S29002[81], 5}, {NULL}};
const Segment SGI342344[] = {{&S29002[12], 14}, {&S29002[41], 11}, {NULL}};
const Segment SGI342345[] = {{&S29002[12], 14}, {NULL}};
const Segment SGI342346[] = {{&S29003[6], 4}, {NULL}};
const Segment SGI342347[] = {{&S29003[80], 5}, {NULL}};
const Segment SGI342348[] = {{&S29003[12], 14}, {&S29003[40], 11}, {NULL}};
const Segment SGI342349[] = {{&S29003[12], 14}, {NULL}};
const Segment SGI342350[] = {{&S29004[6], 4}, {NULL}};
const Segment SGI342351[] = {{&S29004[80], 5}, {NULL}};
const Segment SGI342352[] = {{&S29004[12], 14}, {&S29004[40], 11}, {NULL}};
const Segment SGI342353[] = {{&S29004[12], 14}, {NULL}};
const Segment SGI342354[] = {{&S29005[6], 4}, {NULL}};
const Segment SGI342355[] = {{&S29005[80], 5}, {NULL}};
const Segment SGI342356[] = {{&S29005[12], 14}, {&S29005[40], 11}, {NULL}};
const Segment SGI342357[] = {{&S29005[12], 14}, {NULL}};
const Segment SGI342358[] = {{&S29024[6], 4}, {NULL}};
const Segment SGI342359[] = {{&S29024[73], 5}, {NULL}};
const Segment SGI342360[] = {{&S29024[12], 14}, {NULL}};
const Segment SGI342361[] = {{&S46677[28], 3}, {NULL}};
const Segment SGI342362[] = {{&S46677[102], 5}, {NULL}};
const Segment SGI342363[] = {{&S55571[35], 5}, {NULL}};
const Segment SGI342364[] = {{&S55571[91], 3}, {NULL}};
const Segment SGI342365[] = {{&S55572[34], 5}, {NULL}};
const Segment SGI342366[] = {{&S55572[90], 3}, {NULL}};
const Segment SGI342367[] = {{&S55573[35], 7}, {NULL}};
const Segment SGI342368[] = {{&S55573[93], 3}, {NULL}};
const Segment SGI342369[] = {{&S55605[5], 3}, {&S55605[5], 3}, {NULL}};
const Segment SGI342370[] = {{&S55605[149], 3}, {NULL}};
const Segment SGI342371[] = {{&S55608[5], 5}, {NULL}};
const Segment SGI342372[] = {{&S55608[16], 3}, {NULL}};
const Segment SGI342373[] = {{&S46719[28], 3}, {NULL}};
const Segment SGI342374[] = {{&S46719[102], 5}, {NULL}};
const Segment SGI342375[] = {{&S55609[42], 5}, {NULL}};
const Segment SGI342376[] = {{&S55609[6], 3}, {NULL}};
const Segment SGI342377[] = {{&S55609[68], 3}, {NULL}};
const Segment SGI342378[] = {{&S46720[28], 4}, {NULL}};
const Segment SGI342379[] = {{&S46720[103], 5}, {NULL}};
const Segment SGI342380[] = {{&S46721[5], 3}, {&S46721[5], 3}, {NULL}};
const Segment SGI342381[] = {{&S46721[163], 5}, {NULL}};
const Segment SGI342382[] = {{&S46747[28], 3}, {NULL}};
const Segment SGI342383[] = {{&S46747[102], 5}, {NULL}};
const Segment SGI342384[] = {{&S46748[28], 5}, {NULL}};
const Segment SGI342385[] = {{&S46748[104], 5}, {NULL}};
const Segment SGI342386[] = {{&S46749[5], 3}, {&S46749[5], 3}, {NULL}};
const Segment SGI342387[] = {{&S46749[163], 5}, {NULL}};
const Segment SGI342388[] = {{&S55700[5], 3}, {&S55700[5], 3}, {NULL}};
const Segment SGI342389[] = {{&S55700[9], 3}, {NULL}};
const Segment SGI342390[] = {{&S55700[126], 3}, {NULL}};
const Segment SGI342391[] = {{&S55736[35], 5}, {NULL}};
const Segment SGI342392[] = {{&S55736[91], 3}, {NULL}};
const Segment SGI342393[] = {{&S55737[5], 5}, {NULL}};
const Segment SGI342394[] = {{&S55737[16], 3}, {NULL}};
const Segment SGI342395[] = {{&S31023[57], 5}, {NULL}};
const Segment SGI342396[] = {{&S31023[13], 10}, {NULL}};
const Segment SGI342397[] = {{&S55738[5], 14}, {NULL}};
const Segment SGI342398[] = {{&S55738[20], 3}, {NULL}};
const Segment SGI342399[] = {{&S46862[28], 3}, {NULL}};
const Segment SGI342400[] = {{&S46862[102], 5}, {NULL}};
const Segment SGI342401[] = {{&S46863[28], 3}, {NULL}};
const Segment SGI342402[] = {{&S46863[102], 5}, {NULL}};
const Segment SGI342403[] = {{&S46910[28], 3}, {NULL}};
const Segment SGI342404[] = {{&S46910[102], 5}, {NULL}};
const Segment SGI342405[] = {{&S42165[42], 3}, {&S42165[140], 7}, {NULL}};
const Segment SGI342406[] = {{&S42165[33], 3}, {NULL}};
const Segment SGI342407[] = {{&S31188[72], 5}, {NULL}};
const Segment SGI342408[] = {{&S31188[13], 10}, {NULL}};
const Segment SGI342409[] = {{&S55999[35], 3}, {NULL}};
const Segment SGI342410[] = {{&S55999[89], 3}, {NULL}};
const Segment SGI342411[] = {{&S56000[5], 3}, {NULL}};
const Segment SGI342412[] = {{&S56000[14], 3}, {NULL}};
const Segment SGI342413[] = {{&S56001[34], 5}, {NULL}};
const Segment SGI342414[] = {{&S42294[42], 5}, {NULL}};
const Segment SGI342415[] = {{&S42294[33], 3}, {NULL}};
const Segment SGI342416[] = {{&S56001[90], 3}, {NULL}};
const Segment SGI342417[] = {{&S42295[42], 5}, {&S42295[142], 7}, {NULL}};
const Segment SGI342418[] = {{&S42295[33], 3}, {NULL}};
const Segment SGI342419[] = {{&S56009[42], 3}, {NULL}};
const Segment SGI342420[] = {{&S56009[6], 3}, {NULL}};
const Segment SGI342421[] = {{&S56009[66], 3}, {NULL}};
const Segment SGI342422[] = {{&S56074[35], 3}, {NULL}};
const Segment SGI342423[] = {{&S56074[89], 3}, {NULL}};
const Segment SGI342424[] = {{&S56075[35], 7}, {NULL}};
const Segment SGI342425[] = {{&S56075[93], 3}, {NULL}};
const Segment SGI342426[] = {{&S56076[38], 3}, {NULL}};
const Segment SGI342427[] = {{&S56076[6], 3}, {NULL}};
const Segment SGI342428[] = {{&S56076[62], 3}, {NULL}};
const Segment SGI342429[] = {{&S56077[5], 12}, {NULL}};
const Segment SGI342430[] = {{&S56077[18], 3}, {NULL}};
const Segment SGI342431[] = {{&S56078[5], 16}, {NULL}};
const Segment SGI342432[] = {{&S56078[22], 3}, {NULL}};
const Segment SGI342433[] = {{&S56079[41], 7}, {NULL}};
const Segment SGI342434[] = {{&S56079[6], 3}, {NULL}};
const Segment SGI342435[] = {{&S56079[69], 3}, {NULL}};
const Segment SGI342436[] = {{&S56085[5], 3}, {&S56085[5], 3}, {NULL}};
const Segment SGI342437[] = {{&S56085[67], 3}, {NULL}};
const Segment SGI342438[] = {{&S56085[126], 3}, {NULL}};
const Segment SGI342439[] = {{&S29586[6], 4}, {NULL}};
const Segment SGI342440[] = {{&S29586[59], 5}, {NULL}};
const Segment SGI342441[] = {{&S29586[12], 14}, {NULL}};
const Segment SGI342442[] = {{&S29587[6], 4}, {NULL}};
const Segment SGI342443[] = {{&S29587[81], 5}, {NULL}};
const Segment SGI342444[] = {{&S29587[12], 14}, {&S29587[40], 11}, {NULL}};
const Segment SGI342445[] = {{&S29587[12], 14}, {NULL}};
const Segment SGI342446[] = {{&S29588[6], 4}, {NULL}};
const Segment SGI342447[] = {{&S29588[59], 5}, {NULL}};
const Segment SGI342448[] = {{&S29588[12], 14}, {NULL}};
const Segment SGI342449[] = {{&S29589[6], 4}, {NULL}};
const Segment SGI342450[] = {{&S29589[59], 5}, {NULL}};
const Segment SGI342451[] = {{&S29589[12], 14}, {&S29589[70], 31}, {NULL}};
const Segment SGI342452[] = {{&S29589[12], 14}, {NULL}};
const Segment SGI342453[] = {{&S29590[6], 4}, {NULL}};
const Segment SGI342454[] = {{&S29590[81], 5}, {NULL}};
const Segment SGI342455[] = {{&S29590[12], 14}, {&S29590[40], 11}, {NULL}};
const Segment SGI342456[] = {{&S29590[12], 14}, {NULL}};
const Segment SGI342457[] = {{&S29591[6], 4}, {NULL}};
const Segment SGI342458[] = {{&S29591[81], 5}, {NULL}};
const Segment SGI342459[] = {{&S29591[12], 14}, {&S29591[40], 11}, {NULL}};
const Segment SGI342460[] = {{&S29591[12], 14}, {NULL}};
const Segment SGI342461[] = {{&S29592[6], 4}, {NULL}};
const Segment SGI342462[] = {{&S29592[82], 5}, {NULL}};
const Segment SGI342463[] = {{&S29592[12], 14}, {&S29592[40], 12}, {NULL}};
const Segment SGI342464[] = {{&S29592[12], 14}, {NULL}};
const Segment SGI342465[] = {{&S29593[6], 4}, {NULL}};
const Segment SGI342466[] = {{&S29593[82], 5}, {NULL}};
const Segment SGI342467[] = {{&S29593[12], 14}, {&S29593[40], 12}, {NULL}};
const Segment SGI342468[] = {{&S29593[12], 14}, {NULL}};
const Segment SGI342469[] = {{&S29594[6], 4}, {NULL}};
const Segment SGI342470[] = {{&S29594[81], 5}, {NULL}};
const Segment SGI342471[] = {{&S29594[12], 14}, {&S29594[40], 11}, {NULL}};
const Segment SGI342472[] = {{&S29594[12], 14}, {NULL}};
const Segment SGI342473[] = {{&S29595[6], 4}, {NULL}};
const Segment SGI342474[] = {{&S29595[82], 5}, {NULL}};
const Segment SGI342475[] = {{&S29595[12], 14}, {&S29595[40], 12}, {NULL}};
const Segment SGI342476[] = {{&S29595[12], 14}, {NULL}};
const Segment SGI342477[] = {{&S29596[6], 4}, {NULL}};
const Segment SGI342478[] = {{&S29596[82], 5}, {NULL}};
const Segment SGI342479[] = {{&S29596[12], 14}, {&S29596[41], 11}, {NULL}};
const Segment SGI342480[] = {{&S29596[12], 14}, {NULL}};
const Segment SGI342481[] = {{&S29598[6], 4}, {NULL}};
const Segment SGI342482[] = {{&S29598[82], 5}, {NULL}};
const Segment SGI342483[] = {{&S29598[12], 14}, {&S29598[41], 11}, {NULL}};
const Segment SGI342484[] = {{&S29598[12], 14}, {NULL}};
const Segment SGI342485[] = {{&S29599[6], 4}, {NULL}};
const Segment SGI342486[] = {{&S29599[82], 5}, {NULL}};
const Segment SGI342487[] = {{&S29599[12], 14}, {&S29599[41], 11}, {NULL}};
const Segment SGI342488[] = {{&S29599[12], 14}, {NULL}};
const Segment SGI342489[] = {{&S29600[6], 4}, {NULL}};
const Segment SGI342490[] = {{&S29600[82], 5}, {NULL}};
const Segment SGI342491[] = {{&S29600[12], 14}, {&S29600[40], 12}, {NULL}};
const Segment SGI342492[] = {{&S29600[12], 14}, {NULL}};
const Segment SGI342493[] = {{&S29601[6], 4}, {NULL}};
const Segment SGI342494[] = {{&S29601[82], 5}, {NULL}};
const Segment SGI342495[] = {{&S29601[12], 14}, {&S29601[40], 12}, {NULL}};
const Segment SGI342496[] = {{&S29601[12], 14}, {NULL}};
const Segment SGI342497[] = {{&S29602[6], 4}, {NULL}};
const Segment SGI342498[] = {{&S29602[81], 5}, {NULL}};
const Segment SGI342499[] = {{&S29602[12], 14}, {&S29602[40], 11}, {NULL}};
const Segment SGI342500[] = {{&S29602[12], 14}, {NULL}};
const Segment SGI342501[] = {{&S29603[6], 4}, {NULL}};
const Segment SGI342502[] = {{&S29603[82], 5}, {NULL}};
const Segment SGI342503[] = {{&S29603[12], 14}, {&S29603[40], 12}, {NULL}};
const Segment SGI342504[] = {{&S29603[12], 14}, {NULL}};
const Segment SGI342505[] = {{&S29604[6], 4}, {NULL}};
const Segment SGI342506[] = {{&S29604[82], 5}, {NULL}};
const Segment SGI342507[] = {{&S29604[12], 14}, {&S29604[41], 11}, {NULL}};
const Segment SGI342508[] = {{&S29604[12], 14}, {NULL}};
const Segment SGI342509[] = {{&S29606[6], 4}, {NULL}};
const Segment SGI342510[] = {{&S29606[82], 5}, {NULL}};
const Segment SGI342511[] = {{&S29606[12], 14}, {&S29606[41], 11}, {NULL}};
const Segment SGI342512[] = {{&S56116[5], 5}, {&S56116[97], 5}, {NULL}};
const Segment SGI342513[] = {{&S29606[12], 14}, {NULL}};
const Segment SGI342514[] = {{&S56116[153], 3}, {NULL}};
const Segment SGI342515[] = {{&S29607[6], 4}, {NULL}};
const Segment SGI342516[] = {{&S29607[82], 5}, {NULL}};
const Segment SGI342517[] = {{&S29607[12], 14}, {&S29607[41], 11}, {NULL}};
const Segment SGI342518[] = {{&S29607[12], 14}, {NULL}};
const Segment SGI342519[] = {{&S29608[6], 4}, {NULL}};
const Segment SGI342520[] = {{&S29608[82], 5}, {NULL}};
const Segment SGI342521[] = {{&S29608[12], 14}, {&S29608[41], 11}, {NULL}};
const Segment SGI342522[] = {{&S29608[12], 14}, {NULL}};
const Segment SGI342523[] = {{&S29609[6], 4}, {NULL}};
const Segment SGI342524[] = {{&S29609[82], 5}, {NULL}};
const Segment SGI342525[] = {{&S29609[12], 14}, {&S29609[41], 11}, {NULL}};
const Segment SGI342526[] = {{&S29609[12], 14}, {NULL}};
const Segment SGI342527[] = {{&S29610[6], 4}, {NULL}};
const Segment SGI342528[] = {{&S29610[82], 5}, {NULL}};
const Segment SGI342529[] = {{&S29610[12], 14}, {&S29610[40], 12}, {NULL}};
const Segment SGI342530[] = {{&S29610[12], 14}, {NULL}};
const Segment SGI342531[] = {{&S29611[6], 4}, {NULL}};
const Segment SGI342532[] = {{&S29611[81], 5}, {NULL}};
const Segment SGI342533[] = {{&S29611[12], 14}, {&S29611[40], 11}, {NULL}};
const Segment SGI342534[] = {{&S29611[12], 14}, {NULL}};
const Segment SGI342535[] = {{&S29612[6], 4}, {NULL}};
const Segment SGI342536[] = {{&S29612[82], 5}, {NULL}};
const Segment SGI342537[] = {{&S29612[12], 14}, {&S29612[41], 11}, {NULL}};
const Segment SGI342538[] = {{&S29612[12], 14}, {NULL}};
const Segment SGI342539[] = {{&S29613[6], 4}, {NULL}};
const Segment SGI342540[] = {{&S29613[82], 5}, {NULL}};
const Segment SGI342541[] = {{&S29613[12], 14}, {&S29613[41], 11}, {NULL}};
const Segment SGI342542[] = {{&S29613[12], 14}, {NULL}};
const Segment SGI342543[] = {{&S29614[6], 4}, {NULL}};
const Segment SGI342544[] = {{&S29614[82], 5}, {NULL}};
const Segment SGI342545[] = {{&S29614[12], 14}, {&S29614[40], 12}, {NULL}};
const Segment SGI342546[] = {{&S29614[12], 14}, {NULL}};
const Segment SGI342547[] = {{&S29615[6], 4}, {NULL}};
const Segment SGI342548[] = {{&S29615[82], 5}, {NULL}};
const Segment SGI342549[] = {{&S29615[12], 14}, {&S29615[40], 12}, {NULL}};
const Segment SGI342550[] = {{&S29615[12], 14}, {NULL}};
const Segment SGI342551[] = {{&S29616[6], 4}, {NULL}};
const Segment SGI342552[] = {{&S29616[82], 5}, {NULL}};
const Segment SGI342553[] = {{&S29616[12], 14}, {&S29616[41], 11}, {NULL}};
const Segment SGI342554[] = {{&S29616[12], 14}, {NULL}};
const Segment SGI342555[] = {{&S56128[5], 5}, {&S56128[97], 5}, {NULL}};
const Segment SGI342556[] = {{&S56128[153], 3}, {NULL}};
const Segment SGI342557[] = {{&S29617[6], 4}, {NULL}};
const Segment SGI342558[] = {{&S29617[81], 5}, {NULL}};
const Segment SGI342559[] = {{&S29617[12], 14}, {&S29617[40], 11}, {NULL}};
const Segment SGI342560[] = {{&S29617[12], 14}, {NULL}};
const Segment SGI342561[] = {{&S29618[6], 4}, {NULL}};
const Segment SGI342562[] = {{&S29618[81], 5}, {NULL}};
const Segment SGI342563[] = {{&S29618[12], 14}, {&S29618[40], 11}, {NULL}};
const Segment SGI342564[] = {{&S29618[12], 14}, {NULL}};
const Segment SGI342565[] = {{&S47199[28], 3}, {NULL}};
const Segment SGI342566[] = {{&S29619[6], 4}, {NULL}};
const Segment SGI342567[] = {{&S29619[82], 5}, {NULL}};
const Segment SGI342568[] = {{&S47199[102], 5}, {NULL}};
const Segment SGI342569[] = {{&S29619[12], 14}, {&S29619[40], 12}, {NULL}};
const Segment SGI342570[] = {{&S29619[12], 14}, {NULL}};
const Segment SGI342571[] = {{&S29620[6], 4}, {NULL}};
const Segment SGI342572[] = {{&S29620[81], 5}, {NULL}};
const Segment SGI342573[] = {{&S29620[12], 14}, {&S29620[40], 11}, {NULL}};
const Segment SGI342574[] = {{&S29620[12], 14}, {NULL}};
const Segment SGI342575[] = {{&S29621[6], 4}, {NULL}};
const Segment SGI342576[] = {{&S29621[81], 5}, {NULL}};
const Segment SGI342577[] = {{&S29621[12], 14}, {&S29621[40], 11}, {NULL}};
const Segment SGI342578[] = {{&S29621[12], 14}, {NULL}};
const Segment SGI342579[] = {{&S29622[6], 4}, {NULL}};
const Segment SGI342580[] = {{&S29622[81], 5}, {NULL}};
const Segment SGI342581[] = {{&S29622[12], 14}, {&S29622[40], 11}, {NULL}};
const Segment SGI342582[] = {{&S29622[12], 14}, {NULL}};
const Segment SGI342583[] = {{&S29623[6], 4}, {NULL}};
const Segment SGI342584[] = {{&S29623[82], 5}, {NULL}};
const Segment SGI342585[] = {{&S29623[12], 14}, {&S29623[40], 12}, {NULL}};
const Segment SGI342586[] = {{&S29623[12], 14}, {NULL}};
const Segment SGI342587[] = {{&S29624[6], 4}, {NULL}};
const Segment SGI342588[] = {{&S29624[81], 5}, {NULL}};
const Segment SGI342589[] = {{&S29624[12], 14}, {&S29624[40], 11}, {NULL}};
const Segment SGI342590[] = {{&S29624[12], 14}, {NULL}};
const Segment SGI342591[] = {{&S29625[6], 4}, {NULL}};
const Segment SGI342592[] = {{&S29625[81], 5}, {NULL}};
const Segment SGI342593[] = {{&S29625[12], 14}, {&S29625[40], 11}, {NULL}};
const Segment SGI342594[] = {{&S29625[12], 14}, {NULL}};
const Segment SGI342595[] = {{&S29626[6], 4}, {NULL}};
const Segment SGI342596[] = {{&S29626[82], 5}, {NULL}};
const Segment SGI342597[] = {{&S29626[12], 14}, {&S29626[40], 12}, {NULL}};
const Segment SGI342598[] = {{&S29626[12], 14}, {NULL}};
const Segment SGI342599[] = {{&S29627[6], 4}, {NULL}};
const Segment SGI342600[] = {{&S29627[81], 5}, {NULL}};
const Segment SGI342601[] = {{&S29627[12], 14}, {&S29627[40], 11}, {NULL}};
const Segment SGI342602[] = {{&S29627[12], 14}, {NULL}};
const Segment SGI342603[] = {{&S29628[6], 4}, {NULL}};
const Segment SGI342604[] = {{&S29628[81], 5}, {NULL}};
const Segment SGI342605[] = {{&S29628[12], 14}, {&S29628[40], 11}, {NULL}};
const Segment SGI342606[] = {{&S29628[12], 14}, {NULL}};
const Segment SGI342607[] = {{&S29629[6], 4}, {NULL}};
const Segment SGI342608[] = {{&S29629[81], 5}, {NULL}};
const Segment SGI342609[] = {{&S29629[12], 14}, {&S29629[40], 11}, {NULL}};
const Segment SGI342610[] = {{&S29629[12], 14}, {NULL}};
const Segment SGI342611[] = {{&S29630[6], 4}, {NULL}};
const Segment SGI342612[] = {{&S29630[82], 5}, {NULL}};
const Segment SGI342613[] = {{&S29630[12], 14}, {&S29630[40], 12}, {NULL}};
const Segment SGI342614[] = {{&S29630[12], 14}, {NULL}};
const Segment SGI342615[] = {{&S29631[6], 4}, {NULL}};
const Segment SGI342616[] = {{&S29631[81], 5}, {NULL}};
const Segment SGI342617[] = {{&S29631[12], 14}, {&S29631[40], 11}, {NULL}};
const Segment SGI342618[] = {{&S29631[12], 14}, {NULL}};
const Segment SGI342619[] = {{&S29632[6], 4}, {NULL}};
const Segment SGI342620[] = {{&S29632[82], 5}, {NULL}};
const Segment SGI342621[] = {{&S29632[12], 14}, {&S29632[40], 12}, {NULL}};
const Segment SGI342622[] = {{&S29632[12], 14}, {NULL}};
const Segment SGI342623[] = {{&S29633[6], 4}, {NULL}};
const Segment SGI342624[] = {{&S29633[82], 5}, {NULL}};
const Segment SGI342625[] = {{&S29633[12], 14}, {&S29633[40], 12}, {NULL}};
const Segment SGI342626[] = {{&S29633[12], 14}, {NULL}};
const Segment SGI342627[] = {{&S29634[6], 4}, {NULL}};
const Segment SGI342628[] = {{&S29634[82], 5}, {NULL}};
const Segment SGI342629[] = {{&S29634[12], 14}, {&S29634[40], 12}, {NULL}};
const Segment SGI342630[] = {{&S29634[12], 14}, {NULL}};
const Segment SGI342631[] = {{&S31309[61], 5}, {NULL}};
const Segment SGI342632[] = {{&S31309[13], 10}, {NULL}};
const Segment SGI342633[] = {{&S29635[6], 4}, {NULL}};
const Segment SGI342634[] = {{&S29635[81], 5}, {NULL}};
const Segment SGI342635[] = {{&S29635[12], 14}, {&S29635[40], 11}, {NULL}};
const Segment SGI342636[] = {{&S29635[12], 14}, {NULL}};
const Segment SGI342637[] = {{&S29636[6], 4}, {NULL}};
const Segment SGI342638[] = {{&S29636[82], 5}, {NULL}};
const Segment SGI342639[] = {{&S29636[12], 14}, {&S29636[41], 11}, {NULL}};
const Segment SGI342640[] = {{&S29636[12], 14}, {NULL}};
const Segment SGI342641[] = {{&S29637[6], 4}, {NULL}};
const Segment SGI342642[] = {{&S29637[82], 5}, {NULL}};
const Segment SGI342643[] = {{&S29637[12], 14}, {&S29637[41], 11}, {NULL}};
const Segment SGI342644[] = {{&S29637[12], 14}, {NULL}};
const Segment SGI342645[] = {{&S29638[6], 4}, {NULL}};
const Segment SGI342646[] = {{&S29638[82], 5}, {NULL}};
const Segment SGI342647[] = {{&S29638[12], 14}, {&S29638[41], 11}, {NULL}};
const Segment SGI342648[] = {{&S29638[12], 14}, {NULL}};
const Segment SGI342649[] = {{&S29639[6], 4}, {NULL}};
const Segment SGI342650[] = {{&S29639[81], 5}, {NULL}};
const Segment SGI342651[] = {{&S29639[12], 14}, {&S29639[40], 11}, {NULL}};
const Segment SGI342652[] = {{&S29639[12], 14}, {NULL}};
const Segment SGI342653[] = {{&S29640[6], 4}, {NULL}};
const Segment SGI342654[] = {{&S29640[82], 5}, {NULL}};
const Segment SGI342655[] = {{&S29640[12], 14}, {&S29640[40], 12}, {NULL}};
const Segment SGI342656[] = {{&S29640[12], 14}, {NULL}};
const Segment SGI342657[] = {{&S29641[6], 4}, {NULL}};
const Segment SGI342658[] = {{&S29641[82], 5}, {NULL}};
const Segment SGI342659[] = {{&S29641[12], 14}, {&S29641[40], 12}, {NULL}};
const Segment SGI342660[] = {{&S29641[12], 14}, {NULL}};
const Segment SGI342661[] = {{&S29642[6], 4}, {NULL}};
const Segment SGI342662[] = {{&S29642[109], 5}, {NULL}};
const Segment SGI342663[] = {{&S29642[12], 14}, {&S29642[40], 12}, {NULL}};
const Segment SGI342664[] = {{&S29642[12], 14}, {NULL}};
const Segment SGI342665[] = {{&S29643[6], 4}, {NULL}};
const Segment SGI342666[] = {{&S29643[82], 5}, {NULL}};
const Segment SGI342667[] = {{&S29643[12], 14}, {&S29643[40], 12}, {NULL}};
const Segment SGI342668[] = {{&S29643[12], 14}, {NULL}};
const Segment SGI342669[] = {{&S29644[6], 4}, {NULL}};
const Segment SGI342670[] = {{&S29644[82], 5}, {NULL}};
const Segment SGI342671[] = {{&S29644[12], 14}, {&S29644[41], 11}, {NULL}};
const Segment SGI342672[] = {{&S29644[12], 14}, {NULL}};
const Segment SGI342673[] = {{&S29645[6], 4}, {NULL}};
const Segment SGI342674[] = {{&S29645[81], 5}, {NULL}};
const Segment SGI342675[] = {{&S29645[12], 14}, {&S29645[40], 11}, {NULL}};
const Segment SGI342676[] = {{&S29645[12], 14}, {NULL}};
const Segment SGI342677[] = {{&S29646[6], 4}, {NULL}};
const Segment SGI342678[] = {{&S29646[81], 5}, {NULL}};
const Segment SGI342679[] = {{&S29646[12], 14}, {&S29646[40], 11}, {NULL}};
const Segment SGI342680[] = {{&S29646[12], 14}, {NULL}};
const Segment SGI342681[] = {{&S29647[6], 4}, {NULL}};
const Segment SGI342682[] = {{&S29647[82], 5}, {NULL}};
const Segment SGI342683[] = {{&S29647[12], 14}, {&S29647[40], 12}, {NULL}};
const Segment SGI342684[] = {{&S29647[12], 14}, {NULL}};
const Segment SGI342685[] = {{&S29648[6], 4}, {NULL}};
const Segment SGI342686[] = {{&S29648[82], 5}, {NULL}};
const Segment SGI342687[] = {{&S29648[12], 14}, {&S29648[41], 11}, {NULL}};
const Segment SGI342688[] = {{&S29648[12], 14}, {NULL}};
const Segment SGI342689[] = {{&S29649[6], 4}, {NULL}};
const Segment SGI342690[] = {{&S29649[81], 5}, {NULL}};
const Segment SGI342691[] = {{&S29649[12], 14}, {&S29649[40], 11}, {NULL}};
const Segment SGI342692[] = {{&S29649[12], 14}, {NULL}};
const Segment SGI342693[] = {{&S29650[6], 4}, {NULL}};
const Segment SGI342694[] = {{&S29650[81], 5}, {NULL}};
const Segment SGI342695[] = {{&S29650[12], 14}, {&S29650[40], 11}, {NULL}};
const Segment SGI342696[] = {{&S29650[12], 14}, {NULL}};
const Segment SGI342697[] = {{&S29651[6], 4}, {NULL}};
const Segment SGI342698[] = {{&S29651[81], 5}, {NULL}};
const Segment SGI342699[] = {{&S29651[12], 14}, {&S29651[40], 11}, {NULL}};
const Segment SGI342700[] = {{&S29651[12], 14}, {NULL}};
const Segment SGI342701[] = {{&S29652[6], 4}, {NULL}};
const Segment SGI342702[] = {{&S29652[81], 5}, {NULL}};
const Segment SGI342703[] = {{&S29652[12], 14}, {&S29652[40], 11}, {NULL}};
const Segment SGI342704[] = {{&S29652[12], 14}, {NULL}};
const Segment SGI342705[] = {{&S29653[6], 4}, {NULL}};
const Segment SGI342706[] = {{&S29653[81], 5}, {NULL}};
const Segment SGI342707[] = {{&S29653[12], 14}, {&S29653[40], 11}, {NULL}};
const Segment SGI342708[] = {{&S29653[12], 14}, {NULL}};
const Segment SGI342709[] = {{&S29654[6], 4}, {NULL}};
const Segment SGI342710[] = {{&S29654[82], 5}, {NULL}};
const Segment SGI342711[] = {{&S29654[12], 14}, {&S29654[40], 12}, {NULL}};
const Segment SGI342712[] = {{&S29654[12], 14}, {NULL}};
const Segment SGI342713[] = {{&S29655[6], 4}, {NULL}};
const Segment SGI342714[] = {{&S29655[82], 5}, {NULL}};
const Segment SGI342715[] = {{&S29655[12], 14}, {&S29655[41], 11}, {NULL}};
const Segment SGI342716[] = {{&S29655[12], 14}, {NULL}};
const Segment SGI342717[] = {{&S29656[6], 4}, {NULL}};
const Segment SGI342718[] = {{&S29656[81], 5}, {NULL}};
const Segment SGI342719[] = {{&S29656[12], 14}, {&S29656[40], 11}, {NULL}};
const Segment SGI342720[] = {{&S29656[12], 14}, {NULL}};
const Segment SGI342721[] = {{&S29657[6], 4}, {NULL}};
const Segment SGI342722[] = {{&S29657[59], 5}, {NULL}};
const Segment SGI342723[] = {{&S29657[12], 14}, {&S29657[70], 4}, {NULL}};
const Segment SGI342724[] = {{&S29657[12], 14}, {NULL}};
const Segment SGI342725[] = {{&S29658[6], 4}, {NULL}};
const Segment SGI342726[] = {{&S29658[81], 5}, {NULL}};
const Segment SGI342727[] = {{&S29658[12], 14}, {&S29658[40], 11}, {NULL}};
const Segment SGI342728[] = {{&S29658[12], 14}, {NULL}};
const Segment SGI342729[] = {{&S29659[6], 4}, {NULL}};
const Segment SGI342730[] = {{&S29659[81], 5}, {NULL}};
const Segment SGI342731[] = {{&S29659[12], 14}, {&S29659[40], 11}, {NULL}};
const Segment SGI342732[] = {{&S29659[12], 14}, {NULL}};
const Segment SGI342733[] = {{&S29660[6], 4}, {NULL}};
const Segment SGI342734[] = {{&S29660[81], 5}, {NULL}};
const Segment SGI342735[] = {{&S29660[12], 14}, {&S29660[40], 11}, {NULL}};
const Segment SGI342736[] = {{&S29660[12], 14}, {NULL}};
const Segment SGI342737[] = {{&S29661[6], 4}, {NULL}};
const Segment SGI342738[] = {{&S29661[81], 5}, {NULL}};
const Segment SGI342739[] = {{&S29661[12], 14}, {&S29661[40], 11}, {NULL}};
const Segment SGI342740[] = {{&S29661[12], 14}, {NULL}};
const Segment SGI342741[] = {{&S29662[6], 4}, {NULL}};
const Segment SGI342742[] = {{&S29662[81], 5}, {NULL}};
const Segment SGI342743[] = {{&S29662[12], 14}, {&S29662[40], 11}, {NULL}};
const Segment SGI342744[] = {{&S29662[12], 14}, {NULL}};
const Segment SGI342745[] = {{&S29663[6], 4}, {NULL}};
const Segment SGI342746[] = {{&S29663[81], 5}, {NULL}};
const Segment SGI342747[] = {{&S29663[12], 14}, {&S29663[40], 11}, {NULL}};
const Segment SGI342748[] = {{&S29663[12], 14}, {NULL}};
const Segment SGI342749[] = {{&S29664[6], 4}, {NULL}};
const Segment SGI342750[] = {{&S29664[81], 5}, {NULL}};
const Segment SGI342751[] = {{&S29664[12], 14}, {&S29664[40], 11}, {NULL}};
const Segment SGI342752[] = {{&S29664[12], 14}, {NULL}};
const Segment SGI342753[] = {{&S29665[6], 4}, {NULL}};
const Segment SGI342754[] = {{&S29665[81], 5}, {NULL}};
const Segment SGI342755[] = {{&S29665[12], 14}, {&S29665[40], 11}, {NULL}};
const Segment SGI342756[] = {{&S29665[12], 14}, {NULL}};
const Segment SGI342757[] = {{&S29666[6], 4}, {NULL}};
const Segment SGI342758[] = {{&S29666[81], 5}, {NULL}};
const Segment SGI342759[] = {{&S29666[12], 14}, {&S29666[40], 11}, {NULL}};
const Segment SGI342760[] = {{&S29666[12], 14}, {NULL}};
const Segment SGI342761[] = {{&S29667[6], 4}, {NULL}};
const Segment SGI342762[] = {{&S29667[81], 5}, {NULL}};
const Segment SGI342763[] = {{&S29667[12], 14}, {&S29667[40], 11}, {NULL}};
const Segment SGI342764[] = {{&S29667[12], 14}, {NULL}};
const Segment SGI342765[] = {{&S29668[6], 4}, {NULL}};
const Segment SGI342766[] = {{&S29668[81], 5}, {NULL}};
const Segment SGI342767[] = {{&S29668[12], 14}, {&S29668[40], 11}, {NULL}};
const Segment SGI342768[] = {{&S29668[12], 14}, {NULL}};
const Segment SGI342769[] = {{&S29669[6], 4}, {NULL}};
const Segment SGI342770[] = {{&S29669[81], 5}, {NULL}};
const Segment SGI342771[] = {{&S29669[12], 14}, {&S29669[40], 11}, {NULL}};
const Segment SGI342772[] = {{&S29669[12], 14}, {NULL}};
const Segment SGI342773[] = {{&S29670[6], 4}, {NULL}};
const Segment SGI342774[] = {{&S29670[81], 5}, {NULL}};
const Segment SGI342775[] = {{&S29670[12], 14}, {&S29670[40], 11}, {NULL}};
const Segment SGI342776[] = {{&S29670[12], 14}, {NULL}};
const Segment SGI342777[] = {{&S56205[5], 4}, {NULL}};
const Segment SGI342778[] = {{&S29671[6], 4}, {NULL}};
const Segment SGI342779[] = {{&S29671[81], 5}, {NULL}};
const Segment SGI342780[] = {{&S29671[12], 14}, {&S29671[40], 11}, {NULL}};
const Segment SGI342781[] = {{&S29671[12], 14}, {NULL}};
const Segment SGI342782[] = {{&S56206[5], 7}, {NULL}};
const Segment SGI342783[] = {{&S29672[6], 4}, {NULL}};
const Segment SGI342784[] = {{&S29672[81], 5}, {NULL}};
const Segment SGI342785[] = {{&S29672[12], 14}, {&S29672[40], 11}, {NULL}};
const Segment SGI342786[] = {{&S29672[12], 14}, {NULL}};
const Segment SGI342787[] = {{&S56207[5], 16}, {NULL}};
const Segment SGI342788[] = {{&S56208[5], 9}, {NULL}};
const Segment SGI342789[] = {{&S31378[61], 5}, {NULL}};
const Segment SGI342790[] = {{&S31378[13], 10}, {NULL}};
const Segment SGI342791[] = {{&S56263[5], 5}, {&S56263[105], 5}, {NULL}};
const Segment SGI342792[] = {{&S56263[69], 3}, {NULL}};
const Segment SGI342793[] = {{&S56263[131], 3}, {NULL}};
const Segment SGI342794[] = {{&S47350[28], 3}, {NULL}};
const Segment SGI342795[] = {{&S47350[102], 5}, {NULL}};
const Segment SGI342796[] = {{&S47351[28], 3}, {NULL}};
const Segment SGI342797[] = {{&S47351[102], 5}, {NULL}};
const Segment SGI342798[] = {{&S47352[5], 3}, {&S47352[5], 3}, {NULL}};
const Segment SGI342799[] = {{&S47352[163], 5}, {NULL}};
const Segment SGI342800[] = {{&S56298[5], 3}, {&S56298[5], 3}, {NULL}};
const Segment SGI342801[] = {{&S56298[109], 3}, {NULL}};
const Segment SGI342802[] = {{&S31499[57], 5}, {NULL}};
const Segment SGI342803[] = {{&S31499[13], 10}, {NULL}};
const Segment SGI342804[] = {{&S31602[64], 5}, {NULL}};
const Segment SGI342805[] = {{&S31602[13], 10}, {NULL}};
const Segment SGI342806[] = {{&S47531[28], 3}, {NULL}};
const Segment SGI342807[] = {{&S47531[102], 5}, {NULL}};
const Segment SGI342808[] = {{&S31661[64], 5}, {NULL}};
const Segment SGI342809[] = {{&S31661[13], 10}, {NULL}};
const Segment SGI342810[] = {{&S47599[28], 5}, {NULL}};
const Segment SGI342811[] = {{&S47599[104], 5}, {NULL}};
const Segment SGI342812[] = {{&S47600[28], 7}, {NULL}};
const Segment SGI342813[] = {{&S47600[106], 5}, {NULL}};
const Segment SGI342814[] = {{&S47673[28], 3}, {NULL}};
const Segment SGI342815[] = {{&S47673[102], 5}, {NULL}};
const Segment SGI342816[] = {{&S47727[28], 5}, {NULL}};
const Segment SGI342817[] = {{&S47727[104], 5}, {NULL}};
const Segment SGI342818[] = {{&S31861[61], 5}, {NULL}};
const Segment SGI342819[] = {{&S31861[13], 10}, {NULL}};
const Segment SGI342820[] = {{&S42951[42], 14}, {&S42951[134], 7}, {NULL}};
const Segment SGI342821[] = {{&S42951[33], 3}, {NULL}};
const Segment SGI342822[] = {{&S42952[42], 5}, {&S42952[147], 7}, {NULL}};
const Segment SGI342823[] = {{&S42952[33], 3}, {NULL}};
const Segment SGI342824[] = {{&S47816[28], 5}, {NULL}};
const Segment SGI342825[] = {{&S47816[104], 5}, {NULL}};
const Segment SGI342826[] = {{&S47817[28], 5}, {NULL}};
const Segment SGI342827[] = {{&S47817[104], 5}, {NULL}};
const Segment SGI342828[] = {{&S47818[28], 7}, {NULL}};
const Segment SGI342829[] = {{&S47818[106], 5}, {NULL}};
const Segment SGI342830[] = {{&S47819[28], 7}, {NULL}};
const Segment SGI342831[] = {{&S47819[107], 5}, {NULL}};
const Segment SGI342832[] = {{&S30112[6], 4}, {NULL}};
const Segment SGI342833[] = {{&S30112[58], 5}, {NULL}};
const Segment SGI342834[] = {{&S30112[12], 14}, {NULL}};
const Segment SGI342835[] = {{&S30113[6], 4}, {NULL}};
const Segment SGI342836[] = {{&S30113[80], 5}, {NULL}};
const Segment SGI342837[] = {{&S30113[12], 14}, {&S30113[40], 11}, {NULL}};
const Segment SGI342838[] = {{&S30113[12], 14}, {NULL}};
const Segment SGI342839[] = {{&S30114[6], 4}, {NULL}};
const Segment SGI342840[] = {{&S30114[80], 5}, {NULL}};
const Segment SGI342841[] = {{&S30114[12], 14}, {&S30114[40], 11}, {NULL}};
const Segment SGI342842[] = {{&S30114[12], 14}, {NULL}};
const Segment SGI342843[] = {{&S30115[6], 4}, {NULL}};
const Segment SGI342844[] = {{&S30115[81], 5}, {NULL}};
const Segment SGI342845[] = {{&S30115[12], 14}, {&S30115[40], 12}, {NULL}};
const Segment SGI342846[] = {{&S30115[12], 14}, {NULL}};
const Segment SGI342847[] = {{&S30116[6], 4}, {NULL}};
const Segment SGI342848[] = {{&S30116[80], 5}, {NULL}};
const Segment SGI342849[] = {{&S30116[12], 14}, {&S30116[40], 11}, {NULL}};
const Segment SGI342850[] = {{&S30116[12], 14}, {NULL}};
const Segment SGI342851[] = {{&S30117[6], 4}, {NULL}};
const Segment SGI342852[] = {{&S30117[81], 5}, {NULL}};
const Segment SGI342853[] = {{&S30117[12], 14}, {&S30117[40], 12}, {NULL}};
const Segment SGI342854[] = {{&S30117[12], 14}, {NULL}};
const Segment SGI342855[] = {{&S30118[6], 4}, {NULL}};
const Segment SGI342856[] = {{&S30118[81], 5}, {NULL}};
const Segment SGI342857[] = {{&S30118[12], 14}, {&S30118[40], 12}, {NULL}};
const Segment SGI342858[] = {{&S30118[12], 14}, {NULL}};
const Segment SGI342859[] = {{&S30119[6], 4}, {NULL}};
const Segment SGI342860[] = {{&S30119[80], 5}, {NULL}};
const Segment SGI342861[] = {{&S30119[12], 14}, {&S30119[40], 11}, {NULL}};
const Segment SGI342862[] = {{&S30119[12], 14}, {NULL}};
const Segment SGI342863[] = {{&S30120[6], 4}, {NULL}};
const Segment SGI342864[] = {{&S30120[81], 5}, {NULL}};
const Segment SGI342865[] = {{&S30120[12], 14}, {&S30120[40], 12}, {NULL}};
const Segment SGI342866[] = {{&S30120[12], 14}, {NULL}};
const Segment SGI342867[] = {{&S30121[6], 4}, {NULL}};
const Segment SGI342868[] = {{&S30121[58], 5}, {NULL}};
const Segment SGI342869[] = {{&S30121[12], 14}, {&S30121[69], 31}, {NULL}};
const Segment SGI342870[] = {{&S30121[12], 14}, {NULL}};
const Segment SGI342871[] = {{&S30122[6], 4}, {NULL}};
const Segment SGI342872[] = {{&S30122[81], 5}, {NULL}};
const Segment SGI342873[] = {{&S30122[12], 14}, {&S30122[40], 12}, {NULL}};
const Segment SGI342874[] = {{&S30122[12], 14}, {NULL}};
const Segment SGI342875[] = {{&S30123[6], 4}, {NULL}};
const Segment SGI342876[] = {{&S30123[80], 5}, {NULL}};
const Segment SGI342877[] = {{&S30123[12], 14}, {&S30123[40], 11}, {NULL}};
const Segment SGI342878[] = {{&S30123[12], 14}, {NULL}};
const Segment SGI342879[] = {{&S30125[6], 4}, {NULL}};
const Segment SGI342880[] = {{&S30125[80], 5}, {NULL}};
const Segment SGI342881[] = {{&S30125[12], 14}, {&S30125[40], 11}, {NULL}};
const Segment SGI342882[] = {{&S30125[12], 14}, {NULL}};
const Segment SGI342883[] = {{&S30126[6], 4}, {NULL}};
const Segment SGI342884[] = {{&S30126[80], 5}, {NULL}};
const Segment SGI342885[] = {{&S30126[12], 14}, {&S30126[40], 11}, {NULL}};
const Segment SGI342886[] = {{&S30126[12], 14}, {NULL}};
const Segment SGI342887[] = {{&S30127[6], 4}, {NULL}};
const Segment SGI342888[] = {{&S30127[81], 5}, {NULL}};
const Segment SGI342889[] = {{&S30127[12], 14}, {&S30127[40], 12}, {NULL}};
const Segment SGI342890[] = {{&S30127[12], 14}, {NULL}};
const Segment SGI342891[] = {{&S30128[6], 4}, {NULL}};
const Segment SGI342892[] = {{&S30128[80], 5}, {NULL}};
const Segment SGI342893[] = {{&S30128[12], 14}, {&S30128[40], 11}, {NULL}};
const Segment SGI342894[] = {{&S30128[12], 14}, {NULL}};
const Segment SGI342895[] = {{&S30129[6], 4}, {NULL}};
const Segment SGI342896[] = {{&S30129[81], 5}, {NULL}};
const Segment SGI342897[] = {{&S30129[12], 14}, {&S30129[40], 12}, {NULL}};
const Segment SGI342898[] = {{&S30129[12], 14}, {NULL}};
const Segment SGI342899[] = {{&S30130[6], 4}, {NULL}};
const Segment SGI342900[] = {{&S30130[80], 5}, {NULL}};
const Segment SGI342901[] = {{&S30130[12], 14}, {&S30130[40], 11}, {NULL}};
const Segment SGI342902[] = {{&S30130[12], 14}, {NULL}};
const Segment SGI342903[] = {{&S30131[6], 4}, {NULL}};
const Segment SGI342904[] = {{&S30131[80], 5}, {NULL}};
const Segment SGI342905[] = {{&S30131[12], 14}, {&S30131[40], 11}, {NULL}};
const Segment SGI342906[] = {{&S30131[12], 14}, {NULL}};
const Segment SGI342907[] = {{&S30132[6], 4}, {NULL}};
const Segment SGI342908[] = {{&S30132[81], 5}, {NULL}};
const Segment SGI342909[] = {{&S30132[12], 14}, {&S30132[40], 12}, {NULL}};
const Segment SGI342910[] = {{&S30132[12], 14}, {NULL}};
const Segment SGI342911[] = {{&S30133[6], 4}, {NULL}};
const Segment SGI342912[] = {{&S30133[80], 5}, {NULL}};
const Segment SGI342913[] = {{&S30133[12], 14}, {&S30133[40], 11}, {NULL}};
const Segment SGI342914[] = {{&S30133[12], 14}, {NULL}};
const Segment SGI342915[] = {{&S30134[6], 4}, {NULL}};
const Segment SGI342916[] = {{&S30134[81], 5}, {NULL}};
const Segment SGI342917[] = {{&S30134[12], 14}, {&S30134[40], 12}, {NULL}};
const Segment SGI342918[] = {{&S30134[12], 14}, {NULL}};
const Segment SGI342919[] = {{&S30135[6], 4}, {NULL}};
const Segment SGI342920[] = {{&S30135[80], 5}, {NULL}};
const Segment SGI342921[] = {{&S30135[12], 14}, {&S30135[40], 11}, {NULL}};
const Segment SGI342922[] = {{&S30135[12], 14}, {NULL}};
const Segment SGI342923[] = {{&S30136[6], 4}, {NULL}};
const Segment SGI342924[] = {{&S30136[80], 5}, {NULL}};
const Segment SGI342925[] = {{&S30136[12], 14}, {&S30136[40], 11}, {NULL}};
const Segment SGI342926[] = {{&S30136[12], 14}, {NULL}};
const Segment SGI342927[] = {{&S30137[6], 4}, {NULL}};
const Segment SGI342928[] = {{&S30137[80], 5}, {NULL}};
const Segment SGI342929[] = {{&S30137[12], 14}, {&S30137[40], 11}, {NULL}};
const Segment SGI342930[] = {{&S30137[12], 14}, {NULL}};
const Segment SGI342931[] = {{&S30138[6], 4}, {NULL}};
const Segment SGI342932[] = {{&S30138[81], 5}, {NULL}};
const Segment SGI342933[] = {{&S30138[12], 14}, {&S30138[40], 12}, {NULL}};
const Segment SGI342934[] = {{&S30138[12], 14}, {NULL}};
const Segment SGI342935[] = {{&S30139[6], 4}, {NULL}};
const Segment SGI342936[] = {{&S30139[81], 5}, {NULL}};
const Segment SGI342937[] = {{&S30139[12], 14}, {&S30139[41], 11}, {NULL}};
const Segment SGI342938[] = {{&S30139[12], 14}, {NULL}};
const Segment SGI342939[] = {{&S30140[6], 4}, {NULL}};
const Segment SGI342940[] = {{&S30140[80], 5}, {NULL}};
const Segment SGI342941[] = {{&S30140[12], 14}, {&S30140[40], 11}, {NULL}};
const Segment SGI342942[] = {{&S30140[12], 14}, {NULL}};
const Segment SGI342943[] = {{&S30141[6], 4}, {NULL}};
const Segment SGI342944[] = {{&S30141[81], 5}, {NULL}};
const Segment SGI342945[] = {{&S30141[12], 14}, {&S30141[40], 12}, {NULL}};
const Segment SGI342946[] = {{&S30141[12], 14}, {NULL}};
const Segment SGI342947[] = {{&S30142[6], 4}, {NULL}};
const Segment SGI342948[] = {{&S30142[81], 5}, {NULL}};
const Segment SGI342949[] = {{&S30142[12], 14}, {&S30142[41], 11}, {NULL}};
const Segment SGI342950[] = {{&S30142[12], 14}, {NULL}};
const Segment SGI342951[] = {{&S30143[6], 4}, {NULL}};
const Segment SGI342952[] = {{&S30143[81], 5}, {NULL}};
const Segment SGI342953[] = {{&S30143[12], 14}, {&S30143[41], 11}, {NULL}};
const Segment SGI342954[] = {{&S30143[12], 14}, {NULL}};
const Segment SGI342955[] = {{&S30144[6], 4}, {NULL}};
const Segment SGI342956[] = {{&S30144[81], 5}, {NULL}};
const Segment SGI342957[] = {{&S30144[12], 14}, {&S30144[41], 11}, {NULL}};
const Segment SGI342958[] = {{&S30144[12], 14}, {NULL}};
const Segment SGI342959[] = {{&S30146[6], 4}, {NULL}};
const Segment SGI342960[] = {{&S30146[80], 5}, {NULL}};
const Segment SGI342961[] = {{&S30146[12], 14}, {&S30146[40], 11}, {NULL}};
const Segment SGI342962[] = {{&S30146[12], 14}, {NULL}};
const Segment SGI342963[] = {{&S30147[6], 4}, {NULL}};
const Segment SGI342964[] = {{&S30147[80], 5}, {NULL}};
const Segment SGI342965[] = {{&S30147[12], 14}, {&S30147[40], 11}, {NULL}};
const Segment SGI342966[] = {{&S30147[12], 14}, {NULL}};
const Segment SGI342967[] = {{&S30148[6], 4}, {NULL}};
const Segment SGI342968[] = {{&S30148[80], 5}, {NULL}};
const Segment SGI342969[] = {{&S30148[12], 14}, {&S30148[40], 11}, {NULL}};
const Segment SGI342970[] = {{&S30148[12], 14}, {NULL}};
const Segment SGI342971[] = {{&S30149[6], 4}, {NULL}};
const Segment SGI342972[] = {{&S30149[80], 5}, {NULL}};
const Segment SGI342973[] = {{&S30149[12], 14}, {&S30149[40], 11}, {NULL}};
const Segment SGI342974[] = {{&S30149[12], 14}, {NULL}};
const Segment SGI342975[] = {{&S30150[6], 4}, {NULL}};
const Segment SGI342976[] = {{&S30150[80], 5}, {NULL}};
const Segment SGI342977[] = {{&S30150[12], 14}, {&S30150[40], 11}, {NULL}};
const Segment SGI342978[] = {{&S30150[12], 14}, {NULL}};
const Segment SGI342979[] = {{&S30151[6], 4}, {NULL}};
const Segment SGI342980[] = {{&S30151[80], 5}, {NULL}};
const Segment SGI342981[] = {{&S30151[12], 14}, {&S30151[40], 11}, {NULL}};
const Segment SGI342982[] = {{&S30151[12], 14}, {NULL}};
const Segment SGI342983[] = {{&S30152[6], 4}, {NULL}};
const Segment SGI342984[] = {{&S30152[80], 5}, {NULL}};
const Segment SGI342985[] = {{&S30152[12], 14}, {&S30152[40], 11}, {NULL}};
const Segment SGI342986[] = {{&S30152[12], 14}, {NULL}};
const Segment SGI342987[] = {{&S30153[6], 4}, {NULL}};
const Segment SGI342988[] = {{&S30153[81], 5}, {NULL}};
const Segment SGI342989[] = {{&S30153[12], 14}, {&S30153[41], 11}, {NULL}};
const Segment SGI342990[] = {{&S30153[12], 14}, {NULL}};
const Segment SGI342991[] = {{&S30154[6], 4}, {NULL}};
const Segment SGI342992[] = {{&S30154[81], 5}, {NULL}};
const Segment SGI342993[] = {{&S30154[12], 14}, {&S30154[41], 11}, {NULL}};
const Segment SGI342994[] = {{&S30154[12], 14}, {NULL}};
const Segment SGI342995[] = {{&S30155[6], 4}, {NULL}};
const Segment SGI342996[] = {{&S30155[80], 5}, {NULL}};
const Segment SGI342997[] = {{&S30155[12], 14}, {&S30155[40], 11}, {NULL}};
const Segment SGI342998[] = {{&S30155[12], 14}, {NULL}};
const Segment SGI342999[] = {{&S30156[6], 4}, {NULL}};
const Segment SGI343000[] = {{&S30156[80], 5}, {NULL}};
const Segment SGI343001[] = {{&S30156[12], 14}, {&S30156[40], 11}, {NULL}};
const Segment SGI343002[] = {{&S30156[12], 14}, {NULL}};
const Segment SGI343003[] = {{&S30157[6], 4}, {NULL}};
const Segment SGI343004[] = {{&S30157[58], 5}, {NULL}};
const Segment SGI343005[] = {{&S30157[12], 14}, {&S30157[69], 4}, {NULL}};
const Segment SGI343006[] = {{&S30157[12], 14}, {NULL}};
const Segment SGI343007[] = {{&S30158[6], 4}, {NULL}};
const Segment SGI343008[] = {{&S30158[80], 5}, {NULL}};
const Segment SGI343009[] = {{&S30158[12], 14}, {&S30158[40], 11}, {NULL}};
const Segment SGI343010[] = {{&S30158[12], 14}, {NULL}};
const Segment SGI343011[] = {{&S30159[6], 4}, {NULL}};
const Segment SGI343012[] = {{&S30159[80], 5}, {NULL}};
const Segment SGI343013[] = {{&S30159[12], 14}, {&S30159[40], 11}, {NULL}};
const Segment SGI343014[] = {{&S30159[12], 14}, {NULL}};
const Segment SGI343015[] = {{&S30160[6], 4}, {NULL}};
const Segment SGI343016[] = {{&S30160[80], 5}, {NULL}};
const Segment SGI343017[] = {{&S30160[12], 14}, {&S30160[40], 11}, {NULL}};
const Segment SGI343018[] = {{&S30160[12], 14}, {NULL}};
const Segment SGI343019[] = {{&S30161[6], 4}, {NULL}};
const Segment SGI343020[] = {{&S30161[80], 5}, {NULL}};
const Segment SGI343021[] = {{&S30161[12], 14}, {&S30161[40], 11}, {NULL}};
const Segment SGI343022[] = {{&S30161[12], 14}, {NULL}};
const Segment SGI343023[] = {{&S30162[6], 4}, {NULL}};
const Segment SGI343024[] = {{&S30162[80], 5}, {NULL}};
const Segment SGI343025[] = {{&S30162[12], 14}, {&S30162[40], 11}, {NULL}};
const Segment SGI343026[] = {{&S30162[12], 14}, {NULL}};
const Segment SGI343027[] = {{&S30163[6], 4}, {NULL}};
const Segment SGI343028[] = {{&S30163[80], 5}, {NULL}};
const Segment SGI343029[] = {{&S30163[12], 14}, {&S30163[40], 11}, {NULL}};
const Segment SGI343030[] = {{&S30163[12], 14}, {NULL}};
const Segment SGI343031[] = {{&S56743[5], 5}, {&S56743[97], 5}, {NULL}};
const Segment SGI343032[] = {{&S56743[113], 3}, {NULL}};
const Segment SGI343033[] = {{&S56744[5], 7}, {&S56744[5], 7}, {NULL}};
const Segment SGI343034[] = {{&S56744[157], 3}, {NULL}};
const Segment SGI343035[] = {{&S56745[5], 4}, {&S56745[5], 4}, {NULL}};
const Segment SGI343036[] = {{&S56745[151], 3}, {NULL}};
const Segment SGI343037[] = {{&S43298[42], 3}, {&S43298[145], 7}, {NULL}};
const Segment SGI343038[] = {{&S43298[33], 3}, {NULL}};
const Segment SGI343039[] = {{&S56867[5], 5}, {&S56867[105], 5}, {NULL}};
const Segment SGI343040[] = {{&S56867[11], 3}, {NULL}};
const Segment SGI343041[] = {{&S56867[131], 3}, {NULL}};
const Segment SGI343042[] = {{&S56888[5], 14}, {NULL}};
const Segment SGI343043[] = {{&S56888[20], 3}, {NULL}};
const Segment SGI343044[] = {{&S32169[61], 5}, {NULL}};
const Segment SGI343045[] = {{&S32169[13], 10}, {NULL}};
const Segment SGI343046[] = {{&S56889[35], 5}, {NULL}};
const Segment SGI343047[] = {{&S32170[61], 5}, {NULL}};
const Segment SGI343048[] = {{&S56889[91], 3}, {NULL}};
const Segment SGI343049[] = {{&S32170[13], 10}, {NULL}};
const Segment SGI343050[] = {{&S56890[41], 5}, {NULL}};
const Segment SGI343051[] = {{&S56890[6], 3}, {NULL}};
const Segment SGI343052[] = {{&S56890[67], 3}, {NULL}};
const Segment SGI343053[] = {{&S56907[5], 7}, {&S56907[5], 7}, {NULL}};
const Segment SGI343054[] = {{&S56907[13], 3}, {NULL}};
const Segment SGI343055[] = {{&S56907[135], 3}, {NULL}};
const Segment SGI343056[] = {{&S56908[5], 4}, {&S56908[5], 4}, {NULL}};
const Segment SGI343057[] = {{&S56908[10], 3}, {NULL}};
const Segment SGI343058[] = {{&S56908[129], 3}, {NULL}};
const Segment SGI343059[] = {{&S30489[6], 4}, {NULL}};
const Segment SGI343060[] = {{&S30489[59], 5}, {NULL}};
const Segment SGI343061[] = {{&S30489[12], 14}, {NULL}};
const Segment SGI343062[] = {{&S30490[6], 4}, {NULL}};
const Segment SGI343063[] = {{&S30490[82], 5}, {NULL}};
const Segment SGI343064[] = {{&S30490[12], 14}, {&S30490[41], 11}, {NULL}};
const Segment SGI343065[] = {{&S30490[12], 14}, {NULL}};
const Segment SGI343066[] = {{&S30491[6], 4}, {NULL}};
const Segment SGI343067[] = {{&S30491[81], 5}, {NULL}};
const Segment SGI343068[] = {{&S30491[12], 14}, {&S30491[40], 11}, {NULL}};
const Segment SGI343069[] = {{&S30491[12], 14}, {NULL}};
const Segment SGI343070[] = {{&S30492[6], 4}, {NULL}};
const Segment SGI343071[] = {{&S30492[81], 5}, {NULL}};
const Segment SGI343072[] = {{&S30492[12], 14}, {&S30492[40], 11}, {NULL}};
const Segment SGI343073[] = {{&S30492[12], 14}, {NULL}};
const Segment SGI343074[] = {{&S30493[6], 4}, {NULL}};
const Segment SGI343075[] = {{&S30493[82], 5}, {NULL}};
const Segment SGI343076[] = {{&S30493[12], 14}, {&S30493[40], 12}, {NULL}};
const Segment SGI343077[] = {{&S30493[12], 14}, {NULL}};
const Segment SGI343078[] = {{&S30494[6], 4}, {NULL}};
const Segment SGI343079[] = {{&S30494[82], 5}, {NULL}};
const Segment SGI343080[] = {{&S30494[12], 14}, {&S30494[40], 12}, {NULL}};
const Segment SGI343081[] = {{&S30494[12], 14}, {NULL}};
const Segment SGI343082[] = {{&S30495[6], 4}, {NULL}};
const Segment SGI343083[] = {{&S30495[82], 5}, {NULL}};
const Segment SGI343084[] = {{&S30495[12], 14}, {&S30495[41], 11}, {NULL}};
const Segment SGI343085[] = {{&S30495[12], 14}, {NULL}};
const Segment SGI343086[] = {{&S30496[6], 4}, {NULL}};
const Segment SGI343087[] = {{&S30496[81], 5}, {NULL}};
const Segment SGI343088[] = {{&S30496[12], 14}, {&S30496[40], 11}, {NULL}};
const Segment SGI343089[] = {{&S30496[12], 14}, {NULL}};
const Segment SGI343090[] = {{&S30497[6], 4}, {NULL}};
const Segment SGI343091[] = {{&S30497[81], 5}, {NULL}};
const Segment SGI343092[] = {{&S30497[12], 14}, {&S30497[40], 11}, {NULL}};
const Segment SGI343093[] = {{&S30497[12], 14}, {NULL}};
const Segment SGI343094[] = {{&S30498[6], 4}, {NULL}};
const Segment SGI343095[] = {{&S30498[81], 5}, {NULL}};
const Segment SGI343096[] = {{&S30498[12], 14}, {&S30498[40], 11}, {NULL}};
const Segment SGI343097[] = {{&S30498[12], 14}, {NULL}};
const Segment SGI343098[] = {{&S30499[6], 4}, {NULL}};
const Segment SGI343099[] = {{&S30499[81], 5}, {NULL}};
const Segment SGI343100[] = {{&S30499[12], 14}, {&S30499[40], 11}, {NULL}};
const Segment SGI343101[] = {{&S30499[12], 14}, {NULL}};
const Segment SGI343102[] = {{&S30500[6], 4}, {NULL}};
const Segment SGI343103[] = {{&S30500[82], 5}, {NULL}};
const Segment SGI343104[] = {{&S30500[12], 14}, {&S30500[40], 12}, {NULL}};
const Segment SGI343105[] = {{&S30500[12], 14}, {NULL}};
const Segment SGI343106[] = {{&S30501[6], 4}, {NULL}};
const Segment SGI343107[] = {{&S30501[81], 5}, {NULL}};
const Segment SGI343108[] = {{&S30501[12], 14}, {&S30501[40], 11}, {NULL}};
const Segment SGI343109[] = {{&S30501[12], 14}, {NULL}};
const Segment SGI343110[] = {{&S30502[6], 4}, {NULL}};
const Segment SGI343111[] = {{&S30502[81], 5}, {NULL}};
const Segment SGI343112[] = {{&S30502[12], 14}, {&S30502[40], 11}, {NULL}};
const Segment SGI343113[] = {{&S30502[12], 14}, {NULL}};
const Segment SGI343114[] = {{&S30504[6], 4}, {NULL}};
const Segment SGI343115[] = {{&S30504[82], 5}, {NULL}};
const Segment SGI343116[] = {{&S30504[12], 14}, {&S30504[41], 11}, {NULL}};
const Segment SGI343117[] = {{&S30504[12], 14}, {NULL}};
const Segment SGI343118[] = {{&S30505[6], 4}, {NULL}};
const Segment SGI343119[] = {{&S30505[82], 5}, {NULL}};
const Segment SGI343120[] = {{&S30505[12], 14}, {&S30505[41], 11}, {NULL}};
const Segment SGI343121[] = {{&S30505[12], 14}, {NULL}};
const Segment SGI343122[] = {{&S30506[6], 4}, {NULL}};
const Segment SGI343123[] = {{&S30506[82], 5}, {NULL}};
const Segment SGI343124[] = {{&S30506[12], 14}, {&S30506[41], 11}, {NULL}};
const Segment SGI343125[] = {{&S30506[12], 14}, {NULL}};
const Segment SGI343126[] = {{&S30507[6], 4}, {NULL}};
const Segment SGI343127[] = {{&S30507[82], 5}, {NULL}};
const Segment SGI343128[] = {{&S30507[12], 14}, {&S30507[40], 12}, {NULL}};
const Segment SGI343129[] = {{&S30507[12], 14}, {NULL}};
const Segment SGI343130[] = {{&S30508[6], 4}, {NULL}};
const Segment SGI343131[] = {{&S30508[81], 5}, {NULL}};
const Segment SGI343132[] = {{&S30508[12], 14}, {&S30508[40], 11}, {NULL}};
const Segment SGI343133[] = {{&S30508[12], 14}, {NULL}};
const Segment SGI343134[] = {{&S30509[6], 4}, {NULL}};
const Segment SGI343135[] = {{&S30509[82], 5}, {NULL}};
const Segment SGI343136[] = {{&S30509[12], 14}, {&S30509[41], 11}, {NULL}};
const Segment SGI343137[] = {{&S30509[12], 14}, {NULL}};
const Segment SGI343138[] = {{&S30510[6], 4}, {NULL}};
const Segment SGI343139[] = {{&S30510[82], 5}, {NULL}};
const Segment SGI343140[] = {{&S30510[12], 14}, {&S30510[41], 11}, {NULL}};
const Segment SGI343141[] = {{&S30510[12], 14}, {NULL}};
const Segment SGI343142[] = {{&S30511[6], 4}, {NULL}};
const Segment SGI343143[] = {{&S30511[82], 5}, {NULL}};
const Segment SGI343144[] = {{&S30511[12], 14}, {&S30511[41], 11}, {NULL}};
const Segment SGI343145[] = {{&S30511[12], 14}, {NULL}};
const Segment SGI343146[] = {{&S30512[6], 4}, {NULL}};
const Segment SGI343147[] = {{&S30512[82], 5}, {NULL}};
const Segment SGI343148[] = {{&S30512[12], 14}, {&S30512[41], 11}, {NULL}};
const Segment SGI343149[] = {{&S30512[12], 14}, {NULL}};
const Segment SGI343150[] = {{&S30513[6], 4}, {NULL}};
const Segment SGI343151[] = {{&S30513[59], 5}, {NULL}};
const Segment SGI343152[] = {{&S30513[12], 14}, {NULL}};
const Segment SGI343153[] = {{&S30514[6], 4}, {NULL}};
const Segment SGI343154[] = {{&S30514[81], 5}, {NULL}};
const Segment SGI343155[] = {{&S30514[12], 14}, {&S30514[40], 11}, {NULL}};
const Segment SGI343156[] = {{&S30514[12], 14}, {NULL}};
const Segment SGI343157[] = {{&S30515[6], 4}, {NULL}};
const Segment SGI343158[] = {{&S30515[82], 5}, {NULL}};
const Segment SGI343159[] = {{&S30515[12], 14}, {&S30515[41], 11}, {NULL}};
const Segment SGI343160[] = {{&S30515[12], 14}, {NULL}};
const Segment SGI343161[] = {{&S30516[6], 4}, {NULL}};
const Segment SGI343162[] = {{&S30516[82], 5}, {NULL}};
const Segment SGI343163[] = {{&S30516[12], 14}, {&S30516[40], 12}, {NULL}};
const Segment SGI343164[] = {{&S30516[12], 14}, {NULL}};
const Segment SGI343165[] = {{&S30517[6], 4}, {NULL}};
const Segment SGI343166[] = {{&S30517[82], 5}, {NULL}};
const Segment SGI343167[] = {{&S30517[12], 14}, {&S30517[41], 11}, {NULL}};
const Segment SGI343168[] = {{&S30517[12], 14}, {NULL}};
const Segment SGI343169[] = {{&S30518[6], 4}, {NULL}};
const Segment SGI343170[] = {{&S30518[82], 5}, {NULL}};
const Segment SGI343171[] = {{&S30518[12], 14}, {&S30518[40], 12}, {NULL}};
const Segment SGI343172[] = {{&S30518[12], 14}, {NULL}};
const Segment SGI343173[] = {{&S30519[6], 4}, {NULL}};
const Segment SGI343174[] = {{&S30519[81], 5}, {NULL}};
const Segment SGI343175[] = {{&S30519[12], 14}, {&S30519[40], 11}, {NULL}};
const Segment SGI343176[] = {{&S30519[12], 14}, {NULL}};
const Segment SGI343177[] = {{&S30520[6], 4}, {NULL}};
const Segment SGI343178[] = {{&S30520[82], 5}, {NULL}};
const Segment SGI343179[] = {{&S30520[12], 14}, {&S30520[40], 12}, {NULL}};
const Segment SGI343180[] = {{&S30520[12], 14}, {NULL}};
const Segment SGI343181[] = {{&S30521[6], 4}, {NULL}};
const Segment SGI343182[] = {{&S30521[82], 5}, {NULL}};
const Segment SGI343183[] = {{&S30521[12], 14}, {&S30521[41], 11}, {NULL}};
const Segment SGI343184[] = {{&S30521[12], 14}, {NULL}};
const Segment SGI343185[] = {{&S30522[6], 4}, {NULL}};
const Segment SGI343186[] = {{&S30522[82], 5}, {NULL}};
const Segment SGI343187[] = {{&S30522[12], 14}, {&S30522[40], 12}, {NULL}};
const Segment SGI343188[] = {{&S30522[12], 14}, {NULL}};
const Segment SGI343189[] = {{&S30523[6], 4}, {NULL}};
const Segment SGI343190[] = {{&S30523[81], 5}, {NULL}};
const Segment SGI343191[] = {{&S30523[12], 14}, {&S30523[40], 11}, {NULL}};
const Segment SGI343192[] = {{&S30523[12], 14}, {NULL}};
const Segment SGI343193[] = {{&S30524[6], 4}, {NULL}};
const Segment SGI343194[] = {{&S30524[82], 5}, {NULL}};
const Segment SGI343195[] = {{&S30524[12], 14}, {&S30524[41], 11}, {NULL}};
const Segment SGI343196[] = {{&S30524[12], 14}, {NULL}};
const Segment SGI343197[] = {{&S30525[6], 4}, {NULL}};
const Segment SGI343198[] = {{&S30525[82], 5}, {NULL}};
const Segment SGI343199[] = {{&S30525[12], 14}, {&S30525[40], 12}, {NULL}};
const Segment SGI343200[] = {{&S30525[12], 14}, {NULL}};
const Segment SGI343201[] = {{&S30526[6], 4}, {NULL}};
const Segment SGI343202[] = {{&S30526[82], 5}, {NULL}};
const Segment SGI343203[] = {{&S30526[12], 14}, {&S30526[40], 12}, {NULL}};
const Segment SGI343204[] = {{&S30526[12], 14}, {NULL}};
const Segment SGI343205[] = {{&S30527[6], 4}, {NULL}};
const Segment SGI343206[] = {{&S30527[81], 5}, {NULL}};
const Segment SGI343207[] = {{&S30527[12], 14}, {&S30527[40], 11}, {NULL}};
const Segment SGI343208[] = {{&S30527[12], 14}, {NULL}};
const Segment SGI343209[] = {{&S30528[6], 4}, {NULL}};
const Segment SGI343210[] = {{&S30528[82], 5}, {NULL}};
const Segment SGI343211[] = {{&S30528[12], 14}, {&S30528[41], 11}, {NULL}};
const Segment SGI343212[] = {{&S30528[12], 14}, {NULL}};
const Segment SGI343213[] = {{&S30529[6], 4}, {NULL}};
const Segment SGI343214[] = {{&S30529[59], 5}, {NULL}};
const Segment SGI343215[] = {{&S30529[12], 14}, {&S30529[70], 31}, {NULL}};
const Segment SGI343216[] = {{&S30529[12], 14}, {NULL}};
const Segment SGI343217[] = {{&S30530[6], 4}, {NULL}};
const Segment SGI343218[] = {{&S30530[81], 5}, {NULL}};
const Segment SGI343219[] = {{&S30530[12], 14}, {&S30530[40], 11}, {NULL}};
const Segment SGI343220[] = {{&S30530[12], 14}, {NULL}};
const Segment SGI343221[] = {{&S30531[6], 4}, {NULL}};
const Segment SGI343222[] = {{&S30531[82], 5}, {NULL}};
const Segment SGI343223[] = {{&S30531[12], 14}, {&S30531[41], 11}, {NULL}};
const Segment SGI343224[] = {{&S30531[12], 14}, {NULL}};
const Segment SGI343225[] = {{&S30532[6], 4}, {NULL}};
const Segment SGI343226[] = {{&S30532[82], 5}, {NULL}};
const Segment SGI343227[] = {{&S30532[12], 14}, {&S30532[41], 11}, {NULL}};
const Segment SGI343228[] = {{&S30532[12], 14}, {NULL}};
const Segment SGI343229[] = {{&S30533[6], 4}, {NULL}};
const Segment SGI343230[] = {{&S30533[82], 5}, {NULL}};
const Segment SGI343231[] = {{&S30533[12], 14}, {&S30533[40], 12}, {NULL}};
const Segment SGI343232[] = {{&S30533[12], 14}, {NULL}};
const Segment SGI343233[] = {{&S30534[6], 4}, {NULL}};
const Segment SGI343234[] = {{&S30534[81], 5}, {NULL}};
const Segment SGI343235[] = {{&S30534[12], 14}, {&S30534[40], 11}, {NULL}};
const Segment SGI343236[] = {{&S30534[12], 14}, {NULL}};
const Segment SGI343237[] = {{&S30535[6], 4}, {NULL}};
const Segment SGI343238[] = {{&S30535[81], 5}, {NULL}};
const Segment SGI343239[] = {{&S30535[12], 14}, {&S30535[40], 11}, {NULL}};
const Segment SGI343240[] = {{&S30535[12], 14}, {NULL}};
const Segment SGI343241[] = {{&S30537[6], 4}, {NULL}};
const Segment SGI343242[] = {{&S30537[81], 5}, {NULL}};
const Segment SGI343243[] = {{&S30537[12], 14}, {&S30537[40], 11}, {NULL}};
const Segment SGI343244[] = {{&S30537[12], 14}, {NULL}};
const Segment SGI343245[] = {{&S30538[6], 4}, {NULL}};
const Segment SGI343246[] = {{&S30538[82], 5}, {NULL}};
const Segment SGI343247[] = {{&S30538[12], 14}, {&S30538[41], 11}, {NULL}};
const Segment SGI343248[] = {{&S30538[12], 14}, {NULL}};
const Segment SGI343249[] = {{&S30539[6], 4}, {NULL}};
const Segment SGI343250[] = {{&S30539[82], 5}, {NULL}};
const Segment SGI343251[] = {{&S30539[12], 14}, {&S30539[41], 11}, {NULL}};
const Segment SGI343252[] = {{&S30539[12], 14}, {NULL}};
const Segment SGI343253[] = {{&S30540[6], 4}, {NULL}};
const Segment SGI343254[] = {{&S30540[82], 5}, {NULL}};
const Segment SGI343255[] = {{&S30540[12], 14}, {&S30540[40], 12}, {NULL}};
const Segment SGI343256[] = {{&S30540[12], 14}, {NULL}};
const Segment SGI343257[] = {{&S30541[6], 4}, {NULL}};
const Segment SGI343258[] = {{&S30541[81], 5}, {NULL}};
const Segment SGI343259[] = {{&S30541[12], 14}, {&S30541[40], 11}, {NULL}};
const Segment SGI343260[] = {{&S30541[12], 14}, {NULL}};
const Segment SGI343261[] = {{&S30542[6], 4}, {NULL}};
const Segment SGI343262[] = {{&S30542[82], 5}, {NULL}};
const Segment SGI343263[] = {{&S30542[12], 14}, {&S30542[41], 11}, {NULL}};
const Segment SGI343264[] = {{&S30542[12], 14}, {NULL}};
const Segment SGI343265[] = {{&S30543[6], 4}, {NULL}};
const Segment SGI343266[] = {{&S30543[81], 5}, {NULL}};
const Segment SGI343267[] = {{&S30543[12], 14}, {&S30543[40], 11}, {NULL}};
const Segment SGI343268[] = {{&S30543[12], 14}, {NULL}};
const Segment SGI343269[] = {{&S30544[6], 4}, {NULL}};
const Segment SGI343270[] = {{&S30544[81], 5}, {NULL}};
const Segment SGI343271[] = {{&S30544[12], 14}, {&S30544[40], 11}, {NULL}};
const Segment SGI343272[] = {{&S30544[12], 14}, {NULL}};
const Segment SGI343273[] = {{&S30545[6], 4}, {NULL}};
const Segment SGI343274[] = {{&S30545[81], 5}, {NULL}};
const Segment SGI343275[] = {{&S30545[12], 14}, {&S30545[40], 11}, {NULL}};
const Segment SGI343276[] = {{&S30545[12], 14}, {NULL}};
const Segment SGI343277[] = {{&S30546[6], 4}, {NULL}};
const Segment SGI343278[] = {{&S30546[81], 5}, {NULL}};
const Segment SGI343279[] = {{&S30546[12], 14}, {&S30546[40], 11}, {NULL}};
const Segment SGI343280[] = {{&S30546[12], 14}, {NULL}};
const Segment SGI343281[] = {{&S30547[6], 4}, {NULL}};
const Segment SGI343282[] = {{&S30547[82], 5}, {NULL}};
const Segment SGI343283[] = {{&S30547[12], 14}, {&S30547[40], 12}, {NULL}};
const Segment SGI343284[] = {{&S30547[12], 14}, {NULL}};
const Segment SGI343285[] = {{&S30548[6], 4}, {NULL}};
const Segment SGI343286[] = {{&S30548[82], 5}, {NULL}};
const Segment SGI343287[] = {{&S30548[12], 14}, {&S30548[40], 12}, {NULL}};
const Segment SGI343288[] = {{&S30548[12], 14}, {NULL}};
const Segment SGI343289[] = {{&S30549[6], 4}, {NULL}};
const Segment SGI343290[] = {{&S30549[82], 5}, {NULL}};
const Segment SGI343291[] = {{&S30549[12], 14}, {&S30549[40], 12}, {NULL}};
const Segment SGI343292[] = {{&S30549[12], 14}, {NULL}};
const Segment SGI343293[] = {{&S30550[6], 4}, {NULL}};
const Segment SGI343294[] = {{&S30550[82], 5}, {NULL}};
const Segment SGI343295[] = {{&S30550[12], 14}, {&S30550[40], 12}, {NULL}};
const Segment SGI343296[] = {{&S30550[12], 14}, {NULL}};
const Segment SGI343297[] = {{&S30551[6], 4}, {NULL}};
const Segment SGI343298[] = {{&S30551[81], 5}, {NULL}};
const Segment SGI343299[] = {{&S30551[12], 14}, {&S30551[40], 11}, {NULL}};
const Segment SGI343300[] = {{&S30551[12], 14}, {NULL}};
const Segment SGI343301[] = {{&S30552[6], 4}, {NULL}};
const Segment SGI343302[] = {{&S30552[81], 5}, {NULL}};
const Segment SGI343303[] = {{&S30552[12], 14}, {&S30552[40], 11}, {NULL}};
const Segment SGI343304[] = {{&S30552[12], 14}, {NULL}};
const Segment SGI343305[] = {{&S30553[6], 4}, {NULL}};
const Segment SGI343306[] = {{&S30553[81], 5}, {NULL}};
const Segment SGI343307[] = {{&S30553[12], 14}, {&S30553[40], 11}, {NULL}};
const Segment SGI343308[] = {{&S30553[12], 14}, {NULL}};
const Segment SGI343309[] = {{&S57014[5], 5}, {&S57014[105], 5}, {NULL}};
const Segment SGI343310[] = {{&S57014[11], 3}, {NULL}};
const Segment SGI343311[] = {{&S30554[6], 4}, {NULL}};
const Segment SGI343312[] = {{&S30554[81], 5}, {NULL}};
const Segment SGI343313[] = {{&S30554[12], 14}, {&S30554[40], 11}, {NULL}};
const Segment SGI343314[] = {{&S57014[131], 3}, {NULL}};
const Segment SGI343315[] = {{&S30554[12], 14}, {NULL}};
const Segment SGI343316[] = {{&S30555[6], 4}, {NULL}};
const Segment SGI343317[] = {{&S30555[82], 5}, {NULL}};
const Segment SGI343318[] = {{&S30555[12], 14}, {&S30555[40], 12}, {NULL}};
const Segment SGI343319[] = {{&S30555[12], 14}, {NULL}};
const Segment SGI343320[] = {{&S30556[6], 4}, {NULL}};
const Segment SGI343321[] = {{&S30556[82], 5}, {NULL}};
const Segment SGI343322[] = {{&S30556[12], 14}, {&S30556[40], 12}, {NULL}};
const Segment SGI343323[] = {{&S30556[12], 14}, {NULL}};
const Segment SGI343324[] = {{&S30557[6], 4}, {NULL}};
const Segment SGI343325[] = {{&S30557[59], 5}, {NULL}};
const Segment SGI343326[] = {{&S30557[12], 14}, {&S30557[70], 4}, {NULL}};
const Segment SGI343327[] = {{&S30557[12], 14}, {NULL}};
const Segment SGI343328[] = {{&S30558[6], 4}, {NULL}};
const Segment SGI343329[] = {{&S30558[82], 5}, {NULL}};
const Segment SGI343330[] = {{&S30558[12], 14}, {&S30558[40], 12}, {NULL}};
const Segment SGI343331[] = {{&S30558[12], 14}, {NULL}};
const Segment SGI343332[] = {{&S57019[5], 5}, {&S57019[105], 5}, {NULL}};
const Segment SGI343333[] = {{&S57019[11], 3}, {NULL}};
const Segment SGI343334[] = {{&S30559[6], 4}, {NULL}};
const Segment SGI343335[] = {{&S57019[131], 3}, {NULL}};
const Segment SGI343336[] = {{&S30559[82], 5}, {NULL}};
const Segment SGI343337[] = {{&S30559[12], 14}, {&S30559[40], 12}, {NULL}};
const Segment SGI343338[] = {{&S30559[12], 14}, {NULL}};
const Segment SGI343339[] = {{&S30560[6], 4}, {NULL}};
const Segment SGI343340[] = {{&S30560[81], 5}, {NULL}};
const Segment SGI343341[] = {{&S30560[12], 14}, {&S30560[40], 11}, {NULL}};
const Segment SGI343342[] = {{&S30560[12], 14}, {NULL}};
const Segment SGI343343[] = {{&S30561[6], 4}, {NULL}};
const Segment SGI343344[] = {{&S30561[81], 5}, {NULL}};
const Segment SGI343345[] = {{&S30561[12], 14}, {&S30561[40], 11}, {NULL}};
const Segment SGI343346[] = {{&S30561[12], 14}, {NULL}};
const Segment SGI343347[] = {{&S30562[6], 4}, {NULL}};
const Segment SGI343348[] = {{&S30562[81], 5}, {NULL}};
const Segment SGI343349[] = {{&S30562[12], 14}, {&S30562[40], 11}, {NULL}};
const Segment SGI343350[] = {{&S57023[35], 5}, {NULL}};
const Segment SGI343351[] = {{&S30562[12], 14}, {NULL}};
const Segment SGI343352[] = {{&S57023[91], 3}, {NULL}};
const Segment SGI343353[] = {{&S30563[6], 4}, {NULL}};
const Segment SGI343354[] = {{&S30563[81], 5}, {NULL}};
const Segment SGI343355[] = {{&S30563[12], 14}, {&S30563[40], 11}, {NULL}};
const Segment SGI343356[] = {{&S30563[12], 14}, {NULL}};
const Segment SGI343357[] = {{&S57024[5], 14}, {NULL}};
const Segment SGI343358[] = {{&S57024[20], 3}, {NULL}};
const Segment SGI343359[] = {{&S30564[6], 4}, {NULL}};
const Segment SGI343360[] = {{&S30564[81], 5}, {NULL}};
const Segment SGI343361[] = {{&S30564[12], 14}, {&S30564[40], 11}, {NULL}};
const Segment SGI343362[] = {{&S30564[12], 14}, {NULL}};
const Segment SGI343363[] = {{&S30565[6], 4}, {NULL}};
const Segment SGI343364[] = {{&S30565[81], 5}, {NULL}};
const Segment SGI343365[] = {{&S30565[12], 14}, {&S30565[40], 11}, {NULL}};
const Segment SGI343366[] = {{&S30565[12], 14}, {NULL}};
const Segment SGI343367[] = {{&S30566[6], 4}, {NULL}};
const Segment SGI343368[] = {{&S30566[81], 5}, {NULL}};
const Segment SGI343369[] = {{&S30566[12], 14}, {&S30566[40], 11}, {NULL}};
const Segment SGI343370[] = {{&S30566[12], 14}, {NULL}};
const Segment SGI343371[] = {{&S30567[6], 4}, {NULL}};
const Segment SGI343372[] = {{&S30567[81], 5}, {NULL}};
const Segment SGI343373[] = {{&S30567[12], 14}, {&S30567[40], 11}, {NULL}};
const Segment SGI343374[] = {{&S30567[12], 14}, {NULL}};
const Segment SGI343375[] = {{&S30568[6], 4}, {NULL}};
const Segment SGI343376[] = {{&S30568[81], 5}, {NULL}};
const Segment SGI343377[] = {{&S30568[12], 14}, {&S30568[40], 11}, {NULL}};
const Segment SGI343378[] = {{&S30568[12], 14}, {NULL}};
const Segment SGI343379[] = {{&S30569[6], 4}, {NULL}};
const Segment SGI343380[] = {{&S30569[89], 5}, {&S30569[89], 5}, {NULL}};
const Segment SGI343381[] = {{&S30569[12], 14}, {NULL}};
const Segment SGI343382[] = {{&S30586[6], 4}, {NULL}};
const Segment SGI343383[] = {{&S30586[59], 5}, {NULL}};
const Segment SGI343384[] = {{&S30586[12], 3}, {NULL}};
const Segment SGI343385[] = {{&S30587[6], 4}, {NULL}};
const Segment SGI343386[] = {{&S30587[59], 5}, {NULL}};
const Segment SGI343387[] = {{&S30587[12], 3}, {NULL}};
const Segment SGI343388[] = {{&S48307[28], 5}, {NULL}};
const Segment SGI343389[] = {{&S48307[104], 5}, {NULL}};
const Segment SGI343390[] = {{&S57074[5], 3}, {&S57074[5], 3}, {NULL}};
const Segment SGI343391[] = {{&S57074[149], 3}, {NULL}};
const Segment SGI343392[] = {{&S57082[5], 5}, {NULL}};
const Segment SGI343393[] = {{&S57085[5], 4}, {NULL}};
const Segment SGI343394[] = {{&S43638[42], 5}, {&S43638[142], 7}, {NULL}};
const Segment SGI343395[] = {{&S43638[33], 3}, {NULL}};
const Segment SGI343396[] = {{&S43646[42], 3}, {&S43646[145], 7}, {NULL}};
const Segment SGI343397[] = {{&S43646[33], 3}, {NULL}};
const Segment SGI343398[] = {{&S43647[42], 3}, {&S43647[145], 7}, {NULL}};
const Segment SGI343399[] = {{&S43647[33], 3}, {NULL}};
const Segment SGI343400[] = {{&S43648[42], 12}, {&S43648[132], 7}, {NULL}};
const Segment SGI343401[] = {{&S43648[33], 3}, {NULL}};
const Segment SGI343402[] = {{&S43649[42], 3}, {&S43649[145], 7}, {NULL}};
const Segment SGI343403[] = {{&S43649[33], 3}, {NULL}};
const Segment SGI343404[] = {{&S57265[5], 5}, {&S57265[97], 5}, {NULL}};
const Segment SGI343405[] = {{&S57265[113], 3}, {NULL}};
const Segment SGI343406[] = {{&S57275[5], 4}, {NULL}};
const Segment SGI343407[] = {{&S57276[5], 9}, {NULL}};
const Segment SGI343408[] = {{&S57314[5], 5}, {&S57314[97], 5}, {NULL}};
const Segment SGI343409[] = {{&S57314[153], 3}, {NULL}};
const Segment SGI343410[] = {{&S57325[5], 7}, {NULL}};
const Segment SGI343411[] = {{&S57447[5], 4}, {NULL}};
const Segment SGI343412[] = {{&S57467[5], 5}, {&S57467[105], 5}, {NULL}};
const Segment SGI343413[] = {{&S57467[11], 3}, {NULL}};
const Segment SGI343414[] = {{&S57467[131], 3}, {NULL}};
const Segment SGI343415[] = {{&S57468[5], 5}, {NULL}};
const Segment SGI343416[] = {{&S57469[5], 5}, {&S57469[5], 5}, {NULL}};
const Segment SGI343417[] = {{&S57507[5], 5}, {&S57507[97], 5}, {NULL}};
const Segment SGI343418[] = {{&S57507[113], 3}, {NULL}};
const Segment SGI343419[] = {{&S32693[61], 5}, {NULL}};
const Segment SGI343420[] = {{&S32693[13], 10}, {NULL}};
const Segment SGI343421[] = {{&S57540[5], 5}, {&S57540[97], 5}, {NULL}};
const Segment SGI343422[] = {{&S57540[113], 3}, {NULL}};
const Segment SGI343423[] = {{&S31012[6], 4}, {NULL}};
const Segment SGI343424[] = {{&S31012[59], 5}, {NULL}};
const Segment SGI343425[] = {{&S31012[12], 14}, {NULL}};
const Segment SGI343426[] = {{&S31013[6], 4}, {NULL}};
const Segment SGI343427[] = {{&S31013[74], 5}, {NULL}};
const Segment SGI343428[] = {{&S31013[12], 14}, {&S31013[37], 42}, {NULL}};
const Segment SGI343429[] = {{&S31013[12], 14}, {NULL}};
const Segment SGI343430[] = {{&S31014[6], 4}, {NULL}};
const Segment SGI343431[] = {{&S31014[52], 5}, {NULL}};
const Segment SGI343432[] = {{&S31014[12], 45}, {NULL}};
const Segment SGI343433[] = {{&S31014[12], 14}, {NULL}};
const Segment SGI343434[] = {{&S31015[6], 4}, {NULL}};
const Segment SGI343435[] = {{&S31015[74], 5}, {NULL}};
const Segment SGI343436[] = {{&S31015[12], 14}, {&S31015[37], 42}, {NULL}};
const Segment SGI343437[] = {{&S31015[12], 14}, {NULL}};
const Segment SGI343438[] = {{&S31016[6], 4}, {NULL}};
const Segment SGI343439[] = {{&S31016[52], 5}, {NULL}};
const Segment SGI343440[] = {{&S31016[12], 55}, {NULL}};
const Segment SGI343441[] = {{&S31016[12], 14}, {NULL}};
const Segment SGI343442[] = {{&S31017[6], 4}, {NULL}};
const Segment SGI343443[] = {{&S31017[75], 5}, {NULL}};
const Segment SGI343444[] = {{&S31017[12], 14}, {&S31017[37], 43}, {NULL}};
const Segment SGI343445[] = {{&S31017[12], 14}, {NULL}};
const Segment SGI343446[] = {{&S31019[6], 4}, {NULL}};
const Segment SGI343447[] = {{&S31019[74], 5}, {NULL}};
const Segment SGI343448[] = {{&S31019[12], 14}, {&S31019[37], 42}, {NULL}};
const Segment SGI343449[] = {{&S31019[12], 14}, {NULL}};
const Segment SGI343450[] = {{&S31020[6], 4}, {NULL}};
const Segment SGI343451[] = {{&S31020[74], 5}, {NULL}};
const Segment SGI343452[] = {{&S31020[12], 14}, {&S31020[37], 42}, {NULL}};
const Segment SGI343453[] = {{&S31020[12], 14}, {NULL}};
const Segment SGI343454[] = {{&S57600[5], 4}, {NULL}};
const Segment SGI343455[] = {{&S57600[15], 3}, {NULL}};
const Segment SGI343456[] = {{&S31021[6], 4}, {NULL}};
const Segment SGI343457[] = {{&S31021[75], 5}, {NULL}};
const Segment SGI343458[] = {{&S31021[12], 14}, {&S31021[37], 43}, {NULL}};
const Segment SGI343459[] = {{&S31021[12], 14}, {NULL}};
const Segment SGI343460[] = {{&S57601[35], 4}, {NULL}};
const Segment SGI343461[] = {{&S31022[6], 4}, {NULL}};
const Segment SGI343462[] = {{&S31022[75], 5}, {NULL}};
const Segment SGI343463[] = {{&S57601[90], 3}, {NULL}};
const Segment SGI343464[] = {{&S31022[12], 14}, {&S31022[38], 42}, {NULL}};
const Segment SGI343465[] = {{&S31022[12], 14}, {NULL}};
const Segment SGI343466[] = {{&S31024[6], 4}, {NULL}};
const Segment SGI343467[] = {{&S31024[75], 5}, {NULL}};
const Segment SGI343468[] = {{&S31024[12], 14}, {&S31024[38], 42}, {NULL}};
const Segment SGI343469[] = {{&S31024[12], 14}, {NULL}};
const Segment SGI343470[] = {{&S31025[6], 4}, {NULL}};
const Segment SGI343471[] = {{&S31025[74], 5}, {NULL}};
const Segment SGI343472[] = {{&S31025[12], 14}, {&S31025[37], 42}, {NULL}};
const Segment SGI343473[] = {{&S31025[12], 14}, {NULL}};
const Segment SGI343474[] = {{&S31026[6], 4}, {NULL}};
const Segment SGI343475[] = {{&S31026[75], 5}, {NULL}};
const Segment SGI343476[] = {{&S31026[12], 14}, {&S31026[37], 43}, {NULL}};
const Segment SGI343477[] = {{&S31026[12], 14}, {NULL}};
const Segment SGI343478[] = {{&S31027[6], 4}, {NULL}};
const Segment SGI343479[] = {{&S31027[74], 5}, {NULL}};
const Segment SGI343480[] = {{&S31027[12], 14}, {&S31027[37], 42}, {NULL}};
const Segment SGI343481[] = {{&S31027[12], 14}, {NULL}};
const Segment SGI343482[] = {{&S31028[6], 4}, {NULL}};
const Segment SGI343483[] = {{&S31028[101], 5}, {NULL}};
const Segment SGI343484[] = {{&S31028[12], 14}, {&S31028[37], 12}, {NULL}};
const Segment SGI343485[] = {{&S31028[12], 14}, {NULL}};
const Segment SGI343486[] = {{&S57610[5], 5}, {&S57610[105], 5}, {NULL}};
const Segment SGI343487[] = {{&S31029[6], 4}, {NULL}};
const Segment SGI343488[] = {{&S31029[74], 5}, {NULL}};
const Segment SGI343489[] = {{&S31029[12], 14}, {&S31029[37], 42}, {NULL}};
const Segment SGI343490[] = {{&S57610[69], 3}, {NULL}};
const Segment SGI343491[] = {{&S31029[12], 14}, {NULL}};
const Segment SGI343492[] = {{&S57610[131], 3}, {NULL}};
const Segment SGI343493[] = {{&S31030[6], 4}, {NULL}};
const Segment SGI343494[] = {{&S31030[74], 5}, {NULL}};
const Segment SGI343495[] = {{&S31030[12], 14}, {&S31030[37], 42}, {NULL}};
const Segment SGI343496[] = {{&S31030[12], 14}, {NULL}};
const Segment SGI343497[] = {{&S31031[6], 4}, {NULL}};
const Segment SGI343498[] = {{&S31031[74], 5}, {NULL}};
const Segment SGI343499[] = {{&S31031[12], 14}, {&S31031[37], 42}, {NULL}};
const Segment SGI343500[] = {{&S31031[12], 14}, {NULL}};
const Segment SGI343501[] = {{&S31032[6], 4}, {NULL}};
const Segment SGI343502[] = {{&S31032[74], 5}, {NULL}};
const Segment SGI343503[] = {{&S31032[12], 14}, {&S31032[37], 42}, {NULL}};
const Segment SGI343504[] = {{&S31032[12], 14}, {NULL}};
const Segment SGI343505[] = {{&S31033[6], 4}, {NULL}};
const Segment SGI343506[] = {{&S31033[74], 5}, {NULL}};
const Segment SGI343507[] = {{&S31033[12], 14}, {&S31033[37], 42}, {NULL}};
const Segment SGI343508[] = {{&S31033[12], 14}, {NULL}};
const Segment SGI343509[] = {{&S31034[6], 4}, {NULL}};
const Segment SGI343510[] = {{&S31034[74], 5}, {NULL}};
const Segment SGI343511[] = {{&S31034[12], 14}, {&S31034[37], 42}, {NULL}};
const Segment SGI343512[] = {{&S31034[12], 14}, {NULL}};
const Segment SGI343513[] = {{&S31036[6], 4}, {NULL}};
const Segment SGI343514[] = {{&S31036[74], 5}, {NULL}};
const Segment SGI343515[] = {{&S31036[12], 14}, {&S31036[37], 42}, {NULL}};
const Segment SGI343516[] = {{&S31036[12], 14}, {NULL}};
const Segment SGI343517[] = {{&S31037[6], 4}, {NULL}};
const Segment SGI343518[] = {{&S31037[74], 5}, {NULL}};
const Segment SGI343519[] = {{&S31037[12], 14}, {&S31037[37], 42}, {NULL}};
const Segment SGI343520[] = {{&S31037[12], 14}, {NULL}};
const Segment SGI343521[] = {{&S31038[6], 4}, {NULL}};
const Segment SGI343522[] = {{&S31038[74], 5}, {NULL}};
const Segment SGI343523[] = {{&S31038[12], 14}, {&S31038[37], 42}, {NULL}};
const Segment SGI343524[] = {{&S31038[12], 14}, {NULL}};
const Segment SGI343525[] = {{&S31039[6], 4}, {NULL}};
const Segment SGI343526[] = {{&S31039[75], 5}, {NULL}};
const Segment SGI343527[] = {{&S31039[12], 14}, {&S31039[37], 43}, {NULL}};
const Segment SGI343528[] = {{&S31039[12], 14}, {NULL}};
const Segment SGI343529[] = {{&S31040[6], 4}, {NULL}};
const Segment SGI343530[] = {{&S31040[74], 5}, {NULL}};
const Segment SGI343531[] = {{&S31040[12], 14}, {&S31040[37], 42}, {NULL}};
const Segment SGI343532[] = {{&S31040[12], 14}, {NULL}};
const Segment SGI343533[] = {{&S31041[6], 4}, {NULL}};
const Segment SGI343534[] = {{&S31041[74], 5}, {NULL}};
const Segment SGI343535[] = {{&S31041[12], 14}, {&S31041[37], 42}, {NULL}};
const Segment SGI343536[] = {{&S31041[12], 14}, {NULL}};
const Segment SGI343537[] = {{&S31042[6], 4}, {NULL}};
const Segment SGI343538[] = {{&S31042[74], 5}, {NULL}};
const Segment SGI343539[] = {{&S31042[12], 14}, {&S31042[37], 42}, {NULL}};
const Segment SGI343540[] = {{&S31042[12], 14}, {NULL}};
const Segment SGI343541[] = {{&S31043[6], 4}, {NULL}};
const Segment SGI343542[] = {{&S31043[74], 5}, {NULL}};
const Segment SGI343543[] = {{&S31043[12], 14}, {&S31043[37], 42}, {NULL}};
const Segment SGI343544[] = {{&S31043[12], 14}, {NULL}};
const Segment SGI343545[] = {{&S31044[6], 4}, {NULL}};
const Segment SGI343546[] = {{&S31044[74], 5}, {NULL}};
const Segment SGI343547[] = {{&S31044[12], 14}, {&S31044[37], 42}, {NULL}};
const Segment SGI343548[] = {{&S31044[12], 14}, {NULL}};
const Segment SGI343549[] = {{&S31045[6], 4}, {NULL}};
const Segment SGI343550[] = {{&S31045[74], 5}, {NULL}};
const Segment SGI343551[] = {{&S31045[12], 14}, {&S31045[37], 42}, {NULL}};
const Segment SGI343552[] = {{&S31045[12], 14}, {NULL}};
const Segment SGI343553[] = {{&S31046[6], 4}, {NULL}};
const Segment SGI343554[] = {{&S31046[74], 5}, {NULL}};
const Segment SGI343555[] = {{&S31046[12], 14}, {&S31046[37], 42}, {NULL}};
const Segment SGI343556[] = {{&S31046[12], 14}, {NULL}};
const Segment SGI343557[] = {{&S31047[6], 4}, {NULL}};
const Segment SGI343558[] = {{&S31047[75], 5}, {NULL}};
const Segment SGI343559[] = {{&S31047[12], 14}, {&S31047[37], 43}, {NULL}};
const Segment SGI343560[] = {{&S31047[12], 14}, {NULL}};
const Segment SGI343561[] = {{&S31048[6], 4}, {NULL}};
const Segment SGI343562[] = {{&S31048[74], 5}, {NULL}};
const Segment SGI343563[] = {{&S31048[12], 14}, {&S31048[37], 42}, {NULL}};
const Segment SGI343564[] = {{&S31048[12], 14}, {NULL}};
const Segment SGI343565[] = {{&S31049[6], 4}, {NULL}};
const Segment SGI343566[] = {{&S31049[74], 5}, {NULL}};
const Segment SGI343567[] = {{&S31049[12], 14}, {&S31049[37], 42}, {NULL}};
const Segment SGI343568[] = {{&S31049[12], 14}, {NULL}};
const Segment SGI343569[] = {{&S31050[6], 4}, {NULL}};
const Segment SGI343570[] = {{&S31050[74], 5}, {NULL}};
const Segment SGI343571[] = {{&S31050[12], 14}, {&S31050[37], 42}, {NULL}};
const Segment SGI343572[] = {{&S31050[12], 14}, {NULL}};
const Segment SGI343573[] = {{&S31051[6], 4}, {NULL}};
const Segment SGI343574[] = {{&S31051[74], 5}, {NULL}};
const Segment SGI343575[] = {{&S31051[12], 14}, {&S31051[37], 42}, {NULL}};
const Segment SGI343576[] = {{&S31051[12], 14}, {NULL}};
const Segment SGI343577[] = {{&S31052[6], 4}, {NULL}};
const Segment SGI343578[] = {{&S31052[73], 5}, {NULL}};
const Segment SGI343579[] = {{&S31052[12], 14}, {&S31052[84], 4}, {NULL}};
const Segment SGI343580[] = {{&S31052[12], 14}, {NULL}};
const Segment SGI343581[] = {{&S57638[100], 19}, {NULL}};
const Segment SGI343582[] = {{&S31053[6], 4}, {NULL}};
const Segment SGI343583[] = {{&S31053[74], 5}, {NULL}};
const Segment SGI343584[] = {{&S31053[12], 14}, {&S31053[37], 42}, {NULL}};
const Segment SGI343585[] = {{&S31053[12], 14}, {NULL}};
const Segment SGI343586[] = {{&S57639[100], 21}, {NULL}};
const Segment SGI343587[] = {{&S31054[6], 4}, {NULL}};
const Segment SGI343588[] = {{&S31054[75], 5}, {NULL}};
const Segment SGI343589[] = {{&S31054[12], 14}, {&S31054[37], 43}, {NULL}};
const Segment SGI343590[] = {{&S31054[12], 14}, {NULL}};
const Segment SGI343591[] = {{&S57640[100], 11}, {NULL}};
const Segment SGI343592[] = {{&S31055[6], 4}, {NULL}};
const Segment SGI343593[] = {{&S31055[74], 5}, {NULL}};
const Segment SGI343594[] = {{&S31055[12], 14}, {&S31055[37], 42}, {NULL}};
const Segment SGI343595[] = {{&S31055[12], 14}, {NULL}};
const Segment SGI343596[] = {{&S57641[100], 18}, {NULL}};
const Segment SGI343597[] = {{&S31056[6], 4}, {NULL}};
const Segment SGI343598[] = {{&S31056[74], 5}, {NULL}};
const Segment SGI343599[] = {{&S31056[12], 14}, {&S31056[37], 42}, {NULL}};
const Segment SGI343600[] = {{&S31056[12], 14}, {NULL}};
const Segment SGI343601[] = {{&S57642[100], 19}, {NULL}};
const Segment SGI343602[] = {{&S31058[6], 4}, {NULL}};
const Segment SGI343603[] = {{&S31058[74], 5}, {NULL}};
const Segment SGI343604[] = {{&S31058[12], 14}, {&S31058[37], 42}, {NULL}};
const Segment SGI343605[] = {{&S31058[12], 14}, {NULL}};
const Segment SGI343606[] = {{&S31059[6], 4}, {NULL}};
const Segment SGI343607[] = {{&S31059[74], 5}, {NULL}};
const Segment SGI343608[] = {{&S31059[12], 14}, {&S31059[37], 42}, {NULL}};
const Segment SGI343609[] = {{&S31059[12], 14}, {NULL}};
const Segment SGI343610[] = {{&S31060[6], 4}, {NULL}};
const Segment SGI343611[] = {{&S31060[74], 5}, {NULL}};
const Segment SGI343612[] = {{&S31060[12], 14}, {&S31060[37], 42}, {NULL}};
const Segment SGI343613[] = {{&S31060[12], 14}, {NULL}};
const Segment SGI343614[] = {{&S31061[6], 4}, {NULL}};
const Segment SGI343615[] = {{&S31061[75], 5}, {NULL}};
const Segment SGI343616[] = {{&S31061[12], 14}, {&S31061[37], 43}, {NULL}};
const Segment SGI343617[] = {{&S31061[12], 14}, {NULL}};
const Segment SGI343618[] = {{&S31062[6], 4}, {NULL}};
const Segment SGI343619[] = {{&S31062[74], 5}, {NULL}};
const Segment SGI343620[] = {{&S31062[12], 14}, {&S31062[37], 42}, {NULL}};
const Segment SGI343621[] = {{&S31062[12], 14}, {NULL}};
const Segment SGI343622[] = {{&S31063[6], 4}, {NULL}};
const Segment SGI343623[] = {{&S31063[74], 5}, {NULL}};
const Segment SGI343624[] = {{&S31063[12], 14}, {&S31063[37], 42}, {NULL}};
const Segment SGI343625[] = {{&S31063[12], 14}, {NULL}};
const Segment SGI343626[] = {{&S31064[6], 4}, {NULL}};
const Segment SGI343627[] = {{&S31064[74], 5}, {NULL}};
const Segment SGI343628[] = {{&S31064[12], 14}, {&S31064[37], 42}, {NULL}};
const Segment SGI343629[] = {{&S31064[12], 14}, {NULL}};
const Segment SGI343630[] = {{&S31065[6], 4}, {NULL}};
const Segment SGI343631[] = {{&S31065[74], 5}, {NULL}};
const Segment SGI343632[] = {{&S31065[12], 14}, {&S31065[37], 42}, {NULL}};
const Segment SGI343633[] = {{&S31065[12], 14}, {NULL}};
const Segment SGI343634[] = {{&S57681[5], 5}, {NULL}};
const Segment SGI343635[] = {{&S48827[28], 5}, {NULL}};
const Segment SGI343636[] = {{&S48827[104], 5}, {NULL}};
const Segment SGI343637[] = {{&S57714[38], 5}, {NULL}};
const Segment SGI343638[] = {{&S57714[94], 3}, {NULL}};
const Segment SGI343639[] = {{&S57715[41], 5}, {NULL}};
const Segment SGI343640[] = {{&S57715[6], 3}, {NULL}};
const Segment SGI343641[] = {{&S57715[67], 3}, {NULL}};
const Segment SGI343642[] = {{&S57716[5], 14}, {NULL}};
const Segment SGI343643[] = {{&S57716[20], 3}, {NULL}};
const Segment SGI343644[] = {{&S57718[5], 4}, {&S57718[5], 4}, {NULL}};
const Segment SGI343645[] = {{&S57719[5], 4}, {NULL}};
const Segment SGI343646[] = {{&S57724[5], 5}, {&S57724[97], 5}, {NULL}};
const Segment SGI343647[] = {{&S57724[153], 3}, {NULL}};
const Segment SGI343648[] = {{&S30361[0], 22}, {NULL}};
const Segment SGI343649[] = {{&S30361[35], 3}, {NULL}};
const Segment SGI343650[] = {{&S30361[76], 13}, {NULL}};
const Segment SGI343651[] = {{&S57753[42], 7}, {NULL}};
const Segment SGI343652[] = {{&S57753[6], 3}, {NULL}};
const Segment SGI343653[] = {{&S57753[70], 3}, {NULL}};
const Segment SGI343654[] = {{&S57754[43], 7}, {NULL}};
const Segment SGI343655[] = {{&S57754[6], 3}, {NULL}};
const Segment SGI343656[] = {{&S57754[71], 3}, {NULL}};
const Segment SGI343657[] = {{&S57755[43], 5}, {NULL}};
const Segment SGI343658[] = {{&S57755[6], 3}, {NULL}};
const Segment SGI343659[] = {{&S57755[69], 3}, {NULL}};
const Segment SGI343660[] = {{&S57756[42], 5}, {NULL}};
const Segment SGI343661[] = {{&S57756[6], 3}, {NULL}};
const Segment SGI343662[] = {{&S57756[68], 3}, {NULL}};
const Segment SGI343663[] = {{&S32940[61], 5}, {NULL}};
const Segment SGI343664[] = {{&S32940[13], 10}, {NULL}};
const Segment SGI343665[] = {{&S57844[5], 5}, {NULL}};
const Segment SGI343666[] = {{&S31279[6], 4}, {NULL}};
const Segment SGI343667[] = {{&S31279[58], 5}, {NULL}};
const Segment SGI343668[] = {{&S31279[12], 14}, {NULL}};
const Segment SGI343669[] = {{&S31280[6], 4}, {NULL}};
const Segment SGI343670[] = {{&S31280[80], 5}, {NULL}};
const Segment SGI343671[] = {{&S31280[12], 14}, {&S31280[40], 11}, {NULL}};
const Segment SGI343672[] = {{&S31280[12], 14}, {NULL}};
const Segment SGI343673[] = {{&S31281[6], 4}, {NULL}};
const Segment SGI343674[] = {{&S31281[58], 5}, {NULL}};
const Segment SGI343675[] = {{&S31281[12], 14}, {NULL}};
const Segment SGI343676[] = {{&S31282[6], 4}, {NULL}};
const Segment SGI343677[] = {{&S31282[81], 5}, {NULL}};
const Segment SGI343678[] = {{&S31282[12], 14}, {&S31282[40], 12}, {NULL}};
const Segment SGI343679[] = {{&S31282[12], 14}, {NULL}};
const Segment SGI343680[] = {{&S31283[6], 4}, {NULL}};
const Segment SGI343681[] = {{&S31283[80], 5}, {NULL}};
const Segment SGI343682[] = {{&S31283[12], 14}, {&S31283[40], 11}, {NULL}};
const Segment SGI343683[] = {{&S31283[12], 14}, {NULL}};
const Segment SGI343684[] = {{&S31284[6], 4}, {NULL}};
const Segment SGI343685[] = {{&S31284[81], 5}, {NULL}};
const Segment SGI343686[] = {{&S31284[12], 14}, {&S31284[40], 12}, {NULL}};
const Segment SGI343687[] = {{&S31284[12], 14}, {NULL}};
const Segment SGI343688[] = {{&S31285[6], 4}, {NULL}};
const Segment SGI343689[] = {{&S31285[58], 5}, {NULL}};
const Segment SGI343690[] = {{&S31285[12], 14}, {&S31285[69], 31}, {NULL}};
const Segment SGI343691[] = {{&S31285[12], 14}, {NULL}};
const Segment SGI343692[] = {{&S33023[61], 5}, {NULL}};
const Segment SGI343693[] = {{&S33023[13], 10}, {NULL}};
const Segment SGI343694[] = {{&S31286[6], 4}, {NULL}};
const Segment SGI343695[] = {{&S31286[80], 5}, {NULL}};
const Segment SGI343696[] = {{&S31286[12], 14}, {&S31286[40], 11}, {NULL}};
const Segment SGI343697[] = {{&S31286[12], 14}, {NULL}};
const Segment SGI343698[] = {{&S57884[5], 5}, {NULL}};
const Segment SGI343699[] = {{&S31287[6], 4}, {NULL}};
const Segment SGI343700[] = {{&S31287[81], 5}, {NULL}};
const Segment SGI343701[] = {{&S31287[12], 14}, {&S31287[40], 12}, {NULL}};
const Segment SGI343702[] = {{&S31287[12], 14}, {NULL}};
const Segment SGI343703[] = {{&S57885[5], 7}, {NULL}};
const Segment SGI343704[] = {{&S31288[6], 4}, {NULL}};
const Segment SGI343705[] = {{&S31288[80], 5}, {NULL}};
const Segment SGI343706[] = {{&S31288[12], 14}, {&S31288[40], 11}, {NULL}};
const Segment SGI343707[] = {{&S31288[12], 14}, {NULL}};
const Segment SGI343708[] = {{&S31289[6], 4}, {NULL}};
const Segment SGI343709[] = {{&S31289[80], 5}, {NULL}};
const Segment SGI343710[] = {{&S31289[12], 14}, {&S31289[40], 11}, {NULL}};
const Segment SGI343711[] = {{&S31289[12], 14}, {NULL}};
const Segment SGI343712[] = {{&S31290[6], 4}, {NULL}};
const Segment SGI343713[] = {{&S31290[81], 5}, {NULL}};
const Segment SGI343714[] = {{&S31290[12], 14}, {&S31290[40], 12}, {NULL}};
const Segment SGI343715[] = {{&S31290[12], 14}, {NULL}};
const Segment SGI343716[] = {{&S31291[6], 4}, {NULL}};
const Segment SGI343717[] = {{&S31291[81], 5}, {NULL}};
const Segment SGI343718[] = {{&S31291[12], 14}, {&S31291[40], 12}, {NULL}};
const Segment SGI343719[] = {{&S31291[12], 14}, {NULL}};
const Segment SGI343720[] = {{&S31292[6], 4}, {NULL}};
const Segment SGI343721[] = {{&S31292[81], 5}, {NULL}};
const Segment SGI343722[] = {{&S31292[12], 14}, {&S31292[40], 12}, {NULL}};
const Segment SGI343723[] = {{&S31292[12], 14}, {NULL}};
const Segment SGI343724[] = {{&S57891[42], 5}, {NULL}};
const Segment SGI343725[] = {{&S57891[6], 3}, {NULL}};
const Segment SGI343726[] = {{&S57891[68], 3}, {NULL}};
const Segment SGI343727[] = {{&S31293[6], 4}, {NULL}};
const Segment SGI343728[] = {{&S31293[81], 5}, {NULL}};
const Segment SGI343729[] = {{&S31293[12], 14}, {&S31293[40], 12}, {NULL}};
const Segment SGI343730[] = {{&S31293[12], 14}, {NULL}};
const Segment SGI343731[] = {{&S57892[5], 5}, {NULL}};
const Segment SGI343732[] = {{&S57892[16], 3}, {NULL}};
const Segment SGI343733[] = {{&S31294[6], 4}, {NULL}};
const Segment SGI343734[] = {{&S31294[107], 5}, {NULL}};
const Segment SGI343735[] = {{&S57893[41], 5}, {NULL}};
const Segment SGI343736[] = {{&S31294[12], 14}, {&S31294[40], 12}, {NULL}};
const Segment SGI343737[] = {{&S57893[6], 3}, {NULL}};
const Segment SGI343738[] = {{&S31294[12], 14}, {NULL}};
const Segment SGI343739[] = {{&S57893[67], 3}, {NULL}};
const Segment SGI343740[] = {{&S31295[6], 4}, {NULL}};
const Segment SGI343741[] = {{&S31295[80], 5}, {NULL}};
const Segment SGI343742[] = {{&S31295[12], 14}, {&S31295[40], 11}, {NULL}};
const Segment SGI343743[] = {{&S31295[12], 14}, {NULL}};
const Segment SGI343744[] = {{&S31296[6], 4}, {NULL}};
const Segment SGI343745[] = {{&S31296[80], 5}, {NULL}};
const Segment SGI343746[] = {{&S31296[12], 14}, {&S31296[40], 11}, {NULL}};
const Segment SGI343747[] = {{&S31296[12], 14}, {NULL}};
const Segment SGI343748[] = {{&S31297[6], 4}, {NULL}};
const Segment SGI343749[] = {{&S31297[80], 5}, {NULL}};
const Segment SGI343750[] = {{&S31297[12], 14}, {&S31297[40], 11}, {NULL}};
const Segment SGI343751[] = {{&S31297[12], 14}, {NULL}};
const Segment SGI343752[] = {{&S31298[6], 4}, {NULL}};
const Segment SGI343753[] = {{&S31298[81], 5}, {NULL}};
const Segment SGI343754[] = {{&S31298[12], 14}, {&S31298[40], 12}, {NULL}};
const Segment SGI343755[] = {{&S31298[12], 14}, {NULL}};
const Segment SGI343756[] = {{&S31299[6], 4}, {NULL}};
const Segment SGI343757[] = {{&S31299[80], 5}, {NULL}};
const Segment SGI343758[] = {{&S31299[12], 14}, {&S31299[40], 11}, {NULL}};
const Segment SGI343759[] = {{&S31299[12], 14}, {NULL}};
const Segment SGI343760[] = {{&S31300[6], 4}, {NULL}};
const Segment SGI343761[] = {{&S31300[80], 5}, {NULL}};
const Segment SGI343762[] = {{&S31300[12], 14}, {&S31300[40], 11}, {NULL}};
const Segment SGI343763[] = {{&S31300[12], 14}, {NULL}};
const Segment SGI343764[] = {{&S31301[6], 4}, {NULL}};
const Segment SGI343765[] = {{&S31301[81], 5}, {NULL}};
const Segment SGI343766[] = {{&S31301[12], 14}, {&S31301[40], 12}, {NULL}};
const Segment SGI343767[] = {{&S31301[12], 14}, {NULL}};
const Segment SGI343768[] = {{&S31302[6], 4}, {NULL}};
const Segment SGI343769[] = {{&S31302[81], 5}, {NULL}};
const Segment SGI343770[] = {{&S31302[12], 14}, {&S31302[40], 12}, {NULL}};
const Segment SGI343771[] = {{&S31302[12], 14}, {NULL}};
const Segment SGI343772[] = {{&S31303[6], 4}, {NULL}};
const Segment SGI343773[] = {{&S31303[80], 5}, {NULL}};
const Segment SGI343774[] = {{&S31303[12], 14}, {&S31303[40], 11}, {NULL}};
const Segment SGI343775[] = {{&S31303[12], 14}, {NULL}};
const Segment SGI343776[] = {{&S31304[6], 4}, {NULL}};
const Segment SGI343777[] = {{&S31304[80], 5}, {NULL}};
const Segment SGI343778[] = {{&S31304[12], 14}, {&S31304[40], 11}, {NULL}};
const Segment SGI343779[] = {{&S31304[12], 14}, {NULL}};
const Segment SGI343780[] = {{&S31305[6], 4}, {NULL}};
const Segment SGI343781[] = {{&S31305[81], 5}, {NULL}};
const Segment SGI343782[] = {{&S31305[12], 14}, {&S31305[40], 12}, {NULL}};
const Segment SGI343783[] = {{&S31305[12], 14}, {NULL}};
const Segment SGI343784[] = {{&S31306[6], 4}, {NULL}};
const Segment SGI343785[] = {{&S31306[80], 5}, {NULL}};
const Segment SGI343786[] = {{&S31306[12], 14}, {&S31306[40], 11}, {NULL}};
const Segment SGI343787[] = {{&S31306[12], 14}, {NULL}};
const Segment SGI343788[] = {{&S31307[6], 4}, {NULL}};
const Segment SGI343789[] = {{&S31307[81], 5}, {NULL}};
const Segment SGI343790[] = {{&S31307[12], 14}, {&S31307[41], 11}, {NULL}};
const Segment SGI343791[] = {{&S31307[12], 14}, {NULL}};
const Segment SGI343792[] = {{&S31308[6], 4}, {NULL}};
const Segment SGI343793[] = {{&S31308[81], 5}, {NULL}};
const Segment SGI343794[] = {{&S31308[12], 14}, {&S31308[41], 11}, {NULL}};
const Segment SGI343795[] = {{&S31308[12], 14}, {NULL}};
const Segment SGI343796[] = {{&S31310[6], 4}, {NULL}};
const Segment SGI343797[] = {{&S31310[80], 5}, {NULL}};
const Segment SGI343798[] = {{&S31310[12], 14}, {&S31310[40], 11}, {NULL}};
const Segment SGI343799[] = {{&S31310[12], 14}, {NULL}};
const Segment SGI343800[] = {{&S31311[6], 4}, {NULL}};
const Segment SGI343801[] = {{&S31311[80], 5}, {NULL}};
const Segment SGI343802[] = {{&S31311[12], 14}, {&S31311[40], 11}, {NULL}};
const Segment SGI343803[] = {{&S31311[12], 14}, {NULL}};
const Segment SGI343804[] = {{&S31313[6], 4}, {NULL}};
const Segment SGI343805[] = {{&S57916[5], 8}, {NULL}};
const Segment SGI343806[] = {{&S31313[81], 5}, {NULL}};
const Segment SGI343807[] = {{&S31313[12], 14}, {&S31313[40], 12}, {NULL}};
const Segment SGI343808[] = {{&S31313[12], 14}, {NULL}};
const Segment SGI343809[] = {{&S31314[6], 4}, {NULL}};
const Segment SGI343810[] = {{&S31314[81], 5}, {NULL}};
const Segment SGI343811[] = {{&S31314[12], 14}, {&S31314[40], 12}, {NULL}};
const Segment SGI343812[] = {{&S31314[12], 14}, {NULL}};
const Segment SGI343813[] = {{&S31315[6], 4}, {NULL}};
const Segment SGI343814[] = {{&S31315[81], 5}, {NULL}};
const Segment SGI343815[] = {{&S31315[12], 14}, {&S31315[41], 11}, {NULL}};
const Segment SGI343816[] = {{&S31315[12], 14}, {NULL}};
const Segment SGI343817[] = {{&S31316[6], 4}, {NULL}};
const Segment SGI343818[] = {{&S31316[81], 5}, {NULL}};
const Segment SGI343819[] = {{&S31316[12], 14}, {&S31316[41], 11}, {NULL}};
const Segment SGI343820[] = {{&S31316[12], 14}, {NULL}};
const Segment SGI343821[] = {{&S31317[6], 4}, {NULL}};
const Segment SGI343822[] = {{&S31317[80], 5}, {NULL}};
const Segment SGI343823[] = {{&S31317[12], 14}, {&S31317[40], 11}, {NULL}};
const Segment SGI343824[] = {{&S31317[12], 14}, {NULL}};
const Segment SGI343825[] = {{&S31318[6], 4}, {NULL}};
const Segment SGI343826[] = {{&S31318[81], 5}, {NULL}};
const Segment SGI343827[] = {{&S31318[12], 14}, {&S31318[40], 12}, {NULL}};
const Segment SGI343828[] = {{&S31318[12], 14}, {NULL}};
const Segment SGI343829[] = {{&S31319[6], 4}, {NULL}};
const Segment SGI343830[] = {{&S31319[80], 5}, {NULL}};
const Segment SGI343831[] = {{&S31319[12], 14}, {&S31319[40], 11}, {NULL}};
const Segment SGI343832[] = {{&S31319[12], 14}, {NULL}};
const Segment SGI343833[] = {{&S31320[6], 4}, {NULL}};
const Segment SGI343834[] = {{&S31320[81], 5}, {NULL}};
const Segment SGI343835[] = {{&S31320[12], 14}, {&S31320[41], 11}, {NULL}};
const Segment SGI343836[] = {{&S31320[12], 14}, {NULL}};
const Segment SGI343837[] = {{&S31321[6], 4}, {NULL}};
const Segment SGI343838[] = {{&S31321[80], 5}, {NULL}};
const Segment SGI343839[] = {{&S31321[12], 14}, {&S31321[40], 11}, {NULL}};
const Segment SGI343840[] = {{&S31321[12], 14}, {NULL}};
const Segment SGI343841[] = {{&S31322[6], 4}, {NULL}};
const Segment SGI343842[] = {{&S31322[80], 5}, {NULL}};
const Segment SGI343843[] = {{&S31322[12], 14}, {&S31322[40], 11}, {NULL}};
const Segment SGI343844[] = {{&S31322[12], 14}, {NULL}};
const Segment SGI343845[] = {{&S31323[6], 4}, {NULL}};
const Segment SGI343846[] = {{&S31323[80], 5}, {NULL}};
const Segment SGI343847[] = {{&S31323[12], 14}, {&S31323[40], 11}, {NULL}};
const Segment SGI343848[] = {{&S31323[12], 14}, {NULL}};
const Segment SGI343849[] = {{&S57929[42], 5}, {NULL}};
const Segment SGI343850[] = {{&S57929[6], 3}, {NULL}};
const Segment SGI343851[] = {{&S57929[68], 3}, {NULL}};
const Segment SGI343852[] = {{&S57930[43], 3}, {NULL}};
const Segment SGI343853[] = {{&S57930[6], 3}, {NULL}};
const Segment SGI343854[] = {{&S57930[67], 3}, {NULL}};
const Segment SGI343855[] = {{&S57931[5], 3}, {NULL}};
const Segment SGI343856[] = {{&S57931[14], 3}, {NULL}};
const Segment SGI343857[] = {{&S31324[6], 4}, {NULL}};
const Segment SGI343858[] = {{&S31324[58], 5}, {NULL}};
const Segment SGI343859[] = {{&S31324[12], 14}, {&S31324[69], 4}, {NULL}};
const Segment SGI343860[] = {{&S31324[12], 14}, {NULL}};
const Segment SGI343861[] = {{&S31325[6], 4}, {NULL}};
const Segment SGI343862[] = {{&S31325[80], 5}, {NULL}};
const Segment SGI343863[] = {{&S31325[12], 14}, {&S31325[40], 11}, {NULL}};
const Segment SGI343864[] = {{&S31325[12], 14}, {NULL}};
const Segment SGI343865[] = {{&S31326[6], 4}, {NULL}};
const Segment SGI343866[] = {{&S31326[80], 5}, {NULL}};
const Segment SGI343867[] = {{&S31326[12], 14}, {&S31326[40], 11}, {NULL}};
const Segment SGI343868[] = {{&S31326[12], 14}, {NULL}};
const Segment SGI343869[] = {{&S31327[6], 4}, {NULL}};
const Segment SGI343870[] = {{&S31327[80], 5}, {NULL}};
const Segment SGI343871[] = {{&S31327[12], 14}, {&S31327[40], 11}, {NULL}};
const Segment SGI343872[] = {{&S31327[12], 14}, {NULL}};
const Segment SGI343873[] = {{&S31328[6], 4}, {NULL}};
const Segment SGI343874[] = {{&S31328[80], 5}, {NULL}};
const Segment SGI343875[] = {{&S31328[12], 14}, {&S31328[40], 11}, {NULL}};
const Segment SGI343876[] = {{&S31328[12], 14}, {NULL}};
const Segment SGI343877[] = {{&S31329[6], 4}, {NULL}};
const Segment SGI343878[] = {{&S31329[80], 5}, {NULL}};
const Segment SGI343879[] = {{&S31329[12], 14}, {&S31329[40], 11}, {NULL}};
const Segment SGI343880[] = {{&S31329[12], 14}, {NULL}};
const Segment SGI343881[] = {{&S31330[6], 4}, {NULL}};
const Segment SGI343882[] = {{&S31330[80], 5}, {NULL}};
const Segment SGI343883[] = {{&S31330[12], 14}, {&S31330[40], 11}, {NULL}};
const Segment SGI343884[] = {{&S31330[12], 14}, {NULL}};
const Segment SGI343885[] = {{&S31331[6], 4}, {NULL}};
const Segment SGI343886[] = {{&S31331[80], 5}, {NULL}};
const Segment SGI343887[] = {{&S31331[12], 14}, {&S31331[40], 11}, {NULL}};
const Segment SGI343888[] = {{&S31331[12], 14}, {NULL}};
const Segment SGI343889[] = {{&S31342[6], 4}, {NULL}};
const Segment SGI343890[] = {{&S31342[82], 5}, {NULL}};
const Segment SGI343891[] = {{&S31342[12], 14}, {&S31342[40], 12}, {NULL}};
const Segment SGI343892[] = {{&S31342[12], 14}, {NULL}};
const Segment SGI343893[] = {{&S31343[6], 4}, {NULL}};
const Segment SGI343894[] = {{&S31343[81], 5}, {NULL}};
const Segment SGI343895[] = {{&S31343[12], 14}, {&S31343[40], 11}, {NULL}};
const Segment SGI343896[] = {{&S31343[12], 14}, {NULL}};
const Segment SGI343897[] = {{&S31344[6], 4}, {NULL}};
const Segment SGI343898[] = {{&S31344[81], 5}, {NULL}};
const Segment SGI343899[] = {{&S31344[12], 14}, {&S31344[40], 11}, {NULL}};
const Segment SGI343900[] = {{&S31344[12], 14}, {NULL}};
const Segment SGI343901[] = {{&S31345[6], 4}, {NULL}};
const Segment SGI343902[] = {{&S31345[81], 5}, {NULL}};
const Segment SGI343903[] = {{&S31345[12], 14}, {&S31345[40], 11}, {NULL}};
const Segment SGI343904[] = {{&S31345[12], 14}, {NULL}};
const Segment SGI343905[] = {{&S31346[6], 4}, {NULL}};
const Segment SGI343906[] = {{&S31346[81], 5}, {NULL}};
const Segment SGI343907[] = {{&S31346[12], 14}, {&S31346[40], 11}, {NULL}};
const Segment SGI343908[] = {{&S31346[12], 14}, {NULL}};
const Segment SGI343909[] = {{&S31347[6], 4}, {NULL}};
const Segment SGI343910[] = {{&S31347[81], 5}, {NULL}};
const Segment SGI343911[] = {{&S31347[12], 14}, {&S31347[40], 11}, {NULL}};
const Segment SGI343912[] = {{&S31347[12], 14}, {NULL}};
const Segment SGI343913[] = {{&S31348[6], 4}, {NULL}};
const Segment SGI343914[] = {{&S31348[81], 5}, {NULL}};
const Segment SGI343915[] = {{&S31348[12], 14}, {&S31348[40], 11}, {NULL}};
const Segment SGI343916[] = {{&S31348[12], 14}, {NULL}};
const Segment SGI343917[] = {{&S31349[6], 4}, {NULL}};
const Segment SGI343918[] = {{&S31349[81], 5}, {NULL}};
const Segment SGI343919[] = {{&S31349[12], 14}, {&S31349[40], 11}, {NULL}};
const Segment SGI343920[] = {{&S31349[12], 14}, {NULL}};
const Segment SGI343921[] = {{&S31350[6], 4}, {NULL}};
const Segment SGI343922[] = {{&S31350[82], 5}, {NULL}};
const Segment SGI343923[] = {{&S31350[12], 14}, {&S31350[40], 12}, {NULL}};
const Segment SGI343924[] = {{&S31350[12], 14}, {NULL}};
const Segment SGI343925[] = {{&S31351[6], 4}, {NULL}};
const Segment SGI343926[] = {{&S31351[82], 5}, {NULL}};
const Segment SGI343927[] = {{&S31351[12], 14}, {&S31351[40], 12}, {NULL}};
const Segment SGI343928[] = {{&S31351[12], 14}, {NULL}};
const Segment SGI343929[] = {{&S31352[6], 4}, {NULL}};
const Segment SGI343930[] = {{&S31352[82], 5}, {NULL}};
const Segment SGI343931[] = {{&S31352[12], 14}, {&S31352[40], 12}, {NULL}};
const Segment SGI343932[] = {{&S31352[12], 14}, {NULL}};
const Segment SGI343933[] = {{&S31353[6], 4}, {NULL}};
const Segment SGI343934[] = {{&S31353[81], 5}, {NULL}};
const Segment SGI343935[] = {{&S31353[12], 14}, {&S31353[40], 11}, {NULL}};
const Segment SGI343936[] = {{&S31353[12], 14}, {NULL}};
const Segment SGI343937[] = {{&S31354[6], 4}, {NULL}};
const Segment SGI343938[] = {{&S31354[82], 5}, {NULL}};
const Segment SGI343939[] = {{&S31354[12], 14}, {&S31354[40], 12}, {NULL}};
const Segment SGI343940[] = {{&S31354[12], 14}, {NULL}};
const Segment SGI343941[] = {{&S31355[6], 4}, {NULL}};
const Segment SGI343942[] = {{&S31355[82], 5}, {NULL}};
const Segment SGI343943[] = {{&S31355[12], 14}, {&S31355[40], 12}, {NULL}};
const Segment SGI343944[] = {{&S31355[12], 14}, {NULL}};
const Segment SGI343945[] = {{&S31356[6], 4}, {NULL}};
const Segment SGI343946[] = {{&S31356[81], 5}, {NULL}};
const Segment SGI343947[] = {{&S31356[12], 14}, {&S31356[40], 11}, {NULL}};
const Segment SGI343948[] = {{&S31356[12], 14}, {NULL}};
const Segment SGI343949[] = {{&S31357[6], 4}, {NULL}};
const Segment SGI343950[] = {{&S31357[81], 5}, {NULL}};
const Segment SGI343951[] = {{&S31357[12], 14}, {&S31357[40], 11}, {NULL}};
const Segment SGI343952[] = {{&S31357[12], 14}, {NULL}};
const Segment SGI343953[] = {{&S31358[6], 4}, {NULL}};
const Segment SGI343954[] = {{&S31358[81], 5}, {NULL}};
const Segment SGI343955[] = {{&S31358[12], 14}, {&S31358[40], 11}, {NULL}};
const Segment SGI343956[] = {{&S31358[12], 14}, {NULL}};
const Segment SGI343957[] = {{&S31359[6], 4}, {NULL}};
const Segment SGI343958[] = {{&S31359[82], 5}, {NULL}};
const Segment SGI343959[] = {{&S31359[12], 14}, {&S31359[40], 12}, {NULL}};
const Segment SGI343960[] = {{&S31359[12], 14}, {NULL}};
const Segment SGI343961[] = {{&S31360[6], 4}, {NULL}};
const Segment SGI343962[] = {{&S31360[81], 5}, {NULL}};
const Segment SGI343963[] = {{&S31360[12], 14}, {&S31360[40], 11}, {NULL}};
const Segment SGI343964[] = {{&S31360[12], 14}, {NULL}};
const Segment SGI343965[] = {{&S31361[6], 4}, {NULL}};
const Segment SGI343966[] = {{&S31361[82], 5}, {NULL}};
const Segment SGI343967[] = {{&S31361[12], 14}, {&S31361[40], 12}, {NULL}};
const Segment SGI343968[] = {{&S31361[12], 14}, {NULL}};
const Segment SGI343969[] = {{&S31362[6], 4}, {NULL}};
const Segment SGI343970[] = {{&S31362[82], 5}, {NULL}};
const Segment SGI343971[] = {{&S31362[12], 14}, {&S31362[40], 12}, {NULL}};
const Segment SGI343972[] = {{&S31362[12], 14}, {NULL}};
const Segment SGI343973[] = {{&S31363[6], 4}, {NULL}};
const Segment SGI343974[] = {{&S31363[82], 5}, {NULL}};
const Segment SGI343975[] = {{&S31363[12], 14}, {&S31363[40], 12}, {NULL}};
const Segment SGI343976[] = {{&S31363[12], 14}, {NULL}};
const Segment SGI343977[] = {{&S31364[6], 4}, {NULL}};
const Segment SGI343978[] = {{&S31364[81], 5}, {NULL}};
const Segment SGI343979[] = {{&S31364[12], 14}, {&S31364[40], 11}, {NULL}};
const Segment SGI343980[] = {{&S31364[12], 14}, {NULL}};
const Segment SGI343981[] = {{&S31365[6], 4}, {NULL}};
const Segment SGI343982[] = {{&S31365[81], 5}, {NULL}};
const Segment SGI343983[] = {{&S31365[12], 14}, {&S31365[40], 11}, {NULL}};
const Segment SGI343984[] = {{&S31365[12], 14}, {NULL}};
const Segment SGI343985[] = {{&S31366[6], 4}, {NULL}};
const Segment SGI343986[] = {{&S31366[82], 5}, {NULL}};
const Segment SGI343987[] = {{&S31366[12], 14}, {&S31366[41], 11}, {NULL}};
const Segment SGI343988[] = {{&S31366[12], 14}, {NULL}};
const Segment SGI343989[] = {{&S31367[6], 4}, {NULL}};
const Segment SGI343990[] = {{&S31367[82], 5}, {NULL}};
const Segment SGI343991[] = {{&S31367[12], 14}, {&S31367[41], 11}, {NULL}};
const Segment SGI343992[] = {{&S31367[12], 14}, {NULL}};
const Segment SGI343993[] = {{&S31368[6], 4}, {NULL}};
const Segment SGI343994[] = {{&S31368[59], 5}, {NULL}};
const Segment SGI343995[] = {{&S31368[12], 14}, {NULL}};
const Segment SGI343996[] = {{&S31369[6], 4}, {NULL}};
const Segment SGI343997[] = {{&S31369[81], 5}, {NULL}};
const Segment SGI343998[] = {{&S31369[12], 14}, {&S31369[40], 11}, {NULL}};
const Segment SGI343999[] = {{&S31369[12], 14}, {NULL}};
const Segment SGI344000[] = {{&S31370[6], 4}, {NULL}};
const Segment SGI344001[] = {{&S31370[59], 5}, {NULL}};
const Segment SGI344002[] = {{&S31370[12], 14}, {&S31370[70], 31}, {NULL}};
const Segment SGI344003[] = {{&S31370[12], 14}, {NULL}};
const Segment SGI344004[] = {{&S31371[6], 4}, {NULL}};
const Segment SGI344005[] = {{&S31371[82], 5}, {NULL}};
const Segment SGI344006[] = {{&S31371[12], 14}, {&S31371[40], 12}, {NULL}};
const Segment SGI344007[] = {{&S31371[12], 14}, {NULL}};
const Segment SGI344008[] = {{&S31372[6], 4}, {NULL}};
const Segment SGI344009[] = {{&S31372[81], 5}, {NULL}};
const Segment SGI344010[] = {{&S31372[12], 14}, {&S31372[40], 11}, {NULL}};
const Segment SGI344011[] = {{&S31372[12], 14}, {NULL}};
const Segment SGI344012[] = {{&S31373[6], 4}, {NULL}};
const Segment SGI344013[] = {{&S31373[81], 5}, {NULL}};
const Segment SGI344014[] = {{&S31373[12], 14}, {&S31373[40], 11}, {NULL}};
const Segment SGI344015[] = {{&S31373[12], 14}, {NULL}};
const Segment SGI344016[] = {{&S31374[6], 4}, {NULL}};
const Segment SGI344017[] = {{&S31374[82], 5}, {NULL}};
const Segment SGI344018[] = {{&S31374[12], 14}, {&S31374[41], 11}, {NULL}};
const Segment SGI344019[] = {{&S31374[12], 14}, {NULL}};
const Segment SGI344020[] = {{&S31375[6], 4}, {NULL}};
const Segment SGI344021[] = {{&S31375[82], 5}, {NULL}};
const Segment SGI344022[] = {{&S31375[12], 14}, {&S31375[40], 12}, {NULL}};
const Segment SGI344023[] = {{&S31375[12], 14}, {NULL}};
const Segment SGI344024[] = {{&S31376[6], 4}, {NULL}};
const Segment SGI344025[] = {{&S31376[82], 5}, {NULL}};
const Segment SGI344026[] = {{&S31376[12], 14}, {&S31376[41], 11}, {NULL}};
const Segment SGI344027[] = {{&S31376[12], 14}, {NULL}};
const Segment SGI344028[] = {{&S31377[6], 4}, {NULL}};
const Segment SGI344029[] = {{&S31377[81], 5}, {NULL}};
const Segment SGI344030[] = {{&S31377[12], 14}, {&S31377[40], 11}, {NULL}};
const Segment SGI344031[] = {{&S31377[12], 14}, {NULL}};
const Segment SGI344032[] = {{&S31379[6], 4}, {NULL}};
const Segment SGI344033[] = {{&S31379[82], 5}, {NULL}};
const Segment SGI344034[] = {{&S31379[12], 14}, {&S31379[40], 12}, {NULL}};
const Segment SGI344035[] = {{&S31379[12], 14}, {NULL}};
const Segment SGI344036[] = {{&S31380[6], 4}, {NULL}};
const Segment SGI344037[] = {{&S31380[82], 5}, {NULL}};
const Segment SGI344038[] = {{&S31380[12], 14}, {&S31380[40], 12}, {NULL}};
const Segment SGI344039[] = {{&S31380[12], 14}, {NULL}};
const Segment SGI344040[] = {{&S31381[6], 4}, {NULL}};
const Segment SGI344041[] = {{&S31381[82], 5}, {NULL}};
const Segment SGI344042[] = {{&S31381[12], 14}, {&S31381[41], 11}, {NULL}};
const Segment SGI344043[] = {{&S31381[12], 14}, {NULL}};
const Segment SGI344044[] = {{&S31382[6], 4}, {NULL}};
const Segment SGI344045[] = {{&S31382[81], 5}, {NULL}};
const Segment SGI344046[] = {{&S31382[12], 14}, {&S31382[40], 11}, {NULL}};
const Segment SGI344047[] = {{&S31382[12], 14}, {NULL}};
const Segment SGI344048[] = {{&S31383[6], 4}, {NULL}};
const Segment SGI344049[] = {{&S31383[82], 5}, {NULL}};
const Segment SGI344050[] = {{&S31383[12], 14}, {&S31383[40], 12}, {NULL}};
const Segment SGI344051[] = {{&S31383[12], 14}, {NULL}};
const Segment SGI344052[] = {{&S31384[6], 4}, {NULL}};
const Segment SGI344053[] = {{&S31384[82], 5}, {NULL}};
const Segment SGI344054[] = {{&S31384[12], 14}, {&S31384[40], 12}, {NULL}};
const Segment SGI344055[] = {{&S31384[12], 14}, {NULL}};
const Segment SGI344056[] = {{&S31386[6], 4}, {NULL}};
const Segment SGI344057[] = {{&S31386[82], 5}, {NULL}};
const Segment SGI344058[] = {{&S31386[12], 14}, {&S31386[40], 12}, {NULL}};
const Segment SGI344059[] = {{&S31386[12], 14}, {NULL}};
const Segment SGI344060[] = {{&S31387[6], 4}, {NULL}};
const Segment SGI344061[] = {{&S31387[82], 5}, {NULL}};
const Segment SGI344062[] = {{&S31387[12], 14}, {&S31387[41], 11}, {NULL}};
const Segment SGI344063[] = {{&S31387[12], 14}, {NULL}};
const Segment SGI344064[] = {{&S31388[6], 4}, {NULL}};
const Segment SGI344065[] = {{&S31388[81], 5}, {NULL}};
const Segment SGI344066[] = {{&S31388[12], 14}, {&S31388[40], 11}, {NULL}};
const Segment SGI344067[] = {{&S31388[12], 14}, {NULL}};
const Segment SGI344068[] = {{&S31389[6], 4}, {NULL}};
const Segment SGI344069[] = {{&S31389[82], 5}, {NULL}};
const Segment SGI344070[] = {{&S31389[12], 14}, {&S31389[41], 11}, {NULL}};
const Segment SGI344071[] = {{&S31389[12], 14}, {NULL}};
const Segment SGI344072[] = {{&S31390[6], 4}, {NULL}};
const Segment SGI344073[] = {{&S31390[82], 5}, {NULL}};
const Segment SGI344074[] = {{&S31390[12], 14}, {&S31390[41], 11}, {NULL}};
const Segment SGI344075[] = {{&S31390[12], 14}, {NULL}};
const Segment SGI344076[] = {{&S31391[6], 4}, {NULL}};
const Segment SGI344077[] = {{&S31391[81], 5}, {NULL}};
const Segment SGI344078[] = {{&S31391[12], 14}, {&S31391[40], 11}, {NULL}};
const Segment SGI344079[] = {{&S31391[12], 14}, {NULL}};
const Segment SGI344080[] = {{&S31392[6], 4}, {NULL}};
const Segment SGI344081[] = {{&S31392[82], 5}, {NULL}};
const Segment SGI344082[] = {{&S31392[12], 14}, {&S31392[41], 11}, {NULL}};
const Segment SGI344083[] = {{&S31392[12], 14}, {NULL}};
const Segment SGI344084[] = {{&S31393[6], 4}, {NULL}};
const Segment SGI344085[] = {{&S31393[81], 5}, {NULL}};
const Segment SGI344086[] = {{&S31393[12], 14}, {&S31393[40], 11}, {NULL}};
const Segment SGI344087[] = {{&S31393[12], 14}, {NULL}};
const Segment SGI344088[] = {{&S31394[6], 4}, {NULL}};
const Segment SGI344089[] = {{&S31394[81], 5}, {NULL}};
const Segment SGI344090[] = {{&S31394[12], 14}, {&S31394[40], 11}, {NULL}};
const Segment SGI344091[] = {{&S31394[12], 14}, {NULL}};
const Segment SGI344092[] = {{&S31395[6], 4}, {NULL}};
const Segment SGI344093[] = {{&S31395[81], 5}, {NULL}};
const Segment SGI344094[] = {{&S31395[12], 14}, {&S31395[40], 11}, {NULL}};
const Segment SGI344095[] = {{&S31395[12], 14}, {NULL}};
const Segment SGI344096[] = {{&S31396[6], 4}, {NULL}};
const Segment SGI344097[] = {{&S31396[81], 5}, {NULL}};
const Segment SGI344098[] = {{&S31396[12], 14}, {&S31396[40], 11}, {NULL}};
const Segment SGI344099[] = {{&S31396[12], 14}, {NULL}};
const Segment SGI344100[] = {{&S58069[5], 5}, {&S58069[105], 5}, {NULL}};
const Segment SGI344101[] = {{&S31397[6], 4}, {NULL}};
const Segment SGI344102[] = {{&S58069[69], 3}, {NULL}};
const Segment SGI344103[] = {{&S31397[82], 5}, {NULL}};
const Segment SGI344104[] = {{&S31397[12], 14}, {&S31397[40], 12}, {NULL}};
const Segment SGI344105[] = {{&S31397[12], 14}, {NULL}};
const Segment SGI344106[] = {{&S58069[131], 3}, {NULL}};
const Segment SGI344107[] = {{&S31398[6], 4}, {NULL}};
const Segment SGI344108[] = {{&S31398[81], 5}, {NULL}};
const Segment SGI344109[] = {{&S31398[12], 14}, {&S31398[40], 11}, {NULL}};
const Segment SGI344110[] = {{&S31398[12], 14}, {NULL}};
const Segment SGI344111[] = {{&S31399[6], 4}, {NULL}};
const Segment SGI344112[] = {{&S31399[82], 5}, {NULL}};
const Segment SGI344113[] = {{&S31399[12], 14}, {&S31399[41], 11}, {NULL}};
const Segment SGI344114[] = {{&S31399[12], 14}, {NULL}};
const Segment SGI344115[] = {{&S31400[6], 4}, {NULL}};
const Segment SGI344116[] = {{&S31400[82], 5}, {NULL}};
const Segment SGI344117[] = {{&S31400[12], 14}, {&S31400[41], 11}, {NULL}};
const Segment SGI344118[] = {{&S31400[12], 14}, {NULL}};
const Segment SGI344119[] = {{&S31401[6], 4}, {NULL}};
const Segment SGI344120[] = {{&S31401[59], 5}, {NULL}};
const Segment SGI344121[] = {{&S31401[12], 14}, {&S31401[70], 4}, {NULL}};
const Segment SGI344122[] = {{&S31401[12], 14}, {NULL}};
const Segment SGI344123[] = {{&S31402[6], 4}, {NULL}};
const Segment SGI344124[] = {{&S31402[81], 5}, {NULL}};
const Segment SGI344125[] = {{&S31402[12], 14}, {&S31402[40], 11}, {NULL}};
const Segment SGI344126[] = {{&S31402[12], 14}, {NULL}};
const Segment SGI344127[] = {{&S31403[6], 4}, {NULL}};
const Segment SGI344128[] = {{&S31403[81], 5}, {NULL}};
const Segment SGI344129[] = {{&S31403[12], 14}, {&S31403[40], 11}, {NULL}};
const Segment SGI344130[] = {{&S31403[12], 14}, {NULL}};
const Segment SGI344131[] = {{&S31404[6], 4}, {NULL}};
const Segment SGI344132[] = {{&S31404[81], 5}, {NULL}};
const Segment SGI344133[] = {{&S31404[12], 14}, {&S31404[40], 11}, {NULL}};
const Segment SGI344134[] = {{&S31404[12], 14}, {NULL}};
const Segment SGI344135[] = {{&S31405[6], 4}, {NULL}};
const Segment SGI344136[] = {{&S31405[81], 5}, {NULL}};
const Segment SGI344137[] = {{&S31405[12], 14}, {&S31405[40], 11}, {NULL}};
const Segment SGI344138[] = {{&S31405[12], 14}, {NULL}};
const Segment SGI344139[] = {{&S31406[6], 4}, {NULL}};
const Segment SGI344140[] = {{&S31406[81], 5}, {NULL}};
const Segment SGI344141[] = {{&S31406[12], 14}, {&S31406[40], 11}, {NULL}};
const Segment SGI344142[] = {{&S31406[12], 14}, {NULL}};
const Segment SGI344143[] = {{&S31407[6], 4}, {NULL}};
const Segment SGI344144[] = {{&S31407[82], 5}, {NULL}};
const Segment SGI344145[] = {{&S31407[12], 14}, {&S31407[41], 11}, {NULL}};
const Segment SGI344146[] = {{&S31407[12], 14}, {NULL}};
const Segment SGI344147[] = {{&S31408[6], 4}, {NULL}};
const Segment SGI344148[] = {{&S31408[81], 5}, {NULL}};
const Segment SGI344149[] = {{&S31408[12], 14}, {&S31408[40], 11}, {NULL}};
const Segment SGI344150[] = {{&S31408[12], 14}, {NULL}};
const Segment SGI344151[] = {{&S58083[42], 5}, {NULL}};
const Segment SGI344152[] = {{&S58083[6], 3}, {NULL}};
const Segment SGI344153[] = {{&S31409[6], 4}, {NULL}};
const Segment SGI344154[] = {{&S31409[81], 5}, {NULL}};
const Segment SGI344155[] = {{&S58083[68], 3}, {NULL}};
const Segment SGI344156[] = {{&S31409[12], 14}, {&S31409[40], 11}, {NULL}};
const Segment SGI344157[] = {{&S31409[12], 14}, {NULL}};
const Segment SGI344158[] = {{&S58084[41], 5}, {NULL}};
const Segment SGI344159[] = {{&S58084[6], 3}, {NULL}};
const Segment SGI344160[] = {{&S31410[6], 4}, {NULL}};
const Segment SGI344161[] = {{&S31410[81], 5}, {NULL}};
const Segment SGI344162[] = {{&S58084[67], 3}, {NULL}};
const Segment SGI344163[] = {{&S31410[12], 14}, {&S31410[40], 11}, {NULL}};
const Segment SGI344164[] = {{&S31410[12], 14}, {NULL}};
const Segment SGI344165[] = {{&S58085[42], 5}, {NULL}};
const Segment SGI344166[] = {{&S58085[6], 3}, {NULL}};
const Segment SGI344167[] = {{&S58085[68], 3}, {NULL}};
const Segment SGI344168[] = {{&S31411[6], 4}, {NULL}};
const Segment SGI344169[] = {{&S31411[81], 5}, {NULL}};
const Segment SGI344170[] = {{&S31411[12], 14}, {&S31411[40], 11}, {NULL}};
const Segment SGI344171[] = {{&S31411[12], 14}, {NULL}};
const Segment SGI344172[] = {{&S58086[5], 5}, {NULL}};
const Segment SGI344173[] = {{&S58086[16], 3}, {NULL}};
const Segment SGI344174[] = {{&S31412[6], 4}, {NULL}};
const Segment SGI344175[] = {{&S31412[81], 5}, {NULL}};
const Segment SGI344176[] = {{&S31412[12], 14}, {&S31412[40], 11}, {NULL}};
const Segment SGI344177[] = {{&S31412[12], 14}, {NULL}};
const Segment SGI344178[] = {{&S58087[42], 3}, {NULL}};
const Segment SGI344179[] = {{&S58087[6], 3}, {NULL}};
const Segment SGI344180[] = {{&S58087[66], 3}, {NULL}};
const Segment SGI344181[] = {{&S31413[6], 4}, {NULL}};
const Segment SGI344182[] = {{&S31413[81], 5}, {NULL}};
const Segment SGI344183[] = {{&S31413[12], 14}, {&S31413[40], 11}, {NULL}};
const Segment SGI344184[] = {{&S31413[12], 14}, {NULL}};
const Segment SGI344185[] = {{&S31414[6], 4}, {NULL}};
const Segment SGI344186[] = {{&S31414[81], 5}, {NULL}};
const Segment SGI344187[] = {{&S31414[12], 14}, {&S31414[40], 11}, {NULL}};
const Segment SGI344188[] = {{&S31414[12], 14}, {NULL}};
const Segment SGI344189[] = {{&S31415[6], 4}, {NULL}};
const Segment SGI344190[] = {{&S31415[81], 5}, {NULL}};
const Segment SGI344191[] = {{&S31415[12], 14}, {&S31415[40], 11}, {NULL}};
const Segment SGI344192[] = {{&S31415[12], 14}, {NULL}};
const Segment SGI344193[] = {{&S31416[6], 4}, {NULL}};
const Segment SGI344194[] = {{&S31416[81], 5}, {NULL}};
const Segment SGI344195[] = {{&S31416[12], 14}, {&S31416[40], 11}, {NULL}};
const Segment SGI344196[] = {{&S31416[12], 14}, {NULL}};
const Segment SGI344197[] = {{&S31417[6], 4}, {NULL}};
const Segment SGI344198[] = {{&S31417[81], 5}, {NULL}};
const Segment SGI344199[] = {{&S31417[12], 14}, {&S31417[40], 11}, {NULL}};
const Segment SGI344200[] = {{&S31417[12], 14}, {NULL}};
const Segment SGI344201[] = {{&S31418[6], 4}, {NULL}};
const Segment SGI344202[] = {{&S31418[81], 5}, {NULL}};
const Segment SGI344203[] = {{&S31418[12], 14}, {&S31418[40], 11}, {NULL}};
const Segment SGI344204[] = {{&S31418[12], 14}, {NULL}};
const Segment SGI344205[] = {{&S31419[6], 4}, {NULL}};
const Segment SGI344206[] = {{&S31419[81], 5}, {NULL}};
const Segment SGI344207[] = {{&S31419[12], 14}, {&S31419[40], 11}, {NULL}};
const Segment SGI344208[] = {{&S31419[12], 14}, {NULL}};
const Segment SGI344209[] = {{&S31420[6], 4}, {NULL}};
const Segment SGI344210[] = {{&S31420[81], 5}, {NULL}};
const Segment SGI344211[] = {{&S31420[12], 14}, {&S31420[40], 11}, {NULL}};
const Segment SGI344212[] = {{&S31420[12], 14}, {NULL}};
const Segment SGI344213[] = {{&S31421[6], 4}, {NULL}};
const Segment SGI344214[] = {{&S31421[81], 5}, {NULL}};
const Segment SGI344215[] = {{&S31421[12], 14}, {&S31421[40], 11}, {NULL}};
const Segment SGI344216[] = {{&S31421[12], 14}, {NULL}};
const Segment SGI344217[] = {{&S31422[6], 4}, {NULL}};
const Segment SGI344218[] = {{&S31422[81], 5}, {NULL}};
const Segment SGI344219[] = {{&S31422[12], 14}, {&S31422[40], 11}, {NULL}};
const Segment SGI344220[] = {{&S31422[12], 14}, {NULL}};
const Segment SGI344221[] = {{&S31423[6], 4}, {NULL}};
const Segment SGI344222[] = {{&S31423[81], 5}, {NULL}};
const Segment SGI344223[] = {{&S31423[12], 14}, {&S31423[40], 11}, {NULL}};
const Segment SGI344224[] = {{&S31423[12], 14}, {NULL}};
const Segment SGI344225[] = {{&S58131[5], 3}, {NULL}};
const Segment SGI344226[] = {{&S58132[5], 3}, {&S58132[5], 3}, {NULL}};
const Segment SGI344227[] = {{&S31469[6], 4}, {NULL}};
const Segment SGI344228[] = {{&S31469[52], 5}, {NULL}};
const Segment SGI344229[] = {{&S31469[12], 45}, {NULL}};
const Segment SGI344230[] = {{&S31469[12], 14}, {NULL}};
const Segment SGI344231[] = {{&S31470[6], 4}, {NULL}};
const Segment SGI344232[] = {{&S31470[75], 5}, {NULL}};
const Segment SGI344233[] = {{&S31470[12], 14}, {&S31470[37], 43}, {NULL}};
const Segment SGI344234[] = {{&S31470[12], 14}, {NULL}};
const Segment SGI344235[] = {{&S31471[6], 4}, {NULL}};
const Segment SGI344236[] = {{&S31471[75], 5}, {NULL}};
const Segment SGI344237[] = {{&S31471[12], 14}, {&S31471[37], 43}, {NULL}};
const Segment SGI344238[] = {{&S31471[12], 14}, {NULL}};
const Segment SGI344239[] = {{&S31472[6], 4}, {NULL}};
const Segment SGI344240[] = {{&S31472[52], 5}, {NULL}};
const Segment SGI344241[] = {{&S31472[12], 55}, {NULL}};
const Segment SGI344242[] = {{&S31472[12], 14}, {NULL}};
const Segment SGI344243[] = {{&S31473[6], 4}, {NULL}};
const Segment SGI344244[] = {{&S31473[74], 5}, {NULL}};
const Segment SGI344245[] = {{&S31473[12], 14}, {&S31473[37], 42}, {NULL}};
const Segment SGI344246[] = {{&S31473[12], 14}, {NULL}};
const Segment SGI344247[] = {{&S31474[6], 4}, {NULL}};
const Segment SGI344248[] = {{&S31474[59], 5}, {NULL}};
const Segment SGI344249[] = {{&S31474[12], 14}, {NULL}};
const Segment SGI344250[] = {{&S31475[6], 4}, {NULL}};
const Segment SGI344251[] = {{&S31475[74], 5}, {NULL}};
const Segment SGI344252[] = {{&S31475[12], 14}, {&S31475[37], 42}, {NULL}};
const Segment SGI344253[] = {{&S31475[12], 14}, {NULL}};
const Segment SGI344254[] = {{&S31476[6], 4}, {NULL}};
const Segment SGI344255[] = {{&S31476[74], 5}, {NULL}};
const Segment SGI344256[] = {{&S31476[12], 14}, {&S31476[37], 42}, {NULL}};
const Segment SGI344257[] = {{&S31476[12], 14}, {NULL}};
const Segment SGI344258[] = {{&S31478[6], 4}, {NULL}};
const Segment SGI344259[] = {{&S31478[75], 5}, {NULL}};
const Segment SGI344260[] = {{&S31478[12], 14}, {&S31478[38], 42}, {NULL}};
const Segment SGI344261[] = {{&S31478[12], 14}, {NULL}};
const Segment SGI344262[] = {{&S31479[6], 4}, {NULL}};
const Segment SGI344263[] = {{&S31479[74], 5}, {NULL}};
const Segment SGI344264[] = {{&S31479[12], 14}, {&S31479[37], 42}, {NULL}};
const Segment SGI344265[] = {{&S31479[12], 14}, {NULL}};
const Segment SGI344266[] = {{&S31481[6], 4}, {NULL}};
const Segment SGI344267[] = {{&S31481[74], 5}, {NULL}};
const Segment SGI344268[] = {{&S31481[12], 14}, {&S31481[37], 42}, {NULL}};
const Segment SGI344269[] = {{&S31481[12], 14}, {NULL}};
const Segment SGI344270[] = {{&S31482[6], 4}, {NULL}};
const Segment SGI344271[] = {{&S31482[74], 5}, {NULL}};
const Segment SGI344272[] = {{&S31482[12], 14}, {&S31482[37], 42}, {NULL}};
const Segment SGI344273[] = {{&S31482[12], 14}, {NULL}};
const Segment SGI344274[] = {{&S31483[6], 4}, {NULL}};
const Segment SGI344275[] = {{&S31483[75], 5}, {NULL}};
const Segment SGI344276[] = {{&S31483[12], 14}, {&S31483[37], 43}, {NULL}};
const Segment SGI344277[] = {{&S31483[12], 14}, {NULL}};
const Segment SGI344278[] = {{&S31484[6], 4}, {NULL}};
const Segment SGI344279[] = {{&S31484[74], 5}, {NULL}};
const Segment SGI344280[] = {{&S31484[12], 14}, {&S31484[37], 42}, {NULL}};
const Segment SGI344281[] = {{&S31484[12], 14}, {NULL}};
const Segment SGI344282[] = {{&S31485[6], 4}, {NULL}};
const Segment SGI344283[] = {{&S31485[75], 5}, {NULL}};
const Segment SGI344284[] = {{&S31485[12], 14}, {&S31485[38], 42}, {NULL}};
const Segment SGI344285[] = {{&S31485[12], 14}, {NULL}};
const Segment SGI344286[] = {{&S31486[6], 4}, {NULL}};
const Segment SGI344287[] = {{&S31486[74], 5}, {NULL}};
const Segment SGI344288[] = {{&S31486[12], 14}, {&S31486[37], 42}, {NULL}};
const Segment SGI344289[] = {{&S31486[12], 14}, {NULL}};
const Segment SGI344290[] = {{&S31487[6], 4}, {NULL}};
const Segment SGI344291[] = {{&S31487[74], 5}, {NULL}};
const Segment SGI344292[] = {{&S31487[12], 14}, {&S31487[37], 42}, {NULL}};
const Segment SGI344293[] = {{&S31487[12], 14}, {NULL}};
const Segment SGI344294[] = {{&S31488[6], 4}, {NULL}};
const Segment SGI344295[] = {{&S31488[74], 5}, {NULL}};
const Segment SGI344296[] = {{&S31488[12], 14}, {&S31488[37], 42}, {NULL}};
const Segment SGI344297[] = {{&S31488[12], 14}, {NULL}};
const Segment SGI344298[] = {{&S31489[6], 4}, {NULL}};
const Segment SGI344299[] = {{&S31489[75], 5}, {NULL}};
const Segment SGI344300[] = {{&S31489[12], 14}, {&S31489[37], 43}, {NULL}};
const Segment SGI344301[] = {{&S31489[12], 14}, {NULL}};
const Segment SGI344302[] = {{&S31490[6], 4}, {NULL}};
const Segment SGI344303[] = {{&S31490[74], 5}, {NULL}};
const Segment SGI344304[] = {{&S31490[12], 14}, {&S31490[37], 42}, {NULL}};
const Segment SGI344305[] = {{&S31490[12], 14}, {NULL}};
const Segment SGI344306[] = {{&S31491[6], 4}, {NULL}};
const Segment SGI344307[] = {{&S31491[74], 5}, {NULL}};
const Segment SGI344308[] = {{&S31491[12], 14}, {&S31491[37], 42}, {NULL}};
const Segment SGI344309[] = {{&S31491[12], 14}, {NULL}};
const Segment SGI344310[] = {{&S31492[6], 4}, {NULL}};
const Segment SGI344311[] = {{&S31492[73], 5}, {NULL}};
const Segment SGI344312[] = {{&S31492[12], 14}, {&S31492[84], 4}, {NULL}};
const Segment SGI344313[] = {{&S31492[12], 14}, {NULL}};
const Segment SGI344314[] = {{&S31493[6], 4}, {NULL}};
const Segment SGI344315[] = {{&S31493[74], 5}, {NULL}};
const Segment SGI344316[] = {{&S31493[12], 14}, {&S31493[37], 42}, {NULL}};
const Segment SGI344317[] = {{&S31493[12], 14}, {NULL}};
const Segment SGI344318[] = {{&S31494[6], 4}, {NULL}};
const Segment SGI344319[] = {{&S31494[74], 5}, {NULL}};
const Segment SGI344320[] = {{&S31494[12], 14}, {&S31494[37], 42}, {NULL}};
const Segment SGI344321[] = {{&S31494[12], 14}, {NULL}};
const Segment SGI344322[] = {{&S31495[6], 4}, {NULL}};
const Segment SGI344323[] = {{&S31495[74], 5}, {NULL}};
const Segment SGI344324[] = {{&S31495[12], 14}, {&S31495[37], 42}, {NULL}};
const Segment SGI344325[] = {{&S31495[12], 14}, {NULL}};
const Segment SGI344326[] = {{&S31496[6], 4}, {NULL}};
const Segment SGI344327[] = {{&S31496[74], 5}, {NULL}};
const Segment SGI344328[] = {{&S31496[12], 14}, {&S31496[37], 42}, {NULL}};
const Segment SGI344329[] = {{&S31496[12], 14}, {NULL}};
const Segment SGI344330[] = {{&S31497[6], 4}, {NULL}};
const Segment SGI344331[] = {{&S31497[74], 5}, {NULL}};
const Segment SGI344332[] = {{&S31497[12], 14}, {&S31497[37], 42}, {NULL}};
const Segment SGI344333[] = {{&S31497[12], 14}, {NULL}};
const Segment SGI344334[] = {{&S31498[6], 4}, {NULL}};
const Segment SGI344335[] = {{&S31498[74], 5}, {NULL}};
const Segment SGI344336[] = {{&S31498[12], 14}, {&S31498[37], 42}, {NULL}};
const Segment SGI344337[] = {{&S31498[12], 14}, {NULL}};
const Segment SGI344338[] = {{&S31500[6], 4}, {NULL}};
const Segment SGI344339[] = {{&S31500[74], 5}, {NULL}};
const Segment SGI344340[] = {{&S31500[12], 14}, {&S31500[37], 42}, {NULL}};
const Segment SGI344341[] = {{&S31500[12], 14}, {NULL}};
const Segment SGI344342[] = {{&S31501[6], 4}, {NULL}};
const Segment SGI344343[] = {{&S31501[75], 5}, {NULL}};
const Segment SGI344344[] = {{&S31501[12], 14}, {&S31501[37], 43}, {NULL}};
const Segment SGI344345[] = {{&S31501[12], 14}, {NULL}};
const Segment SGI344346[] = {{&S31502[6], 4}, {NULL}};
const Segment SGI344347[] = {{&S31502[74], 5}, {NULL}};
const Segment SGI344348[] = {{&S31502[12], 14}, {&S31502[37], 42}, {NULL}};
const Segment SGI344349[] = {{&S31502[12], 14}, {NULL}};
const Segment SGI344350[] = {{&S31503[6], 4}, {NULL}};
const Segment SGI344351[] = {{&S31503[74], 5}, {NULL}};
const Segment SGI344352[] = {{&S31503[12], 14}, {&S31503[37], 42}, {NULL}};
const Segment SGI344353[] = {{&S31503[12], 14}, {NULL}};
const Segment SGI344354[] = {{&S31504[6], 4}, {NULL}};
const Segment SGI344355[] = {{&S31504[74], 5}, {NULL}};
const Segment SGI344356[] = {{&S31504[12], 14}, {&S31504[37], 42}, {NULL}};
const Segment SGI344357[] = {{&S31504[12], 14}, {NULL}};
const Segment SGI344358[] = {{&S31505[6], 4}, {NULL}};
const Segment SGI344359[] = {{&S31505[74], 5}, {NULL}};
const Segment SGI344360[] = {{&S31505[12], 14}, {&S31505[37], 42}, {NULL}};
const Segment SGI344361[] = {{&S31505[12], 14}, {NULL}};
const Segment SGI344362[] = {{&S31506[6], 4}, {NULL}};
const Segment SGI344363[] = {{&S31506[101], 5}, {NULL}};
const Segment SGI344364[] = {{&S31506[12], 14}, {&S31506[37], 12}, {NULL}};
const Segment SGI344365[] = {{&S31506[12], 14}, {NULL}};
const Segment SGI344366[] = {{&S31507[6], 4}, {NULL}};
const Segment SGI344367[] = {{&S31507[74], 5}, {NULL}};
const Segment SGI344368[] = {{&S31507[12], 14}, {&S31507[37], 42}, {NULL}};
const Segment SGI344369[] = {{&S31507[12], 14}, {NULL}};
const Segment SGI344370[] = {{&S31508[6], 4}, {NULL}};
const Segment SGI344371[] = {{&S31508[75], 5}, {NULL}};
const Segment SGI344372[] = {{&S31508[12], 14}, {&S31508[37], 43}, {NULL}};
const Segment SGI344373[] = {{&S31508[12], 14}, {NULL}};
const Segment SGI344374[] = {{&S31509[6], 4}, {NULL}};
const Segment SGI344375[] = {{&S31509[74], 5}, {NULL}};
const Segment SGI344376[] = {{&S31509[12], 14}, {&S31509[37], 42}, {NULL}};
const Segment SGI344377[] = {{&S31509[12], 14}, {NULL}};
const Segment SGI344378[] = {{&S44209[49], 5}, {NULL}};
const Segment SGI344379[] = {{&S44209[40], 3}, {NULL}};
const Segment SGI344380[] = {{&S44210[49], 5}, {NULL}};
const Segment SGI344381[] = {{&S44210[40], 3}, {NULL}};
const Segment SGI344382[] = {{&S31511[6], 4}, {NULL}};
const Segment SGI344383[] = {{&S31511[74], 5}, {NULL}};
const Segment SGI344384[] = {{&S31511[12], 14}, {&S31511[37], 42}, {NULL}};
const Segment SGI344385[] = {{&S31511[12], 14}, {NULL}};
const Segment SGI344386[] = {{&S31512[6], 4}, {NULL}};
const Segment SGI344387[] = {{&S31512[74], 5}, {NULL}};
const Segment SGI344388[] = {{&S31512[12], 14}, {&S31512[37], 42}, {NULL}};
const Segment SGI344389[] = {{&S31512[12], 14}, {NULL}};
const Segment SGI344390[] = {{&S31513[6], 4}, {NULL}};
const Segment SGI344391[] = {{&S31513[75], 5}, {NULL}};
const Segment SGI344392[] = {{&S31513[12], 14}, {&S31513[38], 42}, {NULL}};
const Segment SGI344393[] = {{&S31513[12], 14}, {NULL}};
const Segment SGI344394[] = {{&S31514[6], 4}, {NULL}};
const Segment SGI344395[] = {{&S31514[74], 5}, {NULL}};
const Segment SGI344396[] = {{&S31514[12], 14}, {&S31514[37], 42}, {NULL}};
const Segment SGI344397[] = {{&S31514[12], 14}, {NULL}};
const Segment SGI344398[] = {{&S31515[6], 4}, {NULL}};
const Segment SGI344399[] = {{&S31515[74], 5}, {NULL}};
const Segment SGI344400[] = {{&S31515[12], 14}, {&S31515[37], 42}, {NULL}};
const Segment SGI344401[] = {{&S31515[12], 14}, {NULL}};
const Segment SGI344402[] = {{&S31516[6], 4}, {NULL}};
const Segment SGI344403[] = {{&S31516[74], 5}, {NULL}};
const Segment SGI344404[] = {{&S31516[12], 14}, {&S31516[37], 42}, {NULL}};
const Segment SGI344405[] = {{&S31516[12], 14}, {NULL}};
const Segment SGI344406[] = {{&S31517[6], 4}, {NULL}};
const Segment SGI344407[] = {{&S31517[74], 5}, {NULL}};
const Segment SGI344408[] = {{&S31517[12], 14}, {&S31517[37], 42}, {NULL}};
const Segment SGI344409[] = {{&S31517[12], 14}, {NULL}};
const Segment SGI344410[] = {{&S31518[6], 4}, {NULL}};
const Segment SGI344411[] = {{&S31518[74], 5}, {NULL}};
const Segment SGI344412[] = {{&S31518[12], 14}, {&S31518[37], 42}, {NULL}};
const Segment SGI344413[] = {{&S31518[12], 14}, {NULL}};
const Segment SGI344414[] = {{&S31519[6], 4}, {NULL}};
const Segment SGI344415[] = {{&S31519[74], 5}, {NULL}};
const Segment SGI344416[] = {{&S31519[12], 14}, {&S31519[37], 42}, {NULL}};
const Segment SGI344417[] = {{&S31519[12], 14}, {NULL}};
const Segment SGI344418[] = {{&S31520[6], 4}, {NULL}};
const Segment SGI344419[] = {{&S31520[75], 5}, {NULL}};
const Segment SGI344420[] = {{&S31520[12], 14}, {&S31520[37], 43}, {NULL}};
const Segment SGI344421[] = {{&S31520[12], 14}, {NULL}};
const Segment SGI344422[] = {{&S31521[6], 4}, {NULL}};
const Segment SGI344423[] = {{&S31521[74], 5}, {NULL}};
const Segment SGI344424[] = {{&S31521[12], 14}, {&S31521[37], 42}, {NULL}};
const Segment SGI344425[] = {{&S31521[12], 14}, {NULL}};
const Segment SGI344426[] = {{&S31522[6], 4}, {NULL}};
const Segment SGI344427[] = {{&S31522[74], 5}, {NULL}};
const Segment SGI344428[] = {{&S31522[12], 14}, {&S31522[37], 42}, {NULL}};
const Segment SGI344429[] = {{&S31522[12], 14}, {NULL}};
const Segment SGI344430[] = {{&S58227[5], 7}, {&S58227[5], 7}, {NULL}};
const Segment SGI344431[] = {{&S58227[13], 3}, {NULL}};
const Segment SGI344432[] = {{&S31523[6], 4}, {NULL}};
const Segment SGI344433[] = {{&S31523[74], 5}, {NULL}};
const Segment SGI344434[] = {{&S58227[134], 3}, {NULL}};
const Segment SGI344435[] = {{&S31523[12], 14}, {&S31523[37], 42}, {NULL}};
const Segment SGI344436[] = {{&S31523[12], 14}, {NULL}};
const Segment SGI344437[] = {{&S58228[5], 4}, {&S58228[5], 4}, {NULL}};
const Segment SGI344438[] = {{&S31524[6], 4}, {NULL}};
const Segment SGI344439[] = {{&S31524[74], 5}, {NULL}};
const Segment SGI344440[] = {{&S58228[10], 3}, {NULL}};
const Segment SGI344441[] = {{&S31524[12], 14}, {&S31524[37], 42}, {NULL}};
const Segment SGI344442[] = {{&S31524[12], 14}, {NULL}};
const Segment SGI344443[] = {{&S58228[128], 3}, {NULL}};
const Segment SGI344444[] = {{&S31525[6], 4}, {NULL}};
const Segment SGI344445[] = {{&S31525[75], 5}, {NULL}};
const Segment SGI344446[] = {{&S31525[12], 14}, {&S31525[38], 42}, {NULL}};
const Segment SGI344447[] = {{&S31525[12], 14}, {NULL}};
const Segment SGI344448[] = {{&S31526[6], 4}, {NULL}};
const Segment SGI344449[] = {{&S31526[74], 5}, {NULL}};
const Segment SGI344450[] = {{&S31526[12], 14}, {&S31526[37], 42}, {NULL}};
const Segment SGI344451[] = {{&S31526[12], 14}, {NULL}};
const Segment SGI344452[] = {{&S31527[6], 4}, {NULL}};
const Segment SGI344453[] = {{&S31527[74], 5}, {NULL}};
const Segment SGI344454[] = {{&S31527[12], 14}, {&S31527[37], 42}, {NULL}};
const Segment SGI344455[] = {{&S31527[12], 14}, {NULL}};
const Segment SGI344456[] = {{&S58257[35], 5}, {NULL}};
const Segment SGI344457[] = {{&S58257[52], 3}, {NULL}};
const Segment SGI344458[] = {{&S58258[5], 5}, {NULL}};
const Segment SGI344459[] = {{&S58258[16], 3}, {NULL}};
const Segment SGI344460[] = {{&S58259[35], 7}, {NULL}};
const Segment SGI344461[] = {{&S58259[54], 3}, {NULL}};
const Segment SGI344462[] = {{&S58260[34], 5}, {NULL}};
const Segment SGI344463[] = {{&S58260[50], 3}, {NULL}};
const Segment SGI344464[] = {{&S58283[5], 7}, {NULL}};
const Segment SGI344465[] = {{&S31579[6], 4}, {NULL}};
const Segment SGI344466[] = {{&S58284[5], 8}, {NULL}};
const Segment SGI344467[] = {{&S31579[81], 5}, {NULL}};
const Segment SGI344468[] = {{&S31579[12], 14}, {&S31579[44], 42}, {NULL}};
const Segment SGI344469[] = {{&S31579[12], 14}, {NULL}};
const Segment SGI344470[] = {{&S58285[5], 4}, {NULL}};
const Segment SGI344471[] = {{&S31580[6], 4}, {NULL}};
const Segment SGI344472[] = {{&S31580[52], 5}, {NULL}};
const Segment SGI344473[] = {{&S31580[12], 45}, {NULL}};
const Segment SGI344474[] = {{&S31580[12], 14}, {NULL}};
const Segment SGI344475[] = {{&S31581[6], 4}, {NULL}};
const Segment SGI344476[] = {{&S31581[59], 5}, {NULL}};
const Segment SGI344477[] = {{&S31581[12], 14}, {NULL}};
const Segment SGI344478[] = {{&S31582[6], 4}, {NULL}};
const Segment SGI344479[] = {{&S31582[81], 5}, {NULL}};
const Segment SGI344480[] = {{&S31582[12], 14}, {&S31582[44], 42}, {NULL}};
const Segment SGI344481[] = {{&S31582[12], 14}, {NULL}};
const Segment SGI344482[] = {{&S31583[6], 4}, {NULL}};
const Segment SGI344483[] = {{&S31583[59], 5}, {NULL}};
const Segment SGI344484[] = {{&S31583[12], 14}, {&S31583[70], 4}, {NULL}};
const Segment SGI344485[] = {{&S31583[12], 14}, {NULL}};
const Segment SGI344486[] = {{&S31584[6], 4}, {NULL}};
const Segment SGI344487[] = {{&S31584[81], 5}, {NULL}};
const Segment SGI344488[] = {{&S31584[12], 14}, {&S31584[44], 42}, {NULL}};
const Segment SGI344489[] = {{&S31584[12], 14}, {NULL}};
const Segment SGI344490[] = {{&S31585[6], 4}, {NULL}};
const Segment SGI344491[] = {{&S31585[81], 5}, {NULL}};
const Segment SGI344492[] = {{&S31585[12], 14}, {&S31585[44], 42}, {NULL}};
const Segment SGI344493[] = {{&S31585[12], 14}, {NULL}};
const Segment SGI344494[] = {{&S31586[6], 4}, {NULL}};
const Segment SGI344495[] = {{&S31586[81], 5}, {NULL}};
const Segment SGI344496[] = {{&S31586[12], 14}, {&S31586[44], 42}, {NULL}};
const Segment SGI344497[] = {{&S31586[12], 14}, {NULL}};
const Segment SGI344498[] = {{&S31587[6], 4}, {NULL}};
const Segment SGI344499[] = {{&S31587[81], 5}, {NULL}};
const Segment SGI344500[] = {{&S31587[12], 14}, {&S31587[44], 42}, {NULL}};
const Segment SGI344501[] = {{&S31587[12], 14}, {NULL}};
const Segment SGI344502[] = {{&S31588[6], 4}, {NULL}};
const Segment SGI344503[] = {{&S31588[81], 5}, {NULL}};
const Segment SGI344504[] = {{&S31588[12], 14}, {&S31588[44], 42}, {NULL}};
const Segment SGI344505[] = {{&S31588[12], 14}, {NULL}};
const Segment SGI344506[] = {{&S31589[6], 4}, {NULL}};
const Segment SGI344507[] = {{&S31589[81], 5}, {NULL}};
const Segment SGI344508[] = {{&S31589[12], 14}, {&S31589[44], 42}, {NULL}};
const Segment SGI344509[] = {{&S31589[12], 14}, {NULL}};
const Segment SGI344510[] = {{&S31590[6], 4}, {NULL}};
const Segment SGI344511[] = {{&S31590[81], 5}, {NULL}};
const Segment SGI344512[] = {{&S31590[12], 14}, {&S31590[44], 42}, {NULL}};
const Segment SGI344513[] = {{&S31590[12], 14}, {NULL}};
const Segment SGI344514[] = {{&S31591[6], 4}, {NULL}};
const Segment SGI344515[] = {{&S31591[81], 5}, {NULL}};
const Segment SGI344516[] = {{&S31591[12], 14}, {&S31591[44], 42}, {NULL}};
const Segment SGI344517[] = {{&S31591[12], 14}, {NULL}};
const Segment SGI344518[] = {{&S31592[6], 4}, {NULL}};
const Segment SGI344519[] = {{&S31592[81], 5}, {NULL}};
const Segment SGI344520[] = {{&S31592[12], 14}, {&S31592[44], 42}, {NULL}};
const Segment SGI344521[] = {{&S31592[12], 14}, {NULL}};
const Segment SGI344522[] = {{&S31593[6], 4}, {NULL}};
const Segment SGI344523[] = {{&S31593[81], 5}, {NULL}};
const Segment SGI344524[] = {{&S31593[12], 14}, {&S31593[44], 42}, {NULL}};
const Segment SGI344525[] = {{&S31593[12], 14}, {NULL}};
const Segment SGI344526[] = {{&S31594[6], 4}, {NULL}};
const Segment SGI344527[] = {{&S31594[81], 5}, {NULL}};
const Segment SGI344528[] = {{&S31594[12], 14}, {&S31594[44], 42}, {NULL}};
const Segment SGI344529[] = {{&S31594[12], 14}, {NULL}};
const Segment SGI344530[] = {{&S58301[5], 5}, {&S58301[105], 5}, {NULL}};
const Segment SGI344531[] = {{&S31595[6], 4}, {NULL}};
const Segment SGI344532[] = {{&S58301[11], 3}, {NULL}};
const Segment SGI344533[] = {{&S31595[81], 5}, {NULL}};
const Segment SGI344534[] = {{&S31595[12], 14}, {&S31595[44], 42}, {NULL}};
const Segment SGI344535[] = {{&S31595[12], 14}, {NULL}};
const Segment SGI344536[] = {{&S58301[131], 3}, {NULL}};
const Segment SGI344537[] = {{&S31596[6], 4}, {NULL}};
const Segment SGI344538[] = {{&S31596[81], 5}, {NULL}};
const Segment SGI344539[] = {{&S31596[12], 14}, {&S31596[44], 42}, {NULL}};
const Segment SGI344540[] = {{&S31596[12], 14}, {NULL}};
const Segment SGI344541[] = {{&S31597[6], 4}, {NULL}};
const Segment SGI344542[] = {{&S31597[81], 5}, {NULL}};
const Segment SGI344543[] = {{&S31597[12], 14}, {&S31597[44], 42}, {NULL}};
const Segment SGI344544[] = {{&S31597[12], 14}, {NULL}};
const Segment SGI344545[] = {{&S31598[6], 4}, {NULL}};
const Segment SGI344546[] = {{&S31598[52], 5}, {NULL}};
const Segment SGI344547[] = {{&S31598[12], 55}, {NULL}};
const Segment SGI344548[] = {{&S31598[12], 14}, {NULL}};
const Segment SGI344549[] = {{&S31599[6], 4}, {NULL}};
const Segment SGI344550[] = {{&S31599[81], 5}, {NULL}};
const Segment SGI344551[] = {{&S31599[12], 14}, {&S31599[44], 42}, {NULL}};
const Segment SGI344552[] = {{&S31599[12], 14}, {NULL}};
const Segment SGI344553[] = {{&S31600[6], 4}, {NULL}};
const Segment SGI344554[] = {{&S31600[81], 5}, {NULL}};
const Segment SGI344555[] = {{&S31600[12], 14}, {&S31600[44], 42}, {NULL}};
const Segment SGI344556[] = {{&S31600[12], 14}, {NULL}};
const Segment SGI344557[] = {{&S31603[6], 4}, {NULL}};
const Segment SGI344558[] = {{&S31603[81], 5}, {NULL}};
const Segment SGI344559[] = {{&S31603[12], 14}, {&S31603[44], 42}, {NULL}};
const Segment SGI344560[] = {{&S31603[12], 14}, {NULL}};
const Segment SGI344561[] = {{&S31604[6], 4}, {NULL}};
const Segment SGI344562[] = {{&S31604[81], 5}, {NULL}};
const Segment SGI344563[] = {{&S31604[12], 14}, {&S31604[44], 42}, {NULL}};
const Segment SGI344564[] = {{&S31604[12], 14}, {NULL}};
const Segment SGI344565[] = {{&S31605[6], 4}, {NULL}};
const Segment SGI344566[] = {{&S31605[81], 5}, {NULL}};
const Segment SGI344567[] = {{&S31605[12], 14}, {&S31605[44], 42}, {NULL}};
const Segment SGI344568[] = {{&S31605[12], 14}, {NULL}};
const Segment SGI344569[] = {{&S31606[6], 4}, {NULL}};
const Segment SGI344570[] = {{&S31606[81], 5}, {NULL}};
const Segment SGI344571[] = {{&S31606[12], 14}, {&S31606[44], 42}, {NULL}};
const Segment SGI344572[] = {{&S31606[12], 14}, {NULL}};
const Segment SGI344573[] = {{&S31607[6], 4}, {NULL}};
const Segment SGI344574[] = {{&S31607[81], 5}, {NULL}};
const Segment SGI344575[] = {{&S31607[12], 14}, {&S31607[44], 42}, {NULL}};
const Segment SGI344576[] = {{&S31607[12], 14}, {NULL}};
const Segment SGI344577[] = {{&S31608[6], 4}, {NULL}};
const Segment SGI344578[] = {{&S31608[81], 5}, {NULL}};
const Segment SGI344579[] = {{&S31608[12], 14}, {&S31608[44], 42}, {NULL}};
const Segment SGI344580[] = {{&S31608[12], 14}, {NULL}};
const Segment SGI344581[] = {{&S31609[6], 4}, {NULL}};
const Segment SGI344582[] = {{&S31609[81], 5}, {NULL}};
const Segment SGI344583[] = {{&S31609[12], 14}, {&S31609[44], 42}, {NULL}};
const Segment SGI344584[] = {{&S31609[12], 14}, {NULL}};
const Segment SGI344585[] = {{&S31610[6], 4}, {NULL}};
const Segment SGI344586[] = {{&S31610[81], 5}, {NULL}};
const Segment SGI344587[] = {{&S31610[12], 14}, {&S31610[44], 42}, {NULL}};
const Segment SGI344588[] = {{&S31610[12], 14}, {NULL}};
const Segment SGI344589[] = {{&S31611[6], 4}, {NULL}};
const Segment SGI344590[] = {{&S31611[81], 5}, {NULL}};
const Segment SGI344591[] = {{&S31611[12], 14}, {&S31611[44], 42}, {NULL}};
const Segment SGI344592[] = {{&S31611[12], 14}, {NULL}};
const Segment SGI344593[] = {{&S31612[6], 4}, {NULL}};
const Segment SGI344594[] = {{&S31612[81], 5}, {NULL}};
const Segment SGI344595[] = {{&S31612[12], 14}, {&S31612[44], 42}, {NULL}};
const Segment SGI344596[] = {{&S31612[12], 14}, {NULL}};
const Segment SGI344597[] = {{&S31613[6], 4}, {NULL}};
const Segment SGI344598[] = {{&S31613[81], 5}, {NULL}};
const Segment SGI344599[] = {{&S31613[12], 14}, {&S31613[44], 42}, {NULL}};
const Segment SGI344600[] = {{&S31613[12], 14}, {NULL}};
const Segment SGI344601[] = {{&S31614[6], 4}, {NULL}};
const Segment SGI344602[] = {{&S31614[81], 5}, {NULL}};
const Segment SGI344603[] = {{&S31614[12], 14}, {&S31614[44], 42}, {NULL}};
const Segment SGI344604[] = {{&S31614[12], 14}, {NULL}};
const Segment SGI344605[] = {{&S31615[6], 4}, {NULL}};
const Segment SGI344606[] = {{&S31615[81], 5}, {NULL}};
const Segment SGI344607[] = {{&S31615[12], 14}, {&S31615[44], 42}, {NULL}};
const Segment SGI344608[] = {{&S31615[12], 14}, {NULL}};
const Segment SGI344609[] = {{&S31616[6], 4}, {NULL}};
const Segment SGI344610[] = {{&S31616[81], 5}, {NULL}};
const Segment SGI344611[] = {{&S31616[12], 14}, {&S31616[44], 42}, {NULL}};
const Segment SGI344612[] = {{&S31616[12], 14}, {NULL}};
const Segment SGI344613[] = {{&S33468[61], 5}, {NULL}};
const Segment SGI344614[] = {{&S33468[13], 10}, {NULL}};
const Segment SGI344615[] = {{&S31617[6], 4}, {NULL}};
const Segment SGI344616[] = {{&S31617[81], 5}, {NULL}};
const Segment SGI344617[] = {{&S31617[12], 14}, {&S31617[44], 42}, {NULL}};
const Segment SGI344618[] = {{&S31617[12], 14}, {NULL}};
const Segment SGI344619[] = {{&S31618[6], 4}, {NULL}};
const Segment SGI344620[] = {{&S31618[81], 5}, {NULL}};
const Segment SGI344621[] = {{&S31618[12], 14}, {&S31618[44], 42}, {NULL}};
const Segment SGI344622[] = {{&S31618[12], 14}, {NULL}};
const Segment SGI344623[] = {{&S31619[6], 4}, {NULL}};
const Segment SGI344624[] = {{&S31619[81], 5}, {NULL}};
const Segment SGI344625[] = {{&S31619[12], 14}, {&S31619[44], 42}, {NULL}};
const Segment SGI344626[] = {{&S31619[12], 14}, {NULL}};
const Segment SGI344627[] = {{&S31620[6], 4}, {NULL}};
const Segment SGI344628[] = {{&S31620[81], 5}, {NULL}};
const Segment SGI344629[] = {{&S31620[12], 14}, {&S31620[44], 42}, {NULL}};
const Segment SGI344630[] = {{&S31620[12], 14}, {NULL}};
const Segment SGI344631[] = {{&S31621[6], 4}, {NULL}};
const Segment SGI344632[] = {{&S31621[82], 5}, {NULL}};
const Segment SGI344633[] = {{&S31621[12], 14}, {&S31621[45], 42}, {NULL}};
const Segment SGI344634[] = {{&S31621[12], 14}, {NULL}};
const Segment SGI344635[] = {{&S31622[6], 4}, {NULL}};
const Segment SGI344636[] = {{&S31622[81], 5}, {NULL}};
const Segment SGI344637[] = {{&S31622[12], 14}, {&S31622[44], 42}, {NULL}};
const Segment SGI344638[] = {{&S31622[12], 14}, {NULL}};
const Segment SGI344639[] = {{&S31623[6], 4}, {NULL}};
const Segment SGI344640[] = {{&S31623[81], 5}, {NULL}};
const Segment SGI344641[] = {{&S31623[12], 14}, {&S31623[44], 42}, {NULL}};
const Segment SGI344642[] = {{&S31623[12], 14}, {NULL}};
const Segment SGI344643[] = {{&S31624[6], 4}, {NULL}};
const Segment SGI344644[] = {{&S31624[59], 5}, {NULL}};
const Segment SGI344645[] = {{&S31624[12], 14}, {NULL}};
const Segment SGI344646[] = {{&S31625[6], 4}, {NULL}};
const Segment SGI344647[] = {{&S31625[52], 5}, {NULL}};
const Segment SGI344648[] = {{&S31625[12], 45}, {NULL}};
const Segment SGI344649[] = {{&S31625[12], 14}, {NULL}};
const Segment SGI344650[] = {{&S31626[6], 4}, {NULL}};
const Segment SGI344651[] = {{&S31626[81], 5}, {NULL}};
const Segment SGI344652[] = {{&S31626[12], 14}, {&S31626[44], 42}, {NULL}};
const Segment SGI344653[] = {{&S31626[12], 14}, {NULL}};
const Segment SGI344654[] = {{&S31627[6], 4}, {NULL}};
const Segment SGI344655[] = {{&S31627[81], 5}, {NULL}};
const Segment SGI344656[] = {{&S31627[12], 14}, {&S31627[44], 42}, {NULL}};
const Segment SGI344657[] = {{&S31627[12], 14}, {NULL}};
const Segment SGI344658[] = {{&S31628[6], 4}, {NULL}};
const Segment SGI344659[] = {{&S31628[81], 5}, {NULL}};
const Segment SGI344660[] = {{&S31628[12], 14}, {&S31628[44], 42}, {NULL}};
const Segment SGI344661[] = {{&S31628[12], 14}, {NULL}};
const Segment SGI344662[] = {{&S31629[6], 4}, {NULL}};
const Segment SGI344663[] = {{&S31629[81], 5}, {NULL}};
const Segment SGI344664[] = {{&S31629[12], 14}, {&S31629[44], 42}, {NULL}};
const Segment SGI344665[] = {{&S31629[12], 14}, {NULL}};
const Segment SGI344666[] = {{&S31630[6], 4}, {NULL}};
const Segment SGI344667[] = {{&S31630[81], 5}, {NULL}};
const Segment SGI344668[] = {{&S31630[12], 14}, {&S31630[44], 42}, {NULL}};
const Segment SGI344669[] = {{&S31630[12], 14}, {NULL}};
const Segment SGI344670[] = {{&S31631[6], 4}, {NULL}};
const Segment SGI344671[] = {{&S31631[59], 5}, {NULL}};
const Segment SGI344672[] = {{&S31631[12], 14}, {&S31631[70], 4}, {NULL}};
const Segment SGI344673[] = {{&S31631[12], 14}, {NULL}};
const Segment SGI344674[] = {{&S31632[6], 4}, {NULL}};
const Segment SGI344675[] = {{&S31632[81], 5}, {NULL}};
const Segment SGI344676[] = {{&S31632[12], 14}, {&S31632[44], 42}, {NULL}};
const Segment SGI344677[] = {{&S31632[12], 14}, {NULL}};
const Segment SGI344678[] = {{&S31633[6], 4}, {NULL}};
const Segment SGI344679[] = {{&S31633[81], 5}, {NULL}};
const Segment SGI344680[] = {{&S31633[12], 14}, {&S31633[44], 42}, {NULL}};
const Segment SGI344681[] = {{&S31633[12], 14}, {NULL}};
const Segment SGI344682[] = {{&S31634[6], 4}, {NULL}};
const Segment SGI344683[] = {{&S31634[81], 5}, {NULL}};
const Segment SGI344684[] = {{&S31634[12], 14}, {&S31634[44], 42}, {NULL}};
const Segment SGI344685[] = {{&S31634[12], 14}, {NULL}};
const Segment SGI344686[] = {{&S31635[6], 4}, {NULL}};
const Segment SGI344687[] = {{&S31635[81], 5}, {NULL}};
const Segment SGI344688[] = {{&S31635[12], 14}, {&S31635[44], 42}, {NULL}};
const Segment SGI344689[] = {{&S31635[12], 14}, {NULL}};
const Segment SGI344690[] = {{&S31636[6], 4}, {NULL}};
const Segment SGI344691[] = {{&S31636[81], 5}, {NULL}};
const Segment SGI344692[] = {{&S31636[12], 14}, {&S31636[44], 42}, {NULL}};
const Segment SGI344693[] = {{&S31636[12], 14}, {NULL}};
const Segment SGI344694[] = {{&S31637[6], 4}, {NULL}};
const Segment SGI344695[] = {{&S31637[81], 5}, {NULL}};
const Segment SGI344696[] = {{&S31637[12], 14}, {&S31637[44], 42}, {NULL}};
const Segment SGI344697[] = {{&S31637[12], 14}, {NULL}};
const Segment SGI344698[] = {{&S31638[6], 4}, {NULL}};
const Segment SGI344699[] = {{&S31638[81], 5}, {NULL}};
const Segment SGI344700[] = {{&S31638[12], 14}, {&S31638[44], 42}, {NULL}};
const Segment SGI344701[] = {{&S31638[12], 14}, {NULL}};
const Segment SGI344702[] = {{&S31639[6], 4}, {NULL}};
const Segment SGI344703[] = {{&S31639[81], 5}, {NULL}};
const Segment SGI344704[] = {{&S31639[12], 14}, {&S31639[44], 42}, {NULL}};
const Segment SGI344705[] = {{&S31639[12], 14}, {NULL}};
const Segment SGI344706[] = {{&S31640[6], 4}, {NULL}};
const Segment SGI344707[] = {{&S31640[81], 5}, {NULL}};
const Segment SGI344708[] = {{&S31640[12], 14}, {&S31640[44], 42}, {NULL}};
const Segment SGI344709[] = {{&S31640[12], 14}, {NULL}};
const Segment SGI344710[] = {{&S31641[6], 4}, {NULL}};
const Segment SGI344711[] = {{&S31641[81], 5}, {NULL}};
const Segment SGI344712[] = {{&S31641[12], 14}, {&S31641[44], 42}, {NULL}};
const Segment SGI344713[] = {{&S31641[12], 14}, {NULL}};
const Segment SGI344714[] = {{&S31642[6], 4}, {NULL}};
const Segment SGI344715[] = {{&S31642[81], 5}, {NULL}};
const Segment SGI344716[] = {{&S31642[12], 14}, {&S31642[44], 42}, {NULL}};
const Segment SGI344717[] = {{&S31642[12], 14}, {NULL}};
const Segment SGI344718[] = {{&S31643[6], 4}, {NULL}};
const Segment SGI344719[] = {{&S31643[81], 5}, {NULL}};
const Segment SGI344720[] = {{&S31643[12], 14}, {&S31643[44], 42}, {NULL}};
const Segment SGI344721[] = {{&S31643[12], 14}, {NULL}};
const Segment SGI344722[] = {{&S31644[6], 4}, {NULL}};
const Segment SGI344723[] = {{&S31644[81], 5}, {NULL}};
const Segment SGI344724[] = {{&S31644[12], 14}, {&S31644[44], 42}, {NULL}};
const Segment SGI344725[] = {{&S31644[12], 14}, {NULL}};
const Segment SGI344726[] = {{&S31645[6], 4}, {NULL}};
const Segment SGI344727[] = {{&S31645[81], 5}, {NULL}};
const Segment SGI344728[] = {{&S31645[12], 14}, {&S31645[44], 42}, {NULL}};
const Segment SGI344729[] = {{&S31645[12], 14}, {NULL}};
const Segment SGI344730[] = {{&S31646[6], 4}, {NULL}};
const Segment SGI344731[] = {{&S31646[81], 5}, {NULL}};
const Segment SGI344732[] = {{&S31646[12], 14}, {&S31646[44], 42}, {NULL}};
const Segment SGI344733[] = {{&S31646[12], 14}, {NULL}};
const Segment SGI344734[] = {{&S31647[6], 4}, {NULL}};
const Segment SGI344735[] = {{&S31647[81], 5}, {NULL}};
const Segment SGI344736[] = {{&S31647[12], 14}, {&S31647[44], 42}, {NULL}};
const Segment SGI344737[] = {{&S31647[12], 14}, {NULL}};
const Segment SGI344738[] = {{&S31648[6], 4}, {NULL}};
const Segment SGI344739[] = {{&S31648[81], 5}, {NULL}};
const Segment SGI344740[] = {{&S31648[12], 14}, {&S31648[44], 42}, {NULL}};
const Segment SGI344741[] = {{&S31648[12], 14}, {NULL}};
const Segment SGI344742[] = {{&S31649[6], 4}, {NULL}};
const Segment SGI344743[] = {{&S31649[81], 5}, {NULL}};
const Segment SGI344744[] = {{&S31649[12], 14}, {&S31649[44], 42}, {NULL}};
const Segment SGI344745[] = {{&S31649[12], 14}, {NULL}};
const Segment SGI344746[] = {{&S31650[6], 4}, {NULL}};
const Segment SGI344747[] = {{&S31650[81], 5}, {NULL}};
const Segment SGI344748[] = {{&S31650[12], 14}, {&S31650[44], 42}, {NULL}};
const Segment SGI344749[] = {{&S31650[12], 14}, {NULL}};
const Segment SGI344750[] = {{&S31651[6], 4}, {NULL}};
const Segment SGI344751[] = {{&S31651[81], 5}, {NULL}};
const Segment SGI344752[] = {{&S31651[12], 14}, {&S31651[44], 42}, {NULL}};
const Segment SGI344753[] = {{&S31651[12], 14}, {NULL}};
const Segment SGI344754[] = {{&S31652[6], 4}, {NULL}};
const Segment SGI344755[] = {{&S31652[81], 5}, {NULL}};
const Segment SGI344756[] = {{&S31652[12], 14}, {&S31652[44], 42}, {NULL}};
const Segment SGI344757[] = {{&S31652[12], 14}, {NULL}};
const Segment SGI344758[] = {{&S31654[6], 4}, {NULL}};
const Segment SGI344759[] = {{&S31654[52], 5}, {NULL}};
const Segment SGI344760[] = {{&S31654[12], 55}, {NULL}};
const Segment SGI344761[] = {{&S31654[12], 14}, {NULL}};
const Segment SGI344762[] = {{&S31655[6], 4}, {NULL}};
const Segment SGI344763[] = {{&S31655[81], 5}, {NULL}};
const Segment SGI344764[] = {{&S31655[12], 14}, {&S31655[44], 42}, {NULL}};
const Segment SGI344765[] = {{&S31655[12], 14}, {NULL}};
const Segment SGI344766[] = {{&S31656[6], 4}, {NULL}};
const Segment SGI344767[] = {{&S31656[81], 5}, {NULL}};
const Segment SGI344768[] = {{&S31656[12], 14}, {&S31656[44], 42}, {NULL}};
const Segment SGI344769[] = {{&S31656[12], 14}, {NULL}};
const Segment SGI344770[] = {{&S31657[6], 4}, {NULL}};
const Segment SGI344771[] = {{&S31657[81], 5}, {NULL}};
const Segment SGI344772[] = {{&S31657[12], 14}, {&S31657[44], 42}, {NULL}};
const Segment SGI344773[] = {{&S31657[12], 14}, {NULL}};
const Segment SGI344774[] = {{&S58376[5], 5}, {&S58376[97], 5}, {NULL}};
const Segment SGI344775[] = {{&S58376[113], 3}, {NULL}};
const Segment SGI344776[] = {{&S31658[6], 4}, {NULL}};
const Segment SGI344777[] = {{&S31658[81], 5}, {NULL}};
const Segment SGI344778[] = {{&S31658[12], 14}, {&S31658[44], 42}, {NULL}};
const Segment SGI344779[] = {{&S31658[12], 14}, {NULL}};
const Segment SGI344780[] = {{&S31659[6], 4}, {NULL}};
const Segment SGI344781[] = {{&S31659[81], 5}, {NULL}};
const Segment SGI344782[] = {{&S31659[12], 14}, {&S31659[44], 42}, {NULL}};
const Segment SGI344783[] = {{&S31659[12], 14}, {NULL}};
const Segment SGI344784[] = {{&S31660[6], 4}, {NULL}};
const Segment SGI344785[] = {{&S31660[81], 5}, {NULL}};
const Segment SGI344786[] = {{&S31660[12], 14}, {&S31660[44], 42}, {NULL}};
const Segment SGI344787[] = {{&S31660[12], 14}, {NULL}};
const Segment SGI344788[] = {{&S31662[6], 4}, {NULL}};
const Segment SGI344789[] = {{&S31662[81], 5}, {NULL}};
const Segment SGI344790[] = {{&S31662[12], 14}, {&S31662[44], 42}, {NULL}};
const Segment SGI344791[] = {{&S31662[12], 14}, {NULL}};
const Segment SGI344792[] = {{&S31663[6], 4}, {NULL}};
const Segment SGI344793[] = {{&S31663[81], 5}, {NULL}};
const Segment SGI344794[] = {{&S31663[12], 14}, {&S31663[44], 42}, {NULL}};
const Segment SGI344795[] = {{&S31663[12], 14}, {NULL}};
const Segment SGI344796[] = {{&S31664[6], 4}, {NULL}};
const Segment SGI344797[] = {{&S31664[81], 5}, {NULL}};
const Segment SGI344798[] = {{&S31664[12], 14}, {&S31664[44], 42}, {NULL}};
const Segment SGI344799[] = {{&S31664[12], 14}, {NULL}};
const Segment SGI344800[] = {{&S31665[6], 4}, {NULL}};
const Segment SGI344801[] = {{&S31665[82], 5}, {NULL}};
const Segment SGI344802[] = {{&S31665[12], 14}, {&S31665[45], 42}, {NULL}};
const Segment SGI344803[] = {{&S31665[12], 14}, {NULL}};
const Segment SGI344804[] = {{&S31666[6], 4}, {NULL}};
const Segment SGI344805[] = {{&S31666[81], 5}, {NULL}};
const Segment SGI344806[] = {{&S31666[12], 14}, {&S31666[44], 42}, {NULL}};
const Segment SGI344807[] = {{&S31666[12], 14}, {NULL}};
const Segment SGI344808[] = {{&S31667[6], 4}, {NULL}};
const Segment SGI344809[] = {{&S31667[81], 5}, {NULL}};
const Segment SGI344810[] = {{&S31667[12], 14}, {&S31667[44], 42}, {NULL}};
const Segment SGI344811[] = {{&S31667[12], 14}, {NULL}};
const Segment SGI344812[] = {{&S31668[6], 4}, {NULL}};
const Segment SGI344813[] = {{&S31668[81], 5}, {NULL}};
const Segment SGI344814[] = {{&S31668[12], 14}, {&S31668[44], 42}, {NULL}};
const Segment SGI344815[] = {{&S31668[12], 14}, {NULL}};
const Segment SGI344816[] = {{&S31669[6], 4}, {NULL}};
const Segment SGI344817[] = {{&S31669[81], 5}, {NULL}};
const Segment SGI344818[] = {{&S31669[12], 14}, {&S31669[44], 42}, {NULL}};
const Segment SGI344819[] = {{&S31669[12], 14}, {NULL}};
const Segment SGI344820[] = {{&S31670[6], 4}, {NULL}};
const Segment SGI344821[] = {{&S31670[81], 5}, {NULL}};
const Segment SGI344822[] = {{&S31670[12], 14}, {&S31670[44], 42}, {NULL}};
const Segment SGI344823[] = {{&S31670[12], 14}, {NULL}};
const Segment SGI344824[] = {{&S31671[6], 4}, {NULL}};
const Segment SGI344825[] = {{&S31671[81], 5}, {NULL}};
const Segment SGI344826[] = {{&S31671[12], 14}, {&S31671[44], 42}, {NULL}};
const Segment SGI344827[] = {{&S31671[12], 14}, {NULL}};
const Segment SGI344828[] = {{&S31672[6], 4}, {NULL}};
const Segment SGI344829[] = {{&S31672[81], 5}, {NULL}};
const Segment SGI344830[] = {{&S31672[12], 14}, {&S31672[44], 42}, {NULL}};
const Segment SGI344831[] = {{&S31672[12], 14}, {NULL}};
const Segment SGI344832[] = {{&S31673[6], 4}, {NULL}};
const Segment SGI344833[] = {{&S31673[59], 5}, {NULL}};
const Segment SGI344834[] = {{&S31673[12], 14}, {&S31673[70], 14}, {NULL}};
const Segment SGI344835[] = {{&S31673[12], 14}, {NULL}};
const Segment SGI344836[] = {{&S31675[6], 4}, {NULL}};
const Segment SGI344837[] = {{&S31675[81], 5}, {NULL}};
const Segment SGI344838[] = {{&S31675[12], 14}, {&S31675[44], 42}, {NULL}};
const Segment SGI344839[] = {{&S31675[12], 14}, {NULL}};
const Segment SGI344840[] = {{&S31676[6], 4}, {NULL}};
const Segment SGI344841[] = {{&S31676[81], 5}, {NULL}};
const Segment SGI344842[] = {{&S31676[12], 14}, {&S31676[44], 42}, {NULL}};
const Segment SGI344843[] = {{&S31676[12], 14}, {NULL}};
const Segment SGI344844[] = {{&S31677[6], 4}, {NULL}};
const Segment SGI344845[] = {{&S31677[81], 5}, {NULL}};
const Segment SGI344846[] = {{&S31677[12], 14}, {&S31677[44], 42}, {NULL}};
const Segment SGI344847[] = {{&S31677[12], 14}, {NULL}};
const Segment SGI344848[] = {{&S31678[6], 4}, {NULL}};
const Segment SGI344849[] = {{&S31678[81], 5}, {NULL}};
const Segment SGI344850[] = {{&S31678[12], 14}, {&S31678[44], 42}, {NULL}};
const Segment SGI344851[] = {{&S31678[12], 14}, {NULL}};
const Segment SGI344852[] = {{&S31679[6], 4}, {NULL}};
const Segment SGI344853[] = {{&S31679[81], 5}, {NULL}};
const Segment SGI344854[] = {{&S31679[12], 14}, {&S31679[44], 42}, {NULL}};
const Segment SGI344855[] = {{&S31679[12], 14}, {NULL}};
const Segment SGI344856[] = {{&S31680[6], 4}, {NULL}};
const Segment SGI344857[] = {{&S31680[81], 5}, {NULL}};
const Segment SGI344858[] = {{&S31680[12], 14}, {&S31680[44], 42}, {NULL}};
const Segment SGI344859[] = {{&S31680[12], 14}, {NULL}};
const Segment SGI344860[] = {{&S31681[6], 4}, {NULL}};
const Segment SGI344861[] = {{&S31681[81], 5}, {NULL}};
const Segment SGI344862[] = {{&S31681[12], 14}, {&S31681[44], 42}, {NULL}};
const Segment SGI344863[] = {{&S31681[12], 14}, {NULL}};
const Segment SGI344864[] = {{&S31682[6], 4}, {NULL}};
const Segment SGI344865[] = {{&S31682[81], 5}, {NULL}};
const Segment SGI344866[] = {{&S31682[12], 14}, {&S31682[44], 42}, {NULL}};
const Segment SGI344867[] = {{&S31682[12], 14}, {NULL}};
const Segment SGI344868[] = {{&S31683[6], 4}, {NULL}};
const Segment SGI344869[] = {{&S31683[81], 5}, {NULL}};
const Segment SGI344870[] = {{&S31683[12], 14}, {&S31683[44], 42}, {NULL}};
const Segment SGI344871[] = {{&S31683[12], 14}, {NULL}};
const Segment SGI344872[] = {{&S31684[6], 4}, {NULL}};
const Segment SGI344873[] = {{&S31684[81], 5}, {NULL}};
const Segment SGI344874[] = {{&S31684[12], 14}, {&S31684[44], 42}, {NULL}};
const Segment SGI344875[] = {{&S31684[12], 14}, {NULL}};
const Segment SGI344876[] = {{&S31685[6], 4}, {NULL}};
const Segment SGI344877[] = {{&S31685[81], 5}, {NULL}};
const Segment SGI344878[] = {{&S31685[12], 14}, {&S31685[44], 42}, {NULL}};
const Segment SGI344879[] = {{&S31685[12], 14}, {NULL}};
const Segment SGI344880[] = {{&S31686[6], 4}, {NULL}};
const Segment SGI344881[] = {{&S31686[81], 5}, {NULL}};
const Segment SGI344882[] = {{&S31686[12], 14}, {&S31686[44], 42}, {NULL}};
const Segment SGI344883[] = {{&S31686[12], 14}, {NULL}};
const Segment SGI344884[] = {{&S31687[6], 4}, {NULL}};
const Segment SGI344885[] = {{&S31687[81], 5}, {NULL}};
const Segment SGI344886[] = {{&S31687[12], 14}, {&S31687[44], 42}, {NULL}};
const Segment SGI344887[] = {{&S31687[12], 14}, {NULL}};
const Segment SGI344888[] = {{&S31688[6], 4}, {NULL}};
const Segment SGI344889[] = {{&S31688[81], 5}, {NULL}};
const Segment SGI344890[] = {{&S31688[12], 14}, {&S31688[44], 42}, {NULL}};
const Segment SGI344891[] = {{&S31688[12], 14}, {NULL}};
const Segment SGI344892[] = {{&S31689[6], 4}, {NULL}};
const Segment SGI344893[] = {{&S31689[81], 5}, {NULL}};
const Segment SGI344894[] = {{&S31689[12], 14}, {&S31689[44], 42}, {NULL}};
const Segment SGI344895[] = {{&S31689[12], 14}, {NULL}};
const Segment SGI344896[] = {{&S58417[5], 7}, {NULL}};
const Segment SGI344897[] = {{&S58418[5], 7}, {&S58418[5], 7}, {NULL}};
const Segment SGI344898[] = {{&S58419[5], 8}, {NULL}};
const Segment SGI344899[] = {{&S58420[5], 5}, {NULL}};
const Segment SGI344900[] = {{&S58421[5], 10}, {NULL}};
const Segment SGI344901[] = {{&S58428[5], 5}, {&S58428[104], 5}, {NULL}};
const Segment SGI344902[] = {{&S58428[11], 3}, {NULL}};
const Segment SGI344903[] = {{&S58428[130], 3}, {NULL}};
const Segment SGI344904[] = {{&S58491[5], 4}, {NULL}};
const Segment SGI344905[] = {{&S31843[6], 4}, {NULL}};
const Segment SGI344906[] = {{&S31843[57], 5}, {NULL}};
const Segment SGI344907[] = {{&S31843[12], 14}, {NULL}};
const Segment SGI344908[] = {{&S31844[6], 4}, {NULL}};
const Segment SGI344909[] = {{&S31844[57], 5}, {NULL}};
const Segment SGI344910[] = {{&S31844[12], 14}, {NULL}};
const Segment SGI344911[] = {{&S31845[6], 4}, {NULL}};
const Segment SGI344912[] = {{&S31845[80], 5}, {NULL}};
const Segment SGI344913[] = {{&S31845[12], 14}, {&S31845[40], 12}, {NULL}};
const Segment SGI344914[] = {{&S31845[12], 14}, {NULL}};
const Segment SGI344915[] = {{&S31846[6], 4}, {NULL}};
const Segment SGI344916[] = {{&S31846[80], 5}, {NULL}};
const Segment SGI344917[] = {{&S31846[12], 14}, {&S31846[40], 12}, {NULL}};
const Segment SGI344918[] = {{&S31846[12], 14}, {NULL}};
const Segment SGI344919[] = {{&S31847[6], 4}, {NULL}};
const Segment SGI344920[] = {{&S31847[80], 5}, {NULL}};
const Segment SGI344921[] = {{&S31847[12], 14}, {&S31847[40], 12}, {NULL}};
const Segment SGI344922[] = {{&S31847[12], 14}, {NULL}};
const Segment SGI344923[] = {{&S31848[6], 4}, {NULL}};
const Segment SGI344924[] = {{&S31848[80], 5}, {NULL}};
const Segment SGI344925[] = {{&S31848[12], 14}, {&S31848[40], 12}, {NULL}};
const Segment SGI344926[] = {{&S31848[12], 14}, {NULL}};
const Segment SGI344927[] = {{&S31849[6], 4}, {NULL}};
const Segment SGI344928[] = {{&S31849[57], 5}, {NULL}};
const Segment SGI344929[] = {{&S31849[12], 14}, {&S31849[68], 31}, {NULL}};
const Segment SGI344930[] = {{&S31849[12], 14}, {NULL}};
const Segment SGI344931[] = {{&S31850[6], 4}, {NULL}};
const Segment SGI344932[] = {{&S31850[79], 5}, {NULL}};
const Segment SGI344933[] = {{&S31850[12], 14}, {&S31850[40], 11}, {NULL}};
const Segment SGI344934[] = {{&S31850[12], 14}, {NULL}};
const Segment SGI344935[] = {{&S31851[6], 4}, {NULL}};
const Segment SGI344936[] = {{&S31851[79], 5}, {NULL}};
const Segment SGI344937[] = {{&S31851[12], 14}, {&S31851[40], 11}, {NULL}};
const Segment SGI344938[] = {{&S31851[12], 14}, {NULL}};
const Segment SGI344939[] = {{&S31852[6], 4}, {NULL}};
const Segment SGI344940[] = {{&S31852[80], 5}, {NULL}};
const Segment SGI344941[] = {{&S31852[12], 14}, {&S31852[40], 12}, {NULL}};
const Segment SGI344942[] = {{&S31852[12], 14}, {NULL}};
const Segment SGI344943[] = {{&S31853[6], 4}, {NULL}};
const Segment SGI344944[] = {{&S31853[80], 5}, {NULL}};
const Segment SGI344945[] = {{&S31853[12], 14}, {&S31853[40], 12}, {NULL}};
const Segment SGI344946[] = {{&S31853[12], 14}, {NULL}};
const Segment SGI344947[] = {{&S31854[6], 4}, {NULL}};
const Segment SGI344948[] = {{&S31854[79], 5}, {NULL}};
const Segment SGI344949[] = {{&S31854[12], 14}, {&S31854[40], 11}, {NULL}};
const Segment SGI344950[] = {{&S31854[12], 14}, {NULL}};
const Segment SGI344951[] = {{&S31855[6], 4}, {NULL}};
const Segment SGI344952[] = {{&S31855[79], 5}, {NULL}};
const Segment SGI344953[] = {{&S31855[12], 14}, {&S31855[40], 11}, {NULL}};
const Segment SGI344954[] = {{&S31855[12], 14}, {NULL}};
const Segment SGI344955[] = {{&S31856[6], 4}, {NULL}};
const Segment SGI344956[] = {{&S31856[79], 5}, {NULL}};
const Segment SGI344957[] = {{&S31856[12], 14}, {&S31856[40], 11}, {NULL}};
const Segment SGI344958[] = {{&S31856[12], 14}, {NULL}};
const Segment SGI344959[] = {{&S31857[6], 4}, {NULL}};
const Segment SGI344960[] = {{&S31857[80], 5}, {NULL}};
const Segment SGI344961[] = {{&S31857[12], 14}, {&S31857[41], 11}, {NULL}};
const Segment SGI344962[] = {{&S31857[12], 14}, {NULL}};
const Segment SGI344963[] = {{&S31858[6], 4}, {NULL}};
const Segment SGI344964[] = {{&S31858[80], 5}, {NULL}};
const Segment SGI344965[] = {{&S31858[12], 14}, {&S31858[41], 11}, {NULL}};
const Segment SGI344966[] = {{&S31858[12], 14}, {NULL}};
const Segment SGI344967[] = {{&S31859[6], 4}, {NULL}};
const Segment SGI344968[] = {{&S31859[79], 5}, {NULL}};
const Segment SGI344969[] = {{&S31859[12], 14}, {&S31859[40], 11}, {NULL}};
const Segment SGI344970[] = {{&S31859[12], 14}, {NULL}};
const Segment SGI344971[] = {{&S31860[6], 4}, {NULL}};
const Segment SGI344972[] = {{&S31860[79], 5}, {NULL}};
const Segment SGI344973[] = {{&S31860[12], 14}, {&S31860[40], 11}, {NULL}};
const Segment SGI344974[] = {{&S31860[12], 14}, {NULL}};
const Segment SGI344975[] = {{&S31862[6], 4}, {NULL}};
const Segment SGI344976[] = {{&S31862[80], 5}, {NULL}};
const Segment SGI344977[] = {{&S31862[12], 14}, {&S31862[41], 11}, {NULL}};
const Segment SGI344978[] = {{&S31862[12], 14}, {NULL}};
const Segment SGI344979[] = {{&S31864[6], 4}, {NULL}};
const Segment SGI344980[] = {{&S31864[80], 5}, {NULL}};
const Segment SGI344981[] = {{&S31864[12], 14}, {&S31864[41], 11}, {NULL}};
const Segment SGI344982[] = {{&S31864[12], 14}, {NULL}};
const Segment SGI344983[] = {{&S31865[6], 4}, {NULL}};
const Segment SGI344984[] = {{&S31865[80], 5}, {NULL}};
const Segment SGI344985[] = {{&S31865[12], 14}, {&S31865[40], 12}, {NULL}};
const Segment SGI344986[] = {{&S31865[12], 14}, {NULL}};
const Segment SGI344987[] = {{&S31866[6], 4}, {NULL}};
const Segment SGI344988[] = {{&S31866[79], 5}, {NULL}};
const Segment SGI344989[] = {{&S31866[12], 14}, {&S31866[40], 11}, {NULL}};
const Segment SGI344990[] = {{&S31866[12], 14}, {NULL}};
const Segment SGI344991[] = {{&S31867[6], 4}, {NULL}};
const Segment SGI344992[] = {{&S31867[79], 5}, {NULL}};
const Segment SGI344993[] = {{&S31867[12], 14}, {&S31867[40], 11}, {NULL}};
const Segment SGI344994[] = {{&S31867[12], 14}, {NULL}};
const Segment SGI344995[] = {{&S31868[6], 4}, {NULL}};
const Segment SGI344996[] = {{&S31868[80], 5}, {NULL}};
const Segment SGI344997[] = {{&S31868[12], 14}, {&S31868[41], 11}, {NULL}};
const Segment SGI344998[] = {{&S31868[12], 14}, {NULL}};
const Segment SGI344999[] = {{&S31869[6], 4}, {NULL}};
const Segment SGI345000[] = {{&S31869[79], 5}, {NULL}};
const Segment SGI345001[] = {{&S31869[12], 14}, {&S31869[40], 11}, {NULL}};
const Segment SGI345002[] = {{&S31869[12], 14}, {NULL}};
const Segment SGI345003[] = {{&S31870[6], 4}, {NULL}};
const Segment SGI345004[] = {{&S31870[80], 5}, {NULL}};
const Segment SGI345005[] = {{&S31870[12], 14}, {&S31870[41], 11}, {NULL}};
const Segment SGI345006[] = {{&S31870[12], 14}, {NULL}};
const Segment SGI345007[] = {{&S31871[6], 4}, {NULL}};
const Segment SGI345008[] = {{&S31871[80], 5}, {NULL}};
const Segment SGI345009[] = {{&S31871[12], 14}, {&S31871[41], 11}, {NULL}};
const Segment SGI345010[] = {{&S31871[12], 14}, {NULL}};
const Segment SGI345011[] = {{&S31872[6], 4}, {NULL}};
const Segment SGI345012[] = {{&S31872[80], 5}, {NULL}};
const Segment SGI345013[] = {{&S31872[12], 14}, {&S31872[41], 11}, {NULL}};
const Segment SGI345014[] = {{&S31872[12], 14}, {NULL}};
const Segment SGI345015[] = {{&S31873[6], 4}, {NULL}};
const Segment SGI345016[] = {{&S31873[79], 5}, {NULL}};
const Segment SGI345017[] = {{&S31873[12], 14}, {&S31873[40], 11}, {NULL}};
const Segment SGI345018[] = {{&S31873[12], 14}, {NULL}};
const Segment SGI345019[] = {{&S31874[6], 4}, {NULL}};
const Segment SGI345020[] = {{&S31874[79], 5}, {NULL}};
const Segment SGI345021[] = {{&S31874[12], 14}, {&S31874[40], 11}, {NULL}};
const Segment SGI345022[] = {{&S31874[12], 14}, {NULL}};
const Segment SGI345023[] = {{&S58550[5], 5}, {&S58550[97], 5}, {NULL}};
const Segment SGI345024[] = {{&S58550[153], 3}, {NULL}};
const Segment SGI345025[] = {{&S31875[6], 4}, {NULL}};
const Segment SGI345026[] = {{&S31875[79], 5}, {NULL}};
const Segment SGI345027[] = {{&S31875[12], 14}, {&S31875[40], 11}, {NULL}};
const Segment SGI345028[] = {{&S31875[12], 14}, {NULL}};
const Segment SGI345029[] = {{&S31876[6], 4}, {NULL}};
const Segment SGI345030[] = {{&S31876[80], 5}, {NULL}};
const Segment SGI345031[] = {{&S31876[12], 14}, {&S31876[40], 12}, {NULL}};
const Segment SGI345032[] = {{&S31876[12], 14}, {NULL}};
const Segment SGI345033[] = {{&S31877[6], 4}, {NULL}};
const Segment SGI345034[] = {{&S31877[80], 5}, {NULL}};
const Segment SGI345035[] = {{&S31877[12], 14}, {&S31877[41], 11}, {NULL}};
const Segment SGI345036[] = {{&S31877[12], 14}, {NULL}};
const Segment SGI345037[] = {{&S31878[6], 4}, {NULL}};
const Segment SGI345038[] = {{&S31878[79], 5}, {NULL}};
const Segment SGI345039[] = {{&S31878[12], 14}, {&S31878[40], 11}, {NULL}};
const Segment SGI345040[] = {{&S31878[12], 14}, {NULL}};
const Segment SGI345041[] = {{&S31879[6], 4}, {NULL}};
const Segment SGI345042[] = {{&S31879[79], 5}, {NULL}};
const Segment SGI345043[] = {{&S31879[12], 14}, {&S31879[40], 11}, {NULL}};
const Segment SGI345044[] = {{&S31879[12], 14}, {NULL}};
const Segment SGI345045[] = {{&S31880[6], 4}, {NULL}};
const Segment SGI345046[] = {{&S31880[79], 5}, {NULL}};
const Segment SGI345047[] = {{&S31880[12], 14}, {&S31880[40], 11}, {NULL}};
const Segment SGI345048[] = {{&S31880[12], 14}, {NULL}};
const Segment SGI345049[] = {{&S31881[6], 4}, {NULL}};
const Segment SGI345050[] = {{&S31881[80], 5}, {NULL}};
const Segment SGI345051[] = {{&S31881[12], 14}, {&S31881[40], 12}, {NULL}};
const Segment SGI345052[] = {{&S31881[12], 14}, {NULL}};
const Segment SGI345053[] = {{&S31882[6], 4}, {NULL}};
const Segment SGI345054[] = {{&S31882[80], 5}, {NULL}};
const Segment SGI345055[] = {{&S31882[12], 14}, {&S31882[40], 12}, {NULL}};
const Segment SGI345056[] = {{&S31882[12], 14}, {NULL}};
const Segment SGI345057[] = {{&S31883[6], 4}, {NULL}};
const Segment SGI345058[] = {{&S31883[79], 5}, {NULL}};
const Segment SGI345059[] = {{&S31883[12], 14}, {&S31883[40], 11}, {NULL}};
const Segment SGI345060[] = {{&S31883[12], 14}, {NULL}};
const Segment SGI345061[] = {{&S31884[6], 4}, {NULL}};
const Segment SGI345062[] = {{&S31884[79], 5}, {NULL}};
const Segment SGI345063[] = {{&S31884[12], 14}, {&S31884[40], 11}, {NULL}};
const Segment SGI345064[] = {{&S31884[12], 14}, {NULL}};
const Segment SGI345065[] = {{&S31885[6], 4}, {NULL}};
const Segment SGI345066[] = {{&S31885[79], 5}, {NULL}};
const Segment SGI345067[] = {{&S31885[12], 14}, {&S31885[40], 11}, {NULL}};
const Segment SGI345068[] = {{&S31885[12], 14}, {NULL}};
const Segment SGI345069[] = {{&S31886[6], 4}, {NULL}};
const Segment SGI345070[] = {{&S31886[80], 5}, {NULL}};
const Segment SGI345071[] = {{&S31886[12], 14}, {&S31886[41], 11}, {NULL}};
const Segment SGI345072[] = {{&S31886[12], 14}, {NULL}};
const Segment SGI345073[] = {{&S31887[6], 4}, {NULL}};
const Segment SGI345074[] = {{&S31887[57], 5}, {NULL}};
const Segment SGI345075[] = {{&S31887[12], 14}, {&S31887[68], 4}, {NULL}};
const Segment SGI345076[] = {{&S31887[12], 14}, {NULL}};
const Segment SGI345077[] = {{&S58573[42], 5}, {NULL}};
const Segment SGI345078[] = {{&S58573[6], 3}, {NULL}};
const Segment SGI345079[] = {{&S31888[6], 4}, {NULL}};
const Segment SGI345080[] = {{&S58573[68], 3}, {NULL}};
const Segment SGI345081[] = {{&S31888[79], 5}, {NULL}};
const Segment SGI345082[] = {{&S31888[12], 14}, {&S31888[40], 11}, {NULL}};
const Segment SGI345083[] = {{&S31888[12], 14}, {NULL}};
const Segment SGI345084[] = {{&S58574[43], 5}, {NULL}};
const Segment SGI345085[] = {{&S31889[6], 4}, {NULL}};
const Segment SGI345086[] = {{&S31889[79], 5}, {NULL}};
const Segment SGI345087[] = {{&S58574[6], 3}, {NULL}};
const Segment SGI345088[] = {{&S31889[12], 14}, {&S31889[40], 11}, {NULL}};
const Segment SGI345089[] = {{&S31889[12], 14}, {NULL}};
const Segment SGI345090[] = {{&S58574[69], 3}, {NULL}};
const Segment SGI345091[] = {{&S31890[6], 4}, {NULL}};
const Segment SGI345092[] = {{&S31890[79], 5}, {NULL}};
const Segment SGI345093[] = {{&S58575[43], 7}, {NULL}};
const Segment SGI345094[] = {{&S31890[12], 14}, {&S31890[40], 11}, {NULL}};
const Segment SGI345095[] = {{&S58575[6], 3}, {NULL}};
const Segment SGI345096[] = {{&S31890[12], 14}, {NULL}};
const Segment SGI345097[] = {{&S58575[71], 3}, {NULL}};
const Segment SGI345098[] = {{&S31891[6], 4}, {NULL}};
const Segment SGI345099[] = {{&S31891[79], 5}, {NULL}};
const Segment SGI345100[] = {{&S31891[12], 14}, {&S31891[40], 11}, {NULL}};
const Segment SGI345101[] = {{&S31891[12], 14}, {NULL}};
const Segment SGI345102[] = {{&S31892[6], 4}, {NULL}};
const Segment SGI345103[] = {{&S31892[79], 5}, {NULL}};
const Segment SGI345104[] = {{&S31892[12], 14}, {&S31892[40], 11}, {NULL}};
const Segment SGI345105[] = {{&S31892[12], 14}, {NULL}};
const Segment SGI345106[] = {{&S31893[6], 4}, {NULL}};
const Segment SGI345107[] = {{&S31893[79], 5}, {NULL}};
const Segment SGI345108[] = {{&S31893[12], 14}, {&S31893[40], 11}, {NULL}};
const Segment SGI345109[] = {{&S31893[12], 14}, {NULL}};
const Segment SGI345110[] = {{&S31894[6], 4}, {NULL}};
const Segment SGI345111[] = {{&S31894[79], 5}, {NULL}};
const Segment SGI345112[] = {{&S31894[12], 14}, {&S31894[40], 11}, {NULL}};
const Segment SGI345113[] = {{&S31894[12], 14}, {NULL}};
const Segment SGI345114[] = {{&S31895[6], 4}, {NULL}};
const Segment SGI345115[] = {{&S31895[79], 5}, {NULL}};
const Segment SGI345116[] = {{&S31895[12], 14}, {&S31895[40], 11}, {NULL}};
const Segment SGI345117[] = {{&S31895[12], 14}, {NULL}};
const Segment SGI345118[] = {{&S58586[5], 4}, {NULL}};
const Segment SGI345119[] = {{&S44669[42], 3}, {&S44669[145], 7}, {NULL}};
const Segment SGI345120[] = {{&S44669[33], 3}, {NULL}};
const Segment SGI345121[] = {{&S44670[42], 3}, {&S44670[145], 7}, {NULL}};
const Segment SGI345122[] = {{&S44670[33], 3}, {NULL}};
const Segment SGI345123[] = {{&S44671[42], 3}, {&S44671[145], 7}, {NULL}};
const Segment SGI345124[] = {{&S44671[33], 3}, {NULL}};
const Segment SGI345125[] = {{&S58604[5], 5}, {&S58604[97], 5}, {NULL}};
const Segment SGI345126[] = {{&S58604[113], 3}, {NULL}};
const Segment SGI345127[] = {{&S31277[0], 21}, {NULL}};
const Segment SGI345128[] = {{&S31277[34], 3}, {NULL}};
const Segment SGI345129[] = {{&S31278[0], 15}, {NULL}};
const Segment SGI345130[] = {{&S31278[28], 3}, {NULL}};
const Segment SGI345131[] = {{&S31278[66], 13}, {NULL}};
const Segment SGI345132[] = {{&S49708[28], 3}, {NULL}};
const Segment SGI345133[] = {{&S49708[102], 5}, {NULL}};
const Segment SGI345134[] = {{&S58645[5], 3}, {&S58645[5], 3}, {NULL}};
const Segment SGI345135[] = {{&S58645[109], 3}, {NULL}};
const Segment SGI345136[] = {{&S33933[57], 5}, {NULL}};
const Segment SGI345137[] = {{&S33933[13], 10}, {NULL}};
const Segment SGI345138[] = {{&S58703[5], 5}, {&S58703[105], 5}, {NULL}};
const Segment SGI345139[] = {{&S58703[11], 3}, {NULL}};
const Segment SGI345140[] = {{&S58703[131], 3}, {NULL}};
const Segment SGI345141[] = {{&S58726[5], 5}, {&S58726[105], 5}, {NULL}};
const Segment SGI345142[] = {{&S58726[69], 3}, {NULL}};
const Segment SGI345143[] = {{&S58726[131], 3}, {NULL}};
const Segment SGI345144[] = {{&S58727[5], 7}, {&S58727[107], 7}, {NULL}};
const Segment SGI345145[] = {{&S58727[71], 3}, {NULL}};
const Segment SGI345146[] = {{&S58727[135], 3}, {NULL}};
const Segment SGI345147[] = {{&S58728[42], 5}, {NULL}};
const Segment SGI345148[] = {{&S58728[6], 3}, {NULL}};
const Segment SGI345149[] = {{&S58728[105], 17}, {NULL}};
const Segment SGI345150[] = {{&S58728[68], 3}, {NULL}};
const Segment SGI345151[] = {{&S32149[6], 4}, {NULL}};
const Segment SGI345152[] = {{&S32149[57], 5}, {NULL}};
const Segment SGI345153[] = {{&S32149[12], 14}, {NULL}};
const Segment SGI345154[] = {{&S58729[5], 4}, {NULL}};
const Segment SGI345155[] = {{&S32150[6], 4}, {NULL}};
const Segment SGI345156[] = {{&S32150[57], 5}, {NULL}};
const Segment SGI345157[] = {{&S32150[12], 14}, {NULL}};
const Segment SGI345158[] = {{&S58730[5], 4}, {&S58730[5], 4}, {NULL}};
const Segment SGI345159[] = {{&S32151[6], 4}, {NULL}};
const Segment SGI345160[] = {{&S32151[57], 5}, {NULL}};
const Segment SGI345161[] = {{&S32151[12], 14}, {NULL}};
const Segment SGI345162[] = {{&S32152[6], 5}, {NULL}};
const Segment SGI345163[] = {{&S32152[13], 14}, {NULL}};
const Segment SGI345164[] = {{&S32153[6], 4}, {NULL}};
const Segment SGI345165[] = {{&S32153[80], 5}, {NULL}};
const Segment SGI345166[] = {{&S32153[12], 14}, {&S32153[40], 12}, {NULL}};
const Segment SGI345167[] = {{&S32153[12], 14}, {NULL}};
const Segment SGI345168[] = {{&S32154[6], 4}, {NULL}};
const Segment SGI345169[] = {{&S32154[80], 5}, {NULL}};
const Segment SGI345170[] = {{&S32154[12], 14}, {&S32154[40], 12}, {NULL}};
const Segment SGI345171[] = {{&S32154[12], 14}, {NULL}};
const Segment SGI345172[] = {{&S32155[6], 4}, {NULL}};
const Segment SGI345173[] = {{&S32155[79], 5}, {NULL}};
const Segment SGI345174[] = {{&S32155[12], 14}, {&S32155[40], 11}, {NULL}};
const Segment SGI345175[] = {{&S32155[12], 14}, {NULL}};
const Segment SGI345176[] = {{&S32156[6], 4}, {NULL}};
const Segment SGI345177[] = {{&S32156[80], 5}, {NULL}};
const Segment SGI345178[] = {{&S32156[12], 14}, {&S32156[40], 12}, {NULL}};
const Segment SGI345179[] = {{&S32156[12], 14}, {NULL}};
const Segment SGI345180[] = {{&S32157[6], 4}, {NULL}};
const Segment SGI345181[] = {{&S32157[80], 5}, {NULL}};
const Segment SGI345182[] = {{&S32157[12], 14}, {&S32157[40], 12}, {NULL}};
const Segment SGI345183[] = {{&S32157[12], 14}, {NULL}};
const Segment SGI345184[] = {{&S32158[6], 4}, {NULL}};
const Segment SGI345185[] = {{&S32158[57], 5}, {NULL}};
const Segment SGI345186[] = {{&S32158[12], 14}, {NULL}};
const Segment SGI345187[] = {{&S32160[6], 4}, {NULL}};
const Segment SGI345188[] = {{&S32160[80], 5}, {NULL}};
const Segment SGI345189[] = {{&S32160[12], 14}, {&S32160[40], 12}, {NULL}};
const Segment SGI345190[] = {{&S32160[12], 14}, {NULL}};
const Segment SGI345191[] = {{&S32161[6], 4}, {NULL}};
const Segment SGI345192[] = {{&S32161[79], 5}, {NULL}};
const Segment SGI345193[] = {{&S32161[12], 14}, {&S32161[40], 11}, {NULL}};
const Segment SGI345194[] = {{&S32161[12], 14}, {NULL}};
const Segment SGI345195[] = {{&S32162[6], 4}, {NULL}};
const Segment SGI345196[] = {{&S32162[79], 5}, {NULL}};
const Segment SGI345197[] = {{&S32162[12], 14}, {&S32162[40], 11}, {NULL}};
const Segment SGI345198[] = {{&S32162[12], 14}, {NULL}};
const Segment SGI345199[] = {{&S32163[6], 4}, {NULL}};
const Segment SGI345200[] = {{&S32163[80], 5}, {NULL}};
const Segment SGI345201[] = {{&S32163[12], 14}, {&S32163[41], 11}, {NULL}};
const Segment SGI345202[] = {{&S32163[12], 14}, {NULL}};
const Segment SGI345203[] = {{&S32164[6], 4}, {NULL}};
const Segment SGI345204[] = {{&S32164[79], 5}, {NULL}};
const Segment SGI345205[] = {{&S32164[12], 14}, {&S32164[40], 11}, {NULL}};
const Segment SGI345206[] = {{&S32164[12], 14}, {NULL}};
const Segment SGI345207[] = {{&S32165[6], 4}, {NULL}};
const Segment SGI345208[] = {{&S32165[80], 5}, {NULL}};
const Segment SGI345209[] = {{&S32165[12], 14}, {&S32165[41], 11}, {NULL}};
const Segment SGI345210[] = {{&S32165[12], 14}, {NULL}};
const Segment SGI345211[] = {{&S32166[6], 4}, {NULL}};
const Segment SGI345212[] = {{&S32166[79], 5}, {NULL}};
const Segment SGI345213[] = {{&S32166[12], 14}, {&S32166[40], 11}, {NULL}};
const Segment SGI345214[] = {{&S32166[12], 14}, {NULL}};
const Segment SGI345215[] = {{&S32167[6], 4}, {NULL}};
const Segment SGI345216[] = {{&S32167[57], 5}, {NULL}};
const Segment SGI345217[] = {{&S32167[12], 14}, {&S32167[68], 31}, {NULL}};
const Segment SGI345218[] = {{&S32167[12], 14}, {NULL}};
const Segment SGI345219[] = {{&S32171[6], 4}, {NULL}};
const Segment SGI345220[] = {{&S32171[80], 5}, {NULL}};
const Segment SGI345221[] = {{&S32171[12], 14}, {&S32171[40], 12}, {NULL}};
const Segment SGI345222[] = {{&S32171[12], 14}, {NULL}};
const Segment SGI345223[] = {{&S58755[5], 5}, {&S58755[5], 5}, {NULL}};
const Segment SGI345224[] = {{&S32172[6], 4}, {NULL}};
const Segment SGI345225[] = {{&S32172[80], 5}, {NULL}};
const Segment SGI345226[] = {{&S32172[12], 14}, {&S32172[40], 12}, {NULL}};
const Segment SGI345227[] = {{&S32172[12], 14}, {NULL}};
const Segment SGI345228[] = {{&S58756[5], 5}, {NULL}};
const Segment SGI345229[] = {{&S32173[6], 4}, {NULL}};
const Segment SGI345230[] = {{&S32173[80], 5}, {NULL}};
const Segment SGI345231[] = {{&S32173[12], 14}, {&S32173[41], 11}, {NULL}};
const Segment SGI345232[] = {{&S32173[12], 14}, {NULL}};
const Segment SGI345233[] = {{&S58757[5], 7}, {NULL}};
const Segment SGI345234[] = {{&S32174[6], 4}, {NULL}};
const Segment SGI345235[] = {{&S32174[79], 5}, {NULL}};
const Segment SGI345236[] = {{&S32174[12], 14}, {&S32174[40], 11}, {NULL}};
const Segment SGI345237[] = {{&S32174[12], 14}, {NULL}};
const Segment SGI345238[] = {{&S32175[6], 4}, {NULL}};
const Segment SGI345239[] = {{&S32175[80], 5}, {NULL}};
const Segment SGI345240[] = {{&S32175[12], 14}, {&S32175[40], 12}, {NULL}};
const Segment SGI345241[] = {{&S32175[12], 14}, {NULL}};
const Segment SGI345242[] = {{&S32176[6], 4}, {NULL}};
const Segment SGI345243[] = {{&S32176[79], 5}, {NULL}};
const Segment SGI345244[] = {{&S32176[12], 14}, {&S32176[40], 11}, {NULL}};
const Segment SGI345245[] = {{&S32176[12], 14}, {NULL}};
const Segment SGI345246[] = {{&S32177[6], 4}, {NULL}};
const Segment SGI345247[] = {{&S32177[80], 5}, {NULL}};
const Segment SGI345248[] = {{&S32177[12], 14}, {&S32177[41], 11}, {NULL}};
const Segment SGI345249[] = {{&S32177[12], 14}, {NULL}};
const Segment SGI345250[] = {{&S32178[6], 4}, {NULL}};
const Segment SGI345251[] = {{&S32178[79], 5}, {NULL}};
const Segment SGI345252[] = {{&S32178[12], 14}, {&S32178[40], 11}, {NULL}};
const Segment SGI345253[] = {{&S32178[12], 14}, {NULL}};
const Segment SGI345254[] = {{&S32179[6], 4}, {NULL}};
const Segment SGI345255[] = {{&S32179[80], 5}, {NULL}};
const Segment SGI345256[] = {{&S32179[12], 14}, {&S32179[41], 11}, {NULL}};
const Segment SGI345257[] = {{&S32179[12], 14}, {NULL}};
const Segment SGI345258[] = {{&S32180[6], 4}, {NULL}};
const Segment SGI345259[] = {{&S32180[79], 5}, {NULL}};
const Segment SGI345260[] = {{&S32180[12], 14}, {&S32180[40], 11}, {NULL}};
const Segment SGI345261[] = {{&S32180[12], 14}, {NULL}};
const Segment SGI345262[] = {{&S32181[6], 5}, {NULL}};
const Segment SGI345263[] = {{&S32181[13], 14}, {NULL}};
const Segment SGI345264[] = {{&S32182[6], 4}, {NULL}};
const Segment SGI345265[] = {{&S32182[66], 5}, {NULL}};
const Segment SGI345266[] = {{&S32182[12], 14}, {NULL}};
const Segment SGI345267[] = {{&S32183[6], 4}, {NULL}};
const Segment SGI345268[] = {{&S32183[79], 5}, {NULL}};
const Segment SGI345269[] = {{&S32183[12], 14}, {&S32183[40], 11}, {NULL}};
const Segment SGI345270[] = {{&S32183[12], 14}, {NULL}};
const Segment SGI345271[] = {{&S32184[6], 4}, {NULL}};
const Segment SGI345272[] = {{&S32184[79], 5}, {NULL}};
const Segment SGI345273[] = {{&S32184[12], 14}, {&S32184[40], 11}, {NULL}};
const Segment SGI345274[] = {{&S32184[12], 14}, {NULL}};
const Segment SGI345275[] = {{&S32185[6], 4}, {NULL}};
const Segment SGI345276[] = {{&S32185[57], 5}, {NULL}};
const Segment SGI345277[] = {{&S32185[12], 14}, {&S32185[68], 4}, {NULL}};
const Segment SGI345278[] = {{&S32185[12], 14}, {NULL}};
const Segment SGI345279[] = {{&S32186[6], 4}, {NULL}};
const Segment SGI345280[] = {{&S32186[79], 5}, {NULL}};
const Segment SGI345281[] = {{&S32186[12], 14}, {&S32186[40], 11}, {NULL}};
const Segment SGI345282[] = {{&S32186[12], 14}, {NULL}};
const Segment SGI345283[] = {{&S32187[6], 4}, {NULL}};
const Segment SGI345284[] = {{&S32187[79], 5}, {NULL}};
const Segment SGI345285[] = {{&S32187[12], 14}, {&S32187[40], 11}, {NULL}};
const Segment SGI345286[] = {{&S32187[12], 14}, {NULL}};
const Segment SGI345287[] = {{&S32188[6], 4}, {NULL}};
const Segment SGI345288[] = {{&S32188[79], 5}, {NULL}};
const Segment SGI345289[] = {{&S32188[12], 14}, {&S32188[40], 11}, {NULL}};
const Segment SGI345290[] = {{&S32188[12], 14}, {NULL}};
const Segment SGI345291[] = {{&S32189[6], 4}, {NULL}};
const Segment SGI345292[] = {{&S32189[79], 5}, {NULL}};
const Segment SGI345293[] = {{&S32189[12], 14}, {&S32189[40], 11}, {NULL}};
const Segment SGI345294[] = {{&S32189[12], 14}, {NULL}};
const Segment SGI345295[] = {{&S32190[6], 4}, {NULL}};
const Segment SGI345296[] = {{&S32190[79], 5}, {NULL}};
const Segment SGI345297[] = {{&S32190[12], 14}, {&S32190[40], 11}, {NULL}};
const Segment SGI345298[] = {{&S32190[12], 14}, {NULL}};
const Segment SGI345299[] = {{&S32191[6], 4}, {NULL}};
const Segment SGI345300[] = {{&S32191[79], 5}, {NULL}};
const Segment SGI345301[] = {{&S32191[12], 14}, {&S32191[40], 11}, {NULL}};
const Segment SGI345302[] = {{&S32191[12], 14}, {NULL}};
const Segment SGI345303[] = {{&S32192[6], 4}, {NULL}};
const Segment SGI345304[] = {{&S32192[12], 14}, {NULL}};
const Segment SGI345305[] = {{&S32193[6], 4}, {NULL}};
const Segment SGI345306[] = {{&S32193[79], 5}, {NULL}};
const Segment SGI345307[] = {{&S32193[12], 14}, {&S32193[40], 11}, {NULL}};
const Segment SGI345308[] = {{&S32193[12], 14}, {NULL}};
const Segment SGI345309[] = {{&S58788[5], 4}, {NULL}};
const Segment SGI345310[] = {{&S58789[5], 7}, {NULL}};
const Segment SGI345311[] = {{&S58796[5], 4}, {NULL}};
const Segment SGI345312[] = {{&S58796[15], 3}, {NULL}};
const Segment SGI345313[] = {{&S58797[43], 4}, {NULL}};
const Segment SGI345314[] = {{&S58797[6], 3}, {NULL}};
const Segment SGI345315[] = {{&S58797[68], 3}, {NULL}};
const Segment SGI345316[] = {{&S58798[43], 6}, {NULL}};
const Segment SGI345317[] = {{&S58798[6], 3}, {NULL}};
const Segment SGI345318[] = {{&S58798[70], 3}, {NULL}};
const Segment SGI345319[] = {{&S58799[43], 7}, {NULL}};
const Segment SGI345320[] = {{&S58799[6], 3}, {NULL}};
const Segment SGI345321[] = {{&S58799[71], 3}, {NULL}};
const Segment SGI345322[] = {{&S58815[5], 3}, {&S58815[5], 3}, {NULL}};
const Segment SGI345323[] = {{&S58815[109], 3}, {NULL}};
const Segment SGI345324[] = {{&S58820[5], 5}, {&S58820[105], 5}, {NULL}};
const Segment SGI345325[] = {{&S58820[69], 3}, {NULL}};
const Segment SGI345326[] = {{&S58820[131], 3}, {NULL}};
const Segment SGI345327[] = {{&S58821[5], 7}, {&S58821[107], 7}, {NULL}};
const Segment SGI345328[] = {{&S58821[71], 3}, {NULL}};
const Segment SGI345329[] = {{&S58821[135], 3}, {NULL}};
const Segment SGI345330[] = {{&S49982[28], 5}, {NULL}};
const Segment SGI345331[] = {{&S49982[104], 5}, {NULL}};
const Segment SGI345332[] = {{&S49983[28], 7}, {NULL}};
const Segment SGI345333[] = {{&S49983[106], 5}, {NULL}};
const Segment SGI345334[] = {{&S49984[28], 5}, {NULL}};
const Segment SGI345335[] = {{&S49984[104], 5}, {NULL}};
const Segment SGI345336[] = {{&S49985[28], 7}, {NULL}};
const Segment SGI345337[] = {{&S49985[106], 5}, {NULL}};
const Segment SGI345338[] = {{&S58843[5], 5}, {&S58843[104], 5}, {NULL}};
const Segment SGI345339[] = {{&S58843[69], 3}, {NULL}};
const Segment SGI345340[] = {{&S58843[130], 3}, {NULL}};
const Segment SGI345341[] = {{&S58885[35], 5}, {NULL}};
const Segment SGI345342[] = {{&S58885[52], 3}, {NULL}};
const Segment SGI345343[] = {{&S58886[35], 7}, {NULL}};
const Segment SGI345344[] = {{&S58886[54], 3}, {NULL}};
const Segment SGI345345[] = {{&S58887[34], 7}, {NULL}};
const Segment SGI345346[] = {{&S58887[52], 3}, {NULL}};
const Segment SGI345347[] = {{&S58894[5], 5}, {&S58894[105], 5}, {NULL}};
const Segment SGI345348[] = {{&S58894[11], 3}, {NULL}};
const Segment SGI345349[] = {{&S58894[131], 3}, {NULL}};
const Segment SGI345350[] = {{&S50101[28], 3}, {NULL}};
const Segment SGI345351[] = {{&S50101[102], 5}, {NULL}};
const Segment SGI345352[] = {{&S50103[28], 5}, {NULL}};
const Segment SGI345353[] = {{&S50103[104], 5}, {NULL}};
const Segment SGI345354[] = {{&S50104[28], 7}, {NULL}};
const Segment SGI345355[] = {{&S50104[106], 5}, {NULL}};
const Segment SGI345356[] = {{&S59028[42], 5}, {NULL}};
const Segment SGI345357[] = {{&S59028[6], 3}, {NULL}};
const Segment SGI345358[] = {{&S59028[68], 3}, {NULL}};
const Segment SGI345359[] = {{&S59054[5], 5}, {&S59054[105], 5}, {NULL}};
const Segment SGI345360[] = {{&S59054[11], 3}, {NULL}};
const Segment SGI345361[] = {{&S59054[131], 3}, {NULL}};
const Segment SGI345362[] = {{&S59055[5], 3}, {&S59055[5], 3}, {NULL}};
const Segment SGI345363[] = {{&S59055[9], 3}, {NULL}};
const Segment SGI345364[] = {{&S59055[127], 3}, {NULL}};
const Segment SGI345365[] = {{&S50223[28], 5}, {NULL}};
const Segment SGI345366[] = {{&S50223[104], 5}, {NULL}};
const Segment SGI345367[] = {{&S50224[28], 5}, {NULL}};
const Segment SGI345368[] = {{&S50224[104], 5}, {NULL}};
const Segment SGI345369[] = {{&S59076[35], 4}, {NULL}};
const Segment SGI345370[] = {{&S59076[90], 3}, {NULL}};
const Segment SGI345371[] = {{&S59077[5], 13}, {NULL}};
const Segment SGI345372[] = {{&S59077[19], 3}, {NULL}};
const Segment SGI345373[] = {{&S59078[5], 4}, {NULL}};
const Segment SGI345374[] = {{&S59078[15], 3}, {NULL}};
const Segment SGI345375[] = {{&S59082[42], 5}, {NULL}};
const Segment SGI345376[] = {{&S59082[6], 3}, {NULL}};
const Segment SGI345377[] = {{&S59082[68], 3}, {NULL}};
const Segment SGI345378[] = {{&S59083[43], 3}, {NULL}};
const Segment SGI345379[] = {{&S59083[6], 3}, {NULL}};
const Segment SGI345380[] = {{&S59083[67], 3}, {NULL}};
const Segment SGI345381[] = {{&S59148[5], 3}, {&S59148[5], 3}, {NULL}};
const Segment SGI345382[] = {{&S59148[9], 3}, {NULL}};
const Segment SGI345383[] = {{&S59148[127], 3}, {NULL}};
const Segment SGI345384[] = {{&S31711[0], 24}, {NULL}};
const Segment SGI345385[] = {{&S31711[37], 3}, {NULL}};
const Segment SGI345386[] = {{&S31711[78], 13}, {NULL}};
const Segment SGI345387[] = {{&S59149[5], 3}, {&S59149[5], 3}, {NULL}};
const Segment SGI345388[] = {{&S59149[9], 3}, {NULL}};
const Segment SGI345389[] = {{&S59149[127], 3}, {NULL}};
const Segment SGI345390[] = {{&S59164[35], 3}, {NULL}};
const Segment SGI345391[] = {{&S59164[50], 3}, {NULL}};
const Segment SGI345392[] = {{&S59165[5], 3}, {NULL}};
const Segment SGI345393[] = {{&S59165[14], 3}, {NULL}};
const Segment SGI345394[] = {{&S59166[34], 5}, {NULL}};
const Segment SGI345395[] = {{&S59166[50], 3}, {NULL}};
const Segment SGI345396[] = {{&S50352[28], 3}, {NULL}};
const Segment SGI345397[] = {{&S50352[102], 5}, {NULL}};
const Segment SGI345398[] = {{&S50353[5], 3}, {&S50353[5], 3}, {NULL}};
const Segment SGI345399[] = {{&S50353[163], 5}, {NULL}};
const Segment SGI345400[] = {{&S45476[46], 5}, {&S45476[146], 7}, {NULL}};
const Segment SGI345401[] = {{&S45476[37], 3}, {NULL}};
const Segment SGI345402[] = {{&S45477[46], 8}, {&S45477[149], 7}, {NULL}};
const Segment SGI345403[] = {{&S45477[37], 3}, {NULL}};
const Segment SGI345404[] = {{&S45478[46], 8}, {NULL}};
const Segment SGI345405[] = {{&S45478[37], 3}, {NULL}};
const Segment SGI345406[] = {{&S45479[46], 5}, {NULL}};
const Segment SGI345407[] = {{&S45479[37], 3}, {NULL}};
const Segment SGI345408[] = {{&S45480[8], 5}, {NULL}};
const Segment SGI345409[] = {{&S45480[37], 3}, {NULL}};
const Segment SGI345410[] = {{&S34688[61], 4}, {NULL}};
const Segment SGI345411[] = {{&S34688[13], 10}, {NULL}};
const Segment SGI345412[] = {{&S34689[61], 4}, {NULL}};
const Segment SGI345413[] = {{&S34689[13], 10}, {NULL}};
const Segment SGI345414[] = {{&S34690[61], 4}, {NULL}};
const Segment SGI345415[] = {{&S34690[13], 10}, {NULL}};
const Segment SGI345416[] = {{&S34691[61], 4}, {NULL}};
const Segment SGI345417[] = {{&S34691[13], 10}, {NULL}};
const Segment SGI345418[] = {{&S34692[42], 4}, {NULL}};
const Segment SGI345419[] = {{&S34692[13], 14}, {NULL}};
const Segment SGI345420[] = {{&S34693[61], 4}, {&S34693[61], 3}, {NULL}};
const Segment SGI345421[] = {{&S34693[13], 10}, {NULL}};
const Segment SGI345422[] = {{&S34694[61], 4}, {NULL}};
const Segment SGI345423[] = {{&S34694[13], 10}, {NULL}};
const Segment SGI345424[] = {{&S34695[42], 4}, {NULL}};
const Segment SGI345425[] = {{&S34695[13], 14}, {NULL}};
const Segment SGI345426[] = {{&S34696[42], 4}, {NULL}};
const Segment SGI345427[] = {{&S34696[13], 14}, {NULL}};
const Segment SGI345428[] = {{&S34697[42], 4}, {NULL}};
const Segment SGI345429[] = {{&S34697[13], 14}, {NULL}};
const Segment SGI345430[] = {{&S34698[42], 4}, {NULL}};
const Segment SGI345431[] = {{&S34698[13], 14}, {NULL}};
const Segment SGI345432[] = {{&S34699[63], 4}, {&S34699[63], 3}, {NULL}};
const Segment SGI345433[] = {{&S34699[1], 24}, {NULL}};
const Segment SGI345434[] = {{&S50637[28], 3}, {NULL}};
const Segment SGI345435[] = {{&S50637[102], 5}, {NULL}};
const Segment SGI345436[] = {{&S50638[28], 5}, {NULL}};
const Segment SGI345437[] = {{&S50638[104], 5}, {NULL}};
const Segment SGI345438[] = {{&S50639[28], 5}, {NULL}};
const Segment SGI345439[] = {{&S50639[104], 5}, {NULL}};
const Segment SGI345440[] = {{&S50640[28], 4}, {NULL}};
const Segment SGI345441[] = {{&S50640[103], 5}, {NULL}};
const Segment SGI345442[] = {{&S50641[28], 4}, {NULL}};
const Segment SGI345443[] = {{&S50641[103], 5}, {NULL}};
const Segment SGI345444[] = {{&S50642[28], 4}, {NULL}};
const Segment SGI345445[] = {{&S50642[103], 5}, {NULL}};
const Segment SGI345446[] = {{&S45616[42], 14}, {&S45616[134], 7}, {NULL}};
const Segment SGI345447[] = {{&S45616[33], 3}, {NULL}};
const Segment SGI345448[] = {{&S45617[42], 5}, {&S45617[147], 7}, {NULL}};
const Segment SGI345449[] = {{&S45617[33], 3}, {NULL}};
const Segment SGI345450[] = {{&S32658[6], 4}, {NULL}};
const Segment SGI345451[] = {{&S32658[52], 5}, {NULL}};
const Segment SGI345452[] = {{&S32658[12], 45}, {NULL}};
const Segment SGI345453[] = {{&S32658[12], 14}, {NULL}};
const Segment SGI345454[] = {{&S32659[6], 4}, {NULL}};
const Segment SGI345455[] = {{&S32659[59], 5}, {NULL}};
const Segment SGI345456[] = {{&S32659[12], 14}, {NULL}};
const Segment SGI345457[] = {{&S32660[6], 4}, {NULL}};
const Segment SGI345458[] = {{&S32660[81], 5}, {NULL}};
const Segment SGI345459[] = {{&S32660[12], 14}, {&S32660[44], 42}, {NULL}};
const Segment SGI345460[] = {{&S32660[12], 14}, {NULL}};
const Segment SGI345461[] = {{&S32661[6], 4}, {NULL}};
const Segment SGI345462[] = {{&S32661[81], 5}, {NULL}};
const Segment SGI345463[] = {{&S32661[12], 14}, {&S32661[44], 42}, {NULL}};
const Segment SGI345464[] = {{&S32661[12], 14}, {NULL}};
const Segment SGI345465[] = {{&S59462[5], 5}, {NULL}};
const Segment SGI345466[] = {{&S59462[16], 3}, {NULL}};
const Segment SGI345467[] = {{&S32662[6], 4}, {NULL}};
const Segment SGI345468[] = {{&S32662[81], 5}, {NULL}};
const Segment SGI345469[] = {{&S32662[12], 14}, {&S32662[44], 42}, {NULL}};
const Segment SGI345470[] = {{&S32662[12], 14}, {NULL}};
const Segment SGI345471[] = {{&S59463[35], 5}, {NULL}};
const Segment SGI345472[] = {{&S59463[91], 3}, {NULL}};
const Segment SGI345473[] = {{&S32663[6], 4}, {NULL}};
const Segment SGI345474[] = {{&S32663[81], 5}, {NULL}};
const Segment SGI345475[] = {{&S32663[12], 14}, {&S32663[44], 42}, {NULL}};
const Segment SGI345476[] = {{&S32663[12], 14}, {NULL}};
const Segment SGI345477[] = {{&S32664[6], 4}, {NULL}};
const Segment SGI345478[] = {{&S32664[59], 5}, {NULL}};
const Segment SGI345479[] = {{&S32664[12], 14}, {&S32664[70], 4}, {NULL}};
const Segment SGI345480[] = {{&S32664[12], 14}, {NULL}};
const Segment SGI345481[] = {{&S32665[6], 4}, {NULL}};
const Segment SGI345482[] = {{&S32665[81], 5}, {NULL}};
const Segment SGI345483[] = {{&S32665[12], 14}, {&S32665[44], 42}, {NULL}};
const Segment SGI345484[] = {{&S32665[12], 14}, {NULL}};
const Segment SGI345485[] = {{&S32666[6], 4}, {NULL}};
const Segment SGI345486[] = {{&S32666[81], 5}, {NULL}};
const Segment SGI345487[] = {{&S32666[12], 14}, {&S32666[44], 42}, {NULL}};
const Segment SGI345488[] = {{&S32666[12], 14}, {NULL}};
const Segment SGI345489[] = {{&S32667[6], 4}, {NULL}};
const Segment SGI345490[] = {{&S32667[81], 5}, {NULL}};
const Segment SGI345491[] = {{&S32667[12], 14}, {&S32667[44], 42}, {NULL}};
const Segment SGI345492[] = {{&S32667[12], 14}, {NULL}};
const Segment SGI345493[] = {{&S32668[6], 4}, {NULL}};
const Segment SGI345494[] = {{&S32668[81], 5}, {NULL}};
const Segment SGI345495[] = {{&S32668[12], 14}, {&S32668[44], 42}, {NULL}};
const Segment SGI345496[] = {{&S32668[12], 14}, {NULL}};
const Segment SGI345497[] = {{&S32669[6], 4}, {NULL}};
const Segment SGI345498[] = {{&S32669[81], 5}, {NULL}};
const Segment SGI345499[] = {{&S32669[12], 14}, {&S32669[44], 42}, {NULL}};
const Segment SGI345500[] = {{&S32669[12], 14}, {NULL}};
const Segment SGI345501[] = {{&S32670[6], 4}, {NULL}};
const Segment SGI345502[] = {{&S32670[81], 5}, {NULL}};
const Segment SGI345503[] = {{&S32670[12], 14}, {&S32670[44], 42}, {NULL}};
const Segment SGI345504[] = {{&S32670[12], 14}, {NULL}};
const Segment SGI345505[] = {{&S32684[6], 4}, {NULL}};
const Segment SGI345506[] = {{&S32684[82], 5}, {NULL}};
const Segment SGI345507[] = {{&S32684[12], 14}, {&S32684[40], 12}, {NULL}};
const Segment SGI345508[] = {{&S32684[12], 14}, {NULL}};
const Segment SGI345509[] = {{&S32685[6], 4}, {NULL}};
const Segment SGI345510[] = {{&S32685[81], 5}, {NULL}};
const Segment SGI345511[] = {{&S32685[12], 14}, {&S32685[40], 11}, {NULL}};
const Segment SGI345512[] = {{&S32685[12], 14}, {NULL}};
const Segment SGI345513[] = {{&S32686[6], 4}, {NULL}};
const Segment SGI345514[] = {{&S32686[82], 5}, {NULL}};
const Segment SGI345515[] = {{&S32686[12], 14}, {&S32686[40], 12}, {NULL}};
const Segment SGI345516[] = {{&S32686[12], 14}, {NULL}};
const Segment SGI345517[] = {{&S32687[6], 4}, {NULL}};
const Segment SGI345518[] = {{&S32687[81], 5}, {NULL}};
const Segment SGI345519[] = {{&S32687[12], 14}, {&S32687[40], 11}, {NULL}};
const Segment SGI345520[] = {{&S32687[12], 14}, {NULL}};
const Segment SGI345521[] = {{&S32688[6], 4}, {NULL}};
const Segment SGI345522[] = {{&S32688[81], 5}, {NULL}};
const Segment SGI345523[] = {{&S32688[12], 14}, {&S32688[40], 11}, {NULL}};
const Segment SGI345524[] = {{&S32688[12], 14}, {NULL}};
const Segment SGI345525[] = {{&S32689[6], 4}, {NULL}};
const Segment SGI345526[] = {{&S32689[59], 5}, {NULL}};
const Segment SGI345527[] = {{&S32689[12], 14}, {&S32689[70], 31}, {NULL}};
const Segment SGI345528[] = {{&S32689[12], 14}, {NULL}};
const Segment SGI345529[] = {{&S32690[6], 4}, {NULL}};
const Segment SGI345530[] = {{&S32690[82], 5}, {NULL}};
const Segment SGI345531[] = {{&S32690[12], 14}, {&S32690[40], 12}, {NULL}};
const Segment SGI345532[] = {{&S32690[12], 14}, {NULL}};
const Segment SGI345533[] = {{&S32691[6], 4}, {NULL}};
const Segment SGI345534[] = {{&S32691[81], 5}, {NULL}};
const Segment SGI345535[] = {{&S32691[12], 14}, {&S32691[40], 11}, {NULL}};
const Segment SGI345536[] = {{&S32691[12], 14}, {NULL}};
const Segment SGI345537[] = {{&S32692[6], 4}, {NULL}};
const Segment SGI345538[] = {{&S32692[82], 5}, {NULL}};
const Segment SGI345539[] = {{&S32692[12], 14}, {&S32692[40], 12}, {NULL}};
const Segment SGI345540[] = {{&S32692[12], 14}, {NULL}};
const Segment SGI345541[] = {{&S32694[6], 4}, {NULL}};
const Segment SGI345542[] = {{&S32694[81], 5}, {NULL}};
const Segment SGI345543[] = {{&S32694[12], 14}, {&S32694[40], 11}, {NULL}};
const Segment SGI345544[] = {{&S32694[12], 14}, {NULL}};
const Segment SGI345545[] = {{&S32695[6], 4}, {NULL}};
const Segment SGI345546[] = {{&S32695[82], 5}, {NULL}};
const Segment SGI345547[] = {{&S32695[12], 14}, {&S32695[40], 12}, {NULL}};
const Segment SGI345548[] = {{&S32695[12], 14}, {NULL}};
const Segment SGI345549[] = {{&S32696[6], 4}, {NULL}};
const Segment SGI345550[] = {{&S32696[81], 5}, {NULL}};
const Segment SGI345551[] = {{&S32696[12], 14}, {&S32696[40], 11}, {NULL}};
const Segment SGI345552[] = {{&S32696[12], 14}, {NULL}};
const Segment SGI345553[] = {{&S32697[6], 4}, {NULL}};
const Segment SGI345554[] = {{&S32697[81], 5}, {NULL}};
const Segment SGI345555[] = {{&S32697[12], 14}, {&S32697[40], 11}, {NULL}};
const Segment SGI345556[] = {{&S32697[12], 14}, {NULL}};
const Segment SGI345557[] = {{&S32698[6], 4}, {NULL}};
const Segment SGI345558[] = {{&S32698[81], 5}, {NULL}};
const Segment SGI345559[] = {{&S32698[12], 14}, {&S32698[40], 11}, {NULL}};
const Segment SGI345560[] = {{&S32698[12], 14}, {NULL}};
const Segment SGI345561[] = {{&S32699[6], 4}, {NULL}};
const Segment SGI345562[] = {{&S32699[81], 5}, {NULL}};
const Segment SGI345563[] = {{&S32699[12], 14}, {&S32699[40], 11}, {NULL}};
const Segment SGI345564[] = {{&S32699[12], 14}, {NULL}};
const Segment SGI345565[] = {{&S32700[6], 4}, {NULL}};
const Segment SGI345566[] = {{&S32700[81], 5}, {NULL}};
const Segment SGI345567[] = {{&S32700[12], 14}, {&S32700[40], 11}, {NULL}};
const Segment SGI345568[] = {{&S32700[12], 14}, {NULL}};
const Segment SGI345569[] = {{&S32701[6], 4}, {NULL}};
const Segment SGI345570[] = {{&S32701[82], 5}, {NULL}};
const Segment SGI345571[] = {{&S32701[12], 14}, {&S32701[40], 12}, {NULL}};
const Segment SGI345572[] = {{&S32701[12], 14}, {NULL}};
const Segment SGI345573[] = {{&S32702[6], 4}, {NULL}};
const Segment SGI345574[] = {{&S32702[81], 5}, {NULL}};
const Segment SGI345575[] = {{&S32702[12], 14}, {&S32702[40], 11}, {NULL}};
const Segment SGI345576[] = {{&S32702[12], 14}, {NULL}};
const Segment SGI345577[] = {{&S32703[6], 4}, {NULL}};
const Segment SGI345578[] = {{&S32703[82], 5}, {NULL}};
const Segment SGI345579[] = {{&S32703[12], 14}, {&S32703[40], 12}, {NULL}};
const Segment SGI345580[] = {{&S32703[12], 14}, {NULL}};
const Segment SGI345581[] = {{&S32704[6], 4}, {NULL}};
const Segment SGI345582[] = {{&S32704[82], 5}, {NULL}};
const Segment SGI345583[] = {{&S32704[12], 14}, {&S32704[40], 12}, {NULL}};
const Segment SGI345584[] = {{&S32704[12], 14}, {NULL}};
const Segment SGI345585[] = {{&S32705[6], 4}, {NULL}};
const Segment SGI345586[] = {{&S32705[108], 5}, {NULL}};
const Segment SGI345587[] = {{&S32705[12], 14}, {&S32705[40], 12}, {NULL}};
const Segment SGI345588[] = {{&S32705[12], 14}, {NULL}};
const Segment SGI345589[] = {{&S50720[28], 5}, {NULL}};
const Segment SGI345590[] = {{&S50720[104], 5}, {NULL}};
const Segment SGI345591[] = {{&S32706[6], 4}, {NULL}};
const Segment SGI345592[] = {{&S32706[82], 5}, {NULL}};
const Segment SGI345593[] = {{&S32706[12], 14}, {&S32706[40], 12}, {NULL}};
const Segment SGI345594[] = {{&S32706[12], 14}, {NULL}};
const Segment SGI345595[] = {{&S50721[28], 7}, {NULL}};
const Segment SGI345596[] = {{&S32707[6], 4}, {NULL}};
const Segment SGI345597[] = {{&S32707[81], 5}, {NULL}};
const Segment SGI345598[] = {{&S50721[106], 5}, {NULL}};
const Segment SGI345599[] = {{&S32707[12], 14}, {&S32707[40], 11}, {NULL}};
const Segment SGI345600[] = {{&S32707[12], 14}, {NULL}};
const Segment SGI345601[] = {{&S50722[28], 5}, {NULL}};
const Segment SGI345602[] = {{&S50722[104], 5}, {NULL}};
const Segment SGI345603[] = {{&S32708[6], 4}, {NULL}};
const Segment SGI345604[] = {{&S32708[82], 5}, {NULL}};
const Segment SGI345605[] = {{&S32708[12], 14}, {&S32708[40], 12}, {NULL}};
const Segment SGI345606[] = {{&S32708[12], 14}, {NULL}};
const Segment SGI345607[] = {{&S32709[6], 4}, {NULL}};
const Segment SGI345608[] = {{&S32709[81], 5}, {NULL}};
const Segment SGI345609[] = {{&S32709[12], 14}, {&S32709[40], 11}, {NULL}};
const Segment SGI345610[] = {{&S32709[12], 14}, {NULL}};
const Segment SGI345611[] = {{&S32710[6], 4}, {NULL}};
const Segment SGI345612[] = {{&S32710[82], 5}, {NULL}};
const Segment SGI345613[] = {{&S32710[12], 14}, {&S32710[41], 11}, {NULL}};
const Segment SGI345614[] = {{&S32710[12], 14}, {NULL}};
const Segment SGI345615[] = {{&S32711[6], 4}, {NULL}};
const Segment SGI345616[] = {{&S32711[59], 5}, {NULL}};
const Segment SGI345617[] = {{&S32711[12], 14}, {NULL}};
const Segment SGI345618[] = {{&S32712[6], 4}, {NULL}};
const Segment SGI345619[] = {{&S32712[82], 5}, {NULL}};
const Segment SGI345620[] = {{&S32712[12], 14}, {&S32712[41], 11}, {NULL}};
const Segment SGI345621[] = {{&S32712[12], 14}, {NULL}};
const Segment SGI345622[] = {{&S32713[6], 4}, {NULL}};
const Segment SGI345623[] = {{&S32713[82], 5}, {NULL}};
const Segment SGI345624[] = {{&S32713[12], 14}, {&S32713[41], 11}, {NULL}};
const Segment SGI345625[] = {{&S32713[12], 14}, {NULL}};
const Segment SGI345626[] = {{&S32714[6], 4}, {NULL}};
const Segment SGI345627[] = {{&S32714[82], 5}, {NULL}};
const Segment SGI345628[] = {{&S32714[12], 14}, {&S32714[41], 11}, {NULL}};
const Segment SGI345629[] = {{&S32714[12], 14}, {NULL}};
const Segment SGI345630[] = {{&S32715[6], 4}, {NULL}};
const Segment SGI345631[] = {{&S32715[82], 5}, {NULL}};
const Segment SGI345632[] = {{&S32715[12], 14}, {&S32715[41], 11}, {NULL}};
const Segment SGI345633[] = {{&S32715[12], 14}, {NULL}};
const Segment SGI345634[] = {{&S32716[6], 4}, {NULL}};
const Segment SGI345635[] = {{&S32716[59], 5}, {NULL}};
const Segment SGI345636[] = {{&S32716[12], 14}, {NULL}};
const Segment SGI345637[] = {{&S32717[6], 4}, {NULL}};
const Segment SGI345638[] = {{&S32717[81], 5}, {NULL}};
const Segment SGI345639[] = {{&S32717[12], 14}, {&S32717[40], 11}, {NULL}};
const Segment SGI345640[] = {{&S32717[12], 14}, {NULL}};
const Segment SGI345641[] = {{&S32718[6], 4}, {NULL}};
const Segment SGI345642[] = {{&S32718[81], 5}, {NULL}};
const Segment SGI345643[] = {{&S32718[12], 14}, {&S32718[40], 11}, {NULL}};
const Segment SGI345644[] = {{&S32718[12], 14}, {NULL}};
const Segment SGI345645[] = {{&S32719[6], 4}, {NULL}};
const Segment SGI345646[] = {{&S32719[82], 5}, {NULL}};
const Segment SGI345647[] = {{&S32719[12], 14}, {&S32719[40], 12}, {NULL}};
const Segment SGI345648[] = {{&S32719[12], 14}, {NULL}};
const Segment SGI345649[] = {{&S32721[6], 4}, {NULL}};
const Segment SGI345650[] = {{&S32721[81], 5}, {NULL}};
const Segment SGI345651[] = {{&S32721[12], 14}, {&S32721[40], 11}, {NULL}};
const Segment SGI345652[] = {{&S32721[12], 14}, {NULL}};
const Segment SGI345653[] = {{&S32722[6], 4}, {NULL}};
const Segment SGI345654[] = {{&S32722[81], 5}, {NULL}};
const Segment SGI345655[] = {{&S32722[12], 14}, {&S32722[40], 11}, {NULL}};
const Segment SGI345656[] = {{&S32722[12], 14}, {NULL}};
const Segment SGI345657[] = {{&S32723[6], 4}, {NULL}};
const Segment SGI345658[] = {{&S32723[82], 5}, {NULL}};
const Segment SGI345659[] = {{&S32723[12], 14}, {&S32723[40], 12}, {NULL}};
const Segment SGI345660[] = {{&S32723[12], 14}, {NULL}};
const Segment SGI345661[] = {{&S32724[6], 4}, {NULL}};
const Segment SGI345662[] = {{&S32724[81], 5}, {NULL}};
const Segment SGI345663[] = {{&S32724[12], 14}, {&S32724[40], 11}, {NULL}};
const Segment SGI345664[] = {{&S32724[12], 14}, {NULL}};
const Segment SGI345665[] = {{&S32725[6], 4}, {NULL}};
const Segment SGI345666[] = {{&S32725[82], 5}, {NULL}};
const Segment SGI345667[] = {{&S32725[12], 14}, {&S32725[40], 12}, {NULL}};
const Segment SGI345668[] = {{&S32725[12], 14}, {NULL}};
const Segment SGI345669[] = {{&S32726[6], 4}, {NULL}};
const Segment SGI345670[] = {{&S32726[82], 5}, {NULL}};
const Segment SGI345671[] = {{&S32726[12], 14}, {&S32726[40], 12}, {NULL}};
const Segment SGI345672[] = {{&S32726[12], 14}, {NULL}};
const Segment SGI345673[] = {{&S32727[6], 4}, {NULL}};
const Segment SGI345674[] = {{&S32727[82], 5}, {NULL}};
const Segment SGI345675[] = {{&S32727[12], 14}, {&S32727[40], 12}, {NULL}};
const Segment SGI345676[] = {{&S32727[12], 14}, {NULL}};
const Segment SGI345677[] = {{&S32728[6], 4}, {NULL}};
const Segment SGI345678[] = {{&S32728[82], 5}, {NULL}};
const Segment SGI345679[] = {{&S32728[12], 14}, {&S32728[40], 12}, {NULL}};
const Segment SGI345680[] = {{&S32728[12], 14}, {NULL}};
const Segment SGI345681[] = {{&S32729[6], 4}, {NULL}};
const Segment SGI345682[] = {{&S32729[82], 5}, {NULL}};
const Segment SGI345683[] = {{&S32729[12], 14}, {&S32729[40], 12}, {NULL}};
const Segment SGI345684[] = {{&S32729[12], 14}, {NULL}};
const Segment SGI345685[] = {{&S32730[6], 4}, {NULL}};
const Segment SGI345686[] = {{&S32730[82], 5}, {NULL}};
const Segment SGI345687[] = {{&S32730[12], 14}, {&S32730[41], 11}, {NULL}};
const Segment SGI345688[] = {{&S32730[12], 14}, {NULL}};
const Segment SGI345689[] = {{&S32731[6], 4}, {NULL}};
const Segment SGI345690[] = {{&S32731[82], 5}, {NULL}};
const Segment SGI345691[] = {{&S32731[12], 14}, {&S32731[41], 11}, {NULL}};
const Segment SGI345692[] = {{&S32731[12], 14}, {NULL}};
const Segment SGI345693[] = {{&S50750[28], 3}, {NULL}};
const Segment SGI345694[] = {{&S50750[102], 5}, {NULL}};
const Segment SGI345695[] = {{&S32732[6], 4}, {NULL}};
const Segment SGI345696[] = {{&S32732[82], 5}, {NULL}};
const Segment SGI345697[] = {{&S32732[12], 14}, {&S32732[41], 11}, {NULL}};
const Segment SGI345698[] = {{&S32732[12], 14}, {NULL}};
const Segment SGI345699[] = {{&S32733[6], 4}, {NULL}};
const Segment SGI345700[] = {{&S32733[81], 5}, {NULL}};
const Segment SGI345701[] = {{&S32733[12], 14}, {&S32733[40], 11}, {NULL}};
const Segment SGI345702[] = {{&S32733[12], 14}, {NULL}};
const Segment SGI345703[] = {{&S32734[6], 4}, {NULL}};
const Segment SGI345704[] = {{&S32734[81], 5}, {NULL}};
const Segment SGI345705[] = {{&S32734[12], 14}, {&S32734[40], 11}, {NULL}};
const Segment SGI345706[] = {{&S32734[12], 14}, {NULL}};
const Segment SGI345707[] = {{&S32735[6], 4}, {NULL}};
const Segment SGI345708[] = {{&S32735[81], 5}, {NULL}};
const Segment SGI345709[] = {{&S32735[12], 14}, {&S32735[40], 11}, {NULL}};
const Segment SGI345710[] = {{&S32735[12], 14}, {NULL}};
const Segment SGI345711[] = {{&S32736[6], 4}, {NULL}};
const Segment SGI345712[] = {{&S32736[82], 5}, {NULL}};
const Segment SGI345713[] = {{&S32736[12], 14}, {&S32736[41], 11}, {NULL}};
const Segment SGI345714[] = {{&S32736[12], 14}, {NULL}};
const Segment SGI345715[] = {{&S32737[6], 4}, {NULL}};
const Segment SGI345716[] = {{&S32737[81], 5}, {NULL}};
const Segment SGI345717[] = {{&S32737[12], 14}, {&S32737[40], 11}, {NULL}};
const Segment SGI345718[] = {{&S32737[12], 14}, {NULL}};
const Segment SGI345719[] = {{&S32738[6], 4}, {NULL}};
const Segment SGI345720[] = {{&S32738[81], 5}, {NULL}};
const Segment SGI345721[] = {{&S32738[12], 14}, {&S32738[40], 11}, {NULL}};
const Segment SGI345722[] = {{&S32738[12], 14}, {NULL}};
const Segment SGI345723[] = {{&S32739[6], 4}, {NULL}};
const Segment SGI345724[] = {{&S32739[81], 5}, {NULL}};
const Segment SGI345725[] = {{&S32739[12], 14}, {&S32739[40], 11}, {NULL}};
const Segment SGI345726[] = {{&S32739[12], 14}, {NULL}};
const Segment SGI345727[] = {{&S32740[6], 4}, {NULL}};
const Segment SGI345728[] = {{&S32740[81], 5}, {NULL}};
const Segment SGI345729[] = {{&S32740[12], 14}, {&S32740[40], 11}, {NULL}};
const Segment SGI345730[] = {{&S32740[12], 14}, {NULL}};
const Segment SGI345731[] = {{&S32741[6], 4}, {NULL}};
const Segment SGI345732[] = {{&S32741[82], 5}, {NULL}};
const Segment SGI345733[] = {{&S32741[12], 14}, {&S32741[40], 12}, {NULL}};
const Segment SGI345734[] = {{&S32741[12], 14}, {NULL}};
const Segment SGI345735[] = {{&S32742[6], 4}, {NULL}};
const Segment SGI345736[] = {{&S32742[81], 5}, {NULL}};
const Segment SGI345737[] = {{&S32742[12], 14}, {&S32742[40], 11}, {NULL}};
const Segment SGI345738[] = {{&S32742[12], 14}, {NULL}};
const Segment SGI345739[] = {{&S32743[6], 4}, {NULL}};
const Segment SGI345740[] = {{&S32743[82], 5}, {NULL}};
const Segment SGI345741[] = {{&S32743[12], 14}, {&S32743[41], 11}, {NULL}};
const Segment SGI345742[] = {{&S32743[12], 14}, {NULL}};
const Segment SGI345743[] = {{&S32744[6], 4}, {NULL}};
const Segment SGI345744[] = {{&S32744[81], 5}, {NULL}};
const Segment SGI345745[] = {{&S32744[12], 14}, {&S32744[40], 11}, {NULL}};
const Segment SGI345746[] = {{&S32744[12], 14}, {NULL}};
const Segment SGI345747[] = {{&S32745[6], 4}, {NULL}};
const Segment SGI345748[] = {{&S32745[82], 5}, {NULL}};
const Segment SGI345749[] = {{&S32745[12], 14}, {&S32745[41], 11}, {NULL}};
const Segment SGI345750[] = {{&S32745[12], 14}, {NULL}};
const Segment SGI345751[] = {{&S32746[6], 4}, {NULL}};
const Segment SGI345752[] = {{&S32746[59], 5}, {NULL}};
const Segment SGI345753[] = {{&S32746[12], 14}, {&S32746[70], 4}, {NULL}};
const Segment SGI345754[] = {{&S32746[12], 14}, {NULL}};
const Segment SGI345755[] = {{&S32747[6], 4}, {NULL}};
const Segment SGI345756[] = {{&S32747[81], 5}, {NULL}};
const Segment SGI345757[] = {{&S32747[12], 14}, {&S32747[40], 11}, {NULL}};
const Segment SGI345758[] = {{&S32747[12], 14}, {NULL}};
const Segment SGI345759[] = {{&S32748[6], 4}, {NULL}};
const Segment SGI345760[] = {{&S32748[81], 5}, {NULL}};
const Segment SGI345761[] = {{&S32748[12], 14}, {&S32748[40], 11}, {NULL}};
const Segment SGI345762[] = {{&S32748[12], 14}, {NULL}};
const Segment SGI345763[] = {{&S32749[6], 4}, {NULL}};
const Segment SGI345764[] = {{&S32749[81], 5}, {NULL}};
const Segment SGI345765[] = {{&S32749[12], 14}, {&S32749[40], 11}, {NULL}};
const Segment SGI345766[] = {{&S32749[12], 14}, {NULL}};
const Segment SGI345767[] = {{&S32750[6], 4}, {NULL}};
const Segment SGI345768[] = {{&S32750[81], 5}, {NULL}};
const Segment SGI345769[] = {{&S32750[12], 14}, {&S32750[40], 11}, {NULL}};
const Segment SGI345770[] = {{&S32750[12], 14}, {NULL}};
const Segment SGI345771[] = {{&S32751[6], 4}, {NULL}};
const Segment SGI345772[] = {{&S32751[81], 5}, {NULL}};
const Segment SGI345773[] = {{&S32751[12], 14}, {&S32751[40], 11}, {NULL}};
const Segment SGI345774[] = {{&S32751[12], 14}, {NULL}};
const Segment SGI345775[] = {{&S32752[6], 4}, {NULL}};
const Segment SGI345776[] = {{&S32752[81], 5}, {NULL}};
const Segment SGI345777[] = {{&S32752[12], 14}, {&S32752[40], 11}, {NULL}};
const Segment SGI345778[] = {{&S32752[12], 14}, {NULL}};
const Segment SGI345779[] = {{&S32753[6], 4}, {NULL}};
const Segment SGI345780[] = {{&S32753[81], 5}, {NULL}};
const Segment SGI345781[] = {{&S32753[12], 14}, {&S32753[40], 11}, {NULL}};
const Segment SGI345782[] = {{&S32753[12], 14}, {NULL}};
const Segment SGI345783[] = {{&S32754[6], 4}, {NULL}};
const Segment SGI345784[] = {{&S32754[81], 5}, {NULL}};
const Segment SGI345785[] = {{&S32754[12], 14}, {&S32754[40], 11}, {NULL}};
const Segment SGI345786[] = {{&S32754[12], 14}, {NULL}};
const Segment SGI345787[] = {{&S32755[6], 4}, {NULL}};
const Segment SGI345788[] = {{&S32755[81], 5}, {NULL}};
const Segment SGI345789[] = {{&S32755[12], 14}, {&S32755[40], 11}, {NULL}};
const Segment SGI345790[] = {{&S32755[12], 14}, {NULL}};
const Segment SGI345791[] = {{&S32756[6], 4}, {NULL}};
const Segment SGI345792[] = {{&S32756[81], 5}, {NULL}};
const Segment SGI345793[] = {{&S32756[12], 14}, {&S32756[40], 11}, {NULL}};
const Segment SGI345794[] = {{&S32756[12], 14}, {NULL}};
const Segment SGI345795[] = {{&S32757[6], 4}, {NULL}};
const Segment SGI345796[] = {{&S32757[81], 5}, {NULL}};
const Segment SGI345797[] = {{&S32757[12], 14}, {&S32757[40], 11}, {NULL}};
const Segment SGI345798[] = {{&S32757[12], 14}, {NULL}};
const Segment SGI345799[] = {{&S32758[6], 4}, {NULL}};
const Segment SGI345800[] = {{&S32758[81], 5}, {NULL}};
const Segment SGI345801[] = {{&S32758[12], 14}, {&S32758[40], 11}, {NULL}};
const Segment SGI345802[] = {{&S32758[12], 14}, {NULL}};
const Segment SGI345803[] = {{&S32759[6], 4}, {NULL}};
const Segment SGI345804[] = {{&S32759[81], 5}, {NULL}};
const Segment SGI345805[] = {{&S32759[12], 14}, {&S32759[40], 11}, {NULL}};
const Segment SGI345806[] = {{&S32759[12], 14}, {NULL}};
const Segment SGI345807[] = {{&S32767[6], 4}, {NULL}};
const Segment SGI345808[] = {{&S32767[59], 5}, {NULL}};
const Segment SGI345809[] = {{&S32767[12], 14}, {NULL}};
const Segment SGI345810[] = {{&S32768[6], 4}, {NULL}};
const Segment SGI345811[] = {{&S32768[81], 5}, {NULL}};
const Segment SGI345812[] = {{&S32768[12], 14}, {&S32768[40], 11}, {NULL}};
const Segment SGI345813[] = {{&S32768[12], 14}, {NULL}};
const Segment SGI345814[] = {{&S32769[6], 4}, {NULL}};
const Segment SGI345815[] = {{&S32769[82], 5}, {NULL}};
const Segment SGI345816[] = {{&S32769[12], 14}, {&S32769[41], 11}, {NULL}};
const Segment SGI345817[] = {{&S32769[12], 14}, {NULL}};
const Segment SGI345818[] = {{&S32770[6], 4}, {NULL}};
const Segment SGI345819[] = {{&S32770[82], 5}, {NULL}};
const Segment SGI345820[] = {{&S32770[12], 14}, {&S32770[41], 11}, {NULL}};
const Segment SGI345821[] = {{&S32770[12], 14}, {NULL}};
const Segment SGI345822[] = {{&S32771[6], 4}, {NULL}};
const Segment SGI345823[] = {{&S32771[82], 5}, {NULL}};
const Segment SGI345824[] = {{&S32771[12], 14}, {&S32771[40], 12}, {NULL}};
const Segment SGI345825[] = {{&S32771[12], 14}, {NULL}};
const Segment SGI345826[] = {{&S32772[6], 4}, {NULL}};
const Segment SGI345827[] = {{&S32772[82], 5}, {NULL}};
const Segment SGI345828[] = {{&S32772[12], 14}, {&S32772[40], 12}, {NULL}};
const Segment SGI345829[] = {{&S32772[12], 14}, {NULL}};
const Segment SGI345830[] = {{&S32774[6], 4}, {NULL}};
const Segment SGI345831[] = {{&S32774[81], 5}, {NULL}};
const Segment SGI345832[] = {{&S32774[12], 14}, {&S32774[40], 11}, {NULL}};
const Segment SGI345833[] = {{&S32774[12], 14}, {NULL}};
const Segment SGI345834[] = {{&S32775[6], 4}, {NULL}};
const Segment SGI345835[] = {{&S32775[82], 5}, {NULL}};
const Segment SGI345836[] = {{&S32775[12], 14}, {&S32775[40], 12}, {NULL}};
const Segment SGI345837[] = {{&S32775[12], 14}, {NULL}};
const Segment SGI345838[] = {{&S32776[6], 4}, {NULL}};
const Segment SGI345839[] = {{&S32776[111], 5}, {NULL}};
const Segment SGI345840[] = {{&S32776[12], 14}, {&S32776[44], 10}, {NULL}};
const Segment SGI345841[] = {{&S32776[12], 14}, {&S32776[66], 14}, {NULL}};
const Segment SGI345842[] = {{&S32778[6], 4}, {NULL}};
const Segment SGI345843[] = {{&S32778[81], 5}, {NULL}};
const Segment SGI345844[] = {{&S32778[12], 14}, {&S32778[40], 11}, {NULL}};
const Segment SGI345845[] = {{&S32778[12], 14}, {NULL}};
const Segment SGI345846[] = {{&S32779[6], 4}, {NULL}};
const Segment SGI345847[] = {{&S32779[81], 5}, {NULL}};
const Segment SGI345848[] = {{&S32779[12], 14}, {&S32779[40], 11}, {NULL}};
const Segment SGI345849[] = {{&S32779[12], 14}, {NULL}};
const Segment SGI345850[] = {{&S32780[6], 4}, {NULL}};
const Segment SGI345851[] = {{&S32780[82], 5}, {NULL}};
const Segment SGI345852[] = {{&S32780[12], 14}, {&S32780[40], 12}, {NULL}};
const Segment SGI345853[] = {{&S32780[12], 14}, {NULL}};
const Segment SGI345854[] = {{&S32781[6], 4}, {NULL}};
const Segment SGI345855[] = {{&S32781[59], 5}, {NULL}};
const Segment SGI345856[] = {{&S32781[12], 14}, {&S32781[70], 31}, {NULL}};
const Segment SGI345857[] = {{&S32781[12], 14}, {NULL}};
const Segment SGI345858[] = {{&S32782[6], 4}, {NULL}};
const Segment SGI345859[] = {{&S32782[59], 5}, {NULL}};
const Segment SGI345860[] = {{&S32782[12], 14}, {NULL}};
const Segment SGI345861[] = {{&S32783[6], 4}, {NULL}};
const Segment SGI345862[] = {{&S32783[82], 5}, {NULL}};
const Segment SGI345863[] = {{&S32783[12], 14}, {&S32783[40], 12}, {NULL}};
const Segment SGI345864[] = {{&S32783[12], 14}, {NULL}};
const Segment SGI345865[] = {{&S32784[6], 4}, {NULL}};
const Segment SGI345866[] = {{&S32784[82], 5}, {NULL}};
const Segment SGI345867[] = {{&S32784[12], 14}, {&S32784[40], 12}, {NULL}};
const Segment SGI345868[] = {{&S32784[12], 14}, {NULL}};
const Segment SGI345869[] = {{&S32785[6], 4}, {NULL}};
const Segment SGI345870[] = {{&S32785[82], 5}, {NULL}};
const Segment SGI345871[] = {{&S32785[12], 14}, {&S32785[41], 11}, {NULL}};
const Segment SGI345872[] = {{&S32785[12], 14}, {NULL}};
const Segment SGI345873[] = {{&S32786[6], 4}, {NULL}};
const Segment SGI345874[] = {{&S32786[82], 5}, {NULL}};
const Segment SGI345875[] = {{&S32786[12], 14}, {&S32786[41], 11}, {NULL}};
const Segment SGI345876[] = {{&S32786[12], 14}, {NULL}};
const Segment SGI345877[] = {{&S32788[6], 4}, {NULL}};
const Segment SGI345878[] = {{&S32788[81], 5}, {NULL}};
const Segment SGI345879[] = {{&S32788[12], 14}, {&S32788[40], 11}, {NULL}};
const Segment SGI345880[] = {{&S32788[12], 14}, {NULL}};
const Segment SGI345881[] = {{&S32789[6], 4}, {NULL}};
const Segment SGI345882[] = {{&S32789[81], 5}, {NULL}};
const Segment SGI345883[] = {{&S32789[12], 14}, {&S32789[40], 11}, {NULL}};
const Segment SGI345884[] = {{&S32789[12], 14}, {NULL}};
const Segment SGI345885[] = {{&S32790[6], 4}, {NULL}};
const Segment SGI345886[] = {{&S32790[82], 5}, {NULL}};
const Segment SGI345887[] = {{&S32790[12], 14}, {&S32790[40], 12}, {NULL}};
const Segment SGI345888[] = {{&S32790[12], 14}, {NULL}};
const Segment SGI345889[] = {{&S32791[6], 4}, {NULL}};
const Segment SGI345890[] = {{&S32791[81], 5}, {NULL}};
const Segment SGI345891[] = {{&S32791[12], 14}, {&S32791[40], 11}, {NULL}};
const Segment SGI345892[] = {{&S32791[12], 14}, {NULL}};
const Segment SGI345893[] = {{&S32792[6], 4}, {NULL}};
const Segment SGI345894[] = {{&S32792[82], 5}, {NULL}};
const Segment SGI345895[] = {{&S32792[12], 14}, {&S32792[40], 12}, {NULL}};
const Segment SGI345896[] = {{&S32792[12], 14}, {NULL}};
const Segment SGI345897[] = {{&S32793[6], 4}, {NULL}};
const Segment SGI345898[] = {{&S32793[82], 5}, {NULL}};
const Segment SGI345899[] = {{&S32793[12], 14}, {&S32793[40], 12}, {NULL}};
const Segment SGI345900[] = {{&S32793[12], 14}, {NULL}};
const Segment SGI345901[] = {{&S32794[6], 4}, {NULL}};
const Segment SGI345902[] = {{&S32794[81], 5}, {NULL}};
const Segment SGI345903[] = {{&S32794[12], 14}, {&S32794[40], 11}, {NULL}};
const Segment SGI345904[] = {{&S32794[12], 14}, {NULL}};
const Segment SGI345905[] = {{&S32795[6], 4}, {NULL}};
const Segment SGI345906[] = {{&S32795[59], 5}, {NULL}};
const Segment SGI345907[] = {{&S32795[12], 14}, {&S32795[70], 31}, {NULL}};
const Segment SGI345908[] = {{&S32795[12], 14}, {NULL}};
const Segment SGI345909[] = {{&S32796[6], 4}, {NULL}};
const Segment SGI345910[] = {{&S32796[81], 5}, {NULL}};
const Segment SGI345911[] = {{&S32796[12], 14}, {&S32796[40], 11}, {NULL}};
const Segment SGI345912[] = {{&S32796[12], 14}, {NULL}};
const Segment SGI345913[] = {{&S32798[6], 4}, {NULL}};
const Segment SGI345914[] = {{&S32798[58], 5}, {NULL}};
const Segment SGI345915[] = {{&S32798[12], 14}, {NULL}};
const Segment SGI345916[] = {{&S32799[6], 4}, {NULL}};
const Segment SGI345917[] = {{&S32799[81], 5}, {NULL}};
const Segment SGI345918[] = {{&S32799[12], 14}, {&S32799[40], 11}, {NULL}};
const Segment SGI345919[] = {{&S32799[12], 14}, {NULL}};
const Segment SGI345920[] = {{&S32800[6], 4}, {NULL}};
const Segment SGI345921[] = {{&S32800[81], 5}, {NULL}};
const Segment SGI345922[] = {{&S32800[12], 14}, {&S32800[40], 11}, {NULL}};
const Segment SGI345923[] = {{&S32800[12], 14}, {NULL}};
const Segment SGI345924[] = {{&S32801[6], 4}, {NULL}};
const Segment SGI345925[] = {{&S32801[81], 5}, {NULL}};
const Segment SGI345926[] = {{&S32801[12], 14}, {&S32801[40], 11}, {NULL}};
const Segment SGI345927[] = {{&S32801[12], 14}, {NULL}};
const Segment SGI345928[] = {{&S32802[6], 4}, {NULL}};
const Segment SGI345929[] = {{&S32802[81], 5}, {NULL}};
const Segment SGI345930[] = {{&S32802[12], 14}, {&S32802[40], 11}, {NULL}};
const Segment SGI345931[] = {{&S32802[12], 14}, {NULL}};
const Segment SGI345932[] = {{&S32803[6], 4}, {NULL}};
const Segment SGI345933[] = {{&S32803[108], 5}, {NULL}};
const Segment SGI345934[] = {{&S32803[12], 14}, {&S32803[40], 12}, {NULL}};
const Segment SGI345935[] = {{&S32803[12], 14}, {NULL}};
const Segment SGI345936[] = {{&S32804[6], 4}, {NULL}};
const Segment SGI345937[] = {{&S32804[81], 5}, {NULL}};
const Segment SGI345938[] = {{&S32804[12], 14}, {&S32804[40], 11}, {NULL}};
const Segment SGI345939[] = {{&S32804[12], 14}, {NULL}};
const Segment SGI345940[] = {{&S59530[5], 4}, {&S59530[5], 4}, {NULL}};
const Segment SGI345941[] = {{&S59530[111], 3}, {NULL}};
const Segment SGI345942[] = {{&S32936[6], 4}, {NULL}};
const Segment SGI345943[] = {{&S32936[57], 5}, {NULL}};
const Segment SGI345944[] = {{&S32936[12], 14}, {NULL}};
const Segment SGI345945[] = {{&S32937[6], 4}, {NULL}};
const Segment SGI345946[] = {{&S32937[57], 5}, {NULL}};
const Segment SGI345947[] = {{&S32937[12], 14}, {NULL}};
const Segment SGI345948[] = {{&S32938[6], 4}, {NULL}};
const Segment SGI345949[] = {{&S32938[80], 5}, {NULL}};
const Segment SGI345950[] = {{&S32938[12], 14}, {&S32938[40], 12}, {NULL}};
const Segment SGI345951[] = {{&S32938[12], 14}, {NULL}};
const Segment SGI345952[] = {{&S32939[6], 4}, {NULL}};
const Segment SGI345953[] = {{&S32939[80], 5}, {NULL}};
const Segment SGI345954[] = {{&S32939[12], 14}, {&S32939[40], 12}, {NULL}};
const Segment SGI345955[] = {{&S32939[12], 14}, {NULL}};
const Segment SGI345956[] = {{&S32941[6], 4}, {NULL}};
const Segment SGI345957[] = {{&S32941[80], 5}, {NULL}};
const Segment SGI345958[] = {{&S32941[12], 14}, {&S32941[40], 12}, {NULL}};
const Segment SGI345959[] = {{&S32941[12], 14}, {NULL}};
const Segment SGI345960[] = {{&S59564[42], 5}, {NULL}};
const Segment SGI345961[] = {{&S32943[6], 4}, {NULL}};
const Segment SGI345962[] = {{&S59564[6], 3}, {NULL}};
const Segment SGI345963[] = {{&S32943[80], 5}, {NULL}};
const Segment SGI345964[] = {{&S32943[12], 14}, {&S32943[40], 12}, {NULL}};
const Segment SGI345965[] = {{&S59564[68], 3}, {NULL}};
const Segment SGI345966[] = {{&S32943[12], 14}, {NULL}};
const Segment SGI345967[] = {{&S59565[5], 5}, {NULL}};
const Segment SGI345968[] = {{&S59565[16], 3}, {NULL}};
const Segment SGI345969[] = {{&S33011[6], 4}, {NULL}};
const Segment SGI345970[] = {{&S33011[58], 5}, {NULL}};
const Segment SGI345971[] = {{&S33011[12], 14}, {NULL}};
const Segment SGI345972[] = {{&S33012[6], 4}, {NULL}};
const Segment SGI345973[] = {{&S33012[58], 5}, {NULL}};
const Segment SGI345974[] = {{&S33012[12], 14}, {NULL}};
const Segment SGI345975[] = {{&S33013[6], 4}, {NULL}};
const Segment SGI345976[] = {{&S33013[80], 5}, {NULL}};
const Segment SGI345977[] = {{&S33013[12], 14}, {&S33013[40], 11}, {NULL}};
const Segment SGI345978[] = {{&S33013[12], 14}, {NULL}};
const Segment SGI345979[] = {{&S33014[6], 4}, {NULL}};
const Segment SGI345980[] = {{&S33014[80], 5}, {NULL}};
const Segment SGI345981[] = {{&S33014[12], 14}, {&S33014[40], 11}, {NULL}};
const Segment SGI345982[] = {{&S33014[12], 14}, {NULL}};
const Segment SGI345983[] = {{&S33015[6], 4}, {NULL}};
const Segment SGI345984[] = {{&S33015[80], 5}, {NULL}};
const Segment SGI345985[] = {{&S33015[12], 14}, {&S33015[40], 11}, {NULL}};
const Segment SGI345986[] = {{&S33015[12], 14}, {NULL}};
const Segment SGI345987[] = {{&S33016[6], 4}, {NULL}};
const Segment SGI345988[] = {{&S33016[81], 5}, {NULL}};
const Segment SGI345989[] = {{&S33016[12], 14}, {&S33016[40], 12}, {NULL}};
const Segment SGI345990[] = {{&S33016[12], 14}, {NULL}};
const Segment SGI345991[] = {{&S33017[6], 4}, {NULL}};
const Segment SGI345992[] = {{&S33017[81], 5}, {NULL}};
const Segment SGI345993[] = {{&S33017[12], 14}, {&S33017[40], 12}, {NULL}};
const Segment SGI345994[] = {{&S33017[12], 14}, {NULL}};
const Segment SGI345995[] = {{&S33018[6], 4}, {NULL}};
const Segment SGI345996[] = {{&S33018[81], 5}, {NULL}};
const Segment SGI345997[] = {{&S33018[12], 14}, {&S33018[40], 12}, {NULL}};
const Segment SGI345998[] = {{&S33018[12], 14}, {NULL}};
const Segment SGI345999[] = {{&S33019[6], 4}, {NULL}};
const Segment SGI346000[] = {{&S33019[81], 5}, {NULL}};
const Segment SGI346001[] = {{&S33019[12], 14}, {&S33019[40], 12}, {NULL}};
const Segment SGI346002[] = {{&S33019[12], 14}, {NULL}};
const Segment SGI346003[] = {{&S33020[6], 4}, {NULL}};
const Segment SGI346004[] = {{&S33020[81], 5}, {NULL}};
const Segment SGI346005[] = {{&S33020[12], 14}, {&S33020[40], 12}, {NULL}};
const Segment SGI346006[] = {{&S33020[12], 14}, {NULL}};
const Segment SGI346007[] = {{&S33021[6], 4}, {NULL}};
const Segment SGI346008[] = {{&S33021[80], 5}, {NULL}};
const Segment SGI346009[] = {{&S33021[12], 14}, {&S33021[40], 11}, {NULL}};
const Segment SGI346010[] = {{&S33021[12], 14}, {NULL}};
const Segment SGI346011[] = {{&S33022[6], 4}, {NULL}};
const Segment SGI346012[] = {{&S33022[80], 5}, {NULL}};
const Segment SGI346013[] = {{&S33022[12], 14}, {&S33022[40], 11}, {NULL}};
const Segment SGI346014[] = {{&S33022[12], 14}, {NULL}};
const Segment SGI346015[] = {{&S33024[6], 4}, {NULL}};
const Segment SGI346016[] = {{&S33024[81], 5}, {NULL}};
const Segment SGI346017[] = {{&S33024[12], 14}, {&S33024[40], 12}, {NULL}};
const Segment SGI346018[] = {{&S33024[12], 14}, {NULL}};
const Segment SGI346019[] = {{&S33025[6], 4}, {NULL}};
const Segment SGI346020[] = {{&S33025[81], 5}, {NULL}};
const Segment SGI346021[] = {{&S33025[12], 14}, {&S33025[40], 12}, {NULL}};
const Segment SGI346022[] = {{&S33025[12], 14}, {NULL}};
const Segment SGI346023[] = {{&S33026[6], 4}, {NULL}};
const Segment SGI346024[] = {{&S33026[81], 5}, {NULL}};
const Segment SGI346025[] = {{&S33026[12], 14}, {&S33026[40], 12}, {NULL}};
const Segment SGI346026[] = {{&S33026[12], 14}, {NULL}};
const Segment SGI346027[] = {{&S33027[6], 4}, {NULL}};
const Segment SGI346028[] = {{&S33027[80], 5}, {NULL}};
const Segment SGI346029[] = {{&S33027[12], 14}, {&S33027[40], 11}, {NULL}};
const Segment SGI346030[] = {{&S33027[12], 14}, {NULL}};
const Segment SGI346031[] = {{&S33028[6], 4}, {NULL}};
const Segment SGI346032[] = {{&S33028[80], 5}, {NULL}};
const Segment SGI346033[] = {{&S33028[12], 14}, {&S33028[40], 11}, {NULL}};
const Segment SGI346034[] = {{&S33028[12], 14}, {NULL}};
const Segment SGI346035[] = {{&S33029[6], 4}, {NULL}};
const Segment SGI346036[] = {{&S33029[81], 5}, {NULL}};
const Segment SGI346037[] = {{&S33029[12], 14}, {&S33029[40], 12}, {NULL}};
const Segment SGI346038[] = {{&S33029[12], 14}, {NULL}};
const Segment SGI346039[] = {{&S33030[6], 4}, {NULL}};
const Segment SGI346040[] = {{&S33030[81], 5}, {NULL}};
const Segment SGI346041[] = {{&S33030[12], 14}, {&S33030[40], 12}, {NULL}};
const Segment SGI346042[] = {{&S33030[12], 14}, {NULL}};
const Segment SGI346043[] = {{&S33031[6], 4}, {NULL}};
const Segment SGI346044[] = {{&S33031[80], 5}, {NULL}};
const Segment SGI346045[] = {{&S33031[12], 14}, {&S33031[40], 11}, {NULL}};
const Segment SGI346046[] = {{&S33031[12], 14}, {NULL}};
const Segment SGI346047[] = {{&S33032[6], 4}, {NULL}};
const Segment SGI346048[] = {{&S33032[81], 5}, {NULL}};
const Segment SGI346049[] = {{&S33032[12], 14}, {&S33032[40], 12}, {NULL}};
const Segment SGI346050[] = {{&S33032[12], 14}, {NULL}};
const Segment SGI346051[] = {{&S33034[6], 4}, {NULL}};
const Segment SGI346052[] = {{&S33034[80], 5}, {NULL}};
const Segment SGI346053[] = {{&S33034[12], 14}, {&S33034[40], 11}, {NULL}};
const Segment SGI346054[] = {{&S33034[12], 14}, {NULL}};
const Segment SGI346055[] = {{&S33035[6], 4}, {NULL}};
const Segment SGI346056[] = {{&S33035[61], 5}, {NULL}};
const Segment SGI346057[] = {{&S33035[12], 14}, {&S33035[72], 31}, {NULL}};
const Segment SGI346058[] = {{&S33035[12], 14}, {NULL}};
const Segment SGI346059[] = {{&S33036[6], 4}, {NULL}};
const Segment SGI346060[] = {{&S33036[80], 5}, {NULL}};
const Segment SGI346061[] = {{&S33036[12], 14}, {&S33036[40], 11}, {NULL}};
const Segment SGI346062[] = {{&S33036[12], 14}, {NULL}};
const Segment SGI346063[] = {{&S33037[6], 4}, {NULL}};
const Segment SGI346064[] = {{&S33037[81], 5}, {NULL}};
const Segment SGI346065[] = {{&S33037[12], 14}, {&S33037[40], 12}, {NULL}};
const Segment SGI346066[] = {{&S33037[12], 14}, {NULL}};
const Segment SGI346067[] = {{&S33038[6], 4}, {NULL}};
const Segment SGI346068[] = {{&S33038[81], 5}, {NULL}};
const Segment SGI346069[] = {{&S33038[12], 14}, {&S33038[41], 11}, {NULL}};
const Segment SGI346070[] = {{&S33038[12], 14}, {NULL}};
const Segment SGI346071[] = {{&S33039[6], 4}, {NULL}};
const Segment SGI346072[] = {{&S33039[81], 5}, {NULL}};
const Segment SGI346073[] = {{&S33039[12], 14}, {&S33039[41], 11}, {NULL}};
const Segment SGI346074[] = {{&S33039[12], 14}, {NULL}};
const Segment SGI346075[] = {{&S33040[6], 4}, {NULL}};
const Segment SGI346076[] = {{&S33040[80], 5}, {NULL}};
const Segment SGI346077[] = {{&S33040[12], 14}, {&S33040[40], 11}, {NULL}};
const Segment SGI346078[] = {{&S33040[12], 14}, {NULL}};
const Segment SGI346079[] = {{&S33041[6], 4}, {NULL}};
const Segment SGI346080[] = {{&S33041[80], 5}, {NULL}};
const Segment SGI346081[] = {{&S33041[12], 14}, {&S33041[40], 11}, {NULL}};
const Segment SGI346082[] = {{&S33041[12], 14}, {NULL}};
const Segment SGI346083[] = {{&S33042[6], 4}, {NULL}};
const Segment SGI346084[] = {{&S33042[80], 5}, {NULL}};
const Segment SGI346085[] = {{&S33042[12], 14}, {&S33042[40], 11}, {NULL}};
const Segment SGI346086[] = {{&S33042[12], 14}, {NULL}};
const Segment SGI346087[] = {{&S33043[6], 4}, {NULL}};
const Segment SGI346088[] = {{&S33043[81], 5}, {NULL}};
const Segment SGI346089[] = {{&S33043[12], 14}, {&S33043[40], 12}, {NULL}};
const Segment SGI346090[] = {{&S33043[12], 14}, {NULL}};
const Segment SGI346091[] = {{&S33044[6], 4}, {NULL}};
const Segment SGI346092[] = {{&S33044[81], 5}, {NULL}};
const Segment SGI346093[] = {{&S33044[12], 14}, {&S33044[40], 12}, {NULL}};
const Segment SGI346094[] = {{&S33044[12], 14}, {NULL}};
const Segment SGI346095[] = {{&S33045[6], 4}, {NULL}};
const Segment SGI346096[] = {{&S33045[81], 5}, {NULL}};
const Segment SGI346097[] = {{&S33045[12], 14}, {&S33045[41], 11}, {NULL}};
const Segment SGI346098[] = {{&S33045[12], 14}, {NULL}};
const Segment SGI346099[] = {{&S33046[6], 4}, {NULL}};
const Segment SGI346100[] = {{&S33046[81], 5}, {NULL}};
const Segment SGI346101[] = {{&S33046[12], 14}, {&S33046[40], 12}, {NULL}};
const Segment SGI346102[] = {{&S33046[12], 14}, {NULL}};
const Segment SGI346103[] = {{&S33047[6], 4}, {NULL}};
const Segment SGI346104[] = {{&S33047[81], 5}, {NULL}};
const Segment SGI346105[] = {{&S33047[12], 14}, {&S33047[41], 11}, {NULL}};
const Segment SGI346106[] = {{&S33047[12], 14}, {NULL}};
const Segment SGI346107[] = {{&S33048[6], 4}, {NULL}};
const Segment SGI346108[] = {{&S33048[81], 5}, {NULL}};
const Segment SGI346109[] = {{&S33048[12], 14}, {&S33048[40], 12}, {NULL}};
const Segment SGI346110[] = {{&S33048[12], 14}, {NULL}};
const Segment SGI346111[] = {{&S33049[6], 4}, {NULL}};
const Segment SGI346112[] = {{&S33049[81], 5}, {NULL}};
const Segment SGI346113[] = {{&S33049[12], 14}, {&S33049[41], 11}, {NULL}};
const Segment SGI346114[] = {{&S33049[12], 14}, {NULL}};
const Segment SGI346115[] = {{&S33050[6], 4}, {NULL}};
const Segment SGI346116[] = {{&S33050[81], 5}, {NULL}};
const Segment SGI346117[] = {{&S33050[12], 14}, {&S33050[40], 12}, {NULL}};
const Segment SGI346118[] = {{&S33050[12], 14}, {NULL}};
const Segment SGI346119[] = {{&S33051[6], 4}, {NULL}};
const Segment SGI346120[] = {{&S33051[80], 5}, {NULL}};
const Segment SGI346121[] = {{&S33051[12], 14}, {&S33051[40], 11}, {NULL}};
const Segment SGI346122[] = {{&S33051[12], 14}, {NULL}};
const Segment SGI346123[] = {{&S33052[6], 4}, {NULL}};
const Segment SGI346124[] = {{&S33052[80], 5}, {NULL}};
const Segment SGI346125[] = {{&S33052[12], 14}, {&S33052[40], 11}, {NULL}};
const Segment SGI346126[] = {{&S33052[12], 14}, {NULL}};
const Segment SGI346127[] = {{&S33053[6], 4}, {NULL}};
const Segment SGI346128[] = {{&S33053[80], 5}, {NULL}};
const Segment SGI346129[] = {{&S33053[12], 14}, {&S33053[40], 11}, {NULL}};
const Segment SGI346130[] = {{&S33053[12], 14}, {NULL}};
const Segment SGI346131[] = {{&S33054[6], 4}, {NULL}};
const Segment SGI346132[] = {{&S33054[80], 5}, {NULL}};
const Segment SGI346133[] = {{&S33054[12], 14}, {&S33054[40], 11}, {NULL}};
const Segment SGI346134[] = {{&S33054[12], 14}, {NULL}};
const Segment SGI346135[] = {{&S33055[6], 4}, {NULL}};
const Segment SGI346136[] = {{&S33055[80], 5}, {NULL}};
const Segment SGI346137[] = {{&S33055[12], 14}, {&S33055[40], 11}, {NULL}};
const Segment SGI346138[] = {{&S33055[12], 14}, {NULL}};
const Segment SGI346139[] = {{&S33056[6], 4}, {NULL}};
const Segment SGI346140[] = {{&S33056[80], 5}, {NULL}};
const Segment SGI346141[] = {{&S33056[12], 14}, {&S33056[40], 11}, {NULL}};
const Segment SGI346142[] = {{&S33056[12], 14}, {NULL}};
const Segment SGI346143[] = {{&S33057[6], 4}, {NULL}};
const Segment SGI346144[] = {{&S33057[81], 5}, {NULL}};
const Segment SGI346145[] = {{&S33057[12], 14}, {&S33057[41], 11}, {NULL}};
const Segment SGI346146[] = {{&S33057[12], 14}, {NULL}};
const Segment SGI346147[] = {{&S33058[6], 4}, {NULL}};
const Segment SGI346148[] = {{&S33058[80], 5}, {NULL}};
const Segment SGI346149[] = {{&S33058[12], 14}, {&S33058[40], 11}, {NULL}};
const Segment SGI346150[] = {{&S33058[12], 14}, {NULL}};
const Segment SGI346151[] = {{&S33059[6], 4}, {NULL}};
const Segment SGI346152[] = {{&S33059[80], 5}, {NULL}};
const Segment SGI346153[] = {{&S33059[12], 14}, {&S33059[40], 11}, {NULL}};
const Segment SGI346154[] = {{&S33059[12], 14}, {NULL}};
const Segment SGI346155[] = {{&S33060[6], 4}, {NULL}};
const Segment SGI346156[] = {{&S33060[80], 5}, {NULL}};
const Segment SGI346157[] = {{&S33060[12], 14}, {&S33060[40], 11}, {NULL}};
const Segment SGI346158[] = {{&S33060[12], 14}, {NULL}};
const Segment SGI346159[] = {{&S33061[6], 4}, {NULL}};
const Segment SGI346160[] = {{&S33061[58], 5}, {NULL}};
const Segment SGI346161[] = {{&S33061[12], 14}, {&S33061[69], 4}, {NULL}};
const Segment SGI346162[] = {{&S33061[12], 14}, {NULL}};
const Segment SGI346163[] = {{&S33062[6], 4}, {NULL}};
const Segment SGI346164[] = {{&S33062[80], 5}, {NULL}};
const Segment SGI346165[] = {{&S33062[12], 14}, {&S33062[40], 11}, {NULL}};
const Segment SGI346166[] = {{&S33062[12], 14}, {NULL}};
const Segment SGI346167[] = {{&S33063[6], 4}, {NULL}};
const Segment SGI346168[] = {{&S33063[80], 5}, {NULL}};
const Segment SGI346169[] = {{&S33063[12], 14}, {&S33063[40], 11}, {NULL}};
const Segment SGI346170[] = {{&S33063[12], 14}, {NULL}};
const Segment SGI346171[] = {{&S33064[6], 4}, {NULL}};
const Segment SGI346172[] = {{&S33064[80], 5}, {NULL}};
const Segment SGI346173[] = {{&S33064[12], 14}, {&S33064[40], 11}, {NULL}};
const Segment SGI346174[] = {{&S33064[12], 14}, {NULL}};
const Segment SGI346175[] = {{&S33065[6], 4}, {NULL}};
const Segment SGI346176[] = {{&S33065[80], 5}, {NULL}};
const Segment SGI346177[] = {{&S33065[12], 14}, {&S33065[40], 11}, {NULL}};
const Segment SGI346178[] = {{&S33065[12], 14}, {NULL}};
const Segment SGI346179[] = {{&S33066[6], 4}, {NULL}};
const Segment SGI346180[] = {{&S33066[80], 5}, {NULL}};
const Segment SGI346181[] = {{&S33066[12], 14}, {&S33066[40], 11}, {NULL}};
const Segment SGI346182[] = {{&S33066[12], 14}, {NULL}};
const Segment SGI346183[] = {{&S33067[6], 4}, {NULL}};
const Segment SGI346184[] = {{&S33067[80], 5}, {NULL}};
const Segment SGI346185[] = {{&S33067[12], 14}, {&S33067[40], 11}, {NULL}};
const Segment SGI346186[] = {{&S33067[12], 14}, {NULL}};
const Segment SGI346187[] = {{&S33068[6], 4}, {NULL}};
const Segment SGI346188[] = {{&S33068[80], 5}, {NULL}};
const Segment SGI346189[] = {{&S33068[12], 14}, {&S33068[40], 11}, {NULL}};
const Segment SGI346190[] = {{&S33068[12], 14}, {NULL}};
const Segment SGI346191[] = {{&S33069[6], 4}, {NULL}};
const Segment SGI346192[] = {{&S33069[57], 5}, {NULL}};
const Segment SGI346193[] = {{&S33069[12], 14}, {NULL}};
const Segment SGI346194[] = {{&S33070[6], 4}, {NULL}};
const Segment SGI346195[] = {{&S33070[57], 5}, {NULL}};
const Segment SGI346196[] = {{&S33070[12], 14}, {NULL}};
const Segment SGI346197[] = {{&S33071[6], 4}, {NULL}};
const Segment SGI346198[] = {{&S33071[57], 5}, {NULL}};
const Segment SGI346199[] = {{&S33071[12], 14}, {NULL}};
const Segment SGI346200[] = {{&S33072[6], 4}, {NULL}};
const Segment SGI346201[] = {{&S33072[57], 5}, {NULL}};
const Segment SGI346202[] = {{&S33072[12], 14}, {NULL}};
const Segment SGI346203[] = {{&S33073[6], 4}, {NULL}};
const Segment SGI346204[] = {{&S33073[57], 5}, {NULL}};
const Segment SGI346205[] = {{&S33073[12], 14}, {NULL}};
const Segment SGI346206[] = {{&S59715[5], 3}, {&S59715[5], 3}, {NULL}};
const Segment SGI346207[] = {{&S59715[9], 3}, {NULL}};
const Segment SGI346208[] = {{&S59715[127], 3}, {NULL}};
const Segment SGI346209[] = {{&S33074[6], 4}, {NULL}};
const Segment SGI346210[] = {{&S33074[80], 5}, {NULL}};
const Segment SGI346211[] = {{&S33074[12], 14}, {&S33074[40], 12}, {NULL}};
const Segment SGI346212[] = {{&S33074[12], 14}, {NULL}};
const Segment SGI346213[] = {{&S33075[6], 4}, {NULL}};
const Segment SGI346214[] = {{&S33075[80], 5}, {NULL}};
const Segment SGI346215[] = {{&S33075[12], 14}, {&S33075[40], 12}, {NULL}};
const Segment SGI346216[] = {{&S33075[12], 14}, {NULL}};
const Segment SGI346217[] = {{&S33076[6], 4}, {NULL}};
const Segment SGI346218[] = {{&S33076[57], 5}, {NULL}};
const Segment SGI346219[] = {{&S33076[12], 14}, {&S33076[68], 31}, {NULL}};
const Segment SGI346220[] = {{&S33076[12], 14}, {NULL}};
const Segment SGI346221[] = {{&S33077[6], 4}, {NULL}};
const Segment SGI346222[] = {{&S33077[80], 5}, {NULL}};
const Segment SGI346223[] = {{&S33077[12], 14}, {&S33077[40], 12}, {NULL}};
const Segment SGI346224[] = {{&S33077[12], 14}, {NULL}};
const Segment SGI346225[] = {{&S33078[6], 4}, {NULL}};
const Segment SGI346226[] = {{&S33078[80], 5}, {NULL}};
const Segment SGI346227[] = {{&S33078[12], 14}, {&S33078[40], 12}, {NULL}};
const Segment SGI346228[] = {{&S33078[12], 14}, {NULL}};
const Segment SGI346229[] = {{&S33079[6], 4}, {NULL}};
const Segment SGI346230[] = {{&S33079[57], 5}, {NULL}};
const Segment SGI346231[] = {{&S33079[12], 14}, {NULL}};
const Segment SGI346232[] = {{&S33080[6], 4}, {NULL}};
const Segment SGI346233[] = {{&S33080[80], 5}, {NULL}};
const Segment SGI346234[] = {{&S33080[12], 14}, {&S33080[40], 12}, {NULL}};
const Segment SGI346235[] = {{&S33080[12], 14}, {NULL}};
const Segment SGI346236[] = {{&S33081[6], 4}, {NULL}};
const Segment SGI346237[] = {{&S33081[80], 5}, {NULL}};
const Segment SGI346238[] = {{&S33081[12], 14}, {&S33081[40], 12}, {NULL}};
const Segment SGI346239[] = {{&S33081[12], 14}, {NULL}};
const Segment SGI346240[] = {{&S33082[6], 4}, {NULL}};
const Segment SGI346241[] = {{&S33082[57], 5}, {NULL}};
const Segment SGI346242[] = {{&S33082[12], 14}, {NULL}};
const Segment SGI346243[] = {{&S33083[6], 4}, {NULL}};
const Segment SGI346244[] = {{&S33083[80], 5}, {NULL}};
const Segment SGI346245[] = {{&S33083[12], 14}, {&S33083[40], 12}, {NULL}};
const Segment SGI346246[] = {{&S33083[12], 14}, {NULL}};
const Segment SGI346247[] = {{&S33084[6], 4}, {NULL}};
const Segment SGI346248[] = {{&S33084[57], 5}, {NULL}};
const Segment SGI346249[] = {{&S33084[12], 14}, {NULL}};
const Segment SGI346250[] = {{&S33085[6], 4}, {NULL}};
const Segment SGI346251[] = {{&S33085[79], 5}, {NULL}};
const Segment SGI346252[] = {{&S33085[12], 14}, {&S33085[40], 11}, {NULL}};
const Segment SGI346253[] = {{&S33085[12], 14}, {NULL}};
const Segment SGI346254[] = {{&S33086[6], 4}, {NULL}};
const Segment SGI346255[] = {{&S33086[80], 5}, {NULL}};
const Segment SGI346256[] = {{&S33086[12], 14}, {&S33086[40], 12}, {NULL}};
const Segment SGI346257[] = {{&S33086[12], 14}, {NULL}};
const Segment SGI346258[] = {{&S33087[6], 4}, {NULL}};
const Segment SGI346259[] = {{&S33087[80], 5}, {NULL}};
const Segment SGI346260[] = {{&S33087[12], 14}, {&S33087[40], 12}, {NULL}};
const Segment SGI346261[] = {{&S33087[12], 14}, {NULL}};
const Segment SGI346262[] = {{&S33088[6], 4}, {NULL}};
const Segment SGI346263[] = {{&S33088[79], 5}, {NULL}};
const Segment SGI346264[] = {{&S33088[12], 14}, {&S33088[40], 11}, {NULL}};
const Segment SGI346265[] = {{&S33088[12], 14}, {NULL}};
const Segment SGI346266[] = {{&S33089[6], 4}, {NULL}};
const Segment SGI346267[] = {{&S33089[79], 5}, {NULL}};
const Segment SGI346268[] = {{&S33089[12], 14}, {&S33089[40], 11}, {NULL}};
const Segment SGI346269[] = {{&S33089[12], 14}, {NULL}};
const Segment SGI346270[] = {{&S33090[6], 4}, {NULL}};
const Segment SGI346271[] = {{&S33090[79], 5}, {NULL}};
const Segment SGI346272[] = {{&S33090[12], 14}, {&S33090[40], 11}, {NULL}};
const Segment SGI346273[] = {{&S33090[12], 14}, {NULL}};
const Segment SGI346274[] = {{&S59738[5], 5}, {&S59738[97], 5}, {NULL}};
const Segment SGI346275[] = {{&S33091[6], 4}, {NULL}};
const Segment SGI346276[] = {{&S59738[153], 3}, {NULL}};
const Segment SGI346277[] = {{&S33091[80], 5}, {NULL}};
const Segment SGI346278[] = {{&S33091[12], 14}, {&S33091[41], 11}, {NULL}};
const Segment SGI346279[] = {{&S33091[12], 14}, {NULL}};
const Segment SGI346280[] = {{&S33092[6], 4}, {NULL}};
const Segment SGI346281[] = {{&S33092[80], 5}, {NULL}};
const Segment SGI346282[] = {{&S33092[12], 14}, {&S33092[40], 12}, {NULL}};
const Segment SGI346283[] = {{&S33092[12], 14}, {NULL}};
const Segment SGI346284[] = {{&S33093[6], 4}, {NULL}};
const Segment SGI346285[] = {{&S33093[80], 5}, {NULL}};
const Segment SGI346286[] = {{&S33093[12], 14}, {&S33093[40], 12}, {NULL}};
const Segment SGI346287[] = {{&S33093[12], 14}, {NULL}};
const Segment SGI346288[] = {{&S33094[6], 4}, {NULL}};
const Segment SGI346289[] = {{&S33094[79], 5}, {NULL}};
const Segment SGI346290[] = {{&S33094[12], 14}, {&S33094[40], 11}, {NULL}};
const Segment SGI346291[] = {{&S33094[12], 14}, {NULL}};
const Segment SGI346292[] = {{&S33095[6], 4}, {NULL}};
const Segment SGI346293[] = {{&S33095[79], 5}, {NULL}};
const Segment SGI346294[] = {{&S33095[12], 14}, {&S33095[40], 11}, {NULL}};
const Segment SGI346295[] = {{&S33095[12], 14}, {NULL}};
const Segment SGI346296[] = {{&S33096[6], 4}, {NULL}};
const Segment SGI346297[] = {{&S33096[57], 5}, {NULL}};
const Segment SGI346298[] = {{&S33096[12], 14}, {&S33096[68], 4}, {NULL}};
const Segment SGI346299[] = {{&S33096[12], 14}, {NULL}};
const Segment SGI346300[] = {{&S33097[6], 4}, {NULL}};
const Segment SGI346301[] = {{&S33097[79], 5}, {NULL}};
const Segment SGI346302[] = {{&S33097[12], 14}, {&S33097[40], 11}, {NULL}};
const Segment SGI346303[] = {{&S33097[12], 14}, {NULL}};
const Segment SGI346304[] = {{&S33098[6], 4}, {NULL}};
const Segment SGI346305[] = {{&S33098[79], 5}, {NULL}};
const Segment SGI346306[] = {{&S33098[12], 14}, {&S33098[40], 11}, {NULL}};
const Segment SGI346307[] = {{&S33098[12], 14}, {NULL}};
const Segment SGI346308[] = {{&S33099[6], 4}, {NULL}};
const Segment SGI346309[] = {{&S33099[80], 5}, {NULL}};
const Segment SGI346310[] = {{&S33099[12], 14}, {&S33099[41], 11}, {NULL}};
const Segment SGI346311[] = {{&S33099[12], 14}, {NULL}};
const Segment SGI346312[] = {{&S33100[6], 4}, {NULL}};
const Segment SGI346313[] = {{&S33100[79], 5}, {NULL}};
const Segment SGI346314[] = {{&S33100[12], 14}, {&S33100[40], 11}, {NULL}};
const Segment SGI346315[] = {{&S33100[12], 14}, {NULL}};
const Segment SGI346316[] = {{&S32672[0], 15}, {NULL}};
const Segment SGI346317[] = {{&S32672[28], 3}, {NULL}};
const Segment SGI346318[] = {{&S32672[63], 13}, {NULL}};
const Segment SGI346319[] = {{&S51300[28], 3}, {NULL}};
const Segment SGI346320[] = {{&S51300[103], 5}, {NULL}};
const Segment SGI346321[] = {{&S51301[28], 4}, {NULL}};
const Segment SGI346322[] = {{&S51301[105], 5}, {NULL}};
const Segment SGI346323[] = {{&S33323[6], 4}, {NULL}};
const Segment SGI346324[] = {{&S33323[61], 5}, {NULL}};
const Segment SGI346325[] = {{&S33323[12], 3}, {NULL}};
const Segment SGI346326[] = {{&S59856[20], 4}, {NULL}};
const Segment SGI346327[] = {{&S33324[6], 4}, {NULL}};
const Segment SGI346328[] = {{&S33324[56], 5}, {NULL}};
const Segment SGI346329[] = {{&S33324[12], 12}, {NULL}};
const Segment SGI346330[] = {{&S35486[57], 5}, {NULL}};
const Segment SGI346331[] = {{&S35486[13], 10}, {NULL}};
const Segment SGI346332[] = {{&S59916[43], 16}, {NULL}};
const Segment SGI346333[] = {{&S59916[6], 3}, {NULL}};
const Segment SGI346334[] = {{&S59916[80], 3}, {NULL}};
const Segment SGI346335[] = {{&S33425[6], 4}, {NULL}};
const Segment SGI346336[] = {{&S33425[58], 5}, {NULL}};
const Segment SGI346337[] = {{&S33425[12], 14}, {NULL}};
const Segment SGI346338[] = {{&S33426[6], 4}, {NULL}};
const Segment SGI346339[] = {{&S33426[81], 5}, {NULL}};
const Segment SGI346340[] = {{&S33426[12], 14}, {&S33426[40], 12}, {NULL}};
const Segment SGI346341[] = {{&S33426[12], 14}, {NULL}};
const Segment SGI346342[] = {{&S33427[6], 4}, {NULL}};
const Segment SGI346343[] = {{&S33427[81], 5}, {NULL}};
const Segment SGI346344[] = {{&S33427[12], 14}, {&S33427[40], 12}, {NULL}};
const Segment SGI346345[] = {{&S33427[12], 14}, {NULL}};
const Segment SGI346346[] = {{&S51421[28], 3}, {NULL}};
const Segment SGI346347[] = {{&S59964[42], 5}, {NULL}};
const Segment SGI346348[] = {{&S51421[102], 5}, {NULL}};
const Segment SGI346349[] = {{&S59964[6], 3}, {NULL}};
const Segment SGI346350[] = {{&S59964[68], 3}, {NULL}};
const Segment SGI346351[] = {{&S33428[6], 4}, {NULL}};
const Segment SGI346352[] = {{&S33428[80], 5}, {NULL}};
const Segment SGI346353[] = {{&S33428[12], 14}, {&S33428[40], 11}, {NULL}};
const Segment SGI346354[] = {{&S33428[12], 14}, {NULL}};
const Segment SGI346355[] = {{&S51422[28], 5}, {NULL}};
const Segment SGI346356[] = {{&S51422[104], 5}, {NULL}};
const Segment SGI346357[] = {{&S59965[43], 3}, {NULL}};
const Segment SGI346358[] = {{&S59965[6], 3}, {NULL}};
const Segment SGI346359[] = {{&S33429[6], 4}, {NULL}};
const Segment SGI346360[] = {{&S59965[67], 3}, {NULL}};
const Segment SGI346361[] = {{&S33429[81], 5}, {NULL}};
const Segment SGI346362[] = {{&S33429[12], 14}, {&S33429[40], 12}, {NULL}};
const Segment SGI346363[] = {{&S33429[12], 14}, {NULL}};
const Segment SGI346364[] = {{&S59966[43], 4}, {NULL}};
const Segment SGI346365[] = {{&S33430[6], 4}, {NULL}};
const Segment SGI346366[] = {{&S59966[6], 3}, {NULL}};
const Segment SGI346367[] = {{&S33430[80], 5}, {NULL}};
const Segment SGI346368[] = {{&S33430[12], 14}, {&S33430[40], 11}, {NULL}};
const Segment SGI346369[] = {{&S59966[68], 3}, {NULL}};
const Segment SGI346370[] = {{&S33430[12], 14}, {NULL}};
const Segment SGI346371[] = {{&S59967[5], 3}, {NULL}};
const Segment SGI346372[] = {{&S59967[14], 3}, {NULL}};
const Segment SGI346373[] = {{&S33431[6], 4}, {NULL}};
const Segment SGI346374[] = {{&S33431[81], 5}, {NULL}};
const Segment SGI346375[] = {{&S33431[12], 14}, {&S33431[40], 12}, {NULL}};
const Segment SGI346376[] = {{&S33431[12], 14}, {NULL}};
const Segment SGI346377[] = {{&S33432[6], 4}, {NULL}};
const Segment SGI346378[] = {{&S33432[58], 5}, {NULL}};
const Segment SGI346379[] = {{&S33432[12], 14}, {&S33432[69], 31}, {NULL}};
const Segment SGI346380[] = {{&S33432[12], 14}, {NULL}};
const Segment SGI346381[] = {{&S33433[6], 4}, {NULL}};
const Segment SGI346382[] = {{&S33433[80], 5}, {NULL}};
const Segment SGI346383[] = {{&S33433[12], 14}, {&S33433[40], 11}, {NULL}};
const Segment SGI346384[] = {{&S33433[12], 14}, {NULL}};
const Segment SGI346385[] = {{&S33434[6], 4}, {NULL}};
const Segment SGI346386[] = {{&S33434[81], 5}, {NULL}};
const Segment SGI346387[] = {{&S33434[12], 14}, {&S33434[40], 12}, {NULL}};
const Segment SGI346388[] = {{&S33434[12], 14}, {NULL}};
const Segment SGI346389[] = {{&S33435[6], 4}, {NULL}};
const Segment SGI346390[] = {{&S33435[80], 5}, {NULL}};
const Segment SGI346391[] = {{&S33435[12], 14}, {&S33435[40], 11}, {NULL}};
const Segment SGI346392[] = {{&S33435[12], 14}, {NULL}};
const Segment SGI346393[] = {{&S33436[6], 4}, {NULL}};
const Segment SGI346394[] = {{&S33436[80], 5}, {NULL}};
const Segment SGI346395[] = {{&S33436[12], 14}, {&S33436[40], 11}, {NULL}};
const Segment SGI346396[] = {{&S33436[12], 14}, {NULL}};
const Segment SGI346397[] = {{&S33437[6], 4}, {NULL}};
const Segment SGI346398[] = {{&S33437[80], 5}, {NULL}};
const Segment SGI346399[] = {{&S33437[12], 14}, {&S33437[40], 11}, {NULL}};
const Segment SGI346400[] = {{&S33437[12], 14}, {NULL}};
const Segment SGI346401[] = {{&S33438[6], 4}, {NULL}};
const Segment SGI346402[] = {{&S33438[81], 5}, {NULL}};
const Segment SGI346403[] = {{&S33438[12], 14}, {&S33438[40], 12}, {NULL}};
const Segment SGI346404[] = {{&S33438[12], 14}, {NULL}};
const Segment SGI346405[] = {{&S33439[6], 4}, {NULL}};
const Segment SGI346406[] = {{&S33439[80], 5}, {NULL}};
const Segment SGI346407[] = {{&S33439[12], 14}, {&S33439[40], 11}, {NULL}};
const Segment SGI346408[] = {{&S33439[12], 14}, {NULL}};
const Segment SGI346409[] = {{&S33440[6], 4}, {NULL}};
const Segment SGI346410[] = {{&S33440[81], 5}, {NULL}};
const Segment SGI346411[] = {{&S33440[12], 14}, {&S33440[40], 12}, {NULL}};
const Segment SGI346412[] = {{&S33440[12], 14}, {NULL}};
const Segment SGI346413[] = {{&S33441[6], 4}, {NULL}};
const Segment SGI346414[] = {{&S33441[80], 5}, {NULL}};
const Segment SGI346415[] = {{&S33441[12], 14}, {&S33441[40], 11}, {NULL}};
const Segment SGI346416[] = {{&S33441[12], 14}, {NULL}};
const Segment SGI346417[] = {{&S33442[6], 4}, {NULL}};
const Segment SGI346418[] = {{&S33442[81], 5}, {NULL}};
const Segment SGI346419[] = {{&S33442[12], 14}, {&S33442[41], 11}, {NULL}};
const Segment SGI346420[] = {{&S33442[12], 14}, {NULL}};
const Segment SGI346421[] = {{&S33443[6], 4}, {NULL}};
const Segment SGI346422[] = {{&S33443[81], 5}, {NULL}};
const Segment SGI346423[] = {{&S33443[12], 14}, {&S33443[40], 12}, {NULL}};
const Segment SGI346424[] = {{&S33443[12], 14}, {NULL}};
const Segment SGI346425[] = {{&S33444[6], 4}, {NULL}};
const Segment SGI346426[] = {{&S33444[80], 5}, {NULL}};
const Segment SGI346427[] = {{&S33444[12], 14}, {&S33444[40], 11}, {NULL}};
const Segment SGI346428[] = {{&S33444[12], 14}, {NULL}};
const Segment SGI346429[] = {{&S33445[6], 4}, {NULL}};
const Segment SGI346430[] = {{&S33445[80], 5}, {NULL}};
const Segment SGI346431[] = {{&S33445[12], 14}, {&S33445[40], 11}, {NULL}};
const Segment SGI346432[] = {{&S33445[12], 14}, {NULL}};
const Segment SGI346433[] = {{&S33446[6], 4}, {NULL}};
const Segment SGI346434[] = {{&S33446[81], 5}, {NULL}};
const Segment SGI346435[] = {{&S33446[12], 14}, {&S33446[41], 11}, {NULL}};
const Segment SGI346436[] = {{&S33446[12], 14}, {NULL}};
const Segment SGI346437[] = {{&S33448[6], 4}, {NULL}};
const Segment SGI346438[] = {{&S33448[81], 5}, {NULL}};
const Segment SGI346439[] = {{&S33448[12], 14}, {&S33448[40], 12}, {NULL}};
const Segment SGI346440[] = {{&S33448[12], 14}, {NULL}};
const Segment SGI346441[] = {{&S33449[6], 4}, {NULL}};
const Segment SGI346442[] = {{&S33449[80], 5}, {NULL}};
const Segment SGI346443[] = {{&S33449[12], 14}, {&S33449[40], 11}, {NULL}};
const Segment SGI346444[] = {{&S33449[12], 14}, {NULL}};
const Segment SGI346445[] = {{&S33450[6], 4}, {NULL}};
const Segment SGI346446[] = {{&S33450[80], 5}, {NULL}};
const Segment SGI346447[] = {{&S33450[12], 14}, {&S33450[40], 11}, {NULL}};
const Segment SGI346448[] = {{&S33450[12], 14}, {NULL}};
const Segment SGI346449[] = {{&S33451[6], 4}, {NULL}};
const Segment SGI346450[] = {{&S33451[81], 5}, {NULL}};
const Segment SGI346451[] = {{&S33451[12], 14}, {&S33451[41], 11}, {NULL}};
const Segment SGI346452[] = {{&S33451[12], 14}, {NULL}};
const Segment SGI346453[] = {{&S33452[6], 4}, {NULL}};
const Segment SGI346454[] = {{&S33452[81], 5}, {NULL}};
const Segment SGI346455[] = {{&S33452[12], 14}, {&S33452[40], 12}, {NULL}};
const Segment SGI346456[] = {{&S33452[12], 14}, {NULL}};
const Segment SGI346457[] = {{&S33453[6], 4}, {NULL}};
const Segment SGI346458[] = {{&S33453[81], 5}, {NULL}};
const Segment SGI346459[] = {{&S33453[12], 14}, {&S33453[41], 11}, {NULL}};
const Segment SGI346460[] = {{&S33453[12], 14}, {NULL}};
const Segment SGI346461[] = {{&S33454[6], 4}, {NULL}};
const Segment SGI346462[] = {{&S33454[81], 5}, {NULL}};
const Segment SGI346463[] = {{&S33454[12], 14}, {&S33454[40], 12}, {NULL}};
const Segment SGI346464[] = {{&S33454[12], 14}, {NULL}};
const Segment SGI346465[] = {{&S33455[6], 4}, {NULL}};
const Segment SGI346466[] = {{&S33455[81], 5}, {NULL}};
const Segment SGI346467[] = {{&S33455[12], 14}, {&S33455[40], 12}, {NULL}};
const Segment SGI346468[] = {{&S33455[12], 14}, {NULL}};
const Segment SGI346469[] = {{&S33456[6], 4}, {NULL}};
const Segment SGI346470[] = {{&S33456[80], 5}, {NULL}};
const Segment SGI346471[] = {{&S33456[12], 14}, {&S33456[40], 11}, {NULL}};
const Segment SGI346472[] = {{&S33456[12], 14}, {NULL}};
const Segment SGI346473[] = {{&S33457[6], 4}, {NULL}};
const Segment SGI346474[] = {{&S33457[81], 5}, {NULL}};
const Segment SGI346475[] = {{&S33457[12], 14}, {&S33457[40], 12}, {NULL}};
const Segment SGI346476[] = {{&S33457[12], 14}, {NULL}};
const Segment SGI346477[] = {{&S33458[6], 4}, {NULL}};
const Segment SGI346478[] = {{&S33458[80], 5}, {NULL}};
const Segment SGI346479[] = {{&S33458[12], 14}, {&S33458[40], 11}, {NULL}};
const Segment SGI346480[] = {{&S33458[12], 14}, {NULL}};
const Segment SGI346481[] = {{&S33459[6], 4}, {NULL}};
const Segment SGI346482[] = {{&S33459[80], 5}, {NULL}};
const Segment SGI346483[] = {{&S33459[12], 14}, {&S33459[40], 11}, {NULL}};
const Segment SGI346484[] = {{&S33459[12], 14}, {NULL}};
const Segment SGI346485[] = {{&S33460[6], 4}, {NULL}};
const Segment SGI346486[] = {{&S33460[81], 5}, {NULL}};
const Segment SGI346487[] = {{&S33460[12], 14}, {&S33460[41], 11}, {NULL}};
const Segment SGI346488[] = {{&S33460[12], 14}, {NULL}};
const Segment SGI346489[] = {{&S33461[6], 4}, {NULL}};
const Segment SGI346490[] = {{&S33461[80], 5}, {NULL}};
const Segment SGI346491[] = {{&S33461[12], 14}, {&S33461[40], 11}, {NULL}};
const Segment SGI346492[] = {{&S33461[12], 14}, {NULL}};
const Segment SGI346493[] = {{&S33462[6], 4}, {NULL}};
const Segment SGI346494[] = {{&S33462[80], 5}, {NULL}};
const Segment SGI346495[] = {{&S33462[12], 14}, {&S33462[40], 11}, {NULL}};
const Segment SGI346496[] = {{&S60007[5], 3}, {&S60007[5], 3}, {NULL}};
const Segment SGI346497[] = {{&S33462[12], 14}, {NULL}};
const Segment SGI346498[] = {{&S60007[149], 3}, {NULL}};
const Segment SGI346499[] = {{&S33463[6], 4}, {NULL}};
const Segment SGI346500[] = {{&S33463[80], 5}, {NULL}};
const Segment SGI346501[] = {{&S33463[12], 14}, {&S33463[40], 11}, {NULL}};
const Segment SGI346502[] = {{&S33463[12], 14}, {NULL}};
const Segment SGI346503[] = {{&S33464[6], 4}, {NULL}};
const Segment SGI346504[] = {{&S33464[80], 5}, {NULL}};
const Segment SGI346505[] = {{&S33464[12], 14}, {&S33464[40], 11}, {NULL}};
const Segment SGI346506[] = {{&S33464[12], 14}, {NULL}};
const Segment SGI346507[] = {{&S33465[6], 4}, {NULL}};
const Segment SGI346508[] = {{&S33465[80], 5}, {NULL}};
const Segment SGI346509[] = {{&S33465[12], 14}, {&S33465[40], 11}, {NULL}};
const Segment SGI346510[] = {{&S33465[12], 14}, {NULL}};
const Segment SGI346511[] = {{&S33466[6], 4}, {NULL}};
const Segment SGI346512[] = {{&S33466[53], 5}, {NULL}};
const Segment SGI346513[] = {{&S33466[12], 9}, {NULL}};
const Segment SGI346514[] = {{&S33467[6], 4}, {NULL}};
const Segment SGI346515[] = {{&S33467[81], 5}, {NULL}};
const Segment SGI346516[] = {{&S33467[12], 14}, {&S33467[40], 12}, {NULL}};
const Segment SGI346517[] = {{&S33467[12], 14}, {NULL}};
const Segment SGI346518[] = {{&S33469[6], 4}, {NULL}};
const Segment SGI346519[] = {{&S33469[81], 5}, {NULL}};
const Segment SGI346520[] = {{&S33469[12], 14}, {&S33469[41], 11}, {NULL}};
const Segment SGI346521[] = {{&S33469[12], 14}, {NULL}};
const Segment SGI346522[] = {{&S60015[5], 3}, {NULL}};
const Segment SGI346523[] = {{&S33470[6], 4}, {NULL}};
const Segment SGI346524[] = {{&S33470[81], 5}, {NULL}};
const Segment SGI346525[] = {{&S33470[12], 14}, {&S33470[41], 11}, {NULL}};
const Segment SGI346526[] = {{&S33470[12], 14}, {NULL}};
const Segment SGI346527[] = {{&S60016[5], 3}, {&S60016[5], 3}, {NULL}};
const Segment SGI346528[] = {{&S33471[6], 4}, {NULL}};
const Segment SGI346529[] = {{&S33471[81], 5}, {NULL}};
const Segment SGI346530[] = {{&S33471[12], 14}, {&S33471[41], 11}, {NULL}};
const Segment SGI346531[] = {{&S33471[12], 14}, {NULL}};
const Segment SGI346532[] = {{&S33472[6], 4}, {NULL}};
const Segment SGI346533[] = {{&S33472[58], 5}, {NULL}};
const Segment SGI346534[] = {{&S33472[12], 14}, {&S33472[69], 4}, {NULL}};
const Segment SGI346535[] = {{&S33472[12], 14}, {NULL}};
const Segment SGI346536[] = {{&S33473[6], 4}, {NULL}};
const Segment SGI346537[] = {{&S33473[81], 5}, {NULL}};
const Segment SGI346538[] = {{&S33473[12], 14}, {&S33473[41], 11}, {NULL}};
const Segment SGI346539[] = {{&S33473[12], 14}, {NULL}};
const Segment SGI346540[] = {{&S33474[6], 4}, {NULL}};
const Segment SGI346541[] = {{&S33474[80], 5}, {NULL}};
const Segment SGI346542[] = {{&S33474[12], 14}, {&S33474[40], 11}, {NULL}};
const Segment SGI346543[] = {{&S33474[12], 14}, {NULL}};
const Segment SGI346544[] = {{&S33475[6], 4}, {NULL}};
const Segment SGI346545[] = {{&S33475[80], 5}, {NULL}};
const Segment SGI346546[] = {{&S33475[12], 14}, {&S33475[40], 11}, {NULL}};
const Segment SGI346547[] = {{&S33475[12], 14}, {NULL}};
const Segment SGI346548[] = {{&S33476[6], 4}, {NULL}};
const Segment SGI346549[] = {{&S33476[80], 5}, {NULL}};
const Segment SGI346550[] = {{&S33476[12], 14}, {&S33476[40], 11}, {NULL}};
const Segment SGI346551[] = {{&S33476[12], 14}, {NULL}};
const Segment SGI346552[] = {{&S33477[6], 4}, {NULL}};
const Segment SGI346553[] = {{&S33477[80], 5}, {NULL}};
const Segment SGI346554[] = {{&S33477[12], 14}, {&S33477[40], 11}, {NULL}};
const Segment SGI346555[] = {{&S33477[12], 14}, {NULL}};
const Segment SGI346556[] = {{&S33478[6], 4}, {NULL}};
const Segment SGI346557[] = {{&S33478[80], 5}, {NULL}};
const Segment SGI346558[] = {{&S33478[12], 14}, {&S33478[40], 11}, {NULL}};
const Segment SGI346559[] = {{&S33478[12], 14}, {NULL}};
const Segment SGI346560[] = {{&S33479[6], 4}, {NULL}};
const Segment SGI346561[] = {{&S33479[80], 5}, {NULL}};
const Segment SGI346562[] = {{&S33479[12], 14}, {&S33479[40], 11}, {NULL}};
const Segment SGI346563[] = {{&S33479[12], 14}, {NULL}};
const Segment SGI346564[] = {{&S33480[6], 4}, {NULL}};
const Segment SGI346565[] = {{&S33480[80], 5}, {NULL}};
const Segment SGI346566[] = {{&S33480[12], 14}, {&S33480[40], 11}, {NULL}};
const Segment SGI346567[] = {{&S33480[12], 14}, {NULL}};
const Segment SGI346568[] = {{&S33481[6], 4}, {NULL}};
const Segment SGI346569[] = {{&S33481[80], 5}, {NULL}};
const Segment SGI346570[] = {{&S33481[12], 14}, {&S33481[40], 11}, {NULL}};
const Segment SGI346571[] = {{&S33481[12], 14}, {NULL}};
const Segment SGI346572[] = {{&S33482[6], 4}, {NULL}};
const Segment SGI346573[] = {{&S33482[80], 5}, {NULL}};
const Segment SGI346574[] = {{&S33482[12], 14}, {&S33482[40], 11}, {NULL}};
const Segment SGI346575[] = {{&S33482[12], 14}, {NULL}};
const Segment SGI346576[] = {{&S33483[6], 4}, {NULL}};
const Segment SGI346577[] = {{&S33483[80], 5}, {NULL}};
const Segment SGI346578[] = {{&S33483[12], 14}, {&S33483[40], 11}, {NULL}};
const Segment SGI346579[] = {{&S33483[12], 14}, {NULL}};
const Segment SGI346580[] = {{&S60060[5], 5}, {NULL}};
const Segment SGI346581[] = {{&S60061[5], 5}, {&S60061[68], 5}, {NULL}};
const Segment SGI346582[] = {{&S33522[6], 4}, {NULL}};
const Segment SGI346583[] = {{&S33522[81], 5}, {NULL}};
const Segment SGI346584[] = {{&S33522[12], 14}, {&S33522[40], 12}, {NULL}};
const Segment SGI346585[] = {{&S33522[12], 14}, {NULL}};
const Segment SGI346586[] = {{&S33523[6], 4}, {NULL}};
const Segment SGI346587[] = {{&S33523[58], 5}, {NULL}};
const Segment SGI346588[] = {{&S33523[12], 14}, {&S33523[69], 31}, {NULL}};
const Segment SGI346589[] = {{&S33523[12], 14}, {NULL}};
const Segment SGI346590[] = {{&S33524[6], 4}, {NULL}};
const Segment SGI346591[] = {{&S33524[58], 5}, {NULL}};
const Segment SGI346592[] = {{&S33524[12], 14}, {NULL}};
const Segment SGI346593[] = {{&S33525[6], 4}, {NULL}};
const Segment SGI346594[] = {{&S33525[81], 5}, {NULL}};
const Segment SGI346595[] = {{&S33525[12], 14}, {&S33525[40], 12}, {NULL}};
const Segment SGI346596[] = {{&S33525[12], 14}, {NULL}};
const Segment SGI346597[] = {{&S60079[42], 3}, {NULL}};
const Segment SGI346598[] = {{&S33526[6], 4}, {NULL}};
const Segment SGI346599[] = {{&S33526[58], 5}, {NULL}};
const Segment SGI346600[] = {{&S60079[6], 3}, {NULL}};
const Segment SGI346601[] = {{&S33526[12], 14}, {NULL}};
const Segment SGI346602[] = {{&S60079[66], 3}, {NULL}};
const Segment SGI346603[] = {{&S60080[42], 7}, {NULL}};
const Segment SGI346604[] = {{&S33527[6], 4}, {NULL}};
const Segment SGI346605[] = {{&S60080[6], 3}, {NULL}};
const Segment SGI346606[] = {{&S33527[80], 5}, {NULL}};
const Segment SGI346607[] = {{&S33527[12], 14}, {&S33527[40], 11}, {NULL}};
const Segment SGI346608[] = {{&S60080[70], 3}, {NULL}};
const Segment SGI346609[] = {{&S33527[12], 14}, {NULL}};
const Segment SGI346610[] = {{&S33528[6], 4}, {NULL}};
const Segment SGI346611[] = {{&S33528[80], 5}, {NULL}};
const Segment SGI346612[] = {{&S33528[12], 14}, {&S33528[40], 11}, {NULL}};
const Segment SGI346613[] = {{&S33528[12], 14}, {NULL}};
const Segment SGI346614[] = {{&S33529[6], 4}, {NULL}};
const Segment SGI346615[] = {{&S33529[80], 5}, {NULL}};
const Segment SGI346616[] = {{&S33529[12], 14}, {&S33529[40], 11}, {NULL}};
const Segment SGI346617[] = {{&S33529[12], 14}, {NULL}};
const Segment SGI346618[] = {{&S33530[6], 4}, {NULL}};
const Segment SGI346619[] = {{&S33530[58], 5}, {NULL}};
const Segment SGI346620[] = {{&S33530[12], 14}, {NULL}};
const Segment SGI346621[] = {{&S33531[6], 4}, {NULL}};
const Segment SGI346622[] = {{&S33531[81], 5}, {NULL}};
const Segment SGI346623[] = {{&S33531[12], 14}, {&S33531[40], 12}, {NULL}};
const Segment SGI346624[] = {{&S33531[12], 14}, {NULL}};
const Segment SGI346625[] = {{&S33532[6], 4}, {NULL}};
const Segment SGI346626[] = {{&S33532[81], 5}, {NULL}};
const Segment SGI346627[] = {{&S33532[12], 14}, {&S33532[40], 12}, {NULL}};
const Segment SGI346628[] = {{&S33532[12], 14}, {NULL}};
const Segment SGI346629[] = {{&S33533[6], 4}, {NULL}};
const Segment SGI346630[] = {{&S33533[80], 5}, {NULL}};
const Segment SGI346631[] = {{&S33533[12], 14}, {&S33533[40], 11}, {NULL}};
const Segment SGI346632[] = {{&S33533[12], 14}, {NULL}};
const Segment SGI346633[] = {{&S33534[6], 4}, {NULL}};
const Segment SGI346634[] = {{&S33534[81], 5}, {NULL}};
const Segment SGI346635[] = {{&S33534[12], 14}, {&S33534[40], 12}, {NULL}};
const Segment SGI346636[] = {{&S33534[12], 14}, {NULL}};
const Segment SGI346637[] = {{&S33535[6], 4}, {NULL}};
const Segment SGI346638[] = {{&S33535[81], 5}, {NULL}};
const Segment SGI346639[] = {{&S33535[12], 14}, {&S33535[40], 12}, {NULL}};
const Segment SGI346640[] = {{&S33535[12], 14}, {NULL}};
const Segment SGI346641[] = {{&S33536[6], 4}, {NULL}};
const Segment SGI346642[] = {{&S33536[81], 5}, {NULL}};
const Segment SGI346643[] = {{&S33536[12], 14}, {&S33536[40], 12}, {NULL}};
const Segment SGI346644[] = {{&S33536[12], 14}, {NULL}};
const Segment SGI346645[] = {{&S33537[6], 4}, {NULL}};
const Segment SGI346646[] = {{&S33537[81], 5}, {NULL}};
const Segment SGI346647[] = {{&S33537[12], 14}, {&S33537[40], 12}, {NULL}};
const Segment SGI346648[] = {{&S33537[12], 14}, {NULL}};
const Segment SGI346649[] = {{&S33538[6], 4}, {NULL}};
const Segment SGI346650[] = {{&S33538[81], 5}, {NULL}};
const Segment SGI346651[] = {{&S33538[12], 14}, {&S33538[40], 12}, {NULL}};
const Segment SGI346652[] = {{&S33538[12], 14}, {NULL}};
const Segment SGI346653[] = {{&S33539[6], 4}, {NULL}};
const Segment SGI346654[] = {{&S33539[81], 5}, {NULL}};
const Segment SGI346655[] = {{&S33539[12], 14}, {&S33539[40], 12}, {NULL}};
const Segment SGI346656[] = {{&S33539[12], 14}, {NULL}};
const Segment SGI346657[] = {{&S33540[6], 4}, {NULL}};
const Segment SGI346658[] = {{&S33540[81], 5}, {NULL}};
const Segment SGI346659[] = {{&S33540[12], 14}, {&S33540[40], 12}, {NULL}};
const Segment SGI346660[] = {{&S33540[12], 14}, {NULL}};
const Segment SGI346661[] = {{&S33541[6], 4}, {NULL}};
const Segment SGI346662[] = {{&S33541[81], 5}, {NULL}};
const Segment SGI346663[] = {{&S33541[12], 14}, {&S33541[40], 12}, {NULL}};
const Segment SGI346664[] = {{&S33541[12], 14}, {NULL}};
const Segment SGI346665[] = {{&S33542[6], 4}, {NULL}};
const Segment SGI346666[] = {{&S33542[80], 5}, {NULL}};
const Segment SGI346667[] = {{&S33542[12], 14}, {&S33542[40], 11}, {NULL}};
const Segment SGI346668[] = {{&S33542[12], 14}, {NULL}};
const Segment SGI346669[] = {{&S51554[28], 3}, {NULL}};
const Segment SGI346670[] = {{&S51554[102], 5}, {NULL}};
const Segment SGI346671[] = {{&S33543[6], 4}, {NULL}};
const Segment SGI346672[] = {{&S33543[81], 5}, {NULL}};
const Segment SGI346673[] = {{&S33543[12], 14}, {&S33543[40], 12}, {NULL}};
const Segment SGI346674[] = {{&S33543[12], 14}, {NULL}};
const Segment SGI346675[] = {{&S33544[6], 4}, {NULL}};
const Segment SGI346676[] = {{&S33544[80], 5}, {NULL}};
const Segment SGI346677[] = {{&S33544[12], 14}, {&S33544[40], 11}, {NULL}};
const Segment SGI346678[] = {{&S33544[12], 14}, {NULL}};
const Segment SGI346679[] = {{&S33545[6], 4}, {NULL}};
const Segment SGI346680[] = {{&S33545[80], 5}, {NULL}};
const Segment SGI346681[] = {{&S33545[12], 14}, {&S33545[40], 11}, {NULL}};
const Segment SGI346682[] = {{&S33545[12], 14}, {NULL}};
const Segment SGI346683[] = {{&S33546[6], 4}, {NULL}};
const Segment SGI346684[] = {{&S33546[81], 5}, {NULL}};
const Segment SGI346685[] = {{&S33546[12], 14}, {&S33546[40], 12}, {NULL}};
const Segment SGI346686[] = {{&S33546[12], 14}, {NULL}};
const Segment SGI346687[] = {{&S33547[6], 4}, {NULL}};
const Segment SGI346688[] = {{&S33547[80], 5}, {NULL}};
const Segment SGI346689[] = {{&S33547[12], 14}, {&S33547[40], 11}, {NULL}};
const Segment SGI346690[] = {{&S33547[12], 14}, {NULL}};
const Segment SGI346691[] = {{&S33548[6], 4}, {NULL}};
const Segment SGI346692[] = {{&S33548[58], 5}, {NULL}};
const Segment SGI346693[] = {{&S33548[12], 14}, {NULL}};
const Segment SGI346694[] = {{&S60108[5], 5}, {&S60108[97], 5}, {NULL}};
const Segment SGI346695[] = {{&S33549[6], 4}, {NULL}};
const Segment SGI346696[] = {{&S33549[81], 5}, {NULL}};
const Segment SGI346697[] = {{&S33549[12], 14}, {&S33549[40], 12}, {NULL}};
const Segment SGI346698[] = {{&S33549[12], 14}, {NULL}};
const Segment SGI346699[] = {{&S60108[113], 3}, {NULL}};
const Segment SGI346700[] = {{&S33550[6], 4}, {NULL}};
const Segment SGI346701[] = {{&S33550[58], 5}, {NULL}};
const Segment SGI346702[] = {{&S33550[12], 14}, {&S33550[69], 31}, {NULL}};
const Segment SGI346703[] = {{&S33550[12], 14}, {NULL}};
const Segment SGI346704[] = {{&S33551[6], 4}, {NULL}};
const Segment SGI346705[] = {{&S33551[81], 5}, {NULL}};
const Segment SGI346706[] = {{&S33551[12], 14}, {&S33551[40], 12}, {NULL}};
const Segment SGI346707[] = {{&S33551[12], 14}, {NULL}};
const Segment SGI346708[] = {{&S33552[6], 4}, {NULL}};
const Segment SGI346709[] = {{&S33552[80], 5}, {NULL}};
const Segment SGI346710[] = {{&S33552[12], 14}, {&S33552[40], 11}, {NULL}};
const Segment SGI346711[] = {{&S33552[12], 14}, {NULL}};
const Segment SGI346712[] = {{&S33553[6], 4}, {NULL}};
const Segment SGI346713[] = {{&S33553[80], 5}, {NULL}};
const Segment SGI346714[] = {{&S33553[12], 14}, {&S33553[40], 11}, {NULL}};
const Segment SGI346715[] = {{&S33553[12], 14}, {NULL}};
const Segment SGI346716[] = {{&S33554[6], 4}, {NULL}};
const Segment SGI346717[] = {{&S33554[80], 5}, {NULL}};
const Segment SGI346718[] = {{&S33554[12], 14}, {&S33554[40], 11}, {NULL}};
const Segment SGI346719[] = {{&S33554[12], 14}, {NULL}};
const Segment SGI346720[] = {{&S33555[6], 4}, {NULL}};
const Segment SGI346721[] = {{&S33555[80], 5}, {NULL}};
const Segment SGI346722[] = {{&S33555[12], 14}, {&S33555[40], 11}, {NULL}};
const Segment SGI346723[] = {{&S33555[12], 14}, {NULL}};
const Segment SGI346724[] = {{&S33556[6], 4}, {NULL}};
const Segment SGI346725[] = {{&S33556[12], 22}, {NULL}};
const Segment SGI346726[] = {{&S33556[12], 14}, {NULL}};
const Segment SGI346727[] = {{&S33557[6], 4}, {NULL}};
const Segment SGI346728[] = {{&S33557[81], 5}, {NULL}};
const Segment SGI346729[] = {{&S33557[12], 14}, {&S33557[40], 12}, {NULL}};
const Segment SGI346730[] = {{&S33557[12], 14}, {NULL}};
const Segment SGI346731[] = {{&S33558[6], 4}, {NULL}};
const Segment SGI346732[] = {{&S33558[80], 5}, {NULL}};
const Segment SGI346733[] = {{&S33558[12], 14}, {&S33558[40], 11}, {NULL}};
const Segment SGI346734[] = {{&S33558[12], 14}, {NULL}};
const Segment SGI346735[] = {{&S33559[6], 4}, {NULL}};
const Segment SGI346736[] = {{&S33559[81], 5}, {NULL}};
const Segment SGI346737[] = {{&S33559[12], 14}, {&S33559[40], 12}, {NULL}};
const Segment SGI346738[] = {{&S33559[12], 14}, {NULL}};
const Segment SGI346739[] = {{&S33560[6], 4}, {NULL}};
const Segment SGI346740[] = {{&S33560[81], 5}, {NULL}};
const Segment SGI346741[] = {{&S33560[12], 14}, {&S33560[40], 12}, {NULL}};
const Segment SGI346742[] = {{&S33560[12], 14}, {NULL}};
const Segment SGI346743[] = {{&S33561[6], 4}, {NULL}};
const Segment SGI346744[] = {{&S33561[81], 5}, {NULL}};
const Segment SGI346745[] = {{&S33561[12], 14}, {&S33561[40], 12}, {NULL}};
const Segment SGI346746[] = {{&S33561[12], 14}, {NULL}};
const Segment SGI346747[] = {{&S33562[6], 4}, {NULL}};
const Segment SGI346748[] = {{&S33562[81], 5}, {NULL}};
const Segment SGI346749[] = {{&S33562[12], 14}, {&S33562[41], 11}, {NULL}};
const Segment SGI346750[] = {{&S33562[12], 14}, {NULL}};
const Segment SGI346751[] = {{&S33563[6], 4}, {NULL}};
const Segment SGI346752[] = {{&S33563[81], 5}, {NULL}};
const Segment SGI346753[] = {{&S33563[12], 14}, {&S33563[41], 11}, {NULL}};
const Segment SGI346754[] = {{&S33563[12], 14}, {NULL}};
const Segment SGI346755[] = {{&S33564[6], 4}, {NULL}};
const Segment SGI346756[] = {{&S33564[81], 5}, {NULL}};
const Segment SGI346757[] = {{&S33564[12], 14}, {&S33564[41], 11}, {NULL}};
const Segment SGI346758[] = {{&S33564[12], 14}, {NULL}};
const Segment SGI346759[] = {{&S33565[6], 4}, {NULL}};
const Segment SGI346760[] = {{&S33565[80], 5}, {NULL}};
const Segment SGI346761[] = {{&S33565[12], 14}, {&S33565[40], 11}, {NULL}};
const Segment SGI346762[] = {{&S33565[12], 14}, {NULL}};
const Segment SGI346763[] = {{&S35660[57], 5}, {NULL}};
const Segment SGI346764[] = {{&S35660[13], 10}, {NULL}};
const Segment SGI346765[] = {{&S33566[6], 4}, {NULL}};
const Segment SGI346766[] = {{&S33566[80], 5}, {NULL}};
const Segment SGI346767[] = {{&S33566[12], 14}, {&S33566[40], 11}, {NULL}};
const Segment SGI346768[] = {{&S33566[12], 14}, {NULL}};
const Segment SGI346769[] = {{&S33567[6], 4}, {NULL}};
const Segment SGI346770[] = {{&S33567[80], 5}, {NULL}};
const Segment SGI346771[] = {{&S33567[12], 14}, {&S33567[40], 11}, {NULL}};
const Segment SGI346772[] = {{&S33567[12], 14}, {NULL}};
const Segment SGI346773[] = {{&S33568[6], 4}, {NULL}};
const Segment SGI346774[] = {{&S33568[80], 5}, {NULL}};
const Segment SGI346775[] = {{&S33568[12], 14}, {&S33568[40], 11}, {NULL}};
const Segment SGI346776[] = {{&S60132[5], 3}, {&S60132[5], 3}, {NULL}};
const Segment SGI346777[] = {{&S33568[12], 14}, {NULL}};
const Segment SGI346778[] = {{&S60132[9], 3}, {NULL}};
const Segment SGI346779[] = {{&S60132[127], 3}, {NULL}};
const Segment SGI346780[] = {{&S33569[6], 4}, {NULL}};
const Segment SGI346781[] = {{&S33569[80], 5}, {NULL}};
const Segment SGI346782[] = {{&S33569[12], 14}, {&S33569[40], 11}, {NULL}};
const Segment SGI346783[] = {{&S33569[12], 14}, {NULL}};
const Segment SGI346784[] = {{&S33570[6], 4}, {NULL}};
const Segment SGI346785[] = {{&S33570[80], 5}, {NULL}};
const Segment SGI346786[] = {{&S33570[12], 14}, {&S33570[40], 11}, {NULL}};
const Segment SGI346787[] = {{&S33570[12], 14}, {NULL}};
const Segment SGI346788[] = {{&S33571[6], 4}, {NULL}};
const Segment SGI346789[] = {{&S33571[80], 5}, {NULL}};
const Segment SGI346790[] = {{&S33571[12], 14}, {&S33571[40], 11}, {NULL}};
const Segment SGI346791[] = {{&S33571[12], 14}, {NULL}};
const Segment SGI346792[] = {{&S33572[6], 4}, {NULL}};
const Segment SGI346793[] = {{&S33572[81], 5}, {NULL}};
const Segment SGI346794[] = {{&S33572[12], 14}, {&S33572[40], 12}, {NULL}};
const Segment SGI346795[] = {{&S33572[12], 14}, {NULL}};
const Segment SGI346796[] = {{&S33573[6], 4}, {NULL}};
const Segment SGI346797[] = {{&S33573[81], 5}, {NULL}};
const Segment SGI346798[] = {{&S33573[12], 14}, {&S33573[40], 12}, {NULL}};
const Segment SGI346799[] = {{&S33573[12], 14}, {NULL}};
const Segment SGI346800[] = {{&S33574[6], 4}, {NULL}};
const Segment SGI346801[] = {{&S33574[81], 5}, {NULL}};
const Segment SGI346802[] = {{&S33574[12], 14}, {&S33574[41], 11}, {NULL}};
const Segment SGI346803[] = {{&S33574[12], 14}, {NULL}};
const Segment SGI346804[] = {{&S33575[6], 4}, {NULL}};
const Segment SGI346805[] = {{&S33575[81], 5}, {NULL}};
const Segment SGI346806[] = {{&S33575[12], 14}, {&S33575[40], 12}, {NULL}};
const Segment SGI346807[] = {{&S33575[12], 14}, {NULL}};
const Segment SGI346808[] = {{&S33576[6], 4}, {NULL}};
const Segment SGI346809[] = {{&S33576[81], 5}, {NULL}};
const Segment SGI346810[] = {{&S33576[12], 14}, {&S33576[40], 12}, {NULL}};
const Segment SGI346811[] = {{&S33576[12], 14}, {NULL}};
const Segment SGI346812[] = {{&S33577[6], 4}, {NULL}};
const Segment SGI346813[] = {{&S33577[81], 5}, {NULL}};
const Segment SGI346814[] = {{&S33577[12], 14}, {&S33577[40], 12}, {NULL}};
const Segment SGI346815[] = {{&S33577[12], 14}, {NULL}};
const Segment SGI346816[] = {{&S33578[6], 4}, {NULL}};
const Segment SGI346817[] = {{&S33578[80], 5}, {NULL}};
const Segment SGI346818[] = {{&S33578[12], 14}, {&S33578[40], 11}, {NULL}};
const Segment SGI346819[] = {{&S33578[12], 14}, {NULL}};
const Segment SGI346820[] = {{&S33579[6], 4}, {NULL}};
const Segment SGI346821[] = {{&S33579[81], 5}, {NULL}};
const Segment SGI346822[] = {{&S33579[12], 14}, {&S33579[41], 11}, {NULL}};
const Segment SGI346823[] = {{&S33579[12], 14}, {NULL}};
const Segment SGI346824[] = {{&S46524[44], 3}, {NULL}};
const Segment SGI346825[] = {{&S33580[6], 4}, {NULL}};
const Segment SGI346826[] = {{&S46524[35], 3}, {NULL}};
const Segment SGI346827[] = {{&S33580[81], 5}, {NULL}};
const Segment SGI346828[] = {{&S33580[12], 14}, {&S33580[41], 11}, {NULL}};
const Segment SGI346829[] = {{&S33580[12], 14}, {NULL}};
const Segment SGI346830[] = {{&S46525[5], 5}, {NULL}};
const Segment SGI346831[] = {{&S46525[11], 3}, {NULL}};
const Segment SGI346832[] = {{&S33581[6], 4}, {NULL}};
const Segment SGI346833[] = {{&S33581[81], 5}, {NULL}};
const Segment SGI346834[] = {{&S33581[12], 14}, {&S33581[41], 11}, {NULL}};
const Segment SGI346835[] = {{&S33581[12], 14}, {NULL}};
const Segment SGI346836[] = {{&S46526[5], 5}, {NULL}};
const Segment SGI346837[] = {{&S46526[11], 3}, {NULL}};
const Segment SGI346838[] = {{&S33582[6], 4}, {NULL}};
const Segment SGI346839[] = {{&S33582[80], 5}, {NULL}};
const Segment SGI346840[] = {{&S33582[12], 14}, {&S33582[40], 11}, {NULL}};
const Segment SGI346841[] = {{&S33582[12], 14}, {NULL}};
const Segment SGI346842[] = {{&S46527[5], 5}, {NULL}};
const Segment SGI346843[] = {{&S46527[33], 3}, {&S46527[33], 3}, {NULL}};
const Segment SGI346844[] = {{&S33583[6], 4}, {NULL}};
const Segment SGI346845[] = {{&S33583[80], 5}, {NULL}};
const Segment SGI346846[] = {{&S33583[12], 14}, {&S33583[40], 11}, {NULL}};
const Segment SGI346847[] = {{&S33583[12], 14}, {NULL}};
const Segment SGI346848[] = {{&S46528[5], 5}, {NULL}};
const Segment SGI346849[] = {{&S46528[34], 3}, {NULL}};
const Segment SGI346850[] = {{&S33584[6], 4}, {NULL}};
const Segment SGI346851[] = {{&S33584[80], 5}, {NULL}};
const Segment SGI346852[] = {{&S33584[12], 14}, {&S33584[40], 11}, {NULL}};
const Segment SGI346853[] = {{&S33584[12], 14}, {NULL}};
const Segment SGI346854[] = {{&S33585[6], 4}, {NULL}};
const Segment SGI346855[] = {{&S33585[80], 5}, {NULL}};
const Segment SGI346856[] = {{&S33585[12], 14}, {&S33585[40], 11}, {NULL}};
const Segment SGI346857[] = {{&S33585[12], 14}, {NULL}};
const Segment SGI346858[] = {{&S33586[6], 4}, {NULL}};
const Segment SGI346859[] = {{&S33586[80], 5}, {NULL}};
const Segment SGI346860[] = {{&S33586[12], 14}, {&S33586[40], 11}, {NULL}};
const Segment SGI346861[] = {{&S33586[12], 14}, {NULL}};
const Segment SGI346862[] = {{&S33587[6], 4}, {NULL}};
const Segment SGI346863[] = {{&S33587[80], 5}, {NULL}};
const Segment SGI346864[] = {{&S33587[12], 14}, {&S33587[40], 11}, {NULL}};
const Segment SGI346865[] = {{&S33587[12], 14}, {NULL}};
const Segment SGI346866[] = {{&S33588[6], 4}, {NULL}};
const Segment SGI346867[] = {{&S33588[81], 5}, {NULL}};
const Segment SGI346868[] = {{&S33588[12], 14}, {&S33588[40], 12}, {NULL}};
const Segment SGI346869[] = {{&S33588[12], 14}, {NULL}};
const Segment SGI346870[] = {{&S33589[6], 4}, {NULL}};
const Segment SGI346871[] = {{&S33589[81], 5}, {NULL}};
const Segment SGI346872[] = {{&S33589[12], 14}, {&S33589[40], 12}, {NULL}};
const Segment SGI346873[] = {{&S33589[12], 14}, {NULL}};
const Segment SGI346874[] = {{&S33590[6], 4}, {NULL}};
const Segment SGI346875[] = {{&S33590[80], 5}, {NULL}};
const Segment SGI346876[] = {{&S33590[12], 14}, {&S33590[40], 11}, {NULL}};
const Segment SGI346877[] = {{&S33590[12], 14}, {NULL}};
const Segment SGI346878[] = {{&S33591[6], 4}, {NULL}};
const Segment SGI346879[] = {{&S33591[80], 5}, {NULL}};
const Segment SGI346880[] = {{&S33591[12], 14}, {&S33591[40], 11}, {NULL}};
const Segment SGI346881[] = {{&S33591[12], 14}, {NULL}};
const Segment SGI346882[] = {{&S33592[6], 4}, {NULL}};
const Segment SGI346883[] = {{&S33592[81], 5}, {NULL}};
const Segment SGI346884[] = {{&S33592[12], 14}, {&S33592[41], 11}, {NULL}};
const Segment SGI346885[] = {{&S33592[12], 14}, {NULL}};
const Segment SGI346886[] = {{&S33593[6], 4}, {NULL}};
const Segment SGI346887[] = {{&S33593[81], 5}, {NULL}};
const Segment SGI346888[] = {{&S33593[12], 14}, {&S33593[41], 11}, {NULL}};
const Segment SGI346889[] = {{&S33593[12], 14}, {NULL}};
const Segment SGI346890[] = {{&S33594[6], 4}, {NULL}};
const Segment SGI346891[] = {{&S33594[80], 5}, {NULL}};
const Segment SGI346892[] = {{&S33594[12], 14}, {&S33594[40], 11}, {NULL}};
const Segment SGI346893[] = {{&S33594[12], 14}, {NULL}};
const Segment SGI346894[] = {{&S33595[6], 4}, {NULL}};
const Segment SGI346895[] = {{&S33595[80], 5}, {NULL}};
const Segment SGI346896[] = {{&S33595[12], 14}, {&S33595[40], 11}, {NULL}};
const Segment SGI346897[] = {{&S33595[12], 14}, {NULL}};
const Segment SGI346898[] = {{&S33596[6], 4}, {NULL}};
const Segment SGI346899[] = {{&S33596[80], 5}, {NULL}};
const Segment SGI346900[] = {{&S33596[12], 14}, {&S33596[40], 11}, {NULL}};
const Segment SGI346901[] = {{&S33596[12], 14}, {NULL}};
const Segment SGI346902[] = {{&S33597[6], 4}, {NULL}};
const Segment SGI346903[] = {{&S33597[12], 14}, {NULL}};
const Segment SGI346904[] = {{&S33598[6], 4}, {NULL}};
const Segment SGI346905[] = {{&S33598[81], 5}, {NULL}};
const Segment SGI346906[] = {{&S33598[12], 14}, {&S33598[41], 11}, {NULL}};
const Segment SGI346907[] = {{&S33598[12], 14}, {NULL}};
const Segment SGI346908[] = {{&S33599[6], 4}, {NULL}};
const Segment SGI346909[] = {{&S33599[80], 5}, {NULL}};
const Segment SGI346910[] = {{&S33599[12], 14}, {&S33599[40], 11}, {NULL}};
const Segment SGI346911[] = {{&S33599[12], 14}, {NULL}};
const Segment SGI346912[] = {{&S33600[6], 4}, {NULL}};
const Segment SGI346913[] = {{&S33600[80], 5}, {NULL}};
const Segment SGI346914[] = {{&S33600[12], 14}, {&S33600[40], 11}, {NULL}};
const Segment SGI346915[] = {{&S33600[12], 14}, {NULL}};
const Segment SGI346916[] = {{&S33601[6], 4}, {NULL}};
const Segment SGI346917[] = {{&S33601[12], 17}, {NULL}};
const Segment SGI346918[] = {{&S33601[12], 14}, {NULL}};
const Segment SGI346919[] = {{&S33602[6], 4}, {NULL}};
const Segment SGI346920[] = {{&S33602[58], 5}, {NULL}};
const Segment SGI346921[] = {{&S33602[12], 14}, {&S33602[69], 4}, {NULL}};
const Segment SGI346922[] = {{&S33602[12], 14}, {NULL}};
const Segment SGI346923[] = {{&S33603[6], 4}, {NULL}};
const Segment SGI346924[] = {{&S33603[80], 5}, {NULL}};
const Segment SGI346925[] = {{&S33603[12], 14}, {&S33603[40], 11}, {NULL}};
const Segment SGI346926[] = {{&S33603[12], 14}, {NULL}};
const Segment SGI346927[] = {{&S33604[6], 4}, {NULL}};
const Segment SGI346928[] = {{&S33604[80], 5}, {NULL}};
const Segment SGI346929[] = {{&S33604[12], 14}, {&S33604[40], 11}, {NULL}};
const Segment SGI346930[] = {{&S33604[12], 14}, {NULL}};
const Segment SGI346931[] = {{&S33605[6], 4}, {NULL}};
const Segment SGI346932[] = {{&S33605[58], 5}, {NULL}};
const Segment SGI346933[] = {{&S33605[12], 14}, {&S33605[69], 4}, {NULL}};
const Segment SGI346934[] = {{&S33605[12], 14}, {NULL}};
const Segment SGI346935[] = {{&S33606[6], 4}, {NULL}};
const Segment SGI346936[] = {{&S33606[80], 5}, {NULL}};
const Segment SGI346937[] = {{&S33606[12], 14}, {&S33606[40], 11}, {NULL}};
const Segment SGI346938[] = {{&S33606[12], 14}, {NULL}};
const Segment SGI346939[] = {{&S33607[6], 4}, {NULL}};
const Segment SGI346940[] = {{&S33607[80], 5}, {NULL}};
const Segment SGI346941[] = {{&S33607[12], 14}, {&S33607[40], 11}, {NULL}};
const Segment SGI346942[] = {{&S33607[12], 14}, {NULL}};
const Segment SGI346943[] = {{&S33608[6], 4}, {NULL}};
const Segment SGI346944[] = {{&S33608[80], 5}, {NULL}};
const Segment SGI346945[] = {{&S33608[12], 14}, {&S33608[40], 11}, {NULL}};
const Segment SGI346946[] = {{&S33608[12], 14}, {NULL}};
const Segment SGI346947[] = {{&S33609[6], 4}, {NULL}};
const Segment SGI346948[] = {{&S33609[80], 5}, {NULL}};
const Segment SGI346949[] = {{&S33609[12], 14}, {&S33609[40], 11}, {NULL}};
const Segment SGI346950[] = {{&S33609[12], 14}, {NULL}};
const Segment SGI346951[] = {{&S33610[6], 4}, {NULL}};
const Segment SGI346952[] = {{&S33610[80], 5}, {NULL}};
const Segment SGI346953[] = {{&S33610[12], 14}, {&S33610[40], 11}, {NULL}};
const Segment SGI346954[] = {{&S33610[12], 14}, {NULL}};
const Segment SGI346955[] = {{&S33611[6], 4}, {NULL}};
const Segment SGI346956[] = {{&S33611[80], 5}, {NULL}};
const Segment SGI346957[] = {{&S33611[12], 14}, {&S33611[40], 11}, {NULL}};
const Segment SGI346958[] = {{&S33611[12], 14}, {NULL}};
const Segment SGI346959[] = {{&S33612[6], 4}, {NULL}};
const Segment SGI346960[] = {{&S33612[80], 5}, {NULL}};
const Segment SGI346961[] = {{&S33612[12], 14}, {&S33612[40], 11}, {NULL}};
const Segment SGI346962[] = {{&S33612[12], 14}, {NULL}};
const Segment SGI346963[] = {{&S33613[6], 4}, {NULL}};
const Segment SGI346964[] = {{&S33613[80], 5}, {NULL}};
const Segment SGI346965[] = {{&S33613[12], 14}, {&S33613[40], 11}, {NULL}};
const Segment SGI346966[] = {{&S33613[12], 14}, {NULL}};
const Segment SGI346967[] = {{&S33614[6], 4}, {NULL}};
const Segment SGI346968[] = {{&S33614[80], 5}, {NULL}};
const Segment SGI346969[] = {{&S33614[12], 14}, {&S33614[40], 11}, {NULL}};
const Segment SGI346970[] = {{&S33614[12], 14}, {NULL}};
const Segment SGI346971[] = {{&S33615[6], 4}, {NULL}};
const Segment SGI346972[] = {{&S33615[80], 5}, {NULL}};
const Segment SGI346973[] = {{&S33615[12], 14}, {&S33615[40], 11}, {NULL}};
const Segment SGI346974[] = {{&S33615[12], 14}, {NULL}};
const Segment SGI346975[] = {{&S33616[6], 4}, {NULL}};
const Segment SGI346976[] = {{&S33616[80], 5}, {NULL}};
const Segment SGI346977[] = {{&S33616[12], 14}, {&S33616[40], 11}, {NULL}};
const Segment SGI346978[] = {{&S33616[12], 14}, {NULL}};
const Segment SGI346979[] = {{&S33617[6], 4}, {NULL}};
const Segment SGI346980[] = {{&S33617[80], 5}, {NULL}};
const Segment SGI346981[] = {{&S33617[12], 14}, {&S33617[40], 11}, {NULL}};
const Segment SGI346982[] = {{&S33617[12], 14}, {NULL}};
const Segment SGI346983[] = {{&S51687[28], 5}, {NULL}};
const Segment SGI346984[] = {{&S51687[104], 5}, {NULL}};
const Segment SGI346985[] = {{&S51688[28], 5}, {NULL}};
const Segment SGI346986[] = {{&S51688[104], 5}, {NULL}};
const Segment SGI346987[] = {{&S60191[43], 5}, {NULL}};
const Segment SGI346988[] = {{&S60191[6], 3}, {NULL}};
const Segment SGI346989[] = {{&S60191[69], 3}, {NULL}};
const Segment SGI346990[] = {{&S60192[5], 5}, {NULL}};
const Segment SGI346991[] = {{&S60192[16], 3}, {NULL}};
const Segment SGI346992[] = {{&S60228[5], 7}, {&S60228[5], 7}, {NULL}};
const Segment SGI346993[] = {{&S60228[117], 3}, {NULL}};
const Segment SGI346994[] = {{&S51810[28], 3}, {NULL}};
const Segment SGI346995[] = {{&S51810[102], 5}, {NULL}};
const Segment SGI346996[] = {{&S46787[42], 5}, {&S46787[147], 7}, {NULL}};
const Segment SGI346997[] = {{&S46787[33], 3}, {NULL}};
const Segment SGI346998[] = {{&S60328[5], 5}, {&S60328[104], 5}, {NULL}};
const Segment SGI346999[] = {{&S60328[11], 3}, {NULL}};
const Segment SGI347000[] = {{&S60328[130], 3}, {NULL}};
const Segment SGI347001[] = {{&S51958[28], 5}, {NULL}};
const Segment SGI347002[] = {{&S51958[104], 5}, {NULL}};
const Segment SGI347003[] = {{&S51959[28], 7}, {NULL}};
const Segment SGI347004[] = {{&S51959[106], 5}, {NULL}};
const Segment SGI347005[] = {{&S33896[6], 4}, {NULL}};
const Segment SGI347006[] = {{&S33896[52], 5}, {NULL}};
const Segment SGI347007[] = {{&S33896[12], 45}, {NULL}};
const Segment SGI347008[] = {{&S33896[12], 14}, {NULL}};
const Segment SGI347009[] = {{&S33897[6], 4}, {NULL}};
const Segment SGI347010[] = {{&S33897[59], 5}, {NULL}};
const Segment SGI347011[] = {{&S33897[12], 14}, {NULL}};
const Segment SGI347012[] = {{&S33898[6], 4}, {NULL}};
const Segment SGI347013[] = {{&S33898[75], 5}, {NULL}};
const Segment SGI347014[] = {{&S33898[12], 14}, {&S33898[37], 43}, {NULL}};
const Segment SGI347015[] = {{&S33898[12], 14}, {NULL}};
const Segment SGI347016[] = {{&S33899[6], 4}, {NULL}};
const Segment SGI347017[] = {{&S33899[74], 5}, {NULL}};
const Segment SGI347018[] = {{&S33899[12], 14}, {&S33899[37], 42}, {NULL}};
const Segment SGI347019[] = {{&S33899[12], 14}, {NULL}};
const Segment SGI347020[] = {{&S33900[6], 4}, {NULL}};
const Segment SGI347021[] = {{&S33900[74], 5}, {NULL}};
const Segment SGI347022[] = {{&S33900[12], 14}, {&S33900[37], 42}, {NULL}};
const Segment SGI347023[] = {{&S33900[12], 14}, {NULL}};
const Segment SGI347024[] = {{&S33901[6], 4}, {NULL}};
const Segment SGI347025[] = {{&S33901[52], 5}, {NULL}};
const Segment SGI347026[] = {{&S33901[12], 55}, {NULL}};
const Segment SGI347027[] = {{&S33901[12], 14}, {NULL}};
const Segment SGI347028[] = {{&S33902[6], 4}, {NULL}};
const Segment SGI347029[] = {{&S33902[74], 5}, {NULL}};
const Segment SGI347030[] = {{&S33902[12], 14}, {&S33902[37], 42}, {NULL}};
const Segment SGI347031[] = {{&S33902[12], 14}, {NULL}};
const Segment SGI347032[] = {{&S33903[6], 4}, {NULL}};
const Segment SGI347033[] = {{&S33903[74], 5}, {NULL}};
const Segment SGI347034[] = {{&S33903[12], 14}, {&S33903[37], 42}, {NULL}};
const Segment SGI347035[] = {{&S33903[12], 14}, {NULL}};
const Segment SGI347036[] = {{&S33905[6], 4}, {NULL}};
const Segment SGI347037[] = {{&S33905[74], 5}, {NULL}};
const Segment SGI347038[] = {{&S33905[12], 14}, {&S33905[37], 42}, {NULL}};
const Segment SGI347039[] = {{&S33905[12], 14}, {NULL}};
const Segment SGI347040[] = {{&S33906[6], 4}, {NULL}};
const Segment SGI347041[] = {{&S33906[75], 5}, {NULL}};
const Segment SGI347042[] = {{&S33906[12], 14}, {&S33906[37], 43}, {NULL}};
const Segment SGI347043[] = {{&S33906[12], 14}, {NULL}};
const Segment SGI347044[] = {{&S33908[6], 4}, {NULL}};
const Segment SGI347045[] = {{&S33908[74], 5}, {NULL}};
const Segment SGI347046[] = {{&S33908[12], 14}, {&S33908[37], 42}, {NULL}};
const Segment SGI347047[] = {{&S33908[12], 14}, {NULL}};
const Segment SGI347048[] = {{&S33909[6], 4}, {NULL}};
const Segment SGI347049[] = {{&S33909[75], 5}, {NULL}};
const Segment SGI347050[] = {{&S33909[12], 14}, {&S33909[37], 43}, {NULL}};
const Segment SGI347051[] = {{&S33909[12], 14}, {NULL}};
const Segment SGI347052[] = {{&S33910[6], 4}, {NULL}};
const Segment SGI347053[] = {{&S33910[75], 5}, {NULL}};
const Segment SGI347054[] = {{&S33910[12], 14}, {&S33910[38], 42}, {NULL}};
const Segment SGI347055[] = {{&S33910[12], 14}, {NULL}};
const Segment SGI347056[] = {{&S33911[6], 4}, {NULL}};
const Segment SGI347057[] = {{&S33911[74], 5}, {NULL}};
const Segment SGI347058[] = {{&S33911[12], 14}, {&S33911[37], 42}, {NULL}};
const Segment SGI347059[] = {{&S33911[12], 14}, {NULL}};
const Segment SGI347060[] = {{&S33912[6], 4}, {NULL}};
const Segment SGI347061[] = {{&S33912[74], 5}, {NULL}};
const Segment SGI347062[] = {{&S33912[12], 14}, {&S33912[37], 42}, {NULL}};
const Segment SGI347063[] = {{&S33912[12], 14}, {NULL}};
const Segment SGI347064[] = {{&S33913[6], 4}, {NULL}};
const Segment SGI347065[] = {{&S33913[74], 5}, {NULL}};
const Segment SGI347066[] = {{&S33913[12], 14}, {&S33913[37], 42}, {NULL}};
const Segment SGI347067[] = {{&S33913[12], 14}, {NULL}};
const Segment SGI347068[] = {{&S33914[6], 4}, {NULL}};
const Segment SGI347069[] = {{&S33914[74], 5}, {NULL}};
const Segment SGI347070[] = {{&S33914[12], 14}, {&S33914[37], 42}, {NULL}};
const Segment SGI347071[] = {{&S33914[12], 14}, {NULL}};
const Segment SGI347072[] = {{&S33915[6], 4}, {NULL}};
const Segment SGI347073[] = {{&S33915[74], 5}, {NULL}};
const Segment SGI347074[] = {{&S33915[12], 14}, {&S33915[37], 42}, {NULL}};
const Segment SGI347075[] = {{&S33915[12], 14}, {NULL}};
const Segment SGI347076[] = {{&S33916[6], 4}, {NULL}};
const Segment SGI347077[] = {{&S33916[75], 5}, {NULL}};
const Segment SGI347078[] = {{&S33916[12], 14}, {&S33916[38], 42}, {NULL}};
const Segment SGI347079[] = {{&S33916[12], 14}, {NULL}};
const Segment SGI347080[] = {{&S33917[6], 4}, {NULL}};
const Segment SGI347081[] = {{&S33917[74], 5}, {NULL}};
const Segment SGI347082[] = {{&S33917[12], 14}, {&S33917[37], 42}, {NULL}};
const Segment SGI347083[] = {{&S33917[12], 14}, {NULL}};
const Segment SGI347084[] = {{&S33918[6], 4}, {NULL}};
const Segment SGI347085[] = {{&S33918[75], 5}, {NULL}};
const Segment SGI347086[] = {{&S33918[12], 14}, {&S33918[37], 43}, {NULL}};
const Segment SGI347087[] = {{&S33918[12], 14}, {NULL}};
const Segment SGI347088[] = {{&S33919[6], 4}, {NULL}};
const Segment SGI347089[] = {{&S33919[73], 5}, {NULL}};
const Segment SGI347090[] = {{&S33919[12], 14}, {&S33919[84], 4}, {NULL}};
const Segment SGI347091[] = {{&S33919[12], 14}, {NULL}};
const Segment SGI347092[] = {{&S33920[6], 4}, {NULL}};
const Segment SGI347093[] = {{&S33920[74], 5}, {NULL}};
const Segment SGI347094[] = {{&S33920[12], 14}, {&S33920[37], 42}, {NULL}};
const Segment SGI347095[] = {{&S33920[12], 14}, {NULL}};
const Segment SGI347096[] = {{&S52004[28], 3}, {NULL}};
const Segment SGI347097[] = {{&S52004[102], 5}, {NULL}};
const Segment SGI347098[] = {{&S33921[6], 4}, {NULL}};
const Segment SGI347099[] = {{&S33921[74], 5}, {NULL}};
const Segment SGI347100[] = {{&S33921[12], 14}, {&S33921[37], 42}, {NULL}};
const Segment SGI347101[] = {{&S33921[12], 14}, {NULL}};
const Segment SGI347102[] = {{&S52005[28], 5}, {NULL}};
const Segment SGI347103[] = {{&S33922[6], 4}, {NULL}};
const Segment SGI347104[] = {{&S33922[75], 5}, {NULL}};
const Segment SGI347105[] = {{&S33922[12], 14}, {&S33922[37], 43}, {NULL}};
const Segment SGI347106[] = {{&S52005[104], 5}, {NULL}};
const Segment SGI347107[] = {{&S33922[12], 14}, {NULL}};
const Segment SGI347108[] = {{&S33923[6], 4}, {NULL}};
const Segment SGI347109[] = {{&S33923[74], 5}, {NULL}};
const Segment SGI347110[] = {{&S33923[12], 14}, {&S33923[37], 42}, {NULL}};
const Segment SGI347111[] = {{&S33923[12], 14}, {NULL}};
const Segment SGI347112[] = {{&S33924[6], 4}, {NULL}};
const Segment SGI347113[] = {{&S33924[74], 5}, {NULL}};
const Segment SGI347114[] = {{&S33924[12], 14}, {&S33924[37], 42}, {NULL}};
const Segment SGI347115[] = {{&S33924[12], 14}, {NULL}};
const Segment SGI347116[] = {{&S33925[6], 4}, {NULL}};
const Segment SGI347117[] = {{&S33925[74], 5}, {NULL}};
const Segment SGI347118[] = {{&S33925[12], 14}, {&S33925[37], 42}, {NULL}};
const Segment SGI347119[] = {{&S33925[12], 14}, {NULL}};
const Segment SGI347120[] = {{&S33926[6], 4}, {NULL}};
const Segment SGI347121[] = {{&S33926[74], 5}, {NULL}};
const Segment SGI347122[] = {{&S33926[12], 14}, {&S33926[37], 42}, {NULL}};
const Segment SGI347123[] = {{&S33926[12], 14}, {NULL}};
const Segment SGI347124[] = {{&S33927[6], 4}, {NULL}};
const Segment SGI347125[] = {{&S33927[52], 5}, {NULL}};
const Segment SGI347126[] = {{&S33927[12], 65}, {NULL}};
const Segment SGI347127[] = {{&S33927[12], 14}, {NULL}};
const Segment SGI347128[] = {{&S33928[6], 4}, {NULL}};
const Segment SGI347129[] = {{&S33928[74], 5}, {NULL}};
const Segment SGI347130[] = {{&S33928[12], 14}, {&S33928[37], 42}, {NULL}};
const Segment SGI347131[] = {{&S33928[12], 14}, {NULL}};
const Segment SGI347132[] = {{&S33929[6], 4}, {NULL}};
const Segment SGI347133[] = {{&S33929[74], 5}, {NULL}};
const Segment SGI347134[] = {{&S33929[12], 14}, {&S33929[37], 42}, {NULL}};
const Segment SGI347135[] = {{&S33929[12], 14}, {NULL}};
const Segment SGI347136[] = {{&S33930[6], 4}, {NULL}};
const Segment SGI347137[] = {{&S33930[74], 5}, {NULL}};
const Segment SGI347138[] = {{&S33930[12], 14}, {&S33930[37], 42}, {NULL}};
const Segment SGI347139[] = {{&S33930[12], 14}, {NULL}};
const Segment SGI347140[] = {{&S33931[6], 4}, {NULL}};
const Segment SGI347141[] = {{&S33931[74], 5}, {NULL}};
const Segment SGI347142[] = {{&S33931[12], 14}, {&S33931[37], 42}, {NULL}};
const Segment SGI347143[] = {{&S33931[12], 14}, {NULL}};
const Segment SGI347144[] = {{&S33932[6], 4}, {NULL}};
const Segment SGI347145[] = {{&S33932[101], 5}, {NULL}};
const Segment SGI347146[] = {{&S33932[12], 14}, {&S33932[37], 12}, {NULL}};
const Segment SGI347147[] = {{&S33932[12], 14}, {NULL}};
const Segment SGI347148[] = {{&S33934[6], 4}, {NULL}};
const Segment SGI347149[] = {{&S33934[74], 5}, {NULL}};
const Segment SGI347150[] = {{&S33934[12], 14}, {&S33934[37], 42}, {NULL}};
const Segment SGI347151[] = {{&S33934[12], 14}, {NULL}};
const Segment SGI347152[] = {{&S33935[6], 4}, {NULL}};
const Segment SGI347153[] = {{&S33935[74], 5}, {NULL}};
const Segment SGI347154[] = {{&S33935[12], 14}, {&S33935[37], 42}, {NULL}};
const Segment SGI347155[] = {{&S33935[12], 14}, {NULL}};
const Segment SGI347156[] = {{&S33936[6], 4}, {NULL}};
const Segment SGI347157[] = {{&S33936[74], 5}, {NULL}};
const Segment SGI347158[] = {{&S33936[12], 14}, {&S33936[37], 42}, {NULL}};
const Segment SGI347159[] = {{&S33936[12], 14}, {NULL}};
const Segment SGI347160[] = {{&S33937[6], 4}, {NULL}};
const Segment SGI347161[] = {{&S33937[74], 5}, {NULL}};
const Segment SGI347162[] = {{&S33937[12], 14}, {&S33937[37], 42}, {NULL}};
const Segment SGI347163[] = {{&S33937[12], 14}, {NULL}};
const Segment SGI347164[] = {{&S33938[6], 4}, {NULL}};
const Segment SGI347165[] = {{&S33938[74], 5}, {NULL}};
const Segment SGI347166[] = {{&S33938[12], 14}, {&S33938[37], 42}, {NULL}};
const Segment SGI347167[] = {{&S33938[12], 14}, {NULL}};
const Segment SGI347168[] = {{&S33939[6], 4}, {NULL}};
const Segment SGI347169[] = {{&S33939[74], 5}, {NULL}};
const Segment SGI347170[] = {{&S33939[12], 14}, {&S33939[37], 42}, {NULL}};
const Segment SGI347171[] = {{&S33939[12], 14}, {NULL}};
const Segment SGI347172[] = {{&S33940[6], 4}, {NULL}};
const Segment SGI347173[] = {{&S33940[74], 5}, {NULL}};
const Segment SGI347174[] = {{&S33940[12], 14}, {&S33940[37], 42}, {NULL}};
const Segment SGI347175[] = {{&S33940[12], 14}, {NULL}};
const Segment SGI347176[] = {{&S36080[67], 4}, {NULL}};
const Segment SGI347177[] = {{&S36080[13], 10}, {NULL}};
const Segment SGI347178[] = {{&S33941[6], 4}, {NULL}};
const Segment SGI347179[] = {{&S33941[74], 5}, {NULL}};
const Segment SGI347180[] = {{&S33941[12], 14}, {&S33941[37], 42}, {NULL}};
const Segment SGI347181[] = {{&S33941[12], 14}, {NULL}};
const Segment SGI347182[] = {{&S36082[38], 4}, {NULL}};
const Segment SGI347183[] = {{&S36082[13], 14}, {NULL}};
const Segment SGI347184[] = {{&S33942[6], 4}, {NULL}};
const Segment SGI347185[] = {{&S33942[75], 5}, {NULL}};
const Segment SGI347186[] = {{&S33942[12], 14}, {&S33942[37], 43}, {NULL}};
const Segment SGI347187[] = {{&S33942[12], 14}, {NULL}};
const Segment SGI347188[] = {{&S33943[6], 4}, {NULL}};
const Segment SGI347189[] = {{&S36084[38], 4}, {NULL}};
const Segment SGI347190[] = {{&S36084[13], 14}, {NULL}};
const Segment SGI347191[] = {{&S33943[74], 5}, {NULL}};
const Segment SGI347192[] = {{&S33943[12], 14}, {&S33943[37], 42}, {NULL}};
const Segment SGI347193[] = {{&S33943[12], 14}, {NULL}};
const Segment SGI347194[] = {{&S36086[38], 4}, {NULL}};
const Segment SGI347195[] = {{&S36086[13], 14}, {NULL}};
const Segment SGI347196[] = {{&S36087[67], 4}, {NULL}};
const Segment SGI347197[] = {{&S36087[13], 10}, {NULL}};
const Segment SGI347198[] = {{&S36088[38], 4}, {NULL}};
const Segment SGI347199[] = {{&S36088[13], 14}, {NULL}};
const Segment SGI347200[] = {{&S36089[67], 4}, {NULL}};
const Segment SGI347201[] = {{&S36089[13], 10}, {NULL}};
const Segment SGI347202[] = {{&S36090[57], 4}, {NULL}};
const Segment SGI347203[] = {{&S36090[13], 10}, {NULL}};
const Segment SGI347204[] = {{&S36091[57], 4}, {NULL}};
const Segment SGI347205[] = {{&S36091[13], 10}, {NULL}};
const Segment SGI347206[] = {{&S36095[57], 4}, {NULL}};
const Segment SGI347207[] = {{&S36095[13], 10}, {NULL}};
const Segment SGI347208[] = {{&S36096[57], 4}, {NULL}};
const Segment SGI347209[] = {{&S36096[13], 10}, {NULL}};
const Segment SGI347210[] = {{&S36097[57], 4}, {NULL}};
const Segment SGI347211[] = {{&S36097[13], 10}, {NULL}};
const Segment SGI347212[] = {{&S36098[57], 4}, {NULL}};
const Segment SGI347213[] = {{&S36098[13], 10}, {NULL}};
const Segment SGI347214[] = {{&S36100[57], 4}, {NULL}};
const Segment SGI347215[] = {{&S36100[13], 10}, {NULL}};
const Segment SGI347216[] = {{&S60526[5], 10}, {NULL}};
const Segment SGI347217[] = {{&S60527[5], 5}, {NULL}};
const Segment SGI347218[] = {{&S60540[5], 5}, {&S60540[97], 5}, {NULL}};
const Segment SGI347219[] = {{&S60540[153], 3}, {NULL}};
const Segment SGI347220[] = {{&S47041[5], 5}, {NULL}};
const Segment SGI347221[] = {{&S47041[33], 3}, {NULL}};
const Segment SGI347222[] = {{&S47041[87], 3}, {NULL}};
const Segment SGI347223[] = {{&S47042[5], 3}, {NULL}};
const Segment SGI347224[] = {{&S47042[32], 3}, {NULL}};
const Segment SGI347225[] = {{&S47042[86], 3}, {NULL}};
const Segment SGI347226[] = {{&S47078[5], 7}, {NULL}};
const Segment SGI347227[] = {{&S47078[42], 3}, {NULL}};
const Segment SGI347228[] = {{&S47079[42], 5}, {&S47079[142], 7}, {NULL}};
const Segment SGI347229[] = {{&S47079[33], 3}, {NULL}};
const Segment SGI347230[] = {{&S60618[5], 5}, {NULL}};
const Segment SGI347231[] = {{&S47080[42], 7}, {&S47080[144], 7}, {NULL}};
const Segment SGI347232[] = {{&S47080[33], 3}, {NULL}};
const Segment SGI347233[] = {{&S60624[42], 5}, {NULL}};
const Segment SGI347234[] = {{&S60624[6], 3}, {NULL}};
const Segment SGI347235[] = {{&S60624[68], 3}, {NULL}};
const Segment SGI347236[] = {{&S60625[43], 5}, {NULL}};
const Segment SGI347237[] = {{&S60625[6], 3}, {NULL}};
const Segment SGI347238[] = {{&S60625[69], 3}, {NULL}};
const Segment SGI347239[] = {{&S60626[43], 5}, {NULL}};
const Segment SGI347240[] = {{&S60626[6], 3}, {NULL}};
const Segment SGI347241[] = {{&S60626[69], 3}, {NULL}};
const Segment SGI347242[] = {{&S60627[5], 5}, {NULL}};
const Segment SGI347243[] = {{&S60627[16], 3}, {NULL}};
const Segment SGI347244[] = {{&S60632[5], 9}, {NULL}};
const Segment SGI347245[] = {{&S60633[5], 5}, {NULL}};
const Segment SGI347246[] = {{&S60672[42], 5}, {NULL}};
const Segment SGI347247[] = {{&S60672[6], 3}, {NULL}};
const Segment SGI347248[] = {{&S60672[68], 3}, {NULL}};
const Segment SGI347249[] = {{&S47344[42], 11}, {&S47344[153], 7}, {NULL}};
const Segment SGI347250[] = {{&S47344[33], 3}, {NULL}};
const Segment SGI347251[] = {{&S47345[42], 11}, {&S47345[153], 7}, {NULL}};
const Segment SGI347252[] = {{&S47345[33], 3}, {NULL}};
const Segment SGI347253[] = {{&S47346[5], 3}, {&S47346[157], 7}, {NULL}};
const Segment SGI347254[] = {{&S47346[43], 3}, {NULL}};
const Segment SGI347255[] = {{&S60731[5], 3}, {&S60731[5], 3}, {NULL}};
const Segment SGI347256[] = {{&S60731[9], 3}, {NULL}};
const Segment SGI347257[] = {{&S60731[127], 3}, {NULL}};
const Segment SGI347258[] = {{&S60774[5], 7}, {&S60774[5], 7}, {NULL}};
const Segment SGI347259[] = {{&S60774[117], 3}, {NULL}};
const Segment SGI347260[] = {{&S60777[5], 4}, {&S60777[5], 4}, {NULL}};
const Segment SGI347261[] = {{&S60778[5], 4}, {NULL}};
const Segment SGI347262[] = {{&S60779[5], 4}, {NULL}};
const Segment SGI347263[] = {{&S60779[15], 3}, {NULL}};
const Segment SGI347264[] = {{&S60780[5], 4}, {NULL}};
const Segment SGI347265[] = {{&S60780[20], 3}, {NULL}};
const Segment SGI347266[] = {{&S60816[64], 4}, {NULL}};
const Segment SGI347267[] = {{&S60816[53], 4}, {&S60816[53], 4}, {NULL}};
const Segment SGI347268[] = {{&S60817[52], 4}, {NULL}};
const Segment SGI347269[] = {{&S60817[83], 4}, {NULL}};
const Segment SGI347270[] = {{&S60818[52], 4}, {NULL}};
const Segment SGI347271[] = {{&S60818[83], 4}, {NULL}};
const Segment SGI347272[] = {{&S52207[28], 3}, {NULL}};
const Segment SGI347273[] = {{&S52207[102], 5}, {NULL}};
const Segment SGI347274[] = {{&S60861[5], 5}, {&S60861[105], 5}, {NULL}};
const Segment SGI347275[] = {{&S60861[69], 3}, {NULL}};
const Segment SGI347276[] = {{&S60861[131], 3}, {NULL}};
const Segment SGI347277[] = {{&S60889[5], 3}, {&S60889[5], 3}, {NULL}};
const Segment SGI347278[] = {{&S60889[67], 3}, {NULL}};
const Segment SGI347279[] = {{&S60889[127], 3}, {NULL}};
const Segment SGI347280[] = {{&S60917[5], 5}, {&S60917[97], 5}, {NULL}};
const Segment SGI347281[] = {{&S60917[113], 3}, {NULL}};
const Segment SGI347282[] = {{&S47550[49], 3}, {NULL}};
const Segment SGI347283[] = {{&S47550[40], 3}, {NULL}};
const Segment SGI347284[] = {{&S36701[58], 5}, {NULL}};
const Segment SGI347285[] = {{&S36701[13], 10}, {NULL}};
const Segment SGI347286[] = {{&S36714[64], 5}, {NULL}};
const Segment SGI347287[] = {{&S36714[13], 10}, {NULL}};
const Segment SGI347288[] = {{&S60980[35], 5}, {NULL}};
const Segment SGI347289[] = {{&S60980[91], 3}, {NULL}};
const Segment SGI347290[] = {{&S60981[5], 5}, {NULL}};
const Segment SGI347291[] = {{&S60981[16], 3}, {NULL}};
const Segment SGI347292[] = {{&S60982[34], 5}, {NULL}};
const Segment SGI347293[] = {{&S60982[90], 3}, {NULL}};
const Segment SGI347294[] = {{&S60990[5], 3}, {&S60990[5], 3}, {NULL}};
const Segment SGI347295[] = {{&S60990[9], 3}, {NULL}};
const Segment SGI347296[] = {{&S60990[127], 3}, {NULL}};
const Segment SGI347297[] = {{&S34018[0], 16}, {NULL}};
const Segment SGI347298[] = {{&S34018[29], 3}, {NULL}};
const Segment SGI347299[] = {{&S34018[71], 9}, {NULL}};
const Segment SGI347300[] = {{&S60996[5], 4}, {NULL}};
const Segment SGI347301[] = {{&S36806[57], 5}, {NULL}};
const Segment SGI347302[] = {{&S36806[13], 10}, {NULL}};
const Segment SGI347303[] = {{&S61050[5], 4}, {NULL}};
const Segment SGI347304[] = {{&S61050[15], 3}, {NULL}};
const Segment SGI347305[] = {{&S61051[35], 4}, {NULL}};
const Segment SGI347306[] = {{&S61051[51], 3}, {NULL}};
const Segment SGI347307[] = {{&S61052[35], 5}, {NULL}};
const Segment SGI347308[] = {{&S61052[52], 3}, {NULL}};
const Segment SGI347309[] = {{&S61053[34], 4}, {NULL}};
const Segment SGI347310[] = {{&S61053[49], 3}, {NULL}};
const Segment SGI347311[] = {{&S52556[28], 3}, {NULL}};
const Segment SGI347312[] = {{&S52556[102], 5}, {NULL}};
const Segment SGI347313[] = {{&S36925[61], 4}, {NULL}};
const Segment SGI347314[] = {{&S36925[13], 10}, {NULL}};
const Segment SGI347315[] = {{&S36926[8], 4}, {&S36926[8], 3}, {NULL}};
const Segment SGI347316[] = {{&S36926[1], 27}, {NULL}};
const Segment SGI347317[] = {{&S61181[5], 5}, {&S61181[105], 5}, {NULL}};
const Segment SGI347318[] = {{&S61181[11], 3}, {NULL}};
const Segment SGI347319[] = {{&S61181[131], 3}, {NULL}};
const Segment SGI347320[] = {{&S61184[35], 16}, {NULL}};
const Segment SGI347321[] = {{&S61184[63], 3}, {NULL}};
const Segment SGI347322[] = {{&S61185[5], 16}, {NULL}};
const Segment SGI347323[] = {{&S61185[27], 3}, {NULL}};
const Segment SGI347324[] = {{&S61186[5], 17}, {NULL}};
const Segment SGI347325[] = {{&S61186[28], 3}, {NULL}};
const Segment SGI347326[] = {{&S61187[34], 7}, {NULL}};
const Segment SGI347327[] = {{&S61187[52], 3}, {NULL}};
const Segment SGI347328[] = {{&S61192[35], 4}, {NULL}};
const Segment SGI347329[] = {{&S61192[90], 3}, {NULL}};
const Segment SGI347330[] = {{&S61193[5], 13}, {NULL}};
const Segment SGI347331[] = {{&S61193[19], 3}, {NULL}};
const Segment SGI347332[] = {{&S52788[28], 3}, {NULL}};
const Segment SGI347333[] = {{&S52788[102], 5}, {NULL}};
const Segment SGI347334[] = {{&S52797[28], 3}, {NULL}};
const Segment SGI347335[] = {{&S52797[102], 5}, {NULL}};
const Segment SGI347336[] = {{&S52798[28], 3}, {NULL}};
const Segment SGI347337[] = {{&S52798[102], 5}, {NULL}};
const Segment SGI347338[] = {{&S61351[35], 3}, {NULL}};
const Segment SGI347339[] = {{&S61351[89], 3}, {NULL}};
const Segment SGI347340[] = {{&S61352[5], 3}, {NULL}};
const Segment SGI347341[] = {{&S61352[14], 3}, {NULL}};
const Segment SGI347342[] = {{&S61353[34], 5}, {NULL}};
const Segment SGI347343[] = {{&S61353[90], 3}, {NULL}};
const Segment SGI347344[] = {{&S47959[5], 5}, {NULL}};
const Segment SGI347345[] = {{&S47959[33], 3}, {NULL}};
const Segment SGI347346[] = {{&S47960[5], 3}, {NULL}};
const Segment SGI347347[] = {{&S47960[32], 3}, {NULL}};
const Segment SGI347348[] = {{&S47961[5], 3}, {NULL}};
const Segment SGI347349[] = {{&S47961[32], 3}, {NULL}};
const Segment SGI347350[] = {{&S47962[5], 5}, {NULL}};
const Segment SGI347351[] = {{&S47962[34], 3}, {NULL}};
const Segment SGI347352[] = {{&S61417[5], 4}, {&S61417[5], 4}, {NULL}};
const Segment SGI347353[] = {{&S61417[151], 3}, {NULL}};
const Segment SGI347354[] = {{&S37184[57], 5}, {NULL}};
const Segment SGI347355[] = {{&S37184[13], 10}, {NULL}};
const Segment SGI347356[] = {{&S61500[5], 7}, {NULL}};
const Segment SGI347357[] = {{&S61501[5], 8}, {NULL}};
const Segment SGI347358[] = {{&S61502[5], 5}, {NULL}};
const Segment SGI347359[] = {{&S48126[46], 5}, {NULL}};
const Segment SGI347360[] = {{&S48126[37], 3}, {NULL}};
const Segment SGI347361[] = {{&S48127[48], 5}, {NULL}};
const Segment SGI347362[] = {{&S48127[40], 11}, {NULL}};
const Segment SGI347363[] = {{&S48128[49], 6}, {NULL}};
const Segment SGI347364[] = {{&S48128[40], 3}, {NULL}};
const Segment SGI347365[] = {{&S48129[46], 5}, {NULL}};
const Segment SGI347366[] = {{&S48129[37], 3}, {NULL}};
const Segment SGI347367[] = {{&S37301[62], 5}, {NULL}};
const Segment SGI347368[] = {{&S37301[13], 10}, {NULL}};
const Segment SGI347369[] = {{&S52978[28], 5}, {NULL}};
const Segment SGI347370[] = {{&S52978[104], 5}, {NULL}};
const Segment SGI347371[] = {{&S52984[28], 5}, {NULL}};
const Segment SGI347372[] = {{&S52984[104], 5}, {NULL}};
const Segment SGI347373[] = {{&S61663[5], 3}, {&S61663[5], 3}, {NULL}};
const Segment SGI347374[] = {{&S61663[149], 3}, {NULL}};
const Segment SGI347375[] = {{&S61681[5], 3}, {&S61681[5], 3}, {NULL}};
const Segment SGI347376[] = {{&S61681[9], 3}, {NULL}};
const Segment SGI347377[] = {{&S61681[126], 3}, {NULL}};
const Segment SGI347378[] = {{&S61682[35], 3}, {NULL}};
const Segment SGI347379[] = {{&S61682[6], 3}, {NULL}};
const Segment SGI347380[] = {{&S61682[65], 3}, {NULL}};
const Segment SGI347381[] = {{&S61683[5], 3}, {&S61683[5], 3}, {NULL}};
const Segment SGI347382[] = {{&S61683[9], 3}, {NULL}};
const Segment SGI347383[] = {{&S61683[126], 3}, {NULL}};
const Segment SGI347384[] = {{&S61766[5], 5}, {&S61766[105], 5}, {NULL}};
const Segment SGI347385[] = {{&S61766[11], 3}, {NULL}};
const Segment SGI347386[] = {{&S61766[131], 3}, {NULL}};
const Segment SGI347387[] = {{&S61774[5], 7}, {&S61774[5], 7}, {NULL}};
const Segment SGI347388[] = {{&S61774[157], 3}, {NULL}};
const Segment SGI347389[] = {{&S61784[5], 5}, {&S61784[105], 5}, {NULL}};
const Segment SGI347390[] = {{&S61784[69], 3}, {NULL}};
const Segment SGI347391[] = {{&S61784[131], 3}, {NULL}};
const Segment SGI347392[] = {{&S53150[28], 5}, {NULL}};
const Segment SGI347393[] = {{&S53150[104], 5}, {NULL}};
const Segment SGI347394[] = {{&S53151[28], 7}, {NULL}};
const Segment SGI347395[] = {{&S53151[106], 5}, {NULL}};
const Segment SGI347396[] = {{&S53158[28], 3}, {NULL}};
const Segment SGI347397[] = {{&S53158[102], 5}, {NULL}};
const Segment SGI347398[] = {{&S53159[28], 5}, {NULL}};
const Segment SGI347399[] = {{&S53159[104], 5}, {NULL}};
const Segment SGI347400[] = {{&S37531[6], 5}, {&S37531[42], 4}, {NULL}};
const Segment SGI347401[] = {{&S37531[12], 45}, {NULL}};
const Segment SGI347402[] = {{&S37531[99], 3}, {NULL}};
const Segment SGI347403[] = {{&S37531[6], 5}, {NULL}};
const Segment SGI347404[] = {{&S53177[28], 5}, {NULL}};
const Segment SGI347405[] = {{&S53177[111], 5}, {NULL}};
const Segment SGI347406[] = {{&S53178[28], 7}, {NULL}};
const Segment SGI347407[] = {{&S53178[106], 5}, {NULL}};
const Segment SGI347408[] = {{&S61814[5], 3}, {&S61814[5], 3}, {NULL}};
const Segment SGI347409[] = {{&S61814[9], 3}, {NULL}};
const Segment SGI347410[] = {{&S61814[127], 3}, {NULL}};
const Segment SGI347411[] = {{&S61886[35], 3}, {NULL}};
const Segment SGI347412[] = {{&S61886[89], 3}, {NULL}};
const Segment SGI347413[] = {{&S61898[5], 3}, {NULL}};
const Segment SGI347414[] = {{&S61898[19], 3}, {NULL}};
const Segment SGI347415[] = {{&S61899[5], 3}, {NULL}};
const Segment SGI347416[] = {{&S61899[14], 3}, {NULL}};
const Segment SGI347417[] = {{&S53286[28], 5}, {NULL}};
const Segment SGI347418[] = {{&S53286[104], 5}, {NULL}};
const Segment SGI347419[] = {{&S53287[28], 5}, {NULL}};
const Segment SGI347420[] = {{&S53287[104], 5}, {NULL}};
const Segment SGI347421[] = {{&S35471[6], 4}, {NULL}};
const Segment SGI347422[] = {{&S35471[52], 5}, {NULL}};
const Segment SGI347423[] = {{&S35471[12], 45}, {NULL}};
const Segment SGI347424[] = {{&S35471[12], 14}, {NULL}};
const Segment SGI347425[] = {{&S35472[6], 4}, {NULL}};
const Segment SGI347426[] = {{&S35472[75], 5}, {NULL}};
const Segment SGI347427[] = {{&S35472[12], 14}, {&S35472[37], 43}, {NULL}};
const Segment SGI347428[] = {{&S35472[12], 14}, {NULL}};
const Segment SGI347429[] = {{&S35473[6], 4}, {NULL}};
const Segment SGI347430[] = {{&S35473[74], 5}, {NULL}};
const Segment SGI347431[] = {{&S35473[12], 14}, {&S35473[37], 42}, {NULL}};
const Segment SGI347432[] = {{&S35473[12], 14}, {NULL}};
const Segment SGI347433[] = {{&S35474[6], 4}, {NULL}};
const Segment SGI347434[] = {{&S35474[75], 5}, {NULL}};
const Segment SGI347435[] = {{&S35474[12], 14}, {&S35474[38], 42}, {NULL}};
const Segment SGI347436[] = {{&S35474[12], 14}, {NULL}};
const Segment SGI347437[] = {{&S35475[6], 4}, {NULL}};
const Segment SGI347438[] = {{&S35475[74], 5}, {NULL}};
const Segment SGI347439[] = {{&S35475[12], 14}, {&S35475[37], 42}, {NULL}};
const Segment SGI347440[] = {{&S35475[12], 14}, {NULL}};
const Segment SGI347441[] = {{&S61954[35], 3}, {NULL}};
const Segment SGI347442[] = {{&S61954[50], 3}, {NULL}};
const Segment SGI347443[] = {{&S35476[6], 4}, {NULL}};
const Segment SGI347444[] = {{&S35476[52], 5}, {NULL}};
const Segment SGI347445[] = {{&S35476[12], 55}, {NULL}};
const Segment SGI347446[] = {{&S35476[12], 14}, {NULL}};
const Segment SGI347447[] = {{&S61955[5], 3}, {NULL}};
const Segment SGI347448[] = {{&S35477[6], 4}, {NULL}};
const Segment SGI347449[] = {{&S61955[14], 3}, {NULL}};
const Segment SGI347450[] = {{&S35477[74], 5}, {NULL}};
const Segment SGI347451[] = {{&S35477[12], 14}, {&S35477[37], 42}, {NULL}};
const Segment SGI347452[] = {{&S35477[12], 14}, {NULL}};
const Segment SGI347453[] = {{&S61956[34], 5}, {NULL}};
const Segment SGI347454[] = {{&S35478[6], 4}, {NULL}};
const Segment SGI347455[] = {{&S35478[59], 5}, {NULL}};
const Segment SGI347456[] = {{&S35478[12], 14}, {NULL}};
const Segment SGI347457[] = {{&S61956[90], 3}, {NULL}};
const Segment SGI347458[] = {{&S35479[6], 4}, {NULL}};
const Segment SGI347459[] = {{&S35479[74], 5}, {NULL}};
const Segment SGI347460[] = {{&S35479[12], 14}, {&S35479[37], 42}, {NULL}};
const Segment SGI347461[] = {{&S35479[12], 14}, {NULL}};
const Segment SGI347462[] = {{&S35480[6], 4}, {NULL}};
const Segment SGI347463[] = {{&S35480[74], 5}, {NULL}};
const Segment SGI347464[] = {{&S35480[12], 14}, {&S35480[37], 42}, {NULL}};
const Segment SGI347465[] = {{&S35480[12], 14}, {NULL}};
const Segment SGI347466[] = {{&S35481[6], 4}, {NULL}};
const Segment SGI347467[] = {{&S35481[74], 5}, {NULL}};
const Segment SGI347468[] = {{&S35481[12], 14}, {&S35481[37], 42}, {NULL}};
const Segment SGI347469[] = {{&S35481[12], 14}, {NULL}};
const Segment SGI347470[] = {{&S35482[6], 4}, {NULL}};
const Segment SGI347471[] = {{&S35482[104], 5}, {NULL}};
const Segment SGI347472[] = {{&S35482[12], 14}, {&S35482[41], 10}, {NULL}};
const Segment SGI347473[] = {{&S35482[12], 14}, {&S35482[63], 14}, {NULL}};
const Segment SGI347474[] = {{&S35483[6], 4}, {NULL}};
const Segment SGI347475[] = {{&S35483[75], 5}, {NULL}};
const Segment SGI347476[] = {{&S35483[12], 14}, {&S35483[37], 43}, {NULL}};
const Segment SGI347477[] = {{&S35483[12], 14}, {NULL}};
const Segment SGI347478[] = {{&S61964[5], 7}, {&S61964[5], 7}, {NULL}};
const Segment SGI347479[] = {{&S61964[13], 3}, {NULL}};
const Segment SGI347480[] = {{&S35484[6], 4}, {NULL}};
const Segment SGI347481[] = {{&S35484[74], 5}, {NULL}};
const Segment SGI347482[] = {{&S35484[12], 14}, {&S35484[37], 42}, {NULL}};
const Segment SGI347483[] = {{&S61964[135], 3}, {NULL}};
const Segment SGI347484[] = {{&S35484[12], 14}, {NULL}};
const Segment SGI347485[] = {{&S35485[6], 4}, {NULL}};
const Segment SGI347486[] = {{&S61965[5], 5}, {&S61965[5], 5}, {NULL}};
const Segment SGI347487[] = {{&S35485[75], 5}, {NULL}};
const Segment SGI347488[] = {{&S35485[12], 14}, {&S35485[37], 43}, {NULL}};
const Segment SGI347489[] = {{&S61965[11], 3}, {NULL}};
const Segment SGI347490[] = {{&S35485[12], 14}, {NULL}};
const Segment SGI347491[] = {{&S61965[131], 3}, {NULL}};
const Segment SGI347492[] = {{&S35487[6], 4}, {NULL}};
const Segment SGI347493[] = {{&S35487[74], 5}, {NULL}};
const Segment SGI347494[] = {{&S35487[12], 14}, {&S35487[37], 42}, {NULL}};
const Segment SGI347495[] = {{&S35487[12], 14}, {NULL}};
const Segment SGI347496[] = {{&S35488[6], 4}, {NULL}};
const Segment SGI347497[] = {{&S35488[75], 5}, {NULL}};
const Segment SGI347498[] = {{&S35488[12], 14}, {&S35488[38], 42}, {NULL}};
const Segment SGI347499[] = {{&S35488[12], 14}, {NULL}};
const Segment SGI347500[] = {{&S35489[6], 4}, {NULL}};
const Segment SGI347501[] = {{&S35489[74], 5}, {NULL}};
const Segment SGI347502[] = {{&S35489[12], 14}, {&S35489[37], 42}, {NULL}};
const Segment SGI347503[] = {{&S35489[12], 14}, {NULL}};
const Segment SGI347504[] = {{&S35492[6], 4}, {NULL}};
const Segment SGI347505[] = {{&S35492[75], 5}, {NULL}};
const Segment SGI347506[] = {{&S35492[12], 14}, {&S35492[37], 43}, {NULL}};
const Segment SGI347507[] = {{&S35492[12], 14}, {NULL}};
const Segment SGI347508[] = {{&S35493[6], 4}, {NULL}};
const Segment SGI347509[] = {{&S35493[74], 5}, {NULL}};
const Segment SGI347510[] = {{&S35493[12], 14}, {&S35493[37], 42}, {NULL}};
const Segment SGI347511[] = {{&S35493[12], 14}, {NULL}};
const Segment SGI347512[] = {{&S35494[6], 4}, {NULL}};
const Segment SGI347513[] = {{&S35494[74], 5}, {NULL}};
const Segment SGI347514[] = {{&S35494[12], 14}, {&S35494[37], 42}, {NULL}};
const Segment SGI347515[] = {{&S35494[12], 14}, {NULL}};
const Segment SGI347516[] = {{&S35495[6], 4}, {NULL}};
const Segment SGI347517[] = {{&S35495[74], 5}, {NULL}};
const Segment SGI347518[] = {{&S35495[12], 14}, {&S35495[37], 42}, {NULL}};
const Segment SGI347519[] = {{&S35495[12], 14}, {NULL}};
const Segment SGI347520[] = {{&S35497[6], 4}, {NULL}};
const Segment SGI347521[] = {{&S35497[74], 5}, {NULL}};
const Segment SGI347522[] = {{&S35497[12], 14}, {&S35497[37], 42}, {NULL}};
const Segment SGI347523[] = {{&S35497[12], 14}, {NULL}};
const Segment SGI347524[] = {{&S35498[6], 4}, {NULL}};
const Segment SGI347525[] = {{&S35498[74], 5}, {NULL}};
const Segment SGI347526[] = {{&S35498[12], 14}, {&S35498[37], 42}, {NULL}};
const Segment SGI347527[] = {{&S35498[12], 14}, {NULL}};
const Segment SGI347528[] = {{&S35499[6], 4}, {NULL}};
const Segment SGI347529[] = {{&S35499[74], 5}, {NULL}};
const Segment SGI347530[] = {{&S35499[12], 14}, {&S35499[37], 42}, {NULL}};
const Segment SGI347531[] = {{&S35499[12], 14}, {NULL}};
const Segment SGI347532[] = {{&S35500[6], 4}, {NULL}};
const Segment SGI347533[] = {{&S35500[90], 5}, {NULL}};
const Segment SGI347534[] = {{&S35500[12], 14}, {NULL}};
const Segment SGI347535[] = {{&S35500[12], 14}, {&S35500[50], 14}, {NULL}};
const Segment SGI347536[] = {{&S35501[6], 4}, {NULL}};
const Segment SGI347537[] = {{&S35501[74], 5}, {NULL}};
const Segment SGI347538[] = {{&S35501[12], 14}, {&S35501[37], 42}, {NULL}};
const Segment SGI347539[] = {{&S35501[12], 14}, {NULL}};
const Segment SGI347540[] = {{&S35502[6], 4}, {NULL}};
const Segment SGI347541[] = {{&S35502[101], 5}, {NULL}};
const Segment SGI347542[] = {{&S35502[12], 14}, {&S35502[37], 12}, {NULL}};
const Segment SGI347543[] = {{&S35502[12], 14}, {NULL}};
const Segment SGI347544[] = {{&S35503[6], 4}, {NULL}};
const Segment SGI347545[] = {{&S35503[74], 5}, {NULL}};
const Segment SGI347546[] = {{&S35503[12], 14}, {&S35503[37], 42}, {NULL}};
const Segment SGI347547[] = {{&S35503[12], 14}, {NULL}};
const Segment SGI347548[] = {{&S35504[6], 4}, {NULL}};
const Segment SGI347549[] = {{&S35504[74], 5}, {NULL}};
const Segment SGI347550[] = {{&S35504[12], 14}, {&S35504[37], 42}, {NULL}};
const Segment SGI347551[] = {{&S35504[12], 14}, {NULL}};
const Segment SGI347552[] = {{&S35505[6], 4}, {NULL}};
const Segment SGI347553[] = {{&S35505[75], 5}, {NULL}};
const Segment SGI347554[] = {{&S35505[12], 14}, {&S35505[38], 42}, {NULL}};
const Segment SGI347555[] = {{&S35505[12], 14}, {NULL}};
const Segment SGI347556[] = {{&S35506[6], 4}, {NULL}};
const Segment SGI347557[] = {{&S35506[75], 5}, {NULL}};
const Segment SGI347558[] = {{&S35506[12], 14}, {&S35506[37], 43}, {NULL}};
const Segment SGI347559[] = {{&S35506[12], 14}, {NULL}};
const Segment SGI347560[] = {{&S35507[6], 4}, {NULL}};
const Segment SGI347561[] = {{&S35507[74], 5}, {NULL}};
const Segment SGI347562[] = {{&S35507[12], 14}, {&S35507[37], 42}, {NULL}};
const Segment SGI347563[] = {{&S35507[12], 14}, {NULL}};
const Segment SGI347564[] = {{&S35508[6], 4}, {NULL}};
const Segment SGI347565[] = {{&S35508[74], 5}, {NULL}};
const Segment SGI347566[] = {{&S35508[12], 14}, {&S35508[37], 42}, {NULL}};
const Segment SGI347567[] = {{&S35508[12], 14}, {NULL}};
const Segment SGI347568[] = {{&S35509[6], 4}, {NULL}};
const Segment SGI347569[] = {{&S35509[75], 5}, {NULL}};
const Segment SGI347570[] = {{&S35509[12], 14}, {&S35509[37], 43}, {NULL}};
const Segment SGI347571[] = {{&S35509[12], 14}, {NULL}};
const Segment SGI347572[] = {{&S35510[6], 4}, {NULL}};
const Segment SGI347573[] = {{&S35510[75], 5}, {NULL}};
const Segment SGI347574[] = {{&S35510[12], 14}, {&S35510[37], 43}, {NULL}};
const Segment SGI347575[] = {{&S35510[12], 14}, {NULL}};
const Segment SGI347576[] = {{&S35511[6], 4}, {NULL}};
const Segment SGI347577[] = {{&S35511[74], 5}, {NULL}};
const Segment SGI347578[] = {{&S35511[12], 14}, {&S35511[37], 42}, {NULL}};
const Segment SGI347579[] = {{&S35511[12], 14}, {NULL}};
const Segment SGI347580[] = {{&S61996[5], 3}, {&S61996[5], 3}, {NULL}};
const Segment SGI347581[] = {{&S61996[67], 3}, {NULL}};
const Segment SGI347582[] = {{&S61996[127], 3}, {NULL}};
const Segment SGI347583[] = {{&S62007[42], 5}, {NULL}};
const Segment SGI347584[] = {{&S62007[6], 3}, {NULL}};
const Segment SGI347585[] = {{&S62007[68], 3}, {NULL}};
const Segment SGI347586[] = {{&S62016[5], 15}, {NULL}};
const Segment SGI347587[] = {{&S62017[5], 4}, {NULL}};
const Segment SGI347588[] = {{&S53302[28], 3}, {NULL}};
const Segment SGI347589[] = {{&S53302[102], 5}, {NULL}};
const Segment SGI347590[] = {{&S35630[6], 4}, {NULL}};
const Segment SGI347591[] = {{&S35630[52], 5}, {NULL}};
const Segment SGI347592[] = {{&S35630[12], 45}, {NULL}};
const Segment SGI347593[] = {{&S35630[12], 14}, {NULL}};
const Segment SGI347594[] = {{&S35632[6], 4}, {NULL}};
const Segment SGI347595[] = {{&S35632[74], 5}, {NULL}};
const Segment SGI347596[] = {{&S35632[12], 14}, {&S35632[37], 42}, {NULL}};
const Segment SGI347597[] = {{&S35632[12], 14}, {NULL}};
const Segment SGI347598[] = {{&S35633[6], 4}, {NULL}};
const Segment SGI347599[] = {{&S35633[75], 5}, {NULL}};
const Segment SGI347600[] = {{&S35633[12], 14}, {&S35633[38], 42}, {NULL}};
const Segment SGI347601[] = {{&S35633[12], 14}, {NULL}};
const Segment SGI347602[] = {{&S35634[6], 4}, {NULL}};
const Segment SGI347603[] = {{&S35634[59], 5}, {NULL}};
const Segment SGI347604[] = {{&S35634[12], 14}, {NULL}};
const Segment SGI347605[] = {{&S35635[6], 4}, {NULL}};
const Segment SGI347606[] = {{&S35635[74], 5}, {NULL}};
const Segment SGI347607[] = {{&S35635[12], 14}, {&S35635[37], 42}, {NULL}};
const Segment SGI347608[] = {{&S35635[12], 14}, {NULL}};
const Segment SGI347609[] = {{&S35636[6], 4}, {NULL}};
const Segment SGI347610[] = {{&S35636[52], 5}, {NULL}};
const Segment SGI347611[] = {{&S35636[12], 45}, {NULL}};
const Segment SGI347612[] = {{&S35636[12], 14}, {NULL}};
const Segment SGI347613[] = {{&S35637[6], 4}, {NULL}};
const Segment SGI347614[] = {{&S35637[74], 5}, {NULL}};
const Segment SGI347615[] = {{&S35637[12], 14}, {&S35637[37], 42}, {NULL}};
const Segment SGI347616[] = {{&S35637[12], 14}, {NULL}};
const Segment SGI347617[] = {{&S35638[6], 4}, {NULL}};
const Segment SGI347618[] = {{&S35638[74], 5}, {NULL}};
const Segment SGI347619[] = {{&S35638[12], 14}, {&S35638[37], 42}, {NULL}};
const Segment SGI347620[] = {{&S35638[12], 14}, {NULL}};
const Segment SGI347621[] = {{&S35639[6], 4}, {NULL}};
const Segment SGI347622[] = {{&S35639[74], 5}, {NULL}};
const Segment SGI347623[] = {{&S35639[12], 14}, {&S35639[37], 42}, {NULL}};
const Segment SGI347624[] = {{&S35639[12], 14}, {NULL}};
const Segment SGI347625[] = {{&S35640[6], 4}, {NULL}};
const Segment SGI347626[] = {{&S35640[52], 5}, {NULL}};
const Segment SGI347627[] = {{&S35640[12], 55}, {NULL}};
const Segment SGI347628[] = {{&S35640[12], 14}, {NULL}};
const Segment SGI347629[] = {{&S35641[6], 4}, {NULL}};
const Segment SGI347630[] = {{&S35641[75], 5}, {NULL}};
const Segment SGI347631[] = {{&S35641[12], 14}, {&S35641[38], 42}, {NULL}};
const Segment SGI347632[] = {{&S35641[12], 14}, {NULL}};
const Segment SGI347633[] = {{&S35642[6], 4}, {NULL}};
const Segment SGI347634[] = {{&S35642[74], 5}, {NULL}};
const Segment SGI347635[] = {{&S35642[12], 14}, {&S35642[37], 42}, {NULL}};
const Segment SGI347636[] = {{&S35642[12], 14}, {NULL}};
const Segment SGI347637[] = {{&S35643[6], 4}, {NULL}};
const Segment SGI347638[] = {{&S35643[74], 5}, {NULL}};
const Segment SGI347639[] = {{&S35643[12], 14}, {&S35643[37], 42}, {NULL}};
const Segment SGI347640[] = {{&S35643[12], 14}, {NULL}};
const Segment SGI347641[] = {{&S35644[6], 4}, {NULL}};
const Segment SGI347642[] = {{&S35644[73], 5}, {NULL}};
const Segment SGI347643[] = {{&S35644[12], 14}, {NULL}};
const Segment SGI347644[] = {{&S35645[6], 4}, {NULL}};
const Segment SGI347645[] = {{&S35645[74], 5}, {NULL}};
const Segment SGI347646[] = {{&S35645[12], 14}, {&S35645[37], 42}, {NULL}};
const Segment SGI347647[] = {{&S35645[12], 14}, {NULL}};
const Segment SGI347648[] = {{&S35646[6], 4}, {NULL}};
const Segment SGI347649[] = {{&S35646[74], 5}, {NULL}};
const Segment SGI347650[] = {{&S35646[12], 14}, {&S35646[37], 42}, {NULL}};
const Segment SGI347651[] = {{&S35646[12], 14}, {NULL}};
const Segment SGI347652[] = {{&S35647[6], 4}, {NULL}};
const Segment SGI347653[] = {{&S35647[75], 5}, {NULL}};
const Segment SGI347654[] = {{&S35647[12], 14}, {&S35647[37], 43}, {NULL}};
const Segment SGI347655[] = {{&S35647[12], 14}, {NULL}};
const Segment SGI347656[] = {{&S35648[6], 4}, {NULL}};
const Segment SGI347657[] = {{&S35648[74], 5}, {NULL}};
const Segment SGI347658[] = {{&S35648[12], 14}, {&S35648[37], 42}, {NULL}};
const Segment SGI347659[] = {{&S35648[12], 14}, {NULL}};
const Segment SGI347660[] = {{&S35649[6], 4}, {NULL}};
const Segment SGI347661[] = {{&S35649[75], 5}, {NULL}};
const Segment SGI347662[] = {{&S35649[12], 14}, {&S35649[37], 43}, {NULL}};
const Segment SGI347663[] = {{&S35649[12], 14}, {NULL}};
const Segment SGI347664[] = {{&S35650[6], 4}, {NULL}};
const Segment SGI347665[] = {{&S35650[74], 5}, {NULL}};
const Segment SGI347666[] = {{&S35650[12], 14}, {&S35650[37], 42}, {NULL}};
const Segment SGI347667[] = {{&S35650[12], 14}, {NULL}};
const Segment SGI347668[] = {{&S35651[6], 4}, {NULL}};
const Segment SGI347669[] = {{&S35651[74], 5}, {NULL}};
const Segment SGI347670[] = {{&S35651[12], 14}, {&S35651[37], 42}, {NULL}};
const Segment SGI347671[] = {{&S35651[12], 14}, {NULL}};
const Segment SGI347672[] = {{&S35652[6], 4}, {NULL}};
const Segment SGI347673[] = {{&S35652[74], 5}, {NULL}};
const Segment SGI347674[] = {{&S35652[12], 14}, {&S35652[37], 42}, {NULL}};
const Segment SGI347675[] = {{&S35652[12], 14}, {NULL}};
const Segment SGI347676[] = {{&S35653[6], 4}, {NULL}};
const Segment SGI347677[] = {{&S35653[75], 5}, {NULL}};
const Segment SGI347678[] = {{&S35653[12], 14}, {&S35653[37], 43}, {NULL}};
const Segment SGI347679[] = {{&S35653[12], 14}, {NULL}};
const Segment SGI347680[] = {{&S35654[6], 4}, {NULL}};
const Segment SGI347681[] = {{&S35654[74], 5}, {NULL}};
const Segment SGI347682[] = {{&S35654[12], 14}, {&S35654[37], 42}, {NULL}};
const Segment SGI347683[] = {{&S35654[12], 14}, {NULL}};
const Segment SGI347684[] = {{&S35655[6], 4}, {NULL}};
const Segment SGI347685[] = {{&S35655[74], 5}, {NULL}};
const Segment SGI347686[] = {{&S35655[12], 14}, {&S35655[37], 42}, {NULL}};
const Segment SGI347687[] = {{&S35655[12], 14}, {NULL}};
const Segment SGI347688[] = {{&S35656[6], 4}, {NULL}};
const Segment SGI347689[] = {{&S35656[75], 5}, {NULL}};
const Segment SGI347690[] = {{&S35656[12], 14}, {&S35656[38], 42}, {NULL}};
const Segment SGI347691[] = {{&S35656[12], 14}, {NULL}};
const Segment SGI347692[] = {{&S35657[6], 4}, {NULL}};
const Segment SGI347693[] = {{&S35657[75], 5}, {NULL}};
const Segment SGI347694[] = {{&S35657[12], 14}, {&S35657[38], 42}, {NULL}};
const Segment SGI347695[] = {{&S35657[12], 14}, {NULL}};
const Segment SGI347696[] = {{&S35658[6], 4}, {NULL}};
const Segment SGI347697[] = {{&S35658[75], 5}, {NULL}};
const Segment SGI347698[] = {{&S35658[12], 14}, {&S35658[37], 43}, {NULL}};
const Segment SGI347699[] = {{&S35658[12], 14}, {NULL}};
const Segment SGI347700[] = {{&S35659[6], 4}, {NULL}};
const Segment SGI347701[] = {{&S35659[74], 5}, {NULL}};
const Segment SGI347702[] = {{&S35659[12], 14}, {&S35659[37], 42}, {NULL}};
const Segment SGI347703[] = {{&S35659[12], 14}, {NULL}};
const Segment SGI347704[] = {{&S48731[49], 4}, {NULL}};
const Segment SGI347705[] = {{&S48731[8], 3}, {NULL}};
const Segment SGI347706[] = {{&S35661[6], 4}, {NULL}};
const Segment SGI347707[] = {{&S35661[74], 5}, {NULL}};
const Segment SGI347708[] = {{&S35661[12], 14}, {&S35661[37], 42}, {NULL}};
const Segment SGI347709[] = {{&S35661[12], 14}, {NULL}};
const Segment SGI347710[] = {{&S35662[6], 4}, {NULL}};
const Segment SGI347711[] = {{&S35662[74], 5}, {NULL}};
const Segment SGI347712[] = {{&S35662[12], 14}, {&S35662[37], 42}, {NULL}};
const Segment SGI347713[] = {{&S35662[12], 14}, {NULL}};
const Segment SGI347714[] = {{&S35663[6], 4}, {NULL}};
const Segment SGI347715[] = {{&S35663[74], 5}, {NULL}};
const Segment SGI347716[] = {{&S35663[12], 14}, {&S35663[37], 42}, {NULL}};
const Segment SGI347717[] = {{&S35663[12], 14}, {NULL}};
const Segment SGI347718[] = {{&S35664[6], 4}, {NULL}};
const Segment SGI347719[] = {{&S35664[74], 5}, {NULL}};
const Segment SGI347720[] = {{&S35664[12], 14}, {&S35664[37], 42}, {NULL}};
const Segment SGI347721[] = {{&S35664[12], 14}, {NULL}};
const Segment SGI347722[] = {{&S35665[6], 4}, {NULL}};
const Segment SGI347723[] = {{&S35665[74], 5}, {NULL}};
const Segment SGI347724[] = {{&S35665[12], 14}, {&S35665[37], 42}, {NULL}};
const Segment SGI347725[] = {{&S35665[12], 14}, {NULL}};
const Segment SGI347726[] = {{&S35666[6], 4}, {NULL}};
const Segment SGI347727[] = {{&S35666[74], 5}, {NULL}};
const Segment SGI347728[] = {{&S35666[12], 14}, {&S35666[37], 42}, {NULL}};
const Segment SGI347729[] = {{&S35666[12], 14}, {NULL}};
const Segment SGI347730[] = {{&S35667[6], 4}, {NULL}};
const Segment SGI347731[] = {{&S35667[75], 5}, {NULL}};
const Segment SGI347732[] = {{&S35667[12], 14}, {&S35667[37], 43}, {NULL}};
const Segment SGI347733[] = {{&S35667[12], 14}, {NULL}};
const Segment SGI347734[] = {{&S35668[6], 4}, {NULL}};
const Segment SGI347735[] = {{&S35668[74], 5}, {NULL}};
const Segment SGI347736[] = {{&S35668[12], 14}, {&S35668[37], 42}, {NULL}};
const Segment SGI347737[] = {{&S35668[12], 14}, {NULL}};
const Segment SGI347738[] = {{&S35669[6], 4}, {NULL}};
const Segment SGI347739[] = {{&S35669[74], 5}, {NULL}};
const Segment SGI347740[] = {{&S35669[12], 14}, {&S35669[37], 42}, {NULL}};
const Segment SGI347741[] = {{&S35669[12], 14}, {NULL}};
const Segment SGI347742[] = {{&S35670[6], 4}, {NULL}};
const Segment SGI347743[] = {{&S35670[74], 5}, {NULL}};
const Segment SGI347744[] = {{&S35670[12], 14}, {&S35670[37], 42}, {NULL}};
const Segment SGI347745[] = {{&S35670[12], 14}, {NULL}};
const Segment SGI347746[] = {{&S35671[6], 4}, {NULL}};
const Segment SGI347747[] = {{&S35671[74], 5}, {NULL}};
const Segment SGI347748[] = {{&S35671[12], 14}, {&S35671[37], 42}, {NULL}};
const Segment SGI347749[] = {{&S35671[12], 14}, {NULL}};
const Segment SGI347750[] = {{&S35672[6], 4}, {NULL}};
const Segment SGI347751[] = {{&S35672[74], 5}, {NULL}};
const Segment SGI347752[] = {{&S35672[12], 14}, {&S35672[37], 42}, {NULL}};
const Segment SGI347753[] = {{&S35672[12], 14}, {NULL}};
const Segment SGI347754[] = {{&S35673[6], 4}, {NULL}};
const Segment SGI347755[] = {{&S35673[74], 5}, {NULL}};
const Segment SGI347756[] = {{&S35673[12], 14}, {&S35673[37], 42}, {NULL}};
const Segment SGI347757[] = {{&S35673[12], 14}, {NULL}};
const Segment SGI347758[] = {{&S35674[6], 4}, {NULL}};
const Segment SGI347759[] = {{&S35674[74], 5}, {NULL}};
const Segment SGI347760[] = {{&S35674[12], 14}, {&S35674[37], 42}, {NULL}};
const Segment SGI347761[] = {{&S35674[12], 14}, {NULL}};
const Segment SGI347762[] = {{&S35675[6], 4}, {NULL}};
const Segment SGI347763[] = {{&S35675[75], 5}, {NULL}};
const Segment SGI347764[] = {{&S35675[12], 14}, {&S35675[38], 42}, {NULL}};
const Segment SGI347765[] = {{&S35675[12], 14}, {NULL}};
const Segment SGI347766[] = {{&S35676[6], 4}, {NULL}};
const Segment SGI347767[] = {{&S35676[75], 5}, {NULL}};
const Segment SGI347768[] = {{&S35676[12], 14}, {&S35676[38], 42}, {NULL}};
const Segment SGI347769[] = {{&S35676[12], 14}, {NULL}};
const Segment SGI347770[] = {{&S35677[6], 4}, {NULL}};
const Segment SGI347771[] = {{&S35677[52], 5}, {NULL}};
const Segment SGI347772[] = {{&S35677[12], 62}, {NULL}};
const Segment SGI347773[] = {{&S35677[12], 14}, {NULL}};
const Segment SGI347774[] = {{&S35678[6], 4}, {NULL}};
const Segment SGI347775[] = {{&S35678[75], 5}, {NULL}};
const Segment SGI347776[] = {{&S35678[12], 14}, {&S35678[37], 43}, {NULL}};
const Segment SGI347777[] = {{&S35678[12], 14}, {NULL}};
const Segment SGI347778[] = {{&S35679[6], 4}, {NULL}};
const Segment SGI347779[] = {{&S35679[74], 5}, {NULL}};
const Segment SGI347780[] = {{&S35679[12], 14}, {&S35679[37], 42}, {NULL}};
const Segment SGI347781[] = {{&S35679[12], 14}, {NULL}};
const Segment SGI347782[] = {{&S35680[6], 4}, {NULL}};
const Segment SGI347783[] = {{&S35680[74], 5}, {NULL}};
const Segment SGI347784[] = {{&S35680[12], 14}, {&S35680[37], 42}, {NULL}};
const Segment SGI347785[] = {{&S35680[12], 14}, {NULL}};
const Segment SGI347786[] = {{&S35681[6], 4}, {NULL}};
const Segment SGI347787[] = {{&S35681[75], 5}, {NULL}};
const Segment SGI347788[] = {{&S35681[12], 14}, {&S35681[38], 42}, {NULL}};
const Segment SGI347789[] = {{&S35681[12], 14}, {NULL}};
const Segment SGI347790[] = {{&S35682[6], 4}, {NULL}};
const Segment SGI347791[] = {{&S35682[75], 5}, {NULL}};
const Segment SGI347792[] = {{&S35682[12], 14}, {&S35682[37], 43}, {NULL}};
const Segment SGI347793[] = {{&S35682[12], 14}, {NULL}};
const Segment SGI347794[] = {{&S35683[6], 4}, {NULL}};
const Segment SGI347795[] = {{&S35683[74], 5}, {NULL}};
const Segment SGI347796[] = {{&S35683[12], 14}, {&S35683[37], 42}, {NULL}};
const Segment SGI347797[] = {{&S35683[12], 14}, {NULL}};
const Segment SGI347798[] = {{&S35684[6], 4}, {NULL}};
const Segment SGI347799[] = {{&S35684[74], 5}, {NULL}};
const Segment SGI347800[] = {{&S35684[12], 14}, {&S35684[37], 42}, {NULL}};
const Segment SGI347801[] = {{&S35684[12], 14}, {NULL}};
const Segment SGI347802[] = {{&S35685[6], 4}, {NULL}};
const Segment SGI347803[] = {{&S35685[74], 5}, {NULL}};
const Segment SGI347804[] = {{&S35685[12], 14}, {&S35685[37], 42}, {NULL}};
const Segment SGI347805[] = {{&S35685[12], 14}, {NULL}};
const Segment SGI347806[] = {{&S35686[6], 4}, {NULL}};
const Segment SGI347807[] = {{&S35686[101], 5}, {NULL}};
const Segment SGI347808[] = {{&S35686[12], 14}, {&S35686[37], 12}, {NULL}};
const Segment SGI347809[] = {{&S35686[12], 14}, {NULL}};
const Segment SGI347810[] = {{&S35687[6], 4}, {NULL}};
const Segment SGI347811[] = {{&S35687[75], 5}, {NULL}};
const Segment SGI347812[] = {{&S35687[12], 14}, {&S35687[38], 42}, {NULL}};
const Segment SGI347813[] = {{&S35687[12], 14}, {NULL}};
const Segment SGI347814[] = {{&S62072[5], 5}, {&S62072[97], 5}, {NULL}};
const Segment SGI347815[] = {{&S35688[6], 4}, {NULL}};
const Segment SGI347816[] = {{&S35688[75], 5}, {NULL}};
const Segment SGI347817[] = {{&S35688[12], 14}, {&S35688[38], 42}, {NULL}};
const Segment SGI347818[] = {{&S35688[12], 14}, {NULL}};
const Segment SGI347819[] = {{&S62072[113], 3}, {NULL}};
const Segment SGI347820[] = {{&S35689[6], 4}, {NULL}};
const Segment SGI347821[] = {{&S35689[75], 5}, {NULL}};
const Segment SGI347822[] = {{&S35689[12], 14}, {&S35689[38], 42}, {NULL}};
const Segment SGI347823[] = {{&S35689[12], 14}, {NULL}};
const Segment SGI347824[] = {{&S35690[6], 4}, {NULL}};
const Segment SGI347825[] = {{&S35690[75], 5}, {NULL}};
const Segment SGI347826[] = {{&S35690[12], 14}, {&S35690[37], 43}, {NULL}};
const Segment SGI347827[] = {{&S35690[12], 14}, {NULL}};
const Segment SGI347828[] = {{&S37938[61], 5}, {NULL}};
const Segment SGI347829[] = {{&S37938[13], 10}, {NULL}};
const Segment SGI347830[] = {{&S35691[6], 4}, {NULL}};
const Segment SGI347831[] = {{&S35691[74], 5}, {NULL}};
const Segment SGI347832[] = {{&S35691[12], 14}, {&S35691[37], 42}, {NULL}};
const Segment SGI347833[] = {{&S35691[12], 14}, {NULL}};
const Segment SGI347834[] = {{&S35692[6], 4}, {NULL}};
const Segment SGI347835[] = {{&S35692[74], 5}, {NULL}};
const Segment SGI347836[] = {{&S35692[12], 14}, {&S35692[37], 42}, {NULL}};
const Segment SGI347837[] = {{&S35692[12], 14}, {NULL}};
const Segment SGI347838[] = {{&S35693[6], 4}, {NULL}};
const Segment SGI347839[] = {{&S35693[74], 5}, {NULL}};
const Segment SGI347840[] = {{&S35693[12], 14}, {&S35693[37], 42}, {NULL}};
const Segment SGI347841[] = {{&S35693[12], 14}, {NULL}};
const Segment SGI347842[] = {{&S35694[6], 4}, {NULL}};
const Segment SGI347843[] = {{&S35694[74], 5}, {NULL}};
const Segment SGI347844[] = {{&S35694[12], 14}, {&S35694[37], 42}, {NULL}};
const Segment SGI347845[] = {{&S35694[12], 14}, {NULL}};
const Segment SGI347846[] = {{&S35695[6], 4}, {NULL}};
const Segment SGI347847[] = {{&S35695[74], 5}, {NULL}};
const Segment SGI347848[] = {{&S35695[12], 14}, {&S35695[37], 42}, {NULL}};
const Segment SGI347849[] = {{&S35695[12], 14}, {NULL}};
const Segment SGI347850[] = {{&S35696[6], 4}, {NULL}};
const Segment SGI347851[] = {{&S35696[74], 5}, {NULL}};
const Segment SGI347852[] = {{&S35696[12], 14}, {&S35696[37], 42}, {NULL}};
const Segment SGI347853[] = {{&S35696[12], 14}, {NULL}};
const Segment SGI347854[] = {{&S35697[6], 4}, {NULL}};
const Segment SGI347855[] = {{&S35697[74], 5}, {NULL}};
const Segment SGI347856[] = {{&S35697[12], 14}, {&S35697[37], 42}, {NULL}};
const Segment SGI347857[] = {{&S35697[12], 14}, {NULL}};
const Segment SGI347858[] = {{&S35698[6], 4}, {NULL}};
const Segment SGI347859[] = {{&S35698[75], 5}, {NULL}};
const Segment SGI347860[] = {{&S35698[12], 14}, {&S35698[37], 43}, {NULL}};
const Segment SGI347861[] = {{&S35698[12], 14}, {NULL}};
const Segment SGI347862[] = {{&S35699[6], 4}, {NULL}};
const Segment SGI347863[] = {{&S35699[74], 5}, {NULL}};
const Segment SGI347864[] = {{&S35699[12], 14}, {&S35699[37], 42}, {NULL}};
const Segment SGI347865[] = {{&S35699[12], 14}, {NULL}};
const Segment SGI347866[] = {{&S35700[6], 4}, {NULL}};
const Segment SGI347867[] = {{&S35700[74], 5}, {NULL}};
const Segment SGI347868[] = {{&S35700[12], 14}, {&S35700[37], 42}, {NULL}};
const Segment SGI347869[] = {{&S35700[12], 14}, {NULL}};
const Segment SGI347870[] = {{&S35701[6], 4}, {NULL}};
const Segment SGI347871[] = {{&S35701[74], 5}, {NULL}};
const Segment SGI347872[] = {{&S35701[12], 14}, {&S35701[37], 42}, {NULL}};
const Segment SGI347873[] = {{&S35701[12], 14}, {NULL}};
const Segment SGI347874[] = {{&S35702[6], 4}, {NULL}};
const Segment SGI347875[] = {{&S35702[75], 5}, {NULL}};
const Segment SGI347876[] = {{&S35702[12], 14}, {&S35702[38], 42}, {NULL}};
const Segment SGI347877[] = {{&S35702[12], 14}, {NULL}};
const Segment SGI347878[] = {{&S35703[6], 4}, {NULL}};
const Segment SGI347879[] = {{&S35703[74], 5}, {NULL}};
const Segment SGI347880[] = {{&S35703[12], 14}, {&S35703[37], 42}, {NULL}};
const Segment SGI347881[] = {{&S35703[12], 14}, {NULL}};
const Segment SGI347882[] = {{&S35704[6], 4}, {NULL}};
const Segment SGI347883[] = {{&S35704[74], 5}, {NULL}};
const Segment SGI347884[] = {{&S35704[12], 14}, {&S35704[37], 42}, {NULL}};
const Segment SGI347885[] = {{&S35704[12], 14}, {NULL}};
const Segment SGI347886[] = {{&S35705[6], 4}, {NULL}};
const Segment SGI347887[] = {{&S35705[74], 5}, {NULL}};
const Segment SGI347888[] = {{&S35705[12], 14}, {&S35705[37], 42}, {NULL}};
const Segment SGI347889[] = {{&S35705[12], 14}, {NULL}};
const Segment SGI347890[] = {{&S35706[6], 4}, {NULL}};
const Segment SGI347891[] = {{&S35706[74], 5}, {NULL}};
const Segment SGI347892[] = {{&S35706[12], 14}, {&S35706[37], 42}, {NULL}};
const Segment SGI347893[] = {{&S35706[12], 14}, {NULL}};
const Segment SGI347894[] = {{&S35707[6], 4}, {NULL}};
const Segment SGI347895[] = {{&S35707[74], 5}, {NULL}};
const Segment SGI347896[] = {{&S35707[12], 14}, {&S35707[37], 42}, {NULL}};
const Segment SGI347897[] = {{&S35707[12], 14}, {NULL}};
const Segment SGI347898[] = {{&S62096[5], 16}, {NULL}};
const Segment SGI347899[] = {{&S62096[27], 3}, {NULL}};
const Segment SGI347900[] = {{&S35708[6], 4}, {NULL}};
const Segment SGI347901[] = {{&S35708[74], 5}, {NULL}};
const Segment SGI347902[] = {{&S35708[12], 14}, {&S35708[37], 42}, {NULL}};
const Segment SGI347903[] = {{&S35708[12], 14}, {NULL}};
const Segment SGI347904[] = {{&S62097[35], 16}, {NULL}};
const Segment SGI347905[] = {{&S62097[63], 3}, {NULL}};
const Segment SGI347906[] = {{&S35709[6], 4}, {NULL}};
const Segment SGI347907[] = {{&S35709[74], 5}, {NULL}};
const Segment SGI347908[] = {{&S35709[12], 14}, {&S35709[37], 42}, {NULL}};
const Segment SGI347909[] = {{&S35709[12], 14}, {NULL}};
const Segment SGI347910[] = {{&S62098[34], 7}, {NULL}};
const Segment SGI347911[] = {{&S35710[6], 4}, {NULL}};
const Segment SGI347912[] = {{&S35710[74], 5}, {NULL}};
const Segment SGI347913[] = {{&S35710[12], 14}, {&S35710[37], 42}, {NULL}};
const Segment SGI347914[] = {{&S62098[92], 3}, {NULL}};
const Segment SGI347915[] = {{&S35710[12], 14}, {NULL}};
const Segment SGI347916[] = {{&S35178[0], 21}, {NULL}};
const Segment SGI347917[] = {{&S35178[34], 3}, {NULL}};
const Segment SGI347918[] = {{&S62152[5], 3}, {NULL}};
const Segment SGI347919[] = {{&S62153[5], 5}, {NULL}};
const Segment SGI347920[] = {{&S62154[5], 4}, {NULL}};
const Segment SGI347921[] = {{&S62155[5], 5}, {NULL}};
const Segment SGI347922[] = {{&S53426[28], 3}, {NULL}};
const Segment SGI347923[] = {{&S53426[102], 5}, {NULL}};
const Segment SGI347924[] = {{&S53436[28], 3}, {NULL}};
const Segment SGI347925[] = {{&S53436[102], 5}, {NULL}};
const Segment SGI347926[] = {{&S53437[28], 3}, {NULL}};
const Segment SGI347927[] = {{&S53437[102], 5}, {NULL}};
const Segment SGI347928[] = {{&S62224[5], 16}, {NULL}};
const Segment SGI347929[] = {{&S62224[27], 3}, {NULL}};
const Segment SGI347930[] = {{&S62225[5], 16}, {NULL}};
const Segment SGI347931[] = {{&S62225[32], 3}, {NULL}};
const Segment SGI347932[] = {{&S62226[5], 17}, {NULL}};
const Segment SGI347933[] = {{&S62226[28], 3}, {NULL}};
const Segment SGI347934[] = {{&S38179[53], 4}, {NULL}};
const Segment SGI347935[] = {{&S38179[13], 10}, {NULL}};
const Segment SGI347936[] = {{&S53609[28], 5}, {NULL}};
const Segment SGI347937[] = {{&S53609[104], 5}, {NULL}};
const Segment SGI347938[] = {{&S38181[53], 4}, {NULL}};
const Segment SGI347939[] = {{&S38181[13], 10}, {NULL}};
const Segment SGI347940[] = {{&S38182[63], 4}, {NULL}};
const Segment SGI347941[] = {{&S38182[13], 10}, {NULL}};
const Segment SGI347942[] = {{&S38183[53], 4}, {NULL}};
const Segment SGI347943[] = {{&S38183[13], 10}, {NULL}};
const Segment SGI347944[] = {{&S38184[63], 4}, {NULL}};
const Segment SGI347945[] = {{&S38184[13], 10}, {NULL}};
const Segment SGI347946[] = {{&S38185[6], 3}, {&S38185[26], 4}, {NULL}};
const Segment SGI347947[] = {{&S38185[11], 31}, {NULL}};
const Segment SGI347948[] = {{&S38186[53], 4}, {NULL}};
const Segment SGI347949[] = {{&S38186[13], 10}, {NULL}};
const Segment SGI347950[] = {{&S62276[5], 4}, {&S62276[5], 4}, {NULL}};
const Segment SGI347951[] = {{&S62276[178], 4}, {NULL}};
const Segment SGI347952[] = {{&S62276[147], 13}, {NULL}};
const Segment SGI347953[] = {{&S62288[5], 3}, {&S62288[5], 3}, {NULL}};
const Segment SGI347954[] = {{&S62288[9], 3}, {NULL}};
const Segment SGI347955[] = {{&S62288[126], 3}, {NULL}};
const Segment SGI347956[] = {{&S62296[5], 3}, {&S62296[5], 3}, {NULL}};
const Segment SGI347957[] = {{&S62296[67], 3}, {NULL}};
const Segment SGI347958[] = {{&S62296[127], 3}, {NULL}};
const Segment SGI347959[] = {{&S62335[5], 4}, {NULL}};
const Segment SGI347960[] = {{&S62335[15], 3}, {NULL}};
const Segment SGI347961[] = {{&S62336[42], 4}, {NULL}};
const Segment SGI347962[] = {{&S62336[6], 3}, {NULL}};
const Segment SGI347963[] = {{&S62336[67], 3}, {NULL}};
const Segment SGI347964[] = {{&S53734[28], 3}, {NULL}};
const Segment SGI347965[] = {{&S53734[102], 5}, {NULL}};
const Segment SGI347966[] = {{&S53741[28], 5}, {NULL}};
const Segment SGI347967[] = {{&S53741[104], 5}, {NULL}};
const Segment SGI347968[] = {{&S53742[28], 7}, {NULL}};
const Segment SGI347969[] = {{&S53742[106], 5}, {NULL}};
const Segment SGI347970[] = {{&S62388[5], 5}, {&S62388[97], 5}, {NULL}};
const Segment SGI347971[] = {{&S62388[153], 3}, {NULL}};
const Segment SGI347972[] = {{&S62402[49], 20}, {NULL}};
const Segment SGI347973[] = {{&S62403[49], 20}, {NULL}};
const Segment SGI347974[] = {{&S62404[49], 20}, {NULL}};
const Segment SGI347975[] = {{&S62405[49], 20}, {NULL}};
const Segment SGI347976[] = {{&S62406[49], 20}, {NULL}};
const Segment SGI347977[] = {{&S62407[42], 5}, {NULL}};
const Segment SGI347978[] = {{&S62407[6], 3}, {NULL}};
const Segment SGI347979[] = {{&S62407[68], 3}, {NULL}};
const Segment SGI347980[] = {{&S62408[43], 3}, {NULL}};
const Segment SGI347981[] = {{&S62408[6], 3}, {NULL}};
const Segment SGI347982[] = {{&S62408[67], 3}, {NULL}};
const Segment SGI347983[] = {{&S62409[43], 4}, {NULL}};
const Segment SGI347984[] = {{&S62409[6], 3}, {NULL}};
const Segment SGI347985[] = {{&S62409[68], 3}, {NULL}};
const Segment SGI347986[] = {{&S62415[35], 5}, {NULL}};
const Segment SGI347987[] = {{&S62415[91], 3}, {NULL}};
const Segment SGI347988[] = {{&S35613[0], 21}, {NULL}};
const Segment SGI347989[] = {{&S35613[34], 3}, {NULL}};
const Segment SGI347990[] = {{&S35613[73], 13}, {NULL}};
const Segment SGI347991[] = {{&S35614[0], 21}, {NULL}};
const Segment SGI347992[] = {{&S35614[34], 3}, {NULL}};
const Segment SGI347993[] = {{&S35614[75], 13}, {NULL}};
const Segment SGI347994[] = {{&S62468[35], 5}, {NULL}};
const Segment SGI347995[] = {{&S62468[52], 3}, {NULL}};
const Segment SGI347996[] = {{&S62469[35], 7}, {NULL}};
const Segment SGI347997[] = {{&S62469[54], 3}, {NULL}};
const Segment SGI347998[] = {{&S62470[34], 5}, {NULL}};
const Segment SGI347999[] = {{&S62470[90], 3}, {NULL}};
const Segment SGI348000[] = {{&S36081[6], 4}, {NULL}};
const Segment SGI348001[] = {{&S36081[12], 14}, {NULL}};
const Segment SGI348002[] = {{&S36083[6], 4}, {NULL}};
const Segment SGI348003[] = {{&S36083[12], 14}, {NULL}};
const Segment SGI348004[] = {{&S36085[6], 4}, {NULL}};
const Segment SGI348005[] = {{&S36085[12], 14}, {NULL}};
const Segment SGI348006[] = {{&S36092[6], 4}, {NULL}};
const Segment SGI348007[] = {{&S36092[12], 14}, {NULL}};
const Segment SGI348008[] = {{&S36093[6], 4}, {NULL}};
const Segment SGI348009[] = {{&S36093[12], 14}, {NULL}};
const Segment SGI348010[] = {{&S36094[6], 4}, {NULL}};
const Segment SGI348011[] = {{&S36094[12], 14}, {&S36094[38], 24}, {NULL}};
const Segment SGI348012[] = {{&S36094[12], 14}, {NULL}};
const Segment SGI348013[] = {{&S36099[6], 4}, {NULL}};
const Segment SGI348014[] = {{&S36099[12], 14}, {NULL}};
const Segment SGI348015[] = {{&S36101[6], 4}, {NULL}};
const Segment SGI348016[] = {{&S36101[12], 14}, {NULL}};
const Segment SGI348017[] = {{&S62529[43], 4}, {NULL}};
const Segment SGI348018[] = {{&S62529[6], 3}, {NULL}};
const Segment SGI348019[] = {{&S62529[68], 3}, {NULL}};
const Segment SGI348020[] = {{&S62530[43], 6}, {NULL}};
const Segment SGI348021[] = {{&S62530[6], 3}, {NULL}};
const Segment SGI348022[] = {{&S62530[70], 3}, {NULL}};
const Segment SGI348023[] = {{&S36141[6], 4}, {NULL}};
const Segment SGI348024[] = {{&S36141[59], 5}, {NULL}};
const Segment SGI348025[] = {{&S36141[12], 14}, {NULL}};
const Segment SGI348026[] = {{&S36142[6], 4}, {NULL}};
const Segment SGI348027[] = {{&S36142[88], 5}, {NULL}};
const Segment SGI348028[] = {{&S36142[12], 14}, {&S36142[47], 11}, {NULL}};
const Segment SGI348029[] = {{&S36142[12], 14}, {NULL}};
const Segment SGI348030[] = {{&S36143[6], 4}, {NULL}};
const Segment SGI348031[] = {{&S36143[88], 5}, {NULL}};
const Segment SGI348032[] = {{&S36143[12], 14}, {&S36143[47], 11}, {NULL}};
const Segment SGI348033[] = {{&S36143[12], 14}, {NULL}};
const Segment SGI348034[] = {{&S36144[6], 4}, {NULL}};
const Segment SGI348035[] = {{&S36144[88], 5}, {NULL}};
const Segment SGI348036[] = {{&S36144[12], 14}, {&S36144[47], 11}, {NULL}};
const Segment SGI348037[] = {{&S36144[12], 14}, {NULL}};
const Segment SGI348038[] = {{&S36145[6], 4}, {NULL}};
const Segment SGI348039[] = {{&S36145[88], 5}, {NULL}};
const Segment SGI348040[] = {{&S36145[12], 14}, {&S36145[47], 11}, {NULL}};
const Segment SGI348041[] = {{&S36145[12], 14}, {NULL}};
const Segment SGI348042[] = {{&S36146[6], 4}, {NULL}};
const Segment SGI348043[] = {{&S36146[88], 5}, {NULL}};
const Segment SGI348044[] = {{&S36146[12], 14}, {&S36146[47], 11}, {NULL}};
const Segment SGI348045[] = {{&S36146[12], 14}, {NULL}};
const Segment SGI348046[] = {{&S36147[6], 4}, {NULL}};
const Segment SGI348047[] = {{&S36147[88], 5}, {NULL}};
const Segment SGI348048[] = {{&S36147[12], 14}, {&S36147[47], 11}, {NULL}};
const Segment SGI348049[] = {{&S36147[12], 14}, {NULL}};
const Segment SGI348050[] = {{&S62583[35], 3}, {NULL}};
const Segment SGI348051[] = {{&S62583[89], 3}, {NULL}};
const Segment SGI348052[] = {{&S62584[5], 12}, {NULL}};
const Segment SGI348053[] = {{&S62584[18], 3}, {NULL}};
const Segment SGI348054[] = {{&S62585[41], 5}, {NULL}};
const Segment SGI348055[] = {{&S62585[6], 3}, {NULL}};
const Segment SGI348056[] = {{&S62585[67], 3}, {NULL}};
const Segment SGI348057[] = {{&S62671[35], 5}, {NULL}};
const Segment SGI348058[] = {{&S62671[91], 3}, {NULL}};
const Segment SGI348059[] = {{&S62672[5], 5}, {NULL}};
const Segment SGI348060[] = {{&S62672[16], 3}, {NULL}};
const Segment SGI348061[] = {{&S62673[34], 5}, {NULL}};
const Segment SGI348062[] = {{&S62673[90], 3}, {NULL}};
const Segment SGI348063[] = {{&S62678[5], 13}, {NULL}};
const Segment SGI348064[] = {{&S62679[5], 5}, {NULL}};
const Segment SGI348065[] = {{&S38655[67], 3}, {NULL}};
const Segment SGI348066[] = {{&S38655[13], 10}, {NULL}};
const Segment SGI348067[] = {{&S38657[57], 4}, {NULL}};
const Segment SGI348068[] = {{&S38657[13], 10}, {NULL}};
const Segment SGI348069[] = {{&S38659[57], 4}, {NULL}};
const Segment SGI348070[] = {{&S38659[13], 10}, {NULL}};
const Segment SGI348071[] = {{&S54026[28], 3}, {NULL}};
const Segment SGI348072[] = {{&S54026[102], 5}, {NULL}};
const Segment SGI348073[] = {{&S38728[55], 3}, {NULL}};
const Segment SGI348074[] = {{&S38728[13], 10}, {NULL}};
const Segment SGI348075[] = {{&S38730[55], 4}, {NULL}};
const Segment SGI348076[] = {{&S38730[13], 10}, {NULL}};
const Segment SGI348077[] = {{&S38731[36], 4}, {NULL}};
const Segment SGI348078[] = {{&S38731[13], 12}, {NULL}};
const Segment SGI348079[] = {{&S38733[37], 4}, {NULL}};
const Segment SGI348080[] = {{&S38733[14], 12}, {NULL}};
const Segment SGI348081[] = {{&S62687[5], 4}, {&S62687[5], 4}, {NULL}};
const Segment SGI348082[] = {{&S62687[10], 3}, {NULL}};
const Segment SGI348083[] = {{&S62687[129], 3}, {NULL}};
const Segment SGI348084[] = {{&S62703[38], 5}, {NULL}};
const Segment SGI348085[] = {{&S62703[94], 3}, {NULL}};
const Segment SGI348086[] = {{&S62704[5], 5}, {NULL}};
const Segment SGI348087[] = {{&S62704[16], 3}, {NULL}};
const Segment SGI348088[] = {{&S54109[28], 3}, {NULL}};
const Segment SGI348089[] = {{&S54109[102], 5}, {NULL}};
const Segment SGI348090[] = {{&S62705[34], 5}, {NULL}};
const Segment SGI348091[] = {{&S62705[90], 3}, {NULL}};
const Segment SGI348092[] = {{&S62763[5], 7}, {&S62763[5], 7}, {NULL}};
const Segment SGI348093[] = {{&S62763[117], 3}, {NULL}};
const Segment SGI348094[] = {{&S62764[5], 4}, {&S62764[5], 4}, {NULL}};
const Segment SGI348095[] = {{&S62764[111], 3}, {NULL}};
const Segment SGI348096[] = {{&S62779[5], 4}, {NULL}};
const Segment SGI348097[] = {{&S62779[15], 3}, {NULL}};
const Segment SGI348098[] = {{&S62780[5], 4}, {NULL}};
const Segment SGI348099[] = {{&S62780[20], 3}, {NULL}};
const Segment SGI348100[] = {{&S49769[42], 14}, {&S49769[134], 7}, {NULL}};
const Segment SGI348101[] = {{&S49769[33], 3}, {NULL}};
const Segment SGI348102[] = {{&S49770[42], 5}, {&S49770[147], 7}, {NULL}};
const Segment SGI348103[] = {{&S49770[33], 3}, {NULL}};
const Segment SGI348104[] = {{&S62795[35], 5}, {NULL}};
const Segment SGI348105[] = {{&S62795[52], 3}, {NULL}};
const Segment SGI348106[] = {{&S62796[35], 7}, {NULL}};
const Segment SGI348107[] = {{&S62796[54], 3}, {NULL}};
const Segment SGI348108[] = {{&S62797[34], 5}, {NULL}};
const Segment SGI348109[] = {{&S62797[90], 3}, {NULL}};
const Segment SGI348110[] = {{&S62798[5], 5}, {NULL}};
const Segment SGI348111[] = {{&S62798[16], 3}, {NULL}};
const Segment SGI348112[] = {{&S62799[5], 5}, {&S62799[97], 5}, {NULL}};
const Segment SGI348113[] = {{&S62799[113], 3}, {NULL}};
const Segment SGI348114[] = {{&S54196[28], 3}, {NULL}};
const Segment SGI348115[] = {{&S54196[102], 5}, {NULL}};
const Segment SGI348116[] = {{&S54197[28], 3}, {NULL}};
const Segment SGI348117[] = {{&S54197[102], 5}, {NULL}};
const Segment SGI348118[] = {{&S62868[35], 4}, {NULL}};
const Segment SGI348119[] = {{&S62868[51], 3}, {NULL}};
const Segment SGI348120[] = {{&S62869[5], 4}, {NULL}};
const Segment SGI348121[] = {{&S62869[15], 3}, {NULL}};
const Segment SGI348122[] = {{&S49877[5], 13}, {NULL}};
const Segment SGI348123[] = {{&S49877[32], 3}, {NULL}};
const Segment SGI348124[] = {{&S62870[35], 6}, {NULL}};
const Segment SGI348125[] = {{&S62870[53], 3}, {NULL}};
const Segment SGI348126[] = {{&S49878[49], 5}, {NULL}};
const Segment SGI348127[] = {{&S49878[40], 3}, {NULL}};
const Segment SGI348128[] = {{&S49879[5], 3}, {NULL}};
const Segment SGI348129[] = {{&S49879[33], 3}, {NULL}};
const Segment SGI348130[] = {{&S49880[49], 5}, {NULL}};
const Segment SGI348131[] = {{&S49880[40], 3}, {NULL}};
const Segment SGI348132[] = {{&S49882[5], 10}, {NULL}};
const Segment SGI348133[] = {{&S49882[39], 3}, {NULL}};
const Segment SGI348134[] = {{&S62953[49], 20}, {NULL}};
const Segment SGI348135[] = {{&S62954[49], 20}, {NULL}};
const Segment SGI348136[] = {{&S62955[49], 20}, {NULL}};
const Segment SGI348137[] = {{&S62956[49], 20}, {NULL}};
const Segment SGI348138[] = {{&S62957[49], 20}, {NULL}};
const Segment SGI348139[] = {{&S62958[49], 20}, {NULL}};
const Segment SGI348140[] = {{&S62959[49], 20}, {NULL}};
const Segment SGI348141[] = {{&S62962[42], 3}, {NULL}};
const Segment SGI348142[] = {{&S62962[6], 3}, {NULL}};
const Segment SGI348143[] = {{&S62962[66], 3}, {NULL}};
const Segment SGI348144[] = {{&S62998[42], 5}, {NULL}};
const Segment SGI348145[] = {{&S62998[6], 3}, {NULL}};
const Segment SGI348146[] = {{&S62998[68], 3}, {NULL}};
const Segment SGI348147[] = {{&S62999[43], 5}, {NULL}};
const Segment SGI348148[] = {{&S62999[6], 3}, {NULL}};
const Segment SGI348149[] = {{&S62999[69], 3}, {NULL}};
const Segment SGI348150[] = {{&S63000[5], 5}, {NULL}};
const Segment SGI348151[] = {{&S63000[16], 3}, {NULL}};
const Segment SGI348152[] = {{&S63001[43], 3}, {NULL}};
const Segment SGI348153[] = {{&S63001[6], 3}, {NULL}};
const Segment SGI348154[] = {{&S63001[67], 3}, {NULL}};
const Segment SGI348155[] = {{&S54451[28], 5}, {NULL}};
const Segment SGI348156[] = {{&S54451[104], 5}, {NULL}};
const Segment SGI348157[] = {{&S54452[28], 7}, {NULL}};
const Segment SGI348158[] = {{&S54452[106], 5}, {NULL}};
const Segment SGI348159[] = {{&S54453[28], 5}, {NULL}};
const Segment SGI348160[] = {{&S54453[104], 5}, {NULL}};
const Segment SGI348161[] = {{&S36699[6], 4}, {NULL}};
const Segment SGI348162[] = {{&S36699[53], 5}, {NULL}};
const Segment SGI348163[] = {{&S36699[12], 3}, {NULL}};
const Segment SGI348164[] = {{&S36700[6], 4}, {NULL}};
const Segment SGI348165[] = {{&S36700[53], 5}, {NULL}};
const Segment SGI348166[] = {{&S36700[12], 3}, {&S36700[64], 4}, {NULL}};
const Segment SGI348167[] = {{&S36703[6], 4}, {NULL}};
const Segment SGI348168[] = {{&S36703[52], 5}, {NULL}};
const Segment SGI348169[] = {{&S36703[12], 45}, {NULL}};
const Segment SGI348170[] = {{&S36703[12], 14}, {NULL}};
const Segment SGI348171[] = {{&S36704[6], 4}, {NULL}};
const Segment SGI348172[] = {{&S36704[59], 5}, {NULL}};
const Segment SGI348173[] = {{&S36704[12], 14}, {NULL}};
const Segment SGI348174[] = {{&S36705[6], 4}, {NULL}};
const Segment SGI348175[] = {{&S36705[81], 5}, {NULL}};
const Segment SGI348176[] = {{&S36705[12], 14}, {&S36705[44], 42}, {NULL}};
const Segment SGI348177[] = {{&S36705[12], 14}, {NULL}};
const Segment SGI348178[] = {{&S36706[6], 4}, {NULL}};
const Segment SGI348179[] = {{&S36706[81], 5}, {NULL}};
const Segment SGI348180[] = {{&S36706[12], 14}, {&S36706[44], 42}, {NULL}};
const Segment SGI348181[] = {{&S36706[12], 14}, {NULL}};
const Segment SGI348182[] = {{&S36707[6], 4}, {NULL}};
const Segment SGI348183[] = {{&S36707[81], 5}, {NULL}};
const Segment SGI348184[] = {{&S36707[12], 14}, {&S36707[44], 42}, {NULL}};
const Segment SGI348185[] = {{&S36707[12], 14}, {NULL}};
const Segment SGI348186[] = {{&S36708[6], 4}, {NULL}};
const Segment SGI348187[] = {{&S36708[81], 5}, {NULL}};
const Segment SGI348188[] = {{&S36708[12], 14}, {&S36708[44], 42}, {NULL}};
const Segment SGI348189[] = {{&S36708[12], 14}, {NULL}};
const Segment SGI348190[] = {{&S36709[6], 4}, {NULL}};
const Segment SGI348191[] = {{&S36709[81], 5}, {NULL}};
const Segment SGI348192[] = {{&S36709[12], 14}, {&S36709[44], 42}, {NULL}};
const Segment SGI348193[] = {{&S36709[12], 14}, {NULL}};
const Segment SGI348194[] = {{&S36710[6], 4}, {NULL}};
const Segment SGI348195[] = {{&S36710[59], 5}, {NULL}};
const Segment SGI348196[] = {{&S36710[12], 14}, {&S36710[70], 4}, {NULL}};
const Segment SGI348197[] = {{&S36710[12], 14}, {NULL}};
const Segment SGI348198[] = {{&S36711[6], 4}, {NULL}};
const Segment SGI348199[] = {{&S36711[81], 5}, {NULL}};
const Segment SGI348200[] = {{&S36711[12], 14}, {&S36711[44], 42}, {NULL}};
const Segment SGI348201[] = {{&S36711[12], 14}, {NULL}};
const Segment SGI348202[] = {{&S36712[6], 4}, {NULL}};
const Segment SGI348203[] = {{&S36712[81], 5}, {NULL}};
const Segment SGI348204[] = {{&S36712[12], 14}, {&S36712[44], 42}, {NULL}};
const Segment SGI348205[] = {{&S36712[12], 14}, {NULL}};
const Segment SGI348206[] = {{&S36713[6], 4}, {NULL}};
const Segment SGI348207[] = {{&S36713[81], 5}, {NULL}};
const Segment SGI348208[] = {{&S36713[12], 14}, {&S36713[44], 42}, {NULL}};
const Segment SGI348209[] = {{&S36713[12], 14}, {NULL}};
const Segment SGI348210[] = {{&S36715[6], 4}, {NULL}};
const Segment SGI348211[] = {{&S36715[52], 5}, {NULL}};
const Segment SGI348212[] = {{&S36715[12], 55}, {NULL}};
const Segment SGI348213[] = {{&S36715[12], 14}, {NULL}};
const Segment SGI348214[] = {{&S36716[6], 4}, {NULL}};
const Segment SGI348215[] = {{&S36716[81], 5}, {NULL}};
const Segment SGI348216[] = {{&S36716[12], 14}, {&S36716[44], 42}, {NULL}};
const Segment SGI348217[] = {{&S36716[12], 14}, {NULL}};
const Segment SGI348218[] = {{&S36717[6], 4}, {NULL}};
const Segment SGI348219[] = {{&S36717[81], 5}, {NULL}};
const Segment SGI348220[] = {{&S36717[12], 14}, {&S36717[44], 42}, {NULL}};
const Segment SGI348221[] = {{&S36717[12], 14}, {NULL}};
const Segment SGI348222[] = {{&S36718[6], 4}, {NULL}};
const Segment SGI348223[] = {{&S36718[81], 5}, {NULL}};
const Segment SGI348224[] = {{&S36718[12], 14}, {&S36718[44], 42}, {NULL}};
const Segment SGI348225[] = {{&S36718[12], 14}, {NULL}};
const Segment SGI348226[] = {{&S36719[6], 4}, {NULL}};
const Segment SGI348227[] = {{&S36719[81], 5}, {NULL}};
const Segment SGI348228[] = {{&S36719[12], 14}, {&S36719[44], 42}, {NULL}};
const Segment SGI348229[] = {{&S36719[12], 14}, {NULL}};
const Segment SGI348230[] = {{&S36720[6], 4}, {NULL}};
const Segment SGI348231[] = {{&S36720[81], 5}, {NULL}};
const Segment SGI348232[] = {{&S36720[12], 14}, {&S36720[44], 42}, {NULL}};
const Segment SGI348233[] = {{&S36720[12], 14}, {NULL}};
const Segment SGI348234[] = {{&S36721[6], 4}, {NULL}};
const Segment SGI348235[] = {{&S36721[81], 5}, {NULL}};
const Segment SGI348236[] = {{&S36721[12], 14}, {&S36721[44], 42}, {NULL}};
const Segment SGI348237[] = {{&S36721[12], 14}, {NULL}};
const Segment SGI348238[] = {{&S36722[6], 4}, {NULL}};
const Segment SGI348239[] = {{&S36722[81], 5}, {NULL}};
const Segment SGI348240[] = {{&S36722[12], 14}, {&S36722[44], 42}, {NULL}};
const Segment SGI348241[] = {{&S36722[12], 14}, {NULL}};
const Segment SGI348242[] = {{&S36723[6], 4}, {NULL}};
const Segment SGI348243[] = {{&S36723[81], 5}, {NULL}};
const Segment SGI348244[] = {{&S36723[12], 14}, {&S36723[44], 42}, {NULL}};
const Segment SGI348245[] = {{&S36723[12], 14}, {NULL}};
const Segment SGI348246[] = {{&S36724[6], 4}, {NULL}};
const Segment SGI348247[] = {{&S36724[81], 5}, {NULL}};
const Segment SGI348248[] = {{&S36724[12], 14}, {&S36724[44], 42}, {NULL}};
const Segment SGI348249[] = {{&S36724[12], 14}, {NULL}};
const Segment SGI348250[] = {{&S36791[6], 4}, {NULL}};
const Segment SGI348251[] = {{&S36791[52], 5}, {NULL}};
const Segment SGI348252[] = {{&S36791[12], 45}, {NULL}};
const Segment SGI348253[] = {{&S36791[12], 14}, {NULL}};
const Segment SGI348254[] = {{&S36792[6], 4}, {NULL}};
const Segment SGI348255[] = {{&S36792[59], 5}, {NULL}};
const Segment SGI348256[] = {{&S36792[12], 14}, {NULL}};
const Segment SGI348257[] = {{&S36793[6], 4}, {NULL}};
const Segment SGI348258[] = {{&S36793[75], 5}, {NULL}};
const Segment SGI348259[] = {{&S36793[12], 14}, {&S36793[37], 43}, {NULL}};
const Segment SGI348260[] = {{&S36793[12], 14}, {NULL}};
const Segment SGI348261[] = {{&S36794[6], 4}, {NULL}};
const Segment SGI348262[] = {{&S36794[74], 5}, {NULL}};
const Segment SGI348263[] = {{&S36794[12], 14}, {&S36794[37], 42}, {NULL}};
const Segment SGI348264[] = {{&S36794[12], 14}, {NULL}};
const Segment SGI348265[] = {{&S36795[6], 4}, {NULL}};
const Segment SGI348266[] = {{&S36795[75], 5}, {NULL}};
const Segment SGI348267[] = {{&S36795[12], 14}, {&S36795[37], 43}, {NULL}};
const Segment SGI348268[] = {{&S36795[12], 14}, {NULL}};
const Segment SGI348269[] = {{&S36796[6], 4}, {NULL}};
const Segment SGI348270[] = {{&S36796[74], 5}, {NULL}};
const Segment SGI348271[] = {{&S36796[12], 14}, {&S36796[37], 42}, {NULL}};
const Segment SGI348272[] = {{&S36796[12], 14}, {NULL}};
const Segment SGI348273[] = {{&S36797[6], 4}, {NULL}};
const Segment SGI348274[] = {{&S36797[74], 5}, {NULL}};
const Segment SGI348275[] = {{&S36797[12], 14}, {&S36797[37], 42}, {NULL}};
const Segment SGI348276[] = {{&S36797[12], 14}, {NULL}};
const Segment SGI348277[] = {{&S36798[6], 4}, {NULL}};
const Segment SGI348278[] = {{&S36798[52], 5}, {NULL}};
const Segment SGI348279[] = {{&S36798[12], 55}, {NULL}};
const Segment SGI348280[] = {{&S36798[12], 14}, {NULL}};
const Segment SGI348281[] = {{&S36800[6], 4}, {NULL}};
const Segment SGI348282[] = {{&S36800[74], 5}, {NULL}};
const Segment SGI348283[] = {{&S36800[12], 14}, {&S36800[37], 42}, {NULL}};
const Segment SGI348284[] = {{&S36800[12], 14}, {NULL}};
const Segment SGI348285[] = {{&S36802[6], 4}, {NULL}};
const Segment SGI348286[] = {{&S36802[75], 5}, {NULL}};
const Segment SGI348287[] = {{&S36802[12], 14}, {&S36802[38], 42}, {NULL}};
const Segment SGI348288[] = {{&S36802[12], 14}, {NULL}};
const Segment SGI348289[] = {{&S36803[6], 4}, {NULL}};
const Segment SGI348290[] = {{&S36803[74], 5}, {NULL}};
const Segment SGI348291[] = {{&S36803[12], 14}, {&S36803[37], 42}, {NULL}};
const Segment SGI348292[] = {{&S36803[12], 14}, {NULL}};
const Segment SGI348293[] = {{&S36804[6], 4}, {NULL}};
const Segment SGI348294[] = {{&S36804[75], 5}, {NULL}};
const Segment SGI348295[] = {{&S36804[12], 14}, {&S36804[37], 43}, {NULL}};
const Segment SGI348296[] = {{&S36804[12], 14}, {NULL}};
const Segment SGI348297[] = {{&S36805[6], 4}, {NULL}};
const Segment SGI348298[] = {{&S36805[75], 5}, {NULL}};
const Segment SGI348299[] = {{&S36805[12], 14}, {&S36805[37], 43}, {NULL}};
const Segment SGI348300[] = {{&S36805[12], 14}, {NULL}};
const Segment SGI348301[] = {{&S39237[63], 4}, {NULL}};
const Segment SGI348302[] = {{&S39237[13], 10}, {NULL}};
const Segment SGI348303[] = {{&S36807[6], 4}, {NULL}};
const Segment SGI348304[] = {{&S36807[74], 5}, {NULL}};
const Segment SGI348305[] = {{&S36807[12], 14}, {&S36807[37], 42}, {NULL}};
const Segment SGI348306[] = {{&S36807[12], 14}, {NULL}};
const Segment SGI348307[] = {{&S36808[6], 4}, {NULL}};
const Segment SGI348308[] = {{&S36808[75], 5}, {NULL}};
const Segment SGI348309[] = {{&S36808[12], 14}, {&S36808[37], 43}, {NULL}};
const Segment SGI348310[] = {{&S36808[12], 14}, {NULL}};
const Segment SGI348311[] = {{&S36809[6], 4}, {NULL}};
const Segment SGI348312[] = {{&S36809[74], 5}, {NULL}};
const Segment SGI348313[] = {{&S36809[12], 14}, {&S36809[37], 42}, {NULL}};
const Segment SGI348314[] = {{&S36809[12], 14}, {NULL}};
const Segment SGI348315[] = {{&S54612[28], 5}, {NULL}};
const Segment SGI348316[] = {{&S54612[111], 5}, {NULL}};
const Segment SGI348317[] = {{&S63033[5], 5}, {&S63033[97], 5}, {NULL}};
const Segment SGI348318[] = {{&S63033[113], 3}, {NULL}};
const Segment SGI348319[] = {{&S63077[5], 7}, {&S63077[5], 7}, {NULL}};
const Segment SGI348320[] = {{&S63077[13], 3}, {NULL}};
const Segment SGI348321[] = {{&S63077[135], 3}, {NULL}};
const Segment SGI348322[] = {{&S63078[5], 5}, {&S63078[5], 5}, {NULL}};
const Segment SGI348323[] = {{&S63078[11], 3}, {NULL}};
const Segment SGI348324[] = {{&S63078[131], 3}, {NULL}};
const Segment SGI348325[] = {{&S50212[49], 7}, {NULL}};
const Segment SGI348326[] = {{&S50212[8], 3}, {NULL}};
const Segment SGI348327[] = {{&S50213[46], 7}, {&S50213[149], 7}, {NULL}};
const Segment SGI348328[] = {{&S50213[8], 3}, {NULL}};
const Segment SGI348329[] = {{&S50214[46], 7}, {&S50214[149], 7}, {NULL}};
const Segment SGI348330[] = {{&S50214[8], 3}, {NULL}};
const Segment SGI348331[] = {{&S50215[46], 7}, {NULL}};
const Segment SGI348332[] = {{&S50215[8], 3}, {NULL}};
const Segment SGI348333[] = {{&S50216[49], 4}, {NULL}};
const Segment SGI348334[] = {{&S50216[8], 3}, {NULL}};
const Segment SGI348335[] = {{&S50217[46], 4}, {&S50217[146], 7}, {NULL}};
const Segment SGI348336[] = {{&S50217[8], 3}, {NULL}};
const Segment SGI348337[] = {{&S50219[46], 5}, {NULL}};
const Segment SGI348338[] = {{&S50219[37], 3}, {NULL}};
const Segment SGI348339[] = {{&S63095[5], 5}, {&S63095[105], 5}, {NULL}};
const Segment SGI348340[] = {{&S63095[69], 3}, {NULL}};
const Segment SGI348341[] = {{&S63095[131], 3}, {NULL}};
const Segment SGI348342[] = {{&S50220[5], 3}, {NULL}};
const Segment SGI348343[] = {{&S50220[44], 3}, {NULL}};
const Segment SGI348344[] = {{&S50221[46], 5}, {NULL}};
const Segment SGI348345[] = {{&S50221[37], 3}, {NULL}};
const Segment SGI348346[] = {{&S50222[5], 6}, {NULL}};
const Segment SGI348347[] = {{&S50222[35], 3}, {NULL}};
const Segment SGI348348[] = {{&S63107[5], 5}, {NULL}};
const Segment SGI348349[] = {{&S63108[5], 5}, {&S63108[68], 5}, {NULL}};
const Segment SGI348350[] = {{&S54777[28], 7}, {NULL}};
const Segment SGI348351[] = {{&S54777[106], 5}, {NULL}};
const Segment SGI348352[] = {{&S54778[28], 5}, {NULL}};
const Segment SGI348353[] = {{&S54778[104], 5}, {NULL}};
const Segment SGI348354[] = {{&S54779[28], 3}, {NULL}};
const Segment SGI348355[] = {{&S54779[102], 5}, {NULL}};
const Segment SGI348356[] = {{&S36917[6], 4}, {NULL}};
const Segment SGI348357[] = {{&S36917[12], 14}, {NULL}};
const Segment SGI348358[] = {{&S36918[6], 4}, {NULL}};
const Segment SGI348359[] = {{&S36918[12], 14}, {NULL}};
const Segment SGI348360[] = {{&S36919[6], 4}, {NULL}};
const Segment SGI348361[] = {{&S36919[12], 14}, {NULL}};
const Segment SGI348362[] = {{&S36920[6], 4}, {NULL}};
const Segment SGI348363[] = {{&S36920[12], 14}, {NULL}};
const Segment SGI348364[] = {{&S36921[6], 4}, {NULL}};
const Segment SGI348365[] = {{&S36921[12], 14}, {NULL}};
const Segment SGI348366[] = {{&S36922[6], 4}, {NULL}};
const Segment SGI348367[] = {{&S36922[12], 14}, {NULL}};
const Segment SGI348368[] = {{&S36923[6], 4}, {NULL}};
const Segment SGI348369[] = {{&S36923[12], 14}, {NULL}};
const Segment SGI348370[] = {{&S36924[6], 4}, {NULL}};
const Segment SGI348371[] = {{&S36924[12], 14}, {NULL}};
const Segment SGI348372[] = {{&S50309[49], 5}, {NULL}};
const Segment SGI348373[] = {{&S50309[40], 3}, {NULL}};
const Segment SGI348374[] = {{&S63228[5], 4}, {NULL}};
const Segment SGI348375[] = {{&S50310[5], 3}, {NULL}};
const Segment SGI348376[] = {{&S50310[32], 3}, {NULL}};
const Segment SGI348377[] = {{&S50311[49], 5}, {NULL}};
const Segment SGI348378[] = {{&S50311[40], 3}, {NULL}};
const Segment SGI348379[] = {{&S50315[49], 5}, {NULL}};
const Segment SGI348380[] = {{&S50315[40], 3}, {NULL}};
const Segment SGI348381[] = {{&S50316[5], 3}, {NULL}};
const Segment SGI348382[] = {{&S50316[32], 3}, {NULL}};
const Segment SGI348383[] = {{&S50317[49], 5}, {NULL}};
const Segment SGI348384[] = {{&S50317[40], 3}, {NULL}};
const Segment SGI348385[] = {{&S63267[43], 3}, {NULL}};
const Segment SGI348386[] = {{&S63267[6], 3}, {NULL}};
const Segment SGI348387[] = {{&S63267[67], 3}, {NULL}};
const Segment SGI348388[] = {{&S39487[53], 4}, {NULL}};
const Segment SGI348389[] = {{&S39487[13], 10}, {NULL}};
const Segment SGI348390[] = {{&S39488[53], 4}, {NULL}};
const Segment SGI348391[] = {{&S39488[13], 10}, {NULL}};
const Segment SGI348392[] = {{&S63282[5], 3}, {&S63282[5], 3}, {NULL}};
const Segment SGI348393[] = {{&S63282[149], 3}, {NULL}};
const Segment SGI348394[] = {{&S63323[5], 5}, {NULL}};
const Segment SGI348395[] = {{&S63324[5], 5}, {NULL}};
const Segment SGI348396[] = {{&S63325[5], 3}, {NULL}};
const Segment SGI348397[] = {{&S54979[28], 5}, {NULL}};
const Segment SGI348398[] = {{&S54979[111], 5}, {NULL}};
const Segment SGI348399[] = {{&S63417[5], 5}, {&S63417[105], 5}, {NULL}};
const Segment SGI348400[] = {{&S63417[69], 3}, {NULL}};
const Segment SGI348401[] = {{&S63417[131], 3}, {NULL}};
const Segment SGI348402[] = {{&S63433[5], 3}, {&S63433[5], 3}, {NULL}};
const Segment SGI348403[] = {{&S63433[9], 3}, {NULL}};
const Segment SGI348404[] = {{&S63433[127], 3}, {NULL}};
const Segment SGI348405[] = {{&S63434[5], 5}, {&S63434[5], 5}, {NULL}};
const Segment SGI348406[] = {{&S63434[11], 3}, {NULL}};
const Segment SGI348407[] = {{&S63434[131], 3}, {NULL}};
const Segment SGI348408[] = {{&S63459[5], 5}, {&S63459[104], 5}, {NULL}};
const Segment SGI348409[] = {{&S63459[69], 3}, {NULL}};
const Segment SGI348410[] = {{&S63459[130], 3}, {NULL}};
const Segment SGI348411[] = {{&S63462[5], 3}, {&S63462[5], 3}, {NULL}};
const Segment SGI348412[] = {{&S63462[9], 3}, {NULL}};
const Segment SGI348413[] = {{&S63462[127], 3}, {NULL}};
const Segment SGI348414[] = {{&S63463[5], 5}, {&S63463[5], 5}, {NULL}};
const Segment SGI348415[] = {{&S63463[11], 3}, {NULL}};
const Segment SGI348416[] = {{&S63463[131], 3}, {NULL}};
const Segment SGI348417[] = {{&S37177[6], 4}, {NULL}};
const Segment SGI348418[] = {{&S37177[52], 5}, {NULL}};
const Segment SGI348419[] = {{&S37177[12], 45}, {NULL}};
const Segment SGI348420[] = {{&S37177[12], 14}, {NULL}};
const Segment SGI348421[] = {{&S37178[6], 4}, {NULL}};
const Segment SGI348422[] = {{&S37178[59], 5}, {NULL}};
const Segment SGI348423[] = {{&S37178[12], 14}, {NULL}};
const Segment SGI348424[] = {{&S37179[6], 4}, {NULL}};
const Segment SGI348425[] = {{&S37179[74], 5}, {NULL}};
const Segment SGI348426[] = {{&S37179[12], 14}, {&S37179[37], 42}, {NULL}};
const Segment SGI348427[] = {{&S37179[12], 14}, {NULL}};
const Segment SGI348428[] = {{&S37180[6], 4}, {NULL}};
const Segment SGI348429[] = {{&S37180[75], 5}, {NULL}};
const Segment SGI348430[] = {{&S37180[12], 14}, {&S37180[37], 43}, {NULL}};
const Segment SGI348431[] = {{&S37180[12], 14}, {NULL}};
const Segment SGI348432[] = {{&S37181[6], 4}, {NULL}};
const Segment SGI348433[] = {{&S37181[75], 5}, {NULL}};
const Segment SGI348434[] = {{&S37181[12], 14}, {&S37181[37], 43}, {NULL}};
const Segment SGI348435[] = {{&S37181[12], 14}, {NULL}};
const Segment SGI348436[] = {{&S37182[6], 4}, {NULL}};
const Segment SGI348437[] = {{&S37182[74], 5}, {NULL}};
const Segment SGI348438[] = {{&S37182[12], 14}, {&S37182[37], 42}, {NULL}};
const Segment SGI348439[] = {{&S37182[12], 14}, {NULL}};
const Segment SGI348440[] = {{&S37183[6], 4}, {NULL}};
const Segment SGI348441[] = {{&S37183[52], 5}, {NULL}};
const Segment SGI348442[] = {{&S37183[12], 55}, {NULL}};
const Segment SGI348443[] = {{&S37183[12], 14}, {NULL}};
const Segment SGI348444[] = {{&S63537[43], 5}, {NULL}};
const Segment SGI348445[] = {{&S63537[6], 3}, {NULL}};
const Segment SGI348446[] = {{&S63537[69], 3}, {NULL}};
const Segment SGI348447[] = {{&S37185[6], 4}, {NULL}};
const Segment SGI348448[] = {{&S37185[74], 5}, {NULL}};
const Segment SGI348449[] = {{&S37185[12], 14}, {&S37185[37], 42}, {NULL}};
const Segment SGI348450[] = {{&S37185[12], 14}, {NULL}};
const Segment SGI348451[] = {{&S63539[35], 4}, {NULL}};
const Segment SGI348452[] = {{&S63539[51], 3}, {NULL}};
const Segment SGI348453[] = {{&S37187[6], 4}, {NULL}};
const Segment SGI348454[] = {{&S37187[75], 5}, {NULL}};
const Segment SGI348455[] = {{&S37187[12], 14}, {&S37187[38], 42}, {NULL}};
const Segment SGI348456[] = {{&S63540[5], 4}, {NULL}};
const Segment SGI348457[] = {{&S37187[12], 14}, {NULL}};
const Segment SGI348458[] = {{&S63540[15], 3}, {NULL}};
const Segment SGI348459[] = {{&S37188[6], 4}, {NULL}};
const Segment SGI348460[] = {{&S37188[74], 5}, {NULL}};
const Segment SGI348461[] = {{&S37188[12], 14}, {&S37188[37], 42}, {NULL}};
const Segment SGI348462[] = {{&S37188[12], 14}, {NULL}};
const Segment SGI348463[] = {{&S55104[28], 3}, {NULL}};
const Segment SGI348464[] = {{&S37189[6], 4}, {NULL}};
const Segment SGI348465[] = {{&S55104[102], 5}, {NULL}};
const Segment SGI348466[] = {{&S37189[74], 5}, {NULL}};
const Segment SGI348467[] = {{&S37189[12], 14}, {&S37189[37], 42}, {NULL}};
const Segment SGI348468[] = {{&S37189[12], 14}, {NULL}};
const Segment SGI348469[] = {{&S55105[28], 3}, {NULL}};
const Segment SGI348470[] = {{&S37190[6], 4}, {NULL}};
const Segment SGI348471[] = {{&S37190[104], 5}, {NULL}};
const Segment SGI348472[] = {{&S55105[102], 5}, {NULL}};
const Segment SGI348473[] = {{&S37190[12], 14}, {&S37190[41], 10}, {NULL}};
const Segment SGI348474[] = {{&S37190[12], 14}, {&S37190[63], 14}, {NULL}};
const Segment SGI348475[] = {{&S37191[6], 4}, {NULL}};
const Segment SGI348476[] = {{&S37191[90], 5}, {NULL}};
const Segment SGI348477[] = {{&S37191[12], 14}, {NULL}};
const Segment SGI348478[] = {{&S37191[12], 14}, {&S37191[50], 14}, {NULL}};
const Segment SGI348479[] = {{&S37192[6], 4}, {NULL}};
const Segment SGI348480[] = {{&S37192[74], 5}, {NULL}};
const Segment SGI348481[] = {{&S37192[12], 14}, {&S37192[37], 42}, {NULL}};
const Segment SGI348482[] = {{&S37192[12], 14}, {NULL}};
const Segment SGI348483[] = {{&S37194[6], 4}, {NULL}};
const Segment SGI348484[] = {{&S37194[74], 5}, {NULL}};
const Segment SGI348485[] = {{&S37194[12], 14}, {&S37194[37], 42}, {NULL}};
const Segment SGI348486[] = {{&S37194[12], 14}, {NULL}};
const Segment SGI348487[] = {{&S37195[6], 4}, {NULL}};
const Segment SGI348488[] = {{&S37195[75], 5}, {NULL}};
const Segment SGI348489[] = {{&S37195[12], 14}, {&S37195[37], 43}, {NULL}};
const Segment SGI348490[] = {{&S37195[12], 14}, {NULL}};
const Segment SGI348491[] = {{&S37197[6], 4}, {NULL}};
const Segment SGI348492[] = {{&S37197[74], 5}, {NULL}};
const Segment SGI348493[] = {{&S37197[12], 14}, {&S37197[37], 42}, {NULL}};
const Segment SGI348494[] = {{&S37197[12], 14}, {NULL}};
const Segment SGI348495[] = {{&S37198[6], 4}, {NULL}};
const Segment SGI348496[] = {{&S37198[101], 5}, {NULL}};
const Segment SGI348497[] = {{&S37198[12], 14}, {&S37198[37], 12}, {NULL}};
const Segment SGI348498[] = {{&S37198[12], 14}, {NULL}};
const Segment SGI348499[] = {{&S37199[6], 4}, {NULL}};
const Segment SGI348500[] = {{&S37199[75], 5}, {NULL}};
const Segment SGI348501[] = {{&S37199[12], 14}, {&S37199[38], 42}, {NULL}};
const Segment SGI348502[] = {{&S37199[12], 14}, {NULL}};
const Segment SGI348503[] = {{&S37200[6], 4}, {NULL}};
const Segment SGI348504[] = {{&S37200[74], 5}, {NULL}};
const Segment SGI348505[] = {{&S37200[12], 14}, {&S37200[37], 42}, {NULL}};
const Segment SGI348506[] = {{&S37200[12], 14}, {NULL}};
const Segment SGI348507[] = {{&S37201[6], 4}, {NULL}};
const Segment SGI348508[] = {{&S37201[75], 5}, {NULL}};
const Segment SGI348509[] = {{&S37201[12], 14}, {&S37201[37], 43}, {NULL}};
const Segment SGI348510[] = {{&S37201[12], 14}, {NULL}};
const Segment SGI348511[] = {{&S37202[6], 4}, {NULL}};
const Segment SGI348512[] = {{&S37202[74], 5}, {NULL}};
const Segment SGI348513[] = {{&S37202[12], 14}, {&S37202[37], 42}, {NULL}};
const Segment SGI348514[] = {{&S37202[12], 14}, {NULL}};
const Segment SGI348515[] = {{&S37203[6], 4}, {NULL}};
const Segment SGI348516[] = {{&S37203[74], 5}, {NULL}};
const Segment SGI348517[] = {{&S37203[12], 14}, {&S37203[37], 42}, {NULL}};
const Segment SGI348518[] = {{&S37203[12], 14}, {NULL}};
const Segment SGI348519[] = {{&S37204[6], 4}, {NULL}};
const Segment SGI348520[] = {{&S37204[74], 5}, {NULL}};
const Segment SGI348521[] = {{&S37204[12], 14}, {&S37204[37], 42}, {NULL}};
const Segment SGI348522[] = {{&S37204[12], 14}, {NULL}};
const Segment SGI348523[] = {{&S37205[6], 4}, {NULL}};
const Segment SGI348524[] = {{&S37205[74], 5}, {NULL}};
const Segment SGI348525[] = {{&S37205[12], 14}, {&S37205[37], 42}, {NULL}};
const Segment SGI348526[] = {{&S37205[12], 14}, {NULL}};
const Segment SGI348527[] = {{&S37206[6], 4}, {NULL}};
const Segment SGI348528[] = {{&S37206[75], 5}, {NULL}};
const Segment SGI348529[] = {{&S37206[12], 14}, {&S37206[37], 43}, {NULL}};
const Segment SGI348530[] = {{&S37206[12], 14}, {NULL}};
const Segment SGI348531[] = {{&S37207[6], 4}, {NULL}};
const Segment SGI348532[] = {{&S37207[74], 5}, {NULL}};
const Segment SGI348533[] = {{&S37207[12], 14}, {&S37207[37], 42}, {NULL}};
const Segment SGI348534[] = {{&S37207[12], 14}, {NULL}};
const Segment SGI348535[] = {{&S37208[6], 4}, {NULL}};
const Segment SGI348536[] = {{&S37208[74], 5}, {NULL}};
const Segment SGI348537[] = {{&S37208[12], 14}, {&S37208[37], 42}, {NULL}};
const Segment SGI348538[] = {{&S37208[12], 14}, {NULL}};
const Segment SGI348539[] = {{&S37209[6], 4}, {NULL}};
const Segment SGI348540[] = {{&S37209[74], 5}, {NULL}};
const Segment SGI348541[] = {{&S37209[12], 14}, {&S37209[37], 42}, {NULL}};
const Segment SGI348542[] = {{&S37209[12], 14}, {NULL}};
const Segment SGI348543[] = {{&S37210[6], 4}, {NULL}};
const Segment SGI348544[] = {{&S37210[74], 5}, {NULL}};
const Segment SGI348545[] = {{&S37210[12], 14}, {&S37210[37], 42}, {NULL}};
const Segment SGI348546[] = {{&S37210[12], 14}, {NULL}};
const Segment SGI348547[] = {{&S50567[49], 5}, {NULL}};
const Segment SGI348548[] = {{&S50567[40], 3}, {NULL}};
const Segment SGI348549[] = {{&S50568[5], 3}, {NULL}};
const Segment SGI348550[] = {{&S50568[32], 3}, {NULL}};
const Segment SGI348551[] = {{&S50569[49], 5}, {NULL}};
const Segment SGI348552[] = {{&S50569[40], 3}, {NULL}};
const Segment SGI348553[] = {{&S50570[8], 3}, {NULL}};
const Segment SGI348554[] = {{&S50570[35], 3}, {NULL}};
const Segment SGI348555[] = {{&S63598[5], 5}, {NULL}};
const Segment SGI348556[] = {{&S63598[16], 3}, {NULL}};
const Segment SGI348557[] = {{&S63599[5], 5}, {NULL}};
const Segment SGI348558[] = {{&S63599[21], 3}, {NULL}};
const Segment SGI348559[] = {{&S63600[5], 7}, {NULL}};
const Segment SGI348560[] = {{&S63600[23], 3}, {NULL}};
const Segment SGI348561[] = {{&S63603[43], 5}, {NULL}};
const Segment SGI348562[] = {{&S63603[6], 3}, {NULL}};
const Segment SGI348563[] = {{&S63603[69], 3}, {NULL}};
const Segment SGI348564[] = {{&S55221[28], 5}, {NULL}};
const Segment SGI348565[] = {{&S55221[104], 5}, {NULL}};
const Segment SGI348566[] = {{&S55222[28], 5}, {NULL}};
const Segment SGI348567[] = {{&S55222[104], 5}, {NULL}};
const Segment SGI348568[] = {{&S63704[35], 5}, {NULL}};
const Segment SGI348569[] = {{&S63704[91], 3}, {NULL}};
const Segment SGI348570[] = {{&S63705[5], 5}, {NULL}};
const Segment SGI348571[] = {{&S63705[16], 3}, {NULL}};
const Segment SGI348572[] = {{&S63710[5], 4}, {&S63710[5], 4}, {NULL}};
const Segment SGI348573[] = {{&S63710[111], 3}, {NULL}};
const Segment SGI348574[] = {{&S50702[49], 13}, {NULL}};
const Segment SGI348575[] = {{&S50702[40], 3}, {NULL}};
const Segment SGI348576[] = {{&S55280[28], 3}, {NULL}};
const Segment SGI348577[] = {{&S55280[102], 5}, {NULL}};
const Segment SGI348578[] = {{&S50704[5], 5}, {NULL}};
const Segment SGI348579[] = {{&S50704[45], 3}, {NULL}};
const Segment SGI348580[] = {{&S37299[6], 4}, {NULL}};
const Segment SGI348581[] = {{&S37299[62], 5}, {NULL}};
const Segment SGI348582[] = {{&S37299[12], 3}, {NULL}};
const Segment SGI348583[] = {{&S63740[5], 3}, {&S63740[5], 3}, {NULL}};
const Segment SGI348584[] = {{&S63740[109], 3}, {NULL}};
const Segment SGI348585[] = {{&S63759[43], 5}, {NULL}};
const Segment SGI348586[] = {{&S63759[6], 3}, {NULL}};
const Segment SGI348587[] = {{&S63759[69], 3}, {NULL}};
const Segment SGI348588[] = {{&S63775[49], 20}, {NULL}};
const Segment SGI348589[] = {{&S63776[49], 20}, {NULL}};
const Segment SGI348590[] = {{&S63777[49], 20}, {NULL}};
const Segment SGI348591[] = {{&S63778[49], 20}, {NULL}};
const Segment SGI348592[] = {{&S63779[49], 24}, {NULL}};
const Segment SGI348593[] = {{&S63780[49], 20}, {NULL}};
const Segment SGI348594[] = {{&S63815[5], 5}, {&S63815[97], 5}, {NULL}};
const Segment SGI348595[] = {{&S63815[153], 3}, {NULL}};
const Segment SGI348596[] = {{&S63816[20], 4}, {NULL}};
const Segment SGI348597[] = {{&S63816[0], 14}, {NULL}};
const Segment SGI348598[] = {{&S63817[21], 4}, {NULL}};
const Segment SGI348599[] = {{&S63828[35], 3}, {NULL}};
const Segment SGI348600[] = {{&S63828[89], 3}, {NULL}};
const Segment SGI348601[] = {{&S63829[35], 7}, {NULL}};
const Segment SGI348602[] = {{&S63829[93], 3}, {NULL}};
const Segment SGI348603[] = {{&S63830[34], 3}, {NULL}};
const Segment SGI348604[] = {{&S63830[88], 3}, {NULL}};
const Segment SGI348605[] = {{&S63831[34], 7}, {NULL}};
const Segment SGI348606[] = {{&S63831[92], 3}, {NULL}};
const Segment SGI348607[] = {{&S50830[49], 6}, {NULL}};
const Segment SGI348608[] = {{&S50830[40], 3}, {NULL}};
const Segment SGI348609[] = {{&S50831[49], 5}, {NULL}};
const Segment SGI348610[] = {{&S50831[40], 3}, {NULL}};
const Segment SGI348611[] = {{&S63853[5], 3}, {NULL}};
const Segment SGI348612[] = {{&S63853[14], 3}, {NULL}};
const Segment SGI348613[] = {{&S63854[35], 3}, {NULL}};
const Segment SGI348614[] = {{&S63854[50], 3}, {NULL}};
const Segment SGI348615[] = {{&S63855[34], 5}, {NULL}};
const Segment SGI348616[] = {{&S63855[50], 3}, {NULL}};
const Segment SGI348617[] = {{&S50891[46], 5}, {&S50891[147], 7}, {NULL}};
const Segment SGI348618[] = {{&S50891[8], 3}, {NULL}};
const Segment SGI348619[] = {{&S50892[5], 5}, {NULL}};
const Segment SGI348620[] = {{&S50892[48], 3}, {NULL}};
const Segment SGI348621[] = {{&S63906[5], 3}, {NULL}};
const Segment SGI348622[] = {{&S55404[28], 5}, {NULL}};
const Segment SGI348623[] = {{&S55404[104], 5}, {NULL}};
const Segment SGI348624[] = {{&S37530[6], 4}, {NULL}};
const Segment SGI348625[] = {{&S37530[76], 5}, {NULL}};
const Segment SGI348626[] = {{&S37530[12], 9}, {NULL}};
const Segment SGI348627[] = {{&S37530[29], 13}, {NULL}};
const Segment SGI348628[] = {{&S50943[46], 4}, {NULL}};
const Segment SGI348629[] = {{&S50943[8], 3}, {NULL}};
const Segment SGI348630[] = {{&S50944[46], 4}, {&S50944[146], 7}, {NULL}};
const Segment SGI348631[] = {{&S50944[8], 3}, {NULL}};
const Segment SGI348632[] = {{&S55450[28], 5}, {NULL}};
const Segment SGI348633[] = {{&S55450[104], 5}, {NULL}};
const Segment SGI348634[] = {{&S63939[5], 3}, {&S63939[5], 3}, {NULL}};
const Segment SGI348635[] = {{&S63939[109], 3}, {NULL}};
const Segment SGI348636[] = {{&S63943[5], 3}, {&S63943[5], 3}, {NULL}};
const Segment SGI348637[] = {{&S63943[9], 3}, {NULL}};
const Segment SGI348638[] = {{&S63943[126], 3}, {NULL}};
const Segment SGI348639[] = {{&S63944[5], 7}, {&S63944[5], 7}, {NULL}};
const Segment SGI348640[] = {{&S63944[13], 3}, {NULL}};
const Segment SGI348641[] = {{&S63944[135], 3}, {NULL}};
const Segment SGI348642[] = {{&S55581[28], 5}, {NULL}};
const Segment SGI348643[] = {{&S55581[104], 5}, {NULL}};
const Segment SGI348644[] = {{&S55582[28], 5}, {NULL}};
const Segment SGI348645[] = {{&S55582[104], 5}, {NULL}};
const Segment SGI348646[] = {{&S55583[28], 7}, {NULL}};
const Segment SGI348647[] = {{&S55583[106], 5}, {NULL}};
const Segment SGI348648[] = {{&S40248[61], 5}, {NULL}};
const Segment SGI348649[] = {{&S40248[13], 10}, {NULL}};
const Segment SGI348650[] = {{&S40296[61], 5}, {NULL}};
const Segment SGI348651[] = {{&S40296[13], 10}, {NULL}};
const Segment SGI348652[] = {{&S64061[5], 5}, {&S64061[105], 5}, {NULL}};
const Segment SGI348653[] = {{&S64061[11], 3}, {NULL}};
const Segment SGI348654[] = {{&S64061[131], 3}, {NULL}};
const Segment SGI348655[] = {{&S64095[5], 5}, {&S64095[104], 5}, {NULL}};
const Segment SGI348656[] = {{&S64095[11], 3}, {NULL}};
const Segment SGI348657[] = {{&S64095[130], 3}, {NULL}};
const Segment SGI348658[] = {{&S64098[42], 5}, {NULL}};
const Segment SGI348659[] = {{&S64098[6], 3}, {NULL}};
const Segment SGI348660[] = {{&S64098[68], 3}, {NULL}};
const Segment SGI348661[] = {{&S64099[43], 5}, {NULL}};
const Segment SGI348662[] = {{&S64099[6], 3}, {NULL}};
const Segment SGI348663[] = {{&S64099[69], 3}, {NULL}};
const Segment SGI348664[] = {{&S64100[43], 5}, {NULL}};
const Segment SGI348665[] = {{&S64100[6], 3}, {NULL}};
const Segment SGI348666[] = {{&S64100[69], 3}, {NULL}};
const Segment SGI348667[] = {{&S55701[28], 5}, {NULL}};
const Segment SGI348668[] = {{&S55701[104], 5}, {NULL}};
const Segment SGI348669[] = {{&S55702[28], 7}, {NULL}};
const Segment SGI348670[] = {{&S55702[106], 5}, {NULL}};
const Segment SGI348671[] = {{&S51122[5], 3}, {NULL}};
const Segment SGI348672[] = {{&S51122[32], 3}, {NULL}};
const Segment SGI348673[] = {{&S51123[49], 5}, {NULL}};
const Segment SGI348674[] = {{&S51123[40], 3}, {NULL}};
const Segment SGI348675[] = {{&S51124[49], 5}, {NULL}};
const Segment SGI348676[] = {{&S51124[40], 3}, {NULL}};
const Segment SGI348677[] = {{&S37914[6], 4}, {NULL}};
const Segment SGI348678[] = {{&S37914[58], 5}, {NULL}};
const Segment SGI348679[] = {{&S37914[12], 14}, {NULL}};
const Segment SGI348680[] = {{&S37915[6], 4}, {NULL}};
const Segment SGI348681[] = {{&S37915[81], 5}, {NULL}};
const Segment SGI348682[] = {{&S37915[12], 14}, {&S37915[40], 12}, {NULL}};
const Segment SGI348683[] = {{&S37915[12], 14}, {NULL}};
const Segment SGI348684[] = {{&S37916[6], 4}, {NULL}};
const Segment SGI348685[] = {{&S37916[80], 5}, {NULL}};
const Segment SGI348686[] = {{&S37916[12], 14}, {&S37916[40], 11}, {NULL}};
const Segment SGI348687[] = {{&S37916[12], 14}, {NULL}};
const Segment SGI348688[] = {{&S37917[6], 4}, {NULL}};
const Segment SGI348689[] = {{&S37917[80], 5}, {NULL}};
const Segment SGI348690[] = {{&S37917[12], 14}, {&S37917[40], 11}, {NULL}};
const Segment SGI348691[] = {{&S37917[12], 14}, {NULL}};
const Segment SGI348692[] = {{&S37918[6], 4}, {NULL}};
const Segment SGI348693[] = {{&S37918[81], 5}, {NULL}};
const Segment SGI348694[] = {{&S37918[12], 14}, {&S37918[40], 12}, {NULL}};
const Segment SGI348695[] = {{&S37918[12], 14}, {NULL}};
const Segment SGI348696[] = {{&S37919[6], 4}, {NULL}};
const Segment SGI348697[] = {{&S37919[80], 5}, {NULL}};
const Segment SGI348698[] = {{&S37919[12], 14}, {&S37919[40], 11}, {NULL}};
const Segment SGI348699[] = {{&S37919[12], 14}, {NULL}};
const Segment SGI348700[] = {{&S37920[6], 4}, {NULL}};
const Segment SGI348701[] = {{&S37920[80], 5}, {NULL}};
const Segment SGI348702[] = {{&S37920[12], 14}, {&S37920[40], 11}, {NULL}};
const Segment SGI348703[] = {{&S37920[12], 14}, {NULL}};
const Segment SGI348704[] = {{&S37921[6], 4}, {NULL}};
const Segment SGI348705[] = {{&S37921[80], 5}, {NULL}};
const Segment SGI348706[] = {{&S37921[12], 14}, {&S37921[40], 11}, {NULL}};
const Segment SGI348707[] = {{&S37921[12], 14}, {NULL}};
const Segment SGI348708[] = {{&S37922[6], 4}, {NULL}};
const Segment SGI348709[] = {{&S37922[58], 5}, {NULL}};
const Segment SGI348710[] = {{&S37922[12], 14}, {NULL}};
const Segment SGI348711[] = {{&S37923[6], 4}, {NULL}};
const Segment SGI348712[] = {{&S37923[81], 5}, {NULL}};
const Segment SGI348713[] = {{&S37923[12], 14}, {&S37923[40], 12}, {NULL}};
const Segment SGI348714[] = {{&S37923[12], 14}, {NULL}};
const Segment SGI348715[] = {{&S37924[6], 4}, {NULL}};
const Segment SGI348716[] = {{&S37924[81], 5}, {NULL}};
const Segment SGI348717[] = {{&S37924[12], 14}, {&S37924[40], 12}, {NULL}};
const Segment SGI348718[] = {{&S37924[12], 14}, {NULL}};
const Segment SGI348719[] = {{&S37925[6], 4}, {NULL}};
const Segment SGI348720[] = {{&S37925[80], 5}, {NULL}};
const Segment SGI348721[] = {{&S37925[12], 14}, {&S37925[40], 11}, {NULL}};
const Segment SGI348722[] = {{&S37925[12], 14}, {NULL}};
const Segment SGI348723[] = {{&S37926[6], 4}, {NULL}};
const Segment SGI348724[] = {{&S37926[81], 5}, {NULL}};
const Segment SGI348725[] = {{&S37926[12], 14}, {&S37926[40], 12}, {NULL}};
const Segment SGI348726[] = {{&S37926[12], 14}, {NULL}};
const Segment SGI348727[] = {{&S37927[6], 4}, {NULL}};
const Segment SGI348728[] = {{&S37927[80], 5}, {NULL}};
const Segment SGI348729[] = {{&S37927[12], 14}, {&S37927[40], 11}, {NULL}};
const Segment SGI348730[] = {{&S37927[12], 14}, {NULL}};
const Segment SGI348731[] = {{&S37928[6], 4}, {NULL}};
const Segment SGI348732[] = {{&S37928[81], 5}, {NULL}};
const Segment SGI348733[] = {{&S37928[12], 14}, {&S37928[40], 12}, {NULL}};
const Segment SGI348734[] = {{&S37928[12], 14}, {NULL}};
const Segment SGI348735[] = {{&S37929[6], 4}, {NULL}};
const Segment SGI348736[] = {{&S37929[80], 5}, {NULL}};
const Segment SGI348737[] = {{&S37929[12], 14}, {&S37929[40], 11}, {NULL}};
const Segment SGI348738[] = {{&S37929[12], 14}, {NULL}};
const Segment SGI348739[] = {{&S37930[6], 4}, {NULL}};
const Segment SGI348740[] = {{&S37930[58], 5}, {NULL}};
const Segment SGI348741[] = {{&S37930[12], 14}, {&S37930[69], 31}, {NULL}};
const Segment SGI348742[] = {{&S37930[12], 14}, {NULL}};
const Segment SGI348743[] = {{&S37931[6], 4}, {NULL}};
const Segment SGI348744[] = {{&S37931[81], 5}, {NULL}};
const Segment SGI348745[] = {{&S37931[12], 14}, {&S37931[40], 12}, {NULL}};
const Segment SGI348746[] = {{&S37931[12], 14}, {NULL}};
const Segment SGI348747[] = {{&S37932[6], 4}, {NULL}};
const Segment SGI348748[] = {{&S37932[80], 5}, {NULL}};
const Segment SGI348749[] = {{&S37932[12], 14}, {&S37932[40], 11}, {NULL}};
const Segment SGI348750[] = {{&S37932[12], 14}, {NULL}};
const Segment SGI348751[] = {{&S37933[6], 4}, {NULL}};
const Segment SGI348752[] = {{&S37933[80], 5}, {NULL}};
const Segment SGI348753[] = {{&S37933[12], 14}, {&S37933[40], 11}, {NULL}};
const Segment SGI348754[] = {{&S37933[12], 14}, {NULL}};
const Segment SGI348755[] = {{&S37934[6], 4}, {NULL}};
const Segment SGI348756[] = {{&S37934[80], 5}, {NULL}};
const Segment SGI348757[] = {{&S37934[12], 14}, {&S37934[40], 11}, {NULL}};
const Segment SGI348758[] = {{&S37934[12], 14}, {NULL}};
const Segment SGI348759[] = {{&S37935[6], 4}, {NULL}};
const Segment SGI348760[] = {{&S37935[80], 5}, {NULL}};
const Segment SGI348761[] = {{&S37935[12], 14}, {&S37935[40], 11}, {NULL}};
const Segment SGI348762[] = {{&S37935[12], 14}, {NULL}};
const Segment SGI348763[] = {{&S37936[6], 4}, {NULL}};
const Segment SGI348764[] = {{&S37936[80], 5}, {NULL}};
const Segment SGI348765[] = {{&S37936[12], 14}, {&S37936[40], 11}, {NULL}};
const Segment SGI348766[] = {{&S37936[12], 14}, {NULL}};
const Segment SGI348767[] = {{&S37937[6], 4}, {NULL}};
const Segment SGI348768[] = {{&S37937[81], 5}, {NULL}};
const Segment SGI348769[] = {{&S37937[12], 14}, {&S37937[40], 12}, {NULL}};
const Segment SGI348770[] = {{&S37937[12], 14}, {NULL}};
const Segment SGI348771[] = {{&S37939[6], 4}, {NULL}};
const Segment SGI348772[] = {{&S37939[81], 5}, {NULL}};
const Segment SGI348773[] = {{&S37939[12], 14}, {&S37939[40], 12}, {NULL}};
const Segment SGI348774[] = {{&S37939[12], 14}, {NULL}};
const Segment SGI348775[] = {{&S37940[6], 4}, {NULL}};
const Segment SGI348776[] = {{&S37940[81], 5}, {NULL}};
const Segment SGI348777[] = {{&S37940[12], 14}, {&S37940[40], 12}, {NULL}};
const Segment SGI348778[] = {{&S37940[12], 14}, {NULL}};
const Segment SGI348779[] = {{&S37941[6], 4}, {NULL}};
const Segment SGI348780[] = {{&S37941[81], 5}, {NULL}};
const Segment SGI348781[] = {{&S37941[12], 14}, {&S37941[40], 12}, {NULL}};
const Segment SGI348782[] = {{&S37941[12], 14}, {NULL}};
const Segment SGI348783[] = {{&S37942[6], 4}, {NULL}};
const Segment SGI348784[] = {{&S37942[81], 5}, {NULL}};
const Segment SGI348785[] = {{&S37942[12], 14}, {&S37942[40], 12}, {NULL}};
const Segment SGI348786[] = {{&S37942[12], 14}, {NULL}};
const Segment SGI348787[] = {{&S37943[6], 4}, {NULL}};
const Segment SGI348788[] = {{&S37943[80], 5}, {NULL}};
const Segment SGI348789[] = {{&S37943[12], 14}, {&S37943[40], 11}, {NULL}};
const Segment SGI348790[] = {{&S37943[12], 14}, {NULL}};
const Segment SGI348791[] = {{&S37944[6], 4}, {NULL}};
const Segment SGI348792[] = {{&S37944[80], 5}, {NULL}};
const Segment SGI348793[] = {{&S37944[12], 14}, {&S37944[40], 11}, {NULL}};
const Segment SGI348794[] = {{&S37944[12], 14}, {NULL}};
const Segment SGI348795[] = {{&S37945[6], 4}, {NULL}};
const Segment SGI348796[] = {{&S37945[81], 5}, {NULL}};
const Segment SGI348797[] = {{&S37945[12], 14}, {&S37945[41], 11}, {NULL}};
const Segment SGI348798[] = {{&S37945[12], 14}, {NULL}};
const Segment SGI348799[] = {{&S37946[6], 4}, {NULL}};
const Segment SGI348800[] = {{&S37946[81], 5}, {NULL}};
const Segment SGI348801[] = {{&S37946[12], 14}, {&S37946[40], 12}, {NULL}};
const Segment SGI348802[] = {{&S37946[12], 14}, {NULL}};
const Segment SGI348803[] = {{&S37947[6], 4}, {NULL}};
const Segment SGI348804[] = {{&S37947[80], 5}, {NULL}};
const Segment SGI348805[] = {{&S37947[12], 14}, {&S37947[40], 11}, {NULL}};
const Segment SGI348806[] = {{&S37947[12], 14}, {NULL}};
const Segment SGI348807[] = {{&S37948[6], 4}, {NULL}};
const Segment SGI348808[] = {{&S37948[80], 5}, {NULL}};
const Segment SGI348809[] = {{&S37948[12], 14}, {&S37948[40], 11}, {NULL}};
const Segment SGI348810[] = {{&S37948[12], 14}, {NULL}};
const Segment SGI348811[] = {{&S37950[6], 4}, {NULL}};
const Segment SGI348812[] = {{&S37950[81], 5}, {NULL}};
const Segment SGI348813[] = {{&S37950[12], 14}, {&S37950[41], 11}, {NULL}};
const Segment SGI348814[] = {{&S37950[12], 14}, {NULL}};
const Segment SGI348815[] = {{&S37951[6], 4}, {NULL}};
const Segment SGI348816[] = {{&S37951[81], 5}, {NULL}};
const Segment SGI348817[] = {{&S37951[12], 14}, {&S37951[40], 12}, {NULL}};
const Segment SGI348818[] = {{&S37951[12], 14}, {NULL}};
const Segment SGI348819[] = {{&S37952[6], 4}, {NULL}};
const Segment SGI348820[] = {{&S37952[81], 5}, {NULL}};
const Segment SGI348821[] = {{&S37952[12], 14}, {&S37952[40], 12}, {NULL}};
const Segment SGI348822[] = {{&S37952[12], 14}, {NULL}};
const Segment SGI348823[] = {{&S37953[6], 4}, {NULL}};
const Segment SGI348824[] = {{&S37953[81], 5}, {NULL}};
const Segment SGI348825[] = {{&S37953[12], 14}, {&S37953[40], 12}, {NULL}};
const Segment SGI348826[] = {{&S37953[12], 14}, {NULL}};
const Segment SGI348827[] = {{&S37954[6], 4}, {NULL}};
const Segment SGI348828[] = {{&S37954[81], 5}, {NULL}};
const Segment SGI348829[] = {{&S37954[12], 14}, {&S37954[40], 12}, {NULL}};
const Segment SGI348830[] = {{&S37954[12], 14}, {NULL}};
const Segment SGI348831[] = {{&S37955[6], 4}, {NULL}};
const Segment SGI348832[] = {{&S37955[81], 5}, {NULL}};
const Segment SGI348833[] = {{&S37955[12], 14}, {&S37955[41], 11}, {NULL}};
const Segment SGI348834[] = {{&S37955[12], 14}, {NULL}};
const Segment SGI348835[] = {{&S37956[6], 4}, {NULL}};
const Segment SGI348836[] = {{&S37956[81], 5}, {NULL}};
const Segment SGI348837[] = {{&S37956[12], 14}, {&S37956[41], 11}, {NULL}};
const Segment SGI348838[] = {{&S37956[12], 14}, {NULL}};
const Segment SGI348839[] = {{&S37957[6], 4}, {NULL}};
const Segment SGI348840[] = {{&S37957[81], 5}, {NULL}};
const Segment SGI348841[] = {{&S37957[12], 14}, {&S37957[41], 11}, {NULL}};
const Segment SGI348842[] = {{&S37957[12], 14}, {NULL}};
const Segment SGI348843[] = {{&S37958[6], 4}, {NULL}};
const Segment SGI348844[] = {{&S37958[80], 5}, {NULL}};
const Segment SGI348845[] = {{&S37958[12], 14}, {&S37958[40], 11}, {NULL}};
const Segment SGI348846[] = {{&S37958[12], 14}, {NULL}};
const Segment SGI348847[] = {{&S37959[6], 4}, {NULL}};
const Segment SGI348848[] = {{&S37959[81], 5}, {NULL}};
const Segment SGI348849[] = {{&S37959[12], 14}, {&S37959[41], 11}, {NULL}};
const Segment SGI348850[] = {{&S37959[12], 14}, {NULL}};
const Segment SGI348851[] = {{&S64164[5], 3}, {NULL}};
const Segment SGI348852[] = {{&S64164[19], 3}, {NULL}};
const Segment SGI348853[] = {{&S37960[6], 4}, {NULL}};
const Segment SGI348854[] = {{&S37960[80], 5}, {NULL}};
const Segment SGI348855[] = {{&S37960[12], 14}, {&S37960[40], 11}, {NULL}};
const Segment SGI348856[] = {{&S37960[12], 14}, {NULL}};
const Segment SGI348857[] = {{&S37961[6], 4}, {NULL}};
const Segment SGI348858[] = {{&S37961[108], 5}, {NULL}};
const Segment SGI348859[] = {{&S37961[12], 14}, {&S37961[40], 12}, {NULL}};
const Segment SGI348860[] = {{&S37961[12], 14}, {NULL}};
const Segment SGI348861[] = {{&S37962[6], 4}, {NULL}};
const Segment SGI348862[] = {{&S37962[80], 5}, {NULL}};
const Segment SGI348863[] = {{&S37962[12], 14}, {&S37962[40], 11}, {NULL}};
const Segment SGI348864[] = {{&S37962[12], 14}, {NULL}};
const Segment SGI348865[] = {{&S37963[6], 4}, {NULL}};
const Segment SGI348866[] = {{&S37963[80], 5}, {NULL}};
const Segment SGI348867[] = {{&S37963[12], 14}, {&S37963[40], 11}, {NULL}};
const Segment SGI348868[] = {{&S37963[12], 14}, {NULL}};
const Segment SGI348869[] = {{&S37964[6], 4}, {NULL}};
const Segment SGI348870[] = {{&S37964[81], 5}, {NULL}};
const Segment SGI348871[] = {{&S37964[12], 14}, {&S37964[40], 12}, {NULL}};
const Segment SGI348872[] = {{&S37964[12], 14}, {NULL}};
const Segment SGI348873[] = {{&S37965[6], 4}, {NULL}};
const Segment SGI348874[] = {{&S37965[80], 5}, {NULL}};
const Segment SGI348875[] = {{&S37965[12], 14}, {&S37965[40], 11}, {NULL}};
const Segment SGI348876[] = {{&S37965[12], 14}, {NULL}};
const Segment SGI348877[] = {{&S37966[6], 4}, {NULL}};
const Segment SGI348878[] = {{&S37966[81], 5}, {NULL}};
const Segment SGI348879[] = {{&S37966[12], 14}, {&S37966[41], 11}, {NULL}};
const Segment SGI348880[] = {{&S37966[12], 14}, {NULL}};
const Segment SGI348881[] = {{&S64173[21], 3}, {NULL}};
const Segment SGI348882[] = {{&S37967[6], 4}, {NULL}};
const Segment SGI348883[] = {{&S64173[0], 14}, {NULL}};
const Segment SGI348884[] = {{&S37967[58], 5}, {NULL}};
const Segment SGI348885[] = {{&S37967[12], 14}, {&S37967[69], 4}, {NULL}};
const Segment SGI348886[] = {{&S37967[12], 14}, {NULL}};
const Segment SGI348887[] = {{&S64174[42], 5}, {NULL}};
const Segment SGI348888[] = {{&S37968[6], 4}, {NULL}};
const Segment SGI348889[] = {{&S64174[6], 3}, {NULL}};
const Segment SGI348890[] = {{&S37968[81], 5}, {NULL}};
const Segment SGI348891[] = {{&S37968[12], 14}, {&S37968[41], 11}, {NULL}};
const Segment SGI348892[] = {{&S64174[68], 3}, {NULL}};
const Segment SGI348893[] = {{&S37968[12], 14}, {NULL}};
const Segment SGI348894[] = {{&S37969[6], 4}, {NULL}};
const Segment SGI348895[] = {{&S37969[80], 5}, {NULL}};
const Segment SGI348896[] = {{&S37969[12], 14}, {&S37969[40], 11}, {NULL}};
const Segment SGI348897[] = {{&S37969[12], 14}, {NULL}};
const Segment SGI348898[] = {{&S37970[6], 4}, {NULL}};
const Segment SGI348899[] = {{&S37970[80], 5}, {NULL}};
const Segment SGI348900[] = {{&S37970[12], 14}, {&S37970[40], 11}, {NULL}};
const Segment SGI348901[] = {{&S37970[12], 14}, {NULL}};
const Segment SGI348902[] = {{&S37971[6], 4}, {NULL}};
const Segment SGI348903[] = {{&S37971[80], 5}, {NULL}};
const Segment SGI348904[] = {{&S37971[12], 14}, {&S37971[40], 11}, {NULL}};
const Segment SGI348905[] = {{&S37971[12], 14}, {NULL}};
const Segment SGI348906[] = {{&S37972[6], 4}, {NULL}};
const Segment SGI348907[] = {{&S37972[80], 5}, {NULL}};
const Segment SGI348908[] = {{&S37972[12], 14}, {&S37972[40], 11}, {NULL}};
const Segment SGI348909[] = {{&S37972[12], 14}, {NULL}};
const Segment SGI348910[] = {{&S37973[6], 4}, {NULL}};
const Segment SGI348911[] = {{&S37973[80], 5}, {NULL}};
const Segment SGI348912[] = {{&S37973[12], 14}, {&S37973[40], 11}, {NULL}};
const Segment SGI348913[] = {{&S37973[12], 14}, {NULL}};
const Segment SGI348914[] = {{&S37974[6], 4}, {NULL}};
const Segment SGI348915[] = {{&S37974[80], 5}, {NULL}};
const Segment SGI348916[] = {{&S37974[12], 14}, {&S37974[40], 11}, {NULL}};
const Segment SGI348917[] = {{&S37974[12], 14}, {NULL}};
const Segment SGI348918[] = {{&S37975[6], 4}, {NULL}};
const Segment SGI348919[] = {{&S37975[80], 5}, {NULL}};
const Segment SGI348920[] = {{&S37975[12], 14}, {&S37975[40], 11}, {NULL}};
const Segment SGI348921[] = {{&S37975[12], 14}, {NULL}};
const Segment SGI348922[] = {{&S37976[6], 4}, {NULL}};
const Segment SGI348923[] = {{&S37976[81], 5}, {NULL}};
const Segment SGI348924[] = {{&S37976[12], 14}, {&S37976[41], 11}, {NULL}};
const Segment SGI348925[] = {{&S37976[12], 14}, {NULL}};
const Segment SGI348926[] = {{&S37977[6], 4}, {NULL}};
const Segment SGI348927[] = {{&S37977[80], 5}, {NULL}};
const Segment SGI348928[] = {{&S37977[12], 14}, {&S37977[40], 11}, {NULL}};
const Segment SGI348929[] = {{&S37977[12], 14}, {NULL}};
const Segment SGI348930[] = {{&S55865[28], 3}, {NULL}};
const Segment SGI348931[] = {{&S55865[102], 5}, {NULL}};
const Segment SGI348932[] = {{&S55866[28], 3}, {NULL}};
const Segment SGI348933[] = {{&S55866[102], 5}, {NULL}};
const Segment SGI348934[] = {{&S55883[28], 3}, {NULL}};
const Segment SGI348935[] = {{&S55883[102], 5}, {NULL}};
const Segment SGI348936[] = {{&S64225[35], 5}, {NULL}};
const Segment SGI348937[] = {{&S64225[52], 3}, {NULL}};
const Segment SGI348938[] = {{&S64226[5], 5}, {NULL}};
const Segment SGI348939[] = {{&S64226[16], 3}, {NULL}};
const Segment SGI348940[] = {{&S64248[5], 5}, {&S64248[104], 5}, {NULL}};
const Segment SGI348941[] = {{&S64248[69], 3}, {NULL}};
const Segment SGI348942[] = {{&S64248[130], 3}, {NULL}};
const Segment SGI348943[] = {{&S38054[6], 4}, {NULL}};
const Segment SGI348944[] = {{&S38054[52], 5}, {NULL}};
const Segment SGI348945[] = {{&S38054[12], 45}, {NULL}};
const Segment SGI348946[] = {{&S38054[12], 14}, {NULL}};
const Segment SGI348947[] = {{&S38055[6], 4}, {NULL}};
const Segment SGI348948[] = {{&S38055[59], 5}, {NULL}};
const Segment SGI348949[] = {{&S38055[12], 14}, {NULL}};
const Segment SGI348950[] = {{&S38056[6], 4}, {NULL}};
const Segment SGI348951[] = {{&S38056[81], 5}, {NULL}};
const Segment SGI348952[] = {{&S38056[12], 14}, {&S38056[44], 42}, {NULL}};
const Segment SGI348953[] = {{&S38056[12], 14}, {NULL}};
const Segment SGI348954[] = {{&S38057[6], 4}, {NULL}};
const Segment SGI348955[] = {{&S38057[81], 5}, {NULL}};
const Segment SGI348956[] = {{&S38057[12], 14}, {&S38057[44], 42}, {NULL}};
const Segment SGI348957[] = {{&S38057[12], 14}, {NULL}};
const Segment SGI348958[] = {{&S38058[6], 4}, {NULL}};
const Segment SGI348959[] = {{&S38058[81], 5}, {NULL}};
const Segment SGI348960[] = {{&S38058[12], 14}, {&S38058[44], 42}, {NULL}};
const Segment SGI348961[] = {{&S38058[12], 14}, {NULL}};
const Segment SGI348962[] = {{&S38059[6], 4}, {NULL}};
const Segment SGI348963[] = {{&S38059[81], 5}, {NULL}};
const Segment SGI348964[] = {{&S38059[12], 14}, {&S38059[44], 42}, {NULL}};
const Segment SGI348965[] = {{&S38059[12], 14}, {NULL}};
const Segment SGI348966[] = {{&S38060[6], 4}, {NULL}};
const Segment SGI348967[] = {{&S38060[59], 5}, {NULL}};
const Segment SGI348968[] = {{&S38060[12], 14}, {&S38060[70], 4}, {NULL}};
const Segment SGI348969[] = {{&S38060[12], 14}, {NULL}};
const Segment SGI348970[] = {{&S64322[5], 5}, {&S64322[105], 5}, {NULL}};
const Segment SGI348971[] = {{&S64322[69], 3}, {NULL}};
const Segment SGI348972[] = {{&S64322[131], 3}, {NULL}};
const Segment SGI348973[] = {{&S64323[5], 3}, {NULL}};
const Segment SGI348974[] = {{&S64349[5], 3}, {&S64349[5], 3}, {NULL}};
const Segment SGI348975[] = {{&S64349[9], 3}, {NULL}};
const Segment SGI348976[] = {{&S64349[127], 3}, {NULL}};
const Segment SGI348977[] = {{&S38143[6], 4}, {NULL}};
const Segment SGI348978[] = {{&S38143[12], 14}, {NULL}};
const Segment SGI348979[] = {{&S38144[6], 4}, {NULL}};
const Segment SGI348980[] = {{&S38144[12], 14}, {NULL}};
const Segment SGI348981[] = {{&S38145[6], 4}, {NULL}};
const Segment SGI348982[] = {{&S38145[12], 14}, {NULL}};
const Segment SGI348983[] = {{&S38146[6], 4}, {NULL}};
const Segment SGI348984[] = {{&S38146[12], 14}, {NULL}};
const Segment SGI348985[] = {{&S38147[6], 4}, {NULL}};
const Segment SGI348986[] = {{&S38147[12], 14}, {&S38147[40], 12}, {NULL}};
const Segment SGI348987[] = {{&S38147[12], 14}, {NULL}};
const Segment SGI348988[] = {{&S38148[6], 4}, {NULL}};
const Segment SGI348989[] = {{&S38148[12], 14}, {NULL}};
const Segment SGI348990[] = {{&S38149[6], 4}, {NULL}};
const Segment SGI348991[] = {{&S38149[12], 14}, {&S38149[40], 12}, {NULL}};
const Segment SGI348992[] = {{&S38149[12], 14}, {NULL}};
const Segment SGI348993[] = {{&S38180[6], 4}, {NULL}};
const Segment SGI348994[] = {{&S38180[12], 10}, {NULL}};
const Segment SGI348995[] = {{&S38187[6], 4}, {NULL}};
const Segment SGI348996[] = {{&S38187[12], 10}, {NULL}};
const Segment SGI348997[] = {{&S56080[28], 5}, {NULL}};
const Segment SGI348998[] = {{&S56080[104], 5}, {NULL}};
const Segment SGI348999[] = {{&S56162[28], 3}, {NULL}};
const Segment SGI349000[] = {{&S56162[102], 5}, {NULL}};
const Segment SGI349001[] = {{&S56163[28], 5}, {NULL}};
const Segment SGI349002[] = {{&S56163[104], 5}, {NULL}};
const Segment SGI349003[] = {{&S64422[5], 4}, {&S64422[5], 4}, {NULL}};
const Segment SGI349004[] = {{&S64422[10], 3}, {NULL}};
const Segment SGI349005[] = {{&S64422[129], 3}, {NULL}};
const Segment SGI349006[] = {{&S64423[42], 4}, {NULL}};
const Segment SGI349007[] = {{&S64423[6], 3}, {NULL}};
const Segment SGI349008[] = {{&S64423[67], 3}, {NULL}};
const Segment SGI349009[] = {{&S64437[5], 3}, {&S64437[5], 3}, {NULL}};
const Segment SGI349010[] = {{&S64437[67], 3}, {NULL}};
const Segment SGI349011[] = {{&S64437[127], 3}, {NULL}};
const Segment SGI349012[] = {{&S64443[5], 3}, {&S64443[5], 3}, {NULL}};
const Segment SGI349013[] = {{&S64443[107], 3}, {NULL}};
const Segment SGI349014[] = {{&S64444[35], 5}, {NULL}};
const Segment SGI349015[] = {{&S64444[52], 3}, {NULL}};
const Segment SGI349016[] = {{&S64445[5], 5}, {NULL}};
const Segment SGI349017[] = {{&S64445[16], 3}, {NULL}};
const Segment SGI349018[] = {{&S40812[71], 4}, {NULL}};
const Segment SGI349019[] = {{&S40812[13], 10}, {NULL}};
const Segment SGI349020[] = {{&S64517[5], 3}, {&S64517[5], 3}, {NULL}};
const Segment SGI349021[] = {{&S64517[9], 3}, {NULL}};
const Segment SGI349022[] = {{&S64517[127], 3}, {NULL}};
const Segment SGI349023[] = {{&S64518[5], 5}, {&S64518[5], 5}, {NULL}};
const Segment SGI349024[] = {{&S64518[11], 3}, {NULL}};
const Segment SGI349025[] = {{&S64518[131], 3}, {NULL}};
const Segment SGI349026[] = {{&S38405[6], 4}, {NULL}};
const Segment SGI349027[] = {{&S38405[57], 5}, {NULL}};
const Segment SGI349028[] = {{&S38405[12], 14}, {NULL}};
const Segment SGI349029[] = {{&S64574[5], 7}, {&S64574[5], 7}, {NULL}};
const Segment SGI349030[] = {{&S64574[117], 3}, {NULL}};
const Segment SGI349031[] = {{&S64613[5], 5}, {&S64613[105], 5}, {NULL}};
const Segment SGI349032[] = {{&S64613[69], 3}, {NULL}};
const Segment SGI349033[] = {{&S64613[131], 3}, {NULL}};
const Segment SGI349034[] = {{&S64614[5], 7}, {&S64614[107], 7}, {NULL}};
const Segment SGI349035[] = {{&S64614[71], 3}, {NULL}};
const Segment SGI349036[] = {{&S64614[135], 3}, {NULL}};
const Segment SGI349037[] = {{&S64623[5], 5}, {&S64623[105], 5}, {NULL}};
const Segment SGI349038[] = {{&S64623[69], 3}, {NULL}};
const Segment SGI349039[] = {{&S64623[131], 3}, {NULL}};
const Segment SGI349040[] = {{&S64740[5], 3}, {NULL}};
const Segment SGI349041[] = {{&S64752[5], 3}, {&S64752[5], 3}, {NULL}};
const Segment SGI349042[] = {{&S64752[9], 3}, {NULL}};
const Segment SGI349043[] = {{&S64752[127], 3}, {NULL}};
const Segment SGI349044[] = {{&S38656[6], 3}, {NULL}};
const Segment SGI349045[] = {{&S38656[11], 14}, {NULL}};
const Segment SGI349046[] = {{&S38658[6], 4}, {NULL}};
const Segment SGI349047[] = {{&S38658[12], 14}, {NULL}};
const Segment SGI349048[] = {{&S64846[42], 3}, {NULL}};
const Segment SGI349049[] = {{&S64846[6], 3}, {NULL}};
const Segment SGI349050[] = {{&S64846[66], 3}, {NULL}};
const Segment SGI349051[] = {{&S64847[42], 7}, {NULL}};
const Segment SGI349052[] = {{&S64847[6], 3}, {NULL}};
const Segment SGI349053[] = {{&S64847[70], 3}, {NULL}};
const Segment SGI349054[] = {{&S64848[43], 3}, {NULL}};
const Segment SGI349055[] = {{&S64848[6], 3}, {NULL}};
const Segment SGI349056[] = {{&S64848[67], 3}, {NULL}};
const Segment SGI349057[] = {{&S64849[43], 7}, {NULL}};
const Segment SGI349058[] = {{&S64849[6], 3}, {NULL}};
const Segment SGI349059[] = {{&S64849[71], 3}, {NULL}};
const Segment SGI349060[] = {{&S38706[6], 4}, {NULL}};
const Segment SGI349061[] = {{&S38706[57], 5}, {NULL}};
const Segment SGI349062[] = {{&S38706[12], 14}, {NULL}};
const Segment SGI349063[] = {{&S38707[6], 4}, {NULL}};
const Segment SGI349064[] = {{&S38707[57], 5}, {NULL}};
const Segment SGI349065[] = {{&S38707[12], 14}, {NULL}};
const Segment SGI349066[] = {{&S38708[6], 4}, {NULL}};
const Segment SGI349067[] = {{&S38708[57], 5}, {NULL}};
const Segment SGI349068[] = {{&S38708[12], 14}, {NULL}};
const Segment SGI349069[] = {{&S38709[6], 4}, {NULL}};
const Segment SGI349070[] = {{&S38709[57], 5}, {NULL}};
const Segment SGI349071[] = {{&S38709[12], 14}, {NULL}};
const Segment SGI349072[] = {{&S38710[6], 4}, {NULL}};
const Segment SGI349073[] = {{&S38710[80], 5}, {NULL}};
const Segment SGI349074[] = {{&S38710[12], 14}, {&S38710[40], 12}, {NULL}};
const Segment SGI349075[] = {{&S38710[12], 14}, {NULL}};
const Segment SGI349076[] = {{&S38711[6], 4}, {NULL}};
const Segment SGI349077[] = {{&S38711[80], 5}, {NULL}};
const Segment SGI349078[] = {{&S38711[12], 14}, {&S38711[40], 12}, {NULL}};
const Segment SGI349079[] = {{&S38711[12], 14}, {NULL}};
const Segment SGI349080[] = {{&S38712[6], 4}, {NULL}};
const Segment SGI349081[] = {{&S38712[80], 5}, {NULL}};
const Segment SGI349082[] = {{&S38712[12], 14}, {&S38712[40], 12}, {NULL}};
const Segment SGI349083[] = {{&S38712[12], 14}, {NULL}};
const Segment SGI349084[] = {{&S64862[5], 3}, {NULL}};
const Segment SGI349085[] = {{&S38713[6], 4}, {NULL}};
const Segment SGI349086[] = {{&S38713[80], 5}, {NULL}};
const Segment SGI349087[] = {{&S38713[12], 14}, {&S38713[40], 12}, {NULL}};
const Segment SGI349088[] = {{&S38713[12], 14}, {NULL}};
const Segment SGI349089[] = {{&S38714[6], 4}, {NULL}};
const Segment SGI349090[] = {{&S38714[80], 5}, {NULL}};
const Segment SGI349091[] = {{&S38714[12], 14}, {&S38714[40], 12}, {NULL}};
const Segment SGI349092[] = {{&S38714[12], 14}, {NULL}};
const Segment SGI349093[] = {{&S64864[5], 3}, {&S64864[5], 3}, {NULL}};
const Segment SGI349094[] = {{&S64864[9], 3}, {NULL}};
const Segment SGI349095[] = {{&S64864[126], 3}, {NULL}};
const Segment SGI349096[] = {{&S38729[6], 4}, {NULL}};
const Segment SGI349097[] = {{&S38729[12], 12}, {NULL}};
const Segment SGI349098[] = {{&S38732[6], 3}, {NULL}};
const Segment SGI349099[] = {{&S38732[11], 12}, {NULL}};
const Segment SGI349100[] = {{&S38734[6], 4}, {NULL}};
const Segment SGI349101[] = {{&S38734[12], 12}, {NULL}};
const Segment SGI349102[] = {{&S38735[6], 4}, {NULL}};
const Segment SGI349103[] = {{&S38735[12], 12}, {NULL}};
const Segment SGI349104[] = {{&S64964[5], 14}, {NULL}};
const Segment SGI349105[] = {{&S64964[20], 3}, {NULL}};
const Segment SGI349106[] = {{&S64965[5], 16}, {NULL}};
const Segment SGI349107[] = {{&S64965[22], 3}, {NULL}};
const Segment SGI349108[] = {{&S64991[5], 3}, {&S64991[5], 3}, {NULL}};
const Segment SGI349109[] = {{&S64991[9], 3}, {NULL}};
const Segment SGI349110[] = {{&S64991[127], 3}, {NULL}};
const Segment SGI349111[] = {{&S64999[38], 3}, {NULL}};
const Segment SGI349112[] = {{&S64999[92], 3}, {NULL}};
const Segment SGI349113[] = {{&S65000[41], 3}, {NULL}};
const Segment SGI349114[] = {{&S65000[6], 3}, {NULL}};
const Segment SGI349115[] = {{&S65000[65], 3}, {NULL}};
const Segment SGI349116[] = {{&S65001[5], 12}, {NULL}};
const Segment SGI349117[] = {{&S65001[18], 3}, {NULL}};
const Segment SGI349118[] = {{&S65012[5], 4}, {&S65012[5], 4}, {NULL}};
const Segment SGI349119[] = {{&S65012[10], 3}, {NULL}};
const Segment SGI349120[] = {{&S65012[128], 3}, {NULL}};
const Segment SGI349121[] = {{&S65026[5], 7}, {&S65026[5], 7}, {NULL}};
const Segment SGI349122[] = {{&S65026[13], 3}, {NULL}};
const Segment SGI349123[] = {{&S65026[135], 3}, {NULL}};
const Segment SGI349124[] = {{&S65027[5], 4}, {&S65027[5], 4}, {NULL}};
const Segment SGI349125[] = {{&S65027[10], 3}, {NULL}};
const Segment SGI349126[] = {{&S65027[129], 3}, {NULL}};
const Segment SGI349127[] = {{&S38890[6], 4}, {NULL}};
const Segment SGI349128[] = {{&S38890[59], 5}, {NULL}};
const Segment SGI349129[] = {{&S38890[12], 3}, {NULL}};
const Segment SGI349130[] = {{&S65049[5], 4}, {NULL}};
const Segment SGI349131[] = {{&S65050[5], 7}, {NULL}};
const Segment SGI349132[] = {{&S65073[35], 4}, {NULL}};
const Segment SGI349133[] = {{&S65073[90], 3}, {NULL}};
const Segment SGI349134[] = {{&S65074[5], 13}, {NULL}};
const Segment SGI349135[] = {{&S65074[19], 3}, {NULL}};
const Segment SGI349136[] = {{&S65075[41], 4}, {NULL}};
const Segment SGI349137[] = {{&S65075[6], 3}, {NULL}};
const Segment SGI349138[] = {{&S65075[66], 3}, {NULL}};
const Segment SGI349139[] = {{&S65088[5], 5}, {&S65088[105], 5}, {NULL}};
const Segment SGI349140[] = {{&S65088[69], 3}, {NULL}};
const Segment SGI349141[] = {{&S65088[131], 3}, {NULL}};
const Segment SGI349142[] = {{&S65089[5], 5}, {&S65089[105], 5}, {NULL}};
const Segment SGI349143[] = {{&S65089[11], 3}, {NULL}};
const Segment SGI349144[] = {{&S65089[131], 3}, {NULL}};
const Segment SGI349145[] = {{&S65130[5], 5}, {&S65130[5], 5}, {NULL}};
const Segment SGI349146[] = {{&S65130[153], 3}, {NULL}};
const Segment SGI349147[] = {{&S65137[5], 3}, {&S65137[5], 3}, {NULL}};
const Segment SGI349148[] = {{&S65137[9], 3}, {NULL}};
const Segment SGI349149[] = {{&S65137[127], 3}, {NULL}};
const Segment SGI349150[] = {{&S65201[5], 5}, {&S65201[68], 5}, {NULL}};
const Segment SGI349151[] = {{&S65202[5], 10}, {NULL}};
const Segment SGI349152[] = {{&S65203[5], 5}, {NULL}};
const Segment SGI349153[] = {{&S65205[43], 4}, {NULL}};
const Segment SGI349154[] = {{&S65205[6], 3}, {NULL}};
const Segment SGI349155[] = {{&S65205[68], 3}, {NULL}};
const Segment SGI349156[] = {{&S65222[5], 5}, {&S65222[105], 5}, {NULL}};
const Segment SGI349157[] = {{&S65222[11], 3}, {NULL}};
const Segment SGI349158[] = {{&S65222[131], 3}, {NULL}};
const Segment SGI349159[] = {{&S56793[28], 3}, {NULL}};
const Segment SGI349160[] = {{&S56793[102], 5}, {NULL}};
const Segment SGI349161[] = {{&S65259[5], 5}, {&S65259[68], 5}, {NULL}};
const Segment SGI349162[] = {{&S65259[84], 3}, {NULL}};
const Segment SGI349163[] = {{&S41441[58], 4}, {NULL}};
const Segment SGI349164[] = {{&S41441[13], 10}, {NULL}};
const Segment SGI349165[] = {{&S56854[28], 5}, {NULL}};
const Segment SGI349166[] = {{&S56854[105], 5}, {NULL}};
const Segment SGI349167[] = {{&S56855[28], 3}, {NULL}};
const Segment SGI349168[] = {{&S56855[102], 5}, {NULL}};
const Segment SGI349169[] = {{&S56856[28], 5}, {NULL}};
const Segment SGI349170[] = {{&S56856[104], 5}, {NULL}};
const Segment SGI349171[] = {{&S56871[28], 3}, {NULL}};
const Segment SGI349172[] = {{&S56871[102], 5}, {NULL}};
const Segment SGI349173[] = {{&S65335[5], 4}, {NULL}};
const Segment SGI349174[] = {{&S65336[5], 8}, {NULL}};
const Segment SGI349175[] = {{&S41445[71], 4}, {NULL}};
const Segment SGI349176[] = {{&S65345[35], 3}, {NULL}};
const Segment SGI349177[] = {{&S41445[13], 11}, {NULL}};
const Segment SGI349178[] = {{&S65345[89], 3}, {NULL}};
const Segment SGI349179[] = {{&S65346[5], 12}, {NULL}};
const Segment SGI349180[] = {{&S65346[18], 3}, {NULL}};
const Segment SGI349181[] = {{&S65347[41], 5}, {NULL}};
const Segment SGI349182[] = {{&S65347[6], 3}, {NULL}};
const Segment SGI349183[] = {{&S65347[67], 3}, {NULL}};
const Segment SGI349184[] = {{&S65348[5], 3}, {NULL}};
const Segment SGI349185[] = {{&S65348[14], 3}, {NULL}};
const Segment SGI349186[] = {{&S65384[5], 4}, {NULL}};
const Segment SGI349187[] = {{&S65413[5], 5}, {&S65413[97], 5}, {NULL}};
const Segment SGI349188[] = {{&S65413[113], 3}, {NULL}};
const Segment SGI349189[] = {{&S56980[28], 3}, {NULL}};
const Segment SGI349190[] = {{&S56980[102], 5}, {NULL}};
const Segment SGI349191[] = {{&S56981[28], 5}, {NULL}};
const Segment SGI349192[] = {{&S56981[104], 5}, {NULL}};
const Segment SGI349193[] = {{&S56982[28], 5}, {NULL}};
const Segment SGI349194[] = {{&S56982[104], 5}, {NULL}};
const Segment SGI349195[] = {{&S56983[28], 5}, {NULL}};
const Segment SGI349196[] = {{&S56983[104], 5}, {NULL}};
const Segment SGI349197[] = {{&S56984[28], 5}, {NULL}};
const Segment SGI349198[] = {{&S56984[104], 5}, {NULL}};
const Segment SGI349199[] = {{&S65460[43], 5}, {NULL}};
const Segment SGI349200[] = {{&S65460[6], 3}, {NULL}};
const Segment SGI349201[] = {{&S65460[69], 3}, {NULL}};
const Segment SGI349202[] = {{&S65461[5], 5}, {NULL}};
const Segment SGI349203[] = {{&S65461[16], 3}, {NULL}};
const Segment SGI349204[] = {{&S65467[5], 3}, {&S65467[5], 3}, {NULL}};
const Segment SGI349205[] = {{&S65467[67], 3}, {NULL}};
const Segment SGI349206[] = {{&S65467[126], 3}, {NULL}};
const Segment SGI349207[] = {{&S65472[43], 5}, {NULL}};
const Segment SGI349208[] = {{&S65472[6], 3}, {NULL}};
const Segment SGI349209[] = {{&S65472[69], 3}, {NULL}};
const Segment SGI349210[] = {{&S65522[5], 4}, {&S65522[5], 4}, {NULL}};
const Segment SGI349211[] = {{&S65522[10], 3}, {NULL}};
const Segment SGI349212[] = {{&S65522[128], 3}, {NULL}};
const Segment SGI349213[] = {{&S65523[41], 4}, {NULL}};
const Segment SGI349214[] = {{&S65523[6], 3}, {NULL}};
const Segment SGI349215[] = {{&S65523[66], 3}, {NULL}};
const Segment SGI349216[] = {{&S57115[28], 5}, {NULL}};
const Segment SGI349217[] = {{&S57115[104], 5}, {NULL}};
const Segment SGI349218[] = {{&S57116[28], 7}, {NULL}};
const Segment SGI349219[] = {{&S57116[107], 5}, {NULL}};
const Segment SGI349220[] = {{&S41651[61], 5}, {NULL}};
const Segment SGI349221[] = {{&S41651[13], 10}, {NULL}};
const Segment SGI349222[] = {{&S65554[5], 4}, {NULL}};
const Segment SGI349223[] = {{&S65559[5], 5}, {&S65559[97], 5}, {NULL}};
const Segment SGI349224[] = {{&S65559[113], 3}, {NULL}};
const Segment SGI349225[] = {{&S65633[5], 7}, {&S65633[5], 7}, {NULL}};
const Segment SGI349226[] = {{&S65633[157], 3}, {NULL}};
const Segment SGI349227[] = {{&S65634[38], 5}, {NULL}};
const Segment SGI349228[] = {{&S65634[94], 3}, {NULL}};
const Segment SGI349229[] = {{&S65635[41], 5}, {NULL}};
const Segment SGI349230[] = {{&S65635[6], 3}, {NULL}};
const Segment SGI349231[] = {{&S65635[67], 3}, {NULL}};
const Segment SGI349232[] = {{&S65636[5], 14}, {NULL}};
const Segment SGI349233[] = {{&S65636[20], 3}, {NULL}};
const Segment SGI349234[] = {{&S65684[49], 20}, {NULL}};
const Segment SGI349235[] = {{&S65685[49], 20}, {NULL}};
const Segment SGI349236[] = {{&S65686[49], 20}, {NULL}};
const Segment SGI349237[] = {{&S65687[49], 20}, {NULL}};
const Segment SGI349238[] = {{&S41817[64], 5}, {NULL}};
const Segment SGI349239[] = {{&S41817[13], 10}, {NULL}};
const Segment SGI349240[] = {{&S65697[35], 5}, {NULL}};
const Segment SGI349241[] = {{&S65697[91], 3}, {NULL}};
const Segment SGI349242[] = {{&S65712[42], 4}, {NULL}};
const Segment SGI349243[] = {{&S65712[6], 3}, {NULL}};
const Segment SGI349244[] = {{&S65712[67], 3}, {NULL}};
const Segment SGI349245[] = {{&S65721[35], 4}, {NULL}};
const Segment SGI349246[] = {{&S65721[90], 3}, {NULL}};
const Segment SGI349247[] = {{&S65722[5], 4}, {NULL}};
const Segment SGI349248[] = {{&S65722[15], 3}, {NULL}};
const Segment SGI349249[] = {{&S39222[6], 4}, {NULL}};
const Segment SGI349250[] = {{&S39222[57], 5}, {NULL}};
const Segment SGI349251[] = {{&S39222[12], 14}, {NULL}};
const Segment SGI349252[] = {{&S39223[6], 4}, {NULL}};
const Segment SGI349253[] = {{&S39223[57], 5}, {NULL}};
const Segment SGI349254[] = {{&S39223[12], 14}, {NULL}};
const Segment SGI349255[] = {{&S39224[6], 4}, {NULL}};
const Segment SGI349256[] = {{&S39224[57], 5}, {NULL}};
const Segment SGI349257[] = {{&S39224[12], 14}, {NULL}};
const Segment SGI349258[] = {{&S39225[6], 4}, {NULL}};
const Segment SGI349259[] = {{&S39225[57], 5}, {NULL}};
const Segment SGI349260[] = {{&S39225[12], 14}, {NULL}};
const Segment SGI349261[] = {{&S39226[6], 4}, {NULL}};
const Segment SGI349262[] = {{&S39226[57], 5}, {NULL}};
const Segment SGI349263[] = {{&S39226[12], 14}, {NULL}};
const Segment SGI349264[] = {{&S39227[6], 4}, {NULL}};
const Segment SGI349265[] = {{&S39227[57], 5}, {NULL}};
const Segment SGI349266[] = {{&S39227[12], 14}, {&S39227[68], 31}, {NULL}};
const Segment SGI349267[] = {{&S39227[12], 14}, {NULL}};
const Segment SGI349268[] = {{&S39228[6], 4}, {NULL}};
const Segment SGI349269[] = {{&S39228[80], 5}, {NULL}};
const Segment SGI349270[] = {{&S39228[12], 14}, {&S39228[40], 12}, {NULL}};
const Segment SGI349271[] = {{&S39228[12], 14}, {NULL}};
const Segment SGI349272[] = {{&S39229[6], 4}, {NULL}};
const Segment SGI349273[] = {{&S39229[80], 5}, {NULL}};
const Segment SGI349274[] = {{&S39229[12], 14}, {&S39229[40], 12}, {NULL}};
const Segment SGI349275[] = {{&S39229[12], 14}, {NULL}};
const Segment SGI349276[] = {{&S65752[5], 5}, {NULL}};
const Segment SGI349277[] = {{&S39230[6], 4}, {NULL}};
const Segment SGI349278[] = {{&S39230[80], 5}, {NULL}};
const Segment SGI349279[] = {{&S39230[12], 14}, {&S39230[41], 11}, {NULL}};
const Segment SGI349280[] = {{&S39230[12], 14}, {NULL}};
const Segment SGI349281[] = {{&S65753[5], 4}, {NULL}};
const Segment SGI349282[] = {{&S39231[6], 4}, {NULL}};
const Segment SGI349283[] = {{&S39231[80], 5}, {NULL}};
const Segment SGI349284[] = {{&S39231[12], 14}, {&S39231[40], 12}, {NULL}};
const Segment SGI349285[] = {{&S39231[12], 14}, {NULL}};
const Segment SGI349286[] = {{&S39232[6], 4}, {NULL}};
const Segment SGI349287[] = {{&S39232[80], 5}, {NULL}};
const Segment SGI349288[] = {{&S39232[12], 14}, {&S39232[40], 12}, {NULL}};
const Segment SGI349289[] = {{&S39232[12], 14}, {NULL}};
const Segment SGI349290[] = {{&S39233[6], 4}, {NULL}};
const Segment SGI349291[] = {{&S39233[79], 5}, {NULL}};
const Segment SGI349292[] = {{&S39233[12], 14}, {&S39233[40], 11}, {NULL}};
const Segment SGI349293[] = {{&S39233[12], 14}, {NULL}};
const Segment SGI349294[] = {{&S39234[6], 4}, {NULL}};
const Segment SGI349295[] = {{&S39234[79], 5}, {NULL}};
const Segment SGI349296[] = {{&S39234[12], 14}, {&S39234[40], 11}, {NULL}};
const Segment SGI349297[] = {{&S39234[12], 14}, {NULL}};
const Segment SGI349298[] = {{&S39235[6], 4}, {NULL}};
const Segment SGI349299[] = {{&S39235[80], 5}, {NULL}};
const Segment SGI349300[] = {{&S39235[12], 14}, {&S39235[41], 11}, {NULL}};
const Segment SGI349301[] = {{&S39235[12], 14}, {NULL}};
const Segment SGI349302[] = {{&S39236[6], 4}, {NULL}};
const Segment SGI349303[] = {{&S39236[12], 10}, {NULL}};
const Segment SGI349304[] = {{&S65781[5], 5}, {&S65781[97], 5}, {NULL}};
const Segment SGI349305[] = {{&S65781[113], 3}, {NULL}};
const Segment SGI349306[] = {{&S65802[5], 5}, {&S65802[97], 5}, {NULL}};
const Segment SGI349307[] = {{&S65802[153], 3}, {NULL}};
const Segment SGI349308[] = {{&S65806[5], 3}, {NULL}};
const Segment SGI349309[] = {{&S65807[5], 5}, {NULL}};
const Segment SGI349310[] = {{&S57408[28], 5}, {NULL}};
const Segment SGI349311[] = {{&S57408[104], 5}, {NULL}};
const Segment SGI349312[] = {{&S65820[5], 7}, {&S65820[5], 7}, {NULL}};
const Segment SGI349313[] = {{&S65820[13], 3}, {NULL}};
const Segment SGI349314[] = {{&S65820[134], 3}, {NULL}};
const Segment SGI349315[] = {{&S52804[45], 3}, {NULL}};
const Segment SGI349316[] = {{&S52804[37], 11}, {NULL}};
const Segment SGI349317[] = {{&S52805[46], 3}, {NULL}};
const Segment SGI349318[] = {{&S52805[37], 3}, {NULL}};
const Segment SGI349319[] = {{&S65878[43], 4}, {NULL}};
const Segment SGI349320[] = {{&S65878[6], 3}, {NULL}};
const Segment SGI349321[] = {{&S65878[68], 3}, {NULL}};
const Segment SGI349322[] = {{&S65879[5], 4}, {NULL}};
const Segment SGI349323[] = {{&S65879[15], 3}, {NULL}};
const Segment SGI349324[] = {{&S65911[42], 7}, {NULL}};
const Segment SGI349325[] = {{&S65911[6], 3}, {NULL}};
const Segment SGI349326[] = {{&S65911[70], 3}, {NULL}};
const Segment SGI349327[] = {{&S65912[5], 17}, {NULL}};
const Segment SGI349328[] = {{&S65912[28], 3}, {NULL}};
const Segment SGI349329[] = {{&S65913[5], 16}, {NULL}};
const Segment SGI349330[] = {{&S65913[27], 3}, {NULL}};
const Segment SGI349331[] = {{&S65914[40], 16}, {NULL}};
const Segment SGI349332[] = {{&S65914[6], 3}, {NULL}};
const Segment SGI349333[] = {{&S65914[77], 3}, {NULL}};
const Segment SGI349334[] = {{&S65915[43], 17}, {NULL}};
const Segment SGI349335[] = {{&S65915[6], 3}, {NULL}};
const Segment SGI349336[] = {{&S65915[81], 3}, {NULL}};
const Segment SGI349337[] = {{&S65948[5], 3}, {&S65948[5], 3}, {NULL}};
const Segment SGI349338[] = {{&S65948[67], 3}, {NULL}};
const Segment SGI349339[] = {{&S65948[127], 3}, {NULL}};
const Segment SGI349340[] = {{&S65966[5], 7}, {&S65966[5], 7}, {NULL}};
const Segment SGI349341[] = {{&S65966[13], 3}, {NULL}};
const Segment SGI349342[] = {{&S65966[134], 3}, {NULL}};
const Segment SGI349343[] = {{&S65974[5], 3}, {&S65974[5], 3}, {NULL}};
const Segment SGI349344[] = {{&S65974[149], 3}, {NULL}};
const Segment SGI349345[] = {{&S65987[5], 5}, {NULL}};
const Segment SGI349346[] = {{&S65987[16], 3}, {NULL}};
const Segment SGI349347[] = {{&S65988[35], 5}, {NULL}};
const Segment SGI349348[] = {{&S65988[52], 3}, {NULL}};
const Segment SGI349349[] = {{&S65989[34], 5}, {NULL}};
const Segment SGI349350[] = {{&S65989[50], 3}, {NULL}};
const Segment SGI349351[] = {{&S65999[5], 5}, {&S65999[105], 5}, {NULL}};
const Segment SGI349352[] = {{&S65999[11], 3}, {NULL}};
const Segment SGI349353[] = {{&S65999[131], 3}, {NULL}};
const Segment SGI349354[] = {{&S39489[6], 4}, {NULL}};
const Segment SGI349355[] = {{&S39489[76], 5}, {NULL}};
const Segment SGI349356[] = {{&S39489[12], 9}, {NULL}};
const Segment SGI349357[] = {{&S39489[29], 13}, {NULL}};
const Segment SGI349358[] = {{&S39490[6], 4}, {NULL}};
const Segment SGI349359[] = {{&S39490[76], 5}, {NULL}};
const Segment SGI349360[] = {{&S39490[12], 9}, {NULL}};
const Segment SGI349361[] = {{&S39490[29], 13}, {NULL}};
const Segment SGI349362[] = {{&S66016[21], 5}, {NULL}};
const Segment SGI349363[] = {{&S66016[0], 14}, {NULL}};
const Segment SGI349364[] = {{&S57609[28], 5}, {NULL}};
const Segment SGI349365[] = {{&S57609[104], 5}, {NULL}};
const Segment SGI349366[] = {{&S66035[5], 5}, {&S66035[97], 5}, {NULL}};
const Segment SGI349367[] = {{&S66035[153], 3}, {NULL}};
const Segment SGI349368[] = {{&S66054[5], 7}, {&S66054[5], 7}, {NULL}};
const Segment SGI349369[] = {{&S66054[13], 3}, {NULL}};
const Segment SGI349370[] = {{&S66054[135], 3}, {NULL}};
const Segment SGI349371[] = {{&S57647[28], 7}, {NULL}};
const Segment SGI349372[] = {{&S57647[106], 5}, {NULL}};
const Segment SGI349373[] = {{&S66063[5], 5}, {NULL}};
const Segment SGI349374[] = {{&S66064[5], 5}, {NULL}};
const Segment SGI349375[] = {{&S66064[21], 3}, {NULL}};
const Segment SGI349376[] = {{&S66071[5], 3}, {&S66071[5], 3}, {NULL}};
const Segment SGI349377[] = {{&S66071[109], 3}, {NULL}};
const Segment SGI349378[] = {{&S66073[5], 3}, {&S66073[5], 3}, {NULL}};
const Segment SGI349379[] = {{&S66073[67], 3}, {NULL}};
const Segment SGI349380[] = {{&S66073[126], 3}, {NULL}};
const Segment SGI349381[] = {{&S57660[28], 5}, {NULL}};
const Segment SGI349382[] = {{&S57660[111], 5}, {NULL}};
const Segment SGI349383[] = {{&S39585[6], 4}, {NULL}};
const Segment SGI349384[] = {{&S39585[12], 14}, {NULL}};
const Segment SGI349385[] = {{&S57737[28], 3}, {NULL}};
const Segment SGI349386[] = {{&S57737[102], 5}, {NULL}};
const Segment SGI349387[] = {{&S66159[5], 5}, {&S66159[104], 5}, {NULL}};
const Segment SGI349388[] = {{&S66159[11], 3}, {NULL}};
const Segment SGI349389[] = {{&S66159[130], 3}, {NULL}};
const Segment SGI349390[] = {{&S66169[35], 3}, {NULL}};
const Segment SGI349391[] = {{&S66169[89], 3}, {NULL}};
const Segment SGI349392[] = {{&S66170[5], 12}, {NULL}};
const Segment SGI349393[] = {{&S66170[18], 3}, {NULL}};
const Segment SGI349394[] = {{&S66171[41], 5}, {NULL}};
const Segment SGI349395[] = {{&S66171[6], 3}, {NULL}};
const Segment SGI349396[] = {{&S66171[67], 3}, {NULL}};
const Segment SGI349397[] = {{&S57771[28], 5}, {NULL}};
const Segment SGI349398[] = {{&S57771[104], 5}, {NULL}};
const Segment SGI349399[] = {{&S39676[6], 4}, {NULL}};
const Segment SGI349400[] = {{&S39676[85], 5}, {NULL}};
const Segment SGI349401[] = {{&S39676[12], 3}, {&S39676[48], 9}, {NULL}};
const Segment SGI349402[] = {{&S53195[5], 5}, {NULL}};
const Segment SGI349403[] = {{&S53195[45], 3}, {NULL}};
const Segment SGI349404[] = {{&S53196[49], 7}, {NULL}};
const Segment SGI349405[] = {{&S53196[40], 3}, {NULL}};
const Segment SGI349406[] = {{&S57839[28], 3}, {NULL}};
const Segment SGI349407[] = {{&S57839[102], 5}, {NULL}};
const Segment SGI349408[] = {{&S66307[43], 5}, {NULL}};
const Segment SGI349409[] = {{&S66307[6], 3}, {NULL}};
const Segment SGI349410[] = {{&S53264[5], 4}, {NULL}};
const Segment SGI349411[] = {{&S53264[44], 3}, {NULL}};
const Segment SGI349412[] = {{&S66307[69], 3}, {NULL}};
const Segment SGI349413[] = {{&S53265[49], 7}, {NULL}};
const Segment SGI349414[] = {{&S53265[40], 3}, {NULL}};
const Segment SGI349415[] = {{&S53266[5], 6}, {NULL}};
const Segment SGI349416[] = {{&S53266[42], 3}, {NULL}};
const Segment SGI349417[] = {{&S66311[35], 3}, {NULL}};
const Segment SGI349418[] = {{&S66311[50], 3}, {NULL}};
const Segment SGI349419[] = {{&S53267[19], 3}, {&S53267[79], 7}, {NULL}};
const Segment SGI349420[] = {{&S66312[5], 3}, {NULL}};
const Segment SGI349421[] = {{&S66312[14], 3}, {NULL}};
const Segment SGI349422[] = {{&S53267[40], 3}, {&S53267[40], 3}, {NULL}};
const Segment SGI349423[] = {{&S53268[49], 8}, {NULL}};
const Segment SGI349424[] = {{&S53268[40], 3}, {NULL}};
const Segment SGI349425[] = {{&S53269[49], 19}, {NULL}};
const Segment SGI349426[] = {{&S53269[40], 3}, {NULL}};
const Segment SGI349427[] = {{&S53270[49], 18}, {NULL}};
const Segment SGI349428[] = {{&S53270[40], 3}, {NULL}};
const Segment SGI349429[] = {{&S53271[5], 24}, {NULL}};
const Segment SGI349430[] = {{&S53271[53], 3}, {NULL}};
const Segment SGI349431[] = {{&S53272[5], 9}, {NULL}};
const Segment SGI349432[] = {{&S53272[50], 3}, {NULL}};
const Segment SGI349433[] = {{&S53273[5], 11}, {NULL}};
const Segment SGI349434[] = {{&S53273[40], 3}, {NULL}};
const Segment SGI349435[] = {{&S66336[42], 3}, {NULL}};
const Segment SGI349436[] = {{&S66336[6], 3}, {NULL}};
const Segment SGI349437[] = {{&S66336[66], 3}, {NULL}};
const Segment SGI349438[] = {{&S66348[5], 5}, {NULL}};
const Segment SGI349439[] = {{&S66397[5], 4}, {NULL}};
const Segment SGI349440[] = {{&S58057[28], 5}, {NULL}};
const Segment SGI349441[] = {{&S58057[104], 5}, {NULL}};
const Segment SGI349442[] = {{&S66465[5], 3}, {&S66465[5], 3}, {NULL}};
const Segment SGI349443[] = {{&S66465[9], 3}, {NULL}};
const Segment SGI349444[] = {{&S66465[126], 3}, {NULL}};
const Segment SGI349445[] = {{&S66515[53], 4}, {NULL}};
const Segment SGI349446[] = {{&S66516[53], 5}, {NULL}};
const Segment SGI349447[] = {{&S66517[53], 13}, {NULL}};
const Segment SGI349448[] = {{&S66569[5], 3}, {&S66569[5], 3}, {NULL}};
const Segment SGI349449[] = {{&S66569[9], 3}, {NULL}};
const Segment SGI349450[] = {{&S66569[127], 3}, {NULL}};
const Segment SGI349451[] = {{&S66570[5], 3}, {&S66570[5], 3}, {NULL}};
const Segment SGI349452[] = {{&S66570[9], 3}, {NULL}};
const Segment SGI349453[] = {{&S66570[127], 3}, {NULL}};
const Segment SGI349454[] = {{&S66578[38], 5}, {NULL}};
const Segment SGI349455[] = {{&S66578[94], 3}, {NULL}};
const Segment SGI349456[] = {{&S66579[5], 5}, {NULL}};
const Segment SGI349457[] = {{&S66579[16], 3}, {NULL}};
const Segment SGI349458[] = {{&S66580[34], 5}, {NULL}};
const Segment SGI349459[] = {{&S66580[90], 3}, {NULL}};
const Segment SGI349460[] = {{&S66601[42], 5}, {NULL}};
const Segment SGI349461[] = {{&S66601[6], 3}, {NULL}};
const Segment SGI349462[] = {{&S66601[68], 3}, {NULL}};
const Segment SGI349463[] = {{&S66602[42], 7}, {NULL}};
const Segment SGI349464[] = {{&S66602[6], 3}, {NULL}};
const Segment SGI349465[] = {{&S66602[70], 3}, {NULL}};
const Segment SGI349466[] = {{&S66603[43], 7}, {NULL}};
const Segment SGI349467[] = {{&S66603[6], 3}, {NULL}};
const Segment SGI349468[] = {{&S66603[71], 3}, {NULL}};
const Segment SGI349469[] = {{&S66604[43], 5}, {NULL}};
const Segment SGI349470[] = {{&S66604[6], 3}, {NULL}};
const Segment SGI349471[] = {{&S66604[69], 3}, {NULL}};
const Segment SGI349472[] = {{&S66689[5], 7}, {&S66689[5], 7}, {NULL}};
const Segment SGI349473[] = {{&S66689[13], 3}, {NULL}};
const Segment SGI349474[] = {{&S66689[135], 3}, {NULL}};
const Segment SGI349475[] = {{&S66690[5], 4}, {&S66690[5], 4}, {NULL}};
const Segment SGI349476[] = {{&S66690[10], 3}, {NULL}};
const Segment SGI349477[] = {{&S66690[129], 3}, {NULL}};
const Segment SGI349478[] = {{&S40286[6], 4}, {NULL}};
const Segment SGI349479[] = {{&S40286[59], 5}, {NULL}};
const Segment SGI349480[] = {{&S40286[12], 14}, {NULL}};
const Segment SGI349481[] = {{&S40287[6], 4}, {NULL}};
const Segment SGI349482[] = {{&S40287[82], 5}, {NULL}};
const Segment SGI349483[] = {{&S40287[12], 14}, {&S40287[40], 12}, {NULL}};
const Segment SGI349484[] = {{&S40287[12], 14}, {NULL}};
const Segment SGI349485[] = {{&S40288[6], 4}, {NULL}};
const Segment SGI349486[] = {{&S40288[82], 5}, {NULL}};
const Segment SGI349487[] = {{&S40288[12], 14}, {&S40288[41], 11}, {NULL}};
const Segment SGI349488[] = {{&S40288[12], 14}, {NULL}};
const Segment SGI349489[] = {{&S66694[5], 3}, {&S66694[5], 3}, {NULL}};
const Segment SGI349490[] = {{&S66694[9], 3}, {NULL}};
const Segment SGI349491[] = {{&S66694[127], 3}, {NULL}};
const Segment SGI349492[] = {{&S40289[6], 4}, {NULL}};
const Segment SGI349493[] = {{&S40289[82], 5}, {NULL}};
const Segment SGI349494[] = {{&S40289[12], 14}, {&S40289[40], 12}, {NULL}};
const Segment SGI349495[] = {{&S40289[12], 14}, {NULL}};
const Segment SGI349496[] = {{&S40291[6], 4}, {NULL}};
const Segment SGI349497[] = {{&S40291[81], 5}, {NULL}};
const Segment SGI349498[] = {{&S40291[12], 14}, {&S40291[40], 11}, {NULL}};
const Segment SGI349499[] = {{&S40291[12], 14}, {NULL}};
const Segment SGI349500[] = {{&S40292[6], 4}, {NULL}};
const Segment SGI349501[] = {{&S40292[111], 5}, {NULL}};
const Segment SGI349502[] = {{&S40292[12], 14}, {&S40292[44], 10}, {NULL}};
const Segment SGI349503[] = {{&S40292[12], 14}, {&S40292[66], 14}, {NULL}};
const Segment SGI349504[] = {{&S40293[6], 4}, {NULL}};
const Segment SGI349505[] = {{&S40293[82], 5}, {NULL}};
const Segment SGI349506[] = {{&S40293[12], 14}, {&S40293[40], 12}, {NULL}};
const Segment SGI349507[] = {{&S40293[12], 14}, {NULL}};
const Segment SGI349508[] = {{&S40294[6], 4}, {NULL}};
const Segment SGI349509[] = {{&S40294[82], 5}, {NULL}};
const Segment SGI349510[] = {{&S40294[12], 14}, {&S40294[40], 12}, {NULL}};
const Segment SGI349511[] = {{&S40294[12], 14}, {NULL}};
const Segment SGI349512[] = {{&S40295[6], 4}, {NULL}};
const Segment SGI349513[] = {{&S40295[82], 5}, {NULL}};
const Segment SGI349514[] = {{&S40295[12], 14}, {&S40295[41], 11}, {NULL}};
const Segment SGI349515[] = {{&S40295[12], 14}, {NULL}};
const Segment SGI349516[] = {{&S40297[6], 4}, {NULL}};
const Segment SGI349517[] = {{&S40297[82], 5}, {NULL}};
const Segment SGI349518[] = {{&S40297[12], 14}, {&S40297[40], 12}, {NULL}};
const Segment SGI349519[] = {{&S40297[12], 14}, {NULL}};
const Segment SGI349520[] = {{&S66723[43], 5}, {NULL}};
const Segment SGI349521[] = {{&S66723[6], 3}, {NULL}};
const Segment SGI349522[] = {{&S66723[69], 3}, {NULL}};
const Segment SGI349523[] = {{&S40298[6], 4}, {NULL}};
const Segment SGI349524[] = {{&S40298[81], 5}, {NULL}};
const Segment SGI349525[] = {{&S40298[12], 14}, {&S40298[40], 11}, {NULL}};
const Segment SGI349526[] = {{&S40298[12], 14}, {NULL}};
const Segment SGI349527[] = {{&S40299[6], 4}, {NULL}};
const Segment SGI349528[] = {{&S40299[82], 5}, {NULL}};
const Segment SGI349529[] = {{&S40299[12], 14}, {&S40299[41], 11}, {NULL}};
const Segment SGI349530[] = {{&S40299[12], 14}, {NULL}};
const Segment SGI349531[] = {{&S42640[58], 5}, {NULL}};
const Segment SGI349532[] = {{&S42640[13], 10}, {NULL}};
const Segment SGI349533[] = {{&S58393[28], 5}, {NULL}};
const Segment SGI349534[] = {{&S58393[104], 5}, {NULL}};
const Segment SGI349535[] = {{&S66806[5], 3}, {&S66806[5], 3}, {NULL}};
const Segment SGI349536[] = {{&S66806[149], 3}, {NULL}};
const Segment SGI349537[] = {{&S66821[5], 3}, {NULL}};
const Segment SGI349538[] = {{&S66821[14], 3}, {NULL}};
const Segment SGI349539[] = {{&S66822[5], 3}, {NULL}};
const Segment SGI349540[] = {{&S66822[19], 3}, {NULL}};
const Segment SGI349541[] = {{&S66824[35], 8}, {NULL}};
const Segment SGI349542[] = {{&S66824[94], 3}, {NULL}};
const Segment SGI349543[] = {{&S66825[5], 17}, {NULL}};
const Segment SGI349544[] = {{&S66825[23], 3}, {NULL}};
const Segment SGI349545[] = {{&S66885[5], 3}, {&S66885[5], 3}, {NULL}};
const Segment SGI349546[] = {{&S66885[67], 3}, {NULL}};
const Segment SGI349547[] = {{&S66885[127], 3}, {NULL}};
const Segment SGI349548[] = {{&S58554[28], 3}, {NULL}};
const Segment SGI349549[] = {{&S58554[102], 5}, {NULL}};
const Segment SGI349550[] = {{&S66909[5], 7}, {&S66909[5], 7}, {NULL}};
const Segment SGI349551[] = {{&S66909[88], 3}, {NULL}};
const Segment SGI349552[] = {{&S66910[5], 4}, {&S66910[5], 4}, {NULL}};
const Segment SGI349553[] = {{&S66910[82], 3}, {NULL}};
const Segment SGI349554[] = {{&S66911[5], 7}, {NULL}};
const Segment SGI349555[] = {{&S66911[95], 3}, {NULL}};
const Segment SGI349556[] = {{&S66947[43], 3}, {NULL}};
const Segment SGI349557[] = {{&S66947[6], 3}, {NULL}};
const Segment SGI349558[] = {{&S66947[67], 3}, {NULL}};
const Segment SGI349559[] = {{&S66949[5], 4}, {NULL}};
const Segment SGI349560[] = {{&S66950[5], 5}, {NULL}};
const Segment SGI349561[] = {{&S66959[38], 3}, {NULL}};
const Segment SGI349562[] = {{&S66959[92], 3}, {NULL}};
const Segment SGI349563[] = {{&S40598[6], 4}, {NULL}};
const Segment SGI349564[] = {{&S40598[70], 5}, {NULL}};
const Segment SGI349565[] = {{&S40598[12], 9}, {NULL}};
const Segment SGI349566[] = {{&S40598[29], 13}, {NULL}};
const Segment SGI349567[] = {{&S67020[5], 4}, {&S67020[5], 4}, {NULL}};
const Segment SGI349568[] = {{&S67021[5], 4}, {NULL}};
const Segment SGI349569[] = {{&S67022[5], 9}, {NULL}};
const Segment SGI349570[] = {{&S67035[5], 7}, {&S67035[5], 7}, {NULL}};
const Segment SGI349571[] = {{&S67035[117], 3}, {NULL}};
const Segment SGI349572[] = {{&S58700[28], 5}, {NULL}};
const Segment SGI349573[] = {{&S58700[104], 5}, {NULL}};
const Segment SGI349574[] = {{&S58751[28], 5}, {NULL}};
const Segment SGI349575[] = {{&S58751[104], 5}, {NULL}};
const Segment SGI349576[] = {{&S58752[28], 3}, {NULL}};
const Segment SGI349577[] = {{&S58752[102], 5}, {NULL}};
const Segment SGI349578[] = {{&S58793[28], 3}, {NULL}};
const Segment SGI349579[] = {{&S58793[102], 5}, {NULL}};
const Segment SGI349580[] = {{&S67158[5], 5}, {&S67158[105], 5}, {NULL}};
const Segment SGI349581[] = {{&S67158[11], 3}, {NULL}};
const Segment SGI349582[] = {{&S67158[131], 3}, {NULL}};
const Segment SGI349583[] = {{&S40805[6], 4}, {NULL}};
const Segment SGI349584[] = {{&S40805[12], 14}, {NULL}};
const Segment SGI349585[] = {{&S40806[6], 4}, {NULL}};
const Segment SGI349586[] = {{&S40806[12], 14}, {NULL}};
const Segment SGI349587[] = {{&S40807[6], 4}, {NULL}};
const Segment SGI349588[] = {{&S40807[12], 14}, {NULL}};
const Segment SGI349589[] = {{&S40808[6], 4}, {NULL}};
const Segment SGI349590[] = {{&S40808[12], 14}, {NULL}};
const Segment SGI349591[] = {{&S40809[6], 4}, {NULL}};
const Segment SGI349592[] = {{&S40809[12], 14}, {NULL}};
const Segment SGI349593[] = {{&S40811[6], 4}, {NULL}};
const Segment SGI349594[] = {{&S40811[12], 14}, {&S40811[40], 12}, {NULL}};
const Segment SGI349595[] = {{&S40811[12], 14}, {NULL}};
const Segment SGI349596[] = {{&S67216[5], 5}, {&S67216[104], 5}, {NULL}};
const Segment SGI349597[] = {{&S67216[69], 3}, {NULL}};
const Segment SGI349598[] = {{&S67216[130], 3}, {NULL}};
const Segment SGI349599[] = {{&S40857[6], 4}, {NULL}};
const Segment SGI349600[] = {{&S40857[57], 5}, {NULL}};
const Segment SGI349601[] = {{&S40857[12], 14}, {NULL}};
const Segment SGI349602[] = {{&S40858[6], 4}, {NULL}};
const Segment SGI349603[] = {{&S40858[57], 5}, {NULL}};
const Segment SGI349604[] = {{&S40858[12], 14}, {NULL}};
const Segment SGI349605[] = {{&S40859[6], 5}, {NULL}};
const Segment SGI349606[] = {{&S40859[13], 14}, {NULL}};
const Segment SGI349607[] = {{&S40860[6], 4}, {NULL}};
const Segment SGI349608[] = {{&S40860[80], 5}, {NULL}};
const Segment SGI349609[] = {{&S40860[12], 14}, {&S40860[40], 12}, {NULL}};
const Segment SGI349610[] = {{&S40860[12], 14}, {NULL}};
const Segment SGI349611[] = {{&S40861[6], 4}, {NULL}};
const Segment SGI349612[] = {{&S40861[79], 5}, {NULL}};
const Segment SGI349613[] = {{&S40861[12], 14}, {&S40861[40], 11}, {NULL}};
const Segment SGI349614[] = {{&S40861[12], 14}, {NULL}};
const Segment SGI349615[] = {{&S40862[6], 4}, {NULL}};
const Segment SGI349616[] = {{&S40862[80], 5}, {NULL}};
const Segment SGI349617[] = {{&S40862[12], 14}, {&S40862[40], 12}, {NULL}};
const Segment SGI349618[] = {{&S40862[12], 14}, {NULL}};
const Segment SGI349619[] = {{&S40863[6], 4}, {NULL}};
const Segment SGI349620[] = {{&S40863[57], 5}, {NULL}};
const Segment SGI349621[] = {{&S40863[12], 14}, {&S40863[68], 31}, {NULL}};
const Segment SGI349622[] = {{&S67239[35], 3}, {NULL}};
const Segment SGI349623[] = {{&S40863[12], 14}, {NULL}};
const Segment SGI349624[] = {{&S67239[50], 3}, {NULL}};
const Segment SGI349625[] = {{&S40864[6], 4}, {NULL}};
const Segment SGI349626[] = {{&S67240[5], 3}, {NULL}};
const Segment SGI349627[] = {{&S40864[79], 5}, {NULL}};
const Segment SGI349628[] = {{&S67240[14], 3}, {NULL}};
const Segment SGI349629[] = {{&S40864[12], 14}, {&S40864[40], 11}, {NULL}};
const Segment SGI349630[] = {{&S40864[12], 14}, {NULL}};
const Segment SGI349631[] = {{&S67241[35], 4}, {NULL}};
const Segment SGI349632[] = {{&S67241[51], 3}, {NULL}};
const Segment SGI349633[] = {{&S40866[6], 4}, {NULL}};
const Segment SGI349634[] = {{&S40866[80], 5}, {NULL}};
const Segment SGI349635[] = {{&S40866[12], 14}, {&S40866[40], 12}, {NULL}};
const Segment SGI349636[] = {{&S40866[12], 14}, {NULL}};
const Segment SGI349637[] = {{&S67242[34], 5}, {NULL}};
const Segment SGI349638[] = {{&S67242[90], 3}, {NULL}};
const Segment SGI349639[] = {{&S40867[6], 4}, {NULL}};
const Segment SGI349640[] = {{&S40867[79], 5}, {NULL}};
const Segment SGI349641[] = {{&S40867[12], 14}, {&S40867[40], 11}, {NULL}};
const Segment SGI349642[] = {{&S40867[12], 14}, {NULL}};
const Segment SGI349643[] = {{&S40869[6], 4}, {NULL}};
const Segment SGI349644[] = {{&S40869[80], 5}, {NULL}};
const Segment SGI349645[] = {{&S40869[12], 14}, {&S40869[40], 12}, {NULL}};
const Segment SGI349646[] = {{&S40869[12], 14}, {NULL}};
const Segment SGI349647[] = {{&S40870[6], 4}, {NULL}};
const Segment SGI349648[] = {{&S40870[80], 5}, {NULL}};
const Segment SGI349649[] = {{&S40870[12], 14}, {&S40870[41], 11}, {NULL}};
const Segment SGI349650[] = {{&S40870[12], 14}, {NULL}};
const Segment SGI349651[] = {{&S40871[6], 4}, {NULL}};
const Segment SGI349652[] = {{&S40871[79], 5}, {NULL}};
const Segment SGI349653[] = {{&S40871[12], 14}, {&S40871[40], 11}, {NULL}};
const Segment SGI349654[] = {{&S40871[12], 14}, {NULL}};
const Segment SGI349655[] = {{&S40872[6], 5}, {NULL}};
const Segment SGI349656[] = {{&S40872[13], 14}, {NULL}};
const Segment SGI349657[] = {{&S40873[6], 4}, {NULL}};
const Segment SGI349658[] = {{&S40873[79], 5}, {NULL}};
const Segment SGI349659[] = {{&S40873[12], 14}, {&S40873[40], 11}, {NULL}};
const Segment SGI349660[] = {{&S40873[12], 14}, {NULL}};
const Segment SGI349661[] = {{&S40874[6], 4}, {NULL}};
const Segment SGI349662[] = {{&S40874[80], 5}, {NULL}};
const Segment SGI349663[] = {{&S40874[12], 14}, {&S40874[41], 11}, {NULL}};
const Segment SGI349664[] = {{&S40874[12], 14}, {NULL}};
const Segment SGI349665[] = {{&S40875[6], 4}, {NULL}};
const Segment SGI349666[] = {{&S40875[79], 5}, {NULL}};
const Segment SGI349667[] = {{&S40875[12], 14}, {&S40875[40], 11}, {NULL}};
const Segment SGI349668[] = {{&S40875[12], 14}, {NULL}};
const Segment SGI349669[] = {{&S40876[6], 4}, {NULL}};
const Segment SGI349670[] = {{&S40876[79], 5}, {NULL}};
const Segment SGI349671[] = {{&S40876[12], 14}, {&S40876[40], 11}, {NULL}};
const Segment SGI349672[] = {{&S40876[12], 14}, {NULL}};
const Segment SGI349673[] = {{&S67289[42], 8}, {NULL}};
const Segment SGI349674[] = {{&S67289[6], 3}, {NULL}};
const Segment SGI349675[] = {{&S67289[71], 3}, {NULL}};
const Segment SGI349676[] = {{&S67290[5], 8}, {NULL}};
const Segment SGI349677[] = {{&S67290[19], 3}, {NULL}};
const Segment SGI349678[] = {{&S67310[35], 5}, {NULL}};
const Segment SGI349679[] = {{&S67310[52], 3}, {NULL}};
const Segment SGI349680[] = {{&S67311[5], 5}, {NULL}};
const Segment SGI349681[] = {{&S67311[16], 3}, {NULL}};
const Segment SGI349682[] = {{&S67312[35], 5}, {NULL}};
const Segment SGI349683[] = {{&S67312[52], 3}, {NULL}};
const Segment SGI349684[] = {{&S67313[5], 5}, {NULL}};
const Segment SGI349685[] = {{&S67313[16], 3}, {NULL}};
const Segment SGI349686[] = {{&S67314[34], 5}, {NULL}};
const Segment SGI349687[] = {{&S67314[90], 3}, {NULL}};
const Segment SGI349688[] = {{&S67376[5], 7}, {&S67376[5], 7}, {NULL}};
const Segment SGI349689[] = {{&S67376[88], 3}, {NULL}};
const Segment SGI349690[] = {{&S67386[5], 3}, {&S67386[5], 3}, {NULL}};
const Segment SGI349691[] = {{&S67386[9], 3}, {NULL}};
const Segment SGI349692[] = {{&S67386[127], 3}, {NULL}};
const Segment SGI349693[] = {{&S67396[5], 3}, {&S67396[5], 3}, {NULL}};
const Segment SGI349694[] = {{&S67396[67], 3}, {NULL}};
const Segment SGI349695[] = {{&S67396[126], 3}, {NULL}};
const Segment SGI349696[] = {{&S59072[28], 7}, {NULL}};
const Segment SGI349697[] = {{&S59072[106], 5}, {NULL}};
const Segment SGI349698[] = {{&S59073[28], 5}, {NULL}};
const Segment SGI349699[] = {{&S59073[104], 5}, {NULL}};
const Segment SGI349700[] = {{&S59074[28], 7}, {NULL}};
const Segment SGI349701[] = {{&S59074[106], 5}, {NULL}};
const Segment SGI349702[] = {{&S59075[28], 5}, {NULL}};
const Segment SGI349703[] = {{&S67427[5], 3}, {NULL}};
const Segment SGI349704[] = {{&S67427[19], 3}, {NULL}};
const Segment SGI349705[] = {{&S59075[104], 5}, {NULL}};
const Segment SGI349706[] = {{&S67428[5], 3}, {NULL}};
const Segment SGI349707[] = {{&S67428[14], 3}, {NULL}};
const Segment SGI349708[] = {{&S67429[5], 5}, {&S67429[68], 5}, {NULL}};
const Segment SGI349709[] = {{&S67429[84], 3}, {NULL}};
const Segment SGI349710[] = {{&S67455[43], 7}, {NULL}};
const Segment SGI349711[] = {{&S67455[6], 3}, {NULL}};
const Segment SGI349712[] = {{&S67455[71], 3}, {NULL}};
const Segment SGI349713[] = {{&S67456[43], 3}, {NULL}};
const Segment SGI349714[] = {{&S67456[6], 3}, {NULL}};
const Segment SGI349715[] = {{&S67456[67], 3}, {NULL}};
const Segment SGI349716[] = {{&S67457[5], 3}, {&S67457[5], 3}, {NULL}};
const Segment SGI349717[] = {{&S67457[9], 3}, {NULL}};
const Segment SGI349718[] = {{&S67457[127], 3}, {NULL}};
const Segment SGI349719[] = {{&S59143[28], 5}, {NULL}};
const Segment SGI349720[] = {{&S59143[111], 5}, {NULL}};
const Segment SGI349721[] = {{&S59144[28], 7}, {NULL}};
const Segment SGI349722[] = {{&S59144[106], 5}, {NULL}};
const Segment SGI349723[] = {{&S43508[64], 4}, {NULL}};
const Segment SGI349724[] = {{&S43508[13], 10}, {NULL}};
const Segment SGI349725[] = {{&S67470[35], 5}, {NULL}};
const Segment SGI349726[] = {{&S67470[52], 3}, {NULL}};
const Segment SGI349727[] = {{&S67471[34], 5}, {NULL}};
const Segment SGI349728[] = {{&S67471[50], 3}, {NULL}};
const Segment SGI349729[] = {{&S54449[5], 3}, {NULL}};
const Segment SGI349730[] = {{&S54449[31], 3}, {NULL}};
const Segment SGI349731[] = {{&S54449[85], 3}, {NULL}};
const Segment SGI349732[] = {{&S54450[5], 3}, {NULL}};
const Segment SGI349733[] = {{&S54450[32], 3}, {NULL}};
const Segment SGI349734[] = {{&S54450[81], 3}, {NULL}};
const Segment SGI349735[] = {{&S67509[5], 7}, {&S67509[5], 7}, {NULL}};
const Segment SGI349736[] = {{&S67509[157], 3}, {NULL}};
const Segment SGI349737[] = {{&S67512[86], 7}, {NULL}};
const Segment SGI349738[] = {{&S67529[5], 7}, {NULL}};
const Segment SGI349739[] = {{&S67530[5], 4}, {NULL}};
const Segment SGI349740[] = {{&S67536[38], 3}, {NULL}};
const Segment SGI349741[] = {{&S67536[92], 3}, {NULL}};
const Segment SGI349742[] = {{&S67537[35], 5}, {NULL}};
const Segment SGI349743[] = {{&S67537[91], 3}, {NULL}};
const Segment SGI349744[] = {{&S67538[5], 14}, {NULL}};
const Segment SGI349745[] = {{&S67538[20], 3}, {NULL}};
const Segment SGI349746[] = {{&S67539[41], 5}, {NULL}};
const Segment SGI349747[] = {{&S67539[6], 3}, {NULL}};
const Segment SGI349748[] = {{&S67539[67], 3}, {NULL}};
const Segment SGI349749[] = {{&S59352[28], 5}, {NULL}};
const Segment SGI349750[] = {{&S59352[104], 5}, {NULL}};
const Segment SGI349751[] = {{&S67569[35], 3}, {NULL}};
const Segment SGI349752[] = {{&S67569[50], 3}, {NULL}};
const Segment SGI349753[] = {{&S67584[43], 5}, {NULL}};
const Segment SGI349754[] = {{&S67584[6], 3}, {NULL}};
const Segment SGI349755[] = {{&S67584[69], 3}, {NULL}};
const Segment SGI349756[] = {{&S67621[5], 4}, {&S67621[5], 4}, {NULL}};
const Segment SGI349757[] = {{&S67621[68], 3}, {NULL}};
const Segment SGI349758[] = {{&S67621[129], 3}, {NULL}};
const Segment SGI349759[] = {{&S67624[38], 5}, {NULL}};
const Segment SGI349760[] = {{&S67624[55], 3}, {NULL}};
const Segment SGI349761[] = {{&S67625[5], 5}, {NULL}};
const Segment SGI349762[] = {{&S67625[16], 3}, {NULL}};
const Segment SGI349763[] = {{&S67626[31], 5}, {NULL}};
const Segment SGI349764[] = {{&S67626[87], 3}, {NULL}};
const Segment SGI349765[] = {{&S67648[5], 5}, {NULL}};
const Segment SGI349766[] = {{&S67648[21], 3}, {NULL}};
const Segment SGI349767[] = {{&S67649[5], 5}, {NULL}};
const Segment SGI349768[] = {{&S67649[16], 3}, {NULL}};
const Segment SGI349769[] = {{&S67694[43], 5}, {NULL}};
const Segment SGI349770[] = {{&S67694[6], 3}, {NULL}};
const Segment SGI349771[] = {{&S67694[69], 3}, {NULL}};
const Segment SGI349772[] = {{&S67708[21], 5}, {NULL}};
const Segment SGI349773[] = {{&S67708[0], 14}, {NULL}};
const Segment SGI349774[] = {{&S67745[5], 4}, {NULL}};
const Segment SGI349775[] = {{&S41440[6], 4}, {NULL}};
const Segment SGI349776[] = {{&S41440[12], 12}, {NULL}};
const Segment SGI349777[] = {{&S41442[6], 4}, {NULL}};
const Segment SGI349778[] = {{&S41442[12], 11}, {NULL}};
const Segment SGI349779[] = {{&S41443[6], 4}, {NULL}};
const Segment SGI349780[] = {{&S41443[12], 13}, {&S41443[43], 65}, {NULL}};
const Segment SGI349781[] = {{&S41444[6], 4}, {NULL}};
const Segment SGI349782[] = {{&S41444[12], 13}, {NULL}};
const Segment SGI349783[] = {{&S41446[6], 4}, {NULL}};
const Segment SGI349784[] = {{&S41446[12], 13}, {NULL}};
const Segment SGI349785[] = {{&S41447[6], 4}, {NULL}};
const Segment SGI349786[] = {{&S41447[12], 17}, {NULL}};
const Segment SGI349787[] = {{&S41448[6], 4}, {NULL}};
const Segment SGI349788[] = {{&S41448[12], 13}, {&S41448[43], 65}, {NULL}};
const Segment SGI349789[] = {{&S43915[60], 5}, {NULL}};
const Segment SGI349790[] = {{&S43915[13], 10}, {NULL}};
const Segment SGI349791[] = {{&S67842[42], 3}, {NULL}};
const Segment SGI349792[] = {{&S67842[6], 3}, {NULL}};
const Segment SGI349793[] = {{&S67842[66], 3}, {NULL}};
const Segment SGI349794[] = {{&S67954[5], 5}, {NULL}};
const Segment SGI349795[] = {{&S67955[5], 7}, {NULL}};
const Segment SGI349796[] = {{&S67956[5], 10}, {NULL}};
const Segment SGI349797[] = {{&S67957[5], 8}, {NULL}};
const Segment SGI349798[] = {{&S41645[6], 4}, {NULL}};
const Segment SGI349799[] = {{&S41645[57], 5}, {NULL}};
const Segment SGI349800[] = {{&S41645[12], 14}, {NULL}};
const Segment SGI349801[] = {{&S41646[6], 4}, {NULL}};
const Segment SGI349802[] = {{&S41646[80], 5}, {NULL}};
const Segment SGI349803[] = {{&S41646[12], 14}, {&S41646[40], 12}, {NULL}};
const Segment SGI349804[] = {{&S41646[12], 14}, {NULL}};
const Segment SGI349805[] = {{&S41647[6], 4}, {NULL}};
const Segment SGI349806[] = {{&S41647[80], 5}, {NULL}};
const Segment SGI349807[] = {{&S41647[12], 14}, {&S41647[40], 12}, {NULL}};
const Segment SGI349808[] = {{&S41647[12], 14}, {NULL}};
const Segment SGI349809[] = {{&S41648[6], 4}, {NULL}};
const Segment SGI349810[] = {{&S41648[80], 5}, {NULL}};
const Segment SGI349811[] = {{&S41648[12], 14}, {&S41648[40], 12}, {NULL}};
const Segment SGI349812[] = {{&S41648[12], 14}, {NULL}};
const Segment SGI349813[] = {{&S41649[6], 4}, {NULL}};
const Segment SGI349814[] = {{&S41649[79], 5}, {NULL}};
const Segment SGI349815[] = {{&S41649[12], 14}, {&S41649[40], 11}, {NULL}};
const Segment SGI349816[] = {{&S41649[12], 14}, {NULL}};
const Segment SGI349817[] = {{&S41650[6], 4}, {NULL}};
const Segment SGI349818[] = {{&S41650[79], 5}, {NULL}};
const Segment SGI349819[] = {{&S41650[12], 14}, {&S41650[40], 11}, {NULL}};
const Segment SGI349820[] = {{&S41650[12], 14}, {NULL}};
const Segment SGI349821[] = {{&S41652[6], 4}, {NULL}};
const Segment SGI349822[] = {{&S41652[79], 5}, {NULL}};
const Segment SGI349823[] = {{&S41652[12], 14}, {&S41652[40], 11}, {NULL}};
const Segment SGI349824[] = {{&S41652[12], 14}, {NULL}};
const Segment SGI349825[] = {{&S41653[6], 4}, {NULL}};
const Segment SGI349826[] = {{&S41653[79], 5}, {NULL}};
const Segment SGI349827[] = {{&S41653[12], 14}, {&S41653[40], 11}, {NULL}};
const Segment SGI349828[] = {{&S41653[12], 14}, {NULL}};
const Segment SGI349829[] = {{&S41654[6], 4}, {NULL}};
const Segment SGI349830[] = {{&S41654[80], 5}, {NULL}};
const Segment SGI349831[] = {{&S41654[12], 14}, {&S41654[41], 11}, {NULL}};
const Segment SGI349832[] = {{&S41654[12], 14}, {NULL}};
const Segment SGI349833[] = {{&S41655[6], 4}, {NULL}};
const Segment SGI349834[] = {{&S41655[80], 5}, {NULL}};
const Segment SGI349835[] = {{&S41655[12], 14}, {&S41655[41], 11}, {NULL}};
const Segment SGI349836[] = {{&S41655[12], 14}, {NULL}};
const Segment SGI349837[] = {{&S41656[6], 4}, {NULL}};
const Segment SGI349838[] = {{&S41656[79], 5}, {NULL}};
const Segment SGI349839[] = {{&S41656[12], 14}, {&S41656[40], 11}, {NULL}};
const Segment SGI349840[] = {{&S41656[12], 14}, {NULL}};
const Segment SGI349841[] = {{&S41657[6], 4}, {NULL}};
const Segment SGI349842[] = {{&S41657[79], 5}, {NULL}};
const Segment SGI349843[] = {{&S41657[12], 14}, {&S41657[40], 11}, {NULL}};
const Segment SGI349844[] = {{&S41657[12], 14}, {NULL}};
const Segment SGI349845[] = {{&S41658[6], 4}, {NULL}};
const Segment SGI349846[] = {{&S41658[79], 5}, {NULL}};
const Segment SGI349847[] = {{&S41658[12], 14}, {&S41658[40], 11}, {NULL}};
const Segment SGI349848[] = {{&S41658[12], 14}, {NULL}};
const Segment SGI349849[] = {{&S67983[5], 3}, {&S67983[5], 3}, {NULL}};
const Segment SGI349850[] = {{&S67983[9], 3}, {NULL}};
const Segment SGI349851[] = {{&S41659[6], 4}, {NULL}};
const Segment SGI349852[] = {{&S67983[127], 3}, {NULL}};
const Segment SGI349853[] = {{&S41659[80], 5}, {NULL}};
const Segment SGI349854[] = {{&S41659[12], 14}, {&S41659[41], 11}, {NULL}};
const Segment SGI349855[] = {{&S41659[12], 14}, {NULL}};
const Segment SGI349856[] = {{&S41660[6], 4}, {NULL}};
const Segment SGI349857[] = {{&S41660[79], 5}, {NULL}};
const Segment SGI349858[] = {{&S41660[12], 14}, {&S41660[40], 11}, {NULL}};
const Segment SGI349859[] = {{&S41660[12], 14}, {NULL}};
const Segment SGI349860[] = {{&S41661[6], 4}, {NULL}};
const Segment SGI349861[] = {{&S41661[79], 5}, {NULL}};
const Segment SGI349862[] = {{&S41661[12], 14}, {&S41661[40], 11}, {NULL}};
const Segment SGI349863[] = {{&S41661[12], 14}, {NULL}};
const Segment SGI349864[] = {{&S41662[6], 4}, {NULL}};
const Segment SGI349865[] = {{&S41662[79], 5}, {NULL}};
const Segment SGI349866[] = {{&S41662[12], 14}, {&S41662[40], 11}, {NULL}};
const Segment SGI349867[] = {{&S41662[12], 14}, {NULL}};
const Segment SGI349868[] = {{&S41663[6], 4}, {NULL}};
const Segment SGI349869[] = {{&S41663[79], 5}, {NULL}};
const Segment SGI349870[] = {{&S41663[12], 14}, {&S41663[40], 11}, {NULL}};
const Segment SGI349871[] = {{&S41663[12], 14}, {NULL}};
const Segment SGI349872[] = {{&S41664[6], 4}, {NULL}};
const Segment SGI349873[] = {{&S41664[79], 5}, {NULL}};
const Segment SGI349874[] = {{&S41664[12], 14}, {&S41664[40], 11}, {NULL}};
const Segment SGI349875[] = {{&S41664[12], 14}, {NULL}};
const Segment SGI349876[] = {{&S41665[6], 4}, {NULL}};
const Segment SGI349877[] = {{&S41665[79], 5}, {NULL}};
const Segment SGI349878[] = {{&S41665[12], 14}, {&S41665[40], 11}, {NULL}};
const Segment SGI349879[] = {{&S41665[12], 14}, {NULL}};
const Segment SGI349880[] = {{&S41666[6], 4}, {NULL}};
const Segment SGI349881[] = {{&S41666[79], 5}, {NULL}};
const Segment SGI349882[] = {{&S41666[12], 14}, {&S41666[40], 11}, {NULL}};
const Segment SGI349883[] = {{&S41666[12], 14}, {NULL}};
const Segment SGI349884[] = {{&S67995[23], 2}, {NULL}};
const Segment SGI349885[] = {{&S41667[6], 4}, {NULL}};
const Segment SGI349886[] = {{&S41667[79], 5}, {NULL}};
const Segment SGI349887[] = {{&S41667[12], 14}, {&S41667[40], 11}, {NULL}};
const Segment SGI349888[] = {{&S41667[12], 14}, {NULL}};
const Segment SGI349889[] = {{&S67997[100], 7}, {NULL}};
const Segment SGI349890[] = {{&S59716[28], 5}, {NULL}};
const Segment SGI349891[] = {{&S59716[104], 5}, {NULL}};
const Segment SGI349892[] = {{&S68046[5], 7}, {NULL}};
const Segment SGI349893[] = {{&S68047[5], 5}, {NULL}};
const Segment SGI349894[] = {{&S68143[5], 7}, {NULL}};
const Segment SGI349895[] = {{&S68144[5], 5}, {NULL}};
const Segment SGI349896[] = {{&S68145[5], 7}, {&S68145[5], 7}, {NULL}};
const Segment SGI349897[] = {{&S41814[6], 4}, {NULL}};
const Segment SGI349898[] = {{&S41814[45], 5}, {NULL}};
const Segment SGI349899[] = {{&S41814[12], 5}, {NULL}};
const Segment SGI349900[] = {{&S41815[6], 4}, {NULL}};
const Segment SGI349901[] = {{&S41815[60], 5}, {NULL}};
const Segment SGI349902[] = {{&S41815[12], 3}, {NULL}};
const Segment SGI349903[] = {{&S41816[6], 4}, {NULL}};
const Segment SGI349904[] = {{&S41816[58], 5}, {NULL}};
const Segment SGI349905[] = {{&S41816[12], 3}, {NULL}};
const Segment SGI349906[] = {{&S68169[5], 5}, {&S68169[104], 5}, {NULL}};
const Segment SGI349907[] = {{&S68169[69], 3}, {NULL}};
const Segment SGI349908[] = {{&S68169[130], 3}, {NULL}};
const Segment SGI349909[] = {{&S68170[5], 7}, {&S68170[106], 7}, {NULL}};
const Segment SGI349910[] = {{&S68170[71], 3}, {NULL}};
const Segment SGI349911[] = {{&S68170[134], 3}, {NULL}};
const Segment SGI349912[] = {{&S41858[6], 4}, {NULL}};
const Segment SGI349913[] = {{&S41858[12], 12}, {NULL}};
const Segment SGI349914[] = {{&S68201[5], 5}, {&S68201[105], 5}, {NULL}};
const Segment SGI349915[] = {{&S68201[69], 3}, {NULL}};
const Segment SGI349916[] = {{&S68201[131], 3}, {NULL}};
const Segment SGI349917[] = {{&S68218[42], 5}, {NULL}};
const Segment SGI349918[] = {{&S68218[6], 3}, {NULL}};
const Segment SGI349919[] = {{&S68218[68], 3}, {NULL}};
const Segment SGI349920[] = {{&S68219[43], 5}, {NULL}};
const Segment SGI349921[] = {{&S68219[6], 3}, {NULL}};
const Segment SGI349922[] = {{&S68219[69], 3}, {NULL}};
const Segment SGI349923[] = {{&S68220[43], 7}, {NULL}};
const Segment SGI349924[] = {{&S68220[6], 3}, {NULL}};
const Segment SGI349925[] = {{&S68220[71], 3}, {NULL}};
const Segment SGI349926[] = {{&S44348[61], 5}, {NULL}};
const Segment SGI349927[] = {{&S44348[13], 10}, {NULL}};
const Segment SGI349928[] = {{&S68348[5], 5}, {&S68348[105], 5}, {NULL}};
const Segment SGI349929[] = {{&S68348[11], 3}, {NULL}};
const Segment SGI349930[] = {{&S68348[131], 3}, {NULL}};
const Segment SGI349931[] = {{&S60051[28], 3}, {NULL}};
const Segment SGI349932[] = {{&S60051[102], 5}, {NULL}};
const Segment SGI349933[] = {{&S68386[38], 5}, {NULL}};
const Segment SGI349934[] = {{&S68386[94], 3}, {NULL}};
const Segment SGI349935[] = {{&S68387[34], 5}, {NULL}};
const Segment SGI349936[] = {{&S68387[90], 3}, {NULL}};
const Segment SGI349937[] = {{&S68388[5], 5}, {NULL}};
const Segment SGI349938[] = {{&S68388[16], 3}, {NULL}};
const Segment SGI349939[] = {{&S68435[35], 5}, {NULL}};
const Segment SGI349940[] = {{&S68435[52], 3}, {NULL}};
const Segment SGI349941[] = {{&S68436[5], 5}, {NULL}};
const Segment SGI349942[] = {{&S68436[16], 3}, {NULL}};
const Segment SGI349943[] = {{&S68437[34], 5}, {NULL}};
const Segment SGI349944[] = {{&S68437[90], 3}, {NULL}};
const Segment SGI349945[] = {{&S68471[5], 3}, {&S68471[5], 3}, {NULL}};
const Segment SGI349946[] = {{&S68471[9], 3}, {NULL}};
const Segment SGI349947[] = {{&S68471[127], 3}, {NULL}};
const Segment SGI349948[] = {{&S44572[58], 5}, {NULL}};
const Segment SGI349949[] = {{&S44572[13], 10}, {NULL}};
const Segment SGI349950[] = {{&S68524[5], 4}, {&S68524[5], 4}, {NULL}};
const Segment SGI349951[] = {{&S68524[151], 3}, {NULL}};
const Segment SGI349952[] = {{&S44618[61], 5}, {NULL}};
const Segment SGI349953[] = {{&S44618[13], 10}, {NULL}};
const Segment SGI349954[] = {{&S55122[5], 4}, {NULL}};
const Segment SGI349955[] = {{&S55122[47], 3}, {NULL}};
const Segment SGI349956[] = {{&S68642[5], 4}, {NULL}};
const Segment SGI349957[] = {{&S60301[28], 3}, {NULL}};
const Segment SGI349958[] = {{&S60301[102], 5}, {NULL}};
const Segment SGI349959[] = {{&S44729[53], 3}, {NULL}};
const Segment SGI349960[] = {{&S60302[28], 3}, {NULL}};
const Segment SGI349961[] = {{&S44729[13], 10}, {NULL}};
const Segment SGI349962[] = {{&S60302[102], 5}, {NULL}};
const Segment SGI349963[] = {{&S44730[34], 4}, {NULL}};
const Segment SGI349964[] = {{&S44730[13], 10}, {NULL}};
const Segment SGI349965[] = {{&S44731[53], 4}, {NULL}};
const Segment SGI349966[] = {{&S44731[13], 10}, {NULL}};
const Segment SGI349967[] = {{&S44732[53], 4}, {NULL}};
const Segment SGI349968[] = {{&S44732[13], 10}, {NULL}};
const Segment SGI349969[] = {{&S44733[34], 4}, {NULL}};
const Segment SGI349970[] = {{&S44733[13], 10}, {NULL}};
const Segment SGI349971[] = {{&S44734[53], 4}, {NULL}};
const Segment SGI349972[] = {{&S44734[13], 10}, {NULL}};
const Segment SGI349973[] = {{&S44736[53], 4}, {NULL}};
const Segment SGI349974[] = {{&S44736[13], 10}, {NULL}};
const Segment SGI349975[] = {{&S44738[53], 4}, {NULL}};
const Segment SGI349976[] = {{&S44738[13], 10}, {NULL}};
const Segment SGI349977[] = {{&S44740[67], 4}, {NULL}};
const Segment SGI349978[] = {{&S44740[13], 10}, {NULL}};
const Segment SGI349979[] = {{&S60343[28], 3}, {NULL}};
const Segment SGI349980[] = {{&S60343[102], 5}, {NULL}};
const Segment SGI349981[] = {{&S68690[5], 4}, {NULL}};
const Segment SGI349982[] = {{&S60369[28], 3}, {NULL}};
const Segment SGI349983[] = {{&S60369[102], 5}, {NULL}};
const Segment SGI349984[] = {{&S42300[0], 21}, {NULL}};
const Segment SGI349985[] = {{&S42301[0], 21}, {NULL}};
const Segment SGI349986[] = {{&S42302[0], 21}, {NULL}};
const Segment SGI349987[] = {{&S42302[57], 13}, {NULL}};
const Segment SGI349988[] = {{&S42303[0], 21}, {NULL}};
const Segment SGI349989[] = {{&S42303[57], 13}, {NULL}};
const Segment SGI349990[] = {{&S42304[0], 21}, {NULL}};
const Segment SGI349991[] = {{&S42305[0], 21}, {NULL}};
const Segment SGI349992[] = {{&S42305[23], 14}, {NULL}};
const Segment SGI349993[] = {{&S42305[66], 13}, {NULL}};
const Segment SGI349994[] = {{&S60400[28], 3}, {NULL}};
const Segment SGI349995[] = {{&S60400[102], 5}, {NULL}};
const Segment SGI349996[] = {{&S68761[44], 21}, {NULL}};
const Segment SGI349997[] = {{&S68761[6], 3}, {NULL}};
const Segment SGI349998[] = {{&S68761[86], 3}, {NULL}};
const Segment SGI349999[] = {{&S68762[44], 21}, {NULL}};
const Segment SGI350000[] = {{&S68762[6], 3}, {NULL}};
const Segment SGI350001[] = {{&S68762[86], 3}, {NULL}};
const Segment SGI350002[] = {{&S68763[44], 21}, {NULL}};
const Segment SGI350003[] = {{&S68763[6], 3}, {NULL}};
const Segment SGI350004[] = {{&S68763[86], 3}, {NULL}};
const Segment SGI350005[] = {{&S68764[44], 21}, {NULL}};
const Segment SGI350006[] = {{&S68764[6], 3}, {NULL}};
const Segment SGI350007[] = {{&S68764[86], 3}, {NULL}};
const Segment SGI350008[] = {{&S68765[41], 25}, {NULL}};
const Segment SGI350009[] = {{&S68765[6], 3}, {NULL}};
const Segment SGI350010[] = {{&S68765[87], 3}, {NULL}};
const Segment SGI350011[] = {{&S68766[44], 28}, {NULL}};
const Segment SGI350012[] = {{&S68766[6], 3}, {NULL}};
const Segment SGI350013[] = {{&S68766[93], 3}, {NULL}};
const Segment SGI350014[] = {{&S68767[44], 21}, {NULL}};
const Segment SGI350015[] = {{&S68767[6], 3}, {NULL}};
const Segment SGI350016[] = {{&S68767[86], 3}, {NULL}};
const Segment SGI350017[] = {{&S55364[5], 3}, {NULL}};
const Segment SGI350018[] = {{&S55364[31], 3}, {NULL}};
const Segment SGI350019[] = {{&S55365[5], 3}, {&S55365[5], 3}, {NULL}};
const Segment SGI350020[] = {{&S55365[31], 3}, {&S55365[31], 3}, {NULL}};
const Segment SGI350021[] = {{&S55366[5], 3}, {NULL}};
const Segment SGI350022[] = {{&S55366[32], 3}, {NULL}};
const Segment SGI350023[] = {{&S60457[28], 3}, {NULL}};
const Segment SGI350024[] = {{&S60457[102], 5}, {NULL}};
const Segment SGI350025[] = {{&S60474[28], 3}, {NULL}};
const Segment SGI350026[] = {{&S60474[102], 5}, {NULL}};
const Segment SGI350027[] = {{&S60498[28], 3}, {NULL}};
const Segment SGI350028[] = {{&S60498[102], 5}, {NULL}};
const Segment SGI350029[] = {{&S42306[0], 23}, {NULL}};
const Segment SGI350030[] = {{&S42306[59], 13}, {NULL}};
const Segment SGI350031[] = {{&S42307[0], 21}, {NULL}};
const Segment SGI350032[] = {{&S42307[23], 14}, {NULL}};
const Segment SGI350033[] = {{&S42307[66], 13}, {NULL}};
const Segment SGI350034[] = {{&S42308[0], 21}, {NULL}};
const Segment SGI350035[] = {{&S42308[23], 14}, {NULL}};
const Segment SGI350036[] = {{&S42308[66], 13}, {NULL}};
const Segment SGI350037[] = {{&S42309[0], 21}, {NULL}};
const Segment SGI350038[] = {{&S42309[23], 14}, {NULL}};
const Segment SGI350039[] = {{&S42309[66], 13}, {NULL}};
const Segment SGI350040[] = {{&S68878[5], 5}, {&S68878[105], 5}, {NULL}};
const Segment SGI350041[] = {{&S68878[11], 3}, {NULL}};
const Segment SGI350042[] = {{&S68878[131], 3}, {NULL}};
const Segment SGI350043[] = {{&S68880[43], 5}, {NULL}};
const Segment SGI350044[] = {{&S68880[6], 3}, {NULL}};
const Segment SGI350045[] = {{&S68880[69], 3}, {NULL}};
const Segment SGI350046[] = {{&S68907[35], 4}, {NULL}};
const Segment SGI350047[] = {{&S68907[90], 3}, {NULL}};
const Segment SGI350048[] = {{&S42608[6], 4}, {NULL}};
const Segment SGI350049[] = {{&S42608[12], 14}, {NULL}};
const Segment SGI350050[] = {{&S68927[42], 5}, {NULL}};
const Segment SGI350051[] = {{&S68927[6], 3}, {NULL}};
const Segment SGI350052[] = {{&S68927[68], 3}, {NULL}};
const Segment SGI350053[] = {{&S68928[5], 5}, {NULL}};
const Segment SGI350054[] = {{&S68928[16], 3}, {NULL}};
const Segment SGI350055[] = {{&S68942[5], 4}, {&S68942[5], 4}, {NULL}};
const Segment SGI350056[] = {{&S68942[111], 3}, {NULL}};
const Segment SGI350057[] = {{&S42638[6], 4}, {NULL}};
const Segment SGI350058[] = {{&S42638[55], 5}, {NULL}};
const Segment SGI350059[] = {{&S42638[12], 3}, {NULL}};
const Segment SGI350060[] = {{&S42641[6], 4}, {NULL}};
const Segment SGI350061[] = {{&S42641[53], 5}, {NULL}};
const Segment SGI350062[] = {{&S42641[12], 3}, {&S42641[64], 4}, {NULL}};
const Segment SGI350063[] = {{&S68982[5], 3}, {&S68982[5], 3}, {NULL}};
const Segment SGI350064[] = {{&S68982[9], 3}, {NULL}};
const Segment SGI350065[] = {{&S68982[127], 3}, {NULL}};
const Segment SGI350066[] = {{&S68983[5], 5}, {&S68983[5], 5}, {NULL}};
const Segment SGI350067[] = {{&S68983[11], 3}, {NULL}};
const Segment SGI350068[] = {{&S68983[131], 3}, {NULL}};
const Segment SGI350069[] = {{&S69056[5], 5}, {&S69056[104], 5}, {NULL}};
const Segment SGI350070[] = {{&S69056[11], 3}, {NULL}};
const Segment SGI350071[] = {{&S69056[130], 3}, {NULL}};
const Segment SGI350072[] = {{&S69057[5], 12}, {&S69057[5], 12}, {NULL}};
const Segment SGI350073[] = {{&S69057[76], 3}, {NULL}};
const Segment SGI350074[] = {{&S69057[145], 3}, {NULL}};
const Segment SGI350075[] = {{&S69058[5], 7}, {&S69058[5], 7}, {NULL}};
const Segment SGI350076[] = {{&S69058[13], 3}, {NULL}};
const Segment SGI350077[] = {{&S69058[135], 3}, {NULL}};
const Segment SGI350078[] = {{&S69065[5], 5}, {NULL}};
const Segment SGI350079[] = {{&S69065[16], 3}, {NULL}};
const Segment SGI350080[] = {{&S69066[5], 5}, {NULL}};
const Segment SGI350081[] = {{&S69066[21], 3}, {NULL}};
const Segment SGI350082[] = {{&S69087[5], 4}, {&S69087[5], 4}, {NULL}};
const Segment SGI350083[] = {{&S69087[151], 3}, {NULL}};
const Segment SGI350084[] = {{&S60664[28], 7}, {NULL}};
const Segment SGI350085[] = {{&S60664[106], 5}, {NULL}};
const Segment SGI350086[] = {{&S60665[28], 4}, {NULL}};
const Segment SGI350087[] = {{&S60665[103], 5}, {NULL}};
const Segment SGI350088[] = {{&S69191[5], 17}, {NULL}};
const Segment SGI350089[] = {{&S69191[28], 3}, {NULL}};
const Segment SGI350090[] = {{&S69192[5], 16}, {NULL}};
const Segment SGI350091[] = {{&S69192[27], 3}, {NULL}};
const Segment SGI350092[] = {{&S69193[5], 16}, {NULL}};
const Segment SGI350093[] = {{&S69193[32], 3}, {NULL}};
const Segment SGI350094[] = {{&S69195[35], 5}, {NULL}};
const Segment SGI350095[] = {{&S69195[52], 3}, {NULL}};
const Segment SGI350096[] = {{&S69207[5], 5}, {NULL}};
const Segment SGI350097[] = {{&S69207[16], 3}, {NULL}};
const Segment SGI350098[] = {{&S69208[35], 5}, {NULL}};
const Segment SGI350099[] = {{&S69208[52], 3}, {NULL}};
const Segment SGI350100[] = {{&S69234[5], 4}, {NULL}};
const Segment SGI350101[] = {{&S69318[5], 3}, {&S69318[5], 3}, {NULL}};
const Segment SGI350102[] = {{&S69318[109], 3}, {NULL}};
const Segment SGI350103[] = {{&S69319[43], 3}, {NULL}};
const Segment SGI350104[] = {{&S69319[6], 3}, {NULL}};
const Segment SGI350105[] = {{&S69319[67], 3}, {NULL}};
const Segment SGI350106[] = {{&S60879[28], 3}, {NULL}};
const Segment SGI350107[] = {{&S60879[102], 5}, {NULL}};
const Segment SGI350108[] = {{&S45338[62], 4}, {NULL}};
const Segment SGI350109[] = {{&S45338[13], 10}, {NULL}};
const Segment SGI350110[] = {{&S45339[62], 4}, {NULL}};
const Segment SGI350111[] = {{&S45339[13], 10}, {NULL}};
const Segment SGI350112[] = {{&S69373[5], 5}, {&S69373[5], 5}, {NULL}};
const Segment SGI350113[] = {{&S69373[113], 3}, {NULL}};
const Segment SGI350114[] = {{&S60896[28], 5}, {NULL}};
const Segment SGI350115[] = {{&S60896[104], 5}, {NULL}};
const Segment SGI350116[] = {{&S60897[28], 7}, {NULL}};
const Segment SGI350117[] = {{&S60897[107], 5}, {NULL}};
const Segment SGI350118[] = {{&S69386[49], 20}, {NULL}};
const Segment SGI350119[] = {{&S69387[49], 20}, {NULL}};
const Segment SGI350120[] = {{&S69388[49], 20}, {NULL}};
const Segment SGI350121[] = {{&S69389[49], 20}, {NULL}};
const Segment SGI350122[] = {{&S69508[35], 3}, {NULL}};
const Segment SGI350123[] = {{&S69508[50], 3}, {NULL}};
const Segment SGI350124[] = {{&S69509[35], 7}, {NULL}};
const Segment SGI350125[] = {{&S69509[54], 3}, {NULL}};
const Segment SGI350126[] = {{&S69510[34], 3}, {NULL}};
const Segment SGI350127[] = {{&S69510[48], 3}, {NULL}};
const Segment SGI350128[] = {{&S69511[34], 7}, {NULL}};
const Segment SGI350129[] = {{&S69511[52], 3}, {NULL}};
const Segment SGI350130[] = {{&S61038[28], 3}, {NULL}};
const Segment SGI350131[] = {{&S61038[102], 5}, {NULL}};
const Segment SGI350132[] = {{&S69531[43], 4}, {NULL}};
const Segment SGI350133[] = {{&S69531[6], 3}, {NULL}};
const Segment SGI350134[] = {{&S69531[68], 3}, {NULL}};
const Segment SGI350135[] = {{&S69544[5], 7}, {&S69544[5], 7}, {NULL}};
const Segment SGI350136[] = {{&S69544[13], 3}, {NULL}};
const Segment SGI350137[] = {{&S69544[134], 3}, {NULL}};
const Segment SGI350138[] = {{&S69545[5], 4}, {&S69545[5], 4}, {NULL}};
const Segment SGI350139[] = {{&S69545[10], 3}, {NULL}};
const Segment SGI350140[] = {{&S69545[128], 3}, {NULL}};
const Segment SGI350141[] = {{&S69557[5], 5}, {NULL}};
const Segment SGI350142[] = {{&S69557[21], 3}, {NULL}};
const Segment SGI350143[] = {{&S69558[5], 5}, {NULL}};
const Segment SGI350144[] = {{&S69558[16], 3}, {NULL}};
const Segment SGI350145[] = {{&S69559[35], 3}, {NULL}};
const Segment SGI350146[] = {{&S69559[89], 3}, {NULL}};
const Segment SGI350147[] = {{&S69560[34], 5}, {NULL}};
const Segment SGI350148[] = {{&S69560[90], 3}, {NULL}};
const Segment SGI350149[] = {{&S69577[5], 4}, {NULL}};
const Segment SGI350150[] = {{&S69577[15], 3}, {NULL}};
const Segment SGI350151[] = {{&S69578[5], 4}, {NULL}};
const Segment SGI350152[] = {{&S69578[20], 3}, {NULL}};
const Segment SGI350153[] = {{&S69579[5], 6}, {NULL}};
const Segment SGI350154[] = {{&S69579[22], 3}, {NULL}};
const Segment SGI350155[] = {{&S69581[5], 5}, {NULL}};
const Segment SGI350156[] = {{&S69581[16], 3}, {NULL}};
const Segment SGI350157[] = {{&S69582[35], 5}, {NULL}};
const Segment SGI350158[] = {{&S69582[52], 3}, {NULL}};
const Segment SGI350159[] = {{&S69589[35], 5}, {NULL}};
const Segment SGI350160[] = {{&S69589[91], 3}, {NULL}};
const Segment SGI350161[] = {{&S69601[5], 3}, {&S69601[5], 3}, {NULL}};
const Segment SGI350162[] = {{&S69601[9], 3}, {NULL}};
const Segment SGI350163[] = {{&S69601[127], 3}, {NULL}};
const Segment SGI350164[] = {{&S61121[28], 5}, {NULL}};
const Segment SGI350165[] = {{&S61121[104], 5}, {NULL}};
const Segment SGI350166[] = {{&S69610[5], 5}, {&S69610[97], 5}, {NULL}};
const Segment SGI350167[] = {{&S69610[113], 3}, {NULL}};
const Segment SGI350168[] = {{&S69611[5], 7}, {&S69611[99], 7}, {NULL}};
const Segment SGI350169[] = {{&S69611[117], 3}, {NULL}};
const Segment SGI350170[] = {{&S69627[43], 5}, {NULL}};
const Segment SGI350171[] = {{&S69627[6], 3}, {NULL}};
const Segment SGI350172[] = {{&S69627[69], 3}, {NULL}};
const Segment SGI350173[] = {{&S69642[21], 7}, {NULL}};
const Segment SGI350174[] = {{&S69642[0], 14}, {NULL}};
const Segment SGI350175[] = {{&S56245[5], 9}, {NULL}};
const Segment SGI350176[] = {{&S56245[51], 5}, {NULL}};
const Segment SGI350177[] = {{&S56246[48], 7}, {NULL}};
const Segment SGI350178[] = {{&S56246[40], 13}, {NULL}};
const Segment SGI350179[] = {{&S56247[49], 7}, {NULL}};
const Segment SGI350180[] = {{&S56247[40], 3}, {NULL}};
const Segment SGI350181[] = {{&S56248[51], 7}, {NULL}};
const Segment SGI350182[] = {{&S56248[40], 5}, {NULL}};
const Segment SGI350183[] = {{&S43354[6], 4}, {NULL}};
const Segment SGI350184[] = {{&S43354[82], 5}, {NULL}};
const Segment SGI350185[] = {{&S43354[12], 14}, {&S43354[40], 12}, {NULL}};
const Segment SGI350186[] = {{&S43354[12], 14}, {NULL}};
const Segment SGI350187[] = {{&S56249[5], 8}, {NULL}};
const Segment SGI350188[] = {{&S56249[19], 3}, {NULL}};
const Segment SGI350189[] = {{&S43355[6], 4}, {NULL}};
const Segment SGI350190[] = {{&S43355[82], 5}, {NULL}};
const Segment SGI350191[] = {{&S43355[12], 14}, {&S43355[40], 12}, {NULL}};
const Segment SGI350192[] = {{&S43355[12], 14}, {NULL}};
const Segment SGI350193[] = {{&S56250[49], 8}, {NULL}};
const Segment SGI350194[] = {{&S56250[40], 3}, {NULL}};
const Segment SGI350195[] = {{&S43356[6], 4}, {NULL}};
const Segment SGI350196[] = {{&S43356[81], 5}, {NULL}};
const Segment SGI350197[] = {{&S43356[12], 14}, {&S43356[40], 11}, {NULL}};
const Segment SGI350198[] = {{&S43356[12], 14}, {NULL}};
const Segment SGI350199[] = {{&S43357[6], 4}, {NULL}};
const Segment SGI350200[] = {{&S43357[59], 5}, {NULL}};
const Segment SGI350201[] = {{&S43357[12], 14}, {&S43357[70], 31}, {NULL}};
const Segment SGI350202[] = {{&S43357[12], 14}, {NULL}};
const Segment SGI350203[] = {{&S43358[6], 4}, {NULL}};
const Segment SGI350204[] = {{&S43358[82], 5}, {NULL}};
const Segment SGI350205[] = {{&S43358[12], 14}, {&S43358[40], 12}, {NULL}};
const Segment SGI350206[] = {{&S43358[12], 14}, {NULL}};
const Segment SGI350207[] = {{&S42919[0], 16}, {NULL}};
const Segment SGI350208[] = {{&S42919[29], 3}, {NULL}};
const Segment SGI350209[] = {{&S42919[66], 13}, {NULL}};
const Segment SGI350210[] = {{&S43359[6], 4}, {NULL}};
const Segment SGI350211[] = {{&S43359[82], 5}, {NULL}};
const Segment SGI350212[] = {{&S43359[12], 14}, {&S43359[40], 12}, {NULL}};
const Segment SGI350213[] = {{&S43359[12], 14}, {NULL}};
const Segment SGI350214[] = {{&S42920[0], 16}, {NULL}};
const Segment SGI350215[] = {{&S42920[29], 3}, {NULL}};
const Segment SGI350216[] = {{&S42920[73], 11}, {NULL}};
const Segment SGI350217[] = {{&S43360[6], 4}, {NULL}};
const Segment SGI350218[] = {{&S43360[82], 5}, {NULL}};
const Segment SGI350219[] = {{&S43360[12], 14}, {&S43360[41], 11}, {NULL}};
const Segment SGI350220[] = {{&S43360[12], 14}, {NULL}};
const Segment SGI350221[] = {{&S43361[6], 4}, {NULL}};
const Segment SGI350222[] = {{&S43361[59], 5}, {NULL}};
const Segment SGI350223[] = {{&S43361[12], 14}, {NULL}};
const Segment SGI350224[] = {{&S43362[6], 4}, {NULL}};
const Segment SGI350225[] = {{&S43362[81], 5}, {NULL}};
const Segment SGI350226[] = {{&S43362[12], 14}, {&S43362[40], 11}, {NULL}};
const Segment SGI350227[] = {{&S43362[12], 14}, {NULL}};
const Segment SGI350228[] = {{&S43363[6], 4}, {NULL}};
const Segment SGI350229[] = {{&S43363[82], 5}, {NULL}};
const Segment SGI350230[] = {{&S43363[12], 14}, {&S43363[40], 12}, {NULL}};
const Segment SGI350231[] = {{&S43363[12], 14}, {NULL}};
const Segment SGI350232[] = {{&S43365[6], 4}, {NULL}};
const Segment SGI350233[] = {{&S43365[82], 5}, {NULL}};
const Segment SGI350234[] = {{&S43365[12], 14}, {&S43365[40], 12}, {NULL}};
const Segment SGI350235[] = {{&S43365[12], 14}, {NULL}};
const Segment SGI350236[] = {{&S43367[6], 4}, {NULL}};
const Segment SGI350237[] = {{&S43367[81], 5}, {NULL}};
const Segment SGI350238[] = {{&S43367[12], 14}, {&S43367[40], 11}, {NULL}};
const Segment SGI350239[] = {{&S43367[12], 14}, {NULL}};
const Segment SGI350240[] = {{&S43368[6], 5}, {NULL}};
const Segment SGI350241[] = {{&S43368[13], 14}, {NULL}};
const Segment SGI350242[] = {{&S43452[6], 4}, {NULL}};
const Segment SGI350243[] = {{&S43452[57], 5}, {NULL}};
const Segment SGI350244[] = {{&S43452[12], 14}, {NULL}};
const Segment SGI350245[] = {{&S43453[6], 4}, {NULL}};
const Segment SGI350246[] = {{&S43453[58], 5}, {NULL}};
const Segment SGI350247[] = {{&S43453[12], 14}, {NULL}};
const Segment SGI350248[] = {{&S43454[6], 4}, {NULL}};
const Segment SGI350249[] = {{&S43454[81], 5}, {NULL}};
const Segment SGI350250[] = {{&S43454[12], 14}, {&S43454[40], 12}, {NULL}};
const Segment SGI350251[] = {{&S43454[12], 14}, {NULL}};
const Segment SGI350252[] = {{&S43455[6], 4}, {NULL}};
const Segment SGI350253[] = {{&S43455[12], 22}, {NULL}};
const Segment SGI350254[] = {{&S43455[12], 14}, {NULL}};
const Segment SGI350255[] = {{&S43456[6], 4}, {NULL}};
const Segment SGI350256[] = {{&S43456[81], 5}, {NULL}};
const Segment SGI350257[] = {{&S43456[12], 14}, {&S43456[40], 12}, {NULL}};
const Segment SGI350258[] = {{&S43456[12], 14}, {NULL}};
const Segment SGI350259[] = {{&S43457[6], 4}, {NULL}};
const Segment SGI350260[] = {{&S43457[81], 5}, {NULL}};
const Segment SGI350261[] = {{&S43457[12], 14}, {&S43457[40], 12}, {NULL}};
const Segment SGI350262[] = {{&S43457[12], 14}, {NULL}};
const Segment SGI350263[] = {{&S43458[6], 4}, {NULL}};
const Segment SGI350264[] = {{&S43458[81], 5}, {NULL}};
const Segment SGI350265[] = {{&S43458[12], 14}, {&S43458[40], 12}, {NULL}};
const Segment SGI350266[] = {{&S43458[12], 14}, {NULL}};
const Segment SGI350267[] = {{&S43459[6], 4}, {NULL}};
const Segment SGI350268[] = {{&S43459[81], 5}, {NULL}};
const Segment SGI350269[] = {{&S43459[12], 14}, {&S43459[40], 12}, {NULL}};
const Segment SGI350270[] = {{&S43459[12], 14}, {NULL}};
const Segment SGI350271[] = {{&S43460[6], 4}, {NULL}};
const Segment SGI350272[] = {{&S43460[81], 5}, {NULL}};
const Segment SGI350273[] = {{&S43460[12], 14}, {&S43460[40], 12}, {NULL}};
const Segment SGI350274[] = {{&S43460[12], 14}, {NULL}};
const Segment SGI350275[] = {{&S43461[6], 4}, {NULL}};
const Segment SGI350276[] = {{&S43461[12], 14}, {NULL}};
const Segment SGI350277[] = {{&S69767[35], 5}, {NULL}};
const Segment SGI350278[] = {{&S69767[52], 3}, {NULL}};
const Segment SGI350279[] = {{&S69768[35], 7}, {NULL}};
const Segment SGI350280[] = {{&S69768[54], 3}, {NULL}};
const Segment SGI350281[] = {{&S69769[34], 7}, {NULL}};
const Segment SGI350282[] = {{&S69769[52], 3}, {NULL}};
const Segment SGI350283[] = {{&S69770[34], 5}, {NULL}};
const Segment SGI350284[] = {{&S69770[50], 3}, {NULL}};
const Segment SGI350285[] = {{&S69771[20], 4}, {NULL}};
const Segment SGI350286[] = {{&S69795[5], 5}, {&S69795[68], 5}, {NULL}};
const Segment SGI350287[] = {{&S69795[84], 3}, {NULL}};
const Segment SGI350288[] = {{&S43503[6], 4}, {NULL}};
const Segment SGI350289[] = {{&S43503[12], 3}, {NULL}};
const Segment SGI350290[] = {{&S43504[6], 4}, {NULL}};
const Segment SGI350291[] = {{&S43504[12], 3}, {NULL}};
const Segment SGI350292[] = {{&S43505[6], 4}, {NULL}};
const Segment SGI350293[] = {{&S43505[12], 3}, {NULL}};
const Segment SGI350294[] = {{&S43506[6], 4}, {NULL}};
const Segment SGI350295[] = {{&S43506[12], 3}, {NULL}};
const Segment SGI350296[] = {{&S43507[6], 4}, {NULL}};
const Segment SGI350297[] = {{&S43507[12], 3}, {NULL}};
const Segment SGI350298[] = {{&S69817[38], 5}, {NULL}};
const Segment SGI350299[] = {{&S69817[94], 3}, {NULL}};
const Segment SGI350300[] = {{&S45820[61], 3}, {NULL}};
const Segment SGI350301[] = {{&S45820[13], 10}, {NULL}};
const Segment SGI350302[] = {{&S45824[61], 4}, {NULL}};
const Segment SGI350303[] = {{&S45824[13], 10}, {NULL}};
const Segment SGI350304[] = {{&S69824[5], 3}, {&S69824[5], 3}, {NULL}};
const Segment SGI350305[] = {{&S69824[67], 3}, {NULL}};
const Segment SGI350306[] = {{&S69824[127], 3}, {NULL}};
const Segment SGI350307[] = {{&S69855[21], 7}, {NULL}};
const Segment SGI350308[] = {{&S69855[0], 14}, {NULL}};
const Segment SGI350309[] = {{&S69856[42], 5}, {NULL}};
const Segment SGI350310[] = {{&S69856[6], 3}, {NULL}};
const Segment SGI350311[] = {{&S43140[0], 16}, {NULL}};
const Segment SGI350312[] = {{&S43140[29], 3}, {NULL}};
const Segment SGI350313[] = {{&S69856[68], 3}, {NULL}};
const Segment SGI350314[] = {{&S43140[64], 13}, {NULL}};
const Segment SGI350315[] = {{&S69881[35], 5}, {NULL}};
const Segment SGI350316[] = {{&S69881[52], 3}, {NULL}};
const Segment SGI350317[] = {{&S69898[35], 5}, {NULL}};
const Segment SGI350318[] = {{&S69898[52], 3}, {NULL}};
const Segment SGI350319[] = {{&S69899[34], 5}, {NULL}};
const Segment SGI350320[] = {{&S69899[50], 3}, {NULL}};
const Segment SGI350321[] = {{&S69909[5], 3}, {&S69909[5], 3}, {NULL}};
const Segment SGI350322[] = {{&S69909[149], 3}, {NULL}};
const Segment SGI350323[] = {{&S69910[21], 5}, {NULL}};
const Segment SGI350324[] = {{&S69910[0], 14}, {NULL}};
const Segment SGI350325[] = {{&S69917[5], 5}, {NULL}};
const Segment SGI350326[] = {{&S69918[5], 7}, {NULL}};
const Segment SGI350327[] = {{&S45952[66], 4}, {NULL}};
const Segment SGI350328[] = {{&S45952[13], 10}, {NULL}};
const Segment SGI350329[] = {{&S69984[5], 12}, {&S69984[5], 12}, {NULL}};
const Segment SGI350330[] = {{&S69984[76], 3}, {NULL}};
const Segment SGI350331[] = {{&S69984[144], 3}, {NULL}};
const Segment SGI350332[] = {{&S70086[38], 5}, {NULL}};
const Segment SGI350333[] = {{&S70086[94], 3}, {NULL}};
const Segment SGI350334[] = {{&S70087[41], 5}, {NULL}};
const Segment SGI350335[] = {{&S70087[6], 3}, {NULL}};
const Segment SGI350336[] = {{&S70087[67], 3}, {NULL}};
const Segment SGI350337[] = {{&S70088[5], 14}, {NULL}};
const Segment SGI350338[] = {{&S70088[20], 3}, {NULL}};
const Segment SGI350339[] = {{&S70089[38], 7}, {NULL}};
const Segment SGI350340[] = {{&S70089[96], 3}, {NULL}};
const Segment SGI350341[] = {{&S70106[5], 5}, {&S70106[104], 5}, {NULL}};
const Segment SGI350342[] = {{&S70106[11], 3}, {NULL}};
const Segment SGI350343[] = {{&S70106[130], 3}, {NULL}};
const Segment SGI350344[] = {{&S70198[5], 5}, {&S70198[105], 5}, {NULL}};
const Segment SGI350345[] = {{&S70198[69], 3}, {NULL}};
const Segment SGI350346[] = {{&S70198[131], 3}, {NULL}};
const Segment SGI350347[] = {{&S70312[35], 16}, {NULL}};
const Segment SGI350348[] = {{&S70312[63], 3}, {NULL}};
const Segment SGI350349[] = {{&S70313[5], 16}, {NULL}};
const Segment SGI350350[] = {{&S70313[27], 3}, {NULL}};
const Segment SGI350351[] = {{&S70314[34], 7}, {NULL}};
const Segment SGI350352[] = {{&S70314[52], 3}, {NULL}};
const Segment SGI350353[] = {{&S70325[42], 5}, {NULL}};
const Segment SGI350354[] = {{&S70325[6], 3}, {NULL}};
const Segment SGI350355[] = {{&S70325[68], 3}, {NULL}};
const Segment SGI350356[] = {{&S70326[43], 3}, {NULL}};
const Segment SGI350357[] = {{&S70326[6], 3}, {NULL}};
const Segment SGI350358[] = {{&S70326[67], 3}, {NULL}};
const Segment SGI350359[] = {{&S70340[5], 3}, {NULL}};
const Segment SGI350360[] = {{&S70340[14], 3}, {NULL}};
const Segment SGI350361[] = {{&S70341[35], 3}, {NULL}};
const Segment SGI350362[] = {{&S70341[50], 3}, {NULL}};
const Segment SGI350363[] = {{&S70342[34], 5}, {NULL}};
const Segment SGI350364[] = {{&S70342[50], 3}, {NULL}};
const Segment SGI350365[] = {{&S70375[35], 9}, {NULL}};
const Segment SGI350366[] = {{&S70375[95], 3}, {NULL}};
const Segment SGI350367[] = {{&S70376[5], 18}, {NULL}};
const Segment SGI350368[] = {{&S70376[24], 3}, {NULL}};
const Segment SGI350369[] = {{&S70377[41], 5}, {NULL}};
const Segment SGI350370[] = {{&S70377[6], 3}, {NULL}};
const Segment SGI350371[] = {{&S70377[67], 3}, {NULL}};
const Segment SGI350372[] = {{&S70378[35], 13}, {NULL}};
const Segment SGI350373[] = {{&S70378[99], 3}, {NULL}};
const Segment SGI350374[] = {{&S70398[35], 5}, {NULL}};
const Segment SGI350375[] = {{&S70398[52], 3}, {NULL}};
const Segment SGI350376[] = {{&S70399[5], 5}, {NULL}};
const Segment SGI350377[] = {{&S70399[16], 3}, {NULL}};
const Segment SGI350378[] = {{&S70400[34], 5}, {NULL}};
const Segment SGI350379[] = {{&S70400[90], 3}, {NULL}};
const Segment SGI350380[] = {{&S43913[6], 4}, {NULL}};
const Segment SGI350381[] = {{&S43913[55], 5}, {NULL}};
const Segment SGI350382[] = {{&S43913[12], 3}, {NULL}};
const Segment SGI350383[] = {{&S43916[6], 4}, {NULL}};
const Segment SGI350384[] = {{&S43916[53], 5}, {NULL}};
const Segment SGI350385[] = {{&S43916[12], 3}, {&S43916[64], 4}, {NULL}};
const Segment SGI350386[] = {{&S43919[6], 4}, {NULL}};
const Segment SGI350387[] = {{&S43919[59], 5}, {NULL}};
const Segment SGI350388[] = {{&S43919[12], 14}, {NULL}};
const Segment SGI350389[] = {{&S43920[6], 4}, {NULL}};
const Segment SGI350390[] = {{&S43920[81], 5}, {NULL}};
const Segment SGI350391[] = {{&S43920[12], 14}, {&S43920[40], 11}, {NULL}};
const Segment SGI350392[] = {{&S43920[12], 14}, {NULL}};
const Segment SGI350393[] = {{&S46349[58], 5}, {NULL}};
const Segment SGI350394[] = {{&S46349[13], 10}, {NULL}};
const Segment SGI350395[] = {{&S62068[28], 3}, {NULL}};
const Segment SGI350396[] = {{&S62068[102], 5}, {NULL}};
const Segment SGI350397[] = {{&S62127[28], 3}, {NULL}};
const Segment SGI350398[] = {{&S62127[102], 5}, {NULL}};
const Segment SGI350399[] = {{&S62128[28], 5}, {NULL}};
const Segment SGI350400[] = {{&S62128[104], 5}, {NULL}};
const Segment SGI350401[] = {{&S70620[35], 3}, {NULL}};
const Segment SGI350402[] = {{&S70620[89], 3}, {NULL}};
const Segment SGI350403[] = {{&S70647[5], 5}, {&S70647[104], 5}, {NULL}};
const Segment SGI350404[] = {{&S70647[69], 3}, {NULL}};
const Segment SGI350405[] = {{&S70647[130], 3}, {NULL}};
const Segment SGI350406[] = {{&S70654[5], 4}, {NULL}};
const Segment SGI350407[] = {{&S62234[28], 3}, {NULL}};
const Segment SGI350408[] = {{&S62234[102], 5}, {NULL}};
const Segment SGI350409[] = {{&S70668[35], 3}, {NULL}};
const Segment SGI350410[] = {{&S70668[89], 3}, {NULL}};
const Segment SGI350411[] = {{&S62247[28], 7}, {NULL}};
const Segment SGI350412[] = {{&S62247[106], 5}, {NULL}};
const Segment SGI350413[] = {{&S62248[28], 5}, {NULL}};
const Segment SGI350414[] = {{&S62248[104], 5}, {NULL}};
const Segment SGI350415[] = {{&S56843[49], 7}, {NULL}};
const Segment SGI350416[] = {{&S56843[40], 3}, {NULL}};
const Segment SGI350417[] = {{&S70684[42], 3}, {NULL}};
const Segment SGI350418[] = {{&S70684[6], 3}, {NULL}};
const Segment SGI350419[] = {{&S70684[66], 3}, {NULL}};
const Segment SGI350420[] = {{&S70756[5], 3}, {&S70756[5], 3}, {NULL}};
const Segment SGI350421[] = {{&S70756[9], 3}, {NULL}};
const Segment SGI350422[] = {{&S70756[126], 3}, {NULL}};
const Segment SGI350423[] = {{&S62331[28], 3}, {NULL}};
const Segment SGI350424[] = {{&S62331[102], 5}, {NULL}};
const Segment SGI350425[] = {{&S62355[28], 3}, {NULL}};
const Segment SGI350426[] = {{&S62355[102], 5}, {NULL}};
const Segment SGI350427[] = {{&S62386[28], 3}, {NULL}};
const Segment SGI350428[] = {{&S62386[102], 5}, {NULL}};
const Segment SGI350429[] = {{&S70892[5], 3}, {&S70892[5], 3}, {NULL}};
const Segment SGI350430[] = {{&S70892[9], 3}, {NULL}};
const Segment SGI350431[] = {{&S70892[127], 3}, {NULL}};
const Segment SGI350432[] = {{&S70919[5], 5}, {NULL}};
const Segment SGI350433[] = {{&S70919[21], 3}, {NULL}};
const Segment SGI350434[] = {{&S70920[5], 7}, {NULL}};
const Segment SGI350435[] = {{&S70920[23], 3}, {NULL}};
const Segment SGI350436[] = {{&S57083[5], 4}, {&S57083[5], 4}, {NULL}};
const Segment SGI350437[] = {{&S57083[47], 3}, {&S57083[47], 3}, {NULL}};
const Segment SGI350438[] = {{&S57084[5], 4}, {NULL}};
const Segment SGI350439[] = {{&S57084[47], 3}, {NULL}};
const Segment SGI350440[] = {{&S44344[6], 4}, {NULL}};
const Segment SGI350441[] = {{&S44344[57], 5}, {NULL}};
const Segment SGI350442[] = {{&S44344[12], 14}, {NULL}};
const Segment SGI350443[] = {{&S44345[6], 4}, {NULL}};
const Segment SGI350444[] = {{&S44345[57], 5}, {NULL}};
const Segment SGI350445[] = {{&S44345[12], 14}, {NULL}};
const Segment SGI350446[] = {{&S44346[6], 4}, {NULL}};
const Segment SGI350447[] = {{&S44346[80], 5}, {NULL}};
const Segment SGI350448[] = {{&S44346[12], 14}, {&S44346[40], 12}, {NULL}};
const Segment SGI350449[] = {{&S44346[12], 14}, {NULL}};
const Segment SGI350450[] = {{&S44347[6], 5}, {NULL}};
const Segment SGI350451[] = {{&S44347[13], 14}, {NULL}};
const Segment SGI350452[] = {{&S44207[0], 21}, {NULL}};
const Segment SGI350453[] = {{&S44207[34], 3}, {NULL}};
const Segment SGI350454[] = {{&S44208[0], 21}, {NULL}};
const Segment SGI350455[] = {{&S44208[34], 3}, {NULL}};
const Segment SGI350456[] = {{&S62588[28], 5}, {NULL}};
const Segment SGI350457[] = {{&S62588[104], 5}, {NULL}};
const Segment SGI350458[] = {{&S62589[28], 7}, {NULL}};
const Segment SGI350459[] = {{&S62589[106], 5}, {NULL}};
const Segment SGI350460[] = {{&S62623[28], 7}, {NULL}};
const Segment SGI350461[] = {{&S62623[106], 5}, {NULL}};
const Segment SGI350462[] = {{&S71016[5], 3}, {&S71016[5], 3}, {NULL}};
const Segment SGI350463[] = {{&S71016[109], 3}, {NULL}};
const Segment SGI350464[] = {{&S44570[6], 4}, {NULL}};
const Segment SGI350465[] = {{&S44570[53], 5}, {NULL}};
const Segment SGI350466[] = {{&S44570[12], 3}, {NULL}};
const Segment SGI350467[] = {{&S57323[46], 7}, {NULL}};
const Segment SGI350468[] = {{&S71028[5], 5}, {NULL}};
const Segment SGI350469[] = {{&S71028[21], 3}, {NULL}};
const Segment SGI350470[] = {{&S57323[37], 3}, {NULL}};
const Segment SGI350471[] = {{&S57324[46], 5}, {NULL}};
const Segment SGI350472[] = {{&S57324[37], 3}, {NULL}};
const Segment SGI350473[] = {{&S71047[43], 4}, {NULL}};
const Segment SGI350474[] = {{&S71047[6], 3}, {NULL}};
const Segment SGI350475[] = {{&S71047[68], 3}, {NULL}};
const Segment SGI350476[] = {{&S71065[5], 4}, {NULL}};
const Segment SGI350477[] = {{&S71067[5], 5}, {NULL}};
const Segment SGI350478[] = {{&S44616[6], 4}, {NULL}};
const Segment SGI350479[] = {{&S44616[108], 5}, {NULL}};
const Segment SGI350480[] = {{&S44616[12], 14}, {&S44616[40], 12}, {NULL}};
const Segment SGI350481[] = {{&S44616[12], 14}, {NULL}};
const Segment SGI350482[] = {{&S44617[6], 4}, {NULL}};
const Segment SGI350483[] = {{&S44617[59], 5}, {NULL}};
const Segment SGI350484[] = {{&S44617[12], 14}, {&S44617[70], 31}, {NULL}};
const Segment SGI350485[] = {{&S44617[12], 14}, {NULL}};
const Segment SGI350486[] = {{&S44619[6], 4}, {NULL}};
const Segment SGI350487[] = {{&S44619[82], 5}, {NULL}};
const Segment SGI350488[] = {{&S44619[12], 14}, {&S44619[40], 12}, {NULL}};
const Segment SGI350489[] = {{&S44619[12], 14}, {NULL}};
const Segment SGI350490[] = {{&S44620[6], 4}, {NULL}};
const Segment SGI350491[] = {{&S44620[81], 5}, {NULL}};
const Segment SGI350492[] = {{&S44620[12], 14}, {&S44620[40], 11}, {NULL}};
const Segment SGI350493[] = {{&S44620[12], 14}, {NULL}};
const Segment SGI350494[] = {{&S44621[6], 4}, {NULL}};
const Segment SGI350495[] = {{&S44621[82], 5}, {NULL}};
const Segment SGI350496[] = {{&S44621[12], 14}, {&S44621[41], 11}, {NULL}};
const Segment SGI350497[] = {{&S44621[12], 14}, {NULL}};
const Segment SGI350498[] = {{&S44622[6], 4}, {NULL}};
const Segment SGI350499[] = {{&S44622[82], 5}, {NULL}};
const Segment SGI350500[] = {{&S44622[12], 14}, {&S44622[41], 11}, {NULL}};
const Segment SGI350501[] = {{&S44622[12], 14}, {NULL}};
const Segment SGI350502[] = {{&S44623[6], 4}, {NULL}};
const Segment SGI350503[] = {{&S44623[59], 5}, {NULL}};
const Segment SGI350504[] = {{&S44623[12], 14}, {NULL}};
const Segment SGI350505[] = {{&S44624[6], 4}, {NULL}};
const Segment SGI350506[] = {{&S44624[82], 5}, {NULL}};
const Segment SGI350507[] = {{&S44624[12], 14}, {&S44624[40], 12}, {NULL}};
const Segment SGI350508[] = {{&S44624[12], 14}, {NULL}};
const Segment SGI350509[] = {{&S44625[6], 4}, {NULL}};
const Segment SGI350510[] = {{&S44625[81], 5}, {NULL}};
const Segment SGI350511[] = {{&S44625[12], 14}, {&S44625[40], 11}, {NULL}};
const Segment SGI350512[] = {{&S44625[12], 14}, {NULL}};
const Segment SGI350513[] = {{&S44626[6], 4}, {NULL}};
const Segment SGI350514[] = {{&S44626[82], 5}, {NULL}};
const Segment SGI350515[] = {{&S44626[12], 14}, {&S44626[40], 12}, {NULL}};
const Segment SGI350516[] = {{&S44626[12], 14}, {NULL}};
const Segment SGI350517[] = {{&S71085[5], 3}, {&S71085[5], 3}, {NULL}};
const Segment SGI350518[] = {{&S71085[9], 3}, {NULL}};
const Segment SGI350519[] = {{&S71085[127], 3}, {NULL}};
const Segment SGI350520[] = {{&S44627[6], 4}, {NULL}};
const Segment SGI350521[] = {{&S44627[82], 5}, {NULL}};
const Segment SGI350522[] = {{&S44627[12], 14}, {&S44627[40], 12}, {NULL}};
const Segment SGI350523[] = {{&S44627[12], 14}, {NULL}};
const Segment SGI350524[] = {{&S44629[6], 4}, {NULL}};
const Segment SGI350525[] = {{&S44629[81], 5}, {NULL}};
const Segment SGI350526[] = {{&S44629[12], 14}, {&S44629[40], 11}, {NULL}};
const Segment SGI350527[] = {{&S44629[12], 14}, {NULL}};
const Segment SGI350528[] = {{&S44630[6], 4}, {NULL}};
const Segment SGI350529[] = {{&S44630[82], 5}, {NULL}};
const Segment SGI350530[] = {{&S44630[12], 14}, {&S44630[40], 12}, {NULL}};
const Segment SGI350531[] = {{&S44630[12], 14}, {NULL}};
const Segment SGI350532[] = {{&S44631[6], 4}, {NULL}};
const Segment SGI350533[] = {{&S44631[82], 5}, {NULL}};
const Segment SGI350534[] = {{&S44631[12], 14}, {&S44631[40], 12}, {NULL}};
const Segment SGI350535[] = {{&S44631[12], 14}, {NULL}};
const Segment SGI350536[] = {{&S44632[6], 4}, {NULL}};
const Segment SGI350537[] = {{&S44632[82], 5}, {NULL}};
const Segment SGI350538[] = {{&S44632[12], 14}, {&S44632[41], 11}, {NULL}};
const Segment SGI350539[] = {{&S44632[12], 14}, {NULL}};
const Segment SGI350540[] = {{&S44634[6], 4}, {NULL}};
const Segment SGI350541[] = {{&S44634[81], 5}, {NULL}};
const Segment SGI350542[] = {{&S44634[12], 14}, {&S44634[40], 11}, {NULL}};
const Segment SGI350543[] = {{&S44634[12], 14}, {NULL}};
const Segment SGI350544[] = {{&S44635[6], 4}, {NULL}};
const Segment SGI350545[] = {{&S44635[81], 5}, {NULL}};
const Segment SGI350546[] = {{&S44635[12], 14}, {&S44635[40], 11}, {NULL}};
const Segment SGI350547[] = {{&S44635[12], 14}, {NULL}};
const Segment SGI350548[] = {{&S71109[5], 13}, {&S71109[105], 13}, {NULL}};
const Segment SGI350549[] = {{&S71109[169], 3}, {NULL}};
const Segment SGI350550[] = {{&S71110[35], 3}, {NULL}};
const Segment SGI350551[] = {{&S71110[89], 3}, {NULL}};
const Segment SGI350552[] = {{&S47047[35], 5}, {NULL}};
const Segment SGI350553[] = {{&S47047[13], 10}, {NULL}};
const Segment SGI350554[] = {{&S71132[43], 5}, {NULL}};
const Segment SGI350555[] = {{&S71132[6], 3}, {NULL}};
const Segment SGI350556[] = {{&S71132[69], 3}, {NULL}};
const Segment SGI350557[] = {{&S47050[57], 5}, {NULL}};
const Segment SGI350558[] = {{&S47050[13], 10}, {NULL}};
const Segment SGI350559[] = {{&S57448[5], 4}, {NULL}};
const Segment SGI350560[] = {{&S57448[49], 3}, {NULL}};
const Segment SGI350561[] = {{&S57449[5], 4}, {NULL}};
const Segment SGI350562[] = {{&S57449[33], 3}, {NULL}};
const Segment SGI350563[] = {{&S71152[5], 3}, {&S71152[5], 3}, {NULL}};
const Segment SGI350564[] = {{&S71152[9], 3}, {NULL}};
const Segment SGI350565[] = {{&S71152[127], 3}, {NULL}};
const Segment SGI350566[] = {{&S71159[5], 4}, {&S71159[5], 4}, {NULL}};
const Segment SGI350567[] = {{&S71159[10], 3}, {NULL}};
const Segment SGI350568[] = {{&S71159[129], 3}, {NULL}};
const Segment SGI350569[] = {{&S44735[6], 4}, {NULL}};
const Segment SGI350570[] = {{&S44735[12], 10}, {NULL}};
const Segment SGI350571[] = {{&S44737[6], 4}, {NULL}};
const Segment SGI350572[] = {{&S44737[12], 10}, {NULL}};
const Segment SGI350573[] = {{&S44739[6], 4}, {NULL}};
const Segment SGI350574[] = {{&S44739[12], 10}, {NULL}};
const Segment SGI350575[] = {{&S71200[5], 13}, {&S71200[105], 13}, {NULL}};
const Segment SGI350576[] = {{&S71200[129], 3}, {NULL}};
const Segment SGI350577[] = {{&S44781[6], 4}, {NULL}};
const Segment SGI350578[] = {{&S44781[70], 5}, {NULL}};
const Segment SGI350579[] = {{&S44781[12], 9}, {NULL}};
const Segment SGI350580[] = {{&S44781[29], 13}, {NULL}};
const Segment SGI350581[] = {{&S71256[42], 7}, {NULL}};
const Segment SGI350582[] = {{&S71256[6], 3}, {NULL}};
const Segment SGI350583[] = {{&S71256[70], 3}, {NULL}};
const Segment SGI350584[] = {{&S71257[43], 16}, {NULL}};
const Segment SGI350585[] = {{&S71257[6], 3}, {NULL}};
const Segment SGI350586[] = {{&S71257[80], 3}, {NULL}};
const Segment SGI350587[] = {{&S71299[5], 3}, {&S71299[5], 3}, {NULL}};
const Segment SGI350588[] = {{&S71299[80], 3}, {NULL}};
const Segment SGI350589[] = {{&S71330[42], 3}, {NULL}};
const Segment SGI350590[] = {{&S71330[6], 3}, {NULL}};
const Segment SGI350591[] = {{&S71330[66], 3}, {NULL}};
const Segment SGI350592[] = {{&S71337[5], 16}, {NULL}};
const Segment SGI350593[] = {{&S71337[31], 3}, {NULL}};
const Segment SGI350594[] = {{&S71338[5], 16}, {NULL}};
const Segment SGI350595[] = {{&S71391[5], 4}, {&S71391[5], 4}, {NULL}};
const Segment SGI350596[] = {{&S71391[68], 3}, {NULL}};
const Segment SGI350597[] = {{&S71391[128], 3}, {NULL}};
const Segment SGI350598[] = {{&S71399[42], 5}, {NULL}};
const Segment SGI350599[] = {{&S71399[6], 3}, {NULL}};
const Segment SGI350600[] = {{&S71399[68], 3}, {NULL}};
const Segment SGI350601[] = {{&S71400[43], 3}, {NULL}};
const Segment SGI350602[] = {{&S71400[6], 3}, {NULL}};
const Segment SGI350603[] = {{&S71400[67], 3}, {NULL}};
const Segment SGI350604[] = {{&S71406[5], 7}, {NULL}};
const Segment SGI350605[] = {{&S71406[23], 3}, {NULL}};
const Segment SGI350606[] = {{&S71407[5], 5}, {NULL}};
const Segment SGI350607[] = {{&S71407[21], 3}, {NULL}};
const Segment SGI350608[] = {{&S44969[6], 4}, {NULL}};
const Segment SGI350609[] = {{&S44969[59], 5}, {NULL}};
const Segment SGI350610[] = {{&S44969[12], 14}, {NULL}};
const Segment SGI350611[] = {{&S44970[6], 4}, {NULL}};
const Segment SGI350612[] = {{&S44970[52], 5}, {NULL}};
const Segment SGI350613[] = {{&S44970[12], 45}, {NULL}};
const Segment SGI350614[] = {{&S44970[12], 14}, {NULL}};
const Segment SGI350615[] = {{&S44971[6], 4}, {NULL}};
const Segment SGI350616[] = {{&S44971[75], 5}, {NULL}};
const Segment SGI350617[] = {{&S44971[12], 14}, {&S44971[37], 43}, {NULL}};
const Segment SGI350618[] = {{&S44971[12], 14}, {NULL}};
const Segment SGI350619[] = {{&S44972[6], 4}, {NULL}};
const Segment SGI350620[] = {{&S44972[52], 5}, {NULL}};
const Segment SGI350621[] = {{&S44972[12], 55}, {NULL}};
const Segment SGI350622[] = {{&S44972[12], 14}, {NULL}};
const Segment SGI350623[] = {{&S44973[6], 4}, {NULL}};
const Segment SGI350624[] = {{&S44973[74], 5}, {NULL}};
const Segment SGI350625[] = {{&S44973[12], 14}, {&S44973[37], 42}, {NULL}};
const Segment SGI350626[] = {{&S44973[12], 14}, {NULL}};
const Segment SGI350627[] = {{&S44974[6], 4}, {NULL}};
const Segment SGI350628[] = {{&S44974[74], 5}, {NULL}};
const Segment SGI350629[] = {{&S44974[12], 14}, {&S44974[37], 42}, {NULL}};
const Segment SGI350630[] = {{&S44974[12], 14}, {NULL}};
const Segment SGI350631[] = {{&S44975[6], 4}, {NULL}};
const Segment SGI350632[] = {{&S44975[75], 5}, {NULL}};
const Segment SGI350633[] = {{&S44975[12], 14}, {&S44975[38], 42}, {NULL}};
const Segment SGI350634[] = {{&S44975[12], 14}, {NULL}};
const Segment SGI350635[] = {{&S44976[6], 4}, {NULL}};
const Segment SGI350636[] = {{&S44976[74], 5}, {NULL}};
const Segment SGI350637[] = {{&S44976[12], 14}, {&S44976[37], 42}, {NULL}};
const Segment SGI350638[] = {{&S44976[12], 14}, {NULL}};
const Segment SGI350639[] = {{&S44977[6], 4}, {NULL}};
const Segment SGI350640[] = {{&S44977[74], 5}, {NULL}};
const Segment SGI350641[] = {{&S44977[12], 14}, {&S44977[37], 42}, {NULL}};
const Segment SGI350642[] = {{&S44977[12], 14}, {NULL}};
const Segment SGI350643[] = {{&S44978[6], 4}, {NULL}};
const Segment SGI350644[] = {{&S44978[75], 5}, {NULL}};
const Segment SGI350645[] = {{&S44978[12], 14}, {&S44978[37], 43}, {NULL}};
const Segment SGI350646[] = {{&S44978[12], 14}, {NULL}};
const Segment SGI350647[] = {{&S63115[28], 5}, {NULL}};
const Segment SGI350648[] = {{&S63115[104], 5}, {NULL}};
const Segment SGI350649[] = {{&S71525[5], 3}, {&S71525[5], 3}, {NULL}};
const Segment SGI350650[] = {{&S71525[9], 3}, {NULL}};
const Segment SGI350651[] = {{&S71525[127], 3}, {NULL}};
const Segment SGI350652[] = {{&S71553[5], 5}, {&S71553[68], 5}, {NULL}};
const Segment SGI350653[] = {{&S71553[84], 3}, {NULL}};
const Segment SGI350654[] = {{&S57680[46], 5}, {NULL}};
const Segment SGI350655[] = {{&S57680[37], 3}, {NULL}};
const Segment SGI350656[] = {{&S71592[35], 5}, {NULL}};
const Segment SGI350657[] = {{&S71592[91], 3}, {NULL}};
const Segment SGI350658[] = {{&S71594[5], 13}, {&S71594[113], 13}, {NULL}};
const Segment SGI350659[] = {{&S71594[19], 3}, {NULL}};
const Segment SGI350660[] = {{&S71594[147], 3}, {NULL}};
const Segment SGI350661[] = {{&S71631[42], 4}, {NULL}};
const Segment SGI350662[] = {{&S71631[6], 3}, {NULL}};
const Segment SGI350663[] = {{&S71631[67], 3}, {NULL}};
const Segment SGI350664[] = {{&S71670[5], 5}, {&S71670[68], 5}, {NULL}};
const Segment SGI350665[] = {{&S71670[84], 3}, {NULL}};
const Segment SGI350666[] = {{&S63285[28], 5}, {NULL}};
const Segment SGI350667[] = {{&S63285[104], 5}, {NULL}};
const Segment SGI350668[] = {{&S71682[5], 4}, {&S71682[5], 4}, {NULL}};
const Segment SGI350669[] = {{&S71682[111], 3}, {NULL}};
const Segment SGI350670[] = {{&S71713[35], 4}, {NULL}};
const Segment SGI350671[] = {{&S71713[90], 3}, {NULL}};
const Segment SGI350672[] = {{&S71714[5], 13}, {NULL}};
const Segment SGI350673[] = {{&S71714[19], 3}, {NULL}};
const Segment SGI350674[] = {{&S71715[41], 7}, {NULL}};
const Segment SGI350675[] = {{&S71715[6], 3}, {NULL}};
const Segment SGI350676[] = {{&S71715[69], 3}, {NULL}};
const Segment SGI350677[] = {{&S71795[35], 5}, {NULL}};
const Segment SGI350678[] = {{&S71795[52], 3}, {NULL}};
const Segment SGI350679[] = {{&S71796[5], 4}, {&S71796[5], 4}, {NULL}};
const Segment SGI350680[] = {{&S71796[111], 3}, {NULL}};
const Segment SGI350681[] = {{&S45330[6], 4}, {NULL}};
const Segment SGI350682[] = {{&S45330[12], 3}, {NULL}};
const Segment SGI350683[] = {{&S45331[6], 4}, {NULL}};
const Segment SGI350684[] = {{&S45331[12], 3}, {NULL}};
const Segment SGI350685[] = {{&S45332[6], 4}, {NULL}};
const Segment SGI350686[] = {{&S45332[12], 3}, {NULL}};
const Segment SGI350687[] = {{&S45333[6], 4}, {NULL}};
const Segment SGI350688[] = {{&S45333[12], 3}, {NULL}};
const Segment SGI350689[] = {{&S45334[6], 4}, {NULL}};
const Segment SGI350690[] = {{&S45334[12], 3}, {NULL}};
const Segment SGI350691[] = {{&S45335[6], 4}, {NULL}};
const Segment SGI350692[] = {{&S45335[12], 3}, {NULL}};
const Segment SGI350693[] = {{&S45336[6], 4}, {NULL}};
const Segment SGI350694[] = {{&S45336[12], 3}, {NULL}};
const Segment SGI350695[] = {{&S45337[6], 4}, {NULL}};
const Segment SGI350696[] = {{&S45337[12], 3}, {NULL}};
const Segment SGI350697[] = {{&S71825[5], 5}, {NULL}};
const Segment SGI350698[] = {{&S71826[5], 10}, {NULL}};
const Segment SGI350699[] = {{&S71832[43], 8}, {NULL}};
const Segment SGI350700[] = {{&S71832[6], 3}, {NULL}};
const Segment SGI350701[] = {{&S71832[72], 3}, {NULL}};
const Segment SGI350702[] = {{&S71833[5], 8}, {NULL}};
const Segment SGI350703[] = {{&S71833[19], 3}, {NULL}};
const Segment SGI350704[] = {{&S57845[49], 8}, {NULL}};
const Segment SGI350705[] = {{&S57845[40], 3}, {NULL}};
const Segment SGI350706[] = {{&S57846[49], 5}, {NULL}};
const Segment SGI350707[] = {{&S57846[40], 3}, {NULL}};
const Segment SGI350708[] = {{&S71971[5], 4}, {NULL}};
const Segment SGI350709[] = {{&S63570[28], 3}, {NULL}};
const Segment SGI350710[] = {{&S63570[102], 5}, {NULL}};
const Segment SGI350711[] = {{&S71986[49], 20}, {NULL}};
const Segment SGI350712[] = {{&S71987[49], 20}, {NULL}};
const Segment SGI350713[] = {{&S71988[49], 20}, {NULL}};
const Segment SGI350714[] = {{&S72004[5], 8}, {NULL}};
const Segment SGI350715[] = {{&S72004[19], 3}, {NULL}};
const Segment SGI350716[] = {{&S72005[35], 8}, {NULL}};
const Segment SGI350717[] = {{&S72005[94], 3}, {NULL}};
const Segment SGI350718[] = {{&S72021[5], 4}, {&S72021[5], 4}, {NULL}};
const Segment SGI350719[] = {{&S72021[10], 3}, {NULL}};
const Segment SGI350720[] = {{&S72021[129], 3}, {NULL}};
const Segment SGI350721[] = {{&S72022[42], 4}, {NULL}};
const Segment SGI350722[] = {{&S72022[6], 3}, {NULL}};
const Segment SGI350723[] = {{&S72022[67], 3}, {NULL}};
const Segment SGI350724[] = {{&S47949[61], 5}, {NULL}};
const Segment SGI350725[] = {{&S47949[13], 10}, {NULL}};
const Segment SGI350726[] = {{&S72116[5], 5}, {&S72116[68], 5}, {NULL}};
const Segment SGI350727[] = {{&S72116[84], 3}, {NULL}};
const Segment SGI350728[] = {{&S45481[6], 4}, {NULL}};
const Segment SGI350729[] = {{&S45481[52], 5}, {NULL}};
const Segment SGI350730[] = {{&S45481[12], 45}, {NULL}};
const Segment SGI350731[] = {{&S45481[12], 14}, {NULL}};
const Segment SGI350732[] = {{&S45482[6], 4}, {NULL}};
const Segment SGI350733[] = {{&S45482[59], 5}, {NULL}};
const Segment SGI350734[] = {{&S45482[12], 14}, {NULL}};
const Segment SGI350735[] = {{&S45483[6], 4}, {NULL}};
const Segment SGI350736[] = {{&S45483[81], 5}, {NULL}};
const Segment SGI350737[] = {{&S45483[12], 14}, {&S45483[44], 42}, {NULL}};
const Segment SGI350738[] = {{&S45483[12], 14}, {NULL}};
const Segment SGI350739[] = {{&S45484[6], 4}, {NULL}};
const Segment SGI350740[] = {{&S45484[81], 5}, {NULL}};
const Segment SGI350741[] = {{&S45484[12], 14}, {&S45484[44], 42}, {NULL}};
const Segment SGI350742[] = {{&S45484[12], 14}, {NULL}};
const Segment SGI350743[] = {{&S45485[6], 4}, {NULL}};
const Segment SGI350744[] = {{&S45485[81], 5}, {NULL}};
const Segment SGI350745[] = {{&S45485[12], 14}, {&S45485[44], 42}, {NULL}};
const Segment SGI350746[] = {{&S45485[12], 14}, {NULL}};
const Segment SGI350747[] = {{&S45486[6], 4}, {NULL}};
const Segment SGI350748[] = {{&S45486[81], 5}, {NULL}};
const Segment SGI350749[] = {{&S45486[12], 14}, {&S45486[44], 42}, {NULL}};
const Segment SGI350750[] = {{&S45486[12], 14}, {NULL}};
const Segment SGI350751[] = {{&S72136[42], 5}, {NULL}};
const Segment SGI350752[] = {{&S72136[6], 3}, {NULL}};
const Segment SGI350753[] = {{&S72136[68], 3}, {NULL}};
const Segment SGI350754[] = {{&S72137[43], 3}, {NULL}};
const Segment SGI350755[] = {{&S72137[6], 3}, {NULL}};
const Segment SGI350756[] = {{&S72137[67], 3}, {NULL}};
const Segment SGI350757[] = {{&S45518[6], 4}, {NULL}};
const Segment SGI350758[] = {{&S45518[70], 5}, {NULL}};
const Segment SGI350759[] = {{&S45518[12], 9}, {NULL}};
const Segment SGI350760[] = {{&S45518[29], 13}, {NULL}};
const Segment SGI350761[] = {{&S72160[21], 5}, {NULL}};
const Segment SGI350762[] = {{&S72160[0], 14}, {NULL}};
const Segment SGI350763[] = {{&S48049[6], 3}, {NULL}};
const Segment SGI350764[] = {{&S48049[11], 14}, {NULL}};
const Segment SGI350765[] = {{&S72184[5], 3}, {&S72184[5], 3}, {NULL}};
const Segment SGI350766[] = {{&S72184[149], 3}, {NULL}};
const Segment SGI350767[] = {{&S48064[58], 5}, {NULL}};
const Segment SGI350768[] = {{&S48064[13], 10}, {NULL}};
const Segment SGI350769[] = {{&S63789[28], 3}, {NULL}};
const Segment SGI350770[] = {{&S63789[102], 5}, {NULL}};
const Segment SGI350771[] = {{&S63793[28], 7}, {NULL}};
const Segment SGI350772[] = {{&S63793[106], 5}, {NULL}};
const Segment SGI350773[] = {{&S72207[5], 4}, {NULL}};
const Segment SGI350774[] = {{&S63802[28], 3}, {NULL}};
const Segment SGI350775[] = {{&S63802[102], 5}, {NULL}};
const Segment SGI350776[] = {{&S72214[5], 7}, {NULL}};
const Segment SGI350777[] = {{&S63803[28], 5}, {NULL}};
const Segment SGI350778[] = {{&S63803[104], 5}, {NULL}};
const Segment SGI350779[] = {{&S72215[101], 5}, {NULL}};
const Segment SGI350780[] = {{&S72216[5], 5}, {&S72216[97], 5}, {NULL}};
const Segment SGI350781[] = {{&S72216[113], 3}, {NULL}};
const Segment SGI350782[] = {{&S72223[18], 4}, {NULL}};
const Segment SGI350783[] = {{&S72243[5], 3}, {&S72243[5], 3}, {NULL}};
const Segment SGI350784[] = {{&S72243[67], 3}, {NULL}};
const Segment SGI350785[] = {{&S72243[127], 3}, {NULL}};
const Segment SGI350786[] = {{&S72252[35], 3}, {NULL}};
const Segment SGI350787[] = {{&S72252[50], 3}, {NULL}};
const Segment SGI350788[] = {{&S72253[35], 7}, {NULL}};
const Segment SGI350789[] = {{&S72253[54], 3}, {NULL}};
const Segment SGI350790[] = {{&S72254[31], 7}, {NULL}};
const Segment SGI350791[] = {{&S72254[89], 3}, {NULL}};
const Segment SGI350792[] = {{&S72264[5], 3}, {&S72264[5], 3}, {NULL}};
const Segment SGI350793[] = {{&S72264[9], 3}, {NULL}};
const Segment SGI350794[] = {{&S72264[127], 3}, {NULL}};
const Segment SGI350795[] = {{&S72265[43], 4}, {NULL}};
const Segment SGI350796[] = {{&S72265[6], 3}, {NULL}};
const Segment SGI350797[] = {{&S72265[68], 3}, {NULL}};
const Segment SGI350798[] = {{&S72351[42], 3}, {NULL}};
const Segment SGI350799[] = {{&S72351[6], 3}, {NULL}};
const Segment SGI350800[] = {{&S72351[66], 3}, {NULL}};
const Segment SGI350801[] = {{&S72382[5], 5}, {&S72382[97], 5}, {NULL}};
const Segment SGI350802[] = {{&S72382[113], 3}, {NULL}};
const Segment SGI350803[] = {{&S72401[5], 4}, {NULL}};
const Segment SGI350804[] = {{&S45773[6], 4}, {NULL}};
const Segment SGI350805[] = {{&S45773[77], 5}, {NULL}};
const Segment SGI350806[] = {{&S45773[12], 9}, {NULL}};
const Segment SGI350807[] = {{&S45773[29], 13}, {NULL}};
const Segment SGI350808[] = {{&S72420[35], 4}, {NULL}};
const Segment SGI350809[] = {{&S72420[51], 3}, {NULL}};
const Segment SGI350810[] = {{&S72421[5], 4}, {NULL}};
const Segment SGI350811[] = {{&S72421[15], 3}, {NULL}};
const Segment SGI350812[] = {{&S72437[5], 7}, {&S72437[5], 7}, {NULL}};
const Segment SGI350813[] = {{&S72437[13], 3}, {NULL}};
const Segment SGI350814[] = {{&S72437[135], 3}, {NULL}};
const Segment SGI350815[] = {{&S45821[6], 4}, {NULL}};
const Segment SGI350816[] = {{&S45821[12], 14}, {NULL}};
const Segment SGI350817[] = {{&S45822[6], 4}, {NULL}};
const Segment SGI350818[] = {{&S45822[12], 14}, {NULL}};
const Segment SGI350819[] = {{&S45823[6], 4}, {NULL}};
const Segment SGI350820[] = {{&S45823[12], 14}, {NULL}};
const Segment SGI350821[] = {{&S45825[6], 3}, {NULL}};
const Segment SGI350822[] = {{&S45825[11], 14}, {NULL}};
const Segment SGI350823[] = {{&S72473[5], 5}, {NULL}};
const Segment SGI350824[] = {{&S72474[5], 3}, {NULL}};
const Segment SGI350825[] = {{&S64080[28], 3}, {NULL}};
const Segment SGI350826[] = {{&S64080[102], 5}, {NULL}};
const Segment SGI350827[] = {{&S45845[0], 16}, {NULL}};
const Segment SGI350828[] = {{&S45845[29], 3}, {NULL}};
const Segment SGI350829[] = {{&S45845[71], 16}, {NULL}};
const Segment SGI350830[] = {{&S72498[5], 7}, {&S72498[5], 7}, {NULL}};
const Segment SGI350831[] = {{&S72498[13], 3}, {NULL}};
const Segment SGI350832[] = {{&S72498[135], 3}, {NULL}};
const Segment SGI350833[] = {{&S72507[5], 5}, {&S72507[5], 5}, {NULL}};
const Segment SGI350834[] = {{&S72507[113], 3}, {NULL}};
const Segment SGI350835[] = {{&S72532[5], 5}, {NULL}};
const Segment SGI350836[] = {{&S72532[21], 3}, {NULL}};
const Segment SGI350837[] = {{&S72536[35], 3}, {NULL}};
const Segment SGI350838[] = {{&S72536[50], 3}, {NULL}};
const Segment SGI350839[] = {{&S72540[34], 5}, {NULL}};
const Segment SGI350840[] = {{&S72540[50], 3}, {NULL}};
const Segment SGI350841[] = {{&S45915[6], 4}, {NULL}};
const Segment SGI350842[] = {{&S45915[62], 5}, {NULL}};
const Segment SGI350843[] = {{&S45915[12], 3}, {NULL}};
const Segment SGI350844[] = {{&S72559[5], 4}, {NULL}};
const Segment SGI350845[] = {{&S72581[5], 7}, {&S72581[5], 7}, {NULL}};
const Segment SGI350846[] = {{&S72581[117], 3}, {NULL}};
const Segment SGI350847[] = {{&S45948[6], 4}, {NULL}};
const Segment SGI350848[] = {{&S45948[12], 14}, {NULL}};
const Segment SGI350849[] = {{&S45949[6], 4}, {NULL}};
const Segment SGI350850[] = {{&S45949[12], 14}, {NULL}};
const Segment SGI350851[] = {{&S45951[6], 4}, {NULL}};
const Segment SGI350852[] = {{&S45951[12], 14}, {NULL}};
const Segment SGI350853[] = {{&S72635[5], 5}, {&S72635[105], 5}, {NULL}};
const Segment SGI350854[] = {{&S72635[69], 3}, {NULL}};
const Segment SGI350855[] = {{&S72635[131], 3}, {NULL}};
const Segment SGI350856[] = {{&S72636[5], 7}, {&S72636[107], 7}, {NULL}};
const Segment SGI350857[] = {{&S72636[71], 3}, {NULL}};
const Segment SGI350858[] = {{&S72636[135], 3}, {NULL}};
const Segment SGI350859[] = {{&S72639[5], 3}, {&S72639[5], 3}, {NULL}};
const Segment SGI350860[] = {{&S72639[67], 3}, {NULL}};
const Segment SGI350861[] = {{&S72639[127], 3}, {NULL}};
const Segment SGI350862[] = {{&S72664[5], 13}, {&S72664[112], 13}, {NULL}};
const Segment SGI350863[] = {{&S72664[19], 3}, {NULL}};
const Segment SGI350864[] = {{&S72664[146], 3}, {NULL}};
const Segment SGI350865[] = {{&S72672[5], 3}, {&S72672[5], 3}, {NULL}};
const Segment SGI350866[] = {{&S72672[9], 3}, {NULL}};
const Segment SGI350867[] = {{&S72672[127], 3}, {NULL}};
const Segment SGI350868[] = {{&S72675[42], 5}, {NULL}};
const Segment SGI350869[] = {{&S72675[6], 3}, {NULL}};
const Segment SGI350870[] = {{&S72675[68], 3}, {NULL}};
const Segment SGI350871[] = {{&S72676[5], 5}, {NULL}};
const Segment SGI350872[] = {{&S72676[16], 3}, {NULL}};
const Segment SGI350873[] = {{&S72686[5], 7}, {NULL}};
const Segment SGI350874[] = {{&S72686[23], 3}, {NULL}};
const Segment SGI350875[] = {{&S72687[5], 3}, {NULL}};
const Segment SGI350876[] = {{&S72687[19], 3}, {NULL}};
const Segment SGI350877[] = {{&S72779[35], 3}, {NULL}};
const Segment SGI350878[] = {{&S72779[89], 3}, {NULL}};
const Segment SGI350879[] = {{&S72780[34], 5}, {NULL}};
const Segment SGI350880[] = {{&S72780[90], 3}, {NULL}};
const Segment SGI350881[] = {{&S72785[35], 4}, {NULL}};
const Segment SGI350882[] = {{&S72785[90], 3}, {NULL}};
const Segment SGI350883[] = {{&S72786[5], 13}, {NULL}};
const Segment SGI350884[] = {{&S72786[19], 3}, {NULL}};
const Segment SGI350885[] = {{&S72801[5], 3}, {&S72801[5], 3}, {NULL}};
const Segment SGI350886[] = {{&S72801[9], 3}, {NULL}};
const Segment SGI350887[] = {{&S72801[127], 3}, {NULL}};
const Segment SGI350888[] = {{&S72802[5], 5}, {&S72802[5], 5}, {NULL}};
const Segment SGI350889[] = {{&S72802[11], 3}, {NULL}};
const Segment SGI350890[] = {{&S72802[131], 3}, {NULL}};
const Segment SGI350891[] = {{&S72824[5], 5}, {&S72824[104], 5}, {NULL}};
const Segment SGI350892[] = {{&S72824[11], 3}, {NULL}};
const Segment SGI350893[] = {{&S72824[130], 3}, {NULL}};
const Segment SGI350894[] = {{&S72867[5], 3}, {NULL}};
const Segment SGI350895[] = {{&S72868[5], 5}, {NULL}};
const Segment SGI350896[] = {{&S72869[42], 5}, {NULL}};
const Segment SGI350897[] = {{&S72869[6], 3}, {NULL}};
const Segment SGI350898[] = {{&S72869[68], 3}, {NULL}};
const Segment SGI350899[] = {{&S72870[43], 5}, {NULL}};
const Segment SGI350900[] = {{&S72870[6], 3}, {NULL}};
const Segment SGI350901[] = {{&S72870[69], 3}, {NULL}};
const Segment SGI350902[] = {{&S72871[5], 5}, {NULL}};
const Segment SGI350903[] = {{&S72871[16], 3}, {NULL}};
const Segment SGI350904[] = {{&S58626[5], 13}, {&S58626[5], 13}, {NULL}};
const Segment SGI350905[] = {{&S58626[52], 3}, {NULL}};
const Segment SGI350906[] = {{&S58627[49], 13}, {NULL}};
const Segment SGI350907[] = {{&S58627[40], 3}, {NULL}};
const Segment SGI350908[] = {{&S58628[49], 4}, {NULL}};
const Segment SGI350909[] = {{&S58628[40], 3}, {NULL}};
const Segment SGI350910[] = {{&S72936[42], 5}, {NULL}};
const Segment SGI350911[] = {{&S72936[6], 3}, {NULL}};
const Segment SGI350912[] = {{&S72936[68], 3}, {NULL}};
const Segment SGI350913[] = {{&S72937[43], 5}, {NULL}};
const Segment SGI350914[] = {{&S72937[6], 3}, {NULL}};
const Segment SGI350915[] = {{&S72937[69], 3}, {NULL}};
const Segment SGI350916[] = {{&S72946[5], 5}, {&S72946[68], 5}, {NULL}};
const Segment SGI350917[] = {{&S72946[84], 3}, {NULL}};
const Segment SGI350918[] = {{&S46347[6], 4}, {NULL}};
const Segment SGI350919[] = {{&S46347[53], 5}, {NULL}};
const Segment SGI350920[] = {{&S46347[12], 3}, {NULL}};
const Segment SGI350921[] = {{&S46350[6], 4}, {NULL}};
const Segment SGI350922[] = {{&S46350[107], 5}, {NULL}};
const Segment SGI350923[] = {{&S46350[12], 3}, {&S46350[44], 10}, {NULL}};
const Segment SGI350924[] = {{&S46350[66], 14}, {NULL}};
const Segment SGI350925[] = {{&S46351[6], 4}, {NULL}};
const Segment SGI350926[] = {{&S46351[53], 5}, {NULL}};
const Segment SGI350927[] = {{&S46351[12], 3}, {&S46351[64], 4}, {NULL}};
const Segment SGI350928[] = {{&S73004[35], 3}, {NULL}};
const Segment SGI350929[] = {{&S73004[89], 3}, {NULL}};
const Segment SGI350930[] = {{&S73035[5], 3}, {&S73035[5], 3}, {NULL}};
const Segment SGI350931[] = {{&S73035[9], 3}, {NULL}};
const Segment SGI350932[] = {{&S73035[126], 3}, {NULL}};
const Segment SGI350933[] = {{&S73036[5], 5}, {&S73036[5], 5}, {NULL}};
const Segment SGI350934[] = {{&S73036[11], 3}, {NULL}};
const Segment SGI350935[] = {{&S73036[130], 3}, {NULL}};
const Segment SGI350936[] = {{&S73067[5], 3}, {&S73067[5], 3}, {NULL}};
const Segment SGI350937[] = {{&S73067[9], 3}, {NULL}};
const Segment SGI350938[] = {{&S73067[127], 3}, {NULL}};
const Segment SGI350939[] = {{&S46491[0], 21}, {NULL}};
const Segment SGI350940[] = {{&S46491[34], 3}, {NULL}};
const Segment SGI350941[] = {{&S46491[73], 13}, {NULL}};
const Segment SGI350942[] = {{&S46492[0], 21}, {NULL}};
const Segment SGI350943[] = {{&S46492[34], 3}, {NULL}};
const Segment SGI350944[] = {{&S46492[75], 13}, {NULL}};
const Segment SGI350945[] = {{&S73118[5], 5}, {&S73118[104], 5}, {NULL}};
const Segment SGI350946[] = {{&S73118[69], 3}, {NULL}};
const Segment SGI350947[] = {{&S73118[130], 3}, {NULL}};
const Segment SGI350948[] = {{&S73130[43], 3}, {NULL}};
const Segment SGI350949[] = {{&S73130[6], 3}, {NULL}};
const Segment SGI350950[] = {{&S73130[67], 3}, {NULL}};
const Segment SGI350951[] = {{&S73131[5], 5}, {&S73131[105], 5}, {NULL}};
const Segment SGI350952[] = {{&S73131[69], 3}, {NULL}};
const Segment SGI350953[] = {{&S73131[131], 3}, {NULL}};
const Segment SGI350954[] = {{&S73132[5], 7}, {&S73132[107], 7}, {NULL}};
const Segment SGI350955[] = {{&S73132[71], 3}, {NULL}};
const Segment SGI350956[] = {{&S73132[135], 3}, {NULL}};
const Segment SGI350957[] = {{&S73143[5], 5}, {NULL}};
const Segment SGI350958[] = {{&S73152[5], 4}, {&S73152[5], 4}, {NULL}};
const Segment SGI350959[] = {{&S73152[151], 3}, {NULL}};
const Segment SGI350960[] = {{&S73174[5], 5}, {NULL}};
const Segment SGI350961[] = {{&S73201[42], 4}, {NULL}};
const Segment SGI350962[] = {{&S73201[6], 3}, {NULL}};
const Segment SGI350963[] = {{&S73201[67], 3}, {NULL}};
const Segment SGI350964[] = {{&S73220[5], 3}, {NULL}};
const Segment SGI350965[] = {{&S73220[19], 3}, {NULL}};
const Segment SGI350966[] = {{&S73221[5], 3}, {NULL}};
const Segment SGI350967[] = {{&S73221[14], 3}, {NULL}};
const Segment SGI350968[] = {{&S73255[5], 3}, {&S73255[5], 3}, {NULL}};
const Segment SGI350969[] = {{&S73255[9], 3}, {NULL}};
const Segment SGI350970[] = {{&S73255[126], 3}, {NULL}};
const Segment SGI350971[] = {{&S73268[5], 4}, {&S73268[5], 4}, {NULL}};
const Segment SGI350972[] = {{&S73268[82], 3}, {NULL}};
const Segment SGI350973[] = {{&S73269[5], 4}, {NULL}};
const Segment SGI350974[] = {{&S73269[92], 3}, {NULL}};
const Segment SGI350975[] = {{&S73330[5], 5}, {&S73330[105], 5}, {NULL}};
const Segment SGI350976[] = {{&S73330[11], 3}, {NULL}};
const Segment SGI350977[] = {{&S73330[131], 3}, {NULL}};
const Segment SGI350978[] = {{&S73359[45], 11}, {NULL}};
const Segment SGI350979[] = {{&S73359[6], 3}, {NULL}};
const Segment SGI350980[] = {{&S73359[77], 3}, {NULL}};
const Segment SGI350981[] = {{&S73362[5], 3}, {&S73362[5], 3}, {NULL}};
const Segment SGI350982[] = {{&S73362[67], 3}, {NULL}};
const Segment SGI350983[] = {{&S73362[127], 3}, {NULL}};
const Segment SGI350984[] = {{&S73371[5], 4}, {NULL}};
const Segment SGI350985[] = {{&S73388[42], 4}, {NULL}};
const Segment SGI350986[] = {{&S73388[6], 3}, {NULL}};
const Segment SGI350987[] = {{&S73388[67], 3}, {NULL}};
const Segment SGI350988[] = {{&S73421[5], 4}, {&S73421[5], 4}, {NULL}};
const Segment SGI350989[] = {{&S73421[10], 3}, {NULL}};
const Segment SGI350990[] = {{&S73421[129], 3}, {NULL}};
const Segment SGI350991[] = {{&S73422[42], 4}, {NULL}};
const Segment SGI350992[] = {{&S73422[6], 3}, {NULL}};
const Segment SGI350993[] = {{&S73422[67], 3}, {NULL}};
const Segment SGI350994[] = {{&S73423[43], 3}, {NULL}};
const Segment SGI350995[] = {{&S73423[6], 3}, {NULL}};
const Segment SGI350996[] = {{&S73423[67], 3}, {NULL}};
const Segment SGI350997[] = {{&S73436[35], 4}, {NULL}};
const Segment SGI350998[] = {{&S73436[90], 3}, {NULL}};
const Segment SGI350999[] = {{&S64988[28], 5}, {NULL}};
const Segment SGI351000[] = {{&S64988[104], 5}, {NULL}};
const Segment SGI351001[] = {{&S73439[35], 4}, {NULL}};
const Segment SGI351002[] = {{&S73439[90], 3}, {NULL}};
const Segment SGI351003[] = {{&S73440[5], 13}, {NULL}};
const Segment SGI351004[] = {{&S73440[19], 3}, {NULL}};
const Segment SGI351005[] = {{&S73441[41], 5}, {NULL}};
const Segment SGI351006[] = {{&S73441[6], 3}, {NULL}};
const Segment SGI351007[] = {{&S73441[67], 3}, {NULL}};
const Segment SGI351008[] = {{&S46761[6], 4}, {NULL}};
const Segment SGI351009[] = {{&S46761[73], 5}, {NULL}};
const Segment SGI351010[] = {{&S46761[12], 14}, {NULL}};
const Segment SGI351011[] = {{&S73467[35], 4}, {NULL}};
const Segment SGI351012[] = {{&S73467[51], 3}, {NULL}};
const Segment SGI351013[] = {{&S73468[5], 4}, {NULL}};
const Segment SGI351014[] = {{&S73468[15], 3}, {NULL}};
const Segment SGI351015[] = {{&S73484[35], 5}, {NULL}};
const Segment SGI351016[] = {{&S73484[91], 3}, {NULL}};
const Segment SGI351017[] = {{&S73485[38], 3}, {NULL}};
const Segment SGI351018[] = {{&S73485[92], 3}, {NULL}};
const Segment SGI351019[] = {{&S65161[28], 5}, {NULL}};
const Segment SGI351020[] = {{&S65161[104], 5}, {NULL}};
const Segment SGI351021[] = {{&S65162[28], 7}, {NULL}};
const Segment SGI351022[] = {{&S65162[106], 5}, {NULL}};
const Segment SGI351023[] = {{&S73517[5], 3}, {&S73517[5], 3}, {NULL}};
const Segment SGI351024[] = {{&S73517[67], 3}, {NULL}};
const Segment SGI351025[] = {{&S73517[126], 3}, {NULL}};
const Segment SGI351026[] = {{&S73605[5], 4}, {NULL}};
const Segment SGI351027[] = {{&S73605[20], 3}, {NULL}};
const Segment SGI351028[] = {{&S47048[6], 4}, {NULL}};
const Segment SGI351029[] = {{&S47048[53], 5}, {NULL}};
const Segment SGI351030[] = {{&S47048[12], 10}, {NULL}};
const Segment SGI351031[] = {{&S47049[6], 4}, {NULL}};
const Segment SGI351032[] = {{&S47049[53], 5}, {NULL}};
const Segment SGI351033[] = {{&S47049[12], 10}, {NULL}};
const Segment SGI351034[] = {{&S73618[5], 11}, {&S73618[5], 11}, {NULL}};
const Segment SGI351035[] = {{&S73618[17], 3}, {NULL}};
const Segment SGI351036[] = {{&S73618[143], 3}, {NULL}};
const Segment SGI351037[] = {{&S73622[5], 4}, {&S73622[5], 4}, {NULL}};
const Segment SGI351038[] = {{&S73622[10], 3}, {NULL}};
const Segment SGI351039[] = {{&S73622[129], 3}, {NULL}};
const Segment SGI351040[] = {{&S73636[5], 5}, {&S73636[68], 5}, {NULL}};
const Segment SGI351041[] = {{&S73636[84], 3}, {NULL}};
const Segment SGI351042[] = {{&S73643[38], 3}, {NULL}};
const Segment SGI351043[] = {{&S73643[53], 3}, {NULL}};
const Segment SGI351044[] = {{&S73644[34], 3}, {NULL}};
const Segment SGI351045[] = {{&S73644[88], 3}, {NULL}};
const Segment SGI351046[] = {{&S47099[6], 4}, {NULL}};
const Segment SGI351047[] = {{&S47099[70], 5}, {NULL}};
const Segment SGI351048[] = {{&S47099[12], 9}, {NULL}};
const Segment SGI351049[] = {{&S47099[29], 13}, {NULL}};
const Segment SGI351050[] = {{&S65414[28], 3}, {NULL}};
const Segment SGI351051[] = {{&S65414[102], 5}, {NULL}};
const Segment SGI351052[] = {{&S73718[35], 5}, {NULL}};
const Segment SGI351053[] = {{&S73718[91], 3}, {NULL}};
const Segment SGI351054[] = {{&S73727[5], 15}, {&S73727[115], 15}, {NULL}};
const Segment SGI351055[] = {{&S73727[21], 3}, {NULL}};
const Segment SGI351056[] = {{&S73727[151], 3}, {NULL}};
const Segment SGI351057[] = {{&S65462[28], 3}, {NULL}};
const Segment SGI351058[] = {{&S65462[102], 5}, {NULL}};
const Segment SGI351059[] = {{&S59449[46], 7}, {NULL}};
const Segment SGI351060[] = {{&S59449[37], 3}, {NULL}};
const Segment SGI351061[] = {{&S59450[46], 5}, {NULL}};
const Segment SGI351062[] = {{&S59450[37], 3}, {NULL}};
const Segment SGI351063[] = {{&S65602[28], 5}, {NULL}};
const Segment SGI351064[] = {{&S65602[104], 5}, {NULL}};
const Segment SGI351065[] = {{&S65613[28], 3}, {NULL}};
const Segment SGI351066[] = {{&S65613[102], 5}, {NULL}};
const Segment SGI351067[] = {{&S73894[42], 5}, {NULL}};
const Segment SGI351068[] = {{&S73894[6], 3}, {NULL}};
const Segment SGI351069[] = {{&S73894[68], 3}, {NULL}};
const Segment SGI351070[] = {{&S73895[43], 3}, {NULL}};
const Segment SGI351071[] = {{&S73895[6], 3}, {NULL}};
const Segment SGI351072[] = {{&S73895[67], 3}, {NULL}};
const Segment SGI351073[] = {{&S73896[42], 5}, {NULL}};
const Segment SGI351074[] = {{&S73896[6], 3}, {NULL}};
const Segment SGI351075[] = {{&S73896[68], 3}, {NULL}};
const Segment SGI351076[] = {{&S73897[43], 5}, {NULL}};
const Segment SGI351077[] = {{&S73897[6], 3}, {NULL}};
const Segment SGI351078[] = {{&S73897[69], 3}, {NULL}};
const Segment SGI351079[] = {{&S73929[45], 11}, {NULL}};
const Segment SGI351080[] = {{&S73929[6], 3}, {NULL}};
const Segment SGI351081[] = {{&S73929[77], 3}, {NULL}};
const Segment SGI351082[] = {{&S65881[28], 3}, {NULL}};
const Segment SGI351083[] = {{&S65881[102], 5}, {NULL}};
const Segment SGI351084[] = {{&S47663[6], 4}, {NULL}};
const Segment SGI351085[] = {{&S47663[70], 5}, {NULL}};
const Segment SGI351086[] = {{&S47663[12], 9}, {NULL}};
const Segment SGI351087[] = {{&S47663[29], 13}, {NULL}};
const Segment SGI351088[] = {{&S47664[6], 4}, {NULL}};
const Segment SGI351089[] = {{&S47664[66], 5}, {NULL}};
const Segment SGI351090[] = {{&S47664[12], 14}, {NULL}};
const Segment SGI351091[] = {{&S47665[6], 4}, {NULL}};
const Segment SGI351092[] = {{&S47665[66], 5}, {NULL}};
const Segment SGI351093[] = {{&S47665[12], 14}, {NULL}};
const Segment SGI351094[] = {{&S73950[42], 5}, {NULL}};
const Segment SGI351095[] = {{&S73950[6], 3}, {NULL}};
const Segment SGI351096[] = {{&S73950[68], 3}, {NULL}};
const Segment SGI351097[] = {{&S73951[42], 5}, {NULL}};
const Segment SGI351098[] = {{&S73951[6], 3}, {NULL}};
const Segment SGI351099[] = {{&S73951[68], 3}, {NULL}};
const Segment SGI351100[] = {{&S73962[45], 11}, {NULL}};
const Segment SGI351101[] = {{&S73962[6], 3}, {NULL}};
const Segment SGI351102[] = {{&S73962[77], 3}, {NULL}};
const Segment SGI351103[] = {{&S73972[5], 7}, {&S73972[5], 7}, {NULL}};
const Segment SGI351104[] = {{&S73972[88], 3}, {NULL}};
const Segment SGI351105[] = {{&S73973[5], 7}, {NULL}};
const Segment SGI351106[] = {{&S73973[238], 3}, {NULL}};
const Segment SGI351107[] = {{&S44088[84], 5}, {NULL}};
const Segment SGI351108[] = {{&S44088[12], 14}, {NULL}};
const Segment SGI351109[] = {{&S44089[97], 5}, {NULL}};
const Segment SGI351110[] = {{&S44089[13], 10}, {NULL}};
const Segment SGI351111[] = {{&S50097[61], 5}, {NULL}};
const Segment SGI351112[] = {{&S50097[13], 10}, {NULL}};
const Segment SGI351113[] = {{&S74048[5], 5}, {NULL}};
const Segment SGI351114[] = {{&S74049[5], 5}, {NULL}};
const Segment SGI351115[] = {{&S74050[5], 4}, {NULL}};
const Segment SGI351116[] = {{&S47788[0], 21}, {NULL}};
const Segment SGI351117[] = {{&S47788[34], 3}, {NULL}};
const Segment SGI351118[] = {{&S74078[5], 5}, {&S74078[97], 5}, {NULL}};
const Segment SGI351119[] = {{&S74078[113], 3}, {NULL}};
const Segment SGI351120[] = {{&S74096[43], 4}, {NULL}};
const Segment SGI351121[] = {{&S74096[6], 3}, {NULL}};
const Segment SGI351122[] = {{&S74096[68], 3}, {NULL}};
const Segment SGI351123[] = {{&S74168[38], 5}, {NULL}};
const Segment SGI351124[] = {{&S74168[55], 3}, {NULL}};
const Segment SGI351125[] = {{&S74169[5], 5}, {NULL}};
const Segment SGI351126[] = {{&S74169[16], 3}, {NULL}};
const Segment SGI351127[] = {{&S74170[34], 5}, {NULL}};
const Segment SGI351128[] = {{&S74170[90], 3}, {NULL}};
const Segment SGI351129[] = {{&S74237[18], 11}, {NULL}};
const Segment SGI351130[] = {{&S66133[28], 5}, {NULL}};
const Segment SGI351131[] = {{&S66133[111], 5}, {NULL}};
const Segment SGI351132[] = {{&S66163[28], 3}, {NULL}};
const Segment SGI351133[] = {{&S66163[102], 5}, {NULL}};
const Segment SGI351134[] = {{&S66164[28], 5}, {NULL}};
const Segment SGI351135[] = {{&S66164[104], 5}, {NULL}};
const Segment SGI351136[] = {{&S74301[5], 5}, {&S74301[97], 5}, {NULL}};
const Segment SGI351137[] = {{&S74301[153], 3}, {NULL}};
const Segment SGI351138[] = {{&S74303[5], 11}, {&S74303[5], 11}, {NULL}};
const Segment SGI351139[] = {{&S74303[17], 3}, {NULL}};
const Segment SGI351140[] = {{&S74303[143], 3}, {NULL}};
const Segment SGI351141[] = {{&S74328[5], 5}, {&S74328[105], 5}, {NULL}};
const Segment SGI351142[] = {{&S74328[11], 3}, {NULL}};
const Segment SGI351143[] = {{&S74328[131], 3}, {NULL}};
const Segment SGI351144[] = {{&S44426[73], 5}, {NULL}};
const Segment SGI351145[] = {{&S44426[12], 3}, {NULL}};
const Segment SGI351146[] = {{&S47893[6], 4}, {NULL}};
const Segment SGI351147[] = {{&S47893[58], 5}, {NULL}};
const Segment SGI351148[] = {{&S47893[12], 3}, {NULL}};
const Segment SGI351149[] = {{&S47894[6], 4}, {NULL}};
const Segment SGI351150[] = {{&S47894[58], 5}, {NULL}};
const Segment SGI351151[] = {{&S47894[12], 3}, {NULL}};
const Segment SGI351152[] = {{&S47895[6], 4}, {NULL}};
const Segment SGI351153[] = {{&S47895[58], 5}, {NULL}};
const Segment SGI351154[] = {{&S47895[12], 3}, {NULL}};
const Segment SGI351155[] = {{&S47896[6], 4}, {NULL}};
const Segment SGI351156[] = {{&S47896[61], 5}, {NULL}};
const Segment SGI351157[] = {{&S47896[12], 3}, {NULL}};
const Segment SGI351158[] = {{&S47897[6], 5}, {NULL}};
const Segment SGI351159[] = {{&S47897[13], 13}, {NULL}};
const Segment SGI351160[] = {{&S47898[6], 4}, {NULL}};
const Segment SGI351161[] = {{&S47898[82], 5}, {NULL}};
const Segment SGI351162[] = {{&S47898[12], 13}, {NULL}};
const Segment SGI351163[] = {{&S47899[6], 4}, {NULL}};
const Segment SGI351164[] = {{&S47899[81], 5}, {NULL}};
const Segment SGI351165[] = {{&S47899[12], 13}, {NULL}};
const Segment SGI351166[] = {{&S66242[28], 7}, {NULL}};
const Segment SGI351167[] = {{&S66242[106], 5}, {NULL}};
const Segment SGI351168[] = {{&S66243[28], 4}, {NULL}};
const Segment SGI351169[] = {{&S66243[103], 5}, {NULL}};
const Segment SGI351170[] = {{&S74367[43], 3}, {NULL}};
const Segment SGI351171[] = {{&S74367[6], 3}, {NULL}};
const Segment SGI351172[] = {{&S74367[67], 3}, {NULL}};
const Segment SGI351173[] = {{&S74377[5], 10}, {NULL}};
const Segment SGI351174[] = {{&S74378[5], 5}, {NULL}};
const Segment SGI351175[] = {{&S74391[5], 12}, {&S74391[5], 12}, {NULL}};
const Segment SGI351176[] = {{&S74391[127], 3}, {NULL}};
const Segment SGI351177[] = {{&S47938[6], 4}, {NULL}};
const Segment SGI351178[] = {{&S47938[82], 5}, {NULL}};
const Segment SGI351179[] = {{&S47938[12], 14}, {&S47938[40], 12}, {NULL}};
const Segment SGI351180[] = {{&S47938[12], 14}, {NULL}};
const Segment SGI351181[] = {{&S47939[6], 4}, {NULL}};
const Segment SGI351182[] = {{&S47939[81], 5}, {NULL}};
const Segment SGI351183[] = {{&S47939[12], 14}, {&S47939[40], 11}, {NULL}};
const Segment SGI351184[] = {{&S47939[12], 14}, {NULL}};
const Segment SGI351185[] = {{&S47940[6], 4}, {NULL}};
const Segment SGI351186[] = {{&S47940[81], 5}, {NULL}};
const Segment SGI351187[] = {{&S47940[12], 14}, {&S47940[40], 11}, {NULL}};
const Segment SGI351188[] = {{&S47940[12], 14}, {NULL}};
const Segment SGI351189[] = {{&S47941[6], 4}, {NULL}};
const Segment SGI351190[] = {{&S47941[82], 5}, {NULL}};
const Segment SGI351191[] = {{&S47941[12], 14}, {&S47941[40], 12}, {NULL}};
const Segment SGI351192[] = {{&S47941[12], 14}, {NULL}};
const Segment SGI351193[] = {{&S47942[6], 4}, {NULL}};
const Segment SGI351194[] = {{&S47942[81], 5}, {NULL}};
const Segment SGI351195[] = {{&S47942[12], 14}, {&S47942[40], 11}, {NULL}};
const Segment SGI351196[] = {{&S47942[12], 14}, {NULL}};
const Segment SGI351197[] = {{&S47943[6], 4}, {NULL}};
const Segment SGI351198[] = {{&S47943[81], 5}, {NULL}};
const Segment SGI351199[] = {{&S47943[12], 14}, {&S47943[40], 11}, {NULL}};
const Segment SGI351200[] = {{&S47943[12], 14}, {NULL}};
const Segment SGI351201[] = {{&S47944[6], 4}, {NULL}};
const Segment SGI351202[] = {{&S47944[59], 5}, {NULL}};
const Segment SGI351203[] = {{&S47944[12], 14}, {NULL}};
const Segment SGI351204[] = {{&S47945[6], 4}, {NULL}};
const Segment SGI351205[] = {{&S47945[82], 5}, {NULL}};
const Segment SGI351206[] = {{&S47945[12], 14}, {&S47945[40], 12}, {NULL}};
const Segment SGI351207[] = {{&S47945[12], 14}, {NULL}};
const Segment SGI351208[] = {{&S47946[6], 4}, {NULL}};
const Segment SGI351209[] = {{&S47946[81], 5}, {NULL}};
const Segment SGI351210[] = {{&S47946[12], 14}, {&S47946[40], 11}, {NULL}};
const Segment SGI351211[] = {{&S47946[12], 14}, {NULL}};
const Segment SGI351212[] = {{&S47947[6], 4}, {NULL}};
const Segment SGI351213[] = {{&S47947[82], 5}, {NULL}};
const Segment SGI351214[] = {{&S47947[12], 14}, {&S47947[40], 12}, {NULL}};
const Segment SGI351215[] = {{&S47947[12], 14}, {NULL}};
const Segment SGI351216[] = {{&S47948[6], 4}, {NULL}};
const Segment SGI351217[] = {{&S47948[82], 5}, {NULL}};
const Segment SGI351218[] = {{&S47948[12], 14}, {&S47948[40], 12}, {NULL}};
const Segment SGI351219[] = {{&S47948[12], 14}, {NULL}};
const Segment SGI351220[] = {{&S47950[6], 4}, {NULL}};
const Segment SGI351221[] = {{&S47950[82], 5}, {NULL}};
const Segment SGI351222[] = {{&S47950[12], 14}, {&S47950[40], 12}, {NULL}};
const Segment SGI351223[] = {{&S47950[12], 14}, {NULL}};
const Segment SGI351224[] = {{&S47951[6], 4}, {NULL}};
const Segment SGI351225[] = {{&S47951[81], 5}, {NULL}};
const Segment SGI351226[] = {{&S47951[12], 14}, {&S47951[40], 11}, {NULL}};
const Segment SGI351227[] = {{&S47951[12], 14}, {NULL}};
const Segment SGI351228[] = {{&S47952[6], 4}, {NULL}};
const Segment SGI351229[] = {{&S47952[111], 5}, {NULL}};
const Segment SGI351230[] = {{&S47952[12], 14}, {&S47952[44], 10}, {NULL}};
const Segment SGI351231[] = {{&S47952[12], 14}, {&S47952[66], 14}, {NULL}};
const Segment SGI351232[] = {{&S47953[6], 4}, {NULL}};
const Segment SGI351233[] = {{&S47953[82], 5}, {NULL}};
const Segment SGI351234[] = {{&S47953[12], 14}, {&S47953[40], 12}, {NULL}};
const Segment SGI351235[] = {{&S47953[12], 14}, {NULL}};
const Segment SGI351236[] = {{&S47954[6], 4}, {NULL}};
const Segment SGI351237[] = {{&S47954[81], 5}, {NULL}};
const Segment SGI351238[] = {{&S47954[12], 14}, {&S47954[40], 11}, {NULL}};
const Segment SGI351239[] = {{&S47954[12], 14}, {NULL}};
const Segment SGI351240[] = {{&S47955[6], 4}, {NULL}};
const Segment SGI351241[] = {{&S47955[81], 5}, {NULL}};
const Segment SGI351242[] = {{&S47955[12], 14}, {&S47955[40], 11}, {NULL}};
const Segment SGI351243[] = {{&S47955[12], 14}, {NULL}};
const Segment SGI351244[] = {{&S47956[6], 4}, {NULL}};
const Segment SGI351245[] = {{&S47956[81], 5}, {NULL}};
const Segment SGI351246[] = {{&S47956[12], 14}, {&S47956[40], 11}, {NULL}};
const Segment SGI351247[] = {{&S47956[12], 14}, {NULL}};
const Segment SGI351248[] = {{&S48046[6], 4}, {NULL}};
const Segment SGI351249[] = {{&S48046[12], 13}, {NULL}};
const Segment SGI351250[] = {{&S48047[6], 4}, {NULL}};
const Segment SGI351251[] = {{&S48047[12], 13}, {NULL}};
const Segment SGI351252[] = {{&S48048[6], 4}, {NULL}};
const Segment SGI351253[] = {{&S48048[12], 14}, {NULL}};
const Segment SGI351254[] = {{&S48062[6], 4}, {NULL}};
const Segment SGI351255[] = {{&S48062[53], 5}, {NULL}};
const Segment SGI351256[] = {{&S48062[12], 3}, {NULL}};
const Segment SGI351257[] = {{&S48065[6], 4}, {NULL}};
const Segment SGI351258[] = {{&S48065[71], 5}, {NULL}};
const Segment SGI351259[] = {{&S48065[12], 12}, {NULL}};
const Segment SGI351260[] = {{&S48066[6], 4}, {NULL}};
const Segment SGI351261[] = {{&S48066[124], 5}, {NULL}};
const Segment SGI351262[] = {{&S48066[12], 12}, {&S48066[37], 11}, {NULL}};
const Segment SGI351263[] = {{&S48066[40], 39}, {NULL}};
const Segment SGI351264[] = {{&S48067[6], 4}, {NULL}};
const Segment SGI351265[] = {{&S48067[118], 5}, {NULL}};
const Segment SGI351266[] = {{&S48067[12], 12}, {&S48067[44], 6}, {NULL}};
const Segment SGI351267[] = {{&S48067[76], 8}, {NULL}};
const Segment SGI351268[] = {{&S74455[5], 3}, {&S74455[5], 3}, {NULL}};
const Segment SGI351269[] = {{&S74455[9], 3}, {NULL}};
const Segment SGI351270[] = {{&S74455[127], 3}, {NULL}};
const Segment SGI351271[] = {{&S74457[43], 3}, {NULL}};
const Segment SGI351272[] = {{&S74457[6], 3}, {NULL}};
const Segment SGI351273[] = {{&S74457[67], 3}, {NULL}};
const Segment SGI351274[] = {{&S74484[35], 3}, {NULL}};
const Segment SGI351275[] = {{&S74484[89], 3}, {NULL}};
const Segment SGI351276[] = {{&S74485[34], 5}, {NULL}};
const Segment SGI351277[] = {{&S74485[90], 3}, {NULL}};
const Segment SGI351278[] = {{&S66488[28], 5}, {NULL}};
const Segment SGI351279[] = {{&S66488[104], 5}, {NULL}};
const Segment SGI351280[] = {{&S74529[35], 5}, {NULL}};
const Segment SGI351281[] = {{&S74529[52], 3}, {NULL}};
const Segment SGI351282[] = {{&S74543[42], 9}, {NULL}};
const Segment SGI351283[] = {{&S74543[6], 3}, {NULL}};
const Segment SGI351284[] = {{&S74543[72], 3}, {NULL}};
const Segment SGI351285[] = {{&S74544[42], 13}, {NULL}};
const Segment SGI351286[] = {{&S74544[6], 3}, {NULL}};
const Segment SGI351287[] = {{&S74544[76], 3}, {NULL}};
const Segment SGI351288[] = {{&S74602[5], 4}, {NULL}};
const Segment SGI351289[] = {{&S74602[20], 3}, {NULL}};
const Segment SGI351290[] = {{&S74634[5], 11}, {&S74634[5], 11}, {NULL}};
const Segment SGI351291[] = {{&S74634[17], 3}, {NULL}};
const Segment SGI351292[] = {{&S74634[142], 3}, {NULL}};
const Segment SGI351293[] = {{&S74667[5], 7}, {&S74667[5], 7}, {NULL}};
const Segment SGI351294[] = {{&S74667[157], 3}, {NULL}};
const Segment SGI351295[] = {{&S74681[5], 3}, {&S74681[5], 3}, {NULL}};
const Segment SGI351296[] = {{&S74681[9], 3}, {NULL}};
const Segment SGI351297[] = {{&S74681[127], 3}, {NULL}};
const Segment SGI351298[] = {{&S48378[6], 4}, {NULL}};
const Segment SGI351299[] = {{&S48378[77], 5}, {NULL}};
const Segment SGI351300[] = {{&S48378[12], 9}, {NULL}};
const Segment SGI351301[] = {{&S48378[29], 13}, {NULL}};
const Segment SGI351302[] = {{&S66749[28], 5}, {NULL}};
const Segment SGI351303[] = {{&S66749[104], 5}, {NULL}};
const Segment SGI351304[] = {{&S74696[5], 10}, {NULL}};
const Segment SGI351305[] = {{&S74696[21], 3}, {NULL}};
const Segment SGI351306[] = {{&S50816[61], 5}, {NULL}};
const Segment SGI351307[] = {{&S50816[13], 10}, {NULL}};
const Segment SGI351308[] = {{&S74711[5], 5}, {NULL}};
const Segment SGI351309[] = {{&S74712[5], 10}, {NULL}};
const Segment SGI351310[] = {{&S74713[19], 7}, {NULL}};
const Segment SGI351311[] = {{&S74714[5], 13}, {NULL}};
const Segment SGI351312[] = {{&S74717[5], 8}, {NULL}};
const Segment SGI351313[] = {{&S74717[23], 3}, {NULL}};
const Segment SGI351314[] = {{&S74718[5], 8}, {NULL}};
const Segment SGI351315[] = {{&S74723[5], 5}, {NULL}};
const Segment SGI351316[] = {{&S74723[16], 3}, {NULL}};
const Segment SGI351317[] = {{&S74724[5], 5}, {NULL}};
const Segment SGI351318[] = {{&S74724[21], 3}, {NULL}};
const Segment SGI351319[] = {{&S48459[6], 4}, {NULL}};
const Segment SGI351320[] = {{&S48459[58], 5}, {NULL}};
const Segment SGI351321[] = {{&S48459[12], 14}, {NULL}};
const Segment SGI351322[] = {{&S48460[6], 4}, {NULL}};
const Segment SGI351323[] = {{&S48460[81], 5}, {NULL}};
const Segment SGI351324[] = {{&S48460[12], 14}, {&S48460[40], 12}, {NULL}};
const Segment SGI351325[] = {{&S48460[12], 14}, {NULL}};
const Segment SGI351326[] = {{&S48461[6], 4}, {NULL}};
const Segment SGI351327[] = {{&S48461[81], 5}, {NULL}};
const Segment SGI351328[] = {{&S48461[12], 14}, {&S48461[40], 12}, {NULL}};
const Segment SGI351329[] = {{&S48461[12], 14}, {NULL}};
const Segment SGI351330[] = {{&S48462[6], 4}, {NULL}};
const Segment SGI351331[] = {{&S48462[81], 5}, {NULL}};
const Segment SGI351332[] = {{&S48462[12], 14}, {&S48462[40], 12}, {NULL}};
const Segment SGI351333[] = {{&S48462[12], 14}, {NULL}};
const Segment SGI351334[] = {{&S48463[6], 4}, {NULL}};
const Segment SGI351335[] = {{&S48463[80], 5}, {NULL}};
const Segment SGI351336[] = {{&S48463[12], 14}, {&S48463[40], 11}, {NULL}};
const Segment SGI351337[] = {{&S48463[12], 14}, {NULL}};
const Segment SGI351338[] = {{&S48465[6], 4}, {NULL}};
const Segment SGI351339[] = {{&S48465[80], 5}, {NULL}};
const Segment SGI351340[] = {{&S48465[12], 14}, {&S48465[40], 11}, {NULL}};
const Segment SGI351341[] = {{&S48465[12], 14}, {NULL}};
const Segment SGI351342[] = {{&S48466[6], 4}, {NULL}};
const Segment SGI351343[] = {{&S48466[81], 5}, {NULL}};
const Segment SGI351344[] = {{&S48466[12], 14}, {&S48466[40], 12}, {NULL}};
const Segment SGI351345[] = {{&S48466[12], 14}, {NULL}};
const Segment SGI351346[] = {{&S48467[6], 4}, {NULL}};
const Segment SGI351347[] = {{&S48467[80], 5}, {NULL}};
const Segment SGI351348[] = {{&S48467[12], 14}, {&S48467[40], 11}, {NULL}};
const Segment SGI351349[] = {{&S48467[12], 14}, {NULL}};
const Segment SGI351350[] = {{&S48468[6], 4}, {NULL}};
const Segment SGI351351[] = {{&S48468[81], 5}, {NULL}};
const Segment SGI351352[] = {{&S48468[12], 14}, {&S48468[40], 12}, {NULL}};
const Segment SGI351353[] = {{&S48468[12], 14}, {NULL}};
const Segment SGI351354[] = {{&S74771[5], 4}, {&S74771[5], 4}, {NULL}};
const Segment SGI351355[] = {{&S74771[10], 3}, {NULL}};
const Segment SGI351356[] = {{&S48469[6], 4}, {NULL}};
const Segment SGI351357[] = {{&S48469[81], 5}, {NULL}};
const Segment SGI351358[] = {{&S48469[12], 14}, {&S48469[40], 12}, {NULL}};
const Segment SGI351359[] = {{&S48469[12], 14}, {NULL}};
const Segment SGI351360[] = {{&S74771[128], 3}, {NULL}};
const Segment SGI351361[] = {{&S48470[6], 4}, {NULL}};
const Segment SGI351362[] = {{&S48470[81], 5}, {NULL}};
const Segment SGI351363[] = {{&S48470[12], 14}, {&S48470[40], 12}, {NULL}};
const Segment SGI351364[] = {{&S48470[12], 14}, {NULL}};
const Segment SGI351365[] = {{&S48471[6], 5}, {NULL}};
const Segment SGI351366[] = {{&S48471[13], 14}, {NULL}};
const Segment SGI351367[] = {{&S74816[43], 5}, {NULL}};
const Segment SGI351368[] = {{&S74816[6], 3}, {NULL}};
const Segment SGI351369[] = {{&S74816[69], 3}, {NULL}};
const Segment SGI351370[] = {{&S74824[5], 5}, {NULL}};
const Segment SGI351371[] = {{&S74825[5], 4}, {NULL}};
const Segment SGI351372[] = {{&S74841[5], 15}, {&S74841[114], 15}, {NULL}};
const Segment SGI351373[] = {{&S74841[21], 3}, {NULL}};
const Segment SGI351374[] = {{&S74841[150], 3}, {NULL}};
const Segment SGI351375[] = {{&S74852[5], 5}, {NULL}};
const Segment SGI351376[] = {{&S74852[16], 3}, {NULL}};
const Segment SGI351377[] = {{&S74853[5], 5}, {NULL}};
const Segment SGI351378[] = {{&S74853[21], 3}, {NULL}};
const Segment SGI351379[] = {{&S74917[5], 5}, {&S74917[68], 5}, {NULL}};
const Segment SGI351380[] = {{&S74917[84], 3}, {NULL}};
const Segment SGI351381[] = {{&S74926[5], 3}, {&S74926[5], 3}, {NULL}};
const Segment SGI351382[] = {{&S74926[149], 3}, {NULL}};
const Segment SGI351383[] = {{&S74927[35], 7}, {NULL}};
const Segment SGI351384[] = {{&S74927[93], 3}, {NULL}};
const Segment SGI351385[] = {{&S74954[5], 3}, {&S74954[5], 3}, {NULL}};
const Segment SGI351386[] = {{&S74954[9], 3}, {NULL}};
const Segment SGI351387[] = {{&S74954[127], 3}, {NULL}};
const Segment SGI351388[] = {{&S60994[5], 4}, {&S60994[5], 4}, {NULL}};
const Segment SGI351389[] = {{&S60994[49], 3}, {&S60994[49], 3}, {NULL}};
const Segment SGI351390[] = {{&S60995[5], 4}, {NULL}};
const Segment SGI351391[] = {{&S60995[49], 3}, {NULL}};
const Segment SGI351392[] = {{&S74967[5], 5}, {&S74967[105], 5}, {NULL}};
const Segment SGI351393[] = {{&S74967[69], 3}, {NULL}};
const Segment SGI351394[] = {{&S74967[131], 3}, {NULL}};
const Segment SGI351395[] = {{&S74987[5], 3}, {&S74987[5], 3}, {NULL}};
const Segment SGI351396[] = {{&S74987[149], 3}, {NULL}};
const Segment SGI351397[] = {{&S74999[43], 4}, {NULL}};
const Segment SGI351398[] = {{&S74999[6], 3}, {NULL}};
const Segment SGI351399[] = {{&S74999[68], 3}, {NULL}};
const Segment SGI351400[] = {{&S66846[28], 3}, {NULL}};
const Segment SGI351401[] = {{&S66846[102], 5}, {NULL}};
const Segment SGI351402[] = {{&S75018[35], 4}, {NULL}};
const Segment SGI351403[] = {{&S75018[51], 3}, {NULL}};
const Segment SGI351404[] = {{&S48730[6], 4}, {NULL}};
const Segment SGI351405[] = {{&S48730[70], 5}, {NULL}};
const Segment SGI351406[] = {{&S48730[12], 9}, {NULL}};
const Segment SGI351407[] = {{&S48730[29], 13}, {NULL}};
const Segment SGI351408[] = {{&S75049[5], 5}, {&S75049[97], 5}, {NULL}};
const Segment SGI351409[] = {{&S75049[113], 3}, {NULL}};
const Segment SGI351410[] = {{&S66884[28], 5}, {NULL}};
const Segment SGI351411[] = {{&S66884[104], 5}, {NULL}};
const Segment SGI351412[] = {{&S75131[5], 4}, {NULL}};
const Segment SGI351413[] = {{&S75131[20], 3}, {NULL}};
const Segment SGI351414[] = {{&S75139[5], 5}, {&S75139[97], 5}, {NULL}};
const Segment SGI351415[] = {{&S75139[113], 3}, {NULL}};
const Segment SGI351416[] = {{&S67048[28], 3}, {NULL}};
const Segment SGI351417[] = {{&S67048[102], 5}, {NULL}};
const Segment SGI351418[] = {{&S67049[28], 5}, {NULL}};
const Segment SGI351419[] = {{&S67049[104], 5}, {NULL}};
const Segment SGI351420[] = {{&S51314[139], 4}, {NULL}};
const Segment SGI351421[] = {{&S51314[145], 3}, {NULL}};
const Segment SGI351422[] = {{&S51314[167], 3}, {NULL}};
const Segment SGI351423[] = {{&S51314[182], 5}, {NULL}};
const Segment SGI351424[] = {{&S75188[5], 3}, {&S75188[5], 3}, {NULL}};
const Segment SGI351425[] = {{&S75188[9], 3}, {NULL}};
const Segment SGI351426[] = {{&S75188[126], 3}, {NULL}};
const Segment SGI351427[] = {{&S75193[42], 7}, {NULL}};
const Segment SGI351428[] = {{&S75193[6], 3}, {NULL}};
const Segment SGI351429[] = {{&S75193[70], 3}, {NULL}};
const Segment SGI351430[] = {{&S75199[5], 3}, {NULL}};
const Segment SGI351431[] = {{&S75224[35], 5}, {NULL}};
const Segment SGI351432[] = {{&S75224[91], 3}, {NULL}};
const Segment SGI351433[] = {{&S75302[35], 5}, {NULL}};
const Segment SGI351434[] = {{&S75302[91], 3}, {NULL}};
const Segment SGI351435[] = {{&S75311[43], 3}, {NULL}};
const Segment SGI351436[] = {{&S75311[6], 3}, {NULL}};
const Segment SGI351437[] = {{&S75311[67], 3}, {NULL}};
const Segment SGI351438[] = {{&S75361[42], 5}, {NULL}};
const Segment SGI351439[] = {{&S75361[6], 3}, {NULL}};
const Segment SGI351440[] = {{&S75361[68], 3}, {NULL}};
const Segment SGI351441[] = {{&S75362[5], 5}, {NULL}};
const Segment SGI351442[] = {{&S75362[16], 3}, {NULL}};
const Segment SGI351443[] = {{&S49288[6], 4}, {NULL}};
const Segment SGI351444[] = {{&S49288[68], 5}, {NULL}};
const Segment SGI351445[] = {{&S49288[12], 9}, {NULL}};
const Segment SGI351446[] = {{&S49289[6], 4}, {NULL}};
const Segment SGI351447[] = {{&S49289[68], 5}, {NULL}};
const Segment SGI351448[] = {{&S49289[12], 9}, {NULL}};
const Segment SGI351449[] = {{&S49290[6], 4}, {NULL}};
const Segment SGI351450[] = {{&S49290[66], 5}, {NULL}};
const Segment SGI351451[] = {{&S49290[12], 9}, {NULL}};
const Segment SGI351452[] = {{&S49291[6], 4}, {NULL}};
const Segment SGI351453[] = {{&S49291[68], 5}, {NULL}};
const Segment SGI351454[] = {{&S49291[12], 9}, {NULL}};
const Segment SGI351455[] = {{&S75409[18], 13}, {NULL}};
const Segment SGI351456[] = {{&S75421[5], 3}, {&S75421[5], 3}, {NULL}};
const Segment SGI351457[] = {{&S75421[9], 3}, {NULL}};
const Segment SGI351458[] = {{&S75421[126], 3}, {NULL}};
const Segment SGI351459[] = {{&S75422[5], 5}, {&S75422[5], 5}, {NULL}};
const Segment SGI351460[] = {{&S75422[11], 3}, {NULL}};
const Segment SGI351461[] = {{&S75422[130], 3}, {NULL}};
const Segment SGI351462[] = {{&S49316[6], 4}, {NULL}};
const Segment SGI351463[] = {{&S49316[70], 5}, {NULL}};
const Segment SGI351464[] = {{&S49316[12], 9}, {NULL}};
const Segment SGI351465[] = {{&S49316[29], 13}, {NULL}};
const Segment SGI351466[] = {{&S75449[35], 4}, {NULL}};
const Segment SGI351467[] = {{&S75449[51], 3}, {NULL}};
const Segment SGI351468[] = {{&S51608[7], 5}, {NULL}};
const Segment SGI351469[] = {{&S67467[28], 7}, {NULL}};
const Segment SGI351470[] = {{&S67467[106], 5}, {NULL}};
const Segment SGI351471[] = {{&S75492[5], 11}, {&S75492[5], 11}, {NULL}};
const Segment SGI351472[] = {{&S75492[17], 3}, {NULL}};
const Segment SGI351473[] = {{&S75492[142], 3}, {NULL}};
const Segment SGI351474[] = {{&S67468[28], 5}, {NULL}};
const Segment SGI351475[] = {{&S67468[104], 5}, {NULL}};
const Segment SGI351476[] = {{&S67469[28], 7}, {NULL}};
const Segment SGI351477[] = {{&S67469[106], 5}, {NULL}};
const Segment SGI351478[] = {{&S75505[44], 21}, {NULL}};
const Segment SGI351479[] = {{&S75505[6], 3}, {NULL}};
const Segment SGI351480[] = {{&S75505[86], 3}, {NULL}};
const Segment SGI351481[] = {{&S75506[44], 21}, {NULL}};
const Segment SGI351482[] = {{&S75506[6], 3}, {NULL}};
const Segment SGI351483[] = {{&S75506[86], 3}, {NULL}};
const Segment SGI351484[] = {{&S75508[5], 5}, {NULL}};
const Segment SGI351485[] = {{&S75511[5], 4}, {NULL}};
const Segment SGI351486[] = {{&S75511[20], 3}, {NULL}};
const Segment SGI351487[] = {{&S75512[5], 4}, {NULL}};
const Segment SGI351488[] = {{&S75512[15], 3}, {NULL}};
const Segment SGI351489[] = {{&S61653[46], 7}, {NULL}};
const Segment SGI351490[] = {{&S61653[37], 3}, {NULL}};
const Segment SGI351491[] = {{&S75562[5], 3}, {&S75562[5], 3}, {NULL}};
const Segment SGI351492[] = {{&S75562[9], 3}, {NULL}};
const Segment SGI351493[] = {{&S75562[127], 3}, {NULL}};
const Segment SGI351494[] = {{&S75608[5], 3}, {&S75608[5], 3}, {NULL}};
const Segment SGI351495[] = {{&S75608[149], 3}, {NULL}};
const Segment SGI351496[] = {{&S75615[5], 7}, {NULL}};
const Segment SGI351497[] = {{&S75658[5], 5}, {&S75658[68], 5}, {NULL}};
const Segment SGI351498[] = {{&S75658[84], 3}, {NULL}};
const Segment SGI351499[] = {{&S75660[5], 12}, {&S75660[5], 12}, {NULL}};
const Segment SGI351500[] = {{&S75660[167], 3}, {NULL}};
const Segment SGI351501[] = {{&S75701[5], 4}, {&S75701[5], 4}, {NULL}};
const Segment SGI351502[] = {{&S75701[151], 3}, {NULL}};
const Segment SGI351503[] = {{&S75782[34], 5}, {NULL}};
const Segment SGI351504[] = {{&S75782[50], 3}, {NULL}};
const Segment SGI351505[] = {{&S75788[42], 4}, {NULL}};
const Segment SGI351506[] = {{&S75788[6], 3}, {NULL}};
const Segment SGI351507[] = {{&S75788[67], 3}, {NULL}};
const Segment SGI351508[] = {{&S75797[42], 5}, {NULL}};
const Segment SGI351509[] = {{&S75797[6], 3}, {NULL}};
const Segment SGI351510[] = {{&S75797[68], 3}, {NULL}};
const Segment SGI351511[] = {{&S49780[6], 4}, {NULL}};
const Segment SGI351512[] = {{&S49780[57], 5}, {NULL}};
const Segment SGI351513[] = {{&S49780[12], 14}, {NULL}};
const Segment SGI351514[] = {{&S49781[6], 4}, {NULL}};
const Segment SGI351515[] = {{&S49781[57], 5}, {NULL}};
const Segment SGI351516[] = {{&S49781[12], 14}, {NULL}};
const Segment SGI351517[] = {{&S49782[6], 4}, {NULL}};
const Segment SGI351518[] = {{&S49782[57], 5}, {NULL}};
const Segment SGI351519[] = {{&S49782[12], 14}, {NULL}};
const Segment SGI351520[] = {{&S49783[6], 4}, {NULL}};
const Segment SGI351521[] = {{&S49783[57], 5}, {NULL}};
const Segment SGI351522[] = {{&S49783[12], 14}, {NULL}};
const Segment SGI351523[] = {{&S49784[6], 4}, {NULL}};
const Segment SGI351524[] = {{&S49784[57], 5}, {NULL}};
const Segment SGI351525[] = {{&S49784[12], 14}, {NULL}};
const Segment SGI351526[] = {{&S51913[61], 4}, {NULL}};
const Segment SGI351527[] = {{&S51913[13], 10}, {NULL}};
const Segment SGI351528[] = {{&S51914[61], 3}, {NULL}};
const Segment SGI351529[] = {{&S51914[13], 10}, {NULL}};
const Segment SGI351530[] = {{&S51918[61], 4}, {NULL}};
const Segment SGI351531[] = {{&S51918[13], 10}, {NULL}};
const Segment SGI351532[] = {{&S51924[6], 3}, {&S51924[6], 3}, {NULL}};
const Segment SGI351533[] = {{&S51924[11], 34}, {NULL}};
const Segment SGI351534[] = {{&S75839[34], 5}, {NULL}};
const Segment SGI351535[] = {{&S75839[90], 3}, {NULL}};
const Segment SGI351536[] = {{&S75840[34], 3}, {NULL}};
const Segment SGI351537[] = {{&S75840[88], 3}, {NULL}};
const Segment SGI351538[] = {{&S75841[35], 3}, {NULL}};
const Segment SGI351539[] = {{&S75841[89], 3}, {NULL}};
const Segment SGI351540[] = {{&S75842[41], 5}, {NULL}};
const Segment SGI351541[] = {{&S75842[6], 3}, {NULL}};
const Segment SGI351542[] = {{&S75842[67], 3}, {NULL}};
const Segment SGI351543[] = {{&S75843[5], 12}, {NULL}};
const Segment SGI351544[] = {{&S75843[18], 3}, {NULL}};
const Segment SGI351545[] = {{&S75845[42], 3}, {NULL}};
const Segment SGI351546[] = {{&S75845[6], 3}, {NULL}};
const Segment SGI351547[] = {{&S75845[66], 3}, {NULL}};
const Segment SGI351548[] = {{&S49922[6], 4}, {NULL}};
const Segment SGI351549[] = {{&S49922[12], 9}, {NULL}};
const Segment SGI351550[] = {{&S49923[6], 4}, {NULL}};
const Segment SGI351551[] = {{&S49923[12], 9}, {NULL}};
const Segment SGI351552[] = {{&S49924[6], 4}, {NULL}};
const Segment SGI351553[] = {{&S49924[12], 9}, {NULL}};
const Segment SGI351554[] = {{&S49925[6], 4}, {NULL}};
const Segment SGI351555[] = {{&S49925[12], 9}, {NULL}};
const Segment SGI351556[] = {{&S49926[6], 4}, {NULL}};
const Segment SGI351557[] = {{&S49926[12], 9}, {NULL}};
const Segment SGI351558[] = {{&S49927[6], 4}, {NULL}};
const Segment SGI351559[] = {{&S49927[12], 9}, {NULL}};
const Segment SGI351560[] = {{&S49928[6], 4}, {NULL}};
const Segment SGI351561[] = {{&S49928[12], 9}, {NULL}};
const Segment SGI351562[] = {{&S49929[6], 4}, {NULL}};
const Segment SGI351563[] = {{&S49929[12], 9}, {NULL}};
const Segment SGI351564[] = {{&S49930[6], 4}, {NULL}};
const Segment SGI351565[] = {{&S49930[12], 8}, {NULL}};
const Segment SGI351566[] = {{&S75878[34], 3}, {NULL}};
const Segment SGI351567[] = {{&S75878[48], 3}, {NULL}};
const Segment SGI351568[] = {{&S75887[42], 4}, {NULL}};
const Segment SGI351569[] = {{&S75887[6], 3}, {NULL}};
const Segment SGI351570[] = {{&S75887[67], 3}, {NULL}};
const Segment SGI351571[] = {{&S75888[5], 4}, {&S75888[5], 4}, {NULL}};
const Segment SGI351572[] = {{&S75888[10], 3}, {NULL}};
const Segment SGI351573[] = {{&S75888[131], 3}, {NULL}};
const Segment SGI351574[] = {{&S75968[5], 3}, {NULL}};
const Segment SGI351575[] = {{&S75968[14], 3}, {NULL}};
const Segment SGI351576[] = {{&S75969[34], 5}, {NULL}};
const Segment SGI351577[] = {{&S75969[50], 3}, {NULL}};
const Segment SGI351578[] = {{&S75991[42], 5}, {NULL}};
const Segment SGI351579[] = {{&S75991[6], 3}, {NULL}};
const Segment SGI351580[] = {{&S75991[68], 3}, {NULL}};
const Segment SGI351581[] = {{&S75993[42], 3}, {NULL}};
const Segment SGI351582[] = {{&S75993[6], 3}, {NULL}};
const Segment SGI351583[] = {{&S75993[66], 3}, {NULL}};
const Segment SGI351584[] = {{&S75994[5], 3}, {NULL}};
const Segment SGI351585[] = {{&S75994[14], 3}, {NULL}};
const Segment SGI351586[] = {{&S76010[100], 7}, {NULL}};
const Segment SGI351587[] = {{&S76011[100], 7}, {NULL}};
const Segment SGI351588[] = {{&S76012[100], 17}, {NULL}};
const Segment SGI351589[] = {{&S76025[43], 5}, {NULL}};
const Segment SGI351590[] = {{&S76025[6], 3}, {NULL}};
const Segment SGI351591[] = {{&S76025[69], 3}, {NULL}};
const Segment SGI351592[] = {{&S50089[6], 4}, {NULL}};
const Segment SGI351593[] = {{&S50089[58], 5}, {NULL}};
const Segment SGI351594[] = {{&S50089[12], 14}, {NULL}};
const Segment SGI351595[] = {{&S50090[6], 4}, {NULL}};
const Segment SGI351596[] = {{&S50090[81], 5}, {NULL}};
const Segment SGI351597[] = {{&S50090[12], 14}, {&S50090[40], 12}, {NULL}};
const Segment SGI351598[] = {{&S50090[12], 14}, {NULL}};
const Segment SGI351599[] = {{&S50091[6], 4}, {NULL}};
const Segment SGI351600[] = {{&S50091[81], 5}, {NULL}};
const Segment SGI351601[] = {{&S50091[12], 14}, {&S50091[40], 12}, {NULL}};
const Segment SGI351602[] = {{&S50091[12], 14}, {NULL}};
const Segment SGI351603[] = {{&S50092[6], 4}, {NULL}};
const Segment SGI351604[] = {{&S50092[107], 5}, {NULL}};
const Segment SGI351605[] = {{&S50092[12], 14}, {&S50092[40], 12}, {NULL}};
const Segment SGI351606[] = {{&S50092[12], 14}, {NULL}};
const Segment SGI351607[] = {{&S50093[6], 4}, {NULL}};
const Segment SGI351608[] = {{&S50093[81], 5}, {NULL}};
const Segment SGI351609[] = {{&S50093[12], 14}, {&S50093[40], 12}, {NULL}};
const Segment SGI351610[] = {{&S50093[12], 14}, {NULL}};
const Segment SGI351611[] = {{&S50094[6], 4}, {NULL}};
const Segment SGI351612[] = {{&S50094[80], 5}, {NULL}};
const Segment SGI351613[] = {{&S50094[12], 14}, {&S50094[40], 11}, {NULL}};
const Segment SGI351614[] = {{&S50094[12], 14}, {NULL}};
const Segment SGI351615[] = {{&S50095[6], 4}, {NULL}};
const Segment SGI351616[] = {{&S50095[58], 5}, {NULL}};
const Segment SGI351617[] = {{&S50095[12], 14}, {&S50095[69], 31}, {NULL}};
const Segment SGI351618[] = {{&S50095[12], 14}, {NULL}};
const Segment SGI351619[] = {{&S50096[6], 4}, {NULL}};
const Segment SGI351620[] = {{&S50096[81], 5}, {NULL}};
const Segment SGI351621[] = {{&S50096[12], 14}, {&S50096[40], 12}, {NULL}};
const Segment SGI351622[] = {{&S50096[12], 14}, {NULL}};
const Segment SGI351623[] = {{&S50023[0], 22}, {NULL}};
const Segment SGI351624[] = {{&S50023[35], 3}, {NULL}};
const Segment SGI351625[] = {{&S76038[5], 5}, {&S76038[97], 5}, {NULL}};
const Segment SGI351626[] = {{&S50023[76], 13}, {NULL}};
const Segment SGI351627[] = {{&S76038[113], 3}, {NULL}};
const Segment SGI351628[] = {{&S50098[6], 4}, {NULL}};
const Segment SGI351629[] = {{&S50098[77], 5}, {NULL}};
const Segment SGI351630[] = {{&S50098[12], 9}, {NULL}};
const Segment SGI351631[] = {{&S50098[29], 13}, {NULL}};
const Segment SGI351632[] = {{&S76039[21], 4}, {NULL}};
const Segment SGI351633[] = {{&S76039[0], 14}, {NULL}};
const Segment SGI351634[] = {{&S50102[6], 4}, {NULL}};
const Segment SGI351635[] = {{&S50102[70], 5}, {NULL}};
const Segment SGI351636[] = {{&S50102[12], 9}, {NULL}};
const Segment SGI351637[] = {{&S50102[29], 13}, {NULL}};
const Segment SGI351638[] = {{&S76052[5], 3}, {&S76052[5], 3}, {NULL}};
const Segment SGI351639[] = {{&S76052[9], 3}, {NULL}};
const Segment SGI351640[] = {{&S76052[127], 3}, {NULL}};
const Segment SGI351641[] = {{&S76053[42], 5}, {NULL}};
const Segment SGI351642[] = {{&S76053[6], 3}, {NULL}};
const Segment SGI351643[] = {{&S76053[68], 3}, {NULL}};
const Segment SGI351644[] = {{&S76054[43], 3}, {NULL}};
const Segment SGI351645[] = {{&S76054[6], 3}, {NULL}};
const Segment SGI351646[] = {{&S76054[67], 3}, {NULL}};
const Segment SGI351647[] = {{&S76073[41], 4}, {NULL}};
const Segment SGI351648[] = {{&S76073[6], 3}, {NULL}};
const Segment SGI351649[] = {{&S76073[66], 3}, {NULL}};
const Segment SGI351650[] = {{&S76088[43], 3}, {NULL}};
const Segment SGI351651[] = {{&S76088[6], 3}, {NULL}};
const Segment SGI351652[] = {{&S76088[67], 3}, {NULL}};
const Segment SGI351653[] = {{&S76089[5], 3}, {&S76089[5], 3}, {NULL}};
const Segment SGI351654[] = {{&S76089[9], 3}, {NULL}};
const Segment SGI351655[] = {{&S76089[127], 3}, {NULL}};
const Segment SGI351656[] = {{&S76137[5], 7}, {&S76137[5], 7}, {NULL}};
const Segment SGI351657[] = {{&S76137[88], 3}, {NULL}};
const Segment SGI351658[] = {{&S76229[5], 4}, {NULL}};
const Segment SGI351659[] = {{&S76229[15], 3}, {NULL}};
const Segment SGI351660[] = {{&S76230[35], 4}, {NULL}};
const Segment SGI351661[] = {{&S76230[51], 3}, {NULL}};
const Segment SGI351662[] = {{&S50370[6], 4}, {NULL}};
const Segment SGI351663[] = {{&S50370[77], 5}, {NULL}};
const Segment SGI351664[] = {{&S50370[12], 9}, {NULL}};
const Segment SGI351665[] = {{&S50370[29], 13}, {NULL}};
const Segment SGI351666[] = {{&S50371[6], 4}, {NULL}};
const Segment SGI351667[] = {{&S50371[77], 5}, {NULL}};
const Segment SGI351668[] = {{&S50371[12], 9}, {NULL}};
const Segment SGI351669[] = {{&S50371[29], 13}, {NULL}};
const Segment SGI351670[] = {{&S50375[6], 4}, {NULL}};
const Segment SGI351671[] = {{&S50375[12], 14}, {NULL}};
const Segment SGI351672[] = {{&S50376[6], 4}, {NULL}};
const Segment SGI351673[] = {{&S50376[12], 14}, {NULL}};
const Segment SGI351674[] = {{&S50377[6], 4}, {NULL}};
const Segment SGI351675[] = {{&S50377[12], 14}, {NULL}};
const Segment SGI351676[] = {{&S50378[6], 4}, {NULL}};
const Segment SGI351677[] = {{&S50378[12], 14}, {NULL}};
const Segment SGI351678[] = {{&S50379[6], 4}, {NULL}};
const Segment SGI351679[] = {{&S50379[12], 14}, {NULL}};
const Segment SGI351680[] = {{&S50380[6], 4}, {NULL}};
const Segment SGI351681[] = {{&S50380[12], 14}, {NULL}};
const Segment SGI351682[] = {{&S50381[6], 4}, {NULL}};
const Segment SGI351683[] = {{&S50381[12], 14}, {NULL}};
const Segment SGI351684[] = {{&S50382[6], 4}, {NULL}};
const Segment SGI351685[] = {{&S50382[12], 14}, {NULL}};
const Segment SGI351686[] = {{&S76392[5], 3}, {&S76392[5], 3}, {NULL}};
const Segment SGI351687[] = {{&S76392[149], 3}, {NULL}};
const Segment SGI351688[] = {{&S76461[35], 3}, {NULL}};
const Segment SGI351689[] = {{&S76461[89], 3}, {NULL}};
const Segment SGI351690[] = {{&S76483[5], 5}, {NULL}};
const Segment SGI351691[] = {{&S76483[21], 3}, {NULL}};
const Segment SGI351692[] = {{&S76484[5], 5}, {NULL}};
const Segment SGI351693[] = {{&S76484[16], 3}, {NULL}};
const Segment SGI351694[] = {{&S76526[5], 3}, {&S76526[5], 3}, {NULL}};
const Segment SGI351695[] = {{&S76526[9], 3}, {NULL}};
const Segment SGI351696[] = {{&S76526[127], 3}, {NULL}};
const Segment SGI351697[] = {{&S50485[6], 4}, {NULL}};
const Segment SGI351698[] = {{&S50485[70], 5}, {NULL}};
const Segment SGI351699[] = {{&S50485[12], 9}, {NULL}};
const Segment SGI351700[] = {{&S50485[29], 13}, {NULL}};
const Segment SGI351701[] = {{&S76540[5], 12}, {&S76540[5], 12}, {NULL}};
const Segment SGI351702[] = {{&S76540[76], 3}, {NULL}};
const Segment SGI351703[] = {{&S76540[145], 3}, {NULL}};
const Segment SGI351704[] = {{&S76543[5], 4}, {NULL}};
const Segment SGI351705[] = {{&S76544[5], 7}, {&S76544[5], 7}, {NULL}};
const Segment SGI351706[] = {{&S76544[88], 3}, {NULL}};
const Segment SGI351707[] = {{&S68241[28], 7}, {NULL}};
const Segment SGI351708[] = {{&S68241[106], 5}, {NULL}};
const Segment SGI351709[] = {{&S68242[28], 5}, {NULL}};
const Segment SGI351710[] = {{&S68242[104], 5}, {NULL}};
const Segment SGI351711[] = {{&S76588[5], 5}, {&S76588[105], 5}, {NULL}};
const Segment SGI351712[] = {{&S76588[11], 3}, {NULL}};
const Segment SGI351713[] = {{&S76588[131], 3}, {NULL}};
const Segment SGI351714[] = {{&S76614[35], 3}, {NULL}};
const Segment SGI351715[] = {{&S76614[50], 3}, {NULL}};
const Segment SGI351716[] = {{&S76616[5], 3}, {&S76616[5], 3}, {NULL}};
const Segment SGI351717[] = {{&S76616[149], 3}, {NULL}};
const Segment SGI351718[] = {{&S52785[63], 5}, {NULL}};
const Segment SGI351719[] = {{&S52785[13], 10}, {NULL}};
const Segment SGI351720[] = {{&S76660[21], 5}, {NULL}};
const Segment SGI351721[] = {{&S76660[0], 14}, {NULL}};
const Segment SGI351722[] = {{&S76672[43], 8}, {NULL}};
const Segment SGI351723[] = {{&S76672[6], 3}, {NULL}};
const Segment SGI351724[] = {{&S76672[72], 3}, {NULL}};
const Segment SGI351725[] = {{&S76674[43], 3}, {NULL}};
const Segment SGI351726[] = {{&S76674[6], 3}, {NULL}};
const Segment SGI351727[] = {{&S76674[67], 3}, {NULL}};
const Segment SGI351728[] = {{&S76712[34], 5}, {NULL}};
const Segment SGI351729[] = {{&S76712[90], 3}, {NULL}};
const Segment SGI351730[] = {{&S76713[42], 3}, {NULL}};
const Segment SGI351731[] = {{&S76713[6], 3}, {NULL}};
const Segment SGI351732[] = {{&S76713[66], 3}, {NULL}};
const Segment SGI351733[] = {{&S76737[5], 3}, {&S76737[5], 3}, {NULL}};
const Segment SGI351734[] = {{&S76737[80], 3}, {NULL}};
const Segment SGI351735[] = {{&S76738[35], 5}, {NULL}};
const Segment SGI351736[] = {{&S76738[91], 3}, {NULL}};
const Segment SGI351737[] = {{&S63032[5], 4}, {NULL}};
const Segment SGI351738[] = {{&S63032[47], 3}, {NULL}};
const Segment SGI351739[] = {{&S63032[101], 3}, {NULL}};
const Segment SGI351740[] = {{&S68315[28], 5}, {NULL}};
const Segment SGI351741[] = {{&S68315[104], 5}, {NULL}};
const Segment SGI351742[] = {{&S50740[6], 4}, {NULL}};
const Segment SGI351743[] = {{&S50740[12], 3}, {NULL}};
const Segment SGI351744[] = {{&S63054[5], 4}, {&S63054[5], 4}, {NULL}};
const Segment SGI351745[] = {{&S63054[43], 3}, {NULL}};
const Segment SGI351746[] = {{&S63055[49], 4}, {NULL}};
const Segment SGI351747[] = {{&S63055[40], 3}, {NULL}};
const Segment SGI351748[] = {{&S63056[49], 4}, {NULL}};
const Segment SGI351749[] = {{&S63056[40], 3}, {NULL}};
const Segment SGI351750[] = {{&S76772[5], 3}, {NULL}};
const Segment SGI351751[] = {{&S76772[19], 3}, {NULL}};
const Segment SGI351752[] = {{&S76787[42], 5}, {NULL}};
const Segment SGI351753[] = {{&S76787[6], 3}, {NULL}};
const Segment SGI351754[] = {{&S76787[68], 3}, {NULL}};
const Segment SGI351755[] = {{&S50705[0], 21}, {NULL}};
const Segment SGI351756[] = {{&S50705[34], 3}, {NULL}};
const Segment SGI351757[] = {{&S50705[75], 13}, {NULL}};
const Segment SGI351758[] = {{&S76796[43], 3}, {NULL}};
const Segment SGI351759[] = {{&S76796[6], 3}, {NULL}};
const Segment SGI351760[] = {{&S76796[67], 3}, {NULL}};
const Segment SGI351761[] = {{&S68400[28], 3}, {NULL}};
const Segment SGI351762[] = {{&S68400[102], 5}, {NULL}};
const Segment SGI351763[] = {{&S76826[21], 5}, {NULL}};
const Segment SGI351764[] = {{&S76826[0], 14}, {NULL}};
const Segment SGI351765[] = {{&S53003[53], 4}, {NULL}};
const Segment SGI351766[] = {{&S53003[13], 10}, {NULL}};
const Segment SGI351767[] = {{&S53005[54], 4}, {NULL}};
const Segment SGI351768[] = {{&S53005[13], 10}, {NULL}};
const Segment SGI351769[] = {{&S53006[53], 4}, {NULL}};
const Segment SGI351770[] = {{&S53006[13], 10}, {NULL}};
const Segment SGI351771[] = {{&S76856[5], 4}, {NULL}};
const Segment SGI351772[] = {{&S76924[21], 3}, {NULL}};
const Segment SGI351773[] = {{&S76924[0], 14}, {NULL}};
const Segment SGI351774[] = {{&S76932[43], 9}, {NULL}};
const Segment SGI351775[] = {{&S76932[6], 3}, {NULL}};
const Segment SGI351776[] = {{&S76932[73], 3}, {NULL}};
const Segment SGI351777[] = {{&S76933[42], 5}, {NULL}};
const Segment SGI351778[] = {{&S76933[6], 3}, {NULL}};
const Segment SGI351779[] = {{&S76933[68], 3}, {NULL}};
const Segment SGI351780[] = {{&S76934[43], 13}, {NULL}};
const Segment SGI351781[] = {{&S76934[6], 3}, {NULL}};
const Segment SGI351782[] = {{&S76934[77], 3}, {NULL}};
const Segment SGI351783[] = {{&S76935[43], 3}, {NULL}};
const Segment SGI351784[] = {{&S76935[6], 3}, {NULL}};
const Segment SGI351785[] = {{&S76935[67], 3}, {NULL}};
const Segment SGI351786[] = {{&S50811[6], 4}, {NULL}};
const Segment SGI351787[] = {{&S50811[85], 5}, {NULL}};
const Segment SGI351788[] = {{&S50811[12], 13}, {NULL}};
const Segment SGI351789[] = {{&S50812[6], 4}, {NULL}};
const Segment SGI351790[] = {{&S50812[85], 5}, {NULL}};
const Segment SGI351791[] = {{&S50812[12], 13}, {NULL}};
const Segment SGI351792[] = {{&S50813[6], 4}, {NULL}};
const Segment SGI351793[] = {{&S50813[104], 5}, {NULL}};
const Segment SGI351794[] = {{&S50813[12], 32}, {NULL}};
const Segment SGI351795[] = {{&S76959[5], 5}, {&S76959[68], 5}, {NULL}};
const Segment SGI351796[] = {{&S50814[6], 4}, {NULL}};
const Segment SGI351797[] = {{&S50814[57], 5}, {NULL}};
const Segment SGI351798[] = {{&S76959[84], 3}, {NULL}};
const Segment SGI351799[] = {{&S50814[12], 14}, {NULL}};
const Segment SGI351800[] = {{&S76960[5], 5}, {NULL}};
const Segment SGI351801[] = {{&S76960[236], 3}, {NULL}};
const Segment SGI351802[] = {{&S50817[6], 4}, {NULL}};
const Segment SGI351803[] = {{&S50817[12], 14}, {NULL}};
const Segment SGI351804[] = {{&S76961[5], 5}, {NULL}};
const Segment SGI351805[] = {{&S76997[5], 7}, {NULL}};
const Segment SGI351806[] = {{&S76998[5], 7}, {NULL}};
const Segment SGI351807[] = {{&S77027[5], 5}, {&S77027[104], 5}, {NULL}};
const Segment SGI351808[] = {{&S77027[69], 3}, {NULL}};
const Segment SGI351809[] = {{&S77027[130], 3}, {NULL}};
const Segment SGI351810[] = {{&S77028[5], 7}, {&S77028[106], 7}, {NULL}};
const Segment SGI351811[] = {{&S77028[71], 3}, {NULL}};
const Segment SGI351812[] = {{&S77028[134], 3}, {NULL}};
const Segment SGI351813[] = {{&S77057[5], 4}, {NULL}};
const Segment SGI351814[] = {{&S77088[5], 5}, {NULL}};
const Segment SGI351815[] = {{&S77089[5], 5}, {NULL}};
const Segment SGI351816[] = {{&S77089[20], 3}, {NULL}};
const Segment SGI351817[] = {{&S77113[101], 4}, {NULL}};
const Segment SGI351818[] = {{&S77114[5], 21}, {NULL}};
const Segment SGI351819[] = {{&S77129[53], 4}, {NULL}};
const Segment SGI351820[] = {{&S77156[42], 5}, {NULL}};
const Segment SGI351821[] = {{&S77156[6], 3}, {NULL}};
const Segment SGI351822[] = {{&S77156[68], 3}, {NULL}};
const Segment SGI351823[] = {{&S77157[43], 5}, {NULL}};
const Segment SGI351824[] = {{&S77157[6], 3}, {NULL}};
const Segment SGI351825[] = {{&S77157[69], 3}, {NULL}};
const Segment SGI351826[] = {{&S53305[62], 5}, {NULL}};
const Segment SGI351827[] = {{&S53305[13], 10}, {NULL}};
const Segment SGI351828[] = {{&S77202[5], 3}, {&S77202[5], 3}, {NULL}};
const Segment SGI351829[] = {{&S77202[80], 3}, {NULL}};
const Segment SGI351830[] = {{&S77225[5], 3}, {&S77225[5], 3}, {NULL}};
const Segment SGI351831[] = {{&S77225[149], 3}, {NULL}};
const Segment SGI351832[] = {{&S77286[35], 3}, {NULL}};
const Segment SGI351833[] = {{&S77286[89], 3}, {NULL}};
const Segment SGI351834[] = {{&S77298[5], 3}, {&S77298[5], 3}, {NULL}};
const Segment SGI351835[] = {{&S77298[9], 3}, {NULL}};
const Segment SGI351836[] = {{&S77298[126], 3}, {NULL}};
const Segment SGI351837[] = {{&S51011[6], 4}, {NULL}};
const Segment SGI351838[] = {{&S51011[58], 5}, {NULL}};
const Segment SGI351839[] = {{&S51011[12], 14}, {NULL}};
const Segment SGI351840[] = {{&S51012[6], 4}, {NULL}};
const Segment SGI351841[] = {{&S51012[81], 5}, {NULL}};
const Segment SGI351842[] = {{&S51012[12], 14}, {&S51012[40], 12}, {NULL}};
const Segment SGI351843[] = {{&S51012[12], 14}, {NULL}};
const Segment SGI351844[] = {{&S51013[6], 4}, {NULL}};
const Segment SGI351845[] = {{&S51013[58], 5}, {NULL}};
const Segment SGI351846[] = {{&S51013[12], 14}, {&S51013[69], 31}, {NULL}};
const Segment SGI351847[] = {{&S51013[12], 14}, {NULL}};
const Segment SGI351848[] = {{&S51014[6], 4}, {NULL}};
const Segment SGI351849[] = {{&S51014[81], 5}, {NULL}};
const Segment SGI351850[] = {{&S51014[12], 14}, {&S51014[40], 12}, {NULL}};
const Segment SGI351851[] = {{&S51014[12], 14}, {NULL}};
const Segment SGI351852[] = {{&S51015[6], 4}, {NULL}};
const Segment SGI351853[] = {{&S51015[81], 5}, {NULL}};
const Segment SGI351854[] = {{&S51015[12], 14}, {&S51015[40], 12}, {NULL}};
const Segment SGI351855[] = {{&S51015[12], 14}, {NULL}};
const Segment SGI351856[] = {{&S51016[6], 4}, {NULL}};
const Segment SGI351857[] = {{&S51016[80], 5}, {NULL}};
const Segment SGI351858[] = {{&S51016[12], 14}, {&S51016[40], 11}, {NULL}};
const Segment SGI351859[] = {{&S51016[12], 14}, {NULL}};
const Segment SGI351860[] = {{&S51017[6], 4}, {NULL}};
const Segment SGI351861[] = {{&S51017[81], 5}, {NULL}};
const Segment SGI351862[] = {{&S51017[12], 14}, {&S51017[40], 12}, {NULL}};
const Segment SGI351863[] = {{&S51017[12], 14}, {NULL}};
const Segment SGI351864[] = {{&S51018[6], 4}, {NULL}};
const Segment SGI351865[] = {{&S51018[80], 5}, {NULL}};
const Segment SGI351866[] = {{&S51018[12], 14}, {&S51018[40], 11}, {NULL}};
const Segment SGI351867[] = {{&S51018[12], 14}, {NULL}};
const Segment SGI351868[] = {{&S51019[6], 4}, {NULL}};
const Segment SGI351869[] = {{&S51019[81], 5}, {NULL}};
const Segment SGI351870[] = {{&S51019[12], 14}, {&S51019[40], 12}, {NULL}};
const Segment SGI351871[] = {{&S51019[12], 14}, {NULL}};
const Segment SGI351872[] = {{&S51020[6], 4}, {NULL}};
const Segment SGI351873[] = {{&S51020[81], 5}, {NULL}};
const Segment SGI351874[] = {{&S51020[12], 14}, {&S51020[40], 12}, {NULL}};
const Segment SGI351875[] = {{&S51020[12], 14}, {NULL}};
const Segment SGI351876[] = {{&S51021[6], 4}, {NULL}};
const Segment SGI351877[] = {{&S51021[75], 5}, {NULL}};
const Segment SGI351878[] = {{&S51021[12], 3}, {NULL}};
const Segment SGI351879[] = {{&S51021[39], 10}, {NULL}};
const Segment SGI351880[] = {{&S68611[28], 5}, {NULL}};
const Segment SGI351881[] = {{&S68611[104], 5}, {NULL}};
const Segment SGI351882[] = {{&S68612[28], 7}, {NULL}};
const Segment SGI351883[] = {{&S68612[106], 5}, {NULL}};
const Segment SGI351884[] = {{&S47685[208], 5}, {NULL}};
const Segment SGI351885[] = {{&S47685[145], 3}, {NULL}};
const Segment SGI351886[] = {{&S47687[310], 5}, {NULL}};
const Segment SGI351887[] = {{&S47687[15], 235}, {NULL}};
const Segment SGI351888[] = {{&S47689[309], 5}, {NULL}};
const Segment SGI351889[] = {{&S47689[15], 234}, {NULL}};
const Segment SGI351890[] = {{&S47690[216], 5}, {NULL}};
const Segment SGI351891[] = {{&S47690[75], 81}, {NULL}};
const Segment SGI351892[] = {{&S51060[6], 4}, {NULL}};
const Segment SGI351893[] = {{&S51060[70], 5}, {NULL}};
const Segment SGI351894[] = {{&S51060[12], 9}, {NULL}};
const Segment SGI351895[] = {{&S51060[29], 13}, {NULL}};
const Segment SGI351896[] = {{&S77321[5], 4}, {NULL}};
const Segment SGI351897[] = {{&S53539[35], 4}, {NULL}};
const Segment SGI351898[] = {{&S53539[14], 10}, {NULL}};
const Segment SGI351899[] = {{&S53542[53], 4}, {NULL}};
const Segment SGI351900[] = {{&S53542[13], 10}, {NULL}};
const Segment SGI351901[] = {{&S53544[53], 4}, {NULL}};
const Segment SGI351902[] = {{&S53544[13], 10}, {NULL}};
const Segment SGI351903[] = {{&S77381[5], 5}, {&S77381[97], 5}, {NULL}};
const Segment SGI351904[] = {{&S77381[113], 3}, {NULL}};
const Segment SGI351905[] = {{&S77397[5], 5}, {&S77397[97], 5}, {NULL}};
const Segment SGI351906[] = {{&S77397[113], 3}, {NULL}};
const Segment SGI351907[] = {{&S77398[21], 3}, {NULL}};
const Segment SGI351908[] = {{&S77398[0], 14}, {NULL}};
const Segment SGI351909[] = {{&S77420[5], 4}, {NULL}};
const Segment SGI351910[] = {{&S77420[15], 3}, {NULL}};
const Segment SGI351911[] = {{&S77421[34], 4}, {NULL}};
const Segment SGI351912[] = {{&S77421[49], 3}, {NULL}};
const Segment SGI351913[] = {{&S47691[309], 5}, {NULL}};
const Segment SGI351914[] = {{&S47691[15], 234}, {NULL}};
const Segment SGI351915[] = {{&S47692[309], 5}, {NULL}};
const Segment SGI351916[] = {{&S47692[15], 234}, {NULL}};
const Segment SGI351917[] = {{&S47693[342], 5}, {NULL}};
const Segment SGI351918[] = {{&S47693[15], 7}, {&S47693[30], 21}, {&S47693[279], 3}, {NULL}};
const Segment SGI351919[] = {{&S47742[74], 4}, {NULL}};
const Segment SGI351920[] = {{&S47742[12], 3}, {NULL}};
const Segment SGI351921[] = {{&S77509[5], 5}, {&S77509[105], 5}, {NULL}};
const Segment SGI351922[] = {{&S77509[69], 3}, {NULL}};
const Segment SGI351923[] = {{&S77509[131], 3}, {NULL}};
const Segment SGI351924[] = {{&S77533[35], 5}, {NULL}};
const Segment SGI351925[] = {{&S77533[52], 3}, {NULL}};
const Segment SGI351926[] = {{&S77539[5], 3}, {&S77539[5], 3}, {NULL}};
const Segment SGI351927[] = {{&S77539[109], 3}, {NULL}};
const Segment SGI351928[] = {{&S77591[5], 7}, {NULL}};
const Segment SGI351929[] = {{&S77591[23], 3}, {NULL}};
const Segment SGI351930[] = {{&S77592[5], 5}, {NULL}};
const Segment SGI351931[] = {{&S77592[21], 3}, {NULL}};
const Segment SGI351932[] = {{&S77595[38], 5}, {NULL}};
const Segment SGI351933[] = {{&S77595[55], 3}, {NULL}};
const Segment SGI351934[] = {{&S77596[5], 5}, {NULL}};
const Segment SGI351935[] = {{&S77596[16], 3}, {NULL}};
const Segment SGI351936[] = {{&S77599[5], 15}, {&S77599[107], 15}, {NULL}};
const Segment SGI351937[] = {{&S77599[133], 3}, {NULL}};
const Segment SGI351938[] = {{&S77623[5], 4}, {NULL}};
const Segment SGI351939[] = {{&S77625[5], 4}, {NULL}};
const Segment SGI351940[] = {{&S77647[5], 7}, {&S77647[5], 7}, {NULL}};
const Segment SGI351941[] = {{&S77647[13], 3}, {NULL}};
const Segment SGI351942[] = {{&S77647[134], 3}, {NULL}};
const Segment SGI351943[] = {{&S77648[5], 10}, {NULL}};
const Segment SGI351944[] = {{&S77648[21], 3}, {NULL}};
const Segment SGI351945[] = {{&S77675[5], 3}, {&S77675[5], 3}, {NULL}};
const Segment SGI351946[] = {{&S77675[67], 3}, {NULL}};
const Segment SGI351947[] = {{&S77675[127], 3}, {NULL}};
const Segment SGI351948[] = {{&S51253[6], 4}, {NULL}};
const Segment SGI351949[] = {{&S51253[12], 14}, {NULL}};
const Segment SGI351950[] = {{&S51254[6], 4}, {NULL}};
const Segment SGI351951[] = {{&S51254[12], 14}, {NULL}};
const Segment SGI351952[] = {{&S63973[5], 11}, {&S63973[39], 5}, {NULL}};
const Segment SGI351953[] = {{&S51255[6], 4}, {NULL}};
const Segment SGI351954[] = {{&S63973[30], 3}, {NULL}};
const Segment SGI351955[] = {{&S51255[12], 14}, {NULL}};
const Segment SGI351956[] = {{&S51256[6], 4}, {NULL}};
const Segment SGI351957[] = {{&S51256[12], 14}, {NULL}};
const Segment SGI351958[] = {{&S63974[5], 6}, {NULL}};
const Segment SGI351959[] = {{&S63974[12], 3}, {NULL}};
const Segment SGI351960[] = {{&S51257[6], 4}, {NULL}};
const Segment SGI351961[] = {{&S51257[12], 14}, {NULL}};
const Segment SGI351962[] = {{&S63975[5], 5}, {NULL}};
const Segment SGI351963[] = {{&S63975[11], 3}, {NULL}};
const Segment SGI351964[] = {{&S51258[6], 4}, {NULL}};
const Segment SGI351965[] = {{&S51258[12], 14}, {NULL}};
const Segment SGI351966[] = {{&S63976[58], 3}, {NULL}};
const Segment SGI351967[] = {{&S63976[49], 3}, {NULL}};
const Segment SGI351968[] = {{&S51259[6], 4}, {NULL}};
const Segment SGI351969[] = {{&S51259[12], 14}, {NULL}};
const Segment SGI351970[] = {{&S51260[6], 4}, {NULL}};
const Segment SGI351971[] = {{&S51260[12], 14}, {NULL}};
const Segment SGI351972[] = {{&S63998[5], 4}, {NULL}};
const Segment SGI351973[] = {{&S63998[49], 3}, {NULL}};
const Segment SGI351974[] = {{&S63998[103], 3}, {NULL}};
const Segment SGI351975[] = {{&S77745[49], 20}, {NULL}};
const Segment SGI351976[] = {{&S77746[49], 20}, {NULL}};
const Segment SGI351977[] = {{&S77749[5], 5}, {&S77749[105], 5}, {NULL}};
const Segment SGI351978[] = {{&S77749[11], 3}, {NULL}};
const Segment SGI351979[] = {{&S77749[131], 3}, {NULL}};
const Segment SGI351980[] = {{&S77762[5], 4}, {&S77762[5], 4}, {NULL}};
const Segment SGI351981[] = {{&S77762[82], 3}, {NULL}};
const Segment SGI351982[] = {{&S69164[28], 5}, {NULL}};
const Segment SGI351983[] = {{&S69164[104], 5}, {NULL}};
const Segment SGI351984[] = {{&S69165[28], 7}, {NULL}};
const Segment SGI351985[] = {{&S69165[106], 5}, {NULL}};
const Segment SGI351986[] = {{&S77772[21], 5}, {NULL}};
const Segment SGI351987[] = {{&S77772[0], 14}, {NULL}};
const Segment SGI351988[] = {{&S77830[5], 3}, {&S77830[5], 3}, {NULL}};
const Segment SGI351989[] = {{&S77830[109], 3}, {NULL}};
const Segment SGI351990[] = {{&S77854[43], 4}, {NULL}};
const Segment SGI351991[] = {{&S77854[6], 3}, {NULL}};
const Segment SGI351992[] = {{&S77854[68], 3}, {NULL}};
const Segment SGI351993[] = {{&S77858[5], 4}, {&S77858[5], 4}, {NULL}};
const Segment SGI351994[] = {{&S77858[68], 3}, {NULL}};
const Segment SGI351995[] = {{&S77858[129], 3}, {NULL}};
const Segment SGI351996[] = {{&S77862[5], 5}, {NULL}};
const Segment SGI351997[] = {{&S77867[5], 7}, {&S77867[5], 7}, {NULL}};
const Segment SGI351998[] = {{&S77867[19], 3}, {NULL}};
const Segment SGI351999[] = {{&S77868[5], 4}, {&S77868[5], 4}, {NULL}};
const Segment SGI352000[] = {{&S77868[16], 3}, {NULL}};
const Segment SGI352001[] = {{&S69269[28], 5}, {NULL}};
const Segment SGI352002[] = {{&S69269[104], 5}, {NULL}};
const Segment SGI352003[] = {{&S77887[5], 5}, {&S77887[104], 5}, {NULL}};
const Segment SGI352004[] = {{&S77887[11], 3}, {NULL}};
const Segment SGI352005[] = {{&S77887[130], 3}, {NULL}};
const Segment SGI352006[] = {{&S77962[5], 10}, {NULL}};
const Segment SGI352007[] = {{&S77962[21], 3}, {NULL}};
const Segment SGI352008[] = {{&S77976[5], 4}, {&S77976[5], 4}, {NULL}};
const Segment SGI352009[] = {{&S77976[10], 3}, {NULL}};
const Segment SGI352010[] = {{&S77976[128], 3}, {NULL}};
const Segment SGI352011[] = {{&S77989[43], 3}, {NULL}};
const Segment SGI352012[] = {{&S77989[6], 3}, {NULL}};
const Segment SGI352013[] = {{&S77989[67], 3}, {NULL}};
const Segment SGI352014[] = {{&S78001[5], 4}, {&S78001[5], 4}, {NULL}};
const Segment SGI352015[] = {{&S78001[68], 3}, {NULL}};
const Segment SGI352016[] = {{&S78001[129], 3}, {NULL}};
const Segment SGI352017[] = {{&S69315[28], 5}, {NULL}};
const Segment SGI352018[] = {{&S69315[104], 5}, {NULL}};
const Segment SGI352019[] = {{&S69316[28], 7}, {NULL}};
const Segment SGI352020[] = {{&S69316[106], 5}, {NULL}};
const Segment SGI352021[] = {{&S51702[6], 4}, {NULL}};
const Segment SGI352022[] = {{&S51702[70], 5}, {NULL}};
const Segment SGI352023[] = {{&S51702[12], 9}, {NULL}};
const Segment SGI352024[] = {{&S51702[29], 13}, {NULL}};
const Segment SGI352025[] = {{&S78040[5], 3}, {&S78040[5], 3}, {NULL}};
const Segment SGI352026[] = {{&S78040[9], 3}, {NULL}};
const Segment SGI352027[] = {{&S78040[127], 3}, {NULL}};
const Segment SGI352028[] = {{&S78063[5], 5}, {NULL}};
const Segment SGI352029[] = {{&S78064[5], 10}, {NULL}};
const Segment SGI352030[] = {{&S78072[5], 5}, {NULL}};
const Segment SGI352031[] = {{&S78072[21], 3}, {NULL}};
const Segment SGI352032[] = {{&S78075[35], 3}, {NULL}};
const Segment SGI352033[] = {{&S78075[50], 3}, {NULL}};
const Segment SGI352034[] = {{&S78076[5], 3}, {NULL}};
const Segment SGI352035[] = {{&S78076[14], 3}, {NULL}};
const Segment SGI352036[] = {{&S78092[5], 3}, {&S78092[5], 3}, {NULL}};
const Segment SGI352037[] = {{&S78092[149], 3}, {NULL}};
const Segment SGI352038[] = {{&S78093[5], 7}, {&S78093[5], 7}, {NULL}};
const Segment SGI352039[] = {{&S78093[13], 3}, {NULL}};
const Segment SGI352040[] = {{&S78093[135], 3}, {NULL}};
const Segment SGI352041[] = {{&S78095[43], 4}, {NULL}};
const Segment SGI352042[] = {{&S78095[6], 3}, {NULL}};
const Segment SGI352043[] = {{&S78095[68], 3}, {NULL}};
const Segment SGI352044[] = {{&S78100[45], 11}, {NULL}};
const Segment SGI352045[] = {{&S78100[6], 3}, {NULL}};
const Segment SGI352046[] = {{&S78100[77], 3}, {NULL}};
const Segment SGI352047[] = {{&S51770[6], 4}, {NULL}};
const Segment SGI352048[] = {{&S51770[73], 5}, {NULL}};
const Segment SGI352049[] = {{&S51770[12], 3}, {NULL}};
const Segment SGI352050[] = {{&S51770[37], 10}, {NULL}};
const Segment SGI352051[] = {{&S78108[43], 5}, {NULL}};
const Segment SGI352052[] = {{&S78108[6], 3}, {NULL}};
const Segment SGI352053[] = {{&S78108[69], 3}, {NULL}};
const Segment SGI352054[] = {{&S78111[5], 4}, {NULL}};
const Segment SGI352055[] = {{&S69410[28], 7}, {NULL}};
const Segment SGI352056[] = {{&S69410[107], 5}, {NULL}};
const Segment SGI352057[] = {{&S69411[28], 5}, {NULL}};
const Segment SGI352058[] = {{&S69411[104], 5}, {NULL}};
const Segment SGI352059[] = {{&S78128[5], 5}, {NULL}};
const Segment SGI352060[] = {{&S78128[16], 3}, {NULL}};
const Segment SGI352061[] = {{&S78129[35], 5}, {NULL}};
const Segment SGI352062[] = {{&S78129[52], 3}, {NULL}};
const Segment SGI352063[] = {{&S51937[0], 23}, {NULL}};
const Segment SGI352064[] = {{&S51937[35], 3}, {NULL}};
const Segment SGI352065[] = {{&S51915[6], 4}, {NULL}};
const Segment SGI352066[] = {{&S51915[12], 14}, {NULL}};
const Segment SGI352067[] = {{&S51916[6], 4}, {NULL}};
const Segment SGI352068[] = {{&S51916[12], 14}, {NULL}};
const Segment SGI352069[] = {{&S51917[6], 4}, {NULL}};
const Segment SGI352070[] = {{&S51917[12], 14}, {NULL}};
const Segment SGI352071[] = {{&S51919[6], 4}, {NULL}};
const Segment SGI352072[] = {{&S51919[12], 14}, {NULL}};
const Segment SGI352073[] = {{&S51920[6], 4}, {NULL}};
const Segment SGI352074[] = {{&S51920[12], 14}, {NULL}};
const Segment SGI352075[] = {{&S51921[6], 4}, {NULL}};
const Segment SGI352076[] = {{&S51921[12], 14}, {NULL}};
const Segment SGI352077[] = {{&S51922[6], 3}, {NULL}};
const Segment SGI352078[] = {{&S51922[11], 14}, {NULL}};
const Segment SGI352079[] = {{&S51923[6], 4}, {NULL}};
const Segment SGI352080[] = {{&S51923[12], 14}, {NULL}};
const Segment SGI352081[] = {{&S52003[6], 4}, {NULL}};
const Segment SGI352082[] = {{&S52003[70], 5}, {NULL}};
const Segment SGI352083[] = {{&S52003[12], 9}, {NULL}};
const Segment SGI352084[] = {{&S52003[29], 13}, {NULL}};
const Segment SGI352085[] = {{&S69659[28], 3}, {NULL}};
const Segment SGI352086[] = {{&S69659[102], 5}, {NULL}};
const Segment SGI352087[] = {{&S64738[5], 3}, {NULL}};
const Segment SGI352088[] = {{&S64738[37], 3}, {NULL}};
const Segment SGI352089[] = {{&S64739[49], 5}, {NULL}};
const Segment SGI352090[] = {{&S64739[40], 3}, {NULL}};
const Segment SGI352091[] = {{&S69766[28], 5}, {NULL}};
const Segment SGI352092[] = {{&S69766[104], 5}, {NULL}};
const Segment SGI352093[] = {{&S52194[6], 4}, {NULL}};
const Segment SGI352094[] = {{&S52194[77], 5}, {NULL}};
const Segment SGI352095[] = {{&S52194[12], 9}, {NULL}};
const Segment SGI352096[] = {{&S52194[29], 13}, {NULL}};
const Segment SGI352097[] = {{&S78234[35], 3}, {NULL}};
const Segment SGI352098[] = {{&S78234[50], 3}, {NULL}};
const Segment SGI352099[] = {{&S52212[6], 4}, {NULL}};
const Segment SGI352100[] = {{&S52212[12], 9}, {NULL}};
const Segment SGI352101[] = {{&S52213[6], 4}, {NULL}};
const Segment SGI352102[] = {{&S52213[12], 9}, {NULL}};
const Segment SGI352103[] = {{&S52214[6], 4}, {NULL}};
const Segment SGI352104[] = {{&S52214[12], 9}, {NULL}};
const Segment SGI352105[] = {{&S52215[6], 4}, {NULL}};
const Segment SGI352106[] = {{&S52215[12], 9}, {NULL}};
const Segment SGI352107[] = {{&S52216[6], 4}, {NULL}};
const Segment SGI352108[] = {{&S52216[12], 9}, {NULL}};
const Segment SGI352109[] = {{&S78263[43], 5}, {NULL}};
const Segment SGI352110[] = {{&S78263[6], 3}, {NULL}};
const Segment SGI352111[] = {{&S78263[69], 3}, {NULL}};
const Segment SGI352112[] = {{&S78299[5], 3}, {&S78299[5], 3}, {NULL}};
const Segment SGI352113[] = {{&S78299[149], 3}, {NULL}};
const Segment SGI352114[] = {{&S69895[28], 5}, {NULL}};
const Segment SGI352115[] = {{&S69895[104], 5}, {NULL}};
const Segment SGI352116[] = {{&S78309[44], 21}, {NULL}};
const Segment SGI352117[] = {{&S78309[6], 3}, {NULL}};
const Segment SGI352118[] = {{&S78309[86], 3}, {NULL}};
const Segment SGI352119[] = {{&S78310[44], 21}, {NULL}};
const Segment SGI352120[] = {{&S78310[6], 3}, {NULL}};
const Segment SGI352121[] = {{&S78310[86], 3}, {NULL}};
const Segment SGI352122[] = {{&S78311[44], 21}, {NULL}};
const Segment SGI352123[] = {{&S78311[6], 3}, {NULL}};
const Segment SGI352124[] = {{&S78311[86], 3}, {NULL}};
const Segment SGI352125[] = {{&S78312[44], 21}, {NULL}};
const Segment SGI352126[] = {{&S78312[6], 3}, {NULL}};
const Segment SGI352127[] = {{&S78312[86], 3}, {NULL}};
const Segment SGI352128[] = {{&S78322[5], 3}, {NULL}};
const Segment SGI352129[] = {{&S78323[5], 5}, {NULL}};
const Segment SGI352130[] = {{&S78323[20], 3}, {NULL}};
const Segment SGI352131[] = {{&S78324[5], 5}, {NULL}};
const Segment SGI352132[] = {{&S78329[5], 13}, {&S78329[105], 13}, {NULL}};
const Segment SGI352133[] = {{&S78329[129], 3}, {NULL}};
const Segment SGI352134[] = {{&S78330[5], 5}, {&S78330[5], 5}, {NULL}};
const Segment SGI352135[] = {{&S78330[113], 3}, {NULL}};
const Segment SGI352136[] = {{&S69990[28], 5}, {NULL}};
const Segment SGI352137[] = {{&S69990[104], 5}, {NULL}};
const Segment SGI352138[] = {{&S78394[21], 3}, {NULL}};
const Segment SGI352139[] = {{&S78394[0], 14}, {NULL}};
const Segment SGI352140[] = {{&S78395[5], 10}, {NULL}};
const Segment SGI352141[] = {{&S78395[21], 3}, {NULL}};
const Segment SGI352142[] = {{&S70010[28], 5}, {NULL}};
const Segment SGI352143[] = {{&S70010[104], 5}, {NULL}};
const Segment SGI352144[] = {{&S70016[28], 7}, {NULL}};
const Segment SGI352145[] = {{&S70016[106], 5}, {NULL}};
const Segment SGI352146[] = {{&S78431[5], 5}, {&S78431[97], 5}, {NULL}};
const Segment SGI352147[] = {{&S78431[113], 3}, {NULL}};
const Segment SGI352148[] = {{&S78553[44], 20}, {NULL}};
const Segment SGI352149[] = {{&S78553[6], 3}, {NULL}};
const Segment SGI352150[] = {{&S78553[85], 3}, {NULL}};
const Segment SGI352151[] = {{&S78554[44], 21}, {NULL}};
const Segment SGI352152[] = {{&S78554[6], 3}, {NULL}};
const Segment SGI352153[] = {{&S78554[86], 3}, {NULL}};
const Segment SGI352154[] = {{&S78556[5], 12}, {&S78556[5], 12}, {NULL}};
const Segment SGI352155[] = {{&S78556[76], 3}, {NULL}};
const Segment SGI352156[] = {{&S78556[145], 3}, {NULL}};
const Segment SGI352157[] = {{&S78574[5], 4}, {NULL}};
const Segment SGI352158[] = {{&S78574[20], 3}, {NULL}};
const Segment SGI352159[] = {{&S78575[35], 3}, {NULL}};
const Segment SGI352160[] = {{&S78575[50], 3}, {NULL}};
const Segment SGI352161[] = {{&S78576[5], 3}, {NULL}};
const Segment SGI352162[] = {{&S78576[14], 3}, {NULL}};
const Segment SGI352163[] = {{&S78577[34], 5}, {NULL}};
const Segment SGI352164[] = {{&S78577[50], 3}, {NULL}};
const Segment SGI352165[] = {{&S78580[5], 8}, {NULL}};
const Segment SGI352166[] = {{&S78580[19], 3}, {NULL}};
const Segment SGI352167[] = {{&S78581[35], 8}, {NULL}};
const Segment SGI352168[] = {{&S78581[55], 3}, {NULL}};
const Segment SGI352169[] = {{&S78590[34], 3}, {NULL}};
const Segment SGI352170[] = {{&S78590[48], 3}, {NULL}};
const Segment SGI352171[] = {{&S78619[5], 5}, {NULL}};
const Segment SGI352172[] = {{&S78619[21], 3}, {NULL}};
const Segment SGI352173[] = {{&S78642[38], 3}, {NULL}};
const Segment SGI352174[] = {{&S78642[92], 3}, {NULL}};
const Segment SGI352175[] = {{&S78647[42], 3}, {NULL}};
const Segment SGI352176[] = {{&S78647[6], 3}, {NULL}};
const Segment SGI352177[] = {{&S78647[66], 3}, {NULL}};
const Segment SGI352178[] = {{&S78747[5], 5}, {&S78747[105], 5}, {NULL}};
const Segment SGI352179[] = {{&S78747[11], 3}, {NULL}};
const Segment SGI352180[] = {{&S78747[131], 3}, {NULL}};
const Segment SGI352181[] = {{&S78748[43], 5}, {NULL}};
const Segment SGI352182[] = {{&S78748[6], 3}, {NULL}};
const Segment SGI352183[] = {{&S78748[69], 3}, {NULL}};
const Segment SGI352184[] = {{&S52782[6], 4}, {NULL}};
const Segment SGI352185[] = {{&S52782[58], 5}, {NULL}};
const Segment SGI352186[] = {{&S52782[12], 3}, {NULL}};
const Segment SGI352187[] = {{&S52783[6], 4}, {NULL}};
const Segment SGI352188[] = {{&S52783[58], 5}, {NULL}};
const Segment SGI352189[] = {{&S52783[12], 3}, {NULL}};
const Segment SGI352190[] = {{&S52784[6], 5}, {NULL}};
const Segment SGI352191[] = {{&S52784[13], 3}, {NULL}};
const Segment SGI352192[] = {{&S52786[6], 4}, {NULL}};
const Segment SGI352193[] = {{&S52786[12], 3}, {NULL}};
const Segment SGI352194[] = {{&S52802[6], 4}, {NULL}};
const Segment SGI352195[] = {{&S52802[57], 5}, {NULL}};
const Segment SGI352196[] = {{&S52802[12], 14}, {NULL}};
const Segment SGI352197[] = {{&S52803[6], 4}, {NULL}};
const Segment SGI352198[] = {{&S52803[79], 5}, {NULL}};
const Segment SGI352199[] = {{&S52803[12], 14}, {&S52803[40], 11}, {NULL}};
const Segment SGI352200[] = {{&S52803[12], 14}, {NULL}};
const Segment SGI352201[] = {{&S78780[5], 3}, {&S78780[5], 3}, {NULL}};
const Segment SGI352202[] = {{&S78780[149], 3}, {NULL}};
const Segment SGI352203[] = {{&S78795[43], 5}, {NULL}};
const Segment SGI352204[] = {{&S78795[6], 3}, {NULL}};
const Segment SGI352205[] = {{&S78795[69], 3}, {NULL}};
const Segment SGI352206[] = {{&S78796[43], 5}, {NULL}};
const Segment SGI352207[] = {{&S78796[6], 3}, {NULL}};
const Segment SGI352208[] = {{&S78796[69], 3}, {NULL}};
const Segment SGI352209[] = {{&S78804[5], 16}, {NULL}};
const Segment SGI352210[] = {{&S78804[32], 3}, {NULL}};
const Segment SGI352211[] = {{&S78805[5], 7}, {NULL}};
const Segment SGI352212[] = {{&S78805[23], 3}, {NULL}};
const Segment SGI352213[] = {{&S78807[5], 5}, {&S78807[68], 5}, {NULL}};
const Segment SGI352214[] = {{&S78807[84], 3}, {NULL}};
const Segment SGI352215[] = {{&S78866[5], 5}, {&S78866[68], 5}, {NULL}};
const Segment SGI352216[] = {{&S78866[84], 3}, {NULL}};
const Segment SGI352217[] = {{&S78867[34], 5}, {NULL}};
const Segment SGI352218[] = {{&S78867[50], 3}, {NULL}};
const Segment SGI352219[] = {{&S78868[5], 5}, {NULL}};
const Segment SGI352220[] = {{&S78868[16], 3}, {NULL}};
const Segment SGI352221[] = {{&S78869[5], 13}, {&S78869[76], 13}, {NULL}};
const Segment SGI352222[] = {{&S78869[100], 3}, {NULL}};
const Segment SGI352223[] = {{&S78874[35], 3}, {NULL}};
const Segment SGI352224[] = {{&S78874[50], 3}, {NULL}};
const Segment SGI352225[] = {{&S70630[28], 3}, {NULL}};
const Segment SGI352226[] = {{&S70630[102], 5}, {NULL}};
const Segment SGI352227[] = {{&S78910[5], 3}, {&S78910[5], 3}, {NULL}};
const Segment SGI352228[] = {{&S78910[9], 3}, {NULL}};
const Segment SGI352229[] = {{&S78910[127], 3}, {NULL}};
const Segment SGI352230[] = {{&S53004[6], 4}, {NULL}};
const Segment SGI352231[] = {{&S53004[12], 10}, {NULL}};
const Segment SGI352232[] = {{&S70709[28], 3}, {NULL}};
const Segment SGI352233[] = {{&S70709[102], 5}, {NULL}};
const Segment SGI352234[] = {{&S70771[28], 5}, {NULL}};
const Segment SGI352235[] = {{&S70771[104], 5}, {NULL}};
const Segment SGI352236[] = {{&S79033[5], 5}, {&S79033[97], 5}, {NULL}};
const Segment SGI352237[] = {{&S79033[153], 3}, {NULL}};
const Segment SGI352238[] = {{&S79034[5], 10}, {NULL}};
const Segment SGI352239[] = {{&S79034[21], 3}, {NULL}};
const Segment SGI352240[] = {{&S79072[5], 5}, {NULL}};
const Segment SGI352241[] = {{&S79086[5], 3}, {&S79086[5], 3}, {NULL}};
const Segment SGI352242[] = {{&S79086[109], 3}, {NULL}};
const Segment SGI352243[] = {{&S79093[21], 3}, {NULL}};
const Segment SGI352244[] = {{&S79093[0], 14}, {NULL}};
const Segment SGI352245[] = {{&S79122[53], 4}, {NULL}};
const Segment SGI352246[] = {{&S70900[28], 7}, {NULL}};
const Segment SGI352247[] = {{&S70900[106], 5}, {NULL}};
const Segment SGI352248[] = {{&S79150[5], 15}, {&S79150[107], 15}, {NULL}};
const Segment SGI352249[] = {{&S79150[173], 3}, {NULL}};
const Segment SGI352250[] = {{&S79155[42], 14}, {NULL}};
const Segment SGI352251[] = {{&S79155[6], 3}, {NULL}};
const Segment SGI352252[] = {{&S79155[77], 3}, {NULL}};
const Segment SGI352253[] = {{&S65746[5], 12}, {NULL}};
const Segment SGI352254[] = {{&S65746[60], 3}, {NULL}};
const Segment SGI352255[] = {{&S65747[5], 3}, {NULL}};
const Segment SGI352256[] = {{&S65747[9], 3}, {NULL}};
const Segment SGI352257[] = {{&S65748[5], 5}, {NULL}};
const Segment SGI352258[] = {{&S65748[11], 3}, {NULL}};
const Segment SGI352259[] = {{&S53292[6], 4}, {NULL}};
const Segment SGI352260[] = {{&S53292[60], 5}, {NULL}};
const Segment SGI352261[] = {{&S53292[12], 3}, {NULL}};
const Segment SGI352262[] = {{&S79205[35], 5}, {NULL}};
const Segment SGI352263[] = {{&S79205[91], 3}, {NULL}};
const Segment SGI352264[] = {{&S53303[6], 4}, {NULL}};
const Segment SGI352265[] = {{&S53303[58], 5}, {NULL}};
const Segment SGI352266[] = {{&S53303[12], 3}, {NULL}};
const Segment SGI352267[] = {{&S53304[6], 4}, {NULL}};
const Segment SGI352268[] = {{&S53304[104], 4}, {NULL}};
const Segment SGI352269[] = {{&S53304[12], 3}, {NULL}};
const Segment SGI352270[] = {{&S53306[6], 4}, {NULL}};
const Segment SGI352271[] = {{&S53306[73], 5}, {NULL}};
const Segment SGI352272[] = {{&S53306[12], 3}, {NULL}};
const Segment SGI352273[] = {{&S79209[5], 10}, {NULL}};
const Segment SGI352274[] = {{&S79209[21], 3}, {NULL}};
const Segment SGI352275[] = {{&S53310[6], 4}, {NULL}};
const Segment SGI352276[] = {{&S53310[70], 5}, {NULL}};
const Segment SGI352277[] = {{&S53310[12], 9}, {NULL}};
const Segment SGI352278[] = {{&S53310[29], 13}, {NULL}};
const Segment SGI352279[] = {{&S53326[0], 16}, {NULL}};
const Segment SGI352280[] = {{&S53326[29], 3}, {NULL}};
const Segment SGI352281[] = {{&S53326[71], 11}, {NULL}};
const Segment SGI352282[] = {{&S79273[35], 5}, {NULL}};
const Segment SGI352283[] = {{&S79273[52], 3}, {NULL}};
const Segment SGI352284[] = {{&S79274[34], 5}, {NULL}};
const Segment SGI352285[] = {{&S79274[50], 3}, {NULL}};
const Segment SGI352286[] = {{&S79285[34], 5}, {NULL}};
const Segment SGI352287[] = {{&S79285[90], 3}, {NULL}};
const Segment SGI352288[] = {{&S79290[42], 3}, {NULL}};
const Segment SGI352289[] = {{&S79290[6], 3}, {NULL}};
const Segment SGI352290[] = {{&S79290[66], 3}, {NULL}};
const Segment SGI352291[] = {{&S53388[6], 4}, {NULL}};
const Segment SGI352292[] = {{&S53388[80], 5}, {NULL}};
const Segment SGI352293[] = {{&S53388[12], 3}, {NULL}};
const Segment SGI352294[] = {{&S53388[40], 10}, {NULL}};
const Segment SGI352295[] = {{&S79300[5], 7}, {NULL}};
const Segment SGI352296[] = {{&S79306[5], 3}, {NULL}};
const Segment SGI352297[] = {{&S79306[19], 3}, {NULL}};
const Segment SGI352298[] = {{&S79335[43], 4}, {NULL}};
const Segment SGI352299[] = {{&S79335[6], 3}, {NULL}};
const Segment SGI352300[] = {{&S79335[68], 3}, {NULL}};
const Segment SGI352301[] = {{&S79336[42], 7}, {NULL}};
const Segment SGI352302[] = {{&S79336[6], 3}, {NULL}};
const Segment SGI352303[] = {{&S79336[70], 3}, {NULL}};
const Segment SGI352304[] = {{&S79338[44], 21}, {NULL}};
const Segment SGI352305[] = {{&S79338[6], 3}, {NULL}};
const Segment SGI352306[] = {{&S79338[86], 3}, {NULL}};
const Segment SGI352307[] = {{&S79339[44], 21}, {NULL}};
const Segment SGI352308[] = {{&S79339[6], 3}, {NULL}};
const Segment SGI352309[] = {{&S79339[86], 3}, {NULL}};
const Segment SGI352310[] = {{&S79366[5], 5}, {NULL}};
const Segment SGI352311[] = {{&S79366[21], 3}, {NULL}};
const Segment SGI352312[] = {{&S79371[5], 11}, {&S79371[5], 11}, {NULL}};
const Segment SGI352313[] = {{&S79371[125], 3}, {NULL}};
const Segment SGI352314[] = {{&S79394[43], 3}, {NULL}};
const Segment SGI352315[] = {{&S79394[6], 3}, {NULL}};
const Segment SGI352316[] = {{&S79394[67], 3}, {NULL}};
const Segment SGI352317[] = {{&S79406[5], 4}, {NULL}};
const Segment SGI352318[] = {{&S53540[6], 4}, {NULL}};
const Segment SGI352319[] = {{&S53540[12], 10}, {NULL}};
const Segment SGI352320[] = {{&S53541[6], 4}, {NULL}};
const Segment SGI352321[] = {{&S53541[12], 10}, {NULL}};
const Segment SGI352322[] = {{&S53543[6], 4}, {NULL}};
const Segment SGI352323[] = {{&S53543[12], 10}, {NULL}};
const Segment SGI352324[] = {{&S79440[42], 5}, {NULL}};
const Segment SGI352325[] = {{&S79440[6], 3}, {NULL}};
const Segment SGI352326[] = {{&S79440[68], 3}, {NULL}};
const Segment SGI352327[] = {{&S79462[43], 5}, {NULL}};
const Segment SGI352328[] = {{&S79462[6], 3}, {NULL}};
const Segment SGI352329[] = {{&S79462[69], 3}, {NULL}};
const Segment SGI352330[] = {{&S79472[5], 5}, {&S79472[68], 5}, {NULL}};
const Segment SGI352331[] = {{&S79472[84], 3}, {NULL}};
const Segment SGI352332[] = {{&S56032[61], 4}, {NULL}};
const Segment SGI352333[] = {{&S56032[13], 10}, {NULL}};
const Segment SGI352334[] = {{&S79490[5], 4}, {&S79490[5], 4}, {NULL}};
const Segment SGI352335[] = {{&S79490[151], 3}, {NULL}};
const Segment SGI352336[] = {{&S79524[5], 3}, {NULL}};
const Segment SGI352337[] = {{&S79524[19], 3}, {NULL}};
const Segment SGI352338[] = {{&S79525[5], 5}, {NULL}};
const Segment SGI352339[] = {{&S79525[21], 3}, {NULL}};
const Segment SGI352340[] = {{&S79530[5], 5}, {&S79530[97], 5}, {NULL}};
const Segment SGI352341[] = {{&S79530[113], 3}, {NULL}};
const Segment SGI352342[] = {{&S79531[35], 3}, {NULL}};
const Segment SGI352343[] = {{&S79531[50], 3}, {NULL}};
const Segment SGI352344[] = {{&S79532[5], 3}, {NULL}};
const Segment SGI352345[] = {{&S79532[14], 3}, {NULL}};
const Segment SGI352346[] = {{&S79544[35], 7}, {NULL}};
const Segment SGI352347[] = {{&S79544[93], 3}, {NULL}};
const Segment SGI352348[] = {{&S53657[6], 4}, {NULL}};
const Segment SGI352349[] = {{&S53657[111], 5}, {NULL}};
const Segment SGI352350[] = {{&S53657[12], 11}, {NULL}};
const Segment SGI352351[] = {{&S53658[6], 4}, {NULL}};
const Segment SGI352352[] = {{&S53658[112], 5}, {NULL}};
const Segment SGI352353[] = {{&S53658[12], 10}, {NULL}};
const Segment SGI352354[] = {{&S79561[5], 7}, {&S79561[5], 7}, {NULL}};
const Segment SGI352355[] = {{&S79561[13], 3}, {NULL}};
const Segment SGI352356[] = {{&S79561[135], 3}, {NULL}};
const Segment SGI352357[] = {{&S79570[43], 5}, {NULL}};
const Segment SGI352358[] = {{&S79570[6], 3}, {NULL}};
const Segment SGI352359[] = {{&S79570[69], 3}, {NULL}};
const Segment SGI352360[] = {{&S71235[28], 3}, {NULL}};
const Segment SGI352361[] = {{&S71235[102], 5}, {NULL}};
const Segment SGI352362[] = {{&S71236[28], 3}, {NULL}};
const Segment SGI352363[] = {{&S71236[102], 5}, {NULL}};
const Segment SGI352364[] = {{&S53708[0], 21}, {NULL}};
const Segment SGI352365[] = {{&S53708[34], 3}, {NULL}};
const Segment SGI352366[] = {{&S79592[35], 5}, {NULL}};
const Segment SGI352367[] = {{&S79592[52], 3}, {NULL}};
const Segment SGI352368[] = {{&S79593[34], 5}, {NULL}};
const Segment SGI352369[] = {{&S79593[50], 3}, {NULL}};
const Segment SGI352370[] = {{&S53722[0], 16}, {NULL}};
const Segment SGI352371[] = {{&S53722[29], 3}, {NULL}};
const Segment SGI352372[] = {{&S53722[70], 13}, {NULL}};
const Segment SGI352373[] = {{&S79613[43], 5}, {NULL}};
const Segment SGI352374[] = {{&S79613[6], 3}, {NULL}};
const Segment SGI352375[] = {{&S79613[69], 3}, {NULL}};
const Segment SGI352376[] = {{&S71297[28], 3}, {NULL}};
const Segment SGI352377[] = {{&S71297[102], 5}, {NULL}};
const Segment SGI352378[] = {{&S53760[6], 4}, {NULL}};
const Segment SGI352379[] = {{&S53760[60], 5}, {NULL}};
const Segment SGI352380[] = {{&S53760[12], 3}, {NULL}};
const Segment SGI352381[] = {{&S71323[28], 5}, {NULL}};
const Segment SGI352382[] = {{&S71323[104], 5}, {NULL}};
const Segment SGI352383[] = {{&S79666[43], 3}, {NULL}};
const Segment SGI352384[] = {{&S79666[6], 3}, {NULL}};
const Segment SGI352385[] = {{&S79666[67], 3}, {NULL}};
const Segment SGI352386[] = {{&S53788[6], 4}, {NULL}};
const Segment SGI352387[] = {{&S53788[57], 5}, {NULL}};
const Segment SGI352388[] = {{&S53788[12], 14}, {NULL}};
const Segment SGI352389[] = {{&S53789[6], 4}, {NULL}};
const Segment SGI352390[] = {{&S53789[57], 5}, {NULL}};
const Segment SGI352391[] = {{&S53789[12], 14}, {NULL}};
const Segment SGI352392[] = {{&S53790[6], 4}, {NULL}};
const Segment SGI352393[] = {{&S53790[57], 5}, {NULL}};
const Segment SGI352394[] = {{&S53790[12], 14}, {NULL}};
const Segment SGI352395[] = {{&S53791[6], 4}, {NULL}};
const Segment SGI352396[] = {{&S53791[57], 5}, {NULL}};
const Segment SGI352397[] = {{&S53791[12], 14}, {NULL}};
const Segment SGI352398[] = {{&S79686[5], 5}, {&S79686[68], 5}, {NULL}};
const Segment SGI352399[] = {{&S79686[84], 3}, {NULL}};
const Segment SGI352400[] = {{&S79730[43], 5}, {NULL}};
const Segment SGI352401[] = {{&S79730[6], 3}, {NULL}};
const Segment SGI352402[] = {{&S79730[69], 3}, {NULL}};
const Segment SGI352403[] = {{&S79747[5], 4}, {NULL}};
const Segment SGI352404[] = {{&S79759[5], 5}, {&S79759[5], 5}, {NULL}};
const Segment SGI352405[] = {{&S79759[84], 3}, {NULL}};
const Segment SGI352406[] = {{&S79760[5], 5}, {&S79760[68], 5}, {NULL}};
const Segment SGI352407[] = {{&S79760[84], 3}, {NULL}};
const Segment SGI352408[] = {{&S79762[5], 7}, {&S79762[5], 7}, {NULL}};
const Segment SGI352409[] = {{&S79762[117], 3}, {NULL}};
const Segment SGI352410[] = {{&S79774[42], 3}, {NULL}};
const Segment SGI352411[] = {{&S79774[6], 3}, {NULL}};
const Segment SGI352412[] = {{&S79774[66], 3}, {NULL}};
const Segment SGI352413[] = {{&S53897[6], 4}, {NULL}};
const Segment SGI352414[] = {{&S53897[71], 5}, {NULL}};
const Segment SGI352415[] = {{&S53897[12], 9}, {NULL}};
const Segment SGI352416[] = {{&S53898[6], 4}, {NULL}};
const Segment SGI352417[] = {{&S53898[66], 5}, {NULL}};
const Segment SGI352418[] = {{&S53898[12], 9}, {NULL}};
const Segment SGI352419[] = {{&S53899[6], 4}, {NULL}};
const Segment SGI352420[] = {{&S53899[68], 5}, {NULL}};
const Segment SGI352421[] = {{&S53899[12], 9}, {NULL}};
const Segment SGI352422[] = {{&S79808[43], 3}, {NULL}};
const Segment SGI352423[] = {{&S79808[6], 3}, {NULL}};
const Segment SGI352424[] = {{&S79808[67], 3}, {NULL}};
const Segment SGI352425[] = {{&S66346[5], 5}, {&S66346[5], 5}, {NULL}};
const Segment SGI352426[] = {{&S66346[43], 3}, {&S66346[43], 3}, {NULL}};
const Segment SGI352427[] = {{&S66347[5], 3}, {NULL}};
const Segment SGI352428[] = {{&S66347[41], 3}, {NULL}};
const Segment SGI352429[] = {{&S79877[5], 4}, {NULL}};
const Segment SGI352430[] = {{&S66349[5], 5}, {NULL}};
const Segment SGI352431[] = {{&S66349[43], 3}, {NULL}};
const Segment SGI352432[] = {{&S79882[5], 5}, {NULL}};
const Segment SGI352433[] = {{&S79882[21], 3}, {NULL}};
const Segment SGI352434[] = {{&S79886[5], 5}, {NULL}};
const Segment SGI352435[] = {{&S79886[21], 3}, {NULL}};
const Segment SGI352436[] = {{&S79887[5], 7}, {NULL}};
const Segment SGI352437[] = {{&S79887[23], 3}, {NULL}};
const Segment SGI352438[] = {{&S79889[35], 8}, {NULL}};
const Segment SGI352439[] = {{&S79889[55], 3}, {NULL}};
const Segment SGI352440[] = {{&S79890[5], 8}, {NULL}};
const Segment SGI352441[] = {{&S79890[19], 3}, {NULL}};
const Segment SGI352442[] = {{&S79905[5], 3}, {&S79905[5], 3}, {NULL}};
const Segment SGI352443[] = {{&S79905[9], 3}, {NULL}};
const Segment SGI352444[] = {{&S79905[126], 3}, {NULL}};
const Segment SGI352445[] = {{&S79943[35], 4}, {NULL}};
const Segment SGI352446[] = {{&S79943[51], 3}, {NULL}};
const Segment SGI352447[] = {{&S79944[5], 4}, {NULL}};
const Segment SGI352448[] = {{&S79944[15], 3}, {NULL}};
const Segment SGI352449[] = {{&S79947[5], 12}, {&S79947[5], 12}, {NULL}};
const Segment SGI352450[] = {{&S79947[167], 3}, {NULL}};
const Segment SGI352451[] = {{&S54054[0], 16}, {NULL}};
const Segment SGI352452[] = {{&S54054[29], 3}, {NULL}};
const Segment SGI352453[] = {{&S54054[71], 16}, {NULL}};
const Segment SGI352454[] = {{&S53960[6], 4}, {NULL}};
const Segment SGI352455[] = {{&S53960[70], 5}, {NULL}};
const Segment SGI352456[] = {{&S53960[12], 9}, {NULL}};
const Segment SGI352457[] = {{&S53960[29], 13}, {NULL}};
const Segment SGI352458[] = {{&S53969[6], 4}, {NULL}};
const Segment SGI352459[] = {{&S53969[68], 5}, {NULL}};
const Segment SGI352460[] = {{&S53969[12], 9}, {NULL}};
const Segment SGI352461[] = {{&S53970[6], 4}, {NULL}};
const Segment SGI352462[] = {{&S53970[66], 5}, {NULL}};
const Segment SGI352463[] = {{&S53970[12], 9}, {NULL}};
const Segment SGI352464[] = {{&S53972[6], 5}, {NULL}};
const Segment SGI352465[] = {{&S53972[13], 9}, {NULL}};
const Segment SGI352466[] = {{&S53986[6], 4}, {NULL}};
const Segment SGI352467[] = {{&S53986[66], 5}, {NULL}};
const Segment SGI352468[] = {{&S53986[12], 9}, {NULL}};
const Segment SGI352469[] = {{&S79999[5], 3}, {&S79999[5], 3}, {NULL}};
const Segment SGI352470[] = {{&S79999[109], 3}, {NULL}};
const Segment SGI352471[] = {{&S80000[5], 5}, {NULL}};
const Segment SGI352472[] = {{&S80000[16], 3}, {NULL}};
const Segment SGI352473[] = {{&S80001[34], 5}, {NULL}};
const Segment SGI352474[] = {{&S80001[50], 3}, {NULL}};
const Segment SGI352475[] = {{&S80034[5], 5}, {NULL}};
const Segment SGI352476[] = {{&S80050[5], 5}, {&S80050[68], 5}, {NULL}};
const Segment SGI352477[] = {{&S80050[84], 3}, {NULL}};
const Segment SGI352478[] = {{&S80052[5], 5}, {NULL}};
const Segment SGI352479[] = {{&S80052[21], 3}, {NULL}};
const Segment SGI352480[] = {{&S80053[5], 3}, {NULL}};
const Segment SGI352481[] = {{&S80053[19], 3}, {NULL}};
const Segment SGI352482[] = {{&S80070[5], 3}, {&S80070[5], 3}, {NULL}};
const Segment SGI352483[] = {{&S80070[109], 3}, {NULL}};
const Segment SGI352484[] = {{&S80073[5], 5}, {&S80073[97], 5}, {NULL}};
const Segment SGI352485[] = {{&S80073[113], 3}, {NULL}};
const Segment SGI352486[] = {{&S80083[5], 4}, {NULL}};
const Segment SGI352487[] = {{&S80087[42], 4}, {NULL}};
const Segment SGI352488[] = {{&S80087[6], 3}, {NULL}};
const Segment SGI352489[] = {{&S80087[67], 3}, {NULL}};
const Segment SGI352490[] = {{&S80088[42], 3}, {NULL}};
const Segment SGI352491[] = {{&S80088[6], 3}, {NULL}};
const Segment SGI352492[] = {{&S80088[66], 3}, {NULL}};
const Segment SGI352493[] = {{&S54143[6], 4}, {NULL}};
const Segment SGI352494[] = {{&S54143[70], 5}, {NULL}};
const Segment SGI352495[] = {{&S54143[12], 9}, {NULL}};
const Segment SGI352496[] = {{&S54143[29], 13}, {NULL}};
const Segment SGI352497[] = {{&S54239[6], 4}, {NULL}};
const Segment SGI352498[] = {{&S54239[12], 14}, {NULL}};
const Segment SGI352499[] = {{&S54240[6], 4}, {NULL}};
const Segment SGI352500[] = {{&S54240[12], 14}, {NULL}};
const Segment SGI352501[] = {{&S54241[6], 4}, {NULL}};
const Segment SGI352502[] = {{&S54241[12], 14}, {NULL}};
const Segment SGI352503[] = {{&S80111[5], 3}, {&S80111[5], 3}, {NULL}};
const Segment SGI352504[] = {{&S80111[9], 3}, {NULL}};
const Segment SGI352505[] = {{&S80111[127], 3}, {NULL}};
const Segment SGI352506[] = {{&S80119[5], 4}, {NULL}};
const Segment SGI352507[] = {{&S80119[20], 3}, {NULL}};
const Segment SGI352508[] = {{&S80120[5], 6}, {NULL}};
const Segment SGI352509[] = {{&S80120[22], 3}, {NULL}};
const Segment SGI352510[] = {{&S54407[6], 4}, {NULL}};
const Segment SGI352511[] = {{&S54407[73], 5}, {NULL}};
const Segment SGI352512[] = {{&S54407[12], 3}, {NULL}};
const Segment SGI352513[] = {{&S54407[37], 10}, {NULL}};
const Segment SGI352514[] = {{&S66888[5], 4}, {NULL}};
const Segment SGI352515[] = {{&S66888[47], 3}, {NULL}};
const Segment SGI352516[] = {{&S66908[46], 5}, {NULL}};
const Segment SGI352517[] = {{&S66908[37], 3}, {NULL}};
const Segment SGI352518[] = {{&S66933[5], 4}, {NULL}};
const Segment SGI352519[] = {{&S66933[47], 3}, {NULL}};
const Segment SGI352520[] = {{&S66933[101], 3}, {NULL}};
const Segment SGI352521[] = {{&S80139[5], 4}, {NULL}};
const Segment SGI352522[] = {{&S80139[20], 3}, {NULL}};
const Segment SGI352523[] = {{&S80144[35], 3}, {NULL}};
const Segment SGI352524[] = {{&S80144[50], 3}, {NULL}};
const Segment SGI352525[] = {{&S80202[5], 3}, {NULL}};
const Segment SGI352526[] = {{&S80202[19], 3}, {NULL}};
const Segment SGI352527[] = {{&S80213[35], 5}, {NULL}};
const Segment SGI352528[] = {{&S80213[52], 3}, {NULL}};
const Segment SGI352529[] = {{&S80251[44], 21}, {NULL}};
const Segment SGI352530[] = {{&S80251[6], 3}, {NULL}};
const Segment SGI352531[] = {{&S80251[86], 3}, {NULL}};
const Segment SGI352532[] = {{&S80252[44], 20}, {NULL}};
const Segment SGI352533[] = {{&S80252[6], 3}, {NULL}};
const Segment SGI352534[] = {{&S80252[85], 3}, {NULL}};
const Segment SGI352535[] = {{&S80253[44], 21}, {NULL}};
const Segment SGI352536[] = {{&S80253[6], 3}, {NULL}};
const Segment SGI352537[] = {{&S80253[86], 3}, {NULL}};
const Segment SGI352538[] = {{&S80301[5], 3}, {NULL}};
const Segment SGI352539[] = {{&S80388[43], 5}, {NULL}};
const Segment SGI352540[] = {{&S80388[6], 3}, {NULL}};
const Segment SGI352541[] = {{&S80388[69], 3}, {NULL}};
const Segment SGI352542[] = {{&S72390[28], 5}, {NULL}};
const Segment SGI352543[] = {{&S72390[104], 5}, {NULL}};
const Segment SGI352544[] = {{&S80420[5], 10}, {NULL}};
const Segment SGI352545[] = {{&S80420[21], 3}, {NULL}};
const Segment SGI352546[] = {{&S67259[5], 8}, {NULL}};
const Segment SGI352547[] = {{&S67259[41], 3}, {NULL}};
const Segment SGI352548[] = {{&S67259[95], 3}, {NULL}};
const Segment SGI352549[] = {{&S80434[43], 7}, {NULL}};
const Segment SGI352550[] = {{&S80434[6], 3}, {NULL}};
const Segment SGI352551[] = {{&S80434[71], 3}, {NULL}};
const Segment SGI352552[] = {{&S72426[28], 5}, {NULL}};
const Segment SGI352553[] = {{&S72426[104], 5}, {NULL}};
const Segment SGI352554[] = {{&S80453[5], 4}, {NULL}};
const Segment SGI352555[] = {{&S80454[5], 10}, {NULL}};
const Segment SGI352556[] = {{&S80455[5], 5}, {NULL}};
const Segment SGI352557[] = {{&S80456[5], 5}, {&S80456[5], 5}, {NULL}};
const Segment SGI352558[] = {{&S80459[5], 4}, {NULL}};
const Segment SGI352559[] = {{&S80459[20], 3}, {NULL}};
const Segment SGI352560[] = {{&S80476[5], 5}, {&S80476[5], 5}, {NULL}};
const Segment SGI352561[] = {{&S80476[153], 3}, {NULL}};
const Segment SGI352562[] = {{&S72476[28], 5}, {NULL}};
const Segment SGI352563[] = {{&S72476[104], 5}, {NULL}};
const Segment SGI352564[] = {{&S54864[6], 4}, {NULL}};
const Segment SGI352565[] = {{&S54864[12], 9}, {NULL}};
const Segment SGI352566[] = {{&S54865[6], 4}, {NULL}};
const Segment SGI352567[] = {{&S54865[12], 9}, {NULL}};
const Segment SGI352568[] = {{&S54866[6], 4}, {NULL}};
const Segment SGI352569[] = {{&S54866[12], 9}, {NULL}};
const Segment SGI352570[] = {{&S80506[5], 10}, {NULL}};
const Segment SGI352571[] = {{&S80507[5], 5}, {NULL}};
const Segment SGI352572[] = {{&S80525[5], 10}, {NULL}};
const Segment SGI352573[] = {{&S80525[21], 3}, {NULL}};
const Segment SGI352574[] = {{&S72513[28], 4}, {NULL}};
const Segment SGI352575[] = {{&S72513[103], 5}, {NULL}};
const Segment SGI352576[] = {{&S72514[28], 7}, {NULL}};
const Segment SGI352577[] = {{&S72514[106], 5}, {NULL}};
const Segment SGI352578[] = {{&S80543[5], 5}, {&S80543[5], 5}, {NULL}};
const Segment SGI352579[] = {{&S80543[11], 3}, {NULL}};
const Segment SGI352580[] = {{&S80543[131], 3}, {NULL}};
const Segment SGI352581[] = {{&S80544[5], 11}, {&S80544[5], 11}, {NULL}};
const Segment SGI352582[] = {{&S80544[165], 3}, {NULL}};
const Segment SGI352583[] = {{&S80561[5], 3}, {&S80561[5], 3}, {NULL}};
const Segment SGI352584[] = {{&S80561[9], 3}, {NULL}};
const Segment SGI352585[] = {{&S80561[127], 3}, {NULL}};
const Segment SGI352586[] = {{&S80595[35], 5}, {NULL}};
const Segment SGI352587[] = {{&S80595[52], 3}, {NULL}};
const Segment SGI352588[] = {{&S80596[5], 5}, {NULL}};
const Segment SGI352589[] = {{&S80596[16], 3}, {NULL}};
const Segment SGI352590[] = {{&S80597[34], 5}, {NULL}};
const Segment SGI352591[] = {{&S80597[50], 3}, {NULL}};
const Segment SGI352592[] = {{&S80598[35], 5}, {NULL}};
const Segment SGI352593[] = {{&S80598[52], 3}, {NULL}};
const Segment SGI352594[] = {{&S80606[5], 4}, {&S80606[5], 4}, {NULL}};
const Segment SGI352595[] = {{&S80606[111], 3}, {NULL}};
const Segment SGI352596[] = {{&S80607[35], 5}, {NULL}};
const Segment SGI352597[] = {{&S80607[91], 3}, {NULL}};
const Segment SGI352598[] = {{&S80616[42], 5}, {NULL}};
const Segment SGI352599[] = {{&S80616[6], 3}, {NULL}};
const Segment SGI352600[] = {{&S80616[68], 3}, {NULL}};
const Segment SGI352601[] = {{&S80643[42], 3}, {NULL}};
const Segment SGI352602[] = {{&S80643[6], 3}, {NULL}};
const Segment SGI352603[] = {{&S80643[66], 3}, {NULL}};
const Segment SGI352604[] = {{&S80644[43], 3}, {NULL}};
const Segment SGI352605[] = {{&S80644[6], 3}, {NULL}};
const Segment SGI352606[] = {{&S80644[67], 3}, {NULL}};
const Segment SGI352607[] = {{&S80645[43], 7}, {NULL}};
const Segment SGI352608[] = {{&S80645[6], 3}, {NULL}};
const Segment SGI352609[] = {{&S80645[71], 3}, {NULL}};
const Segment SGI352610[] = {{&S80699[43], 5}, {NULL}};
const Segment SGI352611[] = {{&S80699[6], 3}, {NULL}};
const Segment SGI352612[] = {{&S80699[69], 3}, {NULL}};
const Segment SGI352613[] = {{&S72745[28], 3}, {NULL}};
const Segment SGI352614[] = {{&S72745[102], 5}, {NULL}};
const Segment SGI352615[] = {{&S80751[34], 3}, {NULL}};
const Segment SGI352616[] = {{&S80751[48], 3}, {NULL}};
const Segment SGI352617[] = {{&S80790[42], 5}, {NULL}};
const Segment SGI352618[] = {{&S80790[6], 3}, {NULL}};
const Segment SGI352619[] = {{&S80790[68], 3}, {NULL}};
const Segment SGI352620[] = {{&S80808[5], 3}, {NULL}};
const Segment SGI352621[] = {{&S80808[19], 3}, {NULL}};
const Segment SGI352622[] = {{&S80809[5], 3}, {NULL}};
const Segment SGI352623[] = {{&S80809[14], 3}, {NULL}};
const Segment SGI352624[] = {{&S67741[55], 4}, {NULL}};
const Segment SGI352625[] = {{&S67741[46], 3}, {NULL}};
const Segment SGI352626[] = {{&S67742[46], 4}, {&S67742[142], 7}, {NULL}};
const Segment SGI352627[] = {{&S67742[37], 3}, {NULL}};
const Segment SGI352628[] = {{&S67743[49], 4}, {NULL}};
const Segment SGI352629[] = {{&S67743[40], 3}, {NULL}};
const Segment SGI352630[] = {{&S67744[5], 4}, {NULL}};
const Segment SGI352631[] = {{&S67744[45], 3}, {NULL}};
const Segment SGI352632[] = {{&S73012[28], 3}, {NULL}};
const Segment SGI352633[] = {{&S73012[102], 5}, {NULL}};
const Segment SGI352634[] = {{&S80884[5], 5}, {&S80884[68], 5}, {NULL}};
const Segment SGI352635[] = {{&S80884[84], 3}, {NULL}};
const Segment SGI352636[] = {{&S80891[34], 5}, {NULL}};
const Segment SGI352637[] = {{&S80891[50], 3}, {NULL}};
const Segment SGI352638[] = {{&S80899[35], 14}, {NULL}};
const Segment SGI352639[] = {{&S80899[100], 3}, {NULL}};
const Segment SGI352640[] = {{&S51908[77], 5}, {NULL}};
const Segment SGI352641[] = {{&S51908[12], 3}, {NULL}};
const Segment SGI352642[] = {{&S80954[5], 4}, {&S80954[5], 4}, {NULL}};
const Segment SGI352643[] = {{&S80954[111], 3}, {NULL}};
const Segment SGI352644[] = {{&S80958[42], 4}, {NULL}};
const Segment SGI352645[] = {{&S80958[6], 3}, {NULL}};
const Segment SGI352646[] = {{&S80958[67], 3}, {NULL}};
const Segment SGI352647[] = {{&S80965[43], 3}, {NULL}};
const Segment SGI352648[] = {{&S80965[6], 3}, {NULL}};
const Segment SGI352649[] = {{&S80965[67], 3}, {NULL}};
const Segment SGI352650[] = {{&S80975[43], 4}, {NULL}};
const Segment SGI352651[] = {{&S80975[6], 3}, {NULL}};
const Segment SGI352652[] = {{&S80975[68], 3}, {NULL}};
const Segment SGI352653[] = {{&S81010[35], 4}, {NULL}};
const Segment SGI352654[] = {{&S81010[90], 3}, {NULL}};
const Segment SGI352655[] = {{&S81014[35], 3}, {NULL}};
const Segment SGI352656[] = {{&S81014[89], 3}, {NULL}};
const Segment SGI352657[] = {{&S81016[5], 12}, {&S81016[5], 12}, {NULL}};
const Segment SGI352658[] = {{&S81016[76], 3}, {NULL}};
const Segment SGI352659[] = {{&S81016[144], 3}, {NULL}};
const Segment SGI352660[] = {{&S81044[42], 7}, {NULL}};
const Segment SGI352661[] = {{&S81044[6], 3}, {NULL}};
const Segment SGI352662[] = {{&S81044[70], 3}, {NULL}};
const Segment SGI352663[] = {{&S81045[42], 5}, {NULL}};
const Segment SGI352664[] = {{&S81045[6], 3}, {NULL}};
const Segment SGI352665[] = {{&S81045[68], 3}, {NULL}};
const Segment SGI352666[] = {{&S81054[5], 5}, {NULL}};
const Segment SGI352667[] = {{&S81080[35], 5}, {NULL}};
const Segment SGI352668[] = {{&S81080[91], 3}, {NULL}};
const Segment SGI352669[] = {{&S55501[6], 3}, {NULL}};
const Segment SGI352670[] = {{&S55501[11], 14}, {NULL}};
const Segment SGI352671[] = {{&S81122[5], 5}, {NULL}};
const Segment SGI352672[] = {{&S81122[16], 3}, {NULL}};
const Segment SGI352673[] = {{&S81123[5], 5}, {NULL}};
const Segment SGI352674[] = {{&S81123[21], 3}, {NULL}};
const Segment SGI352675[] = {{&S81130[34], 5}, {NULL}};
const Segment SGI352676[] = {{&S81130[50], 3}, {NULL}};
const Segment SGI352677[] = {{&S81131[35], 3}, {NULL}};
const Segment SGI352678[] = {{&S81131[50], 3}, {NULL}};
const Segment SGI352679[] = {{&S81132[5], 3}, {NULL}};
const Segment SGI352680[] = {{&S81132[14], 3}, {NULL}};
const Segment SGI352681[] = {{&S55603[6], 4}, {NULL}};
const Segment SGI352682[] = {{&S55603[53], 5}, {NULL}};
const Segment SGI352683[] = {{&S55603[12], 3}, {NULL}};
const Segment SGI352684[] = {{&S81259[5], 3}, {&S81259[5], 3}, {NULL}};
const Segment SGI352685[] = {{&S81259[109], 3}, {NULL}};
const Segment SGI352686[] = {{&S55699[6], 4}, {NULL}};
const Segment SGI352687[] = {{&S55699[70], 5}, {NULL}};
const Segment SGI352688[] = {{&S55699[12], 9}, {NULL}};
const Segment SGI352689[] = {{&S55699[29], 13}, {NULL}};
const Segment SGI352690[] = {{&S81276[5], 5}, {&S81276[104], 5}, {NULL}};
const Segment SGI352691[] = {{&S81276[11], 3}, {NULL}};
const Segment SGI352692[] = {{&S81276[130], 3}, {NULL}};
const Segment SGI352693[] = {{&S81278[42], 7}, {NULL}};
const Segment SGI352694[] = {{&S81278[6], 3}, {NULL}};
const Segment SGI352695[] = {{&S81278[70], 3}, {NULL}};
const Segment SGI352696[] = {{&S81281[21], 5}, {NULL}};
const Segment SGI352697[] = {{&S81281[0], 14}, {NULL}};
const Segment SGI352698[] = {{&S81286[45], 11}, {NULL}};
const Segment SGI352699[] = {{&S81286[6], 3}, {NULL}};
const Segment SGI352700[] = {{&S81286[77], 3}, {NULL}};
const Segment SGI352701[] = {{&S81301[5], 5}, {&S81301[105], 5}, {NULL}};
const Segment SGI352702[] = {{&S81301[69], 3}, {NULL}};
const Segment SGI352703[] = {{&S81301[131], 3}, {NULL}};
const Segment SGI352704[] = {{&S81317[5], 12}, {&S81317[5], 12}, {NULL}};
const Segment SGI352705[] = {{&S81317[98], 3}, {NULL}};
const Segment SGI352706[] = {{&S81318[5], 5}, {NULL}};
const Segment SGI352707[] = {{&S81318[16], 3}, {NULL}};
const Segment SGI352708[] = {{&S81319[5], 5}, {NULL}};
const Segment SGI352709[] = {{&S81319[21], 3}, {NULL}};
const Segment SGI352710[] = {{&S81335[21], 5}, {NULL}};
const Segment SGI352711[] = {{&S81335[0], 14}, {NULL}};
const Segment SGI352712[] = {{&S81337[5], 10}, {NULL}};
const Segment SGI352713[] = {{&S81337[21], 3}, {NULL}};
const Segment SGI352714[] = {{&S81372[43], 3}, {NULL}};
const Segment SGI352715[] = {{&S81372[6], 3}, {NULL}};
const Segment SGI352716[] = {{&S81372[67], 3}, {NULL}};
const Segment SGI352717[] = {{&S81390[5], 8}, {NULL}};
const Segment SGI352718[] = {{&S81390[19], 3}, {NULL}};
const Segment SGI352719[] = {{&S81391[5], 8}, {NULL}};
const Segment SGI352720[] = {{&S81391[24], 3}, {NULL}};
const Segment SGI352721[] = {{&S73629[28], 3}, {NULL}};
const Segment SGI352722[] = {{&S73629[102], 5}, {NULL}};
const Segment SGI352723[] = {{&S81408[35], 4}, {NULL}};
const Segment SGI352724[] = {{&S81408[90], 3}, {NULL}};
const Segment SGI352725[] = {{&S81409[5], 5}, {&S81409[97], 5}, {NULL}};
const Segment SGI352726[] = {{&S81409[17], 3}, {NULL}};
const Segment SGI352727[] = {{&S58153[91], 3}, {NULL}};
const Segment SGI352728[] = {{&S58153[13], 10}, {NULL}};
const Segment SGI352729[] = {{&S58155[48], 4}, {NULL}};
const Segment SGI352730[] = {{&S58155[13], 24}, {NULL}};
const Segment SGI352731[] = {{&S58156[64], 4}, {NULL}};
const Segment SGI352732[] = {{&S58156[13], 10}, {NULL}};
const Segment SGI352733[] = {{&S81425[5], 3}, {&S81425[5], 3}, {NULL}};
const Segment SGI352734[] = {{&S81425[9], 3}, {NULL}};
const Segment SGI352735[] = {{&S81425[127], 3}, {NULL}};
const Segment SGI352736[] = {{&S55874[6], 4}, {NULL}};
const Segment SGI352737[] = {{&S55874[73], 5}, {NULL}};
const Segment SGI352738[] = {{&S55874[12], 14}, {NULL}};
const Segment SGI352739[] = {{&S81437[5], 3}, {&S81437[5], 3}, {NULL}};
const Segment SGI352740[] = {{&S81437[9], 3}, {NULL}};
const Segment SGI352741[] = {{&S81437[127], 3}, {NULL}};
const Segment SGI352742[] = {{&S81455[5], 4}, {NULL}};
const Segment SGI352743[] = {{&S55905[6], 4}, {NULL}};
const Segment SGI352744[] = {{&S55905[59], 5}, {NULL}};
const Segment SGI352745[] = {{&S81455[20], 3}, {NULL}};
const Segment SGI352746[] = {{&S55905[12], 14}, {NULL}};
const Segment SGI352747[] = {{&S55906[6], 4}, {NULL}};
const Segment SGI352748[] = {{&S55906[59], 5}, {NULL}};
const Segment SGI352749[] = {{&S55906[12], 14}, {NULL}};
const Segment SGI352750[] = {{&S81458[35], 4}, {NULL}};
const Segment SGI352751[] = {{&S81458[51], 3}, {NULL}};
const Segment SGI352752[] = {{&S81459[5], 4}, {NULL}};
const Segment SGI352753[] = {{&S81459[15], 3}, {NULL}};
const Segment SGI352754[] = {{&S55951[6], 4}, {NULL}};
const Segment SGI352755[] = {{&S55951[70], 5}, {NULL}};
const Segment SGI352756[] = {{&S55951[12], 9}, {NULL}};
const Segment SGI352757[] = {{&S55951[29], 13}, {NULL}};
const Segment SGI352758[] = {{&S55998[6], 4}, {NULL}};
const Segment SGI352759[] = {{&S55998[77], 5}, {NULL}};
const Segment SGI352760[] = {{&S55998[12], 9}, {NULL}};
const Segment SGI352761[] = {{&S55998[29], 13}, {NULL}};
const Segment SGI352762[] = {{&S81557[43], 5}, {NULL}};
const Segment SGI352763[] = {{&S81557[6], 3}, {NULL}};
const Segment SGI352764[] = {{&S81557[69], 3}, {NULL}};
const Segment SGI352765[] = {{&S81571[5], 5}, {&S81571[105], 5}, {NULL}};
const Segment SGI352766[] = {{&S81571[11], 3}, {NULL}};
const Segment SGI352767[] = {{&S81571[131], 3}, {NULL}};
const Segment SGI352768[] = {{&S56028[6], 4}, {NULL}};
const Segment SGI352769[] = {{&S56028[12], 14}, {NULL}};
const Segment SGI352770[] = {{&S56029[6], 4}, {NULL}};
const Segment SGI352771[] = {{&S56029[12], 14}, {NULL}};
const Segment SGI352772[] = {{&S56030[6], 4}, {NULL}};
const Segment SGI352773[] = {{&S56030[12], 14}, {NULL}};
const Segment SGI352774[] = {{&S56031[6], 4}, {NULL}};
const Segment SGI352775[] = {{&S56031[12], 14}, {NULL}};
const Segment SGI352776[] = {{&S56033[6], 4}, {NULL}};
const Segment SGI352777[] = {{&S56033[12], 14}, {NULL}};
const Segment SGI352778[] = {{&S81599[5], 5}, {NULL}};
const Segment SGI352779[] = {{&S81599[21], 3}, {NULL}};
const Segment SGI352780[] = {{&S81601[34], 5}, {NULL}};
const Segment SGI352781[] = {{&S81601[50], 3}, {NULL}};
const Segment SGI352782[] = {{&S81627[42], 7}, {NULL}};
const Segment SGI352783[] = {{&S81627[6], 3}, {NULL}};
const Segment SGI352784[] = {{&S81627[70], 3}, {NULL}};
const Segment SGI352785[] = {{&S81639[21], 12}, {NULL}};
const Segment SGI352786[] = {{&S81639[0], 14}, {NULL}};
const Segment SGI352787[] = {{&S81667[5], 4}, {NULL}};
const Segment SGI352788[] = {{&S56121[6], 4}, {NULL}};
const Segment SGI352789[] = {{&S56121[59], 5}, {NULL}};
const Segment SGI352790[] = {{&S56121[12], 3}, {NULL}};
const Segment SGI352791[] = {{&S56122[6], 4}, {NULL}};
const Segment SGI352792[] = {{&S56122[59], 5}, {NULL}};
const Segment SGI352793[] = {{&S56122[12], 3}, {&S56122[66], 8}, {NULL}};
const Segment SGI352794[] = {{&S81683[5], 5}, {&S81683[97], 5}, {NULL}};
const Segment SGI352795[] = {{&S81683[153], 3}, {NULL}};
const Segment SGI352796[] = {{&S81684[5], 3}, {&S81684[5], 3}, {NULL}};
const Segment SGI352797[] = {{&S81684[80], 3}, {NULL}};
const Segment SGI352798[] = {{&S81699[5], 5}, {&S81699[105], 5}, {NULL}};
const Segment SGI352799[] = {{&S81699[11], 3}, {NULL}};
const Segment SGI352800[] = {{&S81699[131], 3}, {NULL}};
const Segment SGI352801[] = {{&S81725[43], 4}, {NULL}};
const Segment SGI352802[] = {{&S81725[6], 3}, {NULL}};
const Segment SGI352803[] = {{&S81725[68], 3}, {NULL}};
const Segment SGI352804[] = {{&S81735[5], 4}, {NULL}};
const Segment SGI352805[] = {{&S81752[35], 3}, {NULL}};
const Segment SGI352806[] = {{&S81752[89], 3}, {NULL}};
const Segment SGI352807[] = {{&S73861[28], 7}, {NULL}};
const Segment SGI352808[] = {{&S73861[106], 5}, {NULL}};
const Segment SGI352809[] = {{&S81790[5], 5}, {NULL}};
const Segment SGI352810[] = {{&S81790[21], 3}, {NULL}};
const Segment SGI352811[] = {{&S81809[35], 4}, {NULL}};
const Segment SGI352812[] = {{&S81809[90], 3}, {NULL}};
const Segment SGI352813[] = {{&S81812[38], 12}, {NULL}};
const Segment SGI352814[] = {{&S81812[101], 3}, {NULL}};
const Segment SGI352815[] = {{&S81838[5], 4}, {&S81838[5], 4}, {NULL}};
const Segment SGI352816[] = {{&S81838[10], 3}, {NULL}};
const Segment SGI352817[] = {{&S81838[129], 3}, {NULL}};
const Segment SGI352818[] = {{&S81890[38], 5}, {NULL}};
const Segment SGI352819[] = {{&S81890[94], 3}, {NULL}};
const Segment SGI352820[] = {{&S81903[42], 3}, {NULL}};
const Segment SGI352821[] = {{&S81903[6], 3}, {NULL}};
const Segment SGI352822[] = {{&S81903[66], 3}, {NULL}};
const Segment SGI352823[] = {{&S81912[5], 3}, {&S81912[5], 3}, {NULL}};
const Segment SGI352824[] = {{&S81912[149], 3}, {NULL}};
const Segment SGI352825[] = {{&S81947[5], 5}, {NULL}};
const Segment SGI352826[] = {{&S81947[21], 3}, {NULL}};
const Segment SGI352827[] = {{&S81948[5], 5}, {NULL}};
const Segment SGI352828[] = {{&S81948[21], 3}, {NULL}};
const Segment SGI352829[] = {{&S74136[28], 5}, {NULL}};
const Segment SGI352830[] = {{&S74136[104], 5}, {NULL}};
const Segment SGI352831[] = {{&S81978[5], 5}, {&S81978[104], 5}, {NULL}};
const Segment SGI352832[] = {{&S81978[11], 3}, {NULL}};
const Segment SGI352833[] = {{&S81978[130], 3}, {NULL}};
const Segment SGI352834[] = {{&S81981[42], 3}, {NULL}};
const Segment SGI352835[] = {{&S81981[6], 3}, {NULL}};
const Segment SGI352836[] = {{&S81981[66], 3}, {NULL}};
const Segment SGI352837[] = {{&S82099[5], 4}, {NULL}};
const Segment SGI352838[] = {{&S82106[5], 5}, {NULL}};
const Segment SGI352839[] = {{&S82106[21], 3}, {NULL}};
const Segment SGI352840[] = {{&S74279[28], 4}, {NULL}};
const Segment SGI352841[] = {{&S74279[103], 5}, {NULL}};
const Segment SGI352842[] = {{&S82124[35], 7}, {NULL}};
const Segment SGI352843[] = {{&S82124[93], 3}, {NULL}};
const Segment SGI352844[] = {{&S82125[42], 3}, {NULL}};
const Segment SGI352845[] = {{&S82125[6], 3}, {NULL}};
const Segment SGI352846[] = {{&S82125[66], 3}, {NULL}};
const Segment SGI352847[] = {{&S82126[41], 5}, {NULL}};
const Segment SGI352848[] = {{&S82126[6], 3}, {NULL}};
const Segment SGI352849[] = {{&S82126[67], 3}, {NULL}};
const Segment SGI352850[] = {{&S82129[5], 3}, {&S82129[5], 3}, {NULL}};
const Segment SGI352851[] = {{&S82129[9], 3}, {NULL}};
const Segment SGI352852[] = {{&S82129[126], 3}, {NULL}};
const Segment SGI352853[] = {{&S74309[28], 7}, {NULL}};
const Segment SGI352854[] = {{&S74309[106], 5}, {NULL}};
const Segment SGI352855[] = {{&S82143[45], 11}, {NULL}};
const Segment SGI352856[] = {{&S82143[6], 3}, {NULL}};
const Segment SGI352857[] = {{&S82143[77], 3}, {NULL}};
const Segment SGI352858[] = {{&S82144[101], 3}, {NULL}};
const Segment SGI352859[] = {{&S82145[5], 3}, {&S82145[5], 3}, {NULL}};
const Segment SGI352860[] = {{&S82145[149], 3}, {NULL}};
const Segment SGI352861[] = {{&S82174[5], 5}, {&S82174[105], 5}, {NULL}};
const Segment SGI352862[] = {{&S82174[11], 3}, {NULL}};
const Segment SGI352863[] = {{&S82174[131], 3}, {NULL}};
const Segment SGI352864[] = {{&S82197[5], 4}, {NULL}};
const Segment SGI352865[] = {{&S82206[5], 3}, {&S82206[5], 3}, {NULL}};
const Segment SGI352866[] = {{&S82206[109], 3}, {NULL}};
const Segment SGI352867[] = {{&S82216[5], 10}, {NULL}};
const Segment SGI352868[] = {{&S82216[21], 3}, {NULL}};
const Segment SGI352869[] = {{&S56736[6], 4}, {NULL}};
const Segment SGI352870[] = {{&S56736[90], 5}, {NULL}};
const Segment SGI352871[] = {{&S56736[12], 9}, {NULL}};
const Segment SGI352872[] = {{&S56736[29], 13}, {NULL}};
const Segment SGI352873[] = {{&S82249[5], 3}, {&S82249[5], 3}, {NULL}};
const Segment SGI352874[] = {{&S82249[9], 3}, {NULL}};
const Segment SGI352875[] = {{&S82249[127], 3}, {NULL}};
const Segment SGI352876[] = {{&S82252[5], 3}, {NULL}};
const Segment SGI352877[] = {{&S82252[167], 3}, {NULL}};
const Segment SGI352878[] = {{&S82285[5], 3}, {&S82285[5], 3}, {NULL}};
const Segment SGI352879[] = {{&S82285[80], 3}, {NULL}};
const Segment SGI352880[] = {{&S82404[49], 20}, {NULL}};
const Segment SGI352881[] = {{&S82405[49], 20}, {NULL}};
const Segment SGI352882[] = {{&S82421[42], 5}, {NULL}};
const Segment SGI352883[] = {{&S82421[6], 3}, {NULL}};
const Segment SGI352884[] = {{&S82421[68], 3}, {NULL}};
const Segment SGI352885[] = {{&S82422[43], 4}, {NULL}};
const Segment SGI352886[] = {{&S82422[6], 3}, {NULL}};
const Segment SGI352887[] = {{&S82422[68], 3}, {NULL}};
const Segment SGI352888[] = {{&S82423[101], 3}, {NULL}};
const Segment SGI352889[] = {{&S82440[43], 5}, {NULL}};
const Segment SGI352890[] = {{&S82440[6], 3}, {NULL}};
const Segment SGI352891[] = {{&S82440[69], 3}, {NULL}};
const Segment SGI352892[] = {{&S82445[5], 5}, {NULL}};
const Segment SGI352893[] = {{&S82445[20], 3}, {NULL}};
const Segment SGI352894[] = {{&S82446[5], 5}, {NULL}};
const Segment SGI352895[] = {{&S82447[5], 4}, {NULL}};
const Segment SGI352896[] = {{&S82471[5], 3}, {&S82471[5], 3}, {NULL}};
const Segment SGI352897[] = {{&S82471[149], 3}, {NULL}};
const Segment SGI352898[] = {{&S82529[5], 4}, {&S82529[5], 4}, {NULL}};
const Segment SGI352899[] = {{&S82529[82], 3}, {NULL}};
const Segment SGI352900[] = {{&S74708[28], 5}, {NULL}};
const Segment SGI352901[] = {{&S74708[104], 5}, {NULL}};
const Segment SGI352902[] = {{&S57022[0], 21}, {NULL}};
const Segment SGI352903[] = {{&S57022[34], 3}, {NULL}};
const Segment SGI352904[] = {{&S57022[69], 13}, {NULL}};
const Segment SGI352905[] = {{&S57025[6], 4}, {NULL}};
const Segment SGI352906[] = {{&S57025[70], 5}, {NULL}};
const Segment SGI352907[] = {{&S57025[12], 9}, {NULL}};
const Segment SGI352908[] = {{&S57025[29], 13}, {NULL}};
const Segment SGI352909[] = {{&S82626[5], 3}, {&S82626[5], 3}, {NULL}};
const Segment SGI352910[] = {{&S82626[67], 3}, {NULL}};
const Segment SGI352911[] = {{&S82626[126], 3}, {NULL}};
const Segment SGI352912[] = {{&S82674[5], 4}, {NULL}};
const Segment SGI352913[] = {{&S82674[20], 3}, {NULL}};
const Segment SGI352914[] = {{&S82676[5], 5}, {NULL}};
const Segment SGI352915[] = {{&S82676[21], 3}, {NULL}};
const Segment SGI352916[] = {{&S53629[201], 5}, {NULL}};
const Segment SGI352917[] = {{&S53629[138], 3}, {NULL}};
const Segment SGI352918[] = {{&S82691[5], 5}, {&S82691[97], 5}, {NULL}};
const Segment SGI352919[] = {{&S82691[153], 3}, {NULL}};
const Segment SGI352920[] = {{&S82709[5], 13}, {&S82709[113], 13}, {NULL}};
const Segment SGI352921[] = {{&S82709[19], 3}, {NULL}};
const Segment SGI352922[] = {{&S82709[147], 3}, {NULL}};
const Segment SGI352923[] = {{&S74983[28], 3}, {NULL}};
const Segment SGI352924[] = {{&S74983[102], 5}, {NULL}};
const Segment SGI352925[] = {{&S82769[5], 10}, {NULL}};
const Segment SGI352926[] = {{&S82769[21], 3}, {NULL}};
const Segment SGI352927[] = {{&S82774[5], 3}, {&S82774[5], 3}, {NULL}};
const Segment SGI352928[] = {{&S82774[149], 3}, {NULL}};
const Segment SGI352929[] = {{&S82775[42], 5}, {NULL}};
const Segment SGI352930[] = {{&S82775[6], 3}, {NULL}};
const Segment SGI352931[] = {{&S82775[68], 3}, {NULL}};
const Segment SGI352932[] = {{&S82776[43], 3}, {NULL}};
const Segment SGI352933[] = {{&S82776[6], 3}, {NULL}};
const Segment SGI352934[] = {{&S82776[67], 3}, {NULL}};
const Segment SGI352935[] = {{&S82779[42], 5}, {NULL}};
const Segment SGI352936[] = {{&S82779[6], 3}, {NULL}};
const Segment SGI352937[] = {{&S82779[68], 3}, {NULL}};
const Segment SGI352938[] = {{&S82846[41], 5}, {NULL}};
const Segment SGI352939[] = {{&S82846[6], 3}, {NULL}};
const Segment SGI352940[] = {{&S82846[67], 3}, {NULL}};
const Segment SGI352941[] = {{&S82847[5], 5}, {NULL}};
const Segment SGI352942[] = {{&S82847[16], 3}, {NULL}};
const Segment SGI352943[] = {{&S82873[43], 5}, {NULL}};
const Segment SGI352944[] = {{&S82873[6], 3}, {NULL}};
const Segment SGI352945[] = {{&S82873[69], 3}, {NULL}};
const Segment SGI352946[] = {{&S69603[5], 4}, {NULL}};
const Segment SGI352947[] = {{&S69603[47], 3}, {NULL}};
const Segment SGI352948[] = {{&S82907[34], 4}, {NULL}};
const Segment SGI352949[] = {{&S82907[89], 3}, {NULL}};
const Segment SGI352950[] = {{&S82909[5], 3}, {&S82909[5], 3}, {NULL}};
const Segment SGI352951[] = {{&S82909[149], 3}, {NULL}};
const Segment SGI352952[] = {{&S59726[15], 4}, {NULL}};
const Segment SGI352953[] = {{&S59726[21], 3}, {NULL}};
const Segment SGI352954[] = {{&S59726[52], 3}, {NULL}};
const Segment SGI352955[] = {{&S59726[67], 5}, {NULL}};
const Segment SGI352956[] = {{&S82945[5], 5}, {NULL}};
const Segment SGI352957[] = {{&S82945[169], 3}, {NULL}};
const Segment SGI352958[] = {{&S82965[5], 3}, {NULL}};
const Segment SGI352959[] = {{&S82965[19], 3}, {NULL}};
const Segment SGI352960[] = {{&S82973[5], 7}, {NULL}};
const Segment SGI352961[] = {{&S82973[23], 3}, {NULL}};
const Segment SGI352962[] = {{&S82983[35], 3}, {NULL}};
const Segment SGI352963[] = {{&S82983[50], 3}, {NULL}};
const Segment SGI352964[] = {{&S82989[5], 5}, {&S82989[5], 5}, {NULL}};
const Segment SGI352965[] = {{&S82989[11], 3}, {NULL}};
const Segment SGI352966[] = {{&S82989[130], 3}, {NULL}};
const Segment SGI352967[] = {{&S82993[5], 5}, {&S82993[97], 5}, {NULL}};
const Segment SGI352968[] = {{&S82993[153], 3}, {NULL}};
const Segment SGI352969[] = {{&S83015[21], 3}, {NULL}};
const Segment SGI352970[] = {{&S83015[0], 14}, {NULL}};
const Segment SGI352971[] = {{&S83034[42], 4}, {NULL}};
const Segment SGI352972[] = {{&S83034[6], 3}, {NULL}};
const Segment SGI352973[] = {{&S83034[67], 3}, {NULL}};
const Segment SGI352974[] = {{&S83043[5], 5}, {NULL}};
const Segment SGI352975[] = {{&S83043[21], 3}, {NULL}};
const Segment SGI352976[] = {{&S75267[28], 5}, {NULL}};
const Segment SGI352977[] = {{&S75267[104], 5}, {NULL}};
const Segment SGI352978[] = {{&S83126[5], 7}, {NULL}};
const Segment SGI352979[] = {{&S83144[35], 3}, {NULL}};
const Segment SGI352980[] = {{&S83144[50], 3}, {NULL}};
const Segment SGI352981[] = {{&S83151[35], 3}, {NULL}};
const Segment SGI352982[] = {{&S83151[89], 3}, {NULL}};
const Segment SGI352983[] = {{&S83159[5], 4}, {&S83159[5], 4}, {NULL}};
const Segment SGI352984[] = {{&S83159[68], 3}, {NULL}};
const Segment SGI352985[] = {{&S83159[128], 3}, {NULL}};
const Segment SGI352986[] = {{&S83194[43], 4}, {NULL}};
const Segment SGI352987[] = {{&S83194[6], 3}, {NULL}};
const Segment SGI352988[] = {{&S83194[68], 3}, {NULL}};
const Segment SGI352989[] = {{&S57470[6], 4}, {NULL}};
const Segment SGI352990[] = {{&S57470[90], 5}, {NULL}};
const Segment SGI352991[] = {{&S57470[12], 9}, {NULL}};
const Segment SGI352992[] = {{&S57470[29], 13}, {NULL}};
const Segment SGI352993[] = {{&S83201[5], 4}, {NULL}};
const Segment SGI352994[] = {{&S83202[5], 9}, {NULL}};
const Segment SGI352995[] = {{&S83206[5], 5}, {&S83206[68], 5}, {NULL}};
const Segment SGI352996[] = {{&S83206[84], 3}, {NULL}};
const Segment SGI352997[] = {{&S83214[38], 3}, {NULL}};
const Segment SGI352998[] = {{&S83214[92], 3}, {NULL}};
const Segment SGI352999[] = {{&S69896[5], 5}, {NULL}};
const Segment SGI353000[] = {{&S69896[33], 3}, {NULL}};
const Segment SGI353001[] = {{&S69896[87], 3}, {NULL}};
const Segment SGI353002[] = {{&S69897[5], 3}, {NULL}};
const Segment SGI353003[] = {{&S69897[32], 3}, {NULL}};
const Segment SGI353004[] = {{&S69897[86], 3}, {NULL}};
const Segment SGI353005[] = {{&S83228[45], 11}, {NULL}};
const Segment SGI353006[] = {{&S83228[6], 3}, {NULL}};
const Segment SGI353007[] = {{&S83228[77], 3}, {NULL}};
const Segment SGI353008[] = {{&S75450[28], 12}, {NULL}};
const Segment SGI353009[] = {{&S75450[111], 5}, {NULL}};
const Segment SGI353010[] = {{&S83301[5], 5}, {&S83301[68], 5}, {NULL}};
const Segment SGI353011[] = {{&S83301[84], 3}, {NULL}};
const Segment SGI353012[] = {{&S83306[5], 3}, {NULL}};
const Segment SGI353013[] = {{&S83306[19], 3}, {NULL}};
const Segment SGI353014[] = {{&S83313[35], 5}, {NULL}};
const Segment SGI353015[] = {{&S83313[52], 3}, {NULL}};
const Segment SGI353016[] = {{&S83314[35], 5}, {NULL}};
const Segment SGI353017[] = {{&S83314[52], 3}, {NULL}};
const Segment SGI353018[] = {{&S83319[34], 5}, {NULL}};
const Segment SGI353019[] = {{&S83319[50], 3}, {NULL}};
const Segment SGI353020[] = {{&S83324[5], 7}, {&S83324[5], 7}, {NULL}};
const Segment SGI353021[] = {{&S83324[13], 3}, {NULL}};
const Segment SGI353022[] = {{&S83324[134], 3}, {NULL}};
const Segment SGI353023[] = {{&S83327[42], 3}, {NULL}};
const Segment SGI353024[] = {{&S83327[6], 3}, {NULL}};
const Segment SGI353025[] = {{&S83327[66], 3}, {NULL}};
const Segment SGI353026[] = {{&S75513[28], 5}, {NULL}};
const Segment SGI353027[] = {{&S75513[104], 5}, {NULL}};
const Segment SGI353028[] = {{&S75542[28], 5}, {NULL}};
const Segment SGI353029[] = {{&S75542[104], 5}, {NULL}};
const Segment SGI353030[] = {{&S83384[5], 3}, {NULL}};
const Segment SGI353031[] = {{&S83384[19], 3}, {NULL}};
const Segment SGI353032[] = {{&S83385[5], 5}, {NULL}};
const Segment SGI353033[] = {{&S83385[21], 3}, {NULL}};
const Segment SGI353034[] = {{&S83418[34], 5}, {NULL}};
const Segment SGI353035[] = {{&S83418[50], 3}, {NULL}};
const Segment SGI353036[] = {{&S57649[12], 13}, {NULL}};
const Segment SGI353037[] = {{&S57650[12], 13}, {NULL}};
const Segment SGI353038[] = {{&S57651[12], 13}, {NULL}};
const Segment SGI353039[] = {{&S57652[12], 13}, {NULL}};
const Segment SGI353040[] = {{&S57653[12], 13}, {NULL}};
const Segment SGI353041[] = {{&S57654[6], 4}, {NULL}};
const Segment SGI353042[] = {{&S57654[86], 5}, {NULL}};
const Segment SGI353043[] = {{&S57654[12], 11}, {NULL}};
const Segment SGI353044[] = {{&S57655[12], 13}, {NULL}};
const Segment SGI353045[] = {{&S57707[12], 13}, {NULL}};
const Segment SGI353046[] = {{&S57708[12], 13}, {NULL}};
const Segment SGI353047[] = {{&S57709[12], 13}, {NULL}};
const Segment SGI353048[] = {{&S57710[12], 13}, {NULL}};
const Segment SGI353049[] = {{&S83491[5], 4}, {NULL}};
const Segment SGI353050[] = {{&S83491[15], 3}, {NULL}};
const Segment SGI353051[] = {{&S57711[12], 13}, {NULL}};
const Segment SGI353052[] = {{&S83492[5], 4}, {NULL}};
const Segment SGI353053[] = {{&S83492[20], 3}, {NULL}};
const Segment SGI353054[] = {{&S57712[6], 4}, {NULL}};
const Segment SGI353055[] = {{&S57712[76], 5}, {NULL}};
const Segment SGI353056[] = {{&S57712[12], 11}, {NULL}};
const Segment SGI353057[] = {{&S57713[12], 13}, {NULL}};
const Segment SGI353058[] = {{&S70223[5], 4}, {NULL}};
const Segment SGI353059[] = {{&S70223[47], 3}, {NULL}};
const Segment SGI353060[] = {{&S83524[35], 5}, {NULL}};
const Segment SGI353061[] = {{&S83524[91], 3}, {NULL}};
const Segment SGI353062[] = {{&S83526[21], 7}, {NULL}};
const Segment SGI353063[] = {{&S83526[0], 14}, {NULL}};
const Segment SGI353064[] = {{&S83551[5], 5}, {&S83551[105], 5}, {NULL}};
const Segment SGI353065[] = {{&S83551[69], 3}, {NULL}};
const Segment SGI353066[] = {{&S83551[131], 3}, {NULL}};
const Segment SGI353067[] = {{&S83583[5], 3}, {NULL}};
const Segment SGI353068[] = {{&S83583[14], 3}, {NULL}};
const Segment SGI353069[] = {{&S83584[34], 5}, {NULL}};
const Segment SGI353070[] = {{&S83584[50], 3}, {NULL}};
const Segment SGI353071[] = {{&S83585[35], 5}, {NULL}};
const Segment SGI353072[] = {{&S83585[52], 3}, {NULL}};
const Segment SGI353073[] = {{&S83586[5], 5}, {NULL}};
const Segment SGI353074[] = {{&S83586[16], 3}, {NULL}};
const Segment SGI353075[] = {{&S70299[8], 3}, {NULL}};
const Segment SGI353076[] = {{&S70299[35], 3}, {NULL}};
const Segment SGI353077[] = {{&S75853[28], 5}, {NULL}};
const Segment SGI353078[] = {{&S75853[104], 5}, {NULL}};
const Segment SGI353079[] = {{&S83666[5], 4}, {NULL}};
const Segment SGI353080[] = {{&S83678[34], 5}, {NULL}};
const Segment SGI353081[] = {{&S83678[50], 3}, {NULL}};
const Segment SGI353082[] = {{&S83683[34], 7}, {NULL}};
const Segment SGI353083[] = {{&S83683[52], 3}, {NULL}};
const Segment SGI353084[] = {{&S83684[34], 5}, {NULL}};
const Segment SGI353085[] = {{&S83684[50], 3}, {NULL}};
const Segment SGI353086[] = {{&S83707[42], 5}, {NULL}};
const Segment SGI353087[] = {{&S83707[6], 3}, {NULL}};
const Segment SGI353088[] = {{&S83707[68], 3}, {NULL}};
const Segment SGI353089[] = {{&S57826[6], 4}, {NULL}};
const Segment SGI353090[] = {{&S57826[60], 5}, {NULL}};
const Segment SGI353091[] = {{&S83714[5], 3}, {&S83714[5], 3}, {NULL}};
const Segment SGI353092[] = {{&S57826[12], 3}, {NULL}};
const Segment SGI353093[] = {{&S83714[67], 3}, {NULL}};
const Segment SGI353094[] = {{&S83714[127], 3}, {NULL}};
const Segment SGI353095[] = {{&S83728[5], 11}, {&S83728[5], 11}, {NULL}};
const Segment SGI353096[] = {{&S83728[17], 3}, {NULL}};
const Segment SGI353097[] = {{&S83728[143], 3}, {NULL}};
const Segment SGI353098[] = {{&S83737[5], 8}, {NULL}};
const Segment SGI353099[] = {{&S83774[5], 3}, {&S83774[5], 3}, {NULL}};
const Segment SGI353100[] = {{&S83774[9], 3}, {NULL}};
const Segment SGI353101[] = {{&S83774[126], 3}, {NULL}};
const Segment SGI353102[] = {{&S57896[6], 4}, {NULL}};
const Segment SGI353103[] = {{&S57896[66], 5}, {NULL}};
const Segment SGI353104[] = {{&S57896[12], 14}, {NULL}};
const Segment SGI353105[] = {{&S83797[5], 11}, {&S83797[5], 11}, {NULL}};
const Segment SGI353106[] = {{&S83797[17], 3}, {NULL}};
const Segment SGI353107[] = {{&S83797[143], 3}, {NULL}};
const Segment SGI353108[] = {{&S57945[6], 4}, {NULL}};
const Segment SGI353109[] = {{&S57945[77], 5}, {NULL}};
const Segment SGI353110[] = {{&S57945[12], 9}, {NULL}};
const Segment SGI353111[] = {{&S57945[29], 13}, {NULL}};
const Segment SGI353112[] = {{&S57946[6], 4}, {NULL}};
const Segment SGI353113[] = {{&S57946[77], 5}, {NULL}};
const Segment SGI353114[] = {{&S57946[12], 9}, {NULL}};
const Segment SGI353115[] = {{&S57946[29], 13}, {NULL}};
const Segment SGI353116[] = {{&S83838[5], 13}, {&S83838[105], 13}, {NULL}};
const Segment SGI353117[] = {{&S83838[129], 3}, {NULL}};
const Segment SGI353118[] = {{&S83849[35], 3}, {NULL}};
const Segment SGI353119[] = {{&S83849[89], 3}, {NULL}};
const Segment SGI353120[] = {{&S57973[6], 4}, {NULL}};
const Segment SGI353121[] = {{&S57973[75], 5}, {NULL}};
const Segment SGI353122[] = {{&S57973[12], 3}, {NULL}};
const Segment SGI353123[] = {{&S57973[39], 10}, {NULL}};
const Segment SGI353124[] = {{&S83872[5], 13}, {&S83872[113], 13}, {NULL}};
const Segment SGI353125[] = {{&S83872[19], 3}, {NULL}};
const Segment SGI353126[] = {{&S83872[147], 3}, {NULL}};
const Segment SGI353127[] = {{&S83891[45], 11}, {NULL}};
const Segment SGI353128[] = {{&S83891[6], 3}, {NULL}};
const Segment SGI353129[] = {{&S83891[77], 3}, {NULL}};
const Segment SGI353130[] = {{&S83978[43], 9}, {NULL}};
const Segment SGI353131[] = {{&S83978[6], 3}, {NULL}};
const Segment SGI353132[] = {{&S83978[73], 3}, {NULL}};
const Segment SGI353133[] = {{&S83979[43], 13}, {NULL}};
const Segment SGI353134[] = {{&S83979[6], 3}, {NULL}};
const Segment SGI353135[] = {{&S83979[77], 3}, {NULL}};
const Segment SGI353136[] = {{&S83987[5], 3}, {&S83987[5], 3}, {NULL}};
const Segment SGI353137[] = {{&S83987[9], 3}, {NULL}};
const Segment SGI353138[] = {{&S83987[127], 3}, {NULL}};
const Segment SGI353139[] = {{&S75967[28], 5}, {NULL}};
const Segment SGI353140[] = {{&S75967[104], 5}, {NULL}};
const Segment SGI353141[] = {{&S83995[5], 5}, {NULL}};
const Segment SGI353142[] = {{&S83995[16], 3}, {NULL}};
const Segment SGI353143[] = {{&S83996[34], 5}, {NULL}};
const Segment SGI353144[] = {{&S83996[50], 3}, {NULL}};
const Segment SGI353145[] = {{&S84022[21], 5}, {NULL}};
const Segment SGI353146[] = {{&S84022[0], 14}, {NULL}};
const Segment SGI353147[] = {{&S76024[28], 3}, {NULL}};
const Segment SGI353148[] = {{&S76024[102], 5}, {NULL}};
const Segment SGI353149[] = {{&S84047[5], 3}, {&S84047[5], 3}, {NULL}};
const Segment SGI353150[] = {{&S84047[9], 3}, {NULL}};
const Segment SGI353151[] = {{&S84047[127], 3}, {NULL}};
const Segment SGI353152[] = {{&S84095[35], 7}, {NULL}};
const Segment SGI353153[] = {{&S84095[54], 3}, {NULL}};
const Segment SGI353154[] = {{&S84096[35], 3}, {NULL}};
const Segment SGI353155[] = {{&S84096[50], 3}, {NULL}};
const Segment SGI353156[] = {{&S84097[34], 3}, {NULL}};
const Segment SGI353157[] = {{&S84097[48], 3}, {NULL}};
const Segment SGI353158[] = {{&S58154[6], 2}, {NULL}};
const Segment SGI353159[] = {{&S58154[11], 22}, {NULL}};
const Segment SGI353160[] = {{&S84098[34], 7}, {NULL}};
const Segment SGI353161[] = {{&S84098[52], 3}, {NULL}};
const Segment SGI353162[] = {{&S84099[5], 5}, {NULL}};
const Segment SGI353163[] = {{&S84099[16], 3}, {NULL}};
const Segment SGI353164[] = {{&S84100[34], 5}, {NULL}};
const Segment SGI353165[] = {{&S84100[50], 3}, {NULL}};
const Segment SGI353166[] = {{&S58159[6], 4}, {NULL}};
const Segment SGI353167[] = {{&S58159[59], 5}, {NULL}};
const Segment SGI353168[] = {{&S58159[12], 14}, {NULL}};
const Segment SGI353169[] = {{&S76109[28], 5}, {NULL}};
const Segment SGI353170[] = {{&S76109[111], 5}, {NULL}};
const Segment SGI353171[] = {{&S84125[5], 5}, {NULL}};
const Segment SGI353172[] = {{&S84125[16], 3}, {NULL}};
const Segment SGI353173[] = {{&S84126[41], 5}, {NULL}};
const Segment SGI353174[] = {{&S84126[6], 3}, {NULL}};
const Segment SGI353175[] = {{&S84126[67], 3}, {NULL}};
const Segment SGI353176[] = {{&S84136[42], 8}, {NULL}};
const Segment SGI353177[] = {{&S84136[6], 3}, {NULL}};
const Segment SGI353178[] = {{&S84136[71], 3}, {NULL}};
const Segment SGI353179[] = {{&S84137[42], 5}, {NULL}};
const Segment SGI353180[] = {{&S84137[6], 3}, {NULL}};
const Segment SGI353181[] = {{&S84137[68], 3}, {NULL}};
const Segment SGI353182[] = {{&S76140[28], 5}, {NULL}};
const Segment SGI353183[] = {{&S76140[104], 5}, {NULL}};
const Segment SGI353184[] = {{&S76162[28], 5}, {NULL}};
const Segment SGI353185[] = {{&S76162[104], 5}, {NULL}};
const Segment SGI353186[] = {{&S84196[35], 4}, {NULL}};
const Segment SGI353187[] = {{&S84196[90], 3}, {NULL}};
const Segment SGI353188[] = {{&S84197[35], 3}, {NULL}};
const Segment SGI353189[] = {{&S84197[89], 3}, {NULL}};
const Segment SGI353190[] = {{&S84198[41], 6}, {NULL}};
const Segment SGI353191[] = {{&S84198[6], 3}, {NULL}};
const Segment SGI353192[] = {{&S84198[68], 3}, {NULL}};
const Segment SGI353193[] = {{&S84199[5], 13}, {NULL}};
const Segment SGI353194[] = {{&S84199[19], 3}, {NULL}};
const Segment SGI353195[] = {{&S84232[43], 4}, {NULL}};
const Segment SGI353196[] = {{&S84232[6], 3}, {NULL}};
const Segment SGI353197[] = {{&S84232[68], 3}, {NULL}};
const Segment SGI353198[] = {{&S84233[43], 3}, {NULL}};
const Segment SGI353199[] = {{&S84233[6], 3}, {NULL}};
const Segment SGI353200[] = {{&S84233[67], 3}, {NULL}};
const Segment SGI353201[] = {{&S84234[42], 6}, {NULL}};
const Segment SGI353202[] = {{&S84234[6], 3}, {NULL}};
const Segment SGI353203[] = {{&S84234[69], 3}, {NULL}};
const Segment SGI353204[] = {{&S84273[5], 7}, {NULL}};
const Segment SGI353205[] = {{&S84273[23], 3}, {NULL}};
const Segment SGI353206[] = {{&S84274[5], 11}, {&S84274[5], 11}, {NULL}};
const Segment SGI353207[] = {{&S84274[96], 3}, {NULL}};
const Segment SGI353208[] = {{&S84287[35], 4}, {NULL}};
const Segment SGI353209[] = {{&S84287[90], 3}, {NULL}};
const Segment SGI353210[] = {{&S84293[35], 7}, {NULL}};
const Segment SGI353211[] = {{&S84293[93], 3}, {NULL}};
const Segment SGI353212[] = {{&S84328[5], 3}, {NULL}};
const Segment SGI353213[] = {{&S58492[0], 16}, {NULL}};
const Segment SGI353214[] = {{&S58492[29], 3}, {NULL}};
const Segment SGI353215[] = {{&S58492[71], 10}, {NULL}};
const Segment SGI353216[] = {{&S58395[6], 4}, {NULL}};
const Segment SGI353217[] = {{&S58395[77], 5}, {NULL}};
const Segment SGI353218[] = {{&S58395[12], 9}, {NULL}};
const Segment SGI353219[] = {{&S58395[29], 13}, {NULL}};
const Segment SGI353220[] = {{&S84376[5], 7}, {NULL}};
const Segment SGI353221[] = {{&S84376[23], 3}, {NULL}};
const Segment SGI353222[] = {{&S84377[5], 3}, {NULL}};
const Segment SGI353223[] = {{&S84377[19], 3}, {NULL}};
const Segment SGI353224[] = {{&S58606[0], 16}, {NULL}};
const Segment SGI353225[] = {{&S58606[29], 3}, {NULL}};
const Segment SGI353226[] = {{&S58606[64], 13}, {NULL}};
const Segment SGI353227[] = {{&S71066[49], 5}, {NULL}};
const Segment SGI353228[] = {{&S71066[40], 3}, {NULL}};
const Segment SGI353229[] = {{&S84497[5], 10}, {NULL}};
const Segment SGI353230[] = {{&S84497[21], 3}, {NULL}};
const Segment SGI353231[] = {{&S71089[5], 4}, {NULL}};
const Segment SGI353232[] = {{&S71089[49], 3}, {NULL}};
const Segment SGI353233[] = {{&S58650[0], 21}, {NULL}};
const Segment SGI353234[] = {{&S58650[34], 3}, {NULL}};
const Segment SGI353235[] = {{&S58651[0], 21}, {NULL}};
const Segment SGI353236[] = {{&S58651[34], 3}, {NULL}};
const Segment SGI353237[] = {{&S58651[73], 13}, {NULL}};
const Segment SGI353238[] = {{&S61342[57], 4}, {NULL}};
const Segment SGI353239[] = {{&S61342[13], 10}, {NULL}};
const Segment SGI353240[] = {{&S61343[57], 4}, {NULL}};
const Segment SGI353241[] = {{&S61343[13], 10}, {NULL}};
const Segment SGI353242[] = {{&S61345[38], 4}, {NULL}};
const Segment SGI353243[] = {{&S61345[13], 14}, {NULL}};
const Segment SGI353244[] = {{&S61346[57], 3}, {NULL}};
const Segment SGI353245[] = {{&S61346[13], 10}, {NULL}};
const Segment SGI353246[] = {{&S61347[57], 4}, {NULL}};
const Segment SGI353247[] = {{&S61347[13], 10}, {NULL}};
const Segment SGI353248[] = {{&S61363[49], 4}, {NULL}};
const Segment SGI353249[] = {{&S61363[13], 25}, {NULL}};
const Segment SGI353250[] = {{&S61365[58], 4}, {NULL}};
const Segment SGI353251[] = {{&S61365[13], 10}, {NULL}};
const Segment SGI353252[] = {{&S61366[39], 4}, {NULL}};
const Segment SGI353253[] = {{&S61366[13], 3}, {NULL}};
const Segment SGI353254[] = {{&S61367[39], 4}, {NULL}};
const Segment SGI353255[] = {{&S61367[13], 3}, {NULL}};
const Segment SGI353256[] = {{&S61368[40], 4}, {NULL}};
const Segment SGI353257[] = {{&S61368[6], 8}, {NULL}};
const Segment SGI353258[] = {{&S61370[58], 4}, {NULL}};
const Segment SGI353259[] = {{&S61370[13], 10}, {NULL}};
const Segment SGI353260[] = {{&S61376[7], 5}, {NULL}};
const Segment SGI353261[] = {{&S84551[5], 15}, {&S84551[115], 15}, {NULL}};
const Segment SGI353262[] = {{&S84551[21], 3}, {NULL}};
const Segment SGI353263[] = {{&S84551[151], 3}, {NULL}};
const Segment SGI353264[] = {{&S76557[28], 7}, {NULL}};
const Segment SGI353265[] = {{&S76557[106], 5}, {NULL}};
const Segment SGI353266[] = {{&S76558[28], 5}, {NULL}};
const Segment SGI353267[] = {{&S76558[104], 5}, {NULL}};
const Segment SGI353268[] = {{&S76569[28], 5}, {NULL}};
const Segment SGI353269[] = {{&S76569[104], 5}, {NULL}};
const Segment SGI353270[] = {{&S84577[5], 4}, {NULL}};
const Segment SGI353271[] = {{&S84578[5], 5}, {NULL}};
const Segment SGI353272[] = {{&S84589[5], 5}, {NULL}};
const Segment SGI353273[] = {{&S84589[21], 3}, {NULL}};
const Segment SGI353274[] = {{&S84606[35], 5}, {NULL}};
const Segment SGI353275[] = {{&S84606[91], 3}, {NULL}};
const Segment SGI353276[] = {{&S84611[21], 3}, {NULL}};
const Segment SGI353277[] = {{&S84611[0], 14}, {NULL}};
const Segment SGI353278[] = {{&S76606[28], 7}, {NULL}};
const Segment SGI353279[] = {{&S76606[106], 5}, {NULL}};
const Segment SGI353280[] = {{&S76607[28], 5}, {NULL}};
const Segment SGI353281[] = {{&S76607[104], 5}, {NULL}};
const Segment SGI353282[] = {{&S84636[43], 4}, {NULL}};
const Segment SGI353283[] = {{&S84636[6], 3}, {NULL}};
const Segment SGI353284[] = {{&S84636[68], 3}, {NULL}};
const Segment SGI353285[] = {{&S84645[43], 14}, {NULL}};
const Segment SGI353286[] = {{&S84645[6], 3}, {NULL}};
const Segment SGI353287[] = {{&S84645[78], 3}, {NULL}};
const Segment SGI353288[] = {{&S84666[5], 4}, {&S84666[5], 4}, {NULL}};
const Segment SGI353289[] = {{&S84666[82], 3}, {NULL}};
const Segment SGI353290[] = {{&S84667[5], 4}, {NULL}};
const Segment SGI353291[] = {{&S84667[235], 3}, {NULL}};
const Segment SGI353292[] = {{&S84683[5], 5}, {&S84683[5], 5}, {NULL}};
const Segment SGI353293[] = {{&S84683[84], 3}, {NULL}};
const Segment SGI353294[] = {{&S84707[38], 5}, {NULL}};
const Segment SGI353295[] = {{&S84707[55], 3}, {NULL}};
const Segment SGI353296[] = {{&S84710[5], 5}, {&S84710[97], 5}, {NULL}};
const Segment SGI353297[] = {{&S84710[153], 3}, {NULL}};
const Segment SGI353298[] = {{&S84721[5], 5}, {&S84721[104], 5}, {NULL}};
const Segment SGI353299[] = {{&S84721[69], 3}, {NULL}};
const Segment SGI353300[] = {{&S84721[130], 3}, {NULL}};
const Segment SGI353301[] = {{&S84765[5], 5}, {NULL}};
const Segment SGI353302[] = {{&S84765[169], 3}, {NULL}};
const Segment SGI353303[] = {{&S84789[5], 3}, {&S84789[5], 3}, {NULL}};
const Segment SGI353304[] = {{&S84789[109], 3}, {NULL}};
const Segment SGI353305[] = {{&S84791[34], 3}, {NULL}};
const Segment SGI353306[] = {{&S84791[88], 3}, {NULL}};
const Segment SGI353307[] = {{&S58642[6], 4}, {NULL}};
const Segment SGI353308[] = {{&S58642[58], 5}, {NULL}};
const Segment SGI353309[] = {{&S58642[12], 14}, {NULL}};
const Segment SGI353310[] = {{&S58643[6], 4}, {NULL}};
const Segment SGI353311[] = {{&S58643[58], 5}, {NULL}};
const Segment SGI353312[] = {{&S58643[12], 14}, {NULL}};
const Segment SGI353313[] = {{&S84796[35], 3}, {NULL}};
const Segment SGI353314[] = {{&S84796[89], 3}, {NULL}};
const Segment SGI353315[] = {{&S58644[6], 4}, {NULL}};
const Segment SGI353316[] = {{&S58644[12], 17}, {NULL}};
const Segment SGI353317[] = {{&S58644[12], 14}, {NULL}};
const Segment SGI353318[] = {{&S84799[35], 4}, {NULL}};
const Segment SGI353319[] = {{&S84799[90], 3}, {NULL}};
const Segment SGI353320[] = {{&S84800[41], 4}, {NULL}};
const Segment SGI353321[] = {{&S84800[6], 3}, {NULL}};
const Segment SGI353322[] = {{&S84800[66], 3}, {NULL}};
const Segment SGI353323[] = {{&S84819[101], 4}, {NULL}};
const Segment SGI353324[] = {{&S84838[45], 11}, {NULL}};
const Segment SGI353325[] = {{&S84838[6], 3}, {NULL}};
const Segment SGI353326[] = {{&S84838[77], 3}, {NULL}};
const Segment SGI353327[] = {{&S58707[6], 4}, {NULL}};
const Segment SGI353328[] = {{&S58707[70], 5}, {NULL}};
const Segment SGI353329[] = {{&S58707[12], 9}, {NULL}};
const Segment SGI353330[] = {{&S58707[29], 13}, {NULL}};
const Segment SGI353331[] = {{&S84878[35], 3}, {NULL}};
const Segment SGI353332[] = {{&S84878[89], 3}, {NULL}};
const Segment SGI353333[] = {{&S76890[28], 12}, {NULL}};
const Segment SGI353334[] = {{&S76890[111], 5}, {NULL}};
const Segment SGI353335[] = {{&S84907[45], 11}, {NULL}};
const Segment SGI353336[] = {{&S84907[6], 3}, {NULL}};
const Segment SGI353337[] = {{&S84907[77], 3}, {NULL}};
const Segment SGI353338[] = {{&S58775[6], 4}, {NULL}};
const Segment SGI353339[] = {{&S58775[59], 5}, {NULL}};
const Segment SGI353340[] = {{&S58775[12], 14}, {NULL}};
const Segment SGI353341[] = {{&S84933[43], 5}, {NULL}};
const Segment SGI353342[] = {{&S84933[6], 3}, {NULL}};
const Segment SGI353343[] = {{&S84933[69], 3}, {NULL}};
const Segment SGI353344[] = {{&S84938[5], 13}, {&S84938[113], 13}, {NULL}};
const Segment SGI353345[] = {{&S84938[19], 3}, {NULL}};
const Segment SGI353346[] = {{&S84938[147], 3}, {NULL}};
const Segment SGI353347[] = {{&S84978[35], 9}, {NULL}};
const Segment SGI353348[] = {{&S84978[95], 3}, {NULL}};
const Segment SGI353349[] = {{&S85066[5], 5}, {NULL}};
const Segment SGI353350[] = {{&S85066[16], 3}, {NULL}};
const Segment SGI353351[] = {{&S77100[28], 5}, {NULL}};
const Segment SGI353352[] = {{&S77100[105], 5}, {NULL}};
const Segment SGI353353[] = {{&S85165[23], 5}, {NULL}};
const Segment SGI353354[] = {{&S85165[0], 16}, {NULL}};
const Segment SGI353355[] = {{&S85207[43], 7}, {NULL}};
const Segment SGI353356[] = {{&S85207[6], 3}, {NULL}};
const Segment SGI353357[] = {{&S85207[71], 3}, {NULL}};
const Segment SGI353358[] = {{&S77220[28], 3}, {NULL}};
const Segment SGI353359[] = {{&S77220[103], 5}, {NULL}};
const Segment SGI353360[] = {{&S85235[5], 5}, {&S85235[5], 5}, {NULL}};
const Segment SGI353361[] = {{&S85235[11], 3}, {NULL}};
const Segment SGI353362[] = {{&S85235[131], 3}, {NULL}};
const Segment SGI353363[] = {{&S77268[28], 5}, {NULL}};
const Segment SGI353364[] = {{&S77268[111], 5}, {NULL}};
const Segment SGI353365[] = {{&S85261[5], 12}, {&S85261[5], 12}, {NULL}};
const Segment SGI353366[] = {{&S85261[127], 3}, {NULL}};
const Segment SGI353367[] = {{&S85270[5], 10}, {NULL}};
const Segment SGI353368[] = {{&S85270[21], 3}, {NULL}};
const Segment SGI353369[] = {{&S85272[5], 5}, {&S85272[97], 5}, {NULL}};
const Segment SGI353370[] = {{&S85272[153], 3}, {NULL}};
const Segment SGI353371[] = {{&S85280[21], 4}, {NULL}};
const Segment SGI353372[] = {{&S85280[0], 14}, {NULL}};
const Segment SGI353373[] = {{&S85346[5], 5}, {NULL}};
const Segment SGI353374[] = {{&S85346[21], 3}, {NULL}};
const Segment SGI353375[] = {{&S85353[35], 3}, {NULL}};
const Segment SGI353376[] = {{&S85353[89], 3}, {NULL}};
const Segment SGI353377[] = {{&S85354[34], 5}, {NULL}};
const Segment SGI353378[] = {{&S85354[90], 3}, {NULL}};
const Segment SGI353379[] = {{&S77388[28], 5}, {NULL}};
const Segment SGI353380[] = {{&S77388[104], 5}, {NULL}};
const Segment SGI353381[] = {{&S85384[5], 4}, {&S85384[5], 4}, {NULL}};
const Segment SGI353382[] = {{&S85384[68], 3}, {NULL}};
const Segment SGI353383[] = {{&S85384[129], 3}, {NULL}};
const Segment SGI353384[] = {{&S85387[43], 5}, {NULL}};
const Segment SGI353385[] = {{&S85387[6], 3}, {NULL}};
const Segment SGI353386[] = {{&S85387[69], 3}, {NULL}};
const Segment SGI353387[] = {{&S59340[0], 16}, {NULL}};
const Segment SGI353388[] = {{&S59340[29], 3}, {NULL}};
const Segment SGI353389[] = {{&S59340[64], 13}, {NULL}};
const Segment SGI353390[] = {{&S77478[28], 5}, {NULL}};
const Segment SGI353391[] = {{&S77478[104], 5}, {NULL}};
const Segment SGI353392[] = {{&S77479[28], 7}, {NULL}};
const Segment SGI353393[] = {{&S77479[106], 5}, {NULL}};
const Segment SGI353394[] = {{&S85464[38], 5}, {NULL}};
const Segment SGI353395[] = {{&S85464[94], 3}, {NULL}};
const Segment SGI353396[] = {{&S85467[5], 4}, {&S85467[5], 4}, {NULL}};
const Segment SGI353397[] = {{&S85467[10], 3}, {NULL}};
const Segment SGI353398[] = {{&S85467[128], 3}, {NULL}};
const Segment SGI353399[] = {{&S85492[5], 7}, {&S85492[5], 7}, {NULL}};
const Segment SGI353400[] = {{&S85492[71], 3}, {NULL}};
const Segment SGI353401[] = {{&S85492[135], 3}, {NULL}};
const Segment SGI353402[] = {{&S85522[5], 11}, {&S85522[5], 11}, {NULL}};
const Segment SGI353403[] = {{&S85522[17], 3}, {NULL}};
const Segment SGI353404[] = {{&S85522[143], 3}, {NULL}};
const Segment SGI353405[] = {{&S85523[42], 5}, {NULL}};
const Segment SGI353406[] = {{&S85523[6], 3}, {NULL}};
const Segment SGI353407[] = {{&S85523[68], 3}, {NULL}};
const Segment SGI353408[] = {{&S85526[43], 4}, {NULL}};
const Segment SGI353409[] = {{&S85526[6], 3}, {NULL}};
const Segment SGI353410[] = {{&S85526[68], 3}, {NULL}};
const Segment SGI353411[] = {{&S85528[43], 7}, {NULL}};
const Segment SGI353412[] = {{&S85528[6], 3}, {NULL}};
const Segment SGI353413[] = {{&S85528[71], 3}, {NULL}};
const Segment SGI353414[] = {{&S85530[43], 3}, {NULL}};
const Segment SGI353415[] = {{&S85530[6], 3}, {NULL}};
const Segment SGI353416[] = {{&S85530[67], 3}, {NULL}};
const Segment SGI353417[] = {{&S85532[5], 11}, {&S85532[5], 11}, {NULL}};
const Segment SGI353418[] = {{&S85532[17], 3}, {NULL}};
const Segment SGI353419[] = {{&S85532[143], 3}, {NULL}};
const Segment SGI353420[] = {{&S85559[5], 4}, {NULL}};
const Segment SGI353421[] = {{&S85559[20], 3}, {NULL}};
const Segment SGI353422[] = {{&S85560[5], 16}, {NULL}};
const Segment SGI353423[] = {{&S85560[32], 3}, {NULL}};
const Segment SGI353424[] = {{&S85561[5], 3}, {NULL}};
const Segment SGI353425[] = {{&S85561[19], 3}, {NULL}};
const Segment SGI353426[] = {{&S85565[5], 15}, {&S85565[78], 15}, {NULL}};
const Segment SGI353427[] = {{&S85565[104], 3}, {NULL}};
const Segment SGI353428[] = {{&S85572[5], 11}, {&S85572[5], 11}, {NULL}};
const Segment SGI353429[] = {{&S85572[165], 3}, {NULL}};
const Segment SGI353430[] = {{&S85573[35], 4}, {NULL}};
const Segment SGI353431[] = {{&S85573[90], 3}, {NULL}};
const Segment SGI353432[] = {{&S85574[5], 11}, {&S85574[5], 11}, {NULL}};
const Segment SGI353433[] = {{&S85574[125], 3}, {NULL}};
const Segment SGI353434[] = {{&S85622[5], 5}, {&S85622[5], 5}, {NULL}};
const Segment SGI353435[] = {{&S85622[11], 3}, {NULL}};
const Segment SGI353436[] = {{&S85622[131], 3}, {NULL}};
const Segment SGI353437[] = {{&S59549[0], 21}, {NULL}};
const Segment SGI353438[] = {{&S59549[34], 3}, {NULL}};
const Segment SGI353439[] = {{&S85637[42], 5}, {NULL}};
const Segment SGI353440[] = {{&S85637[6], 3}, {NULL}};
const Segment SGI353441[] = {{&S85637[68], 3}, {NULL}};
const Segment SGI353442[] = {{&S85638[43], 3}, {NULL}};
const Segment SGI353443[] = {{&S85638[6], 3}, {NULL}};
const Segment SGI353444[] = {{&S85638[67], 3}, {NULL}};
const Segment SGI353445[] = {{&S85647[5], 12}, {&S85647[5], 12}, {NULL}};
const Segment SGI353446[] = {{&S85647[98], 3}, {NULL}};
const Segment SGI353447[] = {{&S85682[23], 3}, {NULL}};
const Segment SGI353448[] = {{&S85682[0], 16}, {NULL}};
const Segment SGI353449[] = {{&S85686[38], 5}, {NULL}};
const Segment SGI353450[] = {{&S85686[94], 3}, {NULL}};
const Segment SGI353451[] = {{&S85700[41], 3}, {NULL}};
const Segment SGI353452[] = {{&S85700[6], 3}, {NULL}};
const Segment SGI353453[] = {{&S85700[65], 3}, {NULL}};
const Segment SGI353454[] = {{&S85701[42], 4}, {NULL}};
const Segment SGI353455[] = {{&S85701[6], 3}, {NULL}};
const Segment SGI353456[] = {{&S85701[67], 3}, {NULL}};
const Segment SGI353457[] = {{&S62554[74], 4}, {NULL}};
const Segment SGI353458[] = {{&S62556[133], 4}, {NULL}};
const Segment SGI353459[] = {{&S62556[42], 10}, {NULL}};
const Segment SGI353460[] = {{&S77831[28], 3}, {NULL}};
const Segment SGI353461[] = {{&S77831[102], 5}, {NULL}};
const Segment SGI353462[] = {{&S77832[28], 5}, {NULL}};
const Segment SGI353463[] = {{&S77832[104], 5}, {NULL}};
const Segment SGI353464[] = {{&S85727[43], 3}, {NULL}};
const Segment SGI353465[] = {{&S85727[6], 3}, {NULL}};
const Segment SGI353466[] = {{&S85727[67], 3}, {NULL}};
const Segment SGI353467[] = {{&S85755[43], 4}, {NULL}};
const Segment SGI353468[] = {{&S85755[6], 3}, {NULL}};
const Segment SGI353469[] = {{&S85755[68], 3}, {NULL}};
const Segment SGI353470[] = {{&S85759[5], 4}, {NULL}};
const Segment SGI353471[] = {{&S85799[35], 5}, {NULL}};
const Segment SGI353472[] = {{&S85799[52], 3}, {NULL}};
const Segment SGI353473[] = {{&S56728[71], 5}, {NULL}};
const Segment SGI353474[] = {{&S56728[12], 12}, {NULL}};
const Segment SGI353475[] = {{&S56729[73], 5}, {NULL}};
const Segment SGI353476[] = {{&S56729[12], 10}, {NULL}};
const Segment SGI353477[] = {{&S77957[28], 5}, {NULL}};
const Segment SGI353478[] = {{&S77957[104], 5}, {NULL}};
const Segment SGI353479[] = {{&S72340[54], 9}, {NULL}};
const Segment SGI353480[] = {{&S72340[45], 3}, {NULL}};
const Segment SGI353481[] = {{&S72341[5], 9}, {NULL}};
const Segment SGI353482[] = {{&S72341[19], 3}, {NULL}};
const Segment SGI353483[] = {{&S72342[49], 8}, {NULL}};
const Segment SGI353484[] = {{&S72342[40], 3}, {NULL}};
const Segment SGI353485[] = {{&S59597[6], 4}, {NULL}};
const Segment SGI353486[] = {{&S59597[70], 5}, {NULL}};
const Segment SGI353487[] = {{&S59597[12], 9}, {NULL}};
const Segment SGI353488[] = {{&S59597[29], 13}, {NULL}};
const Segment SGI353489[] = {{&S85853[5], 12}, {&S85853[5], 12}, {NULL}};
const Segment SGI353490[] = {{&S85853[76], 3}, {NULL}};
const Segment SGI353491[] = {{&S85853[145], 3}, {NULL}};
const Segment SGI353492[] = {{&S62736[63], 4}, {NULL}};
const Segment SGI353493[] = {{&S62736[13], 10}, {NULL}};
const Segment SGI353494[] = {{&S62738[53], 4}, {NULL}};
const Segment SGI353495[] = {{&S62738[13], 10}, {NULL}};
const Segment SGI353496[] = {{&S62739[53], 4}, {NULL}};
const Segment SGI353497[] = {{&S62739[13], 10}, {NULL}};
const Segment SGI353498[] = {{&S85858[5], 3}, {&S85858[5], 3}, {NULL}};
const Segment SGI353499[] = {{&S85858[9], 3}, {NULL}};
const Segment SGI353500[] = {{&S62742[63], 4}, {NULL}};
const Segment SGI353501[] = {{&S62742[13], 10}, {NULL}};
const Segment SGI353502[] = {{&S85858[127], 3}, {NULL}};
const Segment SGI353503[] = {{&S62744[53], 4}, {NULL}};
const Segment SGI353504[] = {{&S62744[13], 10}, {NULL}};
const Segment SGI353505[] = {{&S62747[53], 4}, {NULL}};
const Segment SGI353506[] = {{&S62747[13], 10}, {NULL}};
const Segment SGI353507[] = {{&S62749[53], 4}, {NULL}};
const Segment SGI353508[] = {{&S62749[13], 10}, {NULL}};
const Segment SGI353509[] = {{&S62750[53], 4}, {NULL}};
const Segment SGI353510[] = {{&S62750[13], 10}, {NULL}};
const Segment SGI353511[] = {{&S62751[53], 4}, {NULL}};
const Segment SGI353512[] = {{&S62751[13], 10}, {NULL}};
const Segment SGI353513[] = {{&S62752[63], 4}, {NULL}};
const Segment SGI353514[] = {{&S62752[13], 10}, {NULL}};
const Segment SGI353515[] = {{&S62753[63], 3}, {NULL}};
const Segment SGI353516[] = {{&S62753[13], 10}, {NULL}};
const Segment SGI353517[] = {{&S85902[5], 5}, {NULL}};
const Segment SGI353518[] = {{&S85902[21], 3}, {NULL}};
const Segment SGI353519[] = {{&S85903[5], 5}, {NULL}};
const Segment SGI353520[] = {{&S85903[21], 3}, {NULL}};
const Segment SGI353521[] = {{&S85908[5], 5}, {NULL}};
const Segment SGI353522[] = {{&S85908[21], 3}, {NULL}};
const Segment SGI353523[] = {{&S85924[35], 3}, {NULL}};
const Segment SGI353524[] = {{&S85924[50], 3}, {NULL}};
const Segment SGI353525[] = {{&S85927[5], 5}, {NULL}};
const Segment SGI353526[] = {{&S85927[21], 3}, {NULL}};
const Segment SGI353527[] = {{&S85939[35], 3}, {NULL}};
const Segment SGI353528[] = {{&S85939[89], 3}, {NULL}};
const Segment SGI353529[] = {{&S59723[6], 4}, {NULL}};
const Segment SGI353530[] = {{&S59723[58], 5}, {NULL}};
const Segment SGI353531[] = {{&S59723[12], 3}, {NULL}};
const Segment SGI353532[] = {{&S85980[5], 5}, {&S85980[105], 5}, {NULL}};
const Segment SGI353533[] = {{&S85980[69], 3}, {NULL}};
const Segment SGI353534[] = {{&S85980[131], 3}, {NULL}};
const Segment SGI353535[] = {{&S86001[43], 3}, {NULL}};
const Segment SGI353536[] = {{&S86001[6], 3}, {NULL}};
const Segment SGI353537[] = {{&S86001[67], 3}, {NULL}};
const Segment SGI353538[] = {{&S86041[35], 5}, {NULL}};
const Segment SGI353539[] = {{&S86041[52], 3}, {NULL}};
const Segment SGI353540[] = {{&S86050[34], 5}, {NULL}};
const Segment SGI353541[] = {{&S86050[50], 3}, {NULL}};
const Segment SGI353542[] = {{&S86061[41], 4}, {NULL}};
const Segment SGI353543[] = {{&S86061[6], 3}, {NULL}};
const Segment SGI353544[] = {{&S86061[66], 3}, {NULL}};
const Segment SGI353545[] = {{&S86062[5], 4}, {&S86062[5], 4}, {NULL}};
const Segment SGI353546[] = {{&S86062[10], 3}, {NULL}};
const Segment SGI353547[] = {{&S86062[130], 3}, {NULL}};
const Segment SGI353548[] = {{&S86063[5], 3}, {&S86063[5], 3}, {NULL}};
const Segment SGI353549[] = {{&S86063[9], 3}, {NULL}};
const Segment SGI353550[] = {{&S86063[126], 3}, {NULL}};
const Segment SGI353551[] = {{&S72475[5], 5}, {NULL}};
const Segment SGI353552[] = {{&S72475[31], 3}, {NULL}};
const Segment SGI353553[] = {{&S86105[5], 4}, {NULL}};
const Segment SGI353554[] = {{&S78268[28], 3}, {NULL}};
const Segment SGI353555[] = {{&S78268[102], 5}, {NULL}};
const Segment SGI353556[] = {{&S86111[5], 3}, {NULL}};
const Segment SGI353557[] = {{&S86141[5], 10}, {NULL}};
const Segment SGI353558[] = {{&S86141[21], 3}, {NULL}};
const Segment SGI353559[] = {{&S86162[42], 5}, {NULL}};
const Segment SGI353560[] = {{&S86162[6], 3}, {NULL}};
const Segment SGI353561[] = {{&S86162[68], 3}, {NULL}};
const Segment SGI353562[] = {{&S86163[43], 3}, {NULL}};
const Segment SGI353563[] = {{&S86163[6], 3}, {NULL}};
const Segment SGI353564[] = {{&S86163[67], 3}, {NULL}};
const Segment SGI353565[] = {{&S86173[43], 3}, {NULL}};
const Segment SGI353566[] = {{&S86173[6], 3}, {NULL}};
const Segment SGI353567[] = {{&S86173[67], 3}, {NULL}};
const Segment SGI353568[] = {{&S60221[0], 16}, {NULL}};
const Segment SGI353569[] = {{&S60221[29], 3}, {NULL}};
const Segment SGI353570[] = {{&S60221[73], 16}, {NULL}};
const Segment SGI353571[] = {{&S86194[42], 5}, {NULL}};
const Segment SGI353572[] = {{&S86194[6], 3}, {NULL}};
const Segment SGI353573[] = {{&S86194[68], 3}, {NULL}};
const Segment SGI353574[] = {{&S72528[5], 4}, {NULL}};
const Segment SGI353575[] = {{&S72528[47], 3}, {NULL}};
const Segment SGI353576[] = {{&S86203[43], 5}, {NULL}};
const Segment SGI353577[] = {{&S86203[6], 3}, {NULL}};
const Segment SGI353578[] = {{&S86203[69], 3}, {NULL}};
const Segment SGI353579[] = {{&S86226[5], 3}, {NULL}};
const Segment SGI353580[] = {{&S86226[19], 3}, {NULL}};
const Segment SGI353581[] = {{&S86227[5], 5}, {NULL}};
const Segment SGI353582[] = {{&S86227[21], 3}, {NULL}};
const Segment SGI353583[] = {{&S86228[5], 5}, {NULL}};
const Segment SGI353584[] = {{&S86228[21], 3}, {NULL}};
const Segment SGI353585[] = {{&S86229[5], 3}, {NULL}};
const Segment SGI353586[] = {{&S86229[14], 3}, {NULL}};
const Segment SGI353587[] = {{&S86230[5], 3}, {NULL}};
const Segment SGI353588[] = {{&S86230[19], 3}, {NULL}};
const Segment SGI353589[] = {{&S86233[5], 4}, {&S86233[5], 4}, {NULL}};
const Segment SGI353590[] = {{&S86233[82], 3}, {NULL}};
const Segment SGI353591[] = {{&S72573[5], 4}, {NULL}};
const Segment SGI353592[] = {{&S72573[47], 3}, {NULL}};
const Segment SGI353593[] = {{&S72573[101], 3}, {NULL}};
const Segment SGI353594[] = {{&S86246[25], 5}, {NULL}};
const Segment SGI353595[] = {{&S86246[0], 18}, {NULL}};
const Segment SGI353596[] = {{&S78418[28], 5}, {NULL}};
const Segment SGI353597[] = {{&S78418[104], 5}, {NULL}};
const Segment SGI353598[] = {{&S78419[28], 4}, {NULL}};
const Segment SGI353599[] = {{&S78419[110], 5}, {NULL}};
const Segment SGI353600[] = {{&S78420[28], 7}, {NULL}};
const Segment SGI353601[] = {{&S78420[113], 5}, {NULL}};
const Segment SGI353602[] = {{&S86286[21], 7}, {NULL}};
const Segment SGI353603[] = {{&S86286[0], 14}, {NULL}};
const Segment SGI353604[] = {{&S59963[6], 5}, {NULL}};
const Segment SGI353605[] = {{&S59963[13], 14}, {NULL}};
const Segment SGI353606[] = {{&S86316[5], 3}, {NULL}};
const Segment SGI353607[] = {{&S86316[19], 3}, {NULL}};
const Segment SGI353608[] = {{&S86345[21], 7}, {NULL}};
const Segment SGI353609[] = {{&S86345[0], 14}, {NULL}};
const Segment SGI353610[] = {{&S86356[35], 3}, {NULL}};
const Segment SGI353611[] = {{&S86356[89], 3}, {NULL}};
const Segment SGI353612[] = {{&S86374[5], 5}, {&S86374[5], 5}, {NULL}};
const Segment SGI353613[] = {{&S86374[11], 3}, {NULL}};
const Segment SGI353614[] = {{&S86374[131], 3}, {NULL}};
const Segment SGI353615[] = {{&S78587[28], 7}, {NULL}};
const Segment SGI353616[] = {{&S78587[106], 5}, {NULL}};
const Segment SGI353617[] = {{&S86418[5], 3}, {NULL}};
const Segment SGI353618[] = {{&S86418[19], 3}, {NULL}};
const Segment SGI353619[] = {{&S59981[6], 4}, {NULL}};
const Segment SGI353620[] = {{&S59981[90], 5}, {NULL}};
const Segment SGI353621[] = {{&S59981[12], 9}, {NULL}};
const Segment SGI353622[] = {{&S59981[29], 13}, {NULL}};
const Segment SGI353623[] = {{&S86429[5], 5}, {&S86429[68], 5}, {NULL}};
const Segment SGI353624[] = {{&S86429[84], 3}, {NULL}};
const Segment SGI353625[] = {{&S78613[28], 7}, {NULL}};
const Segment SGI353626[] = {{&S78613[106], 5}, {NULL}};
const Segment SGI353627[] = {{&S86447[5], 13}, {&S86447[105], 13}, {NULL}};
const Segment SGI353628[] = {{&S86447[169], 3}, {NULL}};
const Segment SGI353629[] = {{&S86455[21], 7}, {NULL}};
const Segment SGI353630[] = {{&S86455[0], 14}, {NULL}};
const Segment SGI353631[] = {{&S86489[101], 3}, {NULL}};
const Segment SGI353632[] = {{&S86490[43], 3}, {NULL}};
const Segment SGI353633[] = {{&S86490[6], 3}, {NULL}};
const Segment SGI353634[] = {{&S86490[67], 3}, {NULL}};
const Segment SGI353635[] = {{&S86509[5], 5}, {&S86509[105], 5}, {NULL}};
const Segment SGI353636[] = {{&S86509[11], 3}, {NULL}};
const Segment SGI353637[] = {{&S86509[131], 3}, {NULL}};
const Segment SGI353638[] = {{&S86524[5], 5}, {NULL}};
const Segment SGI353639[] = {{&S86524[169], 3}, {NULL}};
const Segment SGI353640[] = {{&S86529[5], 12}, {NULL}};
const Segment SGI353641[] = {{&S86557[5], 4}, {NULL}};
const Segment SGI353642[] = {{&S86557[20], 3}, {NULL}};
const Segment SGI353643[] = {{&S78772[28], 3}, {NULL}};
const Segment SGI353644[] = {{&S78772[102], 5}, {NULL}};
const Segment SGI353645[] = {{&S86649[43], 3}, {NULL}};
const Segment SGI353646[] = {{&S86649[6], 3}, {NULL}};
const Segment SGI353647[] = {{&S86649[67], 3}, {NULL}};
const Segment SGI353648[] = {{&S86667[5], 5}, {&S86667[68], 5}, {NULL}};
const Segment SGI353649[] = {{&S86667[84], 3}, {NULL}};
const Segment SGI353650[] = {{&S86679[35], 4}, {NULL}};
const Segment SGI353651[] = {{&S86679[51], 3}, {NULL}};
const Segment SGI353652[] = {{&S86680[5], 4}, {NULL}};
const Segment SGI353653[] = {{&S86680[15], 3}, {NULL}};
const Segment SGI353654[] = {{&S86685[23], 7}, {NULL}};
const Segment SGI353655[] = {{&S86685[0], 16}, {NULL}};
const Segment SGI353656[] = {{&S86687[25], 5}, {NULL}};
const Segment SGI353657[] = {{&S86687[0], 18}, {NULL}};
const Segment SGI353658[] = {{&S86691[35], 14}, {NULL}};
const Segment SGI353659[] = {{&S86691[100], 3}, {NULL}};
const Segment SGI353660[] = {{&S86703[35], 7}, {NULL}};
const Segment SGI353661[] = {{&S86703[93], 3}, {NULL}};
const Segment SGI353662[] = {{&S86732[101], 16}, {NULL}};
const Segment SGI353663[] = {{&S86789[5], 5}, {NULL}};
const Segment SGI353664[] = {{&S86789[21], 3}, {NULL}};
const Segment SGI353665[] = {{&S86803[35], 7}, {NULL}};
const Segment SGI353666[] = {{&S86803[93], 3}, {NULL}};
const Segment SGI353667[] = {{&S86812[35], 3}, {NULL}};
const Segment SGI353668[] = {{&S86812[89], 3}, {NULL}};
const Segment SGI353669[] = {{&S86818[21], 3}, {NULL}};
const Segment SGI353670[] = {{&S86818[0], 14}, {NULL}};
const Segment SGI353671[] = {{&S63763[30], 3}, {NULL}};
const Segment SGI353672[] = {{&S63763[13], 10}, {NULL}};
const Segment SGI353673[] = {{&S86900[5], 5}, {&S86900[68], 5}, {NULL}};
const Segment SGI353674[] = {{&S86900[84], 3}, {NULL}};
const Segment SGI353675[] = {{&S60470[6], 4}, {NULL}};
const Segment SGI353676[] = {{&S60470[53], 5}, {NULL}};
const Segment SGI353677[] = {{&S60470[12], 10}, {NULL}};
const Segment SGI353678[] = {{&S60471[6], 4}, {NULL}};
const Segment SGI353679[] = {{&S60471[53], 5}, {NULL}};
const Segment SGI353680[] = {{&S60471[12], 10}, {NULL}};
const Segment SGI353681[] = {{&S60472[6], 4}, {NULL}};
const Segment SGI353682[] = {{&S60472[53], 5}, {NULL}};
const Segment SGI353683[] = {{&S60472[12], 10}, {NULL}};
const Segment SGI353684[] = {{&S86918[5], 5}, {NULL}};
const Segment SGI353685[] = {{&S86918[21], 3}, {NULL}};
const Segment SGI353686[] = {{&S86931[35], 3}, {NULL}};
const Segment SGI353687[] = {{&S86931[50], 3}, {NULL}};
const Segment SGI353688[] = {{&S86935[34], 3}, {NULL}};
const Segment SGI353689[] = {{&S86935[48], 3}, {NULL}};
const Segment SGI353690[] = {{&S60510[6], 4}, {NULL}};
const Segment SGI353691[] = {{&S60510[90], 5}, {NULL}};
const Segment SGI353692[] = {{&S60510[12], 9}, {NULL}};
const Segment SGI353693[] = {{&S60510[29], 13}, {NULL}};
const Segment SGI353694[] = {{&S60516[6], 4}, {NULL}};
const Segment SGI353695[] = {{&S60516[70], 5}, {NULL}};
const Segment SGI353696[] = {{&S60516[12], 9}, {NULL}};
const Segment SGI353697[] = {{&S60516[29], 13}, {NULL}};
const Segment SGI353698[] = {{&S79059[28], 5}, {NULL}};
const Segment SGI353699[] = {{&S79059[104], 5}, {NULL}};
const Segment SGI353700[] = {{&S86979[43], 14}, {NULL}};
const Segment SGI353701[] = {{&S86979[6], 3}, {NULL}};
const Segment SGI353702[] = {{&S86979[78], 3}, {NULL}};
const Segment SGI353703[] = {{&S63916[6], 3}, {NULL}};
const Segment SGI353704[] = {{&S63916[11], 14}, {NULL}};
const Segment SGI353705[] = {{&S63917[6], 3}, {&S63917[6], 3}, {NULL}};
const Segment SGI353706[] = {{&S63917[11], 14}, {NULL}};
const Segment SGI353707[] = {{&S60734[6], 4}, {NULL}};
const Segment SGI353708[] = {{&S60734[57], 5}, {NULL}};
const Segment SGI353709[] = {{&S60734[12], 14}, {NULL}};
const Segment SGI353710[] = {{&S60735[6], 4}, {NULL}};
const Segment SGI353711[] = {{&S60735[12], 14}, {NULL}};
const Segment SGI353712[] = {{&S87036[5], 16}, {NULL}};
const Segment SGI353713[] = {{&S87036[32], 3}, {NULL}};
const Segment SGI353714[] = {{&S87037[5], 4}, {NULL}};
const Segment SGI353715[] = {{&S87037[20], 3}, {NULL}};
const Segment SGI353716[] = {{&S87038[5], 6}, {NULL}};
const Segment SGI353717[] = {{&S87038[22], 3}, {NULL}};
const Segment SGI353718[] = {{&S87041[5], 7}, {NULL}};
const Segment SGI353719[] = {{&S87041[23], 3}, {NULL}};
const Segment SGI353720[] = {{&S87042[5], 3}, {NULL}};
const Segment SGI353721[] = {{&S87042[19], 3}, {NULL}};
const Segment SGI353722[] = {{&S87043[5], 3}, {NULL}};
const Segment SGI353723[] = {{&S87043[19], 3}, {NULL}};
const Segment SGI353724[] = {{&S87044[5], 7}, {NULL}};
const Segment SGI353725[] = {{&S87044[23], 3}, {NULL}};
const Segment SGI353726[] = {{&S79310[28], 5}, {NULL}};
const Segment SGI353727[] = {{&S79310[104], 5}, {NULL}};
const Segment SGI353728[] = {{&S87054[35], 3}, {NULL}};
const Segment SGI353729[] = {{&S87054[50], 3}, {NULL}};
const Segment SGI353730[] = {{&S87061[5], 11}, {&S87061[5], 11}, {NULL}};
const Segment SGI353731[] = {{&S87061[125], 3}, {NULL}};
const Segment SGI353732[] = {{&S87083[5], 3}, {&S87083[5], 3}, {NULL}};
const Segment SGI353733[] = {{&S87083[80], 3}, {NULL}};
const Segment SGI353734[] = {{&S87085[5], 3}, {NULL}};
const Segment SGI353735[] = {{&S87085[233], 3}, {NULL}};
const Segment SGI353736[] = {{&S87097[5], 4}, {&S87097[5], 4}, {NULL}};
const Segment SGI353737[] = {{&S87097[10], 3}, {NULL}};
const Segment SGI353738[] = {{&S87097[128], 3}, {NULL}};
const Segment SGI353739[] = {{&S87112[5], 3}, {&S87112[5], 3}, {NULL}};
const Segment SGI353740[] = {{&S87112[80], 3}, {NULL}};
const Segment SGI353741[] = {{&S60860[6], 4}, {NULL}};
const Segment SGI353742[] = {{&S60860[78], 5}, {NULL}};
const Segment SGI353743[] = {{&S60860[12], 14}, {NULL}};
const Segment SGI353744[] = {{&S87168[42], 4}, {NULL}};
const Segment SGI353745[] = {{&S87168[6], 3}, {NULL}};
const Segment SGI353746[] = {{&S87168[67], 3}, {NULL}};
const Segment SGI353747[] = {{&S60895[6], 4}, {NULL}};
const Segment SGI353748[] = {{&S60895[59], 5}, {NULL}};
const Segment SGI353749[] = {{&S60895[12], 14}, {NULL}};
const Segment SGI353750[] = {{&S87183[5], 5}, {NULL}};
const Segment SGI353751[] = {{&S87204[35], 5}, {NULL}};
const Segment SGI353752[] = {{&S87204[52], 3}, {NULL}};
const Segment SGI353753[] = {{&S87220[5], 5}, {&S87220[104], 5}, {NULL}};
const Segment SGI353754[] = {{&S87220[69], 3}, {NULL}};
const Segment SGI353755[] = {{&S87220[130], 3}, {NULL}};
const Segment SGI353756[] = {{&S87232[44], 21}, {NULL}};
const Segment SGI353757[] = {{&S87232[6], 3}, {NULL}};
const Segment SGI353758[] = {{&S87232[86], 3}, {NULL}};
const Segment SGI353759[] = {{&S87235[42], 4}, {NULL}};
const Segment SGI353760[] = {{&S87235[6], 3}, {NULL}};
const Segment SGI353761[] = {{&S87235[67], 3}, {NULL}};
const Segment SGI353762[] = {{&S87238[5], 5}, {&S87238[105], 5}, {NULL}};
const Segment SGI353763[] = {{&S87238[11], 3}, {NULL}};
const Segment SGI353764[] = {{&S87238[131], 3}, {NULL}};
const Segment SGI353765[] = {{&S87239[5], 3}, {&S87239[5], 3}, {NULL}};
const Segment SGI353766[] = {{&S87239[67], 3}, {NULL}};
const Segment SGI353767[] = {{&S87239[127], 3}, {NULL}};
const Segment SGI353768[] = {{&S87242[43], 5}, {NULL}};
const Segment SGI353769[] = {{&S87242[6], 3}, {NULL}};
const Segment SGI353770[] = {{&S87242[69], 3}, {NULL}};
const Segment SGI353771[] = {{&S79588[28], 5}, {NULL}};
const Segment SGI353772[] = {{&S79588[104], 5}, {NULL}};
const Segment SGI353773[] = {{&S87270[43], 5}, {NULL}};
const Segment SGI353774[] = {{&S87270[6], 3}, {NULL}};
const Segment SGI353775[] = {{&S87270[69], 3}, {NULL}};
const Segment SGI353776[] = {{&S79624[28], 15}, {NULL}};
const Segment SGI353777[] = {{&S79624[111], 5}, {NULL}};
const Segment SGI353778[] = {{&S87292[5], 3}, {NULL}};
const Segment SGI353779[] = {{&S87292[19], 3}, {NULL}};
const Segment SGI353780[] = {{&S87297[5], 4}, {&S87297[5], 4}, {NULL}};
const Segment SGI353781[] = {{&S87297[82], 3}, {NULL}};
const Segment SGI353782[] = {{&S87304[5], 3}, {NULL}};
const Segment SGI353783[] = {{&S87304[19], 3}, {NULL}};
const Segment SGI353784[] = {{&S87314[35], 9}, {NULL}};
const Segment SGI353785[] = {{&S87314[56], 3}, {NULL}};
const Segment SGI353786[] = {{&S87315[35], 13}, {NULL}};
const Segment SGI353787[] = {{&S87315[60], 3}, {NULL}};
const Segment SGI353788[] = {{&S87318[5], 5}, {&S87318[5], 5}, {NULL}};
const Segment SGI353789[] = {{&S87318[153], 3}, {NULL}};
const Segment SGI353790[] = {{&S87342[42], 5}, {NULL}};
const Segment SGI353791[] = {{&S87342[6], 3}, {NULL}};
const Segment SGI353792[] = {{&S87342[68], 3}, {NULL}};
const Segment SGI353793[] = {{&S87343[42], 7}, {NULL}};
const Segment SGI353794[] = {{&S87343[6], 3}, {NULL}};
const Segment SGI353795[] = {{&S87343[70], 3}, {NULL}};
const Segment SGI353796[] = {{&S79694[28], 7}, {NULL}};
const Segment SGI353797[] = {{&S79694[106], 5}, {NULL}};
const Segment SGI353798[] = {{&S87351[5], 5}, {&S87351[97], 5}, {NULL}};
const Segment SGI353799[] = {{&S87351[153], 3}, {NULL}};
const Segment SGI353800[] = {{&S61144[6], 4}, {NULL}};
const Segment SGI353801[] = {{&S61144[98], 5}, {NULL}};
const Segment SGI353802[] = {{&S61144[12], 9}, {NULL}};
const Segment SGI353803[] = {{&S61144[29], 13}, {NULL}};
const Segment SGI353804[] = {{&S87374[43], 4}, {NULL}};
const Segment SGI353805[] = {{&S87374[6], 3}, {NULL}};
const Segment SGI353806[] = {{&S87374[68], 3}, {NULL}};
const Segment SGI353807[] = {{&S87375[43], 3}, {NULL}};
const Segment SGI353808[] = {{&S87375[6], 3}, {NULL}};
const Segment SGI353809[] = {{&S87375[67], 3}, {NULL}};
const Segment SGI353810[] = {{&S87396[43], 3}, {NULL}};
const Segment SGI353811[] = {{&S87396[6], 3}, {NULL}};
const Segment SGI353812[] = {{&S87396[67], 3}, {NULL}};
const Segment SGI353813[] = {{&S87412[5], 16}, {NULL}};
const Segment SGI353814[] = {{&S87412[22], 3}, {NULL}};
const Segment SGI353815[] = {{&S61208[6], 4}, {NULL}};
const Segment SGI353816[] = {{&S61208[70], 5}, {NULL}};
const Segment SGI353817[] = {{&S61208[12], 9}, {NULL}};
const Segment SGI353818[] = {{&S61208[29], 13}, {NULL}};
const Segment SGI353819[] = {{&S87485[43], 5}, {NULL}};
const Segment SGI353820[] = {{&S87485[6], 3}, {NULL}};
const Segment SGI353821[] = {{&S87485[69], 3}, {NULL}};
const Segment SGI353822[] = {{&S87492[43], 8}, {NULL}};
const Segment SGI353823[] = {{&S87492[6], 3}, {NULL}};
const Segment SGI353824[] = {{&S87492[72], 3}, {NULL}};
const Segment SGI353825[] = {{&S87493[43], 5}, {NULL}};
const Segment SGI353826[] = {{&S87493[6], 3}, {NULL}};
const Segment SGI353827[] = {{&S87493[69], 3}, {NULL}};
const Segment SGI353828[] = {{&S87513[5], 5}, {&S87513[5], 5}, {NULL}};
const Segment SGI353829[] = {{&S87513[11], 3}, {NULL}};
const Segment SGI353830[] = {{&S87513[131], 3}, {NULL}};
const Segment SGI353831[] = {{&S64581[20], 3}, {NULL}};
const Segment SGI353832[] = {{&S64581[25], 14}, {NULL}};
const Segment SGI353833[] = {{&S74102[46], 7}, {NULL}};
const Segment SGI353834[] = {{&S74102[8], 3}, {NULL}};
const Segment SGI353835[] = {{&S79888[28], 5}, {NULL}};
const Segment SGI353836[] = {{&S79888[104], 5}, {NULL}};
const Segment SGI353837[] = {{&S87555[5], 5}, {NULL}};
const Segment SGI353838[] = {{&S87555[21], 3}, {NULL}};
const Segment SGI353839[] = {{&S87557[35], 4}, {NULL}};
const Segment SGI353840[] = {{&S87557[51], 3}, {NULL}};
const Segment SGI353841[] = {{&S87561[35], 3}, {NULL}};
const Segment SGI353842[] = {{&S87561[50], 3}, {NULL}};
const Segment SGI353843[] = {{&S87570[23], 5}, {NULL}};
const Segment SGI353844[] = {{&S87570[0], 16}, {NULL}};
const Segment SGI353845[] = {{&S87576[34], 3}, {NULL}};
const Segment SGI353846[] = {{&S87576[48], 3}, {NULL}};
const Segment SGI353847[] = {{&S61339[6], 2}, {NULL}};
const Segment SGI353848[] = {{&S61339[11], 14}, {NULL}};
const Segment SGI353849[] = {{&S61340[6], 4}, {NULL}};
const Segment SGI353850[] = {{&S61340[12], 14}, {NULL}};
const Segment SGI353851[] = {{&S61341[6], 4}, {NULL}};
const Segment SGI353852[] = {{&S61341[12], 14}, {NULL}};
const Segment SGI353853[] = {{&S87603[21], 4}, {NULL}};
const Segment SGI353854[] = {{&S87603[0], 14}, {NULL}};
const Segment SGI353855[] = {{&S61344[6], 4}, {NULL}};
const Segment SGI353856[] = {{&S61344[12], 14}, {NULL}};
const Segment SGI353857[] = {{&S87607[43], 3}, {NULL}};
const Segment SGI353858[] = {{&S87607[6], 3}, {NULL}};
const Segment SGI353859[] = {{&S87607[67], 3}, {NULL}};
const Segment SGI353860[] = {{&S87609[49], 20}, {NULL}};
const Segment SGI353861[] = {{&S87610[49], 20}, {NULL}};
const Segment SGI353862[] = {{&S61362[6], 4}, {NULL}};
const Segment SGI353863[] = {{&S61362[12], 6}, {NULL}};
const Segment SGI353864[] = {{&S61364[6], 4}, {NULL}};
const Segment SGI353865[] = {{&S61364[12], 11}, {NULL}};
const Segment SGI353866[] = {{&S61369[6], 4}, {NULL}};
const Segment SGI353867[] = {{&S61369[12], 3}, {NULL}};
const Segment SGI353868[] = {{&S61371[6], 4}, {NULL}};
const Segment SGI353869[] = {{&S61371[12], 3}, {NULL}};
const Segment SGI353870[] = {{&S61372[6], 4}, {NULL}};
const Segment SGI353871[] = {{&S61372[12], 3}, {NULL}};
const Segment SGI353872[] = {{&S87657[5], 6}, {NULL}};
const Segment SGI353873[] = {{&S80002[28], 4}, {NULL}};
const Segment SGI353874[] = {{&S80002[103], 5}, {NULL}};
const Segment SGI353875[] = {{&S61405[6], 4}, {NULL}};
const Segment SGI353876[] = {{&S80003[28], 7}, {NULL}};
const Segment SGI353877[] = {{&S61405[78], 4}, {NULL}};
const Segment SGI353878[] = {{&S61405[12], 14}, {NULL}};
const Segment SGI353879[] = {{&S80003[106], 5}, {NULL}};
const Segment SGI353880[] = {{&S87665[5], 3}, {NULL}};
const Segment SGI353881[] = {{&S87665[19], 3}, {NULL}};
const Segment SGI353882[] = {{&S87703[5], 13}, {&S87703[112], 13}, {NULL}};
const Segment SGI353883[] = {{&S87703[19], 3}, {NULL}};
const Segment SGI353884[] = {{&S87703[146], 3}, {NULL}};
const Segment SGI353885[] = {{&S87714[5], 3}, {&S87714[5], 3}, {NULL}};
const Segment SGI353886[] = {{&S87714[80], 3}, {NULL}};
const Segment SGI353887[] = {{&S87767[5], 15}, {&S87767[115], 15}, {NULL}};
const Segment SGI353888[] = {{&S87767[21], 3}, {NULL}};
const Segment SGI353889[] = {{&S87767[151], 3}, {NULL}};
const Segment SGI353890[] = {{&S87771[5], 5}, {&S87771[105], 5}, {NULL}};
const Segment SGI353891[] = {{&S87771[69], 3}, {NULL}};
const Segment SGI353892[] = {{&S87771[131], 3}, {NULL}};
const Segment SGI353893[] = {{&S87793[5], 5}, {NULL}};
const Segment SGI353894[] = {{&S87793[21], 3}, {NULL}};
const Segment SGI353895[] = {{&S87794[5], 7}, {NULL}};
const Segment SGI353896[] = {{&S87794[23], 3}, {NULL}};
const Segment SGI353897[] = {{&S80138[28], 7}, {NULL}};
const Segment SGI353898[] = {{&S80138[106], 5}, {NULL}};
const Segment SGI353899[] = {{&S87828[41], 5}, {NULL}};
const Segment SGI353900[] = {{&S87828[6], 3}, {NULL}};
const Segment SGI353901[] = {{&S87828[67], 3}, {NULL}};
const Segment SGI353902[] = {{&S87834[5], 5}, {&S87834[5], 5}, {NULL}};
const Segment SGI353903[] = {{&S87834[11], 3}, {NULL}};
const Segment SGI353904[] = {{&S87834[130], 3}, {NULL}};
const Segment SGI353905[] = {{&S87841[34], 5}, {NULL}};
const Segment SGI353906[] = {{&S87841[90], 3}, {NULL}};
const Segment SGI353907[] = {{&S87861[43], 5}, {NULL}};
const Segment SGI353908[] = {{&S87861[6], 3}, {NULL}};
const Segment SGI353909[] = {{&S87861[69], 3}, {NULL}};
const Segment SGI353910[] = {{&S80208[28], 3}, {NULL}};
const Segment SGI353911[] = {{&S80208[102], 5}, {NULL}};
const Segment SGI353912[] = {{&S87876[42], 4}, {NULL}};
const Segment SGI353913[] = {{&S87876[6], 3}, {NULL}};
const Segment SGI353914[] = {{&S87876[67], 3}, {NULL}};
const Segment SGI353915[] = {{&S87892[42], 3}, {NULL}};
const Segment SGI353916[] = {{&S87892[6], 3}, {NULL}};
const Segment SGI353917[] = {{&S87892[66], 3}, {NULL}};
const Segment SGI353918[] = {{&S74406[5], 4}, {NULL}};
const Segment SGI353919[] = {{&S74406[47], 3}, {NULL}};
const Segment SGI353920[] = {{&S74406[101], 3}, {NULL}};
const Segment SGI353921[] = {{&S87898[43], 3}, {NULL}};
const Segment SGI353922[] = {{&S87898[6], 3}, {NULL}};
const Segment SGI353923[] = {{&S87898[67], 3}, {NULL}};
const Segment SGI353924[] = {{&S87901[5], 5}, {&S87901[5], 5}, {NULL}};
const Segment SGI353925[] = {{&S87901[11], 3}, {NULL}};
const Segment SGI353926[] = {{&S87901[131], 3}, {NULL}};
const Segment SGI353927[] = {{&S87902[5], 5}, {&S87902[5], 5}, {NULL}};
const Segment SGI353928[] = {{&S87902[69], 3}, {NULL}};
const Segment SGI353929[] = {{&S87902[131], 3}, {NULL}};
const Segment SGI353930[] = {{&S87905[5], 5}, {&S87905[68], 5}, {NULL}};
const Segment SGI353931[] = {{&S87905[84], 3}, {NULL}};
const Segment SGI353932[] = {{&S87924[5], 5}, {NULL}};
const Segment SGI353933[] = {{&S87924[21], 3}, {NULL}};
const Segment SGI353934[] = {{&S87925[5], 5}, {NULL}};
const Segment SGI353935[] = {{&S87925[21], 3}, {NULL}};
const Segment SGI353936[] = {{&S87942[38], 3}, {NULL}};
const Segment SGI353937[] = {{&S87942[53], 3}, {NULL}};
const Segment SGI353938[] = {{&S87943[35], 4}, {NULL}};
const Segment SGI353939[] = {{&S87943[51], 3}, {NULL}};
const Segment SGI353940[] = {{&S87948[5], 3}, {NULL}};
const Segment SGI353941[] = {{&S87948[233], 3}, {NULL}};
const Segment SGI353942[] = {{&S64927[152], 4}, {NULL}};
const Segment SGI353943[] = {{&S64927[158], 3}, {NULL}};
const Segment SGI353944[] = {{&S64927[180], 3}, {NULL}};
const Segment SGI353945[] = {{&S64927[195], 5}, {NULL}};
const Segment SGI353946[] = {{&S88021[43], 12}, {NULL}};
const Segment SGI353947[] = {{&S88021[6], 3}, {NULL}};
const Segment SGI353948[] = {{&S88021[76], 3}, {NULL}};
const Segment SGI353949[] = {{&S62165[0], 16}, {NULL}};
const Segment SGI353950[] = {{&S62165[29], 3}, {NULL}};
const Segment SGI353951[] = {{&S62165[73], 16}, {NULL}};
const Segment SGI353952[] = {{&S88036[5], 5}, {&S88036[5], 5}, {NULL}};
const Segment SGI353953[] = {{&S88036[69], 3}, {NULL}};
const Segment SGI353954[] = {{&S88036[131], 3}, {NULL}};
const Segment SGI353955[] = {{&S88040[5], 4}, {NULL}};
const Segment SGI353956[] = {{&S88042[5], 5}, {NULL}};
const Segment SGI353957[] = {{&S88045[5], 3}, {NULL}};
const Segment SGI353958[] = {{&S88045[19], 3}, {NULL}};
const Segment SGI353959[] = {{&S88049[5], 5}, {NULL}};
const Segment SGI353960[] = {{&S80405[28], 7}, {NULL}};
const Segment SGI353961[] = {{&S80405[106], 5}, {NULL}};
const Segment SGI353962[] = {{&S88059[5], 3}, {NULL}};
const Segment SGI353963[] = {{&S88059[19], 3}, {NULL}};
const Segment SGI353964[] = {{&S88071[5], 3}, {&S88071[5], 3}, {NULL}};
const Segment SGI353965[] = {{&S88071[109], 3}, {NULL}};
const Segment SGI353966[] = {{&S88077[35], 4}, {NULL}};
const Segment SGI353967[] = {{&S88077[90], 3}, {NULL}};
const Segment SGI353968[] = {{&S88078[41], 13}, {NULL}};
const Segment SGI353969[] = {{&S88078[6], 3}, {NULL}};
const Segment SGI353970[] = {{&S88078[75], 3}, {NULL}};
const Segment SGI353971[] = {{&S61670[6], 4}, {NULL}};
const Segment SGI353972[] = {{&S61670[12], 3}, {NULL}};
const Segment SGI353973[] = {{&S61726[6], 4}, {NULL}};
const Segment SGI353974[] = {{&S61726[77], 5}, {NULL}};
const Segment SGI353975[] = {{&S61726[12], 9}, {NULL}};
const Segment SGI353976[] = {{&S61726[29], 13}, {NULL}};
const Segment SGI353977[] = {{&S61727[6], 4}, {NULL}};
const Segment SGI353978[] = {{&S61727[77], 5}, {NULL}};
const Segment SGI353979[] = {{&S61727[12], 9}, {NULL}};
const Segment SGI353980[] = {{&S61727[29], 13}, {NULL}};
const Segment SGI353981[] = {{&S88155[43], 8}, {NULL}};
const Segment SGI353982[] = {{&S88155[6], 3}, {NULL}};
const Segment SGI353983[] = {{&S88155[72], 3}, {NULL}};
const Segment SGI353984[] = {{&S61757[6], 4}, {NULL}};
const Segment SGI353985[] = {{&S61757[78], 5}, {NULL}};
const Segment SGI353986[] = {{&S61757[12], 13}, {NULL}};
const Segment SGI353987[] = {{&S61758[6], 4}, {NULL}};
const Segment SGI353988[] = {{&S61758[78], 5}, {NULL}};
const Segment SGI353989[] = {{&S61758[12], 13}, {NULL}};
const Segment SGI353990[] = {{&S61759[6], 4}, {NULL}};
const Segment SGI353991[] = {{&S61759[78], 5}, {NULL}};
const Segment SGI353992[] = {{&S61759[12], 13}, {&S61759[89], 4}, {NULL}};
const Segment SGI353993[] = {{&S61760[6], 4}, {NULL}};
const Segment SGI353994[] = {{&S61760[78], 5}, {NULL}};
const Segment SGI353995[] = {{&S61760[12], 13}, {&S61760[89], 4}, {NULL}};
const Segment SGI353996[] = {{&S61773[6], 4}, {NULL}};
const Segment SGI353997[] = {{&S61773[90], 5}, {NULL}};
const Segment SGI353998[] = {{&S61773[12], 9}, {NULL}};
const Segment SGI353999[] = {{&S61773[29], 13}, {NULL}};
const Segment SGI354000[] = {{&S88197[5], 5}, {NULL}};
const Segment SGI354001[] = {{&S88197[21], 3}, {NULL}};
const Segment SGI354002[] = {{&S88240[43], 7}, {NULL}};
const Segment SGI354003[] = {{&S88240[6], 3}, {NULL}};
const Segment SGI354004[] = {{&S88240[71], 3}, {NULL}};
const Segment SGI354005[] = {{&S88248[42], 5}, {NULL}};
const Segment SGI354006[] = {{&S88248[6], 3}, {NULL}};
const Segment SGI354007[] = {{&S88248[68], 3}, {NULL}};
const Segment SGI354008[] = {{&S88249[42], 4}, {NULL}};
const Segment SGI354009[] = {{&S88249[6], 3}, {NULL}};
const Segment SGI354010[] = {{&S88249[67], 3}, {NULL}};
const Segment SGI354011[] = {{&S88250[39], 4}, {NULL}};
const Segment SGI354012[] = {{&S88250[6], 3}, {NULL}};
const Segment SGI354013[] = {{&S88250[64], 3}, {NULL}};
const Segment SGI354014[] = {{&S74715[5], 7}, {&S74715[148], 7}, {NULL}};
const Segment SGI354015[] = {{&S74715[47], 3}, {NULL}};
const Segment SGI354016[] = {{&S74716[5], 7}, {NULL}};
const Segment SGI354017[] = {{&S74716[47], 3}, {NULL}};
const Segment SGI354018[] = {{&S88269[5], 5}, {NULL}};
const Segment SGI354019[] = {{&S88269[169], 3}, {NULL}};
const Segment SGI354020[] = {{&S88291[5], 16}, {NULL}};
const Segment SGI354021[] = {{&S88291[32], 3}, {NULL}};
const Segment SGI354022[] = {{&S88293[5], 5}, {NULL}};
const Segment SGI354023[] = {{&S88293[21], 3}, {NULL}};
const Segment SGI354024[] = {{&S88348[21], 4}, {NULL}};
const Segment SGI354025[] = {{&S88348[0], 14}, {NULL}};
const Segment SGI354026[] = {{&S88397[5], 3}, {NULL}};
const Segment SGI354027[] = {{&S88397[167], 3}, {NULL}};
const Segment SGI354028[] = {{&S88403[5], 5}, {NULL}};
const Segment SGI354029[] = {{&S88403[169], 3}, {NULL}};
const Segment SGI354030[] = {{&S88409[5], 7}, {&S88409[5], 7}, {NULL}};
const Segment SGI354031[] = {{&S88409[88], 3}, {NULL}};
const Segment SGI354032[] = {{&S88410[5], 4}, {NULL}};
const Segment SGI354033[] = {{&S88410[20], 3}, {NULL}};
const Segment SGI354034[] = {{&S88411[5], 5}, {NULL}};
const Segment SGI354035[] = {{&S88411[21], 3}, {NULL}};
const Segment SGI354036[] = {{&S80733[28], 7}, {NULL}};
const Segment SGI354037[] = {{&S80733[106], 5}, {NULL}};
const Segment SGI354038[] = {{&S88412[5], 5}, {NULL}};
const Segment SGI354039[] = {{&S88412[21], 3}, {NULL}};
const Segment SGI354040[] = {{&S80734[28], 5}, {NULL}};
const Segment SGI354041[] = {{&S80734[104], 5}, {NULL}};
const Segment SGI354042[] = {{&S62018[6], 4}, {NULL}};
const Segment SGI354043[] = {{&S62018[57], 5}, {NULL}};
const Segment SGI354044[] = {{&S62018[12], 14}, {NULL}};
const Segment SGI354045[] = {{&S62019[6], 4}, {NULL}};
const Segment SGI354046[] = {{&S62019[57], 5}, {NULL}};
const Segment SGI354047[] = {{&S62019[12], 14}, {NULL}};
const Segment SGI354048[] = {{&S88429[5], 3}, {NULL}};
const Segment SGI354049[] = {{&S88429[19], 3}, {NULL}};
const Segment SGI354050[] = {{&S88440[5], 5}, {NULL}};
const Segment SGI354051[] = {{&S88440[21], 3}, {NULL}};
const Segment SGI354052[] = {{&S59027[76], 5}, {NULL}};
const Segment SGI354053[] = {{&S59027[12], 3}, {NULL}};
const Segment SGI354054[] = {{&S88458[34], 4}, {NULL}};
const Segment SGI354055[] = {{&S88458[89], 3}, {NULL}};
const Segment SGI354056[] = {{&S88477[5], 4}, {&S88477[5], 4}, {NULL}};
const Segment SGI354057[] = {{&S88477[10], 3}, {NULL}};
const Segment SGI354058[] = {{&S88477[129], 3}, {NULL}};
const Segment SGI354059[] = {{&S62146[6], 4}, {NULL}};
const Segment SGI354060[] = {{&S62146[12], 14}, {NULL}};
const Segment SGI354061[] = {{&S62209[6], 5}, {NULL}};
const Segment SGI354062[] = {{&S62209[13], 11}, {NULL}};
const Segment SGI354063[] = {{&S62210[6], 4}, {NULL}};
const Segment SGI354064[] = {{&S62210[106], 5}, {NULL}};
const Segment SGI354065[] = {{&S62210[12], 9}, {NULL}};
const Segment SGI354066[] = {{&S62211[6], 5}, {NULL}};
const Segment SGI354067[] = {{&S62211[13], 10}, {NULL}};
const Segment SGI354068[] = {{&S62212[6], 4}, {NULL}};
const Segment SGI354069[] = {{&S62212[112], 5}, {NULL}};
const Segment SGI354070[] = {{&S62212[12], 10}, {NULL}};
const Segment SGI354071[] = {{&S88508[5], 4}, {&S88508[5], 4}, {NULL}};
const Segment SGI354072[] = {{&S88508[10], 3}, {NULL}};
const Segment SGI354073[] = {{&S88508[129], 3}, {NULL}};
const Segment SGI354074[] = {{&S62213[6], 5}, {NULL}};
const Segment SGI354075[] = {{&S62213[13], 10}, {&S62213[41], 4}, {&S62213[92], 13}, {NULL}};
const Segment SGI354076[] = {{&S62214[6], 4}, {NULL}};
const Segment SGI354077[] = {{&S62214[133], 5}, {NULL}};
const Segment SGI354078[] = {{&S62214[12], 9}, {&S62214[39], 4}, {&S62214[90], 13}, {NULL}};
const Segment SGI354079[] = {{&S62215[6], 4}, {NULL}};
const Segment SGI354080[] = {{&S62215[107], 5}, {NULL}};
const Segment SGI354081[] = {{&S62215[12], 9}, {NULL}};
const Segment SGI354082[] = {{&S62216[6], 4}, {NULL}};
const Segment SGI354083[] = {{&S62216[97], 5}, {NULL}};
const Segment SGI354084[] = {{&S62216[12], 12}, {NULL}};
const Segment SGI354085[] = {{&S62217[6], 5}, {NULL}};
const Segment SGI354086[] = {{&S62217[13], 10}, {&S62217[82], 11}, {NULL}};
const Segment SGI354087[] = {{&S62218[6], 4}, {NULL}};
const Segment SGI354088[] = {{&S62218[133], 5}, {NULL}};
const Segment SGI354089[] = {{&S62218[12], 9}, {&S62218[39], 4}, {&S62218[90], 13}, {NULL}};
const Segment SGI354090[] = {{&S62219[6], 4}, {NULL}};
const Segment SGI354091[] = {{&S62219[93], 5}, {NULL}};
const Segment SGI354092[] = {{&S62219[12], 12}, {NULL}};
const Segment SGI354093[] = {{&S62220[6], 4}, {NULL}};
const Segment SGI354094[] = {{&S62220[110], 5}, {NULL}};
const Segment SGI354095[] = {{&S62220[12], 12}, {NULL}};
const Segment SGI354096[] = {{&S62239[6], 3}, {NULL}};
const Segment SGI354097[] = {{&S62239[11], 17}, {NULL}};
const Segment SGI354098[] = {{&S65308[48], 4}, {NULL}};
const Segment SGI354099[] = {{&S65308[13], 10}, {NULL}};
const Segment SGI354100[] = {{&S65309[49], 4}, {NULL}};
const Segment SGI354101[] = {{&S65309[13], 10}, {NULL}};
const Segment SGI354102[] = {{&S81005[28], 4}, {NULL}};
const Segment SGI354103[] = {{&S81005[103], 5}, {NULL}};
const Segment SGI354104[] = {{&S88605[38], 5}, {NULL}};
const Segment SGI354105[] = {{&S88605[55], 3}, {NULL}};
const Segment SGI354106[] = {{&S75166[5], 5}, {NULL}};
const Segment SGI354107[] = {{&S75166[11], 3}, {NULL}};
const Segment SGI354108[] = {{&S75167[5], 3}, {NULL}};
const Segment SGI354109[] = {{&S75167[24], 3}, {NULL}};
const Segment SGI354110[] = {{&S62304[6], 4}, {NULL}};
const Segment SGI354111[] = {{&S62304[12], 3}, {NULL}};
const Segment SGI354112[] = {{&S88611[5], 5}, {&S88611[97], 5}, {NULL}};
const Segment SGI354113[] = {{&S88611[153], 3}, {NULL}};
const Segment SGI354114[] = {{&S62368[6], 4}, {NULL}};
const Segment SGI354115[] = {{&S62368[76], 5}, {NULL}};
const Segment SGI354116[] = {{&S62368[12], 9}, {NULL}};
const Segment SGI354117[] = {{&S62368[29], 13}, {NULL}};
const Segment SGI354118[] = {{&S62369[6], 4}, {NULL}};
const Segment SGI354119[] = {{&S62369[76], 5}, {NULL}};
const Segment SGI354120[] = {{&S62369[12], 9}, {NULL}};
const Segment SGI354121[] = {{&S62369[29], 13}, {NULL}};
const Segment SGI354122[] = {{&S62370[6], 4}, {NULL}};
const Segment SGI354123[] = {{&S62370[85], 5}, {NULL}};
const Segment SGI354124[] = {{&S62370[12], 13}, {NULL}};
const Segment SGI354125[] = {{&S62371[6], 4}, {NULL}};
const Segment SGI354126[] = {{&S62371[85], 5}, {NULL}};
const Segment SGI354127[] = {{&S62371[12], 13}, {NULL}};
const Segment SGI354128[] = {{&S62372[6], 4}, {NULL}};
const Segment SGI354129[] = {{&S62372[83], 5}, {NULL}};
const Segment SGI354130[] = {{&S62372[12], 11}, {NULL}};
const Segment SGI354131[] = {{&S62373[6], 4}, {NULL}};
const Segment SGI354132[] = {{&S62373[88], 5}, {NULL}};
const Segment SGI354133[] = {{&S62373[12], 13}, {NULL}};
const Segment SGI354134[] = {{&S75240[49], 5}, {NULL}};
const Segment SGI354135[] = {{&S75240[40], 3}, {NULL}};
const Segment SGI354136[] = {{&S88707[5], 7}, {NULL}};
const Segment SGI354137[] = {{&S88707[23], 3}, {NULL}};
const Segment SGI354138[] = {{&S88708[5], 5}, {NULL}};
const Segment SGI354139[] = {{&S88708[21], 3}, {NULL}};
const Segment SGI354140[] = {{&S59335[11], 14}, {NULL}};
const Segment SGI354141[] = {{&S59336[11], 13}, {NULL}};
const Segment SGI354142[] = {{&S88720[35], 4}, {NULL}};
const Segment SGI354143[] = {{&S88720[51], 3}, {NULL}};
const Segment SGI354144[] = {{&S88728[35], 5}, {NULL}};
const Segment SGI354145[] = {{&S88728[52], 3}, {NULL}};
const Segment SGI354146[] = {{&S88730[35], 4}, {NULL}};
const Segment SGI354147[] = {{&S88730[51], 3}, {NULL}};
const Segment SGI354148[] = {{&S88767[101], 5}, {NULL}};
const Segment SGI354149[] = {{&S88775[5], 3}, {&S88775[5], 3}, {NULL}};
const Segment SGI354150[] = {{&S88775[149], 3}, {NULL}};
const Segment SGI354151[] = {{&S88799[43], 4}, {NULL}};
const Segment SGI354152[] = {{&S88799[6], 3}, {NULL}};
const Segment SGI354153[] = {{&S88799[68], 3}, {NULL}};
const Segment SGI354154[] = {{&S88810[5], 3}, {NULL}};
const Segment SGI354155[] = {{&S88810[167], 3}, {NULL}};
const Segment SGI354156[] = {{&S88833[5], 5}, {NULL}};
const Segment SGI354157[] = {{&S88833[21], 3}, {NULL}};
const Segment SGI354158[] = {{&S88834[5], 7}, {NULL}};
const Segment SGI354159[] = {{&S88834[23], 3}, {NULL}};
const Segment SGI354160[] = {{&S88835[5], 8}, {NULL}};
const Segment SGI354161[] = {{&S88835[24], 3}, {NULL}};
const Segment SGI354162[] = {{&S88836[5], 8}, {NULL}};
const Segment SGI354163[] = {{&S88836[19], 3}, {NULL}};
const Segment SGI354164[] = {{&S88866[21], 13}, {NULL}};
const Segment SGI354165[] = {{&S88866[0], 14}, {NULL}};
const Segment SGI354166[] = {{&S88876[5], 11}, {&S88876[5], 11}, {NULL}};
const Segment SGI354167[] = {{&S88876[17], 3}, {NULL}};
const Segment SGI354168[] = {{&S88876[142], 3}, {NULL}};
const Segment SGI354169[] = {{&S88944[5], 5}, {NULL}};
const Segment SGI354170[] = {{&S88944[169], 3}, {NULL}};
const Segment SGI354171[] = {{&S81448[28], 5}, {NULL}};
const Segment SGI354172[] = {{&S81448[104], 5}, {NULL}};
const Segment SGI354173[] = {{&S88965[5], 6}, {NULL}};
const Segment SGI354174[] = {{&S62737[6], 4}, {NULL}};
const Segment SGI354175[] = {{&S62737[12], 10}, {NULL}};
const Segment SGI354176[] = {{&S62740[6], 4}, {NULL}};
const Segment SGI354177[] = {{&S62740[12], 10}, {NULL}};
const Segment SGI354178[] = {{&S62741[6], 4}, {NULL}};
const Segment SGI354179[] = {{&S62741[12], 10}, {NULL}};
const Segment SGI354180[] = {{&S62743[6], 3}, {NULL}};
const Segment SGI354181[] = {{&S62743[11], 10}, {NULL}};
const Segment SGI354182[] = {{&S62745[6], 4}, {NULL}};
const Segment SGI354183[] = {{&S62745[12], 10}, {NULL}};
const Segment SGI354184[] = {{&S62746[6], 4}, {NULL}};
const Segment SGI354185[] = {{&S62746[12], 10}, {NULL}};
const Segment SGI354186[] = {{&S62748[6], 4}, {NULL}};
const Segment SGI354187[] = {{&S62748[12], 10}, {NULL}};
const Segment SGI354188[] = {{&S89000[35], 3}, {NULL}};
const Segment SGI354189[] = {{&S89000[89], 3}, {NULL}};
const Segment SGI354190[] = {{&S62754[6], 4}, {NULL}};
const Segment SGI354191[] = {{&S62754[12], 10}, {NULL}};
const Segment SGI354192[] = {{&S89004[5], 11}, {&S89004[5], 11}, {NULL}};
const Segment SGI354193[] = {{&S89004[165], 3}, {NULL}};
const Segment SGI354194[] = {{&S89010[35], 5}, {NULL}};
const Segment SGI354195[] = {{&S89010[91], 3}, {NULL}};
const Segment SGI354196[] = {{&S65804[7], 5}, {NULL}};
const Segment SGI354197[] = {{&S62849[6], 4}, {NULL}};
const Segment SGI354198[] = {{&S62849[85], 5}, {NULL}};
const Segment SGI354199[] = {{&S62849[12], 13}, {NULL}};
const Segment SGI354200[] = {{&S62850[6], 4}, {NULL}};
const Segment SGI354201[] = {{&S62850[85], 5}, {NULL}};
const Segment SGI354202[] = {{&S62850[12], 13}, {NULL}};
const Segment SGI354203[] = {{&S89127[35], 3}, {NULL}};
const Segment SGI354204[] = {{&S89127[50], 3}, {NULL}};
const Segment SGI354205[] = {{&S89128[35], 5}, {NULL}};
const Segment SGI354206[] = {{&S89128[52], 3}, {NULL}};
const Segment SGI354207[] = {{&S89129[34], 5}, {NULL}};
const Segment SGI354208[] = {{&S89129[50], 3}, {NULL}};
const Segment SGI354209[] = {{&S89132[5], 5}, {NULL}};
const Segment SGI354210[] = {{&S89132[16], 3}, {NULL}};
const Segment SGI354211[] = {{&S89133[35], 4}, {NULL}};
const Segment SGI354212[] = {{&S89133[51], 3}, {NULL}};
const Segment SGI354213[] = {{&S89148[5], 5}, {&S89148[97], 5}, {NULL}};
const Segment SGI354214[] = {{&S89148[113], 3}, {NULL}};
const Segment SGI354215[] = {{&S89149[5], 15}, {&S89149[107], 15}, {NULL}};
const Segment SGI354216[] = {{&S89149[173], 3}, {NULL}};
const Segment SGI354217[] = {{&S89153[41], 4}, {NULL}};
const Segment SGI354218[] = {{&S89153[6], 3}, {NULL}};
const Segment SGI354219[] = {{&S89153[66], 3}, {NULL}};
const Segment SGI354220[] = {{&S89155[5], 3}, {&S89155[5], 3}, {NULL}};
const Segment SGI354221[] = {{&S89155[9], 3}, {NULL}};
const Segment SGI354222[] = {{&S89155[126], 3}, {NULL}};
const Segment SGI354223[] = {{&S89203[42], 3}, {NULL}};
const Segment SGI354224[] = {{&S89203[6], 3}, {NULL}};
const Segment SGI354225[] = {{&S89203[66], 3}, {NULL}};
const Segment SGI354226[] = {{&S89218[43], 3}, {NULL}};
const Segment SGI354227[] = {{&S89218[6], 3}, {NULL}};
const Segment SGI354228[] = {{&S89218[67], 3}, {NULL}};
const Segment SGI354229[] = {{&S89224[5], 10}, {NULL}};
const Segment SGI354230[] = {{&S89225[5], 5}, {NULL}};
const Segment SGI354231[] = {{&S89232[5], 5}, {NULL}};
const Segment SGI354232[] = {{&S89232[21], 3}, {NULL}};
const Segment SGI354233[] = {{&S81598[28], 7}, {NULL}};
const Segment SGI354234[] = {{&S81598[106], 5}, {NULL}};
const Segment SGI354235[] = {{&S89241[5], 5}, {&S89241[68], 5}, {NULL}};
const Segment SGI354236[] = {{&S89241[84], 3}, {NULL}};
const Segment SGI354237[] = {{&S89252[5], 4}, {&S89252[5], 4}, {NULL}};
const Segment SGI354238[] = {{&S89252[82], 3}, {NULL}};
const Segment SGI354239[] = {{&S89261[23], 5}, {NULL}};
const Segment SGI354240[] = {{&S89261[0], 16}, {NULL}};
const Segment SGI354241[] = {{&S75765[5], 5}, {NULL}};
const Segment SGI354242[] = {{&S75765[11], 3}, {NULL}};
const Segment SGI354243[] = {{&S75766[5], 5}, {NULL}};
const Segment SGI354244[] = {{&S75766[11], 3}, {NULL}};
const Segment SGI354245[] = {{&S89300[42], 7}, {NULL}};
const Segment SGI354246[] = {{&S89300[6], 3}, {NULL}};
const Segment SGI354247[] = {{&S89300[70], 3}, {NULL}};
const Segment SGI354248[] = {{&S60008[17], 17}, {NULL}};
const Segment SGI354249[] = {{&S89320[101], 5}, {NULL}};
const Segment SGI354250[] = {{&S89322[5], 5}, {&S89322[105], 5}, {NULL}};
const Segment SGI354251[] = {{&S89322[69], 3}, {NULL}};
const Segment SGI354252[] = {{&S89322[131], 3}, {NULL}};
const Segment SGI354253[] = {{&S89353[43], 3}, {NULL}};
const Segment SGI354254[] = {{&S89353[6], 3}, {NULL}};
const Segment SGI354255[] = {{&S89353[67], 3}, {NULL}};
const Segment SGI354256[] = {{&S75832[19], 14}, {NULL}};
const Segment SGI354257[] = {{&S75832[47], 3}, {NULL}};
const Segment SGI354258[] = {{&S89369[43], 3}, {NULL}};
const Segment SGI354259[] = {{&S89369[6], 3}, {NULL}};
const Segment SGI354260[] = {{&S89369[67], 3}, {NULL}};
const Segment SGI354261[] = {{&S89390[5], 4}, {NULL}};
const Segment SGI354262[] = {{&S89395[5], 3}, {NULL}};
const Segment SGI354263[] = {{&S89395[19], 3}, {NULL}};
const Segment SGI354264[] = {{&S89397[5], 5}, {NULL}};
const Segment SGI354265[] = {{&S89397[21], 3}, {NULL}};
const Segment SGI354266[] = {{&S89398[5], 7}, {NULL}};
const Segment SGI354267[] = {{&S89398[23], 3}, {NULL}};
const Segment SGI354268[] = {{&S81741[28], 5}, {NULL}};
const Segment SGI354269[] = {{&S81741[104], 5}, {NULL}};
const Segment SGI354270[] = {{&S89411[5], 3}, {NULL}};
const Segment SGI354271[] = {{&S89411[19], 3}, {NULL}};
const Segment SGI354272[] = {{&S89415[5], 5}, {&S89415[68], 5}, {NULL}};
const Segment SGI354273[] = {{&S89415[84], 3}, {NULL}};
const Segment SGI354274[] = {{&S89427[35], 3}, {NULL}};
const Segment SGI354275[] = {{&S89427[50], 3}, {NULL}};
const Segment SGI354276[] = {{&S89428[34], 5}, {NULL}};
const Segment SGI354277[] = {{&S89428[50], 3}, {NULL}};
const Segment SGI354278[] = {{&S89473[42], 3}, {NULL}};
const Segment SGI354279[] = {{&S66239[62], 4}, {NULL}};
const Segment SGI354280[] = {{&S89473[6], 3}, {NULL}};
const Segment SGI354281[] = {{&S66239[13], 10}, {NULL}};
const Segment SGI354282[] = {{&S89473[66], 3}, {NULL}};
const Segment SGI354283[] = {{&S89474[42], 7}, {NULL}};
const Segment SGI354284[] = {{&S89474[6], 3}, {NULL}};
const Segment SGI354285[] = {{&S89474[70], 3}, {NULL}};
const Segment SGI354286[] = {{&S89489[43], 3}, {NULL}};
const Segment SGI354287[] = {{&S89489[6], 3}, {NULL}};
const Segment SGI354288[] = {{&S89489[67], 3}, {NULL}};
const Segment SGI354289[] = {{&S89491[5], 11}, {&S89491[5], 11}, {NULL}};
const Segment SGI354290[] = {{&S89491[17], 3}, {NULL}};
const Segment SGI354291[] = {{&S89491[143], 3}, {NULL}};
const Segment SGI354292[] = {{&S89505[42], 3}, {NULL}};
const Segment SGI354293[] = {{&S89505[6], 3}, {NULL}};
const Segment SGI354294[] = {{&S89505[66], 3}, {NULL}};
const Segment SGI354295[] = {{&S89511[43], 5}, {NULL}};
const Segment SGI354296[] = {{&S89511[6], 3}, {NULL}};
const Segment SGI354297[] = {{&S89511[69], 3}, {NULL}};
const Segment SGI354298[] = {{&S89512[43], 7}, {NULL}};
const Segment SGI354299[] = {{&S89512[6], 3}, {NULL}};
const Segment SGI354300[] = {{&S89512[71], 3}, {NULL}};
const Segment SGI354301[] = {{&S89548[5], 4}, {NULL}};
const Segment SGI354302[] = {{&S89564[5], 5}, {NULL}};
const Segment SGI354303[] = {{&S89581[35], 5}, {NULL}};
const Segment SGI354304[] = {{&S89581[52], 3}, {NULL}};
const Segment SGI354305[] = {{&S66356[64], 5}, {NULL}};
const Segment SGI354306[] = {{&S66356[13], 10}, {NULL}};
const Segment SGI354307[] = {{&S89590[38], 3}, {NULL}};
const Segment SGI354308[] = {{&S89590[53], 3}, {NULL}};
const Segment SGI354309[] = {{&S89596[35], 7}, {NULL}};
const Segment SGI354310[] = {{&S89596[93], 3}, {NULL}};
const Segment SGI354311[] = {{&S89622[21], 5}, {NULL}};
const Segment SGI354312[] = {{&S89622[0], 14}, {NULL}};
const Segment SGI354313[] = {{&S89637[101], 5}, {NULL}};
const Segment SGI354314[] = {{&S63278[6], 4}, {NULL}};
const Segment SGI354315[] = {{&S63278[57], 5}, {NULL}};
const Segment SGI354316[] = {{&S63278[12], 14}, {NULL}};
const Segment SGI354317[] = {{&S63279[6], 4}, {NULL}};
const Segment SGI354318[] = {{&S63279[57], 5}, {NULL}};
const Segment SGI354319[] = {{&S63279[12], 14}, {NULL}};
const Segment SGI354320[] = {{&S63280[6], 4}, {NULL}};
const Segment SGI354321[] = {{&S63280[57], 5}, {NULL}};
const Segment SGI354322[] = {{&S63280[12], 14}, {NULL}};
const Segment SGI354323[] = {{&S63281[6], 4}, {NULL}};
const Segment SGI354324[] = {{&S63281[57], 5}, {NULL}};
const Segment SGI354325[] = {{&S63281[12], 14}, {NULL}};
const Segment SGI354326[] = {{&S89690[5], 3}, {&S89690[5], 3}, {NULL}};
const Segment SGI354327[] = {{&S89690[67], 3}, {NULL}};
const Segment SGI354328[] = {{&S89690[127], 3}, {NULL}};
const Segment SGI354329[] = {{&S82038[28], 5}, {NULL}};
const Segment SGI354330[] = {{&S82038[104], 5}, {NULL}};
const Segment SGI354331[] = {{&S89698[5], 5}, {NULL}};
const Segment SGI354332[] = {{&S89698[21], 3}, {NULL}};
const Segment SGI354333[] = {{&S89699[5], 5}, {NULL}};
const Segment SGI354334[] = {{&S89699[21], 3}, {NULL}};
const Segment SGI354335[] = {{&S89707[5], 5}, {NULL}};
const Segment SGI354336[] = {{&S89707[21], 3}, {NULL}};
const Segment SGI354337[] = {{&S89727[35], 5}, {NULL}};
const Segment SGI354338[] = {{&S89727[52], 3}, {NULL}};
const Segment SGI354339[] = {{&S63380[6], 4}, {NULL}};
const Segment SGI354340[] = {{&S63380[66], 5}, {NULL}};
const Segment SGI354341[] = {{&S89750[5], 3}, {&S89750[5], 3}, {NULL}};
const Segment SGI354342[] = {{&S63380[12], 14}, {NULL}};
const Segment SGI354343[] = {{&S89750[9], 3}, {NULL}};
const Segment SGI354344[] = {{&S89750[126], 3}, {NULL}};
const Segment SGI354345[] = {{&S63381[6], 4}, {NULL}};
const Segment SGI354346[] = {{&S63381[89], 5}, {NULL}};
const Segment SGI354347[] = {{&S63381[12], 14}, {&S63381[48], 11}, {NULL}};
const Segment SGI354348[] = {{&S63381[12], 14}, {NULL}};
const Segment SGI354349[] = {{&S63472[6], 4}, {NULL}};
const Segment SGI354350[] = {{&S63472[73], 5}, {NULL}};
const Segment SGI354351[] = {{&S63472[12], 3}, {NULL}};
const Segment SGI354352[] = {{&S63472[37], 10}, {NULL}};
const Segment SGI354353[] = {{&S89850[5], 5}, {NULL}};
const Segment SGI354354[] = {{&S89884[35], 7}, {NULL}};
const Segment SGI354355[] = {{&S89884[54], 3}, {NULL}};
const Segment SGI354356[] = {{&S89902[5], 10}, {NULL}};
const Segment SGI354357[] = {{&S89902[21], 3}, {NULL}};
const Segment SGI354358[] = {{&S82339[28], 3}, {NULL}};
const Segment SGI354359[] = {{&S82339[102], 5}, {NULL}};
const Segment SGI354360[] = {{&S66834[8], 3}, {NULL}};
const Segment SGI354361[] = {{&S66834[13], 15}, {NULL}};
const Segment SGI354362[] = {{&S63645[6], 4}, {NULL}};
const Segment SGI354363[] = {{&S63645[12], 14}, {NULL}};
const Segment SGI354364[] = {{&S63646[6], 4}, {NULL}};
const Segment SGI354365[] = {{&S63646[12], 14}, {NULL}};
const Segment SGI354366[] = {{&S63647[6], 4}, {NULL}};
const Segment SGI354367[] = {{&S63647[12], 14}, {NULL}};
const Segment SGI354368[] = {{&S63648[6], 3}, {NULL}};
const Segment SGI354369[] = {{&S63648[11], 14}, {NULL}};
const Segment SGI354370[] = {{&S76615[19], 15}, {NULL}};
const Segment SGI354371[] = {{&S76615[48], 3}, {NULL}};
const Segment SGI354372[] = {{&S89974[5], 5}, {&S89974[105], 5}, {NULL}};
const Segment SGI354373[] = {{&S89974[11], 3}, {NULL}};
const Segment SGI354374[] = {{&S89974[131], 3}, {NULL}};
const Segment SGI354375[] = {{&S90005[5], 5}, {&S90005[105], 5}, {NULL}};
const Segment SGI354376[] = {{&S90005[11], 3}, {NULL}};
const Segment SGI354377[] = {{&S90005[131], 3}, {NULL}};
const Segment SGI354378[] = {{&S63762[6], 3}, {NULL}};
const Segment SGI354379[] = {{&S63762[11], 17}, {NULL}};
const Segment SGI354380[] = {{&S82538[28], 7}, {NULL}};
const Segment SGI354381[] = {{&S82538[106], 5}, {NULL}};
const Segment SGI354382[] = {{&S90010[5], 4}, {NULL}};
const Segment SGI354383[] = {{&S90021[5], 5}, {NULL}};
const Segment SGI354384[] = {{&S90022[5], 3}, {NULL}};
const Segment SGI354385[] = {{&S90023[5], 4}, {NULL}};
const Segment SGI354386[] = {{&S90029[5], 5}, {&S90029[68], 5}, {NULL}};
const Segment SGI354387[] = {{&S90029[84], 3}, {NULL}};
const Segment SGI354388[] = {{&S90041[5], 4}, {&S90041[5], 4}, {NULL}};
const Segment SGI354389[] = {{&S90041[82], 3}, {NULL}};
const Segment SGI354390[] = {{&S90053[5], 15}, {&S90053[107], 15}, {NULL}};
const Segment SGI354391[] = {{&S90053[133], 3}, {NULL}};
const Segment SGI354392[] = {{&S90069[5], 3}, {&S90069[5], 3}, {NULL}};
const Segment SGI354393[] = {{&S90069[9], 3}, {NULL}};
const Segment SGI354394[] = {{&S90069[126], 3}, {NULL}};
const Segment SGI354395[] = {{&S90070[5], 11}, {&S90070[5], 11}, {NULL}};
const Segment SGI354396[] = {{&S90070[165], 3}, {NULL}};
const Segment SGI354397[] = {{&S90090[42], 4}, {NULL}};
const Segment SGI354398[] = {{&S90090[6], 3}, {NULL}};
const Segment SGI354399[] = {{&S90090[67], 3}, {NULL}};
const Segment SGI354400[] = {{&S90161[43], 4}, {NULL}};
const Segment SGI354401[] = {{&S90161[6], 3}, {NULL}};
const Segment SGI354402[] = {{&S90161[68], 3}, {NULL}};
const Segment SGI354403[] = {{&S90185[5], 4}, {NULL}};
const Segment SGI354404[] = {{&S90185[19], 3}, {NULL}};
const Segment SGI354405[] = {{&S90186[5], 4}, {NULL}};
const Segment SGI354406[] = {{&S90186[20], 3}, {NULL}};
const Segment SGI354407[] = {{&S90197[5], 3}, {NULL}};
const Segment SGI354408[] = {{&S90197[19], 3}, {NULL}};
const Segment SGI354409[] = {{&S90198[5], 4}, {NULL}};
const Segment SGI354410[] = {{&S90198[20], 3}, {NULL}};
const Segment SGI354411[] = {{&S90199[5], 5}, {&S90199[68], 5}, {NULL}};
const Segment SGI354412[] = {{&S90199[84], 3}, {NULL}};
const Segment SGI354413[] = {{&S90200[5], 5}, {NULL}};
const Segment SGI354414[] = {{&S90200[21], 3}, {NULL}};
const Segment SGI354415[] = {{&S90205[5], 3}, {NULL}};
const Segment SGI354416[] = {{&S90205[19], 3}, {NULL}};
const Segment SGI354417[] = {{&S90219[5], 11}, {&S90219[5], 11}, {NULL}};
const Segment SGI354418[] = {{&S90219[125], 3}, {NULL}};
const Segment SGI354419[] = {{&S90222[21], 5}, {NULL}};
const Segment SGI354420[] = {{&S90222[0], 14}, {NULL}};
const Segment SGI354421[] = {{&S90229[5], 3}, {&S90229[5], 3}, {NULL}};
const Segment SGI354422[] = {{&S90229[80], 3}, {NULL}};
const Segment SGI354423[] = {{&S64015[6], 4}, {NULL}};
const Segment SGI354424[] = {{&S64015[12], 12}, {&S64015[67], 15}, {NULL}};
const Segment SGI354425[] = {{&S64016[6], 4}, {NULL}};
const Segment SGI354426[] = {{&S64016[12], 3}, {NULL}};
const Segment SGI354427[] = {{&S64017[6], 4}, {NULL}};
const Segment SGI354428[] = {{&S64017[12], 15}, {NULL}};
const Segment SGI354429[] = {{&S64018[6], 4}, {NULL}};
const Segment SGI354430[] = {{&S64018[12], 3}, {NULL}};
const Segment SGI354431[] = {{&S64019[6], 4}, {NULL}};
const Segment SGI354432[] = {{&S64019[12], 3}, {NULL}};
const Segment SGI354433[] = {{&S82830[28], 3}, {NULL}};
const Segment SGI354434[] = {{&S82830[102], 5}, {NULL}};
const Segment SGI354435[] = {{&S64071[12], 13}, {NULL}};
const Segment SGI354436[] = {{&S90291[21], 11}, {NULL}};
const Segment SGI354437[] = {{&S90291[0], 14}, {NULL}};
const Segment SGI354438[] = {{&S64072[6], 4}, {NULL}};
const Segment SGI354439[] = {{&S64072[83], 5}, {NULL}};
const Segment SGI354440[] = {{&S64072[12], 11}, {NULL}};
const Segment SGI354441[] = {{&S64073[12], 13}, {NULL}};
const Segment SGI354442[] = {{&S64096[6], 5}, {NULL}};
const Segment SGI354443[] = {{&S64096[13], 14}, {NULL}};
const Segment SGI354444[] = {{&S64097[6], 5}, {NULL}};
const Segment SGI354445[] = {{&S64097[60], 5}, {NULL}};
const Segment SGI354446[] = {{&S64097[13], 14}, {NULL}};
const Segment SGI354447[] = {{&S64101[6], 4}, {NULL}};
const Segment SGI354448[] = {{&S64101[78], 5}, {NULL}};
const Segment SGI354449[] = {{&S64101[12], 13}, {NULL}};
const Segment SGI354450[] = {{&S64102[6], 4}, {NULL}};
const Segment SGI354451[] = {{&S64102[78], 5}, {NULL}};
const Segment SGI354452[] = {{&S64102[12], 13}, {NULL}};
const Segment SGI354453[] = {{&S64103[6], 4}, {NULL}};
const Segment SGI354454[] = {{&S64103[78], 5}, {NULL}};
const Segment SGI354455[] = {{&S64103[12], 13}, {&S64103[89], 4}, {NULL}};
const Segment SGI354456[] = {{&S64104[6], 4}, {NULL}};
const Segment SGI354457[] = {{&S64104[76], 5}, {NULL}};
const Segment SGI354458[] = {{&S64104[12], 11}, {NULL}};
const Segment SGI354459[] = {{&S64105[6], 4}, {NULL}};
const Segment SGI354460[] = {{&S64105[78], 5}, {NULL}};
const Segment SGI354461[] = {{&S64105[12], 13}, {&S64105[89], 4}, {NULL}};
const Segment SGI354462[] = {{&S64107[6], 4}, {NULL}};
const Segment SGI354463[] = {{&S64107[58], 5}, {NULL}};
const Segment SGI354464[] = {{&S64107[12], 14}, {NULL}};
const Segment SGI354465[] = {{&S82875[28], 5}, {NULL}};
const Segment SGI354466[] = {{&S82875[104], 5}, {NULL}};
const Segment SGI354467[] = {{&S82876[28], 7}, {NULL}};
const Segment SGI354468[] = {{&S82876[106], 5}, {NULL}};
const Segment SGI354469[] = {{&S90347[45], 11}, {NULL}};
const Segment SGI354470[] = {{&S90347[6], 3}, {NULL}};
const Segment SGI354471[] = {{&S90347[77], 3}, {NULL}};
const Segment SGI354472[] = {{&S76956[5], 3}, {NULL}};
const Segment SGI354473[] = {{&S76956[9], 3}, {NULL}};
const Segment SGI354474[] = {{&S76957[5], 5}, {NULL}};
const Segment SGI354475[] = {{&S76957[11], 3}, {NULL}};
const Segment SGI354476[] = {{&S76958[5], 5}, {NULL}};
const Segment SGI354477[] = {{&S76958[11], 3}, {NULL}};
const Segment SGI354478[] = {{&S90373[5], 5}, {NULL}};
const Segment SGI354479[] = {{&S90373[21], 3}, {NULL}};
const Segment SGI354480[] = {{&S90374[5], 5}, {NULL}};
const Segment SGI354481[] = {{&S90374[21], 3}, {NULL}};
const Segment SGI354482[] = {{&S90375[5], 13}, {&S90375[76], 13}, {NULL}};
const Segment SGI354483[] = {{&S90375[100], 3}, {NULL}};
const Segment SGI354484[] = {{&S64171[6], 4}, {NULL}};
const Segment SGI354485[] = {{&S64171[70], 5}, {NULL}};
const Segment SGI354486[] = {{&S64171[12], 9}, {NULL}};
const Segment SGI354487[] = {{&S64171[29], 13}, {NULL}};
const Segment SGI354488[] = {{&S90406[38], 3}, {NULL}};
const Segment SGI354489[] = {{&S90406[53], 3}, {NULL}};
const Segment SGI354490[] = {{&S90434[5], 5}, {&S90434[97], 5}, {NULL}};
const Segment SGI354491[] = {{&S90434[113], 3}, {NULL}};
const Segment SGI354492[] = {{&S90435[35], 5}, {NULL}};
const Segment SGI354493[] = {{&S90435[91], 3}, {NULL}};
const Segment SGI354494[] = {{&S90548[5], 4}, {NULL}};
const Segment SGI354495[] = {{&S90548[20], 3}, {NULL}};
const Segment SGI354496[] = {{&S90571[35], 3}, {NULL}};
const Segment SGI354497[] = {{&S90571[50], 3}, {NULL}};
const Segment SGI354498[] = {{&S90575[5], 3}, {&S90575[5], 3}, {NULL}};
const Segment SGI354499[] = {{&S90575[80], 3}, {NULL}};
const Segment SGI354500[] = {{&S90587[5], 15}, {&S90587[114], 15}, {NULL}};
const Segment SGI354501[] = {{&S90587[21], 3}, {NULL}};
const Segment SGI354502[] = {{&S90587[150], 3}, {NULL}};
const Segment SGI354503[] = {{&S90591[21], 5}, {NULL}};
const Segment SGI354504[] = {{&S90591[0], 14}, {NULL}};
const Segment SGI354505[] = {{&S83105[28], 5}, {NULL}};
const Segment SGI354506[] = {{&S83105[104], 5}, {NULL}};
const Segment SGI354507[] = {{&S90628[42], 4}, {NULL}};
const Segment SGI354508[] = {{&S90628[6], 3}, {NULL}};
const Segment SGI354509[] = {{&S90628[67], 3}, {NULL}};
const Segment SGI354510[] = {{&S90630[101], 5}, {NULL}};
const Segment SGI354511[] = {{&S90638[43], 4}, {NULL}};
const Segment SGI354512[] = {{&S90638[6], 3}, {NULL}};
const Segment SGI354513[] = {{&S90638[68], 3}, {NULL}};
const Segment SGI354514[] = {{&S90640[21], 5}, {NULL}};
const Segment SGI354515[] = {{&S90640[0], 14}, {NULL}};
const Segment SGI354516[] = {{&S90648[45], 11}, {NULL}};
const Segment SGI354517[] = {{&S90648[6], 3}, {NULL}};
const Segment SGI354518[] = {{&S90648[77], 3}, {NULL}};
const Segment SGI354519[] = {{&S61156[81], 5}, {NULL}};
const Segment SGI354520[] = {{&S61156[12], 12}, {NULL}};
const Segment SGI354521[] = {{&S61157[81], 5}, {NULL}};
const Segment SGI354522[] = {{&S61157[12], 12}, {NULL}};
const Segment SGI354523[] = {{&S90741[5], 5}, {NULL}};
const Segment SGI354524[] = {{&S90742[5], 5}, {NULL}};
const Segment SGI354525[] = {{&S90773[5], 16}, {NULL}};
const Segment SGI354526[] = {{&S90773[27], 3}, {NULL}};
const Segment SGI354527[] = {{&S90774[5], 17}, {NULL}};
const Segment SGI354528[] = {{&S90774[28], 3}, {NULL}};
const Segment SGI354529[] = {{&S90775[34], 7}, {NULL}};
const Segment SGI354530[] = {{&S90775[52], 3}, {NULL}};
const Segment SGI354531[] = {{&S90791[23], 5}, {NULL}};
const Segment SGI354532[] = {{&S90791[0], 16}, {NULL}};
const Segment SGI354533[] = {{&S90807[35], 3}, {NULL}};
const Segment SGI354534[] = {{&S90807[89], 3}, {NULL}};
const Segment SGI354535[] = {{&S90814[5], 3}, {&S90814[5], 3}, {NULL}};
const Segment SGI354536[] = {{&S90814[107], 3}, {NULL}};
const Segment SGI354537[] = {{&S90816[5], 3}, {&S90816[5], 3}, {NULL}};
const Segment SGI354538[] = {{&S90816[67], 3}, {NULL}};
const Segment SGI354539[] = {{&S90816[126], 3}, {NULL}};
const Segment SGI354540[] = {{&S90874[5], 4}, {&S90874[5], 4}, {NULL}};
const Segment SGI354541[] = {{&S90874[10], 3}, {NULL}};
const Segment SGI354542[] = {{&S90874[129], 3}, {NULL}};
const Segment SGI354543[] = {{&S90928[43], 14}, {NULL}};
const Segment SGI354544[] = {{&S90928[6], 3}, {NULL}};
const Segment SGI354545[] = {{&S90928[78], 3}, {NULL}};
const Segment SGI354546[] = {{&S90936[5], 4}, {NULL}};
const Segment SGI354547[] = {{&S90942[5], 10}, {NULL}};
const Segment SGI354548[] = {{&S90943[5], 5}, {NULL}};
const Segment SGI354549[] = {{&S90947[5], 9}, {NULL}};
const Segment SGI354550[] = {{&S90958[5], 3}, {&S90958[5], 3}, {NULL}};
const Segment SGI354551[] = {{&S90958[80], 3}, {NULL}};
const Segment SGI354552[] = {{&S90959[34], 5}, {NULL}};
const Segment SGI354553[] = {{&S90959[50], 3}, {NULL}};
const Segment SGI354554[] = {{&S77530[5], 3}, {NULL}};
const Segment SGI354555[] = {{&S77530[31], 3}, {NULL}};
const Segment SGI354556[] = {{&S77530[85], 3}, {NULL}};
const Segment SGI354557[] = {{&S90981[41], 5}, {NULL}};
const Segment SGI354558[] = {{&S90981[6], 3}, {NULL}};
const Segment SGI354559[] = {{&S90981[67], 3}, {NULL}};
const Segment SGI354560[] = {{&S90990[5], 3}, {NULL}};
const Segment SGI354561[] = {{&S64839[6], 4}, {NULL}};
const Segment SGI354562[] = {{&S64839[70], 5}, {NULL}};
const Segment SGI354563[] = {{&S64839[12], 3}, {NULL}};
const Segment SGI354564[] = {{&S64839[34], 7}, {NULL}};
const Segment SGI354565[] = {{&S91025[42], 5}, {NULL}};
const Segment SGI354566[] = {{&S91025[6], 3}, {NULL}};
const Segment SGI354567[] = {{&S91025[68], 3}, {NULL}};
const Segment SGI354568[] = {{&S91026[43], 3}, {NULL}};
const Segment SGI354569[] = {{&S91026[6], 3}, {NULL}};
const Segment SGI354570[] = {{&S91026[67], 3}, {NULL}};
const Segment SGI354571[] = {{&S91034[5], 3}, {&S91034[5], 3}, {NULL}};
const Segment SGI354572[] = {{&S91034[9], 3}, {NULL}};
const Segment SGI354573[] = {{&S91034[127], 3}, {NULL}};
const Segment SGI354574[] = {{&S91043[21], 3}, {NULL}};
const Segment SGI354575[] = {{&S91043[0], 14}, {NULL}};
const Segment SGI354576[] = {{&S91044[21], 3}, {NULL}};
const Segment SGI354577[] = {{&S91044[0], 14}, {NULL}};
const Segment SGI354578[] = {{&S91068[43], 7}, {NULL}};
const Segment SGI354579[] = {{&S91068[6], 3}, {NULL}};
const Segment SGI354580[] = {{&S91068[71], 3}, {NULL}};
const Segment SGI354581[] = {{&S91078[5], 7}, {NULL}};
const Segment SGI354582[] = {{&S91078[171], 3}, {NULL}};
const Segment SGI354583[] = {{&S91084[43], 4}, {NULL}};
const Segment SGI354584[] = {{&S91084[6], 3}, {NULL}};
const Segment SGI354585[] = {{&S91084[68], 3}, {NULL}};
const Segment SGI354586[] = {{&S91095[5], 4}, {NULL}};
const Segment SGI354587[] = {{&S91096[5], 4}, {NULL}};
const Segment SGI354588[] = {{&S91119[5], 5}, {NULL}};
const Segment SGI354589[] = {{&S91119[21], 3}, {NULL}};
const Segment SGI354590[] = {{&S91120[5], 3}, {NULL}};
const Segment SGI354591[] = {{&S91120[19], 3}, {NULL}};
const Segment SGI354592[] = {{&S64956[6], 4}, {NULL}};
const Segment SGI354593[] = {{&S64956[53], 5}, {NULL}};
const Segment SGI354594[] = {{&S64956[12], 10}, {NULL}};
const Segment SGI354595[] = {{&S64958[6], 4}, {NULL}};
const Segment SGI354596[] = {{&S64958[59], 5}, {NULL}};
const Segment SGI354597[] = {{&S64958[12], 14}, {NULL}};
const Segment SGI354598[] = {{&S64959[6], 4}, {NULL}};
const Segment SGI354599[] = {{&S64959[77], 5}, {NULL}};
const Segment SGI354600[] = {{&S64959[12], 9}, {NULL}};
const Segment SGI354601[] = {{&S64959[29], 13}, {NULL}};
const Segment SGI354602[] = {{&S64960[6], 4}, {NULL}};
const Segment SGI354603[] = {{&S64960[77], 5}, {NULL}};
const Segment SGI354604[] = {{&S64960[12], 9}, {NULL}};
const Segment SGI354605[] = {{&S64960[29], 13}, {NULL}};
const Segment SGI354606[] = {{&S91151[23], 5}, {NULL}};
const Segment SGI354607[] = {{&S91151[0], 16}, {NULL}};
const Segment SGI354608[] = {{&S91159[23], 5}, {NULL}};
const Segment SGI354609[] = {{&S91159[0], 16}, {NULL}};
const Segment SGI354610[] = {{&S91167[35], 5}, {NULL}};
const Segment SGI354611[] = {{&S91167[52], 3}, {NULL}};
const Segment SGI354612[] = {{&S65041[6], 4}, {NULL}};
const Segment SGI354613[] = {{&S65041[12], 10}, {NULL}};
const Segment SGI354614[] = {{&S65042[6], 4}, {NULL}};
const Segment SGI354615[] = {{&S65042[12], 10}, {NULL}};
const Segment SGI354616[] = {{&S65043[6], 4}, {NULL}};
const Segment SGI354617[] = {{&S65043[12], 10}, {NULL}};
const Segment SGI354618[] = {{&S65044[6], 4}, {NULL}};
const Segment SGI354619[] = {{&S65044[12], 10}, {NULL}};
const Segment SGI354620[] = {{&S91226[5], 3}, {&S91226[5], 3}, {NULL}};
const Segment SGI354621[] = {{&S91226[80], 3}, {NULL}};
const Segment SGI354622[] = {{&S91241[5], 3}, {NULL}};
const Segment SGI354623[] = {{&S91241[167], 3}, {NULL}};
const Segment SGI354624[] = {{&S91243[43], 7}, {NULL}};
const Segment SGI354625[] = {{&S91243[6], 3}, {NULL}};
const Segment SGI354626[] = {{&S91243[71], 3}, {NULL}};
const Segment SGI354627[] = {{&S91247[5], 5}, {NULL}};
const Segment SGI354628[] = {{&S91247[169], 3}, {NULL}};
const Segment SGI354629[] = {{&S65080[6], 4}, {NULL}};
const Segment SGI354630[] = {{&S65080[12], 14}, {NULL}};
const Segment SGI354631[] = {{&S65081[6], 4}, {NULL}};
const Segment SGI354632[] = {{&S65081[12], 14}, {NULL}};
const Segment SGI354633[] = {{&S91276[5], 10}, {NULL}};
const Segment SGI354634[] = {{&S91285[5], 4}, {NULL}};
const Segment SGI354635[] = {{&S91285[20], 3}, {NULL}};
const Segment SGI354636[] = {{&S91315[5], 5}, {NULL}};
const Segment SGI354637[] = {{&S91315[21], 3}, {NULL}};
const Segment SGI354638[] = {{&S91323[38], 3}, {NULL}};
const Segment SGI354639[] = {{&S91323[92], 3}, {NULL}};
const Segment SGI354640[] = {{&S91326[41], 5}, {NULL}};
const Segment SGI354641[] = {{&S91326[6], 3}, {NULL}};
const Segment SGI354642[] = {{&S91326[67], 3}, {NULL}};
const Segment SGI354643[] = {{&S91327[41], 3}, {NULL}};
const Segment SGI354644[] = {{&S91327[6], 3}, {NULL}};
const Segment SGI354645[] = {{&S91327[65], 3}, {NULL}};
const Segment SGI354646[] = {{&S91371[42], 4}, {NULL}};
const Segment SGI354647[] = {{&S91371[6], 3}, {NULL}};
const Segment SGI354648[] = {{&S91371[67], 3}, {NULL}};
const Segment SGI354649[] = {{&S91406[101], 5}, {NULL}};
const Segment SGI354650[] = {{&S65243[6], 4}, {NULL}};
const Segment SGI354651[] = {{&S65243[76], 5}, {NULL}};
const Segment SGI354652[] = {{&S65243[12], 9}, {NULL}};
const Segment SGI354653[] = {{&S65243[29], 13}, {NULL}};
const Segment SGI354654[] = {{&S65244[6], 4}, {NULL}};
const Segment SGI354655[] = {{&S65244[98], 5}, {NULL}};
const Segment SGI354656[] = {{&S65244[12], 9}, {NULL}};
const Segment SGI354657[] = {{&S65244[29], 13}, {NULL}};
const Segment SGI354658[] = {{&S91421[86], 11}, {NULL}};
const Segment SGI354659[] = {{&S91445[5], 5}, {NULL}};
const Segment SGI354660[] = {{&S91445[169], 3}, {NULL}};
const Segment SGI354661[] = {{&S65304[6], 4}, {NULL}};
const Segment SGI354662[] = {{&S65304[90], 5}, {NULL}};
const Segment SGI354663[] = {{&S65304[12], 9}, {NULL}};
const Segment SGI354664[] = {{&S65304[29], 13}, {NULL}};
const Segment SGI354665[] = {{&S83561[28], 5}, {NULL}};
const Segment SGI354666[] = {{&S83561[104], 5}, {NULL}};
const Segment SGI354667[] = {{&S91483[5], 5}, {&S91483[68], 5}, {NULL}};
const Segment SGI354668[] = {{&S91483[84], 3}, {NULL}};
const Segment SGI354669[] = {{&S91484[5], 5}, {NULL}};
const Segment SGI354670[] = {{&S91484[21], 3}, {NULL}};
const Segment SGI354671[] = {{&S91509[5], 3}, {&S91509[5], 3}, {NULL}};
const Segment SGI354672[] = {{&S91509[109], 3}, {NULL}};
const Segment SGI354673[] = {{&S91514[35], 3}, {NULL}};
const Segment SGI354674[] = {{&S91514[50], 3}, {NULL}};
const Segment SGI354675[] = {{&S91519[35], 4}, {NULL}};
const Segment SGI354676[] = {{&S91519[51], 3}, {NULL}};
const Segment SGI354677[] = {{&S91522[5], 3}, {NULL}};
const Segment SGI354678[] = {{&S91529[5], 3}, {&S91529[5], 3}, {NULL}};
const Segment SGI354679[] = {{&S91529[80], 3}, {NULL}};
const Segment SGI354680[] = {{&S91531[25], 5}, {NULL}};
const Segment SGI354681[] = {{&S91531[0], 18}, {NULL}};
const Segment SGI354682[] = {{&S91535[5], 5}, {&S91535[104], 5}, {NULL}};
const Segment SGI354683[] = {{&S91535[11], 3}, {NULL}};
const Segment SGI354684[] = {{&S91535[130], 3}, {NULL}};
const Segment SGI354685[] = {{&S91537[21], 3}, {NULL}};
const Segment SGI354686[] = {{&S91537[0], 14}, {NULL}};
const Segment SGI354687[] = {{&S91546[35], 3}, {NULL}};
const Segment SGI354688[] = {{&S91546[89], 3}, {NULL}};
const Segment SGI354689[] = {{&S91570[42], 12}, {NULL}};
const Segment SGI354690[] = {{&S91570[6], 3}, {NULL}};
const Segment SGI354691[] = {{&S91570[75], 3}, {NULL}};
const Segment SGI354692[] = {{&S68336[61], 4}, {NULL}};
const Segment SGI354693[] = {{&S68336[13], 10}, {NULL}};
const Segment SGI354694[] = {{&S68337[61], 4}, {NULL}};
const Segment SGI354695[] = {{&S68337[13], 10}, {NULL}};
const Segment SGI354696[] = {{&S68338[61], 4}, {NULL}};
const Segment SGI354697[] = {{&S68338[13], 10}, {NULL}};
const Segment SGI354698[] = {{&S68339[42], 4}, {NULL}};
const Segment SGI354699[] = {{&S68339[13], 14}, {NULL}};
const Segment SGI354700[] = {{&S68340[61], 4}, {NULL}};
const Segment SGI354701[] = {{&S68340[13], 10}, {NULL}};
const Segment SGI354702[] = {{&S68341[61], 4}, {NULL}};
const Segment SGI354703[] = {{&S68341[13], 10}, {NULL}};
const Segment SGI354704[] = {{&S65415[6], 4}, {NULL}};
const Segment SGI354705[] = {{&S65415[80], 5}, {NULL}};
const Segment SGI354706[] = {{&S65415[12], 3}, {NULL}};
const Segment SGI354707[] = {{&S65415[40], 10}, {NULL}};
const Segment SGI354708[] = {{&S65416[6], 4}, {NULL}};
const Segment SGI354709[] = {{&S65416[80], 5}, {NULL}};
const Segment SGI354710[] = {{&S65416[12], 3}, {NULL}};
const Segment SGI354711[] = {{&S65416[40], 10}, {NULL}};
const Segment SGI354712[] = {{&S91642[43], 8}, {NULL}};
const Segment SGI354713[] = {{&S91642[6], 3}, {NULL}};
const Segment SGI354714[] = {{&S91642[72], 3}, {NULL}};
const Segment SGI354715[] = {{&S91643[43], 5}, {NULL}};
const Segment SGI354716[] = {{&S91643[6], 3}, {NULL}};
const Segment SGI354717[] = {{&S91643[69], 3}, {NULL}};
const Segment SGI354718[] = {{&S91689[5], 5}, {NULL}};
const Segment SGI354719[] = {{&S91689[21], 3}, {NULL}};
const Segment SGI354720[] = {{&S91702[5], 5}, {&S91702[5], 5}, {NULL}};
const Segment SGI354721[] = {{&S91702[84], 3}, {NULL}};
const Segment SGI354722[] = {{&S91724[23], 4}, {NULL}};
const Segment SGI354723[] = {{&S91724[0], 16}, {NULL}};
const Segment SGI354724[] = {{&S91730[5], 4}, {NULL}};
const Segment SGI354725[] = {{&S91730[235], 3}, {NULL}};
const Segment SGI354726[] = {{&S91769[41], 4}, {NULL}};
const Segment SGI354727[] = {{&S91769[6], 3}, {NULL}};
const Segment SGI354728[] = {{&S91769[66], 3}, {NULL}};
const Segment SGI354729[] = {{&S91777[42], 5}, {NULL}};
const Segment SGI354730[] = {{&S91777[6], 3}, {NULL}};
const Segment SGI354731[] = {{&S91777[68], 3}, {NULL}};
const Segment SGI354732[] = {{&S91785[43], 12}, {NULL}};
const Segment SGI354733[] = {{&S91785[6], 3}, {NULL}};
const Segment SGI354734[] = {{&S91785[76], 3}, {NULL}};
const Segment SGI354735[] = {{&S91786[43], 7}, {NULL}};
const Segment SGI354736[] = {{&S91786[6], 3}, {NULL}};
const Segment SGI354737[] = {{&S91786[71], 3}, {NULL}};
const Segment SGI354738[] = {{&S91853[5], 7}, {NULL}};
const Segment SGI354739[] = {{&S91853[171], 3}, {NULL}};
const Segment SGI354740[] = {{&S68447[53], 4}, {NULL}};
const Segment SGI354741[] = {{&S68447[13], 10}, {NULL}};
const Segment SGI354742[] = {{&S68450[53], 4}, {NULL}};
const Segment SGI354743[] = {{&S68450[13], 10}, {NULL}};
const Segment SGI354744[] = {{&S68451[53], 3}, {NULL}};
const Segment SGI354745[] = {{&S68451[13], 10}, {NULL}};
const Segment SGI354746[] = {{&S68452[53], 4}, {NULL}};
const Segment SGI354747[] = {{&S68452[13], 10}, {NULL}};
const Segment SGI354748[] = {{&S91897[5], 5}, {&S91897[68], 5}, {NULL}};
const Segment SGI354749[] = {{&S91897[84], 3}, {NULL}};
const Segment SGI354750[] = {{&S91902[5], 3}, {NULL}};
const Segment SGI354751[] = {{&S91902[19], 3}, {NULL}};
const Segment SGI354752[] = {{&S65696[6], 4}, {NULL}};
const Segment SGI354753[] = {{&S65696[69], 5}, {NULL}};
const Segment SGI354754[] = {{&S65696[12], 3}, {NULL}};
const Segment SGI354755[] = {{&S91917[35], 7}, {NULL}};
const Segment SGI354756[] = {{&S91917[54], 3}, {NULL}};
const Segment SGI354757[] = {{&S91919[21], 4}, {NULL}};
const Segment SGI354758[] = {{&S91919[0], 14}, {NULL}};
const Segment SGI354759[] = {{&S91923[21], 5}, {NULL}};
const Segment SGI354760[] = {{&S91923[0], 14}, {NULL}};
const Segment SGI354761[] = {{&S91925[5], 10}, {NULL}};
const Segment SGI354762[] = {{&S91925[21], 3}, {NULL}};
const Segment SGI354763[] = {{&S65731[6], 4}, {NULL}};
const Segment SGI354764[] = {{&S65731[52], 5}, {NULL}};
const Segment SGI354765[] = {{&S65731[12], 45}, {NULL}};
const Segment SGI354766[] = {{&S65731[12], 14}, {NULL}};
const Segment SGI354767[] = {{&S65732[6], 4}, {NULL}};
const Segment SGI354768[] = {{&S65732[64], 5}, {NULL}};
const Segment SGI354769[] = {{&S65732[12], 14}, {NULL}};
const Segment SGI354770[] = {{&S65741[6], 4}, {NULL}};
const Segment SGI354771[] = {{&S65741[77], 5}, {NULL}};
const Segment SGI354772[] = {{&S65741[12], 9}, {NULL}};
const Segment SGI354773[] = {{&S65741[29], 13}, {NULL}};
const Segment SGI354774[] = {{&S91951[42], 5}, {NULL}};
const Segment SGI354775[] = {{&S91951[6], 3}, {NULL}};
const Segment SGI354776[] = {{&S91951[68], 3}, {NULL}};
const Segment SGI354777[] = {{&S91952[42], 3}, {NULL}};
const Segment SGI354778[] = {{&S91952[6], 3}, {NULL}};
const Segment SGI354779[] = {{&S91952[66], 3}, {NULL}};
const Segment SGI354780[] = {{&S91961[5], 3}, {&S91961[5], 3}, {NULL}};
const Segment SGI354781[] = {{&S91961[80], 3}, {NULL}};
const Segment SGI354782[] = {{&S65841[6], 4}, {NULL}};
const Segment SGI354783[] = {{&S65841[58], 5}, {NULL}};
const Segment SGI354784[] = {{&S65841[12], 14}, {NULL}};
const Segment SGI354785[] = {{&S92049[43], 5}, {NULL}};
const Segment SGI354786[] = {{&S92049[6], 3}, {NULL}};
const Segment SGI354787[] = {{&S92049[69], 3}, {NULL}};
const Segment SGI354788[] = {{&S92058[5], 7}, {&S92058[5], 7}, {NULL}};
const Segment SGI354789[] = {{&S92058[13], 3}, {NULL}};
const Segment SGI354790[] = {{&S92058[135], 3}, {NULL}};
const Segment SGI354791[] = {{&S92067[5], 4}, {NULL}};
const Segment SGI354792[] = {{&S92095[5], 11}, {&S92095[5], 11}, {NULL}};
const Segment SGI354793[] = {{&S92095[96], 3}, {NULL}};
const Segment SGI354794[] = {{&S92121[35], 3}, {NULL}};
const Segment SGI354795[] = {{&S92121[50], 3}, {NULL}};
const Segment SGI354796[] = {{&S92123[34], 5}, {NULL}};
const Segment SGI354797[] = {{&S92123[50], 3}, {NULL}};
const Segment SGI354798[] = {{&S92140[5], 5}, {&S92140[97], 5}, {NULL}};
const Segment SGI354799[] = {{&S92140[113], 3}, {NULL}};
const Segment SGI354800[] = {{&S92170[5], 10}, {NULL}};
const Segment SGI354801[] = {{&S92170[21], 3}, {NULL}};
const Segment SGI354802[] = {{&S92198[101], 4}, {NULL}};
const Segment SGI354803[] = {{&S92252[21], 3}, {NULL}};
const Segment SGI354804[] = {{&S92252[0], 14}, {NULL}};
const Segment SGI354805[] = {{&S92259[42], 5}, {NULL}};
const Segment SGI354806[] = {{&S92259[6], 3}, {NULL}};
const Segment SGI354807[] = {{&S92259[68], 3}, {NULL}};
const Segment SGI354808[] = {{&S92264[43], 3}, {NULL}};
const Segment SGI354809[] = {{&S92264[6], 3}, {NULL}};
const Segment SGI354810[] = {{&S92264[67], 3}, {NULL}};
const Segment SGI354811[] = {{&S92273[5], 4}, {NULL}};
const Segment SGI354812[] = {{&S92291[5], 5}, {NULL}};
const Segment SGI354813[] = {{&S92291[21], 3}, {NULL}};
const Segment SGI354814[] = {{&S92292[5], 5}, {NULL}};
const Segment SGI354815[] = {{&S92292[21], 3}, {NULL}};
const Segment SGI354816[] = {{&S92293[5], 3}, {NULL}};
const Segment SGI354817[] = {{&S92293[19], 3}, {NULL}};
const Segment SGI354818[] = {{&S92294[5], 5}, {NULL}};
const Segment SGI354819[] = {{&S92294[21], 3}, {NULL}};
const Segment SGI354820[] = {{&S92302[5], 16}, {NULL}};
const Segment SGI354821[] = {{&S92302[32], 3}, {NULL}};
const Segment SGI354822[] = {{&S92308[5], 9}, {NULL}};
const Segment SGI354823[] = {{&S92308[25], 3}, {NULL}};
const Segment SGI354824[] = {{&S92309[5], 13}, {NULL}};
const Segment SGI354825[] = {{&S92309[29], 3}, {NULL}};
const Segment SGI354826[] = {{&S92318[5], 5}, {NULL}};
const Segment SGI354827[] = {{&S92318[21], 3}, {NULL}};
const Segment SGI354828[] = {{&S92319[5], 5}, {NULL}};
const Segment SGI354829[] = {{&S92319[21], 3}, {NULL}};
const Segment SGI354830[] = {{&S92322[35], 5}, {NULL}};
const Segment SGI354831[] = {{&S92322[52], 3}, {NULL}};
const Segment SGI354832[] = {{&S92341[5], 5}, {NULL}};
const Segment SGI354833[] = {{&S92341[17], 3}, {NULL}};
const Segment SGI354834[] = {{&S92342[35], 5}, {NULL}};
const Segment SGI354835[] = {{&S92342[91], 3}, {NULL}};
const Segment SGI354836[] = {{&S92357[35], 5}, {NULL}};
const Segment SGI354837[] = {{&S92357[91], 3}, {NULL}};
const Segment SGI354838[] = {{&S92396[101], 5}, {NULL}};
const Segment SGI354839[] = {{&S92426[43], 3}, {NULL}};
const Segment SGI354840[] = {{&S92426[6], 3}, {NULL}};
const Segment SGI354841[] = {{&S92426[67], 3}, {NULL}};
const Segment SGI354842[] = {{&S92441[5], 5}, {&S92441[105], 5}, {NULL}};
const Segment SGI354843[] = {{&S92441[11], 3}, {NULL}};
const Segment SGI354844[] = {{&S92441[131], 3}, {NULL}};
const Segment SGI354845[] = {{&S92442[42], 5}, {NULL}};
const Segment SGI354846[] = {{&S92442[6], 3}, {NULL}};
const Segment SGI354847[] = {{&S92442[68], 3}, {NULL}};
const Segment SGI354848[] = {{&S92455[43], 5}, {NULL}};
const Segment SGI354849[] = {{&S92455[6], 3}, {NULL}};
const Segment SGI354850[] = {{&S92455[69], 3}, {NULL}};
const Segment SGI354851[] = {{&S92469[5], 3}, {NULL}};
const Segment SGI354852[] = {{&S92469[19], 3}, {NULL}};
const Segment SGI354853[] = {{&S92482[5], 5}, {NULL}};
const Segment SGI354854[] = {{&S92482[16], 3}, {NULL}};
const Segment SGI354855[] = {{&S92483[5], 5}, {NULL}};
const Segment SGI354856[] = {{&S92483[21], 3}, {NULL}};
const Segment SGI354857[] = {{&S92488[5], 5}, {NULL}};
const Segment SGI354858[] = {{&S92488[21], 3}, {NULL}};
const Segment SGI354859[] = {{&S92501[5], 5}, {NULL}};
const Segment SGI354860[] = {{&S92501[21], 3}, {NULL}};
const Segment SGI354861[] = {{&S92506[35], 3}, {NULL}};
const Segment SGI354862[] = {{&S92506[50], 3}, {NULL}};
const Segment SGI354863[] = {{&S92516[35], 7}, {NULL}};
const Segment SGI354864[] = {{&S92516[54], 3}, {NULL}};
const Segment SGI354865[] = {{&S78990[19], 13}, {NULL}};
const Segment SGI354866[] = {{&S78990[46], 3}, {NULL}};
const Segment SGI354867[] = {{&S92517[5], 15}, {&S92517[107], 15}, {NULL}};
const Segment SGI354868[] = {{&S92517[133], 3}, {NULL}};
const Segment SGI354869[] = {{&S92521[35], 5}, {NULL}};
const Segment SGI354870[] = {{&S92521[52], 3}, {NULL}};
const Segment SGI354871[] = {{&S92535[35], 5}, {NULL}};
const Segment SGI354872[] = {{&S92535[91], 3}, {NULL}};
const Segment SGI354873[] = {{&S66136[6], 4}, {NULL}};
const Segment SGI354874[] = {{&S66136[59], 5}, {NULL}};
const Segment SGI354875[] = {{&S66136[12], 14}, {NULL}};
const Segment SGI354876[] = {{&S66151[6], 4}, {NULL}};
const Segment SGI354877[] = {{&S66151[76], 5}, {NULL}};
const Segment SGI354878[] = {{&S66151[12], 9}, {NULL}};
const Segment SGI354879[] = {{&S66151[29], 13}, {NULL}};
const Segment SGI354880[] = {{&S92563[34], 3}, {NULL}};
const Segment SGI354881[] = {{&S92563[48], 3}, {NULL}};
const Segment SGI354882[] = {{&S92565[5], 3}, {&S92565[5], 3}, {NULL}};
const Segment SGI354883[] = {{&S92565[109], 3}, {NULL}};
const Segment SGI354884[] = {{&S92584[43], 4}, {NULL}};
const Segment SGI354885[] = {{&S92584[6], 3}, {NULL}};
const Segment SGI354886[] = {{&S92584[68], 3}, {NULL}};
const Segment SGI354887[] = {{&S92592[5], 3}, {&S92592[5], 3}, {NULL}};
const Segment SGI354888[] = {{&S92592[149], 3}, {NULL}};
const Segment SGI354889[] = {{&S66211[6], 4}, {NULL}};
const Segment SGI354890[] = {{&S66211[70], 5}, {NULL}};
const Segment SGI354891[] = {{&S66211[12], 9}, {NULL}};
const Segment SGI354892[] = {{&S66211[29], 13}, {NULL}};
const Segment SGI354893[] = {{&S92623[43], 3}, {NULL}};
const Segment SGI354894[] = {{&S92623[6], 3}, {NULL}};
const Segment SGI354895[] = {{&S92623[67], 3}, {NULL}};
const Segment SGI354896[] = {{&S92634[5], 4}, {NULL}};
const Segment SGI354897[] = {{&S92634[168], 3}, {NULL}};
const Segment SGI354898[] = {{&S66233[6], 4}, {NULL}};
const Segment SGI354899[] = {{&S66233[12], 3}, {NULL}};
const Segment SGI354900[] = {{&S66234[6], 3}, {NULL}};
const Segment SGI354901[] = {{&S66234[11], 4}, {NULL}};
const Segment SGI354902[] = {{&S66235[6], 4}, {NULL}};
const Segment SGI354903[] = {{&S66235[12], 3}, {NULL}};
const Segment SGI354904[] = {{&S92643[42], 4}, {NULL}};
const Segment SGI354905[] = {{&S92643[6], 3}, {NULL}};
const Segment SGI354906[] = {{&S66236[6], 4}, {NULL}};
const Segment SGI354907[] = {{&S66236[12], 3}, {NULL}};
const Segment SGI354908[] = {{&S92643[67], 3}, {NULL}};
const Segment SGI354909[] = {{&S66237[6], 3}, {NULL}};
const Segment SGI354910[] = {{&S66237[11], 3}, {NULL}};
const Segment SGI354911[] = {{&S66238[6], 4}, {NULL}};
const Segment SGI354912[] = {{&S92645[43], 4}, {NULL}};
const Segment SGI354913[] = {{&S66238[12], 4}, {NULL}};
const Segment SGI354914[] = {{&S92645[6], 3}, {NULL}};
const Segment SGI354915[] = {{&S92645[68], 3}, {NULL}};
const Segment SGI354916[] = {{&S84563[28], 5}, {NULL}};
const Segment SGI354917[] = {{&S84563[104], 5}, {NULL}};
const Segment SGI354918[] = {{&S92659[5], 4}, {NULL}};
const Segment SGI354919[] = {{&S92659[42], 3}, {NULL}};
const Segment SGI354920[] = {{&S92664[67], 5}, {NULL}};
const Segment SGI354921[] = {{&S92671[5], 5}, {NULL}};
const Segment SGI354922[] = {{&S92671[21], 3}, {NULL}};
const Segment SGI354923[] = {{&S92677[5], 5}, {&S92677[68], 5}, {NULL}};
const Segment SGI354924[] = {{&S92677[84], 3}, {NULL}};
const Segment SGI354925[] = {{&S92678[5], 5}, {NULL}};
const Segment SGI354926[] = {{&S92678[21], 3}, {NULL}};
const Segment SGI354927[] = {{&S92696[5], 7}, {NULL}};
const Segment SGI354928[] = {{&S92696[23], 3}, {NULL}};
const Segment SGI354929[] = {{&S92697[5], 3}, {NULL}};
const Segment SGI354930[] = {{&S92697[19], 3}, {NULL}};
const Segment SGI354931[] = {{&S92699[5], 10}, {NULL}};
const Segment SGI354932[] = {{&S92699[21], 3}, {NULL}};
const Segment SGI354933[] = {{&S92700[35], 5}, {NULL}};
const Segment SGI354934[] = {{&S92700[52], 3}, {NULL}};
const Segment SGI354935[] = {{&S92704[5], 3}, {NULL}};
const Segment SGI354936[] = {{&S92704[233], 3}, {NULL}};
const Segment SGI354937[] = {{&S92708[5], 5}, {&S92708[5], 5}, {NULL}};
const Segment SGI354938[] = {{&S92708[113], 3}, {NULL}};
const Segment SGI354939[] = {{&S66310[6], 3}, {NULL}};
const Segment SGI354940[] = {{&S66310[11], 14}, {NULL}};
const Segment SGI354941[] = {{&S84711[28], 5}, {NULL}};
const Segment SGI354942[] = {{&S84711[104], 5}, {NULL}};
const Segment SGI354943[] = {{&S79303[5], 5}, {NULL}};
const Segment SGI354944[] = {{&S79303[33], 3}, {NULL}};
const Segment SGI354945[] = {{&S79303[87], 3}, {NULL}};
const Segment SGI354946[] = {{&S79304[5], 3}, {NULL}};
const Segment SGI354947[] = {{&S79304[32], 3}, {NULL}};
const Segment SGI354948[] = {{&S79304[86], 3}, {NULL}};
const Segment SGI354949[] = {{&S92819[42], 3}, {NULL}};
const Segment SGI354950[] = {{&S92819[6], 3}, {NULL}};
const Segment SGI354951[] = {{&S92819[66], 3}, {NULL}};
const Segment SGI354952[] = {{&S92830[5], 4}, {NULL}};
const Segment SGI354953[] = {{&S92840[5], 5}, {NULL}};
const Segment SGI354954[] = {{&S92840[21], 3}, {NULL}};
const Segment SGI354955[] = {{&S63210[76], 5}, {NULL}};
const Segment SGI354956[] = {{&S63210[12], 13}, {NULL}};
const Segment SGI354957[] = {{&S63211[76], 5}, {NULL}};
const Segment SGI354958[] = {{&S63211[12], 13}, {NULL}};
const Segment SGI354959[] = {{&S92855[5], 4}, {NULL}};
const Segment SGI354960[] = {{&S92855[20], 3}, {NULL}};
const Segment SGI354961[] = {{&S92861[5], 5}, {&S92861[68], 5}, {NULL}};
const Segment SGI354962[] = {{&S92861[84], 3}, {NULL}};
const Segment SGI354963[] = {{&S92869[5], 5}, {&S92869[68], 5}, {NULL}};
const Segment SGI354964[] = {{&S92869[84], 3}, {NULL}};
const Segment SGI354965[] = {{&S92870[5], 3}, {&S92870[5], 3}, {NULL}};
const Segment SGI354966[] = {{&S92870[80], 3}, {NULL}};
const Segment SGI354967[] = {{&S92873[34], 4}, {NULL}};
const Segment SGI354968[] = {{&S92873[49], 3}, {NULL}};
const Segment SGI354969[] = {{&S92883[5], 10}, {NULL}};
const Segment SGI354970[] = {{&S92883[21], 3}, {NULL}};
const Segment SGI354971[] = {{&S92885[25], 5}, {NULL}};
const Segment SGI354972[] = {{&S92885[0], 18}, {NULL}};
const Segment SGI354973[] = {{&S92897[5], 5}, {NULL}};
const Segment SGI354974[] = {{&S92897[21], 3}, {NULL}};
const Segment SGI354975[] = {{&S92900[5], 10}, {NULL}};
const Segment SGI354976[] = {{&S92900[21], 3}, {NULL}};
const Segment SGI354977[] = {{&S92905[5], 5}, {&S92905[104], 5}, {NULL}};
const Segment SGI354978[] = {{&S92905[11], 3}, {NULL}};
const Segment SGI354979[] = {{&S92905[130], 3}, {NULL}};
const Segment SGI354980[] = {{&S84839[28], 3}, {NULL}};
const Segment SGI354981[] = {{&S84839[102], 5}, {NULL}};
const Segment SGI354982[] = {{&S84860[28], 5}, {NULL}};
const Segment SGI354983[] = {{&S84860[104], 5}, {NULL}};
const Segment SGI354984[] = {{&S85153[28], 11}, {NULL}};
const Segment SGI354985[] = {{&S85153[110], 5}, {NULL}};
const Segment SGI354986[] = {{&S85253[28], 4}, {NULL}};
const Segment SGI354987[] = {{&S85253[103], 5}, {NULL}};
const Segment SGI354988[] = {{&S85254[28], 5}, {NULL}};
const Segment SGI354989[] = {{&S85254[104], 5}, {NULL}};
const Segment SGI354990[] = {{&S63903[70], 5}, {NULL}};
const Segment SGI354991[] = {{&S63903[12], 7}, {NULL}};
const Segment SGI354992[] = {{&S66954[6], 4}, {NULL}};
const Segment SGI354993[] = {{&S66954[57], 5}, {NULL}};
const Segment SGI354994[] = {{&S66954[12], 14}, {NULL}};
const Segment SGI354995[] = {{&S66955[6], 4}, {NULL}};
const Segment SGI354996[] = {{&S66955[57], 5}, {NULL}};
const Segment SGI354997[] = {{&S66955[12], 14}, {NULL}};
const Segment SGI354998[] = {{&S93019[42], 5}, {NULL}};
const Segment SGI354999[] = {{&S93019[6], 3}, {NULL}};
const Segment SGI355000[] = {{&S93019[68], 3}, {NULL}};
const Segment SGI355001[] = {{&S93025[42], 5}, {NULL}};
const Segment SGI355002[] = {{&S93025[6], 3}, {NULL}};
const Segment SGI355003[] = {{&S93025[68], 3}, {NULL}};
const Segment SGI355004[] = {{&S67023[6], 4}, {NULL}};
const Segment SGI355005[] = {{&S93026[43], 3}, {NULL}};
const Segment SGI355006[] = {{&S67023[77], 5}, {NULL}};
const Segment SGI355007[] = {{&S67023[12], 9}, {NULL}};
const Segment SGI355008[] = {{&S93026[6], 3}, {NULL}};
const Segment SGI355009[] = {{&S67023[29], 13}, {NULL}};
const Segment SGI355010[] = {{&S93026[67], 3}, {NULL}};
const Segment SGI355011[] = {{&S93033[5], 3}, {&S93033[5], 3}, {NULL}};
const Segment SGI355012[] = {{&S93033[9], 3}, {NULL}};
const Segment SGI355013[] = {{&S93033[127], 3}, {NULL}};
const Segment SGI355014[] = {{&S93050[5], 4}, {NULL}};
const Segment SGI355015[] = {{&S93070[35], 3}, {NULL}};
const Segment SGI355016[] = {{&S93070[50], 3}, {NULL}};
const Segment SGI355017[] = {{&S93074[5], 3}, {&S93074[5], 3}, {NULL}};
const Segment SGI355018[] = {{&S93074[80], 3}, {NULL}};
const Segment SGI355019[] = {{&S93077[5], 3}, {&S93077[5], 3}, {NULL}};
const Segment SGI355020[] = {{&S93077[109], 3}, {NULL}};
const Segment SGI355021[] = {{&S93095[5], 10}, {NULL}};
const Segment SGI355022[] = {{&S93095[21], 3}, {NULL}};
const Segment SGI355023[] = {{&S93097[5], 4}, {NULL}};
const Segment SGI355024[] = {{&S93097[234], 3}, {NULL}};
const Segment SGI355025[] = {{&S93125[21], 5}, {NULL}};
const Segment SGI355026[] = {{&S93125[0], 14}, {NULL}};
const Segment SGI355027[] = {{&S93167[5], 15}, {&S93167[115], 15}, {NULL}};
const Segment SGI355028[] = {{&S93167[21], 3}, {NULL}};
const Segment SGI355029[] = {{&S93167[151], 3}, {NULL}};
const Segment SGI355030[] = {{&S93192[5], 3}, {&S93192[5], 3}, {NULL}};
const Segment SGI355031[] = {{&S93192[9], 3}, {NULL}};
const Segment SGI355032[] = {{&S93192[127], 3}, {NULL}};
const Segment SGI355033[] = {{&S93198[42], 5}, {NULL}};
const Segment SGI355034[] = {{&S93198[6], 3}, {NULL}};
const Segment SGI355035[] = {{&S93198[68], 3}, {NULL}};
const Segment SGI355036[] = {{&S93199[43], 3}, {NULL}};
const Segment SGI355037[] = {{&S93199[6], 3}, {NULL}};
const Segment SGI355038[] = {{&S93199[67], 3}, {NULL}};
const Segment SGI355039[] = {{&S93211[5], 3}, {&S93211[5], 3}, {NULL}};
const Segment SGI355040[] = {{&S93211[9], 3}, {NULL}};
const Segment SGI355041[] = {{&S93211[127], 3}, {NULL}};
const Segment SGI355042[] = {{&S93214[43], 5}, {NULL}};
const Segment SGI355043[] = {{&S93214[6], 3}, {NULL}};
const Segment SGI355044[] = {{&S93214[69], 3}, {NULL}};
const Segment SGI355045[] = {{&S93227[100], 11}, {NULL}};
const Segment SGI355046[] = {{&S93243[5], 3}, {NULL}};
const Segment SGI355047[] = {{&S93243[167], 3}, {NULL}};
const Segment SGI355048[] = {{&S93283[5], 5}, {NULL}};
const Segment SGI355049[] = {{&S93283[21], 3}, {NULL}};
const Segment SGI355050[] = {{&S93284[5], 5}, {NULL}};
const Segment SGI355051[] = {{&S93284[21], 3}, {NULL}};
const Segment SGI355052[] = {{&S93311[5], 7}, {&S93311[5], 7}, {NULL}};
const Segment SGI355053[] = {{&S93311[88], 3}, {NULL}};
const Segment SGI355054[] = {{&S93319[35], 4}, {NULL}};
const Segment SGI355055[] = {{&S93319[51], 3}, {NULL}};
const Segment SGI355056[] = {{&S93349[5], 5}, {NULL}};
const Segment SGI355057[] = {{&S93349[235], 3}, {NULL}};
const Segment SGI355058[] = {{&S93352[5], 5}, {&S93352[5], 5}, {NULL}};
const Segment SGI355059[] = {{&S93352[11], 3}, {NULL}};
const Segment SGI355060[] = {{&S93352[131], 3}, {NULL}};
const Segment SGI355061[] = {{&S93428[43], 7}, {NULL}};
const Segment SGI355062[] = {{&S93428[6], 3}, {NULL}};
const Segment SGI355063[] = {{&S93428[71], 3}, {NULL}};
const Segment SGI355064[] = {{&S67111[6], 4}, {NULL}};
const Segment SGI355065[] = {{&S67111[58], 5}, {NULL}};
const Segment SGI355066[] = {{&S67111[12], 14}, {NULL}};
const Segment SGI355067[] = {{&S93442[5], 7}, {&S93442[5], 7}, {NULL}};
const Segment SGI355068[] = {{&S93442[13], 3}, {NULL}};
const Segment SGI355069[] = {{&S93442[135], 3}, {NULL}};
const Segment SGI355070[] = {{&S68124[0], 16}, {NULL}};
const Segment SGI355071[] = {{&S68124[29], 3}, {NULL}};
const Segment SGI355072[] = {{&S68124[73], 9}, {NULL}};
const Segment SGI355073[] = {{&S93532[5], 5}, {NULL}};
const Segment SGI355074[] = {{&S93532[21], 3}, {NULL}};
const Segment SGI355075[] = {{&S93534[5], 3}, {NULL}};
const Segment SGI355076[] = {{&S93534[19], 3}, {NULL}};
const Segment SGI355077[] = {{&S93535[5], 7}, {NULL}};
const Segment SGI355078[] = {{&S93535[23], 3}, {NULL}};
const Segment SGI355079[] = {{&S93537[5], 7}, {NULL}};
const Segment SGI355080[] = {{&S93537[23], 3}, {NULL}};
const Segment SGI355081[] = {{&S93538[5], 3}, {NULL}};
const Segment SGI355082[] = {{&S93538[19], 3}, {NULL}};
const Segment SGI355083[] = {{&S68229[0], 21}, {NULL}};
const Segment SGI355084[] = {{&S68229[34], 3}, {NULL}};
const Segment SGI355085[] = {{&S68229[67], 13}, {NULL}};
const Segment SGI355086[] = {{&S67221[6], 4}, {NULL}};
const Segment SGI355087[] = {{&S67221[12], 3}, {NULL}};
const Segment SGI355088[] = {{&S68230[0], 21}, {NULL}};
const Segment SGI355089[] = {{&S67221[32], 10}, {NULL}};
const Segment SGI355090[] = {{&S68230[34], 3}, {NULL}};
const Segment SGI355091[] = {{&S68230[67], 13}, {NULL}};
const Segment SGI355092[] = {{&S93559[35], 4}, {NULL}};
const Segment SGI355093[] = {{&S93559[51], 3}, {NULL}};
const Segment SGI355094[] = {{&S67222[6], 4}, {NULL}};
const Segment SGI355095[] = {{&S67222[12], 3}, {NULL}};
const Segment SGI355096[] = {{&S67222[32], 10}, {NULL}};
const Segment SGI355097[] = {{&S67223[6], 4}, {NULL}};
const Segment SGI355098[] = {{&S67223[12], 3}, {NULL}};
const Segment SGI355099[] = {{&S67223[32], 10}, {NULL}};
const Segment SGI355100[] = {{&S93572[5], 5}, {NULL}};
const Segment SGI355101[] = {{&S93573[35], 4}, {NULL}};
const Segment SGI355102[] = {{&S93573[51], 3}, {NULL}};
const Segment SGI355103[] = {{&S93574[35], 3}, {NULL}};
const Segment SGI355104[] = {{&S93574[50], 3}, {NULL}};
const Segment SGI355105[] = {{&S93577[35], 4}, {NULL}};
const Segment SGI355106[] = {{&S93577[51], 3}, {NULL}};
const Segment SGI355107[] = {{&S93585[25], 3}, {NULL}};
const Segment SGI355108[] = {{&S93585[0], 18}, {NULL}};
const Segment SGI355109[] = {{&S93615[21], 5}, {NULL}};
const Segment SGI355110[] = {{&S93615[0], 14}, {NULL}};
const Segment SGI355111[] = {{&S93653[5], 7}, {&S93653[5], 7}, {NULL}};
const Segment SGI355112[] = {{&S93653[13], 3}, {NULL}};
const Segment SGI355113[] = {{&S93653[134], 3}, {NULL}};
const Segment SGI355114[] = {{&S93727[5], 5}, {NULL}};
const Segment SGI355115[] = {{&S93742[5], 3}, {NULL}};
const Segment SGI355116[] = {{&S93742[19], 3}, {NULL}};
const Segment SGI355117[] = {{&S93750[5], 3}, {NULL}};
const Segment SGI355118[] = {{&S93750[14], 3}, {NULL}};
const Segment SGI355119[] = {{&S93751[5], 5}, {&S93751[68], 5}, {NULL}};
const Segment SGI355120[] = {{&S93751[84], 3}, {NULL}};
const Segment SGI355121[] = {{&S93753[5], 3}, {NULL}};
const Segment SGI355122[] = {{&S93753[19], 3}, {NULL}};
const Segment SGI355123[] = {{&S93754[5], 5}, {NULL}};
const Segment SGI355124[] = {{&S93754[21], 3}, {NULL}};
const Segment SGI355125[] = {{&S93755[5], 5}, {NULL}};
const Segment SGI355126[] = {{&S93755[21], 3}, {NULL}};
const Segment SGI355127[] = {{&S93769[5], 3}, {NULL}};
const Segment SGI355128[] = {{&S93769[19], 3}, {NULL}};
const Segment SGI355129[] = {{&S93800[5], 4}, {&S93800[5], 4}, {NULL}};
const Segment SGI355130[] = {{&S93800[82], 3}, {NULL}};
const Segment SGI355131[] = {{&S93802[35], 5}, {NULL}};
const Segment SGI355132[] = {{&S93802[52], 3}, {NULL}};
const Segment SGI355133[] = {{&S93811[5], 5}, {&S93811[68], 5}, {NULL}};
const Segment SGI355134[] = {{&S93811[84], 3}, {NULL}};
const Segment SGI355135[] = {{&S93825[5], 5}, {NULL}};
const Segment SGI355136[] = {{&S93825[236], 3}, {NULL}};
const Segment SGI355137[] = {{&S93860[5], 5}, {&S93860[5], 5}, {NULL}};
const Segment SGI355138[] = {{&S93860[113], 3}, {NULL}};
const Segment SGI355139[] = {{&S80934[5], 5}, {NULL}};
const Segment SGI355140[] = {{&S80934[46], 3}, {NULL}};
const Segment SGI355141[] = {{&S80993[5], 5}, {NULL}};
const Segment SGI355142[] = {{&S80993[33], 3}, {NULL}};
const Segment SGI355143[] = {{&S80993[82], 3}, {NULL}};
const Segment SGI355144[] = {{&S80994[5], 3}, {NULL}};
const Segment SGI355145[] = {{&S80994[32], 3}, {NULL}};
const Segment SGI355146[] = {{&S80994[81], 3}, {NULL}};
const Segment SGI355147[] = {{&S67483[6], 4}, {NULL}};
const Segment SGI355148[] = {{&S67483[12], 3}, {NULL}};
const Segment SGI355149[] = {{&S67484[6], 4}, {NULL}};
const Segment SGI355150[] = {{&S67484[12], 3}, {NULL}};
const Segment SGI355151[] = {{&S67506[6], 4}, {NULL}};
const Segment SGI355152[] = {{&S67506[12], 10}, {NULL}};
const Segment SGI355153[] = {{&S67507[6], 4}, {NULL}};
const Segment SGI355154[] = {{&S67507[12], 10}, {NULL}};
const Segment SGI355155[] = {{&S86054[28], 7}, {NULL}};
const Segment SGI355156[] = {{&S86054[106], 5}, {NULL}};
const Segment SGI355157[] = {{&S86058[28], 7}, {NULL}};
const Segment SGI355158[] = {{&S86058[106], 5}, {NULL}};
const Segment SGI355159[] = {{&S93932[43], 5}, {NULL}};
const Segment SGI355160[] = {{&S93932[6], 3}, {NULL}};
const Segment SGI355161[] = {{&S93932[69], 3}, {NULL}};
const Segment SGI355162[] = {{&S93933[42], 3}, {NULL}};
const Segment SGI355163[] = {{&S93933[6], 3}, {NULL}};
const Segment SGI355164[] = {{&S93933[66], 3}, {NULL}};
const Segment SGI355165[] = {{&S67611[6], 4}, {NULL}};
const Segment SGI355166[] = {{&S67611[97], 5}, {NULL}};
const Segment SGI355167[] = {{&S67611[12], 9}, {NULL}};
const Segment SGI355168[] = {{&S67611[29], 13}, {NULL}};
const Segment SGI355169[] = {{&S93948[43], 5}, {NULL}};
const Segment SGI355170[] = {{&S93948[6], 3}, {NULL}};
const Segment SGI355171[] = {{&S93948[69], 3}, {NULL}};
const Segment SGI355172[] = {{&S93956[5], 7}, {&S93956[5], 7}, {NULL}};
const Segment SGI355173[] = {{&S93956[13], 3}, {NULL}};
const Segment SGI355174[] = {{&S93956[135], 3}, {NULL}};
const Segment SGI355175[] = {{&S93976[5], 5}, {&S93976[68], 5}, {NULL}};
const Segment SGI355176[] = {{&S93976[84], 3}, {NULL}};
const Segment SGI355177[] = {{&S93983[69], 11}, {NULL}};
const Segment SGI355178[] = {{&S64957[72], 5}, {NULL}};
const Segment SGI355179[] = {{&S64957[12], 3}, {NULL}};
const Segment SGI355180[] = {{&S94000[25], 7}, {&S94000[81], 7}, {NULL}};
const Segment SGI355181[] = {{&S86224[28], 5}, {NULL}};
const Segment SGI355182[] = {{&S86224[111], 5}, {NULL}};
const Segment SGI355183[] = {{&S94043[5], 3}, {&S94043[5], 3}, {NULL}};
const Segment SGI355184[] = {{&S94043[109], 3}, {NULL}};
const Segment SGI355185[] = {{&S86239[28], 12}, {NULL}};
const Segment SGI355186[] = {{&S86239[111], 5}, {NULL}};
const Segment SGI355187[] = {{&S86249[28], 11}, {NULL}};
const Segment SGI355188[] = {{&S86249[110], 5}, {NULL}};
const Segment SGI355189[] = {{&S94084[48], 20}, {NULL}};
const Segment SGI355190[] = {{&S94086[45], 11}, {NULL}};
const Segment SGI355191[] = {{&S94086[6], 3}, {NULL}};
const Segment SGI355192[] = {{&S94086[77], 3}, {NULL}};
const Segment SGI355193[] = {{&S94149[43], 7}, {NULL}};
const Segment SGI355194[] = {{&S94149[6], 3}, {NULL}};
const Segment SGI355195[] = {{&S94149[71], 3}, {NULL}};
const Segment SGI355196[] = {{&S94155[43], 4}, {NULL}};
const Segment SGI355197[] = {{&S94155[6], 3}, {NULL}};
const Segment SGI355198[] = {{&S94155[68], 3}, {NULL}};
const Segment SGI355199[] = {{&S94164[5], 5}, {NULL}};
const Segment SGI355200[] = {{&S94164[169], 3}, {NULL}};
const Segment SGI355201[] = {{&S94197[5], 4}, {NULL}};
const Segment SGI355202[] = {{&S94215[5], 5}, {NULL}};
const Segment SGI355203[] = {{&S94215[21], 3}, {NULL}};
const Segment SGI355204[] = {{&S94233[35], 5}, {NULL}};
const Segment SGI355205[] = {{&S94233[52], 3}, {NULL}};
const Segment SGI355206[] = {{&S67934[6], 4}, {NULL}};
const Segment SGI355207[] = {{&S67934[57], 5}, {NULL}};
const Segment SGI355208[] = {{&S67934[12], 14}, {NULL}};
const Segment SGI355209[] = {{&S67935[6], 4}, {NULL}};
const Segment SGI355210[] = {{&S67935[12], 14}, {NULL}};
const Segment SGI355211[] = {{&S67943[6], 4}, {NULL}};
const Segment SGI355212[] = {{&S67943[77], 5}, {NULL}};
const Segment SGI355213[] = {{&S67943[12], 9}, {NULL}};
const Segment SGI355214[] = {{&S67943[29], 13}, {NULL}};
const Segment SGI355215[] = {{&S67944[6], 4}, {NULL}};
const Segment SGI355216[] = {{&S67944[77], 5}, {NULL}};
const Segment SGI355217[] = {{&S67944[12], 9}, {NULL}};
const Segment SGI355218[] = {{&S67944[29], 13}, {NULL}};
const Segment SGI355219[] = {{&S70774[13], 10}, {NULL}};
const Segment SGI355220[] = {{&S81517[49], 5}, {NULL}};
const Segment SGI355221[] = {{&S81517[40], 3}, {NULL}};
const Segment SGI355222[] = {{&S94380[43], 4}, {NULL}};
const Segment SGI355223[] = {{&S94380[6], 3}, {NULL}};
const Segment SGI355224[] = {{&S94380[68], 3}, {NULL}};
const Segment SGI355225[] = {{&S94395[5], 4}, {NULL}};
const Segment SGI355226[] = {{&S94411[5], 7}, {NULL}};
const Segment SGI355227[] = {{&S94413[5], 5}, {NULL}};
const Segment SGI355228[] = {{&S94413[21], 3}, {NULL}};
const Segment SGI355229[] = {{&S94414[5], 5}, {NULL}};
const Segment SGI355230[] = {{&S94414[21], 3}, {NULL}};
const Segment SGI355231[] = {{&S94456[5], 4}, {NULL}};
const Segment SGI355232[] = {{&S94456[20], 3}, {NULL}};
const Segment SGI355233[] = {{&S94457[5], 7}, {NULL}};
const Segment SGI355234[] = {{&S94457[237], 3}, {NULL}};
const Segment SGI355235[] = {{&S94462[5], 4}, {NULL}};
const Segment SGI355236[] = {{&S94462[20], 3}, {NULL}};
const Segment SGI355237[] = {{&S94485[21], 3}, {NULL}};
const Segment SGI355238[] = {{&S94485[0], 14}, {NULL}};
const Segment SGI355239[] = {{&S94508[42], 3}, {NULL}};
const Segment SGI355240[] = {{&S94508[6], 3}, {NULL}};
const Segment SGI355241[] = {{&S94508[66], 3}, {NULL}};
const Segment SGI355242[] = {{&S94510[42], 4}, {NULL}};
const Segment SGI355243[] = {{&S94510[6], 3}, {NULL}};
const Segment SGI355244[] = {{&S94510[67], 3}, {NULL}};
const Segment SGI355245[] = {{&S94555[5], 3}, {&S94555[5], 3}, {NULL}};
const Segment SGI355246[] = {{&S94555[67], 3}, {NULL}};
const Segment SGI355247[] = {{&S94555[127], 3}, {NULL}};
const Segment SGI355248[] = {{&S94580[5], 3}, {NULL}};
const Segment SGI355249[] = {{&S94580[167], 3}, {NULL}};
const Segment SGI355250[] = {{&S94586[43], 4}, {NULL}};
const Segment SGI355251[] = {{&S94586[6], 3}, {NULL}};
const Segment SGI355252[] = {{&S94586[68], 3}, {NULL}};
const Segment SGI355253[] = {{&S94590[42], 5}, {NULL}};
const Segment SGI355254[] = {{&S94590[6], 3}, {NULL}};
const Segment SGI355255[] = {{&S94590[68], 3}, {NULL}};
const Segment SGI355256[] = {{&S94591[42], 3}, {NULL}};
const Segment SGI355257[] = {{&S94591[6], 3}, {NULL}};
const Segment SGI355258[] = {{&S94591[66], 3}, {NULL}};
const Segment SGI355259[] = {{&S71116[41], 4}, {NULL}};
const Segment SGI355260[] = {{&S71116[13], 10}, {NULL}};
const Segment SGI355261[] = {{&S94650[5], 3}, {NULL}};
const Segment SGI355262[] = {{&S94650[19], 3}, {NULL}};
const Segment SGI355263[] = {{&S94654[5], 3}, {NULL}};
const Segment SGI355264[] = {{&S94654[19], 3}, {NULL}};
const Segment SGI355265[] = {{&S94661[5], 11}, {&S94661[5], 11}, {NULL}};
const Segment SGI355266[] = {{&S94661[96], 3}, {NULL}};
const Segment SGI355267[] = {{&S94666[52], 4}, {NULL}};
const Segment SGI355268[] = {{&S94666[81], 4}, {NULL}};
const Segment SGI355269[] = {{&S94702[34], 4}, {NULL}};
const Segment SGI355270[] = {{&S94702[49], 3}, {NULL}};
const Segment SGI355271[] = {{&S94708[35], 3}, {NULL}};
const Segment SGI355272[] = {{&S94708[50], 3}, {NULL}};
const Segment SGI355273[] = {{&S81749[19], 25}, {NULL}};
const Segment SGI355274[] = {{&S81749[58], 3}, {NULL}};
const Segment SGI355275[] = {{&S94709[35], 3}, {NULL}};
const Segment SGI355276[] = {{&S94709[89], 3}, {NULL}};
const Segment SGI355277[] = {{&S94731[21], 3}, {NULL}};
const Segment SGI355278[] = {{&S94731[0], 14}, {NULL}};
const Segment SGI355279[] = {{&S69370[0], 21}, {NULL}};
const Segment SGI355280[] = {{&S69370[34], 3}, {NULL}};
const Segment SGI355281[] = {{&S69370[62], 3}, {NULL}};
const Segment SGI355282[] = {{&S69371[0], 26}, {NULL}};
const Segment SGI355283[] = {{&S69371[65], 3}, {NULL}};
const Segment SGI355284[] = {{&S69371[23], 3}, {NULL}};
const Segment SGI355285[] = {{&S69371[121], 13}, {NULL}};
const Segment SGI355286[] = {{&S69372[0], 21}, {NULL}};
const Segment SGI355287[] = {{&S69372[34], 3}, {NULL}};
const Segment SGI355288[] = {{&S69372[61], 3}, {NULL}};
const Segment SGI355289[] = {{&S69372[102], 13}, {NULL}};
const Segment SGI355290[] = {{&S94751[5], 7}, {&S94751[5], 7}, {NULL}};
const Segment SGI355291[] = {{&S94751[71], 3}, {NULL}};
const Segment SGI355292[] = {{&S94751[134], 3}, {NULL}};
const Segment SGI355293[] = {{&S94778[35], 7}, {NULL}};
const Segment SGI355294[] = {{&S94778[93], 3}, {NULL}};
const Segment SGI355295[] = {{&S94786[43], 4}, {NULL}};
const Segment SGI355296[] = {{&S94786[6], 3}, {NULL}};
const Segment SGI355297[] = {{&S94786[68], 3}, {NULL}};
const Segment SGI355298[] = {{&S71272[57], 3}, {NULL}};
const Segment SGI355299[] = {{&S71272[13], 10}, {NULL}};
const Segment SGI355300[] = {{&S71273[57], 4}, {NULL}};
const Segment SGI355301[] = {{&S71273[13], 10}, {NULL}};
const Segment SGI355302[] = {{&S71274[57], 4}, {NULL}};
const Segment SGI355303[] = {{&S71274[13], 10}, {NULL}};
const Segment SGI355304[] = {{&S94797[101], 5}, {NULL}};
const Segment SGI355305[] = {{&S68448[6], 4}, {NULL}};
const Segment SGI355306[] = {{&S68448[12], 10}, {NULL}};
const Segment SGI355307[] = {{&S68449[6], 4}, {NULL}};
const Segment SGI355308[] = {{&S68449[12], 10}, {NULL}};
const Segment SGI355309[] = {{&S68453[6], 4}, {NULL}};
const Segment SGI355310[] = {{&S68453[12], 10}, {NULL}};
const Segment SGI355311[] = {{&S68454[6], 4}, {NULL}};
const Segment SGI355312[] = {{&S68454[12], 10}, {NULL}};
const Segment SGI355313[] = {{&S94832[43], 4}, {NULL}};
const Segment SGI355314[] = {{&S94832[6], 3}, {NULL}};
const Segment SGI355315[] = {{&S94832[68], 3}, {NULL}};
const Segment SGI355316[] = {{&S86844[28], 3}, {NULL}};
const Segment SGI355317[] = {{&S86844[102], 5}, {NULL}};
const Segment SGI355318[] = {{&S86873[28], 7}, {NULL}};
const Segment SGI355319[] = {{&S86873[106], 5}, {NULL}};
const Segment SGI355320[] = {{&S94881[5], 7}, {&S94881[5], 7}, {NULL}};
const Segment SGI355321[] = {{&S94881[88], 3}, {NULL}};
const Segment SGI355322[] = {{&S94889[5], 4}, {NULL}};
const Segment SGI355323[] = {{&S94891[52], 5}, {NULL}};
const Segment SGI355324[] = {{&S94891[79], 4}, {NULL}};
const Segment SGI355325[] = {{&S81945[49], 4}, {NULL}};
const Segment SGI355326[] = {{&S81945[40], 3}, {NULL}};
const Segment SGI355327[] = {{&S81946[54], 4}, {NULL}};
const Segment SGI355328[] = {{&S81946[45], 3}, {NULL}};
const Segment SGI355329[] = {{&S94988[44], 21}, {NULL}};
const Segment SGI355330[] = {{&S94988[6], 3}, {NULL}};
const Segment SGI355331[] = {{&S94988[86], 3}, {NULL}};
const Segment SGI355332[] = {{&S82018[5], 6}, {NULL}};
const Segment SGI355333[] = {{&S82018[39], 3}, {NULL}};
const Segment SGI355334[] = {{&S82019[5], 8}, {NULL}};
const Segment SGI355335[] = {{&S82019[41], 3}, {NULL}};
const Segment SGI355336[] = {{&S87016[28], 3}, {NULL}};
const Segment SGI355337[] = {{&S87016[102], 5}, {NULL}};
const Segment SGI355338[] = {{&S82044[19], 15}, {NULL}};
const Segment SGI355339[] = {{&S82044[48], 3}, {NULL}};
const Segment SGI355340[] = {{&S69706[0], 21}, {NULL}};
const Segment SGI355341[] = {{&S69706[6], 3}, {NULL}};
const Segment SGI355342[] = {{&S69706[93], 13}, {NULL}};
const Segment SGI355343[] = {{&S87129[28], 4}, {NULL}};
const Segment SGI355344[] = {{&S87129[103], 5}, {NULL}};
const Segment SGI355345[] = {{&S87135[28], 11}, {NULL}};
const Segment SGI355346[] = {{&S87135[110], 5}, {NULL}};
const Segment SGI355347[] = {{&S68616[6], 4}, {NULL}};
const Segment SGI355348[] = {{&S68616[77], 5}, {NULL}};
const Segment SGI355349[] = {{&S68616[12], 9}, {NULL}};
const Segment SGI355350[] = {{&S68616[29], 13}, {NULL}};
const Segment SGI355351[] = {{&S95118[5], 4}, {&S95118[5], 4}, {NULL}};
const Segment SGI355352[] = {{&S95118[10], 3}, {NULL}};
const Segment SGI355353[] = {{&S95118[129], 3}, {NULL}};
const Segment SGI355354[] = {{&S71607[57], 4}, {NULL}};
const Segment SGI355355[] = {{&S71607[13], 10}, {NULL}};
const Segment SGI355356[] = {{&S71608[39], 4}, {NULL}};
const Segment SGI355357[] = {{&S71608[13], 3}, {NULL}};
const Segment SGI355358[] = {{&S95174[5], 5}, {NULL}};
const Segment SGI355359[] = {{&S95174[21], 3}, {NULL}};
const Segment SGI355360[] = {{&S95192[35], 3}, {NULL}};
const Segment SGI355361[] = {{&S95192[50], 3}, {NULL}};
const Segment SGI355362[] = {{&S95201[5], 5}, {NULL}};
const Segment SGI355363[] = {{&S95201[21], 3}, {NULL}};
const Segment SGI355364[] = {{&S95203[25], 5}, {NULL}};
const Segment SGI355365[] = {{&S95203[0], 18}, {NULL}};
const Segment SGI355366[] = {{&S82256[5], 5}, {NULL}};
const Segment SGI355367[] = {{&S82256[11], 3}, {NULL}};
const Segment SGI355368[] = {{&S87256[28], 5}, {NULL}};
const Segment SGI355369[] = {{&S87256[104], 5}, {NULL}};
const Segment SGI355370[] = {{&S95245[42], 3}, {NULL}};
const Segment SGI355371[] = {{&S95245[6], 3}, {NULL}};
const Segment SGI355372[] = {{&S95245[66], 3}, {NULL}};
const Segment SGI355373[] = {{&S95271[21], 5}, {NULL}};
const Segment SGI355374[] = {{&S95271[0], 14}, {NULL}};
const Segment SGI355375[] = {{&S95305[42], 3}, {NULL}};
const Segment SGI355376[] = {{&S95305[6], 3}, {NULL}};
const Segment SGI355377[] = {{&S95305[66], 3}, {NULL}};
const Segment SGI355378[] = {{&S68869[6], 4}, {NULL}};
const Segment SGI355379[] = {{&S68869[52], 5}, {NULL}};
const Segment SGI355380[] = {{&S68869[12], 45}, {NULL}};
const Segment SGI355381[] = {{&S68869[12], 14}, {NULL}};
const Segment SGI355382[] = {{&S68870[6], 4}, {NULL}};
const Segment SGI355383[] = {{&S68870[64], 5}, {NULL}};
const Segment SGI355384[] = {{&S68870[12], 14}, {NULL}};
const Segment SGI355385[] = {{&S95353[43], 5}, {NULL}};
const Segment SGI355386[] = {{&S95353[6], 3}, {NULL}};
const Segment SGI355387[] = {{&S95353[69], 3}, {NULL}};
const Segment SGI355388[] = {{&S95379[5], 5}, {NULL}};
const Segment SGI355389[] = {{&S95379[21], 3}, {NULL}};
const Segment SGI355390[] = {{&S95407[5], 4}, {NULL}};
const Segment SGI355391[] = {{&S95407[20], 3}, {NULL}};
const Segment SGI355392[] = {{&S95422[5], 3}, {NULL}};
const Segment SGI355393[] = {{&S95422[19], 3}, {NULL}};
const Segment SGI355394[] = {{&S95423[5], 7}, {NULL}};
const Segment SGI355395[] = {{&S95423[23], 3}, {NULL}};
const Segment SGI355396[] = {{&S87477[28], 3}, {NULL}};
const Segment SGI355397[] = {{&S87477[102], 5}, {NULL}};
const Segment SGI355398[] = {{&S95444[5], 3}, {&S95444[5], 3}, {NULL}};
const Segment SGI355399[] = {{&S95444[109], 3}, {NULL}};
const Segment SGI355400[] = {{&S95457[5], 5}, {&S95457[5], 5}, {NULL}};
const Segment SGI355401[] = {{&S95457[11], 3}, {NULL}};
const Segment SGI355402[] = {{&S95457[130], 3}, {NULL}};
const Segment SGI355403[] = {{&S95519[42], 4}, {NULL}};
const Segment SGI355404[] = {{&S95519[6], 3}, {NULL}};
const Segment SGI355405[] = {{&S95519[67], 3}, {NULL}};
const Segment SGI355406[] = {{&S95565[101], 3}, {NULL}};
const Segment SGI355407[] = {{&S69982[0], 16}, {NULL}};
const Segment SGI355408[] = {{&S69982[29], 3}, {NULL}};
const Segment SGI355409[] = {{&S69982[71], 10}, {NULL}};
const Segment SGI355410[] = {{&S69132[6], 4}, {NULL}};
const Segment SGI355411[] = {{&S69132[57], 5}, {NULL}};
const Segment SGI355412[] = {{&S69132[12], 14}, {NULL}};
const Segment SGI355413[] = {{&S69133[6], 4}, {NULL}};
const Segment SGI355414[] = {{&S69133[57], 5}, {NULL}};
const Segment SGI355415[] = {{&S69133[12], 14}, {NULL}};
const Segment SGI355416[] = {{&S95599[5], 5}, {NULL}};
const Segment SGI355417[] = {{&S95599[169], 3}, {NULL}};
const Segment SGI355418[] = {{&S95661[5], 5}, {&S95661[5], 5}, {NULL}};
const Segment SGI355419[] = {{&S95661[11], 3}, {NULL}};
const Segment SGI355420[] = {{&S95661[131], 3}, {NULL}};
const Segment SGI355421[] = {{&S95662[5], 5}, {&S95662[5], 5}, {NULL}};
const Segment SGI355422[] = {{&S95662[69], 3}, {NULL}};
const Segment SGI355423[] = {{&S95662[131], 3}, {NULL}};
const Segment SGI355424[] = {{&S69246[6], 5}, {NULL}};
const Segment SGI355425[] = {{&S69246[13], 14}, {NULL}};
const Segment SGI355426[] = {{&S69253[6], 4}, {NULL}};
const Segment SGI355427[] = {{&S69253[84], 5}, {NULL}};
const Segment SGI355428[] = {{&S69253[12], 13}, {NULL}};
const Segment SGI355429[] = {{&S69254[6], 4}, {NULL}};
const Segment SGI355430[] = {{&S69254[87], 5}, {NULL}};
const Segment SGI355431[] = {{&S69254[12], 13}, {NULL}};
const Segment SGI355432[] = {{&S87813[28], 5}, {NULL}};
const Segment SGI355433[] = {{&S87813[104], 5}, {NULL}};
const Segment SGI355434[] = {{&S95672[5], 5}, {NULL}};
const Segment SGI355435[] = {{&S95695[5], 5}, {NULL}};
const Segment SGI355436[] = {{&S95695[21], 3}, {NULL}};
const Segment SGI355437[] = {{&S95722[5], 5}, {&S95722[95], 5}, {NULL}};
const Segment SGI355438[] = {{&S95722[111], 3}, {NULL}};
const Segment SGI355439[] = {{&S95723[35], 9}, {NULL}};
const Segment SGI355440[] = {{&S95723[56], 3}, {NULL}};
const Segment SGI355441[] = {{&S95724[5], 5}, {NULL}};
const Segment SGI355442[] = {{&S95724[16], 3}, {NULL}};
const Segment SGI355443[] = {{&S95725[34], 5}, {NULL}};
const Segment SGI355444[] = {{&S95725[50], 3}, {NULL}};
const Segment SGI355445[] = {{&S95743[23], 7}, {NULL}};
const Segment SGI355446[] = {{&S95743[0], 16}, {NULL}};
const Segment SGI355447[] = {{&S95745[5], 3}, {NULL}};
const Segment SGI355448[] = {{&S95745[19], 3}, {NULL}};
const Segment SGI355449[] = {{&S95748[35], 4}, {NULL}};
const Segment SGI355450[] = {{&S95748[90], 3}, {NULL}};
const Segment SGI355451[] = {{&S87927[28], 5}, {NULL}};
const Segment SGI355452[] = {{&S87927[104], 5}, {NULL}};
const Segment SGI355453[] = {{&S95773[5], 3}, {&S95773[5], 3}, {NULL}};
const Segment SGI355454[] = {{&S95773[67], 3}, {NULL}};
const Segment SGI355455[] = {{&S95773[126], 3}, {NULL}};
const Segment SGI355456[] = {{&S95827[5], 11}, {&S95827[5], 11}, {NULL}};
const Segment SGI355457[] = {{&S95827[17], 3}, {NULL}};
const Segment SGI355458[] = {{&S95827[143], 3}, {NULL}};
const Segment SGI355459[] = {{&S66643[72], 5}, {NULL}};
const Segment SGI355460[] = {{&S66643[12], 12}, {NULL}};
const Segment SGI355461[] = {{&S95837[5], 3}, {&S95837[5], 3}, {NULL}};
const Segment SGI355462[] = {{&S95837[80], 3}, {NULL}};
const Segment SGI355463[] = {{&S95861[101], 5}, {NULL}};
const Segment SGI355464[] = {{&S95874[5], 3}, {&S95874[5], 3}, {NULL}};
const Segment SGI355465[] = {{&S95874[149], 3}, {NULL}};
const Segment SGI355466[] = {{&S95886[5], 5}, {&S95886[105], 5}, {NULL}};
const Segment SGI355467[] = {{&S95886[11], 3}, {NULL}};
const Segment SGI355468[] = {{&S95886[131], 3}, {NULL}};
const Segment SGI355469[] = {{&S95976[5], 4}, {NULL}};
const Segment SGI355470[] = {{&S95976[20], 3}, {NULL}};
const Segment SGI355471[] = {{&S95999[5], 4}, {&S95999[5], 4}, {NULL}};
const Segment SGI355472[] = {{&S95999[82], 3}, {NULL}};
const Segment SGI355473[] = {{&S96035[34], 4}, {NULL}};
const Segment SGI355474[] = {{&S96035[89], 3}, {NULL}};
const Segment SGI355475[] = {{&S69590[6], 4}, {NULL}};
const Segment SGI355476[] = {{&S69590[52], 5}, {NULL}};
const Segment SGI355477[] = {{&S69590[12], 45}, {NULL}};
const Segment SGI355478[] = {{&S69590[12], 14}, {NULL}};
const Segment SGI355479[] = {{&S69591[6], 4}, {NULL}};
const Segment SGI355480[] = {{&S69591[64], 5}, {NULL}};
const Segment SGI355481[] = {{&S69591[12], 14}, {NULL}};
const Segment SGI355482[] = {{&S69604[6], 4}, {NULL}};
const Segment SGI355483[] = {{&S69604[58], 5}, {NULL}};
const Segment SGI355484[] = {{&S69604[12], 14}, {NULL}};
const Segment SGI355485[] = {{&S96138[25], 7}, {NULL}};
const Segment SGI355486[] = {{&S96138[0], 18}, {NULL}};
const Segment SGI355487[] = {{&S69703[6], 4}, {NULL}};
const Segment SGI355488[] = {{&S69703[12], 14}, {NULL}};
const Segment SGI355489[] = {{&S69704[6], 4}, {NULL}};
const Segment SGI355490[] = {{&S69704[12], 14}, {NULL}};
const Segment SGI355491[] = {{&S69705[6], 4}, {NULL}};
const Segment SGI355492[] = {{&S69705[12], 14}, {NULL}};
const Segment SGI355493[] = {{&S96184[101], 3}, {NULL}};
const Segment SGI355494[] = {{&S69719[6], 4}, {NULL}};
const Segment SGI355495[] = {{&S69719[43], 4}, {NULL}};
const Segment SGI355496[] = {{&S69719[12], 14}, {NULL}};
const Segment SGI355497[] = {{&S69720[6], 4}, {NULL}};
const Segment SGI355498[] = {{&S69720[43], 4}, {NULL}};
const Segment SGI355499[] = {{&S69720[12], 14}, {NULL}};
const Segment SGI355500[] = {{&S69721[6], 4}, {NULL}};
const Segment SGI355501[] = {{&S69721[57], 5}, {NULL}};
const Segment SGI355502[] = {{&S69721[12], 14}, {NULL}};
const Segment SGI355503[] = {{&S69722[6], 4}, {NULL}};
const Segment SGI355504[] = {{&S69722[43], 4}, {NULL}};
const Segment SGI355505[] = {{&S69722[12], 14}, {NULL}};
const Segment SGI355506[] = {{&S69723[6], 4}, {NULL}};
const Segment SGI355507[] = {{&S69723[43], 4}, {NULL}};
const Segment SGI355508[] = {{&S69723[12], 14}, {NULL}};
const Segment SGI355509[] = {{&S69724[6], 4}, {NULL}};
const Segment SGI355510[] = {{&S69724[57], 5}, {NULL}};
const Segment SGI355511[] = {{&S69724[12], 14}, {NULL}};
const Segment SGI355512[] = {{&S69725[6], 4}, {NULL}};
const Segment SGI355513[] = {{&S69725[57], 5}, {NULL}};
const Segment SGI355514[] = {{&S69725[12], 14}, {NULL}};
const Segment SGI355515[] = {{&S69726[6], 4}, {NULL}};
const Segment SGI355516[] = {{&S69726[57], 5}, {NULL}};
const Segment SGI355517[] = {{&S69726[12], 14}, {NULL}};
const Segment SGI355518[] = {{&S96217[5], 5}, {NULL}};
const Segment SGI355519[] = {{&S96217[169], 3}, {NULL}};
const Segment SGI355520[] = {{&S96219[5], 7}, {&S96219[5], 7}, {NULL}};
const Segment SGI355521[] = {{&S96219[13], 3}, {NULL}};
const Segment SGI355522[] = {{&S96219[135], 3}, {NULL}};
const Segment SGI355523[] = {{&S69785[6], 4}, {NULL}};
const Segment SGI355524[] = {{&S69785[59], 5}, {NULL}};
const Segment SGI355525[] = {{&S69785[12], 3}, {NULL}};
const Segment SGI355526[] = {{&S69786[6], 4}, {NULL}};
const Segment SGI355527[] = {{&S69786[61], 5}, {NULL}};
const Segment SGI355528[] = {{&S69786[12], 3}, {NULL}};
const Segment SGI355529[] = {{&S69787[6], 4}, {NULL}};
const Segment SGI355530[] = {{&S69787[53], 5}, {NULL}};
const Segment SGI355531[] = {{&S69787[12], 10}, {NULL}};
const Segment SGI355532[] = {{&S83148[19], 15}, {NULL}};
const Segment SGI355533[] = {{&S83148[48], 3}, {NULL}};
const Segment SGI355534[] = {{&S96313[5], 4}, {NULL}};
const Segment SGI355535[] = {{&S96313[20], 3}, {NULL}};
const Segment SGI355536[] = {{&S96317[5], 4}, {NULL}};
const Segment SGI355537[] = {{&S96336[5], 5}, {NULL}};
const Segment SGI355538[] = {{&S96336[21], 3}, {NULL}};
const Segment SGI355539[] = {{&S96362[5], 3}, {NULL}};
const Segment SGI355540[] = {{&S96362[233], 3}, {NULL}};
const Segment SGI355541[] = {{&S88544[28], 5}, {NULL}};
const Segment SGI355542[] = {{&S88544[104], 5}, {NULL}};
const Segment SGI355543[] = {{&S88545[28], 7}, {NULL}};
const Segment SGI355544[] = {{&S88545[106], 5}, {NULL}};
const Segment SGI355545[] = {{&S96404[5], 4}, {&S96404[5], 4}, {NULL}};
const Segment SGI355546[] = {{&S96404[10], 3}, {NULL}};
const Segment SGI355547[] = {{&S96404[129], 3}, {NULL}};
const Segment SGI355548[] = {{&S96416[34], 4}, {NULL}};
const Segment SGI355549[] = {{&S96416[89], 3}, {NULL}};
const Segment SGI355550[] = {{&S96433[42], 3}, {NULL}};
const Segment SGI355551[] = {{&S96433[6], 3}, {NULL}};
const Segment SGI355552[] = {{&S96433[66], 3}, {NULL}};
const Segment SGI355553[] = {{&S96466[21], 15}, {NULL}};
const Segment SGI355554[] = {{&S96466[0], 14}, {NULL}};
const Segment SGI355555[] = {{&S69966[6], 4}, {NULL}};
const Segment SGI355556[] = {{&S69966[76], 5}, {NULL}};
const Segment SGI355557[] = {{&S69966[12], 9}, {NULL}};
const Segment SGI355558[] = {{&S69966[29], 13}, {NULL}};
const Segment SGI355559[] = {{&S96572[43], 5}, {NULL}};
const Segment SGI355560[] = {{&S96572[6], 3}, {NULL}};
const Segment SGI355561[] = {{&S96572[69], 3}, {NULL}};
const Segment SGI355562[] = {{&S96590[5], 7}, {&S96590[5], 7}, {NULL}};
const Segment SGI355563[] = {{&S96590[88], 3}, {NULL}};
const Segment SGI355564[] = {{&S96591[5], 4}, {&S96591[5], 4}, {NULL}};
const Segment SGI355565[] = {{&S96591[82], 3}, {NULL}};
const Segment SGI355566[] = {{&S96609[5], 16}, {NULL}};
const Segment SGI355567[] = {{&S96609[32], 3}, {NULL}};
const Segment SGI355568[] = {{&S96619[69], 11}, {NULL}};
const Segment SGI355569[] = {{&S73110[61], 5}, {NULL}};
const Segment SGI355570[] = {{&S73110[13], 10}, {NULL}};
const Segment SGI355571[] = {{&S96628[5], 4}, {NULL}};
const Segment SGI355572[] = {{&S96628[20], 3}, {NULL}};
const Segment SGI355573[] = {{&S70097[6], 4}, {NULL}};
const Segment SGI355574[] = {{&S70097[59], 5}, {NULL}};
const Segment SGI355575[] = {{&S70097[12], 14}, {NULL}};
const Segment SGI355576[] = {{&S67375[72], 5}, {NULL}};
const Segment SGI355577[] = {{&S67375[12], 14}, {NULL}};
const Segment SGI355578[] = {{&S70189[6], 4}, {NULL}};
const Segment SGI355579[] = {{&S70189[90], 5}, {NULL}};
const Segment SGI355580[] = {{&S70189[12], 9}, {NULL}};
const Segment SGI355581[] = {{&S70189[29], 13}, {NULL}};
const Segment SGI355582[] = {{&S96633[5], 4}, {NULL}};
const Segment SGI355583[] = {{&S96633[20], 3}, {NULL}};
const Segment SGI355584[] = {{&S96636[35], 3}, {NULL}};
const Segment SGI355585[] = {{&S96636[50], 3}, {NULL}};
const Segment SGI355586[] = {{&S96637[35], 5}, {NULL}};
const Segment SGI355587[] = {{&S96637[52], 3}, {NULL}};
const Segment SGI355588[] = {{&S96658[35], 7}, {NULL}};
const Segment SGI355589[] = {{&S96658[54], 3}, {NULL}};
const Segment SGI355590[] = {{&S70228[6], 4}, {NULL}};
const Segment SGI355591[] = {{&S70228[12], 14}, {NULL}};
const Segment SGI355592[] = {{&S96668[35], 3}, {NULL}};
const Segment SGI355593[] = {{&S96668[50], 3}, {NULL}};
const Segment SGI355594[] = {{&S70229[6], 4}, {NULL}};
const Segment SGI355595[] = {{&S70229[12], 14}, {NULL}};
const Segment SGI355596[] = {{&S70230[6], 4}, {NULL}};
const Segment SGI355597[] = {{&S70230[12], 14}, {NULL}};
const Segment SGI355598[] = {{&S70231[6], 4}, {NULL}};
const Segment SGI355599[] = {{&S70231[12], 14}, {NULL}};
const Segment SGI355600[] = {{&S96673[5], 3}, {&S96673[5], 3}, {NULL}};
const Segment SGI355601[] = {{&S96673[9], 3}, {NULL}};
const Segment SGI355602[] = {{&S96673[126], 3}, {NULL}};
const Segment SGI355603[] = {{&S96676[5], 11}, {&S96676[5], 11}, {NULL}};
const Segment SGI355604[] = {{&S96676[17], 3}, {NULL}};
const Segment SGI355605[] = {{&S96676[142], 3}, {NULL}};
const Segment SGI355606[] = {{&S96707[21], 3}, {NULL}};
const Segment SGI355607[] = {{&S96707[0], 14}, {NULL}};
const Segment SGI355608[] = {{&S96716[5], 5}, {NULL}};
const Segment SGI355609[] = {{&S96716[235], 3}, {NULL}};
const Segment SGI355610[] = {{&S96733[5], 7}, {&S96733[5], 7}, {NULL}};
const Segment SGI355611[] = {{&S96733[13], 3}, {NULL}};
const Segment SGI355612[] = {{&S96733[135], 3}, {NULL}};
const Segment SGI355613[] = {{&S96743[21], 5}, {NULL}};
const Segment SGI355614[] = {{&S96743[0], 14}, {NULL}};
const Segment SGI355615[] = {{&S96752[5], 3}, {&S96752[5], 3}, {NULL}};
const Segment SGI355616[] = {{&S96752[149], 3}, {NULL}};
const Segment SGI355617[] = {{&S96832[5], 7}, {NULL}};
const Segment SGI355618[] = {{&S96832[171], 3}, {NULL}};
const Segment SGI355619[] = {{&S96875[42], 5}, {NULL}};
const Segment SGI355620[] = {{&S96875[6], 3}, {NULL}};
const Segment SGI355621[] = {{&S96875[68], 3}, {NULL}};
const Segment SGI355622[] = {{&S96881[43], 3}, {NULL}};
const Segment SGI355623[] = {{&S96881[6], 3}, {NULL}};
const Segment SGI355624[] = {{&S96881[67], 3}, {NULL}};
const Segment SGI355625[] = {{&S96896[48], 20}, {NULL}};
const Segment SGI355626[] = {{&S96921[5], 9}, {NULL}};
const Segment SGI355627[] = {{&S96929[5], 7}, {&S96929[5], 7}, {NULL}};
const Segment SGI355628[] = {{&S96929[88], 3}, {NULL}};
const Segment SGI355629[] = {{&S96940[5], 4}, {NULL}};
const Segment SGI355630[] = {{&S96940[20], 3}, {NULL}};
const Segment SGI355631[] = {{&S96941[5], 5}, {&S96941[68], 5}, {NULL}};
const Segment SGI355632[] = {{&S96941[84], 3}, {NULL}};
const Segment SGI355633[] = {{&S96976[35], 8}, {NULL}};
const Segment SGI355634[] = {{&S96976[55], 3}, {NULL}};
const Segment SGI355635[] = {{&S96977[5], 8}, {NULL}};
const Segment SGI355636[] = {{&S96977[19], 3}, {NULL}};
const Segment SGI355637[] = {{&S97000[35], 3}, {NULL}};
const Segment SGI355638[] = {{&S97000[50], 3}, {NULL}};
const Segment SGI355639[] = {{&S83936[21], 15}, {NULL}};
const Segment SGI355640[] = {{&S83936[50], 3}, {NULL}};
const Segment SGI355641[] = {{&S97008[5], 3}, {NULL}};
const Segment SGI355642[] = {{&S97008[234], 3}, {NULL}};
const Segment SGI355643[] = {{&S73610[6], 3}, {NULL}};
const Segment SGI355644[] = {{&S73610[11], 14}, {NULL}};
const Segment SGI355645[] = {{&S97098[5], 5}, {&S97098[97], 5}, {NULL}};
const Segment SGI355646[] = {{&S97098[153], 3}, {NULL}};
const Segment SGI355647[] = {{&S97104[5], 4}, {&S97104[5], 4}, {NULL}};
const Segment SGI355648[] = {{&S97104[10], 3}, {NULL}};
const Segment SGI355649[] = {{&S97104[129], 3}, {NULL}};
const Segment SGI355650[] = {{&S97199[42], 3}, {NULL}};
const Segment SGI355651[] = {{&S97199[6], 3}, {NULL}};
const Segment SGI355652[] = {{&S97199[66], 3}, {NULL}};
const Segment SGI355653[] = {{&S97209[5], 13}, {NULL}};
const Segment SGI355654[] = {{&S97209[177], 3}, {NULL}};
const Segment SGI355655[] = {{&S97215[43], 7}, {NULL}};
const Segment SGI355656[] = {{&S97215[6], 3}, {NULL}};
const Segment SGI355657[] = {{&S97215[71], 3}, {NULL}};
const Segment SGI355658[] = {{&S97218[43], 3}, {NULL}};
const Segment SGI355659[] = {{&S97218[6], 3}, {NULL}};
const Segment SGI355660[] = {{&S97218[67], 3}, {NULL}};
const Segment SGI355661[] = {{&S97226[43], 5}, {NULL}};
const Segment SGI355662[] = {{&S97226[6], 3}, {NULL}};
const Segment SGI355663[] = {{&S97226[69], 3}, {NULL}};
const Segment SGI355664[] = {{&S97273[5], 4}, {NULL}};
const Segment SGI355665[] = {{&S97273[15], 3}, {NULL}};
const Segment SGI355666[] = {{&S89603[28], 12}, {NULL}};
const Segment SGI355667[] = {{&S89603[111], 5}, {NULL}};
const Segment SGI355668[] = {{&S97298[5], 3}, {NULL}};
const Segment SGI355669[] = {{&S97298[19], 3}, {NULL}};
const Segment SGI355670[] = {{&S97299[5], 7}, {&S97299[5], 7}, {NULL}};
const Segment SGI355671[] = {{&S97299[88], 3}, {NULL}};
const Segment SGI355672[] = {{&S97310[23], 3}, {NULL}};
const Segment SGI355673[] = {{&S97310[0], 16}, {NULL}};
const Segment SGI355674[] = {{&S97313[5], 5}, {NULL}};
const Segment SGI355675[] = {{&S97313[21], 3}, {NULL}};
const Segment SGI355676[] = {{&S97317[34], 3}, {NULL}};
const Segment SGI355677[] = {{&S97317[48], 3}, {NULL}};
const Segment SGI355678[] = {{&S97320[5], 5}, {NULL}};
const Segment SGI355679[] = {{&S97320[235], 3}, {NULL}};
const Segment SGI355680[] = {{&S97324[35], 14}, {NULL}};
const Segment SGI355681[] = {{&S97324[61], 3}, {NULL}};
const Segment SGI355682[] = {{&S97325[5], 5}, {NULL}};
const Segment SGI355683[] = {{&S97325[21], 3}, {NULL}};
const Segment SGI355684[] = {{&S84177[21], 14}, {NULL}};
const Segment SGI355685[] = {{&S84177[49], 3}, {NULL}};
const Segment SGI355686[] = {{&S97369[5], 7}, {NULL}};
const Segment SGI355687[] = {{&S97369[238], 3}, {NULL}};
const Segment SGI355688[] = {{&S97418[5], 4}, {NULL}};
const Segment SGI355689[] = {{&S97418[20], 3}, {NULL}};
const Segment SGI355690[] = {{&S84262[5], 4}, {NULL}};
const Segment SGI355691[] = {{&S84262[49], 3}, {NULL}};
const Segment SGI355692[] = {{&S97432[42], 3}, {NULL}};
const Segment SGI355693[] = {{&S97432[6], 3}, {NULL}};
const Segment SGI355694[] = {{&S97432[66], 3}, {NULL}};
const Segment SGI355695[] = {{&S97453[101], 3}, {NULL}};
const Segment SGI355696[] = {{&S97479[21], 5}, {NULL}};
const Segment SGI355697[] = {{&S97479[0], 14}, {NULL}};
const Segment SGI355698[] = {{&S70951[6], 4}, {NULL}};
const Segment SGI355699[] = {{&S70951[67], 5}, {NULL}};
const Segment SGI355700[] = {{&S70951[12], 3}, {NULL}};
const Segment SGI355701[] = {{&S97514[42], 5}, {NULL}};
const Segment SGI355702[] = {{&S97514[6], 3}, {NULL}};
const Segment SGI355703[] = {{&S97514[68], 3}, {NULL}};
const Segment SGI355704[] = {{&S97515[43], 3}, {NULL}};
const Segment SGI355705[] = {{&S97515[6], 3}, {NULL}};
const Segment SGI355706[] = {{&S97515[67], 3}, {NULL}};
const Segment SGI355707[] = {{&S97560[5], 3}, {NULL}};
const Segment SGI355708[] = {{&S97560[167], 3}, {NULL}};
const Segment SGI355709[] = {{&S97585[43], 5}, {NULL}};
const Segment SGI355710[] = {{&S97585[6], 3}, {NULL}};
const Segment SGI355711[] = {{&S97585[69], 3}, {NULL}};
const Segment SGI355712[] = {{&S97604[43], 3}, {NULL}};
const Segment SGI355713[] = {{&S97604[6], 3}, {NULL}};
const Segment SGI355714[] = {{&S97604[67], 3}, {NULL}};
const Segment SGI355715[] = {{&S97610[5], 3}, {&S97610[5], 3}, {NULL}};
const Segment SGI355716[] = {{&S97610[9], 3}, {NULL}};
const Segment SGI355717[] = {{&S97610[127], 3}, {NULL}};
const Segment SGI355718[] = {{&S97616[5], 5}, {&S97616[5], 5}, {NULL}};
const Segment SGI355719[] = {{&S97616[11], 3}, {NULL}};
const Segment SGI355720[] = {{&S97616[131], 3}, {NULL}};
const Segment SGI355721[] = {{&S97654[5], 9}, {NULL}};
const Segment SGI355722[] = {{&S97661[5], 5}, {NULL}};
const Segment SGI355723[] = {{&S97661[21], 3}, {NULL}};
const Segment SGI355724[] = {{&S97669[5], 11}, {&S97669[5], 11}, {NULL}};
const Segment SGI355725[] = {{&S97669[96], 3}, {NULL}};
const Segment SGI355726[] = {{&S97672[5], 5}, {NULL}};
const Segment SGI355727[] = {{&S97672[21], 3}, {NULL}};
const Segment SGI355728[] = {{&S97673[5], 5}, {NULL}};
const Segment SGI355729[] = {{&S97673[21], 3}, {NULL}};
const Segment SGI355730[] = {{&S97676[5], 7}, {&S97676[5], 7}, {NULL}};
const Segment SGI355731[] = {{&S97676[88], 3}, {NULL}};
const Segment SGI355732[] = {{&S71142[6], 4}, {NULL}};
const Segment SGI355733[] = {{&S71142[83], 5}, {NULL}};
const Segment SGI355734[] = {{&S71142[12], 11}, {NULL}};
const Segment SGI355735[] = {{&S97707[5], 5}, {NULL}};
const Segment SGI355736[] = {{&S97707[21], 3}, {NULL}};
const Segment SGI355737[] = {{&S97708[5], 5}, {NULL}};
const Segment SGI355738[] = {{&S97708[21], 3}, {NULL}};
const Segment SGI355739[] = {{&S97725[5], 7}, {&S97725[5], 7}, {NULL}};
const Segment SGI355740[] = {{&S97725[157], 3}, {NULL}};
const Segment SGI355741[] = {{&S97735[21], 5}, {NULL}};
const Segment SGI355742[] = {{&S97735[0], 14}, {NULL}};
const Segment SGI355743[] = {{&S97739[5], 5}, {&S97739[95], 5}, {NULL}};
const Segment SGI355744[] = {{&S97739[111], 3}, {NULL}};
const Segment SGI355745[] = {{&S97761[34], 5}, {NULL}};
const Segment SGI355746[] = {{&S97761[50], 3}, {NULL}};
const Segment SGI355747[] = {{&S97775[5], 5}, {&S97775[68], 5}, {NULL}};
const Segment SGI355748[] = {{&S97775[84], 3}, {NULL}};
const Segment SGI355749[] = {{&S71268[6], 4}, {NULL}};
const Segment SGI355750[] = {{&S71268[12], 10}, {NULL}};
const Segment SGI355751[] = {{&S71269[6], 4}, {NULL}};
const Segment SGI355752[] = {{&S71269[12], 10}, {NULL}};
const Segment SGI355753[] = {{&S71270[6], 4}, {NULL}};
const Segment SGI355754[] = {{&S71270[12], 10}, {NULL}};
const Segment SGI355755[] = {{&S71271[6], 4}, {NULL}};
const Segment SGI355756[] = {{&S71271[12], 10}, {NULL}};
const Segment SGI355757[] = {{&S97858[43], 3}, {NULL}};
const Segment SGI355758[] = {{&S97858[6], 3}, {NULL}};
const Segment SGI355759[] = {{&S97858[67], 3}, {NULL}};
const Segment SGI355760[] = {{&S97873[5], 3}, {&S97873[5], 3}, {NULL}};
const Segment SGI355761[] = {{&S97873[149], 3}, {NULL}};
const Segment SGI355762[] = {{&S97879[43], 3}, {NULL}};
const Segment SGI355763[] = {{&S97879[6], 3}, {NULL}};
const Segment SGI355764[] = {{&S97879[67], 3}, {NULL}};
const Segment SGI355765[] = {{&S97886[42], 5}, {NULL}};
const Segment SGI355766[] = {{&S97886[6], 3}, {NULL}};
const Segment SGI355767[] = {{&S97886[68], 3}, {NULL}};
const Segment SGI355768[] = {{&S71366[6], 4}, {NULL}};
const Segment SGI355769[] = {{&S71366[53], 5}, {NULL}};
const Segment SGI355770[] = {{&S71366[12], 10}, {NULL}};
const Segment SGI355771[] = {{&S71367[6], 4}, {NULL}};
const Segment SGI355772[] = {{&S71367[53], 5}, {NULL}};
const Segment SGI355773[] = {{&S71367[12], 10}, {NULL}};
const Segment SGI355774[] = {{&S71368[6], 4}, {NULL}};
const Segment SGI355775[] = {{&S71368[53], 5}, {NULL}};
const Segment SGI355776[] = {{&S71368[12], 10}, {NULL}};
const Segment SGI355777[] = {{&S97945[5], 5}, {NULL}};
const Segment SGI355778[] = {{&S97945[169], 3}, {NULL}};
const Segment SGI355779[] = {{&S97973[5], 3}, {NULL}};
const Segment SGI355780[] = {{&S97973[167], 3}, {NULL}};
const Segment SGI355781[] = {{&S97988[43], 7}, {NULL}};
const Segment SGI355782[] = {{&S97988[6], 3}, {NULL}};
const Segment SGI355783[] = {{&S97988[71], 3}, {NULL}};
const Segment SGI355784[] = {{&S97994[43], 7}, {NULL}};
const Segment SGI355785[] = {{&S97994[6], 3}, {NULL}};
const Segment SGI355786[] = {{&S97994[71], 3}, {NULL}};
const Segment SGI355787[] = {{&S98070[5], 12}, {&S98070[5], 12}, {NULL}};
const Segment SGI355788[] = {{&S98070[98], 3}, {NULL}};
const Segment SGI355789[] = {{&S84841[49], 5}, {NULL}};
const Segment SGI355790[] = {{&S84841[40], 3}, {NULL}};
const Segment SGI355791[] = {{&S98071[5], 5}, {&S98071[68], 5}, {NULL}};
const Segment SGI355792[] = {{&S98071[84], 3}, {NULL}};
const Segment SGI355793[] = {{&S98076[5], 5}, {NULL}};
const Segment SGI355794[] = {{&S98076[21], 3}, {NULL}};
const Segment SGI355795[] = {{&S98082[5], 5}, {NULL}};
const Segment SGI355796[] = {{&S98082[21], 3}, {NULL}};
const Segment SGI355797[] = {{&S98083[5], 4}, {NULL}};
const Segment SGI355798[] = {{&S98083[20], 3}, {NULL}};
const Segment SGI355799[] = {{&S98100[5], 4}, {NULL}};
const Segment SGI355800[] = {{&S98100[20], 3}, {NULL}};
const Segment SGI355801[] = {{&S98101[5], 3}, {NULL}};
const Segment SGI355802[] = {{&S98101[19], 3}, {NULL}};
const Segment SGI355803[] = {{&S98107[5], 4}, {NULL}};
const Segment SGI355804[] = {{&S98107[15], 3}, {NULL}};
const Segment SGI355805[] = {{&S98119[23], 5}, {NULL}};
const Segment SGI355806[] = {{&S98119[0], 16}, {NULL}};
const Segment SGI355807[] = {{&S98141[5], 5}, {NULL}};
const Segment SGI355808[] = {{&S98141[236], 3}, {NULL}};
const Segment SGI355809[] = {{&S98143[5], 4}, {NULL}};
const Segment SGI355810[] = {{&S98143[20], 3}, {NULL}};
const Segment SGI355811[] = {{&S74405[57], 4}, {NULL}};
const Segment SGI355812[] = {{&S74405[13], 10}, {NULL}};
const Segment SGI355813[] = {{&S98157[34], 7}, {NULL}};
const Segment SGI355814[] = {{&S98157[52], 3}, {NULL}};
const Segment SGI355815[] = {{&S98160[5], 4}, {&S98160[5], 4}, {NULL}};
const Segment SGI355816[] = {{&S98160[10], 3}, {NULL}};
const Segment SGI355817[] = {{&S98160[128], 3}, {NULL}};
const Segment SGI355818[] = {{&S84845[5], 5}, {NULL}};
const Segment SGI355819[] = {{&S84845[33], 3}, {NULL}};
const Segment SGI355820[] = {{&S84845[87], 3}, {NULL}};
const Segment SGI355821[] = {{&S84846[5], 3}, {NULL}};
const Segment SGI355822[] = {{&S84846[32], 3}, {NULL}};
const Segment SGI355823[] = {{&S84846[86], 3}, {NULL}};
const Segment SGI355824[] = {{&S98225[5], 3}, {&S98225[5], 3}, {NULL}};
const Segment SGI355825[] = {{&S98225[109], 3}, {NULL}};
const Segment SGI355826[] = {{&S98248[34], 3}, {NULL}};
const Segment SGI355827[] = {{&S98248[88], 3}, {NULL}};
const Segment SGI355828[] = {{&S98251[5], 3}, {&S98251[5], 3}, {NULL}};
const Segment SGI355829[] = {{&S98251[80], 3}, {NULL}};
const Segment SGI355830[] = {{&S98257[21], 5}, {NULL}};
const Segment SGI355831[] = {{&S98257[0], 14}, {NULL}};
const Segment SGI355832[] = {{&S98315[42], 5}, {NULL}};
const Segment SGI355833[] = {{&S98315[6], 3}, {NULL}};
const Segment SGI355834[] = {{&S98315[68], 3}, {NULL}};
const Segment SGI355835[] = {{&S98316[42], 3}, {NULL}};
const Segment SGI355836[] = {{&S98316[6], 3}, {NULL}};
const Segment SGI355837[] = {{&S98316[66], 3}, {NULL}};
const Segment SGI355838[] = {{&S98318[41], 4}, {NULL}};
const Segment SGI355839[] = {{&S98318[6], 3}, {NULL}};
const Segment SGI355840[] = {{&S98318[66], 3}, {NULL}};
const Segment SGI355841[] = {{&S98320[42], 3}, {NULL}};
const Segment SGI355842[] = {{&S98320[6], 3}, {NULL}};
const Segment SGI355843[] = {{&S98320[66], 3}, {NULL}};
const Segment SGI355844[] = {{&S98324[43], 4}, {NULL}};
const Segment SGI355845[] = {{&S98324[6], 3}, {NULL}};
const Segment SGI355846[] = {{&S98324[68], 3}, {NULL}};
const Segment SGI355847[] = {{&S98332[5], 5}, {NULL}};
const Segment SGI355848[] = {{&S98332[21], 3}, {NULL}};
const Segment SGI355849[] = {{&S98335[21], 3}, {NULL}};
const Segment SGI355850[] = {{&S98335[0], 14}, {NULL}};
const Segment SGI355851[] = {{&S98358[43], 3}, {NULL}};
const Segment SGI355852[] = {{&S98358[6], 3}, {NULL}};
const Segment SGI355853[] = {{&S98358[67], 3}, {NULL}};
const Segment SGI355854[] = {{&S98361[101], 5}, {NULL}};
const Segment SGI355855[] = {{&S98387[5], 4}, {NULL}};
const Segment SGI355856[] = {{&S98387[168], 3}, {NULL}};
const Segment SGI355857[] = {{&S98393[5], 5}, {NULL}};
const Segment SGI355858[] = {{&S98393[169], 3}, {NULL}};
const Segment SGI355859[] = {{&S98395[43], 5}, {NULL}};
const Segment SGI355860[] = {{&S98395[6], 3}, {NULL}};
const Segment SGI355861[] = {{&S98395[69], 3}, {NULL}};
const Segment SGI355862[] = {{&S98426[43], 3}, {NULL}};
const Segment SGI355863[] = {{&S98426[6], 3}, {NULL}};
const Segment SGI355864[] = {{&S98426[67], 3}, {NULL}};
const Segment SGI355865[] = {{&S98464[5], 3}, {&S98464[5], 3}, {NULL}};
const Segment SGI355866[] = {{&S98464[9], 3}, {NULL}};
const Segment SGI355867[] = {{&S98464[126], 3}, {NULL}};
const Segment SGI355868[] = {{&S98479[43], 5}, {NULL}};
const Segment SGI355869[] = {{&S98479[6], 3}, {NULL}};
const Segment SGI355870[] = {{&S98479[69], 3}, {NULL}};
const Segment SGI355871[] = {{&S98482[42], 5}, {NULL}};
const Segment SGI355872[] = {{&S98482[6], 3}, {NULL}};
const Segment SGI355873[] = {{&S98482[68], 3}, {NULL}};
const Segment SGI355874[] = {{&S98532[5], 5}, {NULL}};
const Segment SGI355875[] = {{&S71897[6], 4}, {NULL}};
const Segment SGI355876[] = {{&S71897[12], 14}, {NULL}};
const Segment SGI355877[] = {{&S74854[67], 4}, {NULL}};
const Segment SGI355878[] = {{&S74854[13], 10}, {NULL}};
const Segment SGI355879[] = {{&S74856[51], 3}, {NULL}};
const Segment SGI355880[] = {{&S74856[13], 9}, {NULL}};
const Segment SGI355881[] = {{&S98544[5], 3}, {NULL}};
const Segment SGI355882[] = {{&S98544[19], 3}, {NULL}};
const Segment SGI355883[] = {{&S98545[5], 5}, {&S98545[68], 5}, {NULL}};
const Segment SGI355884[] = {{&S98545[84], 3}, {NULL}};
const Segment SGI355885[] = {{&S98546[5], 4}, {NULL}};
const Segment SGI355886[] = {{&S98546[15], 3}, {NULL}};
const Segment SGI355887[] = {{&S98547[5], 3}, {NULL}};
const Segment SGI355888[] = {{&S98547[19], 3}, {NULL}};
const Segment SGI355889[] = {{&S98554[5], 4}, {NULL}};
const Segment SGI355890[] = {{&S98554[20], 3}, {NULL}};
const Segment SGI355891[] = {{&S98564[5], 5}, {NULL}};
const Segment SGI355892[] = {{&S98564[21], 3}, {NULL}};
const Segment SGI355893[] = {{&S98602[5], 5}, {NULL}};
const Segment SGI355894[] = {{&S98602[21], 3}, {NULL}};
const Segment SGI355895[] = {{&S98604[5], 10}, {NULL}};
const Segment SGI355896[] = {{&S98604[21], 3}, {NULL}};
const Segment SGI355897[] = {{&S98616[35], 5}, {NULL}};
const Segment SGI355898[] = {{&S98616[91], 3}, {NULL}};
const Segment SGI355899[] = {{&S98703[5], 3}, {&S98703[5], 3}, {NULL}};
const Segment SGI355900[] = {{&S98703[80], 3}, {NULL}};
const Segment SGI355901[] = {{&S98710[34], 4}, {NULL}};
const Segment SGI355902[] = {{&S98710[89], 3}, {NULL}};
const Segment SGI355903[] = {{&S98714[5], 7}, {&S98714[5], 7}, {NULL}};
const Segment SGI355904[] = {{&S98714[157], 3}, {NULL}};
const Segment SGI355905[] = {{&S98720[42], 5}, {NULL}};
const Segment SGI355906[] = {{&S98720[6], 3}, {NULL}};
const Segment SGI355907[] = {{&S98720[68], 3}, {NULL}};
const Segment SGI355908[] = {{&S75119[149], 4}, {NULL}};
const Segment SGI355909[] = {{&S75119[13], 10}, {NULL}};
const Segment SGI355910[] = {{&S98810[101], 8}, {NULL}};
const Segment SGI355911[] = {{&S90887[28], 5}, {NULL}};
const Segment SGI355912[] = {{&S90887[104], 5}, {NULL}};
const Segment SGI355913[] = {{&S98871[101], 4}, {NULL}};
const Segment SGI355914[] = {{&S98884[5], 5}, {NULL}};
const Segment SGI355915[] = {{&S98884[169], 3}, {NULL}};
const Segment SGI355916[] = {{&S98896[5], 3}, {&S98896[5], 3}, {NULL}};
const Segment SGI355917[] = {{&S98896[80], 3}, {NULL}};
const Segment SGI355918[] = {{&S98911[49], 20}, {NULL}};
const Segment SGI355919[] = {{&S98943[5], 4}, {&S98943[5], 4}, {NULL}};
const Segment SGI355920[] = {{&S98943[10], 3}, {NULL}};
const Segment SGI355921[] = {{&S98943[129], 3}, {NULL}};
const Segment SGI355922[] = {{&S91046[28], 7}, {NULL}};
const Segment SGI355923[] = {{&S91046[106], 5}, {NULL}};
const Segment SGI355924[] = {{&S98973[42], 5}, {NULL}};
const Segment SGI355925[] = {{&S98973[6], 3}, {NULL}};
const Segment SGI355926[] = {{&S98973[68], 3}, {NULL}};
const Segment SGI355927[] = {{&S98974[43], 9}, {NULL}};
const Segment SGI355928[] = {{&S98974[6], 3}, {NULL}};
const Segment SGI355929[] = {{&S98974[73], 3}, {NULL}};
const Segment SGI355930[] = {{&S85642[5], 6}, {NULL}};
const Segment SGI355931[] = {{&S85642[44], 3}, {NULL}};
const Segment SGI355932[] = {{&S98995[5], 3}, {&S98995[5], 3}, {NULL}};
const Segment SGI355933[] = {{&S98995[9], 3}, {NULL}};
const Segment SGI355934[] = {{&S98995[127], 3}, {NULL}};
const Segment SGI355935[] = {{&S91130[28], 5}, {NULL}};
const Segment SGI355936[] = {{&S91130[104], 5}, {NULL}};
const Segment SGI355937[] = {{&S91140[28], 5}, {NULL}};
const Segment SGI355938[] = {{&S69706[101], 5}, {NULL}};
const Segment SGI355939[] = {{&S69706[12], 9}, {&S69706[40], 3}, {NULL}};
const Segment SGI355940[] = {{&S91140[104], 5}, {NULL}};
const Segment SGI355941[] = {{&S99085[5], 5}, {NULL}};
const Segment SGI355942[] = {{&S99085[21], 3}, {NULL}};
const Segment SGI355943[] = {{&S99099[5], 5}, {NULL}};
const Segment SGI355944[] = {{&S99099[21], 3}, {NULL}};
const Segment SGI355945[] = {{&S99108[5], 5}, {NULL}};
const Segment SGI355946[] = {{&S99108[16], 3}, {NULL}};
const Segment SGI355947[] = {{&S99109[5], 5}, {NULL}};
const Segment SGI355948[] = {{&S99109[235], 3}, {NULL}};
const Segment SGI355949[] = {{&S99110[38], 5}, {NULL}};
const Segment SGI355950[] = {{&S99110[55], 3}, {NULL}};
const Segment SGI355951[] = {{&S99116[5], 5}, {NULL}};
const Segment SGI355952[] = {{&S99116[21], 3}, {NULL}};
const Segment SGI355953[] = {{&S99124[5], 11}, {&S99124[5], 11}, {NULL}};
const Segment SGI355954[] = {{&S99124[125], 3}, {NULL}};
const Segment SGI355955[] = {{&S99126[23], 5}, {NULL}};
const Segment SGI355956[] = {{&S99126[0], 16}, {NULL}};
const Segment SGI355957[] = {{&S99130[5], 5}, {&S99130[68], 5}, {NULL}};
const Segment SGI355958[] = {{&S99130[84], 3}, {NULL}};
const Segment SGI355959[] = {{&S99140[5], 5}, {&S99140[104], 5}, {NULL}};
const Segment SGI355960[] = {{&S99140[11], 3}, {NULL}};
const Segment SGI355961[] = {{&S99140[130], 3}, {NULL}};
const Segment SGI355962[] = {{&S91228[28], 4}, {NULL}};
const Segment SGI355963[] = {{&S91228[103], 5}, {NULL}};
const Segment SGI355964[] = {{&S91265[28], 3}, {NULL}};
const Segment SGI355965[] = {{&S91265[102], 5}, {NULL}};
const Segment SGI355966[] = {{&S99183[5], 4}, {&S99183[5], 4}, {NULL}};
const Segment SGI355967[] = {{&S99183[151], 3}, {NULL}};
const Segment SGI355968[] = {{&S72454[6], 4}, {NULL}};
const Segment SGI355969[] = {{&S72454[77], 5}, {NULL}};
const Segment SGI355970[] = {{&S72454[12], 9}, {NULL}};
const Segment SGI355971[] = {{&S72454[29], 13}, {NULL}};
const Segment SGI355972[] = {{&S99195[5], 5}, {&S99195[97], 5}, {NULL}};
const Segment SGI355973[] = {{&S99195[153], 3}, {NULL}};
const Segment SGI355974[] = {{&S72460[6], 4}, {NULL}};
const Segment SGI355975[] = {{&S72460[82], 5}, {NULL}};
const Segment SGI355976[] = {{&S72460[12], 3}, {NULL}};
const Segment SGI355977[] = {{&S72460[42], 10}, {NULL}};
const Segment SGI355978[] = {{&S99196[35], 3}, {NULL}};
const Segment SGI355979[] = {{&S99196[89], 3}, {NULL}};
const Segment SGI355980[] = {{&S99211[21], 13}, {NULL}};
const Segment SGI355981[] = {{&S99211[0], 14}, {NULL}};
const Segment SGI355982[] = {{&S99215[5], 4}, {&S99215[5], 4}, {NULL}};
const Segment SGI355983[] = {{&S99215[151], 3}, {NULL}};
const Segment SGI355984[] = {{&S99219[41], 5}, {NULL}};
const Segment SGI355985[] = {{&S99219[6], 3}, {NULL}};
const Segment SGI355986[] = {{&S99219[67], 3}, {NULL}};
const Segment SGI355987[] = {{&S99225[5], 5}, {NULL}};
const Segment SGI355988[] = {{&S99225[16], 3}, {NULL}};
const Segment SGI355989[] = {{&S99229[5], 3}, {&S99229[5], 3}, {NULL}};
const Segment SGI355990[] = {{&S72499[6], 4}, {NULL}};
const Segment SGI355991[] = {{&S72499[76], 5}, {NULL}};
const Segment SGI355992[] = {{&S72499[12], 9}, {NULL}};
const Segment SGI355993[] = {{&S99229[109], 3}, {NULL}};
const Segment SGI355994[] = {{&S72499[29], 13}, {NULL}};
const Segment SGI355995[] = {{&S75482[21], 4}, {NULL}};
const Segment SGI355996[] = {{&S75482[27], 3}, {NULL}};
const Segment SGI355997[] = {{&S75482[49], 3}, {NULL}};
const Segment SGI355998[] = {{&S75482[64], 5}, {NULL}};
const Segment SGI355999[] = {{&S99279[5], 5}, {NULL}};
const Segment SGI356000[] = {{&S99279[21], 3}, {NULL}};
const Segment SGI356001[] = {{&S99284[21], 7}, {NULL}};
const Segment SGI356002[] = {{&S99284[0], 14}, {NULL}};
const Segment SGI356003[] = {{&S72586[6], 4}, {NULL}};
const Segment SGI356004[] = {{&S72586[52], 5}, {NULL}};
const Segment SGI356005[] = {{&S72586[12], 45}, {NULL}};
const Segment SGI356006[] = {{&S72586[12], 14}, {NULL}};
const Segment SGI356007[] = {{&S72587[6], 4}, {NULL}};
const Segment SGI356008[] = {{&S72587[64], 5}, {NULL}};
const Segment SGI356009[] = {{&S72587[12], 14}, {NULL}};
const Segment SGI356010[] = {{&S99318[43], 3}, {NULL}};
const Segment SGI356011[] = {{&S99318[6], 3}, {NULL}};
const Segment SGI356012[] = {{&S99318[67], 3}, {NULL}};
const Segment SGI356013[] = {{&S91425[28], 5}, {NULL}};
const Segment SGI356014[] = {{&S91425[104], 5}, {NULL}};
const Segment SGI356015[] = {{&S99353[21], 5}, {NULL}};
const Segment SGI356016[] = {{&S99353[0], 14}, {NULL}};
const Segment SGI356017[] = {{&S85932[19], 14}, {NULL}};
const Segment SGI356018[] = {{&S85932[47], 3}, {NULL}};
const Segment SGI356019[] = {{&S99399[5], 3}, {NULL}};
const Segment SGI356020[] = {{&S99399[167], 3}, {NULL}};
const Segment SGI356021[] = {{&S99431[5], 3}, {&S99431[5], 3}, {NULL}};
const Segment SGI356022[] = {{&S99431[109], 3}, {NULL}};
const Segment SGI356023[] = {{&S99496[5], 3}, {&S99496[5], 3}, {NULL}};
const Segment SGI356024[] = {{&S99496[9], 3}, {NULL}};
const Segment SGI356025[] = {{&S99496[127], 3}, {NULL}};
const Segment SGI356026[] = {{&S99573[5], 15}, {&S99573[78], 15}, {NULL}};
const Segment SGI356027[] = {{&S99573[104], 3}, {NULL}};
const Segment SGI356028[] = {{&S99574[5], 5}, {NULL}};
const Segment SGI356029[] = {{&S99574[21], 3}, {NULL}};
const Segment SGI356030[] = {{&S99575[5], 4}, {NULL}};
const Segment SGI356031[] = {{&S99597[5], 5}, {&S99597[68], 5}, {NULL}};
const Segment SGI356032[] = {{&S99597[84], 3}, {NULL}};
const Segment SGI356033[] = {{&S99644[5], 5}, {NULL}};
const Segment SGI356034[] = {{&S99644[21], 3}, {NULL}};
const Segment SGI356035[] = {{&S99651[5], 5}, {&S99651[68], 5}, {NULL}};
const Segment SGI356036[] = {{&S99651[84], 3}, {NULL}};
const Segment SGI356037[] = {{&S99670[25], 7}, {NULL}};
const Segment SGI356038[] = {{&S99670[0], 18}, {NULL}};
const Segment SGI356039[] = {{&S99679[35], 3}, {NULL}};
const Segment SGI356040[] = {{&S99679[89], 3}, {NULL}};
const Segment SGI356041[] = {{&S99680[5], 5}, {NULL}};
const Segment SGI356042[] = {{&S99680[21], 3}, {NULL}};
const Segment SGI356043[] = {{&S99681[5], 5}, {&S99681[97], 5}, {NULL}};
const Segment SGI356044[] = {{&S99681[153], 3}, {NULL}};
const Segment SGI356045[] = {{&S91663[28], 5}, {NULL}};
const Segment SGI356046[] = {{&S91663[104], 5}, {NULL}};
const Segment SGI356047[] = {{&S91664[28], 7}, {NULL}};
const Segment SGI356048[] = {{&S91664[106], 5}, {NULL}};
const Segment SGI356049[] = {{&S99686[5], 3}, {&S99686[5], 3}, {NULL}};
const Segment SGI356050[] = {{&S99686[80], 3}, {NULL}};
const Segment SGI356051[] = {{&S72991[6], 4}, {NULL}};
const Segment SGI356052[] = {{&S72991[58], 5}, {NULL}};
const Segment SGI356053[] = {{&S72991[12], 14}, {NULL}};
const Segment SGI356054[] = {{&S72992[6], 4}, {NULL}};
const Segment SGI356055[] = {{&S72992[58], 5}, {NULL}};
const Segment SGI356056[] = {{&S72992[12], 14}, {NULL}};
const Segment SGI356057[] = {{&S99715[5], 13}, {&S99715[112], 13}, {NULL}};
const Segment SGI356058[] = {{&S99715[19], 3}, {NULL}};
const Segment SGI356059[] = {{&S99715[146], 3}, {NULL}};
const Segment SGI356060[] = {{&S99724[5], 5}, {&S99724[104], 5}, {NULL}};
const Segment SGI356061[] = {{&S99724[11], 3}, {NULL}};
const Segment SGI356062[] = {{&S99724[130], 3}, {NULL}};
const Segment SGI356063[] = {{&S99759[5], 7}, {&S99759[5], 7}, {NULL}};
const Segment SGI356064[] = {{&S99759[117], 3}, {NULL}};
const Segment SGI356065[] = {{&S99760[35], 4}, {NULL}};
const Segment SGI356066[] = {{&S99760[90], 3}, {NULL}};
const Segment SGI356067[] = {{&S99761[5], 4}, {&S99761[5], 4}, {NULL}};
const Segment SGI356068[] = {{&S99761[111], 3}, {NULL}};
const Segment SGI356069[] = {{&S99768[35], 3}, {NULL}};
const Segment SGI356070[] = {{&S99768[89], 3}, {NULL}};
const Segment SGI356071[] = {{&S99769[5], 7}, {&S99769[5], 7}, {NULL}};
const Segment SGI356072[] = {{&S99769[157], 3}, {NULL}};
const Segment SGI356073[] = {{&S99775[35], 5}, {NULL}};
const Segment SGI356074[] = {{&S99775[91], 3}, {NULL}};
const Segment SGI356075[] = {{&S86315[5], 4}, {NULL}};
const Segment SGI356076[] = {{&S86315[49], 3}, {NULL}};
const Segment SGI356077[] = {{&S86315[103], 3}, {NULL}};
const Segment SGI356078[] = {{&S99785[35], 7}, {NULL}};
const Segment SGI356079[] = {{&S99785[93], 3}, {NULL}};
const Segment SGI356080[] = {{&S99830[42], 4}, {NULL}};
const Segment SGI356081[] = {{&S99830[6], 3}, {NULL}};
const Segment SGI356082[] = {{&S99830[67], 3}, {NULL}};
const Segment SGI356083[] = {{&S99841[42], 6}, {NULL}};
const Segment SGI356084[] = {{&S99841[6], 3}, {NULL}};
const Segment SGI356085[] = {{&S99841[69], 3}, {NULL}};
const Segment SGI356086[] = {{&S99851[23], 5}, {NULL}};
const Segment SGI356087[] = {{&S99851[0], 16}, {NULL}};
const Segment SGI356088[] = {{&S99855[5], 5}, {NULL}};
const Segment SGI356089[] = {{&S99855[236], 3}, {NULL}};
const Segment SGI356090[] = {{&S86419[5], 5}, {NULL}};
const Segment SGI356091[] = {{&S86419[33], 3}, {NULL}};
const Segment SGI356092[] = {{&S86419[87], 3}, {NULL}};
const Segment SGI356093[] = {{&S86420[5], 3}, {NULL}};
const Segment SGI356094[] = {{&S86420[32], 3}, {NULL}};
const Segment SGI356095[] = {{&S86420[86], 3}, {NULL}};
const Segment SGI356096[] = {{&S99937[41], 4}, {NULL}};
const Segment SGI356097[] = {{&S99937[6], 3}, {NULL}};
const Segment SGI356098[] = {{&S99937[66], 3}, {NULL}};
const Segment SGI356099[] = {{&S99939[21], 11}, {NULL}};
const Segment SGI356100[] = {{&S99939[0], 14}, {NULL}};
const Segment SGI356101[] = {{&S99976[42], 4}, {NULL}};
const Segment SGI356102[] = {{&S99976[6], 3}, {NULL}};
const Segment SGI356103[] = {{&S99976[67], 3}, {NULL}};
const Segment SGI356104[] = {{&S99986[21], 3}, {NULL}};
const Segment SGI356105[] = {{&S99986[0], 14}, {NULL}};
const Segment SGI356106[] = {{&S99992[42], 5}, {NULL}};
const Segment SGI356107[] = {{&S99992[6], 3}, {NULL}};
const Segment SGI356108[] = {{&S99992[68], 3}, {NULL}};
const Segment SGI356109[] = {{&S100009[5], 5}, {NULL}};
const Segment SGI356110[] = {{&S100009[169], 3}, {NULL}};
const Segment SGI356111[] = {{&S86547[5], 3}, {NULL}};
const Segment SGI356112[] = {{&S86547[31], 3}, {NULL}};
const Segment SGI356113[] = {{&S86547[85], 3}, {NULL}};
const Segment SGI356114[] = {{&S100023[5], 7}, {NULL}};
const Segment SGI356115[] = {{&S100023[171], 3}, {NULL}};
const Segment SGI356116[] = {{&S91904[28], 5}, {NULL}};
const Segment SGI356117[] = {{&S91904[104], 5}, {NULL}};
const Segment SGI356118[] = {{&S100042[5], 3}, {&S100042[5], 3}, {NULL}};
const Segment SGI356119[] = {{&S100042[9], 3}, {NULL}};
const Segment SGI356120[] = {{&S100042[126], 3}, {NULL}};
const Segment SGI356121[] = {{&S100045[5], 3}, {&S100045[5], 3}, {NULL}};
const Segment SGI356122[] = {{&S100045[149], 3}, {NULL}};
const Segment SGI356123[] = {{&S100063[43], 4}, {NULL}};
const Segment SGI356124[] = {{&S100063[6], 3}, {NULL}};
const Segment SGI356125[] = {{&S100063[68], 3}, {NULL}};
const Segment SGI356126[] = {{&S73339[6], 4}, {NULL}};
const Segment SGI356127[] = {{&S73339[58], 5}, {NULL}};
const Segment SGI356128[] = {{&S73339[12], 14}, {NULL}};
const Segment SGI356129[] = {{&S100099[5], 3}, {&S100099[5], 3}, {NULL}};
const Segment SGI356130[] = {{&S100099[9], 3}, {NULL}};
const Segment SGI356131[] = {{&S100099[127], 3}, {NULL}};
const Segment SGI356132[] = {{&S91972[28], 4}, {NULL}};
const Segment SGI356133[] = {{&S91972[103], 5}, {NULL}};
const Segment SGI356134[] = {{&S76256[74], 4}, {NULL}};
const Segment SGI356135[] = {{&S76257[132], 4}, {NULL}};
const Segment SGI356136[] = {{&S76257[41], 10}, {NULL}};
const Segment SGI356137[] = {{&S86688[5], 4}, {NULL}};
const Segment SGI356138[] = {{&S86688[49], 3}, {NULL}};
const Segment SGI356139[] = {{&S86688[103], 3}, {NULL}};
const Segment SGI356140[] = {{&S73466[6], 4}, {NULL}};
const Segment SGI356141[] = {{&S73466[77], 5}, {NULL}};
const Segment SGI356142[] = {{&S73466[12], 9}, {NULL}};
const Segment SGI356143[] = {{&S73466[29], 13}, {NULL}};
const Segment SGI356144[] = {{&S100126[5], 10}, {NULL}};
const Segment SGI356145[] = {{&S100148[5], 5}, {NULL}};
const Segment SGI356146[] = {{&S100148[21], 3}, {NULL}};
const Segment SGI356147[] = {{&S73513[6], 4}, {NULL}};
const Segment SGI356148[] = {{&S73513[12], 11}, {&S73513[41], 65}, {NULL}};
const Segment SGI356149[] = {{&S100171[5], 5}, {NULL}};
const Segment SGI356150[] = {{&S100171[21], 3}, {NULL}};
const Segment SGI356151[] = {{&S100194[5], 4}, {&S100194[5], 4}, {NULL}};
const Segment SGI356152[] = {{&S100194[82], 3}, {NULL}};
const Segment SGI356153[] = {{&S100199[5], 5}, {NULL}};
const Segment SGI356154[] = {{&S100199[21], 3}, {NULL}};
const Segment SGI356155[] = {{&S100215[35], 14}, {NULL}};
const Segment SGI356156[] = {{&S100215[61], 3}, {NULL}};
const Segment SGI356157[] = {{&S100226[5], 4}, {&S100226[5], 4}, {NULL}};
const Segment SGI356158[] = {{&S100226[111], 3}, {NULL}};
const Segment SGI356159[] = {{&S100229[5], 5}, {NULL}};
const Segment SGI356160[] = {{&S100229[21], 3}, {NULL}};
const Segment SGI356161[] = {{&S100234[5], 3}, {NULL}};
const Segment SGI356162[] = {{&S100234[19], 3}, {NULL}};
const Segment SGI356163[] = {{&S100235[5], 5}, {NULL}};
const Segment SGI356164[] = {{&S100235[21], 3}, {NULL}};
const Segment SGI356165[] = {{&S100237[5], 5}, {&S100237[68], 5}, {NULL}};
const Segment SGI356166[] = {{&S100237[84], 3}, {NULL}};
const Segment SGI356167[] = {{&S100239[35], 7}, {NULL}};
const Segment SGI356168[] = {{&S100239[54], 3}, {NULL}};
const Segment SGI356169[] = {{&S73606[6], 1}, {NULL}};
const Segment SGI356170[] = {{&S73606[9], 22}, {NULL}};
const Segment SGI356171[] = {{&S73606[9], 14}, {NULL}};
const Segment SGI356172[] = {{&S100244[35], 4}, {NULL}};
const Segment SGI356173[] = {{&S100244[51], 3}, {NULL}};
const Segment SGI356174[] = {{&S73607[6], 4}, {NULL}};
const Segment SGI356175[] = {{&S73607[12], 14}, {NULL}};
const Segment SGI356176[] = {{&S73608[6], 4}, {NULL}};
const Segment SGI356177[] = {{&S73608[12], 14}, {NULL}};
const Segment SGI356178[] = {{&S73609[6], 3}, {NULL}};
const Segment SGI356179[] = {{&S73609[11], 14}, {NULL}};
const Segment SGI356180[] = {{&S100251[5], 7}, {NULL}};
const Segment SGI356181[] = {{&S100251[23], 3}, {NULL}};
const Segment SGI356182[] = {{&S100252[5], 3}, {&S100252[5], 3}, {NULL}};
const Segment SGI356183[] = {{&S100252[80], 3}, {NULL}};
const Segment SGI356184[] = {{&S100260[5], 3}, {&S100260[5], 3}, {NULL}};
const Segment SGI356185[] = {{&S100260[109], 3}, {NULL}};
const Segment SGI356186[] = {{&S100269[5], 7}, {&S100269[5], 7}, {NULL}};
const Segment SGI356187[] = {{&S100269[117], 3}, {NULL}};
const Segment SGI356188[] = {{&S100285[5], 4}, {&S100285[5], 4}, {NULL}};
const Segment SGI356189[] = {{&S100285[111], 3}, {NULL}};
const Segment SGI356190[] = {{&S92268[28], 3}, {NULL}};
const Segment SGI356191[] = {{&S92268[102], 5}, {NULL}};
const Segment SGI356192[] = {{&S100305[25], 3}, {NULL}};
const Segment SGI356193[] = {{&S100305[0], 18}, {NULL}};
const Segment SGI356194[] = {{&S100316[5], 4}, {NULL}};
const Segment SGI356195[] = {{&S100316[20], 3}, {NULL}};
const Segment SGI356196[] = {{&S92288[28], 5}, {NULL}};
const Segment SGI356197[] = {{&S92288[104], 5}, {NULL}};
const Segment SGI356198[] = {{&S100325[23], 3}, {NULL}};
const Segment SGI356199[] = {{&S100325[0], 16}, {NULL}};
const Segment SGI356200[] = {{&S100331[21], 5}, {NULL}};
const Segment SGI356201[] = {{&S100331[0], 14}, {NULL}};
const Segment SGI356202[] = {{&S100342[35], 4}, {NULL}};
const Segment SGI356203[] = {{&S100342[90], 3}, {NULL}};
const Segment SGI356204[] = {{&S100376[5], 5}, {&S100376[105], 5}, {NULL}};
const Segment SGI356205[] = {{&S100376[11], 3}, {NULL}};
const Segment SGI356206[] = {{&S100376[131], 3}, {NULL}};
const Segment SGI356207[] = {{&S100402[5], 4}, {NULL}};
const Segment SGI356208[] = {{&S100402[235], 3}, {NULL}};
const Segment SGI356209[] = {{&S100404[5], 5}, {&S100404[97], 5}, {NULL}};
const Segment SGI356210[] = {{&S73783[6], 4}, {NULL}};
const Segment SGI356211[] = {{&S73783[82], 5}, {NULL}};
const Segment SGI356212[] = {{&S73783[12], 11}, {NULL}};
const Segment SGI356213[] = {{&S100404[17], 3}, {NULL}};
const Segment SGI356214[] = {{&S100405[21], 5}, {NULL}};
const Segment SGI356215[] = {{&S100405[0], 14}, {NULL}};
const Segment SGI356216[] = {{&S73784[12], 13}, {NULL}};
const Segment SGI356217[] = {{&S100406[21], 5}, {NULL}};
const Segment SGI356218[] = {{&S100406[0], 14}, {NULL}};
const Segment SGI356219[] = {{&S73785[12], 13}, {NULL}};
const Segment SGI356220[] = {{&S100410[5], 4}, {&S100410[5], 4}, {NULL}};
const Segment SGI356221[] = {{&S100410[151], 3}, {NULL}};
const Segment SGI356222[] = {{&S100429[43], 4}, {NULL}};
const Segment SGI356223[] = {{&S100429[6], 3}, {NULL}};
const Segment SGI356224[] = {{&S100429[68], 3}, {NULL}};
const Segment SGI356225[] = {{&S100461[35], 5}, {NULL}};
const Segment SGI356226[] = {{&S100461[91], 3}, {NULL}};
const Segment SGI356227[] = {{&S100478[5], 7}, {NULL}};
const Segment SGI356228[] = {{&S100478[171], 3}, {NULL}};
const Segment SGI356229[] = {{&S100510[5], 3}, {&S100510[5], 3}, {NULL}};
const Segment SGI356230[] = {{&S100510[80], 3}, {NULL}};
const Segment SGI356231[] = {{&S92499[28], 5}, {NULL}};
const Segment SGI356232[] = {{&S92499[104], 5}, {NULL}};
const Segment SGI356233[] = {{&S100541[41], 3}, {NULL}};
const Segment SGI356234[] = {{&S100541[6], 3}, {NULL}};
const Segment SGI356235[] = {{&S100541[65], 3}, {NULL}};
const Segment SGI356236[] = {{&S87187[5], 4}, {NULL}};
const Segment SGI356237[] = {{&S87187[33], 3}, {NULL}};
const Segment SGI356238[] = {{&S87215[19], 17}, {NULL}};
const Segment SGI356239[] = {{&S87215[50], 3}, {NULL}};
const Segment SGI356240[] = {{&S87216[19], 20}, {NULL}};
const Segment SGI356241[] = {{&S87216[53], 3}, {NULL}};
const Segment SGI356242[] = {{&S87217[19], 17}, {NULL}};
const Segment SGI356243[] = {{&S87217[50], 3}, {NULL}};
const Segment SGI356244[] = {{&S100592[42], 7}, {NULL}};
const Segment SGI356245[] = {{&S100592[6], 3}, {NULL}};
const Segment SGI356246[] = {{&S100592[70], 3}, {NULL}};
const Segment SGI356247[] = {{&S100612[21], 5}, {NULL}};
const Segment SGI356248[] = {{&S100612[0], 14}, {NULL}};
const Segment SGI356249[] = {{&S100647[43], 7}, {NULL}};
const Segment SGI356250[] = {{&S100647[6], 3}, {NULL}};
const Segment SGI356251[] = {{&S100647[71], 3}, {NULL}};
const Segment SGI356252[] = {{&S100665[5], 7}, {&S100665[5], 7}, {NULL}};
const Segment SGI356253[] = {{&S100665[71], 3}, {NULL}};
const Segment SGI356254[] = {{&S100665[135], 3}, {NULL}};
const Segment SGI356255[] = {{&S87316[19], 15}, {NULL}};
const Segment SGI356256[] = {{&S87316[48], 3}, {NULL}};
const Segment SGI356257[] = {{&S92691[28], 7}, {NULL}};
const Segment SGI356258[] = {{&S92691[106], 5}, {NULL}};
const Segment SGI356259[] = {{&S92706[28], 5}, {NULL}};
const Segment SGI356260[] = {{&S92706[104], 5}, {NULL}};
const Segment SGI356261[] = {{&S100714[43], 4}, {NULL}};
const Segment SGI356262[] = {{&S100714[6], 3}, {NULL}};
const Segment SGI356263[] = {{&S100714[68], 3}, {NULL}};
const Segment SGI356264[] = {{&S100748[5], 4}, {NULL}};
const Segment SGI356265[] = {{&S100748[20], 3}, {NULL}};
const Segment SGI356266[] = {{&S100749[5], 5}, {NULL}};
const Segment SGI356267[] = {{&S100749[21], 3}, {NULL}};
const Segment SGI356268[] = {{&S100750[5], 4}, {NULL}};
const Segment SGI356269[] = {{&S100750[20], 3}, {NULL}};
const Segment SGI356270[] = {{&S100773[5], 5}, {NULL}};
const Segment SGI356271[] = {{&S100773[69], 4}, {NULL}};
const Segment SGI356272[] = {{&S100773[96], 26}, {NULL}};
const Segment SGI356273[] = {{&S100774[5], 7}, {&S100774[5], 7}, {NULL}};
const Segment SGI356274[] = {{&S100774[88], 3}, {NULL}};
const Segment SGI356275[] = {{&S87430[5], 3}, {NULL}};
const Segment SGI356276[] = {{&S87430[31], 3}, {NULL}};
const Segment SGI356277[] = {{&S87430[80], 3}, {NULL}};
const Segment SGI356278[] = {{&S71491[74], 5}, {NULL}};
const Segment SGI356279[] = {{&S71491[12], 5}, {NULL}};
const Segment SGI356280[] = {{&S100815[5], 5}, {NULL}};
const Segment SGI356281[] = {{&S100815[21], 3}, {NULL}};
const Segment SGI356282[] = {{&S100825[5], 7}, {NULL}};
const Segment SGI356283[] = {{&S100825[237], 3}, {NULL}};
const Segment SGI356284[] = {{&S100827[5], 7}, {NULL}};
const Segment SGI356285[] = {{&S100827[238], 3}, {NULL}};
const Segment SGI356286[] = {{&S100830[41], 3}, {NULL}};
const Segment SGI356287[] = {{&S100830[6], 3}, {NULL}};
const Segment SGI356288[] = {{&S100830[65], 3}, {NULL}};
const Segment SGI356289[] = {{&S100837[5], 3}, {NULL}};
const Segment SGI356290[] = {{&S100837[234], 3}, {NULL}};
const Segment SGI356291[] = {{&S100838[25], 4}, {NULL}};
const Segment SGI356292[] = {{&S100838[0], 18}, {NULL}};
const Segment SGI356293[] = {{&S100845[5], 7}, {&S100845[5], 7}, {NULL}};
const Segment SGI356294[] = {{&S100845[13], 3}, {NULL}};
const Segment SGI356295[] = {{&S100845[134], 3}, {NULL}};
const Segment SGI356296[] = {{&S74275[6], 4}, {NULL}};
const Segment SGI356297[] = {{&S74275[12], 10}, {NULL}};
const Segment SGI356298[] = {{&S74276[6], 4}, {NULL}};
const Segment SGI356299[] = {{&S74276[12], 10}, {NULL}};
const Segment SGI356300[] = {{&S74277[6], 4}, {NULL}};
const Segment SGI356301[] = {{&S74277[12], 10}, {NULL}};
const Segment SGI356302[] = {{&S74278[6], 4}, {NULL}};
const Segment SGI356303[] = {{&S74278[12], 10}, {NULL}};
const Segment SGI356304[] = {{&S100867[5], 3}, {&S100867[5], 3}, {NULL}};
const Segment SGI356305[] = {{&S100867[9], 3}, {NULL}};
const Segment SGI356306[] = {{&S100867[126], 3}, {NULL}};
const Segment SGI356307[] = {{&S100874[5], 5}, {NULL}};
const Segment SGI356308[] = {{&S100874[21], 3}, {NULL}};
const Segment SGI356309[] = {{&S100877[5], 5}, {NULL}};
const Segment SGI356310[] = {{&S100877[21], 3}, {NULL}};
const Segment SGI356311[] = {{&S87539[5], 5}, {NULL}};
const Segment SGI356312[] = {{&S87539[11], 3}, {NULL}};
const Segment SGI356313[] = {{&S100883[34], 3}, {NULL}};
const Segment SGI356314[] = {{&S100883[48], 3}, {NULL}};
const Segment SGI356315[] = {{&S92908[28], 5}, {NULL}};
const Segment SGI356316[] = {{&S92908[120], 5}, {NULL}};
const Segment SGI356317[] = {{&S100915[5], 7}, {&S100915[5], 7}, {NULL}};
const Segment SGI356318[] = {{&S87577[19], 23}, {NULL}};
const Segment SGI356319[] = {{&S87577[56], 3}, {NULL}};
const Segment SGI356320[] = {{&S100915[71], 3}, {NULL}};
const Segment SGI356321[] = {{&S100915[134], 3}, {NULL}};
const Segment SGI356322[] = {{&S87578[19], 23}, {NULL}};
const Segment SGI356323[] = {{&S87578[56], 3}, {NULL}};
const Segment SGI356324[] = {{&S87579[19], 23}, {NULL}};
const Segment SGI356325[] = {{&S87579[56], 3}, {NULL}};
const Segment SGI356326[] = {{&S74343[6], 4}, {NULL}};
const Segment SGI356327[] = {{&S74343[12], 3}, {NULL}};
const Segment SGI356328[] = {{&S100940[5], 5}, {NULL}};
const Segment SGI356329[] = {{&S100940[21], 3}, {NULL}};
const Segment SGI356330[] = {{&S100943[5], 7}, {NULL}};
const Segment SGI356331[] = {{&S100943[171], 3}, {NULL}};
const Segment SGI356332[] = {{&S100952[5], 10}, {NULL}};
const Segment SGI356333[] = {{&S100978[5], 4}, {NULL}};
const Segment SGI356334[] = {{&S100978[20], 3}, {NULL}};
const Segment SGI356335[] = {{&S100982[5], 16}, {NULL}};
const Segment SGI356336[] = {{&S100982[22], 3}, {NULL}};
const Segment SGI356337[] = {{&S100985[5], 5}, {NULL}};
const Segment SGI356338[] = {{&S100985[21], 3}, {NULL}};
const Segment SGI356339[] = {{&S87661[5], 10}, {&S87661[5], 10}, {NULL}};
const Segment SGI356340[] = {{&S87661[49], 3}, {NULL}};
const Segment SGI356341[] = {{&S87662[5], 12}, {NULL}};
const Segment SGI356342[] = {{&S87662[41], 3}, {NULL}};
const Segment SGI356343[] = {{&S87677[19], 14}, {NULL}};
const Segment SGI356344[] = {{&S87677[47], 3}, {NULL}};
const Segment SGI356345[] = {{&S101037[5], 5}, {NULL}};
const Segment SGI356346[] = {{&S101037[21], 3}, {NULL}};
const Segment SGI356347[] = {{&S101048[5], 13}, {&S101048[76], 13}, {NULL}};
const Segment SGI356348[] = {{&S101048[100], 3}, {NULL}};
const Segment SGI356349[] = {{&S101050[35], 3}, {NULL}};
const Segment SGI356350[] = {{&S101050[50], 3}, {NULL}};
const Segment SGI356351[] = {{&S101052[5], 7}, {NULL}};
const Segment SGI356352[] = {{&S101052[23], 3}, {NULL}};
const Segment SGI356353[] = {{&S101059[25], 5}, {NULL}};
const Segment SGI356354[] = {{&S101059[0], 18}, {NULL}};
const Segment SGI356355[] = {{&S101077[35], 5}, {NULL}};
const Segment SGI356356[] = {{&S101077[52], 3}, {NULL}};
const Segment SGI356357[] = {{&S74486[6], 4}, {NULL}};
const Segment SGI356358[] = {{&S74486[76], 5}, {NULL}};
const Segment SGI356359[] = {{&S74486[12], 11}, {NULL}};
const Segment SGI356360[] = {{&S74487[6], 4}, {NULL}};
const Segment SGI356361[] = {{&S74487[78], 5}, {NULL}};
const Segment SGI356362[] = {{&S74487[12], 13}, {NULL}};
const Segment SGI356363[] = {{&S74488[6], 4}, {NULL}};
const Segment SGI356364[] = {{&S74488[78], 5}, {NULL}};
const Segment SGI356365[] = {{&S74488[12], 13}, {NULL}};
const Segment SGI356366[] = {{&S101081[5], 5}, {NULL}};
const Segment SGI356367[] = {{&S101081[21], 3}, {NULL}};
const Segment SGI356368[] = {{&S101084[5], 4}, {NULL}};
const Segment SGI356369[] = {{&S101084[20], 3}, {NULL}};
const Segment SGI356370[] = {{&S101089[35], 4}, {NULL}};
const Segment SGI356371[] = {{&S101089[51], 3}, {NULL}};
const Segment SGI356372[] = {{&S101095[5], 7}, {NULL}};
const Segment SGI356373[] = {{&S101095[237], 3}, {NULL}};
const Segment SGI356374[] = {{&S101099[35], 4}, {NULL}};
const Segment SGI356375[] = {{&S101099[90], 3}, {NULL}};
const Segment SGI356376[] = {{&S101131[5], 7}, {NULL}};
const Segment SGI356377[] = {{&S101131[238], 3}, {NULL}};
const Segment SGI356378[] = {{&S87810[21], 14}, {NULL}};
const Segment SGI356379[] = {{&S87810[49], 3}, {NULL}};
const Segment SGI356380[] = {{&S101153[5], 4}, {NULL}};
const Segment SGI356381[] = {{&S101166[5], 9}, {NULL}};
const Segment SGI356382[] = {{&S101168[5], 5}, {NULL}};
const Segment SGI356383[] = {{&S101168[43], 3}, {NULL}};
const Segment SGI356384[] = {{&S101177[5], 4}, {NULL}};
const Segment SGI356385[] = {{&S101177[20], 3}, {NULL}};
const Segment SGI356386[] = {{&S101185[5], 4}, {NULL}};
const Segment SGI356387[] = {{&S101185[20], 3}, {NULL}};
const Segment SGI356388[] = {{&S101187[5], 4}, {NULL}};
const Segment SGI356389[] = {{&S101187[20], 3}, {NULL}};
const Segment SGI356390[] = {{&S101188[5], 3}, {NULL}};
const Segment SGI356391[] = {{&S101188[19], 3}, {NULL}};
const Segment SGI356392[] = {{&S101189[5], 8}, {NULL}};
const Segment SGI356393[] = {{&S101189[24], 3}, {NULL}};
const Segment SGI356394[] = {{&S101191[5], 5}, {NULL}};
const Segment SGI356395[] = {{&S101191[21], 3}, {NULL}};
const Segment SGI356396[] = {{&S101195[5], 5}, {NULL}};
const Segment SGI356397[] = {{&S101195[21], 3}, {NULL}};
const Segment SGI356398[] = {{&S101207[5], 4}, {NULL}};
const Segment SGI356399[] = {{&S101207[15], 3}, {NULL}};
const Segment SGI356400[] = {{&S101209[5], 3}, {&S101209[5], 3}, {NULL}};
const Segment SGI356401[] = {{&S101209[109], 3}, {NULL}};
const Segment SGI356402[] = {{&S74623[6], 4}, {NULL}};
const Segment SGI356403[] = {{&S74623[76], 5}, {NULL}};
const Segment SGI356404[] = {{&S74623[12], 9}, {NULL}};
const Segment SGI356405[] = {{&S74623[29], 13}, {NULL}};
const Segment SGI356406[] = {{&S101224[5], 5}, {NULL}};
const Segment SGI356407[] = {{&S101224[21], 3}, {NULL}};
const Segment SGI356408[] = {{&S101227[5], 5}, {NULL}};
const Segment SGI356409[] = {{&S101227[235], 3}, {NULL}};
const Segment SGI356410[] = {{&S101229[5], 4}, {NULL}};
const Segment SGI356411[] = {{&S101229[234], 3}, {NULL}};
const Segment SGI356412[] = {{&S101235[42], 3}, {NULL}};
const Segment SGI356413[] = {{&S101235[6], 3}, {NULL}};
const Segment SGI356414[] = {{&S101235[66], 3}, {NULL}};
const Segment SGI356415[] = {{&S101263[35], 7}, {NULL}};
const Segment SGI356416[] = {{&S101263[54], 3}, {NULL}};
const Segment SGI356417[] = {{&S74725[6], 4}, {NULL}};
const Segment SGI356418[] = {{&S74725[80], 5}, {NULL}};
const Segment SGI356419[] = {{&S74725[12], 3}, {NULL}};
const Segment SGI356420[] = {{&S74725[40], 10}, {NULL}};
const Segment SGI356421[] = {{&S101319[5], 4}, {NULL}};
const Segment SGI356422[] = {{&S101319[42], 3}, {NULL}};
const Segment SGI356423[] = {{&S101326[5], 5}, {NULL}};
const Segment SGI356424[] = {{&S101326[21], 3}, {NULL}};
const Segment SGI356425[] = {{&S101327[5], 5}, {NULL}};
const Segment SGI356426[] = {{&S101327[21], 3}, {NULL}};
const Segment SGI356427[] = {{&S101328[5], 3}, {NULL}};
const Segment SGI356428[] = {{&S101328[19], 3}, {NULL}};
const Segment SGI356429[] = {{&S101329[5], 15}, {&S101329[78], 15}, {NULL}};
const Segment SGI356430[] = {{&S101329[104], 3}, {NULL}};
const Segment SGI356431[] = {{&S101331[5], 4}, {NULL}};
const Segment SGI356432[] = {{&S93312[28], 7}, {NULL}};
const Segment SGI356433[] = {{&S93312[106], 5}, {NULL}};
const Segment SGI356434[] = {{&S101371[35], 3}, {NULL}};
const Segment SGI356435[] = {{&S101371[50], 3}, {NULL}};
const Segment SGI356436[] = {{&S101372[35], 3}, {NULL}};
const Segment SGI356437[] = {{&S101372[50], 3}, {NULL}};
const Segment SGI356438[] = {{&S93350[28], 7}, {NULL}};
const Segment SGI356439[] = {{&S93350[106], 5}, {NULL}};
const Segment SGI356440[] = {{&S101409[25], 3}, {NULL}};
const Segment SGI356441[] = {{&S101409[0], 18}, {NULL}};
const Segment SGI356442[] = {{&S101413[5], 5}, {NULL}};
const Segment SGI356443[] = {{&S101413[235], 3}, {NULL}};
const Segment SGI356444[] = {{&S77130[15], 4}, {NULL}};
const Segment SGI356445[] = {{&S77130[21], 15}, {NULL}};
const Segment SGI356446[] = {{&S101418[5], 4}, {&S101418[5], 4}, {NULL}};
const Segment SGI356447[] = {{&S101418[10], 3}, {NULL}};
const Segment SGI356448[] = {{&S101418[129], 3}, {NULL}};
const Segment SGI356449[] = {{&S101468[5], 5}, {NULL}};
const Segment SGI356450[] = {{&S101468[21], 3}, {NULL}};
const Segment SGI356451[] = {{&S101474[5], 7}, {&S101474[5], 7}, {NULL}};
const Segment SGI356452[] = {{&S101474[88], 3}, {NULL}};
const Segment SGI356453[] = {{&S101477[5], 4}, {NULL}};
const Segment SGI356454[] = {{&S101477[15], 3}, {NULL}};
const Segment SGI356455[] = {{&S101478[5], 7}, {&S101478[5], 7}, {NULL}};
const Segment SGI356456[] = {{&S101478[88], 3}, {NULL}};
const Segment SGI356457[] = {{&S101482[5], 3}, {NULL}};
const Segment SGI356458[] = {{&S101489[5], 5}, {NULL}};
const Segment SGI356459[] = {{&S101489[16], 3}, {NULL}};
const Segment SGI356460[] = {{&S101508[25], 5}, {NULL}};
const Segment SGI356461[] = {{&S101508[0], 18}, {NULL}};
const Segment SGI356462[] = {{&S93473[28], 4}, {NULL}};
const Segment SGI356463[] = {{&S93473[103], 5}, {NULL}};
const Segment SGI356464[] = {{&S101524[23], 4}, {NULL}};
const Segment SGI356465[] = {{&S101524[0], 16}, {NULL}};
const Segment SGI356466[] = {{&S101540[35], 4}, {NULL}};
const Segment SGI356467[] = {{&S101540[51], 3}, {NULL}};
const Segment SGI356468[] = {{&S101541[35], 3}, {NULL}};
const Segment SGI356469[] = {{&S101541[50], 3}, {NULL}};
const Segment SGI356470[] = {{&S88212[21], 13}, {NULL}};
const Segment SGI356471[] = {{&S88212[48], 3}, {NULL}};
const Segment SGI356472[] = {{&S74842[6], 4}, {NULL}};
const Segment SGI356473[] = {{&S74842[84], 5}, {NULL}};
const Segment SGI356474[] = {{&S74842[12], 13}, {NULL}};
const Segment SGI356475[] = {{&S74843[6], 4}, {NULL}};
const Segment SGI356476[] = {{&S74843[84], 5}, {NULL}};
const Segment SGI356477[] = {{&S74843[12], 13}, {NULL}};
const Segment SGI356478[] = {{&S74855[6], 4}, {NULL}};
const Segment SGI356479[] = {{&S74855[12], 9}, {NULL}};
const Segment SGI356480[] = {{&S74857[6], 3}, {NULL}};
const Segment SGI356481[] = {{&S74857[11], 9}, {NULL}};
const Segment SGI356482[] = {{&S74858[6], 3}, {NULL}};
const Segment SGI356483[] = {{&S74858[11], 9}, {NULL}};
const Segment SGI356484[] = {{&S101544[33], 5}, {NULL}};
const Segment SGI356485[] = {{&S101544[50], 3}, {NULL}};
const Segment SGI356486[] = {{&S101546[35], 7}, {NULL}};
const Segment SGI356487[] = {{&S101546[54], 3}, {NULL}};
const Segment SGI356488[] = {{&S101550[5], 5}, {&S101550[97], 5}, {NULL}};
const Segment SGI356489[] = {{&S101550[113], 3}, {NULL}};
const Segment SGI356490[] = {{&S88292[5], 4}, {NULL}};
const Segment SGI356491[] = {{&S88292[47], 3}, {NULL}};
const Segment SGI356492[] = {{&S88292[101], 3}, {NULL}};
const Segment SGI356493[] = {{&S101596[5], 7}, {NULL}};
const Segment SGI356494[] = {{&S101596[237], 3}, {NULL}};
const Segment SGI356495[] = {{&S101600[5], 5}, {&S101600[5], 5}, {NULL}};
const Segment SGI356496[] = {{&S101600[11], 3}, {NULL}};
const Segment SGI356497[] = {{&S101600[131], 3}, {NULL}};
const Segment SGI356498[] = {{&S101601[5], 7}, {&S101601[5], 7}, {NULL}};
const Segment SGI356499[] = {{&S101601[13], 3}, {NULL}};
const Segment SGI356500[] = {{&S101601[135], 3}, {NULL}};
const Segment SGI356501[] = {{&S101641[5], 4}, {&S101641[5], 4}, {NULL}};
const Segment SGI356502[] = {{&S101641[82], 3}, {NULL}};
const Segment SGI356503[] = {{&S101644[5], 5}, {&S101644[68], 5}, {NULL}};
const Segment SGI356504[] = {{&S101644[84], 3}, {NULL}};
const Segment SGI356505[] = {{&S101650[5], 7}, {NULL}};
const Segment SGI356506[] = {{&S75017[6], 4}, {NULL}};
const Segment SGI356507[] = {{&S75017[49], 5}, {NULL}};
const Segment SGI356508[] = {{&S75017[12], 10}, {NULL}};
const Segment SGI356509[] = {{&S101665[5], 3}, {NULL}};
const Segment SGI356510[] = {{&S101665[19], 3}, {NULL}};
const Segment SGI356511[] = {{&S101667[5], 5}, {&S101667[68], 5}, {NULL}};
const Segment SGI356512[] = {{&S101667[84], 3}, {NULL}};
const Segment SGI356513[] = {{&S101668[5], 7}, {&S101668[5], 7}, {NULL}};
const Segment SGI356514[] = {{&S101668[88], 3}, {NULL}};
const Segment SGI356515[] = {{&S101669[5], 4}, {NULL}};
const Segment SGI356516[] = {{&S101669[20], 3}, {NULL}};
const Segment SGI356517[] = {{&S101670[5], 4}, {NULL}};
const Segment SGI356518[] = {{&S101670[20], 3}, {NULL}};
const Segment SGI356519[] = {{&S101675[52], 5}, {NULL}};
const Segment SGI356520[] = {{&S101675[78], 4}, {NULL}};
const Segment SGI356521[] = {{&S101676[52], 5}, {NULL}};
const Segment SGI356522[] = {{&S101676[78], 4}, {NULL}};
const Segment SGI356523[] = {{&S88406[5], 3}, {NULL}};
const Segment SGI356524[] = {{&S88406[9], 3}, {NULL}};
const Segment SGI356525[] = {{&S88407[5], 5}, {NULL}};
const Segment SGI356526[] = {{&S88407[11], 3}, {NULL}};
const Segment SGI356527[] = {{&S101692[5], 5}, {&S101692[68], 5}, {NULL}};
const Segment SGI356528[] = {{&S75055[6], 4}, {NULL}};
const Segment SGI356529[] = {{&S75055[76], 5}, {NULL}};
const Segment SGI356530[] = {{&S101692[84], 3}, {NULL}};
const Segment SGI356531[] = {{&S75055[12], 11}, {NULL}};
const Segment SGI356532[] = {{&S75056[6], 4}, {NULL}};
const Segment SGI356533[] = {{&S75056[78], 5}, {NULL}};
const Segment SGI356534[] = {{&S75056[12], 13}, {NULL}};
const Segment SGI356535[] = {{&S101694[5], 4}, {NULL}};
const Segment SGI356536[] = {{&S101694[20], 3}, {NULL}};
const Segment SGI356537[] = {{&S75057[6], 4}, {NULL}};
const Segment SGI356538[] = {{&S75057[78], 5}, {NULL}};
const Segment SGI356539[] = {{&S75057[12], 13}, {NULL}};
const Segment SGI356540[] = {{&S75058[6], 4}, {NULL}};
const Segment SGI356541[] = {{&S75058[76], 5}, {NULL}};
const Segment SGI356542[] = {{&S75058[12], 11}, {&S75058[87], 4}, {NULL}};
const Segment SGI356543[] = {{&S75059[6], 4}, {NULL}};
const Segment SGI356544[] = {{&S75059[57], 5}, {NULL}};
const Segment SGI356545[] = {{&S75059[12], 14}, {NULL}};
const Segment SGI356546[] = {{&S75060[6], 4}, {NULL}};
const Segment SGI356547[] = {{&S75060[57], 5}, {NULL}};
const Segment SGI356548[] = {{&S75060[12], 14}, {NULL}};
const Segment SGI356549[] = {{&S75061[6], 4}, {NULL}};
const Segment SGI356550[] = {{&S75061[57], 5}, {NULL}};
const Segment SGI356551[] = {{&S75061[12], 14}, {NULL}};
const Segment SGI356552[] = {{&S75062[6], 4}, {NULL}};
const Segment SGI356553[] = {{&S75062[57], 5}, {NULL}};
const Segment SGI356554[] = {{&S75062[12], 14}, {NULL}};
const Segment SGI356555[] = {{&S75063[6], 4}, {NULL}};
const Segment SGI356556[] = {{&S75063[57], 5}, {NULL}};
const Segment SGI356557[] = {{&S75063[12], 14}, {NULL}};
const Segment SGI356558[] = {{&S101705[5], 3}, {NULL}};
const Segment SGI356559[] = {{&S101705[19], 3}, {NULL}};
const Segment SGI356560[] = {{&S88439[19], 25}, {NULL}};
const Segment SGI356561[] = {{&S88439[58], 3}, {NULL}};
const Segment SGI356562[] = {{&S88442[19], 15}, {NULL}};
const Segment SGI356563[] = {{&S88442[48], 3}, {NULL}};
const Segment SGI356564[] = {{&S101718[5], 4}, {NULL}};
const Segment SGI356565[] = {{&S101718[20], 3}, {NULL}};
const Segment SGI356566[] = {{&S101739[5], 5}, {NULL}};
const Segment SGI356567[] = {{&S101739[235], 3}, {NULL}};
const Segment SGI356568[] = {{&S101744[23], 4}, {NULL}};
const Segment SGI356569[] = {{&S93801[28], 5}, {NULL}};
const Segment SGI356570[] = {{&S101744[0], 16}, {NULL}};
const Segment SGI356571[] = {{&S93801[104], 5}, {NULL}};
const Segment SGI356572[] = {{&S101787[42], 5}, {NULL}};
const Segment SGI356573[] = {{&S101787[6], 3}, {NULL}};
const Segment SGI356574[] = {{&S101787[68], 3}, {NULL}};
const Segment SGI356575[] = {{&S101797[5], 5}, {NULL}};
const Segment SGI356576[] = {{&S77598[151], 4}, {NULL}};
const Segment SGI356577[] = {{&S77598[157], 3}, {NULL}};
const Segment SGI356578[] = {{&S77598[179], 3}, {NULL}};
const Segment SGI356579[] = {{&S77598[194], 5}, {NULL}};
const Segment SGI356580[] = {{&S101838[5], 5}, {&S101838[68], 5}, {NULL}};
const Segment SGI356581[] = {{&S101838[84], 3}, {NULL}};
const Segment SGI356582[] = {{&S101848[5], 7}, {&S101848[5], 7}, {NULL}};
const Segment SGI356583[] = {{&S101848[88], 3}, {NULL}};
const Segment SGI356584[] = {{&S101849[5], 4}, {NULL}};
const Segment SGI356585[] = {{&S101849[20], 3}, {NULL}};
const Segment SGI356586[] = {{&S101854[5], 3}, {NULL}};
const Segment SGI356587[] = {{&S101854[14], 3}, {NULL}};
const Segment SGI356588[] = {{&S101855[5], 10}, {NULL}};
const Segment SGI356589[] = {{&S101855[21], 3}, {NULL}};
const Segment SGI356590[] = {{&S75215[12], 13}, {NULL}};
const Segment SGI356591[] = {{&S75216[12], 13}, {NULL}};
const Segment SGI356592[] = {{&S75217[12], 13}, {NULL}};
const Segment SGI356593[] = {{&S75218[12], 13}, {NULL}};
const Segment SGI356594[] = {{&S101875[5], 10}, {NULL}};
const Segment SGI356595[] = {{&S101875[21], 3}, {NULL}};
const Segment SGI356596[] = {{&S75219[6], 4}, {NULL}};
const Segment SGI356597[] = {{&S75219[82], 5}, {NULL}};
const Segment SGI356598[] = {{&S75219[12], 11}, {NULL}};
const Segment SGI356599[] = {{&S75220[12], 13}, {NULL}};
const Segment SGI356600[] = {{&S101881[23], 3}, {NULL}};
const Segment SGI356601[] = {{&S101881[0], 16}, {NULL}};
const Segment SGI356602[] = {{&S101882[35], 3}, {NULL}};
const Segment SGI356603[] = {{&S101882[50], 3}, {NULL}};
const Segment SGI356604[] = {{&S101886[35], 5}, {NULL}};
const Segment SGI356605[] = {{&S101886[52], 3}, {NULL}};
const Segment SGI356606[] = {{&S101955[5], 4}, {NULL}};
const Segment SGI356607[] = {{&S101956[5], 7}, {&S101956[5], 7}, {NULL}};
const Segment SGI356608[] = {{&S101956[88], 3}, {NULL}};
const Segment SGI356609[] = {{&S88732[21], 25}, {NULL}};
const Segment SGI356610[] = {{&S88732[60], 3}, {NULL}};
const Segment SGI356611[] = {{&S101987[5], 4}, {NULL}};
const Segment SGI356612[] = {{&S101987[42], 3}, {NULL}};
const Segment SGI356613[] = {{&S101988[5], 4}, {NULL}};
const Segment SGI356614[] = {{&S75241[6], 4}, {NULL}};
const Segment SGI356615[] = {{&S75241[12], 14}, {NULL}};
const Segment SGI356616[] = {{&S102010[5], 5}, {&S102010[68], 5}, {NULL}};
const Segment SGI356617[] = {{&S102010[84], 3}, {NULL}};
const Segment SGI356618[] = {{&S102018[5], 4}, {NULL}};
const Segment SGI356619[] = {{&S102029[52], 5}, {NULL}};
const Segment SGI356620[] = {{&S102029[80], 4}, {NULL}};
const Segment SGI356621[] = {{&S102033[5], 5}, {NULL}};
const Segment SGI356622[] = {{&S102033[21], 3}, {NULL}};
const Segment SGI356623[] = {{&S102039[5], 11}, {&S102039[5], 11}, {NULL}};
const Segment SGI356624[] = {{&S102039[96], 3}, {NULL}};
const Segment SGI356625[] = {{&S102046[5], 5}, {NULL}};
const Segment SGI356626[] = {{&S102046[21], 3}, {NULL}};
const Segment SGI356627[] = {{&S102047[5], 3}, {NULL}};
const Segment SGI356628[] = {{&S102047[14], 3}, {NULL}};
const Segment SGI356629[] = {{&S102051[5], 3}, {NULL}};
const Segment SGI356630[] = {{&S102051[19], 3}, {NULL}};
const Segment SGI356631[] = {{&S102052[5], 7}, {NULL}};
const Segment SGI356632[] = {{&S102052[23], 3}, {NULL}};
const Segment SGI356633[] = {{&S76026[0], 15}, {NULL}};
const Segment SGI356634[] = {{&S76026[31], 3}, {NULL}};
const Segment SGI356635[] = {{&S102079[35], 3}, {NULL}};
const Segment SGI356636[] = {{&S102079[50], 3}, {NULL}};
const Segment SGI356637[] = {{&S75408[6], 4}, {NULL}};
const Segment SGI356638[] = {{&S75408[87], 5}, {NULL}};
const Segment SGI356639[] = {{&S75408[12], 14}, {NULL}};
const Segment SGI356640[] = {{&S102123[5], 3}, {&S102123[5], 3}, {NULL}};
const Segment SGI356641[] = {{&S102123[9], 3}, {NULL}};
const Segment SGI356642[] = {{&S102123[126], 3}, {NULL}};
const Segment SGI356643[] = {{&S102127[5], 4}, {&S102127[5], 4}, {NULL}};
const Segment SGI356644[] = {{&S102127[10], 3}, {NULL}};
const Segment SGI356645[] = {{&S102127[129], 3}, {NULL}};
const Segment SGI356646[] = {{&S102133[42], 3}, {NULL}};
const Segment SGI356647[] = {{&S102133[6], 3}, {NULL}};
const Segment SGI356648[] = {{&S102133[66], 3}, {NULL}};
const Segment SGI356649[] = {{&S102147[5], 16}, {NULL}};
const Segment SGI356650[] = {{&S102147[22], 3}, {NULL}};
const Segment SGI356651[] = {{&S102171[5], 9}, {NULL}};
const Segment SGI356652[] = {{&S102172[5], 4}, {NULL}};
const Segment SGI356653[] = {{&S102183[5], 5}, {&S102183[68], 5}, {NULL}};
const Segment SGI356654[] = {{&S102183[84], 3}, {NULL}};
const Segment SGI356655[] = {{&S102193[5], 3}, {NULL}};
const Segment SGI356656[] = {{&S102193[19], 3}, {NULL}};
const Segment SGI356657[] = {{&S102205[5], 5}, {NULL}};
const Segment SGI356658[] = {{&S102205[21], 3}, {NULL}};
const Segment SGI356659[] = {{&S102207[5], 3}, {NULL}};
const Segment SGI356660[] = {{&S102207[14], 3}, {NULL}};
const Segment SGI356661[] = {{&S102208[5], 3}, {NULL}};
const Segment SGI356662[] = {{&S102208[19], 3}, {NULL}};
const Segment SGI356663[] = {{&S102210[5], 8}, {NULL}};
const Segment SGI356664[] = {{&S102210[24], 3}, {NULL}};
const Segment SGI356665[] = {{&S102233[5], 15}, {&S102233[78], 15}, {NULL}};
const Segment SGI356666[] = {{&S102233[104], 3}, {NULL}};
const Segment SGI356667[] = {{&S102234[5], 5}, {&S102234[68], 5}, {NULL}};
const Segment SGI356668[] = {{&S102234[84], 3}, {NULL}};
const Segment SGI356669[] = {{&S94387[28], 5}, {NULL}};
const Segment SGI356670[] = {{&S94387[104], 5}, {NULL}};
const Segment SGI356671[] = {{&S102245[5], 10}, {NULL}};
const Segment SGI356672[] = {{&S102245[21], 3}, {NULL}};
const Segment SGI356673[] = {{&S102263[5], 10}, {NULL}};
const Segment SGI356674[] = {{&S102263[21], 3}, {NULL}};
const Segment SGI356675[] = {{&S102265[5], 4}, {&S102265[5], 4}, {NULL}};
const Segment SGI356676[] = {{&S75616[6], 4}, {NULL}};
const Segment SGI356677[] = {{&S75616[76], 5}, {NULL}};
const Segment SGI356678[] = {{&S75616[12], 9}, {NULL}};
const Segment SGI356679[] = {{&S102265[111], 3}, {NULL}};
const Segment SGI356680[] = {{&S75616[29], 13}, {NULL}};
const Segment SGI356681[] = {{&S75617[6], 4}, {NULL}};
const Segment SGI356682[] = {{&S75617[76], 5}, {NULL}};
const Segment SGI356683[] = {{&S75617[12], 9}, {NULL}};
const Segment SGI356684[] = {{&S75617[29], 13}, {NULL}};
const Segment SGI356685[] = {{&S102267[35], 3}, {NULL}};
const Segment SGI356686[] = {{&S102267[50], 3}, {NULL}};
const Segment SGI356687[] = {{&S94427[28], 4}, {NULL}};
const Segment SGI356688[] = {{&S94427[103], 5}, {NULL}};
const Segment SGI356689[] = {{&S89145[21], 15}, {NULL}};
const Segment SGI356690[] = {{&S89145[50], 3}, {NULL}};
const Segment SGI356691[] = {{&S102315[5], 3}, {&S102315[5], 3}, {NULL}};
const Segment SGI356692[] = {{&S102315[9], 3}, {NULL}};
const Segment SGI356693[] = {{&S102315[127], 3}, {NULL}};
const Segment SGI356694[] = {{&S102319[86], 11}, {NULL}};
const Segment SGI356695[] = {{&S102324[5], 4}, {&S102324[5], 4}, {NULL}};
const Segment SGI356696[] = {{&S102324[10], 3}, {NULL}};
const Segment SGI356697[] = {{&S102324[129], 3}, {NULL}};
const Segment SGI356698[] = {{&S102325[5], 7}, {&S102325[5], 7}, {NULL}};
const Segment SGI356699[] = {{&S102325[13], 3}, {NULL}};
const Segment SGI356700[] = {{&S102325[135], 3}, {NULL}};
const Segment SGI356701[] = {{&S102326[5], 4}, {NULL}};
const Segment SGI356702[] = {{&S102326[42], 3}, {NULL}};
const Segment SGI356703[] = {{&S102327[5], 5}, {NULL}};
const Segment SGI356704[] = {{&S102327[43], 3}, {NULL}};
const Segment SGI356705[] = {{&S102334[5], 4}, {NULL}};
const Segment SGI356706[] = {{&S102352[5], 4}, {&S102352[5], 4}, {NULL}};
const Segment SGI356707[] = {{&S102352[82], 3}, {NULL}};
const Segment SGI356708[] = {{&S102362[5], 3}, {NULL}};
const Segment SGI356709[] = {{&S102362[19], 3}, {NULL}};
const Segment SGI356710[] = {{&S102369[5], 3}, {NULL}};
const Segment SGI356711[] = {{&S102369[19], 3}, {NULL}};
const Segment SGI356712[] = {{&S102380[5], 3}, {NULL}};
const Segment SGI356713[] = {{&S102380[19], 3}, {NULL}};
const Segment SGI356714[] = {{&S102392[5], 3}, {NULL}};
const Segment SGI356715[] = {{&S102392[19], 3}, {NULL}};
const Segment SGI356716[] = {{&S102395[5], 3}, {NULL}};
const Segment SGI356717[] = {{&S102395[19], 3}, {NULL}};
const Segment SGI356718[] = {{&S102396[5], 4}, {NULL}};
const Segment SGI356719[] = {{&S102396[20], 3}, {NULL}};
const Segment SGI356720[] = {{&S102405[52], 5}, {NULL}};
const Segment SGI356721[] = {{&S102405[79], 4}, {NULL}};
const Segment SGI356722[] = {{&S102419[5], 3}, {NULL}};
const Segment SGI356723[] = {{&S102419[19], 3}, {NULL}};
const Segment SGI356724[] = {{&S102433[5], 5}, {&S102433[5], 5}, {NULL}};
const Segment SGI356725[] = {{&S102433[84], 3}, {NULL}};
const Segment SGI356726[] = {{&S78231[57], 4}, {NULL}};
const Segment SGI356727[] = {{&S78231[13], 10}, {NULL}};
const Segment SGI356728[] = {{&S102440[5], 5}, {&S102440[5], 5}, {NULL}};
const Segment SGI356729[] = {{&S102440[84], 3}, {NULL}};
const Segment SGI356730[] = {{&S102447[5], 5}, {&S102447[68], 5}, {NULL}};
const Segment SGI356731[] = {{&S102447[84], 3}, {NULL}};
const Segment SGI356732[] = {{&S102457[5], 16}, {NULL}};
const Segment SGI356733[] = {{&S102457[27], 3}, {NULL}};
const Segment SGI356734[] = {{&S102458[5], 17}, {NULL}};
const Segment SGI356735[] = {{&S102458[28], 3}, {NULL}};
const Segment SGI356736[] = {{&S75826[6], 3}, {NULL}};
const Segment SGI356737[] = {{&S75826[11], 9}, {NULL}};
const Segment SGI356738[] = {{&S75827[6], 4}, {NULL}};
const Segment SGI356739[] = {{&S75827[12], 9}, {NULL}};
const Segment SGI356740[] = {{&S75828[6], 4}, {NULL}};
const Segment SGI356741[] = {{&S75828[12], 9}, {NULL}};
const Segment SGI356742[] = {{&S75829[6], 4}, {NULL}};
const Segment SGI356743[] = {{&S75829[12], 9}, {NULL}};
const Segment SGI356744[] = {{&S78280[62], 4}, {NULL}};
const Segment SGI356745[] = {{&S78280[13], 10}, {NULL}};
const Segment SGI356746[] = {{&S102486[23], 7}, {NULL}};
const Segment SGI356747[] = {{&S102486[0], 16}, {NULL}};
const Segment SGI356748[] = {{&S78281[62], 4}, {NULL}};
const Segment SGI356749[] = {{&S78281[13], 10}, {NULL}};
const Segment SGI356750[] = {{&S78282[62], 4}, {NULL}};
const Segment SGI356751[] = {{&S78282[13], 10}, {NULL}};
const Segment SGI356752[] = {{&S78283[43], 4}, {NULL}};
const Segment SGI356753[] = {{&S78283[13], 3}, {NULL}};
const Segment SGI356754[] = {{&S102496[5], 3}, {&S102496[5], 3}, {NULL}};
const Segment SGI356755[] = {{&S102496[109], 3}, {NULL}};
const Segment SGI356756[] = {{&S75864[6], 3}, {NULL}};
const Segment SGI356757[] = {{&S75865[6], 4}, {NULL}};
const Segment SGI356758[] = {{&S102536[42], 3}, {NULL}};
const Segment SGI356759[] = {{&S102536[6], 3}, {NULL}};
const Segment SGI356760[] = {{&S102536[66], 3}, {NULL}};
const Segment SGI356761[] = {{&S102581[5], 4}, {NULL}};
const Segment SGI356762[] = {{&S102592[5], 5}, {NULL}};
const Segment SGI356763[] = {{&S102592[21], 3}, {NULL}};
const Segment SGI356764[] = {{&S102594[5], 3}, {NULL}};
const Segment SGI356765[] = {{&S102594[14], 3}, {NULL}};
const Segment SGI356766[] = {{&S102607[5], 4}, {NULL}};
const Segment SGI356767[] = {{&S102607[15], 3}, {NULL}};
const Segment SGI356768[] = {{&S102610[5], 3}, {NULL}};
const Segment SGI356769[] = {{&S102610[19], 3}, {NULL}};
const Segment SGI356770[] = {{&S102642[5], 5}, {NULL}};
const Segment SGI356771[] = {{&S102642[21], 3}, {NULL}};
const Segment SGI356772[] = {{&S102646[5], 5}, {NULL}};
const Segment SGI356773[] = {{&S102646[16], 3}, {NULL}};
const Segment SGI356774[] = {{&S102662[5], 3}, {&S102662[5], 3}, {NULL}};
const Segment SGI356775[] = {{&S102662[80], 3}, {NULL}};
const Segment SGI356776[] = {{&S102676[35], 5}, {NULL}};
const Segment SGI356777[] = {{&S102676[52], 3}, {NULL}};
const Segment SGI356778[] = {{&S102677[5], 5}, {&S102677[97], 5}, {NULL}};
const Segment SGI356779[] = {{&S102677[113], 3}, {NULL}};
const Segment SGI356780[] = {{&S102683[35], 4}, {NULL}};
const Segment SGI356781[] = {{&S102683[51], 3}, {NULL}};
const Segment SGI356782[] = {{&S102684[23], 5}, {NULL}};
const Segment SGI356783[] = {{&S102684[0], 16}, {NULL}};
const Segment SGI356784[] = {{&S102685[5], 3}, {&S102685[5], 3}, {NULL}};
const Segment SGI356785[] = {{&S102685[109], 3}, {NULL}};
const Segment SGI356786[] = {{&S76031[6], 4}, {NULL}};
const Segment SGI356787[] = {{&S76031[53], 5}, {NULL}};
const Segment SGI356788[] = {{&S76031[12], 10}, {NULL}};
const Segment SGI356789[] = {{&S102719[5], 5}, {&S102719[104], 5}, {NULL}};
const Segment SGI356790[] = {{&S102719[69], 3}, {NULL}};
const Segment SGI356791[] = {{&S102719[130], 3}, {NULL}};
const Segment SGI356792[] = {{&S102721[42], 5}, {NULL}};
const Segment SGI356793[] = {{&S102721[6], 3}, {NULL}};
const Segment SGI356794[] = {{&S102721[68], 3}, {NULL}};
const Segment SGI356795[] = {{&S102742[5], 7}, {&S102742[5], 7}, {NULL}};
const Segment SGI356796[] = {{&S102742[88], 3}, {NULL}};
const Segment SGI356797[] = {{&S102778[5], 4}, {&S102778[5], 4}, {NULL}};
const Segment SGI356798[] = {{&S102778[82], 3}, {NULL}};
const Segment SGI356799[] = {{&S102782[5], 4}, {&S102782[5], 4}, {NULL}};
const Segment SGI356800[] = {{&S102782[82], 3}, {NULL}};
const Segment SGI356801[] = {{&S102783[5], 4}, {&S102783[5], 4}, {NULL}};
const Segment SGI356802[] = {{&S102783[82], 3}, {NULL}};
const Segment SGI356803[] = {{&S76067[6], 4}, {NULL}};
const Segment SGI356804[] = {{&S76067[97], 5}, {NULL}};
const Segment SGI356805[] = {{&S76067[12], 9}, {NULL}};
const Segment SGI356806[] = {{&S76067[29], 13}, {NULL}};
const Segment SGI356807[] = {{&S102810[5], 16}, {NULL}};
const Segment SGI356808[] = {{&S102810[22], 3}, {NULL}};
const Segment SGI356809[] = {{&S102814[5], 3}, {NULL}};
const Segment SGI356810[] = {{&S102814[14], 3}, {NULL}};
const Segment SGI356811[] = {{&S102816[5], 3}, {NULL}};
const Segment SGI356812[] = {{&S102816[19], 3}, {NULL}};
const Segment SGI356813[] = {{&S102819[5], 4}, {NULL}};
const Segment SGI356814[] = {{&S102819[15], 3}, {NULL}};
const Segment SGI356815[] = {{&S102820[5], 5}, {NULL}};
const Segment SGI356816[] = {{&S102820[16], 3}, {NULL}};
const Segment SGI356817[] = {{&S102821[5], 4}, {&S102821[5], 4}, {NULL}};
const Segment SGI356818[] = {{&S102821[82], 3}, {NULL}};
const Segment SGI356819[] = {{&S102843[5], 9}, {NULL}};
const Segment SGI356820[] = {{&S102843[25], 3}, {NULL}};
const Segment SGI356821[] = {{&S102852[25], 5}, {NULL}};
const Segment SGI356822[] = {{&S102852[0], 18}, {NULL}};
const Segment SGI356823[] = {{&S102863[35], 3}, {NULL}};
const Segment SGI356824[] = {{&S102863[50], 3}, {NULL}};
const Segment SGI356825[] = {{&S102865[5], 5}, {NULL}};
const Segment SGI356826[] = {{&S102865[16], 3}, {NULL}};
const Segment SGI356827[] = {{&S102893[5], 11}, {&S102893[5], 11}, {NULL}};
const Segment SGI356828[] = {{&S102893[125], 3}, {NULL}};
const Segment SGI356829[] = {{&S102930[5], 15}, {&S102930[115], 15}, {NULL}};
const Segment SGI356830[] = {{&S102930[21], 3}, {NULL}};
const Segment SGI356831[] = {{&S102930[151], 3}, {NULL}};
const Segment SGI356832[] = {{&S102943[42], 5}, {NULL}};
const Segment SGI356833[] = {{&S102943[6], 3}, {NULL}};
const Segment SGI356834[] = {{&S102943[68], 3}, {NULL}};
const Segment SGI356835[] = {{&S102948[5], 5}, {NULL}};
const Segment SGI356836[] = {{&S102948[169], 3}, {NULL}};
const Segment SGI356837[] = {{&S102958[5], 7}, {&S102958[5], 7}, {NULL}};
const Segment SGI356838[] = {{&S102958[88], 3}, {NULL}};
const Segment SGI356839[] = {{&S102985[52], 4}, {NULL}};
const Segment SGI356840[] = {{&S102985[83], 4}, {NULL}};
const Segment SGI356841[] = {{&S103014[5], 4}, {NULL}};
const Segment SGI356842[] = {{&S103016[5], 4}, {NULL}};
const Segment SGI356843[] = {{&S103016[20], 3}, {NULL}};
const Segment SGI356844[] = {{&S103033[5], 16}, {NULL}};
const Segment SGI356845[] = {{&S103033[22], 3}, {NULL}};
const Segment SGI356846[] = {{&S103036[5], 5}, {NULL}};
const Segment SGI356847[] = {{&S103036[26], 3}, {NULL}};
const Segment SGI356848[] = {{&S103041[5], 4}, {NULL}};
const Segment SGI356849[] = {{&S103041[15], 3}, {NULL}};
const Segment SGI356850[] = {{&S103042[5], 7}, {&S103042[5], 7}, {NULL}};
const Segment SGI356851[] = {{&S103042[88], 3}, {NULL}};
const Segment SGI356852[] = {{&S103043[5], 17}, {NULL}};
const Segment SGI356853[] = {{&S103043[28], 3}, {NULL}};
const Segment SGI356854[] = {{&S103044[5], 16}, {NULL}};
const Segment SGI356855[] = {{&S103044[27], 3}, {NULL}};
const Segment SGI356856[] = {{&S103047[5], 5}, {NULL}};
const Segment SGI356857[] = {{&S103047[21], 3}, {NULL}};
const Segment SGI356858[] = {{&S103048[5], 5}, {NULL}};
const Segment SGI356859[] = {{&S103048[21], 3}, {NULL}};
const Segment SGI356860[] = {{&S103049[5], 4}, {NULL}};
const Segment SGI356861[] = {{&S103049[20], 3}, {NULL}};
const Segment SGI356862[] = {{&S103051[5], 3}, {NULL}};
const Segment SGI356863[] = {{&S103052[5], 5}, {NULL}};
const Segment SGI356864[] = {{&S103068[25], 4}, {NULL}};
const Segment SGI356865[] = {{&S103068[0], 18}, {NULL}};
const Segment SGI356866[] = {{&S78932[57], 4}, {NULL}};
const Segment SGI356867[] = {{&S78932[13], 10}, {NULL}};
const Segment SGI356868[] = {{&S78933[57], 4}, {NULL}};
const Segment SGI356869[] = {{&S78933[13], 10}, {NULL}};
const Segment SGI356870[] = {{&S78934[57], 4}, {NULL}};
const Segment SGI356871[] = {{&S78934[13], 10}, {NULL}};
const Segment SGI356872[] = {{&S78935[57], 4}, {NULL}};
const Segment SGI356873[] = {{&S78935[13], 10}, {NULL}};
const Segment SGI356874[] = {{&S103081[5], 8}, {NULL}};
const Segment SGI356875[] = {{&S103081[19], 3}, {NULL}};
const Segment SGI356876[] = {{&S76374[6], 4}, {NULL}};
const Segment SGI356877[] = {{&S76374[97], 5}, {NULL}};
const Segment SGI356878[] = {{&S76374[12], 9}, {NULL}};
const Segment SGI356879[] = {{&S76374[29], 13}, {NULL}};
const Segment SGI356880[] = {{&S76375[6], 4}, {NULL}};
const Segment SGI356881[] = {{&S76375[57], 5}, {NULL}};
const Segment SGI356882[] = {{&S76375[12], 14}, {NULL}};
const Segment SGI356883[] = {{&S76376[6], 4}, {NULL}};
const Segment SGI356884[] = {{&S76376[57], 5}, {NULL}};
const Segment SGI356885[] = {{&S76376[12], 14}, {NULL}};
const Segment SGI356886[] = {{&S76377[6], 4}, {NULL}};
const Segment SGI356887[] = {{&S76377[57], 5}, {NULL}};
const Segment SGI356888[] = {{&S76377[12], 14}, {NULL}};
const Segment SGI356889[] = {{&S76378[6], 4}, {NULL}};
const Segment SGI356890[] = {{&S76378[57], 5}, {NULL}};
const Segment SGI356891[] = {{&S76378[12], 14}, {NULL}};
const Segment SGI356892[] = {{&S103092[5], 10}, {NULL}};
const Segment SGI356893[] = {{&S103092[21], 3}, {NULL}};
const Segment SGI356894[] = {{&S103093[5], 10}, {NULL}};
const Segment SGI356895[] = {{&S103093[21], 3}, {NULL}};
const Segment SGI356896[] = {{&S103094[5], 5}, {NULL}};
const Segment SGI356897[] = {{&S103094[235], 3}, {NULL}};
const Segment SGI356898[] = {{&S78979[64], 4}, {NULL}};
const Segment SGI356899[] = {{&S78979[13], 10}, {NULL}};
const Segment SGI356900[] = {{&S78982[59], 4}, {NULL}};
const Segment SGI356901[] = {{&S78982[13], 10}, {NULL}};
const Segment SGI356902[] = {{&S103128[5], 4}, {NULL}};
const Segment SGI356903[] = {{&S103128[20], 3}, {NULL}};
const Segment SGI356904[] = {{&S103129[5], 4}, {&S103129[5], 4}, {NULL}};
const Segment SGI356905[] = {{&S103129[10], 3}, {NULL}};
const Segment SGI356906[] = {{&S103129[128], 3}, {NULL}};
const Segment SGI356907[] = {{&S76442[6], 4}, {NULL}};
const Segment SGI356908[] = {{&S76442[98], 5}, {NULL}};
const Segment SGI356909[] = {{&S76442[12], 9}, {NULL}};
const Segment SGI356910[] = {{&S76442[29], 13}, {NULL}};
const Segment SGI356911[] = {{&S76443[6], 4}, {NULL}};
const Segment SGI356912[] = {{&S76443[75], 5}, {NULL}};
const Segment SGI356913[] = {{&S76443[12], 9}, {NULL}};
const Segment SGI356914[] = {{&S76443[29], 13}, {NULL}};
const Segment SGI356915[] = {{&S89895[21], 14}, {NULL}};
const Segment SGI356916[] = {{&S89895[49], 3}, {NULL}};
const Segment SGI356917[] = {{&S103200[5], 7}, {&S103200[5], 7}, {NULL}};
const Segment SGI356918[] = {{&S103200[13], 3}, {NULL}};
const Segment SGI356919[] = {{&S103200[134], 3}, {NULL}};
const Segment SGI356920[] = {{&S103204[42], 4}, {NULL}};
const Segment SGI356921[] = {{&S103204[6], 3}, {NULL}};
const Segment SGI356922[] = {{&S103204[67], 3}, {NULL}};
const Segment SGI356923[] = {{&S103224[5], 3}, {&S103224[5], 3}, {NULL}};
const Segment SGI356924[] = {{&S103224[109], 3}, {NULL}};
const Segment SGI356925[] = {{&S103225[5], 7}, {&S103225[5], 7}, {NULL}};
const Segment SGI356926[] = {{&S103225[88], 3}, {NULL}};
const Segment SGI356927[] = {{&S103258[5], 4}, {&S103258[5], 4}, {NULL}};
const Segment SGI356928[] = {{&S103258[82], 3}, {NULL}};
const Segment SGI356929[] = {{&S103290[5], 4}, {NULL}};
const Segment SGI356930[] = {{&S103311[5], 16}, {NULL}};
const Segment SGI356931[] = {{&S103311[27], 3}, {NULL}};
const Segment SGI356932[] = {{&S103313[5], 4}, {NULL}};
const Segment SGI356933[] = {{&S103313[20], 3}, {NULL}};
const Segment SGI356934[] = {{&S103314[5], 8}, {NULL}};
const Segment SGI356935[] = {{&S103314[24], 3}, {NULL}};
const Segment SGI356936[] = {{&S103316[5], 3}, {NULL}};
const Segment SGI356937[] = {{&S103316[19], 3}, {NULL}};
const Segment SGI356938[] = {{&S103317[5], 4}, {NULL}};
const Segment SGI356939[] = {{&S103317[20], 3}, {NULL}};
const Segment SGI356940[] = {{&S103319[5], 4}, {NULL}};
const Segment SGI356941[] = {{&S103319[20], 3}, {NULL}};
const Segment SGI356942[] = {{&S103348[5], 5}, {NULL}};
const Segment SGI356943[] = {{&S103348[21], 3}, {NULL}};
const Segment SGI356944[] = {{&S103354[5], 4}, {NULL}};
const Segment SGI356945[] = {{&S103354[15], 3}, {NULL}};
const Segment SGI356946[] = {{&S103364[5], 5}, {NULL}};
const Segment SGI356947[] = {{&S103364[21], 3}, {NULL}};
const Segment SGI356948[] = {{&S103367[5], 5}, {NULL}};
const Segment SGI356949[] = {{&S103367[16], 3}, {NULL}};
const Segment SGI356950[] = {{&S103372[25], 5}, {NULL}};
const Segment SGI356951[] = {{&S103372[0], 18}, {NULL}};
const Segment SGI356952[] = {{&S103373[35], 3}, {NULL}};
const Segment SGI356953[] = {{&S103373[50], 3}, {NULL}};
const Segment SGI356954[] = {{&S73859[83], 5}, {NULL}};
const Segment SGI356955[] = {{&S73859[12], 9}, {NULL}};
const Segment SGI356956[] = {{&S103374[5], 10}, {NULL}};
const Segment SGI356957[] = {{&S103374[21], 3}, {NULL}};
const Segment SGI356958[] = {{&S103378[5], 7}, {NULL}};
const Segment SGI356959[] = {{&S103378[23], 3}, {NULL}};
const Segment SGI356960[] = {{&S103379[25], 5}, {NULL}};
const Segment SGI356961[] = {{&S103379[0], 18}, {NULL}};
const Segment SGI356962[] = {{&S103411[5], 5}, {&S103411[97], 5}, {NULL}};
const Segment SGI356963[] = {{&S103411[113], 3}, {NULL}};
const Segment SGI356964[] = {{&S103412[23], 5}, {NULL}};
const Segment SGI356965[] = {{&S103412[0], 16}, {NULL}};
const Segment SGI356966[] = {{&S103413[5], 3}, {&S103413[5], 3}, {NULL}};
const Segment SGI356967[] = {{&S103413[109], 3}, {NULL}};
const Segment SGI356968[] = {{&S95411[28], 7}, {NULL}};
const Segment SGI356969[] = {{&S95411[106], 5}, {NULL}};
const Segment SGI356970[] = {{&S76736[6], 4}, {NULL}};
const Segment SGI356971[] = {{&S76736[90], 5}, {NULL}};
const Segment SGI356972[] = {{&S76736[12], 9}, {NULL}};
const Segment SGI356973[] = {{&S76736[29], 13}, {NULL}};
const Segment SGI356974[] = {{&S95425[28], 3}, {NULL}};
const Segment SGI356975[] = {{&S95425[102], 5}, {NULL}};
const Segment SGI356976[] = {{&S76771[6], 4}, {NULL}};
const Segment SGI356977[] = {{&S76771[57], 5}, {NULL}};
const Segment SGI356978[] = {{&S76771[12], 14}, {NULL}};
const Segment SGI356979[] = {{&S103469[5], 4}, {&S103469[5], 4}, {NULL}};
const Segment SGI356980[] = {{&S103469[10], 3}, {NULL}};
const Segment SGI356981[] = {{&S103469[128], 3}, {NULL}};
const Segment SGI356982[] = {{&S103504[86], 11}, {NULL}};
const Segment SGI356983[] = {{&S74002[204], 5}, {NULL}};
const Segment SGI356984[] = {{&S74002[141], 3}, {NULL}};
const Segment SGI356985[] = {{&S103513[5], 4}, {NULL}};
const Segment SGI356986[] = {{&S103513[235], 3}, {NULL}};
const Segment SGI356987[] = {{&S103514[5], 4}, {&S103514[5], 4}, {NULL}};
const Segment SGI356988[] = {{&S103514[82], 3}, {NULL}};
const Segment SGI356989[] = {{&S103518[5], 5}, {NULL}};
const Segment SGI356990[] = {{&S103562[5], 4}, {NULL}};
const Segment SGI356991[] = {{&S103572[5], 4}, {NULL}};
const Segment SGI356992[] = {{&S103575[5], 3}, {NULL}};
const Segment SGI356993[] = {{&S103575[19], 3}, {NULL}};
const Segment SGI356994[] = {{&S103580[5], 4}, {NULL}};
const Segment SGI356995[] = {{&S103581[5], 5}, {NULL}};
const Segment SGI356996[] = {{&S103583[5], 5}, {NULL}};
const Segment SGI356997[] = {{&S103588[5], 4}, {NULL}};
const Segment SGI356998[] = {{&S103588[15], 3}, {NULL}};
const Segment SGI356999[] = {{&S103592[5], 5}, {NULL}};
const Segment SGI357000[] = {{&S103592[21], 3}, {NULL}};
const Segment SGI357001[] = {{&S95593[28], 3}, {NULL}};
const Segment SGI357002[] = {{&S95593[102], 5}, {NULL}};
const Segment SGI357003[] = {{&S103617[5], 8}, {NULL}};
const Segment SGI357004[] = {{&S103617[24], 3}, {NULL}};
const Segment SGI357005[] = {{&S103623[5], 4}, {NULL}};
const Segment SGI357006[] = {{&S103623[20], 3}, {NULL}};
const Segment SGI357007[] = {{&S103625[5], 5}, {NULL}};
const Segment SGI357008[] = {{&S103625[21], 3}, {NULL}};
const Segment SGI357009[] = {{&S103626[5], 5}, {NULL}};
const Segment SGI357010[] = {{&S103626[21], 3}, {NULL}};
const Segment SGI357011[] = {{&S103628[5], 4}, {NULL}};
const Segment SGI357012[] = {{&S103628[20], 3}, {NULL}};
const Segment SGI357013[] = {{&S103635[52], 5}, {NULL}};
const Segment SGI357014[] = {{&S103635[80], 4}, {NULL}};
const Segment SGI357015[] = {{&S103671[5], 4}, {NULL}};
const Segment SGI357016[] = {{&S103671[20], 3}, {NULL}};
const Segment SGI357017[] = {{&S103680[5], 5}, {NULL}};
const Segment SGI357018[] = {{&S103680[16], 3}, {NULL}};
const Segment SGI357019[] = {{&S103681[5], 4}, {NULL}};
const Segment SGI357020[] = {{&S103681[15], 3}, {NULL}};
const Segment SGI357021[] = {{&S103686[5], 3}, {NULL}};
const Segment SGI357022[] = {{&S103686[14], 3}, {NULL}};
const Segment SGI357023[] = {{&S90404[19], 15}, {NULL}};
const Segment SGI357024[] = {{&S90404[48], 3}, {NULL}};
const Segment SGI357025[] = {{&S103696[5], 5}, {NULL}};
const Segment SGI357026[] = {{&S103696[16], 3}, {NULL}};
const Segment SGI357027[] = {{&S103703[5], 10}, {NULL}};
const Segment SGI357028[] = {{&S103703[21], 3}, {NULL}};
const Segment SGI357029[] = {{&S103722[23], 7}, {NULL}};
const Segment SGI357030[] = {{&S103722[0], 16}, {NULL}};
const Segment SGI357031[] = {{&S103723[35], 3}, {NULL}};
const Segment SGI357032[] = {{&S103723[50], 3}, {NULL}};
const Segment SGI357033[] = {{&S103724[25], 5}, {NULL}};
const Segment SGI357034[] = {{&S103724[0], 18}, {NULL}};
const Segment SGI357035[] = {{&S103726[5], 5}, {&S103726[97], 5}, {NULL}};
const Segment SGI357036[] = {{&S103726[113], 3}, {NULL}};
const Segment SGI357037[] = {{&S103728[5], 10}, {NULL}};
const Segment SGI357038[] = {{&S103728[21], 3}, {NULL}};
const Segment SGI357039[] = {{&S103732[5], 15}, {&S103732[114], 15}, {NULL}};
const Segment SGI357040[] = {{&S103732[21], 3}, {NULL}};
const Segment SGI357041[] = {{&S103732[150], 3}, {NULL}};
const Segment SGI357042[] = {{&S103734[41], 3}, {NULL}};
const Segment SGI357043[] = {{&S103734[6], 3}, {NULL}};
const Segment SGI357044[] = {{&S103734[65], 3}, {NULL}};
const Segment SGI357045[] = {{&S77071[6], 4}, {NULL}};
const Segment SGI357046[] = {{&S77071[77], 5}, {NULL}};
const Segment SGI357047[] = {{&S77071[12], 9}, {NULL}};
const Segment SGI357048[] = {{&S77071[29], 13}, {NULL}};
const Segment SGI357049[] = {{&S77102[6], 4}, {NULL}};
const Segment SGI357050[] = {{&S77102[55], 5}, {NULL}};
const Segment SGI357051[] = {{&S77102[12], 10}, {NULL}};
const Segment SGI357052[] = {{&S103810[23], 5}, {NULL}};
const Segment SGI357053[] = {{&S103810[0], 16}, {NULL}};
const Segment SGI357054[] = {{&S103863[5], 4}, {NULL}};
const Segment SGI357055[] = {{&S103877[5], 5}, {&S103877[68], 5}, {NULL}};
const Segment SGI357056[] = {{&S103877[84], 3}, {NULL}};
const Segment SGI357057[] = {{&S103890[5], 5}, {NULL}};
const Segment SGI357058[] = {{&S103916[5], 3}, {NULL}};
const Segment SGI357059[] = {{&S103916[14], 3}, {NULL}};
const Segment SGI357060[] = {{&S103920[5], 5}, {NULL}};
const Segment SGI357061[] = {{&S103920[16], 3}, {NULL}};
const Segment SGI357062[] = {{&S103921[5], 5}, {NULL}};
const Segment SGI357063[] = {{&S103921[16], 3}, {NULL}};
const Segment SGI357064[] = {{&S103922[5], 4}, {NULL}};
const Segment SGI357065[] = {{&S103922[15], 3}, {NULL}};
const Segment SGI357066[] = {{&S103925[5], 3}, {NULL}};
const Segment SGI357067[] = {{&S103925[14], 3}, {NULL}};
const Segment SGI357068[] = {{&S103926[5], 3}, {NULL}};
const Segment SGI357069[] = {{&S103926[14], 3}, {NULL}};
const Segment SGI357070[] = {{&S103929[5], 4}, {NULL}};
const Segment SGI357071[] = {{&S103929[20], 3}, {NULL}};
const Segment SGI357072[] = {{&S103943[52], 4}, {NULL}};
const Segment SGI357073[] = {{&S103943[81], 4}, {NULL}};
const Segment SGI357074[] = {{&S103962[5], 4}, {NULL}};
const Segment SGI357075[] = {{&S103962[15], 3}, {NULL}};
const Segment SGI357076[] = {{&S103966[5], 4}, {NULL}};
const Segment SGI357077[] = {{&S103966[20], 3}, {NULL}};
const Segment SGI357078[] = {{&S103967[5], 4}, {NULL}};
const Segment SGI357079[] = {{&S103967[20], 3}, {NULL}};
const Segment SGI357080[] = {{&S95981[28], 5}, {NULL}};
const Segment SGI357081[] = {{&S95981[104], 5}, {NULL}};
const Segment SGI357082[] = {{&S103986[5], 9}, {NULL}};
const Segment SGI357083[] = {{&S103986[25], 3}, {NULL}};
const Segment SGI357084[] = {{&S103988[5], 5}, {NULL}};
const Segment SGI357085[] = {{&S103988[16], 3}, {NULL}};
const Segment SGI357086[] = {{&S103989[5], 5}, {NULL}};
const Segment SGI357087[] = {{&S103989[16], 3}, {NULL}};
const Segment SGI357088[] = {{&S77323[6], 4}, {NULL}};
const Segment SGI357089[] = {{&S77323[53], 5}, {NULL}};
const Segment SGI357090[] = {{&S77323[12], 10}, {NULL}};
const Segment SGI357091[] = {{&S77324[6], 4}, {NULL}};
const Segment SGI357092[] = {{&S77324[53], 5}, {NULL}};
const Segment SGI357093[] = {{&S77324[12], 10}, {NULL}};
const Segment SGI357094[] = {{&S77325[6], 4}, {NULL}};
const Segment SGI357095[] = {{&S77325[53], 5}, {NULL}};
const Segment SGI357096[] = {{&S77325[12], 10}, {NULL}};
const Segment SGI357097[] = {{&S77326[6], 4}, {NULL}};
const Segment SGI357098[] = {{&S77326[53], 5}, {NULL}};
const Segment SGI357099[] = {{&S77326[12], 10}, {NULL}};
const Segment SGI357100[] = {{&S77327[6], 4}, {NULL}};
const Segment SGI357101[] = {{&S77327[53], 5}, {NULL}};
const Segment SGI357102[] = {{&S77327[12], 10}, {NULL}};
const Segment SGI357103[] = {{&S77898[0], 21}, {NULL}};
const Segment SGI357104[] = {{&S77898[34], 3}, {NULL}};
const Segment SGI357105[] = {{&S77898[75], 13}, {NULL}};
const Segment SGI357106[] = {{&S90541[5], 8}, {&S90541[160], 7}, {NULL}};
const Segment SGI357107[] = {{&S90541[59], 3}, {NULL}};
const Segment SGI357108[] = {{&S104127[5], 5}, {&S104127[5], 5}, {NULL}};
const Segment SGI357109[] = {{&S104127[11], 3}, {NULL}};
const Segment SGI357110[] = {{&S104127[131], 3}, {NULL}};
const Segment SGI357111[] = {{&S104150[5], 4}, {&S104150[5], 4}, {NULL}};
const Segment SGI357112[] = {{&S104150[82], 3}, {NULL}};
const Segment SGI357113[] = {{&S104152[5], 13}, {&S104152[76], 13}, {NULL}};
const Segment SGI357114[] = {{&S104152[100], 3}, {NULL}};
const Segment SGI357115[] = {{&S104156[5], 4}, {&S104156[5], 4}, {NULL}};
const Segment SGI357116[] = {{&S104156[82], 3}, {NULL}};
const Segment SGI357117[] = {{&S96304[28], 5}, {NULL}};
const Segment SGI357118[] = {{&S96304[104], 5}, {NULL}};
const Segment SGI357119[] = {{&S104250[5], 4}, {NULL}};
const Segment SGI357120[] = {{&S104250[41], 3}, {NULL}};
const Segment SGI357121[] = {{&S104253[5], 4}, {NULL}};
const Segment SGI357122[] = {{&S104272[5], 5}, {NULL}};
const Segment SGI357123[] = {{&S104272[21], 3}, {NULL}};
const Segment SGI357124[] = {{&S104273[5], 5}, {NULL}};
const Segment SGI357125[] = {{&S104273[21], 3}, {NULL}};
const Segment SGI357126[] = {{&S104274[5], 3}, {NULL}};
const Segment SGI357127[] = {{&S104274[14], 3}, {NULL}};
const Segment SGI357128[] = {{&S104278[5], 5}, {NULL}};
const Segment SGI357129[] = {{&S104278[16], 3}, {NULL}};
const Segment SGI357130[] = {{&S104279[5], 14}, {NULL}};
const Segment SGI357131[] = {{&S104279[30], 3}, {NULL}};
const Segment SGI357132[] = {{&S104280[5], 4}, {NULL}};
const Segment SGI357133[] = {{&S104280[15], 3}, {NULL}};
const Segment SGI357134[] = {{&S104281[5], 7}, {NULL}};
const Segment SGI357135[] = {{&S104281[23], 3}, {NULL}};
const Segment SGI357136[] = {{&S104285[5], 12}, {&S104285[5], 12}, {NULL}};
const Segment SGI357137[] = {{&S104285[98], 3}, {NULL}};
const Segment SGI357138[] = {{&S90792[21], 14}, {NULL}};
const Segment SGI357139[] = {{&S90792[49], 3}, {NULL}};
const Segment SGI357140[] = {{&S90802[19], 15}, {NULL}};
const Segment SGI357141[] = {{&S90802[48], 3}, {NULL}};
const Segment SGI357142[] = {{&S77678[6], 4}, {NULL}};
const Segment SGI357143[] = {{&S77678[82], 5}, {NULL}};
const Segment SGI357144[] = {{&S77678[12], 11}, {NULL}};
const Segment SGI357145[] = {{&S77679[6], 4}, {NULL}};
const Segment SGI357146[] = {{&S77679[84], 5}, {NULL}};
const Segment SGI357147[] = {{&S77679[12], 13}, {NULL}};
const Segment SGI357148[] = {{&S77680[6], 4}, {NULL}};
const Segment SGI357149[] = {{&S77680[84], 5}, {NULL}};
const Segment SGI357150[] = {{&S77680[12], 13}, {NULL}};
const Segment SGI357151[] = {{&S104315[52], 9}, {NULL}};
const Segment SGI357152[] = {{&S77681[6], 4}, {NULL}};
const Segment SGI357153[] = {{&S77681[84], 5}, {NULL}};
const Segment SGI357154[] = {{&S104315[69], 4}, {NULL}};
const Segment SGI357155[] = {{&S77681[12], 13}, {NULL}};
const Segment SGI357156[] = {{&S104336[5], 3}, {NULL}};
const Segment SGI357157[] = {{&S104336[14], 3}, {NULL}};
const Segment SGI357158[] = {{&S104337[5], 3}, {NULL}};
const Segment SGI357159[] = {{&S104337[14], 3}, {NULL}};
const Segment SGI357160[] = {{&S104339[5], 3}, {NULL}};
const Segment SGI357161[] = {{&S104339[14], 3}, {NULL}};
const Segment SGI357162[] = {{&S104341[5], 4}, {NULL}};
const Segment SGI357163[] = {{&S104341[20], 3}, {NULL}};
const Segment SGI357164[] = {{&S104351[25], 3}, {NULL}};
const Segment SGI357165[] = {{&S104351[0], 18}, {NULL}};
const Segment SGI357166[] = {{&S104356[5], 10}, {NULL}};
const Segment SGI357167[] = {{&S104359[5], 9}, {NULL}};
const Segment SGI357168[] = {{&S104359[25], 3}, {NULL}};
const Segment SGI357169[] = {{&S104361[5], 5}, {NULL}};
const Segment SGI357170[] = {{&S104361[16], 3}, {NULL}};
const Segment SGI357171[] = {{&S104367[25], 5}, {NULL}};
const Segment SGI357172[] = {{&S104367[0], 18}, {NULL}};
const Segment SGI357173[] = {{&S104390[5], 8}, {NULL}};
const Segment SGI357174[] = {{&S104390[19], 3}, {NULL}};
const Segment SGI357175[] = {{&S104395[5], 3}, {NULL}};
const Segment SGI357176[] = {{&S104395[14], 3}, {NULL}};
const Segment SGI357177[] = {{&S104396[25], 5}, {NULL}};
const Segment SGI357178[] = {{&S104396[0], 18}, {NULL}};
const Segment SGI357179[] = {{&S104401[21], 3}, {NULL}};
const Segment SGI357180[] = {{&S104401[0], 14}, {NULL}};
const Segment SGI357181[] = {{&S104404[5], 5}, {&S104404[97], 5}, {NULL}};
const Segment SGI357182[] = {{&S104404[113], 3}, {NULL}};
const Segment SGI357183[] = {{&S96639[28], 3}, {NULL}};
const Segment SGI357184[] = {{&S96639[102], 5}, {NULL}};
const Segment SGI357185[] = {{&S104532[42], 4}, {NULL}};
const Segment SGI357186[] = {{&S104532[6], 3}, {NULL}};
const Segment SGI357187[] = {{&S104532[67], 3}, {NULL}};
const Segment SGI357188[] = {{&S104553[42], 5}, {NULL}};
const Segment SGI357189[] = {{&S104553[6], 3}, {NULL}};
const Segment SGI357190[] = {{&S104553[68], 3}, {NULL}};
const Segment SGI357191[] = {{&S91094[5], 4}, {NULL}};
const Segment SGI357192[] = {{&S91094[37], 3}, {NULL}};
const Segment SGI357193[] = {{&S104574[5], 7}, {&S104574[5], 7}, {NULL}};
const Segment SGI357194[] = {{&S104574[88], 3}, {NULL}};
const Segment SGI357195[] = {{&S104577[5], 7}, {&S104577[5], 7}, {NULL}};
const Segment SGI357196[] = {{&S104577[88], 3}, {NULL}};
const Segment SGI357197[] = {{&S77884[6], 4}, {NULL}};
const Segment SGI357198[] = {{&S77884[52], 5}, {NULL}};
const Segment SGI357199[] = {{&S77884[12], 45}, {NULL}};
const Segment SGI357200[] = {{&S77884[12], 14}, {NULL}};
const Segment SGI357201[] = {{&S104591[5], 5}, {NULL}};
const Segment SGI357202[] = {{&S104591[26], 3}, {NULL}};
const Segment SGI357203[] = {{&S91141[19], 15}, {NULL}};
const Segment SGI357204[] = {{&S91141[48], 3}, {NULL}};
const Segment SGI357205[] = {{&S104618[5], 5}, {&S104618[68], 5}, {NULL}};
const Segment SGI357206[] = {{&S104618[84], 3}, {NULL}};
const Segment SGI357207[] = {{&S91145[25], 5}, {NULL}};
const Segment SGI357208[] = {{&S91145[53], 3}, {NULL}};
const Segment SGI357209[] = {{&S104629[5], 15}, {&S104629[78], 15}, {NULL}};
const Segment SGI357210[] = {{&S104629[104], 3}, {NULL}};
const Segment SGI357211[] = {{&S91145[0], 18}, {NULL}};
const Segment SGI357212[] = {{&S104647[5], 5}, {NULL}};
const Segment SGI357213[] = {{&S104647[43], 3}, {NULL}};
const Segment SGI357214[] = {{&S104668[5], 4}, {NULL}};
const Segment SGI357215[] = {{&S104668[41], 3}, {NULL}};
const Segment SGI357216[] = {{&S104669[5], 4}, {NULL}};
const Segment SGI357217[] = {{&S104669[25], 3}, {NULL}};
const Segment SGI357218[] = {{&S104672[5], 4}, {NULL}};
const Segment SGI357219[] = {{&S104672[20], 3}, {NULL}};
const Segment SGI357220[] = {{&S104686[5], 4}, {NULL}};
const Segment SGI357221[] = {{&S104686[42], 3}, {NULL}};
const Segment SGI357222[] = {{&S104692[5], 13}, {&S104692[76], 13}, {NULL}};
const Segment SGI357223[] = {{&S104692[100], 3}, {NULL}};
const Segment SGI357224[] = {{&S104694[5], 7}, {&S104694[5], 7}, {NULL}};
const Segment SGI357225[] = {{&S104694[88], 3}, {NULL}};
const Segment SGI357226[] = {{&S104695[5], 7}, {NULL}};
const Segment SGI357227[] = {{&S104695[238], 3}, {NULL}};
const Segment SGI357228[] = {{&S104696[5], 4}, {NULL}};
const Segment SGI357229[] = {{&S104696[20], 3}, {NULL}};
const Segment SGI357230[] = {{&S104701[5], 5}, {NULL}};
const Segment SGI357231[] = {{&S104701[16], 3}, {NULL}};
const Segment SGI357232[] = {{&S104707[5], 13}, {&S104707[76], 13}, {NULL}};
const Segment SGI357233[] = {{&S104707[100], 3}, {NULL}};
const Segment SGI357234[] = {{&S104708[5], 3}, {NULL}};
const Segment SGI357235[] = {{&S104708[14], 3}, {NULL}};
const Segment SGI357236[] = {{&S104713[5], 5}, {NULL}};
const Segment SGI357237[] = {{&S104713[21], 3}, {NULL}};
const Segment SGI357238[] = {{&S104714[5], 8}, {NULL}};
const Segment SGI357239[] = {{&S104714[19], 3}, {NULL}};
const Segment SGI357240[] = {{&S104716[5], 4}, {&S104716[5], 4}, {NULL}};
const Segment SGI357241[] = {{&S104716[82], 3}, {NULL}};
const Segment SGI357242[] = {{&S104717[5], 5}, {&S104717[68], 5}, {NULL}};
const Segment SGI357243[] = {{&S104717[84], 3}, {NULL}};
const Segment SGI357244[] = {{&S104718[5], 5}, {&S104718[68], 5}, {NULL}};
const Segment SGI357245[] = {{&S104718[84], 3}, {NULL}};
const Segment SGI357246[] = {{&S104728[5], 4}, {NULL}};
const Segment SGI357247[] = {{&S78042[6], 4}, {NULL}};
const Segment SGI357248[] = {{&S78042[99], 5}, {NULL}};
const Segment SGI357249[] = {{&S78042[12], 12}, {NULL}};
const Segment SGI357250[] = {{&S96874[28], 4}, {NULL}};
const Segment SGI357251[] = {{&S96874[103], 5}, {NULL}};
const Segment SGI357252[] = {{&S104784[5], 8}, {NULL}};
const Segment SGI357253[] = {{&S104784[19], 3}, {NULL}};
const Segment SGI357254[] = {{&S104787[5], 5}, {NULL}};
const Segment SGI357255[] = {{&S104787[16], 3}, {NULL}};
const Segment SGI357256[] = {{&S104788[23], 5}, {NULL}};
const Segment SGI357257[] = {{&S104788[0], 16}, {NULL}};
const Segment SGI357258[] = {{&S78083[6], 5}, {NULL}};
const Segment SGI357259[] = {{&S78083[60], 5}, {NULL}};
const Segment SGI357260[] = {{&S78083[13], 14}, {NULL}};
const Segment SGI357261[] = {{&S104798[5], 3}, {NULL}};
const Segment SGI357262[] = {{&S104798[14], 3}, {NULL}};
const Segment SGI357263[] = {{&S104800[5], 5}, {NULL}};
const Segment SGI357264[] = {{&S104800[16], 3}, {NULL}};
const Segment SGI357265[] = {{&S104810[5], 4}, {NULL}};
const Segment SGI357266[] = {{&S104810[15], 3}, {NULL}};
const Segment SGI357267[] = {{&S104812[5], 10}, {NULL}};
const Segment SGI357268[] = {{&S104812[21], 3}, {NULL}};
const Segment SGI357269[] = {{&S96928[28], 5}, {NULL}};
const Segment SGI357270[] = {{&S96928[104], 5}, {NULL}};
const Segment SGI357271[] = {{&S78121[6], 4}, {NULL}};
const Segment SGI357272[] = {{&S78121[12], 10}, {NULL}};
const Segment SGI357273[] = {{&S78122[6], 4}, {NULL}};
const Segment SGI357274[] = {{&S78122[12], 10}, {NULL}};
const Segment SGI357275[] = {{&S78123[6], 4}, {NULL}};
const Segment SGI357276[] = {{&S78123[12], 10}, {NULL}};
const Segment SGI357277[] = {{&S78124[6], 4}, {NULL}};
const Segment SGI357278[] = {{&S78124[12], 10}, {NULL}};
const Segment SGI357279[] = {{&S78125[6], 4}, {NULL}};
const Segment SGI357280[] = {{&S78125[12], 10}, {NULL}};
const Segment SGI357281[] = {{&S78126[6], 4}, {NULL}};
const Segment SGI357282[] = {{&S78126[12], 10}, {NULL}};
const Segment SGI357283[] = {{&S104838[5], 10}, {NULL}};
const Segment SGI357284[] = {{&S104838[21], 3}, {NULL}};
const Segment SGI357285[] = {{&S104849[41], 4}, {NULL}};
const Segment SGI357286[] = {{&S104849[6], 3}, {NULL}};
const Segment SGI357287[] = {{&S104849[66], 3}, {NULL}};
const Segment SGI357288[] = {{&S91314[21], 15}, {NULL}};
const Segment SGI357289[] = {{&S91314[50], 3}, {NULL}};
const Segment SGI357290[] = {{&S104860[5], 5}, {&S104860[5], 5}, {NULL}};
const Segment SGI357291[] = {{&S104860[11], 3}, {NULL}};
const Segment SGI357292[] = {{&S104860[130], 3}, {NULL}};
const Segment SGI357293[] = {{&S78229[6], 4}, {NULL}};
const Segment SGI357294[] = {{&S78229[12], 10}, {NULL}};
const Segment SGI357295[] = {{&S78230[6], 4}, {NULL}};
const Segment SGI357296[] = {{&S78230[12], 10}, {NULL}};
const Segment SGI357297[] = {{&S80522[38], 4}, {NULL}};
const Segment SGI357298[] = {{&S80522[13], 10}, {NULL}};
const Segment SGI357299[] = {{&S80523[57], 4}, {NULL}};
const Segment SGI357300[] = {{&S80523[13], 10}, {NULL}};
const Segment SGI357301[] = {{&S91478[5], 5}, {NULL}};
const Segment SGI357302[] = {{&S91478[33], 3}, {NULL}};
const Segment SGI357303[] = {{&S91478[87], 3}, {NULL}};
const Segment SGI357304[] = {{&S105028[5], 4}, {&S105028[5], 4}, {NULL}};
const Segment SGI357305[] = {{&S105028[10], 3}, {NULL}};
const Segment SGI357306[] = {{&S105028[128], 3}, {NULL}};
const Segment SGI357307[] = {{&S91504[21], 25}, {NULL}};
const Segment SGI357308[] = {{&S91504[60], 3}, {NULL}};
const Segment SGI357309[] = {{&S97158[28], 4}, {NULL}};
const Segment SGI357310[] = {{&S97158[103], 5}, {NULL}};
const Segment SGI357311[] = {{&S91532[19], 15}, {NULL}};
const Segment SGI357312[] = {{&S91532[48], 3}, {NULL}};
const Segment SGI357313[] = {{&S105072[42], 3}, {NULL}};
const Segment SGI357314[] = {{&S105072[6], 3}, {NULL}};
const Segment SGI357315[] = {{&S105072[66], 3}, {NULL}};
const Segment SGI357316[] = {{&S105083[42], 4}, {NULL}};
const Segment SGI357317[] = {{&S105083[6], 3}, {NULL}};
const Segment SGI357318[] = {{&S105083[67], 3}, {NULL}};
const Segment SGI357319[] = {{&S105087[42], 5}, {NULL}};
const Segment SGI357320[] = {{&S105087[6], 3}, {NULL}};
const Segment SGI357321[] = {{&S105087[68], 3}, {NULL}};
const Segment SGI357322[] = {{&S105090[5], 4}, {&S105090[5], 4}, {NULL}};
const Segment SGI357323[] = {{&S105090[82], 3}, {NULL}};
const Segment SGI357324[] = {{&S105092[5], 7}, {&S105092[5], 7}, {NULL}};
const Segment SGI357325[] = {{&S105092[88], 3}, {NULL}};
const Segment SGI357326[] = {{&S105094[5], 5}, {&S105094[5], 5}, {NULL}};
const Segment SGI357327[] = {{&S105094[84], 3}, {NULL}};
const Segment SGI357328[] = {{&S105096[5], 12}, {&S105096[5], 12}, {NULL}};
const Segment SGI357329[] = {{&S105096[98], 3}, {NULL}};
const Segment SGI357330[] = {{&S105099[5], 15}, {&S105099[78], 15}, {NULL}};
const Segment SGI357331[] = {{&S105099[104], 3}, {NULL}};
const Segment SGI357332[] = {{&S105107[5], 11}, {&S105107[5], 11}, {NULL}};
const Segment SGI357333[] = {{&S105107[96], 3}, {NULL}};
const Segment SGI357334[] = {{&S105108[5], 11}, {&S105108[5], 11}, {NULL}};
const Segment SGI357335[] = {{&S105108[96], 3}, {NULL}};
const Segment SGI357336[] = {{&S105112[5], 7}, {&S105112[5], 7}, {NULL}};
const Segment SGI357337[] = {{&S105112[88], 3}, {NULL}};
const Segment SGI357338[] = {{&S105117[5], 4}, {NULL}};
const Segment SGI357339[] = {{&S105117[20], 3}, {NULL}};
const Segment SGI357340[] = {{&S105120[5], 5}, {&S105120[68], 5}, {NULL}};
const Segment SGI357341[] = {{&S105120[84], 3}, {NULL}};
const Segment SGI357342[] = {{&S105121[5], 5}, {&S105121[68], 5}, {NULL}};
const Segment SGI357343[] = {{&S105121[84], 3}, {NULL}};
const Segment SGI357344[] = {{&S105132[5], 5}, {NULL}};
const Segment SGI357345[] = {{&S105132[43], 3}, {NULL}};
const Segment SGI357346[] = {{&S80661[95], 4}, {NULL}};
const Segment SGI357347[] = {{&S80661[101], 3}, {NULL}};
const Segment SGI357348[] = {{&S80661[123], 3}, {NULL}};
const Segment SGI357349[] = {{&S80661[138], 5}, {NULL}};
const Segment SGI357350[] = {{&S105139[5], 4}, {NULL}};
const Segment SGI357351[] = {{&S105139[20], 3}, {NULL}};
const Segment SGI357352[] = {{&S78484[6], 4}, {NULL}};
const Segment SGI357353[] = {{&S78484[12], 11}, {&S78484[41], 65}, {NULL}};
const Segment SGI357354[] = {{&S105194[5], 4}, {&S105194[5], 4}, {NULL}};
const Segment SGI357355[] = {{&S105194[82], 3}, {NULL}};
const Segment SGI357356[] = {{&S91667[5], 5}, {NULL}};
const Segment SGI357357[] = {{&S91667[11], 3}, {NULL}};
const Segment SGI357358[] = {{&S91687[5], 3}, {NULL}};
const Segment SGI357359[] = {{&S91687[31], 3}, {NULL}};
const Segment SGI357360[] = {{&S91687[85], 3}, {NULL}};
const Segment SGI357361[] = {{&S105217[5], 5}, {&S105217[68], 5}, {NULL}};
const Segment SGI357362[] = {{&S105217[84], 3}, {NULL}};
const Segment SGI357363[] = {{&S78525[6], 4}, {NULL}};
const Segment SGI357364[] = {{&S78525[72], 5}, {NULL}};
const Segment SGI357365[] = {{&S78525[12], 3}, {NULL}};
const Segment SGI357366[] = {{&S78525[32], 10}, {NULL}};
const Segment SGI357367[] = {{&S105227[5], 4}, {&S105227[5], 4}, {NULL}};
const Segment SGI357368[] = {{&S105227[82], 3}, {NULL}};
const Segment SGI357369[] = {{&S78534[6], 4}, {NULL}};
const Segment SGI357370[] = {{&S78534[82], 5}, {NULL}};
const Segment SGI357371[] = {{&S78534[12], 11}, {NULL}};
const Segment SGI357372[] = {{&S80752[61], 5}, {NULL}};
const Segment SGI357373[] = {{&S80752[13], 10}, {NULL}};
const Segment SGI357374[] = {{&S105260[5], 4}, {NULL}};
const Segment SGI357375[] = {{&S105260[20], 3}, {NULL}};
const Segment SGI357376[] = {{&S105283[5], 5}, {NULL}};
const Segment SGI357377[] = {{&S105283[42], 3}, {NULL}};
const Segment SGI357378[] = {{&S78586[6], 4}, {NULL}};
const Segment SGI357379[] = {{&S78586[57], 5}, {NULL}};
const Segment SGI357380[] = {{&S78586[12], 14}, {NULL}};
const Segment SGI357381[] = {{&S105287[5], 5}, {&S105287[68], 5}, {NULL}};
const Segment SGI357382[] = {{&S105287[84], 3}, {NULL}};
const Segment SGI357383[] = {{&S91760[21], 14}, {NULL}};
const Segment SGI357384[] = {{&S91760[49], 3}, {NULL}};
const Segment SGI357385[] = {{&S105303[5], 16}, {NULL}};
const Segment SGI357386[] = {{&S105303[22], 3}, {NULL}};
const Segment SGI357387[] = {{&S105312[5], 16}, {NULL}};
const Segment SGI357388[] = {{&S105312[22], 3}, {NULL}};
const Segment SGI357389[] = {{&S105314[5], 5}, {&S105314[5], 5}, {NULL}};
const Segment SGI357390[] = {{&S105314[84], 3}, {NULL}};
const Segment SGI357391[] = {{&S78726[6], 4}, {NULL}};
const Segment SGI357392[] = {{&S78726[75], 5}, {NULL}};
const Segment SGI357393[] = {{&S78726[12], 9}, {NULL}};
const Segment SGI357394[] = {{&S78726[29], 13}, {NULL}};
const Segment SGI357395[] = {{&S78727[6], 4}, {NULL}};
const Segment SGI357396[] = {{&S78727[75], 5}, {NULL}};
const Segment SGI357397[] = {{&S78727[12], 9}, {NULL}};
const Segment SGI357398[] = {{&S78727[29], 13}, {NULL}};
const Segment SGI357399[] = {{&S78735[6], 4}, {NULL}};
const Segment SGI357400[] = {{&S78735[90], 5}, {NULL}};
const Segment SGI357401[] = {{&S78735[12], 9}, {NULL}};
const Segment SGI357402[] = {{&S78735[29], 13}, {NULL}};
const Segment SGI357403[] = {{&S91922[19], 14}, {NULL}};
const Segment SGI357404[] = {{&S91922[47], 3}, {NULL}};
const Segment SGI357405[] = {{&S78811[6], 4}, {NULL}};
const Segment SGI357406[] = {{&S78811[56], 5}, {NULL}};
const Segment SGI357407[] = {{&S78811[12], 12}, {NULL}};
const Segment SGI357408[] = {{&S78977[6], 4}, {NULL}};
const Segment SGI357409[] = {{&S78977[12], 21}, {NULL}};
const Segment SGI357410[] = {{&S78978[6], 4}, {NULL}};
const Segment SGI357411[] = {{&S78978[12], 21}, {NULL}};
const Segment SGI357412[] = {{&S78980[6], 4}, {NULL}};
const Segment SGI357413[] = {{&S78980[12], 23}, {NULL}};
const Segment SGI357414[] = {{&S78981[6], 4}, {NULL}};
const Segment SGI357415[] = {{&S78981[12], 21}, {NULL}};
const Segment SGI357416[] = {{&S79027[6], 4}, {NULL}};
const Segment SGI357417[] = {{&S79027[57], 5}, {NULL}};
const Segment SGI357418[] = {{&S79027[12], 14}, {NULL}};
const Segment SGI357419[] = {{&S79028[6], 4}, {NULL}};
const Segment SGI357420[] = {{&S79028[57], 5}, {NULL}};
const Segment SGI357421[] = {{&S79028[12], 14}, {NULL}};
const Segment SGI357422[] = {{&S79029[6], 5}, {NULL}};
const Segment SGI357423[] = {{&S79029[13], 14}, {NULL}};
const Segment SGI357424[] = {{&S79436[0], 16}, {NULL}};
const Segment SGI357425[] = {{&S79436[29], 3}, {NULL}};
const Segment SGI357426[] = {{&S79436[67], 13}, {NULL}};
const Segment SGI357427[] = {{&S105316[5], 16}, {NULL}};
const Segment SGI357428[] = {{&S105316[27], 3}, {NULL}};
const Segment SGI357429[] = {{&S105317[5], 4}, {NULL}};
const Segment SGI357430[] = {{&S105317[20], 3}, {NULL}};
const Segment SGI357431[] = {{&S105318[5], 3}, {NULL}};
const Segment SGI357432[] = {{&S105318[14], 3}, {NULL}};
const Segment SGI357433[] = {{&S105319[5], 4}, {NULL}};
const Segment SGI357434[] = {{&S105320[5], 4}, {NULL}};
const Segment SGI357435[] = {{&S105320[20], 3}, {NULL}};
const Segment SGI357436[] = {{&S105321[5], 5}, {NULL}};
const Segment SGI357437[] = {{&S105321[16], 3}, {NULL}};
const Segment SGI357438[] = {{&S105325[5], 4}, {NULL}};
const Segment SGI357439[] = {{&S105325[20], 3}, {NULL}};
const Segment SGI357440[] = {{&S105331[5], 5}, {NULL}};
const Segment SGI357441[] = {{&S105331[16], 3}, {NULL}};
const Segment SGI357442[] = {{&S105333[5], 4}, {NULL}};
const Segment SGI357443[] = {{&S105333[20], 3}, {NULL}};
const Segment SGI357444[] = {{&S105334[5], 4}, {NULL}};
const Segment SGI357445[] = {{&S105334[20], 3}, {NULL}};
const Segment SGI357446[] = {{&S105337[5], 5}, {NULL}};
const Segment SGI357447[] = {{&S105337[16], 3}, {NULL}};
const Segment SGI357448[] = {{&S105342[5], 4}, {NULL}};
const Segment SGI357449[] = {{&S105342[20], 3}, {NULL}};
const Segment SGI357450[] = {{&S105343[5], 11}, {&S105343[5], 11}, {NULL}};
const Segment SGI357451[] = {{&S105343[96], 3}, {NULL}};
const Segment SGI357452[] = {{&S105345[5], 12}, {&S105345[5], 12}, {NULL}};
const Segment SGI357453[] = {{&S105345[98], 3}, {NULL}};
const Segment SGI357454[] = {{&S105347[5], 13}, {&S105347[76], 13}, {NULL}};
const Segment SGI357455[] = {{&S105347[100], 3}, {NULL}};
const Segment SGI357456[] = {{&S105361[5], 4}, {NULL}};
const Segment SGI357457[] = {{&S105362[5], 5}, {NULL}};
const Segment SGI357458[] = {{&S105474[5], 16}, {NULL}};
const Segment SGI357459[] = {{&S105474[27], 3}, {NULL}};
const Segment SGI357460[] = {{&S105475[5], 8}, {NULL}};
const Segment SGI357461[] = {{&S105475[19], 3}, {NULL}};
const Segment SGI357462[] = {{&S105476[5], 3}, {NULL}};
const Segment SGI357463[] = {{&S105476[14], 3}, {NULL}};
const Segment SGI357464[] = {{&S105477[5], 3}, {NULL}};
const Segment SGI357465[] = {{&S105477[14], 3}, {NULL}};
const Segment SGI357466[] = {{&S105478[5], 5}, {&S105478[68], 5}, {NULL}};
const Segment SGI357467[] = {{&S105478[84], 3}, {NULL}};
const Segment SGI357468[] = {{&S105511[5], 10}, {NULL}};
const Segment SGI357469[] = {{&S79256[6], 4}, {NULL}};
const Segment SGI357470[] = {{&S79256[12], 14}, {&S79256[44], 20}, {NULL}};
const Segment SGI357471[] = {{&S79256[12], 14}, {NULL}};
const Segment SGI357472[] = {{&S105516[100], 7}, {NULL}};
const Segment SGI357473[] = {{&S105519[5], 5}, {NULL}};
const Segment SGI357474[] = {{&S105519[16], 3}, {NULL}};
const Segment SGI357475[] = {{&S105557[5], 4}, {NULL}};
const Segment SGI357476[] = {{&S105557[20], 3}, {NULL}};
const Segment SGI357477[] = {{&S105558[5], 5}, {NULL}};
const Segment SGI357478[] = {{&S105558[16], 3}, {NULL}};
const Segment SGI357479[] = {{&S105563[5], 3}, {&S105563[5], 3}, {NULL}};
const Segment SGI357480[] = {{&S105563[109], 3}, {NULL}};
const Segment SGI357481[] = {{&S105565[41], 4}, {NULL}};
const Segment SGI357482[] = {{&S105565[6], 3}, {NULL}};
const Segment SGI357483[] = {{&S105565[66], 3}, {NULL}};
const Segment SGI357484[] = {{&S105579[42], 4}, {NULL}};
const Segment SGI357485[] = {{&S105579[6], 3}, {NULL}};
const Segment SGI357486[] = {{&S105579[67], 3}, {NULL}};
const Segment SGI357487[] = {{&S105582[5], 4}, {&S105582[5], 4}, {NULL}};
const Segment SGI357488[] = {{&S105582[10], 3}, {NULL}};
const Segment SGI357489[] = {{&S105582[128], 3}, {NULL}};
const Segment SGI357490[] = {{&S92470[5], 3}, {NULL}};
const Segment SGI357491[] = {{&S92470[31], 3}, {NULL}};
const Segment SGI357492[] = {{&S92470[85], 3}, {NULL}};
const Segment SGI357493[] = {{&S92515[21], 15}, {NULL}};
const Segment SGI357494[] = {{&S92515[50], 3}, {NULL}};
const Segment SGI357495[] = {{&S92518[19], 15}, {NULL}};
const Segment SGI357496[] = {{&S92518[48], 3}, {NULL}};
const Segment SGI357497[] = {{&S79478[6], 4}, {NULL}};
const Segment SGI357498[] = {{&S79478[80], 5}, {NULL}};
const Segment SGI357499[] = {{&S79478[12], 3}, {NULL}};
const Segment SGI357500[] = {{&S79478[40], 10}, {NULL}};
const Segment SGI357501[] = {{&S79587[6], 4}, {NULL}};
const Segment SGI357502[] = {{&S79587[12], 3}, {NULL}};
const Segment SGI357503[] = {{&S79604[6], 4}, {NULL}};
const Segment SGI357504[] = {{&S79604[48], 5}, {NULL}};
const Segment SGI357505[] = {{&S79604[12], 41}, {NULL}};
const Segment SGI357506[] = {{&S79625[6], 4}, {NULL}};
const Segment SGI357507[] = {{&S79625[90], 5}, {NULL}};
const Segment SGI357508[] = {{&S79625[12], 9}, {NULL}};
const Segment SGI357509[] = {{&S79625[29], 13}, {NULL}};
const Segment SGI357510[] = {{&S79695[6], 4}, {NULL}};
const Segment SGI357511[] = {{&S79695[55], 5}, {NULL}};
const Segment SGI357512[] = {{&S79695[12], 10}, {NULL}};
const Segment SGI357513[] = {{&S79696[6], 4}, {NULL}};
const Segment SGI357514[] = {{&S79696[55], 5}, {NULL}};
const Segment SGI357515[] = {{&S79696[12], 10}, {NULL}};
const Segment SGI357516[] = {{&S98344[28], 7}, {NULL}};
const Segment SGI357517[] = {{&S98344[106], 5}, {NULL}};
const Segment SGI357518[] = {{&S92894[21], 14}, {NULL}};
const Segment SGI357519[] = {{&S92894[49], 3}, {NULL}};
const Segment SGI357520[] = {{&S77008[73], 5}, {NULL}};
const Segment SGI357521[] = {{&S77008[12], 3}, {NULL}};
const Segment SGI357522[] = {{&S80172[0], 21}, {NULL}};
const Segment SGI357523[] = {{&S80172[34], 3}, {NULL}};
const Segment SGI357524[] = {{&S80172[75], 13}, {NULL}};
const Segment SGI357525[] = {{&S79883[6], 4}, {NULL}};
const Segment SGI357526[] = {{&S79883[58], 5}, {NULL}};
const Segment SGI357527[] = {{&S79883[12], 14}, {NULL}};
const Segment SGI357528[] = {{&S79907[6], 4}, {NULL}};
const Segment SGI357529[] = {{&S79907[48], 5}, {NULL}};
const Segment SGI357530[] = {{&S79907[12], 41}, {NULL}};
const Segment SGI357531[] = {{&S98590[28], 4}, {NULL}};
const Segment SGI357532[] = {{&S98590[103], 5}, {NULL}};
const Segment SGI357533[] = {{&S105901[42], 4}, {NULL}};
const Segment SGI357534[] = {{&S105901[6], 3}, {NULL}};
const Segment SGI357535[] = {{&S105901[67], 3}, {NULL}};
const Segment SGI357536[] = {{&S105913[42], 4}, {NULL}};
const Segment SGI357537[] = {{&S105913[6], 3}, {NULL}};
const Segment SGI357538[] = {{&S105913[67], 3}, {NULL}};
const Segment SGI357539[] = {{&S105929[5], 4}, {&S105929[5], 4}, {NULL}};
const Segment SGI357540[] = {{&S105929[10], 3}, {NULL}};
const Segment SGI357541[] = {{&S105929[129], 3}, {NULL}};
const Segment SGI357542[] = {{&S105963[5], 4}, {&S105963[5], 4}, {NULL}};
const Segment SGI357543[] = {{&S105963[10], 3}, {NULL}};
const Segment SGI357544[] = {{&S105963[128], 3}, {NULL}};
const Segment SGI357545[] = {{&S105964[42], 4}, {NULL}};
const Segment SGI357546[] = {{&S105964[6], 3}, {NULL}};
const Segment SGI357547[] = {{&S105964[67], 3}, {NULL}};
const Segment SGI357548[] = {{&S105976[42], 5}, {NULL}};
const Segment SGI357549[] = {{&S105976[6], 3}, {NULL}};
const Segment SGI357550[] = {{&S105976[68], 3}, {NULL}};
const Segment SGI357551[] = {{&S105977[5], 4}, {&S105977[5], 4}, {NULL}};
const Segment SGI357552[] = {{&S105977[10], 3}, {NULL}};
const Segment SGI357553[] = {{&S105977[129], 3}, {NULL}};
const Segment SGI357554[] = {{&S105998[42], 4}, {NULL}};
const Segment SGI357555[] = {{&S105998[6], 3}, {NULL}};
const Segment SGI357556[] = {{&S105998[67], 3}, {NULL}};
const Segment SGI357557[] = {{&S106001[5], 5}, {&S106001[5], 5}, {NULL}};
const Segment SGI357558[] = {{&S106001[11], 3}, {NULL}};
const Segment SGI357559[] = {{&S106001[131], 3}, {NULL}};
const Segment SGI357560[] = {{&S106003[5], 4}, {&S106003[5], 4}, {NULL}};
const Segment SGI357561[] = {{&S106003[82], 3}, {NULL}};
const Segment SGI357562[] = {{&S106006[5], 4}, {&S106006[5], 4}, {NULL}};
const Segment SGI357563[] = {{&S106006[82], 3}, {NULL}};
const Segment SGI357564[] = {{&S93280[5], 4}, {NULL}};
const Segment SGI357565[] = {{&S93280[47], 3}, {NULL}};
const Segment SGI357566[] = {{&S93280[101], 3}, {NULL}};
const Segment SGI357567[] = {{&S82373[61], 4}, {NULL}};
const Segment SGI357568[] = {{&S82373[13], 10}, {NULL}};
const Segment SGI357569[] = {{&S82374[61], 4}, {NULL}};
const Segment SGI357570[] = {{&S82374[13], 10}, {NULL}};
const Segment SGI357571[] = {{&S82375[61], 4}, {NULL}};
const Segment SGI357572[] = {{&S82375[13], 10}, {NULL}};
const Segment SGI357573[] = {{&S82376[61], 4}, {NULL}};
const Segment SGI357574[] = {{&S82376[13], 10}, {NULL}};
const Segment SGI357575[] = {{&S80364[6], 4}, {NULL}};
const Segment SGI357576[] = {{&S80364[78], 5}, {NULL}};
const Segment SGI357577[] = {{&S80364[12], 13}, {NULL}};
const Segment SGI357578[] = {{&S80365[6], 4}, {NULL}};
const Segment SGI357579[] = {{&S80365[78], 5}, {NULL}};
const Segment SGI357580[] = {{&S80365[12], 13}, {NULL}};
const Segment SGI357581[] = {{&S80369[6], 4}, {NULL}};
const Segment SGI357582[] = {{&S80369[68], 5}, {NULL}};
const Segment SGI357583[] = {{&S80369[12], 3}, {NULL}};
const Segment SGI357584[] = {{&S93465[21], 13}, {NULL}};
const Segment SGI357585[] = {{&S93465[48], 3}, {NULL}};
const Segment SGI357586[] = {{&S80520[6], 4}, {NULL}};
const Segment SGI357587[] = {{&S80520[72], 5}, {NULL}};
const Segment SGI357588[] = {{&S80520[12], 3}, {NULL}};
const Segment SGI357589[] = {{&S80520[34], 10}, {NULL}};
const Segment SGI357590[] = {{&S93523[5], 4}, {NULL}};
const Segment SGI357591[] = {{&S93523[46], 3}, {NULL}};
const Segment SGI357592[] = {{&S99205[28], 4}, {NULL}};
const Segment SGI357593[] = {{&S99205[103], 5}, {NULL}};
const Segment SGI357594[] = {{&S93715[5], 3}, {NULL}};
const Segment SGI357595[] = {{&S93715[31], 3}, {NULL}};
const Segment SGI357596[] = {{&S93715[85], 3}, {NULL}};
const Segment SGI357597[] = {{&S93737[5], 5}, {NULL}};
const Segment SGI357598[] = {{&S93737[27], 3}, {NULL}};
const Segment SGI357599[] = {{&S93752[5], 8}, {NULL}};
const Segment SGI357600[] = {{&S93752[41], 3}, {NULL}};
const Segment SGI357601[] = {{&S93752[95], 3}, {NULL}};
const Segment SGI357602[] = {{&S80868[6], 4}, {NULL}};
const Segment SGI357603[] = {{&S80868[12], 14}, {NULL}};
const Segment SGI357604[] = {{&S93782[21], 15}, {NULL}};
const Segment SGI357605[] = {{&S93782[50], 3}, {NULL}};
const Segment SGI357606[] = {{&S81117[0], 15}, {NULL}};
const Segment SGI357607[] = {{&S81117[31], 3}, {NULL}};
const Segment SGI357608[] = {{&S93826[21], 15}, {NULL}};
const Segment SGI357609[] = {{&S93826[50], 3}, {NULL}};
const Segment SGI357610[] = {{&S78086[149], 5}, {NULL}};
const Segment SGI357611[] = {{&S78086[86], 3}, {NULL}};
const Segment SGI357612[] = {{&S81342[6], 4}, {NULL}};
const Segment SGI357613[] = {{&S81342[48], 5}, {NULL}};
const Segment SGI357614[] = {{&S81342[12], 41}, {NULL}};
const Segment SGI357615[] = {{&S94240[19], 20}, {NULL}};
const Segment SGI357616[] = {{&S94240[53], 3}, {NULL}};
const Segment SGI357617[] = {{&S99612[28], 4}, {NULL}};
const Segment SGI357618[] = {{&S99612[103], 5}, {NULL}};
const Segment SGI357619[] = {{&S99630[28], 7}, {NULL}};
const Segment SGI357620[] = {{&S99630[106], 5}, {NULL}};
const Segment SGI357621[] = {{&S94290[19], 15}, {NULL}};
const Segment SGI357622[] = {{&S94290[48], 3}, {NULL}};
const Segment SGI357623[] = {{&S81480[6], 4}, {NULL}};
const Segment SGI357624[] = {{&S81480[76], 5}, {NULL}};
const Segment SGI357625[] = {{&S81480[12], 11}, {NULL}};
const Segment SGI357626[] = {{&S81481[12], 13}, {NULL}};
const Segment SGI357627[] = {{&S81482[12], 13}, {NULL}};
const Segment SGI357628[] = {{&S81539[6], 4}, {NULL}};
const Segment SGI357629[] = {{&S81539[75], 5}, {NULL}};
const Segment SGI357630[] = {{&S81539[12], 3}, {NULL}};
const Segment SGI357631[] = {{&S81539[39], 10}, {NULL}};
const Segment SGI357632[] = {{&S78592[77], 5}, {NULL}};
const Segment SGI357633[] = {{&S78592[12], 14}, {NULL}};
const Segment SGI357634[] = {{&S94463[21], 14}, {NULL}};
const Segment SGI357635[] = {{&S94463[49], 3}, {NULL}};
const Segment SGI357636[] = {{&S81615[6], 4}, {NULL}};
const Segment SGI357637[] = {{&S81615[48], 5}, {NULL}};
const Segment SGI357638[] = {{&S81615[12], 41}, {NULL}};
const Segment SGI357639[] = {{&S94658[5], 5}, {NULL}};
const Segment SGI357640[] = {{&S94658[34], 3}, {NULL}};
const Segment SGI357641[] = {{&S94660[5], 4}, {NULL}};
const Segment SGI357642[] = {{&S94660[47], 3}, {NULL}};
const Segment SGI357643[] = {{&S94660[101], 3}, {NULL}};
const Segment SGI357644[] = {{&S94703[21], 14}, {NULL}};
const Segment SGI357645[] = {{&S94703[49], 3}, {NULL}};
const Segment SGI357646[] = {{&S94706[19], 13}, {NULL}};
const Segment SGI357647[] = {{&S94706[46], 3}, {NULL}};
const Segment SGI357648[] = {{&S94707[21], 15}, {NULL}};
const Segment SGI357649[] = {{&S94707[50], 3}, {NULL}};
const Segment SGI357650[] = {{&S94717[19], 15}, {NULL}};
const Segment SGI357651[] = {{&S94717[48], 3}, {NULL}};
const Segment SGI357652[] = {{&S100130[28], 4}, {NULL}};
const Segment SGI357653[] = {{&S100130[103], 5}, {NULL}};
const Segment SGI357654[] = {{&S94884[5], 8}, {NULL}};
const Segment SGI357655[] = {{&S94884[41], 3}, {NULL}};
const Segment SGI357656[] = {{&S94884[95], 3}, {NULL}};
const Segment SGI357657[] = {{&S100232[28], 5}, {NULL}};
const Segment SGI357658[] = {{&S100232[104], 5}, {NULL}};
const Segment SGI357659[] = {{&S100233[28], 5}, {NULL}};
const Segment SGI357660[] = {{&S100233[104], 5}, {NULL}};
const Segment SGI357661[] = {{&S79276[82], 5}, {NULL}};
const Segment SGI357662[] = {{&S79276[24], 9}, {NULL}};
const Segment SGI357663[] = {{&S79277[82], 5}, {NULL}};
const Segment SGI357664[] = {{&S79277[24], 9}, {NULL}};
const Segment SGI357665[] = {{&S79278[82], 5}, {NULL}};
const Segment SGI357666[] = {{&S79278[24], 9}, {NULL}};
const Segment SGI357667[] = {{&S79279[82], 5}, {NULL}};
const Segment SGI357668[] = {{&S79279[24], 9}, {NULL}};
const Segment SGI357669[] = {{&S79280[82], 5}, {NULL}};
const Segment SGI357670[] = {{&S79280[24], 9}, {NULL}};
const Segment SGI357671[] = {{&S82198[6], 4}, {NULL}};
const Segment SGI357672[] = {{&S82198[97], 5}, {NULL}};
const Segment SGI357673[] = {{&S82198[12], 9}, {NULL}};
const Segment SGI357674[] = {{&S82198[29], 13}, {NULL}};
const Segment SGI357675[] = {{&S94994[21], 15}, {NULL}};
const Segment SGI357676[] = {{&S94994[50], 3}, {NULL}};
const Segment SGI357677[] = {{&S95158[21], 14}, {NULL}};
const Segment SGI357678[] = {{&S95158[49], 3}, {NULL}};
const Segment SGI357679[] = {{&S100394[28], 4}, {NULL}};
const Segment SGI357680[] = {{&S100394[103], 5}, {NULL}};
const Segment SGI357681[] = {{&S82464[6], 4}, {NULL}};
const Segment SGI357682[] = {{&S82464[43], 4}, {NULL}};
const Segment SGI357683[] = {{&S82464[12], 14}, {NULL}};
const Segment SGI357684[] = {{&S82465[6], 4}, {NULL}};
const Segment SGI357685[] = {{&S82465[57], 5}, {NULL}};
const Segment SGI357686[] = {{&S82465[12], 14}, {NULL}};
const Segment SGI357687[] = {{&S82466[6], 4}, {NULL}};
const Segment SGI357688[] = {{&S82466[59], 5}, {NULL}};
const Segment SGI357689[] = {{&S82466[12], 14}, {NULL}};
const Segment SGI357690[] = {{&S95204[21], 15}, {NULL}};
const Segment SGI357691[] = {{&S95204[50], 3}, {NULL}};
const Segment SGI357692[] = {{&S95211[19], 15}, {NULL}};
const Segment SGI357693[] = {{&S95211[48], 3}, {NULL}};
const Segment SGI357694[] = {{&S95216[21], 15}, {NULL}};
const Segment SGI357695[] = {{&S95216[50], 3}, {NULL}};
const Segment SGI357696[] = {{&S84089[151], 4}, {NULL}};
const Segment SGI357697[] = {{&S84089[157], 3}, {NULL}};
const Segment SGI357698[] = {{&S84089[179], 3}, {NULL}};
const Segment SGI357699[] = {{&S84089[194], 5}, {NULL}};
const Segment SGI357700[] = {{&S82562[6], 4}, {NULL}};
const Segment SGI357701[] = {{&S82562[81], 5}, {NULL}};
const Segment SGI357702[] = {{&S82562[12], 3}, {NULL}};
const Segment SGI357703[] = {{&S82611[6], 4}, {NULL}};
const Segment SGI357704[] = {{&S82611[73], 4}, {NULL}};
const Segment SGI357705[] = {{&S82611[12], 3}, {NULL}};
const Segment SGI357706[] = {{&S95377[5], 4}, {NULL}};
const Segment SGI357707[] = {{&S95377[49], 3}, {NULL}};
const Segment SGI357708[] = {{&S95377[103], 3}, {NULL}};
const Segment SGI357709[] = {{&S95399[21], 14}, {NULL}};
const Segment SGI357710[] = {{&S95399[49], 3}, {NULL}};
const Segment SGI357711[] = {{&S95443[5], 4}, {NULL}};
const Segment SGI357712[] = {{&S95443[49], 3}, {NULL}};
const Segment SGI357713[] = {{&S95443[103], 3}, {NULL}};
const Segment SGI357714[] = {{&S95462[21], 14}, {NULL}};
const Segment SGI357715[] = {{&S95462[49], 3}, {NULL}};
const Segment SGI357716[] = {{&S82773[6], 4}, {NULL}};
const Segment SGI357717[] = {{&S82773[52], 5}, {NULL}};
const Segment SGI357718[] = {{&S82773[12], 45}, {NULL}};
const Segment SGI357719[] = {{&S82773[12], 14}, {NULL}};
const Segment SGI357720[] = {{&S84378[30], 3}, {NULL}};
const Segment SGI357721[] = {{&S84378[13], 10}, {NULL}};
const Segment SGI357722[] = {{&S95547[21], 14}, {NULL}};
const Segment SGI357723[] = {{&S95547[49], 3}, {NULL}};
const Segment SGI357724[] = {{&S82832[6], 4}, {NULL}};
const Segment SGI357725[] = {{&S82832[43], 4}, {NULL}};
const Segment SGI357726[] = {{&S82832[12], 14}, {NULL}};
const Segment SGI357727[] = {{&S82833[6], 4}, {NULL}};
const Segment SGI357728[] = {{&S82833[57], 5}, {NULL}};
const Segment SGI357729[] = {{&S82833[12], 14}, {NULL}};
const Segment SGI357730[] = {{&S82834[6], 4}, {NULL}};
const Segment SGI357731[] = {{&S82834[12], 7}, {NULL}};
const Segment SGI357732[] = {{&S82834[24], 14}, {NULL}};
const Segment SGI357733[] = {{&S82837[6], 4}, {NULL}};
const Segment SGI357734[] = {{&S82837[84], 5}, {NULL}};
const Segment SGI357735[] = {{&S82837[12], 13}, {NULL}};
const Segment SGI357736[] = {{&S82838[6], 4}, {NULL}};
const Segment SGI357737[] = {{&S82838[84], 5}, {NULL}};
const Segment SGI357738[] = {{&S82838[12], 13}, {NULL}};
const Segment SGI357739[] = {{&S95671[5], 5}, {NULL}};
const Segment SGI357740[] = {{&S95671[11], 3}, {NULL}};
const Segment SGI357741[] = {{&S100640[28], 5}, {NULL}};
const Segment SGI357742[] = {{&S100640[104], 5}, {NULL}};
const Segment SGI357743[] = {{&S95754[19], 13}, {NULL}};
const Segment SGI357744[] = {{&S95754[46], 3}, {NULL}};
const Segment SGI357745[] = {{&S95755[19], 14}, {NULL}};
const Segment SGI357746[] = {{&S95755[47], 3}, {NULL}};
const Segment SGI357747[] = {{&S83311[0], 21}, {NULL}};
const Segment SGI357748[] = {{&S83311[34], 3}, {NULL}};
const Segment SGI357749[] = {{&S100655[28], 7}, {NULL}};
const Segment SGI357750[] = {{&S100655[106], 5}, {NULL}};
const Segment SGI357751[] = {{&S100819[28], 3}, {NULL}};
const Segment SGI357752[] = {{&S100819[102], 5}, {NULL}};
const Segment SGI357753[] = {{&S95955[5], 4}, {NULL}};
const Segment SGI357754[] = {{&S95955[15], 3}, {NULL}};
const Segment SGI357755[] = {{&S96000[19], 14}, {NULL}};
const Segment SGI357756[] = {{&S96000[47], 3}, {NULL}};
const Segment SGI357757[] = {{&S83307[6], 4}, {NULL}};
const Segment SGI357758[] = {{&S83307[59], 5}, {NULL}};
const Segment SGI357759[] = {{&S83307[12], 3}, {NULL}};
const Segment SGI357760[] = {{&S83308[6], 4}, {NULL}};
const Segment SGI357761[] = {{&S83308[59], 5}, {NULL}};
const Segment SGI357762[] = {{&S83308[12], 3}, {NULL}};
const Segment SGI357763[] = {{&S96053[19], 15}, {NULL}};
const Segment SGI357764[] = {{&S96053[48], 3}, {NULL}};
const Segment SGI357765[] = {{&S83328[6], 4}, {NULL}};
const Segment SGI357766[] = {{&S83328[76], 5}, {NULL}};
const Segment SGI357767[] = {{&S83328[12], 11}, {NULL}};
const Segment SGI357768[] = {{&S83329[6], 4}, {NULL}};
const Segment SGI357769[] = {{&S83329[84], 5}, {NULL}};
const Segment SGI357770[] = {{&S83329[12], 19}, {NULL}};
const Segment SGI357771[] = {{&S96071[19], 16}, {NULL}};
const Segment SGI357772[] = {{&S96071[49], 3}, {NULL}};
const Segment SGI357773[] = {{&S83471[0], 16}, {NULL}};
const Segment SGI357774[] = {{&S83471[29], 3}, {NULL}};
const Segment SGI357775[] = {{&S83471[73], 16}, {NULL}};
const Segment SGI357776[] = {{&S96100[21], 14}, {NULL}};
const Segment SGI357777[] = {{&S96100[49], 3}, {NULL}};
const Segment SGI357778[] = {{&S83523[6], 4}, {NULL}};
const Segment SGI357779[] = {{&S83523[125], 5}, {NULL}};
const Segment SGI357780[] = {{&S83523[12], 12}, {&S83523[47], 6}, {NULL}};
const Segment SGI357781[] = {{&S83523[79], 8}, {NULL}};
const Segment SGI357782[] = {{&S83612[6], 4}, {NULL}};
const Segment SGI357783[] = {{&S83612[57], 5}, {NULL}};
const Segment SGI357784[] = {{&S83612[12], 10}, {NULL}};
const Segment SGI357785[] = {{&S83646[6], 4}, {NULL}};
const Segment SGI357786[] = {{&S83646[123], 5}, {NULL}};
const Segment SGI357787[] = {{&S83646[12], 10}, {&S83646[46], 6}, {NULL}};
const Segment SGI357788[] = {{&S83646[78], 7}, {NULL}};
const Segment SGI357789[] = {{&S83647[6], 4}, {NULL}};
const Segment SGI357790[] = {{&S83647[123], 5}, {NULL}};
const Segment SGI357791[] = {{&S83647[12], 10}, {&S83647[46], 6}, {NULL}};
const Segment SGI357792[] = {{&S83647[78], 7}, {NULL}};
const Segment SGI357793[] = {{&S83648[6], 4}, {NULL}};
const Segment SGI357794[] = {{&S83648[76], 5}, {NULL}};
const Segment SGI357795[] = {{&S83648[12], 10}, {&S83648[93], 11}, {NULL}};
const Segment SGI357796[] = {{&S83648[96], 39}, {NULL}};
const Segment SGI357797[] = {{&S83649[6], 4}, {NULL}};
const Segment SGI357798[] = {{&S83649[101], 5}, {NULL}};
const Segment SGI357799[] = {{&S83649[12], 10}, {&S83649[52], 19}, {NULL}};
const Segment SGI357800[] = {{&S83649[55], 16}, {NULL}};
const Segment SGI357801[] = {{&S83650[6], 4}, {NULL}};
const Segment SGI357802[] = {{&S83650[123], 5}, {NULL}};
const Segment SGI357803[] = {{&S83650[12], 10}, {&S83650[46], 6}, {NULL}};
const Segment SGI357804[] = {{&S83650[78], 7}, {NULL}};
const Segment SGI357805[] = {{&S83651[6], 4}, {NULL}};
const Segment SGI357806[] = {{&S83651[70], 5}, {NULL}};
const Segment SGI357807[] = {{&S83651[12], 10}, {NULL}};
const Segment SGI357808[] = {{&S83652[6], 4}, {NULL}};
const Segment SGI357809[] = {{&S83652[123], 5}, {NULL}};
const Segment SGI357810[] = {{&S83652[12], 10}, {&S83652[46], 6}, {NULL}};
const Segment SGI357811[] = {{&S83652[78], 7}, {NULL}};
const Segment SGI357812[] = {{&S83673[6], 4}, {NULL}};
const Segment SGI357813[] = {{&S83673[12], 9}, {NULL}};
const Segment SGI357814[] = {{&S85435[53], 4}, {NULL}};
const Segment SGI357815[] = {{&S85435[13], 10}, {NULL}};
const Segment SGI357816[] = {{&S85436[53], 3}, {NULL}};
const Segment SGI357817[] = {{&S85436[13], 10}, {NULL}};
const Segment SGI357818[] = {{&S83926[6], 4}, {NULL}};
const Segment SGI357819[] = {{&S83926[82], 5}, {NULL}};
const Segment SGI357820[] = {{&S83926[12], 11}, {NULL}};
const Segment SGI357821[] = {{&S83927[12], 13}, {NULL}};
const Segment SGI357822[] = {{&S83928[12], 13}, {NULL}};
const Segment SGI357823[] = {{&S84161[0], 15}, {NULL}};
const Segment SGI357824[] = {{&S84161[31], 3}, {NULL}};
const Segment SGI357825[] = {{&S96632[21], 14}, {NULL}};
const Segment SGI357826[] = {{&S96632[49], 3}, {NULL}};
const Segment SGI357827[] = {{&S96663[21], 15}, {NULL}};
const Segment SGI357828[] = {{&S96663[50], 3}, {NULL}};
const Segment SGI357829[] = {{&S84188[6], 4}, {NULL}};
const Segment SGI357830[] = {{&S84188[43], 4}, {NULL}};
const Segment SGI357831[] = {{&S84188[12], 14}, {NULL}};
const Segment SGI357832[] = {{&S84189[6], 4}, {NULL}};
const Segment SGI357833[] = {{&S84189[57], 5}, {NULL}};
const Segment SGI357834[] = {{&S84189[12], 14}, {NULL}};
const Segment SGI357835[] = {{&S96965[21], 15}, {NULL}};
const Segment SGI357836[] = {{&S96965[50], 3}, {NULL}};
const Segment SGI357837[] = {{&S97021[19], 17}, {NULL}};
const Segment SGI357838[] = {{&S97021[50], 3}, {NULL}};
const Segment SGI357839[] = {{&S97049[19], 13}, {NULL}};
const Segment SGI357840[] = {{&S97049[46], 3}, {NULL}};
const Segment SGI357841[] = {{&S80872[75], 5}, {NULL}};
const Segment SGI357842[] = {{&S80872[12], 3}, {NULL}};
const Segment SGI357843[] = {{&S80873[12], 14}, {NULL}};
const Segment SGI357844[] = {{&S84658[0], 15}, {NULL}};
const Segment SGI357845[] = {{&S84658[31], 3}, {NULL}};
const Segment SGI357846[] = {{&S97181[5], 5}, {NULL}};
const Segment SGI357847[] = {{&S97181[14], 3}, {NULL}};
const Segment SGI357848[] = {{&S97260[49], 8}, {NULL}};
const Segment SGI357849[] = {{&S97260[40], 3}, {NULL}};
const Segment SGI357850[] = {{&S97265[5], 8}, {NULL}};
const Segment SGI357851[] = {{&S97265[41], 3}, {NULL}};
const Segment SGI357852[] = {{&S97265[95], 3}, {NULL}};
const Segment SGI357853[] = {{&S97274[21], 15}, {NULL}};
const Segment SGI357854[] = {{&S97274[50], 3}, {NULL}};
const Segment SGI357855[] = {{&S97278[21], 15}, {NULL}};
const Segment SGI357856[] = {{&S97278[50], 3}, {NULL}};
const Segment SGI357857[] = {{&S97323[19], 15}, {NULL}};
const Segment SGI357858[] = {{&S97323[48], 3}, {NULL}};
const Segment SGI357859[] = {{&S97331[21], 15}, {NULL}};
const Segment SGI357860[] = {{&S97331[50], 3}, {NULL}};
const Segment SGI357861[] = {{&S97335[21], 15}, {NULL}};
const Segment SGI357862[] = {{&S97335[50], 3}, {NULL}};
const Segment SGI357863[] = {{&S84706[6], 4}, {NULL}};
const Segment SGI357864[] = {{&S84706[91], 5}, {NULL}};
const Segment SGI357865[] = {{&S84706[12], 9}, {NULL}};
const Segment SGI357866[] = {{&S84706[29], 13}, {NULL}};
const Segment SGI357867[] = {{&S97350[21], 14}, {NULL}};
const Segment SGI357868[] = {{&S97350[49], 3}, {NULL}};
const Segment SGI357869[] = {{&S97398[21], 15}, {NULL}};
const Segment SGI357870[] = {{&S97398[50], 3}, {NULL}};
const Segment SGI357871[] = {{&S84780[6], 4}, {NULL}};
const Segment SGI357872[] = {{&S84780[53], 5}, {NULL}};
const Segment SGI357873[] = {{&S84780[12], 10}, {NULL}};
const Segment SGI357874[] = {{&S84781[6], 4}, {NULL}};
const Segment SGI357875[] = {{&S84781[53], 5}, {NULL}};
const Segment SGI357876[] = {{&S84781[12], 10}, {NULL}};
const Segment SGI357877[] = {{&S97713[21], 15}, {NULL}};
const Segment SGI357878[] = {{&S97713[50], 3}, {NULL}};
const Segment SGI357879[] = {{&S97719[21], 14}, {NULL}};
const Segment SGI357880[] = {{&S97719[49], 3}, {NULL}};
const Segment SGI357881[] = {{&S85059[6], 4}, {NULL}};
const Segment SGI357882[] = {{&S85059[82], 5}, {NULL}};
const Segment SGI357883[] = {{&S85059[12], 11}, {NULL}};
const Segment SGI357884[] = {{&S85147[0], 21}, {NULL}};
const Segment SGI357885[] = {{&S85147[34], 3}, {NULL}};
const Segment SGI357886[] = {{&S85147[73], 13}, {NULL}};
const Segment SGI357887[] = {{&S85148[0], 21}, {NULL}};
const Segment SGI357888[] = {{&S85148[34], 3}, {NULL}};
const Segment SGI357889[] = {{&S85148[71], 13}, {NULL}};
const Segment SGI357890[] = {{&S85073[6], 4}, {NULL}};
const Segment SGI357891[] = {{&S85073[48], 5}, {NULL}};
const Segment SGI357892[] = {{&S85073[12], 41}, {NULL}};
const Segment SGI357893[] = {{&S97947[21], 14}, {NULL}};
const Segment SGI357894[] = {{&S97947[49], 3}, {NULL}};
const Segment SGI357895[] = {{&S85164[6], 4}, {NULL}};
const Segment SGI357896[] = {{&S85164[91], 5}, {NULL}};
const Segment SGI357897[] = {{&S85164[12], 9}, {NULL}};
const Segment SGI357898[] = {{&S85164[29], 13}, {NULL}};
const Segment SGI357899[] = {{&S85179[6], 4}, {NULL}};
const Segment SGI357900[] = {{&S85179[109], 5}, {NULL}};
const Segment SGI357901[] = {{&S85179[12], 11}, {NULL}};
const Segment SGI357902[] = {{&S85180[6], 4}, {NULL}};
const Segment SGI357903[] = {{&S85180[105], 5}, {NULL}};
const Segment SGI357904[] = {{&S85180[12], 10}, {NULL}};
const Segment SGI357905[] = {{&S98084[5], 4}, {NULL}};
const Segment SGI357906[] = {{&S98084[49], 3}, {NULL}};
const Segment SGI357907[] = {{&S98084[103], 3}, {NULL}};
const Segment SGI357908[] = {{&S85434[6], 4}, {NULL}};
const Segment SGI357909[] = {{&S85434[77], 5}, {NULL}};
const Segment SGI357910[] = {{&S85434[12], 9}, {NULL}};
const Segment SGI357911[] = {{&S85434[29], 13}, {NULL}};
const Segment SGI357912[] = {{&S85437[6], 4}, {NULL}};
const Segment SGI357913[] = {{&S85437[12], 10}, {NULL}};
const Segment SGI357914[] = {{&S85438[6], 4}, {NULL}};
const Segment SGI357915[] = {{&S85438[12], 10}, {NULL}};
const Segment SGI357916[] = {{&S85439[6], 4}, {NULL}};
const Segment SGI357917[] = {{&S85439[12], 10}, {NULL}};
const Segment SGI357918[] = {{&S85440[6], 4}, {NULL}};
const Segment SGI357919[] = {{&S85440[12], 10}, {NULL}};
const Segment SGI357920[] = {{&S85566[6], 4}, {NULL}};
const Segment SGI357921[] = {{&S85566[89], 5}, {NULL}};
const Segment SGI357922[] = {{&S85566[12], 9}, {NULL}};
const Segment SGI357923[] = {{&S85668[6], 4}, {NULL}};
const Segment SGI357924[] = {{&S85668[77], 5}, {NULL}};
const Segment SGI357925[] = {{&S85668[12], 9}, {NULL}};
const Segment SGI357926[] = {{&S85668[29], 13}, {NULL}};
const Segment SGI357927[] = {{&S85735[6], 4}, {NULL}};
const Segment SGI357928[] = {{&S85735[70], 5}, {NULL}};
const Segment SGI357929[] = {{&S85735[12], 9}, {NULL}};
const Segment SGI357930[] = {{&S85735[29], 13}, {NULL}};
const Segment SGI357931[] = {{&S98578[21], 15}, {NULL}};
const Segment SGI357932[] = {{&S98578[50], 3}, {NULL}};
const Segment SGI357933[] = {{&S85805[6], 4}, {NULL}};
const Segment SGI357934[] = {{&S85805[91], 5}, {NULL}};
const Segment SGI357935[] = {{&S85805[12], 9}, {NULL}};
const Segment SGI357936[] = {{&S85805[29], 13}, {NULL}};
const Segment SGI357937[] = {{&S86057[0], 21}, {NULL}};
const Segment SGI357938[] = {{&S86057[34], 3}, {NULL}};
const Segment SGI357939[] = {{&S86057[50], 3}, {NULL}};
const Segment SGI357940[] = {{&S86057[103], 13}, {NULL}};
const Segment SGI357941[] = {{&S86059[0], 21}, {NULL}};
const Segment SGI357942[] = {{&S86059[34], 3}, {NULL}};
const Segment SGI357943[] = {{&S86059[75], 13}, {NULL}};
const Segment SGI357944[] = {{&S86248[0], 21}, {NULL}};
const Segment SGI357945[] = {{&S86248[6], 3}, {NULL}};
const Segment SGI357946[] = {{&S86248[74], 13}, {NULL}};
const Segment SGI357947[] = {{&S99052[5], 4}, {NULL}};
const Segment SGI357948[] = {{&S99052[47], 3}, {NULL}};
const Segment SGI357949[] = {{&S99052[101], 3}, {NULL}};
const Segment SGI357950[] = {{&S99056[5], 5}, {NULL}};
const Segment SGI357951[] = {{&S99056[34], 3}, {NULL}};
const Segment SGI357952[] = {{&S86348[0], 16}, {NULL}};
const Segment SGI357953[] = {{&S86348[29], 3}, {NULL}};
const Segment SGI357954[] = {{&S86348[71], 9}, {NULL}};
const Segment SGI357955[] = {{&S99086[21], 14}, {NULL}};
const Segment SGI357956[] = {{&S99086[11], 3}, {NULL}};
const Segment SGI357957[] = {{&S99102[19], 13}, {NULL}};
const Segment SGI357958[] = {{&S99102[46], 3}, {NULL}};
const Segment SGI357959[] = {{&S99123[21], 15}, {NULL}};
const Segment SGI357960[] = {{&S99123[50], 3}, {NULL}};
const Segment SGI357961[] = {{&S86034[6], 4}, {NULL}};
const Segment SGI357962[] = {{&S86034[85], 5}, {NULL}};
const Segment SGI357963[] = {{&S86034[12], 13}, {NULL}};
const Segment SGI357964[] = {{&S86035[6], 4}, {NULL}};
const Segment SGI357965[] = {{&S86035[85], 5}, {NULL}};
const Segment SGI357966[] = {{&S86035[12], 13}, {NULL}};
const Segment SGI357967[] = {{&S86036[6], 4}, {NULL}};
const Segment SGI357968[] = {{&S86036[83], 5}, {NULL}};
const Segment SGI357969[] = {{&S86036[12], 11}, {NULL}};
const Segment SGI357970[] = {{&S86121[6], 4}, {NULL}};
const Segment SGI357971[] = {{&S86121[12], 7}, {NULL}};
const Segment SGI357972[] = {{&S86122[6], 4}, {NULL}};
const Segment SGI357973[] = {{&S86122[12], 7}, {NULL}};
const Segment SGI357974[] = {{&S86568[0], 21}, {NULL}};
const Segment SGI357975[] = {{&S86568[57], 13}, {NULL}};
const Segment SGI357976[] = {{&S86231[6], 4}, {NULL}};
const Segment SGI357977[] = {{&S86231[71], 5}, {NULL}};
const Segment SGI357978[] = {{&S86231[12], 3}, {NULL}};
const Segment SGI357979[] = {{&S86231[32], 10}, {NULL}};
const Segment SGI357980[] = {{&S86365[6], 4}, {NULL}};
const Segment SGI357981[] = {{&S86365[52], 5}, {NULL}};
const Segment SGI357982[] = {{&S86365[12], 45}, {NULL}};
const Segment SGI357983[] = {{&S86365[12], 14}, {NULL}};
const Segment SGI357984[] = {{&S86883[0], 15}, {NULL}};
const Segment SGI357985[] = {{&S86883[31], 3}, {NULL}};
const Segment SGI357986[] = {{&S86451[6], 4}, {NULL}};
const Segment SGI357987[] = {{&S86451[124], 5}, {NULL}};
const Segment SGI357988[] = {{&S86451[12], 12}, {&S86451[47], 6}, {NULL}};
const Segment SGI357989[] = {{&S86451[79], 8}, {NULL}};
const Segment SGI357990[] = {{&S86452[6], 4}, {NULL}};
const Segment SGI357991[] = {{&S86452[124], 5}, {NULL}};
const Segment SGI357992[] = {{&S86452[12], 12}, {&S86452[47], 6}, {&S86452[135], 4}, {NULL}};
const Segment SGI357993[] = {{&S86452[79], 8}, {NULL}};
const Segment SGI357994[] = {{&S99623[21], 15}, {NULL}};
const Segment SGI357995[] = {{&S99623[50], 3}, {NULL}};
const Segment SGI357996[] = {{&S99649[21], 15}, {NULL}};
const Segment SGI357997[] = {{&S99649[50], 3}, {NULL}};
const Segment SGI357998[] = {{&S99669[19], 14}, {NULL}};
const Segment SGI357999[] = {{&S99669[47], 3}, {NULL}};
const Segment SGI358000[] = {{&S99713[21], 14}, {NULL}};
const Segment SGI358001[] = {{&S99713[49], 3}, {NULL}};
const Segment SGI358002[] = {{&S99780[21], 14}, {NULL}};
const Segment SGI358003[] = {{&S99780[49], 3}, {NULL}};
const Segment SGI358004[] = {{&S83145[108], 5}, {NULL}};
const Segment SGI358005[] = {{&S83145[45], 3}, {NULL}};
const Segment SGI358006[] = {{&S86664[6], 4}, {NULL}};
const Segment SGI358007[] = {{&S86664[53], 5}, {NULL}};
const Segment SGI358008[] = {{&S86664[12], 10}, {NULL}};
const Segment SGI358009[] = {{&S86665[6], 4}, {NULL}};
const Segment SGI358010[] = {{&S86665[53], 5}, {NULL}};
const Segment SGI358011[] = {{&S86665[12], 10}, {NULL}};
const Segment SGI358012[] = {{&S86666[6], 4}, {NULL}};
const Segment SGI358013[] = {{&S86666[53], 5}, {NULL}};
const Segment SGI358014[] = {{&S86666[12], 10}, {NULL}};
const Segment SGI358015[] = {{&S100121[5], 4}, {NULL}};
const Segment SGI358016[] = {{&S100121[47], 3}, {NULL}};
const Segment SGI358017[] = {{&S100121[101], 3}, {NULL}};
const Segment SGI358018[] = {{&S100135[42], 3}, {&S100135[145], 7}, {NULL}};
const Segment SGI358019[] = {{&S100135[33], 3}, {NULL}};
const Segment SGI358020[] = {{&S100145[5], 4}, {NULL}};
const Segment SGI358021[] = {{&S100145[44], 3}, {NULL}};
const Segment SGI358022[] = {{&S100146[5], 4}, {NULL}};
const Segment SGI358023[] = {{&S100146[44], 3}, {NULL}};
const Segment SGI358024[] = {{&S100153[5], 4}, {NULL}};
const Segment SGI358025[] = {{&S100153[47], 3}, {NULL}};
const Segment SGI358026[] = {{&S100153[101], 3}, {NULL}};
const Segment SGI358027[] = {{&S100162[21], 13}, {NULL}};
const Segment SGI358028[] = {{&S100162[48], 3}, {NULL}};
const Segment SGI358029[] = {{&S104656[28], 5}, {NULL}};
const Segment SGI358030[] = {{&S104656[111], 5}, {NULL}};
const Segment SGI358031[] = {{&S87045[6], 4}, {NULL}};
const Segment SGI358032[] = {{&S87045[77], 5}, {NULL}};
const Segment SGI358033[] = {{&S87045[12], 14}, {NULL}};
const Segment SGI358034[] = {{&S87668[0], 21}, {NULL}};
const Segment SGI358035[] = {{&S87668[34], 3}, {NULL}};
const Segment SGI358036[] = {{&S100287[21], 15}, {NULL}};
const Segment SGI358037[] = {{&S100287[50], 3}, {NULL}};
const Segment SGI358038[] = {{&S100308[21], 13}, {NULL}};
const Segment SGI358039[] = {{&S100308[48], 3}, {NULL}};
const Segment SGI358040[] = {{&S100327[21], 15}, {NULL}};
const Segment SGI358041[] = {{&S100327[50], 3}, {NULL}};
const Segment SGI358042[] = {{&S83580[76], 5}, {NULL}};
const Segment SGI358043[] = {{&S83580[12], 14}, {NULL}};
const Segment SGI358044[] = {{&S83745[72], 5}, {NULL}};
const Segment SGI358045[] = {{&S83745[12], 14}, {NULL}};
const Segment SGI358046[] = {{&S105093[35], 7}, {NULL}};
const Segment SGI358047[] = {{&S105093[21], 7}, {NULL}};
const Segment SGI358048[] = {{&S105093[120], 5}, {NULL}};
const Segment SGI358049[] = {{&S105195[28], 7}, {NULL}};
const Segment SGI358050[] = {{&S105195[113], 5}, {NULL}};
const Segment SGI358051[] = {{&S105199[28], 5}, {NULL}};
const Segment SGI358052[] = {{&S105199[111], 5}, {NULL}};
const Segment SGI358053[] = {{&S105200[28], 7}, {NULL}};
const Segment SGI358054[] = {{&S105200[113], 5}, {NULL}};
const Segment SGI358055[] = {{&S100782[21], 14}, {NULL}};
const Segment SGI358056[] = {{&S100782[11], 3}, {NULL}};
const Segment SGI358057[] = {{&S100872[19], 13}, {NULL}};
const Segment SGI358058[] = {{&S100872[46], 3}, {NULL}};
const Segment SGI358059[] = {{&S100878[21], 17}, {NULL}};
const Segment SGI358060[] = {{&S100878[52], 3}, {NULL}};
const Segment SGI358061[] = {{&S105440[28], 5}, {NULL}};
const Segment SGI358062[] = {{&S105440[111], 5}, {NULL}};
const Segment SGI358063[] = {{&S89116[61], 5}, {NULL}};
const Segment SGI358064[] = {{&S89116[13], 10}, {&S89116[68], 8}, {NULL}};
const Segment SGI358065[] = {{&S89117[61], 5}, {NULL}};
const Segment SGI358066[] = {{&S89117[13], 10}, {NULL}};
const Segment SGI358067[] = {{&S88566[0], 15}, {NULL}};
const Segment SGI358068[] = {{&S88566[31], 3}, {NULL}};
const Segment SGI358069[] = {{&S87605[6], 4}, {NULL}};
const Segment SGI358070[] = {{&S87605[52], 5}, {NULL}};
const Segment SGI358071[] = {{&S87605[12], 45}, {NULL}};
const Segment SGI358072[] = {{&S87605[12], 14}, {NULL}};
const Segment SGI358073[] = {{&S100881[19], 14}, {NULL}};
const Segment SGI358074[] = {{&S100881[47], 3}, {NULL}};
const Segment SGI358075[] = {{&S100938[19], 15}, {NULL}};
const Segment SGI358076[] = {{&S100938[48], 3}, {NULL}};
const Segment SGI358077[] = {{&S100974[5], 4}, {NULL}};
const Segment SGI358078[] = {{&S100974[49], 3}, {NULL}};
const Segment SGI358079[] = {{&S100974[103], 3}, {NULL}};
const Segment SGI358080[] = {{&S101009[21], 14}, {NULL}};
const Segment SGI358081[] = {{&S101009[49], 3}, {NULL}};
const Segment SGI358082[] = {{&S101012[21], 14}, {NULL}};
const Segment SGI358083[] = {{&S101012[49], 3}, {NULL}};
const Segment SGI358084[] = {{&S87672[6], 4}, {NULL}};
const Segment SGI358085[] = {{&S87672[12], 9}, {NULL}};
const Segment SGI358086[] = {{&S87673[6], 4}, {NULL}};
const Segment SGI358087[] = {{&S87673[12], 14}, {NULL}};
const Segment SGI358088[] = {{&S87674[6], 4}, {NULL}};
const Segment SGI358089[] = {{&S87674[82], 5}, {NULL}};
const Segment SGI358090[] = {{&S87674[12], 11}, {NULL}};
const Segment SGI358091[] = {{&S87675[6], 4}, {NULL}};
const Segment SGI358092[] = {{&S87675[84], 5}, {NULL}};
const Segment SGI358093[] = {{&S87675[12], 13}, {NULL}};
const Segment SGI358094[] = {{&S87676[6], 4}, {NULL}};
const Segment SGI358095[] = {{&S87676[54], 5}, {NULL}};
const Segment SGI358096[] = {{&S87676[12], 10}, {NULL}};
const Segment SGI358097[] = {{&S101176[5], 5}, {NULL}};
const Segment SGI358098[] = {{&S101176[33], 3}, {NULL}};
const Segment SGI358099[] = {{&S101176[87], 3}, {NULL}};
const Segment SGI358100[] = {{&S101181[5], 4}, {NULL}};
const Segment SGI358101[] = {{&S101181[33], 3}, {NULL}};
const Segment SGI358102[] = {{&S101204[21], 15}, {NULL}};
const Segment SGI358103[] = {{&S101204[50], 3}, {NULL}};
const Segment SGI358104[] = {{&S101205[21], 25}, {NULL}};
const Segment SGI358105[] = {{&S101205[60], 3}, {NULL}};
const Segment SGI358106[] = {{&S101206[21], 26}, {NULL}};
const Segment SGI358107[] = {{&S101206[61], 3}, {NULL}};
const Segment SGI358108[] = {{&S106004[28], 4}, {NULL}};
const Segment SGI358109[] = {{&S106004[110], 5}, {NULL}};
const Segment SGI358110[] = {{&S87807[6], 4}, {NULL}};
const Segment SGI358111[] = {{&S87807[55], 5}, {NULL}};
const Segment SGI358112[] = {{&S87807[12], 10}, {NULL}};
const Segment SGI358113[] = {{&S87920[6], 4}, {NULL}};
const Segment SGI358114[] = {{&S87920[12], 14}, {NULL}};
const Segment SGI358115[] = {{&S87921[6], 4}, {NULL}};
const Segment SGI358116[] = {{&S87921[12], 14}, {NULL}};
const Segment SGI358117[] = {{&S85047[79], 4}, {NULL}};
const Segment SGI358118[] = {{&S85047[12], 12}, {NULL}};
const Segment SGI358119[] = {{&S101452[5], 8}, {NULL}};
const Segment SGI358120[] = {{&S101452[42], 3}, {NULL}};
const Segment SGI358121[] = {{&S88109[6], 4}, {NULL}};
const Segment SGI358122[] = {{&S88109[48], 5}, {NULL}};
const Segment SGI358123[] = {{&S88109[12], 41}, {NULL}};
const Segment SGI358124[] = {{&S101475[5], 4}, {NULL}};
const Segment SGI358125[] = {{&S101475[47], 3}, {NULL}};
const Segment SGI358126[] = {{&S101475[101], 3}, {NULL}};
const Segment SGI358127[] = {{&S101481[5], 4}, {NULL}};
const Segment SGI358128[] = {{&S101481[47], 3}, {NULL}};
const Segment SGI358129[] = {{&S101481[101], 3}, {NULL}};
const Segment SGI358130[] = {{&S101615[5], 8}, {NULL}};
const Segment SGI358131[] = {{&S101615[42], 3}, {NULL}};
const Segment SGI358132[] = {{&S101666[5], 4}, {NULL}};
const Segment SGI358133[] = {{&S101666[47], 3}, {NULL}};
const Segment SGI358134[] = {{&S101666[101], 3}, {NULL}};
const Segment SGI358135[] = {{&S101686[21], 14}, {NULL}};
const Segment SGI358136[] = {{&S101686[49], 3}, {NULL}};
const Segment SGI358137[] = {{&S101736[21], 15}, {NULL}};
const Segment SGI358138[] = {{&S101736[50], 3}, {NULL}};
const Segment SGI358139[] = {{&S89545[0], 15}, {NULL}};
const Segment SGI358140[] = {{&S89545[31], 3}, {NULL}};
const Segment SGI358141[] = {{&S101825[5], 4}, {NULL}};
const Segment SGI358142[] = {{&S101825[47], 3}, {NULL}};
const Segment SGI358143[] = {{&S101825[101], 3}, {NULL}};
const Segment SGI358144[] = {{&S85567[87], 5}, {NULL}};
const Segment SGI358145[] = {{&S85567[12], 9}, {NULL}};
const Segment SGI358146[] = {{&S88430[6], 4}, {NULL}};
const Segment SGI358147[] = {{&S88430[12], 14}, {NULL}};
const Segment SGI358148[] = {{&S88431[6], 4}, {NULL}};
const Segment SGI358149[] = {{&S88431[12], 14}, {NULL}};
const Segment SGI358150[] = {{&S102002[42], 3}, {&S102002[145], 7}, {NULL}};
const Segment SGI358151[] = {{&S102002[33], 3}, {NULL}};
const Segment SGI358152[] = {{&S102005[5], 4}, {NULL}};
const Segment SGI358153[] = {{&S102005[47], 3}, {NULL}};
const Segment SGI358154[] = {{&S102005[101], 3}, {NULL}};
const Segment SGI358155[] = {{&S102011[5], 9}, {NULL}};
const Segment SGI358156[] = {{&S102011[54], 3}, {NULL}};
const Segment SGI358157[] = {{&S102025[5], 4}, {NULL}};
const Segment SGI358158[] = {{&S102025[49], 3}, {NULL}};
const Segment SGI358159[] = {{&S102025[103], 3}, {NULL}};
const Segment SGI358160[] = {{&S102089[21], 15}, {NULL}};
const Segment SGI358161[] = {{&S102089[50], 3}, {NULL}};
const Segment SGI358162[] = {{&S89687[0], 15}, {NULL}};
const Segment SGI358163[] = {{&S89687[31], 3}, {NULL}};
const Segment SGI358164[] = {{&S102128[21], 14}, {NULL}};
const Segment SGI358165[] = {{&S102128[49], 3}, {NULL}};
const Segment SGI358166[] = {{&S88726[6], 4}, {NULL}};
const Segment SGI358167[] = {{&S88726[121], 5}, {NULL}};
const Segment SGI358168[] = {{&S88726[12], 10}, {&S88726[46], 6}, {NULL}};
const Segment SGI358169[] = {{&S88726[78], 7}, {NULL}};
const Segment SGI358170[] = {{&S88738[6], 4}, {NULL}};
const Segment SGI358171[] = {{&S88738[59], 5}, {NULL}};
const Segment SGI358172[] = {{&S88738[12], 14}, {NULL}};
const Segment SGI358173[] = {{&S102194[5], 4}, {NULL}};
const Segment SGI358174[] = {{&S102194[47], 3}, {NULL}};
const Segment SGI358175[] = {{&S102194[101], 3}, {NULL}};
const Segment SGI358176[] = {{&S102235[21], 23}, {NULL}};
const Segment SGI358177[] = {{&S102235[58], 3}, {NULL}};
const Segment SGI358178[] = {{&S102236[21], 23}, {NULL}};
const Segment SGI358179[] = {{&S102236[58], 3}, {NULL}};
const Segment SGI358180[] = {{&S90763[54], 3}, {NULL}};
const Segment SGI358181[] = {{&S90763[13], 10}, {NULL}};
const Segment SGI358182[] = {{&S88842[12], 13}, {NULL}};
const Segment SGI358183[] = {{&S88843[6], 4}, {NULL}};
const Segment SGI358184[] = {{&S88843[81], 5}, {NULL}};
const Segment SGI358185[] = {{&S88843[12], 11}, {NULL}};
const Segment SGI358186[] = {{&S89002[6], 4}, {NULL}};
const Segment SGI358187[] = {{&S89002[124], 5}, {NULL}};
const Segment SGI358188[] = {{&S89002[12], 12}, {&S89002[47], 6}, {NULL}};
const Segment SGI358189[] = {{&S89002[79], 8}, {NULL}};
const Segment SGI358190[] = {{&S102394[5], 4}, {NULL}};
const Segment SGI358191[] = {{&S102394[33], 3}, {NULL}};
const Segment SGI358192[] = {{&S89112[6], 4}, {NULL}};
const Segment SGI358193[] = {{&S89112[55], 5}, {NULL}};
const Segment SGI358194[] = {{&S89112[12], 10}, {NULL}};
const Segment SGI358195[] = {{&S89137[6], 4}, {NULL}};
const Segment SGI358196[] = {{&S89137[121], 5}, {NULL}};
const Segment SGI358197[] = {{&S89137[12], 10}, {&S89137[46], 6}, {NULL}};
const Segment SGI358198[] = {{&S89137[78], 7}, {NULL}};
const Segment SGI358199[] = {{&S89138[6], 4}, {NULL}};
const Segment SGI358200[] = {{&S89138[121], 5}, {NULL}};
const Segment SGI358201[] = {{&S89138[12], 10}, {&S89138[46], 6}, {NULL}};
const Segment SGI358202[] = {{&S89138[78], 7}, {NULL}};
const Segment SGI358203[] = {{&S89139[6], 4}, {NULL}};
const Segment SGI358204[] = {{&S89139[121], 5}, {NULL}};
const Segment SGI358205[] = {{&S89139[12], 10}, {&S89139[46], 6}, {NULL}};
const Segment SGI358206[] = {{&S102490[19], 15}, {NULL}};
const Segment SGI358207[] = {{&S102490[48], 3}, {NULL}};
const Segment SGI358208[] = {{&S89139[78], 7}, {NULL}};
const Segment SGI358209[] = {{&S102591[5], 3}, {NULL}};
const Segment SGI358210[] = {{&S102591[31], 3}, {NULL}};
const Segment SGI358211[] = {{&S102591[85], 3}, {NULL}};
const Segment SGI358212[] = {{&S102608[5], 8}, {NULL}};
const Segment SGI358213[] = {{&S102608[41], 3}, {NULL}};
const Segment SGI358214[] = {{&S102608[95], 3}, {NULL}};
const Segment SGI358215[] = {{&S102643[21], 17}, {NULL}};
const Segment SGI358216[] = {{&S102643[52], 3}, {NULL}};
const Segment SGI358217[] = {{&S102644[21], 17}, {NULL}};
const Segment SGI358218[] = {{&S102644[52], 3}, {NULL}};
const Segment SGI358219[] = {{&S102658[21], 15}, {NULL}};
const Segment SGI358220[] = {{&S102658[50], 3}, {NULL}};
const Segment SGI358221[] = {{&S102660[21], 14}, {NULL}};
const Segment SGI358222[] = {{&S102660[49], 3}, {NULL}};
const Segment SGI358223[] = {{&S86422[79], 5}, {NULL}};
const Segment SGI358224[] = {{&S86422[12], 12}, {NULL}};
const Segment SGI358225[] = {{&S102722[21], 15}, {NULL}};
const Segment SGI358226[] = {{&S102722[11], 3}, {NULL}};
const Segment SGI358227[] = {{&S102738[5], 8}, {NULL}};
const Segment SGI358228[] = {{&S102738[41], 3}, {NULL}};
const Segment SGI358229[] = {{&S102739[5], 6}, {NULL}};
const Segment SGI358230[] = {{&S102739[46], 3}, {NULL}};
const Segment SGI358231[] = {{&S89281[6], 4}, {NULL}};
const Segment SGI358232[] = {{&S89281[52], 5}, {NULL}};
const Segment SGI358233[] = {{&S89281[12], 45}, {NULL}};
const Segment SGI358234[] = {{&S89281[12], 14}, {NULL}};
const Segment SGI358235[] = {{&S89288[6], 4}, {NULL}};
const Segment SGI358236[] = {{&S89288[125], 5}, {NULL}};
const Segment SGI358237[] = {{&S89288[12], 12}, {&S89288[47], 6}, {NULL}};
const Segment SGI358238[] = {{&S89288[79], 8}, {NULL}};
const Segment SGI358239[] = {{&S102815[5], 5}, {NULL}};
const Segment SGI358240[] = {{&S102815[34], 3}, {NULL}};
const Segment SGI358241[] = {{&S102849[21], 15}, {NULL}};
const Segment SGI358242[] = {{&S102849[50], 3}, {NULL}};
const Segment SGI358243[] = {{&S102886[21], 25}, {NULL}};
const Segment SGI358244[] = {{&S102886[60], 3}, {NULL}};
const Segment SGI358245[] = {{&S89431[6], 4}, {NULL}};
const Segment SGI358246[] = {{&S89431[121], 5}, {NULL}};
const Segment SGI358247[] = {{&S89431[12], 10}, {&S89431[46], 6}, {NULL}};
const Segment SGI358248[] = {{&S89431[78], 7}, {NULL}};
const Segment SGI358249[] = {{&S89463[6], 4}, {NULL}};
const Segment SGI358250[] = {{&S89463[121], 5}, {NULL}};
const Segment SGI358251[] = {{&S89463[12], 10}, {&S89463[46], 6}, {NULL}};
const Segment SGI358252[] = {{&S89463[78], 7}, {NULL}};
const Segment SGI358253[] = {{&S103080[21], 14}, {NULL}};
const Segment SGI358254[] = {{&S103080[49], 3}, {NULL}};
const Segment SGI358255[] = {{&S89589[6], 4}, {NULL}};
const Segment SGI358256[] = {{&S89589[121], 5}, {NULL}};
const Segment SGI358257[] = {{&S89589[12], 10}, {&S89589[46], 6}, {NULL}};
const Segment SGI358258[] = {{&S89589[78], 7}, {NULL}};
const Segment SGI358259[] = {{&S89597[6], 4}, {NULL}};
const Segment SGI358260[] = {{&S89597[124], 5}, {NULL}};
const Segment SGI358261[] = {{&S89597[12], 12}, {&S89597[47], 6}, {NULL}};
const Segment SGI358262[] = {{&S89597[79], 8}, {NULL}};
const Segment SGI358263[] = {{&S103196[21], 15}, {NULL}};
const Segment SGI358264[] = {{&S103196[11], 3}, {NULL}};
const Segment SGI358265[] = {{&S103278[5], 4}, {NULL}};
const Segment SGI358266[] = {{&S103278[40], 3}, {NULL}};
const Segment SGI358267[] = {{&S89730[6], 4}, {NULL}};
const Segment SGI358268[] = {{&S89730[12], 3}, {NULL}};
const Segment SGI358269[] = {{&S103309[5], 4}, {NULL}};
const Segment SGI358270[] = {{&S103309[49], 3}, {NULL}};
const Segment SGI358271[] = {{&S103309[103], 3}, {NULL}};
const Segment SGI358272[] = {{&S89751[6], 4}, {NULL}};
const Segment SGI358273[] = {{&S89751[68], 5}, {NULL}};
const Segment SGI358274[] = {{&S89751[12], 3}, {NULL}};
const Segment SGI358275[] = {{&S89770[6], 4}, {NULL}};
const Segment SGI358276[] = {{&S89770[12], 14}, {NULL}};
const Segment SGI358277[] = {{&S89771[6], 4}, {NULL}};
const Segment SGI358278[] = {{&S89771[12], 14}, {NULL}};
const Segment SGI358279[] = {{&S103346[5], 7}, {&S103346[159], 7}, {NULL}};
const Segment SGI358280[] = {{&S103346[58], 3}, {NULL}};
const Segment SGI358281[] = {{&S103353[5], 5}, {NULL}};
const Segment SGI358282[] = {{&S103353[34], 3}, {NULL}};
const Segment SGI358283[] = {{&S89885[6], 4}, {NULL}};
const Segment SGI358284[] = {{&S89885[77], 5}, {NULL}};
const Segment SGI358285[] = {{&S89885[12], 9}, {NULL}};
const Segment SGI358286[] = {{&S89885[29], 13}, {NULL}};
const Segment SGI358287[] = {{&S89918[6], 4}, {NULL}};
const Segment SGI358288[] = {{&S89918[48], 5}, {NULL}};
const Segment SGI358289[] = {{&S89918[12], 41}, {NULL}};
const Segment SGI358290[] = {{&S90049[6], 4}, {&S90049[6], 4}, {NULL}};
const Segment SGI358291[] = {{&S90049[12], 14}, {&S90049[43], 3}, {NULL}};
const Segment SGI358292[] = {{&S90049[12], 14}, {NULL}};
const Segment SGI358293[] = {{&S103624[5], 8}, {NULL}};
const Segment SGI358294[] = {{&S103624[41], 3}, {NULL}};
const Segment SGI358295[] = {{&S103624[95], 3}, {NULL}};
const Segment SGI358296[] = {{&S103627[5], 4}, {NULL}};
const Segment SGI358297[] = {{&S103627[47], 3}, {NULL}};
const Segment SGI358298[] = {{&S103627[101], 3}, {NULL}};
const Segment SGI358299[] = {{&S103629[5], 15}, {NULL}};
const Segment SGI358300[] = {{&S103629[44], 3}, {NULL}};
const Segment SGI358301[] = {{&S103670[21], 14}, {NULL}};
const Segment SGI358302[] = {{&S103670[49], 3}, {NULL}};
const Segment SGI358303[] = {{&S90188[6], 3}, {NULL}};
const Segment SGI358304[] = {{&S90188[11], 14}, {NULL}};
const Segment SGI358305[] = {{&S103841[5], 5}, {NULL}};
const Segment SGI358306[] = {{&S103841[39], 3}, {NULL}};
const Segment SGI358307[] = {{&S90402[6], 4}, {NULL}};
const Segment SGI358308[] = {{&S90402[55], 5}, {NULL}};
const Segment SGI358309[] = {{&S90402[12], 10}, {NULL}};
const Segment SGI358310[] = {{&S103919[5], 4}, {NULL}};
const Segment SGI358311[] = {{&S103919[49], 3}, {NULL}};
const Segment SGI358312[] = {{&S103919[103], 3}, {NULL}};
const Segment SGI358313[] = {{&S103923[5], 4}, {NULL}};
const Segment SGI358314[] = {{&S103923[47], 3}, {NULL}};
const Segment SGI358315[] = {{&S103923[101], 3}, {NULL}};
const Segment SGI358316[] = {{&S103928[5], 4}, {NULL}};
const Segment SGI358317[] = {{&S103928[47], 3}, {NULL}};
const Segment SGI358318[] = {{&S103928[101], 3}, {NULL}};
const Segment SGI358319[] = {{&S103992[21], 14}, {NULL}};
const Segment SGI358320[] = {{&S103992[49], 3}, {NULL}};
const Segment SGI358321[] = {{&S91590[0], 21}, {NULL}};
const Segment SGI358322[] = {{&S90561[6], 4}, {NULL}};
const Segment SGI358323[] = {{&S90561[12], 3}, {NULL}};
const Segment SGI358324[] = {{&S90565[6], 4}, {NULL}};
const Segment SGI358325[] = {{&S90565[83], 5}, {NULL}};
const Segment SGI358326[] = {{&S90565[12], 11}, {NULL}};
const Segment SGI358327[] = {{&S90566[6], 4}, {NULL}};
const Segment SGI358328[] = {{&S90566[91], 5}, {NULL}};
const Segment SGI358329[] = {{&S90566[12], 19}, {NULL}};
const Segment SGI358330[] = {{&S90567[6], 4}, {NULL}};
const Segment SGI358331[] = {{&S90567[76], 5}, {NULL}};
const Segment SGI358332[] = {{&S90567[12], 11}, {NULL}};
const Segment SGI358333[] = {{&S104238[5], 4}, {NULL}};
const Segment SGI358334[] = {{&S104238[49], 3}, {NULL}};
const Segment SGI358335[] = {{&S104238[15], 3}, {NULL}};
const Segment SGI358336[] = {{&S104239[5], 4}, {NULL}};
const Segment SGI358337[] = {{&S104239[38], 3}, {NULL}};
const Segment SGI358338[] = {{&S90764[6], 4}, {NULL}};
const Segment SGI358339[] = {{&S90764[12], 3}, {NULL}};
const Segment SGI358340[] = {{&S90770[12], 13}, {NULL}};
const Segment SGI358341[] = {{&S104275[5], 5}, {NULL}};
const Segment SGI358342[] = {{&S104275[34], 3}, {NULL}};
const Segment SGI358343[] = {{&S90771[12], 13}, {NULL}};
const Segment SGI358344[] = {{&S90772[6], 4}, {NULL}};
const Segment SGI358345[] = {{&S90772[82], 5}, {NULL}};
const Segment SGI358346[] = {{&S90772[12], 11}, {NULL}};
const Segment SGI358347[] = {{&S90778[6], 4}, {NULL}};
const Segment SGI358348[] = {{&S90778[121], 5}, {NULL}};
const Segment SGI358349[] = {{&S90778[12], 10}, {&S90778[46], 6}, {NULL}};
const Segment SGI358350[] = {{&S90778[78], 7}, {NULL}};
const Segment SGI358351[] = {{&S90779[6], 4}, {NULL}};
const Segment SGI358352[] = {{&S90779[121], 5}, {NULL}};
const Segment SGI358353[] = {{&S90779[12], 10}, {&S90779[46], 6}, {NULL}};
const Segment SGI358354[] = {{&S90779[78], 7}, {NULL}};
const Segment SGI358355[] = {{&S91883[0], 21}, {NULL}};
const Segment SGI358356[] = {{&S91883[34], 3}, {NULL}};
const Segment SGI358357[] = {{&S91883[73], 13}, {NULL}};
const Segment SGI358358[] = {{&S91884[0], 21}, {NULL}};
const Segment SGI358359[] = {{&S91884[34], 3}, {NULL}};
const Segment SGI358360[] = {{&S91884[71], 13}, {NULL}};
const Segment SGI358361[] = {{&S92464[6], 4}, {NULL}};
const Segment SGI358362[] = {{&S92464[12], 14}, {NULL}};
const Segment SGI358363[] = {{&S90831[6], 4}, {NULL}};
const Segment SGI358364[] = {{&S90831[125], 5}, {NULL}};
const Segment SGI358365[] = {{&S90831[12], 12}, {&S90831[47], 6}, {NULL}};
const Segment SGI358366[] = {{&S90831[79], 8}, {NULL}};
const Segment SGI358367[] = {{&S104360[21], 13}, {NULL}};
const Segment SGI358368[] = {{&S104360[48], 3}, {NULL}};
const Segment SGI358369[] = {{&S90978[6], 4}, {NULL}};
const Segment SGI358370[] = {{&S90978[52], 5}, {NULL}};
const Segment SGI358371[] = {{&S90978[12], 45}, {NULL}};
const Segment SGI358372[] = {{&S90978[12], 14}, {NULL}};
const Segment SGI358373[] = {{&S90986[6], 4}, {NULL}};
const Segment SGI358374[] = {{&S90986[54], 5}, {NULL}};
const Segment SGI358375[] = {{&S90986[12], 11}, {NULL}};
const Segment SGI358376[] = {{&S91122[6], 4}, {NULL}};
const Segment SGI358377[] = {{&S91122[12], 9}, {NULL}};
const Segment SGI358378[] = {{&S104699[5], 7}, {NULL}};
const Segment SGI358379[] = {{&S104699[36], 3}, {NULL}};
const Segment SGI358380[] = {{&S104700[5], 3}, {NULL}};
const Segment SGI358381[] = {{&S104700[31], 3}, {NULL}};
const Segment SGI358382[] = {{&S104700[80], 3}, {NULL}};
const Segment SGI358383[] = {{&S104705[5], 4}, {NULL}};
const Segment SGI358384[] = {{&S104705[47], 3}, {NULL}};
const Segment SGI358385[] = {{&S104705[101], 3}, {NULL}};
const Segment SGI358386[] = {{&S104727[5], 4}, {NULL}};
const Segment SGI358387[] = {{&S104727[37], 3}, {NULL}};
const Segment SGI358388[] = {{&S104763[57], 24}, {NULL}};
const Segment SGI358389[] = {{&S104763[8], 3}, {NULL}};
const Segment SGI358390[] = {{&S92865[53], 4}, {NULL}};
const Segment SGI358391[] = {{&S92865[13], 10}, {NULL}};
const Segment SGI358392[] = {{&S104841[21], 17}, {NULL}};
const Segment SGI358393[] = {{&S104841[52], 3}, {NULL}};
const Segment SGI358394[] = {{&S93069[61], 3}, {NULL}};
const Segment SGI358395[] = {{&S93069[13], 10}, {NULL}};
const Segment SGI358396[] = {{&S88986[70], 4}, {NULL}};
const Segment SGI358397[] = {{&S88986[12], 10}, {NULL}};
const Segment SGI358398[] = {{&S92896[0], 21}, {NULL}};
const Segment SGI358399[] = {{&S92896[34], 3}, {NULL}};
const Segment SGI358400[] = {{&S92896[75], 13}, {NULL}};
const Segment SGI358401[] = {{&S93315[36], 3}, {NULL}};
const Segment SGI358402[] = {{&S93315[13], 12}, {NULL}};
const Segment SGI358403[] = {{&S91515[6], 4}, {NULL}};
const Segment SGI358404[] = {{&S91515[55], 5}, {NULL}};
const Segment SGI358405[] = {{&S91515[12], 10}, {NULL}};
const Segment SGI358406[] = {{&S105165[53], 4}, {NULL}};
const Segment SGI358407[] = {{&S105165[44], 3}, {NULL}};
const Segment SGI358408[] = {{&S93045[0], 15}, {NULL}};
const Segment SGI358409[] = {{&S93045[31], 3}, {NULL}};
const Segment SGI358410[] = {{&S91552[6], 4}, {NULL}};
const Segment SGI358411[] = {{&S91552[48], 5}, {NULL}};
const Segment SGI358412[] = {{&S91552[12], 41}, {NULL}};
const Segment SGI358413[] = {{&S105255[5], 8}, {NULL}};
const Segment SGI358414[] = {{&S105255[41], 3}, {NULL}};
const Segment SGI358415[] = {{&S105255[95], 3}, {NULL}};
const Segment SGI358416[] = {{&S105282[5], 4}, {NULL}};
const Segment SGI358417[] = {{&S105282[47], 3}, {NULL}};
const Segment SGI358418[] = {{&S105282[101], 3}, {NULL}};
const Segment SGI358419[] = {{&S105326[5], 4}, {NULL}};
const Segment SGI358420[] = {{&S105326[33], 3}, {NULL}};
const Segment SGI358421[] = {{&S105360[5], 4}, {NULL}};
const Segment SGI358422[] = {{&S105360[37], 3}, {NULL}};
const Segment SGI358423[] = {{&S91891[6], 4}, {NULL}};
const Segment SGI358424[] = {{&S91891[74], 5}, {NULL}};
const Segment SGI358425[] = {{&S91891[12], 3}, {NULL}};
const Segment SGI358426[] = {{&S93493[0], 15}, {NULL}};
const Segment SGI358427[] = {{&S93493[31], 3}, {NULL}};
const Segment SGI358428[] = {{&S92120[6], 4}, {NULL}};
const Segment SGI358429[] = {{&S92120[83], 5}, {NULL}};
const Segment SGI358430[] = {{&S92120[12], 13}, {NULL}};
const Segment SGI358431[] = {{&S92127[6], 4}, {NULL}};
const Segment SGI358432[] = {{&S92127[57], 5}, {NULL}};
const Segment SGI358433[] = {{&S92127[12], 14}, {NULL}};
const Segment SGI358434[] = {{&S92176[6], 4}, {NULL}};
const Segment SGI358435[] = {{&S92176[83], 5}, {NULL}};
const Segment SGI358436[] = {{&S92176[12], 18}, {NULL}};
const Segment SGI358437[] = {{&S92177[6], 4}, {NULL}};
const Segment SGI358438[] = {{&S92177[77], 5}, {NULL}};
const Segment SGI358439[] = {{&S92177[12], 12}, {NULL}};
const Segment SGI358440[] = {{&S92301[6], 4}, {NULL}};
const Segment SGI358441[] = {{&S92301[12], 16}, {NULL}};
const Segment SGI358442[] = {{&S92403[6], 4}, {NULL}};
const Segment SGI358443[] = {{&S92403[85], 5}, {NULL}};
const Segment SGI358444[] = {{&S92403[12], 13}, {NULL}};
const Segment SGI358445[] = {{&S92689[6], 4}, {NULL}};
const Segment SGI358446[] = {{&S92689[57], 5}, {NULL}};
const Segment SGI358447[] = {{&S92689[12], 14}, {NULL}};
const Segment SGI358448[] = {{&S94042[0], 21}, {NULL}};
const Segment SGI358449[] = {{&S94042[34], 3}, {NULL}};
const Segment SGI358450[] = {{&S92856[6], 4}, {NULL}};
const Segment SGI358451[] = {{&S92856[12], 10}, {NULL}};
const Segment SGI358452[] = {{&S92857[6], 4}, {NULL}};
const Segment SGI358453[] = {{&S92857[12], 10}, {NULL}};
const Segment SGI358454[] = {{&S92868[6], 4}, {NULL}};
const Segment SGI358455[] = {{&S92868[87], 5}, {NULL}};
const Segment SGI358456[] = {{&S92868[12], 13}, {NULL}};
const Segment SGI358457[] = {{&S92902[6], 4}, {NULL}};
const Segment SGI358458[] = {{&S92902[73], 5}, {NULL}};
const Segment SGI358459[] = {{&S92902[12], 9}, {NULL}};
const Segment SGI358460[] = {{&S92902[27], 13}, {NULL}};
const Segment SGI358461[] = {{&S94680[61], 5}, {NULL}};
const Segment SGI358462[] = {{&S94680[13], 10}, {NULL}};
const Segment SGI358463[] = {{&S93066[6], 4}, {NULL}};
const Segment SGI358464[] = {{&S93066[54], 5}, {NULL}};
const Segment SGI358465[] = {{&S93066[12], 10}, {NULL}};
const Segment SGI358466[] = {{&S93314[6], 3}, {NULL}};
const Segment SGI358467[] = {{&S93314[11], 12}, {NULL}};
const Segment SGI358468[] = {{&S93321[6], 4}, {NULL}};
const Segment SGI358469[] = {{&S93321[87], 5}, {NULL}};
const Segment SGI358470[] = {{&S93321[12], 13}, {NULL}};
const Segment SGI358471[] = {{&S93334[6], 4}, {NULL}};
const Segment SGI358472[] = {{&S93334[121], 5}, {NULL}};
const Segment SGI358473[] = {{&S93334[12], 10}, {&S93334[46], 6}, {NULL}};
const Segment SGI358474[] = {{&S93334[78], 7}, {NULL}};
const Segment SGI358475[] = {{&S93343[6], 4}, {NULL}};
const Segment SGI358476[] = {{&S93343[55], 5}, {NULL}};
const Segment SGI358477[] = {{&S93343[12], 10}, {NULL}};
const Segment SGI358478[] = {{&S95105[0], 15}, {NULL}};
const Segment SGI358479[] = {{&S95105[31], 3}, {NULL}};
const Segment SGI358480[] = {{&S91144[64], 5}, {NULL}};
const Segment SGI358481[] = {{&S91144[12], 5}, {NULL}};
const Segment SGI358482[] = {{&S93555[6], 4}, {NULL}};
const Segment SGI358483[] = {{&S93555[121], 5}, {NULL}};
const Segment SGI358484[] = {{&S93555[12], 10}, {&S93555[46], 6}, {NULL}};
const Segment SGI358485[] = {{&S93555[78], 7}, {NULL}};
const Segment SGI358486[] = {{&S93652[6], 4}, {NULL}};
const Segment SGI358487[] = {{&S93652[123], 5}, {NULL}};
const Segment SGI358488[] = {{&S93652[12], 10}, {&S93652[46], 6}, {NULL}};
const Segment SGI358489[] = {{&S93652[78], 7}, {NULL}};
const Segment SGI358490[] = {{&S93798[6], 4}, {NULL}};
const Segment SGI358491[] = {{&S93798[58], 5}, {NULL}};
const Segment SGI358492[] = {{&S93798[12], 3}, {NULL}};
const Segment SGI358493[] = {{&S93835[6], 4}, {NULL}};
const Segment SGI358494[] = {{&S93835[123], 5}, {NULL}};
const Segment SGI358495[] = {{&S93835[12], 10}, {&S93835[46], 6}, {NULL}};
const Segment SGI358496[] = {{&S93835[78], 7}, {NULL}};
const Segment SGI358497[] = {{&S93853[6], 4}, {NULL}};
const Segment SGI358498[] = {{&S93853[48], 5}, {NULL}};
const Segment SGI358499[] = {{&S93853[12], 41}, {NULL}};
const Segment SGI358500[] = {{&S93889[6], 4}, {NULL}};
const Segment SGI358501[] = {{&S93889[12], 3}, {NULL}};
const Segment SGI358502[] = {{&S93984[6], 4}, {NULL}};
const Segment SGI358503[] = {{&S93984[12], 10}, {NULL}};
const Segment SGI358504[] = {{&S91712[72], 5}, {NULL}};
const Segment SGI358505[] = {{&S91712[12], 14}, {NULL}};
const Segment SGI358506[] = {{&S94060[6], 4}, {NULL}};
const Segment SGI358507[] = {{&S94060[123], 5}, {NULL}};
const Segment SGI358508[] = {{&S94060[12], 10}, {&S94060[46], 6}, {NULL}};
const Segment SGI358509[] = {{&S94060[78], 7}, {NULL}};
const Segment SGI358510[] = {{&S94066[6], 4}, {NULL}};
const Segment SGI358511[] = {{&S94066[48], 5}, {NULL}};
const Segment SGI358512[] = {{&S94066[12], 41}, {NULL}};
const Segment SGI358513[] = {{&S95946[79], 4}, {NULL}};
const Segment SGI358514[] = {{&S95946[34], 10}, {NULL}};
const Segment SGI358515[] = {{&S94221[6], 4}, {NULL}};
const Segment SGI358516[] = {{&S94221[12], 3}, {NULL}};
const Segment SGI358517[] = {{&S94241[6], 4}, {NULL}};
const Segment SGI358518[] = {{&S94241[12], 14}, {NULL}};
const Segment SGI358519[] = {{&S94276[6], 4}, {NULL}};
const Segment SGI358520[] = {{&S94276[125], 5}, {NULL}};
const Segment SGI358521[] = {{&S94276[12], 12}, {&S94276[47], 6}, {NULL}};
const Segment SGI358522[] = {{&S94276[79], 8}, {NULL}};
const Segment SGI358523[] = {{&S92093[75], 5}, {NULL}};
const Segment SGI358524[] = {{&S92093[12], 14}, {NULL}};
const Segment SGI358525[] = {{&S94682[6], 4}, {NULL}};
const Segment SGI358526[] = {{&S94682[12], 12}, {NULL}};
const Segment SGI358527[] = {{&S94685[6], 4}, {NULL}};
const Segment SGI358528[] = {{&S94685[73], 5}, {NULL}};
const Segment SGI358529[] = {{&S94685[12], 3}, {NULL}};
const Segment SGI358530[] = {{&S96320[55], 4}, {NULL}};
const Segment SGI358531[] = {{&S96320[13], 10}, {NULL}};
const Segment SGI358532[] = {{&S94722[6], 4}, {NULL}};
const Segment SGI358533[] = {{&S94722[123], 5}, {NULL}};
const Segment SGI358534[] = {{&S94722[12], 10}, {&S94722[46], 6}, {NULL}};
const Segment SGI358535[] = {{&S94722[78], 7}, {NULL}};
const Segment SGI358536[] = {{&S96272[0], 15}, {NULL}};
const Segment SGI358537[] = {{&S96272[31], 3}, {NULL}};
const Segment SGI358538[] = {{&S96277[0], 15}, {NULL}};
const Segment SGI358539[] = {{&S96277[31], 3}, {NULL}};
const Segment SGI358540[] = {{&S92536[221], 5}, {NULL}};
const Segment SGI358541[] = {{&S92536[158], 3}, {NULL}};
const Segment SGI358542[] = {{&S96625[57], 4}, {NULL}};
const Segment SGI358543[] = {{&S96625[13], 10}, {NULL}};
const Segment SGI358544[] = {{&S94902[6], 4}, {NULL}};
const Segment SGI358545[] = {{&S94902[73], 5}, {NULL}};
const Segment SGI358546[] = {{&S94902[12], 3}, {NULL}};
const Segment SGI358547[] = {{&S94919[6], 4}, {NULL}};
const Segment SGI358548[] = {{&S94919[12], 14}, {NULL}};
const Segment SGI358549[] = {{&S94942[6], 4}, {NULL}};
const Segment SGI358550[] = {{&S94942[12], 3}, {&S94942[37], 7}, {NULL}};
const Segment SGI358551[] = {{&S94961[6], 4}, {NULL}};
const Segment SGI358552[] = {{&S94961[76], 5}, {NULL}};
const Segment SGI358553[] = {{&S94961[12], 9}, {NULL}};
const Segment SGI358554[] = {{&S94961[29], 13}, {NULL}};
const Segment SGI358555[] = {{&S94962[6], 4}, {NULL}};
const Segment SGI358556[] = {{&S94962[52], 5}, {NULL}};
const Segment SGI358557[] = {{&S94962[12], 45}, {NULL}};
const Segment SGI358558[] = {{&S94962[12], 14}, {NULL}};
const Segment SGI358559[] = {{&S94977[6], 4}, {NULL}};
const Segment SGI358560[] = {{&S94977[123], 5}, {NULL}};
const Segment SGI358561[] = {{&S94977[12], 10}, {&S94977[46], 6}, {NULL}};
const Segment SGI358562[] = {{&S94977[78], 7}, {NULL}};
const Segment SGI358563[] = {{&S96578[0], 15}, {NULL}};
const Segment SGI358564[] = {{&S96578[31], 3}, {NULL}};
const Segment SGI358565[] = {{&S96581[0], 15}, {NULL}};
const Segment SGI358566[] = {{&S96581[31], 3}, {NULL}};
const Segment SGI358567[] = {{&S96671[0], 16}, {NULL}};
const Segment SGI358568[] = {{&S96671[29], 3}, {NULL}};
const Segment SGI358569[] = {{&S96671[66], 13}, {NULL}};
const Segment SGI358570[] = {{&S95161[6], 4}, {NULL}};
const Segment SGI358571[] = {{&S95161[12], 10}, {NULL}};
const Segment SGI358572[] = {{&S95175[6], 4}, {NULL}};
const Segment SGI358573[] = {{&S95175[121], 5}, {NULL}};
const Segment SGI358574[] = {{&S95175[12], 10}, {&S95175[46], 6}, {NULL}};
const Segment SGI358575[] = {{&S95175[78], 7}, {NULL}};
const Segment SGI358576[] = {{&S95185[6], 4}, {NULL}};
const Segment SGI358577[] = {{&S95185[82], 5}, {NULL}};
const Segment SGI358578[] = {{&S95185[12], 11}, {NULL}};
const Segment SGI358579[] = {{&S95186[6], 4}, {NULL}};
const Segment SGI358580[] = {{&S95186[90], 5}, {NULL}};
const Segment SGI358581[] = {{&S95186[12], 19}, {NULL}};
const Segment SGI358582[] = {{&S95197[6], 4}, {NULL}};
const Segment SGI358583[] = {{&S95197[121], 5}, {NULL}};
const Segment SGI358584[] = {{&S95197[12], 10}, {&S95197[46], 6}, {NULL}};
const Segment SGI358585[] = {{&S95197[78], 7}, {NULL}};
const Segment SGI358586[] = {{&S95220[6], 4}, {NULL}};
const Segment SGI358587[] = {{&S95220[125], 5}, {NULL}};
const Segment SGI358588[] = {{&S95220[12], 12}, {&S95220[47], 6}, {NULL}};
const Segment SGI358589[] = {{&S95220[79], 8}, {NULL}};
const Segment SGI358590[] = {{&S97011[0], 21}, {NULL}};
const Segment SGI358591[] = {{&S97011[58], 3}, {NULL}};
const Segment SGI358592[] = {{&S97011[111], 13}, {NULL}};
const Segment SGI358593[] = {{&S95409[6], 4}, {NULL}};
const Segment SGI358594[] = {{&S95409[54], 5}, {NULL}};
const Segment SGI358595[] = {{&S95409[12], 10}, {NULL}};
const Segment SGI358596[] = {{&S95466[6], 4}, {NULL}};
const Segment SGI358597[] = {{&S95466[123], 5}, {NULL}};
const Segment SGI358598[] = {{&S95466[12], 10}, {&S95466[46], 6}, {NULL}};
const Segment SGI358599[] = {{&S95466[78], 7}, {NULL}};
const Segment SGI358600[] = {{&S97228[0], 15}, {NULL}};
const Segment SGI358601[] = {{&S97228[31], 3}, {NULL}};
const Segment SGI358602[] = {{&S95501[6], 4}, {NULL}};
const Segment SGI358603[] = {{&S95501[12], 15}, {NULL}};
const Segment SGI358604[] = {{&S95501[12], 14}, {NULL}};
const Segment SGI358605[] = {{&S97536[0], 21}, {NULL}};
const Segment SGI358606[] = {{&S97536[34], 3}, {NULL}};
const Segment SGI358607[] = {{&S96010[6], 4}, {NULL}};
const Segment SGI358608[] = {{&S96010[12], 11}, {NULL}};
const Segment SGI358609[] = {{&S96124[6], 4}, {NULL}};
const Segment SGI358610[] = {{&S96124[66], 5}, {NULL}};
const Segment SGI358611[] = {{&S96124[12], 14}, {NULL}};
const Segment SGI358612[] = {{&S96124[36], 5}, {NULL}};
const Segment SGI358613[] = {{&S96319[6], 1}, {NULL}};
const Segment SGI358614[] = {{&S96342[6], 4}, {NULL}};
const Segment SGI358615[] = {{&S96342[74], 5}, {NULL}};
const Segment SGI358616[] = {{&S96342[12], 3}, {NULL}};
const Segment SGI358617[] = {{&S94220[81], 5}, {NULL}};
const Segment SGI358618[] = {{&S94220[23], 9}, {NULL}};
const Segment SGI358619[] = {{&S96386[6], 4}, {NULL}};
const Segment SGI358620[] = {{&S96386[66], 5}, {NULL}};
const Segment SGI358621[] = {{&S96386[12], 14}, {NULL}};
const Segment SGI358622[] = {{&S96386[36], 5}, {NULL}};
const Segment SGI358623[] = {{&S98025[0], 15}, {NULL}};
const Segment SGI358624[] = {{&S98025[31], 3}, {NULL}};
const Segment SGI358625[] = {{&S94256[70], 5}, {NULL}};
const Segment SGI358626[] = {{&S94256[12], 14}, {NULL}};
const Segment SGI358627[] = {{&S94262[77], 5}, {NULL}};
const Segment SGI358628[] = {{&S94262[12], 14}, {NULL}};
const Segment SGI358629[] = {{&S98110[63], 4}, {NULL}};
const Segment SGI358630[] = {{&S98110[13], 10}, {NULL}};
const Segment SGI358631[] = {{&S94686[81], 5}, {NULL}};
const Segment SGI358632[] = {{&S94686[23], 9}, {NULL}};
const Segment SGI358633[] = {{&S96732[6], 4}, {NULL}};
const Segment SGI358634[] = {{&S96732[123], 5}, {NULL}};
const Segment SGI358635[] = {{&S96732[12], 10}, {&S96732[46], 6}, {NULL}};
const Segment SGI358636[] = {{&S96732[78], 7}, {NULL}};
const Segment SGI358637[] = {{&S98582[57], 4}, {NULL}};
const Segment SGI358638[] = {{&S98582[13], 10}, {NULL}};
const Segment SGI358639[] = {{&S98583[38], 4}, {NULL}};
const Segment SGI358640[] = {{&S98583[13], 14}, {NULL}};
const Segment SGI358641[] = {{&S96967[6], 4}, {NULL}};
const Segment SGI358642[] = {{&S96967[77], 5}, {NULL}};
const Segment SGI358643[] = {{&S96967[12], 14}, {NULL}};
const Segment SGI358644[] = {{&S96972[6], 4}, {NULL}};
const Segment SGI358645[] = {{&S96972[121], 5}, {NULL}};
const Segment SGI358646[] = {{&S96972[12], 10}, {&S96972[46], 6}, {NULL}};
const Segment SGI358647[] = {{&S96972[78], 7}, {NULL}};
const Segment SGI358648[] = {{&S96982[6], 4}, {NULL}};
const Segment SGI358649[] = {{&S96982[121], 5}, {NULL}};
const Segment SGI358650[] = {{&S96982[12], 10}, {&S96982[46], 6}, {NULL}};
const Segment SGI358651[] = {{&S96982[78], 7}, {NULL}};
const Segment SGI358652[] = {{&S96998[6], 4}, {NULL}};
const Segment SGI358653[] = {{&S96998[78], 5}, {NULL}};
const Segment SGI358654[] = {{&S96998[12], 3}, {NULL}};
const Segment SGI358655[] = {{&S97002[6], 4}, {NULL}};
const Segment SGI358656[] = {{&S97002[73], 5}, {NULL}};
const Segment SGI358657[] = {{&S97002[12], 14}, {NULL}};
const Segment SGI358658[] = {{&S97069[6], 4}, {NULL}};
const Segment SGI358659[] = {{&S97069[123], 5}, {NULL}};
const Segment SGI358660[] = {{&S97069[12], 10}, {&S97069[46], 6}, {NULL}};
const Segment SGI358661[] = {{&S97069[78], 7}, {NULL}};
const Segment SGI358662[] = {{&S97091[6], 4}, {NULL}};
const Segment SGI358663[] = {{&S97091[123], 5}, {NULL}};
const Segment SGI358664[] = {{&S97091[12], 10}, {&S97091[46], 6}, {NULL}};
const Segment SGI358665[] = {{&S97091[78], 7}, {NULL}};
const Segment SGI358666[] = {{&S97139[6], 4}, {NULL}};
const Segment SGI358667[] = {{&S97139[68], 5}, {NULL}};
const Segment SGI358668[] = {{&S97139[12], 3}, {NULL}};
const Segment SGI358669[] = {{&S99058[68], 4}, {NULL}};
const Segment SGI358670[] = {{&S99058[13], 10}, {NULL}};
const Segment SGI358671[] = {{&S99080[51], 4}, {NULL}};
const Segment SGI358672[] = {{&S99080[13], 9}, {NULL}};
const Segment SGI358673[] = {{&S97367[6], 4}, {NULL}};
const Segment SGI358674[] = {{&S97367[121], 5}, {NULL}};
const Segment SGI358675[] = {{&S97367[12], 10}, {&S97367[46], 6}, {NULL}};
const Segment SGI358676[] = {{&S97367[78], 7}, {NULL}};
const Segment SGI358677[] = {{&S97551[6], 4}, {NULL}};
const Segment SGI358678[] = {{&S97551[59], 5}, {NULL}};
const Segment SGI358679[] = {{&S97551[12], 14}, {NULL}};
const Segment SGI358680[] = {{&S97681[6], 4}, {NULL}};
const Segment SGI358681[] = {{&S97681[12], 12}, {NULL}};
const Segment SGI358682[] = {{&S97726[6], 4}, {NULL}};
const Segment SGI358683[] = {{&S97726[121], 5}, {NULL}};
const Segment SGI358684[] = {{&S97726[12], 10}, {&S97726[46], 6}, {NULL}};
const Segment SGI358685[] = {{&S97726[78], 7}, {NULL}};
const Segment SGI358686[] = {{&S97737[6], 4}, {NULL}};
const Segment SGI358687[] = {{&S97737[76], 5}, {NULL}};
const Segment SGI358688[] = {{&S97737[12], 9}, {NULL}};
const Segment SGI358689[] = {{&S97737[29], 13}, {NULL}};
const Segment SGI358690[] = {{&S97764[6], 4}, {NULL}};
const Segment SGI358691[] = {{&S97764[48], 5}, {NULL}};
const Segment SGI358692[] = {{&S97764[12], 41}, {NULL}};
const Segment SGI358693[] = {{&S95997[81], 5}, {NULL}};
const Segment SGI358694[] = {{&S95997[23], 9}, {NULL}};
const Segment SGI358695[] = {{&S98097[6], 4}, {NULL}};
const Segment SGI358696[] = {{&S98097[12], 10}, {NULL}};
const Segment SGI358697[] = {{&S98098[6], 4}, {NULL}};
const Segment SGI358698[] = {{&S98098[12], 10}, {NULL}};
const Segment SGI358699[] = {{&S100168[57], 4}, {NULL}};
const Segment SGI358700[] = {{&S100168[13], 10}, {NULL}};
const Segment SGI358701[] = {{&S98208[6], 4}, {NULL}};
const Segment SGI358702[] = {{&S98208[126], 5}, {NULL}};
const Segment SGI358703[] = {{&S98208[12], 10}, {&S98208[46], 6}, {NULL}};
const Segment SGI358704[] = {{&S98208[78], 7}, {NULL}};
const Segment SGI358705[] = {{&S98252[6], 4}, {NULL}};
const Segment SGI358706[] = {{&S98252[123], 5}, {NULL}};
const Segment SGI358707[] = {{&S98252[12], 10}, {&S98252[46], 6}, {NULL}};
const Segment SGI358708[] = {{&S98252[78], 7}, {NULL}};
const Segment SGI358709[] = {{&S100033[0], 15}, {NULL}};
const Segment SGI358710[] = {{&S100033[31], 3}, {NULL}};
const Segment SGI358711[] = {{&S98408[6], 4}, {NULL}};
const Segment SGI358712[] = {{&S98408[125], 5}, {NULL}};
const Segment SGI358713[] = {{&S98408[12], 12}, {&S98408[47], 6}, {NULL}};
const Segment SGI358714[] = {{&S98408[79], 8}, {NULL}};
const Segment SGI358715[] = {{&S98409[6], 4}, {NULL}};
const Segment SGI358716[] = {{&S98409[59], 5}, {NULL}};
const Segment SGI358717[] = {{&S98409[12], 12}, {&S98409[84], 22}, {NULL}};
const Segment SGI358718[] = {{&S98409[87], 19}, {NULL}};
const Segment SGI358719[] = {{&S98430[6], 4}, {NULL}};
const Segment SGI358720[] = {{&S98430[83], 5}, {NULL}};
const Segment SGI358721[] = {{&S98430[12], 13}, {NULL}};
const Segment SGI358722[] = {{&S100084[0], 15}, {NULL}};
const Segment SGI358723[] = {{&S100084[31], 3}, {NULL}};
const Segment SGI358724[] = {{&S98555[6], 4}, {NULL}};
const Segment SGI358725[] = {{&S98555[12], 14}, {NULL}};
const Segment SGI358726[] = {{&S100242[0], 16}, {NULL}};
const Segment SGI358727[] = {{&S100242[29], 3}, {NULL}};
const Segment SGI358728[] = {{&S100242[71], 9}, {NULL}};
const Segment SGI358729[] = {{&S96623[23], 9}, {NULL}};
const Segment SGI358730[] = {{&S98793[6], 4}, {NULL}};
const Segment SGI358731[] = {{&S98793[121], 5}, {NULL}};
const Segment SGI358732[] = {{&S98793[12], 10}, {&S98793[46], 6}, {NULL}};
const Segment SGI358733[] = {{&S98793[78], 7}, {NULL}};
const Segment SGI358734[] = {{&S100803[57], 4}, {NULL}};
const Segment SGI358735[] = {{&S100803[13], 10}, {NULL}};
const Segment SGI358736[] = {{&S96911[17], 17}, {NULL}};
const Segment SGI358737[] = {{&S99091[6], 4}, {NULL}};
const Segment SGI358738[] = {{&S99091[81], 5}, {NULL}};
const Segment SGI358739[] = {{&S99091[12], 11}, {NULL}};
const Segment SGI358740[] = {{&S99092[6], 4}, {NULL}};
const Segment SGI358741[] = {{&S99092[83], 5}, {NULL}};
const Segment SGI358742[] = {{&S99092[12], 13}, {NULL}};
const Segment SGI358743[] = {{&S99117[6], 4}, {NULL}};
const Segment SGI358744[] = {{&S99117[121], 5}, {NULL}};
const Segment SGI358745[] = {{&S99117[12], 10}, {&S99117[46], 6}, {NULL}};
const Segment SGI358746[] = {{&S99117[78], 7}, {NULL}};
const Segment SGI358747[] = {{&S99160[6], 4}, {NULL}};
const Segment SGI358748[] = {{&S99160[76], 5}, {NULL}};
const Segment SGI358749[] = {{&S99160[12], 9}, {NULL}};
const Segment SGI358750[] = {{&S99160[29], 13}, {NULL}};
const Segment SGI358751[] = {{&S97311[81], 5}, {NULL}};
const Segment SGI358752[] = {{&S97311[23], 9}, {NULL}};
const Segment SGI358753[] = {{&S97316[81], 5}, {NULL}};
const Segment SGI358754[] = {{&S97316[23], 9}, {NULL}};
const Segment SGI358755[] = {{&S100768[0], 21}, {NULL}};
const Segment SGI358756[] = {{&S100768[34], 3}, {NULL}};
const Segment SGI358757[] = {{&S100768[73], 13}, {NULL}};
const Segment SGI358758[] = {{&S101499[8], 3}, {NULL}};
const Segment SGI358759[] = {{&S101499[13], 10}, {NULL}};
const Segment SGI358760[] = {{&S99645[6], 4}, {NULL}};
const Segment SGI358761[] = {{&S99645[121], 5}, {NULL}};
const Segment SGI358762[] = {{&S99645[12], 10}, {&S99645[46], 6}, {NULL}};
const Segment SGI358763[] = {{&S99645[78], 7}, {NULL}};
const Segment SGI358764[] = {{&S101515[47], 4}, {NULL}};
const Segment SGI358765[] = {{&S101515[13], 10}, {NULL}};
const Segment SGI358766[] = {{&S99663[6], 4}, {NULL}};
const Segment SGI358767[] = {{&S99663[121], 5}, {NULL}};
const Segment SGI358768[] = {{&S99663[12], 10}, {&S99663[46], 6}, {NULL}};
const Segment SGI358769[] = {{&S99663[78], 7}, {NULL}};
const Segment SGI358770[] = {{&S99690[6], 4}, {NULL}};
const Segment SGI358771[] = {{&S99690[121], 5}, {NULL}};
const Segment SGI358772[] = {{&S99690[12], 10}, {&S99690[46], 6}, {NULL}};
const Segment SGI358773[] = {{&S99690[78], 7}, {NULL}};
const Segment SGI358774[] = {{&S99734[6], 4}, {NULL}};
const Segment SGI358775[] = {{&S99734[57], 5}, {NULL}};
const Segment SGI358776[] = {{&S99734[12], 14}, {NULL}};
const Segment SGI358777[] = {{&S99753[6], 4}, {NULL}};
const Segment SGI358778[] = {{&S99753[57], 5}, {NULL}};
const Segment SGI358779[] = {{&S99753[12], 14}, {NULL}};
const Segment SGI358780[] = {{&S99758[6], 4}, {NULL}};
const Segment SGI358781[] = {{&S99758[55], 5}, {NULL}};
const Segment SGI358782[] = {{&S99758[12], 10}, {NULL}};
const Segment SGI358783[] = {{&S101647[73], 4}, {NULL}};
const Segment SGI358784[] = {{&S101647[13], 10}, {NULL}};
const Segment SGI358785[] = {{&S101715[6], 4}, {NULL}};
const Segment SGI358786[] = {{&S101715[12], 14}, {NULL}};
const Segment SGI358787[] = {{&S99936[6], 4}, {NULL}};
const Segment SGI358788[] = {{&S99936[12], 14}, {NULL}};
const Segment SGI358789[] = {{&S101015[0], 21}, {NULL}};
const Segment SGI358790[] = {{&S101015[34], 3}, {NULL}};
const Segment SGI358791[] = {{&S101015[73], 13}, {NULL}};
const Segment SGI358792[] = {{&S101016[0], 21}, {NULL}};
const Segment SGI358793[] = {{&S101016[34], 3}, {NULL}};
const Segment SGI358794[] = {{&S100209[6], 4}, {NULL}};
const Segment SGI358795[] = {{&S100209[70], 5}, {NULL}};
const Segment SGI358796[] = {{&S100209[12], 3}, {NULL}};
const Segment SGI358797[] = {{&S100209[32], 10}, {NULL}};
const Segment SGI358798[] = {{&S100262[6], 4}, {NULL}};
const Segment SGI358799[] = {{&S100262[12], 15}, {NULL}};
const Segment SGI358800[] = {{&S100262[12], 14}, {NULL}};
const Segment SGI358801[] = {{&S102154[76], 4}, {NULL}};
const Segment SGI358802[] = {{&S102154[31], 10}, {NULL}};
const Segment SGI358803[] = {{&S100383[6], 4}, {NULL}};
const Segment SGI358804[] = {{&S100383[12], 11}, {NULL}};
const Segment SGI358805[] = {{&S102330[136], 4}, {NULL}};
const Segment SGI358806[] = {{&S102330[13], 10}, {NULL}};
const Segment SGI358807[] = {{&S102358[136], 4}, {NULL}};
const Segment SGI358808[] = {{&S102358[13], 10}, {NULL}};
const Segment SGI358809[] = {{&S98115[81], 5}, {NULL}};
const Segment SGI358810[] = {{&S98115[23], 9}, {NULL}};
const Segment SGI358811[] = {{&S100817[6], 4}, {NULL}};
const Segment SGI358812[] = {{&S100817[55], 5}, {NULL}};
const Segment SGI358813[] = {{&S100817[12], 10}, {NULL}};
const Segment SGI358814[] = {{&S102833[45], 3}, {NULL}};
const Segment SGI358815[] = {{&S102833[13], 10}, {NULL}};
const Segment SGI358816[] = {{&S100876[6], 4}, {NULL}};
const Segment SGI358817[] = {{&S100876[121], 5}, {NULL}};
const Segment SGI358818[] = {{&S100876[12], 10}, {&S100876[46], 6}, {NULL}};
const Segment SGI358819[] = {{&S100876[78], 7}, {NULL}};
const Segment SGI358820[] = {{&S101042[6], 4}, {NULL}};
const Segment SGI358821[] = {{&S101042[121], 5}, {NULL}};
const Segment SGI358822[] = {{&S101042[12], 10}, {&S101042[46], 6}, {NULL}};
const Segment SGI358823[] = {{&S101042[78], 7}, {NULL}};
const Segment SGI358824[] = {{&S101212[6], 4}, {NULL}};
const Segment SGI358825[] = {{&S101212[12], 24}, {NULL}};
const Segment SGI358826[] = {{&S99079[70], 5}, {NULL}};
const Segment SGI358827[] = {{&S99079[12], 12}, {NULL}};
const Segment SGI358828[] = {{&S101354[6], 4}, {NULL}};
const Segment SGI358829[] = {{&S101354[60], 5}, {NULL}};
const Segment SGI358830[] = {{&S101354[12], 3}, {NULL}};
const Segment SGI358831[] = {{&S101370[6], 4}, {NULL}};
const Segment SGI358832[] = {{&S101370[121], 5}, {NULL}};
const Segment SGI358833[] = {{&S101370[12], 10}, {&S101370[46], 6}, {NULL}};
const Segment SGI358834[] = {{&S101370[78], 7}, {NULL}};
const Segment SGI358835[] = {{&S101407[6], 4}, {NULL}};
const Segment SGI358836[] = {{&S101407[12], 10}, {NULL}};
const Segment SGI358837[] = {{&S101412[6], 4}, {NULL}};
const Segment SGI358838[] = {{&S101412[97], 5}, {NULL}};
const Segment SGI358839[] = {{&S101412[12], 10}, {NULL}};
const Segment SGI358840[] = {{&S101538[6], 4}, {NULL}};
const Segment SGI358841[] = {{&S101538[57], 5}, {NULL}};
const Segment SGI358842[] = {{&S101538[12], 14}, {NULL}};
const Segment SGI358843[] = {{&S101543[6], 4}, {NULL}};
const Segment SGI358844[] = {{&S101543[121], 5}, {NULL}};
const Segment SGI358845[] = {{&S101543[12], 10}, {&S101543[46], 6}, {NULL}};
const Segment SGI358846[] = {{&S101543[78], 7}, {NULL}};
const Segment SGI358847[] = {{&S103593[15], 3}, {&S103593[29], 4}, {NULL}};
const Segment SGI358848[] = {{&S103593[35], 14}, {NULL}};
const Segment SGI358849[] = {{&S101735[6], 4}, {NULL}};
const Segment SGI358850[] = {{&S101735[121], 5}, {NULL}};
const Segment SGI358851[] = {{&S101735[12], 10}, {&S101735[46], 6}, {NULL}};
const Segment SGI358852[] = {{&S101735[78], 7}, {NULL}};
const Segment SGI358853[] = {{&S101850[6], 4}, {NULL}};
const Segment SGI358854[] = {{&S101850[77], 5}, {NULL}};
const Segment SGI358855[] = {{&S101850[12], 14}, {NULL}};
const Segment SGI358856[] = {{&S101851[6], 4}, {NULL}};
const Segment SGI358857[] = {{&S101851[76], 3}, {NULL}};
const Segment SGI358858[] = {{&S101851[12], 3}, {NULL}};
const Segment SGI358859[] = {{&S103917[74], 4}, {NULL}};
const Segment SGI358860[] = {{&S103917[13], 10}, {NULL}};
const Segment SGI358861[] = {{&S99611[81], 5}, {NULL}};
const Segment SGI358862[] = {{&S99611[23], 9}, {NULL}};
const Segment SGI358863[] = {{&S99625[81], 5}, {NULL}};
const Segment SGI358864[] = {{&S99625[23], 9}, {NULL}};
const Segment SGI358865[] = {{&S103977[64], 4}, {NULL}};
const Segment SGI358866[] = {{&S103977[13], 10}, {NULL}};
const Segment SGI358867[] = {{&S102065[6], 4}, {NULL}};
const Segment SGI358868[] = {{&S102065[90], 5}, {NULL}};
const Segment SGI358869[] = {{&S102065[12], 19}, {NULL}};
const Segment SGI358870[] = {{&S102067[12], 13}, {NULL}};
const Segment SGI358871[] = {{&S102146[6], 4}, {NULL}};
const Segment SGI358872[] = {{&S102146[125], 5}, {NULL}};
const Segment SGI358873[] = {{&S102146[12], 12}, {&S102146[47], 6}, {NULL}};
const Segment SGI358874[] = {{&S102146[79], 8}, {NULL}};
const Segment SGI358875[] = {{&S102242[6], 4}, {NULL}};
const Segment SGI358876[] = {{&S102242[78], 5}, {NULL}};
const Segment SGI358877[] = {{&S102242[12], 14}, {NULL}};
const Segment SGI358878[] = {{&S102259[6], 4}, {NULL}};
const Segment SGI358879[] = {{&S102259[90], 5}, {NULL}};
const Segment SGI358880[] = {{&S102259[12], 18}, {NULL}};
const Segment SGI358881[] = {{&S102311[6], 4}, {NULL}};
const Segment SGI358882[] = {{&S102311[121], 5}, {NULL}};
const Segment SGI358883[] = {{&S102311[12], 10}, {&S102311[46], 6}, {NULL}};
const Segment SGI358884[] = {{&S102311[78], 7}, {NULL}};
const Segment SGI358885[] = {{&S103334[0], 21}, {NULL}};
const Segment SGI358886[] = {{&S103334[34], 3}, {NULL}};
const Segment SGI358887[] = {{&S104620[100], 4}, {NULL}};
const Segment SGI358888[] = {{&S104620[29], 10}, {NULL}};
const Segment SGI358889[] = {{&S102532[6], 4}, {NULL}};
const Segment SGI358890[] = {{&S102532[12], 14}, {NULL}};
const Segment SGI358891[] = {{&S100200[81], 5}, {NULL}};
const Segment SGI358892[] = {{&S100200[23], 9}, {NULL}};
const Segment SGI358893[] = {{&S102678[6], 4}, {NULL}};
const Segment SGI358894[] = {{&S102678[121], 5}, {NULL}};
const Segment SGI358895[] = {{&S102678[12], 10}, {&S102678[46], 6}, {NULL}};
const Segment SGI358896[] = {{&S102678[78], 7}, {NULL}};
const Segment SGI358897[] = {{&S102728[6], 4}, {NULL}};
const Segment SGI358898[] = {{&S102728[56], 5}, {NULL}};
const Segment SGI358899[] = {{&S102728[12], 12}, {NULL}};
const Segment SGI358900[] = {{&S102857[6], 4}, {NULL}};
const Segment SGI358901[] = {{&S102858[6], 4}, {NULL}};
const Segment SGI358902[] = {{&S102888[6], 4}, {NULL}};
const Segment SGI358903[] = {{&S102888[55], 5}, {NULL}};
const Segment SGI358904[] = {{&S102888[12], 10}, {NULL}};
const Segment SGI358905[] = {{&S102933[6], 4}, {NULL}};
const Segment SGI358906[] = {{&S102933[56], 5}, {NULL}};
const Segment SGI358907[] = {{&S102933[12], 12}, {NULL}};
const Segment SGI358908[] = {{&S103072[6], 4}, {NULL}};
const Segment SGI358909[] = {{&S103072[76], 5}, {NULL}};
const Segment SGI358910[] = {{&S103072[12], 9}, {NULL}};
const Segment SGI358911[] = {{&S103072[29], 13}, {NULL}};
const Segment SGI358912[] = {{&S100781[23], 9}, {NULL}};
const Segment SGI358913[] = {{&S100783[81], 5}, {NULL}};
const Segment SGI358914[] = {{&S100783[23], 9}, {NULL}};
const Segment SGI358915[] = {{&S105409[63], 3}, {NULL}};
const Segment SGI358916[] = {{&S105409[13], 10}, {NULL}};
const Segment SGI358917[] = {{&S104563[0], 21}, {NULL}};
const Segment SGI358918[] = {{&S104563[75], 13}, {NULL}};
const Segment SGI358919[] = {{&S103349[6], 4}, {NULL}};
const Segment SGI358920[] = {{&S103349[57], 5}, {NULL}};
const Segment SGI358921[] = {{&S103349[12], 14}, {NULL}};
const Segment SGI358922[] = {{&S103355[6], 4}, {NULL}};
const Segment SGI358923[] = {{&S103355[72], 5}, {NULL}};
const Segment SGI358924[] = {{&S103355[12], 3}, {NULL}};
const Segment SGI358925[] = {{&S103414[6], 4}, {NULL}};
const Segment SGI358926[] = {{&S103414[74], 5}, {NULL}};
const Segment SGI358927[] = {{&S103414[12], 3}, {NULL}};
const Segment SGI358928[] = {{&S101035[81], 5}, {NULL}};
const Segment SGI358929[] = {{&S101035[23], 9}, {NULL}};
const Segment SGI358930[] = {{&S103981[6], 4}, {NULL}};
const Segment SGI358931[] = {{&S103981[12], 7}, {NULL}};
const Segment SGI358932[] = {{&S104019[6], 4}, {NULL}};
const Segment SGI358933[] = {{&S104019[74], 5}, {NULL}};
const Segment SGI358934[] = {{&S104019[12], 3}, {NULL}};
const Segment SGI358935[] = {{&S104130[6], 4}, {NULL}};
const Segment SGI358936[] = {{&S104130[56], 5}, {NULL}};
const Segment SGI358937[] = {{&S104130[12], 12}, {NULL}};
const Segment SGI358938[] = {{&S101837[75], 5}, {NULL}};
const Segment SGI358939[] = {{&S101837[12], 14}, {NULL}};
const Segment SGI358940[] = {{&S101873[71], 5}, {NULL}};
const Segment SGI358941[] = {{&S101873[12], 8}, {NULL}};
const Segment SGI358942[] = {{&S102034[23], 9}, {NULL}};
const Segment SGI358943[] = {{&S102048[76], 5}, {NULL}};
const Segment SGI358944[] = {{&S102048[12], 14}, {NULL}};
const Segment SGI358945[] = {{&S104533[6], 4}, {NULL}};
const Segment SGI358946[] = {{&S104533[56], 5}, {NULL}};
const Segment SGI358947[] = {{&S104533[12], 12}, {NULL}};
const Segment SGI358948[] = {{&S104534[6], 4}, {NULL}};
const Segment SGI358949[] = {{&S104534[56], 5}, {NULL}};
const Segment SGI358950[] = {{&S104534[12], 12}, {NULL}};
const Segment SGI358951[] = {{&S104535[6], 4}, {NULL}};
const Segment SGI358952[] = {{&S104535[56], 5}, {NULL}};
const Segment SGI358953[] = {{&S104535[12], 12}, {NULL}};
const Segment SGI358954[] = {{&S102249[23], 9}, {NULL}};
const Segment SGI358955[] = {{&S104785[6], 4}, {NULL}};
const Segment SGI358956[] = {{&S104785[70], 5}, {NULL}};
const Segment SGI358957[] = {{&S104785[12], 10}, {NULL}};
const Segment SGI358958[] = {{&S104786[6], 4}, {NULL}};
const Segment SGI358959[] = {{&S104786[80], 5}, {NULL}};
const Segment SGI358960[] = {{&S104786[12], 10}, {NULL}};
const Segment SGI358961[] = {{&S102639[81], 5}, {NULL}};
const Segment SGI358962[] = {{&S102639[23], 9}, {NULL}};
const Segment SGI358963[] = {{&S104846[6], 4}, {NULL}};
const Segment SGI358964[] = {{&S104846[78], 5}, {NULL}};
const Segment SGI358965[] = {{&S104846[12], 7}, {NULL}};
const Segment SGI358966[] = {{&S105042[6], 4}, {NULL}};
const Segment SGI358967[] = {{&S105042[56], 5}, {NULL}};
const Segment SGI358968[] = {{&S105042[12], 12}, {NULL}};
const Segment SGI358969[] = {{&S105062[6], 4}, {NULL}};
const Segment SGI358970[] = {{&S105062[78], 5}, {NULL}};
const Segment SGI358971[] = {{&S105062[12], 10}, {&S105062[95], 11}, {NULL}};
const Segment SGI358972[] = {{&S105062[98], 39}, {NULL}};
const Segment SGI358973[] = {{&S105212[6], 4}, {NULL}};
const Segment SGI358974[] = {{&S105212[12], 14}, {NULL}};
const Segment SGI358975[] = {{&S103648[74], 4}, {NULL}};
const Segment SGI358976[] = {{&S103648[12], 14}, {NULL}};
const Segment SGI358977[] = {{&S105485[6], 4}, {NULL}};
const Segment SGI358978[] = {{&S105485[78], 5}, {NULL}};
const Segment SGI358979[] = {{&S105485[12], 14}, {NULL}};
const Segment SGI358980[] = {{&S105486[6], 4}, {NULL}};
const Segment SGI358981[] = {{&S105486[78], 5}, {NULL}};
const Segment SGI358982[] = {{&S105486[12], 14}, {NULL}};
const Segment SGI358983[] = {{&S105487[6], 4}, {NULL}};
const Segment SGI358984[] = {{&S105487[77], 5}, {NULL}};
const Segment SGI358985[] = {{&S105487[12], 14}, {NULL}};
const Segment SGI358986[] = {{&S105517[6], 4}, {NULL}};
const Segment SGI358987[] = {{&S105517[71], 5}, {NULL}};
const Segment SGI358988[] = {{&S105517[12], 3}, {NULL}};
const Segment SGI358989[] = {{&S105517[32], 10}, {NULL}};
const Segment SGI358990[] = {{&S103838[1], 134}, {NULL}};
const Segment SGI358991[] = {{&S103984[79], 4}, {NULL}};
const Segment SGI358992[] = {{&S103984[12], 14}, {NULL}};
const Segment SGI358993[] = {{&S104327[76], 5}, {NULL}};
const Segment SGI358994[] = {{&S104327[12], 14}, {NULL}};
const Segment SGI358995[] = {{&S104328[75], 5}, {NULL}};
const Segment SGI358996[] = {{&S104328[12], 14}, {NULL}};
const Segment SGI358997[] = {{&S105958[6], 4}, {NULL}};
const Segment SGI358998[] = {{&S105958[76], 5}, {NULL}};
const Segment SGI358999[] = {{&S105958[12], 10}, {&S105958[93], 11}, {NULL}};
const Segment SGI359000[] = {{&S105958[96], 39}, {NULL}};
const Segment SGI359001[] = {{&S105959[6], 4}, {NULL}};
const Segment SGI359002[] = {{&S105959[76], 5}, {NULL}};
const Segment SGI359003[] = {{&S105959[12], 10}, {&S105959[93], 11}, {NULL}};
const Segment SGI359004[] = {{&S105959[96], 39}, {NULL}};
const Segment SGI359005[] = {{&S105960[6], 4}, {NULL}};
const Segment SGI359006[] = {{&S105960[76], 5}, {NULL}};
const Segment SGI359007[] = {{&S105960[12], 10}, {&S105960[93], 11}, {NULL}};
const Segment SGI359008[] = {{&S105960[96], 39}, {NULL}};
const Segment SGI359009[] = {{&S105961[6], 4}, {NULL}};
const Segment SGI359010[] = {{&S105961[129], 5}, {NULL}};
const Segment SGI359011[] = {{&S105961[12], 10}, {&S105961[38], 11}, {NULL}};
const Segment SGI359012[] = {{&S105961[41], 39}, {NULL}};
const Segment SGI359013[] = {{&S106008[6], 1}, {NULL}};
const Segment SGI359014[] = {{&S106009[6], 1}, {NULL}};
const Segment SGI359015[] = {{&S104768[74], 4}, {NULL}};
const Segment SGI359016[] = {{&S104768[12], 14}, {NULL}};
const Segment SGI359017[] = {{&S105556[86], 5}, {NULL}};
const Segment SGI359018[] = {{&S105556[12], 9}, {NULL}};
const Segment SGI359019[] = {{&S3176[68], 6}, {NULL}};
const Segment SGI359020[] = {{&S3176[8], 3}, {NULL}};
const Segment SGI359021[] = {{&S3176[32], 15}, {NULL}};
const Segment SGI359022[] = {{&S3176[48], 6}, {NULL}};
const Segment SGI359023[] = {{&S3176[32], 13}, {NULL}};
const Segment SGI359024[] = {{&S3177[68], 6}, {NULL}};
const Segment SGI359025[] = {{&S3177[8], 3}, {NULL}};
const Segment SGI359026[] = {{&S3177[13], 9}, {NULL}};
const Segment SGI359027[] = {{&S3177[33], 15}, {NULL}};
const Segment SGI359028[] = {{&S3177[49], 5}, {NULL}};
const Segment SGI359029[] = {{&S3177[33], 13}, {NULL}};
const Segment SGI359030[] = {{&S3178[68], 6}, {NULL}};
const Segment SGI359031[] = {{&S3178[8], 3}, {NULL}};
const Segment SGI359032[] = {{&S3178[13], 9}, {NULL}};
const Segment SGI359033[] = {{&S3179[69], 6}, {NULL}};
const Segment SGI359034[] = {{&S3179[8], 3}, {NULL}};
const Segment SGI359035[] = {{&S3179[13], 9}, {NULL}};
const Segment SGI359036[] = {{&S3179[33], 15}, {NULL}};
const Segment SGI359037[] = {{&S3179[49], 6}, {NULL}};
const Segment SGI359038[] = {{&S3179[33], 13}, {NULL}};
const Segment SGI359039[] = {{&S3180[69], 6}, {NULL}};
const Segment SGI359040[] = {{&S3180[8], 3}, {NULL}};
const Segment SGI359041[] = {{&S3180[13], 9}, {NULL}};
const Segment SGI359042[] = {{&S3180[33], 15}, {NULL}};
const Segment SGI359043[] = {{&S3180[49], 6}, {NULL}};
const Segment SGI359044[] = {{&S3180[33], 13}, {NULL}};
const Segment SGI359045[] = {{&S3181[69], 6}, {NULL}};
const Segment SGI359046[] = {{&S3181[8], 3}, {NULL}};
const Segment SGI359047[] = {{&S3181[13], 9}, {NULL}};
const Segment SGI359048[] = {{&S3181[33], 15}, {NULL}};
const Segment SGI359049[] = {{&S3181[49], 6}, {NULL}};
const Segment SGI359050[] = {{&S3181[33], 13}, {NULL}};
const Segment SGI359051[] = {{&S3182[69], 6}, {NULL}};
const Segment SGI359052[] = {{&S3182[8], 3}, {NULL}};
const Segment SGI359053[] = {{&S3182[13], 9}, {NULL}};
const Segment SGI359054[] = {{&S3182[33], 15}, {NULL}};
const Segment SGI359055[] = {{&S3182[49], 6}, {NULL}};
const Segment SGI359056[] = {{&S3182[33], 13}, {NULL}};
const Segment SGI359057[] = {{&S3183[69], 6}, {NULL}};
const Segment SGI359058[] = {{&S3183[8], 3}, {NULL}};
const Segment SGI359059[] = {{&S3183[13], 9}, {NULL}};
const Segment SGI359060[] = {{&S3183[33], 15}, {NULL}};
const Segment SGI359061[] = {{&S3183[49], 6}, {NULL}};
const Segment SGI359062[] = {{&S3183[33], 13}, {NULL}};
const Segment SGI359063[] = {{&S3184[69], 6}, {NULL}};
const Segment SGI359064[] = {{&S3184[8], 3}, {NULL}};
const Segment SGI359065[] = {{&S3184[13], 9}, {NULL}};
const Segment SGI359066[] = {{&S3184[33], 15}, {NULL}};
const Segment SGI359067[] = {{&S3184[49], 6}, {NULL}};
const Segment SGI359068[] = {{&S3184[33], 13}, {NULL}};
const Segment SGI359069[] = {{&S3185[69], 6}, {NULL}};
const Segment SGI359070[] = {{&S3185[8], 3}, {NULL}};
const Segment SGI359071[] = {{&S3185[13], 9}, {NULL}};
const Segment SGI359072[] = {{&S3185[33], 15}, {NULL}};
const Segment SGI359073[] = {{&S3185[49], 6}, {NULL}};
const Segment SGI359074[] = {{&S3185[33], 13}, {NULL}};
const Segment SGI359075[] = {{&S3186[69], 6}, {NULL}};
const Segment SGI359076[] = {{&S3186[8], 3}, {NULL}};
const Segment SGI359077[] = {{&S3186[13], 9}, {NULL}};
const Segment SGI359078[] = {{&S3186[33], 15}, {NULL}};
const Segment SGI359079[] = {{&S3186[49], 6}, {NULL}};
const Segment SGI359080[] = {{&S3186[33], 13}, {NULL}};
const Segment SGI359081[] = {{&S3187[69], 6}, {NULL}};
const Segment SGI359082[] = {{&S3187[8], 3}, {NULL}};
const Segment SGI359083[] = {{&S3187[13], 9}, {NULL}};
const Segment SGI359084[] = {{&S3187[33], 15}, {NULL}};
const Segment SGI359085[] = {{&S3187[49], 6}, {NULL}};
const Segment SGI359086[] = {{&S3187[33], 13}, {NULL}};
const Segment SGI359087[] = {{&S3188[69], 6}, {NULL}};
const Segment SGI359088[] = {{&S3188[8], 3}, {NULL}};
const Segment SGI359089[] = {{&S3188[13], 9}, {NULL}};
const Segment SGI359090[] = {{&S3188[33], 15}, {NULL}};
const Segment SGI359091[] = {{&S3188[49], 6}, {NULL}};
const Segment SGI359092[] = {{&S3188[33], 13}, {NULL}};
const Segment SGI359093[] = {{&S3189[69], 6}, {NULL}};
const Segment SGI359094[] = {{&S3189[8], 3}, {NULL}};
const Segment SGI359095[] = {{&S3189[13], 9}, {NULL}};
const Segment SGI359096[] = {{&S3189[33], 15}, {NULL}};
const Segment SGI359097[] = {{&S3189[49], 6}, {NULL}};
const Segment SGI359098[] = {{&S3189[33], 13}, {NULL}};
const Segment SGI359099[] = {{&S3190[69], 6}, {NULL}};
const Segment SGI359100[] = {{&S3190[8], 3}, {NULL}};
const Segment SGI359101[] = {{&S3190[13], 9}, {NULL}};
const Segment SGI359102[] = {{&S3190[33], 15}, {NULL}};
const Segment SGI359103[] = {{&S3190[49], 6}, {NULL}};
const Segment SGI359104[] = {{&S3190[33], 13}, {NULL}};
const Segment SGI359105[] = {{&S3191[69], 6}, {NULL}};
const Segment SGI359106[] = {{&S3191[8], 3}, {NULL}};
const Segment SGI359107[] = {{&S3191[13], 9}, {NULL}};
const Segment SGI359108[] = {{&S3191[33], 15}, {NULL}};
const Segment SGI359109[] = {{&S3191[49], 6}, {NULL}};
const Segment SGI359110[] = {{&S3191[33], 13}, {NULL}};
const Segment SGI359111[] = {{&S3192[69], 6}, {NULL}};
const Segment SGI359112[] = {{&S3192[8], 3}, {NULL}};
const Segment SGI359113[] = {{&S3192[13], 9}, {NULL}};
const Segment SGI359114[] = {{&S3192[33], 15}, {NULL}};
const Segment SGI359115[] = {{&S3192[49], 6}, {NULL}};
const Segment SGI359116[] = {{&S3192[33], 13}, {NULL}};
const Segment SGI359117[] = {{&S3193[69], 6}, {NULL}};
const Segment SGI359118[] = {{&S3193[8], 3}, {NULL}};
const Segment SGI359119[] = {{&S3193[13], 9}, {NULL}};
const Segment SGI359120[] = {{&S3193[33], 15}, {NULL}};
const Segment SGI359121[] = {{&S3193[49], 6}, {NULL}};
const Segment SGI359122[] = {{&S3193[33], 13}, {NULL}};
const Segment SGI359123[] = {{&S3194[69], 6}, {NULL}};
const Segment SGI359124[] = {{&S3194[8], 3}, {NULL}};
const Segment SGI359125[] = {{&S3194[13], 9}, {NULL}};
const Segment SGI359126[] = {{&S3194[33], 15}, {NULL}};
const Segment SGI359127[] = {{&S3194[49], 6}, {NULL}};
const Segment SGI359128[] = {{&S3194[33], 13}, {NULL}};
const Segment SGI359129[] = {{&S3195[69], 6}, {NULL}};
const Segment SGI359130[] = {{&S3195[8], 3}, {NULL}};
const Segment SGI359131[] = {{&S3195[13], 9}, {NULL}};
const Segment SGI359132[] = {{&S3195[33], 15}, {NULL}};
const Segment SGI359133[] = {{&S3195[49], 6}, {NULL}};
const Segment SGI359134[] = {{&S3195[33], 13}, {NULL}};
const Segment SGI359135[] = {{&S3196[69], 6}, {NULL}};
const Segment SGI359136[] = {{&S3196[8], 3}, {NULL}};
const Segment SGI359137[] = {{&S3196[13], 9}, {NULL}};
const Segment SGI359138[] = {{&S3196[33], 15}, {NULL}};
const Segment SGI359139[] = {{&S3196[49], 6}, {NULL}};
const Segment SGI359140[] = {{&S3196[33], 13}, {NULL}};
const Segment SGI359141[] = {{&S3197[69], 6}, {NULL}};
const Segment SGI359142[] = {{&S3197[8], 3}, {NULL}};
const Segment SGI359143[] = {{&S3197[13], 9}, {NULL}};
const Segment SGI359144[] = {{&S3197[33], 15}, {NULL}};
const Segment SGI359145[] = {{&S3197[49], 6}, {NULL}};
const Segment SGI359146[] = {{&S3197[33], 13}, {NULL}};
const Segment SGI359147[] = {{&S3198[66], 6}, {NULL}};
const Segment SGI359148[] = {{&S3198[8], 3}, {NULL}};
const Segment SGI359149[] = {{&S3198[13], 9}, {NULL}};
const Segment SGI359150[] = {{&S3198[33], 15}, {NULL}};
const Segment SGI359151[] = {{&S3198[49], 3}, {NULL}};
const Segment SGI359152[] = {{&S3198[33], 13}, {NULL}};
const Segment SGI359153[] = {{&S3199[69], 6}, {NULL}};
const Segment SGI359154[] = {{&S3199[8], 3}, {NULL}};
const Segment SGI359155[] = {{&S3199[13], 9}, {NULL}};
const Segment SGI359156[] = {{&S3199[33], 15}, {NULL}};
const Segment SGI359157[] = {{&S3199[49], 6}, {NULL}};
const Segment SGI359158[] = {{&S3199[33], 13}, {NULL}};
const Segment SGI359159[] = {{&S3200[69], 6}, {NULL}};
const Segment SGI359160[] = {{&S3200[8], 3}, {NULL}};
const Segment SGI359161[] = {{&S3200[13], 9}, {NULL}};
const Segment SGI359162[] = {{&S3200[33], 15}, {NULL}};
const Segment SGI359163[] = {{&S3200[49], 6}, {NULL}};
const Segment SGI359164[] = {{&S3200[33], 13}, {NULL}};
const Segment SGI359165[] = {{&S3201[66], 6}, {NULL}};
const Segment SGI359166[] = {{&S3201[8], 3}, {NULL}};
const Segment SGI359167[] = {{&S3201[13], 9}, {NULL}};
const Segment SGI359168[] = {{&S3201[33], 15}, {NULL}};
const Segment SGI359169[] = {{&S3201[49], 3}, {NULL}};
const Segment SGI359170[] = {{&S3201[33], 13}, {NULL}};
const Segment SGI359171[] = {{&S3202[69], 6}, {NULL}};
const Segment SGI359172[] = {{&S3202[8], 3}, {NULL}};
const Segment SGI359173[] = {{&S3202[13], 9}, {NULL}};
const Segment SGI359174[] = {{&S3202[33], 15}, {NULL}};
const Segment SGI359175[] = {{&S3202[49], 6}, {NULL}};
const Segment SGI359176[] = {{&S3202[33], 13}, {NULL}};
const Segment SGI359177[] = {{&S3203[69], 6}, {NULL}};
const Segment SGI359178[] = {{&S3203[8], 3}, {NULL}};
const Segment SGI359179[] = {{&S3203[13], 9}, {NULL}};
const Segment SGI359180[] = {{&S3203[33], 15}, {NULL}};
const Segment SGI359181[] = {{&S3203[49], 6}, {NULL}};
const Segment SGI359182[] = {{&S3203[33], 13}, {NULL}};
const Segment SGI359183[] = {{&S3204[69], 6}, {NULL}};
const Segment SGI359184[] = {{&S3204[8], 3}, {NULL}};
const Segment SGI359185[] = {{&S3204[13], 9}, {NULL}};
const Segment SGI359186[] = {{&S3204[33], 15}, {NULL}};
const Segment SGI359187[] = {{&S3204[49], 6}, {NULL}};
const Segment SGI359188[] = {{&S3204[33], 13}, {NULL}};
const Segment SGI359189[] = {{&S3205[69], 6}, {NULL}};
const Segment SGI359190[] = {{&S3205[8], 3}, {NULL}};
const Segment SGI359191[] = {{&S3205[13], 9}, {NULL}};
const Segment SGI359192[] = {{&S3205[33], 15}, {NULL}};
const Segment SGI359193[] = {{&S3205[49], 6}, {NULL}};
const Segment SGI359194[] = {{&S3205[33], 13}, {NULL}};
const Segment SGI359195[] = {{&S3206[69], 6}, {NULL}};
const Segment SGI359196[] = {{&S3206[8], 3}, {NULL}};
const Segment SGI359197[] = {{&S3206[13], 9}, {NULL}};
const Segment SGI359198[] = {{&S3206[33], 15}, {NULL}};
const Segment SGI359199[] = {{&S3206[49], 6}, {NULL}};
const Segment SGI359200[] = {{&S3206[33], 13}, {NULL}};
const Segment SGI359201[] = {{&S3207[69], 6}, {NULL}};
const Segment SGI359202[] = {{&S3207[8], 3}, {NULL}};
const Segment SGI359203[] = {{&S3207[13], 9}, {NULL}};
const Segment SGI359204[] = {{&S3207[33], 15}, {NULL}};
const Segment SGI359205[] = {{&S3207[49], 6}, {NULL}};
const Segment SGI359206[] = {{&S3207[33], 13}, {NULL}};
const Segment SGI359207[] = {{&S3208[69], 6}, {NULL}};
const Segment SGI359208[] = {{&S3208[8], 3}, {NULL}};
const Segment SGI359209[] = {{&S3208[13], 9}, {NULL}};
const Segment SGI359210[] = {{&S3208[33], 15}, {NULL}};
const Segment SGI359211[] = {{&S3208[49], 6}, {NULL}};
const Segment SGI359212[] = {{&S3208[33], 13}, {NULL}};
const Segment SGI359213[] = {{&S3209[69], 6}, {NULL}};
const Segment SGI359214[] = {{&S3209[8], 3}, {NULL}};
const Segment SGI359215[] = {{&S3209[13], 9}, {NULL}};
const Segment SGI359216[] = {{&S3209[33], 15}, {NULL}};
const Segment SGI359217[] = {{&S3209[49], 6}, {NULL}};
const Segment SGI359218[] = {{&S3209[33], 13}, {NULL}};
const Segment SGI359219[] = {{&S3210[69], 6}, {NULL}};
const Segment SGI359220[] = {{&S3210[8], 3}, {NULL}};
const Segment SGI359221[] = {{&S3210[13], 9}, {NULL}};
const Segment SGI359222[] = {{&S3210[33], 15}, {NULL}};
const Segment SGI359223[] = {{&S3210[49], 6}, {NULL}};
const Segment SGI359224[] = {{&S3210[33], 13}, {NULL}};
const Segment SGI359225[] = {{&S3211[69], 6}, {NULL}};
const Segment SGI359226[] = {{&S3211[8], 3}, {NULL}};
const Segment SGI359227[] = {{&S3211[13], 9}, {NULL}};
const Segment SGI359228[] = {{&S3211[33], 15}, {NULL}};
const Segment SGI359229[] = {{&S3211[49], 6}, {NULL}};
const Segment SGI359230[] = {{&S3211[33], 13}, {NULL}};
const Segment SGI359231[] = {{&S3212[69], 6}, {NULL}};
const Segment SGI359232[] = {{&S3212[8], 3}, {NULL}};
const Segment SGI359233[] = {{&S3212[13], 9}, {NULL}};
const Segment SGI359234[] = {{&S3212[33], 15}, {NULL}};
const Segment SGI359235[] = {{&S3212[49], 6}, {NULL}};
const Segment SGI359236[] = {{&S3212[33], 13}, {NULL}};
const Segment SGI359237[] = {{&S3213[69], 6}, {NULL}};
const Segment SGI359238[] = {{&S3213[8], 3}, {NULL}};
const Segment SGI359239[] = {{&S3213[13], 9}, {NULL}};
const Segment SGI359240[] = {{&S3213[33], 15}, {NULL}};
const Segment SGI359241[] = {{&S3213[49], 6}, {NULL}};
const Segment SGI359242[] = {{&S3213[33], 13}, {NULL}};
const Segment SGI359243[] = {{&S3214[69], 6}, {NULL}};
const Segment SGI359244[] = {{&S3214[8], 3}, {NULL}};
const Segment SGI359245[] = {{&S3214[13], 9}, {NULL}};
const Segment SGI359246[] = {{&S3214[33], 15}, {NULL}};
const Segment SGI359247[] = {{&S3214[49], 6}, {NULL}};
const Segment SGI359248[] = {{&S3214[33], 13}, {NULL}};
const Segment SGI359249[] = {{&S3215[69], 6}, {NULL}};
const Segment SGI359250[] = {{&S3215[8], 3}, {NULL}};
const Segment SGI359251[] = {{&S3215[13], 9}, {NULL}};
const Segment SGI359252[] = {{&S3215[33], 15}, {NULL}};
const Segment SGI359253[] = {{&S3215[49], 6}, {NULL}};
const Segment SGI359254[] = {{&S3215[33], 13}, {NULL}};
const Segment SGI359255[] = {{&S3216[69], 6}, {NULL}};
const Segment SGI359256[] = {{&S3216[8], 3}, {NULL}};
const Segment SGI359257[] = {{&S3216[13], 9}, {NULL}};
const Segment SGI359258[] = {{&S3216[33], 15}, {NULL}};
const Segment SGI359259[] = {{&S3216[49], 6}, {NULL}};
const Segment SGI359260[] = {{&S3216[33], 13}, {NULL}};
const Segment SGI359261[] = {{&S3217[69], 6}, {NULL}};
const Segment SGI359262[] = {{&S3217[8], 3}, {NULL}};
const Segment SGI359263[] = {{&S3217[13], 9}, {NULL}};
const Segment SGI359264[] = {{&S3217[33], 15}, {NULL}};
const Segment SGI359265[] = {{&S3217[49], 6}, {NULL}};
const Segment SGI359266[] = {{&S3217[33], 13}, {NULL}};
const Segment SGI359267[] = {{&S3218[69], 6}, {NULL}};
const Segment SGI359268[] = {{&S3218[8], 3}, {NULL}};
const Segment SGI359269[] = {{&S3218[13], 9}, {NULL}};
const Segment SGI359270[] = {{&S3218[33], 15}, {NULL}};
const Segment SGI359271[] = {{&S3218[49], 6}, {NULL}};
const Segment SGI359272[] = {{&S3218[33], 13}, {NULL}};
const Segment SGI359273[] = {{&S3219[66], 6}, {NULL}};
const Segment SGI359274[] = {{&S3219[8], 3}, {NULL}};
const Segment SGI359275[] = {{&S3219[13], 9}, {NULL}};
const Segment SGI359276[] = {{&S3219[33], 15}, {NULL}};
const Segment SGI359277[] = {{&S3219[49], 3}, {NULL}};
const Segment SGI359278[] = {{&S3219[33], 13}, {NULL}};
const Segment SGI359279[] = {{&S3220[69], 6}, {NULL}};
const Segment SGI359280[] = {{&S3220[8], 3}, {NULL}};
const Segment SGI359281[] = {{&S3220[13], 9}, {NULL}};
const Segment SGI359282[] = {{&S3220[33], 15}, {NULL}};
const Segment SGI359283[] = {{&S3220[49], 6}, {NULL}};
const Segment SGI359284[] = {{&S3220[33], 13}, {NULL}};
const Segment SGI359285[] = {{&S3221[69], 6}, {NULL}};
const Segment SGI359286[] = {{&S3221[8], 3}, {NULL}};
const Segment SGI359287[] = {{&S3221[13], 9}, {NULL}};
const Segment SGI359288[] = {{&S3221[33], 15}, {NULL}};
const Segment SGI359289[] = {{&S3221[49], 6}, {NULL}};
const Segment SGI359290[] = {{&S3221[33], 13}, {NULL}};
const Segment SGI359291[] = {{&S3222[69], 6}, {NULL}};
const Segment SGI359292[] = {{&S3222[8], 3}, {NULL}};
const Segment SGI359293[] = {{&S3222[13], 9}, {NULL}};
const Segment SGI359294[] = {{&S3222[33], 15}, {NULL}};
const Segment SGI359295[] = {{&S3222[49], 6}, {NULL}};
const Segment SGI359296[] = {{&S3222[33], 13}, {NULL}};
const Segment SGI359297[] = {{&S3223[69], 6}, {NULL}};
const Segment SGI359298[] = {{&S3223[8], 3}, {NULL}};
const Segment SGI359299[] = {{&S3223[13], 9}, {NULL}};
const Segment SGI359300[] = {{&S3223[33], 15}, {NULL}};
const Segment SGI359301[] = {{&S3223[49], 6}, {NULL}};
const Segment SGI359302[] = {{&S3223[33], 13}, {NULL}};
const Segment SGI359303[] = {{&S3224[69], 6}, {NULL}};
const Segment SGI359304[] = {{&S3224[8], 3}, {NULL}};
const Segment SGI359305[] = {{&S3224[13], 9}, {NULL}};
const Segment SGI359306[] = {{&S3224[33], 15}, {NULL}};
const Segment SGI359307[] = {{&S3224[49], 6}, {NULL}};
const Segment SGI359308[] = {{&S3224[33], 13}, {NULL}};
const Segment SGI359309[] = {{&S3225[69], 6}, {NULL}};
const Segment SGI359310[] = {{&S3225[8], 3}, {NULL}};
const Segment SGI359311[] = {{&S3225[13], 9}, {NULL}};
const Segment SGI359312[] = {{&S3225[33], 15}, {NULL}};
const Segment SGI359313[] = {{&S3225[49], 6}, {NULL}};
const Segment SGI359314[] = {{&S3225[33], 13}, {NULL}};
const Segment SGI359315[] = {{&S3226[69], 6}, {NULL}};
const Segment SGI359316[] = {{&S3226[8], 3}, {NULL}};
const Segment SGI359317[] = {{&S3226[13], 9}, {NULL}};
const Segment SGI359318[] = {{&S3226[33], 15}, {NULL}};
const Segment SGI359319[] = {{&S3226[49], 6}, {NULL}};
const Segment SGI359320[] = {{&S3226[33], 13}, {NULL}};
const Segment SGI359321[] = {{&S3227[69], 6}, {NULL}};
const Segment SGI359322[] = {{&S3227[8], 3}, {NULL}};
const Segment SGI359323[] = {{&S3227[13], 9}, {NULL}};
const Segment SGI359324[] = {{&S3227[33], 15}, {NULL}};
const Segment SGI359325[] = {{&S3227[49], 6}, {NULL}};
const Segment SGI359326[] = {{&S3227[33], 13}, {NULL}};
const Segment SGI359327[] = {{&S3228[69], 6}, {NULL}};
const Segment SGI359328[] = {{&S3228[8], 3}, {NULL}};
const Segment SGI359329[] = {{&S3228[13], 9}, {NULL}};
const Segment SGI359330[] = {{&S3228[33], 15}, {NULL}};
const Segment SGI359331[] = {{&S3228[49], 6}, {NULL}};
const Segment SGI359332[] = {{&S3228[33], 13}, {NULL}};
const Segment SGI359333[] = {{&S3229[69], 6}, {NULL}};
const Segment SGI359334[] = {{&S3229[8], 3}, {NULL}};
const Segment SGI359335[] = {{&S3229[13], 9}, {NULL}};
const Segment SGI359336[] = {{&S3229[33], 15}, {NULL}};
const Segment SGI359337[] = {{&S3229[49], 6}, {NULL}};
const Segment SGI359338[] = {{&S3229[33], 13}, {NULL}};
const Segment SGI359339[] = {{&S3230[69], 6}, {NULL}};
const Segment SGI359340[] = {{&S3230[8], 3}, {NULL}};
const Segment SGI359341[] = {{&S3230[13], 9}, {NULL}};
const Segment SGI359342[] = {{&S3230[33], 15}, {NULL}};
const Segment SGI359343[] = {{&S3230[49], 6}, {NULL}};
const Segment SGI359344[] = {{&S3230[33], 13}, {NULL}};
const Segment SGI359345[] = {{&S3231[69], 6}, {NULL}};
const Segment SGI359346[] = {{&S3231[8], 3}, {NULL}};
const Segment SGI359347[] = {{&S3231[13], 9}, {NULL}};
const Segment SGI359348[] = {{&S3231[33], 15}, {NULL}};
const Segment SGI359349[] = {{&S3231[49], 6}, {NULL}};
const Segment SGI359350[] = {{&S3231[33], 13}, {NULL}};
const Segment SGI359351[] = {{&S3232[69], 6}, {NULL}};
const Segment SGI359352[] = {{&S3232[8], 3}, {NULL}};
const Segment SGI359353[] = {{&S3232[13], 9}, {NULL}};
const Segment SGI359354[] = {{&S3232[33], 15}, {NULL}};
const Segment SGI359355[] = {{&S3232[49], 6}, {NULL}};
const Segment SGI359356[] = {{&S3232[33], 13}, {NULL}};
const Segment SGI359357[] = {{&S3233[69], 6}, {NULL}};
const Segment SGI359358[] = {{&S3233[8], 3}, {NULL}};
const Segment SGI359359[] = {{&S3233[13], 9}, {NULL}};
const Segment SGI359360[] = {{&S3233[33], 15}, {NULL}};
const Segment SGI359361[] = {{&S3233[49], 6}, {NULL}};
const Segment SGI359362[] = {{&S3233[33], 13}, {NULL}};
const Segment SGI359363[] = {{&S3234[69], 6}, {NULL}};
const Segment SGI359364[] = {{&S3234[8], 3}, {NULL}};
const Segment SGI359365[] = {{&S3234[13], 9}, {NULL}};
const Segment SGI359366[] = {{&S3234[33], 15}, {NULL}};
const Segment SGI359367[] = {{&S3234[49], 6}, {NULL}};
const Segment SGI359368[] = {{&S3234[33], 13}, {NULL}};
const Segment SGI359369[] = {{&S3235[69], 6}, {NULL}};
const Segment SGI359370[] = {{&S3235[8], 3}, {NULL}};
const Segment SGI359371[] = {{&S3235[13], 9}, {NULL}};
const Segment SGI359372[] = {{&S3235[33], 15}, {NULL}};
const Segment SGI359373[] = {{&S3235[49], 6}, {NULL}};
const Segment SGI359374[] = {{&S3235[33], 13}, {NULL}};
const Segment SGI359375[] = {{&S3236[69], 6}, {NULL}};
const Segment SGI359376[] = {{&S3236[8], 3}, {NULL}};
const Segment SGI359377[] = {{&S3236[13], 9}, {NULL}};
const Segment SGI359378[] = {{&S3236[33], 15}, {NULL}};
const Segment SGI359379[] = {{&S3236[49], 6}, {NULL}};
const Segment SGI359380[] = {{&S3236[33], 13}, {NULL}};
const Segment SGI359381[] = {{&S3237[69], 6}, {NULL}};
const Segment SGI359382[] = {{&S3237[8], 3}, {NULL}};
const Segment SGI359383[] = {{&S3237[13], 9}, {NULL}};
const Segment SGI359384[] = {{&S3237[33], 15}, {NULL}};
const Segment SGI359385[] = {{&S3237[49], 6}, {NULL}};
const Segment SGI359386[] = {{&S3237[33], 13}, {NULL}};
const Segment SGI359387[] = {{&S3238[69], 6}, {NULL}};
const Segment SGI359388[] = {{&S3238[8], 3}, {NULL}};
const Segment SGI359389[] = {{&S3238[13], 9}, {NULL}};
const Segment SGI359390[] = {{&S3238[33], 15}, {NULL}};
const Segment SGI359391[] = {{&S3238[49], 6}, {NULL}};
const Segment SGI359392[] = {{&S3238[33], 13}, {NULL}};
const Segment SGI359393[] = {{&S3239[69], 6}, {NULL}};
const Segment SGI359394[] = {{&S3239[8], 3}, {NULL}};
const Segment SGI359395[] = {{&S3239[13], 9}, {NULL}};
const Segment SGI359396[] = {{&S3239[33], 15}, {NULL}};
const Segment SGI359397[] = {{&S3239[49], 6}, {NULL}};
const Segment SGI359398[] = {{&S3239[33], 13}, {NULL}};
const Segment SGI359399[] = {{&S3240[66], 6}, {NULL}};
const Segment SGI359400[] = {{&S3240[8], 3}, {NULL}};
const Segment SGI359401[] = {{&S3240[13], 9}, {NULL}};
const Segment SGI359402[] = {{&S3240[33], 15}, {NULL}};
const Segment SGI359403[] = {{&S3240[49], 3}, {NULL}};
const Segment SGI359404[] = {{&S3240[33], 13}, {NULL}};
const Segment SGI359405[] = {{&S3241[69], 6}, {NULL}};
const Segment SGI359406[] = {{&S3241[8], 3}, {NULL}};
const Segment SGI359407[] = {{&S3241[13], 9}, {NULL}};
const Segment SGI359408[] = {{&S3241[33], 15}, {NULL}};
const Segment SGI359409[] = {{&S3241[49], 6}, {NULL}};
const Segment SGI359410[] = {{&S3241[33], 13}, {NULL}};
const Segment SGI359411[] = {{&S3242[69], 6}, {NULL}};
const Segment SGI359412[] = {{&S3242[8], 3}, {NULL}};
const Segment SGI359413[] = {{&S3242[13], 9}, {NULL}};
const Segment SGI359414[] = {{&S3242[33], 15}, {NULL}};
const Segment SGI359415[] = {{&S3242[49], 6}, {NULL}};
const Segment SGI359416[] = {{&S3242[33], 13}, {NULL}};
const Segment SGI359417[] = {{&S3243[69], 6}, {NULL}};
const Segment SGI359418[] = {{&S3243[8], 3}, {NULL}};
const Segment SGI359419[] = {{&S3243[13], 9}, {NULL}};
const Segment SGI359420[] = {{&S3243[33], 15}, {NULL}};
const Segment SGI359421[] = {{&S3243[49], 6}, {NULL}};
const Segment SGI359422[] = {{&S3243[33], 13}, {NULL}};
const Segment SGI359423[] = {{&S3244[69], 6}, {NULL}};
const Segment SGI359424[] = {{&S3244[8], 3}, {NULL}};
const Segment SGI359425[] = {{&S3244[13], 9}, {NULL}};
const Segment SGI359426[] = {{&S3244[33], 15}, {NULL}};
const Segment SGI359427[] = {{&S3244[49], 6}, {NULL}};
const Segment SGI359428[] = {{&S3244[33], 13}, {NULL}};
const Segment SGI359429[] = {{&S3245[69], 6}, {NULL}};
const Segment SGI359430[] = {{&S3245[8], 3}, {NULL}};
const Segment SGI359431[] = {{&S3245[13], 9}, {NULL}};
const Segment SGI359432[] = {{&S3245[33], 15}, {NULL}};
const Segment SGI359433[] = {{&S3245[49], 6}, {NULL}};
const Segment SGI359434[] = {{&S3245[33], 13}, {NULL}};
const Segment SGI359435[] = {{&S3246[69], 6}, {NULL}};
const Segment SGI359436[] = {{&S3246[8], 3}, {NULL}};
const Segment SGI359437[] = {{&S3246[13], 9}, {NULL}};
const Segment SGI359438[] = {{&S3246[33], 15}, {NULL}};
const Segment SGI359439[] = {{&S3246[49], 6}, {NULL}};
const Segment SGI359440[] = {{&S3246[33], 13}, {NULL}};
const Segment SGI359441[] = {{&S3247[69], 6}, {NULL}};
const Segment SGI359442[] = {{&S3247[8], 3}, {NULL}};
const Segment SGI359443[] = {{&S3247[13], 9}, {NULL}};
const Segment SGI359444[] = {{&S3247[33], 15}, {NULL}};
const Segment SGI359445[] = {{&S3247[49], 6}, {NULL}};
const Segment SGI359446[] = {{&S3247[33], 13}, {NULL}};
const Segment SGI359447[] = {{&S3248[69], 6}, {NULL}};
const Segment SGI359448[] = {{&S3248[8], 3}, {NULL}};
const Segment SGI359449[] = {{&S3248[13], 9}, {NULL}};
const Segment SGI359450[] = {{&S3248[33], 15}, {NULL}};
const Segment SGI359451[] = {{&S3248[49], 6}, {NULL}};
const Segment SGI359452[] = {{&S3248[33], 13}, {NULL}};
const Segment SGI359453[] = {{&S3249[69], 6}, {NULL}};
const Segment SGI359454[] = {{&S3249[8], 3}, {NULL}};
const Segment SGI359455[] = {{&S3249[13], 9}, {NULL}};
const Segment SGI359456[] = {{&S3249[33], 15}, {NULL}};
const Segment SGI359457[] = {{&S3249[49], 6}, {NULL}};
const Segment SGI359458[] = {{&S3249[33], 13}, {NULL}};
const Segment SGI359459[] = {{&S3250[69], 6}, {NULL}};
const Segment SGI359460[] = {{&S3250[8], 3}, {NULL}};
const Segment SGI359461[] = {{&S3250[13], 9}, {NULL}};
const Segment SGI359462[] = {{&S3250[33], 15}, {NULL}};
const Segment SGI359463[] = {{&S3250[49], 6}, {NULL}};
const Segment SGI359464[] = {{&S3250[33], 13}, {NULL}};
const Segment SGI359465[] = {{&S3251[69], 6}, {NULL}};
const Segment SGI359466[] = {{&S3251[8], 3}, {NULL}};
const Segment SGI359467[] = {{&S3251[13], 9}, {NULL}};
const Segment SGI359468[] = {{&S3251[33], 15}, {NULL}};
const Segment SGI359469[] = {{&S3251[49], 6}, {NULL}};
const Segment SGI359470[] = {{&S3251[33], 13}, {NULL}};
const Segment SGI359471[] = {{&S3252[69], 6}, {NULL}};
const Segment SGI359472[] = {{&S3252[8], 3}, {NULL}};
const Segment SGI359473[] = {{&S3252[13], 9}, {NULL}};
const Segment SGI359474[] = {{&S3252[33], 15}, {NULL}};
const Segment SGI359475[] = {{&S3252[49], 6}, {NULL}};
const Segment SGI359476[] = {{&S3252[33], 13}, {NULL}};
const Segment SGI359477[] = {{&S3253[69], 6}, {NULL}};
const Segment SGI359478[] = {{&S3253[8], 3}, {NULL}};
const Segment SGI359479[] = {{&S3253[13], 9}, {NULL}};
const Segment SGI359480[] = {{&S3253[33], 15}, {NULL}};
const Segment SGI359481[] = {{&S3253[49], 6}, {NULL}};
const Segment SGI359482[] = {{&S3253[33], 13}, {NULL}};
const Segment SGI359483[] = {{&S3254[69], 6}, {NULL}};
const Segment SGI359484[] = {{&S3254[8], 3}, {NULL}};
const Segment SGI359485[] = {{&S3254[13], 9}, {NULL}};
const Segment SGI359486[] = {{&S3254[33], 15}, {NULL}};
const Segment SGI359487[] = {{&S3254[49], 6}, {NULL}};
const Segment SGI359488[] = {{&S3254[33], 13}, {NULL}};
const Segment SGI359489[] = {{&S3255[69], 6}, {NULL}};
const Segment SGI359490[] = {{&S3255[8], 3}, {NULL}};
const Segment SGI359491[] = {{&S3255[13], 9}, {NULL}};
const Segment SGI359492[] = {{&S3255[33], 15}, {NULL}};
const Segment SGI359493[] = {{&S3255[49], 6}, {NULL}};
const Segment SGI359494[] = {{&S3255[33], 13}, {NULL}};
const Segment SGI359495[] = {{&S3256[69], 6}, {NULL}};
const Segment SGI359496[] = {{&S3256[8], 3}, {NULL}};
const Segment SGI359497[] = {{&S3256[13], 9}, {NULL}};
const Segment SGI359498[] = {{&S3256[33], 15}, {NULL}};
const Segment SGI359499[] = {{&S3256[49], 6}, {NULL}};
const Segment SGI359500[] = {{&S3256[33], 13}, {NULL}};
const Segment SGI359501[] = {{&S3257[69], 6}, {NULL}};
const Segment SGI359502[] = {{&S3257[8], 3}, {NULL}};
const Segment SGI359503[] = {{&S3257[13], 9}, {NULL}};
const Segment SGI359504[] = {{&S3257[33], 15}, {NULL}};
const Segment SGI359505[] = {{&S3257[49], 6}, {NULL}};
const Segment SGI359506[] = {{&S3257[33], 13}, {NULL}};
const Segment SGI359507[] = {{&S3258[69], 6}, {NULL}};
const Segment SGI359508[] = {{&S3258[8], 3}, {NULL}};
const Segment SGI359509[] = {{&S3258[13], 9}, {NULL}};
const Segment SGI359510[] = {{&S3258[33], 15}, {NULL}};
const Segment SGI359511[] = {{&S3258[49], 6}, {NULL}};
const Segment SGI359512[] = {{&S3258[33], 13}, {NULL}};
const Segment SGI359513[] = {{&S3259[69], 6}, {NULL}};
const Segment SGI359514[] = {{&S3259[8], 3}, {NULL}};
const Segment SGI359515[] = {{&S3259[13], 9}, {NULL}};
const Segment SGI359516[] = {{&S3259[33], 15}, {NULL}};
const Segment SGI359517[] = {{&S3259[49], 6}, {NULL}};
const Segment SGI359518[] = {{&S3259[33], 13}, {NULL}};
const Segment SGI359519[] = {{&S3260[69], 6}, {NULL}};
const Segment SGI359520[] = {{&S3260[8], 3}, {NULL}};
const Segment SGI359521[] = {{&S3260[13], 9}, {NULL}};
const Segment SGI359522[] = {{&S3260[33], 15}, {NULL}};
const Segment SGI359523[] = {{&S3260[49], 6}, {NULL}};
const Segment SGI359524[] = {{&S3260[33], 13}, {NULL}};
const Segment SGI359525[] = {{&S3261[69], 6}, {NULL}};
const Segment SGI359526[] = {{&S3261[8], 3}, {NULL}};
const Segment SGI359527[] = {{&S3261[13], 9}, {NULL}};
const Segment SGI359528[] = {{&S3261[33], 15}, {NULL}};
const Segment SGI359529[] = {{&S3261[49], 6}, {NULL}};
const Segment SGI359530[] = {{&S3261[33], 13}, {NULL}};
const Segment SGI359531[] = {{&S3262[69], 6}, {NULL}};
const Segment SGI359532[] = {{&S3262[8], 3}, {NULL}};
const Segment SGI359533[] = {{&S3262[13], 9}, {NULL}};
const Segment SGI359534[] = {{&S3262[33], 15}, {NULL}};
const Segment SGI359535[] = {{&S3262[49], 6}, {NULL}};
const Segment SGI359536[] = {{&S3262[33], 13}, {NULL}};
const Segment SGI359537[] = {{&S3263[69], 6}, {NULL}};
const Segment SGI359538[] = {{&S3263[8], 3}, {NULL}};
const Segment SGI359539[] = {{&S3263[13], 9}, {NULL}};
const Segment SGI359540[] = {{&S3263[33], 15}, {NULL}};
const Segment SGI359541[] = {{&S3263[49], 6}, {NULL}};
const Segment SGI359542[] = {{&S3263[33], 13}, {NULL}};
const Segment SGI359543[] = {{&S3264[69], 6}, {NULL}};
const Segment SGI359544[] = {{&S3264[8], 3}, {NULL}};
const Segment SGI359545[] = {{&S3264[13], 9}, {NULL}};
const Segment SGI359546[] = {{&S3264[33], 15}, {NULL}};
const Segment SGI359547[] = {{&S3264[49], 6}, {NULL}};
const Segment SGI359548[] = {{&S3264[33], 13}, {NULL}};
const Segment SGI359549[] = {{&S3265[69], 6}, {NULL}};
const Segment SGI359550[] = {{&S3265[8], 3}, {NULL}};
const Segment SGI359551[] = {{&S3265[13], 9}, {NULL}};
const Segment SGI359552[] = {{&S3265[33], 15}, {NULL}};
const Segment SGI359553[] = {{&S3265[49], 6}, {NULL}};
const Segment SGI359554[] = {{&S3265[33], 13}, {NULL}};
const Segment SGI359555[] = {{&S3266[69], 6}, {NULL}};
const Segment SGI359556[] = {{&S3266[8], 3}, {NULL}};
const Segment SGI359557[] = {{&S3266[13], 9}, {NULL}};
const Segment SGI359558[] = {{&S3266[33], 15}, {NULL}};
const Segment SGI359559[] = {{&S3266[49], 6}, {NULL}};
const Segment SGI359560[] = {{&S3266[33], 13}, {NULL}};
const Segment SGI359561[] = {{&S3267[69], 6}, {NULL}};
const Segment SGI359562[] = {{&S3267[8], 3}, {NULL}};
const Segment SGI359563[] = {{&S3267[13], 9}, {NULL}};
const Segment SGI359564[] = {{&S3267[33], 15}, {NULL}};
const Segment SGI359565[] = {{&S3267[49], 6}, {NULL}};
const Segment SGI359566[] = {{&S3267[33], 13}, {NULL}};
const Segment SGI359567[] = {{&S3268[69], 6}, {NULL}};
const Segment SGI359568[] = {{&S3268[8], 3}, {NULL}};
const Segment SGI359569[] = {{&S3268[13], 9}, {NULL}};
const Segment SGI359570[] = {{&S3268[33], 15}, {NULL}};
const Segment SGI359571[] = {{&S3268[49], 6}, {NULL}};
const Segment SGI359572[] = {{&S3268[33], 13}, {NULL}};
const Segment SGI359573[] = {{&S3269[69], 6}, {NULL}};
const Segment SGI359574[] = {{&S3269[8], 3}, {NULL}};
const Segment SGI359575[] = {{&S3269[13], 9}, {NULL}};
const Segment SGI359576[] = {{&S3269[33], 15}, {NULL}};
const Segment SGI359577[] = {{&S3269[49], 6}, {NULL}};
const Segment SGI359578[] = {{&S3269[33], 13}, {NULL}};
const Segment SGI359579[] = {{&S3270[69], 6}, {NULL}};
const Segment SGI359580[] = {{&S3270[8], 3}, {NULL}};
const Segment SGI359581[] = {{&S3270[13], 9}, {NULL}};
const Segment SGI359582[] = {{&S3270[33], 15}, {NULL}};
const Segment SGI359583[] = {{&S3270[49], 6}, {NULL}};
const Segment SGI359584[] = {{&S3270[33], 13}, {NULL}};
const Segment SGI359585[] = {{&S3271[69], 6}, {NULL}};
const Segment SGI359586[] = {{&S3271[8], 3}, {NULL}};
const Segment SGI359587[] = {{&S3271[13], 9}, {NULL}};
const Segment SGI359588[] = {{&S3271[33], 15}, {NULL}};
const Segment SGI359589[] = {{&S3271[49], 6}, {NULL}};
const Segment SGI359590[] = {{&S3271[33], 13}, {NULL}};
const Segment SGI359591[] = {{&S3272[69], 6}, {NULL}};
const Segment SGI359592[] = {{&S3272[8], 3}, {NULL}};
const Segment SGI359593[] = {{&S3272[13], 9}, {NULL}};
const Segment SGI359594[] = {{&S3272[33], 15}, {NULL}};
const Segment SGI359595[] = {{&S3272[49], 6}, {NULL}};
const Segment SGI359596[] = {{&S3272[33], 13}, {NULL}};
const Segment SGI359597[] = {{&S3273[69], 6}, {NULL}};
const Segment SGI359598[] = {{&S3273[8], 3}, {NULL}};
const Segment SGI359599[] = {{&S3273[13], 9}, {NULL}};
const Segment SGI359600[] = {{&S3273[33], 15}, {NULL}};
const Segment SGI359601[] = {{&S3273[49], 6}, {NULL}};
const Segment SGI359602[] = {{&S3273[33], 13}, {NULL}};
const Segment SGI359603[] = {{&S3274[69], 6}, {NULL}};
const Segment SGI359604[] = {{&S3274[8], 3}, {NULL}};
const Segment SGI359605[] = {{&S3274[13], 9}, {NULL}};
const Segment SGI359606[] = {{&S3274[33], 15}, {NULL}};
const Segment SGI359607[] = {{&S3274[49], 6}, {NULL}};
const Segment SGI359608[] = {{&S3274[33], 13}, {NULL}};
const Segment SGI359609[] = {{&S3275[69], 6}, {NULL}};
const Segment SGI359610[] = {{&S3275[8], 3}, {NULL}};
const Segment SGI359611[] = {{&S3275[13], 9}, {NULL}};
const Segment SGI359612[] = {{&S3275[33], 15}, {NULL}};
const Segment SGI359613[] = {{&S3275[49], 6}, {NULL}};
const Segment SGI359614[] = {{&S3275[33], 13}, {NULL}};
const Segment SGI359615[] = {{&S3276[69], 6}, {NULL}};
const Segment SGI359616[] = {{&S3276[8], 3}, {NULL}};
const Segment SGI359617[] = {{&S3276[13], 9}, {NULL}};
const Segment SGI359618[] = {{&S3276[33], 15}, {NULL}};
const Segment SGI359619[] = {{&S3276[49], 6}, {NULL}};
const Segment SGI359620[] = {{&S3276[33], 13}, {NULL}};
const Segment SGI359621[] = {{&S3277[69], 6}, {NULL}};
const Segment SGI359622[] = {{&S3277[8], 3}, {NULL}};
const Segment SGI359623[] = {{&S3277[13], 9}, {NULL}};
const Segment SGI359624[] = {{&S3277[33], 15}, {NULL}};
const Segment SGI359625[] = {{&S3277[49], 6}, {NULL}};
const Segment SGI359626[] = {{&S3277[33], 13}, {NULL}};
const Segment SGI359627[] = {{&S3278[69], 6}, {NULL}};
const Segment SGI359628[] = {{&S3278[8], 3}, {NULL}};
const Segment SGI359629[] = {{&S3278[13], 9}, {NULL}};
const Segment SGI359630[] = {{&S3278[33], 15}, {NULL}};
const Segment SGI359631[] = {{&S3278[49], 6}, {NULL}};
const Segment SGI359632[] = {{&S3278[33], 13}, {NULL}};
const Segment SGI359633[] = {{&S3279[69], 6}, {NULL}};
const Segment SGI359634[] = {{&S3279[8], 3}, {NULL}};
const Segment SGI359635[] = {{&S3279[13], 9}, {NULL}};
const Segment SGI359636[] = {{&S3279[33], 15}, {NULL}};
const Segment SGI359637[] = {{&S3279[49], 6}, {NULL}};
const Segment SGI359638[] = {{&S3279[33], 13}, {NULL}};
const Segment SGI359639[] = {{&S3280[69], 6}, {NULL}};
const Segment SGI359640[] = {{&S3280[8], 3}, {NULL}};
const Segment SGI359641[] = {{&S3280[13], 9}, {NULL}};
const Segment SGI359642[] = {{&S3280[33], 15}, {NULL}};
const Segment SGI359643[] = {{&S3280[49], 6}, {NULL}};
const Segment SGI359644[] = {{&S3280[33], 13}, {NULL}};
const Segment SGI359645[] = {{&S3281[69], 6}, {NULL}};
const Segment SGI359646[] = {{&S3281[8], 3}, {NULL}};
const Segment SGI359647[] = {{&S3281[13], 9}, {NULL}};
const Segment SGI359648[] = {{&S3281[33], 15}, {NULL}};
const Segment SGI359649[] = {{&S3281[49], 6}, {NULL}};
const Segment SGI359650[] = {{&S3281[33], 13}, {NULL}};
const Segment SGI359651[] = {{&S3282[69], 6}, {NULL}};
const Segment SGI359652[] = {{&S3282[8], 3}, {NULL}};
const Segment SGI359653[] = {{&S3282[13], 9}, {NULL}};
const Segment SGI359654[] = {{&S3282[33], 15}, {NULL}};
const Segment SGI359655[] = {{&S3282[49], 6}, {NULL}};
const Segment SGI359656[] = {{&S3282[33], 13}, {NULL}};
const Segment SGI359657[] = {{&S3283[69], 6}, {NULL}};
const Segment SGI359658[] = {{&S3283[8], 3}, {NULL}};
const Segment SGI359659[] = {{&S3283[13], 9}, {NULL}};
const Segment SGI359660[] = {{&S3283[33], 15}, {NULL}};
const Segment SGI359661[] = {{&S3283[49], 6}, {NULL}};
const Segment SGI359662[] = {{&S3283[33], 13}, {NULL}};
const Segment SGI359663[] = {{&S3284[69], 6}, {NULL}};
const Segment SGI359664[] = {{&S3284[8], 3}, {NULL}};
const Segment SGI359665[] = {{&S3284[13], 9}, {NULL}};
const Segment SGI359666[] = {{&S3284[33], 15}, {NULL}};
const Segment SGI359667[] = {{&S3284[49], 6}, {NULL}};
const Segment SGI359668[] = {{&S3284[33], 13}, {NULL}};
const Segment SGI359669[] = {{&S3285[69], 6}, {NULL}};
const Segment SGI359670[] = {{&S3285[8], 3}, {NULL}};
const Segment SGI359671[] = {{&S3285[13], 9}, {NULL}};
const Segment SGI359672[] = {{&S3285[33], 15}, {NULL}};
const Segment SGI359673[] = {{&S3285[49], 6}, {NULL}};
const Segment SGI359674[] = {{&S3285[33], 13}, {NULL}};
const Segment SGI359675[] = {{&S3286[69], 6}, {NULL}};
const Segment SGI359676[] = {{&S3286[8], 3}, {NULL}};
const Segment SGI359677[] = {{&S3286[13], 9}, {NULL}};
const Segment SGI359678[] = {{&S3286[33], 15}, {NULL}};
const Segment SGI359679[] = {{&S3286[49], 6}, {NULL}};
const Segment SGI359680[] = {{&S3286[33], 13}, {NULL}};
const Segment SGI359681[] = {{&S3287[69], 6}, {NULL}};
const Segment SGI359682[] = {{&S3287[8], 3}, {NULL}};
const Segment SGI359683[] = {{&S3287[13], 9}, {NULL}};
const Segment SGI359684[] = {{&S3287[33], 15}, {NULL}};
const Segment SGI359685[] = {{&S3287[49], 6}, {NULL}};
const Segment SGI359686[] = {{&S3287[33], 13}, {NULL}};
const Segment SGI359687[] = {{&S3288[69], 6}, {NULL}};
const Segment SGI359688[] = {{&S3288[8], 3}, {NULL}};
const Segment SGI359689[] = {{&S3288[13], 9}, {NULL}};
const Segment SGI359690[] = {{&S3288[33], 15}, {NULL}};
const Segment SGI359691[] = {{&S3288[49], 6}, {NULL}};
const Segment SGI359692[] = {{&S3288[33], 13}, {NULL}};
const Segment SGI359693[] = {{&S3289[69], 6}, {NULL}};
const Segment SGI359694[] = {{&S3289[8], 3}, {NULL}};
const Segment SGI359695[] = {{&S3289[13], 9}, {NULL}};
const Segment SGI359696[] = {{&S3289[33], 15}, {NULL}};
const Segment SGI359697[] = {{&S3289[49], 6}, {NULL}};
const Segment SGI359698[] = {{&S3289[33], 13}, {NULL}};
const Segment SGI359699[] = {{&S3290[69], 6}, {NULL}};
const Segment SGI359700[] = {{&S3290[8], 3}, {NULL}};
const Segment SGI359701[] = {{&S3290[13], 9}, {NULL}};
const Segment SGI359702[] = {{&S3290[33], 15}, {NULL}};
const Segment SGI359703[] = {{&S3290[49], 6}, {NULL}};
const Segment SGI359704[] = {{&S3290[33], 13}, {NULL}};
const Segment SGI359705[] = {{&S3291[69], 6}, {NULL}};
const Segment SGI359706[] = {{&S3291[8], 3}, {NULL}};
const Segment SGI359707[] = {{&S3291[13], 9}, {NULL}};
const Segment SGI359708[] = {{&S3291[33], 15}, {NULL}};
const Segment SGI359709[] = {{&S3291[49], 6}, {NULL}};
const Segment SGI359710[] = {{&S3291[33], 13}, {NULL}};
const Segment SGI359711[] = {{&S3292[105], 7}, {NULL}};
const Segment SGI359712[] = {{&S3292[8], 3}, {NULL}};
const Segment SGI359713[] = {{&S3292[13], 9}, {NULL}};
const Segment SGI359714[] = {{&S3292[33], 15}, {NULL}};
const Segment SGI359715[] = {{&S3292[49], 6}, {NULL}};
const Segment SGI359716[] = {{&S3292[33], 13}, {NULL}};
const Segment SGI359717[] = {{&S3293[69], 6}, {NULL}};
const Segment SGI359718[] = {{&S3293[8], 3}, {NULL}};
const Segment SGI359719[] = {{&S3293[13], 9}, {NULL}};
const Segment SGI359720[] = {{&S3293[33], 15}, {NULL}};
const Segment SGI359721[] = {{&S3293[49], 6}, {NULL}};
const Segment SGI359722[] = {{&S3293[33], 13}, {NULL}};
const Segment SGI359723[] = {{&S3294[69], 6}, {NULL}};
const Segment SGI359724[] = {{&S3294[8], 3}, {NULL}};
const Segment SGI359725[] = {{&S3294[13], 9}, {NULL}};
const Segment SGI359726[] = {{&S3294[33], 15}, {NULL}};
const Segment SGI359727[] = {{&S3294[49], 6}, {NULL}};
const Segment SGI359728[] = {{&S3294[33], 13}, {NULL}};
const Segment SGI359729[] = {{&S3295[69], 6}, {NULL}};
const Segment SGI359730[] = {{&S3295[8], 3}, {NULL}};
const Segment SGI359731[] = {{&S3295[13], 9}, {NULL}};
const Segment SGI359732[] = {{&S3295[33], 15}, {NULL}};
const Segment SGI359733[] = {{&S3295[49], 6}, {NULL}};
const Segment SGI359734[] = {{&S3295[33], 13}, {NULL}};
const Segment SGI359735[] = {{&S3296[69], 6}, {NULL}};
const Segment SGI359736[] = {{&S3296[8], 3}, {NULL}};
const Segment SGI359737[] = {{&S3296[13], 9}, {NULL}};
const Segment SGI359738[] = {{&S3296[33], 15}, {NULL}};
const Segment SGI359739[] = {{&S3296[49], 6}, {NULL}};
const Segment SGI359740[] = {{&S3296[33], 13}, {NULL}};
const Segment SGI359741[] = {{&S3297[69], 6}, {NULL}};
const Segment SGI359742[] = {{&S3297[8], 3}, {NULL}};
const Segment SGI359743[] = {{&S3297[13], 9}, {NULL}};
const Segment SGI359744[] = {{&S3297[33], 15}, {NULL}};
const Segment SGI359745[] = {{&S3297[49], 6}, {NULL}};
const Segment SGI359746[] = {{&S3297[33], 13}, {NULL}};
const Segment SGI359747[] = {{&S3298[69], 6}, {NULL}};
const Segment SGI359748[] = {{&S3298[8], 3}, {NULL}};
const Segment SGI359749[] = {{&S3298[13], 9}, {NULL}};
const Segment SGI359750[] = {{&S3298[33], 15}, {NULL}};
const Segment SGI359751[] = {{&S3298[49], 6}, {NULL}};
const Segment SGI359752[] = {{&S3298[33], 13}, {NULL}};
const Segment SGI359753[] = {{&S3299[69], 6}, {NULL}};
const Segment SGI359754[] = {{&S3299[8], 3}, {NULL}};
const Segment SGI359755[] = {{&S3299[13], 9}, {NULL}};
const Segment SGI359756[] = {{&S3299[33], 15}, {NULL}};
const Segment SGI359757[] = {{&S3299[49], 6}, {NULL}};
const Segment SGI359758[] = {{&S3299[33], 13}, {NULL}};
const Segment SGI359759[] = {{&S3300[69], 6}, {NULL}};
const Segment SGI359760[] = {{&S3300[8], 3}, {NULL}};
const Segment SGI359761[] = {{&S3300[13], 9}, {NULL}};
const Segment SGI359762[] = {{&S3300[33], 15}, {NULL}};
const Segment SGI359763[] = {{&S3300[49], 6}, {NULL}};
const Segment SGI359764[] = {{&S3300[33], 13}, {NULL}};
const Segment SGI359765[] = {{&S3301[69], 6}, {NULL}};
const Segment SGI359766[] = {{&S3301[8], 3}, {NULL}};
const Segment SGI359767[] = {{&S3301[13], 9}, {NULL}};
const Segment SGI359768[] = {{&S3301[33], 15}, {NULL}};
const Segment SGI359769[] = {{&S3301[49], 6}, {NULL}};
const Segment SGI359770[] = {{&S3301[33], 13}, {NULL}};
const Segment SGI359771[] = {{&S3302[69], 6}, {NULL}};
const Segment SGI359772[] = {{&S3302[8], 3}, {NULL}};
const Segment SGI359773[] = {{&S3302[13], 9}, {NULL}};
const Segment SGI359774[] = {{&S3302[33], 15}, {NULL}};
const Segment SGI359775[] = {{&S3302[49], 6}, {NULL}};
const Segment SGI359776[] = {{&S3302[33], 13}, {NULL}};
const Segment SGI359777[] = {{&S3303[69], 6}, {NULL}};
const Segment SGI359778[] = {{&S3303[8], 3}, {NULL}};
const Segment SGI359779[] = {{&S3303[13], 9}, {NULL}};
const Segment SGI359780[] = {{&S3303[33], 15}, {NULL}};
const Segment SGI359781[] = {{&S3303[49], 6}, {NULL}};
const Segment SGI359782[] = {{&S3303[33], 13}, {NULL}};
const Segment SGI359783[] = {{&S3304[69], 6}, {NULL}};
const Segment SGI359784[] = {{&S3304[8], 3}, {NULL}};
const Segment SGI359785[] = {{&S3304[13], 9}, {NULL}};
const Segment SGI359786[] = {{&S3304[33], 15}, {NULL}};
const Segment SGI359787[] = {{&S3304[49], 6}, {NULL}};
const Segment SGI359788[] = {{&S3304[33], 13}, {NULL}};
const Segment SGI359789[] = {{&S3305[69], 6}, {NULL}};
const Segment SGI359790[] = {{&S3305[8], 3}, {NULL}};
const Segment SGI359791[] = {{&S3305[13], 9}, {NULL}};
const Segment SGI359792[] = {{&S3305[33], 15}, {NULL}};
const Segment SGI359793[] = {{&S3305[49], 6}, {NULL}};
const Segment SGI359794[] = {{&S3305[33], 13}, {NULL}};
const Segment SGI359795[] = {{&S3306[69], 6}, {NULL}};
const Segment SGI359796[] = {{&S3306[8], 3}, {NULL}};
const Segment SGI359797[] = {{&S3306[13], 9}, {NULL}};
const Segment SGI359798[] = {{&S3306[33], 15}, {NULL}};
const Segment SGI359799[] = {{&S3306[49], 6}, {NULL}};
const Segment SGI359800[] = {{&S3306[33], 13}, {NULL}};
const Segment SGI359801[] = {{&S3307[69], 6}, {NULL}};
const Segment SGI359802[] = {{&S3307[8], 3}, {NULL}};
const Segment SGI359803[] = {{&S3307[13], 9}, {NULL}};
const Segment SGI359804[] = {{&S3307[33], 15}, {NULL}};
const Segment SGI359805[] = {{&S3307[49], 6}, {NULL}};
const Segment SGI359806[] = {{&S3307[33], 13}, {NULL}};
const Segment SGI359807[] = {{&S3308[69], 6}, {NULL}};
const Segment SGI359808[] = {{&S3308[8], 3}, {NULL}};
const Segment SGI359809[] = {{&S3308[13], 9}, {NULL}};
const Segment SGI359810[] = {{&S3308[33], 15}, {NULL}};
const Segment SGI359811[] = {{&S3308[49], 6}, {NULL}};
const Segment SGI359812[] = {{&S3308[33], 13}, {NULL}};
const Segment SGI359813[] = {{&S3309[69], 6}, {NULL}};
const Segment SGI359814[] = {{&S3309[8], 3}, {NULL}};
const Segment SGI359815[] = {{&S3309[13], 9}, {NULL}};
const Segment SGI359816[] = {{&S3309[33], 15}, {NULL}};
const Segment SGI359817[] = {{&S3309[49], 6}, {NULL}};
const Segment SGI359818[] = {{&S3309[33], 13}, {NULL}};
const Segment SGI359819[] = {{&S3310[69], 6}, {NULL}};
const Segment SGI359820[] = {{&S3310[8], 3}, {NULL}};
const Segment SGI359821[] = {{&S3310[13], 9}, {NULL}};
const Segment SGI359822[] = {{&S3310[33], 15}, {NULL}};
const Segment SGI359823[] = {{&S3310[49], 6}, {NULL}};
const Segment SGI359824[] = {{&S3310[33], 13}, {NULL}};
const Segment SGI359825[] = {{&S3311[69], 6}, {NULL}};
const Segment SGI359826[] = {{&S3311[8], 3}, {NULL}};
const Segment SGI359827[] = {{&S3311[13], 9}, {NULL}};
const Segment SGI359828[] = {{&S3311[33], 15}, {NULL}};
const Segment SGI359829[] = {{&S3311[49], 6}, {NULL}};
const Segment SGI359830[] = {{&S3311[33], 13}, {NULL}};
const Segment SGI359831[] = {{&S3312[69], 6}, {NULL}};
const Segment SGI359832[] = {{&S3312[8], 3}, {NULL}};
const Segment SGI359833[] = {{&S3312[13], 9}, {NULL}};
const Segment SGI359834[] = {{&S3312[33], 15}, {NULL}};
const Segment SGI359835[] = {{&S3312[49], 6}, {NULL}};
const Segment SGI359836[] = {{&S3312[33], 13}, {NULL}};
const Segment SGI359837[] = {{&S3313[69], 6}, {NULL}};
const Segment SGI359838[] = {{&S3313[8], 3}, {NULL}};
const Segment SGI359839[] = {{&S3313[13], 9}, {NULL}};
const Segment SGI359840[] = {{&S3313[33], 15}, {NULL}};
const Segment SGI359841[] = {{&S3313[49], 6}, {NULL}};
const Segment SGI359842[] = {{&S3313[33], 13}, {NULL}};
const Segment SGI359843[] = {{&S3314[69], 6}, {NULL}};
const Segment SGI359844[] = {{&S3314[8], 3}, {NULL}};
const Segment SGI359845[] = {{&S3314[13], 9}, {NULL}};
const Segment SGI359846[] = {{&S3314[33], 15}, {NULL}};
const Segment SGI359847[] = {{&S3314[49], 6}, {NULL}};
const Segment SGI359848[] = {{&S3314[33], 13}, {NULL}};
const Segment SGI359849[] = {{&S3315[69], 6}, {NULL}};
const Segment SGI359850[] = {{&S3315[8], 3}, {NULL}};
const Segment SGI359851[] = {{&S3315[13], 9}, {NULL}};
const Segment SGI359852[] = {{&S3315[33], 15}, {NULL}};
const Segment SGI359853[] = {{&S3315[49], 6}, {NULL}};
const Segment SGI359854[] = {{&S3315[33], 13}, {NULL}};
const Segment SGI359855[] = {{&S3316[69], 6}, {NULL}};
const Segment SGI359856[] = {{&S3316[8], 3}, {NULL}};
const Segment SGI359857[] = {{&S3316[13], 9}, {NULL}};
const Segment SGI359858[] = {{&S3316[33], 15}, {NULL}};
const Segment SGI359859[] = {{&S3316[49], 6}, {NULL}};
const Segment SGI359860[] = {{&S3316[33], 13}, {NULL}};
const Segment SGI359861[] = {{&S3317[69], 6}, {NULL}};
const Segment SGI359862[] = {{&S3317[8], 3}, {NULL}};
const Segment SGI359863[] = {{&S3317[13], 9}, {NULL}};
const Segment SGI359864[] = {{&S3317[33], 15}, {NULL}};
const Segment SGI359865[] = {{&S3317[49], 6}, {NULL}};
const Segment SGI359866[] = {{&S3317[33], 13}, {NULL}};
const Segment SGI359867[] = {{&S3318[69], 6}, {NULL}};
const Segment SGI359868[] = {{&S3318[8], 3}, {NULL}};
const Segment SGI359869[] = {{&S3318[13], 9}, {NULL}};
const Segment SGI359870[] = {{&S3318[33], 15}, {NULL}};
const Segment SGI359871[] = {{&S3318[49], 6}, {NULL}};
const Segment SGI359872[] = {{&S3318[33], 13}, {NULL}};
const Segment SGI359873[] = {{&S3319[69], 6}, {NULL}};
const Segment SGI359874[] = {{&S3319[8], 3}, {NULL}};
const Segment SGI359875[] = {{&S3319[13], 9}, {NULL}};
const Segment SGI359876[] = {{&S3319[33], 15}, {NULL}};
const Segment SGI359877[] = {{&S3319[49], 6}, {NULL}};
const Segment SGI359878[] = {{&S3319[33], 13}, {NULL}};
const Segment SGI359879[] = {{&S3320[69], 6}, {NULL}};
const Segment SGI359880[] = {{&S3320[8], 3}, {NULL}};
const Segment SGI359881[] = {{&S3320[13], 9}, {NULL}};
const Segment SGI359882[] = {{&S3320[33], 15}, {NULL}};
const Segment SGI359883[] = {{&S3320[49], 6}, {NULL}};
const Segment SGI359884[] = {{&S3320[33], 13}, {NULL}};
const Segment SGI359885[] = {{&S3321[69], 6}, {NULL}};
const Segment SGI359886[] = {{&S3321[8], 3}, {NULL}};
const Segment SGI359887[] = {{&S3321[13], 9}, {NULL}};
const Segment SGI359888[] = {{&S3321[33], 15}, {NULL}};
const Segment SGI359889[] = {{&S3321[49], 6}, {NULL}};
const Segment SGI359890[] = {{&S3321[33], 13}, {NULL}};
const Segment SGI359891[] = {{&S3322[69], 6}, {NULL}};
const Segment SGI359892[] = {{&S3322[8], 3}, {NULL}};
const Segment SGI359893[] = {{&S3322[13], 9}, {NULL}};
const Segment SGI359894[] = {{&S3322[33], 15}, {NULL}};
const Segment SGI359895[] = {{&S3322[49], 6}, {NULL}};
const Segment SGI359896[] = {{&S3322[33], 13}, {NULL}};
const Segment SGI359897[] = {{&S3323[69], 6}, {NULL}};
const Segment SGI359898[] = {{&S3323[8], 3}, {NULL}};
const Segment SGI359899[] = {{&S3323[13], 9}, {NULL}};
const Segment SGI359900[] = {{&S3323[33], 15}, {NULL}};
const Segment SGI359901[] = {{&S3323[49], 6}, {NULL}};
const Segment SGI359902[] = {{&S3323[33], 13}, {NULL}};
const Segment SGI359903[] = {{&S3324[69], 6}, {NULL}};
const Segment SGI359904[] = {{&S3324[8], 3}, {NULL}};
const Segment SGI359905[] = {{&S3324[13], 9}, {NULL}};
const Segment SGI359906[] = {{&S3324[33], 15}, {NULL}};
const Segment SGI359907[] = {{&S3324[49], 6}, {NULL}};
const Segment SGI359908[] = {{&S3324[33], 13}, {NULL}};
const Segment SGI359909[] = {{&S3325[69], 6}, {NULL}};
const Segment SGI359910[] = {{&S3325[8], 3}, {NULL}};
const Segment SGI359911[] = {{&S3325[13], 9}, {NULL}};
const Segment SGI359912[] = {{&S3325[33], 15}, {NULL}};
const Segment SGI359913[] = {{&S3325[49], 6}, {NULL}};
const Segment SGI359914[] = {{&S3325[33], 13}, {NULL}};
const Segment SGI359915[] = {{&S3326[69], 6}, {NULL}};
const Segment SGI359916[] = {{&S3326[8], 3}, {NULL}};
const Segment SGI359917[] = {{&S3326[13], 9}, {NULL}};
const Segment SGI359918[] = {{&S3326[33], 15}, {NULL}};
const Segment SGI359919[] = {{&S3326[49], 6}, {NULL}};
const Segment SGI359920[] = {{&S3326[33], 13}, {NULL}};
const Segment SGI359921[] = {{&S3327[69], 6}, {NULL}};
const Segment SGI359922[] = {{&S3327[8], 3}, {NULL}};
const Segment SGI359923[] = {{&S3327[13], 9}, {NULL}};
const Segment SGI359924[] = {{&S3327[33], 15}, {NULL}};
const Segment SGI359925[] = {{&S3327[49], 6}, {NULL}};
const Segment SGI359926[] = {{&S3327[33], 13}, {NULL}};
const Segment SGI359927[] = {{&S3328[69], 6}, {NULL}};
const Segment SGI359928[] = {{&S3328[8], 3}, {NULL}};
const Segment SGI359929[] = {{&S3328[13], 9}, {NULL}};
const Segment SGI359930[] = {{&S3328[33], 15}, {NULL}};
const Segment SGI359931[] = {{&S3328[49], 6}, {NULL}};
const Segment SGI359932[] = {{&S3328[33], 13}, {NULL}};
const Segment SGI359933[] = {{&S3329[69], 6}, {NULL}};
const Segment SGI359934[] = {{&S3329[8], 3}, {NULL}};
const Segment SGI359935[] = {{&S3329[13], 9}, {NULL}};
const Segment SGI359936[] = {{&S3329[33], 15}, {NULL}};
const Segment SGI359937[] = {{&S3329[49], 6}, {NULL}};
const Segment SGI359938[] = {{&S3329[33], 13}, {NULL}};
const Segment SGI359939[] = {{&S3330[69], 6}, {NULL}};
const Segment SGI359940[] = {{&S3330[8], 3}, {NULL}};
const Segment SGI359941[] = {{&S3330[13], 9}, {NULL}};
const Segment SGI359942[] = {{&S3330[33], 15}, {NULL}};
const Segment SGI359943[] = {{&S3330[49], 6}, {NULL}};
const Segment SGI359944[] = {{&S3330[33], 13}, {NULL}};
const Segment SGI359945[] = {{&S3331[69], 6}, {NULL}};
const Segment SGI359946[] = {{&S3331[8], 3}, {NULL}};
const Segment SGI359947[] = {{&S3331[13], 9}, {NULL}};
const Segment SGI359948[] = {{&S3331[33], 15}, {NULL}};
const Segment SGI359949[] = {{&S3331[49], 6}, {NULL}};
const Segment SGI359950[] = {{&S3331[33], 13}, {NULL}};
const Segment SGI359951[] = {{&S3332[118], 6}, {NULL}};
const Segment SGI359952[] = {{&S3332[8], 3}, {NULL}};
const Segment SGI359953[] = {{&S3332[13], 9}, {NULL}};
const Segment SGI359954[] = {{&S3332[33], 15}, {NULL}};
const Segment SGI359955[] = {{&S3332[49], 6}, {NULL}};
const Segment SGI359956[] = {{&S3332[33], 13}, {NULL}};
const Segment SGI359957[] = {{&S3333[69], 6}, {NULL}};
const Segment SGI359958[] = {{&S3333[8], 3}, {NULL}};
const Segment SGI359959[] = {{&S3333[13], 9}, {NULL}};
const Segment SGI359960[] = {{&S3333[33], 15}, {NULL}};
const Segment SGI359961[] = {{&S3333[49], 6}, {NULL}};
const Segment SGI359962[] = {{&S3333[33], 13}, {NULL}};
const Segment SGI359963[] = {{&S3334[69], 6}, {NULL}};
const Segment SGI359964[] = {{&S3334[8], 3}, {NULL}};
const Segment SGI359965[] = {{&S3334[13], 9}, {NULL}};
const Segment SGI359966[] = {{&S3334[33], 15}, {NULL}};
const Segment SGI359967[] = {{&S3334[49], 6}, {NULL}};
const Segment SGI359968[] = {{&S3334[33], 13}, {NULL}};
const Segment SGI359969[] = {{&S3335[121], 7}, {NULL}};
const Segment SGI359970[] = {{&S3335[8], 3}, {NULL}};
const Segment SGI359971[] = {{&S3335[13], 9}, {NULL}};
const Segment SGI359972[] = {{&S3335[33], 15}, {NULL}};
const Segment SGI359973[] = {{&S3335[49], 6}, {NULL}};
const Segment SGI359974[] = {{&S3335[33], 13}, {NULL}};
const Segment SGI359975[] = {{&S3336[69], 6}, {NULL}};
const Segment SGI359976[] = {{&S3336[8], 3}, {NULL}};
const Segment SGI359977[] = {{&S3336[13], 9}, {NULL}};
const Segment SGI359978[] = {{&S3336[33], 15}, {NULL}};
const Segment SGI359979[] = {{&S3336[49], 6}, {NULL}};
const Segment SGI359980[] = {{&S3336[33], 13}, {NULL}};
const Segment SGI359981[] = {{&S3337[69], 6}, {NULL}};
const Segment SGI359982[] = {{&S3337[8], 3}, {NULL}};
const Segment SGI359983[] = {{&S3337[13], 9}, {NULL}};
const Segment SGI359984[] = {{&S3337[33], 15}, {NULL}};
const Segment SGI359985[] = {{&S3337[49], 6}, {NULL}};
const Segment SGI359986[] = {{&S3337[33], 13}, {NULL}};
const Segment SGI359987[] = {{&S3338[69], 6}, {NULL}};
const Segment SGI359988[] = {{&S3338[8], 3}, {NULL}};
const Segment SGI359989[] = {{&S3338[13], 9}, {NULL}};
const Segment SGI359990[] = {{&S3338[33], 15}, {NULL}};
const Segment SGI359991[] = {{&S3338[49], 6}, {NULL}};
const Segment SGI359992[] = {{&S3338[33], 13}, {NULL}};
const Segment SGI359993[] = {{&S3339[69], 6}, {NULL}};
const Segment SGI359994[] = {{&S3339[8], 3}, {NULL}};
const Segment SGI359995[] = {{&S3339[13], 9}, {NULL}};
const Segment SGI359996[] = {{&S3339[33], 15}, {NULL}};
const Segment SGI359997[] = {{&S3339[49], 6}, {NULL}};
const Segment SGI359998[] = {{&S3339[33], 13}, {NULL}};
const Segment SGI359999[] = {{&S3340[69], 6}, {NULL}};
const Segment SGI360000[] = {{&S3340[8], 3}, {NULL}};
const Segment SGI360001[] = {{&S3340[13], 9}, {NULL}};
const Segment SGI360002[] = {{&S3340[33], 15}, {NULL}};
const Segment SGI360003[] = {{&S3340[49], 6}, {NULL}};
const Segment SGI360004[] = {{&S3340[33], 13}, {NULL}};
const Segment SGI360005[] = {{&S3341[69], 6}, {NULL}};
const Segment SGI360006[] = {{&S3341[8], 3}, {NULL}};
const Segment SGI360007[] = {{&S3341[13], 9}, {NULL}};
const Segment SGI360008[] = {{&S3341[33], 15}, {NULL}};
const Segment SGI360009[] = {{&S3341[49], 6}, {NULL}};
const Segment SGI360010[] = {{&S3341[33], 13}, {NULL}};
const Segment SGI360011[] = {{&S3342[117], 6}, {NULL}};
const Segment SGI360012[] = {{&S3342[8], 3}, {NULL}};
const Segment SGI360013[] = {{&S3342[13], 9}, {NULL}};
const Segment SGI360014[] = {{&S3342[33], 15}, {NULL}};
const Segment SGI360015[] = {{&S3342[49], 6}, {NULL}};
const Segment SGI360016[] = {{&S3342[33], 13}, {NULL}};
const Segment SGI360017[] = {{&S3343[69], 6}, {NULL}};
const Segment SGI360018[] = {{&S3343[8], 3}, {NULL}};
const Segment SGI360019[] = {{&S3343[13], 9}, {NULL}};
const Segment SGI360020[] = {{&S3343[33], 15}, {NULL}};
const Segment SGI360021[] = {{&S3343[49], 6}, {NULL}};
const Segment SGI360022[] = {{&S3343[33], 13}, {NULL}};
const Segment SGI360023[] = {{&S3344[66], 6}, {NULL}};
const Segment SGI360024[] = {{&S3344[8], 3}, {NULL}};
const Segment SGI360025[] = {{&S3344[13], 9}, {NULL}};
const Segment SGI360026[] = {{&S3344[33], 15}, {NULL}};
const Segment SGI360027[] = {{&S3344[49], 3}, {NULL}};
const Segment SGI360028[] = {{&S3344[33], 13}, {NULL}};
const Segment SGI360029[] = {{&S3345[69], 6}, {NULL}};
const Segment SGI360030[] = {{&S3345[8], 3}, {NULL}};
const Segment SGI360031[] = {{&S3345[13], 9}, {NULL}};
const Segment SGI360032[] = {{&S3345[33], 15}, {NULL}};
const Segment SGI360033[] = {{&S3345[49], 6}, {NULL}};
const Segment SGI360034[] = {{&S3345[33], 13}, {NULL}};
const Segment SGI360035[] = {{&S3346[69], 6}, {NULL}};
const Segment SGI360036[] = {{&S3346[8], 3}, {NULL}};
const Segment SGI360037[] = {{&S3346[13], 9}, {NULL}};
const Segment SGI360038[] = {{&S3346[33], 15}, {NULL}};
const Segment SGI360039[] = {{&S3346[49], 6}, {NULL}};
const Segment SGI360040[] = {{&S3346[33], 13}, {NULL}};
const Segment SGI360041[] = {{&S3347[69], 6}, {NULL}};
const Segment SGI360042[] = {{&S3347[8], 3}, {NULL}};
const Segment SGI360043[] = {{&S3347[13], 9}, {NULL}};
const Segment SGI360044[] = {{&S3347[33], 15}, {NULL}};
const Segment SGI360045[] = {{&S3347[49], 6}, {NULL}};
const Segment SGI360046[] = {{&S3347[33], 13}, {NULL}};
const Segment SGI360047[] = {{&S3348[69], 6}, {NULL}};
const Segment SGI360048[] = {{&S3348[8], 3}, {NULL}};
const Segment SGI360049[] = {{&S3348[13], 9}, {NULL}};
const Segment SGI360050[] = {{&S3348[33], 15}, {NULL}};
const Segment SGI360051[] = {{&S3348[49], 6}, {NULL}};
const Segment SGI360052[] = {{&S3348[33], 13}, {NULL}};
const Segment SGI360053[] = {{&S3349[69], 6}, {NULL}};
const Segment SGI360054[] = {{&S3349[8], 3}, {NULL}};
const Segment SGI360055[] = {{&S3349[13], 9}, {NULL}};
const Segment SGI360056[] = {{&S3349[33], 15}, {NULL}};
const Segment SGI360057[] = {{&S3349[49], 6}, {NULL}};
const Segment SGI360058[] = {{&S3349[33], 13}, {NULL}};
const Segment SGI360059[] = {{&S3350[69], 6}, {NULL}};
const Segment SGI360060[] = {{&S3350[8], 3}, {NULL}};
const Segment SGI360061[] = {{&S3350[13], 9}, {NULL}};
const Segment SGI360062[] = {{&S3350[33], 15}, {NULL}};
const Segment SGI360063[] = {{&S3350[49], 6}, {NULL}};
const Segment SGI360064[] = {{&S3350[33], 13}, {NULL}};
const Segment SGI360065[] = {{&S3351[69], 6}, {NULL}};
const Segment SGI360066[] = {{&S3351[8], 3}, {NULL}};
const Segment SGI360067[] = {{&S3351[13], 9}, {NULL}};
const Segment SGI360068[] = {{&S3351[33], 15}, {NULL}};
const Segment SGI360069[] = {{&S3351[49], 6}, {NULL}};
const Segment SGI360070[] = {{&S3351[33], 13}, {NULL}};
const Segment SGI360071[] = {{&S3352[69], 6}, {NULL}};
const Segment SGI360072[] = {{&S3352[8], 3}, {NULL}};
const Segment SGI360073[] = {{&S3352[13], 9}, {NULL}};
const Segment SGI360074[] = {{&S3352[33], 15}, {NULL}};
const Segment SGI360075[] = {{&S3352[49], 6}, {NULL}};
const Segment SGI360076[] = {{&S3352[33], 13}, {NULL}};
const Segment SGI360077[] = {{&S3353[69], 6}, {NULL}};
const Segment SGI360078[] = {{&S3353[8], 3}, {NULL}};
const Segment SGI360079[] = {{&S3353[13], 9}, {NULL}};
const Segment SGI360080[] = {{&S3353[33], 15}, {NULL}};
const Segment SGI360081[] = {{&S3353[49], 6}, {NULL}};
const Segment SGI360082[] = {{&S3353[33], 13}, {NULL}};
const Segment SGI360083[] = {{&S3354[69], 6}, {NULL}};
const Segment SGI360084[] = {{&S3354[8], 3}, {NULL}};
const Segment SGI360085[] = {{&S3354[13], 9}, {NULL}};
const Segment SGI360086[] = {{&S3354[33], 15}, {NULL}};
const Segment SGI360087[] = {{&S3354[49], 6}, {NULL}};
const Segment SGI360088[] = {{&S3354[33], 13}, {NULL}};
const Segment SGI360089[] = {{&S3355[69], 6}, {NULL}};
const Segment SGI360090[] = {{&S3355[8], 3}, {NULL}};
const Segment SGI360091[] = {{&S3355[13], 9}, {NULL}};
const Segment SGI360092[] = {{&S3355[33], 15}, {NULL}};
const Segment SGI360093[] = {{&S3355[49], 6}, {NULL}};
const Segment SGI360094[] = {{&S3355[33], 13}, {NULL}};
const Segment SGI360095[] = {{&S3356[69], 6}, {NULL}};
const Segment SGI360096[] = {{&S3356[8], 3}, {NULL}};
const Segment SGI360097[] = {{&S3356[13], 9}, {NULL}};
const Segment SGI360098[] = {{&S3356[33], 15}, {NULL}};
const Segment SGI360099[] = {{&S3356[49], 6}, {NULL}};
const Segment SGI360100[] = {{&S3356[33], 13}, {NULL}};
const Segment SGI360101[] = {{&S3357[69], 6}, {NULL}};
const Segment SGI360102[] = {{&S3357[8], 3}, {NULL}};
const Segment SGI360103[] = {{&S3357[13], 9}, {NULL}};
const Segment SGI360104[] = {{&S3357[33], 15}, {NULL}};
const Segment SGI360105[] = {{&S3357[49], 6}, {NULL}};
const Segment SGI360106[] = {{&S3357[33], 13}, {NULL}};
const Segment SGI360107[] = {{&S3358[69], 6}, {NULL}};
const Segment SGI360108[] = {{&S3358[8], 3}, {NULL}};
const Segment SGI360109[] = {{&S3358[13], 9}, {NULL}};
const Segment SGI360110[] = {{&S3358[33], 15}, {NULL}};
const Segment SGI360111[] = {{&S3358[49], 6}, {NULL}};
const Segment SGI360112[] = {{&S3358[33], 13}, {NULL}};
const Segment SGI360113[] = {{&S3359[69], 6}, {NULL}};
const Segment SGI360114[] = {{&S3359[8], 3}, {NULL}};
const Segment SGI360115[] = {{&S3359[13], 9}, {NULL}};
const Segment SGI360116[] = {{&S3359[33], 15}, {NULL}};
const Segment SGI360117[] = {{&S3359[49], 6}, {NULL}};
const Segment SGI360118[] = {{&S3359[33], 13}, {NULL}};
const Segment SGI360119[] = {{&S3360[69], 6}, {NULL}};
const Segment SGI360120[] = {{&S3360[8], 3}, {NULL}};
const Segment SGI360121[] = {{&S3360[13], 9}, {NULL}};
const Segment SGI360122[] = {{&S3360[33], 15}, {NULL}};
const Segment SGI360123[] = {{&S3360[49], 6}, {NULL}};
const Segment SGI360124[] = {{&S3360[33], 13}, {NULL}};
const Segment SGI360125[] = {{&S3361[69], 6}, {NULL}};
const Segment SGI360126[] = {{&S3361[8], 3}, {NULL}};
const Segment SGI360127[] = {{&S3361[13], 9}, {NULL}};
const Segment SGI360128[] = {{&S3361[33], 15}, {NULL}};
const Segment SGI360129[] = {{&S3361[49], 6}, {NULL}};
const Segment SGI360130[] = {{&S3361[33], 13}, {NULL}};
const Segment SGI360131[] = {{&S3362[69], 6}, {NULL}};
const Segment SGI360132[] = {{&S3362[8], 3}, {NULL}};
const Segment SGI360133[] = {{&S3362[13], 9}, {NULL}};
const Segment SGI360134[] = {{&S3362[33], 15}, {NULL}};
const Segment SGI360135[] = {{&S3362[49], 6}, {NULL}};
const Segment SGI360136[] = {{&S3362[33], 13}, {NULL}};
const Segment SGI360137[] = {{&S3363[69], 6}, {NULL}};
const Segment SGI360138[] = {{&S3363[8], 3}, {NULL}};
const Segment SGI360139[] = {{&S3363[13], 9}, {NULL}};
const Segment SGI360140[] = {{&S3363[33], 15}, {NULL}};
const Segment SGI360141[] = {{&S3363[49], 6}, {NULL}};
const Segment SGI360142[] = {{&S3363[33], 13}, {NULL}};
const Segment SGI360143[] = {{&S3364[69], 6}, {NULL}};
const Segment SGI360144[] = {{&S3364[8], 3}, {NULL}};
const Segment SGI360145[] = {{&S3364[13], 9}, {NULL}};
const Segment SGI360146[] = {{&S3364[33], 15}, {NULL}};
const Segment SGI360147[] = {{&S3364[49], 6}, {NULL}};
const Segment SGI360148[] = {{&S3364[33], 13}, {NULL}};
const Segment SGI360149[] = {{&S3365[69], 6}, {NULL}};
const Segment SGI360150[] = {{&S3365[8], 3}, {NULL}};
const Segment SGI360151[] = {{&S3365[13], 9}, {NULL}};
const Segment SGI360152[] = {{&S3365[33], 15}, {NULL}};
const Segment SGI360153[] = {{&S3365[49], 6}, {NULL}};
const Segment SGI360154[] = {{&S3365[33], 13}, {NULL}};
const Segment SGI360155[] = {{&S3366[69], 6}, {NULL}};
const Segment SGI360156[] = {{&S3366[8], 3}, {NULL}};
const Segment SGI360157[] = {{&S3366[13], 9}, {NULL}};
const Segment SGI360158[] = {{&S3366[33], 15}, {NULL}};
const Segment SGI360159[] = {{&S3366[49], 6}, {NULL}};
const Segment SGI360160[] = {{&S3366[33], 13}, {NULL}};
const Segment SGI360161[] = {{&S3367[69], 6}, {NULL}};
const Segment SGI360162[] = {{&S3367[8], 3}, {NULL}};
const Segment SGI360163[] = {{&S3367[13], 9}, {NULL}};
const Segment SGI360164[] = {{&S3367[33], 15}, {NULL}};
const Segment SGI360165[] = {{&S3367[49], 6}, {NULL}};
const Segment SGI360166[] = {{&S3367[33], 13}, {NULL}};
const Segment SGI360167[] = {{&S3368[69], 6}, {NULL}};
const Segment SGI360168[] = {{&S3368[8], 3}, {NULL}};
const Segment SGI360169[] = {{&S3368[13], 9}, {NULL}};
const Segment SGI360170[] = {{&S3368[33], 15}, {NULL}};
const Segment SGI360171[] = {{&S3368[49], 6}, {NULL}};
const Segment SGI360172[] = {{&S3368[33], 13}, {NULL}};
const Segment SGI360173[] = {{&S3369[69], 6}, {NULL}};
const Segment SGI360174[] = {{&S3369[8], 3}, {NULL}};
const Segment SGI360175[] = {{&S3369[13], 9}, {NULL}};
const Segment SGI360176[] = {{&S3369[33], 15}, {NULL}};
const Segment SGI360177[] = {{&S3369[49], 6}, {NULL}};
const Segment SGI360178[] = {{&S3369[33], 13}, {NULL}};
const Segment SGI360179[] = {{&S3370[69], 6}, {NULL}};
const Segment SGI360180[] = {{&S3370[8], 3}, {NULL}};
const Segment SGI360181[] = {{&S3370[13], 9}, {NULL}};
const Segment SGI360182[] = {{&S3370[33], 15}, {NULL}};
const Segment SGI360183[] = {{&S3370[49], 6}, {NULL}};
const Segment SGI360184[] = {{&S3370[33], 13}, {NULL}};
const Segment SGI360185[] = {{&S3371[69], 6}, {NULL}};
const Segment SGI360186[] = {{&S3371[8], 3}, {NULL}};
const Segment SGI360187[] = {{&S3371[13], 9}, {NULL}};
const Segment SGI360188[] = {{&S3371[33], 15}, {NULL}};
const Segment SGI360189[] = {{&S3371[49], 6}, {NULL}};
const Segment SGI360190[] = {{&S3371[33], 13}, {NULL}};
const Segment SGI360191[] = {{&S3372[69], 6}, {NULL}};
const Segment SGI360192[] = {{&S3372[8], 3}, {NULL}};
const Segment SGI360193[] = {{&S3372[13], 9}, {NULL}};
const Segment SGI360194[] = {{&S3372[33], 15}, {NULL}};
const Segment SGI360195[] = {{&S3372[49], 6}, {NULL}};
const Segment SGI360196[] = {{&S3372[33], 13}, {NULL}};
const Segment SGI360197[] = {{&S3373[69], 6}, {NULL}};
const Segment SGI360198[] = {{&S3373[8], 3}, {NULL}};
const Segment SGI360199[] = {{&S3373[13], 9}, {NULL}};
const Segment SGI360200[] = {{&S3373[33], 15}, {NULL}};
const Segment SGI360201[] = {{&S3373[49], 6}, {NULL}};
const Segment SGI360202[] = {{&S3373[33], 13}, {NULL}};
const Segment SGI360203[] = {{&S3374[69], 6}, {NULL}};
const Segment SGI360204[] = {{&S3374[8], 3}, {NULL}};
const Segment SGI360205[] = {{&S3374[13], 9}, {NULL}};
const Segment SGI360206[] = {{&S3374[33], 15}, {NULL}};
const Segment SGI360207[] = {{&S3374[49], 6}, {NULL}};
const Segment SGI360208[] = {{&S3374[33], 13}, {NULL}};
const Segment SGI360209[] = {{&S3375[120], 6}, {NULL}};
const Segment SGI360210[] = {{&S3375[8], 3}, {NULL}};
const Segment SGI360211[] = {{&S3375[13], 9}, {NULL}};
const Segment SGI360212[] = {{&S3375[33], 15}, {NULL}};
const Segment SGI360213[] = {{&S3375[49], 6}, {NULL}};
const Segment SGI360214[] = {{&S3375[33], 13}, {NULL}};
const Segment SGI360215[] = {{&S3376[69], 6}, {NULL}};
const Segment SGI360216[] = {{&S3376[8], 3}, {NULL}};
const Segment SGI360217[] = {{&S3376[13], 9}, {NULL}};
const Segment SGI360218[] = {{&S3376[33], 15}, {NULL}};
const Segment SGI360219[] = {{&S3376[49], 6}, {NULL}};
const Segment SGI360220[] = {{&S3376[33], 13}, {NULL}};
const Segment SGI360221[] = {{&S3377[69], 6}, {NULL}};
const Segment SGI360222[] = {{&S3377[8], 3}, {NULL}};
const Segment SGI360223[] = {{&S3377[13], 9}, {NULL}};
const Segment SGI360224[] = {{&S3377[33], 15}, {NULL}};
const Segment SGI360225[] = {{&S3377[49], 6}, {NULL}};
const Segment SGI360226[] = {{&S3377[33], 13}, {NULL}};
const Segment SGI360227[] = {{&S3378[69], 6}, {NULL}};
const Segment SGI360228[] = {{&S3378[8], 3}, {NULL}};
const Segment SGI360229[] = {{&S3378[13], 9}, {NULL}};
const Segment SGI360230[] = {{&S3378[33], 15}, {NULL}};
const Segment SGI360231[] = {{&S3378[49], 6}, {NULL}};
const Segment SGI360232[] = {{&S3378[33], 13}, {NULL}};
const Segment SGI360233[] = {{&S3379[69], 6}, {NULL}};
const Segment SGI360234[] = {{&S3379[8], 3}, {NULL}};
const Segment SGI360235[] = {{&S3379[13], 9}, {NULL}};
const Segment SGI360236[] = {{&S3379[33], 15}, {NULL}};
const Segment SGI360237[] = {{&S3379[49], 6}, {NULL}};
const Segment SGI360238[] = {{&S3379[33], 13}, {NULL}};
const Segment SGI360239[] = {{&S3380[69], 6}, {NULL}};
const Segment SGI360240[] = {{&S3380[8], 3}, {NULL}};
const Segment SGI360241[] = {{&S3380[13], 9}, {NULL}};
const Segment SGI360242[] = {{&S3380[33], 15}, {NULL}};
const Segment SGI360243[] = {{&S3380[49], 6}, {NULL}};
const Segment SGI360244[] = {{&S3380[33], 13}, {NULL}};
const Segment SGI360245[] = {{&S3381[69], 6}, {NULL}};
const Segment SGI360246[] = {{&S3381[8], 3}, {NULL}};
const Segment SGI360247[] = {{&S3381[13], 9}, {NULL}};
const Segment SGI360248[] = {{&S3381[33], 15}, {NULL}};
const Segment SGI360249[] = {{&S3381[49], 6}, {NULL}};
const Segment SGI360250[] = {{&S3381[33], 13}, {NULL}};
const Segment SGI360251[] = {{&S3382[69], 6}, {NULL}};
const Segment SGI360252[] = {{&S3382[8], 3}, {NULL}};
const Segment SGI360253[] = {{&S3382[13], 9}, {NULL}};
const Segment SGI360254[] = {{&S3382[33], 15}, {NULL}};
const Segment SGI360255[] = {{&S3382[49], 6}, {NULL}};
const Segment SGI360256[] = {{&S3382[33], 13}, {NULL}};
const Segment SGI360257[] = {{&S3383[69], 6}, {NULL}};
const Segment SGI360258[] = {{&S3383[8], 3}, {NULL}};
const Segment SGI360259[] = {{&S3383[13], 9}, {NULL}};
const Segment SGI360260[] = {{&S3383[33], 15}, {NULL}};
const Segment SGI360261[] = {{&S3383[49], 6}, {NULL}};
const Segment SGI360262[] = {{&S3383[33], 13}, {NULL}};
const Segment SGI360263[] = {{&S3384[69], 6}, {NULL}};
const Segment SGI360264[] = {{&S3384[8], 3}, {NULL}};
const Segment SGI360265[] = {{&S3384[13], 9}, {NULL}};
const Segment SGI360266[] = {{&S3384[33], 15}, {NULL}};
const Segment SGI360267[] = {{&S3384[49], 6}, {NULL}};
const Segment SGI360268[] = {{&S3384[33], 13}, {NULL}};
const Segment SGI360269[] = {{&S3385[69], 6}, {NULL}};
const Segment SGI360270[] = {{&S3385[8], 3}, {NULL}};
const Segment SGI360271[] = {{&S3385[13], 9}, {NULL}};
const Segment SGI360272[] = {{&S3385[33], 15}, {NULL}};
const Segment SGI360273[] = {{&S3385[49], 6}, {NULL}};
const Segment SGI360274[] = {{&S3385[33], 13}, {NULL}};
const Segment SGI360275[] = {{&S3386[69], 6}, {NULL}};
const Segment SGI360276[] = {{&S3386[8], 3}, {NULL}};
const Segment SGI360277[] = {{&S3386[13], 9}, {NULL}};
const Segment SGI360278[] = {{&S3386[33], 15}, {NULL}};
const Segment SGI360279[] = {{&S3386[49], 6}, {NULL}};
const Segment SGI360280[] = {{&S3386[33], 13}, {NULL}};
const Segment SGI360281[] = {{&S3387[69], 6}, {NULL}};
const Segment SGI360282[] = {{&S3387[8], 3}, {NULL}};
const Segment SGI360283[] = {{&S3387[13], 9}, {NULL}};
const Segment SGI360284[] = {{&S3387[33], 15}, {NULL}};
const Segment SGI360285[] = {{&S3387[49], 6}, {NULL}};
const Segment SGI360286[] = {{&S3387[33], 13}, {NULL}};
const Segment SGI360287[] = {{&S3388[69], 6}, {NULL}};
const Segment SGI360288[] = {{&S3388[8], 3}, {NULL}};
const Segment SGI360289[] = {{&S3388[13], 9}, {NULL}};
const Segment SGI360290[] = {{&S3388[33], 15}, {NULL}};
const Segment SGI360291[] = {{&S3388[49], 6}, {NULL}};
const Segment SGI360292[] = {{&S3388[33], 13}, {NULL}};
const Segment SGI360293[] = {{&S3389[69], 6}, {NULL}};
const Segment SGI360294[] = {{&S3389[8], 3}, {NULL}};
const Segment SGI360295[] = {{&S3389[13], 9}, {NULL}};
const Segment SGI360296[] = {{&S3389[33], 15}, {NULL}};
const Segment SGI360297[] = {{&S3389[49], 6}, {NULL}};
const Segment SGI360298[] = {{&S3389[33], 13}, {NULL}};
const Segment SGI360299[] = {{&S3390[69], 6}, {NULL}};
const Segment SGI360300[] = {{&S3390[8], 3}, {NULL}};
const Segment SGI360301[] = {{&S3390[13], 9}, {NULL}};
const Segment SGI360302[] = {{&S3390[33], 15}, {NULL}};
const Segment SGI360303[] = {{&S3390[49], 6}, {NULL}};
const Segment SGI360304[] = {{&S3390[33], 13}, {NULL}};
const Segment SGI360305[] = {{&S3391[69], 6}, {NULL}};
const Segment SGI360306[] = {{&S3391[8], 3}, {NULL}};
const Segment SGI360307[] = {{&S3391[13], 9}, {NULL}};
const Segment SGI360308[] = {{&S3391[33], 15}, {NULL}};
const Segment SGI360309[] = {{&S3391[49], 6}, {NULL}};
const Segment SGI360310[] = {{&S3391[33], 13}, {NULL}};
const Segment SGI360311[] = {{&S3392[69], 6}, {NULL}};
const Segment SGI360312[] = {{&S3392[8], 3}, {NULL}};
const Segment SGI360313[] = {{&S3392[13], 9}, {NULL}};
const Segment SGI360314[] = {{&S3392[33], 15}, {NULL}};
const Segment SGI360315[] = {{&S3392[49], 6}, {NULL}};
const Segment SGI360316[] = {{&S3392[33], 13}, {NULL}};
const Segment SGI360317[] = {{&S3393[69], 6}, {NULL}};
const Segment SGI360318[] = {{&S3393[8], 3}, {NULL}};
const Segment SGI360319[] = {{&S3393[13], 9}, {NULL}};
const Segment SGI360320[] = {{&S3393[33], 15}, {NULL}};
const Segment SGI360321[] = {{&S3393[49], 6}, {NULL}};
const Segment SGI360322[] = {{&S3393[33], 13}, {NULL}};
const Segment SGI360323[] = {{&S3394[69], 6}, {NULL}};
const Segment SGI360324[] = {{&S3394[8], 3}, {NULL}};
const Segment SGI360325[] = {{&S3394[13], 9}, {NULL}};
const Segment SGI360326[] = {{&S3394[33], 15}, {NULL}};
const Segment SGI360327[] = {{&S3394[49], 6}, {NULL}};
const Segment SGI360328[] = {{&S3394[33], 13}, {NULL}};
const Segment SGI360329[] = {{&S3395[69], 6}, {NULL}};
const Segment SGI360330[] = {{&S3395[8], 3}, {NULL}};
const Segment SGI360331[] = {{&S3395[13], 9}, {NULL}};
const Segment SGI360332[] = {{&S3395[33], 15}, {NULL}};
const Segment SGI360333[] = {{&S3395[49], 6}, {NULL}};
const Segment SGI360334[] = {{&S3395[33], 13}, {NULL}};
const Segment SGI360335[] = {{&S3396[69], 6}, {NULL}};
const Segment SGI360336[] = {{&S3396[8], 3}, {NULL}};
const Segment SGI360337[] = {{&S3396[13], 9}, {NULL}};
const Segment SGI360338[] = {{&S3396[33], 15}, {NULL}};
const Segment SGI360339[] = {{&S3396[49], 6}, {NULL}};
const Segment SGI360340[] = {{&S3396[33], 13}, {NULL}};
const Segment SGI360341[] = {{&S3397[69], 6}, {NULL}};
const Segment SGI360342[] = {{&S3397[8], 3}, {NULL}};
const Segment SGI360343[] = {{&S3397[13], 9}, {NULL}};
const Segment SGI360344[] = {{&S3397[33], 15}, {NULL}};
const Segment SGI360345[] = {{&S3397[49], 6}, {NULL}};
const Segment SGI360346[] = {{&S3397[33], 13}, {NULL}};
const Segment SGI360347[] = {{&S3398[69], 6}, {NULL}};
const Segment SGI360348[] = {{&S3398[8], 3}, {NULL}};
const Segment SGI360349[] = {{&S3398[13], 9}, {NULL}};
const Segment SGI360350[] = {{&S3398[33], 15}, {NULL}};
const Segment SGI360351[] = {{&S3398[49], 6}, {NULL}};
const Segment SGI360352[] = {{&S3398[33], 13}, {NULL}};
const Segment SGI360353[] = {{&S3399[69], 6}, {NULL}};
const Segment SGI360354[] = {{&S3399[8], 3}, {NULL}};
const Segment SGI360355[] = {{&S3399[13], 9}, {NULL}};
const Segment SGI360356[] = {{&S3399[33], 15}, {NULL}};
const Segment SGI360357[] = {{&S3399[49], 6}, {NULL}};
const Segment SGI360358[] = {{&S3399[33], 13}, {NULL}};
const Segment SGI360359[] = {{&S3400[69], 6}, {NULL}};
const Segment SGI360360[] = {{&S3400[8], 3}, {NULL}};
const Segment SGI360361[] = {{&S3400[13], 9}, {NULL}};
const Segment SGI360362[] = {{&S3400[33], 15}, {NULL}};
const Segment SGI360363[] = {{&S3400[49], 6}, {NULL}};
const Segment SGI360364[] = {{&S3400[33], 13}, {NULL}};
const Segment SGI360365[] = {{&S3401[122], 7}, {NULL}};
const Segment SGI360366[] = {{&S3401[8], 3}, {NULL}};
const Segment SGI360367[] = {{&S3401[13], 9}, {NULL}};
const Segment SGI360368[] = {{&S3401[33], 15}, {NULL}};
const Segment SGI360369[] = {{&S3401[49], 6}, {NULL}};
const Segment SGI360370[] = {{&S3401[33], 13}, {NULL}};
const Segment SGI360371[] = {{&S3402[69], 6}, {NULL}};
const Segment SGI360372[] = {{&S3402[8], 3}, {NULL}};
const Segment SGI360373[] = {{&S3402[13], 9}, {NULL}};
const Segment SGI360374[] = {{&S3402[33], 15}, {NULL}};
const Segment SGI360375[] = {{&S3402[49], 6}, {NULL}};
const Segment SGI360376[] = {{&S3402[33], 13}, {NULL}};
const Segment SGI360377[] = {{&S3403[69], 6}, {NULL}};
const Segment SGI360378[] = {{&S3403[8], 3}, {NULL}};
const Segment SGI360379[] = {{&S3403[13], 9}, {NULL}};
const Segment SGI360380[] = {{&S3403[33], 15}, {NULL}};
const Segment SGI360381[] = {{&S3403[49], 6}, {NULL}};
const Segment SGI360382[] = {{&S3403[33], 13}, {NULL}};
const Segment SGI360383[] = {{&S3404[69], 6}, {NULL}};
const Segment SGI360384[] = {{&S3404[8], 3}, {NULL}};
const Segment SGI360385[] = {{&S3404[13], 9}, {NULL}};
const Segment SGI360386[] = {{&S3404[33], 15}, {NULL}};
const Segment SGI360387[] = {{&S3404[49], 6}, {NULL}};
const Segment SGI360388[] = {{&S3404[33], 13}, {NULL}};
const Segment SGI360389[] = {{&S3405[66], 6}, {NULL}};
const Segment SGI360390[] = {{&S3405[8], 3}, {NULL}};
const Segment SGI360391[] = {{&S3405[13], 9}, {NULL}};
const Segment SGI360392[] = {{&S3405[33], 15}, {NULL}};
const Segment SGI360393[] = {{&S3405[49], 3}, {NULL}};
const Segment SGI360394[] = {{&S3405[33], 13}, {NULL}};
const Segment SGI360395[] = {{&S3406[69], 6}, {NULL}};
const Segment SGI360396[] = {{&S3406[8], 3}, {NULL}};
const Segment SGI360397[] = {{&S3406[13], 9}, {NULL}};
const Segment SGI360398[] = {{&S3406[33], 15}, {NULL}};
const Segment SGI360399[] = {{&S3406[49], 6}, {NULL}};
const Segment SGI360400[] = {{&S3406[33], 13}, {NULL}};
const Segment SGI360401[] = {{&S3407[69], 6}, {NULL}};
const Segment SGI360402[] = {{&S3407[8], 3}, {NULL}};
const Segment SGI360403[] = {{&S3407[13], 9}, {NULL}};
const Segment SGI360404[] = {{&S3407[33], 15}, {NULL}};
const Segment SGI360405[] = {{&S3407[49], 6}, {NULL}};
const Segment SGI360406[] = {{&S3407[33], 13}, {NULL}};
const Segment SGI360407[] = {{&S3408[69], 6}, {NULL}};
const Segment SGI360408[] = {{&S3408[8], 3}, {NULL}};
const Segment SGI360409[] = {{&S3408[13], 9}, {NULL}};
const Segment SGI360410[] = {{&S3408[33], 15}, {NULL}};
const Segment SGI360411[] = {{&S3408[49], 6}, {NULL}};
const Segment SGI360412[] = {{&S3408[33], 13}, {NULL}};
const Segment SGI360413[] = {{&S3409[69], 6}, {NULL}};
const Segment SGI360414[] = {{&S3409[8], 3}, {NULL}};
const Segment SGI360415[] = {{&S3409[13], 9}, {NULL}};
const Segment SGI360416[] = {{&S3409[33], 15}, {NULL}};
const Segment SGI360417[] = {{&S3409[49], 6}, {NULL}};
const Segment SGI360418[] = {{&S3409[33], 13}, {NULL}};
const Segment SGI360419[] = {{&S3410[69], 6}, {NULL}};
const Segment SGI360420[] = {{&S3410[8], 3}, {NULL}};
const Segment SGI360421[] = {{&S3410[13], 9}, {NULL}};
const Segment SGI360422[] = {{&S3410[33], 15}, {NULL}};
const Segment SGI360423[] = {{&S3410[49], 6}, {NULL}};
const Segment SGI360424[] = {{&S3410[33], 13}, {NULL}};
const Segment SGI360425[] = {{&S3411[69], 6}, {NULL}};
const Segment SGI360426[] = {{&S3411[8], 3}, {NULL}};
const Segment SGI360427[] = {{&S3411[13], 9}, {NULL}};
const Segment SGI360428[] = {{&S3411[33], 15}, {NULL}};
const Segment SGI360429[] = {{&S3411[49], 6}, {NULL}};
const Segment SGI360430[] = {{&S3411[33], 13}, {NULL}};
const Segment SGI360431[] = {{&S3412[69], 6}, {NULL}};
const Segment SGI360432[] = {{&S3412[8], 3}, {NULL}};
const Segment SGI360433[] = {{&S3412[13], 9}, {NULL}};
const Segment SGI360434[] = {{&S3412[33], 15}, {NULL}};
const Segment SGI360435[] = {{&S3412[49], 6}, {NULL}};
const Segment SGI360436[] = {{&S3412[33], 13}, {NULL}};
const Segment SGI360437[] = {{&S3413[69], 6}, {NULL}};
const Segment SGI360438[] = {{&S3413[8], 3}, {NULL}};
const Segment SGI360439[] = {{&S3413[13], 9}, {NULL}};
const Segment SGI360440[] = {{&S3413[33], 15}, {NULL}};
const Segment SGI360441[] = {{&S3413[49], 6}, {NULL}};
const Segment SGI360442[] = {{&S3413[33], 13}, {NULL}};
const Segment SGI360443[] = {{&S3414[69], 6}, {NULL}};
const Segment SGI360444[] = {{&S3414[8], 3}, {NULL}};
const Segment SGI360445[] = {{&S3414[13], 9}, {NULL}};
const Segment SGI360446[] = {{&S3414[33], 15}, {NULL}};
const Segment SGI360447[] = {{&S3414[49], 6}, {NULL}};
const Segment SGI360448[] = {{&S3414[33], 13}, {NULL}};
const Segment SGI360449[] = {{&S3415[69], 6}, {NULL}};
const Segment SGI360450[] = {{&S3415[8], 3}, {NULL}};
const Segment SGI360451[] = {{&S3415[13], 9}, {NULL}};
const Segment SGI360452[] = {{&S3415[33], 15}, {NULL}};
const Segment SGI360453[] = {{&S3415[49], 6}, {NULL}};
const Segment SGI360454[] = {{&S3415[33], 13}, {NULL}};
const Segment SGI360455[] = {{&S3416[69], 6}, {NULL}};
const Segment SGI360456[] = {{&S3416[8], 3}, {NULL}};
const Segment SGI360457[] = {{&S3416[13], 9}, {NULL}};
const Segment SGI360458[] = {{&S3416[33], 15}, {NULL}};
const Segment SGI360459[] = {{&S3416[49], 6}, {NULL}};
const Segment SGI360460[] = {{&S3416[33], 13}, {NULL}};
const Segment SGI360461[] = {{&S3417[69], 6}, {NULL}};
const Segment SGI360462[] = {{&S3417[8], 3}, {NULL}};
const Segment SGI360463[] = {{&S3417[13], 9}, {NULL}};
const Segment SGI360464[] = {{&S3417[33], 15}, {NULL}};
const Segment SGI360465[] = {{&S3417[49], 6}, {NULL}};
const Segment SGI360466[] = {{&S3417[33], 13}, {NULL}};
const Segment SGI360467[] = {{&S3418[69], 6}, {NULL}};
const Segment SGI360468[] = {{&S3418[8], 3}, {NULL}};
const Segment SGI360469[] = {{&S3418[13], 9}, {NULL}};
const Segment SGI360470[] = {{&S3418[33], 15}, {NULL}};
const Segment SGI360471[] = {{&S3418[49], 6}, {NULL}};
const Segment SGI360472[] = {{&S3418[33], 13}, {NULL}};
const Segment SGI360473[] = {{&S3419[69], 6}, {NULL}};
const Segment SGI360474[] = {{&S3419[8], 3}, {NULL}};
const Segment SGI360475[] = {{&S3419[13], 9}, {NULL}};
const Segment SGI360476[] = {{&S3419[33], 15}, {NULL}};
const Segment SGI360477[] = {{&S3419[49], 6}, {NULL}};
const Segment SGI360478[] = {{&S3419[33], 13}, {NULL}};
const Segment SGI360479[] = {{&S3420[69], 6}, {NULL}};
const Segment SGI360480[] = {{&S3420[8], 3}, {NULL}};
const Segment SGI360481[] = {{&S3420[13], 9}, {NULL}};
const Segment SGI360482[] = {{&S3420[33], 15}, {NULL}};
const Segment SGI360483[] = {{&S3420[49], 6}, {NULL}};
const Segment SGI360484[] = {{&S3420[33], 13}, {NULL}};
const Segment SGI360485[] = {{&S3421[69], 6}, {NULL}};
const Segment SGI360486[] = {{&S3421[8], 3}, {NULL}};
const Segment SGI360487[] = {{&S3421[13], 9}, {NULL}};
const Segment SGI360488[] = {{&S3421[33], 15}, {NULL}};
const Segment SGI360489[] = {{&S3421[49], 6}, {NULL}};
const Segment SGI360490[] = {{&S3421[33], 13}, {NULL}};
const Segment SGI360491[] = {{&S3422[69], 6}, {NULL}};
const Segment SGI360492[] = {{&S3422[8], 3}, {NULL}};
const Segment SGI360493[] = {{&S3422[13], 9}, {NULL}};
const Segment SGI360494[] = {{&S3422[33], 15}, {NULL}};
const Segment SGI360495[] = {{&S3422[49], 6}, {NULL}};
const Segment SGI360496[] = {{&S3422[33], 13}, {NULL}};
const Segment SGI360497[] = {{&S3423[69], 6}, {NULL}};
const Segment SGI360498[] = {{&S3423[8], 3}, {NULL}};
const Segment SGI360499[] = {{&S3423[13], 9}, {NULL}};
const Segment SGI360500[] = {{&S3423[33], 15}, {NULL}};
const Segment SGI360501[] = {{&S3423[49], 6}, {NULL}};
const Segment SGI360502[] = {{&S3423[33], 13}, {NULL}};
const Segment SGI360503[] = {{&S3424[69], 6}, {NULL}};
const Segment SGI360504[] = {{&S3424[8], 3}, {NULL}};
const Segment SGI360505[] = {{&S3424[13], 9}, {NULL}};
const Segment SGI360506[] = {{&S3424[33], 15}, {NULL}};
const Segment SGI360507[] = {{&S3424[49], 6}, {NULL}};
const Segment SGI360508[] = {{&S3424[33], 13}, {NULL}};
const Segment SGI360509[] = {{&S3425[69], 6}, {NULL}};
const Segment SGI360510[] = {{&S3425[8], 3}, {NULL}};
const Segment SGI360511[] = {{&S3425[13], 9}, {NULL}};
const Segment SGI360512[] = {{&S3425[33], 15}, {NULL}};
const Segment SGI360513[] = {{&S3425[49], 6}, {NULL}};
const Segment SGI360514[] = {{&S3425[33], 13}, {NULL}};
const Segment SGI360515[] = {{&S3426[69], 6}, {NULL}};
const Segment SGI360516[] = {{&S3426[8], 3}, {NULL}};
const Segment SGI360517[] = {{&S3426[13], 9}, {NULL}};
const Segment SGI360518[] = {{&S3426[33], 15}, {NULL}};
const Segment SGI360519[] = {{&S3426[49], 6}, {NULL}};
const Segment SGI360520[] = {{&S3426[33], 13}, {NULL}};
const Segment SGI360521[] = {{&S3427[69], 6}, {NULL}};
const Segment SGI360522[] = {{&S3427[8], 3}, {NULL}};
const Segment SGI360523[] = {{&S3427[13], 9}, {NULL}};
const Segment SGI360524[] = {{&S3427[33], 15}, {NULL}};
const Segment SGI360525[] = {{&S3427[49], 6}, {NULL}};
const Segment SGI360526[] = {{&S3427[33], 13}, {NULL}};
const Segment SGI360527[] = {{&S3428[69], 6}, {NULL}};
const Segment SGI360528[] = {{&S3428[8], 3}, {NULL}};
const Segment SGI360529[] = {{&S3428[13], 9}, {NULL}};
const Segment SGI360530[] = {{&S3428[33], 15}, {NULL}};
const Segment SGI360531[] = {{&S3428[49], 6}, {NULL}};
const Segment SGI360532[] = {{&S3428[33], 13}, {NULL}};
const Segment SGI360533[] = {{&S3429[69], 6}, {NULL}};
const Segment SGI360534[] = {{&S3429[8], 3}, {NULL}};
const Segment SGI360535[] = {{&S3429[13], 9}, {NULL}};
const Segment SGI360536[] = {{&S3429[33], 15}, {NULL}};
const Segment SGI360537[] = {{&S3429[49], 6}, {NULL}};
const Segment SGI360538[] = {{&S3429[33], 13}, {NULL}};
const Segment SGI360539[] = {{&S3430[69], 6}, {NULL}};
const Segment SGI360540[] = {{&S3430[8], 3}, {NULL}};
const Segment SGI360541[] = {{&S3430[13], 9}, {NULL}};
const Segment SGI360542[] = {{&S3430[33], 15}, {NULL}};
const Segment SGI360543[] = {{&S3430[49], 6}, {NULL}};
const Segment SGI360544[] = {{&S3430[33], 13}, {NULL}};
const Segment SGI360545[] = {{&S3431[121], 7}, {NULL}};
const Segment SGI360546[] = {{&S3431[8], 3}, {NULL}};
const Segment SGI360547[] = {{&S3431[13], 9}, {NULL}};
const Segment SGI360548[] = {{&S3431[33], 15}, {NULL}};
const Segment SGI360549[] = {{&S3431[49], 6}, {NULL}};
const Segment SGI360550[] = {{&S3431[33], 13}, {NULL}};
const Segment SGI360551[] = {{&S3432[69], 6}, {NULL}};
const Segment SGI360552[] = {{&S3432[8], 3}, {NULL}};
const Segment SGI360553[] = {{&S3432[13], 9}, {NULL}};
const Segment SGI360554[] = {{&S3432[33], 15}, {NULL}};
const Segment SGI360555[] = {{&S3432[49], 6}, {NULL}};
const Segment SGI360556[] = {{&S3432[33], 13}, {NULL}};
const Segment SGI360557[] = {{&S3433[69], 6}, {NULL}};
const Segment SGI360558[] = {{&S3433[8], 3}, {NULL}};
const Segment SGI360559[] = {{&S3433[13], 9}, {NULL}};
const Segment SGI360560[] = {{&S3433[33], 15}, {NULL}};
const Segment SGI360561[] = {{&S3433[49], 6}, {NULL}};
const Segment SGI360562[] = {{&S3433[33], 13}, {NULL}};
const Segment SGI360563[] = {{&S3434[66], 6}, {NULL}};
const Segment SGI360564[] = {{&S3434[8], 3}, {NULL}};
const Segment SGI360565[] = {{&S3434[13], 9}, {NULL}};
const Segment SGI360566[] = {{&S3434[33], 15}, {NULL}};
const Segment SGI360567[] = {{&S3434[49], 3}, {NULL}};
const Segment SGI360568[] = {{&S3434[33], 13}, {NULL}};
const Segment SGI360569[] = {{&S3435[69], 6}, {NULL}};
const Segment SGI360570[] = {{&S3435[8], 3}, {NULL}};
const Segment SGI360571[] = {{&S3435[13], 9}, {NULL}};
const Segment SGI360572[] = {{&S3435[33], 15}, {NULL}};
const Segment SGI360573[] = {{&S3435[49], 6}, {NULL}};
const Segment SGI360574[] = {{&S3435[33], 13}, {NULL}};
const Segment SGI360575[] = {{&S3436[119], 6}, {NULL}};
const Segment SGI360576[] = {{&S3436[8], 3}, {NULL}};
const Segment SGI360577[] = {{&S3436[13], 9}, {NULL}};
const Segment SGI360578[] = {{&S3436[33], 15}, {NULL}};
const Segment SGI360579[] = {{&S3436[49], 6}, {NULL}};
const Segment SGI360580[] = {{&S3436[33], 13}, {NULL}};
const Segment SGI360581[] = {{&S3437[69], 6}, {NULL}};
const Segment SGI360582[] = {{&S3437[8], 3}, {NULL}};
const Segment SGI360583[] = {{&S3437[13], 9}, {NULL}};
const Segment SGI360584[] = {{&S3437[33], 15}, {NULL}};
const Segment SGI360585[] = {{&S3437[49], 6}, {NULL}};
const Segment SGI360586[] = {{&S3437[33], 13}, {NULL}};
const Segment SGI360587[] = {{&S3438[69], 6}, {NULL}};
const Segment SGI360588[] = {{&S3438[8], 3}, {NULL}};
const Segment SGI360589[] = {{&S3438[13], 9}, {NULL}};
const Segment SGI360590[] = {{&S3438[33], 15}, {NULL}};
const Segment SGI360591[] = {{&S3438[49], 6}, {NULL}};
const Segment SGI360592[] = {{&S3438[33], 13}, {NULL}};
const Segment SGI360593[] = {{&S3439[69], 6}, {NULL}};
const Segment SGI360594[] = {{&S3439[8], 3}, {NULL}};
const Segment SGI360595[] = {{&S3439[13], 9}, {NULL}};
const Segment SGI360596[] = {{&S3439[33], 15}, {NULL}};
const Segment SGI360597[] = {{&S3439[49], 6}, {NULL}};
const Segment SGI360598[] = {{&S3439[33], 13}, {NULL}};
const Segment SGI360599[] = {{&S3440[69], 6}, {NULL}};
const Segment SGI360600[] = {{&S3440[8], 3}, {NULL}};
const Segment SGI360601[] = {{&S3440[13], 9}, {NULL}};
const Segment SGI360602[] = {{&S3440[33], 15}, {NULL}};
const Segment SGI360603[] = {{&S3440[49], 6}, {NULL}};
const Segment SGI360604[] = {{&S3440[33], 13}, {NULL}};
const Segment SGI360605[] = {{&S3441[66], 6}, {NULL}};
const Segment SGI360606[] = {{&S3441[8], 3}, {NULL}};
const Segment SGI360607[] = {{&S3441[13], 9}, {NULL}};
const Segment SGI360608[] = {{&S3441[33], 15}, {NULL}};
const Segment SGI360609[] = {{&S3441[49], 3}, {NULL}};
const Segment SGI360610[] = {{&S3441[33], 13}, {NULL}};
const Segment SGI360611[] = {{&S3442[69], 6}, {NULL}};
const Segment SGI360612[] = {{&S3442[8], 3}, {NULL}};
const Segment SGI360613[] = {{&S3442[13], 9}, {NULL}};
const Segment SGI360614[] = {{&S3442[33], 15}, {NULL}};
const Segment SGI360615[] = {{&S3442[49], 6}, {NULL}};
const Segment SGI360616[] = {{&S3442[33], 13}, {NULL}};
const Segment SGI360617[] = {{&S3443[69], 6}, {NULL}};
const Segment SGI360618[] = {{&S3443[8], 3}, {NULL}};
const Segment SGI360619[] = {{&S3443[13], 9}, {NULL}};
const Segment SGI360620[] = {{&S3443[33], 15}, {NULL}};
const Segment SGI360621[] = {{&S3443[49], 6}, {NULL}};
const Segment SGI360622[] = {{&S3443[33], 13}, {NULL}};
const Segment SGI360623[] = {{&S3444[69], 6}, {NULL}};
const Segment SGI360624[] = {{&S3444[8], 3}, {NULL}};
const Segment SGI360625[] = {{&S3444[13], 9}, {NULL}};
const Segment SGI360626[] = {{&S3444[33], 15}, {NULL}};
const Segment SGI360627[] = {{&S3444[49], 6}, {NULL}};
const Segment SGI360628[] = {{&S3444[33], 13}, {NULL}};
const Segment SGI360629[] = {{&S3445[69], 6}, {NULL}};
const Segment SGI360630[] = {{&S3445[8], 3}, {NULL}};
const Segment SGI360631[] = {{&S3445[13], 9}, {NULL}};
const Segment SGI360632[] = {{&S3445[33], 15}, {NULL}};
const Segment SGI360633[] = {{&S3445[49], 6}, {NULL}};
const Segment SGI360634[] = {{&S3445[33], 13}, {NULL}};
const Segment SGI360635[] = {{&S3446[69], 6}, {NULL}};
const Segment SGI360636[] = {{&S3446[8], 3}, {NULL}};
const Segment SGI360637[] = {{&S3446[13], 9}, {NULL}};
const Segment SGI360638[] = {{&S3446[33], 15}, {NULL}};
const Segment SGI360639[] = {{&S3446[49], 6}, {NULL}};
const Segment SGI360640[] = {{&S3446[33], 13}, {NULL}};
const Segment SGI360641[] = {{&S3447[69], 6}, {NULL}};
const Segment SGI360642[] = {{&S3447[8], 3}, {NULL}};
const Segment SGI360643[] = {{&S3447[13], 9}, {NULL}};
const Segment SGI360644[] = {{&S3447[33], 15}, {NULL}};
const Segment SGI360645[] = {{&S3447[49], 6}, {NULL}};
const Segment SGI360646[] = {{&S3447[33], 13}, {NULL}};
const Segment SGI360647[] = {{&S3448[69], 6}, {NULL}};
const Segment SGI360648[] = {{&S3448[8], 3}, {NULL}};
const Segment SGI360649[] = {{&S3448[13], 9}, {NULL}};
const Segment SGI360650[] = {{&S3448[33], 15}, {NULL}};
const Segment SGI360651[] = {{&S3448[49], 6}, {NULL}};
const Segment SGI360652[] = {{&S3448[33], 13}, {NULL}};
const Segment SGI360653[] = {{&S3449[69], 6}, {NULL}};
const Segment SGI360654[] = {{&S3449[8], 3}, {NULL}};
const Segment SGI360655[] = {{&S3449[13], 9}, {NULL}};
const Segment SGI360656[] = {{&S3449[33], 15}, {NULL}};
const Segment SGI360657[] = {{&S3449[49], 6}, {NULL}};
const Segment SGI360658[] = {{&S3449[33], 13}, {NULL}};
const Segment SGI360659[] = {{&S3450[69], 6}, {NULL}};
const Segment SGI360660[] = {{&S3450[8], 3}, {NULL}};
const Segment SGI360661[] = {{&S3450[13], 9}, {NULL}};
const Segment SGI360662[] = {{&S3450[33], 15}, {NULL}};
const Segment SGI360663[] = {{&S3450[49], 6}, {NULL}};
const Segment SGI360664[] = {{&S3450[33], 13}, {NULL}};
const Segment SGI360665[] = {{&S3451[69], 6}, {NULL}};
const Segment SGI360666[] = {{&S3451[8], 3}, {NULL}};
const Segment SGI360667[] = {{&S3451[13], 9}, {NULL}};
const Segment SGI360668[] = {{&S3451[33], 15}, {NULL}};
const Segment SGI360669[] = {{&S3451[49], 6}, {NULL}};
const Segment SGI360670[] = {{&S3451[33], 13}, {NULL}};
const Segment SGI360671[] = {{&S3452[69], 6}, {NULL}};
const Segment SGI360672[] = {{&S3452[8], 3}, {NULL}};
const Segment SGI360673[] = {{&S3452[13], 9}, {NULL}};
const Segment SGI360674[] = {{&S3452[33], 15}, {NULL}};
const Segment SGI360675[] = {{&S3452[49], 6}, {NULL}};
const Segment SGI360676[] = {{&S3452[33], 13}, {NULL}};
const Segment SGI360677[] = {{&S3453[69], 6}, {NULL}};
const Segment SGI360678[] = {{&S3453[8], 3}, {NULL}};
const Segment SGI360679[] = {{&S3453[13], 9}, {NULL}};
const Segment SGI360680[] = {{&S3453[33], 15}, {NULL}};
const Segment SGI360681[] = {{&S3453[49], 6}, {NULL}};
const Segment SGI360682[] = {{&S3453[33], 13}, {NULL}};
const Segment SGI360683[] = {{&S3454[69], 6}, {NULL}};
const Segment SGI360684[] = {{&S3454[8], 3}, {NULL}};
const Segment SGI360685[] = {{&S3454[13], 9}, {NULL}};
const Segment SGI360686[] = {{&S3454[33], 15}, {NULL}};
const Segment SGI360687[] = {{&S3454[49], 6}, {NULL}};
const Segment SGI360688[] = {{&S3454[33], 13}, {NULL}};
const Segment SGI360689[] = {{&S3455[69], 6}, {NULL}};
const Segment SGI360690[] = {{&S3455[8], 3}, {NULL}};
const Segment SGI360691[] = {{&S3455[13], 9}, {NULL}};
const Segment SGI360692[] = {{&S3455[33], 15}, {NULL}};
const Segment SGI360693[] = {{&S3455[49], 6}, {NULL}};
const Segment SGI360694[] = {{&S3455[33], 13}, {NULL}};
const Segment SGI360695[] = {{&S3456[69], 6}, {NULL}};
const Segment SGI360696[] = {{&S3456[8], 3}, {NULL}};
const Segment SGI360697[] = {{&S3456[13], 9}, {NULL}};
const Segment SGI360698[] = {{&S3456[33], 15}, {NULL}};
const Segment SGI360699[] = {{&S3456[49], 6}, {NULL}};
const Segment SGI360700[] = {{&S3456[33], 13}, {NULL}};
const Segment SGI360701[] = {{&S3457[69], 6}, {NULL}};
const Segment SGI360702[] = {{&S3457[8], 3}, {NULL}};
const Segment SGI360703[] = {{&S3457[13], 9}, {NULL}};
const Segment SGI360704[] = {{&S3457[33], 15}, {NULL}};
const Segment SGI360705[] = {{&S3457[49], 6}, {NULL}};
const Segment SGI360706[] = {{&S3457[33], 13}, {NULL}};
const Segment SGI360707[] = {{&S3458[69], 6}, {NULL}};
const Segment SGI360708[] = {{&S3458[8], 3}, {NULL}};
const Segment SGI360709[] = {{&S3458[13], 9}, {NULL}};
const Segment SGI360710[] = {{&S3458[33], 15}, {NULL}};
const Segment SGI360711[] = {{&S3458[49], 6}, {NULL}};
const Segment SGI360712[] = {{&S3458[33], 13}, {NULL}};
const Segment SGI360713[] = {{&S3459[69], 6}, {NULL}};
const Segment SGI360714[] = {{&S3459[8], 3}, {NULL}};
const Segment SGI360715[] = {{&S3459[13], 9}, {NULL}};
const Segment SGI360716[] = {{&S3459[33], 15}, {NULL}};
const Segment SGI360717[] = {{&S3459[49], 6}, {NULL}};
const Segment SGI360718[] = {{&S3459[33], 13}, {NULL}};
const Segment SGI360719[] = {{&S3460[69], 6}, {NULL}};
const Segment SGI360720[] = {{&S3460[8], 3}, {NULL}};
const Segment SGI360721[] = {{&S3460[13], 9}, {NULL}};
const Segment SGI360722[] = {{&S3460[33], 15}, {NULL}};
const Segment SGI360723[] = {{&S3460[49], 6}, {NULL}};
const Segment SGI360724[] = {{&S3460[33], 13}, {NULL}};
const Segment SGI360725[] = {{&S3461[69], 6}, {NULL}};
const Segment SGI360726[] = {{&S3461[8], 3}, {NULL}};
const Segment SGI360727[] = {{&S3461[13], 9}, {NULL}};
const Segment SGI360728[] = {{&S3461[33], 15}, {NULL}};
const Segment SGI360729[] = {{&S3461[49], 6}, {NULL}};
const Segment SGI360730[] = {{&S3461[33], 13}, {NULL}};
const Segment SGI360731[] = {{&S3462[69], 6}, {NULL}};
const Segment SGI360732[] = {{&S3462[8], 3}, {NULL}};
const Segment SGI360733[] = {{&S3462[13], 9}, {NULL}};
const Segment SGI360734[] = {{&S3462[33], 15}, {NULL}};
const Segment SGI360735[] = {{&S3462[49], 6}, {NULL}};
const Segment SGI360736[] = {{&S3462[33], 13}, {NULL}};
const Segment SGI360737[] = {{&S3463[69], 6}, {NULL}};
const Segment SGI360738[] = {{&S3463[8], 3}, {NULL}};
const Segment SGI360739[] = {{&S3463[13], 9}, {NULL}};
const Segment SGI360740[] = {{&S3463[33], 15}, {NULL}};
const Segment SGI360741[] = {{&S3463[49], 6}, {NULL}};
const Segment SGI360742[] = {{&S3463[33], 13}, {NULL}};
const Segment SGI360743[] = {{&S3464[69], 6}, {NULL}};
const Segment SGI360744[] = {{&S3464[8], 3}, {NULL}};
const Segment SGI360745[] = {{&S3464[13], 9}, {NULL}};
const Segment SGI360746[] = {{&S3464[33], 15}, {NULL}};
const Segment SGI360747[] = {{&S3464[49], 6}, {NULL}};
const Segment SGI360748[] = {{&S3464[33], 13}, {NULL}};
const Segment SGI360749[] = {{&S3465[69], 6}, {NULL}};
const Segment SGI360750[] = {{&S3465[8], 3}, {NULL}};
const Segment SGI360751[] = {{&S3465[13], 9}, {NULL}};
const Segment SGI360752[] = {{&S3465[33], 15}, {NULL}};
const Segment SGI360753[] = {{&S3465[49], 6}, {NULL}};
const Segment SGI360754[] = {{&S3465[33], 13}, {NULL}};
const Segment SGI360755[] = {{&S3466[69], 6}, {NULL}};
const Segment SGI360756[] = {{&S3466[8], 3}, {NULL}};
const Segment SGI360757[] = {{&S3466[13], 9}, {NULL}};
const Segment SGI360758[] = {{&S3466[33], 15}, {NULL}};
const Segment SGI360759[] = {{&S3466[49], 6}, {NULL}};
const Segment SGI360760[] = {{&S3466[33], 13}, {NULL}};
const Segment SGI360761[] = {{&S3467[69], 6}, {NULL}};
const Segment SGI360762[] = {{&S3467[8], 3}, {NULL}};
const Segment SGI360763[] = {{&S3467[13], 9}, {NULL}};
const Segment SGI360764[] = {{&S3467[33], 15}, {NULL}};
const Segment SGI360765[] = {{&S3467[49], 6}, {NULL}};
const Segment SGI360766[] = {{&S3467[33], 13}, {NULL}};
const Segment SGI360767[] = {{&S3468[69], 6}, {NULL}};
const Segment SGI360768[] = {{&S3468[8], 3}, {NULL}};
const Segment SGI360769[] = {{&S3468[13], 9}, {NULL}};
const Segment SGI360770[] = {{&S3468[33], 15}, {NULL}};
const Segment SGI360771[] = {{&S3468[49], 6}, {NULL}};
const Segment SGI360772[] = {{&S3468[33], 13}, {NULL}};
const Segment SGI360773[] = {{&S3469[69], 6}, {NULL}};
const Segment SGI360774[] = {{&S3469[8], 3}, {NULL}};
const Segment SGI360775[] = {{&S3469[13], 9}, {NULL}};
const Segment SGI360776[] = {{&S3469[33], 15}, {NULL}};
const Segment SGI360777[] = {{&S3469[49], 6}, {NULL}};
const Segment SGI360778[] = {{&S3469[33], 13}, {NULL}};
const Segment SGI360779[] = {{&S3470[69], 6}, {NULL}};
const Segment SGI360780[] = {{&S3470[8], 3}, {NULL}};
const Segment SGI360781[] = {{&S3470[13], 9}, {NULL}};
const Segment SGI360782[] = {{&S3470[33], 15}, {NULL}};
const Segment SGI360783[] = {{&S3470[49], 6}, {NULL}};
const Segment SGI360784[] = {{&S3470[33], 13}, {NULL}};
const Segment SGI360785[] = {{&S3471[69], 6}, {NULL}};
const Segment SGI360786[] = {{&S3471[8], 3}, {NULL}};
const Segment SGI360787[] = {{&S3471[13], 9}, {NULL}};
const Segment SGI360788[] = {{&S3471[33], 15}, {NULL}};
const Segment SGI360789[] = {{&S3471[49], 6}, {NULL}};
const Segment SGI360790[] = {{&S3471[33], 13}, {NULL}};
const Segment SGI360791[] = {{&S3472[69], 6}, {NULL}};
const Segment SGI360792[] = {{&S3472[8], 3}, {NULL}};
const Segment SGI360793[] = {{&S3472[13], 9}, {NULL}};
const Segment SGI360794[] = {{&S3472[33], 15}, {NULL}};
const Segment SGI360795[] = {{&S3472[49], 6}, {NULL}};
const Segment SGI360796[] = {{&S3472[33], 13}, {NULL}};
const Segment SGI360797[] = {{&S3473[69], 6}, {NULL}};
const Segment SGI360798[] = {{&S3473[8], 3}, {NULL}};
const Segment SGI360799[] = {{&S3473[13], 9}, {NULL}};
const Segment SGI360800[] = {{&S3473[33], 15}, {NULL}};
const Segment SGI360801[] = {{&S3473[49], 6}, {NULL}};
const Segment SGI360802[] = {{&S3473[33], 13}, {NULL}};
const Segment SGI360803[] = {{&S3474[69], 6}, {NULL}};
const Segment SGI360804[] = {{&S3474[8], 3}, {NULL}};
const Segment SGI360805[] = {{&S3474[13], 9}, {NULL}};
const Segment SGI360806[] = {{&S3474[33], 15}, {NULL}};
const Segment SGI360807[] = {{&S3474[49], 6}, {NULL}};
const Segment SGI360808[] = {{&S3474[33], 13}, {NULL}};
const Segment SGI360809[] = {{&S3475[69], 6}, {NULL}};
const Segment SGI360810[] = {{&S3475[8], 3}, {NULL}};
const Segment SGI360811[] = {{&S3475[13], 9}, {NULL}};
const Segment SGI360812[] = {{&S3475[33], 15}, {NULL}};
const Segment SGI360813[] = {{&S3475[49], 6}, {NULL}};
const Segment SGI360814[] = {{&S3475[33], 13}, {NULL}};
const Segment SGI360815[] = {{&S3476[69], 6}, {NULL}};
const Segment SGI360816[] = {{&S3476[8], 3}, {NULL}};
const Segment SGI360817[] = {{&S3476[13], 9}, {NULL}};
const Segment SGI360818[] = {{&S3476[33], 15}, {NULL}};
const Segment SGI360819[] = {{&S3476[49], 6}, {NULL}};
const Segment SGI360820[] = {{&S3476[33], 13}, {NULL}};
const Segment SGI360821[] = {{&S3477[69], 6}, {NULL}};
const Segment SGI360822[] = {{&S3477[8], 3}, {NULL}};
const Segment SGI360823[] = {{&S3477[13], 9}, {NULL}};
const Segment SGI360824[] = {{&S3477[33], 15}, {NULL}};
const Segment SGI360825[] = {{&S3477[49], 6}, {NULL}};
const Segment SGI360826[] = {{&S3477[33], 13}, {NULL}};
const Segment SGI360827[] = {{&S3478[118], 6}, {NULL}};
const Segment SGI360828[] = {{&S3478[8], 3}, {NULL}};
const Segment SGI360829[] = {{&S3478[13], 9}, {NULL}};
const Segment SGI360830[] = {{&S3478[33], 15}, {NULL}};
const Segment SGI360831[] = {{&S3478[49], 6}, {NULL}};
const Segment SGI360832[] = {{&S3478[33], 13}, {NULL}};
const Segment SGI360833[] = {{&S3479[69], 6}, {NULL}};
const Segment SGI360834[] = {{&S3479[8], 3}, {NULL}};
const Segment SGI360835[] = {{&S3479[13], 9}, {NULL}};
const Segment SGI360836[] = {{&S3479[33], 15}, {NULL}};
const Segment SGI360837[] = {{&S3479[49], 6}, {NULL}};
const Segment SGI360838[] = {{&S3479[33], 13}, {NULL}};
const Segment SGI360839[] = {{&S3480[69], 6}, {NULL}};
const Segment SGI360840[] = {{&S3480[8], 3}, {NULL}};
const Segment SGI360841[] = {{&S3480[13], 9}, {NULL}};
const Segment SGI360842[] = {{&S3480[33], 15}, {NULL}};
const Segment SGI360843[] = {{&S3480[49], 6}, {NULL}};
const Segment SGI360844[] = {{&S3480[33], 13}, {NULL}};
const Segment SGI360845[] = {{&S3481[69], 6}, {NULL}};
const Segment SGI360846[] = {{&S3481[8], 3}, {NULL}};
const Segment SGI360847[] = {{&S3481[13], 9}, {NULL}};
const Segment SGI360848[] = {{&S3481[33], 15}, {NULL}};
const Segment SGI360849[] = {{&S3481[49], 6}, {NULL}};
const Segment SGI360850[] = {{&S3481[33], 13}, {NULL}};
const Segment SGI360851[] = {{&S3482[69], 6}, {NULL}};
const Segment SGI360852[] = {{&S3482[8], 3}, {NULL}};
const Segment SGI360853[] = {{&S3482[13], 9}, {NULL}};
const Segment SGI360854[] = {{&S3482[33], 15}, {NULL}};
const Segment SGI360855[] = {{&S3482[49], 6}, {NULL}};
const Segment SGI360856[] = {{&S3482[33], 13}, {NULL}};
const Segment SGI360857[] = {{&S3483[69], 6}, {NULL}};
const Segment SGI360858[] = {{&S3483[8], 3}, {NULL}};
const Segment SGI360859[] = {{&S3483[13], 9}, {NULL}};
const Segment SGI360860[] = {{&S3483[33], 15}, {NULL}};
const Segment SGI360861[] = {{&S3483[49], 6}, {NULL}};
const Segment SGI360862[] = {{&S3483[33], 13}, {NULL}};
const Segment SGI360863[] = {{&S3484[69], 6}, {NULL}};
const Segment SGI360864[] = {{&S3484[8], 3}, {NULL}};
const Segment SGI360865[] = {{&S3484[13], 9}, {NULL}};
const Segment SGI360866[] = {{&S3484[33], 15}, {NULL}};
const Segment SGI360867[] = {{&S3484[49], 6}, {NULL}};
const Segment SGI360868[] = {{&S3484[33], 13}, {NULL}};
const Segment SGI360869[] = {{&S3485[69], 6}, {NULL}};
const Segment SGI360870[] = {{&S3485[8], 3}, {NULL}};
const Segment SGI360871[] = {{&S3485[13], 9}, {NULL}};
const Segment SGI360872[] = {{&S3485[33], 15}, {NULL}};
const Segment SGI360873[] = {{&S3485[49], 6}, {NULL}};
const Segment SGI360874[] = {{&S3485[33], 13}, {NULL}};
const Segment SGI360875[] = {{&S3486[69], 6}, {NULL}};
const Segment SGI360876[] = {{&S3486[8], 3}, {NULL}};
const Segment SGI360877[] = {{&S3486[13], 9}, {NULL}};
const Segment SGI360878[] = {{&S3486[33], 15}, {NULL}};
const Segment SGI360879[] = {{&S3486[49], 6}, {NULL}};
const Segment SGI360880[] = {{&S3486[33], 13}, {NULL}};
const Segment SGI360881[] = {{&S3487[69], 6}, {NULL}};
const Segment SGI360882[] = {{&S3487[8], 3}, {NULL}};
const Segment SGI360883[] = {{&S3487[13], 9}, {NULL}};
const Segment SGI360884[] = {{&S3487[33], 15}, {NULL}};
const Segment SGI360885[] = {{&S3487[49], 6}, {NULL}};
const Segment SGI360886[] = {{&S3487[33], 13}, {NULL}};
const Segment SGI360887[] = {{&S3488[69], 6}, {NULL}};
const Segment SGI360888[] = {{&S3488[8], 3}, {NULL}};
const Segment SGI360889[] = {{&S3488[13], 9}, {NULL}};
const Segment SGI360890[] = {{&S3488[33], 15}, {NULL}};
const Segment SGI360891[] = {{&S3488[49], 6}, {NULL}};
const Segment SGI360892[] = {{&S3488[33], 13}, {NULL}};
const Segment SGI360893[] = {{&S3489[69], 6}, {NULL}};
const Segment SGI360894[] = {{&S3489[8], 3}, {NULL}};
const Segment SGI360895[] = {{&S3489[13], 9}, {NULL}};
const Segment SGI360896[] = {{&S3489[33], 15}, {NULL}};
const Segment SGI360897[] = {{&S3489[49], 6}, {NULL}};
const Segment SGI360898[] = {{&S3489[33], 13}, {NULL}};
const Segment SGI360899[] = {{&S3490[69], 6}, {NULL}};
const Segment SGI360900[] = {{&S3490[8], 3}, {NULL}};
const Segment SGI360901[] = {{&S3490[13], 9}, {NULL}};
const Segment SGI360902[] = {{&S3490[33], 15}, {NULL}};
const Segment SGI360903[] = {{&S3490[49], 6}, {NULL}};
const Segment SGI360904[] = {{&S3490[33], 13}, {NULL}};
const Segment SGI360905[] = {{&S3491[69], 6}, {NULL}};
const Segment SGI360906[] = {{&S3491[8], 3}, {NULL}};
const Segment SGI360907[] = {{&S3491[13], 9}, {NULL}};
const Segment SGI360908[] = {{&S3491[33], 15}, {NULL}};
const Segment SGI360909[] = {{&S3491[49], 6}, {NULL}};
const Segment SGI360910[] = {{&S3491[33], 13}, {NULL}};
const Segment SGI360911[] = {{&S3492[69], 6}, {NULL}};
const Segment SGI360912[] = {{&S3492[8], 3}, {NULL}};
const Segment SGI360913[] = {{&S3492[13], 9}, {NULL}};
const Segment SGI360914[] = {{&S3492[33], 15}, {NULL}};
const Segment SGI360915[] = {{&S3492[49], 6}, {NULL}};
const Segment SGI360916[] = {{&S3492[33], 13}, {NULL}};
const Segment SGI360917[] = {{&S3493[69], 6}, {NULL}};
const Segment SGI360918[] = {{&S3493[8], 3}, {NULL}};
const Segment SGI360919[] = {{&S3493[13], 9}, {NULL}};
const Segment SGI360920[] = {{&S3493[33], 15}, {NULL}};
const Segment SGI360921[] = {{&S3493[49], 6}, {NULL}};
const Segment SGI360922[] = {{&S3493[33], 13}, {NULL}};
const Segment SGI360923[] = {{&S3494[121], 7}, {NULL}};
const Segment SGI360924[] = {{&S3494[8], 3}, {NULL}};
const Segment SGI360925[] = {{&S3494[13], 9}, {NULL}};
const Segment SGI360926[] = {{&S3494[33], 15}, {NULL}};
const Segment SGI360927[] = {{&S3494[49], 6}, {NULL}};
const Segment SGI360928[] = {{&S3494[33], 13}, {NULL}};
const Segment SGI360929[] = {{&S3495[69], 6}, {NULL}};
const Segment SGI360930[] = {{&S3495[8], 3}, {NULL}};
const Segment SGI360931[] = {{&S3495[13], 9}, {NULL}};
const Segment SGI360932[] = {{&S3495[33], 15}, {NULL}};
const Segment SGI360933[] = {{&S3495[49], 6}, {NULL}};
const Segment SGI360934[] = {{&S3495[33], 13}, {NULL}};
const Segment SGI360935[] = {{&S3496[69], 6}, {NULL}};
const Segment SGI360936[] = {{&S3496[8], 3}, {NULL}};
const Segment SGI360937[] = {{&S3496[13], 9}, {NULL}};
const Segment SGI360938[] = {{&S3496[33], 15}, {NULL}};
const Segment SGI360939[] = {{&S3496[49], 6}, {NULL}};
const Segment SGI360940[] = {{&S3496[33], 13}, {NULL}};
const Segment SGI360941[] = {{&S3497[69], 6}, {NULL}};
const Segment SGI360942[] = {{&S3497[8], 3}, {NULL}};
const Segment SGI360943[] = {{&S3497[13], 9}, {NULL}};
const Segment SGI360944[] = {{&S3497[33], 15}, {NULL}};
const Segment SGI360945[] = {{&S3497[49], 6}, {NULL}};
const Segment SGI360946[] = {{&S3497[33], 13}, {NULL}};
const Segment SGI360947[] = {{&S3498[69], 6}, {NULL}};
const Segment SGI360948[] = {{&S3498[8], 3}, {NULL}};
const Segment SGI360949[] = {{&S3498[13], 9}, {NULL}};
const Segment SGI360950[] = {{&S3498[33], 15}, {NULL}};
const Segment SGI360951[] = {{&S3498[49], 6}, {NULL}};
const Segment SGI360952[] = {{&S3498[33], 13}, {NULL}};
const Segment SGI360953[] = {{&S3499[69], 6}, {NULL}};
const Segment SGI360954[] = {{&S3499[8], 3}, {NULL}};
const Segment SGI360955[] = {{&S3499[13], 9}, {NULL}};
const Segment SGI360956[] = {{&S3499[33], 15}, {NULL}};
const Segment SGI360957[] = {{&S3499[49], 6}, {NULL}};
const Segment SGI360958[] = {{&S3499[33], 13}, {NULL}};
const Segment SGI360959[] = {{&S3500[69], 6}, {NULL}};
const Segment SGI360960[] = {{&S3500[8], 3}, {NULL}};
const Segment SGI360961[] = {{&S3500[13], 9}, {NULL}};
const Segment SGI360962[] = {{&S3500[33], 15}, {NULL}};
const Segment SGI360963[] = {{&S3500[49], 6}, {NULL}};
const Segment SGI360964[] = {{&S3500[33], 13}, {NULL}};
const Segment SGI360965[] = {{&S3501[69], 6}, {NULL}};
const Segment SGI360966[] = {{&S3501[8], 3}, {NULL}};
const Segment SGI360967[] = {{&S3501[13], 9}, {NULL}};
const Segment SGI360968[] = {{&S3501[33], 15}, {NULL}};
const Segment SGI360969[] = {{&S3501[49], 6}, {NULL}};
const Segment SGI360970[] = {{&S3501[33], 13}, {NULL}};
const Segment SGI360971[] = {{&S3502[69], 6}, {NULL}};
const Segment SGI360972[] = {{&S3502[8], 3}, {NULL}};
const Segment SGI360973[] = {{&S3502[13], 9}, {NULL}};
const Segment SGI360974[] = {{&S3502[33], 15}, {NULL}};
const Segment SGI360975[] = {{&S3502[49], 6}, {NULL}};
const Segment SGI360976[] = {{&S3502[33], 13}, {NULL}};
const Segment SGI360977[] = {{&S3503[66], 6}, {NULL}};
const Segment SGI360978[] = {{&S3503[8], 3}, {NULL}};
const Segment SGI360979[] = {{&S3503[13], 9}, {NULL}};
const Segment SGI360980[] = {{&S3503[33], 15}, {NULL}};
const Segment SGI360981[] = {{&S3503[49], 3}, {NULL}};
const Segment SGI360982[] = {{&S3503[33], 13}, {NULL}};
const Segment SGI360983[] = {{&S3504[69], 6}, {NULL}};
const Segment SGI360984[] = {{&S3504[8], 3}, {NULL}};
const Segment SGI360985[] = {{&S3504[13], 9}, {NULL}};
const Segment SGI360986[] = {{&S3504[33], 15}, {NULL}};
const Segment SGI360987[] = {{&S3504[49], 6}, {NULL}};
const Segment SGI360988[] = {{&S3504[33], 13}, {NULL}};
const Segment SGI360989[] = {{&S3505[119], 6}, {NULL}};
const Segment SGI360990[] = {{&S3505[8], 3}, {NULL}};
const Segment SGI360991[] = {{&S3505[13], 9}, {NULL}};
const Segment SGI360992[] = {{&S3505[33], 15}, {NULL}};
const Segment SGI360993[] = {{&S3505[49], 6}, {NULL}};
const Segment SGI360994[] = {{&S3505[33], 13}, {NULL}};
const Segment SGI360995[] = {{&S3506[69], 6}, {NULL}};
const Segment SGI360996[] = {{&S3506[8], 3}, {NULL}};
const Segment SGI360997[] = {{&S3506[13], 9}, {NULL}};
const Segment SGI360998[] = {{&S3506[33], 15}, {NULL}};
const Segment SGI360999[] = {{&S3506[49], 6}, {NULL}};
const Segment SGI361000[] = {{&S3506[33], 13}, {NULL}};
const Segment SGI361001[] = {{&S3507[69], 6}, {NULL}};
const Segment SGI361002[] = {{&S3507[8], 3}, {NULL}};
const Segment SGI361003[] = {{&S3507[13], 9}, {NULL}};
const Segment SGI361004[] = {{&S3507[33], 15}, {NULL}};
const Segment SGI361005[] = {{&S3507[49], 6}, {NULL}};
const Segment SGI361006[] = {{&S3507[33], 13}, {NULL}};
const Segment SGI361007[] = {{&S3508[69], 6}, {NULL}};
const Segment SGI361008[] = {{&S3508[8], 3}, {NULL}};
const Segment SGI361009[] = {{&S3508[13], 9}, {NULL}};
const Segment SGI361010[] = {{&S3508[33], 15}, {NULL}};
const Segment SGI361011[] = {{&S3508[49], 6}, {NULL}};
const Segment SGI361012[] = {{&S3508[33], 13}, {NULL}};
const Segment SGI361013[] = {{&S3509[69], 6}, {NULL}};
const Segment SGI361014[] = {{&S3509[8], 3}, {NULL}};
const Segment SGI361015[] = {{&S3509[13], 9}, {NULL}};
const Segment SGI361016[] = {{&S3509[33], 15}, {NULL}};
const Segment SGI361017[] = {{&S3509[49], 6}, {NULL}};
const Segment SGI361018[] = {{&S3509[33], 13}, {NULL}};
const Segment SGI361019[] = {{&S3510[69], 6}, {NULL}};
const Segment SGI361020[] = {{&S3510[8], 3}, {NULL}};
const Segment SGI361021[] = {{&S3510[13], 9}, {NULL}};
const Segment SGI361022[] = {{&S3510[33], 15}, {NULL}};
const Segment SGI361023[] = {{&S3510[49], 6}, {NULL}};
const Segment SGI361024[] = {{&S3510[33], 13}, {NULL}};
const Segment SGI361025[] = {{&S3511[119], 6}, {NULL}};
const Segment SGI361026[] = {{&S3511[8], 3}, {NULL}};
const Segment SGI361027[] = {{&S3511[13], 9}, {NULL}};
const Segment SGI361028[] = {{&S3511[33], 15}, {NULL}};
const Segment SGI361029[] = {{&S3511[49], 6}, {NULL}};
const Segment SGI361030[] = {{&S3511[33], 13}, {NULL}};
const Segment SGI361031[] = {{&S3512[69], 6}, {NULL}};
const Segment SGI361032[] = {{&S3512[8], 3}, {NULL}};
const Segment SGI361033[] = {{&S3512[13], 9}, {NULL}};
const Segment SGI361034[] = {{&S3512[33], 15}, {NULL}};
const Segment SGI361035[] = {{&S3512[49], 6}, {NULL}};
const Segment SGI361036[] = {{&S3512[33], 13}, {NULL}};
const Segment SGI361037[] = {{&S3513[69], 6}, {NULL}};
const Segment SGI361038[] = {{&S3513[8], 3}, {NULL}};
const Segment SGI361039[] = {{&S3513[13], 9}, {NULL}};
const Segment SGI361040[] = {{&S3513[33], 15}, {NULL}};
const Segment SGI361041[] = {{&S3513[49], 6}, {NULL}};
const Segment SGI361042[] = {{&S3513[33], 13}, {NULL}};
const Segment SGI361043[] = {{&S3514[119], 6}, {NULL}};
const Segment SGI361044[] = {{&S3514[8], 3}, {NULL}};
const Segment SGI361045[] = {{&S3514[13], 9}, {NULL}};
const Segment SGI361046[] = {{&S3514[33], 15}, {NULL}};
const Segment SGI361047[] = {{&S3514[49], 6}, {NULL}};
const Segment SGI361048[] = {{&S3514[33], 13}, {NULL}};
const Segment SGI361049[] = {{&S3515[69], 6}, {NULL}};
const Segment SGI361050[] = {{&S3515[8], 3}, {NULL}};
const Segment SGI361051[] = {{&S3515[13], 9}, {NULL}};
const Segment SGI361052[] = {{&S3515[33], 15}, {NULL}};
const Segment SGI361053[] = {{&S3515[49], 6}, {NULL}};
const Segment SGI361054[] = {{&S3515[33], 13}, {NULL}};
const Segment SGI361055[] = {{&S3516[69], 6}, {NULL}};
const Segment SGI361056[] = {{&S3516[8], 3}, {NULL}};
const Segment SGI361057[] = {{&S3516[13], 9}, {NULL}};
const Segment SGI361058[] = {{&S3516[33], 15}, {NULL}};
const Segment SGI361059[] = {{&S3516[49], 6}, {NULL}};
const Segment SGI361060[] = {{&S3516[33], 13}, {NULL}};
const Segment SGI361061[] = {{&S3517[69], 6}, {NULL}};
const Segment SGI361062[] = {{&S3517[8], 3}, {NULL}};
const Segment SGI361063[] = {{&S3517[13], 9}, {NULL}};
const Segment SGI361064[] = {{&S3517[33], 15}, {NULL}};
const Segment SGI361065[] = {{&S3517[49], 6}, {NULL}};
const Segment SGI361066[] = {{&S3517[33], 13}, {NULL}};
const Segment SGI361067[] = {{&S3518[119], 6}, {NULL}};
const Segment SGI361068[] = {{&S3518[8], 3}, {NULL}};
const Segment SGI361069[] = {{&S3518[13], 9}, {NULL}};
const Segment SGI361070[] = {{&S3518[33], 15}, {NULL}};
const Segment SGI361071[] = {{&S3518[49], 6}, {NULL}};
const Segment SGI361072[] = {{&S3518[33], 13}, {NULL}};
const Segment SGI361073[] = {{&S3519[69], 6}, {NULL}};
const Segment SGI361074[] = {{&S3519[8], 3}, {NULL}};
const Segment SGI361075[] = {{&S3519[13], 9}, {NULL}};
const Segment SGI361076[] = {{&S3519[33], 15}, {NULL}};
const Segment SGI361077[] = {{&S3519[49], 6}, {NULL}};
const Segment SGI361078[] = {{&S3519[33], 13}, {NULL}};
const Segment SGI361079[] = {{&S3520[69], 6}, {NULL}};
const Segment SGI361080[] = {{&S3520[8], 3}, {NULL}};
const Segment SGI361081[] = {{&S3520[13], 9}, {NULL}};
const Segment SGI361082[] = {{&S3520[33], 15}, {NULL}};
const Segment SGI361083[] = {{&S3520[49], 6}, {NULL}};
const Segment SGI361084[] = {{&S3520[33], 13}, {NULL}};
const Segment SGI361085[] = {{&S3521[69], 6}, {NULL}};
const Segment SGI361086[] = {{&S3521[8], 3}, {NULL}};
const Segment SGI361087[] = {{&S3521[13], 9}, {NULL}};
const Segment SGI361088[] = {{&S3521[33], 15}, {NULL}};
const Segment SGI361089[] = {{&S3521[49], 6}, {NULL}};
const Segment SGI361090[] = {{&S3521[33], 13}, {NULL}};
const Segment SGI361091[] = {{&S3522[69], 6}, {NULL}};
const Segment SGI361092[] = {{&S3522[8], 3}, {NULL}};
const Segment SGI361093[] = {{&S3522[13], 9}, {NULL}};
const Segment SGI361094[] = {{&S3522[33], 15}, {NULL}};
const Segment SGI361095[] = {{&S3522[49], 6}, {NULL}};
const Segment SGI361096[] = {{&S3522[33], 13}, {NULL}};
const Segment SGI361097[] = {{&S3523[69], 6}, {NULL}};
const Segment SGI361098[] = {{&S3523[8], 3}, {NULL}};
const Segment SGI361099[] = {{&S3523[13], 9}, {NULL}};
const Segment SGI361100[] = {{&S3523[33], 15}, {NULL}};
const Segment SGI361101[] = {{&S3523[49], 6}, {NULL}};
const Segment SGI361102[] = {{&S3523[33], 13}, {NULL}};
const Segment SGI361103[] = {{&S3524[69], 6}, {NULL}};
const Segment SGI361104[] = {{&S3524[8], 3}, {NULL}};
const Segment SGI361105[] = {{&S3524[13], 9}, {NULL}};
const Segment SGI361106[] = {{&S3524[33], 15}, {NULL}};
const Segment SGI361107[] = {{&S3524[49], 6}, {NULL}};
const Segment SGI361108[] = {{&S3524[33], 13}, {NULL}};
const Segment SGI361109[] = {{&S3525[117], 6}, {NULL}};
const Segment SGI361110[] = {{&S3525[8], 3}, {NULL}};
const Segment SGI361111[] = {{&S3525[13], 9}, {NULL}};
const Segment SGI361112[] = {{&S3525[33], 15}, {NULL}};
const Segment SGI361113[] = {{&S3525[49], 6}, {NULL}};
const Segment SGI361114[] = {{&S3525[33], 13}, {NULL}};
const Segment SGI361115[] = {{&S3526[66], 6}, {NULL}};
const Segment SGI361116[] = {{&S3526[8], 3}, {NULL}};
const Segment SGI361117[] = {{&S3526[13], 9}, {NULL}};
const Segment SGI361118[] = {{&S3526[33], 15}, {NULL}};
const Segment SGI361119[] = {{&S3526[49], 3}, {NULL}};
const Segment SGI361120[] = {{&S3526[33], 13}, {NULL}};
const Segment SGI361121[] = {{&S3527[69], 6}, {NULL}};
const Segment SGI361122[] = {{&S3527[8], 3}, {NULL}};
const Segment SGI361123[] = {{&S3527[13], 9}, {NULL}};
const Segment SGI361124[] = {{&S3527[33], 15}, {NULL}};
const Segment SGI361125[] = {{&S3527[49], 6}, {NULL}};
const Segment SGI361126[] = {{&S3527[33], 13}, {NULL}};
const Segment SGI361127[] = {{&S3528[66], 6}, {NULL}};
const Segment SGI361128[] = {{&S3528[8], 3}, {NULL}};
const Segment SGI361129[] = {{&S3528[13], 9}, {NULL}};
const Segment SGI361130[] = {{&S3528[33], 15}, {NULL}};
const Segment SGI361131[] = {{&S3528[49], 3}, {NULL}};
const Segment SGI361132[] = {{&S3528[33], 13}, {NULL}};
const Segment SGI361133[] = {{&S3529[69], 6}, {NULL}};
const Segment SGI361134[] = {{&S3529[8], 3}, {NULL}};
const Segment SGI361135[] = {{&S3529[13], 9}, {NULL}};
const Segment SGI361136[] = {{&S3529[33], 15}, {NULL}};
const Segment SGI361137[] = {{&S3529[49], 6}, {NULL}};
const Segment SGI361138[] = {{&S3529[33], 13}, {NULL}};
const Segment SGI361139[] = {{&S3530[69], 6}, {NULL}};
const Segment SGI361140[] = {{&S3530[8], 3}, {NULL}};
const Segment SGI361141[] = {{&S3530[13], 9}, {NULL}};
const Segment SGI361142[] = {{&S3530[33], 15}, {NULL}};
const Segment SGI361143[] = {{&S3530[49], 6}, {NULL}};
const Segment SGI361144[] = {{&S3530[33], 13}, {NULL}};
const Segment SGI361145[] = {{&S3531[66], 6}, {NULL}};
const Segment SGI361146[] = {{&S3531[8], 3}, {NULL}};
const Segment SGI361147[] = {{&S3531[13], 9}, {NULL}};
const Segment SGI361148[] = {{&S3531[33], 15}, {NULL}};
const Segment SGI361149[] = {{&S3531[49], 3}, {NULL}};
const Segment SGI361150[] = {{&S3531[33], 13}, {NULL}};
const Segment SGI361151[] = {{&S3532[66], 6}, {NULL}};
const Segment SGI361152[] = {{&S3532[8], 3}, {NULL}};
const Segment SGI361153[] = {{&S3532[13], 9}, {NULL}};
const Segment SGI361154[] = {{&S3532[33], 15}, {NULL}};
const Segment SGI361155[] = {{&S3532[49], 3}, {NULL}};
const Segment SGI361156[] = {{&S3532[33], 13}, {NULL}};
const Segment SGI361157[] = {{&S3533[69], 6}, {NULL}};
const Segment SGI361158[] = {{&S3533[8], 3}, {NULL}};
const Segment SGI361159[] = {{&S3533[13], 9}, {NULL}};
const Segment SGI361160[] = {{&S3533[33], 15}, {NULL}};
const Segment SGI361161[] = {{&S3533[49], 6}, {NULL}};
const Segment SGI361162[] = {{&S3533[33], 13}, {NULL}};
const Segment SGI361163[] = {{&S3534[69], 6}, {NULL}};
const Segment SGI361164[] = {{&S3534[8], 3}, {NULL}};
const Segment SGI361165[] = {{&S3534[13], 9}, {NULL}};
const Segment SGI361166[] = {{&S3534[33], 15}, {NULL}};
const Segment SGI361167[] = {{&S3534[49], 6}, {NULL}};
const Segment SGI361168[] = {{&S3534[33], 13}, {NULL}};
const Segment SGI361169[] = {{&S3535[69], 6}, {NULL}};
const Segment SGI361170[] = {{&S3535[8], 3}, {NULL}};
const Segment SGI361171[] = {{&S3535[13], 9}, {NULL}};
const Segment SGI361172[] = {{&S3535[33], 15}, {NULL}};
const Segment SGI361173[] = {{&S3535[49], 6}, {NULL}};
const Segment SGI361174[] = {{&S3535[33], 13}, {NULL}};
const Segment SGI361175[] = {{&S3536[69], 6}, {NULL}};
const Segment SGI361176[] = {{&S3536[8], 3}, {NULL}};
const Segment SGI361177[] = {{&S3536[13], 9}, {NULL}};
const Segment SGI361178[] = {{&S3536[33], 15}, {NULL}};
const Segment SGI361179[] = {{&S3536[49], 6}, {NULL}};
const Segment SGI361180[] = {{&S3536[33], 13}, {NULL}};
const Segment SGI361181[] = {{&S3537[82], 6}, {NULL}};
const Segment SGI361182[] = {{&S3537[8], 3}, {NULL}};
const Segment SGI361183[] = {{&S3537[13], 9}, {NULL}};
const Segment SGI361184[] = {{&S3537[33], 15}, {NULL}};
const Segment SGI361185[] = {{&S3537[49], 12}, {NULL}};
const Segment SGI361186[] = {{&S3537[33], 13}, {NULL}};
const Segment SGI361187[] = {{&S3538[69], 6}, {NULL}};
const Segment SGI361188[] = {{&S3538[8], 3}, {NULL}};
const Segment SGI361189[] = {{&S3538[13], 9}, {NULL}};
const Segment SGI361190[] = {{&S3538[33], 15}, {NULL}};
const Segment SGI361191[] = {{&S3538[49], 6}, {NULL}};
const Segment SGI361192[] = {{&S3538[33], 13}, {NULL}};
const Segment SGI361193[] = {{&S3539[69], 6}, {NULL}};
const Segment SGI361194[] = {{&S3539[8], 3}, {NULL}};
const Segment SGI361195[] = {{&S3539[13], 9}, {NULL}};
const Segment SGI361196[] = {{&S3539[33], 15}, {NULL}};
const Segment SGI361197[] = {{&S3539[49], 6}, {NULL}};
const Segment SGI361198[] = {{&S3539[33], 13}, {NULL}};
const Segment SGI361199[] = {{&S3540[69], 6}, {NULL}};
const Segment SGI361200[] = {{&S3540[8], 3}, {NULL}};
const Segment SGI361201[] = {{&S3540[13], 9}, {NULL}};
const Segment SGI361202[] = {{&S3540[33], 15}, {NULL}};
const Segment SGI361203[] = {{&S3540[49], 6}, {NULL}};
const Segment SGI361204[] = {{&S3540[33], 13}, {NULL}};
const Segment SGI361205[] = {{&S3541[69], 6}, {NULL}};
const Segment SGI361206[] = {{&S3541[8], 3}, {NULL}};
const Segment SGI361207[] = {{&S3541[13], 9}, {NULL}};
const Segment SGI361208[] = {{&S3541[33], 15}, {NULL}};
const Segment SGI361209[] = {{&S3541[49], 6}, {NULL}};
const Segment SGI361210[] = {{&S3541[33], 13}, {NULL}};
const Segment SGI361211[] = {{&S3542[69], 6}, {NULL}};
const Segment SGI361212[] = {{&S3542[8], 3}, {NULL}};
const Segment SGI361213[] = {{&S3542[13], 9}, {NULL}};
const Segment SGI361214[] = {{&S3542[33], 15}, {NULL}};
const Segment SGI361215[] = {{&S3542[49], 6}, {NULL}};
const Segment SGI361216[] = {{&S3542[33], 13}, {NULL}};
const Segment SGI361217[] = {{&S3543[69], 6}, {NULL}};
const Segment SGI361218[] = {{&S3543[8], 3}, {NULL}};
const Segment SGI361219[] = {{&S3543[13], 9}, {NULL}};
const Segment SGI361220[] = {{&S3543[33], 15}, {NULL}};
const Segment SGI361221[] = {{&S3543[49], 6}, {NULL}};
const Segment SGI361222[] = {{&S3543[33], 13}, {NULL}};
const Segment SGI361223[] = {{&S3544[69], 6}, {NULL}};
const Segment SGI361224[] = {{&S3544[8], 3}, {NULL}};
const Segment SGI361225[] = {{&S3544[13], 9}, {NULL}};
const Segment SGI361226[] = {{&S3544[33], 15}, {NULL}};
const Segment SGI361227[] = {{&S3544[49], 6}, {NULL}};
const Segment SGI361228[] = {{&S3544[33], 13}, {NULL}};
const Segment SGI361229[] = {{&S3545[69], 6}, {NULL}};
const Segment SGI361230[] = {{&S3545[8], 3}, {NULL}};
const Segment SGI361231[] = {{&S3545[13], 9}, {NULL}};
const Segment SGI361232[] = {{&S3545[33], 15}, {NULL}};
const Segment SGI361233[] = {{&S3545[49], 6}, {NULL}};
const Segment SGI361234[] = {{&S3545[33], 13}, {NULL}};
const Segment SGI361235[] = {{&S3546[69], 6}, {NULL}};
const Segment SGI361236[] = {{&S3546[8], 3}, {NULL}};
const Segment SGI361237[] = {{&S3546[13], 9}, {NULL}};
const Segment SGI361238[] = {{&S3546[33], 15}, {NULL}};
const Segment SGI361239[] = {{&S3546[49], 6}, {NULL}};
const Segment SGI361240[] = {{&S3546[33], 13}, {NULL}};
const Segment SGI361241[] = {{&S3547[69], 6}, {NULL}};
const Segment SGI361242[] = {{&S3547[8], 3}, {NULL}};
const Segment SGI361243[] = {{&S3547[13], 9}, {NULL}};
const Segment SGI361244[] = {{&S3547[33], 15}, {NULL}};
const Segment SGI361245[] = {{&S3547[49], 6}, {NULL}};
const Segment SGI361246[] = {{&S3547[33], 13}, {NULL}};
const Segment SGI361247[] = {{&S3548[69], 6}, {NULL}};
const Segment SGI361248[] = {{&S3548[8], 3}, {NULL}};
const Segment SGI361249[] = {{&S3548[13], 9}, {NULL}};
const Segment SGI361250[] = {{&S3548[33], 15}, {NULL}};
const Segment SGI361251[] = {{&S3548[49], 6}, {NULL}};
const Segment SGI361252[] = {{&S3548[33], 13}, {NULL}};
const Segment SGI361253[] = {{&S3549[69], 6}, {NULL}};
const Segment SGI361254[] = {{&S3549[8], 3}, {NULL}};
const Segment SGI361255[] = {{&S3549[13], 9}, {NULL}};
const Segment SGI361256[] = {{&S3549[33], 15}, {NULL}};
const Segment SGI361257[] = {{&S3549[49], 6}, {NULL}};
const Segment SGI361258[] = {{&S3549[33], 13}, {NULL}};
const Segment SGI361259[] = {{&S3550[69], 6}, {NULL}};
const Segment SGI361260[] = {{&S3550[8], 3}, {NULL}};
const Segment SGI361261[] = {{&S3550[13], 9}, {NULL}};
const Segment SGI361262[] = {{&S3550[33], 15}, {NULL}};
const Segment SGI361263[] = {{&S3550[49], 6}, {NULL}};
const Segment SGI361264[] = {{&S3550[33], 13}, {NULL}};
const Segment SGI361265[] = {{&S3551[69], 6}, {NULL}};
const Segment SGI361266[] = {{&S3551[8], 3}, {NULL}};
const Segment SGI361267[] = {{&S3551[13], 9}, {NULL}};
const Segment SGI361268[] = {{&S3551[33], 15}, {NULL}};
const Segment SGI361269[] = {{&S3551[49], 6}, {NULL}};
const Segment SGI361270[] = {{&S3551[33], 13}, {NULL}};
const Segment SGI361271[] = {{&S4589[13], 43}, {NULL}};
const Segment SGI361272[] = {{&S3552[69], 6}, {NULL}};
const Segment SGI361273[] = {{&S4589[70], 5}, {NULL}};
const Segment SGI361274[] = {{&S3552[8], 3}, {NULL}};
const Segment SGI361275[] = {{&S4589[104], 3}, {NULL}};
const Segment SGI361276[] = {{&S3552[13], 9}, {NULL}};
const Segment SGI361277[] = {{&S3552[33], 15}, {NULL}};
const Segment SGI361278[] = {{&S3552[49], 6}, {NULL}};
const Segment SGI361279[] = {{&S3552[33], 13}, {NULL}};
const Segment SGI361280[] = {{&S4590[13], 43}, {NULL}};
const Segment SGI361281[] = {{&S4590[70], 5}, {NULL}};
const Segment SGI361282[] = {{&S4590[104], 3}, {NULL}};
const Segment SGI361283[] = {{&S3553[69], 6}, {NULL}};
const Segment SGI361284[] = {{&S3553[8], 3}, {NULL}};
const Segment SGI361285[] = {{&S3553[13], 9}, {NULL}};
const Segment SGI361286[] = {{&S3553[33], 15}, {NULL}};
const Segment SGI361287[] = {{&S3553[49], 6}, {NULL}};
const Segment SGI361288[] = {{&S3553[33], 13}, {NULL}};
const Segment SGI361289[] = {{&S3554[69], 6}, {NULL}};
const Segment SGI361290[] = {{&S3554[8], 3}, {NULL}};
const Segment SGI361291[] = {{&S3554[13], 9}, {NULL}};
const Segment SGI361292[] = {{&S3554[33], 15}, {NULL}};
const Segment SGI361293[] = {{&S3554[49], 6}, {NULL}};
const Segment SGI361294[] = {{&S3554[33], 13}, {NULL}};
const Segment SGI361295[] = {{&S3555[69], 6}, {NULL}};
const Segment SGI361296[] = {{&S3555[8], 3}, {NULL}};
const Segment SGI361297[] = {{&S3555[13], 9}, {NULL}};
const Segment SGI361298[] = {{&S3555[33], 15}, {NULL}};
const Segment SGI361299[] = {{&S3555[49], 6}, {NULL}};
const Segment SGI361300[] = {{&S3555[33], 13}, {NULL}};
const Segment SGI361301[] = {{&S3556[69], 6}, {NULL}};
const Segment SGI361302[] = {{&S3556[8], 3}, {NULL}};
const Segment SGI361303[] = {{&S3556[13], 9}, {NULL}};
const Segment SGI361304[] = {{&S3556[33], 15}, {NULL}};
const Segment SGI361305[] = {{&S3556[49], 6}, {NULL}};
const Segment SGI361306[] = {{&S3556[33], 13}, {NULL}};
const Segment SGI361307[] = {{&S3557[69], 6}, {NULL}};
const Segment SGI361308[] = {{&S3557[8], 3}, {NULL}};
const Segment SGI361309[] = {{&S3557[13], 9}, {NULL}};
const Segment SGI361310[] = {{&S3557[33], 15}, {NULL}};
const Segment SGI361311[] = {{&S3557[49], 6}, {NULL}};
const Segment SGI361312[] = {{&S3557[33], 13}, {NULL}};
const Segment SGI361313[] = {{&S3558[69], 6}, {NULL}};
const Segment SGI361314[] = {{&S3558[8], 3}, {NULL}};
const Segment SGI361315[] = {{&S3558[13], 9}, {NULL}};
const Segment SGI361316[] = {{&S3558[33], 15}, {NULL}};
const Segment SGI361317[] = {{&S3558[49], 6}, {NULL}};
const Segment SGI361318[] = {{&S3558[33], 13}, {NULL}};
const Segment SGI361319[] = {{&S3559[69], 6}, {NULL}};
const Segment SGI361320[] = {{&S3559[8], 3}, {NULL}};
const Segment SGI361321[] = {{&S3559[13], 9}, {NULL}};
const Segment SGI361322[] = {{&S3559[33], 15}, {NULL}};
const Segment SGI361323[] = {{&S3559[49], 6}, {NULL}};
const Segment SGI361324[] = {{&S3559[33], 13}, {NULL}};
const Segment SGI361325[] = {{&S4603[13], 43}, {NULL}};
const Segment SGI361326[] = {{&S4603[70], 5}, {NULL}};
const Segment SGI361327[] = {{&S4603[104], 3}, {NULL}};
const Segment SGI361328[] = {{&S3560[69], 6}, {NULL}};
const Segment SGI361329[] = {{&S3560[8], 3}, {NULL}};
const Segment SGI361330[] = {{&S3560[13], 9}, {NULL}};
const Segment SGI361331[] = {{&S3560[33], 15}, {NULL}};
const Segment SGI361332[] = {{&S3560[49], 6}, {NULL}};
const Segment SGI361333[] = {{&S3560[33], 13}, {NULL}};
const Segment SGI361334[] = {{&S4605[13], 43}, {NULL}};
const Segment SGI361335[] = {{&S4605[70], 5}, {NULL}};
const Segment SGI361336[] = {{&S4605[104], 3}, {NULL}};
const Segment SGI361337[] = {{&S3561[69], 6}, {NULL}};
const Segment SGI361338[] = {{&S3561[8], 3}, {NULL}};
const Segment SGI361339[] = {{&S3561[13], 9}, {NULL}};
const Segment SGI361340[] = {{&S3561[33], 15}, {NULL}};
const Segment SGI361341[] = {{&S3561[49], 6}, {NULL}};
const Segment SGI361342[] = {{&S3561[33], 13}, {NULL}};
const Segment SGI361343[] = {{&S4606[13], 43}, {NULL}};
const Segment SGI361344[] = {{&S4606[70], 5}, {NULL}};
const Segment SGI361345[] = {{&S4606[104], 3}, {NULL}};
const Segment SGI361346[] = {{&S3562[121], 7}, {NULL}};
const Segment SGI361347[] = {{&S3562[8], 3}, {NULL}};
const Segment SGI361348[] = {{&S3562[13], 9}, {NULL}};
const Segment SGI361349[] = {{&S3562[33], 15}, {NULL}};
const Segment SGI361350[] = {{&S3562[49], 6}, {NULL}};
const Segment SGI361351[] = {{&S3562[33], 13}, {NULL}};
const Segment SGI361352[] = {{&S4607[13], 43}, {NULL}};
const Segment SGI361353[] = {{&S4607[70], 5}, {NULL}};
const Segment SGI361354[] = {{&S4607[104], 3}, {NULL}};
const Segment SGI361355[] = {{&S3563[66], 6}, {NULL}};
const Segment SGI361356[] = {{&S3563[8], 3}, {NULL}};
const Segment SGI361357[] = {{&S3563[13], 9}, {NULL}};
const Segment SGI361358[] = {{&S3563[33], 15}, {NULL}};
const Segment SGI361359[] = {{&S3563[49], 3}, {NULL}};
const Segment SGI361360[] = {{&S3563[33], 13}, {NULL}};
const Segment SGI361361[] = {{&S4608[13], 43}, {NULL}};
const Segment SGI361362[] = {{&S4608[70], 5}, {NULL}};
const Segment SGI361363[] = {{&S4608[104], 3}, {NULL}};
const Segment SGI361364[] = {{&S4609[13], 43}, {NULL}};
const Segment SGI361365[] = {{&S3564[69], 6}, {NULL}};
const Segment SGI361366[] = {{&S4609[70], 5}, {NULL}};
const Segment SGI361367[] = {{&S3564[8], 3}, {NULL}};
const Segment SGI361368[] = {{&S4609[104], 3}, {NULL}};
const Segment SGI361369[] = {{&S3564[13], 9}, {NULL}};
const Segment SGI361370[] = {{&S3564[33], 15}, {NULL}};
const Segment SGI361371[] = {{&S3564[49], 6}, {NULL}};
const Segment SGI361372[] = {{&S3564[33], 13}, {NULL}};
const Segment SGI361373[] = {{&S3565[69], 6}, {NULL}};
const Segment SGI361374[] = {{&S3565[8], 3}, {NULL}};
const Segment SGI361375[] = {{&S3565[13], 9}, {NULL}};
const Segment SGI361376[] = {{&S3565[33], 15}, {NULL}};
const Segment SGI361377[] = {{&S3565[49], 6}, {NULL}};
const Segment SGI361378[] = {{&S3565[33], 13}, {NULL}};
const Segment SGI361379[] = {{&S3566[69], 6}, {NULL}};
const Segment SGI361380[] = {{&S3566[8], 3}, {NULL}};
const Segment SGI361381[] = {{&S3566[13], 9}, {NULL}};
const Segment SGI361382[] = {{&S3566[33], 15}, {NULL}};
const Segment SGI361383[] = {{&S3566[49], 6}, {NULL}};
const Segment SGI361384[] = {{&S3566[33], 13}, {NULL}};
const Segment SGI361385[] = {{&S3567[66], 6}, {NULL}};
const Segment SGI361386[] = {{&S3567[8], 3}, {NULL}};
const Segment SGI361387[] = {{&S3567[13], 9}, {NULL}};
const Segment SGI361388[] = {{&S3567[33], 15}, {NULL}};
const Segment SGI361389[] = {{&S3567[49], 3}, {NULL}};
const Segment SGI361390[] = {{&S3567[33], 13}, {NULL}};
const Segment SGI361391[] = {{&S3568[69], 6}, {NULL}};
const Segment SGI361392[] = {{&S3568[8], 3}, {NULL}};
const Segment SGI361393[] = {{&S3568[13], 9}, {NULL}};
const Segment SGI361394[] = {{&S3568[33], 15}, {NULL}};
const Segment SGI361395[] = {{&S3568[49], 6}, {NULL}};
const Segment SGI361396[] = {{&S3568[33], 13}, {NULL}};
const Segment SGI361397[] = {{&S3569[69], 6}, {NULL}};
const Segment SGI361398[] = {{&S3569[8], 3}, {NULL}};
const Segment SGI361399[] = {{&S3569[13], 9}, {NULL}};
const Segment SGI361400[] = {{&S3569[33], 15}, {NULL}};
const Segment SGI361401[] = {{&S3569[49], 6}, {NULL}};
const Segment SGI361402[] = {{&S3569[33], 13}, {NULL}};
const Segment SGI361403[] = {{&S3570[69], 6}, {NULL}};
const Segment SGI361404[] = {{&S3570[8], 3}, {NULL}};
const Segment SGI361405[] = {{&S3570[13], 9}, {NULL}};
const Segment SGI361406[] = {{&S3570[33], 15}, {NULL}};
const Segment SGI361407[] = {{&S3570[49], 6}, {NULL}};
const Segment SGI361408[] = {{&S3570[33], 13}, {NULL}};
const Segment SGI361409[] = {{&S3571[69], 6}, {NULL}};
const Segment SGI361410[] = {{&S3571[8], 3}, {NULL}};
const Segment SGI361411[] = {{&S3571[13], 9}, {NULL}};
const Segment SGI361412[] = {{&S3571[33], 15}, {NULL}};
const Segment SGI361413[] = {{&S3571[49], 6}, {NULL}};
const Segment SGI361414[] = {{&S3571[33], 13}, {NULL}};
const Segment SGI361415[] = {{&S3572[69], 6}, {NULL}};
const Segment SGI361416[] = {{&S3572[8], 3}, {NULL}};
const Segment SGI361417[] = {{&S3572[13], 9}, {NULL}};
const Segment SGI361418[] = {{&S3572[33], 15}, {NULL}};
const Segment SGI361419[] = {{&S3572[49], 6}, {NULL}};
const Segment SGI361420[] = {{&S3572[33], 13}, {NULL}};
const Segment SGI361421[] = {{&S3573[69], 6}, {NULL}};
const Segment SGI361422[] = {{&S3573[8], 3}, {NULL}};
const Segment SGI361423[] = {{&S3573[13], 9}, {NULL}};
const Segment SGI361424[] = {{&S3573[33], 15}, {NULL}};
const Segment SGI361425[] = {{&S3573[49], 6}, {NULL}};
const Segment SGI361426[] = {{&S3573[33], 13}, {NULL}};
const Segment SGI361427[] = {{&S3574[69], 6}, {NULL}};
const Segment SGI361428[] = {{&S3574[8], 3}, {NULL}};
const Segment SGI361429[] = {{&S3574[13], 9}, {NULL}};
const Segment SGI361430[] = {{&S3574[33], 15}, {NULL}};
const Segment SGI361431[] = {{&S3574[49], 6}, {NULL}};
const Segment SGI361432[] = {{&S3574[33], 13}, {NULL}};
const Segment SGI361433[] = {{&S4629[13], 43}, {NULL}};
const Segment SGI361434[] = {{&S4629[70], 5}, {NULL}};
const Segment SGI361435[] = {{&S4629[104], 3}, {NULL}};
const Segment SGI361436[] = {{&S4637[13], 43}, {NULL}};
const Segment SGI361437[] = {{&S4637[70], 5}, {NULL}};
const Segment SGI361438[] = {{&S4637[104], 3}, {NULL}};
const Segment SGI361439[] = {{&S4639[13], 43}, {NULL}};
const Segment SGI361440[] = {{&S4639[70], 5}, {NULL}};
const Segment SGI361441[] = {{&S4639[104], 3}, {NULL}};
const Segment SGI361442[] = {{&S4640[13], 43}, {NULL}};
const Segment SGI361443[] = {{&S4640[70], 5}, {NULL}};
const Segment SGI361444[] = {{&S4640[104], 3}, {NULL}};
const Segment SGI361445[] = {{&S4641[13], 43}, {NULL}};
const Segment SGI361446[] = {{&S4641[70], 5}, {NULL}};
const Segment SGI361447[] = {{&S4641[104], 3}, {NULL}};
const Segment SGI361448[] = {{&S4645[13], 43}, {NULL}};
const Segment SGI361449[] = {{&S4645[70], 5}, {NULL}};
const Segment SGI361450[] = {{&S4645[104], 3}, {NULL}};
const Segment SGI361451[] = {{&S4680[13], 43}, {NULL}};
const Segment SGI361452[] = {{&S4680[70], 5}, {NULL}};
const Segment SGI361453[] = {{&S4680[104], 3}, {NULL}};
const Segment SGI361454[] = {{&S4684[13], 43}, {NULL}};
const Segment SGI361455[] = {{&S4684[70], 5}, {NULL}};
const Segment SGI361456[] = {{&S4684[104], 3}, {NULL}};
const Segment SGI361457[] = {{&S4685[13], 43}, {NULL}};
const Segment SGI361458[] = {{&S4685[70], 5}, {NULL}};
const Segment SGI361459[] = {{&S4685[104], 3}, {NULL}};
const Segment SGI361460[] = {{&S4687[13], 43}, {NULL}};
const Segment SGI361461[] = {{&S4687[70], 5}, {NULL}};
const Segment SGI361462[] = {{&S4687[104], 3}, {NULL}};
const Segment SGI361463[] = {{&S4688[13], 43}, {NULL}};
const Segment SGI361464[] = {{&S4688[70], 5}, {NULL}};
const Segment SGI361465[] = {{&S4688[104], 3}, {NULL}};
const Segment SGI361466[] = {{&S3575[69], 6}, {NULL}};
const Segment SGI361467[] = {{&S3575[8], 3}, {NULL}};
const Segment SGI361468[] = {{&S3575[13], 9}, {NULL}};
const Segment SGI361469[] = {{&S3575[33], 15}, {NULL}};
const Segment SGI361470[] = {{&S3575[49], 6}, {NULL}};
const Segment SGI361471[] = {{&S3575[33], 13}, {NULL}};
const Segment SGI361472[] = {{&S3576[69], 6}, {NULL}};
const Segment SGI361473[] = {{&S3576[8], 3}, {NULL}};
const Segment SGI361474[] = {{&S3576[13], 9}, {NULL}};
const Segment SGI361475[] = {{&S3576[33], 15}, {NULL}};
const Segment SGI361476[] = {{&S3576[49], 6}, {NULL}};
const Segment SGI361477[] = {{&S3576[33], 13}, {NULL}};
const Segment SGI361478[] = {{&S3577[69], 6}, {NULL}};
const Segment SGI361479[] = {{&S3577[8], 3}, {NULL}};
const Segment SGI361480[] = {{&S3577[13], 9}, {NULL}};
const Segment SGI361481[] = {{&S3577[33], 15}, {NULL}};
const Segment SGI361482[] = {{&S3577[49], 6}, {NULL}};
const Segment SGI361483[] = {{&S3577[33], 13}, {NULL}};
const Segment SGI361484[] = {{&S3578[69], 6}, {NULL}};
const Segment SGI361485[] = {{&S3578[8], 3}, {NULL}};
const Segment SGI361486[] = {{&S3578[13], 9}, {NULL}};
const Segment SGI361487[] = {{&S3578[33], 15}, {NULL}};
const Segment SGI361488[] = {{&S3578[49], 6}, {NULL}};
const Segment SGI361489[] = {{&S3578[33], 13}, {NULL}};
const Segment SGI361490[] = {{&S3579[69], 6}, {NULL}};
const Segment SGI361491[] = {{&S3579[8], 3}, {NULL}};
const Segment SGI361492[] = {{&S3579[13], 9}, {NULL}};
const Segment SGI361493[] = {{&S3579[33], 15}, {NULL}};
const Segment SGI361494[] = {{&S3579[49], 6}, {NULL}};
const Segment SGI361495[] = {{&S3579[33], 13}, {NULL}};
const Segment SGI361496[] = {{&S3580[69], 6}, {NULL}};
const Segment SGI361497[] = {{&S3580[8], 3}, {NULL}};
const Segment SGI361498[] = {{&S3580[13], 9}, {NULL}};
const Segment SGI361499[] = {{&S3580[33], 15}, {NULL}};
const Segment SGI361500[] = {{&S3580[49], 6}, {NULL}};
const Segment SGI361501[] = {{&S3580[33], 13}, {NULL}};
const Segment SGI361502[] = {{&S3581[69], 6}, {NULL}};
const Segment SGI361503[] = {{&S3581[8], 3}, {NULL}};
const Segment SGI361504[] = {{&S3581[13], 9}, {NULL}};
const Segment SGI361505[] = {{&S3581[33], 15}, {NULL}};
const Segment SGI361506[] = {{&S3581[49], 6}, {NULL}};
const Segment SGI361507[] = {{&S3581[33], 13}, {NULL}};
const Segment SGI361508[] = {{&S3582[69], 6}, {NULL}};
const Segment SGI361509[] = {{&S3582[8], 3}, {NULL}};
const Segment SGI361510[] = {{&S3582[13], 9}, {NULL}};
const Segment SGI361511[] = {{&S3582[33], 15}, {NULL}};
const Segment SGI361512[] = {{&S3582[49], 6}, {NULL}};
const Segment SGI361513[] = {{&S3582[33], 13}, {NULL}};
const Segment SGI361514[] = {{&S3583[69], 6}, {NULL}};
const Segment SGI361515[] = {{&S3583[8], 3}, {NULL}};
const Segment SGI361516[] = {{&S3583[13], 9}, {NULL}};
const Segment SGI361517[] = {{&S3583[33], 15}, {NULL}};
const Segment SGI361518[] = {{&S3583[49], 6}, {NULL}};
const Segment SGI361519[] = {{&S3583[33], 13}, {NULL}};
const Segment SGI361520[] = {{&S3584[66], 6}, {NULL}};
const Segment SGI361521[] = {{&S3584[8], 3}, {NULL}};
const Segment SGI361522[] = {{&S3584[13], 9}, {NULL}};
const Segment SGI361523[] = {{&S3584[33], 15}, {NULL}};
const Segment SGI361524[] = {{&S3584[49], 3}, {NULL}};
const Segment SGI361525[] = {{&S3584[33], 13}, {NULL}};
const Segment SGI361526[] = {{&S3585[69], 6}, {NULL}};
const Segment SGI361527[] = {{&S3585[8], 3}, {NULL}};
const Segment SGI361528[] = {{&S3585[13], 9}, {NULL}};
const Segment SGI361529[] = {{&S3585[33], 15}, {NULL}};
const Segment SGI361530[] = {{&S3585[49], 6}, {NULL}};
const Segment SGI361531[] = {{&S3585[33], 13}, {NULL}};
const Segment SGI361532[] = {{&S3586[69], 6}, {NULL}};
const Segment SGI361533[] = {{&S3586[8], 3}, {NULL}};
const Segment SGI361534[] = {{&S3586[13], 9}, {NULL}};
const Segment SGI361535[] = {{&S3586[33], 15}, {NULL}};
const Segment SGI361536[] = {{&S3586[49], 6}, {NULL}};
const Segment SGI361537[] = {{&S3586[33], 13}, {NULL}};
const Segment SGI361538[] = {{&S3587[69], 6}, {NULL}};
const Segment SGI361539[] = {{&S3587[8], 3}, {NULL}};
const Segment SGI361540[] = {{&S3587[13], 9}, {NULL}};
const Segment SGI361541[] = {{&S3587[33], 15}, {NULL}};
const Segment SGI361542[] = {{&S3587[49], 6}, {NULL}};
const Segment SGI361543[] = {{&S3587[33], 13}, {NULL}};
const Segment SGI361544[] = {{&S3588[66], 6}, {NULL}};
const Segment SGI361545[] = {{&S3588[8], 3}, {NULL}};
const Segment SGI361546[] = {{&S3588[13], 9}, {NULL}};
const Segment SGI361547[] = {{&S3588[33], 15}, {NULL}};
const Segment SGI361548[] = {{&S3588[49], 3}, {NULL}};
const Segment SGI361549[] = {{&S3588[33], 13}, {NULL}};
const Segment SGI361550[] = {{&S3589[69], 6}, {NULL}};
const Segment SGI361551[] = {{&S3589[8], 3}, {NULL}};
const Segment SGI361552[] = {{&S3589[13], 9}, {NULL}};
const Segment SGI361553[] = {{&S3589[33], 15}, {NULL}};
const Segment SGI361554[] = {{&S3589[49], 6}, {NULL}};
const Segment SGI361555[] = {{&S3589[33], 13}, {NULL}};
const Segment SGI361556[] = {{&S4726[13], 66}, {NULL}};
const Segment SGI361557[] = {{&S4726[93], 3}, {NULL}};
const Segment SGI361558[] = {{&S3590[69], 6}, {NULL}};
const Segment SGI361559[] = {{&S3590[8], 3}, {NULL}};
const Segment SGI361560[] = {{&S3590[13], 9}, {NULL}};
const Segment SGI361561[] = {{&S3590[33], 15}, {NULL}};
const Segment SGI361562[] = {{&S3590[49], 6}, {NULL}};
const Segment SGI361563[] = {{&S3590[33], 13}, {NULL}};
const Segment SGI361564[] = {{&S3591[69], 6}, {NULL}};
const Segment SGI361565[] = {{&S3591[8], 3}, {NULL}};
const Segment SGI361566[] = {{&S3591[13], 9}, {NULL}};
const Segment SGI361567[] = {{&S3591[33], 15}, {NULL}};
const Segment SGI361568[] = {{&S3591[49], 6}, {NULL}};
const Segment SGI361569[] = {{&S3591[33], 13}, {NULL}};
const Segment SGI361570[] = {{&S3592[69], 6}, {NULL}};
const Segment SGI361571[] = {{&S3592[8], 3}, {NULL}};
const Segment SGI361572[] = {{&S3592[13], 9}, {NULL}};
const Segment SGI361573[] = {{&S3592[33], 15}, {NULL}};
const Segment SGI361574[] = {{&S3592[49], 6}, {NULL}};
const Segment SGI361575[] = {{&S3592[33], 13}, {NULL}};
const Segment SGI361576[] = {{&S3593[69], 6}, {NULL}};
const Segment SGI361577[] = {{&S3593[8], 3}, {NULL}};
const Segment SGI361578[] = {{&S3593[13], 9}, {NULL}};
const Segment SGI361579[] = {{&S3593[33], 15}, {NULL}};
const Segment SGI361580[] = {{&S3593[49], 6}, {NULL}};
const Segment SGI361581[] = {{&S3593[33], 13}, {NULL}};
const Segment SGI361582[] = {{&S3594[69], 6}, {NULL}};
const Segment SGI361583[] = {{&S3594[8], 3}, {NULL}};
const Segment SGI361584[] = {{&S3594[13], 9}, {NULL}};
const Segment SGI361585[] = {{&S3594[33], 15}, {NULL}};
const Segment SGI361586[] = {{&S3594[49], 6}, {NULL}};
const Segment SGI361587[] = {{&S3594[33], 13}, {NULL}};
const Segment SGI361588[] = {{&S3595[69], 6}, {NULL}};
const Segment SGI361589[] = {{&S3595[8], 3}, {NULL}};
const Segment SGI361590[] = {{&S3595[13], 9}, {NULL}};
const Segment SGI361591[] = {{&S3595[33], 15}, {NULL}};
const Segment SGI361592[] = {{&S3595[49], 6}, {NULL}};
const Segment SGI361593[] = {{&S3595[33], 13}, {NULL}};
const Segment SGI361594[] = {{&S3596[69], 6}, {NULL}};
const Segment SGI361595[] = {{&S3596[8], 3}, {NULL}};
const Segment SGI361596[] = {{&S3596[13], 9}, {NULL}};
const Segment SGI361597[] = {{&S3596[33], 15}, {NULL}};
const Segment SGI361598[] = {{&S3596[49], 6}, {NULL}};
const Segment SGI361599[] = {{&S3596[33], 13}, {NULL}};
const Segment SGI361600[] = {{&S3597[119], 6}, {NULL}};
const Segment SGI361601[] = {{&S3597[8], 3}, {NULL}};
const Segment SGI361602[] = {{&S3597[13], 9}, {NULL}};
const Segment SGI361603[] = {{&S3597[33], 15}, {NULL}};
const Segment SGI361604[] = {{&S3597[49], 6}, {NULL}};
const Segment SGI361605[] = {{&S3597[33], 13}, {NULL}};
const Segment SGI361606[] = {{&S3598[69], 6}, {NULL}};
const Segment SGI361607[] = {{&S3598[8], 3}, {NULL}};
const Segment SGI361608[] = {{&S3598[13], 9}, {NULL}};
const Segment SGI361609[] = {{&S3598[33], 15}, {NULL}};
const Segment SGI361610[] = {{&S3598[49], 6}, {NULL}};
const Segment SGI361611[] = {{&S3598[33], 13}, {NULL}};
const Segment SGI361612[] = {{&S3599[69], 6}, {NULL}};
const Segment SGI361613[] = {{&S3599[8], 3}, {NULL}};
const Segment SGI361614[] = {{&S3599[13], 9}, {NULL}};
const Segment SGI361615[] = {{&S3599[33], 15}, {NULL}};
const Segment SGI361616[] = {{&S3599[49], 6}, {NULL}};
const Segment SGI361617[] = {{&S3599[33], 13}, {NULL}};
const Segment SGI361618[] = {{&S3600[66], 6}, {NULL}};
const Segment SGI361619[] = {{&S3600[8], 3}, {NULL}};
const Segment SGI361620[] = {{&S3600[13], 9}, {NULL}};
const Segment SGI361621[] = {{&S3600[33], 15}, {NULL}};
const Segment SGI361622[] = {{&S3600[49], 3}, {NULL}};
const Segment SGI361623[] = {{&S3600[33], 13}, {NULL}};
const Segment SGI361624[] = {{&S3601[69], 6}, {NULL}};
const Segment SGI361625[] = {{&S3601[8], 3}, {NULL}};
const Segment SGI361626[] = {{&S3601[13], 9}, {NULL}};
const Segment SGI361627[] = {{&S3601[33], 15}, {NULL}};
const Segment SGI361628[] = {{&S3601[49], 6}, {NULL}};
const Segment SGI361629[] = {{&S3601[33], 13}, {NULL}};
const Segment SGI361630[] = {{&S3602[69], 6}, {NULL}};
const Segment SGI361631[] = {{&S3602[8], 3}, {NULL}};
const Segment SGI361632[] = {{&S3602[13], 9}, {NULL}};
const Segment SGI361633[] = {{&S3602[33], 15}, {NULL}};
const Segment SGI361634[] = {{&S3602[49], 6}, {NULL}};
const Segment SGI361635[] = {{&S3602[33], 13}, {NULL}};
const Segment SGI361636[] = {{&S3603[69], 6}, {NULL}};
const Segment SGI361637[] = {{&S3603[8], 3}, {NULL}};
const Segment SGI361638[] = {{&S3603[13], 9}, {NULL}};
const Segment SGI361639[] = {{&S3603[33], 15}, {NULL}};
const Segment SGI361640[] = {{&S3603[49], 6}, {NULL}};
const Segment SGI361641[] = {{&S3603[33], 13}, {NULL}};
const Segment SGI361642[] = {{&S3604[69], 6}, {NULL}};
const Segment SGI361643[] = {{&S3604[8], 3}, {NULL}};
const Segment SGI361644[] = {{&S3604[13], 9}, {NULL}};
const Segment SGI361645[] = {{&S3604[33], 15}, {NULL}};
const Segment SGI361646[] = {{&S3604[49], 6}, {NULL}};
const Segment SGI361647[] = {{&S3604[33], 13}, {NULL}};
const Segment SGI361648[] = {{&S3605[69], 6}, {NULL}};
const Segment SGI361649[] = {{&S3605[8], 3}, {NULL}};
const Segment SGI361650[] = {{&S3605[13], 9}, {NULL}};
const Segment SGI361651[] = {{&S3605[33], 15}, {NULL}};
const Segment SGI361652[] = {{&S3605[49], 6}, {NULL}};
const Segment SGI361653[] = {{&S3605[33], 13}, {NULL}};
const Segment SGI361654[] = {{&S3606[69], 6}, {NULL}};
const Segment SGI361655[] = {{&S3606[8], 3}, {NULL}};
const Segment SGI361656[] = {{&S3606[13], 9}, {NULL}};
const Segment SGI361657[] = {{&S3606[33], 15}, {NULL}};
const Segment SGI361658[] = {{&S3606[49], 6}, {NULL}};
const Segment SGI361659[] = {{&S3606[33], 13}, {NULL}};
const Segment SGI361660[] = {{&S3607[69], 6}, {NULL}};
const Segment SGI361661[] = {{&S3607[8], 3}, {NULL}};
const Segment SGI361662[] = {{&S3607[13], 9}, {NULL}};
const Segment SGI361663[] = {{&S3607[33], 15}, {NULL}};
const Segment SGI361664[] = {{&S3607[49], 6}, {NULL}};
const Segment SGI361665[] = {{&S3607[33], 13}, {NULL}};
const Segment SGI361666[] = {{&S3608[69], 6}, {NULL}};
const Segment SGI361667[] = {{&S3608[8], 3}, {NULL}};
const Segment SGI361668[] = {{&S3608[13], 9}, {NULL}};
const Segment SGI361669[] = {{&S3608[33], 15}, {NULL}};
const Segment SGI361670[] = {{&S3608[49], 6}, {NULL}};
const Segment SGI361671[] = {{&S3608[33], 13}, {NULL}};
const Segment SGI361672[] = {{&S3609[69], 6}, {NULL}};
const Segment SGI361673[] = {{&S3609[8], 3}, {NULL}};
const Segment SGI361674[] = {{&S3609[13], 9}, {NULL}};
const Segment SGI361675[] = {{&S3609[33], 15}, {NULL}};
const Segment SGI361676[] = {{&S3609[49], 6}, {NULL}};
const Segment SGI361677[] = {{&S3609[33], 13}, {NULL}};
const Segment SGI361678[] = {{&S3610[69], 6}, {NULL}};
const Segment SGI361679[] = {{&S3610[8], 3}, {NULL}};
const Segment SGI361680[] = {{&S3610[13], 9}, {NULL}};
const Segment SGI361681[] = {{&S3610[33], 15}, {NULL}};
const Segment SGI361682[] = {{&S3610[49], 6}, {NULL}};
const Segment SGI361683[] = {{&S3610[33], 13}, {NULL}};
const Segment SGI361684[] = {{&S3611[69], 6}, {NULL}};
const Segment SGI361685[] = {{&S3611[8], 3}, {NULL}};
const Segment SGI361686[] = {{&S3611[13], 9}, {NULL}};
const Segment SGI361687[] = {{&S3611[33], 15}, {NULL}};
const Segment SGI361688[] = {{&S3611[49], 6}, {NULL}};
const Segment SGI361689[] = {{&S3611[33], 13}, {NULL}};
const Segment SGI361690[] = {{&S3612[69], 6}, {NULL}};
const Segment SGI361691[] = {{&S3612[8], 3}, {NULL}};
const Segment SGI361692[] = {{&S3612[13], 9}, {NULL}};
const Segment SGI361693[] = {{&S3612[33], 15}, {NULL}};
const Segment SGI361694[] = {{&S3612[49], 6}, {NULL}};
const Segment SGI361695[] = {{&S3612[33], 13}, {NULL}};
const Segment SGI361696[] = {{&S3613[69], 6}, {NULL}};
const Segment SGI361697[] = {{&S3613[8], 3}, {NULL}};
const Segment SGI361698[] = {{&S3613[13], 9}, {NULL}};
const Segment SGI361699[] = {{&S3613[33], 15}, {NULL}};
const Segment SGI361700[] = {{&S3613[49], 6}, {NULL}};
const Segment SGI361701[] = {{&S3613[33], 13}, {NULL}};
const Segment SGI361702[] = {{&S3614[69], 6}, {NULL}};
const Segment SGI361703[] = {{&S3614[8], 3}, {NULL}};
const Segment SGI361704[] = {{&S3614[13], 9}, {NULL}};
const Segment SGI361705[] = {{&S3614[33], 15}, {NULL}};
const Segment SGI361706[] = {{&S3614[49], 6}, {NULL}};
const Segment SGI361707[] = {{&S3614[33], 13}, {NULL}};
const Segment SGI361708[] = {{&S3615[69], 6}, {NULL}};
const Segment SGI361709[] = {{&S3615[8], 3}, {NULL}};
const Segment SGI361710[] = {{&S3615[13], 9}, {NULL}};
const Segment SGI361711[] = {{&S3615[33], 15}, {NULL}};
const Segment SGI361712[] = {{&S3615[49], 6}, {NULL}};
const Segment SGI361713[] = {{&S3615[33], 13}, {NULL}};
const Segment SGI361714[] = {{&S3616[122], 7}, {NULL}};
const Segment SGI361715[] = {{&S3616[8], 3}, {NULL}};
const Segment SGI361716[] = {{&S3616[13], 9}, {NULL}};
const Segment SGI361717[] = {{&S3616[33], 15}, {NULL}};
const Segment SGI361718[] = {{&S3616[49], 6}, {NULL}};
const Segment SGI361719[] = {{&S3616[33], 13}, {NULL}};
const Segment SGI361720[] = {{&S3617[69], 6}, {NULL}};
const Segment SGI361721[] = {{&S3617[8], 3}, {NULL}};
const Segment SGI361722[] = {{&S3617[13], 9}, {NULL}};
const Segment SGI361723[] = {{&S3617[33], 15}, {NULL}};
const Segment SGI361724[] = {{&S3617[49], 6}, {NULL}};
const Segment SGI361725[] = {{&S3617[33], 13}, {NULL}};
const Segment SGI361726[] = {{&S3618[69], 6}, {NULL}};
const Segment SGI361727[] = {{&S3618[8], 3}, {NULL}};
const Segment SGI361728[] = {{&S3618[13], 9}, {NULL}};
const Segment SGI361729[] = {{&S3618[33], 15}, {NULL}};
const Segment SGI361730[] = {{&S3618[49], 6}, {NULL}};
const Segment SGI361731[] = {{&S3618[33], 13}, {NULL}};
const Segment SGI361732[] = {{&S3619[66], 6}, {NULL}};
const Segment SGI361733[] = {{&S3619[8], 3}, {NULL}};
const Segment SGI361734[] = {{&S3619[13], 9}, {NULL}};
const Segment SGI361735[] = {{&S3619[33], 15}, {NULL}};
const Segment SGI361736[] = {{&S3619[49], 3}, {NULL}};
const Segment SGI361737[] = {{&S3619[33], 13}, {NULL}};
const Segment SGI361738[] = {{&S3620[69], 6}, {NULL}};
const Segment SGI361739[] = {{&S3620[8], 3}, {NULL}};
const Segment SGI361740[] = {{&S3620[13], 9}, {NULL}};
const Segment SGI361741[] = {{&S3620[33], 15}, {NULL}};
const Segment SGI361742[] = {{&S3620[49], 6}, {NULL}};
const Segment SGI361743[] = {{&S3620[33], 13}, {NULL}};
const Segment SGI361744[] = {{&S3621[69], 6}, {NULL}};
const Segment SGI361745[] = {{&S3621[8], 3}, {NULL}};
const Segment SGI361746[] = {{&S3621[13], 9}, {NULL}};
const Segment SGI361747[] = {{&S3621[33], 15}, {NULL}};
const Segment SGI361748[] = {{&S3621[49], 6}, {NULL}};
const Segment SGI361749[] = {{&S3621[33], 13}, {NULL}};
const Segment SGI361750[] = {{&S3622[69], 6}, {NULL}};
const Segment SGI361751[] = {{&S3622[8], 3}, {NULL}};
const Segment SGI361752[] = {{&S3622[13], 9}, {NULL}};
const Segment SGI361753[] = {{&S3622[33], 15}, {NULL}};
const Segment SGI361754[] = {{&S3622[49], 6}, {NULL}};
const Segment SGI361755[] = {{&S3622[33], 13}, {NULL}};
const Segment SGI361756[] = {{&S3623[69], 6}, {NULL}};
const Segment SGI361757[] = {{&S3623[8], 3}, {NULL}};
const Segment SGI361758[] = {{&S3623[13], 9}, {NULL}};
const Segment SGI361759[] = {{&S3623[33], 15}, {NULL}};
const Segment SGI361760[] = {{&S3623[49], 6}, {NULL}};
const Segment SGI361761[] = {{&S3623[33], 13}, {NULL}};
const Segment SGI361762[] = {{&S3624[66], 6}, {NULL}};
const Segment SGI361763[] = {{&S3624[8], 3}, {NULL}};
const Segment SGI361764[] = {{&S3624[13], 9}, {NULL}};
const Segment SGI361765[] = {{&S3624[33], 15}, {NULL}};
const Segment SGI361766[] = {{&S3624[49], 3}, {NULL}};
const Segment SGI361767[] = {{&S3624[33], 13}, {NULL}};
const Segment SGI361768[] = {{&S3625[69], 6}, {NULL}};
const Segment SGI361769[] = {{&S3625[8], 3}, {NULL}};
const Segment SGI361770[] = {{&S3625[13], 9}, {NULL}};
const Segment SGI361771[] = {{&S3625[33], 15}, {NULL}};
const Segment SGI361772[] = {{&S3625[49], 6}, {NULL}};
const Segment SGI361773[] = {{&S3625[33], 13}, {NULL}};
const Segment SGI361774[] = {{&S3626[69], 6}, {NULL}};
const Segment SGI361775[] = {{&S3626[8], 3}, {NULL}};
const Segment SGI361776[] = {{&S3626[13], 9}, {NULL}};
const Segment SGI361777[] = {{&S3626[33], 15}, {NULL}};
const Segment SGI361778[] = {{&S3626[49], 6}, {NULL}};
const Segment SGI361779[] = {{&S3626[33], 13}, {NULL}};
const Segment SGI361780[] = {{&S3627[69], 6}, {NULL}};
const Segment SGI361781[] = {{&S3627[8], 3}, {NULL}};
const Segment SGI361782[] = {{&S3627[13], 9}, {NULL}};
const Segment SGI361783[] = {{&S3627[33], 15}, {NULL}};
const Segment SGI361784[] = {{&S3627[49], 6}, {NULL}};
const Segment SGI361785[] = {{&S3627[33], 13}, {NULL}};
const Segment SGI361786[] = {{&S3628[69], 6}, {NULL}};
const Segment SGI361787[] = {{&S3628[8], 3}, {NULL}};
const Segment SGI361788[] = {{&S3628[13], 9}, {NULL}};
const Segment SGI361789[] = {{&S3628[33], 15}, {NULL}};
const Segment SGI361790[] = {{&S3628[49], 6}, {NULL}};
const Segment SGI361791[] = {{&S3628[33], 13}, {NULL}};
const Segment SGI361792[] = {{&S3629[69], 6}, {NULL}};
const Segment SGI361793[] = {{&S3629[8], 3}, {NULL}};
const Segment SGI361794[] = {{&S3629[13], 9}, {NULL}};
const Segment SGI361795[] = {{&S3629[33], 15}, {NULL}};
const Segment SGI361796[] = {{&S3629[49], 6}, {NULL}};
const Segment SGI361797[] = {{&S3629[33], 13}, {NULL}};
const Segment SGI361798[] = {{&S3630[69], 6}, {NULL}};
const Segment SGI361799[] = {{&S3630[8], 3}, {NULL}};
const Segment SGI361800[] = {{&S3630[13], 9}, {NULL}};
const Segment SGI361801[] = {{&S3630[33], 15}, {NULL}};
const Segment SGI361802[] = {{&S3630[49], 6}, {NULL}};
const Segment SGI361803[] = {{&S3630[33], 13}, {NULL}};
const Segment SGI361804[] = {{&S3631[69], 6}, {NULL}};
const Segment SGI361805[] = {{&S3631[8], 3}, {NULL}};
const Segment SGI361806[] = {{&S3631[13], 9}, {NULL}};
const Segment SGI361807[] = {{&S3631[33], 15}, {NULL}};
const Segment SGI361808[] = {{&S3631[49], 6}, {NULL}};
const Segment SGI361809[] = {{&S3631[33], 13}, {NULL}};
const Segment SGI361810[] = {{&S3632[69], 6}, {NULL}};
const Segment SGI361811[] = {{&S3632[8], 3}, {NULL}};
const Segment SGI361812[] = {{&S3632[13], 9}, {NULL}};
const Segment SGI361813[] = {{&S3632[33], 15}, {NULL}};
const Segment SGI361814[] = {{&S3632[49], 6}, {NULL}};
const Segment SGI361815[] = {{&S3632[33], 13}, {NULL}};
const Segment SGI361816[] = {{&S3633[69], 6}, {NULL}};
const Segment SGI361817[] = {{&S3633[8], 3}, {NULL}};
const Segment SGI361818[] = {{&S3633[13], 9}, {NULL}};
const Segment SGI361819[] = {{&S3633[33], 15}, {NULL}};
const Segment SGI361820[] = {{&S3633[49], 6}, {NULL}};
const Segment SGI361821[] = {{&S3633[33], 13}, {NULL}};
const Segment SGI361822[] = {{&S3634[69], 6}, {NULL}};
const Segment SGI361823[] = {{&S3634[8], 3}, {NULL}};
const Segment SGI361824[] = {{&S3634[13], 9}, {NULL}};
const Segment SGI361825[] = {{&S3634[33], 15}, {NULL}};
const Segment SGI361826[] = {{&S3634[49], 6}, {NULL}};
const Segment SGI361827[] = {{&S3634[33], 13}, {NULL}};
const Segment SGI361828[] = {{&S3635[117], 6}, {NULL}};
const Segment SGI361829[] = {{&S3635[8], 3}, {NULL}};
const Segment SGI361830[] = {{&S3635[13], 9}, {NULL}};
const Segment SGI361831[] = {{&S3635[33], 15}, {NULL}};
const Segment SGI361832[] = {{&S3635[49], 6}, {NULL}};
const Segment SGI361833[] = {{&S3635[33], 13}, {NULL}};
const Segment SGI361834[] = {{&S3636[69], 6}, {NULL}};
const Segment SGI361835[] = {{&S3636[8], 3}, {NULL}};
const Segment SGI361836[] = {{&S3636[13], 9}, {NULL}};
const Segment SGI361837[] = {{&S3636[33], 15}, {NULL}};
const Segment SGI361838[] = {{&S3636[49], 6}, {NULL}};
const Segment SGI361839[] = {{&S3636[33], 13}, {NULL}};
const Segment SGI361840[] = {{&S3637[69], 6}, {NULL}};
const Segment SGI361841[] = {{&S3637[8], 3}, {NULL}};
const Segment SGI361842[] = {{&S3637[13], 9}, {NULL}};
const Segment SGI361843[] = {{&S3637[33], 15}, {NULL}};
const Segment SGI361844[] = {{&S3637[49], 6}, {NULL}};
const Segment SGI361845[] = {{&S3637[33], 13}, {NULL}};
const Segment SGI361846[] = {{&S3638[69], 6}, {NULL}};
const Segment SGI361847[] = {{&S3638[8], 3}, {NULL}};
const Segment SGI361848[] = {{&S3638[13], 9}, {NULL}};
const Segment SGI361849[] = {{&S3638[33], 15}, {NULL}};
const Segment SGI361850[] = {{&S3638[49], 6}, {NULL}};
const Segment SGI361851[] = {{&S3638[33], 13}, {NULL}};
const Segment SGI361852[] = {{&S3639[69], 6}, {NULL}};
const Segment SGI361853[] = {{&S3639[8], 3}, {NULL}};
const Segment SGI361854[] = {{&S3639[13], 9}, {NULL}};
const Segment SGI361855[] = {{&S3639[33], 15}, {NULL}};
const Segment SGI361856[] = {{&S3639[49], 6}, {NULL}};
const Segment SGI361857[] = {{&S3639[33], 13}, {NULL}};
const Segment SGI361858[] = {{&S3640[66], 6}, {NULL}};
const Segment SGI361859[] = {{&S3640[8], 3}, {NULL}};
const Segment SGI361860[] = {{&S3640[13], 9}, {NULL}};
const Segment SGI361861[] = {{&S3640[33], 15}, {NULL}};
const Segment SGI361862[] = {{&S3640[49], 3}, {NULL}};
const Segment SGI361863[] = {{&S3640[33], 13}, {NULL}};
const Segment SGI361864[] = {{&S3641[69], 6}, {NULL}};
const Segment SGI361865[] = {{&S3641[8], 3}, {NULL}};
const Segment SGI361866[] = {{&S3641[13], 9}, {NULL}};
const Segment SGI361867[] = {{&S3641[33], 15}, {NULL}};
const Segment SGI361868[] = {{&S3641[49], 6}, {NULL}};
const Segment SGI361869[] = {{&S3641[33], 13}, {NULL}};
const Segment SGI361870[] = {{&S3642[69], 6}, {NULL}};
const Segment SGI361871[] = {{&S3642[8], 3}, {NULL}};
const Segment SGI361872[] = {{&S3642[13], 9}, {NULL}};
const Segment SGI361873[] = {{&S3642[33], 15}, {NULL}};
const Segment SGI361874[] = {{&S3642[49], 6}, {NULL}};
const Segment SGI361875[] = {{&S3642[33], 13}, {NULL}};
const Segment SGI361876[] = {{&S3643[69], 6}, {NULL}};
const Segment SGI361877[] = {{&S3643[8], 3}, {NULL}};
const Segment SGI361878[] = {{&S3643[13], 9}, {NULL}};
const Segment SGI361879[] = {{&S3643[33], 15}, {NULL}};
const Segment SGI361880[] = {{&S3643[49], 6}, {NULL}};
const Segment SGI361881[] = {{&S3643[33], 13}, {NULL}};
const Segment SGI361882[] = {{&S3644[66], 6}, {NULL}};
const Segment SGI361883[] = {{&S3644[8], 3}, {NULL}};
const Segment SGI361884[] = {{&S3644[13], 9}, {NULL}};
const Segment SGI361885[] = {{&S3644[33], 15}, {NULL}};
const Segment SGI361886[] = {{&S3644[49], 3}, {NULL}};
const Segment SGI361887[] = {{&S3644[33], 13}, {NULL}};
const Segment SGI361888[] = {{&S3645[69], 6}, {NULL}};
const Segment SGI361889[] = {{&S3645[8], 3}, {NULL}};
const Segment SGI361890[] = {{&S3645[13], 9}, {NULL}};
const Segment SGI361891[] = {{&S3645[33], 15}, {NULL}};
const Segment SGI361892[] = {{&S3645[49], 6}, {NULL}};
const Segment SGI361893[] = {{&S3645[33], 13}, {NULL}};
const Segment SGI361894[] = {{&S3646[69], 6}, {NULL}};
const Segment SGI361895[] = {{&S3646[8], 3}, {NULL}};
const Segment SGI361896[] = {{&S3646[13], 9}, {NULL}};
const Segment SGI361897[] = {{&S3646[33], 15}, {NULL}};
const Segment SGI361898[] = {{&S3646[49], 6}, {NULL}};
const Segment SGI361899[] = {{&S3646[33], 13}, {NULL}};
const Segment SGI361900[] = {{&S3647[69], 6}, {NULL}};
const Segment SGI361901[] = {{&S3647[8], 3}, {NULL}};
const Segment SGI361902[] = {{&S3647[13], 9}, {NULL}};
const Segment SGI361903[] = {{&S3647[33], 15}, {NULL}};
const Segment SGI361904[] = {{&S3647[49], 6}, {NULL}};
const Segment SGI361905[] = {{&S3647[33], 13}, {NULL}};
const Segment SGI361906[] = {{&S3648[69], 6}, {NULL}};
const Segment SGI361907[] = {{&S3648[8], 3}, {NULL}};
const Segment SGI361908[] = {{&S3648[13], 9}, {NULL}};
const Segment SGI361909[] = {{&S3648[33], 15}, {NULL}};
const Segment SGI361910[] = {{&S3648[49], 6}, {NULL}};
const Segment SGI361911[] = {{&S3648[33], 13}, {NULL}};
const Segment SGI361912[] = {{&S3649[119], 6}, {NULL}};
const Segment SGI361913[] = {{&S3649[8], 3}, {NULL}};
const Segment SGI361914[] = {{&S3649[13], 9}, {NULL}};
const Segment SGI361915[] = {{&S3649[33], 15}, {NULL}};
const Segment SGI361916[] = {{&S3649[49], 6}, {NULL}};
const Segment SGI361917[] = {{&S3649[33], 13}, {NULL}};
const Segment SGI361918[] = {{&S3650[69], 6}, {NULL}};
const Segment SGI361919[] = {{&S3650[8], 3}, {NULL}};
const Segment SGI361920[] = {{&S3650[13], 9}, {NULL}};
const Segment SGI361921[] = {{&S3650[33], 15}, {NULL}};
const Segment SGI361922[] = {{&S3650[49], 6}, {NULL}};
const Segment SGI361923[] = {{&S3650[33], 13}, {NULL}};
const Segment SGI361924[] = {{&S3651[69], 6}, {NULL}};
const Segment SGI361925[] = {{&S3651[8], 3}, {NULL}};
const Segment SGI361926[] = {{&S3651[13], 9}, {NULL}};
const Segment SGI361927[] = {{&S3651[33], 15}, {NULL}};
const Segment SGI361928[] = {{&S3651[49], 6}, {NULL}};
const Segment SGI361929[] = {{&S3651[33], 13}, {NULL}};
const Segment SGI361930[] = {{&S3652[119], 6}, {NULL}};
const Segment SGI361931[] = {{&S3652[8], 3}, {NULL}};
const Segment SGI361932[] = {{&S3652[13], 9}, {NULL}};
const Segment SGI361933[] = {{&S3652[33], 15}, {NULL}};
const Segment SGI361934[] = {{&S3652[49], 6}, {NULL}};
const Segment SGI361935[] = {{&S3652[33], 13}, {NULL}};
const Segment SGI361936[] = {{&S3653[119], 6}, {NULL}};
const Segment SGI361937[] = {{&S3653[8], 3}, {NULL}};
const Segment SGI361938[] = {{&S3653[13], 9}, {NULL}};
const Segment SGI361939[] = {{&S3653[33], 15}, {NULL}};
const Segment SGI361940[] = {{&S3653[49], 6}, {NULL}};
const Segment SGI361941[] = {{&S3653[33], 13}, {NULL}};
const Segment SGI361942[] = {{&S3654[66], 6}, {NULL}};
const Segment SGI361943[] = {{&S3654[8], 3}, {NULL}};
const Segment SGI361944[] = {{&S3654[13], 9}, {NULL}};
const Segment SGI361945[] = {{&S3654[33], 15}, {NULL}};
const Segment SGI361946[] = {{&S3654[49], 3}, {NULL}};
const Segment SGI361947[] = {{&S3654[33], 13}, {NULL}};
const Segment SGI361948[] = {{&S3655[121], 7}, {NULL}};
const Segment SGI361949[] = {{&S3655[8], 3}, {NULL}};
const Segment SGI361950[] = {{&S3655[13], 9}, {NULL}};
const Segment SGI361951[] = {{&S3655[33], 15}, {NULL}};
const Segment SGI361952[] = {{&S3655[49], 6}, {NULL}};
const Segment SGI361953[] = {{&S3655[33], 13}, {NULL}};
const Segment SGI361954[] = {{&S3656[69], 6}, {NULL}};
const Segment SGI361955[] = {{&S3656[8], 3}, {NULL}};
const Segment SGI361956[] = {{&S3656[13], 9}, {NULL}};
const Segment SGI361957[] = {{&S3656[33], 15}, {NULL}};
const Segment SGI361958[] = {{&S3656[49], 6}, {NULL}};
const Segment SGI361959[] = {{&S3656[33], 13}, {NULL}};
const Segment SGI361960[] = {{&S3657[69], 6}, {NULL}};
const Segment SGI361961[] = {{&S3657[8], 3}, {NULL}};
const Segment SGI361962[] = {{&S3657[13], 9}, {NULL}};
const Segment SGI361963[] = {{&S3657[33], 15}, {NULL}};
const Segment SGI361964[] = {{&S3657[49], 6}, {NULL}};
const Segment SGI361965[] = {{&S3657[33], 13}, {NULL}};
const Segment SGI361966[] = {{&S3658[69], 6}, {NULL}};
const Segment SGI361967[] = {{&S3658[8], 3}, {NULL}};
const Segment SGI361968[] = {{&S3658[13], 9}, {NULL}};
const Segment SGI361969[] = {{&S3658[33], 15}, {NULL}};
const Segment SGI361970[] = {{&S3658[49], 6}, {NULL}};
const Segment SGI361971[] = {{&S3658[33], 13}, {NULL}};
const Segment SGI361972[] = {{&S3659[69], 6}, {NULL}};
const Segment SGI361973[] = {{&S3659[8], 3}, {NULL}};
const Segment SGI361974[] = {{&S3659[13], 9}, {NULL}};
const Segment SGI361975[] = {{&S3659[33], 15}, {NULL}};
const Segment SGI361976[] = {{&S3659[49], 6}, {NULL}};
const Segment SGI361977[] = {{&S3659[33], 13}, {NULL}};
const Segment SGI361978[] = {{&S3660[69], 6}, {NULL}};
const Segment SGI361979[] = {{&S3660[8], 3}, {NULL}};
const Segment SGI361980[] = {{&S3660[13], 9}, {NULL}};
const Segment SGI361981[] = {{&S3660[33], 15}, {NULL}};
const Segment SGI361982[] = {{&S3660[49], 6}, {NULL}};
const Segment SGI361983[] = {{&S3660[33], 13}, {NULL}};
const Segment SGI361984[] = {{&S3661[69], 6}, {NULL}};
const Segment SGI361985[] = {{&S3661[8], 3}, {NULL}};
const Segment SGI361986[] = {{&S3661[13], 9}, {NULL}};
const Segment SGI361987[] = {{&S3661[33], 15}, {NULL}};
const Segment SGI361988[] = {{&S3661[49], 6}, {NULL}};
const Segment SGI361989[] = {{&S3661[33], 13}, {NULL}};
const Segment SGI361990[] = {{&S3662[81], 6}, {NULL}};
const Segment SGI361991[] = {{&S3662[8], 3}, {NULL}};
const Segment SGI361992[] = {{&S3662[13], 9}, {NULL}};
const Segment SGI361993[] = {{&S3662[33], 15}, {NULL}};
const Segment SGI361994[] = {{&S3662[49], 11}, {NULL}};
const Segment SGI361995[] = {{&S3662[33], 13}, {NULL}};
const Segment SGI361996[] = {{&S3663[69], 6}, {NULL}};
const Segment SGI361997[] = {{&S3663[8], 3}, {NULL}};
const Segment SGI361998[] = {{&S3663[13], 9}, {NULL}};
const Segment SGI361999[] = {{&S3663[33], 15}, {NULL}};
const Segment SGI362000[] = {{&S3663[49], 6}, {NULL}};
const Segment SGI362001[] = {{&S3663[33], 13}, {NULL}};
const Segment SGI362002[] = {{&S3664[69], 6}, {NULL}};
const Segment SGI362003[] = {{&S3664[8], 3}, {NULL}};
const Segment SGI362004[] = {{&S3664[13], 9}, {NULL}};
const Segment SGI362005[] = {{&S3664[33], 15}, {NULL}};
const Segment SGI362006[] = {{&S3664[49], 6}, {NULL}};
const Segment SGI362007[] = {{&S3664[33], 13}, {NULL}};
const Segment SGI362008[] = {{&S3665[69], 6}, {NULL}};
const Segment SGI362009[] = {{&S3665[8], 3}, {NULL}};
const Segment SGI362010[] = {{&S3665[13], 9}, {NULL}};
const Segment SGI362011[] = {{&S3665[33], 15}, {NULL}};
const Segment SGI362012[] = {{&S3665[49], 6}, {NULL}};
const Segment SGI362013[] = {{&S3665[33], 13}, {NULL}};
const Segment SGI362014[] = {{&S3666[69], 6}, {NULL}};
const Segment SGI362015[] = {{&S3666[8], 3}, {NULL}};
const Segment SGI362016[] = {{&S3666[13], 9}, {NULL}};
const Segment SGI362017[] = {{&S3666[33], 15}, {NULL}};
const Segment SGI362018[] = {{&S3666[49], 6}, {NULL}};
const Segment SGI362019[] = {{&S3666[33], 13}, {NULL}};
const Segment SGI362020[] = {{&S3667[69], 6}, {NULL}};
const Segment SGI362021[] = {{&S3667[8], 3}, {NULL}};
const Segment SGI362022[] = {{&S3667[13], 9}, {NULL}};
const Segment SGI362023[] = {{&S3667[33], 15}, {NULL}};
const Segment SGI362024[] = {{&S3667[49], 6}, {NULL}};
const Segment SGI362025[] = {{&S3667[33], 13}, {NULL}};
const Segment SGI362026[] = {{&S3668[69], 6}, {NULL}};
const Segment SGI362027[] = {{&S3668[8], 3}, {NULL}};
const Segment SGI362028[] = {{&S3668[13], 9}, {NULL}};
const Segment SGI362029[] = {{&S3668[33], 15}, {NULL}};
const Segment SGI362030[] = {{&S3668[49], 6}, {NULL}};
const Segment SGI362031[] = {{&S3668[33], 13}, {NULL}};
const Segment SGI362032[] = {{&S3669[69], 6}, {NULL}};
const Segment SGI362033[] = {{&S3669[8], 3}, {NULL}};
const Segment SGI362034[] = {{&S3669[13], 9}, {NULL}};
const Segment SGI362035[] = {{&S3669[33], 15}, {NULL}};
const Segment SGI362036[] = {{&S3669[49], 6}, {NULL}};
const Segment SGI362037[] = {{&S3669[33], 13}, {NULL}};
const Segment SGI362038[] = {{&S3670[69], 6}, {NULL}};
const Segment SGI362039[] = {{&S3670[8], 3}, {NULL}};
const Segment SGI362040[] = {{&S3670[13], 9}, {NULL}};
const Segment SGI362041[] = {{&S3670[33], 15}, {NULL}};
const Segment SGI362042[] = {{&S3670[49], 6}, {NULL}};
const Segment SGI362043[] = {{&S3670[33], 13}, {NULL}};
const Segment SGI362044[] = {{&S3671[69], 6}, {NULL}};
const Segment SGI362045[] = {{&S3671[8], 3}, {NULL}};
const Segment SGI362046[] = {{&S3671[13], 9}, {NULL}};
const Segment SGI362047[] = {{&S3671[33], 15}, {NULL}};
const Segment SGI362048[] = {{&S3671[49], 6}, {NULL}};
const Segment SGI362049[] = {{&S3671[33], 13}, {NULL}};
const Segment SGI362050[] = {{&S3672[69], 6}, {NULL}};
const Segment SGI362051[] = {{&S3672[8], 3}, {NULL}};
const Segment SGI362052[] = {{&S3672[13], 9}, {NULL}};
const Segment SGI362053[] = {{&S3672[33], 15}, {NULL}};
const Segment SGI362054[] = {{&S3672[49], 6}, {NULL}};
const Segment SGI362055[] = {{&S3672[33], 13}, {NULL}};
const Segment SGI362056[] = {{&S3673[69], 6}, {NULL}};
const Segment SGI362057[] = {{&S3673[8], 3}, {NULL}};
const Segment SGI362058[] = {{&S3673[13], 9}, {NULL}};
const Segment SGI362059[] = {{&S3673[33], 15}, {NULL}};
const Segment SGI362060[] = {{&S3673[49], 6}, {NULL}};
const Segment SGI362061[] = {{&S3673[33], 13}, {NULL}};
const Segment SGI362062[] = {{&S3674[69], 6}, {NULL}};
const Segment SGI362063[] = {{&S3674[8], 3}, {NULL}};
const Segment SGI362064[] = {{&S3674[13], 9}, {NULL}};
const Segment SGI362065[] = {{&S3674[33], 15}, {NULL}};
const Segment SGI362066[] = {{&S3674[49], 6}, {NULL}};
const Segment SGI362067[] = {{&S3674[33], 13}, {NULL}};
const Segment SGI362068[] = {{&S3675[69], 6}, {NULL}};
const Segment SGI362069[] = {{&S3675[8], 3}, {NULL}};
const Segment SGI362070[] = {{&S3675[13], 9}, {NULL}};
const Segment SGI362071[] = {{&S3675[33], 15}, {NULL}};
const Segment SGI362072[] = {{&S3675[49], 6}, {NULL}};
const Segment SGI362073[] = {{&S3675[33], 13}, {NULL}};
const Segment SGI362074[] = {{&S3676[71], 6}, {NULL}};
const Segment SGI362075[] = {{&S3676[8], 3}, {NULL}};
const Segment SGI362076[] = {{&S3676[13], 9}, {NULL}};
const Segment SGI362077[] = {{&S3676[33], 15}, {NULL}};
const Segment SGI362078[] = {{&S3676[49], 8}, {NULL}};
const Segment SGI362079[] = {{&S3676[33], 13}, {NULL}};
const Segment SGI362080[] = {{&S3677[69], 6}, {NULL}};
const Segment SGI362081[] = {{&S3677[8], 3}, {NULL}};
const Segment SGI362082[] = {{&S3677[13], 9}, {NULL}};
const Segment SGI362083[] = {{&S3677[33], 15}, {NULL}};
const Segment SGI362084[] = {{&S3677[49], 6}, {NULL}};
const Segment SGI362085[] = {{&S3677[33], 13}, {NULL}};
const Segment SGI362086[] = {{&S3678[69], 6}, {NULL}};
const Segment SGI362087[] = {{&S3678[8], 3}, {NULL}};
const Segment SGI362088[] = {{&S3678[13], 9}, {NULL}};
const Segment SGI362089[] = {{&S3678[33], 15}, {NULL}};
const Segment SGI362090[] = {{&S3678[49], 6}, {NULL}};
const Segment SGI362091[] = {{&S3678[33], 13}, {NULL}};
const Segment SGI362092[] = {{&S3679[69], 6}, {NULL}};
const Segment SGI362093[] = {{&S3679[8], 3}, {NULL}};
const Segment SGI362094[] = {{&S3679[13], 9}, {NULL}};
const Segment SGI362095[] = {{&S3679[33], 15}, {NULL}};
const Segment SGI362096[] = {{&S3679[49], 6}, {NULL}};
const Segment SGI362097[] = {{&S3679[33], 13}, {NULL}};
const Segment SGI362098[] = {{&S3680[71], 6}, {NULL}};
const Segment SGI362099[] = {{&S3680[8], 3}, {NULL}};
const Segment SGI362100[] = {{&S3680[13], 9}, {NULL}};
const Segment SGI362101[] = {{&S3680[33], 15}, {NULL}};
const Segment SGI362102[] = {{&S3680[49], 8}, {NULL}};
const Segment SGI362103[] = {{&S3680[33], 13}, {NULL}};
const Segment SGI362104[] = {{&S3681[69], 6}, {NULL}};
const Segment SGI362105[] = {{&S3681[8], 3}, {NULL}};
const Segment SGI362106[] = {{&S3681[13], 9}, {NULL}};
const Segment SGI362107[] = {{&S3681[33], 15}, {NULL}};
const Segment SGI362108[] = {{&S3681[49], 6}, {NULL}};
const Segment SGI362109[] = {{&S3681[33], 13}, {NULL}};
const Segment SGI362110[] = {{&S3682[119], 6}, {NULL}};
const Segment SGI362111[] = {{&S3682[8], 3}, {NULL}};
const Segment SGI362112[] = {{&S3682[13], 9}, {NULL}};
const Segment SGI362113[] = {{&S3682[33], 15}, {NULL}};
const Segment SGI362114[] = {{&S3682[49], 6}, {NULL}};
const Segment SGI362115[] = {{&S3682[33], 13}, {NULL}};
const Segment SGI362116[] = {{&S3683[69], 6}, {NULL}};
const Segment SGI362117[] = {{&S3683[8], 3}, {NULL}};
const Segment SGI362118[] = {{&S3683[13], 9}, {NULL}};
const Segment SGI362119[] = {{&S3683[33], 15}, {NULL}};
const Segment SGI362120[] = {{&S3683[49], 6}, {NULL}};
const Segment SGI362121[] = {{&S3683[33], 13}, {NULL}};
const Segment SGI362122[] = {{&S3684[66], 6}, {NULL}};
const Segment SGI362123[] = {{&S3684[8], 3}, {NULL}};
const Segment SGI362124[] = {{&S3684[13], 9}, {NULL}};
const Segment SGI362125[] = {{&S3684[33], 15}, {NULL}};
const Segment SGI362126[] = {{&S3684[49], 3}, {NULL}};
const Segment SGI362127[] = {{&S3684[33], 13}, {NULL}};
const Segment SGI362128[] = {{&S3685[69], 6}, {NULL}};
const Segment SGI362129[] = {{&S3685[8], 3}, {NULL}};
const Segment SGI362130[] = {{&S3685[13], 9}, {NULL}};
const Segment SGI362131[] = {{&S3685[33], 15}, {NULL}};
const Segment SGI362132[] = {{&S3685[49], 6}, {NULL}};
const Segment SGI362133[] = {{&S3685[33], 13}, {NULL}};
const Segment SGI362134[] = {{&S3686[66], 6}, {NULL}};
const Segment SGI362135[] = {{&S3686[8], 3}, {NULL}};
const Segment SGI362136[] = {{&S3686[13], 9}, {NULL}};
const Segment SGI362137[] = {{&S3686[33], 15}, {NULL}};
const Segment SGI362138[] = {{&S3686[49], 3}, {NULL}};
const Segment SGI362139[] = {{&S3686[33], 13}, {NULL}};
const Segment SGI362140[] = {{&S3687[69], 6}, {NULL}};
const Segment SGI362141[] = {{&S3687[8], 3}, {NULL}};
const Segment SGI362142[] = {{&S3687[13], 9}, {NULL}};
const Segment SGI362143[] = {{&S3687[33], 15}, {NULL}};
const Segment SGI362144[] = {{&S3687[49], 6}, {NULL}};
const Segment SGI362145[] = {{&S3687[33], 13}, {NULL}};
const Segment SGI362146[] = {{&S3688[69], 6}, {NULL}};
const Segment SGI362147[] = {{&S3688[8], 3}, {NULL}};
const Segment SGI362148[] = {{&S3688[13], 9}, {NULL}};
const Segment SGI362149[] = {{&S3688[33], 15}, {NULL}};
const Segment SGI362150[] = {{&S3688[49], 6}, {NULL}};
const Segment SGI362151[] = {{&S3688[33], 13}, {NULL}};
const Segment SGI362152[] = {{&S3689[69], 6}, {NULL}};
const Segment SGI362153[] = {{&S3689[8], 3}, {NULL}};
const Segment SGI362154[] = {{&S3689[13], 9}, {NULL}};
const Segment SGI362155[] = {{&S3689[33], 15}, {NULL}};
const Segment SGI362156[] = {{&S3689[49], 6}, {NULL}};
const Segment SGI362157[] = {{&S3689[33], 13}, {NULL}};
const Segment SGI362158[] = {{&S3690[69], 6}, {NULL}};
const Segment SGI362159[] = {{&S3690[8], 3}, {NULL}};
const Segment SGI362160[] = {{&S3690[13], 9}, {NULL}};
const Segment SGI362161[] = {{&S3690[33], 15}, {NULL}};
const Segment SGI362162[] = {{&S3690[49], 6}, {NULL}};
const Segment SGI362163[] = {{&S3690[33], 13}, {NULL}};
const Segment SGI362164[] = {{&S3691[69], 6}, {NULL}};
const Segment SGI362165[] = {{&S3691[8], 3}, {NULL}};
const Segment SGI362166[] = {{&S3691[13], 9}, {NULL}};
const Segment SGI362167[] = {{&S3691[33], 15}, {NULL}};
const Segment SGI362168[] = {{&S3691[49], 6}, {NULL}};
const Segment SGI362169[] = {{&S3691[33], 13}, {NULL}};
const Segment SGI362170[] = {{&S3692[69], 6}, {NULL}};
const Segment SGI362171[] = {{&S3692[8], 3}, {NULL}};
const Segment SGI362172[] = {{&S3692[13], 9}, {NULL}};
const Segment SGI362173[] = {{&S3692[33], 15}, {NULL}};
const Segment SGI362174[] = {{&S3692[49], 6}, {NULL}};
const Segment SGI362175[] = {{&S3692[33], 13}, {NULL}};
const Segment SGI362176[] = {{&S3693[66], 6}, {NULL}};
const Segment SGI362177[] = {{&S3693[8], 3}, {NULL}};
const Segment SGI362178[] = {{&S3693[13], 9}, {NULL}};
const Segment SGI362179[] = {{&S3693[33], 15}, {NULL}};
const Segment SGI362180[] = {{&S3693[49], 3}, {NULL}};
const Segment SGI362181[] = {{&S3693[33], 13}, {NULL}};
const Segment SGI362182[] = {{&S3694[119], 6}, {NULL}};
const Segment SGI362183[] = {{&S3694[8], 3}, {NULL}};
const Segment SGI362184[] = {{&S3694[13], 9}, {NULL}};
const Segment SGI362185[] = {{&S3694[33], 15}, {NULL}};
const Segment SGI362186[] = {{&S3694[49], 6}, {NULL}};
const Segment SGI362187[] = {{&S3694[33], 13}, {NULL}};
const Segment SGI362188[] = {{&S3695[69], 6}, {NULL}};
const Segment SGI362189[] = {{&S3695[8], 3}, {NULL}};
const Segment SGI362190[] = {{&S3695[13], 9}, {NULL}};
const Segment SGI362191[] = {{&S3695[33], 15}, {NULL}};
const Segment SGI362192[] = {{&S3695[49], 6}, {NULL}};
const Segment SGI362193[] = {{&S3695[33], 13}, {NULL}};
const Segment SGI362194[] = {{&S3696[81], 6}, {NULL}};
const Segment SGI362195[] = {{&S3696[8], 3}, {NULL}};
const Segment SGI362196[] = {{&S3696[13], 9}, {NULL}};
const Segment SGI362197[] = {{&S3696[33], 15}, {NULL}};
const Segment SGI362198[] = {{&S3696[49], 11}, {NULL}};
const Segment SGI362199[] = {{&S3696[33], 13}, {NULL}};
const Segment SGI362200[] = {{&S3697[69], 6}, {NULL}};
const Segment SGI362201[] = {{&S3697[8], 3}, {NULL}};
const Segment SGI362202[] = {{&S3697[13], 9}, {NULL}};
const Segment SGI362203[] = {{&S3697[33], 15}, {NULL}};
const Segment SGI362204[] = {{&S3697[49], 6}, {NULL}};
const Segment SGI362205[] = {{&S3697[33], 13}, {NULL}};
const Segment SGI362206[] = {{&S3698[66], 6}, {NULL}};
const Segment SGI362207[] = {{&S3698[8], 3}, {NULL}};
const Segment SGI362208[] = {{&S3698[13], 9}, {NULL}};
const Segment SGI362209[] = {{&S3698[33], 15}, {NULL}};
const Segment SGI362210[] = {{&S3698[49], 3}, {NULL}};
const Segment SGI362211[] = {{&S3698[33], 13}, {NULL}};
const Segment SGI362212[] = {{&S3699[69], 6}, {NULL}};
const Segment SGI362213[] = {{&S3699[8], 3}, {NULL}};
const Segment SGI362214[] = {{&S3699[13], 9}, {NULL}};
const Segment SGI362215[] = {{&S3699[33], 15}, {NULL}};
const Segment SGI362216[] = {{&S3699[49], 6}, {NULL}};
const Segment SGI362217[] = {{&S3699[33], 13}, {NULL}};
const Segment SGI362218[] = {{&S3700[69], 6}, {NULL}};
const Segment SGI362219[] = {{&S3700[8], 3}, {NULL}};
const Segment SGI362220[] = {{&S3700[13], 9}, {NULL}};
const Segment SGI362221[] = {{&S3700[33], 15}, {NULL}};
const Segment SGI362222[] = {{&S3700[49], 6}, {NULL}};
const Segment SGI362223[] = {{&S3700[33], 13}, {NULL}};
const Segment SGI362224[] = {{&S3701[69], 6}, {NULL}};
const Segment SGI362225[] = {{&S3701[8], 3}, {NULL}};
const Segment SGI362226[] = {{&S3701[13], 9}, {NULL}};
const Segment SGI362227[] = {{&S3701[33], 15}, {NULL}};
const Segment SGI362228[] = {{&S3701[49], 6}, {NULL}};
const Segment SGI362229[] = {{&S3701[33], 13}, {NULL}};
const Segment SGI362230[] = {{&S3702[69], 6}, {NULL}};
const Segment SGI362231[] = {{&S3702[8], 3}, {NULL}};
const Segment SGI362232[] = {{&S3702[13], 9}, {NULL}};
const Segment SGI362233[] = {{&S3702[33], 15}, {NULL}};
const Segment SGI362234[] = {{&S3702[49], 6}, {NULL}};
const Segment SGI362235[] = {{&S3702[33], 13}, {NULL}};
const Segment SGI362236[] = {{&S3703[69], 6}, {NULL}};
const Segment SGI362237[] = {{&S3703[8], 3}, {NULL}};
const Segment SGI362238[] = {{&S3703[13], 9}, {NULL}};
const Segment SGI362239[] = {{&S3703[33], 15}, {NULL}};
const Segment SGI362240[] = {{&S3703[49], 6}, {NULL}};
const Segment SGI362241[] = {{&S3703[33], 13}, {NULL}};
const Segment SGI362242[] = {{&S3704[69], 6}, {NULL}};
const Segment SGI362243[] = {{&S3704[8], 3}, {NULL}};
const Segment SGI362244[] = {{&S3704[13], 9}, {NULL}};
const Segment SGI362245[] = {{&S3704[33], 15}, {NULL}};
const Segment SGI362246[] = {{&S3704[49], 6}, {NULL}};
const Segment SGI362247[] = {{&S3704[33], 13}, {NULL}};
const Segment SGI362248[] = {{&S3705[69], 6}, {NULL}};
const Segment SGI362249[] = {{&S3705[8], 3}, {NULL}};
const Segment SGI362250[] = {{&S3705[13], 9}, {NULL}};
const Segment SGI362251[] = {{&S3705[33], 15}, {NULL}};
const Segment SGI362252[] = {{&S3705[49], 6}, {NULL}};
const Segment SGI362253[] = {{&S3705[33], 13}, {NULL}};
const Segment SGI362254[] = {{&S3706[69], 6}, {NULL}};
const Segment SGI362255[] = {{&S3706[8], 3}, {NULL}};
const Segment SGI362256[] = {{&S3706[13], 9}, {NULL}};
const Segment SGI362257[] = {{&S3706[33], 15}, {NULL}};
const Segment SGI362258[] = {{&S3706[49], 6}, {NULL}};
const Segment SGI362259[] = {{&S3706[33], 13}, {NULL}};
const Segment SGI362260[] = {{&S3707[69], 6}, {NULL}};
const Segment SGI362261[] = {{&S3707[8], 3}, {NULL}};
const Segment SGI362262[] = {{&S3707[13], 9}, {NULL}};
const Segment SGI362263[] = {{&S3707[33], 15}, {NULL}};
const Segment SGI362264[] = {{&S3707[49], 6}, {NULL}};
const Segment SGI362265[] = {{&S3707[33], 13}, {NULL}};
const Segment SGI362266[] = {{&S3708[66], 6}, {NULL}};
const Segment SGI362267[] = {{&S3708[8], 3}, {NULL}};
const Segment SGI362268[] = {{&S3708[13], 9}, {NULL}};
const Segment SGI362269[] = {{&S3708[33], 15}, {NULL}};
const Segment SGI362270[] = {{&S3708[49], 3}, {NULL}};
const Segment SGI362271[] = {{&S3708[33], 13}, {NULL}};
const Segment SGI362272[] = {{&S3709[66], 6}, {NULL}};
const Segment SGI362273[] = {{&S3709[8], 3}, {NULL}};
const Segment SGI362274[] = {{&S3709[13], 9}, {NULL}};
const Segment SGI362275[] = {{&S3709[33], 15}, {NULL}};
const Segment SGI362276[] = {{&S3709[49], 3}, {NULL}};
const Segment SGI362277[] = {{&S3709[33], 13}, {NULL}};
const Segment SGI362278[] = {{&S3710[69], 6}, {NULL}};
const Segment SGI362279[] = {{&S3710[8], 3}, {NULL}};
const Segment SGI362280[] = {{&S3710[13], 9}, {NULL}};
const Segment SGI362281[] = {{&S3710[33], 15}, {NULL}};
const Segment SGI362282[] = {{&S3710[49], 6}, {NULL}};
const Segment SGI362283[] = {{&S3710[33], 13}, {NULL}};
const Segment SGI362284[] = {{&S3711[69], 6}, {NULL}};
const Segment SGI362285[] = {{&S3711[8], 3}, {NULL}};
const Segment SGI362286[] = {{&S3711[13], 9}, {NULL}};
const Segment SGI362287[] = {{&S3711[33], 15}, {NULL}};
const Segment SGI362288[] = {{&S3711[49], 6}, {NULL}};
const Segment SGI362289[] = {{&S3711[33], 13}, {NULL}};
const Segment SGI362290[] = {{&S3712[119], 6}, {NULL}};
const Segment SGI362291[] = {{&S3712[8], 3}, {NULL}};
const Segment SGI362292[] = {{&S3712[13], 9}, {NULL}};
const Segment SGI362293[] = {{&S3712[33], 15}, {NULL}};
const Segment SGI362294[] = {{&S3712[49], 6}, {NULL}};
const Segment SGI362295[] = {{&S3712[33], 13}, {NULL}};
const Segment SGI362296[] = {{&S3713[69], 6}, {NULL}};
const Segment SGI362297[] = {{&S3713[8], 3}, {NULL}};
const Segment SGI362298[] = {{&S3713[13], 9}, {NULL}};
const Segment SGI362299[] = {{&S3713[33], 15}, {NULL}};
const Segment SGI362300[] = {{&S3713[49], 6}, {NULL}};
const Segment SGI362301[] = {{&S3713[33], 13}, {NULL}};
const Segment SGI362302[] = {{&S3714[69], 6}, {NULL}};
const Segment SGI362303[] = {{&S3714[8], 3}, {NULL}};
const Segment SGI362304[] = {{&S3714[13], 9}, {NULL}};
const Segment SGI362305[] = {{&S3714[33], 15}, {NULL}};
const Segment SGI362306[] = {{&S3714[49], 6}, {NULL}};
const Segment SGI362307[] = {{&S3714[33], 13}, {NULL}};
const Segment SGI362308[] = {{&S3715[69], 6}, {NULL}};
const Segment SGI362309[] = {{&S3715[8], 3}, {NULL}};
const Segment SGI362310[] = {{&S3715[13], 9}, {NULL}};
const Segment SGI362311[] = {{&S3715[33], 15}, {NULL}};
const Segment SGI362312[] = {{&S3715[49], 6}, {NULL}};
const Segment SGI362313[] = {{&S3715[33], 13}, {NULL}};
const Segment SGI362314[] = {{&S3716[69], 6}, {NULL}};
const Segment SGI362315[] = {{&S3716[8], 3}, {NULL}};
const Segment SGI362316[] = {{&S3716[13], 9}, {NULL}};
const Segment SGI362317[] = {{&S3716[33], 15}, {NULL}};
const Segment SGI362318[] = {{&S3716[49], 6}, {NULL}};
const Segment SGI362319[] = {{&S3716[33], 13}, {NULL}};
const Segment SGI362320[] = {{&S3717[69], 6}, {NULL}};
const Segment SGI362321[] = {{&S3717[8], 3}, {NULL}};
const Segment SGI362322[] = {{&S3717[13], 9}, {NULL}};
const Segment SGI362323[] = {{&S3717[33], 15}, {NULL}};
const Segment SGI362324[] = {{&S3717[49], 6}, {NULL}};
const Segment SGI362325[] = {{&S3717[33], 13}, {NULL}};
const Segment SGI362326[] = {{&S3718[69], 6}, {NULL}};
const Segment SGI362327[] = {{&S3718[8], 3}, {NULL}};
const Segment SGI362328[] = {{&S3718[13], 9}, {NULL}};
const Segment SGI362329[] = {{&S3718[33], 15}, {NULL}};
const Segment SGI362330[] = {{&S3718[49], 6}, {NULL}};
const Segment SGI362331[] = {{&S3718[33], 13}, {NULL}};
const Segment SGI362332[] = {{&S3719[66], 6}, {NULL}};
const Segment SGI362333[] = {{&S3719[8], 3}, {NULL}};
const Segment SGI362334[] = {{&S3719[13], 9}, {NULL}};
const Segment SGI362335[] = {{&S3719[33], 15}, {NULL}};
const Segment SGI362336[] = {{&S3719[49], 3}, {NULL}};
const Segment SGI362337[] = {{&S3719[33], 13}, {NULL}};
const Segment SGI362338[] = {{&S3720[69], 6}, {NULL}};
const Segment SGI362339[] = {{&S3720[8], 3}, {NULL}};
const Segment SGI362340[] = {{&S3720[13], 9}, {NULL}};
const Segment SGI362341[] = {{&S3720[33], 15}, {NULL}};
const Segment SGI362342[] = {{&S3720[49], 6}, {NULL}};
const Segment SGI362343[] = {{&S3720[33], 13}, {NULL}};
const Segment SGI362344[] = {{&S3721[69], 6}, {NULL}};
const Segment SGI362345[] = {{&S3721[8], 3}, {NULL}};
const Segment SGI362346[] = {{&S3721[13], 9}, {NULL}};
const Segment SGI362347[] = {{&S3721[33], 15}, {NULL}};
const Segment SGI362348[] = {{&S3721[49], 6}, {NULL}};
const Segment SGI362349[] = {{&S3721[33], 13}, {NULL}};
const Segment SGI362350[] = {{&S3722[66], 6}, {NULL}};
const Segment SGI362351[] = {{&S3722[8], 3}, {NULL}};
const Segment SGI362352[] = {{&S3722[13], 9}, {NULL}};
const Segment SGI362353[] = {{&S3722[33], 15}, {NULL}};
const Segment SGI362354[] = {{&S3722[49], 3}, {NULL}};
const Segment SGI362355[] = {{&S3722[33], 13}, {NULL}};
const Segment SGI362356[] = {{&S3723[69], 6}, {NULL}};
const Segment SGI362357[] = {{&S3723[8], 3}, {NULL}};
const Segment SGI362358[] = {{&S3723[13], 9}, {NULL}};
const Segment SGI362359[] = {{&S3723[33], 15}, {NULL}};
const Segment SGI362360[] = {{&S3723[49], 6}, {NULL}};
const Segment SGI362361[] = {{&S3723[33], 13}, {NULL}};
const Segment SGI362362[] = {{&S3724[69], 6}, {NULL}};
const Segment SGI362363[] = {{&S3724[8], 3}, {NULL}};
const Segment SGI362364[] = {{&S3724[13], 9}, {NULL}};
const Segment SGI362365[] = {{&S3724[33], 15}, {NULL}};
const Segment SGI362366[] = {{&S3724[49], 6}, {NULL}};
const Segment SGI362367[] = {{&S3724[33], 13}, {NULL}};
const Segment SGI362368[] = {{&S3725[69], 6}, {NULL}};
const Segment SGI362369[] = {{&S3725[8], 3}, {NULL}};
const Segment SGI362370[] = {{&S3725[13], 9}, {NULL}};
const Segment SGI362371[] = {{&S3725[33], 15}, {NULL}};
const Segment SGI362372[] = {{&S3725[49], 6}, {NULL}};
const Segment SGI362373[] = {{&S3725[33], 13}, {NULL}};
const Segment SGI362374[] = {{&S3726[69], 6}, {NULL}};
const Segment SGI362375[] = {{&S3726[8], 3}, {NULL}};
const Segment SGI362376[] = {{&S3726[13], 9}, {NULL}};
const Segment SGI362377[] = {{&S3726[33], 15}, {NULL}};
const Segment SGI362378[] = {{&S3726[49], 6}, {NULL}};
const Segment SGI362379[] = {{&S3726[33], 13}, {NULL}};
const Segment SGI362380[] = {{&S3727[69], 6}, {NULL}};
const Segment SGI362381[] = {{&S3727[8], 3}, {NULL}};
const Segment SGI362382[] = {{&S3727[13], 9}, {NULL}};
const Segment SGI362383[] = {{&S3727[33], 15}, {NULL}};
const Segment SGI362384[] = {{&S3727[49], 6}, {NULL}};
const Segment SGI362385[] = {{&S3727[33], 13}, {NULL}};
const Segment SGI362386[] = {{&S3728[69], 6}, {NULL}};
const Segment SGI362387[] = {{&S3728[8], 3}, {NULL}};
const Segment SGI362388[] = {{&S3728[13], 9}, {NULL}};
const Segment SGI362389[] = {{&S3728[33], 15}, {NULL}};
const Segment SGI362390[] = {{&S3728[49], 6}, {NULL}};
const Segment SGI362391[] = {{&S3728[33], 13}, {NULL}};
const Segment SGI362392[] = {{&S3729[69], 6}, {NULL}};
const Segment SGI362393[] = {{&S3729[8], 3}, {NULL}};
const Segment SGI362394[] = {{&S3729[13], 9}, {NULL}};
const Segment SGI362395[] = {{&S3729[33], 15}, {NULL}};
const Segment SGI362396[] = {{&S3729[49], 6}, {NULL}};
const Segment SGI362397[] = {{&S3729[33], 13}, {NULL}};
const Segment SGI362398[] = {{&S3730[69], 6}, {NULL}};
const Segment SGI362399[] = {{&S3730[8], 3}, {NULL}};
const Segment SGI362400[] = {{&S3730[13], 9}, {NULL}};
const Segment SGI362401[] = {{&S3730[33], 15}, {NULL}};
const Segment SGI362402[] = {{&S3730[49], 6}, {NULL}};
const Segment SGI362403[] = {{&S3730[33], 13}, {NULL}};
const Segment SGI362404[] = {{&S3731[69], 6}, {NULL}};
const Segment SGI362405[] = {{&S3731[8], 3}, {NULL}};
const Segment SGI362406[] = {{&S3731[13], 9}, {NULL}};
const Segment SGI362407[] = {{&S3731[33], 15}, {NULL}};
const Segment SGI362408[] = {{&S3731[49], 6}, {NULL}};
const Segment SGI362409[] = {{&S3731[33], 13}, {NULL}};
const Segment SGI362410[] = {{&S3732[69], 6}, {NULL}};
const Segment SGI362411[] = {{&S3732[8], 3}, {NULL}};
const Segment SGI362412[] = {{&S3732[13], 9}, {NULL}};
const Segment SGI362413[] = {{&S3732[33], 15}, {NULL}};
const Segment SGI362414[] = {{&S3732[49], 6}, {NULL}};
const Segment SGI362415[] = {{&S3732[33], 13}, {NULL}};
const Segment SGI362416[] = {{&S3733[69], 6}, {NULL}};
const Segment SGI362417[] = {{&S3733[8], 3}, {NULL}};
const Segment SGI362418[] = {{&S3733[13], 9}, {NULL}};
const Segment SGI362419[] = {{&S3733[33], 15}, {NULL}};
const Segment SGI362420[] = {{&S3733[49], 6}, {NULL}};
const Segment SGI362421[] = {{&S3733[33], 13}, {NULL}};
const Segment SGI362422[] = {{&S3734[66], 6}, {NULL}};
const Segment SGI362423[] = {{&S3734[8], 3}, {NULL}};
const Segment SGI362424[] = {{&S3734[13], 9}, {NULL}};
const Segment SGI362425[] = {{&S3734[33], 15}, {NULL}};
const Segment SGI362426[] = {{&S3734[49], 3}, {NULL}};
const Segment SGI362427[] = {{&S3734[33], 13}, {NULL}};
const Segment SGI362428[] = {{&S3735[69], 6}, {NULL}};
const Segment SGI362429[] = {{&S3735[8], 3}, {NULL}};
const Segment SGI362430[] = {{&S3735[13], 9}, {NULL}};
const Segment SGI362431[] = {{&S3735[33], 15}, {NULL}};
const Segment SGI362432[] = {{&S3735[49], 6}, {NULL}};
const Segment SGI362433[] = {{&S3735[33], 13}, {NULL}};
const Segment SGI362434[] = {{&S3736[66], 6}, {NULL}};
const Segment SGI362435[] = {{&S3736[8], 3}, {NULL}};
const Segment SGI362436[] = {{&S3736[13], 9}, {NULL}};
const Segment SGI362437[] = {{&S3736[33], 15}, {NULL}};
const Segment SGI362438[] = {{&S3736[49], 3}, {NULL}};
const Segment SGI362439[] = {{&S3736[33], 13}, {NULL}};
const Segment SGI362440[] = {{&S3737[69], 6}, {NULL}};
const Segment SGI362441[] = {{&S3737[8], 3}, {NULL}};
const Segment SGI362442[] = {{&S3737[13], 9}, {NULL}};
const Segment SGI362443[] = {{&S3737[33], 15}, {NULL}};
const Segment SGI362444[] = {{&S3737[49], 6}, {NULL}};
const Segment SGI362445[] = {{&S3737[33], 13}, {NULL}};
const Segment SGI362446[] = {{&S3738[105], 7}, {NULL}};
const Segment SGI362447[] = {{&S3738[8], 3}, {NULL}};
const Segment SGI362448[] = {{&S3738[13], 9}, {NULL}};
const Segment SGI362449[] = {{&S3738[33], 15}, {NULL}};
const Segment SGI362450[] = {{&S3738[49], 6}, {NULL}};
const Segment SGI362451[] = {{&S3738[33], 13}, {NULL}};
const Segment SGI362452[] = {{&S3739[66], 6}, {NULL}};
const Segment SGI362453[] = {{&S3739[8], 3}, {NULL}};
const Segment SGI362454[] = {{&S3739[13], 9}, {NULL}};
const Segment SGI362455[] = {{&S3739[33], 15}, {NULL}};
const Segment SGI362456[] = {{&S3739[49], 3}, {NULL}};
const Segment SGI362457[] = {{&S3739[33], 13}, {NULL}};
const Segment SGI362458[] = {{&S3740[69], 6}, {NULL}};
const Segment SGI362459[] = {{&S3740[8], 3}, {NULL}};
const Segment SGI362460[] = {{&S3740[13], 9}, {NULL}};
const Segment SGI362461[] = {{&S3740[33], 15}, {NULL}};
const Segment SGI362462[] = {{&S3740[49], 6}, {NULL}};
const Segment SGI362463[] = {{&S3740[33], 13}, {NULL}};
const Segment SGI362464[] = {{&S3741[69], 6}, {NULL}};
const Segment SGI362465[] = {{&S3741[8], 3}, {NULL}};
const Segment SGI362466[] = {{&S3741[13], 9}, {NULL}};
const Segment SGI362467[] = {{&S3741[33], 15}, {NULL}};
const Segment SGI362468[] = {{&S3741[49], 6}, {NULL}};
const Segment SGI362469[] = {{&S3741[33], 13}, {NULL}};
const Segment SGI362470[] = {{&S3742[69], 6}, {NULL}};
const Segment SGI362471[] = {{&S3742[8], 3}, {NULL}};
const Segment SGI362472[] = {{&S3742[13], 9}, {NULL}};
const Segment SGI362473[] = {{&S3742[33], 15}, {NULL}};
const Segment SGI362474[] = {{&S3742[49], 6}, {NULL}};
const Segment SGI362475[] = {{&S3742[33], 13}, {NULL}};
const Segment SGI362476[] = {{&S3743[69], 6}, {NULL}};
const Segment SGI362477[] = {{&S3743[8], 3}, {NULL}};
const Segment SGI362478[] = {{&S3743[13], 9}, {NULL}};
const Segment SGI362479[] = {{&S3743[33], 15}, {NULL}};
const Segment SGI362480[] = {{&S3743[49], 6}, {NULL}};
const Segment SGI362481[] = {{&S3743[33], 13}, {NULL}};
const Segment SGI362482[] = {{&S3744[69], 6}, {NULL}};
const Segment SGI362483[] = {{&S3744[8], 3}, {NULL}};
const Segment SGI362484[] = {{&S3744[13], 9}, {NULL}};
const Segment SGI362485[] = {{&S3744[33], 15}, {NULL}};
const Segment SGI362486[] = {{&S3744[49], 6}, {NULL}};
const Segment SGI362487[] = {{&S3744[33], 13}, {NULL}};
const Segment SGI362488[] = {{&S3745[69], 6}, {NULL}};
const Segment SGI362489[] = {{&S3745[8], 3}, {NULL}};
const Segment SGI362490[] = {{&S3745[13], 9}, {NULL}};
const Segment SGI362491[] = {{&S3745[33], 15}, {NULL}};
const Segment SGI362492[] = {{&S3745[49], 6}, {NULL}};
const Segment SGI362493[] = {{&S3745[33], 13}, {NULL}};
const Segment SGI362494[] = {{&S3746[69], 6}, {NULL}};
const Segment SGI362495[] = {{&S3746[8], 3}, {NULL}};
const Segment SGI362496[] = {{&S3746[13], 9}, {NULL}};
const Segment SGI362497[] = {{&S3746[33], 15}, {NULL}};
const Segment SGI362498[] = {{&S3746[49], 6}, {NULL}};
const Segment SGI362499[] = {{&S3746[33], 13}, {NULL}};
const Segment SGI362500[] = {{&S3747[69], 6}, {NULL}};
const Segment SGI362501[] = {{&S3747[8], 3}, {NULL}};
const Segment SGI362502[] = {{&S3747[13], 9}, {NULL}};
const Segment SGI362503[] = {{&S3747[33], 15}, {NULL}};
const Segment SGI362504[] = {{&S3747[49], 6}, {NULL}};
const Segment SGI362505[] = {{&S3747[33], 13}, {NULL}};
const Segment SGI362506[] = {{&S3748[69], 6}, {NULL}};
const Segment SGI362507[] = {{&S3748[8], 3}, {NULL}};
const Segment SGI362508[] = {{&S3748[13], 9}, {NULL}};
const Segment SGI362509[] = {{&S3748[33], 15}, {NULL}};
const Segment SGI362510[] = {{&S3748[49], 6}, {NULL}};
const Segment SGI362511[] = {{&S3748[33], 13}, {NULL}};
const Segment SGI362512[] = {{&S3749[69], 6}, {NULL}};
const Segment SGI362513[] = {{&S3749[8], 3}, {NULL}};
const Segment SGI362514[] = {{&S3749[13], 9}, {NULL}};
const Segment SGI362515[] = {{&S3749[33], 15}, {NULL}};
const Segment SGI362516[] = {{&S3749[49], 6}, {NULL}};
const Segment SGI362517[] = {{&S3749[33], 13}, {NULL}};
const Segment SGI362518[] = {{&S3750[69], 6}, {NULL}};
const Segment SGI362519[] = {{&S3750[8], 3}, {NULL}};
const Segment SGI362520[] = {{&S3750[13], 9}, {NULL}};
const Segment SGI362521[] = {{&S3750[33], 15}, {NULL}};
const Segment SGI362522[] = {{&S3750[49], 6}, {NULL}};
const Segment SGI362523[] = {{&S3750[33], 13}, {NULL}};
const Segment SGI362524[] = {{&S3751[69], 6}, {NULL}};
const Segment SGI362525[] = {{&S3751[8], 3}, {NULL}};
const Segment SGI362526[] = {{&S3751[13], 9}, {NULL}};
const Segment SGI362527[] = {{&S3751[33], 15}, {NULL}};
const Segment SGI362528[] = {{&S3751[49], 6}, {NULL}};
const Segment SGI362529[] = {{&S3751[33], 13}, {NULL}};
const Segment SGI362530[] = {{&S3752[69], 6}, {NULL}};
const Segment SGI362531[] = {{&S3752[8], 3}, {NULL}};
const Segment SGI362532[] = {{&S3752[13], 9}, {NULL}};
const Segment SGI362533[] = {{&S3752[33], 15}, {NULL}};
const Segment SGI362534[] = {{&S3752[49], 6}, {NULL}};
const Segment SGI362535[] = {{&S3752[33], 13}, {NULL}};
const Segment SGI362536[] = {{&S3753[119], 6}, {NULL}};
const Segment SGI362537[] = {{&S3753[8], 3}, {NULL}};
const Segment SGI362538[] = {{&S3753[13], 9}, {NULL}};
const Segment SGI362539[] = {{&S3753[33], 15}, {NULL}};
const Segment SGI362540[] = {{&S3753[49], 6}, {NULL}};
const Segment SGI362541[] = {{&S3753[33], 13}, {NULL}};
const Segment SGI362542[] = {{&S3754[69], 6}, {NULL}};
const Segment SGI362543[] = {{&S3754[8], 3}, {NULL}};
const Segment SGI362544[] = {{&S3754[13], 9}, {NULL}};
const Segment SGI362545[] = {{&S3754[33], 15}, {NULL}};
const Segment SGI362546[] = {{&S3754[49], 6}, {NULL}};
const Segment SGI362547[] = {{&S3754[33], 13}, {NULL}};
const Segment SGI362548[] = {{&S3755[66], 6}, {NULL}};
const Segment SGI362549[] = {{&S3755[8], 3}, {NULL}};
const Segment SGI362550[] = {{&S3755[13], 9}, {NULL}};
const Segment SGI362551[] = {{&S3755[33], 15}, {NULL}};
const Segment SGI362552[] = {{&S3755[49], 3}, {NULL}};
const Segment SGI362553[] = {{&S3755[33], 13}, {NULL}};
const Segment SGI362554[] = {{&S3756[66], 6}, {NULL}};
const Segment SGI362555[] = {{&S3756[8], 3}, {NULL}};
const Segment SGI362556[] = {{&S3756[13], 9}, {NULL}};
const Segment SGI362557[] = {{&S3756[33], 15}, {NULL}};
const Segment SGI362558[] = {{&S3756[49], 3}, {NULL}};
const Segment SGI362559[] = {{&S3756[33], 13}, {NULL}};
const Segment SGI362560[] = {{&S3757[66], 6}, {NULL}};
const Segment SGI362561[] = {{&S3757[8], 3}, {NULL}};
const Segment SGI362562[] = {{&S3757[13], 9}, {NULL}};
const Segment SGI362563[] = {{&S3757[33], 15}, {NULL}};
const Segment SGI362564[] = {{&S3757[49], 3}, {NULL}};
const Segment SGI362565[] = {{&S3757[33], 13}, {NULL}};
const Segment SGI362566[] = {{&S3758[69], 6}, {NULL}};
const Segment SGI362567[] = {{&S3758[8], 3}, {NULL}};
const Segment SGI362568[] = {{&S3758[13], 9}, {NULL}};
const Segment SGI362569[] = {{&S3758[33], 15}, {NULL}};
const Segment SGI362570[] = {{&S3758[49], 6}, {NULL}};
const Segment SGI362571[] = {{&S3758[33], 13}, {NULL}};
const Segment SGI362572[] = {{&S3759[69], 6}, {NULL}};
const Segment SGI362573[] = {{&S3759[8], 3}, {NULL}};
const Segment SGI362574[] = {{&S3759[13], 9}, {NULL}};
const Segment SGI362575[] = {{&S3759[33], 15}, {NULL}};
const Segment SGI362576[] = {{&S3759[49], 6}, {NULL}};
const Segment SGI362577[] = {{&S3759[33], 13}, {NULL}};
const Segment SGI362578[] = {{&S3760[69], 6}, {NULL}};
const Segment SGI362579[] = {{&S3760[8], 3}, {NULL}};
const Segment SGI362580[] = {{&S3760[13], 9}, {NULL}};
const Segment SGI362581[] = {{&S3760[33], 15}, {NULL}};
const Segment SGI362582[] = {{&S3760[49], 6}, {NULL}};
const Segment SGI362583[] = {{&S3760[33], 13}, {NULL}};
const Segment SGI362584[] = {{&S3761[69], 6}, {NULL}};
const Segment SGI362585[] = {{&S3761[8], 3}, {NULL}};
const Segment SGI362586[] = {{&S3761[13], 9}, {NULL}};
const Segment SGI362587[] = {{&S3761[33], 15}, {NULL}};
const Segment SGI362588[] = {{&S3761[49], 6}, {NULL}};
const Segment SGI362589[] = {{&S3761[33], 13}, {NULL}};
const Segment SGI362590[] = {{&S3762[66], 6}, {NULL}};
const Segment SGI362591[] = {{&S3762[8], 3}, {NULL}};
const Segment SGI362592[] = {{&S3762[13], 9}, {NULL}};
const Segment SGI362593[] = {{&S3762[33], 15}, {NULL}};
const Segment SGI362594[] = {{&S3762[49], 3}, {NULL}};
const Segment SGI362595[] = {{&S3762[33], 13}, {NULL}};
const Segment SGI362596[] = {{&S3763[119], 6}, {NULL}};
const Segment SGI362597[] = {{&S3763[8], 3}, {NULL}};
const Segment SGI362598[] = {{&S3763[13], 9}, {NULL}};
const Segment SGI362599[] = {{&S3763[33], 15}, {NULL}};
const Segment SGI362600[] = {{&S3763[49], 6}, {NULL}};
const Segment SGI362601[] = {{&S3763[33], 13}, {NULL}};
const Segment SGI362602[] = {{&S3764[121], 7}, {NULL}};
const Segment SGI362603[] = {{&S3764[8], 3}, {NULL}};
const Segment SGI362604[] = {{&S3764[13], 9}, {NULL}};
const Segment SGI362605[] = {{&S3764[33], 15}, {NULL}};
const Segment SGI362606[] = {{&S3764[49], 6}, {NULL}};
const Segment SGI362607[] = {{&S3764[33], 13}, {NULL}};
const Segment SGI362608[] = {{&S3765[69], 6}, {NULL}};
const Segment SGI362609[] = {{&S3765[8], 3}, {NULL}};
const Segment SGI362610[] = {{&S3765[13], 9}, {NULL}};
const Segment SGI362611[] = {{&S3765[33], 15}, {NULL}};
const Segment SGI362612[] = {{&S3765[49], 6}, {NULL}};
const Segment SGI362613[] = {{&S3765[33], 13}, {NULL}};
const Segment SGI362614[] = {{&S3766[117], 6}, {NULL}};
const Segment SGI362615[] = {{&S3766[8], 3}, {NULL}};
const Segment SGI362616[] = {{&S3766[13], 9}, {NULL}};
const Segment SGI362617[] = {{&S3766[33], 15}, {NULL}};
const Segment SGI362618[] = {{&S3766[49], 6}, {NULL}};
const Segment SGI362619[] = {{&S3766[33], 13}, {NULL}};
const Segment SGI362620[] = {{&S3767[66], 6}, {NULL}};
const Segment SGI362621[] = {{&S3767[8], 3}, {NULL}};
const Segment SGI362622[] = {{&S3767[13], 9}, {NULL}};
const Segment SGI362623[] = {{&S3767[33], 15}, {NULL}};
const Segment SGI362624[] = {{&S3767[49], 3}, {NULL}};
const Segment SGI362625[] = {{&S3767[33], 13}, {NULL}};
const Segment SGI362626[] = {{&S3768[66], 6}, {NULL}};
const Segment SGI362627[] = {{&S3768[8], 3}, {NULL}};
const Segment SGI362628[] = {{&S3768[13], 9}, {NULL}};
const Segment SGI362629[] = {{&S3768[33], 15}, {NULL}};
const Segment SGI362630[] = {{&S3768[49], 3}, {NULL}};
const Segment SGI362631[] = {{&S3768[33], 13}, {NULL}};
const Segment SGI362632[] = {{&S3769[119], 6}, {NULL}};
const Segment SGI362633[] = {{&S3769[8], 3}, {NULL}};
const Segment SGI362634[] = {{&S3769[13], 9}, {NULL}};
const Segment SGI362635[] = {{&S3769[33], 15}, {NULL}};
const Segment SGI362636[] = {{&S3769[49], 6}, {NULL}};
const Segment SGI362637[] = {{&S3769[33], 13}, {NULL}};
const Segment SGI362638[] = {{&S3770[119], 6}, {NULL}};
const Segment SGI362639[] = {{&S3770[8], 3}, {NULL}};
const Segment SGI362640[] = {{&S3770[13], 9}, {NULL}};
const Segment SGI362641[] = {{&S3770[33], 15}, {NULL}};
const Segment SGI362642[] = {{&S3770[49], 6}, {NULL}};
const Segment SGI362643[] = {{&S3770[33], 13}, {NULL}};
const Segment SGI362644[] = {{&S3771[121], 7}, {NULL}};
const Segment SGI362645[] = {{&S3771[8], 3}, {NULL}};
const Segment SGI362646[] = {{&S3771[13], 9}, {NULL}};
const Segment SGI362647[] = {{&S3771[33], 15}, {NULL}};
const Segment SGI362648[] = {{&S3771[49], 6}, {NULL}};
const Segment SGI362649[] = {{&S3771[33], 13}, {NULL}};
const Segment SGI362650[] = {{&S3772[69], 6}, {NULL}};
const Segment SGI362651[] = {{&S3772[8], 3}, {NULL}};
const Segment SGI362652[] = {{&S3772[13], 9}, {NULL}};
const Segment SGI362653[] = {{&S3772[33], 15}, {NULL}};
const Segment SGI362654[] = {{&S3772[49], 6}, {NULL}};
const Segment SGI362655[] = {{&S3772[33], 13}, {NULL}};
const Segment SGI362656[] = {{&S3773[66], 6}, {NULL}};
const Segment SGI362657[] = {{&S3773[8], 3}, {NULL}};
const Segment SGI362658[] = {{&S3773[13], 9}, {NULL}};
const Segment SGI362659[] = {{&S3773[33], 15}, {NULL}};
const Segment SGI362660[] = {{&S3773[49], 3}, {NULL}};
const Segment SGI362661[] = {{&S3773[33], 13}, {NULL}};
const Segment SGI362662[] = {{&S3774[66], 6}, {NULL}};
const Segment SGI362663[] = {{&S3774[8], 3}, {NULL}};
const Segment SGI362664[] = {{&S3774[13], 9}, {NULL}};
const Segment SGI362665[] = {{&S3774[33], 15}, {NULL}};
const Segment SGI362666[] = {{&S3774[49], 3}, {NULL}};
const Segment SGI362667[] = {{&S3774[33], 13}, {NULL}};
const Segment SGI362668[] = {{&S3775[69], 6}, {NULL}};
const Segment SGI362669[] = {{&S3775[8], 3}, {NULL}};
const Segment SGI362670[] = {{&S3775[13], 9}, {NULL}};
const Segment SGI362671[] = {{&S3775[33], 15}, {NULL}};
const Segment SGI362672[] = {{&S3775[49], 6}, {NULL}};
const Segment SGI362673[] = {{&S3775[33], 13}, {NULL}};
const Segment SGI362674[] = {{&S3776[69], 6}, {NULL}};
const Segment SGI362675[] = {{&S3776[8], 3}, {NULL}};
const Segment SGI362676[] = {{&S3776[13], 9}, {NULL}};
const Segment SGI362677[] = {{&S3776[33], 15}, {NULL}};
const Segment SGI362678[] = {{&S3776[49], 6}, {NULL}};
const Segment SGI362679[] = {{&S3776[33], 13}, {NULL}};
const Segment SGI362680[] = {{&S3777[121], 7}, {NULL}};
const Segment SGI362681[] = {{&S3777[8], 3}, {NULL}};
const Segment SGI362682[] = {{&S3777[13], 9}, {NULL}};
const Segment SGI362683[] = {{&S3777[33], 15}, {NULL}};
const Segment SGI362684[] = {{&S3777[49], 6}, {NULL}};
const Segment SGI362685[] = {{&S3777[33], 13}, {NULL}};
const Segment SGI362686[] = {{&S3778[69], 6}, {NULL}};
const Segment SGI362687[] = {{&S3778[8], 3}, {NULL}};
const Segment SGI362688[] = {{&S3778[13], 9}, {NULL}};
const Segment SGI362689[] = {{&S3778[33], 15}, {NULL}};
const Segment SGI362690[] = {{&S3778[49], 6}, {NULL}};
const Segment SGI362691[] = {{&S3778[33], 13}, {NULL}};
const Segment SGI362692[] = {{&S3779[109], 6}, {NULL}};
const Segment SGI362693[] = {{&S3779[8], 3}, {NULL}};
const Segment SGI362694[] = {{&S3779[13], 9}, {NULL}};
const Segment SGI362695[] = {{&S3779[30], 13}, {NULL}};
const Segment SGI362696[] = {{&S3780[66], 6}, {NULL}};
const Segment SGI362697[] = {{&S3780[8], 3}, {NULL}};
const Segment SGI362698[] = {{&S3780[13], 9}, {NULL}};
const Segment SGI362699[] = {{&S3780[33], 15}, {NULL}};
const Segment SGI362700[] = {{&S3780[49], 3}, {NULL}};
const Segment SGI362701[] = {{&S3780[33], 13}, {NULL}};
const Segment SGI362702[] = {{&S3781[69], 6}, {NULL}};
const Segment SGI362703[] = {{&S3781[8], 3}, {NULL}};
const Segment SGI362704[] = {{&S3781[13], 9}, {NULL}};
const Segment SGI362705[] = {{&S3781[33], 15}, {NULL}};
const Segment SGI362706[] = {{&S3781[49], 6}, {NULL}};
const Segment SGI362707[] = {{&S3781[33], 13}, {NULL}};
const Segment SGI362708[] = {{&S3782[87], 6}, {NULL}};
const Segment SGI362709[] = {{&S3782[8], 3}, {NULL}};
const Segment SGI362710[] = {{&S3782[13], 9}, {NULL}};
const Segment SGI362711[] = {{&S3782[33], 15}, {NULL}};
const Segment SGI362712[] = {{&S3782[49], 17}, {NULL}};
const Segment SGI362713[] = {{&S3782[33], 13}, {NULL}};
const Segment SGI362714[] = {{&S3783[69], 6}, {NULL}};
const Segment SGI362715[] = {{&S3783[8], 3}, {NULL}};
const Segment SGI362716[] = {{&S3783[13], 9}, {NULL}};
const Segment SGI362717[] = {{&S3783[33], 15}, {NULL}};
const Segment SGI362718[] = {{&S3783[49], 6}, {NULL}};
const Segment SGI362719[] = {{&S3783[33], 13}, {NULL}};
const Segment SGI362720[] = {{&S3784[69], 6}, {NULL}};
const Segment SGI362721[] = {{&S3784[8], 3}, {NULL}};
const Segment SGI362722[] = {{&S3784[13], 9}, {NULL}};
const Segment SGI362723[] = {{&S3784[33], 15}, {NULL}};
const Segment SGI362724[] = {{&S3784[49], 6}, {NULL}};
const Segment SGI362725[] = {{&S3784[33], 13}, {NULL}};
const Segment SGI362726[] = {{&S3785[69], 6}, {NULL}};
const Segment SGI362727[] = {{&S3785[8], 3}, {NULL}};
const Segment SGI362728[] = {{&S3785[13], 9}, {NULL}};
const Segment SGI362729[] = {{&S3785[33], 15}, {NULL}};
const Segment SGI362730[] = {{&S3785[49], 6}, {NULL}};
const Segment SGI362731[] = {{&S3785[33], 13}, {NULL}};
const Segment SGI362732[] = {{&S3786[78], 6}, {NULL}};
const Segment SGI362733[] = {{&S3786[8], 3}, {NULL}};
const Segment SGI362734[] = {{&S3786[13], 9}, {NULL}};
const Segment SGI362735[] = {{&S3786[33], 15}, {NULL}};
const Segment SGI362736[] = {{&S3786[49], 8}, {NULL}};
const Segment SGI362737[] = {{&S3786[33], 13}, {NULL}};
const Segment SGI362738[] = {{&S3787[69], 6}, {NULL}};
const Segment SGI362739[] = {{&S3787[8], 3}, {NULL}};
const Segment SGI362740[] = {{&S3787[13], 9}, {NULL}};
const Segment SGI362741[] = {{&S3787[33], 15}, {NULL}};
const Segment SGI362742[] = {{&S3787[49], 6}, {NULL}};
const Segment SGI362743[] = {{&S3787[33], 13}, {NULL}};
const Segment SGI362744[] = {{&S3788[69], 6}, {NULL}};
const Segment SGI362745[] = {{&S3788[8], 3}, {NULL}};
const Segment SGI362746[] = {{&S3788[13], 9}, {NULL}};
const Segment SGI362747[] = {{&S3788[33], 15}, {NULL}};
const Segment SGI362748[] = {{&S3788[49], 6}, {NULL}};
const Segment SGI362749[] = {{&S3788[33], 13}, {NULL}};
const Segment SGI362750[] = {{&S3789[107], 6}, {NULL}};
const Segment SGI362751[] = {{&S3789[8], 3}, {NULL}};
const Segment SGI362752[] = {{&S3789[13], 9}, {NULL}};
const Segment SGI362753[] = {{&S3789[30], 13}, {NULL}};
const Segment SGI362754[] = {{&S3790[69], 6}, {NULL}};
const Segment SGI362755[] = {{&S3790[8], 3}, {NULL}};
const Segment SGI362756[] = {{&S3790[13], 9}, {NULL}};
const Segment SGI362757[] = {{&S3790[33], 15}, {NULL}};
const Segment SGI362758[] = {{&S3790[49], 6}, {NULL}};
const Segment SGI362759[] = {{&S3790[33], 13}, {NULL}};
const Segment SGI362760[] = {{&S3791[119], 7}, {NULL}};
const Segment SGI362761[] = {{&S3791[8], 3}, {NULL}};
const Segment SGI362762[] = {{&S3791[13], 9}, {NULL}};
const Segment SGI362763[] = {{&S3791[33], 15}, {NULL}};
const Segment SGI362764[] = {{&S3791[49], 6}, {NULL}};
const Segment SGI362765[] = {{&S3791[33], 13}, {NULL}};
const Segment SGI362766[] = {{&S3792[69], 6}, {NULL}};
const Segment SGI362767[] = {{&S3792[8], 3}, {NULL}};
const Segment SGI362768[] = {{&S3792[13], 9}, {NULL}};
const Segment SGI362769[] = {{&S3792[33], 15}, {NULL}};
const Segment SGI362770[] = {{&S3792[49], 6}, {NULL}};
const Segment SGI362771[] = {{&S3792[33], 13}, {NULL}};
const Segment SGI362772[] = {{&S3793[69], 6}, {NULL}};
const Segment SGI362773[] = {{&S3793[8], 3}, {NULL}};
const Segment SGI362774[] = {{&S3793[13], 9}, {NULL}};
const Segment SGI362775[] = {{&S3793[33], 15}, {NULL}};
const Segment SGI362776[] = {{&S3793[49], 6}, {NULL}};
const Segment SGI362777[] = {{&S3793[33], 13}, {NULL}};
const Segment SGI362778[] = {{&S3794[69], 6}, {NULL}};
const Segment SGI362779[] = {{&S3794[8], 3}, {NULL}};
const Segment SGI362780[] = {{&S3794[13], 9}, {NULL}};
const Segment SGI362781[] = {{&S3794[33], 15}, {NULL}};
const Segment SGI362782[] = {{&S3794[49], 6}, {NULL}};
const Segment SGI362783[] = {{&S3794[33], 13}, {NULL}};
const Segment SGI362784[] = {{&S3795[69], 6}, {NULL}};
const Segment SGI362785[] = {{&S3795[8], 3}, {NULL}};
const Segment SGI362786[] = {{&S3795[13], 9}, {NULL}};
const Segment SGI362787[] = {{&S3795[33], 15}, {NULL}};
const Segment SGI362788[] = {{&S3795[49], 6}, {NULL}};
const Segment SGI362789[] = {{&S3795[33], 13}, {NULL}};
const Segment SGI362790[] = {{&S3796[119], 6}, {NULL}};
const Segment SGI362791[] = {{&S3796[8], 3}, {NULL}};
const Segment SGI362792[] = {{&S3796[13], 9}, {NULL}};
const Segment SGI362793[] = {{&S3796[33], 15}, {NULL}};
const Segment SGI362794[] = {{&S3796[49], 6}, {NULL}};
const Segment SGI362795[] = {{&S3796[33], 13}, {NULL}};
const Segment SGI362796[] = {{&S3797[69], 6}, {NULL}};
const Segment SGI362797[] = {{&S3797[8], 3}, {NULL}};
const Segment SGI362798[] = {{&S3797[13], 9}, {NULL}};
const Segment SGI362799[] = {{&S3797[33], 15}, {NULL}};
const Segment SGI362800[] = {{&S3797[49], 6}, {NULL}};
const Segment SGI362801[] = {{&S3797[33], 13}, {NULL}};
const Segment SGI362802[] = {{&S3798[109], 6}, {NULL}};
const Segment SGI362803[] = {{&S3798[8], 3}, {NULL}};
const Segment SGI362804[] = {{&S3798[13], 9}, {NULL}};
const Segment SGI362805[] = {{&S3798[30], 13}, {NULL}};
const Segment SGI362806[] = {{&S3799[109], 6}, {NULL}};
const Segment SGI362807[] = {{&S3799[8], 3}, {NULL}};
const Segment SGI362808[] = {{&S3799[13], 9}, {NULL}};
const Segment SGI362809[] = {{&S3799[30], 13}, {NULL}};
const Segment SGI362810[] = {{&S3800[69], 6}, {NULL}};
const Segment SGI362811[] = {{&S3800[8], 3}, {NULL}};
const Segment SGI362812[] = {{&S3800[13], 9}, {NULL}};
const Segment SGI362813[] = {{&S3800[33], 15}, {NULL}};
const Segment SGI362814[] = {{&S3800[49], 6}, {NULL}};
const Segment SGI362815[] = {{&S3800[33], 13}, {NULL}};
const Segment SGI362816[] = {{&S3801[69], 6}, {NULL}};
const Segment SGI362817[] = {{&S3801[8], 3}, {NULL}};
const Segment SGI362818[] = {{&S3801[13], 9}, {NULL}};
const Segment SGI362819[] = {{&S3801[33], 15}, {NULL}};
const Segment SGI362820[] = {{&S3801[49], 6}, {NULL}};
const Segment SGI362821[] = {{&S3801[33], 13}, {NULL}};
const Segment SGI362822[] = {{&S3802[69], 6}, {NULL}};
const Segment SGI362823[] = {{&S3802[8], 3}, {NULL}};
const Segment SGI362824[] = {{&S3802[13], 9}, {NULL}};
const Segment SGI362825[] = {{&S3802[33], 15}, {NULL}};
const Segment SGI362826[] = {{&S3802[49], 6}, {NULL}};
const Segment SGI362827[] = {{&S3802[33], 13}, {NULL}};
const Segment SGI362828[] = {{&S3803[69], 6}, {NULL}};
const Segment SGI362829[] = {{&S3803[8], 3}, {NULL}};
const Segment SGI362830[] = {{&S3803[13], 9}, {NULL}};
const Segment SGI362831[] = {{&S3803[33], 15}, {NULL}};
const Segment SGI362832[] = {{&S3803[49], 6}, {NULL}};
const Segment SGI362833[] = {{&S3803[33], 13}, {NULL}};
const Segment SGI362834[] = {{&S3804[69], 6}, {NULL}};
const Segment SGI362835[] = {{&S3804[8], 3}, {NULL}};
const Segment SGI362836[] = {{&S3804[13], 9}, {NULL}};
const Segment SGI362837[] = {{&S3804[33], 15}, {NULL}};
const Segment SGI362838[] = {{&S3804[49], 6}, {NULL}};
const Segment SGI362839[] = {{&S3804[33], 13}, {NULL}};
const Segment SGI362840[] = {{&S3805[117], 6}, {NULL}};
const Segment SGI362841[] = {{&S3805[8], 3}, {NULL}};
const Segment SGI362842[] = {{&S3805[13], 9}, {NULL}};
const Segment SGI362843[] = {{&S3805[33], 15}, {NULL}};
const Segment SGI362844[] = {{&S3805[49], 6}, {NULL}};
const Segment SGI362845[] = {{&S3805[33], 13}, {NULL}};
const Segment SGI362846[] = {{&S3806[109], 6}, {NULL}};
const Segment SGI362847[] = {{&S3806[8], 3}, {NULL}};
const Segment SGI362848[] = {{&S3806[13], 9}, {NULL}};
const Segment SGI362849[] = {{&S3806[30], 13}, {NULL}};
const Segment SGI362850[] = {{&S3807[69], 6}, {NULL}};
const Segment SGI362851[] = {{&S3807[8], 3}, {NULL}};
const Segment SGI362852[] = {{&S3807[13], 9}, {NULL}};
const Segment SGI362853[] = {{&S3807[33], 15}, {NULL}};
const Segment SGI362854[] = {{&S3807[49], 6}, {NULL}};
const Segment SGI362855[] = {{&S3807[33], 13}, {NULL}};
const Segment SGI362856[] = {{&S3808[69], 6}, {NULL}};
const Segment SGI362857[] = {{&S3808[8], 3}, {NULL}};
const Segment SGI362858[] = {{&S3808[13], 9}, {NULL}};
const Segment SGI362859[] = {{&S3808[33], 15}, {NULL}};
const Segment SGI362860[] = {{&S3808[49], 6}, {NULL}};
const Segment SGI362861[] = {{&S3808[33], 13}, {NULL}};
const Segment SGI362862[] = {{&S3809[69], 6}, {NULL}};
const Segment SGI362863[] = {{&S3809[8], 3}, {NULL}};
const Segment SGI362864[] = {{&S3809[13], 9}, {NULL}};
const Segment SGI362865[] = {{&S3809[33], 15}, {NULL}};
const Segment SGI362866[] = {{&S3809[49], 6}, {NULL}};
const Segment SGI362867[] = {{&S3809[33], 13}, {NULL}};
const Segment SGI362868[] = {{&S3810[69], 6}, {NULL}};
const Segment SGI362869[] = {{&S3810[8], 3}, {NULL}};
const Segment SGI362870[] = {{&S3810[13], 9}, {NULL}};
const Segment SGI362871[] = {{&S3810[33], 15}, {NULL}};
const Segment SGI362872[] = {{&S3810[49], 6}, {NULL}};
const Segment SGI362873[] = {{&S3810[33], 13}, {NULL}};
const Segment SGI362874[] = {{&S3811[78], 6}, {NULL}};
const Segment SGI362875[] = {{&S3811[8], 3}, {NULL}};
const Segment SGI362876[] = {{&S3811[13], 9}, {NULL}};
const Segment SGI362877[] = {{&S3811[33], 15}, {NULL}};
const Segment SGI362878[] = {{&S3811[49], 8}, {NULL}};
const Segment SGI362879[] = {{&S3811[33], 13}, {NULL}};
const Segment SGI362880[] = {{&S3812[69], 6}, {NULL}};
const Segment SGI362881[] = {{&S3812[8], 3}, {NULL}};
const Segment SGI362882[] = {{&S3812[13], 9}, {NULL}};
const Segment SGI362883[] = {{&S3812[33], 15}, {NULL}};
const Segment SGI362884[] = {{&S3812[49], 6}, {NULL}};
const Segment SGI362885[] = {{&S3812[33], 13}, {NULL}};
const Segment SGI362886[] = {{&S3813[69], 6}, {NULL}};
const Segment SGI362887[] = {{&S3813[8], 3}, {NULL}};
const Segment SGI362888[] = {{&S3813[13], 9}, {NULL}};
const Segment SGI362889[] = {{&S3813[33], 15}, {NULL}};
const Segment SGI362890[] = {{&S3813[49], 6}, {NULL}};
const Segment SGI362891[] = {{&S3813[33], 13}, {NULL}};
const Segment SGI362892[] = {{&S3814[69], 6}, {NULL}};
const Segment SGI362893[] = {{&S3814[8], 3}, {NULL}};
const Segment SGI362894[] = {{&S3814[13], 9}, {NULL}};
const Segment SGI362895[] = {{&S3814[33], 15}, {NULL}};
const Segment SGI362896[] = {{&S3814[49], 6}, {NULL}};
const Segment SGI362897[] = {{&S3814[33], 13}, {NULL}};
const Segment SGI362898[] = {{&S3815[69], 6}, {NULL}};
const Segment SGI362899[] = {{&S3815[8], 3}, {NULL}};
const Segment SGI362900[] = {{&S3815[13], 9}, {NULL}};
const Segment SGI362901[] = {{&S3815[33], 15}, {NULL}};
const Segment SGI362902[] = {{&S3815[49], 6}, {NULL}};
const Segment SGI362903[] = {{&S3815[33], 13}, {NULL}};
const Segment SGI362904[] = {{&S3816[69], 6}, {NULL}};
const Segment SGI362905[] = {{&S3816[8], 3}, {NULL}};
const Segment SGI362906[] = {{&S3816[13], 9}, {NULL}};
const Segment SGI362907[] = {{&S3816[33], 15}, {NULL}};
const Segment SGI362908[] = {{&S3816[49], 6}, {NULL}};
const Segment SGI362909[] = {{&S3816[33], 13}, {NULL}};
const Segment SGI362910[] = {{&S3817[69], 6}, {NULL}};
const Segment SGI362911[] = {{&S3817[8], 3}, {NULL}};
const Segment SGI362912[] = {{&S3817[13], 9}, {NULL}};
const Segment SGI362913[] = {{&S3817[33], 15}, {NULL}};
const Segment SGI362914[] = {{&S3817[49], 6}, {NULL}};
const Segment SGI362915[] = {{&S3817[33], 13}, {NULL}};
const Segment SGI362916[] = {{&S3818[69], 6}, {NULL}};
const Segment SGI362917[] = {{&S3818[8], 3}, {NULL}};
const Segment SGI362918[] = {{&S3818[13], 9}, {NULL}};
const Segment SGI362919[] = {{&S3818[33], 15}, {NULL}};
const Segment SGI362920[] = {{&S3818[49], 6}, {NULL}};
const Segment SGI362921[] = {{&S3818[33], 13}, {NULL}};
const Segment SGI362922[] = {{&S3819[69], 6}, {NULL}};
const Segment SGI362923[] = {{&S3819[8], 3}, {NULL}};
const Segment SGI362924[] = {{&S3819[13], 9}, {NULL}};
const Segment SGI362925[] = {{&S3819[33], 15}, {NULL}};
const Segment SGI362926[] = {{&S3819[49], 6}, {NULL}};
const Segment SGI362927[] = {{&S3819[33], 13}, {NULL}};
const Segment SGI362928[] = {{&S3820[69], 6}, {NULL}};
const Segment SGI362929[] = {{&S3820[8], 3}, {NULL}};
const Segment SGI362930[] = {{&S3820[13], 9}, {NULL}};
const Segment SGI362931[] = {{&S3820[33], 15}, {NULL}};
const Segment SGI362932[] = {{&S3820[49], 6}, {NULL}};
const Segment SGI362933[] = {{&S3820[33], 13}, {NULL}};
const Segment SGI362934[] = {{&S3821[69], 6}, {NULL}};
const Segment SGI362935[] = {{&S3821[8], 3}, {NULL}};
const Segment SGI362936[] = {{&S3821[13], 9}, {NULL}};
const Segment SGI362937[] = {{&S3821[33], 15}, {NULL}};
const Segment SGI362938[] = {{&S3821[49], 6}, {NULL}};
const Segment SGI362939[] = {{&S3821[33], 13}, {NULL}};
const Segment SGI362940[] = {{&S3822[69], 6}, {NULL}};
const Segment SGI362941[] = {{&S3822[8], 3}, {NULL}};
const Segment SGI362942[] = {{&S3822[13], 9}, {NULL}};
const Segment SGI362943[] = {{&S3822[33], 15}, {NULL}};
const Segment SGI362944[] = {{&S3822[49], 6}, {NULL}};
const Segment SGI362945[] = {{&S3822[33], 13}, {NULL}};
const Segment SGI362946[] = {{&S3823[66], 6}, {NULL}};
const Segment SGI362947[] = {{&S3823[8], 3}, {NULL}};
const Segment SGI362948[] = {{&S3823[13], 9}, {NULL}};
const Segment SGI362949[] = {{&S3823[33], 15}, {NULL}};
const Segment SGI362950[] = {{&S3823[49], 3}, {NULL}};
const Segment SGI362951[] = {{&S3823[33], 13}, {NULL}};
const Segment SGI362952[] = {{&S3824[69], 6}, {NULL}};
const Segment SGI362953[] = {{&S3824[8], 3}, {NULL}};
const Segment SGI362954[] = {{&S3824[13], 9}, {NULL}};
const Segment SGI362955[] = {{&S3824[33], 15}, {NULL}};
const Segment SGI362956[] = {{&S3824[49], 6}, {NULL}};
const Segment SGI362957[] = {{&S3824[33], 13}, {NULL}};
const Segment SGI362958[] = {{&S3825[69], 6}, {NULL}};
const Segment SGI362959[] = {{&S3825[8], 3}, {NULL}};
const Segment SGI362960[] = {{&S3825[13], 9}, {NULL}};
const Segment SGI362961[] = {{&S3825[33], 15}, {NULL}};
const Segment SGI362962[] = {{&S3825[49], 6}, {NULL}};
const Segment SGI362963[] = {{&S3825[33], 13}, {NULL}};
const Segment SGI362964[] = {{&S3826[69], 6}, {NULL}};
const Segment SGI362965[] = {{&S3826[8], 3}, {NULL}};
const Segment SGI362966[] = {{&S3826[13], 9}, {NULL}};
const Segment SGI362967[] = {{&S3826[33], 15}, {NULL}};
const Segment SGI362968[] = {{&S3826[49], 6}, {NULL}};
const Segment SGI362969[] = {{&S3826[33], 13}, {NULL}};
const Segment SGI362970[] = {{&S3827[69], 6}, {NULL}};
const Segment SGI362971[] = {{&S3827[8], 3}, {NULL}};
const Segment SGI362972[] = {{&S3827[13], 9}, {NULL}};
const Segment SGI362973[] = {{&S3827[33], 15}, {NULL}};
const Segment SGI362974[] = {{&S3827[49], 6}, {NULL}};
const Segment SGI362975[] = {{&S3827[33], 13}, {NULL}};
const Segment SGI362976[] = {{&S3828[109], 6}, {NULL}};
const Segment SGI362977[] = {{&S3828[8], 3}, {NULL}};
const Segment SGI362978[] = {{&S3828[13], 9}, {NULL}};
const Segment SGI362979[] = {{&S3828[30], 13}, {NULL}};
const Segment SGI362980[] = {{&S3829[69], 6}, {NULL}};
const Segment SGI362981[] = {{&S3829[8], 3}, {NULL}};
const Segment SGI362982[] = {{&S3829[13], 9}, {NULL}};
const Segment SGI362983[] = {{&S3829[33], 15}, {NULL}};
const Segment SGI362984[] = {{&S3829[49], 6}, {NULL}};
const Segment SGI362985[] = {{&S3829[33], 13}, {NULL}};
const Segment SGI362986[] = {{&S3830[81], 6}, {NULL}};
const Segment SGI362987[] = {{&S3830[8], 3}, {NULL}};
const Segment SGI362988[] = {{&S3830[13], 9}, {NULL}};
const Segment SGI362989[] = {{&S3830[33], 15}, {NULL}};
const Segment SGI362990[] = {{&S3830[49], 11}, {NULL}};
const Segment SGI362991[] = {{&S3830[33], 13}, {NULL}};
const Segment SGI362992[] = {{&S3831[69], 6}, {NULL}};
const Segment SGI362993[] = {{&S3831[8], 3}, {NULL}};
const Segment SGI362994[] = {{&S3831[13], 9}, {NULL}};
const Segment SGI362995[] = {{&S3831[33], 15}, {NULL}};
const Segment SGI362996[] = {{&S3831[49], 6}, {NULL}};
const Segment SGI362997[] = {{&S3831[33], 13}, {NULL}};
const Segment SGI362998[] = {{&S3832[69], 6}, {NULL}};
const Segment SGI362999[] = {{&S3832[8], 3}, {NULL}};
const Segment SGI363000[] = {{&S3832[13], 9}, {NULL}};
const Segment SGI363001[] = {{&S3832[33], 15}, {NULL}};
const Segment SGI363002[] = {{&S3832[49], 6}, {NULL}};
const Segment SGI363003[] = {{&S3832[33], 13}, {NULL}};
const Segment SGI363004[] = {{&S3833[66], 6}, {NULL}};
const Segment SGI363005[] = {{&S3833[8], 3}, {NULL}};
const Segment SGI363006[] = {{&S3833[13], 9}, {NULL}};
const Segment SGI363007[] = {{&S3833[33], 15}, {NULL}};
const Segment SGI363008[] = {{&S3833[49], 3}, {NULL}};
const Segment SGI363009[] = {{&S3833[33], 13}, {NULL}};
const Segment SGI363010[] = {{&S3834[66], 6}, {NULL}};
const Segment SGI363011[] = {{&S3834[8], 3}, {NULL}};
const Segment SGI363012[] = {{&S3834[13], 9}, {NULL}};
const Segment SGI363013[] = {{&S3834[33], 15}, {NULL}};
const Segment SGI363014[] = {{&S3834[49], 3}, {NULL}};
const Segment SGI363015[] = {{&S3834[33], 13}, {NULL}};
const Segment SGI363016[] = {{&S3835[69], 6}, {NULL}};
const Segment SGI363017[] = {{&S3835[8], 3}, {NULL}};
const Segment SGI363018[] = {{&S3835[13], 9}, {NULL}};
const Segment SGI363019[] = {{&S3835[33], 15}, {NULL}};
const Segment SGI363020[] = {{&S3835[49], 6}, {NULL}};
const Segment SGI363021[] = {{&S3835[33], 13}, {NULL}};
const Segment SGI363022[] = {{&S3836[109], 6}, {NULL}};
const Segment SGI363023[] = {{&S3836[8], 3}, {NULL}};
const Segment SGI363024[] = {{&S3836[13], 9}, {NULL}};
const Segment SGI363025[] = {{&S3836[30], 13}, {NULL}};
const Segment SGI363026[] = {{&S3837[69], 6}, {NULL}};
const Segment SGI363027[] = {{&S3837[8], 3}, {NULL}};
const Segment SGI363028[] = {{&S3837[13], 9}, {NULL}};
const Segment SGI363029[] = {{&S3837[33], 15}, {NULL}};
const Segment SGI363030[] = {{&S3837[49], 6}, {NULL}};
const Segment SGI363031[] = {{&S3837[33], 13}, {NULL}};
const Segment SGI363032[] = {{&S3838[69], 6}, {NULL}};
const Segment SGI363033[] = {{&S3838[8], 3}, {NULL}};
const Segment SGI363034[] = {{&S3838[13], 9}, {NULL}};
const Segment SGI363035[] = {{&S3838[33], 15}, {NULL}};
const Segment SGI363036[] = {{&S3838[49], 6}, {NULL}};
const Segment SGI363037[] = {{&S3838[33], 13}, {NULL}};
const Segment SGI363038[] = {{&S3839[69], 6}, {NULL}};
const Segment SGI363039[] = {{&S3839[8], 3}, {NULL}};
const Segment SGI363040[] = {{&S3839[13], 9}, {NULL}};
const Segment SGI363041[] = {{&S3839[33], 15}, {NULL}};
const Segment SGI363042[] = {{&S3839[49], 6}, {NULL}};
const Segment SGI363043[] = {{&S3839[33], 13}, {NULL}};
const Segment SGI363044[] = {{&S3840[69], 6}, {NULL}};
const Segment SGI363045[] = {{&S3840[8], 3}, {NULL}};
const Segment SGI363046[] = {{&S3840[13], 9}, {NULL}};
const Segment SGI363047[] = {{&S3840[33], 15}, {NULL}};
const Segment SGI363048[] = {{&S3840[49], 6}, {NULL}};
const Segment SGI363049[] = {{&S3840[33], 13}, {NULL}};
const Segment SGI363050[] = {{&S3841[69], 6}, {NULL}};
const Segment SGI363051[] = {{&S3841[8], 3}, {NULL}};
const Segment SGI363052[] = {{&S3841[13], 9}, {NULL}};
const Segment SGI363053[] = {{&S3841[33], 15}, {NULL}};
const Segment SGI363054[] = {{&S3841[49], 6}, {NULL}};
const Segment SGI363055[] = {{&S3841[33], 13}, {NULL}};
const Segment SGI363056[] = {{&S3842[69], 6}, {NULL}};
const Segment SGI363057[] = {{&S3842[8], 3}, {NULL}};
const Segment SGI363058[] = {{&S3842[13], 9}, {NULL}};
const Segment SGI363059[] = {{&S3842[33], 15}, {NULL}};
const Segment SGI363060[] = {{&S3842[49], 6}, {NULL}};
const Segment SGI363061[] = {{&S3842[33], 13}, {NULL}};
const Segment SGI363062[] = {{&S3843[69], 6}, {NULL}};
const Segment SGI363063[] = {{&S3843[8], 3}, {NULL}};
const Segment SGI363064[] = {{&S3843[13], 9}, {NULL}};
const Segment SGI363065[] = {{&S3843[33], 15}, {NULL}};
const Segment SGI363066[] = {{&S3843[49], 6}, {NULL}};
const Segment SGI363067[] = {{&S3843[33], 13}, {NULL}};
const Segment SGI363068[] = {{&S3844[69], 6}, {NULL}};
const Segment SGI363069[] = {{&S3844[8], 3}, {NULL}};
const Segment SGI363070[] = {{&S3844[13], 9}, {NULL}};
const Segment SGI363071[] = {{&S3844[33], 15}, {NULL}};
const Segment SGI363072[] = {{&S3844[49], 6}, {NULL}};
const Segment SGI363073[] = {{&S3844[33], 13}, {NULL}};
const Segment SGI363074[] = {{&S3845[119], 6}, {NULL}};
const Segment SGI363075[] = {{&S3845[8], 3}, {NULL}};
const Segment SGI363076[] = {{&S3845[13], 9}, {NULL}};
const Segment SGI363077[] = {{&S3845[33], 15}, {NULL}};
const Segment SGI363078[] = {{&S3845[49], 6}, {NULL}};
const Segment SGI363079[] = {{&S3845[33], 13}, {NULL}};
const Segment SGI363080[] = {{&S3846[119], 6}, {NULL}};
const Segment SGI363081[] = {{&S3846[8], 3}, {NULL}};
const Segment SGI363082[] = {{&S3846[13], 9}, {NULL}};
const Segment SGI363083[] = {{&S3846[33], 15}, {NULL}};
const Segment SGI363084[] = {{&S3846[49], 6}, {NULL}};
const Segment SGI363085[] = {{&S3846[33], 13}, {NULL}};
const Segment SGI363086[] = {{&S3847[69], 6}, {NULL}};
const Segment SGI363087[] = {{&S3847[8], 3}, {NULL}};
const Segment SGI363088[] = {{&S3847[13], 9}, {NULL}};
const Segment SGI363089[] = {{&S3847[33], 15}, {NULL}};
const Segment SGI363090[] = {{&S3847[49], 6}, {NULL}};
const Segment SGI363091[] = {{&S3847[33], 13}, {NULL}};
const Segment SGI363092[] = {{&S3848[69], 6}, {NULL}};
const Segment SGI363093[] = {{&S3848[8], 3}, {NULL}};
const Segment SGI363094[] = {{&S3848[13], 9}, {NULL}};
const Segment SGI363095[] = {{&S3848[33], 15}, {NULL}};
const Segment SGI363096[] = {{&S3848[49], 6}, {NULL}};
const Segment SGI363097[] = {{&S3848[33], 13}, {NULL}};
const Segment SGI363098[] = {{&S3849[69], 6}, {NULL}};
const Segment SGI363099[] = {{&S3849[8], 3}, {NULL}};
const Segment SGI363100[] = {{&S3849[13], 9}, {NULL}};
const Segment SGI363101[] = {{&S3849[33], 15}, {NULL}};
const Segment SGI363102[] = {{&S3849[49], 6}, {NULL}};
const Segment SGI363103[] = {{&S3849[33], 13}, {NULL}};
const Segment SGI363104[] = {{&S3850[121], 7}, {NULL}};
const Segment SGI363105[] = {{&S3850[8], 3}, {NULL}};
const Segment SGI363106[] = {{&S3850[13], 9}, {NULL}};
const Segment SGI363107[] = {{&S3850[33], 15}, {NULL}};
const Segment SGI363108[] = {{&S3850[49], 6}, {NULL}};
const Segment SGI363109[] = {{&S3850[33], 13}, {NULL}};
const Segment SGI363110[] = {{&S3851[69], 6}, {NULL}};
const Segment SGI363111[] = {{&S3851[8], 3}, {NULL}};
const Segment SGI363112[] = {{&S3851[13], 9}, {NULL}};
const Segment SGI363113[] = {{&S3851[33], 15}, {NULL}};
const Segment SGI363114[] = {{&S3851[49], 6}, {NULL}};
const Segment SGI363115[] = {{&S3851[33], 13}, {NULL}};
const Segment SGI363116[] = {{&S3852[69], 6}, {NULL}};
const Segment SGI363117[] = {{&S3852[8], 3}, {NULL}};
const Segment SGI363118[] = {{&S3852[13], 9}, {NULL}};
const Segment SGI363119[] = {{&S3852[33], 15}, {NULL}};
const Segment SGI363120[] = {{&S3852[49], 6}, {NULL}};
const Segment SGI363121[] = {{&S3852[33], 13}, {NULL}};
const Segment SGI363122[] = {{&S3853[69], 6}, {NULL}};
const Segment SGI363123[] = {{&S3853[8], 3}, {NULL}};
const Segment SGI363124[] = {{&S3853[13], 9}, {NULL}};
const Segment SGI363125[] = {{&S3853[33], 15}, {NULL}};
const Segment SGI363126[] = {{&S3853[49], 6}, {NULL}};
const Segment SGI363127[] = {{&S3853[33], 13}, {NULL}};
const Segment SGI363128[] = {{&S3854[69], 6}, {NULL}};
const Segment SGI363129[] = {{&S3854[8], 3}, {NULL}};
const Segment SGI363130[] = {{&S3854[13], 9}, {NULL}};
const Segment SGI363131[] = {{&S3854[33], 15}, {NULL}};
const Segment SGI363132[] = {{&S3854[49], 6}, {NULL}};
const Segment SGI363133[] = {{&S3854[33], 13}, {NULL}};
const Segment SGI363134[] = {{&S3855[69], 6}, {NULL}};
const Segment SGI363135[] = {{&S3855[8], 3}, {NULL}};
const Segment SGI363136[] = {{&S3855[13], 9}, {NULL}};
const Segment SGI363137[] = {{&S3855[33], 15}, {NULL}};
const Segment SGI363138[] = {{&S3855[49], 6}, {NULL}};
const Segment SGI363139[] = {{&S3855[33], 13}, {NULL}};
const Segment SGI363140[] = {{&S3856[69], 6}, {NULL}};
const Segment SGI363141[] = {{&S3856[8], 3}, {NULL}};
const Segment SGI363142[] = {{&S3856[13], 9}, {NULL}};
const Segment SGI363143[] = {{&S3856[33], 15}, {NULL}};
const Segment SGI363144[] = {{&S3856[49], 6}, {NULL}};
const Segment SGI363145[] = {{&S3856[33], 13}, {NULL}};
const Segment SGI363146[] = {{&S3857[69], 6}, {NULL}};
const Segment SGI363147[] = {{&S3857[8], 3}, {NULL}};
const Segment SGI363148[] = {{&S3857[13], 9}, {NULL}};
const Segment SGI363149[] = {{&S3857[33], 15}, {NULL}};
const Segment SGI363150[] = {{&S3857[49], 6}, {NULL}};
const Segment SGI363151[] = {{&S3857[33], 13}, {NULL}};
const Segment SGI363152[] = {{&S3858[69], 6}, {NULL}};
const Segment SGI363153[] = {{&S3858[8], 3}, {NULL}};
const Segment SGI363154[] = {{&S3858[13], 9}, {NULL}};
const Segment SGI363155[] = {{&S3858[33], 15}, {NULL}};
const Segment SGI363156[] = {{&S3858[49], 6}, {NULL}};
const Segment SGI363157[] = {{&S3858[33], 13}, {NULL}};
const Segment SGI363158[] = {{&S3859[119], 6}, {NULL}};
const Segment SGI363159[] = {{&S3859[8], 3}, {NULL}};
const Segment SGI363160[] = {{&S3859[13], 9}, {NULL}};
const Segment SGI363161[] = {{&S3859[33], 15}, {NULL}};
const Segment SGI363162[] = {{&S3859[49], 6}, {NULL}};
const Segment SGI363163[] = {{&S3859[33], 13}, {NULL}};
const Segment SGI363164[] = {{&S3860[69], 6}, {NULL}};
const Segment SGI363165[] = {{&S3860[8], 3}, {NULL}};
const Segment SGI363166[] = {{&S3860[13], 9}, {NULL}};
const Segment SGI363167[] = {{&S3860[33], 15}, {NULL}};
const Segment SGI363168[] = {{&S3860[49], 6}, {NULL}};
const Segment SGI363169[] = {{&S3860[33], 13}, {NULL}};
const Segment SGI363170[] = {{&S3861[69], 6}, {NULL}};
const Segment SGI363171[] = {{&S3861[8], 3}, {NULL}};
const Segment SGI363172[] = {{&S3861[13], 9}, {NULL}};
const Segment SGI363173[] = {{&S3861[33], 15}, {NULL}};
const Segment SGI363174[] = {{&S3861[49], 6}, {NULL}};
const Segment SGI363175[] = {{&S3861[33], 13}, {NULL}};
const Segment SGI363176[] = {{&S3862[69], 5}, {NULL}};
const Segment SGI363177[] = {{&S3862[8], 3}, {NULL}};
const Segment SGI363178[] = {{&S3862[13], 9}, {NULL}};
const Segment SGI363179[] = {{&S3862[33], 15}, {NULL}};
const Segment SGI363180[] = {{&S3862[49], 6}, {NULL}};
const Segment SGI363181[] = {{&S3862[33], 13}, {NULL}};
const Segment SGI363182[] = {{&S3863[69], 6}, {NULL}};
const Segment SGI363183[] = {{&S3863[8], 3}, {NULL}};
const Segment SGI363184[] = {{&S3863[13], 9}, {NULL}};
const Segment SGI363185[] = {{&S3863[33], 15}, {NULL}};
const Segment SGI363186[] = {{&S3863[49], 6}, {NULL}};
const Segment SGI363187[] = {{&S3863[33], 13}, {NULL}};
const Segment SGI363188[] = {{&S3864[69], 6}, {NULL}};
const Segment SGI363189[] = {{&S3864[8], 3}, {NULL}};
const Segment SGI363190[] = {{&S3864[13], 9}, {NULL}};
const Segment SGI363191[] = {{&S3864[33], 15}, {NULL}};
const Segment SGI363192[] = {{&S3864[49], 6}, {NULL}};
const Segment SGI363193[] = {{&S3864[33], 13}, {NULL}};
const Segment SGI363194[] = {{&S3865[69], 6}, {NULL}};
const Segment SGI363195[] = {{&S3865[8], 3}, {NULL}};
const Segment SGI363196[] = {{&S3865[13], 9}, {NULL}};
const Segment SGI363197[] = {{&S3865[33], 15}, {NULL}};
const Segment SGI363198[] = {{&S3865[49], 6}, {NULL}};
const Segment SGI363199[] = {{&S3865[33], 13}, {NULL}};
const Segment SGI363200[] = {{&S3866[69], 6}, {NULL}};
const Segment SGI363201[] = {{&S3866[8], 3}, {NULL}};
const Segment SGI363202[] = {{&S3866[13], 9}, {NULL}};
const Segment SGI363203[] = {{&S3866[33], 15}, {NULL}};
const Segment SGI363204[] = {{&S3866[49], 6}, {NULL}};
const Segment SGI363205[] = {{&S3866[33], 13}, {NULL}};
const Segment SGI363206[] = {{&S3867[69], 6}, {NULL}};
const Segment SGI363207[] = {{&S3867[8], 3}, {NULL}};
const Segment SGI363208[] = {{&S3867[13], 9}, {NULL}};
const Segment SGI363209[] = {{&S3867[33], 15}, {NULL}};
const Segment SGI363210[] = {{&S3867[49], 6}, {NULL}};
const Segment SGI363211[] = {{&S3867[33], 13}, {NULL}};
const Segment SGI363212[] = {{&S3868[69], 6}, {NULL}};
const Segment SGI363213[] = {{&S3868[8], 3}, {NULL}};
const Segment SGI363214[] = {{&S3868[13], 9}, {NULL}};
const Segment SGI363215[] = {{&S3868[33], 15}, {NULL}};
const Segment SGI363216[] = {{&S3868[49], 6}, {NULL}};
const Segment SGI363217[] = {{&S3868[33], 13}, {NULL}};
const Segment SGI363218[] = {{&S3869[119], 6}, {NULL}};
const Segment SGI363219[] = {{&S3869[8], 3}, {NULL}};
const Segment SGI363220[] = {{&S3869[13], 9}, {NULL}};
const Segment SGI363221[] = {{&S3869[33], 15}, {NULL}};
const Segment SGI363222[] = {{&S3869[49], 6}, {NULL}};
const Segment SGI363223[] = {{&S3869[33], 13}, {NULL}};
const Segment SGI363224[] = {{&S3870[69], 6}, {NULL}};
const Segment SGI363225[] = {{&S3870[8], 3}, {NULL}};
const Segment SGI363226[] = {{&S3870[13], 9}, {NULL}};
const Segment SGI363227[] = {{&S3870[33], 15}, {NULL}};
const Segment SGI363228[] = {{&S3870[49], 6}, {NULL}};
const Segment SGI363229[] = {{&S3870[33], 13}, {NULL}};
const Segment SGI363230[] = {{&S3871[69], 6}, {NULL}};
const Segment SGI363231[] = {{&S3871[8], 3}, {NULL}};
const Segment SGI363232[] = {{&S3871[13], 9}, {NULL}};
const Segment SGI363233[] = {{&S3871[33], 15}, {NULL}};
const Segment SGI363234[] = {{&S3871[49], 6}, {NULL}};
const Segment SGI363235[] = {{&S3871[33], 13}, {NULL}};
const Segment SGI363236[] = {{&S3872[69], 6}, {NULL}};
const Segment SGI363237[] = {{&S3872[8], 3}, {NULL}};
const Segment SGI363238[] = {{&S3872[13], 9}, {NULL}};
const Segment SGI363239[] = {{&S3872[33], 15}, {NULL}};
const Segment SGI363240[] = {{&S3872[49], 6}, {NULL}};
const Segment SGI363241[] = {{&S3872[33], 13}, {NULL}};
const Segment SGI363242[] = {{&S3873[69], 6}, {NULL}};
const Segment SGI363243[] = {{&S3873[8], 3}, {NULL}};
const Segment SGI363244[] = {{&S3873[13], 9}, {NULL}};
const Segment SGI363245[] = {{&S3873[33], 15}, {NULL}};
const Segment SGI363246[] = {{&S3873[49], 6}, {NULL}};
const Segment SGI363247[] = {{&S3873[33], 13}, {NULL}};
const Segment SGI363248[] = {{&S3874[69], 6}, {NULL}};
const Segment SGI363249[] = {{&S3874[8], 3}, {NULL}};
const Segment SGI363250[] = {{&S3874[13], 9}, {NULL}};
const Segment SGI363251[] = {{&S3874[33], 15}, {NULL}};
const Segment SGI363252[] = {{&S3874[49], 6}, {NULL}};
const Segment SGI363253[] = {{&S3874[33], 13}, {NULL}};
const Segment SGI363254[] = {{&S3875[69], 6}, {NULL}};
const Segment SGI363255[] = {{&S3875[8], 3}, {NULL}};
const Segment SGI363256[] = {{&S3875[13], 9}, {NULL}};
const Segment SGI363257[] = {{&S3875[33], 15}, {NULL}};
const Segment SGI363258[] = {{&S3875[49], 6}, {NULL}};
const Segment SGI363259[] = {{&S3875[33], 13}, {NULL}};
const Segment SGI363260[] = {{&S3876[69], 6}, {NULL}};
const Segment SGI363261[] = {{&S3876[8], 3}, {NULL}};
const Segment SGI363262[] = {{&S3876[13], 9}, {NULL}};
const Segment SGI363263[] = {{&S3876[33], 15}, {NULL}};
const Segment SGI363264[] = {{&S3876[49], 6}, {NULL}};
const Segment SGI363265[] = {{&S3876[33], 13}, {NULL}};
const Segment SGI363266[] = {{&S3877[69], 6}, {NULL}};
const Segment SGI363267[] = {{&S3877[8], 3}, {NULL}};
const Segment SGI363268[] = {{&S3877[13], 9}, {NULL}};
const Segment SGI363269[] = {{&S3877[33], 15}, {NULL}};
const Segment SGI363270[] = {{&S3877[49], 6}, {NULL}};
const Segment SGI363271[] = {{&S3877[33], 13}, {NULL}};
const Segment SGI363272[] = {{&S3878[69], 6}, {NULL}};
const Segment SGI363273[] = {{&S3878[8], 3}, {NULL}};
const Segment SGI363274[] = {{&S3878[13], 9}, {NULL}};
const Segment SGI363275[] = {{&S3878[33], 15}, {NULL}};
const Segment SGI363276[] = {{&S3878[49], 6}, {NULL}};
const Segment SGI363277[] = {{&S3878[33], 13}, {NULL}};
const Segment SGI363278[] = {{&S3879[66], 6}, {NULL}};
const Segment SGI363279[] = {{&S3879[8], 3}, {NULL}};
const Segment SGI363280[] = {{&S3879[13], 9}, {NULL}};
const Segment SGI363281[] = {{&S3879[33], 15}, {NULL}};
const Segment SGI363282[] = {{&S3879[49], 3}, {NULL}};
const Segment SGI363283[] = {{&S3879[33], 13}, {NULL}};
const Segment SGI363284[] = {{&S3880[119], 6}, {NULL}};
const Segment SGI363285[] = {{&S3880[8], 3}, {NULL}};
const Segment SGI363286[] = {{&S3880[13], 9}, {NULL}};
const Segment SGI363287[] = {{&S3880[33], 15}, {NULL}};
const Segment SGI363288[] = {{&S3880[49], 6}, {NULL}};
const Segment SGI363289[] = {{&S3880[33], 13}, {NULL}};
const Segment SGI363290[] = {{&S3881[69], 6}, {NULL}};
const Segment SGI363291[] = {{&S3881[8], 3}, {NULL}};
const Segment SGI363292[] = {{&S3881[13], 9}, {NULL}};
const Segment SGI363293[] = {{&S3881[33], 15}, {NULL}};
const Segment SGI363294[] = {{&S3881[49], 6}, {NULL}};
const Segment SGI363295[] = {{&S3881[33], 13}, {NULL}};
const Segment SGI363296[] = {{&S3882[69], 6}, {NULL}};
const Segment SGI363297[] = {{&S3882[8], 3}, {NULL}};
const Segment SGI363298[] = {{&S3882[13], 9}, {NULL}};
const Segment SGI363299[] = {{&S3882[33], 15}, {NULL}};
const Segment SGI363300[] = {{&S3882[49], 6}, {NULL}};
const Segment SGI363301[] = {{&S3882[33], 13}, {NULL}};
const Segment SGI363302[] = {{&S3883[69], 6}, {NULL}};
const Segment SGI363303[] = {{&S3883[8], 3}, {NULL}};
const Segment SGI363304[] = {{&S3883[13], 9}, {NULL}};
const Segment SGI363305[] = {{&S3883[33], 15}, {NULL}};
const Segment SGI363306[] = {{&S3883[49], 6}, {NULL}};
const Segment SGI363307[] = {{&S3883[33], 13}, {NULL}};
const Segment SGI363308[] = {{&S3884[69], 6}, {NULL}};
const Segment SGI363309[] = {{&S3884[8], 3}, {NULL}};
const Segment SGI363310[] = {{&S3884[13], 9}, {NULL}};
const Segment SGI363311[] = {{&S3884[33], 15}, {NULL}};
const Segment SGI363312[] = {{&S3884[49], 6}, {NULL}};
const Segment SGI363313[] = {{&S3884[33], 13}, {NULL}};
const Segment SGI363314[] = {{&S3885[81], 6}, {NULL}};
const Segment SGI363315[] = {{&S3885[8], 3}, {NULL}};
const Segment SGI363316[] = {{&S3885[13], 9}, {NULL}};
const Segment SGI363317[] = {{&S3885[33], 15}, {NULL}};
const Segment SGI363318[] = {{&S3885[49], 11}, {NULL}};
const Segment SGI363319[] = {{&S3885[33], 13}, {NULL}};
const Segment SGI363320[] = {{&S3886[69], 6}, {NULL}};
const Segment SGI363321[] = {{&S3886[8], 3}, {NULL}};
const Segment SGI363322[] = {{&S3886[13], 9}, {NULL}};
const Segment SGI363323[] = {{&S3886[33], 15}, {NULL}};
const Segment SGI363324[] = {{&S3886[49], 6}, {NULL}};
const Segment SGI363325[] = {{&S3886[33], 13}, {NULL}};
const Segment SGI363326[] = {{&S3887[69], 6}, {NULL}};
const Segment SGI363327[] = {{&S3887[8], 3}, {NULL}};
const Segment SGI363328[] = {{&S3887[13], 9}, {NULL}};
const Segment SGI363329[] = {{&S3887[33], 15}, {NULL}};
const Segment SGI363330[] = {{&S3887[49], 6}, {NULL}};
const Segment SGI363331[] = {{&S3887[33], 13}, {NULL}};
const Segment SGI363332[] = {{&S3888[69], 6}, {NULL}};
const Segment SGI363333[] = {{&S3888[8], 3}, {NULL}};
const Segment SGI363334[] = {{&S3888[13], 9}, {NULL}};
const Segment SGI363335[] = {{&S3888[33], 15}, {NULL}};
const Segment SGI363336[] = {{&S3888[49], 6}, {NULL}};
const Segment SGI363337[] = {{&S3888[33], 13}, {NULL}};
const Segment SGI363338[] = {{&S3889[66], 6}, {NULL}};
const Segment SGI363339[] = {{&S3889[8], 3}, {NULL}};
const Segment SGI363340[] = {{&S3889[13], 9}, {NULL}};
const Segment SGI363341[] = {{&S3889[33], 15}, {NULL}};
const Segment SGI363342[] = {{&S3889[49], 3}, {NULL}};
const Segment SGI363343[] = {{&S3889[33], 13}, {NULL}};
const Segment SGI363344[] = {{&S3890[69], 6}, {NULL}};
const Segment SGI363345[] = {{&S3890[8], 3}, {NULL}};
const Segment SGI363346[] = {{&S3890[13], 9}, {NULL}};
const Segment SGI363347[] = {{&S3890[33], 15}, {NULL}};
const Segment SGI363348[] = {{&S3890[49], 6}, {NULL}};
const Segment SGI363349[] = {{&S3890[33], 13}, {NULL}};
const Segment SGI363350[] = {{&S3891[119], 6}, {NULL}};
const Segment SGI363351[] = {{&S3891[8], 3}, {NULL}};
const Segment SGI363352[] = {{&S3891[13], 9}, {NULL}};
const Segment SGI363353[] = {{&S3891[33], 15}, {NULL}};
const Segment SGI363354[] = {{&S3891[49], 6}, {NULL}};
const Segment SGI363355[] = {{&S3891[33], 13}, {NULL}};
const Segment SGI363356[] = {{&S3892[69], 6}, {NULL}};
const Segment SGI363357[] = {{&S3892[8], 3}, {NULL}};
const Segment SGI363358[] = {{&S3892[13], 9}, {NULL}};
const Segment SGI363359[] = {{&S3892[33], 15}, {NULL}};
const Segment SGI363360[] = {{&S3892[49], 6}, {NULL}};
const Segment SGI363361[] = {{&S3892[33], 13}, {NULL}};
const Segment SGI363362[] = {{&S3893[71], 6}, {NULL}};
const Segment SGI363363[] = {{&S3893[8], 3}, {NULL}};
const Segment SGI363364[] = {{&S3893[13], 9}, {NULL}};
const Segment SGI363365[] = {{&S3893[33], 15}, {NULL}};
const Segment SGI363366[] = {{&S3893[49], 8}, {NULL}};
const Segment SGI363367[] = {{&S3893[33], 13}, {NULL}};
const Segment SGI363368[] = {{&S3894[66], 6}, {NULL}};
const Segment SGI363369[] = {{&S3894[8], 3}, {NULL}};
const Segment SGI363370[] = {{&S3894[13], 9}, {NULL}};
const Segment SGI363371[] = {{&S3894[33], 15}, {NULL}};
const Segment SGI363372[] = {{&S3894[49], 3}, {NULL}};
const Segment SGI363373[] = {{&S3894[33], 13}, {NULL}};
const Segment SGI363374[] = {{&S3895[69], 6}, {NULL}};
const Segment SGI363375[] = {{&S3895[8], 3}, {NULL}};
const Segment SGI363376[] = {{&S3895[13], 9}, {NULL}};
const Segment SGI363377[] = {{&S3895[33], 15}, {NULL}};
const Segment SGI363378[] = {{&S3895[49], 6}, {NULL}};
const Segment SGI363379[] = {{&S3895[33], 13}, {NULL}};
const Segment SGI363380[] = {{&S3896[69], 6}, {NULL}};
const Segment SGI363381[] = {{&S3896[8], 3}, {NULL}};
const Segment SGI363382[] = {{&S3896[13], 9}, {NULL}};
const Segment SGI363383[] = {{&S3896[33], 15}, {NULL}};
const Segment SGI363384[] = {{&S3896[49], 6}, {NULL}};
const Segment SGI363385[] = {{&S3896[33], 13}, {NULL}};
const Segment SGI363386[] = {{&S3897[69], 6}, {NULL}};
const Segment SGI363387[] = {{&S3897[8], 3}, {NULL}};
const Segment SGI363388[] = {{&S3897[13], 9}, {NULL}};
const Segment SGI363389[] = {{&S3897[33], 15}, {NULL}};
const Segment SGI363390[] = {{&S3897[49], 6}, {NULL}};
const Segment SGI363391[] = {{&S3897[33], 13}, {NULL}};
const Segment SGI363392[] = {{&S3898[69], 6}, {NULL}};
const Segment SGI363393[] = {{&S3898[8], 3}, {NULL}};
const Segment SGI363394[] = {{&S3898[13], 9}, {NULL}};
const Segment SGI363395[] = {{&S3898[33], 15}, {NULL}};
const Segment SGI363396[] = {{&S3898[49], 6}, {NULL}};
const Segment SGI363397[] = {{&S3898[33], 13}, {NULL}};
const Segment SGI363398[] = {{&S3899[69], 6}, {NULL}};
const Segment SGI363399[] = {{&S3899[8], 3}, {NULL}};
const Segment SGI363400[] = {{&S3899[13], 9}, {NULL}};
const Segment SGI363401[] = {{&S3899[33], 15}, {NULL}};
const Segment SGI363402[] = {{&S3899[49], 6}, {NULL}};
const Segment SGI363403[] = {{&S3899[33], 13}, {NULL}};
const Segment SGI363404[] = {{&S3900[69], 6}, {NULL}};
const Segment SGI363405[] = {{&S3900[8], 3}, {NULL}};
const Segment SGI363406[] = {{&S3900[13], 9}, {NULL}};
const Segment SGI363407[] = {{&S3900[33], 15}, {NULL}};
const Segment SGI363408[] = {{&S3900[49], 6}, {NULL}};
const Segment SGI363409[] = {{&S3900[33], 13}, {NULL}};
const Segment SGI363410[] = {{&S3901[69], 6}, {NULL}};
const Segment SGI363411[] = {{&S3901[8], 3}, {NULL}};
const Segment SGI363412[] = {{&S3901[13], 9}, {NULL}};
const Segment SGI363413[] = {{&S3901[33], 15}, {NULL}};
const Segment SGI363414[] = {{&S3901[49], 6}, {NULL}};
const Segment SGI363415[] = {{&S3901[33], 13}, {NULL}};
const Segment SGI363416[] = {{&S3902[69], 6}, {NULL}};
const Segment SGI363417[] = {{&S3902[8], 3}, {NULL}};
const Segment SGI363418[] = {{&S3902[13], 9}, {NULL}};
const Segment SGI363419[] = {{&S3902[33], 15}, {NULL}};
const Segment SGI363420[] = {{&S3902[49], 6}, {NULL}};
const Segment SGI363421[] = {{&S3902[33], 13}, {NULL}};
const Segment SGI363422[] = {{&S3903[69], 6}, {NULL}};
const Segment SGI363423[] = {{&S3903[8], 3}, {NULL}};
const Segment SGI363424[] = {{&S3903[13], 9}, {NULL}};
const Segment SGI363425[] = {{&S3903[33], 15}, {NULL}};
const Segment SGI363426[] = {{&S3903[49], 6}, {NULL}};
const Segment SGI363427[] = {{&S3903[33], 13}, {NULL}};
const Segment SGI363428[] = {{&S3904[69], 6}, {NULL}};
const Segment SGI363429[] = {{&S3904[8], 3}, {NULL}};
const Segment SGI363430[] = {{&S3904[13], 9}, {NULL}};
const Segment SGI363431[] = {{&S3904[33], 15}, {NULL}};
const Segment SGI363432[] = {{&S3904[49], 6}, {NULL}};
const Segment SGI363433[] = {{&S3904[33], 13}, {NULL}};
const Segment SGI363434[] = {{&S3905[69], 6}, {NULL}};
const Segment SGI363435[] = {{&S3905[8], 3}, {NULL}};
const Segment SGI363436[] = {{&S3905[13], 9}, {NULL}};
const Segment SGI363437[] = {{&S3905[33], 15}, {NULL}};
const Segment SGI363438[] = {{&S3905[49], 6}, {NULL}};
const Segment SGI363439[] = {{&S3905[33], 13}, {NULL}};
const Segment SGI363440[] = {{&S3906[69], 6}, {NULL}};
const Segment SGI363441[] = {{&S3906[8], 3}, {NULL}};
const Segment SGI363442[] = {{&S3906[13], 9}, {NULL}};
const Segment SGI363443[] = {{&S3906[33], 15}, {NULL}};
const Segment SGI363444[] = {{&S3906[49], 6}, {NULL}};
const Segment SGI363445[] = {{&S3906[33], 13}, {NULL}};
const Segment SGI363446[] = {{&S3907[66], 6}, {NULL}};
const Segment SGI363447[] = {{&S3907[8], 3}, {NULL}};
const Segment SGI363448[] = {{&S3907[13], 9}, {NULL}};
const Segment SGI363449[] = {{&S3907[33], 15}, {NULL}};
const Segment SGI363450[] = {{&S3907[49], 3}, {NULL}};
const Segment SGI363451[] = {{&S3907[33], 13}, {NULL}};
const Segment SGI363452[] = {{&S3908[69], 6}, {NULL}};
const Segment SGI363453[] = {{&S3908[8], 3}, {NULL}};
const Segment SGI363454[] = {{&S3908[13], 9}, {NULL}};
const Segment SGI363455[] = {{&S3908[33], 15}, {NULL}};
const Segment SGI363456[] = {{&S3908[49], 6}, {NULL}};
const Segment SGI363457[] = {{&S3908[33], 13}, {NULL}};
const Segment SGI363458[] = {{&S3909[69], 5}, {NULL}};
const Segment SGI363459[] = {{&S3909[8], 3}, {NULL}};
const Segment SGI363460[] = {{&S3909[13], 9}, {NULL}};
const Segment SGI363461[] = {{&S3909[33], 15}, {NULL}};
const Segment SGI363462[] = {{&S3909[49], 6}, {NULL}};
const Segment SGI363463[] = {{&S3909[33], 13}, {NULL}};
const Segment SGI363464[] = {{&S3910[66], 6}, {NULL}};
const Segment SGI363465[] = {{&S3910[8], 3}, {NULL}};
const Segment SGI363466[] = {{&S3910[13], 9}, {NULL}};
const Segment SGI363467[] = {{&S3910[33], 15}, {NULL}};
const Segment SGI363468[] = {{&S3910[49], 3}, {NULL}};
const Segment SGI363469[] = {{&S3910[33], 13}, {NULL}};
const Segment SGI363470[] = {{&S3911[118], 6}, {NULL}};
const Segment SGI363471[] = {{&S3911[8], 3}, {NULL}};
const Segment SGI363472[] = {{&S3911[13], 9}, {NULL}};
const Segment SGI363473[] = {{&S3911[33], 15}, {NULL}};
const Segment SGI363474[] = {{&S3911[49], 6}, {NULL}};
const Segment SGI363475[] = {{&S3911[33], 13}, {NULL}};
const Segment SGI363476[] = {{&S3912[69], 7}, {NULL}};
const Segment SGI363477[] = {{&S3912[8], 3}, {NULL}};
const Segment SGI363478[] = {{&S3912[13], 9}, {NULL}};
const Segment SGI363479[] = {{&S3912[33], 15}, {NULL}};
const Segment SGI363480[] = {{&S3912[49], 6}, {NULL}};
const Segment SGI363481[] = {{&S3912[33], 13}, {NULL}};
const Segment SGI363482[] = {{&S3913[69], 7}, {NULL}};
const Segment SGI363483[] = {{&S3913[8], 3}, {NULL}};
const Segment SGI363484[] = {{&S3913[13], 9}, {NULL}};
const Segment SGI363485[] = {{&S3913[33], 15}, {NULL}};
const Segment SGI363486[] = {{&S3913[49], 6}, {NULL}};
const Segment SGI363487[] = {{&S3913[33], 13}, {NULL}};
const Segment SGI363488[] = {{&S3914[69], 7}, {NULL}};
const Segment SGI363489[] = {{&S3914[8], 3}, {NULL}};
const Segment SGI363490[] = {{&S3914[13], 9}, {NULL}};
const Segment SGI363491[] = {{&S3914[33], 15}, {NULL}};
const Segment SGI363492[] = {{&S3914[49], 6}, {NULL}};
const Segment SGI363493[] = {{&S3914[33], 13}, {NULL}};
const Segment SGI363494[] = {{&S3915[69], 7}, {NULL}};
const Segment SGI363495[] = {{&S3915[8], 3}, {NULL}};
const Segment SGI363496[] = {{&S3915[13], 9}, {NULL}};
const Segment SGI363497[] = {{&S3915[33], 15}, {NULL}};
const Segment SGI363498[] = {{&S3915[49], 6}, {NULL}};
const Segment SGI363499[] = {{&S3915[33], 13}, {NULL}};
const Segment SGI363500[] = {{&S3916[69], 7}, {NULL}};
const Segment SGI363501[] = {{&S3916[8], 3}, {NULL}};
const Segment SGI363502[] = {{&S3916[13], 9}, {NULL}};
const Segment SGI363503[] = {{&S3916[33], 15}, {NULL}};
const Segment SGI363504[] = {{&S3916[49], 6}, {NULL}};
const Segment SGI363505[] = {{&S3916[33], 13}, {NULL}};
const Segment SGI363506[] = {{&S3917[69], 7}, {NULL}};
const Segment SGI363507[] = {{&S3917[8], 3}, {NULL}};
const Segment SGI363508[] = {{&S3917[13], 9}, {NULL}};
const Segment SGI363509[] = {{&S3917[33], 15}, {NULL}};
const Segment SGI363510[] = {{&S3917[49], 6}, {NULL}};
const Segment SGI363511[] = {{&S3917[33], 13}, {NULL}};
const Segment SGI363512[] = {{&S3918[69], 6}, {NULL}};
const Segment SGI363513[] = {{&S3918[8], 3}, {NULL}};
const Segment SGI363514[] = {{&S3918[13], 9}, {NULL}};
const Segment SGI363515[] = {{&S3918[33], 15}, {NULL}};
const Segment SGI363516[] = {{&S3918[49], 6}, {NULL}};
const Segment SGI363517[] = {{&S3918[33], 13}, {NULL}};
const Segment SGI363518[] = {{&S3919[69], 6}, {NULL}};
const Segment SGI363519[] = {{&S3919[8], 3}, {NULL}};
const Segment SGI363520[] = {{&S3919[13], 9}, {NULL}};
const Segment SGI363521[] = {{&S3919[33], 15}, {NULL}};
const Segment SGI363522[] = {{&S3919[49], 6}, {NULL}};
const Segment SGI363523[] = {{&S3919[33], 13}, {NULL}};
const Segment SGI363524[] = {{&S3920[69], 6}, {NULL}};
const Segment SGI363525[] = {{&S3920[8], 3}, {NULL}};
const Segment SGI363526[] = {{&S3920[13], 9}, {NULL}};
const Segment SGI363527[] = {{&S3920[33], 15}, {NULL}};
const Segment SGI363528[] = {{&S3920[49], 6}, {NULL}};
const Segment SGI363529[] = {{&S3920[33], 13}, {NULL}};
const Segment SGI363530[] = {{&S3921[69], 6}, {NULL}};
const Segment SGI363531[] = {{&S3921[8], 3}, {NULL}};
const Segment SGI363532[] = {{&S3921[13], 9}, {NULL}};
const Segment SGI363533[] = {{&S3921[33], 15}, {NULL}};
const Segment SGI363534[] = {{&S3921[49], 6}, {NULL}};
const Segment SGI363535[] = {{&S3921[33], 13}, {NULL}};
const Segment SGI363536[] = {{&S3922[69], 6}, {NULL}};
const Segment SGI363537[] = {{&S3922[8], 3}, {NULL}};
const Segment SGI363538[] = {{&S3922[13], 9}, {NULL}};
const Segment SGI363539[] = {{&S3922[33], 15}, {NULL}};
const Segment SGI363540[] = {{&S3922[49], 6}, {NULL}};
const Segment SGI363541[] = {{&S3922[33], 13}, {NULL}};
const Segment SGI363542[] = {{&S3923[69], 6}, {NULL}};
const Segment SGI363543[] = {{&S3923[8], 3}, {NULL}};
const Segment SGI363544[] = {{&S3923[13], 9}, {NULL}};
const Segment SGI363545[] = {{&S3923[33], 15}, {NULL}};
const Segment SGI363546[] = {{&S3923[49], 6}, {NULL}};
const Segment SGI363547[] = {{&S3923[33], 13}, {NULL}};
const Segment SGI363548[] = {{&S3924[69], 6}, {NULL}};
const Segment SGI363549[] = {{&S3924[8], 3}, {NULL}};
const Segment SGI363550[] = {{&S3924[13], 9}, {NULL}};
const Segment SGI363551[] = {{&S3924[33], 15}, {NULL}};
const Segment SGI363552[] = {{&S3924[49], 6}, {NULL}};
const Segment SGI363553[] = {{&S3924[33], 13}, {NULL}};
const Segment SGI363554[] = {{&S3925[69], 6}, {NULL}};
const Segment SGI363555[] = {{&S3925[8], 3}, {NULL}};
const Segment SGI363556[] = {{&S3925[13], 9}, {NULL}};
const Segment SGI363557[] = {{&S3925[33], 15}, {NULL}};
const Segment SGI363558[] = {{&S3925[49], 6}, {NULL}};
const Segment SGI363559[] = {{&S3925[33], 13}, {NULL}};
const Segment SGI363560[] = {{&S3926[69], 6}, {NULL}};
const Segment SGI363561[] = {{&S3926[8], 3}, {NULL}};
const Segment SGI363562[] = {{&S3926[13], 9}, {NULL}};
const Segment SGI363563[] = {{&S3926[33], 15}, {NULL}};
const Segment SGI363564[] = {{&S3926[49], 6}, {NULL}};
const Segment SGI363565[] = {{&S3926[33], 13}, {NULL}};
const Segment SGI363566[] = {{&S3927[69], 6}, {NULL}};
const Segment SGI363567[] = {{&S3927[8], 3}, {NULL}};
const Segment SGI363568[] = {{&S3927[13], 9}, {NULL}};
const Segment SGI363569[] = {{&S3927[33], 15}, {NULL}};
const Segment SGI363570[] = {{&S3927[49], 6}, {NULL}};
const Segment SGI363571[] = {{&S3927[33], 13}, {NULL}};
const Segment SGI363572[] = {{&S3928[69], 6}, {NULL}};
const Segment SGI363573[] = {{&S3928[8], 3}, {NULL}};
const Segment SGI363574[] = {{&S3928[13], 9}, {NULL}};
const Segment SGI363575[] = {{&S3928[33], 15}, {NULL}};
const Segment SGI363576[] = {{&S3928[49], 6}, {NULL}};
const Segment SGI363577[] = {{&S3928[33], 13}, {NULL}};
const Segment SGI363578[] = {{&S3929[69], 6}, {NULL}};
const Segment SGI363579[] = {{&S3929[8], 3}, {NULL}};
const Segment SGI363580[] = {{&S3929[13], 9}, {NULL}};
const Segment SGI363581[] = {{&S3929[33], 15}, {NULL}};
const Segment SGI363582[] = {{&S3929[49], 6}, {NULL}};
const Segment SGI363583[] = {{&S3929[33], 13}, {NULL}};
const Segment SGI363584[] = {{&S3930[69], 6}, {NULL}};
const Segment SGI363585[] = {{&S3930[8], 3}, {NULL}};
const Segment SGI363586[] = {{&S3930[13], 9}, {NULL}};
const Segment SGI363587[] = {{&S3930[33], 15}, {NULL}};
const Segment SGI363588[] = {{&S3930[49], 6}, {NULL}};
const Segment SGI363589[] = {{&S3930[33], 13}, {NULL}};
const Segment SGI363590[] = {{&S3931[69], 6}, {NULL}};
const Segment SGI363591[] = {{&S3931[8], 3}, {NULL}};
const Segment SGI363592[] = {{&S3931[13], 9}, {NULL}};
const Segment SGI363593[] = {{&S3931[33], 15}, {NULL}};
const Segment SGI363594[] = {{&S3931[49], 6}, {NULL}};
const Segment SGI363595[] = {{&S3931[33], 13}, {NULL}};
const Segment SGI363596[] = {{&S3932[69], 6}, {NULL}};
const Segment SGI363597[] = {{&S3932[8], 3}, {NULL}};
const Segment SGI363598[] = {{&S3932[13], 9}, {NULL}};
const Segment SGI363599[] = {{&S3932[33], 15}, {NULL}};
const Segment SGI363600[] = {{&S3932[49], 6}, {NULL}};
const Segment SGI363601[] = {{&S3932[33], 13}, {NULL}};
const Segment SGI363602[] = {{&S3933[66], 6}, {NULL}};
const Segment SGI363603[] = {{&S3933[8], 3}, {NULL}};
const Segment SGI363604[] = {{&S3933[13], 9}, {NULL}};
const Segment SGI363605[] = {{&S3933[33], 15}, {NULL}};
const Segment SGI363606[] = {{&S3933[49], 3}, {NULL}};
const Segment SGI363607[] = {{&S3933[33], 13}, {NULL}};
const Segment SGI363608[] = {{&S3934[66], 6}, {NULL}};
const Segment SGI363609[] = {{&S3934[8], 3}, {NULL}};
const Segment SGI363610[] = {{&S3934[13], 9}, {NULL}};
const Segment SGI363611[] = {{&S3934[33], 15}, {NULL}};
const Segment SGI363612[] = {{&S3934[49], 3}, {NULL}};
const Segment SGI363613[] = {{&S3934[33], 13}, {NULL}};
const Segment SGI363614[] = {{&S3935[69], 6}, {NULL}};
const Segment SGI363615[] = {{&S3935[8], 3}, {NULL}};
const Segment SGI363616[] = {{&S3935[13], 9}, {NULL}};
const Segment SGI363617[] = {{&S3935[33], 15}, {NULL}};
const Segment SGI363618[] = {{&S3935[49], 6}, {NULL}};
const Segment SGI363619[] = {{&S3935[33], 13}, {NULL}};
const Segment SGI363620[] = {{&S3936[69], 6}, {NULL}};
const Segment SGI363621[] = {{&S3936[8], 3}, {NULL}};
const Segment SGI363622[] = {{&S3936[13], 9}, {NULL}};
const Segment SGI363623[] = {{&S3936[33], 15}, {NULL}};
const Segment SGI363624[] = {{&S3936[49], 6}, {NULL}};
const Segment SGI363625[] = {{&S3936[33], 13}, {NULL}};
const Segment SGI363626[] = {{&S3937[69], 6}, {NULL}};
const Segment SGI363627[] = {{&S3937[8], 3}, {NULL}};
const Segment SGI363628[] = {{&S3937[13], 9}, {NULL}};
const Segment SGI363629[] = {{&S3937[33], 15}, {NULL}};
const Segment SGI363630[] = {{&S3937[49], 6}, {NULL}};
const Segment SGI363631[] = {{&S3937[33], 13}, {NULL}};
const Segment SGI363632[] = {{&S3938[66], 6}, {NULL}};
const Segment SGI363633[] = {{&S3938[8], 3}, {NULL}};
const Segment SGI363634[] = {{&S3938[13], 9}, {NULL}};
const Segment SGI363635[] = {{&S3938[33], 15}, {NULL}};
const Segment SGI363636[] = {{&S3938[49], 3}, {NULL}};
const Segment SGI363637[] = {{&S3938[33], 13}, {NULL}};
const Segment SGI363638[] = {{&S3939[69], 6}, {NULL}};
const Segment SGI363639[] = {{&S3939[8], 3}, {NULL}};
const Segment SGI363640[] = {{&S3939[13], 9}, {NULL}};
const Segment SGI363641[] = {{&S3939[33], 15}, {NULL}};
const Segment SGI363642[] = {{&S3939[49], 6}, {NULL}};
const Segment SGI363643[] = {{&S3939[33], 13}, {NULL}};
const Segment SGI363644[] = {{&S3940[69], 6}, {NULL}};
const Segment SGI363645[] = {{&S3940[8], 3}, {NULL}};
const Segment SGI363646[] = {{&S3940[13], 9}, {NULL}};
const Segment SGI363647[] = {{&S3940[33], 15}, {NULL}};
const Segment SGI363648[] = {{&S3940[49], 6}, {NULL}};
const Segment SGI363649[] = {{&S3940[33], 13}, {NULL}};
const Segment SGI363650[] = {{&S3941[69], 6}, {NULL}};
const Segment SGI363651[] = {{&S3941[8], 3}, {NULL}};
const Segment SGI363652[] = {{&S3941[13], 9}, {NULL}};
const Segment SGI363653[] = {{&S3941[33], 15}, {NULL}};
const Segment SGI363654[] = {{&S3941[49], 6}, {NULL}};
const Segment SGI363655[] = {{&S3941[33], 13}, {NULL}};
const Segment SGI363656[] = {{&S3942[69], 6}, {NULL}};
const Segment SGI363657[] = {{&S3942[8], 3}, {NULL}};
const Segment SGI363658[] = {{&S3942[13], 9}, {NULL}};
const Segment SGI363659[] = {{&S3942[33], 15}, {NULL}};
const Segment SGI363660[] = {{&S3942[49], 6}, {NULL}};
const Segment SGI363661[] = {{&S3942[33], 13}, {NULL}};
const Segment SGI363662[] = {{&S3943[69], 6}, {NULL}};
const Segment SGI363663[] = {{&S3943[8], 3}, {NULL}};
const Segment SGI363664[] = {{&S3943[13], 9}, {NULL}};
const Segment SGI363665[] = {{&S3943[33], 15}, {NULL}};
const Segment SGI363666[] = {{&S3943[49], 6}, {NULL}};
const Segment SGI363667[] = {{&S3943[33], 13}, {NULL}};
const Segment SGI363668[] = {{&S3944[69], 6}, {NULL}};
const Segment SGI363669[] = {{&S3944[8], 3}, {NULL}};
const Segment SGI363670[] = {{&S3944[13], 9}, {NULL}};
const Segment SGI363671[] = {{&S3944[33], 15}, {NULL}};
const Segment SGI363672[] = {{&S3944[49], 6}, {NULL}};
const Segment SGI363673[] = {{&S3944[33], 13}, {NULL}};
const Segment SGI363674[] = {{&S3945[69], 6}, {NULL}};
const Segment SGI363675[] = {{&S3945[8], 3}, {NULL}};
const Segment SGI363676[] = {{&S3945[13], 9}, {NULL}};
const Segment SGI363677[] = {{&S3945[33], 15}, {NULL}};
const Segment SGI363678[] = {{&S3945[49], 6}, {NULL}};
const Segment SGI363679[] = {{&S3945[33], 13}, {NULL}};
const Segment SGI363680[] = {{&S3946[69], 6}, {NULL}};
const Segment SGI363681[] = {{&S3946[8], 3}, {NULL}};
const Segment SGI363682[] = {{&S3946[13], 9}, {NULL}};
const Segment SGI363683[] = {{&S3946[33], 15}, {NULL}};
const Segment SGI363684[] = {{&S3946[49], 6}, {NULL}};
const Segment SGI363685[] = {{&S3946[33], 13}, {NULL}};
const Segment SGI363686[] = {{&S3947[69], 6}, {NULL}};
const Segment SGI363687[] = {{&S3947[8], 3}, {NULL}};
const Segment SGI363688[] = {{&S3947[13], 9}, {NULL}};
const Segment SGI363689[] = {{&S3947[33], 15}, {NULL}};
const Segment SGI363690[] = {{&S3947[49], 6}, {NULL}};
const Segment SGI363691[] = {{&S3947[33], 13}, {NULL}};
const Segment SGI363692[] = {{&S3948[69], 5}, {NULL}};
const Segment SGI363693[] = {{&S3948[8], 3}, {NULL}};
const Segment SGI363694[] = {{&S3948[13], 9}, {NULL}};
const Segment SGI363695[] = {{&S3948[33], 15}, {NULL}};
const Segment SGI363696[] = {{&S3948[49], 6}, {NULL}};
const Segment SGI363697[] = {{&S3948[33], 13}, {NULL}};
const Segment SGI363698[] = {{&S3949[66], 6}, {NULL}};
const Segment SGI363699[] = {{&S3949[8], 3}, {NULL}};
const Segment SGI363700[] = {{&S3949[13], 9}, {NULL}};
const Segment SGI363701[] = {{&S3949[33], 15}, {NULL}};
const Segment SGI363702[] = {{&S3949[49], 3}, {NULL}};
const Segment SGI363703[] = {{&S3949[33], 13}, {NULL}};
const Segment SGI363704[] = {{&S3950[66], 6}, {NULL}};
const Segment SGI363705[] = {{&S3950[8], 3}, {NULL}};
const Segment SGI363706[] = {{&S3950[13], 9}, {NULL}};
const Segment SGI363707[] = {{&S3950[33], 15}, {NULL}};
const Segment SGI363708[] = {{&S3950[49], 3}, {NULL}};
const Segment SGI363709[] = {{&S3950[33], 13}, {NULL}};
const Segment SGI363710[] = {{&S3951[69], 6}, {NULL}};
const Segment SGI363711[] = {{&S3951[8], 3}, {NULL}};
const Segment SGI363712[] = {{&S3951[13], 9}, {NULL}};
const Segment SGI363713[] = {{&S3951[33], 15}, {NULL}};
const Segment SGI363714[] = {{&S3951[49], 6}, {NULL}};
const Segment SGI363715[] = {{&S3951[33], 13}, {NULL}};
const Segment SGI363716[] = {{&S3952[66], 6}, {NULL}};
const Segment SGI363717[] = {{&S3952[8], 3}, {NULL}};
const Segment SGI363718[] = {{&S3952[13], 9}, {NULL}};
const Segment SGI363719[] = {{&S3952[33], 15}, {NULL}};
const Segment SGI363720[] = {{&S3952[49], 3}, {NULL}};
const Segment SGI363721[] = {{&S3952[33], 13}, {NULL}};
const Segment SGI363722[] = {{&S3953[66], 6}, {NULL}};
const Segment SGI363723[] = {{&S3953[8], 3}, {NULL}};
const Segment SGI363724[] = {{&S3953[13], 9}, {NULL}};
const Segment SGI363725[] = {{&S3953[33], 15}, {NULL}};
const Segment SGI363726[] = {{&S3953[49], 3}, {NULL}};
const Segment SGI363727[] = {{&S3953[33], 13}, {NULL}};
const Segment SGI363728[] = {{&S3954[69], 6}, {NULL}};
const Segment SGI363729[] = {{&S3954[8], 3}, {NULL}};
const Segment SGI363730[] = {{&S3954[13], 9}, {NULL}};
const Segment SGI363731[] = {{&S3954[33], 15}, {NULL}};
const Segment SGI363732[] = {{&S3954[49], 6}, {NULL}};
const Segment SGI363733[] = {{&S3954[33], 13}, {NULL}};
const Segment SGI363734[] = {{&S3955[121], 7}, {NULL}};
const Segment SGI363735[] = {{&S3955[8], 3}, {NULL}};
const Segment SGI363736[] = {{&S3955[13], 9}, {NULL}};
const Segment SGI363737[] = {{&S3955[33], 15}, {NULL}};
const Segment SGI363738[] = {{&S3955[49], 6}, {NULL}};
const Segment SGI363739[] = {{&S3955[33], 13}, {NULL}};
const Segment SGI363740[] = {{&S3956[66], 6}, {NULL}};
const Segment SGI363741[] = {{&S3956[8], 3}, {NULL}};
const Segment SGI363742[] = {{&S3956[13], 9}, {NULL}};
const Segment SGI363743[] = {{&S3956[33], 15}, {NULL}};
const Segment SGI363744[] = {{&S3956[49], 3}, {NULL}};
const Segment SGI363745[] = {{&S3956[33], 13}, {NULL}};
const Segment SGI363746[] = {{&S3957[69], 6}, {NULL}};
const Segment SGI363747[] = {{&S3957[8], 3}, {NULL}};
const Segment SGI363748[] = {{&S3957[13], 9}, {NULL}};
const Segment SGI363749[] = {{&S3957[33], 15}, {NULL}};
const Segment SGI363750[] = {{&S3957[49], 6}, {NULL}};
const Segment SGI363751[] = {{&S3957[33], 13}, {NULL}};
const Segment SGI363752[] = {{&S3958[117], 6}, {NULL}};
const Segment SGI363753[] = {{&S3958[8], 3}, {NULL}};
const Segment SGI363754[] = {{&S3958[13], 9}, {NULL}};
const Segment SGI363755[] = {{&S3958[33], 15}, {NULL}};
const Segment SGI363756[] = {{&S3958[49], 6}, {NULL}};
const Segment SGI363757[] = {{&S3958[33], 13}, {NULL}};
const Segment SGI363758[] = {{&S3959[121], 7}, {NULL}};
const Segment SGI363759[] = {{&S3959[8], 3}, {NULL}};
const Segment SGI363760[] = {{&S3959[13], 9}, {NULL}};
const Segment SGI363761[] = {{&S3959[33], 15}, {NULL}};
const Segment SGI363762[] = {{&S3959[49], 6}, {NULL}};
const Segment SGI363763[] = {{&S3959[33], 13}, {NULL}};
const Segment SGI363764[] = {{&S3960[66], 6}, {NULL}};
const Segment SGI363765[] = {{&S3960[8], 3}, {NULL}};
const Segment SGI363766[] = {{&S3960[13], 9}, {NULL}};
const Segment SGI363767[] = {{&S3960[33], 15}, {NULL}};
const Segment SGI363768[] = {{&S3960[49], 3}, {NULL}};
const Segment SGI363769[] = {{&S3960[33], 13}, {NULL}};
const Segment SGI363770[] = {{&S3961[117], 6}, {NULL}};
const Segment SGI363771[] = {{&S3961[8], 3}, {NULL}};
const Segment SGI363772[] = {{&S3961[13], 9}, {NULL}};
const Segment SGI363773[] = {{&S3961[33], 15}, {NULL}};
const Segment SGI363774[] = {{&S3961[49], 6}, {NULL}};
const Segment SGI363775[] = {{&S3961[33], 13}, {NULL}};
const Segment SGI363776[] = {{&S3962[69], 6}, {NULL}};
const Segment SGI363777[] = {{&S3962[8], 3}, {NULL}};
const Segment SGI363778[] = {{&S3962[13], 9}, {NULL}};
const Segment SGI363779[] = {{&S3962[33], 15}, {NULL}};
const Segment SGI363780[] = {{&S3962[49], 6}, {NULL}};
const Segment SGI363781[] = {{&S3962[33], 13}, {NULL}};
const Segment SGI363782[] = {{&S3963[69], 6}, {NULL}};
const Segment SGI363783[] = {{&S3963[8], 3}, {NULL}};
const Segment SGI363784[] = {{&S3963[13], 9}, {NULL}};
const Segment SGI363785[] = {{&S3963[33], 15}, {NULL}};
const Segment SGI363786[] = {{&S3963[49], 6}, {NULL}};
const Segment SGI363787[] = {{&S3963[33], 13}, {NULL}};
const Segment SGI363788[] = {{&S3964[69], 6}, {NULL}};
const Segment SGI363789[] = {{&S3964[8], 3}, {NULL}};
const Segment SGI363790[] = {{&S3964[13], 9}, {NULL}};
const Segment SGI363791[] = {{&S3964[33], 15}, {NULL}};
const Segment SGI363792[] = {{&S3964[49], 6}, {NULL}};
const Segment SGI363793[] = {{&S3964[33], 13}, {NULL}};
const Segment SGI363794[] = {{&S3965[69], 6}, {NULL}};
const Segment SGI363795[] = {{&S3965[8], 3}, {NULL}};
const Segment SGI363796[] = {{&S3965[13], 9}, {NULL}};
const Segment SGI363797[] = {{&S3965[33], 15}, {NULL}};
const Segment SGI363798[] = {{&S3965[49], 6}, {NULL}};
const Segment SGI363799[] = {{&S3965[33], 13}, {NULL}};
const Segment SGI363800[] = {{&S3966[69], 6}, {NULL}};
const Segment SGI363801[] = {{&S3966[8], 3}, {NULL}};
const Segment SGI363802[] = {{&S3966[13], 9}, {NULL}};
const Segment SGI363803[] = {{&S3966[33], 15}, {NULL}};
const Segment SGI363804[] = {{&S3966[49], 6}, {NULL}};
const Segment SGI363805[] = {{&S3966[33], 13}, {NULL}};
const Segment SGI363806[] = {{&S3967[69], 7}, {NULL}};
const Segment SGI363807[] = {{&S3967[8], 3}, {NULL}};
const Segment SGI363808[] = {{&S3967[13], 9}, {NULL}};
const Segment SGI363809[] = {{&S3967[33], 15}, {NULL}};
const Segment SGI363810[] = {{&S3967[49], 6}, {NULL}};
const Segment SGI363811[] = {{&S3967[33], 13}, {NULL}};
const Segment SGI363812[] = {{&S3968[69], 6}, {NULL}};
const Segment SGI363813[] = {{&S3968[8], 3}, {NULL}};
const Segment SGI363814[] = {{&S3968[13], 9}, {NULL}};
const Segment SGI363815[] = {{&S3968[33], 15}, {NULL}};
const Segment SGI363816[] = {{&S3968[49], 6}, {NULL}};
const Segment SGI363817[] = {{&S3968[33], 13}, {NULL}};
const Segment SGI363818[] = {{&S3969[69], 6}, {NULL}};
const Segment SGI363819[] = {{&S3969[8], 3}, {NULL}};
const Segment SGI363820[] = {{&S3969[13], 9}, {NULL}};
const Segment SGI363821[] = {{&S3969[33], 15}, {NULL}};
const Segment SGI363822[] = {{&S3969[49], 6}, {NULL}};
const Segment SGI363823[] = {{&S3969[33], 13}, {NULL}};
const Segment SGI363824[] = {{&S3970[69], 6}, {NULL}};
const Segment SGI363825[] = {{&S3970[8], 3}, {NULL}};
const Segment SGI363826[] = {{&S3970[13], 9}, {NULL}};
const Segment SGI363827[] = {{&S3970[33], 15}, {NULL}};
const Segment SGI363828[] = {{&S3970[49], 6}, {NULL}};
const Segment SGI363829[] = {{&S3970[33], 13}, {NULL}};
const Segment SGI363830[] = {{&S3971[69], 6}, {NULL}};
const Segment SGI363831[] = {{&S3971[8], 3}, {NULL}};
const Segment SGI363832[] = {{&S3971[13], 9}, {NULL}};
const Segment SGI363833[] = {{&S3971[33], 15}, {NULL}};
const Segment SGI363834[] = {{&S3971[49], 6}, {NULL}};
const Segment SGI363835[] = {{&S3971[33], 13}, {NULL}};
const Segment SGI363836[] = {{&S3972[82], 6}, {NULL}};
const Segment SGI363837[] = {{&S3972[8], 3}, {NULL}};
const Segment SGI363838[] = {{&S3972[13], 9}, {NULL}};
const Segment SGI363839[] = {{&S3972[33], 15}, {NULL}};
const Segment SGI363840[] = {{&S3972[49], 12}, {NULL}};
const Segment SGI363841[] = {{&S3972[33], 13}, {NULL}};
const Segment SGI363842[] = {{&S3973[69], 6}, {NULL}};
const Segment SGI363843[] = {{&S3973[8], 3}, {NULL}};
const Segment SGI363844[] = {{&S3973[13], 9}, {NULL}};
const Segment SGI363845[] = {{&S3973[33], 15}, {NULL}};
const Segment SGI363846[] = {{&S3973[49], 6}, {NULL}};
const Segment SGI363847[] = {{&S3973[33], 13}, {NULL}};
const Segment SGI363848[] = {{&S3974[80], 6}, {NULL}};
const Segment SGI363849[] = {{&S3974[8], 3}, {NULL}};
const Segment SGI363850[] = {{&S3974[13], 9}, {NULL}};
const Segment SGI363851[] = {{&S3974[33], 15}, {NULL}};
const Segment SGI363852[] = {{&S3974[49], 10}, {NULL}};
const Segment SGI363853[] = {{&S3974[33], 13}, {NULL}};
const Segment SGI363854[] = {{&S3975[69], 6}, {NULL}};
const Segment SGI363855[] = {{&S3975[8], 3}, {NULL}};
const Segment SGI363856[] = {{&S3975[13], 9}, {NULL}};
const Segment SGI363857[] = {{&S3975[33], 15}, {NULL}};
const Segment SGI363858[] = {{&S3975[49], 6}, {NULL}};
const Segment SGI363859[] = {{&S3975[33], 13}, {NULL}};
const Segment SGI363860[] = {{&S3976[69], 6}, {NULL}};
const Segment SGI363861[] = {{&S3976[8], 3}, {NULL}};
const Segment SGI363862[] = {{&S3976[13], 9}, {NULL}};
const Segment SGI363863[] = {{&S3976[33], 15}, {NULL}};
const Segment SGI363864[] = {{&S3976[49], 6}, {NULL}};
const Segment SGI363865[] = {{&S3976[33], 13}, {NULL}};
const Segment SGI363866[] = {{&S3977[69], 6}, {NULL}};
const Segment SGI363867[] = {{&S3977[8], 3}, {NULL}};
const Segment SGI363868[] = {{&S3977[13], 9}, {NULL}};
const Segment SGI363869[] = {{&S3977[33], 15}, {NULL}};
const Segment SGI363870[] = {{&S3977[49], 6}, {NULL}};
const Segment SGI363871[] = {{&S3977[33], 13}, {NULL}};
const Segment SGI363872[] = {{&S3978[69], 6}, {NULL}};
const Segment SGI363873[] = {{&S3978[8], 3}, {NULL}};
const Segment SGI363874[] = {{&S3978[13], 9}, {NULL}};
const Segment SGI363875[] = {{&S3978[33], 15}, {NULL}};
const Segment SGI363876[] = {{&S3978[49], 6}, {NULL}};
const Segment SGI363877[] = {{&S3978[33], 13}, {NULL}};
const Segment SGI363878[] = {{&S3979[69], 6}, {NULL}};
const Segment SGI363879[] = {{&S3979[8], 3}, {NULL}};
const Segment SGI363880[] = {{&S3979[13], 9}, {NULL}};
const Segment SGI363881[] = {{&S3979[33], 15}, {NULL}};
const Segment SGI363882[] = {{&S3979[49], 6}, {NULL}};
const Segment SGI363883[] = {{&S3979[33], 13}, {NULL}};
const Segment SGI363884[] = {{&S3980[69], 6}, {NULL}};
const Segment SGI363885[] = {{&S3980[8], 3}, {NULL}};
const Segment SGI363886[] = {{&S3980[13], 9}, {NULL}};
const Segment SGI363887[] = {{&S3980[33], 15}, {NULL}};
const Segment SGI363888[] = {{&S3980[49], 6}, {NULL}};
const Segment SGI363889[] = {{&S3980[33], 13}, {NULL}};
const Segment SGI363890[] = {{&S3981[66], 6}, {NULL}};
const Segment SGI363891[] = {{&S3981[8], 3}, {NULL}};
const Segment SGI363892[] = {{&S3981[13], 9}, {NULL}};
const Segment SGI363893[] = {{&S3981[33], 15}, {NULL}};
const Segment SGI363894[] = {{&S3981[49], 3}, {NULL}};
const Segment SGI363895[] = {{&S3981[33], 13}, {NULL}};
const Segment SGI363896[] = {{&S3982[69], 6}, {NULL}};
const Segment SGI363897[] = {{&S3982[8], 3}, {NULL}};
const Segment SGI363898[] = {{&S3982[13], 9}, {NULL}};
const Segment SGI363899[] = {{&S3982[33], 15}, {NULL}};
const Segment SGI363900[] = {{&S3982[49], 6}, {NULL}};
const Segment SGI363901[] = {{&S3982[33], 13}, {NULL}};
const Segment SGI363902[] = {{&S3983[69], 6}, {NULL}};
const Segment SGI363903[] = {{&S3983[8], 3}, {NULL}};
const Segment SGI363904[] = {{&S3983[13], 9}, {NULL}};
const Segment SGI363905[] = {{&S3983[33], 15}, {NULL}};
const Segment SGI363906[] = {{&S3983[49], 6}, {NULL}};
const Segment SGI363907[] = {{&S3983[33], 13}, {NULL}};
const Segment SGI363908[] = {{&S3984[69], 6}, {NULL}};
const Segment SGI363909[] = {{&S3984[8], 3}, {NULL}};
const Segment SGI363910[] = {{&S3984[13], 9}, {NULL}};
const Segment SGI363911[] = {{&S3984[33], 15}, {NULL}};
const Segment SGI363912[] = {{&S3984[49], 6}, {NULL}};
const Segment SGI363913[] = {{&S3984[33], 13}, {NULL}};
const Segment SGI363914[] = {{&S3985[69], 6}, {NULL}};
const Segment SGI363915[] = {{&S3985[8], 3}, {NULL}};
const Segment SGI363916[] = {{&S3985[13], 9}, {NULL}};
const Segment SGI363917[] = {{&S3985[33], 15}, {NULL}};
const Segment SGI363918[] = {{&S3985[49], 6}, {NULL}};
const Segment SGI363919[] = {{&S3985[33], 13}, {NULL}};
const Segment SGI363920[] = {{&S3986[69], 6}, {NULL}};
const Segment SGI363921[] = {{&S3986[8], 3}, {NULL}};
const Segment SGI363922[] = {{&S3986[13], 9}, {NULL}};
const Segment SGI363923[] = {{&S3986[33], 15}, {NULL}};
const Segment SGI363924[] = {{&S3986[49], 6}, {NULL}};
const Segment SGI363925[] = {{&S3986[33], 13}, {NULL}};
const Segment SGI363926[] = {{&S3987[69], 7}, {NULL}};
const Segment SGI363927[] = {{&S3987[8], 3}, {NULL}};
const Segment SGI363928[] = {{&S3987[13], 9}, {NULL}};
const Segment SGI363929[] = {{&S3987[33], 15}, {NULL}};
const Segment SGI363930[] = {{&S3987[49], 6}, {NULL}};
const Segment SGI363931[] = {{&S3987[33], 13}, {NULL}};
const Segment SGI363932[] = {{&S3988[69], 6}, {NULL}};
const Segment SGI363933[] = {{&S3988[8], 3}, {NULL}};
const Segment SGI363934[] = {{&S3988[13], 9}, {NULL}};
const Segment SGI363935[] = {{&S3988[33], 15}, {NULL}};
const Segment SGI363936[] = {{&S3988[49], 6}, {NULL}};
const Segment SGI363937[] = {{&S3988[33], 13}, {NULL}};
const Segment SGI363938[] = {{&S3989[69], 6}, {NULL}};
const Segment SGI363939[] = {{&S3989[8], 3}, {NULL}};
const Segment SGI363940[] = {{&S3989[13], 9}, {NULL}};
const Segment SGI363941[] = {{&S3989[33], 15}, {NULL}};
const Segment SGI363942[] = {{&S3989[49], 6}, {NULL}};
const Segment SGI363943[] = {{&S3989[33], 13}, {NULL}};
const Segment SGI363944[] = {{&S3990[119], 6}, {NULL}};
const Segment SGI363945[] = {{&S3990[8], 3}, {NULL}};
const Segment SGI363946[] = {{&S3990[13], 9}, {NULL}};
const Segment SGI363947[] = {{&S3990[33], 15}, {NULL}};
const Segment SGI363948[] = {{&S3990[49], 6}, {NULL}};
const Segment SGI363949[] = {{&S3990[33], 13}, {NULL}};
const Segment SGI363950[] = {{&S3991[71], 6}, {NULL}};
const Segment SGI363951[] = {{&S3991[8], 3}, {NULL}};
const Segment SGI363952[] = {{&S3991[13], 9}, {NULL}};
const Segment SGI363953[] = {{&S3991[33], 15}, {NULL}};
const Segment SGI363954[] = {{&S3991[49], 8}, {NULL}};
const Segment SGI363955[] = {{&S3991[33], 13}, {NULL}};
const Segment SGI363956[] = {{&S3992[69], 6}, {NULL}};
const Segment SGI363957[] = {{&S3992[8], 3}, {NULL}};
const Segment SGI363958[] = {{&S3992[13], 9}, {NULL}};
const Segment SGI363959[] = {{&S3992[33], 15}, {NULL}};
const Segment SGI363960[] = {{&S3992[49], 6}, {NULL}};
const Segment SGI363961[] = {{&S3992[33], 13}, {NULL}};
const Segment SGI363962[] = {{&S3993[69], 6}, {NULL}};
const Segment SGI363963[] = {{&S3993[8], 3}, {NULL}};
const Segment SGI363964[] = {{&S3993[13], 9}, {NULL}};
const Segment SGI363965[] = {{&S3993[33], 15}, {NULL}};
const Segment SGI363966[] = {{&S3993[49], 6}, {NULL}};
const Segment SGI363967[] = {{&S3993[33], 13}, {NULL}};
const Segment SGI363968[] = {{&S3994[69], 6}, {NULL}};
const Segment SGI363969[] = {{&S3994[8], 3}, {NULL}};
const Segment SGI363970[] = {{&S3994[13], 9}, {NULL}};
const Segment SGI363971[] = {{&S3994[33], 15}, {NULL}};
const Segment SGI363972[] = {{&S3994[49], 6}, {NULL}};
const Segment SGI363973[] = {{&S3994[33], 13}, {NULL}};
const Segment SGI363974[] = {{&S3995[69], 6}, {NULL}};
const Segment SGI363975[] = {{&S3995[8], 3}, {NULL}};
const Segment SGI363976[] = {{&S3995[13], 9}, {NULL}};
const Segment SGI363977[] = {{&S3995[33], 15}, {NULL}};
const Segment SGI363978[] = {{&S3995[49], 6}, {NULL}};
const Segment SGI363979[] = {{&S3995[33], 13}, {NULL}};
const Segment SGI363980[] = {{&S3996[69], 6}, {NULL}};
const Segment SGI363981[] = {{&S3996[8], 3}, {NULL}};
const Segment SGI363982[] = {{&S3996[13], 9}, {NULL}};
const Segment SGI363983[] = {{&S3996[33], 15}, {NULL}};
const Segment SGI363984[] = {{&S3996[49], 6}, {NULL}};
const Segment SGI363985[] = {{&S3996[33], 13}, {NULL}};
const Segment SGI363986[] = {{&S3997[69], 6}, {NULL}};
const Segment SGI363987[] = {{&S3997[8], 3}, {NULL}};
const Segment SGI363988[] = {{&S3997[13], 9}, {NULL}};
const Segment SGI363989[] = {{&S3997[33], 15}, {NULL}};
const Segment SGI363990[] = {{&S3997[49], 6}, {NULL}};
const Segment SGI363991[] = {{&S3997[33], 13}, {NULL}};
const Segment SGI363992[] = {{&S3998[69], 6}, {NULL}};
const Segment SGI363993[] = {{&S3998[8], 3}, {NULL}};
const Segment SGI363994[] = {{&S3998[13], 9}, {NULL}};
const Segment SGI363995[] = {{&S3998[33], 15}, {NULL}};
const Segment SGI363996[] = {{&S3998[49], 6}, {NULL}};
const Segment SGI363997[] = {{&S3998[33], 13}, {NULL}};
const Segment SGI363998[] = {{&S3999[69], 6}, {NULL}};
const Segment SGI363999[] = {{&S3999[8], 3}, {NULL}};
const Segment SGI364000[] = {{&S3999[13], 9}, {NULL}};
const Segment SGI364001[] = {{&S3999[33], 15}, {NULL}};
const Segment SGI364002[] = {{&S3999[49], 6}, {NULL}};
const Segment SGI364003[] = {{&S3999[33], 13}, {NULL}};
const Segment SGI364004[] = {{&S4000[69], 6}, {NULL}};
const Segment SGI364005[] = {{&S4000[8], 3}, {NULL}};
const Segment SGI364006[] = {{&S4000[13], 9}, {NULL}};
const Segment SGI364007[] = {{&S4000[33], 15}, {NULL}};
const Segment SGI364008[] = {{&S4000[49], 6}, {NULL}};
const Segment SGI364009[] = {{&S4000[33], 13}, {NULL}};
const Segment SGI364010[] = {{&S4001[69], 6}, {NULL}};
const Segment SGI364011[] = {{&S4001[8], 3}, {NULL}};
const Segment SGI364012[] = {{&S4001[13], 9}, {NULL}};
const Segment SGI364013[] = {{&S4001[33], 15}, {NULL}};
const Segment SGI364014[] = {{&S4001[49], 6}, {NULL}};
const Segment SGI364015[] = {{&S4001[33], 13}, {NULL}};
const Segment SGI364016[] = {{&S4002[69], 6}, {NULL}};
const Segment SGI364017[] = {{&S4002[8], 3}, {NULL}};
const Segment SGI364018[] = {{&S4002[13], 9}, {NULL}};
const Segment SGI364019[] = {{&S4002[33], 15}, {NULL}};
const Segment SGI364020[] = {{&S4002[49], 6}, {NULL}};
const Segment SGI364021[] = {{&S4002[33], 13}, {NULL}};
const Segment SGI364022[] = {{&S4003[69], 6}, {NULL}};
const Segment SGI364023[] = {{&S4003[8], 3}, {NULL}};
const Segment SGI364024[] = {{&S4003[13], 9}, {NULL}};
const Segment SGI364025[] = {{&S4003[33], 15}, {NULL}};
const Segment SGI364026[] = {{&S4003[49], 6}, {NULL}};
const Segment SGI364027[] = {{&S4003[33], 13}, {NULL}};
const Segment SGI364028[] = {{&S4004[69], 6}, {NULL}};
const Segment SGI364029[] = {{&S4004[8], 3}, {NULL}};
const Segment SGI364030[] = {{&S4004[13], 9}, {NULL}};
const Segment SGI364031[] = {{&S4004[33], 15}, {NULL}};
const Segment SGI364032[] = {{&S4004[49], 6}, {NULL}};
const Segment SGI364033[] = {{&S4004[33], 13}, {NULL}};
const Segment SGI364034[] = {{&S4005[69], 6}, {NULL}};
const Segment SGI364035[] = {{&S4005[8], 3}, {NULL}};
const Segment SGI364036[] = {{&S4005[13], 9}, {NULL}};
const Segment SGI364037[] = {{&S4005[33], 15}, {NULL}};
const Segment SGI364038[] = {{&S4005[49], 6}, {NULL}};
const Segment SGI364039[] = {{&S4005[33], 13}, {NULL}};
const Segment SGI364040[] = {{&S4006[69], 6}, {NULL}};
const Segment SGI364041[] = {{&S4006[8], 3}, {NULL}};
const Segment SGI364042[] = {{&S4006[13], 9}, {NULL}};
const Segment SGI364043[] = {{&S4006[33], 15}, {NULL}};
const Segment SGI364044[] = {{&S4006[49], 6}, {NULL}};
const Segment SGI364045[] = {{&S4006[33], 13}, {NULL}};
const Segment SGI364046[] = {{&S4007[69], 6}, {NULL}};
const Segment SGI364047[] = {{&S4007[8], 3}, {NULL}};
const Segment SGI364048[] = {{&S4007[13], 9}, {NULL}};
const Segment SGI364049[] = {{&S4007[33], 15}, {NULL}};
const Segment SGI364050[] = {{&S4007[49], 6}, {NULL}};
const Segment SGI364051[] = {{&S4007[33], 13}, {NULL}};
const Segment SGI364052[] = {{&S4008[69], 6}, {NULL}};
const Segment SGI364053[] = {{&S4008[8], 3}, {NULL}};
const Segment SGI364054[] = {{&S4008[13], 9}, {NULL}};
const Segment SGI364055[] = {{&S4008[33], 15}, {NULL}};
const Segment SGI364056[] = {{&S4008[49], 6}, {NULL}};
const Segment SGI364057[] = {{&S4008[33], 13}, {NULL}};
const Segment SGI364058[] = {{&S4009[69], 6}, {NULL}};
const Segment SGI364059[] = {{&S4009[8], 3}, {NULL}};
const Segment SGI364060[] = {{&S4009[13], 9}, {NULL}};
const Segment SGI364061[] = {{&S4009[33], 15}, {NULL}};
const Segment SGI364062[] = {{&S4009[49], 6}, {NULL}};
const Segment SGI364063[] = {{&S4009[33], 13}, {NULL}};
const Segment SGI364064[] = {{&S4010[69], 6}, {NULL}};
const Segment SGI364065[] = {{&S4010[8], 3}, {NULL}};
const Segment SGI364066[] = {{&S4010[13], 9}, {NULL}};
const Segment SGI364067[] = {{&S4010[33], 15}, {NULL}};
const Segment SGI364068[] = {{&S4010[49], 6}, {NULL}};
const Segment SGI364069[] = {{&S4010[33], 13}, {NULL}};
const Segment SGI364070[] = {{&S4011[66], 6}, {NULL}};
const Segment SGI364071[] = {{&S4011[8], 3}, {NULL}};
const Segment SGI364072[] = {{&S4011[13], 9}, {NULL}};
const Segment SGI364073[] = {{&S4011[33], 15}, {NULL}};
const Segment SGI364074[] = {{&S4011[49], 3}, {NULL}};
const Segment SGI364075[] = {{&S4011[33], 13}, {NULL}};
const Segment SGI364076[] = {{&S4012[69], 6}, {NULL}};
const Segment SGI364077[] = {{&S4012[8], 3}, {NULL}};
const Segment SGI364078[] = {{&S4012[13], 9}, {NULL}};
const Segment SGI364079[] = {{&S4012[33], 15}, {NULL}};
const Segment SGI364080[] = {{&S4012[49], 6}, {NULL}};
const Segment SGI364081[] = {{&S4012[33], 13}, {NULL}};
const Segment SGI364082[] = {{&S4013[69], 6}, {NULL}};
const Segment SGI364083[] = {{&S4013[8], 3}, {NULL}};
const Segment SGI364084[] = {{&S4013[13], 9}, {NULL}};
const Segment SGI364085[] = {{&S4013[33], 15}, {NULL}};
const Segment SGI364086[] = {{&S4013[49], 6}, {NULL}};
const Segment SGI364087[] = {{&S4013[33], 13}, {NULL}};
const Segment SGI364088[] = {{&S4014[66], 6}, {NULL}};
const Segment SGI364089[] = {{&S4014[8], 3}, {NULL}};
const Segment SGI364090[] = {{&S4014[13], 9}, {NULL}};
const Segment SGI364091[] = {{&S4014[33], 15}, {NULL}};
const Segment SGI364092[] = {{&S4014[49], 3}, {NULL}};
const Segment SGI364093[] = {{&S4014[33], 13}, {NULL}};
const Segment SGI364094[] = {{&S4015[66], 6}, {NULL}};
const Segment SGI364095[] = {{&S4015[8], 3}, {NULL}};
const Segment SGI364096[] = {{&S4015[13], 9}, {NULL}};
const Segment SGI364097[] = {{&S4015[33], 15}, {NULL}};
const Segment SGI364098[] = {{&S4015[49], 3}, {NULL}};
const Segment SGI364099[] = {{&S4015[33], 13}, {NULL}};
const Segment SGI364100[] = {{&S4016[69], 6}, {NULL}};
const Segment SGI364101[] = {{&S4016[8], 3}, {NULL}};
const Segment SGI364102[] = {{&S4016[13], 9}, {NULL}};
const Segment SGI364103[] = {{&S4016[33], 15}, {NULL}};
const Segment SGI364104[] = {{&S4016[49], 6}, {NULL}};
const Segment SGI364105[] = {{&S4016[33], 13}, {NULL}};
const Segment SGI364106[] = {{&S4017[119], 6}, {NULL}};
const Segment SGI364107[] = {{&S4017[8], 3}, {NULL}};
const Segment SGI364108[] = {{&S4017[13], 9}, {NULL}};
const Segment SGI364109[] = {{&S4017[33], 15}, {NULL}};
const Segment SGI364110[] = {{&S4017[49], 6}, {NULL}};
const Segment SGI364111[] = {{&S4017[33], 13}, {NULL}};
const Segment SGI364112[] = {{&S4018[66], 6}, {NULL}};
const Segment SGI364113[] = {{&S4018[8], 3}, {NULL}};
const Segment SGI364114[] = {{&S4018[13], 9}, {NULL}};
const Segment SGI364115[] = {{&S4018[33], 15}, {NULL}};
const Segment SGI364116[] = {{&S4018[49], 3}, {NULL}};
const Segment SGI364117[] = {{&S4018[33], 13}, {NULL}};
const Segment SGI364118[] = {{&S4019[121], 7}, {NULL}};
const Segment SGI364119[] = {{&S4019[8], 3}, {NULL}};
const Segment SGI364120[] = {{&S4019[13], 9}, {NULL}};
const Segment SGI364121[] = {{&S4019[33], 15}, {NULL}};
const Segment SGI364122[] = {{&S4019[49], 6}, {NULL}};
const Segment SGI364123[] = {{&S4019[33], 13}, {NULL}};
const Segment SGI364124[] = {{&S4020[69], 6}, {NULL}};
const Segment SGI364125[] = {{&S4020[8], 3}, {NULL}};
const Segment SGI364126[] = {{&S4020[13], 9}, {NULL}};
const Segment SGI364127[] = {{&S4020[33], 15}, {NULL}};
const Segment SGI364128[] = {{&S4020[49], 6}, {NULL}};
const Segment SGI364129[] = {{&S4020[33], 13}, {NULL}};
const Segment SGI364130[] = {{&S4021[69], 6}, {NULL}};
const Segment SGI364131[] = {{&S4021[8], 3}, {NULL}};
const Segment SGI364132[] = {{&S4021[13], 9}, {NULL}};
const Segment SGI364133[] = {{&S4021[33], 15}, {NULL}};
const Segment SGI364134[] = {{&S4021[49], 6}, {NULL}};
const Segment SGI364135[] = {{&S4021[33], 13}, {NULL}};
const Segment SGI364136[] = {{&S4022[69], 6}, {NULL}};
const Segment SGI364137[] = {{&S4022[8], 3}, {NULL}};
const Segment SGI364138[] = {{&S4022[13], 9}, {NULL}};
const Segment SGI364139[] = {{&S4022[33], 15}, {NULL}};
const Segment SGI364140[] = {{&S4022[49], 6}, {NULL}};
const Segment SGI364141[] = {{&S4022[33], 13}, {NULL}};
const Segment SGI364142[] = {{&S4023[69], 6}, {NULL}};
const Segment SGI364143[] = {{&S4023[8], 3}, {NULL}};
const Segment SGI364144[] = {{&S4023[13], 9}, {NULL}};
const Segment SGI364145[] = {{&S4023[33], 15}, {NULL}};
const Segment SGI364146[] = {{&S4023[49], 6}, {NULL}};
const Segment SGI364147[] = {{&S4023[33], 13}, {NULL}};
const Segment SGI364148[] = {{&S4024[69], 6}, {NULL}};
const Segment SGI364149[] = {{&S4024[8], 3}, {NULL}};
const Segment SGI364150[] = {{&S4024[13], 9}, {NULL}};
const Segment SGI364151[] = {{&S4024[33], 15}, {NULL}};
const Segment SGI364152[] = {{&S4024[49], 6}, {NULL}};
const Segment SGI364153[] = {{&S4024[33], 13}, {NULL}};
const Segment SGI364154[] = {{&S4025[69], 6}, {NULL}};
const Segment SGI364155[] = {{&S4025[8], 3}, {NULL}};
const Segment SGI364156[] = {{&S4025[13], 9}, {NULL}};
const Segment SGI364157[] = {{&S4025[33], 15}, {NULL}};
const Segment SGI364158[] = {{&S4025[49], 6}, {NULL}};
const Segment SGI364159[] = {{&S4025[33], 13}, {NULL}};
const Segment SGI364160[] = {{&S4026[69], 6}, {NULL}};
const Segment SGI364161[] = {{&S4026[8], 3}, {NULL}};
const Segment SGI364162[] = {{&S4026[13], 9}, {NULL}};
const Segment SGI364163[] = {{&S4026[33], 15}, {NULL}};
const Segment SGI364164[] = {{&S4026[49], 6}, {NULL}};
const Segment SGI364165[] = {{&S4026[33], 13}, {NULL}};
const Segment SGI364166[] = {{&S4027[69], 6}, {NULL}};
const Segment SGI364167[] = {{&S4027[8], 3}, {NULL}};
const Segment SGI364168[] = {{&S4027[13], 9}, {NULL}};
const Segment SGI364169[] = {{&S4027[33], 15}, {NULL}};
const Segment SGI364170[] = {{&S4027[49], 6}, {NULL}};
const Segment SGI364171[] = {{&S4027[33], 13}, {NULL}};
const Segment SGI364172[] = {{&S4028[69], 6}, {NULL}};
const Segment SGI364173[] = {{&S4028[8], 3}, {NULL}};
const Segment SGI364174[] = {{&S4028[13], 9}, {NULL}};
const Segment SGI364175[] = {{&S4028[33], 15}, {NULL}};
const Segment SGI364176[] = {{&S4028[49], 6}, {NULL}};
const Segment SGI364177[] = {{&S4028[33], 13}, {NULL}};
const Segment SGI364178[] = {{&S4029[69], 6}, {NULL}};
const Segment SGI364179[] = {{&S4029[8], 3}, {NULL}};
const Segment SGI364180[] = {{&S4029[13], 9}, {NULL}};
const Segment SGI364181[] = {{&S4029[33], 15}, {NULL}};
const Segment SGI364182[] = {{&S4029[49], 6}, {NULL}};
const Segment SGI364183[] = {{&S4029[33], 13}, {NULL}};
const Segment SGI364184[] = {{&S4030[66], 6}, {NULL}};
const Segment SGI364185[] = {{&S4030[8], 3}, {NULL}};
const Segment SGI364186[] = {{&S4030[13], 9}, {NULL}};
const Segment SGI364187[] = {{&S4030[33], 15}, {NULL}};
const Segment SGI364188[] = {{&S4030[49], 3}, {NULL}};
const Segment SGI364189[] = {{&S4030[33], 13}, {NULL}};
const Segment SGI364190[] = {{&S4031[69], 6}, {NULL}};
const Segment SGI364191[] = {{&S4031[8], 3}, {NULL}};
const Segment SGI364192[] = {{&S4031[13], 9}, {NULL}};
const Segment SGI364193[] = {{&S4031[33], 15}, {NULL}};
const Segment SGI364194[] = {{&S4031[49], 6}, {NULL}};
const Segment SGI364195[] = {{&S4031[33], 13}, {NULL}};
const Segment SGI364196[] = {{&S4032[66], 6}, {NULL}};
const Segment SGI364197[] = {{&S4032[8], 3}, {NULL}};
const Segment SGI364198[] = {{&S4032[13], 9}, {NULL}};
const Segment SGI364199[] = {{&S4032[33], 15}, {NULL}};
const Segment SGI364200[] = {{&S4032[49], 3}, {NULL}};
const Segment SGI364201[] = {{&S4032[33], 13}, {NULL}};
const Segment SGI364202[] = {{&S4033[69], 6}, {NULL}};
const Segment SGI364203[] = {{&S4033[8], 3}, {NULL}};
const Segment SGI364204[] = {{&S4033[13], 9}, {NULL}};
const Segment SGI364205[] = {{&S4033[33], 15}, {NULL}};
const Segment SGI364206[] = {{&S4033[49], 6}, {NULL}};
const Segment SGI364207[] = {{&S4033[33], 13}, {NULL}};
const Segment SGI364208[] = {{&S4034[71], 6}, {NULL}};
const Segment SGI364209[] = {{&S4034[8], 3}, {NULL}};
const Segment SGI364210[] = {{&S4034[13], 9}, {NULL}};
const Segment SGI364211[] = {{&S4034[33], 15}, {NULL}};
const Segment SGI364212[] = {{&S4034[49], 8}, {NULL}};
const Segment SGI364213[] = {{&S4034[33], 13}, {NULL}};
const Segment SGI364214[] = {{&S4035[66], 6}, {NULL}};
const Segment SGI364215[] = {{&S4035[8], 3}, {NULL}};
const Segment SGI364216[] = {{&S4035[13], 9}, {NULL}};
const Segment SGI364217[] = {{&S4035[33], 15}, {NULL}};
const Segment SGI364218[] = {{&S4035[49], 3}, {NULL}};
const Segment SGI364219[] = {{&S4035[33], 13}, {NULL}};
const Segment SGI364220[] = {{&S4036[69], 6}, {NULL}};
const Segment SGI364221[] = {{&S4036[8], 3}, {NULL}};
const Segment SGI364222[] = {{&S4036[13], 9}, {NULL}};
const Segment SGI364223[] = {{&S4036[33], 15}, {NULL}};
const Segment SGI364224[] = {{&S4036[49], 6}, {NULL}};
const Segment SGI364225[] = {{&S4036[33], 13}, {NULL}};
const Segment SGI364226[] = {{&S4037[69], 6}, {NULL}};
const Segment SGI364227[] = {{&S4037[8], 3}, {NULL}};
const Segment SGI364228[] = {{&S4037[13], 9}, {NULL}};
const Segment SGI364229[] = {{&S4037[33], 15}, {NULL}};
const Segment SGI364230[] = {{&S4037[49], 6}, {NULL}};
const Segment SGI364231[] = {{&S4037[33], 13}, {NULL}};
const Segment SGI364232[] = {{&S4038[119], 6}, {NULL}};
const Segment SGI364233[] = {{&S4038[8], 3}, {NULL}};
const Segment SGI364234[] = {{&S4038[13], 9}, {NULL}};
const Segment SGI364235[] = {{&S4038[33], 15}, {NULL}};
const Segment SGI364236[] = {{&S4038[49], 6}, {NULL}};
const Segment SGI364237[] = {{&S4038[33], 13}, {NULL}};
const Segment SGI364238[] = {{&S4039[66], 6}, {NULL}};
const Segment SGI364239[] = {{&S4039[8], 3}, {NULL}};
const Segment SGI364240[] = {{&S4039[13], 9}, {NULL}};
const Segment SGI364241[] = {{&S4039[33], 15}, {NULL}};
const Segment SGI364242[] = {{&S4039[49], 3}, {NULL}};
const Segment SGI364243[] = {{&S4039[33], 13}, {NULL}};
const Segment SGI364244[] = {{&S4040[69], 6}, {NULL}};
const Segment SGI364245[] = {{&S4040[8], 3}, {NULL}};
const Segment SGI364246[] = {{&S4040[13], 9}, {NULL}};
const Segment SGI364247[] = {{&S4040[33], 15}, {NULL}};
const Segment SGI364248[] = {{&S4040[49], 6}, {NULL}};
const Segment SGI364249[] = {{&S4040[33], 13}, {NULL}};
const Segment SGI364250[] = {{&S4041[69], 6}, {NULL}};
const Segment SGI364251[] = {{&S4041[8], 3}, {NULL}};
const Segment SGI364252[] = {{&S4041[13], 9}, {NULL}};
const Segment SGI364253[] = {{&S4041[33], 15}, {NULL}};
const Segment SGI364254[] = {{&S4041[49], 6}, {NULL}};
const Segment SGI364255[] = {{&S4041[33], 13}, {NULL}};
const Segment SGI364256[] = {{&S4042[69], 6}, {NULL}};
const Segment SGI364257[] = {{&S4042[8], 3}, {NULL}};
const Segment SGI364258[] = {{&S4042[13], 9}, {NULL}};
const Segment SGI364259[] = {{&S4042[33], 15}, {NULL}};
const Segment SGI364260[] = {{&S4042[49], 6}, {NULL}};
const Segment SGI364261[] = {{&S4042[33], 13}, {NULL}};
const Segment SGI364262[] = {{&S4043[69], 6}, {NULL}};
const Segment SGI364263[] = {{&S4043[8], 3}, {NULL}};
const Segment SGI364264[] = {{&S4043[13], 9}, {NULL}};
const Segment SGI364265[] = {{&S4043[33], 15}, {NULL}};
const Segment SGI364266[] = {{&S4043[49], 6}, {NULL}};
const Segment SGI364267[] = {{&S4043[33], 13}, {NULL}};
const Segment SGI364268[] = {{&S4044[69], 6}, {NULL}};
const Segment SGI364269[] = {{&S4044[8], 3}, {NULL}};
const Segment SGI364270[] = {{&S4044[13], 9}, {NULL}};
const Segment SGI364271[] = {{&S4044[33], 15}, {NULL}};
const Segment SGI364272[] = {{&S4044[49], 6}, {NULL}};
const Segment SGI364273[] = {{&S4044[33], 13}, {NULL}};
const Segment SGI364274[] = {{&S4045[119], 6}, {NULL}};
const Segment SGI364275[] = {{&S4045[8], 3}, {NULL}};
const Segment SGI364276[] = {{&S4045[13], 9}, {NULL}};
const Segment SGI364277[] = {{&S4045[33], 15}, {NULL}};
const Segment SGI364278[] = {{&S4045[49], 6}, {NULL}};
const Segment SGI364279[] = {{&S4045[33], 13}, {NULL}};
const Segment SGI364280[] = {{&S4046[69], 6}, {NULL}};
const Segment SGI364281[] = {{&S4046[8], 3}, {NULL}};
const Segment SGI364282[] = {{&S4046[13], 9}, {NULL}};
const Segment SGI364283[] = {{&S4046[33], 15}, {NULL}};
const Segment SGI364284[] = {{&S4046[49], 6}, {NULL}};
const Segment SGI364285[] = {{&S4046[33], 13}, {NULL}};
const Segment SGI364286[] = {{&S4047[69], 6}, {NULL}};
const Segment SGI364287[] = {{&S4047[8], 3}, {NULL}};
const Segment SGI364288[] = {{&S4047[13], 9}, {NULL}};
const Segment SGI364289[] = {{&S4047[33], 15}, {NULL}};
const Segment SGI364290[] = {{&S4047[49], 6}, {NULL}};
const Segment SGI364291[] = {{&S4047[33], 13}, {NULL}};
const Segment SGI364292[] = {{&S4048[69], 6}, {NULL}};
const Segment SGI364293[] = {{&S4048[8], 3}, {NULL}};
const Segment SGI364294[] = {{&S4048[13], 9}, {NULL}};
const Segment SGI364295[] = {{&S4048[33], 15}, {NULL}};
const Segment SGI364296[] = {{&S4048[49], 6}, {NULL}};
const Segment SGI364297[] = {{&S4048[33], 13}, {NULL}};
const Segment SGI364298[] = {{&S4049[69], 6}, {NULL}};
const Segment SGI364299[] = {{&S4049[8], 3}, {NULL}};
const Segment SGI364300[] = {{&S4049[13], 9}, {NULL}};
const Segment SGI364301[] = {{&S4049[33], 15}, {NULL}};
const Segment SGI364302[] = {{&S4049[49], 6}, {NULL}};
const Segment SGI364303[] = {{&S4049[33], 13}, {NULL}};
const Segment SGI364304[] = {{&S4050[69], 6}, {NULL}};
const Segment SGI364305[] = {{&S4050[8], 3}, {NULL}};
const Segment SGI364306[] = {{&S4050[13], 9}, {NULL}};
const Segment SGI364307[] = {{&S4050[33], 15}, {NULL}};
const Segment SGI364308[] = {{&S4050[49], 6}, {NULL}};
const Segment SGI364309[] = {{&S4050[33], 13}, {NULL}};
const Segment SGI364310[] = {{&S4051[69], 6}, {NULL}};
const Segment SGI364311[] = {{&S4051[8], 3}, {NULL}};
const Segment SGI364312[] = {{&S4051[13], 9}, {NULL}};
const Segment SGI364313[] = {{&S4051[33], 15}, {NULL}};
const Segment SGI364314[] = {{&S4051[49], 6}, {NULL}};
const Segment SGI364315[] = {{&S4051[33], 13}, {NULL}};
const Segment SGI364316[] = {{&S4052[69], 6}, {NULL}};
const Segment SGI364317[] = {{&S4052[8], 3}, {NULL}};
const Segment SGI364318[] = {{&S4052[13], 9}, {NULL}};
const Segment SGI364319[] = {{&S4052[33], 15}, {NULL}};
const Segment SGI364320[] = {{&S4052[49], 6}, {NULL}};
const Segment SGI364321[] = {{&S4052[33], 13}, {NULL}};
const Segment SGI364322[] = {{&S4053[69], 6}, {NULL}};
const Segment SGI364323[] = {{&S4053[8], 3}, {NULL}};
const Segment SGI364324[] = {{&S4053[13], 9}, {NULL}};
const Segment SGI364325[] = {{&S4053[33], 15}, {NULL}};
const Segment SGI364326[] = {{&S4053[49], 6}, {NULL}};
const Segment SGI364327[] = {{&S4053[33], 13}, {NULL}};
const Segment SGI364328[] = {{&S4054[69], 6}, {NULL}};
const Segment SGI364329[] = {{&S4054[8], 3}, {NULL}};
const Segment SGI364330[] = {{&S4054[13], 9}, {NULL}};
const Segment SGI364331[] = {{&S4054[33], 15}, {NULL}};
const Segment SGI364332[] = {{&S4054[49], 6}, {NULL}};
const Segment SGI364333[] = {{&S4054[33], 13}, {NULL}};
const Segment SGI364334[] = {{&S4055[69], 6}, {NULL}};
const Segment SGI364335[] = {{&S4055[8], 3}, {NULL}};
const Segment SGI364336[] = {{&S4055[13], 9}, {NULL}};
const Segment SGI364337[] = {{&S4055[33], 15}, {NULL}};
const Segment SGI364338[] = {{&S4055[49], 6}, {NULL}};
const Segment SGI364339[] = {{&S4055[33], 13}, {NULL}};
const Segment SGI364340[] = {{&S4056[69], 6}, {NULL}};
const Segment SGI364341[] = {{&S4056[8], 3}, {NULL}};
const Segment SGI364342[] = {{&S4056[13], 9}, {NULL}};
const Segment SGI364343[] = {{&S4056[33], 15}, {NULL}};
const Segment SGI364344[] = {{&S4056[49], 6}, {NULL}};
const Segment SGI364345[] = {{&S4056[33], 13}, {NULL}};
const Segment SGI364346[] = {{&S4057[69], 6}, {NULL}};
const Segment SGI364347[] = {{&S4057[8], 3}, {NULL}};
const Segment SGI364348[] = {{&S4057[13], 9}, {NULL}};
const Segment SGI364349[] = {{&S4057[33], 15}, {NULL}};
const Segment SGI364350[] = {{&S4057[49], 6}, {NULL}};
const Segment SGI364351[] = {{&S4057[33], 13}, {NULL}};
const Segment SGI364352[] = {{&S4058[69], 6}, {NULL}};
const Segment SGI364353[] = {{&S4058[8], 3}, {NULL}};
const Segment SGI364354[] = {{&S4058[13], 9}, {NULL}};
const Segment SGI364355[] = {{&S4058[33], 15}, {NULL}};
const Segment SGI364356[] = {{&S4058[49], 6}, {NULL}};
const Segment SGI364357[] = {{&S4058[33], 13}, {NULL}};
const Segment SGI364358[] = {{&S4059[105], 6}, {NULL}};
const Segment SGI364359[] = {{&S4059[8], 3}, {NULL}};
const Segment SGI364360[] = {{&S4059[13], 9}, {NULL}};
const Segment SGI364361[] = {{&S4059[30], 13}, {NULL}};
const Segment SGI364362[] = {{&S4060[69], 6}, {NULL}};
const Segment SGI364363[] = {{&S4060[8], 3}, {NULL}};
const Segment SGI364364[] = {{&S4060[13], 9}, {NULL}};
const Segment SGI364365[] = {{&S4060[33], 15}, {NULL}};
const Segment SGI364366[] = {{&S4060[49], 6}, {NULL}};
const Segment SGI364367[] = {{&S4060[33], 13}, {NULL}};
const Segment SGI364368[] = {{&S4061[69], 6}, {NULL}};
const Segment SGI364369[] = {{&S4061[8], 3}, {NULL}};
const Segment SGI364370[] = {{&S4061[13], 9}, {NULL}};
const Segment SGI364371[] = {{&S4061[33], 15}, {NULL}};
const Segment SGI364372[] = {{&S4061[49], 6}, {NULL}};
const Segment SGI364373[] = {{&S4061[33], 13}, {NULL}};
const Segment SGI364374[] = {{&S4062[69], 6}, {NULL}};
const Segment SGI364375[] = {{&S4062[8], 3}, {NULL}};
const Segment SGI364376[] = {{&S4062[13], 9}, {NULL}};
const Segment SGI364377[] = {{&S4062[33], 15}, {NULL}};
const Segment SGI364378[] = {{&S4062[49], 6}, {NULL}};
const Segment SGI364379[] = {{&S4062[33], 13}, {NULL}};
const Segment SGI364380[] = {{&S4063[69], 6}, {NULL}};
const Segment SGI364381[] = {{&S4063[8], 3}, {NULL}};
const Segment SGI364382[] = {{&S4063[13], 9}, {NULL}};
const Segment SGI364383[] = {{&S4063[33], 15}, {NULL}};
const Segment SGI364384[] = {{&S4063[49], 6}, {NULL}};
const Segment SGI364385[] = {{&S4063[33], 13}, {NULL}};
const Segment SGI364386[] = {{&S4064[69], 6}, {NULL}};
const Segment SGI364387[] = {{&S4064[8], 3}, {NULL}};
const Segment SGI364388[] = {{&S4064[13], 9}, {NULL}};
const Segment SGI364389[] = {{&S4064[33], 15}, {NULL}};
const Segment SGI364390[] = {{&S4064[49], 6}, {NULL}};
const Segment SGI364391[] = {{&S4064[33], 13}, {NULL}};
const Segment SGI364392[] = {{&S4065[69], 6}, {NULL}};
const Segment SGI364393[] = {{&S4065[8], 3}, {NULL}};
const Segment SGI364394[] = {{&S4065[13], 9}, {NULL}};
const Segment SGI364395[] = {{&S4065[33], 15}, {NULL}};
const Segment SGI364396[] = {{&S4065[49], 6}, {NULL}};
const Segment SGI364397[] = {{&S4065[33], 13}, {NULL}};
const Segment SGI364398[] = {{&S4066[69], 6}, {NULL}};
const Segment SGI364399[] = {{&S4066[8], 3}, {NULL}};
const Segment SGI364400[] = {{&S4066[13], 9}, {NULL}};
const Segment SGI364401[] = {{&S4066[33], 15}, {NULL}};
const Segment SGI364402[] = {{&S4066[49], 6}, {NULL}};
const Segment SGI364403[] = {{&S4066[33], 13}, {NULL}};
const Segment SGI364404[] = {{&S4067[69], 6}, {NULL}};
const Segment SGI364405[] = {{&S4067[8], 3}, {NULL}};
const Segment SGI364406[] = {{&S4067[13], 9}, {NULL}};
const Segment SGI364407[] = {{&S4067[33], 15}, {NULL}};
const Segment SGI364408[] = {{&S4067[49], 6}, {NULL}};
const Segment SGI364409[] = {{&S4067[33], 13}, {NULL}};
const Segment SGI364410[] = {{&S4068[69], 6}, {NULL}};
const Segment SGI364411[] = {{&S4068[8], 3}, {NULL}};
const Segment SGI364412[] = {{&S4068[13], 9}, {NULL}};
const Segment SGI364413[] = {{&S4068[33], 15}, {NULL}};
const Segment SGI364414[] = {{&S4068[49], 6}, {NULL}};
const Segment SGI364415[] = {{&S4068[33], 13}, {NULL}};
const Segment SGI364416[] = {{&S4069[69], 6}, {NULL}};
const Segment SGI364417[] = {{&S4069[8], 3}, {NULL}};
const Segment SGI364418[] = {{&S4069[13], 9}, {NULL}};
const Segment SGI364419[] = {{&S4069[33], 15}, {NULL}};
const Segment SGI364420[] = {{&S4069[49], 6}, {NULL}};
const Segment SGI364421[] = {{&S4069[33], 13}, {NULL}};
const Segment SGI364422[] = {{&S4070[69], 6}, {NULL}};
const Segment SGI364423[] = {{&S4070[8], 3}, {NULL}};
const Segment SGI364424[] = {{&S4070[13], 9}, {NULL}};
const Segment SGI364425[] = {{&S4070[33], 15}, {NULL}};
const Segment SGI364426[] = {{&S4070[49], 6}, {NULL}};
const Segment SGI364427[] = {{&S4070[33], 13}, {NULL}};
const Segment SGI364428[] = {{&S4071[69], 6}, {NULL}};
const Segment SGI364429[] = {{&S4071[8], 3}, {NULL}};
const Segment SGI364430[] = {{&S4071[13], 9}, {NULL}};
const Segment SGI364431[] = {{&S4071[33], 15}, {NULL}};
const Segment SGI364432[] = {{&S4071[49], 6}, {NULL}};
const Segment SGI364433[] = {{&S4071[33], 13}, {NULL}};
const Segment SGI364434[] = {{&S4072[69], 6}, {NULL}};
const Segment SGI364435[] = {{&S4072[8], 3}, {NULL}};
const Segment SGI364436[] = {{&S4072[13], 9}, {NULL}};
const Segment SGI364437[] = {{&S4072[33], 15}, {NULL}};
const Segment SGI364438[] = {{&S4072[49], 6}, {NULL}};
const Segment SGI364439[] = {{&S4072[33], 13}, {NULL}};
const Segment SGI364440[] = {{&S4073[69], 6}, {NULL}};
const Segment SGI364441[] = {{&S4073[8], 3}, {NULL}};
const Segment SGI364442[] = {{&S4073[13], 9}, {NULL}};
const Segment SGI364443[] = {{&S4073[33], 15}, {NULL}};
const Segment SGI364444[] = {{&S4073[49], 6}, {NULL}};
const Segment SGI364445[] = {{&S5379[13], 31}, {NULL}};
const Segment SGI364446[] = {{&S4073[33], 13}, {NULL}};
const Segment SGI364447[] = {{&S5379[58], 6}, {NULL}};
const Segment SGI364448[] = {{&S5379[93], 3}, {NULL}};
const Segment SGI364449[] = {{&S4074[71], 6}, {NULL}};
const Segment SGI364450[] = {{&S4074[8], 3}, {NULL}};
const Segment SGI364451[] = {{&S4074[13], 9}, {NULL}};
const Segment SGI364452[] = {{&S4074[33], 17}, {NULL}};
const Segment SGI364453[] = {{&S4075[69], 6}, {NULL}};
const Segment SGI364454[] = {{&S4075[8], 3}, {NULL}};
const Segment SGI364455[] = {{&S4075[13], 9}, {NULL}};
const Segment SGI364456[] = {{&S4075[33], 15}, {NULL}};
const Segment SGI364457[] = {{&S4075[49], 6}, {NULL}};
const Segment SGI364458[] = {{&S4075[33], 13}, {NULL}};
const Segment SGI364459[] = {{&S4076[69], 6}, {NULL}};
const Segment SGI364460[] = {{&S4076[8], 3}, {NULL}};
const Segment SGI364461[] = {{&S4076[13], 9}, {NULL}};
const Segment SGI364462[] = {{&S4076[33], 15}, {NULL}};
const Segment SGI364463[] = {{&S4076[49], 6}, {NULL}};
const Segment SGI364464[] = {{&S4076[33], 13}, {NULL}};
const Segment SGI364465[] = {{&S4077[69], 6}, {NULL}};
const Segment SGI364466[] = {{&S4077[8], 3}, {NULL}};
const Segment SGI364467[] = {{&S4077[13], 9}, {NULL}};
const Segment SGI364468[] = {{&S4077[33], 15}, {NULL}};
const Segment SGI364469[] = {{&S4077[49], 6}, {NULL}};
const Segment SGI364470[] = {{&S4077[33], 13}, {NULL}};
const Segment SGI364471[] = {{&S4078[69], 6}, {NULL}};
const Segment SGI364472[] = {{&S4078[8], 3}, {NULL}};
const Segment SGI364473[] = {{&S4078[13], 9}, {NULL}};
const Segment SGI364474[] = {{&S4078[33], 15}, {NULL}};
const Segment SGI364475[] = {{&S4078[49], 6}, {NULL}};
const Segment SGI364476[] = {{&S4078[33], 13}, {NULL}};
const Segment SGI364477[] = {{&S4079[70], 6}, {NULL}};
const Segment SGI364478[] = {{&S4079[8], 3}, {NULL}};
const Segment SGI364479[] = {{&S4079[13], 10}, {NULL}};
const Segment SGI364480[] = {{&S4079[34], 15}, {NULL}};
const Segment SGI364481[] = {{&S4079[50], 6}, {NULL}};
const Segment SGI364482[] = {{&S4079[34], 13}, {NULL}};
const Segment SGI364483[] = {{&S4080[80], 6}, {NULL}};
const Segment SGI364484[] = {{&S4080[8], 3}, {NULL}};
const Segment SGI364485[] = {{&S4080[13], 9}, {NULL}};
const Segment SGI364486[] = {{&S4080[33], 15}, {NULL}};
const Segment SGI364487[] = {{&S4080[49], 10}, {NULL}};
const Segment SGI364488[] = {{&S4080[33], 13}, {NULL}};
const Segment SGI364489[] = {{&S4081[69], 6}, {NULL}};
const Segment SGI364490[] = {{&S4081[8], 3}, {NULL}};
const Segment SGI364491[] = {{&S4081[13], 9}, {NULL}};
const Segment SGI364492[] = {{&S4081[33], 15}, {NULL}};
const Segment SGI364493[] = {{&S4081[49], 6}, {NULL}};
const Segment SGI364494[] = {{&S4081[33], 13}, {NULL}};
const Segment SGI364495[] = {{&S4082[69], 6}, {NULL}};
const Segment SGI364496[] = {{&S4082[8], 3}, {NULL}};
const Segment SGI364497[] = {{&S4082[13], 9}, {NULL}};
const Segment SGI364498[] = {{&S4082[33], 15}, {NULL}};
const Segment SGI364499[] = {{&S4082[49], 6}, {NULL}};
const Segment SGI364500[] = {{&S4082[33], 13}, {NULL}};
const Segment SGI364501[] = {{&S4083[69], 7}, {NULL}};
const Segment SGI364502[] = {{&S4083[8], 3}, {NULL}};
const Segment SGI364503[] = {{&S4083[13], 9}, {NULL}};
const Segment SGI364504[] = {{&S4083[33], 15}, {NULL}};
const Segment SGI364505[] = {{&S4083[49], 6}, {NULL}};
const Segment SGI364506[] = {{&S4083[33], 13}, {NULL}};
const Segment SGI364507[] = {{&S4084[69], 6}, {NULL}};
const Segment SGI364508[] = {{&S4084[8], 3}, {NULL}};
const Segment SGI364509[] = {{&S4084[13], 9}, {NULL}};
const Segment SGI364510[] = {{&S4084[33], 15}, {NULL}};
const Segment SGI364511[] = {{&S4084[49], 6}, {NULL}};
const Segment SGI364512[] = {{&S4084[33], 13}, {NULL}};
const Segment SGI364513[] = {{&S4085[69], 6}, {NULL}};
const Segment SGI364514[] = {{&S4085[8], 3}, {NULL}};
const Segment SGI364515[] = {{&S4085[13], 9}, {NULL}};
const Segment SGI364516[] = {{&S4085[33], 15}, {NULL}};
const Segment SGI364517[] = {{&S4085[49], 6}, {NULL}};
const Segment SGI364518[] = {{&S4085[33], 13}, {NULL}};
const Segment SGI364519[] = {{&S4086[117], 6}, {NULL}};
const Segment SGI364520[] = {{&S4086[8], 3}, {NULL}};
const Segment SGI364521[] = {{&S4086[13], 9}, {NULL}};
const Segment SGI364522[] = {{&S4086[33], 15}, {NULL}};
const Segment SGI364523[] = {{&S4086[49], 6}, {NULL}};
const Segment SGI364524[] = {{&S4086[33], 13}, {NULL}};
const Segment SGI364525[] = {{&S4087[69], 6}, {NULL}};
const Segment SGI364526[] = {{&S4087[8], 3}, {NULL}};
const Segment SGI364527[] = {{&S4087[13], 9}, {NULL}};
const Segment SGI364528[] = {{&S4087[33], 15}, {NULL}};
const Segment SGI364529[] = {{&S4087[49], 6}, {NULL}};
const Segment SGI364530[] = {{&S4087[33], 13}, {NULL}};
const Segment SGI364531[] = {{&S4088[69], 6}, {NULL}};
const Segment SGI364532[] = {{&S4088[8], 3}, {NULL}};
const Segment SGI364533[] = {{&S4088[13], 9}, {NULL}};
const Segment SGI364534[] = {{&S4088[33], 15}, {NULL}};
const Segment SGI364535[] = {{&S4088[49], 6}, {NULL}};
const Segment SGI364536[] = {{&S4088[33], 13}, {NULL}};
const Segment SGI364537[] = {{&S4089[69], 6}, {NULL}};
const Segment SGI364538[] = {{&S4089[8], 3}, {NULL}};
const Segment SGI364539[] = {{&S4089[13], 9}, {NULL}};
const Segment SGI364540[] = {{&S4089[33], 15}, {NULL}};
const Segment SGI364541[] = {{&S4089[49], 6}, {NULL}};
const Segment SGI364542[] = {{&S4089[33], 13}, {NULL}};
const Segment SGI364543[] = {{&S4090[67], 6}, {NULL}};
const Segment SGI364544[] = {{&S4090[8], 3}, {NULL}};
const Segment SGI364545[] = {{&S4090[13], 9}, {NULL}};
const Segment SGI364546[] = {{&S4090[33], 13}, {NULL}};
const Segment SGI364547[] = {{&S4091[67], 6}, {NULL}};
const Segment SGI364548[] = {{&S4091[8], 3}, {NULL}};
const Segment SGI364549[] = {{&S4091[13], 9}, {NULL}};
const Segment SGI364550[] = {{&S4091[33], 13}, {NULL}};
const Segment SGI364551[] = {{&S4092[67], 6}, {NULL}};
const Segment SGI364552[] = {{&S4092[8], 3}, {NULL}};
const Segment SGI364553[] = {{&S4092[13], 9}, {NULL}};
const Segment SGI364554[] = {{&S4092[33], 13}, {NULL}};
const Segment SGI364555[] = {{&S4093[67], 6}, {NULL}};
const Segment SGI364556[] = {{&S4093[8], 3}, {NULL}};
const Segment SGI364557[] = {{&S4093[13], 9}, {NULL}};
const Segment SGI364558[] = {{&S4093[33], 13}, {NULL}};
const Segment SGI364559[] = {{&S4094[67], 6}, {NULL}};
const Segment SGI364560[] = {{&S4094[8], 3}, {NULL}};
const Segment SGI364561[] = {{&S4094[13], 9}, {NULL}};
const Segment SGI364562[] = {{&S4094[33], 13}, {NULL}};
const Segment SGI364563[] = {{&S4095[68], 6}, {NULL}};
const Segment SGI364564[] = {{&S4095[8], 3}, {NULL}};
const Segment SGI364565[] = {{&S4095[13], 9}, {NULL}};
const Segment SGI364566[] = {{&S4095[33], 13}, {NULL}};
const Segment SGI364567[] = {{&S4096[119], 6}, {NULL}};
const Segment SGI364568[] = {{&S4096[8], 3}, {NULL}};
const Segment SGI364569[] = {{&S4096[13], 9}, {NULL}};
const Segment SGI364570[] = {{&S4096[33], 15}, {NULL}};
const Segment SGI364571[] = {{&S4096[49], 6}, {NULL}};
const Segment SGI364572[] = {{&S4096[33], 13}, {NULL}};
const Segment SGI364573[] = {{&S4097[67], 6}, {NULL}};
const Segment SGI364574[] = {{&S4097[8], 3}, {NULL}};
const Segment SGI364575[] = {{&S4097[13], 9}, {NULL}};
const Segment SGI364576[] = {{&S4097[33], 13}, {NULL}};
const Segment SGI364577[] = {{&S4098[69], 6}, {NULL}};
const Segment SGI364578[] = {{&S4098[8], 3}, {NULL}};
const Segment SGI364579[] = {{&S4098[13], 9}, {NULL}};
const Segment SGI364580[] = {{&S4098[33], 15}, {NULL}};
const Segment SGI364581[] = {{&S4098[49], 6}, {NULL}};
const Segment SGI364582[] = {{&S4098[33], 13}, {NULL}};
const Segment SGI364583[] = {{&S4099[69], 6}, {NULL}};
const Segment SGI364584[] = {{&S4099[8], 3}, {NULL}};
const Segment SGI364585[] = {{&S4099[13], 9}, {NULL}};
const Segment SGI364586[] = {{&S4099[33], 15}, {NULL}};
const Segment SGI364587[] = {{&S4099[49], 6}, {NULL}};
const Segment SGI364588[] = {{&S4099[33], 13}, {NULL}};
const Segment SGI364589[] = {{&S4100[66], 6}, {NULL}};
const Segment SGI364590[] = {{&S4100[8], 3}, {NULL}};
const Segment SGI364591[] = {{&S4100[13], 9}, {NULL}};
const Segment SGI364592[] = {{&S4100[33], 15}, {NULL}};
const Segment SGI364593[] = {{&S4100[49], 3}, {NULL}};
const Segment SGI364594[] = {{&S4100[33], 13}, {NULL}};
const Segment SGI364595[] = {{&S4101[81], 6}, {NULL}};
const Segment SGI364596[] = {{&S4101[8], 3}, {NULL}};
const Segment SGI364597[] = {{&S4101[13], 9}, {NULL}};
const Segment SGI364598[] = {{&S4101[33], 15}, {NULL}};
const Segment SGI364599[] = {{&S4101[49], 11}, {NULL}};
const Segment SGI364600[] = {{&S4101[33], 13}, {NULL}};
const Segment SGI364601[] = {{&S4102[69], 6}, {NULL}};
const Segment SGI364602[] = {{&S4102[8], 3}, {NULL}};
const Segment SGI364603[] = {{&S4102[13], 9}, {NULL}};
const Segment SGI364604[] = {{&S4102[33], 15}, {NULL}};
const Segment SGI364605[] = {{&S4102[49], 6}, {NULL}};
const Segment SGI364606[] = {{&S4102[33], 13}, {NULL}};
const Segment SGI364607[] = {{&S4103[69], 6}, {NULL}};
const Segment SGI364608[] = {{&S4103[8], 3}, {NULL}};
const Segment SGI364609[] = {{&S4103[13], 9}, {NULL}};
const Segment SGI364610[] = {{&S4103[33], 15}, {NULL}};
const Segment SGI364611[] = {{&S4103[49], 6}, {NULL}};
const Segment SGI364612[] = {{&S4103[33], 13}, {NULL}};
const Segment SGI364613[] = {{&S4104[69], 6}, {NULL}};
const Segment SGI364614[] = {{&S4104[8], 3}, {NULL}};
const Segment SGI364615[] = {{&S4104[13], 9}, {NULL}};
const Segment SGI364616[] = {{&S4104[33], 15}, {NULL}};
const Segment SGI364617[] = {{&S4104[49], 6}, {NULL}};
const Segment SGI364618[] = {{&S4104[33], 13}, {NULL}};
const Segment SGI364619[] = {{&S4105[71], 6}, {NULL}};
const Segment SGI364620[] = {{&S4105[8], 3}, {NULL}};
const Segment SGI364621[] = {{&S4105[13], 9}, {NULL}};
const Segment SGI364622[] = {{&S4105[33], 15}, {NULL}};
const Segment SGI364623[] = {{&S4105[49], 8}, {NULL}};
const Segment SGI364624[] = {{&S4105[33], 13}, {NULL}};
const Segment SGI364625[] = {{&S4106[66], 6}, {NULL}};
const Segment SGI364626[] = {{&S4106[8], 3}, {NULL}};
const Segment SGI364627[] = {{&S4106[13], 9}, {NULL}};
const Segment SGI364628[] = {{&S4106[33], 15}, {NULL}};
const Segment SGI364629[] = {{&S4106[49], 3}, {NULL}};
const Segment SGI364630[] = {{&S4106[33], 13}, {NULL}};
const Segment SGI364631[] = {{&S4107[119], 6}, {NULL}};
const Segment SGI364632[] = {{&S4107[8], 3}, {NULL}};
const Segment SGI364633[] = {{&S4107[13], 9}, {NULL}};
const Segment SGI364634[] = {{&S4107[33], 15}, {NULL}};
const Segment SGI364635[] = {{&S4107[49], 6}, {NULL}};
const Segment SGI364636[] = {{&S4107[33], 13}, {NULL}};
const Segment SGI364637[] = {{&S4108[118], 6}, {NULL}};
const Segment SGI364638[] = {{&S4108[8], 3}, {NULL}};
const Segment SGI364639[] = {{&S4108[13], 9}, {NULL}};
const Segment SGI364640[] = {{&S4108[33], 15}, {NULL}};
const Segment SGI364641[] = {{&S4108[49], 6}, {NULL}};
const Segment SGI364642[] = {{&S4108[33], 13}, {NULL}};
const Segment SGI364643[] = {{&S4109[69], 6}, {NULL}};
const Segment SGI364644[] = {{&S4109[8], 3}, {NULL}};
const Segment SGI364645[] = {{&S4109[13], 9}, {NULL}};
const Segment SGI364646[] = {{&S4109[33], 15}, {NULL}};
const Segment SGI364647[] = {{&S4109[49], 6}, {NULL}};
const Segment SGI364648[] = {{&S4109[33], 13}, {NULL}};
const Segment SGI364649[] = {{&S4110[69], 6}, {NULL}};
const Segment SGI364650[] = {{&S4110[8], 3}, {NULL}};
const Segment SGI364651[] = {{&S4110[13], 9}, {NULL}};
const Segment SGI364652[] = {{&S4110[33], 15}, {NULL}};
const Segment SGI364653[] = {{&S4110[49], 6}, {NULL}};
const Segment SGI364654[] = {{&S4110[33], 13}, {NULL}};
const Segment SGI364655[] = {{&S4111[69], 6}, {NULL}};
const Segment SGI364656[] = {{&S4111[8], 3}, {NULL}};
const Segment SGI364657[] = {{&S4111[13], 9}, {NULL}};
const Segment SGI364658[] = {{&S4111[33], 15}, {NULL}};
const Segment SGI364659[] = {{&S4111[49], 6}, {NULL}};
const Segment SGI364660[] = {{&S4111[33], 13}, {NULL}};
const Segment SGI364661[] = {{&S4112[69], 6}, {NULL}};
const Segment SGI364662[] = {{&S4112[8], 3}, {NULL}};
const Segment SGI364663[] = {{&S4112[13], 9}, {NULL}};
const Segment SGI364664[] = {{&S4112[33], 15}, {NULL}};
const Segment SGI364665[] = {{&S4112[49], 6}, {NULL}};
const Segment SGI364666[] = {{&S4112[33], 13}, {NULL}};
const Segment SGI364667[] = {{&S4113[69], 6}, {NULL}};
const Segment SGI364668[] = {{&S4113[8], 3}, {NULL}};
const Segment SGI364669[] = {{&S4113[13], 9}, {NULL}};
const Segment SGI364670[] = {{&S4113[33], 15}, {NULL}};
const Segment SGI364671[] = {{&S4113[49], 6}, {NULL}};
const Segment SGI364672[] = {{&S4113[33], 13}, {NULL}};
const Segment SGI364673[] = {{&S4114[115], 6}, {NULL}};
const Segment SGI364674[] = {{&S4114[8], 3}, {NULL}};
const Segment SGI364675[] = {{&S4114[13], 9}, {NULL}};
const Segment SGI364676[] = {{&S4114[33], 13}, {NULL}};
const Segment SGI364677[] = {{&S4115[69], 6}, {NULL}};
const Segment SGI364678[] = {{&S4115[8], 3}, {NULL}};
const Segment SGI364679[] = {{&S4115[13], 9}, {NULL}};
const Segment SGI364680[] = {{&S4115[33], 15}, {NULL}};
const Segment SGI364681[] = {{&S4115[49], 6}, {NULL}};
const Segment SGI364682[] = {{&S4115[33], 13}, {NULL}};
const Segment SGI364683[] = {{&S5467[13], 28}, {NULL}};
const Segment SGI364684[] = {{&S5467[55], 5}, {NULL}};
const Segment SGI364685[] = {{&S4116[69], 6}, {NULL}};
const Segment SGI364686[] = {{&S4116[8], 3}, {NULL}};
const Segment SGI364687[] = {{&S4116[13], 9}, {NULL}};
const Segment SGI364688[] = {{&S4116[33], 15}, {NULL}};
const Segment SGI364689[] = {{&S4116[49], 6}, {NULL}};
const Segment SGI364690[] = {{&S4116[33], 13}, {NULL}};
const Segment SGI364691[] = {{&S4117[66], 6}, {NULL}};
const Segment SGI364692[] = {{&S4117[8], 3}, {NULL}};
const Segment SGI364693[] = {{&S4117[13], 9}, {NULL}};
const Segment SGI364694[] = {{&S4117[33], 15}, {NULL}};
const Segment SGI364695[] = {{&S4117[49], 3}, {NULL}};
const Segment SGI364696[] = {{&S4117[33], 13}, {NULL}};
const Segment SGI364697[] = {{&S4118[118], 6}, {NULL}};
const Segment SGI364698[] = {{&S4118[8], 3}, {NULL}};
const Segment SGI364699[] = {{&S4118[13], 9}, {NULL}};
const Segment SGI364700[] = {{&S4118[33], 15}, {NULL}};
const Segment SGI364701[] = {{&S4118[49], 6}, {NULL}};
const Segment SGI364702[] = {{&S4118[33], 13}, {NULL}};
const Segment SGI364703[] = {{&S5474[13], 28}, {NULL}};
const Segment SGI364704[] = {{&S5474[55], 5}, {NULL}};
const Segment SGI364705[] = {{&S4120[66], 6}, {NULL}};
const Segment SGI364706[] = {{&S4120[8], 3}, {NULL}};
const Segment SGI364707[] = {{&S4120[13], 9}, {NULL}};
const Segment SGI364708[] = {{&S4120[33], 15}, {NULL}};
const Segment SGI364709[] = {{&S4120[49], 3}, {NULL}};
const Segment SGI364710[] = {{&S4120[33], 13}, {NULL}};
const Segment SGI364711[] = {{&S5475[13], 28}, {NULL}};
const Segment SGI364712[] = {{&S5475[55], 5}, {NULL}};
const Segment SGI364713[] = {{&S4121[69], 6}, {NULL}};
const Segment SGI364714[] = {{&S4121[8], 3}, {NULL}};
const Segment SGI364715[] = {{&S4121[13], 9}, {NULL}};
const Segment SGI364716[] = {{&S4121[33], 15}, {NULL}};
const Segment SGI364717[] = {{&S4121[49], 6}, {NULL}};
const Segment SGI364718[] = {{&S5476[13], 19}, {NULL}};
const Segment SGI364719[] = {{&S4121[33], 13}, {NULL}};
const Segment SGI364720[] = {{&S5476[46], 6}, {NULL}};
const Segment SGI364721[] = {{&S4122[69], 6}, {NULL}};
const Segment SGI364722[] = {{&S4122[8], 3}, {NULL}};
const Segment SGI364723[] = {{&S4122[13], 9}, {NULL}};
const Segment SGI364724[] = {{&S4122[33], 15}, {NULL}};
const Segment SGI364725[] = {{&S4122[49], 6}, {NULL}};
const Segment SGI364726[] = {{&S4122[33], 13}, {NULL}};
const Segment SGI364727[] = {{&S4123[69], 6}, {NULL}};
const Segment SGI364728[] = {{&S4123[8], 3}, {NULL}};
const Segment SGI364729[] = {{&S4123[13], 9}, {NULL}};
const Segment SGI364730[] = {{&S4123[33], 15}, {NULL}};
const Segment SGI364731[] = {{&S5480[13], 8}, {NULL}};
const Segment SGI364732[] = {{&S4123[49], 6}, {NULL}};
const Segment SGI364733[] = {{&S5480[35], 5}, {NULL}};
const Segment SGI364734[] = {{&S4123[33], 13}, {NULL}};
const Segment SGI364735[] = {{&S4124[69], 6}, {NULL}};
const Segment SGI364736[] = {{&S4124[8], 3}, {NULL}};
const Segment SGI364737[] = {{&S4124[13], 9}, {NULL}};
const Segment SGI364738[] = {{&S4124[33], 15}, {NULL}};
const Segment SGI364739[] = {{&S4124[49], 6}, {NULL}};
const Segment SGI364740[] = {{&S4124[33], 13}, {NULL}};
const Segment SGI364741[] = {{&S5482[13], 28}, {NULL}};
const Segment SGI364742[] = {{&S5482[55], 5}, {NULL}};
const Segment SGI364743[] = {{&S4125[69], 6}, {NULL}};
const Segment SGI364744[] = {{&S4125[8], 3}, {NULL}};
const Segment SGI364745[] = {{&S4125[13], 9}, {NULL}};
const Segment SGI364746[] = {{&S4125[33], 15}, {NULL}};
const Segment SGI364747[] = {{&S4125[49], 6}, {NULL}};
const Segment SGI364748[] = {{&S4125[33], 13}, {NULL}};
const Segment SGI364749[] = {{&S4126[69], 6}, {NULL}};
const Segment SGI364750[] = {{&S4126[8], 3}, {NULL}};
const Segment SGI364751[] = {{&S4126[13], 9}, {NULL}};
const Segment SGI364752[] = {{&S4126[33], 15}, {NULL}};
const Segment SGI364753[] = {{&S4126[49], 6}, {NULL}};
const Segment SGI364754[] = {{&S4126[33], 13}, {NULL}};
const Segment SGI364755[] = {{&S5486[13], 28}, {NULL}};
const Segment SGI364756[] = {{&S5486[55], 5}, {NULL}};
const Segment SGI364757[] = {{&S5487[13], 28}, {NULL}};
const Segment SGI364758[] = {{&S5487[55], 5}, {NULL}};
const Segment SGI364759[] = {{&S5488[13], 28}, {NULL}};
const Segment SGI364760[] = {{&S5488[55], 5}, {NULL}};
const Segment SGI364761[] = {{&S5294[57], 6}, {NULL}};
const Segment SGI364762[] = {{&S5294[8], 3}, {NULL}};
const Segment SGI364763[] = {{&S5294[13], 9}, {NULL}};
const Segment SGI364764[] = {{&S5294[92], 3}, {NULL}};
const Segment SGI364765[] = {{&S5294[30], 13}, {NULL}};
const Segment SGI364766[] = {{&S5295[57], 6}, {NULL}};
const Segment SGI364767[] = {{&S5295[8], 3}, {NULL}};
const Segment SGI364768[] = {{&S5295[13], 9}, {NULL}};
const Segment SGI364769[] = {{&S5295[92], 3}, {NULL}};
const Segment SGI364770[] = {{&S5295[30], 13}, {NULL}};
const Segment SGI364771[] = {{&S5296[103], 6}, {NULL}};
const Segment SGI364772[] = {{&S5296[8], 3}, {NULL}};
const Segment SGI364773[] = {{&S5296[13], 9}, {NULL}};
const Segment SGI364774[] = {{&S5296[92], 3}, {NULL}};
const Segment SGI364775[] = {{&S5296[30], 13}, {NULL}};
const Segment SGI364776[] = {{&S5297[57], 6}, {NULL}};
const Segment SGI364777[] = {{&S5297[8], 3}, {NULL}};
const Segment SGI364778[] = {{&S5297[13], 9}, {NULL}};
const Segment SGI364779[] = {{&S5297[92], 3}, {NULL}};
const Segment SGI364780[] = {{&S5297[30], 13}, {NULL}};
const Segment SGI364781[] = {{&S5298[57], 6}, {NULL}};
const Segment SGI364782[] = {{&S5298[8], 3}, {NULL}};
const Segment SGI364783[] = {{&S5298[13], 9}, {NULL}};
const Segment SGI364784[] = {{&S5298[92], 3}, {NULL}};
const Segment SGI364785[] = {{&S5298[30], 13}, {NULL}};
const Segment SGI364786[] = {{&S5299[57], 6}, {NULL}};
const Segment SGI364787[] = {{&S5299[8], 3}, {NULL}};
const Segment SGI364788[] = {{&S5299[13], 9}, {NULL}};
const Segment SGI364789[] = {{&S5299[92], 3}, {NULL}};
const Segment SGI364790[] = {{&S5299[30], 13}, {NULL}};
const Segment SGI364791[] = {{&S5300[57], 6}, {NULL}};
const Segment SGI364792[] = {{&S5300[8], 3}, {NULL}};
const Segment SGI364793[] = {{&S5300[13], 9}, {NULL}};
const Segment SGI364794[] = {{&S5300[92], 3}, {NULL}};
const Segment SGI364795[] = {{&S5300[30], 13}, {NULL}};
const Segment SGI364796[] = {{&S5301[57], 6}, {NULL}};
const Segment SGI364797[] = {{&S5301[8], 3}, {NULL}};
const Segment SGI364798[] = {{&S5301[13], 9}, {NULL}};
const Segment SGI364799[] = {{&S5301[92], 3}, {NULL}};
const Segment SGI364800[] = {{&S5301[30], 13}, {NULL}};
const Segment SGI364801[] = {{&S5302[107], 7}, {NULL}};
const Segment SGI364802[] = {{&S5302[8], 3}, {NULL}};
const Segment SGI364803[] = {{&S5302[13], 9}, {NULL}};
const Segment SGI364804[] = {{&S5302[30], 13}, {NULL}};
const Segment SGI364805[] = {{&S5303[59], 6}, {NULL}};
const Segment SGI364806[] = {{&S5303[8], 3}, {NULL}};
const Segment SGI364807[] = {{&S5303[13], 9}, {NULL}};
const Segment SGI364808[] = {{&S5303[97], 3}, {NULL}};
const Segment SGI364809[] = {{&S5303[30], 13}, {NULL}};
const Segment SGI364810[] = {{&S5304[97], 7}, {NULL}};
const Segment SGI364811[] = {{&S5304[8], 3}, {NULL}};
const Segment SGI364812[] = {{&S5304[13], 9}, {NULL}};
const Segment SGI364813[] = {{&S5304[30], 13}, {NULL}};
const Segment SGI364814[] = {{&S5305[66], 6}, {NULL}};
const Segment SGI364815[] = {{&S5305[8], 3}, {NULL}};
const Segment SGI364816[] = {{&S5305[13], 9}, {NULL}};
const Segment SGI364817[] = {{&S5305[33], 15}, {NULL}};
const Segment SGI364818[] = {{&S5305[101], 3}, {NULL}};
const Segment SGI364819[] = {{&S5305[49], 3}, {NULL}};
const Segment SGI364820[] = {{&S5305[33], 13}, {NULL}};
const Segment SGI364821[] = {{&S5306[59], 7}, {NULL}};
const Segment SGI364822[] = {{&S5306[8], 3}, {NULL}};
const Segment SGI364823[] = {{&S5306[13], 9}, {NULL}};
const Segment SGI364824[] = {{&S5306[98], 3}, {NULL}};
const Segment SGI364825[] = {{&S5307[57], 6}, {NULL}};
const Segment SGI364826[] = {{&S5307[8], 3}, {NULL}};
const Segment SGI364827[] = {{&S5307[13], 9}, {NULL}};
const Segment SGI364828[] = {{&S5307[91], 3}, {NULL}};
const Segment SGI364829[] = {{&S5308[59], 6}, {NULL}};
const Segment SGI364830[] = {{&S5308[8], 3}, {NULL}};
const Segment SGI364831[] = {{&S5308[13], 9}, {NULL}};
const Segment SGI364832[] = {{&S5308[97], 3}, {NULL}};
const Segment SGI364833[] = {{&S5308[30], 13}, {NULL}};
const Segment SGI364834[] = {{&S5309[57], 7}, {NULL}};
const Segment SGI364835[] = {{&S5309[8], 3}, {NULL}};
const Segment SGI364836[] = {{&S5309[13], 9}, {NULL}};
const Segment SGI364837[] = {{&S5309[93], 3}, {NULL}};
const Segment SGI364838[] = {{&S5310[97], 7}, {NULL}};
const Segment SGI364839[] = {{&S5310[8], 3}, {NULL}};
const Segment SGI364840[] = {{&S5310[13], 9}, {NULL}};
const Segment SGI364841[] = {{&S5310[30], 13}, {NULL}};
const Segment SGI364842[] = {{&S5311[56], 6}, {NULL}};
const Segment SGI364843[] = {{&S5311[8], 3}, {NULL}};
const Segment SGI364844[] = {{&S5311[13], 9}, {NULL}};
const Segment SGI364845[] = {{&S5311[90], 3}, {NULL}};
const Segment SGI364846[] = {{&S5312[101], 6}, {NULL}};
const Segment SGI364847[] = {{&S5312[8], 3}, {NULL}};
const Segment SGI364848[] = {{&S5312[13], 9}, {NULL}};
const Segment SGI364849[] = {{&S5312[90], 3}, {NULL}};
const Segment SGI364850[] = {{&S5313[57], 6}, {NULL}};
const Segment SGI364851[] = {{&S5313[8], 3}, {NULL}};
const Segment SGI364852[] = {{&S5313[13], 9}, {NULL}};
const Segment SGI364853[] = {{&S5313[94], 3}, {NULL}};
const Segment SGI364854[] = {{&S5314[97], 7}, {NULL}};
const Segment SGI364855[] = {{&S5314[8], 3}, {NULL}};
const Segment SGI364856[] = {{&S5314[13], 9}, {NULL}};
const Segment SGI364857[] = {{&S5314[30], 13}, {NULL}};
const Segment SGI364858[] = {{&S5315[59], 6}, {NULL}};
const Segment SGI364859[] = {{&S5315[8], 3}, {NULL}};
const Segment SGI364860[] = {{&S5315[13], 9}, {NULL}};
const Segment SGI364861[] = {{&S5315[97], 3}, {NULL}};
const Segment SGI364862[] = {{&S5315[30], 13}, {NULL}};
const Segment SGI364863[] = {{&S5316[59], 6}, {NULL}};
const Segment SGI364864[] = {{&S5316[8], 3}, {&S5316[8], 3}, {NULL}};
const Segment SGI364865[] = {{&S5316[13], 9}, {&S5316[13], 9}, {NULL}};
const Segment SGI364866[] = {{&S5316[30], 15}, {NULL}};
const Segment SGI364867[] = {{&S5316[193], 3}, {NULL}};
const Segment SGI364868[] = {{&S5316[138], 3}, {NULL}};
const Segment SGI364869[] = {{&S5316[30], 13}, {&S5316[30], 13}, {NULL}};
const Segment SGI364870[] = {{&S5317[59], 6}, {NULL}};
const Segment SGI364871[] = {{&S5317[8], 3}, {NULL}};
const Segment SGI364872[] = {{&S5317[13], 9}, {NULL}};
const Segment SGI364873[] = {{&S5317[96], 3}, {NULL}};
const Segment SGI364874[] = {{&S5317[30], 13}, {NULL}};
const Segment SGI364875[] = {{&S5318[96], 7}, {NULL}};
const Segment SGI364876[] = {{&S5318[8], 3}, {NULL}};
const Segment SGI364877[] = {{&S5318[13], 9}, {NULL}};
const Segment SGI364878[] = {{&S5318[30], 13}, {NULL}};
const Segment SGI364879[] = {{&S5319[57], 6}, {NULL}};
const Segment SGI364880[] = {{&S5319[8], 3}, {NULL}};
const Segment SGI364881[] = {{&S5319[13], 9}, {NULL}};
const Segment SGI364882[] = {{&S5319[93], 3}, {NULL}};
const Segment SGI364883[] = {{&S5320[59], 7}, {NULL}};
const Segment SGI364884[] = {{&S5320[8], 3}, {NULL}};
const Segment SGI364885[] = {{&S5320[13], 9}, {NULL}};
const Segment SGI364886[] = {{&S5320[97], 3}, {NULL}};
const Segment SGI364887[] = {{&S5321[59], 5}, {NULL}};
const Segment SGI364888[] = {{&S5321[8], 3}, {NULL}};
const Segment SGI364889[] = {{&S5321[13], 9}, {NULL}};
const Segment SGI364890[] = {{&S5321[94], 3}, {NULL}};
const Segment SGI364891[] = {{&S5321[30], 13}, {NULL}};
const Segment SGI364892[] = {{&S5360[59], 6}, {NULL}};
const Segment SGI364893[] = {{&S5360[8], 3}, {NULL}};
const Segment SGI364894[] = {{&S5360[13], 9}, {NULL}};
const Segment SGI364895[] = {{&S5360[94], 3}, {NULL}};
const Segment SGI364896[] = {{&S5360[30], 13}, {NULL}};
const Segment SGI364897[] = {{&S5361[59], 6}, {NULL}};
const Segment SGI364898[] = {{&S5361[8], 3}, {NULL}};
const Segment SGI364899[] = {{&S5361[13], 9}, {NULL}};
const Segment SGI364900[] = {{&S5361[94], 3}, {NULL}};
const Segment SGI364901[] = {{&S5361[30], 13}, {NULL}};
const Segment SGI364902[] = {{&S5362[109], 7}, {NULL}};
const Segment SGI364903[] = {{&S5362[8], 3}, {NULL}};
const Segment SGI364904[] = {{&S5362[13], 9}, {NULL}};
const Segment SGI364905[] = {{&S5362[30], 13}, {NULL}};
const Segment SGI364906[] = {{&S5363[59], 6}, {NULL}};
const Segment SGI364907[] = {{&S5363[8], 3}, {NULL}};
const Segment SGI364908[] = {{&S5363[13], 9}, {NULL}};
const Segment SGI364909[] = {{&S5363[97], 3}, {NULL}};
const Segment SGI364910[] = {{&S5363[30], 13}, {NULL}};
const Segment SGI364911[] = {{&S5364[69], 6}, {NULL}};
const Segment SGI364912[] = {{&S5364[8], 3}, {NULL}};
const Segment SGI364913[] = {{&S5364[13], 9}, {NULL}};
const Segment SGI364914[] = {{&S5364[33], 15}, {NULL}};
const Segment SGI364915[] = {{&S5364[104], 3}, {NULL}};
const Segment SGI364916[] = {{&S5364[49], 6}, {NULL}};
const Segment SGI364917[] = {{&S5364[33], 13}, {NULL}};
const Segment SGI364918[] = {{&S5365[114], 7}, {NULL}};
const Segment SGI364919[] = {{&S5365[8], 3}, {NULL}};
const Segment SGI364920[] = {{&S5365[13], 9}, {NULL}};
const Segment SGI364921[] = {{&S5365[30], 13}, {NULL}};
const Segment SGI364922[] = {{&S5367[69], 6}, {NULL}};
const Segment SGI364923[] = {{&S5367[8], 3}, {NULL}};
const Segment SGI364924[] = {{&S5367[13], 9}, {NULL}};
const Segment SGI364925[] = {{&S5367[33], 15}, {NULL}};
const Segment SGI364926[] = {{&S5367[106], 3}, {NULL}};
const Segment SGI364927[] = {{&S5367[49], 6}, {NULL}};
const Segment SGI364928[] = {{&S5367[33], 13}, {NULL}};
const Segment SGI364929[] = {{&S5369[59], 6}, {NULL}};
const Segment SGI364930[] = {{&S5369[8], 3}, {NULL}};
const Segment SGI364931[] = {{&S5369[13], 9}, {NULL}};
const Segment SGI364932[] = {{&S5369[97], 3}, {NULL}};
const Segment SGI364933[] = {{&S5369[30], 13}, {NULL}};
const Segment SGI364934[] = {{&S5370[115], 7}, {NULL}};
const Segment SGI364935[] = {{&S5370[8], 3}, {NULL}};
const Segment SGI364936[] = {{&S5370[13], 9}, {NULL}};
const Segment SGI364937[] = {{&S5370[30], 13}, {NULL}};
const Segment SGI364938[] = {{&S5371[69], 6}, {NULL}};
const Segment SGI364939[] = {{&S5371[8], 3}, {NULL}};
const Segment SGI364940[] = {{&S5371[13], 9}, {NULL}};
const Segment SGI364941[] = {{&S5371[33], 15}, {NULL}};
const Segment SGI364942[] = {{&S5371[104], 3}, {NULL}};
const Segment SGI364943[] = {{&S5371[49], 6}, {NULL}};
const Segment SGI364944[] = {{&S5371[33], 13}, {NULL}};
const Segment SGI364945[] = {{&S5372[59], 6}, {NULL}};
const Segment SGI364946[] = {{&S5372[8], 3}, {NULL}};
const Segment SGI364947[] = {{&S5372[13], 9}, {NULL}};
const Segment SGI364948[] = {{&S5372[96], 3}, {NULL}};
const Segment SGI364949[] = {{&S5372[30], 13}, {NULL}};
const Segment SGI364950[] = {{&S5373[59], 7}, {NULL}};
const Segment SGI364951[] = {{&S5373[8], 3}, {NULL}};
const Segment SGI364952[] = {{&S5373[13], 9}, {NULL}};
const Segment SGI364953[] = {{&S5373[97], 3}, {NULL}};
const Segment SGI364954[] = {{&S5374[59], 6}, {NULL}};
const Segment SGI364955[] = {{&S5374[8], 3}, {NULL}};
const Segment SGI364956[] = {{&S5374[13], 9}, {NULL}};
const Segment SGI364957[] = {{&S5374[96], 3}, {NULL}};
const Segment SGI364958[] = {{&S5374[30], 13}, {NULL}};
const Segment SGI364959[] = {{&S5375[59], 6}, {NULL}};
const Segment SGI364960[] = {{&S5375[8], 3}, {NULL}};
const Segment SGI364961[] = {{&S5375[13], 9}, {NULL}};
const Segment SGI364962[] = {{&S5375[96], 3}, {NULL}};
const Segment SGI364963[] = {{&S5375[30], 13}, {NULL}};
const Segment SGI364964[] = {{&S5376[114], 7}, {NULL}};
const Segment SGI364965[] = {{&S5376[8], 3}, {NULL}};
const Segment SGI364966[] = {{&S5376[13], 9}, {NULL}};
const Segment SGI364967[] = {{&S5376[30], 13}, {NULL}};
const Segment SGI364968[] = {{&S5377[95], 7}, {NULL}};
const Segment SGI364969[] = {{&S5377[8], 3}, {NULL}};
const Segment SGI364970[] = {{&S5377[13], 9}, {NULL}};
const Segment SGI364971[] = {{&S5377[30], 13}, {NULL}};
const Segment SGI364972[] = {{&S5378[69], 6}, {NULL}};
const Segment SGI364973[] = {{&S5378[8], 3}, {NULL}};
const Segment SGI364974[] = {{&S5378[13], 9}, {NULL}};
const Segment SGI364975[] = {{&S5378[33], 15}, {NULL}};
const Segment SGI364976[] = {{&S5378[107], 3}, {NULL}};
const Segment SGI364977[] = {{&S5378[49], 6}, {NULL}};
const Segment SGI364978[] = {{&S5378[33], 13}, {NULL}};
const Segment SGI364979[] = {{&S5380[69], 6}, {NULL}};
const Segment SGI364980[] = {{&S5380[8], 3}, {NULL}};
const Segment SGI364981[] = {{&S5380[13], 9}, {NULL}};
const Segment SGI364982[] = {{&S5380[33], 15}, {NULL}};
const Segment SGI364983[] = {{&S5380[106], 3}, {NULL}};
const Segment SGI364984[] = {{&S5380[49], 6}, {NULL}};
const Segment SGI364985[] = {{&S5380[33], 13}, {NULL}};
const Segment SGI364986[] = {{&S5381[96], 7}, {NULL}};
const Segment SGI364987[] = {{&S5381[8], 3}, {NULL}};
const Segment SGI364988[] = {{&S5381[13], 9}, {NULL}};
const Segment SGI364989[] = {{&S5381[30], 13}, {NULL}};
const Segment SGI364990[] = {{&S5382[69], 6}, {NULL}};
const Segment SGI364991[] = {{&S5382[8], 3}, {NULL}};
const Segment SGI364992[] = {{&S5382[13], 9}, {NULL}};
const Segment SGI364993[] = {{&S5382[33], 15}, {NULL}};
const Segment SGI364994[] = {{&S5382[106], 3}, {NULL}};
const Segment SGI364995[] = {{&S5382[49], 6}, {NULL}};
const Segment SGI364996[] = {{&S5382[33], 13}, {NULL}};
const Segment SGI364997[] = {{&S5383[69], 5}, {NULL}};
const Segment SGI364998[] = {{&S5383[8], 3}, {NULL}};
const Segment SGI364999[] = {{&S5383[13], 9}, {NULL}};
const Segment SGI365000[] = {{&S5383[33], 15}, {NULL}};
const Segment SGI365001[] = {{&S5383[104], 3}, {NULL}};
const Segment SGI365002[] = {{&S5383[49], 6}, {NULL}};
const Segment SGI365003[] = {{&S5383[33], 13}, {NULL}};
const Segment SGI365004[] = {{&S5384[59], 7}, {NULL}};
const Segment SGI365005[] = {{&S5384[8], 3}, {NULL}};
const Segment SGI365006[] = {{&S5384[13], 9}, {NULL}};
const Segment SGI365007[] = {{&S5384[97], 3}, {NULL}};
const Segment SGI365008[] = {{&S5385[69], 6}, {NULL}};
const Segment SGI365009[] = {{&S5385[8], 3}, {NULL}};
const Segment SGI365010[] = {{&S5385[13], 9}, {NULL}};
const Segment SGI365011[] = {{&S5385[33], 15}, {NULL}};
const Segment SGI365012[] = {{&S5385[107], 3}, {NULL}};
const Segment SGI365013[] = {{&S5385[49], 6}, {NULL}};
const Segment SGI365014[] = {{&S5385[33], 13}, {NULL}};
const Segment SGI365015[] = {{&S5386[69], 6}, {NULL}};
const Segment SGI365016[] = {{&S5386[8], 3}, {NULL}};
const Segment SGI365017[] = {{&S5386[13], 9}, {NULL}};
const Segment SGI365018[] = {{&S5386[33], 15}, {NULL}};
const Segment SGI365019[] = {{&S5386[104], 3}, {NULL}};
const Segment SGI365020[] = {{&S5386[49], 6}, {NULL}};
const Segment SGI365021[] = {{&S5386[33], 13}, {NULL}};
const Segment SGI365022[] = {{&S5387[69], 6}, {NULL}};
const Segment SGI365023[] = {{&S5387[8], 3}, {NULL}};
const Segment SGI365024[] = {{&S5387[13], 9}, {NULL}};
const Segment SGI365025[] = {{&S5387[33], 15}, {NULL}};
const Segment SGI365026[] = {{&S5387[106], 3}, {NULL}};
const Segment SGI365027[] = {{&S5387[49], 6}, {NULL}};
const Segment SGI365028[] = {{&S5387[33], 13}, {NULL}};
const Segment SGI365029[] = {{&S5388[59], 6}, {NULL}};
const Segment SGI365030[] = {{&S5388[8], 3}, {NULL}};
const Segment SGI365031[] = {{&S5388[13], 9}, {NULL}};
const Segment SGI365032[] = {{&S5388[30], 13}, {NULL}};
const Segment SGI365033[] = {{&S5389[61], 6}, {NULL}};
const Segment SGI365034[] = {{&S5389[8], 3}, {NULL}};
const Segment SGI365035[] = {{&S5389[13], 9}, {NULL}};
const Segment SGI365036[] = {{&S5389[99], 3}, {NULL}};
const Segment SGI365037[] = {{&S5390[59], 6}, {NULL}};
const Segment SGI365038[] = {{&S5390[8], 3}, {NULL}};
const Segment SGI365039[] = {{&S5390[13], 9}, {NULL}};
const Segment SGI365040[] = {{&S5390[96], 3}, {NULL}};
const Segment SGI365041[] = {{&S5390[30], 13}, {NULL}};
const Segment SGI365042[] = {{&S5391[114], 7}, {NULL}};
const Segment SGI365043[] = {{&S5391[8], 3}, {NULL}};
const Segment SGI365044[] = {{&S5391[13], 9}, {NULL}};
const Segment SGI365045[] = {{&S5391[30], 13}, {NULL}};
const Segment SGI365046[] = {{&S5392[59], 6}, {NULL}};
const Segment SGI365047[] = {{&S5392[8], 3}, {NULL}};
const Segment SGI365048[] = {{&S5392[13], 9}, {NULL}};
const Segment SGI365049[] = {{&S5392[96], 3}, {NULL}};
const Segment SGI365050[] = {{&S5392[30], 13}, {NULL}};
const Segment SGI365051[] = {{&S5393[114], 7}, {NULL}};
const Segment SGI365052[] = {{&S5393[8], 3}, {NULL}};
const Segment SGI365053[] = {{&S5393[13], 9}, {NULL}};
const Segment SGI365054[] = {{&S5393[30], 13}, {NULL}};
const Segment SGI365055[] = {{&S5394[57], 6}, {NULL}};
const Segment SGI365056[] = {{&S5394[8], 3}, {NULL}};
const Segment SGI365057[] = {{&S5394[13], 9}, {NULL}};
const Segment SGI365058[] = {{&S5394[93], 3}, {NULL}};
const Segment SGI365059[] = {{&S5395[69], 6}, {NULL}};
const Segment SGI365060[] = {{&S5395[8], 3}, {NULL}};
const Segment SGI365061[] = {{&S5395[13], 9}, {NULL}};
const Segment SGI365062[] = {{&S5395[33], 15}, {NULL}};
const Segment SGI365063[] = {{&S5395[106], 3}, {NULL}};
const Segment SGI365064[] = {{&S5395[49], 6}, {NULL}};
const Segment SGI365065[] = {{&S5395[33], 13}, {NULL}};
const Segment SGI365066[] = {{&S5396[61], 6}, {NULL}};
const Segment SGI365067[] = {{&S5396[8], 3}, {NULL}};
const Segment SGI365068[] = {{&S5396[13], 9}, {NULL}};
const Segment SGI365069[] = {{&S5396[96], 3}, {NULL}};
const Segment SGI365070[] = {{&S5397[59], 6}, {NULL}};
const Segment SGI365071[] = {{&S5397[8], 3}, {&S5397[8], 3}, {NULL}};
const Segment SGI365072[] = {{&S5397[13], 9}, {&S5397[13], 9}, {NULL}};
const Segment SGI365073[] = {{&S5397[30], 15}, {NULL}};
const Segment SGI365074[] = {{&S5397[191], 3}, {NULL}};
const Segment SGI365075[] = {{&S5397[137], 3}, {NULL}};
const Segment SGI365076[] = {{&S5397[30], 13}, {&S5397[30], 13}, {NULL}};
const Segment SGI365077[] = {{&S5450[54], 5}, {NULL}};
const Segment SGI365078[] = {{&S5450[8], 3}, {NULL}};
const Segment SGI365079[] = {{&S5450[13], 3}, {NULL}};
const Segment SGI365080[] = {{&S5455[47], 5}, {NULL}};
const Segment SGI365081[] = {{&S5455[8], 3}, {NULL}};
const Segment SGI365082[] = {{&S5455[13], 3}, {NULL}};
const Segment SGI365083[] = {{&S5461[52], 5}, {NULL}};
const Segment SGI365084[] = {{&S5461[8], 3}, {NULL}};
const Segment SGI365085[] = {{&S5461[13], 3}, {NULL}};
const Segment SGI365086[] = {{&S5462[52], 5}, {NULL}};
const Segment SGI365087[] = {{&S5462[8], 3}, {NULL}};
const Segment SGI365088[] = {{&S5462[13], 3}, {NULL}};
const Segment SGI365089[] = {{&S5463[79], 5}, {NULL}};
const Segment SGI365090[] = {{&S5463[8], 3}, {NULL}};
const Segment SGI365091[] = {{&S5463[13], 3}, {NULL}};
const Segment SGI365092[] = {{&S5470[54], 5}, {NULL}};
const Segment SGI365093[] = {{&S5470[8], 3}, {NULL}};
const Segment SGI365094[] = {{&S5470[13], 3}, {NULL}};
const Segment SGI365095[] = {{&S5478[48], 5}, {NULL}};
const Segment SGI365096[] = {{&S5478[8], 3}, {NULL}};
const Segment SGI365097[] = {{&S5478[13], 3}, {NULL}};
const Segment SGI365098[] = {{&S5483[42], 6}, {NULL}};
const Segment SGI365099[] = {{&S5483[8], 3}, {NULL}};
const Segment SGI365100[] = {{&S5483[13], 3}, {NULL}};
const Segment SGI365101[] = {{&S5485[42], 5}, {NULL}};
const Segment SGI365102[] = {{&S5485[8], 3}, {NULL}};
const Segment SGI365103[] = {{&S5485[13], 3}, {NULL}};
const Segment SGI365104[] = {{&S5495[44], 5}, {NULL}};
const Segment SGI365105[] = {{&S5495[8], 3}, {NULL}};
const Segment SGI365106[] = {{&S5495[13], 3}, {NULL}};
const Segment SGI365107[] = {{&S5495[70], 8}, {NULL}};
const Segment SGI365108[] = {{&S5496[44], 6}, {NULL}};
const Segment SGI365109[] = {{&S5496[8], 3}, {NULL}};
const Segment SGI365110[] = {{&S5496[13], 3}, {NULL}};
const Segment SGI365111[] = {{&S5504[44], 5}, {NULL}};
const Segment SGI365112[] = {{&S5504[8], 3}, {NULL}};
const Segment SGI365113[] = {{&S5504[13], 3}, {NULL}};
const Segment SGI365114[] = {{&S5504[70], 8}, {NULL}};
const Segment SGI365115[] = {{&S5510[44], 6}, {NULL}};
const Segment SGI365116[] = {{&S5510[8], 3}, {NULL}};
const Segment SGI365117[] = {{&S5510[13], 3}, {NULL}};
const Segment SGI365118[] = {{&S5510[71], 8}, {NULL}};
const Segment SGI365119[] = {{&S5517[44], 6}, {NULL}};
const Segment SGI365120[] = {{&S5517[8], 3}, {NULL}};
const Segment SGI365121[] = {{&S5517[13], 3}, {NULL}};
const Segment SGI365122[] = {{&S5517[71], 8}, {NULL}};
const Segment SGI365123[] = {{&S5520[44], 6}, {NULL}};
const Segment SGI365124[] = {{&S5520[8], 3}, {NULL}};
const Segment SGI365125[] = {{&S5520[13], 3}, {NULL}};
const Segment SGI365126[] = {{&S5523[44], 6}, {NULL}};
const Segment SGI365127[] = {{&S5523[8], 3}, {NULL}};
const Segment SGI365128[] = {{&S5523[13], 3}, {NULL}};
const Segment SGI365129[] = {{&S5524[44], 6}, {NULL}};
const Segment SGI365130[] = {{&S5524[8], 3}, {NULL}};
const Segment SGI365131[] = {{&S5524[13], 3}, {NULL}};
const Segment SGI365132[] = {{&S5524[71], 8}, {NULL}};
const Segment SGI365133[] = {{&S5533[91], 6}, {NULL}};
const Segment SGI365134[] = {{&S5533[8], 3}, {NULL}};
const Segment SGI365135[] = {{&S5533[13], 9}, {NULL}};
const Segment SGI365136[] = {{&S5534[90], 5}, {NULL}};
const Segment SGI365137[] = {{&S5534[8], 3}, {NULL}};
const Segment SGI365138[] = {{&S5534[13], 9}, {NULL}};
const Segment SGI365139[] = {{&S5535[90], 5}, {NULL}};
const Segment SGI365140[] = {{&S5535[8], 3}, {NULL}};
const Segment SGI365141[] = {{&S5535[13], 9}, {NULL}};
const Segment SGI365142[] = {{&S5536[88], 5}, {NULL}};
const Segment SGI365143[] = {{&S5536[8], 3}, {NULL}};
const Segment SGI365144[] = {{&S5536[13], 9}, {NULL}};
const Segment SGI365145[] = {{&S5537[90], 5}, {NULL}};
const Segment SGI365146[] = {{&S5537[8], 3}, {NULL}};
const Segment SGI365147[] = {{&S5537[13], 9}, {NULL}};
const Segment SGI365148[] = {{&S5538[90], 5}, {NULL}};
const Segment SGI365149[] = {{&S5538[8], 3}, {NULL}};
const Segment SGI365150[] = {{&S5538[13], 9}, {NULL}};
const Segment SGI365151[] = {{&S5539[88], 5}, {NULL}};
const Segment SGI365152[] = {{&S5539[8], 3}, {NULL}};
const Segment SGI365153[] = {{&S5539[13], 9}, {NULL}};
const Segment SGI365154[] = {{&S5540[92], 4}, {NULL}};
const Segment SGI365155[] = {{&S5540[8], 3}, {NULL}};
const Segment SGI365156[] = {{&S5540[13], 9}, {NULL}};
const Segment SGI365157[] = {{&S5541[90], 5}, {NULL}};
const Segment SGI365158[] = {{&S5541[8], 3}, {NULL}};
const Segment SGI365159[] = {{&S5541[13], 9}, {NULL}};
const Segment SGI365160[] = {{&S5542[90], 5}, {NULL}};
const Segment SGI365161[] = {{&S5542[8], 3}, {NULL}};
const Segment SGI365162[] = {{&S5542[13], 9}, {NULL}};
const Segment SGI365163[] = {{&S5544[91], 6}, {NULL}};
const Segment SGI365164[] = {{&S5544[8], 3}, {NULL}};
const Segment SGI365165[] = {{&S5544[13], 9}, {NULL}};
const Segment SGI365166[] = {{&S5545[92], 6}, {NULL}};
const Segment SGI365167[] = {{&S5545[8], 3}, {NULL}};
const Segment SGI365168[] = {{&S5545[13], 9}, {NULL}};
const Segment SGI365169[] = {{&S5547[90], 5}, {NULL}};
const Segment SGI365170[] = {{&S5547[8], 3}, {NULL}};
const Segment SGI365171[] = {{&S5547[13], 9}, {NULL}};
const Segment SGI365172[] = {{&S5549[8], 3}, {NULL}};
const Segment SGI365173[] = {{&S5549[13], 9}, {NULL}};
const Segment SGI365174[] = {{&S5550[93], 3}, {NULL}};
const Segment SGI365175[] = {{&S5550[8], 3}, {NULL}};
const Segment SGI365176[] = {{&S5550[13], 9}, {NULL}};
const Segment SGI365177[] = {{&S5558[93], 5}, {NULL}};
const Segment SGI365178[] = {{&S5558[8], 3}, {NULL}};
const Segment SGI365179[] = {{&S5558[13], 9}, {NULL}};
const Segment SGI365180[] = {{&S5560[90], 5}, {NULL}};
const Segment SGI365181[] = {{&S5560[8], 3}, {NULL}};
const Segment SGI365182[] = {{&S5560[13], 9}, {NULL}};
const Segment SGI365183[] = {{&S5561[92], 5}, {NULL}};
const Segment SGI365184[] = {{&S5561[8], 3}, {NULL}};
const Segment SGI365185[] = {{&S5561[13], 9}, {NULL}};
const Segment SGI365186[] = {{&S5562[91], 6}, {NULL}};
const Segment SGI365187[] = {{&S5562[8], 3}, {NULL}};
const Segment SGI365188[] = {{&S5562[13], 9}, {NULL}};
const Segment SGI365189[] = {{&S5564[92], 6}, {NULL}};
const Segment SGI365190[] = {{&S5564[8], 3}, {NULL}};
const Segment SGI365191[] = {{&S5564[13], 9}, {NULL}};
const Segment SGI365192[] = {{&S5570[93], 5}, {NULL}};
const Segment SGI365193[] = {{&S5570[8], 3}, {NULL}};
const Segment SGI365194[] = {{&S5570[13], 9}, {NULL}};
const Segment SGI365195[] = {{&S5571[57], 6}, {NULL}};
const Segment SGI365196[] = {{&S5571[8], 3}, {NULL}};
const Segment SGI365197[] = {{&S5571[13], 9}, {NULL}};
const Segment SGI365198[] = {{&S5573[60], 4}, {NULL}};
const Segment SGI365199[] = {{&S5573[8], 3}, {NULL}};
const Segment SGI365200[] = {{&S5573[13], 9}, {NULL}};
const Segment SGI365201[] = {{&S5575[91], 5}, {NULL}};
const Segment SGI365202[] = {{&S5575[8], 3}, {NULL}};
const Segment SGI365203[] = {{&S5575[13], 9}, {NULL}};
const Segment SGI365204[] = {{&S5579[90], 4}, {NULL}};
const Segment SGI365205[] = {{&S5579[8], 3}, {NULL}};
const Segment SGI365206[] = {{&S5579[13], 9}, {NULL}};
const Segment SGI365207[] = {{&S5581[62], 5}, {NULL}};
const Segment SGI365208[] = {{&S5581[8], 3}, {NULL}};
const Segment SGI365209[] = {{&S5581[13], 9}, {NULL}};
const Segment SGI365210[] = {{&S5584[59], 6}, {NULL}};
const Segment SGI365211[] = {{&S5584[8], 3}, {NULL}};
const Segment SGI365212[] = {{&S5584[13], 9}, {NULL}};
const Segment SGI365213[] = {{&S5585[92], 4}, {NULL}};
const Segment SGI365214[] = {{&S5585[8], 3}, {NULL}};
const Segment SGI365215[] = {{&S5585[13], 9}, {NULL}};
const Segment SGI365216[] = {{&S5587[57], 5}, {NULL}};
const Segment SGI365217[] = {{&S5587[8], 3}, {NULL}};
const Segment SGI365218[] = {{&S5587[13], 9}, {NULL}};
const Segment SGI365219[] = {{&S5591[90], 5}, {NULL}};
const Segment SGI365220[] = {{&S5591[8], 3}, {NULL}};
const Segment SGI365221[] = {{&S5591[13], 9}, {NULL}};
const Segment SGI365222[] = {{&S5595[93], 5}, {NULL}};
const Segment SGI365223[] = {{&S5595[8], 3}, {NULL}};
const Segment SGI365224[] = {{&S5595[13], 9}, {NULL}};
const Segment SGI365225[] = {{&S5598[52], 6}, {NULL}};
const Segment SGI365226[] = {{&S5598[8], 3}, {NULL}};
const Segment SGI365227[] = {{&S5598[13], 9}, {NULL}};
const Segment SGI365228[] = {{&S5598[89], 3}, {NULL}};
const Segment SGI365229[] = {{&S5599[52], 6}, {NULL}};
const Segment SGI365230[] = {{&S5599[8], 3}, {NULL}};
const Segment SGI365231[] = {{&S5599[13], 9}, {NULL}};
const Segment SGI365232[] = {{&S5600[90], 5}, {NULL}};
const Segment SGI365233[] = {{&S5600[8], 3}, {NULL}};
const Segment SGI365234[] = {{&S5600[13], 9}, {NULL}};
const Segment SGI365235[] = {{&S5605[52], 6}, {NULL}};
const Segment SGI365236[] = {{&S5605[8], 3}, {NULL}};
const Segment SGI365237[] = {{&S5605[13], 9}, {NULL}};
const Segment SGI365238[] = {{&S5605[89], 3}, {NULL}};
const Segment SGI365239[] = {{&S5606[59], 5}, {NULL}};
const Segment SGI365240[] = {{&S5606[8], 3}, {NULL}};
const Segment SGI365241[] = {{&S5606[13], 9}, {NULL}};
const Segment SGI365242[] = {{&S5612[52], 6}, {NULL}};
const Segment SGI365243[] = {{&S5612[8], 3}, {NULL}};
const Segment SGI365244[] = {{&S5612[13], 9}, {NULL}};
const Segment SGI365245[] = {{&S5616[59], 6}, {NULL}};
const Segment SGI365246[] = {{&S5616[8], 3}, {NULL}};
const Segment SGI365247[] = {{&S5616[13], 9}, {NULL}};
const Segment SGI365248[] = {{&S5708[67], 6}, {NULL}};
const Segment SGI365249[] = {{&S5708[8], 3}, {NULL}};
const Segment SGI365250[] = {{&S5708[13], 9}, {NULL}};
const Segment SGI365251[] = {{&S5708[33], 13}, {NULL}};
const Segment SGI365252[] = {{&S5708[104], 3}, {NULL}};
const Segment SGI365253[] = {{&S5709[57], 6}, {NULL}};
const Segment SGI365254[] = {{&S5709[8], 3}, {NULL}};
const Segment SGI365255[] = {{&S5709[13], 9}, {NULL}};
const Segment SGI365256[] = {{&S5709[94], 3}, {NULL}};
const Segment SGI365257[] = {{&S5709[30], 13}, {NULL}};
const Segment SGI365258[] = {{&S5710[57], 6}, {NULL}};
const Segment SGI365259[] = {{&S5710[8], 3}, {NULL}};
const Segment SGI365260[] = {{&S5710[13], 9}, {NULL}};
const Segment SGI365261[] = {{&S5710[92], 3}, {NULL}};
const Segment SGI365262[] = {{&S5710[30], 13}, {NULL}};
const Segment SGI365263[] = {{&S5711[57], 6}, {NULL}};
const Segment SGI365264[] = {{&S5711[8], 3}, {NULL}};
const Segment SGI365265[] = {{&S5711[13], 9}, {NULL}};
const Segment SGI365266[] = {{&S5711[94], 3}, {NULL}};
const Segment SGI365267[] = {{&S5711[30], 13}, {NULL}};
const Segment SGI365268[] = {{&S5712[103], 6}, {NULL}};
const Segment SGI365269[] = {{&S5712[8], 3}, {NULL}};
const Segment SGI365270[] = {{&S5712[13], 9}, {NULL}};
const Segment SGI365271[] = {{&S5712[92], 3}, {NULL}};
const Segment SGI365272[] = {{&S5712[30], 13}, {NULL}};
const Segment SGI365273[] = {{&S5713[57], 6}, {NULL}};
const Segment SGI365274[] = {{&S5713[8], 3}, {NULL}};
const Segment SGI365275[] = {{&S5713[13], 9}, {NULL}};
const Segment SGI365276[] = {{&S5713[94], 3}, {NULL}};
const Segment SGI365277[] = {{&S5713[30], 13}, {NULL}};
const Segment SGI365278[] = {{&S5714[59], 6}, {NULL}};
const Segment SGI365279[] = {{&S5714[8], 3}, {NULL}};
const Segment SGI365280[] = {{&S5714[13], 9}, {NULL}};
const Segment SGI365281[] = {{&S5714[96], 3}, {NULL}};
const Segment SGI365282[] = {{&S5714[30], 13}, {NULL}};
const Segment SGI365283[] = {{&S5715[59], 6}, {NULL}};
const Segment SGI365284[] = {{&S5715[8], 3}, {NULL}};
const Segment SGI365285[] = {{&S5715[13], 9}, {NULL}};
const Segment SGI365286[] = {{&S5715[97], 3}, {NULL}};
const Segment SGI365287[] = {{&S5715[30], 13}, {NULL}};
const Segment SGI365288[] = {{&S5716[97], 7}, {NULL}};
const Segment SGI365289[] = {{&S5716[8], 3}, {NULL}};
const Segment SGI365290[] = {{&S5716[13], 9}, {NULL}};
const Segment SGI365291[] = {{&S5716[30], 13}, {NULL}};
const Segment SGI365292[] = {{&S5717[59], 6}, {NULL}};
const Segment SGI365293[] = {{&S5717[8], 3}, {NULL}};
const Segment SGI365294[] = {{&S5717[13], 9}, {NULL}};
const Segment SGI365295[] = {{&S5717[96], 3}, {NULL}};
const Segment SGI365296[] = {{&S5717[30], 13}, {NULL}};
const Segment SGI365297[] = {{&S5718[57], 6}, {NULL}};
const Segment SGI365298[] = {{&S5718[8], 3}, {NULL}};
const Segment SGI365299[] = {{&S5718[13], 9}, {NULL}};
const Segment SGI365300[] = {{&S5718[94], 3}, {NULL}};
const Segment SGI365301[] = {{&S5718[30], 13}, {NULL}};
const Segment SGI365302[] = {{&S5719[57], 6}, {NULL}};
const Segment SGI365303[] = {{&S5719[8], 3}, {NULL}};
const Segment SGI365304[] = {{&S5719[13], 9}, {NULL}};
const Segment SGI365305[] = {{&S5719[94], 3}, {NULL}};
const Segment SGI365306[] = {{&S5719[30], 13}, {NULL}};
const Segment SGI365307[] = {{&S5720[94], 7}, {NULL}};
const Segment SGI365308[] = {{&S5720[8], 3}, {NULL}};
const Segment SGI365309[] = {{&S5720[13], 9}, {NULL}};
const Segment SGI365310[] = {{&S5720[30], 13}, {NULL}};
const Segment SGI365311[] = {{&S5721[107], 6}, {NULL}};
const Segment SGI365312[] = {{&S5721[8], 3}, {NULL}};
const Segment SGI365313[] = {{&S5721[13], 9}, {NULL}};
const Segment SGI365314[] = {{&S5721[96], 3}, {NULL}};
const Segment SGI365315[] = {{&S5721[30], 13}, {NULL}};
const Segment SGI365316[] = {{&S5722[59], 6}, {NULL}};
const Segment SGI365317[] = {{&S5722[8], 3}, {NULL}};
const Segment SGI365318[] = {{&S5722[13], 9}, {NULL}};
const Segment SGI365319[] = {{&S5722[97], 3}, {NULL}};
const Segment SGI365320[] = {{&S5722[30], 13}, {NULL}};
const Segment SGI365321[] = {{&S5723[97], 7}, {NULL}};
const Segment SGI365322[] = {{&S5723[8], 3}, {NULL}};
const Segment SGI365323[] = {{&S5723[13], 9}, {NULL}};
const Segment SGI365324[] = {{&S5723[30], 13}, {NULL}};
const Segment SGI365325[] = {{&S5724[97], 7}, {NULL}};
const Segment SGI365326[] = {{&S5724[8], 3}, {NULL}};
const Segment SGI365327[] = {{&S5724[13], 9}, {NULL}};
const Segment SGI365328[] = {{&S5724[30], 13}, {NULL}};
const Segment SGI365329[] = {{&S5725[59], 6}, {NULL}};
const Segment SGI365330[] = {{&S5725[8], 3}, {NULL}};
const Segment SGI365331[] = {{&S5725[13], 9}, {NULL}};
const Segment SGI365332[] = {{&S5725[96], 3}, {NULL}};
const Segment SGI365333[] = {{&S5725[30], 13}, {NULL}};
const Segment SGI365334[] = {{&S5726[59], 6}, {NULL}};
const Segment SGI365335[] = {{&S5726[8], 3}, {NULL}};
const Segment SGI365336[] = {{&S5726[13], 9}, {NULL}};
const Segment SGI365337[] = {{&S5726[96], 3}, {NULL}};
const Segment SGI365338[] = {{&S5726[30], 13}, {NULL}};
const Segment SGI365339[] = {{&S5727[59], 6}, {NULL}};
const Segment SGI365340[] = {{&S5727[8], 3}, {NULL}};
const Segment SGI365341[] = {{&S5727[13], 9}, {NULL}};
const Segment SGI365342[] = {{&S5727[97], 3}, {NULL}};
const Segment SGI365343[] = {{&S5727[30], 13}, {NULL}};
const Segment SGI365344[] = {{&S5728[97], 7}, {NULL}};
const Segment SGI365345[] = {{&S5728[8], 3}, {NULL}};
const Segment SGI365346[] = {{&S5728[13], 9}, {NULL}};
const Segment SGI365347[] = {{&S5728[30], 13}, {NULL}};
const Segment SGI365348[] = {{&S5729[114], 7}, {NULL}};
const Segment SGI365349[] = {{&S5729[8], 3}, {NULL}};
const Segment SGI365350[] = {{&S5729[13], 9}, {NULL}};
const Segment SGI365351[] = {{&S5729[30], 13}, {NULL}};
const Segment SGI365352[] = {{&S5730[56], 6}, {NULL}};
const Segment SGI365353[] = {{&S5730[8], 3}, {NULL}};
const Segment SGI365354[] = {{&S5730[90], 3}, {NULL}};
const Segment SGI365355[] = {{&S5730[28], 13}, {NULL}};
const Segment SGI365356[] = {{&S5731[57], 6}, {NULL}};
const Segment SGI365357[] = {{&S5731[8], 3}, {NULL}};
const Segment SGI365358[] = {{&S5731[13], 9}, {NULL}};
const Segment SGI365359[] = {{&S5731[92], 3}, {NULL}};
const Segment SGI365360[] = {{&S5731[30], 13}, {NULL}};
const Segment SGI365361[] = {{&S5732[96], 7}, {NULL}};
const Segment SGI365362[] = {{&S5732[8], 3}, {NULL}};
const Segment SGI365363[] = {{&S5732[13], 9}, {NULL}};
const Segment SGI365364[] = {{&S5732[30], 13}, {NULL}};
const Segment SGI365365[] = {{&S5733[59], 7}, {NULL}};
const Segment SGI365366[] = {{&S5733[8], 3}, {NULL}};
const Segment SGI365367[] = {{&S5733[13], 9}, {NULL}};
const Segment SGI365368[] = {{&S5733[98], 3}, {NULL}};
const Segment SGI365369[] = {{&S5734[111], 7}, {NULL}};
const Segment SGI365370[] = {{&S5734[8], 3}, {NULL}};
const Segment SGI365371[] = {{&S5734[13], 9}, {NULL}};
const Segment SGI365372[] = {{&S5734[98], 3}, {NULL}};
const Segment SGI365373[] = {{&S5735[59], 6}, {NULL}};
const Segment SGI365374[] = {{&S5735[8], 3}, {NULL}};
const Segment SGI365375[] = {{&S5735[13], 9}, {NULL}};
const Segment SGI365376[] = {{&S5735[94], 3}, {NULL}};
const Segment SGI365377[] = {{&S5735[30], 13}, {NULL}};
const Segment SGI365378[] = {{&S5736[110], 7}, {NULL}};
const Segment SGI365379[] = {{&S5736[8], 3}, {NULL}};
const Segment SGI365380[] = {{&S5736[13], 9}, {NULL}};
const Segment SGI365381[] = {{&S5736[97], 3}, {NULL}};
const Segment SGI365382[] = {{&S5737[59], 6}, {NULL}};
const Segment SGI365383[] = {{&S5737[8], 3}, {NULL}};
const Segment SGI365384[] = {{&S5737[13], 9}, {NULL}};
const Segment SGI365385[] = {{&S5737[96], 3}, {NULL}};
const Segment SGI365386[] = {{&S5737[30], 13}, {NULL}};
const Segment SGI365387[] = {{&S5738[96], 7}, {NULL}};
const Segment SGI365388[] = {{&S5738[8], 3}, {NULL}};
const Segment SGI365389[] = {{&S5738[13], 9}, {NULL}};
const Segment SGI365390[] = {{&S5738[30], 13}, {NULL}};
const Segment SGI365391[] = {{&S5739[67], 6}, {NULL}};
const Segment SGI365392[] = {{&S5739[8], 3}, {NULL}};
const Segment SGI365393[] = {{&S5739[13], 9}, {NULL}};
const Segment SGI365394[] = {{&S5739[33], 13}, {NULL}};
const Segment SGI365395[] = {{&S5739[72], 3}, {NULL}};
const Segment SGI365396[] = {{&S5740[57], 6}, {NULL}};
const Segment SGI365397[] = {{&S5740[8], 3}, {NULL}};
const Segment SGI365398[] = {{&S5740[13], 9}, {NULL}};
const Segment SGI365399[] = {{&S5740[92], 3}, {NULL}};
const Segment SGI365400[] = {{&S5740[30], 13}, {NULL}};
const Segment SGI365401[] = {{&S5741[61], 6}, {NULL}};
const Segment SGI365402[] = {{&S5741[8], 3}, {NULL}};
const Segment SGI365403[] = {{&S5741[13], 9}, {NULL}};
const Segment SGI365404[] = {{&S5741[99], 3}, {NULL}};
const Segment SGI365405[] = {{&S5742[56], 6}, {NULL}};
const Segment SGI365406[] = {{&S5742[8], 3}, {NULL}};
const Segment SGI365407[] = {{&S5742[13], 9}, {NULL}};
const Segment SGI365408[] = {{&S5742[92], 3}, {NULL}};
const Segment SGI365409[] = {{&S5743[61], 6}, {NULL}};
const Segment SGI365410[] = {{&S5743[8], 3}, {NULL}};
const Segment SGI365411[] = {{&S5743[13], 9}, {NULL}};
const Segment SGI365412[] = {{&S5743[98], 3}, {NULL}};
const Segment SGI365413[] = {{&S5744[109], 6}, {NULL}};
const Segment SGI365414[] = {{&S5744[8], 3}, {NULL}};
const Segment SGI365415[] = {{&S5744[13], 9}, {NULL}};
const Segment SGI365416[] = {{&S5744[96], 3}, {NULL}};
const Segment SGI365417[] = {{&S5744[30], 13}, {NULL}};
const Segment SGI365418[] = {{&S5745[109], 6}, {NULL}};
const Segment SGI365419[] = {{&S5745[8], 3}, {NULL}};
const Segment SGI365420[] = {{&S5745[13], 9}, {NULL}};
const Segment SGI365421[] = {{&S5745[96], 3}, {NULL}};
const Segment SGI365422[] = {{&S5745[30], 13}, {NULL}};
const Segment SGI365423[] = {{&S5749[59], 6}, {NULL}};
const Segment SGI365424[] = {{&S5749[8], 3}, {&S5749[8], 3}, {NULL}};
const Segment SGI365425[] = {{&S5749[13], 9}, {&S5749[13], 9}, {NULL}};
const Segment SGI365426[] = {{&S5749[30], 15}, {NULL}};
const Segment SGI365427[] = {{&S5749[191], 3}, {NULL}};
const Segment SGI365428[] = {{&S5749[137], 3}, {NULL}};
const Segment SGI365429[] = {{&S5749[30], 13}, {&S5749[30], 13}, {NULL}};
const Segment SGI365430[] = {{&S6532[8], 3}, {NULL}};
const Segment SGI365431[] = {{&S6532[13], 5}, {NULL}};
const Segment SGI365432[] = {{&S9557[13], 43}, {NULL}};
const Segment SGI365433[] = {{&S9557[70], 5}, {NULL}};
const Segment SGI365434[] = {{&S9557[104], 3}, {NULL}};
const Segment SGI365435[] = {{&S9558[13], 43}, {NULL}};
const Segment SGI365436[] = {{&S9558[70], 5}, {NULL}};
const Segment SGI365437[] = {{&S9558[104], 3}, {NULL}};
const Segment SGI365438[] = {{&S9559[13], 43}, {NULL}};
const Segment SGI365439[] = {{&S9559[70], 5}, {NULL}};
const Segment SGI365440[] = {{&S9559[104], 3}, {NULL}};
const Segment SGI365441[] = {{&S9561[13], 43}, {NULL}};
const Segment SGI365442[] = {{&S9561[70], 5}, {NULL}};
const Segment SGI365443[] = {{&S9561[104], 3}, {NULL}};
const Segment SGI365444[] = {{&S9563[13], 43}, {NULL}};
const Segment SGI365445[] = {{&S9563[70], 5}, {NULL}};
const Segment SGI365446[] = {{&S9563[104], 3}, {NULL}};
const Segment SGI365447[] = {{&S9566[13], 43}, {NULL}};
const Segment SGI365448[] = {{&S9566[70], 5}, {NULL}};
const Segment SGI365449[] = {{&S9566[104], 3}, {NULL}};
const Segment SGI365450[] = {{&S9585[13], 43}, {NULL}};
const Segment SGI365451[] = {{&S9585[70], 5}, {NULL}};
const Segment SGI365452[] = {{&S9585[104], 3}, {NULL}};
const Segment SGI365453[] = {{&S9594[13], 43}, {NULL}};
const Segment SGI365454[] = {{&S9594[70], 5}, {NULL}};
const Segment SGI365455[] = {{&S9594[104], 3}, {NULL}};
const Segment SGI365456[] = {{&S9219[69], 6}, {NULL}};
const Segment SGI365457[] = {{&S9219[8], 3}, {NULL}};
const Segment SGI365458[] = {{&S9219[13], 9}, {NULL}};
const Segment SGI365459[] = {{&S9219[33], 15}, {NULL}};
const Segment SGI365460[] = {{&S9219[49], 6}, {NULL}};
const Segment SGI365461[] = {{&S9219[33], 13}, {NULL}};
const Segment SGI365462[] = {{&S9220[69], 6}, {NULL}};
const Segment SGI365463[] = {{&S9220[8], 3}, {NULL}};
const Segment SGI365464[] = {{&S9220[13], 9}, {NULL}};
const Segment SGI365465[] = {{&S9220[33], 15}, {NULL}};
const Segment SGI365466[] = {{&S9220[49], 6}, {NULL}};
const Segment SGI365467[] = {{&S9220[33], 13}, {NULL}};
const Segment SGI365468[] = {{&S9221[69], 6}, {NULL}};
const Segment SGI365469[] = {{&S9221[8], 3}, {NULL}};
const Segment SGI365470[] = {{&S9221[13], 9}, {NULL}};
const Segment SGI365471[] = {{&S9221[33], 15}, {NULL}};
const Segment SGI365472[] = {{&S9221[49], 6}, {NULL}};
const Segment SGI365473[] = {{&S9221[33], 13}, {NULL}};
const Segment SGI365474[] = {{&S9222[69], 6}, {NULL}};
const Segment SGI365475[] = {{&S9222[8], 3}, {NULL}};
const Segment SGI365476[] = {{&S9222[13], 9}, {NULL}};
const Segment SGI365477[] = {{&S9222[33], 15}, {NULL}};
const Segment SGI365478[] = {{&S9222[49], 6}, {NULL}};
const Segment SGI365479[] = {{&S9222[33], 13}, {NULL}};
const Segment SGI365480[] = {{&S9223[69], 6}, {NULL}};
const Segment SGI365481[] = {{&S9223[8], 3}, {NULL}};
const Segment SGI365482[] = {{&S9223[13], 9}, {NULL}};
const Segment SGI365483[] = {{&S9223[33], 15}, {NULL}};
const Segment SGI365484[] = {{&S9223[49], 6}, {NULL}};
const Segment SGI365485[] = {{&S9223[33], 13}, {NULL}};
const Segment SGI365486[] = {{&S9224[69], 6}, {NULL}};
const Segment SGI365487[] = {{&S9224[8], 3}, {NULL}};
const Segment SGI365488[] = {{&S9224[13], 9}, {NULL}};
const Segment SGI365489[] = {{&S9224[33], 15}, {NULL}};
const Segment SGI365490[] = {{&S9224[49], 6}, {NULL}};
const Segment SGI365491[] = {{&S9224[33], 13}, {NULL}};
const Segment SGI365492[] = {{&S9225[69], 6}, {NULL}};
const Segment SGI365493[] = {{&S9225[8], 3}, {NULL}};
const Segment SGI365494[] = {{&S9225[13], 9}, {NULL}};
const Segment SGI365495[] = {{&S9225[33], 15}, {NULL}};
const Segment SGI365496[] = {{&S9225[49], 6}, {NULL}};
const Segment SGI365497[] = {{&S9225[33], 13}, {NULL}};
const Segment SGI365498[] = {{&S9226[69], 6}, {NULL}};
const Segment SGI365499[] = {{&S9226[8], 3}, {NULL}};
const Segment SGI365500[] = {{&S9226[13], 9}, {NULL}};
const Segment SGI365501[] = {{&S9226[33], 15}, {NULL}};
const Segment SGI365502[] = {{&S9226[49], 6}, {NULL}};
const Segment SGI365503[] = {{&S9226[33], 13}, {NULL}};
const Segment SGI365504[] = {{&S9227[69], 6}, {NULL}};
const Segment SGI365505[] = {{&S9227[8], 3}, {NULL}};
const Segment SGI365506[] = {{&S9227[13], 9}, {NULL}};
const Segment SGI365507[] = {{&S9227[33], 15}, {NULL}};
const Segment SGI365508[] = {{&S9227[49], 6}, {NULL}};
const Segment SGI365509[] = {{&S9227[33], 13}, {NULL}};
const Segment SGI365510[] = {{&S9228[69], 6}, {NULL}};
const Segment SGI365511[] = {{&S9228[8], 3}, {NULL}};
const Segment SGI365512[] = {{&S9228[13], 9}, {NULL}};
const Segment SGI365513[] = {{&S9228[33], 15}, {NULL}};
const Segment SGI365514[] = {{&S9228[49], 6}, {NULL}};
const Segment SGI365515[] = {{&S9228[33], 13}, {NULL}};
const Segment SGI365516[] = {{&S9229[69], 6}, {NULL}};
const Segment SGI365517[] = {{&S9229[8], 3}, {NULL}};
const Segment SGI365518[] = {{&S9229[13], 9}, {NULL}};
const Segment SGI365519[] = {{&S9229[33], 15}, {NULL}};
const Segment SGI365520[] = {{&S9229[49], 6}, {NULL}};
const Segment SGI365521[] = {{&S9229[33], 13}, {NULL}};
const Segment SGI365522[] = {{&S9230[69], 6}, {NULL}};
const Segment SGI365523[] = {{&S9230[8], 3}, {NULL}};
const Segment SGI365524[] = {{&S9230[13], 9}, {NULL}};
const Segment SGI365525[] = {{&S9230[33], 15}, {NULL}};
const Segment SGI365526[] = {{&S9230[49], 6}, {NULL}};
const Segment SGI365527[] = {{&S9230[33], 13}, {NULL}};
const Segment SGI365528[] = {{&S9231[69], 6}, {NULL}};
const Segment SGI365529[] = {{&S9231[8], 3}, {NULL}};
const Segment SGI365530[] = {{&S9231[13], 9}, {NULL}};
const Segment SGI365531[] = {{&S9231[33], 15}, {NULL}};
const Segment SGI365532[] = {{&S9231[49], 6}, {NULL}};
const Segment SGI365533[] = {{&S9231[33], 13}, {NULL}};
const Segment SGI365534[] = {{&S9232[69], 6}, {NULL}};
const Segment SGI365535[] = {{&S9232[8], 3}, {NULL}};
const Segment SGI365536[] = {{&S9232[13], 9}, {NULL}};
const Segment SGI365537[] = {{&S9232[33], 15}, {NULL}};
const Segment SGI365538[] = {{&S9232[49], 6}, {NULL}};
const Segment SGI365539[] = {{&S9232[33], 13}, {NULL}};
const Segment SGI365540[] = {{&S9233[69], 6}, {NULL}};
const Segment SGI365541[] = {{&S9233[8], 3}, {NULL}};
const Segment SGI365542[] = {{&S9233[13], 9}, {NULL}};
const Segment SGI365543[] = {{&S9233[33], 15}, {NULL}};
const Segment SGI365544[] = {{&S9233[49], 6}, {NULL}};
const Segment SGI365545[] = {{&S9233[33], 13}, {NULL}};
const Segment SGI365546[] = {{&S9234[69], 6}, {NULL}};
const Segment SGI365547[] = {{&S9234[8], 3}, {NULL}};
const Segment SGI365548[] = {{&S9234[13], 9}, {NULL}};
const Segment SGI365549[] = {{&S9234[33], 15}, {NULL}};
const Segment SGI365550[] = {{&S9234[49], 6}, {NULL}};
const Segment SGI365551[] = {{&S9234[33], 13}, {NULL}};
const Segment SGI365552[] = {{&S9235[69], 6}, {NULL}};
const Segment SGI365553[] = {{&S9235[8], 3}, {NULL}};
const Segment SGI365554[] = {{&S9235[13], 9}, {NULL}};
const Segment SGI365555[] = {{&S9235[33], 15}, {NULL}};
const Segment SGI365556[] = {{&S9235[49], 6}, {NULL}};
const Segment SGI365557[] = {{&S9235[33], 13}, {NULL}};
const Segment SGI365558[] = {{&S9236[69], 6}, {NULL}};
const Segment SGI365559[] = {{&S9236[8], 3}, {NULL}};
const Segment SGI365560[] = {{&S9236[13], 9}, {NULL}};
const Segment SGI365561[] = {{&S9236[33], 15}, {NULL}};
const Segment SGI365562[] = {{&S9236[49], 6}, {NULL}};
const Segment SGI365563[] = {{&S9236[33], 13}, {NULL}};
const Segment SGI365564[] = {{&S9237[69], 6}, {NULL}};
const Segment SGI365565[] = {{&S9237[8], 3}, {NULL}};
const Segment SGI365566[] = {{&S9237[13], 9}, {NULL}};
const Segment SGI365567[] = {{&S9237[33], 15}, {NULL}};
const Segment SGI365568[] = {{&S9237[49], 6}, {NULL}};
const Segment SGI365569[] = {{&S9237[33], 13}, {NULL}};
const Segment SGI365570[] = {{&S9238[69], 6}, {NULL}};
const Segment SGI365571[] = {{&S9238[8], 3}, {NULL}};
const Segment SGI365572[] = {{&S9238[13], 9}, {NULL}};
const Segment SGI365573[] = {{&S9238[33], 15}, {NULL}};
const Segment SGI365574[] = {{&S9238[49], 6}, {NULL}};
const Segment SGI365575[] = {{&S9238[33], 13}, {NULL}};
const Segment SGI365576[] = {{&S9239[69], 6}, {NULL}};
const Segment SGI365577[] = {{&S9239[8], 3}, {NULL}};
const Segment SGI365578[] = {{&S9239[13], 9}, {NULL}};
const Segment SGI365579[] = {{&S9239[33], 15}, {NULL}};
const Segment SGI365580[] = {{&S9239[49], 6}, {NULL}};
const Segment SGI365581[] = {{&S9239[33], 13}, {NULL}};
const Segment SGI365582[] = {{&S9240[69], 6}, {NULL}};
const Segment SGI365583[] = {{&S9240[8], 3}, {NULL}};
const Segment SGI365584[] = {{&S9240[13], 9}, {NULL}};
const Segment SGI365585[] = {{&S9240[33], 15}, {NULL}};
const Segment SGI365586[] = {{&S9240[49], 6}, {NULL}};
const Segment SGI365587[] = {{&S9240[33], 13}, {NULL}};
const Segment SGI365588[] = {{&S9241[69], 6}, {NULL}};
const Segment SGI365589[] = {{&S9241[8], 3}, {NULL}};
const Segment SGI365590[] = {{&S9241[13], 9}, {NULL}};
const Segment SGI365591[] = {{&S9241[33], 15}, {NULL}};
const Segment SGI365592[] = {{&S9241[49], 6}, {NULL}};
const Segment SGI365593[] = {{&S9241[33], 13}, {NULL}};
const Segment SGI365594[] = {{&S9242[69], 6}, {NULL}};
const Segment SGI365595[] = {{&S9242[8], 3}, {NULL}};
const Segment SGI365596[] = {{&S9242[13], 9}, {NULL}};
const Segment SGI365597[] = {{&S9242[33], 15}, {NULL}};
const Segment SGI365598[] = {{&S9242[49], 6}, {NULL}};
const Segment SGI365599[] = {{&S9242[33], 13}, {NULL}};
const Segment SGI365600[] = {{&S9243[69], 6}, {NULL}};
const Segment SGI365601[] = {{&S9243[8], 3}, {NULL}};
const Segment SGI365602[] = {{&S9243[13], 9}, {NULL}};
const Segment SGI365603[] = {{&S9243[33], 15}, {NULL}};
const Segment SGI365604[] = {{&S9243[49], 6}, {NULL}};
const Segment SGI365605[] = {{&S9243[33], 13}, {NULL}};
const Segment SGI365606[] = {{&S9244[69], 6}, {NULL}};
const Segment SGI365607[] = {{&S9244[8], 3}, {NULL}};
const Segment SGI365608[] = {{&S9244[13], 9}, {NULL}};
const Segment SGI365609[] = {{&S9244[33], 15}, {NULL}};
const Segment SGI365610[] = {{&S9244[49], 6}, {NULL}};
const Segment SGI365611[] = {{&S9244[33], 13}, {NULL}};
const Segment SGI365612[] = {{&S9245[69], 6}, {NULL}};
const Segment SGI365613[] = {{&S9245[8], 3}, {NULL}};
const Segment SGI365614[] = {{&S9245[13], 9}, {NULL}};
const Segment SGI365615[] = {{&S9245[33], 15}, {NULL}};
const Segment SGI365616[] = {{&S9245[49], 6}, {NULL}};
const Segment SGI365617[] = {{&S9245[33], 13}, {NULL}};
const Segment SGI365618[] = {{&S9246[69], 6}, {NULL}};
const Segment SGI365619[] = {{&S9246[8], 3}, {NULL}};
const Segment SGI365620[] = {{&S9246[13], 9}, {NULL}};
const Segment SGI365621[] = {{&S9246[33], 15}, {NULL}};
const Segment SGI365622[] = {{&S9246[49], 6}, {NULL}};
const Segment SGI365623[] = {{&S9246[33], 13}, {NULL}};
const Segment SGI365624[] = {{&S9247[69], 6}, {NULL}};
const Segment SGI365625[] = {{&S9247[8], 3}, {NULL}};
const Segment SGI365626[] = {{&S9247[13], 9}, {NULL}};
const Segment SGI365627[] = {{&S9247[33], 15}, {NULL}};
const Segment SGI365628[] = {{&S9247[49], 6}, {NULL}};
const Segment SGI365629[] = {{&S9247[33], 13}, {NULL}};
const Segment SGI365630[] = {{&S9248[69], 6}, {NULL}};
const Segment SGI365631[] = {{&S9248[8], 3}, {NULL}};
const Segment SGI365632[] = {{&S9248[13], 9}, {NULL}};
const Segment SGI365633[] = {{&S9248[33], 15}, {NULL}};
const Segment SGI365634[] = {{&S9248[49], 6}, {NULL}};
const Segment SGI365635[] = {{&S9248[33], 13}, {NULL}};
const Segment SGI365636[] = {{&S9249[69], 6}, {NULL}};
const Segment SGI365637[] = {{&S9249[8], 3}, {NULL}};
const Segment SGI365638[] = {{&S9249[13], 9}, {NULL}};
const Segment SGI365639[] = {{&S9249[33], 15}, {NULL}};
const Segment SGI365640[] = {{&S9249[49], 6}, {NULL}};
const Segment SGI365641[] = {{&S9249[33], 13}, {NULL}};
const Segment SGI365642[] = {{&S9250[69], 6}, {NULL}};
const Segment SGI365643[] = {{&S9250[8], 3}, {NULL}};
const Segment SGI365644[] = {{&S9250[13], 9}, {NULL}};
const Segment SGI365645[] = {{&S9250[33], 15}, {NULL}};
const Segment SGI365646[] = {{&S9250[49], 6}, {NULL}};
const Segment SGI365647[] = {{&S9250[33], 13}, {NULL}};
const Segment SGI365648[] = {{&S9251[69], 6}, {NULL}};
const Segment SGI365649[] = {{&S9251[8], 3}, {NULL}};
const Segment SGI365650[] = {{&S9251[13], 9}, {NULL}};
const Segment SGI365651[] = {{&S9251[33], 15}, {NULL}};
const Segment SGI365652[] = {{&S9251[49], 6}, {NULL}};
const Segment SGI365653[] = {{&S9251[33], 13}, {NULL}};
const Segment SGI365654[] = {{&S9252[69], 6}, {NULL}};
const Segment SGI365655[] = {{&S9252[8], 3}, {NULL}};
const Segment SGI365656[] = {{&S9252[13], 9}, {NULL}};
const Segment SGI365657[] = {{&S9252[33], 15}, {NULL}};
const Segment SGI365658[] = {{&S9252[49], 6}, {NULL}};
const Segment SGI365659[] = {{&S9252[33], 13}, {NULL}};
const Segment SGI365660[] = {{&S9253[69], 6}, {NULL}};
const Segment SGI365661[] = {{&S9253[8], 3}, {NULL}};
const Segment SGI365662[] = {{&S9253[13], 9}, {NULL}};
const Segment SGI365663[] = {{&S9253[33], 15}, {NULL}};
const Segment SGI365664[] = {{&S9253[49], 6}, {NULL}};
const Segment SGI365665[] = {{&S9253[33], 13}, {NULL}};
const Segment SGI365666[] = {{&S9254[69], 6}, {NULL}};
const Segment SGI365667[] = {{&S9254[8], 3}, {NULL}};
const Segment SGI365668[] = {{&S9254[13], 9}, {NULL}};
const Segment SGI365669[] = {{&S9254[33], 15}, {NULL}};
const Segment SGI365670[] = {{&S9254[49], 6}, {NULL}};
const Segment SGI365671[] = {{&S9254[33], 13}, {NULL}};
const Segment SGI365672[] = {{&S9255[69], 6}, {NULL}};
const Segment SGI365673[] = {{&S9255[8], 3}, {NULL}};
const Segment SGI365674[] = {{&S9255[13], 9}, {NULL}};
const Segment SGI365675[] = {{&S9255[33], 15}, {NULL}};
const Segment SGI365676[] = {{&S9255[49], 6}, {NULL}};
const Segment SGI365677[] = {{&S9255[33], 13}, {NULL}};
const Segment SGI365678[] = {{&S9256[69], 6}, {NULL}};
const Segment SGI365679[] = {{&S9256[8], 3}, {NULL}};
const Segment SGI365680[] = {{&S9256[13], 9}, {NULL}};
const Segment SGI365681[] = {{&S9256[33], 15}, {NULL}};
const Segment SGI365682[] = {{&S9256[49], 6}, {NULL}};
const Segment SGI365683[] = {{&S9256[33], 13}, {NULL}};
const Segment SGI365684[] = {{&S9257[69], 6}, {NULL}};
const Segment SGI365685[] = {{&S9257[8], 3}, {NULL}};
const Segment SGI365686[] = {{&S9257[13], 9}, {NULL}};
const Segment SGI365687[] = {{&S9257[33], 15}, {NULL}};
const Segment SGI365688[] = {{&S9257[49], 6}, {NULL}};
const Segment SGI365689[] = {{&S9257[33], 13}, {NULL}};
const Segment SGI365690[] = {{&S9258[69], 6}, {NULL}};
const Segment SGI365691[] = {{&S9258[8], 3}, {NULL}};
const Segment SGI365692[] = {{&S9258[13], 9}, {NULL}};
const Segment SGI365693[] = {{&S9258[33], 15}, {NULL}};
const Segment SGI365694[] = {{&S9258[49], 6}, {NULL}};
const Segment SGI365695[] = {{&S9258[33], 13}, {NULL}};
const Segment SGI365696[] = {{&S9259[69], 6}, {NULL}};
const Segment SGI365697[] = {{&S9259[8], 3}, {NULL}};
const Segment SGI365698[] = {{&S9259[13], 9}, {NULL}};
const Segment SGI365699[] = {{&S9259[33], 15}, {NULL}};
const Segment SGI365700[] = {{&S9259[49], 6}, {NULL}};
const Segment SGI365701[] = {{&S9259[33], 13}, {NULL}};
const Segment SGI365702[] = {{&S9260[69], 6}, {NULL}};
const Segment SGI365703[] = {{&S9260[8], 3}, {NULL}};
const Segment SGI365704[] = {{&S9260[13], 9}, {NULL}};
const Segment SGI365705[] = {{&S9260[33], 15}, {NULL}};
const Segment SGI365706[] = {{&S9260[49], 6}, {NULL}};
const Segment SGI365707[] = {{&S9260[33], 13}, {NULL}};
const Segment SGI365708[] = {{&S9261[69], 6}, {NULL}};
const Segment SGI365709[] = {{&S9261[8], 3}, {NULL}};
const Segment SGI365710[] = {{&S9261[13], 9}, {NULL}};
const Segment SGI365711[] = {{&S9261[33], 15}, {NULL}};
const Segment SGI365712[] = {{&S9261[49], 6}, {NULL}};
const Segment SGI365713[] = {{&S9261[33], 13}, {NULL}};
const Segment SGI365714[] = {{&S9262[69], 6}, {NULL}};
const Segment SGI365715[] = {{&S9262[8], 3}, {NULL}};
const Segment SGI365716[] = {{&S9262[13], 9}, {NULL}};
const Segment SGI365717[] = {{&S9262[33], 15}, {NULL}};
const Segment SGI365718[] = {{&S9262[49], 6}, {NULL}};
const Segment SGI365719[] = {{&S9262[33], 13}, {NULL}};
const Segment SGI365720[] = {{&S9263[69], 6}, {NULL}};
const Segment SGI365721[] = {{&S9263[8], 3}, {NULL}};
const Segment SGI365722[] = {{&S9263[13], 9}, {NULL}};
const Segment SGI365723[] = {{&S9263[33], 15}, {NULL}};
const Segment SGI365724[] = {{&S9263[49], 6}, {NULL}};
const Segment SGI365725[] = {{&S9263[33], 13}, {NULL}};
const Segment SGI365726[] = {{&S9264[69], 6}, {NULL}};
const Segment SGI365727[] = {{&S9264[8], 3}, {NULL}};
const Segment SGI365728[] = {{&S9264[13], 9}, {NULL}};
const Segment SGI365729[] = {{&S9264[33], 15}, {NULL}};
const Segment SGI365730[] = {{&S9264[49], 6}, {NULL}};
const Segment SGI365731[] = {{&S9264[33], 13}, {NULL}};
const Segment SGI365732[] = {{&S9265[69], 6}, {NULL}};
const Segment SGI365733[] = {{&S9265[8], 3}, {NULL}};
const Segment SGI365734[] = {{&S9265[13], 9}, {NULL}};
const Segment SGI365735[] = {{&S9265[33], 15}, {NULL}};
const Segment SGI365736[] = {{&S9265[49], 6}, {NULL}};
const Segment SGI365737[] = {{&S9265[33], 13}, {NULL}};
const Segment SGI365738[] = {{&S9266[69], 6}, {NULL}};
const Segment SGI365739[] = {{&S9266[8], 3}, {NULL}};
const Segment SGI365740[] = {{&S9266[13], 9}, {NULL}};
const Segment SGI365741[] = {{&S9266[33], 15}, {NULL}};
const Segment SGI365742[] = {{&S9266[49], 6}, {NULL}};
const Segment SGI365743[] = {{&S9266[33], 13}, {NULL}};
const Segment SGI365744[] = {{&S9267[69], 6}, {NULL}};
const Segment SGI365745[] = {{&S9267[8], 3}, {NULL}};
const Segment SGI365746[] = {{&S9267[13], 9}, {NULL}};
const Segment SGI365747[] = {{&S9267[33], 15}, {NULL}};
const Segment SGI365748[] = {{&S9267[49], 6}, {NULL}};
const Segment SGI365749[] = {{&S9267[33], 13}, {NULL}};
const Segment SGI365750[] = {{&S9268[69], 6}, {NULL}};
const Segment SGI365751[] = {{&S9268[8], 3}, {NULL}};
const Segment SGI365752[] = {{&S9268[13], 9}, {NULL}};
const Segment SGI365753[] = {{&S9268[33], 15}, {NULL}};
const Segment SGI365754[] = {{&S9268[49], 6}, {NULL}};
const Segment SGI365755[] = {{&S9268[33], 13}, {NULL}};
const Segment SGI365756[] = {{&S9269[69], 6}, {NULL}};
const Segment SGI365757[] = {{&S9269[8], 3}, {NULL}};
const Segment SGI365758[] = {{&S9269[13], 9}, {NULL}};
const Segment SGI365759[] = {{&S9269[33], 15}, {NULL}};
const Segment SGI365760[] = {{&S9269[49], 6}, {NULL}};
const Segment SGI365761[] = {{&S9269[33], 13}, {NULL}};
const Segment SGI365762[] = {{&S9270[69], 6}, {NULL}};
const Segment SGI365763[] = {{&S9270[8], 3}, {NULL}};
const Segment SGI365764[] = {{&S9270[13], 9}, {NULL}};
const Segment SGI365765[] = {{&S9270[33], 15}, {NULL}};
const Segment SGI365766[] = {{&S9270[49], 6}, {NULL}};
const Segment SGI365767[] = {{&S9270[33], 13}, {NULL}};
const Segment SGI365768[] = {{&S9271[66], 6}, {NULL}};
const Segment SGI365769[] = {{&S9271[8], 3}, {NULL}};
const Segment SGI365770[] = {{&S9271[13], 9}, {NULL}};
const Segment SGI365771[] = {{&S9271[33], 15}, {NULL}};
const Segment SGI365772[] = {{&S9271[49], 3}, {NULL}};
const Segment SGI365773[] = {{&S9271[33], 13}, {NULL}};
const Segment SGI365774[] = {{&S9272[69], 6}, {NULL}};
const Segment SGI365775[] = {{&S9272[8], 3}, {NULL}};
const Segment SGI365776[] = {{&S9272[13], 9}, {NULL}};
const Segment SGI365777[] = {{&S9272[33], 15}, {NULL}};
const Segment SGI365778[] = {{&S9272[49], 6}, {NULL}};
const Segment SGI365779[] = {{&S9272[33], 13}, {NULL}};
const Segment SGI365780[] = {{&S9273[69], 6}, {NULL}};
const Segment SGI365781[] = {{&S9273[8], 3}, {NULL}};
const Segment SGI365782[] = {{&S9273[13], 9}, {NULL}};
const Segment SGI365783[] = {{&S9273[33], 15}, {NULL}};
const Segment SGI365784[] = {{&S9273[49], 6}, {NULL}};
const Segment SGI365785[] = {{&S9273[33], 13}, {NULL}};
const Segment SGI365786[] = {{&S9274[69], 6}, {NULL}};
const Segment SGI365787[] = {{&S9274[8], 3}, {NULL}};
const Segment SGI365788[] = {{&S9274[13], 9}, {NULL}};
const Segment SGI365789[] = {{&S9274[33], 15}, {NULL}};
const Segment SGI365790[] = {{&S9274[49], 6}, {NULL}};
const Segment SGI365791[] = {{&S9274[33], 13}, {NULL}};
const Segment SGI365792[] = {{&S9275[69], 6}, {NULL}};
const Segment SGI365793[] = {{&S9275[8], 3}, {NULL}};
const Segment SGI365794[] = {{&S9275[13], 9}, {NULL}};
const Segment SGI365795[] = {{&S9275[33], 15}, {NULL}};
const Segment SGI365796[] = {{&S9275[49], 6}, {NULL}};
const Segment SGI365797[] = {{&S9275[33], 13}, {NULL}};
const Segment SGI365798[] = {{&S9276[69], 6}, {NULL}};
const Segment SGI365799[] = {{&S9276[8], 3}, {NULL}};
const Segment SGI365800[] = {{&S9276[13], 9}, {NULL}};
const Segment SGI365801[] = {{&S9276[33], 15}, {NULL}};
const Segment SGI365802[] = {{&S9276[49], 6}, {NULL}};
const Segment SGI365803[] = {{&S9276[33], 13}, {NULL}};
const Segment SGI365804[] = {{&S9277[69], 6}, {NULL}};
const Segment SGI365805[] = {{&S9277[8], 3}, {NULL}};
const Segment SGI365806[] = {{&S9277[13], 9}, {NULL}};
const Segment SGI365807[] = {{&S9277[33], 15}, {NULL}};
const Segment SGI365808[] = {{&S9277[49], 6}, {NULL}};
const Segment SGI365809[] = {{&S9277[33], 13}, {NULL}};
const Segment SGI365810[] = {{&S9278[69], 6}, {NULL}};
const Segment SGI365811[] = {{&S9278[8], 3}, {NULL}};
const Segment SGI365812[] = {{&S9278[13], 9}, {NULL}};
const Segment SGI365813[] = {{&S9278[33], 15}, {NULL}};
const Segment SGI365814[] = {{&S9278[49], 6}, {NULL}};
const Segment SGI365815[] = {{&S9278[33], 13}, {NULL}};
const Segment SGI365816[] = {{&S9279[69], 6}, {NULL}};
const Segment SGI365817[] = {{&S9279[8], 3}, {NULL}};
const Segment SGI365818[] = {{&S9279[13], 9}, {NULL}};
const Segment SGI365819[] = {{&S9279[33], 15}, {NULL}};
const Segment SGI365820[] = {{&S9279[49], 6}, {NULL}};
const Segment SGI365821[] = {{&S9279[33], 13}, {NULL}};
const Segment SGI365822[] = {{&S9280[69], 6}, {NULL}};
const Segment SGI365823[] = {{&S9280[8], 3}, {NULL}};
const Segment SGI365824[] = {{&S9280[13], 9}, {NULL}};
const Segment SGI365825[] = {{&S9280[33], 15}, {NULL}};
const Segment SGI365826[] = {{&S9280[49], 6}, {NULL}};
const Segment SGI365827[] = {{&S9280[33], 13}, {NULL}};
const Segment SGI365828[] = {{&S9281[69], 6}, {NULL}};
const Segment SGI365829[] = {{&S9281[8], 3}, {NULL}};
const Segment SGI365830[] = {{&S9281[13], 9}, {NULL}};
const Segment SGI365831[] = {{&S9281[33], 15}, {NULL}};
const Segment SGI365832[] = {{&S9281[49], 6}, {NULL}};
const Segment SGI365833[] = {{&S9281[33], 13}, {NULL}};
const Segment SGI365834[] = {{&S9282[69], 6}, {NULL}};
const Segment SGI365835[] = {{&S9282[8], 3}, {NULL}};
const Segment SGI365836[] = {{&S9282[13], 9}, {NULL}};
const Segment SGI365837[] = {{&S9282[33], 15}, {NULL}};
const Segment SGI365838[] = {{&S9282[49], 6}, {NULL}};
const Segment SGI365839[] = {{&S9282[33], 13}, {NULL}};
const Segment SGI365840[] = {{&S9283[69], 6}, {NULL}};
const Segment SGI365841[] = {{&S9283[8], 3}, {NULL}};
const Segment SGI365842[] = {{&S9283[13], 9}, {NULL}};
const Segment SGI365843[] = {{&S9283[33], 15}, {NULL}};
const Segment SGI365844[] = {{&S9283[49], 6}, {NULL}};
const Segment SGI365845[] = {{&S9283[33], 13}, {NULL}};
const Segment SGI365846[] = {{&S9284[69], 6}, {NULL}};
const Segment SGI365847[] = {{&S9284[8], 3}, {NULL}};
const Segment SGI365848[] = {{&S9284[13], 9}, {NULL}};
const Segment SGI365849[] = {{&S9284[33], 15}, {NULL}};
const Segment SGI365850[] = {{&S9284[49], 6}, {NULL}};
const Segment SGI365851[] = {{&S9284[33], 13}, {NULL}};
const Segment SGI365852[] = {{&S9285[69], 6}, {NULL}};
const Segment SGI365853[] = {{&S9285[8], 3}, {NULL}};
const Segment SGI365854[] = {{&S9285[13], 9}, {NULL}};
const Segment SGI365855[] = {{&S9285[33], 15}, {NULL}};
const Segment SGI365856[] = {{&S9285[49], 6}, {NULL}};
const Segment SGI365857[] = {{&S9285[33], 13}, {NULL}};
const Segment SGI365858[] = {{&S9286[69], 6}, {NULL}};
const Segment SGI365859[] = {{&S9286[8], 3}, {NULL}};
const Segment SGI365860[] = {{&S9286[13], 9}, {NULL}};
const Segment SGI365861[] = {{&S9286[33], 15}, {NULL}};
const Segment SGI365862[] = {{&S9286[49], 6}, {NULL}};
const Segment SGI365863[] = {{&S9286[33], 13}, {NULL}};
const Segment SGI365864[] = {{&S9287[69], 6}, {NULL}};
const Segment SGI365865[] = {{&S9287[8], 3}, {NULL}};
const Segment SGI365866[] = {{&S9287[13], 9}, {NULL}};
const Segment SGI365867[] = {{&S9287[33], 15}, {NULL}};
const Segment SGI365868[] = {{&S9287[49], 6}, {NULL}};
const Segment SGI365869[] = {{&S9287[33], 13}, {NULL}};
const Segment SGI365870[] = {{&S9288[105], 7}, {NULL}};
const Segment SGI365871[] = {{&S9288[8], 3}, {NULL}};
const Segment SGI365872[] = {{&S9288[13], 9}, {NULL}};
const Segment SGI365873[] = {{&S9288[33], 15}, {NULL}};
const Segment SGI365874[] = {{&S9288[49], 6}, {NULL}};
const Segment SGI365875[] = {{&S9288[33], 13}, {NULL}};
const Segment SGI365876[] = {{&S9289[69], 6}, {NULL}};
const Segment SGI365877[] = {{&S9289[8], 3}, {NULL}};
const Segment SGI365878[] = {{&S9289[13], 9}, {NULL}};
const Segment SGI365879[] = {{&S9289[33], 15}, {NULL}};
const Segment SGI365880[] = {{&S9289[49], 6}, {NULL}};
const Segment SGI365881[] = {{&S9289[33], 13}, {NULL}};
const Segment SGI365882[] = {{&S9290[69], 6}, {NULL}};
const Segment SGI365883[] = {{&S9290[8], 3}, {NULL}};
const Segment SGI365884[] = {{&S9290[13], 9}, {NULL}};
const Segment SGI365885[] = {{&S9290[33], 15}, {NULL}};
const Segment SGI365886[] = {{&S9290[49], 6}, {NULL}};
const Segment SGI365887[] = {{&S9290[33], 13}, {NULL}};
const Segment SGI365888[] = {{&S9291[69], 6}, {NULL}};
const Segment SGI365889[] = {{&S9291[8], 3}, {NULL}};
const Segment SGI365890[] = {{&S9291[13], 9}, {NULL}};
const Segment SGI365891[] = {{&S9291[33], 15}, {NULL}};
const Segment SGI365892[] = {{&S9291[49], 6}, {NULL}};
const Segment SGI365893[] = {{&S9291[33], 13}, {NULL}};
const Segment SGI365894[] = {{&S9292[69], 6}, {NULL}};
const Segment SGI365895[] = {{&S9292[8], 3}, {NULL}};
const Segment SGI365896[] = {{&S9292[13], 9}, {NULL}};
const Segment SGI365897[] = {{&S9292[33], 15}, {NULL}};
const Segment SGI365898[] = {{&S9292[49], 6}, {NULL}};
const Segment SGI365899[] = {{&S9292[33], 13}, {NULL}};
const Segment SGI365900[] = {{&S9293[69], 6}, {NULL}};
const Segment SGI365901[] = {{&S9293[8], 3}, {NULL}};
const Segment SGI365902[] = {{&S9293[13], 9}, {NULL}};
const Segment SGI365903[] = {{&S9293[33], 15}, {NULL}};
const Segment SGI365904[] = {{&S9293[49], 6}, {NULL}};
const Segment SGI365905[] = {{&S9293[33], 13}, {NULL}};
const Segment SGI365906[] = {{&S9294[69], 6}, {NULL}};
const Segment SGI365907[] = {{&S9294[8], 3}, {NULL}};
const Segment SGI365908[] = {{&S9294[13], 9}, {NULL}};
const Segment SGI365909[] = {{&S9294[33], 15}, {NULL}};
const Segment SGI365910[] = {{&S9294[49], 6}, {NULL}};
const Segment SGI365911[] = {{&S9294[33], 13}, {NULL}};
const Segment SGI365912[] = {{&S9295[69], 6}, {NULL}};
const Segment SGI365913[] = {{&S9295[8], 3}, {NULL}};
const Segment SGI365914[] = {{&S9295[13], 9}, {NULL}};
const Segment SGI365915[] = {{&S9295[33], 15}, {NULL}};
const Segment SGI365916[] = {{&S9295[49], 6}, {NULL}};
const Segment SGI365917[] = {{&S9295[33], 13}, {NULL}};
const Segment SGI365918[] = {{&S9296[69], 6}, {NULL}};
const Segment SGI365919[] = {{&S9296[8], 3}, {NULL}};
const Segment SGI365920[] = {{&S9296[13], 9}, {NULL}};
const Segment SGI365921[] = {{&S9296[33], 15}, {NULL}};
const Segment SGI365922[] = {{&S9296[49], 6}, {NULL}};
const Segment SGI365923[] = {{&S9296[33], 13}, {NULL}};
const Segment SGI365924[] = {{&S9297[69], 6}, {NULL}};
const Segment SGI365925[] = {{&S9297[8], 3}, {NULL}};
const Segment SGI365926[] = {{&S9297[13], 9}, {NULL}};
const Segment SGI365927[] = {{&S9297[33], 15}, {NULL}};
const Segment SGI365928[] = {{&S9297[49], 6}, {NULL}};
const Segment SGI365929[] = {{&S9297[33], 13}, {NULL}};
const Segment SGI365930[] = {{&S9298[69], 6}, {NULL}};
const Segment SGI365931[] = {{&S9298[8], 3}, {NULL}};
const Segment SGI365932[] = {{&S9298[13], 9}, {NULL}};
const Segment SGI365933[] = {{&S9298[33], 15}, {NULL}};
const Segment SGI365934[] = {{&S9298[49], 6}, {NULL}};
const Segment SGI365935[] = {{&S9298[33], 13}, {NULL}};
const Segment SGI365936[] = {{&S9299[69], 6}, {NULL}};
const Segment SGI365937[] = {{&S9299[8], 3}, {NULL}};
const Segment SGI365938[] = {{&S9299[13], 9}, {NULL}};
const Segment SGI365939[] = {{&S9299[33], 15}, {NULL}};
const Segment SGI365940[] = {{&S9299[49], 6}, {NULL}};
const Segment SGI365941[] = {{&S9299[33], 13}, {NULL}};
const Segment SGI365942[] = {{&S9300[69], 6}, {NULL}};
const Segment SGI365943[] = {{&S9300[8], 3}, {NULL}};
const Segment SGI365944[] = {{&S9300[13], 9}, {NULL}};
const Segment SGI365945[] = {{&S9300[33], 15}, {NULL}};
const Segment SGI365946[] = {{&S9300[49], 6}, {NULL}};
const Segment SGI365947[] = {{&S9300[33], 13}, {NULL}};
const Segment SGI365948[] = {{&S9301[69], 6}, {NULL}};
const Segment SGI365949[] = {{&S9301[8], 3}, {NULL}};
const Segment SGI365950[] = {{&S9301[13], 9}, {NULL}};
const Segment SGI365951[] = {{&S9301[33], 15}, {NULL}};
const Segment SGI365952[] = {{&S9301[49], 6}, {NULL}};
const Segment SGI365953[] = {{&S9301[33], 13}, {NULL}};
const Segment SGI365954[] = {{&S9302[69], 6}, {NULL}};
const Segment SGI365955[] = {{&S9302[8], 3}, {NULL}};
const Segment SGI365956[] = {{&S9302[13], 9}, {NULL}};
const Segment SGI365957[] = {{&S9302[33], 15}, {NULL}};
const Segment SGI365958[] = {{&S9302[49], 6}, {NULL}};
const Segment SGI365959[] = {{&S9302[33], 13}, {NULL}};
const Segment SGI365960[] = {{&S9303[69], 6}, {NULL}};
const Segment SGI365961[] = {{&S9303[8], 3}, {NULL}};
const Segment SGI365962[] = {{&S9303[13], 9}, {NULL}};
const Segment SGI365963[] = {{&S9303[33], 15}, {NULL}};
const Segment SGI365964[] = {{&S9303[49], 6}, {NULL}};
const Segment SGI365965[] = {{&S9303[33], 13}, {NULL}};
const Segment SGI365966[] = {{&S9304[69], 6}, {NULL}};
const Segment SGI365967[] = {{&S9304[8], 3}, {NULL}};
const Segment SGI365968[] = {{&S9304[13], 9}, {NULL}};
const Segment SGI365969[] = {{&S9304[33], 15}, {NULL}};
const Segment SGI365970[] = {{&S9304[49], 6}, {NULL}};
const Segment SGI365971[] = {{&S9304[33], 13}, {NULL}};
const Segment SGI365972[] = {{&S9305[66], 6}, {NULL}};
const Segment SGI365973[] = {{&S9305[8], 3}, {NULL}};
const Segment SGI365974[] = {{&S9305[13], 9}, {NULL}};
const Segment SGI365975[] = {{&S9305[33], 15}, {NULL}};
const Segment SGI365976[] = {{&S9305[49], 3}, {NULL}};
const Segment SGI365977[] = {{&S9305[33], 13}, {NULL}};
const Segment SGI365978[] = {{&S9306[69], 6}, {NULL}};
const Segment SGI365979[] = {{&S9306[8], 3}, {NULL}};
const Segment SGI365980[] = {{&S9306[13], 9}, {NULL}};
const Segment SGI365981[] = {{&S9306[33], 15}, {NULL}};
const Segment SGI365982[] = {{&S9306[49], 6}, {NULL}};
const Segment SGI365983[] = {{&S9306[33], 13}, {NULL}};
const Segment SGI365984[] = {{&S9307[69], 6}, {NULL}};
const Segment SGI365985[] = {{&S9307[8], 3}, {NULL}};
const Segment SGI365986[] = {{&S9307[13], 9}, {NULL}};
const Segment SGI365987[] = {{&S9307[33], 15}, {NULL}};
const Segment SGI365988[] = {{&S9307[49], 6}, {NULL}};
const Segment SGI365989[] = {{&S9307[33], 13}, {NULL}};
const Segment SGI365990[] = {{&S9308[69], 6}, {NULL}};
const Segment SGI365991[] = {{&S9308[8], 3}, {NULL}};
const Segment SGI365992[] = {{&S9308[13], 9}, {NULL}};
const Segment SGI365993[] = {{&S9308[33], 15}, {NULL}};
const Segment SGI365994[] = {{&S9308[49], 6}, {NULL}};
const Segment SGI365995[] = {{&S9308[33], 13}, {NULL}};
const Segment SGI365996[] = {{&S9309[69], 6}, {NULL}};
const Segment SGI365997[] = {{&S9309[8], 3}, {NULL}};
const Segment SGI365998[] = {{&S9309[13], 9}, {NULL}};
const Segment SGI365999[] = {{&S9309[33], 15}, {NULL}};
const Segment SGI366000[] = {{&S9309[49], 6}, {NULL}};
const Segment SGI366001[] = {{&S9309[33], 13}, {NULL}};
const Segment SGI366002[] = {{&S9310[69], 6}, {NULL}};
const Segment SGI366003[] = {{&S9310[8], 3}, {NULL}};
const Segment SGI366004[] = {{&S9310[13], 9}, {NULL}};
const Segment SGI366005[] = {{&S9310[33], 15}, {NULL}};
const Segment SGI366006[] = {{&S9310[49], 6}, {NULL}};
const Segment SGI366007[] = {{&S9310[33], 13}, {NULL}};
const Segment SGI366008[] = {{&S9311[69], 6}, {NULL}};
const Segment SGI366009[] = {{&S9311[8], 3}, {NULL}};
const Segment SGI366010[] = {{&S9311[13], 9}, {NULL}};
const Segment SGI366011[] = {{&S9311[33], 15}, {NULL}};
const Segment SGI366012[] = {{&S9311[49], 6}, {NULL}};
const Segment SGI366013[] = {{&S9311[33], 13}, {NULL}};
const Segment SGI366014[] = {{&S9312[69], 6}, {NULL}};
const Segment SGI366015[] = {{&S9312[8], 3}, {NULL}};
const Segment SGI366016[] = {{&S9312[13], 9}, {NULL}};
const Segment SGI366017[] = {{&S9312[33], 15}, {NULL}};
const Segment SGI366018[] = {{&S9312[49], 6}, {NULL}};
const Segment SGI366019[] = {{&S9312[33], 13}, {NULL}};
const Segment SGI366020[] = {{&S9313[69], 6}, {NULL}};
const Segment SGI366021[] = {{&S9313[8], 3}, {NULL}};
const Segment SGI366022[] = {{&S9313[13], 9}, {NULL}};
const Segment SGI366023[] = {{&S9313[33], 15}, {NULL}};
const Segment SGI366024[] = {{&S9313[49], 6}, {NULL}};
const Segment SGI366025[] = {{&S9313[33], 13}, {NULL}};
const Segment SGI366026[] = {{&S9314[69], 6}, {NULL}};
const Segment SGI366027[] = {{&S9314[8], 3}, {NULL}};
const Segment SGI366028[] = {{&S9314[13], 9}, {NULL}};
const Segment SGI366029[] = {{&S9314[33], 15}, {NULL}};
const Segment SGI366030[] = {{&S9314[49], 6}, {NULL}};
const Segment SGI366031[] = {{&S9314[33], 13}, {NULL}};
const Segment SGI366032[] = {{&S9315[69], 6}, {NULL}};
const Segment SGI366033[] = {{&S9315[8], 3}, {NULL}};
const Segment SGI366034[] = {{&S9315[13], 9}, {NULL}};
const Segment SGI366035[] = {{&S9315[33], 15}, {NULL}};
const Segment SGI366036[] = {{&S9315[49], 6}, {NULL}};
const Segment SGI366037[] = {{&S9315[33], 13}, {NULL}};
const Segment SGI366038[] = {{&S9316[69], 6}, {NULL}};
const Segment SGI366039[] = {{&S9316[8], 3}, {NULL}};
const Segment SGI366040[] = {{&S9316[13], 9}, {NULL}};
const Segment SGI366041[] = {{&S9316[33], 15}, {NULL}};
const Segment SGI366042[] = {{&S9316[49], 6}, {NULL}};
const Segment SGI366043[] = {{&S9316[33], 13}, {NULL}};
const Segment SGI366044[] = {{&S9317[69], 6}, {NULL}};
const Segment SGI366045[] = {{&S9317[8], 3}, {NULL}};
const Segment SGI366046[] = {{&S9317[13], 9}, {NULL}};
const Segment SGI366047[] = {{&S9317[33], 15}, {NULL}};
const Segment SGI366048[] = {{&S9317[49], 6}, {NULL}};
const Segment SGI366049[] = {{&S9317[33], 13}, {NULL}};
const Segment SGI366050[] = {{&S9318[69], 6}, {NULL}};
const Segment SGI366051[] = {{&S9318[8], 3}, {NULL}};
const Segment SGI366052[] = {{&S9318[13], 9}, {NULL}};
const Segment SGI366053[] = {{&S9318[33], 15}, {NULL}};
const Segment SGI366054[] = {{&S9318[49], 6}, {NULL}};
const Segment SGI366055[] = {{&S9318[33], 13}, {NULL}};
const Segment SGI366056[] = {{&S9319[69], 6}, {NULL}};
const Segment SGI366057[] = {{&S9319[8], 3}, {NULL}};
const Segment SGI366058[] = {{&S9319[13], 9}, {NULL}};
const Segment SGI366059[] = {{&S9319[33], 15}, {NULL}};
const Segment SGI366060[] = {{&S9319[49], 6}, {NULL}};
const Segment SGI366061[] = {{&S9319[33], 13}, {NULL}};
const Segment SGI366062[] = {{&S9320[69], 6}, {NULL}};
const Segment SGI366063[] = {{&S9320[8], 3}, {NULL}};
const Segment SGI366064[] = {{&S9320[13], 9}, {NULL}};
const Segment SGI366065[] = {{&S9320[33], 15}, {NULL}};
const Segment SGI366066[] = {{&S9320[49], 6}, {NULL}};
const Segment SGI366067[] = {{&S9320[33], 13}, {NULL}};
const Segment SGI366068[] = {{&S9321[69], 6}, {NULL}};
const Segment SGI366069[] = {{&S9321[8], 3}, {NULL}};
const Segment SGI366070[] = {{&S9321[13], 9}, {NULL}};
const Segment SGI366071[] = {{&S9321[33], 15}, {NULL}};
const Segment SGI366072[] = {{&S9321[49], 6}, {NULL}};
const Segment SGI366073[] = {{&S9321[33], 13}, {NULL}};
const Segment SGI366074[] = {{&S9322[69], 6}, {NULL}};
const Segment SGI366075[] = {{&S9322[8], 3}, {NULL}};
const Segment SGI366076[] = {{&S9322[13], 9}, {NULL}};
const Segment SGI366077[] = {{&S9322[33], 15}, {NULL}};
const Segment SGI366078[] = {{&S9322[49], 6}, {NULL}};
const Segment SGI366079[] = {{&S9322[33], 13}, {NULL}};
const Segment SGI366080[] = {{&S9323[69], 6}, {NULL}};
const Segment SGI366081[] = {{&S9323[8], 3}, {NULL}};
const Segment SGI366082[] = {{&S9323[13], 9}, {NULL}};
const Segment SGI366083[] = {{&S9323[33], 15}, {NULL}};
const Segment SGI366084[] = {{&S9323[49], 6}, {NULL}};
const Segment SGI366085[] = {{&S9323[33], 13}, {NULL}};
const Segment SGI366086[] = {{&S9324[69], 6}, {NULL}};
const Segment SGI366087[] = {{&S9324[8], 3}, {NULL}};
const Segment SGI366088[] = {{&S9324[13], 9}, {NULL}};
const Segment SGI366089[] = {{&S9324[33], 15}, {NULL}};
const Segment SGI366090[] = {{&S9324[49], 6}, {NULL}};
const Segment SGI366091[] = {{&S9324[33], 13}, {NULL}};
const Segment SGI366092[] = {{&S9325[69], 6}, {NULL}};
const Segment SGI366093[] = {{&S9325[8], 3}, {NULL}};
const Segment SGI366094[] = {{&S9325[13], 9}, {NULL}};
const Segment SGI366095[] = {{&S9325[33], 15}, {NULL}};
const Segment SGI366096[] = {{&S9325[49], 6}, {NULL}};
const Segment SGI366097[] = {{&S9325[33], 13}, {NULL}};
const Segment SGI366098[] = {{&S9326[69], 6}, {NULL}};
const Segment SGI366099[] = {{&S9326[8], 3}, {NULL}};
const Segment SGI366100[] = {{&S9326[13], 9}, {NULL}};
const Segment SGI366101[] = {{&S9326[33], 15}, {NULL}};
const Segment SGI366102[] = {{&S9326[49], 6}, {NULL}};
const Segment SGI366103[] = {{&S9326[33], 13}, {NULL}};
const Segment SGI366104[] = {{&S9327[69], 6}, {NULL}};
const Segment SGI366105[] = {{&S9327[8], 3}, {NULL}};
const Segment SGI366106[] = {{&S9327[13], 9}, {NULL}};
const Segment SGI366107[] = {{&S9327[33], 15}, {NULL}};
const Segment SGI366108[] = {{&S9327[49], 6}, {NULL}};
const Segment SGI366109[] = {{&S9327[33], 13}, {NULL}};
const Segment SGI366110[] = {{&S9328[69], 6}, {NULL}};
const Segment SGI366111[] = {{&S9328[8], 3}, {NULL}};
const Segment SGI366112[] = {{&S9328[13], 9}, {NULL}};
const Segment SGI366113[] = {{&S9328[33], 15}, {NULL}};
const Segment SGI366114[] = {{&S9328[49], 6}, {NULL}};
const Segment SGI366115[] = {{&S9328[33], 13}, {NULL}};
const Segment SGI366116[] = {{&S9329[69], 6}, {NULL}};
const Segment SGI366117[] = {{&S9329[8], 3}, {NULL}};
const Segment SGI366118[] = {{&S9329[13], 9}, {NULL}};
const Segment SGI366119[] = {{&S9329[33], 15}, {NULL}};
const Segment SGI366120[] = {{&S9329[49], 6}, {NULL}};
const Segment SGI366121[] = {{&S9329[33], 13}, {NULL}};
const Segment SGI366122[] = {{&S9330[69], 6}, {NULL}};
const Segment SGI366123[] = {{&S9330[8], 3}, {NULL}};
const Segment SGI366124[] = {{&S9330[13], 9}, {NULL}};
const Segment SGI366125[] = {{&S9330[33], 15}, {NULL}};
const Segment SGI366126[] = {{&S9330[49], 6}, {NULL}};
const Segment SGI366127[] = {{&S9330[33], 13}, {NULL}};
const Segment SGI366128[] = {{&S9331[69], 6}, {NULL}};
const Segment SGI366129[] = {{&S9331[8], 3}, {NULL}};
const Segment SGI366130[] = {{&S9331[13], 9}, {NULL}};
const Segment SGI366131[] = {{&S9331[33], 15}, {NULL}};
const Segment SGI366132[] = {{&S9331[49], 6}, {NULL}};
const Segment SGI366133[] = {{&S9331[33], 13}, {NULL}};
const Segment SGI366134[] = {{&S9332[69], 6}, {NULL}};
const Segment SGI366135[] = {{&S9332[8], 3}, {NULL}};
const Segment SGI366136[] = {{&S9332[13], 9}, {NULL}};
const Segment SGI366137[] = {{&S9332[33], 15}, {NULL}};
const Segment SGI366138[] = {{&S9332[49], 6}, {NULL}};
const Segment SGI366139[] = {{&S9332[33], 13}, {NULL}};
const Segment SGI366140[] = {{&S9333[69], 6}, {NULL}};
const Segment SGI366141[] = {{&S9333[8], 3}, {NULL}};
const Segment SGI366142[] = {{&S9333[13], 9}, {NULL}};
const Segment SGI366143[] = {{&S9333[33], 15}, {NULL}};
const Segment SGI366144[] = {{&S9333[49], 6}, {NULL}};
const Segment SGI366145[] = {{&S9333[33], 13}, {NULL}};
const Segment SGI366146[] = {{&S9334[69], 6}, {NULL}};
const Segment SGI366147[] = {{&S9334[8], 3}, {NULL}};
const Segment SGI366148[] = {{&S9334[13], 9}, {NULL}};
const Segment SGI366149[] = {{&S9334[33], 15}, {NULL}};
const Segment SGI366150[] = {{&S9334[49], 6}, {NULL}};
const Segment SGI366151[] = {{&S9334[33], 13}, {NULL}};
const Segment SGI366152[] = {{&S9335[69], 6}, {NULL}};
const Segment SGI366153[] = {{&S9335[8], 3}, {NULL}};
const Segment SGI366154[] = {{&S9335[13], 9}, {NULL}};
const Segment SGI366155[] = {{&S9335[33], 15}, {NULL}};
const Segment SGI366156[] = {{&S9335[49], 6}, {NULL}};
const Segment SGI366157[] = {{&S9335[33], 13}, {NULL}};
const Segment SGI366158[] = {{&S9336[69], 6}, {NULL}};
const Segment SGI366159[] = {{&S9336[8], 3}, {NULL}};
const Segment SGI366160[] = {{&S9336[13], 9}, {NULL}};
const Segment SGI366161[] = {{&S9336[33], 15}, {NULL}};
const Segment SGI366162[] = {{&S9336[49], 6}, {NULL}};
const Segment SGI366163[] = {{&S9336[33], 13}, {NULL}};
const Segment SGI366164[] = {{&S9337[69], 6}, {NULL}};
const Segment SGI366165[] = {{&S9337[8], 3}, {NULL}};
const Segment SGI366166[] = {{&S9337[13], 9}, {NULL}};
const Segment SGI366167[] = {{&S9337[33], 15}, {NULL}};
const Segment SGI366168[] = {{&S9337[49], 6}, {NULL}};
const Segment SGI366169[] = {{&S9337[33], 13}, {NULL}};
const Segment SGI366170[] = {{&S9338[69], 6}, {NULL}};
const Segment SGI366171[] = {{&S9338[8], 3}, {NULL}};
const Segment SGI366172[] = {{&S9338[13], 9}, {NULL}};
const Segment SGI366173[] = {{&S9338[33], 15}, {NULL}};
const Segment SGI366174[] = {{&S9338[49], 6}, {NULL}};
const Segment SGI366175[] = {{&S9338[33], 13}, {NULL}};
const Segment SGI366176[] = {{&S9339[69], 6}, {NULL}};
const Segment SGI366177[] = {{&S9339[8], 3}, {NULL}};
const Segment SGI366178[] = {{&S9339[13], 9}, {NULL}};
const Segment SGI366179[] = {{&S9339[33], 15}, {NULL}};
const Segment SGI366180[] = {{&S9339[49], 6}, {NULL}};
const Segment SGI366181[] = {{&S9339[33], 13}, {NULL}};
const Segment SGI366182[] = {{&S9340[69], 6}, {NULL}};
const Segment SGI366183[] = {{&S9340[8], 3}, {NULL}};
const Segment SGI366184[] = {{&S9340[13], 9}, {NULL}};
const Segment SGI366185[] = {{&S9340[33], 15}, {NULL}};
const Segment SGI366186[] = {{&S9340[49], 6}, {NULL}};
const Segment SGI366187[] = {{&S9340[33], 13}, {NULL}};
const Segment SGI366188[] = {{&S9341[69], 6}, {NULL}};
const Segment SGI366189[] = {{&S9341[8], 3}, {NULL}};
const Segment SGI366190[] = {{&S9341[13], 9}, {NULL}};
const Segment SGI366191[] = {{&S9341[33], 15}, {NULL}};
const Segment SGI366192[] = {{&S9341[49], 6}, {NULL}};
const Segment SGI366193[] = {{&S9341[33], 13}, {NULL}};
const Segment SGI366194[] = {{&S9342[69], 6}, {NULL}};
const Segment SGI366195[] = {{&S9342[8], 3}, {NULL}};
const Segment SGI366196[] = {{&S9342[13], 9}, {NULL}};
const Segment SGI366197[] = {{&S9342[33], 15}, {NULL}};
const Segment SGI366198[] = {{&S9342[49], 6}, {NULL}};
const Segment SGI366199[] = {{&S9342[33], 13}, {NULL}};
const Segment SGI366200[] = {{&S9343[69], 6}, {NULL}};
const Segment SGI366201[] = {{&S9343[8], 3}, {NULL}};
const Segment SGI366202[] = {{&S9343[13], 9}, {NULL}};
const Segment SGI366203[] = {{&S9343[33], 15}, {NULL}};
const Segment SGI366204[] = {{&S9343[49], 6}, {NULL}};
const Segment SGI366205[] = {{&S9343[33], 13}, {NULL}};
const Segment SGI366206[] = {{&S9344[69], 6}, {NULL}};
const Segment SGI366207[] = {{&S9344[8], 3}, {NULL}};
const Segment SGI366208[] = {{&S9344[13], 9}, {NULL}};
const Segment SGI366209[] = {{&S9344[33], 15}, {NULL}};
const Segment SGI366210[] = {{&S9344[49], 6}, {NULL}};
const Segment SGI366211[] = {{&S9344[33], 13}, {NULL}};
const Segment SGI366212[] = {{&S9345[105], 7}, {NULL}};
const Segment SGI366213[] = {{&S9345[8], 3}, {NULL}};
const Segment SGI366214[] = {{&S9345[13], 9}, {NULL}};
const Segment SGI366215[] = {{&S9345[33], 15}, {NULL}};
const Segment SGI366216[] = {{&S9345[49], 6}, {NULL}};
const Segment SGI366217[] = {{&S9345[33], 13}, {NULL}};
const Segment SGI366218[] = {{&S9346[69], 6}, {NULL}};
const Segment SGI366219[] = {{&S9346[8], 3}, {NULL}};
const Segment SGI366220[] = {{&S9346[13], 9}, {NULL}};
const Segment SGI366221[] = {{&S9346[33], 15}, {NULL}};
const Segment SGI366222[] = {{&S9346[49], 6}, {NULL}};
const Segment SGI366223[] = {{&S9346[33], 13}, {NULL}};
const Segment SGI366224[] = {{&S9347[69], 6}, {NULL}};
const Segment SGI366225[] = {{&S9347[8], 3}, {NULL}};
const Segment SGI366226[] = {{&S9347[13], 9}, {NULL}};
const Segment SGI366227[] = {{&S9347[33], 15}, {NULL}};
const Segment SGI366228[] = {{&S9347[49], 6}, {NULL}};
const Segment SGI366229[] = {{&S9347[33], 13}, {NULL}};
const Segment SGI366230[] = {{&S9348[69], 6}, {NULL}};
const Segment SGI366231[] = {{&S9348[8], 3}, {NULL}};
const Segment SGI366232[] = {{&S9348[13], 9}, {NULL}};
const Segment SGI366233[] = {{&S9348[33], 15}, {NULL}};
const Segment SGI366234[] = {{&S9348[49], 6}, {NULL}};
const Segment SGI366235[] = {{&S9348[33], 13}, {NULL}};
const Segment SGI366236[] = {{&S9349[69], 6}, {NULL}};
const Segment SGI366237[] = {{&S9349[8], 3}, {NULL}};
const Segment SGI366238[] = {{&S9349[13], 9}, {NULL}};
const Segment SGI366239[] = {{&S9349[33], 15}, {NULL}};
const Segment SGI366240[] = {{&S9349[49], 6}, {NULL}};
const Segment SGI366241[] = {{&S9349[33], 13}, {NULL}};
const Segment SGI366242[] = {{&S9350[69], 6}, {NULL}};
const Segment SGI366243[] = {{&S9350[8], 3}, {NULL}};
const Segment SGI366244[] = {{&S9350[13], 9}, {NULL}};
const Segment SGI366245[] = {{&S9350[33], 15}, {NULL}};
const Segment SGI366246[] = {{&S9350[49], 6}, {NULL}};
const Segment SGI366247[] = {{&S9350[33], 13}, {NULL}};
const Segment SGI366248[] = {{&S9351[69], 6}, {NULL}};
const Segment SGI366249[] = {{&S9351[8], 3}, {NULL}};
const Segment SGI366250[] = {{&S9351[13], 9}, {NULL}};
const Segment SGI366251[] = {{&S9351[33], 15}, {NULL}};
const Segment SGI366252[] = {{&S9351[49], 6}, {NULL}};
const Segment SGI366253[] = {{&S9351[33], 13}, {NULL}};
const Segment SGI366254[] = {{&S9352[69], 6}, {NULL}};
const Segment SGI366255[] = {{&S9352[8], 3}, {NULL}};
const Segment SGI366256[] = {{&S9352[13], 9}, {NULL}};
const Segment SGI366257[] = {{&S9352[33], 15}, {NULL}};
const Segment SGI366258[] = {{&S9352[49], 6}, {NULL}};
const Segment SGI366259[] = {{&S9352[33], 13}, {NULL}};
const Segment SGI366260[] = {{&S9353[69], 6}, {NULL}};
const Segment SGI366261[] = {{&S9353[8], 3}, {NULL}};
const Segment SGI366262[] = {{&S9353[13], 9}, {NULL}};
const Segment SGI366263[] = {{&S9353[33], 15}, {NULL}};
const Segment SGI366264[] = {{&S9353[49], 6}, {NULL}};
const Segment SGI366265[] = {{&S9353[33], 13}, {NULL}};
const Segment SGI366266[] = {{&S9354[69], 6}, {NULL}};
const Segment SGI366267[] = {{&S9354[8], 3}, {NULL}};
const Segment SGI366268[] = {{&S9354[13], 9}, {NULL}};
const Segment SGI366269[] = {{&S9354[33], 15}, {NULL}};
const Segment SGI366270[] = {{&S9354[49], 6}, {NULL}};
const Segment SGI366271[] = {{&S9354[33], 13}, {NULL}};
const Segment SGI366272[] = {{&S9355[69], 6}, {NULL}};
const Segment SGI366273[] = {{&S9355[8], 3}, {NULL}};
const Segment SGI366274[] = {{&S9355[13], 9}, {NULL}};
const Segment SGI366275[] = {{&S9355[33], 15}, {NULL}};
const Segment SGI366276[] = {{&S9355[49], 6}, {NULL}};
const Segment SGI366277[] = {{&S9355[33], 13}, {NULL}};
const Segment SGI366278[] = {{&S9356[69], 6}, {NULL}};
const Segment SGI366279[] = {{&S9356[8], 3}, {NULL}};
const Segment SGI366280[] = {{&S9356[13], 9}, {NULL}};
const Segment SGI366281[] = {{&S9356[33], 15}, {NULL}};
const Segment SGI366282[] = {{&S9356[49], 6}, {NULL}};
const Segment SGI366283[] = {{&S9356[33], 13}, {NULL}};
const Segment SGI366284[] = {{&S9357[69], 6}, {NULL}};
const Segment SGI366285[] = {{&S9357[8], 3}, {NULL}};
const Segment SGI366286[] = {{&S9357[13], 9}, {NULL}};
const Segment SGI366287[] = {{&S9357[33], 15}, {NULL}};
const Segment SGI366288[] = {{&S9357[49], 6}, {NULL}};
const Segment SGI366289[] = {{&S9357[33], 13}, {NULL}};
const Segment SGI366290[] = {{&S9358[69], 6}, {NULL}};
const Segment SGI366291[] = {{&S9358[8], 3}, {NULL}};
const Segment SGI366292[] = {{&S9358[13], 9}, {NULL}};
const Segment SGI366293[] = {{&S9358[33], 15}, {NULL}};
const Segment SGI366294[] = {{&S9358[49], 6}, {NULL}};
const Segment SGI366295[] = {{&S9358[33], 13}, {NULL}};
const Segment SGI366296[] = {{&S9359[69], 6}, {NULL}};
const Segment SGI366297[] = {{&S9359[8], 3}, {NULL}};
const Segment SGI366298[] = {{&S9359[13], 9}, {NULL}};
const Segment SGI366299[] = {{&S9359[33], 15}, {NULL}};
const Segment SGI366300[] = {{&S9359[49], 6}, {NULL}};
const Segment SGI366301[] = {{&S9359[33], 13}, {NULL}};
const Segment SGI366302[] = {{&S9360[69], 6}, {NULL}};
const Segment SGI366303[] = {{&S9360[8], 3}, {NULL}};
const Segment SGI366304[] = {{&S9360[13], 9}, {NULL}};
const Segment SGI366305[] = {{&S9360[33], 15}, {NULL}};
const Segment SGI366306[] = {{&S9360[49], 6}, {NULL}};
const Segment SGI366307[] = {{&S9360[33], 13}, {NULL}};
const Segment SGI366308[] = {{&S9361[69], 6}, {NULL}};
const Segment SGI366309[] = {{&S9361[8], 3}, {NULL}};
const Segment SGI366310[] = {{&S9361[13], 9}, {NULL}};
const Segment SGI366311[] = {{&S9361[33], 15}, {NULL}};
const Segment SGI366312[] = {{&S9361[49], 6}, {NULL}};
const Segment SGI366313[] = {{&S9361[33], 13}, {NULL}};
const Segment SGI366314[] = {{&S9362[66], 6}, {NULL}};
const Segment SGI366315[] = {{&S9362[8], 3}, {NULL}};
const Segment SGI366316[] = {{&S9362[13], 9}, {NULL}};
const Segment SGI366317[] = {{&S9362[33], 15}, {NULL}};
const Segment SGI366318[] = {{&S9362[49], 3}, {NULL}};
const Segment SGI366319[] = {{&S9362[33], 13}, {NULL}};
const Segment SGI366320[] = {{&S9363[69], 6}, {NULL}};
const Segment SGI366321[] = {{&S9363[8], 3}, {NULL}};
const Segment SGI366322[] = {{&S9363[13], 9}, {NULL}};
const Segment SGI366323[] = {{&S9363[33], 15}, {NULL}};
const Segment SGI366324[] = {{&S9363[49], 6}, {NULL}};
const Segment SGI366325[] = {{&S9363[33], 13}, {NULL}};
const Segment SGI366326[] = {{&S9364[69], 6}, {NULL}};
const Segment SGI366327[] = {{&S9364[8], 3}, {NULL}};
const Segment SGI366328[] = {{&S9364[13], 9}, {NULL}};
const Segment SGI366329[] = {{&S9364[33], 15}, {NULL}};
const Segment SGI366330[] = {{&S9364[49], 6}, {NULL}};
const Segment SGI366331[] = {{&S9364[33], 13}, {NULL}};
const Segment SGI366332[] = {{&S9365[69], 6}, {NULL}};
const Segment SGI366333[] = {{&S9365[8], 3}, {NULL}};
const Segment SGI366334[] = {{&S9365[13], 9}, {NULL}};
const Segment SGI366335[] = {{&S9365[33], 15}, {NULL}};
const Segment SGI366336[] = {{&S9365[49], 6}, {NULL}};
const Segment SGI366337[] = {{&S9365[33], 13}, {NULL}};
const Segment SGI366338[] = {{&S9366[66], 6}, {NULL}};
const Segment SGI366339[] = {{&S9366[8], 3}, {NULL}};
const Segment SGI366340[] = {{&S9366[13], 9}, {NULL}};
const Segment SGI366341[] = {{&S9366[33], 15}, {NULL}};
const Segment SGI366342[] = {{&S9366[49], 3}, {NULL}};
const Segment SGI366343[] = {{&S9366[33], 13}, {NULL}};
const Segment SGI366344[] = {{&S9367[69], 6}, {NULL}};
const Segment SGI366345[] = {{&S9367[8], 3}, {NULL}};
const Segment SGI366346[] = {{&S9367[13], 9}, {NULL}};
const Segment SGI366347[] = {{&S9367[33], 15}, {NULL}};
const Segment SGI366348[] = {{&S9367[49], 6}, {NULL}};
const Segment SGI366349[] = {{&S9367[33], 13}, {NULL}};
const Segment SGI366350[] = {{&S9368[69], 6}, {NULL}};
const Segment SGI366351[] = {{&S9368[8], 3}, {NULL}};
const Segment SGI366352[] = {{&S9368[13], 9}, {NULL}};
const Segment SGI366353[] = {{&S9368[33], 15}, {NULL}};
const Segment SGI366354[] = {{&S9368[49], 6}, {NULL}};
const Segment SGI366355[] = {{&S9368[33], 13}, {NULL}};
const Segment SGI366356[] = {{&S9369[66], 6}, {NULL}};
const Segment SGI366357[] = {{&S9369[8], 3}, {NULL}};
const Segment SGI366358[] = {{&S9369[13], 9}, {NULL}};
const Segment SGI366359[] = {{&S9369[33], 15}, {NULL}};
const Segment SGI366360[] = {{&S9369[49], 3}, {NULL}};
const Segment SGI366361[] = {{&S9369[33], 13}, {NULL}};
const Segment SGI366362[] = {{&S9370[111], 7}, {NULL}};
const Segment SGI366363[] = {{&S9370[8], 3}, {NULL}};
const Segment SGI366364[] = {{&S9370[13], 9}, {NULL}};
const Segment SGI366365[] = {{&S9370[30], 13}, {NULL}};
const Segment SGI366366[] = {{&S9371[109], 6}, {NULL}};
const Segment SGI366367[] = {{&S9371[8], 3}, {NULL}};
const Segment SGI366368[] = {{&S9371[13], 9}, {NULL}};
const Segment SGI366369[] = {{&S9371[30], 13}, {NULL}};
const Segment SGI366370[] = {{&S9372[109], 6}, {NULL}};
const Segment SGI366371[] = {{&S9372[8], 3}, {NULL}};
const Segment SGI366372[] = {{&S9372[13], 9}, {NULL}};
const Segment SGI366373[] = {{&S9372[30], 13}, {NULL}};
const Segment SGI366374[] = {{&S9373[109], 6}, {NULL}};
const Segment SGI366375[] = {{&S9373[8], 3}, {NULL}};
const Segment SGI366376[] = {{&S9373[13], 9}, {NULL}};
const Segment SGI366377[] = {{&S9373[30], 13}, {NULL}};
const Segment SGI366378[] = {{&S9374[107], 6}, {NULL}};
const Segment SGI366379[] = {{&S9374[8], 3}, {NULL}};
const Segment SGI366380[] = {{&S9374[13], 9}, {NULL}};
const Segment SGI366381[] = {{&S9374[30], 13}, {NULL}};
const Segment SGI366382[] = {{&S9375[109], 6}, {NULL}};
const Segment SGI366383[] = {{&S9375[8], 3}, {NULL}};
const Segment SGI366384[] = {{&S9375[13], 9}, {NULL}};
const Segment SGI366385[] = {{&S9375[30], 13}, {NULL}};
const Segment SGI366386[] = {{&S9376[109], 6}, {NULL}};
const Segment SGI366387[] = {{&S9376[8], 3}, {NULL}};
const Segment SGI366388[] = {{&S9376[13], 9}, {NULL}};
const Segment SGI366389[] = {{&S9376[30], 13}, {NULL}};
const Segment SGI366390[] = {{&S9377[109], 6}, {NULL}};
const Segment SGI366391[] = {{&S9377[8], 3}, {NULL}};
const Segment SGI366392[] = {{&S9377[13], 9}, {NULL}};
const Segment SGI366393[] = {{&S9377[30], 13}, {NULL}};
const Segment SGI366394[] = {{&S9378[69], 6}, {NULL}};
const Segment SGI366395[] = {{&S9378[8], 3}, {NULL}};
const Segment SGI366396[] = {{&S9378[13], 9}, {NULL}};
const Segment SGI366397[] = {{&S9378[33], 15}, {NULL}};
const Segment SGI366398[] = {{&S9378[49], 6}, {NULL}};
const Segment SGI366399[] = {{&S9378[33], 13}, {NULL}};
const Segment SGI366400[] = {{&S9379[69], 6}, {NULL}};
const Segment SGI366401[] = {{&S9379[8], 3}, {NULL}};
const Segment SGI366402[] = {{&S9379[13], 9}, {NULL}};
const Segment SGI366403[] = {{&S9379[33], 15}, {NULL}};
const Segment SGI366404[] = {{&S9379[49], 6}, {NULL}};
const Segment SGI366405[] = {{&S9379[33], 13}, {NULL}};
const Segment SGI366406[] = {{&S9380[69], 6}, {NULL}};
const Segment SGI366407[] = {{&S9380[8], 3}, {NULL}};
const Segment SGI366408[] = {{&S9380[13], 9}, {NULL}};
const Segment SGI366409[] = {{&S9380[33], 15}, {NULL}};
const Segment SGI366410[] = {{&S9380[49], 6}, {NULL}};
const Segment SGI366411[] = {{&S9380[33], 13}, {NULL}};
const Segment SGI366412[] = {{&S9381[69], 6}, {NULL}};
const Segment SGI366413[] = {{&S9381[8], 3}, {NULL}};
const Segment SGI366414[] = {{&S9381[13], 9}, {NULL}};
const Segment SGI366415[] = {{&S9381[33], 15}, {NULL}};
const Segment SGI366416[] = {{&S9381[49], 6}, {NULL}};
const Segment SGI366417[] = {{&S9381[33], 13}, {NULL}};
const Segment SGI366418[] = {{&S10290[13], 59}, {NULL}};
const Segment SGI366419[] = {{&S10290[86], 5}, {NULL}};
const Segment SGI366420[] = {{&S10290[120], 3}, {NULL}};
const Segment SGI366421[] = {{&S9382[69], 6}, {NULL}};
const Segment SGI366422[] = {{&S9382[8], 3}, {NULL}};
const Segment SGI366423[] = {{&S9382[13], 9}, {NULL}};
const Segment SGI366424[] = {{&S9382[33], 15}, {NULL}};
const Segment SGI366425[] = {{&S9382[49], 6}, {NULL}};
const Segment SGI366426[] = {{&S9382[33], 13}, {NULL}};
const Segment SGI366427[] = {{&S9383[69], 6}, {NULL}};
const Segment SGI366428[] = {{&S9383[8], 3}, {NULL}};
const Segment SGI366429[] = {{&S9383[13], 9}, {NULL}};
const Segment SGI366430[] = {{&S9383[33], 15}, {NULL}};
const Segment SGI366431[] = {{&S9383[49], 6}, {NULL}};
const Segment SGI366432[] = {{&S9383[33], 13}, {NULL}};
const Segment SGI366433[] = {{&S10294[13], 59}, {NULL}};
const Segment SGI366434[] = {{&S9384[69], 6}, {NULL}};
const Segment SGI366435[] = {{&S10294[86], 5}, {NULL}};
const Segment SGI366436[] = {{&S9384[8], 3}, {NULL}};
const Segment SGI366437[] = {{&S10294[120], 3}, {NULL}};
const Segment SGI366438[] = {{&S9384[13], 9}, {NULL}};
const Segment SGI366439[] = {{&S9384[33], 15}, {NULL}};
const Segment SGI366440[] = {{&S9384[49], 6}, {NULL}};
const Segment SGI366441[] = {{&S9384[33], 13}, {NULL}};
const Segment SGI366442[] = {{&S9385[69], 6}, {NULL}};
const Segment SGI366443[] = {{&S9385[8], 3}, {NULL}};
const Segment SGI366444[] = {{&S9385[13], 9}, {NULL}};
const Segment SGI366445[] = {{&S9385[33], 15}, {NULL}};
const Segment SGI366446[] = {{&S9385[49], 6}, {NULL}};
const Segment SGI366447[] = {{&S9385[33], 13}, {NULL}};
const Segment SGI366448[] = {{&S9386[69], 6}, {NULL}};
const Segment SGI366449[] = {{&S9386[8], 3}, {NULL}};
const Segment SGI366450[] = {{&S9386[13], 9}, {NULL}};
const Segment SGI366451[] = {{&S9386[33], 15}, {NULL}};
const Segment SGI366452[] = {{&S9386[49], 6}, {NULL}};
const Segment SGI366453[] = {{&S9386[33], 13}, {NULL}};
const Segment SGI366454[] = {{&S9387[69], 6}, {NULL}};
const Segment SGI366455[] = {{&S9387[8], 3}, {NULL}};
const Segment SGI366456[] = {{&S9387[13], 9}, {NULL}};
const Segment SGI366457[] = {{&S9387[33], 15}, {NULL}};
const Segment SGI366458[] = {{&S9387[49], 6}, {NULL}};
const Segment SGI366459[] = {{&S9387[33], 13}, {NULL}};
const Segment SGI366460[] = {{&S9388[69], 6}, {NULL}};
const Segment SGI366461[] = {{&S9388[8], 3}, {NULL}};
const Segment SGI366462[] = {{&S9388[13], 9}, {NULL}};
const Segment SGI366463[] = {{&S9388[33], 15}, {NULL}};
const Segment SGI366464[] = {{&S9388[49], 6}, {NULL}};
const Segment SGI366465[] = {{&S9388[33], 13}, {NULL}};
const Segment SGI366466[] = {{&S9389[69], 6}, {NULL}};
const Segment SGI366467[] = {{&S9389[8], 3}, {NULL}};
const Segment SGI366468[] = {{&S9389[13], 9}, {NULL}};
const Segment SGI366469[] = {{&S9389[33], 15}, {NULL}};
const Segment SGI366470[] = {{&S9389[49], 6}, {NULL}};
const Segment SGI366471[] = {{&S9389[33], 13}, {NULL}};
const Segment SGI366472[] = {{&S9390[69], 6}, {NULL}};
const Segment SGI366473[] = {{&S9390[8], 3}, {NULL}};
const Segment SGI366474[] = {{&S9390[13], 9}, {NULL}};
const Segment SGI366475[] = {{&S9390[33], 15}, {NULL}};
const Segment SGI366476[] = {{&S9390[49], 6}, {NULL}};
const Segment SGI366477[] = {{&S9390[33], 13}, {NULL}};
const Segment SGI366478[] = {{&S9391[120], 7}, {NULL}};
const Segment SGI366479[] = {{&S9391[8], 3}, {NULL}};
const Segment SGI366480[] = {{&S9391[13], 9}, {NULL}};
const Segment SGI366481[] = {{&S9391[33], 15}, {NULL}};
const Segment SGI366482[] = {{&S9391[49], 6}, {NULL}};
const Segment SGI366483[] = {{&S9391[33], 13}, {NULL}};
const Segment SGI366484[] = {{&S9392[69], 6}, {NULL}};
const Segment SGI366485[] = {{&S9392[8], 3}, {NULL}};
const Segment SGI366486[] = {{&S9392[13], 9}, {NULL}};
const Segment SGI366487[] = {{&S9392[33], 15}, {NULL}};
const Segment SGI366488[] = {{&S9392[49], 6}, {NULL}};
const Segment SGI366489[] = {{&S9392[33], 13}, {NULL}};
const Segment SGI366490[] = {{&S9393[69], 6}, {NULL}};
const Segment SGI366491[] = {{&S9393[8], 3}, {NULL}};
const Segment SGI366492[] = {{&S9393[13], 9}, {NULL}};
const Segment SGI366493[] = {{&S9393[33], 15}, {NULL}};
const Segment SGI366494[] = {{&S9393[49], 6}, {NULL}};
const Segment SGI366495[] = {{&S9393[33], 13}, {NULL}};
const Segment SGI366496[] = {{&S9394[69], 6}, {NULL}};
const Segment SGI366497[] = {{&S9394[8], 3}, {NULL}};
const Segment SGI366498[] = {{&S9394[13], 9}, {NULL}};
const Segment SGI366499[] = {{&S9394[33], 15}, {NULL}};
const Segment SGI366500[] = {{&S9394[49], 6}, {NULL}};
const Segment SGI366501[] = {{&S9394[33], 13}, {NULL}};
const Segment SGI366502[] = {{&S9395[69], 6}, {NULL}};
const Segment SGI366503[] = {{&S9395[8], 3}, {NULL}};
const Segment SGI366504[] = {{&S9395[13], 9}, {NULL}};
const Segment SGI366505[] = {{&S9395[33], 15}, {NULL}};
const Segment SGI366506[] = {{&S9395[49], 6}, {NULL}};
const Segment SGI366507[] = {{&S9395[33], 13}, {NULL}};
const Segment SGI366508[] = {{&S10314[13], 59}, {NULL}};
const Segment SGI366509[] = {{&S10314[86], 5}, {NULL}};
const Segment SGI366510[] = {{&S9396[69], 6}, {NULL}};
const Segment SGI366511[] = {{&S10314[120], 3}, {NULL}};
const Segment SGI366512[] = {{&S9396[8], 3}, {NULL}};
const Segment SGI366513[] = {{&S9396[13], 9}, {NULL}};
const Segment SGI366514[] = {{&S9396[33], 15}, {NULL}};
const Segment SGI366515[] = {{&S9396[49], 6}, {NULL}};
const Segment SGI366516[] = {{&S9396[33], 13}, {NULL}};
const Segment SGI366517[] = {{&S9397[105], 7}, {NULL}};
const Segment SGI366518[] = {{&S9397[8], 3}, {NULL}};
const Segment SGI366519[] = {{&S9397[13], 9}, {NULL}};
const Segment SGI366520[] = {{&S9397[33], 15}, {NULL}};
const Segment SGI366521[] = {{&S9397[49], 6}, {NULL}};
const Segment SGI366522[] = {{&S9397[33], 13}, {NULL}};
const Segment SGI366523[] = {{&S9398[69], 6}, {NULL}};
const Segment SGI366524[] = {{&S9398[8], 3}, {NULL}};
const Segment SGI366525[] = {{&S9398[13], 9}, {NULL}};
const Segment SGI366526[] = {{&S10318[13], 59}, {NULL}};
const Segment SGI366527[] = {{&S9398[33], 15}, {NULL}};
const Segment SGI366528[] = {{&S10318[86], 5}, {NULL}};
const Segment SGI366529[] = {{&S10318[120], 3}, {NULL}};
const Segment SGI366530[] = {{&S9398[49], 6}, {NULL}};
const Segment SGI366531[] = {{&S9398[33], 13}, {NULL}};
const Segment SGI366532[] = {{&S9399[69], 6}, {NULL}};
const Segment SGI366533[] = {{&S9399[8], 3}, {NULL}};
const Segment SGI366534[] = {{&S9399[13], 9}, {NULL}};
const Segment SGI366535[] = {{&S9399[33], 15}, {NULL}};
const Segment SGI366536[] = {{&S9399[49], 6}, {NULL}};
const Segment SGI366537[] = {{&S9399[33], 13}, {NULL}};
const Segment SGI366538[] = {{&S9400[69], 6}, {NULL}};
const Segment SGI366539[] = {{&S9400[8], 3}, {NULL}};
const Segment SGI366540[] = {{&S9400[13], 9}, {NULL}};
const Segment SGI366541[] = {{&S9400[33], 15}, {NULL}};
const Segment SGI366542[] = {{&S9400[49], 6}, {NULL}};
const Segment SGI366543[] = {{&S9400[33], 13}, {NULL}};
const Segment SGI366544[] = {{&S9401[69], 6}, {NULL}};
const Segment SGI366545[] = {{&S9401[8], 3}, {NULL}};
const Segment SGI366546[] = {{&S9401[13], 9}, {NULL}};
const Segment SGI366547[] = {{&S9401[33], 15}, {NULL}};
const Segment SGI366548[] = {{&S9401[49], 6}, {NULL}};
const Segment SGI366549[] = {{&S9401[33], 13}, {NULL}};
const Segment SGI366550[] = {{&S9402[69], 6}, {NULL}};
const Segment SGI366551[] = {{&S9402[8], 3}, {NULL}};
const Segment SGI366552[] = {{&S9402[13], 9}, {NULL}};
const Segment SGI366553[] = {{&S9402[33], 15}, {NULL}};
const Segment SGI366554[] = {{&S9402[49], 6}, {NULL}};
const Segment SGI366555[] = {{&S9402[33], 13}, {NULL}};
const Segment SGI366556[] = {{&S9403[69], 6}, {NULL}};
const Segment SGI366557[] = {{&S9403[8], 3}, {NULL}};
const Segment SGI366558[] = {{&S9403[13], 9}, {NULL}};
const Segment SGI366559[] = {{&S9403[33], 15}, {NULL}};
const Segment SGI366560[] = {{&S9403[49], 6}, {NULL}};
const Segment SGI366561[] = {{&S9403[33], 13}, {NULL}};
const Segment SGI366562[] = {{&S9404[69], 6}, {NULL}};
const Segment SGI366563[] = {{&S9404[8], 3}, {NULL}};
const Segment SGI366564[] = {{&S9404[13], 9}, {NULL}};
const Segment SGI366565[] = {{&S9404[33], 15}, {NULL}};
const Segment SGI366566[] = {{&S9404[49], 6}, {NULL}};
const Segment SGI366567[] = {{&S9404[33], 13}, {NULL}};
const Segment SGI366568[] = {{&S9405[69], 6}, {NULL}};
const Segment SGI366569[] = {{&S9405[8], 3}, {NULL}};
const Segment SGI366570[] = {{&S9405[13], 9}, {NULL}};
const Segment SGI366571[] = {{&S9405[33], 15}, {NULL}};
const Segment SGI366572[] = {{&S9405[49], 6}, {NULL}};
const Segment SGI366573[] = {{&S9405[33], 13}, {NULL}};
const Segment SGI366574[] = {{&S9406[69], 6}, {NULL}};
const Segment SGI366575[] = {{&S9406[8], 3}, {NULL}};
const Segment SGI366576[] = {{&S9406[13], 9}, {NULL}};
const Segment SGI366577[] = {{&S9406[33], 15}, {NULL}};
const Segment SGI366578[] = {{&S9406[49], 6}, {NULL}};
const Segment SGI366579[] = {{&S9406[33], 13}, {NULL}};
const Segment SGI366580[] = {{&S9407[69], 6}, {NULL}};
const Segment SGI366581[] = {{&S9407[8], 3}, {NULL}};
const Segment SGI366582[] = {{&S9407[13], 9}, {NULL}};
const Segment SGI366583[] = {{&S9407[33], 15}, {NULL}};
const Segment SGI366584[] = {{&S9407[49], 6}, {NULL}};
const Segment SGI366585[] = {{&S9407[33], 13}, {NULL}};
const Segment SGI366586[] = {{&S9408[66], 6}, {NULL}};
const Segment SGI366587[] = {{&S9408[8], 3}, {NULL}};
const Segment SGI366588[] = {{&S9408[13], 9}, {NULL}};
const Segment SGI366589[] = {{&S9408[33], 15}, {NULL}};
const Segment SGI366590[] = {{&S9408[49], 3}, {NULL}};
const Segment SGI366591[] = {{&S9408[33], 13}, {NULL}};
const Segment SGI366592[] = {{&S9409[69], 6}, {NULL}};
const Segment SGI366593[] = {{&S9409[8], 3}, {NULL}};
const Segment SGI366594[] = {{&S9409[13], 9}, {NULL}};
const Segment SGI366595[] = {{&S9409[33], 15}, {NULL}};
const Segment SGI366596[] = {{&S9409[49], 6}, {NULL}};
const Segment SGI366597[] = {{&S9409[33], 13}, {NULL}};
const Segment SGI366598[] = {{&S9410[66], 6}, {NULL}};
const Segment SGI366599[] = {{&S9410[8], 3}, {NULL}};
const Segment SGI366600[] = {{&S9410[13], 9}, {NULL}};
const Segment SGI366601[] = {{&S9410[33], 15}, {NULL}};
const Segment SGI366602[] = {{&S9410[49], 3}, {NULL}};
const Segment SGI366603[] = {{&S9410[33], 13}, {NULL}};
const Segment SGI366604[] = {{&S9411[69], 6}, {NULL}};
const Segment SGI366605[] = {{&S9411[8], 3}, {NULL}};
const Segment SGI366606[] = {{&S9411[13], 9}, {NULL}};
const Segment SGI366607[] = {{&S9411[33], 15}, {NULL}};
const Segment SGI366608[] = {{&S9411[49], 6}, {NULL}};
const Segment SGI366609[] = {{&S9411[33], 13}, {NULL}};
const Segment SGI366610[] = {{&S9412[106], 7}, {NULL}};
const Segment SGI366611[] = {{&S9412[8], 3}, {NULL}};
const Segment SGI366612[] = {{&S9412[13], 9}, {NULL}};
const Segment SGI366613[] = {{&S9412[33], 15}, {NULL}};
const Segment SGI366614[] = {{&S9412[49], 6}, {NULL}};
const Segment SGI366615[] = {{&S9412[33], 13}, {NULL}};
const Segment SGI366616[] = {{&S9413[69], 6}, {NULL}};
const Segment SGI366617[] = {{&S9413[8], 3}, {NULL}};
const Segment SGI366618[] = {{&S9413[13], 9}, {NULL}};
const Segment SGI366619[] = {{&S9413[33], 15}, {NULL}};
const Segment SGI366620[] = {{&S9413[49], 6}, {NULL}};
const Segment SGI366621[] = {{&S9413[33], 13}, {NULL}};
const Segment SGI366622[] = {{&S9414[69], 6}, {NULL}};
const Segment SGI366623[] = {{&S9414[8], 3}, {NULL}};
const Segment SGI366624[] = {{&S9414[13], 9}, {NULL}};
const Segment SGI366625[] = {{&S9414[33], 15}, {NULL}};
const Segment SGI366626[] = {{&S9414[49], 6}, {NULL}};
const Segment SGI366627[] = {{&S9414[33], 13}, {NULL}};
const Segment SGI366628[] = {{&S9415[69], 6}, {NULL}};
const Segment SGI366629[] = {{&S9415[8], 3}, {NULL}};
const Segment SGI366630[] = {{&S9415[13], 9}, {NULL}};
const Segment SGI366631[] = {{&S9415[33], 15}, {NULL}};
const Segment SGI366632[] = {{&S9415[49], 6}, {NULL}};
const Segment SGI366633[] = {{&S9415[33], 13}, {NULL}};
const Segment SGI366634[] = {{&S9416[69], 6}, {NULL}};
const Segment SGI366635[] = {{&S9416[8], 3}, {NULL}};
const Segment SGI366636[] = {{&S9416[13], 9}, {NULL}};
const Segment SGI366637[] = {{&S9416[33], 15}, {NULL}};
const Segment SGI366638[] = {{&S9416[49], 6}, {NULL}};
const Segment SGI366639[] = {{&S9416[33], 13}, {NULL}};
const Segment SGI366640[] = {{&S9417[67], 6}, {NULL}};
const Segment SGI366641[] = {{&S9417[8], 3}, {NULL}};
const Segment SGI366642[] = {{&S9417[13], 9}, {NULL}};
const Segment SGI366643[] = {{&S9417[31], 15}, {NULL}};
const Segment SGI366644[] = {{&S9417[47], 6}, {NULL}};
const Segment SGI366645[] = {{&S9417[31], 13}, {NULL}};
const Segment SGI366646[] = {{&S9418[69], 6}, {NULL}};
const Segment SGI366647[] = {{&S9418[8], 3}, {NULL}};
const Segment SGI366648[] = {{&S9418[13], 9}, {NULL}};
const Segment SGI366649[] = {{&S9418[33], 15}, {NULL}};
const Segment SGI366650[] = {{&S9418[49], 6}, {NULL}};
const Segment SGI366651[] = {{&S9418[33], 13}, {NULL}};
const Segment SGI366652[] = {{&S9419[69], 6}, {NULL}};
const Segment SGI366653[] = {{&S9419[8], 3}, {NULL}};
const Segment SGI366654[] = {{&S9419[13], 9}, {NULL}};
const Segment SGI366655[] = {{&S9419[33], 15}, {NULL}};
const Segment SGI366656[] = {{&S9419[49], 6}, {NULL}};
const Segment SGI366657[] = {{&S9419[33], 13}, {NULL}};
const Segment SGI366658[] = {{&S9420[69], 6}, {NULL}};
const Segment SGI366659[] = {{&S9420[8], 3}, {NULL}};
const Segment SGI366660[] = {{&S9420[13], 9}, {NULL}};
const Segment SGI366661[] = {{&S9420[33], 15}, {NULL}};
const Segment SGI366662[] = {{&S9420[49], 6}, {NULL}};
const Segment SGI366663[] = {{&S9420[33], 13}, {NULL}};
const Segment SGI366664[] = {{&S9421[67], 6}, {NULL}};
const Segment SGI366665[] = {{&S9421[8], 3}, {NULL}};
const Segment SGI366666[] = {{&S9421[13], 9}, {NULL}};
const Segment SGI366667[] = {{&S9421[31], 15}, {NULL}};
const Segment SGI366668[] = {{&S9421[47], 6}, {NULL}};
const Segment SGI366669[] = {{&S9421[31], 13}, {NULL}};
const Segment SGI366670[] = {{&S9422[66], 6}, {NULL}};
const Segment SGI366671[] = {{&S9422[8], 3}, {NULL}};
const Segment SGI366672[] = {{&S9422[13], 9}, {NULL}};
const Segment SGI366673[] = {{&S9422[33], 15}, {NULL}};
const Segment SGI366674[] = {{&S9422[49], 3}, {NULL}};
const Segment SGI366675[] = {{&S9422[33], 13}, {NULL}};
const Segment SGI366676[] = {{&S9423[69], 6}, {NULL}};
const Segment SGI366677[] = {{&S9423[8], 3}, {NULL}};
const Segment SGI366678[] = {{&S9423[13], 9}, {NULL}};
const Segment SGI366679[] = {{&S9423[33], 15}, {NULL}};
const Segment SGI366680[] = {{&S9423[49], 6}, {NULL}};
const Segment SGI366681[] = {{&S9423[33], 13}, {NULL}};
const Segment SGI366682[] = {{&S9424[69], 6}, {NULL}};
const Segment SGI366683[] = {{&S9424[8], 3}, {NULL}};
const Segment SGI366684[] = {{&S9424[13], 9}, {NULL}};
const Segment SGI366685[] = {{&S9424[33], 15}, {NULL}};
const Segment SGI366686[] = {{&S9424[49], 6}, {NULL}};
const Segment SGI366687[] = {{&S9424[33], 13}, {NULL}};
const Segment SGI366688[] = {{&S9425[69], 6}, {NULL}};
const Segment SGI366689[] = {{&S9425[8], 3}, {NULL}};
const Segment SGI366690[] = {{&S9425[13], 9}, {NULL}};
const Segment SGI366691[] = {{&S9425[33], 15}, {NULL}};
const Segment SGI366692[] = {{&S9425[49], 6}, {NULL}};
const Segment SGI366693[] = {{&S9425[33], 13}, {NULL}};
const Segment SGI366694[] = {{&S9426[69], 6}, {NULL}};
const Segment SGI366695[] = {{&S9426[8], 3}, {NULL}};
const Segment SGI366696[] = {{&S9426[13], 9}, {NULL}};
const Segment SGI366697[] = {{&S9426[33], 15}, {NULL}};
const Segment SGI366698[] = {{&S9426[49], 6}, {NULL}};
const Segment SGI366699[] = {{&S9426[33], 13}, {NULL}};
const Segment SGI366700[] = {{&S9427[69], 6}, {NULL}};
const Segment SGI366701[] = {{&S9427[8], 3}, {NULL}};
const Segment SGI366702[] = {{&S9427[13], 9}, {NULL}};
const Segment SGI366703[] = {{&S9427[33], 15}, {NULL}};
const Segment SGI366704[] = {{&S9427[49], 6}, {NULL}};
const Segment SGI366705[] = {{&S9427[33], 13}, {NULL}};
const Segment SGI366706[] = {{&S9428[69], 6}, {NULL}};
const Segment SGI366707[] = {{&S9428[8], 3}, {NULL}};
const Segment SGI366708[] = {{&S9428[13], 9}, {NULL}};
const Segment SGI366709[] = {{&S9428[33], 15}, {NULL}};
const Segment SGI366710[] = {{&S9428[49], 6}, {NULL}};
const Segment SGI366711[] = {{&S9428[33], 13}, {NULL}};
const Segment SGI366712[] = {{&S9429[69], 6}, {NULL}};
const Segment SGI366713[] = {{&S9429[8], 3}, {NULL}};
const Segment SGI366714[] = {{&S9429[13], 9}, {NULL}};
const Segment SGI366715[] = {{&S9429[33], 15}, {NULL}};
const Segment SGI366716[] = {{&S9429[49], 6}, {NULL}};
const Segment SGI366717[] = {{&S9429[33], 13}, {NULL}};
const Segment SGI366718[] = {{&S9430[69], 6}, {NULL}};
const Segment SGI366719[] = {{&S9430[8], 3}, {NULL}};
const Segment SGI366720[] = {{&S9430[13], 9}, {NULL}};
const Segment SGI366721[] = {{&S9430[33], 15}, {NULL}};
const Segment SGI366722[] = {{&S9430[49], 6}, {NULL}};
const Segment SGI366723[] = {{&S9430[33], 13}, {NULL}};
const Segment SGI366724[] = {{&S9431[69], 6}, {NULL}};
const Segment SGI366725[] = {{&S9431[8], 3}, {NULL}};
const Segment SGI366726[] = {{&S9431[13], 9}, {NULL}};
const Segment SGI366727[] = {{&S9431[33], 15}, {NULL}};
const Segment SGI366728[] = {{&S9431[49], 6}, {NULL}};
const Segment SGI366729[] = {{&S9431[33], 13}, {NULL}};
const Segment SGI366730[] = {{&S9432[69], 6}, {NULL}};
const Segment SGI366731[] = {{&S9432[8], 3}, {NULL}};
const Segment SGI366732[] = {{&S9432[13], 9}, {NULL}};
const Segment SGI366733[] = {{&S9432[33], 15}, {NULL}};
const Segment SGI366734[] = {{&S9432[49], 6}, {NULL}};
const Segment SGI366735[] = {{&S9432[33], 13}, {NULL}};
const Segment SGI366736[] = {{&S9433[69], 6}, {NULL}};
const Segment SGI366737[] = {{&S9433[8], 3}, {NULL}};
const Segment SGI366738[] = {{&S9433[13], 9}, {NULL}};
const Segment SGI366739[] = {{&S9433[33], 15}, {NULL}};
const Segment SGI366740[] = {{&S9433[49], 6}, {NULL}};
const Segment SGI366741[] = {{&S9433[33], 13}, {NULL}};
const Segment SGI366742[] = {{&S9434[69], 6}, {NULL}};
const Segment SGI366743[] = {{&S9434[8], 3}, {NULL}};
const Segment SGI366744[] = {{&S9434[13], 9}, {NULL}};
const Segment SGI366745[] = {{&S9434[33], 15}, {NULL}};
const Segment SGI366746[] = {{&S9434[49], 6}, {NULL}};
const Segment SGI366747[] = {{&S9434[33], 13}, {NULL}};
const Segment SGI366748[] = {{&S9435[69], 6}, {NULL}};
const Segment SGI366749[] = {{&S9435[8], 3}, {NULL}};
const Segment SGI366750[] = {{&S9435[13], 9}, {NULL}};
const Segment SGI366751[] = {{&S9435[33], 15}, {NULL}};
const Segment SGI366752[] = {{&S9435[49], 6}, {NULL}};
const Segment SGI366753[] = {{&S9435[33], 13}, {NULL}};
const Segment SGI366754[] = {{&S9436[69], 6}, {NULL}};
const Segment SGI366755[] = {{&S9436[8], 3}, {NULL}};
const Segment SGI366756[] = {{&S9436[13], 9}, {NULL}};
const Segment SGI366757[] = {{&S9436[33], 15}, {NULL}};
const Segment SGI366758[] = {{&S9436[49], 6}, {NULL}};
const Segment SGI366759[] = {{&S9436[33], 13}, {NULL}};
const Segment SGI366760[] = {{&S9437[69], 6}, {NULL}};
const Segment SGI366761[] = {{&S9437[8], 3}, {NULL}};
const Segment SGI366762[] = {{&S9437[13], 9}, {NULL}};
const Segment SGI366763[] = {{&S9437[33], 15}, {NULL}};
const Segment SGI366764[] = {{&S9437[49], 6}, {NULL}};
const Segment SGI366765[] = {{&S9437[33], 13}, {NULL}};
const Segment SGI366766[] = {{&S9438[69], 6}, {NULL}};
const Segment SGI366767[] = {{&S9438[8], 3}, {NULL}};
const Segment SGI366768[] = {{&S9438[13], 9}, {NULL}};
const Segment SGI366769[] = {{&S9438[33], 15}, {NULL}};
const Segment SGI366770[] = {{&S9438[49], 6}, {NULL}};
const Segment SGI366771[] = {{&S9438[33], 13}, {NULL}};
const Segment SGI366772[] = {{&S9439[69], 6}, {NULL}};
const Segment SGI366773[] = {{&S9439[8], 3}, {NULL}};
const Segment SGI366774[] = {{&S9439[13], 9}, {NULL}};
const Segment SGI366775[] = {{&S9439[33], 15}, {NULL}};
const Segment SGI366776[] = {{&S9439[49], 6}, {NULL}};
const Segment SGI366777[] = {{&S9439[33], 13}, {NULL}};
const Segment SGI366778[] = {{&S9440[69], 6}, {NULL}};
const Segment SGI366779[] = {{&S9440[8], 3}, {NULL}};
const Segment SGI366780[] = {{&S9440[13], 9}, {NULL}};
const Segment SGI366781[] = {{&S9440[33], 15}, {NULL}};
const Segment SGI366782[] = {{&S9440[49], 6}, {NULL}};
const Segment SGI366783[] = {{&S9440[33], 13}, {NULL}};
const Segment SGI366784[] = {{&S9441[66], 6}, {NULL}};
const Segment SGI366785[] = {{&S9441[8], 3}, {NULL}};
const Segment SGI366786[] = {{&S9441[13], 9}, {NULL}};
const Segment SGI366787[] = {{&S9441[33], 15}, {NULL}};
const Segment SGI366788[] = {{&S9441[49], 3}, {NULL}};
const Segment SGI366789[] = {{&S9441[33], 13}, {NULL}};
const Segment SGI366790[] = {{&S9442[69], 6}, {NULL}};
const Segment SGI366791[] = {{&S9442[8], 3}, {NULL}};
const Segment SGI366792[] = {{&S9442[13], 9}, {NULL}};
const Segment SGI366793[] = {{&S9442[33], 15}, {NULL}};
const Segment SGI366794[] = {{&S9442[49], 6}, {NULL}};
const Segment SGI366795[] = {{&S9442[33], 13}, {NULL}};
const Segment SGI366796[] = {{&S9443[69], 6}, {NULL}};
const Segment SGI366797[] = {{&S9443[8], 3}, {NULL}};
const Segment SGI366798[] = {{&S9443[13], 9}, {NULL}};
const Segment SGI366799[] = {{&S9443[33], 15}, {NULL}};
const Segment SGI366800[] = {{&S9443[49], 6}, {NULL}};
const Segment SGI366801[] = {{&S9443[33], 13}, {NULL}};
const Segment SGI366802[] = {{&S9444[69], 6}, {NULL}};
const Segment SGI366803[] = {{&S9444[8], 3}, {NULL}};
const Segment SGI366804[] = {{&S9444[13], 9}, {NULL}};
const Segment SGI366805[] = {{&S9444[33], 15}, {NULL}};
const Segment SGI366806[] = {{&S9444[49], 6}, {NULL}};
const Segment SGI366807[] = {{&S9444[33], 13}, {NULL}};
const Segment SGI366808[] = {{&S9445[69], 6}, {NULL}};
const Segment SGI366809[] = {{&S9445[8], 3}, {NULL}};
const Segment SGI366810[] = {{&S9445[13], 9}, {NULL}};
const Segment SGI366811[] = {{&S9445[33], 15}, {NULL}};
const Segment SGI366812[] = {{&S9445[49], 6}, {NULL}};
const Segment SGI366813[] = {{&S9445[33], 13}, {NULL}};
const Segment SGI366814[] = {{&S9446[69], 6}, {NULL}};
const Segment SGI366815[] = {{&S9446[8], 3}, {NULL}};
const Segment SGI366816[] = {{&S9446[13], 9}, {NULL}};
const Segment SGI366817[] = {{&S9446[33], 15}, {NULL}};
const Segment SGI366818[] = {{&S9446[49], 6}, {NULL}};
const Segment SGI366819[] = {{&S9446[33], 13}, {NULL}};
const Segment SGI366820[] = {{&S9447[69], 6}, {NULL}};
const Segment SGI366821[] = {{&S9447[8], 3}, {NULL}};
const Segment SGI366822[] = {{&S9447[13], 9}, {NULL}};
const Segment SGI366823[] = {{&S9447[33], 15}, {NULL}};
const Segment SGI366824[] = {{&S9447[49], 6}, {NULL}};
const Segment SGI366825[] = {{&S9447[33], 13}, {NULL}};
const Segment SGI366826[] = {{&S9448[118], 7}, {NULL}};
const Segment SGI366827[] = {{&S9448[8], 3}, {NULL}};
const Segment SGI366828[] = {{&S9448[13], 9}, {NULL}};
const Segment SGI366829[] = {{&S9448[33], 15}, {NULL}};
const Segment SGI366830[] = {{&S9448[49], 6}, {NULL}};
const Segment SGI366831[] = {{&S9448[33], 13}, {NULL}};
const Segment SGI366832[] = {{&S9449[69], 6}, {NULL}};
const Segment SGI366833[] = {{&S9449[8], 3}, {NULL}};
const Segment SGI366834[] = {{&S9449[13], 9}, {NULL}};
const Segment SGI366835[] = {{&S9449[33], 15}, {NULL}};
const Segment SGI366836[] = {{&S9449[49], 6}, {NULL}};
const Segment SGI366837[] = {{&S9449[33], 13}, {NULL}};
const Segment SGI366838[] = {{&S9450[66], 6}, {NULL}};
const Segment SGI366839[] = {{&S9450[8], 3}, {NULL}};
const Segment SGI366840[] = {{&S9450[13], 9}, {NULL}};
const Segment SGI366841[] = {{&S9450[33], 15}, {NULL}};
const Segment SGI366842[] = {{&S9450[49], 3}, {NULL}};
const Segment SGI366843[] = {{&S9450[33], 13}, {NULL}};
const Segment SGI366844[] = {{&S9451[66], 6}, {NULL}};
const Segment SGI366845[] = {{&S9451[8], 3}, {NULL}};
const Segment SGI366846[] = {{&S9451[13], 9}, {NULL}};
const Segment SGI366847[] = {{&S9451[33], 15}, {NULL}};
const Segment SGI366848[] = {{&S9451[49], 3}, {NULL}};
const Segment SGI366849[] = {{&S9451[33], 13}, {NULL}};
const Segment SGI366850[] = {{&S9452[69], 6}, {NULL}};
const Segment SGI366851[] = {{&S9452[8], 3}, {NULL}};
const Segment SGI366852[] = {{&S9452[13], 9}, {NULL}};
const Segment SGI366853[] = {{&S9452[33], 15}, {NULL}};
const Segment SGI366854[] = {{&S9452[49], 6}, {NULL}};
const Segment SGI366855[] = {{&S9452[33], 13}, {NULL}};
const Segment SGI366856[] = {{&S9453[69], 6}, {NULL}};
const Segment SGI366857[] = {{&S9453[8], 3}, {NULL}};
const Segment SGI366858[] = {{&S9453[13], 9}, {NULL}};
const Segment SGI366859[] = {{&S9453[33], 15}, {NULL}};
const Segment SGI366860[] = {{&S9453[49], 6}, {NULL}};
const Segment SGI366861[] = {{&S9453[33], 13}, {NULL}};
const Segment SGI366862[] = {{&S9454[69], 6}, {NULL}};
const Segment SGI366863[] = {{&S9454[8], 3}, {NULL}};
const Segment SGI366864[] = {{&S9454[13], 9}, {NULL}};
const Segment SGI366865[] = {{&S9454[33], 15}, {NULL}};
const Segment SGI366866[] = {{&S9454[49], 6}, {NULL}};
const Segment SGI366867[] = {{&S9454[33], 13}, {NULL}};
const Segment SGI366868[] = {{&S9455[69], 6}, {NULL}};
const Segment SGI366869[] = {{&S9455[8], 3}, {NULL}};
const Segment SGI366870[] = {{&S9455[13], 9}, {NULL}};
const Segment SGI366871[] = {{&S9455[33], 15}, {NULL}};
const Segment SGI366872[] = {{&S9455[49], 6}, {NULL}};
const Segment SGI366873[] = {{&S9455[33], 13}, {NULL}};
const Segment SGI366874[] = {{&S9456[66], 6}, {NULL}};
const Segment SGI366875[] = {{&S9456[8], 3}, {NULL}};
const Segment SGI366876[] = {{&S9456[13], 9}, {NULL}};
const Segment SGI366877[] = {{&S9456[33], 15}, {NULL}};
const Segment SGI366878[] = {{&S9456[49], 3}, {NULL}};
const Segment SGI366879[] = {{&S9456[33], 13}, {NULL}};
const Segment SGI366880[] = {{&S9457[69], 6}, {NULL}};
const Segment SGI366881[] = {{&S9457[8], 3}, {NULL}};
const Segment SGI366882[] = {{&S9457[13], 9}, {NULL}};
const Segment SGI366883[] = {{&S9457[33], 15}, {NULL}};
const Segment SGI366884[] = {{&S9457[49], 6}, {NULL}};
const Segment SGI366885[] = {{&S9457[33], 13}, {NULL}};
const Segment SGI366886[] = {{&S9458[69], 6}, {NULL}};
const Segment SGI366887[] = {{&S9458[8], 3}, {NULL}};
const Segment SGI366888[] = {{&S9458[13], 9}, {NULL}};
const Segment SGI366889[] = {{&S9458[33], 15}, {NULL}};
const Segment SGI366890[] = {{&S9458[49], 6}, {NULL}};
const Segment SGI366891[] = {{&S9458[33], 13}, {NULL}};
const Segment SGI366892[] = {{&S9459[69], 6}, {NULL}};
const Segment SGI366893[] = {{&S9459[8], 3}, {NULL}};
const Segment SGI366894[] = {{&S9459[13], 9}, {NULL}};
const Segment SGI366895[] = {{&S9459[33], 15}, {NULL}};
const Segment SGI366896[] = {{&S9459[49], 6}, {NULL}};
const Segment SGI366897[] = {{&S9459[33], 13}, {NULL}};
const Segment SGI366898[] = {{&S9460[69], 6}, {NULL}};
const Segment SGI366899[] = {{&S9460[8], 3}, {NULL}};
const Segment SGI366900[] = {{&S9460[13], 9}, {NULL}};
const Segment SGI366901[] = {{&S9460[33], 15}, {NULL}};
const Segment SGI366902[] = {{&S9460[49], 6}, {NULL}};
const Segment SGI366903[] = {{&S9460[33], 13}, {NULL}};
const Segment SGI366904[] = {{&S9461[69], 6}, {NULL}};
const Segment SGI366905[] = {{&S9461[8], 3}, {NULL}};
const Segment SGI366906[] = {{&S9461[13], 9}, {NULL}};
const Segment SGI366907[] = {{&S9461[33], 15}, {NULL}};
const Segment SGI366908[] = {{&S9461[49], 6}, {NULL}};
const Segment SGI366909[] = {{&S9461[33], 13}, {NULL}};
const Segment SGI366910[] = {{&S9462[69], 6}, {NULL}};
const Segment SGI366911[] = {{&S9462[8], 3}, {NULL}};
const Segment SGI366912[] = {{&S9462[13], 9}, {NULL}};
const Segment SGI366913[] = {{&S9462[33], 15}, {NULL}};
const Segment SGI366914[] = {{&S9462[49], 6}, {NULL}};
const Segment SGI366915[] = {{&S9462[33], 13}, {NULL}};
const Segment SGI366916[] = {{&S9463[69], 6}, {NULL}};
const Segment SGI366917[] = {{&S9463[8], 3}, {NULL}};
const Segment SGI366918[] = {{&S9463[13], 9}, {NULL}};
const Segment SGI366919[] = {{&S9463[33], 15}, {NULL}};
const Segment SGI366920[] = {{&S9463[49], 6}, {NULL}};
const Segment SGI366921[] = {{&S9463[33], 13}, {NULL}};
const Segment SGI366922[] = {{&S9464[69], 6}, {NULL}};
const Segment SGI366923[] = {{&S9464[8], 3}, {NULL}};
const Segment SGI366924[] = {{&S9464[13], 9}, {NULL}};
const Segment SGI366925[] = {{&S9464[33], 15}, {NULL}};
const Segment SGI366926[] = {{&S9464[49], 6}, {NULL}};
const Segment SGI366927[] = {{&S9464[33], 13}, {NULL}};
const Segment SGI366928[] = {{&S9465[69], 6}, {NULL}};
const Segment SGI366929[] = {{&S9465[8], 3}, {NULL}};
const Segment SGI366930[] = {{&S9465[13], 9}, {NULL}};
const Segment SGI366931[] = {{&S9465[33], 15}, {NULL}};
const Segment SGI366932[] = {{&S9465[49], 6}, {NULL}};
const Segment SGI366933[] = {{&S9465[33], 13}, {NULL}};
const Segment SGI366934[] = {{&S9466[69], 6}, {NULL}};
const Segment SGI366935[] = {{&S9466[8], 3}, {NULL}};
const Segment SGI366936[] = {{&S9466[13], 9}, {NULL}};
const Segment SGI366937[] = {{&S9466[33], 15}, {NULL}};
const Segment SGI366938[] = {{&S9466[49], 6}, {NULL}};
const Segment SGI366939[] = {{&S9466[33], 13}, {NULL}};
const Segment SGI366940[] = {{&S9467[69], 6}, {NULL}};
const Segment SGI366941[] = {{&S9467[8], 3}, {NULL}};
const Segment SGI366942[] = {{&S9467[13], 9}, {NULL}};
const Segment SGI366943[] = {{&S9467[33], 15}, {NULL}};
const Segment SGI366944[] = {{&S9467[49], 6}, {NULL}};
const Segment SGI366945[] = {{&S9467[33], 13}, {NULL}};
const Segment SGI366946[] = {{&S9468[69], 6}, {NULL}};
const Segment SGI366947[] = {{&S9468[8], 3}, {NULL}};
const Segment SGI366948[] = {{&S9468[13], 9}, {NULL}};
const Segment SGI366949[] = {{&S9468[33], 15}, {NULL}};
const Segment SGI366950[] = {{&S9468[49], 6}, {NULL}};
const Segment SGI366951[] = {{&S9468[33], 13}, {NULL}};
const Segment SGI366952[] = {{&S9469[69], 6}, {NULL}};
const Segment SGI366953[] = {{&S9469[8], 3}, {NULL}};
const Segment SGI366954[] = {{&S9469[13], 9}, {NULL}};
const Segment SGI366955[] = {{&S9469[33], 15}, {NULL}};
const Segment SGI366956[] = {{&S9469[49], 6}, {NULL}};
const Segment SGI366957[] = {{&S9469[33], 13}, {NULL}};
const Segment SGI366958[] = {{&S9470[69], 6}, {NULL}};
const Segment SGI366959[] = {{&S9470[8], 3}, {NULL}};
const Segment SGI366960[] = {{&S9470[13], 9}, {NULL}};
const Segment SGI366961[] = {{&S9470[33], 15}, {NULL}};
const Segment SGI366962[] = {{&S9470[49], 6}, {NULL}};
const Segment SGI366963[] = {{&S9470[33], 13}, {NULL}};
const Segment SGI366964[] = {{&S9471[69], 5}, {NULL}};
const Segment SGI366965[] = {{&S9471[8], 3}, {NULL}};
const Segment SGI366966[] = {{&S9471[13], 9}, {NULL}};
const Segment SGI366967[] = {{&S9471[33], 15}, {NULL}};
const Segment SGI366968[] = {{&S9471[49], 6}, {NULL}};
const Segment SGI366969[] = {{&S9471[33], 13}, {NULL}};
const Segment SGI366970[] = {{&S9472[66], 6}, {NULL}};
const Segment SGI366971[] = {{&S9472[8], 3}, {NULL}};
const Segment SGI366972[] = {{&S9472[13], 9}, {NULL}};
const Segment SGI366973[] = {{&S9472[33], 15}, {NULL}};
const Segment SGI366974[] = {{&S9472[49], 3}, {NULL}};
const Segment SGI366975[] = {{&S9472[33], 13}, {NULL}};
const Segment SGI366976[] = {{&S9473[66], 6}, {NULL}};
const Segment SGI366977[] = {{&S9473[8], 3}, {NULL}};
const Segment SGI366978[] = {{&S9473[13], 9}, {NULL}};
const Segment SGI366979[] = {{&S9473[33], 15}, {NULL}};
const Segment SGI366980[] = {{&S9473[49], 3}, {NULL}};
const Segment SGI366981[] = {{&S9473[33], 13}, {NULL}};
const Segment SGI366982[] = {{&S9474[69], 6}, {NULL}};
const Segment SGI366983[] = {{&S9474[8], 3}, {NULL}};
const Segment SGI366984[] = {{&S9474[13], 9}, {NULL}};
const Segment SGI366985[] = {{&S9474[33], 15}, {NULL}};
const Segment SGI366986[] = {{&S9474[49], 6}, {NULL}};
const Segment SGI366987[] = {{&S9474[33], 13}, {NULL}};
const Segment SGI366988[] = {{&S9475[69], 6}, {NULL}};
const Segment SGI366989[] = {{&S9475[8], 3}, {NULL}};
const Segment SGI366990[] = {{&S9475[13], 9}, {NULL}};
const Segment SGI366991[] = {{&S9475[33], 15}, {NULL}};
const Segment SGI366992[] = {{&S9475[49], 6}, {NULL}};
const Segment SGI366993[] = {{&S9475[33], 13}, {NULL}};
const Segment SGI366994[] = {{&S9476[69], 6}, {NULL}};
const Segment SGI366995[] = {{&S9476[8], 3}, {NULL}};
const Segment SGI366996[] = {{&S9476[13], 9}, {NULL}};
const Segment SGI366997[] = {{&S9476[33], 15}, {NULL}};
const Segment SGI366998[] = {{&S9476[49], 6}, {NULL}};
const Segment SGI366999[] = {{&S9476[33], 13}, {NULL}};
const Segment SGI367000[] = {{&S9477[69], 6}, {NULL}};
const Segment SGI367001[] = {{&S9477[8], 3}, {NULL}};
const Segment SGI367002[] = {{&S9477[13], 9}, {NULL}};
const Segment SGI367003[] = {{&S9477[33], 15}, {NULL}};
const Segment SGI367004[] = {{&S9477[49], 6}, {NULL}};
const Segment SGI367005[] = {{&S9477[33], 13}, {NULL}};
const Segment SGI367006[] = {{&S9478[66], 6}, {NULL}};
const Segment SGI367007[] = {{&S9478[8], 3}, {NULL}};
const Segment SGI367008[] = {{&S9478[13], 9}, {NULL}};
const Segment SGI367009[] = {{&S9478[33], 15}, {NULL}};
const Segment SGI367010[] = {{&S9478[49], 3}, {NULL}};
const Segment SGI367011[] = {{&S9478[33], 13}, {NULL}};
const Segment SGI367012[] = {{&S9479[69], 6}, {NULL}};
const Segment SGI367013[] = {{&S9479[8], 3}, {NULL}};
const Segment SGI367014[] = {{&S9479[13], 9}, {NULL}};
const Segment SGI367015[] = {{&S9479[33], 15}, {NULL}};
const Segment SGI367016[] = {{&S9479[49], 6}, {NULL}};
const Segment SGI367017[] = {{&S9479[33], 13}, {NULL}};
const Segment SGI367018[] = {{&S9480[69], 6}, {NULL}};
const Segment SGI367019[] = {{&S9480[8], 3}, {NULL}};
const Segment SGI367020[] = {{&S9480[13], 9}, {NULL}};
const Segment SGI367021[] = {{&S9480[33], 15}, {NULL}};
const Segment SGI367022[] = {{&S9480[49], 6}, {NULL}};
const Segment SGI367023[] = {{&S9480[33], 13}, {NULL}};
const Segment SGI367024[] = {{&S9481[69], 6}, {NULL}};
const Segment SGI367025[] = {{&S9481[8], 3}, {NULL}};
const Segment SGI367026[] = {{&S9481[13], 9}, {NULL}};
const Segment SGI367027[] = {{&S9481[33], 15}, {NULL}};
const Segment SGI367028[] = {{&S9481[49], 6}, {NULL}};
const Segment SGI367029[] = {{&S9481[33], 13}, {NULL}};
const Segment SGI367030[] = {{&S9482[69], 6}, {NULL}};
const Segment SGI367031[] = {{&S9482[8], 3}, {NULL}};
const Segment SGI367032[] = {{&S9482[13], 9}, {NULL}};
const Segment SGI367033[] = {{&S9482[33], 15}, {NULL}};
const Segment SGI367034[] = {{&S9482[49], 6}, {NULL}};
const Segment SGI367035[] = {{&S9482[33], 13}, {NULL}};
const Segment SGI367036[] = {{&S9483[66], 6}, {NULL}};
const Segment SGI367037[] = {{&S9483[8], 3}, {NULL}};
const Segment SGI367038[] = {{&S9483[13], 9}, {NULL}};
const Segment SGI367039[] = {{&S9483[33], 15}, {NULL}};
const Segment SGI367040[] = {{&S9483[49], 3}, {NULL}};
const Segment SGI367041[] = {{&S9483[33], 13}, {NULL}};
const Segment SGI367042[] = {{&S9484[66], 6}, {NULL}};
const Segment SGI367043[] = {{&S9484[8], 3}, {NULL}};
const Segment SGI367044[] = {{&S9484[13], 9}, {NULL}};
const Segment SGI367045[] = {{&S9484[33], 15}, {NULL}};
const Segment SGI367046[] = {{&S9484[49], 3}, {NULL}};
const Segment SGI367047[] = {{&S9484[33], 13}, {NULL}};
const Segment SGI367048[] = {{&S9485[69], 6}, {NULL}};
const Segment SGI367049[] = {{&S9485[8], 3}, {NULL}};
const Segment SGI367050[] = {{&S9485[13], 9}, {NULL}};
const Segment SGI367051[] = {{&S9485[33], 15}, {NULL}};
const Segment SGI367052[] = {{&S9485[49], 6}, {NULL}};
const Segment SGI367053[] = {{&S9485[33], 13}, {NULL}};
const Segment SGI367054[] = {{&S9486[69], 6}, {NULL}};
const Segment SGI367055[] = {{&S9486[8], 3}, {NULL}};
const Segment SGI367056[] = {{&S9486[13], 9}, {NULL}};
const Segment SGI367057[] = {{&S9486[33], 15}, {NULL}};
const Segment SGI367058[] = {{&S9486[49], 6}, {NULL}};
const Segment SGI367059[] = {{&S9486[33], 13}, {NULL}};
const Segment SGI367060[] = {{&S9487[69], 6}, {NULL}};
const Segment SGI367061[] = {{&S9487[8], 3}, {NULL}};
const Segment SGI367062[] = {{&S9487[13], 9}, {NULL}};
const Segment SGI367063[] = {{&S9487[33], 15}, {NULL}};
const Segment SGI367064[] = {{&S9487[49], 6}, {NULL}};
const Segment SGI367065[] = {{&S9487[33], 13}, {NULL}};
const Segment SGI367066[] = {{&S9488[69], 6}, {NULL}};
const Segment SGI367067[] = {{&S9488[8], 3}, {NULL}};
const Segment SGI367068[] = {{&S9488[13], 9}, {NULL}};
const Segment SGI367069[] = {{&S9488[33], 15}, {NULL}};
const Segment SGI367070[] = {{&S9488[49], 6}, {NULL}};
const Segment SGI367071[] = {{&S9488[33], 13}, {NULL}};
const Segment SGI367072[] = {{&S9489[66], 6}, {NULL}};
const Segment SGI367073[] = {{&S9489[8], 3}, {NULL}};
const Segment SGI367074[] = {{&S9489[13], 9}, {NULL}};
const Segment SGI367075[] = {{&S9489[33], 15}, {NULL}};
const Segment SGI367076[] = {{&S9489[49], 3}, {NULL}};
const Segment SGI367077[] = {{&S9489[33], 13}, {NULL}};
const Segment SGI367078[] = {{&S9490[69], 6}, {NULL}};
const Segment SGI367079[] = {{&S9490[8], 3}, {NULL}};
const Segment SGI367080[] = {{&S9490[13], 9}, {NULL}};
const Segment SGI367081[] = {{&S9490[33], 15}, {NULL}};
const Segment SGI367082[] = {{&S9490[49], 6}, {NULL}};
const Segment SGI367083[] = {{&S9490[33], 13}, {NULL}};
const Segment SGI367084[] = {{&S9491[69], 6}, {NULL}};
const Segment SGI367085[] = {{&S9491[8], 3}, {NULL}};
const Segment SGI367086[] = {{&S9491[13], 9}, {NULL}};
const Segment SGI367087[] = {{&S9491[33], 15}, {NULL}};
const Segment SGI367088[] = {{&S9491[49], 6}, {NULL}};
const Segment SGI367089[] = {{&S9491[33], 13}, {NULL}};
const Segment SGI367090[] = {{&S9492[69], 6}, {NULL}};
const Segment SGI367091[] = {{&S9492[8], 3}, {NULL}};
const Segment SGI367092[] = {{&S9492[13], 9}, {NULL}};
const Segment SGI367093[] = {{&S9492[33], 15}, {NULL}};
const Segment SGI367094[] = {{&S9492[49], 6}, {NULL}};
const Segment SGI367095[] = {{&S9492[33], 13}, {NULL}};
const Segment SGI367096[] = {{&S9493[69], 6}, {NULL}};
const Segment SGI367097[] = {{&S9493[8], 3}, {NULL}};
const Segment SGI367098[] = {{&S9493[13], 9}, {NULL}};
const Segment SGI367099[] = {{&S9493[33], 15}, {NULL}};
const Segment SGI367100[] = {{&S9493[49], 6}, {NULL}};
const Segment SGI367101[] = {{&S9493[33], 13}, {NULL}};
const Segment SGI367102[] = {{&S9494[69], 6}, {NULL}};
const Segment SGI367103[] = {{&S9494[8], 3}, {NULL}};
const Segment SGI367104[] = {{&S9494[13], 9}, {NULL}};
const Segment SGI367105[] = {{&S9494[33], 15}, {NULL}};
const Segment SGI367106[] = {{&S9494[49], 6}, {NULL}};
const Segment SGI367107[] = {{&S9494[33], 13}, {NULL}};
const Segment SGI367108[] = {{&S9495[69], 6}, {NULL}};
const Segment SGI367109[] = {{&S9495[8], 3}, {NULL}};
const Segment SGI367110[] = {{&S9495[13], 9}, {NULL}};
const Segment SGI367111[] = {{&S9495[33], 15}, {NULL}};
const Segment SGI367112[] = {{&S9495[49], 6}, {NULL}};
const Segment SGI367113[] = {{&S9495[33], 13}, {NULL}};
const Segment SGI367114[] = {{&S9496[69], 6}, {NULL}};
const Segment SGI367115[] = {{&S9496[8], 3}, {NULL}};
const Segment SGI367116[] = {{&S9496[13], 9}, {NULL}};
const Segment SGI367117[] = {{&S9496[33], 15}, {NULL}};
const Segment SGI367118[] = {{&S9496[49], 6}, {NULL}};
const Segment SGI367119[] = {{&S9496[33], 13}, {NULL}};
const Segment SGI367120[] = {{&S9497[69], 6}, {NULL}};
const Segment SGI367121[] = {{&S9497[8], 3}, {NULL}};
const Segment SGI367122[] = {{&S9497[13], 9}, {NULL}};
const Segment SGI367123[] = {{&S9497[33], 15}, {NULL}};
const Segment SGI367124[] = {{&S9497[49], 6}, {NULL}};
const Segment SGI367125[] = {{&S9497[33], 13}, {NULL}};
const Segment SGI367126[] = {{&S9498[66], 6}, {NULL}};
const Segment SGI367127[] = {{&S9498[8], 3}, {NULL}};
const Segment SGI367128[] = {{&S9498[13], 9}, {NULL}};
const Segment SGI367129[] = {{&S9498[33], 15}, {NULL}};
const Segment SGI367130[] = {{&S9498[49], 3}, {NULL}};
const Segment SGI367131[] = {{&S9498[33], 13}, {NULL}};
const Segment SGI367132[] = {{&S9499[69], 6}, {NULL}};
const Segment SGI367133[] = {{&S9499[8], 3}, {NULL}};
const Segment SGI367134[] = {{&S9499[13], 9}, {NULL}};
const Segment SGI367135[] = {{&S9499[33], 15}, {NULL}};
const Segment SGI367136[] = {{&S9499[49], 6}, {NULL}};
const Segment SGI367137[] = {{&S9499[33], 13}, {NULL}};
const Segment SGI367138[] = {{&S9500[69], 6}, {NULL}};
const Segment SGI367139[] = {{&S9500[8], 3}, {NULL}};
const Segment SGI367140[] = {{&S9500[13], 9}, {NULL}};
const Segment SGI367141[] = {{&S9500[33], 15}, {NULL}};
const Segment SGI367142[] = {{&S9500[49], 6}, {NULL}};
const Segment SGI367143[] = {{&S9500[33], 13}, {NULL}};
const Segment SGI367144[] = {{&S9501[69], 6}, {NULL}};
const Segment SGI367145[] = {{&S9501[8], 3}, {NULL}};
const Segment SGI367146[] = {{&S9501[13], 9}, {NULL}};
const Segment SGI367147[] = {{&S9501[33], 15}, {NULL}};
const Segment SGI367148[] = {{&S9501[49], 6}, {NULL}};
const Segment SGI367149[] = {{&S9501[33], 13}, {NULL}};
const Segment SGI367150[] = {{&S9502[69], 6}, {NULL}};
const Segment SGI367151[] = {{&S9502[8], 3}, {NULL}};
const Segment SGI367152[] = {{&S9502[13], 9}, {NULL}};
const Segment SGI367153[] = {{&S9502[33], 15}, {NULL}};
const Segment SGI367154[] = {{&S9502[49], 6}, {NULL}};
const Segment SGI367155[] = {{&S9502[33], 13}, {NULL}};
const Segment SGI367156[] = {{&S9503[66], 6}, {NULL}};
const Segment SGI367157[] = {{&S9503[8], 3}, {NULL}};
const Segment SGI367158[] = {{&S9503[13], 9}, {NULL}};
const Segment SGI367159[] = {{&S9503[33], 15}, {NULL}};
const Segment SGI367160[] = {{&S9503[49], 3}, {NULL}};
const Segment SGI367161[] = {{&S9503[33], 13}, {NULL}};
const Segment SGI367162[] = {{&S9504[69], 6}, {NULL}};
const Segment SGI367163[] = {{&S9504[8], 3}, {NULL}};
const Segment SGI367164[] = {{&S9504[13], 9}, {NULL}};
const Segment SGI367165[] = {{&S9504[33], 15}, {NULL}};
const Segment SGI367166[] = {{&S9504[49], 6}, {NULL}};
const Segment SGI367167[] = {{&S9504[33], 13}, {NULL}};
const Segment SGI367168[] = {{&S9505[69], 6}, {NULL}};
const Segment SGI367169[] = {{&S9505[8], 3}, {NULL}};
const Segment SGI367170[] = {{&S9505[13], 9}, {NULL}};
const Segment SGI367171[] = {{&S9505[33], 15}, {NULL}};
const Segment SGI367172[] = {{&S9505[49], 6}, {NULL}};
const Segment SGI367173[] = {{&S9505[33], 13}, {NULL}};
const Segment SGI367174[] = {{&S9506[69], 6}, {NULL}};
const Segment SGI367175[] = {{&S9506[8], 3}, {NULL}};
const Segment SGI367176[] = {{&S9506[13], 9}, {NULL}};
const Segment SGI367177[] = {{&S9506[33], 15}, {NULL}};
const Segment SGI367178[] = {{&S9506[49], 6}, {NULL}};
const Segment SGI367179[] = {{&S9506[33], 13}, {NULL}};
const Segment SGI367180[] = {{&S9507[66], 6}, {NULL}};
const Segment SGI367181[] = {{&S9507[8], 3}, {NULL}};
const Segment SGI367182[] = {{&S9507[13], 9}, {NULL}};
const Segment SGI367183[] = {{&S9507[33], 15}, {NULL}};
const Segment SGI367184[] = {{&S9507[49], 3}, {NULL}};
const Segment SGI367185[] = {{&S9507[33], 13}, {NULL}};
const Segment SGI367186[] = {{&S9508[69], 6}, {NULL}};
const Segment SGI367187[] = {{&S9508[8], 3}, {NULL}};
const Segment SGI367188[] = {{&S9508[13], 9}, {NULL}};
const Segment SGI367189[] = {{&S9508[33], 15}, {NULL}};
const Segment SGI367190[] = {{&S9508[49], 6}, {NULL}};
const Segment SGI367191[] = {{&S9508[33], 13}, {NULL}};
const Segment SGI367192[] = {{&S9509[119], 6}, {NULL}};
const Segment SGI367193[] = {{&S9509[8], 3}, {NULL}};
const Segment SGI367194[] = {{&S9509[13], 9}, {NULL}};
const Segment SGI367195[] = {{&S9509[33], 15}, {NULL}};
const Segment SGI367196[] = {{&S9509[49], 6}, {NULL}};
const Segment SGI367197[] = {{&S9509[33], 13}, {NULL}};
const Segment SGI367198[] = {{&S9510[69], 6}, {NULL}};
const Segment SGI367199[] = {{&S9510[8], 3}, {NULL}};
const Segment SGI367200[] = {{&S9510[13], 9}, {NULL}};
const Segment SGI367201[] = {{&S9510[33], 15}, {NULL}};
const Segment SGI367202[] = {{&S9510[49], 6}, {NULL}};
const Segment SGI367203[] = {{&S9510[33], 13}, {NULL}};
const Segment SGI367204[] = {{&S9511[69], 6}, {NULL}};
const Segment SGI367205[] = {{&S9511[8], 3}, {NULL}};
const Segment SGI367206[] = {{&S9511[13], 9}, {NULL}};
const Segment SGI367207[] = {{&S9511[33], 15}, {NULL}};
const Segment SGI367208[] = {{&S9511[49], 6}, {NULL}};
const Segment SGI367209[] = {{&S9511[33], 13}, {NULL}};
const Segment SGI367210[] = {{&S9512[69], 6}, {NULL}};
const Segment SGI367211[] = {{&S9512[8], 3}, {NULL}};
const Segment SGI367212[] = {{&S9512[13], 9}, {NULL}};
const Segment SGI367213[] = {{&S9512[33], 15}, {NULL}};
const Segment SGI367214[] = {{&S9512[49], 6}, {NULL}};
const Segment SGI367215[] = {{&S9512[33], 13}, {NULL}};
const Segment SGI367216[] = {{&S9513[69], 6}, {NULL}};
const Segment SGI367217[] = {{&S9513[8], 3}, {NULL}};
const Segment SGI367218[] = {{&S9513[13], 9}, {NULL}};
const Segment SGI367219[] = {{&S9513[33], 15}, {NULL}};
const Segment SGI367220[] = {{&S9513[49], 6}, {NULL}};
const Segment SGI367221[] = {{&S9513[33], 13}, {NULL}};
const Segment SGI367222[] = {{&S9514[69], 6}, {NULL}};
const Segment SGI367223[] = {{&S9514[8], 3}, {NULL}};
const Segment SGI367224[] = {{&S9514[13], 9}, {NULL}};
const Segment SGI367225[] = {{&S9514[33], 15}, {NULL}};
const Segment SGI367226[] = {{&S9514[49], 6}, {NULL}};
const Segment SGI367227[] = {{&S9514[33], 13}, {NULL}};
const Segment SGI367228[] = {{&S9515[69], 6}, {NULL}};
const Segment SGI367229[] = {{&S9515[8], 3}, {NULL}};
const Segment SGI367230[] = {{&S9515[13], 9}, {NULL}};
const Segment SGI367231[] = {{&S9515[33], 15}, {NULL}};
const Segment SGI367232[] = {{&S9515[49], 6}, {NULL}};
const Segment SGI367233[] = {{&S9515[33], 13}, {NULL}};
const Segment SGI367234[] = {{&S9516[66], 6}, {NULL}};
const Segment SGI367235[] = {{&S9516[8], 3}, {NULL}};
const Segment SGI367236[] = {{&S9516[13], 9}, {NULL}};
const Segment SGI367237[] = {{&S9516[33], 15}, {NULL}};
const Segment SGI367238[] = {{&S9516[49], 3}, {NULL}};
const Segment SGI367239[] = {{&S9516[33], 13}, {NULL}};
const Segment SGI367240[] = {{&S9517[66], 6}, {NULL}};
const Segment SGI367241[] = {{&S9517[8], 3}, {NULL}};
const Segment SGI367242[] = {{&S9517[13], 9}, {NULL}};
const Segment SGI367243[] = {{&S9517[33], 15}, {NULL}};
const Segment SGI367244[] = {{&S9517[49], 3}, {NULL}};
const Segment SGI367245[] = {{&S9517[33], 13}, {NULL}};
const Segment SGI367246[] = {{&S9518[66], 6}, {NULL}};
const Segment SGI367247[] = {{&S9518[8], 3}, {NULL}};
const Segment SGI367248[] = {{&S9518[13], 9}, {NULL}};
const Segment SGI367249[] = {{&S9518[33], 15}, {NULL}};
const Segment SGI367250[] = {{&S9518[49], 3}, {NULL}};
const Segment SGI367251[] = {{&S9518[33], 13}, {NULL}};
const Segment SGI367252[] = {{&S9519[111], 6}, {NULL}};
const Segment SGI367253[] = {{&S9519[8], 3}, {NULL}};
const Segment SGI367254[] = {{&S9519[13], 9}, {NULL}};
const Segment SGI367255[] = {{&S9610[105], 6}, {NULL}};
const Segment SGI367256[] = {{&S9610[8], 3}, {NULL}};
const Segment SGI367257[] = {{&S9610[13], 9}, {NULL}};
const Segment SGI367258[] = {{&S9610[94], 3}, {NULL}};
const Segment SGI367259[] = {{&S9610[30], 13}, {NULL}};
const Segment SGI367260[] = {{&S9611[59], 6}, {NULL}};
const Segment SGI367261[] = {{&S9611[8], 3}, {NULL}};
const Segment SGI367262[] = {{&S9611[13], 9}, {NULL}};
const Segment SGI367263[] = {{&S9611[94], 3}, {NULL}};
const Segment SGI367264[] = {{&S9611[30], 13}, {NULL}};
const Segment SGI367265[] = {{&S9612[59], 6}, {NULL}};
const Segment SGI367266[] = {{&S9612[8], 3}, {NULL}};
const Segment SGI367267[] = {{&S9612[13], 9}, {NULL}};
const Segment SGI367268[] = {{&S9612[94], 3}, {NULL}};
const Segment SGI367269[] = {{&S9612[30], 13}, {NULL}};
const Segment SGI367270[] = {{&S9613[111], 7}, {NULL}};
const Segment SGI367271[] = {{&S9613[8], 3}, {NULL}};
const Segment SGI367272[] = {{&S9613[13], 9}, {NULL}};
const Segment SGI367273[] = {{&S9613[30], 13}, {NULL}};
const Segment SGI367274[] = {{&S9614[59], 6}, {NULL}};
const Segment SGI367275[] = {{&S9614[8], 3}, {NULL}};
const Segment SGI367276[] = {{&S9614[13], 9}, {NULL}};
const Segment SGI367277[] = {{&S9614[96], 3}, {NULL}};
const Segment SGI367278[] = {{&S9614[30], 13}, {NULL}};
const Segment SGI367279[] = {{&S9615[96], 7}, {NULL}};
const Segment SGI367280[] = {{&S9615[8], 3}, {NULL}};
const Segment SGI367281[] = {{&S9615[13], 9}, {NULL}};
const Segment SGI367282[] = {{&S9615[30], 13}, {NULL}};
const Segment SGI367283[] = {{&S9616[59], 6}, {NULL}};
const Segment SGI367284[] = {{&S9616[8], 3}, {NULL}};
const Segment SGI367285[] = {{&S9616[13], 9}, {NULL}};
const Segment SGI367286[] = {{&S9616[97], 3}, {NULL}};
const Segment SGI367287[] = {{&S9616[30], 13}, {NULL}};
const Segment SGI367288[] = {{&S9617[59], 6}, {NULL}};
const Segment SGI367289[] = {{&S9617[8], 3}, {NULL}};
const Segment SGI367290[] = {{&S9617[13], 9}, {NULL}};
const Segment SGI367291[] = {{&S9617[96], 3}, {NULL}};
const Segment SGI367292[] = {{&S9617[30], 13}, {NULL}};
const Segment SGI367293[] = {{&S9618[110], 7}, {NULL}};
const Segment SGI367294[] = {{&S9618[8], 3}, {NULL}};
const Segment SGI367295[] = {{&S9618[13], 9}, {NULL}};
const Segment SGI367296[] = {{&S9618[97], 3}, {NULL}};
const Segment SGI367297[] = {{&S9619[59], 6}, {NULL}};
const Segment SGI367298[] = {{&S9619[8], 3}, {NULL}};
const Segment SGI367299[] = {{&S9619[13], 9}, {NULL}};
const Segment SGI367300[] = {{&S9619[97], 3}, {NULL}};
const Segment SGI367301[] = {{&S9619[30], 13}, {NULL}};
const Segment SGI367302[] = {{&S9620[96], 7}, {NULL}};
const Segment SGI367303[] = {{&S9620[8], 3}, {NULL}};
const Segment SGI367304[] = {{&S9620[13], 9}, {NULL}};
const Segment SGI367305[] = {{&S9620[30], 13}, {NULL}};
const Segment SGI367306[] = {{&S9621[59], 6}, {NULL}};
const Segment SGI367307[] = {{&S9621[8], 3}, {NULL}};
const Segment SGI367308[] = {{&S9621[13], 9}, {NULL}};
const Segment SGI367309[] = {{&S9621[97], 3}, {NULL}};
const Segment SGI367310[] = {{&S9621[30], 13}, {NULL}};
const Segment SGI367311[] = {{&S9622[96], 7}, {NULL}};
const Segment SGI367312[] = {{&S9622[8], 3}, {NULL}};
const Segment SGI367313[] = {{&S9622[13], 9}, {NULL}};
const Segment SGI367314[] = {{&S9622[30], 13}, {NULL}};
const Segment SGI367315[] = {{&S9623[59], 6}, {NULL}};
const Segment SGI367316[] = {{&S9623[8], 3}, {NULL}};
const Segment SGI367317[] = {{&S9623[13], 9}, {NULL}};
const Segment SGI367318[] = {{&S9623[96], 3}, {NULL}};
const Segment SGI367319[] = {{&S9623[30], 13}, {NULL}};
const Segment SGI367320[] = {{&S9624[96], 7}, {NULL}};
const Segment SGI367321[] = {{&S9624[8], 3}, {NULL}};
const Segment SGI367322[] = {{&S9624[13], 9}, {NULL}};
const Segment SGI367323[] = {{&S9624[30], 13}, {NULL}};
const Segment SGI367324[] = {{&S10052[69], 6}, {NULL}};
const Segment SGI367325[] = {{&S10052[8], 3}, {NULL}};
const Segment SGI367326[] = {{&S10052[13], 9}, {NULL}};
const Segment SGI367327[] = {{&S10052[33], 15}, {NULL}};
const Segment SGI367328[] = {{&S10052[106], 3}, {NULL}};
const Segment SGI367329[] = {{&S10052[49], 6}, {NULL}};
const Segment SGI367330[] = {{&S10052[33], 13}, {NULL}};
const Segment SGI367331[] = {{&S10053[69], 5}, {NULL}};
const Segment SGI367332[] = {{&S10053[8], 3}, {NULL}};
const Segment SGI367333[] = {{&S10053[13], 9}, {NULL}};
const Segment SGI367334[] = {{&S10053[33], 15}, {NULL}};
const Segment SGI367335[] = {{&S10053[103], 3}, {NULL}};
const Segment SGI367336[] = {{&S10053[49], 6}, {NULL}};
const Segment SGI367337[] = {{&S10053[33], 13}, {NULL}};
const Segment SGI367338[] = {{&S10054[69], 6}, {NULL}};
const Segment SGI367339[] = {{&S10054[8], 3}, {NULL}};
const Segment SGI367340[] = {{&S10054[13], 9}, {NULL}};
const Segment SGI367341[] = {{&S10054[33], 15}, {NULL}};
const Segment SGI367342[] = {{&S10054[106], 3}, {NULL}};
const Segment SGI367343[] = {{&S10054[49], 6}, {NULL}};
const Segment SGI367344[] = {{&S10054[33], 13}, {NULL}};
const Segment SGI367345[] = {{&S10055[69], 5}, {NULL}};
const Segment SGI367346[] = {{&S10055[8], 3}, {NULL}};
const Segment SGI367347[] = {{&S10055[13], 9}, {NULL}};
const Segment SGI367348[] = {{&S10055[33], 15}, {NULL}};
const Segment SGI367349[] = {{&S10055[103], 3}, {NULL}};
const Segment SGI367350[] = {{&S10055[49], 6}, {NULL}};
const Segment SGI367351[] = {{&S10055[33], 13}, {NULL}};
const Segment SGI367352[] = {{&S10056[69], 6}, {NULL}};
const Segment SGI367353[] = {{&S10056[8], 3}, {NULL}};
const Segment SGI367354[] = {{&S10056[13], 9}, {NULL}};
const Segment SGI367355[] = {{&S10056[33], 15}, {NULL}};
const Segment SGI367356[] = {{&S10056[104], 3}, {NULL}};
const Segment SGI367357[] = {{&S10056[49], 6}, {NULL}};
const Segment SGI367358[] = {{&S10056[33], 13}, {NULL}};
const Segment SGI367359[] = {{&S10057[69], 5}, {NULL}};
const Segment SGI367360[] = {{&S10057[8], 3}, {NULL}};
const Segment SGI367361[] = {{&S10057[13], 9}, {NULL}};
const Segment SGI367362[] = {{&S10057[33], 15}, {NULL}};
const Segment SGI367363[] = {{&S10057[103], 3}, {NULL}};
const Segment SGI367364[] = {{&S10057[49], 6}, {NULL}};
const Segment SGI367365[] = {{&S10057[33], 13}, {NULL}};
const Segment SGI367366[] = {{&S10058[69], 6}, {NULL}};
const Segment SGI367367[] = {{&S10058[8], 3}, {NULL}};
const Segment SGI367368[] = {{&S10058[13], 9}, {NULL}};
const Segment SGI367369[] = {{&S10058[33], 15}, {NULL}};
const Segment SGI367370[] = {{&S10058[106], 3}, {NULL}};
const Segment SGI367371[] = {{&S10058[49], 6}, {NULL}};
const Segment SGI367372[] = {{&S10058[33], 13}, {NULL}};
const Segment SGI367373[] = {{&S10059[69], 6}, {NULL}};
const Segment SGI367374[] = {{&S10059[8], 3}, {NULL}};
const Segment SGI367375[] = {{&S10059[13], 9}, {NULL}};
const Segment SGI367376[] = {{&S10059[33], 15}, {NULL}};
const Segment SGI367377[] = {{&S10059[106], 3}, {NULL}};
const Segment SGI367378[] = {{&S10059[49], 6}, {NULL}};
const Segment SGI367379[] = {{&S10059[33], 13}, {NULL}};
const Segment SGI367380[] = {{&S10060[69], 6}, {NULL}};
const Segment SGI367381[] = {{&S10060[8], 3}, {NULL}};
const Segment SGI367382[] = {{&S10060[13], 9}, {NULL}};
const Segment SGI367383[] = {{&S10060[33], 15}, {NULL}};
const Segment SGI367384[] = {{&S10060[106], 3}, {NULL}};
const Segment SGI367385[] = {{&S10060[49], 6}, {NULL}};
const Segment SGI367386[] = {{&S10060[33], 13}, {NULL}};
const Segment SGI367387[] = {{&S10061[69], 5}, {NULL}};
const Segment SGI367388[] = {{&S10061[8], 3}, {NULL}};
const Segment SGI367389[] = {{&S10061[13], 9}, {NULL}};
const Segment SGI367390[] = {{&S10061[33], 15}, {NULL}};
const Segment SGI367391[] = {{&S10061[103], 3}, {NULL}};
const Segment SGI367392[] = {{&S10061[49], 6}, {NULL}};
const Segment SGI367393[] = {{&S10061[33], 13}, {NULL}};
const Segment SGI367394[] = {{&S10062[69], 6}, {NULL}};
const Segment SGI367395[] = {{&S10062[8], 3}, {NULL}};
const Segment SGI367396[] = {{&S10062[13], 9}, {NULL}};
const Segment SGI367397[] = {{&S10062[33], 15}, {NULL}};
const Segment SGI367398[] = {{&S10062[106], 3}, {NULL}};
const Segment SGI367399[] = {{&S10062[49], 6}, {NULL}};
const Segment SGI367400[] = {{&S10062[33], 13}, {NULL}};
const Segment SGI367401[] = {{&S10063[69], 6}, {NULL}};
const Segment SGI367402[] = {{&S10063[8], 3}, {NULL}};
const Segment SGI367403[] = {{&S10063[13], 9}, {NULL}};
const Segment SGI367404[] = {{&S10063[33], 15}, {NULL}};
const Segment SGI367405[] = {{&S10063[106], 3}, {NULL}};
const Segment SGI367406[] = {{&S10063[49], 6}, {NULL}};
const Segment SGI367407[] = {{&S10063[33], 13}, {NULL}};
const Segment SGI367408[] = {{&S10064[69], 5}, {NULL}};
const Segment SGI367409[] = {{&S10064[8], 3}, {NULL}};
const Segment SGI367410[] = {{&S10064[13], 9}, {NULL}};
const Segment SGI367411[] = {{&S10064[33], 15}, {NULL}};
const Segment SGI367412[] = {{&S10064[103], 3}, {NULL}};
const Segment SGI367413[] = {{&S10064[49], 6}, {NULL}};
const Segment SGI367414[] = {{&S10064[33], 13}, {NULL}};
const Segment SGI367415[] = {{&S10065[69], 5}, {NULL}};
const Segment SGI367416[] = {{&S10065[8], 3}, {NULL}};
const Segment SGI367417[] = {{&S10065[13], 9}, {NULL}};
const Segment SGI367418[] = {{&S10065[33], 15}, {NULL}};
const Segment SGI367419[] = {{&S10065[103], 3}, {NULL}};
const Segment SGI367420[] = {{&S10065[49], 6}, {NULL}};
const Segment SGI367421[] = {{&S10065[33], 13}, {NULL}};
const Segment SGI367422[] = {{&S10066[69], 6}, {NULL}};
const Segment SGI367423[] = {{&S10066[8], 3}, {NULL}};
const Segment SGI367424[] = {{&S10066[13], 9}, {NULL}};
const Segment SGI367425[] = {{&S10066[33], 15}, {NULL}};
const Segment SGI367426[] = {{&S10066[106], 3}, {NULL}};
const Segment SGI367427[] = {{&S10066[49], 6}, {NULL}};
const Segment SGI367428[] = {{&S10066[33], 13}, {NULL}};
const Segment SGI367429[] = {{&S10067[69], 6}, {NULL}};
const Segment SGI367430[] = {{&S10067[8], 3}, {NULL}};
const Segment SGI367431[] = {{&S10067[13], 9}, {NULL}};
const Segment SGI367432[] = {{&S10067[33], 15}, {NULL}};
const Segment SGI367433[] = {{&S10067[106], 3}, {NULL}};
const Segment SGI367434[] = {{&S10067[49], 6}, {NULL}};
const Segment SGI367435[] = {{&S10067[33], 13}, {NULL}};
const Segment SGI367436[] = {{&S10068[69], 6}, {NULL}};
const Segment SGI367437[] = {{&S10068[8], 3}, {NULL}};
const Segment SGI367438[] = {{&S10068[13], 9}, {NULL}};
const Segment SGI367439[] = {{&S10068[33], 15}, {NULL}};
const Segment SGI367440[] = {{&S10068[106], 3}, {NULL}};
const Segment SGI367441[] = {{&S10068[49], 6}, {NULL}};
const Segment SGI367442[] = {{&S10068[33], 13}, {NULL}};
const Segment SGI367443[] = {{&S10069[69], 6}, {NULL}};
const Segment SGI367444[] = {{&S10069[8], 3}, {NULL}};
const Segment SGI367445[] = {{&S10069[13], 9}, {NULL}};
const Segment SGI367446[] = {{&S10069[33], 15}, {NULL}};
const Segment SGI367447[] = {{&S10069[106], 3}, {NULL}};
const Segment SGI367448[] = {{&S10069[49], 6}, {NULL}};
const Segment SGI367449[] = {{&S10069[33], 13}, {NULL}};
const Segment SGI367450[] = {{&S10070[69], 6}, {NULL}};
const Segment SGI367451[] = {{&S10070[8], 3}, {NULL}};
const Segment SGI367452[] = {{&S10070[13], 9}, {NULL}};
const Segment SGI367453[] = {{&S10070[33], 15}, {NULL}};
const Segment SGI367454[] = {{&S10070[106], 3}, {NULL}};
const Segment SGI367455[] = {{&S10070[49], 6}, {NULL}};
const Segment SGI367456[] = {{&S10070[33], 13}, {NULL}};
const Segment SGI367457[] = {{&S10071[69], 6}, {NULL}};
const Segment SGI367458[] = {{&S10071[8], 3}, {NULL}};
const Segment SGI367459[] = {{&S10071[13], 9}, {NULL}};
const Segment SGI367460[] = {{&S10071[33], 15}, {NULL}};
const Segment SGI367461[] = {{&S10071[106], 3}, {NULL}};
const Segment SGI367462[] = {{&S10071[49], 6}, {NULL}};
const Segment SGI367463[] = {{&S10071[33], 13}, {NULL}};
const Segment SGI367464[] = {{&S10072[69], 6}, {NULL}};
const Segment SGI367465[] = {{&S10072[8], 3}, {NULL}};
const Segment SGI367466[] = {{&S10072[13], 9}, {NULL}};
const Segment SGI367467[] = {{&S10072[33], 15}, {NULL}};
const Segment SGI367468[] = {{&S10072[106], 3}, {NULL}};
const Segment SGI367469[] = {{&S10072[49], 6}, {NULL}};
const Segment SGI367470[] = {{&S10072[33], 13}, {NULL}};
const Segment SGI367471[] = {{&S10073[69], 6}, {NULL}};
const Segment SGI367472[] = {{&S10073[8], 3}, {NULL}};
const Segment SGI367473[] = {{&S10073[13], 9}, {NULL}};
const Segment SGI367474[] = {{&S10073[33], 15}, {NULL}};
const Segment SGI367475[] = {{&S10073[104], 3}, {NULL}};
const Segment SGI367476[] = {{&S10073[49], 6}, {NULL}};
const Segment SGI367477[] = {{&S10073[33], 13}, {NULL}};
const Segment SGI367478[] = {{&S10074[69], 6}, {NULL}};
const Segment SGI367479[] = {{&S10074[8], 3}, {NULL}};
const Segment SGI367480[] = {{&S10074[13], 9}, {NULL}};
const Segment SGI367481[] = {{&S10074[33], 15}, {NULL}};
const Segment SGI367482[] = {{&S10074[106], 3}, {NULL}};
const Segment SGI367483[] = {{&S10074[49], 6}, {NULL}};
const Segment SGI367484[] = {{&S10074[33], 13}, {NULL}};
const Segment SGI367485[] = {{&S10075[69], 6}, {NULL}};
const Segment SGI367486[] = {{&S10075[8], 3}, {NULL}};
const Segment SGI367487[] = {{&S10075[13], 9}, {NULL}};
const Segment SGI367488[] = {{&S10075[33], 15}, {NULL}};
const Segment SGI367489[] = {{&S10075[106], 3}, {NULL}};
const Segment SGI367490[] = {{&S10075[49], 6}, {NULL}};
const Segment SGI367491[] = {{&S10075[33], 13}, {NULL}};
const Segment SGI367492[] = {{&S10076[69], 6}, {NULL}};
const Segment SGI367493[] = {{&S10076[8], 3}, {NULL}};
const Segment SGI367494[] = {{&S10076[13], 9}, {NULL}};
const Segment SGI367495[] = {{&S10076[33], 15}, {NULL}};
const Segment SGI367496[] = {{&S10076[106], 3}, {NULL}};
const Segment SGI367497[] = {{&S10076[49], 6}, {NULL}};
const Segment SGI367498[] = {{&S10076[33], 13}, {NULL}};
const Segment SGI367499[] = {{&S10077[69], 5}, {NULL}};
const Segment SGI367500[] = {{&S10077[8], 3}, {NULL}};
const Segment SGI367501[] = {{&S10077[13], 9}, {NULL}};
const Segment SGI367502[] = {{&S10077[33], 15}, {NULL}};
const Segment SGI367503[] = {{&S10077[103], 3}, {NULL}};
const Segment SGI367504[] = {{&S10077[49], 6}, {NULL}};
const Segment SGI367505[] = {{&S10077[33], 13}, {NULL}};
const Segment SGI367506[] = {{&S10078[69], 6}, {NULL}};
const Segment SGI367507[] = {{&S10078[8], 3}, {NULL}};
const Segment SGI367508[] = {{&S10078[13], 9}, {NULL}};
const Segment SGI367509[] = {{&S10078[33], 15}, {NULL}};
const Segment SGI367510[] = {{&S10078[106], 3}, {NULL}};
const Segment SGI367511[] = {{&S10078[49], 6}, {NULL}};
const Segment SGI367512[] = {{&S10078[33], 13}, {NULL}};
const Segment SGI367513[] = {{&S10079[69], 6}, {NULL}};
const Segment SGI367514[] = {{&S10079[8], 3}, {NULL}};
const Segment SGI367515[] = {{&S10079[13], 9}, {NULL}};
const Segment SGI367516[] = {{&S10079[33], 15}, {NULL}};
const Segment SGI367517[] = {{&S10079[106], 3}, {NULL}};
const Segment SGI367518[] = {{&S10079[49], 6}, {NULL}};
const Segment SGI367519[] = {{&S10079[33], 13}, {NULL}};
const Segment SGI367520[] = {{&S10080[69], 5}, {NULL}};
const Segment SGI367521[] = {{&S10080[8], 3}, {NULL}};
const Segment SGI367522[] = {{&S10080[13], 9}, {NULL}};
const Segment SGI367523[] = {{&S10080[33], 15}, {NULL}};
const Segment SGI367524[] = {{&S10080[103], 3}, {NULL}};
const Segment SGI367525[] = {{&S10080[49], 6}, {NULL}};
const Segment SGI367526[] = {{&S10080[33], 13}, {NULL}};
const Segment SGI367527[] = {{&S10081[69], 6}, {NULL}};
const Segment SGI367528[] = {{&S10081[8], 3}, {NULL}};
const Segment SGI367529[] = {{&S10081[13], 9}, {NULL}};
const Segment SGI367530[] = {{&S10081[33], 15}, {NULL}};
const Segment SGI367531[] = {{&S10081[106], 3}, {NULL}};
const Segment SGI367532[] = {{&S10081[49], 6}, {NULL}};
const Segment SGI367533[] = {{&S10081[33], 13}, {NULL}};
const Segment SGI367534[] = {{&S10082[69], 6}, {NULL}};
const Segment SGI367535[] = {{&S10082[8], 3}, {NULL}};
const Segment SGI367536[] = {{&S10082[13], 9}, {NULL}};
const Segment SGI367537[] = {{&S10082[33], 15}, {NULL}};
const Segment SGI367538[] = {{&S10082[106], 3}, {NULL}};
const Segment SGI367539[] = {{&S10082[49], 6}, {NULL}};
const Segment SGI367540[] = {{&S10082[33], 13}, {NULL}};
const Segment SGI367541[] = {{&S10083[69], 6}, {NULL}};
const Segment SGI367542[] = {{&S10083[8], 3}, {NULL}};
const Segment SGI367543[] = {{&S10083[13], 9}, {NULL}};
const Segment SGI367544[] = {{&S10083[33], 15}, {NULL}};
const Segment SGI367545[] = {{&S10083[106], 3}, {NULL}};
const Segment SGI367546[] = {{&S10083[49], 6}, {NULL}};
const Segment SGI367547[] = {{&S10083[33], 13}, {NULL}};
const Segment SGI367548[] = {{&S10084[69], 5}, {NULL}};
const Segment SGI367549[] = {{&S10084[8], 3}, {NULL}};
const Segment SGI367550[] = {{&S10084[13], 9}, {NULL}};
const Segment SGI367551[] = {{&S10084[33], 15}, {NULL}};
const Segment SGI367552[] = {{&S10084[103], 3}, {NULL}};
const Segment SGI367553[] = {{&S10084[49], 6}, {NULL}};
const Segment SGI367554[] = {{&S10084[33], 13}, {NULL}};
const Segment SGI367555[] = {{&S10085[69], 6}, {NULL}};
const Segment SGI367556[] = {{&S10085[8], 3}, {NULL}};
const Segment SGI367557[] = {{&S10085[13], 9}, {NULL}};
const Segment SGI367558[] = {{&S10085[33], 15}, {NULL}};
const Segment SGI367559[] = {{&S10085[104], 3}, {NULL}};
const Segment SGI367560[] = {{&S10085[49], 6}, {NULL}};
const Segment SGI367561[] = {{&S10085[33], 13}, {NULL}};
const Segment SGI367562[] = {{&S10086[69], 6}, {NULL}};
const Segment SGI367563[] = {{&S10086[8], 3}, {NULL}};
const Segment SGI367564[] = {{&S10086[13], 9}, {NULL}};
const Segment SGI367565[] = {{&S10086[33], 15}, {NULL}};
const Segment SGI367566[] = {{&S10086[106], 3}, {NULL}};
const Segment SGI367567[] = {{&S10086[49], 6}, {NULL}};
const Segment SGI367568[] = {{&S10086[33], 13}, {NULL}};
const Segment SGI367569[] = {{&S10087[69], 6}, {NULL}};
const Segment SGI367570[] = {{&S10087[8], 3}, {NULL}};
const Segment SGI367571[] = {{&S10087[13], 9}, {NULL}};
const Segment SGI367572[] = {{&S10087[33], 15}, {NULL}};
const Segment SGI367573[] = {{&S10087[106], 3}, {NULL}};
const Segment SGI367574[] = {{&S10087[49], 6}, {NULL}};
const Segment SGI367575[] = {{&S10087[33], 13}, {NULL}};
const Segment SGI367576[] = {{&S10088[69], 6}, {NULL}};
const Segment SGI367577[] = {{&S10088[8], 3}, {NULL}};
const Segment SGI367578[] = {{&S10088[13], 9}, {NULL}};
const Segment SGI367579[] = {{&S10088[33], 15}, {NULL}};
const Segment SGI367580[] = {{&S10088[106], 3}, {NULL}};
const Segment SGI367581[] = {{&S10088[49], 6}, {NULL}};
const Segment SGI367582[] = {{&S10088[33], 13}, {NULL}};
const Segment SGI367583[] = {{&S10089[69], 6}, {NULL}};
const Segment SGI367584[] = {{&S10089[8], 3}, {NULL}};
const Segment SGI367585[] = {{&S10089[13], 9}, {NULL}};
const Segment SGI367586[] = {{&S10089[33], 15}, {NULL}};
const Segment SGI367587[] = {{&S10089[106], 3}, {NULL}};
const Segment SGI367588[] = {{&S10089[49], 6}, {NULL}};
const Segment SGI367589[] = {{&S10089[33], 13}, {NULL}};
const Segment SGI367590[] = {{&S10090[69], 6}, {NULL}};
const Segment SGI367591[] = {{&S10090[8], 3}, {NULL}};
const Segment SGI367592[] = {{&S10090[13], 9}, {NULL}};
const Segment SGI367593[] = {{&S10090[33], 15}, {NULL}};
const Segment SGI367594[] = {{&S10090[106], 3}, {NULL}};
const Segment SGI367595[] = {{&S10090[49], 6}, {NULL}};
const Segment SGI367596[] = {{&S10090[33], 13}, {NULL}};
const Segment SGI367597[] = {{&S10091[69], 5}, {NULL}};
const Segment SGI367598[] = {{&S10091[8], 3}, {NULL}};
const Segment SGI367599[] = {{&S10091[13], 9}, {NULL}};
const Segment SGI367600[] = {{&S10091[33], 15}, {NULL}};
const Segment SGI367601[] = {{&S10091[103], 3}, {NULL}};
const Segment SGI367602[] = {{&S10091[49], 6}, {NULL}};
const Segment SGI367603[] = {{&S10091[33], 13}, {NULL}};
const Segment SGI367604[] = {{&S10092[69], 6}, {NULL}};
const Segment SGI367605[] = {{&S10092[8], 3}, {NULL}};
const Segment SGI367606[] = {{&S10092[13], 9}, {NULL}};
const Segment SGI367607[] = {{&S10092[33], 15}, {NULL}};
const Segment SGI367608[] = {{&S10092[106], 3}, {NULL}};
const Segment SGI367609[] = {{&S10092[49], 6}, {NULL}};
const Segment SGI367610[] = {{&S10092[33], 13}, {NULL}};
const Segment SGI367611[] = {{&S10093[69], 6}, {NULL}};
const Segment SGI367612[] = {{&S10093[8], 3}, {NULL}};
const Segment SGI367613[] = {{&S10093[13], 9}, {NULL}};
const Segment SGI367614[] = {{&S10093[33], 15}, {NULL}};
const Segment SGI367615[] = {{&S10093[106], 3}, {NULL}};
const Segment SGI367616[] = {{&S10093[49], 6}, {NULL}};
const Segment SGI367617[] = {{&S10093[33], 13}, {NULL}};
const Segment SGI367618[] = {{&S10094[66], 6}, {NULL}};
const Segment SGI367619[] = {{&S10094[8], 3}, {NULL}};
const Segment SGI367620[] = {{&S10094[13], 9}, {NULL}};
const Segment SGI367621[] = {{&S10094[33], 15}, {NULL}};
const Segment SGI367622[] = {{&S10094[103], 3}, {NULL}};
const Segment SGI367623[] = {{&S10094[49], 3}, {NULL}};
const Segment SGI367624[] = {{&S10094[33], 13}, {NULL}};
const Segment SGI367625[] = {{&S10095[69], 6}, {NULL}};
const Segment SGI367626[] = {{&S10095[8], 3}, {NULL}};
const Segment SGI367627[] = {{&S10095[13], 9}, {NULL}};
const Segment SGI367628[] = {{&S10095[33], 15}, {NULL}};
const Segment SGI367629[] = {{&S10095[106], 3}, {NULL}};
const Segment SGI367630[] = {{&S10095[49], 6}, {NULL}};
const Segment SGI367631[] = {{&S10095[33], 13}, {NULL}};
const Segment SGI367632[] = {{&S10096[69], 6}, {NULL}};
const Segment SGI367633[] = {{&S10096[8], 3}, {NULL}};
const Segment SGI367634[] = {{&S10096[13], 9}, {NULL}};
const Segment SGI367635[] = {{&S10096[33], 15}, {NULL}};
const Segment SGI367636[] = {{&S10096[104], 3}, {NULL}};
const Segment SGI367637[] = {{&S10096[49], 6}, {NULL}};
const Segment SGI367638[] = {{&S10096[33], 13}, {NULL}};
const Segment SGI367639[] = {{&S10097[69], 5}, {NULL}};
const Segment SGI367640[] = {{&S10097[8], 3}, {NULL}};
const Segment SGI367641[] = {{&S10097[13], 9}, {NULL}};
const Segment SGI367642[] = {{&S10097[33], 15}, {NULL}};
const Segment SGI367643[] = {{&S10097[103], 3}, {NULL}};
const Segment SGI367644[] = {{&S10097[49], 6}, {NULL}};
const Segment SGI367645[] = {{&S10097[33], 13}, {NULL}};
const Segment SGI367646[] = {{&S10098[69], 6}, {NULL}};
const Segment SGI367647[] = {{&S10098[8], 3}, {NULL}};
const Segment SGI367648[] = {{&S10098[13], 9}, {NULL}};
const Segment SGI367649[] = {{&S10098[33], 15}, {NULL}};
const Segment SGI367650[] = {{&S10098[106], 3}, {NULL}};
const Segment SGI367651[] = {{&S10098[49], 6}, {NULL}};
const Segment SGI367652[] = {{&S10098[33], 13}, {NULL}};
const Segment SGI367653[] = {{&S10099[69], 6}, {NULL}};
const Segment SGI367654[] = {{&S10099[8], 3}, {NULL}};
const Segment SGI367655[] = {{&S10099[13], 9}, {NULL}};
const Segment SGI367656[] = {{&S10099[33], 15}, {NULL}};
const Segment SGI367657[] = {{&S10099[106], 3}, {NULL}};
const Segment SGI367658[] = {{&S10099[49], 6}, {NULL}};
const Segment SGI367659[] = {{&S10099[33], 13}, {NULL}};
const Segment SGI367660[] = {{&S10100[69], 6}, {NULL}};
const Segment SGI367661[] = {{&S10100[8], 3}, {NULL}};
const Segment SGI367662[] = {{&S10100[13], 9}, {NULL}};
const Segment SGI367663[] = {{&S10100[33], 15}, {NULL}};
const Segment SGI367664[] = {{&S10100[106], 3}, {NULL}};
const Segment SGI367665[] = {{&S10100[49], 6}, {NULL}};
const Segment SGI367666[] = {{&S10100[33], 13}, {NULL}};
const Segment SGI367667[] = {{&S10101[69], 5}, {NULL}};
const Segment SGI367668[] = {{&S10101[8], 3}, {NULL}};
const Segment SGI367669[] = {{&S10101[13], 9}, {NULL}};
const Segment SGI367670[] = {{&S10101[33], 15}, {NULL}};
const Segment SGI367671[] = {{&S10101[103], 3}, {NULL}};
const Segment SGI367672[] = {{&S10101[49], 6}, {NULL}};
const Segment SGI367673[] = {{&S10101[33], 13}, {NULL}};
const Segment SGI367674[] = {{&S10102[69], 6}, {NULL}};
const Segment SGI367675[] = {{&S10102[8], 3}, {NULL}};
const Segment SGI367676[] = {{&S10102[13], 9}, {NULL}};
const Segment SGI367677[] = {{&S10102[33], 15}, {NULL}};
const Segment SGI367678[] = {{&S10102[49], 6}, {NULL}};
const Segment SGI367679[] = {{&S10102[33], 13}, {NULL}};
const Segment SGI367680[] = {{&S10103[69], 6}, {NULL}};
const Segment SGI367681[] = {{&S10103[8], 3}, {NULL}};
const Segment SGI367682[] = {{&S10103[13], 9}, {NULL}};
const Segment SGI367683[] = {{&S10103[33], 15}, {NULL}};
const Segment SGI367684[] = {{&S10103[106], 3}, {NULL}};
const Segment SGI367685[] = {{&S10103[49], 6}, {NULL}};
const Segment SGI367686[] = {{&S10103[33], 13}, {NULL}};
const Segment SGI367687[] = {{&S10104[69], 5}, {NULL}};
const Segment SGI367688[] = {{&S10104[8], 3}, {NULL}};
const Segment SGI367689[] = {{&S10104[13], 9}, {NULL}};
const Segment SGI367690[] = {{&S10104[33], 15}, {NULL}};
const Segment SGI367691[] = {{&S10104[103], 3}, {NULL}};
const Segment SGI367692[] = {{&S10104[49], 6}, {NULL}};
const Segment SGI367693[] = {{&S10104[33], 13}, {NULL}};
const Segment SGI367694[] = {{&S10105[69], 6}, {NULL}};
const Segment SGI367695[] = {{&S10105[8], 3}, {NULL}};
const Segment SGI367696[] = {{&S10105[13], 9}, {NULL}};
const Segment SGI367697[] = {{&S10105[33], 15}, {NULL}};
const Segment SGI367698[] = {{&S10105[106], 3}, {NULL}};
const Segment SGI367699[] = {{&S10105[49], 6}, {NULL}};
const Segment SGI367700[] = {{&S10105[33], 13}, {NULL}};
const Segment SGI367701[] = {{&S10106[69], 6}, {NULL}};
const Segment SGI367702[] = {{&S10106[8], 3}, {NULL}};
const Segment SGI367703[] = {{&S10106[13], 9}, {NULL}};
const Segment SGI367704[] = {{&S10106[33], 15}, {NULL}};
const Segment SGI367705[] = {{&S10106[106], 3}, {NULL}};
const Segment SGI367706[] = {{&S10106[49], 6}, {NULL}};
const Segment SGI367707[] = {{&S10106[33], 13}, {NULL}};
const Segment SGI367708[] = {{&S10107[69], 6}, {NULL}};
const Segment SGI367709[] = {{&S10107[8], 3}, {NULL}};
const Segment SGI367710[] = {{&S10107[13], 9}, {NULL}};
const Segment SGI367711[] = {{&S10107[33], 15}, {NULL}};
const Segment SGI367712[] = {{&S10107[106], 3}, {NULL}};
const Segment SGI367713[] = {{&S10107[49], 6}, {NULL}};
const Segment SGI367714[] = {{&S10107[33], 13}, {NULL}};
const Segment SGI367715[] = {{&S10108[69], 6}, {NULL}};
const Segment SGI367716[] = {{&S10108[8], 3}, {NULL}};
const Segment SGI367717[] = {{&S10108[13], 9}, {NULL}};
const Segment SGI367718[] = {{&S10108[33], 15}, {NULL}};
const Segment SGI367719[] = {{&S10108[106], 3}, {NULL}};
const Segment SGI367720[] = {{&S10108[49], 6}, {NULL}};
const Segment SGI367721[] = {{&S10108[33], 13}, {NULL}};
const Segment SGI367722[] = {{&S10109[69], 6}, {NULL}};
const Segment SGI367723[] = {{&S10109[8], 3}, {NULL}};
const Segment SGI367724[] = {{&S10109[13], 9}, {NULL}};
const Segment SGI367725[] = {{&S10109[33], 15}, {NULL}};
const Segment SGI367726[] = {{&S10109[104], 3}, {NULL}};
const Segment SGI367727[] = {{&S10109[49], 6}, {NULL}};
const Segment SGI367728[] = {{&S10109[33], 13}, {NULL}};
const Segment SGI367729[] = {{&S10110[69], 6}, {NULL}};
const Segment SGI367730[] = {{&S10110[8], 3}, {NULL}};
const Segment SGI367731[] = {{&S10110[13], 9}, {NULL}};
const Segment SGI367732[] = {{&S10110[33], 15}, {NULL}};
const Segment SGI367733[] = {{&S10110[106], 3}, {NULL}};
const Segment SGI367734[] = {{&S10110[49], 6}, {NULL}};
const Segment SGI367735[] = {{&S10110[33], 13}, {NULL}};
const Segment SGI367736[] = {{&S10111[69], 6}, {NULL}};
const Segment SGI367737[] = {{&S10111[8], 3}, {NULL}};
const Segment SGI367738[] = {{&S10111[13], 9}, {NULL}};
const Segment SGI367739[] = {{&S10111[33], 15}, {NULL}};
const Segment SGI367740[] = {{&S10111[106], 3}, {NULL}};
const Segment SGI367741[] = {{&S10111[49], 6}, {NULL}};
const Segment SGI367742[] = {{&S10111[33], 13}, {NULL}};
const Segment SGI367743[] = {{&S10112[69], 6}, {NULL}};
const Segment SGI367744[] = {{&S10112[8], 3}, {NULL}};
const Segment SGI367745[] = {{&S10112[13], 9}, {NULL}};
const Segment SGI367746[] = {{&S10112[33], 15}, {NULL}};
const Segment SGI367747[] = {{&S10112[106], 3}, {NULL}};
const Segment SGI367748[] = {{&S10112[49], 6}, {NULL}};
const Segment SGI367749[] = {{&S10112[33], 13}, {NULL}};
const Segment SGI367750[] = {{&S10113[69], 6}, {NULL}};
const Segment SGI367751[] = {{&S10113[8], 3}, {NULL}};
const Segment SGI367752[] = {{&S10113[13], 9}, {NULL}};
const Segment SGI367753[] = {{&S10113[33], 15}, {NULL}};
const Segment SGI367754[] = {{&S10113[106], 3}, {NULL}};
const Segment SGI367755[] = {{&S10113[49], 6}, {NULL}};
const Segment SGI367756[] = {{&S10113[33], 13}, {NULL}};
const Segment SGI367757[] = {{&S10114[69], 5}, {NULL}};
const Segment SGI367758[] = {{&S10114[8], 3}, {NULL}};
const Segment SGI367759[] = {{&S10114[13], 9}, {NULL}};
const Segment SGI367760[] = {{&S10114[33], 15}, {NULL}};
const Segment SGI367761[] = {{&S10114[103], 3}, {NULL}};
const Segment SGI367762[] = {{&S10114[49], 6}, {NULL}};
const Segment SGI367763[] = {{&S10114[33], 13}, {NULL}};
const Segment SGI367764[] = {{&S10115[69], 5}, {NULL}};
const Segment SGI367765[] = {{&S10115[8], 3}, {NULL}};
const Segment SGI367766[] = {{&S10115[13], 9}, {NULL}};
const Segment SGI367767[] = {{&S10115[33], 15}, {NULL}};
const Segment SGI367768[] = {{&S10115[103], 3}, {NULL}};
const Segment SGI367769[] = {{&S10115[49], 6}, {NULL}};
const Segment SGI367770[] = {{&S10115[33], 13}, {NULL}};
const Segment SGI367771[] = {{&S10116[69], 6}, {NULL}};
const Segment SGI367772[] = {{&S10116[8], 3}, {NULL}};
const Segment SGI367773[] = {{&S10116[13], 9}, {NULL}};
const Segment SGI367774[] = {{&S10116[33], 15}, {NULL}};
const Segment SGI367775[] = {{&S10116[104], 3}, {NULL}};
const Segment SGI367776[] = {{&S10116[49], 6}, {NULL}};
const Segment SGI367777[] = {{&S10116[33], 13}, {NULL}};
const Segment SGI367778[] = {{&S10117[69], 5}, {NULL}};
const Segment SGI367779[] = {{&S10117[8], 3}, {NULL}};
const Segment SGI367780[] = {{&S10117[13], 9}, {NULL}};
const Segment SGI367781[] = {{&S10117[33], 15}, {NULL}};
const Segment SGI367782[] = {{&S10117[103], 3}, {NULL}};
const Segment SGI367783[] = {{&S10117[49], 6}, {NULL}};
const Segment SGI367784[] = {{&S10117[33], 13}, {NULL}};
const Segment SGI367785[] = {{&S10118[69], 6}, {NULL}};
const Segment SGI367786[] = {{&S10118[8], 3}, {NULL}};
const Segment SGI367787[] = {{&S10118[13], 9}, {NULL}};
const Segment SGI367788[] = {{&S10118[33], 15}, {NULL}};
const Segment SGI367789[] = {{&S10118[106], 3}, {NULL}};
const Segment SGI367790[] = {{&S10118[49], 6}, {NULL}};
const Segment SGI367791[] = {{&S10118[33], 13}, {NULL}};
const Segment SGI367792[] = {{&S10119[69], 6}, {NULL}};
const Segment SGI367793[] = {{&S10119[8], 3}, {NULL}};
const Segment SGI367794[] = {{&S10119[13], 9}, {NULL}};
const Segment SGI367795[] = {{&S10119[33], 15}, {NULL}};
const Segment SGI367796[] = {{&S10119[106], 3}, {NULL}};
const Segment SGI367797[] = {{&S10119[49], 6}, {NULL}};
const Segment SGI367798[] = {{&S10119[33], 13}, {NULL}};
const Segment SGI367799[] = {{&S10120[69], 6}, {NULL}};
const Segment SGI367800[] = {{&S10120[8], 3}, {NULL}};
const Segment SGI367801[] = {{&S10120[13], 9}, {NULL}};
const Segment SGI367802[] = {{&S10120[33], 15}, {NULL}};
const Segment SGI367803[] = {{&S10120[106], 3}, {NULL}};
const Segment SGI367804[] = {{&S10120[49], 6}, {NULL}};
const Segment SGI367805[] = {{&S10120[33], 13}, {NULL}};
const Segment SGI367806[] = {{&S10121[69], 6}, {NULL}};
const Segment SGI367807[] = {{&S10121[8], 3}, {NULL}};
const Segment SGI367808[] = {{&S10121[13], 9}, {NULL}};
const Segment SGI367809[] = {{&S10121[33], 15}, {NULL}};
const Segment SGI367810[] = {{&S10121[106], 3}, {NULL}};
const Segment SGI367811[] = {{&S10121[49], 6}, {NULL}};
const Segment SGI367812[] = {{&S10121[33], 13}, {NULL}};
const Segment SGI367813[] = {{&S10122[69], 5}, {NULL}};
const Segment SGI367814[] = {{&S10122[8], 3}, {NULL}};
const Segment SGI367815[] = {{&S10122[13], 9}, {NULL}};
const Segment SGI367816[] = {{&S10122[33], 15}, {NULL}};
const Segment SGI367817[] = {{&S10122[103], 3}, {NULL}};
const Segment SGI367818[] = {{&S10122[49], 6}, {NULL}};
const Segment SGI367819[] = {{&S10122[33], 13}, {NULL}};
const Segment SGI367820[] = {{&S10123[69], 6}, {NULL}};
const Segment SGI367821[] = {{&S10123[8], 3}, {NULL}};
const Segment SGI367822[] = {{&S10123[13], 9}, {NULL}};
const Segment SGI367823[] = {{&S10123[33], 15}, {NULL}};
const Segment SGI367824[] = {{&S10123[106], 3}, {NULL}};
const Segment SGI367825[] = {{&S10123[49], 6}, {NULL}};
const Segment SGI367826[] = {{&S10123[33], 13}, {NULL}};
const Segment SGI367827[] = {{&S10124[69], 6}, {NULL}};
const Segment SGI367828[] = {{&S10124[8], 3}, {NULL}};
const Segment SGI367829[] = {{&S10124[13], 9}, {NULL}};
const Segment SGI367830[] = {{&S10124[33], 15}, {NULL}};
const Segment SGI367831[] = {{&S10124[106], 3}, {NULL}};
const Segment SGI367832[] = {{&S10124[49], 6}, {NULL}};
const Segment SGI367833[] = {{&S10124[33], 13}, {NULL}};
const Segment SGI367834[] = {{&S10125[69], 6}, {NULL}};
const Segment SGI367835[] = {{&S10125[8], 3}, {NULL}};
const Segment SGI367836[] = {{&S10125[13], 9}, {NULL}};
const Segment SGI367837[] = {{&S10125[33], 15}, {NULL}};
const Segment SGI367838[] = {{&S10125[106], 3}, {NULL}};
const Segment SGI367839[] = {{&S10125[49], 6}, {NULL}};
const Segment SGI367840[] = {{&S10125[33], 13}, {NULL}};
const Segment SGI367841[] = {{&S10126[69], 6}, {NULL}};
const Segment SGI367842[] = {{&S10126[8], 3}, {NULL}};
const Segment SGI367843[] = {{&S10126[13], 9}, {NULL}};
const Segment SGI367844[] = {{&S10126[33], 15}, {NULL}};
const Segment SGI367845[] = {{&S10126[106], 3}, {NULL}};
const Segment SGI367846[] = {{&S10126[49], 6}, {NULL}};
const Segment SGI367847[] = {{&S10126[33], 13}, {NULL}};
const Segment SGI367848[] = {{&S10127[116], 5}, {NULL}};
const Segment SGI367849[] = {{&S10127[8], 3}, {NULL}};
const Segment SGI367850[] = {{&S10127[13], 9}, {NULL}};
const Segment SGI367851[] = {{&S10127[33], 15}, {NULL}};
const Segment SGI367852[] = {{&S10127[103], 3}, {NULL}};
const Segment SGI367853[] = {{&S10127[49], 6}, {NULL}};
const Segment SGI367854[] = {{&S10127[33], 13}, {NULL}};
const Segment SGI367855[] = {{&S10128[69], 6}, {NULL}};
const Segment SGI367856[] = {{&S10128[8], 3}, {NULL}};
const Segment SGI367857[] = {{&S10128[13], 9}, {NULL}};
const Segment SGI367858[] = {{&S10128[33], 15}, {NULL}};
const Segment SGI367859[] = {{&S10128[106], 3}, {NULL}};
const Segment SGI367860[] = {{&S10128[49], 6}, {NULL}};
const Segment SGI367861[] = {{&S10128[33], 13}, {NULL}};
const Segment SGI367862[] = {{&S10129[69], 6}, {NULL}};
const Segment SGI367863[] = {{&S10129[8], 3}, {NULL}};
const Segment SGI367864[] = {{&S10129[13], 9}, {NULL}};
const Segment SGI367865[] = {{&S10129[33], 15}, {NULL}};
const Segment SGI367866[] = {{&S10129[106], 3}, {NULL}};
const Segment SGI367867[] = {{&S10129[49], 6}, {NULL}};
const Segment SGI367868[] = {{&S10129[33], 13}, {NULL}};
const Segment SGI367869[] = {{&S10130[69], 5}, {NULL}};
const Segment SGI367870[] = {{&S10130[8], 3}, {NULL}};
const Segment SGI367871[] = {{&S10130[13], 9}, {NULL}};
const Segment SGI367872[] = {{&S10130[33], 15}, {NULL}};
const Segment SGI367873[] = {{&S10130[103], 3}, {NULL}};
const Segment SGI367874[] = {{&S10130[49], 6}, {NULL}};
const Segment SGI367875[] = {{&S10130[33], 13}, {NULL}};
const Segment SGI367876[] = {{&S10131[69], 6}, {NULL}};
const Segment SGI367877[] = {{&S10131[8], 3}, {NULL}};
const Segment SGI367878[] = {{&S10131[13], 9}, {NULL}};
const Segment SGI367879[] = {{&S10131[33], 15}, {NULL}};
const Segment SGI367880[] = {{&S10131[106], 3}, {NULL}};
const Segment SGI367881[] = {{&S10131[49], 6}, {NULL}};
const Segment SGI367882[] = {{&S10131[33], 13}, {NULL}};
const Segment SGI367883[] = {{&S10132[69], 6}, {NULL}};
const Segment SGI367884[] = {{&S10132[8], 3}, {NULL}};
const Segment SGI367885[] = {{&S10132[13], 9}, {NULL}};
const Segment SGI367886[] = {{&S10132[33], 15}, {NULL}};
const Segment SGI367887[] = {{&S10132[106], 3}, {NULL}};
const Segment SGI367888[] = {{&S10132[49], 6}, {NULL}};
const Segment SGI367889[] = {{&S10132[33], 13}, {NULL}};
const Segment SGI367890[] = {{&S10133[69], 6}, {NULL}};
const Segment SGI367891[] = {{&S10133[8], 3}, {NULL}};
const Segment SGI367892[] = {{&S10133[13], 9}, {NULL}};
const Segment SGI367893[] = {{&S10133[33], 15}, {NULL}};
const Segment SGI367894[] = {{&S10133[106], 3}, {NULL}};
const Segment SGI367895[] = {{&S10133[49], 6}, {NULL}};
const Segment SGI367896[] = {{&S10133[33], 13}, {NULL}};
const Segment SGI367897[] = {{&S10134[115], 6}, {NULL}};
const Segment SGI367898[] = {{&S10134[8], 3}, {NULL}};
const Segment SGI367899[] = {{&S10134[13], 9}, {NULL}};
const Segment SGI367900[] = {{&S10134[33], 15}, {NULL}};
const Segment SGI367901[] = {{&S10134[104], 3}, {NULL}};
const Segment SGI367902[] = {{&S10134[49], 6}, {NULL}};
const Segment SGI367903[] = {{&S10134[33], 13}, {NULL}};
const Segment SGI367904[] = {{&S10135[69], 6}, {NULL}};
const Segment SGI367905[] = {{&S10135[8], 3}, {NULL}};
const Segment SGI367906[] = {{&S10135[13], 9}, {NULL}};
const Segment SGI367907[] = {{&S10135[33], 15}, {NULL}};
const Segment SGI367908[] = {{&S10135[106], 3}, {NULL}};
const Segment SGI367909[] = {{&S10135[49], 6}, {NULL}};
const Segment SGI367910[] = {{&S10135[33], 13}, {NULL}};
const Segment SGI367911[] = {{&S10136[69], 6}, {NULL}};
const Segment SGI367912[] = {{&S10136[8], 3}, {NULL}};
const Segment SGI367913[] = {{&S10136[13], 9}, {NULL}};
const Segment SGI367914[] = {{&S10136[33], 15}, {NULL}};
const Segment SGI367915[] = {{&S10136[104], 3}, {NULL}};
const Segment SGI367916[] = {{&S10136[49], 6}, {NULL}};
const Segment SGI367917[] = {{&S10136[33], 13}, {NULL}};
const Segment SGI367918[] = {{&S10137[69], 5}, {NULL}};
const Segment SGI367919[] = {{&S10137[8], 3}, {NULL}};
const Segment SGI367920[] = {{&S10137[13], 9}, {NULL}};
const Segment SGI367921[] = {{&S10137[33], 15}, {NULL}};
const Segment SGI367922[] = {{&S10137[103], 3}, {NULL}};
const Segment SGI367923[] = {{&S10137[49], 6}, {NULL}};
const Segment SGI367924[] = {{&S10137[33], 13}, {NULL}};
const Segment SGI367925[] = {{&S10138[67], 6}, {NULL}};
const Segment SGI367926[] = {{&S10138[8], 3}, {NULL}};
const Segment SGI367927[] = {{&S10138[13], 9}, {NULL}};
const Segment SGI367928[] = {{&S10138[33], 15}, {NULL}};
const Segment SGI367929[] = {{&S10138[102], 3}, {NULL}};
const Segment SGI367930[] = {{&S10138[49], 4}, {NULL}};
const Segment SGI367931[] = {{&S10138[33], 13}, {NULL}};
const Segment SGI367932[] = {{&S10139[69], 6}, {NULL}};
const Segment SGI367933[] = {{&S10139[8], 3}, {NULL}};
const Segment SGI367934[] = {{&S10139[13], 9}, {NULL}};
const Segment SGI367935[] = {{&S10139[33], 15}, {NULL}};
const Segment SGI367936[] = {{&S10139[104], 3}, {NULL}};
const Segment SGI367937[] = {{&S10139[49], 6}, {NULL}};
const Segment SGI367938[] = {{&S10139[33], 13}, {NULL}};
const Segment SGI367939[] = {{&S10140[109], 6}, {NULL}};
const Segment SGI367940[] = {{&S10140[8], 3}, {NULL}};
const Segment SGI367941[] = {{&S10140[13], 9}, {NULL}};
const Segment SGI367942[] = {{&S10140[96], 3}, {NULL}};
const Segment SGI367943[] = {{&S10140[30], 13}, {NULL}};
const Segment SGI367944[] = {{&S10141[109], 6}, {NULL}};
const Segment SGI367945[] = {{&S10141[8], 3}, {NULL}};
const Segment SGI367946[] = {{&S10141[13], 9}, {NULL}};
const Segment SGI367947[] = {{&S10141[96], 3}, {NULL}};
const Segment SGI367948[] = {{&S10141[30], 13}, {NULL}};
const Segment SGI367949[] = {{&S10142[109], 5}, {NULL}};
const Segment SGI367950[] = {{&S10142[8], 3}, {NULL}};
const Segment SGI367951[] = {{&S10142[13], 9}, {NULL}};
const Segment SGI367952[] = {{&S10142[96], 3}, {NULL}};
const Segment SGI367953[] = {{&S10142[30], 13}, {NULL}};
const Segment SGI367954[] = {{&S10143[109], 6}, {NULL}};
const Segment SGI367955[] = {{&S10143[8], 3}, {NULL}};
const Segment SGI367956[] = {{&S10143[13], 9}, {NULL}};
const Segment SGI367957[] = {{&S10143[96], 3}, {NULL}};
const Segment SGI367958[] = {{&S10143[30], 13}, {NULL}};
const Segment SGI367959[] = {{&S10144[107], 6}, {NULL}};
const Segment SGI367960[] = {{&S10144[8], 3}, {NULL}};
const Segment SGI367961[] = {{&S10144[13], 9}, {NULL}};
const Segment SGI367962[] = {{&S10144[96], 3}, {NULL}};
const Segment SGI367963[] = {{&S10144[30], 13}, {NULL}};
const Segment SGI367964[] = {{&S10145[109], 6}, {NULL}};
const Segment SGI367965[] = {{&S10145[8], 3}, {NULL}};
const Segment SGI367966[] = {{&S10145[13], 9}, {NULL}};
const Segment SGI367967[] = {{&S10145[96], 3}, {NULL}};
const Segment SGI367968[] = {{&S10145[30], 13}, {NULL}};
const Segment SGI367969[] = {{&S10146[69], 6}, {NULL}};
const Segment SGI367970[] = {{&S10146[8], 3}, {NULL}};
const Segment SGI367971[] = {{&S10146[13], 9}, {NULL}};
const Segment SGI367972[] = {{&S10146[33], 15}, {NULL}};
const Segment SGI367973[] = {{&S10146[106], 3}, {NULL}};
const Segment SGI367974[] = {{&S10146[49], 6}, {NULL}};
const Segment SGI367975[] = {{&S10146[33], 13}, {NULL}};
const Segment SGI367976[] = {{&S10147[69], 6}, {NULL}};
const Segment SGI367977[] = {{&S10147[8], 3}, {NULL}};
const Segment SGI367978[] = {{&S10147[13], 9}, {NULL}};
const Segment SGI367979[] = {{&S10147[33], 15}, {NULL}};
const Segment SGI367980[] = {{&S10147[106], 3}, {NULL}};
const Segment SGI367981[] = {{&S10147[49], 6}, {NULL}};
const Segment SGI367982[] = {{&S10147[33], 13}, {NULL}};
const Segment SGI367983[] = {{&S10148[69], 6}, {NULL}};
const Segment SGI367984[] = {{&S10148[8], 3}, {NULL}};
const Segment SGI367985[] = {{&S10148[13], 9}, {NULL}};
const Segment SGI367986[] = {{&S10148[33], 15}, {NULL}};
const Segment SGI367987[] = {{&S10148[106], 3}, {NULL}};
const Segment SGI367988[] = {{&S10148[49], 6}, {NULL}};
const Segment SGI367989[] = {{&S10148[33], 13}, {NULL}};
const Segment SGI367990[] = {{&S10149[69], 6}, {NULL}};
const Segment SGI367991[] = {{&S10149[8], 3}, {NULL}};
const Segment SGI367992[] = {{&S10149[13], 9}, {NULL}};
const Segment SGI367993[] = {{&S10149[33], 15}, {NULL}};
const Segment SGI367994[] = {{&S10149[106], 3}, {NULL}};
const Segment SGI367995[] = {{&S10149[49], 6}, {NULL}};
const Segment SGI367996[] = {{&S10149[33], 13}, {NULL}};
const Segment SGI367997[] = {{&S10150[69], 6}, {NULL}};
const Segment SGI367998[] = {{&S10150[8], 3}, {NULL}};
const Segment SGI367999[] = {{&S10150[13], 9}, {NULL}};
const Segment SGI368000[] = {{&S10150[33], 15}, {NULL}};
const Segment SGI368001[] = {{&S10150[106], 3}, {NULL}};
const Segment SGI368002[] = {{&S10150[49], 6}, {NULL}};
const Segment SGI368003[] = {{&S10150[33], 13}, {NULL}};
const Segment SGI368004[] = {{&S10151[69], 6}, {NULL}};
const Segment SGI368005[] = {{&S10151[8], 3}, {NULL}};
const Segment SGI368006[] = {{&S10151[13], 9}, {NULL}};
const Segment SGI368007[] = {{&S10151[33], 15}, {NULL}};
const Segment SGI368008[] = {{&S10151[106], 3}, {NULL}};
const Segment SGI368009[] = {{&S10151[49], 6}, {NULL}};
const Segment SGI368010[] = {{&S10151[33], 13}, {NULL}};
const Segment SGI368011[] = {{&S10152[69], 6}, {NULL}};
const Segment SGI368012[] = {{&S10152[8], 3}, {NULL}};
const Segment SGI368013[] = {{&S10152[13], 9}, {NULL}};
const Segment SGI368014[] = {{&S10152[33], 15}, {NULL}};
const Segment SGI368015[] = {{&S10152[106], 3}, {NULL}};
const Segment SGI368016[] = {{&S10152[49], 6}, {NULL}};
const Segment SGI368017[] = {{&S10152[33], 13}, {NULL}};
const Segment SGI368018[] = {{&S10153[69], 6}, {NULL}};
const Segment SGI368019[] = {{&S10153[8], 3}, {NULL}};
const Segment SGI368020[] = {{&S10153[13], 9}, {NULL}};
const Segment SGI368021[] = {{&S10153[33], 15}, {NULL}};
const Segment SGI368022[] = {{&S10153[106], 3}, {NULL}};
const Segment SGI368023[] = {{&S10153[49], 6}, {NULL}};
const Segment SGI368024[] = {{&S10153[33], 13}, {NULL}};
const Segment SGI368025[] = {{&S10154[69], 6}, {NULL}};
const Segment SGI368026[] = {{&S10154[8], 3}, {NULL}};
const Segment SGI368027[] = {{&S10154[13], 9}, {NULL}};
const Segment SGI368028[] = {{&S10154[33], 15}, {NULL}};
const Segment SGI368029[] = {{&S10154[106], 3}, {NULL}};
const Segment SGI368030[] = {{&S10154[49], 6}, {NULL}};
const Segment SGI368031[] = {{&S10154[33], 13}, {NULL}};
const Segment SGI368032[] = {{&S10155[69], 6}, {NULL}};
const Segment SGI368033[] = {{&S10155[8], 3}, {NULL}};
const Segment SGI368034[] = {{&S10155[13], 9}, {NULL}};
const Segment SGI368035[] = {{&S10155[33], 15}, {NULL}};
const Segment SGI368036[] = {{&S10155[106], 3}, {NULL}};
const Segment SGI368037[] = {{&S10155[49], 6}, {NULL}};
const Segment SGI368038[] = {{&S10155[33], 13}, {NULL}};
const Segment SGI368039[] = {{&S10156[69], 5}, {NULL}};
const Segment SGI368040[] = {{&S10156[8], 3}, {NULL}};
const Segment SGI368041[] = {{&S10156[13], 9}, {NULL}};
const Segment SGI368042[] = {{&S10156[33], 15}, {NULL}};
const Segment SGI368043[] = {{&S10156[103], 3}, {NULL}};
const Segment SGI368044[] = {{&S10156[49], 6}, {NULL}};
const Segment SGI368045[] = {{&S10156[33], 13}, {NULL}};
const Segment SGI368046[] = {{&S10157[69], 5}, {NULL}};
const Segment SGI368047[] = {{&S10157[8], 3}, {NULL}};
const Segment SGI368048[] = {{&S10157[13], 9}, {NULL}};
const Segment SGI368049[] = {{&S10157[33], 15}, {NULL}};
const Segment SGI368050[] = {{&S10157[103], 3}, {NULL}};
const Segment SGI368051[] = {{&S10157[49], 6}, {NULL}};
const Segment SGI368052[] = {{&S10157[33], 13}, {NULL}};
const Segment SGI368053[] = {{&S10158[69], 6}, {NULL}};
const Segment SGI368054[] = {{&S10158[8], 3}, {NULL}};
const Segment SGI368055[] = {{&S10158[13], 9}, {NULL}};
const Segment SGI368056[] = {{&S10158[33], 15}, {NULL}};
const Segment SGI368057[] = {{&S10158[106], 3}, {NULL}};
const Segment SGI368058[] = {{&S10158[49], 6}, {NULL}};
const Segment SGI368059[] = {{&S10158[33], 13}, {NULL}};
const Segment SGI368060[] = {{&S10159[69], 5}, {NULL}};
const Segment SGI368061[] = {{&S10159[8], 3}, {NULL}};
const Segment SGI368062[] = {{&S10159[13], 9}, {NULL}};
const Segment SGI368063[] = {{&S10159[33], 15}, {NULL}};
const Segment SGI368064[] = {{&S10159[103], 3}, {NULL}};
const Segment SGI368065[] = {{&S10159[49], 6}, {NULL}};
const Segment SGI368066[] = {{&S10159[33], 13}, {NULL}};
const Segment SGI368067[] = {{&S10160[69], 6}, {NULL}};
const Segment SGI368068[] = {{&S10160[8], 3}, {NULL}};
const Segment SGI368069[] = {{&S10160[13], 9}, {NULL}};
const Segment SGI368070[] = {{&S10160[33], 15}, {NULL}};
const Segment SGI368071[] = {{&S10160[106], 3}, {NULL}};
const Segment SGI368072[] = {{&S10160[49], 6}, {NULL}};
const Segment SGI368073[] = {{&S10160[33], 13}, {NULL}};
const Segment SGI368074[] = {{&S10161[69], 6}, {NULL}};
const Segment SGI368075[] = {{&S10161[8], 3}, {NULL}};
const Segment SGI368076[] = {{&S10161[13], 9}, {NULL}};
const Segment SGI368077[] = {{&S10161[33], 15}, {NULL}};
const Segment SGI368078[] = {{&S10161[106], 3}, {NULL}};
const Segment SGI368079[] = {{&S10161[49], 6}, {NULL}};
const Segment SGI368080[] = {{&S10161[33], 13}, {NULL}};
const Segment SGI368081[] = {{&S10162[69], 5}, {NULL}};
const Segment SGI368082[] = {{&S10162[8], 3}, {NULL}};
const Segment SGI368083[] = {{&S10162[13], 9}, {NULL}};
const Segment SGI368084[] = {{&S10162[33], 15}, {NULL}};
const Segment SGI368085[] = {{&S10162[103], 3}, {NULL}};
const Segment SGI368086[] = {{&S10162[49], 6}, {NULL}};
const Segment SGI368087[] = {{&S10162[33], 13}, {NULL}};
const Segment SGI368088[] = {{&S10163[69], 6}, {NULL}};
const Segment SGI368089[] = {{&S10163[8], 3}, {NULL}};
const Segment SGI368090[] = {{&S10163[13], 9}, {NULL}};
const Segment SGI368091[] = {{&S10163[33], 15}, {NULL}};
const Segment SGI368092[] = {{&S10163[104], 3}, {NULL}};
const Segment SGI368093[] = {{&S10163[49], 6}, {NULL}};
const Segment SGI368094[] = {{&S10163[33], 13}, {NULL}};
const Segment SGI368095[] = {{&S10164[69], 6}, {NULL}};
const Segment SGI368096[] = {{&S10164[8], 3}, {NULL}};
const Segment SGI368097[] = {{&S10164[13], 9}, {NULL}};
const Segment SGI368098[] = {{&S10164[33], 15}, {NULL}};
const Segment SGI368099[] = {{&S10164[106], 3}, {NULL}};
const Segment SGI368100[] = {{&S10164[49], 6}, {NULL}};
const Segment SGI368101[] = {{&S10164[33], 13}, {NULL}};
const Segment SGI368102[] = {{&S10165[66], 6}, {NULL}};
const Segment SGI368103[] = {{&S10165[8], 3}, {NULL}};
const Segment SGI368104[] = {{&S10165[13], 9}, {NULL}};
const Segment SGI368105[] = {{&S10165[33], 15}, {NULL}};
const Segment SGI368106[] = {{&S10165[103], 3}, {NULL}};
const Segment SGI368107[] = {{&S10165[49], 3}, {NULL}};
const Segment SGI368108[] = {{&S10165[33], 13}, {NULL}};
const Segment SGI368109[] = {{&S10166[69], 6}, {NULL}};
const Segment SGI368110[] = {{&S10166[8], 3}, {NULL}};
const Segment SGI368111[] = {{&S10166[13], 9}, {NULL}};
const Segment SGI368112[] = {{&S10166[33], 15}, {NULL}};
const Segment SGI368113[] = {{&S10166[106], 3}, {NULL}};
const Segment SGI368114[] = {{&S10166[49], 6}, {NULL}};
const Segment SGI368115[] = {{&S10166[33], 13}, {NULL}};
const Segment SGI368116[] = {{&S10167[69], 5}, {NULL}};
const Segment SGI368117[] = {{&S10167[8], 3}, {NULL}};
const Segment SGI368118[] = {{&S10167[13], 9}, {NULL}};
const Segment SGI368119[] = {{&S10167[33], 15}, {NULL}};
const Segment SGI368120[] = {{&S10167[103], 3}, {NULL}};
const Segment SGI368121[] = {{&S10167[49], 6}, {NULL}};
const Segment SGI368122[] = {{&S10167[33], 13}, {NULL}};
const Segment SGI368123[] = {{&S10168[117], 6}, {NULL}};
const Segment SGI368124[] = {{&S10168[8], 3}, {NULL}};
const Segment SGI368125[] = {{&S10168[13], 9}, {NULL}};
const Segment SGI368126[] = {{&S10168[33], 15}, {NULL}};
const Segment SGI368127[] = {{&S10168[106], 3}, {NULL}};
const Segment SGI368128[] = {{&S10168[49], 6}, {NULL}};
const Segment SGI368129[] = {{&S10168[33], 13}, {NULL}};
const Segment SGI368130[] = {{&S10169[69], 6}, {NULL}};
const Segment SGI368131[] = {{&S10169[8], 3}, {NULL}};
const Segment SGI368132[] = {{&S10169[13], 9}, {NULL}};
const Segment SGI368133[] = {{&S10169[33], 15}, {NULL}};
const Segment SGI368134[] = {{&S10169[104], 3}, {NULL}};
const Segment SGI368135[] = {{&S10169[49], 6}, {NULL}};
const Segment SGI368136[] = {{&S10169[33], 13}, {NULL}};
const Segment SGI368137[] = {{&S10170[69], 6}, {NULL}};
const Segment SGI368138[] = {{&S10170[8], 3}, {NULL}};
const Segment SGI368139[] = {{&S10170[13], 9}, {NULL}};
const Segment SGI368140[] = {{&S10170[33], 15}, {NULL}};
const Segment SGI368141[] = {{&S10170[106], 3}, {NULL}};
const Segment SGI368142[] = {{&S10170[49], 6}, {NULL}};
const Segment SGI368143[] = {{&S10170[33], 13}, {NULL}};
const Segment SGI368144[] = {{&S10171[69], 6}, {NULL}};
const Segment SGI368145[] = {{&S10171[8], 3}, {NULL}};
const Segment SGI368146[] = {{&S10171[13], 9}, {NULL}};
const Segment SGI368147[] = {{&S10171[33], 15}, {NULL}};
const Segment SGI368148[] = {{&S10171[104], 3}, {NULL}};
const Segment SGI368149[] = {{&S10171[49], 6}, {NULL}};
const Segment SGI368150[] = {{&S10171[33], 13}, {NULL}};
const Segment SGI368151[] = {{&S10172[69], 6}, {NULL}};
const Segment SGI368152[] = {{&S10172[8], 3}, {NULL}};
const Segment SGI368153[] = {{&S10172[13], 9}, {NULL}};
const Segment SGI368154[] = {{&S10172[33], 15}, {NULL}};
const Segment SGI368155[] = {{&S10172[104], 3}, {NULL}};
const Segment SGI368156[] = {{&S10172[49], 6}, {NULL}};
const Segment SGI368157[] = {{&S10172[33], 13}, {NULL}};
const Segment SGI368158[] = {{&S10173[68], 6}, {NULL}};
const Segment SGI368159[] = {{&S10173[8], 3}, {NULL}};
const Segment SGI368160[] = {{&S10173[32], 15}, {NULL}};
const Segment SGI368161[] = {{&S10173[103], 3}, {NULL}};
const Segment SGI368162[] = {{&S10173[48], 6}, {NULL}};
const Segment SGI368163[] = {{&S10173[32], 13}, {NULL}};
const Segment SGI368164[] = {{&S10174[69], 5}, {NULL}};
const Segment SGI368165[] = {{&S10174[8], 3}, {NULL}};
const Segment SGI368166[] = {{&S10174[13], 9}, {NULL}};
const Segment SGI368167[] = {{&S10174[33], 15}, {NULL}};
const Segment SGI368168[] = {{&S10174[103], 3}, {NULL}};
const Segment SGI368169[] = {{&S10174[49], 6}, {NULL}};
const Segment SGI368170[] = {{&S10174[33], 13}, {NULL}};
const Segment SGI368171[] = {{&S10175[107], 6}, {NULL}};
const Segment SGI368172[] = {{&S10175[8], 3}, {NULL}};
const Segment SGI368173[] = {{&S10175[13], 9}, {NULL}};
const Segment SGI368174[] = {{&S10175[96], 3}, {NULL}};
const Segment SGI368175[] = {{&S10175[30], 13}, {NULL}};
const Segment SGI368176[] = {{&S10176[69], 5}, {NULL}};
const Segment SGI368177[] = {{&S10176[8], 3}, {NULL}};
const Segment SGI368178[] = {{&S10176[13], 9}, {NULL}};
const Segment SGI368179[] = {{&S10176[33], 15}, {NULL}};
const Segment SGI368180[] = {{&S10176[103], 3}, {NULL}};
const Segment SGI368181[] = {{&S10176[49], 6}, {NULL}};
const Segment SGI368182[] = {{&S10176[33], 13}, {NULL}};
const Segment SGI368183[] = {{&S10177[69], 6}, {NULL}};
const Segment SGI368184[] = {{&S10177[8], 3}, {NULL}};
const Segment SGI368185[] = {{&S10177[13], 9}, {NULL}};
const Segment SGI368186[] = {{&S10177[33], 15}, {NULL}};
const Segment SGI368187[] = {{&S10177[106], 3}, {NULL}};
const Segment SGI368188[] = {{&S10177[49], 6}, {NULL}};
const Segment SGI368189[] = {{&S10177[33], 13}, {NULL}};
const Segment SGI368190[] = {{&S10178[120], 7}, {NULL}};
const Segment SGI368191[] = {{&S10178[8], 3}, {NULL}};
const Segment SGI368192[] = {{&S10178[13], 9}, {NULL}};
const Segment SGI368193[] = {{&S10178[33], 15}, {NULL}};
const Segment SGI368194[] = {{&S10178[49], 6}, {NULL}};
const Segment SGI368195[] = {{&S10178[33], 13}, {NULL}};
const Segment SGI368196[] = {{&S10179[69], 6}, {NULL}};
const Segment SGI368197[] = {{&S10179[8], 3}, {NULL}};
const Segment SGI368198[] = {{&S10179[13], 9}, {NULL}};
const Segment SGI368199[] = {{&S10179[33], 15}, {NULL}};
const Segment SGI368200[] = {{&S10179[106], 3}, {NULL}};
const Segment SGI368201[] = {{&S10179[49], 6}, {NULL}};
const Segment SGI368202[] = {{&S10179[33], 13}, {NULL}};
const Segment SGI368203[] = {{&S10180[69], 6}, {NULL}};
const Segment SGI368204[] = {{&S10180[8], 3}, {NULL}};
const Segment SGI368205[] = {{&S10180[13], 9}, {NULL}};
const Segment SGI368206[] = {{&S10180[33], 15}, {NULL}};
const Segment SGI368207[] = {{&S10180[106], 3}, {NULL}};
const Segment SGI368208[] = {{&S10180[49], 6}, {NULL}};
const Segment SGI368209[] = {{&S10180[33], 13}, {NULL}};
const Segment SGI368210[] = {{&S10181[69], 6}, {NULL}};
const Segment SGI368211[] = {{&S10181[8], 3}, {NULL}};
const Segment SGI368212[] = {{&S10181[13], 9}, {NULL}};
const Segment SGI368213[] = {{&S10181[33], 15}, {NULL}};
const Segment SGI368214[] = {{&S10181[106], 3}, {NULL}};
const Segment SGI368215[] = {{&S10181[49], 6}, {NULL}};
const Segment SGI368216[] = {{&S10181[33], 13}, {NULL}};
const Segment SGI368217[] = {{&S10182[69], 6}, {NULL}};
const Segment SGI368218[] = {{&S10182[8], 3}, {NULL}};
const Segment SGI368219[] = {{&S10182[13], 9}, {NULL}};
const Segment SGI368220[] = {{&S10182[33], 15}, {NULL}};
const Segment SGI368221[] = {{&S10182[104], 3}, {NULL}};
const Segment SGI368222[] = {{&S10182[49], 6}, {NULL}};
const Segment SGI368223[] = {{&S10182[33], 13}, {NULL}};
const Segment SGI368224[] = {{&S10183[120], 7}, {NULL}};
const Segment SGI368225[] = {{&S10183[8], 3}, {NULL}};
const Segment SGI368226[] = {{&S10183[13], 9}, {NULL}};
const Segment SGI368227[] = {{&S10183[33], 15}, {NULL}};
const Segment SGI368228[] = {{&S10183[49], 6}, {NULL}};
const Segment SGI368229[] = {{&S10183[33], 13}, {NULL}};
const Segment SGI368230[] = {{&S10184[69], 6}, {NULL}};
const Segment SGI368231[] = {{&S10184[8], 3}, {NULL}};
const Segment SGI368232[] = {{&S10184[13], 9}, {NULL}};
const Segment SGI368233[] = {{&S10184[33], 15}, {NULL}};
const Segment SGI368234[] = {{&S10184[106], 3}, {NULL}};
const Segment SGI368235[] = {{&S10184[49], 6}, {NULL}};
const Segment SGI368236[] = {{&S10184[33], 13}, {NULL}};
const Segment SGI368237[] = {{&S10185[66], 6}, {NULL}};
const Segment SGI368238[] = {{&S10185[8], 3}, {NULL}};
const Segment SGI368239[] = {{&S10185[13], 9}, {NULL}};
const Segment SGI368240[] = {{&S10185[33], 15}, {NULL}};
const Segment SGI368241[] = {{&S10185[103], 3}, {NULL}};
const Segment SGI368242[] = {{&S10185[49], 3}, {NULL}};
const Segment SGI368243[] = {{&S10185[33], 13}, {NULL}};
const Segment SGI368244[] = {{&S10186[119], 6}, {NULL}};
const Segment SGI368245[] = {{&S10186[8], 3}, {NULL}};
const Segment SGI368246[] = {{&S10186[13], 9}, {NULL}};
const Segment SGI368247[] = {{&S10186[33], 15}, {NULL}};
const Segment SGI368248[] = {{&S10186[106], 3}, {NULL}};
const Segment SGI368249[] = {{&S10186[49], 6}, {NULL}};
const Segment SGI368250[] = {{&S10186[33], 13}, {NULL}};
const Segment SGI368251[] = {{&S10187[69], 5}, {NULL}};
const Segment SGI368252[] = {{&S10187[8], 3}, {NULL}};
const Segment SGI368253[] = {{&S10187[13], 9}, {NULL}};
const Segment SGI368254[] = {{&S10187[33], 15}, {NULL}};
const Segment SGI368255[] = {{&S10187[103], 3}, {NULL}};
const Segment SGI368256[] = {{&S10187[49], 6}, {NULL}};
const Segment SGI368257[] = {{&S10187[33], 13}, {NULL}};
const Segment SGI368258[] = {{&S10188[69], 6}, {NULL}};
const Segment SGI368259[] = {{&S10188[8], 3}, {NULL}};
const Segment SGI368260[] = {{&S10188[13], 9}, {NULL}};
const Segment SGI368261[] = {{&S10188[33], 15}, {NULL}};
const Segment SGI368262[] = {{&S10188[106], 3}, {NULL}};
const Segment SGI368263[] = {{&S10188[49], 6}, {NULL}};
const Segment SGI368264[] = {{&S10188[33], 13}, {NULL}};
const Segment SGI368265[] = {{&S10189[69], 6}, {NULL}};
const Segment SGI368266[] = {{&S10189[8], 3}, {NULL}};
const Segment SGI368267[] = {{&S10189[13], 9}, {NULL}};
const Segment SGI368268[] = {{&S10189[33], 15}, {NULL}};
const Segment SGI368269[] = {{&S10189[106], 3}, {NULL}};
const Segment SGI368270[] = {{&S10189[49], 6}, {NULL}};
const Segment SGI368271[] = {{&S10189[33], 13}, {NULL}};
const Segment SGI368272[] = {{&S10190[69], 6}, {NULL}};
const Segment SGI368273[] = {{&S10190[8], 3}, {NULL}};
const Segment SGI368274[] = {{&S10190[13], 9}, {NULL}};
const Segment SGI368275[] = {{&S10190[33], 15}, {NULL}};
const Segment SGI368276[] = {{&S10190[106], 3}, {NULL}};
const Segment SGI368277[] = {{&S10190[49], 6}, {NULL}};
const Segment SGI368278[] = {{&S10190[33], 13}, {NULL}};
const Segment SGI368279[] = {{&S10191[69], 6}, {NULL}};
const Segment SGI368280[] = {{&S10191[8], 3}, {NULL}};
const Segment SGI368281[] = {{&S10191[13], 9}, {NULL}};
const Segment SGI368282[] = {{&S10191[33], 15}, {NULL}};
const Segment SGI368283[] = {{&S10191[106], 3}, {NULL}};
const Segment SGI368284[] = {{&S10191[49], 6}, {NULL}};
const Segment SGI368285[] = {{&S10191[33], 13}, {NULL}};
const Segment SGI368286[] = {{&S10192[69], 6}, {NULL}};
const Segment SGI368287[] = {{&S10192[8], 3}, {NULL}};
const Segment SGI368288[] = {{&S10192[13], 9}, {NULL}};
const Segment SGI368289[] = {{&S10192[33], 15}, {NULL}};
const Segment SGI368290[] = {{&S10192[106], 3}, {NULL}};
const Segment SGI368291[] = {{&S10192[49], 6}, {NULL}};
const Segment SGI368292[] = {{&S10192[33], 13}, {NULL}};
const Segment SGI368293[] = {{&S10193[69], 6}, {NULL}};
const Segment SGI368294[] = {{&S10193[8], 3}, {NULL}};
const Segment SGI368295[] = {{&S10193[13], 9}, {NULL}};
const Segment SGI368296[] = {{&S10193[33], 15}, {NULL}};
const Segment SGI368297[] = {{&S10193[106], 3}, {NULL}};
const Segment SGI368298[] = {{&S10193[49], 6}, {NULL}};
const Segment SGI368299[] = {{&S10193[33], 13}, {NULL}};
const Segment SGI368300[] = {{&S10194[69], 6}, {NULL}};
const Segment SGI368301[] = {{&S10194[8], 3}, {NULL}};
const Segment SGI368302[] = {{&S10194[13], 9}, {NULL}};
const Segment SGI368303[] = {{&S10194[33], 15}, {NULL}};
const Segment SGI368304[] = {{&S10194[106], 3}, {NULL}};
const Segment SGI368305[] = {{&S10194[49], 6}, {NULL}};
const Segment SGI368306[] = {{&S10194[33], 13}, {NULL}};
const Segment SGI368307[] = {{&S10195[69], 6}, {NULL}};
const Segment SGI368308[] = {{&S10195[8], 3}, {NULL}};
const Segment SGI368309[] = {{&S10195[13], 9}, {NULL}};
const Segment SGI368310[] = {{&S10195[33], 15}, {NULL}};
const Segment SGI368311[] = {{&S10195[106], 3}, {NULL}};
const Segment SGI368312[] = {{&S10195[49], 6}, {NULL}};
const Segment SGI368313[] = {{&S10195[33], 13}, {NULL}};
const Segment SGI368314[] = {{&S10196[69], 6}, {NULL}};
const Segment SGI368315[] = {{&S10196[8], 3}, {NULL}};
const Segment SGI368316[] = {{&S10196[13], 9}, {NULL}};
const Segment SGI368317[] = {{&S10196[33], 15}, {NULL}};
const Segment SGI368318[] = {{&S10196[106], 3}, {NULL}};
const Segment SGI368319[] = {{&S10196[49], 6}, {NULL}};
const Segment SGI368320[] = {{&S10196[33], 13}, {NULL}};
const Segment SGI368321[] = {{&S10197[69], 6}, {NULL}};
const Segment SGI368322[] = {{&S10197[8], 3}, {NULL}};
const Segment SGI368323[] = {{&S10197[13], 9}, {NULL}};
const Segment SGI368324[] = {{&S10197[33], 15}, {NULL}};
const Segment SGI368325[] = {{&S10197[106], 3}, {NULL}};
const Segment SGI368326[] = {{&S10197[49], 6}, {NULL}};
const Segment SGI368327[] = {{&S10197[33], 13}, {NULL}};
const Segment SGI368328[] = {{&S10198[68], 6}, {NULL}};
const Segment SGI368329[] = {{&S10198[8], 3}, {NULL}};
const Segment SGI368330[] = {{&S10198[13], 9}, {NULL}};
const Segment SGI368331[] = {{&S10198[33], 15}, {NULL}};
const Segment SGI368332[] = {{&S10198[103], 3}, {NULL}};
const Segment SGI368333[] = {{&S10198[49], 5}, {NULL}};
const Segment SGI368334[] = {{&S10198[33], 13}, {NULL}};
const Segment SGI368335[] = {{&S10199[69], 6}, {NULL}};
const Segment SGI368336[] = {{&S10199[8], 3}, {NULL}};
const Segment SGI368337[] = {{&S10199[13], 9}, {NULL}};
const Segment SGI368338[] = {{&S10199[33], 15}, {NULL}};
const Segment SGI368339[] = {{&S10199[106], 3}, {NULL}};
const Segment SGI368340[] = {{&S10199[49], 6}, {NULL}};
const Segment SGI368341[] = {{&S10199[33], 13}, {NULL}};
const Segment SGI368342[] = {{&S10200[66], 6}, {NULL}};
const Segment SGI368343[] = {{&S10200[8], 3}, {NULL}};
const Segment SGI368344[] = {{&S10200[13], 9}, {NULL}};
const Segment SGI368345[] = {{&S10200[33], 15}, {NULL}};
const Segment SGI368346[] = {{&S10200[103], 3}, {NULL}};
const Segment SGI368347[] = {{&S10200[49], 3}, {NULL}};
const Segment SGI368348[] = {{&S10200[33], 13}, {NULL}};
const Segment SGI368349[] = {{&S10201[69], 6}, {NULL}};
const Segment SGI368350[] = {{&S10201[8], 3}, {NULL}};
const Segment SGI368351[] = {{&S10201[13], 9}, {NULL}};
const Segment SGI368352[] = {{&S10201[33], 15}, {NULL}};
const Segment SGI368353[] = {{&S10201[106], 3}, {NULL}};
const Segment SGI368354[] = {{&S10201[49], 6}, {NULL}};
const Segment SGI368355[] = {{&S10201[33], 13}, {NULL}};
const Segment SGI368356[] = {{&S10202[69], 5}, {NULL}};
const Segment SGI368357[] = {{&S10202[8], 3}, {NULL}};
const Segment SGI368358[] = {{&S10202[13], 9}, {NULL}};
const Segment SGI368359[] = {{&S10202[33], 15}, {NULL}};
const Segment SGI368360[] = {{&S10202[103], 3}, {NULL}};
const Segment SGI368361[] = {{&S10202[49], 6}, {NULL}};
const Segment SGI368362[] = {{&S10202[33], 13}, {NULL}};
const Segment SGI368363[] = {{&S10203[69], 6}, {NULL}};
const Segment SGI368364[] = {{&S10203[8], 3}, {NULL}};
const Segment SGI368365[] = {{&S10203[13], 9}, {NULL}};
const Segment SGI368366[] = {{&S10203[33], 15}, {NULL}};
const Segment SGI368367[] = {{&S10203[104], 3}, {NULL}};
const Segment SGI368368[] = {{&S10203[49], 6}, {NULL}};
const Segment SGI368369[] = {{&S10203[33], 13}, {NULL}};
const Segment SGI368370[] = {{&S10204[69], 6}, {NULL}};
const Segment SGI368371[] = {{&S10204[8], 3}, {NULL}};
const Segment SGI368372[] = {{&S10204[13], 9}, {NULL}};
const Segment SGI368373[] = {{&S10204[33], 15}, {NULL}};
const Segment SGI368374[] = {{&S10204[106], 3}, {NULL}};
const Segment SGI368375[] = {{&S10204[49], 6}, {NULL}};
const Segment SGI368376[] = {{&S10204[33], 13}, {NULL}};
const Segment SGI368377[] = {{&S10205[69], 5}, {NULL}};
const Segment SGI368378[] = {{&S10205[8], 3}, {NULL}};
const Segment SGI368379[] = {{&S10205[13], 9}, {NULL}};
const Segment SGI368380[] = {{&S10205[33], 15}, {NULL}};
const Segment SGI368381[] = {{&S10205[103], 3}, {NULL}};
const Segment SGI368382[] = {{&S10205[49], 6}, {NULL}};
const Segment SGI368383[] = {{&S10205[33], 13}, {NULL}};
const Segment SGI368384[] = {{&S10206[69], 6}, {NULL}};
const Segment SGI368385[] = {{&S10206[8], 3}, {NULL}};
const Segment SGI368386[] = {{&S10206[13], 9}, {NULL}};
const Segment SGI368387[] = {{&S10206[33], 15}, {NULL}};
const Segment SGI368388[] = {{&S10206[106], 3}, {NULL}};
const Segment SGI368389[] = {{&S10206[49], 6}, {NULL}};
const Segment SGI368390[] = {{&S10206[33], 13}, {NULL}};
const Segment SGI368391[] = {{&S10207[69], 6}, {NULL}};
const Segment SGI368392[] = {{&S10207[8], 3}, {NULL}};
const Segment SGI368393[] = {{&S10207[13], 9}, {NULL}};
const Segment SGI368394[] = {{&S10207[33], 15}, {NULL}};
const Segment SGI368395[] = {{&S10207[106], 3}, {NULL}};
const Segment SGI368396[] = {{&S10207[49], 6}, {NULL}};
const Segment SGI368397[] = {{&S10207[33], 13}, {NULL}};
const Segment SGI368398[] = {{&S10208[69], 6}, {NULL}};
const Segment SGI368399[] = {{&S10208[8], 3}, {NULL}};
const Segment SGI368400[] = {{&S10208[13], 9}, {NULL}};
const Segment SGI368401[] = {{&S10208[33], 15}, {NULL}};
const Segment SGI368402[] = {{&S10208[106], 3}, {NULL}};
const Segment SGI368403[] = {{&S10208[49], 6}, {NULL}};
const Segment SGI368404[] = {{&S10208[33], 13}, {NULL}};
const Segment SGI368405[] = {{&S10209[69], 6}, {NULL}};
const Segment SGI368406[] = {{&S10209[8], 3}, {NULL}};
const Segment SGI368407[] = {{&S10209[13], 9}, {NULL}};
const Segment SGI368408[] = {{&S10209[33], 15}, {NULL}};
const Segment SGI368409[] = {{&S10209[106], 3}, {NULL}};
const Segment SGI368410[] = {{&S10209[49], 6}, {NULL}};
const Segment SGI368411[] = {{&S10209[33], 13}, {NULL}};
const Segment SGI368412[] = {{&S10210[69], 5}, {NULL}};
const Segment SGI368413[] = {{&S10210[8], 3}, {NULL}};
const Segment SGI368414[] = {{&S10210[13], 9}, {NULL}};
const Segment SGI368415[] = {{&S10210[33], 15}, {NULL}};
const Segment SGI368416[] = {{&S10210[103], 3}, {NULL}};
const Segment SGI368417[] = {{&S10210[49], 6}, {NULL}};
const Segment SGI368418[] = {{&S10210[33], 13}, {NULL}};
const Segment SGI368419[] = {{&S10211[118], 6}, {NULL}};
const Segment SGI368420[] = {{&S10211[8], 3}, {NULL}};
const Segment SGI368421[] = {{&S10211[13], 9}, {NULL}};
const Segment SGI368422[] = {{&S10211[33], 15}, {NULL}};
const Segment SGI368423[] = {{&S10211[104], 3}, {NULL}};
const Segment SGI368424[] = {{&S10211[49], 6}, {NULL}};
const Segment SGI368425[] = {{&S10211[33], 13}, {NULL}};
const Segment SGI368426[] = {{&S10212[66], 6}, {NULL}};
const Segment SGI368427[] = {{&S10212[8], 3}, {NULL}};
const Segment SGI368428[] = {{&S10212[13], 9}, {NULL}};
const Segment SGI368429[] = {{&S10212[33], 15}, {NULL}};
const Segment SGI368430[] = {{&S10212[103], 3}, {NULL}};
const Segment SGI368431[] = {{&S10212[49], 3}, {NULL}};
const Segment SGI368432[] = {{&S10212[33], 13}, {NULL}};
const Segment SGI368433[] = {{&S10213[69], 6}, {NULL}};
const Segment SGI368434[] = {{&S10213[8], 3}, {NULL}};
const Segment SGI368435[] = {{&S10213[13], 9}, {NULL}};
const Segment SGI368436[] = {{&S10213[33], 15}, {NULL}};
const Segment SGI368437[] = {{&S10213[106], 3}, {NULL}};
const Segment SGI368438[] = {{&S10213[49], 6}, {NULL}};
const Segment SGI368439[] = {{&S10213[33], 13}, {NULL}};
const Segment SGI368440[] = {{&S10214[69], 6}, {NULL}};
const Segment SGI368441[] = {{&S10214[8], 3}, {NULL}};
const Segment SGI368442[] = {{&S10214[13], 9}, {NULL}};
const Segment SGI368443[] = {{&S10214[33], 15}, {NULL}};
const Segment SGI368444[] = {{&S10214[106], 3}, {NULL}};
const Segment SGI368445[] = {{&S10214[49], 6}, {NULL}};
const Segment SGI368446[] = {{&S10214[33], 13}, {NULL}};
const Segment SGI368447[] = {{&S10215[69], 6}, {NULL}};
const Segment SGI368448[] = {{&S10215[8], 3}, {NULL}};
const Segment SGI368449[] = {{&S10215[13], 9}, {NULL}};
const Segment SGI368450[] = {{&S10215[33], 15}, {NULL}};
const Segment SGI368451[] = {{&S10215[104], 3}, {NULL}};
const Segment SGI368452[] = {{&S10215[49], 6}, {NULL}};
const Segment SGI368453[] = {{&S10215[33], 13}, {NULL}};
const Segment SGI368454[] = {{&S10216[69], 6}, {NULL}};
const Segment SGI368455[] = {{&S10216[8], 3}, {NULL}};
const Segment SGI368456[] = {{&S10216[13], 9}, {NULL}};
const Segment SGI368457[] = {{&S10216[33], 15}, {NULL}};
const Segment SGI368458[] = {{&S10216[106], 3}, {NULL}};
const Segment SGI368459[] = {{&S10216[49], 6}, {NULL}};
const Segment SGI368460[] = {{&S10216[33], 13}, {NULL}};
const Segment SGI368461[] = {{&S10217[69], 6}, {NULL}};
const Segment SGI368462[] = {{&S10217[8], 3}, {NULL}};
const Segment SGI368463[] = {{&S10217[13], 9}, {NULL}};
const Segment SGI368464[] = {{&S10217[33], 15}, {NULL}};
const Segment SGI368465[] = {{&S10217[106], 3}, {NULL}};
const Segment SGI368466[] = {{&S10217[49], 6}, {NULL}};
const Segment SGI368467[] = {{&S10217[33], 13}, {NULL}};
const Segment SGI368468[] = {{&S10218[69], 6}, {NULL}};
const Segment SGI368469[] = {{&S10218[8], 3}, {NULL}};
const Segment SGI368470[] = {{&S10218[13], 9}, {NULL}};
const Segment SGI368471[] = {{&S10218[33], 15}, {NULL}};
const Segment SGI368472[] = {{&S10218[106], 3}, {NULL}};
const Segment SGI368473[] = {{&S10218[49], 6}, {NULL}};
const Segment SGI368474[] = {{&S10218[33], 13}, {NULL}};
const Segment SGI368475[] = {{&S10219[66], 6}, {NULL}};
const Segment SGI368476[] = {{&S10219[8], 3}, {NULL}};
const Segment SGI368477[] = {{&S10219[13], 9}, {NULL}};
const Segment SGI368478[] = {{&S10219[33], 15}, {NULL}};
const Segment SGI368479[] = {{&S10219[103], 3}, {NULL}};
const Segment SGI368480[] = {{&S10219[49], 3}, {NULL}};
const Segment SGI368481[] = {{&S10219[33], 13}, {NULL}};
const Segment SGI368482[] = {{&S10220[69], 6}, {NULL}};
const Segment SGI368483[] = {{&S10220[8], 3}, {NULL}};
const Segment SGI368484[] = {{&S10220[13], 9}, {NULL}};
const Segment SGI368485[] = {{&S10220[33], 15}, {NULL}};
const Segment SGI368486[] = {{&S10220[106], 3}, {NULL}};
const Segment SGI368487[] = {{&S10220[49], 6}, {NULL}};
const Segment SGI368488[] = {{&S10220[33], 13}, {NULL}};
const Segment SGI368489[] = {{&S10221[69], 6}, {NULL}};
const Segment SGI368490[] = {{&S10221[8], 3}, {NULL}};
const Segment SGI368491[] = {{&S10221[13], 9}, {NULL}};
const Segment SGI368492[] = {{&S10221[33], 15}, {NULL}};
const Segment SGI368493[] = {{&S10221[106], 3}, {NULL}};
const Segment SGI368494[] = {{&S10221[49], 6}, {NULL}};
const Segment SGI368495[] = {{&S10221[33], 13}, {NULL}};
const Segment SGI368496[] = {{&S10222[69], 6}, {NULL}};
const Segment SGI368497[] = {{&S10222[8], 3}, {NULL}};
const Segment SGI368498[] = {{&S10222[13], 9}, {NULL}};
const Segment SGI368499[] = {{&S10222[33], 15}, {NULL}};
const Segment SGI368500[] = {{&S10222[106], 3}, {NULL}};
const Segment SGI368501[] = {{&S10222[49], 6}, {NULL}};
const Segment SGI368502[] = {{&S10222[33], 13}, {NULL}};
const Segment SGI368503[] = {{&S10223[69], 6}, {NULL}};
const Segment SGI368504[] = {{&S10223[8], 3}, {NULL}};
const Segment SGI368505[] = {{&S10223[13], 9}, {NULL}};
const Segment SGI368506[] = {{&S10223[33], 15}, {NULL}};
const Segment SGI368507[] = {{&S10223[104], 3}, {NULL}};
const Segment SGI368508[] = {{&S10223[49], 6}, {NULL}};
const Segment SGI368509[] = {{&S10223[33], 13}, {NULL}};
const Segment SGI368510[] = {{&S10224[69], 6}, {NULL}};
const Segment SGI368511[] = {{&S10224[8], 3}, {NULL}};
const Segment SGI368512[] = {{&S10224[13], 9}, {NULL}};
const Segment SGI368513[] = {{&S10224[33], 15}, {NULL}};
const Segment SGI368514[] = {{&S10224[106], 3}, {NULL}};
const Segment SGI368515[] = {{&S10224[49], 6}, {NULL}};
const Segment SGI368516[] = {{&S10224[33], 13}, {NULL}};
const Segment SGI368517[] = {{&S10225[66], 6}, {NULL}};
const Segment SGI368518[] = {{&S10225[8], 3}, {NULL}};
const Segment SGI368519[] = {{&S10225[13], 9}, {NULL}};
const Segment SGI368520[] = {{&S10225[33], 15}, {NULL}};
const Segment SGI368521[] = {{&S10225[103], 3}, {NULL}};
const Segment SGI368522[] = {{&S10225[49], 3}, {NULL}};
const Segment SGI368523[] = {{&S10225[33], 13}, {NULL}};
const Segment SGI368524[] = {{&S10226[69], 6}, {NULL}};
const Segment SGI368525[] = {{&S10226[8], 3}, {NULL}};
const Segment SGI368526[] = {{&S10226[13], 9}, {NULL}};
const Segment SGI368527[] = {{&S10226[33], 15}, {NULL}};
const Segment SGI368528[] = {{&S10226[106], 3}, {NULL}};
const Segment SGI368529[] = {{&S10226[49], 6}, {NULL}};
const Segment SGI368530[] = {{&S10226[33], 13}, {NULL}};
const Segment SGI368531[] = {{&S10227[127], 6}, {NULL}};
const Segment SGI368532[] = {{&S10227[8], 3}, {NULL}};
const Segment SGI368533[] = {{&S10227[13], 9}, {NULL}};
const Segment SGI368534[] = {{&S10227[33], 15}, {NULL}};
const Segment SGI368535[] = {{&S10227[49], 6}, {NULL}};
const Segment SGI368536[] = {{&S10227[33], 13}, {NULL}};
const Segment SGI368537[] = {{&S10228[69], 6}, {NULL}};
const Segment SGI368538[] = {{&S10228[8], 3}, {NULL}};
const Segment SGI368539[] = {{&S10228[13], 9}, {NULL}};
const Segment SGI368540[] = {{&S10228[33], 15}, {NULL}};
const Segment SGI368541[] = {{&S10228[106], 3}, {NULL}};
const Segment SGI368542[] = {{&S10228[49], 6}, {NULL}};
const Segment SGI368543[] = {{&S10228[33], 13}, {NULL}};
const Segment SGI368544[] = {{&S10229[69], 6}, {NULL}};
const Segment SGI368545[] = {{&S10229[8], 3}, {NULL}};
const Segment SGI368546[] = {{&S10229[13], 9}, {NULL}};
const Segment SGI368547[] = {{&S10229[33], 15}, {NULL}};
const Segment SGI368548[] = {{&S10229[106], 3}, {NULL}};
const Segment SGI368549[] = {{&S10229[49], 6}, {NULL}};
const Segment SGI368550[] = {{&S10229[33], 13}, {NULL}};
const Segment SGI368551[] = {{&S10230[69], 5}, {NULL}};
const Segment SGI368552[] = {{&S10230[8], 3}, {NULL}};
const Segment SGI368553[] = {{&S10230[13], 9}, {NULL}};
const Segment SGI368554[] = {{&S10230[33], 15}, {NULL}};
const Segment SGI368555[] = {{&S10230[103], 3}, {NULL}};
const Segment SGI368556[] = {{&S10230[49], 6}, {NULL}};
const Segment SGI368557[] = {{&S10230[33], 13}, {NULL}};
const Segment SGI368558[] = {{&S10231[69], 6}, {NULL}};
const Segment SGI368559[] = {{&S10231[8], 3}, {NULL}};
const Segment SGI368560[] = {{&S10231[13], 9}, {NULL}};
const Segment SGI368561[] = {{&S10231[33], 15}, {NULL}};
const Segment SGI368562[] = {{&S10231[106], 3}, {NULL}};
const Segment SGI368563[] = {{&S10231[49], 6}, {NULL}};
const Segment SGI368564[] = {{&S10231[33], 13}, {NULL}};
const Segment SGI368565[] = {{&S10232[69], 6}, {NULL}};
const Segment SGI368566[] = {{&S10232[8], 3}, {NULL}};
const Segment SGI368567[] = {{&S10232[13], 9}, {NULL}};
const Segment SGI368568[] = {{&S10232[33], 15}, {NULL}};
const Segment SGI368569[] = {{&S10232[106], 3}, {NULL}};
const Segment SGI368570[] = {{&S10232[49], 6}, {NULL}};
const Segment SGI368571[] = {{&S10232[33], 13}, {NULL}};
const Segment SGI368572[] = {{&S10233[67], 6}, {NULL}};
const Segment SGI368573[] = {{&S10233[8], 3}, {NULL}};
const Segment SGI368574[] = {{&S10233[13], 9}, {NULL}};
const Segment SGI368575[] = {{&S10233[31], 15}, {NULL}};
const Segment SGI368576[] = {{&S10233[102], 3}, {NULL}};
const Segment SGI368577[] = {{&S10233[47], 6}, {NULL}};
const Segment SGI368578[] = {{&S10233[31], 13}, {NULL}};
const Segment SGI368579[] = {{&S10234[69], 5}, {NULL}};
const Segment SGI368580[] = {{&S10234[8], 3}, {NULL}};
const Segment SGI368581[] = {{&S10234[13], 9}, {NULL}};
const Segment SGI368582[] = {{&S10234[33], 15}, {NULL}};
const Segment SGI368583[] = {{&S10234[103], 3}, {NULL}};
const Segment SGI368584[] = {{&S10234[49], 6}, {NULL}};
const Segment SGI368585[] = {{&S10234[33], 13}, {NULL}};
const Segment SGI368586[] = {{&S10235[107], 6}, {NULL}};
const Segment SGI368587[] = {{&S10235[8], 3}, {NULL}};
const Segment SGI368588[] = {{&S10235[13], 9}, {NULL}};
const Segment SGI368589[] = {{&S10235[96], 3}, {NULL}};
const Segment SGI368590[] = {{&S10235[30], 13}, {NULL}};
const Segment SGI368591[] = {{&S10236[69], 6}, {NULL}};
const Segment SGI368592[] = {{&S10236[8], 3}, {NULL}};
const Segment SGI368593[] = {{&S10236[13], 9}, {NULL}};
const Segment SGI368594[] = {{&S10236[33], 15}, {NULL}};
const Segment SGI368595[] = {{&S10236[104], 3}, {NULL}};
const Segment SGI368596[] = {{&S10236[49], 6}, {NULL}};
const Segment SGI368597[] = {{&S10236[33], 13}, {NULL}};
const Segment SGI368598[] = {{&S10237[69], 6}, {NULL}};
const Segment SGI368599[] = {{&S10237[8], 3}, {NULL}};
const Segment SGI368600[] = {{&S10237[13], 9}, {NULL}};
const Segment SGI368601[] = {{&S10237[33], 15}, {NULL}};
const Segment SGI368602[] = {{&S10237[106], 3}, {NULL}};
const Segment SGI368603[] = {{&S10237[49], 6}, {NULL}};
const Segment SGI368604[] = {{&S10237[33], 13}, {NULL}};
const Segment SGI368605[] = {{&S10238[69], 6}, {NULL}};
const Segment SGI368606[] = {{&S10238[8], 3}, {NULL}};
const Segment SGI368607[] = {{&S10238[13], 9}, {NULL}};
const Segment SGI368608[] = {{&S10238[33], 15}, {NULL}};
const Segment SGI368609[] = {{&S10238[106], 3}, {NULL}};
const Segment SGI368610[] = {{&S10238[49], 6}, {NULL}};
const Segment SGI368611[] = {{&S10238[33], 13}, {NULL}};
const Segment SGI368612[] = {{&S10239[119], 6}, {NULL}};
const Segment SGI368613[] = {{&S10239[8], 3}, {NULL}};
const Segment SGI368614[] = {{&S10239[13], 9}, {NULL}};
const Segment SGI368615[] = {{&S10239[33], 15}, {NULL}};
const Segment SGI368616[] = {{&S10239[105], 3}, {NULL}};
const Segment SGI368617[] = {{&S10239[49], 6}, {NULL}};
const Segment SGI368618[] = {{&S10239[33], 13}, {NULL}};
const Segment SGI368619[] = {{&S10240[69], 6}, {NULL}};
const Segment SGI368620[] = {{&S10240[8], 3}, {NULL}};
const Segment SGI368621[] = {{&S10240[13], 9}, {NULL}};
const Segment SGI368622[] = {{&S10240[33], 15}, {NULL}};
const Segment SGI368623[] = {{&S10240[106], 3}, {NULL}};
const Segment SGI368624[] = {{&S10240[49], 6}, {NULL}};
const Segment SGI368625[] = {{&S10240[33], 13}, {NULL}};
const Segment SGI368626[] = {{&S10241[8], 3}, {NULL}};
const Segment SGI368627[] = {{&S10241[13], 9}, {NULL}};
const Segment SGI368628[] = {{&S10241[33], 15}, {NULL}};
const Segment SGI368629[] = {{&S10241[106], 3}, {NULL}};
const Segment SGI368630[] = {{&S10241[49], 6}, {NULL}};
const Segment SGI368631[] = {{&S10241[33], 13}, {NULL}};
const Segment SGI368632[] = {{&S10242[69], 6}, {NULL}};
const Segment SGI368633[] = {{&S10242[8], 3}, {NULL}};
const Segment SGI368634[] = {{&S10242[13], 9}, {NULL}};
const Segment SGI368635[] = {{&S10242[33], 15}, {NULL}};
const Segment SGI368636[] = {{&S10242[104], 3}, {NULL}};
const Segment SGI368637[] = {{&S10242[49], 6}, {NULL}};
const Segment SGI368638[] = {{&S10242[33], 13}, {NULL}};
const Segment SGI368639[] = {{&S10243[67], 6}, {NULL}};
const Segment SGI368640[] = {{&S10243[8], 3}, {NULL}};
const Segment SGI368641[] = {{&S10243[13], 9}, {NULL}};
const Segment SGI368642[] = {{&S10243[33], 15}, {NULL}};
const Segment SGI368643[] = {{&S10243[102], 3}, {NULL}};
const Segment SGI368644[] = {{&S10243[49], 4}, {NULL}};
const Segment SGI368645[] = {{&S10243[33], 13}, {NULL}};
const Segment SGI368646[] = {{&S10244[69], 5}, {NULL}};
const Segment SGI368647[] = {{&S10244[8], 3}, {NULL}};
const Segment SGI368648[] = {{&S10244[13], 9}, {NULL}};
const Segment SGI368649[] = {{&S10244[33], 15}, {NULL}};
const Segment SGI368650[] = {{&S10244[103], 3}, {NULL}};
const Segment SGI368651[] = {{&S10244[49], 6}, {NULL}};
const Segment SGI368652[] = {{&S10244[33], 13}, {NULL}};
const Segment SGI368653[] = {{&S10245[69], 6}, {NULL}};
const Segment SGI368654[] = {{&S10245[8], 3}, {NULL}};
const Segment SGI368655[] = {{&S10245[13], 9}, {NULL}};
const Segment SGI368656[] = {{&S10245[33], 15}, {NULL}};
const Segment SGI368657[] = {{&S10245[104], 3}, {NULL}};
const Segment SGI368658[] = {{&S10245[49], 6}, {NULL}};
const Segment SGI368659[] = {{&S10245[33], 13}, {NULL}};
const Segment SGI368660[] = {{&S10246[66], 6}, {NULL}};
const Segment SGI368661[] = {{&S10246[8], 3}, {NULL}};
const Segment SGI368662[] = {{&S10246[13], 9}, {NULL}};
const Segment SGI368663[] = {{&S10246[33], 15}, {NULL}};
const Segment SGI368664[] = {{&S10246[103], 3}, {NULL}};
const Segment SGI368665[] = {{&S10246[49], 3}, {NULL}};
const Segment SGI368666[] = {{&S10246[33], 13}, {NULL}};
const Segment SGI368667[] = {{&S10247[69], 5}, {NULL}};
const Segment SGI368668[] = {{&S10247[8], 3}, {NULL}};
const Segment SGI368669[] = {{&S10247[13], 9}, {NULL}};
const Segment SGI368670[] = {{&S10247[33], 15}, {NULL}};
const Segment SGI368671[] = {{&S10247[103], 3}, {NULL}};
const Segment SGI368672[] = {{&S10247[49], 6}, {NULL}};
const Segment SGI368673[] = {{&S10247[33], 13}, {NULL}};
const Segment SGI368674[] = {{&S10248[69], 5}, {NULL}};
const Segment SGI368675[] = {{&S10248[8], 3}, {NULL}};
const Segment SGI368676[] = {{&S10248[13], 9}, {NULL}};
const Segment SGI368677[] = {{&S10248[33], 15}, {NULL}};
const Segment SGI368678[] = {{&S10248[103], 3}, {NULL}};
const Segment SGI368679[] = {{&S10248[49], 6}, {NULL}};
const Segment SGI368680[] = {{&S10248[33], 13}, {NULL}};
const Segment SGI368681[] = {{&S10249[69], 6}, {NULL}};
const Segment SGI368682[] = {{&S10249[8], 3}, {NULL}};
const Segment SGI368683[] = {{&S10249[13], 9}, {NULL}};
const Segment SGI368684[] = {{&S10249[33], 15}, {NULL}};
const Segment SGI368685[] = {{&S10249[106], 3}, {NULL}};
const Segment SGI368686[] = {{&S10249[49], 6}, {NULL}};
const Segment SGI368687[] = {{&S10249[33], 13}, {NULL}};
const Segment SGI368688[] = {{&S10250[69], 5}, {NULL}};
const Segment SGI368689[] = {{&S10250[8], 3}, {NULL}};
const Segment SGI368690[] = {{&S10250[13], 9}, {NULL}};
const Segment SGI368691[] = {{&S10250[33], 15}, {NULL}};
const Segment SGI368692[] = {{&S10250[103], 3}, {NULL}};
const Segment SGI368693[] = {{&S10250[49], 6}, {NULL}};
const Segment SGI368694[] = {{&S10250[33], 13}, {NULL}};
const Segment SGI368695[] = {{&S10251[69], 6}, {NULL}};
const Segment SGI368696[] = {{&S10251[8], 3}, {NULL}};
const Segment SGI368697[] = {{&S10251[13], 9}, {NULL}};
const Segment SGI368698[] = {{&S10251[33], 15}, {NULL}};
const Segment SGI368699[] = {{&S10251[106], 3}, {NULL}};
const Segment SGI368700[] = {{&S10251[49], 6}, {NULL}};
const Segment SGI368701[] = {{&S10251[33], 13}, {NULL}};
const Segment SGI368702[] = {{&S10252[69], 6}, {NULL}};
const Segment SGI368703[] = {{&S10252[8], 3}, {NULL}};
const Segment SGI368704[] = {{&S10252[13], 9}, {NULL}};
const Segment SGI368705[] = {{&S10252[33], 15}, {NULL}};
const Segment SGI368706[] = {{&S10252[106], 3}, {NULL}};
const Segment SGI368707[] = {{&S10252[49], 6}, {NULL}};
const Segment SGI368708[] = {{&S10252[33], 13}, {NULL}};
const Segment SGI368709[] = {{&S10253[67], 6}, {NULL}};
const Segment SGI368710[] = {{&S10253[8], 3}, {NULL}};
const Segment SGI368711[] = {{&S10253[13], 9}, {NULL}};
const Segment SGI368712[] = {{&S10253[31], 15}, {NULL}};
const Segment SGI368713[] = {{&S10253[102], 3}, {NULL}};
const Segment SGI368714[] = {{&S10253[47], 6}, {NULL}};
const Segment SGI368715[] = {{&S10253[31], 13}, {NULL}};
const Segment SGI368716[] = {{&S10254[69], 6}, {NULL}};
const Segment SGI368717[] = {{&S10254[8], 3}, {NULL}};
const Segment SGI368718[] = {{&S10254[13], 9}, {NULL}};
const Segment SGI368719[] = {{&S10254[33], 15}, {NULL}};
const Segment SGI368720[] = {{&S10254[104], 3}, {NULL}};
const Segment SGI368721[] = {{&S10254[49], 6}, {NULL}};
const Segment SGI368722[] = {{&S10254[33], 13}, {NULL}};
const Segment SGI368723[] = {{&S10255[69], 6}, {NULL}};
const Segment SGI368724[] = {{&S10255[8], 3}, {NULL}};
const Segment SGI368725[] = {{&S10255[13], 9}, {NULL}};
const Segment SGI368726[] = {{&S10255[33], 15}, {NULL}};
const Segment SGI368727[] = {{&S10255[106], 3}, {NULL}};
const Segment SGI368728[] = {{&S10255[49], 6}, {NULL}};
const Segment SGI368729[] = {{&S10255[33], 13}, {NULL}};
const Segment SGI368730[] = {{&S10256[117], 6}, {NULL}};
const Segment SGI368731[] = {{&S10256[8], 3}, {NULL}};
const Segment SGI368732[] = {{&S10256[13], 9}, {NULL}};
const Segment SGI368733[] = {{&S10256[33], 15}, {NULL}};
const Segment SGI368734[] = {{&S10256[49], 6}, {NULL}};
const Segment SGI368735[] = {{&S10256[33], 13}, {NULL}};
const Segment SGI368736[] = {{&S10257[69], 6}, {NULL}};
const Segment SGI368737[] = {{&S10257[8], 3}, {NULL}};
const Segment SGI368738[] = {{&S10257[13], 9}, {NULL}};
const Segment SGI368739[] = {{&S10257[33], 15}, {NULL}};
const Segment SGI368740[] = {{&S10257[106], 3}, {NULL}};
const Segment SGI368741[] = {{&S10257[49], 6}, {NULL}};
const Segment SGI368742[] = {{&S10257[33], 13}, {NULL}};
const Segment SGI368743[] = {{&S10258[69], 6}, {NULL}};
const Segment SGI368744[] = {{&S10258[8], 3}, {NULL}};
const Segment SGI368745[] = {{&S10258[13], 9}, {NULL}};
const Segment SGI368746[] = {{&S10258[33], 15}, {NULL}};
const Segment SGI368747[] = {{&S10258[104], 3}, {NULL}};
const Segment SGI368748[] = {{&S10258[49], 6}, {NULL}};
const Segment SGI368749[] = {{&S10258[33], 13}, {NULL}};
const Segment SGI368750[] = {{&S10259[69], 6}, {NULL}};
const Segment SGI368751[] = {{&S10259[8], 3}, {NULL}};
const Segment SGI368752[] = {{&S10259[13], 9}, {NULL}};
const Segment SGI368753[] = {{&S10259[33], 15}, {NULL}};
const Segment SGI368754[] = {{&S10259[106], 3}, {NULL}};
const Segment SGI368755[] = {{&S10259[49], 6}, {NULL}};
const Segment SGI368756[] = {{&S10259[33], 13}, {NULL}};
const Segment SGI368757[] = {{&S10260[118], 6}, {NULL}};
const Segment SGI368758[] = {{&S10260[8], 3}, {NULL}};
const Segment SGI368759[] = {{&S10260[13], 9}, {NULL}};
const Segment SGI368760[] = {{&S10260[33], 15}, {NULL}};
const Segment SGI368761[] = {{&S10260[104], 3}, {NULL}};
const Segment SGI368762[] = {{&S10260[49], 6}, {NULL}};
const Segment SGI368763[] = {{&S10260[33], 13}, {NULL}};
const Segment SGI368764[] = {{&S10261[69], 6}, {NULL}};
const Segment SGI368765[] = {{&S10261[8], 3}, {NULL}};
const Segment SGI368766[] = {{&S10261[13], 9}, {NULL}};
const Segment SGI368767[] = {{&S10261[33], 15}, {NULL}};
const Segment SGI368768[] = {{&S10261[106], 3}, {NULL}};
const Segment SGI368769[] = {{&S10261[49], 6}, {NULL}};
const Segment SGI368770[] = {{&S10261[33], 13}, {NULL}};
const Segment SGI368771[] = {{&S10262[69], 6}, {NULL}};
const Segment SGI368772[] = {{&S10262[8], 3}, {NULL}};
const Segment SGI368773[] = {{&S10262[13], 9}, {NULL}};
const Segment SGI368774[] = {{&S10262[33], 15}, {NULL}};
const Segment SGI368775[] = {{&S10262[106], 3}, {NULL}};
const Segment SGI368776[] = {{&S10262[49], 6}, {NULL}};
const Segment SGI368777[] = {{&S10262[33], 13}, {NULL}};
const Segment SGI368778[] = {{&S10263[69], 5}, {NULL}};
const Segment SGI368779[] = {{&S10263[8], 3}, {NULL}};
const Segment SGI368780[] = {{&S10263[13], 9}, {NULL}};
const Segment SGI368781[] = {{&S10263[33], 15}, {NULL}};
const Segment SGI368782[] = {{&S10263[103], 3}, {NULL}};
const Segment SGI368783[] = {{&S10263[49], 6}, {NULL}};
const Segment SGI368784[] = {{&S10263[33], 13}, {NULL}};
const Segment SGI368785[] = {{&S10264[69], 6}, {NULL}};
const Segment SGI368786[] = {{&S10264[8], 3}, {NULL}};
const Segment SGI368787[] = {{&S10264[13], 9}, {NULL}};
const Segment SGI368788[] = {{&S10264[33], 15}, {NULL}};
const Segment SGI368789[] = {{&S10264[104], 3}, {NULL}};
const Segment SGI368790[] = {{&S10264[49], 6}, {NULL}};
const Segment SGI368791[] = {{&S10264[33], 13}, {NULL}};
const Segment SGI368792[] = {{&S10265[66], 5}, {NULL}};
const Segment SGI368793[] = {{&S10265[8], 3}, {NULL}};
const Segment SGI368794[] = {{&S10265[13], 9}, {NULL}};
const Segment SGI368795[] = {{&S10265[33], 15}, {NULL}};
const Segment SGI368796[] = {{&S10265[100], 3}, {NULL}};
const Segment SGI368797[] = {{&S10265[49], 3}, {NULL}};
const Segment SGI368798[] = {{&S10265[33], 13}, {NULL}};
const Segment SGI368799[] = {{&S10266[69], 6}, {NULL}};
const Segment SGI368800[] = {{&S10266[8], 3}, {NULL}};
const Segment SGI368801[] = {{&S10266[13], 9}, {NULL}};
const Segment SGI368802[] = {{&S10266[33], 15}, {NULL}};
const Segment SGI368803[] = {{&S10266[104], 3}, {NULL}};
const Segment SGI368804[] = {{&S10266[49], 6}, {NULL}};
const Segment SGI368805[] = {{&S10266[33], 13}, {NULL}};
const Segment SGI368806[] = {{&S10267[69], 5}, {NULL}};
const Segment SGI368807[] = {{&S10267[8], 3}, {NULL}};
const Segment SGI368808[] = {{&S10267[13], 9}, {NULL}};
const Segment SGI368809[] = {{&S10267[33], 15}, {NULL}};
const Segment SGI368810[] = {{&S10267[103], 3}, {NULL}};
const Segment SGI368811[] = {{&S10267[49], 6}, {NULL}};
const Segment SGI368812[] = {{&S10267[33], 13}, {NULL}};
const Segment SGI368813[] = {{&S10268[66], 6}, {NULL}};
const Segment SGI368814[] = {{&S10268[8], 3}, {NULL}};
const Segment SGI368815[] = {{&S10268[13], 9}, {NULL}};
const Segment SGI368816[] = {{&S10268[33], 15}, {NULL}};
const Segment SGI368817[] = {{&S10268[101], 3}, {NULL}};
const Segment SGI368818[] = {{&S10268[49], 3}, {NULL}};
const Segment SGI368819[] = {{&S10268[33], 13}, {NULL}};
const Segment SGI368820[] = {{&S10269[69], 6}, {NULL}};
const Segment SGI368821[] = {{&S10269[8], 3}, {NULL}};
const Segment SGI368822[] = {{&S10269[13], 9}, {NULL}};
const Segment SGI368823[] = {{&S10269[33], 15}, {NULL}};
const Segment SGI368824[] = {{&S10269[106], 3}, {NULL}};
const Segment SGI368825[] = {{&S10269[49], 6}, {NULL}};
const Segment SGI368826[] = {{&S10269[33], 13}, {NULL}};
const Segment SGI368827[] = {{&S10270[69], 6}, {NULL}};
const Segment SGI368828[] = {{&S10270[8], 3}, {NULL}};
const Segment SGI368829[] = {{&S10270[13], 9}, {NULL}};
const Segment SGI368830[] = {{&S10270[33], 15}, {NULL}};
const Segment SGI368831[] = {{&S10270[106], 3}, {NULL}};
const Segment SGI368832[] = {{&S10270[49], 6}, {NULL}};
const Segment SGI368833[] = {{&S10270[33], 13}, {NULL}};
const Segment SGI368834[] = {{&S10271[117], 6}, {NULL}};
const Segment SGI368835[] = {{&S10271[8], 3}, {NULL}};
const Segment SGI368836[] = {{&S10271[13], 9}, {NULL}};
const Segment SGI368837[] = {{&S10271[33], 15}, {NULL}};
const Segment SGI368838[] = {{&S10271[106], 3}, {NULL}};
const Segment SGI368839[] = {{&S10271[49], 6}, {NULL}};
const Segment SGI368840[] = {{&S10271[33], 13}, {NULL}};
const Segment SGI368841[] = {{&S10272[69], 6}, {NULL}};
const Segment SGI368842[] = {{&S10272[8], 3}, {NULL}};
const Segment SGI368843[] = {{&S10272[13], 9}, {NULL}};
const Segment SGI368844[] = {{&S10272[33], 15}, {NULL}};
const Segment SGI368845[] = {{&S10272[106], 3}, {NULL}};
const Segment SGI368846[] = {{&S10272[49], 6}, {NULL}};
const Segment SGI368847[] = {{&S10272[33], 13}, {NULL}};
const Segment SGI368848[] = {{&S10273[69], 6}, {NULL}};
const Segment SGI368849[] = {{&S10273[8], 3}, {NULL}};
const Segment SGI368850[] = {{&S10273[13], 9}, {NULL}};
const Segment SGI368851[] = {{&S10273[33], 15}, {NULL}};
const Segment SGI368852[] = {{&S10273[106], 3}, {NULL}};
const Segment SGI368853[] = {{&S10273[49], 6}, {NULL}};
const Segment SGI368854[] = {{&S10273[33], 13}, {NULL}};
const Segment SGI368855[] = {{&S10274[69], 6}, {NULL}};
const Segment SGI368856[] = {{&S10274[8], 3}, {NULL}};
const Segment SGI368857[] = {{&S10274[13], 9}, {NULL}};
const Segment SGI368858[] = {{&S10274[33], 15}, {NULL}};
const Segment SGI368859[] = {{&S10274[49], 6}, {NULL}};
const Segment SGI368860[] = {{&S10274[33], 13}, {NULL}};
const Segment SGI368861[] = {{&S10275[69], 6}, {NULL}};
const Segment SGI368862[] = {{&S10275[8], 3}, {NULL}};
const Segment SGI368863[] = {{&S10275[13], 9}, {NULL}};
const Segment SGI368864[] = {{&S10275[33], 15}, {NULL}};
const Segment SGI368865[] = {{&S10275[106], 3}, {NULL}};
const Segment SGI368866[] = {{&S10275[49], 6}, {NULL}};
const Segment SGI368867[] = {{&S10275[33], 13}, {NULL}};
const Segment SGI368868[] = {{&S10276[69], 6}, {NULL}};
const Segment SGI368869[] = {{&S10276[8], 3}, {NULL}};
const Segment SGI368870[] = {{&S10276[13], 9}, {NULL}};
const Segment SGI368871[] = {{&S10276[33], 15}, {NULL}};
const Segment SGI368872[] = {{&S10276[104], 3}, {NULL}};
const Segment SGI368873[] = {{&S10276[49], 6}, {NULL}};
const Segment SGI368874[] = {{&S10276[33], 13}, {NULL}};
const Segment SGI368875[] = {{&S10277[69], 6}, {NULL}};
const Segment SGI368876[] = {{&S10277[8], 3}, {NULL}};
const Segment SGI368877[] = {{&S10277[13], 9}, {NULL}};
const Segment SGI368878[] = {{&S10277[33], 15}, {NULL}};
const Segment SGI368879[] = {{&S10277[106], 3}, {NULL}};
const Segment SGI368880[] = {{&S10277[49], 6}, {NULL}};
const Segment SGI368881[] = {{&S10277[33], 13}, {NULL}};
const Segment SGI368882[] = {{&S10278[66], 6}, {NULL}};
const Segment SGI368883[] = {{&S10278[8], 3}, {NULL}};
const Segment SGI368884[] = {{&S10278[13], 9}, {NULL}};
const Segment SGI368885[] = {{&S10278[33], 15}, {NULL}};
const Segment SGI368886[] = {{&S10278[101], 3}, {NULL}};
const Segment SGI368887[] = {{&S10278[49], 3}, {NULL}};
const Segment SGI368888[] = {{&S10278[33], 13}, {NULL}};
const Segment SGI368889[] = {{&S10279[66], 6}, {NULL}};
const Segment SGI368890[] = {{&S10279[8], 3}, {NULL}};
const Segment SGI368891[] = {{&S10279[13], 9}, {NULL}};
const Segment SGI368892[] = {{&S10279[33], 15}, {NULL}};
const Segment SGI368893[] = {{&S10279[103], 3}, {NULL}};
const Segment SGI368894[] = {{&S10279[49], 3}, {NULL}};
const Segment SGI368895[] = {{&S10279[33], 13}, {NULL}};
const Segment SGI368896[] = {{&S10280[67], 6}, {NULL}};
const Segment SGI368897[] = {{&S10280[8], 3}, {NULL}};
const Segment SGI368898[] = {{&S10280[13], 9}, {NULL}};
const Segment SGI368899[] = {{&S10280[31], 15}, {NULL}};
const Segment SGI368900[] = {{&S10280[102], 3}, {NULL}};
const Segment SGI368901[] = {{&S10280[47], 6}, {NULL}};
const Segment SGI368902[] = {{&S10280[31], 13}, {NULL}};
const Segment SGI368903[] = {{&S10281[69], 6}, {NULL}};
const Segment SGI368904[] = {{&S10281[8], 3}, {NULL}};
const Segment SGI368905[] = {{&S10281[13], 9}, {NULL}};
const Segment SGI368906[] = {{&S10281[33], 15}, {NULL}};
const Segment SGI368907[] = {{&S10281[106], 3}, {NULL}};
const Segment SGI368908[] = {{&S10281[49], 6}, {NULL}};
const Segment SGI368909[] = {{&S10281[33], 13}, {NULL}};
const Segment SGI368910[] = {{&S10282[69], 6}, {NULL}};
const Segment SGI368911[] = {{&S10282[8], 3}, {NULL}};
const Segment SGI368912[] = {{&S10282[13], 9}, {NULL}};
const Segment SGI368913[] = {{&S10282[33], 15}, {NULL}};
const Segment SGI368914[] = {{&S10282[104], 3}, {NULL}};
const Segment SGI368915[] = {{&S10282[49], 6}, {NULL}};
const Segment SGI368916[] = {{&S10282[33], 13}, {NULL}};
const Segment SGI368917[] = {{&S10283[69], 6}, {NULL}};
const Segment SGI368918[] = {{&S10283[8], 3}, {NULL}};
const Segment SGI368919[] = {{&S10283[13], 9}, {NULL}};
const Segment SGI368920[] = {{&S10283[33], 15}, {NULL}};
const Segment SGI368921[] = {{&S10283[104], 3}, {NULL}};
const Segment SGI368922[] = {{&S10283[49], 6}, {NULL}};
const Segment SGI368923[] = {{&S10283[33], 13}, {NULL}};
const Segment SGI368924[] = {{&S10284[69], 6}, {NULL}};
const Segment SGI368925[] = {{&S10284[8], 3}, {NULL}};
const Segment SGI368926[] = {{&S10284[13], 9}, {NULL}};
const Segment SGI368927[] = {{&S10284[33], 15}, {NULL}};
const Segment SGI368928[] = {{&S10284[104], 3}, {NULL}};
const Segment SGI368929[] = {{&S10284[49], 6}, {NULL}};
const Segment SGI368930[] = {{&S10284[33], 13}, {NULL}};
const Segment SGI368931[] = {{&S10285[69], 6}, {NULL}};
const Segment SGI368932[] = {{&S10285[8], 3}, {NULL}};
const Segment SGI368933[] = {{&S10285[13], 9}, {NULL}};
const Segment SGI368934[] = {{&S10285[33], 15}, {NULL}};
const Segment SGI368935[] = {{&S10285[106], 3}, {NULL}};
const Segment SGI368936[] = {{&S10285[49], 6}, {NULL}};
const Segment SGI368937[] = {{&S10285[33], 13}, {NULL}};
const Segment SGI368938[] = {{&S10286[69], 6}, {NULL}};
const Segment SGI368939[] = {{&S10286[8], 3}, {NULL}};
const Segment SGI368940[] = {{&S10286[13], 9}, {NULL}};
const Segment SGI368941[] = {{&S10286[33], 15}, {NULL}};
const Segment SGI368942[] = {{&S10286[104], 3}, {NULL}};
const Segment SGI368943[] = {{&S10286[49], 6}, {NULL}};
const Segment SGI368944[] = {{&S10286[33], 13}, {NULL}};
const Segment SGI368945[] = {{&S10287[69], 6}, {NULL}};
const Segment SGI368946[] = {{&S10287[8], 3}, {NULL}};
const Segment SGI368947[] = {{&S10287[13], 9}, {NULL}};
const Segment SGI368948[] = {{&S10287[33], 15}, {NULL}};
const Segment SGI368949[] = {{&S10287[105], 3}, {NULL}};
const Segment SGI368950[] = {{&S10287[49], 6}, {NULL}};
const Segment SGI368951[] = {{&S10287[33], 13}, {NULL}};
const Segment SGI368952[] = {{&S10288[115], 5}, {NULL}};
const Segment SGI368953[] = {{&S10288[8], 3}, {NULL}};
const Segment SGI368954[] = {{&S10288[13], 9}, {NULL}};
const Segment SGI368955[] = {{&S10288[33], 15}, {NULL}};
const Segment SGI368956[] = {{&S10288[104], 3}, {NULL}};
const Segment SGI368957[] = {{&S10288[49], 6}, {NULL}};
const Segment SGI368958[] = {{&S10288[33], 13}, {NULL}};
const Segment SGI368959[] = {{&S10289[114], 5}, {NULL}};
const Segment SGI368960[] = {{&S10289[8], 3}, {NULL}};
const Segment SGI368961[] = {{&S10289[13], 9}, {NULL}};
const Segment SGI368962[] = {{&S10289[33], 15}, {NULL}};
const Segment SGI368963[] = {{&S10289[103], 3}, {NULL}};
const Segment SGI368964[] = {{&S10289[49], 6}, {NULL}};
const Segment SGI368965[] = {{&S10289[33], 13}, {NULL}};
const Segment SGI368966[] = {{&S10291[69], 6}, {NULL}};
const Segment SGI368967[] = {{&S10291[8], 3}, {NULL}};
const Segment SGI368968[] = {{&S10291[13], 9}, {NULL}};
const Segment SGI368969[] = {{&S10291[33], 15}, {NULL}};
const Segment SGI368970[] = {{&S10291[106], 3}, {NULL}};
const Segment SGI368971[] = {{&S10291[49], 6}, {NULL}};
const Segment SGI368972[] = {{&S10291[33], 13}, {NULL}};
const Segment SGI368973[] = {{&S10292[66], 5}, {NULL}};
const Segment SGI368974[] = {{&S10292[8], 3}, {NULL}};
const Segment SGI368975[] = {{&S10292[13], 9}, {NULL}};
const Segment SGI368976[] = {{&S10292[33], 15}, {NULL}};
const Segment SGI368977[] = {{&S10292[100], 3}, {NULL}};
const Segment SGI368978[] = {{&S10292[49], 3}, {NULL}};
const Segment SGI368979[] = {{&S10292[33], 13}, {NULL}};
const Segment SGI368980[] = {{&S10293[69], 6}, {NULL}};
const Segment SGI368981[] = {{&S10293[8], 3}, {NULL}};
const Segment SGI368982[] = {{&S10293[13], 9}, {NULL}};
const Segment SGI368983[] = {{&S10293[33], 15}, {NULL}};
const Segment SGI368984[] = {{&S10293[104], 3}, {NULL}};
const Segment SGI368985[] = {{&S10293[49], 6}, {NULL}};
const Segment SGI368986[] = {{&S10293[33], 13}, {NULL}};
const Segment SGI368987[] = {{&S10295[69], 6}, {NULL}};
const Segment SGI368988[] = {{&S10295[8], 3}, {NULL}};
const Segment SGI368989[] = {{&S10295[13], 9}, {NULL}};
const Segment SGI368990[] = {{&S10295[33], 15}, {NULL}};
const Segment SGI368991[] = {{&S10295[104], 3}, {NULL}};
const Segment SGI368992[] = {{&S10295[49], 6}, {NULL}};
const Segment SGI368993[] = {{&S10295[33], 13}, {NULL}};
const Segment SGI368994[] = {{&S10296[69], 6}, {NULL}};
const Segment SGI368995[] = {{&S10296[8], 3}, {NULL}};
const Segment SGI368996[] = {{&S10296[13], 9}, {NULL}};
const Segment SGI368997[] = {{&S10296[33], 15}, {NULL}};
const Segment SGI368998[] = {{&S10296[104], 3}, {NULL}};
const Segment SGI368999[] = {{&S10296[49], 6}, {NULL}};
const Segment SGI369000[] = {{&S10296[33], 13}, {NULL}};
const Segment SGI369001[] = {{&S10297[69], 5}, {NULL}};
const Segment SGI369002[] = {{&S10297[8], 3}, {NULL}};
const Segment SGI369003[] = {{&S10297[13], 9}, {NULL}};
const Segment SGI369004[] = {{&S10297[33], 15}, {NULL}};
const Segment SGI369005[] = {{&S10297[103], 3}, {NULL}};
const Segment SGI369006[] = {{&S10297[49], 6}, {NULL}};
const Segment SGI369007[] = {{&S10297[33], 13}, {NULL}};
const Segment SGI369008[] = {{&S10298[69], 5}, {NULL}};
const Segment SGI369009[] = {{&S10298[8], 3}, {NULL}};
const Segment SGI369010[] = {{&S10298[13], 9}, {NULL}};
const Segment SGI369011[] = {{&S10298[33], 15}, {NULL}};
const Segment SGI369012[] = {{&S10298[103], 3}, {NULL}};
const Segment SGI369013[] = {{&S10298[49], 6}, {NULL}};
const Segment SGI369014[] = {{&S10298[33], 13}, {NULL}};
const Segment SGI369015[] = {{&S10299[115], 5}, {NULL}};
const Segment SGI369016[] = {{&S10299[8], 3}, {NULL}};
const Segment SGI369017[] = {{&S10299[13], 9}, {NULL}};
const Segment SGI369018[] = {{&S10299[33], 15}, {NULL}};
const Segment SGI369019[] = {{&S10299[104], 3}, {NULL}};
const Segment SGI369020[] = {{&S10299[49], 6}, {NULL}};
const Segment SGI369021[] = {{&S10299[33], 13}, {NULL}};
const Segment SGI369022[] = {{&S10300[69], 6}, {NULL}};
const Segment SGI369023[] = {{&S10300[8], 3}, {NULL}};
const Segment SGI369024[] = {{&S10300[13], 9}, {NULL}};
const Segment SGI369025[] = {{&S10300[33], 15}, {NULL}};
const Segment SGI369026[] = {{&S10300[106], 3}, {NULL}};
const Segment SGI369027[] = {{&S10300[49], 6}, {NULL}};
const Segment SGI369028[] = {{&S10300[33], 13}, {NULL}};
const Segment SGI369029[] = {{&S10301[69], 6}, {NULL}};
const Segment SGI369030[] = {{&S10301[8], 3}, {NULL}};
const Segment SGI369031[] = {{&S10301[13], 9}, {NULL}};
const Segment SGI369032[] = {{&S10301[33], 15}, {NULL}};
const Segment SGI369033[] = {{&S10301[106], 3}, {NULL}};
const Segment SGI369034[] = {{&S10301[49], 6}, {NULL}};
const Segment SGI369035[] = {{&S10301[33], 13}, {NULL}};
const Segment SGI369036[] = {{&S10302[69], 5}, {NULL}};
const Segment SGI369037[] = {{&S10302[8], 3}, {NULL}};
const Segment SGI369038[] = {{&S10302[13], 9}, {NULL}};
const Segment SGI369039[] = {{&S10302[33], 15}, {NULL}};
const Segment SGI369040[] = {{&S10302[103], 3}, {NULL}};
const Segment SGI369041[] = {{&S10302[49], 6}, {NULL}};
const Segment SGI369042[] = {{&S10302[33], 13}, {NULL}};
const Segment SGI369043[] = {{&S10303[69], 6}, {NULL}};
const Segment SGI369044[] = {{&S10303[8], 3}, {NULL}};
const Segment SGI369045[] = {{&S10303[13], 9}, {NULL}};
const Segment SGI369046[] = {{&S10303[33], 15}, {NULL}};
const Segment SGI369047[] = {{&S10303[106], 3}, {NULL}};
const Segment SGI369048[] = {{&S10303[49], 6}, {NULL}};
const Segment SGI369049[] = {{&S10303[33], 13}, {NULL}};
const Segment SGI369050[] = {{&S10304[69], 6}, {NULL}};
const Segment SGI369051[] = {{&S10304[8], 3}, {NULL}};
const Segment SGI369052[] = {{&S10304[13], 9}, {NULL}};
const Segment SGI369053[] = {{&S10304[33], 15}, {NULL}};
const Segment SGI369054[] = {{&S10304[106], 3}, {NULL}};
const Segment SGI369055[] = {{&S10304[49], 6}, {NULL}};
const Segment SGI369056[] = {{&S10304[33], 13}, {NULL}};
const Segment SGI369057[] = {{&S10305[69], 5}, {NULL}};
const Segment SGI369058[] = {{&S10305[8], 3}, {NULL}};
const Segment SGI369059[] = {{&S10305[13], 9}, {NULL}};
const Segment SGI369060[] = {{&S10305[33], 15}, {NULL}};
const Segment SGI369061[] = {{&S10305[103], 3}, {NULL}};
const Segment SGI369062[] = {{&S10305[49], 6}, {NULL}};
const Segment SGI369063[] = {{&S10305[33], 13}, {NULL}};
const Segment SGI369064[] = {{&S10306[69], 6}, {NULL}};
const Segment SGI369065[] = {{&S10306[8], 3}, {NULL}};
const Segment SGI369066[] = {{&S10306[13], 9}, {NULL}};
const Segment SGI369067[] = {{&S10306[33], 15}, {NULL}};
const Segment SGI369068[] = {{&S10306[104], 3}, {NULL}};
const Segment SGI369069[] = {{&S10306[49], 6}, {NULL}};
const Segment SGI369070[] = {{&S10306[33], 13}, {NULL}};
const Segment SGI369071[] = {{&S10307[69], 6}, {NULL}};
const Segment SGI369072[] = {{&S10307[8], 3}, {NULL}};
const Segment SGI369073[] = {{&S10307[13], 9}, {NULL}};
const Segment SGI369074[] = {{&S10307[33], 15}, {NULL}};
const Segment SGI369075[] = {{&S10307[106], 3}, {NULL}};
const Segment SGI369076[] = {{&S10307[49], 6}, {NULL}};
const Segment SGI369077[] = {{&S10307[33], 13}, {NULL}};
const Segment SGI369078[] = {{&S10308[69], 6}, {NULL}};
const Segment SGI369079[] = {{&S10308[8], 3}, {NULL}};
const Segment SGI369080[] = {{&S10308[13], 9}, {NULL}};
const Segment SGI369081[] = {{&S10308[33], 15}, {NULL}};
const Segment SGI369082[] = {{&S10308[106], 3}, {NULL}};
const Segment SGI369083[] = {{&S10308[49], 6}, {NULL}};
const Segment SGI369084[] = {{&S10308[33], 13}, {NULL}};
const Segment SGI369085[] = {{&S10309[69], 6}, {NULL}};
const Segment SGI369086[] = {{&S10309[8], 3}, {NULL}};
const Segment SGI369087[] = {{&S10309[13], 9}, {NULL}};
const Segment SGI369088[] = {{&S10309[33], 15}, {NULL}};
const Segment SGI369089[] = {{&S10309[106], 3}, {NULL}};
const Segment SGI369090[] = {{&S10309[49], 6}, {NULL}};
const Segment SGI369091[] = {{&S10309[33], 13}, {NULL}};
const Segment SGI369092[] = {{&S10310[66], 6}, {NULL}};
const Segment SGI369093[] = {{&S10310[8], 3}, {NULL}};
const Segment SGI369094[] = {{&S10310[13], 9}, {NULL}};
const Segment SGI369095[] = {{&S10310[33], 15}, {NULL}};
const Segment SGI369096[] = {{&S10310[103], 3}, {NULL}};
const Segment SGI369097[] = {{&S10310[49], 3}, {NULL}};
const Segment SGI369098[] = {{&S10310[33], 13}, {NULL}};
const Segment SGI369099[] = {{&S10311[69], 5}, {NULL}};
const Segment SGI369100[] = {{&S10311[8], 3}, {NULL}};
const Segment SGI369101[] = {{&S10311[13], 9}, {NULL}};
const Segment SGI369102[] = {{&S10311[33], 15}, {NULL}};
const Segment SGI369103[] = {{&S10311[103], 3}, {NULL}};
const Segment SGI369104[] = {{&S10311[49], 6}, {NULL}};
const Segment SGI369105[] = {{&S10311[33], 13}, {NULL}};
const Segment SGI369106[] = {{&S10312[67], 5}, {NULL}};
const Segment SGI369107[] = {{&S10312[8], 3}, {NULL}};
const Segment SGI369108[] = {{&S10312[13], 9}, {NULL}};
const Segment SGI369109[] = {{&S10312[31], 15}, {NULL}};
const Segment SGI369110[] = {{&S10312[101], 3}, {NULL}};
const Segment SGI369111[] = {{&S10312[47], 6}, {NULL}};
const Segment SGI369112[] = {{&S10312[31], 13}, {NULL}};
const Segment SGI369113[] = {{&S10313[69], 6}, {NULL}};
const Segment SGI369114[] = {{&S10313[8], 3}, {NULL}};
const Segment SGI369115[] = {{&S10313[13], 9}, {NULL}};
const Segment SGI369116[] = {{&S10313[33], 15}, {NULL}};
const Segment SGI369117[] = {{&S10313[106], 3}, {NULL}};
const Segment SGI369118[] = {{&S10313[49], 6}, {NULL}};
const Segment SGI369119[] = {{&S10313[33], 13}, {NULL}};
const Segment SGI369120[] = {{&S10315[67], 6}, {NULL}};
const Segment SGI369121[] = {{&S10315[8], 3}, {NULL}};
const Segment SGI369122[] = {{&S10315[13], 9}, {NULL}};
const Segment SGI369123[] = {{&S10315[31], 15}, {NULL}};
const Segment SGI369124[] = {{&S10315[104], 3}, {NULL}};
const Segment SGI369125[] = {{&S10315[47], 6}, {NULL}};
const Segment SGI369126[] = {{&S10315[31], 13}, {NULL}};
const Segment SGI369127[] = {{&S10316[69], 6}, {NULL}};
const Segment SGI369128[] = {{&S10316[8], 3}, {NULL}};
const Segment SGI369129[] = {{&S10316[13], 9}, {NULL}};
const Segment SGI369130[] = {{&S10316[33], 15}, {NULL}};
const Segment SGI369131[] = {{&S10316[104], 3}, {NULL}};
const Segment SGI369132[] = {{&S10316[49], 6}, {NULL}};
const Segment SGI369133[] = {{&S10316[33], 13}, {NULL}};
const Segment SGI369134[] = {{&S10317[67], 5}, {NULL}};
const Segment SGI369135[] = {{&S10317[8], 3}, {NULL}};
const Segment SGI369136[] = {{&S10317[13], 9}, {NULL}};
const Segment SGI369137[] = {{&S10317[31], 15}, {NULL}};
const Segment SGI369138[] = {{&S10317[101], 3}, {NULL}};
const Segment SGI369139[] = {{&S10317[47], 6}, {NULL}};
const Segment SGI369140[] = {{&S10317[31], 13}, {NULL}};
const Segment SGI369141[] = {{&S10319[69], 6}, {NULL}};
const Segment SGI369142[] = {{&S10319[8], 3}, {NULL}};
const Segment SGI369143[] = {{&S10319[13], 9}, {NULL}};
const Segment SGI369144[] = {{&S10319[33], 15}, {NULL}};
const Segment SGI369145[] = {{&S10319[106], 3}, {NULL}};
const Segment SGI369146[] = {{&S10319[49], 6}, {NULL}};
const Segment SGI369147[] = {{&S10319[33], 13}, {NULL}};
const Segment SGI369148[] = {{&S10505[69], 6}, {NULL}};
const Segment SGI369149[] = {{&S10505[8], 3}, {NULL}};
const Segment SGI369150[] = {{&S10505[13], 9}, {NULL}};
const Segment SGI369151[] = {{&S10505[33], 15}, {NULL}};
const Segment SGI369152[] = {{&S10505[106], 3}, {NULL}};
const Segment SGI369153[] = {{&S10505[49], 6}, {NULL}};
const Segment SGI369154[] = {{&S10505[33], 13}, {NULL}};
const Segment SGI369155[] = {{&S10506[69], 6}, {NULL}};
const Segment SGI369156[] = {{&S10506[8], 3}, {NULL}};
const Segment SGI369157[] = {{&S10506[13], 9}, {NULL}};
const Segment SGI369158[] = {{&S10506[33], 15}, {NULL}};
const Segment SGI369159[] = {{&S10506[106], 3}, {NULL}};
const Segment SGI369160[] = {{&S10506[49], 6}, {NULL}};
const Segment SGI369161[] = {{&S10506[33], 13}, {NULL}};
const Segment SGI369162[] = {{&S10507[69], 6}, {NULL}};
const Segment SGI369163[] = {{&S10507[8], 3}, {NULL}};
const Segment SGI369164[] = {{&S10507[13], 9}, {NULL}};
const Segment SGI369165[] = {{&S10507[33], 15}, {NULL}};
const Segment SGI369166[] = {{&S10507[106], 3}, {NULL}};
const Segment SGI369167[] = {{&S10507[49], 6}, {NULL}};
const Segment SGI369168[] = {{&S10507[33], 13}, {NULL}};
const Segment SGI369169[] = {{&S10508[69], 6}, {NULL}};
const Segment SGI369170[] = {{&S10508[8], 3}, {NULL}};
const Segment SGI369171[] = {{&S10508[13], 9}, {NULL}};
const Segment SGI369172[] = {{&S10508[33], 15}, {NULL}};
const Segment SGI369173[] = {{&S10508[106], 3}, {NULL}};
const Segment SGI369174[] = {{&S10508[49], 6}, {NULL}};
const Segment SGI369175[] = {{&S10508[33], 13}, {NULL}};
const Segment SGI369176[] = {{&S10509[69], 5}, {NULL}};
const Segment SGI369177[] = {{&S10509[8], 3}, {NULL}};
const Segment SGI369178[] = {{&S10509[13], 9}, {NULL}};
const Segment SGI369179[] = {{&S10509[33], 15}, {NULL}};
const Segment SGI369180[] = {{&S10509[103], 3}, {NULL}};
const Segment SGI369181[] = {{&S10509[49], 6}, {NULL}};
const Segment SGI369182[] = {{&S10509[33], 13}, {NULL}};
const Segment SGI369183[] = {{&S10510[69], 6}, {NULL}};
const Segment SGI369184[] = {{&S10510[8], 3}, {NULL}};
const Segment SGI369185[] = {{&S10510[13], 9}, {NULL}};
const Segment SGI369186[] = {{&S10510[33], 15}, {NULL}};
const Segment SGI369187[] = {{&S10510[106], 3}, {NULL}};
const Segment SGI369188[] = {{&S10510[49], 6}, {NULL}};
const Segment SGI369189[] = {{&S10510[33], 13}, {NULL}};
const Segment SGI369190[] = {{&S10511[69], 5}, {NULL}};
const Segment SGI369191[] = {{&S10511[8], 3}, {NULL}};
const Segment SGI369192[] = {{&S10511[13], 9}, {NULL}};
const Segment SGI369193[] = {{&S10511[33], 15}, {NULL}};
const Segment SGI369194[] = {{&S10511[103], 3}, {NULL}};
const Segment SGI369195[] = {{&S10511[49], 6}, {NULL}};
const Segment SGI369196[] = {{&S10511[33], 13}, {NULL}};
const Segment SGI369197[] = {{&S10512[69], 6}, {NULL}};
const Segment SGI369198[] = {{&S10512[8], 3}, {NULL}};
const Segment SGI369199[] = {{&S10512[13], 9}, {NULL}};
const Segment SGI369200[] = {{&S10512[33], 15}, {NULL}};
const Segment SGI369201[] = {{&S10512[106], 3}, {NULL}};
const Segment SGI369202[] = {{&S10512[49], 6}, {NULL}};
const Segment SGI369203[] = {{&S10512[33], 13}, {NULL}};
const Segment SGI369204[] = {{&S10513[69], 6}, {NULL}};
const Segment SGI369205[] = {{&S10513[8], 3}, {NULL}};
const Segment SGI369206[] = {{&S10513[13], 9}, {NULL}};
const Segment SGI369207[] = {{&S10513[33], 15}, {NULL}};
const Segment SGI369208[] = {{&S10513[106], 3}, {NULL}};
const Segment SGI369209[] = {{&S10513[49], 6}, {NULL}};
const Segment SGI369210[] = {{&S10513[33], 13}, {NULL}};
const Segment SGI369211[] = {{&S10514[69], 6}, {NULL}};
const Segment SGI369212[] = {{&S10514[8], 3}, {NULL}};
const Segment SGI369213[] = {{&S10514[13], 9}, {NULL}};
const Segment SGI369214[] = {{&S10514[33], 15}, {NULL}};
const Segment SGI369215[] = {{&S10514[106], 3}, {NULL}};
const Segment SGI369216[] = {{&S10514[49], 6}, {NULL}};
const Segment SGI369217[] = {{&S10514[33], 13}, {NULL}};
const Segment SGI369218[] = {{&S10515[69], 6}, {NULL}};
const Segment SGI369219[] = {{&S10515[8], 3}, {NULL}};
const Segment SGI369220[] = {{&S10515[13], 9}, {NULL}};
const Segment SGI369221[] = {{&S10515[33], 15}, {NULL}};
const Segment SGI369222[] = {{&S10515[106], 3}, {NULL}};
const Segment SGI369223[] = {{&S10515[49], 6}, {NULL}};
const Segment SGI369224[] = {{&S10515[33], 13}, {NULL}};
const Segment SGI369225[] = {{&S10516[69], 6}, {NULL}};
const Segment SGI369226[] = {{&S10516[8], 3}, {NULL}};
const Segment SGI369227[] = {{&S10516[13], 9}, {NULL}};
const Segment SGI369228[] = {{&S10516[33], 15}, {NULL}};
const Segment SGI369229[] = {{&S10516[106], 3}, {NULL}};
const Segment SGI369230[] = {{&S10516[49], 6}, {NULL}};
const Segment SGI369231[] = {{&S10516[33], 13}, {NULL}};
const Segment SGI369232[] = {{&S10517[69], 6}, {NULL}};
const Segment SGI369233[] = {{&S10517[8], 3}, {NULL}};
const Segment SGI369234[] = {{&S10517[13], 9}, {NULL}};
const Segment SGI369235[] = {{&S10517[33], 15}, {NULL}};
const Segment SGI369236[] = {{&S10517[107], 3}, {NULL}};
const Segment SGI369237[] = {{&S10517[49], 6}, {NULL}};
const Segment SGI369238[] = {{&S10517[33], 13}, {NULL}};
const Segment SGI369239[] = {{&S10518[69], 6}, {NULL}};
const Segment SGI369240[] = {{&S10518[8], 3}, {NULL}};
const Segment SGI369241[] = {{&S10518[13], 9}, {NULL}};
const Segment SGI369242[] = {{&S10518[33], 15}, {NULL}};
const Segment SGI369243[] = {{&S10518[106], 3}, {NULL}};
const Segment SGI369244[] = {{&S10518[49], 6}, {NULL}};
const Segment SGI369245[] = {{&S10518[33], 13}, {NULL}};
const Segment SGI369246[] = {{&S10519[69], 5}, {NULL}};
const Segment SGI369247[] = {{&S10519[8], 3}, {NULL}};
const Segment SGI369248[] = {{&S10519[13], 9}, {NULL}};
const Segment SGI369249[] = {{&S10519[33], 15}, {NULL}};
const Segment SGI369250[] = {{&S10519[103], 3}, {NULL}};
const Segment SGI369251[] = {{&S10519[49], 6}, {NULL}};
const Segment SGI369252[] = {{&S10519[33], 13}, {NULL}};
const Segment SGI369253[] = {{&S10520[69], 6}, {NULL}};
const Segment SGI369254[] = {{&S10520[8], 3}, {NULL}};
const Segment SGI369255[] = {{&S10520[13], 9}, {NULL}};
const Segment SGI369256[] = {{&S10520[33], 15}, {NULL}};
const Segment SGI369257[] = {{&S10520[106], 3}, {NULL}};
const Segment SGI369258[] = {{&S10520[49], 6}, {NULL}};
const Segment SGI369259[] = {{&S10520[33], 13}, {NULL}};
const Segment SGI369260[] = {{&S10521[69], 6}, {NULL}};
const Segment SGI369261[] = {{&S10521[8], 3}, {NULL}};
const Segment SGI369262[] = {{&S10521[13], 9}, {NULL}};
const Segment SGI369263[] = {{&S10521[33], 15}, {NULL}};
const Segment SGI369264[] = {{&S10521[106], 3}, {NULL}};
const Segment SGI369265[] = {{&S10521[49], 6}, {NULL}};
const Segment SGI369266[] = {{&S10521[33], 13}, {NULL}};
const Segment SGI369267[] = {{&S10522[69], 6}, {NULL}};
const Segment SGI369268[] = {{&S10522[8], 3}, {NULL}};
const Segment SGI369269[] = {{&S10522[13], 9}, {NULL}};
const Segment SGI369270[] = {{&S10522[33], 15}, {NULL}};
const Segment SGI369271[] = {{&S10522[106], 3}, {NULL}};
const Segment SGI369272[] = {{&S10522[49], 6}, {NULL}};
const Segment SGI369273[] = {{&S10522[33], 13}, {NULL}};
const Segment SGI369274[] = {{&S10523[69], 6}, {NULL}};
const Segment SGI369275[] = {{&S10523[8], 3}, {NULL}};
const Segment SGI369276[] = {{&S10523[13], 9}, {NULL}};
const Segment SGI369277[] = {{&S10523[33], 15}, {NULL}};
const Segment SGI369278[] = {{&S10523[106], 3}, {NULL}};
const Segment SGI369279[] = {{&S10523[49], 6}, {NULL}};
const Segment SGI369280[] = {{&S10523[33], 13}, {NULL}};
const Segment SGI369281[] = {{&S10524[69], 6}, {NULL}};
const Segment SGI369282[] = {{&S10524[8], 3}, {NULL}};
const Segment SGI369283[] = {{&S10524[13], 9}, {NULL}};
const Segment SGI369284[] = {{&S10524[33], 15}, {NULL}};
const Segment SGI369285[] = {{&S10524[107], 3}, {NULL}};
const Segment SGI369286[] = {{&S10524[49], 6}, {NULL}};
const Segment SGI369287[] = {{&S10524[33], 13}, {NULL}};
const Segment SGI369288[] = {{&S10525[113], 6}, {NULL}};
const Segment SGI369289[] = {{&S10525[8], 3}, {NULL}};
const Segment SGI369290[] = {{&S10525[13], 9}, {NULL}};
const Segment SGI369291[] = {{&S10525[33], 13}, {NULL}};
const Segment SGI369292[] = {{&S10525[102], 3}, {NULL}};
const Segment SGI369293[] = {{&S10526[69], 6}, {NULL}};
const Segment SGI369294[] = {{&S10526[8], 3}, {NULL}};
const Segment SGI369295[] = {{&S10526[13], 9}, {NULL}};
const Segment SGI369296[] = {{&S10526[33], 15}, {NULL}};
const Segment SGI369297[] = {{&S10526[107], 3}, {NULL}};
const Segment SGI369298[] = {{&S10526[49], 6}, {NULL}};
const Segment SGI369299[] = {{&S10526[33], 13}, {NULL}};
const Segment SGI369300[] = {{&S10527[69], 5}, {NULL}};
const Segment SGI369301[] = {{&S10527[8], 3}, {NULL}};
const Segment SGI369302[] = {{&S10527[13], 9}, {NULL}};
const Segment SGI369303[] = {{&S10527[33], 15}, {NULL}};
const Segment SGI369304[] = {{&S10527[103], 3}, {NULL}};
const Segment SGI369305[] = {{&S10527[49], 6}, {NULL}};
const Segment SGI369306[] = {{&S10527[33], 13}, {NULL}};
const Segment SGI369307[] = {{&S10528[69], 5}, {NULL}};
const Segment SGI369308[] = {{&S10528[8], 3}, {NULL}};
const Segment SGI369309[] = {{&S10528[13], 9}, {NULL}};
const Segment SGI369310[] = {{&S10528[33], 15}, {NULL}};
const Segment SGI369311[] = {{&S10528[103], 3}, {NULL}};
const Segment SGI369312[] = {{&S10528[49], 6}, {NULL}};
const Segment SGI369313[] = {{&S10528[33], 13}, {NULL}};
const Segment SGI369314[] = {{&S10529[69], 6}, {NULL}};
const Segment SGI369315[] = {{&S10529[8], 3}, {NULL}};
const Segment SGI369316[] = {{&S10529[13], 9}, {NULL}};
const Segment SGI369317[] = {{&S10529[33], 15}, {NULL}};
const Segment SGI369318[] = {{&S10529[106], 3}, {NULL}};
const Segment SGI369319[] = {{&S10529[49], 6}, {NULL}};
const Segment SGI369320[] = {{&S10529[33], 13}, {NULL}};
const Segment SGI369321[] = {{&S10530[69], 5}, {NULL}};
const Segment SGI369322[] = {{&S10530[8], 3}, {NULL}};
const Segment SGI369323[] = {{&S10530[13], 9}, {NULL}};
const Segment SGI369324[] = {{&S10530[33], 15}, {NULL}};
const Segment SGI369325[] = {{&S10530[103], 3}, {NULL}};
const Segment SGI369326[] = {{&S10530[49], 6}, {NULL}};
const Segment SGI369327[] = {{&S10530[33], 13}, {NULL}};
const Segment SGI369328[] = {{&S10531[69], 6}, {NULL}};
const Segment SGI369329[] = {{&S10531[8], 3}, {NULL}};
const Segment SGI369330[] = {{&S10531[13], 9}, {NULL}};
const Segment SGI369331[] = {{&S10531[33], 15}, {NULL}};
const Segment SGI369332[] = {{&S10531[107], 3}, {NULL}};
const Segment SGI369333[] = {{&S10531[49], 6}, {NULL}};
const Segment SGI369334[] = {{&S10531[33], 13}, {NULL}};
const Segment SGI369335[] = {{&S10532[69], 5}, {NULL}};
const Segment SGI369336[] = {{&S10532[8], 3}, {NULL}};
const Segment SGI369337[] = {{&S10532[13], 9}, {NULL}};
const Segment SGI369338[] = {{&S10532[33], 15}, {NULL}};
const Segment SGI369339[] = {{&S10532[103], 3}, {NULL}};
const Segment SGI369340[] = {{&S10532[49], 6}, {NULL}};
const Segment SGI369341[] = {{&S10532[33], 13}, {NULL}};
const Segment SGI369342[] = {{&S10533[69], 6}, {NULL}};
const Segment SGI369343[] = {{&S10533[8], 3}, {NULL}};
const Segment SGI369344[] = {{&S10533[13], 9}, {NULL}};
const Segment SGI369345[] = {{&S10533[33], 15}, {NULL}};
const Segment SGI369346[] = {{&S10533[106], 3}, {NULL}};
const Segment SGI369347[] = {{&S10533[49], 6}, {NULL}};
const Segment SGI369348[] = {{&S10533[33], 13}, {NULL}};
const Segment SGI369349[] = {{&S10534[69], 5}, {NULL}};
const Segment SGI369350[] = {{&S10534[8], 3}, {NULL}};
const Segment SGI369351[] = {{&S10534[13], 9}, {NULL}};
const Segment SGI369352[] = {{&S10534[33], 15}, {NULL}};
const Segment SGI369353[] = {{&S10534[103], 3}, {NULL}};
const Segment SGI369354[] = {{&S10534[49], 6}, {NULL}};
const Segment SGI369355[] = {{&S10534[33], 13}, {NULL}};
const Segment SGI369356[] = {{&S10535[69], 6}, {NULL}};
const Segment SGI369357[] = {{&S10535[8], 3}, {NULL}};
const Segment SGI369358[] = {{&S10535[13], 9}, {NULL}};
const Segment SGI369359[] = {{&S10535[33], 15}, {NULL}};
const Segment SGI369360[] = {{&S10535[107], 3}, {NULL}};
const Segment SGI369361[] = {{&S10535[49], 6}, {NULL}};
const Segment SGI369362[] = {{&S10535[33], 13}, {NULL}};
const Segment SGI369363[] = {{&S10536[69], 6}, {NULL}};
const Segment SGI369364[] = {{&S10536[8], 3}, {NULL}};
const Segment SGI369365[] = {{&S10536[13], 9}, {NULL}};
const Segment SGI369366[] = {{&S10536[33], 15}, {NULL}};
const Segment SGI369367[] = {{&S10536[106], 3}, {NULL}};
const Segment SGI369368[] = {{&S10536[49], 6}, {NULL}};
const Segment SGI369369[] = {{&S10536[33], 13}, {NULL}};
const Segment SGI369370[] = {{&S10537[67], 5}, {NULL}};
const Segment SGI369371[] = {{&S10537[8], 3}, {NULL}};
const Segment SGI369372[] = {{&S10537[13], 9}, {NULL}};
const Segment SGI369373[] = {{&S10537[33], 13}, {NULL}};
const Segment SGI369374[] = {{&S10537[101], 3}, {NULL}};
const Segment SGI369375[] = {{&S10538[69], 6}, {NULL}};
const Segment SGI369376[] = {{&S10538[8], 3}, {NULL}};
const Segment SGI369377[] = {{&S10538[13], 9}, {NULL}};
const Segment SGI369378[] = {{&S10538[33], 15}, {NULL}};
const Segment SGI369379[] = {{&S10538[106], 3}, {NULL}};
const Segment SGI369380[] = {{&S10538[49], 6}, {NULL}};
const Segment SGI369381[] = {{&S10538[33], 13}, {NULL}};
const Segment SGI369382[] = {{&S10539[69], 5}, {NULL}};
const Segment SGI369383[] = {{&S10539[8], 3}, {NULL}};
const Segment SGI369384[] = {{&S10539[13], 9}, {NULL}};
const Segment SGI369385[] = {{&S10539[33], 15}, {NULL}};
const Segment SGI369386[] = {{&S10539[103], 3}, {NULL}};
const Segment SGI369387[] = {{&S10539[49], 6}, {NULL}};
const Segment SGI369388[] = {{&S10539[33], 13}, {NULL}};
const Segment SGI369389[] = {{&S10540[69], 6}, {NULL}};
const Segment SGI369390[] = {{&S10540[8], 3}, {NULL}};
const Segment SGI369391[] = {{&S10540[13], 9}, {NULL}};
const Segment SGI369392[] = {{&S10540[33], 15}, {NULL}};
const Segment SGI369393[] = {{&S10540[107], 3}, {NULL}};
const Segment SGI369394[] = {{&S10540[49], 6}, {NULL}};
const Segment SGI369395[] = {{&S10540[33], 13}, {NULL}};
const Segment SGI369396[] = {{&S10541[69], 6}, {NULL}};
const Segment SGI369397[] = {{&S10541[8], 3}, {NULL}};
const Segment SGI369398[] = {{&S10541[13], 9}, {NULL}};
const Segment SGI369399[] = {{&S10541[33], 15}, {NULL}};
const Segment SGI369400[] = {{&S10541[107], 3}, {NULL}};
const Segment SGI369401[] = {{&S10541[49], 6}, {NULL}};
const Segment SGI369402[] = {{&S10541[33], 13}, {NULL}};
const Segment SGI369403[] = {{&S10542[69], 6}, {NULL}};
const Segment SGI369404[] = {{&S10542[8], 3}, {NULL}};
const Segment SGI369405[] = {{&S10542[13], 9}, {NULL}};
const Segment SGI369406[] = {{&S10542[33], 15}, {NULL}};
const Segment SGI369407[] = {{&S10542[106], 3}, {NULL}};
const Segment SGI369408[] = {{&S10542[49], 6}, {NULL}};
const Segment SGI369409[] = {{&S10542[33], 13}, {NULL}};
const Segment SGI369410[] = {{&S10543[69], 5}, {NULL}};
const Segment SGI369411[] = {{&S10543[8], 3}, {NULL}};
const Segment SGI369412[] = {{&S10543[13], 9}, {NULL}};
const Segment SGI369413[] = {{&S10543[33], 15}, {NULL}};
const Segment SGI369414[] = {{&S10543[103], 3}, {NULL}};
const Segment SGI369415[] = {{&S10543[49], 6}, {NULL}};
const Segment SGI369416[] = {{&S10543[33], 13}, {NULL}};
const Segment SGI369417[] = {{&S10544[69], 6}, {NULL}};
const Segment SGI369418[] = {{&S10544[8], 3}, {NULL}};
const Segment SGI369419[] = {{&S10544[13], 9}, {NULL}};
const Segment SGI369420[] = {{&S10544[33], 15}, {NULL}};
const Segment SGI369421[] = {{&S10544[107], 3}, {NULL}};
const Segment SGI369422[] = {{&S10544[49], 6}, {NULL}};
const Segment SGI369423[] = {{&S10544[33], 13}, {NULL}};
const Segment SGI369424[] = {{&S10545[69], 6}, {NULL}};
const Segment SGI369425[] = {{&S10545[8], 3}, {NULL}};
const Segment SGI369426[] = {{&S10545[13], 9}, {NULL}};
const Segment SGI369427[] = {{&S10545[33], 15}, {NULL}};
const Segment SGI369428[] = {{&S10545[106], 3}, {NULL}};
const Segment SGI369429[] = {{&S10545[49], 6}, {NULL}};
const Segment SGI369430[] = {{&S10545[33], 13}, {NULL}};
const Segment SGI369431[] = {{&S10546[69], 6}, {NULL}};
const Segment SGI369432[] = {{&S10546[8], 3}, {NULL}};
const Segment SGI369433[] = {{&S10546[13], 9}, {NULL}};
const Segment SGI369434[] = {{&S10546[33], 15}, {NULL}};
const Segment SGI369435[] = {{&S10546[107], 3}, {NULL}};
const Segment SGI369436[] = {{&S10546[49], 6}, {NULL}};
const Segment SGI369437[] = {{&S10546[33], 13}, {NULL}};
const Segment SGI369438[] = {{&S10547[69], 6}, {NULL}};
const Segment SGI369439[] = {{&S10547[8], 3}, {NULL}};
const Segment SGI369440[] = {{&S10547[13], 9}, {NULL}};
const Segment SGI369441[] = {{&S10547[33], 15}, {NULL}};
const Segment SGI369442[] = {{&S10547[106], 3}, {NULL}};
const Segment SGI369443[] = {{&S10547[49], 6}, {NULL}};
const Segment SGI369444[] = {{&S10547[33], 13}, {NULL}};
const Segment SGI369445[] = {{&S10548[69], 6}, {NULL}};
const Segment SGI369446[] = {{&S10548[8], 3}, {NULL}};
const Segment SGI369447[] = {{&S10548[13], 9}, {NULL}};
const Segment SGI369448[] = {{&S10548[33], 15}, {NULL}};
const Segment SGI369449[] = {{&S10548[107], 3}, {NULL}};
const Segment SGI369450[] = {{&S10548[49], 6}, {NULL}};
const Segment SGI369451[] = {{&S10548[33], 13}, {NULL}};
const Segment SGI369452[] = {{&S10549[69], 6}, {NULL}};
const Segment SGI369453[] = {{&S10549[8], 3}, {NULL}};
const Segment SGI369454[] = {{&S10549[13], 9}, {NULL}};
const Segment SGI369455[] = {{&S10549[33], 15}, {NULL}};
const Segment SGI369456[] = {{&S10549[106], 3}, {NULL}};
const Segment SGI369457[] = {{&S10549[49], 6}, {NULL}};
const Segment SGI369458[] = {{&S10549[33], 13}, {NULL}};
const Segment SGI369459[] = {{&S10550[69], 6}, {NULL}};
const Segment SGI369460[] = {{&S10550[8], 3}, {NULL}};
const Segment SGI369461[] = {{&S10550[13], 9}, {NULL}};
const Segment SGI369462[] = {{&S10550[33], 15}, {NULL}};
const Segment SGI369463[] = {{&S10550[106], 3}, {NULL}};
const Segment SGI369464[] = {{&S10550[49], 6}, {NULL}};
const Segment SGI369465[] = {{&S10550[33], 13}, {NULL}};
const Segment SGI369466[] = {{&S10551[69], 6}, {NULL}};
const Segment SGI369467[] = {{&S10551[8], 3}, {NULL}};
const Segment SGI369468[] = {{&S10551[13], 9}, {NULL}};
const Segment SGI369469[] = {{&S10551[33], 15}, {NULL}};
const Segment SGI369470[] = {{&S10551[107], 3}, {NULL}};
const Segment SGI369471[] = {{&S10551[49], 6}, {NULL}};
const Segment SGI369472[] = {{&S10551[33], 13}, {NULL}};
const Segment SGI369473[] = {{&S10552[69], 6}, {NULL}};
const Segment SGI369474[] = {{&S10552[8], 3}, {NULL}};
const Segment SGI369475[] = {{&S10552[13], 9}, {NULL}};
const Segment SGI369476[] = {{&S10552[33], 15}, {NULL}};
const Segment SGI369477[] = {{&S10552[106], 3}, {NULL}};
const Segment SGI369478[] = {{&S10552[49], 6}, {NULL}};
const Segment SGI369479[] = {{&S10552[33], 13}, {NULL}};
const Segment SGI369480[] = {{&S10553[69], 6}, {NULL}};
const Segment SGI369481[] = {{&S10553[8], 3}, {NULL}};
const Segment SGI369482[] = {{&S10553[13], 9}, {NULL}};
const Segment SGI369483[] = {{&S10553[33], 15}, {NULL}};
const Segment SGI369484[] = {{&S10553[106], 3}, {NULL}};
const Segment SGI369485[] = {{&S10553[49], 6}, {NULL}};
const Segment SGI369486[] = {{&S10553[33], 13}, {NULL}};
const Segment SGI369487[] = {{&S10554[69], 5}, {NULL}};
const Segment SGI369488[] = {{&S10554[8], 3}, {NULL}};
const Segment SGI369489[] = {{&S10554[13], 9}, {NULL}};
const Segment SGI369490[] = {{&S10554[33], 15}, {NULL}};
const Segment SGI369491[] = {{&S10554[103], 3}, {NULL}};
const Segment SGI369492[] = {{&S10554[49], 6}, {NULL}};
const Segment SGI369493[] = {{&S10554[33], 13}, {NULL}};
const Segment SGI369494[] = {{&S10555[69], 6}, {NULL}};
const Segment SGI369495[] = {{&S10555[8], 3}, {NULL}};
const Segment SGI369496[] = {{&S10555[13], 9}, {NULL}};
const Segment SGI369497[] = {{&S10555[33], 15}, {NULL}};
const Segment SGI369498[] = {{&S10555[106], 3}, {NULL}};
const Segment SGI369499[] = {{&S10555[49], 6}, {NULL}};
const Segment SGI369500[] = {{&S10555[33], 13}, {NULL}};
const Segment SGI369501[] = {{&S10556[69], 5}, {NULL}};
const Segment SGI369502[] = {{&S10556[8], 3}, {NULL}};
const Segment SGI369503[] = {{&S10556[13], 9}, {NULL}};
const Segment SGI369504[] = {{&S10556[33], 15}, {NULL}};
const Segment SGI369505[] = {{&S10556[103], 3}, {NULL}};
const Segment SGI369506[] = {{&S10556[49], 6}, {NULL}};
const Segment SGI369507[] = {{&S10556[33], 13}, {NULL}};
const Segment SGI369508[] = {{&S10557[69], 5}, {NULL}};
const Segment SGI369509[] = {{&S10557[8], 3}, {NULL}};
const Segment SGI369510[] = {{&S10557[13], 9}, {NULL}};
const Segment SGI369511[] = {{&S10557[33], 15}, {NULL}};
const Segment SGI369512[] = {{&S10557[103], 3}, {NULL}};
const Segment SGI369513[] = {{&S10557[49], 6}, {NULL}};
const Segment SGI369514[] = {{&S10557[33], 13}, {NULL}};
const Segment SGI369515[] = {{&S10558[69], 6}, {NULL}};
const Segment SGI369516[] = {{&S10558[8], 3}, {NULL}};
const Segment SGI369517[] = {{&S10558[13], 9}, {NULL}};
const Segment SGI369518[] = {{&S10558[33], 15}, {NULL}};
const Segment SGI369519[] = {{&S10558[107], 3}, {NULL}};
const Segment SGI369520[] = {{&S10558[49], 6}, {NULL}};
const Segment SGI369521[] = {{&S10558[33], 13}, {NULL}};
const Segment SGI369522[] = {{&S10559[69], 5}, {NULL}};
const Segment SGI369523[] = {{&S10559[8], 3}, {NULL}};
const Segment SGI369524[] = {{&S10559[13], 9}, {NULL}};
const Segment SGI369525[] = {{&S10559[33], 15}, {NULL}};
const Segment SGI369526[] = {{&S10559[103], 3}, {NULL}};
const Segment SGI369527[] = {{&S10559[49], 6}, {NULL}};
const Segment SGI369528[] = {{&S10559[33], 13}, {NULL}};
const Segment SGI369529[] = {{&S10560[69], 6}, {NULL}};
const Segment SGI369530[] = {{&S10560[8], 3}, {NULL}};
const Segment SGI369531[] = {{&S10560[13], 9}, {NULL}};
const Segment SGI369532[] = {{&S10560[33], 15}, {NULL}};
const Segment SGI369533[] = {{&S10560[107], 3}, {NULL}};
const Segment SGI369534[] = {{&S10560[49], 6}, {NULL}};
const Segment SGI369535[] = {{&S10560[33], 13}, {NULL}};
const Segment SGI369536[] = {{&S10561[69], 5}, {NULL}};
const Segment SGI369537[] = {{&S10561[8], 3}, {NULL}};
const Segment SGI369538[] = {{&S10561[13], 9}, {NULL}};
const Segment SGI369539[] = {{&S10561[33], 15}, {NULL}};
const Segment SGI369540[] = {{&S10561[103], 3}, {NULL}};
const Segment SGI369541[] = {{&S10561[49], 6}, {NULL}};
const Segment SGI369542[] = {{&S10561[33], 13}, {NULL}};
const Segment SGI369543[] = {{&S10562[69], 6}, {NULL}};
const Segment SGI369544[] = {{&S10562[8], 3}, {NULL}};
const Segment SGI369545[] = {{&S10562[13], 9}, {NULL}};
const Segment SGI369546[] = {{&S10562[33], 15}, {NULL}};
const Segment SGI369547[] = {{&S10562[107], 3}, {NULL}};
const Segment SGI369548[] = {{&S10562[49], 6}, {NULL}};
const Segment SGI369549[] = {{&S10562[33], 13}, {NULL}};
const Segment SGI369550[] = {{&S10563[69], 5}, {NULL}};
const Segment SGI369551[] = {{&S10563[8], 3}, {NULL}};
const Segment SGI369552[] = {{&S10563[13], 9}, {NULL}};
const Segment SGI369553[] = {{&S10563[33], 15}, {NULL}};
const Segment SGI369554[] = {{&S10563[103], 3}, {NULL}};
const Segment SGI369555[] = {{&S10563[49], 6}, {NULL}};
const Segment SGI369556[] = {{&S10563[33], 13}, {NULL}};
const Segment SGI369557[] = {{&S10564[119], 6}, {NULL}};
const Segment SGI369558[] = {{&S10564[8], 3}, {NULL}};
const Segment SGI369559[] = {{&S10564[13], 9}, {NULL}};
const Segment SGI369560[] = {{&S10564[33], 15}, {NULL}};
const Segment SGI369561[] = {{&S10564[106], 3}, {NULL}};
const Segment SGI369562[] = {{&S10564[49], 6}, {NULL}};
const Segment SGI369563[] = {{&S10564[33], 13}, {NULL}};
const Segment SGI369564[] = {{&S10565[69], 6}, {NULL}};
const Segment SGI369565[] = {{&S10565[8], 3}, {NULL}};
const Segment SGI369566[] = {{&S10565[13], 9}, {NULL}};
const Segment SGI369567[] = {{&S10565[33], 15}, {NULL}};
const Segment SGI369568[] = {{&S10565[106], 3}, {NULL}};
const Segment SGI369569[] = {{&S10565[49], 6}, {NULL}};
const Segment SGI369570[] = {{&S10565[33], 13}, {NULL}};
const Segment SGI369571[] = {{&S10566[69], 6}, {NULL}};
const Segment SGI369572[] = {{&S10566[8], 3}, {NULL}};
const Segment SGI369573[] = {{&S10566[13], 9}, {NULL}};
const Segment SGI369574[] = {{&S10566[33], 15}, {NULL}};
const Segment SGI369575[] = {{&S10566[106], 3}, {NULL}};
const Segment SGI369576[] = {{&S10566[49], 6}, {NULL}};
const Segment SGI369577[] = {{&S10566[33], 13}, {NULL}};
const Segment SGI369578[] = {{&S10567[69], 5}, {NULL}};
const Segment SGI369579[] = {{&S10567[8], 3}, {NULL}};
const Segment SGI369580[] = {{&S10567[13], 9}, {NULL}};
const Segment SGI369581[] = {{&S10567[33], 15}, {NULL}};
const Segment SGI369582[] = {{&S10567[103], 3}, {NULL}};
const Segment SGI369583[] = {{&S10567[49], 6}, {NULL}};
const Segment SGI369584[] = {{&S10567[33], 13}, {NULL}};
const Segment SGI369585[] = {{&S10568[69], 6}, {NULL}};
const Segment SGI369586[] = {{&S10568[8], 3}, {NULL}};
const Segment SGI369587[] = {{&S10568[13], 9}, {NULL}};
const Segment SGI369588[] = {{&S10568[33], 15}, {NULL}};
const Segment SGI369589[] = {{&S10568[107], 3}, {NULL}};
const Segment SGI369590[] = {{&S10568[49], 6}, {NULL}};
const Segment SGI369591[] = {{&S10568[33], 13}, {NULL}};
const Segment SGI369592[] = {{&S10569[69], 6}, {NULL}};
const Segment SGI369593[] = {{&S10569[8], 3}, {NULL}};
const Segment SGI369594[] = {{&S10569[13], 9}, {NULL}};
const Segment SGI369595[] = {{&S10569[33], 15}, {NULL}};
const Segment SGI369596[] = {{&S10569[106], 3}, {NULL}};
const Segment SGI369597[] = {{&S10569[49], 6}, {NULL}};
const Segment SGI369598[] = {{&S10569[33], 13}, {NULL}};
const Segment SGI369599[] = {{&S10570[69], 5}, {NULL}};
const Segment SGI369600[] = {{&S10570[8], 3}, {NULL}};
const Segment SGI369601[] = {{&S10570[13], 9}, {NULL}};
const Segment SGI369602[] = {{&S10570[33], 15}, {NULL}};
const Segment SGI369603[] = {{&S10570[103], 3}, {NULL}};
const Segment SGI369604[] = {{&S10570[49], 6}, {NULL}};
const Segment SGI369605[] = {{&S10570[33], 13}, {NULL}};
const Segment SGI369606[] = {{&S10571[66], 6}, {NULL}};
const Segment SGI369607[] = {{&S10571[8], 3}, {NULL}};
const Segment SGI369608[] = {{&S10571[13], 9}, {NULL}};
const Segment SGI369609[] = {{&S10571[33], 15}, {NULL}};
const Segment SGI369610[] = {{&S10571[104], 3}, {NULL}};
const Segment SGI369611[] = {{&S10571[49], 3}, {NULL}};
const Segment SGI369612[] = {{&S10571[33], 13}, {NULL}};
const Segment SGI369613[] = {{&S10572[69], 6}, {NULL}};
const Segment SGI369614[] = {{&S10572[8], 3}, {NULL}};
const Segment SGI369615[] = {{&S10572[13], 9}, {NULL}};
const Segment SGI369616[] = {{&S10572[33], 15}, {NULL}};
const Segment SGI369617[] = {{&S10572[107], 3}, {NULL}};
const Segment SGI369618[] = {{&S10572[49], 6}, {NULL}};
const Segment SGI369619[] = {{&S10572[33], 13}, {NULL}};
const Segment SGI369620[] = {{&S10573[69], 6}, {NULL}};
const Segment SGI369621[] = {{&S10573[8], 3}, {NULL}};
const Segment SGI369622[] = {{&S10573[13], 9}, {NULL}};
const Segment SGI369623[] = {{&S10573[33], 15}, {NULL}};
const Segment SGI369624[] = {{&S10573[107], 3}, {NULL}};
const Segment SGI369625[] = {{&S10573[49], 6}, {NULL}};
const Segment SGI369626[] = {{&S10573[33], 13}, {NULL}};
const Segment SGI369627[] = {{&S10574[69], 6}, {NULL}};
const Segment SGI369628[] = {{&S10574[8], 3}, {NULL}};
const Segment SGI369629[] = {{&S10574[13], 9}, {NULL}};
const Segment SGI369630[] = {{&S10574[33], 15}, {NULL}};
const Segment SGI369631[] = {{&S10574[106], 3}, {NULL}};
const Segment SGI369632[] = {{&S10574[49], 6}, {NULL}};
const Segment SGI369633[] = {{&S10574[33], 13}, {NULL}};
const Segment SGI369634[] = {{&S10575[69], 6}, {NULL}};
const Segment SGI369635[] = {{&S10575[8], 3}, {NULL}};
const Segment SGI369636[] = {{&S10575[13], 9}, {NULL}};
const Segment SGI369637[] = {{&S10575[33], 15}, {NULL}};
const Segment SGI369638[] = {{&S10575[106], 3}, {NULL}};
const Segment SGI369639[] = {{&S10575[49], 6}, {NULL}};
const Segment SGI369640[] = {{&S10575[33], 13}, {NULL}};
const Segment SGI369641[] = {{&S10576[69], 6}, {NULL}};
const Segment SGI369642[] = {{&S10576[8], 3}, {NULL}};
const Segment SGI369643[] = {{&S10576[13], 9}, {NULL}};
const Segment SGI369644[] = {{&S10576[33], 15}, {NULL}};
const Segment SGI369645[] = {{&S10576[107], 3}, {NULL}};
const Segment SGI369646[] = {{&S10576[49], 6}, {NULL}};
const Segment SGI369647[] = {{&S10576[33], 13}, {NULL}};
const Segment SGI369648[] = {{&S10577[69], 6}, {NULL}};
const Segment SGI369649[] = {{&S10577[8], 3}, {NULL}};
const Segment SGI369650[] = {{&S10577[13], 9}, {NULL}};
const Segment SGI369651[] = {{&S10577[33], 15}, {NULL}};
const Segment SGI369652[] = {{&S10577[106], 3}, {NULL}};
const Segment SGI369653[] = {{&S10577[49], 6}, {NULL}};
const Segment SGI369654[] = {{&S10577[33], 13}, {NULL}};
const Segment SGI369655[] = {{&S10578[69], 6}, {NULL}};
const Segment SGI369656[] = {{&S10578[8], 3}, {NULL}};
const Segment SGI369657[] = {{&S10578[13], 9}, {NULL}};
const Segment SGI369658[] = {{&S10578[33], 15}, {NULL}};
const Segment SGI369659[] = {{&S10578[107], 3}, {NULL}};
const Segment SGI369660[] = {{&S10578[49], 6}, {NULL}};
const Segment SGI369661[] = {{&S10578[33], 13}, {NULL}};
const Segment SGI369662[] = {{&S10579[69], 6}, {NULL}};
const Segment SGI369663[] = {{&S10579[8], 3}, {NULL}};
const Segment SGI369664[] = {{&S10579[13], 9}, {NULL}};
const Segment SGI369665[] = {{&S10579[33], 15}, {NULL}};
const Segment SGI369666[] = {{&S10579[106], 3}, {NULL}};
const Segment SGI369667[] = {{&S10579[49], 6}, {NULL}};
const Segment SGI369668[] = {{&S10579[33], 13}, {NULL}};
const Segment SGI369669[] = {{&S10580[69], 5}, {NULL}};
const Segment SGI369670[] = {{&S10580[8], 3}, {NULL}};
const Segment SGI369671[] = {{&S10580[13], 9}, {NULL}};
const Segment SGI369672[] = {{&S10580[33], 15}, {NULL}};
const Segment SGI369673[] = {{&S10580[103], 3}, {NULL}};
const Segment SGI369674[] = {{&S10580[49], 6}, {NULL}};
const Segment SGI369675[] = {{&S10580[33], 13}, {NULL}};
const Segment SGI369676[] = {{&S10581[69], 6}, {NULL}};
const Segment SGI369677[] = {{&S10581[8], 3}, {NULL}};
const Segment SGI369678[] = {{&S10581[13], 9}, {NULL}};
const Segment SGI369679[] = {{&S10581[33], 15}, {NULL}};
const Segment SGI369680[] = {{&S10581[106], 3}, {NULL}};
const Segment SGI369681[] = {{&S10581[49], 6}, {NULL}};
const Segment SGI369682[] = {{&S10581[33], 13}, {NULL}};
const Segment SGI369683[] = {{&S10582[69], 6}, {NULL}};
const Segment SGI369684[] = {{&S10582[8], 3}, {NULL}};
const Segment SGI369685[] = {{&S10582[13], 9}, {NULL}};
const Segment SGI369686[] = {{&S10582[33], 15}, {NULL}};
const Segment SGI369687[] = {{&S10582[107], 3}, {NULL}};
const Segment SGI369688[] = {{&S10582[49], 6}, {NULL}};
const Segment SGI369689[] = {{&S10582[33], 13}, {NULL}};
const Segment SGI369690[] = {{&S10583[69], 6}, {NULL}};
const Segment SGI369691[] = {{&S10583[8], 3}, {NULL}};
const Segment SGI369692[] = {{&S10583[13], 9}, {NULL}};
const Segment SGI369693[] = {{&S10583[33], 15}, {NULL}};
const Segment SGI369694[] = {{&S10583[106], 3}, {NULL}};
const Segment SGI369695[] = {{&S10583[49], 6}, {NULL}};
const Segment SGI369696[] = {{&S10583[33], 13}, {NULL}};
const Segment SGI369697[] = {{&S10584[69], 6}, {NULL}};
const Segment SGI369698[] = {{&S10584[8], 3}, {NULL}};
const Segment SGI369699[] = {{&S10584[13], 9}, {NULL}};
const Segment SGI369700[] = {{&S10584[33], 15}, {NULL}};
const Segment SGI369701[] = {{&S10584[107], 3}, {NULL}};
const Segment SGI369702[] = {{&S10584[49], 6}, {NULL}};
const Segment SGI369703[] = {{&S10584[33], 13}, {NULL}};
const Segment SGI369704[] = {{&S10585[69], 6}, {NULL}};
const Segment SGI369705[] = {{&S10585[8], 3}, {NULL}};
const Segment SGI369706[] = {{&S10585[13], 9}, {NULL}};
const Segment SGI369707[] = {{&S10585[33], 15}, {NULL}};
const Segment SGI369708[] = {{&S10585[106], 3}, {NULL}};
const Segment SGI369709[] = {{&S10585[49], 6}, {NULL}};
const Segment SGI369710[] = {{&S10585[33], 13}, {NULL}};
const Segment SGI369711[] = {{&S10586[69], 6}, {NULL}};
const Segment SGI369712[] = {{&S10586[8], 3}, {NULL}};
const Segment SGI369713[] = {{&S10586[13], 9}, {NULL}};
const Segment SGI369714[] = {{&S10586[33], 15}, {NULL}};
const Segment SGI369715[] = {{&S10586[106], 3}, {NULL}};
const Segment SGI369716[] = {{&S10586[49], 6}, {NULL}};
const Segment SGI369717[] = {{&S10586[33], 13}, {NULL}};
const Segment SGI369718[] = {{&S10587[69], 5}, {NULL}};
const Segment SGI369719[] = {{&S10587[8], 3}, {NULL}};
const Segment SGI369720[] = {{&S10587[13], 9}, {NULL}};
const Segment SGI369721[] = {{&S10587[33], 15}, {NULL}};
const Segment SGI369722[] = {{&S10587[103], 3}, {NULL}};
const Segment SGI369723[] = {{&S10587[49], 6}, {NULL}};
const Segment SGI369724[] = {{&S10587[33], 13}, {NULL}};
const Segment SGI369725[] = {{&S10588[69], 5}, {NULL}};
const Segment SGI369726[] = {{&S10588[8], 3}, {NULL}};
const Segment SGI369727[] = {{&S10588[13], 9}, {NULL}};
const Segment SGI369728[] = {{&S10588[33], 15}, {NULL}};
const Segment SGI369729[] = {{&S10588[103], 3}, {NULL}};
const Segment SGI369730[] = {{&S10588[49], 6}, {NULL}};
const Segment SGI369731[] = {{&S10588[33], 13}, {NULL}};
const Segment SGI369732[] = {{&S10589[69], 6}, {NULL}};
const Segment SGI369733[] = {{&S10589[8], 3}, {NULL}};
const Segment SGI369734[] = {{&S10589[13], 9}, {NULL}};
const Segment SGI369735[] = {{&S10589[33], 15}, {NULL}};
const Segment SGI369736[] = {{&S10589[106], 3}, {NULL}};
const Segment SGI369737[] = {{&S10589[49], 6}, {NULL}};
const Segment SGI369738[] = {{&S10589[33], 13}, {NULL}};
const Segment SGI369739[] = {{&S10590[69], 6}, {NULL}};
const Segment SGI369740[] = {{&S10590[8], 3}, {NULL}};
const Segment SGI369741[] = {{&S10590[13], 9}, {NULL}};
const Segment SGI369742[] = {{&S10590[33], 15}, {NULL}};
const Segment SGI369743[] = {{&S10590[106], 3}, {NULL}};
const Segment SGI369744[] = {{&S10590[49], 6}, {NULL}};
const Segment SGI369745[] = {{&S10590[33], 13}, {NULL}};
const Segment SGI369746[] = {{&S10591[69], 6}, {NULL}};
const Segment SGI369747[] = {{&S10591[8], 3}, {NULL}};
const Segment SGI369748[] = {{&S10591[13], 9}, {NULL}};
const Segment SGI369749[] = {{&S10591[33], 15}, {NULL}};
const Segment SGI369750[] = {{&S10591[106], 3}, {NULL}};
const Segment SGI369751[] = {{&S10591[49], 6}, {NULL}};
const Segment SGI369752[] = {{&S10591[33], 13}, {NULL}};
const Segment SGI369753[] = {{&S10592[69], 5}, {NULL}};
const Segment SGI369754[] = {{&S10592[8], 3}, {NULL}};
const Segment SGI369755[] = {{&S10592[13], 9}, {NULL}};
const Segment SGI369756[] = {{&S10592[33], 15}, {NULL}};
const Segment SGI369757[] = {{&S10592[103], 3}, {NULL}};
const Segment SGI369758[] = {{&S10592[49], 6}, {NULL}};
const Segment SGI369759[] = {{&S10592[33], 13}, {NULL}};
const Segment SGI369760[] = {{&S10593[69], 6}, {NULL}};
const Segment SGI369761[] = {{&S10593[8], 3}, {NULL}};
const Segment SGI369762[] = {{&S10593[13], 9}, {NULL}};
const Segment SGI369763[] = {{&S10593[33], 15}, {NULL}};
const Segment SGI369764[] = {{&S10593[106], 3}, {NULL}};
const Segment SGI369765[] = {{&S10593[49], 6}, {NULL}};
const Segment SGI369766[] = {{&S10593[33], 13}, {NULL}};
const Segment SGI369767[] = {{&S10594[69], 5}, {NULL}};
const Segment SGI369768[] = {{&S10594[8], 3}, {NULL}};
const Segment SGI369769[] = {{&S10594[13], 9}, {NULL}};
const Segment SGI369770[] = {{&S10594[33], 15}, {NULL}};
const Segment SGI369771[] = {{&S10594[103], 3}, {NULL}};
const Segment SGI369772[] = {{&S10594[49], 6}, {NULL}};
const Segment SGI369773[] = {{&S10594[33], 13}, {NULL}};
const Segment SGI369774[] = {{&S10595[69], 5}, {NULL}};
const Segment SGI369775[] = {{&S10595[8], 3}, {NULL}};
const Segment SGI369776[] = {{&S10595[13], 9}, {NULL}};
const Segment SGI369777[] = {{&S10595[33], 15}, {NULL}};
const Segment SGI369778[] = {{&S10595[103], 3}, {NULL}};
const Segment SGI369779[] = {{&S10595[49], 6}, {NULL}};
const Segment SGI369780[] = {{&S10595[33], 13}, {NULL}};
const Segment SGI369781[] = {{&S10596[119], 5}, {NULL}};
const Segment SGI369782[] = {{&S10596[8], 3}, {NULL}};
const Segment SGI369783[] = {{&S10596[13], 9}, {NULL}};
const Segment SGI369784[] = {{&S10596[33], 15}, {NULL}};
const Segment SGI369785[] = {{&S10596[106], 3}, {NULL}};
const Segment SGI369786[] = {{&S10596[49], 6}, {NULL}};
const Segment SGI369787[] = {{&S10596[33], 13}, {NULL}};
const Segment SGI369788[] = {{&S10597[67], 5}, {NULL}};
const Segment SGI369789[] = {{&S10597[8], 3}, {NULL}};
const Segment SGI369790[] = {{&S10597[13], 9}, {NULL}};
const Segment SGI369791[] = {{&S10597[33], 13}, {NULL}};
const Segment SGI369792[] = {{&S10597[101], 3}, {NULL}};
const Segment SGI369793[] = {{&S10598[119], 6}, {NULL}};
const Segment SGI369794[] = {{&S10598[8], 3}, {NULL}};
const Segment SGI369795[] = {{&S10598[13], 9}, {NULL}};
const Segment SGI369796[] = {{&S10598[33], 15}, {NULL}};
const Segment SGI369797[] = {{&S10598[106], 3}, {NULL}};
const Segment SGI369798[] = {{&S10598[49], 6}, {NULL}};
const Segment SGI369799[] = {{&S10598[33], 13}, {NULL}};
const Segment SGI369800[] = {{&S10599[119], 5}, {NULL}};
const Segment SGI369801[] = {{&S10599[8], 3}, {NULL}};
const Segment SGI369802[] = {{&S10599[13], 9}, {NULL}};
const Segment SGI369803[] = {{&S10599[33], 15}, {NULL}};
const Segment SGI369804[] = {{&S10599[106], 3}, {NULL}};
const Segment SGI369805[] = {{&S10599[49], 6}, {NULL}};
const Segment SGI369806[] = {{&S10599[33], 13}, {NULL}};
const Segment SGI369807[] = {{&S10600[120], 6}, {NULL}};
const Segment SGI369808[] = {{&S10600[8], 3}, {NULL}};
const Segment SGI369809[] = {{&S10600[13], 9}, {NULL}};
const Segment SGI369810[] = {{&S10600[33], 15}, {NULL}};
const Segment SGI369811[] = {{&S10600[107], 3}, {NULL}};
const Segment SGI369812[] = {{&S10600[49], 6}, {NULL}};
const Segment SGI369813[] = {{&S10600[33], 13}, {NULL}};
const Segment SGI369814[] = {{&S10601[120], 5}, {NULL}};
const Segment SGI369815[] = {{&S10601[8], 3}, {NULL}};
const Segment SGI369816[] = {{&S10601[13], 9}, {NULL}};
const Segment SGI369817[] = {{&S10601[33], 15}, {NULL}};
const Segment SGI369818[] = {{&S10601[107], 3}, {NULL}};
const Segment SGI369819[] = {{&S10601[49], 6}, {NULL}};
const Segment SGI369820[] = {{&S10601[33], 13}, {NULL}};
const Segment SGI369821[] = {{&S10602[69], 6}, {NULL}};
const Segment SGI369822[] = {{&S10602[8], 3}, {NULL}};
const Segment SGI369823[] = {{&S10602[13], 9}, {NULL}};
const Segment SGI369824[] = {{&S10602[33], 15}, {NULL}};
const Segment SGI369825[] = {{&S10602[107], 3}, {NULL}};
const Segment SGI369826[] = {{&S10602[49], 6}, {NULL}};
const Segment SGI369827[] = {{&S10602[33], 13}, {NULL}};
const Segment SGI369828[] = {{&S10603[119], 5}, {NULL}};
const Segment SGI369829[] = {{&S10603[8], 3}, {NULL}};
const Segment SGI369830[] = {{&S10603[13], 9}, {NULL}};
const Segment SGI369831[] = {{&S10603[33], 15}, {NULL}};
const Segment SGI369832[] = {{&S10603[106], 3}, {NULL}};
const Segment SGI369833[] = {{&S10603[49], 6}, {NULL}};
const Segment SGI369834[] = {{&S10603[33], 13}, {NULL}};
const Segment SGI369835[] = {{&S10604[119], 6}, {NULL}};
const Segment SGI369836[] = {{&S10604[8], 3}, {NULL}};
const Segment SGI369837[] = {{&S10604[13], 9}, {NULL}};
const Segment SGI369838[] = {{&S10604[33], 15}, {NULL}};
const Segment SGI369839[] = {{&S10604[106], 3}, {NULL}};
const Segment SGI369840[] = {{&S10604[49], 6}, {NULL}};
const Segment SGI369841[] = {{&S10604[33], 13}, {NULL}};
const Segment SGI369842[] = {{&S10605[119], 6}, {NULL}};
const Segment SGI369843[] = {{&S10605[8], 3}, {NULL}};
const Segment SGI369844[] = {{&S10605[13], 9}, {NULL}};
const Segment SGI369845[] = {{&S10605[33], 15}, {NULL}};
const Segment SGI369846[] = {{&S10605[106], 3}, {NULL}};
const Segment SGI369847[] = {{&S10605[49], 6}, {NULL}};
const Segment SGI369848[] = {{&S10605[33], 13}, {NULL}};
const Segment SGI369849[] = {{&S10606[69], 6}, {NULL}};
const Segment SGI369850[] = {{&S10606[8], 3}, {NULL}};
const Segment SGI369851[] = {{&S10606[13], 9}, {NULL}};
const Segment SGI369852[] = {{&S10606[33], 15}, {NULL}};
const Segment SGI369853[] = {{&S10606[106], 3}, {NULL}};
const Segment SGI369854[] = {{&S10606[49], 6}, {NULL}};
const Segment SGI369855[] = {{&S10606[33], 13}, {NULL}};
const Segment SGI369856[] = {{&S10607[69], 6}, {NULL}};
const Segment SGI369857[] = {{&S10607[8], 3}, {NULL}};
const Segment SGI369858[] = {{&S10607[13], 9}, {NULL}};
const Segment SGI369859[] = {{&S10607[33], 15}, {NULL}};
const Segment SGI369860[] = {{&S10607[106], 3}, {NULL}};
const Segment SGI369861[] = {{&S10607[49], 6}, {NULL}};
const Segment SGI369862[] = {{&S10607[33], 13}, {NULL}};
const Segment SGI369863[] = {{&S10608[69], 5}, {NULL}};
const Segment SGI369864[] = {{&S10608[8], 3}, {NULL}};
const Segment SGI369865[] = {{&S10608[13], 9}, {NULL}};
const Segment SGI369866[] = {{&S10608[33], 15}, {NULL}};
const Segment SGI369867[] = {{&S10608[103], 3}, {NULL}};
const Segment SGI369868[] = {{&S10608[49], 6}, {NULL}};
const Segment SGI369869[] = {{&S10608[33], 13}, {NULL}};
const Segment SGI369870[] = {{&S10609[69], 6}, {NULL}};
const Segment SGI369871[] = {{&S10609[8], 3}, {NULL}};
const Segment SGI369872[] = {{&S10609[13], 9}, {NULL}};
const Segment SGI369873[] = {{&S10609[33], 15}, {NULL}};
const Segment SGI369874[] = {{&S10609[106], 3}, {NULL}};
const Segment SGI369875[] = {{&S10609[49], 6}, {NULL}};
const Segment SGI369876[] = {{&S10609[33], 13}, {NULL}};
const Segment SGI369877[] = {{&S10610[69], 6}, {NULL}};
const Segment SGI369878[] = {{&S10610[8], 3}, {NULL}};
const Segment SGI369879[] = {{&S10610[13], 9}, {NULL}};
const Segment SGI369880[] = {{&S10610[33], 15}, {NULL}};
const Segment SGI369881[] = {{&S10610[106], 3}, {NULL}};
const Segment SGI369882[] = {{&S10610[49], 6}, {NULL}};
const Segment SGI369883[] = {{&S10610[33], 13}, {NULL}};
const Segment SGI369884[] = {{&S10611[69], 6}, {NULL}};
const Segment SGI369885[] = {{&S10611[8], 3}, {NULL}};
const Segment SGI369886[] = {{&S10611[13], 9}, {NULL}};
const Segment SGI369887[] = {{&S10611[33], 15}, {NULL}};
const Segment SGI369888[] = {{&S10611[107], 3}, {NULL}};
const Segment SGI369889[] = {{&S10611[49], 6}, {NULL}};
const Segment SGI369890[] = {{&S10611[33], 13}, {NULL}};
const Segment SGI369891[] = {{&S10612[69], 6}, {NULL}};
const Segment SGI369892[] = {{&S10612[8], 3}, {NULL}};
const Segment SGI369893[] = {{&S10612[13], 9}, {NULL}};
const Segment SGI369894[] = {{&S10612[33], 15}, {NULL}};
const Segment SGI369895[] = {{&S10612[106], 3}, {NULL}};
const Segment SGI369896[] = {{&S10612[49], 6}, {NULL}};
const Segment SGI369897[] = {{&S10612[33], 13}, {NULL}};
const Segment SGI369898[] = {{&S10613[66], 6}, {NULL}};
const Segment SGI369899[] = {{&S10613[8], 3}, {NULL}};
const Segment SGI369900[] = {{&S10613[13], 9}, {NULL}};
const Segment SGI369901[] = {{&S10613[33], 15}, {NULL}};
const Segment SGI369902[] = {{&S10613[103], 3}, {NULL}};
const Segment SGI369903[] = {{&S10613[49], 3}, {NULL}};
const Segment SGI369904[] = {{&S10613[33], 13}, {NULL}};
const Segment SGI369905[] = {{&S10614[69], 6}, {NULL}};
const Segment SGI369906[] = {{&S10614[8], 3}, {NULL}};
const Segment SGI369907[] = {{&S10614[13], 9}, {NULL}};
const Segment SGI369908[] = {{&S10614[33], 15}, {NULL}};
const Segment SGI369909[] = {{&S10614[106], 3}, {NULL}};
const Segment SGI369910[] = {{&S10614[49], 6}, {NULL}};
const Segment SGI369911[] = {{&S10614[33], 13}, {NULL}};
const Segment SGI369912[] = {{&S10615[120], 5}, {NULL}};
const Segment SGI369913[] = {{&S10615[8], 3}, {NULL}};
const Segment SGI369914[] = {{&S10615[13], 9}, {NULL}};
const Segment SGI369915[] = {{&S10615[33], 15}, {NULL}};
const Segment SGI369916[] = {{&S10615[107], 3}, {NULL}};
const Segment SGI369917[] = {{&S10615[49], 6}, {NULL}};
const Segment SGI369918[] = {{&S10615[33], 13}, {NULL}};
const Segment SGI369919[] = {{&S10616[69], 6}, {NULL}};
const Segment SGI369920[] = {{&S10616[8], 3}, {NULL}};
const Segment SGI369921[] = {{&S10616[13], 9}, {NULL}};
const Segment SGI369922[] = {{&S10616[33], 15}, {NULL}};
const Segment SGI369923[] = {{&S10616[106], 3}, {NULL}};
const Segment SGI369924[] = {{&S10616[49], 6}, {NULL}};
const Segment SGI369925[] = {{&S10616[33], 13}, {NULL}};
const Segment SGI369926[] = {{&S10617[69], 6}, {NULL}};
const Segment SGI369927[] = {{&S10617[8], 3}, {NULL}};
const Segment SGI369928[] = {{&S10617[13], 9}, {NULL}};
const Segment SGI369929[] = {{&S10617[33], 15}, {NULL}};
const Segment SGI369930[] = {{&S10617[106], 3}, {NULL}};
const Segment SGI369931[] = {{&S10617[49], 6}, {NULL}};
const Segment SGI369932[] = {{&S10617[33], 13}, {NULL}};
const Segment SGI369933[] = {{&S10618[69], 6}, {NULL}};
const Segment SGI369934[] = {{&S10618[8], 3}, {NULL}};
const Segment SGI369935[] = {{&S10618[13], 9}, {NULL}};
const Segment SGI369936[] = {{&S10618[33], 15}, {NULL}};
const Segment SGI369937[] = {{&S10618[106], 3}, {NULL}};
const Segment SGI369938[] = {{&S10618[49], 6}, {NULL}};
const Segment SGI369939[] = {{&S10618[33], 13}, {NULL}};
const Segment SGI369940[] = {{&S10619[69], 6}, {NULL}};
const Segment SGI369941[] = {{&S10619[8], 3}, {NULL}};
const Segment SGI369942[] = {{&S10619[13], 9}, {NULL}};
const Segment SGI369943[] = {{&S10619[33], 15}, {NULL}};
const Segment SGI369944[] = {{&S10619[106], 3}, {NULL}};
const Segment SGI369945[] = {{&S10619[49], 6}, {NULL}};
const Segment SGI369946[] = {{&S10619[33], 13}, {NULL}};
const Segment SGI369947[] = {{&S10620[69], 6}, {NULL}};
const Segment SGI369948[] = {{&S10620[8], 3}, {NULL}};
const Segment SGI369949[] = {{&S10620[13], 9}, {NULL}};
const Segment SGI369950[] = {{&S10620[33], 15}, {NULL}};
const Segment SGI369951[] = {{&S10620[106], 3}, {NULL}};
const Segment SGI369952[] = {{&S10620[49], 6}, {NULL}};
const Segment SGI369953[] = {{&S10620[33], 13}, {NULL}};
const Segment SGI369954[] = {{&S10621[69], 5}, {NULL}};
const Segment SGI369955[] = {{&S10621[8], 3}, {NULL}};
const Segment SGI369956[] = {{&S10621[13], 9}, {NULL}};
const Segment SGI369957[] = {{&S10621[33], 15}, {NULL}};
const Segment SGI369958[] = {{&S10621[103], 3}, {NULL}};
const Segment SGI369959[] = {{&S10621[49], 6}, {NULL}};
const Segment SGI369960[] = {{&S10621[33], 13}, {NULL}};
const Segment SGI369961[] = {{&S10622[69], 5}, {NULL}};
const Segment SGI369962[] = {{&S10622[8], 3}, {NULL}};
const Segment SGI369963[] = {{&S10622[13], 9}, {NULL}};
const Segment SGI369964[] = {{&S10622[33], 15}, {NULL}};
const Segment SGI369965[] = {{&S10622[103], 3}, {NULL}};
const Segment SGI369966[] = {{&S10622[49], 6}, {NULL}};
const Segment SGI369967[] = {{&S10622[33], 13}, {NULL}};
const Segment SGI369968[] = {{&S10623[69], 6}, {NULL}};
const Segment SGI369969[] = {{&S10623[8], 3}, {NULL}};
const Segment SGI369970[] = {{&S10623[13], 9}, {NULL}};
const Segment SGI369971[] = {{&S10623[33], 15}, {NULL}};
const Segment SGI369972[] = {{&S10623[106], 3}, {NULL}};
const Segment SGI369973[] = {{&S10623[49], 6}, {NULL}};
const Segment SGI369974[] = {{&S10623[33], 13}, {NULL}};
const Segment SGI369975[] = {{&S10624[69], 5}, {NULL}};
const Segment SGI369976[] = {{&S10624[8], 3}, {NULL}};
const Segment SGI369977[] = {{&S10624[13], 9}, {NULL}};
const Segment SGI369978[] = {{&S10624[33], 15}, {NULL}};
const Segment SGI369979[] = {{&S10624[103], 3}, {NULL}};
const Segment SGI369980[] = {{&S10624[49], 6}, {NULL}};
const Segment SGI369981[] = {{&S10624[33], 13}, {NULL}};
const Segment SGI369982[] = {{&S10625[69], 5}, {NULL}};
const Segment SGI369983[] = {{&S10625[8], 3}, {NULL}};
const Segment SGI369984[] = {{&S10625[13], 9}, {NULL}};
const Segment SGI369985[] = {{&S10625[33], 15}, {NULL}};
const Segment SGI369986[] = {{&S10625[103], 3}, {NULL}};
const Segment SGI369987[] = {{&S10625[49], 6}, {NULL}};
const Segment SGI369988[] = {{&S10625[33], 13}, {NULL}};
const Segment SGI369989[] = {{&S10626[69], 6}, {NULL}};
const Segment SGI369990[] = {{&S10626[8], 3}, {NULL}};
const Segment SGI369991[] = {{&S10626[13], 9}, {NULL}};
const Segment SGI369992[] = {{&S10626[33], 15}, {NULL}};
const Segment SGI369993[] = {{&S10626[107], 3}, {NULL}};
const Segment SGI369994[] = {{&S10626[49], 6}, {NULL}};
const Segment SGI369995[] = {{&S10626[33], 13}, {NULL}};
const Segment SGI369996[] = {{&S10627[114], 5}, {NULL}};
const Segment SGI369997[] = {{&S10627[8], 3}, {NULL}};
const Segment SGI369998[] = {{&S10627[13], 9}, {NULL}};
const Segment SGI369999[] = {{&S10627[33], 13}, {NULL}};
const Segment SGI370000[] = {{&S10627[101], 3}, {NULL}};
const Segment SGI370001[] = {{&S10628[69], 5}, {NULL}};
const Segment SGI370002[] = {{&S10628[8], 3}, {NULL}};
const Segment SGI370003[] = {{&S10628[13], 9}, {NULL}};
const Segment SGI370004[] = {{&S10628[33], 15}, {NULL}};
const Segment SGI370005[] = {{&S10628[103], 3}, {NULL}};
const Segment SGI370006[] = {{&S10628[49], 6}, {NULL}};
const Segment SGI370007[] = {{&S10628[33], 13}, {NULL}};
const Segment SGI370008[] = {{&S10629[69], 6}, {NULL}};
const Segment SGI370009[] = {{&S10629[8], 3}, {NULL}};
const Segment SGI370010[] = {{&S10629[13], 9}, {NULL}};
const Segment SGI370011[] = {{&S10629[33], 15}, {NULL}};
const Segment SGI370012[] = {{&S10629[106], 3}, {NULL}};
const Segment SGI370013[] = {{&S10629[49], 6}, {NULL}};
const Segment SGI370014[] = {{&S10629[33], 13}, {NULL}};
const Segment SGI370015[] = {{&S10630[69], 6}, {NULL}};
const Segment SGI370016[] = {{&S10630[8], 3}, {NULL}};
const Segment SGI370017[] = {{&S10630[13], 9}, {NULL}};
const Segment SGI370018[] = {{&S10630[33], 15}, {NULL}};
const Segment SGI370019[] = {{&S10630[107], 3}, {NULL}};
const Segment SGI370020[] = {{&S10630[49], 6}, {NULL}};
const Segment SGI370021[] = {{&S10630[33], 13}, {NULL}};
const Segment SGI370022[] = {{&S10631[69], 5}, {NULL}};
const Segment SGI370023[] = {{&S10631[8], 3}, {NULL}};
const Segment SGI370024[] = {{&S10631[13], 9}, {NULL}};
const Segment SGI370025[] = {{&S10631[33], 15}, {NULL}};
const Segment SGI370026[] = {{&S10631[103], 3}, {NULL}};
const Segment SGI370027[] = {{&S10631[49], 6}, {NULL}};
const Segment SGI370028[] = {{&S10631[33], 13}, {NULL}};
const Segment SGI370029[] = {{&S10632[69], 6}, {NULL}};
const Segment SGI370030[] = {{&S10632[8], 3}, {NULL}};
const Segment SGI370031[] = {{&S10632[13], 9}, {NULL}};
const Segment SGI370032[] = {{&S10632[33], 15}, {NULL}};
const Segment SGI370033[] = {{&S10632[107], 3}, {NULL}};
const Segment SGI370034[] = {{&S10632[49], 6}, {NULL}};
const Segment SGI370035[] = {{&S10632[33], 13}, {NULL}};
const Segment SGI370036[] = {{&S10633[69], 6}, {NULL}};
const Segment SGI370037[] = {{&S10633[8], 3}, {NULL}};
const Segment SGI370038[] = {{&S10633[13], 9}, {NULL}};
const Segment SGI370039[] = {{&S10633[33], 15}, {NULL}};
const Segment SGI370040[] = {{&S10633[107], 3}, {NULL}};
const Segment SGI370041[] = {{&S10633[49], 6}, {NULL}};
const Segment SGI370042[] = {{&S10633[33], 13}, {NULL}};
const Segment SGI370043[] = {{&S10634[69], 6}, {NULL}};
const Segment SGI370044[] = {{&S10634[8], 3}, {NULL}};
const Segment SGI370045[] = {{&S10634[13], 9}, {NULL}};
const Segment SGI370046[] = {{&S10634[33], 15}, {NULL}};
const Segment SGI370047[] = {{&S10634[106], 3}, {NULL}};
const Segment SGI370048[] = {{&S10634[49], 6}, {NULL}};
const Segment SGI370049[] = {{&S10634[33], 13}, {NULL}};
const Segment SGI370050[] = {{&S10635[69], 6}, {NULL}};
const Segment SGI370051[] = {{&S10635[8], 3}, {NULL}};
const Segment SGI370052[] = {{&S10635[13], 9}, {NULL}};
const Segment SGI370053[] = {{&S10635[33], 15}, {NULL}};
const Segment SGI370054[] = {{&S10635[106], 3}, {NULL}};
const Segment SGI370055[] = {{&S10635[49], 6}, {NULL}};
const Segment SGI370056[] = {{&S10635[33], 13}, {NULL}};
const Segment SGI370057[] = {{&S10636[69], 5}, {NULL}};
const Segment SGI370058[] = {{&S10636[8], 3}, {NULL}};
const Segment SGI370059[] = {{&S10636[13], 9}, {NULL}};
const Segment SGI370060[] = {{&S10636[33], 15}, {NULL}};
const Segment SGI370061[] = {{&S10636[103], 3}, {NULL}};
const Segment SGI370062[] = {{&S10636[49], 6}, {NULL}};
const Segment SGI370063[] = {{&S10636[33], 13}, {NULL}};
const Segment SGI370064[] = {{&S10637[119], 6}, {NULL}};
const Segment SGI370065[] = {{&S10637[8], 3}, {NULL}};
const Segment SGI370066[] = {{&S10637[13], 9}, {NULL}};
const Segment SGI370067[] = {{&S10637[33], 15}, {NULL}};
const Segment SGI370068[] = {{&S10637[106], 3}, {NULL}};
const Segment SGI370069[] = {{&S10637[49], 6}, {NULL}};
const Segment SGI370070[] = {{&S10637[33], 13}, {NULL}};
const Segment SGI370071[] = {{&S10638[67], 5}, {NULL}};
const Segment SGI370072[] = {{&S10638[8], 3}, {NULL}};
const Segment SGI370073[] = {{&S10638[13], 9}, {NULL}};
const Segment SGI370074[] = {{&S10638[33], 13}, {NULL}};
const Segment SGI370075[] = {{&S10638[101], 3}, {NULL}};
const Segment SGI370076[] = {{&S10639[66], 5}, {NULL}};
const Segment SGI370077[] = {{&S10639[8], 3}, {NULL}};
const Segment SGI370078[] = {{&S10639[13], 9}, {NULL}};
const Segment SGI370079[] = {{&S10639[33], 15}, {NULL}};
const Segment SGI370080[] = {{&S10639[100], 3}, {NULL}};
const Segment SGI370081[] = {{&S10639[49], 3}, {NULL}};
const Segment SGI370082[] = {{&S10639[33], 13}, {NULL}};
const Segment SGI370083[] = {{&S10640[69], 5}, {NULL}};
const Segment SGI370084[] = {{&S10640[8], 3}, {NULL}};
const Segment SGI370085[] = {{&S10640[13], 9}, {NULL}};
const Segment SGI370086[] = {{&S10640[33], 15}, {NULL}};
const Segment SGI370087[] = {{&S10640[103], 3}, {NULL}};
const Segment SGI370088[] = {{&S10640[49], 6}, {NULL}};
const Segment SGI370089[] = {{&S10640[33], 13}, {NULL}};
const Segment SGI370090[] = {{&S10641[105], 7}, {NULL}};
const Segment SGI370091[] = {{&S10641[8], 3}, {NULL}};
const Segment SGI370092[] = {{&S10641[13], 9}, {NULL}};
const Segment SGI370093[] = {{&S10641[33], 15}, {NULL}};
const Segment SGI370094[] = {{&S10641[49], 6}, {NULL}};
const Segment SGI370095[] = {{&S10641[33], 13}, {NULL}};
const Segment SGI370096[] = {{&S10642[119], 6}, {NULL}};
const Segment SGI370097[] = {{&S10642[8], 3}, {NULL}};
const Segment SGI370098[] = {{&S10642[13], 9}, {NULL}};
const Segment SGI370099[] = {{&S10642[33], 15}, {NULL}};
const Segment SGI370100[] = {{&S10642[106], 3}, {NULL}};
const Segment SGI370101[] = {{&S10642[49], 6}, {NULL}};
const Segment SGI370102[] = {{&S10642[33], 13}, {NULL}};
const Segment SGI370103[] = {{&S10643[69], 5}, {NULL}};
const Segment SGI370104[] = {{&S10643[8], 3}, {NULL}};
const Segment SGI370105[] = {{&S10643[13], 9}, {NULL}};
const Segment SGI370106[] = {{&S10643[33], 15}, {NULL}};
const Segment SGI370107[] = {{&S10643[103], 3}, {NULL}};
const Segment SGI370108[] = {{&S10643[49], 6}, {NULL}};
const Segment SGI370109[] = {{&S10643[33], 13}, {NULL}};
const Segment SGI370110[] = {{&S10644[119], 6}, {NULL}};
const Segment SGI370111[] = {{&S10644[8], 3}, {NULL}};
const Segment SGI370112[] = {{&S10644[13], 9}, {NULL}};
const Segment SGI370113[] = {{&S10644[33], 15}, {NULL}};
const Segment SGI370114[] = {{&S10644[106], 3}, {NULL}};
const Segment SGI370115[] = {{&S10644[49], 6}, {NULL}};
const Segment SGI370116[] = {{&S10644[33], 13}, {NULL}};
const Segment SGI370117[] = {{&S10645[69], 6}, {NULL}};
const Segment SGI370118[] = {{&S10645[8], 3}, {NULL}};
const Segment SGI370119[] = {{&S10645[13], 9}, {NULL}};
const Segment SGI370120[] = {{&S10645[33], 15}, {NULL}};
const Segment SGI370121[] = {{&S10645[106], 3}, {NULL}};
const Segment SGI370122[] = {{&S10645[49], 6}, {NULL}};
const Segment SGI370123[] = {{&S10645[33], 13}, {NULL}};
const Segment SGI370124[] = {{&S10646[69], 5}, {NULL}};
const Segment SGI370125[] = {{&S10646[8], 3}, {NULL}};
const Segment SGI370126[] = {{&S10646[13], 9}, {NULL}};
const Segment SGI370127[] = {{&S10646[33], 15}, {NULL}};
const Segment SGI370128[] = {{&S10646[103], 3}, {NULL}};
const Segment SGI370129[] = {{&S10646[49], 6}, {NULL}};
const Segment SGI370130[] = {{&S10646[33], 13}, {NULL}};
const Segment SGI370131[] = {{&S10647[69], 6}, {NULL}};
const Segment SGI370132[] = {{&S10647[8], 3}, {NULL}};
const Segment SGI370133[] = {{&S10647[13], 9}, {NULL}};
const Segment SGI370134[] = {{&S10647[33], 15}, {NULL}};
const Segment SGI370135[] = {{&S10647[106], 3}, {NULL}};
const Segment SGI370136[] = {{&S10647[49], 6}, {NULL}};
const Segment SGI370137[] = {{&S10647[33], 13}, {NULL}};
const Segment SGI370138[] = {{&S10648[67], 5}, {NULL}};
const Segment SGI370139[] = {{&S10648[8], 3}, {NULL}};
const Segment SGI370140[] = {{&S10648[13], 9}, {NULL}};
const Segment SGI370141[] = {{&S10648[33], 13}, {NULL}};
const Segment SGI370142[] = {{&S10648[101], 3}, {NULL}};
const Segment SGI370143[] = {{&S10649[69], 5}, {NULL}};
const Segment SGI370144[] = {{&S10649[8], 3}, {NULL}};
const Segment SGI370145[] = {{&S10649[13], 9}, {NULL}};
const Segment SGI370146[] = {{&S10649[33], 15}, {NULL}};
const Segment SGI370147[] = {{&S10649[103], 3}, {NULL}};
const Segment SGI370148[] = {{&S10649[49], 6}, {NULL}};
const Segment SGI370149[] = {{&S10649[33], 13}, {NULL}};
const Segment SGI370150[] = {{&S10650[69], 6}, {NULL}};
const Segment SGI370151[] = {{&S10650[8], 3}, {NULL}};
const Segment SGI370152[] = {{&S10650[13], 9}, {NULL}};
const Segment SGI370153[] = {{&S10650[33], 15}, {NULL}};
const Segment SGI370154[] = {{&S10650[106], 3}, {NULL}};
const Segment SGI370155[] = {{&S10650[49], 6}, {NULL}};
const Segment SGI370156[] = {{&S10650[33], 13}, {NULL}};
const Segment SGI370157[] = {{&S10651[119], 6}, {NULL}};
const Segment SGI370158[] = {{&S10651[8], 3}, {NULL}};
const Segment SGI370159[] = {{&S10651[13], 9}, {NULL}};
const Segment SGI370160[] = {{&S10651[33], 15}, {NULL}};
const Segment SGI370161[] = {{&S10651[106], 3}, {NULL}};
const Segment SGI370162[] = {{&S10651[49], 6}, {NULL}};
const Segment SGI370163[] = {{&S10651[33], 13}, {NULL}};
const Segment SGI370164[] = {{&S10652[69], 5}, {NULL}};
const Segment SGI370165[] = {{&S10652[8], 3}, {NULL}};
const Segment SGI370166[] = {{&S10652[13], 9}, {NULL}};
const Segment SGI370167[] = {{&S10652[33], 15}, {NULL}};
const Segment SGI370168[] = {{&S10652[103], 3}, {NULL}};
const Segment SGI370169[] = {{&S10652[49], 6}, {NULL}};
const Segment SGI370170[] = {{&S10652[33], 13}, {NULL}};
const Segment SGI370171[] = {{&S10653[119], 5}, {NULL}};
const Segment SGI370172[] = {{&S10653[8], 3}, {NULL}};
const Segment SGI370173[] = {{&S10653[13], 9}, {NULL}};
const Segment SGI370174[] = {{&S10653[33], 15}, {NULL}};
const Segment SGI370175[] = {{&S10653[106], 3}, {NULL}};
const Segment SGI370176[] = {{&S10653[49], 6}, {NULL}};
const Segment SGI370177[] = {{&S10653[33], 13}, {NULL}};
const Segment SGI370178[] = {{&S10654[74], 6}, {NULL}};
const Segment SGI370179[] = {{&S10654[8], 3}, {NULL}};
const Segment SGI370180[] = {{&S10654[13], 9}, {NULL}};
const Segment SGI370181[] = {{&S10654[31], 15}, {NULL}};
const Segment SGI370182[] = {{&S10654[112], 3}, {NULL}};
const Segment SGI370183[] = {{&S10654[47], 13}, {NULL}};
const Segment SGI370184[] = {{&S10654[31], 13}, {NULL}};
const Segment SGI370185[] = {{&S10655[69], 6}, {NULL}};
const Segment SGI370186[] = {{&S10655[8], 3}, {NULL}};
const Segment SGI370187[] = {{&S10655[13], 9}, {NULL}};
const Segment SGI370188[] = {{&S10655[33], 15}, {NULL}};
const Segment SGI370189[] = {{&S10655[106], 3}, {NULL}};
const Segment SGI370190[] = {{&S10655[49], 6}, {NULL}};
const Segment SGI370191[] = {{&S10655[33], 13}, {NULL}};
const Segment SGI370192[] = {{&S10656[66], 6}, {NULL}};
const Segment SGI370193[] = {{&S10656[8], 3}, {NULL}};
const Segment SGI370194[] = {{&S10656[13], 9}, {NULL}};
const Segment SGI370195[] = {{&S10656[33], 15}, {NULL}};
const Segment SGI370196[] = {{&S10656[104], 3}, {NULL}};
const Segment SGI370197[] = {{&S10656[49], 3}, {NULL}};
const Segment SGI370198[] = {{&S10656[33], 13}, {NULL}};
const Segment SGI370199[] = {{&S10657[69], 6}, {NULL}};
const Segment SGI370200[] = {{&S10657[8], 3}, {NULL}};
const Segment SGI370201[] = {{&S10657[13], 9}, {NULL}};
const Segment SGI370202[] = {{&S10657[33], 15}, {NULL}};
const Segment SGI370203[] = {{&S10657[106], 3}, {NULL}};
const Segment SGI370204[] = {{&S10657[49], 6}, {NULL}};
const Segment SGI370205[] = {{&S10657[33], 13}, {NULL}};
const Segment SGI370206[] = {{&S10658[69], 6}, {NULL}};
const Segment SGI370207[] = {{&S10658[8], 3}, {NULL}};
const Segment SGI370208[] = {{&S10658[13], 9}, {NULL}};
const Segment SGI370209[] = {{&S10658[33], 15}, {NULL}};
const Segment SGI370210[] = {{&S10658[106], 3}, {NULL}};
const Segment SGI370211[] = {{&S10658[49], 6}, {NULL}};
const Segment SGI370212[] = {{&S10658[33], 13}, {NULL}};
const Segment SGI370213[] = {{&S10659[119], 5}, {NULL}};
const Segment SGI370214[] = {{&S10659[8], 3}, {NULL}};
const Segment SGI370215[] = {{&S10659[13], 9}, {NULL}};
const Segment SGI370216[] = {{&S10659[33], 15}, {NULL}};
const Segment SGI370217[] = {{&S10659[106], 3}, {NULL}};
const Segment SGI370218[] = {{&S10659[49], 6}, {NULL}};
const Segment SGI370219[] = {{&S10659[33], 13}, {NULL}};
const Segment SGI370220[] = {{&S10660[69], 5}, {NULL}};
const Segment SGI370221[] = {{&S10660[8], 3}, {NULL}};
const Segment SGI370222[] = {{&S10660[13], 9}, {NULL}};
const Segment SGI370223[] = {{&S10660[33], 15}, {NULL}};
const Segment SGI370224[] = {{&S10660[103], 3}, {NULL}};
const Segment SGI370225[] = {{&S10660[49], 6}, {NULL}};
const Segment SGI370226[] = {{&S10660[33], 13}, {NULL}};
const Segment SGI370227[] = {{&S10661[69], 6}, {NULL}};
const Segment SGI370228[] = {{&S10661[8], 3}, {NULL}};
const Segment SGI370229[] = {{&S10661[13], 9}, {NULL}};
const Segment SGI370230[] = {{&S10661[33], 15}, {NULL}};
const Segment SGI370231[] = {{&S10661[107], 3}, {NULL}};
const Segment SGI370232[] = {{&S10661[49], 6}, {NULL}};
const Segment SGI370233[] = {{&S10661[33], 13}, {NULL}};
const Segment SGI370234[] = {{&S10662[117], 5}, {NULL}};
const Segment SGI370235[] = {{&S10662[8], 3}, {NULL}};
const Segment SGI370236[] = {{&S10662[13], 9}, {NULL}};
const Segment SGI370237[] = {{&S10662[33], 15}, {NULL}};
const Segment SGI370238[] = {{&S10662[104], 3}, {NULL}};
const Segment SGI370239[] = {{&S10662[49], 6}, {NULL}};
const Segment SGI370240[] = {{&S10662[33], 13}, {NULL}};
const Segment SGI370241[] = {{&S10663[69], 6}, {NULL}};
const Segment SGI370242[] = {{&S10663[8], 3}, {NULL}};
const Segment SGI370243[] = {{&S10663[13], 9}, {NULL}};
const Segment SGI370244[] = {{&S10663[33], 15}, {NULL}};
const Segment SGI370245[] = {{&S10663[106], 3}, {NULL}};
const Segment SGI370246[] = {{&S10663[49], 6}, {NULL}};
const Segment SGI370247[] = {{&S10663[33], 13}, {NULL}};
const Segment SGI370248[] = {{&S10664[69], 6}, {NULL}};
const Segment SGI370249[] = {{&S10664[8], 3}, {NULL}};
const Segment SGI370250[] = {{&S10664[13], 9}, {NULL}};
const Segment SGI370251[] = {{&S10664[33], 15}, {NULL}};
const Segment SGI370252[] = {{&S10664[106], 3}, {NULL}};
const Segment SGI370253[] = {{&S10664[49], 6}, {NULL}};
const Segment SGI370254[] = {{&S10664[33], 13}, {NULL}};
const Segment SGI370255[] = {{&S10665[119], 6}, {NULL}};
const Segment SGI370256[] = {{&S10665[8], 3}, {NULL}};
const Segment SGI370257[] = {{&S10665[13], 9}, {NULL}};
const Segment SGI370258[] = {{&S10665[33], 15}, {NULL}};
const Segment SGI370259[] = {{&S10665[106], 3}, {NULL}};
const Segment SGI370260[] = {{&S10665[49], 6}, {NULL}};
const Segment SGI370261[] = {{&S10665[33], 13}, {NULL}};
const Segment SGI370262[] = {{&S10666[69], 5}, {NULL}};
const Segment SGI370263[] = {{&S10666[8], 3}, {NULL}};
const Segment SGI370264[] = {{&S10666[13], 9}, {NULL}};
const Segment SGI370265[] = {{&S10666[33], 15}, {NULL}};
const Segment SGI370266[] = {{&S10666[103], 3}, {NULL}};
const Segment SGI370267[] = {{&S10666[49], 6}, {NULL}};
const Segment SGI370268[] = {{&S10666[33], 13}, {NULL}};
const Segment SGI370269[] = {{&S10667[117], 5}, {NULL}};
const Segment SGI370270[] = {{&S10667[8], 3}, {NULL}};
const Segment SGI370271[] = {{&S10667[13], 9}, {NULL}};
const Segment SGI370272[] = {{&S10667[33], 15}, {NULL}};
const Segment SGI370273[] = {{&S10667[104], 3}, {NULL}};
const Segment SGI370274[] = {{&S10667[49], 6}, {NULL}};
const Segment SGI370275[] = {{&S10667[33], 13}, {NULL}};
const Segment SGI370276[] = {{&S10668[69], 6}, {NULL}};
const Segment SGI370277[] = {{&S10668[8], 3}, {NULL}};
const Segment SGI370278[] = {{&S10668[13], 9}, {NULL}};
const Segment SGI370279[] = {{&S10668[33], 15}, {NULL}};
const Segment SGI370280[] = {{&S10668[106], 3}, {NULL}};
const Segment SGI370281[] = {{&S10668[49], 6}, {NULL}};
const Segment SGI370282[] = {{&S10668[33], 13}, {NULL}};
const Segment SGI370283[] = {{&S10669[69], 6}, {NULL}};
const Segment SGI370284[] = {{&S10669[8], 3}, {NULL}};
const Segment SGI370285[] = {{&S10669[13], 9}, {NULL}};
const Segment SGI370286[] = {{&S10669[33], 15}, {NULL}};
const Segment SGI370287[] = {{&S10669[106], 3}, {NULL}};
const Segment SGI370288[] = {{&S10669[49], 6}, {NULL}};
const Segment SGI370289[] = {{&S10669[33], 13}, {NULL}};
const Segment SGI370290[] = {{&S10670[69], 6}, {NULL}};
const Segment SGI370291[] = {{&S10670[8], 3}, {NULL}};
const Segment SGI370292[] = {{&S10670[13], 9}, {NULL}};
const Segment SGI370293[] = {{&S10670[33], 15}, {NULL}};
const Segment SGI370294[] = {{&S10670[107], 3}, {NULL}};
const Segment SGI370295[] = {{&S10670[49], 6}, {NULL}};
const Segment SGI370296[] = {{&S10670[33], 13}, {NULL}};
const Segment SGI370297[] = {{&S10671[69], 6}, {NULL}};
const Segment SGI370298[] = {{&S10671[8], 3}, {NULL}};
const Segment SGI370299[] = {{&S10671[13], 9}, {NULL}};
const Segment SGI370300[] = {{&S10671[33], 15}, {NULL}};
const Segment SGI370301[] = {{&S10671[106], 3}, {NULL}};
const Segment SGI370302[] = {{&S10671[49], 6}, {NULL}};
const Segment SGI370303[] = {{&S10671[33], 13}, {NULL}};
const Segment SGI370304[] = {{&S10672[69], 6}, {NULL}};
const Segment SGI370305[] = {{&S10672[8], 3}, {NULL}};
const Segment SGI370306[] = {{&S10672[13], 9}, {NULL}};
const Segment SGI370307[] = {{&S10672[33], 15}, {NULL}};
const Segment SGI370308[] = {{&S10672[107], 3}, {NULL}};
const Segment SGI370309[] = {{&S10672[49], 6}, {NULL}};
const Segment SGI370310[] = {{&S10672[33], 13}, {NULL}};
const Segment SGI370311[] = {{&S10673[69], 6}, {NULL}};
const Segment SGI370312[] = {{&S10673[8], 3}, {NULL}};
const Segment SGI370313[] = {{&S10673[13], 9}, {NULL}};
const Segment SGI370314[] = {{&S10673[33], 15}, {NULL}};
const Segment SGI370315[] = {{&S10673[106], 3}, {NULL}};
const Segment SGI370316[] = {{&S10673[49], 6}, {NULL}};
const Segment SGI370317[] = {{&S10673[33], 13}, {NULL}};
const Segment SGI370318[] = {{&S10674[69], 5}, {NULL}};
const Segment SGI370319[] = {{&S10674[8], 3}, {NULL}};
const Segment SGI370320[] = {{&S10674[13], 9}, {NULL}};
const Segment SGI370321[] = {{&S10674[33], 15}, {NULL}};
const Segment SGI370322[] = {{&S10674[103], 3}, {NULL}};
const Segment SGI370323[] = {{&S10674[49], 6}, {NULL}};
const Segment SGI370324[] = {{&S10674[33], 13}, {NULL}};
const Segment SGI370325[] = {{&S10675[69], 6}, {NULL}};
const Segment SGI370326[] = {{&S10675[8], 3}, {NULL}};
const Segment SGI370327[] = {{&S10675[13], 9}, {NULL}};
const Segment SGI370328[] = {{&S10675[33], 15}, {NULL}};
const Segment SGI370329[] = {{&S10675[107], 3}, {NULL}};
const Segment SGI370330[] = {{&S10675[49], 6}, {NULL}};
const Segment SGI370331[] = {{&S10675[33], 13}, {NULL}};
const Segment SGI370332[] = {{&S10676[69], 6}, {NULL}};
const Segment SGI370333[] = {{&S10676[8], 3}, {NULL}};
const Segment SGI370334[] = {{&S10676[13], 9}, {NULL}};
const Segment SGI370335[] = {{&S10676[33], 15}, {NULL}};
const Segment SGI370336[] = {{&S10676[107], 3}, {NULL}};
const Segment SGI370337[] = {{&S10676[49], 6}, {NULL}};
const Segment SGI370338[] = {{&S10676[33], 13}, {NULL}};
const Segment SGI370339[] = {{&S10677[114], 6}, {NULL}};
const Segment SGI370340[] = {{&S10677[8], 3}, {NULL}};
const Segment SGI370341[] = {{&S10677[13], 9}, {NULL}};
const Segment SGI370342[] = {{&S10677[33], 13}, {NULL}};
const Segment SGI370343[] = {{&S10677[101], 3}, {NULL}};
const Segment SGI370344[] = {{&S10678[8], 3}, {NULL}};
const Segment SGI370345[] = {{&S10678[13], 9}, {NULL}};
const Segment SGI370346[] = {{&S10678[33], 15}, {NULL}};
const Segment SGI370347[] = {{&S10678[103], 3}, {NULL}};
const Segment SGI370348[] = {{&S10678[49], 6}, {NULL}};
const Segment SGI370349[] = {{&S10678[33], 13}, {NULL}};
const Segment SGI370350[] = {{&S10679[100], 6}, {NULL}};
const Segment SGI370351[] = {{&S10679[8], 3}, {NULL}};
const Segment SGI370352[] = {{&S10679[13], 9}, {NULL}};
const Segment SGI370353[] = {{&S10679[33], 13}, {NULL}};
const Segment SGI370354[] = {{&S10680[69], 6}, {NULL}};
const Segment SGI370355[] = {{&S10680[8], 3}, {NULL}};
const Segment SGI370356[] = {{&S10680[13], 9}, {NULL}};
const Segment SGI370357[] = {{&S10680[33], 15}, {NULL}};
const Segment SGI370358[] = {{&S10680[106], 3}, {NULL}};
const Segment SGI370359[] = {{&S10680[49], 6}, {NULL}};
const Segment SGI370360[] = {{&S10680[33], 13}, {NULL}};
const Segment SGI370361[] = {{&S10681[114], 6}, {NULL}};
const Segment SGI370362[] = {{&S10681[8], 3}, {NULL}};
const Segment SGI370363[] = {{&S10681[13], 9}, {NULL}};
const Segment SGI370364[] = {{&S10681[33], 13}, {NULL}};
const Segment SGI370365[] = {{&S10681[101], 3}, {NULL}};
const Segment SGI370366[] = {{&S10682[69], 5}, {NULL}};
const Segment SGI370367[] = {{&S10682[8], 3}, {NULL}};
const Segment SGI370368[] = {{&S10682[13], 9}, {NULL}};
const Segment SGI370369[] = {{&S10682[33], 15}, {NULL}};
const Segment SGI370370[] = {{&S10682[103], 3}, {NULL}};
const Segment SGI370371[] = {{&S10682[49], 6}, {NULL}};
const Segment SGI370372[] = {{&S10682[33], 13}, {NULL}};
const Segment SGI370373[] = {{&S10683[69], 5}, {NULL}};
const Segment SGI370374[] = {{&S10683[8], 3}, {NULL}};
const Segment SGI370375[] = {{&S10683[13], 9}, {NULL}};
const Segment SGI370376[] = {{&S10683[33], 15}, {NULL}};
const Segment SGI370377[] = {{&S10683[103], 3}, {NULL}};
const Segment SGI370378[] = {{&S10683[49], 6}, {NULL}};
const Segment SGI370379[] = {{&S10683[33], 13}, {NULL}};
const Segment SGI370380[] = {{&S10684[119], 6}, {NULL}};
const Segment SGI370381[] = {{&S10684[8], 3}, {NULL}};
const Segment SGI370382[] = {{&S10684[13], 9}, {NULL}};
const Segment SGI370383[] = {{&S10684[33], 15}, {NULL}};
const Segment SGI370384[] = {{&S10684[106], 3}, {NULL}};
const Segment SGI370385[] = {{&S10684[49], 6}, {NULL}};
const Segment SGI370386[] = {{&S10684[33], 13}, {NULL}};
const Segment SGI370387[] = {{&S10685[69], 6}, {NULL}};
const Segment SGI370388[] = {{&S10685[8], 3}, {NULL}};
const Segment SGI370389[] = {{&S10685[13], 9}, {NULL}};
const Segment SGI370390[] = {{&S10685[33], 15}, {NULL}};
const Segment SGI370391[] = {{&S10685[107], 3}, {NULL}};
const Segment SGI370392[] = {{&S10685[49], 6}, {NULL}};
const Segment SGI370393[] = {{&S10685[33], 13}, {NULL}};
const Segment SGI370394[] = {{&S10686[105], 7}, {NULL}};
const Segment SGI370395[] = {{&S10686[8], 3}, {NULL}};
const Segment SGI370396[] = {{&S10686[13], 9}, {NULL}};
const Segment SGI370397[] = {{&S10686[33], 15}, {NULL}};
const Segment SGI370398[] = {{&S10686[49], 6}, {NULL}};
const Segment SGI370399[] = {{&S10686[33], 13}, {NULL}};
const Segment SGI370400[] = {{&S10687[69], 6}, {NULL}};
const Segment SGI370401[] = {{&S10687[8], 3}, {NULL}};
const Segment SGI370402[] = {{&S10687[13], 9}, {NULL}};
const Segment SGI370403[] = {{&S10687[33], 15}, {NULL}};
const Segment SGI370404[] = {{&S10687[106], 3}, {NULL}};
const Segment SGI370405[] = {{&S10687[49], 6}, {NULL}};
const Segment SGI370406[] = {{&S10687[33], 13}, {NULL}};
const Segment SGI370407[] = {{&S10688[69], 6}, {NULL}};
const Segment SGI370408[] = {{&S10688[8], 3}, {NULL}};
const Segment SGI370409[] = {{&S10688[13], 9}, {NULL}};
const Segment SGI370410[] = {{&S10688[33], 15}, {NULL}};
const Segment SGI370411[] = {{&S10688[106], 3}, {NULL}};
const Segment SGI370412[] = {{&S10688[49], 6}, {NULL}};
const Segment SGI370413[] = {{&S10688[33], 13}, {NULL}};
const Segment SGI370414[] = {{&S10689[69], 6}, {NULL}};
const Segment SGI370415[] = {{&S10689[8], 3}, {NULL}};
const Segment SGI370416[] = {{&S10689[13], 9}, {NULL}};
const Segment SGI370417[] = {{&S10689[33], 15}, {NULL}};
const Segment SGI370418[] = {{&S10689[106], 3}, {NULL}};
const Segment SGI370419[] = {{&S10689[49], 6}, {NULL}};
const Segment SGI370420[] = {{&S10689[33], 13}, {NULL}};
const Segment SGI370421[] = {{&S10690[119], 5}, {NULL}};
const Segment SGI370422[] = {{&S10690[8], 3}, {NULL}};
const Segment SGI370423[] = {{&S10690[13], 9}, {NULL}};
const Segment SGI370424[] = {{&S10690[33], 15}, {NULL}};
const Segment SGI370425[] = {{&S10690[106], 3}, {NULL}};
const Segment SGI370426[] = {{&S10690[49], 6}, {NULL}};
const Segment SGI370427[] = {{&S10690[33], 13}, {NULL}};
const Segment SGI370428[] = {{&S10691[119], 6}, {NULL}};
const Segment SGI370429[] = {{&S10691[8], 3}, {NULL}};
const Segment SGI370430[] = {{&S10691[13], 9}, {NULL}};
const Segment SGI370431[] = {{&S10691[33], 15}, {NULL}};
const Segment SGI370432[] = {{&S10691[106], 3}, {NULL}};
const Segment SGI370433[] = {{&S10691[49], 6}, {NULL}};
const Segment SGI370434[] = {{&S10691[33], 13}, {NULL}};
const Segment SGI370435[] = {{&S10692[119], 5}, {NULL}};
const Segment SGI370436[] = {{&S10692[8], 3}, {NULL}};
const Segment SGI370437[] = {{&S10692[13], 9}, {NULL}};
const Segment SGI370438[] = {{&S10692[33], 15}, {NULL}};
const Segment SGI370439[] = {{&S10692[106], 3}, {NULL}};
const Segment SGI370440[] = {{&S10692[49], 6}, {NULL}};
const Segment SGI370441[] = {{&S10692[33], 13}, {NULL}};
const Segment SGI370442[] = {{&S10693[119], 5}, {NULL}};
const Segment SGI370443[] = {{&S10693[8], 3}, {NULL}};
const Segment SGI370444[] = {{&S10693[13], 9}, {NULL}};
const Segment SGI370445[] = {{&S10693[33], 15}, {NULL}};
const Segment SGI370446[] = {{&S10693[106], 3}, {NULL}};
const Segment SGI370447[] = {{&S10693[49], 6}, {NULL}};
const Segment SGI370448[] = {{&S10693[33], 13}, {NULL}};
const Segment SGI370449[] = {{&S10694[119], 6}, {NULL}};
const Segment SGI370450[] = {{&S10694[8], 3}, {NULL}};
const Segment SGI370451[] = {{&S10694[13], 9}, {NULL}};
const Segment SGI370452[] = {{&S10694[33], 15}, {NULL}};
const Segment SGI370453[] = {{&S10694[106], 3}, {NULL}};
const Segment SGI370454[] = {{&S10694[49], 6}, {NULL}};
const Segment SGI370455[] = {{&S10694[33], 13}, {NULL}};
const Segment SGI370456[] = {{&S10695[119], 5}, {NULL}};
const Segment SGI370457[] = {{&S10695[8], 3}, {NULL}};
const Segment SGI370458[] = {{&S10695[13], 9}, {NULL}};
const Segment SGI370459[] = {{&S10695[33], 15}, {NULL}};
const Segment SGI370460[] = {{&S10695[106], 3}, {NULL}};
const Segment SGI370461[] = {{&S10695[49], 6}, {NULL}};
const Segment SGI370462[] = {{&S10695[33], 13}, {NULL}};
const Segment SGI370463[] = {{&S10696[69], 6}, {NULL}};
const Segment SGI370464[] = {{&S10696[8], 3}, {NULL}};
const Segment SGI370465[] = {{&S10696[13], 9}, {NULL}};
const Segment SGI370466[] = {{&S10696[33], 15}, {NULL}};
const Segment SGI370467[] = {{&S10696[106], 3}, {NULL}};
const Segment SGI370468[] = {{&S10696[49], 6}, {NULL}};
const Segment SGI370469[] = {{&S10696[33], 13}, {NULL}};
const Segment SGI370470[] = {{&S10697[109], 6}, {NULL}};
const Segment SGI370471[] = {{&S10697[8], 3}, {NULL}};
const Segment SGI370472[] = {{&S10697[13], 9}, {NULL}};
const Segment SGI370473[] = {{&S10697[96], 3}, {NULL}};
const Segment SGI370474[] = {{&S10697[30], 13}, {NULL}};
const Segment SGI370475[] = {{&S10698[69], 6}, {NULL}};
const Segment SGI370476[] = {{&S10698[8], 3}, {NULL}};
const Segment SGI370477[] = {{&S10698[13], 9}, {NULL}};
const Segment SGI370478[] = {{&S10698[33], 15}, {NULL}};
const Segment SGI370479[] = {{&S10698[107], 3}, {NULL}};
const Segment SGI370480[] = {{&S10698[49], 6}, {NULL}};
const Segment SGI370481[] = {{&S10698[33], 13}, {NULL}};
const Segment SGI370482[] = {{&S10699[69], 6}, {NULL}};
const Segment SGI370483[] = {{&S10699[8], 3}, {NULL}};
const Segment SGI370484[] = {{&S10699[13], 9}, {NULL}};
const Segment SGI370485[] = {{&S10699[33], 15}, {NULL}};
const Segment SGI370486[] = {{&S10699[107], 3}, {NULL}};
const Segment SGI370487[] = {{&S10699[49], 6}, {NULL}};
const Segment SGI370488[] = {{&S10699[33], 13}, {NULL}};
const Segment SGI370489[] = {{&S10700[66], 6}, {NULL}};
const Segment SGI370490[] = {{&S10700[8], 3}, {NULL}};
const Segment SGI370491[] = {{&S10700[13], 9}, {NULL}};
const Segment SGI370492[] = {{&S10700[33], 15}, {NULL}};
const Segment SGI370493[] = {{&S10700[104], 3}, {NULL}};
const Segment SGI370494[] = {{&S10700[49], 3}, {NULL}};
const Segment SGI370495[] = {{&S10700[33], 13}, {NULL}};
const Segment SGI370496[] = {{&S10701[69], 5}, {NULL}};
const Segment SGI370497[] = {{&S10701[8], 3}, {NULL}};
const Segment SGI370498[] = {{&S10701[13], 9}, {NULL}};
const Segment SGI370499[] = {{&S10701[33], 15}, {NULL}};
const Segment SGI370500[] = {{&S10701[103], 3}, {NULL}};
const Segment SGI370501[] = {{&S10701[49], 6}, {NULL}};
const Segment SGI370502[] = {{&S10701[33], 13}, {NULL}};
const Segment SGI370503[] = {{&S10702[69], 5}, {NULL}};
const Segment SGI370504[] = {{&S10702[8], 3}, {NULL}};
const Segment SGI370505[] = {{&S10702[13], 9}, {NULL}};
const Segment SGI370506[] = {{&S10702[33], 15}, {NULL}};
const Segment SGI370507[] = {{&S10702[103], 3}, {NULL}};
const Segment SGI370508[] = {{&S10702[49], 6}, {NULL}};
const Segment SGI370509[] = {{&S10702[33], 13}, {NULL}};
const Segment SGI370510[] = {{&S10703[8], 3}, {NULL}};
const Segment SGI370511[] = {{&S10703[13], 9}, {NULL}};
const Segment SGI370512[] = {{&S10703[33], 15}, {NULL}};
const Segment SGI370513[] = {{&S10703[103], 3}, {NULL}};
const Segment SGI370514[] = {{&S10703[49], 6}, {NULL}};
const Segment SGI370515[] = {{&S10703[33], 13}, {NULL}};
const Segment SGI370516[] = {{&S10704[81], 6}, {NULL}};
const Segment SGI370517[] = {{&S10704[8], 3}, {NULL}};
const Segment SGI370518[] = {{&S10704[30], 21}, {NULL}};
const Segment SGI370519[] = {{&S10704[46], 5}, {NULL}};
const Segment SGI370520[] = {{&S10704[30], 13}, {NULL}};
const Segment SGI370521[] = {{&S10705[69], 5}, {NULL}};
const Segment SGI370522[] = {{&S10705[8], 3}, {NULL}};
const Segment SGI370523[] = {{&S10705[13], 9}, {NULL}};
const Segment SGI370524[] = {{&S10705[33], 15}, {NULL}};
const Segment SGI370525[] = {{&S10705[103], 3}, {NULL}};
const Segment SGI370526[] = {{&S10705[49], 6}, {NULL}};
const Segment SGI370527[] = {{&S10705[33], 13}, {NULL}};
const Segment SGI370528[] = {{&S10706[69], 6}, {NULL}};
const Segment SGI370529[] = {{&S10706[8], 3}, {NULL}};
const Segment SGI370530[] = {{&S10706[13], 9}, {NULL}};
const Segment SGI370531[] = {{&S10706[33], 15}, {NULL}};
const Segment SGI370532[] = {{&S10706[107], 3}, {NULL}};
const Segment SGI370533[] = {{&S10706[49], 6}, {NULL}};
const Segment SGI370534[] = {{&S10706[33], 13}, {NULL}};
const Segment SGI370535[] = {{&S10707[119], 6}, {NULL}};
const Segment SGI370536[] = {{&S10707[8], 3}, {NULL}};
const Segment SGI370537[] = {{&S10707[13], 9}, {NULL}};
const Segment SGI370538[] = {{&S10707[33], 15}, {NULL}};
const Segment SGI370539[] = {{&S10707[106], 3}, {NULL}};
const Segment SGI370540[] = {{&S10707[49], 6}, {NULL}};
const Segment SGI370541[] = {{&S10707[33], 13}, {NULL}};
const Segment SGI370542[] = {{&S10708[119], 6}, {NULL}};
const Segment SGI370543[] = {{&S10708[8], 3}, {NULL}};
const Segment SGI370544[] = {{&S10708[13], 9}, {NULL}};
const Segment SGI370545[] = {{&S10708[33], 15}, {NULL}};
const Segment SGI370546[] = {{&S10708[106], 3}, {NULL}};
const Segment SGI370547[] = {{&S10708[49], 6}, {NULL}};
const Segment SGI370548[] = {{&S10708[33], 13}, {NULL}};
const Segment SGI370549[] = {{&S10709[122], 7}, {NULL}};
const Segment SGI370550[] = {{&S10709[8], 3}, {NULL}};
const Segment SGI370551[] = {{&S10709[13], 9}, {NULL}};
const Segment SGI370552[] = {{&S10709[33], 15}, {NULL}};
const Segment SGI370553[] = {{&S10709[49], 6}, {NULL}};
const Segment SGI370554[] = {{&S10709[33], 13}, {NULL}};
const Segment SGI370555[] = {{&S10710[109], 6}, {NULL}};
const Segment SGI370556[] = {{&S10710[8], 3}, {NULL}};
const Segment SGI370557[] = {{&S10710[13], 9}, {NULL}};
const Segment SGI370558[] = {{&S10710[96], 3}, {NULL}};
const Segment SGI370559[] = {{&S10710[30], 13}, {NULL}};
const Segment SGI370560[] = {{&S10711[69], 5}, {NULL}};
const Segment SGI370561[] = {{&S10711[8], 3}, {NULL}};
const Segment SGI370562[] = {{&S10711[13], 9}, {NULL}};
const Segment SGI370563[] = {{&S10711[33], 15}, {NULL}};
const Segment SGI370564[] = {{&S10711[103], 3}, {NULL}};
const Segment SGI370565[] = {{&S10711[49], 6}, {NULL}};
const Segment SGI370566[] = {{&S10711[33], 13}, {NULL}};
const Segment SGI370567[] = {{&S10712[69], 6}, {NULL}};
const Segment SGI370568[] = {{&S10712[8], 3}, {NULL}};
const Segment SGI370569[] = {{&S10712[13], 9}, {NULL}};
const Segment SGI370570[] = {{&S10712[33], 15}, {NULL}};
const Segment SGI370571[] = {{&S10712[106], 3}, {NULL}};
const Segment SGI370572[] = {{&S10712[49], 6}, {NULL}};
const Segment SGI370573[] = {{&S10712[33], 13}, {NULL}};
const Segment SGI370574[] = {{&S10713[69], 5}, {NULL}};
const Segment SGI370575[] = {{&S10713[8], 3}, {NULL}};
const Segment SGI370576[] = {{&S10713[13], 9}, {NULL}};
const Segment SGI370577[] = {{&S10713[33], 15}, {NULL}};
const Segment SGI370578[] = {{&S10713[103], 3}, {NULL}};
const Segment SGI370579[] = {{&S10713[49], 6}, {NULL}};
const Segment SGI370580[] = {{&S10713[33], 13}, {NULL}};
const Segment SGI370581[] = {{&S10714[69], 6}, {NULL}};
const Segment SGI370582[] = {{&S10714[8], 3}, {NULL}};
const Segment SGI370583[] = {{&S10714[13], 9}, {NULL}};
const Segment SGI370584[] = {{&S10714[33], 15}, {NULL}};
const Segment SGI370585[] = {{&S10714[106], 3}, {NULL}};
const Segment SGI370586[] = {{&S10714[49], 6}, {NULL}};
const Segment SGI370587[] = {{&S10714[33], 13}, {NULL}};
const Segment SGI370588[] = {{&S10715[69], 5}, {NULL}};
const Segment SGI370589[] = {{&S10715[8], 3}, {NULL}};
const Segment SGI370590[] = {{&S10715[13], 9}, {NULL}};
const Segment SGI370591[] = {{&S10715[33], 15}, {NULL}};
const Segment SGI370592[] = {{&S10715[103], 3}, {NULL}};
const Segment SGI370593[] = {{&S10715[49], 6}, {NULL}};
const Segment SGI370594[] = {{&S10715[33], 13}, {NULL}};
const Segment SGI370595[] = {{&S10716[69], 6}, {NULL}};
const Segment SGI370596[] = {{&S10716[8], 3}, {NULL}};
const Segment SGI370597[] = {{&S10716[13], 9}, {NULL}};
const Segment SGI370598[] = {{&S10716[33], 15}, {NULL}};
const Segment SGI370599[] = {{&S10716[106], 3}, {NULL}};
const Segment SGI370600[] = {{&S10716[49], 6}, {NULL}};
const Segment SGI370601[] = {{&S10716[33], 13}, {NULL}};
const Segment SGI370602[] = {{&S10717[69], 6}, {NULL}};
const Segment SGI370603[] = {{&S10717[8], 3}, {NULL}};
const Segment SGI370604[] = {{&S10717[13], 9}, {NULL}};
const Segment SGI370605[] = {{&S10717[33], 15}, {NULL}};
const Segment SGI370606[] = {{&S10717[106], 3}, {NULL}};
const Segment SGI370607[] = {{&S10717[49], 6}, {NULL}};
const Segment SGI370608[] = {{&S10717[33], 13}, {NULL}};
const Segment SGI370609[] = {{&S10718[69], 5}, {NULL}};
const Segment SGI370610[] = {{&S10718[8], 3}, {NULL}};
const Segment SGI370611[] = {{&S10718[13], 9}, {NULL}};
const Segment SGI370612[] = {{&S10718[33], 15}, {NULL}};
const Segment SGI370613[] = {{&S10718[103], 3}, {NULL}};
const Segment SGI370614[] = {{&S10718[49], 6}, {NULL}};
const Segment SGI370615[] = {{&S10718[33], 13}, {NULL}};
const Segment SGI370616[] = {{&S10719[69], 5}, {NULL}};
const Segment SGI370617[] = {{&S10719[8], 3}, {NULL}};
const Segment SGI370618[] = {{&S10719[13], 9}, {NULL}};
const Segment SGI370619[] = {{&S10719[33], 15}, {NULL}};
const Segment SGI370620[] = {{&S10719[103], 3}, {NULL}};
const Segment SGI370621[] = {{&S10719[49], 6}, {NULL}};
const Segment SGI370622[] = {{&S10719[33], 13}, {NULL}};
const Segment SGI370623[] = {{&S10720[109], 5}, {NULL}};
const Segment SGI370624[] = {{&S10720[8], 3}, {NULL}};
const Segment SGI370625[] = {{&S10720[13], 9}, {NULL}};
const Segment SGI370626[] = {{&S10720[96], 3}, {NULL}};
const Segment SGI370627[] = {{&S10720[30], 13}, {NULL}};
const Segment SGI370628[] = {{&S10721[69], 5}, {NULL}};
const Segment SGI370629[] = {{&S10721[8], 3}, {NULL}};
const Segment SGI370630[] = {{&S10721[13], 9}, {NULL}};
const Segment SGI370631[] = {{&S10721[33], 15}, {NULL}};
const Segment SGI370632[] = {{&S10721[103], 3}, {NULL}};
const Segment SGI370633[] = {{&S10721[49], 6}, {NULL}};
const Segment SGI370634[] = {{&S10721[33], 13}, {NULL}};
const Segment SGI370635[] = {{&S10722[69], 6}, {NULL}};
const Segment SGI370636[] = {{&S10722[8], 3}, {NULL}};
const Segment SGI370637[] = {{&S10722[13], 9}, {NULL}};
const Segment SGI370638[] = {{&S10722[33], 15}, {NULL}};
const Segment SGI370639[] = {{&S10722[107], 3}, {NULL}};
const Segment SGI370640[] = {{&S10722[49], 6}, {NULL}};
const Segment SGI370641[] = {{&S10722[33], 13}, {NULL}};
const Segment SGI370642[] = {{&S10723[69], 6}, {NULL}};
const Segment SGI370643[] = {{&S10723[8], 3}, {NULL}};
const Segment SGI370644[] = {{&S10723[13], 9}, {NULL}};
const Segment SGI370645[] = {{&S10723[33], 15}, {NULL}};
const Segment SGI370646[] = {{&S10723[106], 3}, {NULL}};
const Segment SGI370647[] = {{&S10723[49], 6}, {NULL}};
const Segment SGI370648[] = {{&S10723[33], 13}, {NULL}};
const Segment SGI370649[] = {{&S10724[69], 6}, {NULL}};
const Segment SGI370650[] = {{&S10724[8], 3}, {NULL}};
const Segment SGI370651[] = {{&S10724[13], 9}, {NULL}};
const Segment SGI370652[] = {{&S10724[33], 15}, {NULL}};
const Segment SGI370653[] = {{&S10724[106], 3}, {NULL}};
const Segment SGI370654[] = {{&S10724[49], 6}, {NULL}};
const Segment SGI370655[] = {{&S10724[33], 13}, {NULL}};
const Segment SGI370656[] = {{&S10725[69], 6}, {NULL}};
const Segment SGI370657[] = {{&S10725[8], 3}, {NULL}};
const Segment SGI370658[] = {{&S10725[13], 9}, {NULL}};
const Segment SGI370659[] = {{&S10725[33], 15}, {NULL}};
const Segment SGI370660[] = {{&S10725[106], 3}, {NULL}};
const Segment SGI370661[] = {{&S10725[49], 6}, {NULL}};
const Segment SGI370662[] = {{&S10725[33], 13}, {NULL}};
const Segment SGI370663[] = {{&S10726[119], 6}, {NULL}};
const Segment SGI370664[] = {{&S10726[8], 3}, {NULL}};
const Segment SGI370665[] = {{&S10726[13], 9}, {NULL}};
const Segment SGI370666[] = {{&S10726[33], 15}, {NULL}};
const Segment SGI370667[] = {{&S10726[106], 3}, {NULL}};
const Segment SGI370668[] = {{&S10726[49], 6}, {NULL}};
const Segment SGI370669[] = {{&S10726[33], 13}, {NULL}};
const Segment SGI370670[] = {{&S10727[69], 5}, {NULL}};
const Segment SGI370671[] = {{&S10727[8], 3}, {NULL}};
const Segment SGI370672[] = {{&S10727[13], 9}, {NULL}};
const Segment SGI370673[] = {{&S10727[33], 15}, {NULL}};
const Segment SGI370674[] = {{&S10727[103], 3}, {NULL}};
const Segment SGI370675[] = {{&S10727[49], 6}, {NULL}};
const Segment SGI370676[] = {{&S10727[33], 13}, {NULL}};
const Segment SGI370677[] = {{&S10728[66], 6}, {NULL}};
const Segment SGI370678[] = {{&S10728[8], 3}, {NULL}};
const Segment SGI370679[] = {{&S10728[13], 9}, {NULL}};
const Segment SGI370680[] = {{&S10728[33], 15}, {NULL}};
const Segment SGI370681[] = {{&S10728[103], 3}, {NULL}};
const Segment SGI370682[] = {{&S10728[49], 3}, {NULL}};
const Segment SGI370683[] = {{&S10728[33], 13}, {NULL}};
const Segment SGI370684[] = {{&S10729[69], 5}, {NULL}};
const Segment SGI370685[] = {{&S10729[8], 3}, {NULL}};
const Segment SGI370686[] = {{&S10729[13], 9}, {NULL}};
const Segment SGI370687[] = {{&S10729[33], 15}, {NULL}};
const Segment SGI370688[] = {{&S10729[103], 3}, {NULL}};
const Segment SGI370689[] = {{&S10729[49], 6}, {NULL}};
const Segment SGI370690[] = {{&S10729[33], 13}, {NULL}};
const Segment SGI370691[] = {{&S10730[69], 5}, {NULL}};
const Segment SGI370692[] = {{&S10730[8], 3}, {NULL}};
const Segment SGI370693[] = {{&S10730[13], 9}, {NULL}};
const Segment SGI370694[] = {{&S10730[33], 15}, {NULL}};
const Segment SGI370695[] = {{&S10730[103], 3}, {NULL}};
const Segment SGI370696[] = {{&S10730[49], 6}, {NULL}};
const Segment SGI370697[] = {{&S10730[33], 13}, {NULL}};
const Segment SGI370698[] = {{&S10731[66], 6}, {NULL}};
const Segment SGI370699[] = {{&S10731[8], 3}, {NULL}};
const Segment SGI370700[] = {{&S10731[13], 9}, {NULL}};
const Segment SGI370701[] = {{&S10731[33], 15}, {NULL}};
const Segment SGI370702[] = {{&S10731[103], 3}, {NULL}};
const Segment SGI370703[] = {{&S10731[49], 3}, {NULL}};
const Segment SGI370704[] = {{&S10731[33], 13}, {NULL}};
const Segment SGI370705[] = {{&S10732[69], 6}, {NULL}};
const Segment SGI370706[] = {{&S10732[8], 3}, {NULL}};
const Segment SGI370707[] = {{&S10732[13], 9}, {NULL}};
const Segment SGI370708[] = {{&S10732[33], 15}, {NULL}};
const Segment SGI370709[] = {{&S10732[106], 3}, {NULL}};
const Segment SGI370710[] = {{&S10732[49], 6}, {NULL}};
const Segment SGI370711[] = {{&S10732[33], 13}, {NULL}};
const Segment SGI370712[] = {{&S10733[69], 6}, {NULL}};
const Segment SGI370713[] = {{&S10733[8], 3}, {NULL}};
const Segment SGI370714[] = {{&S10733[13], 9}, {NULL}};
const Segment SGI370715[] = {{&S10733[33], 15}, {NULL}};
const Segment SGI370716[] = {{&S10733[106], 3}, {NULL}};
const Segment SGI370717[] = {{&S10733[49], 6}, {NULL}};
const Segment SGI370718[] = {{&S10733[33], 13}, {NULL}};
const Segment SGI370719[] = {{&S10734[69], 6}, {NULL}};
const Segment SGI370720[] = {{&S10734[8], 3}, {NULL}};
const Segment SGI370721[] = {{&S10734[13], 9}, {NULL}};
const Segment SGI370722[] = {{&S10734[33], 15}, {NULL}};
const Segment SGI370723[] = {{&S10734[106], 3}, {NULL}};
const Segment SGI370724[] = {{&S10734[49], 6}, {NULL}};
const Segment SGI370725[] = {{&S10734[33], 13}, {NULL}};
const Segment SGI370726[] = {{&S10735[69], 5}, {NULL}};
const Segment SGI370727[] = {{&S10735[8], 3}, {NULL}};
const Segment SGI370728[] = {{&S10735[13], 9}, {NULL}};
const Segment SGI370729[] = {{&S10735[33], 15}, {NULL}};
const Segment SGI370730[] = {{&S10735[103], 3}, {NULL}};
const Segment SGI370731[] = {{&S10735[49], 6}, {NULL}};
const Segment SGI370732[] = {{&S10735[33], 13}, {NULL}};
const Segment SGI370733[] = {{&S10736[69], 6}, {NULL}};
const Segment SGI370734[] = {{&S10736[8], 3}, {NULL}};
const Segment SGI370735[] = {{&S10736[13], 9}, {NULL}};
const Segment SGI370736[] = {{&S10736[33], 15}, {NULL}};
const Segment SGI370737[] = {{&S10736[106], 3}, {NULL}};
const Segment SGI370738[] = {{&S10736[49], 6}, {NULL}};
const Segment SGI370739[] = {{&S10736[33], 13}, {NULL}};
const Segment SGI370740[] = {{&S10737[69], 6}, {NULL}};
const Segment SGI370741[] = {{&S10737[8], 3}, {NULL}};
const Segment SGI370742[] = {{&S10737[13], 9}, {NULL}};
const Segment SGI370743[] = {{&S10737[33], 15}, {NULL}};
const Segment SGI370744[] = {{&S10737[106], 3}, {NULL}};
const Segment SGI370745[] = {{&S10737[49], 6}, {NULL}};
const Segment SGI370746[] = {{&S10737[33], 13}, {NULL}};
const Segment SGI370747[] = {{&S10738[119], 6}, {NULL}};
const Segment SGI370748[] = {{&S10738[8], 3}, {NULL}};
const Segment SGI370749[] = {{&S10738[13], 9}, {NULL}};
const Segment SGI370750[] = {{&S10738[33], 15}, {NULL}};
const Segment SGI370751[] = {{&S10738[106], 3}, {NULL}};
const Segment SGI370752[] = {{&S10738[49], 6}, {NULL}};
const Segment SGI370753[] = {{&S10738[33], 13}, {NULL}};
const Segment SGI370754[] = {{&S10739[69], 6}, {NULL}};
const Segment SGI370755[] = {{&S10739[8], 3}, {NULL}};
const Segment SGI370756[] = {{&S10739[13], 9}, {NULL}};
const Segment SGI370757[] = {{&S10739[33], 15}, {NULL}};
const Segment SGI370758[] = {{&S10739[106], 3}, {NULL}};
const Segment SGI370759[] = {{&S10739[49], 6}, {NULL}};
const Segment SGI370760[] = {{&S10739[33], 13}, {NULL}};
const Segment SGI370761[] = {{&S10740[69], 6}, {NULL}};
const Segment SGI370762[] = {{&S10740[8], 3}, {NULL}};
const Segment SGI370763[] = {{&S10740[13], 9}, {NULL}};
const Segment SGI370764[] = {{&S10740[33], 15}, {NULL}};
const Segment SGI370765[] = {{&S10740[106], 3}, {NULL}};
const Segment SGI370766[] = {{&S10740[49], 6}, {NULL}};
const Segment SGI370767[] = {{&S10740[33], 13}, {NULL}};
const Segment SGI370768[] = {{&S10741[69], 6}, {NULL}};
const Segment SGI370769[] = {{&S10741[8], 3}, {NULL}};
const Segment SGI370770[] = {{&S10741[13], 9}, {NULL}};
const Segment SGI370771[] = {{&S10741[33], 15}, {NULL}};
const Segment SGI370772[] = {{&S10741[106], 3}, {NULL}};
const Segment SGI370773[] = {{&S10741[49], 6}, {NULL}};
const Segment SGI370774[] = {{&S10741[33], 13}, {NULL}};
const Segment SGI370775[] = {{&S10742[69], 6}, {NULL}};
const Segment SGI370776[] = {{&S10742[8], 3}, {NULL}};
const Segment SGI370777[] = {{&S10742[13], 9}, {NULL}};
const Segment SGI370778[] = {{&S10742[33], 15}, {NULL}};
const Segment SGI370779[] = {{&S10742[106], 3}, {NULL}};
const Segment SGI370780[] = {{&S10742[49], 6}, {NULL}};
const Segment SGI370781[] = {{&S10742[33], 13}, {NULL}};
const Segment SGI370782[] = {{&S10743[69], 6}, {NULL}};
const Segment SGI370783[] = {{&S10743[8], 3}, {NULL}};
const Segment SGI370784[] = {{&S10743[13], 9}, {NULL}};
const Segment SGI370785[] = {{&S10743[33], 15}, {NULL}};
const Segment SGI370786[] = {{&S10743[106], 3}, {NULL}};
const Segment SGI370787[] = {{&S10743[49], 6}, {NULL}};
const Segment SGI370788[] = {{&S10743[33], 13}, {NULL}};
const Segment SGI370789[] = {{&S10744[66], 5}, {NULL}};
const Segment SGI370790[] = {{&S10744[8], 3}, {NULL}};
const Segment SGI370791[] = {{&S10744[13], 9}, {NULL}};
const Segment SGI370792[] = {{&S10744[33], 15}, {NULL}};
const Segment SGI370793[] = {{&S10744[100], 3}, {NULL}};
const Segment SGI370794[] = {{&S10744[49], 3}, {NULL}};
const Segment SGI370795[] = {{&S10744[33], 13}, {NULL}};
const Segment SGI370796[] = {{&S10745[119], 6}, {NULL}};
const Segment SGI370797[] = {{&S10745[8], 3}, {NULL}};
const Segment SGI370798[] = {{&S10745[13], 9}, {NULL}};
const Segment SGI370799[] = {{&S10745[33], 15}, {NULL}};
const Segment SGI370800[] = {{&S10745[106], 3}, {NULL}};
const Segment SGI370801[] = {{&S10745[49], 6}, {NULL}};
const Segment SGI370802[] = {{&S10745[33], 13}, {NULL}};
const Segment SGI370803[] = {{&S10746[102], 6}, {NULL}};
const Segment SGI370804[] = {{&S10746[8], 3}, {NULL}};
const Segment SGI370805[] = {{&S10746[13], 9}, {NULL}};
const Segment SGI370806[] = {{&S10746[33], 15}, {NULL}};
const Segment SGI370807[] = {{&S10746[49], 6}, {NULL}};
const Segment SGI370808[] = {{&S10746[33], 13}, {NULL}};
const Segment SGI370809[] = {{&S10747[69], 6}, {NULL}};
const Segment SGI370810[] = {{&S10747[8], 3}, {NULL}};
const Segment SGI370811[] = {{&S10747[13], 9}, {NULL}};
const Segment SGI370812[] = {{&S10747[33], 15}, {NULL}};
const Segment SGI370813[] = {{&S10747[106], 3}, {NULL}};
const Segment SGI370814[] = {{&S10747[49], 6}, {NULL}};
const Segment SGI370815[] = {{&S10747[33], 13}, {NULL}};
const Segment SGI370816[] = {{&S10748[69], 6}, {NULL}};
const Segment SGI370817[] = {{&S10748[8], 3}, {NULL}};
const Segment SGI370818[] = {{&S10748[13], 9}, {NULL}};
const Segment SGI370819[] = {{&S10748[33], 15}, {NULL}};
const Segment SGI370820[] = {{&S10748[106], 3}, {NULL}};
const Segment SGI370821[] = {{&S10748[49], 6}, {NULL}};
const Segment SGI370822[] = {{&S10748[33], 13}, {NULL}};
const Segment SGI370823[] = {{&S10749[69], 6}, {NULL}};
const Segment SGI370824[] = {{&S10749[8], 3}, {NULL}};
const Segment SGI370825[] = {{&S10749[13], 9}, {NULL}};
const Segment SGI370826[] = {{&S10749[33], 15}, {NULL}};
const Segment SGI370827[] = {{&S10749[106], 3}, {NULL}};
const Segment SGI370828[] = {{&S10749[49], 6}, {NULL}};
const Segment SGI370829[] = {{&S10749[33], 13}, {NULL}};
const Segment SGI370830[] = {{&S10750[67], 5}, {NULL}};
const Segment SGI370831[] = {{&S10750[8], 3}, {NULL}};
const Segment SGI370832[] = {{&S10750[13], 9}, {NULL}};
const Segment SGI370833[] = {{&S10750[33], 13}, {NULL}};
const Segment SGI370834[] = {{&S10750[102], 3}, {NULL}};
const Segment SGI370835[] = {{&S10751[102], 6}, {NULL}};
const Segment SGI370836[] = {{&S10751[8], 3}, {NULL}};
const Segment SGI370837[] = {{&S10751[13], 9}, {NULL}};
const Segment SGI370838[] = {{&S10751[33], 15}, {NULL}};
const Segment SGI370839[] = {{&S10751[49], 6}, {NULL}};
const Segment SGI370840[] = {{&S10751[33], 13}, {NULL}};
const Segment SGI370841[] = {{&S10752[69], 5}, {NULL}};
const Segment SGI370842[] = {{&S10752[8], 3}, {NULL}};
const Segment SGI370843[] = {{&S10752[13], 9}, {NULL}};
const Segment SGI370844[] = {{&S10752[33], 15}, {NULL}};
const Segment SGI370845[] = {{&S10752[103], 3}, {NULL}};
const Segment SGI370846[] = {{&S10752[49], 6}, {NULL}};
const Segment SGI370847[] = {{&S10752[33], 13}, {NULL}};
const Segment SGI370848[] = {{&S10753[69], 6}, {NULL}};
const Segment SGI370849[] = {{&S10753[8], 3}, {NULL}};
const Segment SGI370850[] = {{&S10753[13], 9}, {NULL}};
const Segment SGI370851[] = {{&S10753[33], 15}, {NULL}};
const Segment SGI370852[] = {{&S10753[106], 3}, {NULL}};
const Segment SGI370853[] = {{&S10753[49], 6}, {NULL}};
const Segment SGI370854[] = {{&S10753[33], 13}, {NULL}};
const Segment SGI370855[] = {{&S10754[69], 5}, {NULL}};
const Segment SGI370856[] = {{&S10754[8], 3}, {NULL}};
const Segment SGI370857[] = {{&S10754[13], 9}, {NULL}};
const Segment SGI370858[] = {{&S10754[33], 15}, {NULL}};
const Segment SGI370859[] = {{&S10754[103], 3}, {NULL}};
const Segment SGI370860[] = {{&S10754[49], 6}, {NULL}};
const Segment SGI370861[] = {{&S10754[33], 13}, {NULL}};
const Segment SGI370862[] = {{&S10755[69], 6}, {NULL}};
const Segment SGI370863[] = {{&S10755[8], 3}, {NULL}};
const Segment SGI370864[] = {{&S10755[13], 9}, {NULL}};
const Segment SGI370865[] = {{&S10755[33], 15}, {NULL}};
const Segment SGI370866[] = {{&S10755[107], 3}, {NULL}};
const Segment SGI370867[] = {{&S10755[49], 6}, {NULL}};
const Segment SGI370868[] = {{&S10755[33], 13}, {NULL}};
const Segment SGI370869[] = {{&S10756[69], 6}, {NULL}};
const Segment SGI370870[] = {{&S10756[8], 3}, {NULL}};
const Segment SGI370871[] = {{&S10756[13], 9}, {NULL}};
const Segment SGI370872[] = {{&S10756[33], 15}, {NULL}};
const Segment SGI370873[] = {{&S10756[106], 3}, {NULL}};
const Segment SGI370874[] = {{&S10756[49], 6}, {NULL}};
const Segment SGI370875[] = {{&S10756[33], 13}, {NULL}};
const Segment SGI370876[] = {{&S10757[119], 6}, {NULL}};
const Segment SGI370877[] = {{&S10757[8], 3}, {NULL}};
const Segment SGI370878[] = {{&S10757[13], 9}, {NULL}};
const Segment SGI370879[] = {{&S10757[33], 15}, {NULL}};
const Segment SGI370880[] = {{&S10757[106], 3}, {NULL}};
const Segment SGI370881[] = {{&S10757[49], 6}, {NULL}};
const Segment SGI370882[] = {{&S10757[33], 13}, {NULL}};
const Segment SGI370883[] = {{&S10758[69], 6}, {NULL}};
const Segment SGI370884[] = {{&S10758[8], 3}, {NULL}};
const Segment SGI370885[] = {{&S10758[13], 9}, {NULL}};
const Segment SGI370886[] = {{&S10758[33], 15}, {NULL}};
const Segment SGI370887[] = {{&S10758[106], 3}, {NULL}};
const Segment SGI370888[] = {{&S10758[49], 6}, {NULL}};
const Segment SGI370889[] = {{&S10758[33], 13}, {NULL}};
const Segment SGI370890[] = {{&S10759[69], 6}, {NULL}};
const Segment SGI370891[] = {{&S10759[8], 3}, {NULL}};
const Segment SGI370892[] = {{&S10759[13], 9}, {NULL}};
const Segment SGI370893[] = {{&S10759[33], 15}, {NULL}};
const Segment SGI370894[] = {{&S10759[107], 3}, {NULL}};
const Segment SGI370895[] = {{&S10759[49], 6}, {NULL}};
const Segment SGI370896[] = {{&S10759[33], 13}, {NULL}};
const Segment SGI370897[] = {{&S10760[69], 6}, {NULL}};
const Segment SGI370898[] = {{&S10760[8], 3}, {NULL}};
const Segment SGI370899[] = {{&S10760[13], 9}, {NULL}};
const Segment SGI370900[] = {{&S10760[33], 15}, {NULL}};
const Segment SGI370901[] = {{&S10760[106], 3}, {NULL}};
const Segment SGI370902[] = {{&S10760[49], 6}, {NULL}};
const Segment SGI370903[] = {{&S10760[33], 13}, {NULL}};
const Segment SGI370904[] = {{&S10761[69], 6}, {NULL}};
const Segment SGI370905[] = {{&S10761[8], 3}, {NULL}};
const Segment SGI370906[] = {{&S10761[13], 9}, {NULL}};
const Segment SGI370907[] = {{&S10761[33], 15}, {NULL}};
const Segment SGI370908[] = {{&S10761[107], 3}, {NULL}};
const Segment SGI370909[] = {{&S10761[49], 6}, {NULL}};
const Segment SGI370910[] = {{&S10761[33], 13}, {NULL}};
const Segment SGI370911[] = {{&S10762[119], 6}, {NULL}};
const Segment SGI370912[] = {{&S10762[8], 3}, {NULL}};
const Segment SGI370913[] = {{&S10762[13], 9}, {NULL}};
const Segment SGI370914[] = {{&S10762[33], 15}, {NULL}};
const Segment SGI370915[] = {{&S10762[106], 3}, {NULL}};
const Segment SGI370916[] = {{&S10762[49], 6}, {NULL}};
const Segment SGI370917[] = {{&S10762[33], 13}, {NULL}};
const Segment SGI370918[] = {{&S10763[119], 5}, {NULL}};
const Segment SGI370919[] = {{&S10763[8], 3}, {NULL}};
const Segment SGI370920[] = {{&S10763[13], 9}, {NULL}};
const Segment SGI370921[] = {{&S10763[33], 15}, {NULL}};
const Segment SGI370922[] = {{&S10763[106], 3}, {NULL}};
const Segment SGI370923[] = {{&S10763[49], 6}, {NULL}};
const Segment SGI370924[] = {{&S10763[33], 13}, {NULL}};
const Segment SGI370925[] = {{&S10764[119], 6}, {NULL}};
const Segment SGI370926[] = {{&S10764[8], 3}, {NULL}};
const Segment SGI370927[] = {{&S10764[13], 9}, {NULL}};
const Segment SGI370928[] = {{&S10764[33], 15}, {NULL}};
const Segment SGI370929[] = {{&S10764[106], 3}, {NULL}};
const Segment SGI370930[] = {{&S10764[49], 6}, {NULL}};
const Segment SGI370931[] = {{&S10764[33], 13}, {NULL}};
const Segment SGI370932[] = {{&S10765[69], 5}, {NULL}};
const Segment SGI370933[] = {{&S10765[8], 3}, {NULL}};
const Segment SGI370934[] = {{&S10765[13], 9}, {NULL}};
const Segment SGI370935[] = {{&S10765[33], 15}, {NULL}};
const Segment SGI370936[] = {{&S10765[103], 3}, {NULL}};
const Segment SGI370937[] = {{&S10765[49], 6}, {NULL}};
const Segment SGI370938[] = {{&S10765[33], 13}, {NULL}};
const Segment SGI370939[] = {{&S10766[66], 5}, {NULL}};
const Segment SGI370940[] = {{&S10766[8], 3}, {NULL}};
const Segment SGI370941[] = {{&S10766[13], 9}, {NULL}};
const Segment SGI370942[] = {{&S10766[33], 15}, {NULL}};
const Segment SGI370943[] = {{&S10766[100], 3}, {NULL}};
const Segment SGI370944[] = {{&S10766[49], 3}, {NULL}};
const Segment SGI370945[] = {{&S10766[33], 13}, {NULL}};
const Segment SGI370946[] = {{&S10767[67], 5}, {NULL}};
const Segment SGI370947[] = {{&S10767[8], 3}, {NULL}};
const Segment SGI370948[] = {{&S10767[13], 9}, {NULL}};
const Segment SGI370949[] = {{&S10767[33], 13}, {NULL}};
const Segment SGI370950[] = {{&S10767[101], 3}, {NULL}};
const Segment SGI370951[] = {{&S10768[67], 6}, {NULL}};
const Segment SGI370952[] = {{&S10768[8], 3}, {NULL}};
const Segment SGI370953[] = {{&S10768[13], 9}, {NULL}};
const Segment SGI370954[] = {{&S10768[33], 13}, {NULL}};
const Segment SGI370955[] = {{&S10768[102], 3}, {NULL}};
const Segment SGI370956[] = {{&S10769[119], 6}, {NULL}};
const Segment SGI370957[] = {{&S10769[8], 3}, {NULL}};
const Segment SGI370958[] = {{&S10769[13], 9}, {NULL}};
const Segment SGI370959[] = {{&S10769[33], 15}, {NULL}};
const Segment SGI370960[] = {{&S10769[106], 3}, {NULL}};
const Segment SGI370961[] = {{&S10769[49], 6}, {NULL}};
const Segment SGI370962[] = {{&S10769[33], 13}, {NULL}};
const Segment SGI370963[] = {{&S10770[69], 5}, {NULL}};
const Segment SGI370964[] = {{&S10770[8], 3}, {NULL}};
const Segment SGI370965[] = {{&S10770[13], 9}, {NULL}};
const Segment SGI370966[] = {{&S10770[33], 15}, {NULL}};
const Segment SGI370967[] = {{&S10770[103], 3}, {NULL}};
const Segment SGI370968[] = {{&S10770[49], 6}, {NULL}};
const Segment SGI370969[] = {{&S10770[33], 13}, {NULL}};
const Segment SGI370970[] = {{&S10771[69], 6}, {NULL}};
const Segment SGI370971[] = {{&S10771[8], 3}, {NULL}};
const Segment SGI370972[] = {{&S10771[13], 9}, {NULL}};
const Segment SGI370973[] = {{&S10771[33], 15}, {NULL}};
const Segment SGI370974[] = {{&S10771[106], 3}, {NULL}};
const Segment SGI370975[] = {{&S10771[49], 6}, {NULL}};
const Segment SGI370976[] = {{&S10771[33], 13}, {NULL}};
const Segment SGI370977[] = {{&S10772[119], 5}, {NULL}};
const Segment SGI370978[] = {{&S10772[8], 3}, {NULL}};
const Segment SGI370979[] = {{&S10772[13], 9}, {NULL}};
const Segment SGI370980[] = {{&S10772[33], 15}, {NULL}};
const Segment SGI370981[] = {{&S10772[106], 3}, {NULL}};
const Segment SGI370982[] = {{&S10772[49], 6}, {NULL}};
const Segment SGI370983[] = {{&S10772[33], 13}, {NULL}};
const Segment SGI370984[] = {{&S10773[69], 6}, {NULL}};
const Segment SGI370985[] = {{&S10773[8], 3}, {NULL}};
const Segment SGI370986[] = {{&S10773[13], 9}, {NULL}};
const Segment SGI370987[] = {{&S10773[33], 15}, {NULL}};
const Segment SGI370988[] = {{&S10773[106], 3}, {NULL}};
const Segment SGI370989[] = {{&S10773[49], 6}, {NULL}};
const Segment SGI370990[] = {{&S10773[33], 13}, {NULL}};
const Segment SGI370991[] = {{&S10774[117], 6}, {NULL}};
const Segment SGI370992[] = {{&S10774[8], 3}, {NULL}};
const Segment SGI370993[] = {{&S10774[13], 9}, {NULL}};
const Segment SGI370994[] = {{&S10774[33], 13}, {NULL}};
const Segment SGI370995[] = {{&S10774[103], 3}, {NULL}};
const Segment SGI370996[] = {{&S10775[69], 5}, {NULL}};
const Segment SGI370997[] = {{&S10775[8], 3}, {NULL}};
const Segment SGI370998[] = {{&S10775[13], 9}, {NULL}};
const Segment SGI370999[] = {{&S10775[33], 15}, {NULL}};
const Segment SGI371000[] = {{&S10775[103], 3}, {NULL}};
const Segment SGI371001[] = {{&S10775[49], 6}, {NULL}};
const Segment SGI371002[] = {{&S10775[33], 13}, {NULL}};
const Segment SGI371003[] = {{&S10776[69], 6}, {NULL}};
const Segment SGI371004[] = {{&S10776[8], 3}, {NULL}};
const Segment SGI371005[] = {{&S10776[13], 9}, {NULL}};
const Segment SGI371006[] = {{&S10776[33], 15}, {NULL}};
const Segment SGI371007[] = {{&S10776[106], 3}, {NULL}};
const Segment SGI371008[] = {{&S10776[49], 6}, {NULL}};
const Segment SGI371009[] = {{&S10776[33], 13}, {NULL}};
const Segment SGI371010[] = {{&S10777[120], 5}, {NULL}};
const Segment SGI371011[] = {{&S10777[8], 3}, {NULL}};
const Segment SGI371012[] = {{&S10777[13], 9}, {NULL}};
const Segment SGI371013[] = {{&S10777[33], 15}, {NULL}};
const Segment SGI371014[] = {{&S10777[107], 3}, {NULL}};
const Segment SGI371015[] = {{&S10777[49], 6}, {NULL}};
const Segment SGI371016[] = {{&S10777[33], 13}, {NULL}};
const Segment SGI371017[] = {{&S10778[119], 5}, {NULL}};
const Segment SGI371018[] = {{&S10778[8], 3}, {NULL}};
const Segment SGI371019[] = {{&S10778[13], 9}, {NULL}};
const Segment SGI371020[] = {{&S10778[33], 15}, {NULL}};
const Segment SGI371021[] = {{&S10778[106], 3}, {NULL}};
const Segment SGI371022[] = {{&S10778[49], 6}, {NULL}};
const Segment SGI371023[] = {{&S10778[33], 13}, {NULL}};
const Segment SGI371024[] = {{&S10779[69], 5}, {NULL}};
const Segment SGI371025[] = {{&S10779[8], 3}, {NULL}};
const Segment SGI371026[] = {{&S10779[13], 9}, {NULL}};
const Segment SGI371027[] = {{&S10779[33], 15}, {NULL}};
const Segment SGI371028[] = {{&S10779[103], 3}, {NULL}};
const Segment SGI371029[] = {{&S10779[49], 6}, {NULL}};
const Segment SGI371030[] = {{&S10779[33], 13}, {NULL}};
const Segment SGI371031[] = {{&S10780[119], 6}, {NULL}};
const Segment SGI371032[] = {{&S10780[8], 3}, {NULL}};
const Segment SGI371033[] = {{&S10780[13], 9}, {NULL}};
const Segment SGI371034[] = {{&S10780[33], 15}, {NULL}};
const Segment SGI371035[] = {{&S10780[106], 3}, {NULL}};
const Segment SGI371036[] = {{&S10780[49], 6}, {NULL}};
const Segment SGI371037[] = {{&S10780[33], 13}, {NULL}};
const Segment SGI371038[] = {{&S10781[119], 6}, {NULL}};
const Segment SGI371039[] = {{&S10781[8], 3}, {NULL}};
const Segment SGI371040[] = {{&S10781[13], 9}, {NULL}};
const Segment SGI371041[] = {{&S10781[33], 15}, {NULL}};
const Segment SGI371042[] = {{&S10781[106], 3}, {NULL}};
const Segment SGI371043[] = {{&S10781[49], 6}, {NULL}};
const Segment SGI371044[] = {{&S10781[33], 13}, {NULL}};
const Segment SGI371045[] = {{&S10782[57], 5}, {NULL}};
const Segment SGI371046[] = {{&S10782[8], 3}, {NULL}};
const Segment SGI371047[] = {{&S10782[13], 9}, {NULL}};
const Segment SGI371048[] = {{&S10782[91], 3}, {NULL}};
const Segment SGI371049[] = {{&S10782[30], 13}, {NULL}};
const Segment SGI371050[] = {{&S10783[119], 5}, {NULL}};
const Segment SGI371051[] = {{&S10783[8], 3}, {NULL}};
const Segment SGI371052[] = {{&S10783[13], 9}, {NULL}};
const Segment SGI371053[] = {{&S10783[33], 15}, {NULL}};
const Segment SGI371054[] = {{&S10783[106], 3}, {NULL}};
const Segment SGI371055[] = {{&S10783[49], 6}, {NULL}};
const Segment SGI371056[] = {{&S10783[33], 13}, {NULL}};
const Segment SGI371057[] = {{&S10784[69], 5}, {NULL}};
const Segment SGI371058[] = {{&S10784[8], 3}, {NULL}};
const Segment SGI371059[] = {{&S10784[13], 9}, {NULL}};
const Segment SGI371060[] = {{&S10784[33], 15}, {NULL}};
const Segment SGI371061[] = {{&S10784[103], 3}, {NULL}};
const Segment SGI371062[] = {{&S10784[49], 6}, {NULL}};
const Segment SGI371063[] = {{&S10784[33], 13}, {NULL}};
const Segment SGI371064[] = {{&S10785[119], 5}, {NULL}};
const Segment SGI371065[] = {{&S10785[8], 3}, {NULL}};
const Segment SGI371066[] = {{&S10785[13], 9}, {NULL}};
const Segment SGI371067[] = {{&S10785[33], 15}, {NULL}};
const Segment SGI371068[] = {{&S10785[106], 3}, {NULL}};
const Segment SGI371069[] = {{&S10785[49], 6}, {NULL}};
const Segment SGI371070[] = {{&S10785[33], 13}, {NULL}};
const Segment SGI371071[] = {{&S10786[119], 6}, {NULL}};
const Segment SGI371072[] = {{&S10786[8], 3}, {NULL}};
const Segment SGI371073[] = {{&S10786[13], 9}, {NULL}};
const Segment SGI371074[] = {{&S10786[33], 15}, {NULL}};
const Segment SGI371075[] = {{&S10786[106], 3}, {NULL}};
const Segment SGI371076[] = {{&S10786[49], 6}, {NULL}};
const Segment SGI371077[] = {{&S10786[33], 13}, {NULL}};
const Segment SGI371078[] = {{&S10787[115], 6}, {NULL}};
const Segment SGI371079[] = {{&S10787[8], 3}, {NULL}};
const Segment SGI371080[] = {{&S10787[13], 9}, {NULL}};
const Segment SGI371081[] = {{&S10787[33], 15}, {NULL}};
const Segment SGI371082[] = {{&S10787[104], 3}, {NULL}};
const Segment SGI371083[] = {{&S10787[49], 6}, {NULL}};
const Segment SGI371084[] = {{&S10787[33], 13}, {NULL}};
const Segment SGI371085[] = {{&S10788[67], 5}, {NULL}};
const Segment SGI371086[] = {{&S10788[8], 3}, {NULL}};
const Segment SGI371087[] = {{&S10788[13], 9}, {NULL}};
const Segment SGI371088[] = {{&S10788[33], 13}, {NULL}};
const Segment SGI371089[] = {{&S10788[101], 3}, {NULL}};
const Segment SGI371090[] = {{&S10789[69], 5}, {NULL}};
const Segment SGI371091[] = {{&S10789[8], 3}, {NULL}};
const Segment SGI371092[] = {{&S10789[13], 9}, {NULL}};
const Segment SGI371093[] = {{&S10789[33], 15}, {NULL}};
const Segment SGI371094[] = {{&S10789[103], 3}, {NULL}};
const Segment SGI371095[] = {{&S10789[49], 6}, {NULL}};
const Segment SGI371096[] = {{&S10789[33], 13}, {NULL}};
const Segment SGI371097[] = {{&S10790[69], 6}, {NULL}};
const Segment SGI371098[] = {{&S10790[8], 3}, {NULL}};
const Segment SGI371099[] = {{&S10790[13], 9}, {NULL}};
const Segment SGI371100[] = {{&S10790[33], 15}, {&S10790[141], 14}, {NULL}};
const Segment SGI371101[] = {{&S10790[106], 3}, {NULL}};
const Segment SGI371102[] = {{&S10790[49], 6}, {NULL}};
const Segment SGI371103[] = {{&S10790[33], 13}, {NULL}};
const Segment SGI371104[] = {{&S10791[69], 6}, {NULL}};
const Segment SGI371105[] = {{&S10791[8], 3}, {NULL}};
const Segment SGI371106[] = {{&S10791[13], 9}, {NULL}};
const Segment SGI371107[] = {{&S10791[33], 15}, {NULL}};
const Segment SGI371108[] = {{&S10791[107], 3}, {NULL}};
const Segment SGI371109[] = {{&S10791[49], 6}, {NULL}};
const Segment SGI371110[] = {{&S10791[33], 13}, {NULL}};
const Segment SGI371111[] = {{&S10792[69], 6}, {NULL}};
const Segment SGI371112[] = {{&S10792[8], 3}, {&S10792[8], 3}, {NULL}};
const Segment SGI371113[] = {{&S10792[13], 9}, {NULL}};
const Segment SGI371114[] = {{&S10792[33], 15}, {NULL}};
const Segment SGI371115[] = {{&S10792[161], 3}, {NULL}};
const Segment SGI371116[] = {{&S10792[49], 6}, {NULL}};
const Segment SGI371117[] = {{&S10792[33], 13}, {NULL}};
const Segment SGI371118[] = {{&S10793[120], 5}, {NULL}};
const Segment SGI371119[] = {{&S10793[8], 3}, {NULL}};
const Segment SGI371120[] = {{&S10793[13], 9}, {NULL}};
const Segment SGI371121[] = {{&S10793[33], 15}, {NULL}};
const Segment SGI371122[] = {{&S10793[107], 3}, {NULL}};
const Segment SGI371123[] = {{&S10793[49], 6}, {NULL}};
const Segment SGI371124[] = {{&S10793[33], 13}, {NULL}};
const Segment SGI371125[] = {{&S10794[69], 5}, {NULL}};
const Segment SGI371126[] = {{&S10794[8], 3}, {NULL}};
const Segment SGI371127[] = {{&S10794[13], 9}, {NULL}};
const Segment SGI371128[] = {{&S10794[33], 15}, {NULL}};
const Segment SGI371129[] = {{&S10794[103], 3}, {NULL}};
const Segment SGI371130[] = {{&S10794[49], 6}, {NULL}};
const Segment SGI371131[] = {{&S10794[33], 13}, {NULL}};
const Segment SGI371132[] = {{&S10795[67], 5}, {NULL}};
const Segment SGI371133[] = {{&S10795[8], 3}, {NULL}};
const Segment SGI371134[] = {{&S10795[13], 9}, {NULL}};
const Segment SGI371135[] = {{&S10795[33], 13}, {NULL}};
const Segment SGI371136[] = {{&S10795[100], 3}, {NULL}};
const Segment SGI371137[] = {{&S10796[66], 6}, {NULL}};
const Segment SGI371138[] = {{&S10796[8], 3}, {NULL}};
const Segment SGI371139[] = {{&S10796[13], 9}, {NULL}};
const Segment SGI371140[] = {{&S10796[33], 12}, {NULL}};
const Segment SGI371141[] = {{&S10796[102], 3}, {NULL}};
const Segment SGI371142[] = {{&S10797[66], 5}, {NULL}};
const Segment SGI371143[] = {{&S10797[8], 3}, {NULL}};
const Segment SGI371144[] = {{&S10797[13], 9}, {NULL}};
const Segment SGI371145[] = {{&S10797[33], 12}, {NULL}};
const Segment SGI371146[] = {{&S10797[99], 3}, {NULL}};
const Segment SGI371147[] = {{&S10798[63], 5}, {NULL}};
const Segment SGI371148[] = {{&S10798[8], 3}, {NULL}};
const Segment SGI371149[] = {{&S10798[13], 9}, {NULL}};
const Segment SGI371150[] = {{&S10798[33], 12}, {NULL}};
const Segment SGI371151[] = {{&S10798[96], 3}, {NULL}};
const Segment SGI371152[] = {{&S10799[112], 5}, {NULL}};
const Segment SGI371153[] = {{&S10799[8], 3}, {NULL}};
const Segment SGI371154[] = {{&S10799[13], 9}, {NULL}};
const Segment SGI371155[] = {{&S10799[33], 12}, {NULL}};
const Segment SGI371156[] = {{&S10799[99], 3}, {NULL}};
const Segment SGI371157[] = {{&S10800[111], 6}, {NULL}};
const Segment SGI371158[] = {{&S10800[8], 3}, {NULL}};
const Segment SGI371159[] = {{&S10800[13], 9}, {NULL}};
const Segment SGI371160[] = {{&S10800[33], 12}, {NULL}};
const Segment SGI371161[] = {{&S10800[100], 3}, {NULL}};
const Segment SGI371162[] = {{&S10801[119], 6}, {NULL}};
const Segment SGI371163[] = {{&S10801[8], 3}, {NULL}};
const Segment SGI371164[] = {{&S10801[13], 9}, {NULL}};
const Segment SGI371165[] = {{&S10801[33], 15}, {NULL}};
const Segment SGI371166[] = {{&S10801[106], 3}, {NULL}};
const Segment SGI371167[] = {{&S10801[49], 6}, {NULL}};
const Segment SGI371168[] = {{&S10801[33], 13}, {NULL}};
const Segment SGI371169[] = {{&S10802[69], 5}, {NULL}};
const Segment SGI371170[] = {{&S10802[8], 3}, {NULL}};
const Segment SGI371171[] = {{&S10802[13], 9}, {NULL}};
const Segment SGI371172[] = {{&S10802[33], 15}, {NULL}};
const Segment SGI371173[] = {{&S10802[103], 3}, {NULL}};
const Segment SGI371174[] = {{&S10802[49], 6}, {NULL}};
const Segment SGI371175[] = {{&S10802[33], 13}, {NULL}};
const Segment SGI371176[] = {{&S10803[119], 5}, {NULL}};
const Segment SGI371177[] = {{&S10803[8], 3}, {NULL}};
const Segment SGI371178[] = {{&S10803[13], 9}, {NULL}};
const Segment SGI371179[] = {{&S10803[33], 15}, {NULL}};
const Segment SGI371180[] = {{&S10803[106], 3}, {NULL}};
const Segment SGI371181[] = {{&S10803[49], 6}, {NULL}};
const Segment SGI371182[] = {{&S10803[33], 13}, {NULL}};
const Segment SGI371183[] = {{&S10804[115], 6}, {NULL}};
const Segment SGI371184[] = {{&S10804[8], 3}, {NULL}};
const Segment SGI371185[] = {{&S10804[13], 9}, {NULL}};
const Segment SGI371186[] = {{&S10804[33], 15}, {NULL}};
const Segment SGI371187[] = {{&S10804[104], 3}, {NULL}};
const Segment SGI371188[] = {{&S10804[49], 6}, {NULL}};
const Segment SGI371189[] = {{&S10804[33], 13}, {NULL}};
const Segment SGI371190[] = {{&S10805[69], 6}, {NULL}};
const Segment SGI371191[] = {{&S10805[8], 3}, {NULL}};
const Segment SGI371192[] = {{&S10805[13], 9}, {NULL}};
const Segment SGI371193[] = {{&S10805[33], 15}, {NULL}};
const Segment SGI371194[] = {{&S10805[106], 3}, {NULL}};
const Segment SGI371195[] = {{&S10805[49], 6}, {NULL}};
const Segment SGI371196[] = {{&S10805[33], 13}, {NULL}};
const Segment SGI371197[] = {{&S10806[119], 6}, {NULL}};
const Segment SGI371198[] = {{&S10806[8], 3}, {NULL}};
const Segment SGI371199[] = {{&S10806[13], 9}, {NULL}};
const Segment SGI371200[] = {{&S10806[33], 15}, {NULL}};
const Segment SGI371201[] = {{&S10806[106], 3}, {NULL}};
const Segment SGI371202[] = {{&S10806[49], 6}, {NULL}};
const Segment SGI371203[] = {{&S10806[33], 13}, {NULL}};
const Segment SGI371204[] = {{&S10807[66], 5}, {NULL}};
const Segment SGI371205[] = {{&S10807[8], 3}, {NULL}};
const Segment SGI371206[] = {{&S10807[13], 9}, {NULL}};
const Segment SGI371207[] = {{&S10807[33], 15}, {NULL}};
const Segment SGI371208[] = {{&S10807[100], 3}, {NULL}};
const Segment SGI371209[] = {{&S10807[49], 3}, {NULL}};
const Segment SGI371210[] = {{&S10807[33], 13}, {NULL}};
const Segment SGI371211[] = {{&S10808[77], 6}, {NULL}};
const Segment SGI371212[] = {{&S10808[8], 3}, {NULL}};
const Segment SGI371213[] = {{&S10808[13], 9}, {NULL}};
const Segment SGI371214[] = {{&S10808[114], 3}, {NULL}};
const Segment SGI371215[] = {{&S10808[34], 13}, {NULL}};
const Segment SGI371216[] = {{&S11153[60], 6}, {NULL}};
const Segment SGI371217[] = {{&S11153[8], 3}, {NULL}};
const Segment SGI371218[] = {{&S11153[13], 7}, {NULL}};
const Segment SGI371219[] = {{&S11153[25], 14}, {NULL}};
const Segment SGI371220[] = {{&S11154[60], 6}, {NULL}};
const Segment SGI371221[] = {{&S11154[8], 3}, {NULL}};
const Segment SGI371222[] = {{&S11154[13], 7}, {NULL}};
const Segment SGI371223[] = {{&S11154[25], 14}, {NULL}};
const Segment SGI371224[] = {{&S11155[110], 6}, {NULL}};
const Segment SGI371225[] = {{&S11155[8], 3}, {NULL}};
const Segment SGI371226[] = {{&S11155[13], 7}, {NULL}};
const Segment SGI371227[] = {{&S11155[25], 14}, {NULL}};
const Segment SGI371228[] = {{&S11156[60], 6}, {NULL}};
const Segment SGI371229[] = {{&S11156[8], 3}, {NULL}};
const Segment SGI371230[] = {{&S11156[13], 7}, {NULL}};
const Segment SGI371231[] = {{&S11156[25], 14}, {NULL}};
const Segment SGI371232[] = {{&S11157[111], 6}, {NULL}};
const Segment SGI371233[] = {{&S11157[8], 3}, {NULL}};
const Segment SGI371234[] = {{&S11157[13], 7}, {NULL}};
const Segment SGI371235[] = {{&S11157[25], 14}, {NULL}};
const Segment SGI371236[] = {{&S11158[60], 6}, {NULL}};
const Segment SGI371237[] = {{&S11158[8], 3}, {NULL}};
const Segment SGI371238[] = {{&S11158[13], 7}, {NULL}};
const Segment SGI371239[] = {{&S11158[25], 14}, {NULL}};
const Segment SGI371240[] = {{&S11159[53], 6}, {NULL}};
const Segment SGI371241[] = {{&S11159[8], 3}, {NULL}};
const Segment SGI371242[] = {{&S11159[13], 7}, {NULL}};
const Segment SGI371243[] = {{&S11160[53], 6}, {NULL}};
const Segment SGI371244[] = {{&S11160[8], 3}, {NULL}};
const Segment SGI371245[] = {{&S11160[13], 7}, {NULL}};
const Segment SGI371246[] = {{&S11161[98], 6}, {NULL}};
const Segment SGI371247[] = {{&S11161[8], 3}, {NULL}};
const Segment SGI371248[] = {{&S11161[13], 14}, {NULL}};
const Segment SGI371249[] = {{&S11162[98], 6}, {NULL}};
const Segment SGI371250[] = {{&S11162[8], 3}, {NULL}};
const Segment SGI371251[] = {{&S11162[13], 14}, {NULL}};
const Segment SGI371252[] = {{&S11163[91], 6}, {NULL}};
const Segment SGI371253[] = {{&S11163[8], 3}, {NULL}};
const Segment SGI371254[] = {{&S11163[13], 14}, {NULL}};
const Segment SGI371255[] = {{&S11164[98], 6}, {NULL}};
const Segment SGI371256[] = {{&S11164[8], 3}, {NULL}};
const Segment SGI371257[] = {{&S11164[13], 14}, {NULL}};
const Segment SGI371258[] = {{&S11165[91], 6}, {NULL}};
const Segment SGI371259[] = {{&S11165[8], 3}, {NULL}};
const Segment SGI371260[] = {{&S11165[13], 14}, {NULL}};
const Segment SGI371261[] = {{&S11166[98], 6}, {NULL}};
const Segment SGI371262[] = {{&S11166[8], 3}, {NULL}};
const Segment SGI371263[] = {{&S11166[13], 14}, {NULL}};
const Segment SGI371264[] = {{&S11167[89], 6}, {NULL}};
const Segment SGI371265[] = {{&S11167[8], 3}, {NULL}};
const Segment SGI371266[] = {{&S11167[13], 14}, {NULL}};
const Segment SGI371267[] = {{&S11168[91], 6}, {NULL}};
const Segment SGI371268[] = {{&S11168[8], 3}, {NULL}};
const Segment SGI371269[] = {{&S11168[13], 14}, {NULL}};
const Segment SGI371270[] = {{&S11169[96], 6}, {NULL}};
const Segment SGI371271[] = {{&S11169[8], 3}, {NULL}};
const Segment SGI371272[] = {{&S11169[13], 14}, {NULL}};
const Segment SGI371273[] = {{&S11170[91], 6}, {NULL}};
const Segment SGI371274[] = {{&S11170[8], 3}, {NULL}};
const Segment SGI371275[] = {{&S11170[13], 14}, {NULL}};
const Segment SGI371276[] = {{&S11171[98], 6}, {NULL}};
const Segment SGI371277[] = {{&S11171[8], 3}, {NULL}};
const Segment SGI371278[] = {{&S11171[13], 14}, {NULL}};
const Segment SGI371279[] = {{&S11172[91], 6}, {NULL}};
const Segment SGI371280[] = {{&S11172[8], 3}, {NULL}};
const Segment SGI371281[] = {{&S11172[13], 14}, {NULL}};
const Segment SGI371282[] = {{&S11371[41], 6}, {NULL}};
const Segment SGI371283[] = {{&S11371[8], 3}, {NULL}};
const Segment SGI371284[] = {{&S11371[13], 14}, {NULL}};
const Segment SGI371285[] = {{&S11371[76], 3}, {NULL}};
const Segment SGI371286[] = {{&S11372[48], 6}, {NULL}};
const Segment SGI371287[] = {{&S11372[8], 3}, {NULL}};
const Segment SGI371288[] = {{&S11372[13], 14}, {NULL}};
const Segment SGI371289[] = {{&S11372[83], 3}, {NULL}};
const Segment SGI371290[] = {{&S11373[41], 6}, {NULL}};
const Segment SGI371291[] = {{&S11373[8], 3}, {NULL}};
const Segment SGI371292[] = {{&S11373[13], 14}, {NULL}};
const Segment SGI371293[] = {{&S11373[79], 3}, {NULL}};
const Segment SGI371294[] = {{&S11374[48], 6}, {NULL}};
const Segment SGI371295[] = {{&S11374[8], 3}, {NULL}};
const Segment SGI371296[] = {{&S11374[13], 14}, {NULL}};
const Segment SGI371297[] = {{&S11374[86], 3}, {NULL}};
const Segment SGI371298[] = {{&S11375[48], 6}, {NULL}};
const Segment SGI371299[] = {{&S11375[8], 3}, {NULL}};
const Segment SGI371300[] = {{&S11375[13], 14}, {NULL}};
const Segment SGI371301[] = {{&S11375[86], 3}, {NULL}};
const Segment SGI371302[] = {{&S11376[41], 6}, {NULL}};
const Segment SGI371303[] = {{&S11376[8], 3}, {NULL}};
const Segment SGI371304[] = {{&S11376[13], 14}, {NULL}};
const Segment SGI371305[] = {{&S11376[79], 3}, {NULL}};
const Segment SGI371306[] = {{&S11377[41], 6}, {NULL}};
const Segment SGI371307[] = {{&S11377[8], 3}, {NULL}};
const Segment SGI371308[] = {{&S11377[13], 14}, {NULL}};
const Segment SGI371309[] = {{&S11377[78], 3}, {NULL}};
const Segment SGI371310[] = {{&S11378[48], 6}, {NULL}};
const Segment SGI371311[] = {{&S11378[8], 3}, {NULL}};
const Segment SGI371312[] = {{&S11378[13], 14}, {NULL}};
const Segment SGI371313[] = {{&S11378[85], 3}, {NULL}};
const Segment SGI371314[] = {{&S11379[48], 6}, {NULL}};
const Segment SGI371315[] = {{&S11379[8], 3}, {NULL}};
const Segment SGI371316[] = {{&S11379[13], 14}, {NULL}};
const Segment SGI371317[] = {{&S11379[85], 3}, {NULL}};
const Segment SGI371318[] = {{&S11380[41], 6}, {NULL}};
const Segment SGI371319[] = {{&S11380[8], 3}, {NULL}};
const Segment SGI371320[] = {{&S11380[13], 14}, {NULL}};
const Segment SGI371321[] = {{&S11380[78], 3}, {NULL}};
const Segment SGI371322[] = {{&S11381[41], 6}, {NULL}};
const Segment SGI371323[] = {{&S11381[8], 3}, {NULL}};
const Segment SGI371324[] = {{&S11381[13], 14}, {NULL}};
const Segment SGI371325[] = {{&S11381[78], 3}, {NULL}};
const Segment SGI371326[] = {{&S11382[48], 6}, {NULL}};
const Segment SGI371327[] = {{&S11382[8], 3}, {NULL}};
const Segment SGI371328[] = {{&S11382[13], 14}, {NULL}};
const Segment SGI371329[] = {{&S11382[85], 3}, {NULL}};
const Segment SGI371330[] = {{&S11383[60], 6}, {NULL}};
const Segment SGI371331[] = {{&S11383[8], 3}, {NULL}};
const Segment SGI371332[] = {{&S11383[13], 7}, {NULL}};
const Segment SGI371333[] = {{&S11383[25], 14}, {NULL}};
const Segment SGI371334[] = {{&S11383[97], 3}, {NULL}};
const Segment SGI371335[] = {{&S11384[60], 6}, {NULL}};
const Segment SGI371336[] = {{&S11384[8], 3}, {NULL}};
const Segment SGI371337[] = {{&S11384[13], 7}, {NULL}};
const Segment SGI371338[] = {{&S11384[25], 14}, {NULL}};
const Segment SGI371339[] = {{&S11384[97], 3}, {NULL}};
const Segment SGI371340[] = {{&S11385[60], 6}, {NULL}};
const Segment SGI371341[] = {{&S11385[8], 3}, {NULL}};
const Segment SGI371342[] = {{&S11385[13], 7}, {NULL}};
const Segment SGI371343[] = {{&S11385[25], 14}, {NULL}};
const Segment SGI371344[] = {{&S11385[95], 3}, {NULL}};
const Segment SGI371345[] = {{&S11386[60], 6}, {NULL}};
const Segment SGI371346[] = {{&S11386[8], 3}, {NULL}};
const Segment SGI371347[] = {{&S11386[13], 7}, {NULL}};
const Segment SGI371348[] = {{&S11386[25], 14}, {NULL}};
const Segment SGI371349[] = {{&S11386[98], 3}, {NULL}};
const Segment SGI371350[] = {{&S11387[60], 6}, {NULL}};
const Segment SGI371351[] = {{&S11387[8], 3}, {NULL}};
const Segment SGI371352[] = {{&S11387[13], 7}, {NULL}};
const Segment SGI371353[] = {{&S11387[25], 14}, {NULL}};
const Segment SGI371354[] = {{&S11387[97], 3}, {NULL}};
const Segment SGI371355[] = {{&S11388[50], 6}, {NULL}};
const Segment SGI371356[] = {{&S11388[8], 3}, {NULL}};
const Segment SGI371357[] = {{&S11388[13], 7}, {NULL}};
const Segment SGI371358[] = {{&S11388[87], 3}, {NULL}};
const Segment SGI371359[] = {{&S11389[98], 6}, {NULL}};
const Segment SGI371360[] = {{&S11389[8], 3}, {NULL}};
const Segment SGI371361[] = {{&S11389[13], 14}, {NULL}};
const Segment SGI371362[] = {{&S11389[85], 3}, {NULL}};
const Segment SGI371363[] = {{&S11390[91], 6}, {NULL}};
const Segment SGI371364[] = {{&S11390[8], 3}, {NULL}};
const Segment SGI371365[] = {{&S11390[13], 14}, {NULL}};
const Segment SGI371366[] = {{&S11390[78], 3}, {NULL}};
const Segment SGI371367[] = {{&S13031[13], 43}, {NULL}};
const Segment SGI371368[] = {{&S13031[70], 5}, {NULL}};
const Segment SGI371369[] = {{&S13031[104], 3}, {NULL}};
const Segment SGI371370[] = {{&S13043[13], 43}, {NULL}};
const Segment SGI371371[] = {{&S13043[70], 5}, {NULL}};
const Segment SGI371372[] = {{&S13043[104], 3}, {NULL}};
const Segment SGI371373[] = {{&S12650[67], 6}, {NULL}};
const Segment SGI371374[] = {{&S12650[8], 3}, {NULL}};
const Segment SGI371375[] = {{&S12650[13], 9}, {NULL}};
const Segment SGI371376[] = {{&S12650[33], 16}, {NULL}};
const Segment SGI371377[] = {{&S12650[102], 3}, {NULL}};
const Segment SGI371378[] = {{&S12650[33], 13}, {NULL}};
const Segment SGI371379[] = {{&S12651[67], 6}, {NULL}};
const Segment SGI371380[] = {{&S12651[8], 3}, {NULL}};
const Segment SGI371381[] = {{&S12651[13], 9}, {NULL}};
const Segment SGI371382[] = {{&S12651[33], 16}, {NULL}};
const Segment SGI371383[] = {{&S12651[104], 3}, {NULL}};
const Segment SGI371384[] = {{&S12651[33], 13}, {NULL}};
const Segment SGI371385[] = {{&S12652[67], 6}, {NULL}};
const Segment SGI371386[] = {{&S12652[8], 3}, {NULL}};
const Segment SGI371387[] = {{&S12652[13], 9}, {NULL}};
const Segment SGI371388[] = {{&S12652[33], 16}, {NULL}};
const Segment SGI371389[] = {{&S12652[104], 3}, {NULL}};
const Segment SGI371390[] = {{&S12652[33], 13}, {NULL}};
const Segment SGI371391[] = {{&S12653[67], 6}, {NULL}};
const Segment SGI371392[] = {{&S12653[8], 3}, {NULL}};
const Segment SGI371393[] = {{&S12653[13], 9}, {NULL}};
const Segment SGI371394[] = {{&S12653[33], 16}, {NULL}};
const Segment SGI371395[] = {{&S12653[104], 3}, {NULL}};
const Segment SGI371396[] = {{&S12653[33], 13}, {NULL}};
const Segment SGI371397[] = {{&S12654[103], 7}, {NULL}};
const Segment SGI371398[] = {{&S12654[8], 3}, {NULL}};
const Segment SGI371399[] = {{&S12654[13], 9}, {NULL}};
const Segment SGI371400[] = {{&S12654[33], 16}, {NULL}};
const Segment SGI371401[] = {{&S12654[33], 13}, {NULL}};
const Segment SGI371402[] = {{&S12655[111], 6}, {NULL}};
const Segment SGI371403[] = {{&S12655[8], 3}, {NULL}};
const Segment SGI371404[] = {{&S12655[13], 9}, {NULL}};
const Segment SGI371405[] = {{&S12655[31], 16}, {NULL}};
const Segment SGI371406[] = {{&S12655[100], 3}, {NULL}};
const Segment SGI371407[] = {{&S12655[31], 13}, {NULL}};
const Segment SGI371408[] = {{&S12656[64], 6}, {NULL}};
const Segment SGI371409[] = {{&S12656[8], 3}, {NULL}};
const Segment SGI371410[] = {{&S12656[30], 16}, {NULL}};
const Segment SGI371411[] = {{&S12656[101], 3}, {NULL}};
const Segment SGI371412[] = {{&S12656[30], 13}, {NULL}};
const Segment SGI371413[] = {{&S12657[69], 6}, {NULL}};
const Segment SGI371414[] = {{&S12657[8], 3}, {NULL}};
const Segment SGI371415[] = {{&S12657[13], 9}, {NULL}};
const Segment SGI371416[] = {{&S12657[33], 18}, {NULL}};
const Segment SGI371417[] = {{&S12657[106], 3}, {NULL}};
const Segment SGI371418[] = {{&S14539[13], 43}, {NULL}};
const Segment SGI371419[] = {{&S14539[70], 5}, {NULL}};
const Segment SGI371420[] = {{&S14539[104], 3}, {NULL}};
const Segment SGI371421[] = {{&S14548[13], 43}, {NULL}};
const Segment SGI371422[] = {{&S14548[70], 5}, {NULL}};
const Segment SGI371423[] = {{&S14548[104], 3}, {NULL}};
const Segment SGI371424[] = {{&S14549[13], 43}, {NULL}};
const Segment SGI371425[] = {{&S14549[70], 5}, {NULL}};
const Segment SGI371426[] = {{&S14549[104], 3}, {NULL}};
const Segment SGI371427[] = {{&S14550[13], 43}, {NULL}};
const Segment SGI371428[] = {{&S14550[70], 5}, {NULL}};
const Segment SGI371429[] = {{&S14550[104], 3}, {NULL}};
const Segment SGI371430[] = {{&S14551[13], 43}, {NULL}};
const Segment SGI371431[] = {{&S14551[70], 5}, {NULL}};
const Segment SGI371432[] = {{&S14551[104], 3}, {NULL}};
const Segment SGI371433[] = {{&S14552[13], 43}, {NULL}};
const Segment SGI371434[] = {{&S14552[70], 5}, {NULL}};
const Segment SGI371435[] = {{&S14552[104], 3}, {NULL}};
const Segment SGI371436[] = {{&S14553[13], 43}, {NULL}};
const Segment SGI371437[] = {{&S14553[70], 5}, {NULL}};
const Segment SGI371438[] = {{&S14553[104], 3}, {NULL}};
const Segment SGI371439[] = {{&S14554[13], 43}, {NULL}};
const Segment SGI371440[] = {{&S14554[70], 5}, {NULL}};
const Segment SGI371441[] = {{&S14554[104], 3}, {NULL}};
const Segment SGI371442[] = {{&S14563[13], 43}, {NULL}};
const Segment SGI371443[] = {{&S14563[70], 5}, {NULL}};
const Segment SGI371444[] = {{&S14563[104], 3}, {NULL}};
const Segment SGI371445[] = {{&S14565[13], 43}, {NULL}};
const Segment SGI371446[] = {{&S14565[70], 5}, {NULL}};
const Segment SGI371447[] = {{&S14565[104], 3}, {NULL}};
const Segment SGI371448[] = {{&S14573[13], 43}, {NULL}};
const Segment SGI371449[] = {{&S14573[70], 5}, {NULL}};
const Segment SGI371450[] = {{&S14573[104], 3}, {NULL}};
const Segment SGI371451[] = {{&S13197[8], 3}, {NULL}};
const Segment SGI371452[] = {{&S13197[13], 5}, {NULL}};
const Segment SGI371453[] = {{&S13197[49], 3}, {NULL}};
const Segment SGI371454[] = {{&S13319[8], 3}, {NULL}};
const Segment SGI371455[] = {{&S13319[13], 5}, {NULL}};
const Segment SGI371456[] = {{&S13319[54], 3}, {NULL}};
const Segment SGI371457[] = {{&S17100[13], 58}, {NULL}};
const Segment SGI371458[] = {{&S17100[85], 5}, {NULL}};
const Segment SGI371459[] = {{&S17100[119], 3}, {NULL}};
const Segment SGI371460[] = {{&S15686[96], 5}, {NULL}};
const Segment SGI371461[] = {{&S15686[8], 3}, {NULL}};
const Segment SGI371462[] = {{&S15686[13], 9}, {NULL}};
const Segment SGI371463[] = {{&S15686[30], 13}, {NULL}};
const Segment SGI371464[] = {{&S15695[59], 6}, {NULL}};
const Segment SGI371465[] = {{&S15695[8], 3}, {NULL}};
const Segment SGI371466[] = {{&S15695[13], 9}, {NULL}};
const Segment SGI371467[] = {{&S15695[30], 13}, {NULL}};
const Segment SGI371468[] = {{&S15707[127], 6}, {NULL}};
const Segment SGI371469[] = {{&S15707[8], 3}, {NULL}};
const Segment SGI371470[] = {{&S15707[13], 9}, {NULL}};
const Segment SGI371471[] = {{&S15707[30], 13}, {NULL}};
const Segment SGI371472[] = {{&S15709[59], 6}, {NULL}};
const Segment SGI371473[] = {{&S15709[8], 3}, {NULL}};
const Segment SGI371474[] = {{&S15709[13], 9}, {NULL}};
const Segment SGI371475[] = {{&S15709[30], 13}, {NULL}};
const Segment SGI371476[] = {{&S15715[96], 5}, {NULL}};
const Segment SGI371477[] = {{&S15715[8], 3}, {NULL}};
const Segment SGI371478[] = {{&S15715[13], 9}, {NULL}};
const Segment SGI371479[] = {{&S15715[30], 13}, {NULL}};
const Segment SGI371480[] = {{&S15716[59], 6}, {NULL}};
const Segment SGI371481[] = {{&S15716[8], 3}, {NULL}};
const Segment SGI371482[] = {{&S15716[13], 9}, {NULL}};
const Segment SGI371483[] = {{&S15716[30], 13}, {NULL}};
const Segment SGI371484[] = {{&S15718[95], 5}, {NULL}};
const Segment SGI371485[] = {{&S15718[8], 3}, {NULL}};
const Segment SGI371486[] = {{&S15718[13], 9}, {NULL}};
const Segment SGI371487[] = {{&S15718[30], 13}, {NULL}};
const Segment SGI371488[] = {{&S15723[8], 3}, {NULL}};
const Segment SGI371489[] = {{&S15723[13], 9}, {NULL}};
const Segment SGI371490[] = {{&S15723[30], 13}, {NULL}};
const Segment SGI371491[] = {{&S15724[59], 6}, {NULL}};
const Segment SGI371492[] = {{&S15724[8], 3}, {NULL}};
const Segment SGI371493[] = {{&S15724[13], 9}, {NULL}};
const Segment SGI371494[] = {{&S15724[30], 13}, {NULL}};
const Segment SGI371495[] = {{&S18152[13], 43}, {NULL}};
const Segment SGI371496[] = {{&S18152[70], 5}, {NULL}};
const Segment SGI371497[] = {{&S18152[104], 3}, {NULL}};
const Segment SGI371498[] = {{&S18157[13], 43}, {NULL}};
const Segment SGI371499[] = {{&S18157[70], 5}, {NULL}};
const Segment SGI371500[] = {{&S18157[104], 3}, {NULL}};
const Segment SGI371501[] = {{&S18158[13], 43}, {NULL}};
const Segment SGI371502[] = {{&S18158[70], 5}, {NULL}};
const Segment SGI371503[] = {{&S18158[104], 3}, {NULL}};
const Segment SGI371504[] = {{&S18159[13], 43}, {NULL}};
const Segment SGI371505[] = {{&S18159[70], 5}, {NULL}};
const Segment SGI371506[] = {{&S18159[104], 3}, {NULL}};
const Segment SGI371507[] = {{&S18167[13], 43}, {NULL}};
const Segment SGI371508[] = {{&S18167[70], 5}, {NULL}};
const Segment SGI371509[] = {{&S18167[104], 3}, {NULL}};
const Segment SGI371510[] = {{&S18184[13], 17}, {NULL}};
const Segment SGI371511[] = {{&S18184[44], 6}, {NULL}};
const Segment SGI371512[] = {{&S18185[13], 21}, {NULL}};
const Segment SGI371513[] = {{&S18185[48], 6}, {NULL}};
const Segment SGI371514[] = {{&S18186[13], 21}, {NULL}};
const Segment SGI371515[] = {{&S18186[48], 6}, {NULL}};
const Segment SGI371516[] = {{&S18187[13], 21}, {NULL}};
const Segment SGI371517[] = {{&S18187[48], 6}, {NULL}};
const Segment SGI371518[] = {{&S18189[13], 24}, {NULL}};
const Segment SGI371519[] = {{&S18189[51], 5}, {NULL}};
const Segment SGI371520[] = {{&S18189[93], 6}, {NULL}};
const Segment SGI371521[] = {{&S18190[13], 24}, {NULL}};
const Segment SGI371522[] = {{&S18190[51], 5}, {NULL}};
const Segment SGI371523[] = {{&S18190[93], 6}, {NULL}};
const Segment SGI371524[] = {{&S18191[13], 21}, {NULL}};
const Segment SGI371525[] = {{&S18191[48], 6}, {NULL}};
const Segment SGI371526[] = {{&S18904[13], 43}, {NULL}};
const Segment SGI371527[] = {{&S18904[70], 5}, {NULL}};
const Segment SGI371528[] = {{&S18904[104], 3}, {NULL}};
const Segment SGI371529[] = {{&S18908[13], 43}, {NULL}};
const Segment SGI371530[] = {{&S18908[70], 5}, {NULL}};
const Segment SGI371531[] = {{&S18908[104], 3}, {NULL}};
const Segment SGI371532[] = {{&S18912[13], 43}, {NULL}};
const Segment SGI371533[] = {{&S18912[70], 5}, {NULL}};
const Segment SGI371534[] = {{&S18912[104], 3}, {NULL}};
const Segment SGI371535[] = {{&S17362[60], 6}, {NULL}};
const Segment SGI371536[] = {{&S17362[8], 3}, {NULL}};
const Segment SGI371537[] = {{&S17362[13], 7}, {NULL}};
const Segment SGI371538[] = {{&S17362[25], 14}, {NULL}};
const Segment SGI371539[] = {{&S17363[60], 6}, {NULL}};
const Segment SGI371540[] = {{&S17363[8], 3}, {NULL}};
const Segment SGI371541[] = {{&S17363[13], 7}, {NULL}};
const Segment SGI371542[] = {{&S17363[25], 14}, {NULL}};
const Segment SGI371543[] = {{&S17364[60], 6}, {NULL}};
const Segment SGI371544[] = {{&S17364[8], 3}, {NULL}};
const Segment SGI371545[] = {{&S17364[13], 7}, {NULL}};
const Segment SGI371546[] = {{&S17364[25], 14}, {NULL}};
const Segment SGI371547[] = {{&S17365[60], 6}, {NULL}};
const Segment SGI371548[] = {{&S17365[8], 3}, {NULL}};
const Segment SGI371549[] = {{&S17365[13], 7}, {NULL}};
const Segment SGI371550[] = {{&S17365[25], 14}, {NULL}};
const Segment SGI371551[] = {{&S18918[13], 43}, {NULL}};
const Segment SGI371552[] = {{&S18918[70], 5}, {NULL}};
const Segment SGI371553[] = {{&S18918[104], 3}, {NULL}};
const Segment SGI371554[] = {{&S17366[60], 6}, {NULL}};
const Segment SGI371555[] = {{&S17366[8], 3}, {NULL}};
const Segment SGI371556[] = {{&S18919[13], 43}, {NULL}};
const Segment SGI371557[] = {{&S17366[13], 7}, {NULL}};
const Segment SGI371558[] = {{&S18919[70], 5}, {NULL}};
const Segment SGI371559[] = {{&S17366[25], 14}, {NULL}};
const Segment SGI371560[] = {{&S18919[104], 3}, {NULL}};
const Segment SGI371561[] = {{&S18920[13], 43}, {NULL}};
const Segment SGI371562[] = {{&S18920[70], 5}, {NULL}};
const Segment SGI371563[] = {{&S18920[104], 3}, {NULL}};
const Segment SGI371564[] = {{&S17367[111], 6}, {NULL}};
const Segment SGI371565[] = {{&S17367[8], 3}, {NULL}};
const Segment SGI371566[] = {{&S17367[13], 7}, {NULL}};
const Segment SGI371567[] = {{&S17367[25], 14}, {NULL}};
const Segment SGI371568[] = {{&S17368[98], 6}, {NULL}};
const Segment SGI371569[] = {{&S17368[8], 3}, {NULL}};
const Segment SGI371570[] = {{&S17368[13], 14}, {NULL}};
const Segment SGI371571[] = {{&S17369[96], 6}, {NULL}};
const Segment SGI371572[] = {{&S17369[8], 3}, {NULL}};
const Segment SGI371573[] = {{&S17369[13], 14}, {NULL}};
const Segment SGI371574[] = {{&S17370[91], 6}, {NULL}};
const Segment SGI371575[] = {{&S17370[8], 3}, {NULL}};
const Segment SGI371576[] = {{&S17370[13], 14}, {NULL}};
const Segment SGI371577[] = {{&S17371[91], 6}, {NULL}};
const Segment SGI371578[] = {{&S17371[8], 3}, {NULL}};
const Segment SGI371579[] = {{&S17371[13], 14}, {NULL}};
const Segment SGI371580[] = {{&S17372[91], 6}, {NULL}};
const Segment SGI371581[] = {{&S17372[8], 3}, {NULL}};
const Segment SGI371582[] = {{&S17372[13], 14}, {NULL}};
const Segment SGI371583[] = {{&S17373[98], 6}, {NULL}};
const Segment SGI371584[] = {{&S17373[8], 3}, {NULL}};
const Segment SGI371585[] = {{&S17373[13], 14}, {NULL}};
const Segment SGI371586[] = {{&S17374[98], 6}, {NULL}};
const Segment SGI371587[] = {{&S17374[8], 3}, {NULL}};
const Segment SGI371588[] = {{&S17374[13], 14}, {NULL}};
const Segment SGI371589[] = {{&S17375[89], 6}, {NULL}};
const Segment SGI371590[] = {{&S17375[8], 3}, {NULL}};
const Segment SGI371591[] = {{&S17375[13], 14}, {NULL}};
const Segment SGI371592[] = {{&S17376[91], 6}, {NULL}};
const Segment SGI371593[] = {{&S17376[8], 3}, {NULL}};
const Segment SGI371594[] = {{&S17376[13], 14}, {NULL}};
const Segment SGI371595[] = {{&S17377[91], 6}, {NULL}};
const Segment SGI371596[] = {{&S17377[8], 3}, {NULL}};
const Segment SGI371597[] = {{&S17377[13], 14}, {NULL}};
const Segment SGI371598[] = {{&S17378[98], 6}, {NULL}};
const Segment SGI371599[] = {{&S17378[8], 3}, {NULL}};
const Segment SGI371600[] = {{&S17378[13], 14}, {NULL}};
const Segment SGI371601[] = {{&S17379[98], 6}, {NULL}};
const Segment SGI371602[] = {{&S17379[8], 3}, {NULL}};
const Segment SGI371603[] = {{&S17379[13], 14}, {NULL}};
const Segment SGI371604[] = {{&S18181[50], 5}, {NULL}};
const Segment SGI371605[] = {{&S18181[8], 3}, {NULL}};
const Segment SGI371606[] = {{&S18181[13], 3}, {NULL}};
const Segment SGI371607[] = {{&S18181[76], 6}, {NULL}};
const Segment SGI371608[] = {{&S18182[44], 6}, {NULL}};
const Segment SGI371609[] = {{&S18182[8], 3}, {NULL}};
const Segment SGI371610[] = {{&S18182[13], 3}, {NULL}};
const Segment SGI371611[] = {{&S18182[71], 6}, {NULL}};
const Segment SGI371612[] = {{&S18188[44], 6}, {NULL}};
const Segment SGI371613[] = {{&S18188[8], 3}, {NULL}};
const Segment SGI371614[] = {{&S18188[13], 3}, {NULL}};
const Segment SGI371615[] = {{&S18188[71], 6}, {NULL}};
const Segment SGI371616[] = {{&S19947[13], 19}, {NULL}};
const Segment SGI371617[] = {{&S19947[46], 6}, {NULL}};
const Segment SGI371618[] = {{&S19949[13], 23}, {NULL}};
const Segment SGI371619[] = {{&S19949[50], 5}, {NULL}};
const Segment SGI371620[] = {{&S19952[13], 27}, {NULL}};
const Segment SGI371621[] = {{&S19952[54], 5}, {NULL}};
const Segment SGI371622[] = {{&S19953[13], 27}, {NULL}};
const Segment SGI371623[] = {{&S19953[54], 5}, {NULL}};
const Segment SGI371624[] = {{&S19954[13], 17}, {NULL}};
const Segment SGI371625[] = {{&S19954[44], 6}, {NULL}};
const Segment SGI371626[] = {{&S19955[13], 27}, {NULL}};
const Segment SGI371627[] = {{&S19955[54], 5}, {NULL}};
const Segment SGI371628[] = {{&S18580[60], 6}, {NULL}};
const Segment SGI371629[] = {{&S18580[8], 3}, {NULL}};
const Segment SGI371630[] = {{&S18580[13], 7}, {NULL}};
const Segment SGI371631[] = {{&S18580[25], 14}, {NULL}};
const Segment SGI371632[] = {{&S18581[60], 6}, {NULL}};
const Segment SGI371633[] = {{&S18581[8], 3}, {NULL}};
const Segment SGI371634[] = {{&S18581[13], 7}, {NULL}};
const Segment SGI371635[] = {{&S18581[25], 14}, {NULL}};
const Segment SGI371636[] = {{&S18582[60], 6}, {NULL}};
const Segment SGI371637[] = {{&S18582[8], 3}, {NULL}};
const Segment SGI371638[] = {{&S18582[13], 7}, {NULL}};
const Segment SGI371639[] = {{&S18582[25], 14}, {NULL}};
const Segment SGI371640[] = {{&S18583[60], 6}, {NULL}};
const Segment SGI371641[] = {{&S18583[8], 3}, {NULL}};
const Segment SGI371642[] = {{&S18583[13], 7}, {NULL}};
const Segment SGI371643[] = {{&S18583[25], 14}, {NULL}};
const Segment SGI371644[] = {{&S18584[60], 6}, {NULL}};
const Segment SGI371645[] = {{&S18584[8], 3}, {NULL}};
const Segment SGI371646[] = {{&S18584[13], 7}, {NULL}};
const Segment SGI371647[] = {{&S18584[25], 14}, {NULL}};
const Segment SGI371648[] = {{&S18585[60], 6}, {NULL}};
const Segment SGI371649[] = {{&S18585[8], 3}, {NULL}};
const Segment SGI371650[] = {{&S18585[13], 7}, {NULL}};
const Segment SGI371651[] = {{&S18585[25], 14}, {NULL}};
const Segment SGI371652[] = {{&S18586[60], 6}, {NULL}};
const Segment SGI371653[] = {{&S18586[8], 3}, {NULL}};
const Segment SGI371654[] = {{&S18586[13], 7}, {NULL}};
const Segment SGI371655[] = {{&S18586[25], 14}, {NULL}};
const Segment SGI371656[] = {{&S18587[72], 5}, {NULL}};
const Segment SGI371657[] = {{&S18587[8], 3}, {NULL}};
const Segment SGI371658[] = {{&S18587[13], 7}, {NULL}};
const Segment SGI371659[] = {{&S18587[25], 14}, {NULL}};
const Segment SGI371660[] = {{&S18588[60], 6}, {NULL}};
const Segment SGI371661[] = {{&S18588[8], 3}, {NULL}};
const Segment SGI371662[] = {{&S18588[13], 7}, {NULL}};
const Segment SGI371663[] = {{&S18588[25], 14}, {NULL}};
const Segment SGI371664[] = {{&S18589[60], 6}, {NULL}};
const Segment SGI371665[] = {{&S18589[8], 3}, {NULL}};
const Segment SGI371666[] = {{&S18589[13], 7}, {NULL}};
const Segment SGI371667[] = {{&S18589[25], 14}, {NULL}};
const Segment SGI371668[] = {{&S18590[60], 6}, {NULL}};
const Segment SGI371669[] = {{&S18590[8], 3}, {NULL}};
const Segment SGI371670[] = {{&S18590[13], 7}, {NULL}};
const Segment SGI371671[] = {{&S18590[25], 14}, {NULL}};
const Segment SGI371672[] = {{&S18591[72], 5}, {NULL}};
const Segment SGI371673[] = {{&S18591[8], 3}, {NULL}};
const Segment SGI371674[] = {{&S18591[13], 7}, {NULL}};
const Segment SGI371675[] = {{&S18591[25], 14}, {NULL}};
const Segment SGI371676[] = {{&S18592[60], 6}, {NULL}};
const Segment SGI371677[] = {{&S18592[8], 3}, {NULL}};
const Segment SGI371678[] = {{&S18592[13], 7}, {NULL}};
const Segment SGI371679[] = {{&S18592[25], 14}, {NULL}};
const Segment SGI371680[] = {{&S18593[60], 5}, {NULL}};
const Segment SGI371681[] = {{&S18593[8], 3}, {NULL}};
const Segment SGI371682[] = {{&S18593[13], 7}, {NULL}};
const Segment SGI371683[] = {{&S18593[25], 14}, {NULL}};
const Segment SGI371684[] = {{&S18594[98], 6}, {NULL}};
const Segment SGI371685[] = {{&S18594[8], 3}, {NULL}};
const Segment SGI371686[] = {{&S18594[13], 14}, {NULL}};
const Segment SGI371687[] = {{&S18595[60], 6}, {NULL}};
const Segment SGI371688[] = {{&S18595[8], 3}, {NULL}};
const Segment SGI371689[] = {{&S18595[13], 7}, {NULL}};
const Segment SGI371690[] = {{&S18595[25], 14}, {NULL}};
const Segment SGI371691[] = {{&S18596[60], 6}, {NULL}};
const Segment SGI371692[] = {{&S18596[8], 3}, {NULL}};
const Segment SGI371693[] = {{&S18596[13], 7}, {NULL}};
const Segment SGI371694[] = {{&S18596[25], 14}, {NULL}};
const Segment SGI371695[] = {{&S18597[96], 6}, {NULL}};
const Segment SGI371696[] = {{&S18597[8], 3}, {NULL}};
const Segment SGI371697[] = {{&S18597[13], 14}, {NULL}};
const Segment SGI371698[] = {{&S18598[41], 6}, {NULL}};
const Segment SGI371699[] = {{&S18598[8], 3}, {NULL}};
const Segment SGI371700[] = {{&S18598[13], 14}, {NULL}};
const Segment SGI371701[] = {{&S18599[98], 6}, {NULL}};
const Segment SGI371702[] = {{&S18599[8], 3}, {NULL}};
const Segment SGI371703[] = {{&S18599[13], 14}, {NULL}};
const Segment SGI371704[] = {{&S18600[98], 6}, {NULL}};
const Segment SGI371705[] = {{&S18600[8], 3}, {NULL}};
const Segment SGI371706[] = {{&S18600[13], 14}, {NULL}};
const Segment SGI371707[] = {{&S18601[48], 6}, {NULL}};
const Segment SGI371708[] = {{&S18601[8], 3}, {NULL}};
const Segment SGI371709[] = {{&S18601[13], 14}, {NULL}};
const Segment SGI371710[] = {{&S18602[91], 6}, {NULL}};
const Segment SGI371711[] = {{&S18602[8], 3}, {NULL}};
const Segment SGI371712[] = {{&S18602[13], 14}, {NULL}};
const Segment SGI371713[] = {{&S18603[98], 6}, {NULL}};
const Segment SGI371714[] = {{&S18603[8], 3}, {NULL}};
const Segment SGI371715[] = {{&S18603[13], 14}, {NULL}};
const Segment SGI371716[] = {{&S18604[91], 6}, {NULL}};
const Segment SGI371717[] = {{&S18604[8], 3}, {NULL}};
const Segment SGI371718[] = {{&S18604[13], 14}, {NULL}};
const Segment SGI371719[] = {{&S18605[89], 6}, {NULL}};
const Segment SGI371720[] = {{&S18605[8], 3}, {NULL}};
const Segment SGI371721[] = {{&S18605[13], 14}, {NULL}};
const Segment SGI371722[] = {{&S18606[91], 6}, {NULL}};
const Segment SGI371723[] = {{&S18606[8], 3}, {NULL}};
const Segment SGI371724[] = {{&S18606[13], 14}, {NULL}};
const Segment SGI371725[] = {{&S18607[91], 6}, {NULL}};
const Segment SGI371726[] = {{&S18607[8], 3}, {NULL}};
const Segment SGI371727[] = {{&S18607[13], 14}, {NULL}};
const Segment SGI371728[] = {{&S18608[91], 6}, {NULL}};
const Segment SGI371729[] = {{&S18608[8], 3}, {NULL}};
const Segment SGI371730[] = {{&S18608[13], 14}, {NULL}};
const Segment SGI371731[] = {{&S18609[99], 6}, {NULL}};
const Segment SGI371732[] = {{&S18609[8], 3}, {NULL}};
const Segment SGI371733[] = {{&S18609[13], 14}, {NULL}};
const Segment SGI371734[] = {{&S18610[60], 5}, {NULL}};
const Segment SGI371735[] = {{&S18610[8], 3}, {NULL}};
const Segment SGI371736[] = {{&S18610[13], 7}, {NULL}};
const Segment SGI371737[] = {{&S18610[25], 14}, {NULL}};
const Segment SGI371738[] = {{&S19190[41], 6}, {NULL}};
const Segment SGI371739[] = {{&S19190[8], 3}, {NULL}};
const Segment SGI371740[] = {{&S19190[13], 14}, {NULL}};
const Segment SGI371741[] = {{&S19190[76], 3}, {NULL}};
const Segment SGI371742[] = {{&S19191[48], 6}, {NULL}};
const Segment SGI371743[] = {{&S19191[8], 3}, {NULL}};
const Segment SGI371744[] = {{&S19191[13], 14}, {NULL}};
const Segment SGI371745[] = {{&S19191[83], 3}, {NULL}};
const Segment SGI371746[] = {{&S19192[41], 6}, {NULL}};
const Segment SGI371747[] = {{&S19192[8], 3}, {NULL}};
const Segment SGI371748[] = {{&S19192[13], 14}, {NULL}};
const Segment SGI371749[] = {{&S19192[79], 3}, {NULL}};
const Segment SGI371750[] = {{&S19193[48], 6}, {NULL}};
const Segment SGI371751[] = {{&S19193[8], 3}, {NULL}};
const Segment SGI371752[] = {{&S19193[13], 14}, {NULL}};
const Segment SGI371753[] = {{&S19193[86], 3}, {NULL}};
const Segment SGI371754[] = {{&S19194[48], 6}, {NULL}};
const Segment SGI371755[] = {{&S19194[8], 3}, {NULL}};
const Segment SGI371756[] = {{&S19194[13], 14}, {NULL}};
const Segment SGI371757[] = {{&S19194[85], 3}, {NULL}};
const Segment SGI371758[] = {{&S19195[41], 6}, {NULL}};
const Segment SGI371759[] = {{&S19195[8], 3}, {NULL}};
const Segment SGI371760[] = {{&S19195[13], 14}, {NULL}};
const Segment SGI371761[] = {{&S19195[79], 3}, {NULL}};
const Segment SGI371762[] = {{&S19196[48], 6}, {NULL}};
const Segment SGI371763[] = {{&S19196[8], 3}, {NULL}};
const Segment SGI371764[] = {{&S19196[13], 14}, {NULL}};
const Segment SGI371765[] = {{&S19196[86], 3}, {NULL}};
const Segment SGI371766[] = {{&S19197[48], 6}, {NULL}};
const Segment SGI371767[] = {{&S19197[8], 3}, {NULL}};
const Segment SGI371768[] = {{&S19197[13], 14}, {NULL}};
const Segment SGI371769[] = {{&S19197[85], 3}, {NULL}};
const Segment SGI371770[] = {{&S19198[41], 6}, {NULL}};
const Segment SGI371771[] = {{&S19198[8], 3}, {NULL}};
const Segment SGI371772[] = {{&S19198[13], 14}, {NULL}};
const Segment SGI371773[] = {{&S19198[78], 3}, {NULL}};
const Segment SGI371774[] = {{&S19199[41], 6}, {NULL}};
const Segment SGI371775[] = {{&S19199[8], 3}, {NULL}};
const Segment SGI371776[] = {{&S19199[13], 14}, {NULL}};
const Segment SGI371777[] = {{&S19199[78], 3}, {NULL}};
const Segment SGI371778[] = {{&S19200[48], 6}, {NULL}};
const Segment SGI371779[] = {{&S19200[8], 3}, {NULL}};
const Segment SGI371780[] = {{&S19200[13], 14}, {NULL}};
const Segment SGI371781[] = {{&S19200[85], 3}, {NULL}};
const Segment SGI371782[] = {{&S19201[41], 6}, {NULL}};
const Segment SGI371783[] = {{&S19201[8], 3}, {NULL}};
const Segment SGI371784[] = {{&S19201[13], 14}, {NULL}};
const Segment SGI371785[] = {{&S19201[78], 3}, {NULL}};
const Segment SGI371786[] = {{&S19202[91], 6}, {NULL}};
const Segment SGI371787[] = {{&S19202[8], 3}, {NULL}};
const Segment SGI371788[] = {{&S19202[13], 14}, {NULL}};
const Segment SGI371789[] = {{&S19202[78], 3}, {NULL}};
const Segment SGI371790[] = {{&S19203[98], 6}, {NULL}};
const Segment SGI371791[] = {{&S19203[8], 3}, {NULL}};
const Segment SGI371792[] = {{&S19203[13], 14}, {NULL}};
const Segment SGI371793[] = {{&S19203[85], 3}, {NULL}};
const Segment SGI371794[] = {{&S21089[13], 43}, {NULL}};
const Segment SGI371795[] = {{&S21089[70], 5}, {NULL}};
const Segment SGI371796[] = {{&S21089[104], 3}, {NULL}};
const Segment SGI371797[] = {{&S21099[13], 43}, {NULL}};
const Segment SGI371798[] = {{&S21099[70], 5}, {NULL}};
const Segment SGI371799[] = {{&S21099[104], 3}, {NULL}};
const Segment SGI371800[] = {{&S21101[13], 43}, {NULL}};
const Segment SGI371801[] = {{&S21101[70], 5}, {NULL}};
const Segment SGI371802[] = {{&S21101[104], 3}, {NULL}};
const Segment SGI371803[] = {{&S21102[13], 43}, {NULL}};
const Segment SGI371804[] = {{&S21102[70], 5}, {NULL}};
const Segment SGI371805[] = {{&S21102[104], 3}, {NULL}};
const Segment SGI371806[] = {{&S21103[13], 43}, {NULL}};
const Segment SGI371807[] = {{&S21103[70], 5}, {NULL}};
const Segment SGI371808[] = {{&S21103[104], 3}, {NULL}};
const Segment SGI371809[] = {{&S21389[13], 43}, {NULL}};
const Segment SGI371810[] = {{&S21389[70], 5}, {NULL}};
const Segment SGI371811[] = {{&S21389[104], 3}, {NULL}};
const Segment SGI371812[] = {{&S21396[13], 43}, {NULL}};
const Segment SGI371813[] = {{&S21396[70], 5}, {NULL}};
const Segment SGI371814[] = {{&S21396[104], 3}, {NULL}};
const Segment SGI371815[] = {{&S21397[13], 43}, {NULL}};
const Segment SGI371816[] = {{&S21397[70], 5}, {NULL}};
const Segment SGI371817[] = {{&S21397[104], 3}, {NULL}};
const Segment SGI371818[] = {{&S21401[13], 43}, {NULL}};
const Segment SGI371819[] = {{&S21401[70], 5}, {NULL}};
const Segment SGI371820[] = {{&S21401[104], 3}, {NULL}};
const Segment SGI371821[] = {{&S21402[13], 43}, {NULL}};
const Segment SGI371822[] = {{&S21402[70], 5}, {NULL}};
const Segment SGI371823[] = {{&S21402[104], 3}, {NULL}};
const Segment SGI371824[] = {{&S19876[59], 5}, {NULL}};
const Segment SGI371825[] = {{&S19876[8], 3}, {NULL}};
const Segment SGI371826[] = {{&S19876[13], 9}, {NULL}};
const Segment SGI371827[] = {{&S19876[30], 13}, {NULL}};
const Segment SGI371828[] = {{&S19882[116], 6}, {NULL}};
const Segment SGI371829[] = {{&S19882[8], 3}, {NULL}};
const Segment SGI371830[] = {{&S19882[13], 9}, {NULL}};
const Segment SGI371831[] = {{&S19882[30], 13}, {NULL}};
const Segment SGI371832[] = {{&S19896[95], 5}, {NULL}};
const Segment SGI371833[] = {{&S19896[8], 3}, {NULL}};
const Segment SGI371834[] = {{&S19896[13], 9}, {NULL}};
const Segment SGI371835[] = {{&S19896[30], 13}, {NULL}};
const Segment SGI371836[] = {{&S19903[95], 5}, {NULL}};
const Segment SGI371837[] = {{&S19903[8], 3}, {NULL}};
const Segment SGI371838[] = {{&S19903[13], 9}, {NULL}};
const Segment SGI371839[] = {{&S19903[30], 13}, {NULL}};
const Segment SGI371840[] = {{&S19909[8], 3}, {NULL}};
const Segment SGI371841[] = {{&S19909[13], 9}, {NULL}};
const Segment SGI371842[] = {{&S19909[30], 13}, {NULL}};
const Segment SGI371843[] = {{&S19915[95], 5}, {NULL}};
const Segment SGI371844[] = {{&S19915[8], 3}, {NULL}};
const Segment SGI371845[] = {{&S19915[13], 9}, {NULL}};
const Segment SGI371846[] = {{&S19915[30], 13}, {NULL}};
const Segment SGI371847[] = {{&S19923[112], 6}, {NULL}};
const Segment SGI371848[] = {{&S19923[8], 3}, {NULL}};
const Segment SGI371849[] = {{&S19923[13], 9}, {NULL}};
const Segment SGI371850[] = {{&S19923[30], 13}, {NULL}};
const Segment SGI371851[] = {{&S19932[126], 6}, {NULL}};
const Segment SGI371852[] = {{&S19932[8], 3}, {NULL}};
const Segment SGI371853[] = {{&S19932[13], 9}, {NULL}};
const Segment SGI371854[] = {{&S19932[30], 13}, {NULL}};
const Segment SGI371855[] = {{&S19933[126], 6}, {NULL}};
const Segment SGI371856[] = {{&S19933[8], 3}, {NULL}};
const Segment SGI371857[] = {{&S19933[13], 9}, {NULL}};
const Segment SGI371858[] = {{&S19933[30], 13}, {NULL}};
const Segment SGI371859[] = {{&S19940[108], 6}, {NULL}};
const Segment SGI371860[] = {{&S19940[8], 3}, {NULL}};
const Segment SGI371861[] = {{&S19940[13], 9}, {NULL}};
const Segment SGI371862[] = {{&S19940[30], 13}, {NULL}};
const Segment SGI371863[] = {{&S19944[95], 5}, {NULL}};
const Segment SGI371864[] = {{&S19944[8], 3}, {NULL}};
const Segment SGI371865[] = {{&S19944[13], 9}, {NULL}};
const Segment SGI371866[] = {{&S19944[30], 13}, {NULL}};
const Segment SGI371867[] = {{&S19945[50], 5}, {NULL}};
const Segment SGI371868[] = {{&S19945[8], 3}, {NULL}};
const Segment SGI371869[] = {{&S19945[13], 3}, {NULL}};
const Segment SGI371870[] = {{&S19945[76], 6}, {NULL}};
const Segment SGI371871[] = {{&S19946[44], 6}, {NULL}};
const Segment SGI371872[] = {{&S19946[8], 3}, {NULL}};
const Segment SGI371873[] = {{&S19946[13], 3}, {NULL}};
const Segment SGI371874[] = {{&S19946[71], 6}, {NULL}};
const Segment SGI371875[] = {{&S19948[54], 5}, {NULL}};
const Segment SGI371876[] = {{&S19948[8], 3}, {NULL}};
const Segment SGI371877[] = {{&S19948[13], 3}, {NULL}};
const Segment SGI371878[] = {{&S19948[80], 6}, {NULL}};
const Segment SGI371879[] = {{&S19950[54], 6}, {NULL}};
const Segment SGI371880[] = {{&S19950[8], 3}, {NULL}};
const Segment SGI371881[] = {{&S19950[13], 3}, {NULL}};
const Segment SGI371882[] = {{&S19950[81], 6}, {NULL}};
const Segment SGI371883[] = {{&S19951[44], 6}, {NULL}};
const Segment SGI371884[] = {{&S19951[8], 3}, {NULL}};
const Segment SGI371885[] = {{&S19951[13], 3}, {NULL}};
const Segment SGI371886[] = {{&S19951[71], 6}, {NULL}};
const Segment SGI371887[] = {{&S22424[13], 42}, {NULL}};
const Segment SGI371888[] = {{&S22424[69], 6}, {NULL}};
const Segment SGI371889[] = {{&S22424[104], 5}, {NULL}};
const Segment SGI371890[] = {{&S22424[117], 8}, {NULL}};
const Segment SGI371891[] = {{&S21026[107], 6}, {NULL}};
const Segment SGI371892[] = {{&S21026[8], 3}, {NULL}};
const Segment SGI371893[] = {{&S21026[13], 7}, {NULL}};
const Segment SGI371894[] = {{&S21155[41], 5}, {NULL}};
const Segment SGI371895[] = {{&S21155[8], 3}, {NULL}};
const Segment SGI371896[] = {{&S21155[13], 14}, {NULL}};
const Segment SGI371897[] = {{&S21155[67], 7}, {NULL}};
const Segment SGI371898[] = {{&S21511[105], 6}, {NULL}};
const Segment SGI371899[] = {{&S21511[8], 3}, {NULL}};
const Segment SGI371900[] = {{&S21511[13], 9}, {NULL}};
const Segment SGI371901[] = {{&S21511[33], 15}, {NULL}};
const Segment SGI371902[] = {{&S21511[49], 6}, {NULL}};
const Segment SGI371903[] = {{&S21511[33], 13}, {NULL}};
const Segment SGI371904[] = {{&S21513[8], 3}, {NULL}};
const Segment SGI371905[] = {{&S21513[13], 9}, {NULL}};
const Segment SGI371906[] = {{&S21513[33], 15}, {NULL}};
const Segment SGI371907[] = {{&S21513[49], 6}, {NULL}};
const Segment SGI371908[] = {{&S21513[33], 13}, {NULL}};
const Segment SGI371909[] = {{&S21516[106], 7}, {NULL}};
const Segment SGI371910[] = {{&S21516[8], 3}, {NULL}};
const Segment SGI371911[] = {{&S21516[13], 9}, {NULL}};
const Segment SGI371912[] = {{&S21516[33], 15}, {NULL}};
const Segment SGI371913[] = {{&S21516[49], 6}, {NULL}};
const Segment SGI371914[] = {{&S21516[33], 13}, {NULL}};
const Segment SGI371915[] = {{&S21529[95], 5}, {NULL}};
const Segment SGI371916[] = {{&S21529[8], 3}, {NULL}};
const Segment SGI371917[] = {{&S21529[13], 9}, {NULL}};
const Segment SGI371918[] = {{&S21529[30], 13}, {NULL}};
const Segment SGI371919[] = {{&S21530[105], 5}, {NULL}};
const Segment SGI371920[] = {{&S21530[8], 3}, {NULL}};
const Segment SGI371921[] = {{&S21530[13], 9}, {NULL}};
const Segment SGI371922[] = {{&S21530[33], 15}, {NULL}};
const Segment SGI371923[] = {{&S21530[49], 6}, {NULL}};
const Segment SGI371924[] = {{&S21530[33], 13}, {NULL}};
const Segment SGI371925[] = {{&S21536[105], 5}, {NULL}};
const Segment SGI371926[] = {{&S21536[8], 3}, {NULL}};
const Segment SGI371927[] = {{&S21536[13], 9}, {NULL}};
const Segment SGI371928[] = {{&S21536[33], 15}, {NULL}};
const Segment SGI371929[] = {{&S21536[49], 6}, {NULL}};
const Segment SGI371930[] = {{&S21536[33], 13}, {NULL}};
const Segment SGI371931[] = {{&S21537[105], 5}, {NULL}};
const Segment SGI371932[] = {{&S21537[8], 3}, {NULL}};
const Segment SGI371933[] = {{&S21537[13], 9}, {NULL}};
const Segment SGI371934[] = {{&S21537[33], 15}, {NULL}};
const Segment SGI371935[] = {{&S21537[49], 6}, {NULL}};
const Segment SGI371936[] = {{&S21537[33], 13}, {NULL}};
const Segment SGI371937[] = {{&S21538[105], 5}, {NULL}};
const Segment SGI371938[] = {{&S21538[8], 3}, {NULL}};
const Segment SGI371939[] = {{&S21538[13], 9}, {NULL}};
const Segment SGI371940[] = {{&S21538[33], 15}, {NULL}};
const Segment SGI371941[] = {{&S21538[49], 6}, {NULL}};
const Segment SGI371942[] = {{&S21538[33], 13}, {NULL}};
const Segment SGI371943[] = {{&S21544[95], 5}, {NULL}};
const Segment SGI371944[] = {{&S21544[8], 3}, {NULL}};
const Segment SGI371945[] = {{&S21544[13], 9}, {NULL}};
const Segment SGI371946[] = {{&S21544[30], 13}, {NULL}};
const Segment SGI371947[] = {{&S21548[125], 6}, {NULL}};
const Segment SGI371948[] = {{&S21548[8], 3}, {NULL}};
const Segment SGI371949[] = {{&S21548[13], 9}, {NULL}};
const Segment SGI371950[] = {{&S21548[33], 15}, {NULL}};
const Segment SGI371951[] = {{&S21548[49], 6}, {NULL}};
const Segment SGI371952[] = {{&S21548[33], 13}, {NULL}};
const Segment SGI371953[] = {{&S21551[95], 5}, {NULL}};
const Segment SGI371954[] = {{&S21551[8], 3}, {NULL}};
const Segment SGI371955[] = {{&S21551[13], 9}, {NULL}};
const Segment SGI371956[] = {{&S21551[30], 13}, {NULL}};
const Segment SGI371957[] = {{&S21559[69], 6}, {NULL}};
const Segment SGI371958[] = {{&S21559[8], 3}, {NULL}};
const Segment SGI371959[] = {{&S21559[13], 9}, {NULL}};
const Segment SGI371960[] = {{&S21559[33], 15}, {NULL}};
const Segment SGI371961[] = {{&S21559[49], 6}, {NULL}};
const Segment SGI371962[] = {{&S21559[33], 13}, {NULL}};
const Segment SGI371963[] = {{&S21561[125], 6}, {NULL}};
const Segment SGI371964[] = {{&S21561[8], 3}, {NULL}};
const Segment SGI371965[] = {{&S21561[13], 9}, {NULL}};
const Segment SGI371966[] = {{&S21561[33], 15}, {NULL}};
const Segment SGI371967[] = {{&S21561[49], 6}, {NULL}};
const Segment SGI371968[] = {{&S21561[33], 13}, {NULL}};
const Segment SGI371969[] = {{&S21566[8], 3}, {NULL}};
const Segment SGI371970[] = {{&S21566[13], 9}, {NULL}};
const Segment SGI371971[] = {{&S21566[30], 13}, {NULL}};
const Segment SGI371972[] = {{&S21570[125], 6}, {NULL}};
const Segment SGI371973[] = {{&S21570[8], 3}, {NULL}};
const Segment SGI371974[] = {{&S21570[13], 9}, {NULL}};
const Segment SGI371975[] = {{&S21570[33], 15}, {NULL}};
const Segment SGI371976[] = {{&S21570[49], 6}, {NULL}};
const Segment SGI371977[] = {{&S21570[33], 13}, {NULL}};
const Segment SGI371978[] = {{&S21571[110], 6}, {NULL}};
const Segment SGI371979[] = {{&S21571[8], 3}, {NULL}};
const Segment SGI371980[] = {{&S21571[13], 9}, {NULL}};
const Segment SGI371981[] = {{&S21571[30], 13}, {NULL}};
const Segment SGI371982[] = {{&S21572[95], 5}, {NULL}};
const Segment SGI371983[] = {{&S21572[8], 3}, {NULL}};
const Segment SGI371984[] = {{&S21572[13], 9}, {NULL}};
const Segment SGI371985[] = {{&S21572[30], 13}, {NULL}};
const Segment SGI371986[] = {{&S21574[71], 5}, {NULL}};
const Segment SGI371987[] = {{&S21574[8], 3}, {&S21574[8], 3}, {&S21574[46], 3}, {&S21574[59], 3}, {NULL}};
const Segment SGI371988[] = {{&S21574[13], 10}, {NULL}};
const Segment SGI371989[] = {{&S21578[69], 6}, {NULL}};
const Segment SGI371990[] = {{&S21578[8], 3}, {NULL}};
const Segment SGI371991[] = {{&S21578[13], 9}, {NULL}};
const Segment SGI371992[] = {{&S21578[33], 15}, {NULL}};
const Segment SGI371993[] = {{&S21578[49], 6}, {NULL}};
const Segment SGI371994[] = {{&S21578[33], 13}, {NULL}};
const Segment SGI371995[] = {{&S21587[59], 6}, {NULL}};
const Segment SGI371996[] = {{&S21587[8], 3}, {NULL}};
const Segment SGI371997[] = {{&S21587[13], 9}, {NULL}};
const Segment SGI371998[] = {{&S21587[30], 13}, {NULL}};
const Segment SGI371999[] = {{&S21589[69], 6}, {NULL}};
const Segment SGI372000[] = {{&S21589[8], 3}, {NULL}};
const Segment SGI372001[] = {{&S21589[13], 9}, {NULL}};
const Segment SGI372002[] = {{&S21589[33], 15}, {NULL}};
const Segment SGI372003[] = {{&S21589[49], 6}, {NULL}};
const Segment SGI372004[] = {{&S21589[33], 13}, {NULL}};
const Segment SGI372005[] = {{&S21591[59], 6}, {NULL}};
const Segment SGI372006[] = {{&S21591[8], 3}, {NULL}};
const Segment SGI372007[] = {{&S21591[13], 9}, {NULL}};
const Segment SGI372008[] = {{&S21591[30], 13}, {NULL}};
const Segment SGI372009[] = {{&S21728[41], 6}, {NULL}};
const Segment SGI372010[] = {{&S21728[8], 3}, {NULL}};
const Segment SGI372011[] = {{&S21728[13], 14}, {NULL}};
const Segment SGI372012[] = {{&S21728[76], 3}, {NULL}};
const Segment SGI372013[] = {{&S21729[41], 6}, {NULL}};
const Segment SGI372014[] = {{&S21729[8], 3}, {NULL}};
const Segment SGI372015[] = {{&S21729[13], 14}, {NULL}};
const Segment SGI372016[] = {{&S21729[24], 3}, {NULL}};
const Segment SGI372017[] = {{&S21730[48], 6}, {NULL}};
const Segment SGI372018[] = {{&S21730[8], 3}, {NULL}};
const Segment SGI372019[] = {{&S21730[13], 14}, {NULL}};
const Segment SGI372020[] = {{&S21730[24], 3}, {NULL}};
const Segment SGI372021[] = {{&S21731[48], 6}, {NULL}};
const Segment SGI372022[] = {{&S21731[8], 3}, {NULL}};
const Segment SGI372023[] = {{&S21731[13], 14}, {NULL}};
const Segment SGI372024[] = {{&S21731[83], 3}, {NULL}};
const Segment SGI372025[] = {{&S21732[48], 6}, {NULL}};
const Segment SGI372026[] = {{&S21732[8], 3}, {NULL}};
const Segment SGI372027[] = {{&S21732[13], 14}, {NULL}};
const Segment SGI372028[] = {{&S21732[24], 3}, {NULL}};
const Segment SGI372029[] = {{&S21733[41], 6}, {NULL}};
const Segment SGI372030[] = {{&S21733[8], 3}, {NULL}};
const Segment SGI372031[] = {{&S21733[13], 14}, {NULL}};
const Segment SGI372032[] = {{&S21733[24], 3}, {NULL}};
const Segment SGI372033[] = {{&S21734[48], 6}, {NULL}};
const Segment SGI372034[] = {{&S21734[8], 3}, {NULL}};
const Segment SGI372035[] = {{&S21734[13], 14}, {NULL}};
const Segment SGI372036[] = {{&S21734[86], 3}, {NULL}};
const Segment SGI372037[] = {{&S21735[41], 6}, {NULL}};
const Segment SGI372038[] = {{&S21735[8], 3}, {NULL}};
const Segment SGI372039[] = {{&S21735[13], 14}, {NULL}};
const Segment SGI372040[] = {{&S21735[79], 3}, {NULL}};
const Segment SGI372041[] = {{&S21736[48], 6}, {NULL}};
const Segment SGI372042[] = {{&S21736[8], 3}, {NULL}};
const Segment SGI372043[] = {{&S21736[13], 14}, {NULL}};
const Segment SGI372044[] = {{&S21736[24], 3}, {NULL}};
const Segment SGI372045[] = {{&S21737[48], 6}, {NULL}};
const Segment SGI372046[] = {{&S21737[8], 3}, {NULL}};
const Segment SGI372047[] = {{&S21737[13], 14}, {NULL}};
const Segment SGI372048[] = {{&S21737[85], 3}, {NULL}};
const Segment SGI372049[] = {{&S21738[41], 6}, {NULL}};
const Segment SGI372050[] = {{&S21738[8], 3}, {NULL}};
const Segment SGI372051[] = {{&S21738[13], 14}, {NULL}};
const Segment SGI372052[] = {{&S21738[24], 3}, {NULL}};
const Segment SGI372053[] = {{&S21739[48], 6}, {NULL}};
const Segment SGI372054[] = {{&S21739[8], 3}, {NULL}};
const Segment SGI372055[] = {{&S21739[13], 14}, {NULL}};
const Segment SGI372056[] = {{&S21739[24], 3}, {NULL}};
const Segment SGI372057[] = {{&S21740[41], 6}, {NULL}};
const Segment SGI372058[] = {{&S21740[8], 3}, {NULL}};
const Segment SGI372059[] = {{&S21740[13], 14}, {NULL}};
const Segment SGI372060[] = {{&S21740[24], 3}, {NULL}};
const Segment SGI372061[] = {{&S21741[41], 6}, {NULL}};
const Segment SGI372062[] = {{&S21741[8], 3}, {NULL}};
const Segment SGI372063[] = {{&S21741[13], 14}, {NULL}};
const Segment SGI372064[] = {{&S21741[24], 3}, {NULL}};
const Segment SGI372065[] = {{&S21742[41], 6}, {NULL}};
const Segment SGI372066[] = {{&S21742[8], 3}, {NULL}};
const Segment SGI372067[] = {{&S21742[13], 14}, {NULL}};
const Segment SGI372068[] = {{&S21742[24], 3}, {NULL}};
const Segment SGI372069[] = {{&S21743[48], 6}, {NULL}};
const Segment SGI372070[] = {{&S21743[8], 3}, {NULL}};
const Segment SGI372071[] = {{&S21743[13], 14}, {NULL}};
const Segment SGI372072[] = {{&S21743[24], 3}, {NULL}};
const Segment SGI372073[] = {{&S21744[41], 6}, {NULL}};
const Segment SGI372074[] = {{&S21744[8], 3}, {NULL}};
const Segment SGI372075[] = {{&S21744[13], 14}, {NULL}};
const Segment SGI372076[] = {{&S21744[78], 3}, {NULL}};
const Segment SGI372077[] = {{&S21745[48], 6}, {NULL}};
const Segment SGI372078[] = {{&S21745[8], 3}, {NULL}};
const Segment SGI372079[] = {{&S21745[13], 14}, {NULL}};
const Segment SGI372080[] = {{&S21745[24], 3}, {NULL}};
const Segment SGI372081[] = {{&S21746[48], 6}, {NULL}};
const Segment SGI372082[] = {{&S21746[8], 3}, {NULL}};
const Segment SGI372083[] = {{&S21746[13], 14}, {NULL}};
const Segment SGI372084[] = {{&S21746[86], 3}, {NULL}};
const Segment SGI372085[] = {{&S21747[41], 6}, {NULL}};
const Segment SGI372086[] = {{&S21747[8], 3}, {NULL}};
const Segment SGI372087[] = {{&S21747[13], 14}, {NULL}};
const Segment SGI372088[] = {{&S21747[78], 3}, {NULL}};
const Segment SGI372089[] = {{&S21748[48], 6}, {NULL}};
const Segment SGI372090[] = {{&S21748[8], 3}, {NULL}};
const Segment SGI372091[] = {{&S21748[13], 14}, {NULL}};
const Segment SGI372092[] = {{&S21748[85], 3}, {NULL}};
const Segment SGI372093[] = {{&S21749[48], 6}, {NULL}};
const Segment SGI372094[] = {{&S21749[8], 3}, {NULL}};
const Segment SGI372095[] = {{&S21749[13], 14}, {NULL}};
const Segment SGI372096[] = {{&S21749[85], 3}, {NULL}};
const Segment SGI372097[] = {{&S21750[41], 6}, {NULL}};
const Segment SGI372098[] = {{&S21750[8], 3}, {NULL}};
const Segment SGI372099[] = {{&S21750[13], 14}, {NULL}};
const Segment SGI372100[] = {{&S21750[78], 3}, {NULL}};
const Segment SGI372101[] = {{&S21751[41], 6}, {NULL}};
const Segment SGI372102[] = {{&S21751[8], 3}, {NULL}};
const Segment SGI372103[] = {{&S21751[13], 14}, {NULL}};
const Segment SGI372104[] = {{&S21751[79], 3}, {NULL}};
const Segment SGI372105[] = {{&S21752[41], 6}, {NULL}};
const Segment SGI372106[] = {{&S21752[8], 3}, {NULL}};
const Segment SGI372107[] = {{&S21752[13], 14}, {NULL}};
const Segment SGI372108[] = {{&S21752[24], 3}, {NULL}};
const Segment SGI372109[] = {{&S21753[91], 6}, {NULL}};
const Segment SGI372110[] = {{&S21753[8], 3}, {NULL}};
const Segment SGI372111[] = {{&S21753[13], 14}, {NULL}};
const Segment SGI372112[] = {{&S21753[78], 3}, {NULL}};
const Segment SGI372113[] = {{&S21754[98], 6}, {NULL}};
const Segment SGI372114[] = {{&S21754[8], 3}, {NULL}};
const Segment SGI372115[] = {{&S21754[13], 14}, {NULL}};
const Segment SGI372116[] = {{&S21754[85], 3}, {NULL}};
const Segment SGI372117[] = {{&S22328[117], 6}, {NULL}};
const Segment SGI372118[] = {{&S22328[8], 3}, {NULL}};
const Segment SGI372119[] = {{&S22328[13], 9}, {NULL}};
const Segment SGI372120[] = {{&S22328[33], 15}, {NULL}};
const Segment SGI372121[] = {{&S22328[104], 3}, {NULL}};
const Segment SGI372122[] = {{&S22328[49], 6}, {NULL}};
const Segment SGI372123[] = {{&S22328[33], 13}, {NULL}};
const Segment SGI372124[] = {{&S22329[117], 6}, {NULL}};
const Segment SGI372125[] = {{&S22329[8], 3}, {NULL}};
const Segment SGI372126[] = {{&S22329[13], 9}, {NULL}};
const Segment SGI372127[] = {{&S22329[33], 15}, {NULL}};
const Segment SGI372128[] = {{&S22329[104], 3}, {NULL}};
const Segment SGI372129[] = {{&S22329[49], 6}, {NULL}};
const Segment SGI372130[] = {{&S22329[33], 13}, {NULL}};
const Segment SGI372131[] = {{&S22330[117], 6}, {NULL}};
const Segment SGI372132[] = {{&S22330[8], 3}, {NULL}};
const Segment SGI372133[] = {{&S22330[13], 9}, {NULL}};
const Segment SGI372134[] = {{&S22330[33], 15}, {NULL}};
const Segment SGI372135[] = {{&S22330[104], 3}, {NULL}};
const Segment SGI372136[] = {{&S22330[49], 6}, {NULL}};
const Segment SGI372137[] = {{&S22330[33], 13}, {NULL}};
const Segment SGI372138[] = {{&S22331[117], 6}, {NULL}};
const Segment SGI372139[] = {{&S22331[8], 3}, {NULL}};
const Segment SGI372140[] = {{&S22331[13], 9}, {NULL}};
const Segment SGI372141[] = {{&S22331[33], 15}, {NULL}};
const Segment SGI372142[] = {{&S22331[104], 3}, {NULL}};
const Segment SGI372143[] = {{&S22331[49], 6}, {NULL}};
const Segment SGI372144[] = {{&S22331[33], 13}, {NULL}};
const Segment SGI372145[] = {{&S22332[117], 6}, {NULL}};
const Segment SGI372146[] = {{&S22332[8], 3}, {NULL}};
const Segment SGI372147[] = {{&S22332[13], 9}, {NULL}};
const Segment SGI372148[] = {{&S22332[33], 15}, {NULL}};
const Segment SGI372149[] = {{&S22332[104], 3}, {NULL}};
const Segment SGI372150[] = {{&S22332[49], 6}, {NULL}};
const Segment SGI372151[] = {{&S22332[33], 13}, {NULL}};
const Segment SGI372152[] = {{&S22333[117], 6}, {NULL}};
const Segment SGI372153[] = {{&S22333[8], 3}, {NULL}};
const Segment SGI372154[] = {{&S22333[13], 9}, {NULL}};
const Segment SGI372155[] = {{&S22333[33], 15}, {NULL}};
const Segment SGI372156[] = {{&S22333[104], 3}, {NULL}};
const Segment SGI372157[] = {{&S22333[49], 6}, {NULL}};
const Segment SGI372158[] = {{&S22333[33], 13}, {NULL}};
const Segment SGI372159[] = {{&S22334[117], 6}, {NULL}};
const Segment SGI372160[] = {{&S22334[8], 3}, {NULL}};
const Segment SGI372161[] = {{&S22334[13], 9}, {NULL}};
const Segment SGI372162[] = {{&S22334[33], 15}, {NULL}};
const Segment SGI372163[] = {{&S22334[104], 3}, {NULL}};
const Segment SGI372164[] = {{&S22334[49], 6}, {NULL}};
const Segment SGI372165[] = {{&S22334[33], 13}, {NULL}};
const Segment SGI372166[] = {{&S22335[117], 6}, {NULL}};
const Segment SGI372167[] = {{&S22335[8], 3}, {NULL}};
const Segment SGI372168[] = {{&S22335[13], 9}, {NULL}};
const Segment SGI372169[] = {{&S22335[33], 15}, {NULL}};
const Segment SGI372170[] = {{&S22335[104], 3}, {NULL}};
const Segment SGI372171[] = {{&S22335[49], 6}, {NULL}};
const Segment SGI372172[] = {{&S22335[33], 13}, {NULL}};
const Segment SGI372173[] = {{&S22336[117], 6}, {NULL}};
const Segment SGI372174[] = {{&S22336[8], 3}, {NULL}};
const Segment SGI372175[] = {{&S22336[13], 9}, {NULL}};
const Segment SGI372176[] = {{&S22336[33], 15}, {NULL}};
const Segment SGI372177[] = {{&S22336[104], 3}, {NULL}};
const Segment SGI372178[] = {{&S22336[49], 6}, {NULL}};
const Segment SGI372179[] = {{&S22336[33], 13}, {NULL}};
const Segment SGI372180[] = {{&S22337[117], 6}, {NULL}};
const Segment SGI372181[] = {{&S22337[8], 3}, {NULL}};
const Segment SGI372182[] = {{&S22337[13], 9}, {NULL}};
const Segment SGI372183[] = {{&S22337[33], 15}, {NULL}};
const Segment SGI372184[] = {{&S22337[104], 3}, {NULL}};
const Segment SGI372185[] = {{&S22337[49], 6}, {NULL}};
const Segment SGI372186[] = {{&S22337[33], 13}, {NULL}};
const Segment SGI372187[] = {{&S22338[117], 6}, {NULL}};
const Segment SGI372188[] = {{&S22338[8], 3}, {NULL}};
const Segment SGI372189[] = {{&S22338[13], 9}, {NULL}};
const Segment SGI372190[] = {{&S22338[33], 15}, {NULL}};
const Segment SGI372191[] = {{&S22338[104], 3}, {NULL}};
const Segment SGI372192[] = {{&S22338[49], 6}, {NULL}};
const Segment SGI372193[] = {{&S22338[33], 13}, {NULL}};
const Segment SGI372194[] = {{&S22339[117], 6}, {NULL}};
const Segment SGI372195[] = {{&S22339[8], 3}, {NULL}};
const Segment SGI372196[] = {{&S22339[13], 9}, {NULL}};
const Segment SGI372197[] = {{&S22339[33], 15}, {NULL}};
const Segment SGI372198[] = {{&S22339[104], 3}, {NULL}};
const Segment SGI372199[] = {{&S22339[49], 6}, {NULL}};
const Segment SGI372200[] = {{&S22339[33], 13}, {NULL}};
const Segment SGI372201[] = {{&S22340[117], 6}, {NULL}};
const Segment SGI372202[] = {{&S22340[8], 3}, {NULL}};
const Segment SGI372203[] = {{&S22340[13], 9}, {NULL}};
const Segment SGI372204[] = {{&S22340[33], 15}, {NULL}};
const Segment SGI372205[] = {{&S22340[104], 3}, {NULL}};
const Segment SGI372206[] = {{&S22340[49], 6}, {NULL}};
const Segment SGI372207[] = {{&S22340[33], 13}, {NULL}};
const Segment SGI372208[] = {{&S22341[117], 6}, {NULL}};
const Segment SGI372209[] = {{&S22341[8], 3}, {NULL}};
const Segment SGI372210[] = {{&S22341[13], 9}, {NULL}};
const Segment SGI372211[] = {{&S22341[33], 15}, {NULL}};
const Segment SGI372212[] = {{&S22341[104], 3}, {NULL}};
const Segment SGI372213[] = {{&S22341[49], 6}, {NULL}};
const Segment SGI372214[] = {{&S22341[33], 13}, {NULL}};
const Segment SGI372215[] = {{&S22342[117], 6}, {NULL}};
const Segment SGI372216[] = {{&S22342[8], 3}, {NULL}};
const Segment SGI372217[] = {{&S22342[13], 9}, {NULL}};
const Segment SGI372218[] = {{&S22342[33], 15}, {NULL}};
const Segment SGI372219[] = {{&S22342[104], 3}, {NULL}};
const Segment SGI372220[] = {{&S22342[49], 6}, {NULL}};
const Segment SGI372221[] = {{&S22342[33], 13}, {NULL}};
const Segment SGI372222[] = {{&S22343[117], 6}, {NULL}};
const Segment SGI372223[] = {{&S22343[8], 3}, {NULL}};
const Segment SGI372224[] = {{&S22343[13], 9}, {NULL}};
const Segment SGI372225[] = {{&S22343[33], 15}, {NULL}};
const Segment SGI372226[] = {{&S22343[104], 3}, {NULL}};
const Segment SGI372227[] = {{&S22343[49], 6}, {NULL}};
const Segment SGI372228[] = {{&S22343[33], 13}, {NULL}};
const Segment SGI372229[] = {{&S22344[117], 6}, {NULL}};
const Segment SGI372230[] = {{&S22344[8], 3}, {NULL}};
const Segment SGI372231[] = {{&S22344[13], 9}, {NULL}};
const Segment SGI372232[] = {{&S22344[33], 15}, {NULL}};
const Segment SGI372233[] = {{&S22344[104], 3}, {NULL}};
const Segment SGI372234[] = {{&S22344[49], 6}, {NULL}};
const Segment SGI372235[] = {{&S22344[33], 13}, {NULL}};
const Segment SGI372236[] = {{&S22345[117], 6}, {NULL}};
const Segment SGI372237[] = {{&S22345[8], 3}, {NULL}};
const Segment SGI372238[] = {{&S22345[13], 9}, {NULL}};
const Segment SGI372239[] = {{&S22345[33], 15}, {NULL}};
const Segment SGI372240[] = {{&S22345[104], 3}, {NULL}};
const Segment SGI372241[] = {{&S22345[49], 6}, {NULL}};
const Segment SGI372242[] = {{&S22345[33], 13}, {NULL}};
const Segment SGI372243[] = {{&S22346[117], 6}, {NULL}};
const Segment SGI372244[] = {{&S22346[8], 3}, {NULL}};
const Segment SGI372245[] = {{&S22346[13], 9}, {NULL}};
const Segment SGI372246[] = {{&S22346[33], 15}, {NULL}};
const Segment SGI372247[] = {{&S22346[104], 3}, {NULL}};
const Segment SGI372248[] = {{&S22346[49], 6}, {NULL}};
const Segment SGI372249[] = {{&S22346[33], 13}, {NULL}};
const Segment SGI372250[] = {{&S22347[117], 6}, {NULL}};
const Segment SGI372251[] = {{&S22347[8], 3}, {NULL}};
const Segment SGI372252[] = {{&S22347[13], 9}, {NULL}};
const Segment SGI372253[] = {{&S22347[33], 15}, {NULL}};
const Segment SGI372254[] = {{&S22347[104], 3}, {NULL}};
const Segment SGI372255[] = {{&S22347[49], 6}, {NULL}};
const Segment SGI372256[] = {{&S22347[33], 13}, {NULL}};
const Segment SGI372257[] = {{&S22348[117], 6}, {NULL}};
const Segment SGI372258[] = {{&S22348[8], 3}, {NULL}};
const Segment SGI372259[] = {{&S22348[13], 9}, {NULL}};
const Segment SGI372260[] = {{&S22348[33], 15}, {NULL}};
const Segment SGI372261[] = {{&S22348[104], 3}, {NULL}};
const Segment SGI372262[] = {{&S22348[49], 6}, {NULL}};
const Segment SGI372263[] = {{&S22348[33], 13}, {NULL}};
const Segment SGI372264[] = {{&S22349[117], 6}, {NULL}};
const Segment SGI372265[] = {{&S22349[8], 3}, {NULL}};
const Segment SGI372266[] = {{&S22349[13], 9}, {NULL}};
const Segment SGI372267[] = {{&S22349[33], 15}, {NULL}};
const Segment SGI372268[] = {{&S22349[104], 3}, {NULL}};
const Segment SGI372269[] = {{&S22349[49], 6}, {NULL}};
const Segment SGI372270[] = {{&S22349[33], 13}, {NULL}};
const Segment SGI372271[] = {{&S22350[69], 6}, {NULL}};
const Segment SGI372272[] = {{&S22350[8], 3}, {NULL}};
const Segment SGI372273[] = {{&S22350[13], 9}, {NULL}};
const Segment SGI372274[] = {{&S22350[33], 15}, {NULL}};
const Segment SGI372275[] = {{&S22350[104], 3}, {NULL}};
const Segment SGI372276[] = {{&S22350[49], 6}, {NULL}};
const Segment SGI372277[] = {{&S22350[33], 13}, {NULL}};
const Segment SGI372278[] = {{&S22351[117], 6}, {NULL}};
const Segment SGI372279[] = {{&S22351[8], 3}, {NULL}};
const Segment SGI372280[] = {{&S22351[13], 9}, {NULL}};
const Segment SGI372281[] = {{&S22351[33], 15}, {NULL}};
const Segment SGI372282[] = {{&S22351[104], 3}, {NULL}};
const Segment SGI372283[] = {{&S22351[49], 6}, {NULL}};
const Segment SGI372284[] = {{&S22351[33], 13}, {NULL}};
const Segment SGI372285[] = {{&S22352[119], 6}, {NULL}};
const Segment SGI372286[] = {{&S22352[8], 3}, {NULL}};
const Segment SGI372287[] = {{&S22352[13], 9}, {NULL}};
const Segment SGI372288[] = {{&S22352[33], 15}, {NULL}};
const Segment SGI372289[] = {{&S22352[106], 3}, {NULL}};
const Segment SGI372290[] = {{&S22352[49], 6}, {NULL}};
const Segment SGI372291[] = {{&S22352[33], 13}, {NULL}};
const Segment SGI372292[] = {{&S22353[117], 6}, {NULL}};
const Segment SGI372293[] = {{&S22353[8], 3}, {NULL}};
const Segment SGI372294[] = {{&S22353[13], 9}, {NULL}};
const Segment SGI372295[] = {{&S22353[33], 15}, {NULL}};
const Segment SGI372296[] = {{&S22353[104], 3}, {NULL}};
const Segment SGI372297[] = {{&S22353[49], 6}, {NULL}};
const Segment SGI372298[] = {{&S22353[33], 13}, {NULL}};
const Segment SGI372299[] = {{&S22354[119], 6}, {NULL}};
const Segment SGI372300[] = {{&S22354[8], 3}, {NULL}};
const Segment SGI372301[] = {{&S22354[13], 9}, {NULL}};
const Segment SGI372302[] = {{&S22354[33], 15}, {NULL}};
const Segment SGI372303[] = {{&S22354[106], 3}, {NULL}};
const Segment SGI372304[] = {{&S22354[49], 6}, {NULL}};
const Segment SGI372305[] = {{&S22354[33], 13}, {NULL}};
const Segment SGI372306[] = {{&S22355[117], 6}, {NULL}};
const Segment SGI372307[] = {{&S22355[8], 3}, {NULL}};
const Segment SGI372308[] = {{&S22355[13], 9}, {NULL}};
const Segment SGI372309[] = {{&S22355[33], 15}, {NULL}};
const Segment SGI372310[] = {{&S22355[104], 3}, {NULL}};
const Segment SGI372311[] = {{&S22355[49], 6}, {NULL}};
const Segment SGI372312[] = {{&S22355[33], 13}, {NULL}};
const Segment SGI372313[] = {{&S22356[69], 6}, {NULL}};
const Segment SGI372314[] = {{&S22356[8], 3}, {NULL}};
const Segment SGI372315[] = {{&S22356[13], 9}, {NULL}};
const Segment SGI372316[] = {{&S22356[33], 15}, {NULL}};
const Segment SGI372317[] = {{&S22356[104], 3}, {NULL}};
const Segment SGI372318[] = {{&S22356[49], 6}, {NULL}};
const Segment SGI372319[] = {{&S22356[33], 13}, {NULL}};
const Segment SGI372320[] = {{&S22357[117], 6}, {NULL}};
const Segment SGI372321[] = {{&S22357[8], 3}, {NULL}};
const Segment SGI372322[] = {{&S22357[13], 9}, {NULL}};
const Segment SGI372323[] = {{&S22357[33], 15}, {NULL}};
const Segment SGI372324[] = {{&S22357[104], 3}, {NULL}};
const Segment SGI372325[] = {{&S22357[49], 6}, {NULL}};
const Segment SGI372326[] = {{&S22357[33], 13}, {NULL}};
const Segment SGI372327[] = {{&S22358[69], 6}, {NULL}};
const Segment SGI372328[] = {{&S22358[8], 3}, {NULL}};
const Segment SGI372329[] = {{&S22358[13], 9}, {NULL}};
const Segment SGI372330[] = {{&S22358[33], 15}, {NULL}};
const Segment SGI372331[] = {{&S22358[104], 3}, {NULL}};
const Segment SGI372332[] = {{&S22358[49], 6}, {NULL}};
const Segment SGI372333[] = {{&S22358[33], 13}, {NULL}};
const Segment SGI372334[] = {{&S22359[119], 6}, {NULL}};
const Segment SGI372335[] = {{&S22359[8], 3}, {NULL}};
const Segment SGI372336[] = {{&S22359[13], 9}, {NULL}};
const Segment SGI372337[] = {{&S22359[33], 15}, {NULL}};
const Segment SGI372338[] = {{&S22359[106], 3}, {NULL}};
const Segment SGI372339[] = {{&S22359[49], 6}, {NULL}};
const Segment SGI372340[] = {{&S22359[33], 13}, {NULL}};
const Segment SGI372341[] = {{&S22360[117], 6}, {NULL}};
const Segment SGI372342[] = {{&S22360[8], 3}, {NULL}};
const Segment SGI372343[] = {{&S22360[13], 9}, {NULL}};
const Segment SGI372344[] = {{&S22360[33], 15}, {NULL}};
const Segment SGI372345[] = {{&S22360[104], 3}, {NULL}};
const Segment SGI372346[] = {{&S22360[49], 6}, {NULL}};
const Segment SGI372347[] = {{&S22360[33], 13}, {NULL}};
const Segment SGI372348[] = {{&S22361[117], 6}, {NULL}};
const Segment SGI372349[] = {{&S22361[8], 3}, {NULL}};
const Segment SGI372350[] = {{&S22361[13], 9}, {NULL}};
const Segment SGI372351[] = {{&S22361[33], 15}, {NULL}};
const Segment SGI372352[] = {{&S22361[104], 3}, {NULL}};
const Segment SGI372353[] = {{&S22361[49], 6}, {NULL}};
const Segment SGI372354[] = {{&S22361[33], 13}, {NULL}};
const Segment SGI372355[] = {{&S22362[119], 6}, {NULL}};
const Segment SGI372356[] = {{&S22362[8], 3}, {NULL}};
const Segment SGI372357[] = {{&S22362[13], 9}, {NULL}};
const Segment SGI372358[] = {{&S22362[33], 15}, {NULL}};
const Segment SGI372359[] = {{&S22362[106], 3}, {NULL}};
const Segment SGI372360[] = {{&S22362[49], 6}, {NULL}};
const Segment SGI372361[] = {{&S22362[33], 13}, {NULL}};
const Segment SGI372362[] = {{&S22363[119], 6}, {NULL}};
const Segment SGI372363[] = {{&S22363[8], 3}, {NULL}};
const Segment SGI372364[] = {{&S22363[13], 9}, {NULL}};
const Segment SGI372365[] = {{&S22363[33], 15}, {NULL}};
const Segment SGI372366[] = {{&S22363[106], 3}, {NULL}};
const Segment SGI372367[] = {{&S22363[49], 6}, {NULL}};
const Segment SGI372368[] = {{&S22363[33], 13}, {NULL}};
const Segment SGI372369[] = {{&S22364[117], 6}, {NULL}};
const Segment SGI372370[] = {{&S22364[8], 3}, {NULL}};
const Segment SGI372371[] = {{&S22364[13], 9}, {NULL}};
const Segment SGI372372[] = {{&S22364[33], 15}, {NULL}};
const Segment SGI372373[] = {{&S22364[104], 3}, {NULL}};
const Segment SGI372374[] = {{&S22364[49], 6}, {NULL}};
const Segment SGI372375[] = {{&S22364[33], 13}, {NULL}};
const Segment SGI372376[] = {{&S22365[119], 6}, {NULL}};
const Segment SGI372377[] = {{&S22365[8], 3}, {NULL}};
const Segment SGI372378[] = {{&S22365[13], 9}, {NULL}};
const Segment SGI372379[] = {{&S22365[33], 15}, {NULL}};
const Segment SGI372380[] = {{&S22365[106], 3}, {NULL}};
const Segment SGI372381[] = {{&S22365[49], 6}, {NULL}};
const Segment SGI372382[] = {{&S22365[33], 13}, {NULL}};
const Segment SGI372383[] = {{&S22366[115], 6}, {NULL}};
const Segment SGI372384[] = {{&S22366[8], 3}, {NULL}};
const Segment SGI372385[] = {{&S22366[13], 9}, {NULL}};
const Segment SGI372386[] = {{&S22366[31], 15}, {NULL}};
const Segment SGI372387[] = {{&S22366[102], 3}, {NULL}};
const Segment SGI372388[] = {{&S22366[47], 6}, {NULL}};
const Segment SGI372389[] = {{&S22366[31], 13}, {NULL}};
const Segment SGI372390[] = {{&S22367[119], 6}, {NULL}};
const Segment SGI372391[] = {{&S22367[8], 3}, {NULL}};
const Segment SGI372392[] = {{&S22367[13], 9}, {NULL}};
const Segment SGI372393[] = {{&S22367[33], 15}, {NULL}};
const Segment SGI372394[] = {{&S22367[106], 3}, {NULL}};
const Segment SGI372395[] = {{&S22367[49], 6}, {NULL}};
const Segment SGI372396[] = {{&S22367[33], 13}, {NULL}};
const Segment SGI372397[] = {{&S22368[117], 6}, {NULL}};
const Segment SGI372398[] = {{&S22368[8], 3}, {NULL}};
const Segment SGI372399[] = {{&S22368[13], 9}, {NULL}};
const Segment SGI372400[] = {{&S22368[33], 15}, {NULL}};
const Segment SGI372401[] = {{&S22368[104], 3}, {NULL}};
const Segment SGI372402[] = {{&S22368[49], 6}, {NULL}};
const Segment SGI372403[] = {{&S22368[33], 13}, {NULL}};
const Segment SGI372404[] = {{&S22369[117], 6}, {NULL}};
const Segment SGI372405[] = {{&S22369[8], 3}, {NULL}};
const Segment SGI372406[] = {{&S22369[13], 9}, {NULL}};
const Segment SGI372407[] = {{&S22369[33], 15}, {NULL}};
const Segment SGI372408[] = {{&S22369[104], 3}, {NULL}};
const Segment SGI372409[] = {{&S22369[49], 6}, {NULL}};
const Segment SGI372410[] = {{&S22369[33], 13}, {NULL}};
const Segment SGI372411[] = {{&S22370[119], 6}, {NULL}};
const Segment SGI372412[] = {{&S22370[8], 3}, {NULL}};
const Segment SGI372413[] = {{&S22370[13], 9}, {NULL}};
const Segment SGI372414[] = {{&S22370[33], 15}, {NULL}};
const Segment SGI372415[] = {{&S22370[106], 3}, {NULL}};
const Segment SGI372416[] = {{&S22370[49], 6}, {NULL}};
const Segment SGI372417[] = {{&S22370[33], 13}, {NULL}};
const Segment SGI372418[] = {{&S22371[69], 6}, {NULL}};
const Segment SGI372419[] = {{&S22371[8], 3}, {NULL}};
const Segment SGI372420[] = {{&S22371[13], 9}, {NULL}};
const Segment SGI372421[] = {{&S22371[33], 15}, {NULL}};
const Segment SGI372422[] = {{&S22371[104], 3}, {NULL}};
const Segment SGI372423[] = {{&S22371[49], 6}, {NULL}};
const Segment SGI372424[] = {{&S22371[33], 13}, {NULL}};
const Segment SGI372425[] = {{&S22372[69], 6}, {NULL}};
const Segment SGI372426[] = {{&S22372[8], 3}, {NULL}};
const Segment SGI372427[] = {{&S22372[13], 9}, {NULL}};
const Segment SGI372428[] = {{&S22372[33], 15}, {NULL}};
const Segment SGI372429[] = {{&S22372[104], 3}, {NULL}};
const Segment SGI372430[] = {{&S22372[49], 6}, {NULL}};
const Segment SGI372431[] = {{&S22372[33], 13}, {NULL}};
const Segment SGI372432[] = {{&S22373[111], 6}, {NULL}};
const Segment SGI372433[] = {{&S22373[8], 3}, {NULL}};
const Segment SGI372434[] = {{&S22373[13], 9}, {NULL}};
const Segment SGI372435[] = {{&S22373[33], 15}, {NULL}};
const Segment SGI372436[] = {{&S22373[98], 3}, {NULL}};
const Segment SGI372437[] = {{&S22373[33], 13}, {NULL}};
const Segment SGI372438[] = {{&S22374[119], 6}, {NULL}};
const Segment SGI372439[] = {{&S22374[8], 3}, {NULL}};
const Segment SGI372440[] = {{&S22374[13], 9}, {NULL}};
const Segment SGI372441[] = {{&S22374[33], 15}, {NULL}};
const Segment SGI372442[] = {{&S22374[106], 3}, {NULL}};
const Segment SGI372443[] = {{&S22374[49], 6}, {NULL}};
const Segment SGI372444[] = {{&S22374[33], 13}, {NULL}};
const Segment SGI372445[] = {{&S22375[111], 6}, {NULL}};
const Segment SGI372446[] = {{&S22375[8], 3}, {NULL}};
const Segment SGI372447[] = {{&S22375[13], 9}, {NULL}};
const Segment SGI372448[] = {{&S22375[33], 15}, {NULL}};
const Segment SGI372449[] = {{&S22375[98], 3}, {NULL}};
const Segment SGI372450[] = {{&S22375[33], 13}, {NULL}};
const Segment SGI372451[] = {{&S22376[117], 6}, {NULL}};
const Segment SGI372452[] = {{&S22376[8], 3}, {NULL}};
const Segment SGI372453[] = {{&S22376[13], 9}, {NULL}};
const Segment SGI372454[] = {{&S22376[33], 15}, {NULL}};
const Segment SGI372455[] = {{&S22376[104], 3}, {NULL}};
const Segment SGI372456[] = {{&S22376[49], 6}, {NULL}};
const Segment SGI372457[] = {{&S22376[33], 13}, {NULL}};
const Segment SGI372458[] = {{&S22377[119], 6}, {NULL}};
const Segment SGI372459[] = {{&S22377[8], 3}, {NULL}};
const Segment SGI372460[] = {{&S22377[13], 9}, {NULL}};
const Segment SGI372461[] = {{&S22377[33], 15}, {NULL}};
const Segment SGI372462[] = {{&S22377[106], 3}, {NULL}};
const Segment SGI372463[] = {{&S22377[49], 6}, {NULL}};
const Segment SGI372464[] = {{&S22377[33], 13}, {NULL}};
const Segment SGI372465[] = {{&S22378[119], 6}, {NULL}};
const Segment SGI372466[] = {{&S22378[8], 3}, {NULL}};
const Segment SGI372467[] = {{&S22378[13], 9}, {NULL}};
const Segment SGI372468[] = {{&S22378[33], 15}, {NULL}};
const Segment SGI372469[] = {{&S22378[106], 3}, {NULL}};
const Segment SGI372470[] = {{&S22378[49], 6}, {NULL}};
const Segment SGI372471[] = {{&S22378[33], 13}, {NULL}};
const Segment SGI372472[] = {{&S22379[117], 6}, {NULL}};
const Segment SGI372473[] = {{&S22379[8], 3}, {NULL}};
const Segment SGI372474[] = {{&S22379[13], 9}, {NULL}};
const Segment SGI372475[] = {{&S22379[33], 15}, {NULL}};
const Segment SGI372476[] = {{&S22379[104], 3}, {NULL}};
const Segment SGI372477[] = {{&S22379[49], 6}, {NULL}};
const Segment SGI372478[] = {{&S22379[33], 13}, {NULL}};
const Segment SGI372479[] = {{&S22380[117], 6}, {NULL}};
const Segment SGI372480[] = {{&S22380[8], 3}, {NULL}};
const Segment SGI372481[] = {{&S22380[13], 9}, {NULL}};
const Segment SGI372482[] = {{&S22380[33], 15}, {NULL}};
const Segment SGI372483[] = {{&S22380[104], 3}, {NULL}};
const Segment SGI372484[] = {{&S22380[49], 6}, {NULL}};
const Segment SGI372485[] = {{&S22380[33], 13}, {NULL}};
const Segment SGI372486[] = {{&S22381[117], 6}, {NULL}};
const Segment SGI372487[] = {{&S22381[8], 3}, {NULL}};
const Segment SGI372488[] = {{&S22381[13], 9}, {NULL}};
const Segment SGI372489[] = {{&S22381[33], 15}, {NULL}};
const Segment SGI372490[] = {{&S22381[104], 3}, {NULL}};
const Segment SGI372491[] = {{&S22381[49], 6}, {NULL}};
const Segment SGI372492[] = {{&S22381[33], 13}, {NULL}};
const Segment SGI372493[] = {{&S22382[117], 6}, {NULL}};
const Segment SGI372494[] = {{&S22382[8], 3}, {NULL}};
const Segment SGI372495[] = {{&S22382[13], 9}, {NULL}};
const Segment SGI372496[] = {{&S22382[33], 15}, {NULL}};
const Segment SGI372497[] = {{&S22382[104], 3}, {NULL}};
const Segment SGI372498[] = {{&S22382[49], 6}, {NULL}};
const Segment SGI372499[] = {{&S22382[33], 13}, {NULL}};
const Segment SGI372500[] = {{&S22383[117], 6}, {NULL}};
const Segment SGI372501[] = {{&S22383[8], 3}, {NULL}};
const Segment SGI372502[] = {{&S22383[13], 9}, {NULL}};
const Segment SGI372503[] = {{&S22383[33], 15}, {NULL}};
const Segment SGI372504[] = {{&S22383[104], 3}, {NULL}};
const Segment SGI372505[] = {{&S22383[49], 6}, {NULL}};
const Segment SGI372506[] = {{&S22383[33], 13}, {NULL}};
const Segment SGI372507[] = {{&S22384[119], 6}, {NULL}};
const Segment SGI372508[] = {{&S22384[8], 3}, {NULL}};
const Segment SGI372509[] = {{&S22384[13], 9}, {NULL}};
const Segment SGI372510[] = {{&S22384[33], 15}, {NULL}};
const Segment SGI372511[] = {{&S22384[106], 3}, {NULL}};
const Segment SGI372512[] = {{&S22384[49], 6}, {NULL}};
const Segment SGI372513[] = {{&S22384[33], 13}, {NULL}};
const Segment SGI372514[] = {{&S22385[109], 6}, {NULL}};
const Segment SGI372515[] = {{&S22385[8], 3}, {NULL}};
const Segment SGI372516[] = {{&S22385[13], 9}, {NULL}};
const Segment SGI372517[] = {{&S22385[96], 3}, {NULL}};
const Segment SGI372518[] = {{&S22385[30], 13}, {NULL}};
const Segment SGI372519[] = {{&S22386[109], 6}, {NULL}};
const Segment SGI372520[] = {{&S22386[8], 3}, {NULL}};
const Segment SGI372521[] = {{&S22386[13], 9}, {NULL}};
const Segment SGI372522[] = {{&S22386[96], 3}, {NULL}};
const Segment SGI372523[] = {{&S22386[30], 13}, {NULL}};
const Segment SGI372524[] = {{&S22387[117], 6}, {NULL}};
const Segment SGI372525[] = {{&S22387[8], 3}, {NULL}};
const Segment SGI372526[] = {{&S22387[13], 9}, {NULL}};
const Segment SGI372527[] = {{&S22387[33], 15}, {NULL}};
const Segment SGI372528[] = {{&S22387[104], 3}, {NULL}};
const Segment SGI372529[] = {{&S22387[49], 6}, {NULL}};
const Segment SGI372530[] = {{&S22387[33], 13}, {NULL}};
const Segment SGI372531[] = {{&S22388[111], 6}, {NULL}};
const Segment SGI372532[] = {{&S22388[8], 3}, {NULL}};
const Segment SGI372533[] = {{&S22388[13], 9}, {NULL}};
const Segment SGI372534[] = {{&S22388[33], 15}, {NULL}};
const Segment SGI372535[] = {{&S22388[98], 3}, {NULL}};
const Segment SGI372536[] = {{&S22388[33], 13}, {NULL}};
const Segment SGI372537[] = {{&S22389[117], 6}, {NULL}};
const Segment SGI372538[] = {{&S22389[8], 3}, {NULL}};
const Segment SGI372539[] = {{&S22389[13], 9}, {NULL}};
const Segment SGI372540[] = {{&S22389[33], 15}, {NULL}};
const Segment SGI372541[] = {{&S22389[104], 3}, {NULL}};
const Segment SGI372542[] = {{&S22389[49], 6}, {NULL}};
const Segment SGI372543[] = {{&S22389[33], 13}, {NULL}};
const Segment SGI372544[] = {{&S22390[69], 6}, {NULL}};
const Segment SGI372545[] = {{&S22390[8], 3}, {NULL}};
const Segment SGI372546[] = {{&S22390[13], 9}, {NULL}};
const Segment SGI372547[] = {{&S22390[33], 15}, {NULL}};
const Segment SGI372548[] = {{&S22390[104], 3}, {NULL}};
const Segment SGI372549[] = {{&S22390[49], 6}, {NULL}};
const Segment SGI372550[] = {{&S22390[33], 13}, {NULL}};
const Segment SGI372551[] = {{&S22391[119], 6}, {NULL}};
const Segment SGI372552[] = {{&S22391[8], 3}, {NULL}};
const Segment SGI372553[] = {{&S22391[13], 9}, {NULL}};
const Segment SGI372554[] = {{&S22391[33], 15}, {NULL}};
const Segment SGI372555[] = {{&S22391[106], 3}, {NULL}};
const Segment SGI372556[] = {{&S22391[49], 6}, {NULL}};
const Segment SGI372557[] = {{&S22391[33], 13}, {NULL}};
const Segment SGI372558[] = {{&S22392[119], 6}, {NULL}};
const Segment SGI372559[] = {{&S22392[8], 3}, {NULL}};
const Segment SGI372560[] = {{&S22392[13], 9}, {NULL}};
const Segment SGI372561[] = {{&S22392[33], 15}, {NULL}};
const Segment SGI372562[] = {{&S22392[106], 3}, {NULL}};
const Segment SGI372563[] = {{&S22392[49], 6}, {NULL}};
const Segment SGI372564[] = {{&S22392[33], 13}, {NULL}};
const Segment SGI372565[] = {{&S22393[117], 6}, {NULL}};
const Segment SGI372566[] = {{&S22393[8], 3}, {NULL}};
const Segment SGI372567[] = {{&S22393[13], 9}, {NULL}};
const Segment SGI372568[] = {{&S22393[33], 15}, {NULL}};
const Segment SGI372569[] = {{&S22393[104], 3}, {NULL}};
const Segment SGI372570[] = {{&S22393[49], 6}, {NULL}};
const Segment SGI372571[] = {{&S22393[33], 13}, {NULL}};
const Segment SGI372572[] = {{&S22394[119], 6}, {NULL}};
const Segment SGI372573[] = {{&S22394[8], 3}, {NULL}};
const Segment SGI372574[] = {{&S22394[13], 9}, {NULL}};
const Segment SGI372575[] = {{&S22394[33], 15}, {NULL}};
const Segment SGI372576[] = {{&S22394[106], 3}, {NULL}};
const Segment SGI372577[] = {{&S22394[49], 6}, {NULL}};
const Segment SGI372578[] = {{&S22394[33], 13}, {NULL}};
const Segment SGI372579[] = {{&S22395[107], 6}, {NULL}};
const Segment SGI372580[] = {{&S22395[8], 3}, {NULL}};
const Segment SGI372581[] = {{&S22395[13], 9}, {NULL}};
const Segment SGI372582[] = {{&S22395[96], 3}, {NULL}};
const Segment SGI372583[] = {{&S22395[30], 13}, {NULL}};
const Segment SGI372584[] = {{&S22396[117], 6}, {NULL}};
const Segment SGI372585[] = {{&S22396[8], 3}, {NULL}};
const Segment SGI372586[] = {{&S22396[13], 9}, {NULL}};
const Segment SGI372587[] = {{&S22396[33], 15}, {NULL}};
const Segment SGI372588[] = {{&S22396[106], 3}, {NULL}};
const Segment SGI372589[] = {{&S22396[49], 6}, {NULL}};
const Segment SGI372590[] = {{&S22396[33], 13}, {NULL}};
const Segment SGI372591[] = {{&S22397[119], 6}, {NULL}};
const Segment SGI372592[] = {{&S22397[8], 3}, {NULL}};
const Segment SGI372593[] = {{&S22397[13], 9}, {NULL}};
const Segment SGI372594[] = {{&S22397[33], 15}, {NULL}};
const Segment SGI372595[] = {{&S22397[106], 3}, {NULL}};
const Segment SGI372596[] = {{&S22397[49], 6}, {NULL}};
const Segment SGI372597[] = {{&S22397[33], 13}, {NULL}};
const Segment SGI372598[] = {{&S22398[117], 6}, {NULL}};
const Segment SGI372599[] = {{&S22398[8], 3}, {NULL}};
const Segment SGI372600[] = {{&S22398[13], 9}, {NULL}};
const Segment SGI372601[] = {{&S22398[33], 15}, {NULL}};
const Segment SGI372602[] = {{&S22398[104], 3}, {NULL}};
const Segment SGI372603[] = {{&S24440[13], 43}, {NULL}};
const Segment SGI372604[] = {{&S22398[49], 6}, {NULL}};
const Segment SGI372605[] = {{&S22398[33], 13}, {NULL}};
const Segment SGI372606[] = {{&S24440[70], 5}, {NULL}};
const Segment SGI372607[] = {{&S24440[104], 3}, {NULL}};
const Segment SGI372608[] = {{&S22399[69], 6}, {NULL}};
const Segment SGI372609[] = {{&S22399[8], 3}, {NULL}};
const Segment SGI372610[] = {{&S22399[13], 9}, {NULL}};
const Segment SGI372611[] = {{&S22399[33], 15}, {NULL}};
const Segment SGI372612[] = {{&S22399[104], 3}, {NULL}};
const Segment SGI372613[] = {{&S22399[49], 6}, {NULL}};
const Segment SGI372614[] = {{&S22399[33], 13}, {NULL}};
const Segment SGI372615[] = {{&S24442[13], 43}, {NULL}};
const Segment SGI372616[] = {{&S24442[70], 5}, {NULL}};
const Segment SGI372617[] = {{&S24442[104], 3}, {NULL}};
const Segment SGI372618[] = {{&S22400[117], 6}, {NULL}};
const Segment SGI372619[] = {{&S22400[8], 3}, {NULL}};
const Segment SGI372620[] = {{&S22400[13], 9}, {NULL}};
const Segment SGI372621[] = {{&S22400[33], 15}, {NULL}};
const Segment SGI372622[] = {{&S22400[104], 3}, {NULL}};
const Segment SGI372623[] = {{&S22400[49], 6}, {NULL}};
const Segment SGI372624[] = {{&S22400[33], 13}, {NULL}};
const Segment SGI372625[] = {{&S22401[117], 6}, {NULL}};
const Segment SGI372626[] = {{&S22401[8], 3}, {NULL}};
const Segment SGI372627[] = {{&S22401[13], 9}, {NULL}};
const Segment SGI372628[] = {{&S22401[33], 15}, {NULL}};
const Segment SGI372629[] = {{&S22401[104], 3}, {NULL}};
const Segment SGI372630[] = {{&S22401[49], 6}, {NULL}};
const Segment SGI372631[] = {{&S22401[33], 13}, {NULL}};
const Segment SGI372632[] = {{&S22402[117], 6}, {NULL}};
const Segment SGI372633[] = {{&S22402[8], 3}, {NULL}};
const Segment SGI372634[] = {{&S22402[13], 9}, {NULL}};
const Segment SGI372635[] = {{&S22402[33], 15}, {NULL}};
const Segment SGI372636[] = {{&S22402[104], 3}, {NULL}};
const Segment SGI372637[] = {{&S22402[49], 6}, {NULL}};
const Segment SGI372638[] = {{&S22402[33], 13}, {NULL}};
const Segment SGI372639[] = {{&S22403[69], 6}, {NULL}};
const Segment SGI372640[] = {{&S22403[8], 3}, {NULL}};
const Segment SGI372641[] = {{&S22403[13], 9}, {NULL}};
const Segment SGI372642[] = {{&S22403[33], 15}, {NULL}};
const Segment SGI372643[] = {{&S22403[104], 3}, {NULL}};
const Segment SGI372644[] = {{&S22403[49], 6}, {NULL}};
const Segment SGI372645[] = {{&S22403[33], 13}, {NULL}};
const Segment SGI372646[] = {{&S24451[13], 43}, {NULL}};
const Segment SGI372647[] = {{&S24451[70], 5}, {NULL}};
const Segment SGI372648[] = {{&S22404[119], 6}, {NULL}};
const Segment SGI372649[] = {{&S24451[104], 3}, {NULL}};
const Segment SGI372650[] = {{&S22404[8], 3}, {NULL}};
const Segment SGI372651[] = {{&S22404[13], 9}, {NULL}};
const Segment SGI372652[] = {{&S22404[33], 15}, {NULL}};
const Segment SGI372653[] = {{&S22404[106], 3}, {NULL}};
const Segment SGI372654[] = {{&S22404[49], 6}, {NULL}};
const Segment SGI372655[] = {{&S22404[33], 13}, {NULL}};
const Segment SGI372656[] = {{&S24452[13], 43}, {NULL}};
const Segment SGI372657[] = {{&S24452[70], 5}, {NULL}};
const Segment SGI372658[] = {{&S24452[104], 3}, {NULL}};
const Segment SGI372659[] = {{&S22405[119], 6}, {NULL}};
const Segment SGI372660[] = {{&S22405[8], 3}, {NULL}};
const Segment SGI372661[] = {{&S22405[13], 9}, {NULL}};
const Segment SGI372662[] = {{&S22405[33], 15}, {NULL}};
const Segment SGI372663[] = {{&S22405[106], 3}, {NULL}};
const Segment SGI372664[] = {{&S22405[49], 6}, {NULL}};
const Segment SGI372665[] = {{&S22405[33], 13}, {NULL}};
const Segment SGI372666[] = {{&S22406[117], 6}, {NULL}};
const Segment SGI372667[] = {{&S22406[8], 3}, {NULL}};
const Segment SGI372668[] = {{&S24454[13], 43}, {NULL}};
const Segment SGI372669[] = {{&S22406[13], 9}, {NULL}};
const Segment SGI372670[] = {{&S24454[70], 5}, {NULL}};
const Segment SGI372671[] = {{&S22406[33], 15}, {NULL}};
const Segment SGI372672[] = {{&S24454[104], 3}, {NULL}};
const Segment SGI372673[] = {{&S22406[104], 3}, {NULL}};
const Segment SGI372674[] = {{&S22406[49], 6}, {NULL}};
const Segment SGI372675[] = {{&S22406[33], 13}, {NULL}};
const Segment SGI372676[] = {{&S22407[119], 6}, {NULL}};
const Segment SGI372677[] = {{&S22407[8], 3}, {NULL}};
const Segment SGI372678[] = {{&S22407[13], 9}, {NULL}};
const Segment SGI372679[] = {{&S22407[33], 15}, {NULL}};
const Segment SGI372680[] = {{&S22407[106], 3}, {NULL}};
const Segment SGI372681[] = {{&S22407[49], 6}, {NULL}};
const Segment SGI372682[] = {{&S22407[33], 13}, {NULL}};
const Segment SGI372683[] = {{&S22408[69], 6}, {NULL}};
const Segment SGI372684[] = {{&S22408[8], 3}, {NULL}};
const Segment SGI372685[] = {{&S22408[13], 9}, {NULL}};
const Segment SGI372686[] = {{&S22408[33], 15}, {NULL}};
const Segment SGI372687[] = {{&S22408[104], 3}, {NULL}};
const Segment SGI372688[] = {{&S22408[49], 6}, {NULL}};
const Segment SGI372689[] = {{&S22408[33], 13}, {NULL}};
const Segment SGI372690[] = {{&S24458[13], 43}, {NULL}};
const Segment SGI372691[] = {{&S24458[70], 5}, {NULL}};
const Segment SGI372692[] = {{&S24458[104], 3}, {NULL}};
const Segment SGI372693[] = {{&S22409[69], 6}, {NULL}};
const Segment SGI372694[] = {{&S22409[8], 3}, {NULL}};
const Segment SGI372695[] = {{&S22409[13], 9}, {NULL}};
const Segment SGI372696[] = {{&S22409[33], 15}, {NULL}};
const Segment SGI372697[] = {{&S22409[104], 3}, {NULL}};
const Segment SGI372698[] = {{&S22409[49], 6}, {NULL}};
const Segment SGI372699[] = {{&S22409[33], 13}, {NULL}};
const Segment SGI372700[] = {{&S22410[69], 6}, {NULL}};
const Segment SGI372701[] = {{&S22410[8], 3}, {NULL}};
const Segment SGI372702[] = {{&S22410[13], 9}, {NULL}};
const Segment SGI372703[] = {{&S22410[33], 15}, {NULL}};
const Segment SGI372704[] = {{&S22410[104], 3}, {NULL}};
const Segment SGI372705[] = {{&S22410[49], 6}, {NULL}};
const Segment SGI372706[] = {{&S22410[33], 13}, {NULL}};
const Segment SGI372707[] = {{&S22411[117], 6}, {NULL}};
const Segment SGI372708[] = {{&S22411[8], 3}, {NULL}};
const Segment SGI372709[] = {{&S22411[13], 9}, {NULL}};
const Segment SGI372710[] = {{&S22411[33], 15}, {NULL}};
const Segment SGI372711[] = {{&S22411[104], 3}, {NULL}};
const Segment SGI372712[] = {{&S22411[49], 6}, {NULL}};
const Segment SGI372713[] = {{&S22411[33], 13}, {NULL}};
const Segment SGI372714[] = {{&S22412[117], 6}, {NULL}};
const Segment SGI372715[] = {{&S22412[8], 3}, {NULL}};
const Segment SGI372716[] = {{&S22412[13], 9}, {NULL}};
const Segment SGI372717[] = {{&S22412[33], 15}, {NULL}};
const Segment SGI372718[] = {{&S22412[104], 3}, {NULL}};
const Segment SGI372719[] = {{&S22412[49], 6}, {NULL}};
const Segment SGI372720[] = {{&S22412[33], 13}, {NULL}};
const Segment SGI372721[] = {{&S22413[117], 6}, {NULL}};
const Segment SGI372722[] = {{&S22413[8], 3}, {NULL}};
const Segment SGI372723[] = {{&S22413[13], 9}, {NULL}};
const Segment SGI372724[] = {{&S22413[33], 15}, {NULL}};
const Segment SGI372725[] = {{&S22413[104], 3}, {NULL}};
const Segment SGI372726[] = {{&S22413[49], 6}, {NULL}};
const Segment SGI372727[] = {{&S22413[33], 13}, {NULL}};
const Segment SGI372728[] = {{&S22414[119], 6}, {NULL}};
const Segment SGI372729[] = {{&S22414[8], 3}, {NULL}};
const Segment SGI372730[] = {{&S22414[13], 9}, {NULL}};
const Segment SGI372731[] = {{&S22414[33], 15}, {NULL}};
const Segment SGI372732[] = {{&S22414[106], 3}, {NULL}};
const Segment SGI372733[] = {{&S22414[49], 6}, {NULL}};
const Segment SGI372734[] = {{&S22414[33], 13}, {NULL}};
const Segment SGI372735[] = {{&S22415[117], 6}, {NULL}};
const Segment SGI372736[] = {{&S22415[8], 3}, {NULL}};
const Segment SGI372737[] = {{&S22415[13], 9}, {NULL}};
const Segment SGI372738[] = {{&S22415[33], 15}, {NULL}};
const Segment SGI372739[] = {{&S22415[104], 3}, {NULL}};
const Segment SGI372740[] = {{&S22415[49], 6}, {NULL}};
const Segment SGI372741[] = {{&S22415[33], 13}, {NULL}};
const Segment SGI372742[] = {{&S22416[117], 6}, {NULL}};
const Segment SGI372743[] = {{&S22416[8], 3}, {NULL}};
const Segment SGI372744[] = {{&S22416[13], 9}, {NULL}};
const Segment SGI372745[] = {{&S22416[33], 15}, {NULL}};
const Segment SGI372746[] = {{&S22416[104], 3}, {NULL}};
const Segment SGI372747[] = {{&S22416[49], 6}, {NULL}};
const Segment SGI372748[] = {{&S22416[33], 13}, {NULL}};
const Segment SGI372749[] = {{&S22417[119], 6}, {NULL}};
const Segment SGI372750[] = {{&S22417[8], 3}, {NULL}};
const Segment SGI372751[] = {{&S22417[13], 9}, {NULL}};
const Segment SGI372752[] = {{&S22417[33], 15}, {NULL}};
const Segment SGI372753[] = {{&S22417[106], 3}, {NULL}};
const Segment SGI372754[] = {{&S22417[49], 6}, {NULL}};
const Segment SGI372755[] = {{&S22417[33], 13}, {NULL}};
const Segment SGI372756[] = {{&S22418[117], 6}, {NULL}};
const Segment SGI372757[] = {{&S22418[8], 3}, {NULL}};
const Segment SGI372758[] = {{&S22418[13], 9}, {NULL}};
const Segment SGI372759[] = {{&S22418[33], 15}, {NULL}};
const Segment SGI372760[] = {{&S22418[104], 3}, {NULL}};
const Segment SGI372761[] = {{&S22418[49], 6}, {NULL}};
const Segment SGI372762[] = {{&S22418[33], 13}, {NULL}};
const Segment SGI372763[] = {{&S22419[117], 6}, {NULL}};
const Segment SGI372764[] = {{&S22419[8], 3}, {NULL}};
const Segment SGI372765[] = {{&S22419[13], 9}, {NULL}};
const Segment SGI372766[] = {{&S22419[33], 15}, {NULL}};
const Segment SGI372767[] = {{&S22419[104], 3}, {NULL}};
const Segment SGI372768[] = {{&S22419[49], 6}, {NULL}};
const Segment SGI372769[] = {{&S22419[33], 13}, {NULL}};
const Segment SGI372770[] = {{&S22420[69], 6}, {NULL}};
const Segment SGI372771[] = {{&S22420[8], 3}, {NULL}};
const Segment SGI372772[] = {{&S22420[13], 9}, {NULL}};
const Segment SGI372773[] = {{&S22420[33], 15}, {NULL}};
const Segment SGI372774[] = {{&S22420[104], 3}, {NULL}};
const Segment SGI372775[] = {{&S22420[49], 6}, {NULL}};
const Segment SGI372776[] = {{&S22420[33], 13}, {NULL}};
const Segment SGI372777[] = {{&S22421[69], 6}, {NULL}};
const Segment SGI372778[] = {{&S22421[8], 3}, {NULL}};
const Segment SGI372779[] = {{&S22421[13], 9}, {NULL}};
const Segment SGI372780[] = {{&S22421[33], 15}, {NULL}};
const Segment SGI372781[] = {{&S22421[104], 3}, {NULL}};
const Segment SGI372782[] = {{&S22421[49], 6}, {NULL}};
const Segment SGI372783[] = {{&S22421[33], 13}, {NULL}};
const Segment SGI372784[] = {{&S22422[69], 5}, {NULL}};
const Segment SGI372785[] = {{&S22422[8], 3}, {NULL}};
const Segment SGI372786[] = {{&S22422[13], 9}, {NULL}};
const Segment SGI372787[] = {{&S22422[33], 15}, {NULL}};
const Segment SGI372788[] = {{&S22422[104], 3}, {NULL}};
const Segment SGI372789[] = {{&S22422[49], 6}, {NULL}};
const Segment SGI372790[] = {{&S22422[33], 13}, {NULL}};
const Segment SGI372791[] = {{&S22423[66], 5}, {NULL}};
const Segment SGI372792[] = {{&S22423[8], 3}, {NULL}};
const Segment SGI372793[] = {{&S22423[13], 9}, {NULL}};
const Segment SGI372794[] = {{&S22423[33], 15}, {NULL}};
const Segment SGI372795[] = {{&S22423[100], 3}, {NULL}};
const Segment SGI372796[] = {{&S22423[49], 3}, {NULL}};
const Segment SGI372797[] = {{&S22423[33], 13}, {NULL}};
const Segment SGI372798[] = {{&S22425[119], 6}, {NULL}};
const Segment SGI372799[] = {{&S22425[8], 3}, {NULL}};
const Segment SGI372800[] = {{&S22425[13], 9}, {NULL}};
const Segment SGI372801[] = {{&S22425[33], 15}, {NULL}};
const Segment SGI372802[] = {{&S22425[106], 3}, {NULL}};
const Segment SGI372803[] = {{&S22425[49], 6}, {NULL}};
const Segment SGI372804[] = {{&S22425[33], 13}, {NULL}};
const Segment SGI372805[] = {{&S22426[117], 6}, {NULL}};
const Segment SGI372806[] = {{&S22426[8], 3}, {NULL}};
const Segment SGI372807[] = {{&S22426[13], 9}, {NULL}};
const Segment SGI372808[] = {{&S22426[33], 15}, {NULL}};
const Segment SGI372809[] = {{&S22426[104], 3}, {NULL}};
const Segment SGI372810[] = {{&S22426[49], 6}, {NULL}};
const Segment SGI372811[] = {{&S22426[33], 13}, {NULL}};
const Segment SGI372812[] = {{&S22427[119], 6}, {NULL}};
const Segment SGI372813[] = {{&S22427[8], 3}, {NULL}};
const Segment SGI372814[] = {{&S22427[13], 9}, {NULL}};
const Segment SGI372815[] = {{&S22427[33], 15}, {NULL}};
const Segment SGI372816[] = {{&S22427[106], 3}, {NULL}};
const Segment SGI372817[] = {{&S22427[49], 6}, {NULL}};
const Segment SGI372818[] = {{&S22427[33], 13}, {NULL}};
const Segment SGI372819[] = {{&S22428[119], 6}, {NULL}};
const Segment SGI372820[] = {{&S22428[8], 3}, {NULL}};
const Segment SGI372821[] = {{&S22428[13], 9}, {NULL}};
const Segment SGI372822[] = {{&S22428[33], 15}, {NULL}};
const Segment SGI372823[] = {{&S22428[106], 3}, {NULL}};
const Segment SGI372824[] = {{&S22428[49], 6}, {NULL}};
const Segment SGI372825[] = {{&S22428[33], 13}, {NULL}};
const Segment SGI372826[] = {{&S22429[117], 6}, {NULL}};
const Segment SGI372827[] = {{&S22429[8], 3}, {NULL}};
const Segment SGI372828[] = {{&S22429[13], 9}, {NULL}};
const Segment SGI372829[] = {{&S22429[33], 15}, {NULL}};
const Segment SGI372830[] = {{&S22429[104], 3}, {NULL}};
const Segment SGI372831[] = {{&S22429[49], 6}, {NULL}};
const Segment SGI372832[] = {{&S22429[33], 13}, {NULL}};
const Segment SGI372833[] = {{&S22430[119], 6}, {NULL}};
const Segment SGI372834[] = {{&S22430[8], 3}, {NULL}};
const Segment SGI372835[] = {{&S22430[13], 9}, {NULL}};
const Segment SGI372836[] = {{&S22430[33], 15}, {NULL}};
const Segment SGI372837[] = {{&S22430[106], 3}, {NULL}};
const Segment SGI372838[] = {{&S22430[49], 6}, {NULL}};
const Segment SGI372839[] = {{&S22430[33], 13}, {NULL}};
const Segment SGI372840[] = {{&S22431[117], 6}, {NULL}};
const Segment SGI372841[] = {{&S22431[8], 3}, {NULL}};
const Segment SGI372842[] = {{&S22431[13], 9}, {NULL}};
const Segment SGI372843[] = {{&S22431[33], 15}, {NULL}};
const Segment SGI372844[] = {{&S22431[104], 3}, {NULL}};
const Segment SGI372845[] = {{&S22431[49], 6}, {NULL}};
const Segment SGI372846[] = {{&S22431[33], 13}, {NULL}};
const Segment SGI372847[] = {{&S22432[69], 6}, {NULL}};
const Segment SGI372848[] = {{&S22432[8], 3}, {NULL}};
const Segment SGI372849[] = {{&S22432[13], 9}, {NULL}};
const Segment SGI372850[] = {{&S22432[33], 15}, {NULL}};
const Segment SGI372851[] = {{&S22432[104], 3}, {NULL}};
const Segment SGI372852[] = {{&S22432[49], 6}, {NULL}};
const Segment SGI372853[] = {{&S22432[33], 13}, {NULL}};
const Segment SGI372854[] = {{&S22433[117], 6}, {NULL}};
const Segment SGI372855[] = {{&S22433[8], 3}, {NULL}};
const Segment SGI372856[] = {{&S22433[13], 9}, {NULL}};
const Segment SGI372857[] = {{&S22433[33], 15}, {NULL}};
const Segment SGI372858[] = {{&S22433[104], 3}, {NULL}};
const Segment SGI372859[] = {{&S22433[49], 6}, {NULL}};
const Segment SGI372860[] = {{&S22433[33], 13}, {NULL}};
const Segment SGI372861[] = {{&S22434[117], 6}, {NULL}};
const Segment SGI372862[] = {{&S22434[8], 3}, {NULL}};
const Segment SGI372863[] = {{&S22434[13], 9}, {NULL}};
const Segment SGI372864[] = {{&S22434[33], 15}, {NULL}};
const Segment SGI372865[] = {{&S22434[104], 3}, {NULL}};
const Segment SGI372866[] = {{&S22434[49], 6}, {NULL}};
const Segment SGI372867[] = {{&S22434[33], 13}, {NULL}};
const Segment SGI372868[] = {{&S22435[114], 6}, {NULL}};
const Segment SGI372869[] = {{&S22435[8], 3}, {NULL}};
const Segment SGI372870[] = {{&S22435[13], 9}, {NULL}};
const Segment SGI372871[] = {{&S22435[33], 15}, {NULL}};
const Segment SGI372872[] = {{&S22435[101], 3}, {NULL}};
const Segment SGI372873[] = {{&S22435[49], 3}, {NULL}};
const Segment SGI372874[] = {{&S22435[33], 13}, {NULL}};
const Segment SGI372875[] = {{&S22436[119], 6}, {NULL}};
const Segment SGI372876[] = {{&S22436[8], 3}, {NULL}};
const Segment SGI372877[] = {{&S22436[13], 9}, {NULL}};
const Segment SGI372878[] = {{&S22436[33], 15}, {NULL}};
const Segment SGI372879[] = {{&S22436[106], 3}, {NULL}};
const Segment SGI372880[] = {{&S22436[49], 6}, {NULL}};
const Segment SGI372881[] = {{&S22436[33], 13}, {NULL}};
const Segment SGI372882[] = {{&S22437[117], 6}, {NULL}};
const Segment SGI372883[] = {{&S22437[8], 3}, {NULL}};
const Segment SGI372884[] = {{&S22437[13], 9}, {NULL}};
const Segment SGI372885[] = {{&S22437[33], 15}, {NULL}};
const Segment SGI372886[] = {{&S22437[104], 3}, {NULL}};
const Segment SGI372887[] = {{&S22437[49], 6}, {NULL}};
const Segment SGI372888[] = {{&S22437[33], 13}, {NULL}};
const Segment SGI372889[] = {{&S22438[117], 6}, {NULL}};
const Segment SGI372890[] = {{&S22438[8], 3}, {NULL}};
const Segment SGI372891[] = {{&S22438[13], 9}, {NULL}};
const Segment SGI372892[] = {{&S22438[33], 15}, {NULL}};
const Segment SGI372893[] = {{&S22438[104], 3}, {NULL}};
const Segment SGI372894[] = {{&S22438[49], 6}, {NULL}};
const Segment SGI372895[] = {{&S22438[33], 13}, {NULL}};
const Segment SGI372896[] = {{&S22439[111], 5}, {NULL}};
const Segment SGI372897[] = {{&S22439[8], 3}, {NULL}};
const Segment SGI372898[] = {{&S22439[13], 9}, {NULL}};
const Segment SGI372899[] = {{&S22439[31], 15}, {NULL}};
const Segment SGI372900[] = {{&S22439[100], 3}, {NULL}};
const Segment SGI372901[] = {{&S22439[47], 6}, {NULL}};
const Segment SGI372902[] = {{&S22439[31], 13}, {NULL}};
const Segment SGI372903[] = {{&S22440[69], 6}, {NULL}};
const Segment SGI372904[] = {{&S22440[8], 3}, {NULL}};
const Segment SGI372905[] = {{&S22440[13], 9}, {NULL}};
const Segment SGI372906[] = {{&S22440[33], 15}, {NULL}};
const Segment SGI372907[] = {{&S22440[104], 3}, {NULL}};
const Segment SGI372908[] = {{&S22440[49], 6}, {NULL}};
const Segment SGI372909[] = {{&S22440[33], 13}, {NULL}};
const Segment SGI372910[] = {{&S22441[64], 6}, {NULL}};
const Segment SGI372911[] = {{&S22441[8], 3}, {NULL}};
const Segment SGI372912[] = {{&S22441[13], 9}, {NULL}};
const Segment SGI372913[] = {{&S22441[31], 15}, {NULL}};
const Segment SGI372914[] = {{&S22441[99], 3}, {NULL}};
const Segment SGI372915[] = {{&S22441[47], 3}, {NULL}};
const Segment SGI372916[] = {{&S22441[31], 13}, {NULL}};
const Segment SGI372917[] = {{&S22442[117], 6}, {NULL}};
const Segment SGI372918[] = {{&S22442[8], 3}, {NULL}};
const Segment SGI372919[] = {{&S22442[13], 9}, {NULL}};
const Segment SGI372920[] = {{&S22442[33], 15}, {NULL}};
const Segment SGI372921[] = {{&S22442[104], 3}, {NULL}};
const Segment SGI372922[] = {{&S22442[49], 6}, {NULL}};
const Segment SGI372923[] = {{&S22442[33], 13}, {NULL}};
const Segment SGI372924[] = {{&S22443[113], 6}, {NULL}};
const Segment SGI372925[] = {{&S22443[8], 3}, {NULL}};
const Segment SGI372926[] = {{&S22443[13], 9}, {NULL}};
const Segment SGI372927[] = {{&S22443[31], 15}, {NULL}};
const Segment SGI372928[] = {{&S22443[102], 3}, {NULL}};
const Segment SGI372929[] = {{&S22443[47], 6}, {NULL}};
const Segment SGI372930[] = {{&S22443[31], 13}, {NULL}};
const Segment SGI372931[] = {{&S22444[112], 6}, {NULL}};
const Segment SGI372932[] = {{&S22444[8], 3}, {NULL}};
const Segment SGI372933[] = {{&S22444[13], 9}, {NULL}};
const Segment SGI372934[] = {{&S22444[31], 15}, {NULL}};
const Segment SGI372935[] = {{&S22444[99], 3}, {NULL}};
const Segment SGI372936[] = {{&S22444[47], 3}, {NULL}};
const Segment SGI372937[] = {{&S22444[31], 13}, {NULL}};
const Segment SGI372938[] = {{&S22445[69], 6}, {NULL}};
const Segment SGI372939[] = {{&S22445[8], 3}, {NULL}};
const Segment SGI372940[] = {{&S22445[13], 9}, {NULL}};
const Segment SGI372941[] = {{&S22445[33], 15}, {NULL}};
const Segment SGI372942[] = {{&S22445[104], 3}, {NULL}};
const Segment SGI372943[] = {{&S22445[49], 6}, {NULL}};
const Segment SGI372944[] = {{&S22445[33], 13}, {NULL}};
const Segment SGI372945[] = {{&S22446[117], 6}, {NULL}};
const Segment SGI372946[] = {{&S22446[8], 3}, {NULL}};
const Segment SGI372947[] = {{&S22446[13], 9}, {NULL}};
const Segment SGI372948[] = {{&S22446[33], 15}, {NULL}};
const Segment SGI372949[] = {{&S22446[104], 3}, {NULL}};
const Segment SGI372950[] = {{&S22446[49], 6}, {NULL}};
const Segment SGI372951[] = {{&S22446[33], 13}, {NULL}};
const Segment SGI372952[] = {{&S22447[119], 6}, {NULL}};
const Segment SGI372953[] = {{&S22447[8], 3}, {NULL}};
const Segment SGI372954[] = {{&S22447[13], 9}, {NULL}};
const Segment SGI372955[] = {{&S22447[33], 15}, {NULL}};
const Segment SGI372956[] = {{&S22447[106], 3}, {NULL}};
const Segment SGI372957[] = {{&S22447[49], 6}, {NULL}};
const Segment SGI372958[] = {{&S22447[33], 13}, {NULL}};
const Segment SGI372959[] = {{&S22448[116], 6}, {NULL}};
const Segment SGI372960[] = {{&S22448[8], 3}, {NULL}};
const Segment SGI372961[] = {{&S22448[13], 9}, {NULL}};
const Segment SGI372962[] = {{&S22448[33], 15}, {NULL}};
const Segment SGI372963[] = {{&S22448[103], 3}, {NULL}};
const Segment SGI372964[] = {{&S22448[49], 3}, {NULL}};
const Segment SGI372965[] = {{&S22448[33], 13}, {NULL}};
const Segment SGI372966[] = {{&S22449[119], 6}, {NULL}};
const Segment SGI372967[] = {{&S22449[8], 3}, {NULL}};
const Segment SGI372968[] = {{&S22449[13], 9}, {NULL}};
const Segment SGI372969[] = {{&S22449[33], 15}, {NULL}};
const Segment SGI372970[] = {{&S22449[106], 3}, {NULL}};
const Segment SGI372971[] = {{&S22449[49], 6}, {NULL}};
const Segment SGI372972[] = {{&S22449[33], 13}, {NULL}};
const Segment SGI372973[] = {{&S22450[119], 6}, {NULL}};
const Segment SGI372974[] = {{&S22450[8], 3}, {NULL}};
const Segment SGI372975[] = {{&S22450[13], 9}, {NULL}};
const Segment SGI372976[] = {{&S22450[33], 15}, {NULL}};
const Segment SGI372977[] = {{&S22450[106], 3}, {NULL}};
const Segment SGI372978[] = {{&S22450[49], 6}, {NULL}};
const Segment SGI372979[] = {{&S22450[33], 13}, {NULL}};
const Segment SGI372980[] = {{&S22451[69], 6}, {NULL}};
const Segment SGI372981[] = {{&S22451[8], 3}, {NULL}};
const Segment SGI372982[] = {{&S22451[13], 9}, {NULL}};
const Segment SGI372983[] = {{&S22451[33], 15}, {NULL}};
const Segment SGI372984[] = {{&S22451[104], 3}, {NULL}};
const Segment SGI372985[] = {{&S22451[49], 6}, {NULL}};
const Segment SGI372986[] = {{&S22451[33], 13}, {NULL}};
const Segment SGI372987[] = {{&S22452[69], 5}, {NULL}};
const Segment SGI372988[] = {{&S22452[8], 3}, {NULL}};
const Segment SGI372989[] = {{&S22452[13], 9}, {NULL}};
const Segment SGI372990[] = {{&S22452[33], 15}, {NULL}};
const Segment SGI372991[] = {{&S22452[103], 3}, {NULL}};
const Segment SGI372992[] = {{&S22452[49], 6}, {NULL}};
const Segment SGI372993[] = {{&S22452[33], 13}, {NULL}};
const Segment SGI372994[] = {{&S22453[119], 6}, {NULL}};
const Segment SGI372995[] = {{&S22453[8], 3}, {NULL}};
const Segment SGI372996[] = {{&S22453[13], 9}, {NULL}};
const Segment SGI372997[] = {{&S22453[33], 15}, {NULL}};
const Segment SGI372998[] = {{&S22453[106], 3}, {NULL}};
const Segment SGI372999[] = {{&S22453[49], 6}, {NULL}};
const Segment SGI373000[] = {{&S22453[33], 13}, {NULL}};
const Segment SGI373001[] = {{&S22454[117], 6}, {NULL}};
const Segment SGI373002[] = {{&S22454[8], 3}, {NULL}};
const Segment SGI373003[] = {{&S22454[13], 9}, {NULL}};
const Segment SGI373004[] = {{&S22454[33], 15}, {NULL}};
const Segment SGI373005[] = {{&S22454[104], 3}, {NULL}};
const Segment SGI373006[] = {{&S22454[49], 6}, {NULL}};
const Segment SGI373007[] = {{&S22454[33], 13}, {NULL}};
const Segment SGI373008[] = {{&S22455[117], 6}, {NULL}};
const Segment SGI373009[] = {{&S22455[8], 3}, {NULL}};
const Segment SGI373010[] = {{&S22455[13], 9}, {NULL}};
const Segment SGI373011[] = {{&S22455[33], 15}, {NULL}};
const Segment SGI373012[] = {{&S22455[104], 3}, {NULL}};
const Segment SGI373013[] = {{&S22455[49], 6}, {NULL}};
const Segment SGI373014[] = {{&S22455[33], 13}, {NULL}};
const Segment SGI373015[] = {{&S22456[119], 6}, {NULL}};
const Segment SGI373016[] = {{&S22456[8], 3}, {NULL}};
const Segment SGI373017[] = {{&S22456[13], 9}, {NULL}};
const Segment SGI373018[] = {{&S22456[33], 15}, {NULL}};
const Segment SGI373019[] = {{&S22456[106], 3}, {NULL}};
const Segment SGI373020[] = {{&S22456[49], 6}, {NULL}};
const Segment SGI373021[] = {{&S22456[33], 13}, {NULL}};
const Segment SGI373022[] = {{&S22457[69], 6}, {NULL}};
const Segment SGI373023[] = {{&S22457[8], 3}, {NULL}};
const Segment SGI373024[] = {{&S22457[13], 9}, {NULL}};
const Segment SGI373025[] = {{&S22457[33], 15}, {NULL}};
const Segment SGI373026[] = {{&S22457[104], 3}, {NULL}};
const Segment SGI373027[] = {{&S22457[49], 6}, {NULL}};
const Segment SGI373028[] = {{&S22457[33], 13}, {NULL}};
const Segment SGI373029[] = {{&S22458[119], 6}, {NULL}};
const Segment SGI373030[] = {{&S22458[8], 3}, {NULL}};
const Segment SGI373031[] = {{&S22458[13], 9}, {NULL}};
const Segment SGI373032[] = {{&S22458[33], 15}, {NULL}};
const Segment SGI373033[] = {{&S22458[106], 3}, {NULL}};
const Segment SGI373034[] = {{&S22458[49], 6}, {NULL}};
const Segment SGI373035[] = {{&S22458[33], 13}, {NULL}};
const Segment SGI373036[] = {{&S22459[69], 6}, {NULL}};
const Segment SGI373037[] = {{&S22459[8], 3}, {NULL}};
const Segment SGI373038[] = {{&S22459[13], 9}, {NULL}};
const Segment SGI373039[] = {{&S22459[33], 15}, {NULL}};
const Segment SGI373040[] = {{&S22459[104], 3}, {NULL}};
const Segment SGI373041[] = {{&S22459[49], 6}, {NULL}};
const Segment SGI373042[] = {{&S22459[33], 13}, {NULL}};
const Segment SGI373043[] = {{&S22460[117], 6}, {NULL}};
const Segment SGI373044[] = {{&S22460[8], 3}, {NULL}};
const Segment SGI373045[] = {{&S22460[13], 9}, {NULL}};
const Segment SGI373046[] = {{&S22460[33], 15}, {NULL}};
const Segment SGI373047[] = {{&S22460[104], 3}, {NULL}};
const Segment SGI373048[] = {{&S22460[49], 6}, {NULL}};
const Segment SGI373049[] = {{&S22460[33], 13}, {NULL}};
const Segment SGI373050[] = {{&S22461[69], 6}, {NULL}};
const Segment SGI373051[] = {{&S22461[8], 3}, {NULL}};
const Segment SGI373052[] = {{&S22461[13], 9}, {NULL}};
const Segment SGI373053[] = {{&S22461[33], 15}, {NULL}};
const Segment SGI373054[] = {{&S22461[104], 3}, {NULL}};
const Segment SGI373055[] = {{&S22461[49], 6}, {NULL}};
const Segment SGI373056[] = {{&S22461[33], 13}, {NULL}};
const Segment SGI373057[] = {{&S22462[69], 5}, {NULL}};
const Segment SGI373058[] = {{&S22462[8], 3}, {NULL}};
const Segment SGI373059[] = {{&S22462[13], 9}, {NULL}};
const Segment SGI373060[] = {{&S22462[33], 15}, {NULL}};
const Segment SGI373061[] = {{&S22462[103], 3}, {NULL}};
const Segment SGI373062[] = {{&S22462[49], 6}, {NULL}};
const Segment SGI373063[] = {{&S22462[33], 13}, {NULL}};
const Segment SGI373064[] = {{&S22463[117], 6}, {NULL}};
const Segment SGI373065[] = {{&S22463[8], 3}, {NULL}};
const Segment SGI373066[] = {{&S22463[13], 9}, {NULL}};
const Segment SGI373067[] = {{&S22463[33], 15}, {NULL}};
const Segment SGI373068[] = {{&S22463[104], 3}, {NULL}};
const Segment SGI373069[] = {{&S22463[49], 6}, {NULL}};
const Segment SGI373070[] = {{&S22463[33], 13}, {NULL}};
const Segment SGI373071[] = {{&S22464[69], 6}, {NULL}};
const Segment SGI373072[] = {{&S22464[8], 3}, {NULL}};
const Segment SGI373073[] = {{&S22464[13], 9}, {NULL}};
const Segment SGI373074[] = {{&S22464[33], 15}, {NULL}};
const Segment SGI373075[] = {{&S22464[104], 3}, {NULL}};
const Segment SGI373076[] = {{&S22464[49], 6}, {NULL}};
const Segment SGI373077[] = {{&S22464[33], 13}, {NULL}};
const Segment SGI373078[] = {{&S22465[119], 6}, {NULL}};
const Segment SGI373079[] = {{&S22465[8], 3}, {NULL}};
const Segment SGI373080[] = {{&S22465[13], 9}, {NULL}};
const Segment SGI373081[] = {{&S22465[33], 15}, {NULL}};
const Segment SGI373082[] = {{&S22465[106], 3}, {NULL}};
const Segment SGI373083[] = {{&S22465[49], 6}, {NULL}};
const Segment SGI373084[] = {{&S22465[33], 13}, {NULL}};
const Segment SGI373085[] = {{&S22466[69], 5}, {NULL}};
const Segment SGI373086[] = {{&S22466[8], 3}, {NULL}};
const Segment SGI373087[] = {{&S22466[13], 9}, {NULL}};
const Segment SGI373088[] = {{&S22466[33], 15}, {NULL}};
const Segment SGI373089[] = {{&S22466[103], 3}, {NULL}};
const Segment SGI373090[] = {{&S22466[49], 6}, {NULL}};
const Segment SGI373091[] = {{&S22466[33], 13}, {NULL}};
const Segment SGI373092[] = {{&S22467[117], 6}, {NULL}};
const Segment SGI373093[] = {{&S22467[8], 3}, {NULL}};
const Segment SGI373094[] = {{&S22467[13], 9}, {NULL}};
const Segment SGI373095[] = {{&S22467[33], 15}, {NULL}};
const Segment SGI373096[] = {{&S22467[104], 3}, {NULL}};
const Segment SGI373097[] = {{&S22467[49], 6}, {NULL}};
const Segment SGI373098[] = {{&S22467[33], 13}, {NULL}};
const Segment SGI373099[] = {{&S22468[117], 6}, {NULL}};
const Segment SGI373100[] = {{&S22468[8], 3}, {NULL}};
const Segment SGI373101[] = {{&S22468[13], 9}, {NULL}};
const Segment SGI373102[] = {{&S22468[33], 15}, {NULL}};
const Segment SGI373103[] = {{&S22468[104], 3}, {NULL}};
const Segment SGI373104[] = {{&S22468[49], 6}, {NULL}};
const Segment SGI373105[] = {{&S22468[33], 13}, {NULL}};
const Segment SGI373106[] = {{&S22469[115], 6}, {NULL}};
const Segment SGI373107[] = {{&S22469[8], 3}, {NULL}};
const Segment SGI373108[] = {{&S22469[13], 9}, {NULL}};
const Segment SGI373109[] = {{&S22469[31], 15}, {NULL}};
const Segment SGI373110[] = {{&S22469[102], 3}, {NULL}};
const Segment SGI373111[] = {{&S22469[47], 6}, {NULL}};
const Segment SGI373112[] = {{&S22469[31], 13}, {NULL}};
const Segment SGI373113[] = {{&S22470[117], 6}, {NULL}};
const Segment SGI373114[] = {{&S22470[8], 3}, {NULL}};
const Segment SGI373115[] = {{&S22470[13], 9}, {NULL}};
const Segment SGI373116[] = {{&S22470[33], 15}, {NULL}};
const Segment SGI373117[] = {{&S22470[104], 3}, {NULL}};
const Segment SGI373118[] = {{&S22470[49], 6}, {NULL}};
const Segment SGI373119[] = {{&S22470[33], 13}, {NULL}};
const Segment SGI373120[] = {{&S22471[69], 6}, {NULL}};
const Segment SGI373121[] = {{&S22471[8], 3}, {NULL}};
const Segment SGI373122[] = {{&S22471[13], 9}, {NULL}};
const Segment SGI373123[] = {{&S22471[33], 15}, {NULL}};
const Segment SGI373124[] = {{&S22471[104], 3}, {NULL}};
const Segment SGI373125[] = {{&S22471[49], 6}, {NULL}};
const Segment SGI373126[] = {{&S22471[33], 13}, {NULL}};
const Segment SGI373127[] = {{&S22472[69], 6}, {NULL}};
const Segment SGI373128[] = {{&S22472[8], 3}, {NULL}};
const Segment SGI373129[] = {{&S22472[13], 9}, {NULL}};
const Segment SGI373130[] = {{&S22472[33], 15}, {NULL}};
const Segment SGI373131[] = {{&S22472[104], 3}, {NULL}};
const Segment SGI373132[] = {{&S22472[49], 6}, {NULL}};
const Segment SGI373133[] = {{&S22472[33], 13}, {NULL}};
const Segment SGI373134[] = {{&S23950[59], 6}, {NULL}};
const Segment SGI373135[] = {{&S23950[8], 3}, {NULL}};
const Segment SGI373136[] = {{&S23950[13], 9}, {NULL}};
const Segment SGI373137[] = {{&S23950[94], 3}, {NULL}};
const Segment SGI373138[] = {{&S23951[62], 6}, {NULL}};
const Segment SGI373139[] = {{&S23951[8], 3}, {NULL}};
const Segment SGI373140[] = {{&S23951[13], 9}, {NULL}};
const Segment SGI373141[] = {{&S23951[99], 3}, {NULL}};
const Segment SGI373142[] = {{&S23952[62], 6}, {NULL}};
const Segment SGI373143[] = {{&S23952[8], 3}, {NULL}};
const Segment SGI373144[] = {{&S23952[13], 9}, {NULL}};
const Segment SGI373145[] = {{&S23952[99], 3}, {NULL}};
const Segment SGI373146[] = {{&S23953[107], 6}, {NULL}};
const Segment SGI373147[] = {{&S23953[8], 3}, {NULL}};
const Segment SGI373148[] = {{&S23953[13], 9}, {NULL}};
const Segment SGI373149[] = {{&S23953[94], 3}, {NULL}};
const Segment SGI373150[] = {{&S23954[107], 6}, {NULL}};
const Segment SGI373151[] = {{&S23954[8], 3}, {NULL}};
const Segment SGI373152[] = {{&S23954[13], 9}, {NULL}};
const Segment SGI373153[] = {{&S23954[96], 3}, {NULL}};
const Segment SGI373154[] = {{&S23955[112], 6}, {NULL}};
const Segment SGI373155[] = {{&S23955[8], 3}, {NULL}};
const Segment SGI373156[] = {{&S23955[13], 9}, {NULL}};
const Segment SGI373157[] = {{&S23955[99], 3}, {NULL}};
const Segment SGI373158[] = {{&S23956[60], 5}, {NULL}};
const Segment SGI373159[] = {{&S23956[8], 3}, {NULL}};
const Segment SGI373160[] = {{&S23956[13], 9}, {NULL}};
const Segment SGI373161[] = {{&S23956[94], 3}, {NULL}};
const Segment SGI373162[] = {{&S23957[60], 5}, {NULL}};
const Segment SGI373163[] = {{&S23957[8], 3}, {NULL}};
const Segment SGI373164[] = {{&S23957[13], 9}, {NULL}};
const Segment SGI373165[] = {{&S23957[94], 3}, {NULL}};
const Segment SGI373166[] = {{&S23958[62], 5}, {NULL}};
const Segment SGI373167[] = {{&S23958[8], 3}, {NULL}};
const Segment SGI373168[] = {{&S23958[13], 9}, {NULL}};
const Segment SGI373169[] = {{&S23958[120], 3}, {NULL}};
const Segment SGI373170[] = {{&S23959[130], 7}, {NULL}};
const Segment SGI373171[] = {{&S23959[8], 3}, {NULL}};
const Segment SGI373172[] = {{&S23959[13], 9}, {NULL}};
const Segment SGI373173[] = {{&S23959[99], 3}, {NULL}};
const Segment SGI373174[] = {{&S26587[13], 11}, {NULL}};
const Segment SGI373175[] = {{&S26587[38], 6}, {NULL}};
const Segment SGI373176[] = {{&S26587[74], 10}, {NULL}};
const Segment SGI373177[] = {{&S26588[13], 11}, {NULL}};
const Segment SGI373178[] = {{&S26588[38], 6}, {NULL}};
const Segment SGI373179[] = {{&S26588[74], 10}, {NULL}};
const Segment SGI373180[] = {{&S26589[13], 11}, {NULL}};
const Segment SGI373181[] = {{&S26589[38], 6}, {NULL}};
const Segment SGI373182[] = {{&S26589[74], 11}, {NULL}};
const Segment SGI373183[] = {{&S26590[13], 11}, {NULL}};
const Segment SGI373184[] = {{&S26590[38], 6}, {NULL}};
const Segment SGI373185[] = {{&S26590[74], 11}, {NULL}};
const Segment SGI373186[] = {{&S26591[13], 11}, {NULL}};
const Segment SGI373187[] = {{&S26591[38], 6}, {NULL}};
const Segment SGI373188[] = {{&S26591[74], 11}, {NULL}};
const Segment SGI373189[] = {{&S26592[13], 11}, {NULL}};
const Segment SGI373190[] = {{&S26592[38], 6}, {NULL}};
const Segment SGI373191[] = {{&S26592[74], 10}, {NULL}};
const Segment SGI373192[] = {{&S26593[13], 11}, {NULL}};
const Segment SGI373193[] = {{&S26593[38], 6}, {NULL}};
const Segment SGI373194[] = {{&S26593[74], 11}, {NULL}};
const Segment SGI373195[] = {{&S26594[13], 11}, {NULL}};
const Segment SGI373196[] = {{&S26594[38], 6}, {NULL}};
const Segment SGI373197[] = {{&S26594[74], 11}, {NULL}};
const Segment SGI373198[] = {{&S26595[13], 11}, {NULL}};
const Segment SGI373199[] = {{&S26595[38], 6}, {NULL}};
const Segment SGI373200[] = {{&S26595[74], 11}, {NULL}};
const Segment SGI373201[] = {{&S26596[13], 11}, {NULL}};
const Segment SGI373202[] = {{&S26596[38], 6}, {NULL}};
const Segment SGI373203[] = {{&S26596[74], 11}, {NULL}};
const Segment SGI373204[] = {{&S26597[13], 11}, {NULL}};
const Segment SGI373205[] = {{&S26597[38], 6}, {NULL}};
const Segment SGI373206[] = {{&S26597[74], 11}, {NULL}};
const Segment SGI373207[] = {{&S26598[13], 11}, {NULL}};
const Segment SGI373208[] = {{&S26598[38], 6}, {NULL}};
const Segment SGI373209[] = {{&S26598[74], 11}, {NULL}};
const Segment SGI373210[] = {{&S26599[13], 11}, {NULL}};
const Segment SGI373211[] = {{&S26599[38], 6}, {NULL}};
const Segment SGI373212[] = {{&S26599[74], 11}, {NULL}};
const Segment SGI373213[] = {{&S26600[13], 11}, {NULL}};
const Segment SGI373214[] = {{&S26600[38], 6}, {NULL}};
const Segment SGI373215[] = {{&S26600[74], 11}, {NULL}};
const Segment SGI373216[] = {{&S26601[13], 11}, {NULL}};
const Segment SGI373217[] = {{&S26601[38], 6}, {NULL}};
const Segment SGI373218[] = {{&S26601[74], 11}, {NULL}};
const Segment SGI373219[] = {{&S26602[13], 11}, {NULL}};
const Segment SGI373220[] = {{&S26602[38], 6}, {NULL}};
const Segment SGI373221[] = {{&S26602[74], 11}, {NULL}};
const Segment SGI373222[] = {{&S26603[13], 11}, {NULL}};
const Segment SGI373223[] = {{&S26603[38], 6}, {NULL}};
const Segment SGI373224[] = {{&S26603[74], 11}, {NULL}};
const Segment SGI373225[] = {{&S26604[13], 11}, {NULL}};
const Segment SGI373226[] = {{&S26604[38], 6}, {NULL}};
const Segment SGI373227[] = {{&S26604[74], 11}, {NULL}};
const Segment SGI373228[] = {{&S26605[13], 11}, {NULL}};
const Segment SGI373229[] = {{&S26605[38], 6}, {NULL}};
const Segment SGI373230[] = {{&S26605[74], 11}, {NULL}};
const Segment SGI373231[] = {{&S26606[13], 11}, {NULL}};
const Segment SGI373232[] = {{&S26606[38], 6}, {NULL}};
const Segment SGI373233[] = {{&S26606[74], 11}, {NULL}};
const Segment SGI373234[] = {{&S26607[13], 11}, {NULL}};
const Segment SGI373235[] = {{&S26607[38], 6}, {NULL}};
const Segment SGI373236[] = {{&S26607[74], 11}, {NULL}};
const Segment SGI373237[] = {{&S26608[13], 11}, {NULL}};
const Segment SGI373238[] = {{&S26608[38], 6}, {NULL}};
const Segment SGI373239[] = {{&S26608[74], 11}, {NULL}};
const Segment SGI373240[] = {{&S26609[13], 11}, {NULL}};
const Segment SGI373241[] = {{&S26609[38], 6}, {NULL}};
const Segment SGI373242[] = {{&S26609[74], 11}, {NULL}};
const Segment SGI373243[] = {{&S26610[13], 11}, {NULL}};
const Segment SGI373244[] = {{&S26610[38], 6}, {NULL}};
const Segment SGI373245[] = {{&S26610[74], 11}, {NULL}};
const Segment SGI373246[] = {{&S26611[13], 11}, {NULL}};
const Segment SGI373247[] = {{&S26611[38], 6}, {NULL}};
const Segment SGI373248[] = {{&S26611[74], 11}, {NULL}};
const Segment SGI373249[] = {{&S26612[13], 11}, {NULL}};
const Segment SGI373250[] = {{&S26612[38], 6}, {NULL}};
const Segment SGI373251[] = {{&S26612[74], 11}, {NULL}};
const Segment SGI373252[] = {{&S26613[13], 11}, {NULL}};
const Segment SGI373253[] = {{&S26613[38], 6}, {NULL}};
const Segment SGI373254[] = {{&S26613[74], 11}, {NULL}};
const Segment SGI373255[] = {{&S26614[13], 11}, {NULL}};
const Segment SGI373256[] = {{&S26614[38], 6}, {NULL}};
const Segment SGI373257[] = {{&S26614[74], 11}, {NULL}};
const Segment SGI373258[] = {{&S26615[13], 11}, {NULL}};
const Segment SGI373259[] = {{&S26615[38], 6}, {NULL}};
const Segment SGI373260[] = {{&S26615[74], 11}, {NULL}};
const Segment SGI373261[] = {{&S26616[13], 11}, {NULL}};
const Segment SGI373262[] = {{&S26616[38], 6}, {NULL}};
const Segment SGI373263[] = {{&S26616[74], 11}, {NULL}};
const Segment SGI373264[] = {{&S26617[13], 11}, {NULL}};
const Segment SGI373265[] = {{&S26617[38], 6}, {NULL}};
const Segment SGI373266[] = {{&S26617[74], 11}, {NULL}};
const Segment SGI373267[] = {{&S26618[13], 11}, {NULL}};
const Segment SGI373268[] = {{&S26618[38], 6}, {NULL}};
const Segment SGI373269[] = {{&S26618[74], 11}, {NULL}};
const Segment SGI373270[] = {{&S26619[13], 11}, {NULL}};
const Segment SGI373271[] = {{&S26619[38], 6}, {NULL}};
const Segment SGI373272[] = {{&S26619[74], 11}, {NULL}};
const Segment SGI373273[] = {{&S26620[13], 11}, {NULL}};
const Segment SGI373274[] = {{&S26620[38], 6}, {NULL}};
const Segment SGI373275[] = {{&S26620[74], 11}, {NULL}};
const Segment SGI373276[] = {{&S26621[13], 11}, {NULL}};
const Segment SGI373277[] = {{&S26621[38], 6}, {NULL}};
const Segment SGI373278[] = {{&S26621[74], 11}, {NULL}};
const Segment SGI373279[] = {{&S26622[13], 11}, {NULL}};
const Segment SGI373280[] = {{&S26622[38], 6}, {NULL}};
const Segment SGI373281[] = {{&S26622[74], 11}, {NULL}};
const Segment SGI373282[] = {{&S26623[13], 11}, {NULL}};
const Segment SGI373283[] = {{&S26623[38], 6}, {NULL}};
const Segment SGI373284[] = {{&S26623[74], 11}, {NULL}};
const Segment SGI373285[] = {{&S26624[13], 11}, {NULL}};
const Segment SGI373286[] = {{&S26624[38], 6}, {NULL}};
const Segment SGI373287[] = {{&S26624[74], 11}, {NULL}};
const Segment SGI373288[] = {{&S26625[13], 11}, {NULL}};
const Segment SGI373289[] = {{&S26625[38], 6}, {NULL}};
const Segment SGI373290[] = {{&S26625[74], 11}, {NULL}};
const Segment SGI373291[] = {{&S26626[13], 11}, {NULL}};
const Segment SGI373292[] = {{&S26626[38], 6}, {NULL}};
const Segment SGI373293[] = {{&S26626[74], 11}, {NULL}};
const Segment SGI373294[] = {{&S26627[13], 11}, {NULL}};
const Segment SGI373295[] = {{&S26627[38], 6}, {NULL}};
const Segment SGI373296[] = {{&S26627[74], 11}, {NULL}};
const Segment SGI373297[] = {{&S26628[13], 11}, {NULL}};
const Segment SGI373298[] = {{&S26628[38], 6}, {NULL}};
const Segment SGI373299[] = {{&S26628[74], 11}, {NULL}};
const Segment SGI373300[] = {{&S26629[13], 11}, {NULL}};
const Segment SGI373301[] = {{&S26629[38], 6}, {NULL}};
const Segment SGI373302[] = {{&S26629[74], 11}, {NULL}};
const Segment SGI373303[] = {{&S26630[13], 11}, {NULL}};
const Segment SGI373304[] = {{&S26630[38], 6}, {NULL}};
const Segment SGI373305[] = {{&S26630[74], 11}, {NULL}};
const Segment SGI373306[] = {{&S26631[13], 11}, {NULL}};
const Segment SGI373307[] = {{&S26631[38], 6}, {NULL}};
const Segment SGI373308[] = {{&S26631[74], 10}, {NULL}};
const Segment SGI373309[] = {{&S26632[13], 11}, {NULL}};
const Segment SGI373310[] = {{&S26632[38], 6}, {NULL}};
const Segment SGI373311[] = {{&S26632[74], 11}, {NULL}};
const Segment SGI373312[] = {{&S26633[13], 11}, {NULL}};
const Segment SGI373313[] = {{&S26633[38], 6}, {NULL}};
const Segment SGI373314[] = {{&S26633[74], 11}, {NULL}};
const Segment SGI373315[] = {{&S26634[13], 11}, {NULL}};
const Segment SGI373316[] = {{&S26634[38], 6}, {NULL}};
const Segment SGI373317[] = {{&S26634[74], 11}, {NULL}};
const Segment SGI373318[] = {{&S26635[13], 11}, {NULL}};
const Segment SGI373319[] = {{&S26635[38], 6}, {NULL}};
const Segment SGI373320[] = {{&S26635[74], 11}, {NULL}};
const Segment SGI373321[] = {{&S26636[13], 11}, {NULL}};
const Segment SGI373322[] = {{&S26636[38], 6}, {NULL}};
const Segment SGI373323[] = {{&S26636[74], 11}, {NULL}};
const Segment SGI373324[] = {{&S26637[13], 11}, {NULL}};
const Segment SGI373325[] = {{&S26637[38], 6}, {NULL}};
const Segment SGI373326[] = {{&S26637[74], 11}, {NULL}};
const Segment SGI373327[] = {{&S26638[13], 11}, {NULL}};
const Segment SGI373328[] = {{&S26638[38], 6}, {NULL}};
const Segment SGI373329[] = {{&S26638[74], 11}, {NULL}};
const Segment SGI373330[] = {{&S26639[13], 11}, {NULL}};
const Segment SGI373331[] = {{&S26639[38], 6}, {NULL}};
const Segment SGI373332[] = {{&S26639[74], 11}, {NULL}};
const Segment SGI373333[] = {{&S26640[13], 11}, {NULL}};
const Segment SGI373334[] = {{&S26640[38], 6}, {NULL}};
const Segment SGI373335[] = {{&S26640[74], 11}, {NULL}};
const Segment SGI373336[] = {{&S26641[13], 11}, {NULL}};
const Segment SGI373337[] = {{&S26641[38], 6}, {NULL}};
const Segment SGI373338[] = {{&S26641[74], 11}, {NULL}};
const Segment SGI373339[] = {{&S26642[13], 11}, {NULL}};
const Segment SGI373340[] = {{&S26642[38], 6}, {NULL}};
const Segment SGI373341[] = {{&S26642[74], 11}, {NULL}};
const Segment SGI373342[] = {{&S26643[13], 11}, {NULL}};
const Segment SGI373343[] = {{&S26643[38], 6}, {NULL}};
const Segment SGI373344[] = {{&S26643[74], 11}, {NULL}};
const Segment SGI373345[] = {{&S26644[13], 11}, {NULL}};
const Segment SGI373346[] = {{&S26644[38], 6}, {NULL}};
const Segment SGI373347[] = {{&S26644[74], 11}, {NULL}};
const Segment SGI373348[] = {{&S26645[13], 11}, {NULL}};
const Segment SGI373349[] = {{&S26645[38], 6}, {NULL}};
const Segment SGI373350[] = {{&S26645[74], 11}, {NULL}};
const Segment SGI373351[] = {{&S26646[13], 11}, {NULL}};
const Segment SGI373352[] = {{&S26646[38], 6}, {NULL}};
const Segment SGI373353[] = {{&S26646[74], 11}, {NULL}};
const Segment SGI373354[] = {{&S26647[13], 11}, {NULL}};
const Segment SGI373355[] = {{&S26647[38], 6}, {NULL}};
const Segment SGI373356[] = {{&S26647[74], 11}, {NULL}};
const Segment SGI373357[] = {{&S26648[13], 11}, {NULL}};
const Segment SGI373358[] = {{&S26648[38], 6}, {NULL}};
const Segment SGI373359[] = {{&S26648[74], 11}, {NULL}};
const Segment SGI373360[] = {{&S26652[13], 11}, {NULL}};
const Segment SGI373361[] = {{&S26652[38], 6}, {NULL}};
const Segment SGI373362[] = {{&S26652[74], 11}, {NULL}};
const Segment SGI373363[] = {{&S26653[13], 11}, {NULL}};
const Segment SGI373364[] = {{&S26653[38], 6}, {NULL}};
const Segment SGI373365[] = {{&S26653[74], 11}, {NULL}};
const Segment SGI373366[] = {{&S26655[13], 11}, {NULL}};
const Segment SGI373367[] = {{&S26655[38], 6}, {NULL}};
const Segment SGI373368[] = {{&S26655[74], 11}, {NULL}};
const Segment SGI373369[] = {{&S26656[13], 11}, {NULL}};
const Segment SGI373370[] = {{&S26656[38], 6}, {NULL}};
const Segment SGI373371[] = {{&S26656[74], 11}, {NULL}};
const Segment SGI373372[] = {{&S26657[13], 11}, {NULL}};
const Segment SGI373373[] = {{&S26657[38], 6}, {NULL}};
const Segment SGI373374[] = {{&S26657[74], 11}, {NULL}};
const Segment SGI373375[] = {{&S27076[13], 11}, {NULL}};
const Segment SGI373376[] = {{&S27076[38], 5}, {NULL}};
const Segment SGI373377[] = {{&S27076[73], 11}, {NULL}};
const Segment SGI373378[] = {{&S27077[13], 11}, {NULL}};
const Segment SGI373379[] = {{&S27077[38], 5}, {NULL}};
const Segment SGI373380[] = {{&S27077[73], 11}, {NULL}};
const Segment SGI373381[] = {{&S27078[13], 11}, {NULL}};
const Segment SGI373382[] = {{&S27078[38], 5}, {NULL}};
const Segment SGI373383[] = {{&S27078[73], 10}, {NULL}};
const Segment SGI373384[] = {{&S27079[13], 11}, {NULL}};
const Segment SGI373385[] = {{&S27079[38], 6}, {NULL}};
const Segment SGI373386[] = {{&S27079[74], 10}, {NULL}};
const Segment SGI373387[] = {{&S27080[13], 11}, {NULL}};
const Segment SGI373388[] = {{&S27080[38], 6}, {NULL}};
const Segment SGI373389[] = {{&S27080[74], 10}, {NULL}};
const Segment SGI373390[] = {{&S27081[13], 11}, {NULL}};
const Segment SGI373391[] = {{&S27081[38], 6}, {NULL}};
const Segment SGI373392[] = {{&S27081[74], 11}, {NULL}};
const Segment SGI373393[] = {{&S27082[13], 11}, {NULL}};
const Segment SGI373394[] = {{&S27082[38], 6}, {NULL}};
const Segment SGI373395[] = {{&S27082[74], 10}, {NULL}};
const Segment SGI373396[] = {{&S27083[13], 11}, {NULL}};
const Segment SGI373397[] = {{&S27083[38], 6}, {NULL}};
const Segment SGI373398[] = {{&S27083[74], 11}, {NULL}};
const Segment SGI373399[] = {{&S27084[13], 11}, {NULL}};
const Segment SGI373400[] = {{&S27084[38], 6}, {NULL}};
const Segment SGI373401[] = {{&S27084[74], 11}, {NULL}};
const Segment SGI373402[] = {{&S27085[13], 11}, {NULL}};
const Segment SGI373403[] = {{&S27085[38], 6}, {NULL}};
const Segment SGI373404[] = {{&S27085[74], 11}, {NULL}};
const Segment SGI373405[] = {{&S27086[13], 11}, {NULL}};
const Segment SGI373406[] = {{&S27086[38], 6}, {NULL}};
const Segment SGI373407[] = {{&S27086[74], 11}, {NULL}};
const Segment SGI373408[] = {{&S27087[13], 11}, {NULL}};
const Segment SGI373409[] = {{&S27087[38], 6}, {NULL}};
const Segment SGI373410[] = {{&S27087[74], 11}, {NULL}};
const Segment SGI373411[] = {{&S27088[13], 11}, {NULL}};
const Segment SGI373412[] = {{&S27088[38], 6}, {NULL}};
const Segment SGI373413[] = {{&S27088[74], 11}, {NULL}};
const Segment SGI373414[] = {{&S27089[13], 11}, {NULL}};
const Segment SGI373415[] = {{&S27089[38], 6}, {NULL}};
const Segment SGI373416[] = {{&S27089[74], 11}, {NULL}};
const Segment SGI373417[] = {{&S27090[13], 11}, {NULL}};
const Segment SGI373418[] = {{&S27090[38], 6}, {NULL}};
const Segment SGI373419[] = {{&S27090[74], 11}, {NULL}};
const Segment SGI373420[] = {{&S27091[13], 11}, {NULL}};
const Segment SGI373421[] = {{&S27091[38], 6}, {NULL}};
const Segment SGI373422[] = {{&S27091[74], 11}, {NULL}};
const Segment SGI373423[] = {{&S27092[13], 11}, {NULL}};
const Segment SGI373424[] = {{&S27092[38], 6}, {NULL}};
const Segment SGI373425[] = {{&S27092[74], 11}, {NULL}};
const Segment SGI373426[] = {{&S27093[13], 11}, {NULL}};
const Segment SGI373427[] = {{&S27093[38], 6}, {NULL}};
const Segment SGI373428[] = {{&S27093[74], 11}, {NULL}};
const Segment SGI373429[] = {{&S27094[13], 11}, {NULL}};
const Segment SGI373430[] = {{&S27094[38], 6}, {NULL}};
const Segment SGI373431[] = {{&S27094[74], 11}, {NULL}};
const Segment SGI373432[] = {{&S27095[13], 11}, {NULL}};
const Segment SGI373433[] = {{&S27095[38], 6}, {NULL}};
const Segment SGI373434[] = {{&S27095[74], 11}, {NULL}};
const Segment SGI373435[] = {{&S27096[13], 11}, {NULL}};
const Segment SGI373436[] = {{&S27096[38], 6}, {NULL}};
const Segment SGI373437[] = {{&S27096[74], 11}, {NULL}};
const Segment SGI373438[] = {{&S27097[13], 11}, {NULL}};
const Segment SGI373439[] = {{&S27097[38], 6}, {NULL}};
const Segment SGI373440[] = {{&S27097[74], 11}, {NULL}};
const Segment SGI373441[] = {{&S27098[13], 11}, {NULL}};
const Segment SGI373442[] = {{&S27098[38], 6}, {NULL}};
const Segment SGI373443[] = {{&S27098[74], 11}, {NULL}};
const Segment SGI373444[] = {{&S27099[13], 11}, {NULL}};
const Segment SGI373445[] = {{&S27099[38], 6}, {NULL}};
const Segment SGI373446[] = {{&S27099[74], 11}, {NULL}};
const Segment SGI373447[] = {{&S27100[13], 11}, {NULL}};
const Segment SGI373448[] = {{&S27100[38], 6}, {NULL}};
const Segment SGI373449[] = {{&S27100[74], 11}, {NULL}};
const Segment SGI373450[] = {{&S27101[13], 11}, {NULL}};
const Segment SGI373451[] = {{&S27101[38], 6}, {NULL}};
const Segment SGI373452[] = {{&S27101[74], 11}, {NULL}};
const Segment SGI373453[] = {{&S27102[13], 11}, {NULL}};
const Segment SGI373454[] = {{&S27102[38], 6}, {NULL}};
const Segment SGI373455[] = {{&S27102[74], 11}, {NULL}};
const Segment SGI373456[] = {{&S27103[13], 11}, {NULL}};
const Segment SGI373457[] = {{&S27103[38], 6}, {NULL}};
const Segment SGI373458[] = {{&S27103[74], 11}, {NULL}};
const Segment SGI373459[] = {{&S27104[13], 11}, {NULL}};
const Segment SGI373460[] = {{&S27104[38], 6}, {NULL}};
const Segment SGI373461[] = {{&S27104[74], 10}, {NULL}};
const Segment SGI373462[] = {{&S27105[13], 11}, {NULL}};
const Segment SGI373463[] = {{&S27105[38], 6}, {NULL}};
const Segment SGI373464[] = {{&S27105[74], 11}, {NULL}};
const Segment SGI373465[] = {{&S27106[13], 11}, {NULL}};
const Segment SGI373466[] = {{&S27106[38], 6}, {NULL}};
const Segment SGI373467[] = {{&S27106[74], 11}, {NULL}};
const Segment SGI373468[] = {{&S27107[13], 11}, {NULL}};
const Segment SGI373469[] = {{&S27107[38], 6}, {NULL}};
const Segment SGI373470[] = {{&S27107[74], 11}, {NULL}};
const Segment SGI373471[] = {{&S27108[13], 11}, {NULL}};
const Segment SGI373472[] = {{&S27108[38], 6}, {NULL}};
const Segment SGI373473[] = {{&S27108[74], 11}, {NULL}};
const Segment SGI373474[] = {{&S27109[13], 11}, {NULL}};
const Segment SGI373475[] = {{&S27109[38], 6}, {NULL}};
const Segment SGI373476[] = {{&S27109[74], 11}, {NULL}};
const Segment SGI373477[] = {{&S27110[13], 11}, {NULL}};
const Segment SGI373478[] = {{&S27110[38], 6}, {NULL}};
const Segment SGI373479[] = {{&S27110[74], 11}, {NULL}};
const Segment SGI373480[] = {{&S27111[13], 11}, {NULL}};
const Segment SGI373481[] = {{&S27111[38], 6}, {NULL}};
const Segment SGI373482[] = {{&S27111[74], 11}, {NULL}};
const Segment SGI373483[] = {{&S27112[13], 11}, {NULL}};
const Segment SGI373484[] = {{&S27112[38], 6}, {NULL}};
const Segment SGI373485[] = {{&S27112[74], 11}, {NULL}};
const Segment SGI373486[] = {{&S27113[13], 11}, {NULL}};
const Segment SGI373487[] = {{&S27113[38], 6}, {NULL}};
const Segment SGI373488[] = {{&S27113[74], 11}, {NULL}};
const Segment SGI373489[] = {{&S27114[13], 11}, {NULL}};
const Segment SGI373490[] = {{&S27114[38], 6}, {NULL}};
const Segment SGI373491[] = {{&S27114[74], 11}, {NULL}};
const Segment SGI373492[] = {{&S27115[13], 11}, {NULL}};
const Segment SGI373493[] = {{&S27115[38], 6}, {NULL}};
const Segment SGI373494[] = {{&S27115[74], 11}, {NULL}};
const Segment SGI373495[] = {{&S27116[13], 11}, {NULL}};
const Segment SGI373496[] = {{&S27116[38], 6}, {NULL}};
const Segment SGI373497[] = {{&S27116[74], 11}, {NULL}};
const Segment SGI373498[] = {{&S27117[13], 11}, {NULL}};
const Segment SGI373499[] = {{&S27117[38], 6}, {NULL}};
const Segment SGI373500[] = {{&S27117[74], 11}, {NULL}};
const Segment SGI373501[] = {{&S27118[13], 11}, {NULL}};
const Segment SGI373502[] = {{&S27118[38], 6}, {NULL}};
const Segment SGI373503[] = {{&S27118[74], 11}, {NULL}};
const Segment SGI373504[] = {{&S27119[13], 11}, {NULL}};
const Segment SGI373505[] = {{&S27119[38], 6}, {NULL}};
const Segment SGI373506[] = {{&S27119[74], 11}, {NULL}};
const Segment SGI373507[] = {{&S27120[13], 11}, {NULL}};
const Segment SGI373508[] = {{&S27120[38], 6}, {NULL}};
const Segment SGI373509[] = {{&S27120[74], 11}, {NULL}};
const Segment SGI373510[] = {{&S27121[13], 11}, {NULL}};
const Segment SGI373511[] = {{&S27121[38], 6}, {NULL}};
const Segment SGI373512[] = {{&S27121[74], 11}, {NULL}};
const Segment SGI373513[] = {{&S27122[13], 11}, {NULL}};
const Segment SGI373514[] = {{&S27122[38], 6}, {NULL}};
const Segment SGI373515[] = {{&S27122[74], 11}, {NULL}};
const Segment SGI373516[] = {{&S27123[13], 11}, {NULL}};
const Segment SGI373517[] = {{&S27123[38], 6}, {NULL}};
const Segment SGI373518[] = {{&S27123[74], 11}, {NULL}};
const Segment SGI373519[] = {{&S27124[13], 11}, {NULL}};
const Segment SGI373520[] = {{&S27124[38], 6}, {NULL}};
const Segment SGI373521[] = {{&S27124[74], 11}, {NULL}};
const Segment SGI373522[] = {{&S27125[13], 11}, {NULL}};
const Segment SGI373523[] = {{&S27125[38], 6}, {NULL}};
const Segment SGI373524[] = {{&S27125[74], 11}, {NULL}};
const Segment SGI373525[] = {{&S27126[13], 11}, {NULL}};
const Segment SGI373526[] = {{&S27126[38], 6}, {NULL}};
const Segment SGI373527[] = {{&S27126[74], 11}, {NULL}};
const Segment SGI373528[] = {{&S27127[13], 11}, {NULL}};
const Segment SGI373529[] = {{&S27127[38], 6}, {NULL}};
const Segment SGI373530[] = {{&S27127[74], 11}, {NULL}};
const Segment SGI373531[] = {{&S27128[13], 11}, {NULL}};
const Segment SGI373532[] = {{&S27128[38], 6}, {NULL}};
const Segment SGI373533[] = {{&S27128[74], 11}, {NULL}};
const Segment SGI373534[] = {{&S27129[13], 11}, {NULL}};
const Segment SGI373535[] = {{&S27129[38], 5}, {NULL}};
const Segment SGI373536[] = {{&S27129[73], 10}, {NULL}};
const Segment SGI373537[] = {{&S27130[13], 11}, {NULL}};
const Segment SGI373538[] = {{&S27130[38], 6}, {NULL}};
const Segment SGI373539[] = {{&S27130[74], 11}, {NULL}};
const Segment SGI373540[] = {{&S27131[13], 11}, {NULL}};
const Segment SGI373541[] = {{&S27131[38], 5}, {NULL}};
const Segment SGI373542[] = {{&S27131[73], 10}, {NULL}};
const Segment SGI373543[] = {{&S27132[13], 11}, {NULL}};
const Segment SGI373544[] = {{&S27132[38], 6}, {NULL}};
const Segment SGI373545[] = {{&S27132[74], 11}, {NULL}};
const Segment SGI373546[] = {{&S27136[13], 11}, {NULL}};
const Segment SGI373547[] = {{&S27136[38], 6}, {NULL}};
const Segment SGI373548[] = {{&S27136[74], 11}, {NULL}};
const Segment SGI373549[] = {{&S27138[13], 11}, {NULL}};
const Segment SGI373550[] = {{&S27138[38], 6}, {NULL}};
const Segment SGI373551[] = {{&S27138[74], 11}, {NULL}};
const Segment SGI373552[] = {{&S27282[13], 26}, {NULL}};
const Segment SGI373553[] = {{&S27282[53], 6}, {NULL}};
const Segment SGI373554[] = {{&S27282[8], 3}, {NULL}};
const Segment SGI373555[] = {{&S27283[13], 22}, {NULL}};
const Segment SGI373556[] = {{&S27283[49], 6}, {NULL}};
const Segment SGI373557[] = {{&S27283[8], 3}, {NULL}};
const Segment SGI373558[] = {{&S27284[13], 28}, {NULL}};
const Segment SGI373559[] = {{&S27284[55], 5}, {NULL}};
const Segment SGI373560[] = {{&S27284[8], 3}, {NULL}};
const Segment SGI373561[] = {{&S27286[13], 22}, {NULL}};
const Segment SGI373562[] = {{&S27286[49], 6}, {NULL}};
const Segment SGI373563[] = {{&S27286[8], 3}, {NULL}};
const Segment SGI373564[] = {{&S27293[13], 5}, {NULL}};
const Segment SGI373565[] = {{&S27293[32], 8}, {NULL}};
const Segment SGI373566[] = {{&S27293[69], 5}, {NULL}};
const Segment SGI373567[] = {{&S25496[55], 5}, {NULL}};
const Segment SGI373568[] = {{&S25496[8], 3}, {NULL}};
const Segment SGI373569[] = {{&S25496[13], 7}, {NULL}};
const Segment SGI373570[] = {{&S25496[25], 14}, {NULL}};
const Segment SGI373571[] = {{&S25496[81], 7}, {NULL}};
const Segment SGI373572[] = {{&S25497[55], 6}, {NULL}};
const Segment SGI373573[] = {{&S25497[8], 3}, {NULL}};
const Segment SGI373574[] = {{&S25497[13], 7}, {NULL}};
const Segment SGI373575[] = {{&S25497[25], 14}, {NULL}};
const Segment SGI373576[] = {{&S25497[82], 7}, {NULL}};
const Segment SGI373577[] = {{&S25498[55], 6}, {&S25498[55], 6}, {NULL}};
const Segment SGI373578[] = {{&S25498[8], 3}, {&S25498[8], 3}, {NULL}};
const Segment SGI373579[] = {{&S25498[13], 7}, {&S25498[13], 7}, {NULL}};
const Segment SGI373580[] = {{&S25498[25], 14}, {&S25498[25], 14}, {NULL}};
const Segment SGI373581[] = {{&S25498[82], 7}, {&S25498[82], 7}, {NULL}};
const Segment SGI373582[] = {{&S25499[60], 5}, {NULL}};
const Segment SGI373583[] = {{&S25499[8], 3}, {NULL}};
const Segment SGI373584[] = {{&S25499[13], 7}, {NULL}};
const Segment SGI373585[] = {{&S25499[25], 14}, {NULL}};
const Segment SGI373586[] = {{&S25499[86], 7}, {NULL}};
const Segment SGI373587[] = {{&S25500[60], 5}, {NULL}};
const Segment SGI373588[] = {{&S25500[8], 3}, {NULL}};
const Segment SGI373589[] = {{&S25500[13], 7}, {NULL}};
const Segment SGI373590[] = {{&S25500[25], 14}, {NULL}};
const Segment SGI373591[] = {{&S25500[86], 7}, {NULL}};
const Segment SGI373592[] = {{&S25503[60], 5}, {NULL}};
const Segment SGI373593[] = {{&S25503[8], 3}, {NULL}};
const Segment SGI373594[] = {{&S25503[13], 7}, {NULL}};
const Segment SGI373595[] = {{&S25503[25], 14}, {NULL}};
const Segment SGI373596[] = {{&S25503[86], 7}, {NULL}};
const Segment SGI373597[] = {{&S25819[66], 5}, {NULL}};
const Segment SGI373598[] = {{&S25826[64], 4}, {NULL}};
const Segment SGI373599[] = {{&S28032[13], 84}, {NULL}};
const Segment SGI373600[] = {{&S28032[111], 3}, {NULL}};
const Segment SGI373601[] = {{&S28033[13], 84}, {&S28033[13], 84}, {NULL}};
const Segment SGI373602[] = {{&S28033[111], 3}, {&S28033[111], 3}, {NULL}};
const Segment SGI373603[] = {{&S28245[13], 9}, {NULL}};
const Segment SGI373604[] = {{&S28245[36], 6}, {NULL}};
const Segment SGI373605[] = {{&S28245[72], 10}, {NULL}};
const Segment SGI373606[] = {{&S28246[13], 9}, {NULL}};
const Segment SGI373607[] = {{&S28246[36], 6}, {NULL}};
const Segment SGI373608[] = {{&S28246[72], 10}, {NULL}};
const Segment SGI373609[] = {{&S28247[13], 9}, {NULL}};
const Segment SGI373610[] = {{&S28247[36], 6}, {NULL}};
const Segment SGI373611[] = {{&S28247[72], 10}, {NULL}};
const Segment SGI373612[] = {{&S28248[13], 9}, {NULL}};
const Segment SGI373613[] = {{&S28248[36], 6}, {NULL}};
const Segment SGI373614[] = {{&S28248[72], 10}, {NULL}};
const Segment SGI373615[] = {{&S28249[13], 9}, {NULL}};
const Segment SGI373616[] = {{&S28249[36], 6}, {NULL}};
const Segment SGI373617[] = {{&S28249[72], 10}, {NULL}};
const Segment SGI373618[] = {{&S28250[13], 9}, {NULL}};
const Segment SGI373619[] = {{&S28250[36], 6}, {NULL}};
const Segment SGI373620[] = {{&S28250[72], 10}, {NULL}};
const Segment SGI373621[] = {{&S28251[13], 9}, {NULL}};
const Segment SGI373622[] = {{&S28251[36], 6}, {NULL}};
const Segment SGI373623[] = {{&S28251[72], 10}, {NULL}};
const Segment SGI373624[] = {{&S28252[13], 9}, {NULL}};
const Segment SGI373625[] = {{&S28252[36], 6}, {NULL}};
const Segment SGI373626[] = {{&S28252[72], 10}, {NULL}};
const Segment SGI373627[] = {{&S28253[13], 9}, {NULL}};
const Segment SGI373628[] = {{&S28253[36], 6}, {NULL}};
const Segment SGI373629[] = {{&S28253[72], 10}, {NULL}};
const Segment SGI373630[] = {{&S28254[13], 9}, {NULL}};
const Segment SGI373631[] = {{&S28254[36], 6}, {NULL}};
const Segment SGI373632[] = {{&S28254[72], 10}, {NULL}};
const Segment SGI373633[] = {{&S28255[13], 9}, {NULL}};
const Segment SGI373634[] = {{&S28255[36], 6}, {NULL}};
const Segment SGI373635[] = {{&S28255[72], 10}, {NULL}};
const Segment SGI373636[] = {{&S28256[13], 9}, {NULL}};
const Segment SGI373637[] = {{&S28256[36], 6}, {NULL}};
const Segment SGI373638[] = {{&S28256[72], 10}, {NULL}};
const Segment SGI373639[] = {{&S28257[13], 9}, {NULL}};
const Segment SGI373640[] = {{&S28257[36], 6}, {NULL}};
const Segment SGI373641[] = {{&S28257[72], 10}, {NULL}};
const Segment SGI373642[] = {{&S28258[13], 9}, {NULL}};
const Segment SGI373643[] = {{&S28258[36], 6}, {NULL}};
const Segment SGI373644[] = {{&S28258[72], 10}, {NULL}};
const Segment SGI373645[] = {{&S28259[13], 9}, {NULL}};
const Segment SGI373646[] = {{&S28259[36], 6}, {NULL}};
const Segment SGI373647[] = {{&S28259[72], 10}, {NULL}};
const Segment SGI373648[] = {{&S28260[13], 9}, {NULL}};
const Segment SGI373649[] = {{&S28260[36], 6}, {NULL}};
const Segment SGI373650[] = {{&S28260[72], 10}, {NULL}};
const Segment SGI373651[] = {{&S28261[13], 9}, {NULL}};
const Segment SGI373652[] = {{&S28261[36], 6}, {NULL}};
const Segment SGI373653[] = {{&S28261[72], 10}, {NULL}};
const Segment SGI373654[] = {{&S28262[13], 9}, {NULL}};
const Segment SGI373655[] = {{&S28262[36], 6}, {NULL}};
const Segment SGI373656[] = {{&S28262[72], 10}, {NULL}};
const Segment SGI373657[] = {{&S28263[13], 9}, {NULL}};
const Segment SGI373658[] = {{&S28263[36], 6}, {NULL}};
const Segment SGI373659[] = {{&S28263[72], 10}, {NULL}};
const Segment SGI373660[] = {{&S28264[13], 9}, {NULL}};
const Segment SGI373661[] = {{&S28264[36], 6}, {NULL}};
const Segment SGI373662[] = {{&S28264[72], 10}, {NULL}};
const Segment SGI373663[] = {{&S28265[13], 9}, {NULL}};
const Segment SGI373664[] = {{&S28265[36], 6}, {NULL}};
const Segment SGI373665[] = {{&S28265[72], 10}, {NULL}};
const Segment SGI373666[] = {{&S28266[13], 9}, {NULL}};
const Segment SGI373667[] = {{&S28266[36], 6}, {NULL}};
const Segment SGI373668[] = {{&S28266[72], 10}, {NULL}};
const Segment SGI373669[] = {{&S28267[13], 9}, {NULL}};
const Segment SGI373670[] = {{&S28267[36], 6}, {NULL}};
const Segment SGI373671[] = {{&S28267[72], 10}, {NULL}};
const Segment SGI373672[] = {{&S28268[13], 9}, {NULL}};
const Segment SGI373673[] = {{&S28268[36], 6}, {NULL}};
const Segment SGI373674[] = {{&S28268[72], 10}, {NULL}};
const Segment SGI373675[] = {{&S28269[13], 9}, {NULL}};
const Segment SGI373676[] = {{&S28269[36], 6}, {NULL}};
const Segment SGI373677[] = {{&S28269[72], 10}, {NULL}};
const Segment SGI373678[] = {{&S28270[13], 9}, {NULL}};
const Segment SGI373679[] = {{&S28270[36], 6}, {NULL}};
const Segment SGI373680[] = {{&S28270[72], 10}, {NULL}};
const Segment SGI373681[] = {{&S28271[13], 9}, {NULL}};
const Segment SGI373682[] = {{&S28271[36], 6}, {NULL}};
const Segment SGI373683[] = {{&S28271[72], 10}, {NULL}};
const Segment SGI373684[] = {{&S28272[13], 9}, {NULL}};
const Segment SGI373685[] = {{&S28272[36], 6}, {NULL}};
const Segment SGI373686[] = {{&S28272[72], 10}, {NULL}};
const Segment SGI373687[] = {{&S28273[13], 9}, {NULL}};
const Segment SGI373688[] = {{&S28273[36], 6}, {NULL}};
const Segment SGI373689[] = {{&S28273[72], 10}, {NULL}};
const Segment SGI373690[] = {{&S28274[13], 9}, {NULL}};
const Segment SGI373691[] = {{&S28274[36], 6}, {NULL}};
const Segment SGI373692[] = {{&S28274[72], 10}, {NULL}};
const Segment SGI373693[] = {{&S28275[13], 9}, {NULL}};
const Segment SGI373694[] = {{&S28275[36], 6}, {NULL}};
const Segment SGI373695[] = {{&S28275[72], 10}, {NULL}};
const Segment SGI373696[] = {{&S28276[13], 9}, {NULL}};
const Segment SGI373697[] = {{&S28276[36], 6}, {NULL}};
const Segment SGI373698[] = {{&S28276[72], 10}, {NULL}};
const Segment SGI373699[] = {{&S28277[13], 9}, {NULL}};
const Segment SGI373700[] = {{&S28277[36], 6}, {NULL}};
const Segment SGI373701[] = {{&S28277[72], 10}, {NULL}};
const Segment SGI373702[] = {{&S28278[13], 9}, {NULL}};
const Segment SGI373703[] = {{&S28278[36], 6}, {NULL}};
const Segment SGI373704[] = {{&S28278[72], 10}, {NULL}};
const Segment SGI373705[] = {{&S28279[13], 9}, {NULL}};
const Segment SGI373706[] = {{&S28279[36], 6}, {NULL}};
const Segment SGI373707[] = {{&S28279[72], 10}, {NULL}};
const Segment SGI373708[] = {{&S28280[13], 9}, {NULL}};
const Segment SGI373709[] = {{&S28280[36], 6}, {NULL}};
const Segment SGI373710[] = {{&S28280[72], 10}, {NULL}};
const Segment SGI373711[] = {{&S28281[13], 9}, {NULL}};
const Segment SGI373712[] = {{&S28281[36], 6}, {NULL}};
const Segment SGI373713[] = {{&S28281[72], 10}, {NULL}};
const Segment SGI373714[] = {{&S28282[13], 9}, {NULL}};
const Segment SGI373715[] = {{&S28282[36], 6}, {NULL}};
const Segment SGI373716[] = {{&S28282[72], 10}, {NULL}};
const Segment SGI373717[] = {{&S28283[13], 9}, {NULL}};
const Segment SGI373718[] = {{&S28283[36], 6}, {NULL}};
const Segment SGI373719[] = {{&S28283[72], 10}, {NULL}};
const Segment SGI373720[] = {{&S28284[13], 9}, {NULL}};
const Segment SGI373721[] = {{&S28284[36], 6}, {NULL}};
const Segment SGI373722[] = {{&S28284[72], 10}, {NULL}};
const Segment SGI373723[] = {{&S28285[13], 9}, {NULL}};
const Segment SGI373724[] = {{&S28285[36], 6}, {NULL}};
const Segment SGI373725[] = {{&S28285[72], 10}, {NULL}};
const Segment SGI373726[] = {{&S28286[13], 9}, {NULL}};
const Segment SGI373727[] = {{&S28286[36], 6}, {NULL}};
const Segment SGI373728[] = {{&S28286[72], 10}, {NULL}};
const Segment SGI373729[] = {{&S28287[13], 9}, {NULL}};
const Segment SGI373730[] = {{&S28287[36], 6}, {NULL}};
const Segment SGI373731[] = {{&S28287[72], 10}, {NULL}};
const Segment SGI373732[] = {{&S28288[13], 9}, {NULL}};
const Segment SGI373733[] = {{&S28288[36], 6}, {NULL}};
const Segment SGI373734[] = {{&S28288[72], 9}, {NULL}};
const Segment SGI373735[] = {{&S28289[13], 9}, {NULL}};
const Segment SGI373736[] = {{&S28289[36], 6}, {NULL}};
const Segment SGI373737[] = {{&S28289[72], 10}, {NULL}};
const Segment SGI373738[] = {{&S28290[13], 9}, {NULL}};
const Segment SGI373739[] = {{&S28290[36], 6}, {NULL}};
const Segment SGI373740[] = {{&S28290[72], 11}, {NULL}};
const Segment SGI373741[] = {{&S28291[13], 9}, {NULL}};
const Segment SGI373742[] = {{&S28291[36], 6}, {NULL}};
const Segment SGI373743[] = {{&S28291[72], 11}, {NULL}};
const Segment SGI373744[] = {{&S28292[13], 9}, {NULL}};
const Segment SGI373745[] = {{&S28292[36], 6}, {NULL}};
const Segment SGI373746[] = {{&S28292[72], 11}, {NULL}};
const Segment SGI373747[] = {{&S28293[13], 9}, {NULL}};
const Segment SGI373748[] = {{&S28293[36], 6}, {NULL}};
const Segment SGI373749[] = {{&S28293[72], 11}, {NULL}};
const Segment SGI373750[] = {{&S28294[13], 9}, {NULL}};
const Segment SGI373751[] = {{&S28294[36], 6}, {NULL}};
const Segment SGI373752[] = {{&S28294[72], 11}, {NULL}};
const Segment SGI373753[] = {{&S28295[13], 9}, {NULL}};
const Segment SGI373754[] = {{&S28295[36], 6}, {NULL}};
const Segment SGI373755[] = {{&S28295[72], 11}, {NULL}};
const Segment SGI373756[] = {{&S28296[13], 9}, {NULL}};
const Segment SGI373757[] = {{&S28296[36], 6}, {NULL}};
const Segment SGI373758[] = {{&S28296[72], 11}, {NULL}};
const Segment SGI373759[] = {{&S28297[13], 9}, {NULL}};
const Segment SGI373760[] = {{&S28297[36], 6}, {NULL}};
const Segment SGI373761[] = {{&S28297[72], 11}, {NULL}};
const Segment SGI373762[] = {{&S28298[13], 9}, {NULL}};
const Segment SGI373763[] = {{&S28298[36], 6}, {NULL}};
const Segment SGI373764[] = {{&S28298[72], 11}, {NULL}};
const Segment SGI373765[] = {{&S28299[13], 9}, {NULL}};
const Segment SGI373766[] = {{&S28299[36], 6}, {NULL}};
const Segment SGI373767[] = {{&S28299[72], 11}, {NULL}};
const Segment SGI373768[] = {{&S28300[13], 9}, {NULL}};
const Segment SGI373769[] = {{&S28300[36], 6}, {NULL}};
const Segment SGI373770[] = {{&S28300[72], 11}, {NULL}};
const Segment SGI373771[] = {{&S28301[13], 9}, {NULL}};
const Segment SGI373772[] = {{&S28301[36], 6}, {NULL}};
const Segment SGI373773[] = {{&S28301[72], 11}, {NULL}};
const Segment SGI373774[] = {{&S28302[13], 9}, {NULL}};
const Segment SGI373775[] = {{&S28302[36], 6}, {NULL}};
const Segment SGI373776[] = {{&S28302[72], 11}, {NULL}};
const Segment SGI373777[] = {{&S28303[13], 9}, {NULL}};
const Segment SGI373778[] = {{&S28303[36], 6}, {NULL}};
const Segment SGI373779[] = {{&S28303[72], 11}, {NULL}};
const Segment SGI373780[] = {{&S28304[13], 9}, {NULL}};
const Segment SGI373781[] = {{&S28304[36], 6}, {NULL}};
const Segment SGI373782[] = {{&S28304[72], 11}, {NULL}};
const Segment SGI373783[] = {{&S28305[13], 9}, {NULL}};
const Segment SGI373784[] = {{&S28305[36], 6}, {NULL}};
const Segment SGI373785[] = {{&S28305[72], 11}, {NULL}};
const Segment SGI373786[] = {{&S28306[13], 9}, {NULL}};
const Segment SGI373787[] = {{&S28306[36], 6}, {NULL}};
const Segment SGI373788[] = {{&S28306[72], 11}, {NULL}};
const Segment SGI373789[] = {{&S28307[13], 9}, {NULL}};
const Segment SGI373790[] = {{&S28307[36], 6}, {NULL}};
const Segment SGI373791[] = {{&S28307[72], 11}, {NULL}};
const Segment SGI373792[] = {{&S28308[13], 9}, {NULL}};
const Segment SGI373793[] = {{&S28308[36], 6}, {NULL}};
const Segment SGI373794[] = {{&S28308[72], 11}, {NULL}};
const Segment SGI373795[] = {{&S28309[13], 9}, {NULL}};
const Segment SGI373796[] = {{&S28309[36], 6}, {NULL}};
const Segment SGI373797[] = {{&S28309[72], 11}, {NULL}};
const Segment SGI373798[] = {{&S28310[13], 9}, {NULL}};
const Segment SGI373799[] = {{&S28310[36], 6}, {NULL}};
const Segment SGI373800[] = {{&S28310[72], 11}, {NULL}};
const Segment SGI373801[] = {{&S28312[13], 9}, {NULL}};
const Segment SGI373802[] = {{&S28312[36], 6}, {NULL}};
const Segment SGI373803[] = {{&S28312[72], 11}, {NULL}};
const Segment SGI373804[] = {{&S28313[13], 9}, {NULL}};
const Segment SGI373805[] = {{&S28313[36], 6}, {NULL}};
const Segment SGI373806[] = {{&S28313[72], 11}, {NULL}};
const Segment SGI373807[] = {{&S28314[13], 9}, {NULL}};
const Segment SGI373808[] = {{&S28314[36], 6}, {NULL}};
const Segment SGI373809[] = {{&S28314[72], 11}, {NULL}};
const Segment SGI373810[] = {{&S28315[13], 9}, {NULL}};
const Segment SGI373811[] = {{&S28315[36], 6}, {NULL}};
const Segment SGI373812[] = {{&S28315[72], 11}, {NULL}};
const Segment SGI373813[] = {{&S28316[13], 9}, {NULL}};
const Segment SGI373814[] = {{&S28316[36], 6}, {NULL}};
const Segment SGI373815[] = {{&S28316[72], 11}, {NULL}};
const Segment SGI373816[] = {{&S28317[13], 9}, {NULL}};
const Segment SGI373817[] = {{&S28317[36], 6}, {NULL}};
const Segment SGI373818[] = {{&S28317[72], 11}, {NULL}};
const Segment SGI373819[] = {{&S28318[13], 9}, {NULL}};
const Segment SGI373820[] = {{&S28318[36], 6}, {NULL}};
const Segment SGI373821[] = {{&S28318[72], 11}, {NULL}};
const Segment SGI373822[] = {{&S28319[13], 9}, {NULL}};
const Segment SGI373823[] = {{&S28319[36], 6}, {NULL}};
const Segment SGI373824[] = {{&S28319[72], 11}, {NULL}};
const Segment SGI373825[] = {{&S28320[13], 9}, {NULL}};
const Segment SGI373826[] = {{&S28320[36], 6}, {NULL}};
const Segment SGI373827[] = {{&S28320[72], 11}, {NULL}};
const Segment SGI373828[] = {{&S28321[13], 9}, {NULL}};
const Segment SGI373829[] = {{&S28321[36], 6}, {NULL}};
const Segment SGI373830[] = {{&S28321[72], 11}, {NULL}};
const Segment SGI373831[] = {{&S28322[13], 9}, {NULL}};
const Segment SGI373832[] = {{&S28322[36], 6}, {NULL}};
const Segment SGI373833[] = {{&S28322[72], 11}, {NULL}};
const Segment SGI373834[] = {{&S28323[13], 9}, {NULL}};
const Segment SGI373835[] = {{&S28323[36], 6}, {NULL}};
const Segment SGI373836[] = {{&S28323[72], 11}, {NULL}};
const Segment SGI373837[] = {{&S28324[13], 9}, {NULL}};
const Segment SGI373838[] = {{&S28324[36], 6}, {NULL}};
const Segment SGI373839[] = {{&S28324[72], 11}, {NULL}};
const Segment SGI373840[] = {{&S28325[13], 9}, {NULL}};
const Segment SGI373841[] = {{&S28325[36], 6}, {NULL}};
const Segment SGI373842[] = {{&S28325[72], 11}, {NULL}};
const Segment SGI373843[] = {{&S28326[13], 9}, {NULL}};
const Segment SGI373844[] = {{&S28326[36], 6}, {NULL}};
const Segment SGI373845[] = {{&S28326[72], 11}, {NULL}};
const Segment SGI373846[] = {{&S28327[13], 9}, {NULL}};
const Segment SGI373847[] = {{&S28327[36], 6}, {NULL}};
const Segment SGI373848[] = {{&S28327[72], 11}, {NULL}};
const Segment SGI373849[] = {{&S28328[13], 9}, {NULL}};
const Segment SGI373850[] = {{&S28328[36], 6}, {NULL}};
const Segment SGI373851[] = {{&S28328[72], 11}, {NULL}};
const Segment SGI373852[] = {{&S28329[13], 9}, {NULL}};
const Segment SGI373853[] = {{&S28329[36], 6}, {NULL}};
const Segment SGI373854[] = {{&S28329[72], 11}, {NULL}};
const Segment SGI373855[] = {{&S28330[13], 9}, {NULL}};
const Segment SGI373856[] = {{&S28330[36], 6}, {NULL}};
const Segment SGI373857[] = {{&S28330[72], 11}, {NULL}};
const Segment SGI373858[] = {{&S28331[13], 9}, {NULL}};
const Segment SGI373859[] = {{&S28331[36], 6}, {NULL}};
const Segment SGI373860[] = {{&S28331[72], 11}, {NULL}};
const Segment SGI373861[] = {{&S28332[13], 9}, {NULL}};
const Segment SGI373862[] = {{&S28332[36], 6}, {NULL}};
const Segment SGI373863[] = {{&S28332[72], 11}, {NULL}};
const Segment SGI373864[] = {{&S28333[13], 9}, {NULL}};
const Segment SGI373865[] = {{&S28333[36], 6}, {NULL}};
const Segment SGI373866[] = {{&S28333[72], 11}, {NULL}};
const Segment SGI373867[] = {{&S28334[13], 9}, {NULL}};
const Segment SGI373868[] = {{&S28334[36], 6}, {NULL}};
const Segment SGI373869[] = {{&S28334[72], 11}, {NULL}};
const Segment SGI373870[] = {{&S28335[13], 9}, {NULL}};
const Segment SGI373871[] = {{&S28335[36], 6}, {NULL}};
const Segment SGI373872[] = {{&S28335[72], 11}, {NULL}};
const Segment SGI373873[] = {{&S28336[13], 9}, {NULL}};
const Segment SGI373874[] = {{&S28336[36], 6}, {NULL}};
const Segment SGI373875[] = {{&S28336[72], 11}, {NULL}};
const Segment SGI373876[] = {{&S28337[13], 9}, {NULL}};
const Segment SGI373877[] = {{&S28337[36], 6}, {NULL}};
const Segment SGI373878[] = {{&S28337[72], 11}, {NULL}};
const Segment SGI373879[] = {{&S28338[13], 9}, {NULL}};
const Segment SGI373880[] = {{&S28338[36], 6}, {NULL}};
const Segment SGI373881[] = {{&S28338[72], 11}, {NULL}};
const Segment SGI373882[] = {{&S28339[13], 9}, {NULL}};
const Segment SGI373883[] = {{&S28339[36], 6}, {NULL}};
const Segment SGI373884[] = {{&S28339[72], 11}, {NULL}};
const Segment SGI373885[] = {{&S28341[13], 9}, {NULL}};
const Segment SGI373886[] = {{&S28341[36], 6}, {NULL}};
const Segment SGI373887[] = {{&S28341[72], 11}, {NULL}};
const Segment SGI373888[] = {{&S28342[13], 9}, {NULL}};
const Segment SGI373889[] = {{&S28342[36], 6}, {NULL}};
const Segment SGI373890[] = {{&S28342[72], 11}, {NULL}};
const Segment SGI373891[] = {{&S28343[13], 9}, {NULL}};
const Segment SGI373892[] = {{&S28343[36], 6}, {NULL}};
const Segment SGI373893[] = {{&S28343[72], 11}, {NULL}};
const Segment SGI373894[] = {{&S27161[67], 5}, {NULL}};
const Segment SGI373895[] = {{&S27161[8], 3}, {NULL}};
const Segment SGI373896[] = {{&S27161[13], 9}, {NULL}};
const Segment SGI373897[] = {{&S27161[33], 16}, {NULL}};
const Segment SGI373898[] = {{&S27161[101], 3}, {NULL}};
const Segment SGI373899[] = {{&S27161[33], 13}, {NULL}};
const Segment SGI373900[] = {{&S27162[65], 6}, {NULL}};
const Segment SGI373901[] = {{&S27162[8], 3}, {NULL}};
const Segment SGI373902[] = {{&S27162[13], 9}, {NULL}};
const Segment SGI373903[] = {{&S27162[31], 16}, {NULL}};
const Segment SGI373904[] = {{&S27162[100], 3}, {NULL}};
const Segment SGI373905[] = {{&S27162[31], 13}, {NULL}};
const Segment SGI373906[] = {{&S27163[67], 6}, {NULL}};
const Segment SGI373907[] = {{&S27163[8], 3}, {NULL}};
const Segment SGI373908[] = {{&S27163[13], 9}, {NULL}};
const Segment SGI373909[] = {{&S27163[33], 16}, {NULL}};
const Segment SGI373910[] = {{&S27163[104], 3}, {NULL}};
const Segment SGI373911[] = {{&S27163[33], 13}, {NULL}};
const Segment SGI373912[] = {{&S27164[67], 6}, {NULL}};
const Segment SGI373913[] = {{&S27164[8], 3}, {NULL}};
const Segment SGI373914[] = {{&S27164[13], 9}, {NULL}};
const Segment SGI373915[] = {{&S27164[33], 16}, {NULL}};
const Segment SGI373916[] = {{&S27164[104], 3}, {NULL}};
const Segment SGI373917[] = {{&S27164[33], 13}, {NULL}};
const Segment SGI373918[] = {{&S27165[65], 6}, {NULL}};
const Segment SGI373919[] = {{&S27165[8], 3}, {NULL}};
const Segment SGI373920[] = {{&S27165[13], 9}, {NULL}};
const Segment SGI373921[] = {{&S27165[31], 16}, {NULL}};
const Segment SGI373922[] = {{&S27165[102], 3}, {NULL}};
const Segment SGI373923[] = {{&S27165[31], 13}, {NULL}};
const Segment SGI373924[] = {{&S27166[65], 6}, {NULL}};
const Segment SGI373925[] = {{&S27166[8], 3}, {NULL}};
const Segment SGI373926[] = {{&S27166[13], 9}, {NULL}};
const Segment SGI373927[] = {{&S27166[31], 16}, {NULL}};
const Segment SGI373928[] = {{&S27166[100], 3}, {NULL}};
const Segment SGI373929[] = {{&S27166[31], 13}, {NULL}};
const Segment SGI373930[] = {{&S27167[113], 6}, {NULL}};
const Segment SGI373931[] = {{&S27167[8], 3}, {NULL}};
const Segment SGI373932[] = {{&S27167[13], 9}, {NULL}};
const Segment SGI373933[] = {{&S27167[31], 16}, {NULL}};
const Segment SGI373934[] = {{&S27167[102], 3}, {NULL}};
const Segment SGI373935[] = {{&S27167[31], 13}, {NULL}};
const Segment SGI373936[] = {{&S27168[67], 6}, {NULL}};
const Segment SGI373937[] = {{&S27168[8], 3}, {NULL}};
const Segment SGI373938[] = {{&S27168[13], 9}, {NULL}};
const Segment SGI373939[] = {{&S27168[33], 16}, {NULL}};
const Segment SGI373940[] = {{&S27168[33], 13}, {NULL}};
const Segment SGI373941[] = {{&S27169[114], 6}, {NULL}};
const Segment SGI373942[] = {{&S27169[8], 3}, {NULL}};
const Segment SGI373943[] = {{&S27169[13], 9}, {NULL}};
const Segment SGI373944[] = {{&S27169[31], 16}, {NULL}};
const Segment SGI373945[] = {{&S27169[100], 3}, {NULL}};
const Segment SGI373946[] = {{&S27169[31], 13}, {NULL}};
const Segment SGI373947[] = {{&S27281[54], 5}, {NULL}};
const Segment SGI373948[] = {{&S27281[8], 3}, {NULL}};
const Segment SGI373949[] = {{&S27281[13], 3}, {NULL}};
const Segment SGI373950[] = {{&S27285[42], 5}, {NULL}};
const Segment SGI373951[] = {{&S27285[8], 3}, {NULL}};
const Segment SGI373952[] = {{&S27285[13], 3}, {NULL}};
const Segment SGI373953[] = {{&S27287[44], 5}, {NULL}};
const Segment SGI373954[] = {{&S27287[8], 3}, {NULL}};
const Segment SGI373955[] = {{&S27287[13], 3}, {NULL}};
const Segment SGI373956[] = {{&S27288[44], 6}, {NULL}};
const Segment SGI373957[] = {{&S27288[8], 3}, {NULL}};
const Segment SGI373958[] = {{&S27288[13], 3}, {NULL}};
const Segment SGI373959[] = {{&S27289[52], 6}, {NULL}};
const Segment SGI373960[] = {{&S27289[8], 3}, {NULL}};
const Segment SGI373961[] = {{&S27289[13], 3}, {NULL}};
const Segment SGI373962[] = {{&S27290[101], 6}, {NULL}};
const Segment SGI373963[] = {{&S27290[8], 3}, {NULL}};
const Segment SGI373964[] = {{&S27290[13], 3}, {NULL}};
const Segment SGI373965[] = {{&S27291[42], 6}, {NULL}};
const Segment SGI373966[] = {{&S27291[8], 3}, {NULL}};
const Segment SGI373967[] = {{&S27291[13], 3}, {NULL}};
const Segment SGI373968[] = {{&S27292[47], 6}, {NULL}};
const Segment SGI373969[] = {{&S27292[8], 3}, {NULL}};
const Segment SGI373970[] = {{&S27292[13], 3}, {NULL}};
const Segment SGI373971[] = {{&S27455[69], 6}, {NULL}};
const Segment SGI373972[] = {{&S27455[8], 3}, {NULL}};
const Segment SGI373973[] = {{&S27455[13], 9}, {NULL}};
const Segment SGI373974[] = {{&S27455[33], 15}, {NULL}};
const Segment SGI373975[] = {{&S27455[106], 3}, {NULL}};
const Segment SGI373976[] = {{&S27455[49], 6}, {NULL}};
const Segment SGI373977[] = {{&S27455[33], 13}, {NULL}};
const Segment SGI373978[] = {{&S27456[69], 6}, {NULL}};
const Segment SGI373979[] = {{&S27456[8], 3}, {NULL}};
const Segment SGI373980[] = {{&S27456[13], 9}, {NULL}};
const Segment SGI373981[] = {{&S27456[33], 15}, {NULL}};
const Segment SGI373982[] = {{&S27456[106], 3}, {NULL}};
const Segment SGI373983[] = {{&S27456[49], 6}, {NULL}};
const Segment SGI373984[] = {{&S27456[33], 13}, {NULL}};
const Segment SGI373985[] = {{&S27457[119], 6}, {NULL}};
const Segment SGI373986[] = {{&S27457[8], 3}, {NULL}};
const Segment SGI373987[] = {{&S27457[13], 9}, {NULL}};
const Segment SGI373988[] = {{&S27457[33], 15}, {NULL}};
const Segment SGI373989[] = {{&S27457[106], 3}, {NULL}};
const Segment SGI373990[] = {{&S27457[49], 6}, {NULL}};
const Segment SGI373991[] = {{&S27457[33], 13}, {NULL}};
const Segment SGI373992[] = {{&S27458[69], 6}, {NULL}};
const Segment SGI373993[] = {{&S27458[8], 3}, {NULL}};
const Segment SGI373994[] = {{&S27458[13], 9}, {NULL}};
const Segment SGI373995[] = {{&S27458[33], 15}, {NULL}};
const Segment SGI373996[] = {{&S27458[106], 3}, {NULL}};
const Segment SGI373997[] = {{&S27458[49], 6}, {NULL}};
const Segment SGI373998[] = {{&S27458[33], 13}, {NULL}};
const Segment SGI373999[] = {{&S27459[69], 6}, {NULL}};
const Segment SGI374000[] = {{&S27459[8], 3}, {NULL}};
const Segment SGI374001[] = {{&S27459[13], 9}, {NULL}};
const Segment SGI374002[] = {{&S27459[33], 15}, {NULL}};
const Segment SGI374003[] = {{&S27459[106], 3}, {NULL}};
const Segment SGI374004[] = {{&S27459[49], 6}, {NULL}};
const Segment SGI374005[] = {{&S27459[33], 13}, {NULL}};
const Segment SGI374006[] = {{&S27460[69], 6}, {NULL}};
const Segment SGI374007[] = {{&S27460[8], 3}, {NULL}};
const Segment SGI374008[] = {{&S27460[13], 9}, {NULL}};
const Segment SGI374009[] = {{&S27460[33], 15}, {NULL}};
const Segment SGI374010[] = {{&S27460[106], 3}, {NULL}};
const Segment SGI374011[] = {{&S27460[49], 6}, {NULL}};
const Segment SGI374012[] = {{&S27460[33], 13}, {NULL}};
const Segment SGI374013[] = {{&S27461[69], 6}, {NULL}};
const Segment SGI374014[] = {{&S27461[8], 3}, {NULL}};
const Segment SGI374015[] = {{&S27461[13], 9}, {NULL}};
const Segment SGI374016[] = {{&S27461[33], 15}, {NULL}};
const Segment SGI374017[] = {{&S27461[106], 3}, {NULL}};
const Segment SGI374018[] = {{&S27461[49], 6}, {NULL}};
const Segment SGI374019[] = {{&S27461[33], 13}, {NULL}};
const Segment SGI374020[] = {{&S27462[69], 6}, {NULL}};
const Segment SGI374021[] = {{&S27462[8], 3}, {NULL}};
const Segment SGI374022[] = {{&S27462[13], 9}, {NULL}};
const Segment SGI374023[] = {{&S27462[33], 15}, {NULL}};
const Segment SGI374024[] = {{&S27462[106], 3}, {NULL}};
const Segment SGI374025[] = {{&S27462[49], 6}, {NULL}};
const Segment SGI374026[] = {{&S27462[33], 13}, {NULL}};
const Segment SGI374027[] = {{&S27463[69], 6}, {NULL}};
const Segment SGI374028[] = {{&S27463[8], 3}, {NULL}};
const Segment SGI374029[] = {{&S27463[13], 9}, {NULL}};
const Segment SGI374030[] = {{&S27463[33], 15}, {NULL}};
const Segment SGI374031[] = {{&S27463[106], 3}, {NULL}};
const Segment SGI374032[] = {{&S27463[49], 6}, {NULL}};
const Segment SGI374033[] = {{&S27463[33], 13}, {NULL}};
const Segment SGI374034[] = {{&S27464[69], 6}, {NULL}};
const Segment SGI374035[] = {{&S27464[8], 3}, {NULL}};
const Segment SGI374036[] = {{&S27464[13], 9}, {NULL}};
const Segment SGI374037[] = {{&S27464[33], 15}, {NULL}};
const Segment SGI374038[] = {{&S27464[106], 3}, {NULL}};
const Segment SGI374039[] = {{&S27464[49], 6}, {NULL}};
const Segment SGI374040[] = {{&S27464[33], 13}, {NULL}};
const Segment SGI374041[] = {{&S27465[69], 6}, {NULL}};
const Segment SGI374042[] = {{&S27465[8], 3}, {NULL}};
const Segment SGI374043[] = {{&S27465[13], 9}, {NULL}};
const Segment SGI374044[] = {{&S27465[33], 15}, {NULL}};
const Segment SGI374045[] = {{&S27465[106], 3}, {NULL}};
const Segment SGI374046[] = {{&S27465[49], 6}, {NULL}};
const Segment SGI374047[] = {{&S27465[33], 13}, {NULL}};
const Segment SGI374048[] = {{&S27466[69], 6}, {NULL}};
const Segment SGI374049[] = {{&S27466[8], 3}, {NULL}};
const Segment SGI374050[] = {{&S27466[13], 9}, {NULL}};
const Segment SGI374051[] = {{&S27466[33], 15}, {NULL}};
const Segment SGI374052[] = {{&S27466[106], 3}, {NULL}};
const Segment SGI374053[] = {{&S27466[49], 6}, {NULL}};
const Segment SGI374054[] = {{&S27466[33], 13}, {NULL}};
const Segment SGI374055[] = {{&S27467[69], 6}, {NULL}};
const Segment SGI374056[] = {{&S27467[8], 3}, {NULL}};
const Segment SGI374057[] = {{&S27467[13], 9}, {NULL}};
const Segment SGI374058[] = {{&S27467[33], 15}, {NULL}};
const Segment SGI374059[] = {{&S27467[106], 3}, {NULL}};
const Segment SGI374060[] = {{&S27467[49], 6}, {NULL}};
const Segment SGI374061[] = {{&S27467[33], 13}, {NULL}};
const Segment SGI374062[] = {{&S27468[69], 6}, {NULL}};
const Segment SGI374063[] = {{&S27468[8], 3}, {NULL}};
const Segment SGI374064[] = {{&S27468[13], 9}, {NULL}};
const Segment SGI374065[] = {{&S27468[33], 15}, {NULL}};
const Segment SGI374066[] = {{&S27468[106], 3}, {NULL}};
const Segment SGI374067[] = {{&S27468[49], 6}, {NULL}};
const Segment SGI374068[] = {{&S27468[33], 13}, {NULL}};
const Segment SGI374069[] = {{&S27469[69], 6}, {NULL}};
const Segment SGI374070[] = {{&S27469[8], 3}, {NULL}};
const Segment SGI374071[] = {{&S27469[13], 9}, {NULL}};
const Segment SGI374072[] = {{&S27469[33], 15}, {NULL}};
const Segment SGI374073[] = {{&S27469[106], 3}, {NULL}};
const Segment SGI374074[] = {{&S27469[49], 6}, {NULL}};
const Segment SGI374075[] = {{&S27469[33], 13}, {NULL}};
const Segment SGI374076[] = {{&S27470[69], 6}, {NULL}};
const Segment SGI374077[] = {{&S27470[8], 3}, {NULL}};
const Segment SGI374078[] = {{&S27470[13], 9}, {NULL}};
const Segment SGI374079[] = {{&S27470[33], 15}, {NULL}};
const Segment SGI374080[] = {{&S27470[106], 3}, {NULL}};
const Segment SGI374081[] = {{&S27470[49], 6}, {NULL}};
const Segment SGI374082[] = {{&S27470[33], 13}, {NULL}};
const Segment SGI374083[] = {{&S27471[69], 6}, {NULL}};
const Segment SGI374084[] = {{&S27471[8], 3}, {NULL}};
const Segment SGI374085[] = {{&S27471[13], 9}, {NULL}};
const Segment SGI374086[] = {{&S27471[33], 15}, {NULL}};
const Segment SGI374087[] = {{&S27471[104], 3}, {NULL}};
const Segment SGI374088[] = {{&S27471[49], 6}, {NULL}};
const Segment SGI374089[] = {{&S27471[33], 13}, {NULL}};
const Segment SGI374090[] = {{&S27472[69], 6}, {NULL}};
const Segment SGI374091[] = {{&S27472[8], 3}, {NULL}};
const Segment SGI374092[] = {{&S27472[13], 9}, {NULL}};
const Segment SGI374093[] = {{&S27472[33], 15}, {NULL}};
const Segment SGI374094[] = {{&S27472[106], 3}, {NULL}};
const Segment SGI374095[] = {{&S27472[49], 6}, {NULL}};
const Segment SGI374096[] = {{&S27472[33], 13}, {NULL}};
const Segment SGI374097[] = {{&S27473[119], 6}, {NULL}};
const Segment SGI374098[] = {{&S27473[8], 3}, {NULL}};
const Segment SGI374099[] = {{&S27473[13], 9}, {NULL}};
const Segment SGI374100[] = {{&S27473[33], 15}, {NULL}};
const Segment SGI374101[] = {{&S27473[106], 3}, {NULL}};
const Segment SGI374102[] = {{&S27473[49], 6}, {NULL}};
const Segment SGI374103[] = {{&S27473[33], 13}, {NULL}};
const Segment SGI374104[] = {{&S27474[69], 6}, {NULL}};
const Segment SGI374105[] = {{&S27474[8], 3}, {NULL}};
const Segment SGI374106[] = {{&S27474[13], 9}, {NULL}};
const Segment SGI374107[] = {{&S27474[33], 15}, {NULL}};
const Segment SGI374108[] = {{&S27474[106], 3}, {NULL}};
const Segment SGI374109[] = {{&S27474[49], 6}, {NULL}};
const Segment SGI374110[] = {{&S27474[33], 13}, {NULL}};
const Segment SGI374111[] = {{&S27475[69], 6}, {NULL}};
const Segment SGI374112[] = {{&S27475[8], 3}, {NULL}};
const Segment SGI374113[] = {{&S27475[13], 9}, {NULL}};
const Segment SGI374114[] = {{&S27475[33], 15}, {NULL}};
const Segment SGI374115[] = {{&S27475[106], 3}, {NULL}};
const Segment SGI374116[] = {{&S27475[49], 6}, {NULL}};
const Segment SGI374117[] = {{&S27475[33], 13}, {NULL}};
const Segment SGI374118[] = {{&S27476[69], 6}, {NULL}};
const Segment SGI374119[] = {{&S27476[8], 3}, {NULL}};
const Segment SGI374120[] = {{&S27476[13], 9}, {NULL}};
const Segment SGI374121[] = {{&S27476[33], 15}, {NULL}};
const Segment SGI374122[] = {{&S27476[106], 3}, {NULL}};
const Segment SGI374123[] = {{&S27476[49], 6}, {NULL}};
const Segment SGI374124[] = {{&S27476[33], 13}, {NULL}};
const Segment SGI374125[] = {{&S27477[69], 6}, {NULL}};
const Segment SGI374126[] = {{&S27477[8], 3}, {NULL}};
const Segment SGI374127[] = {{&S27477[13], 9}, {NULL}};
const Segment SGI374128[] = {{&S27477[33], 15}, {NULL}};
const Segment SGI374129[] = {{&S27477[106], 3}, {NULL}};
const Segment SGI374130[] = {{&S27477[49], 6}, {NULL}};
const Segment SGI374131[] = {{&S27477[33], 13}, {NULL}};
const Segment SGI374132[] = {{&S27478[69], 6}, {NULL}};
const Segment SGI374133[] = {{&S27478[8], 3}, {NULL}};
const Segment SGI374134[] = {{&S27478[13], 9}, {NULL}};
const Segment SGI374135[] = {{&S27478[33], 15}, {NULL}};
const Segment SGI374136[] = {{&S27478[106], 3}, {NULL}};
const Segment SGI374137[] = {{&S27478[49], 6}, {NULL}};
const Segment SGI374138[] = {{&S27478[33], 13}, {NULL}};
const Segment SGI374139[] = {{&S27479[107], 6}, {NULL}};
const Segment SGI374140[] = {{&S27479[8], 3}, {NULL}};
const Segment SGI374141[] = {{&S27479[13], 9}, {NULL}};
const Segment SGI374142[] = {{&S27479[96], 3}, {NULL}};
const Segment SGI374143[] = {{&S27479[30], 13}, {NULL}};
const Segment SGI374144[] = {{&S27480[109], 6}, {NULL}};
const Segment SGI374145[] = {{&S27480[8], 3}, {NULL}};
const Segment SGI374146[] = {{&S27480[13], 9}, {NULL}};
const Segment SGI374147[] = {{&S27480[96], 3}, {NULL}};
const Segment SGI374148[] = {{&S27480[30], 13}, {NULL}};
const Segment SGI374149[] = {{&S27481[109], 6}, {NULL}};
const Segment SGI374150[] = {{&S27481[8], 3}, {NULL}};
const Segment SGI374151[] = {{&S27481[13], 9}, {NULL}};
const Segment SGI374152[] = {{&S27481[96], 3}, {NULL}};
const Segment SGI374153[] = {{&S27481[30], 13}, {NULL}};
const Segment SGI374154[] = {{&S27482[69], 6}, {NULL}};
const Segment SGI374155[] = {{&S27482[8], 3}, {NULL}};
const Segment SGI374156[] = {{&S27482[13], 9}, {NULL}};
const Segment SGI374157[] = {{&S27482[33], 15}, {NULL}};
const Segment SGI374158[] = {{&S27482[106], 3}, {NULL}};
const Segment SGI374159[] = {{&S27482[49], 6}, {NULL}};
const Segment SGI374160[] = {{&S27482[33], 13}, {NULL}};
const Segment SGI374161[] = {{&S27483[69], 6}, {NULL}};
const Segment SGI374162[] = {{&S27483[8], 3}, {NULL}};
const Segment SGI374163[] = {{&S27483[13], 9}, {NULL}};
const Segment SGI374164[] = {{&S27483[33], 15}, {NULL}};
const Segment SGI374165[] = {{&S27483[106], 3}, {NULL}};
const Segment SGI374166[] = {{&S27483[49], 6}, {NULL}};
const Segment SGI374167[] = {{&S27483[33], 13}, {NULL}};
const Segment SGI374168[] = {{&S27484[119], 6}, {NULL}};
const Segment SGI374169[] = {{&S27484[8], 3}, {NULL}};
const Segment SGI374170[] = {{&S27484[13], 9}, {NULL}};
const Segment SGI374171[] = {{&S27484[33], 15}, {NULL}};
const Segment SGI374172[] = {{&S27484[106], 3}, {NULL}};
const Segment SGI374173[] = {{&S27484[49], 6}, {NULL}};
const Segment SGI374174[] = {{&S27484[33], 13}, {NULL}};
const Segment SGI374175[] = {{&S27485[69], 6}, {NULL}};
const Segment SGI374176[] = {{&S27485[8], 3}, {NULL}};
const Segment SGI374177[] = {{&S27485[13], 9}, {NULL}};
const Segment SGI374178[] = {{&S27485[33], 15}, {NULL}};
const Segment SGI374179[] = {{&S27485[106], 3}, {NULL}};
const Segment SGI374180[] = {{&S27485[49], 6}, {NULL}};
const Segment SGI374181[] = {{&S27485[33], 13}, {NULL}};
const Segment SGI374182[] = {{&S27486[69], 6}, {NULL}};
const Segment SGI374183[] = {{&S27486[8], 3}, {NULL}};
const Segment SGI374184[] = {{&S27486[13], 9}, {NULL}};
const Segment SGI374185[] = {{&S27486[33], 15}, {NULL}};
const Segment SGI374186[] = {{&S27486[106], 3}, {NULL}};
const Segment SGI374187[] = {{&S27486[49], 6}, {NULL}};
const Segment SGI374188[] = {{&S27486[33], 13}, {NULL}};
const Segment SGI374189[] = {{&S27487[69], 6}, {NULL}};
const Segment SGI374190[] = {{&S27487[8], 3}, {NULL}};
const Segment SGI374191[] = {{&S27487[13], 9}, {NULL}};
const Segment SGI374192[] = {{&S27487[33], 15}, {NULL}};
const Segment SGI374193[] = {{&S27487[106], 3}, {NULL}};
const Segment SGI374194[] = {{&S27487[49], 6}, {NULL}};
const Segment SGI374195[] = {{&S27487[33], 13}, {NULL}};
const Segment SGI374196[] = {{&S27488[69], 6}, {NULL}};
const Segment SGI374197[] = {{&S27488[8], 3}, {NULL}};
const Segment SGI374198[] = {{&S27488[13], 9}, {NULL}};
const Segment SGI374199[] = {{&S27488[33], 15}, {NULL}};
const Segment SGI374200[] = {{&S27488[106], 3}, {NULL}};
const Segment SGI374201[] = {{&S27488[49], 6}, {NULL}};
const Segment SGI374202[] = {{&S27488[33], 13}, {NULL}};
const Segment SGI374203[] = {{&S27489[69], 6}, {NULL}};
const Segment SGI374204[] = {{&S27489[8], 3}, {NULL}};
const Segment SGI374205[] = {{&S27489[13], 9}, {NULL}};
const Segment SGI374206[] = {{&S27489[33], 15}, {NULL}};
const Segment SGI374207[] = {{&S27489[106], 3}, {NULL}};
const Segment SGI374208[] = {{&S27489[49], 6}, {NULL}};
const Segment SGI374209[] = {{&S27489[33], 13}, {NULL}};
const Segment SGI374210[] = {{&S27490[69], 6}, {NULL}};
const Segment SGI374211[] = {{&S27490[8], 3}, {NULL}};
const Segment SGI374212[] = {{&S27490[13], 9}, {NULL}};
const Segment SGI374213[] = {{&S27490[33], 15}, {NULL}};
const Segment SGI374214[] = {{&S27490[106], 3}, {NULL}};
const Segment SGI374215[] = {{&S27490[49], 6}, {NULL}};
const Segment SGI374216[] = {{&S27490[33], 13}, {NULL}};
const Segment SGI374217[] = {{&S27491[69], 6}, {NULL}};
const Segment SGI374218[] = {{&S27491[8], 3}, {NULL}};
const Segment SGI374219[] = {{&S27491[13], 9}, {NULL}};
const Segment SGI374220[] = {{&S27491[33], 15}, {NULL}};
const Segment SGI374221[] = {{&S27491[106], 3}, {NULL}};
const Segment SGI374222[] = {{&S27491[49], 6}, {NULL}};
const Segment SGI374223[] = {{&S27491[33], 13}, {NULL}};
const Segment SGI374224[] = {{&S27492[69], 6}, {NULL}};
const Segment SGI374225[] = {{&S27492[8], 3}, {NULL}};
const Segment SGI374226[] = {{&S27492[13], 9}, {NULL}};
const Segment SGI374227[] = {{&S27492[33], 15}, {NULL}};
const Segment SGI374228[] = {{&S27492[106], 3}, {NULL}};
const Segment SGI374229[] = {{&S27492[49], 6}, {NULL}};
const Segment SGI374230[] = {{&S27492[33], 13}, {NULL}};
const Segment SGI374231[] = {{&S27493[69], 6}, {NULL}};
const Segment SGI374232[] = {{&S27493[8], 3}, {NULL}};
const Segment SGI374233[] = {{&S27493[13], 9}, {NULL}};
const Segment SGI374234[] = {{&S27493[33], 15}, {NULL}};
const Segment SGI374235[] = {{&S27493[106], 3}, {NULL}};
const Segment SGI374236[] = {{&S27493[49], 6}, {NULL}};
const Segment SGI374237[] = {{&S27493[33], 13}, {NULL}};
const Segment SGI374238[] = {{&S27494[69], 6}, {NULL}};
const Segment SGI374239[] = {{&S27494[8], 3}, {NULL}};
const Segment SGI374240[] = {{&S27494[13], 9}, {NULL}};
const Segment SGI374241[] = {{&S27494[33], 15}, {NULL}};
const Segment SGI374242[] = {{&S27494[106], 3}, {NULL}};
const Segment SGI374243[] = {{&S27494[49], 6}, {NULL}};
const Segment SGI374244[] = {{&S27494[33], 13}, {NULL}};
const Segment SGI374245[] = {{&S27495[69], 6}, {NULL}};
const Segment SGI374246[] = {{&S27495[8], 3}, {NULL}};
const Segment SGI374247[] = {{&S27495[13], 9}, {NULL}};
const Segment SGI374248[] = {{&S27495[33], 15}, {NULL}};
const Segment SGI374249[] = {{&S27495[106], 3}, {NULL}};
const Segment SGI374250[] = {{&S27495[49], 6}, {NULL}};
const Segment SGI374251[] = {{&S27495[33], 13}, {NULL}};
const Segment SGI374252[] = {{&S27496[69], 6}, {NULL}};
const Segment SGI374253[] = {{&S27496[8], 3}, {NULL}};
const Segment SGI374254[] = {{&S27496[13], 9}, {NULL}};
const Segment SGI374255[] = {{&S27496[33], 15}, {NULL}};
const Segment SGI374256[] = {{&S27496[106], 3}, {NULL}};
const Segment SGI374257[] = {{&S27496[49], 6}, {NULL}};
const Segment SGI374258[] = {{&S27496[33], 13}, {NULL}};
const Segment SGI374259[] = {{&S27497[69], 6}, {NULL}};
const Segment SGI374260[] = {{&S27497[8], 3}, {NULL}};
const Segment SGI374261[] = {{&S27497[13], 9}, {NULL}};
const Segment SGI374262[] = {{&S27497[33], 15}, {NULL}};
const Segment SGI374263[] = {{&S27497[106], 3}, {NULL}};
const Segment SGI374264[] = {{&S27497[49], 6}, {NULL}};
const Segment SGI374265[] = {{&S27497[33], 13}, {NULL}};
const Segment SGI374266[] = {{&S27498[69], 6}, {NULL}};
const Segment SGI374267[] = {{&S27498[8], 3}, {NULL}};
const Segment SGI374268[] = {{&S27498[13], 9}, {NULL}};
const Segment SGI374269[] = {{&S27498[33], 15}, {NULL}};
const Segment SGI374270[] = {{&S27498[106], 3}, {NULL}};
const Segment SGI374271[] = {{&S27498[49], 6}, {NULL}};
const Segment SGI374272[] = {{&S27498[33], 13}, {NULL}};
const Segment SGI374273[] = {{&S27499[69], 6}, {NULL}};
const Segment SGI374274[] = {{&S27499[8], 3}, {NULL}};
const Segment SGI374275[] = {{&S27499[13], 9}, {NULL}};
const Segment SGI374276[] = {{&S27499[33], 15}, {NULL}};
const Segment SGI374277[] = {{&S27499[106], 3}, {NULL}};
const Segment SGI374278[] = {{&S27499[49], 6}, {NULL}};
const Segment SGI374279[] = {{&S27499[33], 13}, {NULL}};
const Segment SGI374280[] = {{&S27500[69], 6}, {NULL}};
const Segment SGI374281[] = {{&S27500[8], 3}, {NULL}};
const Segment SGI374282[] = {{&S27500[13], 9}, {NULL}};
const Segment SGI374283[] = {{&S27500[33], 15}, {NULL}};
const Segment SGI374284[] = {{&S27500[106], 3}, {NULL}};
const Segment SGI374285[] = {{&S27500[49], 6}, {NULL}};
const Segment SGI374286[] = {{&S27500[33], 13}, {NULL}};
const Segment SGI374287[] = {{&S27501[69], 6}, {NULL}};
const Segment SGI374288[] = {{&S27501[8], 3}, {NULL}};
const Segment SGI374289[] = {{&S27501[13], 9}, {NULL}};
const Segment SGI374290[] = {{&S27501[33], 15}, {NULL}};
const Segment SGI374291[] = {{&S27501[106], 3}, {NULL}};
const Segment SGI374292[] = {{&S27501[49], 6}, {NULL}};
const Segment SGI374293[] = {{&S27501[33], 13}, {NULL}};
const Segment SGI374294[] = {{&S27502[66], 6}, {NULL}};
const Segment SGI374295[] = {{&S27502[8], 3}, {NULL}};
const Segment SGI374296[] = {{&S27502[13], 9}, {NULL}};
const Segment SGI374297[] = {{&S27502[33], 15}, {NULL}};
const Segment SGI374298[] = {{&S27502[103], 3}, {NULL}};
const Segment SGI374299[] = {{&S27502[49], 3}, {NULL}};
const Segment SGI374300[] = {{&S27502[33], 13}, {NULL}};
const Segment SGI374301[] = {{&S27503[69], 6}, {NULL}};
const Segment SGI374302[] = {{&S27503[8], 3}, {NULL}};
const Segment SGI374303[] = {{&S27503[13], 9}, {NULL}};
const Segment SGI374304[] = {{&S27503[33], 15}, {NULL}};
const Segment SGI374305[] = {{&S27503[106], 3}, {NULL}};
const Segment SGI374306[] = {{&S27503[49], 6}, {NULL}};
const Segment SGI374307[] = {{&S27503[33], 13}, {NULL}};
const Segment SGI374308[] = {{&S27504[119], 6}, {NULL}};
const Segment SGI374309[] = {{&S27504[8], 3}, {NULL}};
const Segment SGI374310[] = {{&S27504[13], 9}, {NULL}};
const Segment SGI374311[] = {{&S27504[33], 15}, {NULL}};
const Segment SGI374312[] = {{&S27504[106], 3}, {NULL}};
const Segment SGI374313[] = {{&S27504[49], 6}, {NULL}};
const Segment SGI374314[] = {{&S27504[33], 13}, {NULL}};
const Segment SGI374315[] = {{&S27505[117], 6}, {NULL}};
const Segment SGI374316[] = {{&S27505[8], 3}, {NULL}};
const Segment SGI374317[] = {{&S27505[13], 9}, {NULL}};
const Segment SGI374318[] = {{&S27505[33], 15}, {NULL}};
const Segment SGI374319[] = {{&S27505[106], 3}, {NULL}};
const Segment SGI374320[] = {{&S27505[49], 6}, {NULL}};
const Segment SGI374321[] = {{&S27505[33], 13}, {NULL}};
const Segment SGI374322[] = {{&S27506[69], 6}, {NULL}};
const Segment SGI374323[] = {{&S27506[8], 3}, {NULL}};
const Segment SGI374324[] = {{&S27506[13], 9}, {NULL}};
const Segment SGI374325[] = {{&S27506[33], 15}, {NULL}};
const Segment SGI374326[] = {{&S27506[106], 3}, {NULL}};
const Segment SGI374327[] = {{&S27506[49], 6}, {NULL}};
const Segment SGI374328[] = {{&S27506[33], 13}, {NULL}};
const Segment SGI374329[] = {{&S27507[69], 6}, {NULL}};
const Segment SGI374330[] = {{&S27507[8], 3}, {NULL}};
const Segment SGI374331[] = {{&S27507[13], 9}, {NULL}};
const Segment SGI374332[] = {{&S27507[33], 15}, {NULL}};
const Segment SGI374333[] = {{&S27507[106], 3}, {NULL}};
const Segment SGI374334[] = {{&S27507[49], 6}, {NULL}};
const Segment SGI374335[] = {{&S27507[33], 13}, {NULL}};
const Segment SGI374336[] = {{&S27508[117], 6}, {NULL}};
const Segment SGI374337[] = {{&S27508[8], 3}, {NULL}};
const Segment SGI374338[] = {{&S27508[13], 9}, {NULL}};
const Segment SGI374339[] = {{&S27508[31], 15}, {NULL}};
const Segment SGI374340[] = {{&S27508[104], 3}, {NULL}};
const Segment SGI374341[] = {{&S27508[47], 6}, {NULL}};
const Segment SGI374342[] = {{&S27508[31], 13}, {NULL}};
const Segment SGI374343[] = {{&S27509[69], 6}, {NULL}};
const Segment SGI374344[] = {{&S27509[8], 3}, {NULL}};
const Segment SGI374345[] = {{&S27509[13], 9}, {NULL}};
const Segment SGI374346[] = {{&S27509[33], 15}, {NULL}};
const Segment SGI374347[] = {{&S27509[104], 3}, {NULL}};
const Segment SGI374348[] = {{&S27509[49], 6}, {NULL}};
const Segment SGI374349[] = {{&S27509[33], 13}, {NULL}};
const Segment SGI374350[] = {{&S27510[69], 6}, {NULL}};
const Segment SGI374351[] = {{&S27510[8], 3}, {NULL}};
const Segment SGI374352[] = {{&S27510[13], 9}, {NULL}};
const Segment SGI374353[] = {{&S27510[33], 15}, {NULL}};
const Segment SGI374354[] = {{&S27510[106], 3}, {NULL}};
const Segment SGI374355[] = {{&S27510[49], 6}, {NULL}};
const Segment SGI374356[] = {{&S27510[33], 13}, {NULL}};
const Segment SGI374357[] = {{&S27511[69], 6}, {NULL}};
const Segment SGI374358[] = {{&S27511[8], 3}, {NULL}};
const Segment SGI374359[] = {{&S27511[13], 9}, {NULL}};
const Segment SGI374360[] = {{&S27511[33], 15}, {NULL}};
const Segment SGI374361[] = {{&S27511[106], 3}, {NULL}};
const Segment SGI374362[] = {{&S27511[49], 6}, {NULL}};
const Segment SGI374363[] = {{&S27511[33], 13}, {NULL}};
const Segment SGI374364[] = {{&S27512[65], 6}, {NULL}};
const Segment SGI374365[] = {{&S27512[8], 3}, {NULL}};
const Segment SGI374366[] = {{&S27512[13], 9}, {NULL}};
const Segment SGI374367[] = {{&S27512[31], 13}, {NULL}};
const Segment SGI374368[] = {{&S27512[102], 3}, {NULL}};
const Segment SGI374369[] = {{&S27513[69], 6}, {NULL}};
const Segment SGI374370[] = {{&S27513[8], 3}, {NULL}};
const Segment SGI374371[] = {{&S27513[13], 9}, {NULL}};
const Segment SGI374372[] = {{&S27513[33], 15}, {NULL}};
const Segment SGI374373[] = {{&S27513[106], 3}, {NULL}};
const Segment SGI374374[] = {{&S27513[49], 6}, {NULL}};
const Segment SGI374375[] = {{&S27513[33], 13}, {NULL}};
const Segment SGI374376[] = {{&S27514[69], 6}, {NULL}};
const Segment SGI374377[] = {{&S27514[8], 3}, {NULL}};
const Segment SGI374378[] = {{&S27514[13], 9}, {NULL}};
const Segment SGI374379[] = {{&S27514[33], 15}, {NULL}};
const Segment SGI374380[] = {{&S27514[106], 3}, {NULL}};
const Segment SGI374381[] = {{&S27514[49], 6}, {NULL}};
const Segment SGI374382[] = {{&S27514[33], 13}, {NULL}};
const Segment SGI374383[] = {{&S27515[69], 6}, {NULL}};
const Segment SGI374384[] = {{&S27515[8], 3}, {NULL}};
const Segment SGI374385[] = {{&S27515[13], 9}, {NULL}};
const Segment SGI374386[] = {{&S27515[33], 15}, {NULL}};
const Segment SGI374387[] = {{&S27515[106], 3}, {NULL}};
const Segment SGI374388[] = {{&S27515[49], 6}, {NULL}};
const Segment SGI374389[] = {{&S27515[33], 13}, {NULL}};
const Segment SGI374390[] = {{&S27516[117], 6}, {NULL}};
const Segment SGI374391[] = {{&S27516[8], 3}, {NULL}};
const Segment SGI374392[] = {{&S27516[13], 9}, {NULL}};
const Segment SGI374393[] = {{&S27516[31], 15}, {NULL}};
const Segment SGI374394[] = {{&S27516[104], 3}, {NULL}};
const Segment SGI374395[] = {{&S27516[47], 6}, {NULL}};
const Segment SGI374396[] = {{&S27516[31], 13}, {NULL}};
const Segment SGI374397[] = {{&S27517[69], 6}, {NULL}};
const Segment SGI374398[] = {{&S27517[8], 3}, {NULL}};
const Segment SGI374399[] = {{&S27517[13], 9}, {NULL}};
const Segment SGI374400[] = {{&S27517[33], 15}, {NULL}};
const Segment SGI374401[] = {{&S27517[106], 3}, {NULL}};
const Segment SGI374402[] = {{&S27517[49], 6}, {NULL}};
const Segment SGI374403[] = {{&S27517[33], 13}, {NULL}};
const Segment SGI374404[] = {{&S27518[69], 6}, {NULL}};
const Segment SGI374405[] = {{&S27518[8], 3}, {NULL}};
const Segment SGI374406[] = {{&S27518[13], 9}, {NULL}};
const Segment SGI374407[] = {{&S27518[33], 15}, {NULL}};
const Segment SGI374408[] = {{&S27518[106], 3}, {NULL}};
const Segment SGI374409[] = {{&S27518[49], 6}, {NULL}};
const Segment SGI374410[] = {{&S27518[33], 13}, {NULL}};
const Segment SGI374411[] = {{&S27519[69], 6}, {NULL}};
const Segment SGI374412[] = {{&S27519[8], 3}, {NULL}};
const Segment SGI374413[] = {{&S27519[13], 9}, {NULL}};
const Segment SGI374414[] = {{&S27519[33], 15}, {NULL}};
const Segment SGI374415[] = {{&S27519[106], 3}, {NULL}};
const Segment SGI374416[] = {{&S27519[49], 6}, {NULL}};
const Segment SGI374417[] = {{&S27519[33], 13}, {NULL}};
const Segment SGI374418[] = {{&S27520[69], 6}, {NULL}};
const Segment SGI374419[] = {{&S27520[8], 3}, {NULL}};
const Segment SGI374420[] = {{&S27520[13], 9}, {NULL}};
const Segment SGI374421[] = {{&S27520[33], 15}, {NULL}};
const Segment SGI374422[] = {{&S27520[106], 3}, {NULL}};
const Segment SGI374423[] = {{&S27520[49], 6}, {NULL}};
const Segment SGI374424[] = {{&S27520[33], 13}, {NULL}};
const Segment SGI374425[] = {{&S27521[117], 6}, {NULL}};
const Segment SGI374426[] = {{&S27521[8], 3}, {NULL}};
const Segment SGI374427[] = {{&S27521[13], 9}, {NULL}};
const Segment SGI374428[] = {{&S27521[33], 15}, {NULL}};
const Segment SGI374429[] = {{&S27521[104], 3}, {NULL}};
const Segment SGI374430[] = {{&S27521[49], 6}, {NULL}};
const Segment SGI374431[] = {{&S27521[33], 13}, {NULL}};
const Segment SGI374432[] = {{&S27522[69], 6}, {NULL}};
const Segment SGI374433[] = {{&S27522[8], 3}, {NULL}};
const Segment SGI374434[] = {{&S27522[13], 9}, {NULL}};
const Segment SGI374435[] = {{&S27522[33], 15}, {NULL}};
const Segment SGI374436[] = {{&S27522[106], 3}, {NULL}};
const Segment SGI374437[] = {{&S27522[49], 6}, {NULL}};
const Segment SGI374438[] = {{&S27522[33], 13}, {NULL}};
const Segment SGI374439[] = {{&S27523[69], 6}, {NULL}};
const Segment SGI374440[] = {{&S27523[8], 3}, {NULL}};
const Segment SGI374441[] = {{&S27523[13], 9}, {NULL}};
const Segment SGI374442[] = {{&S27523[33], 15}, {NULL}};
const Segment SGI374443[] = {{&S27523[106], 3}, {NULL}};
const Segment SGI374444[] = {{&S27523[49], 6}, {NULL}};
const Segment SGI374445[] = {{&S27523[33], 13}, {NULL}};
const Segment SGI374446[] = {{&S27524[69], 6}, {NULL}};
const Segment SGI374447[] = {{&S27524[8], 3}, {NULL}};
const Segment SGI374448[] = {{&S27524[13], 9}, {NULL}};
const Segment SGI374449[] = {{&S27524[33], 15}, {NULL}};
const Segment SGI374450[] = {{&S27524[106], 3}, {NULL}};
const Segment SGI374451[] = {{&S27524[49], 6}, {NULL}};
const Segment SGI374452[] = {{&S27524[33], 13}, {NULL}};
const Segment SGI374453[] = {{&S27525[66], 6}, {NULL}};
const Segment SGI374454[] = {{&S27525[8], 3}, {NULL}};
const Segment SGI374455[] = {{&S27525[13], 9}, {NULL}};
const Segment SGI374456[] = {{&S27525[33], 15}, {NULL}};
const Segment SGI374457[] = {{&S27525[103], 3}, {NULL}};
const Segment SGI374458[] = {{&S27525[49], 3}, {NULL}};
const Segment SGI374459[] = {{&S27525[33], 13}, {NULL}};
const Segment SGI374460[] = {{&S27526[119], 6}, {NULL}};
const Segment SGI374461[] = {{&S27526[8], 3}, {NULL}};
const Segment SGI374462[] = {{&S27526[13], 9}, {NULL}};
const Segment SGI374463[] = {{&S27526[33], 15}, {NULL}};
const Segment SGI374464[] = {{&S27526[71], 3}, {NULL}};
const Segment SGI374465[] = {{&S27526[49], 6}, {NULL}};
const Segment SGI374466[] = {{&S27526[33], 13}, {NULL}};
const Segment SGI374467[] = {{&S27527[67], 6}, {NULL}};
const Segment SGI374468[] = {{&S27527[8], 3}, {NULL}};
const Segment SGI374469[] = {{&S27527[13], 9}, {NULL}};
const Segment SGI374470[] = {{&S27527[31], 15}, {NULL}};
const Segment SGI374471[] = {{&S27527[104], 3}, {NULL}};
const Segment SGI374472[] = {{&S27527[47], 6}, {NULL}};
const Segment SGI374473[] = {{&S27527[31], 13}, {NULL}};
const Segment SGI374474[] = {{&S27528[69], 6}, {NULL}};
const Segment SGI374475[] = {{&S27528[8], 3}, {NULL}};
const Segment SGI374476[] = {{&S27528[13], 9}, {NULL}};
const Segment SGI374477[] = {{&S27528[33], 15}, {NULL}};
const Segment SGI374478[] = {{&S27528[106], 3}, {NULL}};
const Segment SGI374479[] = {{&S27528[49], 6}, {NULL}};
const Segment SGI374480[] = {{&S27528[33], 13}, {NULL}};
const Segment SGI374481[] = {{&S27529[8], 3}, {NULL}};
const Segment SGI374482[] = {{&S27529[13], 9}, {NULL}};
const Segment SGI374483[] = {{&S27529[33], 15}, {NULL}};
const Segment SGI374484[] = {{&S27529[104], 3}, {NULL}};
const Segment SGI374485[] = {{&S27529[49], 6}, {NULL}};
const Segment SGI374486[] = {{&S27529[33], 13}, {NULL}};
const Segment SGI374487[] = {{&S27530[119], 6}, {NULL}};
const Segment SGI374488[] = {{&S27530[8], 3}, {NULL}};
const Segment SGI374489[] = {{&S27530[13], 9}, {NULL}};
const Segment SGI374490[] = {{&S27530[33], 15}, {NULL}};
const Segment SGI374491[] = {{&S27530[106], 3}, {NULL}};
const Segment SGI374492[] = {{&S27530[49], 6}, {NULL}};
const Segment SGI374493[] = {{&S27530[33], 13}, {NULL}};
const Segment SGI374494[] = {{&S27531[69], 6}, {NULL}};
const Segment SGI374495[] = {{&S27531[8], 3}, {NULL}};
const Segment SGI374496[] = {{&S27531[13], 9}, {NULL}};
const Segment SGI374497[] = {{&S27531[33], 15}, {NULL}};
const Segment SGI374498[] = {{&S27531[106], 3}, {NULL}};
const Segment SGI374499[] = {{&S27531[49], 6}, {NULL}};
const Segment SGI374500[] = {{&S27531[33], 13}, {NULL}};
const Segment SGI374501[] = {{&S27532[69], 6}, {NULL}};
const Segment SGI374502[] = {{&S27532[8], 3}, {NULL}};
const Segment SGI374503[] = {{&S27532[13], 9}, {NULL}};
const Segment SGI374504[] = {{&S27532[33], 15}, {NULL}};
const Segment SGI374505[] = {{&S27532[106], 3}, {NULL}};
const Segment SGI374506[] = {{&S27532[49], 6}, {NULL}};
const Segment SGI374507[] = {{&S27532[33], 13}, {NULL}};
const Segment SGI374508[] = {{&S27533[69], 6}, {NULL}};
const Segment SGI374509[] = {{&S27533[8], 3}, {NULL}};
const Segment SGI374510[] = {{&S27533[13], 9}, {NULL}};
const Segment SGI374511[] = {{&S27533[33], 15}, {NULL}};
const Segment SGI374512[] = {{&S27533[106], 3}, {NULL}};
const Segment SGI374513[] = {{&S27533[49], 6}, {NULL}};
const Segment SGI374514[] = {{&S27533[33], 13}, {NULL}};
const Segment SGI374515[] = {{&S27534[119], 6}, {NULL}};
const Segment SGI374516[] = {{&S27534[8], 3}, {NULL}};
const Segment SGI374517[] = {{&S27534[13], 9}, {NULL}};
const Segment SGI374518[] = {{&S27534[33], 15}, {NULL}};
const Segment SGI374519[] = {{&S27534[106], 3}, {NULL}};
const Segment SGI374520[] = {{&S27534[49], 6}, {NULL}};
const Segment SGI374521[] = {{&S27534[33], 13}, {NULL}};
const Segment SGI374522[] = {{&S27535[69], 6}, {NULL}};
const Segment SGI374523[] = {{&S27535[8], 3}, {NULL}};
const Segment SGI374524[] = {{&S27535[13], 9}, {NULL}};
const Segment SGI374525[] = {{&S27535[33], 15}, {NULL}};
const Segment SGI374526[] = {{&S27535[106], 3}, {NULL}};
const Segment SGI374527[] = {{&S27535[49], 6}, {NULL}};
const Segment SGI374528[] = {{&S27535[33], 13}, {NULL}};
const Segment SGI374529[] = {{&S27536[69], 6}, {NULL}};
const Segment SGI374530[] = {{&S27536[8], 3}, {NULL}};
const Segment SGI374531[] = {{&S27536[13], 9}, {NULL}};
const Segment SGI374532[] = {{&S27536[33], 15}, {NULL}};
const Segment SGI374533[] = {{&S27536[106], 3}, {NULL}};
const Segment SGI374534[] = {{&S27536[49], 6}, {NULL}};
const Segment SGI374535[] = {{&S27536[33], 13}, {NULL}};
const Segment SGI374536[] = {{&S27537[69], 6}, {NULL}};
const Segment SGI374537[] = {{&S27537[8], 3}, {NULL}};
const Segment SGI374538[] = {{&S27537[13], 9}, {NULL}};
const Segment SGI374539[] = {{&S27537[33], 15}, {NULL}};
const Segment SGI374540[] = {{&S27537[106], 3}, {NULL}};
const Segment SGI374541[] = {{&S27537[49], 6}, {NULL}};
const Segment SGI374542[] = {{&S27537[33], 13}, {NULL}};
const Segment SGI374543[] = {{&S27538[119], 6}, {NULL}};
const Segment SGI374544[] = {{&S27538[8], 3}, {NULL}};
const Segment SGI374545[] = {{&S27538[13], 9}, {NULL}};
const Segment SGI374546[] = {{&S27538[33], 15}, {NULL}};
const Segment SGI374547[] = {{&S27538[106], 3}, {NULL}};
const Segment SGI374548[] = {{&S27538[49], 6}, {NULL}};
const Segment SGI374549[] = {{&S27538[33], 13}, {NULL}};
const Segment SGI374550[] = {{&S27539[66], 6}, {NULL}};
const Segment SGI374551[] = {{&S27539[8], 3}, {NULL}};
const Segment SGI374552[] = {{&S27539[13], 9}, {NULL}};
const Segment SGI374553[] = {{&S27539[33], 15}, {NULL}};
const Segment SGI374554[] = {{&S27539[103], 3}, {NULL}};
const Segment SGI374555[] = {{&S27539[49], 3}, {NULL}};
const Segment SGI374556[] = {{&S27539[33], 13}, {NULL}};
const Segment SGI374557[] = {{&S27540[96], 6}, {NULL}};
const Segment SGI374558[] = {{&S27540[8], 3}, {NULL}};
const Segment SGI374559[] = {{&S27540[13], 9}, {NULL}};
const Segment SGI374560[] = {{&S27540[33], 13}, {NULL}};
const Segment SGI374561[] = {{&S27540[83], 3}, {NULL}};
const Segment SGI374562[] = {{&S27997[55], 5}, {NULL}};
const Segment SGI374563[] = {{&S27997[8], 3}, {NULL}};
const Segment SGI374564[] = {{&S27997[13], 7}, {NULL}};
const Segment SGI374565[] = {{&S27997[25], 14}, {NULL}};
const Segment SGI374566[] = {{&S27997[81], 7}, {NULL}};
const Segment SGI374567[] = {{&S27998[55], 6}, {NULL}};
const Segment SGI374568[] = {{&S27998[8], 3}, {NULL}};
const Segment SGI374569[] = {{&S27998[13], 7}, {NULL}};
const Segment SGI374570[] = {{&S27998[25], 14}, {NULL}};
const Segment SGI374571[] = {{&S27998[82], 7}, {NULL}};
const Segment SGI374572[] = {{&S27999[60], 5}, {NULL}};
const Segment SGI374573[] = {{&S27999[8], 3}, {NULL}};
const Segment SGI374574[] = {{&S27999[13], 7}, {NULL}};
const Segment SGI374575[] = {{&S27999[25], 14}, {NULL}};
const Segment SGI374576[] = {{&S27999[86], 7}, {NULL}};
const Segment SGI374577[] = {{&S28000[60], 5}, {NULL}};
const Segment SGI374578[] = {{&S28000[8], 3}, {NULL}};
const Segment SGI374579[] = {{&S28000[13], 7}, {NULL}};
const Segment SGI374580[] = {{&S28000[25], 14}, {NULL}};
const Segment SGI374581[] = {{&S28000[86], 7}, {NULL}};
const Segment SGI374582[] = {{&S28002[55], 5}, {NULL}};
const Segment SGI374583[] = {{&S28002[8], 3}, {NULL}};
const Segment SGI374584[] = {{&S28002[13], 7}, {NULL}};
const Segment SGI374585[] = {{&S28002[25], 14}, {NULL}};
const Segment SGI374586[] = {{&S28002[81], 7}, {NULL}};
const Segment SGI374587[] = {{&S28004[60], 5}, {NULL}};
const Segment SGI374588[] = {{&S28004[8], 3}, {NULL}};
const Segment SGI374589[] = {{&S28004[13], 7}, {NULL}};
const Segment SGI374590[] = {{&S28004[25], 14}, {NULL}};
const Segment SGI374591[] = {{&S28004[86], 7}, {NULL}};
const Segment SGI374592[] = {{&S28005[55], 6}, {NULL}};
const Segment SGI374593[] = {{&S28005[8], 3}, {NULL}};
const Segment SGI374594[] = {{&S28005[13], 7}, {NULL}};
const Segment SGI374595[] = {{&S28005[25], 14}, {NULL}};
const Segment SGI374596[] = {{&S28005[81], 7}, {NULL}};
const Segment SGI374597[] = {{&S28468[60], 5}, {NULL}};
const Segment SGI374598[] = {{&S28468[8], 3}, {NULL}};
const Segment SGI374599[] = {{&S28468[13], 7}, {NULL}};
const Segment SGI374600[] = {{&S28468[25], 14}, {NULL}};
const Segment SGI374601[] = {{&S28468[94], 3}, {NULL}};
const Segment SGI374602[] = {{&S28469[106], 6}, {NULL}};
const Segment SGI374603[] = {{&S28469[8], 3}, {NULL}};
const Segment SGI374604[] = {{&S28469[13], 7}, {NULL}};
const Segment SGI374605[] = {{&S28469[25], 14}, {NULL}};
const Segment SGI374606[] = {{&S28469[95], 3}, {NULL}};
const Segment SGI374607[] = {{&S28470[60], 6}, {NULL}};
const Segment SGI374608[] = {{&S28470[8], 3}, {NULL}};
const Segment SGI374609[] = {{&S28470[13], 7}, {NULL}};
const Segment SGI374610[] = {{&S28470[25], 14}, {NULL}};
const Segment SGI374611[] = {{&S28470[97], 3}, {NULL}};
const Segment SGI374612[] = {{&S28471[60], 6}, {NULL}};
const Segment SGI374613[] = {{&S28471[8], 3}, {NULL}};
const Segment SGI374614[] = {{&S28471[13], 7}, {NULL}};
const Segment SGI374615[] = {{&S28471[25], 14}, {NULL}};
const Segment SGI374616[] = {{&S28471[97], 3}, {NULL}};
const Segment SGI374617[] = {{&S28472[60], 6}, {NULL}};
const Segment SGI374618[] = {{&S28472[8], 3}, {NULL}};
const Segment SGI374619[] = {{&S28472[13], 7}, {NULL}};
const Segment SGI374620[] = {{&S28472[25], 14}, {NULL}};
const Segment SGI374621[] = {{&S28472[97], 3}, {NULL}};
const Segment SGI374622[] = {{&S30362[13], 39}, {NULL}};
const Segment SGI374623[] = {{&S30362[66], 5}, {NULL}};
const Segment SGI374624[] = {{&S28473[60], 6}, {NULL}};
const Segment SGI374625[] = {{&S28473[8], 3}, {NULL}};
const Segment SGI374626[] = {{&S28473[13], 7}, {NULL}};
const Segment SGI374627[] = {{&S28473[25], 14}, {NULL}};
const Segment SGI374628[] = {{&S28473[95], 3}, {NULL}};
const Segment SGI374629[] = {{&S30363[13], 39}, {NULL}};
const Segment SGI374630[] = {{&S30363[66], 5}, {NULL}};
const Segment SGI374631[] = {{&S30364[13], 39}, {NULL}};
const Segment SGI374632[] = {{&S28474[60], 6}, {NULL}};
const Segment SGI374633[] = {{&S30364[66], 5}, {NULL}};
const Segment SGI374634[] = {{&S28474[8], 3}, {NULL}};
const Segment SGI374635[] = {{&S28474[13], 7}, {NULL}};
const Segment SGI374636[] = {{&S28474[25], 14}, {NULL}};
const Segment SGI374637[] = {{&S28474[95], 3}, {NULL}};
const Segment SGI374638[] = {{&S30365[13], 39}, {NULL}};
const Segment SGI374639[] = {{&S30365[66], 5}, {NULL}};
const Segment SGI374640[] = {{&S28475[98], 6}, {NULL}};
const Segment SGI374641[] = {{&S28475[8], 3}, {NULL}};
const Segment SGI374642[] = {{&S28475[13], 14}, {NULL}};
const Segment SGI374643[] = {{&S28475[85], 3}, {NULL}};
const Segment SGI374644[] = {{&S30366[13], 39}, {NULL}};
const Segment SGI374645[] = {{&S30366[66], 5}, {NULL}};
const Segment SGI374646[] = {{&S28476[91], 6}, {NULL}};
const Segment SGI374647[] = {{&S28476[8], 3}, {NULL}};
const Segment SGI374648[] = {{&S28476[13], 14}, {NULL}};
const Segment SGI374649[] = {{&S28476[78], 3}, {NULL}};
const Segment SGI374650[] = {{&S30367[13], 39}, {NULL}};
const Segment SGI374651[] = {{&S30367[66], 5}, {NULL}};
const Segment SGI374652[] = {{&S28477[91], 6}, {NULL}};
const Segment SGI374653[] = {{&S28477[8], 3}, {NULL}};
const Segment SGI374654[] = {{&S28477[13], 14}, {NULL}};
const Segment SGI374655[] = {{&S28477[78], 3}, {NULL}};
const Segment SGI374656[] = {{&S30368[13], 39}, {NULL}};
const Segment SGI374657[] = {{&S30368[66], 5}, {NULL}};
const Segment SGI374658[] = {{&S28478[98], 6}, {NULL}};
const Segment SGI374659[] = {{&S28478[8], 3}, {NULL}};
const Segment SGI374660[] = {{&S28478[13], 14}, {NULL}};
const Segment SGI374661[] = {{&S28478[85], 3}, {NULL}};
const Segment SGI374662[] = {{&S30369[13], 39}, {NULL}};
const Segment SGI374663[] = {{&S30369[66], 5}, {NULL}};
const Segment SGI374664[] = {{&S30370[13], 39}, {NULL}};
const Segment SGI374665[] = {{&S30370[66], 5}, {NULL}};
const Segment SGI374666[] = {{&S30371[13], 39}, {NULL}};
const Segment SGI374667[] = {{&S30371[66], 5}, {NULL}};
const Segment SGI374668[] = {{&S30372[13], 39}, {NULL}};
const Segment SGI374669[] = {{&S30372[66], 5}, {NULL}};
const Segment SGI374670[] = {{&S30373[13], 39}, {NULL}};
const Segment SGI374671[] = {{&S30373[66], 5}, {NULL}};
const Segment SGI374672[] = {{&S30374[13], 39}, {NULL}};
const Segment SGI374673[] = {{&S30374[66], 5}, {NULL}};
const Segment SGI374674[] = {{&S30375[13], 39}, {NULL}};
const Segment SGI374675[] = {{&S30375[66], 5}, {NULL}};
const Segment SGI374676[] = {{&S30376[13], 39}, {NULL}};
const Segment SGI374677[] = {{&S30376[66], 5}, {NULL}};
const Segment SGI374678[] = {{&S30377[13], 39}, {NULL}};
const Segment SGI374679[] = {{&S30377[66], 5}, {NULL}};
const Segment SGI374680[] = {{&S30378[13], 39}, {NULL}};
const Segment SGI374681[] = {{&S30378[66], 5}, {NULL}};
const Segment SGI374682[] = {{&S30379[13], 39}, {NULL}};
const Segment SGI374683[] = {{&S30379[66], 5}, {NULL}};
const Segment SGI374684[] = {{&S30380[13], 39}, {NULL}};
const Segment SGI374685[] = {{&S30380[66], 5}, {NULL}};
const Segment SGI374686[] = {{&S30381[13], 39}, {NULL}};
const Segment SGI374687[] = {{&S30381[66], 5}, {NULL}};
const Segment SGI374688[] = {{&S30382[13], 39}, {NULL}};
const Segment SGI374689[] = {{&S30382[66], 5}, {NULL}};
const Segment SGI374690[] = {{&S30383[13], 39}, {NULL}};
const Segment SGI374691[] = {{&S30383[66], 5}, {NULL}};
const Segment SGI374692[] = {{&S30384[13], 39}, {NULL}};
const Segment SGI374693[] = {{&S30384[66], 5}, {NULL}};
const Segment SGI374694[] = {{&S30385[13], 39}, {NULL}};
const Segment SGI374695[] = {{&S30385[66], 5}, {NULL}};
const Segment SGI374696[] = {{&S30386[13], 39}, {NULL}};
const Segment SGI374697[] = {{&S30386[66], 5}, {NULL}};
const Segment SGI374698[] = {{&S30387[13], 39}, {NULL}};
const Segment SGI374699[] = {{&S30387[66], 5}, {NULL}};
const Segment SGI374700[] = {{&S30388[13], 39}, {NULL}};
const Segment SGI374701[] = {{&S30388[66], 5}, {NULL}};
const Segment SGI374702[] = {{&S30389[13], 39}, {NULL}};
const Segment SGI374703[] = {{&S30389[66], 5}, {NULL}};
const Segment SGI374704[] = {{&S30390[13], 39}, {NULL}};
const Segment SGI374705[] = {{&S30390[66], 5}, {NULL}};
const Segment SGI374706[] = {{&S30391[13], 39}, {NULL}};
const Segment SGI374707[] = {{&S30391[66], 5}, {NULL}};
const Segment SGI374708[] = {{&S30392[13], 39}, {NULL}};
const Segment SGI374709[] = {{&S30392[66], 5}, {NULL}};
const Segment SGI374710[] = {{&S30393[13], 39}, {NULL}};
const Segment SGI374711[] = {{&S30393[66], 5}, {NULL}};
const Segment SGI374712[] = {{&S30394[13], 39}, {NULL}};
const Segment SGI374713[] = {{&S30394[66], 5}, {NULL}};
const Segment SGI374714[] = {{&S30395[13], 39}, {NULL}};
const Segment SGI374715[] = {{&S30395[66], 5}, {NULL}};
const Segment SGI374716[] = {{&S30396[13], 39}, {NULL}};
const Segment SGI374717[] = {{&S30396[66], 5}, {NULL}};
const Segment SGI374718[] = {{&S30397[13], 39}, {NULL}};
const Segment SGI374719[] = {{&S30397[66], 5}, {NULL}};
const Segment SGI374720[] = {{&S30398[13], 39}, {NULL}};
const Segment SGI374721[] = {{&S30398[66], 5}, {NULL}};
const Segment SGI374722[] = {{&S30399[13], 39}, {NULL}};
const Segment SGI374723[] = {{&S30399[66], 5}, {NULL}};
const Segment SGI374724[] = {{&S30400[13], 39}, {NULL}};
const Segment SGI374725[] = {{&S30400[66], 5}, {NULL}};
const Segment SGI374726[] = {{&S30401[13], 39}, {NULL}};
const Segment SGI374727[] = {{&S30401[66], 5}, {NULL}};
const Segment SGI374728[] = {{&S30402[13], 39}, {NULL}};
const Segment SGI374729[] = {{&S30402[66], 5}, {NULL}};
const Segment SGI374730[] = {{&S30403[13], 39}, {NULL}};
const Segment SGI374731[] = {{&S30403[66], 5}, {NULL}};
const Segment SGI374732[] = {{&S30404[13], 39}, {NULL}};
const Segment SGI374733[] = {{&S30404[66], 5}, {NULL}};
const Segment SGI374734[] = {{&S30405[13], 39}, {NULL}};
const Segment SGI374735[] = {{&S30405[66], 5}, {NULL}};
const Segment SGI374736[] = {{&S30406[13], 39}, {NULL}};
const Segment SGI374737[] = {{&S30406[66], 5}, {NULL}};
const Segment SGI374738[] = {{&S30407[13], 39}, {NULL}};
const Segment SGI374739[] = {{&S30407[66], 5}, {NULL}};
const Segment SGI374740[] = {{&S30408[13], 39}, {NULL}};
const Segment SGI374741[] = {{&S30408[66], 5}, {NULL}};
const Segment SGI374742[] = {{&S30409[13], 39}, {NULL}};
const Segment SGI374743[] = {{&S30409[66], 5}, {NULL}};
const Segment SGI374744[] = {{&S30410[13], 39}, {NULL}};
const Segment SGI374745[] = {{&S30410[66], 5}, {NULL}};
const Segment SGI374746[] = {{&S30411[13], 39}, {NULL}};
const Segment SGI374747[] = {{&S30411[66], 5}, {NULL}};
const Segment SGI374748[] = {{&S30412[13], 39}, {NULL}};
const Segment SGI374749[] = {{&S30412[66], 5}, {NULL}};
const Segment SGI374750[] = {{&S30413[13], 39}, {NULL}};
const Segment SGI374751[] = {{&S30413[66], 5}, {NULL}};
const Segment SGI374752[] = {{&S30414[13], 39}, {NULL}};
const Segment SGI374753[] = {{&S30414[66], 5}, {NULL}};
const Segment SGI374754[] = {{&S30415[13], 39}, {NULL}};
const Segment SGI374755[] = {{&S30415[66], 5}, {NULL}};
const Segment SGI374756[] = {{&S30416[13], 39}, {NULL}};
const Segment SGI374757[] = {{&S30416[66], 5}, {NULL}};
const Segment SGI374758[] = {{&S30417[13], 39}, {NULL}};
const Segment SGI374759[] = {{&S30417[66], 5}, {NULL}};
const Segment SGI374760[] = {{&S28718[60], 5}, {NULL}};
const Segment SGI374761[] = {{&S28718[8], 3}, {NULL}};
const Segment SGI374762[] = {{&S28718[13], 7}, {NULL}};
const Segment SGI374763[] = {{&S28718[25], 14}, {NULL}};
const Segment SGI374764[] = {{&S28718[94], 3}, {NULL}};
const Segment SGI374765[] = {{&S28719[60], 6}, {NULL}};
const Segment SGI374766[] = {{&S28719[8], 3}, {NULL}};
const Segment SGI374767[] = {{&S28719[13], 7}, {NULL}};
const Segment SGI374768[] = {{&S28719[25], 14}, {NULL}};
const Segment SGI374769[] = {{&S28719[97], 3}, {NULL}};
const Segment SGI374770[] = {{&S28720[106], 6}, {NULL}};
const Segment SGI374771[] = {{&S28720[8], 3}, {NULL}};
const Segment SGI374772[] = {{&S28720[13], 7}, {NULL}};
const Segment SGI374773[] = {{&S28720[25], 14}, {NULL}};
const Segment SGI374774[] = {{&S28720[95], 3}, {NULL}};
const Segment SGI374775[] = {{&S28721[60], 6}, {NULL}};
const Segment SGI374776[] = {{&S28721[8], 3}, {NULL}};
const Segment SGI374777[] = {{&S28721[13], 7}, {NULL}};
const Segment SGI374778[] = {{&S28721[25], 14}, {NULL}};
const Segment SGI374779[] = {{&S28721[95], 3}, {NULL}};
const Segment SGI374780[] = {{&S28722[60], 6}, {NULL}};
const Segment SGI374781[] = {{&S28722[8], 3}, {NULL}};
const Segment SGI374782[] = {{&S28722[13], 7}, {NULL}};
const Segment SGI374783[] = {{&S28722[25], 14}, {NULL}};
const Segment SGI374784[] = {{&S28722[97], 3}, {NULL}};
const Segment SGI374785[] = {{&S28723[60], 6}, {NULL}};
const Segment SGI374786[] = {{&S28723[8], 3}, {NULL}};
const Segment SGI374787[] = {{&S28723[13], 7}, {NULL}};
const Segment SGI374788[] = {{&S28723[25], 14}, {NULL}};
const Segment SGI374789[] = {{&S28723[95], 3}, {NULL}};
const Segment SGI374790[] = {{&S28724[105], 5}, {NULL}};
const Segment SGI374791[] = {{&S28724[8], 3}, {NULL}};
const Segment SGI374792[] = {{&S28724[13], 7}, {NULL}};
const Segment SGI374793[] = {{&S28724[25], 14}, {NULL}};
const Segment SGI374794[] = {{&S28724[91], 3}, {NULL}};
const Segment SGI374795[] = {{&S28725[60], 6}, {NULL}};
const Segment SGI374796[] = {{&S28725[8], 3}, {NULL}};
const Segment SGI374797[] = {{&S28725[13], 7}, {NULL}};
const Segment SGI374798[] = {{&S28725[25], 14}, {NULL}};
const Segment SGI374799[] = {{&S28725[97], 3}, {NULL}};
const Segment SGI374800[] = {{&S28726[60], 6}, {NULL}};
const Segment SGI374801[] = {{&S28726[8], 3}, {NULL}};
const Segment SGI374802[] = {{&S28726[13], 7}, {NULL}};
const Segment SGI374803[] = {{&S28726[25], 14}, {NULL}};
const Segment SGI374804[] = {{&S28726[97], 3}, {NULL}};
const Segment SGI374805[] = {{&S28727[91], 6}, {NULL}};
const Segment SGI374806[] = {{&S28727[8], 3}, {NULL}};
const Segment SGI374807[] = {{&S28727[13], 14}, {NULL}};
const Segment SGI374808[] = {{&S28727[78], 3}, {NULL}};
const Segment SGI374809[] = {{&S28728[91], 6}, {NULL}};
const Segment SGI374810[] = {{&S28728[8], 3}, {NULL}};
const Segment SGI374811[] = {{&S28728[13], 14}, {NULL}};
const Segment SGI374812[] = {{&S28728[78], 3}, {NULL}};
const Segment SGI374813[] = {{&S28729[60], 6}, {NULL}};
const Segment SGI374814[] = {{&S28729[8], 3}, {NULL}};
const Segment SGI374815[] = {{&S28729[13], 7}, {NULL}};
const Segment SGI374816[] = {{&S28729[25], 14}, {NULL}};
const Segment SGI374817[] = {{&S28729[97], 3}, {NULL}};
const Segment SGI374818[] = {{&S28730[60], 5}, {NULL}};
const Segment SGI374819[] = {{&S28730[8], 3}, {NULL}};
const Segment SGI374820[] = {{&S28730[13], 7}, {NULL}};
const Segment SGI374821[] = {{&S28730[25], 14}, {NULL}};
const Segment SGI374822[] = {{&S28730[94], 3}, {NULL}};
const Segment SGI374823[] = {{&S28731[60], 6}, {NULL}};
const Segment SGI374824[] = {{&S28731[8], 3}, {NULL}};
const Segment SGI374825[] = {{&S28731[13], 7}, {NULL}};
const Segment SGI374826[] = {{&S28731[25], 14}, {NULL}};
const Segment SGI374827[] = {{&S28731[95], 3}, {NULL}};
const Segment SGI374828[] = {{&S28732[109], 6}, {NULL}};
const Segment SGI374829[] = {{&S28732[8], 3}, {NULL}};
const Segment SGI374830[] = {{&S28732[13], 7}, {NULL}};
const Segment SGI374831[] = {{&S28732[25], 14}, {NULL}};
const Segment SGI374832[] = {{&S28732[95], 3}, {NULL}};
const Segment SGI374833[] = {{&S28733[116], 6}, {NULL}};
const Segment SGI374834[] = {{&S28733[8], 3}, {NULL}};
const Segment SGI374835[] = {{&S28733[13], 7}, {NULL}};
const Segment SGI374836[] = {{&S28733[25], 14}, {NULL}};
const Segment SGI374837[] = {{&S28733[105], 3}, {NULL}};
const Segment SGI374838[] = {{&S28734[60], 6}, {NULL}};
const Segment SGI374839[] = {{&S28734[8], 3}, {NULL}};
const Segment SGI374840[] = {{&S28734[13], 7}, {NULL}};
const Segment SGI374841[] = {{&S28734[25], 14}, {NULL}};
const Segment SGI374842[] = {{&S28734[96], 3}, {NULL}};
const Segment SGI374843[] = {{&S29453[57], 6}, {NULL}};
const Segment SGI374844[] = {{&S29453[8], 3}, {NULL}};
const Segment SGI374845[] = {{&S29453[13], 9}, {NULL}};
const Segment SGI374846[] = {{&S29453[94], 3}, {NULL}};
const Segment SGI374847[] = {{&S29453[30], 13}, {NULL}};
const Segment SGI374848[] = {{&S29454[57], 6}, {NULL}};
const Segment SGI374849[] = {{&S29454[8], 3}, {NULL}};
const Segment SGI374850[] = {{&S29454[13], 9}, {NULL}};
const Segment SGI374851[] = {{&S29454[92], 3}, {NULL}};
const Segment SGI374852[] = {{&S29454[30], 13}, {NULL}};
const Segment SGI374853[] = {{&S29455[57], 6}, {NULL}};
const Segment SGI374854[] = {{&S29455[8], 3}, {NULL}};
const Segment SGI374855[] = {{&S29455[13], 9}, {NULL}};
const Segment SGI374856[] = {{&S29455[92], 3}, {NULL}};
const Segment SGI374857[] = {{&S29455[30], 13}, {NULL}};
const Segment SGI374858[] = {{&S29456[57], 6}, {NULL}};
const Segment SGI374859[] = {{&S29456[8], 3}, {NULL}};
const Segment SGI374860[] = {{&S29456[13], 9}, {NULL}};
const Segment SGI374861[] = {{&S29456[94], 3}, {NULL}};
const Segment SGI374862[] = {{&S29456[30], 13}, {NULL}};
const Segment SGI374863[] = {{&S29457[57], 6}, {NULL}};
const Segment SGI374864[] = {{&S29457[8], 3}, {NULL}};
const Segment SGI374865[] = {{&S29457[13], 9}, {NULL}};
const Segment SGI374866[] = {{&S29457[92], 3}, {NULL}};
const Segment SGI374867[] = {{&S29457[30], 13}, {NULL}};
const Segment SGI374868[] = {{&S29458[57], 6}, {NULL}};
const Segment SGI374869[] = {{&S29458[8], 3}, {NULL}};
const Segment SGI374870[] = {{&S29458[13], 9}, {NULL}};
const Segment SGI374871[] = {{&S29458[94], 3}, {NULL}};
const Segment SGI374872[] = {{&S29458[30], 13}, {NULL}};
const Segment SGI374873[] = {{&S29459[57], 6}, {NULL}};
const Segment SGI374874[] = {{&S29459[8], 3}, {NULL}};
const Segment SGI374875[] = {{&S29459[13], 9}, {NULL}};
const Segment SGI374876[] = {{&S29459[92], 3}, {NULL}};
const Segment SGI374877[] = {{&S29459[30], 13}, {NULL}};
const Segment SGI374878[] = {{&S29460[57], 6}, {NULL}};
const Segment SGI374879[] = {{&S29460[8], 3}, {NULL}};
const Segment SGI374880[] = {{&S29460[13], 9}, {NULL}};
const Segment SGI374881[] = {{&S29460[94], 3}, {NULL}};
const Segment SGI374882[] = {{&S29460[30], 13}, {NULL}};
const Segment SGI374883[] = {{&S29461[57], 6}, {NULL}};
const Segment SGI374884[] = {{&S29461[8], 3}, {NULL}};
const Segment SGI374885[] = {{&S29461[13], 9}, {NULL}};
const Segment SGI374886[] = {{&S29461[92], 3}, {NULL}};
const Segment SGI374887[] = {{&S29461[30], 13}, {NULL}};
const Segment SGI374888[] = {{&S29462[57], 6}, {NULL}};
const Segment SGI374889[] = {{&S29462[8], 3}, {NULL}};
const Segment SGI374890[] = {{&S29462[13], 9}, {NULL}};
const Segment SGI374891[] = {{&S29462[92], 3}, {NULL}};
const Segment SGI374892[] = {{&S29462[30], 13}, {NULL}};
const Segment SGI374893[] = {{&S29463[57], 6}, {NULL}};
const Segment SGI374894[] = {{&S29463[8], 3}, {NULL}};
const Segment SGI374895[] = {{&S29463[13], 9}, {NULL}};
const Segment SGI374896[] = {{&S29463[92], 3}, {NULL}};
const Segment SGI374897[] = {{&S29463[30], 13}, {NULL}};
const Segment SGI374898[] = {{&S29464[57], 6}, {NULL}};
const Segment SGI374899[] = {{&S29464[8], 3}, {NULL}};
const Segment SGI374900[] = {{&S29464[13], 9}, {NULL}};
const Segment SGI374901[] = {{&S29464[92], 3}, {NULL}};
const Segment SGI374902[] = {{&S29464[30], 13}, {NULL}};
const Segment SGI374903[] = {{&S29465[57], 6}, {NULL}};
const Segment SGI374904[] = {{&S29465[8], 3}, {NULL}};
const Segment SGI374905[] = {{&S29465[13], 9}, {NULL}};
const Segment SGI374906[] = {{&S29465[92], 3}, {NULL}};
const Segment SGI374907[] = {{&S29465[30], 13}, {NULL}};
const Segment SGI374908[] = {{&S29466[57], 6}, {NULL}};
const Segment SGI374909[] = {{&S29466[8], 3}, {NULL}};
const Segment SGI374910[] = {{&S29466[13], 9}, {NULL}};
const Segment SGI374911[] = {{&S29466[94], 3}, {NULL}};
const Segment SGI374912[] = {{&S29466[30], 13}, {NULL}};
const Segment SGI374913[] = {{&S29467[57], 6}, {NULL}};
const Segment SGI374914[] = {{&S29467[8], 3}, {NULL}};
const Segment SGI374915[] = {{&S29467[13], 9}, {NULL}};
const Segment SGI374916[] = {{&S29467[92], 3}, {NULL}};
const Segment SGI374917[] = {{&S29467[30], 13}, {NULL}};
const Segment SGI374918[] = {{&S29468[57], 5}, {NULL}};
const Segment SGI374919[] = {{&S29468[8], 3}, {NULL}};
const Segment SGI374920[] = {{&S29468[13], 9}, {NULL}};
const Segment SGI374921[] = {{&S29468[91], 3}, {NULL}};
const Segment SGI374922[] = {{&S29468[30], 13}, {NULL}};
const Segment SGI374923[] = {{&S29469[57], 6}, {NULL}};
const Segment SGI374924[] = {{&S29469[8], 3}, {NULL}};
const Segment SGI374925[] = {{&S29469[13], 9}, {NULL}};
const Segment SGI374926[] = {{&S29469[92], 3}, {NULL}};
const Segment SGI374927[] = {{&S29469[30], 13}, {NULL}};
const Segment SGI374928[] = {{&S29470[57], 6}, {NULL}};
const Segment SGI374929[] = {{&S29470[8], 3}, {NULL}};
const Segment SGI374930[] = {{&S29470[13], 9}, {NULL}};
const Segment SGI374931[] = {{&S29470[92], 3}, {NULL}};
const Segment SGI374932[] = {{&S29470[30], 13}, {NULL}};
const Segment SGI374933[] = {{&S29471[57], 6}, {NULL}};
const Segment SGI374934[] = {{&S29471[8], 3}, {NULL}};
const Segment SGI374935[] = {{&S29471[13], 9}, {NULL}};
const Segment SGI374936[] = {{&S29471[92], 3}, {NULL}};
const Segment SGI374937[] = {{&S29471[30], 13}, {NULL}};
const Segment SGI374938[] = {{&S29472[57], 6}, {NULL}};
const Segment SGI374939[] = {{&S29472[8], 3}, {NULL}};
const Segment SGI374940[] = {{&S29472[13], 9}, {NULL}};
const Segment SGI374941[] = {{&S29472[92], 3}, {NULL}};
const Segment SGI374942[] = {{&S29472[30], 13}, {NULL}};
const Segment SGI374943[] = {{&S29473[57], 6}, {NULL}};
const Segment SGI374944[] = {{&S29473[8], 3}, {NULL}};
const Segment SGI374945[] = {{&S29473[13], 9}, {NULL}};
const Segment SGI374946[] = {{&S29473[92], 3}, {NULL}};
const Segment SGI374947[] = {{&S29473[30], 13}, {NULL}};
const Segment SGI374948[] = {{&S29474[57], 6}, {NULL}};
const Segment SGI374949[] = {{&S29474[8], 3}, {NULL}};
const Segment SGI374950[] = {{&S29474[13], 9}, {NULL}};
const Segment SGI374951[] = {{&S29474[92], 3}, {NULL}};
const Segment SGI374952[] = {{&S29474[30], 13}, {NULL}};
const Segment SGI374953[] = {{&S29475[57], 6}, {NULL}};
const Segment SGI374954[] = {{&S29475[8], 3}, {NULL}};
const Segment SGI374955[] = {{&S29475[13], 9}, {NULL}};
const Segment SGI374956[] = {{&S29475[92], 3}, {NULL}};
const Segment SGI374957[] = {{&S29475[30], 13}, {NULL}};
const Segment SGI374958[] = {{&S31699[13], 21}, {NULL}};
const Segment SGI374959[] = {{&S31699[48], 6}, {NULL}};
const Segment SGI374960[] = {{&S31700[13], 19}, {NULL}};
const Segment SGI374961[] = {{&S31700[46], 6}, {NULL}};
const Segment SGI374962[] = {{&S32226[13], 27}, {NULL}};
const Segment SGI374963[] = {{&S32226[54], 6}, {NULL}};
const Segment SGI374964[] = {{&S32226[88], 5}, {NULL}};
const Segment SGI374965[] = {{&S32228[13], 25}, {NULL}};
const Segment SGI374966[] = {{&S32228[52], 5}, {NULL}};
const Segment SGI374967[] = {{&S30616[8], 3}, {NULL}};
const Segment SGI374968[] = {{&S30616[13], 9}, {NULL}};
const Segment SGI374969[] = {{&S32223[47], 5}, {NULL}};
const Segment SGI374970[] = {{&S32223[8], 3}, {NULL}};
const Segment SGI374971[] = {{&S32223[13], 3}, {NULL}};
const Segment SGI374972[] = {{&S32223[73], 6}, {NULL}};
const Segment SGI374973[] = {{&S32224[79], 5}, {NULL}};
const Segment SGI374974[] = {{&S32224[8], 3}, {NULL}};
const Segment SGI374975[] = {{&S32224[13], 3}, {NULL}};
const Segment SGI374976[] = {{&S32224[86], 6}, {NULL}};
const Segment SGI374977[] = {{&S32225[81], 5}, {NULL}};
const Segment SGI374978[] = {{&S32225[8], 3}, {NULL}};
const Segment SGI374979[] = {{&S32225[13], 3}, {NULL}};
const Segment SGI374980[] = {{&S32225[88], 6}, {NULL}};
const Segment SGI374981[] = {{&S32227[54], 5}, {NULL}};
const Segment SGI374982[] = {{&S32227[8], 3}, {NULL}};
const Segment SGI374983[] = {{&S32227[13], 3}, {NULL}};
const Segment SGI374984[] = {{&S32227[80], 6}, {NULL}};
const Segment SGI374985[] = {{&S32229[50], 5}, {NULL}};
const Segment SGI374986[] = {{&S32229[8], 3}, {NULL}};
const Segment SGI374987[] = {{&S32229[13], 3}, {NULL}};
const Segment SGI374988[] = {{&S32229[76], 6}, {NULL}};
const Segment SGI374989[] = {{&S32239[120], 7}, {NULL}};
const Segment SGI374990[] = {{&S32239[8], 3}, {NULL}};
const Segment SGI374991[] = {{&S32239[13], 9}, {NULL}};
const Segment SGI374992[] = {{&S32239[33], 15}, {NULL}};
const Segment SGI374993[] = {{&S32239[49], 6}, {NULL}};
const Segment SGI374994[] = {{&S32239[33], 13}, {NULL}};
const Segment SGI374995[] = {{&S32240[118], 7}, {NULL}};
const Segment SGI374996[] = {{&S32240[8], 3}, {NULL}};
const Segment SGI374997[] = {{&S32240[13], 9}, {NULL}};
const Segment SGI374998[] = {{&S32240[33], 15}, {NULL}};
const Segment SGI374999[] = {{&S32240[49], 6}, {NULL}};
const Segment SGI375000[] = {{&S32240[33], 13}, {NULL}};
const Segment SGI375001[] = {{&S32241[106], 6}, {NULL}};
const Segment SGI375002[] = {{&S32241[8], 3}, {NULL}};
const Segment SGI375003[] = {{&S32241[13], 9}, {NULL}};
const Segment SGI375004[] = {{&S32241[33], 15}, {NULL}};
const Segment SGI375005[] = {{&S32241[49], 6}, {NULL}};
const Segment SGI375006[] = {{&S32241[33], 13}, {NULL}};
const Segment SGI375007[] = {{&S32242[122], 7}, {NULL}};
const Segment SGI375008[] = {{&S32242[8], 3}, {NULL}};
const Segment SGI375009[] = {{&S32242[13], 9}, {NULL}};
const Segment SGI375010[] = {{&S32242[33], 15}, {NULL}};
const Segment SGI375011[] = {{&S32242[49], 6}, {NULL}};
const Segment SGI375012[] = {{&S32242[33], 13}, {NULL}};
const Segment SGI375013[] = {{&S32244[105], 7}, {NULL}};
const Segment SGI375014[] = {{&S32244[8], 3}, {NULL}};
const Segment SGI375015[] = {{&S32244[13], 9}, {NULL}};
const Segment SGI375016[] = {{&S32244[33], 15}, {NULL}};
const Segment SGI375017[] = {{&S32244[49], 6}, {NULL}};
const Segment SGI375018[] = {{&S32244[33], 13}, {NULL}};
const Segment SGI375019[] = {{&S32251[82], 6}, {NULL}};
const Segment SGI375020[] = {{&S32251[8], 3}, {NULL}};
const Segment SGI375021[] = {{&S32251[13], 9}, {NULL}};
const Segment SGI375022[] = {{&S32251[33], 13}, {NULL}};
const Segment SGI375023[] = {{&S32252[105], 6}, {NULL}};
const Segment SGI375024[] = {{&S32252[8], 3}, {NULL}};
const Segment SGI375025[] = {{&S32252[13], 9}, {NULL}};
const Segment SGI375026[] = {{&S32252[33], 15}, {NULL}};
const Segment SGI375027[] = {{&S32252[49], 6}, {NULL}};
const Segment SGI375028[] = {{&S32252[33], 13}, {NULL}};
const Segment SGI375029[] = {{&S32254[95], 5}, {NULL}};
const Segment SGI375030[] = {{&S32254[8], 3}, {NULL}};
const Segment SGI375031[] = {{&S32254[13], 9}, {NULL}};
const Segment SGI375032[] = {{&S32254[30], 13}, {NULL}};
const Segment SGI375033[] = {{&S32255[95], 5}, {NULL}};
const Segment SGI375034[] = {{&S32255[8], 3}, {NULL}};
const Segment SGI375035[] = {{&S32255[13], 9}, {NULL}};
const Segment SGI375036[] = {{&S32255[30], 13}, {NULL}};
const Segment SGI375037[] = {{&S32256[95], 5}, {NULL}};
const Segment SGI375038[] = {{&S32256[8], 3}, {NULL}};
const Segment SGI375039[] = {{&S32256[13], 9}, {NULL}};
const Segment SGI375040[] = {{&S32256[30], 13}, {NULL}};
const Segment SGI375041[] = {{&S32257[8], 3}, {NULL}};
const Segment SGI375042[] = {{&S32257[13], 9}, {NULL}};
const Segment SGI375043[] = {{&S32257[30], 13}, {NULL}};
const Segment SGI375044[] = {{&S32260[122], 7}, {NULL}};
const Segment SGI375045[] = {{&S32260[8], 3}, {NULL}};
const Segment SGI375046[] = {{&S32260[13], 9}, {NULL}};
const Segment SGI375047[] = {{&S32260[33], 15}, {NULL}};
const Segment SGI375048[] = {{&S32260[49], 6}, {NULL}};
const Segment SGI375049[] = {{&S32260[33], 13}, {NULL}};
const Segment SGI375050[] = {{&S32261[122], 7}, {NULL}};
const Segment SGI375051[] = {{&S32261[8], 3}, {NULL}};
const Segment SGI375052[] = {{&S32261[13], 9}, {NULL}};
const Segment SGI375053[] = {{&S32261[33], 15}, {NULL}};
const Segment SGI375054[] = {{&S32261[49], 6}, {NULL}};
const Segment SGI375055[] = {{&S32261[33], 13}, {NULL}};
const Segment SGI375056[] = {{&S32264[122], 7}, {NULL}};
const Segment SGI375057[] = {{&S32264[8], 3}, {NULL}};
const Segment SGI375058[] = {{&S32264[13], 9}, {NULL}};
const Segment SGI375059[] = {{&S32264[33], 15}, {NULL}};
const Segment SGI375060[] = {{&S32264[49], 6}, {NULL}};
const Segment SGI375061[] = {{&S32264[33], 13}, {NULL}};
const Segment SGI375062[] = {{&S32266[8], 3}, {NULL}};
const Segment SGI375063[] = {{&S32266[13], 9}, {NULL}};
const Segment SGI375064[] = {{&S32266[31], 15}, {NULL}};
const Segment SGI375065[] = {{&S32266[47], 6}, {NULL}};
const Segment SGI375066[] = {{&S32266[31], 13}, {NULL}};
const Segment SGI375067[] = {{&S32270[107], 6}, {NULL}};
const Segment SGI375068[] = {{&S32270[8], 3}, {NULL}};
const Segment SGI375069[] = {{&S32270[13], 9}, {NULL}};
const Segment SGI375070[] = {{&S32270[28], 13}, {NULL}};
const Segment SGI375071[] = {{&S32273[122], 6}, {NULL}};
const Segment SGI375072[] = {{&S32273[8], 3}, {NULL}};
const Segment SGI375073[] = {{&S32273[13], 9}, {NULL}};
const Segment SGI375074[] = {{&S32273[33], 15}, {NULL}};
const Segment SGI375075[] = {{&S32273[49], 6}, {NULL}};
const Segment SGI375076[] = {{&S32273[33], 13}, {NULL}};
const Segment SGI375077[] = {{&S32276[59], 6}, {NULL}};
const Segment SGI375078[] = {{&S32276[8], 3}, {NULL}};
const Segment SGI375079[] = {{&S32276[13], 9}, {NULL}};
const Segment SGI375080[] = {{&S32276[30], 13}, {NULL}};
const Segment SGI375081[] = {{&S32280[105], 5}, {NULL}};
const Segment SGI375082[] = {{&S32280[8], 3}, {NULL}};
const Segment SGI375083[] = {{&S32280[13], 9}, {NULL}};
const Segment SGI375084[] = {{&S32280[33], 15}, {NULL}};
const Segment SGI375085[] = {{&S32280[49], 6}, {NULL}};
const Segment SGI375086[] = {{&S32280[33], 13}, {NULL}};
const Segment SGI375087[] = {{&S32281[105], 5}, {NULL}};
const Segment SGI375088[] = {{&S32281[8], 3}, {NULL}};
const Segment SGI375089[] = {{&S32281[13], 9}, {NULL}};
const Segment SGI375090[] = {{&S32281[33], 15}, {NULL}};
const Segment SGI375091[] = {{&S32281[49], 6}, {NULL}};
const Segment SGI375092[] = {{&S32281[33], 13}, {NULL}};
const Segment SGI375093[] = {{&S32357[73], 6}, {NULL}};
const Segment SGI375094[] = {{&S32357[13], 11}, {NULL}};
const Segment SGI375095[] = {{&S32357[38], 14}, {NULL}};
const Segment SGI375096[] = {{&S32441[118], 5}, {NULL}};
const Segment SGI375097[] = {{&S32441[13], 3}, {&S32441[71], 12}, {NULL}};
const Segment SGI375098[] = {{&S33653[41], 6}, {NULL}};
const Segment SGI375099[] = {{&S33653[8], 3}, {NULL}};
const Segment SGI375100[] = {{&S33653[13], 14}, {NULL}};
const Segment SGI375101[] = {{&S33653[79], 3}, {NULL}};
const Segment SGI375102[] = {{&S33654[48], 6}, {NULL}};
const Segment SGI375103[] = {{&S33654[8], 3}, {NULL}};
const Segment SGI375104[] = {{&S33654[13], 14}, {NULL}};
const Segment SGI375105[] = {{&S33654[86], 3}, {NULL}};
const Segment SGI375106[] = {{&S33655[48], 6}, {NULL}};
const Segment SGI375107[] = {{&S33655[8], 3}, {NULL}};
const Segment SGI375108[] = {{&S33655[13], 14}, {NULL}};
const Segment SGI375109[] = {{&S33655[85], 3}, {NULL}};
const Segment SGI375110[] = {{&S33656[41], 6}, {NULL}};
const Segment SGI375111[] = {{&S33656[8], 3}, {NULL}};
const Segment SGI375112[] = {{&S33656[13], 14}, {NULL}};
const Segment SGI375113[] = {{&S33656[78], 3}, {NULL}};
const Segment SGI375114[] = {{&S33657[48], 6}, {NULL}};
const Segment SGI375115[] = {{&S33657[8], 3}, {NULL}};
const Segment SGI375116[] = {{&S33657[13], 14}, {NULL}};
const Segment SGI375117[] = {{&S33657[83], 3}, {NULL}};
const Segment SGI375118[] = {{&S33658[48], 6}, {NULL}};
const Segment SGI375119[] = {{&S33658[8], 3}, {NULL}};
const Segment SGI375120[] = {{&S33658[13], 14}, {NULL}};
const Segment SGI375121[] = {{&S33658[85], 3}, {NULL}};
const Segment SGI375122[] = {{&S33659[48], 6}, {NULL}};
const Segment SGI375123[] = {{&S33659[8], 3}, {NULL}};
const Segment SGI375124[] = {{&S33659[13], 14}, {NULL}};
const Segment SGI375125[] = {{&S33659[85], 3}, {NULL}};
const Segment SGI375126[] = {{&S33660[41], 6}, {NULL}};
const Segment SGI375127[] = {{&S33660[8], 3}, {NULL}};
const Segment SGI375128[] = {{&S33660[13], 14}, {NULL}};
const Segment SGI375129[] = {{&S33660[79], 3}, {NULL}};
const Segment SGI375130[] = {{&S33661[41], 6}, {NULL}};
const Segment SGI375131[] = {{&S33661[8], 3}, {NULL}};
const Segment SGI375132[] = {{&S33661[13], 14}, {NULL}};
const Segment SGI375133[] = {{&S33661[78], 3}, {NULL}};
const Segment SGI375134[] = {{&S33662[41], 6}, {NULL}};
const Segment SGI375135[] = {{&S33662[8], 3}, {NULL}};
const Segment SGI375136[] = {{&S33662[13], 14}, {NULL}};
const Segment SGI375137[] = {{&S33662[78], 3}, {NULL}};
const Segment SGI375138[] = {{&S33663[48], 6}, {NULL}};
const Segment SGI375139[] = {{&S33663[8], 3}, {NULL}};
const Segment SGI375140[] = {{&S33663[13], 14}, {NULL}};
const Segment SGI375141[] = {{&S33663[86], 3}, {NULL}};
const Segment SGI375142[] = {{&S33664[41], 6}, {NULL}};
const Segment SGI375143[] = {{&S33664[8], 3}, {NULL}};
const Segment SGI375144[] = {{&S33664[13], 14}, {NULL}};
const Segment SGI375145[] = {{&S33664[76], 3}, {NULL}};
const Segment SGI375146[] = {{&S33665[91], 6}, {NULL}};
const Segment SGI375147[] = {{&S33665[8], 3}, {NULL}};
const Segment SGI375148[] = {{&S33665[13], 14}, {NULL}};
const Segment SGI375149[] = {{&S33665[78], 3}, {NULL}};
const Segment SGI375150[] = {{&S33666[98], 6}, {NULL}};
const Segment SGI375151[] = {{&S33666[8], 3}, {NULL}};
const Segment SGI375152[] = {{&S33666[13], 14}, {NULL}};
const Segment SGI375153[] = {{&S33666[85], 3}, {NULL}};
const Segment SGI375154[] = {{&S33868[65], 6}, {NULL}};
const Segment SGI375155[] = {{&S33868[8], 3}, {NULL}};
const Segment SGI375156[] = {{&S33868[13], 9}, {NULL}};
const Segment SGI375157[] = {{&S33868[31], 16}, {NULL}};
const Segment SGI375158[] = {{&S33868[100], 3}, {NULL}};
const Segment SGI375159[] = {{&S33868[31], 13}, {NULL}};
const Segment SGI375160[] = {{&S33869[65], 6}, {NULL}};
const Segment SGI375161[] = {{&S33869[8], 3}, {NULL}};
const Segment SGI375162[] = {{&S33869[13], 9}, {NULL}};
const Segment SGI375163[] = {{&S33869[31], 16}, {NULL}};
const Segment SGI375164[] = {{&S33869[100], 3}, {NULL}};
const Segment SGI375165[] = {{&S33869[31], 13}, {NULL}};
const Segment SGI375166[] = {{&S33870[113], 6}, {NULL}};
const Segment SGI375167[] = {{&S33870[8], 3}, {NULL}};
const Segment SGI375168[] = {{&S33870[13], 9}, {NULL}};
const Segment SGI375169[] = {{&S33870[31], 16}, {NULL}};
const Segment SGI375170[] = {{&S33870[100], 3}, {NULL}};
const Segment SGI375171[] = {{&S33870[31], 13}, {NULL}};
const Segment SGI375172[] = {{&S33871[115], 6}, {NULL}};
const Segment SGI375173[] = {{&S33871[8], 3}, {NULL}};
const Segment SGI375174[] = {{&S33871[13], 9}, {NULL}};
const Segment SGI375175[] = {{&S33871[33], 15}, {NULL}};
const Segment SGI375176[] = {{&S33871[104], 3}, {NULL}};
const Segment SGI375177[] = {{&S33871[49], 6}, {NULL}};
const Segment SGI375178[] = {{&S33871[33], 13}, {NULL}};
const Segment SGI375179[] = {{&S37167[13], 39}, {NULL}};
const Segment SGI375180[] = {{&S37167[66], 5}, {NULL}};
const Segment SGI375181[] = {{&S37167[101], 7}, {NULL}};
const Segment SGI375182[] = {{&S37578[13], 21}, {NULL}};
const Segment SGI375183[] = {{&S37578[48], 6}, {NULL}};
const Segment SGI375184[] = {{&S37579[13], 19}, {NULL}};
const Segment SGI375185[] = {{&S37579[46], 6}, {NULL}};
const Segment SGI375186[] = {{&S35792[60], 6}, {NULL}};
const Segment SGI375187[] = {{&S35792[8], 3}, {NULL}};
const Segment SGI375188[] = {{&S35792[13], 7}, {NULL}};
const Segment SGI375189[] = {{&S35792[25], 14}, {NULL}};
const Segment SGI375190[] = {{&S35792[97], 3}, {NULL}};
const Segment SGI375191[] = {{&S35793[60], 6}, {NULL}};
const Segment SGI375192[] = {{&S35793[8], 3}, {NULL}};
const Segment SGI375193[] = {{&S35793[13], 7}, {NULL}};
const Segment SGI375194[] = {{&S35793[25], 14}, {NULL}};
const Segment SGI375195[] = {{&S35793[95], 3}, {NULL}};
const Segment SGI375196[] = {{&S35794[60], 6}, {NULL}};
const Segment SGI375197[] = {{&S35794[8], 3}, {NULL}};
const Segment SGI375198[] = {{&S35794[13], 7}, {NULL}};
const Segment SGI375199[] = {{&S35794[25], 14}, {NULL}};
const Segment SGI375200[] = {{&S35794[95], 3}, {NULL}};
const Segment SGI375201[] = {{&S35795[60], 6}, {NULL}};
const Segment SGI375202[] = {{&S35795[8], 3}, {NULL}};
const Segment SGI375203[] = {{&S35795[13], 7}, {NULL}};
const Segment SGI375204[] = {{&S35795[25], 14}, {NULL}};
const Segment SGI375205[] = {{&S35795[97], 3}, {NULL}};
const Segment SGI375206[] = {{&S35796[60], 5}, {NULL}};
const Segment SGI375207[] = {{&S35796[8], 3}, {NULL}};
const Segment SGI375208[] = {{&S35796[13], 7}, {NULL}};
const Segment SGI375209[] = {{&S35796[25], 14}, {NULL}};
const Segment SGI375210[] = {{&S35796[94], 3}, {NULL}};
const Segment SGI375211[] = {{&S35797[92], 6}, {NULL}};
const Segment SGI375212[] = {{&S35797[8], 3}, {NULL}};
const Segment SGI375213[] = {{&S35797[13], 14}, {NULL}};
const Segment SGI375214[] = {{&S35797[79], 3}, {NULL}};
const Segment SGI375215[] = {{&S35798[106], 6}, {NULL}};
const Segment SGI375216[] = {{&S35798[8], 3}, {NULL}};
const Segment SGI375217[] = {{&S35798[13], 7}, {NULL}};
const Segment SGI375218[] = {{&S35798[25], 14}, {NULL}};
const Segment SGI375219[] = {{&S35798[93], 3}, {NULL}};
const Segment SGI375220[] = {{&S35799[106], 6}, {NULL}};
const Segment SGI375221[] = {{&S35799[8], 3}, {NULL}};
const Segment SGI375222[] = {{&S35799[13], 7}, {NULL}};
const Segment SGI375223[] = {{&S35799[25], 14}, {NULL}};
const Segment SGI375224[] = {{&S35799[95], 3}, {NULL}};
const Segment SGI375225[] = {{&S35800[109], 6}, {NULL}};
const Segment SGI375226[] = {{&S35800[8], 3}, {NULL}};
const Segment SGI375227[] = {{&S35800[13], 7}, {NULL}};
const Segment SGI375228[] = {{&S35800[25], 14}, {NULL}};
const Segment SGI375229[] = {{&S35800[95], 3}, {NULL}};
const Segment SGI375230[] = {{&S35801[98], 6}, {NULL}};
const Segment SGI375231[] = {{&S35801[8], 3}, {NULL}};
const Segment SGI375232[] = {{&S35801[13], 14}, {NULL}};
const Segment SGI375233[] = {{&S35801[85], 3}, {NULL}};
const Segment SGI375234[] = {{&S35802[98], 6}, {NULL}};
const Segment SGI375235[] = {{&S35802[8], 3}, {NULL}};
const Segment SGI375236[] = {{&S35802[13], 14}, {NULL}};
const Segment SGI375237[] = {{&S35802[85], 3}, {NULL}};
const Segment SGI375238[] = {{&S35803[91], 6}, {NULL}};
const Segment SGI375239[] = {{&S35803[8], 3}, {NULL}};
const Segment SGI375240[] = {{&S35803[13], 14}, {NULL}};
const Segment SGI375241[] = {{&S35803[78], 3}, {NULL}};
const Segment SGI375242[] = {{&S36498[57], 5}, {NULL}};
const Segment SGI375243[] = {{&S36498[8], 3}, {NULL}};
const Segment SGI375244[] = {{&S36498[13], 9}, {NULL}};
const Segment SGI375245[] = {{&S36498[91], 3}, {NULL}};
const Segment SGI375246[] = {{&S36498[30], 13}, {NULL}};
const Segment SGI375247[] = {{&S36499[57], 6}, {NULL}};
const Segment SGI375248[] = {{&S36499[8], 3}, {NULL}};
const Segment SGI375249[] = {{&S36499[13], 9}, {NULL}};
const Segment SGI375250[] = {{&S36499[92], 3}, {NULL}};
const Segment SGI375251[] = {{&S36499[30], 13}, {NULL}};
const Segment SGI375252[] = {{&S36500[57], 6}, {NULL}};
const Segment SGI375253[] = {{&S36500[8], 3}, {NULL}};
const Segment SGI375254[] = {{&S36500[13], 9}, {NULL}};
const Segment SGI375255[] = {{&S36500[94], 3}, {NULL}};
const Segment SGI375256[] = {{&S36500[30], 13}, {NULL}};
const Segment SGI375257[] = {{&S36501[59], 6}, {NULL}};
const Segment SGI375258[] = {{&S36501[8], 3}, {NULL}};
const Segment SGI375259[] = {{&S36501[13], 9}, {NULL}};
const Segment SGI375260[] = {{&S36501[94], 3}, {NULL}};
const Segment SGI375261[] = {{&S36501[30], 13}, {NULL}};
const Segment SGI375262[] = {{&S36502[57], 6}, {NULL}};
const Segment SGI375263[] = {{&S36502[8], 3}, {NULL}};
const Segment SGI375264[] = {{&S36502[13], 9}, {NULL}};
const Segment SGI375265[] = {{&S36502[92], 3}, {NULL}};
const Segment SGI375266[] = {{&S36502[30], 13}, {NULL}};
const Segment SGI375267[] = {{&S36503[57], 6}, {NULL}};
const Segment SGI375268[] = {{&S36503[8], 3}, {NULL}};
const Segment SGI375269[] = {{&S36503[13], 9}, {NULL}};
const Segment SGI375270[] = {{&S36503[94], 3}, {NULL}};
const Segment SGI375271[] = {{&S36503[30], 13}, {NULL}};
const Segment SGI375272[] = {{&S36504[59], 5}, {NULL}};
const Segment SGI375273[] = {{&S36504[8], 3}, {NULL}};
const Segment SGI375274[] = {{&S36504[13], 9}, {NULL}};
const Segment SGI375275[] = {{&S36504[93], 3}, {NULL}};
const Segment SGI375276[] = {{&S36504[30], 13}, {NULL}};
const Segment SGI375277[] = {{&S36505[57], 6}, {NULL}};
const Segment SGI375278[] = {{&S36505[8], 3}, {NULL}};
const Segment SGI375279[] = {{&S36505[13], 9}, {NULL}};
const Segment SGI375280[] = {{&S36505[94], 3}, {NULL}};
const Segment SGI375281[] = {{&S36505[30], 13}, {NULL}};
const Segment SGI375282[] = {{&S36506[57], 6}, {NULL}};
const Segment SGI375283[] = {{&S36506[8], 3}, {NULL}};
const Segment SGI375284[] = {{&S36506[13], 9}, {NULL}};
const Segment SGI375285[] = {{&S36506[92], 3}, {NULL}};
const Segment SGI375286[] = {{&S36506[30], 13}, {NULL}};
const Segment SGI375287[] = {{&S36507[103], 6}, {NULL}};
const Segment SGI375288[] = {{&S36507[8], 3}, {NULL}};
const Segment SGI375289[] = {{&S36507[13], 9}, {NULL}};
const Segment SGI375290[] = {{&S36507[92], 3}, {NULL}};
const Segment SGI375291[] = {{&S36507[30], 13}, {NULL}};
const Segment SGI375292[] = {{&S36508[59], 6}, {NULL}};
const Segment SGI375293[] = {{&S36508[8], 3}, {NULL}};
const Segment SGI375294[] = {{&S36508[13], 9}, {NULL}};
const Segment SGI375295[] = {{&S36508[96], 3}, {NULL}};
const Segment SGI375296[] = {{&S36508[30], 13}, {NULL}};
const Segment SGI375297[] = {{&S36509[59], 6}, {NULL}};
const Segment SGI375298[] = {{&S36509[8], 3}, {NULL}};
const Segment SGI375299[] = {{&S36509[13], 9}, {NULL}};
const Segment SGI375300[] = {{&S36509[94], 3}, {NULL}};
const Segment SGI375301[] = {{&S36509[30], 13}, {NULL}};
const Segment SGI375302[] = {{&S36510[59], 6}, {NULL}};
const Segment SGI375303[] = {{&S36510[8], 3}, {NULL}};
const Segment SGI375304[] = {{&S36510[13], 9}, {NULL}};
const Segment SGI375305[] = {{&S36510[97], 3}, {NULL}};
const Segment SGI375306[] = {{&S36510[30], 13}, {NULL}};
const Segment SGI375307[] = {{&S36511[59], 6}, {NULL}};
const Segment SGI375308[] = {{&S36511[8], 3}, {NULL}};
const Segment SGI375309[] = {{&S36511[13], 9}, {NULL}};
const Segment SGI375310[] = {{&S36511[96], 3}, {NULL}};
const Segment SGI375311[] = {{&S36511[30], 13}, {NULL}};
const Segment SGI375312[] = {{&S36512[59], 6}, {NULL}};
const Segment SGI375313[] = {{&S36512[8], 3}, {NULL}};
const Segment SGI375314[] = {{&S36512[13], 9}, {NULL}};
const Segment SGI375315[] = {{&S36512[96], 3}, {NULL}};
const Segment SGI375316[] = {{&S36512[30], 13}, {NULL}};
const Segment SGI375317[] = {{&S36513[59], 6}, {NULL}};
const Segment SGI375318[] = {{&S36513[8], 3}, {NULL}};
const Segment SGI375319[] = {{&S36513[13], 9}, {NULL}};
const Segment SGI375320[] = {{&S36513[96], 3}, {NULL}};
const Segment SGI375321[] = {{&S36513[30], 13}, {NULL}};
const Segment SGI375322[] = {{&S36514[59], 6}, {NULL}};
const Segment SGI375323[] = {{&S36514[8], 3}, {NULL}};
const Segment SGI375324[] = {{&S36514[13], 9}, {NULL}};
const Segment SGI375325[] = {{&S36514[96], 3}, {NULL}};
const Segment SGI375326[] = {{&S36514[30], 13}, {NULL}};
const Segment SGI375327[] = {{&S36515[59], 6}, {NULL}};
const Segment SGI375328[] = {{&S36515[8], 3}, {NULL}};
const Segment SGI375329[] = {{&S36515[13], 9}, {NULL}};
const Segment SGI375330[] = {{&S36515[94], 3}, {NULL}};
const Segment SGI375331[] = {{&S36515[30], 13}, {NULL}};
const Segment SGI375332[] = {{&S36516[59], 6}, {NULL}};
const Segment SGI375333[] = {{&S36516[8], 3}, {NULL}};
const Segment SGI375334[] = {{&S36516[13], 9}, {NULL}};
const Segment SGI375335[] = {{&S36516[96], 3}, {NULL}};
const Segment SGI375336[] = {{&S36516[30], 13}, {NULL}};
const Segment SGI375337[] = {{&S36517[59], 6}, {NULL}};
const Segment SGI375338[] = {{&S36517[8], 3}, {NULL}};
const Segment SGI375339[] = {{&S36517[13], 9}, {NULL}};
const Segment SGI375340[] = {{&S36517[94], 3}, {NULL}};
const Segment SGI375341[] = {{&S36517[30], 13}, {NULL}};
const Segment SGI375342[] = {{&S36518[59], 6}, {NULL}};
const Segment SGI375343[] = {{&S36518[8], 3}, {NULL}};
const Segment SGI375344[] = {{&S36518[13], 9}, {NULL}};
const Segment SGI375345[] = {{&S36518[94], 3}, {NULL}};
const Segment SGI375346[] = {{&S36518[30], 13}, {NULL}};
const Segment SGI375347[] = {{&S36744[109], 6}, {NULL}};
const Segment SGI375348[] = {{&S36744[8], 3}, {NULL}};
const Segment SGI375349[] = {{&S36744[13], 9}, {NULL}};
const Segment SGI375350[] = {{&S36744[30], 13}, {NULL}};
const Segment SGI375351[] = {{&S36745[109], 6}, {NULL}};
const Segment SGI375352[] = {{&S36745[8], 3}, {NULL}};
const Segment SGI375353[] = {{&S36745[13], 9}, {NULL}};
const Segment SGI375354[] = {{&S36745[30], 13}, {NULL}};
const Segment SGI375355[] = {{&S36746[107], 6}, {NULL}};
const Segment SGI375356[] = {{&S36746[8], 3}, {NULL}};
const Segment SGI375357[] = {{&S36746[13], 9}, {NULL}};
const Segment SGI375358[] = {{&S36746[30], 13}, {NULL}};
const Segment SGI375359[] = {{&S36747[109], 6}, {NULL}};
const Segment SGI375360[] = {{&S36747[8], 3}, {NULL}};
const Segment SGI375361[] = {{&S36747[13], 9}, {NULL}};
const Segment SGI375362[] = {{&S36747[30], 13}, {NULL}};
const Segment SGI375363[] = {{&S36748[109], 6}, {NULL}};
const Segment SGI375364[] = {{&S36748[8], 3}, {NULL}};
const Segment SGI375365[] = {{&S36748[13], 9}, {NULL}};
const Segment SGI375366[] = {{&S36748[30], 13}, {NULL}};
const Segment SGI375367[] = {{&S36749[109], 6}, {NULL}};
const Segment SGI375368[] = {{&S36749[8], 3}, {NULL}};
const Segment SGI375369[] = {{&S36749[13], 9}, {NULL}};
const Segment SGI375370[] = {{&S36749[30], 13}, {NULL}};
const Segment SGI375371[] = {{&S36750[117], 6}, {NULL}};
const Segment SGI375372[] = {{&S36750[8], 3}, {NULL}};
const Segment SGI375373[] = {{&S36750[13], 9}, {NULL}};
const Segment SGI375374[] = {{&S36750[33], 13}, {NULL}};
const Segment SGI375375[] = {{&S36751[69], 6}, {NULL}};
const Segment SGI375376[] = {{&S36751[8], 3}, {NULL}};
const Segment SGI375377[] = {{&S36751[13], 9}, {NULL}};
const Segment SGI375378[] = {{&S36751[33], 15}, {NULL}};
const Segment SGI375379[] = {{&S36751[49], 6}, {NULL}};
const Segment SGI375380[] = {{&S36751[33], 13}, {NULL}};
const Segment SGI375381[] = {{&S36752[69], 6}, {NULL}};
const Segment SGI375382[] = {{&S36752[8], 3}, {NULL}};
const Segment SGI375383[] = {{&S36752[13], 9}, {NULL}};
const Segment SGI375384[] = {{&S36752[33], 15}, {NULL}};
const Segment SGI375385[] = {{&S36752[49], 6}, {NULL}};
const Segment SGI375386[] = {{&S36752[33], 13}, {NULL}};
const Segment SGI375387[] = {{&S36753[105], 6}, {NULL}};
const Segment SGI375388[] = {{&S36753[8], 3}, {NULL}};
const Segment SGI375389[] = {{&S36753[13], 9}, {NULL}};
const Segment SGI375390[] = {{&S36753[30], 13}, {NULL}};
const Segment SGI375391[] = {{&S36754[69], 6}, {NULL}};
const Segment SGI375392[] = {{&S36754[8], 3}, {NULL}};
const Segment SGI375393[] = {{&S36754[13], 9}, {NULL}};
const Segment SGI375394[] = {{&S36754[33], 15}, {NULL}};
const Segment SGI375395[] = {{&S36754[49], 6}, {NULL}};
const Segment SGI375396[] = {{&S36754[33], 13}, {NULL}};
const Segment SGI375397[] = {{&S36755[69], 6}, {NULL}};
const Segment SGI375398[] = {{&S36755[8], 3}, {NULL}};
const Segment SGI375399[] = {{&S36755[13], 9}, {NULL}};
const Segment SGI375400[] = {{&S36755[33], 15}, {NULL}};
const Segment SGI375401[] = {{&S36755[49], 6}, {NULL}};
const Segment SGI375402[] = {{&S36755[33], 13}, {NULL}};
const Segment SGI375403[] = {{&S37123[48], 5}, {NULL}};
const Segment SGI375404[] = {{&S37123[8], 3}, {NULL}};
const Segment SGI375405[] = {{&S37123[13], 14}, {NULL}};
const Segment SGI375406[] = {{&S37123[74], 7}, {NULL}};
const Segment SGI375407[] = {{&S37124[41], 5}, {NULL}};
const Segment SGI375408[] = {{&S37124[8], 3}, {NULL}};
const Segment SGI375409[] = {{&S37124[13], 14}, {NULL}};
const Segment SGI375410[] = {{&S37124[67], 7}, {NULL}};
const Segment SGI375411[] = {{&S38122[41], 5}, {NULL}};
const Segment SGI375412[] = {{&S38122[8], 3}, {NULL}};
const Segment SGI375413[] = {{&S38122[13], 14}, {NULL}};
const Segment SGI375414[] = {{&S38122[67], 7}, {NULL}};
const Segment SGI375415[] = {{&S40746[13], 21}, {NULL}};
const Segment SGI375416[] = {{&S40746[48], 6}, {NULL}};
const Segment SGI375417[] = {{&S40747[13], 19}, {NULL}};
const Segment SGI375418[] = {{&S40747[46], 6}, {NULL}};
const Segment SGI375419[] = {{&S39207[55], 6}, {NULL}};
const Segment SGI375420[] = {{&S39207[8], 3}, {NULL}};
const Segment SGI375421[] = {{&S39207[13], 7}, {NULL}};
const Segment SGI375422[] = {{&S39207[25], 14}, {NULL}};
const Segment SGI375423[] = {{&S39207[82], 7}, {NULL}};
const Segment SGI375424[] = {{&S39213[55], 5}, {NULL}};
const Segment SGI375425[] = {{&S39213[8], 3}, {NULL}};
const Segment SGI375426[] = {{&S39213[13], 7}, {NULL}};
const Segment SGI375427[] = {{&S39213[25], 14}, {NULL}};
const Segment SGI375428[] = {{&S39213[81], 7}, {NULL}};
const Segment SGI375429[] = {{&S39328[60], 5}, {NULL}};
const Segment SGI375430[] = {{&S39328[8], 3}, {NULL}};
const Segment SGI375431[] = {{&S39328[13], 7}, {NULL}};
const Segment SGI375432[] = {{&S39328[25], 14}, {NULL}};
const Segment SGI375433[] = {{&S39328[86], 16}, {NULL}};
const Segment SGI375434[] = {{&S41682[13], 21}, {NULL}};
const Segment SGI375435[] = {{&S41682[48], 8}, {NULL}};
const Segment SGI375436[] = {{&S41682[85], 3}, {NULL}};
const Segment SGI375437[] = {{&S41683[13], 23}, {NULL}};
const Segment SGI375438[] = {{&S41683[50], 8}, {NULL}};
const Segment SGI375439[] = {{&S41683[87], 3}, {NULL}};
const Segment SGI375440[] = {{&S41684[13], 21}, {NULL}};
const Segment SGI375441[] = {{&S41684[48], 5}, {NULL}};
const Segment SGI375442[] = {{&S41684[82], 3}, {NULL}};
const Segment SGI375443[] = {{&S41685[13], 21}, {NULL}};
const Segment SGI375444[] = {{&S41685[48], 5}, {NULL}};
const Segment SGI375445[] = {{&S41685[82], 3}, {NULL}};
const Segment SGI375446[] = {{&S41686[13], 21}, {NULL}};
const Segment SGI375447[] = {{&S41686[48], 5}, {NULL}};
const Segment SGI375448[] = {{&S41686[82], 3}, {NULL}};
const Segment SGI375449[] = {{&S41687[13], 21}, {NULL}};
const Segment SGI375450[] = {{&S41687[48], 5}, {NULL}};
const Segment SGI375451[] = {{&S41687[82], 3}, {NULL}};
const Segment SGI375452[] = {{&S41688[13], 21}, {NULL}};
const Segment SGI375453[] = {{&S41688[48], 5}, {NULL}};
const Segment SGI375454[] = {{&S41688[82], 3}, {NULL}};
const Segment SGI375455[] = {{&S41689[13], 23}, {NULL}};
const Segment SGI375456[] = {{&S41689[50], 5}, {NULL}};
const Segment SGI375457[] = {{&S41689[84], 3}, {NULL}};
const Segment SGI375458[] = {{&S41690[13], 23}, {NULL}};
const Segment SGI375459[] = {{&S41690[50], 5}, {NULL}};
const Segment SGI375460[] = {{&S41690[84], 3}, {NULL}};
const Segment SGI375461[] = {{&S41691[13], 23}, {NULL}};
const Segment SGI375462[] = {{&S41691[50], 5}, {NULL}};
const Segment SGI375463[] = {{&S41691[84], 3}, {NULL}};
const Segment SGI375464[] = {{&S41692[13], 21}, {NULL}};
const Segment SGI375465[] = {{&S41692[48], 5}, {NULL}};
const Segment SGI375466[] = {{&S41692[82], 3}, {NULL}};
const Segment SGI375467[] = {{&S41693[13], 21}, {NULL}};
const Segment SGI375468[] = {{&S41693[48], 5}, {NULL}};
const Segment SGI375469[] = {{&S41693[82], 3}, {NULL}};
const Segment SGI375470[] = {{&S41694[13], 21}, {NULL}};
const Segment SGI375471[] = {{&S41694[48], 5}, {NULL}};
const Segment SGI375472[] = {{&S41694[82], 3}, {NULL}};
const Segment SGI375473[] = {{&S41695[13], 21}, {NULL}};
const Segment SGI375474[] = {{&S41695[48], 5}, {NULL}};
const Segment SGI375475[] = {{&S41695[82], 3}, {NULL}};
const Segment SGI375476[] = {{&S41696[13], 21}, {NULL}};
const Segment SGI375477[] = {{&S41696[48], 5}, {NULL}};
const Segment SGI375478[] = {{&S41696[82], 3}, {NULL}};
const Segment SGI375479[] = {{&S41697[13], 23}, {NULL}};
const Segment SGI375480[] = {{&S41697[50], 8}, {NULL}};
const Segment SGI375481[] = {{&S41697[87], 3}, {NULL}};
const Segment SGI375482[] = {{&S41698[13], 21}, {NULL}};
const Segment SGI375483[] = {{&S41698[48], 5}, {NULL}};
const Segment SGI375484[] = {{&S41698[82], 3}, {NULL}};
const Segment SGI375485[] = {{&S41699[49], 5}, {NULL}};
const Segment SGI375486[] = {{&S41699[83], 3}, {NULL}};
const Segment SGI375487[] = {{&S39639[60], 5}, {NULL}};
const Segment SGI375488[] = {{&S39639[8], 3}, {NULL}};
const Segment SGI375489[] = {{&S39639[13], 7}, {NULL}};
const Segment SGI375490[] = {{&S39639[25], 14}, {NULL}};
const Segment SGI375491[] = {{&S39639[86], 7}, {NULL}};
const Segment SGI375492[] = {{&S39640[55], 5}, {NULL}};
const Segment SGI375493[] = {{&S39640[8], 3}, {NULL}};
const Segment SGI375494[] = {{&S39640[13], 7}, {NULL}};
const Segment SGI375495[] = {{&S39640[25], 14}, {NULL}};
const Segment SGI375496[] = {{&S39640[81], 7}, {NULL}};
const Segment SGI375497[] = {{&S39641[55], 6}, {NULL}};
const Segment SGI375498[] = {{&S39641[8], 3}, {NULL}};
const Segment SGI375499[] = {{&S39641[13], 7}, {NULL}};
const Segment SGI375500[] = {{&S39641[25], 14}, {NULL}};
const Segment SGI375501[] = {{&S39641[82], 7}, {NULL}};
const Segment SGI375502[] = {{&S39736[67], 6}, {NULL}};
const Segment SGI375503[] = {{&S39736[8], 3}, {NULL}};
const Segment SGI375504[] = {{&S39736[13], 9}, {NULL}};
const Segment SGI375505[] = {{&S39736[33], 16}, {NULL}};
const Segment SGI375506[] = {{&S39736[104], 3}, {NULL}};
const Segment SGI375507[] = {{&S39736[33], 13}, {NULL}};
const Segment SGI375508[] = {{&S39737[67], 6}, {NULL}};
const Segment SGI375509[] = {{&S39737[8], 3}, {NULL}};
const Segment SGI375510[] = {{&S39737[13], 9}, {NULL}};
const Segment SGI375511[] = {{&S39737[33], 16}, {NULL}};
const Segment SGI375512[] = {{&S39737[104], 3}, {NULL}};
const Segment SGI375513[] = {{&S39737[33], 13}, {NULL}};
const Segment SGI375514[] = {{&S39738[65], 6}, {NULL}};
const Segment SGI375515[] = {{&S39738[8], 3}, {NULL}};
const Segment SGI375516[] = {{&S39738[13], 9}, {NULL}};
const Segment SGI375517[] = {{&S39738[31], 16}, {NULL}};
const Segment SGI375518[] = {{&S39738[102], 3}, {NULL}};
const Segment SGI375519[] = {{&S39738[31], 13}, {NULL}};
const Segment SGI375520[] = {{&S39739[70], 6}, {NULL}};
const Segment SGI375521[] = {{&S39739[8], 3}, {NULL}};
const Segment SGI375522[] = {{&S39739[13], 9}, {NULL}};
const Segment SGI375523[] = {{&S39739[31], 18}, {NULL}};
const Segment SGI375524[] = {{&S39739[107], 3}, {NULL}};
const Segment SGI375525[] = {{&S42377[13], 11}, {NULL}};
const Segment SGI375526[] = {{&S42377[38], 6}, {NULL}};
const Segment SGI375527[] = {{&S42377[74], 10}, {NULL}};
const Segment SGI375528[] = {{&S42378[13], 11}, {NULL}};
const Segment SGI375529[] = {{&S42378[38], 6}, {NULL}};
const Segment SGI375530[] = {{&S42378[74], 10}, {NULL}};
const Segment SGI375531[] = {{&S42379[13], 11}, {NULL}};
const Segment SGI375532[] = {{&S42379[38], 6}, {NULL}};
const Segment SGI375533[] = {{&S42379[74], 11}, {NULL}};
const Segment SGI375534[] = {{&S42380[13], 11}, {NULL}};
const Segment SGI375535[] = {{&S42380[38], 6}, {NULL}};
const Segment SGI375536[] = {{&S42380[74], 10}, {NULL}};
const Segment SGI375537[] = {{&S42381[13], 11}, {NULL}};
const Segment SGI375538[] = {{&S42381[38], 6}, {NULL}};
const Segment SGI375539[] = {{&S42381[74], 10}, {NULL}};
const Segment SGI375540[] = {{&S42382[13], 11}, {NULL}};
const Segment SGI375541[] = {{&S42382[38], 6}, {NULL}};
const Segment SGI375542[] = {{&S42382[74], 10}, {NULL}};
const Segment SGI375543[] = {{&S42383[13], 11}, {NULL}};
const Segment SGI375544[] = {{&S42383[38], 6}, {NULL}};
const Segment SGI375545[] = {{&S42383[74], 9}, {NULL}};
const Segment SGI375546[] = {{&S42384[13], 11}, {NULL}};
const Segment SGI375547[] = {{&S42384[38], 6}, {NULL}};
const Segment SGI375548[] = {{&S42384[74], 10}, {NULL}};
const Segment SGI375549[] = {{&S42385[13], 11}, {NULL}};
const Segment SGI375550[] = {{&S42385[38], 6}, {NULL}};
const Segment SGI375551[] = {{&S42385[74], 10}, {NULL}};
const Segment SGI375552[] = {{&S42386[13], 11}, {NULL}};
const Segment SGI375553[] = {{&S42386[38], 6}, {NULL}};
const Segment SGI375554[] = {{&S42386[74], 10}, {NULL}};
const Segment SGI375555[] = {{&S42387[13], 11}, {NULL}};
const Segment SGI375556[] = {{&S42387[38], 6}, {NULL}};
const Segment SGI375557[] = {{&S42387[74], 10}, {NULL}};
const Segment SGI375558[] = {{&S42388[13], 11}, {NULL}};
const Segment SGI375559[] = {{&S42388[38], 6}, {NULL}};
const Segment SGI375560[] = {{&S42388[74], 10}, {NULL}};
const Segment SGI375561[] = {{&S42389[13], 11}, {NULL}};
const Segment SGI375562[] = {{&S42389[38], 6}, {NULL}};
const Segment SGI375563[] = {{&S42389[74], 10}, {NULL}};
const Segment SGI375564[] = {{&S42390[13], 11}, {NULL}};
const Segment SGI375565[] = {{&S42390[38], 6}, {NULL}};
const Segment SGI375566[] = {{&S42390[74], 10}, {NULL}};
const Segment SGI375567[] = {{&S42391[13], 11}, {NULL}};
const Segment SGI375568[] = {{&S42391[38], 6}, {NULL}};
const Segment SGI375569[] = {{&S42391[74], 10}, {NULL}};
const Segment SGI375570[] = {{&S42392[13], 11}, {NULL}};
const Segment SGI375571[] = {{&S42392[38], 6}, {NULL}};
const Segment SGI375572[] = {{&S42392[74], 10}, {NULL}};
const Segment SGI375573[] = {{&S42393[13], 11}, {NULL}};
const Segment SGI375574[] = {{&S42393[38], 6}, {NULL}};
const Segment SGI375575[] = {{&S42393[74], 10}, {NULL}};
const Segment SGI375576[] = {{&S42394[13], 11}, {NULL}};
const Segment SGI375577[] = {{&S42394[38], 6}, {NULL}};
const Segment SGI375578[] = {{&S42394[74], 10}, {NULL}};
const Segment SGI375579[] = {{&S42395[13], 11}, {NULL}};
const Segment SGI375580[] = {{&S42395[38], 6}, {NULL}};
const Segment SGI375581[] = {{&S42395[74], 11}, {NULL}};
const Segment SGI375582[] = {{&S42396[13], 11}, {NULL}};
const Segment SGI375583[] = {{&S42396[38], 6}, {NULL}};
const Segment SGI375584[] = {{&S42396[74], 11}, {NULL}};
const Segment SGI375585[] = {{&S42397[13], 11}, {NULL}};
const Segment SGI375586[] = {{&S42397[38], 6}, {NULL}};
const Segment SGI375587[] = {{&S42397[74], 11}, {NULL}};
const Segment SGI375588[] = {{&S42398[13], 11}, {NULL}};
const Segment SGI375589[] = {{&S42398[38], 6}, {NULL}};
const Segment SGI375590[] = {{&S42398[74], 11}, {NULL}};
const Segment SGI375591[] = {{&S42399[13], 11}, {NULL}};
const Segment SGI375592[] = {{&S42399[38], 6}, {NULL}};
const Segment SGI375593[] = {{&S42399[74], 11}, {NULL}};
const Segment SGI375594[] = {{&S42400[13], 11}, {NULL}};
const Segment SGI375595[] = {{&S42400[38], 6}, {NULL}};
const Segment SGI375596[] = {{&S42400[74], 11}, {NULL}};
const Segment SGI375597[] = {{&S42401[13], 11}, {NULL}};
const Segment SGI375598[] = {{&S42401[38], 6}, {NULL}};
const Segment SGI375599[] = {{&S42401[74], 11}, {NULL}};
const Segment SGI375600[] = {{&S42402[13], 11}, {NULL}};
const Segment SGI375601[] = {{&S42402[38], 6}, {NULL}};
const Segment SGI375602[] = {{&S42402[74], 11}, {NULL}};
const Segment SGI375603[] = {{&S42403[13], 11}, {NULL}};
const Segment SGI375604[] = {{&S42403[38], 6}, {NULL}};
const Segment SGI375605[] = {{&S42403[74], 11}, {NULL}};
const Segment SGI375606[] = {{&S42404[13], 11}, {NULL}};
const Segment SGI375607[] = {{&S42404[38], 6}, {NULL}};
const Segment SGI375608[] = {{&S42404[74], 11}, {NULL}};
const Segment SGI375609[] = {{&S42405[13], 11}, {NULL}};
const Segment SGI375610[] = {{&S42405[38], 6}, {NULL}};
const Segment SGI375611[] = {{&S42405[74], 9}, {NULL}};
const Segment SGI375612[] = {{&S42406[13], 11}, {NULL}};
const Segment SGI375613[] = {{&S42406[38], 6}, {NULL}};
const Segment SGI375614[] = {{&S42406[74], 11}, {NULL}};
const Segment SGI375615[] = {{&S42407[13], 11}, {NULL}};
const Segment SGI375616[] = {{&S42407[38], 6}, {NULL}};
const Segment SGI375617[] = {{&S42407[74], 11}, {NULL}};
const Segment SGI375618[] = {{&S42408[13], 11}, {NULL}};
const Segment SGI375619[] = {{&S42408[38], 6}, {NULL}};
const Segment SGI375620[] = {{&S42408[74], 11}, {NULL}};
const Segment SGI375621[] = {{&S42409[13], 11}, {NULL}};
const Segment SGI375622[] = {{&S42409[38], 6}, {NULL}};
const Segment SGI375623[] = {{&S42409[74], 11}, {NULL}};
const Segment SGI375624[] = {{&S42410[13], 11}, {NULL}};
const Segment SGI375625[] = {{&S42410[38], 6}, {NULL}};
const Segment SGI375626[] = {{&S42410[74], 11}, {NULL}};
const Segment SGI375627[] = {{&S42411[13], 11}, {NULL}};
const Segment SGI375628[] = {{&S42411[38], 6}, {NULL}};
const Segment SGI375629[] = {{&S42411[74], 11}, {NULL}};
const Segment SGI375630[] = {{&S42412[13], 11}, {NULL}};
const Segment SGI375631[] = {{&S42412[38], 6}, {NULL}};
const Segment SGI375632[] = {{&S42412[74], 10}, {NULL}};
const Segment SGI375633[] = {{&S42413[13], 11}, {NULL}};
const Segment SGI375634[] = {{&S42413[38], 6}, {NULL}};
const Segment SGI375635[] = {{&S42413[74], 11}, {NULL}};
const Segment SGI375636[] = {{&S42414[13], 11}, {NULL}};
const Segment SGI375637[] = {{&S42414[38], 6}, {NULL}};
const Segment SGI375638[] = {{&S42414[74], 11}, {NULL}};
const Segment SGI375639[] = {{&S42415[13], 11}, {NULL}};
const Segment SGI375640[] = {{&S42415[38], 6}, {NULL}};
const Segment SGI375641[] = {{&S42415[74], 11}, {NULL}};
const Segment SGI375642[] = {{&S42416[13], 11}, {NULL}};
const Segment SGI375643[] = {{&S42416[38], 6}, {NULL}};
const Segment SGI375644[] = {{&S42416[74], 11}, {NULL}};
const Segment SGI375645[] = {{&S42417[13], 11}, {NULL}};
const Segment SGI375646[] = {{&S42417[38], 6}, {NULL}};
const Segment SGI375647[] = {{&S42417[74], 11}, {NULL}};
const Segment SGI375648[] = {{&S42418[13], 11}, {NULL}};
const Segment SGI375649[] = {{&S42418[38], 6}, {NULL}};
const Segment SGI375650[] = {{&S42418[74], 11}, {NULL}};
const Segment SGI375651[] = {{&S42419[13], 11}, {NULL}};
const Segment SGI375652[] = {{&S42419[38], 6}, {NULL}};
const Segment SGI375653[] = {{&S42419[74], 11}, {NULL}};
const Segment SGI375654[] = {{&S42420[13], 11}, {NULL}};
const Segment SGI375655[] = {{&S42420[38], 6}, {NULL}};
const Segment SGI375656[] = {{&S42420[74], 11}, {NULL}};
const Segment SGI375657[] = {{&S42421[13], 11}, {NULL}};
const Segment SGI375658[] = {{&S42421[38], 6}, {NULL}};
const Segment SGI375659[] = {{&S42421[74], 11}, {NULL}};
const Segment SGI375660[] = {{&S42422[13], 11}, {NULL}};
const Segment SGI375661[] = {{&S42422[38], 6}, {NULL}};
const Segment SGI375662[] = {{&S42422[74], 11}, {NULL}};
const Segment SGI375663[] = {{&S42423[13], 11}, {NULL}};
const Segment SGI375664[] = {{&S42423[38], 6}, {NULL}};
const Segment SGI375665[] = {{&S42423[74], 10}, {NULL}};
const Segment SGI375666[] = {{&S42424[13], 11}, {NULL}};
const Segment SGI375667[] = {{&S42424[38], 6}, {NULL}};
const Segment SGI375668[] = {{&S42424[74], 11}, {NULL}};
const Segment SGI375669[] = {{&S42425[13], 11}, {NULL}};
const Segment SGI375670[] = {{&S42425[38], 6}, {NULL}};
const Segment SGI375671[] = {{&S42425[74], 10}, {NULL}};
const Segment SGI375672[] = {{&S40270[86], 6}, {NULL}};
const Segment SGI375673[] = {{&S40270[8], 3}, {NULL}};
const Segment SGI375674[] = {{&S40270[13], 9}, {NULL}};
const Segment SGI375675[] = {{&S40270[33], 15}, {NULL}};
const Segment SGI375676[] = {{&S40270[49], 16}, {NULL}};
const Segment SGI375677[] = {{&S40270[33], 13}, {NULL}};
const Segment SGI375678[] = {{&S40271[85], 6}, {NULL}};
const Segment SGI375679[] = {{&S40271[8], 3}, {NULL}};
const Segment SGI375680[] = {{&S40271[13], 9}, {NULL}};
const Segment SGI375681[] = {{&S40271[33], 15}, {NULL}};
const Segment SGI375682[] = {{&S40271[49], 15}, {NULL}};
const Segment SGI375683[] = {{&S40271[33], 13}, {NULL}};
const Segment SGI375684[] = {{&S40272[88], 6}, {NULL}};
const Segment SGI375685[] = {{&S40272[8], 3}, {NULL}};
const Segment SGI375686[] = {{&S40272[13], 9}, {NULL}};
const Segment SGI375687[] = {{&S40272[33], 15}, {NULL}};
const Segment SGI375688[] = {{&S40272[49], 18}, {NULL}};
const Segment SGI375689[] = {{&S40272[33], 13}, {NULL}};
const Segment SGI375690[] = {{&S40273[13], 9}, {NULL}};
const Segment SGI375691[] = {{&S40273[33], 15}, {NULL}};
const Segment SGI375692[] = {{&S40273[49], 100}, {NULL}};
const Segment SGI375693[] = {{&S40273[33], 13}, {NULL}};
const Segment SGI375694[] = {{&S40599[60], 5}, {NULL}};
const Segment SGI375695[] = {{&S40599[8], 3}, {NULL}};
const Segment SGI375696[] = {{&S40599[13], 7}, {NULL}};
const Segment SGI375697[] = {{&S40599[25], 14}, {NULL}};
const Segment SGI375698[] = {{&S40600[60], 5}, {NULL}};
const Segment SGI375699[] = {{&S40600[8], 3}, {NULL}};
const Segment SGI375700[] = {{&S40600[13], 7}, {NULL}};
const Segment SGI375701[] = {{&S40600[25], 14}, {NULL}};
const Segment SGI375702[] = {{&S40601[48], 5}, {NULL}};
const Segment SGI375703[] = {{&S40601[8], 3}, {NULL}};
const Segment SGI375704[] = {{&S40601[13], 14}, {NULL}};
const Segment SGI375705[] = {{&S40602[41], 5}, {NULL}};
const Segment SGI375706[] = {{&S40602[8], 3}, {NULL}};
const Segment SGI375707[] = {{&S40602[13], 14}, {NULL}};
const Segment SGI375708[] = {{&S40603[48], 6}, {NULL}};
const Segment SGI375709[] = {{&S40603[8], 3}, {NULL}};
const Segment SGI375710[] = {{&S40603[13], 14}, {NULL}};
const Segment SGI375711[] = {{&S40604[41], 6}, {NULL}};
const Segment SGI375712[] = {{&S40604[8], 3}, {NULL}};
const Segment SGI375713[] = {{&S40604[13], 14}, {NULL}};
const Segment SGI375714[] = {{&S40605[51], 6}, {NULL}};
const Segment SGI375715[] = {{&S40605[8], 3}, {NULL}};
const Segment SGI375716[] = {{&S40605[13], 1}, {NULL}};
const Segment SGI375717[] = {{&S40605[16], 14}, {NULL}};
const Segment SGI375718[] = {{&S40606[44], 6}, {NULL}};
const Segment SGI375719[] = {{&S40606[8], 3}, {NULL}};
const Segment SGI375720[] = {{&S40606[13], 1}, {NULL}};
const Segment SGI375721[] = {{&S40607[41], 6}, {NULL}};
const Segment SGI375722[] = {{&S40607[8], 3}, {NULL}};
const Segment SGI375723[] = {{&S40607[13], 14}, {NULL}};
const Segment SGI375724[] = {{&S40608[48], 6}, {NULL}};
const Segment SGI375725[] = {{&S40608[8], 3}, {NULL}};
const Segment SGI375726[] = {{&S40608[13], 14}, {NULL}};
const Segment SGI375727[] = {{&S40609[48], 6}, {NULL}};
const Segment SGI375728[] = {{&S40609[8], 3}, {NULL}};
const Segment SGI375729[] = {{&S40609[13], 14}, {NULL}};
const Segment SGI375730[] = {{&S40610[41], 6}, {NULL}};
const Segment SGI375731[] = {{&S40610[8], 3}, {NULL}};
const Segment SGI375732[] = {{&S40610[13], 14}, {NULL}};
const Segment SGI375733[] = {{&S40611[48], 6}, {NULL}};
const Segment SGI375734[] = {{&S40611[8], 3}, {NULL}};
const Segment SGI375735[] = {{&S40611[13], 14}, {NULL}};
const Segment SGI375736[] = {{&S40612[41], 6}, {NULL}};
const Segment SGI375737[] = {{&S40612[8], 3}, {NULL}};
const Segment SGI375738[] = {{&S40612[13], 14}, {NULL}};
const Segment SGI375739[] = {{&S40613[48], 6}, {NULL}};
const Segment SGI375740[] = {{&S40613[8], 3}, {NULL}};
const Segment SGI375741[] = {{&S40613[13], 14}, {NULL}};
const Segment SGI375742[] = {{&S40614[41], 6}, {NULL}};
const Segment SGI375743[] = {{&S40614[8], 3}, {NULL}};
const Segment SGI375744[] = {{&S40614[13], 14}, {NULL}};
const Segment SGI375745[] = {{&S40615[41], 6}, {NULL}};
const Segment SGI375746[] = {{&S40615[8], 3}, {NULL}};
const Segment SGI375747[] = {{&S40615[13], 14}, {NULL}};
const Segment SGI375748[] = {{&S40616[48], 6}, {NULL}};
const Segment SGI375749[] = {{&S40616[8], 3}, {NULL}};
const Segment SGI375750[] = {{&S40616[13], 14}, {NULL}};
const Segment SGI375751[] = {{&S40617[60], 5}, {NULL}};
const Segment SGI375752[] = {{&S40617[8], 3}, {NULL}};
const Segment SGI375753[] = {{&S40617[13], 7}, {NULL}};
const Segment SGI375754[] = {{&S40617[25], 14}, {NULL}};
const Segment SGI375755[] = {{&S40618[60], 6}, {NULL}};
const Segment SGI375756[] = {{&S40618[8], 3}, {NULL}};
const Segment SGI375757[] = {{&S40618[13], 7}, {NULL}};
const Segment SGI375758[] = {{&S40618[25], 14}, {NULL}};
const Segment SGI375759[] = {{&S40619[48], 6}, {NULL}};
const Segment SGI375760[] = {{&S40619[8], 3}, {NULL}};
const Segment SGI375761[] = {{&S40619[13], 14}, {NULL}};
const Segment SGI375762[] = {{&S40620[8], 3}, {NULL}};
const Segment SGI375763[] = {{&S40620[13], 14}, {NULL}};
const Segment SGI375764[] = {{&S40621[48], 6}, {NULL}};
const Segment SGI375765[] = {{&S40621[8], 3}, {NULL}};
const Segment SGI375766[] = {{&S40621[13], 14}, {NULL}};
const Segment SGI375767[] = {{&S40622[48], 6}, {NULL}};
const Segment SGI375768[] = {{&S40622[8], 3}, {NULL}};
const Segment SGI375769[] = {{&S40622[13], 14}, {NULL}};
const Segment SGI375770[] = {{&S40623[48], 6}, {NULL}};
const Segment SGI375771[] = {{&S40623[8], 3}, {NULL}};
const Segment SGI375772[] = {{&S40623[13], 14}, {NULL}};
const Segment SGI375773[] = {{&S40705[69], 6}, {NULL}};
const Segment SGI375774[] = {{&S40705[8], 3}, {NULL}};
const Segment SGI375775[] = {{&S40705[49], 6}, {NULL}};
const Segment SGI375776[] = {{&S40705[33], 13}, {NULL}};
const Segment SGI375777[] = {{&S41125[69], 6}, {NULL}};
const Segment SGI375778[] = {{&S41125[8], 3}, {NULL}};
const Segment SGI375779[] = {{&S41125[13], 9}, {NULL}};
const Segment SGI375780[] = {{&S41125[33], 15}, {NULL}};
const Segment SGI375781[] = {{&S41125[106], 3}, {NULL}};
const Segment SGI375782[] = {{&S41125[49], 6}, {NULL}};
const Segment SGI375783[] = {{&S41125[33], 13}, {NULL}};
const Segment SGI375784[] = {{&S41126[69], 6}, {NULL}};
const Segment SGI375785[] = {{&S41126[8], 3}, {NULL}};
const Segment SGI375786[] = {{&S41126[13], 9}, {NULL}};
const Segment SGI375787[] = {{&S41126[33], 15}, {NULL}};
const Segment SGI375788[] = {{&S41126[106], 3}, {NULL}};
const Segment SGI375789[] = {{&S41126[49], 6}, {NULL}};
const Segment SGI375790[] = {{&S41126[33], 13}, {NULL}};
const Segment SGI375791[] = {{&S41127[109], 6}, {NULL}};
const Segment SGI375792[] = {{&S41127[8], 3}, {NULL}};
const Segment SGI375793[] = {{&S41127[13], 9}, {NULL}};
const Segment SGI375794[] = {{&S41127[96], 3}, {NULL}};
const Segment SGI375795[] = {{&S41127[30], 13}, {NULL}};
const Segment SGI375796[] = {{&S41128[109], 5}, {NULL}};
const Segment SGI375797[] = {{&S41128[8], 3}, {NULL}};
const Segment SGI375798[] = {{&S41128[13], 9}, {NULL}};
const Segment SGI375799[] = {{&S41128[96], 3}, {NULL}};
const Segment SGI375800[] = {{&S41128[30], 13}, {NULL}};
const Segment SGI375801[] = {{&S41129[109], 6}, {NULL}};
const Segment SGI375802[] = {{&S41129[8], 3}, {NULL}};
const Segment SGI375803[] = {{&S41129[13], 9}, {NULL}};
const Segment SGI375804[] = {{&S41129[96], 3}, {NULL}};
const Segment SGI375805[] = {{&S41129[30], 13}, {NULL}};
const Segment SGI375806[] = {{&S41130[69], 6}, {NULL}};
const Segment SGI375807[] = {{&S41130[8], 3}, {NULL}};
const Segment SGI375808[] = {{&S41130[13], 9}, {NULL}};
const Segment SGI375809[] = {{&S41130[33], 15}, {NULL}};
const Segment SGI375810[] = {{&S41130[106], 3}, {NULL}};
const Segment SGI375811[] = {{&S41130[49], 6}, {NULL}};
const Segment SGI375812[] = {{&S41130[33], 13}, {NULL}};
const Segment SGI375813[] = {{&S41393[60], 6}, {NULL}};
const Segment SGI375814[] = {{&S41393[8], 3}, {NULL}};
const Segment SGI375815[] = {{&S41393[13], 7}, {NULL}};
const Segment SGI375816[] = {{&S41393[25], 14}, {NULL}};
const Segment SGI375817[] = {{&S41393[95], 3}, {NULL}};
const Segment SGI375818[] = {{&S41394[57], 6}, {NULL}};
const Segment SGI375819[] = {{&S41394[8], 3}, {NULL}};
const Segment SGI375820[] = {{&S41394[13], 7}, {NULL}};
const Segment SGI375821[] = {{&S41394[25], 14}, {NULL}};
const Segment SGI375822[] = {{&S41394[94], 3}, {NULL}};
const Segment SGI375823[] = {{&S41395[57], 6}, {NULL}};
const Segment SGI375824[] = {{&S41395[8], 3}, {NULL}};
const Segment SGI375825[] = {{&S41395[13], 7}, {NULL}};
const Segment SGI375826[] = {{&S41395[25], 14}, {NULL}};
const Segment SGI375827[] = {{&S41395[94], 3}, {NULL}};
const Segment SGI375828[] = {{&S41396[60], 6}, {NULL}};
const Segment SGI375829[] = {{&S41396[8], 3}, {NULL}};
const Segment SGI375830[] = {{&S41396[13], 7}, {NULL}};
const Segment SGI375831[] = {{&S41396[25], 14}, {NULL}};
const Segment SGI375832[] = {{&S41396[97], 3}, {NULL}};
const Segment SGI375833[] = {{&S41397[57], 6}, {NULL}};
const Segment SGI375834[] = {{&S41397[8], 3}, {NULL}};
const Segment SGI375835[] = {{&S41397[13], 7}, {NULL}};
const Segment SGI375836[] = {{&S41397[25], 14}, {NULL}};
const Segment SGI375837[] = {{&S41397[94], 3}, {NULL}};
const Segment SGI375838[] = {{&S41398[57], 6}, {NULL}};
const Segment SGI375839[] = {{&S41398[8], 3}, {NULL}};
const Segment SGI375840[] = {{&S41398[13], 7}, {NULL}};
const Segment SGI375841[] = {{&S41398[25], 14}, {NULL}};
const Segment SGI375842[] = {{&S41398[94], 3}, {NULL}};
const Segment SGI375843[] = {{&S41399[57], 6}, {NULL}};
const Segment SGI375844[] = {{&S41399[8], 3}, {NULL}};
const Segment SGI375845[] = {{&S41399[13], 7}, {NULL}};
const Segment SGI375846[] = {{&S41399[25], 14}, {NULL}};
const Segment SGI375847[] = {{&S41399[94], 3}, {NULL}};
const Segment SGI375848[] = {{&S41400[57], 6}, {NULL}};
const Segment SGI375849[] = {{&S41400[8], 3}, {NULL}};
const Segment SGI375850[] = {{&S41400[13], 7}, {NULL}};
const Segment SGI375851[] = {{&S41400[25], 14}, {NULL}};
const Segment SGI375852[] = {{&S41400[94], 3}, {NULL}};
const Segment SGI375853[] = {{&S41401[103], 5}, {NULL}};
const Segment SGI375854[] = {{&S41401[8], 3}, {NULL}};
const Segment SGI375855[] = {{&S41401[13], 9}, {NULL}};
const Segment SGI375856[] = {{&S41401[33], 16}, {NULL}};
const Segment SGI375857[] = {{&S41401[33], 13}, {NULL}};
const Segment SGI375858[] = {{&S41402[101], 5}, {NULL}};
const Segment SGI375859[] = {{&S41402[8], 3}, {NULL}};
const Segment SGI375860[] = {{&S41402[13], 9}, {NULL}};
const Segment SGI375861[] = {{&S41402[31], 16}, {NULL}};
const Segment SGI375862[] = {{&S41402[31], 13}, {NULL}};
const Segment SGI375863[] = {{&S41403[103], 5}, {NULL}};
const Segment SGI375864[] = {{&S41403[8], 3}, {NULL}};
const Segment SGI375865[] = {{&S41403[13], 9}, {NULL}};
const Segment SGI375866[] = {{&S41403[33], 16}, {NULL}};
const Segment SGI375867[] = {{&S41403[33], 13}, {NULL}};
const Segment SGI375868[] = {{&S41404[101], 5}, {NULL}};
const Segment SGI375869[] = {{&S41404[8], 3}, {NULL}};
const Segment SGI375870[] = {{&S41404[13], 9}, {NULL}};
const Segment SGI375871[] = {{&S41404[31], 16}, {NULL}};
const Segment SGI375872[] = {{&S41404[31], 13}, {NULL}};
const Segment SGI375873[] = {{&S41405[120], 7}, {NULL}};
const Segment SGI375874[] = {{&S41405[8], 3}, {NULL}};
const Segment SGI375875[] = {{&S41405[13], 9}, {NULL}};
const Segment SGI375876[] = {{&S41405[33], 16}, {NULL}};
const Segment SGI375877[] = {{&S41405[33], 13}, {NULL}};
const Segment SGI375878[] = {{&S41407[8], 3}, {NULL}};
const Segment SGI375879[] = {{&S41407[13], 9}, {NULL}};
const Segment SGI375880[] = {{&S41407[31], 16}, {NULL}};
const Segment SGI375881[] = {{&S41407[31], 13}, {NULL}};
const Segment SGI375882[] = {{&S41408[101], 6}, {NULL}};
const Segment SGI375883[] = {{&S41408[8], 3}, {NULL}};
const Segment SGI375884[] = {{&S41408[13], 9}, {NULL}};
const Segment SGI375885[] = {{&S41408[31], 16}, {NULL}};
const Segment SGI375886[] = {{&S41408[31], 13}, {NULL}};
const Segment SGI375887[] = {{&S41409[101], 5}, {NULL}};
const Segment SGI375888[] = {{&S41409[8], 3}, {NULL}};
const Segment SGI375889[] = {{&S41409[13], 9}, {NULL}};
const Segment SGI375890[] = {{&S41409[31], 16}, {NULL}};
const Segment SGI375891[] = {{&S41409[31], 13}, {NULL}};
const Segment SGI375892[] = {{&S41410[101], 5}, {NULL}};
const Segment SGI375893[] = {{&S41410[8], 3}, {NULL}};
const Segment SGI375894[] = {{&S41410[13], 9}, {NULL}};
const Segment SGI375895[] = {{&S41410[31], 16}, {NULL}};
const Segment SGI375896[] = {{&S41410[31], 13}, {NULL}};
const Segment SGI375897[] = {{&S41411[65], 6}, {NULL}};
const Segment SGI375898[] = {{&S41411[8], 3}, {NULL}};
const Segment SGI375899[] = {{&S41411[13], 9}, {NULL}};
const Segment SGI375900[] = {{&S41411[31], 16}, {NULL}};
const Segment SGI375901[] = {{&S41411[31], 13}, {NULL}};
const Segment SGI375902[] = {{&S41584[8], 3}, {NULL}};
const Segment SGI375903[] = {{&S41584[13], 9}, {NULL}};
const Segment SGI375904[] = {{&S41584[94], 3}, {NULL}};
const Segment SGI375905[] = {{&S41584[30], 13}, {NULL}};
const Segment SGI375906[] = {{&S44430[13], 21}, {NULL}};
const Segment SGI375907[] = {{&S44430[48], 6}, {NULL}};
const Segment SGI375908[] = {{&S44430[90], 13}, {NULL}};
const Segment SGI375909[] = {{&S44431[13], 19}, {NULL}};
const Segment SGI375910[] = {{&S44431[46], 6}, {NULL}};
const Segment SGI375911[] = {{&S44431[88], 13}, {NULL}};
const Segment SGI375912[] = {{&S41883[52], 5}, {NULL}};
const Segment SGI375913[] = {{&S41883[8], 3}, {NULL}};
const Segment SGI375914[] = {{&S41883[13], 8}, {NULL}};
const Segment SGI375915[] = {{&S41883[23], 3}, {NULL}};
const Segment SGI375916[] = {{&S41884[52], 6}, {NULL}};
const Segment SGI375917[] = {{&S41884[8], 3}, {NULL}};
const Segment SGI375918[] = {{&S41884[13], 8}, {NULL}};
const Segment SGI375919[] = {{&S41884[23], 3}, {NULL}};
const Segment SGI375920[] = {{&S44566[13], 39}, {NULL}};
const Segment SGI375921[] = {{&S44566[66], 5}, {NULL}};
const Segment SGI375922[] = {{&S44566[101], 7}, {NULL}};
const Segment SGI375923[] = {{&S44567[13], 39}, {NULL}};
const Segment SGI375924[] = {{&S44567[66], 5}, {NULL}};
const Segment SGI375925[] = {{&S44567[101], 7}, {NULL}};
const Segment SGI375926[] = {{&S44568[13], 39}, {NULL}};
const Segment SGI375927[] = {{&S44568[66], 5}, {NULL}};
const Segment SGI375928[] = {{&S44568[101], 7}, {NULL}};
const Segment SGI375929[] = {{&S45142[13], 21}, {NULL}};
const Segment SGI375930[] = {{&S45142[48], 6}, {NULL}};
const Segment SGI375931[] = {{&S45143[13], 19}, {NULL}};
const Segment SGI375932[] = {{&S45143[46], 6}, {NULL}};
const Segment SGI375933[] = {{&S43418[48], 6}, {NULL}};
const Segment SGI375934[] = {{&S43418[8], 3}, {NULL}};
const Segment SGI375935[] = {{&S43418[13], 14}, {NULL}};
const Segment SGI375936[] = {{&S43418[85], 3}, {NULL}};
const Segment SGI375937[] = {{&S43659[41], 5}, {NULL}};
const Segment SGI375938[] = {{&S43659[8], 3}, {NULL}};
const Segment SGI375939[] = {{&S43659[13], 14}, {NULL}};
const Segment SGI375940[] = {{&S43659[67], 7}, {NULL}};
const Segment SGI375941[] = {{&S43749[84], 6}, {NULL}};
const Segment SGI375942[] = {{&S43749[8], 3}, {NULL}};
const Segment SGI375943[] = {{&S43749[13], 9}, {NULL}};
const Segment SGI375944[] = {{&S43749[33], 15}, {NULL}};
const Segment SGI375945[] = {{&S43749[49], 14}, {NULL}};
const Segment SGI375946[] = {{&S43749[33], 13}, {NULL}};
const Segment SGI375947[] = {{&S43750[98], 6}, {NULL}};
const Segment SGI375948[] = {{&S43750[8], 3}, {NULL}};
const Segment SGI375949[] = {{&S43750[13], 9}, {NULL}};
const Segment SGI375950[] = {{&S43750[33], 15}, {NULL}};
const Segment SGI375951[] = {{&S43750[49], 28}, {NULL}};
const Segment SGI375952[] = {{&S43750[33], 13}, {NULL}};
const Segment SGI375953[] = {{&S43751[103], 6}, {NULL}};
const Segment SGI375954[] = {{&S43751[8], 3}, {NULL}};
const Segment SGI375955[] = {{&S43751[13], 9}, {NULL}};
const Segment SGI375956[] = {{&S43751[33], 15}, {NULL}};
const Segment SGI375957[] = {{&S43751[49], 33}, {NULL}};
const Segment SGI375958[] = {{&S43751[33], 13}, {NULL}};
const Segment SGI375959[] = {{&S43752[98], 6}, {NULL}};
const Segment SGI375960[] = {{&S43752[8], 3}, {NULL}};
const Segment SGI375961[] = {{&S43752[13], 9}, {NULL}};
const Segment SGI375962[] = {{&S43752[33], 15}, {NULL}};
const Segment SGI375963[] = {{&S43752[49], 28}, {NULL}};
const Segment SGI375964[] = {{&S43752[33], 13}, {NULL}};
const Segment SGI375965[] = {{&S43755[106], 5}, {NULL}};
const Segment SGI375966[] = {{&S43755[8], 3}, {NULL}};
const Segment SGI375967[] = {{&S43755[13], 9}, {NULL}};
const Segment SGI375968[] = {{&S43755[113], 10}, {NULL}};
const Segment SGI375969[] = {{&S43756[106], 6}, {NULL}};
const Segment SGI375970[] = {{&S43756[8], 3}, {NULL}};
const Segment SGI375971[] = {{&S43756[13], 9}, {NULL}};
const Segment SGI375972[] = {{&S43756[114], 10}, {NULL}};
const Segment SGI375973[] = {{&S43757[114], 5}, {NULL}};
const Segment SGI375974[] = {{&S43757[8], 3}, {NULL}};
const Segment SGI375975[] = {{&S43757[13], 9}, {NULL}};
const Segment SGI375976[] = {{&S43757[121], 10}, {NULL}};
const Segment SGI375977[] = {{&S43758[106], 6}, {NULL}};
const Segment SGI375978[] = {{&S43758[8], 3}, {NULL}};
const Segment SGI375979[] = {{&S43758[13], 9}, {NULL}};
const Segment SGI375980[] = {{&S43758[114], 10}, {NULL}};
const Segment SGI375981[] = {{&S43759[106], 6}, {NULL}};
const Segment SGI375982[] = {{&S43759[8], 3}, {NULL}};
const Segment SGI375983[] = {{&S43759[13], 9}, {NULL}};
const Segment SGI375984[] = {{&S43759[114], 10}, {NULL}};
const Segment SGI375985[] = {{&S43760[106], 5}, {NULL}};
const Segment SGI375986[] = {{&S43760[8], 3}, {NULL}};
const Segment SGI375987[] = {{&S43760[13], 9}, {NULL}};
const Segment SGI375988[] = {{&S43760[113], 10}, {NULL}};
const Segment SGI375989[] = {{&S43761[106], 5}, {NULL}};
const Segment SGI375990[] = {{&S43761[8], 3}, {NULL}};
const Segment SGI375991[] = {{&S43761[13], 9}, {NULL}};
const Segment SGI375992[] = {{&S43761[113], 10}, {NULL}};
const Segment SGI375993[] = {{&S43933[92], 5}, {NULL}};
const Segment SGI375994[] = {{&S43933[8], 3}, {NULL}};
const Segment SGI375995[] = {{&S43933[13], 7}, {NULL}};
const Segment SGI375996[] = {{&S43933[25], 14}, {NULL}};
const Segment SGI375997[] = {{&S43933[99], 6}, {NULL}};
const Segment SGI375998[] = {{&S46276[13], 19}, {NULL}};
const Segment SGI375999[] = {{&S46276[46], 8}, {NULL}};
const Segment SGI376000[] = {{&S46276[19], 3}, {NULL}};
const Segment SGI376001[] = {{&S46277[13], 19}, {NULL}};
const Segment SGI376002[] = {{&S46277[46], 8}, {NULL}};
const Segment SGI376003[] = {{&S46277[83], 3}, {NULL}};
const Segment SGI376004[] = {{&S46278[13], 19}, {NULL}};
const Segment SGI376005[] = {{&S46278[46], 8}, {NULL}};
const Segment SGI376006[] = {{&S46278[83], 3}, {NULL}};
const Segment SGI376007[] = {{&S46279[13], 21}, {NULL}};
const Segment SGI376008[] = {{&S46279[48], 8}, {NULL}};
const Segment SGI376009[] = {{&S46279[85], 3}, {NULL}};
const Segment SGI376010[] = {{&S46280[13], 21}, {NULL}};
const Segment SGI376011[] = {{&S46280[48], 8}, {NULL}};
const Segment SGI376012[] = {{&S46280[85], 3}, {NULL}};
const Segment SGI376013[] = {{&S46281[13], 23}, {NULL}};
const Segment SGI376014[] = {{&S46281[50], 5}, {NULL}};
const Segment SGI376015[] = {{&S46281[84], 3}, {NULL}};
const Segment SGI376016[] = {{&S46282[13], 21}, {NULL}};
const Segment SGI376017[] = {{&S46282[48], 5}, {NULL}};
const Segment SGI376018[] = {{&S46282[82], 3}, {NULL}};
const Segment SGI376019[] = {{&S46283[13], 21}, {NULL}};
const Segment SGI376020[] = {{&S46283[48], 5}, {NULL}};
const Segment SGI376021[] = {{&S46283[82], 3}, {NULL}};
const Segment SGI376022[] = {{&S46284[13], 21}, {NULL}};
const Segment SGI376023[] = {{&S46284[48], 5}, {NULL}};
const Segment SGI376024[] = {{&S46284[82], 3}, {NULL}};
const Segment SGI376025[] = {{&S46285[13], 21}, {NULL}};
const Segment SGI376026[] = {{&S46285[48], 5}, {NULL}};
const Segment SGI376027[] = {{&S46285[82], 3}, {NULL}};
const Segment SGI376028[] = {{&S46286[13], 21}, {NULL}};
const Segment SGI376029[] = {{&S46286[48], 5}, {NULL}};
const Segment SGI376030[] = {{&S46286[82], 3}, {NULL}};
const Segment SGI376031[] = {{&S46287[13], 21}, {NULL}};
const Segment SGI376032[] = {{&S46287[48], 5}, {NULL}};
const Segment SGI376033[] = {{&S46287[82], 3}, {NULL}};
const Segment SGI376034[] = {{&S46288[13], 21}, {NULL}};
const Segment SGI376035[] = {{&S46288[48], 5}, {NULL}};
const Segment SGI376036[] = {{&S46288[82], 3}, {NULL}};
const Segment SGI376037[] = {{&S46289[13], 21}, {NULL}};
const Segment SGI376038[] = {{&S46289[48], 5}, {NULL}};
const Segment SGI376039[] = {{&S46289[82], 3}, {NULL}};
const Segment SGI376040[] = {{&S46290[13], 21}, {NULL}};
const Segment SGI376041[] = {{&S46290[48], 5}, {NULL}};
const Segment SGI376042[] = {{&S46290[82], 3}, {NULL}};
const Segment SGI376043[] = {{&S46291[13], 21}, {NULL}};
const Segment SGI376044[] = {{&S46291[48], 5}, {NULL}};
const Segment SGI376045[] = {{&S46291[82], 3}, {NULL}};
const Segment SGI376046[] = {{&S46292[13], 21}, {NULL}};
const Segment SGI376047[] = {{&S46292[48], 5}, {NULL}};
const Segment SGI376048[] = {{&S46292[21], 3}, {NULL}};
const Segment SGI376049[] = {{&S46293[13], 23}, {NULL}};
const Segment SGI376050[] = {{&S46293[50], 5}, {NULL}};
const Segment SGI376051[] = {{&S46293[84], 3}, {NULL}};
const Segment SGI376052[] = {{&S46294[13], 23}, {NULL}};
const Segment SGI376053[] = {{&S46294[50], 8}, {NULL}};
const Segment SGI376054[] = {{&S46294[87], 3}, {NULL}};
const Segment SGI376055[] = {{&S46295[13], 21}, {NULL}};
const Segment SGI376056[] = {{&S46295[48], 5}, {NULL}};
const Segment SGI376057[] = {{&S46295[21], 3}, {NULL}};
const Segment SGI376058[] = {{&S46296[13], 21}, {NULL}};
const Segment SGI376059[] = {{&S46296[48], 5}, {NULL}};
const Segment SGI376060[] = {{&S46296[82], 3}, {NULL}};
const Segment SGI376061[] = {{&S46297[13], 21}, {NULL}};
const Segment SGI376062[] = {{&S46297[48], 5}, {NULL}};
const Segment SGI376063[] = {{&S46297[21], 3}, {NULL}};
const Segment SGI376064[] = {{&S46298[13], 21}, {NULL}};
const Segment SGI376065[] = {{&S46298[48], 5}, {NULL}};
const Segment SGI376066[] = {{&S46298[82], 3}, {NULL}};
const Segment SGI376067[] = {{&S46299[13], 21}, {NULL}};
const Segment SGI376068[] = {{&S46299[48], 5}, {NULL}};
const Segment SGI376069[] = {{&S46299[21], 3}, {NULL}};
const Segment SGI376070[] = {{&S46300[13], 21}, {NULL}};
const Segment SGI376071[] = {{&S46300[48], 5}, {NULL}};
const Segment SGI376072[] = {{&S46300[82], 3}, {NULL}};
const Segment SGI376073[] = {{&S46301[13], 19}, {NULL}};
const Segment SGI376074[] = {{&S46301[46], 8}, {NULL}};
const Segment SGI376075[] = {{&S46301[83], 3}, {NULL}};
const Segment SGI376076[] = {{&S46302[13], 21}, {NULL}};
const Segment SGI376077[] = {{&S46302[48], 5}, {NULL}};
const Segment SGI376078[] = {{&S46302[82], 3}, {NULL}};
const Segment SGI376079[] = {{&S46303[13], 21}, {NULL}};
const Segment SGI376080[] = {{&S46303[48], 5}, {NULL}};
const Segment SGI376081[] = {{&S46303[82], 3}, {NULL}};
const Segment SGI376082[] = {{&S46304[13], 21}, {NULL}};
const Segment SGI376083[] = {{&S46304[48], 5}, {NULL}};
const Segment SGI376084[] = {{&S46304[82], 3}, {NULL}};
const Segment SGI376085[] = {{&S44234[108], 6}, {NULL}};
const Segment SGI376086[] = {{&S44234[8], 3}, {NULL}};
const Segment SGI376087[] = {{&S44234[13], 7}, {NULL}};
const Segment SGI376088[] = {{&S44234[25], 14}, {NULL}};
const Segment SGI376089[] = {{&S44234[95], 3}, {NULL}};
const Segment SGI376090[] = {{&S44235[108], 6}, {NULL}};
const Segment SGI376091[] = {{&S44235[8], 3}, {NULL}};
const Segment SGI376092[] = {{&S44235[13], 7}, {NULL}};
const Segment SGI376093[] = {{&S44235[25], 14}, {NULL}};
const Segment SGI376094[] = {{&S44235[95], 3}, {NULL}};
const Segment SGI376095[] = {{&S44236[60], 6}, {NULL}};
const Segment SGI376096[] = {{&S44236[8], 3}, {NULL}};
const Segment SGI376097[] = {{&S44236[13], 7}, {NULL}};
const Segment SGI376098[] = {{&S44236[25], 14}, {NULL}};
const Segment SGI376099[] = {{&S44236[95], 3}, {NULL}};
const Segment SGI376100[] = {{&S44237[60], 6}, {NULL}};
const Segment SGI376101[] = {{&S44237[8], 3}, {NULL}};
const Segment SGI376102[] = {{&S44237[13], 7}, {NULL}};
const Segment SGI376103[] = {{&S44237[25], 14}, {NULL}};
const Segment SGI376104[] = {{&S44237[95], 3}, {NULL}};
const Segment SGI376105[] = {{&S44238[60], 5}, {NULL}};
const Segment SGI376106[] = {{&S44238[8], 3}, {NULL}};
const Segment SGI376107[] = {{&S44238[13], 7}, {NULL}};
const Segment SGI376108[] = {{&S44238[25], 14}, {NULL}};
const Segment SGI376109[] = {{&S44238[95], 3}, {NULL}};
const Segment SGI376110[] = {{&S44239[60], 6}, {NULL}};
const Segment SGI376111[] = {{&S44239[8], 3}, {NULL}};
const Segment SGI376112[] = {{&S44239[13], 7}, {NULL}};
const Segment SGI376113[] = {{&S44239[25], 14}, {NULL}};
const Segment SGI376114[] = {{&S44239[95], 3}, {NULL}};
const Segment SGI376115[] = {{&S44240[108], 6}, {NULL}};
const Segment SGI376116[] = {{&S44240[8], 3}, {NULL}};
const Segment SGI376117[] = {{&S44240[13], 7}, {NULL}};
const Segment SGI376118[] = {{&S44240[25], 14}, {NULL}};
const Segment SGI376119[] = {{&S44240[95], 3}, {NULL}};
const Segment SGI376120[] = {{&S44241[53], 6}, {NULL}};
const Segment SGI376121[] = {{&S44241[8], 3}, {NULL}};
const Segment SGI376122[] = {{&S44241[13], 7}, {NULL}};
const Segment SGI376123[] = {{&S44241[88], 3}, {NULL}};
const Segment SGI376124[] = {{&S44242[57], 5}, {NULL}};
const Segment SGI376125[] = {{&S44242[8], 3}, {NULL}};
const Segment SGI376126[] = {{&S44242[13], 7}, {NULL}};
const Segment SGI376127[] = {{&S44242[25], 14}, {NULL}};
const Segment SGI376128[] = {{&S44242[91], 3}, {NULL}};
const Segment SGI376129[] = {{&S44243[108], 6}, {NULL}};
const Segment SGI376130[] = {{&S44243[8], 3}, {NULL}};
const Segment SGI376131[] = {{&S44243[13], 7}, {NULL}};
const Segment SGI376132[] = {{&S44243[25], 14}, {NULL}};
const Segment SGI376133[] = {{&S44243[95], 3}, {NULL}};
const Segment SGI376134[] = {{&S44244[60], 6}, {NULL}};
const Segment SGI376135[] = {{&S44244[8], 3}, {NULL}};
const Segment SGI376136[] = {{&S44244[13], 7}, {NULL}};
const Segment SGI376137[] = {{&S44244[25], 14}, {NULL}};
const Segment SGI376138[] = {{&S44244[95], 3}, {NULL}};
const Segment SGI376139[] = {{&S44546[60], 5}, {NULL}};
const Segment SGI376140[] = {{&S44546[8], 3}, {NULL}};
const Segment SGI376141[] = {{&S44546[13], 7}, {NULL}};
const Segment SGI376142[] = {{&S44546[25], 14}, {NULL}};
const Segment SGI376143[] = {{&S44546[86], 16}, {NULL}};
const Segment SGI376144[] = {{&S44658[57], 5}, {NULL}};
const Segment SGI376145[] = {{&S44658[8], 3}, {NULL}};
const Segment SGI376146[] = {{&S44658[13], 9}, {NULL}};
const Segment SGI376147[] = {{&S44658[91], 3}, {NULL}};
const Segment SGI376148[] = {{&S44658[30], 13}, {NULL}};
const Segment SGI376149[] = {{&S44659[57], 6}, {NULL}};
const Segment SGI376150[] = {{&S44659[8], 3}, {NULL}};
const Segment SGI376151[] = {{&S44659[13], 9}, {NULL}};
const Segment SGI376152[] = {{&S44659[94], 3}, {NULL}};
const Segment SGI376153[] = {{&S44659[30], 13}, {NULL}};
const Segment SGI376154[] = {{&S44660[57], 5}, {NULL}};
const Segment SGI376155[] = {{&S44660[8], 3}, {NULL}};
const Segment SGI376156[] = {{&S44660[13], 9}, {NULL}};
const Segment SGI376157[] = {{&S44660[93], 3}, {NULL}};
const Segment SGI376158[] = {{&S44660[30], 13}, {NULL}};
const Segment SGI376159[] = {{&S44661[57], 5}, {NULL}};
const Segment SGI376160[] = {{&S44661[8], 3}, {NULL}};
const Segment SGI376161[] = {{&S44661[13], 9}, {NULL}};
const Segment SGI376162[] = {{&S44661[91], 3}, {NULL}};
const Segment SGI376163[] = {{&S44661[30], 13}, {NULL}};
const Segment SGI376164[] = {{&S44662[57], 5}, {NULL}};
const Segment SGI376165[] = {{&S44662[8], 3}, {NULL}};
const Segment SGI376166[] = {{&S44662[13], 9}, {NULL}};
const Segment SGI376167[] = {{&S44662[93], 3}, {NULL}};
const Segment SGI376168[] = {{&S44662[30], 13}, {NULL}};
const Segment SGI376169[] = {{&S44663[57], 5}, {NULL}};
const Segment SGI376170[] = {{&S44663[8], 3}, {NULL}};
const Segment SGI376171[] = {{&S44663[13], 9}, {NULL}};
const Segment SGI376172[] = {{&S44663[93], 3}, {NULL}};
const Segment SGI376173[] = {{&S44663[30], 13}, {NULL}};
const Segment SGI376174[] = {{&S44664[57], 5}, {NULL}};
const Segment SGI376175[] = {{&S44664[8], 3}, {NULL}};
const Segment SGI376176[] = {{&S44664[13], 9}, {NULL}};
const Segment SGI376177[] = {{&S44664[91], 3}, {NULL}};
const Segment SGI376178[] = {{&S44664[30], 13}, {NULL}};
const Segment SGI376179[] = {{&S44665[57], 6}, {NULL}};
const Segment SGI376180[] = {{&S44665[8], 3}, {NULL}};
const Segment SGI376181[] = {{&S44665[13], 9}, {NULL}};
const Segment SGI376182[] = {{&S44665[94], 3}, {NULL}};
const Segment SGI376183[] = {{&S44665[30], 13}, {NULL}};
const Segment SGI376184[] = {{&S44666[57], 6}, {NULL}};
const Segment SGI376185[] = {{&S44666[8], 3}, {NULL}};
const Segment SGI376186[] = {{&S44666[13], 9}, {NULL}};
const Segment SGI376187[] = {{&S44666[92], 3}, {NULL}};
const Segment SGI376188[] = {{&S44666[30], 13}, {NULL}};
const Segment SGI376189[] = {{&S44667[56], 6}, {NULL}};
const Segment SGI376190[] = {{&S44667[8], 3}, {NULL}};
const Segment SGI376191[] = {{&S44667[13], 9}, {NULL}};
const Segment SGI376192[] = {{&S44667[92], 3}, {NULL}};
const Segment SGI376193[] = {{&S44668[56], 6}, {NULL}};
const Segment SGI376194[] = {{&S44668[8], 3}, {NULL}};
const Segment SGI376195[] = {{&S44668[13], 9}, {NULL}};
const Segment SGI376196[] = {{&S44668[92], 3}, {NULL}};
const Segment SGI376197[] = {{&S44796[60], 5}, {NULL}};
const Segment SGI376198[] = {{&S44796[8], 3}, {NULL}};
const Segment SGI376199[] = {{&S44796[13], 7}, {NULL}};
const Segment SGI376200[] = {{&S44796[25], 14}, {NULL}};
const Segment SGI376201[] = {{&S44796[86], 16}, {NULL}};
const Segment SGI376202[] = {{&S45057[60], 5}, {NULL}};
const Segment SGI376203[] = {{&S45057[8], 3}, {NULL}};
const Segment SGI376204[] = {{&S45057[13], 7}, {NULL}};
const Segment SGI376205[] = {{&S45057[25], 14}, {NULL}};
const Segment SGI376206[] = {{&S45057[86], 16}, {NULL}};
const Segment SGI376207[] = {{&S45079[60], 5}, {NULL}};
const Segment SGI376208[] = {{&S45079[8], 3}, {NULL}};
const Segment SGI376209[] = {{&S45079[13], 7}, {NULL}};
const Segment SGI376210[] = {{&S45079[25], 14}, {NULL}};
const Segment SGI376211[] = {{&S45080[50], 5}, {NULL}};
const Segment SGI376212[] = {{&S45080[8], 3}, {NULL}};
const Segment SGI376213[] = {{&S45080[13], 16}, {NULL}};
const Segment SGI376214[] = {{&S45081[92], 5}, {NULL}};
const Segment SGI376215[] = {{&S45081[8], 3}, {NULL}};
const Segment SGI376216[] = {{&S45081[13], 7}, {NULL}};
const Segment SGI376217[] = {{&S45081[25], 14}, {NULL}};
const Segment SGI376218[] = {{&S45082[60], 5}, {NULL}};
const Segment SGI376219[] = {{&S45082[8], 3}, {NULL}};
const Segment SGI376220[] = {{&S45082[13], 7}, {NULL}};
const Segment SGI376221[] = {{&S45082[25], 14}, {NULL}};
const Segment SGI376222[] = {{&S45083[41], 5}, {NULL}};
const Segment SGI376223[] = {{&S45083[8], 3}, {NULL}};
const Segment SGI376224[] = {{&S45083[13], 14}, {NULL}};
const Segment SGI376225[] = {{&S45084[41], 5}, {NULL}};
const Segment SGI376226[] = {{&S45084[8], 3}, {NULL}};
const Segment SGI376227[] = {{&S45084[13], 14}, {NULL}};
const Segment SGI376228[] = {{&S45085[94], 5}, {NULL}};
const Segment SGI376229[] = {{&S45085[8], 3}, {NULL}};
const Segment SGI376230[] = {{&S45085[13], 7}, {NULL}};
const Segment SGI376231[] = {{&S45085[25], 14}, {NULL}};
const Segment SGI376232[] = {{&S45086[41], 6}, {NULL}};
const Segment SGI376233[] = {{&S45086[8], 3}, {NULL}};
const Segment SGI376234[] = {{&S45086[13], 14}, {NULL}};
const Segment SGI376235[] = {{&S45087[74], 5}, {NULL}};
const Segment SGI376236[] = {{&S45087[8], 3}, {NULL}};
const Segment SGI376237[] = {{&S45088[91], 5}, {NULL}};
const Segment SGI376238[] = {{&S45088[8], 3}, {NULL}};
const Segment SGI376239[] = {{&S45088[13], 7}, {NULL}};
const Segment SGI376240[] = {{&S45088[25], 14}, {NULL}};
const Segment SGI376241[] = {{&S45089[41], 6}, {NULL}};
const Segment SGI376242[] = {{&S45089[8], 3}, {NULL}};
const Segment SGI376243[] = {{&S45089[13], 14}, {NULL}};
const Segment SGI376244[] = {{&S45090[44], 6}, {NULL}};
const Segment SGI376245[] = {{&S45090[8], 3}, {NULL}};
const Segment SGI376246[] = {{&S45090[13], 1}, {NULL}};
const Segment SGI376247[] = {{&S45091[41], 6}, {NULL}};
const Segment SGI376248[] = {{&S45091[8], 3}, {NULL}};
const Segment SGI376249[] = {{&S45091[13], 14}, {NULL}};
const Segment SGI376250[] = {{&S45092[41], 6}, {NULL}};
const Segment SGI376251[] = {{&S45092[8], 3}, {NULL}};
const Segment SGI376252[] = {{&S45092[13], 14}, {NULL}};
const Segment SGI376253[] = {{&S45093[91], 5}, {NULL}};
const Segment SGI376254[] = {{&S45093[8], 3}, {NULL}};
const Segment SGI376255[] = {{&S45093[13], 7}, {NULL}};
const Segment SGI376256[] = {{&S45093[25], 14}, {NULL}};
const Segment SGI376257[] = {{&S45094[60], 5}, {NULL}};
const Segment SGI376258[] = {{&S45094[8], 3}, {NULL}};
const Segment SGI376259[] = {{&S45094[13], 7}, {NULL}};
const Segment SGI376260[] = {{&S45094[25], 14}, {NULL}};
const Segment SGI376261[] = {{&S45095[41], 6}, {NULL}};
const Segment SGI376262[] = {{&S45095[8], 3}, {NULL}};
const Segment SGI376263[] = {{&S45095[13], 14}, {NULL}};
const Segment SGI376264[] = {{&S45096[41], 6}, {NULL}};
const Segment SGI376265[] = {{&S45096[8], 3}, {NULL}};
const Segment SGI376266[] = {{&S45096[13], 14}, {NULL}};
const Segment SGI376267[] = {{&S45097[41], 6}, {NULL}};
const Segment SGI376268[] = {{&S45097[8], 3}, {NULL}};
const Segment SGI376269[] = {{&S45097[13], 14}, {NULL}};
const Segment SGI376270[] = {{&S45098[41], 6}, {NULL}};
const Segment SGI376271[] = {{&S45098[8], 3}, {NULL}};
const Segment SGI376272[] = {{&S45098[13], 14}, {NULL}};
const Segment SGI376273[] = {{&S45099[60], 6}, {NULL}};
const Segment SGI376274[] = {{&S45099[8], 3}, {NULL}};
const Segment SGI376275[] = {{&S45099[13], 7}, {NULL}};
const Segment SGI376276[] = {{&S45099[25], 14}, {NULL}};
const Segment SGI376277[] = {{&S45100[41], 6}, {NULL}};
const Segment SGI376278[] = {{&S45100[8], 3}, {NULL}};
const Segment SGI376279[] = {{&S45100[13], 14}, {NULL}};
const Segment SGI376280[] = {{&S45469[110], 6}, {NULL}};
const Segment SGI376281[] = {{&S45469[8], 3}, {NULL}};
const Segment SGI376282[] = {{&S45469[13], 7}, {NULL}};
const Segment SGI376283[] = {{&S45469[25], 14}, {NULL}};
const Segment SGI376284[] = {{&S45469[97], 3}, {NULL}};
const Segment SGI376285[] = {{&S45470[60], 6}, {NULL}};
const Segment SGI376286[] = {{&S45470[8], 3}, {NULL}};
const Segment SGI376287[] = {{&S45470[13], 7}, {NULL}};
const Segment SGI376288[] = {{&S45470[25], 14}, {NULL}};
const Segment SGI376289[] = {{&S45470[97], 3}, {NULL}};
const Segment SGI376290[] = {{&S45471[60], 6}, {NULL}};
const Segment SGI376291[] = {{&S45471[8], 3}, {NULL}};
const Segment SGI376292[] = {{&S45471[13], 7}, {NULL}};
const Segment SGI376293[] = {{&S45471[25], 14}, {NULL}};
const Segment SGI376294[] = {{&S45471[95], 3}, {NULL}};
const Segment SGI376295[] = {{&S45472[60], 5}, {NULL}};
const Segment SGI376296[] = {{&S45472[8], 3}, {NULL}};
const Segment SGI376297[] = {{&S45472[13], 7}, {NULL}};
const Segment SGI376298[] = {{&S45472[25], 14}, {NULL}};
const Segment SGI376299[] = {{&S45472[94], 3}, {NULL}};
const Segment SGI376300[] = {{&S45473[60], 6}, {NULL}};
const Segment SGI376301[] = {{&S45473[8], 3}, {NULL}};
const Segment SGI376302[] = {{&S45473[13], 7}, {NULL}};
const Segment SGI376303[] = {{&S45473[25], 14}, {NULL}};
const Segment SGI376304[] = {{&S45473[97], 3}, {NULL}};
const Segment SGI376305[] = {{&S45474[60], 6}, {NULL}};
const Segment SGI376306[] = {{&S45474[8], 3}, {NULL}};
const Segment SGI376307[] = {{&S45474[13], 7}, {NULL}};
const Segment SGI376308[] = {{&S45474[25], 14}, {NULL}};
const Segment SGI376309[] = {{&S45474[95], 3}, {NULL}};
const Segment SGI376310[] = {{&S45475[106], 6}, {NULL}};
const Segment SGI376311[] = {{&S45475[8], 3}, {NULL}};
const Segment SGI376312[] = {{&S45475[13], 7}, {NULL}};
const Segment SGI376313[] = {{&S45475[25], 14}, {NULL}};
const Segment SGI376314[] = {{&S45475[93], 3}, {NULL}};
const Segment SGI376315[] = {{&S47994[13], 42}, {NULL}};
const Segment SGI376316[] = {{&S47994[69], 5}, {NULL}};
const Segment SGI376317[] = {{&S47995[13], 36}, {NULL}};
const Segment SGI376318[] = {{&S47995[63], 5}, {NULL}};
const Segment SGI376319[] = {{&S47996[13], 42}, {NULL}};
const Segment SGI376320[] = {{&S47996[69], 5}, {NULL}};
const Segment SGI376321[] = {{&S47997[13], 42}, {NULL}};
const Segment SGI376322[] = {{&S47997[69], 5}, {NULL}};
const Segment SGI376323[] = {{&S48365[13], 23}, {NULL}};
const Segment SGI376324[] = {{&S48365[50], 3}, {NULL}};
const Segment SGI376325[] = {{&S48366[13], 20}, {NULL}};
const Segment SGI376326[] = {{&S48366[47], 3}, {NULL}};
const Segment SGI376327[] = {{&S48367[13], 20}, {NULL}};
const Segment SGI376328[] = {{&S48367[47], 3}, {NULL}};
const Segment SGI376329[] = {{&S48525[13], 15}, {NULL}};
const Segment SGI376330[] = {{&S48525[42], 5}, {NULL}};
const Segment SGI376331[] = {{&S48525[92], 3}, {NULL}};
const Segment SGI376332[] = {{&S46906[68], 6}, {NULL}};
const Segment SGI376333[] = {{&S46906[8], 3}, {NULL}};
const Segment SGI376334[] = {{&S46906[13], 9}, {NULL}};
const Segment SGI376335[] = {{&S46906[33], 15}, {NULL}};
const Segment SGI376336[] = {{&S46906[103], 3}, {NULL}};
const Segment SGI376337[] = {{&S46906[49], 5}, {NULL}};
const Segment SGI376338[] = {{&S46906[33], 13}, {NULL}};
const Segment SGI376339[] = {{&S46906[121], 11}, {NULL}};
const Segment SGI376340[] = {{&S46907[69], 6}, {NULL}};
const Segment SGI376341[] = {{&S46907[8], 3}, {NULL}};
const Segment SGI376342[] = {{&S46907[13], 9}, {NULL}};
const Segment SGI376343[] = {{&S46907[33], 15}, {NULL}};
const Segment SGI376344[] = {{&S46907[104], 3}, {NULL}};
const Segment SGI376345[] = {{&S46907[49], 6}, {NULL}};
const Segment SGI376346[] = {{&S46907[33], 13}, {NULL}};
const Segment SGI376347[] = {{&S46907[122], 11}, {NULL}};
const Segment SGI376348[] = {{&S46908[68], 6}, {NULL}};
const Segment SGI376349[] = {{&S46908[8], 3}, {NULL}};
const Segment SGI376350[] = {{&S46908[32], 15}, {NULL}};
const Segment SGI376351[] = {{&S46908[103], 3}, {NULL}};
const Segment SGI376352[] = {{&S46908[48], 6}, {NULL}};
const Segment SGI376353[] = {{&S46908[32], 13}, {NULL}};
const Segment SGI376354[] = {{&S46908[121], 11}, {NULL}};
const Segment SGI376355[] = {{&S46909[115], 6}, {NULL}};
const Segment SGI376356[] = {{&S46909[8], 3}, {NULL}};
const Segment SGI376357[] = {{&S46909[13], 9}, {NULL}};
const Segment SGI376358[] = {{&S46909[33], 15}, {NULL}};
const Segment SGI376359[] = {{&S46909[104], 3}, {NULL}};
const Segment SGI376360[] = {{&S46909[49], 6}, {NULL}};
const Segment SGI376361[] = {{&S46909[33], 13}, {NULL}};
const Segment SGI376362[] = {{&S46909[122], 11}, {NULL}};
const Segment SGI376363[] = {{&S49255[13], 21}, {NULL}};
const Segment SGI376364[] = {{&S49255[48], 6}, {NULL}};
const Segment SGI376365[] = {{&S49255[94], 13}, {NULL}};
const Segment SGI376366[] = {{&S49256[13], 19}, {NULL}};
const Segment SGI376367[] = {{&S49256[46], 6}, {NULL}};
const Segment SGI376368[] = {{&S49256[92], 13}, {NULL}};
const Segment SGI376369[] = {{&S47561[69], 5}, {NULL}};
const Segment SGI376370[] = {{&S47561[8], 3}, {NULL}};
const Segment SGI376371[] = {{&S47561[13], 9}, {NULL}};
const Segment SGI376372[] = {{&S47561[33], 15}, {NULL}};
const Segment SGI376373[] = {{&S47561[49], 6}, {NULL}};
const Segment SGI376374[] = {{&S47561[33], 13}, {NULL}};
const Segment SGI376375[] = {{&S47561[121], 7}, {NULL}};
const Segment SGI376376[] = {{&S47562[69], 5}, {NULL}};
const Segment SGI376377[] = {{&S47562[8], 3}, {NULL}};
const Segment SGI376378[] = {{&S47562[13], 9}, {NULL}};
const Segment SGI376379[] = {{&S47562[33], 15}, {NULL}};
const Segment SGI376380[] = {{&S47562[49], 6}, {NULL}};
const Segment SGI376381[] = {{&S47562[33], 13}, {NULL}};
const Segment SGI376382[] = {{&S47562[121], 7}, {NULL}};
const Segment SGI376383[] = {{&S47563[69], 5}, {NULL}};
const Segment SGI376384[] = {{&S47563[8], 3}, {NULL}};
const Segment SGI376385[] = {{&S47563[13], 9}, {NULL}};
const Segment SGI376386[] = {{&S47563[33], 15}, {NULL}};
const Segment SGI376387[] = {{&S47563[49], 6}, {NULL}};
const Segment SGI376388[] = {{&S47563[33], 13}, {NULL}};
const Segment SGI376389[] = {{&S47563[121], 7}, {NULL}};
const Segment SGI376390[] = {{&S47564[69], 5}, {NULL}};
const Segment SGI376391[] = {{&S47564[8], 3}, {NULL}};
const Segment SGI376392[] = {{&S47564[13], 9}, {NULL}};
const Segment SGI376393[] = {{&S47564[33], 15}, {NULL}};
const Segment SGI376394[] = {{&S47564[49], 6}, {NULL}};
const Segment SGI376395[] = {{&S47564[33], 13}, {NULL}};
const Segment SGI376396[] = {{&S47564[121], 7}, {NULL}};
const Segment SGI376397[] = {{&S47565[69], 5}, {NULL}};
const Segment SGI376398[] = {{&S47565[8], 3}, {NULL}};
const Segment SGI376399[] = {{&S47565[13], 9}, {NULL}};
const Segment SGI376400[] = {{&S47565[33], 15}, {NULL}};
const Segment SGI376401[] = {{&S47565[49], 6}, {NULL}};
const Segment SGI376402[] = {{&S47565[33], 13}, {NULL}};
const Segment SGI376403[] = {{&S47565[121], 7}, {NULL}};
const Segment SGI376404[] = {{&S47566[69], 5}, {NULL}};
const Segment SGI376405[] = {{&S47566[8], 3}, {NULL}};
const Segment SGI376406[] = {{&S47566[13], 9}, {NULL}};
const Segment SGI376407[] = {{&S47566[33], 15}, {NULL}};
const Segment SGI376408[] = {{&S47566[49], 6}, {NULL}};
const Segment SGI376409[] = {{&S47566[33], 13}, {NULL}};
const Segment SGI376410[] = {{&S47566[121], 7}, {NULL}};
const Segment SGI376411[] = {{&S47567[69], 5}, {NULL}};
const Segment SGI376412[] = {{&S47567[8], 3}, {NULL}};
const Segment SGI376413[] = {{&S47567[13], 9}, {NULL}};
const Segment SGI376414[] = {{&S47567[33], 15}, {NULL}};
const Segment SGI376415[] = {{&S47567[49], 6}, {NULL}};
const Segment SGI376416[] = {{&S47567[33], 13}, {NULL}};
const Segment SGI376417[] = {{&S47567[121], 7}, {NULL}};
const Segment SGI376418[] = {{&S47568[69], 5}, {NULL}};
const Segment SGI376419[] = {{&S47568[8], 3}, {NULL}};
const Segment SGI376420[] = {{&S47568[13], 9}, {NULL}};
const Segment SGI376421[] = {{&S47568[33], 15}, {NULL}};
const Segment SGI376422[] = {{&S47568[49], 6}, {NULL}};
const Segment SGI376423[] = {{&S47568[33], 13}, {NULL}};
const Segment SGI376424[] = {{&S47568[121], 7}, {NULL}};
const Segment SGI376425[] = {{&S47569[69], 5}, {NULL}};
const Segment SGI376426[] = {{&S47569[8], 3}, {NULL}};
const Segment SGI376427[] = {{&S47569[13], 9}, {NULL}};
const Segment SGI376428[] = {{&S47569[33], 15}, {NULL}};
const Segment SGI376429[] = {{&S47569[49], 6}, {NULL}};
const Segment SGI376430[] = {{&S47569[33], 13}, {NULL}};
const Segment SGI376431[] = {{&S47569[121], 7}, {NULL}};
const Segment SGI376432[] = {{&S47570[69], 5}, {NULL}};
const Segment SGI376433[] = {{&S47570[8], 3}, {NULL}};
const Segment SGI376434[] = {{&S47570[13], 9}, {NULL}};
const Segment SGI376435[] = {{&S47570[33], 15}, {NULL}};
const Segment SGI376436[] = {{&S47570[49], 6}, {NULL}};
const Segment SGI376437[] = {{&S47570[33], 13}, {NULL}};
const Segment SGI376438[] = {{&S47570[121], 7}, {NULL}};
const Segment SGI376439[] = {{&S47571[69], 5}, {NULL}};
const Segment SGI376440[] = {{&S47571[8], 3}, {NULL}};
const Segment SGI376441[] = {{&S47571[13], 9}, {NULL}};
const Segment SGI376442[] = {{&S47571[33], 15}, {NULL}};
const Segment SGI376443[] = {{&S47571[49], 6}, {NULL}};
const Segment SGI376444[] = {{&S47571[33], 13}, {NULL}};
const Segment SGI376445[] = {{&S47571[121], 7}, {NULL}};
const Segment SGI376446[] = {{&S47572[69], 5}, {NULL}};
const Segment SGI376447[] = {{&S47572[8], 3}, {NULL}};
const Segment SGI376448[] = {{&S47572[13], 9}, {NULL}};
const Segment SGI376449[] = {{&S47572[33], 15}, {NULL}};
const Segment SGI376450[] = {{&S47572[49], 6}, {NULL}};
const Segment SGI376451[] = {{&S47572[33], 13}, {NULL}};
const Segment SGI376452[] = {{&S47572[121], 7}, {NULL}};
const Segment SGI376453[] = {{&S47573[69], 5}, {NULL}};
const Segment SGI376454[] = {{&S47573[8], 3}, {NULL}};
const Segment SGI376455[] = {{&S47573[13], 9}, {NULL}};
const Segment SGI376456[] = {{&S47573[33], 15}, {NULL}};
const Segment SGI376457[] = {{&S47573[49], 6}, {NULL}};
const Segment SGI376458[] = {{&S47573[33], 13}, {NULL}};
const Segment SGI376459[] = {{&S47573[121], 7}, {NULL}};
const Segment SGI376460[] = {{&S47574[69], 5}, {NULL}};
const Segment SGI376461[] = {{&S47574[8], 3}, {NULL}};
const Segment SGI376462[] = {{&S47574[13], 9}, {NULL}};
const Segment SGI376463[] = {{&S47574[33], 15}, {NULL}};
const Segment SGI376464[] = {{&S47574[49], 6}, {NULL}};
const Segment SGI376465[] = {{&S47574[33], 13}, {NULL}};
const Segment SGI376466[] = {{&S47574[121], 7}, {NULL}};
const Segment SGI376467[] = {{&S47575[69], 5}, {NULL}};
const Segment SGI376468[] = {{&S47575[8], 3}, {NULL}};
const Segment SGI376469[] = {{&S47575[13], 9}, {NULL}};
const Segment SGI376470[] = {{&S47575[33], 15}, {NULL}};
const Segment SGI376471[] = {{&S47575[49], 6}, {NULL}};
const Segment SGI376472[] = {{&S47575[33], 13}, {NULL}};
const Segment SGI376473[] = {{&S47575[121], 7}, {NULL}};
const Segment SGI376474[] = {{&S47576[69], 5}, {NULL}};
const Segment SGI376475[] = {{&S47576[8], 3}, {NULL}};
const Segment SGI376476[] = {{&S47576[13], 9}, {NULL}};
const Segment SGI376477[] = {{&S47576[33], 15}, {NULL}};
const Segment SGI376478[] = {{&S47576[49], 6}, {NULL}};
const Segment SGI376479[] = {{&S47576[33], 13}, {NULL}};
const Segment SGI376480[] = {{&S47576[121], 7}, {NULL}};
const Segment SGI376481[] = {{&S47577[69], 5}, {NULL}};
const Segment SGI376482[] = {{&S47577[8], 3}, {NULL}};
const Segment SGI376483[] = {{&S47577[13], 9}, {NULL}};
const Segment SGI376484[] = {{&S47577[33], 15}, {NULL}};
const Segment SGI376485[] = {{&S47577[49], 6}, {NULL}};
const Segment SGI376486[] = {{&S47577[33], 13}, {NULL}};
const Segment SGI376487[] = {{&S47577[121], 7}, {NULL}};
const Segment SGI376488[] = {{&S47578[69], 5}, {NULL}};
const Segment SGI376489[] = {{&S47578[8], 3}, {NULL}};
const Segment SGI376490[] = {{&S47578[13], 9}, {NULL}};
const Segment SGI376491[] = {{&S47578[33], 15}, {NULL}};
const Segment SGI376492[] = {{&S47578[49], 6}, {NULL}};
const Segment SGI376493[] = {{&S47578[33], 13}, {NULL}};
const Segment SGI376494[] = {{&S47578[121], 7}, {NULL}};
const Segment SGI376495[] = {{&S47579[69], 5}, {NULL}};
const Segment SGI376496[] = {{&S47579[8], 3}, {NULL}};
const Segment SGI376497[] = {{&S47579[13], 9}, {NULL}};
const Segment SGI376498[] = {{&S47579[33], 15}, {NULL}};
const Segment SGI376499[] = {{&S47579[49], 6}, {NULL}};
const Segment SGI376500[] = {{&S47579[33], 13}, {NULL}};
const Segment SGI376501[] = {{&S47579[121], 7}, {NULL}};
const Segment SGI376502[] = {{&S47580[69], 5}, {NULL}};
const Segment SGI376503[] = {{&S47580[8], 3}, {NULL}};
const Segment SGI376504[] = {{&S47580[13], 9}, {NULL}};
const Segment SGI376505[] = {{&S47580[33], 15}, {NULL}};
const Segment SGI376506[] = {{&S47580[49], 6}, {NULL}};
const Segment SGI376507[] = {{&S47580[33], 13}, {NULL}};
const Segment SGI376508[] = {{&S47580[121], 7}, {NULL}};
const Segment SGI376509[] = {{&S47581[69], 5}, {NULL}};
const Segment SGI376510[] = {{&S47581[8], 3}, {NULL}};
const Segment SGI376511[] = {{&S47581[13], 9}, {NULL}};
const Segment SGI376512[] = {{&S47581[33], 15}, {NULL}};
const Segment SGI376513[] = {{&S47581[49], 6}, {NULL}};
const Segment SGI376514[] = {{&S47581[33], 13}, {NULL}};
const Segment SGI376515[] = {{&S47581[121], 7}, {NULL}};
const Segment SGI376516[] = {{&S47582[69], 5}, {NULL}};
const Segment SGI376517[] = {{&S47582[8], 3}, {NULL}};
const Segment SGI376518[] = {{&S47582[13], 9}, {NULL}};
const Segment SGI376519[] = {{&S47582[33], 15}, {NULL}};
const Segment SGI376520[] = {{&S47582[49], 6}, {NULL}};
const Segment SGI376521[] = {{&S47582[33], 13}, {NULL}};
const Segment SGI376522[] = {{&S47582[121], 7}, {NULL}};
const Segment SGI376523[] = {{&S47583[69], 5}, {NULL}};
const Segment SGI376524[] = {{&S47583[8], 3}, {NULL}};
const Segment SGI376525[] = {{&S47583[13], 9}, {NULL}};
const Segment SGI376526[] = {{&S47583[33], 15}, {NULL}};
const Segment SGI376527[] = {{&S47583[49], 6}, {NULL}};
const Segment SGI376528[] = {{&S47583[33], 13}, {NULL}};
const Segment SGI376529[] = {{&S47583[121], 7}, {NULL}};
const Segment SGI376530[] = {{&S47584[69], 5}, {NULL}};
const Segment SGI376531[] = {{&S47584[8], 3}, {NULL}};
const Segment SGI376532[] = {{&S47584[13], 9}, {NULL}};
const Segment SGI376533[] = {{&S47584[33], 15}, {NULL}};
const Segment SGI376534[] = {{&S47584[49], 6}, {NULL}};
const Segment SGI376535[] = {{&S47584[33], 13}, {NULL}};
const Segment SGI376536[] = {{&S47584[121], 7}, {NULL}};
const Segment SGI376537[] = {{&S47585[69], 5}, {NULL}};
const Segment SGI376538[] = {{&S47585[8], 3}, {NULL}};
const Segment SGI376539[] = {{&S47585[13], 9}, {NULL}};
const Segment SGI376540[] = {{&S47585[33], 15}, {NULL}};
const Segment SGI376541[] = {{&S47585[49], 6}, {NULL}};
const Segment SGI376542[] = {{&S47585[33], 13}, {NULL}};
const Segment SGI376543[] = {{&S47585[121], 7}, {NULL}};
const Segment SGI376544[] = {{&S47586[69], 5}, {NULL}};
const Segment SGI376545[] = {{&S47586[8], 3}, {NULL}};
const Segment SGI376546[] = {{&S47586[13], 9}, {NULL}};
const Segment SGI376547[] = {{&S47586[33], 15}, {NULL}};
const Segment SGI376548[] = {{&S47586[49], 6}, {NULL}};
const Segment SGI376549[] = {{&S47586[33], 13}, {NULL}};
const Segment SGI376550[] = {{&S47586[121], 7}, {NULL}};
const Segment SGI376551[] = {{&S47587[69], 5}, {NULL}};
const Segment SGI376552[] = {{&S47587[8], 3}, {NULL}};
const Segment SGI376553[] = {{&S47587[13], 9}, {NULL}};
const Segment SGI376554[] = {{&S47587[33], 15}, {NULL}};
const Segment SGI376555[] = {{&S47587[49], 6}, {NULL}};
const Segment SGI376556[] = {{&S47587[33], 13}, {NULL}};
const Segment SGI376557[] = {{&S47587[121], 7}, {NULL}};
const Segment SGI376558[] = {{&S47588[69], 5}, {NULL}};
const Segment SGI376559[] = {{&S47588[8], 3}, {NULL}};
const Segment SGI376560[] = {{&S47588[13], 9}, {NULL}};
const Segment SGI376561[] = {{&S47588[33], 15}, {NULL}};
const Segment SGI376562[] = {{&S47588[49], 6}, {NULL}};
const Segment SGI376563[] = {{&S47588[33], 13}, {NULL}};
const Segment SGI376564[] = {{&S47588[121], 7}, {NULL}};
const Segment SGI376565[] = {{&S47589[69], 5}, {NULL}};
const Segment SGI376566[] = {{&S47589[8], 3}, {NULL}};
const Segment SGI376567[] = {{&S47589[13], 9}, {NULL}};
const Segment SGI376568[] = {{&S47589[33], 15}, {NULL}};
const Segment SGI376569[] = {{&S47589[49], 6}, {NULL}};
const Segment SGI376570[] = {{&S47589[33], 13}, {NULL}};
const Segment SGI376571[] = {{&S47589[121], 7}, {NULL}};
const Segment SGI376572[] = {{&S47590[69], 5}, {NULL}};
const Segment SGI376573[] = {{&S47590[8], 3}, {NULL}};
const Segment SGI376574[] = {{&S47590[13], 9}, {NULL}};
const Segment SGI376575[] = {{&S47590[33], 15}, {NULL}};
const Segment SGI376576[] = {{&S47590[49], 6}, {NULL}};
const Segment SGI376577[] = {{&S47590[33], 13}, {NULL}};
const Segment SGI376578[] = {{&S47590[121], 7}, {NULL}};
const Segment SGI376579[] = {{&S47591[69], 5}, {NULL}};
const Segment SGI376580[] = {{&S47591[8], 3}, {NULL}};
const Segment SGI376581[] = {{&S47591[13], 9}, {NULL}};
const Segment SGI376582[] = {{&S47591[33], 15}, {NULL}};
const Segment SGI376583[] = {{&S47591[49], 6}, {NULL}};
const Segment SGI376584[] = {{&S47591[33], 13}, {NULL}};
const Segment SGI376585[] = {{&S47591[121], 7}, {NULL}};
const Segment SGI376586[] = {{&S47592[69], 5}, {NULL}};
const Segment SGI376587[] = {{&S47592[8], 3}, {NULL}};
const Segment SGI376588[] = {{&S47592[13], 9}, {NULL}};
const Segment SGI376589[] = {{&S47592[33], 15}, {NULL}};
const Segment SGI376590[] = {{&S47592[49], 6}, {NULL}};
const Segment SGI376591[] = {{&S47592[33], 13}, {NULL}};
const Segment SGI376592[] = {{&S47592[121], 7}, {NULL}};
const Segment SGI376593[] = {{&S47593[69], 5}, {NULL}};
const Segment SGI376594[] = {{&S47593[8], 3}, {NULL}};
const Segment SGI376595[] = {{&S47593[13], 9}, {NULL}};
const Segment SGI376596[] = {{&S47593[33], 15}, {NULL}};
const Segment SGI376597[] = {{&S47593[49], 6}, {NULL}};
const Segment SGI376598[] = {{&S47593[33], 13}, {NULL}};
const Segment SGI376599[] = {{&S47593[121], 7}, {NULL}};
const Segment SGI376600[] = {{&S47594[69], 5}, {NULL}};
const Segment SGI376601[] = {{&S47594[8], 3}, {NULL}};
const Segment SGI376602[] = {{&S47594[13], 9}, {NULL}};
const Segment SGI376603[] = {{&S47594[33], 15}, {NULL}};
const Segment SGI376604[] = {{&S47594[49], 6}, {NULL}};
const Segment SGI376605[] = {{&S47594[33], 13}, {NULL}};
const Segment SGI376606[] = {{&S47594[121], 7}, {NULL}};
const Segment SGI376607[] = {{&S47728[52], 5}, {NULL}};
const Segment SGI376608[] = {{&S47728[8], 3}, {NULL}};
const Segment SGI376609[] = {{&S47728[13], 3}, {NULL}};
const Segment SGI376610[] = {{&S47728[78], 7}, {NULL}};
const Segment SGI376611[] = {{&S47729[42], 6}, {NULL}};
const Segment SGI376612[] = {{&S47729[8], 3}, {NULL}};
const Segment SGI376613[] = {{&S47729[13], 3}, {NULL}};
const Segment SGI376614[] = {{&S47729[69], 7}, {NULL}};
const Segment SGI376615[] = {{&S47730[8], 3}, {NULL}};
const Segment SGI376616[] = {{&S47730[13], 3}, {NULL}};
const Segment SGI376617[] = {{&S47730[69], 7}, {NULL}};
const Segment SGI376618[] = {{&S48015[86], 6}, {NULL}};
const Segment SGI376619[] = {{&S48015[8], 3}, {NULL}};
const Segment SGI376620[] = {{&S48015[13], 9}, {NULL}};
const Segment SGI376621[] = {{&S48015[33], 15}, {NULL}};
const Segment SGI376622[] = {{&S48015[49], 16}, {NULL}};
const Segment SGI376623[] = {{&S48015[33], 13}, {NULL}};
const Segment SGI376624[] = {{&S48016[86], 6}, {NULL}};
const Segment SGI376625[] = {{&S48016[8], 3}, {NULL}};
const Segment SGI376626[] = {{&S48016[13], 9}, {NULL}};
const Segment SGI376627[] = {{&S48016[33], 15}, {NULL}};
const Segment SGI376628[] = {{&S48016[49], 16}, {NULL}};
const Segment SGI376629[] = {{&S48016[33], 13}, {NULL}};
const Segment SGI376630[] = {{&S48017[92], 6}, {NULL}};
const Segment SGI376631[] = {{&S48017[8], 3}, {NULL}};
const Segment SGI376632[] = {{&S48017[13], 9}, {NULL}};
const Segment SGI376633[] = {{&S48017[33], 15}, {NULL}};
const Segment SGI376634[] = {{&S48017[49], 22}, {NULL}};
const Segment SGI376635[] = {{&S48017[33], 13}, {NULL}};
const Segment SGI376636[] = {{&S48018[80], 6}, {NULL}};
const Segment SGI376637[] = {{&S48018[8], 3}, {NULL}};
const Segment SGI376638[] = {{&S48018[13], 9}, {NULL}};
const Segment SGI376639[] = {{&S48018[33], 15}, {NULL}};
const Segment SGI376640[] = {{&S48018[49], 10}, {NULL}};
const Segment SGI376641[] = {{&S48018[33], 13}, {NULL}};
const Segment SGI376642[] = {{&S49813[13], 23}, {NULL}};
const Segment SGI376643[] = {{&S49813[50], 3}, {NULL}};
const Segment SGI376644[] = {{&S48107[44], 6}, {NULL}};
const Segment SGI376645[] = {{&S48107[8], 3}, {NULL}};
const Segment SGI376646[] = {{&S48107[13], 3}, {NULL}};
const Segment SGI376647[] = {{&S48107[71], 13}, {NULL}};
const Segment SGI376648[] = {{&S50099[13], 33}, {NULL}};
const Segment SGI376649[] = {{&S48443[90], 6}, {NULL}};
const Segment SGI376650[] = {{&S48443[8], 3}, {NULL}};
const Segment SGI376651[] = {{&S48443[13], 9}, {NULL}};
const Segment SGI376652[] = {{&S48443[33], 15}, {NULL}};
const Segment SGI376653[] = {{&S48443[49], 20}, {NULL}};
const Segment SGI376654[] = {{&S48443[33], 13}, {NULL}};
const Segment SGI376655[] = {{&S48444[87], 6}, {NULL}};
const Segment SGI376656[] = {{&S48444[8], 3}, {NULL}};
const Segment SGI376657[] = {{&S48444[13], 9}, {NULL}};
const Segment SGI376658[] = {{&S48444[33], 15}, {NULL}};
const Segment SGI376659[] = {{&S48444[49], 17}, {NULL}};
const Segment SGI376660[] = {{&S48444[33], 13}, {NULL}};
const Segment SGI376661[] = {{&S48445[82], 5}, {NULL}};
const Segment SGI376662[] = {{&S48445[13], 9}, {NULL}};
const Segment SGI376663[] = {{&S48445[33], 15}, {NULL}};
const Segment SGI376664[] = {{&S48445[49], 10}, {NULL}};
const Segment SGI376665[] = {{&S48445[33], 13}, {NULL}};
const Segment SGI376666[] = {{&S48540[69], 5}, {NULL}};
const Segment SGI376667[] = {{&S48540[8], 3}, {NULL}};
const Segment SGI376668[] = {{&S48540[13], 9}, {NULL}};
const Segment SGI376669[] = {{&S48540[33], 15}, {NULL}};
const Segment SGI376670[] = {{&S48540[49], 6}, {NULL}};
const Segment SGI376671[] = {{&S48540[33], 13}, {NULL}};
const Segment SGI376672[] = {{&S48540[122], 7}, {NULL}};
const Segment SGI376673[] = {{&S49003[44], 6}, {NULL}};
const Segment SGI376674[] = {{&S49003[8], 3}, {NULL}};
const Segment SGI376675[] = {{&S49003[13], 3}, {NULL}};
const Segment SGI376676[] = {{&S49003[71], 13}, {NULL}};
const Segment SGI376677[] = {{&S49004[44], 5}, {NULL}};
const Segment SGI376678[] = {{&S49004[8], 3}, {NULL}};
const Segment SGI376679[] = {{&S49004[13], 3}, {NULL}};
const Segment SGI376680[] = {{&S49004[70], 13}, {NULL}};
const Segment SGI376681[] = {{&S49248[98], 6}, {NULL}};
const Segment SGI376682[] = {{&S49248[8], 3}, {NULL}};
const Segment SGI376683[] = {{&S49248[13], 9}, {NULL}};
const Segment SGI376684[] = {{&S49248[33], 15}, {NULL}};
const Segment SGI376685[] = {{&S49248[49], 28}, {NULL}};
const Segment SGI376686[] = {{&S49248[33], 13}, {NULL}};
const Segment SGI376687[] = {{&S49249[98], 6}, {NULL}};
const Segment SGI376688[] = {{&S49249[8], 3}, {NULL}};
const Segment SGI376689[] = {{&S49249[13], 9}, {NULL}};
const Segment SGI376690[] = {{&S49249[33], 15}, {NULL}};
const Segment SGI376691[] = {{&S49249[49], 28}, {NULL}};
const Segment SGI376692[] = {{&S49249[33], 13}, {NULL}};
const Segment SGI376693[] = {{&S49340[60], 5}, {NULL}};
const Segment SGI376694[] = {{&S49340[8], 3}, {NULL}};
const Segment SGI376695[] = {{&S49340[13], 7}, {NULL}};
const Segment SGI376696[] = {{&S49340[25], 14}, {NULL}};
const Segment SGI376697[] = {{&S49340[86], 16}, {NULL}};
const Segment SGI376698[] = {{&S49661[60], 5}, {NULL}};
const Segment SGI376699[] = {{&S49661[8], 3}, {NULL}};
const Segment SGI376700[] = {{&S49661[13], 7}, {NULL}};
const Segment SGI376701[] = {{&S49661[25], 14}, {NULL}};
const Segment SGI376702[] = {{&S49661[86], 16}, {NULL}};
const Segment SGI376703[] = {{&S49889[78], 5}, {NULL}};
const Segment SGI376704[] = {{&S49889[8], 3}, {NULL}};
const Segment SGI376705[] = {{&S49889[13], 7}, {NULL}};
const Segment SGI376706[] = {{&S49889[85], 6}, {NULL}};
const Segment SGI376707[] = {{&S51748[13], 46}, {NULL}};
const Segment SGI376708[] = {{&S51748[73], 6}, {NULL}};
const Segment SGI376709[] = {{&S51748[108], 3}, {NULL}};
const Segment SGI376710[] = {{&S51749[13], 30}, {NULL}};
const Segment SGI376711[] = {{&S51749[57], 5}, {NULL}};
const Segment SGI376712[] = {{&S51749[91], 3}, {NULL}};
const Segment SGI376713[] = {{&S51750[13], 37}, {NULL}};
const Segment SGI376714[] = {{&S51750[64], 5}, {NULL}};
const Segment SGI376715[] = {{&S51750[98], 3}, {NULL}};
const Segment SGI376716[] = {{&S51751[13], 30}, {NULL}};
const Segment SGI376717[] = {{&S51751[57], 5}, {NULL}};
const Segment SGI376718[] = {{&S51751[91], 3}, {NULL}};
const Segment SGI376719[] = {{&S51752[13], 27}, {NULL}};
const Segment SGI376720[] = {{&S51752[61], 5}, {NULL}};
const Segment SGI376721[] = {{&S51752[95], 3}, {NULL}};
const Segment SGI376722[] = {{&S51753[38], 6}, {NULL}};
const Segment SGI376723[] = {{&S51753[70], 3}, {NULL}};
const Segment SGI376724[] = {{&S51821[13], 56}, {NULL}};
const Segment SGI376725[] = {{&S51821[83], 6}, {NULL}};
const Segment SGI376726[] = {{&S51821[117], 3}, {NULL}};
const Segment SGI376727[] = {{&S51822[13], 46}, {NULL}};
const Segment SGI376728[] = {{&S51822[73], 6}, {NULL}};
const Segment SGI376729[] = {{&S51822[107], 3}, {NULL}};
const Segment SGI376730[] = {{&S51823[13], 56}, {NULL}};
const Segment SGI376731[] = {{&S51823[83], 6}, {NULL}};
const Segment SGI376732[] = {{&S51823[117], 3}, {NULL}};
const Segment SGI376733[] = {{&S51824[13], 56}, {NULL}};
const Segment SGI376734[] = {{&S51824[83], 5}, {NULL}};
const Segment SGI376735[] = {{&S51824[116], 3}, {NULL}};
const Segment SGI376736[] = {{&S51825[13], 56}, {NULL}};
const Segment SGI376737[] = {{&S51825[83], 5}, {NULL}};
const Segment SGI376738[] = {{&S51825[116], 3}, {NULL}};
const Segment SGI376739[] = {{&S51826[13], 56}, {NULL}};
const Segment SGI376740[] = {{&S51826[83], 5}, {NULL}};
const Segment SGI376741[] = {{&S51826[116], 3}, {NULL}};
const Segment SGI376742[] = {{&S51827[13], 56}, {NULL}};
const Segment SGI376743[] = {{&S51827[83], 6}, {NULL}};
const Segment SGI376744[] = {{&S51827[117], 3}, {NULL}};
const Segment SGI376745[] = {{&S51828[13], 56}, {NULL}};
const Segment SGI376746[] = {{&S51828[83], 6}, {NULL}};
const Segment SGI376747[] = {{&S51828[117], 3}, {NULL}};
const Segment SGI376748[] = {{&S51829[13], 56}, {NULL}};
const Segment SGI376749[] = {{&S51829[83], 5}, {NULL}};
const Segment SGI376750[] = {{&S51829[116], 3}, {NULL}};
const Segment SGI376751[] = {{&S51830[13], 54}, {NULL}};
const Segment SGI376752[] = {{&S51830[81], 5}, {NULL}};
const Segment SGI376753[] = {{&S51830[114], 3}, {NULL}};
const Segment SGI376754[] = {{&S51831[13], 56}, {NULL}};
const Segment SGI376755[] = {{&S51831[83], 5}, {NULL}};
const Segment SGI376756[] = {{&S51831[116], 3}, {NULL}};
const Segment SGI376757[] = {{&S51832[13], 56}, {NULL}};
const Segment SGI376758[] = {{&S51832[83], 5}, {NULL}};
const Segment SGI376759[] = {{&S51832[116], 3}, {NULL}};
const Segment SGI376760[] = {{&S51833[13], 54}, {NULL}};
const Segment SGI376761[] = {{&S51833[81], 6}, {NULL}};
const Segment SGI376762[] = {{&S51833[115], 3}, {NULL}};
const Segment SGI376763[] = {{&S51834[13], 54}, {NULL}};
const Segment SGI376764[] = {{&S51834[81], 5}, {NULL}};
const Segment SGI376765[] = {{&S51834[114], 3}, {NULL}};
const Segment SGI376766[] = {{&S51835[13], 56}, {NULL}};
const Segment SGI376767[] = {{&S51835[83], 6}, {NULL}};
const Segment SGI376768[] = {{&S51835[117], 3}, {NULL}};
const Segment SGI376769[] = {{&S51836[13], 56}, {NULL}};
const Segment SGI376770[] = {{&S51836[83], 5}, {NULL}};
const Segment SGI376771[] = {{&S51836[116], 3}, {NULL}};
const Segment SGI376772[] = {{&S51837[13], 54}, {NULL}};
const Segment SGI376773[] = {{&S51837[81], 5}, {NULL}};
const Segment SGI376774[] = {{&S51837[114], 3}, {NULL}};
const Segment SGI376775[] = {{&S51838[13], 56}, {NULL}};
const Segment SGI376776[] = {{&S51838[83], 5}, {NULL}};
const Segment SGI376777[] = {{&S51838[116], 3}, {NULL}};
const Segment SGI376778[] = {{&S51839[13], 56}, {NULL}};
const Segment SGI376779[] = {{&S51839[83], 5}, {NULL}};
const Segment SGI376780[] = {{&S51839[116], 3}, {NULL}};
const Segment SGI376781[] = {{&S51840[13], 56}, {NULL}};
const Segment SGI376782[] = {{&S51840[83], 6}, {NULL}};
const Segment SGI376783[] = {{&S51840[117], 3}, {NULL}};
const Segment SGI376784[] = {{&S51841[13], 56}, {NULL}};
const Segment SGI376785[] = {{&S51841[83], 6}, {NULL}};
const Segment SGI376786[] = {{&S51841[117], 3}, {NULL}};
const Segment SGI376787[] = {{&S51842[13], 56}, {NULL}};
const Segment SGI376788[] = {{&S51842[83], 6}, {NULL}};
const Segment SGI376789[] = {{&S51842[117], 3}, {NULL}};
const Segment SGI376790[] = {{&S51843[13], 54}, {NULL}};
const Segment SGI376791[] = {{&S51843[81], 6}, {NULL}};
const Segment SGI376792[] = {{&S51843[115], 3}, {NULL}};
const Segment SGI376793[] = {{&S51844[13], 15}, {NULL}};
const Segment SGI376794[] = {{&S51844[42], 6}, {NULL}};
const Segment SGI376795[] = {{&S51844[77], 3}, {NULL}};
const Segment SGI376796[] = {{&S51845[13], 43}, {NULL}};
const Segment SGI376797[] = {{&S51845[70], 6}, {NULL}};
const Segment SGI376798[] = {{&S51845[104], 3}, {NULL}};
const Segment SGI376799[] = {{&S51846[13], 9}, {NULL}};
const Segment SGI376800[] = {{&S51846[36], 6}, {NULL}};
const Segment SGI376801[] = {{&S51846[71], 3}, {NULL}};
const Segment SGI376802[] = {{&S51847[13], 43}, {NULL}};
const Segment SGI376803[] = {{&S51847[70], 6}, {NULL}};
const Segment SGI376804[] = {{&S51847[104], 3}, {NULL}};
const Segment SGI376805[] = {{&S51848[13], 52}, {NULL}};
const Segment SGI376806[] = {{&S51848[79], 6}, {NULL}};
const Segment SGI376807[] = {{&S51848[114], 3}, {NULL}};
const Segment SGI376808[] = {{&S51849[13], 46}, {NULL}};
const Segment SGI376809[] = {{&S51849[73], 6}, {NULL}};
const Segment SGI376810[] = {{&S51849[107], 3}, {NULL}};
const Segment SGI376811[] = {{&S51850[13], 43}, {NULL}};
const Segment SGI376812[] = {{&S51850[70], 6}, {NULL}};
const Segment SGI376813[] = {{&S51850[104], 3}, {NULL}};
const Segment SGI376814[] = {{&S51851[13], 43}, {NULL}};
const Segment SGI376815[] = {{&S51851[70], 6}, {NULL}};
const Segment SGI376816[] = {{&S51851[104], 3}, {NULL}};
const Segment SGI376817[] = {{&S51852[13], 43}, {NULL}};
const Segment SGI376818[] = {{&S51852[70], 6}, {NULL}};
const Segment SGI376819[] = {{&S51852[104], 3}, {NULL}};
const Segment SGI376820[] = {{&S51853[13], 60}, {NULL}};
const Segment SGI376821[] = {{&S51853[87], 6}, {NULL}};
const Segment SGI376822[] = {{&S51853[122], 3}, {NULL}};
const Segment SGI376823[] = {{&S51854[13], 23}, {NULL}};
const Segment SGI376824[] = {{&S51854[50], 5}, {NULL}};
const Segment SGI376825[] = {{&S51854[110], 3}, {NULL}};
const Segment SGI376826[] = {{&S51855[13], 58}, {NULL}};
const Segment SGI376827[] = {{&S51855[85], 6}, {NULL}};
const Segment SGI376828[] = {{&S51855[120], 3}, {NULL}};
const Segment SGI376829[] = {{&S51856[13], 23}, {NULL}};
const Segment SGI376830[] = {{&S51856[50], 5}, {NULL}};
const Segment SGI376831[] = {{&S51856[110], 3}, {NULL}};
const Segment SGI376832[] = {{&S51857[13], 23}, {NULL}};
const Segment SGI376833[] = {{&S51857[50], 5}, {NULL}};
const Segment SGI376834[] = {{&S51857[110], 3}, {NULL}};
const Segment SGI376835[] = {{&S51858[13], 76}, {NULL}};
const Segment SGI376836[] = {{&S51858[103], 6}, {NULL}};
const Segment SGI376837[] = {{&S51858[43], 3}, {NULL}};
const Segment SGI376838[] = {{&S51859[13], 43}, {NULL}};
const Segment SGI376839[] = {{&S51859[70], 6}, {NULL}};
const Segment SGI376840[] = {{&S51859[104], 3}, {NULL}};
const Segment SGI376841[] = {{&S50355[44], 6}, {NULL}};
const Segment SGI376842[] = {{&S50355[8], 3}, {NULL}};
const Segment SGI376843[] = {{&S50355[13], 3}, {NULL}};
const Segment SGI376844[] = {{&S50355[71], 13}, {NULL}};
const Segment SGI376845[] = {{&S50356[44], 6}, {NULL}};
const Segment SGI376846[] = {{&S50356[8], 3}, {NULL}};
const Segment SGI376847[] = {{&S50356[13], 3}, {NULL}};
const Segment SGI376848[] = {{&S50356[71], 13}, {NULL}};
const Segment SGI376849[] = {{&S50392[102], 6}, {NULL}};
const Segment SGI376850[] = {{&S50392[8], 3}, {NULL}};
const Segment SGI376851[] = {{&S50392[13], 9}, {NULL}};
const Segment SGI376852[] = {{&S50392[33], 15}, {NULL}};
const Segment SGI376853[] = {{&S50392[49], 6}, {NULL}};
const Segment SGI376854[] = {{&S50392[33], 13}, {NULL}};
const Segment SGI376855[] = {{&S50392[110], 17}, {NULL}};
const Segment SGI376856[] = {{&S50393[107], 6}, {NULL}};
const Segment SGI376857[] = {{&S50393[8], 3}, {NULL}};
const Segment SGI376858[] = {{&S50393[13], 9}, {NULL}};
const Segment SGI376859[] = {{&S50393[35], 15}, {NULL}};
const Segment SGI376860[] = {{&S50393[123], 3}, {NULL}};
const Segment SGI376861[] = {{&S50393[51], 6}, {NULL}};
const Segment SGI376862[] = {{&S50393[35], 13}, {NULL}};
const Segment SGI376863[] = {{&S50393[130], 16}, {NULL}};
const Segment SGI376864[] = {{&S50394[105], 6}, {NULL}};
const Segment SGI376865[] = {{&S50394[8], 3}, {NULL}};
const Segment SGI376866[] = {{&S50394[13], 9}, {NULL}};
const Segment SGI376867[] = {{&S50394[33], 15}, {NULL}};
const Segment SGI376868[] = {{&S50394[123], 3}, {NULL}};
const Segment SGI376869[] = {{&S50394[49], 6}, {NULL}};
const Segment SGI376870[] = {{&S50394[33], 13}, {NULL}};
const Segment SGI376871[] = {{&S50394[128], 16}, {NULL}};
const Segment SGI376872[] = {{&S50395[69], 6}, {NULL}};
const Segment SGI376873[] = {{&S50395[8], 3}, {NULL}};
const Segment SGI376874[] = {{&S50395[13], 9}, {NULL}};
const Segment SGI376875[] = {{&S50395[33], 15}, {NULL}};
const Segment SGI376876[] = {{&S50395[49], 6}, {NULL}};
const Segment SGI376877[] = {{&S50395[33], 13}, {NULL}};
const Segment SGI376878[] = {{&S50395[111], 16}, {NULL}};
const Segment SGI376879[] = {{&S50396[107], 6}, {NULL}};
const Segment SGI376880[] = {{&S50396[8], 3}, {NULL}};
const Segment SGI376881[] = {{&S50396[13], 9}, {NULL}};
const Segment SGI376882[] = {{&S50396[35], 15}, {NULL}};
const Segment SGI376883[] = {{&S50396[125], 3}, {NULL}};
const Segment SGI376884[] = {{&S50396[51], 6}, {NULL}};
const Segment SGI376885[] = {{&S50396[35], 13}, {NULL}};
const Segment SGI376886[] = {{&S50396[132], 16}, {NULL}};
const Segment SGI376887[] = {{&S50397[105], 6}, {NULL}};
const Segment SGI376888[] = {{&S50397[8], 3}, {NULL}};
const Segment SGI376889[] = {{&S50397[13], 9}, {NULL}};
const Segment SGI376890[] = {{&S50397[33], 15}, {NULL}};
const Segment SGI376891[] = {{&S50397[123], 3}, {NULL}};
const Segment SGI376892[] = {{&S50397[49], 6}, {NULL}};
const Segment SGI376893[] = {{&S50397[33], 13}, {NULL}};
const Segment SGI376894[] = {{&S50397[130], 18}, {NULL}};
const Segment SGI376895[] = {{&S52641[13], 39}, {NULL}};
const Segment SGI376896[] = {{&S52641[66], 5}, {NULL}};
const Segment SGI376897[] = {{&S52641[101], 5}, {NULL}};
const Segment SGI376898[] = {{&S52642[13], 39}, {NULL}};
const Segment SGI376899[] = {{&S52642[66], 6}, {NULL}};
const Segment SGI376900[] = {{&S52642[102], 5}, {NULL}};
const Segment SGI376901[] = {{&S52643[13], 39}, {NULL}};
const Segment SGI376902[] = {{&S52643[66], 6}, {NULL}};
const Segment SGI376903[] = {{&S52643[102], 7}, {NULL}};
const Segment SGI376904[] = {{&S50840[93], 6}, {NULL}};
const Segment SGI376905[] = {{&S50840[8], 3}, {NULL}};
const Segment SGI376906[] = {{&S50840[13], 9}, {NULL}};
const Segment SGI376907[] = {{&S50840[33], 15}, {NULL}};
const Segment SGI376908[] = {{&S50840[49], 23}, {NULL}};
const Segment SGI376909[] = {{&S50840[33], 13}, {NULL}};
const Segment SGI376910[] = {{&S53019[13], 9}, {NULL}};
const Segment SGI376911[] = {{&S53019[36], 6}, {NULL}};
const Segment SGI376912[] = {{&S53019[72], 10}, {NULL}};
const Segment SGI376913[] = {{&S53020[13], 9}, {NULL}};
const Segment SGI376914[] = {{&S53020[36], 6}, {NULL}};
const Segment SGI376915[] = {{&S53020[72], 10}, {NULL}};
const Segment SGI376916[] = {{&S53021[13], 9}, {NULL}};
const Segment SGI376917[] = {{&S53021[36], 6}, {NULL}};
const Segment SGI376918[] = {{&S53021[72], 10}, {NULL}};
const Segment SGI376919[] = {{&S53022[13], 9}, {NULL}};
const Segment SGI376920[] = {{&S53022[36], 6}, {NULL}};
const Segment SGI376921[] = {{&S53022[72], 10}, {NULL}};
const Segment SGI376922[] = {{&S53023[13], 9}, {NULL}};
const Segment SGI376923[] = {{&S53023[36], 6}, {NULL}};
const Segment SGI376924[] = {{&S53023[72], 10}, {NULL}};
const Segment SGI376925[] = {{&S53024[13], 9}, {NULL}};
const Segment SGI376926[] = {{&S53024[36], 6}, {NULL}};
const Segment SGI376927[] = {{&S53024[72], 10}, {NULL}};
const Segment SGI376928[] = {{&S53025[13], 9}, {NULL}};
const Segment SGI376929[] = {{&S53025[36], 6}, {NULL}};
const Segment SGI376930[] = {{&S53025[72], 11}, {NULL}};
const Segment SGI376931[] = {{&S53026[13], 9}, {NULL}};
const Segment SGI376932[] = {{&S53026[36], 6}, {NULL}};
const Segment SGI376933[] = {{&S53026[72], 10}, {NULL}};
const Segment SGI376934[] = {{&S53027[13], 9}, {NULL}};
const Segment SGI376935[] = {{&S53027[36], 6}, {NULL}};
const Segment SGI376936[] = {{&S53027[72], 10}, {NULL}};
const Segment SGI376937[] = {{&S53028[13], 9}, {NULL}};
const Segment SGI376938[] = {{&S53028[36], 6}, {NULL}};
const Segment SGI376939[] = {{&S53028[72], 11}, {NULL}};
const Segment SGI376940[] = {{&S53029[13], 9}, {NULL}};
const Segment SGI376941[] = {{&S53029[36], 6}, {NULL}};
const Segment SGI376942[] = {{&S53029[72], 11}, {NULL}};
const Segment SGI376943[] = {{&S53030[13], 9}, {NULL}};
const Segment SGI376944[] = {{&S53030[36], 6}, {NULL}};
const Segment SGI376945[] = {{&S53030[72], 11}, {NULL}};
const Segment SGI376946[] = {{&S53031[13], 9}, {NULL}};
const Segment SGI376947[] = {{&S53031[36], 6}, {NULL}};
const Segment SGI376948[] = {{&S53031[72], 11}, {NULL}};
const Segment SGI376949[] = {{&S53032[13], 9}, {NULL}};
const Segment SGI376950[] = {{&S53032[36], 6}, {NULL}};
const Segment SGI376951[] = {{&S53032[72], 11}, {NULL}};
const Segment SGI376952[] = {{&S53033[13], 9}, {NULL}};
const Segment SGI376953[] = {{&S53033[36], 6}, {NULL}};
const Segment SGI376954[] = {{&S53033[72], 11}, {NULL}};
const Segment SGI376955[] = {{&S53034[13], 9}, {NULL}};
const Segment SGI376956[] = {{&S53034[36], 6}, {NULL}};
const Segment SGI376957[] = {{&S53034[72], 11}, {NULL}};
const Segment SGI376958[] = {{&S53035[13], 9}, {NULL}};
const Segment SGI376959[] = {{&S53035[36], 6}, {NULL}};
const Segment SGI376960[] = {{&S53035[72], 11}, {NULL}};
const Segment SGI376961[] = {{&S53036[13], 9}, {NULL}};
const Segment SGI376962[] = {{&S53036[36], 6}, {NULL}};
const Segment SGI376963[] = {{&S53036[72], 11}, {NULL}};
const Segment SGI376964[] = {{&S53037[13], 9}, {NULL}};
const Segment SGI376965[] = {{&S53037[36], 6}, {NULL}};
const Segment SGI376966[] = {{&S53037[72], 11}, {NULL}};
const Segment SGI376967[] = {{&S53038[13], 9}, {NULL}};
const Segment SGI376968[] = {{&S53038[36], 6}, {NULL}};
const Segment SGI376969[] = {{&S53038[72], 11}, {NULL}};
const Segment SGI376970[] = {{&S53039[13], 9}, {NULL}};
const Segment SGI376971[] = {{&S53039[36], 6}, {NULL}};
const Segment SGI376972[] = {{&S53039[72], 11}, {NULL}};
const Segment SGI376973[] = {{&S53040[13], 9}, {NULL}};
const Segment SGI376974[] = {{&S53040[36], 6}, {NULL}};
const Segment SGI376975[] = {{&S53040[72], 11}, {NULL}};
const Segment SGI376976[] = {{&S53041[13], 9}, {NULL}};
const Segment SGI376977[] = {{&S53041[36], 6}, {NULL}};
const Segment SGI376978[] = {{&S53041[72], 11}, {NULL}};
const Segment SGI376979[] = {{&S53042[13], 9}, {NULL}};
const Segment SGI376980[] = {{&S53042[36], 6}, {NULL}};
const Segment SGI376981[] = {{&S53042[72], 11}, {NULL}};
const Segment SGI376982[] = {{&S53043[13], 9}, {NULL}};
const Segment SGI376983[] = {{&S53043[36], 6}, {NULL}};
const Segment SGI376984[] = {{&S53043[72], 11}, {NULL}};
const Segment SGI376985[] = {{&S53044[13], 9}, {NULL}};
const Segment SGI376986[] = {{&S53044[36], 6}, {NULL}};
const Segment SGI376987[] = {{&S53044[72], 11}, {NULL}};
const Segment SGI376988[] = {{&S53045[13], 9}, {NULL}};
const Segment SGI376989[] = {{&S53045[36], 6}, {NULL}};
const Segment SGI376990[] = {{&S53045[72], 11}, {NULL}};
const Segment SGI376991[] = {{&S51350[60], 5}, {NULL}};
const Segment SGI376992[] = {{&S51350[8], 3}, {NULL}};
const Segment SGI376993[] = {{&S51350[13], 7}, {NULL}};
const Segment SGI376994[] = {{&S51350[25], 14}, {NULL}};
const Segment SGI376995[] = {{&S51350[86], 6}, {NULL}};
const Segment SGI376996[] = {{&S51492[105], 6}, {NULL}};
const Segment SGI376997[] = {{&S51492[8], 3}, {NULL}};
const Segment SGI376998[] = {{&S51492[13], 9}, {NULL}};
const Segment SGI376999[] = {{&S51492[33], 15}, {NULL}};
const Segment SGI377000[] = {{&S51492[123], 3}, {NULL}};
const Segment SGI377001[] = {{&S51492[49], 6}, {NULL}};
const Segment SGI377002[] = {{&S51492[33], 13}, {NULL}};
const Segment SGI377003[] = {{&S51492[130], 16}, {NULL}};
const Segment SGI377004[] = {{&S51493[105], 6}, {NULL}};
const Segment SGI377005[] = {{&S51493[8], 3}, {NULL}};
const Segment SGI377006[] = {{&S51493[13], 9}, {NULL}};
const Segment SGI377007[] = {{&S51493[33], 15}, {NULL}};
const Segment SGI377008[] = {{&S51493[123], 3}, {NULL}};
const Segment SGI377009[] = {{&S51493[49], 6}, {NULL}};
const Segment SGI377010[] = {{&S51493[33], 13}, {NULL}};
const Segment SGI377011[] = {{&S51493[130], 18}, {NULL}};
const Segment SGI377012[] = {{&S51494[105], 6}, {NULL}};
const Segment SGI377013[] = {{&S51494[8], 3}, {NULL}};
const Segment SGI377014[] = {{&S51494[13], 9}, {NULL}};
const Segment SGI377015[] = {{&S51494[33], 15}, {NULL}};
const Segment SGI377016[] = {{&S51494[123], 3}, {NULL}};
const Segment SGI377017[] = {{&S51494[49], 6}, {NULL}};
const Segment SGI377018[] = {{&S51494[33], 13}, {NULL}};
const Segment SGI377019[] = {{&S51494[130], 18}, {NULL}};
const Segment SGI377020[] = {{&S51495[105], 6}, {NULL}};
const Segment SGI377021[] = {{&S51495[8], 3}, {NULL}};
const Segment SGI377022[] = {{&S51495[13], 9}, {NULL}};
const Segment SGI377023[] = {{&S51495[33], 15}, {NULL}};
const Segment SGI377024[] = {{&S51495[123], 3}, {NULL}};
const Segment SGI377025[] = {{&S51495[49], 6}, {NULL}};
const Segment SGI377026[] = {{&S51495[33], 13}, {NULL}};
const Segment SGI377027[] = {{&S51495[130], 18}, {NULL}};
const Segment SGI377028[] = {{&S51496[105], 6}, {NULL}};
const Segment SGI377029[] = {{&S51496[8], 3}, {NULL}};
const Segment SGI377030[] = {{&S51496[13], 9}, {NULL}};
const Segment SGI377031[] = {{&S51496[33], 15}, {NULL}};
const Segment SGI377032[] = {{&S51496[123], 3}, {NULL}};
const Segment SGI377033[] = {{&S51496[49], 6}, {NULL}};
const Segment SGI377034[] = {{&S51496[33], 13}, {NULL}};
const Segment SGI377035[] = {{&S51496[130], 16}, {NULL}};
const Segment SGI377036[] = {{&S51497[105], 6}, {NULL}};
const Segment SGI377037[] = {{&S51497[8], 3}, {NULL}};
const Segment SGI377038[] = {{&S51497[13], 9}, {NULL}};
const Segment SGI377039[] = {{&S51497[33], 15}, {NULL}};
const Segment SGI377040[] = {{&S51497[123], 3}, {NULL}};
const Segment SGI377041[] = {{&S51497[49], 6}, {NULL}};
const Segment SGI377042[] = {{&S51497[33], 13}, {NULL}};
const Segment SGI377043[] = {{&S51497[128], 16}, {NULL}};
const Segment SGI377044[] = {{&S51498[105], 6}, {NULL}};
const Segment SGI377045[] = {{&S51498[8], 3}, {NULL}};
const Segment SGI377046[] = {{&S51498[13], 9}, {NULL}};
const Segment SGI377047[] = {{&S51498[33], 15}, {NULL}};
const Segment SGI377048[] = {{&S51498[123], 3}, {NULL}};
const Segment SGI377049[] = {{&S51498[49], 6}, {NULL}};
const Segment SGI377050[] = {{&S51498[33], 13}, {NULL}};
const Segment SGI377051[] = {{&S51498[130], 16}, {NULL}};
const Segment SGI377052[] = {{&S51499[102], 6}, {NULL}};
const Segment SGI377053[] = {{&S51499[8], 3}, {NULL}};
const Segment SGI377054[] = {{&S51499[13], 9}, {NULL}};
const Segment SGI377055[] = {{&S51499[33], 15}, {NULL}};
const Segment SGI377056[] = {{&S51499[49], 6}, {NULL}};
const Segment SGI377057[] = {{&S51499[33], 13}, {NULL}};
const Segment SGI377058[] = {{&S51499[110], 17}, {NULL}};
const Segment SGI377059[] = {{&S51500[105], 6}, {NULL}};
const Segment SGI377060[] = {{&S51500[8], 3}, {NULL}};
const Segment SGI377061[] = {{&S51500[13], 9}, {NULL}};
const Segment SGI377062[] = {{&S51500[33], 15}, {NULL}};
const Segment SGI377063[] = {{&S51500[123], 3}, {NULL}};
const Segment SGI377064[] = {{&S51500[49], 6}, {NULL}};
const Segment SGI377065[] = {{&S51500[33], 13}, {NULL}};
const Segment SGI377066[] = {{&S51500[130], 18}, {NULL}};
const Segment SGI377067[] = {{&S51501[105], 6}, {NULL}};
const Segment SGI377068[] = {{&S51501[8], 3}, {NULL}};
const Segment SGI377069[] = {{&S51501[13], 9}, {NULL}};
const Segment SGI377070[] = {{&S51501[33], 15}, {NULL}};
const Segment SGI377071[] = {{&S51501[123], 3}, {NULL}};
const Segment SGI377072[] = {{&S51501[49], 6}, {NULL}};
const Segment SGI377073[] = {{&S51501[33], 13}, {NULL}};
const Segment SGI377074[] = {{&S51501[130], 16}, {NULL}};
const Segment SGI377075[] = {{&S51502[105], 6}, {NULL}};
const Segment SGI377076[] = {{&S51502[8], 3}, {NULL}};
const Segment SGI377077[] = {{&S51502[13], 9}, {NULL}};
const Segment SGI377078[] = {{&S51502[33], 15}, {NULL}};
const Segment SGI377079[] = {{&S51502[123], 3}, {NULL}};
const Segment SGI377080[] = {{&S51502[49], 6}, {NULL}};
const Segment SGI377081[] = {{&S51502[33], 13}, {NULL}};
const Segment SGI377082[] = {{&S51502[128], 16}, {NULL}};
const Segment SGI377083[] = {{&S51503[102], 6}, {NULL}};
const Segment SGI377084[] = {{&S51503[8], 3}, {NULL}};
const Segment SGI377085[] = {{&S51503[13], 9}, {NULL}};
const Segment SGI377086[] = {{&S51503[33], 15}, {NULL}};
const Segment SGI377087[] = {{&S51503[49], 6}, {NULL}};
const Segment SGI377088[] = {{&S51503[33], 13}, {NULL}};
const Segment SGI377089[] = {{&S51503[110], 17}, {NULL}};
const Segment SGI377090[] = {{&S51504[105], 6}, {NULL}};
const Segment SGI377091[] = {{&S51504[8], 3}, {NULL}};
const Segment SGI377092[] = {{&S51504[13], 9}, {NULL}};
const Segment SGI377093[] = {{&S51504[33], 15}, {NULL}};
const Segment SGI377094[] = {{&S51504[123], 3}, {NULL}};
const Segment SGI377095[] = {{&S51504[49], 6}, {NULL}};
const Segment SGI377096[] = {{&S51504[33], 13}, {NULL}};
const Segment SGI377097[] = {{&S51504[130], 18}, {NULL}};
const Segment SGI377098[] = {{&S51505[105], 6}, {NULL}};
const Segment SGI377099[] = {{&S51505[8], 3}, {NULL}};
const Segment SGI377100[] = {{&S51505[13], 9}, {NULL}};
const Segment SGI377101[] = {{&S51505[33], 15}, {NULL}};
const Segment SGI377102[] = {{&S51505[121], 3}, {NULL}};
const Segment SGI377103[] = {{&S51505[49], 6}, {NULL}};
const Segment SGI377104[] = {{&S51505[33], 13}, {NULL}};
const Segment SGI377105[] = {{&S51505[128], 16}, {NULL}};
const Segment SGI377106[] = {{&S51506[105], 6}, {NULL}};
const Segment SGI377107[] = {{&S51506[8], 3}, {NULL}};
const Segment SGI377108[] = {{&S51506[13], 9}, {NULL}};
const Segment SGI377109[] = {{&S51506[33], 15}, {NULL}};
const Segment SGI377110[] = {{&S51506[123], 3}, {NULL}};
const Segment SGI377111[] = {{&S51506[49], 6}, {NULL}};
const Segment SGI377112[] = {{&S51506[33], 13}, {NULL}};
const Segment SGI377113[] = {{&S51506[128], 16}, {NULL}};
const Segment SGI377114[] = {{&S51507[102], 6}, {NULL}};
const Segment SGI377115[] = {{&S51507[8], 3}, {NULL}};
const Segment SGI377116[] = {{&S51507[13], 9}, {NULL}};
const Segment SGI377117[] = {{&S51507[33], 15}, {NULL}};
const Segment SGI377118[] = {{&S51507[49], 6}, {NULL}};
const Segment SGI377119[] = {{&S51507[33], 13}, {NULL}};
const Segment SGI377120[] = {{&S51507[110], 17}, {NULL}};
const Segment SGI377121[] = {{&S51508[105], 6}, {NULL}};
const Segment SGI377122[] = {{&S51508[8], 3}, {NULL}};
const Segment SGI377123[] = {{&S51508[13], 9}, {NULL}};
const Segment SGI377124[] = {{&S51508[33], 15}, {NULL}};
const Segment SGI377125[] = {{&S51508[123], 3}, {NULL}};
const Segment SGI377126[] = {{&S51508[49], 6}, {NULL}};
const Segment SGI377127[] = {{&S51508[33], 13}, {NULL}};
const Segment SGI377128[] = {{&S51508[130], 16}, {NULL}};
const Segment SGI377129[] = {{&S51509[105], 6}, {NULL}};
const Segment SGI377130[] = {{&S51509[8], 3}, {NULL}};
const Segment SGI377131[] = {{&S51509[13], 9}, {NULL}};
const Segment SGI377132[] = {{&S51509[33], 15}, {NULL}};
const Segment SGI377133[] = {{&S51509[123], 3}, {NULL}};
const Segment SGI377134[] = {{&S51509[49], 6}, {NULL}};
const Segment SGI377135[] = {{&S51509[33], 13}, {NULL}};
const Segment SGI377136[] = {{&S51509[130], 18}, {NULL}};
const Segment SGI377137[] = {{&S53729[13], 21}, {NULL}};
const Segment SGI377138[] = {{&S53729[48], 6}, {NULL}};
const Segment SGI377139[] = {{&S53930[13], 36}, {NULL}};
const Segment SGI377140[] = {{&S53930[63], 5}, {NULL}};
const Segment SGI377141[] = {{&S52293[60], 5}, {NULL}};
const Segment SGI377142[] = {{&S52293[8], 3}, {NULL}};
const Segment SGI377143[] = {{&S52293[13], 7}, {NULL}};
const Segment SGI377144[] = {{&S52293[25], 14}, {NULL}};
const Segment SGI377145[] = {{&S52293[86], 6}, {NULL}};
const Segment SGI377146[] = {{&S52294[60], 5}, {NULL}};
const Segment SGI377147[] = {{&S52294[8], 3}, {NULL}};
const Segment SGI377148[] = {{&S52294[13], 7}, {NULL}};
const Segment SGI377149[] = {{&S52294[25], 14}, {NULL}};
const Segment SGI377150[] = {{&S52294[86], 6}, {NULL}};
const Segment SGI377151[] = {{&S52755[110], 6}, {NULL}};
const Segment SGI377152[] = {{&S52755[8], 3}, {NULL}};
const Segment SGI377153[] = {{&S52755[13], 7}, {NULL}};
const Segment SGI377154[] = {{&S52755[25], 14}, {NULL}};
const Segment SGI377155[] = {{&S52755[97], 3}, {NULL}};
const Segment SGI377156[] = {{&S52756[60], 6}, {NULL}};
const Segment SGI377157[] = {{&S52756[8], 3}, {NULL}};
const Segment SGI377158[] = {{&S52756[13], 7}, {NULL}};
const Segment SGI377159[] = {{&S52756[25], 14}, {NULL}};
const Segment SGI377160[] = {{&S52756[97], 3}, {NULL}};
const Segment SGI377161[] = {{&S52757[60], 6}, {NULL}};
const Segment SGI377162[] = {{&S52757[8], 3}, {NULL}};
const Segment SGI377163[] = {{&S52757[13], 7}, {NULL}};
const Segment SGI377164[] = {{&S52757[25], 14}, {NULL}};
const Segment SGI377165[] = {{&S52757[95], 3}, {NULL}};
const Segment SGI377166[] = {{&S52758[60], 6}, {NULL}};
const Segment SGI377167[] = {{&S52758[8], 3}, {NULL}};
const Segment SGI377168[] = {{&S52758[13], 7}, {NULL}};
const Segment SGI377169[] = {{&S52758[25], 14}, {NULL}};
const Segment SGI377170[] = {{&S52758[97], 3}, {NULL}};
const Segment SGI377171[] = {{&S53071[117], 6}, {NULL}};
const Segment SGI377172[] = {{&S53071[8], 3}, {NULL}};
const Segment SGI377173[] = {{&S53071[13], 9}, {NULL}};
const Segment SGI377174[] = {{&S53071[33], 16}, {NULL}};
const Segment SGI377175[] = {{&S53071[104], 3}, {NULL}};
const Segment SGI377176[] = {{&S53071[33], 13}, {NULL}};
const Segment SGI377177[] = {{&S55108[13], 55}, {NULL}};
const Segment SGI377178[] = {{&S55108[82], 5}, {NULL}};
const Segment SGI377179[] = {{&S55108[116], 3}, {NULL}};
const Segment SGI377180[] = {{&S53233[102], 6}, {NULL}};
const Segment SGI377181[] = {{&S53233[8], 3}, {NULL}};
const Segment SGI377182[] = {{&S53233[13], 9}, {NULL}};
const Segment SGI377183[] = {{&S53233[33], 15}, {NULL}};
const Segment SGI377184[] = {{&S53233[49], 6}, {NULL}};
const Segment SGI377185[] = {{&S53233[33], 13}, {NULL}};
const Segment SGI377186[] = {{&S53233[110], 17}, {NULL}};
const Segment SGI377187[] = {{&S53234[105], 6}, {NULL}};
const Segment SGI377188[] = {{&S53234[8], 3}, {NULL}};
const Segment SGI377189[] = {{&S53234[13], 9}, {NULL}};
const Segment SGI377190[] = {{&S53234[33], 15}, {NULL}};
const Segment SGI377191[] = {{&S53234[123], 3}, {NULL}};
const Segment SGI377192[] = {{&S53234[49], 6}, {NULL}};
const Segment SGI377193[] = {{&S53234[33], 13}, {NULL}};
const Segment SGI377194[] = {{&S53234[128], 16}, {NULL}};
const Segment SGI377195[] = {{&S53235[105], 6}, {NULL}};
const Segment SGI377196[] = {{&S53235[8], 3}, {NULL}};
const Segment SGI377197[] = {{&S53235[13], 9}, {NULL}};
const Segment SGI377198[] = {{&S53235[33], 15}, {NULL}};
const Segment SGI377199[] = {{&S53235[123], 3}, {NULL}};
const Segment SGI377200[] = {{&S53235[49], 6}, {NULL}};
const Segment SGI377201[] = {{&S53235[33], 13}, {NULL}};
const Segment SGI377202[] = {{&S53235[128], 16}, {NULL}};
const Segment SGI377203[] = {{&S53236[105], 6}, {NULL}};
const Segment SGI377204[] = {{&S53236[8], 3}, {NULL}};
const Segment SGI377205[] = {{&S53236[13], 9}, {NULL}};
const Segment SGI377206[] = {{&S53236[33], 15}, {NULL}};
const Segment SGI377207[] = {{&S53236[121], 3}, {NULL}};
const Segment SGI377208[] = {{&S53236[49], 6}, {NULL}};
const Segment SGI377209[] = {{&S53236[33], 13}, {NULL}};
const Segment SGI377210[] = {{&S53236[128], 16}, {NULL}};
const Segment SGI377211[] = {{&S53237[105], 6}, {NULL}};
const Segment SGI377212[] = {{&S53237[8], 3}, {NULL}};
const Segment SGI377213[] = {{&S53237[13], 9}, {NULL}};
const Segment SGI377214[] = {{&S53237[33], 15}, {NULL}};
const Segment SGI377215[] = {{&S53237[121], 3}, {NULL}};
const Segment SGI377216[] = {{&S53237[49], 6}, {NULL}};
const Segment SGI377217[] = {{&S53237[33], 13}, {NULL}};
const Segment SGI377218[] = {{&S53237[128], 16}, {NULL}};
const Segment SGI377219[] = {{&S53238[102], 6}, {NULL}};
const Segment SGI377220[] = {{&S53238[8], 3}, {NULL}};
const Segment SGI377221[] = {{&S53238[13], 9}, {NULL}};
const Segment SGI377222[] = {{&S53238[33], 15}, {NULL}};
const Segment SGI377223[] = {{&S53238[49], 6}, {NULL}};
const Segment SGI377224[] = {{&S53238[33], 13}, {NULL}};
const Segment SGI377225[] = {{&S53238[110], 17}, {NULL}};
const Segment SGI377226[] = {{&S53239[105], 6}, {NULL}};
const Segment SGI377227[] = {{&S53239[8], 3}, {NULL}};
const Segment SGI377228[] = {{&S53239[13], 9}, {NULL}};
const Segment SGI377229[] = {{&S53239[33], 15}, {NULL}};
const Segment SGI377230[] = {{&S53239[123], 3}, {NULL}};
const Segment SGI377231[] = {{&S53239[49], 6}, {NULL}};
const Segment SGI377232[] = {{&S53239[33], 13}, {NULL}};
const Segment SGI377233[] = {{&S53239[130], 16}, {NULL}};
const Segment SGI377234[] = {{&S53240[102], 6}, {NULL}};
const Segment SGI377235[] = {{&S53240[8], 3}, {NULL}};
const Segment SGI377236[] = {{&S53240[13], 9}, {NULL}};
const Segment SGI377237[] = {{&S53240[33], 15}, {NULL}};
const Segment SGI377238[] = {{&S53240[49], 6}, {NULL}};
const Segment SGI377239[] = {{&S53240[33], 13}, {NULL}};
const Segment SGI377240[] = {{&S53240[110], 17}, {NULL}};
const Segment SGI377241[] = {{&S53241[102], 6}, {NULL}};
const Segment SGI377242[] = {{&S53241[8], 3}, {NULL}};
const Segment SGI377243[] = {{&S53241[13], 9}, {NULL}};
const Segment SGI377244[] = {{&S53241[33], 15}, {NULL}};
const Segment SGI377245[] = {{&S53241[49], 6}, {NULL}};
const Segment SGI377246[] = {{&S53241[33], 13}, {NULL}};
const Segment SGI377247[] = {{&S53241[110], 17}, {NULL}};
const Segment SGI377248[] = {{&S53242[105], 6}, {NULL}};
const Segment SGI377249[] = {{&S53242[8], 3}, {NULL}};
const Segment SGI377250[] = {{&S53242[13], 9}, {NULL}};
const Segment SGI377251[] = {{&S53242[33], 15}, {NULL}};
const Segment SGI377252[] = {{&S53242[121], 3}, {NULL}};
const Segment SGI377253[] = {{&S53242[49], 6}, {NULL}};
const Segment SGI377254[] = {{&S53242[33], 13}, {NULL}};
const Segment SGI377255[] = {{&S53242[128], 16}, {NULL}};
const Segment SGI377256[] = {{&S53243[105], 6}, {NULL}};
const Segment SGI377257[] = {{&S53243[8], 3}, {NULL}};
const Segment SGI377258[] = {{&S53243[13], 9}, {NULL}};
const Segment SGI377259[] = {{&S53243[33], 15}, {NULL}};
const Segment SGI377260[] = {{&S53243[123], 3}, {NULL}};
const Segment SGI377261[] = {{&S53243[49], 6}, {NULL}};
const Segment SGI377262[] = {{&S53243[33], 13}, {NULL}};
const Segment SGI377263[] = {{&S53243[130], 16}, {NULL}};
const Segment SGI377264[] = {{&S53244[102], 6}, {NULL}};
const Segment SGI377265[] = {{&S53244[8], 3}, {NULL}};
const Segment SGI377266[] = {{&S53244[13], 9}, {NULL}};
const Segment SGI377267[] = {{&S53244[33], 15}, {NULL}};
const Segment SGI377268[] = {{&S53244[49], 6}, {NULL}};
const Segment SGI377269[] = {{&S53244[33], 13}, {NULL}};
const Segment SGI377270[] = {{&S53244[110], 17}, {NULL}};
const Segment SGI377271[] = {{&S53245[105], 6}, {NULL}};
const Segment SGI377272[] = {{&S53245[8], 3}, {NULL}};
const Segment SGI377273[] = {{&S53245[13], 9}, {NULL}};
const Segment SGI377274[] = {{&S53245[33], 15}, {NULL}};
const Segment SGI377275[] = {{&S53245[121], 3}, {NULL}};
const Segment SGI377276[] = {{&S53245[49], 6}, {NULL}};
const Segment SGI377277[] = {{&S53245[33], 13}, {NULL}};
const Segment SGI377278[] = {{&S53245[128], 16}, {NULL}};
const Segment SGI377279[] = {{&S53246[102], 6}, {NULL}};
const Segment SGI377280[] = {{&S53246[8], 3}, {NULL}};
const Segment SGI377281[] = {{&S53246[13], 9}, {NULL}};
const Segment SGI377282[] = {{&S53246[33], 15}, {NULL}};
const Segment SGI377283[] = {{&S53246[49], 6}, {NULL}};
const Segment SGI377284[] = {{&S53246[33], 13}, {NULL}};
const Segment SGI377285[] = {{&S53246[110], 17}, {NULL}};
const Segment SGI377286[] = {{&S53247[105], 6}, {NULL}};
const Segment SGI377287[] = {{&S53247[8], 3}, {NULL}};
const Segment SGI377288[] = {{&S53247[13], 9}, {NULL}};
const Segment SGI377289[] = {{&S53247[33], 15}, {NULL}};
const Segment SGI377290[] = {{&S53247[123], 3}, {NULL}};
const Segment SGI377291[] = {{&S53247[49], 6}, {NULL}};
const Segment SGI377292[] = {{&S53247[33], 13}, {NULL}};
const Segment SGI377293[] = {{&S53247[130], 18}, {NULL}};
const Segment SGI377294[] = {{&S53248[105], 6}, {NULL}};
const Segment SGI377295[] = {{&S53248[8], 3}, {NULL}};
const Segment SGI377296[] = {{&S53248[13], 9}, {NULL}};
const Segment SGI377297[] = {{&S53248[33], 15}, {NULL}};
const Segment SGI377298[] = {{&S53248[123], 3}, {NULL}};
const Segment SGI377299[] = {{&S53248[49], 6}, {NULL}};
const Segment SGI377300[] = {{&S53248[33], 13}, {NULL}};
const Segment SGI377301[] = {{&S53248[128], 16}, {NULL}};
const Segment SGI377302[] = {{&S53249[105], 6}, {NULL}};
const Segment SGI377303[] = {{&S53249[8], 3}, {NULL}};
const Segment SGI377304[] = {{&S53249[13], 9}, {NULL}};
const Segment SGI377305[] = {{&S53249[33], 15}, {NULL}};
const Segment SGI377306[] = {{&S53249[123], 3}, {NULL}};
const Segment SGI377307[] = {{&S53249[49], 6}, {NULL}};
const Segment SGI377308[] = {{&S53249[33], 13}, {NULL}};
const Segment SGI377309[] = {{&S53249[130], 18}, {NULL}};
const Segment SGI377310[] = {{&S55551[13], 19}, {NULL}};
const Segment SGI377311[] = {{&S55551[46], 6}, {NULL}};
const Segment SGI377312[] = {{&S55552[13], 21}, {NULL}};
const Segment SGI377313[] = {{&S55552[48], 6}, {NULL}};
const Segment SGI377314[] = {{&S53667[108], 6}, {NULL}};
const Segment SGI377315[] = {{&S53667[8], 3}, {NULL}};
const Segment SGI377316[] = {{&S53667[13], 9}, {NULL}};
const Segment SGI377317[] = {{&S53667[35], 16}, {NULL}};
const Segment SGI377318[] = {{&S53667[124], 3}, {NULL}};
const Segment SGI377319[] = {{&S53667[35], 13}, {NULL}};
const Segment SGI377320[] = {{&S53667[131], 16}, {NULL}};
const Segment SGI377321[] = {{&S53668[108], 6}, {NULL}};
const Segment SGI377322[] = {{&S53668[8], 3}, {NULL}};
const Segment SGI377323[] = {{&S53668[13], 9}, {NULL}};
const Segment SGI377324[] = {{&S53668[35], 16}, {NULL}};
const Segment SGI377325[] = {{&S53668[126], 3}, {NULL}};
const Segment SGI377326[] = {{&S53668[35], 13}, {NULL}};
const Segment SGI377327[] = {{&S53668[131], 16}, {NULL}};
const Segment SGI377328[] = {{&S53669[108], 6}, {NULL}};
const Segment SGI377329[] = {{&S53669[8], 3}, {NULL}};
const Segment SGI377330[] = {{&S53669[13], 9}, {NULL}};
const Segment SGI377331[] = {{&S53669[35], 16}, {NULL}};
const Segment SGI377332[] = {{&S53669[126], 3}, {NULL}};
const Segment SGI377333[] = {{&S53669[35], 13}, {NULL}};
const Segment SGI377334[] = {{&S53669[133], 16}, {NULL}};
const Segment SGI377335[] = {{&S53670[105], 6}, {NULL}};
const Segment SGI377336[] = {{&S53670[8], 3}, {NULL}};
const Segment SGI377337[] = {{&S53670[13], 9}, {NULL}};
const Segment SGI377338[] = {{&S53670[35], 16}, {NULL}};
const Segment SGI377339[] = {{&S53670[35], 13}, {NULL}};
const Segment SGI377340[] = {{&S53670[113], 17}, {NULL}};
const Segment SGI377341[] = {{&S53671[108], 6}, {NULL}};
const Segment SGI377342[] = {{&S53671[8], 3}, {NULL}};
const Segment SGI377343[] = {{&S53671[13], 9}, {NULL}};
const Segment SGI377344[] = {{&S53671[35], 16}, {NULL}};
const Segment SGI377345[] = {{&S53671[126], 3}, {NULL}};
const Segment SGI377346[] = {{&S53671[35], 13}, {NULL}};
const Segment SGI377347[] = {{&S53671[133], 18}, {NULL}};
const Segment SGI377348[] = {{&S53672[105], 6}, {NULL}};
const Segment SGI377349[] = {{&S53672[8], 3}, {NULL}};
const Segment SGI377350[] = {{&S53672[13], 9}, {NULL}};
const Segment SGI377351[] = {{&S53672[33], 15}, {NULL}};
const Segment SGI377352[] = {{&S53672[123], 3}, {NULL}};
const Segment SGI377353[] = {{&S53672[49], 6}, {NULL}};
const Segment SGI377354[] = {{&S53672[33], 13}, {NULL}};
const Segment SGI377355[] = {{&S53672[130], 18}, {NULL}};
const Segment SGI377356[] = {{&S53730[42], 6}, {NULL}};
const Segment SGI377357[] = {{&S53730[8], 3}, {NULL}};
const Segment SGI377358[] = {{&S53730[13], 3}, {NULL}};
const Segment SGI377359[] = {{&S53730[79], 3}, {NULL}};
const Segment SGI377360[] = {{&S53785[108], 6}, {NULL}};
const Segment SGI377361[] = {{&S53785[8], 3}, {NULL}};
const Segment SGI377362[] = {{&S53785[13], 7}, {NULL}};
const Segment SGI377363[] = {{&S53785[25], 14}, {NULL}};
const Segment SGI377364[] = {{&S53785[95], 3}, {NULL}};
const Segment SGI377365[] = {{&S53786[60], 6}, {NULL}};
const Segment SGI377366[] = {{&S53786[8], 3}, {NULL}};
const Segment SGI377367[] = {{&S53786[13], 7}, {NULL}};
const Segment SGI377368[] = {{&S53786[25], 14}, {NULL}};
const Segment SGI377369[] = {{&S53786[95], 3}, {NULL}};
const Segment SGI377370[] = {{&S53787[108], 6}, {NULL}};
const Segment SGI377371[] = {{&S53787[8], 3}, {NULL}};
const Segment SGI377372[] = {{&S53787[13], 7}, {NULL}};
const Segment SGI377373[] = {{&S53787[25], 14}, {NULL}};
const Segment SGI377374[] = {{&S53787[95], 3}, {NULL}};
const Segment SGI377375[] = {{&S53801[60], 6}, {NULL}};
const Segment SGI377376[] = {{&S53801[8], 3}, {NULL}};
const Segment SGI377377[] = {{&S53801[13], 7}, {NULL}};
const Segment SGI377378[] = {{&S53801[25], 14}, {NULL}};
const Segment SGI377379[] = {{&S53801[97], 3}, {NULL}};
const Segment SGI377380[] = {{&S53802[110], 6}, {NULL}};
const Segment SGI377381[] = {{&S53802[8], 3}, {NULL}};
const Segment SGI377382[] = {{&S53802[13], 7}, {NULL}};
const Segment SGI377383[] = {{&S53802[25], 14}, {NULL}};
const Segment SGI377384[] = {{&S53802[97], 3}, {NULL}};
const Segment SGI377385[] = {{&S53803[60], 6}, {NULL}};
const Segment SGI377386[] = {{&S53803[8], 3}, {NULL}};
const Segment SGI377387[] = {{&S53803[13], 7}, {NULL}};
const Segment SGI377388[] = {{&S53803[25], 14}, {NULL}};
const Segment SGI377389[] = {{&S53803[95], 3}, {NULL}};
const Segment SGI377390[] = {{&S53804[60], 6}, {NULL}};
const Segment SGI377391[] = {{&S53804[8], 3}, {NULL}};
const Segment SGI377392[] = {{&S53804[13], 7}, {NULL}};
const Segment SGI377393[] = {{&S53804[25], 14}, {NULL}};
const Segment SGI377394[] = {{&S53804[97], 3}, {NULL}};
const Segment SGI377395[] = {{&S53805[60], 5}, {NULL}};
const Segment SGI377396[] = {{&S53805[8], 3}, {NULL}};
const Segment SGI377397[] = {{&S53805[13], 7}, {NULL}};
const Segment SGI377398[] = {{&S53805[25], 14}, {NULL}};
const Segment SGI377399[] = {{&S53805[94], 3}, {NULL}};
const Segment SGI377400[] = {{&S53806[60], 5}, {NULL}};
const Segment SGI377401[] = {{&S53806[8], 3}, {NULL}};
const Segment SGI377402[] = {{&S53806[13], 7}, {NULL}};
const Segment SGI377403[] = {{&S53806[25], 14}, {NULL}};
const Segment SGI377404[] = {{&S53806[94], 3}, {NULL}};
const Segment SGI377405[] = {{&S53807[60], 5}, {NULL}};
const Segment SGI377406[] = {{&S53807[8], 3}, {NULL}};
const Segment SGI377407[] = {{&S53807[13], 7}, {NULL}};
const Segment SGI377408[] = {{&S53807[25], 14}, {NULL}};
const Segment SGI377409[] = {{&S53807[94], 3}, {NULL}};
const Segment SGI377410[] = {{&S53911[60], 5}, {NULL}};
const Segment SGI377411[] = {{&S53911[8], 3}, {NULL}};
const Segment SGI377412[] = {{&S53911[13], 7}, {NULL}};
const Segment SGI377413[] = {{&S53911[25], 14}, {NULL}};
const Segment SGI377414[] = {{&S53911[86], 16}, {NULL}};
const Segment SGI377415[] = {{&S54363[42], 6}, {NULL}};
const Segment SGI377416[] = {{&S54363[8], 3}, {NULL}};
const Segment SGI377417[] = {{&S54363[13], 3}, {NULL}};
const Segment SGI377418[] = {{&S54363[69], 10}, {NULL}};
const Segment SGI377419[] = {{&S54406[67], 5}, {NULL}};
const Segment SGI377420[] = {{&S54406[8], 3}, {NULL}};
const Segment SGI377421[] = {{&S54406[13], 9}, {NULL}};
const Segment SGI377422[] = {{&S54406[33], 13}, {NULL}};
const Segment SGI377423[] = {{&S54406[101], 3}, {NULL}};
const Segment SGI377424[] = {{&S56753[13], 5}, {NULL}};
const Segment SGI377425[] = {{&S56753[32], 6}, {NULL}};
const Segment SGI377426[] = {{&S56753[95], 3}, {NULL}};
const Segment SGI377427[] = {{&S56754[13], 9}, {NULL}};
const Segment SGI377428[] = {{&S56754[36], 6}, {NULL}};
const Segment SGI377429[] = {{&S56754[71], 5}, {NULL}};
const Segment SGI377430[] = {{&S56754[84], 6}, {NULL}};
const Segment SGI377431[] = {{&S54955[60], 5}, {NULL}};
const Segment SGI377432[] = {{&S54955[8], 3}, {NULL}};
const Segment SGI377433[] = {{&S54955[13], 7}, {NULL}};
const Segment SGI377434[] = {{&S54955[25], 14}, {NULL}};
const Segment SGI377435[] = {{&S54955[86], 14}, {NULL}};
const Segment SGI377436[] = {{&S55056[63], 6}, {NULL}};
const Segment SGI377437[] = {{&S55056[8], 3}, {NULL}};
const Segment SGI377438[] = {{&S55056[13], 9}, {NULL}};
const Segment SGI377439[] = {{&S55056[33], 15}, {NULL}};
const Segment SGI377440[] = {{&S55056[100], 3}, {NULL}};
const Segment SGI377441[] = {{&S55056[33], 13}, {NULL}};
const Segment SGI377442[] = {{&S55057[119], 6}, {NULL}};
const Segment SGI377443[] = {{&S55057[8], 3}, {NULL}};
const Segment SGI377444[] = {{&S55057[13], 9}, {NULL}};
const Segment SGI377445[] = {{&S55057[33], 15}, {NULL}};
const Segment SGI377446[] = {{&S55057[106], 3}, {NULL}};
const Segment SGI377447[] = {{&S55057[49], 6}, {NULL}};
const Segment SGI377448[] = {{&S55057[33], 13}, {NULL}};
const Segment SGI377449[] = {{&S55058[119], 6}, {NULL}};
const Segment SGI377450[] = {{&S55058[8], 3}, {NULL}};
const Segment SGI377451[] = {{&S55058[13], 9}, {NULL}};
const Segment SGI377452[] = {{&S55058[33], 15}, {NULL}};
const Segment SGI377453[] = {{&S55058[106], 3}, {NULL}};
const Segment SGI377454[] = {{&S55058[49], 6}, {NULL}};
const Segment SGI377455[] = {{&S55058[33], 13}, {NULL}};
const Segment SGI377456[] = {{&S55059[115], 6}, {NULL}};
const Segment SGI377457[] = {{&S55059[8], 3}, {NULL}};
const Segment SGI377458[] = {{&S55059[13], 9}, {NULL}};
const Segment SGI377459[] = {{&S55059[31], 15}, {NULL}};
const Segment SGI377460[] = {{&S55059[102], 3}, {NULL}};
const Segment SGI377461[] = {{&S55059[47], 6}, {NULL}};
const Segment SGI377462[] = {{&S55059[31], 13}, {NULL}};
const Segment SGI377463[] = {{&S55060[117], 6}, {NULL}};
const Segment SGI377464[] = {{&S55060[8], 3}, {NULL}};
const Segment SGI377465[] = {{&S55060[13], 9}, {NULL}};
const Segment SGI377466[] = {{&S55060[33], 15}, {NULL}};
const Segment SGI377467[] = {{&S55060[104], 3}, {NULL}};
const Segment SGI377468[] = {{&S55060[49], 6}, {NULL}};
const Segment SGI377469[] = {{&S55060[33], 13}, {NULL}};
const Segment SGI377470[] = {{&S55061[119], 6}, {NULL}};
const Segment SGI377471[] = {{&S55061[8], 3}, {NULL}};
const Segment SGI377472[] = {{&S55061[13], 9}, {NULL}};
const Segment SGI377473[] = {{&S55061[33], 15}, {NULL}};
const Segment SGI377474[] = {{&S55061[106], 3}, {NULL}};
const Segment SGI377475[] = {{&S55061[49], 6}, {NULL}};
const Segment SGI377476[] = {{&S55061[33], 13}, {NULL}};
const Segment SGI377477[] = {{&S55062[117], 6}, {NULL}};
const Segment SGI377478[] = {{&S55062[8], 3}, {NULL}};
const Segment SGI377479[] = {{&S55062[13], 9}, {NULL}};
const Segment SGI377480[] = {{&S55062[33], 15}, {NULL}};
const Segment SGI377481[] = {{&S55062[104], 3}, {NULL}};
const Segment SGI377482[] = {{&S55062[49], 6}, {NULL}};
const Segment SGI377483[] = {{&S55062[33], 13}, {NULL}};
const Segment SGI377484[] = {{&S55063[117], 6}, {NULL}};
const Segment SGI377485[] = {{&S55063[8], 3}, {NULL}};
const Segment SGI377486[] = {{&S55063[13], 9}, {NULL}};
const Segment SGI377487[] = {{&S55063[33], 15}, {NULL}};
const Segment SGI377488[] = {{&S55063[104], 3}, {NULL}};
const Segment SGI377489[] = {{&S55063[49], 6}, {NULL}};
const Segment SGI377490[] = {{&S55063[33], 13}, {NULL}};
const Segment SGI377491[] = {{&S55064[119], 6}, {NULL}};
const Segment SGI377492[] = {{&S55064[8], 3}, {NULL}};
const Segment SGI377493[] = {{&S55064[13], 9}, {NULL}};
const Segment SGI377494[] = {{&S55064[33], 15}, {NULL}};
const Segment SGI377495[] = {{&S55064[106], 3}, {NULL}};
const Segment SGI377496[] = {{&S55064[49], 6}, {NULL}};
const Segment SGI377497[] = {{&S55064[33], 13}, {NULL}};
const Segment SGI377498[] = {{&S55065[119], 6}, {NULL}};
const Segment SGI377499[] = {{&S55065[8], 3}, {NULL}};
const Segment SGI377500[] = {{&S55065[13], 9}, {NULL}};
const Segment SGI377501[] = {{&S55065[33], 15}, {NULL}};
const Segment SGI377502[] = {{&S55065[106], 3}, {NULL}};
const Segment SGI377503[] = {{&S55065[49], 6}, {NULL}};
const Segment SGI377504[] = {{&S55065[33], 13}, {NULL}};
const Segment SGI377505[] = {{&S55066[117], 6}, {NULL}};
const Segment SGI377506[] = {{&S55066[8], 3}, {NULL}};
const Segment SGI377507[] = {{&S55066[13], 9}, {NULL}};
const Segment SGI377508[] = {{&S55066[33], 15}, {NULL}};
const Segment SGI377509[] = {{&S55066[104], 3}, {NULL}};
const Segment SGI377510[] = {{&S55066[49], 6}, {NULL}};
const Segment SGI377511[] = {{&S55066[33], 13}, {NULL}};
const Segment SGI377512[] = {{&S55067[119], 6}, {NULL}};
const Segment SGI377513[] = {{&S55067[8], 3}, {NULL}};
const Segment SGI377514[] = {{&S55067[13], 9}, {NULL}};
const Segment SGI377515[] = {{&S55067[33], 15}, {NULL}};
const Segment SGI377516[] = {{&S55067[106], 3}, {NULL}};
const Segment SGI377517[] = {{&S55067[49], 6}, {NULL}};
const Segment SGI377518[] = {{&S55067[33], 13}, {NULL}};
const Segment SGI377519[] = {{&S55068[117], 6}, {NULL}};
const Segment SGI377520[] = {{&S55068[8], 3}, {NULL}};
const Segment SGI377521[] = {{&S55068[13], 9}, {NULL}};
const Segment SGI377522[] = {{&S55068[33], 15}, {NULL}};
const Segment SGI377523[] = {{&S55068[104], 3}, {NULL}};
const Segment SGI377524[] = {{&S55068[49], 6}, {NULL}};
const Segment SGI377525[] = {{&S55068[33], 13}, {NULL}};
const Segment SGI377526[] = {{&S55069[119], 6}, {NULL}};
const Segment SGI377527[] = {{&S55069[8], 3}, {NULL}};
const Segment SGI377528[] = {{&S55069[13], 9}, {NULL}};
const Segment SGI377529[] = {{&S55069[33], 15}, {NULL}};
const Segment SGI377530[] = {{&S55069[106], 3}, {NULL}};
const Segment SGI377531[] = {{&S55069[49], 6}, {NULL}};
const Segment SGI377532[] = {{&S55069[33], 13}, {NULL}};
const Segment SGI377533[] = {{&S55070[115], 6}, {NULL}};
const Segment SGI377534[] = {{&S55070[8], 3}, {NULL}};
const Segment SGI377535[] = {{&S55070[13], 9}, {NULL}};
const Segment SGI377536[] = {{&S55070[31], 15}, {NULL}};
const Segment SGI377537[] = {{&S55070[102], 3}, {NULL}};
const Segment SGI377538[] = {{&S55070[47], 6}, {NULL}};
const Segment SGI377539[] = {{&S55070[31], 13}, {NULL}};
const Segment SGI377540[] = {{&S55071[115], 6}, {NULL}};
const Segment SGI377541[] = {{&S55071[8], 3}, {NULL}};
const Segment SGI377542[] = {{&S55071[13], 9}, {NULL}};
const Segment SGI377543[] = {{&S55071[33], 13}, {NULL}};
const Segment SGI377544[] = {{&S55071[102], 3}, {NULL}};
const Segment SGI377545[] = {{&S55072[117], 6}, {NULL}};
const Segment SGI377546[] = {{&S55072[8], 3}, {NULL}};
const Segment SGI377547[] = {{&S55072[13], 9}, {NULL}};
const Segment SGI377548[] = {{&S55072[33], 15}, {NULL}};
const Segment SGI377549[] = {{&S55072[104], 3}, {NULL}};
const Segment SGI377550[] = {{&S55072[49], 6}, {NULL}};
const Segment SGI377551[] = {{&S55072[33], 13}, {NULL}};
const Segment SGI377552[] = {{&S55073[119], 6}, {NULL}};
const Segment SGI377553[] = {{&S55073[8], 3}, {NULL}};
const Segment SGI377554[] = {{&S55073[13], 9}, {NULL}};
const Segment SGI377555[] = {{&S55073[33], 15}, {NULL}};
const Segment SGI377556[] = {{&S55073[106], 3}, {NULL}};
const Segment SGI377557[] = {{&S55073[49], 6}, {NULL}};
const Segment SGI377558[] = {{&S55073[33], 13}, {NULL}};
const Segment SGI377559[] = {{&S55074[117], 6}, {NULL}};
const Segment SGI377560[] = {{&S55074[8], 3}, {NULL}};
const Segment SGI377561[] = {{&S55074[13], 9}, {NULL}};
const Segment SGI377562[] = {{&S55074[33], 15}, {NULL}};
const Segment SGI377563[] = {{&S55074[104], 3}, {NULL}};
const Segment SGI377564[] = {{&S55074[49], 6}, {NULL}};
const Segment SGI377565[] = {{&S55074[33], 13}, {NULL}};
const Segment SGI377566[] = {{&S55409[48], 6}, {NULL}};
const Segment SGI377567[] = {{&S55409[8], 3}, {NULL}};
const Segment SGI377568[] = {{&S55409[13], 14}, {NULL}};
const Segment SGI377569[] = {{&S55409[85], 3}, {NULL}};
const Segment SGI377570[] = {{&S55410[48], 6}, {NULL}};
const Segment SGI377571[] = {{&S55410[8], 3}, {NULL}};
const Segment SGI377572[] = {{&S55410[13], 14}, {NULL}};
const Segment SGI377573[] = {{&S55411[41], 6}, {NULL}};
const Segment SGI377574[] = {{&S55411[8], 3}, {NULL}};
const Segment SGI377575[] = {{&S55411[13], 14}, {NULL}};
const Segment SGI377576[] = {{&S55411[78], 3}, {NULL}};
const Segment SGI377577[] = {{&S55553[44], 6}, {NULL}};
const Segment SGI377578[] = {{&S55553[8], 3}, {NULL}};
const Segment SGI377579[] = {{&S55553[13], 3}, {NULL}};
const Segment SGI377580[] = {{&S55553[81], 3}, {NULL}};
const Segment SGI377581[] = {{&S55734[48], 6}, {NULL}};
const Segment SGI377582[] = {{&S55734[8], 3}, {NULL}};
const Segment SGI377583[] = {{&S55734[13], 14}, {NULL}};
const Segment SGI377584[] = {{&S55734[75], 6}, {NULL}};
const Segment SGI377585[] = {{&S55735[41], 6}, {NULL}};
const Segment SGI377586[] = {{&S55735[8], 3}, {NULL}};
const Segment SGI377587[] = {{&S55735[13], 14}, {NULL}};
const Segment SGI377588[] = {{&S55735[68], 6}, {NULL}};
const Segment SGI377589[] = {{&S56231[48], 6}, {NULL}};
const Segment SGI377590[] = {{&S56231[8], 3}, {NULL}};
const Segment SGI377591[] = {{&S56231[13], 14}, {NULL}};
const Segment SGI377592[] = {{&S56232[41], 6}, {NULL}};
const Segment SGI377593[] = {{&S56232[8], 3}, {NULL}};
const Segment SGI377594[] = {{&S56232[13], 14}, {NULL}};
const Segment SGI377595[] = {{&S56233[48], 6}, {NULL}};
const Segment SGI377596[] = {{&S56233[8], 3}, {NULL}};
const Segment SGI377597[] = {{&S56233[13], 14}, {NULL}};
const Segment SGI377598[] = {{&S56234[51], 6}, {NULL}};
const Segment SGI377599[] = {{&S56234[8], 3}, {NULL}};
const Segment SGI377600[] = {{&S56234[13], 1}, {NULL}};
const Segment SGI377601[] = {{&S56234[16], 14}, {NULL}};
const Segment SGI377602[] = {{&S56235[41], 6}, {NULL}};
const Segment SGI377603[] = {{&S56235[8], 3}, {NULL}};
const Segment SGI377604[] = {{&S56235[13], 14}, {NULL}};
const Segment SGI377605[] = {{&S56236[48], 6}, {NULL}};
const Segment SGI377606[] = {{&S56236[8], 3}, {NULL}};
const Segment SGI377607[] = {{&S56236[13], 14}, {NULL}};
const Segment SGI377608[] = {{&S56237[48], 6}, {NULL}};
const Segment SGI377609[] = {{&S56237[8], 3}, {NULL}};
const Segment SGI377610[] = {{&S56237[13], 14}, {NULL}};
const Segment SGI377611[] = {{&S56238[41], 6}, {NULL}};
const Segment SGI377612[] = {{&S56238[8], 3}, {NULL}};
const Segment SGI377613[] = {{&S56238[13], 14}, {NULL}};
const Segment SGI377614[] = {{&S56239[48], 6}, {NULL}};
const Segment SGI377615[] = {{&S56239[8], 3}, {NULL}};
const Segment SGI377616[] = {{&S56239[13], 14}, {NULL}};
const Segment SGI377617[] = {{&S56240[48], 6}, {NULL}};
const Segment SGI377618[] = {{&S56240[8], 3}, {NULL}};
const Segment SGI377619[] = {{&S56240[13], 14}, {NULL}};
const Segment SGI377620[] = {{&S56241[48], 6}, {NULL}};
const Segment SGI377621[] = {{&S56241[8], 3}, {NULL}};
const Segment SGI377622[] = {{&S56241[13], 14}, {NULL}};
const Segment SGI377623[] = {{&S56242[41], 6}, {NULL}};
const Segment SGI377624[] = {{&S56242[8], 3}, {NULL}};
const Segment SGI377625[] = {{&S56242[13], 14}, {NULL}};
const Segment SGI377626[] = {{&S56302[60], 5}, {NULL}};
const Segment SGI377627[] = {{&S56302[8], 3}, {NULL}};
const Segment SGI377628[] = {{&S56302[13], 7}, {NULL}};
const Segment SGI377629[] = {{&S56302[25], 14}, {NULL}};
const Segment SGI377630[] = {{&S56302[86], 16}, {NULL}};
const Segment SGI377631[] = {{&S56815[57], 6}, {NULL}};
const Segment SGI377632[] = {{&S56815[8], 3}, {NULL}};
const Segment SGI377633[] = {{&S56815[13], 7}, {NULL}};
const Segment SGI377634[] = {{&S56815[25], 14}, {NULL}};
const Segment SGI377635[] = {{&S56815[94], 3}, {NULL}};
const Segment SGI377636[] = {{&S56816[60], 6}, {NULL}};
const Segment SGI377637[] = {{&S56816[8], 3}, {NULL}};
const Segment SGI377638[] = {{&S56816[13], 7}, {NULL}};
const Segment SGI377639[] = {{&S56816[25], 14}, {NULL}};
const Segment SGI377640[] = {{&S56816[95], 3}, {NULL}};
const Segment SGI377641[] = {{&S56817[57], 6}, {NULL}};
const Segment SGI377642[] = {{&S56817[8], 3}, {NULL}};
const Segment SGI377643[] = {{&S56817[13], 7}, {NULL}};
const Segment SGI377644[] = {{&S56817[25], 14}, {NULL}};
const Segment SGI377645[] = {{&S56817[94], 3}, {NULL}};
const Segment SGI377646[] = {{&S56818[57], 6}, {NULL}};
const Segment SGI377647[] = {{&S56818[8], 3}, {NULL}};
const Segment SGI377648[] = {{&S56818[13], 7}, {NULL}};
const Segment SGI377649[] = {{&S56818[25], 14}, {NULL}};
const Segment SGI377650[] = {{&S56818[94], 3}, {NULL}};
const Segment SGI377651[] = {{&S56819[57], 6}, {NULL}};
const Segment SGI377652[] = {{&S56819[8], 3}, {NULL}};
const Segment SGI377653[] = {{&S56819[13], 7}, {NULL}};
const Segment SGI377654[] = {{&S56819[25], 14}, {NULL}};
const Segment SGI377655[] = {{&S56819[94], 3}, {NULL}};
const Segment SGI377656[] = {{&S56820[103], 6}, {NULL}};
const Segment SGI377657[] = {{&S56820[8], 3}, {NULL}};
const Segment SGI377658[] = {{&S56820[13], 7}, {NULL}};
const Segment SGI377659[] = {{&S56820[25], 14}, {NULL}};
const Segment SGI377660[] = {{&S56820[90], 3}, {NULL}};
const Segment SGI377661[] = {{&S56821[60], 6}, {NULL}};
const Segment SGI377662[] = {{&S56821[8], 3}, {NULL}};
const Segment SGI377663[] = {{&S56821[13], 7}, {NULL}};
const Segment SGI377664[] = {{&S56821[25], 14}, {NULL}};
const Segment SGI377665[] = {{&S56821[97], 3}, {NULL}};
const Segment SGI377666[] = {{&S56822[60], 6}, {NULL}};
const Segment SGI377667[] = {{&S56822[8], 3}, {NULL}};
const Segment SGI377668[] = {{&S56822[13], 7}, {NULL}};
const Segment SGI377669[] = {{&S56822[25], 14}, {NULL}};
const Segment SGI377670[] = {{&S56822[95], 3}, {NULL}};
const Segment SGI377671[] = {{&S57158[92], 5}, {NULL}};
const Segment SGI377672[] = {{&S57158[8], 3}, {NULL}};
const Segment SGI377673[] = {{&S57158[13], 7}, {NULL}};
const Segment SGI377674[] = {{&S57158[25], 14}, {NULL}};
const Segment SGI377675[] = {{&S57159[60], 5}, {NULL}};
const Segment SGI377676[] = {{&S57159[8], 3}, {NULL}};
const Segment SGI377677[] = {{&S57159[13], 7}, {NULL}};
const Segment SGI377678[] = {{&S57159[25], 14}, {NULL}};
const Segment SGI377679[] = {{&S57160[60], 5}, {NULL}};
const Segment SGI377680[] = {{&S57160[8], 3}, {NULL}};
const Segment SGI377681[] = {{&S57160[13], 7}, {NULL}};
const Segment SGI377682[] = {{&S57160[25], 14}, {NULL}};
const Segment SGI377683[] = {{&S57161[60], 5}, {NULL}};
const Segment SGI377684[] = {{&S57161[8], 3}, {NULL}};
const Segment SGI377685[] = {{&S57161[13], 7}, {NULL}};
const Segment SGI377686[] = {{&S57161[25], 14}, {NULL}};
const Segment SGI377687[] = {{&S57162[41], 5}, {NULL}};
const Segment SGI377688[] = {{&S57162[8], 3}, {NULL}};
const Segment SGI377689[] = {{&S57162[13], 14}, {NULL}};
const Segment SGI377690[] = {{&S57163[41], 6}, {NULL}};
const Segment SGI377691[] = {{&S57163[8], 3}, {NULL}};
const Segment SGI377692[] = {{&S57163[13], 14}, {NULL}};
const Segment SGI377693[] = {{&S57164[92], 5}, {NULL}};
const Segment SGI377694[] = {{&S57164[8], 3}, {NULL}};
const Segment SGI377695[] = {{&S57164[13], 7}, {NULL}};
const Segment SGI377696[] = {{&S57164[25], 14}, {NULL}};
const Segment SGI377697[] = {{&S57165[91], 5}, {NULL}};
const Segment SGI377698[] = {{&S57165[8], 3}, {NULL}};
const Segment SGI377699[] = {{&S57165[13], 7}, {NULL}};
const Segment SGI377700[] = {{&S57165[25], 14}, {NULL}};
const Segment SGI377701[] = {{&S57166[48], 6}, {NULL}};
const Segment SGI377702[] = {{&S57166[8], 3}, {NULL}};
const Segment SGI377703[] = {{&S57166[13], 14}, {NULL}};
const Segment SGI377704[] = {{&S57167[44], 6}, {NULL}};
const Segment SGI377705[] = {{&S57167[8], 3}, {NULL}};
const Segment SGI377706[] = {{&S57167[13], 1}, {NULL}};
const Segment SGI377707[] = {{&S57168[41], 6}, {NULL}};
const Segment SGI377708[] = {{&S57168[8], 3}, {NULL}};
const Segment SGI377709[] = {{&S57168[13], 14}, {NULL}};
const Segment SGI377710[] = {{&S57169[41], 6}, {NULL}};
const Segment SGI377711[] = {{&S57169[8], 3}, {NULL}};
const Segment SGI377712[] = {{&S57169[13], 14}, {NULL}};
const Segment SGI377713[] = {{&S57170[41], 6}, {NULL}};
const Segment SGI377714[] = {{&S57170[8], 3}, {NULL}};
const Segment SGI377715[] = {{&S57170[13], 14}, {NULL}};
const Segment SGI377716[] = {{&S57307[89], 6}, {NULL}};
const Segment SGI377717[] = {{&S57307[8], 3}, {NULL}};
const Segment SGI377718[] = {{&S57307[13], 9}, {NULL}};
const Segment SGI377719[] = {{&S57307[33], 15}, {NULL}};
const Segment SGI377720[] = {{&S57307[49], 19}, {NULL}};
const Segment SGI377721[] = {{&S57307[33], 13}, {NULL}};
const Segment SGI377722[] = {{&S57308[93], 6}, {NULL}};
const Segment SGI377723[] = {{&S57308[8], 3}, {NULL}};
const Segment SGI377724[] = {{&S57308[13], 9}, {NULL}};
const Segment SGI377725[] = {{&S57308[33], 15}, {NULL}};
const Segment SGI377726[] = {{&S57308[49], 23}, {NULL}};
const Segment SGI377727[] = {{&S57308[33], 13}, {NULL}};
const Segment SGI377728[] = {{&S57309[84], 6}, {NULL}};
const Segment SGI377729[] = {{&S57309[8], 3}, {NULL}};
const Segment SGI377730[] = {{&S57309[13], 9}, {NULL}};
const Segment SGI377731[] = {{&S57309[33], 15}, {NULL}};
const Segment SGI377732[] = {{&S57309[49], 14}, {NULL}};
const Segment SGI377733[] = {{&S57309[33], 13}, {NULL}};
const Segment SGI377734[] = {{&S57328[60], 5}, {NULL}};
const Segment SGI377735[] = {{&S57328[8], 3}, {NULL}};
const Segment SGI377736[] = {{&S57328[13], 7}, {NULL}};
const Segment SGI377737[] = {{&S57328[25], 14}, {NULL}};
const Segment SGI377738[] = {{&S57328[86], 16}, {NULL}};
const Segment SGI377739[] = {{&S57361[60], 5}, {NULL}};
const Segment SGI377740[] = {{&S57361[8], 3}, {NULL}};
const Segment SGI377741[] = {{&S57361[13], 7}, {NULL}};
const Segment SGI377742[] = {{&S57361[25], 14}, {NULL}};
const Segment SGI377743[] = {{&S57361[86], 14}, {NULL}};
const Segment SGI377744[] = {{&S57417[105], 6}, {NULL}};
const Segment SGI377745[] = {{&S57417[8], 3}, {NULL}};
const Segment SGI377746[] = {{&S57417[13], 9}, {NULL}};
const Segment SGI377747[] = {{&S57417[33], 15}, {NULL}};
const Segment SGI377748[] = {{&S57417[123], 3}, {NULL}};
const Segment SGI377749[] = {{&S57417[49], 6}, {NULL}};
const Segment SGI377750[] = {{&S57417[33], 13}, {NULL}};
const Segment SGI377751[] = {{&S57417[128], 16}, {NULL}};
const Segment SGI377752[] = {{&S57418[105], 6}, {NULL}};
const Segment SGI377753[] = {{&S57418[8], 3}, {NULL}};
const Segment SGI377754[] = {{&S57418[13], 9}, {NULL}};
const Segment SGI377755[] = {{&S57418[33], 15}, {NULL}};
const Segment SGI377756[] = {{&S57418[123], 3}, {NULL}};
const Segment SGI377757[] = {{&S57418[49], 6}, {NULL}};
const Segment SGI377758[] = {{&S57418[33], 13}, {NULL}};
const Segment SGI377759[] = {{&S57418[128], 16}, {NULL}};
const Segment SGI377760[] = {{&S57419[105], 6}, {NULL}};
const Segment SGI377761[] = {{&S57419[8], 3}, {NULL}};
const Segment SGI377762[] = {{&S57419[13], 9}, {NULL}};
const Segment SGI377763[] = {{&S57419[33], 15}, {NULL}};
const Segment SGI377764[] = {{&S57419[123], 3}, {NULL}};
const Segment SGI377765[] = {{&S57419[49], 6}, {NULL}};
const Segment SGI377766[] = {{&S57419[33], 13}, {NULL}};
const Segment SGI377767[] = {{&S57419[130], 16}, {NULL}};
const Segment SGI377768[] = {{&S57420[105], 6}, {NULL}};
const Segment SGI377769[] = {{&S57420[8], 3}, {NULL}};
const Segment SGI377770[] = {{&S57420[13], 9}, {NULL}};
const Segment SGI377771[] = {{&S57420[33], 15}, {NULL}};
const Segment SGI377772[] = {{&S57420[123], 3}, {NULL}};
const Segment SGI377773[] = {{&S57420[49], 6}, {NULL}};
const Segment SGI377774[] = {{&S57420[33], 13}, {NULL}};
const Segment SGI377775[] = {{&S57420[130], 16}, {NULL}};
const Segment SGI377776[] = {{&S57421[105], 6}, {NULL}};
const Segment SGI377777[] = {{&S57421[8], 3}, {NULL}};
const Segment SGI377778[] = {{&S57421[13], 9}, {NULL}};
const Segment SGI377779[] = {{&S57421[33], 15}, {NULL}};
const Segment SGI377780[] = {{&S57421[123], 3}, {NULL}};
const Segment SGI377781[] = {{&S57421[49], 6}, {NULL}};
const Segment SGI377782[] = {{&S57421[33], 13}, {NULL}};
const Segment SGI377783[] = {{&S57421[130], 16}, {NULL}};
const Segment SGI377784[] = {{&S57422[105], 6}, {NULL}};
const Segment SGI377785[] = {{&S57422[8], 3}, {NULL}};
const Segment SGI377786[] = {{&S57422[13], 9}, {NULL}};
const Segment SGI377787[] = {{&S57422[33], 15}, {NULL}};
const Segment SGI377788[] = {{&S57422[123], 3}, {NULL}};
const Segment SGI377789[] = {{&S57422[49], 6}, {NULL}};
const Segment SGI377790[] = {{&S57422[33], 13}, {NULL}};
const Segment SGI377791[] = {{&S57422[128], 16}, {NULL}};
const Segment SGI377792[] = {{&S57423[105], 6}, {NULL}};
const Segment SGI377793[] = {{&S57423[8], 3}, {NULL}};
const Segment SGI377794[] = {{&S57423[13], 9}, {NULL}};
const Segment SGI377795[] = {{&S57423[33], 15}, {NULL}};
const Segment SGI377796[] = {{&S57423[123], 3}, {NULL}};
const Segment SGI377797[] = {{&S57423[49], 6}, {NULL}};
const Segment SGI377798[] = {{&S57423[33], 13}, {NULL}};
const Segment SGI377799[] = {{&S57423[128], 16}, {NULL}};
const Segment SGI377800[] = {{&S57424[102], 6}, {NULL}};
const Segment SGI377801[] = {{&S57424[8], 3}, {NULL}};
const Segment SGI377802[] = {{&S57424[13], 9}, {NULL}};
const Segment SGI377803[] = {{&S57424[33], 15}, {NULL}};
const Segment SGI377804[] = {{&S57424[49], 6}, {NULL}};
const Segment SGI377805[] = {{&S57424[33], 13}, {NULL}};
const Segment SGI377806[] = {{&S57424[110], 17}, {NULL}};
const Segment SGI377807[] = {{&S57425[105], 6}, {NULL}};
const Segment SGI377808[] = {{&S57425[8], 3}, {NULL}};
const Segment SGI377809[] = {{&S57425[13], 9}, {NULL}};
const Segment SGI377810[] = {{&S57425[33], 15}, {NULL}};
const Segment SGI377811[] = {{&S57425[123], 3}, {NULL}};
const Segment SGI377812[] = {{&S57425[49], 6}, {NULL}};
const Segment SGI377813[] = {{&S57425[33], 13}, {NULL}};
const Segment SGI377814[] = {{&S57425[130], 18}, {NULL}};
const Segment SGI377815[] = {{&S57426[105], 6}, {NULL}};
const Segment SGI377816[] = {{&S57426[8], 3}, {NULL}};
const Segment SGI377817[] = {{&S57426[13], 9}, {NULL}};
const Segment SGI377818[] = {{&S57426[33], 15}, {NULL}};
const Segment SGI377819[] = {{&S57426[121], 3}, {NULL}};
const Segment SGI377820[] = {{&S57426[49], 6}, {NULL}};
const Segment SGI377821[] = {{&S57426[33], 13}, {NULL}};
const Segment SGI377822[] = {{&S57426[128], 16}, {NULL}};
const Segment SGI377823[] = {{&S57427[105], 6}, {NULL}};
const Segment SGI377824[] = {{&S57427[8], 3}, {NULL}};
const Segment SGI377825[] = {{&S57427[13], 9}, {NULL}};
const Segment SGI377826[] = {{&S57427[33], 15}, {NULL}};
const Segment SGI377827[] = {{&S57427[123], 3}, {NULL}};
const Segment SGI377828[] = {{&S57427[49], 6}, {NULL}};
const Segment SGI377829[] = {{&S57427[33], 13}, {NULL}};
const Segment SGI377830[] = {{&S57427[130], 16}, {NULL}};
const Segment SGI377831[] = {{&S57428[105], 6}, {NULL}};
const Segment SGI377832[] = {{&S57428[8], 3}, {NULL}};
const Segment SGI377833[] = {{&S57428[13], 9}, {NULL}};
const Segment SGI377834[] = {{&S57428[33], 15}, {NULL}};
const Segment SGI377835[] = {{&S57428[123], 3}, {NULL}};
const Segment SGI377836[] = {{&S57428[49], 6}, {NULL}};
const Segment SGI377837[] = {{&S57428[33], 13}, {NULL}};
const Segment SGI377838[] = {{&S57428[130], 18}, {NULL}};
const Segment SGI377839[] = {{&S57429[105], 6}, {NULL}};
const Segment SGI377840[] = {{&S57429[8], 3}, {NULL}};
const Segment SGI377841[] = {{&S57429[13], 9}, {NULL}};
const Segment SGI377842[] = {{&S57429[33], 15}, {NULL}};
const Segment SGI377843[] = {{&S57429[123], 3}, {NULL}};
const Segment SGI377844[] = {{&S57429[49], 6}, {NULL}};
const Segment SGI377845[] = {{&S57429[33], 13}, {NULL}};
const Segment SGI377846[] = {{&S57429[128], 16}, {NULL}};
const Segment SGI377847[] = {{&S57430[102], 6}, {NULL}};
const Segment SGI377848[] = {{&S57430[8], 3}, {NULL}};
const Segment SGI377849[] = {{&S57430[13], 9}, {NULL}};
const Segment SGI377850[] = {{&S57430[33], 15}, {NULL}};
const Segment SGI377851[] = {{&S57430[49], 6}, {NULL}};
const Segment SGI377852[] = {{&S57430[33], 13}, {NULL}};
const Segment SGI377853[] = {{&S57430[110], 17}, {NULL}};
const Segment SGI377854[] = {{&S57431[105], 6}, {NULL}};
const Segment SGI377855[] = {{&S57431[8], 3}, {NULL}};
const Segment SGI377856[] = {{&S57431[13], 9}, {NULL}};
const Segment SGI377857[] = {{&S57431[33], 15}, {NULL}};
const Segment SGI377858[] = {{&S57431[121], 3}, {NULL}};
const Segment SGI377859[] = {{&S57431[49], 6}, {NULL}};
const Segment SGI377860[] = {{&S57431[33], 13}, {NULL}};
const Segment SGI377861[] = {{&S57431[128], 16}, {NULL}};
const Segment SGI377862[] = {{&S57661[57], 6}, {NULL}};
const Segment SGI377863[] = {{&S57661[8], 3}, {NULL}};
const Segment SGI377864[] = {{&S57661[13], 9}, {NULL}};
const Segment SGI377865[] = {{&S57661[94], 3}, {NULL}};
const Segment SGI377866[] = {{&S57661[30], 13}, {NULL}};
const Segment SGI377867[] = {{&S57662[57], 6}, {NULL}};
const Segment SGI377868[] = {{&S57662[8], 3}, {NULL}};
const Segment SGI377869[] = {{&S57662[13], 9}, {NULL}};
const Segment SGI377870[] = {{&S57662[94], 3}, {NULL}};
const Segment SGI377871[] = {{&S57662[30], 13}, {NULL}};
const Segment SGI377872[] = {{&S57663[57], 6}, {NULL}};
const Segment SGI377873[] = {{&S57663[8], 3}, {NULL}};
const Segment SGI377874[] = {{&S57663[13], 9}, {NULL}};
const Segment SGI377875[] = {{&S57663[94], 3}, {NULL}};
const Segment SGI377876[] = {{&S57663[30], 13}, {NULL}};
const Segment SGI377877[] = {{&S57664[57], 6}, {NULL}};
const Segment SGI377878[] = {{&S57664[8], 3}, {NULL}};
const Segment SGI377879[] = {{&S57664[13], 9}, {NULL}};
const Segment SGI377880[] = {{&S57664[92], 3}, {NULL}};
const Segment SGI377881[] = {{&S57664[30], 13}, {NULL}};
const Segment SGI377882[] = {{&S57665[57], 6}, {NULL}};
const Segment SGI377883[] = {{&S57665[8], 3}, {NULL}};
const Segment SGI377884[] = {{&S57665[13], 9}, {NULL}};
const Segment SGI377885[] = {{&S57665[95], 3}, {NULL}};
const Segment SGI377886[] = {{&S57665[30], 13}, {NULL}};
const Segment SGI377887[] = {{&S57666[57], 6}, {NULL}};
const Segment SGI377888[] = {{&S57666[8], 3}, {NULL}};
const Segment SGI377889[] = {{&S57666[13], 9}, {NULL}};
const Segment SGI377890[] = {{&S57666[92], 3}, {NULL}};
const Segment SGI377891[] = {{&S57666[30], 13}, {NULL}};
const Segment SGI377892[] = {{&S57667[57], 6}, {NULL}};
const Segment SGI377893[] = {{&S57667[8], 3}, {NULL}};
const Segment SGI377894[] = {{&S57667[13], 9}, {NULL}};
const Segment SGI377895[] = {{&S57667[92], 3}, {NULL}};
const Segment SGI377896[] = {{&S57667[30], 13}, {NULL}};
const Segment SGI377897[] = {{&S57668[57], 6}, {NULL}};
const Segment SGI377898[] = {{&S57668[8], 3}, {NULL}};
const Segment SGI377899[] = {{&S57668[13], 9}, {NULL}};
const Segment SGI377900[] = {{&S57668[92], 3}, {NULL}};
const Segment SGI377901[] = {{&S57668[30], 13}, {NULL}};
const Segment SGI377902[] = {{&S57669[57], 6}, {NULL}};
const Segment SGI377903[] = {{&S57669[8], 3}, {NULL}};
const Segment SGI377904[] = {{&S57669[13], 9}, {NULL}};
const Segment SGI377905[] = {{&S57669[92], 3}, {NULL}};
const Segment SGI377906[] = {{&S57669[30], 13}, {NULL}};
const Segment SGI377907[] = {{&S57670[57], 6}, {NULL}};
const Segment SGI377908[] = {{&S57670[8], 3}, {NULL}};
const Segment SGI377909[] = {{&S57670[13], 9}, {NULL}};
const Segment SGI377910[] = {{&S57670[95], 3}, {NULL}};
const Segment SGI377911[] = {{&S57670[30], 13}, {NULL}};
const Segment SGI377912[] = {{&S57671[57], 6}, {NULL}};
const Segment SGI377913[] = {{&S57671[8], 3}, {NULL}};
const Segment SGI377914[] = {{&S57671[13], 9}, {NULL}};
const Segment SGI377915[] = {{&S57671[92], 3}, {NULL}};
const Segment SGI377916[] = {{&S57671[30], 13}, {NULL}};
const Segment SGI377917[] = {{&S57672[57], 6}, {NULL}};
const Segment SGI377918[] = {{&S57672[8], 3}, {NULL}};
const Segment SGI377919[] = {{&S57672[13], 9}, {NULL}};
const Segment SGI377920[] = {{&S57672[94], 3}, {NULL}};
const Segment SGI377921[] = {{&S57672[30], 13}, {NULL}};
const Segment SGI377922[] = {{&S57673[57], 6}, {NULL}};
const Segment SGI377923[] = {{&S57673[8], 3}, {NULL}};
const Segment SGI377924[] = {{&S57673[13], 9}, {NULL}};
const Segment SGI377925[] = {{&S57673[92], 3}, {NULL}};
const Segment SGI377926[] = {{&S57673[30], 13}, {NULL}};
const Segment SGI377927[] = {{&S57674[57], 6}, {NULL}};
const Segment SGI377928[] = {{&S57674[8], 3}, {NULL}};
const Segment SGI377929[] = {{&S57674[13], 9}, {NULL}};
const Segment SGI377930[] = {{&S57674[92], 3}, {NULL}};
const Segment SGI377931[] = {{&S57674[30], 13}, {NULL}};
const Segment SGI377932[] = {{&S57675[57], 6}, {NULL}};
const Segment SGI377933[] = {{&S57675[8], 3}, {NULL}};
const Segment SGI377934[] = {{&S57675[13], 9}, {NULL}};
const Segment SGI377935[] = {{&S57675[94], 3}, {NULL}};
const Segment SGI377936[] = {{&S57675[30], 13}, {NULL}};
const Segment SGI377937[] = {{&S57676[57], 6}, {NULL}};
const Segment SGI377938[] = {{&S57676[8], 3}, {NULL}};
const Segment SGI377939[] = {{&S57676[13], 9}, {NULL}};
const Segment SGI377940[] = {{&S57676[92], 3}, {NULL}};
const Segment SGI377941[] = {{&S57676[30], 13}, {NULL}};
const Segment SGI377942[] = {{&S59799[13], 102}, {NULL}};
const Segment SGI377943[] = {{&S59799[129], 3}, {NULL}};
const Segment SGI377944[] = {{&S59799[161], 3}, {NULL}};
const Segment SGI377945[] = {{&S59800[13], 102}, {NULL}};
const Segment SGI377946[] = {{&S59800[129], 3}, {NULL}};
const Segment SGI377947[] = {{&S59800[161], 3}, {NULL}};
const Segment SGI377948[] = {{&S59802[13], 102}, {NULL}};
const Segment SGI377949[] = {{&S59802[129], 3}, {NULL}};
const Segment SGI377950[] = {{&S59802[161], 3}, {NULL}};
const Segment SGI377951[] = {{&S59803[13], 102}, {NULL}};
const Segment SGI377952[] = {{&S59803[129], 3}, {NULL}};
const Segment SGI377953[] = {{&S59803[161], 3}, {NULL}};
const Segment SGI377954[] = {{&S59804[13], 102}, {NULL}};
const Segment SGI377955[] = {{&S59804[129], 3}, {NULL}};
const Segment SGI377956[] = {{&S59804[161], 3}, {NULL}};
const Segment SGI377957[] = {{&S59805[13], 86}, {NULL}};
const Segment SGI377958[] = {{&S59805[113], 3}, {NULL}};
const Segment SGI377959[] = {{&S59805[145], 3}, {NULL}};
const Segment SGI377960[] = {{&S57827[110], 6}, {NULL}};
const Segment SGI377961[] = {{&S57827[8], 3}, {NULL}};
const Segment SGI377962[] = {{&S57827[13], 9}, {NULL}};
const Segment SGI377963[] = {{&S57827[97], 3}, {NULL}};
const Segment SGI377964[] = {{&S57827[30], 13}, {NULL}};
const Segment SGI377965[] = {{&S57828[59], 6}, {NULL}};
const Segment SGI377966[] = {{&S57828[8], 3}, {NULL}};
const Segment SGI377967[] = {{&S57828[13], 9}, {NULL}};
const Segment SGI377968[] = {{&S57828[97], 3}, {NULL}};
const Segment SGI377969[] = {{&S57828[30], 13}, {NULL}};
const Segment SGI377970[] = {{&S57829[69], 6}, {NULL}};
const Segment SGI377971[] = {{&S57829[8], 3}, {NULL}};
const Segment SGI377972[] = {{&S57829[13], 9}, {NULL}};
const Segment SGI377973[] = {{&S57829[33], 15}, {NULL}};
const Segment SGI377974[] = {{&S57829[106], 3}, {NULL}};
const Segment SGI377975[] = {{&S57829[49], 6}, {NULL}};
const Segment SGI377976[] = {{&S57829[33], 13}, {NULL}};
const Segment SGI377977[] = {{&S57830[110], 6}, {NULL}};
const Segment SGI377978[] = {{&S57830[8], 3}, {NULL}};
const Segment SGI377979[] = {{&S57830[13], 9}, {NULL}};
const Segment SGI377980[] = {{&S57830[97], 3}, {NULL}};
const Segment SGI377981[] = {{&S57830[30], 13}, {NULL}};
const Segment SGI377982[] = {{&S57831[110], 6}, {NULL}};
const Segment SGI377983[] = {{&S57831[8], 3}, {NULL}};
const Segment SGI377984[] = {{&S57831[13], 9}, {NULL}};
const Segment SGI377985[] = {{&S57831[97], 3}, {NULL}};
const Segment SGI377986[] = {{&S57831[30], 13}, {NULL}};
const Segment SGI377987[] = {{&S57832[59], 6}, {NULL}};
const Segment SGI377988[] = {{&S57832[8], 3}, {NULL}};
const Segment SGI377989[] = {{&S57832[13], 9}, {NULL}};
const Segment SGI377990[] = {{&S57832[97], 3}, {NULL}};
const Segment SGI377991[] = {{&S57832[30], 13}, {NULL}};
const Segment SGI377992[] = {{&S57833[59], 6}, {NULL}};
const Segment SGI377993[] = {{&S57833[8], 3}, {NULL}};
const Segment SGI377994[] = {{&S57833[13], 9}, {NULL}};
const Segment SGI377995[] = {{&S57833[96], 3}, {NULL}};
const Segment SGI377996[] = {{&S57833[30], 13}, {NULL}};
const Segment SGI377997[] = {{&S57834[69], 6}, {NULL}};
const Segment SGI377998[] = {{&S57834[8], 3}, {NULL}};
const Segment SGI377999[] = {{&S57834[13], 9}, {NULL}};
const Segment SGI378000[] = {{&S57834[33], 15}, {NULL}};
const Segment SGI378001[] = {{&S57834[106], 3}, {NULL}};
const Segment SGI378002[] = {{&S57834[49], 6}, {NULL}};
const Segment SGI378003[] = {{&S57834[33], 13}, {NULL}};
const Segment SGI378004[] = {{&S57835[109], 6}, {NULL}};
const Segment SGI378005[] = {{&S57835[8], 3}, {NULL}};
const Segment SGI378006[] = {{&S57835[13], 9}, {NULL}};
const Segment SGI378007[] = {{&S57835[96], 3}, {NULL}};
const Segment SGI378008[] = {{&S57835[30], 13}, {NULL}};
const Segment SGI378009[] = {{&S57899[59], 6}, {NULL}};
const Segment SGI378010[] = {{&S57899[8], 3}, {NULL}};
const Segment SGI378011[] = {{&S57899[13], 9}, {NULL}};
const Segment SGI378012[] = {{&S57900[59], 6}, {NULL}};
const Segment SGI378013[] = {{&S57900[8], 3}, {NULL}};
const Segment SGI378014[] = {{&S57900[13], 9}, {NULL}};
const Segment SGI378015[] = {{&S57901[62], 6}, {NULL}};
const Segment SGI378016[] = {{&S57901[8], 3}, {NULL}};
const Segment SGI378017[] = {{&S57901[13], 9}, {NULL}};
const Segment SGI378018[] = {{&S57901[123], 3}, {NULL}};
const Segment SGI378019[] = {{&S57902[59], 6}, {NULL}};
const Segment SGI378020[] = {{&S57902[8], 3}, {NULL}};
const Segment SGI378021[] = {{&S57902[13], 9}, {NULL}};
const Segment SGI378022[] = {{&S57903[59], 6}, {NULL}};
const Segment SGI378023[] = {{&S57903[8], 3}, {NULL}};
const Segment SGI378024[] = {{&S57903[13], 9}, {NULL}};
const Segment SGI378025[] = {{&S57904[59], 6}, {NULL}};
const Segment SGI378026[] = {{&S57904[8], 3}, {NULL}};
const Segment SGI378027[] = {{&S57904[13], 9}, {NULL}};
const Segment SGI378028[] = {{&S57905[59], 6}, {NULL}};
const Segment SGI378029[] = {{&S57905[8], 3}, {NULL}};
const Segment SGI378030[] = {{&S57905[13], 9}, {NULL}};
const Segment SGI378031[] = {{&S57906[59], 6}, {NULL}};
const Segment SGI378032[] = {{&S57906[8], 3}, {NULL}};
const Segment SGI378033[] = {{&S57906[13], 9}, {NULL}};
const Segment SGI378034[] = {{&S57907[59], 6}, {NULL}};
const Segment SGI378035[] = {{&S57907[8], 3}, {NULL}};
const Segment SGI378036[] = {{&S57907[13], 9}, {NULL}};
const Segment SGI378037[] = {{&S57908[59], 6}, {NULL}};
const Segment SGI378038[] = {{&S57908[8], 3}, {NULL}};
const Segment SGI378039[] = {{&S57908[13], 9}, {NULL}};
const Segment SGI378040[] = {{&S57909[59], 6}, {NULL}};
const Segment SGI378041[] = {{&S57909[8], 3}, {NULL}};
const Segment SGI378042[] = {{&S57909[13], 9}, {NULL}};
const Segment SGI378043[] = {{&S57910[59], 6}, {NULL}};
const Segment SGI378044[] = {{&S57910[8], 3}, {NULL}};
const Segment SGI378045[] = {{&S57910[13], 9}, {NULL}};
const Segment SGI378046[] = {{&S58025[44], 6}, {NULL}};
const Segment SGI378047[] = {{&S58025[8], 3}, {NULL}};
const Segment SGI378048[] = {{&S58025[13], 3}, {NULL}};
const Segment SGI378049[] = {{&S58025[71], 10}, {NULL}};
const Segment SGI378050[] = {{&S58026[44], 6}, {NULL}};
const Segment SGI378051[] = {{&S58026[8], 3}, {NULL}};
const Segment SGI378052[] = {{&S58026[13], 3}, {NULL}};
const Segment SGI378053[] = {{&S58026[71], 10}, {NULL}};
const Segment SGI378054[] = {{&S58252[50], 6}, {NULL}};
const Segment SGI378055[] = {{&S58252[8], 3}, {NULL}};
const Segment SGI378056[] = {{&S58252[13], 4}, {NULL}};
const Segment SGI378057[] = {{&S58252[116], 3}, {NULL}};
const Segment SGI378058[] = {{&S58371[60], 5}, {NULL}};
const Segment SGI378059[] = {{&S58371[8], 3}, {NULL}};
const Segment SGI378060[] = {{&S58371[13], 7}, {NULL}};
const Segment SGI378061[] = {{&S58371[25], 14}, {NULL}};
const Segment SGI378062[] = {{&S58371[86], 16}, {NULL}};
const Segment SGI378063[] = {{&S59155[92], 5}, {NULL}};
const Segment SGI378064[] = {{&S59155[8], 3}, {NULL}};
const Segment SGI378065[] = {{&S59155[13], 7}, {NULL}};
const Segment SGI378066[] = {{&S59155[25], 14}, {NULL}};
const Segment SGI378067[] = {{&S59155[99], 6}, {NULL}};
const Segment SGI378068[] = {{&S59156[92], 5}, {NULL}};
const Segment SGI378069[] = {{&S59156[8], 3}, {NULL}};
const Segment SGI378070[] = {{&S59156[13], 7}, {NULL}};
const Segment SGI378071[] = {{&S59156[25], 14}, {NULL}};
const Segment SGI378072[] = {{&S59156[99], 6}, {NULL}};
const Segment SGI378073[] = {{&S59157[94], 5}, {NULL}};
const Segment SGI378074[] = {{&S59157[8], 3}, {NULL}};
const Segment SGI378075[] = {{&S59157[13], 7}, {NULL}};
const Segment SGI378076[] = {{&S59157[25], 14}, {NULL}};
const Segment SGI378077[] = {{&S59157[101], 6}, {NULL}};
const Segment SGI378078[] = {{&S59158[60], 5}, {NULL}};
const Segment SGI378079[] = {{&S59158[8], 3}, {NULL}};
const Segment SGI378080[] = {{&S59158[13], 7}, {NULL}};
const Segment SGI378081[] = {{&S59158[25], 14}, {NULL}};
const Segment SGI378082[] = {{&S59158[86], 6}, {NULL}};
const Segment SGI378083[] = {{&S59546[60], 5}, {NULL}};
const Segment SGI378084[] = {{&S59546[8], 3}, {NULL}};
const Segment SGI378085[] = {{&S59546[13], 7}, {NULL}};
const Segment SGI378086[] = {{&S59546[25], 14}, {NULL}};
const Segment SGI378087[] = {{&S59546[86], 14}, {NULL}};
const Segment SGI378088[] = {{&S60366[62], 5}, {NULL}};
const Segment SGI378089[] = {{&S60366[8], 3}, {NULL}};
const Segment SGI378090[] = {{&S60366[13], 9}, {NULL}};
const Segment SGI378091[] = {{&S60366[88], 16}, {NULL}};
const Segment SGI378092[] = {{&S63315[13], 56}, {NULL}};
const Segment SGI378093[] = {{&S63315[83], 5}, {NULL}};
const Segment SGI378094[] = {{&S63315[116], 3}, {NULL}};
const Segment SGI378095[] = {{&S63316[13], 56}, {NULL}};
const Segment SGI378096[] = {{&S63316[83], 6}, {NULL}};
const Segment SGI378097[] = {{&S63316[117], 3}, {NULL}};
const Segment SGI378098[] = {{&S63317[13], 15}, {NULL}};
const Segment SGI378099[] = {{&S63317[42], 6}, {NULL}};
const Segment SGI378100[] = {{&S63317[77], 3}, {NULL}};
const Segment SGI378101[] = {{&S61606[79], 5}, {NULL}};
const Segment SGI378102[] = {{&S61606[8], 3}, {NULL}};
const Segment SGI378103[] = {{&S61606[13], 3}, {NULL}};
const Segment SGI378104[] = {{&S61606[86], 10}, {NULL}};
const Segment SGI378105[] = {{&S61995[46], 5}, {NULL}};
const Segment SGI378106[] = {{&S61995[8], 3}, {NULL}};
const Segment SGI378107[] = {{&S61995[13], 7}, {NULL}};
const Segment SGI378108[] = {{&S61995[72], 16}, {NULL}};
const Segment SGI378109[] = {{&S62071[60], 5}, {NULL}};
const Segment SGI378110[] = {{&S62071[8], 3}, {NULL}};
const Segment SGI378111[] = {{&S62071[13], 7}, {NULL}};
const Segment SGI378112[] = {{&S62071[25], 14}, {NULL}};
const Segment SGI378113[] = {{&S62071[91], 3}, {NULL}};
const Segment SGI378114[] = {{&S62099[42], 6}, {NULL}};
const Segment SGI378115[] = {{&S62099[8], 3}, {NULL}};
const Segment SGI378116[] = {{&S62099[13], 3}, {NULL}};
const Segment SGI378117[] = {{&S62099[69], 12}, {NULL}};
const Segment SGI378118[] = {{&S62100[44], 6}, {NULL}};
const Segment SGI378119[] = {{&S62100[8], 3}, {NULL}};
const Segment SGI378120[] = {{&S62100[13], 3}, {NULL}};
const Segment SGI378121[] = {{&S62100[71], 12}, {NULL}};
const Segment SGI378122[] = {{&S62180[69], 6}, {NULL}};
const Segment SGI378123[] = {{&S62180[8], 3}, {NULL}};
const Segment SGI378124[] = {{&S62180[13], 9}, {NULL}};
const Segment SGI378125[] = {{&S62180[33], 15}, {NULL}};
const Segment SGI378126[] = {{&S62180[104], 3}, {NULL}};
const Segment SGI378127[] = {{&S62180[49], 6}, {NULL}};
const Segment SGI378128[] = {{&S62180[33], 13}, {NULL}};
const Segment SGI378129[] = {{&S62180[123], 7}, {NULL}};
const Segment SGI378130[] = {{&S62181[116], 6}, {NULL}};
const Segment SGI378131[] = {{&S62181[8], 3}, {NULL}};
const Segment SGI378132[] = {{&S62181[13], 9}, {NULL}};
const Segment SGI378133[] = {{&S62181[33], 15}, {NULL}};
const Segment SGI378134[] = {{&S62181[103], 3}, {NULL}};
const Segment SGI378135[] = {{&S62181[49], 6}, {NULL}};
const Segment SGI378136[] = {{&S62181[33], 13}, {NULL}};
const Segment SGI378137[] = {{&S62181[126], 7}, {NULL}};
const Segment SGI378138[] = {{&S62267[78], 5}, {NULL}};
const Segment SGI378139[] = {{&S62267[8], 3}, {NULL}};
const Segment SGI378140[] = {{&S62267[13], 7}, {NULL}};
const Segment SGI378141[] = {{&S62268[78], 5}, {NULL}};
const Segment SGI378142[] = {{&S62268[8], 3}, {NULL}};
const Segment SGI378143[] = {{&S62268[13], 7}, {NULL}};
const Segment SGI378144[] = {{&S62269[78], 5}, {NULL}};
const Segment SGI378145[] = {{&S62269[8], 3}, {NULL}};
const Segment SGI378146[] = {{&S62269[13], 7}, {NULL}};
const Segment SGI378147[] = {{&S62270[46], 5}, {NULL}};
const Segment SGI378148[] = {{&S62270[8], 3}, {NULL}};
const Segment SGI378149[] = {{&S62270[13], 7}, {NULL}};
const Segment SGI378150[] = {{&S62332[60], 5}, {NULL}};
const Segment SGI378151[] = {{&S62332[8], 3}, {NULL}};
const Segment SGI378152[] = {{&S62332[13], 7}, {NULL}};
const Segment SGI378153[] = {{&S62332[25], 14}, {NULL}};
const Segment SGI378154[] = {{&S62332[86], 14}, {NULL}};
const Segment SGI378155[] = {{&S62395[42], 6}, {NULL}};
const Segment SGI378156[] = {{&S62395[8], 3}, {NULL}};
const Segment SGI378157[] = {{&S62395[13], 3}, {NULL}};
const Segment SGI378158[] = {{&S62395[79], 3}, {NULL}};
const Segment SGI378159[] = {{&S62396[50], 5}, {NULL}};
const Segment SGI378160[] = {{&S62396[8], 3}, {NULL}};
const Segment SGI378161[] = {{&S62396[13], 3}, {NULL}};
const Segment SGI378162[] = {{&S62396[86], 3}, {NULL}};
const Segment SGI378163[] = {{&S62397[45], 5}, {NULL}};
const Segment SGI378164[] = {{&S62397[8], 3}, {NULL}};
const Segment SGI378165[] = {{&S62397[13], 3}, {NULL}};
const Segment SGI378166[] = {{&S62397[81], 3}, {NULL}};
const Segment SGI378167[] = {{&S62426[60], 5}, {NULL}};
const Segment SGI378168[] = {{&S62426[8], 3}, {NULL}};
const Segment SGI378169[] = {{&S62426[13], 7}, {NULL}};
const Segment SGI378170[] = {{&S62426[25], 14}, {NULL}};
const Segment SGI378171[] = {{&S62426[91], 3}, {NULL}};
const Segment SGI378172[] = {{&S62427[60], 5}, {NULL}};
const Segment SGI378173[] = {{&S62427[8], 3}, {NULL}};
const Segment SGI378174[] = {{&S62427[13], 7}, {NULL}};
const Segment SGI378175[] = {{&S62427[25], 14}, {NULL}};
const Segment SGI378176[] = {{&S62427[91], 3}, {NULL}};
const Segment SGI378177[] = {{&S62428[60], 5}, {NULL}};
const Segment SGI378178[] = {{&S62428[8], 3}, {NULL}};
const Segment SGI378179[] = {{&S62428[13], 7}, {NULL}};
const Segment SGI378180[] = {{&S62428[25], 14}, {NULL}};
const Segment SGI378181[] = {{&S62428[91], 3}, {NULL}};
const Segment SGI378182[] = {{&S62430[53], 5}, {NULL}};
const Segment SGI378183[] = {{&S62430[8], 3}, {NULL}};
const Segment SGI378184[] = {{&S62430[13], 7}, {NULL}};
const Segment SGI378185[] = {{&S62430[84], 3}, {NULL}};
const Segment SGI378186[] = {{&S62541[62], 5}, {NULL}};
const Segment SGI378187[] = {{&S62541[8], 3}, {NULL}};
const Segment SGI378188[] = {{&S62541[13], 9}, {NULL}};
const Segment SGI378189[] = {{&S62541[88], 16}, {NULL}};
const Segment SGI378190[] = {{&S62692[44], 6}, {NULL}};
const Segment SGI378191[] = {{&S62692[8], 3}, {NULL}};
const Segment SGI378192[] = {{&S62692[13], 3}, {NULL}};
const Segment SGI378193[] = {{&S62692[71], 10}, {NULL}};
const Segment SGI378194[] = {{&S62693[44], 6}, {NULL}};
const Segment SGI378195[] = {{&S62693[8], 3}, {NULL}};
const Segment SGI378196[] = {{&S62693[13], 3}, {NULL}};
const Segment SGI378197[] = {{&S62693[71], 10}, {NULL}};
const Segment SGI378198[] = {{&S62781[60], 6}, {NULL}};
const Segment SGI378199[] = {{&S62781[8], 3}, {NULL}};
const Segment SGI378200[] = {{&S62781[13], 7}, {NULL}};
const Segment SGI378201[] = {{&S62781[25], 14}, {NULL}};
const Segment SGI378202[] = {{&S62781[97], 3}, {NULL}};
const Segment SGI378203[] = {{&S62782[57], 6}, {NULL}};
const Segment SGI378204[] = {{&S62782[8], 3}, {NULL}};
const Segment SGI378205[] = {{&S62782[13], 7}, {NULL}};
const Segment SGI378206[] = {{&S62782[25], 14}, {NULL}};
const Segment SGI378207[] = {{&S62782[94], 3}, {NULL}};
const Segment SGI378208[] = {{&S62783[57], 6}, {NULL}};
const Segment SGI378209[] = {{&S62783[8], 3}, {NULL}};
const Segment SGI378210[] = {{&S62783[13], 7}, {NULL}};
const Segment SGI378211[] = {{&S62783[25], 14}, {NULL}};
const Segment SGI378212[] = {{&S62783[94], 3}, {NULL}};
const Segment SGI378213[] = {{&S62784[60], 6}, {NULL}};
const Segment SGI378214[] = {{&S62784[8], 3}, {NULL}};
const Segment SGI378215[] = {{&S62784[13], 7}, {NULL}};
const Segment SGI378216[] = {{&S62784[25], 14}, {NULL}};
const Segment SGI378217[] = {{&S62784[95], 3}, {NULL}};
const Segment SGI378218[] = {{&S62800[46], 5}, {NULL}};
const Segment SGI378219[] = {{&S62800[8], 3}, {NULL}};
const Segment SGI378220[] = {{&S62800[13], 7}, {NULL}};
const Segment SGI378221[] = {{&S62800[72], 16}, {NULL}};
const Segment SGI378222[] = {{&S62821[8], 3}, {NULL}};
const Segment SGI378223[] = {{&S62821[13], 9}, {NULL}};
const Segment SGI378224[] = {{&S66079[13], 21}, {NULL}};
const Segment SGI378225[] = {{&S66079[48], 5}, {NULL}};
const Segment SGI378226[] = {{&S66079[82], 3}, {NULL}};
const Segment SGI378227[] = {{&S66080[13], 21}, {NULL}};
const Segment SGI378228[] = {{&S66080[48], 5}, {NULL}};
const Segment SGI378229[] = {{&S66080[21], 3}, {NULL}};
const Segment SGI378230[] = {{&S66081[13], 19}, {NULL}};
const Segment SGI378231[] = {{&S66081[46], 8}, {NULL}};
const Segment SGI378232[] = {{&S66081[83], 3}, {NULL}};
const Segment SGI378233[] = {{&S66082[13], 21}, {NULL}};
const Segment SGI378234[] = {{&S66082[48], 5}, {NULL}};
const Segment SGI378235[] = {{&S66082[82], 3}, {NULL}};
const Segment SGI378236[] = {{&S66083[13], 21}, {NULL}};
const Segment SGI378237[] = {{&S66083[48], 5}, {NULL}};
const Segment SGI378238[] = {{&S66083[21], 3}, {NULL}};
const Segment SGI378239[] = {{&S66084[13], 21}, {NULL}};
const Segment SGI378240[] = {{&S66084[48], 5}, {NULL}};
const Segment SGI378241[] = {{&S66084[82], 3}, {NULL}};
const Segment SGI378242[] = {{&S66085[13], 21}, {NULL}};
const Segment SGI378243[] = {{&S66085[48], 5}, {NULL}};
const Segment SGI378244[] = {{&S66085[21], 3}, {NULL}};
const Segment SGI378245[] = {{&S66086[13], 21}, {NULL}};
const Segment SGI378246[] = {{&S66086[48], 5}, {NULL}};
const Segment SGI378247[] = {{&S66086[82], 3}, {NULL}};
const Segment SGI378248[] = {{&S66087[13], 21}, {NULL}};
const Segment SGI378249[] = {{&S66087[48], 5}, {NULL}};
const Segment SGI378250[] = {{&S66087[21], 3}, {NULL}};
const Segment SGI378251[] = {{&S66088[13], 21}, {NULL}};
const Segment SGI378252[] = {{&S66088[48], 5}, {NULL}};
const Segment SGI378253[] = {{&S66088[82], 3}, {NULL}};
const Segment SGI378254[] = {{&S66089[13], 21}, {NULL}};
const Segment SGI378255[] = {{&S66089[48], 5}, {NULL}};
const Segment SGI378256[] = {{&S66089[82], 3}, {NULL}};
const Segment SGI378257[] = {{&S66090[13], 21}, {NULL}};
const Segment SGI378258[] = {{&S66090[48], 5}, {NULL}};
const Segment SGI378259[] = {{&S66090[82], 3}, {NULL}};
const Segment SGI378260[] = {{&S66091[13], 21}, {NULL}};
const Segment SGI378261[] = {{&S66091[48], 5}, {NULL}};
const Segment SGI378262[] = {{&S66091[21], 3}, {NULL}};
const Segment SGI378263[] = {{&S66092[13], 21}, {NULL}};
const Segment SGI378264[] = {{&S66092[48], 5}, {NULL}};
const Segment SGI378265[] = {{&S66092[82], 3}, {NULL}};
const Segment SGI378266[] = {{&S66093[13], 21}, {NULL}};
const Segment SGI378267[] = {{&S66093[48], 5}, {NULL}};
const Segment SGI378268[] = {{&S66093[21], 3}, {NULL}};
const Segment SGI378269[] = {{&S63120[50], 6}, {NULL}};
const Segment SGI378270[] = {{&S63120[8], 3}, {NULL}};
const Segment SGI378271[] = {{&S63120[13], 9}, {NULL}};
const Segment SGI378272[] = {{&S63120[77], 6}, {NULL}};
const Segment SGI378273[] = {{&S63121[60], 5}, {NULL}};
const Segment SGI378274[] = {{&S63121[8], 3}, {NULL}};
const Segment SGI378275[] = {{&S63121[13], 9}, {NULL}};
const Segment SGI378276[] = {{&S63121[86], 6}, {NULL}};
const Segment SGI378277[] = {{&S63168[60], 5}, {NULL}};
const Segment SGI378278[] = {{&S63168[8], 3}, {NULL}};
const Segment SGI378279[] = {{&S63168[13], 7}, {NULL}};
const Segment SGI378280[] = {{&S63168[25], 14}, {NULL}};
const Segment SGI378281[] = {{&S63168[86], 16}, {NULL}};
const Segment SGI378282[] = {{&S63295[41], 6}, {NULL}};
const Segment SGI378283[] = {{&S63295[8], 3}, {NULL}};
const Segment SGI378284[] = {{&S63295[13], 14}, {NULL}};
const Segment SGI378285[] = {{&S63295[68], 12}, {NULL}};
const Segment SGI378286[] = {{&S63296[48], 6}, {NULL}};
const Segment SGI378287[] = {{&S63296[8], 3}, {NULL}};
const Segment SGI378288[] = {{&S63296[13], 14}, {NULL}};
const Segment SGI378289[] = {{&S63296[75], 12}, {NULL}};
const Segment SGI378290[] = {{&S63465[65], 6}, {NULL}};
const Segment SGI378291[] = {{&S63465[8], 3}, {NULL}};
const Segment SGI378292[] = {{&S63465[13], 9}, {NULL}};
const Segment SGI378293[] = {{&S63465[31], 16}, {NULL}};
const Segment SGI378294[] = {{&S63465[31], 13}, {NULL}};
const Segment SGI378295[] = {{&S63470[62], 5}, {NULL}};
const Segment SGI378296[] = {{&S63470[8], 3}, {NULL}};
const Segment SGI378297[] = {{&S63470[13], 9}, {NULL}};
const Segment SGI378298[] = {{&S63470[88], 16}, {NULL}};
const Segment SGI378299[] = {{&S63489[96], 7}, {NULL}};
const Segment SGI378300[] = {{&S63489[8], 3}, {NULL}};
const Segment SGI378301[] = {{&S63489[13], 9}, {NULL}};
const Segment SGI378302[] = {{&S63489[30], 13}, {NULL}};
const Segment SGI378303[] = {{&S63490[59], 6}, {NULL}};
const Segment SGI378304[] = {{&S63490[8], 3}, {NULL}};
const Segment SGI378305[] = {{&S63490[13], 9}, {NULL}};
const Segment SGI378306[] = {{&S63490[94], 3}, {NULL}};
const Segment SGI378307[] = {{&S63490[30], 13}, {NULL}};
const Segment SGI378308[] = {{&S63491[59], 6}, {NULL}};
const Segment SGI378309[] = {{&S63491[8], 3}, {NULL}};
const Segment SGI378310[] = {{&S63491[13], 9}, {NULL}};
const Segment SGI378311[] = {{&S63491[30], 13}, {NULL}};
const Segment SGI378312[] = {{&S63492[69], 6}, {NULL}};
const Segment SGI378313[] = {{&S63492[8], 3}, {NULL}};
const Segment SGI378314[] = {{&S63492[13], 9}, {NULL}};
const Segment SGI378315[] = {{&S63492[33], 15}, {NULL}};
const Segment SGI378316[] = {{&S63492[49], 6}, {NULL}};
const Segment SGI378317[] = {{&S63492[33], 13}, {NULL}};
const Segment SGI378318[] = {{&S63493[66], 6}, {NULL}};
const Segment SGI378319[] = {{&S63493[8], 3}, {NULL}};
const Segment SGI378320[] = {{&S63493[13], 9}, {NULL}};
const Segment SGI378321[] = {{&S63493[33], 15}, {NULL}};
const Segment SGI378322[] = {{&S63493[104], 3}, {NULL}};
const Segment SGI378323[] = {{&S63493[49], 3}, {NULL}};
const Segment SGI378324[] = {{&S63493[33], 13}, {NULL}};
const Segment SGI378325[] = {{&S63494[106], 6}, {NULL}};
const Segment SGI378326[] = {{&S63494[8], 3}, {NULL}};
const Segment SGI378327[] = {{&S63494[13], 9}, {NULL}};
const Segment SGI378328[] = {{&S63494[93], 3}, {NULL}};
const Segment SGI378329[] = {{&S63494[30], 13}, {NULL}};
const Segment SGI378330[] = {{&S63596[60], 5}, {NULL}};
const Segment SGI378331[] = {{&S63596[8], 3}, {NULL}};
const Segment SGI378332[] = {{&S63596[13], 7}, {NULL}};
const Segment SGI378333[] = {{&S63596[25], 14}, {NULL}};
const Segment SGI378334[] = {{&S63596[86], 16}, {NULL}};
const Segment SGI378335[] = {{&S63818[48], 6}, {NULL}};
const Segment SGI378336[] = {{&S63818[8], 3}, {NULL}};
const Segment SGI378337[] = {{&S63818[13], 14}, {NULL}};
const Segment SGI378338[] = {{&S63818[85], 3}, {NULL}};
const Segment SGI378339[] = {{&S64207[112], 6}, {NULL}};
const Segment SGI378340[] = {{&S64207[8], 3}, {NULL}};
const Segment SGI378341[] = {{&S64207[13], 9}, {NULL}};
const Segment SGI378342[] = {{&S64207[31], 15}, {NULL}};
const Segment SGI378343[] = {{&S64207[47], 6}, {NULL}};
const Segment SGI378344[] = {{&S64207[31], 13}, {NULL}};
const Segment SGI378345[] = {{&S64207[96], 8}, {NULL}};
const Segment SGI378346[] = {{&S64208[102], 6}, {NULL}};
const Segment SGI378347[] = {{&S64208[8], 3}, {NULL}};
const Segment SGI378348[] = {{&S64208[13], 9}, {NULL}};
const Segment SGI378349[] = {{&S64208[28], 13}, {NULL}};
const Segment SGI378350[] = {{&S64208[86], 8}, {NULL}};
const Segment SGI378351[] = {{&S64618[52], 5}, {NULL}};
const Segment SGI378352[] = {{&S64618[8], 3}, {NULL}};
const Segment SGI378353[] = {{&S64618[13], 3}, {NULL}};
const Segment SGI378354[] = {{&S64618[13], 3}, {&S64618[105], 12}, {NULL}};
const Segment SGI378355[] = {{&S64642[110], 7}, {NULL}};
const Segment SGI378356[] = {{&S64642[8], 3}, {NULL}};
const Segment SGI378357[] = {{&S64642[13], 9}, {NULL}};
const Segment SGI378358[] = {{&S64642[97], 3}, {NULL}};
const Segment SGI378359[] = {{&S64642[30], 13}, {NULL}};
const Segment SGI378360[] = {{&S64642[121], 12}, {NULL}};
const Segment SGI378361[] = {{&S64643[109], 7}, {NULL}};
const Segment SGI378362[] = {{&S64643[8], 3}, {NULL}};
const Segment SGI378363[] = {{&S64643[13], 9}, {NULL}};
const Segment SGI378364[] = {{&S64643[96], 3}, {NULL}};
const Segment SGI378365[] = {{&S64643[30], 13}, {NULL}};
const Segment SGI378366[] = {{&S64643[119], 12}, {NULL}};
const Segment SGI378367[] = {{&S64644[110], 7}, {NULL}};
const Segment SGI378368[] = {{&S64644[8], 3}, {NULL}};
const Segment SGI378369[] = {{&S64644[13], 9}, {NULL}};
const Segment SGI378370[] = {{&S64644[97], 3}, {NULL}};
const Segment SGI378371[] = {{&S64644[30], 13}, {NULL}};
const Segment SGI378372[] = {{&S64644[121], 12}, {NULL}};
const Segment SGI378373[] = {{&S64732[181], 5}, {NULL}};
const Segment SGI378374[] = {{&S64732[8], 3}, {&S64732[8], 3}, {NULL}};
const Segment SGI378375[] = {{&S64732[13], 9}, {&S64732[89], 7}, {NULL}};
const Segment SGI378376[] = {{&S64732[31], 13}, {&S64732[101], 14}, {NULL}};
const Segment SGI378377[] = {{&S64732[167], 3}, {NULL}};
const Segment SGI378378[] = {{&S64753[92], 6}, {NULL}};
const Segment SGI378379[] = {{&S64753[8], 3}, {NULL}};
const Segment SGI378380[] = {{&S64753[13], 9}, {NULL}};
const Segment SGI378381[] = {{&S64753[127], 3}, {NULL}};
const Segment SGI378382[] = {{&S64753[33], 13}, {NULL}};
const Segment SGI378383[] = {{&S64851[60], 6}, {NULL}};
const Segment SGI378384[] = {{&S64851[8], 3}, {NULL}};
const Segment SGI378385[] = {{&S64851[13], 7}, {NULL}};
const Segment SGI378386[] = {{&S64851[25], 14}, {NULL}};
const Segment SGI378387[] = {{&S64852[60], 6}, {NULL}};
const Segment SGI378388[] = {{&S64852[8], 3}, {NULL}};
const Segment SGI378389[] = {{&S64852[13], 7}, {NULL}};
const Segment SGI378390[] = {{&S64852[25], 14}, {NULL}};
const Segment SGI378391[] = {{&S64853[60], 6}, {NULL}};
const Segment SGI378392[] = {{&S64853[8], 3}, {NULL}};
const Segment SGI378393[] = {{&S64853[13], 7}, {NULL}};
const Segment SGI378394[] = {{&S64853[25], 14}, {NULL}};
const Segment SGI378395[] = {{&S64854[60], 6}, {NULL}};
const Segment SGI378396[] = {{&S64854[8], 3}, {NULL}};
const Segment SGI378397[] = {{&S64854[13], 7}, {NULL}};
const Segment SGI378398[] = {{&S64854[25], 14}, {NULL}};
const Segment SGI378399[] = {{&S64855[60], 6}, {NULL}};
const Segment SGI378400[] = {{&S64855[8], 3}, {NULL}};
const Segment SGI378401[] = {{&S64855[13], 7}, {NULL}};
const Segment SGI378402[] = {{&S64855[25], 14}, {NULL}};
const Segment SGI378403[] = {{&S64856[60], 6}, {NULL}};
const Segment SGI378404[] = {{&S64856[8], 3}, {NULL}};
const Segment SGI378405[] = {{&S64856[13], 7}, {NULL}};
const Segment SGI378406[] = {{&S64856[25], 14}, {NULL}};
const Segment SGI378407[] = {{&S64857[98], 6}, {NULL}};
const Segment SGI378408[] = {{&S64857[8], 3}, {NULL}};
const Segment SGI378409[] = {{&S64857[13], 14}, {NULL}};
const Segment SGI378410[] = {{&S64858[98], 6}, {NULL}};
const Segment SGI378411[] = {{&S64858[8], 3}, {NULL}};
const Segment SGI378412[] = {{&S64858[13], 14}, {NULL}};
const Segment SGI378413[] = {{&S64859[96], 6}, {NULL}};
const Segment SGI378414[] = {{&S64859[8], 3}, {NULL}};
const Segment SGI378415[] = {{&S64859[13], 14}, {NULL}};
const Segment SGI378416[] = {{&S64903[59], 6}, {NULL}};
const Segment SGI378417[] = {{&S64903[8], 3}, {NULL}};
const Segment SGI378418[] = {{&S64903[13], 9}, {NULL}};
const Segment SGI378419[] = {{&S64903[97], 3}, {NULL}};
const Segment SGI378420[] = {{&S64903[30], 13}, {NULL}};
const Segment SGI378421[] = {{&S64904[59], 6}, {NULL}};
const Segment SGI378422[] = {{&S64904[8], 3}, {NULL}};
const Segment SGI378423[] = {{&S64904[13], 9}, {NULL}};
const Segment SGI378424[] = {{&S64904[96], 3}, {NULL}};
const Segment SGI378425[] = {{&S64904[30], 13}, {NULL}};
const Segment SGI378426[] = {{&S64905[59], 6}, {NULL}};
const Segment SGI378427[] = {{&S64905[8], 3}, {NULL}};
const Segment SGI378428[] = {{&S64905[13], 9}, {NULL}};
const Segment SGI378429[] = {{&S64905[96], 3}, {NULL}};
const Segment SGI378430[] = {{&S64905[30], 13}, {NULL}};
const Segment SGI378431[] = {{&S64906[59], 6}, {NULL}};
const Segment SGI378432[] = {{&S64906[8], 3}, {NULL}};
const Segment SGI378433[] = {{&S64906[13], 9}, {NULL}};
const Segment SGI378434[] = {{&S64906[96], 3}, {NULL}};
const Segment SGI378435[] = {{&S64906[30], 13}, {NULL}};
const Segment SGI378436[] = {{&S64907[59], 6}, {NULL}};
const Segment SGI378437[] = {{&S64907[8], 3}, {NULL}};
const Segment SGI378438[] = {{&S64907[13], 9}, {NULL}};
const Segment SGI378439[] = {{&S64907[96], 3}, {NULL}};
const Segment SGI378440[] = {{&S64907[30], 13}, {NULL}};
const Segment SGI378441[] = {{&S64908[59], 6}, {NULL}};
const Segment SGI378442[] = {{&S64908[8], 3}, {NULL}};
const Segment SGI378443[] = {{&S64908[13], 9}, {NULL}};
const Segment SGI378444[] = {{&S64908[94], 3}, {NULL}};
const Segment SGI378445[] = {{&S64908[30], 13}, {NULL}};
const Segment SGI378446[] = {{&S64909[59], 6}, {NULL}};
const Segment SGI378447[] = {{&S64909[8], 3}, {NULL}};
const Segment SGI378448[] = {{&S64909[13], 9}, {NULL}};
const Segment SGI378449[] = {{&S64909[94], 3}, {NULL}};
const Segment SGI378450[] = {{&S64909[30], 13}, {NULL}};
const Segment SGI378451[] = {{&S64910[59], 6}, {NULL}};
const Segment SGI378452[] = {{&S64910[8], 3}, {NULL}};
const Segment SGI378453[] = {{&S64910[13], 9}, {NULL}};
const Segment SGI378454[] = {{&S64910[94], 3}, {NULL}};
const Segment SGI378455[] = {{&S64910[30], 13}, {NULL}};
const Segment SGI378456[] = {{&S65126[45], 5}, {NULL}};
const Segment SGI378457[] = {{&S65126[8], 3}, {NULL}};
const Segment SGI378458[] = {{&S65126[13], 3}, {NULL}};
const Segment SGI378459[] = {{&S65126[81], 3}, {NULL}};
const Segment SGI378460[] = {{&S65127[50], 5}, {NULL}};
const Segment SGI378461[] = {{&S65127[8], 3}, {NULL}};
const Segment SGI378462[] = {{&S65127[13], 3}, {NULL}};
const Segment SGI378463[] = {{&S65127[86], 3}, {NULL}};
const Segment SGI378464[] = {{&S65128[42], 6}, {NULL}};
const Segment SGI378465[] = {{&S65128[8], 3}, {NULL}};
const Segment SGI378466[] = {{&S65128[13], 3}, {NULL}};
const Segment SGI378467[] = {{&S65128[79], 3}, {NULL}};
const Segment SGI378468[] = {{&S67904[13], 92}, {NULL}};
const Segment SGI378469[] = {{&S67904[119], 3}, {NULL}};
const Segment SGI378470[] = {{&S67905[13], 92}, {NULL}};
const Segment SGI378471[] = {{&S67905[119], 3}, {NULL}};
const Segment SGI378472[] = {{&S67906[13], 92}, {NULL}};
const Segment SGI378473[] = {{&S67906[119], 3}, {NULL}};
const Segment SGI378474[] = {{&S67907[13], 92}, {NULL}};
const Segment SGI378475[] = {{&S67907[119], 3}, {NULL}};
const Segment SGI378476[] = {{&S67908[13], 92}, {NULL}};
const Segment SGI378477[] = {{&S67908[119], 3}, {NULL}};
const Segment SGI378478[] = {{&S67909[13], 92}, {NULL}};
const Segment SGI378479[] = {{&S67909[119], 3}, {NULL}};
const Segment SGI378480[] = {{&S67910[13], 86}, {NULL}};
const Segment SGI378481[] = {{&S67910[113], 3}, {NULL}};
const Segment SGI378482[] = {{&S67911[13], 92}, {NULL}};
const Segment SGI378483[] = {{&S67911[119], 3}, {NULL}};
const Segment SGI378484[] = {{&S67912[13], 91}, {NULL}};
const Segment SGI378485[] = {{&S67912[118], 3}, {NULL}};
const Segment SGI378486[] = {{&S67913[13], 92}, {NULL}};
const Segment SGI378487[] = {{&S67913[119], 3}, {NULL}};
const Segment SGI378488[] = {{&S67914[13], 92}, {NULL}};
const Segment SGI378489[] = {{&S67914[119], 3}, {NULL}};
const Segment SGI378490[] = {{&S67915[13], 92}, {NULL}};
const Segment SGI378491[] = {{&S67915[119], 3}, {NULL}};
const Segment SGI378492[] = {{&S67916[13], 92}, {NULL}};
const Segment SGI378493[] = {{&S67916[119], 3}, {NULL}};
const Segment SGI378494[] = {{&S67917[13], 92}, {NULL}};
const Segment SGI378495[] = {{&S67917[119], 3}, {NULL}};
const Segment SGI378496[] = {{&S67918[13], 92}, {NULL}};
const Segment SGI378497[] = {{&S67918[119], 3}, {NULL}};
const Segment SGI378498[] = {{&S67919[13], 92}, {NULL}};
const Segment SGI378499[] = {{&S67919[119], 3}, {NULL}};
const Segment SGI378500[] = {{&S67920[13], 92}, {NULL}};
const Segment SGI378501[] = {{&S67920[119], 3}, {NULL}};
const Segment SGI378502[] = {{&S67921[13], 92}, {NULL}};
const Segment SGI378503[] = {{&S67921[119], 3}, {NULL}};
const Segment SGI378504[] = {{&S67922[13], 92}, {NULL}};
const Segment SGI378505[] = {{&S67922[119], 3}, {NULL}};
const Segment SGI378506[] = {{&S67923[13], 92}, {NULL}};
const Segment SGI378507[] = {{&S67923[119], 3}, {NULL}};
const Segment SGI378508[] = {{&S67924[13], 92}, {NULL}};
const Segment SGI378509[] = {{&S67924[119], 3}, {NULL}};
const Segment SGI378510[] = {{&S67925[13], 92}, {NULL}};
const Segment SGI378511[] = {{&S67925[119], 3}, {NULL}};
const Segment SGI378512[] = {{&S67926[13], 94}, {NULL}};
const Segment SGI378513[] = {{&S67926[121], 3}, {NULL}};
const Segment SGI378514[] = {{&S67927[13], 92}, {NULL}};
const Segment SGI378515[] = {{&S67927[119], 3}, {NULL}};
const Segment SGI378516[] = {{&S67928[13], 92}, {NULL}};
const Segment SGI378517[] = {{&S67928[119], 3}, {NULL}};
const Segment SGI378518[] = {{&S67929[13], 92}, {NULL}};
const Segment SGI378519[] = {{&S67929[119], 3}, {NULL}};
const Segment SGI378520[] = {{&S67930[13], 92}, {NULL}};
const Segment SGI378521[] = {{&S67930[119], 3}, {NULL}};
const Segment SGI378522[] = {{&S67931[13], 92}, {NULL}};
const Segment SGI378523[] = {{&S67931[119], 3}, {NULL}};
const Segment SGI378524[] = {{&S67932[13], 92}, {NULL}};
const Segment SGI378525[] = {{&S67932[119], 3}, {NULL}};
const Segment SGI378526[] = {{&S67933[13], 92}, {NULL}};
const Segment SGI378527[] = {{&S67933[119], 3}, {NULL}};
const Segment SGI378528[] = {{&S68056[13], 35}, {NULL}};
const Segment SGI378529[] = {{&S68056[62], 6}, {NULL}};
const Segment SGI378530[] = {{&S68056[96], 3}, {NULL}};
const Segment SGI378531[] = {{&S65457[80], 6}, {NULL}};
const Segment SGI378532[] = {{&S65457[8], 3}, {NULL}};
const Segment SGI378533[] = {{&S65457[13], 9}, {NULL}};
const Segment SGI378534[] = {{&S65457[33], 15}, {NULL}};
const Segment SGI378535[] = {{&S65457[49], 10}, {NULL}};
const Segment SGI378536[] = {{&S65457[33], 13}, {NULL}};
const Segment SGI378537[] = {{&S65524[48], 6}, {NULL}};
const Segment SGI378538[] = {{&S65524[8], 3}, {NULL}};
const Segment SGI378539[] = {{&S65524[13], 14}, {NULL}};
const Segment SGI378540[] = {{&S65524[85], 3}, {NULL}};
const Segment SGI378541[] = {{&S68309[13], 61}, {NULL}};
const Segment SGI378542[] = {{&S68309[88], 6}, {NULL}};
const Segment SGI378543[] = {{&S68309[123], 3}, {NULL}};
const Segment SGI378544[] = {{&S68310[13], 60}, {NULL}};
const Segment SGI378545[] = {{&S68310[87], 6}, {NULL}};
const Segment SGI378546[] = {{&S68310[122], 3}, {NULL}};
const Segment SGI378547[] = {{&S68545[13], 94}, {NULL}};
const Segment SGI378548[] = {{&S68545[121], 3}, {NULL}};
const Segment SGI378549[] = {{&S68546[13], 94}, {NULL}};
const Segment SGI378550[] = {{&S68546[121], 3}, {NULL}};
const Segment SGI378551[] = {{&S68547[13], 94}, {NULL}};
const Segment SGI378552[] = {{&S68547[121], 3}, {NULL}};
const Segment SGI378553[] = {{&S68548[13], 94}, {NULL}};
const Segment SGI378554[] = {{&S68548[121], 3}, {NULL}};
const Segment SGI378555[] = {{&S68549[13], 94}, {NULL}};
const Segment SGI378556[] = {{&S68549[121], 3}, {NULL}};
const Segment SGI378557[] = {{&S68550[13], 94}, {NULL}};
const Segment SGI378558[] = {{&S68550[121], 3}, {NULL}};
const Segment SGI378559[] = {{&S68551[13], 94}, {NULL}};
const Segment SGI378560[] = {{&S68551[121], 3}, {NULL}};
const Segment SGI378561[] = {{&S68552[13], 94}, {NULL}};
const Segment SGI378562[] = {{&S68552[121], 3}, {NULL}};
const Segment SGI378563[] = {{&S68553[13], 94}, {NULL}};
const Segment SGI378564[] = {{&S68553[121], 3}, {NULL}};
const Segment SGI378565[] = {{&S68554[13], 94}, {NULL}};
const Segment SGI378566[] = {{&S68554[121], 3}, {NULL}};
const Segment SGI378567[] = {{&S68555[13], 94}, {NULL}};
const Segment SGI378568[] = {{&S68555[121], 3}, {NULL}};
const Segment SGI378569[] = {{&S68556[13], 94}, {NULL}};
const Segment SGI378570[] = {{&S68556[121], 3}, {NULL}};
const Segment SGI378571[] = {{&S68557[13], 94}, {NULL}};
const Segment SGI378572[] = {{&S68557[121], 3}, {NULL}};
const Segment SGI378573[] = {{&S68558[13], 94}, {NULL}};
const Segment SGI378574[] = {{&S68558[121], 3}, {NULL}};
const Segment SGI378575[] = {{&S68559[13], 94}, {NULL}};
const Segment SGI378576[] = {{&S68559[121], 3}, {NULL}};
const Segment SGI378577[] = {{&S68560[13], 94}, {NULL}};
const Segment SGI378578[] = {{&S68560[121], 3}, {NULL}};
const Segment SGI378579[] = {{&S68561[13], 94}, {NULL}};
const Segment SGI378580[] = {{&S68561[121], 3}, {NULL}};
const Segment SGI378581[] = {{&S68562[13], 94}, {NULL}};
const Segment SGI378582[] = {{&S68562[121], 3}, {NULL}};
const Segment SGI378583[] = {{&S68563[13], 94}, {NULL}};
const Segment SGI378584[] = {{&S68563[121], 3}, {NULL}};
const Segment SGI378585[] = {{&S68564[13], 94}, {NULL}};
const Segment SGI378586[] = {{&S68564[121], 3}, {NULL}};
const Segment SGI378587[] = {{&S68565[13], 95}, {NULL}};
const Segment SGI378588[] = {{&S68565[122], 3}, {NULL}};
const Segment SGI378589[] = {{&S68566[13], 95}, {NULL}};
const Segment SGI378590[] = {{&S68566[122], 3}, {NULL}};
const Segment SGI378591[] = {{&S68567[13], 94}, {NULL}};
const Segment SGI378592[] = {{&S68567[121], 3}, {NULL}};
const Segment SGI378593[] = {{&S68568[13], 94}, {NULL}};
const Segment SGI378594[] = {{&S68568[121], 3}, {NULL}};
const Segment SGI378595[] = {{&S68569[13], 94}, {NULL}};
const Segment SGI378596[] = {{&S68569[121], 3}, {NULL}};
const Segment SGI378597[] = {{&S68570[13], 94}, {NULL}};
const Segment SGI378598[] = {{&S68570[121], 3}, {NULL}};
const Segment SGI378599[] = {{&S68571[13], 94}, {NULL}};
const Segment SGI378600[] = {{&S68571[121], 3}, {NULL}};
const Segment SGI378601[] = {{&S68572[13], 94}, {NULL}};
const Segment SGI378602[] = {{&S68572[121], 3}, {NULL}};
const Segment SGI378603[] = {{&S68573[13], 94}, {NULL}};
const Segment SGI378604[] = {{&S68573[121], 3}, {NULL}};
const Segment SGI378605[] = {{&S68574[13], 95}, {NULL}};
const Segment SGI378606[] = {{&S68574[122], 3}, {NULL}};
const Segment SGI378607[] = {{&S68575[13], 95}, {NULL}};
const Segment SGI378608[] = {{&S68575[122], 3}, {NULL}};
const Segment SGI378609[] = {{&S68576[13], 94}, {NULL}};
const Segment SGI378610[] = {{&S68576[121], 3}, {NULL}};
const Segment SGI378611[] = {{&S68577[13], 94}, {NULL}};
const Segment SGI378612[] = {{&S68577[121], 3}, {NULL}};
const Segment SGI378613[] = {{&S68578[13], 94}, {NULL}};
const Segment SGI378614[] = {{&S68578[121], 3}, {NULL}};
const Segment SGI378615[] = {{&S68579[13], 94}, {NULL}};
const Segment SGI378616[] = {{&S68579[121], 3}, {NULL}};
const Segment SGI378617[] = {{&S68580[13], 94}, {NULL}};
const Segment SGI378618[] = {{&S68580[121], 3}, {NULL}};
const Segment SGI378619[] = {{&S68581[13], 94}, {NULL}};
const Segment SGI378620[] = {{&S68581[121], 3}, {NULL}};
const Segment SGI378621[] = {{&S68582[13], 94}, {NULL}};
const Segment SGI378622[] = {{&S68582[121], 3}, {NULL}};
const Segment SGI378623[] = {{&S68583[13], 94}, {NULL}};
const Segment SGI378624[] = {{&S68583[121], 3}, {NULL}};
const Segment SGI378625[] = {{&S68584[13], 94}, {NULL}};
const Segment SGI378626[] = {{&S68584[121], 3}, {NULL}};
const Segment SGI378627[] = {{&S68585[13], 95}, {NULL}};
const Segment SGI378628[] = {{&S68585[122], 3}, {NULL}};
const Segment SGI378629[] = {{&S68586[13], 94}, {NULL}};
const Segment SGI378630[] = {{&S68586[121], 3}, {NULL}};
const Segment SGI378631[] = {{&S66067[53], 6}, {NULL}};
const Segment SGI378632[] = {{&S66202[62], 5}, {NULL}};
const Segment SGI378633[] = {{&S66202[8], 3}, {NULL}};
const Segment SGI378634[] = {{&S66202[13], 9}, {NULL}};
const Segment SGI378635[] = {{&S66202[88], 16}, {NULL}};
const Segment SGI378636[] = {{&S66261[107], 6}, {NULL}};
const Segment SGI378637[] = {{&S66261[8], 3}, {NULL}};
const Segment SGI378638[] = {{&S66261[13], 7}, {NULL}};
const Segment SGI378639[] = {{&S66261[142], 3}, {NULL}};
const Segment SGI378640[] = {{&S66518[136], 6}, {NULL}};
const Segment SGI378641[] = {{&S66518[8], 3}, {NULL}};
const Segment SGI378642[] = {{&S66518[13], 9}, {NULL}};
const Segment SGI378643[] = {{&S66518[33], 15}, {NULL}};
const Segment SGI378644[] = {{&S66518[125], 3}, {NULL}};
const Segment SGI378645[] = {{&S66518[49], 6}, {NULL}};
const Segment SGI378646[] = {{&S66518[33], 13}, {NULL}};
const Segment SGI378647[] = {{&S66518[108], 3}, {NULL}};
const Segment SGI378648[] = {{&S66519[126], 6}, {NULL}};
const Segment SGI378649[] = {{&S66519[8], 3}, {NULL}};
const Segment SGI378650[] = {{&S66519[13], 9}, {NULL}};
const Segment SGI378651[] = {{&S66519[115], 3}, {NULL}};
const Segment SGI378652[] = {{&S66519[30], 13}, {NULL}};
const Segment SGI378653[] = {{&S66519[98], 3}, {NULL}};
const Segment SGI378654[] = {{&S66555[108], 6}, {NULL}};
const Segment SGI378655[] = {{&S66555[8], 3}, {NULL}};
const Segment SGI378656[] = {{&S66555[13], 7}, {NULL}};
const Segment SGI378657[] = {{&S66555[25], 14}, {NULL}};
const Segment SGI378658[] = {{&S66555[95], 3}, {NULL}};
const Segment SGI378659[] = {{&S66556[108], 6}, {NULL}};
const Segment SGI378660[] = {{&S66556[8], 3}, {NULL}};
const Segment SGI378661[] = {{&S66556[13], 7}, {NULL}};
const Segment SGI378662[] = {{&S66556[25], 14}, {NULL}};
const Segment SGI378663[] = {{&S66556[95], 3}, {NULL}};
const Segment SGI378664[] = {{&S66557[60], 6}, {NULL}};
const Segment SGI378665[] = {{&S66557[8], 3}, {NULL}};
const Segment SGI378666[] = {{&S66557[13], 7}, {NULL}};
const Segment SGI378667[] = {{&S66557[25], 14}, {NULL}};
const Segment SGI378668[] = {{&S66557[95], 3}, {NULL}};
const Segment SGI378669[] = {{&S66616[60], 5}, {NULL}};
const Segment SGI378670[] = {{&S66616[8], 3}, {NULL}};
const Segment SGI378671[] = {{&S66616[13], 7}, {NULL}};
const Segment SGI378672[] = {{&S66616[25], 14}, {NULL}};
const Segment SGI378673[] = {{&S66616[86], 6}, {NULL}};
const Segment SGI378674[] = {{&S66652[52], 6}, {NULL}};
const Segment SGI378675[] = {{&S66652[8], 3}, {NULL}};
const Segment SGI378676[] = {{&S66652[13], 3}, {NULL}};
const Segment SGI378677[] = {{&S66660[60], 5}, {NULL}};
const Segment SGI378678[] = {{&S66660[8], 3}, {NULL}};
const Segment SGI378679[] = {{&S66660[13], 7}, {NULL}};
const Segment SGI378680[] = {{&S66660[25], 14}, {NULL}};
const Segment SGI378681[] = {{&S66660[86], 16}, {NULL}};
const Segment SGI378682[] = {{&S66728[68], 6}, {NULL}};
const Segment SGI378683[] = {{&S66728[8], 3}, {NULL}};
const Segment SGI378684[] = {{&S66728[13], 9}, {NULL}};
const Segment SGI378685[] = {{&S66728[103], 3}, {NULL}};
const Segment SGI378686[] = {{&S66733[46], 5}, {NULL}};
const Segment SGI378687[] = {{&S66733[8], 3}, {NULL}};
const Segment SGI378688[] = {{&S66733[13], 7}, {NULL}};
const Segment SGI378689[] = {{&S66733[72], 14}, {NULL}};
const Segment SGI378690[] = {{&S67024[60], 5}, {NULL}};
const Segment SGI378691[] = {{&S67024[8], 3}, {NULL}};
const Segment SGI378692[] = {{&S67024[13], 7}, {NULL}};
const Segment SGI378693[] = {{&S67024[25], 14}, {NULL}};
const Segment SGI378694[] = {{&S67024[86], 14}, {NULL}};
const Segment SGI378695[] = {{&S69743[13], 15}, {NULL}};
const Segment SGI378696[] = {{&S69743[42], 5}, {NULL}};
const Segment SGI378697[] = {{&S69744[13], 15}, {NULL}};
const Segment SGI378698[] = {{&S69744[42], 5}, {NULL}};
const Segment SGI378699[] = {{&S69745[13], 15}, {NULL}};
const Segment SGI378700[] = {{&S69745[42], 5}, {NULL}};
const Segment SGI378701[] = {{&S69746[13], 5}, {NULL}};
const Segment SGI378702[] = {{&S69746[32], 6}, {NULL}};
const Segment SGI378703[] = {{&S69747[13], 15}, {NULL}};
const Segment SGI378704[] = {{&S69747[42], 5}, {NULL}};
const Segment SGI378705[] = {{&S69748[13], 15}, {NULL}};
const Segment SGI378706[] = {{&S69748[42], 5}, {NULL}};
const Segment SGI378707[] = {{&S69763[13], 24}, {NULL}};
const Segment SGI378708[] = {{&S69763[51], 5}, {NULL}};
const Segment SGI378709[] = {{&S69763[86], 5}, {NULL}};
const Segment SGI378710[] = {{&S67556[41], 6}, {NULL}};
const Segment SGI378711[] = {{&S67556[8], 3}, {NULL}};
const Segment SGI378712[] = {{&S67556[13], 14}, {NULL}};
const Segment SGI378713[] = {{&S67556[68], 12}, {NULL}};
const Segment SGI378714[] = {{&S70286[13], 5}, {NULL}};
const Segment SGI378715[] = {{&S70295[13], 89}, {NULL}};
const Segment SGI378716[] = {{&S70295[116], 3}, {NULL}};
const Segment SGI378717[] = {{&S70296[13], 50}, {NULL}};
const Segment SGI378718[] = {{&S70296[77], 3}, {NULL}};
const Segment SGI378719[] = {{&S67816[105], 6}, {NULL}};
const Segment SGI378720[] = {{&S67816[8], 3}, {NULL}};
const Segment SGI378721[] = {{&S67816[13], 9}, {NULL}};
const Segment SGI378722[] = {{&S67816[30], 13}, {NULL}};
const Segment SGI378723[] = {{&S67816[89], 8}, {NULL}};
const Segment SGI378724[] = {{&S68048[124], 6}, {NULL}};
const Segment SGI378725[] = {{&S68048[8], 3}, {NULL}};
const Segment SGI378726[] = {{&S68048[13], 9}, {NULL}};
const Segment SGI378727[] = {{&S68048[113], 3}, {NULL}};
const Segment SGI378728[] = {{&S68048[28], 13}, {NULL}};
const Segment SGI378729[] = {{&S68048[96], 3}, {NULL}};
const Segment SGI378730[] = {{&S68110[62], 5}, {NULL}};
const Segment SGI378731[] = {{&S68110[8], 3}, {NULL}};
const Segment SGI378732[] = {{&S68110[13], 9}, {NULL}};
const Segment SGI378733[] = {{&S68110[88], 16}, {NULL}};
const Segment SGI378734[] = {{&S68141[48], 6}, {NULL}};
const Segment SGI378735[] = {{&S68141[8], 3}, {NULL}};
const Segment SGI378736[] = {{&S68141[13], 14}, {NULL}};
const Segment SGI378737[] = {{&S68141[85], 3}, {NULL}};
const Segment SGI378738[] = {{&S68350[104], 6}, {NULL}};
const Segment SGI378739[] = {{&S68350[8], 3}, {NULL}};
const Segment SGI378740[] = {{&S68350[13], 9}, {NULL}};
const Segment SGI378741[] = {{&S68350[30], 13}, {NULL}};
const Segment SGI378742[] = {{&S68350[88], 8}, {NULL}};
const Segment SGI378743[] = {{&S68351[114], 6}, {NULL}};
const Segment SGI378744[] = {{&S68351[8], 3}, {NULL}};
const Segment SGI378745[] = {{&S68351[13], 9}, {NULL}};
const Segment SGI378746[] = {{&S68351[33], 15}, {NULL}};
const Segment SGI378747[] = {{&S68351[49], 6}, {NULL}};
const Segment SGI378748[] = {{&S68351[33], 13}, {NULL}};
const Segment SGI378749[] = {{&S68351[98], 8}, {NULL}};
const Segment SGI378750[] = {{&S68418[127], 6}, {NULL}};
const Segment SGI378751[] = {{&S68418[8], 3}, {NULL}};
const Segment SGI378752[] = {{&S68418[13], 9}, {NULL}};
const Segment SGI378753[] = {{&S68418[116], 3}, {NULL}};
const Segment SGI378754[] = {{&S68418[30], 13}, {NULL}};
const Segment SGI378755[] = {{&S68418[99], 3}, {NULL}};
const Segment SGI378756[] = {{&S68505[42], 6}, {NULL}};
const Segment SGI378757[] = {{&S68505[8], 3}, {NULL}};
const Segment SGI378758[] = {{&S68505[13], 3}, {NULL}};
const Segment SGI378759[] = {{&S68505[77], 3}, {NULL}};
const Segment SGI378760[] = {{&S68506[42], 6}, {NULL}};
const Segment SGI378761[] = {{&S68506[8], 3}, {NULL}};
const Segment SGI378762[] = {{&S68506[13], 3}, {NULL}};
const Segment SGI378763[] = {{&S68506[77], 3}, {NULL}};
const Segment SGI378764[] = {{&S68535[59], 5}, {NULL}};
const Segment SGI378765[] = {{&S68535[8], 3}, {NULL}};
const Segment SGI378766[] = {{&S68535[13], 9}, {NULL}};
const Segment SGI378767[] = {{&S68535[95], 3}, {NULL}};
const Segment SGI378768[] = {{&S68535[30], 13}, {NULL}};
const Segment SGI378769[] = {{&S68536[59], 6}, {NULL}};
const Segment SGI378770[] = {{&S68536[8], 3}, {NULL}};
const Segment SGI378771[] = {{&S68536[13], 9}, {NULL}};
const Segment SGI378772[] = {{&S68536[96], 3}, {NULL}};
const Segment SGI378773[] = {{&S68536[30], 13}, {NULL}};
const Segment SGI378774[] = {{&S68537[59], 5}, {NULL}};
const Segment SGI378775[] = {{&S68537[8], 3}, {NULL}};
const Segment SGI378776[] = {{&S68537[13], 9}, {NULL}};
const Segment SGI378777[] = {{&S68537[95], 3}, {NULL}};
const Segment SGI378778[] = {{&S68537[30], 13}, {NULL}};
const Segment SGI378779[] = {{&S68538[59], 6}, {NULL}};
const Segment SGI378780[] = {{&S68538[8], 3}, {NULL}};
const Segment SGI378781[] = {{&S68538[13], 9}, {NULL}};
const Segment SGI378782[] = {{&S68538[96], 3}, {NULL}};
const Segment SGI378783[] = {{&S68538[30], 13}, {NULL}};
const Segment SGI378784[] = {{&S68675[93], 6}, {NULL}};
const Segment SGI378785[] = {{&S68675[8], 3}, {NULL}};
const Segment SGI378786[] = {{&S68675[13], 9}, {NULL}};
const Segment SGI378787[] = {{&S68675[128], 3}, {NULL}};
const Segment SGI378788[] = {{&S68675[33], 13}, {NULL}};
const Segment SGI378789[] = {{&S68702[60], 5}, {NULL}};
const Segment SGI378790[] = {{&S68702[8], 3}, {NULL}};
const Segment SGI378791[] = {{&S68702[13], 7}, {NULL}};
const Segment SGI378792[] = {{&S68702[25], 14}, {NULL}};
const Segment SGI378793[] = {{&S68702[86], 16}, {NULL}};
const Segment SGI378794[] = {{&S68749[91], 6}, {NULL}};
const Segment SGI378795[] = {{&S68749[8], 3}, {NULL}};
const Segment SGI378796[] = {{&S68749[13], 9}, {NULL}};
const Segment SGI378797[] = {{&S68749[126], 3}, {NULL}};
const Segment SGI378798[] = {{&S68749[33], 13}, {NULL}};
const Segment SGI378799[] = {{&S69140[59], 6}, {NULL}};
const Segment SGI378800[] = {{&S69140[8], 3}, {NULL}};
const Segment SGI378801[] = {{&S69140[13], 9}, {NULL}};
const Segment SGI378802[] = {{&S69140[103], 3}, {NULL}};
const Segment SGI378803[] = {{&S69140[30], 13}, {NULL}};
const Segment SGI378804[] = {{&S69141[59], 6}, {NULL}};
const Segment SGI378805[] = {{&S69141[8], 3}, {NULL}};
const Segment SGI378806[] = {{&S69141[13], 9}, {NULL}};
const Segment SGI378807[] = {{&S69141[106], 3}, {NULL}};
const Segment SGI378808[] = {{&S69141[30], 13}, {NULL}};
const Segment SGI378809[] = {{&S69142[59], 6}, {NULL}};
const Segment SGI378810[] = {{&S69142[8], 3}, {NULL}};
const Segment SGI378811[] = {{&S69142[13], 9}, {NULL}};
const Segment SGI378812[] = {{&S69142[106], 3}, {NULL}};
const Segment SGI378813[] = {{&S69142[30], 13}, {NULL}};
const Segment SGI378814[] = {{&S69143[59], 6}, {NULL}};
const Segment SGI378815[] = {{&S69143[8], 3}, {NULL}};
const Segment SGI378816[] = {{&S69143[13], 9}, {NULL}};
const Segment SGI378817[] = {{&S69143[105], 3}, {NULL}};
const Segment SGI378818[] = {{&S69143[30], 13}, {NULL}};
const Segment SGI378819[] = {{&S69174[85], 6}, {NULL}};
const Segment SGI378820[] = {{&S69174[8], 3}, {NULL}};
const Segment SGI378821[] = {{&S69174[13], 9}, {NULL}};
const Segment SGI378822[] = {{&S69174[33], 15}, {NULL}};
const Segment SGI378823[] = {{&S69174[49], 15}, {NULL}};
const Segment SGI378824[] = {{&S69174[33], 13}, {NULL}};
const Segment SGI378825[] = {{&S69175[94], 6}, {NULL}};
const Segment SGI378826[] = {{&S69175[8], 3}, {NULL}};
const Segment SGI378827[] = {{&S69175[13], 9}, {NULL}};
const Segment SGI378828[] = {{&S69175[33], 15}, {NULL}};
const Segment SGI378829[] = {{&S69175[49], 24}, {NULL}};
const Segment SGI378830[] = {{&S69175[33], 13}, {NULL}};
const Segment SGI378831[] = {{&S69176[94], 6}, {NULL}};
const Segment SGI378832[] = {{&S69176[8], 3}, {NULL}};
const Segment SGI378833[] = {{&S69176[13], 9}, {NULL}};
const Segment SGI378834[] = {{&S69176[33], 15}, {NULL}};
const Segment SGI378835[] = {{&S69176[49], 24}, {NULL}};
const Segment SGI378836[] = {{&S69176[33], 13}, {NULL}};
const Segment SGI378837[] = {{&S69177[94], 6}, {NULL}};
const Segment SGI378838[] = {{&S69177[8], 3}, {NULL}};
const Segment SGI378839[] = {{&S69177[13], 9}, {NULL}};
const Segment SGI378840[] = {{&S69177[33], 15}, {NULL}};
const Segment SGI378841[] = {{&S69177[49], 24}, {NULL}};
const Segment SGI378842[] = {{&S69177[33], 13}, {NULL}};
const Segment SGI378843[] = {{&S69505[60], 5}, {NULL}};
const Segment SGI378844[] = {{&S69505[8], 3}, {NULL}};
const Segment SGI378845[] = {{&S69505[13], 7}, {NULL}};
const Segment SGI378846[] = {{&S69505[25], 14}, {NULL}};
const Segment SGI378847[] = {{&S69505[91], 3}, {NULL}};
const Segment SGI378848[] = {{&S69506[8], 3}, {NULL}};
const Segment SGI378849[] = {{&S69506[13], 7}, {NULL}};
const Segment SGI378850[] = {{&S69506[25], 14}, {NULL}};
const Segment SGI378851[] = {{&S69506[91], 3}, {NULL}};
const Segment SGI378852[] = {{&S69507[8], 3}, {NULL}};
const Segment SGI378853[] = {{&S69507[13], 7}, {NULL}};
const Segment SGI378854[] = {{&S69507[25], 14}, {NULL}};
const Segment SGI378855[] = {{&S69507[92], 3}, {NULL}};
const Segment SGI378856[] = {{&S69711[44], 6}, {NULL}};
const Segment SGI378857[] = {{&S69711[8], 3}, {NULL}};
const Segment SGI378858[] = {{&S69711[13], 3}, {NULL}};
const Segment SGI378859[] = {{&S69784[92], 6}, {NULL}};
const Segment SGI378860[] = {{&S69784[8], 3}, {NULL}};
const Segment SGI378861[] = {{&S69784[13], 9}, {NULL}};
const Segment SGI378862[] = {{&S69784[33], 15}, {NULL}};
const Segment SGI378863[] = {{&S69784[49], 22}, {NULL}};
const Segment SGI378864[] = {{&S69784[33], 13}, {NULL}};
const Segment SGI378865[] = {{&S70149[126], 6}, {NULL}};
const Segment SGI378866[] = {{&S70149[8], 3}, {NULL}};
const Segment SGI378867[] = {{&S70149[13], 9}, {NULL}};
const Segment SGI378868[] = {{&S70149[115], 3}, {NULL}};
const Segment SGI378869[] = {{&S70149[30], 13}, {NULL}};
const Segment SGI378870[] = {{&S70149[98], 3}, {NULL}};
const Segment SGI378871[] = {{&S73454[13], 74}, {NULL}};
const Segment SGI378872[] = {{&S73454[101], 6}, {NULL}};
const Segment SGI378873[] = {{&S73454[42], 3}, {NULL}};
const Segment SGI378874[] = {{&S73455[13], 68}, {NULL}};
const Segment SGI378875[] = {{&S73455[95], 6}, {NULL}};
const Segment SGI378876[] = {{&S73455[42], 3}, {NULL}};
const Segment SGI378877[] = {{&S73456[13], 66}, {NULL}};
const Segment SGI378878[] = {{&S73456[107], 6}, {NULL}};
const Segment SGI378879[] = {{&S73456[42], 3}, {NULL}};
const Segment SGI378880[] = {{&S73457[13], 59}, {NULL}};
const Segment SGI378881[] = {{&S73457[86], 6}, {NULL}};
const Segment SGI378882[] = {{&S73457[121], 3}, {NULL}};
const Segment SGI378883[] = {{&S73458[13], 74}, {NULL}};
const Segment SGI378884[] = {{&S73458[101], 6}, {NULL}};
const Segment SGI378885[] = {{&S73458[42], 3}, {NULL}};
const Segment SGI378886[] = {{&S70330[60], 5}, {NULL}};
const Segment SGI378887[] = {{&S70330[8], 3}, {NULL}};
const Segment SGI378888[] = {{&S70330[13], 7}, {NULL}};
const Segment SGI378889[] = {{&S70330[25], 14}, {NULL}};
const Segment SGI378890[] = {{&S70330[91], 3}, {NULL}};
const Segment SGI378891[] = {{&S70331[60], 5}, {NULL}};
const Segment SGI378892[] = {{&S70331[8], 3}, {NULL}};
const Segment SGI378893[] = {{&S70331[13], 7}, {NULL}};
const Segment SGI378894[] = {{&S70331[25], 14}, {NULL}};
const Segment SGI378895[] = {{&S70331[91], 3}, {NULL}};
const Segment SGI378896[] = {{&S70608[80], 6}, {NULL}};
const Segment SGI378897[] = {{&S70608[8], 3}, {NULL}};
const Segment SGI378898[] = {{&S70608[13], 9}, {NULL}};
const Segment SGI378899[] = {{&S70608[33], 15}, {NULL}};
const Segment SGI378900[] = {{&S70608[49], 10}, {NULL}};
const Segment SGI378901[] = {{&S70608[33], 13}, {NULL}};
const Segment SGI378902[] = {{&S70609[84], 6}, {NULL}};
const Segment SGI378903[] = {{&S70609[8], 3}, {NULL}};
const Segment SGI378904[] = {{&S70609[13], 9}, {NULL}};
const Segment SGI378905[] = {{&S70609[33], 15}, {NULL}};
const Segment SGI378906[] = {{&S70609[49], 14}, {NULL}};
const Segment SGI378907[] = {{&S70609[33], 13}, {NULL}};
const Segment SGI378908[] = {{&S70676[42], 6}, {NULL}};
const Segment SGI378909[] = {{&S70676[8], 3}, {NULL}};
const Segment SGI378910[] = {{&S70676[13], 3}, {NULL}};
const Segment SGI378911[] = {{&S70676[13], 3}, {&S70676[97], 12}, {NULL}};
const Segment SGI378912[] = {{&S70677[52], 5}, {NULL}};
const Segment SGI378913[] = {{&S70677[8], 3}, {NULL}};
const Segment SGI378914[] = {{&S70677[13], 3}, {NULL}};
const Segment SGI378915[] = {{&S70677[13], 3}, {&S70677[105], 12}, {NULL}};
const Segment SGI378916[] = {{&S71014[44], 6}, {NULL}};
const Segment SGI378917[] = {{&S71014[8], 3}, {NULL}};
const Segment SGI378918[] = {{&S71014[13], 3}, {NULL}};
const Segment SGI378919[] = {{&S71014[71], 12}, {NULL}};
const Segment SGI378920[] = {{&S71015[44], 6}, {NULL}};
const Segment SGI378921[] = {{&S71015[8], 3}, {NULL}};
const Segment SGI378922[] = {{&S71015[13], 3}, {NULL}};
const Segment SGI378923[] = {{&S71015[71], 12}, {NULL}};
const Segment SGI378924[] = {{&S71092[80], 6}, {NULL}};
const Segment SGI378925[] = {{&S71092[8], 3}, {NULL}};
const Segment SGI378926[] = {{&S71092[13], 9}, {NULL}};
const Segment SGI378927[] = {{&S71092[33], 15}, {NULL}};
const Segment SGI378928[] = {{&S71092[49], 10}, {NULL}};
const Segment SGI378929[] = {{&S71092[33], 13}, {NULL}};
const Segment SGI378930[] = {{&S71193[60], 5}, {NULL}};
const Segment SGI378931[] = {{&S71193[8], 3}, {NULL}};
const Segment SGI378932[] = {{&S71193[13], 7}, {NULL}};
const Segment SGI378933[] = {{&S71193[25], 14}, {NULL}};
const Segment SGI378934[] = {{&S71193[86], 16}, {NULL}};
const Segment SGI378935[] = {{&S71593[52], 6}, {NULL}};
const Segment SGI378936[] = {{&S71593[8], 3}, {NULL}};
const Segment SGI378937[] = {{&S71593[13], 9}, {NULL}};
const Segment SGI378938[] = {{&S71593[89], 3}, {NULL}};
const Segment SGI378939[] = {{&S74239[13], 33}, {NULL}};
const Segment SGI378940[] = {{&S74239[60], 5}, {NULL}};
const Segment SGI378941[] = {{&S74239[124], 5}, {NULL}};
const Segment SGI378942[] = {{&S71653[8], 3}, {NULL}};
const Segment SGI378943[] = {{&S71653[13], 7}, {NULL}};
const Segment SGI378944[] = {{&S71653[25], 14}, {NULL}};
const Segment SGI378945[] = {{&S71653[91], 3}, {NULL}};
const Segment SGI378946[] = {{&S71654[8], 3}, {NULL}};
const Segment SGI378947[] = {{&S71654[13], 7}, {NULL}};
const Segment SGI378948[] = {{&S71654[25], 14}, {NULL}};
const Segment SGI378949[] = {{&S71654[92], 3}, {NULL}};
const Segment SGI378950[] = {{&S71655[60], 5}, {NULL}};
const Segment SGI378951[] = {{&S71655[8], 3}, {NULL}};
const Segment SGI378952[] = {{&S71655[13], 7}, {NULL}};
const Segment SGI378953[] = {{&S71655[25], 14}, {NULL}};
const Segment SGI378954[] = {{&S71655[91], 3}, {NULL}};
const Segment SGI378955[] = {{&S71685[60], 6}, {NULL}};
const Segment SGI378956[] = {{&S71685[8], 3}, {NULL}};
const Segment SGI378957[] = {{&S71685[13], 7}, {NULL}};
const Segment SGI378958[] = {{&S71685[25], 14}, {NULL}};
const Segment SGI378959[] = {{&S71685[97], 3}, {NULL}};
const Segment SGI378960[] = {{&S71686[60], 5}, {NULL}};
const Segment SGI378961[] = {{&S71686[8], 3}, {NULL}};
const Segment SGI378962[] = {{&S71686[13], 7}, {NULL}};
const Segment SGI378963[] = {{&S71686[25], 14}, {NULL}};
const Segment SGI378964[] = {{&S71686[94], 3}, {NULL}};
const Segment SGI378965[] = {{&S71687[60], 6}, {NULL}};
const Segment SGI378966[] = {{&S71687[8], 3}, {NULL}};
const Segment SGI378967[] = {{&S71687[13], 7}, {NULL}};
const Segment SGI378968[] = {{&S71687[25], 14}, {NULL}};
const Segment SGI378969[] = {{&S71687[97], 3}, {NULL}};
const Segment SGI378970[] = {{&S71688[60], 6}, {NULL}};
const Segment SGI378971[] = {{&S71688[8], 3}, {NULL}};
const Segment SGI378972[] = {{&S71688[13], 7}, {NULL}};
const Segment SGI378973[] = {{&S71688[25], 14}, {NULL}};
const Segment SGI378974[] = {{&S71688[97], 3}, {NULL}};
const Segment SGI378975[] = {{&S71729[62], 5}, {NULL}};
const Segment SGI378976[] = {{&S71729[8], 3}, {NULL}};
const Segment SGI378977[] = {{&S71729[13], 9}, {NULL}};
const Segment SGI378978[] = {{&S71729[88], 14}, {NULL}};
const Segment SGI378979[] = {{&S71752[52], 6}, {NULL}};
const Segment SGI378980[] = {{&S71752[8], 3}, {NULL}};
const Segment SGI378981[] = {{&S71752[13], 9}, {NULL}};
const Segment SGI378982[] = {{&S71752[89], 3}, {NULL}};
const Segment SGI378983[] = {{&S71857[80], 6}, {NULL}};
const Segment SGI378984[] = {{&S71857[8], 3}, {NULL}};
const Segment SGI378985[] = {{&S71857[13], 9}, {NULL}};
const Segment SGI378986[] = {{&S71857[33], 15}, {NULL}};
const Segment SGI378987[] = {{&S71857[49], 10}, {NULL}};
const Segment SGI378988[] = {{&S71857[33], 13}, {NULL}};
const Segment SGI378989[] = {{&S71858[84], 6}, {NULL}};
const Segment SGI378990[] = {{&S71858[8], 3}, {NULL}};
const Segment SGI378991[] = {{&S71858[13], 9}, {NULL}};
const Segment SGI378992[] = {{&S71858[33], 15}, {NULL}};
const Segment SGI378993[] = {{&S71858[49], 14}, {NULL}};
const Segment SGI378994[] = {{&S71858[33], 13}, {NULL}};
const Segment SGI378995[] = {{&S74590[13], 61}, {NULL}};
const Segment SGI378996[] = {{&S74590[88], 6}, {NULL}};
const Segment SGI378997[] = {{&S74590[123], 3}, {NULL}};
const Segment SGI378998[] = {{&S74591[13], 60}, {NULL}};
const Segment SGI378999[] = {{&S74591[87], 6}, {NULL}};
const Segment SGI379000[] = {{&S74591[122], 3}, {NULL}};
const Segment SGI379001[] = {{&S74592[13], 61}, {NULL}};
const Segment SGI379002[] = {{&S74592[88], 6}, {NULL}};
const Segment SGI379003[] = {{&S74592[123], 3}, {NULL}};
const Segment SGI379004[] = {{&S74593[13], 61}, {NULL}};
const Segment SGI379005[] = {{&S74593[88], 6}, {NULL}};
const Segment SGI379006[] = {{&S74593[123], 3}, {NULL}};
const Segment SGI379007[] = {{&S74594[13], 61}, {NULL}};
const Segment SGI379008[] = {{&S74594[88], 6}, {NULL}};
const Segment SGI379009[] = {{&S74594[123], 3}, {NULL}};
const Segment SGI379010[] = {{&S74595[13], 61}, {NULL}};
const Segment SGI379011[] = {{&S74595[88], 6}, {NULL}};
const Segment SGI379012[] = {{&S74595[123], 3}, {NULL}};
const Segment SGI379013[] = {{&S74596[13], 61}, {NULL}};
const Segment SGI379014[] = {{&S74596[88], 6}, {NULL}};
const Segment SGI379015[] = {{&S74596[123], 3}, {NULL}};
const Segment SGI379016[] = {{&S74597[13], 61}, {NULL}};
const Segment SGI379017[] = {{&S74597[88], 6}, {NULL}};
const Segment SGI379018[] = {{&S74597[123], 3}, {NULL}};
const Segment SGI379019[] = {{&S74598[13], 61}, {NULL}};
const Segment SGI379020[] = {{&S74598[88], 6}, {NULL}};
const Segment SGI379021[] = {{&S74598[123], 3}, {NULL}};
const Segment SGI379022[] = {{&S74599[13], 61}, {NULL}};
const Segment SGI379023[] = {{&S74599[88], 6}, {NULL}};
const Segment SGI379024[] = {{&S74599[123], 3}, {NULL}};
const Segment SGI379025[] = {{&S74600[13], 61}, {NULL}};
const Segment SGI379026[] = {{&S74600[88], 6}, {NULL}};
const Segment SGI379027[] = {{&S74600[123], 3}, {NULL}};
const Segment SGI379028[] = {{&S74601[13], 61}, {NULL}};
const Segment SGI379029[] = {{&S74601[88], 6}, {NULL}};
const Segment SGI379030[] = {{&S74601[123], 3}, {NULL}};
const Segment SGI379031[] = {{&S72322[44], 6}, {NULL}};
const Segment SGI379032[] = {{&S72322[8], 3}, {NULL}};
const Segment SGI379033[] = {{&S72322[13], 7}, {NULL}};
const Segment SGI379034[] = {{&S74794[13], 38}, {NULL}};
const Segment SGI379035[] = {{&S74794[65], 5}, {NULL}};
const Segment SGI379036[] = {{&S74794[100], 5}, {NULL}};
const Segment SGI379037[] = {{&S74794[113], 8}, {NULL}};
const Segment SGI379038[] = {{&S74795[13], 38}, {NULL}};
const Segment SGI379039[] = {{&S74795[65], 5}, {NULL}};
const Segment SGI379040[] = {{&S74795[100], 5}, {NULL}};
const Segment SGI379041[] = {{&S74795[113], 8}, {NULL}};
const Segment SGI379042[] = {{&S74796[13], 38}, {NULL}};
const Segment SGI379043[] = {{&S74796[65], 5}, {NULL}};
const Segment SGI379044[] = {{&S74796[100], 5}, {NULL}};
const Segment SGI379045[] = {{&S74796[113], 8}, {NULL}};
const Segment SGI379046[] = {{&S72415[59], 6}, {NULL}};
const Segment SGI379047[] = {{&S72415[8], 3}, {NULL}};
const Segment SGI379048[] = {{&S72415[13], 9}, {NULL}};
const Segment SGI379049[] = {{&S72415[30], 13}, {NULL}};
const Segment SGI379050[] = {{&S72452[60], 5}, {NULL}};
const Segment SGI379051[] = {{&S72452[8], 3}, {NULL}};
const Segment SGI379052[] = {{&S72452[13], 7}, {NULL}};
const Segment SGI379053[] = {{&S72452[25], 14}, {NULL}};
const Segment SGI379054[] = {{&S72452[91], 3}, {NULL}};
const Segment SGI379055[] = {{&S72453[60], 5}, {NULL}};
const Segment SGI379056[] = {{&S72453[8], 3}, {NULL}};
const Segment SGI379057[] = {{&S72453[13], 7}, {NULL}};
const Segment SGI379058[] = {{&S72453[25], 14}, {NULL}};
const Segment SGI379059[] = {{&S72453[91], 3}, {NULL}};
const Segment SGI379060[] = {{&S72679[46], 5}, {NULL}};
const Segment SGI379061[] = {{&S72679[8], 3}, {NULL}};
const Segment SGI379062[] = {{&S72679[13], 7}, {NULL}};
const Segment SGI379063[] = {{&S72679[72], 14}, {NULL}};
const Segment SGI379064[] = {{&S73050[46], 5}, {NULL}};
const Segment SGI379065[] = {{&S73050[8], 3}, {NULL}};
const Segment SGI379066[] = {{&S73050[13], 3}, {NULL}};
const Segment SGI379067[] = {{&S73050[72], 6}, {NULL}};
const Segment SGI379068[] = {{&S73190[87], 6}, {NULL}};
const Segment SGI379069[] = {{&S73190[8], 3}, {NULL}};
const Segment SGI379070[] = {{&S73190[13], 9}, {NULL}};
const Segment SGI379071[] = {{&S73190[33], 15}, {NULL}};
const Segment SGI379072[] = {{&S73190[49], 17}, {NULL}};
const Segment SGI379073[] = {{&S73190[33], 13}, {NULL}};
const Segment SGI379074[] = {{&S73191[88], 6}, {NULL}};
const Segment SGI379075[] = {{&S73191[8], 3}, {NULL}};
const Segment SGI379076[] = {{&S73191[13], 9}, {NULL}};
const Segment SGI379077[] = {{&S73191[33], 15}, {NULL}};
const Segment SGI379078[] = {{&S73191[49], 18}, {NULL}};
const Segment SGI379079[] = {{&S73191[33], 13}, {NULL}};
const Segment SGI379080[] = {{&S73192[89], 6}, {NULL}};
const Segment SGI379081[] = {{&S73192[8], 3}, {NULL}};
const Segment SGI379082[] = {{&S73192[13], 9}, {NULL}};
const Segment SGI379083[] = {{&S73192[33], 15}, {NULL}};
const Segment SGI379084[] = {{&S73192[49], 19}, {NULL}};
const Segment SGI379085[] = {{&S73192[33], 13}, {NULL}};
const Segment SGI379086[] = {{&S75620[13], 21}, {NULL}};
const Segment SGI379087[] = {{&S75620[48], 6}, {NULL}};
const Segment SGI379088[] = {{&S75620[94], 13}, {NULL}};
const Segment SGI379089[] = {{&S3082[85], 84}, {NULL}};
const Segment SGI379090[] = {{&S3082[68], 101}, {NULL}};
const Segment SGI379091[] = {{&S3082[36], 11}, {NULL}};
const Segment SGI379092[] = {{&S3086[85], 84}, {NULL}};
const Segment SGI379093[] = {{&S3086[36], 11}, {NULL}};
const Segment SGI379094[] = {{&S73410[110], 7}, {NULL}};
const Segment SGI379095[] = {{&S73410[8], 3}, {NULL}};
const Segment SGI379096[] = {{&S73410[13], 9}, {NULL}};
const Segment SGI379097[] = {{&S73410[97], 3}, {NULL}};
const Segment SGI379098[] = {{&S73410[30], 13}, {NULL}};
const Segment SGI379099[] = {{&S73410[121], 12}, {NULL}};
const Segment SGI379100[] = {{&S73411[110], 7}, {NULL}};
const Segment SGI379101[] = {{&S73411[8], 3}, {NULL}};
const Segment SGI379102[] = {{&S73411[13], 9}, {NULL}};
const Segment SGI379103[] = {{&S73411[97], 3}, {NULL}};
const Segment SGI379104[] = {{&S73411[30], 13}, {NULL}};
const Segment SGI379105[] = {{&S73411[121], 12}, {NULL}};
const Segment SGI379106[] = {{&S73412[110], 7}, {NULL}};
const Segment SGI379107[] = {{&S73412[8], 3}, {NULL}};
const Segment SGI379108[] = {{&S73412[13], 9}, {NULL}};
const Segment SGI379109[] = {{&S73412[97], 3}, {NULL}};
const Segment SGI379110[] = {{&S73412[30], 13}, {NULL}};
const Segment SGI379111[] = {{&S73412[121], 12}, {NULL}};
const Segment SGI379112[] = {{&S73429[93], 6}, {NULL}};
const Segment SGI379113[] = {{&S73429[8], 3}, {NULL}};
const Segment SGI379114[] = {{&S73429[13], 9}, {NULL}};
const Segment SGI379115[] = {{&S73429[128], 3}, {NULL}};
const Segment SGI379116[] = {{&S73429[33], 13}, {NULL}};
const Segment SGI379117[] = {{&S75901[13], 100}, {NULL}};
const Segment SGI379118[] = {{&S75901[127], 3}, {NULL}};
const Segment SGI379119[] = {{&S75902[13], 109}, {NULL}};
const Segment SGI379120[] = {{&S75902[136], 3}, {NULL}};
const Segment SGI379121[] = {{&S75903[13], 109}, {NULL}};
const Segment SGI379122[] = {{&S75903[136], 3}, {NULL}};
const Segment SGI379123[] = {{&S75904[13], 100}, {NULL}};
const Segment SGI379124[] = {{&S75904[127], 3}, {NULL}};
const Segment SGI379125[] = {{&S75905[13], 100}, {NULL}};
const Segment SGI379126[] = {{&S75905[127], 3}, {NULL}};
const Segment SGI379127[] = {{&S75906[13], 100}, {NULL}};
const Segment SGI379128[] = {{&S75906[127], 3}, {NULL}};
const Segment SGI379129[] = {{&S75907[13], 100}, {NULL}};
const Segment SGI379130[] = {{&S75907[127], 3}, {NULL}};
const Segment SGI379131[] = {{&S75908[13], 100}, {NULL}};
const Segment SGI379132[] = {{&S75908[127], 3}, {NULL}};
const Segment SGI379133[] = {{&S75909[13], 100}, {NULL}};
const Segment SGI379134[] = {{&S75909[127], 3}, {NULL}};
const Segment SGI379135[] = {{&S75910[13], 109}, {NULL}};
const Segment SGI379136[] = {{&S75910[136], 3}, {NULL}};
const Segment SGI379137[] = {{&S75911[13], 109}, {NULL}};
const Segment SGI379138[] = {{&S75911[136], 3}, {NULL}};
const Segment SGI379139[] = {{&S75912[13], 109}, {NULL}};
const Segment SGI379140[] = {{&S75912[136], 3}, {NULL}};
const Segment SGI379141[] = {{&S75913[13], 109}, {NULL}};
const Segment SGI379142[] = {{&S75913[136], 3}, {NULL}};
const Segment SGI379143[] = {{&S75914[13], 109}, {NULL}};
const Segment SGI379144[] = {{&S75914[136], 3}, {NULL}};
const Segment SGI379145[] = {{&S75915[13], 109}, {NULL}};
const Segment SGI379146[] = {{&S75915[136], 3}, {NULL}};
const Segment SGI379147[] = {{&S75916[13], 100}, {NULL}};
const Segment SGI379148[] = {{&S75916[127], 3}, {NULL}};
const Segment SGI379149[] = {{&S75917[13], 100}, {NULL}};
const Segment SGI379150[] = {{&S75917[127], 3}, {NULL}};
const Segment SGI379151[] = {{&S75918[13], 100}, {NULL}};
const Segment SGI379152[] = {{&S75918[127], 3}, {NULL}};
const Segment SGI379153[] = {{&S75919[13], 94}, {NULL}};
const Segment SGI379154[] = {{&S75919[121], 3}, {NULL}};
const Segment SGI379155[] = {{&S75920[13], 109}, {NULL}};
const Segment SGI379156[] = {{&S75920[136], 3}, {NULL}};
const Segment SGI379157[] = {{&S73940[96], 6}, {NULL}};
const Segment SGI379158[] = {{&S73940[8], 3}, {NULL}};
const Segment SGI379159[] = {{&S73940[13], 9}, {NULL}};
const Segment SGI379160[] = {{&S73940[104], 16}, {NULL}};
const Segment SGI379161[] = {{&S73941[62], 6}, {NULL}};
const Segment SGI379162[] = {{&S73941[8], 3}, {NULL}};
const Segment SGI379163[] = {{&S73941[13], 9}, {NULL}};
const Segment SGI379164[] = {{&S73941[104], 16}, {NULL}};
const Segment SGI379165[] = {{&S73942[96], 6}, {NULL}};
const Segment SGI379166[] = {{&S73942[8], 3}, {NULL}};
const Segment SGI379167[] = {{&S73942[13], 9}, {NULL}};
const Segment SGI379168[] = {{&S73942[104], 16}, {NULL}};
const Segment SGI379169[] = {{&S74238[60], 5}, {NULL}};
const Segment SGI379170[] = {{&S74238[8], 3}, {NULL}};
const Segment SGI379171[] = {{&S74238[13], 7}, {NULL}};
const Segment SGI379172[] = {{&S74238[25], 14}, {NULL}};
const Segment SGI379173[] = {{&S74240[60], 5}, {NULL}};
const Segment SGI379174[] = {{&S74240[8], 3}, {NULL}};
const Segment SGI379175[] = {{&S74240[13], 7}, {NULL}};
const Segment SGI379176[] = {{&S74240[25], 14}, {NULL}};
const Segment SGI379177[] = {{&S74656[104], 6}, {NULL}};
const Segment SGI379178[] = {{&S74656[8], 3}, {NULL}};
const Segment SGI379179[] = {{&S74656[13], 3}, {NULL}};
const Segment SGI379180[] = {{&S74656[93], 3}, {NULL}};
const Segment SGI379181[] = {{&S74782[110], 7}, {NULL}};
const Segment SGI379182[] = {{&S74782[8], 3}, {NULL}};
const Segment SGI379183[] = {{&S74782[13], 9}, {NULL}};
const Segment SGI379184[] = {{&S74782[97], 3}, {NULL}};
const Segment SGI379185[] = {{&S74782[30], 13}, {NULL}};
const Segment SGI379186[] = {{&S74782[121], 12}, {NULL}};
const Segment SGI379187[] = {{&S74783[109], 7}, {NULL}};
const Segment SGI379188[] = {{&S74783[8], 3}, {NULL}};
const Segment SGI379189[] = {{&S74783[13], 9}, {NULL}};
const Segment SGI379190[] = {{&S74783[96], 3}, {NULL}};
const Segment SGI379191[] = {{&S74783[30], 13}, {NULL}};
const Segment SGI379192[] = {{&S74783[119], 12}, {NULL}};
const Segment SGI379193[] = {{&S4461[75], 105}, {NULL}};
const Segment SGI379194[] = {{&S4461[36], 11}, {NULL}};
const Segment SGI379195[] = {{&S74919[60], 6}, {NULL}};
const Segment SGI379196[] = {{&S74919[8], 3}, {NULL}};
const Segment SGI379197[] = {{&S74919[13], 7}, {NULL}};
const Segment SGI379198[] = {{&S74919[25], 14}, {NULL}};
const Segment SGI379199[] = {{&S74919[93], 3}, {NULL}};
const Segment SGI379200[] = {{&S77276[13], 46}, {NULL}};
const Segment SGI379201[] = {{&S77276[73], 8}, {NULL}};
const Segment SGI379202[] = {{&S77276[110], 5}, {NULL}};
const Segment SGI379203[] = {{&S77276[134], 9}, {NULL}};
const Segment SGI379204[] = {{&S77277[13], 45}, {NULL}};
const Segment SGI379205[] = {{&S77277[72], 8}, {NULL}};
const Segment SGI379206[] = {{&S77277[109], 5}, {NULL}};
const Segment SGI379207[] = {{&S77277[135], 9}, {NULL}};
const Segment SGI379208[] = {{&S77400[13], 14}, {NULL}};
const Segment SGI379209[] = {{&S77400[41], 6}, {NULL}};
const Segment SGI379210[] = {{&S77400[41], 5}, {NULL}};
const Segment SGI379211[] = {{&S77401[13], 14}, {NULL}};
const Segment SGI379212[] = {{&S77401[41], 6}, {NULL}};
const Segment SGI379213[] = {{&S75179[110], 6}, {NULL}};
const Segment SGI379214[] = {{&S75179[8], 3}, {NULL}};
const Segment SGI379215[] = {{&S75179[13], 9}, {NULL}};
const Segment SGI379216[] = {{&S75179[31], 16}, {NULL}};
const Segment SGI379217[] = {{&S75179[31], 13}, {NULL}};
const Segment SGI379218[] = {{&S75179[94], 8}, {NULL}};
const Segment SGI379219[] = {{&S75221[48], 6}, {NULL}};
const Segment SGI379220[] = {{&S75221[8], 3}, {NULL}};
const Segment SGI379221[] = {{&S75221[13], 14}, {NULL}};
const Segment SGI379222[] = {{&S75221[75], 12}, {NULL}};
const Segment SGI379223[] = {{&S75222[41], 6}, {NULL}};
const Segment SGI379224[] = {{&S75222[8], 3}, {NULL}};
const Segment SGI379225[] = {{&S75222[13], 14}, {NULL}};
const Segment SGI379226[] = {{&S75222[68], 12}, {NULL}};
const Segment SGI379227[] = {{&S75266[48], 6}, {NULL}};
const Segment SGI379228[] = {{&S75266[8], 3}, {NULL}};
const Segment SGI379229[] = {{&S75266[13], 14}, {NULL}};
const Segment SGI379230[] = {{&S75266[85], 3}, {NULL}};
const Segment SGI379231[] = {{&S5110[85], 84}, {NULL}};
const Segment SGI379232[] = {{&S5110[36], 11}, {NULL}};
const Segment SGI379233[] = {{&S5111[72], 105}, {NULL}};
const Segment SGI379234[] = {{&S5111[36], 11}, {NULL}};
const Segment SGI379235[] = {{&S75407[62], 5}, {NULL}};
const Segment SGI379236[] = {{&S75407[8], 3}, {NULL}};
const Segment SGI379237[] = {{&S75407[13], 9}, {NULL}};
const Segment SGI379238[] = {{&S75407[88], 16}, {NULL}};
const Segment SGI379239[] = {{&S5191[7], 21}, {NULL}};
const Segment SGI379240[] = {{&S5191[40], 11}, {NULL}};
const Segment SGI379241[] = {{&S77929[13], 94}, {NULL}};
const Segment SGI379242[] = {{&S77929[121], 3}, {NULL}};
const Segment SGI379243[] = {{&S77930[13], 94}, {NULL}};
const Segment SGI379244[] = {{&S77930[121], 3}, {NULL}};
const Segment SGI379245[] = {{&S77931[13], 94}, {NULL}};
const Segment SGI379246[] = {{&S77931[121], 3}, {NULL}};
const Segment SGI379247[] = {{&S77932[13], 94}, {NULL}};
const Segment SGI379248[] = {{&S77932[121], 3}, {NULL}};
const Segment SGI379249[] = {{&S77933[13], 94}, {NULL}};
const Segment SGI379250[] = {{&S77933[121], 3}, {NULL}};
const Segment SGI379251[] = {{&S77934[13], 94}, {NULL}};
const Segment SGI379252[] = {{&S77934[121], 3}, {NULL}};
const Segment SGI379253[] = {{&S77935[13], 94}, {NULL}};
const Segment SGI379254[] = {{&S77935[121], 3}, {NULL}};
const Segment SGI379255[] = {{&S77936[13], 94}, {NULL}};
const Segment SGI379256[] = {{&S77936[121], 3}, {NULL}};
const Segment SGI379257[] = {{&S77937[13], 94}, {NULL}};
const Segment SGI379258[] = {{&S77937[121], 3}, {NULL}};
const Segment SGI379259[] = {{&S77938[13], 94}, {NULL}};
const Segment SGI379260[] = {{&S77938[121], 3}, {NULL}};
const Segment SGI379261[] = {{&S77939[13], 94}, {NULL}};
const Segment SGI379262[] = {{&S77939[121], 3}, {NULL}};
const Segment SGI379263[] = {{&S77940[13], 94}, {NULL}};
const Segment SGI379264[] = {{&S77940[121], 3}, {NULL}};
const Segment SGI379265[] = {{&S77941[13], 94}, {NULL}};
const Segment SGI379266[] = {{&S77941[121], 3}, {NULL}};
const Segment SGI379267[] = {{&S77942[13], 94}, {NULL}};
const Segment SGI379268[] = {{&S77942[121], 3}, {NULL}};
const Segment SGI379269[] = {{&S77943[13], 94}, {NULL}};
const Segment SGI379270[] = {{&S77943[121], 3}, {NULL}};
const Segment SGI379271[] = {{&S77944[13], 94}, {NULL}};
const Segment SGI379272[] = {{&S77944[121], 3}, {NULL}};
const Segment SGI379273[] = {{&S77945[13], 94}, {NULL}};
const Segment SGI379274[] = {{&S77945[121], 3}, {NULL}};
const Segment SGI379275[] = {{&S77946[13], 94}, {NULL}};
const Segment SGI379276[] = {{&S77946[121], 3}, {NULL}};
const Segment SGI379277[] = {{&S77947[13], 94}, {NULL}};
const Segment SGI379278[] = {{&S77947[121], 3}, {NULL}};
const Segment SGI379279[] = {{&S77948[13], 94}, {NULL}};
const Segment SGI379280[] = {{&S77948[121], 3}, {NULL}};
const Segment SGI379281[] = {{&S77949[13], 94}, {NULL}};
const Segment SGI379282[] = {{&S77949[121], 3}, {NULL}};
const Segment SGI379283[] = {{&S77950[13], 94}, {NULL}};
const Segment SGI379284[] = {{&S77950[121], 3}, {NULL}};
const Segment SGI379285[] = {{&S77951[13], 94}, {NULL}};
const Segment SGI379286[] = {{&S77951[121], 3}, {NULL}};
const Segment SGI379287[] = {{&S77952[13], 94}, {NULL}};
const Segment SGI379288[] = {{&S77952[121], 3}, {NULL}};
const Segment SGI379289[] = {{&S77953[13], 94}, {NULL}};
const Segment SGI379290[] = {{&S77953[121], 3}, {NULL}};
const Segment SGI379291[] = {{&S77954[13], 94}, {NULL}};
const Segment SGI379292[] = {{&S77954[121], 3}, {NULL}};
const Segment SGI379293[] = {{&S77955[13], 94}, {NULL}};
const Segment SGI379294[] = {{&S77955[121], 3}, {NULL}};
const Segment SGI379295[] = {{&S75699[62], 5}, {NULL}};
const Segment SGI379296[] = {{&S75699[8], 3}, {NULL}};
const Segment SGI379297[] = {{&S75699[13], 9}, {NULL}};
const Segment SGI379298[] = {{&S75699[88], 14}, {NULL}};
const Segment SGI379299[] = {{&S75704[44], 6}, {NULL}};
const Segment SGI379300[] = {{&S75704[8], 3}, {NULL}};
const Segment SGI379301[] = {{&S75704[13], 3}, {NULL}};
const Segment SGI379302[] = {{&S75704[71], 10}, {NULL}};
const Segment SGI379303[] = {{&S75705[44], 6}, {NULL}};
const Segment SGI379304[] = {{&S75705[8], 3}, {NULL}};
const Segment SGI379305[] = {{&S75705[13], 3}, {NULL}};
const Segment SGI379306[] = {{&S75705[71], 10}, {NULL}};
const Segment SGI379307[] = {{&S5514[56], 17}, {NULL}};
const Segment SGI379308[] = {{&S75879[44], 6}, {NULL}};
const Segment SGI379309[] = {{&S75879[8], 3}, {NULL}};
const Segment SGI379310[] = {{&S75879[13], 3}, {NULL}};
const Segment SGI379311[] = {{&S75879[71], 10}, {NULL}};
const Segment SGI379312[] = {{&S76545[44], 6}, {NULL}};
const Segment SGI379313[] = {{&S76545[8], 3}, {NULL}};
const Segment SGI379314[] = {{&S76545[13], 7}, {NULL}};
const Segment SGI379315[] = {{&S76545[105], 3}, {NULL}};
const Segment SGI379316[] = {{&S76546[108], 6}, {NULL}};
const Segment SGI379317[] = {{&S76546[8], 3}, {NULL}};
const Segment SGI379318[] = {{&S76546[13], 7}, {NULL}};
const Segment SGI379319[] = {{&S76546[97], 3}, {NULL}};
const Segment SGI379320[] = {{&S76570[42], 6}, {NULL}};
const Segment SGI379321[] = {{&S76570[8], 3}, {NULL}};
const Segment SGI379322[] = {{&S76570[13], 3}, {NULL}};
const Segment SGI379323[] = {{&S76570[13], 3}, {&S76570[97], 12}, {NULL}};
const Segment SGI379324[] = {{&S76641[59], 6}, {NULL}};
const Segment SGI379325[] = {{&S76641[8], 3}, {NULL}};
const Segment SGI379326[] = {{&S76641[13], 9}, {NULL}};
const Segment SGI379327[] = {{&S76641[30], 13}, {NULL}};
const Segment SGI379328[] = {{&S76641[94], 3}, {NULL}};
const Segment SGI379329[] = {{&S76642[59], 6}, {NULL}};
const Segment SGI379330[] = {{&S76642[8], 3}, {NULL}};
const Segment SGI379331[] = {{&S76642[13], 9}, {NULL}};
const Segment SGI379332[] = {{&S76642[30], 13}, {NULL}};
const Segment SGI379333[] = {{&S76642[94], 3}, {NULL}};
const Segment SGI379334[] = {{&S76643[69], 6}, {NULL}};
const Segment SGI379335[] = {{&S76643[8], 3}, {NULL}};
const Segment SGI379336[] = {{&S76643[13], 9}, {NULL}};
const Segment SGI379337[] = {{&S76643[33], 15}, {NULL}};
const Segment SGI379338[] = {{&S76643[49], 6}, {NULL}};
const Segment SGI379339[] = {{&S76643[33], 13}, {NULL}};
const Segment SGI379340[] = {{&S76643[104], 3}, {NULL}};
const Segment SGI379341[] = {{&S76644[59], 6}, {NULL}};
const Segment SGI379342[] = {{&S76644[8], 3}, {NULL}};
const Segment SGI379343[] = {{&S76644[13], 9}, {NULL}};
const Segment SGI379344[] = {{&S76644[30], 13}, {NULL}};
const Segment SGI379345[] = {{&S76644[92], 3}, {NULL}};
const Segment SGI379346[] = {{&S76645[59], 6}, {NULL}};
const Segment SGI379347[] = {{&S76645[8], 3}, {NULL}};
const Segment SGI379348[] = {{&S76645[13], 9}, {NULL}};
const Segment SGI379349[] = {{&S76645[30], 13}, {NULL}};
const Segment SGI379350[] = {{&S76645[95], 3}, {NULL}};
const Segment SGI379351[] = {{&S76646[59], 6}, {NULL}};
const Segment SGI379352[] = {{&S76646[8], 3}, {NULL}};
const Segment SGI379353[] = {{&S76646[13], 9}, {NULL}};
const Segment SGI379354[] = {{&S76646[30], 13}, {NULL}};
const Segment SGI379355[] = {{&S76646[94], 3}, {NULL}};
const Segment SGI379356[] = {{&S76647[69], 6}, {NULL}};
const Segment SGI379357[] = {{&S76647[8], 3}, {NULL}};
const Segment SGI379358[] = {{&S76647[13], 9}, {NULL}};
const Segment SGI379359[] = {{&S76647[33], 15}, {NULL}};
const Segment SGI379360[] = {{&S76647[49], 6}, {NULL}};
const Segment SGI379361[] = {{&S76647[33], 13}, {NULL}};
const Segment SGI379362[] = {{&S76647[105], 3}, {NULL}};
const Segment SGI379363[] = {{&S76648[69], 5}, {NULL}};
const Segment SGI379364[] = {{&S76648[8], 3}, {NULL}};
const Segment SGI379365[] = {{&S76648[13], 9}, {NULL}};
const Segment SGI379366[] = {{&S76648[33], 15}, {NULL}};
const Segment SGI379367[] = {{&S76648[49], 6}, {NULL}};
const Segment SGI379368[] = {{&S76648[33], 13}, {NULL}};
const Segment SGI379369[] = {{&S76648[102], 3}, {NULL}};
const Segment SGI379370[] = {{&S76649[59], 6}, {NULL}};
const Segment SGI379371[] = {{&S76649[8], 3}, {NULL}};
const Segment SGI379372[] = {{&S76649[13], 9}, {NULL}};
const Segment SGI379373[] = {{&S76649[30], 13}, {NULL}};
const Segment SGI379374[] = {{&S76649[94], 3}, {NULL}};
const Segment SGI379375[] = {{&S76650[59], 6}, {NULL}};
const Segment SGI379376[] = {{&S76650[8], 3}, {NULL}};
const Segment SGI379377[] = {{&S76650[13], 9}, {NULL}};
const Segment SGI379378[] = {{&S76650[30], 13}, {NULL}};
const Segment SGI379379[] = {{&S76650[94], 3}, {NULL}};
const Segment SGI379380[] = {{&S76657[119], 6}, {NULL}};
const Segment SGI379381[] = {{&S76657[8], 3}, {NULL}};
const Segment SGI379382[] = {{&S76657[13], 9}, {NULL}};
const Segment SGI379383[] = {{&S76657[33], 15}, {NULL}};
const Segment SGI379384[] = {{&S76657[49], 6}, {NULL}};
const Segment SGI379385[] = {{&S76657[33], 13}, {NULL}};
const Segment SGI379386[] = {{&S76657[106], 3}, {NULL}};
const Segment SGI379387[] = {{&S79439[13], 21}, {NULL}};
const Segment SGI379388[] = {{&S79439[48], 5}, {NULL}};
const Segment SGI379389[] = {{&S76925[106], 6}, {NULL}};
const Segment SGI379390[] = {{&S76925[8], 3}, {NULL}};
const Segment SGI379391[] = {{&S76925[13], 9}, {NULL}};
const Segment SGI379392[] = {{&S76925[93], 3}, {NULL}};
const Segment SGI379393[] = {{&S76925[28], 13}, {NULL}};
const Segment SGI379394[] = {{&S76926[69], 6}, {NULL}};
const Segment SGI379395[] = {{&S76926[8], 3}, {NULL}};
const Segment SGI379396[] = {{&S76926[13], 9}, {NULL}};
const Segment SGI379397[] = {{&S76926[33], 15}, {NULL}};
const Segment SGI379398[] = {{&S76926[112], 3}, {NULL}};
const Segment SGI379399[] = {{&S76926[49], 6}, {NULL}};
const Segment SGI379400[] = {{&S76926[33], 13}, {NULL}};
const Segment SGI379401[] = {{&S76927[57], 6}, {NULL}};
const Segment SGI379402[] = {{&S76927[8], 3}, {NULL}};
const Segment SGI379403[] = {{&S76927[13], 9}, {NULL}};
const Segment SGI379404[] = {{&S76927[94], 3}, {NULL}};
const Segment SGI379405[] = {{&S76927[28], 13}, {NULL}};
const Segment SGI379406[] = {{&S76968[104], 6}, {NULL}};
const Segment SGI379407[] = {{&S76968[8], 3}, {NULL}};
const Segment SGI379408[] = {{&S76968[13], 9}, {NULL}};
const Segment SGI379409[] = {{&S76968[30], 13}, {NULL}};
const Segment SGI379410[] = {{&S76968[88], 8}, {NULL}};
const Segment SGI379411[] = {{&S77000[60], 5}, {NULL}};
const Segment SGI379412[] = {{&S77000[8], 3}, {NULL}};
const Segment SGI379413[] = {{&S77000[13], 7}, {NULL}};
const Segment SGI379414[] = {{&S77000[25], 14}, {NULL}};
const Segment SGI379415[] = {{&S77000[86], 16}, {NULL}};
const Segment SGI379416[] = {{&S77133[60], 5}, {NULL}};
const Segment SGI379417[] = {{&S77133[8], 3}, {NULL}};
const Segment SGI379418[] = {{&S77133[13], 7}, {NULL}};
const Segment SGI379419[] = {{&S77133[25], 14}, {NULL}};
const Segment SGI379420[] = {{&S77133[91], 3}, {NULL}};
const Segment SGI379421[] = {{&S77134[60], 5}, {NULL}};
const Segment SGI379422[] = {{&S77134[8], 3}, {NULL}};
const Segment SGI379423[] = {{&S77134[13], 7}, {NULL}};
const Segment SGI379424[] = {{&S77134[25], 14}, {NULL}};
const Segment SGI379425[] = {{&S77134[91], 3}, {NULL}};
const Segment SGI379426[] = {{&S77143[60], 5}, {NULL}};
const Segment SGI379427[] = {{&S77143[8], 3}, {NULL}};
const Segment SGI379428[] = {{&S77143[13], 7}, {NULL}};
const Segment SGI379429[] = {{&S77143[25], 14}, {NULL}};
const Segment SGI379430[] = {{&S77143[86], 14}, {NULL}};
const Segment SGI379431[] = {{&S77278[62], 6}, {NULL}};
const Segment SGI379432[] = {{&S77278[8], 3}, {NULL}};
const Segment SGI379433[] = {{&S77278[13], 9}, {NULL}};
const Segment SGI379434[] = {{&S77334[41], 6}, {NULL}};
const Segment SGI379435[] = {{&S77334[8], 3}, {NULL}};
const Segment SGI379436[] = {{&S77334[13], 14}, {NULL}};
const Segment SGI379437[] = {{&S77334[68], 12}, {NULL}};
const Segment SGI379438[] = {{&S77480[102], 6}, {NULL}};
const Segment SGI379439[] = {{&S77480[8], 3}, {NULL}};
const Segment SGI379440[] = {{&S77480[13], 9}, {NULL}};
const Segment SGI379441[] = {{&S77480[28], 13}, {NULL}};
const Segment SGI379442[] = {{&S77480[86], 8}, {NULL}};
const Segment SGI379443[] = {{&S80189[13], 94}, {NULL}};
const Segment SGI379444[] = {{&S80189[121], 3}, {NULL}};
const Segment SGI379445[] = {{&S80190[13], 94}, {NULL}};
const Segment SGI379446[] = {{&S80190[121], 3}, {NULL}};
const Segment SGI379447[] = {{&S80191[13], 94}, {NULL}};
const Segment SGI379448[] = {{&S80191[121], 3}, {NULL}};
const Segment SGI379449[] = {{&S80192[13], 94}, {NULL}};
const Segment SGI379450[] = {{&S80192[121], 3}, {NULL}};
const Segment SGI379451[] = {{&S80193[13], 94}, {NULL}};
const Segment SGI379452[] = {{&S80193[121], 3}, {NULL}};
const Segment SGI379453[] = {{&S80194[13], 94}, {NULL}};
const Segment SGI379454[] = {{&S80194[121], 3}, {NULL}};
const Segment SGI379455[] = {{&S80195[13], 94}, {NULL}};
const Segment SGI379456[] = {{&S80195[121], 3}, {NULL}};
const Segment SGI379457[] = {{&S80196[13], 94}, {NULL}};
const Segment SGI379458[] = {{&S80196[121], 3}, {NULL}};
const Segment SGI379459[] = {{&S77588[60], 5}, {NULL}};
const Segment SGI379460[] = {{&S77588[8], 3}, {NULL}};
const Segment SGI379461[] = {{&S77588[13], 7}, {NULL}};
const Segment SGI379462[] = {{&S77588[25], 14}, {NULL}};
const Segment SGI379463[] = {{&S77588[91], 3}, {NULL}};
const Segment SGI379464[] = {{&S77971[119], 6}, {NULL}};
const Segment SGI379465[] = {{&S77971[8], 3}, {NULL}};
const Segment SGI379466[] = {{&S77971[13], 9}, {NULL}};
const Segment SGI379467[] = {{&S77971[33], 15}, {NULL}};
const Segment SGI379468[] = {{&S77971[49], 6}, {NULL}};
const Segment SGI379469[] = {{&S77971[33], 13}, {NULL}};
const Segment SGI379470[] = {{&S77971[108], 3}, {NULL}};
const Segment SGI379471[] = {{&S78012[79], 5}, {NULL}};
const Segment SGI379472[] = {{&S78012[8], 3}, {NULL}};
const Segment SGI379473[] = {{&S78012[13], 3}, {NULL}};
const Segment SGI379474[] = {{&S78012[86], 10}, {NULL}};
const Segment SGI379475[] = {{&S78043[107], 7}, {NULL}};
const Segment SGI379476[] = {{&S78043[8], 3}, {NULL}};
const Segment SGI379477[] = {{&S78043[13], 9}, {NULL}};
const Segment SGI379478[] = {{&S78043[96], 3}, {NULL}};
const Segment SGI379479[] = {{&S78043[30], 13}, {NULL}};
const Segment SGI379480[] = {{&S78043[118], 12}, {NULL}};
const Segment SGI379481[] = {{&S78044[107], 7}, {NULL}};
const Segment SGI379482[] = {{&S78044[8], 3}, {NULL}};
const Segment SGI379483[] = {{&S78044[13], 9}, {NULL}};
const Segment SGI379484[] = {{&S78044[96], 3}, {NULL}};
const Segment SGI379485[] = {{&S78044[30], 13}, {NULL}};
const Segment SGI379486[] = {{&S78044[117], 12}, {NULL}};
const Segment SGI379487[] = {{&S78066[95], 6}, {NULL}};
const Segment SGI379488[] = {{&S78066[8], 3}, {NULL}};
const Segment SGI379489[] = {{&S78066[13], 9}, {NULL}};
const Segment SGI379490[] = {{&S78066[33], 15}, {NULL}};
const Segment SGI379491[] = {{&S78066[49], 25}, {NULL}};
const Segment SGI379492[] = {{&S78066[33], 13}, {NULL}};
const Segment SGI379493[] = {{&S78067[95], 6}, {NULL}};
const Segment SGI379494[] = {{&S78067[8], 3}, {NULL}};
const Segment SGI379495[] = {{&S78067[13], 9}, {NULL}};
const Segment SGI379496[] = {{&S78067[33], 15}, {NULL}};
const Segment SGI379497[] = {{&S78067[49], 25}, {NULL}};
const Segment SGI379498[] = {{&S78067[33], 13}, {NULL}};
const Segment SGI379499[] = {{&S78068[95], 6}, {NULL}};
const Segment SGI379500[] = {{&S78068[8], 3}, {NULL}};
const Segment SGI379501[] = {{&S78068[13], 9}, {NULL}};
const Segment SGI379502[] = {{&S78068[33], 15}, {NULL}};
const Segment SGI379503[] = {{&S78068[49], 25}, {NULL}};
const Segment SGI379504[] = {{&S78068[33], 13}, {NULL}};
const Segment SGI379505[] = {{&S78069[90], 6}, {NULL}};
const Segment SGI379506[] = {{&S78069[8], 3}, {NULL}};
const Segment SGI379507[] = {{&S78069[13], 9}, {NULL}};
const Segment SGI379508[] = {{&S78069[33], 15}, {NULL}};
const Segment SGI379509[] = {{&S78069[49], 20}, {NULL}};
const Segment SGI379510[] = {{&S78069[33], 13}, {NULL}};
const Segment SGI379511[] = {{&S78070[95], 6}, {NULL}};
const Segment SGI379512[] = {{&S78070[8], 3}, {NULL}};
const Segment SGI379513[] = {{&S78070[13], 9}, {NULL}};
const Segment SGI379514[] = {{&S78070[33], 15}, {NULL}};
const Segment SGI379515[] = {{&S78070[49], 25}, {NULL}};
const Segment SGI379516[] = {{&S78070[33], 13}, {NULL}};
const Segment SGI379517[] = {{&S78071[95], 6}, {NULL}};
const Segment SGI379518[] = {{&S78071[8], 3}, {NULL}};
const Segment SGI379519[] = {{&S78071[13], 9}, {NULL}};
const Segment SGI379520[] = {{&S78071[33], 15}, {NULL}};
const Segment SGI379521[] = {{&S78071[49], 25}, {NULL}};
const Segment SGI379522[] = {{&S78071[33], 13}, {NULL}};
const Segment SGI379523[] = {{&S8305[75], 105}, {NULL}};
const Segment SGI379524[] = {{&S8305[36], 11}, {NULL}};
const Segment SGI379525[] = {{&S78526[87], 6}, {NULL}};
const Segment SGI379526[] = {{&S78526[8], 3}, {NULL}};
const Segment SGI379527[] = {{&S78526[13], 7}, {NULL}};
const Segment SGI379528[] = {{&S78526[74], 3}, {NULL}};
const Segment SGI379529[] = {{&S78527[105], 6}, {NULL}};
const Segment SGI379530[] = {{&S78527[8], 3}, {NULL}};
const Segment SGI379531[] = {{&S78527[13], 10}, {NULL}};
const Segment SGI379532[] = {{&S78527[81], 3}, {NULL}};
const Segment SGI379533[] = {{&S78573[60], 5}, {NULL}};
const Segment SGI379534[] = {{&S78573[8], 3}, {NULL}};
const Segment SGI379535[] = {{&S78573[13], 7}, {NULL}};
const Segment SGI379536[] = {{&S78573[25], 14}, {NULL}};
const Segment SGI379537[] = {{&S78573[91], 3}, {NULL}};
const Segment SGI379538[] = {{&S78931[8], 3}, {NULL}};
const Segment SGI379539[] = {{&S78931[13], 3}, {NULL}};
const Segment SGI379540[] = {{&S79064[62], 5}, {NULL}};
const Segment SGI379541[] = {{&S79064[8], 3}, {NULL}};
const Segment SGI379542[] = {{&S79064[13], 9}, {NULL}};
const Segment SGI379543[] = {{&S79064[88], 6}, {NULL}};
const Segment SGI379544[] = {{&S79198[60], 6}, {NULL}};
const Segment SGI379545[] = {{&S79198[8], 3}, {NULL}};
const Segment SGI379546[] = {{&S79198[13], 7}, {NULL}};
const Segment SGI379547[] = {{&S79198[25], 14}, {NULL}};
const Segment SGI379548[] = {{&S79198[95], 3}, {NULL}};
const Segment SGI379549[] = {{&S79475[60], 5}, {NULL}};
const Segment SGI379550[] = {{&S79475[8], 3}, {NULL}};
const Segment SGI379551[] = {{&S79475[13], 7}, {NULL}};
const Segment SGI379552[] = {{&S79475[25], 14}, {NULL}};
const Segment SGI379553[] = {{&S79475[91], 3}, {NULL}};
const Segment SGI379554[] = {{&S79476[60], 5}, {NULL}};
const Segment SGI379555[] = {{&S79476[8], 3}, {NULL}};
const Segment SGI379556[] = {{&S79476[13], 7}, {NULL}};
const Segment SGI379557[] = {{&S79476[25], 14}, {NULL}};
const Segment SGI379558[] = {{&S79476[91], 3}, {NULL}};
const Segment SGI379559[] = {{&S79526[41], 6}, {NULL}};
const Segment SGI379560[] = {{&S79526[8], 3}, {NULL}};
const Segment SGI379561[] = {{&S79526[13], 14}, {NULL}};
const Segment SGI379562[] = {{&S79526[78], 3}, {NULL}};
const Segment SGI379563[] = {{&S79527[41], 6}, {NULL}};
const Segment SGI379564[] = {{&S79527[8], 3}, {NULL}};
const Segment SGI379565[] = {{&S79527[13], 14}, {NULL}};
const Segment SGI379566[] = {{&S79527[78], 3}, {NULL}};
const Segment SGI379567[] = {{&S79528[48], 6}, {NULL}};
const Segment SGI379568[] = {{&S79528[8], 3}, {NULL}};
const Segment SGI379569[] = {{&S79528[13], 14}, {NULL}};
const Segment SGI379570[] = {{&S79528[85], 3}, {NULL}};
const Segment SGI379571[] = {{&S82094[13], 33}, {NULL}};
const Segment SGI379572[] = {{&S82094[60], 5}, {NULL}};
const Segment SGI379573[] = {{&S82094[124], 5}, {NULL}};
const Segment SGI379574[] = {{&S79581[60], 5}, {NULL}};
const Segment SGI379575[] = {{&S79581[8], 3}, {NULL}};
const Segment SGI379576[] = {{&S79581[13], 7}, {NULL}};
const Segment SGI379577[] = {{&S79581[25], 14}, {NULL}};
const Segment SGI379578[] = {{&S79581[91], 3}, {NULL}};
const Segment SGI379579[] = {{&S80067[59], 6}, {NULL}};
const Segment SGI379580[] = {{&S80067[8], 3}, {NULL}};
const Segment SGI379581[] = {{&S80067[13], 9}, {NULL}};
const Segment SGI379582[] = {{&S80067[30], 13}, {NULL}};
const Segment SGI379583[] = {{&S80067[95], 3}, {NULL}};
const Segment SGI379584[] = {{&S80068[59], 6}, {NULL}};
const Segment SGI379585[] = {{&S80068[8], 3}, {NULL}};
const Segment SGI379586[] = {{&S80068[13], 9}, {NULL}};
const Segment SGI379587[] = {{&S80068[30], 13}, {NULL}};
const Segment SGI379588[] = {{&S80068[95], 3}, {NULL}};
const Segment SGI379589[] = {{&S80069[59], 6}, {NULL}};
const Segment SGI379590[] = {{&S80069[8], 3}, {NULL}};
const Segment SGI379591[] = {{&S80069[13], 9}, {NULL}};
const Segment SGI379592[] = {{&S80069[30], 13}, {NULL}};
const Segment SGI379593[] = {{&S80069[94], 3}, {NULL}};
const Segment SGI379594[] = {{&S80209[8], 3}, {NULL}};
const Segment SGI379595[] = {{&S80209[13], 9}, {NULL}};
const Segment SGI379596[] = {{&S80209[94], 15}, {NULL}};
const Segment SGI379597[] = {{&S80210[8], 3}, {NULL}};
const Segment SGI379598[] = {{&S80210[13], 9}, {NULL}};
const Segment SGI379599[] = {{&S80210[94], 15}, {NULL}};
const Segment SGI379600[] = {{&S80211[8], 3}, {NULL}};
const Segment SGI379601[] = {{&S80211[13], 9}, {NULL}};
const Segment SGI379602[] = {{&S80211[94], 15}, {NULL}};
const Segment SGI379603[] = {{&S80212[8], 3}, {NULL}};
const Segment SGI379604[] = {{&S80212[13], 9}, {NULL}};
const Segment SGI379605[] = {{&S80212[94], 15}, {NULL}};
const Segment SGI379606[] = {{&S80319[62], 5}, {NULL}};
const Segment SGI379607[] = {{&S80319[8], 3}, {NULL}};
const Segment SGI379608[] = {{&S80319[13], 9}, {NULL}};
const Segment SGI379609[] = {{&S80319[88], 14}, {NULL}};
const Segment SGI379610[] = {{&S80423[44], 6}, {NULL}};
const Segment SGI379611[] = {{&S80423[8], 3}, {NULL}};
const Segment SGI379612[] = {{&S80423[13], 3}, {NULL}};
const Segment SGI379613[] = {{&S80423[71], 10}, {NULL}};
const Segment SGI379614[] = {{&S80619[60], 5}, {NULL}};
const Segment SGI379615[] = {{&S80619[8], 3}, {NULL}};
const Segment SGI379616[] = {{&S80619[13], 7}, {NULL}};
const Segment SGI379617[] = {{&S80619[25], 14}, {NULL}};
const Segment SGI379618[] = {{&S80619[86], 20}, {NULL}};
const Segment SGI379619[] = {{&S80659[60], 5}, {NULL}};
const Segment SGI379620[] = {{&S80659[8], 3}, {NULL}};
const Segment SGI379621[] = {{&S80659[13], 7}, {NULL}};
const Segment SGI379622[] = {{&S80659[25], 14}, {NULL}};
const Segment SGI379623[] = {{&S80659[91], 3}, {NULL}};
const Segment SGI379624[] = {{&S80662[90], 6}, {NULL}};
const Segment SGI379625[] = {{&S80662[8], 3}, {NULL}};
const Segment SGI379626[] = {{&S80662[13], 9}, {NULL}};
const Segment SGI379627[] = {{&S80662[125], 3}, {NULL}};
const Segment SGI379628[] = {{&S80662[33], 13}, {NULL}};
const Segment SGI379629[] = {{&S80750[60], 5}, {NULL}};
const Segment SGI379630[] = {{&S80750[8], 3}, {NULL}};
const Segment SGI379631[] = {{&S80750[13], 7}, {NULL}};
const Segment SGI379632[] = {{&S80750[25], 14}, {NULL}};
const Segment SGI379633[] = {{&S80750[86], 20}, {NULL}};
const Segment SGI379634[] = {{&S80829[59], 6}, {NULL}};
const Segment SGI379635[] = {{&S80829[8], 3}, {NULL}};
const Segment SGI379636[] = {{&S80829[13], 9}, {NULL}};
const Segment SGI379637[] = {{&S80829[105], 3}, {NULL}};
const Segment SGI379638[] = {{&S80829[30], 13}, {NULL}};
const Segment SGI379639[] = {{&S80830[59], 6}, {NULL}};
const Segment SGI379640[] = {{&S80830[8], 3}, {NULL}};
const Segment SGI379641[] = {{&S80830[13], 9}, {NULL}};
const Segment SGI379642[] = {{&S80830[105], 3}, {NULL}};
const Segment SGI379643[] = {{&S80830[30], 13}, {NULL}};
const Segment SGI379644[] = {{&S80837[110], 7}, {NULL}};
const Segment SGI379645[] = {{&S80837[8], 3}, {NULL}};
const Segment SGI379646[] = {{&S80837[13], 9}, {NULL}};
const Segment SGI379647[] = {{&S80837[97], 3}, {NULL}};
const Segment SGI379648[] = {{&S80837[30], 13}, {NULL}};
const Segment SGI379649[] = {{&S80837[121], 12}, {NULL}};
const Segment SGI379650[] = {{&S80945[128], 6}, {NULL}};
const Segment SGI379651[] = {{&S80945[8], 3}, {NULL}};
const Segment SGI379652[] = {{&S80945[13], 9}, {NULL}};
const Segment SGI379653[] = {{&S80945[117], 3}, {NULL}};
const Segment SGI379654[] = {{&S80945[30], 13}, {NULL}};
const Segment SGI379655[] = {{&S80945[98], 3}, {NULL}};
const Segment SGI379656[] = {{&S80946[138], 6}, {NULL}};
const Segment SGI379657[] = {{&S80946[8], 3}, {NULL}};
const Segment SGI379658[] = {{&S80946[13], 9}, {NULL}};
const Segment SGI379659[] = {{&S80946[33], 15}, {NULL}};
const Segment SGI379660[] = {{&S80946[127], 3}, {NULL}};
const Segment SGI379661[] = {{&S80946[49], 6}, {NULL}};
const Segment SGI379662[] = {{&S80946[33], 13}, {NULL}};
const Segment SGI379663[] = {{&S80946[108], 3}, {NULL}};
const Segment SGI379664[] = {{&S80955[46], 5}, {NULL}};
const Segment SGI379665[] = {{&S80955[8], 3}, {NULL}};
const Segment SGI379666[] = {{&S80955[13], 7}, {NULL}};
const Segment SGI379667[] = {{&S80955[72], 14}, {NULL}};
const Segment SGI379668[] = {{&S81188[41], 6}, {NULL}};
const Segment SGI379669[] = {{&S81188[8], 3}, {NULL}};
const Segment SGI379670[] = {{&S81188[13], 14}, {NULL}};
const Segment SGI379671[] = {{&S81188[68], 6}, {NULL}};
const Segment SGI379672[] = {{&S81322[60], 5}, {NULL}};
const Segment SGI379673[] = {{&S81322[8], 3}, {NULL}};
const Segment SGI379674[] = {{&S81322[13], 7}, {NULL}};
const Segment SGI379675[] = {{&S81322[25], 14}, {NULL}};
const Segment SGI379676[] = {{&S81322[86], 14}, {NULL}};
const Segment SGI379677[] = {{&S81426[107], 7}, {NULL}};
const Segment SGI379678[] = {{&S81426[8], 3}, {NULL}};
const Segment SGI379679[] = {{&S81426[13], 9}, {NULL}};
const Segment SGI379680[] = {{&S81426[96], 3}, {NULL}};
const Segment SGI379681[] = {{&S81426[30], 13}, {NULL}};
const Segment SGI379682[] = {{&S81426[117], 12}, {NULL}};
const Segment SGI379683[] = {{&S84390[13], 45}, {NULL}};
const Segment SGI379684[] = {{&S84390[72], 5}, {NULL}};
const Segment SGI379685[] = {{&S84390[106], 3}, {NULL}};
const Segment SGI379686[] = {{&S84403[13], 14}, {NULL}};
const Segment SGI379687[] = {{&S84403[41], 5}, {NULL}};
const Segment SGI379688[] = {{&S84404[13], 14}, {NULL}};
const Segment SGI379689[] = {{&S84404[41], 6}, {NULL}};
const Segment SGI379690[] = {{&S84405[13], 21}, {NULL}};
const Segment SGI379691[] = {{&S84405[48], 6}, {NULL}};
const Segment SGI379692[] = {{&S84405[48], 5}, {NULL}};
const Segment SGI379693[] = {{&S84407[13], 21}, {NULL}};
const Segment SGI379694[] = {{&S84407[48], 6}, {NULL}};
const Segment SGI379695[] = {{&S81680[108], 7}, {NULL}};
const Segment SGI379696[] = {{&S81680[8], 3}, {NULL}};
const Segment SGI379697[] = {{&S81680[13], 9}, {NULL}};
const Segment SGI379698[] = {{&S81680[97], 3}, {NULL}};
const Segment SGI379699[] = {{&S81680[30], 13}, {NULL}};
const Segment SGI379700[] = {{&S81680[119], 12}, {NULL}};
const Segment SGI379701[] = {{&S81681[110], 7}, {NULL}};
const Segment SGI379702[] = {{&S81681[8], 3}, {NULL}};
const Segment SGI379703[] = {{&S81681[13], 9}, {NULL}};
const Segment SGI379704[] = {{&S81681[97], 3}, {NULL}};
const Segment SGI379705[] = {{&S81681[30], 13}, {NULL}};
const Segment SGI379706[] = {{&S81681[121], 12}, {NULL}};
const Segment SGI379707[] = {{&S81754[69], 6}, {NULL}};
const Segment SGI379708[] = {{&S81754[8], 3}, {NULL}};
const Segment SGI379709[] = {{&S81754[13], 9}, {NULL}};
const Segment SGI379710[] = {{&S81754[33], 15}, {NULL}};
const Segment SGI379711[] = {{&S81754[106], 3}, {NULL}};
const Segment SGI379712[] = {{&S81754[49], 6}, {NULL}};
const Segment SGI379713[] = {{&S81754[33], 13}, {NULL}};
const Segment SGI379714[] = {{&S84850[13], 27}, {NULL}};
const Segment SGI379715[] = {{&S84850[54], 6}, {NULL}};
const Segment SGI379716[] = {{&S84850[89], 3}, {NULL}};
const Segment SGI379717[] = {{&S84850[107], 5}, {NULL}};
const Segment SGI379718[] = {{&S84851[13], 23}, {NULL}};
const Segment SGI379719[] = {{&S84851[50], 6}, {NULL}};
const Segment SGI379720[] = {{&S84851[85], 3}, {NULL}};
const Segment SGI379721[] = {{&S84851[103], 5}, {NULL}};
const Segment SGI379722[] = {{&S82104[80], 6}, {NULL}};
const Segment SGI379723[] = {{&S82104[8], 3}, {NULL}};
const Segment SGI379724[] = {{&S82104[13], 9}, {NULL}};
const Segment SGI379725[] = {{&S82104[33], 15}, {NULL}};
const Segment SGI379726[] = {{&S82104[49], 6}, {NULL}};
const Segment SGI379727[] = {{&S82104[33], 13}, {NULL}};
const Segment SGI379728[] = {{&S12705[71], 12}, {NULL}};
const Segment SGI379729[] = {{&S12705[25], 11}, {NULL}};
const Segment SGI379730[] = {{&S82560[60], 5}, {NULL}};
const Segment SGI379731[] = {{&S82560[8], 3}, {NULL}};
const Segment SGI379732[] = {{&S82560[13], 7}, {NULL}};
const Segment SGI379733[] = {{&S82560[25], 14}, {NULL}};
const Segment SGI379734[] = {{&S82560[86], 14}, {NULL}};
const Segment SGI379735[] = {{&S12728[66], 14}, {NULL}};
const Segment SGI379736[] = {{&S12728[23], 11}, {NULL}};
const Segment SGI379737[] = {{&S12751[70], 12}, {NULL}};
const Segment SGI379738[] = {{&S12751[24], 11}, {NULL}};
const Segment SGI379739[] = {{&S82754[57], 6}, {NULL}};
const Segment SGI379740[] = {{&S82754[8], 3}, {NULL}};
const Segment SGI379741[] = {{&S82754[13], 7}, {NULL}};
const Segment SGI379742[] = {{&S82754[25], 14}, {NULL}};
const Segment SGI379743[] = {{&S82754[94], 3}, {NULL}};
const Segment SGI379744[] = {{&S82892[55], 6}, {NULL}};
const Segment SGI379745[] = {{&S82892[8], 3}, {NULL}};
const Segment SGI379746[] = {{&S82892[13], 7}, {NULL}};
const Segment SGI379747[] = {{&S82892[25], 14}, {NULL}};
const Segment SGI379748[] = {{&S82892[82], 7}, {NULL}};
const Segment SGI379749[] = {{&S82895[138], 6}, {NULL}};
const Segment SGI379750[] = {{&S82895[8], 3}, {NULL}};
const Segment SGI379751[] = {{&S82895[13], 9}, {NULL}};
const Segment SGI379752[] = {{&S82895[33], 15}, {NULL}};
const Segment SGI379753[] = {{&S82895[127], 3}, {NULL}};
const Segment SGI379754[] = {{&S82895[49], 6}, {NULL}};
const Segment SGI379755[] = {{&S82895[33], 13}, {NULL}};
const Segment SGI379756[] = {{&S82895[108], 3}, {NULL}};
const Segment SGI379757[] = {{&S82896[138], 6}, {NULL}};
const Segment SGI379758[] = {{&S82896[8], 3}, {NULL}};
const Segment SGI379759[] = {{&S82896[13], 9}, {NULL}};
const Segment SGI379760[] = {{&S82896[33], 15}, {NULL}};
const Segment SGI379761[] = {{&S82896[127], 3}, {NULL}};
const Segment SGI379762[] = {{&S82896[49], 6}, {NULL}};
const Segment SGI379763[] = {{&S82896[33], 13}, {NULL}};
const Segment SGI379764[] = {{&S82896[108], 3}, {NULL}};
const Segment SGI379765[] = {{&S82897[128], 6}, {NULL}};
const Segment SGI379766[] = {{&S82897[8], 3}, {NULL}};
const Segment SGI379767[] = {{&S82897[13], 9}, {NULL}};
const Segment SGI379768[] = {{&S82897[117], 3}, {NULL}};
const Segment SGI379769[] = {{&S82897[30], 13}, {NULL}};
const Segment SGI379770[] = {{&S82897[98], 3}, {NULL}};
const Segment SGI379771[] = {{&S83047[60], 6}, {NULL}};
const Segment SGI379772[] = {{&S83047[8], 3}, {NULL}};
const Segment SGI379773[] = {{&S83047[13], 7}, {NULL}};
const Segment SGI379774[] = {{&S83047[25], 14}, {NULL}};
const Segment SGI379775[] = {{&S83047[97], 3}, {NULL}};
const Segment SGI379776[] = {{&S83420[60], 5}, {NULL}};
const Segment SGI379777[] = {{&S83420[8], 3}, {NULL}};
const Segment SGI379778[] = {{&S83420[13], 7}, {NULL}};
const Segment SGI379779[] = {{&S83420[25], 14}, {NULL}};
const Segment SGI379780[] = {{&S83420[86], 14}, {NULL}};
const Segment SGI379781[] = {{&S83520[105], 6}, {NULL}};
const Segment SGI379782[] = {{&S83520[8], 3}, {NULL}};
const Segment SGI379783[] = {{&S83520[13], 9}, {NULL}};
const Segment SGI379784[] = {{&S83520[30], 13}, {NULL}};
const Segment SGI379785[] = {{&S86134[13], 64}, {NULL}};
const Segment SGI379786[] = {{&S86134[91], 5}, {NULL}};
const Segment SGI379787[] = {{&S86134[125], 3}, {NULL}};
const Segment SGI379788[] = {{&S83679[44], 6}, {NULL}};
const Segment SGI379789[] = {{&S83679[8], 3}, {NULL}};
const Segment SGI379790[] = {{&S83679[13], 3}, {NULL}};
const Segment SGI379791[] = {{&S83679[71], 10}, {NULL}};
const Segment SGI379792[] = {{&S84048[60], 5}, {NULL}};
const Segment SGI379793[] = {{&S84048[8], 3}, {NULL}};
const Segment SGI379794[] = {{&S84048[13], 7}, {NULL}};
const Segment SGI379795[] = {{&S84048[25], 14}, {NULL}};
const Segment SGI379796[] = {{&S84048[86], 14}, {NULL}};
const Segment SGI379797[] = {{&S84092[60], 5}, {NULL}};
const Segment SGI379798[] = {{&S84092[8], 3}, {NULL}};
const Segment SGI379799[] = {{&S84092[13], 7}, {NULL}};
const Segment SGI379800[] = {{&S84092[25], 14}, {NULL}};
const Segment SGI379801[] = {{&S84092[86], 14}, {NULL}};
const Segment SGI379802[] = {{&S84222[69], 6}, {NULL}};
const Segment SGI379803[] = {{&S84222[8], 3}, {NULL}};
const Segment SGI379804[] = {{&S84222[13], 9}, {NULL}};
const Segment SGI379805[] = {{&S84222[33], 15}, {NULL}};
const Segment SGI379806[] = {{&S84222[104], 3}, {NULL}};
const Segment SGI379807[] = {{&S84222[49], 6}, {NULL}};
const Segment SGI379808[] = {{&S84222[33], 13}, {NULL}};
const Segment SGI379809[] = {{&S84222[123], 7}, {NULL}};
const Segment SGI379810[] = {{&S14288[52], 6}, {NULL}};
const Segment SGI379811[] = {{&S14288[23], 11}, {NULL}};
const Segment SGI379812[] = {{&S84275[82], 6}, {NULL}};
const Segment SGI379813[] = {{&S84275[8], 3}, {NULL}};
const Segment SGI379814[] = {{&S84275[13], 9}, {NULL}};
const Segment SGI379815[] = {{&S84275[33], 15}, {NULL}};
const Segment SGI379816[] = {{&S84275[49], 12}, {NULL}};
const Segment SGI379817[] = {{&S84275[33], 13}, {NULL}};
const Segment SGI379818[] = {{&S84285[44], 6}, {NULL}};
const Segment SGI379819[] = {{&S84285[8], 3}, {NULL}};
const Segment SGI379820[] = {{&S84285[13], 3}, {NULL}};
const Segment SGI379821[] = {{&S84285[71], 10}, {NULL}};
const Segment SGI379822[] = {{&S14409[75], 105}, {NULL}};
const Segment SGI379823[] = {{&S14409[36], 11}, {NULL}};
const Segment SGI379824[] = {{&S84776[44], 6}, {NULL}};
const Segment SGI379825[] = {{&S84776[8], 3}, {NULL}};
const Segment SGI379826[] = {{&S84776[13], 7}, {NULL}};
const Segment SGI379827[] = {{&S84776[105], 3}, {NULL}};
const Segment SGI379828[] = {{&S84869[8], 3}, {NULL}};
const Segment SGI379829[] = {{&S84869[13], 7}, {NULL}};
const Segment SGI379830[] = {{&S84869[25], 14}, {NULL}};
const Segment SGI379831[] = {{&S84869[91], 3}, {NULL}};
const Segment SGI379832[] = {{&S84871[46], 5}, {NULL}};
const Segment SGI379833[] = {{&S84871[8], 3}, {NULL}};
const Segment SGI379834[] = {{&S84871[13], 7}, {NULL}};
const Segment SGI379835[] = {{&S84871[72], 16}, {NULL}};
const Segment SGI379836[] = {{&S84971[61], 6}, {NULL}};
const Segment SGI379837[] = {{&S84971[8], 3}, {NULL}};
const Segment SGI379838[] = {{&S84971[13], 7}, {NULL}};
const Segment SGI379839[] = {{&S84971[98], 3}, {NULL}};
const Segment SGI379840[] = {{&S84976[41], 6}, {NULL}};
const Segment SGI379841[] = {{&S84976[8], 3}, {NULL}};
const Segment SGI379842[] = {{&S84976[13], 14}, {NULL}};
const Segment SGI379843[] = {{&S84976[68], 12}, {NULL}};
const Segment SGI379844[] = {{&S84977[48], 6}, {NULL}};
const Segment SGI379845[] = {{&S84977[8], 3}, {NULL}};
const Segment SGI379846[] = {{&S84977[13], 14}, {NULL}};
const Segment SGI379847[] = {{&S84977[75], 12}, {NULL}};
const Segment SGI379848[] = {{&S85075[38], 5}, {NULL}};
const Segment SGI379849[] = {{&S85075[8], 3}, {NULL}};
const Segment SGI379850[] = {{&S85075[13], 7}, {NULL}};
const Segment SGI379851[] = {{&S85075[69], 3}, {NULL}};
const Segment SGI379852[] = {{&S87808[13], 23}, {NULL}};
const Segment SGI379853[] = {{&S87808[50], 6}, {NULL}};
const Segment SGI379854[] = {{&S87808[85], 3}, {NULL}};
const Segment SGI379855[] = {{&S85494[46], 5}, {NULL}};
const Segment SGI379856[] = {{&S85494[8], 3}, {NULL}};
const Segment SGI379857[] = {{&S85494[13], 7}, {NULL}};
const Segment SGI379858[] = {{&S85494[72], 14}, {NULL}};
const Segment SGI379859[] = {{&S85680[41], 6}, {NULL}};
const Segment SGI379860[] = {{&S85680[8], 3}, {NULL}};
const Segment SGI379861[] = {{&S85680[13], 14}, {NULL}};
const Segment SGI379862[] = {{&S85680[68], 12}, {NULL}};
const Segment SGI379863[] = {{&S85953[110], 7}, {NULL}};
const Segment SGI379864[] = {{&S85953[8], 3}, {NULL}};
const Segment SGI379865[] = {{&S85953[13], 9}, {NULL}};
const Segment SGI379866[] = {{&S85953[97], 3}, {NULL}};
const Segment SGI379867[] = {{&S85953[30], 13}, {NULL}};
const Segment SGI379868[] = {{&S85953[121], 12}, {NULL}};
const Segment SGI379869[] = {{&S88830[13], 92}, {NULL}};
const Segment SGI379870[] = {{&S88830[119], 3}, {NULL}};
const Segment SGI379871[] = {{&S88831[124], 3}, {NULL}};
const Segment SGI379872[] = {{&S86117[60], 5}, {NULL}};
const Segment SGI379873[] = {{&S86117[8], 3}, {NULL}};
const Segment SGI379874[] = {{&S86117[13], 7}, {NULL}};
const Segment SGI379875[] = {{&S86117[25], 14}, {NULL}};
const Segment SGI379876[] = {{&S86117[86], 16}, {NULL}};
const Segment SGI379877[] = {{&S86142[46], 5}, {NULL}};
const Segment SGI379878[] = {{&S86142[8], 3}, {NULL}};
const Segment SGI379879[] = {{&S86142[13], 7}, {NULL}};
const Segment SGI379880[] = {{&S86142[72], 14}, {NULL}};
const Segment SGI379881[] = {{&S86235[59], 6}, {NULL}};
const Segment SGI379882[] = {{&S86235[8], 3}, {NULL}};
const Segment SGI379883[] = {{&S86235[13], 9}, {NULL}};
const Segment SGI379884[] = {{&S86235[30], 13}, {NULL}};
const Segment SGI379885[] = {{&S86235[95], 3}, {NULL}};
const Segment SGI379886[] = {{&S86236[59], 6}, {NULL}};
const Segment SGI379887[] = {{&S86236[8], 3}, {NULL}};
const Segment SGI379888[] = {{&S86236[13], 9}, {NULL}};
const Segment SGI379889[] = {{&S86236[30], 13}, {NULL}};
const Segment SGI379890[] = {{&S86236[94], 3}, {NULL}};
const Segment SGI379891[] = {{&S86237[59], 6}, {NULL}};
const Segment SGI379892[] = {{&S86237[8], 3}, {NULL}};
const Segment SGI379893[] = {{&S86237[13], 9}, {NULL}};
const Segment SGI379894[] = {{&S86237[30], 13}, {NULL}};
const Segment SGI379895[] = {{&S86237[94], 3}, {NULL}};
const Segment SGI379896[] = {{&S86262[60], 5}, {NULL}};
const Segment SGI379897[] = {{&S86262[8], 3}, {NULL}};
const Segment SGI379898[] = {{&S86262[13], 7}, {NULL}};
const Segment SGI379899[] = {{&S86262[25], 14}, {NULL}};
const Segment SGI379900[] = {{&S86262[86], 14}, {NULL}};
const Segment SGI379901[] = {{&S89227[13], 85}, {NULL}};
const Segment SGI379902[] = {{&S89227[112], 3}, {NULL}};
const Segment SGI379903[] = {{&S89228[13], 85}, {NULL}};
const Segment SGI379904[] = {{&S89228[112], 3}, {NULL}};
const Segment SGI379905[] = {{&S86464[91], 5}, {NULL}};
const Segment SGI379906[] = {{&S86464[8], 3}, {NULL}};
const Segment SGI379907[] = {{&S86464[13], 7}, {NULL}};
const Segment SGI379908[] = {{&S86464[25], 14}, {NULL}};
const Segment SGI379909[] = {{&S89424[13], 27}, {NULL}};
const Segment SGI379910[] = {{&S89424[54], 5}, {NULL}};
const Segment SGI379911[] = {{&S89424[89], 3}, {NULL}};
const Segment SGI379912[] = {{&S89549[13], 91}, {NULL}};
const Segment SGI379913[] = {{&S89549[118], 3}, {NULL}};
const Segment SGI379914[] = {{&S89550[13], 91}, {NULL}};
const Segment SGI379915[] = {{&S89550[118], 3}, {NULL}};
const Segment SGI379916[] = {{&S89551[13], 91}, {NULL}};
const Segment SGI379917[] = {{&S89551[118], 3}, {NULL}};
const Segment SGI379918[] = {{&S89552[13], 91}, {NULL}};
const Segment SGI379919[] = {{&S89552[118], 3}, {NULL}};
const Segment SGI379920[] = {{&S89553[13], 91}, {NULL}};
const Segment SGI379921[] = {{&S89553[118], 3}, {NULL}};
const Segment SGI379922[] = {{&S89554[13], 91}, {NULL}};
const Segment SGI379923[] = {{&S89554[118], 3}, {NULL}};
const Segment SGI379924[] = {{&S89580[13], 54}, {NULL}};
const Segment SGI379925[] = {{&S89580[81], 6}, {NULL}};
const Segment SGI379926[] = {{&S89580[116], 3}, {NULL}};
const Segment SGI379927[] = {{&S86855[41], 6}, {NULL}};
const Segment SGI379928[] = {{&S86855[8], 3}, {NULL}};
const Segment SGI379929[] = {{&S86855[13], 14}, {NULL}};
const Segment SGI379930[] = {{&S86855[68], 12}, {NULL}};
const Segment SGI379931[] = {{&S86856[41], 6}, {NULL}};
const Segment SGI379932[] = {{&S86856[8], 3}, {NULL}};
const Segment SGI379933[] = {{&S86856[13], 14}, {NULL}};
const Segment SGI379934[] = {{&S86856[68], 12}, {NULL}};
const Segment SGI379935[] = {{&S86919[42], 5}, {NULL}};
const Segment SGI379936[] = {{&S86919[8], 3}, {NULL}};
const Segment SGI379937[] = {{&S86919[13], 3}, {NULL}};
const Segment SGI379938[] = {{&S86919[13], 3}, {&S86919[95], 12}, {NULL}};
const Segment SGI379939[] = {{&S86920[52], 5}, {NULL}};
const Segment SGI379940[] = {{&S86920[8], 3}, {NULL}};
const Segment SGI379941[] = {{&S86920[13], 3}, {NULL}};
const Segment SGI379942[] = {{&S86920[13], 3}, {&S86920[105], 12}, {NULL}};
const Segment SGI379943[] = {{&S87109[87], 5}, {NULL}};
const Segment SGI379944[] = {{&S87109[8], 3}, {NULL}};
const Segment SGI379945[] = {{&S87109[13], 9}, {NULL}};
const Segment SGI379946[] = {{&S87110[87], 6}, {NULL}};
const Segment SGI379947[] = {{&S87110[8], 3}, {NULL}};
const Segment SGI379948[] = {{&S87110[13], 9}, {NULL}};
const Segment SGI379949[] = {{&S87111[87], 6}, {NULL}};
const Segment SGI379950[] = {{&S87111[8], 3}, {NULL}};
const Segment SGI379951[] = {{&S87111[13], 9}, {NULL}};
const Segment SGI379952[] = {{&S87317[48], 6}, {NULL}};
const Segment SGI379953[] = {{&S87317[8], 3}, {NULL}};
const Segment SGI379954[] = {{&S87317[13], 14}, {NULL}};
const Segment SGI379955[] = {{&S87317[85], 3}, {NULL}};
const Segment SGI379956[] = {{&S87319[60], 6}, {NULL}};
const Segment SGI379957[] = {{&S87319[8], 3}, {NULL}};
const Segment SGI379958[] = {{&S87319[13], 7}, {NULL}};
const Segment SGI379959[] = {{&S87319[25], 14}, {NULL}};
const Segment SGI379960[] = {{&S87319[95], 3}, {NULL}};
const Segment SGI379961[] = {{&S87445[62], 5}, {NULL}};
const Segment SGI379962[] = {{&S87445[8], 3}, {NULL}};
const Segment SGI379963[] = {{&S87445[13], 9}, {NULL}};
const Segment SGI379964[] = {{&S87445[88], 14}, {NULL}};
const Segment SGI379965[] = {{&S87491[60], 5}, {NULL}};
const Segment SGI379966[] = {{&S87491[8], 3}, {NULL}};
const Segment SGI379967[] = {{&S87491[13], 7}, {NULL}};
const Segment SGI379968[] = {{&S87491[25], 14}, {NULL}};
const Segment SGI379969[] = {{&S87491[86], 20}, {NULL}};
const Segment SGI379970[] = {{&S17319[75], 105}, {NULL}};
const Segment SGI379971[] = {{&S17319[36], 11}, {NULL}};
const Segment SGI379972[] = {{&S87830[59], 6}, {NULL}};
const Segment SGI379973[] = {{&S87830[8], 3}, {NULL}};
const Segment SGI379974[] = {{&S87830[13], 9}, {NULL}};
const Segment SGI379975[] = {{&S87830[105], 3}, {NULL}};
const Segment SGI379976[] = {{&S87830[30], 13}, {NULL}};
const Segment SGI379977[] = {{&S87831[59], 6}, {NULL}};
const Segment SGI379978[] = {{&S87831[8], 3}, {NULL}};
const Segment SGI379979[] = {{&S87831[13], 9}, {NULL}};
const Segment SGI379980[] = {{&S87831[105], 3}, {NULL}};
const Segment SGI379981[] = {{&S87831[30], 13}, {NULL}};
const Segment SGI379982[] = {{&S87845[109], 7}, {NULL}};
const Segment SGI379983[] = {{&S87845[8], 3}, {NULL}};
const Segment SGI379984[] = {{&S87845[13], 9}, {NULL}};
const Segment SGI379985[] = {{&S87845[96], 3}, {NULL}};
const Segment SGI379986[] = {{&S87845[30], 13}, {NULL}};
const Segment SGI379987[] = {{&S87845[120], 12}, {NULL}};
const Segment SGI379988[] = {{&S87846[109], 7}, {NULL}};
const Segment SGI379989[] = {{&S87846[8], 3}, {NULL}};
const Segment SGI379990[] = {{&S87846[13], 9}, {NULL}};
const Segment SGI379991[] = {{&S87846[96], 3}, {NULL}};
const Segment SGI379992[] = {{&S87846[30], 13}, {NULL}};
const Segment SGI379993[] = {{&S87846[120], 12}, {NULL}};
const Segment SGI379994[] = {{&S88008[60], 5}, {NULL}};
const Segment SGI379995[] = {{&S88008[8], 3}, {NULL}};
const Segment SGI379996[] = {{&S88008[13], 7}, {NULL}};
const Segment SGI379997[] = {{&S88008[25], 14}, {NULL}};
const Segment SGI379998[] = {{&S88008[86], 14}, {NULL}};
const Segment SGI379999[] = {{&S88050[110], 6}, {NULL}};
const Segment SGI380000[] = {{&S88050[8], 3}, {NULL}};
const Segment SGI380001[] = {{&S88050[13], 9}, {NULL}};
const Segment SGI380002[] = {{&S88050[31], 16}, {NULL}};
const Segment SGI380003[] = {{&S88050[31], 13}, {NULL}};
const Segment SGI380004[] = {{&S88050[94], 8}, {NULL}};
const Segment SGI380005[] = {{&S88129[59], 6}, {NULL}};
const Segment SGI380006[] = {{&S88129[8], 3}, {NULL}};
const Segment SGI380007[] = {{&S88129[13], 9}, {NULL}};
const Segment SGI380008[] = {{&S88129[30], 13}, {NULL}};
const Segment SGI380009[] = {{&S88299[89], 6}, {NULL}};
const Segment SGI380010[] = {{&S88299[8], 3}, {NULL}};
const Segment SGI380011[] = {{&S88299[13], 9}, {NULL}};
const Segment SGI380012[] = {{&S88299[33], 15}, {NULL}};
const Segment SGI380013[] = {{&S88299[124], 3}, {NULL}};
const Segment SGI380014[] = {{&S88299[49], 6}, {NULL}};
const Segment SGI380015[] = {{&S88299[33], 13}, {NULL}};
const Segment SGI380016[] = {{&S88596[60], 5}, {NULL}};
const Segment SGI380017[] = {{&S88596[8], 3}, {NULL}};
const Segment SGI380018[] = {{&S88596[13], 7}, {NULL}};
const Segment SGI380019[] = {{&S88596[25], 14}, {NULL}};
const Segment SGI380020[] = {{&S88596[86], 14}, {NULL}};
const Segment SGI380021[] = {{&S18334[66], 14}, {NULL}};
const Segment SGI380022[] = {{&S18334[23], 11}, {NULL}};
const Segment SGI380023[] = {{&S88731[60], 6}, {NULL}};
const Segment SGI380024[] = {{&S88731[8], 3}, {NULL}};
const Segment SGI380025[] = {{&S88731[13], 7}, {NULL}};
const Segment SGI380026[] = {{&S88731[25], 14}, {NULL}};
const Segment SGI380027[] = {{&S88731[36], 3}, {NULL}};
const Segment SGI380028[] = {{&S18962[72], 105}, {NULL}};
const Segment SGI380029[] = {{&S18962[36], 11}, {NULL}};
const Segment SGI380030[] = {{&S89125[42], 6}, {NULL}};
const Segment SGI380031[] = {{&S89125[8], 3}, {NULL}};
const Segment SGI380032[] = {{&S89125[13], 3}, {NULL}};
const Segment SGI380033[] = {{&S89125[69], 12}, {NULL}};
const Segment SGI380034[] = {{&S89233[60], 5}, {NULL}};
const Segment SGI380035[] = {{&S89233[8], 3}, {NULL}};
const Segment SGI380036[] = {{&S89233[13], 7}, {NULL}};
const Segment SGI380037[] = {{&S89233[25], 14}, {NULL}};
const Segment SGI380038[] = {{&S89233[86], 16}, {NULL}};
const Segment SGI380039[] = {{&S89313[107], 7}, {NULL}};
const Segment SGI380040[] = {{&S89313[8], 3}, {NULL}};
const Segment SGI380041[] = {{&S89313[13], 9}, {NULL}};
const Segment SGI380042[] = {{&S89313[96], 3}, {NULL}};
const Segment SGI380043[] = {{&S89313[30], 13}, {NULL}};
const Segment SGI380044[] = {{&S89313[118], 12}, {NULL}};
const Segment SGI380045[] = {{&S19260[69], 12}, {NULL}};
const Segment SGI380046[] = {{&S19260[25], 11}, {NULL}};
const Segment SGI380047[] = {{&S19265[69], 12}, {NULL}};
const Segment SGI380048[] = {{&S19265[25], 11}, {NULL}};
const Segment SGI380049[] = {{&S19273[66], 14}, {NULL}};
const Segment SGI380050[] = {{&S19273[23], 11}, {NULL}};
const Segment SGI380051[] = {{&S89430[60], 5}, {NULL}};
const Segment SGI380052[] = {{&S89430[8], 3}, {NULL}};
const Segment SGI380053[] = {{&S89430[13], 7}, {NULL}};
const Segment SGI380054[] = {{&S89430[25], 14}, {NULL}};
const Segment SGI380055[] = {{&S89430[86], 20}, {NULL}};
const Segment SGI380056[] = {{&S19688[7], 6}, {NULL}};
const Segment SGI380057[] = {{&S89522[48], 6}, {NULL}};
const Segment SGI380058[] = {{&S89522[8], 3}, {NULL}};
const Segment SGI380059[] = {{&S89522[13], 14}, {NULL}};
const Segment SGI380060[] = {{&S89522[75], 12}, {NULL}};
const Segment SGI380061[] = {{&S89523[41], 6}, {NULL}};
const Segment SGI380062[] = {{&S89523[8], 3}, {NULL}};
const Segment SGI380063[] = {{&S89523[13], 14}, {NULL}};
const Segment SGI380064[] = {{&S89523[68], 12}, {NULL}};
const Segment SGI380065[] = {{&S89878[62], 5}, {NULL}};
const Segment SGI380066[] = {{&S89878[8], 3}, {NULL}};
const Segment SGI380067[] = {{&S89878[13], 9}, {NULL}};
const Segment SGI380068[] = {{&S89878[88], 6}, {NULL}};
const Segment SGI380069[] = {{&S89973[41], 6}, {NULL}};
const Segment SGI380070[] = {{&S89973[8], 3}, {NULL}};
const Segment SGI380071[] = {{&S89973[13], 14}, {NULL}};
const Segment SGI380072[] = {{&S89973[68], 12}, {NULL}};
const Segment SGI380073[] = {{&S90038[114], 6}, {NULL}};
const Segment SGI380074[] = {{&S90038[8], 3}, {NULL}};
const Segment SGI380075[] = {{&S90038[13], 9}, {NULL}};
const Segment SGI380076[] = {{&S90038[33], 15}, {NULL}};
const Segment SGI380077[] = {{&S90038[49], 6}, {NULL}};
const Segment SGI380078[] = {{&S90038[33], 13}, {NULL}};
const Segment SGI380079[] = {{&S90038[98], 8}, {NULL}};
const Segment SGI380080[] = {{&S90039[114], 6}, {NULL}};
const Segment SGI380081[] = {{&S90039[8], 3}, {NULL}};
const Segment SGI380082[] = {{&S90039[13], 9}, {NULL}};
const Segment SGI380083[] = {{&S90039[33], 15}, {NULL}};
const Segment SGI380084[] = {{&S90039[49], 6}, {NULL}};
const Segment SGI380085[] = {{&S90039[33], 13}, {NULL}};
const Segment SGI380086[] = {{&S90039[98], 8}, {NULL}};
const Segment SGI380087[] = {{&S90040[104], 6}, {NULL}};
const Segment SGI380088[] = {{&S90040[8], 3}, {NULL}};
const Segment SGI380089[] = {{&S90040[13], 9}, {NULL}};
const Segment SGI380090[] = {{&S90040[30], 13}, {NULL}};
const Segment SGI380091[] = {{&S90040[88], 8}, {NULL}};
const Segment SGI380092[] = {{&S90044[81], 6}, {NULL}};
const Segment SGI380093[] = {{&S90044[8], 3}, {NULL}};
const Segment SGI380094[] = {{&S90044[13], 9}, {NULL}};
const Segment SGI380095[] = {{&S90044[33], 15}, {NULL}};
const Segment SGI380096[] = {{&S90044[49], 11}, {NULL}};
const Segment SGI380097[] = {{&S90044[33], 13}, {NULL}};
const Segment SGI380098[] = {{&S90045[89], 6}, {NULL}};
const Segment SGI380099[] = {{&S90045[8], 3}, {NULL}};
const Segment SGI380100[] = {{&S90045[13], 9}, {NULL}};
const Segment SGI380101[] = {{&S90045[33], 15}, {NULL}};
const Segment SGI380102[] = {{&S90045[49], 19}, {NULL}};
const Segment SGI380103[] = {{&S90045[33], 13}, {NULL}};
const Segment SGI380104[] = {{&S90418[41], 6}, {NULL}};
const Segment SGI380105[] = {{&S90418[8], 3}, {NULL}};
const Segment SGI380106[] = {{&S90418[13], 14}, {NULL}};
const Segment SGI380107[] = {{&S90418[68], 12}, {NULL}};
const Segment SGI380108[] = {{&S90796[115], 6}, {NULL}};
const Segment SGI380109[] = {{&S90796[8], 3}, {NULL}};
const Segment SGI380110[] = {{&S90796[13], 9}, {NULL}};
const Segment SGI380111[] = {{&S90796[33], 15}, {NULL}};
const Segment SGI380112[] = {{&S90796[104], 3}, {NULL}};
const Segment SGI380113[] = {{&S90796[49], 6}, {NULL}};
const Segment SGI380114[] = {{&S90796[33], 13}, {NULL}};
const Segment SGI380115[] = {{&S90797[117], 6}, {NULL}};
const Segment SGI380116[] = {{&S90797[8], 3}, {NULL}};
const Segment SGI380117[] = {{&S90797[13], 9}, {NULL}};
const Segment SGI380118[] = {{&S90797[33], 15}, {NULL}};
const Segment SGI380119[] = {{&S90797[106], 3}, {NULL}};
const Segment SGI380120[] = {{&S90797[49], 6}, {NULL}};
const Segment SGI380121[] = {{&S90797[33], 13}, {NULL}};
const Segment SGI380122[] = {{&S90798[113], 5}, {NULL}};
const Segment SGI380123[] = {{&S90798[8], 3}, {NULL}};
const Segment SGI380124[] = {{&S90798[13], 9}, {NULL}};
const Segment SGI380125[] = {{&S90798[31], 15}, {NULL}};
const Segment SGI380126[] = {{&S90798[102], 3}, {NULL}};
const Segment SGI380127[] = {{&S90798[47], 6}, {NULL}};
const Segment SGI380128[] = {{&S90798[31], 13}, {NULL}};
const Segment SGI380129[] = {{&S94330[13], 21}, {NULL}};
const Segment SGI380130[] = {{&S94330[48], 6}, {NULL}};
const Segment SGI380131[] = {{&S94330[90], 13}, {NULL}};
const Segment SGI380132[] = {{&S94432[13], 36}, {NULL}};
const Segment SGI380133[] = {{&S94432[63], 5}, {NULL}};
const Segment SGI380134[] = {{&S91246[60], 5}, {NULL}};
const Segment SGI380135[] = {{&S91246[8], 3}, {NULL}};
const Segment SGI380136[] = {{&S91246[13], 7}, {NULL}};
const Segment SGI380137[] = {{&S91246[25], 14}, {NULL}};
const Segment SGI380138[] = {{&S91246[86], 14}, {NULL}};
const Segment SGI380139[] = {{&S91321[62], 5}, {NULL}};
const Segment SGI380140[] = {{&S91321[8], 3}, {NULL}};
const Segment SGI380141[] = {{&S91321[13], 9}, {NULL}};
const Segment SGI380142[] = {{&S91321[88], 14}, {NULL}};
const Segment SGI380143[] = {{&S91494[44], 5}, {NULL}};
const Segment SGI380144[] = {{&S91494[8], 3}, {NULL}};
const Segment SGI380145[] = {{&S91494[13], 7}, {NULL}};
const Segment SGI380146[] = {{&S91494[102], 3}, {NULL}};
const Segment SGI380147[] = {{&S91512[60], 5}, {NULL}};
const Segment SGI380148[] = {{&S91512[8], 3}, {NULL}};
const Segment SGI380149[] = {{&S91512[13], 7}, {NULL}};
const Segment SGI380150[] = {{&S91512[25], 14}, {NULL}};
const Segment SGI380151[] = {{&S91512[86], 20}, {NULL}};
const Segment SGI380152[] = {{&S91692[88], 6}, {NULL}};
const Segment SGI380153[] = {{&S91692[8], 3}, {NULL}};
const Segment SGI380154[] = {{&S91692[13], 9}, {NULL}};
const Segment SGI380155[] = {{&S91692[33], 15}, {NULL}};
const Segment SGI380156[] = {{&S91692[49], 18}, {NULL}};
const Segment SGI380157[] = {{&S91692[33], 13}, {NULL}};
const Segment SGI380158[] = {{&S95133[13], 92}, {NULL}};
const Segment SGI380159[] = {{&S95133[119], 3}, {NULL}};
const Segment SGI380160[] = {{&S95134[13], 86}, {NULL}};
const Segment SGI380161[] = {{&S95134[113], 3}, {NULL}};
const Segment SGI380162[] = {{&S91843[48], 6}, {NULL}};
const Segment SGI380163[] = {{&S91843[8], 3}, {NULL}};
const Segment SGI380164[] = {{&S91843[13], 14}, {NULL}};
const Segment SGI380165[] = {{&S91843[85], 3}, {NULL}};
const Segment SGI380166[] = {{&S91903[87], 6}, {NULL}};
const Segment SGI380167[] = {{&S91903[8], 3}, {NULL}};
const Segment SGI380168[] = {{&S91903[13], 9}, {NULL}};
const Segment SGI380169[] = {{&S92381[46], 5}, {NULL}};
const Segment SGI380170[] = {{&S92381[8], 3}, {NULL}};
const Segment SGI380171[] = {{&S92381[13], 7}, {NULL}};
const Segment SGI380172[] = {{&S92381[72], 6}, {NULL}};
const Segment SGI380173[] = {{&S92408[62], 5}, {NULL}};
const Segment SGI380174[] = {{&S92408[8], 3}, {NULL}};
const Segment SGI380175[] = {{&S92408[13], 9}, {NULL}};
const Segment SGI380176[] = {{&S92408[88], 14}, {NULL}};
const Segment SGI380177[] = {{&S92504[86], 6}, {NULL}};
const Segment SGI380178[] = {{&S92504[8], 3}, {NULL}};
const Segment SGI380179[] = {{&S92504[13], 9}, {NULL}};
const Segment SGI380180[] = {{&S92504[33], 15}, {NULL}};
const Segment SGI380181[] = {{&S92504[49], 16}, {NULL}};
const Segment SGI380182[] = {{&S92504[33], 13}, {NULL}};
const Segment SGI380183[] = {{&S22915[65], 9}, {NULL}};
const Segment SGI380184[] = {{&S22915[26], 11}, {NULL}};
const Segment SGI380185[] = {{&S22946[65], 12}, {NULL}};
const Segment SGI380186[] = {{&S22946[25], 11}, {NULL}};
const Segment SGI380187[] = {{&S22977[19], 30}, {NULL}};
const Segment SGI380188[] = {{&S22977[0], 11}, {NULL}};
const Segment SGI380189[] = {{&S93075[87], 6}, {NULL}};
const Segment SGI380190[] = {{&S93075[8], 3}, {NULL}};
const Segment SGI380191[] = {{&S93075[13], 9}, {NULL}};
const Segment SGI380192[] = {{&S93075[33], 15}, {NULL}};
const Segment SGI380193[] = {{&S93075[49], 17}, {NULL}};
const Segment SGI380194[] = {{&S93075[33], 13}, {NULL}};
const Segment SGI380195[] = {{&S23127[88], 82}, {NULL}};
const Segment SGI380196[] = {{&S23127[36], 11}, {NULL}};
const Segment SGI380197[] = {{&S96626[13], 62}, {NULL}};
const Segment SGI380198[] = {{&S96626[89], 6}, {NULL}};
const Segment SGI380199[] = {{&S96626[124], 3}, {NULL}};
const Segment SGI380200[] = {{&S96627[13], 62}, {NULL}};
const Segment SGI380201[] = {{&S96627[89], 6}, {NULL}};
const Segment SGI380202[] = {{&S96627[124], 3}, {NULL}};
const Segment SGI380203[] = {{&S23460[65], 12}, {NULL}};
const Segment SGI380204[] = {{&S23460[25], 11}, {NULL}};
const Segment SGI380205[] = {{&S23462[65], 12}, {NULL}};
const Segment SGI380206[] = {{&S23462[25], 11}, {NULL}};
const Segment SGI380207[] = {{&S23470[64], 14}, {NULL}};
const Segment SGI380208[] = {{&S23470[23], 11}, {NULL}};
const Segment SGI380209[] = {{&S96846[13], 8}, {NULL}};
const Segment SGI380210[] = {{&S96846[35], 8}, {NULL}};
const Segment SGI380211[] = {{&S96846[72], 5}, {NULL}};
const Segment SGI380212[] = {{&S23484[64], 12}, {NULL}};
const Segment SGI380213[] = {{&S23484[24], 11}, {NULL}};
const Segment SGI380214[] = {{&S23491[64], 14}, {NULL}};
const Segment SGI380215[] = {{&S23491[23], 11}, {NULL}};
const Segment SGI380216[] = {{&S97333[13], 10}, {NULL}};
const Segment SGI380217[] = {{&S97333[37], 6}, {NULL}};
const Segment SGI380218[] = {{&S97333[73], 11}, {NULL}};
const Segment SGI380219[] = {{&S94046[46], 5}, {NULL}};
const Segment SGI380220[] = {{&S94046[8], 3}, {NULL}};
const Segment SGI380221[] = {{&S94046[13], 7}, {NULL}};
const Segment SGI380222[] = {{&S94046[72], 16}, {NULL}};
const Segment SGI380223[] = {{&S94054[109], 7}, {NULL}};
const Segment SGI380224[] = {{&S94054[8], 3}, {NULL}};
const Segment SGI380225[] = {{&S94054[13], 9}, {NULL}};
const Segment SGI380226[] = {{&S94054[96], 3}, {NULL}};
const Segment SGI380227[] = {{&S94054[30], 13}, {NULL}};
const Segment SGI380228[] = {{&S94054[120], 12}, {NULL}};
const Segment SGI380229[] = {{&S94250[60], 5}, {NULL}};
const Segment SGI380230[] = {{&S94250[8], 3}, {NULL}};
const Segment SGI380231[] = {{&S94250[13], 7}, {NULL}};
const Segment SGI380232[] = {{&S94250[25], 14}, {NULL}};
const Segment SGI380233[] = {{&S94250[86], 14}, {NULL}};
const Segment SGI380234[] = {{&S24398[65], 12}, {NULL}};
const Segment SGI380235[] = {{&S24398[25], 11}, {NULL}};
const Segment SGI380236[] = {{&S24404[64], 12}, {NULL}};
const Segment SGI380237[] = {{&S24404[24], 11}, {NULL}};
const Segment SGI380238[] = {{&S24408[64], 12}, {NULL}};
const Segment SGI380239[] = {{&S24408[24], 11}, {NULL}};
const Segment SGI380240[] = {{&S24412[64], 14}, {NULL}};
const Segment SGI380241[] = {{&S24412[23], 11}, {NULL}};
const Segment SGI380242[] = {{&S24516[67], 12}, {NULL}};
const Segment SGI380243[] = {{&S24516[25], 11}, {NULL}};
const Segment SGI380244[] = {{&S24546[64], 12}, {NULL}};
const Segment SGI380245[] = {{&S24546[22], 11}, {NULL}};
const Segment SGI380246[] = {{&S24550[66], 14}, {NULL}};
const Segment SGI380247[] = {{&S24550[23], 11}, {NULL}};
const Segment SGI380248[] = {{&S94431[104], 6}, {NULL}};
const Segment SGI380249[] = {{&S94431[8], 3}, {NULL}};
const Segment SGI380250[] = {{&S94431[13], 9}, {NULL}};
const Segment SGI380251[] = {{&S94431[30], 13}, {NULL}};
const Segment SGI380252[] = {{&S94431[88], 8}, {NULL}};
const Segment SGI380253[] = {{&S24567[66], 14}, {NULL}};
const Segment SGI380254[] = {{&S24567[23], 11}, {NULL}};
const Segment SGI380255[] = {{&S98156[13], 9}, {NULL}};
const Segment SGI380256[] = {{&S98156[36], 6}, {NULL}};
const Segment SGI380257[] = {{&S98156[72], 11}, {NULL}};
const Segment SGI380258[] = {{&S94913[59], 6}, {NULL}};
const Segment SGI380259[] = {{&S94913[8], 3}, {NULL}};
const Segment SGI380260[] = {{&S94913[13], 9}, {NULL}};
const Segment SGI380261[] = {{&S94913[30], 13}, {NULL}};
const Segment SGI380262[] = {{&S94913[94], 3}, {NULL}};
const Segment SGI380263[] = {{&S94931[69], 6}, {NULL}};
const Segment SGI380264[] = {{&S94931[8], 3}, {NULL}};
const Segment SGI380265[] = {{&S94931[13], 9}, {NULL}};
const Segment SGI380266[] = {{&S94931[33], 15}, {NULL}};
const Segment SGI380267[] = {{&S94931[49], 6}, {NULL}};
const Segment SGI380268[] = {{&S94931[33], 13}, {NULL}};
const Segment SGI380269[] = {{&S94932[69], 6}, {NULL}};
const Segment SGI380270[] = {{&S94932[8], 3}, {NULL}};
const Segment SGI380271[] = {{&S94932[13], 9}, {NULL}};
const Segment SGI380272[] = {{&S94932[33], 15}, {NULL}};
const Segment SGI380273[] = {{&S94932[49], 6}, {NULL}};
const Segment SGI380274[] = {{&S94932[33], 13}, {NULL}};
const Segment SGI380275[] = {{&S94933[69], 6}, {NULL}};
const Segment SGI380276[] = {{&S94933[8], 3}, {NULL}};
const Segment SGI380277[] = {{&S94933[13], 9}, {NULL}};
const Segment SGI380278[] = {{&S94933[33], 15}, {NULL}};
const Segment SGI380279[] = {{&S94933[49], 6}, {NULL}};
const Segment SGI380280[] = {{&S94933[33], 13}, {NULL}};
const Segment SGI380281[] = {{&S94939[60], 5}, {NULL}};
const Segment SGI380282[] = {{&S94939[8], 3}, {NULL}};
const Segment SGI380283[] = {{&S94939[13], 7}, {NULL}};
const Segment SGI380284[] = {{&S94939[25], 14}, {NULL}};
const Segment SGI380285[] = {{&S94939[86], 14}, {NULL}};
const Segment SGI380286[] = {{&S25059[66], 12}, {NULL}};
const Segment SGI380287[] = {{&S25059[24], 11}, {NULL}};
const Segment SGI380288[] = {{&S25066[39], 10}, {NULL}};
const Segment SGI380289[] = {{&S25066[17], 11}, {NULL}};
const Segment SGI380290[] = {{&S94983[109], 7}, {NULL}};
const Segment SGI380291[] = {{&S94983[8], 3}, {NULL}};
const Segment SGI380292[] = {{&S94983[13], 9}, {NULL}};
const Segment SGI380293[] = {{&S94983[96], 3}, {NULL}};
const Segment SGI380294[] = {{&S94983[30], 13}, {NULL}};
const Segment SGI380295[] = {{&S94983[119], 12}, {NULL}};
const Segment SGI380296[] = {{&S25093[66], 14}, {NULL}};
const Segment SGI380297[] = {{&S25093[23], 11}, {NULL}};
const Segment SGI380298[] = {{&S25130[66], 14}, {NULL}};
const Segment SGI380299[] = {{&S25130[23], 11}, {NULL}};
const Segment SGI380300[] = {{&S25264[70], 12}, {NULL}};
const Segment SGI380301[] = {{&S25264[24], 11}, {NULL}};
const Segment SGI380302[] = {{&S95176[47], 5}, {NULL}};
const Segment SGI380303[] = {{&S95176[8], 3}, {NULL}};
const Segment SGI380304[] = {{&S95176[13], 3}, {NULL}};
const Segment SGI380305[] = {{&S95176[73], 16}, {NULL}};
const Segment SGI380306[] = {{&S25321[15], 11}, {NULL}};
const Segment SGI380307[] = {{&S95223[60], 5}, {NULL}};
const Segment SGI380308[] = {{&S95223[8], 3}, {NULL}};
const Segment SGI380309[] = {{&S95223[13], 7}, {NULL}};
const Segment SGI380310[] = {{&S95223[25], 14}, {NULL}};
const Segment SGI380311[] = {{&S95223[86], 20}, {NULL}};
const Segment SGI380312[] = {{&S25328[60], 9}, {NULL}};
const Segment SGI380313[] = {{&S25328[25], 11}, {NULL}};
const Segment SGI380314[] = {{&S25337[66], 14}, {NULL}};
const Segment SGI380315[] = {{&S25337[23], 11}, {NULL}};
const Segment SGI380316[] = {{&S25339[41], 10}, {NULL}};
const Segment SGI380317[] = {{&S25339[19], 11}, {NULL}};
const Segment SGI380318[] = {{&S95359[108], 6}, {NULL}};
const Segment SGI380319[] = {{&S95359[8], 3}, {NULL}};
const Segment SGI380320[] = {{&S95359[13], 7}, {NULL}};
const Segment SGI380321[] = {{&S95359[25], 14}, {NULL}};
const Segment SGI380322[] = {{&S95359[95], 3}, {NULL}};
const Segment SGI380323[] = {{&S25464[66], 12}, {NULL}};
const Segment SGI380324[] = {{&S25464[24], 11}, {NULL}};
const Segment SGI380325[] = {{&S25478[41], 10}, {NULL}};
const Segment SGI380326[] = {{&S25478[19], 11}, {NULL}};
const Segment SGI380327[] = {{&S25486[66], 14}, {NULL}};
const Segment SGI380328[] = {{&S25486[23], 11}, {NULL}};
const Segment SGI380329[] = {{&S25494[15], 11}, {NULL}};
const Segment SGI380330[] = {{&S95465[60], 5}, {NULL}};
const Segment SGI380331[] = {{&S95465[8], 3}, {NULL}};
const Segment SGI380332[] = {{&S95465[13], 7}, {NULL}};
const Segment SGI380333[] = {{&S95465[25], 14}, {NULL}};
const Segment SGI380334[] = {{&S95465[86], 14}, {NULL}};
const Segment SGI380335[] = {{&S95587[105], 7}, {NULL}};
const Segment SGI380336[] = {{&S95587[8], 3}, {NULL}};
const Segment SGI380337[] = {{&S95587[13], 7}, {NULL}};
const Segment SGI380338[] = {{&S95587[25], 14}, {NULL}};
const Segment SGI380339[] = {{&S95587[94], 3}, {NULL}};
const Segment SGI380340[] = {{&S95717[115], 5}, {NULL}};
const Segment SGI380341[] = {{&S95717[8], 3}, {NULL}};
const Segment SGI380342[] = {{&S95717[13], 9}, {NULL}};
const Segment SGI380343[] = {{&S95717[33], 15}, {NULL}};
const Segment SGI380344[] = {{&S95717[104], 3}, {NULL}};
const Segment SGI380345[] = {{&S95717[49], 6}, {NULL}};
const Segment SGI380346[] = {{&S95717[33], 13}, {NULL}};
const Segment SGI380347[] = {{&S95718[69], 6}, {NULL}};
const Segment SGI380348[] = {{&S95718[8], 3}, {NULL}};
const Segment SGI380349[] = {{&S95718[13], 9}, {NULL}};
const Segment SGI380350[] = {{&S95718[33], 15}, {NULL}};
const Segment SGI380351[] = {{&S95718[104], 3}, {NULL}};
const Segment SGI380352[] = {{&S95718[49], 6}, {NULL}};
const Segment SGI380353[] = {{&S95718[33], 13}, {NULL}};
const Segment SGI380354[] = {{&S99189[13], 42}, {NULL}};
const Segment SGI380355[] = {{&S99189[69], 5}, {NULL}};
const Segment SGI380356[] = {{&S96252[106], 6}, {NULL}};
const Segment SGI380357[] = {{&S96252[8], 3}, {NULL}};
const Segment SGI380358[] = {{&S96252[13], 9}, {NULL}};
const Segment SGI380359[] = {{&S96252[93], 3}, {NULL}};
const Segment SGI380360[] = {{&S96252[30], 13}, {NULL}};
const Segment SGI380361[] = {{&S96364[60], 5}, {NULL}};
const Segment SGI380362[] = {{&S96364[8], 3}, {NULL}};
const Segment SGI380363[] = {{&S96364[13], 7}, {NULL}};
const Segment SGI380364[] = {{&S96364[25], 14}, {NULL}};
const Segment SGI380365[] = {{&S96364[86], 14}, {NULL}};
const Segment SGI380366[] = {{&S96428[37], 6}, {NULL}};
const Segment SGI380367[] = {{&S96428[8], 3}, {NULL}};
const Segment SGI380368[] = {{&S96428[13], 10}, {NULL}};
const Segment SGI380369[] = {{&S96428[74], 3}, {NULL}};
const Segment SGI380370[] = {{&S96652[47], 5}, {NULL}};
const Segment SGI380371[] = {{&S96652[8], 3}, {NULL}};
const Segment SGI380372[] = {{&S96652[13], 3}, {NULL}};
const Segment SGI380373[] = {{&S96652[73], 16}, {NULL}};
const Segment SGI380374[] = {{&S96747[102], 6}, {NULL}};
const Segment SGI380375[] = {{&S96747[8], 3}, {NULL}};
const Segment SGI380376[] = {{&S96747[13], 9}, {NULL}};
const Segment SGI380377[] = {{&S96747[28], 13}, {NULL}};
const Segment SGI380378[] = {{&S96747[86], 8}, {NULL}};
const Segment SGI380379[] = {{&S96959[50], 5}, {NULL}};
const Segment SGI380380[] = {{&S96959[8], 3}, {NULL}};
const Segment SGI380381[] = {{&S96959[13], 3}, {NULL}};
const Segment SGI380382[] = {{&S96959[86], 3}, {NULL}};
const Segment SGI380383[] = {{&S97295[60], 6}, {NULL}};
const Segment SGI380384[] = {{&S97295[8], 3}, {NULL}};
const Segment SGI380385[] = {{&S97295[13], 7}, {NULL}};
const Segment SGI380386[] = {{&S97295[25], 14}, {NULL}};
const Segment SGI380387[] = {{&S97295[95], 3}, {NULL}};
const Segment SGI380388[] = {{&S97296[102], 5}, {NULL}};
const Segment SGI380389[] = {{&S97296[8], 3}, {NULL}};
const Segment SGI380390[] = {{&S97296[13], 7}, {NULL}};
const Segment SGI380391[] = {{&S97296[25], 14}, {NULL}};
const Segment SGI380392[] = {{&S97296[91], 3}, {NULL}};
const Segment SGI380393[] = {{&S27218[22], 16}, {NULL}};
const Segment SGI380394[] = {{&S97318[50], 6}, {NULL}};
const Segment SGI380395[] = {{&S97318[8], 3}, {NULL}};
const Segment SGI380396[] = {{&S97318[13], 9}, {NULL}};
const Segment SGI380397[] = {{&S97344[109], 7}, {NULL}};
const Segment SGI380398[] = {{&S97344[8], 3}, {NULL}};
const Segment SGI380399[] = {{&S97344[13], 9}, {NULL}};
const Segment SGI380400[] = {{&S97344[96], 3}, {NULL}};
const Segment SGI380401[] = {{&S97344[30], 13}, {NULL}};
const Segment SGI380402[] = {{&S97344[119], 12}, {NULL}};
const Segment SGI380403[] = {{&S27425[64], 14}, {NULL}};
const Segment SGI380404[] = {{&S27425[23], 11}, {NULL}};
const Segment SGI380405[] = {{&S97731[109], 7}, {NULL}};
const Segment SGI380406[] = {{&S97731[8], 3}, {NULL}};
const Segment SGI380407[] = {{&S97731[13], 9}, {NULL}};
const Segment SGI380408[] = {{&S97731[96], 3}, {NULL}};
const Segment SGI380409[] = {{&S97731[30], 13}, {NULL}};
const Segment SGI380410[] = {{&S97731[120], 12}, {NULL}};
const Segment SGI380411[] = {{&S101173[13], 94}, {NULL}};
const Segment SGI380412[] = {{&S101173[121], 3}, {NULL}};
const Segment SGI380413[] = {{&S101198[13], 27}, {NULL}};
const Segment SGI380414[] = {{&S101198[54], 5}, {NULL}};
const Segment SGI380415[] = {{&S101199[13], 25}, {NULL}};
const Segment SGI380416[] = {{&S101199[52], 5}, {NULL}};
const Segment SGI380417[] = {{&S98136[69], 6}, {NULL}};
const Segment SGI380418[] = {{&S98136[8], 3}, {NULL}};
const Segment SGI380419[] = {{&S98136[13], 9}, {NULL}};
const Segment SGI380420[] = {{&S98136[33], 15}, {NULL}};
const Segment SGI380421[] = {{&S98136[49], 6}, {NULL}};
const Segment SGI380422[] = {{&S98136[33], 13}, {NULL}};
const Segment SGI380423[] = {{&S98137[69], 6}, {NULL}};
const Segment SGI380424[] = {{&S98137[8], 3}, {NULL}};
const Segment SGI380425[] = {{&S98137[13], 9}, {NULL}};
const Segment SGI380426[] = {{&S98137[33], 15}, {NULL}};
const Segment SGI380427[] = {{&S98137[49], 6}, {NULL}};
const Segment SGI380428[] = {{&S98137[33], 13}, {NULL}};
const Segment SGI380429[] = {{&S98339[48], 6}, {NULL}};
const Segment SGI380430[] = {{&S98339[8], 3}, {NULL}};
const Segment SGI380431[] = {{&S98339[13], 14}, {NULL}};
const Segment SGI380432[] = {{&S98339[75], 6}, {NULL}};
const Segment SGI380433[] = {{&S98429[37], 6}, {NULL}};
const Segment SGI380434[] = {{&S98429[8], 3}, {NULL}};
const Segment SGI380435[] = {{&S98429[13], 10}, {NULL}};
const Segment SGI380436[] = {{&S98429[74], 3}, {NULL}};
const Segment SGI380437[] = {{&S28379[88], 84}, {NULL}};
const Segment SGI380438[] = {{&S28379[71], 101}, {NULL}};
const Segment SGI380439[] = {{&S28379[36], 11}, {NULL}};
const Segment SGI380440[] = {{&S28421[66], 14}, {NULL}};
const Segment SGI380441[] = {{&S28421[23], 11}, {NULL}};
const Segment SGI380442[] = {{&S98706[41], 6}, {NULL}};
const Segment SGI380443[] = {{&S98706[8], 3}, {NULL}};
const Segment SGI380444[] = {{&S98706[13], 14}, {NULL}};
const Segment SGI380445[] = {{&S98706[68], 12}, {NULL}};
const Segment SGI380446[] = {{&S28779[75], 103}, {NULL}};
const Segment SGI380447[] = {{&S28779[36], 11}, {NULL}};
const Segment SGI380448[] = {{&S28835[64], 12}, {NULL}};
const Segment SGI380449[] = {{&S28835[24], 11}, {NULL}};
const Segment SGI380450[] = {{&S28839[52], 9}, {NULL}};
const Segment SGI380451[] = {{&S28839[25], 11}, {NULL}};
const Segment SGI380452[] = {{&S28882[14], 11}, {NULL}};
const Segment SGI380453[] = {{&S28903[51], 17}, {NULL}};
const Segment SGI380454[] = {{&S28903[20], 11}, {NULL}};
const Segment SGI380455[] = {{&S99135[62], 5}, {NULL}};
const Segment SGI380456[] = {{&S99135[8], 3}, {NULL}};
const Segment SGI380457[] = {{&S99135[13], 9}, {NULL}};
const Segment SGI380458[] = {{&S99135[88], 14}, {NULL}};
const Segment SGI380459[] = {{&S102316[13], 22}, {NULL}};
const Segment SGI380460[] = {{&S102316[49], 6}, {NULL}};
const Segment SGI380461[] = {{&S102316[8], 3}, {NULL}};
const Segment SGI380462[] = {{&S102316[104], 14}, {NULL}};
const Segment SGI380463[] = {{&S102316[151], 12}, {NULL}};
const Segment SGI380464[] = {{&S102317[13], 22}, {NULL}};
const Segment SGI380465[] = {{&S102317[49], 6}, {NULL}};
const Segment SGI380466[] = {{&S102317[8], 3}, {NULL}};
const Segment SGI380467[] = {{&S102317[104], 14}, {NULL}};
const Segment SGI380468[] = {{&S102317[151], 12}, {NULL}};
const Segment SGI380469[] = {{&S29544[65], 12}, {NULL}};
const Segment SGI380470[] = {{&S29544[25], 11}, {NULL}};
const Segment SGI380471[] = {{&S29550[65], 12}, {NULL}};
const Segment SGI380472[] = {{&S29550[25], 11}, {NULL}};
const Segment SGI380473[] = {{&S99677[44], 6}, {NULL}};
const Segment SGI380474[] = {{&S99677[8], 3}, {NULL}};
const Segment SGI380475[] = {{&S99677[13], 3}, {NULL}};
const Segment SGI380476[] = {{&S99677[71], 10}, {NULL}};
const Segment SGI380477[] = {{&S103017[13], 95}, {NULL}};
const Segment SGI380478[] = {{&S103017[122], 3}, {NULL}};
const Segment SGI380479[] = {{&S103018[13], 90}, {NULL}};
const Segment SGI380480[] = {{&S103018[117], 3}, {NULL}};
const Segment SGI380481[] = {{&S100183[118], 6}, {NULL}};
const Segment SGI380482[] = {{&S100183[8], 3}, {NULL}};
const Segment SGI380483[] = {{&S100183[13], 9}, {NULL}};
const Segment SGI380484[] = {{&S100183[33], 15}, {NULL}};
const Segment SGI380485[] = {{&S100183[49], 48}, {NULL}};
const Segment SGI380486[] = {{&S100183[33], 13}, {NULL}};
const Segment SGI380487[] = {{&S103210[13], 22}, {NULL}};
const Segment SGI380488[] = {{&S103210[49], 6}, {NULL}};
const Segment SGI380489[] = {{&S103210[8], 3}, {NULL}};
const Segment SGI380490[] = {{&S103210[104], 14}, {NULL}};
const Segment SGI380491[] = {{&S103210[151], 12}, {NULL}};
const Segment SGI380492[] = {{&S103211[13], 22}, {NULL}};
const Segment SGI380493[] = {{&S103211[49], 6}, {NULL}};
const Segment SGI380494[] = {{&S103211[8], 3}, {NULL}};
const Segment SGI380495[] = {{&S103211[104], 14}, {NULL}};
const Segment SGI380496[] = {{&S103211[162], 6}, {NULL}};
const Segment SGI380497[] = {{&S100268[60], 5}, {NULL}};
const Segment SGI380498[] = {{&S100268[8], 3}, {NULL}};
const Segment SGI380499[] = {{&S100268[13], 7}, {NULL}};
const Segment SGI380500[] = {{&S100268[25], 14}, {NULL}};
const Segment SGI380501[] = {{&S100268[86], 20}, {NULL}};
const Segment SGI380502[] = {{&S100321[104], 6}, {NULL}};
const Segment SGI380503[] = {{&S100321[8], 3}, {NULL}};
const Segment SGI380504[] = {{&S100321[13], 9}, {NULL}};
const Segment SGI380505[] = {{&S100321[30], 13}, {NULL}};
const Segment SGI380506[] = {{&S100321[88], 8}, {NULL}};
const Segment SGI380507[] = {{&S100334[107], 7}, {NULL}};
const Segment SGI380508[] = {{&S100334[8], 3}, {NULL}};
const Segment SGI380509[] = {{&S100334[13], 9}, {NULL}};
const Segment SGI380510[] = {{&S100334[96], 3}, {NULL}};
const Segment SGI380511[] = {{&S100334[30], 13}, {NULL}};
const Segment SGI380512[] = {{&S100334[118], 12}, {NULL}};
const Segment SGI380513[] = {{&S103486[13], 22}, {NULL}};
const Segment SGI380514[] = {{&S103486[49], 6}, {NULL}};
const Segment SGI380515[] = {{&S103486[8], 3}, {NULL}};
const Segment SGI380516[] = {{&S103486[104], 14}, {NULL}};
const Segment SGI380517[] = {{&S103486[151], 12}, {NULL}};
const Segment SGI380518[] = {{&S103487[13], 20}, {NULL}};
const Segment SGI380519[] = {{&S103487[47], 6}, {NULL}};
const Segment SGI380520[] = {{&S103487[8], 3}, {NULL}};
const Segment SGI380521[] = {{&S103487[102], 14}, {NULL}};
const Segment SGI380522[] = {{&S103487[160], 12}, {NULL}};
const Segment SGI380523[] = {{&S103488[13], 22}, {NULL}};
const Segment SGI380524[] = {{&S103488[49], 6}, {NULL}};
const Segment SGI380525[] = {{&S103488[8], 3}, {NULL}};
const Segment SGI380526[] = {{&S103488[104], 14}, {NULL}};
const Segment SGI380527[] = {{&S103488[151], 12}, {NULL}};
const Segment SGI380528[] = {{&S103489[13], 22}, {NULL}};
const Segment SGI380529[] = {{&S103489[49], 6}, {NULL}};
const Segment SGI380530[] = {{&S103489[8], 3}, {NULL}};
const Segment SGI380531[] = {{&S103489[104], 14}, {NULL}};
const Segment SGI380532[] = {{&S103489[151], 12}, {NULL}};
const Segment SGI380533[] = {{&S103557[13], 94}, {NULL}};
const Segment SGI380534[] = {{&S103557[121], 3}, {NULL}};
const Segment SGI380535[] = {{&S103558[13], 95}, {NULL}};
const Segment SGI380536[] = {{&S103558[122], 3}, {NULL}};
const Segment SGI380537[] = {{&S30839[75], 107}, {NULL}};
const Segment SGI380538[] = {{&S30839[36], 11}, {NULL}};
const Segment SGI380539[] = {{&S100840[62], 5}, {NULL}};
const Segment SGI380540[] = {{&S100840[8], 3}, {NULL}};
const Segment SGI380541[] = {{&S100840[13], 9}, {NULL}};
const Segment SGI380542[] = {{&S100840[88], 14}, {NULL}};
const Segment SGI380543[] = {{&S100844[82], 6}, {NULL}};
const Segment SGI380544[] = {{&S100844[8], 3}, {NULL}};
const Segment SGI380545[] = {{&S100844[13], 7}, {NULL}};
const Segment SGI380546[] = {{&S100844[25], 14}, {NULL}};
const Segment SGI380547[] = {{&S100844[117], 3}, {NULL}};
const Segment SGI380548[] = {{&S103818[13], 22}, {NULL}};
const Segment SGI380549[] = {{&S103818[49], 6}, {NULL}};
const Segment SGI380550[] = {{&S103818[8], 3}, {NULL}};
const Segment SGI380551[] = {{&S103818[104], 14}, {NULL}};
const Segment SGI380552[] = {{&S103818[151], 12}, {NULL}};
const Segment SGI380553[] = {{&S103843[13], 85}, {NULL}};
const Segment SGI380554[] = {{&S103843[112], 3}, {NULL}};
const Segment SGI380555[] = {{&S103844[13], 85}, {NULL}};
const Segment SGI380556[] = {{&S103844[112], 3}, {NULL}};
const Segment SGI380557[] = {{&S104135[13], 22}, {NULL}};
const Segment SGI380558[] = {{&S104135[49], 6}, {NULL}};
const Segment SGI380559[] = {{&S104135[8], 3}, {NULL}};
const Segment SGI380560[] = {{&S104135[104], 14}, {NULL}};
const Segment SGI380561[] = {{&S104135[151], 12}, {NULL}};
const Segment SGI380562[] = {{&S104136[13], 22}, {NULL}};
const Segment SGI380563[] = {{&S104136[49], 6}, {NULL}};
const Segment SGI380564[] = {{&S104136[8], 3}, {NULL}};
const Segment SGI380565[] = {{&S104136[104], 14}, {NULL}};
const Segment SGI380566[] = {{&S104136[151], 12}, {NULL}};
const Segment SGI380567[] = {{&S104137[13], 22}, {NULL}};
const Segment SGI380568[] = {{&S104137[49], 6}, {NULL}};
const Segment SGI380569[] = {{&S104137[8], 3}, {NULL}};
const Segment SGI380570[] = {{&S104137[104], 14}, {NULL}};
const Segment SGI380571[] = {{&S104137[148], 13}, {NULL}};
const Segment SGI380572[] = {{&S104138[13], 22}, {NULL}};
const Segment SGI380573[] = {{&S104138[49], 6}, {NULL}};
const Segment SGI380574[] = {{&S104138[8], 3}, {NULL}};
const Segment SGI380575[] = {{&S104138[104], 14}, {NULL}};
const Segment SGI380576[] = {{&S104138[151], 13}, {NULL}};
const Segment SGI380577[] = {{&S104346[13], 28}, {NULL}};
const Segment SGI380578[] = {{&S104346[55], 5}, {NULL}};
const Segment SGI380579[] = {{&S104346[96], 14}, {NULL}};
const Segment SGI380580[] = {{&S104346[141], 12}, {NULL}};
const Segment SGI380581[] = {{&S104539[13], 21}, {NULL}};
const Segment SGI380582[] = {{&S104539[48], 5}, {NULL}};
const Segment SGI380583[] = {{&S104539[89], 14}, {NULL}};
const Segment SGI380584[] = {{&S104539[136], 12}, {NULL}};
const Segment SGI380585[] = {{&S104540[13], 21}, {NULL}};
const Segment SGI380586[] = {{&S104540[48], 5}, {NULL}};
const Segment SGI380587[] = {{&S104540[89], 14}, {NULL}};
const Segment SGI380588[] = {{&S104540[136], 13}, {NULL}};
const Segment SGI380589[] = {{&S104541[13], 21}, {NULL}};
const Segment SGI380590[] = {{&S104541[48], 5}, {NULL}};
const Segment SGI380591[] = {{&S104541[89], 14}, {NULL}};
const Segment SGI380592[] = {{&S104541[136], 12}, {NULL}};
const Segment SGI380593[] = {{&S104542[13], 21}, {NULL}};
const Segment SGI380594[] = {{&S104542[48], 5}, {NULL}};
const Segment SGI380595[] = {{&S104542[89], 14}, {NULL}};
const Segment SGI380596[] = {{&S104542[136], 12}, {NULL}};
const Segment SGI380597[] = {{&S104543[13], 21}, {NULL}};
const Segment SGI380598[] = {{&S104543[48], 5}, {NULL}};
const Segment SGI380599[] = {{&S104543[89], 14}, {NULL}};
const Segment SGI380600[] = {{&S104543[136], 12}, {NULL}};
const Segment SGI380601[] = {{&S104544[13], 22}, {NULL}};
const Segment SGI380602[] = {{&S104544[49], 6}, {NULL}};
const Segment SGI380603[] = {{&S104544[8], 3}, {NULL}};
const Segment SGI380604[] = {{&S104544[104], 14}, {NULL}};
const Segment SGI380605[] = {{&S104544[151], 12}, {NULL}};
const Segment SGI380606[] = {{&S104545[13], 22}, {NULL}};
const Segment SGI380607[] = {{&S104545[49], 6}, {NULL}};
const Segment SGI380608[] = {{&S104545[8], 3}, {NULL}};
const Segment SGI380609[] = {{&S104545[104], 14}, {NULL}};
const Segment SGI380610[] = {{&S104545[151], 12}, {NULL}};
const Segment SGI380611[] = {{&S104546[13], 22}, {NULL}};
const Segment SGI380612[] = {{&S104546[49], 6}, {NULL}};
const Segment SGI380613[] = {{&S104546[8], 3}, {NULL}};
const Segment SGI380614[] = {{&S104546[104], 14}, {NULL}};
const Segment SGI380615[] = {{&S104546[151], 12}, {NULL}};
const Segment SGI380616[] = {{&S104547[13], 22}, {NULL}};
const Segment SGI380617[] = {{&S104547[49], 6}, {NULL}};
const Segment SGI380618[] = {{&S104547[8], 3}, {NULL}};
const Segment SGI380619[] = {{&S104547[104], 14}, {NULL}};
const Segment SGI380620[] = {{&S104547[148], 12}, {NULL}};
const Segment SGI380621[] = {{&S104548[13], 22}, {NULL}};
const Segment SGI380622[] = {{&S104548[49], 6}, {NULL}};
const Segment SGI380623[] = {{&S104548[8], 3}, {NULL}};
const Segment SGI380624[] = {{&S104548[104], 14}, {NULL}};
const Segment SGI380625[] = {{&S104548[151], 12}, {NULL}};
const Segment SGI380626[] = {{&S104549[13], 22}, {NULL}};
const Segment SGI380627[] = {{&S104549[49], 6}, {NULL}};
const Segment SGI380628[] = {{&S104549[8], 3}, {NULL}};
const Segment SGI380629[] = {{&S104549[104], 14}, {NULL}};
const Segment SGI380630[] = {{&S104549[151], 12}, {NULL}};
const Segment SGI380631[] = {{&S101843[60], 5}, {NULL}};
const Segment SGI380632[] = {{&S101843[8], 3}, {NULL}};
const Segment SGI380633[] = {{&S101843[13], 7}, {NULL}};
const Segment SGI380634[] = {{&S101843[25], 14}, {NULL}};
const Segment SGI380635[] = {{&S101843[86], 6}, {NULL}};
const Segment SGI380636[] = {{&S101888[60], 5}, {NULL}};
const Segment SGI380637[] = {{&S101888[8], 3}, {NULL}};
const Segment SGI380638[] = {{&S101888[13], 7}, {NULL}};
const Segment SGI380639[] = {{&S101888[25], 14}, {NULL}};
const Segment SGI380640[] = {{&S101888[86], 14}, {NULL}};
const Segment SGI380641[] = {{&S101964[107], 6}, {NULL}};
const Segment SGI380642[] = {{&S101964[8], 3}, {NULL}};
const Segment SGI380643[] = {{&S101964[13], 9}, {NULL}};
const Segment SGI380644[] = {{&S101964[94], 3}, {NULL}};
const Segment SGI380645[] = {{&S102238[60], 5}, {NULL}};
const Segment SGI380646[] = {{&S102238[8], 3}, {NULL}};
const Segment SGI380647[] = {{&S102238[13], 7}, {NULL}};
const Segment SGI380648[] = {{&S102238[25], 14}, {NULL}};
const Segment SGI380649[] = {{&S102238[86], 14}, {NULL}};
const Segment SGI380650[] = {{&S105047[13], 21}, {NULL}};
const Segment SGI380651[] = {{&S105047[48], 5}, {NULL}};
const Segment SGI380652[] = {{&S105047[89], 14}, {NULL}};
const Segment SGI380653[] = {{&S105047[136], 12}, {NULL}};
const Segment SGI380654[] = {{&S105048[13], 21}, {NULL}};
const Segment SGI380655[] = {{&S105048[48], 5}, {NULL}};
const Segment SGI380656[] = {{&S105048[89], 14}, {NULL}};
const Segment SGI380657[] = {{&S105048[136], 12}, {NULL}};
const Segment SGI380658[] = {{&S105049[13], 21}, {NULL}};
const Segment SGI380659[] = {{&S105049[48], 5}, {NULL}};
const Segment SGI380660[] = {{&S105049[89], 14}, {NULL}};
const Segment SGI380661[] = {{&S105049[136], 12}, {NULL}};
const Segment SGI380662[] = {{&S105050[13], 22}, {NULL}};
const Segment SGI380663[] = {{&S105050[49], 6}, {NULL}};
const Segment SGI380664[] = {{&S105050[8], 3}, {NULL}};
const Segment SGI380665[] = {{&S105050[104], 14}, {NULL}};
const Segment SGI380666[] = {{&S105050[151], 12}, {NULL}};
const Segment SGI380667[] = {{&S105051[13], 22}, {NULL}};
const Segment SGI380668[] = {{&S105051[49], 6}, {NULL}};
const Segment SGI380669[] = {{&S105051[8], 3}, {NULL}};
const Segment SGI380670[] = {{&S105051[104], 14}, {NULL}};
const Segment SGI380671[] = {{&S105051[151], 12}, {NULL}};
const Segment SGI380672[] = {{&S105052[13], 28}, {NULL}};
const Segment SGI380673[] = {{&S105052[55], 5}, {NULL}};
const Segment SGI380674[] = {{&S105052[96], 14}, {NULL}};
const Segment SGI380675[] = {{&S105052[135], 12}, {NULL}};
const Segment SGI380676[] = {{&S105053[13], 27}, {NULL}};
const Segment SGI380677[] = {{&S105053[54], 5}, {NULL}};
const Segment SGI380678[] = {{&S105053[97], 14}, {NULL}};
const Segment SGI380679[] = {{&S105053[142], 11}, {NULL}};
const Segment SGI380680[] = {{&S105054[13], 22}, {NULL}};
const Segment SGI380681[] = {{&S105054[49], 6}, {NULL}};
const Segment SGI380682[] = {{&S105054[8], 3}, {NULL}};
const Segment SGI380683[] = {{&S105054[104], 14}, {NULL}};
const Segment SGI380684[] = {{&S105054[151], 12}, {NULL}};
const Segment SGI380685[] = {{&S105055[13], 22}, {NULL}};
const Segment SGI380686[] = {{&S105055[49], 6}, {NULL}};
const Segment SGI380687[] = {{&S105055[8], 3}, {NULL}};
const Segment SGI380688[] = {{&S105055[104], 14}, {NULL}};
const Segment SGI380689[] = {{&S105055[151], 12}, {NULL}};
const Segment SGI380690[] = {{&S105056[13], 22}, {NULL}};
const Segment SGI380691[] = {{&S105056[49], 6}, {NULL}};
const Segment SGI380692[] = {{&S105056[8], 3}, {NULL}};
const Segment SGI380693[] = {{&S105056[104], 14}, {NULL}};
const Segment SGI380694[] = {{&S105056[151], 12}, {NULL}};
const Segment SGI380695[] = {{&S105057[13], 22}, {NULL}};
const Segment SGI380696[] = {{&S105057[49], 6}, {NULL}};
const Segment SGI380697[] = {{&S105057[8], 3}, {NULL}};
const Segment SGI380698[] = {{&S105057[104], 14}, {NULL}};
const Segment SGI380699[] = {{&S105057[151], 12}, {NULL}};
const Segment SGI380700[] = {{&S105058[13], 22}, {NULL}};
const Segment SGI380701[] = {{&S105058[49], 6}, {NULL}};
const Segment SGI380702[] = {{&S105058[8], 3}, {NULL}};
const Segment SGI380703[] = {{&S105058[104], 14}, {NULL}};
const Segment SGI380704[] = {{&S105058[151], 12}, {NULL}};
const Segment SGI380705[] = {{&S105059[13], 22}, {NULL}};
const Segment SGI380706[] = {{&S105059[49], 6}, {NULL}};
const Segment SGI380707[] = {{&S105059[8], 3}, {NULL}};
const Segment SGI380708[] = {{&S105059[104], 14}, {NULL}};
const Segment SGI380709[] = {{&S105059[151], 12}, {NULL}};
const Segment SGI380710[] = {{&S105060[13], 22}, {NULL}};
const Segment SGI380711[] = {{&S105060[49], 6}, {NULL}};
const Segment SGI380712[] = {{&S105060[8], 3}, {NULL}};
const Segment SGI380713[] = {{&S105060[104], 14}, {NULL}};
const Segment SGI380714[] = {{&S105060[151], 12}, {NULL}};
const Segment SGI380715[] = {{&S105061[13], 22}, {NULL}};
const Segment SGI380716[] = {{&S105061[49], 6}, {NULL}};
const Segment SGI380717[] = {{&S105061[8], 3}, {NULL}};
const Segment SGI380718[] = {{&S105061[104], 14}, {NULL}};
const Segment SGI380719[] = {{&S105061[148], 12}, {NULL}};
const Segment SGI380720[] = {{&S102247[69], 6}, {NULL}};
const Segment SGI380721[] = {{&S102247[8], 3}, {NULL}};
const Segment SGI380722[] = {{&S102247[13], 9}, {NULL}};
const Segment SGI380723[] = {{&S102247[33], 15}, {NULL}};
const Segment SGI380724[] = {{&S102247[49], 6}, {NULL}};
const Segment SGI380725[] = {{&S102247[33], 13}, {NULL}};
const Segment SGI380726[] = {{&S102461[59], 6}, {NULL}};
const Segment SGI380727[] = {{&S102461[8], 3}, {NULL}};
const Segment SGI380728[] = {{&S102461[13], 9}, {NULL}};
const Segment SGI380729[] = {{&S102461[105], 3}, {NULL}};
const Segment SGI380730[] = {{&S102461[30], 13}, {NULL}};
const Segment SGI380731[] = {{&S105156[13], 85}, {NULL}};
const Segment SGI380732[] = {{&S105156[112], 3}, {NULL}};
const Segment SGI380733[] = {{&S102640[106], 5}, {NULL}};
const Segment SGI380734[] = {{&S102640[8], 3}, {NULL}};
const Segment SGI380735[] = {{&S102640[13], 9}, {NULL}};
const Segment SGI380736[] = {{&S102640[113], 10}, {NULL}};
const Segment SGI380737[] = {{&S102696[8], 3}, {NULL}};
const Segment SGI380738[] = {{&S102696[13], 5}, {NULL}};
const Segment SGI380739[] = {{&S102696[145], 3}, {NULL}};
const Segment SGI380740[] = {{&S102696[31], 3}, {NULL}};
const Segment SGI380741[] = {{&S102697[8], 3}, {NULL}};
const Segment SGI380742[] = {{&S102697[13], 5}, {NULL}};
const Segment SGI380743[] = {{&S102697[145], 3}, {NULL}};
const Segment SGI380744[] = {{&S102697[31], 3}, {NULL}};
const Segment SGI380745[] = {{&S105408[13], 32}, {NULL}};
const Segment SGI380746[] = {{&S105408[59], 3}, {NULL}};
const Segment SGI380747[] = {{&S105408[92], 3}, {NULL}};
const Segment SGI380748[] = {{&S105408[117], 5}, {NULL}};
const Segment SGI380749[] = {{&S105490[13], 27}, {NULL}};
const Segment SGI380750[] = {{&S105490[54], 5}, {NULL}};
const Segment SGI380751[] = {{&S105490[95], 14}, {NULL}};
const Segment SGI380752[] = {{&S105490[140], 13}, {NULL}};
const Segment SGI380753[] = {{&S105491[13], 28}, {NULL}};
const Segment SGI380754[] = {{&S105491[55], 5}, {NULL}};
const Segment SGI380755[] = {{&S105491[96], 14}, {NULL}};
const Segment SGI380756[] = {{&S105491[132], 13}, {NULL}};
const Segment SGI380757[] = {{&S32967[75], 103}, {NULL}};
const Segment SGI380758[] = {{&S32967[36], 11}, {NULL}};
const Segment SGI380759[] = {{&S105932[13], 21}, {NULL}};
const Segment SGI380760[] = {{&S105932[48], 5}, {NULL}};
const Segment SGI380761[] = {{&S105932[89], 14}, {NULL}};
const Segment SGI380762[] = {{&S105932[136], 12}, {NULL}};
const Segment SGI380763[] = {{&S105933[13], 21}, {NULL}};
const Segment SGI380764[] = {{&S105933[48], 5}, {NULL}};
const Segment SGI380765[] = {{&S105933[89], 14}, {NULL}};
const Segment SGI380766[] = {{&S105933[136], 12}, {NULL}};
const Segment SGI380767[] = {{&S105934[13], 21}, {NULL}};
const Segment SGI380768[] = {{&S105934[48], 5}, {NULL}};
const Segment SGI380769[] = {{&S105934[89], 14}, {NULL}};
const Segment SGI380770[] = {{&S105934[136], 12}, {NULL}};
const Segment SGI380771[] = {{&S105935[13], 21}, {NULL}};
const Segment SGI380772[] = {{&S105935[48], 5}, {NULL}};
const Segment SGI380773[] = {{&S105935[89], 14}, {NULL}};
const Segment SGI380774[] = {{&S105935[136], 12}, {NULL}};
const Segment SGI380775[] = {{&S105936[13], 21}, {NULL}};
const Segment SGI380776[] = {{&S105936[48], 5}, {NULL}};
const Segment SGI380777[] = {{&S105936[89], 14}, {NULL}};
const Segment SGI380778[] = {{&S105936[136], 12}, {NULL}};
const Segment SGI380779[] = {{&S105937[13], 21}, {NULL}};
const Segment SGI380780[] = {{&S105937[48], 5}, {NULL}};
const Segment SGI380781[] = {{&S105937[89], 14}, {NULL}};
const Segment SGI380782[] = {{&S105937[136], 12}, {NULL}};
const Segment SGI380783[] = {{&S105938[13], 21}, {NULL}};
const Segment SGI380784[] = {{&S105938[48], 5}, {NULL}};
const Segment SGI380785[] = {{&S105938[89], 14}, {NULL}};
const Segment SGI380786[] = {{&S105938[136], 12}, {NULL}};
const Segment SGI380787[] = {{&S105939[13], 21}, {NULL}};
const Segment SGI380788[] = {{&S105939[48], 5}, {NULL}};
const Segment SGI380789[] = {{&S105939[89], 14}, {NULL}};
const Segment SGI380790[] = {{&S105939[136], 12}, {NULL}};
const Segment SGI380791[] = {{&S105940[13], 21}, {NULL}};
const Segment SGI380792[] = {{&S105940[48], 5}, {NULL}};
const Segment SGI380793[] = {{&S105940[89], 14}, {NULL}};
const Segment SGI380794[] = {{&S105940[136], 12}, {NULL}};
const Segment SGI380795[] = {{&S105941[13], 21}, {NULL}};
const Segment SGI380796[] = {{&S105941[48], 5}, {NULL}};
const Segment SGI380797[] = {{&S105941[89], 14}, {NULL}};
const Segment SGI380798[] = {{&S105941[133], 12}, {NULL}};
const Segment SGI380799[] = {{&S105942[13], 21}, {NULL}};
const Segment SGI380800[] = {{&S105942[48], 5}, {NULL}};
const Segment SGI380801[] = {{&S105942[89], 14}, {NULL}};
const Segment SGI380802[] = {{&S105942[136], 12}, {NULL}};
const Segment SGI380803[] = {{&S105943[13], 28}, {NULL}};
const Segment SGI380804[] = {{&S105943[55], 5}, {NULL}};
const Segment SGI380805[] = {{&S105943[96], 14}, {NULL}};
const Segment SGI380806[] = {{&S105943[149], 12}, {NULL}};
const Segment SGI380807[] = {{&S105944[13], 22}, {NULL}};
const Segment SGI380808[] = {{&S105944[49], 6}, {NULL}};
const Segment SGI380809[] = {{&S105944[8], 3}, {NULL}};
const Segment SGI380810[] = {{&S105944[104], 14}, {NULL}};
const Segment SGI380811[] = {{&S105944[151], 12}, {NULL}};
const Segment SGI380812[] = {{&S105945[13], 22}, {NULL}};
const Segment SGI380813[] = {{&S105945[49], 6}, {NULL}};
const Segment SGI380814[] = {{&S105945[8], 3}, {NULL}};
const Segment SGI380815[] = {{&S105945[104], 14}, {NULL}};
const Segment SGI380816[] = {{&S105945[148], 12}, {NULL}};
const Segment SGI380817[] = {{&S105946[13], 28}, {NULL}};
const Segment SGI380818[] = {{&S105946[55], 5}, {NULL}};
const Segment SGI380819[] = {{&S105946[96], 14}, {NULL}};
const Segment SGI380820[] = {{&S105946[138], 12}, {NULL}};
const Segment SGI380821[] = {{&S105947[13], 22}, {NULL}};
const Segment SGI380822[] = {{&S105947[49], 6}, {NULL}};
const Segment SGI380823[] = {{&S105947[8], 3}, {NULL}};
const Segment SGI380824[] = {{&S105947[104], 14}, {NULL}};
const Segment SGI380825[] = {{&S105947[148], 12}, {NULL}};
const Segment SGI380826[] = {{&S105948[13], 28}, {NULL}};
const Segment SGI380827[] = {{&S105948[55], 5}, {NULL}};
const Segment SGI380828[] = {{&S105948[96], 14}, {NULL}};
const Segment SGI380829[] = {{&S105948[149], 12}, {NULL}};
const Segment SGI380830[] = {{&S105949[13], 22}, {NULL}};
const Segment SGI380831[] = {{&S105949[49], 6}, {NULL}};
const Segment SGI380832[] = {{&S105949[8], 3}, {NULL}};
const Segment SGI380833[] = {{&S105949[104], 14}, {NULL}};
const Segment SGI380834[] = {{&S105949[151], 12}, {NULL}};
const Segment SGI380835[] = {{&S105950[13], 22}, {NULL}};
const Segment SGI380836[] = {{&S105950[49], 6}, {NULL}};
const Segment SGI380837[] = {{&S105950[8], 3}, {NULL}};
const Segment SGI380838[] = {{&S105950[104], 14}, {NULL}};
const Segment SGI380839[] = {{&S105950[151], 12}, {NULL}};
const Segment SGI380840[] = {{&S105951[13], 22}, {NULL}};
const Segment SGI380841[] = {{&S105951[49], 6}, {NULL}};
const Segment SGI380842[] = {{&S105951[8], 3}, {NULL}};
const Segment SGI380843[] = {{&S105951[104], 14}, {NULL}};
const Segment SGI380844[] = {{&S105951[151], 12}, {NULL}};
const Segment SGI380845[] = {{&S105952[13], 22}, {NULL}};
const Segment SGI380846[] = {{&S105952[49], 6}, {NULL}};
const Segment SGI380847[] = {{&S105952[8], 3}, {NULL}};
const Segment SGI380848[] = {{&S105952[104], 14}, {NULL}};
const Segment SGI380849[] = {{&S105952[151], 12}, {NULL}};
const Segment SGI380850[] = {{&S105953[13], 22}, {NULL}};
const Segment SGI380851[] = {{&S105953[49], 6}, {NULL}};
const Segment SGI380852[] = {{&S105953[8], 3}, {NULL}};
const Segment SGI380853[] = {{&S105953[104], 14}, {NULL}};
const Segment SGI380854[] = {{&S105953[148], 12}, {NULL}};
const Segment SGI380855[] = {{&S105954[13], 28}, {NULL}};
const Segment SGI380856[] = {{&S105954[55], 5}, {NULL}};
const Segment SGI380857[] = {{&S105954[96], 14}, {NULL}};
const Segment SGI380858[] = {{&S105954[141], 11}, {NULL}};
const Segment SGI380859[] = {{&S105955[13], 22}, {NULL}};
const Segment SGI380860[] = {{&S105955[49], 6}, {NULL}};
const Segment SGI380861[] = {{&S105955[8], 3}, {NULL}};
const Segment SGI380862[] = {{&S105955[104], 14}, {NULL}};
const Segment SGI380863[] = {{&S105955[151], 12}, {NULL}};
const Segment SGI380864[] = {{&S105956[13], 22}, {NULL}};
const Segment SGI380865[] = {{&S105956[49], 6}, {NULL}};
const Segment SGI380866[] = {{&S105956[8], 3}, {NULL}};
const Segment SGI380867[] = {{&S105956[104], 14}, {NULL}};
const Segment SGI380868[] = {{&S105956[151], 12}, {NULL}};
const Segment SGI380869[] = {{&S106027[13], 28}, {NULL}};
const Segment SGI380870[] = {{&S106027[55], 5}, {NULL}};
const Segment SGI380871[] = {{&S106027[96], 14}, {NULL}};
const Segment SGI380872[] = {{&S106027[132], 12}, {NULL}};
const Segment SGI380873[] = {{&S106028[13], 28}, {NULL}};
const Segment SGI380874[] = {{&S106028[55], 5}, {NULL}};
const Segment SGI380875[] = {{&S106028[96], 14}, {NULL}};
const Segment SGI380876[] = {{&S106028[138], 12}, {NULL}};
const Segment SGI380877[] = {{&S106029[13], 27}, {NULL}};
const Segment SGI380878[] = {{&S106029[54], 5}, {NULL}};
const Segment SGI380879[] = {{&S106029[95], 14}, {NULL}};
const Segment SGI380880[] = {{&S106029[149], 12}, {NULL}};
const Segment SGI380881[] = {{&S106030[13], 28}, {NULL}};
const Segment SGI380882[] = {{&S106030[55], 5}, {NULL}};
const Segment SGI380883[] = {{&S106030[96], 14}, {NULL}};
const Segment SGI380884[] = {{&S106030[152], 12}, {NULL}};
const Segment SGI380885[] = {{&S106031[13], 27}, {NULL}};
const Segment SGI380886[] = {{&S106031[54], 5}, {NULL}};
const Segment SGI380887[] = {{&S106031[97], 14}, {NULL}};
const Segment SGI380888[] = {{&S106031[143], 12}, {NULL}};
const Segment SGI380889[] = {{&S106032[13], 27}, {NULL}};
const Segment SGI380890[] = {{&S106032[54], 5}, {NULL}};
const Segment SGI380891[] = {{&S106032[95], 14}, {NULL}};
const Segment SGI380892[] = {{&S106032[132], 11}, {NULL}};
const Segment SGI380893[] = {{&S103682[107], 7}, {NULL}};
const Segment SGI380894[] = {{&S103682[8], 3}, {NULL}};
const Segment SGI380895[] = {{&S103682[13], 9}, {NULL}};
const Segment SGI380896[] = {{&S103682[96], 3}, {NULL}};
const Segment SGI380897[] = {{&S103682[30], 13}, {NULL}};
const Segment SGI380898[] = {{&S103682[118], 12}, {NULL}};
const Segment SGI380899[] = {{&S103691[52], 6}, {NULL}};
const Segment SGI380900[] = {{&S103691[8], 3}, {NULL}};
const Segment SGI380901[] = {{&S103691[13], 9}, {NULL}};
const Segment SGI380902[] = {{&S103691[89], 3}, {NULL}};
const Segment SGI380903[] = {{&S103730[48], 6}, {NULL}};
const Segment SGI380904[] = {{&S103730[8], 3}, {NULL}};
const Segment SGI380905[] = {{&S103730[13], 14}, {NULL}};
const Segment SGI380906[] = {{&S103730[85], 3}, {NULL}};
const Segment SGI380907[] = {{&S103760[8], 3}, {NULL}};
const Segment SGI380908[] = {{&S103760[13], 5}, {NULL}};
const Segment SGI380909[] = {{&S103760[145], 3}, {NULL}};
const Segment SGI380910[] = {{&S103760[31], 3}, {NULL}};
const Segment SGI380911[] = {{&S103761[8], 3}, {NULL}};
const Segment SGI380912[] = {{&S103761[13], 5}, {NULL}};
const Segment SGI380913[] = {{&S103761[145], 3}, {NULL}};
const Segment SGI380914[] = {{&S103761[31], 3}, {NULL}};
const Segment SGI380915[] = {{&S103839[88], 6}, {NULL}};
const Segment SGI380916[] = {{&S103839[8], 3}, {NULL}};
const Segment SGI380917[] = {{&S103839[13], 9}, {NULL}};
const Segment SGI380918[] = {{&S103839[33], 15}, {NULL}};
const Segment SGI380919[] = {{&S103839[123], 3}, {NULL}};
const Segment SGI380920[] = {{&S103839[49], 6}, {NULL}};
const Segment SGI380921[] = {{&S103839[33], 13}, {NULL}};
const Segment SGI380922[] = {{&S104352[60], 6}, {NULL}};
const Segment SGI380923[] = {{&S104352[8], 3}, {NULL}};
const Segment SGI380924[] = {{&S104352[13], 7}, {NULL}};
const Segment SGI380925[] = {{&S104352[25], 14}, {NULL}};
const Segment SGI380926[] = {{&S104369[107], 7}, {NULL}};
const Segment SGI380927[] = {{&S104369[8], 3}, {NULL}};
const Segment SGI380928[] = {{&S104369[13], 9}, {NULL}};
const Segment SGI380929[] = {{&S104369[96], 3}, {NULL}};
const Segment SGI380930[] = {{&S104369[30], 13}, {NULL}};
const Segment SGI380931[] = {{&S104369[118], 12}, {NULL}};
const Segment SGI380932[] = {{&S34461[65], 12}, {NULL}};
const Segment SGI380933[] = {{&S34461[25], 11}, {NULL}};
const Segment SGI380934[] = {{&S34462[64], 14}, {NULL}};
const Segment SGI380935[] = {{&S34462[23], 11}, {NULL}};
const Segment SGI380936[] = {{&S104808[60], 5}, {NULL}};
const Segment SGI380937[] = {{&S104808[8], 3}, {NULL}};
const Segment SGI380938[] = {{&S104808[13], 7}, {NULL}};
const Segment SGI380939[] = {{&S104808[25], 14}, {NULL}};
const Segment SGI380940[] = {{&S104808[86], 14}, {NULL}};
const Segment SGI380941[] = {{&S104815[108], 7}, {NULL}};
const Segment SGI380942[] = {{&S104815[8], 3}, {NULL}};
const Segment SGI380943[] = {{&S104815[13], 9}, {NULL}};
const Segment SGI380944[] = {{&S104815[97], 3}, {NULL}};
const Segment SGI380945[] = {{&S104815[30], 13}, {NULL}};
const Segment SGI380946[] = {{&S104815[119], 12}, {NULL}};
const Segment SGI380947[] = {{&S104839[103], 7}, {NULL}};
const Segment SGI380948[] = {{&S104839[8], 3}, {NULL}};
const Segment SGI380949[] = {{&S104839[13], 9}, {NULL}};
const Segment SGI380950[] = {{&S104839[92], 3}, {NULL}};
const Segment SGI380951[] = {{&S104839[30], 13}, {NULL}};
const Segment SGI380952[] = {{&S104839[114], 12}, {NULL}};
const Segment SGI380953[] = {{&S35194[65], 12}, {NULL}};
const Segment SGI380954[] = {{&S35194[25], 11}, {NULL}};
const Segment SGI380955[] = {{&S35202[64], 14}, {NULL}};
const Segment SGI380956[] = {{&S35202[23], 11}, {NULL}};
const Segment SGI380957[] = {{&S105481[42], 6}, {NULL}};
const Segment SGI380958[] = {{&S105481[8], 3}, {NULL}};
const Segment SGI380959[] = {{&S105481[13], 3}, {NULL}};
const Segment SGI380960[] = {{&S105481[69], 12}, {NULL}};
const Segment SGI380961[] = {{&S105524[107], 7}, {NULL}};
const Segment SGI380962[] = {{&S105524[8], 3}, {NULL}};
const Segment SGI380963[] = {{&S105524[13], 9}, {NULL}};
const Segment SGI380964[] = {{&S105524[96], 3}, {NULL}};
const Segment SGI380965[] = {{&S105524[30], 13}, {NULL}};
const Segment SGI380966[] = {{&S105524[118], 12}, {NULL}};
const Segment SGI380967[] = {{&S105560[41], 6}, {NULL}};
const Segment SGI380968[] = {{&S105560[8], 3}, {NULL}};
const Segment SGI380969[] = {{&S105560[13], 14}, {NULL}};
const Segment SGI380970[] = {{&S105560[78], 3}, {NULL}};
const Segment SGI380971[] = {{&S105695[8], 3}, {NULL}};
const Segment SGI380972[] = {{&S105695[13], 5}, {NULL}};
const Segment SGI380973[] = {{&S105695[145], 3}, {NULL}};
const Segment SGI380974[] = {{&S105695[31], 3}, {NULL}};
const Segment SGI380975[] = {{&S35952[24], 10}, {NULL}};
const Segment SGI380976[] = {{&S35952[0], 11}, {NULL}};
const Segment SGI380977[] = {{&S36121[7], 15}, {NULL}};
const Segment SGI380978[] = {{&S36121[23], 4}, {NULL}};
const Segment SGI380979[] = {{&S36121[51], 3}, {NULL}};
const Segment SGI380980[] = {{&S105962[93], 6}, {NULL}};
const Segment SGI380981[] = {{&S105962[8], 3}, {NULL}};
const Segment SGI380982[] = {{&S105962[13], 3}, {NULL}};
const Segment SGI380983[] = {{&S105962[80], 3}, {NULL}};
const Segment SGI380984[] = {{&S106017[8], 3}, {NULL}};
const Segment SGI380985[] = {{&S106017[13], 7}, {NULL}};
const Segment SGI380986[] = {{&S106017[25], 14}, {NULL}};
const Segment SGI380987[] = {{&S37076[75], 103}, {NULL}};
const Segment SGI380988[] = {{&S37076[36], 11}, {NULL}};
const Segment SGI380989[] = {{&S37577[25], 10}, {NULL}};
const Segment SGI380990[] = {{&S37577[0], 11}, {NULL}};
const Segment SGI380991[] = {{&S37803[78], 103}, {NULL}};
const Segment SGI380992[] = {{&S37803[36], 11}, {NULL}};
const Segment SGI380993[] = {{&S38019[66], 12}, {NULL}};
const Segment SGI380994[] = {{&S38019[24], 11}, {NULL}};
const Segment SGI380995[] = {{&S38306[49], 1}, {NULL}};
const Segment SGI380996[] = {{&S38306[16], 11}, {NULL}};
const Segment SGI380997[] = {{&S38759[81], 9}, {NULL}};
const Segment SGI380998[] = {{&S38759[85], 5}, {NULL}};
const Segment SGI380999[] = {{&S38759[6], 4}, {NULL}};
const Segment SGI381000[] = {{&S38759[29], 3}, {NULL}};
const Segment SGI381001[] = {{&S38880[37], 25}, {NULL}};
const Segment SGI381002[] = {{&S39746[77], 103}, {NULL}};
const Segment SGI381003[] = {{&S39746[36], 11}, {NULL}};
const Segment SGI381004[] = {{&S39879[79], 9}, {NULL}};
const Segment SGI381005[] = {{&S39879[83], 5}, {NULL}};
const Segment SGI381006[] = {{&S39879[6], 4}, {NULL}};
const Segment SGI381007[] = {{&S39879[29], 3}, {NULL}};
const Segment SGI381008[] = {{&S41269[7], 8}, {NULL}};
const Segment SGI381009[] = {{&S41673[7], 9}, {NULL}};
const Segment SGI381010[] = {{&S41673[41], 3}, {NULL}};
const Segment SGI381011[] = {{&S41674[7], 10}, {NULL}};
const Segment SGI381012[] = {{&S41674[42], 3}, {NULL}};
const Segment SGI381013[] = {{&S41675[7], 10}, {NULL}};
const Segment SGI381014[] = {{&S41675[31], 3}, {NULL}};
const Segment SGI381015[] = {{&S41676[7], 16}, {NULL}};
const Segment SGI381016[] = {{&S41676[54], 3}, {NULL}};
const Segment SGI381017[] = {{&S41677[7], 18}, {NULL}};
const Segment SGI381018[] = {{&S41677[56], 3}, {NULL}};
const Segment SGI381019[] = {{&S41991[7], 9}, {NULL}};
const Segment SGI381020[] = {{&S41991[124], 4}, {NULL}};
const Segment SGI381021[] = {{&S42110[88], 86}, {NULL}};
const Segment SGI381022[] = {{&S42110[71], 103}, {NULL}};
const Segment SGI381023[] = {{&S42110[36], 11}, {NULL}};
const Segment SGI381024[] = {{&S42624[81], 9}, {NULL}};
const Segment SGI381025[] = {{&S42624[85], 5}, {NULL}};
const Segment SGI381026[] = {{&S42624[6], 4}, {NULL}};
const Segment SGI381027[] = {{&S42624[29], 3}, {NULL}};
const Segment SGI381028[] = {{&S43970[88], 86}, {NULL}};
const Segment SGI381029[] = {{&S43970[36], 11}, {NULL}};
const Segment SGI381030[] = {{&S44424[102], 11}, {NULL}};
const Segment SGI381031[] = {{&S44425[102], 12}, {NULL}};
const Segment SGI381032[] = {{&S44446[7], 6}, {NULL}};
const Segment SGI381033[] = {{&S44446[35], 3}, {NULL}};
const Segment SGI381034[] = {{&S45435[7], 9}, {NULL}};
const Segment SGI381035[] = {{&S45435[124], 4}, {NULL}};
const Segment SGI381036[] = {{&S31576[33], 6}, {NULL}};
const Segment SGI381037[] = {{&S31576[61], 3}, {NULL}};
const Segment SGI381038[] = {{&S31576[118], 3}, {NULL}};
const Segment SGI381039[] = {{&S31577[33], 6}, {NULL}};
const Segment SGI381040[] = {{&S31577[40], 3}, {NULL}};
const Segment SGI381041[] = {{&S31577[110], 3}, {NULL}};
const Segment SGI381042[] = {{&S31578[33], 5}, {NULL}};
const Segment SGI381043[] = {{&S31578[59], 3}, {NULL}};
const Segment SGI381044[] = {{&S31578[116], 3}, {NULL}};
const Segment SGI381045[] = {{&S46065[7], 15}, {NULL}};
const Segment SGI381046[] = {{&S46065[53], 3}, {NULL}};
const Segment SGI381047[] = {{&S46066[7], 6}, {NULL}};
const Segment SGI381048[] = {{&S46066[44], 3}, {NULL}};
const Segment SGI381049[] = {{&S46067[7], 13}, {NULL}};
const Segment SGI381050[] = {{&S46067[51], 3}, {NULL}};
const Segment SGI381051[] = {{&S46244[7], 9}, {NULL}};
const Segment SGI381052[] = {{&S46244[124], 4}, {NULL}};
const Segment SGI381053[] = {{&S46696[7], 6}, {NULL}};
const Segment SGI381054[] = {{&S46696[34], 3}, {NULL}};
const Segment SGI381055[] = {{&S46697[46], 10}, {NULL}};
const Segment SGI381056[] = {{&S46697[9], 5}, {&S46697[9], 5}, {NULL}};
const Segment SGI381057[] = {{&S32842[33], 5}, {NULL}};
const Segment SGI381058[] = {{&S32842[59], 3}, {NULL}};
const Segment SGI381059[] = {{&S32842[116], 3}, {NULL}};
const Segment SGI381060[] = {{&S32843[11], 5}, {NULL}};
const Segment SGI381061[] = {{&S32843[39], 3}, {NULL}};
const Segment SGI381062[] = {{&S46766[7], 9}, {NULL}};
const Segment SGI381063[] = {{&S46766[72], 3}, {NULL}};
const Segment SGI381064[] = {{&S46767[7], 14}, {NULL}};
const Segment SGI381065[] = {{&S46767[77], 3}, {NULL}};
const Segment SGI381066[] = {{&S46768[7], 17}, {NULL}};
const Segment SGI381067[] = {{&S46768[80], 3}, {NULL}};
const Segment SGI381068[] = {{&S46769[7], 16}, {NULL}};
const Segment SGI381069[] = {{&S46769[86], 3}, {NULL}};
const Segment SGI381070[] = {{&S46770[20], 10}, {NULL}};
const Segment SGI381071[] = {{&S46770[92], 3}, {NULL}};
const Segment SGI381072[] = {{&S46771[20], 10}, {NULL}};
const Segment SGI381073[] = {{&S46771[92], 3}, {NULL}};
const Segment SGI381074[] = {{&S46772[20], 10}, {NULL}};
const Segment SGI381075[] = {{&S46772[93], 3}, {NULL}};
const Segment SGI381076[] = {{&S46773[20], 10}, {NULL}};
const Segment SGI381077[] = {{&S46773[93], 3}, {NULL}};
const Segment SGI381078[] = {{&S46774[20], 10}, {NULL}};
const Segment SGI381079[] = {{&S46774[92], 3}, {NULL}};
const Segment SGI381080[] = {{&S46775[7], 10}, {NULL}};
const Segment SGI381081[] = {{&S46775[80], 3}, {NULL}};
const Segment SGI381082[] = {{&S46776[20], 10}, {NULL}};
const Segment SGI381083[] = {{&S46776[92], 3}, {NULL}};
const Segment SGI381084[] = {{&S33891[33], 6}, {NULL}};
const Segment SGI381085[] = {{&S33891[61], 3}, {NULL}};
const Segment SGI381086[] = {{&S33891[119], 3}, {NULL}};
const Segment SGI381087[] = {{&S33892[33], 6}, {NULL}};
const Segment SGI381088[] = {{&S33892[53], 3}, {NULL}};
const Segment SGI381089[] = {{&S33892[111], 3}, {NULL}};
const Segment SGI381090[] = {{&S33893[33], 5}, {NULL}};
const Segment SGI381091[] = {{&S33893[59], 3}, {NULL}};
const Segment SGI381092[] = {{&S33893[117], 3}, {NULL}};
const Segment SGI381093[] = {{&S33894[33], 6}, {NULL}};
const Segment SGI381094[] = {{&S33894[61], 3}, {NULL}};
const Segment SGI381095[] = {{&S33894[119], 3}, {NULL}};
const Segment SGI381096[] = {{&S33895[33], 6}, {NULL}};
const Segment SGI381097[] = {{&S33895[53], 3}, {NULL}};
const Segment SGI381098[] = {{&S33895[108], 3}, {NULL}};
const Segment SGI381099[] = {{&S48660[7], 6}, {NULL}};
const Segment SGI381100[] = {{&S48660[14], 4}, {NULL}};
const Segment SGI381101[] = {{&S48660[42], 3}, {NULL}};
const Segment SGI381102[] = {{&S48663[44], 10}, {NULL}};
const Segment SGI381103[] = {{&S48663[9], 5}, {&S48663[20], 5}, {NULL}};
const Segment SGI381104[] = {{&S48682[7], 9}, {NULL}};
const Segment SGI381105[] = {{&S48682[39], 3}, {NULL}};
const Segment SGI381106[] = {{&S48683[7], 10}, {NULL}};
const Segment SGI381107[] = {{&S48683[46], 3}, {NULL}};
const Segment SGI381108[] = {{&S48684[7], 13}, {NULL}};
const Segment SGI381109[] = {{&S48684[49], 3}, {NULL}};
const Segment SGI381110[] = {{&S34959[11], 11}, {NULL}};
const Segment SGI381111[] = {{&S34959[23], 4}, {NULL}};
const Segment SGI381112[] = {{&S34959[51], 3}, {NULL}};
const Segment SGI381113[] = {{&S49079[7], 10}, {NULL}};
const Segment SGI381114[] = {{&S49079[44], 3}, {NULL}};
const Segment SGI381115[] = {{&S49080[7], 17}, {NULL}};
const Segment SGI381116[] = {{&S49080[51], 3}, {NULL}};
const Segment SGI381117[] = {{&S49081[7], 19}, {NULL}};
const Segment SGI381118[] = {{&S49081[53], 3}, {NULL}};
const Segment SGI381119[] = {{&S49082[7], 19}, {NULL}};
const Segment SGI381120[] = {{&S49082[53], 3}, {NULL}};
const Segment SGI381121[] = {{&S49083[7], 10}, {NULL}};
const Segment SGI381122[] = {{&S49083[42], 3}, {NULL}};
const Segment SGI381123[] = {{&S35405[33], 5}, {NULL}};
const Segment SGI381124[] = {{&S35405[59], 3}, {NULL}};
const Segment SGI381125[] = {{&S35405[117], 3}, {NULL}};
const Segment SGI381126[] = {{&S49369[7], 9}, {NULL}};
const Segment SGI381127[] = {{&S49369[124], 4}, {NULL}};
const Segment SGI381128[] = {{&S49634[79], 9}, {NULL}};
const Segment SGI381129[] = {{&S49634[83], 5}, {NULL}};
const Segment SGI381130[] = {{&S49634[6], 4}, {NULL}};
const Segment SGI381131[] = {{&S49634[29], 3}, {NULL}};
const Segment SGI381132[] = {{&S36120[11], 11}, {NULL}};
const Segment SGI381133[] = {{&S36120[23], 4}, {NULL}};
const Segment SGI381134[] = {{&S36120[51], 3}, {NULL}};
const Segment SGI381135[] = {{&S36478[11], 11}, {NULL}};
const Segment SGI381136[] = {{&S36478[23], 4}, {NULL}};
const Segment SGI381137[] = {{&S36478[51], 3}, {NULL}};
const Segment SGI381138[] = {{&S49973[102], 11}, {NULL}};
const Segment SGI381139[] = {{&S49973[107], 6}, {NULL}};
const Segment SGI381140[] = {{&S50181[3], 5}, {NULL}};
const Segment SGI381141[] = {{&S50181[22], 11}, {NULL}};
const Segment SGI381142[] = {{&S37037[11], 6}, {NULL}};
const Segment SGI381143[] = {{&S37037[40], 3}, {NULL}};
const Segment SGI381144[] = {{&S37038[11], 6}, {NULL}};
const Segment SGI381145[] = {{&S37038[40], 3}, {NULL}};
const Segment SGI381146[] = {{&S37039[11], 5}, {NULL}};
const Segment SGI381147[] = {{&S37039[39], 3}, {NULL}};
const Segment SGI381148[] = {{&S37040[11], 11}, {NULL}};
const Segment SGI381149[] = {{&S37040[23], 4}, {NULL}};
const Segment SGI381150[] = {{&S37040[51], 3}, {NULL}};
const Segment SGI381151[] = {{&S50646[7], 13}, {NULL}};
const Segment SGI381152[] = {{&S50646[51], 3}, {NULL}};
const Segment SGI381153[] = {{&S50647[7], 9}, {NULL}};
const Segment SGI381154[] = {{&S50647[47], 3}, {NULL}};
const Segment SGI381155[] = {{&S50839[7], 9}, {NULL}};
const Segment SGI381156[] = {{&S50839[124], 4}, {NULL}};
const Segment SGI381157[] = {{&S50849[7], 9}, {NULL}};
const Segment SGI381158[] = {{&S50849[47], 3}, {NULL}};
const Segment SGI381159[] = {{&S37306[11], 5}, {&S37306[11], 5}, {NULL}};
const Segment SGI381160[] = {{&S37306[37], 3}, {&S37306[37], 3}, {NULL}};
const Segment SGI381161[] = {{&S37307[11], 6}, {NULL}};
const Segment SGI381162[] = {{&S37307[31], 3}, {NULL}};
const Segment SGI381163[] = {{&S37308[11], 5}, {NULL}};
const Segment SGI381164[] = {{&S37308[37], 3}, {NULL}};
const Segment SGI381165[] = {{&S37309[11], 6}, {&S37309[11], 6}, {NULL}};
const Segment SGI381166[] = {{&S37309[39], 3}, {&S37309[39], 3}, {NULL}};
const Segment SGI381167[] = {{&S37310[11], 6}, {NULL}};
const Segment SGI381168[] = {{&S37310[31], 3}, {NULL}};
const Segment SGI381169[] = {{&S50937[7], 9}, {NULL}};
const Segment SGI381170[] = {{&S50937[47], 3}, {NULL}};
const Segment SGI381171[] = {{&S51423[7], 6}, {NULL}};
const Segment SGI381172[] = {{&S51424[7], 6}, {NULL}};
const Segment SGI381173[] = {{&S51425[7], 13}, {NULL}};
const Segment SGI381174[] = {{&S51425[41], 3}, {NULL}};
const Segment SGI381175[] = {{&S51426[7], 6}, {NULL}};
const Segment SGI381176[] = {{&S51426[27], 3}, {NULL}};
const Segment SGI381177[] = {{&S51428[7], 6}, {NULL}};
const Segment SGI381178[] = {{&S51428[27], 3}, {NULL}};
const Segment SGI381179[] = {{&S51429[20], 5}, {NULL}};
const Segment SGI381180[] = {{&S51429[54], 3}, {NULL}};
const Segment SGI381181[] = {{&S51430[20], 5}, {NULL}};
const Segment SGI381182[] = {{&S51430[54], 3}, {NULL}};
const Segment SGI381183[] = {{&S51431[20], 5}, {NULL}};
const Segment SGI381184[] = {{&S51431[54], 3}, {NULL}};
const Segment SGI381185[] = {{&S51432[20], 5}, {NULL}};
const Segment SGI381186[] = {{&S51432[54], 3}, {NULL}};
const Segment SGI381187[] = {{&S51433[20], 5}, {NULL}};
const Segment SGI381188[] = {{&S51433[54], 3}, {NULL}};
const Segment SGI381189[] = {{&S51434[20], 5}, {NULL}};
const Segment SGI381190[] = {{&S51434[54], 3}, {NULL}};
const Segment SGI381191[] = {{&S51435[20], 5}, {NULL}};
const Segment SGI381192[] = {{&S51435[54], 3}, {NULL}};
const Segment SGI381193[] = {{&S51436[20], 5}, {NULL}};
const Segment SGI381194[] = {{&S51436[54], 3}, {NULL}};
const Segment SGI381195[] = {{&S51437[20], 5}, {NULL}};
const Segment SGI381196[] = {{&S51437[54], 3}, {NULL}};
const Segment SGI381197[] = {{&S38081[11], 5}, {NULL}};
const Segment SGI381198[] = {{&S38081[40], 3}, {NULL}};
const Segment SGI381199[] = {{&S38082[11], 6}, {NULL}};
const Segment SGI381200[] = {{&S38082[56], 3}, {NULL}};
const Segment SGI381201[] = {{&S52420[80], 107}, {NULL}};
const Segment SGI381202[] = {{&S52420[36], 11}, {NULL}};
const Segment SGI381203[] = {{&S52737[102], 12}, {NULL}};
const Segment SGI381204[] = {{&S52737[107], 7}, {NULL}};
const Segment SGI381205[] = {{&S52742[102], 11}, {NULL}};
const Segment SGI381206[] = {{&S53440[3], 6}, {NULL}};
const Segment SGI381207[] = {{&S53440[23], 11}, {NULL}};
const Segment SGI381208[] = {{&S53441[3], 6}, {NULL}};
const Segment SGI381209[] = {{&S53441[23], 11}, {NULL}};
const Segment SGI381210[] = {{&S39141[33], 5}, {NULL}};
const Segment SGI381211[] = {{&S39141[59], 3}, {NULL}};
const Segment SGI381212[] = {{&S39141[116], 3}, {NULL}};
const Segment SGI381213[] = {{&S53592[7], 9}, {NULL}};
const Segment SGI381214[] = {{&S53592[43], 3}, {NULL}};
const Segment SGI381215[] = {{&S53691[7], 15}, {NULL}};
const Segment SGI381216[] = {{&S53691[54], 3}, {NULL}};
const Segment SGI381217[] = {{&S53692[7], 15}, {NULL}};
const Segment SGI381218[] = {{&S53692[54], 3}, {NULL}};
const Segment SGI381219[] = {{&S53693[7], 7}, {NULL}};
const Segment SGI381220[] = {{&S53693[39], 3}, {NULL}};
const Segment SGI381221[] = {{&S53694[7], 13}, {NULL}};
const Segment SGI381222[] = {{&S53694[52], 3}, {NULL}};
const Segment SGI381223[] = {{&S53695[7], 7}, {NULL}};
const Segment SGI381224[] = {{&S53695[39], 3}, {NULL}};
const Segment SGI381225[] = {{&S53696[7], 6}, {NULL}};
const Segment SGI381226[] = {{&S53696[38], 3}, {NULL}};
const Segment SGI381227[] = {{&S53869[7], 9}, {NULL}};
const Segment SGI381228[] = {{&S53869[36], 3}, {NULL}};
const Segment SGI381229[] = {{&S54111[7], 9}, {NULL}};
const Segment SGI381230[] = {{&S54111[47], 3}, {NULL}};
const Segment SGI381231[] = {{&S54182[7], 9}, {NULL}};
const Segment SGI381232[] = {{&S54182[124], 4}, {NULL}};
const Segment SGI381233[] = {{&S40017[11], 12}, {NULL}};
const Segment SGI381234[] = {{&S40017[24], 4}, {NULL}};
const Segment SGI381235[] = {{&S40017[52], 3}, {NULL}};
const Segment SGI381236[] = {{&S40269[33], 5}, {NULL}};
const Segment SGI381237[] = {{&S40269[59], 3}, {NULL}};
const Segment SGI381238[] = {{&S40269[117], 3}, {NULL}};
const Segment SGI381239[] = {{&S40494[11], 6}, {NULL}};
const Segment SGI381240[] = {{&S40494[48], 3}, {NULL}};
const Segment SGI381241[] = {{&S40495[11], 5}, {NULL}};
const Segment SGI381242[] = {{&S40495[47], 3}, {NULL}};
const Segment SGI381243[] = {{&S54665[7], 9}, {NULL}};
const Segment SGI381244[] = {{&S54665[47], 3}, {NULL}};
const Segment SGI381245[] = {{&S40564[33], 6}, {NULL}};
const Segment SGI381246[] = {{&S40564[60], 3}, {NULL}};
const Segment SGI381247[] = {{&S40564[117], 3}, {NULL}};
const Segment SGI381248[] = {{&S54733[7], 9}, {NULL}};
const Segment SGI381249[] = {{&S54733[47], 3}, {NULL}};
const Segment SGI381250[] = {{&S54989[46], 11}, {NULL}};
const Segment SGI381251[] = {{&S54989[9], 5}, {&S54989[9], 5}, {NULL}};
const Segment SGI381252[] = {{&S40826[11], 5}, {NULL}};
const Segment SGI381253[] = {{&S40826[39], 3}, {NULL}};
const Segment SGI381254[] = {{&S41066[33], 5}, {NULL}};
const Segment SGI381255[] = {{&S41066[106], 3}, {NULL}};
const Segment SGI381256[] = {{&S41067[33], 5}, {NULL}};
const Segment SGI381257[] = {{&S41067[59], 3}, {NULL}};
const Segment SGI381258[] = {{&S41067[116], 3}, {NULL}};
const Segment SGI381259[] = {{&S55574[7], 15}, {NULL}};
const Segment SGI381260[] = {{&S55574[35], 3}, {NULL}};
const Segment SGI381261[] = {{&S55575[7], 10}, {NULL}};
const Segment SGI381262[] = {{&S55575[30], 3}, {NULL}};
const Segment SGI381263[] = {{&S55576[7], 9}, {NULL}};
const Segment SGI381264[] = {{&S55576[27], 3}, {NULL}};
const Segment SGI381265[] = {{&S55577[20], 9}, {NULL}};
const Segment SGI381266[] = {{&S55577[40], 3}, {&S55577[40], 3}, {NULL}};
const Segment SGI381267[] = {{&S41288[33], 5}, {NULL}};
const Segment SGI381268[] = {{&S41288[59], 3}, {NULL}};
const Segment SGI381269[] = {{&S41288[117], 3}, {NULL}};
const Segment SGI381270[] = {{&S41458[11], 6}, {NULL}};
const Segment SGI381271[] = {{&S41458[39], 3}, {NULL}};
const Segment SGI381272[] = {{&S41459[11], 5}, {NULL}};
const Segment SGI381273[] = {{&S41459[30], 3}, {NULL}};
const Segment SGI381274[] = {{&S41642[11], 5}, {NULL}};
const Segment SGI381275[] = {{&S41642[30], 3}, {NULL}};
const Segment SGI381276[] = {{&S41643[11], 5}, {NULL}};
const Segment SGI381277[] = {{&S41643[30], 3}, {NULL}};
const Segment SGI381278[] = {{&S41643[80], 4}, {NULL}};
const Segment SGI381279[] = {{&S41644[11], 5}, {NULL}};
const Segment SGI381280[] = {{&S41644[30], 3}, {NULL}};
const Segment SGI381281[] = {{&S41644[80], 4}, {NULL}};
const Segment SGI381282[] = {{&S41644[108], 3}, {NULL}};
const Segment SGI381283[] = {{&S41953[11], 5}, {NULL}};
const Segment SGI381284[] = {{&S41953[46], 3}, {NULL}};
const Segment SGI381285[] = {{&S41954[11], 6}, {NULL}};
const Segment SGI381286[] = {{&S41954[55], 3}, {NULL}};
const Segment SGI381287[] = {{&S56895[7], 10}, {NULL}};
const Segment SGI381288[] = {{&S56895[48], 3}, {NULL}};
const Segment SGI381289[] = {{&S56896[7], 10}, {NULL}};
const Segment SGI381290[] = {{&S56896[48], 3}, {NULL}};
const Segment SGI381291[] = {{&S56897[7], 9}, {NULL}};
const Segment SGI381292[] = {{&S56897[47], 3}, {NULL}};
const Segment SGI381293[] = {{&S57020[7], 9}, {NULL}};
const Segment SGI381294[] = {{&S57020[41], 3}, {NULL}};
const Segment SGI381295[] = {{&S57021[44], 10}, {NULL}};
const Segment SGI381296[] = {{&S57021[9], 5}, {&S57021[20], 5}, {NULL}};
const Segment SGI381297[] = {{&S57290[69], 9}, {NULL}};
const Segment SGI381298[] = {{&S42871[11], 6}, {NULL}};
const Segment SGI381299[] = {{&S42871[18], 3}, {NULL}};
const Segment SGI381300[] = {{&S42872[11], 6}, {NULL}};
const Segment SGI381301[] = {{&S42872[49], 3}, {NULL}};
const Segment SGI381302[] = {{&S42873[11], 5}, {NULL}};
const Segment SGI381303[] = {{&S42873[17], 3}, {NULL}};
const Segment SGI381304[] = {{&S57512[81], 9}, {NULL}};
const Segment SGI381305[] = {{&S57512[85], 5}, {NULL}};
const Segment SGI381306[] = {{&S57512[6], 4}, {NULL}};
const Segment SGI381307[] = {{&S57512[29], 3}, {NULL}};
const Segment SGI381308[] = {{&S57528[3], 5}, {NULL}};
const Segment SGI381309[] = {{&S57528[22], 11}, {NULL}};
const Segment SGI381310[] = {{&S43102[11], 5}, {NULL}};
const Segment SGI381311[] = {{&S43102[37], 3}, {NULL}};
const Segment SGI381312[] = {{&S57798[7], 9}, {&S57798[7], 9}, {NULL}};
const Segment SGI381313[] = {{&S57799[69], 9}, {NULL}};
const Segment SGI381314[] = {{&S57800[7], 15}, {NULL}};
const Segment SGI381315[] = {{&S57801[69], 10}, {NULL}};
const Segment SGI381316[] = {{&S57802[69], 10}, {NULL}};
const Segment SGI381317[] = {{&S57803[69], 10}, {NULL}};
const Segment SGI381318[] = {{&S57804[69], 10}, {NULL}};
const Segment SGI381319[] = {{&S57805[69], 10}, {NULL}};
const Segment SGI381320[] = {{&S57806[69], 10}, {NULL}};
const Segment SGI381321[] = {{&S57807[69], 10}, {NULL}};
const Segment SGI381322[] = {{&S57808[69], 10}, {NULL}};
const Segment SGI381323[] = {{&S57809[69], 10}, {NULL}};
const Segment SGI381324[] = {{&S57810[69], 10}, {NULL}};
const Segment SGI381325[] = {{&S57811[69], 10}, {NULL}};
const Segment SGI381326[] = {{&S57812[69], 10}, {NULL}};
const Segment SGI381327[] = {{&S57813[69], 10}, {NULL}};
const Segment SGI381328[] = {{&S57814[69], 10}, {NULL}};
const Segment SGI381329[] = {{&S57815[69], 10}, {NULL}};
const Segment SGI381330[] = {{&S57816[69], 10}, {NULL}};
const Segment SGI381331[] = {{&S57817[69], 10}, {NULL}};
const Segment SGI381332[] = {{&S57818[7], 9}, {NULL}};
const Segment SGI381333[] = {{&S57819[69], 10}, {NULL}};
const Segment SGI381334[] = {{&S57820[69], 10}, {NULL}};
const Segment SGI381335[] = {{&S57821[69], 10}, {NULL}};
const Segment SGI381336[] = {{&S57822[69], 10}, {NULL}};
const Segment SGI381337[] = {{&S57849[7], 6}, {NULL}};
const Segment SGI381338[] = {{&S57849[69], 3}, {NULL}};
const Segment SGI381339[] = {{&S57850[7], 13}, {NULL}};
const Segment SGI381340[] = {{&S57850[83], 3}, {NULL}};
const Segment SGI381341[] = {{&S57851[20], 6}, {NULL}};
const Segment SGI381342[] = {{&S57851[84], 3}, {NULL}};
const Segment SGI381343[] = {{&S57852[20], 6}, {NULL}};
const Segment SGI381344[] = {{&S57852[84], 3}, {NULL}};
const Segment SGI381345[] = {{&S57853[20], 6}, {NULL}};
const Segment SGI381346[] = {{&S57853[84], 3}, {NULL}};
const Segment SGI381347[] = {{&S57887[7], 9}, {NULL}};
const Segment SGI381348[] = {{&S57887[124], 4}, {NULL}};
const Segment SGI381349[] = {{&S58024[46], 10}, {NULL}};
const Segment SGI381350[] = {{&S58024[9], 5}, {&S58024[9], 5}, {NULL}};
const Segment SGI381351[] = {{&S58065[7], 14}, {NULL}};
const Segment SGI381352[] = {{&S58065[41], 3}, {NULL}};
const Segment SGI381353[] = {{&S58066[7], 9}, {NULL}};
const Segment SGI381354[] = {{&S58066[36], 3}, {NULL}};
const Segment SGI381355[] = {{&S43601[11], 5}, {NULL}};
const Segment SGI381356[] = {{&S43601[47], 3}, {NULL}};
const Segment SGI381357[] = {{&S58974[7], 9}, {NULL}};
const Segment SGI381358[] = {{&S58974[124], 4}, {NULL}};
const Segment SGI381359[] = {{&S44447[11], 6}, {NULL}};
const Segment SGI381360[] = {{&S44447[31], 3}, {NULL}};
const Segment SGI381361[] = {{&S44448[24], 6}, {NULL}};
const Segment SGI381362[] = {{&S44448[46], 3}, {NULL}};
const Segment SGI381363[] = {{&S44449[24], 6}, {NULL}};
const Segment SGI381364[] = {{&S44449[46], 3}, {NULL}};
const Segment SGI381365[] = {{&S44450[24], 6}, {NULL}};
const Segment SGI381366[] = {{&S44450[46], 3}, {NULL}};
const Segment SGI381367[] = {{&S44451[24], 6}, {NULL}};
const Segment SGI381368[] = {{&S44451[46], 3}, {NULL}};
const Segment SGI381369[] = {{&S44452[24], 6}, {NULL}};
const Segment SGI381370[] = {{&S44452[46], 3}, {NULL}};
const Segment SGI381371[] = {{&S44453[24], 6}, {NULL}};
const Segment SGI381372[] = {{&S44453[46], 3}, {NULL}};
const Segment SGI381373[] = {{&S44454[24], 6}, {NULL}};
const Segment SGI381374[] = {{&S44454[46], 3}, {NULL}};
const Segment SGI381375[] = {{&S44455[11], 6}, {NULL}};
const Segment SGI381376[] = {{&S44455[39], 3}, {NULL}};
const Segment SGI381377[] = {{&S44456[24], 6}, {NULL}};
const Segment SGI381378[] = {{&S44456[46], 3}, {NULL}};
const Segment SGI381379[] = {{&S59404[7], 9}, {NULL}};
const Segment SGI381380[] = {{&S59404[124], 4}, {NULL}};
const Segment SGI381381[] = {{&S59721[7], 9}, {NULL}};
const Segment SGI381382[] = {{&S59721[72], 3}, {NULL}};
const Segment SGI381383[] = {{&S59722[7], 16}, {NULL}};
const Segment SGI381384[] = {{&S59722[85], 3}, {NULL}};
const Segment SGI381385[] = {{&S59733[7], 9}, {NULL}};
const Segment SGI381386[] = {{&S59733[45], 3}, {NULL}};
const Segment SGI381387[] = {{&S59745[7], 9}, {NULL}};
const Segment SGI381388[] = {{&S59745[47], 3}, {NULL}};
const Segment SGI381389[] = {{&S59746[7], 9}, {NULL}};
const Segment SGI381390[] = {{&S59746[47], 3}, {NULL}};
const Segment SGI381391[] = {{&S59795[7], 7}, {NULL}};
const Segment SGI381392[] = {{&S45405[11], 5}, {NULL}};
const Segment SGI381393[] = {{&S45405[37], 3}, {NULL}};
const Segment SGI381394[] = {{&S45406[11], 6}, {NULL}};
const Segment SGI381395[] = {{&S45406[31], 3}, {NULL}};
const Segment SGI381396[] = {{&S45833[11], 6}, {NULL}};
const Segment SGI381397[] = {{&S45833[39], 3}, {NULL}};
const Segment SGI381398[] = {{&S45834[11], 5}, {NULL}};
const Segment SGI381399[] = {{&S45834[37], 3}, {NULL}};
const Segment SGI381400[] = {{&S60420[7], 19}, {NULL}};
const Segment SGI381401[] = {{&S60421[7], 16}, {NULL}};
const Segment SGI381402[] = {{&S60511[67], 9}, {NULL}};
const Segment SGI381403[] = {{&S60511[71], 5}, {NULL}};
const Segment SGI381404[] = {{&S60641[42], 9}, {NULL}};
const Segment SGI381405[] = {{&S60641[58], 13}, {NULL}};
const Segment SGI381406[] = {{&S46199[11], 5}, {NULL}};
const Segment SGI381407[] = {{&S46199[46], 3}, {NULL}};
const Segment SGI381408[] = {{&S46355[11], 11}, {NULL}};
const Segment SGI381409[] = {{&S46355[23], 4}, {NULL}};
const Segment SGI381410[] = {{&S46355[51], 3}, {NULL}};
const Segment SGI381411[] = {{&S46361[11], 11}, {NULL}};
const Segment SGI381412[] = {{&S46361[23], 4}, {NULL}};
const Segment SGI381413[] = {{&S46361[51], 3}, {NULL}};
const Segment SGI381414[] = {{&S60903[7], 9}, {NULL}};
const Segment SGI381415[] = {{&S60910[7], 6}, {NULL}};
const Segment SGI381416[] = {{&S46514[11], 12}, {NULL}};
const Segment SGI381417[] = {{&S46514[24], 4}, {NULL}};
const Segment SGI381418[] = {{&S46514[52], 3}, {NULL}};
const Segment SGI381419[] = {{&S46656[11], 6}, {NULL}};
const Segment SGI381420[] = {{&S46656[38], 3}, {NULL}};
const Segment SGI381421[] = {{&S46657[11], 5}, {NULL}};
const Segment SGI381422[] = {{&S46657[30], 3}, {NULL}};
const Segment SGI381423[] = {{&S46695[11], 6}, {NULL}};
const Segment SGI381424[] = {{&S46695[39], 3}, {NULL}};
const Segment SGI381425[] = {{&S61288[7], 10}, {NULL}};
const Segment SGI381426[] = {{&S61288[48], 3}, {NULL}};
const Segment SGI381427[] = {{&S61289[7], 10}, {NULL}};
const Segment SGI381428[] = {{&S61289[48], 3}, {NULL}};
const Segment SGI381429[] = {{&S61290[7], 10}, {NULL}};
const Segment SGI381430[] = {{&S61290[48], 3}, {NULL}};
const Segment SGI381431[] = {{&S61291[7], 9}, {NULL}};
const Segment SGI381432[] = {{&S61291[47], 3}, {NULL}};
const Segment SGI381433[] = {{&S61292[7], 10}, {NULL}};
const Segment SGI381434[] = {{&S61292[48], 3}, {NULL}};
const Segment SGI381435[] = {{&S61511[47], 11}, {NULL}};
const Segment SGI381436[] = {{&S61511[9], 5}, {&S61511[9], 5}, {NULL}};
const Segment SGI381437[] = {{&S47168[33], 6}, {NULL}};
const Segment SGI381438[] = {{&S47168[68], 3}, {NULL}};
const Segment SGI381439[] = {{&S47168[125], 3}, {NULL}};
const Segment SGI381440[] = {{&S61999[7], 6}, {NULL}};
const Segment SGI381441[] = {{&S61999[14], 3}, {NULL}};
const Segment SGI381442[] = {{&S62238[80], 13}, {NULL}};
const Segment SGI381443[] = {{&S62238[84], 9}, {NULL}};
const Segment SGI381444[] = {{&S62238[6], 3}, {NULL}};
const Segment SGI381445[] = {{&S47731[11], 11}, {NULL}};
const Segment SGI381446[] = {{&S47731[23], 4}, {NULL}};
const Segment SGI381447[] = {{&S47731[51], 3}, {NULL}};
const Segment SGI381448[] = {{&S62273[7], 18}, {NULL}};
const Segment SGI381449[] = {{&S62273[56], 3}, {NULL}};
const Segment SGI381450[] = {{&S62274[7], 9}, {NULL}};
const Segment SGI381451[] = {{&S62274[47], 3}, {NULL}};
const Segment SGI381452[] = {{&S62275[7], 11}, {NULL}};
const Segment SGI381453[] = {{&S62275[49], 3}, {NULL}};
const Segment SGI381454[] = {{&S47849[11], 5}, {NULL}};
const Segment SGI381455[] = {{&S47849[80], 3}, {NULL}};
const Segment SGI381456[] = {{&S47850[11], 6}, {NULL}};
const Segment SGI381457[] = {{&S47850[73], 3}, {NULL}};
const Segment SGI381458[] = {{&S47851[24], 5}, {NULL}};
const Segment SGI381459[] = {{&S47851[87], 3}, {NULL}};
const Segment SGI381460[] = {{&S47852[24], 5}, {NULL}};
const Segment SGI381461[] = {{&S47852[87], 3}, {NULL}};
const Segment SGI381462[] = {{&S47853[24], 5}, {NULL}};
const Segment SGI381463[] = {{&S47853[87], 3}, {NULL}};
const Segment SGI381464[] = {{&S47854[24], 5}, {NULL}};
const Segment SGI381465[] = {{&S47854[87], 3}, {NULL}};
const Segment SGI381466[] = {{&S47999[33], 5}, {NULL}};
const Segment SGI381467[] = {{&S47999[59], 3}, {NULL}};
const Segment SGI381468[] = {{&S47999[116], 3}, {NULL}};
const Segment SGI381469[] = {{&S62630[7], 9}, {NULL}};
const Segment SGI381470[] = {{&S62630[78], 3}, {NULL}};
const Segment SGI381471[] = {{&S62643[7], 10}, {NULL}};
const Segment SGI381472[] = {{&S62643[31], 3}, {NULL}};
const Segment SGI381473[] = {{&S62644[7], 9}, {NULL}};
const Segment SGI381474[] = {{&S62644[30], 3}, {NULL}};
const Segment SGI381475[] = {{&S62773[7], 10}, {NULL}};
const Segment SGI381476[] = {{&S62773[80], 3}, {NULL}};
const Segment SGI381477[] = {{&S62774[20], 10}, {NULL}};
const Segment SGI381478[] = {{&S62774[92], 3}, {NULL}};
const Segment SGI381479[] = {{&S62775[20], 10}, {NULL}};
const Segment SGI381480[] = {{&S62775[92], 3}, {NULL}};
const Segment SGI381481[] = {{&S62776[20], 10}, {NULL}};
const Segment SGI381482[] = {{&S62776[92], 3}, {NULL}};
const Segment SGI381483[] = {{&S62842[3], 5}, {NULL}};
const Segment SGI381484[] = {{&S62842[22], 11}, {NULL}};
const Segment SGI381485[] = {{&S62877[85], 9}, {NULL}};
const Segment SGI381486[] = {{&S62877[89], 5}, {NULL}};
const Segment SGI381487[] = {{&S62878[85], 10}, {NULL}};
const Segment SGI381488[] = {{&S62878[89], 6}, {NULL}};
const Segment SGI381489[] = {{&S62914[7], 9}, {NULL}};
const Segment SGI381490[] = {{&S62914[36], 3}, {NULL}};
const Segment SGI381491[] = {{&S63133[7], 10}, {NULL}};
const Segment SGI381492[] = {{&S63133[48], 3}, {NULL}};
const Segment SGI381493[] = {{&S63134[7], 10}, {NULL}};
const Segment SGI381494[] = {{&S63134[48], 3}, {NULL}};
const Segment SGI381495[] = {{&S48760[11], 5}, {NULL}};
const Segment SGI381496[] = {{&S48760[41], 3}, {NULL}};
const Segment SGI381497[] = {{&S63135[7], 10}, {NULL}};
const Segment SGI381498[] = {{&S63135[48], 3}, {NULL}};
const Segment SGI381499[] = {{&S63136[7], 9}, {NULL}};
const Segment SGI381500[] = {{&S63136[47], 3}, {NULL}};
const Segment SGI381501[] = {{&S48906[33], 5}, {NULL}};
const Segment SGI381502[] = {{&S48906[39], 3}, {NULL}};
const Segment SGI381503[] = {{&S48906[106], 3}, {NULL}};
const Segment SGI381504[] = {{&S63373[36], 22}, {NULL}};
const Segment SGI381505[] = {{&S63373[86], 13}, {NULL}};
const Segment SGI381506[] = {{&S63413[67], 9}, {NULL}};
const Segment SGI381507[] = {{&S63533[7], 9}, {NULL}};
const Segment SGI381508[] = {{&S63533[47], 3}, {NULL}};
const Segment SGI381509[] = {{&S63567[36], 29}, {NULL}};
const Segment SGI381510[] = {{&S63568[36], 29}, {NULL}};
const Segment SGI381511[] = {{&S49257[11], 5}, {NULL}};
const Segment SGI381512[] = {{&S63590[44], 10}, {NULL}};
const Segment SGI381513[] = {{&S49257[37], 3}, {NULL}};
const Segment SGI381514[] = {{&S63590[9], 5}, {&S63590[20], 5}, {NULL}};
const Segment SGI381515[] = {{&S49258[11], 6}, {NULL}};
const Segment SGI381516[] = {{&S49258[39], 3}, {NULL}};
const Segment SGI381517[] = {{&S49259[11], 6}, {NULL}};
const Segment SGI381518[] = {{&S49259[31], 3}, {NULL}};
const Segment SGI381519[] = {{&S49260[11], 6}, {NULL}};
const Segment SGI381520[] = {{&S49260[31], 3}, {NULL}};
const Segment SGI381521[] = {{&S49261[11], 6}, {NULL}};
const Segment SGI381522[] = {{&S49261[39], 3}, {NULL}};
const Segment SGI381523[] = {{&S49385[11], 5}, {NULL}};
const Segment SGI381524[] = {{&S49385[30], 3}, {NULL}};
const Segment SGI381525[] = {{&S63682[69], 9}, {NULL}};
const Segment SGI381526[] = {{&S49481[11], 5}, {NULL}};
const Segment SGI381527[] = {{&S49481[37], 3}, {NULL}};
const Segment SGI381528[] = {{&S63824[7], 9}, {NULL}};
const Segment SGI381529[] = {{&S63824[47], 3}, {NULL}};
const Segment SGI381530[] = {{&S63952[67], 9}, {NULL}};
const Segment SGI381531[] = {{&S63952[71], 5}, {NULL}};
const Segment SGI381532[] = {{&S63977[7], 9}, {NULL}};
const Segment SGI381533[] = {{&S63977[45], 3}, {NULL}};
const Segment SGI381534[] = {{&S63978[44], 10}, {NULL}};
const Segment SGI381535[] = {{&S63978[9], 5}, {&S63978[20], 5}, {NULL}};
const Segment SGI381536[] = {{&S49888[11], 6}, {NULL}};
const Segment SGI381537[] = {{&S49888[41], 3}, {NULL}};
const Segment SGI381538[] = {{&S64246[7], 9}, {NULL}};
const Segment SGI381539[] = {{&S64246[78], 3}, {NULL}};
const Segment SGI381540[] = {{&S64293[7], 9}, {NULL}};
const Segment SGI381541[] = {{&S64293[46], 3}, {NULL}};
const Segment SGI381542[] = {{&S49958[33], 6}, {NULL}};
const Segment SGI381543[] = {{&S49958[61], 3}, {NULL}};
const Segment SGI381544[] = {{&S49958[119], 3}, {NULL}};
const Segment SGI381545[] = {{&S49959[33], 5}, {NULL}};
const Segment SGI381546[] = {{&S49959[68], 3}, {NULL}};
const Segment SGI381547[] = {{&S49959[126], 3}, {NULL}};
const Segment SGI381548[] = {{&S64302[7], 9}, {NULL}};
const Segment SGI381549[] = {{&S64302[41], 3}, {NULL}};
const Segment SGI381550[] = {{&S64303[7], 18}, {NULL}};
const Segment SGI381551[] = {{&S64303[56], 3}, {NULL}};
const Segment SGI381552[] = {{&S64304[7], 10}, {NULL}};
const Segment SGI381553[] = {{&S64304[42], 3}, {NULL}};
const Segment SGI381554[] = {{&S50283[11], 5}, {NULL}};
const Segment SGI381555[] = {{&S50283[30], 3}, {NULL}};
const Segment SGI381556[] = {{&S64582[93], 13}, {NULL}};
const Segment SGI381557[] = {{&S64582[97], 9}, {NULL}};
const Segment SGI381558[] = {{&S64654[7], 9}, {NULL}};
const Segment SGI381559[] = {{&S64654[78], 3}, {NULL}};
const Segment SGI381560[] = {{&S50731[11], 5}, {NULL}};
const Segment SGI381561[] = {{&S50731[37], 3}, {NULL}};
const Segment SGI381562[] = {{&S50753[11], 6}, {NULL}};
const Segment SGI381563[] = {{&S50753[49], 3}, {NULL}};
const Segment SGI381564[] = {{&S50770[11], 5}, {NULL}};
const Segment SGI381565[] = {{&S50770[54], 3}, {NULL}};
const Segment SGI381566[] = {{&S65021[3], 5}, {NULL}};
const Segment SGI381567[] = {{&S65021[22], 11}, {NULL}};
const Segment SGI381568[] = {{&S65028[42], 18}, {NULL}};
const Segment SGI381569[] = {{&S50810[11], 5}, {NULL}};
const Segment SGI381570[] = {{&S50810[37], 3}, {NULL}};
const Segment SGI381571[] = {{&S65033[7], 9}, {NULL}};
const Segment SGI381572[] = {{&S65051[7], 9}, {NULL}};
const Segment SGI381573[] = {{&S65051[41], 3}, {NULL}};
const Segment SGI381574[] = {{&S65197[7], 6}, {NULL}};
const Segment SGI381575[] = {{&S65197[14], 4}, {NULL}};
const Segment SGI381576[] = {{&S65197[42], 3}, {NULL}};
const Segment SGI381577[] = {{&S51128[11], 6}, {NULL}};
const Segment SGI381578[] = {{&S51128[40], 3}, {NULL}};
const Segment SGI381579[] = {{&S51129[11], 6}, {NULL}};
const Segment SGI381580[] = {{&S51129[40], 3}, {NULL}};
const Segment SGI381581[] = {{&S51130[11], 6}, {NULL}};
const Segment SGI381582[] = {{&S51130[44], 3}, {NULL}};
const Segment SGI381583[] = {{&S51131[11], 5}, {NULL}};
const Segment SGI381584[] = {{&S51131[46], 3}, {NULL}};
const Segment SGI381585[] = {{&S65209[7], 9}, {NULL}};
const Segment SGI381586[] = {{&S65209[47], 3}, {NULL}};
const Segment SGI381587[] = {{&S51175[11], 5}, {NULL}};
const Segment SGI381588[] = {{&S51175[30], 3}, {NULL}};
const Segment SGI381589[] = {{&S51176[11], 6}, {NULL}};
const Segment SGI381590[] = {{&S51176[31], 3}, {NULL}};
const Segment SGI381591[] = {{&S51177[11], 5}, {&S51177[11], 5}, {NULL}};
const Segment SGI381592[] = {{&S51177[30], 3}, {&S51177[30], 3}, {NULL}};
const Segment SGI381593[] = {{&S65258[7], 9}, {NULL}};
const Segment SGI381594[] = {{&S65258[47], 3}, {NULL}};
const Segment SGI381595[] = {{&S65337[7], 9}, {NULL}};
const Segment SGI381596[] = {{&S51328[11], 5}, {NULL}};
const Segment SGI381597[] = {{&S51328[30], 3}, {NULL}};
const Segment SGI381598[] = {{&S65464[7], 9}, {NULL}};
const Segment SGI381599[] = {{&S65464[72], 3}, {NULL}};
const Segment SGI381600[] = {{&S51427[11], 6}, {NULL}};
const Segment SGI381601[] = {{&S51427[56], 3}, {NULL}};
const Segment SGI381602[] = {{&S65646[7], 9}, {NULL}};
const Segment SGI381603[] = {{&S51601[11], 5}, {NULL}};
const Segment SGI381604[] = {{&S51601[17], 3}, {NULL}};
const Segment SGI381605[] = {{&S51757[11], 12}, {NULL}};
const Segment SGI381606[] = {{&S51757[24], 4}, {NULL}};
const Segment SGI381607[] = {{&S51757[52], 3}, {NULL}};
const Segment SGI381608[] = {{&S51769[33], 5}, {NULL}};
const Segment SGI381609[] = {{&S51769[59], 3}, {NULL}};
const Segment SGI381610[] = {{&S51769[116], 3}, {NULL}};
const Segment SGI381611[] = {{&S65828[7], 9}, {NULL}};
const Segment SGI381612[] = {{&S65828[47], 3}, {NULL}};
const Segment SGI381613[] = {{&S51790[11], 5}, {NULL}};
const Segment SGI381614[] = {{&S51790[30], 3}, {NULL}};
const Segment SGI381615[] = {{&S51876[11], 6}, {NULL}};
const Segment SGI381616[] = {{&S51876[31], 3}, {NULL}};
const Segment SGI381617[] = {{&S51877[11], 5}, {NULL}};
const Segment SGI381618[] = {{&S51877[38], 3}, {NULL}};
const Segment SGI381619[] = {{&S51878[11], 6}, {NULL}};
const Segment SGI381620[] = {{&S51878[39], 3}, {NULL}};
const Segment SGI381621[] = {{&S51897[11], 6}, {NULL}};
const Segment SGI381622[] = {{&S51897[48], 3}, {NULL}};
const Segment SGI381623[] = {{&S66027[7], 9}, {NULL}};
const Segment SGI381624[] = {{&S66027[43], 3}, {NULL}};
const Segment SGI381625[] = {{&S66028[7], 18}, {NULL}};
const Segment SGI381626[] = {{&S66028[52], 3}, {NULL}};
const Segment SGI381627[] = {{&S66042[7], 6}, {NULL}};
const Segment SGI381628[] = {{&S66042[27], 3}, {NULL}};
const Segment SGI381629[] = {{&S66154[46], 10}, {NULL}};
const Segment SGI381630[] = {{&S66154[9], 5}, {&S66154[9], 5}, {NULL}};
const Segment SGI381631[] = {{&S66172[7], 9}, {NULL}};
const Segment SGI381632[] = {{&S66172[47], 3}, {NULL}};
const Segment SGI381633[] = {{&S66173[7], 10}, {NULL}};
const Segment SGI381634[] = {{&S52279[11], 5}, {NULL}};
const Segment SGI381635[] = {{&S52279[39], 3}, {NULL}};
const Segment SGI381636[] = {{&S66173[49], 3}, {NULL}};
const Segment SGI381637[] = {{&S52280[11], 6}, {NULL}};
const Segment SGI381638[] = {{&S52280[40], 3}, {NULL}};
const Segment SGI381639[] = {{&S52281[11], 6}, {NULL}};
const Segment SGI381640[] = {{&S52281[47], 3}, {NULL}};
const Segment SGI381641[] = {{&S52282[11], 6}, {NULL}};
const Segment SGI381642[] = {{&S52282[40], 3}, {NULL}};
const Segment SGI381643[] = {{&S52301[11], 5}, {NULL}};
const Segment SGI381644[] = {{&S52301[79], 3}, {NULL}};
const Segment SGI381645[] = {{&S52302[11], 6}, {NULL}};
const Segment SGI381646[] = {{&S52302[81], 3}, {NULL}};
const Segment SGI381647[] = {{&S52303[11], 6}, {NULL}};
const Segment SGI381648[] = {{&S52303[81], 3}, {NULL}};
const Segment SGI381649[] = {{&S52304[11], 6}, {NULL}};
const Segment SGI381650[] = {{&S52304[73], 3}, {NULL}};
const Segment SGI381651[] = {{&S52318[11], 6}, {NULL}};
const Segment SGI381652[] = {{&S52318[31], 3}, {NULL}};
const Segment SGI381653[] = {{&S52319[11], 5}, {NULL}};
const Segment SGI381654[] = {{&S52319[37], 3}, {NULL}};
const Segment SGI381655[] = {{&S66417[7], 9}, {NULL}};
const Segment SGI381656[] = {{&S66417[78], 3}, {NULL}};
const Segment SGI381657[] = {{&S52559[33], 6}, {NULL}};
const Segment SGI381658[] = {{&S52559[40], 3}, {NULL}};
const Segment SGI381659[] = {{&S52559[110], 3}, {NULL}};
const Segment SGI381660[] = {{&S66611[36], 22}, {NULL}};
const Segment SGI381661[] = {{&S66611[86], 13}, {NULL}};
const Segment SGI381662[] = {{&S66851[7], 9}, {NULL}};
const Segment SGI381663[] = {{&S66851[43], 3}, {NULL}};
const Segment SGI381664[] = {{&S66958[36], 22}, {NULL}};
const Segment SGI381665[] = {{&S66958[86], 13}, {NULL}};
const Segment SGI381666[] = {{&S53000[11], 6}, {NULL}};
const Segment SGI381667[] = {{&S53000[31], 3}, {NULL}};
const Segment SGI381668[] = {{&S53001[11], 6}, {NULL}};
const Segment SGI381669[] = {{&S53001[39], 3}, {NULL}};
const Segment SGI381670[] = {{&S53002[11], 6}, {NULL}};
const Segment SGI381671[] = {{&S53002[31], 3}, {NULL}};
const Segment SGI381672[] = {{&S67123[7], 9}, {NULL}};
const Segment SGI381673[] = {{&S67123[38], 3}, {NULL}};
const Segment SGI381674[] = {{&S53167[11], 5}, {NULL}};
const Segment SGI381675[] = {{&S53167[30], 3}, {NULL}};
const Segment SGI381676[] = {{&S67185[36], 29}, {NULL}};
const Segment SGI381677[] = {{&S67186[36], 29}, {NULL}};
const Segment SGI381678[] = {{&S67258[38], 20}, {NULL}};
const Segment SGI381679[] = {{&S67258[6], 4}, {NULL}};
const Segment SGI381680[] = {{&S67279[7], 6}, {NULL}};
const Segment SGI381681[] = {{&S67280[20], 5}, {NULL}};
const Segment SGI381682[] = {{&S67281[20], 5}, {NULL}};
const Segment SGI381683[] = {{&S67282[20], 5}, {NULL}};
const Segment SGI381684[] = {{&S67283[20], 5}, {NULL}};
const Segment SGI381685[] = {{&S67388[7], 9}, {NULL}};
const Segment SGI381686[] = {{&S67388[30], 3}, {NULL}};
const Segment SGI381687[] = {{&S67389[7], 10}, {NULL}};
const Segment SGI381688[] = {{&S67389[28], 3}, {NULL}};
const Segment SGI381689[] = {{&S67409[7], 9}, {NULL}};
const Segment SGI381690[] = {{&S67409[47], 3}, {NULL}};
const Segment SGI381691[] = {{&S67559[7], 6}, {NULL}};
const Segment SGI381692[] = {{&S67559[34], 3}, {NULL}};
const Segment SGI381693[] = {{&S67880[47], 11}, {NULL}};
const Segment SGI381694[] = {{&S67880[9], 5}, {&S67880[9], 5}, {NULL}};
const Segment SGI381695[] = {{&S67958[7], 9}, {NULL}};
const Segment SGI381696[] = {{&S67958[41], 3}, {NULL}};
const Segment SGI381697[] = {{&S67959[7], 10}, {NULL}};
const Segment SGI381698[] = {{&S67959[31], 3}, {NULL}};
const Segment SGI381699[] = {{&S68126[7], 9}, {NULL}};
const Segment SGI381700[] = {{&S68146[7], 9}, {NULL}};
const Segment SGI381701[] = {{&S68146[47], 3}, {NULL}};
const Segment SGI381702[] = {{&S68209[136], 10}, {NULL}};
const Segment SGI381703[] = {{&S68221[7], 9}, {NULL}};
const Segment SGI381704[] = {{&S68221[43], 3}, {NULL}};
const Segment SGI381705[] = {{&S68306[36], 29}, {NULL}};
const Segment SGI381706[] = {{&S68318[3], 8}, {NULL}};
const Segment SGI381707[] = {{&S68319[3], 8}, {NULL}};
const Segment SGI381708[] = {{&S68319[25], 3}, {NULL}};
const Segment SGI381709[] = {{&S68380[7], 14}, {NULL}};
const Segment SGI381710[] = {{&S53739[11], 11}, {NULL}};
const Segment SGI381711[] = {{&S53739[23], 4}, {NULL}};
const Segment SGI381712[] = {{&S53739[51], 3}, {NULL}};
const Segment SGI381713[] = {{&S68603[36], 22}, {NULL}};
const Segment SGI381714[] = {{&S68603[86], 13}, {NULL}};
const Segment SGI381715[] = {{&S68643[7], 9}, {NULL}};
const Segment SGI381716[] = {{&S68643[47], 3}, {NULL}};
const Segment SGI381717[] = {{&S68746[72], 20}, {NULL}};
const Segment SGI381718[] = {{&S68746[76], 16}, {NULL}};
const Segment SGI381719[] = {{&S68746[6], 3}, {NULL}};
const Segment SGI381720[] = {{&S68747[72], 24}, {NULL}};
const Segment SGI381721[] = {{&S68747[76], 20}, {NULL}};
const Segment SGI381722[] = {{&S68747[6], 3}, {NULL}};
const Segment SGI381723[] = {{&S68748[72], 17}, {NULL}};
const Segment SGI381724[] = {{&S68748[6], 3}, {NULL}};
const Segment SGI381725[] = {{&S68895[7], 9}, {NULL}};
const Segment SGI381726[] = {{&S68895[78], 3}, {NULL}};
const Segment SGI381727[] = {{&S68971[36], 22}, {NULL}};
const Segment SGI381728[] = {{&S68971[86], 13}, {NULL}};
const Segment SGI381729[] = {{&S54364[11], 5}, {NULL}};
const Segment SGI381730[] = {{&S54364[46], 3}, {NULL}};
const Segment SGI381731[] = {{&S69252[7], 6}, {NULL}};
const Segment SGI381732[] = {{&S69252[34], 3}, {NULL}};
const Segment SGI381733[] = {{&S69299[7], 9}, {NULL}};
const Segment SGI381734[] = {{&S69300[7], 10}, {NULL}};
const Segment SGI381735[] = {{&S69301[7], 9}, {NULL}};
const Segment SGI381736[] = {{&S69302[130], 14}, {NULL}};
const Segment SGI381737[] = {{&S69329[36], 29}, {NULL}};
const Segment SGI381738[] = {{&S69362[42], 18}, {NULL}};
const Segment SGI381739[] = {{&S69396[7], 9}, {NULL}};
const Segment SGI381740[] = {{&S69396[47], 3}, {NULL}};
const Segment SGI381741[] = {{&S69443[36], 29}, {NULL}};
const Segment SGI381742[] = {{&S69444[36], 29}, {NULL}};
const Segment SGI381743[] = {{&S54731[11], 5}, {NULL}};
const Segment SGI381744[] = {{&S54731[46], 3}, {NULL}};
const Segment SGI381745[] = {{&S54732[11], 6}, {NULL}};
const Segment SGI381746[] = {{&S54732[48], 3}, {NULL}};
const Segment SGI381747[] = {{&S69608[7], 4}, {NULL}};
const Segment SGI381748[] = {{&S69816[43], 10}, {NULL}};
const Segment SGI381749[] = {{&S69816[9], 3}, {&S69816[9], 3}, {NULL}};
const Segment SGI381750[] = {{&S69868[7], 9}, {NULL}};
const Segment SGI381751[] = {{&S69868[47], 3}, {NULL}};
const Segment SGI381752[] = {{&S69869[7], 16}, {NULL}};
const Segment SGI381753[] = {{&S69869[54], 3}, {NULL}};
const Segment SGI381754[] = {{&S69870[7], 18}, {NULL}};
const Segment SGI381755[] = {{&S69870[56], 3}, {NULL}};
const Segment SGI381756[] = {{&S55215[11], 6}, {NULL}};
const Segment SGI381757[] = {{&S55215[39], 3}, {NULL}};
const Segment SGI381758[] = {{&S69919[7], 9}, {NULL}};
const Segment SGI381759[] = {{&S69919[11], 5}, {NULL}};
const Segment SGI381760[] = {{&S69919[36], 3}, {NULL}};
const Segment SGI381761[] = {{&S69925[36], 29}, {NULL}};
const Segment SGI381762[] = {{&S69926[36], 29}, {NULL}};
const Segment SGI381763[] = {{&S69927[7], 9}, {NULL}};
const Segment SGI381764[] = {{&S69927[17], 8}, {NULL}};
const Segment SGI381765[] = {{&S55281[11], 6}, {NULL}};
const Segment SGI381766[] = {{&S55281[39], 3}, {NULL}};
const Segment SGI381767[] = {{&S55282[11], 5}, {NULL}};
const Segment SGI381768[] = {{&S55282[37], 3}, {NULL}};
const Segment SGI381769[] = {{&S55309[11], 6}, {NULL}};
const Segment SGI381770[] = {{&S55309[49], 3}, {NULL}};
const Segment SGI381771[] = {{&S69992[3], 5}, {NULL}};
const Segment SGI381772[] = {{&S69992[22], 11}, {NULL}};
const Segment SGI381773[] = {{&S70021[36], 29}, {NULL}};
const Segment SGI381774[] = {{&S70063[7], 17}, {NULL}};
const Segment SGI381775[] = {{&S70063[87], 3}, {NULL}};
const Segment SGI381776[] = {{&S70064[7], 10}, {NULL}};
const Segment SGI381777[] = {{&S70064[73], 3}, {NULL}};
const Segment SGI381778[] = {{&S70166[7], 9}, {NULL}};
const Segment SGI381779[] = {{&S70166[41], 3}, {NULL}};
const Segment SGI381780[] = {{&S55567[11], 5}, {NULL}};
const Segment SGI381781[] = {{&S55567[30], 3}, {NULL}};
const Segment SGI381782[] = {{&S55568[11], 5}, {NULL}};
const Segment SGI381783[] = {{&S70256[7], 6}, {NULL}};
const Segment SGI381784[] = {{&S70256[27], 3}, {NULL}};
const Segment SGI381785[] = {{&S70258[20], 5}, {NULL}};
const Segment SGI381786[] = {{&S70258[54], 3}, {NULL}};
const Segment SGI381787[] = {{&S70419[7], 9}, {NULL}};
const Segment SGI381788[] = {{&S70419[30], 3}, {NULL}};
const Segment SGI381789[] = {{&S55774[33], 6}, {NULL}};
const Segment SGI381790[] = {{&S55774[61], 3}, {NULL}};
const Segment SGI381791[] = {{&S55774[118], 3}, {NULL}};
const Segment SGI381792[] = {{&S70554[7], 9}, {NULL}};
const Segment SGI381793[] = {{&S70554[41], 3}, {NULL}};
const Segment SGI381794[] = {{&S70639[7], 9}, {NULL}};
const Segment SGI381795[] = {{&S70639[124], 4}, {NULL}};
const Segment SGI381796[] = {{&S70663[7], 9}, {NULL}};
const Segment SGI381797[] = {{&S55884[11], 5}, {NULL}};
const Segment SGI381798[] = {{&S55884[37], 3}, {NULL}};
const Segment SGI381799[] = {{&S70665[7], 9}, {NULL}};
const Segment SGI381800[] = {{&S70665[11], 5}, {NULL}};
const Segment SGI381801[] = {{&S70747[36], 30}, {NULL}};
const Segment SGI381802[] = {{&S70748[36], 30}, {NULL}};
const Segment SGI381803[] = {{&S56115[11], 11}, {NULL}};
const Segment SGI381804[] = {{&S56115[23], 4}, {NULL}};
const Segment SGI381805[] = {{&S56115[51], 3}, {NULL}};
const Segment SGI381806[] = {{&S70856[36], 23}, {NULL}};
const Segment SGI381807[] = {{&S70856[87], 13}, {NULL}};
const Segment SGI381808[] = {{&S71011[7], 9}, {NULL}};
const Segment SGI381809[] = {{&S71011[78], 3}, {NULL}};
const Segment SGI381810[] = {{&S71051[102], 19}, {NULL}};
const Segment SGI381811[] = {{&S71174[3], 6}, {NULL}};
const Segment SGI381812[] = {{&S71174[23], 11}, {NULL}};
const Segment SGI381813[] = {{&S71209[7], 9}, {NULL}};
const Segment SGI381814[] = {{&S71209[72], 3}, {NULL}};
const Segment SGI381815[] = {{&S71210[20], 9}, {NULL}};
const Segment SGI381816[] = {{&S71210[91], 3}, {NULL}};
const Segment SGI381817[] = {{&S71214[68], 9}, {NULL}};
const Segment SGI381818[] = {{&S71215[7], 9}, {&S71215[7], 9}, {NULL}};
const Segment SGI381819[] = {{&S71216[7], 15}, {NULL}};
const Segment SGI381820[] = {{&S71217[7], 12}, {NULL}};
const Segment SGI381821[] = {{&S71218[7], 12}, {NULL}};
const Segment SGI381822[] = {{&S71219[7], 10}, {NULL}};
const Segment SGI381823[] = {{&S71220[7], 9}, {NULL}};
const Segment SGI381824[] = {{&S71311[36], 22}, {NULL}};
const Segment SGI381825[] = {{&S71311[86], 13}, {NULL}};
const Segment SGI381826[] = {{&S71425[3], 15}, {NULL}};
const Segment SGI381827[] = {{&S71425[32], 3}, {NULL}};
const Segment SGI381828[] = {{&S71426[3], 16}, {NULL}};
const Segment SGI381829[] = {{&S71426[33], 3}, {NULL}};
const Segment SGI381830[] = {{&S71427[3], 8}, {NULL}};
const Segment SGI381831[] = {{&S71427[25], 3}, {NULL}};
const Segment SGI381832[] = {{&S71557[36], 29}, {NULL}};
const Segment SGI381833[] = {{&S71558[36], 29}, {NULL}};
const Segment SGI381834[] = {{&S71591[7], 6}, {NULL}};
const Segment SGI381835[] = {{&S71591[38], 3}, {NULL}};
const Segment SGI381836[] = {{&S71647[36], 29}, {NULL}};
const Segment SGI381837[] = {{&S71648[36], 29}, {NULL}};
const Segment SGI381838[] = {{&S71649[7], 9}, {NULL}};
const Segment SGI381839[] = {{&S71649[17], 8}, {NULL}};
const Segment SGI381840[] = {{&S71668[74], 5}, {NULL}};
const Segment SGI381841[] = {{&S71669[85], 17}, {NULL}};
const Segment SGI381842[] = {{&S71701[7], 6}, {NULL}};
const Segment SGI381843[] = {{&S71701[40], 3}, {NULL}};
const Segment SGI381844[] = {{&S71706[7], 6}, {NULL}};
const Segment SGI381845[] = {{&S71851[7], 18}, {NULL}};
const Segment SGI381846[] = {{&S71851[52], 3}, {NULL}};
const Segment SGI381847[] = {{&S71852[7], 9}, {NULL}};
const Segment SGI381848[] = {{&S71852[41], 3}, {NULL}};
const Segment SGI381849[] = {{&S71890[7], 15}, {NULL}};
const Segment SGI381850[] = {{&S71890[54], 3}, {NULL}};
const Segment SGI381851[] = {{&S71891[7], 6}, {NULL}};
const Segment SGI381852[] = {{&S71891[38], 3}, {NULL}};
const Segment SGI381853[] = {{&S71892[7], 7}, {NULL}};
const Segment SGI381854[] = {{&S71892[39], 3}, {NULL}};
const Segment SGI381855[] = {{&S71931[7], 10}, {NULL}};
const Segment SGI381856[] = {{&S71931[73], 3}, {NULL}};
const Segment SGI381857[] = {{&S71932[7], 9}, {NULL}};
const Segment SGI381858[] = {{&S71932[72], 3}, {NULL}};
const Segment SGI381859[] = {{&S57234[11], 5}, {NULL}};
const Segment SGI381860[] = {{&S57234[30], 3}, {NULL}};
const Segment SGI381861[] = {{&S72127[36], 22}, {NULL}};
const Segment SGI381862[] = {{&S72127[86], 13}, {NULL}};
const Segment SGI381863[] = {{&S72176[36], 29}, {NULL}};
const Segment SGI381864[] = {{&S72177[36], 29}, {NULL}};
const Segment SGI381865[] = {{&S72218[36], 22}, {NULL}};
const Segment SGI381866[] = {{&S72218[86], 13}, {NULL}};
const Segment SGI381867[] = {{&S72251[36], 29}, {NULL}};
const Segment SGI381868[] = {{&S72446[3], 16}, {NULL}};
const Segment SGI381869[] = {{&S72446[20], 3}, {NULL}};
const Segment SGI381870[] = {{&S57645[11], 5}, {NULL}};
const Segment SGI381871[] = {{&S57645[30], 3}, {NULL}};
const Segment SGI381872[] = {{&S57646[11], 5}, {NULL}};
const Segment SGI381873[] = {{&S57678[11], 5}, {NULL}};
const Segment SGI381874[] = {{&S57678[30], 3}, {NULL}};
const Segment SGI381875[] = {{&S57679[11], 6}, {NULL}};
const Segment SGI381876[] = {{&S57679[31], 3}, {NULL}};
const Segment SGI381877[] = {{&S72555[7], 15}, {NULL}};
const Segment SGI381878[] = {{&S72555[54], 3}, {NULL}};
const Segment SGI381879[] = {{&S72556[7], 7}, {NULL}};
const Segment SGI381880[] = {{&S72556[39], 3}, {NULL}};
const Segment SGI381881[] = {{&S72557[7], 6}, {NULL}};
const Segment SGI381882[] = {{&S72557[45], 3}, {NULL}};
const Segment SGI381883[] = {{&S72558[7], 13}, {NULL}};
const Segment SGI381884[] = {{&S72558[52], 3}, {NULL}};
const Segment SGI381885[] = {{&S72688[36], 30}, {NULL}};
const Segment SGI381886[] = {{&S72689[36], 30}, {NULL}};
const Segment SGI381887[] = {{&S72776[51], 20}, {NULL}};
const Segment SGI381888[] = {{&S72777[51], 20}, {NULL}};
const Segment SGI381889[] = {{&S72778[51], 20}, {NULL}};
const Segment SGI381890[] = {{&S72781[36], 23}, {NULL}};
const Segment SGI381891[] = {{&S72781[87], 13}, {NULL}};
const Segment SGI381892[] = {{&S72834[7], 6}, {NULL}};
const Segment SGI381893[] = {{&S72851[7], 16}, {NULL}};
const Segment SGI381894[] = {{&S72851[79], 3}, {NULL}};
const Segment SGI381895[] = {{&S72852[7], 14}, {NULL}};
const Segment SGI381896[] = {{&S72852[77], 3}, {NULL}};
const Segment SGI381897[] = {{&S72853[7], 9}, {NULL}};
const Segment SGI381898[] = {{&S72853[72], 3}, {NULL}};
const Segment SGI381899[] = {{&S72882[7], 9}, {NULL}};
const Segment SGI381900[] = {{&S72882[72], 3}, {NULL}};
const Segment SGI381901[] = {{&S58037[11], 6}, {NULL}};
const Segment SGI381902[] = {{&S58037[73], 3}, {NULL}};
const Segment SGI381903[] = {{&S58038[11], 6}, {NULL}};
const Segment SGI381904[] = {{&S58038[81], 3}, {NULL}};
const Segment SGI381905[] = {{&S58039[11], 6}, {NULL}};
const Segment SGI381906[] = {{&S58039[81], 3}, {NULL}};
const Segment SGI381907[] = {{&S58040[11], 5}, {NULL}};
const Segment SGI381908[] = {{&S58040[72], 3}, {NULL}};
const Segment SGI381909[] = {{&S72983[3], 17}, {NULL}};
const Segment SGI381910[] = {{&S72983[21], 3}, {NULL}};
const Segment SGI381911[] = {{&S58403[11], 54}, {NULL}};
const Segment SGI381912[] = {{&S58403[24], 3}, {NULL}};
const Segment SGI381913[] = {{&S58404[11], 6}, {NULL}};
const Segment SGI381914[] = {{&S58404[60], 3}, {NULL}};
const Segment SGI381915[] = {{&S58405[11], 5}, {NULL}};
const Segment SGI381916[] = {{&S58405[27], 3}, {NULL}};
const Segment SGI381917[] = {{&S58405[49], 3}, {NULL}};
const Segment SGI381918[] = {{&S58406[11], 6}, {NULL}};
const Segment SGI381919[] = {{&S58406[63], 3}, {NULL}};
const Segment SGI381920[] = {{&S58407[11], 6}, {NULL}};
const Segment SGI381921[] = {{&S58407[60], 3}, {NULL}};
const Segment SGI381922[] = {{&S58408[11], 5}, {NULL}};
const Segment SGI381923[] = {{&S58408[59], 3}, {NULL}};
const Segment SGI381924[] = {{&S58409[11], 11}, {NULL}};
const Segment SGI381925[] = {{&S58409[67], 3}, {NULL}};
const Segment SGI381926[] = {{&S73250[3], 8}, {NULL}};
const Segment SGI381927[] = {{&S73250[25], 3}, {NULL}};
const Segment SGI381928[] = {{&S58608[11], 6}, {NULL}};
const Segment SGI381929[] = {{&S58608[41], 3}, {NULL}};
const Segment SGI381930[] = {{&S73427[7], 9}, {NULL}};
const Segment SGI381931[] = {{&S73427[67], 3}, {NULL}};
const Segment SGI381932[] = {{&S73431[36], 29}, {NULL}};
const Segment SGI381933[] = {{&S73539[36], 22}, {NULL}};
const Segment SGI381934[] = {{&S73539[86], 13}, {NULL}};
const Segment SGI381935[] = {{&S59029[11], 5}, {NULL}};
const Segment SGI381936[] = {{&S59029[48], 3}, {NULL}};
const Segment SGI381937[] = {{&S73818[7], 5}, {NULL}};
const Segment SGI381938[] = {{&S73857[36], 29}, {NULL}};
const Segment SGI381939[] = {{&S73858[36], 29}, {NULL}};
const Segment SGI381940[] = {{&S73909[7], 9}, {NULL}};
const Segment SGI381941[] = {{&S73909[47], 3}, {NULL}};
const Segment SGI381942[] = {{&S73974[64], 1}, {NULL}};
const Segment SGI381943[] = {{&S59161[11], 5}, {NULL}};
const Segment SGI381944[] = {{&S59161[40], 3}, {NULL}};
const Segment SGI381945[] = {{&S59162[11], 6}, {NULL}};
const Segment SGI381946[] = {{&S59162[41], 3}, {NULL}};
const Segment SGI381947[] = {{&S74047[7], 6}, {NULL}};
const Segment SGI381948[] = {{&S74047[45], 3}, {NULL}};
const Segment SGI381949[] = {{&S74165[7], 10}, {NULL}};
const Segment SGI381950[] = {{&S74166[7], 9}, {NULL}};
const Segment SGI381951[] = {{&S74221[7], 16}, {NULL}};
const Segment SGI381952[] = {{&S74221[45], 3}, {NULL}};
const Segment SGI381953[] = {{&S74222[7], 9}, {NULL}};
const Segment SGI381954[] = {{&S59409[11], 6}, {NULL}};
const Segment SGI381955[] = {{&S74222[38], 3}, {NULL}};
const Segment SGI381956[] = {{&S59409[31], 3}, {NULL}};
const Segment SGI381957[] = {{&S74223[7], 10}, {NULL}};
const Segment SGI381958[] = {{&S74223[33], 3}, {NULL}};
const Segment SGI381959[] = {{&S59466[11], 5}, {NULL}};
const Segment SGI381960[] = {{&S59466[30], 3}, {NULL}};
const Segment SGI381961[] = {{&S59467[11], 12}, {NULL}};
const Segment SGI381962[] = {{&S59467[44], 3}, {NULL}};
const Segment SGI381963[] = {{&S74371[7], 16}, {NULL}};
const Segment SGI381964[] = {{&S74371[47], 3}, {NULL}};
const Segment SGI381965[] = {{&S74372[7], 7}, {NULL}};
const Segment SGI381966[] = {{&S74372[38], 3}, {NULL}};
const Segment SGI381967[] = {{&S74373[7], 7}, {NULL}};
const Segment SGI381968[] = {{&S74373[38], 3}, {NULL}};
const Segment SGI381969[] = {{&S74374[7], 14}, {NULL}};
const Segment SGI381970[] = {{&S74374[45], 3}, {NULL}};
const Segment SGI381971[] = {{&S74375[7], 6}, {NULL}};
const Segment SGI381972[] = {{&S74375[36], 3}, {NULL}};
const Segment SGI381973[] = {{&S74376[20], 7}, {NULL}};
const Segment SGI381974[] = {{&S74376[49], 3}, {NULL}};
const Segment SGI381975[] = {{&S59732[11], 5}, {NULL}};
const Segment SGI381976[] = {{&S59732[30], 3}, {NULL}};
const Segment SGI381977[] = {{&S59734[11], 5}, {NULL}};
const Segment SGI381978[] = {{&S74652[7], 10}, {NULL}};
const Segment SGI381979[] = {{&S74652[73], 3}, {NULL}};
const Segment SGI381980[] = {{&S74653[7], 9}, {NULL}};
const Segment SGI381981[] = {{&S74653[72], 3}, {NULL}};
const Segment SGI381982[] = {{&S74790[7], 10}, {NULL}};
const Segment SGI381983[] = {{&S74790[73], 3}, {NULL}};
const Segment SGI381984[] = {{&S74791[7], 10}, {NULL}};
const Segment SGI381985[] = {{&S74791[73], 3}, {NULL}};
const Segment SGI381986[] = {{&S74792[7], 9}, {NULL}};
const Segment SGI381987[] = {{&S74792[72], 3}, {NULL}};
const Segment SGI381988[] = {{&S74793[7], 9}, {NULL}};
const Segment SGI381989[] = {{&S74793[36], 3}, {NULL}};
const Segment SGI381990[] = {{&S75210[7], 9}, {NULL}};
const Segment SGI381991[] = {{&S75210[78], 3}, {NULL}};
const Segment SGI381992[] = {{&S75242[36], 29}, {NULL}};
const Segment SGI381993[] = {{&S75243[36], 29}, {NULL}};
const Segment SGI381994[] = {{&S60375[11], 5}, {NULL}};
const Segment SGI381995[] = {{&S60375[30], 3}, {NULL}};
const Segment SGI381996[] = {{&S60376[11], 5}, {NULL}};
const Segment SGI381997[] = {{&S60377[11], 6}, {NULL}};
const Segment SGI381998[] = {{&S60377[31], 3}, {NULL}};
const Segment SGI381999[] = {{&S75344[43], 13}, {NULL}};
const Segment SGI382000[] = {{&S75344[63], 13}, {NULL}};
const Segment SGI382001[] = {{&S75349[44], 13}, {NULL}};
const Segment SGI382002[] = {{&S75349[64], 13}, {NULL}};
const Segment SGI382003[] = {{&S75418[44], 13}, {NULL}};
const Segment SGI382004[] = {{&S75418[64], 13}, {NULL}};
const Segment SGI382005[] = {{&S60723[11], 5}, {NULL}};
const Segment SGI382006[] = {{&S60723[79], 3}, {NULL}};
const Segment SGI382007[] = {{&S60724[11], 6}, {NULL}};
const Segment SGI382008[] = {{&S60724[73], 3}, {NULL}};
const Segment SGI382009[] = {{&S60725[11], 6}, {NULL}};
const Segment SGI382010[] = {{&S60725[81], 3}, {NULL}};
const Segment SGI382011[] = {{&S60781[11], 11}, {NULL}};
const Segment SGI382012[] = {{&S60781[23], 4}, {NULL}};
const Segment SGI382013[] = {{&S60781[51], 3}, {NULL}};
const Segment SGI382014[] = {{&S75546[7], 9}, {NULL}};
const Segment SGI382015[] = {{&S75551[36], 29}, {NULL}};
const Segment SGI382016[] = {{&S75552[36], 29}, {NULL}};
const Segment SGI382017[] = {{&S75568[51], 21}, {NULL}};
const Segment SGI382018[] = {{&S75569[51], 20}, {NULL}};
const Segment SGI382019[] = {{&S75614[7], 10}, {NULL}};
const Segment SGI382020[] = {{&S75614[46], 3}, {NULL}};
const Segment SGI382021[] = {{&S60969[89], 6}, {NULL}};
const Segment SGI382022[] = {{&S60969[30], 3}, {NULL}};
const Segment SGI382023[] = {{&S60970[89], 6}, {NULL}};
const Segment SGI382024[] = {{&S60970[30], 3}, {NULL}};
const Segment SGI382025[] = {{&S60971[11], 6}, {NULL}};
const Segment SGI382026[] = {{&S60971[40], 3}, {NULL}};
const Segment SGI382027[] = {{&S60972[89], 6}, {NULL}};
const Segment SGI382028[] = {{&S60972[30], 3}, {NULL}};
const Segment SGI382029[] = {{&S60973[11], 80}, {NULL}};
const Segment SGI382030[] = {{&S60973[44], 3}, {NULL}};
const Segment SGI382031[] = {{&S60974[89], 6}, {NULL}};
const Segment SGI382032[] = {{&S60974[30], 3}, {NULL}};
const Segment SGI382033[] = {{&S60975[89], 6}, {NULL}};
const Segment SGI382034[] = {{&S60975[30], 3}, {NULL}};
const Segment SGI382035[] = {{&S60976[11], 11}, {NULL}};
const Segment SGI382036[] = {{&S60976[47], 3}, {NULL}};
const Segment SGI382037[] = {{&S60977[11], 5}, {NULL}};
const Segment SGI382038[] = {{&S60977[39], 3}, {NULL}};
const Segment SGI382039[] = {{&S60978[89], 6}, {NULL}};
const Segment SGI382040[] = {{&S60978[30], 3}, {NULL}};
const Segment SGI382041[] = {{&S61014[11], 5}, {NULL}};
const Segment SGI382042[] = {{&S61014[47], 3}, {NULL}};
const Segment SGI382043[] = {{&S61063[11], 5}, {NULL}};
const Segment SGI382044[] = {{&S61064[11], 5}, {NULL}};
const Segment SGI382045[] = {{&S61064[30], 3}, {NULL}};
const Segment SGI382046[] = {{&S75691[7], 9}, {NULL}};
const Segment SGI382047[] = {{&S61094[11], 6}, {NULL}};
const Segment SGI382048[] = {{&S61094[39], 3}, {NULL}};
const Segment SGI382049[] = {{&S61095[11], 6}, {NULL}};
const Segment SGI382050[] = {{&S61095[39], 3}, {NULL}};
const Segment SGI382051[] = {{&S61096[11], 6}, {NULL}};
const Segment SGI382052[] = {{&S61096[31], 3}, {NULL}};
const Segment SGI382053[] = {{&S61097[11], 6}, {NULL}};
const Segment SGI382054[] = {{&S61097[31], 3}, {NULL}};
const Segment SGI382055[] = {{&S61098[11], 5}, {NULL}};
const Segment SGI382056[] = {{&S61098[37], 3}, {NULL}};
const Segment SGI382057[] = {{&S61152[11], 5}, {NULL}};
const Segment SGI382058[] = {{&S61152[30], 3}, {NULL}};
const Segment SGI382059[] = {{&S61386[11], 5}, {NULL}};
const Segment SGI382060[] = {{&S61386[30], 3}, {NULL}};
const Segment SGI382061[] = {{&S75901[55], 7}, {NULL}};
const Segment SGI382062[] = {{&S75902[64], 7}, {NULL}};
const Segment SGI382063[] = {{&S75903[64], 7}, {NULL}};
const Segment SGI382064[] = {{&S75904[55], 7}, {NULL}};
const Segment SGI382065[] = {{&S75905[55], 7}, {NULL}};
const Segment SGI382066[] = {{&S75906[55], 7}, {NULL}};
const Segment SGI382067[] = {{&S75907[55], 7}, {NULL}};
const Segment SGI382068[] = {{&S75908[55], 7}, {NULL}};
const Segment SGI382069[] = {{&S75909[55], 7}, {NULL}};
const Segment SGI382070[] = {{&S75910[64], 7}, {NULL}};
const Segment SGI382071[] = {{&S75911[64], 7}, {NULL}};
const Segment SGI382072[] = {{&S75912[64], 7}, {NULL}};
const Segment SGI382073[] = {{&S75913[64], 7}, {NULL}};
const Segment SGI382074[] = {{&S75914[64], 7}, {NULL}};
const Segment SGI382075[] = {{&S75915[64], 7}, {NULL}};
const Segment SGI382076[] = {{&S75916[55], 7}, {NULL}};
const Segment SGI382077[] = {{&S75917[55], 7}, {NULL}};
const Segment SGI382078[] = {{&S75918[55], 7}, {NULL}};
const Segment SGI382079[] = {{&S75919[51], 9}, {NULL}};
const Segment SGI382080[] = {{&S75920[64], 7}, {NULL}};
const Segment SGI382081[] = {{&S75923[7], 8}, {NULL}};
const Segment SGI382082[] = {{&S75923[39], 3}, {NULL}};
const Segment SGI382083[] = {{&S75924[7], 7}, {NULL}};
const Segment SGI382084[] = {{&S75924[38], 3}, {NULL}};
const Segment SGI382085[] = {{&S75925[7], 7}, {NULL}};
const Segment SGI382086[] = {{&S75925[30], 3}, {NULL}};
const Segment SGI382087[] = {{&S75926[7], 6}, {NULL}};
const Segment SGI382088[] = {{&S75926[29], 3}, {NULL}};
const Segment SGI382089[] = {{&S75927[7], 16}, {NULL}};
const Segment SGI382090[] = {{&S75927[47], 3}, {NULL}};
const Segment SGI382091[] = {{&S61635[11], 5}, {NULL}};
const Segment SGI382092[] = {{&S61635[40], 3}, {NULL}};
const Segment SGI382093[] = {{&S61669[11], 5}, {NULL}};
const Segment SGI382094[] = {{&S61669[55], 3}, {NULL}};
const Segment SGI382095[] = {{&S61770[11], 5}, {NULL}};
const Segment SGI382096[] = {{&S61770[30], 3}, {NULL}};
const Segment SGI382097[] = {{&S61785[11], 6}, {NULL}};
const Segment SGI382098[] = {{&S76225[7], 6}, {NULL}};
const Segment SGI382099[] = {{&S76225[27], 3}, {NULL}};
const Segment SGI382100[] = {{&S76226[7], 7}, {NULL}};
const Segment SGI382101[] = {{&S76226[28], 3}, {NULL}};
const Segment SGI382102[] = {{&S76262[7], 9}, {NULL}};
const Segment SGI382103[] = {{&S76262[41], 3}, {NULL}};
const Segment SGI382104[] = {{&S61896[11], 6}, {NULL}};
const Segment SGI382105[] = {{&S61896[42], 3}, {NULL}};
const Segment SGI382106[] = {{&S61897[11], 5}, {NULL}};
const Segment SGI382107[] = {{&S61897[41], 3}, {NULL}};
const Segment SGI382108[] = {{&S76316[20], 10}, {NULL}};
const Segment SGI382109[] = {{&S76316[92], 3}, {NULL}};
const Segment SGI382110[] = {{&S76317[20], 10}, {NULL}};
const Segment SGI382111[] = {{&S76317[92], 3}, {NULL}};
const Segment SGI382112[] = {{&S76318[20], 10}, {NULL}};
const Segment SGI382113[] = {{&S76318[92], 3}, {NULL}};
const Segment SGI382114[] = {{&S76319[20], 10}, {NULL}};
const Segment SGI382115[] = {{&S76319[92], 3}, {NULL}};
const Segment SGI382116[] = {{&S76320[20], 10}, {NULL}};
const Segment SGI382117[] = {{&S76320[92], 3}, {NULL}};
const Segment SGI382118[] = {{&S76321[20], 10}, {NULL}};
const Segment SGI382119[] = {{&S76321[92], 3}, {NULL}};
const Segment SGI382120[] = {{&S76322[20], 10}, {NULL}};
const Segment SGI382121[] = {{&S76322[92], 3}, {NULL}};
const Segment SGI382122[] = {{&S62000[11], 5}, {NULL}};
const Segment SGI382123[] = {{&S62000[17], 3}, {NULL}};
const Segment SGI382124[] = {{&S76477[7], 6}, {NULL}};
const Segment SGI382125[] = {{&S76477[36], 3}, {NULL}};
const Segment SGI382126[] = {{&S76480[7], 9}, {NULL}};
const Segment SGI382127[] = {{&S76480[47], 3}, {NULL}};
const Segment SGI382128[] = {{&S76481[7], 10}, {NULL}};
const Segment SGI382129[] = {{&S76481[33], 3}, {NULL}};
const Segment SGI382130[] = {{&S76482[7], 9}, {NULL}};
const Segment SGI382131[] = {{&S76482[32], 3}, {NULL}};
const Segment SGI382132[] = {{&S76554[7], 9}, {NULL}};
const Segment SGI382133[] = {{&S76555[36], 29}, {NULL}};
const Segment SGI382134[] = {{&S76556[36], 29}, {NULL}};
const Segment SGI382135[] = {{&S76596[3], 8}, {NULL}};
const Segment SGI382136[] = {{&S76597[3], 8}, {NULL}};
const Segment SGI382137[] = {{&S76597[25], 3}, {NULL}};
const Segment SGI382138[] = {{&S76598[7], 9}, {NULL}};
const Segment SGI382139[] = {{&S76598[46], 3}, {NULL}};
const Segment SGI382140[] = {{&S76651[36], 29}, {NULL}};
const Segment SGI382141[] = {{&S76652[36], 29}, {NULL}};
const Segment SGI382142[] = {{&S76653[7], 9}, {NULL}};
const Segment SGI382143[] = {{&S76653[17], 8}, {NULL}};
const Segment SGI382144[] = {{&S62235[11], 6}, {NULL}};
const Segment SGI382145[] = {{&S62236[11], 5}, {NULL}};
const Segment SGI382146[] = {{&S62237[11], 6}, {NULL}};
const Segment SGI382147[] = {{&S62301[11], 5}, {NULL}};
const Segment SGI382148[] = {{&S62301[30], 3}, {NULL}};
const Segment SGI382149[] = {{&S62305[11], 5}, {NULL}};
const Segment SGI382150[] = {{&S62305[55], 3}, {NULL}};
const Segment SGI382151[] = {{&S76763[7], 14}, {NULL}};
const Segment SGI382152[] = {{&S76763[46], 3}, {NULL}};
const Segment SGI382153[] = {{&S76764[7], 14}, {NULL}};
const Segment SGI382154[] = {{&S76764[46], 3}, {NULL}};
const Segment SGI382155[] = {{&S76765[7], 14}, {NULL}};
const Segment SGI382156[] = {{&S76765[46], 3}, {NULL}};
const Segment SGI382157[] = {{&S76766[7], 14}, {NULL}};
const Segment SGI382158[] = {{&S76766[46], 3}, {NULL}};
const Segment SGI382159[] = {{&S76767[7], 14}, {NULL}};
const Segment SGI382160[] = {{&S76767[46], 3}, {NULL}};
const Segment SGI382161[] = {{&S76768[7], 9}, {NULL}};
const Segment SGI382162[] = {{&S76768[41], 3}, {NULL}};
const Segment SGI382163[] = {{&S76769[7], 14}, {NULL}};
const Segment SGI382164[] = {{&S76769[46], 3}, {NULL}};
const Segment SGI382165[] = {{&S76770[7], 14}, {NULL}};
const Segment SGI382166[] = {{&S76770[46], 3}, {NULL}};
const Segment SGI382167[] = {{&S76854[7], 9}, {NULL}};
const Segment SGI382168[] = {{&S76854[47], 3}, {NULL}};
const Segment SGI382169[] = {{&S76855[7], 18}, {NULL}};
const Segment SGI382170[] = {{&S76855[56], 3}, {NULL}};
const Segment SGI382171[] = {{&S76859[7], 9}, {NULL}};
const Segment SGI382172[] = {{&S76859[11], 5}, {NULL}};
const Segment SGI382173[] = {{&S76859[36], 3}, {NULL}};
const Segment SGI382174[] = {{&S76898[7], 11}, {NULL}};
const Segment SGI382175[] = {{&S76898[35], 3}, {NULL}};
const Segment SGI382176[] = {{&S76954[42], 19}, {NULL}};
const Segment SGI382177[] = {{&S76955[42], 19}, {NULL}};
const Segment SGI382178[] = {{&S77126[7], 15}, {NULL}};
const Segment SGI382179[] = {{&S77126[23], 4}, {NULL}};
const Segment SGI382180[] = {{&S77126[51], 3}, {NULL}};
const Segment SGI382181[] = {{&S77184[7], 10}, {NULL}};
const Segment SGI382182[] = {{&S77184[48], 3}, {NULL}};
const Segment SGI382183[] = {{&S77224[43], 13}, {NULL}};
const Segment SGI382184[] = {{&S77224[63], 13}, {NULL}};
const Segment SGI382185[] = {{&S62769[11], 5}, {NULL}};
const Segment SGI382186[] = {{&S62769[30], 3}, {NULL}};
const Segment SGI382187[] = {{&S62770[11], 12}, {NULL}};
const Segment SGI382188[] = {{&S62770[44], 3}, {NULL}};
const Segment SGI382189[] = {{&S77460[7], 9}, {NULL}};
const Segment SGI382190[] = {{&S77460[41], 3}, {NULL}};
const Segment SGI382191[] = {{&S77516[7], 9}, {NULL}};
const Segment SGI382192[] = {{&S77517[7], 15}, {NULL}};
const Segment SGI382193[] = {{&S77518[68], 9}, {NULL}};
const Segment SGI382194[] = {{&S77519[7], 12}, {NULL}};
const Segment SGI382195[] = {{&S77520[68], 10}, {NULL}};
const Segment SGI382196[] = {{&S77521[68], 9}, {NULL}};
const Segment SGI382197[] = {{&S77725[7], 9}, {NULL}};
const Segment SGI382198[] = {{&S77725[72], 3}, {NULL}};
const Segment SGI382199[] = {{&S77726[7], 10}, {NULL}};
const Segment SGI382200[] = {{&S77726[73], 3}, {NULL}};
const Segment SGI382201[] = {{&S77737[36], 30}, {NULL}};
const Segment SGI382202[] = {{&S77795[7], 9}, {NULL}};
const Segment SGI382203[] = {{&S77795[36], 3}, {NULL}};
const Segment SGI382204[] = {{&S63219[11], 5}, {NULL}};
const Segment SGI382205[] = {{&S63229[11], 5}, {NULL}};
const Segment SGI382206[] = {{&S63229[41], 3}, {NULL}};
const Segment SGI382207[] = {{&S77946[51], 9}, {NULL}};
const Segment SGI382208[] = {{&S63371[11], 5}, {NULL}};
const Segment SGI382209[] = {{&S63371[48], 3}, {NULL}};
const Segment SGI382210[] = {{&S78082[36], 22}, {NULL}};
const Segment SGI382211[] = {{&S78082[86], 13}, {NULL}};
const Segment SGI382212[] = {{&S78132[36], 22}, {NULL}};
const Segment SGI382213[] = {{&S78132[86], 13}, {NULL}};
const Segment SGI382214[] = {{&S78155[7], 6}, {NULL}};
const Segment SGI382215[] = {{&S78155[34], 3}, {NULL}};
const Segment SGI382216[] = {{&S63524[11], 5}, {NULL}};
const Segment SGI382217[] = {{&S63524[47], 3}, {NULL}};
const Segment SGI382218[] = {{&S63525[11], 6}, {NULL}};
const Segment SGI382219[] = {{&S63525[18], 3}, {NULL}};
const Segment SGI382220[] = {{&S78216[7], 6}, {NULL}};
const Segment SGI382221[] = {{&S78216[69], 3}, {NULL}};
const Segment SGI382222[] = {{&S78217[7], 13}, {NULL}};
const Segment SGI382223[] = {{&S78217[83], 3}, {NULL}};
const Segment SGI382224[] = {{&S78222[7], 9}, {NULL}};
const Segment SGI382225[] = {{&S78269[7], 9}, {NULL}};
const Segment SGI382226[] = {{&S78269[47], 3}, {NULL}};
const Segment SGI382227[] = {{&S78348[36], 19}, {NULL}};
const Segment SGI382228[] = {{&S78348[83], 13}, {NULL}};
const Segment SGI382229[] = {{&S78378[7], 6}, {NULL}};
const Segment SGI382230[] = {{&S78378[34], 3}, {NULL}};
const Segment SGI382231[] = {{&S78393[36], 22}, {NULL}};
const Segment SGI382232[] = {{&S78393[86], 13}, {NULL}};
const Segment SGI382233[] = {{&S78459[36], 22}, {NULL}};
const Segment SGI382234[] = {{&S78459[86], 13}, {NULL}};
const Segment SGI382235[] = {{&S63637[11], 6}, {NULL}};
const Segment SGI382236[] = {{&S63637[31], 3}, {NULL}};
const Segment SGI382237[] = {{&S63638[11], 5}, {NULL}};
const Segment SGI382238[] = {{&S63638[79], 3}, {NULL}};
const Segment SGI382239[] = {{&S63639[11], 6}, {NULL}};
const Segment SGI382240[] = {{&S78523[7], 14}, {NULL}};
const Segment SGI382241[] = {{&S78523[77], 3}, {NULL}};
const Segment SGI382242[] = {{&S78524[7], 9}, {NULL}};
const Segment SGI382243[] = {{&S78524[72], 3}, {NULL}};
const Segment SGI382244[] = {{&S78535[7], 9}, {NULL}};
const Segment SGI382245[] = {{&S78570[7], 9}, {NULL}};
const Segment SGI382246[] = {{&S78650[44], 13}, {NULL}};
const Segment SGI382247[] = {{&S78650[64], 13}, {NULL}};
const Segment SGI382248[] = {{&S78671[36], 29}, {NULL}};
const Segment SGI382249[] = {{&S78672[36], 29}, {NULL}};
const Segment SGI382250[] = {{&S63967[11], 5}, {NULL}};
const Segment SGI382251[] = {{&S63967[30], 3}, {NULL}};
const Segment SGI382252[] = {{&S78802[7], 7}, {NULL}};
const Segment SGI382253[] = {{&S78802[30], 3}, {NULL}};
const Segment SGI382254[] = {{&S78875[36], 29}, {NULL}};
const Segment SGI382255[] = {{&S78876[36], 29}, {NULL}};
const Segment SGI382256[] = {{&S64159[11], 5}, {&S64159[98], 6}, {NULL}};
const Segment SGI382257[] = {{&S64159[39], 3}, {NULL}};
const Segment SGI382258[] = {{&S64160[11], 6}, {NULL}};
const Segment SGI382259[] = {{&S64160[40], 3}, {NULL}};
const Segment SGI382260[] = {{&S64161[11], 6}, {NULL}};
const Segment SGI382261[] = {{&S64161[40], 3}, {NULL}};
const Segment SGI382262[] = {{&S64162[11], 11}, {NULL}};
const Segment SGI382263[] = {{&S64162[47], 3}, {NULL}};
const Segment SGI382264[] = {{&S64163[11], 5}, {NULL}};
const Segment SGI382265[] = {{&S64163[39], 3}, {NULL}};
const Segment SGI382266[] = {{&S78920[69], 20}, {NULL}};
const Segment SGI382267[] = {{&S78920[73], 16}, {NULL}};
const Segment SGI382268[] = {{&S78921[69], 17}, {NULL}};
const Segment SGI382269[] = {{&S78922[69], 24}, {NULL}};
const Segment SGI382270[] = {{&S78922[73], 20}, {NULL}};
const Segment SGI382271[] = {{&S79017[7], 9}, {NULL}};
const Segment SGI382272[] = {{&S79017[36], 3}, {NULL}};
const Segment SGI382273[] = {{&S79078[3], 16}, {NULL}};
const Segment SGI382274[] = {{&S79078[20], 3}, {NULL}};
const Segment SGI382275[] = {{&S79123[7], 9}, {NULL}};
const Segment SGI382276[] = {{&S79123[43], 3}, {NULL}};
const Segment SGI382277[] = {{&S79176[7], 15}, {NULL}};
const Segment SGI382278[] = {{&S79177[69], 9}, {NULL}};
const Segment SGI382279[] = {{&S79178[7], 9}, {NULL}};
const Segment SGI382280[] = {{&S79179[69], 10}, {NULL}};
const Segment SGI382281[] = {{&S79191[36], 29}, {NULL}};
const Segment SGI382282[] = {{&S79192[36], 29}, {NULL}};
const Segment SGI382283[] = {{&S64575[11], 5}, {NULL}};
const Segment SGI382284[] = {{&S64575[30], 3}, {NULL}};
const Segment SGI382285[] = {{&S79407[7], 9}, {NULL}};
const Segment SGI382286[] = {{&S79407[41], 3}, {NULL}};
const Segment SGI382287[] = {{&S79467[7], 9}, {NULL}};
const Segment SGI382288[] = {{&S79471[7], 9}, {NULL}};
const Segment SGI382289[] = {{&S79471[120], 4}, {NULL}};
const Segment SGI382290[] = {{&S64828[11], 5}, {NULL}};
const Segment SGI382291[] = {{&S64828[30], 3}, {NULL}};
const Segment SGI382292[] = {{&S64829[11], 6}, {NULL}};
const Segment SGI382293[] = {{&S64829[31], 3}, {NULL}};
const Segment SGI382294[] = {{&S79629[7], 9}, {NULL}};
const Segment SGI382295[] = {{&S79680[33], 21}, {NULL}};
const Segment SGI382296[] = {{&S79681[33], 21}, {NULL}};
const Segment SGI382297[] = {{&S64955[11], 5}, {NULL}};
const Segment SGI382298[] = {{&S64955[30], 3}, {NULL}};
const Segment SGI382299[] = {{&S80033[7], 9}, {NULL}};
const Segment SGI382300[] = {{&S80033[41], 3}, {NULL}};
const Segment SGI382301[] = {{&S80072[36], 22}, {NULL}};
const Segment SGI382302[] = {{&S80072[86], 13}, {NULL}};
const Segment SGI382303[] = {{&S65420[33], 5}, {NULL}};
const Segment SGI382304[] = {{&S65420[66], 3}, {NULL}};
const Segment SGI382305[] = {{&S65420[123], 3}, {NULL}};
const Segment SGI382306[] = {{&S65421[33], 6}, {NULL}};
const Segment SGI382307[] = {{&S65421[61], 3}, {NULL}};
const Segment SGI382308[] = {{&S65421[118], 3}, {NULL}};
const Segment SGI382309[] = {{&S65463[11], 6}, {NULL}};
const Segment SGI382310[] = {{&S65463[18], 3}, {NULL}};
const Segment SGI382311[] = {{&S80336[69], 9}, {NULL}};
const Segment SGI382312[] = {{&S80337[7], 12}, {NULL}};
const Segment SGI382313[] = {{&S80338[7], 15}, {NULL}};
const Segment SGI382314[] = {{&S80339[7], 9}, {NULL}};
const Segment SGI382315[] = {{&S80340[7], 24}, {NULL}};
const Segment SGI382316[] = {{&S80340[32], 5}, {NULL}};
const Segment SGI382317[] = {{&S80341[7], 20}, {NULL}};
const Segment SGI382318[] = {{&S80390[3], 6}, {NULL}};
const Segment SGI382319[] = {{&S80390[23], 11}, {NULL}};
const Segment SGI382320[] = {{&S80418[43], 13}, {NULL}};
const Segment SGI382321[] = {{&S80418[63], 13}, {NULL}};
const Segment SGI382322[] = {{&S65601[11], 5}, {NULL}};
const Segment SGI382323[] = {{&S65601[17], 3}, {NULL}};
const Segment SGI382324[] = {{&S80503[7], 15}, {NULL}};
const Segment SGI382325[] = {{&S80503[76], 4}, {NULL}};
const Segment SGI382326[] = {{&S80504[7], 9}, {NULL}};
const Segment SGI382327[] = {{&S80504[67], 4}, {NULL}};
const Segment SGI382328[] = {{&S80505[7], 10}, {NULL}};
const Segment SGI382329[] = {{&S80505[70], 4}, {NULL}};
const Segment SGI382330[] = {{&S80508[7], 18}, {NULL}};
const Segment SGI382331[] = {{&S80508[50], 3}, {NULL}};
const Segment SGI382332[] = {{&S80509[7], 9}, {NULL}};
const Segment SGI382333[] = {{&S80509[41], 3}, {NULL}};
const Segment SGI382334[] = {{&S65739[11], 6}, {NULL}};
const Segment SGI382335[] = {{&S65739[48], 3}, {NULL}};
const Segment SGI382336[] = {{&S80574[7], 9}, {NULL}};
const Segment SGI382337[] = {{&S80574[72], 3}, {NULL}};
const Segment SGI382338[] = {{&S65749[11], 5}, {NULL}};
const Segment SGI382339[] = {{&S65749[30], 3}, {NULL}};
const Segment SGI382340[] = {{&S80673[44], 13}, {NULL}};
const Segment SGI382341[] = {{&S80673[64], 13}, {NULL}};
const Segment SGI382342[] = {{&S80729[7], 9}, {NULL}};
const Segment SGI382343[] = {{&S80729[51], 3}, {NULL}};
const Segment SGI382344[] = {{&S80877[36], 29}, {NULL}};
const Segment SGI382345[] = {{&S80878[36], 29}, {NULL}};
const Segment SGI382346[] = {{&S80937[3], 8}, {NULL}};
const Segment SGI382347[] = {{&S80937[25], 3}, {NULL}};
const Segment SGI382348[] = {{&S81017[44], 13}, {NULL}};
const Segment SGI382349[] = {{&S81017[64], 13}, {NULL}};
const Segment SGI382350[] = {{&S66216[11], 6}, {NULL}};
const Segment SGI382351[] = {{&S66216[31], 3}, {NULL}};
const Segment SGI382352[] = {{&S81118[7], 9}, {NULL}};
const Segment SGI382353[] = {{&S81118[72], 3}, {NULL}};
const Segment SGI382354[] = {{&S81119[7], 10}, {NULL}};
const Segment SGI382355[] = {{&S81119[73], 3}, {NULL}};
const Segment SGI382356[] = {{&S66308[11], 16}, {NULL}};
const Segment SGI382357[] = {{&S66308[24], 3}, {NULL}};
const Segment SGI382358[] = {{&S66309[11], 5}, {NULL}};
const Segment SGI382359[] = {{&S66309[27], 3}, {NULL}};
const Segment SGI382360[] = {{&S81328[36], 29}, {NULL}};
const Segment SGI382361[] = {{&S81373[33], 21}, {NULL}};
const Segment SGI382362[] = {{&S81440[7], 18}, {NULL}};
const Segment SGI382363[] = {{&S81440[53], 3}, {NULL}};
const Segment SGI382364[] = {{&S81444[7], 10}, {NULL}};
const Segment SGI382365[] = {{&S81444[33], 3}, {NULL}};
const Segment SGI382366[] = {{&S81445[7], 9}, {NULL}};
const Segment SGI382367[] = {{&S81445[32], 3}, {NULL}};
const Segment SGI382368[] = {{&S81530[43], 13}, {NULL}};
const Segment SGI382369[] = {{&S81530[63], 13}, {NULL}};
const Segment SGI382370[] = {{&S81587[72], 17}, {NULL}};
const Segment SGI382371[] = {{&S81587[6], 3}, {NULL}};
const Segment SGI382372[] = {{&S59799[52], 12}, {NULL}};
const Segment SGI382373[] = {{&S59799[8], 3}, {NULL}};
const Segment SGI382374[] = {{&S59799[23], 3}, {NULL}};
const Segment SGI382375[] = {{&S59799[13], 102}, {&S59799[134], 17}, {NULL}};
const Segment SGI382376[] = {{&S59800[52], 12}, {NULL}};
const Segment SGI382377[] = {{&S59800[8], 3}, {NULL}};
const Segment SGI382378[] = {{&S59800[23], 3}, {NULL}};
const Segment SGI382379[] = {{&S59800[13], 102}, {&S59800[134], 17}, {NULL}};
const Segment SGI382380[] = {{&S59801[49], 15}, {NULL}};
const Segment SGI382381[] = {{&S59801[8], 3}, {NULL}};
const Segment SGI382382[] = {{&S59801[23], 3}, {NULL}};
const Segment SGI382383[] = {{&S59801[13], 84}, {&S59801[116], 17}, {NULL}};
const Segment SGI382384[] = {{&S59802[52], 12}, {NULL}};
const Segment SGI382385[] = {{&S59802[8], 3}, {NULL}};
const Segment SGI382386[] = {{&S59802[23], 3}, {NULL}};
const Segment SGI382387[] = {{&S59802[13], 102}, {&S59802[134], 17}, {NULL}};
const Segment SGI382388[] = {{&S59803[52], 12}, {NULL}};
const Segment SGI382389[] = {{&S59803[8], 3}, {NULL}};
const Segment SGI382390[] = {{&S59803[23], 3}, {NULL}};
const Segment SGI382391[] = {{&S59803[13], 102}, {&S59803[134], 17}, {NULL}};
const Segment SGI382392[] = {{&S59804[52], 12}, {NULL}};
const Segment SGI382393[] = {{&S59804[8], 3}, {NULL}};
const Segment SGI382394[] = {{&S59804[23], 3}, {NULL}};
const Segment SGI382395[] = {{&S59804[13], 102}, {&S59804[134], 17}, {NULL}};
const Segment SGI382396[] = {{&S59805[52], 14}, {NULL}};
const Segment SGI382397[] = {{&S59805[8], 3}, {NULL}};
const Segment SGI382398[] = {{&S59805[23], 3}, {NULL}};
const Segment SGI382399[] = {{&S59805[13], 86}, {&S59805[118], 17}, {NULL}};
const Segment SGI382400[] = {{&S66917[11], 5}, {NULL}};
const Segment SGI382401[] = {{&S81734[85], 17}, {NULL}};
const Segment SGI382402[] = {{&S81780[7], 6}, {NULL}};
const Segment SGI382403[] = {{&S81780[38], 3}, {NULL}};
const Segment SGI382404[] = {{&S81782[7], 9}, {NULL}};
const Segment SGI382405[] = {{&S81782[41], 3}, {NULL}};
const Segment SGI382406[] = {{&S81783[3], 5}, {NULL}};
const Segment SGI382407[] = {{&S81784[7], 6}, {NULL}};
const Segment SGI382408[] = {{&S81784[27], 3}, {NULL}};
const Segment SGI382409[] = {{&S81785[7], 6}, {NULL}};
const Segment SGI382410[] = {{&S81847[7], 9}, {NULL}};
const Segment SGI382411[] = {{&S81847[47], 3}, {NULL}};
const Segment SGI382412[] = {{&S81886[51], 20}, {NULL}};
const Segment SGI382413[] = {{&S81887[51], 20}, {NULL}};
const Segment SGI382414[] = {{&S81944[7], 9}, {NULL}};
const Segment SGI382415[] = {{&S81944[78], 3}, {NULL}};
const Segment SGI382416[] = {{&S82195[3], 16}, {NULL}};
const Segment SGI382417[] = {{&S82195[20], 3}, {NULL}};
const Segment SGI382418[] = {{&S82204[36], 29}, {NULL}};
const Segment SGI382419[] = {{&S82344[7], 10}, {NULL}};
const Segment SGI382420[] = {{&S82344[42], 3}, {NULL}};
const Segment SGI382421[] = {{&S82345[7], 10}, {NULL}};
const Segment SGI382422[] = {{&S82345[49], 3}, {NULL}};
const Segment SGI382423[] = {{&S82416[7], 9}, {NULL}};
const Segment SGI382424[] = {{&S82417[7], 9}, {NULL}};
const Segment SGI382425[] = {{&S82418[7], 9}, {NULL}};
const Segment SGI382426[] = {{&S82420[130], 14}, {NULL}};
const Segment SGI382427[] = {{&S67734[11], 5}, {NULL}};
const Segment SGI382428[] = {{&S82624[43], 14}, {NULL}};
const Segment SGI382429[] = {{&S82624[64], 13}, {NULL}};
const Segment SGI382430[] = {{&S82688[36], 29}, {NULL}};
const Segment SGI382431[] = {{&S82696[43], 13}, {NULL}};
const Segment SGI382432[] = {{&S82696[63], 13}, {NULL}};
const Segment SGI382433[] = {{&S82732[7], 9}, {NULL}};
const Segment SGI382434[] = {{&S82732[78], 3}, {NULL}};
const Segment SGI382435[] = {{&S82756[36], 29}, {NULL}};
const Segment SGI382436[] = {{&S82766[36], 22}, {NULL}};
const Segment SGI382437[] = {{&S82766[86], 13}, {NULL}};
const Segment SGI382438[] = {{&S82816[7], 7}, {NULL}};
const Segment SGI382439[] = {{&S82816[38], 3}, {NULL}};
const Segment SGI382440[] = {{&S82817[7], 7}, {NULL}};
const Segment SGI382441[] = {{&S82817[38], 3}, {NULL}};
const Segment SGI382442[] = {{&S82818[20], 7}, {NULL}};
const Segment SGI382443[] = {{&S82818[49], 3}, {NULL}};
const Segment SGI382444[] = {{&S82819[20], 7}, {NULL}};
const Segment SGI382445[] = {{&S82819[49], 3}, {NULL}};
const Segment SGI382446[] = {{&S82878[3], 15}, {NULL}};
const Segment SGI382447[] = {{&S82878[32], 3}, {NULL}};
const Segment SGI382448[] = {{&S82879[3], 8}, {NULL}};
const Segment SGI382449[] = {{&S82879[25], 3}, {NULL}};
const Segment SGI382450[] = {{&S67926[55], 6}, {NULL}};
const Segment SGI382451[] = {{&S68017[11], 5}, {NULL}};
const Segment SGI382452[] = {{&S68017[30], 3}, {NULL}};
const Segment SGI382453[] = {{&S68018[11], 5}, {NULL}};
const Segment SGI382454[] = {{&S82954[7], 9}, {NULL}};
const Segment SGI382455[] = {{&S82954[47], 3}, {NULL}};
const Segment SGI382456[] = {{&S83056[43], 13}, {NULL}};
const Segment SGI382457[] = {{&S83056[63], 13}, {NULL}};
const Segment SGI382458[] = {{&S83088[44], 14}, {NULL}};
const Segment SGI382459[] = {{&S83088[65], 13}, {NULL}};
const Segment SGI382460[] = {{&S83149[40], 13}, {NULL}};
const Segment SGI382461[] = {{&S83149[60], 13}, {NULL}};
const Segment SGI382462[] = {{&S83199[7], 13}, {NULL}};
const Segment SGI382463[] = {{&S83199[51], 3}, {NULL}};
const Segment SGI382464[] = {{&S83200[7], 9}, {NULL}};
const Segment SGI382465[] = {{&S83200[41], 3}, {NULL}};
const Segment SGI382466[] = {{&S83203[7], 9}, {NULL}};
const Segment SGI382467[] = {{&S83203[78], 3}, {NULL}};
const Segment SGI382468[] = {{&S68502[11], 5}, {NULL}};
const Segment SGI382469[] = {{&S68502[30], 3}, {NULL}};
const Segment SGI382470[] = {{&S68532[11], 6}, {NULL}};
const Segment SGI382471[] = {{&S68532[92], 3}, {NULL}};
const Segment SGI382472[] = {{&S68533[11], 5}, {&S68533[11], 5}, {NULL}};
const Segment SGI382473[] = {{&S68533[69], 3}, {NULL}};
const Segment SGI382474[] = {{&S68534[11], 5}, {NULL}};
const Segment SGI382475[] = {{&S68534[69], 3}, {NULL}};
const Segment SGI382476[] = {{&S83298[3], 8}, {NULL}};
const Segment SGI382477[] = {{&S83298[25], 3}, {NULL}};
const Segment SGI382478[] = {{&S83299[3], 16}, {NULL}};
const Segment SGI382479[] = {{&S83299[33], 3}, {NULL}};
const Segment SGI382480[] = {{&S83300[3], 9}, {NULL}};
const Segment SGI382481[] = {{&S83300[26], 3}, {NULL}};
const Segment SGI382482[] = {{&S83302[51], 18}, {NULL}};
const Segment SGI382483[] = {{&S83303[51], 18}, {NULL}};
const Segment SGI382484[] = {{&S83304[51], 18}, {NULL}};
const Segment SGI382485[] = {{&S83305[51], 18}, {NULL}};
const Segment SGI382486[] = {{&S83309[51], 20}, {NULL}};
const Segment SGI382487[] = {{&S83310[51], 20}, {NULL}};
const Segment SGI382488[] = {{&S83321[43], 13}, {NULL}};
const Segment SGI382489[] = {{&S83321[63], 13}, {NULL}};
const Segment SGI382490[] = {{&S83345[3], 5}, {NULL}};
const Segment SGI382491[] = {{&S83424[43], 13}, {NULL}};
const Segment SGI382492[] = {{&S83424[63], 13}, {NULL}};
const Segment SGI382493[] = {{&S83464[7], 15}, {NULL}};
const Segment SGI382494[] = {{&S83464[23], 4}, {NULL}};
const Segment SGI382495[] = {{&S83464[51], 3}, {NULL}};
const Segment SGI382496[] = {{&S83564[7], 9}, {NULL}};
const Segment SGI382497[] = {{&S83564[41], 3}, {NULL}};
const Segment SGI382498[] = {{&S83565[3], 9}, {NULL}};
const Segment SGI382499[] = {{&S83565[26], 3}, {NULL}};
const Segment SGI382500[] = {{&S83566[3], 8}, {NULL}};
const Segment SGI382501[] = {{&S83566[25], 3}, {NULL}};
const Segment SGI382502[] = {{&S83567[7], 16}, {NULL}};
const Segment SGI382503[] = {{&S83567[54], 3}, {NULL}};
const Segment SGI382504[] = {{&S83568[7], 9}, {NULL}};
const Segment SGI382505[] = {{&S83568[47], 3}, {NULL}};
const Segment SGI382506[] = {{&S83569[42], 19}, {NULL}};
const Segment SGI382507[] = {{&S83570[42], 19}, {NULL}};
const Segment SGI382508[] = {{&S83571[7], 9}, {NULL}};
const Segment SGI382509[] = {{&S83571[78], 3}, {NULL}};
const Segment SGI382510[] = {{&S83601[43], 13}, {NULL}};
const Segment SGI382511[] = {{&S83601[63], 13}, {NULL}};
const Segment SGI382512[] = {{&S83667[7], 9}, {NULL}};
const Segment SGI382513[] = {{&S83667[36], 3}, {NULL}};
const Segment SGI382514[] = {{&S83694[44], 14}, {NULL}};
const Segment SGI382515[] = {{&S83694[65], 13}, {NULL}};
const Segment SGI382516[] = {{&S83695[44], 14}, {NULL}};
const Segment SGI382517[] = {{&S83695[65], 13}, {NULL}};
const Segment SGI382518[] = {{&S83712[44], 13}, {NULL}};
const Segment SGI382519[] = {{&S83712[64], 13}, {NULL}};
const Segment SGI382520[] = {{&S83733[44], 13}, {NULL}};
const Segment SGI382521[] = {{&S83733[64], 13}, {NULL}};
const Segment SGI382522[] = {{&S83812[7], 9}, {NULL}};
const Segment SGI382523[] = {{&S83813[7], 16}, {NULL}};
const Segment SGI382524[] = {{&S69163[11], 5}, {NULL}};
const Segment SGI382525[] = {{&S69163[94], 3}, {NULL}};
const Segment SGI382526[] = {{&S83884[44], 13}, {NULL}};
const Segment SGI382527[] = {{&S83884[64], 13}, {NULL}};
const Segment SGI382528[] = {{&S83923[7], 6}, {NULL}};
const Segment SGI382529[] = {{&S83980[37], 25}, {NULL}};
const Segment SGI382530[] = {{&S69401[11], 5}, {NULL}};
const Segment SGI382531[] = {{&S69401[37], 3}, {NULL}};
const Segment SGI382532[] = {{&S84004[36], 22}, {NULL}};
const Segment SGI382533[] = {{&S84004[86], 13}, {NULL}};
const Segment SGI382534[] = {{&S84076[7], 9}, {NULL}};
const Segment SGI382535[] = {{&S84076[72], 3}, {NULL}};
const Segment SGI382536[] = {{&S84116[36], 23}, {NULL}};
const Segment SGI382537[] = {{&S84116[87], 13}, {NULL}};
const Segment SGI382538[] = {{&S69607[11], 6}, {NULL}};
const Segment SGI382539[] = {{&S69609[11], 5}, {NULL}};
const Segment SGI382540[] = {{&S84489[7], 9}, {NULL}};
const Segment SGI382541[] = {{&S69825[11], 6}, {NULL}};
const Segment SGI382542[] = {{&S69825[41], 3}, {NULL}};
const Segment SGI382543[] = {{&S84547[44], 13}, {NULL}};
const Segment SGI382544[] = {{&S84547[64], 13}, {NULL}};
const Segment SGI382545[] = {{&S84579[3], 8}, {NULL}};
const Segment SGI382546[] = {{&S84579[25], 3}, {NULL}};
const Segment SGI382547[] = {{&S84583[3], 5}, {NULL}};
const Segment SGI382548[] = {{&S84583[22], 11}, {NULL}};
const Segment SGI382549[] = {{&S84638[44], 13}, {NULL}};
const Segment SGI382550[] = {{&S84638[64], 13}, {NULL}};
const Segment SGI382551[] = {{&S84665[7], 9}, {NULL}};
const Segment SGI382552[] = {{&S84693[36], 29}, {NULL}};
const Segment SGI382553[] = {{&S84694[36], 29}, {NULL}};
const Segment SGI382554[] = {{&S84770[3], 16}, {NULL}};
const Segment SGI382555[] = {{&S84770[20], 3}, {NULL}};
const Segment SGI382556[] = {{&S84960[7], 9}, {NULL}};
const Segment SGI382557[] = {{&S85028[7], 15}, {NULL}};
const Segment SGI382558[] = {{&S85028[23], 4}, {NULL}};
const Segment SGI382559[] = {{&S85028[51], 3}, {NULL}};
const Segment SGI382560[] = {{&S70255[11], 6}, {NULL}};
const Segment SGI382561[] = {{&S70255[40], 3}, {NULL}};
const Segment SGI382562[] = {{&S70257[11], 6}, {NULL}};
const Segment SGI382563[] = {{&S85133[3], 8}, {NULL}};
const Segment SGI382564[] = {{&S85133[25], 3}, {NULL}};
const Segment SGI382565[] = {{&S85134[3], 9}, {NULL}};
const Segment SGI382566[] = {{&S85134[26], 3}, {NULL}};
const Segment SGI382567[] = {{&S85144[7], 9}, {NULL}};
const Segment SGI382568[] = {{&S85144[41], 3}, {NULL}};
const Segment SGI382569[] = {{&S85154[36], 29}, {NULL}};
const Segment SGI382570[] = {{&S70350[11], 5}, {NULL}};
const Segment SGI382571[] = {{&S70351[11], 6}, {NULL}};
const Segment SGI382572[] = {{&S70351[31], 3}, {NULL}};
const Segment SGI382573[] = {{&S70362[11], 5}, {NULL}};
const Segment SGI382574[] = {{&S70362[79], 3}, {NULL}};
const Segment SGI382575[] = {{&S46906[129], 3}, {NULL}};
const Segment SGI382576[] = {{&S46907[130], 3}, {NULL}};
const Segment SGI382577[] = {{&S46908[129], 3}, {NULL}};
const Segment SGI382578[] = {{&S46909[130], 3}, {NULL}};
const Segment SGI382579[] = {{&S46909[69], 6}, {NULL}};
const Segment SGI382580[] = {{&S70461[11], 5}, {NULL}};
const Segment SGI382581[] = {{&S70461[30], 3}, {NULL}};
const Segment SGI382582[] = {{&S70664[11], 5}, {NULL}};
const Segment SGI382583[] = {{&S70690[11], 11}, {NULL}};
const Segment SGI382584[] = {{&S70690[67], 3}, {NULL}};
const Segment SGI382585[] = {{&S70691[11], 5}, {NULL}};
const Segment SGI382586[] = {{&S70691[59], 3}, {NULL}};
const Segment SGI382587[] = {{&S70845[11], 6}, {NULL}};
const Segment SGI382588[] = {{&S70845[42], 3}, {NULL}};
const Segment SGI382589[] = {{&S70846[11], 5}, {NULL}};
const Segment SGI382590[] = {{&S70846[41], 3}, {NULL}};
const Segment SGI382591[] = {{&S85465[36], 22}, {NULL}};
const Segment SGI382592[] = {{&S85465[86], 13}, {NULL}};
const Segment SGI382593[] = {{&S85541[7], 6}, {NULL}};
const Segment SGI382594[] = {{&S85541[45], 3}, {NULL}};
const Segment SGI382595[] = {{&S85552[7], 9}, {NULL}};
const Segment SGI382596[] = {{&S85552[78], 3}, {NULL}};
const Segment SGI382597[] = {{&S85553[3], 16}, {NULL}};
const Segment SGI382598[] = {{&S85553[20], 3}, {NULL}};
const Segment SGI382599[] = {{&S85634[44], 13}, {NULL}};
const Segment SGI382600[] = {{&S85634[64], 13}, {NULL}};
const Segment SGI382601[] = {{&S85641[7], 9}, {NULL}};
const Segment SGI382602[] = {{&S85641[41], 3}, {NULL}};
const Segment SGI382603[] = {{&S85646[7], 9}, {NULL}};
const Segment SGI382604[] = {{&S85646[41], 3}, {NULL}};
const Segment SGI382605[] = {{&S71211[11], 5}, {NULL}};
const Segment SGI382606[] = {{&S71211[41], 3}, {NULL}};
const Segment SGI382607[] = {{&S71212[11], 6}, {NULL}};
const Segment SGI382608[] = {{&S71212[42], 3}, {NULL}};
const Segment SGI382609[] = {{&S71213[11], 6}, {NULL}};
const Segment SGI382610[] = {{&S71213[42], 3}, {NULL}};
const Segment SGI382611[] = {{&S71234[33], 6}, {NULL}};
const Segment SGI382612[] = {{&S71234[60], 3}, {NULL}};
const Segment SGI382613[] = {{&S71234[118], 3}, {NULL}};
const Segment SGI382614[] = {{&S85731[7], 9}, {NULL}};
const Segment SGI382615[] = {{&S85731[11], 5}, {NULL}};
const Segment SGI382616[] = {{&S85760[7], 9}, {NULL}};
const Segment SGI382617[] = {{&S85760[38], 3}, {NULL}};
const Segment SGI382618[] = {{&S85762[7], 9}, {NULL}};
const Segment SGI382619[] = {{&S85762[72], 3}, {NULL}};
const Segment SGI382620[] = {{&S85765[3], 8}, {NULL}};
const Segment SGI382621[] = {{&S85765[25], 3}, {NULL}};
const Segment SGI382622[] = {{&S85766[3], 8}, {NULL}};
const Segment SGI382623[] = {{&S85766[25], 3}, {NULL}};
const Segment SGI382624[] = {{&S85767[3], 8}, {NULL}};
const Segment SGI382625[] = {{&S85767[25], 3}, {NULL}};
const Segment SGI382626[] = {{&S85768[3], 8}, {NULL}};
const Segment SGI382627[] = {{&S85768[25], 3}, {NULL}};
const Segment SGI382628[] = {{&S85769[69], 17}, {NULL}};
const Segment SGI382629[] = {{&S85894[3], 8}, {NULL}};
const Segment SGI382630[] = {{&S85894[25], 3}, {NULL}};
const Segment SGI382631[] = {{&S85952[43], 13}, {NULL}};
const Segment SGI382632[] = {{&S85952[63], 13}, {NULL}};
const Segment SGI382633[] = {{&S85960[44], 13}, {NULL}};
const Segment SGI382634[] = {{&S85960[64], 13}, {NULL}};
const Segment SGI382635[] = {{&S86003[44], 13}, {NULL}};
const Segment SGI382636[] = {{&S86003[64], 13}, {NULL}};
const Segment SGI382637[] = {{&S86017[7], 9}, {NULL}};
const Segment SGI382638[] = {{&S86017[41], 3}, {NULL}};
const Segment SGI382639[] = {{&S86018[7], 18}, {NULL}};
const Segment SGI382640[] = {{&S86018[56], 3}, {NULL}};
const Segment SGI382641[] = {{&S86019[7], 16}, {NULL}};
const Segment SGI382642[] = {{&S86019[50], 3}, {NULL}};
const Segment SGI382643[] = {{&S86021[7], 9}, {NULL}};
const Segment SGI382644[] = {{&S86021[78], 3}, {NULL}};
const Segment SGI382645[] = {{&S86126[36], 30}, {NULL}};
const Segment SGI382646[] = {{&S86127[36], 30}, {NULL}};
const Segment SGI382647[] = {{&S86168[44], 13}, {NULL}};
const Segment SGI382648[] = {{&S86168[64], 13}, {NULL}};
const Segment SGI382649[] = {{&S86186[44], 13}, {NULL}};
const Segment SGI382650[] = {{&S86186[64], 13}, {NULL}};
const Segment SGI382651[] = {{&S86241[36], 29}, {NULL}};
const Segment SGI382652[] = {{&S86259[43], 13}, {NULL}};
const Segment SGI382653[] = {{&S86259[63], 13}, {NULL}};
const Segment SGI382654[] = {{&S71707[11], 6}, {NULL}};
const Segment SGI382655[] = {{&S71834[11], 12}, {NULL}};
const Segment SGI382656[] = {{&S71834[24], 4}, {NULL}};
const Segment SGI382657[] = {{&S71834[52], 3}, {NULL}};
const Segment SGI382658[] = {{&S71893[11], 6}, {NULL}};
const Segment SGI382659[] = {{&S71894[11], 5}, {NULL}};
const Segment SGI382660[] = {{&S86415[7], 9}, {NULL}};
const Segment SGI382661[] = {{&S86415[41], 3}, {NULL}};
const Segment SGI382662[] = {{&S86430[7], 9}, {NULL}};
const Segment SGI382663[] = {{&S86437[7], 9}, {NULL}};
const Segment SGI382664[] = {{&S86545[3], 8}, {NULL}};
const Segment SGI382665[] = {{&S86545[25], 3}, {NULL}};
const Segment SGI382666[] = {{&S86565[51], 20}, {NULL}};
const Segment SGI382667[] = {{&S72124[11], 5}, {NULL}};
const Segment SGI382668[] = {{&S72124[46], 3}, {NULL}};
const Segment SGI382669[] = {{&S86566[51], 20}, {NULL}};
const Segment SGI382670[] = {{&S86567[51], 20}, {NULL}};
const Segment SGI382671[] = {{&S86607[44], 13}, {NULL}};
const Segment SGI382672[] = {{&S86607[64], 13}, {NULL}};
const Segment SGI382673[] = {{&S86655[3], 5}, {NULL}};
const Segment SGI382674[] = {{&S86656[7], 9}, {NULL}};
const Segment SGI382675[] = {{&S86656[11], 5}, {NULL}};
const Segment SGI382676[] = {{&S86657[7], 9}, {NULL}};
const Segment SGI382677[] = {{&S86657[45], 3}, {NULL}};
const Segment SGI382678[] = {{&S72244[11], 5}, {NULL}};
const Segment SGI382679[] = {{&S72244[79], 3}, {NULL}};
const Segment SGI382680[] = {{&S86701[44], 13}, {NULL}};
const Segment SGI382681[] = {{&S86701[64], 13}, {NULL}};
const Segment SGI382682[] = {{&S86705[44], 10}, {NULL}};
const Segment SGI382683[] = {{&S86705[61], 13}, {NULL}};
const Segment SGI382684[] = {{&S86793[7], 9}, {NULL}};
const Segment SGI382685[] = {{&S86807[43], 14}, {NULL}};
const Segment SGI382686[] = {{&S86807[64], 13}, {NULL}};
const Segment SGI382687[] = {{&S86837[44], 13}, {NULL}};
const Segment SGI382688[] = {{&S86837[64], 13}, {NULL}};
const Segment SGI382689[] = {{&S86840[44], 13}, {NULL}};
const Segment SGI382690[] = {{&S86840[64], 13}, {NULL}};
const Segment SGI382691[] = {{&S86889[7], 9}, {NULL}};
const Segment SGI382692[] = {{&S86889[41], 3}, {NULL}};
const Segment SGI382693[] = {{&S86890[7], 10}, {NULL}};
const Segment SGI382694[] = {{&S86890[42], 3}, {NULL}};
const Segment SGI382695[] = {{&S86895[27], 18}, {NULL}};
const Segment SGI382696[] = {{&S86895[31], 14}, {NULL}};
const Segment SGI382697[] = {{&S86895[46], 3}, {NULL}};
const Segment SGI382698[] = {{&S86896[7], 9}, {NULL}};
const Segment SGI382699[] = {{&S86896[72], 3}, {NULL}};
const Segment SGI382700[] = {{&S86942[43], 13}, {NULL}};
const Segment SGI382701[] = {{&S86942[63], 13}, {NULL}};
const Segment SGI382702[] = {{&S86943[40], 14}, {NULL}};
const Segment SGI382703[] = {{&S86943[61], 13}, {NULL}};
const Segment SGI382704[] = {{&S86946[36], 22}, {NULL}};
const Segment SGI382705[] = {{&S86946[86], 13}, {NULL}};
const Segment SGI382706[] = {{&S87089[43], 14}, {NULL}};
const Segment SGI382707[] = {{&S87089[64], 13}, {NULL}};
const Segment SGI382708[] = {{&S87090[43], 14}, {NULL}};
const Segment SGI382709[] = {{&S87090[64], 13}, {NULL}};
const Segment SGI382710[] = {{&S72681[11], 11}, {NULL}};
const Segment SGI382711[] = {{&S72681[102], 3}, {NULL}};
const Segment SGI382712[] = {{&S72682[11], 5}, {NULL}};
const Segment SGI382713[] = {{&S72682[94], 3}, {NULL}};
const Segment SGI382714[] = {{&S72683[24], 5}, {NULL}};
const Segment SGI382715[] = {{&S72683[109], 3}, {NULL}};
const Segment SGI382716[] = {{&S87184[3], 5}, {NULL}};
const Segment SGI382717[] = {{&S72816[11], 5}, {NULL}};
const Segment SGI382718[] = {{&S72816[30], 3}, {NULL}};
const Segment SGI382719[] = {{&S87283[7], 9}, {NULL}};
const Segment SGI382720[] = {{&S87283[47], 3}, {NULL}};
const Segment SGI382721[] = {{&S87284[3], 16}, {NULL}};
const Segment SGI382722[] = {{&S87284[20], 3}, {NULL}};
const Segment SGI382723[] = {{&S87293[3], 8}, {NULL}};
const Segment SGI382724[] = {{&S87403[7], 9}, {NULL}};
const Segment SGI382725[] = {{&S87403[32], 3}, {NULL}};
const Segment SGI382726[] = {{&S87404[7], 10}, {NULL}};
const Segment SGI382727[] = {{&S87404[33], 3}, {NULL}};
const Segment SGI382728[] = {{&S87405[7], 18}, {NULL}};
const Segment SGI382729[] = {{&S87405[52], 3}, {NULL}};
const Segment SGI382730[] = {{&S87419[3], 5}, {NULL}};
const Segment SGI382731[] = {{&S87419[22], 11}, {NULL}};
const Segment SGI382732[] = {{&S87420[3], 8}, {NULL}};
const Segment SGI382733[] = {{&S87556[7], 9}, {NULL}};
const Segment SGI382734[] = {{&S87556[11], 5}, {NULL}};
const Segment SGI382735[] = {{&S87585[79], 9}, {NULL}};
const Segment SGI382736[] = {{&S87585[83], 5}, {NULL}};
const Segment SGI382737[] = {{&S87585[6], 4}, {NULL}};
const Segment SGI382738[] = {{&S87585[29], 3}, {NULL}};
const Segment SGI382739[] = {{&S73175[11], 5}, {NULL}};
const Segment SGI382740[] = {{&S73175[30], 3}, {NULL}};
const Segment SGI382741[] = {{&S87659[7], 9}, {NULL}};
const Segment SGI382742[] = {{&S87659[41], 3}, {NULL}};
const Segment SGI382743[] = {{&S87660[7], 9}, {NULL}};
const Segment SGI382744[] = {{&S87660[41], 3}, {NULL}};
const Segment SGI382745[] = {{&S87664[7], 9}, {NULL}};
const Segment SGI382746[] = {{&S87664[41], 3}, {NULL}};
const Segment SGI382747[] = {{&S87667[69], 17}, {NULL}};
const Segment SGI382748[] = {{&S87784[7], 9}, {NULL}};
const Segment SGI382749[] = {{&S87784[43], 3}, {NULL}};
const Segment SGI382750[] = {{&S87786[3], 9}, {NULL}};
const Segment SGI382751[] = {{&S87786[26], 3}, {NULL}};
const Segment SGI382752[] = {{&S87789[7], 9}, {NULL}};
const Segment SGI382753[] = {{&S87789[41], 3}, {NULL}};
const Segment SGI382754[] = {{&S87790[7], 6}, {NULL}};
const Segment SGI382755[] = {{&S87824[7], 5}, {NULL}};
const Segment SGI382756[] = {{&S87825[133], 14}, {NULL}};
const Segment SGI382757[] = {{&S73394[11], 5}, {NULL}};
const Segment SGI382758[] = {{&S73394[27], 3}, {NULL}};
const Segment SGI382759[] = {{&S73395[24], 5}, {NULL}};
const Segment SGI382760[] = {{&S73395[51], 3}, {NULL}};
const Segment SGI382761[] = {{&S73401[11], 5}, {NULL}};
const Segment SGI382762[] = {{&S73401[88], 4}, {NULL}};
const Segment SGI382763[] = {{&S73401[116], 3}, {NULL}};
const Segment SGI382764[] = {{&S87951[36], 23}, {NULL}};
const Segment SGI382765[] = {{&S87951[87], 13}, {NULL}};
const Segment SGI382766[] = {{&S88041[7], 9}, {NULL}};
const Segment SGI382767[] = {{&S88041[78], 3}, {NULL}};
const Segment SGI382768[] = {{&S88099[44], 14}, {NULL}};
const Segment SGI382769[] = {{&S88099[65], 13}, {NULL}};
const Segment SGI382770[] = {{&S73654[33], 6}, {NULL}};
const Segment SGI382771[] = {{&S73654[70], 3}, {NULL}};
const Segment SGI382772[] = {{&S73654[128], 3}, {NULL}};
const Segment SGI382773[] = {{&S88358[44], 13}, {NULL}};
const Segment SGI382774[] = {{&S88358[64], 13}, {NULL}};
const Segment SGI382775[] = {{&S73746[11], 6}, {NULL}};
const Segment SGI382776[] = {{&S73746[42], 3}, {NULL}};
const Segment SGI382777[] = {{&S73747[11], 5}, {NULL}};
const Segment SGI382778[] = {{&S73747[48], 3}, {NULL}};
const Segment SGI382779[] = {{&S88374[44], 13}, {NULL}};
const Segment SGI382780[] = {{&S88374[64], 13}, {NULL}};
const Segment SGI382781[] = {{&S88384[44], 13}, {NULL}};
const Segment SGI382782[] = {{&S88384[64], 13}, {NULL}};
const Segment SGI382783[] = {{&S88575[7], 9}, {NULL}};
const Segment SGI382784[] = {{&S88575[72], 3}, {NULL}};
const Segment SGI382785[] = {{&S88597[36], 30}, {NULL}};
const Segment SGI382786[] = {{&S88598[36], 30}, {NULL}};
const Segment SGI382787[] = {{&S88634[44], 13}, {NULL}};
const Segment SGI382788[] = {{&S88634[64], 13}, {NULL}};
const Segment SGI382789[] = {{&S88689[7], 18}, {NULL}};
const Segment SGI382790[] = {{&S88689[50], 3}, {NULL}};
const Segment SGI382791[] = {{&S88692[7], 9}, {NULL}};
const Segment SGI382792[] = {{&S88692[76], 3}, {NULL}};
const Segment SGI382793[] = {{&S88693[7], 6}, {NULL}};
const Segment SGI382794[] = {{&S88693[42], 3}, {NULL}};
const Segment SGI382795[] = {{&S88694[3], 16}, {NULL}};
const Segment SGI382796[] = {{&S88694[20], 3}, {NULL}};
const Segment SGI382797[] = {{&S88697[7], 9}, {NULL}};
const Segment SGI382798[] = {{&S88697[11], 5}, {NULL}};
const Segment SGI382799[] = {{&S73848[11], 6}, {NULL}};
const Segment SGI382800[] = {{&S73848[18], 3}, {NULL}};
const Segment SGI382801[] = {{&S73879[11], 5}, {NULL}};
const Segment SGI382802[] = {{&S73879[30], 3}, {NULL}};
const Segment SGI382803[] = {{&S88823[7], 9}, {NULL}};
const Segment SGI382804[] = {{&S88826[7], 9}, {NULL}};
const Segment SGI382805[] = {{&S88826[43], 3}, {NULL}};
const Segment SGI382806[] = {{&S88829[7], 16}, {NULL}};
const Segment SGI382807[] = {{&S73932[11], 5}, {NULL}};
const Segment SGI382808[] = {{&S73932[17], 3}, {NULL}};
const Segment SGI382809[] = {{&S88886[44], 13}, {NULL}};
const Segment SGI382810[] = {{&S88886[64], 13}, {NULL}};
const Segment SGI382811[] = {{&S88955[7], 10}, {NULL}};
const Segment SGI382812[] = {{&S88955[79], 3}, {NULL}};
const Segment SGI382813[] = {{&S88991[7], 10}, {NULL}};
const Segment SGI382814[] = {{&S89087[44], 13}, {NULL}};
const Segment SGI382815[] = {{&S89087[64], 13}, {NULL}};
const Segment SGI382816[] = {{&S89104[7], 15}, {NULL}};
const Segment SGI382817[] = {{&S89104[43], 3}, {NULL}};
const Segment SGI382818[] = {{&S89105[7], 6}, {NULL}};
const Segment SGI382819[] = {{&S89105[34], 3}, {NULL}};
const Segment SGI382820[] = {{&S89108[7], 16}, {NULL}};
const Segment SGI382821[] = {{&S2757[0], 25}, {NULL}};
const Segment SGI382822[] = {{&S74324[11], 5}, {NULL}};
const Segment SGI382823[] = {{&S74324[37], 3}, {NULL}};
const Segment SGI382824[] = {{&S89272[43], 14}, {NULL}};
const Segment SGI382825[] = {{&S89272[64], 13}, {NULL}};
const Segment SGI382826[] = {{&S89326[47], 14}, {NULL}};
const Segment SGI382827[] = {{&S89326[68], 13}, {NULL}};
const Segment SGI382828[] = {{&S89387[7], 9}, {NULL}};
const Segment SGI382829[] = {{&S89387[41], 3}, {NULL}};
const Segment SGI382830[] = {{&S89391[7], 22}, {NULL}};
const Segment SGI382831[] = {{&S89392[7], 20}, {NULL}};
const Segment SGI382832[] = {{&S89393[7], 15}, {NULL}};
const Segment SGI382833[] = {{&S89446[36], 30}, {NULL}};
const Segment SGI382834[] = {{&S89449[36], 22}, {NULL}};
const Segment SGI382835[] = {{&S89449[86], 13}, {NULL}};
const Segment SGI382836[] = {{&S89455[43], 13}, {NULL}};
const Segment SGI382837[] = {{&S89455[63], 13}, {NULL}};
const Segment SGI382838[] = {{&S89516[41], 13}, {NULL}};
const Segment SGI382839[] = {{&S89516[61], 13}, {NULL}};
const Segment SGI382840[] = {{&S89559[7], 9}, {NULL}};
const Segment SGI382841[] = {{&S89560[7], 9}, {NULL}};
const Segment SGI382842[] = {{&S67904[52], 5}, {NULL}};
const Segment SGI382843[] = {{&S67904[40], 3}, {NULL}};
const Segment SGI382844[] = {{&S67904[23], 3}, {NULL}};
const Segment SGI382845[] = {{&S67904[13], 92}, {&S67904[124], 17}, {NULL}};
const Segment SGI382846[] = {{&S67905[52], 5}, {NULL}};
const Segment SGI382847[] = {{&S67905[40], 3}, {NULL}};
const Segment SGI382848[] = {{&S67905[23], 3}, {NULL}};
const Segment SGI382849[] = {{&S67905[13], 92}, {&S67905[124], 17}, {NULL}};
const Segment SGI382850[] = {{&S67906[52], 5}, {NULL}};
const Segment SGI382851[] = {{&S67906[40], 3}, {NULL}};
const Segment SGI382852[] = {{&S67906[23], 3}, {NULL}};
const Segment SGI382853[] = {{&S67906[13], 92}, {&S67906[124], 17}, {NULL}};
const Segment SGI382854[] = {{&S67907[52], 5}, {NULL}};
const Segment SGI382855[] = {{&S67907[40], 3}, {NULL}};
const Segment SGI382856[] = {{&S67907[23], 3}, {NULL}};
const Segment SGI382857[] = {{&S67907[13], 92}, {&S67907[124], 17}, {NULL}};
const Segment SGI382858[] = {{&S67908[52], 5}, {NULL}};
const Segment SGI382859[] = {{&S67908[40], 3}, {NULL}};
const Segment SGI382860[] = {{&S67908[23], 3}, {NULL}};
const Segment SGI382861[] = {{&S67908[13], 92}, {&S67908[124], 17}, {NULL}};
const Segment SGI382862[] = {{&S67909[52], 5}, {NULL}};
const Segment SGI382863[] = {{&S67909[40], 3}, {NULL}};
const Segment SGI382864[] = {{&S67909[23], 3}, {NULL}};
const Segment SGI382865[] = {{&S67909[13], 92}, {&S67909[124], 17}, {NULL}};
const Segment SGI382866[] = {{&S67910[52], 14}, {NULL}};
const Segment SGI382867[] = {{&S67910[40], 3}, {NULL}};
const Segment SGI382868[] = {{&S67910[23], 3}, {NULL}};
const Segment SGI382869[] = {{&S67910[13], 86}, {&S67910[118], 17}, {NULL}};
const Segment SGI382870[] = {{&S67911[52], 5}, {NULL}};
const Segment SGI382871[] = {{&S67911[40], 3}, {NULL}};
const Segment SGI382872[] = {{&S67911[23], 3}, {NULL}};
const Segment SGI382873[] = {{&S67911[13], 92}, {&S67911[124], 17}, {NULL}};
const Segment SGI382874[] = {{&S67912[52], 5}, {NULL}};
const Segment SGI382875[] = {{&S67912[40], 3}, {NULL}};
const Segment SGI382876[] = {{&S67912[23], 3}, {NULL}};
const Segment SGI382877[] = {{&S67912[13], 91}, {&S67912[123], 17}, {NULL}};
const Segment SGI382878[] = {{&S67913[52], 5}, {NULL}};
const Segment SGI382879[] = {{&S67913[40], 3}, {NULL}};
const Segment SGI382880[] = {{&S67913[23], 3}, {NULL}};
const Segment SGI382881[] = {{&S67913[13], 92}, {&S67913[124], 17}, {NULL}};
const Segment SGI382882[] = {{&S67914[52], 5}, {NULL}};
const Segment SGI382883[] = {{&S67914[40], 3}, {NULL}};
const Segment SGI382884[] = {{&S67914[23], 3}, {NULL}};
const Segment SGI382885[] = {{&S67914[13], 92}, {&S67914[124], 17}, {NULL}};
const Segment SGI382886[] = {{&S67915[52], 5}, {NULL}};
const Segment SGI382887[] = {{&S67915[40], 3}, {NULL}};
const Segment SGI382888[] = {{&S67915[23], 3}, {NULL}};
const Segment SGI382889[] = {{&S67915[13], 92}, {&S67915[124], 17}, {NULL}};
const Segment SGI382890[] = {{&S67916[52], 5}, {NULL}};
const Segment SGI382891[] = {{&S67916[40], 3}, {NULL}};
const Segment SGI382892[] = {{&S67916[23], 3}, {NULL}};
const Segment SGI382893[] = {{&S67916[13], 92}, {&S67916[124], 17}, {NULL}};
const Segment SGI382894[] = {{&S67917[52], 5}, {NULL}};
const Segment SGI382895[] = {{&S67917[40], 3}, {NULL}};
const Segment SGI382896[] = {{&S67917[23], 3}, {NULL}};
const Segment SGI382897[] = {{&S67917[13], 92}, {&S67917[124], 17}, {NULL}};
const Segment SGI382898[] = {{&S67918[52], 5}, {NULL}};
const Segment SGI382899[] = {{&S67918[40], 3}, {NULL}};
const Segment SGI382900[] = {{&S67918[23], 3}, {NULL}};
const Segment SGI382901[] = {{&S67918[13], 92}, {&S67918[124], 17}, {NULL}};
const Segment SGI382902[] = {{&S67919[52], 5}, {NULL}};
const Segment SGI382903[] = {{&S67919[40], 3}, {NULL}};
const Segment SGI382904[] = {{&S67919[23], 3}, {NULL}};
const Segment SGI382905[] = {{&S67919[13], 92}, {&S67919[124], 17}, {NULL}};
const Segment SGI382906[] = {{&S67920[52], 5}, {NULL}};
const Segment SGI382907[] = {{&S67920[40], 3}, {NULL}};
const Segment SGI382908[] = {{&S67920[23], 3}, {NULL}};
const Segment SGI382909[] = {{&S67920[13], 92}, {&S67920[124], 17}, {NULL}};
const Segment SGI382910[] = {{&S67921[52], 5}, {NULL}};
const Segment SGI382911[] = {{&S67921[40], 3}, {NULL}};
const Segment SGI382912[] = {{&S67921[23], 3}, {NULL}};
const Segment SGI382913[] = {{&S67921[13], 92}, {&S67921[124], 17}, {NULL}};
const Segment SGI382914[] = {{&S67922[52], 5}, {NULL}};
const Segment SGI382915[] = {{&S67922[40], 3}, {NULL}};
const Segment SGI382916[] = {{&S67922[23], 3}, {NULL}};
const Segment SGI382917[] = {{&S67922[13], 92}, {&S67922[124], 17}, {NULL}};
const Segment SGI382918[] = {{&S67923[52], 5}, {NULL}};
const Segment SGI382919[] = {{&S67923[40], 3}, {NULL}};
const Segment SGI382920[] = {{&S67923[23], 3}, {NULL}};
const Segment SGI382921[] = {{&S67923[13], 92}, {&S67923[124], 17}, {NULL}};
const Segment SGI382922[] = {{&S67924[52], 5}, {NULL}};
const Segment SGI382923[] = {{&S67924[40], 3}, {NULL}};
const Segment SGI382924[] = {{&S67924[23], 3}, {NULL}};
const Segment SGI382925[] = {{&S67924[13], 92}, {&S67924[124], 17}, {NULL}};
const Segment SGI382926[] = {{&S67925[52], 5}, {NULL}};
const Segment SGI382927[] = {{&S67925[40], 3}, {NULL}};
const Segment SGI382928[] = {{&S67925[23], 3}, {NULL}};
const Segment SGI382929[] = {{&S67925[13], 92}, {&S67925[124], 17}, {NULL}};
const Segment SGI382930[] = {{&S67927[52], 5}, {NULL}};
const Segment SGI382931[] = {{&S67927[40], 3}, {NULL}};
const Segment SGI382932[] = {{&S67927[23], 3}, {NULL}};
const Segment SGI382933[] = {{&S67927[13], 92}, {&S67927[124], 17}, {NULL}};
const Segment SGI382934[] = {{&S67928[52], 5}, {NULL}};
const Segment SGI382935[] = {{&S67928[40], 3}, {NULL}};
const Segment SGI382936[] = {{&S67928[23], 3}, {NULL}};
const Segment SGI382937[] = {{&S67928[13], 92}, {&S67928[124], 17}, {NULL}};
const Segment SGI382938[] = {{&S67929[52], 6}, {NULL}};
const Segment SGI382939[] = {{&S67929[40], 3}, {NULL}};
const Segment SGI382940[] = {{&S67929[23], 3}, {NULL}};
const Segment SGI382941[] = {{&S67929[13], 92}, {&S67929[124], 17}, {NULL}};
const Segment SGI382942[] = {{&S67930[52], 5}, {NULL}};
const Segment SGI382943[] = {{&S67930[40], 3}, {NULL}};
const Segment SGI382944[] = {{&S67930[23], 3}, {NULL}};
const Segment SGI382945[] = {{&S67930[13], 92}, {&S67930[124], 17}, {NULL}};
const Segment SGI382946[] = {{&S67931[52], 5}, {NULL}};
const Segment SGI382947[] = {{&S67931[40], 3}, {NULL}};
const Segment SGI382948[] = {{&S67931[23], 3}, {NULL}};
const Segment SGI382949[] = {{&S67931[13], 92}, {&S67931[124], 17}, {NULL}};
const Segment SGI382950[] = {{&S89676[44], 13}, {NULL}};
const Segment SGI382951[] = {{&S89676[64], 13}, {NULL}};
const Segment SGI382952[] = {{&S67932[52], 5}, {NULL}};
const Segment SGI382953[] = {{&S67932[40], 3}, {NULL}};
const Segment SGI382954[] = {{&S67932[23], 3}, {NULL}};
const Segment SGI382955[] = {{&S67932[13], 92}, {&S67932[124], 17}, {NULL}};
const Segment SGI382956[] = {{&S67933[52], 5}, {NULL}};
const Segment SGI382957[] = {{&S67933[40], 3}, {NULL}};
const Segment SGI382958[] = {{&S67933[23], 3}, {NULL}};
const Segment SGI382959[] = {{&S67933[13], 92}, {&S67933[124], 17}, {NULL}};
const Segment SGI382960[] = {{&S89695[7], 9}, {NULL}};
const Segment SGI382961[] = {{&S89695[47], 3}, {NULL}};
const Segment SGI382962[] = {{&S89696[7], 10}, {NULL}};
const Segment SGI382963[] = {{&S89696[37], 3}, {NULL}};
const Segment SGI382964[] = {{&S89697[3], 8}, {NULL}};
const Segment SGI382965[] = {{&S89697[25], 3}, {NULL}};
const Segment SGI382966[] = {{&S89723[7], 10}, {NULL}};
const Segment SGI382967[] = {{&S89854[3], 8}, {NULL}};
const Segment SGI382968[] = {{&S89854[25], 3}, {NULL}};
const Segment SGI382969[] = {{&S89858[3], 5}, {NULL}};
const Segment SGI382970[] = {{&S89858[22], 10}, {NULL}};
const Segment SGI382971[] = {{&S89859[7], 9}, {NULL}};
const Segment SGI382972[] = {{&S89859[72], 3}, {NULL}};
const Segment SGI382973[] = {{&S89871[7], 9}, {NULL}};
const Segment SGI382974[] = {{&S89879[7], 9}, {NULL}};
const Segment SGI382975[] = {{&S89930[7], 9}, {NULL}};
const Segment SGI382976[] = {{&S89930[11], 5}, {NULL}};
const Segment SGI382977[] = {{&S75044[11], 11}, {NULL}};
const Segment SGI382978[] = {{&S75044[116], 3}, {NULL}};
const Segment SGI382979[] = {{&S75045[11], 11}, {NULL}};
const Segment SGI382980[] = {{&S75045[47], 3}, {NULL}};
const Segment SGI382981[] = {{&S75046[11], 5}, {NULL}};
const Segment SGI382982[] = {{&S75046[39], 3}, {NULL}};
const Segment SGI382983[] = {{&S90011[7], 9}, {NULL}};
const Segment SGI382984[] = {{&S90013[7], 9}, {NULL}};
const Segment SGI382985[] = {{&S90013[41], 3}, {NULL}};
const Segment SGI382986[] = {{&S90017[7], 9}, {NULL}};
const Segment SGI382987[] = {{&S90017[72], 3}, {NULL}};
const Segment SGI382988[] = {{&S90026[3], 8}, {NULL}};
const Segment SGI382989[] = {{&S90027[3], 8}, {NULL}};
const Segment SGI382990[] = {{&S90027[25], 3}, {NULL}};
const Segment SGI382991[] = {{&S90094[36], 22}, {NULL}};
const Segment SGI382992[] = {{&S90094[86], 13}, {NULL}};
const Segment SGI382993[] = {{&S90155[44], 13}, {NULL}};
const Segment SGI382994[] = {{&S90155[64], 13}, {NULL}};
const Segment SGI382995[] = {{&S90177[7], 9}, {NULL}};
const Segment SGI382996[] = {{&S90177[78], 3}, {NULL}};
const Segment SGI382997[] = {{&S90178[7], 9}, {NULL}};
const Segment SGI382998[] = {{&S90178[72], 3}, {NULL}};
const Segment SGI382999[] = {{&S90235[36], 22}, {NULL}};
const Segment SGI383000[] = {{&S90235[86], 13}, {NULL}};
const Segment SGI383001[] = {{&S90247[44], 9}, {NULL}};
const Segment SGI383002[] = {{&S90247[60], 13}, {NULL}};
const Segment SGI383003[] = {{&S75316[11], 6}, {NULL}};
const Segment SGI383004[] = {{&S75316[40], 3}, {NULL}};
const Segment SGI383005[] = {{&S68545[52], 8}, {NULL}};
const Segment SGI383006[] = {{&S68545[40], 3}, {NULL}};
const Segment SGI383007[] = {{&S68545[23], 3}, {NULL}};
const Segment SGI383008[] = {{&S68545[13], 94}, {&S68545[126], 17}, {NULL}};
const Segment SGI383009[] = {{&S68546[52], 8}, {NULL}};
const Segment SGI383010[] = {{&S68546[40], 3}, {NULL}};
const Segment SGI383011[] = {{&S68546[23], 3}, {NULL}};
const Segment SGI383012[] = {{&S68546[13], 94}, {&S68546[126], 17}, {NULL}};
const Segment SGI383013[] = {{&S68547[52], 8}, {NULL}};
const Segment SGI383014[] = {{&S68547[40], 3}, {NULL}};
const Segment SGI383015[] = {{&S68547[23], 3}, {NULL}};
const Segment SGI383016[] = {{&S68547[13], 94}, {&S68547[126], 17}, {NULL}};
const Segment SGI383017[] = {{&S68548[52], 8}, {NULL}};
const Segment SGI383018[] = {{&S68548[40], 3}, {NULL}};
const Segment SGI383019[] = {{&S68548[23], 3}, {NULL}};
const Segment SGI383020[] = {{&S68548[13], 94}, {&S68548[126], 17}, {NULL}};
const Segment SGI383021[] = {{&S68549[52], 8}, {NULL}};
const Segment SGI383022[] = {{&S68549[40], 3}, {NULL}};
const Segment SGI383023[] = {{&S68549[23], 3}, {NULL}};
const Segment SGI383024[] = {{&S68549[13], 94}, {&S68549[126], 17}, {NULL}};
const Segment SGI383025[] = {{&S68550[52], 8}, {NULL}};
const Segment SGI383026[] = {{&S68550[40], 3}, {NULL}};
const Segment SGI383027[] = {{&S68550[23], 3}, {NULL}};
const Segment SGI383028[] = {{&S68550[13], 94}, {&S68550[126], 17}, {NULL}};
const Segment SGI383029[] = {{&S68551[52], 8}, {NULL}};
const Segment SGI383030[] = {{&S68551[40], 3}, {NULL}};
const Segment SGI383031[] = {{&S68551[23], 3}, {NULL}};
const Segment SGI383032[] = {{&S68551[13], 94}, {&S68551[126], 17}, {NULL}};
const Segment SGI383033[] = {{&S68552[52], 8}, {NULL}};
const Segment SGI383034[] = {{&S68552[40], 3}, {NULL}};
const Segment SGI383035[] = {{&S68552[23], 3}, {NULL}};
const Segment SGI383036[] = {{&S68552[13], 94}, {&S68552[126], 17}, {NULL}};
const Segment SGI383037[] = {{&S68553[52], 8}, {NULL}};
const Segment SGI383038[] = {{&S68553[40], 3}, {NULL}};
const Segment SGI383039[] = {{&S68553[23], 3}, {NULL}};
const Segment SGI383040[] = {{&S68553[13], 94}, {&S68553[126], 17}, {NULL}};
const Segment SGI383041[] = {{&S68554[52], 8}, {NULL}};
const Segment SGI383042[] = {{&S68554[40], 3}, {NULL}};
const Segment SGI383043[] = {{&S68554[23], 3}, {NULL}};
const Segment SGI383044[] = {{&S68554[13], 94}, {&S68554[126], 17}, {NULL}};
const Segment SGI383045[] = {{&S68555[52], 8}, {NULL}};
const Segment SGI383046[] = {{&S68555[40], 3}, {NULL}};
const Segment SGI383047[] = {{&S68555[23], 3}, {NULL}};
const Segment SGI383048[] = {{&S68555[13], 94}, {&S68555[126], 17}, {NULL}};
const Segment SGI383049[] = {{&S68556[52], 8}, {NULL}};
const Segment SGI383050[] = {{&S68556[40], 3}, {NULL}};
const Segment SGI383051[] = {{&S68556[23], 3}, {NULL}};
const Segment SGI383052[] = {{&S68556[13], 94}, {&S68556[126], 17}, {NULL}};
const Segment SGI383053[] = {{&S68557[52], 8}, {NULL}};
const Segment SGI383054[] = {{&S68557[40], 3}, {NULL}};
const Segment SGI383055[] = {{&S68557[23], 3}, {NULL}};
const Segment SGI383056[] = {{&S68557[13], 94}, {&S68557[126], 17}, {NULL}};
const Segment SGI383057[] = {{&S68558[52], 8}, {NULL}};
const Segment SGI383058[] = {{&S68558[40], 3}, {NULL}};
const Segment SGI383059[] = {{&S68558[23], 3}, {NULL}};
const Segment SGI383060[] = {{&S68558[13], 94}, {&S68558[126], 17}, {NULL}};
const Segment SGI383061[] = {{&S68559[52], 8}, {NULL}};
const Segment SGI383062[] = {{&S68559[40], 3}, {NULL}};
const Segment SGI383063[] = {{&S68559[23], 3}, {NULL}};
const Segment SGI383064[] = {{&S68559[13], 94}, {&S68559[126], 17}, {NULL}};
const Segment SGI383065[] = {{&S68560[52], 8}, {NULL}};
const Segment SGI383066[] = {{&S68560[40], 3}, {NULL}};
const Segment SGI383067[] = {{&S68560[23], 3}, {NULL}};
const Segment SGI383068[] = {{&S68560[13], 94}, {&S68560[126], 17}, {NULL}};
const Segment SGI383069[] = {{&S68561[52], 8}, {NULL}};
const Segment SGI383070[] = {{&S68561[40], 3}, {NULL}};
const Segment SGI383071[] = {{&S68561[23], 3}, {NULL}};
const Segment SGI383072[] = {{&S68561[13], 94}, {&S68561[126], 17}, {NULL}};
const Segment SGI383073[] = {{&S68562[52], 8}, {NULL}};
const Segment SGI383074[] = {{&S68562[40], 3}, {NULL}};
const Segment SGI383075[] = {{&S68562[23], 3}, {NULL}};
const Segment SGI383076[] = {{&S68562[13], 94}, {&S68562[126], 17}, {NULL}};
const Segment SGI383077[] = {{&S68563[52], 8}, {NULL}};
const Segment SGI383078[] = {{&S68563[40], 3}, {NULL}};
const Segment SGI383079[] = {{&S68563[23], 3}, {NULL}};
const Segment SGI383080[] = {{&S68563[13], 94}, {&S68563[126], 17}, {NULL}};
const Segment SGI383081[] = {{&S68564[52], 8}, {NULL}};
const Segment SGI383082[] = {{&S68564[40], 3}, {NULL}};
const Segment SGI383083[] = {{&S68564[23], 3}, {NULL}};
const Segment SGI383084[] = {{&S68564[13], 94}, {&S68564[126], 17}, {NULL}};
const Segment SGI383085[] = {{&S68565[52], 9}, {NULL}};
const Segment SGI383086[] = {{&S68565[40], 3}, {NULL}};
const Segment SGI383087[] = {{&S68565[23], 3}, {NULL}};
const Segment SGI383088[] = {{&S68565[13], 95}, {&S68565[127], 17}, {NULL}};
const Segment SGI383089[] = {{&S68566[52], 9}, {NULL}};
const Segment SGI383090[] = {{&S68566[40], 3}, {NULL}};
const Segment SGI383091[] = {{&S68566[23], 3}, {NULL}};
const Segment SGI383092[] = {{&S68566[13], 95}, {&S68566[127], 17}, {NULL}};
const Segment SGI383093[] = {{&S68567[52], 8}, {NULL}};
const Segment SGI383094[] = {{&S68567[40], 3}, {NULL}};
const Segment SGI383095[] = {{&S68567[23], 3}, {NULL}};
const Segment SGI383096[] = {{&S68567[13], 94}, {&S68567[126], 17}, {NULL}};
const Segment SGI383097[] = {{&S68568[52], 8}, {NULL}};
const Segment SGI383098[] = {{&S68568[40], 3}, {NULL}};
const Segment SGI383099[] = {{&S68568[23], 3}, {NULL}};
const Segment SGI383100[] = {{&S68568[13], 94}, {&S68568[126], 17}, {NULL}};
const Segment SGI383101[] = {{&S68569[52], 8}, {NULL}};
const Segment SGI383102[] = {{&S68569[40], 3}, {NULL}};
const Segment SGI383103[] = {{&S68569[23], 3}, {NULL}};
const Segment SGI383104[] = {{&S68569[13], 94}, {&S68569[126], 17}, {NULL}};
const Segment SGI383105[] = {{&S68570[52], 8}, {NULL}};
const Segment SGI383106[] = {{&S68570[40], 3}, {NULL}};
const Segment SGI383107[] = {{&S68570[23], 3}, {NULL}};
const Segment SGI383108[] = {{&S68570[13], 94}, {&S68570[126], 17}, {NULL}};
const Segment SGI383109[] = {{&S68571[52], 8}, {NULL}};
const Segment SGI383110[] = {{&S68571[40], 3}, {NULL}};
const Segment SGI383111[] = {{&S68571[23], 3}, {NULL}};
const Segment SGI383112[] = {{&S68571[13], 94}, {&S68571[126], 17}, {NULL}};
const Segment SGI383113[] = {{&S68572[52], 8}, {NULL}};
const Segment SGI383114[] = {{&S68572[40], 3}, {NULL}};
const Segment SGI383115[] = {{&S68572[23], 3}, {NULL}};
const Segment SGI383116[] = {{&S68572[13], 94}, {&S68572[126], 17}, {NULL}};
const Segment SGI383117[] = {{&S68573[52], 8}, {NULL}};
const Segment SGI383118[] = {{&S68573[40], 3}, {NULL}};
const Segment SGI383119[] = {{&S68573[23], 3}, {NULL}};
const Segment SGI383120[] = {{&S68573[13], 94}, {&S68573[126], 17}, {NULL}};
const Segment SGI383121[] = {{&S68574[52], 9}, {NULL}};
const Segment SGI383122[] = {{&S68574[40], 3}, {NULL}};
const Segment SGI383123[] = {{&S68574[23], 3}, {NULL}};
const Segment SGI383124[] = {{&S68574[13], 95}, {&S68574[127], 17}, {NULL}};
const Segment SGI383125[] = {{&S68575[52], 9}, {NULL}};
const Segment SGI383126[] = {{&S68575[40], 3}, {NULL}};
const Segment SGI383127[] = {{&S68575[23], 3}, {NULL}};
const Segment SGI383128[] = {{&S68575[13], 95}, {&S68575[127], 17}, {NULL}};
const Segment SGI383129[] = {{&S68576[52], 8}, {NULL}};
const Segment SGI383130[] = {{&S68576[40], 3}, {NULL}};
const Segment SGI383131[] = {{&S68576[23], 3}, {NULL}};
const Segment SGI383132[] = {{&S68576[13], 94}, {&S68576[126], 17}, {NULL}};
const Segment SGI383133[] = {{&S68577[52], 8}, {NULL}};
const Segment SGI383134[] = {{&S68577[40], 3}, {NULL}};
const Segment SGI383135[] = {{&S68577[23], 3}, {NULL}};
const Segment SGI383136[] = {{&S68577[13], 94}, {&S68577[126], 17}, {NULL}};
const Segment SGI383137[] = {{&S68578[52], 8}, {NULL}};
const Segment SGI383138[] = {{&S68578[40], 3}, {NULL}};
const Segment SGI383139[] = {{&S68578[23], 3}, {NULL}};
const Segment SGI383140[] = {{&S68578[13], 94}, {&S68578[126], 17}, {NULL}};
const Segment SGI383141[] = {{&S90354[7], 6}, {NULL}};
const Segment SGI383142[] = {{&S90354[76], 3}, {NULL}};
const Segment SGI383143[] = {{&S68579[52], 8}, {NULL}};
const Segment SGI383144[] = {{&S68579[40], 3}, {NULL}};
const Segment SGI383145[] = {{&S68579[23], 3}, {NULL}};
const Segment SGI383146[] = {{&S68579[13], 94}, {&S68579[126], 17}, {NULL}};
const Segment SGI383147[] = {{&S68580[52], 8}, {NULL}};
const Segment SGI383148[] = {{&S68580[40], 3}, {NULL}};
const Segment SGI383149[] = {{&S68580[23], 3}, {NULL}};
const Segment SGI383150[] = {{&S68580[13], 94}, {&S68580[126], 17}, {NULL}};
const Segment SGI383151[] = {{&S68581[52], 8}, {NULL}};
const Segment SGI383152[] = {{&S68581[40], 3}, {NULL}};
const Segment SGI383153[] = {{&S68581[23], 3}, {NULL}};
const Segment SGI383154[] = {{&S68581[13], 94}, {&S68581[126], 17}, {NULL}};
const Segment SGI383155[] = {{&S68582[52], 8}, {NULL}};
const Segment SGI383156[] = {{&S90360[3], 8}, {NULL}};
const Segment SGI383157[] = {{&S68582[40], 3}, {NULL}};
const Segment SGI383158[] = {{&S68582[23], 3}, {NULL}};
const Segment SGI383159[] = {{&S90360[50], 3}, {NULL}};
const Segment SGI383160[] = {{&S68582[13], 94}, {&S68582[126], 17}, {NULL}};
const Segment SGI383161[] = {{&S68583[52], 8}, {NULL}};
const Segment SGI383162[] = {{&S68583[40], 3}, {NULL}};
const Segment SGI383163[] = {{&S68583[23], 3}, {NULL}};
const Segment SGI383164[] = {{&S90361[3], 8}, {NULL}};
const Segment SGI383165[] = {{&S68583[13], 94}, {&S68583[126], 17}, {NULL}};
const Segment SGI383166[] = {{&S90361[25], 3}, {NULL}};
const Segment SGI383167[] = {{&S68584[52], 8}, {NULL}};
const Segment SGI383168[] = {{&S90362[3], 12}, {NULL}};
const Segment SGI383169[] = {{&S68584[40], 3}, {NULL}};
const Segment SGI383170[] = {{&S68584[23], 3}, {NULL}};
const Segment SGI383171[] = {{&S68584[13], 94}, {&S68584[126], 17}, {NULL}};
const Segment SGI383172[] = {{&S90363[3], 5}, {NULL}};
const Segment SGI383173[] = {{&S68585[52], 9}, {NULL}};
const Segment SGI383174[] = {{&S68585[40], 3}, {NULL}};
const Segment SGI383175[] = {{&S68585[23], 3}, {NULL}};
const Segment SGI383176[] = {{&S68585[13], 95}, {&S68585[127], 17}, {NULL}};
const Segment SGI383177[] = {{&S68586[52], 8}, {NULL}};
const Segment SGI383178[] = {{&S68586[40], 3}, {NULL}};
const Segment SGI383179[] = {{&S68586[23], 3}, {NULL}};
const Segment SGI383180[] = {{&S68586[13], 94}, {&S68586[126], 17}, {NULL}};
const Segment SGI383181[] = {{&S90387[7], 9}, {NULL}};
const Segment SGI383182[] = {{&S90426[44], 13}, {NULL}};
const Segment SGI383183[] = {{&S90426[64], 13}, {NULL}};
const Segment SGI383184[] = {{&S90427[41], 14}, {NULL}};
const Segment SGI383185[] = {{&S90427[62], 13}, {NULL}};
const Segment SGI383186[] = {{&S75543[11], 5}, {NULL}};
const Segment SGI383187[] = {{&S75543[30], 3}, {NULL}};
const Segment SGI383188[] = {{&S90542[3], 8}, {NULL}};
const Segment SGI383189[] = {{&S90542[25], 3}, {NULL}};
const Segment SGI383190[] = {{&S90577[7], 9}, {NULL}};
const Segment SGI383191[] = {{&S90597[36], 22}, {NULL}};
const Segment SGI383192[] = {{&S90597[86], 13}, {NULL}};
const Segment SGI383193[] = {{&S90631[7], 9}, {NULL}};
const Segment SGI383194[] = {{&S90637[7], 9}, {NULL}};
const Segment SGI383195[] = {{&S75767[11], 5}, {NULL}};
const Segment SGI383196[] = {{&S75767[48], 3}, {NULL}};
const Segment SGI383197[] = {{&S75768[11], 6}, {NULL}};
const Segment SGI383198[] = {{&S75768[42], 3}, {NULL}};
const Segment SGI383199[] = {{&S90736[7], 9}, {NULL}};
const Segment SGI383200[] = {{&S90736[78], 3}, {NULL}};
const Segment SGI383201[] = {{&S90737[7], 10}, {NULL}};
const Segment SGI383202[] = {{&S90737[80], 3}, {NULL}};
const Segment SGI383203[] = {{&S90838[43], 13}, {NULL}};
const Segment SGI383204[] = {{&S90838[63], 13}, {NULL}};
const Segment SGI383205[] = {{&S75928[11], 5}, {NULL}};
const Segment SGI383206[] = {{&S75928[30], 3}, {NULL}};
const Segment SGI383207[] = {{&S90952[7], 9}, {NULL}};
const Segment SGI383208[] = {{&S76050[27], 5}, {NULL}};
const Segment SGI383209[] = {{&S76063[33], 6}, {NULL}};
const Segment SGI383210[] = {{&S76063[61], 3}, {NULL}};
const Segment SGI383211[] = {{&S76063[118], 3}, {NULL}};
const Segment SGI383212[] = {{&S91054[44], 14}, {NULL}};
const Segment SGI383213[] = {{&S91054[65], 13}, {NULL}};
const Segment SGI383214[] = {{&S91055[44], 14}, {NULL}};
const Segment SGI383215[] = {{&S91055[65], 13}, {NULL}};
const Segment SGI383216[] = {{&S91074[44], 14}, {NULL}};
const Segment SGI383217[] = {{&S91074[65], 13}, {NULL}};
const Segment SGI383218[] = {{&S91075[44], 14}, {NULL}};
const Segment SGI383219[] = {{&S91075[65], 13}, {NULL}};
const Segment SGI383220[] = {{&S91118[7], 26}, {NULL}};
const Segment SGI383221[] = {{&S91118[11], 22}, {NULL}};
const Segment SGI383222[] = {{&S91135[7], 9}, {NULL}};
const Segment SGI383223[] = {{&S91156[36], 29}, {NULL}};
const Segment SGI383224[] = {{&S91210[45], 9}, {NULL}};
const Segment SGI383225[] = {{&S91210[61], 13}, {NULL}};
const Segment SGI383226[] = {{&S91257[137], 10}, {NULL}};
const Segment SGI383227[] = {{&S91277[7], 9}, {NULL}};
const Segment SGI383228[] = {{&S91277[11], 5}, {NULL}};
const Segment SGI383229[] = {{&S91374[42], 9}, {NULL}};
const Segment SGI383230[] = {{&S91374[58], 13}, {NULL}};
const Segment SGI383231[] = {{&S91474[7], 9}, {NULL}};
const Segment SGI383232[] = {{&S91474[72], 3}, {NULL}};
const Segment SGI383233[] = {{&S91533[36], 30}, {NULL}};
const Segment SGI383234[] = {{&S91638[44], 13}, {NULL}};
const Segment SGI383235[] = {{&S91638[64], 13}, {NULL}};
const Segment SGI383236[] = {{&S91674[7], 9}, {NULL}};
const Segment SGI383237[] = {{&S91674[47], 3}, {NULL}};
const Segment SGI383238[] = {{&S91675[7], 9}, {NULL}};
const Segment SGI383239[] = {{&S91675[72], 3}, {NULL}};
const Segment SGI383240[] = {{&S91676[7], 10}, {NULL}};
const Segment SGI383241[] = {{&S91676[39], 3}, {NULL}};
const Segment SGI383242[] = {{&S91677[7], 9}, {NULL}};
const Segment SGI383243[] = {{&S91677[38], 3}, {NULL}};
const Segment SGI383244[] = {{&S91678[7], 9}, {NULL}};
const Segment SGI383245[] = {{&S91678[47], 3}, {NULL}};
const Segment SGI383246[] = {{&S91688[3], 8}, {NULL}};
const Segment SGI383247[] = {{&S91688[25], 3}, {NULL}};
const Segment SGI383248[] = {{&S76617[11], 12}, {NULL}};
const Segment SGI383249[] = {{&S76617[24], 4}, {NULL}};
const Segment SGI383250[] = {{&S76617[52], 3}, {NULL}};
const Segment SGI383251[] = {{&S91699[7], 6}, {NULL}};
const Segment SGI383252[] = {{&S91711[7], 9}, {NULL}};
const Segment SGI383253[] = {{&S91720[36], 29}, {NULL}};
const Segment SGI383254[] = {{&S76687[11], 6}, {NULL}};
const Segment SGI383255[] = {{&S91869[7], 6}, {NULL}};
const Segment SGI383256[] = {{&S91869[42], 3}, {NULL}};
const Segment SGI383257[] = {{&S91870[7], 9}, {NULL}};
const Segment SGI383258[] = {{&S91870[76], 3}, {NULL}};
const Segment SGI383259[] = {{&S91871[7], 10}, {NULL}};
const Segment SGI383260[] = {{&S91871[77], 3}, {NULL}};
const Segment SGI383261[] = {{&S91874[7], 9}, {NULL}};
const Segment SGI383262[] = {{&S91950[44], 13}, {NULL}};
const Segment SGI383263[] = {{&S91950[64], 13}, {NULL}};
const Segment SGI383264[] = {{&S76897[11], 5}, {NULL}};
const Segment SGI383265[] = {{&S76897[43], 3}, {NULL}};
const Segment SGI383266[] = {{&S76901[11], 5}, {NULL}};
const Segment SGI383267[] = {{&S76901[39], 3}, {NULL}};
const Segment SGI383268[] = {{&S76902[24], 5}, {NULL}};
const Segment SGI383269[] = {{&S76902[55], 3}, {NULL}};
const Segment SGI383270[] = {{&S76903[24], 5}, {NULL}};
const Segment SGI383271[] = {{&S76903[55], 3}, {NULL}};
const Segment SGI383272[] = {{&S76904[24], 5}, {NULL}};
const Segment SGI383273[] = {{&S76904[55], 3}, {NULL}};
const Segment SGI383274[] = {{&S92065[7], 9}, {NULL}};
const Segment SGI383275[] = {{&S92065[47], 3}, {NULL}};
const Segment SGI383276[] = {{&S92066[7], 9}, {NULL}};
const Segment SGI383277[] = {{&S92066[38], 3}, {NULL}};
const Segment SGI383278[] = {{&S92071[3], 8}, {NULL}};
const Segment SGI383279[] = {{&S92071[25], 3}, {NULL}};
const Segment SGI383280[] = {{&S92078[3], 8}, {NULL}};
const Segment SGI383281[] = {{&S92078[73], 3}, {NULL}};
const Segment SGI383282[] = {{&S92081[7], 9}, {NULL}};
const Segment SGI383283[] = {{&S92083[7], 9}, {NULL}};
const Segment SGI383284[] = {{&S92083[41], 3}, {NULL}};
const Segment SGI383285[] = {{&S92112[7], 9}, {NULL}};
const Segment SGI383286[] = {{&S92119[7], 7}, {NULL}};
const Segment SGI383287[] = {{&S92119[10], 4}, {NULL}};
const Segment SGI383288[] = {{&S92124[36], 29}, {NULL}};
const Segment SGI383289[] = {{&S92129[43], 10}, {NULL}};
const Segment SGI383290[] = {{&S92129[60], 13}, {NULL}};
const Segment SGI383291[] = {{&S92151[23], 10}, {NULL}};
const Segment SGI383292[] = {{&S77190[11], 5}, {NULL}};
const Segment SGI383293[] = {{&S77190[41], 3}, {NULL}};
const Segment SGI383294[] = {{&S77287[11], 6}, {NULL}};
const Segment SGI383295[] = {{&S77287[31], 3}, {NULL}};
const Segment SGI383296[] = {{&S92286[7], 9}, {NULL}};
const Segment SGI383297[] = {{&S92286[47], 3}, {NULL}};
const Segment SGI383298[] = {{&S77463[33], 6}, {NULL}};
const Segment SGI383299[] = {{&S77463[40], 3}, {NULL}};
const Segment SGI383300[] = {{&S77463[107], 3}, {NULL}};
const Segment SGI383301[] = {{&S92456[7], 9}, {NULL}};
const Segment SGI383302[] = {{&S92456[45], 3}, {NULL}};
const Segment SGI383303[] = {{&S92465[7], 10}, {NULL}};
const Segment SGI383304[] = {{&S92465[40], 3}, {NULL}};
const Segment SGI383305[] = {{&S92466[3], 5}, {NULL}};
const Segment SGI383306[] = {{&S92466[22], 10}, {NULL}};
const Segment SGI383307[] = {{&S92467[3], 5}, {NULL}};
const Segment SGI383308[] = {{&S92467[22], 10}, {NULL}};
const Segment SGI383309[] = {{&S92479[7], 9}, {NULL}};
const Segment SGI383310[] = {{&S92479[41], 3}, {NULL}};
const Segment SGI383311[] = {{&S92480[7], 18}, {NULL}};
const Segment SGI383312[] = {{&S92480[50], 3}, {NULL}};
const Segment SGI383313[] = {{&S92481[7], 6}, {NULL}};
const Segment SGI383314[] = {{&S92485[68], 17}, {NULL}};
const Segment SGI383315[] = {{&S77571[11], 5}, {NULL}};
const Segment SGI383316[] = {{&S77571[41], 3}, {NULL}};
const Segment SGI383317[] = {{&S92494[7], 10}, {NULL}};
const Segment SGI383318[] = {{&S92526[7], 10}, {NULL}};
const Segment SGI383319[] = {{&S77760[11], 6}, {NULL}};
const Segment SGI383320[] = {{&S77760[31], 3}, {NULL}};
const Segment SGI383321[] = {{&S92570[47], 14}, {NULL}};
const Segment SGI383322[] = {{&S92570[68], 13}, {NULL}};
const Segment SGI383323[] = {{&S92573[44], 13}, {NULL}};
const Segment SGI383324[] = {{&S92573[64], 13}, {NULL}};
const Segment SGI383325[] = {{&S92596[7], 9}, {NULL}};
const Segment SGI383326[] = {{&S92656[7], 17}, {NULL}};
const Segment SGI383327[] = {{&S92656[68], 3}, {NULL}};
const Segment SGI383328[] = {{&S92657[7], 9}, {NULL}};
const Segment SGI383329[] = {{&S92657[78], 3}, {NULL}};
const Segment SGI383330[] = {{&S92658[7], 9}, {NULL}};
const Segment SGI383331[] = {{&S92658[72], 3}, {NULL}};
const Segment SGI383332[] = {{&S92662[7], 17}, {NULL}};
const Segment SGI383333[] = {{&S92663[7], 9}, {NULL}};
const Segment SGI383334[] = {{&S92665[3], 8}, {NULL}};
const Segment SGI383335[] = {{&S92665[25], 3}, {NULL}};
const Segment SGI383336[] = {{&S92669[7], 20}, {NULL}};
const Segment SGI383337[] = {{&S92712[37], 18}, {NULL}};
const Segment SGI383338[] = {{&S92712[83], 13}, {NULL}};
const Segment SGI383339[] = {{&S92823[69], 10}, {NULL}};
const Segment SGI383340[] = {{&S92824[69], 10}, {NULL}};
const Segment SGI383341[] = {{&S92825[69], 10}, {NULL}};
const Segment SGI383342[] = {{&S92832[7], 6}, {NULL}};
const Segment SGI383343[] = {{&S92832[27], 3}, {NULL}};
const Segment SGI383344[] = {{&S92837[7], 9}, {NULL}};
const Segment SGI383345[] = {{&S92882[36], 30}, {NULL}};
const Segment SGI383346[] = {{&S92941[23], 11}, {NULL}};
const Segment SGI383347[] = {{&S93055[3], 8}, {NULL}};
const Segment SGI383348[] = {{&S93055[25], 3}, {NULL}};
const Segment SGI383349[] = {{&S93068[7], 9}, {NULL}};
const Segment SGI383350[] = {{&S93091[36], 30}, {NULL}};
const Segment SGI383351[] = {{&S93098[7], 9}, {NULL}};
const Segment SGI383352[] = {{&S93220[44], 13}, {NULL}};
const Segment SGI383353[] = {{&S93220[64], 13}, {NULL}};
const Segment SGI383354[] = {{&S93266[3], 5}, {NULL}};
const Segment SGI383355[] = {{&S93266[22], 11}, {NULL}};
const Segment SGI383356[] = {{&S93267[3], 8}, {NULL}};
const Segment SGI383357[] = {{&S93267[25], 3}, {NULL}};
const Segment SGI383358[] = {{&S93268[3], 15}, {NULL}};
const Segment SGI383359[] = {{&S93268[38], 3}, {NULL}};
const Segment SGI383360[] = {{&S78377[11], 5}, {NULL}};
const Segment SGI383361[] = {{&S93269[3], 8}, {NULL}};
const Segment SGI383362[] = {{&S78377[30], 3}, {NULL}};
const Segment SGI383363[] = {{&S93269[25], 3}, {NULL}};
const Segment SGI383364[] = {{&S93270[3], 8}, {NULL}};
const Segment SGI383365[] = {{&S93270[25], 3}, {NULL}};
const Segment SGI383366[] = {{&S93272[7], 17}, {NULL}};
const Segment SGI383367[] = {{&S93273[85], 9}, {NULL}};
const Segment SGI383368[] = {{&S93273[89], 5}, {NULL}};
const Segment SGI383369[] = {{&S93276[7], 9}, {NULL}};
const Segment SGI383370[] = {{&S93276[72], 3}, {NULL}};
const Segment SGI383371[] = {{&S93302[7], 9}, {NULL}};
const Segment SGI383372[] = {{&S93323[7], 10}, {NULL}};
const Segment SGI383373[] = {{&S93332[7], 9}, {NULL}};
const Segment SGI383374[] = {{&S93333[36], 29}, {NULL}};
const Segment SGI383375[] = {{&S78483[11], 6}, {NULL}};
const Segment SGI383376[] = {{&S78483[39], 3}, {NULL}};
const Segment SGI383377[] = {{&S93377[36], 29}, {NULL}};
const Segment SGI383378[] = {{&S93499[7], 9}, {NULL}};
const Segment SGI383379[] = {{&S93499[72], 3}, {NULL}};
const Segment SGI383380[] = {{&S93517[3], 8}, {NULL}};
const Segment SGI383381[] = {{&S93517[25], 3}, {NULL}};
const Segment SGI383382[] = {{&S93522[7], 9}, {NULL}};
const Segment SGI383383[] = {{&S78645[11], 5}, {NULL}};
const Segment SGI383384[] = {{&S78645[30], 3}, {NULL}};
const Segment SGI383385[] = {{&S78651[112], 5}, {NULL}};
const Segment SGI383386[] = {{&S78651[180], 3}, {NULL}};
const Segment SGI383387[] = {{&S93599[23], 11}, {NULL}};
const Segment SGI383388[] = {{&S93601[43], 13}, {NULL}};
const Segment SGI383389[] = {{&S93601[63], 13}, {NULL}};
const Segment SGI383390[] = {{&S93656[44], 13}, {NULL}};
const Segment SGI383391[] = {{&S93656[64], 13}, {NULL}};
const Segment SGI383392[] = {{&S93687[33], 21}, {NULL}};
const Segment SGI383393[] = {{&S93693[44], 13}, {NULL}};
const Segment SGI383394[] = {{&S93693[64], 13}, {NULL}};
const Segment SGI383395[] = {{&S93714[7], 9}, {NULL}};
const Segment SGI383396[] = {{&S93714[11], 5}, {NULL}};
const Segment SGI383397[] = {{&S93714[73], 3}, {NULL}};
const Segment SGI383398[] = {{&S93734[7], 9}, {NULL}};
const Segment SGI383399[] = {{&S93734[72], 3}, {NULL}};
const Segment SGI383400[] = {{&S93768[7], 9}, {NULL}};
const Segment SGI383401[] = {{&S93961[3], 5}, {NULL}};
const Segment SGI383402[] = {{&S93961[22], 10}, {NULL}};
const Segment SGI383403[] = {{&S94011[51], 21}, {NULL}};
const Segment SGI383404[] = {{&S94012[51], 21}, {NULL}};
const Segment SGI383405[] = {{&S94013[51], 21}, {NULL}};
const Segment SGI383406[] = {{&S94018[36], 29}, {NULL}};
const Segment SGI383407[] = {{&S94019[36], 29}, {NULL}};
const Segment SGI383408[] = {{&S78822[11], 5}, {NULL}};
const Segment SGI383409[] = {{&S78822[30], 3}, {NULL}};
const Segment SGI383410[] = {{&S94049[43], 10}, {NULL}};
const Segment SGI383411[] = {{&S94049[60], 13}, {NULL}};
const Segment SGI383412[] = {{&S94055[44], 13}, {NULL}};
const Segment SGI383413[] = {{&S94055[64], 13}, {NULL}};
const Segment SGI383414[] = {{&S94182[7], 9}, {NULL}};
const Segment SGI383415[] = {{&S94182[43], 3}, {NULL}};
const Segment SGI383416[] = {{&S94198[7], 9}, {NULL}};
const Segment SGI383417[] = {{&S94198[43], 3}, {NULL}};
const Segment SGI383418[] = {{&S94236[79], 9}, {NULL}};
const Segment SGI383419[] = {{&S94236[83], 5}, {NULL}};
const Segment SGI383420[] = {{&S94236[6], 4}, {NULL}};
const Segment SGI383421[] = {{&S94236[29], 3}, {NULL}};
const Segment SGI383422[] = {{&S94363[44], 10}, {NULL}};
const Segment SGI383423[] = {{&S94363[61], 13}, {NULL}};
const Segment SGI383424[] = {{&S94392[7], 9}, {NULL}};
const Segment SGI383425[] = {{&S94398[3], 8}, {NULL}};
const Segment SGI383426[] = {{&S94398[25], 3}, {NULL}};
const Segment SGI383427[] = {{&S94402[7], 9}, {NULL}};
const Segment SGI383428[] = {{&S94402[78], 3}, {NULL}};
const Segment SGI383429[] = {{&S94461[43], 13}, {NULL}};
const Segment SGI383430[] = {{&S94461[63], 13}, {NULL}};
const Segment SGI383431[] = {{&S94528[44], 14}, {NULL}};
const Segment SGI383432[] = {{&S94528[65], 13}, {NULL}};
const Segment SGI383433[] = {{&S94552[44], 13}, {NULL}};
const Segment SGI383434[] = {{&S94552[64], 13}, {NULL}};
const Segment SGI383435[] = {{&S94564[44], 13}, {NULL}};
const Segment SGI383436[] = {{&S94564[64], 13}, {NULL}};
const Segment SGI383437[] = {{&S79465[11], 6}, {NULL}};
const Segment SGI383438[] = {{&S79466[11], 5}, {NULL}};
const Segment SGI383439[] = {{&S94626[7], 9}, {NULL}};
const Segment SGI383440[] = {{&S94626[72], 3}, {NULL}};
const Segment SGI383441[] = {{&S94628[7], 10}, {NULL}};
const Segment SGI383442[] = {{&S94628[73], 3}, {NULL}};
const Segment SGI383443[] = {{&S94629[7], 9}, {NULL}};
const Segment SGI383444[] = {{&S94629[72], 3}, {NULL}};
const Segment SGI383445[] = {{&S79538[11], 5}, {NULL}};
const Segment SGI383446[] = {{&S79538[79], 3}, {NULL}};
const Segment SGI383447[] = {{&S94648[7], 9}, {NULL}};
const Segment SGI383448[] = {{&S94648[41], 3}, {NULL}};
const Segment SGI383449[] = {{&S94653[3], 5}, {NULL}};
const Segment SGI383450[] = {{&S94653[22], 10}, {NULL}};
const Segment SGI383451[] = {{&S94656[7], 26}, {NULL}};
const Segment SGI383452[] = {{&S94656[11], 22}, {NULL}};
const Segment SGI383453[] = {{&S94659[3], 8}, {NULL}};
const Segment SGI383454[] = {{&S94804[44], 10}, {NULL}};
const Segment SGI383455[] = {{&S94804[61], 13}, {NULL}};
const Segment SGI383456[] = {{&S94854[44], 13}, {NULL}};
const Segment SGI383457[] = {{&S94854[64], 13}, {NULL}};
const Segment SGI383458[] = {{&S94886[7], 6}, {NULL}};
const Segment SGI383459[] = {{&S94886[76], 3}, {NULL}};
const Segment SGI383460[] = {{&S94912[7], 10}, {NULL}};
const Segment SGI383461[] = {{&S94918[7], 9}, {NULL}};
const Segment SGI383462[] = {{&S94940[7], 9}, {NULL}};
const Segment SGI383463[] = {{&S94981[44], 14}, {NULL}};
const Segment SGI383464[] = {{&S94981[65], 13}, {NULL}};
const Segment SGI383465[] = {{&S79876[11], 5}, {NULL}};
const Segment SGI383466[] = {{&S79876[47], 3}, {NULL}};
const Segment SGI383467[] = {{&S79940[11], 6}, {NULL}};
const Segment SGI383468[] = {{&S79940[18], 3}, {NULL}};
const Segment SGI383469[] = {{&S95135[7], 9}, {NULL}};
const Segment SGI383470[] = {{&S95135[78], 3}, {NULL}};
const Segment SGI383471[] = {{&S95142[7], 10}, {NULL}};
const Segment SGI383472[] = {{&S95142[42], 3}, {NULL}};
const Segment SGI383473[] = {{&S95187[7], 10}, {NULL}};
const Segment SGI383474[] = {{&S95261[44], 13}, {NULL}};
const Segment SGI383475[] = {{&S95261[64], 13}, {NULL}};
const Segment SGI383476[] = {{&S95325[44], 14}, {NULL}};
const Segment SGI383477[] = {{&S95325[65], 13}, {NULL}};
const Segment SGI383478[] = {{&S95361[3], 8}, {NULL}};
const Segment SGI383479[] = {{&S95361[25], 3}, {NULL}};
const Segment SGI383480[] = {{&S95369[7], 9}, {NULL}};
const Segment SGI383481[] = {{&S95369[76], 3}, {NULL}};
const Segment SGI383482[] = {{&S95372[7], 9}, {NULL}};
const Segment SGI383483[] = {{&S95372[36], 3}, {NULL}};
const Segment SGI383484[] = {{&S80230[27], 6}, {NULL}};
const Segment SGI383485[] = {{&S95405[7], 9}, {NULL}};
const Segment SGI383486[] = {{&S95477[44], 14}, {NULL}};
const Segment SGI383487[] = {{&S95477[65], 13}, {NULL}};
const Segment SGI383488[] = {{&S95596[44], 13}, {NULL}};
const Segment SGI383489[] = {{&S95596[64], 13}, {NULL}};
const Segment SGI383490[] = {{&S95666[7], 17}, {NULL}};
const Segment SGI383491[] = {{&S95666[80], 3}, {NULL}};
const Segment SGI383492[] = {{&S95667[7], 10}, {NULL}};
const Segment SGI383493[] = {{&S95667[73], 3}, {NULL}};
const Segment SGI383494[] = {{&S95668[7], 9}, {NULL}};
const Segment SGI383495[] = {{&S95668[72], 3}, {NULL}};
const Segment SGI383496[] = {{&S95675[3], 8}, {NULL}};
const Segment SGI383497[] = {{&S95675[25], 3}, {NULL}};
const Segment SGI383498[] = {{&S95677[3], 16}, {NULL}};
const Segment SGI383499[] = {{&S95677[20], 3}, {NULL}};
const Segment SGI383500[] = {{&S95678[7], 20}, {NULL}};
const Segment SGI383501[] = {{&S95682[7], 9}, {NULL}};
const Segment SGI383502[] = {{&S95686[7], 20}, {NULL}};
const Segment SGI383503[] = {{&S95687[7], 22}, {NULL}};
const Segment SGI383504[] = {{&S95704[7], 9}, {NULL}};
const Segment SGI383505[] = {{&S95710[58], 20}, {NULL}};
const Segment SGI383506[] = {{&S95711[59], 20}, {NULL}};
const Segment SGI383507[] = {{&S95719[7], 10}, {NULL}};
const Segment SGI383508[] = {{&S95751[40], 14}, {NULL}};
const Segment SGI383509[] = {{&S95751[61], 13}, {NULL}};
const Segment SGI383510[] = {{&S80652[11], 11}, {NULL}};
const Segment SGI383511[] = {{&S80652[35], 4}, {NULL}};
const Segment SGI383512[] = {{&S80653[11], 5}, {NULL}};
const Segment SGI383513[] = {{&S80653[27], 4}, {NULL}};
const Segment SGI383514[] = {{&S95962[7], 9}, {NULL}};
const Segment SGI383515[] = {{&S95962[78], 3}, {NULL}};
const Segment SGI383516[] = {{&S95965[3], 8}, {NULL}};
const Segment SGI383517[] = {{&S95965[25], 3}, {NULL}};
const Segment SGI383518[] = {{&S95971[7], 26}, {NULL}};
const Segment SGI383519[] = {{&S95971[11], 22}, {NULL}};
const Segment SGI383520[] = {{&S95984[3], 8}, {NULL}};
const Segment SGI383521[] = {{&S80696[11], 6}, {NULL}};
const Segment SGI383522[] = {{&S80696[31], 3}, {NULL}};
const Segment SGI383523[] = {{&S96096[23], 10}, {NULL}};
const Segment SGI383524[] = {{&S81013[11], 6}, {NULL}};
const Segment SGI383525[] = {{&S81013[31], 3}, {NULL}};
const Segment SGI383526[] = {{&S96288[7], 9}, {NULL}};
const Segment SGI383527[] = {{&S96288[55], 3}, {NULL}};
const Segment SGI383528[] = {{&S96289[7], 14}, {NULL}};
const Segment SGI383529[] = {{&S96289[60], 3}, {NULL}};
const Segment SGI383530[] = {{&S96290[7], 9}, {NULL}};
const Segment SGI383531[] = {{&S96290[78], 3}, {NULL}};
const Segment SGI383532[] = {{&S96291[7], 10}, {NULL}};
const Segment SGI383533[] = {{&S96291[79], 3}, {NULL}};
const Segment SGI383534[] = {{&S81121[11], 6}, {NULL}};
const Segment SGI383535[] = {{&S81121[81], 3}, {NULL}};
const Segment SGI383536[] = {{&S96297[3], 8}, {NULL}};
const Segment SGI383537[] = {{&S96298[7], 9}, {NULL}};
const Segment SGI383538[] = {{&S96298[30], 3}, {NULL}};
const Segment SGI383539[] = {{&S96299[3], 16}, {NULL}};
const Segment SGI383540[] = {{&S96299[20], 3}, {NULL}};
const Segment SGI383541[] = {{&S96306[7], 9}, {NULL}};
const Segment SGI383542[] = {{&S96306[72], 3}, {NULL}};
const Segment SGI383543[] = {{&S96314[7], 9}, {NULL}};
const Segment SGI383544[] = {{&S96314[41], 3}, {NULL}};
const Segment SGI383545[] = {{&S96315[7], 9}, {NULL}};
const Segment SGI383546[] = {{&S96316[7], 9}, {NULL}};
const Segment SGI383547[] = {{&S96316[42], 3}, {NULL}};
const Segment SGI383548[] = {{&S81164[11], 5}, {NULL}};
const Segment SGI383549[] = {{&S81164[36], 3}, {NULL}};
const Segment SGI383550[] = {{&S81166[11], 6}, {NULL}};
const Segment SGI383551[] = {{&S96357[36], 29}, {NULL}};
const Segment SGI383552[] = {{&S96464[7], 9}, {NULL}};
const Segment SGI383553[] = {{&S96509[51], 20}, {NULL}};
const Segment SGI383554[] = {{&S96545[45], 9}, {NULL}};
const Segment SGI383555[] = {{&S96545[61], 13}, {NULL}};
const Segment SGI383556[] = {{&S81320[11], 5}, {NULL}};
const Segment SGI383557[] = {{&S81320[30], 3}, {NULL}};
const Segment SGI383558[] = {{&S96592[7], 9}, {NULL}};
const Segment SGI383559[] = {{&S96592[78], 3}, {NULL}};
const Segment SGI383560[] = {{&S96601[7], 9}, {NULL}};
const Segment SGI383561[] = {{&S96601[72], 3}, {NULL}};
const Segment SGI383562[] = {{&S96604[7], 20}, {NULL}};
const Segment SGI383563[] = {{&S96608[7], 9}, {NULL}};
const Segment SGI383564[] = {{&S96608[72], 3}, {NULL}};
const Segment SGI383565[] = {{&S96611[7], 6}, {NULL}};
const Segment SGI383566[] = {{&S96612[7], 26}, {NULL}};
const Segment SGI383567[] = {{&S96612[11], 22}, {NULL}};
const Segment SGI383568[] = {{&S96665[43], 13}, {NULL}};
const Segment SGI383569[] = {{&S96665[63], 13}, {NULL}};
const Segment SGI383570[] = {{&S96667[43], 13}, {NULL}};
const Segment SGI383571[] = {{&S96667[63], 13}, {NULL}};
const Segment SGI383572[] = {{&S96682[36], 22}, {NULL}};
const Segment SGI383573[] = {{&S96682[86], 13}, {NULL}};
const Segment SGI383574[] = {{&S81515[11], 11}, {NULL}};
const Segment SGI383575[] = {{&S81515[23], 4}, {NULL}};
const Segment SGI383576[] = {{&S81515[51], 3}, {NULL}};
const Segment SGI383577[] = {{&S96863[44], 13}, {NULL}};
const Segment SGI383578[] = {{&S96863[64], 13}, {NULL}};
const Segment SGI383579[] = {{&S96923[7], 9}, {NULL}};
const Segment SGI383580[] = {{&S96923[78], 3}, {NULL}};
const Segment SGI383581[] = {{&S96924[7], 20}, {NULL}};
const Segment SGI383582[] = {{&S96927[7], 9}, {NULL}};
const Segment SGI383583[] = {{&S96927[47], 3}, {NULL}};
const Segment SGI383584[] = {{&S96937[7], 20}, {NULL}};
const Segment SGI383585[] = {{&S97005[7], 9}, {NULL}};
const Segment SGI383586[] = {{&S97017[36], 22}, {NULL}};
const Segment SGI383587[] = {{&S97017[86], 13}, {NULL}};
const Segment SGI383588[] = {{&S97024[7], 10}, {NULL}};
const Segment SGI383589[] = {{&S97070[44], 12}, {NULL}};
const Segment SGI383590[] = {{&S97070[63], 13}, {NULL}};
const Segment SGI383591[] = {{&S97099[36], 23}, {NULL}};
const Segment SGI383592[] = {{&S97099[87], 13}, {NULL}};
const Segment SGI383593[] = {{&S97119[44], 14}, {NULL}};
const Segment SGI383594[] = {{&S97119[65], 13}, {NULL}};
const Segment SGI383595[] = {{&S97120[44], 13}, {NULL}};
const Segment SGI383596[] = {{&S97120[64], 13}, {NULL}};
const Segment SGI383597[] = {{&S97146[51], 20}, {NULL}};
const Segment SGI383598[] = {{&S97211[44], 13}, {NULL}};
const Segment SGI383599[] = {{&S97211[64], 13}, {NULL}};
const Segment SGI383600[] = {{&S97247[7], 9}, {NULL}};
const Segment SGI383601[] = {{&S97247[72], 3}, {NULL}};
const Segment SGI383602[] = {{&S97248[7], 9}, {NULL}};
const Segment SGI383603[] = {{&S97248[41], 3}, {NULL}};
const Segment SGI383604[] = {{&S97264[7], 26}, {NULL}};
const Segment SGI383605[] = {{&S97314[7], 9}, {NULL}};
const Segment SGI383606[] = {{&S82020[11], 6}, {NULL}};
const Segment SGI383607[] = {{&S82020[18], 3}, {NULL}};
const Segment SGI383608[] = {{&S82021[11], 6}, {NULL}};
const Segment SGI383609[] = {{&S82021[18], 3}, {NULL}};
const Segment SGI383610[] = {{&S97412[23], 10}, {NULL}};
const Segment SGI383611[] = {{&S97481[32], 29}, {NULL}};
const Segment SGI383612[] = {{&S97481[48], 8}, {NULL}};
const Segment SGI383613[] = {{&S97485[44], 13}, {NULL}};
const Segment SGI383614[] = {{&S97485[64], 13}, {NULL}};
const Segment SGI383615[] = {{&S82262[11], 5}, {NULL}};
const Segment SGI383616[] = {{&S82346[11], 5}, {NULL}};
const Segment SGI383617[] = {{&S82346[30], 3}, {NULL}};
const Segment SGI383618[] = {{&S97578[44], 14}, {NULL}};
const Segment SGI383619[] = {{&S97578[65], 13}, {NULL}};
const Segment SGI383620[] = {{&S97582[44], 13}, {NULL}};
const Segment SGI383621[] = {{&S97582[64], 13}, {NULL}};
const Segment SGI383622[] = {{&S97623[7], 9}, {NULL}};
const Segment SGI383623[] = {{&S97623[38], 3}, {NULL}};
const Segment SGI383624[] = {{&S82419[11], 5}, {NULL}};
const Segment SGI383625[] = {{&S97633[3], 8}, {NULL}};
const Segment SGI383626[] = {{&S97633[25], 3}, {NULL}};
const Segment SGI383627[] = {{&S97648[3], 9}, {NULL}};
const Segment SGI383628[] = {{&S97649[3], 8}, {NULL}};
const Segment SGI383629[] = {{&S97660[3], 5}, {NULL}};
const Segment SGI383630[] = {{&S97660[21], 3}, {NULL}};
const Segment SGI383631[] = {{&S97680[7], 9}, {NULL}};
const Segment SGI383632[] = {{&S97718[7], 17}, {NULL}};
const Segment SGI383633[] = {{&S82508[11], 12}, {NULL}};
const Segment SGI383634[] = {{&S82508[24], 4}, {NULL}};
const Segment SGI383635[] = {{&S82508[52], 3}, {NULL}};
const Segment SGI383636[] = {{&S97724[43], 13}, {NULL}};
const Segment SGI383637[] = {{&S97724[63], 13}, {NULL}};
const Segment SGI383638[] = {{&S97727[7], 10}, {NULL}};
const Segment SGI383639[] = {{&S82521[11], 5}, {NULL}};
const Segment SGI383640[] = {{&S97742[7], 9}, {NULL}};
const Segment SGI383641[] = {{&S82599[33], 6}, {NULL}};
const Segment SGI383642[] = {{&S82599[40], 3}, {NULL}};
const Segment SGI383643[] = {{&S82599[110], 3}, {NULL}};
const Segment SGI383644[] = {{&S82609[11], 5}, {NULL}};
const Segment SGI383645[] = {{&S82609[39], 3}, {NULL}};
const Segment SGI383646[] = {{&S82612[11], 5}, {NULL}};
const Segment SGI383647[] = {{&S82623[112], 6}, {NULL}};
const Segment SGI383648[] = {{&S82623[182], 3}, {NULL}};
const Segment SGI383649[] = {{&S97871[23], 10}, {NULL}};
const Segment SGI383650[] = {{&S82735[11], 6}, {NULL}};
const Segment SGI383651[] = {{&S82735[31], 3}, {NULL}};
const Segment SGI383652[] = {{&S82736[11], 6}, {NULL}};
const Segment SGI383653[] = {{&S82752[11], 6}, {NULL}};
const Segment SGI383654[] = {{&S98039[7], 9}, {NULL}};
const Segment SGI383655[] = {{&S98039[78], 3}, {NULL}};
const Segment SGI383656[] = {{&S98040[3], 8}, {NULL}};
const Segment SGI383657[] = {{&S98040[25], 3}, {NULL}};
const Segment SGI383658[] = {{&S98041[7], 9}, {NULL}};
const Segment SGI383659[] = {{&S98041[36], 3}, {NULL}};
const Segment SGI383660[] = {{&S98047[3], 8}, {NULL}};
const Segment SGI383661[] = {{&S98048[3], 9}, {NULL}};
const Segment SGI383662[] = {{&S98052[7], 9}, {NULL}};
const Segment SGI383663[] = {{&S98052[41], 3}, {NULL}};
const Segment SGI383664[] = {{&S98053[7], 18}, {NULL}};
const Segment SGI383665[] = {{&S98053[50], 3}, {NULL}};
const Segment SGI383666[] = {{&S98058[7], 9}, {NULL}};
const Segment SGI383667[] = {{&S98058[72], 3}, {NULL}};
const Segment SGI383668[] = {{&S98066[3], 8}, {NULL}};
const Segment SGI383669[] = {{&S98066[25], 3}, {NULL}};
const Segment SGI383670[] = {{&S98077[7], 9}, {NULL}};
const Segment SGI383671[] = {{&S98077[32], 3}, {NULL}};
const Segment SGI383672[] = {{&S98078[3], 12}, {NULL}};
const Segment SGI383673[] = {{&S98078[33], 3}, {NULL}};
const Segment SGI383674[] = {{&S98079[7], 9}, {NULL}};
const Segment SGI383675[] = {{&S98079[72], 3}, {NULL}};
const Segment SGI383676[] = {{&S82820[11], 11}, {NULL}};
const Segment SGI383677[] = {{&S82820[91], 3}, {NULL}};
const Segment SGI383678[] = {{&S82821[11], 5}, {NULL}};
const Segment SGI383679[] = {{&S82821[82], 3}, {NULL}};
const Segment SGI383680[] = {{&S82974[11], 5}, {NULL}};
const Segment SGI383681[] = {{&S82974[39], 3}, {NULL}};
const Segment SGI383682[] = {{&S82974[126], 3}, {NULL}};
const Segment SGI383683[] = {{&S98108[7], 9}, {NULL}};
const Segment SGI383684[] = {{&S83041[11], 5}, {NULL}};
const Segment SGI383685[] = {{&S98131[7], 17}, {NULL}};
const Segment SGI383686[] = {{&S98163[41], 14}, {NULL}};
const Segment SGI383687[] = {{&S98163[62], 13}, {NULL}};
const Segment SGI383688[] = {{&S83089[113], 6}, {NULL}};
const Segment SGI383689[] = {{&S83089[183], 3}, {NULL}};
const Segment SGI383690[] = {{&S98258[43], 13}, {NULL}};
const Segment SGI383691[] = {{&S98258[63], 13}, {NULL}};
const Segment SGI383692[] = {{&S98360[45], 12}, {NULL}};
const Segment SGI383693[] = {{&S98360[64], 13}, {NULL}};
const Segment SGI383694[] = {{&S98516[7], 9}, {NULL}};
const Segment SGI383695[] = {{&S98516[41], 3}, {NULL}};
const Segment SGI383696[] = {{&S98520[3], 5}, {NULL}};
const Segment SGI383697[] = {{&S98521[7], 9}, {NULL}};
const Segment SGI383698[] = {{&S98542[7], 9}, {NULL}};
const Segment SGI383699[] = {{&S98542[41], 3}, {NULL}};
const Segment SGI383700[] = {{&S98548[20], 8}, {NULL}};
const Segment SGI383701[] = {{&S98549[7], 17}, {NULL}};
const Segment SGI383702[] = {{&S98600[36], 29}, {NULL}};
const Segment SGI383703[] = {{&S98634[44], 13}, {NULL}};
const Segment SGI383704[] = {{&S98634[64], 13}, {NULL}};
const Segment SGI383705[] = {{&S98667[36], 22}, {NULL}};
const Segment SGI383706[] = {{&S98667[86], 13}, {NULL}};
const Segment SGI383707[] = {{&S98721[44], 13}, {NULL}};
const Segment SGI383708[] = {{&S98721[64], 13}, {NULL}};
const Segment SGI383709[] = {{&S98770[44], 13}, {NULL}};
const Segment SGI383710[] = {{&S83602[111], 5}, {NULL}};
const Segment SGI383711[] = {{&S98770[64], 13}, {NULL}};
const Segment SGI383712[] = {{&S83602[179], 3}, {NULL}};
const Segment SGI383713[] = {{&S98852[44], 14}, {NULL}};
const Segment SGI383714[] = {{&S98852[65], 13}, {NULL}};
const Segment SGI383715[] = {{&S98946[44], 13}, {NULL}};
const Segment SGI383716[] = {{&S98946[64], 13}, {NULL}};
const Segment SGI383717[] = {{&S98965[36], 22}, {NULL}};
const Segment SGI383718[] = {{&S98965[86], 13}, {NULL}};
const Segment SGI383719[] = {{&S99007[7], 9}, {NULL}};
const Segment SGI383720[] = {{&S99007[36], 3}, {NULL}};
const Segment SGI383721[] = {{&S99020[3], 8}, {NULL}};
const Segment SGI383722[] = {{&S99023[3], 8}, {NULL}};
const Segment SGI383723[] = {{&S99023[25], 3}, {NULL}};
const Segment SGI383724[] = {{&S99024[3], 8}, {NULL}};
const Segment SGI383725[] = {{&S99024[25], 3}, {NULL}};
const Segment SGI383726[] = {{&S99029[3], 9}, {NULL}};
const Segment SGI383727[] = {{&S99029[26], 3}, {NULL}};
const Segment SGI383728[] = {{&S77929[52], 8}, {NULL}};
const Segment SGI383729[] = {{&S77929[40], 3}, {NULL}};
const Segment SGI383730[] = {{&S77929[23], 3}, {NULL}};
const Segment SGI383731[] = {{&S77929[13], 94}, {&S77929[126], 17}, {NULL}};
const Segment SGI383732[] = {{&S77930[52], 8}, {NULL}};
const Segment SGI383733[] = {{&S77930[40], 3}, {NULL}};
const Segment SGI383734[] = {{&S77930[23], 3}, {NULL}};
const Segment SGI383735[] = {{&S77930[13], 94}, {&S77930[126], 17}, {NULL}};
const Segment SGI383736[] = {{&S77931[52], 8}, {NULL}};
const Segment SGI383737[] = {{&S77931[40], 3}, {NULL}};
const Segment SGI383738[] = {{&S77931[23], 3}, {NULL}};
const Segment SGI383739[] = {{&S77931[13], 94}, {&S77931[126], 17}, {NULL}};
const Segment SGI383740[] = {{&S77932[52], 8}, {NULL}};
const Segment SGI383741[] = {{&S77932[40], 3}, {NULL}};
const Segment SGI383742[] = {{&S77932[23], 3}, {NULL}};
const Segment SGI383743[] = {{&S77932[13], 94}, {&S77932[126], 17}, {NULL}};
const Segment SGI383744[] = {{&S77933[52], 8}, {NULL}};
const Segment SGI383745[] = {{&S77933[40], 3}, {NULL}};
const Segment SGI383746[] = {{&S77933[23], 3}, {NULL}};
const Segment SGI383747[] = {{&S77933[13], 94}, {&S77933[126], 17}, {NULL}};
const Segment SGI383748[] = {{&S77934[52], 8}, {NULL}};
const Segment SGI383749[] = {{&S77934[40], 3}, {NULL}};
const Segment SGI383750[] = {{&S77934[23], 3}, {NULL}};
const Segment SGI383751[] = {{&S77934[13], 94}, {&S77934[126], 17}, {NULL}};
const Segment SGI383752[] = {{&S77935[52], 8}, {NULL}};
const Segment SGI383753[] = {{&S77935[40], 3}, {NULL}};
const Segment SGI383754[] = {{&S77935[23], 3}, {NULL}};
const Segment SGI383755[] = {{&S77935[13], 94}, {&S77935[126], 17}, {NULL}};
const Segment SGI383756[] = {{&S77936[52], 8}, {NULL}};
const Segment SGI383757[] = {{&S77936[40], 3}, {NULL}};
const Segment SGI383758[] = {{&S77936[23], 3}, {NULL}};
const Segment SGI383759[] = {{&S77936[13], 94}, {&S77936[126], 17}, {NULL}};
const Segment SGI383760[] = {{&S77937[52], 8}, {NULL}};
const Segment SGI383761[] = {{&S77937[40], 3}, {NULL}};
const Segment SGI383762[] = {{&S77937[23], 3}, {NULL}};
const Segment SGI383763[] = {{&S77937[13], 94}, {&S77937[126], 17}, {NULL}};
const Segment SGI383764[] = {{&S77938[52], 8}, {NULL}};
const Segment SGI383765[] = {{&S77938[40], 3}, {NULL}};
const Segment SGI383766[] = {{&S77938[23], 3}, {NULL}};
const Segment SGI383767[] = {{&S77938[13], 94}, {&S77938[126], 17}, {NULL}};
const Segment SGI383768[] = {{&S99047[7], 10}, {NULL}};
const Segment SGI383769[] = {{&S99047[73], 3}, {NULL}};
const Segment SGI383770[] = {{&S77939[52], 8}, {NULL}};
const Segment SGI383771[] = {{&S77939[40], 3}, {NULL}};
const Segment SGI383772[] = {{&S77939[23], 3}, {NULL}};
const Segment SGI383773[] = {{&S77939[13], 94}, {&S77939[126], 17}, {NULL}};
const Segment SGI383774[] = {{&S77940[52], 8}, {NULL}};
const Segment SGI383775[] = {{&S77940[40], 3}, {NULL}};
const Segment SGI383776[] = {{&S77940[23], 3}, {NULL}};
const Segment SGI383777[] = {{&S77940[13], 94}, {&S77940[126], 17}, {NULL}};
const Segment SGI383778[] = {{&S77941[52], 8}, {NULL}};
const Segment SGI383779[] = {{&S77941[40], 3}, {NULL}};
const Segment SGI383780[] = {{&S77941[23], 3}, {NULL}};
const Segment SGI383781[] = {{&S77941[13], 94}, {&S77941[126], 17}, {NULL}};
const Segment SGI383782[] = {{&S99051[7], 9}, {NULL}};
const Segment SGI383783[] = {{&S99051[41], 3}, {NULL}};
const Segment SGI383784[] = {{&S77942[52], 8}, {NULL}};
const Segment SGI383785[] = {{&S77942[40], 3}, {NULL}};
const Segment SGI383786[] = {{&S77942[23], 3}, {NULL}};
const Segment SGI383787[] = {{&S77942[13], 94}, {&S77942[126], 17}, {NULL}};
const Segment SGI383788[] = {{&S77943[52], 8}, {NULL}};
const Segment SGI383789[] = {{&S77943[40], 3}, {NULL}};
const Segment SGI383790[] = {{&S77943[23], 3}, {NULL}};
const Segment SGI383791[] = {{&S77943[13], 94}, {&S77943[126], 17}, {NULL}};
const Segment SGI383792[] = {{&S99054[7], 9}, {NULL}};
const Segment SGI383793[] = {{&S77944[52], 8}, {NULL}};
const Segment SGI383794[] = {{&S77944[40], 3}, {NULL}};
const Segment SGI383795[] = {{&S77944[23], 3}, {NULL}};
const Segment SGI383796[] = {{&S77944[13], 94}, {&S77944[126], 17}, {NULL}};
const Segment SGI383797[] = {{&S77945[52], 8}, {NULL}};
const Segment SGI383798[] = {{&S77945[40], 3}, {NULL}};
const Segment SGI383799[] = {{&S77945[23], 3}, {NULL}};
const Segment SGI383800[] = {{&S77945[13], 94}, {&S77945[126], 17}, {NULL}};
const Segment SGI383801[] = {{&S77947[52], 8}, {NULL}};
const Segment SGI383802[] = {{&S77947[40], 3}, {NULL}};
const Segment SGI383803[] = {{&S77947[23], 3}, {NULL}};
const Segment SGI383804[] = {{&S77947[13], 94}, {&S77947[126], 17}, {NULL}};
const Segment SGI383805[] = {{&S77948[52], 8}, {NULL}};
const Segment SGI383806[] = {{&S77948[40], 3}, {NULL}};
const Segment SGI383807[] = {{&S77948[23], 3}, {NULL}};
const Segment SGI383808[] = {{&S77948[13], 94}, {&S77948[126], 17}, {NULL}};
const Segment SGI383809[] = {{&S99062[7], 5}, {NULL}};
const Segment SGI383810[] = {{&S77949[52], 8}, {NULL}};
const Segment SGI383811[] = {{&S77949[40], 3}, {NULL}};
const Segment SGI383812[] = {{&S77949[23], 3}, {NULL}};
const Segment SGI383813[] = {{&S77949[13], 94}, {&S77949[126], 17}, {NULL}};
const Segment SGI383814[] = {{&S77950[52], 8}, {NULL}};
const Segment SGI383815[] = {{&S77950[40], 3}, {NULL}};
const Segment SGI383816[] = {{&S77950[23], 3}, {NULL}};
const Segment SGI383817[] = {{&S77950[13], 94}, {&S77950[126], 17}, {NULL}};
const Segment SGI383818[] = {{&S77951[52], 8}, {NULL}};
const Segment SGI383819[] = {{&S77951[40], 3}, {NULL}};
const Segment SGI383820[] = {{&S77951[23], 3}, {NULL}};
const Segment SGI383821[] = {{&S77951[13], 94}, {&S77951[126], 17}, {NULL}};
const Segment SGI383822[] = {{&S77952[52], 8}, {NULL}};
const Segment SGI383823[] = {{&S77952[40], 3}, {NULL}};
const Segment SGI383824[] = {{&S77952[23], 3}, {NULL}};
const Segment SGI383825[] = {{&S77952[13], 94}, {&S77952[126], 17}, {NULL}};
const Segment SGI383826[] = {{&S77953[52], 8}, {NULL}};
const Segment SGI383827[] = {{&S77953[40], 3}, {NULL}};
const Segment SGI383828[] = {{&S77953[23], 3}, {NULL}};
const Segment SGI383829[] = {{&S77953[13], 94}, {&S77953[126], 17}, {NULL}};
const Segment SGI383830[] = {{&S77954[52], 8}, {NULL}};
const Segment SGI383831[] = {{&S77954[40], 3}, {NULL}};
const Segment SGI383832[] = {{&S77954[23], 3}, {NULL}};
const Segment SGI383833[] = {{&S77954[13], 94}, {&S77954[126], 17}, {NULL}};
const Segment SGI383834[] = {{&S77955[52], 8}, {NULL}};
const Segment SGI383835[] = {{&S77955[40], 3}, {NULL}};
const Segment SGI383836[] = {{&S77955[23], 3}, {NULL}};
const Segment SGI383837[] = {{&S77955[13], 94}, {&S77955[126], 17}, {NULL}};
const Segment SGI383838[] = {{&S99087[36], 29}, {NULL}};
const Segment SGI383839[] = {{&S99101[7], 9}, {NULL}};
const Segment SGI383840[] = {{&S99118[7], 9}, {NULL}};
const Segment SGI383841[] = {{&S99122[43], 13}, {NULL}};
const Segment SGI383842[] = {{&S99122[63], 13}, {NULL}};
const Segment SGI383843[] = {{&S99131[36], 29}, {NULL}};
const Segment SGI383844[] = {{&S99134[36], 30}, {NULL}};
const Segment SGI383845[] = {{&S99175[23], 10}, {NULL}};
const Segment SGI383846[] = {{&S99197[44], 14}, {NULL}};
const Segment SGI383847[] = {{&S99197[65], 13}, {NULL}};
const Segment SGI383848[] = {{&S99254[36], 22}, {NULL}};
const Segment SGI383849[] = {{&S99254[86], 13}, {NULL}};
const Segment SGI383850[] = {{&S83994[11], 5}, {NULL}};
const Segment SGI383851[] = {{&S99288[33], 23}, {NULL}};
const Segment SGI383852[] = {{&S99288[84], 13}, {NULL}};
const Segment SGI383853[] = {{&S99312[7], 9}, {NULL}};
const Segment SGI383854[] = {{&S99541[7], 16}, {NULL}};
const Segment SGI383855[] = {{&S99541[85], 3}, {NULL}};
const Segment SGI383856[] = {{&S99542[7], 9}, {NULL}};
const Segment SGI383857[] = {{&S99542[78], 3}, {NULL}};
const Segment SGI383858[] = {{&S99543[7], 7}, {NULL}};
const Segment SGI383859[] = {{&S99543[39], 3}, {NULL}};
const Segment SGI383860[] = {{&S99545[7], 9}, {NULL}};
const Segment SGI383861[] = {{&S99545[72], 3}, {NULL}};
const Segment SGI383862[] = {{&S99546[7], 10}, {NULL}};
const Segment SGI383863[] = {{&S99546[73], 3}, {NULL}};
const Segment SGI383864[] = {{&S99550[7], 9}, {NULL}};
const Segment SGI383865[] = {{&S99550[41], 3}, {NULL}};
const Segment SGI383866[] = {{&S99556[7], 6}, {NULL}};
const Segment SGI383867[] = {{&S99559[3], 8}, {NULL}};
const Segment SGI383868[] = {{&S99562[7], 9}, {NULL}};
const Segment SGI383869[] = {{&S99562[32], 3}, {NULL}};
const Segment SGI383870[] = {{&S99570[3], 5}, {NULL}};
const Segment SGI383871[] = {{&S99570[22], 10}, {NULL}};
const Segment SGI383872[] = {{&S99576[3], 8}, {NULL}};
const Segment SGI383873[] = {{&S99595[7], 10}, {NULL}};
const Segment SGI383874[] = {{&S99596[7], 10}, {NULL}};
const Segment SGI383875[] = {{&S99641[36], 29}, {NULL}};
const Segment SGI383876[] = {{&S99656[36], 30}, {NULL}};
const Segment SGI383877[] = {{&S99664[36], 29}, {NULL}};
const Segment SGI383878[] = {{&S99665[36], 29}, {NULL}};
const Segment SGI383879[] = {{&S99675[36], 30}, {NULL}};
const Segment SGI383880[] = {{&S99755[7], 9}, {NULL}};
const Segment SGI383881[] = {{&S99770[36], 22}, {NULL}};
const Segment SGI383882[] = {{&S99770[86], 13}, {NULL}};
const Segment SGI383883[] = {{&S84684[11], 5}, {NULL}};
const Segment SGI383884[] = {{&S99850[7], 9}, {NULL}};
const Segment SGI383885[] = {{&S100037[46], 13}, {NULL}};
const Segment SGI383886[] = {{&S100037[66], 13}, {NULL}};
const Segment SGI383887[] = {{&S100069[42], 9}, {NULL}};
const Segment SGI383888[] = {{&S100069[58], 13}, {NULL}};
const Segment SGI383889[] = {{&S85097[27], 5}, {NULL}};
const Segment SGI383890[] = {{&S100124[7], 9}, {NULL}};
const Segment SGI383891[] = {{&S100124[78], 3}, {NULL}};
const Segment SGI383892[] = {{&S100127[7], 26}, {NULL}};
const Segment SGI383893[] = {{&S100127[11], 22}, {NULL}};
const Segment SGI383894[] = {{&S100128[7], 9}, {NULL}};
const Segment SGI383895[] = {{&S100128[43], 3}, {NULL}};
const Segment SGI383896[] = {{&S100140[7], 6}, {NULL}};
const Segment SGI383897[] = {{&S100140[69], 3}, {NULL}};
const Segment SGI383898[] = {{&S100149[7], 9}, {NULL}};
const Segment SGI383899[] = {{&S100154[3], 8}, {NULL}};
const Segment SGI383900[] = {{&S85142[11], 6}, {NULL}};
const Segment SGI383901[] = {{&S85143[11], 6}, {NULL}};
const Segment SGI383902[] = {{&S85143[38], 3}, {NULL}};
const Segment SGI383903[] = {{&S100173[7], 9}, {NULL}};
const Segment SGI383904[] = {{&S100173[17], 8}, {NULL}};
const Segment SGI383905[] = {{&S100181[7], 9}, {NULL}};
const Segment SGI383906[] = {{&S100186[7], 9}, {NULL}};
const Segment SGI383907[] = {{&S100210[7], 9}, {NULL}};
const Segment SGI383908[] = {{&S100216[36], 29}, {NULL}};
const Segment SGI383909[] = {{&S100227[7], 9}, {NULL}};
const Segment SGI383910[] = {{&S100228[7], 9}, {NULL}};
const Segment SGI383911[] = {{&S100231[7], 9}, {NULL}};
const Segment SGI383912[] = {{&S100231[17], 8}, {NULL}};
const Segment SGI383913[] = {{&S100263[7], 9}, {NULL}};
const Segment SGI383914[] = {{&S85243[11], 5}, {NULL}};
const Segment SGI383915[] = {{&S85243[17], 3}, {NULL}};
const Segment SGI383916[] = {{&S100267[7], 9}, {NULL}};
const Segment SGI383917[] = {{&S100273[44], 13}, {NULL}};
const Segment SGI383918[] = {{&S100273[64], 13}, {NULL}};
const Segment SGI383919[] = {{&S100274[44], 13}, {NULL}};
const Segment SGI383920[] = {{&S100274[64], 13}, {NULL}};
const Segment SGI383921[] = {{&S100275[51], 18}, {NULL}};
const Segment SGI383922[] = {{&S100311[7], 9}, {NULL}};
const Segment SGI383923[] = {{&S100315[7], 9}, {NULL}};
const Segment SGI383924[] = {{&S100380[7], 10}, {NULL}};
const Segment SGI383925[] = {{&S100380[35], 3}, {NULL}};
const Segment SGI383926[] = {{&S100419[44], 13}, {NULL}};
const Segment SGI383927[] = {{&S100419[64], 13}, {NULL}};
const Segment SGI383928[] = {{&S100456[44], 13}, {NULL}};
const Segment SGI383929[] = {{&S100456[64], 13}, {NULL}};
const Segment SGI383930[] = {{&S100458[7], 10}, {NULL}};
const Segment SGI383931[] = {{&S100599[7], 9}, {NULL}};
const Segment SGI383932[] = {{&S100599[11], 5}, {NULL}};
const Segment SGI383933[] = {{&S85425[11], 5}, {NULL}};
const Segment SGI383934[] = {{&S85425[72], 3}, {NULL}};
const Segment SGI383935[] = {{&S85431[33], 6}, {NULL}};
const Segment SGI383936[] = {{&S85431[40], 3}, {NULL}};
const Segment SGI383937[] = {{&S85431[110], 3}, {NULL}};
const Segment SGI383938[] = {{&S100720[7], 9}, {NULL}};
const Segment SGI383939[] = {{&S100720[101], 3}, {NULL}};
const Segment SGI383940[] = {{&S100722[3], 8}, {NULL}};
const Segment SGI383941[] = {{&S100722[25], 3}, {NULL}};
const Segment SGI383942[] = {{&S100725[3], 8}, {NULL}};
const Segment SGI383943[] = {{&S100725[25], 3}, {NULL}};
const Segment SGI383944[] = {{&S100726[7], 9}, {NULL}};
const Segment SGI383945[] = {{&S100726[27], 3}, {NULL}};
const Segment SGI383946[] = {{&S100727[7], 9}, {NULL}};
const Segment SGI383947[] = {{&S100727[72], 3}, {NULL}};
const Segment SGI383948[] = {{&S100728[3], 8}, {NULL}};
const Segment SGI383949[] = {{&S100736[3], 12}, {NULL}};
const Segment SGI383950[] = {{&S100737[3], 5}, {NULL}};
const Segment SGI383951[] = {{&S100741[3], 8}, {NULL}};
const Segment SGI383952[] = {{&S100742[3], 9}, {NULL}};
const Segment SGI383953[] = {{&S100767[3], 8}, {NULL}};
const Segment SGI383954[] = {{&S100767[25], 3}, {NULL}};
const Segment SGI383955[] = {{&S100767[75], 4}, {NULL}};
const Segment SGI383956[] = {{&S100767[42], 3}, {NULL}};
const Segment SGI383957[] = {{&S100775[7], 9}, {NULL}};
const Segment SGI383958[] = {{&S100775[17], 8}, {NULL}};
const Segment SGI383959[] = {{&S100776[7], 9}, {NULL}};
const Segment SGI383960[] = {{&S100786[7], 9}, {NULL}};
const Segment SGI383961[] = {{&S100786[17], 8}, {NULL}};
const Segment SGI383962[] = {{&S100870[44], 10}, {NULL}};
const Segment SGI383963[] = {{&S100870[61], 13}, {NULL}};
const Segment SGI383964[] = {{&S100875[7], 9}, {NULL}};
const Segment SGI383965[] = {{&S100880[36], 29}, {NULL}};
const Segment SGI383966[] = {{&S100893[36], 23}, {NULL}};
const Segment SGI383967[] = {{&S100893[87], 13}, {NULL}};
const Segment SGI383968[] = {{&S100896[37], 18}, {NULL}};
const Segment SGI383969[] = {{&S100896[83], 13}, {NULL}};
const Segment SGI383970[] = {{&S85531[11], 6}, {NULL}};
const Segment SGI383971[] = {{&S85531[31], 3}, {NULL}};
const Segment SGI383972[] = {{&S100939[44], 13}, {NULL}};
const Segment SGI383973[] = {{&S100939[64], 13}, {NULL}};
const Segment SGI383974[] = {{&S100946[7], 9}, {NULL}};
const Segment SGI383975[] = {{&S100946[72], 3}, {NULL}};
const Segment SGI383976[] = {{&S100955[7], 9}, {NULL}};
const Segment SGI383977[] = {{&S100955[38], 3}, {NULL}};
const Segment SGI383978[] = {{&S100960[7], 9}, {NULL}};
const Segment SGI383979[] = {{&S100960[32], 3}, {NULL}};
const Segment SGI383980[] = {{&S100967[7], 9}, {NULL}};
const Segment SGI383981[] = {{&S100967[41], 3}, {NULL}};
const Segment SGI383982[] = {{&S100976[7], 9}, {NULL}};
const Segment SGI383983[] = {{&S100976[47], 3}, {NULL}};
const Segment SGI383984[] = {{&S100977[7], 6}, {NULL}};
const Segment SGI383985[] = {{&S100977[69], 3}, {NULL}};
const Segment SGI383986[] = {{&S100983[3], 8}, {NULL}};
const Segment SGI383987[] = {{&S100984[7], 9}, {NULL}};
const Segment SGI383988[] = {{&S100984[69], 3}, {NULL}};
const Segment SGI383989[] = {{&S100997[7], 9}, {NULL}};
const Segment SGI383990[] = {{&S101007[7], 9}, {NULL}};
const Segment SGI383991[] = {{&S101008[32], 30}, {NULL}};
const Segment SGI383992[] = {{&S101008[48], 8}, {NULL}};
const Segment SGI383993[] = {{&S101019[7], 10}, {NULL}};
const Segment SGI383994[] = {{&S101046[7], 9}, {NULL}};
const Segment SGI383995[] = {{&S101054[43], 13}, {NULL}};
const Segment SGI383996[] = {{&S101054[63], 13}, {NULL}};
const Segment SGI383997[] = {{&S101057[43], 13}, {NULL}};
const Segment SGI383998[] = {{&S101057[63], 13}, {NULL}};
const Segment SGI383999[] = {{&S101083[7], 10}, {NULL}};
const Segment SGI384000[] = {{&S85696[11], 5}, {NULL}};
const Segment SGI384001[] = {{&S85763[11], 5}, {NULL}};
const Segment SGI384002[] = {{&S85763[72], 3}, {NULL}};
const Segment SGI384003[] = {{&S101152[7], 10}, {NULL}};
const Segment SGI384004[] = {{&S101152[73], 3}, {NULL}};
const Segment SGI384005[] = {{&S101155[7], 9}, {NULL}};
const Segment SGI384006[] = {{&S101155[41], 3}, {NULL}};
const Segment SGI384007[] = {{&S85770[11], 6}, {NULL}};
const Segment SGI384008[] = {{&S85771[11], 6}, {NULL}};
const Segment SGI384009[] = {{&S101159[16], 5}, {NULL}};
const Segment SGI384010[] = {{&S101162[7], 9}, {NULL}};
const Segment SGI384011[] = {{&S101167[3], 8}, {NULL}};
const Segment SGI384012[] = {{&S101167[25], 3}, {NULL}};
const Segment SGI384013[] = {{&S101174[7], 9}, {NULL}};
const Segment SGI384014[] = {{&S101174[72], 3}, {NULL}};
const Segment SGI384015[] = {{&S101179[7], 9}, {NULL}};
const Segment SGI384016[] = {{&S101179[32], 3}, {NULL}};
const Segment SGI384017[] = {{&S85801[11], 5}, {NULL}};
const Segment SGI384018[] = {{&S101231[43], 13}, {NULL}};
const Segment SGI384019[] = {{&S101231[63], 13}, {NULL}};
const Segment SGI384020[] = {{&S101232[43], 13}, {NULL}};
const Segment SGI384021[] = {{&S101232[63], 13}, {NULL}};
const Segment SGI384022[] = {{&S101283[7], 9}, {NULL}};
const Segment SGI384023[] = {{&S101283[41], 3}, {NULL}};
const Segment SGI384024[] = {{&S101287[7], 9}, {NULL}};
const Segment SGI384025[] = {{&S101287[72], 3}, {NULL}};
const Segment SGI384026[] = {{&S101290[7], 9}, {NULL}};
const Segment SGI384027[] = {{&S101290[38], 3}, {NULL}};
const Segment SGI384028[] = {{&S101292[7], 9}, {NULL}};
const Segment SGI384029[] = {{&S101292[72], 3}, {NULL}};
const Segment SGI384030[] = {{&S101293[3], 8}, {NULL}};
const Segment SGI384031[] = {{&S101293[25], 3}, {NULL}};
const Segment SGI384032[] = {{&S101304[7], 9}, {NULL}};
const Segment SGI384033[] = {{&S101304[72], 3}, {NULL}};
const Segment SGI384034[] = {{&S85933[33], 6}, {NULL}};
const Segment SGI384035[] = {{&S85933[53], 3}, {NULL}};
const Segment SGI384036[] = {{&S85933[108], 3}, {NULL}};
const Segment SGI384037[] = {{&S101320[7], 16}, {NULL}};
const Segment SGI384038[] = {{&S101320[85], 3}, {NULL}};
const Segment SGI384039[] = {{&S101417[44], 14}, {NULL}};
const Segment SGI384040[] = {{&S101417[65], 13}, {NULL}};
const Segment SGI384041[] = {{&S101428[7], 9}, {NULL}};
const Segment SGI384042[] = {{&S101430[7], 9}, {NULL}};
const Segment SGI384043[] = {{&S101430[78], 3}, {NULL}};
const Segment SGI384044[] = {{&S101431[7], 10}, {NULL}};
const Segment SGI384045[] = {{&S101431[79], 3}, {NULL}};
const Segment SGI384046[] = {{&S101445[7], 9}, {NULL}};
const Segment SGI384047[] = {{&S101445[72], 3}, {NULL}};
const Segment SGI384048[] = {{&S101454[7], 9}, {NULL}};
const Segment SGI384049[] = {{&S101454[69], 3}, {NULL}};
const Segment SGI384050[] = {{&S101456[7], 10}, {NULL}};
const Segment SGI384051[] = {{&S101456[42], 3}, {NULL}};
const Segment SGI384052[] = {{&S101460[7], 15}, {NULL}};
const Segment SGI384053[] = {{&S101461[3], 5}, {NULL}};
const Segment SGI384054[] = {{&S101462[16], 5}, {NULL}};
const Segment SGI384055[] = {{&S101464[7], 9}, {NULL}};
const Segment SGI384056[] = {{&S80189[52], 8}, {NULL}};
const Segment SGI384057[] = {{&S80189[40], 3}, {NULL}};
const Segment SGI384058[] = {{&S80189[23], 3}, {NULL}};
const Segment SGI384059[] = {{&S80189[13], 94}, {&S80189[126], 17}, {NULL}};
const Segment SGI384060[] = {{&S80190[49], 8}, {NULL}};
const Segment SGI384061[] = {{&S80190[37], 3}, {NULL}};
const Segment SGI384062[] = {{&S80190[23], 3}, {NULL}};
const Segment SGI384063[] = {{&S80190[13], 94}, {&S80190[126], 17}, {NULL}};
const Segment SGI384064[] = {{&S80191[49], 8}, {NULL}};
const Segment SGI384065[] = {{&S80191[37], 3}, {NULL}};
const Segment SGI384066[] = {{&S80191[23], 3}, {NULL}};
const Segment SGI384067[] = {{&S80191[13], 94}, {&S80191[126], 17}, {NULL}};
const Segment SGI384068[] = {{&S80192[49], 8}, {NULL}};
const Segment SGI384069[] = {{&S80192[37], 3}, {NULL}};
const Segment SGI384070[] = {{&S80192[23], 3}, {NULL}};
const Segment SGI384071[] = {{&S80192[13], 94}, {&S80192[126], 17}, {NULL}};
const Segment SGI384072[] = {{&S80193[49], 8}, {NULL}};
const Segment SGI384073[] = {{&S80193[37], 3}, {NULL}};
const Segment SGI384074[] = {{&S80193[23], 3}, {NULL}};
const Segment SGI384075[] = {{&S80193[13], 94}, {&S80193[126], 17}, {NULL}};
const Segment SGI384076[] = {{&S80194[49], 8}, {NULL}};
const Segment SGI384077[] = {{&S80194[37], 3}, {NULL}};
const Segment SGI384078[] = {{&S80194[23], 3}, {NULL}};
const Segment SGI384079[] = {{&S80194[13], 94}, {&S80194[126], 17}, {NULL}};
const Segment SGI384080[] = {{&S80195[49], 8}, {NULL}};
const Segment SGI384081[] = {{&S80195[37], 3}, {NULL}};
const Segment SGI384082[] = {{&S80195[23], 3}, {NULL}};
const Segment SGI384083[] = {{&S80195[13], 94}, {&S80195[126], 17}, {NULL}};
const Segment SGI384084[] = {{&S80196[49], 8}, {NULL}};
const Segment SGI384085[] = {{&S80196[37], 3}, {NULL}};
const Segment SGI384086[] = {{&S80196[23], 3}, {NULL}};
const Segment SGI384087[] = {{&S80196[13], 94}, {&S80196[126], 17}, {NULL}};
const Segment SGI384088[] = {{&S101537[37], 6}, {NULL}};
const Segment SGI384089[] = {{&S101552[41], 14}, {NULL}};
const Segment SGI384090[] = {{&S101552[62], 13}, {NULL}};
const Segment SGI384091[] = {{&S86106[11], 5}, {NULL}};
const Segment SGI384092[] = {{&S86106[30], 3}, {NULL}};
const Segment SGI384093[] = {{&S101605[7], 9}, {NULL}};
const Segment SGI384094[] = {{&S101622[7], 20}, {NULL}};
const Segment SGI384095[] = {{&S101623[7], 15}, {NULL}};
const Segment SGI384096[] = {{&S101626[7], 9}, {NULL}};
const Segment SGI384097[] = {{&S101627[7], 10}, {NULL}};
const Segment SGI384098[] = {{&S101634[3], 5}, {NULL}};
const Segment SGI384099[] = {{&S101634[22], 11}, {NULL}};
const Segment SGI384100[] = {{&S101640[3], 8}, {NULL}};
const Segment SGI384101[] = {{&S101640[25], 3}, {NULL}};
const Segment SGI384102[] = {{&S101658[7], 6}, {NULL}};
const Segment SGI384103[] = {{&S101659[7], 10}, {NULL}};
const Segment SGI384104[] = {{&S101659[73], 3}, {NULL}};
const Segment SGI384105[] = {{&S101680[7], 9}, {NULL}};
const Segment SGI384106[] = {{&S86217[11], 5}, {NULL}};
const Segment SGI384107[] = {{&S86217[30], 3}, {NULL}};
const Segment SGI384108[] = {{&S101790[68], 16}, {NULL}};
const Segment SGI384109[] = {{&S101791[7], 9}, {NULL}};
const Segment SGI384110[] = {{&S101792[16], 5}, {NULL}};
const Segment SGI384111[] = {{&S101798[82], 8}, {NULL}};
const Segment SGI384112[] = {{&S101799[65], 8}, {NULL}};
const Segment SGI384113[] = {{&S101800[82], 8}, {NULL}};
const Segment SGI384114[] = {{&S101803[3], 8}, {NULL}};
const Segment SGI384115[] = {{&S101804[3], 8}, {NULL}};
const Segment SGI384116[] = {{&S101809[7], 9}, {NULL}};
const Segment SGI384117[] = {{&S101809[72], 3}, {NULL}};
const Segment SGI384118[] = {{&S101810[7], 16}, {NULL}};
const Segment SGI384119[] = {{&S101810[79], 3}, {NULL}};
const Segment SGI384120[] = {{&S101812[7], 9}, {NULL}};
const Segment SGI384121[] = {{&S101812[72], 3}, {NULL}};
const Segment SGI384122[] = {{&S101817[3], 16}, {NULL}};
const Segment SGI384123[] = {{&S101817[20], 3}, {NULL}};
const Segment SGI384124[] = {{&S101818[7], 9}, {NULL}};
const Segment SGI384125[] = {{&S101818[72], 3}, {NULL}};
const Segment SGI384126[] = {{&S101826[20], 8}, {NULL}};
const Segment SGI384127[] = {{&S101826[56], 3}, {NULL}};
const Segment SGI384128[] = {{&S101827[20], 8}, {NULL}};
const Segment SGI384129[] = {{&S101827[56], 3}, {NULL}};
const Segment SGI384130[] = {{&S86352[27], 6}, {NULL}};
const Segment SGI384131[] = {{&S101874[25], 10}, {NULL}};
const Segment SGI384132[] = {{&S101913[44], 13}, {NULL}};
const Segment SGI384133[] = {{&S101913[64], 13}, {NULL}};
const Segment SGI384134[] = {{&S101954[7], 6}, {NULL}};
const Segment SGI384135[] = {{&S101954[34], 3}, {NULL}};
const Segment SGI384136[] = {{&S101965[16], 8}, {NULL}};
const Segment SGI384137[] = {{&S101966[16], 8}, {NULL}};
const Segment SGI384138[] = {{&S101967[16], 8}, {NULL}};
const Segment SGI384139[] = {{&S101968[16], 8}, {NULL}};
const Segment SGI384140[] = {{&S101969[16], 8}, {NULL}};
const Segment SGI384141[] = {{&S101976[68], 9}, {NULL}};
const Segment SGI384142[] = {{&S101980[7], 9}, {NULL}};
const Segment SGI384143[] = {{&S101982[7], 10}, {NULL}};
const Segment SGI384144[] = {{&S101982[74], 3}, {NULL}};
const Segment SGI384145[] = {{&S101983[7], 15}, {NULL}};
const Segment SGI384146[] = {{&S101983[79], 3}, {NULL}};
const Segment SGI384147[] = {{&S101985[7], 9}, {NULL}};
const Segment SGI384148[] = {{&S101985[69], 3}, {NULL}};
const Segment SGI384149[] = {{&S101992[16], 5}, {NULL}};
const Segment SGI384150[] = {{&S102004[7], 10}, {NULL}};
const Segment SGI384151[] = {{&S102007[7], 9}, {NULL}};
const Segment SGI384152[] = {{&S102007[76], 3}, {NULL}};
const Segment SGI384153[] = {{&S102009[7], 18}, {NULL}};
const Segment SGI384154[] = {{&S102009[50], 3}, {NULL}};
const Segment SGI384155[] = {{&S102036[7], 6}, {NULL}};
const Segment SGI384156[] = {{&S102083[7], 12}, {NULL}};
const Segment SGI384157[] = {{&S102086[43], 14}, {NULL}};
const Segment SGI384158[] = {{&S102086[64], 13}, {NULL}};
const Segment SGI384159[] = {{&S102088[44], 13}, {NULL}};
const Segment SGI384160[] = {{&S102088[64], 13}, {NULL}};
const Segment SGI384161[] = {{&S86606[111], 5}, {NULL}};
const Segment SGI384162[] = {{&S86606[179], 3}, {NULL}};
const Segment SGI384163[] = {{&S102161[3], 16}, {NULL}};
const Segment SGI384164[] = {{&S102161[20], 3}, {NULL}};
const Segment SGI384165[] = {{&S102162[7], 9}, {NULL}};
const Segment SGI384166[] = {{&S102164[7], 9}, {NULL}};
const Segment SGI384167[] = {{&S102164[72], 3}, {NULL}};
const Segment SGI384168[] = {{&S102165[3], 8}, {NULL}};
const Segment SGI384169[] = {{&S102165[25], 3}, {NULL}};
const Segment SGI384170[] = {{&S102173[7], 17}, {NULL}};
const Segment SGI384171[] = {{&S102173[11], 13}, {NULL}};
const Segment SGI384172[] = {{&S102177[7], 7}, {NULL}};
const Segment SGI384173[] = {{&S102197[7], 10}, {NULL}};
const Segment SGI384174[] = {{&S102199[3], 8}, {NULL}};
const Segment SGI384175[] = {{&S102200[7], 16}, {NULL}};
const Segment SGI384176[] = {{&S102200[85], 3}, {NULL}};
const Segment SGI384177[] = {{&S102204[7], 16}, {NULL}};
const Segment SGI384178[] = {{&S102204[50], 3}, {NULL}};
const Segment SGI384179[] = {{&S102211[20], 8}, {NULL}};
const Segment SGI384180[] = {{&S102212[20], 8}, {NULL}};
const Segment SGI384181[] = {{&S102215[3], 5}, {NULL}};
const Segment SGI384182[] = {{&S102216[7], 8}, {NULL}};
const Segment SGI384183[] = {{&S102232[7], 9}, {NULL}};
const Segment SGI384184[] = {{&S102270[44], 13}, {NULL}};
const Segment SGI384185[] = {{&S102270[64], 13}, {NULL}};
const Segment SGI384186[] = {{&S102271[43], 13}, {NULL}};
const Segment SGI384187[] = {{&S102271[63], 13}, {NULL}};
const Segment SGI384188[] = {{&S102273[51], 21}, {NULL}};
const Segment SGI384189[] = {{&S102314[44], 13}, {NULL}};
const Segment SGI384190[] = {{&S102314[64], 13}, {NULL}};
const Segment SGI384191[] = {{&S102333[7], 10}, {NULL}};
const Segment SGI384192[] = {{&S102333[74], 3}, {NULL}};
const Segment SGI384193[] = {{&S102335[7], 10}, {NULL}};
const Segment SGI384194[] = {{&S102335[49], 3}, {NULL}};
const Segment SGI384195[] = {{&S102345[3], 8}, {NULL}};
const Segment SGI384196[] = {{&S102346[7], 9}, {NULL}};
const Segment SGI384197[] = {{&S102351[7], 9}, {NULL}};
const Segment SGI384198[] = {{&S102351[41], 3}, {NULL}};
const Segment SGI384199[] = {{&S102354[7], 6}, {NULL}};
const Segment SGI384200[] = {{&S102354[42], 3}, {NULL}};
const Segment SGI384201[] = {{&S102361[7], 9}, {NULL}};
const Segment SGI384202[] = {{&S102361[36], 3}, {NULL}};
const Segment SGI384203[] = {{&S86779[11], 5}, {NULL}};
const Segment SGI384204[] = {{&S86779[30], 3}, {NULL}};
const Segment SGI384205[] = {{&S86780[11], 6}, {NULL}};
const Segment SGI384206[] = {{&S102418[7], 9}, {NULL}};
const Segment SGI384207[] = {{&S102432[7], 9}, {NULL}};
const Segment SGI384208[] = {{&S102434[7], 9}, {NULL}};
const Segment SGI384209[] = {{&S102437[7], 9}, {NULL}};
const Segment SGI384210[] = {{&S102446[7], 9}, {NULL}};
const Segment SGI384211[] = {{&S102446[17], 8}, {NULL}};
const Segment SGI384212[] = {{&S102464[20], 8}, {NULL}};
const Segment SGI384213[] = {{&S102464[56], 3}, {NULL}};
const Segment SGI384214[] = {{&S86892[11], 6}, {NULL}};
const Segment SGI384215[] = {{&S86892[40], 3}, {NULL}};
const Segment SGI384216[] = {{&S86893[11], 5}, {NULL}};
const Segment SGI384217[] = {{&S86893[46], 3}, {NULL}};
const Segment SGI384218[] = {{&S102492[43], 13}, {NULL}};
const Segment SGI384219[] = {{&S102492[63], 13}, {NULL}};
const Segment SGI384220[] = {{&S102493[40], 14}, {NULL}};
const Segment SGI384221[] = {{&S102493[61], 13}, {NULL}};
const Segment SGI384222[] = {{&S102494[43], 13}, {NULL}};
const Segment SGI384223[] = {{&S102494[63], 13}, {NULL}};
const Segment SGI384224[] = {{&S102537[44], 13}, {NULL}};
const Segment SGI384225[] = {{&S102537[64], 13}, {NULL}};
const Segment SGI384226[] = {{&S102540[44], 13}, {NULL}};
const Segment SGI384227[] = {{&S102540[64], 13}, {NULL}};
const Segment SGI384228[] = {{&S102541[44], 13}, {NULL}};
const Segment SGI384229[] = {{&S102541[64], 13}, {NULL}};
const Segment SGI384230[] = {{&S102549[7], 9}, {NULL}};
const Segment SGI384231[] = {{&S102549[78], 3}, {NULL}};
const Segment SGI384232[] = {{&S102562[7], 16}, {NULL}};
const Segment SGI384233[] = {{&S102565[7], 9}, {NULL}};
const Segment SGI384234[] = {{&S102565[41], 3}, {NULL}};
const Segment SGI384235[] = {{&S102569[3], 8}, {NULL}};
const Segment SGI384236[] = {{&S102570[3], 8}, {NULL}};
const Segment SGI384237[] = {{&S102576[7], 9}, {NULL}};
const Segment SGI384238[] = {{&S102585[3], 8}, {NULL}};
const Segment SGI384239[] = {{&S102588[3], 8}, {NULL}};
const Segment SGI384240[] = {{&S102593[7], 6}, {NULL}};
const Segment SGI384241[] = {{&S102597[7], 24}, {NULL}};
const Segment SGI384242[] = {{&S102597[32], 5}, {NULL}};
const Segment SGI384243[] = {{&S102598[7], 10}, {NULL}};
const Segment SGI384244[] = {{&S102598[33], 3}, {NULL}};
const Segment SGI384245[] = {{&S102616[3], 13}, {NULL}};
const Segment SGI384246[] = {{&S102617[3], 8}, {NULL}};
const Segment SGI384247[] = {{&S102618[3], 13}, {NULL}};
const Segment SGI384248[] = {{&S102619[3], 5}, {NULL}};
const Segment SGI384249[] = {{&S102619[21], 3}, {NULL}};
const Segment SGI384250[] = {{&S102620[3], 16}, {NULL}};
const Segment SGI384251[] = {{&S102620[20], 3}, {NULL}};
const Segment SGI384252[] = {{&S102621[7], 17}, {NULL}};
const Segment SGI384253[] = {{&S102686[43], 13}, {NULL}};
const Segment SGI384254[] = {{&S102686[63], 13}, {NULL}};
const Segment SGI384255[] = {{&S102687[43], 13}, {NULL}};
const Segment SGI384256[] = {{&S102687[63], 13}, {NULL}};
const Segment SGI384257[] = {{&S87099[11], 5}, {NULL}};
const Segment SGI384258[] = {{&S102688[43], 13}, {NULL}};
const Segment SGI384259[] = {{&S102688[63], 13}, {NULL}};
const Segment SGI384260[] = {{&S102717[41], 13}, {NULL}};
const Segment SGI384261[] = {{&S102717[61], 13}, {NULL}};
const Segment SGI384262[] = {{&S102718[44], 13}, {NULL}};
const Segment SGI384263[] = {{&S102718[64], 13}, {NULL}};
const Segment SGI384264[] = {{&S102720[44], 13}, {NULL}};
const Segment SGI384265[] = {{&S102720[64], 13}, {NULL}};
const Segment SGI384266[] = {{&S102727[44], 13}, {NULL}};
const Segment SGI384267[] = {{&S102727[64], 13}, {NULL}};
const Segment SGI384268[] = {{&S102740[7], 6}, {NULL}};
const Segment SGI384269[] = {{&S102740[27], 3}, {NULL}};
const Segment SGI384270[] = {{&S102743[7], 9}, {NULL}};
const Segment SGI384271[] = {{&S102756[7], 9}, {NULL}};
const Segment SGI384272[] = {{&S102756[34], 3}, {NULL}};
const Segment SGI384273[] = {{&S102757[7], 9}, {NULL}};
const Segment SGI384274[] = {{&S102757[72], 3}, {NULL}};
const Segment SGI384275[] = {{&S102761[3], 8}, {NULL}};
const Segment SGI384276[] = {{&S102762[3], 9}, {NULL}};
const Segment SGI384277[] = {{&S102769[7], 9}, {NULL}};
const Segment SGI384278[] = {{&S102769[43], 3}, {NULL}};
const Segment SGI384279[] = {{&S102770[7], 9}, {NULL}};
const Segment SGI384280[] = {{&S102770[47], 3}, {NULL}};
const Segment SGI384281[] = {{&S102771[7], 9}, {NULL}};
const Segment SGI384282[] = {{&S102771[41], 3}, {NULL}};
const Segment SGI384283[] = {{&S102774[7], 9}, {NULL}};
const Segment SGI384284[] = {{&S102774[69], 3}, {NULL}};
const Segment SGI384285[] = {{&S102785[7], 9}, {NULL}};
const Segment SGI384286[] = {{&S102789[7], 9}, {NULL}};
const Segment SGI384287[] = {{&S102789[41], 3}, {NULL}};
const Segment SGI384288[] = {{&S102794[7], 9}, {NULL}};
const Segment SGI384289[] = {{&S102798[7], 9}, {NULL}};
const Segment SGI384290[] = {{&S102798[78], 3}, {NULL}};
const Segment SGI384291[] = {{&S102804[7], 10}, {NULL}};
const Segment SGI384292[] = {{&S102804[31], 3}, {NULL}};
const Segment SGI384293[] = {{&S102817[7], 5}, {NULL}};
const Segment SGI384294[] = {{&S102823[3], 8}, {NULL}};
const Segment SGI384295[] = {{&S102823[25], 3}, {NULL}};
const Segment SGI384296[] = {{&S102824[3], 8}, {NULL}};
const Segment SGI384297[] = {{&S102824[25], 3}, {NULL}};
const Segment SGI384298[] = {{&S102829[58], 6}, {NULL}};
const Segment SGI384299[] = {{&S102846[7], 9}, {NULL}};
const Segment SGI384300[] = {{&S102895[51], 20}, {NULL}};
const Segment SGI384301[] = {{&S102932[44], 13}, {NULL}};
const Segment SGI384302[] = {{&S102932[64], 13}, {NULL}};
const Segment SGI384303[] = {{&S102969[7], 8}, {NULL}};
const Segment SGI384304[] = {{&S102974[7], 10}, {NULL}};
const Segment SGI384305[] = {{&S102974[33], 3}, {NULL}};
const Segment SGI384306[] = {{&S102975[7], 9}, {NULL}};
const Segment SGI384307[] = {{&S102975[32], 3}, {NULL}};
const Segment SGI384308[] = {{&S102977[7], 9}, {NULL}};
const Segment SGI384309[] = {{&S102977[32], 3}, {NULL}};
const Segment SGI384310[] = {{&S102978[3], 8}, {NULL}};
const Segment SGI384311[] = {{&S102981[7], 9}, {NULL}};
const Segment SGI384312[] = {{&S102982[7], 9}, {NULL}};
const Segment SGI384313[] = {{&S102982[47], 3}, {NULL}};
const Segment SGI384314[] = {{&S102983[7], 10}, {NULL}};
const Segment SGI384315[] = {{&S102983[42], 3}, {NULL}};
const Segment SGI384316[] = {{&S102986[7], 9}, {NULL}};
const Segment SGI384317[] = {{&S102986[69], 3}, {NULL}};
const Segment SGI384318[] = {{&S102989[7], 17}, {NULL}};
const Segment SGI384319[] = {{&S102991[7], 10}, {NULL}};
const Segment SGI384320[] = {{&S102991[42], 3}, {NULL}};
const Segment SGI384321[] = {{&S102992[7], 10}, {NULL}};
const Segment SGI384322[] = {{&S102992[33], 3}, {NULL}};
const Segment SGI384323[] = {{&S102993[3], 8}, {NULL}};
const Segment SGI384324[] = {{&S102993[30], 4}, {NULL}};
const Segment SGI384325[] = {{&S102998[7], 9}, {NULL}};
const Segment SGI384326[] = {{&S103013[3], 8}, {NULL}};
const Segment SGI384327[] = {{&S103027[7], 9}, {NULL}};
const Segment SGI384328[] = {{&S103027[41], 3}, {NULL}};
const Segment SGI384329[] = {{&S103035[7], 9}, {NULL}};
const Segment SGI384330[] = {{&S103035[72], 3}, {NULL}};
const Segment SGI384331[] = {{&S103045[7], 9}, {NULL}};
const Segment SGI384332[] = {{&S103055[20], 8}, {NULL}};
const Segment SGI384333[] = {{&S87459[27], 5}, {NULL}};
const Segment SGI384334[] = {{&S103056[20], 8}, {NULL}};
const Segment SGI384335[] = {{&S87541[11], 5}, {NULL}};
const Segment SGI384336[] = {{&S87541[17], 3}, {NULL}};
const Segment SGI384337[] = {{&S87663[11], 5}, {NULL}};
const Segment SGI384338[] = {{&S87663[79], 3}, {NULL}};
const Segment SGI384339[] = {{&S103191[43], 14}, {NULL}};
const Segment SGI384340[] = {{&S103191[64], 13}, {NULL}};
const Segment SGI384341[] = {{&S103197[44], 13}, {NULL}};
const Segment SGI384342[] = {{&S103197[64], 13}, {NULL}};
const Segment SGI384343[] = {{&S103208[44], 13}, {NULL}};
const Segment SGI384344[] = {{&S103208[64], 13}, {NULL}};
const Segment SGI384345[] = {{&S103209[44], 14}, {NULL}};
const Segment SGI384346[] = {{&S103209[65], 13}, {NULL}};
const Segment SGI384347[] = {{&S103226[7], 9}, {NULL}};
const Segment SGI384348[] = {{&S103226[41], 3}, {NULL}};
const Segment SGI384349[] = {{&S103227[7], 9}, {NULL}};
const Segment SGI384350[] = {{&S103232[7], 9}, {NULL}};
const Segment SGI384351[] = {{&S103232[30], 3}, {NULL}};
const Segment SGI384352[] = {{&S103233[7], 6}, {NULL}};
const Segment SGI384353[] = {{&S103233[76], 3}, {NULL}};
const Segment SGI384354[] = {{&S103234[7], 9}, {NULL}};
const Segment SGI384355[] = {{&S103236[7], 9}, {NULL}};
const Segment SGI384356[] = {{&S103236[38], 3}, {NULL}};
const Segment SGI384357[] = {{&S103241[7], 6}, {NULL}};
const Segment SGI384358[] = {{&S103241[34], 3}, {NULL}};
const Segment SGI384359[] = {{&S103250[16], 5}, {NULL}};
const Segment SGI384360[] = {{&S103256[7], 9}, {NULL}};
const Segment SGI384361[] = {{&S103256[41], 3}, {NULL}};
const Segment SGI384362[] = {{&S103257[7], 9}, {NULL}};
const Segment SGI384363[] = {{&S103257[69], 3}, {NULL}};
const Segment SGI384364[] = {{&S103262[3], 16}, {NULL}};
const Segment SGI384365[] = {{&S103262[20], 3}, {NULL}};
const Segment SGI384366[] = {{&S103268[3], 16}, {NULL}};
const Segment SGI384367[] = {{&S103268[20], 3}, {NULL}};
const Segment SGI384368[] = {{&S103271[7], 17}, {NULL}};
const Segment SGI384369[] = {{&S103271[11], 13}, {NULL}};
const Segment SGI384370[] = {{&S87787[11], 6}, {NULL}};
const Segment SGI384371[] = {{&S87787[42], 3}, {NULL}};
const Segment SGI384372[] = {{&S87788[11], 5}, {NULL}};
const Segment SGI384373[] = {{&S87788[41], 3}, {NULL}};
const Segment SGI384374[] = {{&S103282[7], 9}, {NULL}};
const Segment SGI384375[] = {{&S103302[16], 5}, {NULL}};
const Segment SGI384376[] = {{&S103315[3], 8}, {NULL}};
const Segment SGI384377[] = {{&S103324[7], 9}, {NULL}};
const Segment SGI384378[] = {{&S103324[72], 3}, {NULL}};
const Segment SGI384379[] = {{&S103328[7], 9}, {NULL}};
const Segment SGI384380[] = {{&S103328[72], 3}, {NULL}};
const Segment SGI384381[] = {{&S87904[11], 5}, {NULL}};
const Segment SGI384382[] = {{&S87947[11], 6}, {NULL}};
const Segment SGI384383[] = {{&S87947[81], 3}, {NULL}};
const Segment SGI384384[] = {{&S87953[11], 11}, {NULL}};
const Segment SGI384385[] = {{&S87953[23], 4}, {NULL}};
const Segment SGI384386[] = {{&S87976[11], 5}, {NULL}};
const Segment SGI384387[] = {{&S87976[30], 3}, {NULL}};
const Segment SGI384388[] = {{&S103420[43], 13}, {NULL}};
const Segment SGI384389[] = {{&S103420[63], 13}, {NULL}};
const Segment SGI384390[] = {{&S103479[44], 14}, {NULL}};
const Segment SGI384391[] = {{&S103479[65], 13}, {NULL}};
const Segment SGI384392[] = {{&S88043[11], 5}, {NULL}};
const Segment SGI384393[] = {{&S103502[44], 13}, {NULL}};
const Segment SGI384394[] = {{&S103502[64], 13}, {NULL}};
const Segment SGI384395[] = {{&S88080[27], 5}, {NULL}};
const Segment SGI384396[] = {{&S103528[7], 10}, {NULL}};
const Segment SGI384397[] = {{&S103528[33], 3}, {NULL}};
const Segment SGI384398[] = {{&S103529[7], 10}, {NULL}};
const Segment SGI384399[] = {{&S103529[33], 3}, {NULL}};
const Segment SGI384400[] = {{&S103530[7], 9}, {NULL}};
const Segment SGI384401[] = {{&S103530[32], 3}, {NULL}};
const Segment SGI384402[] = {{&S103531[3], 18}, {NULL}};
const Segment SGI384403[] = {{&S103532[3], 8}, {NULL}};
const Segment SGI384404[] = {{&S103541[7], 9}, {NULL}};
const Segment SGI384405[] = {{&S103541[31], 3}, {NULL}};
const Segment SGI384406[] = {{&S103542[3], 8}, {NULL}};
const Segment SGI384407[] = {{&S103542[27], 3}, {NULL}};
const Segment SGI384408[] = {{&S103543[3], 19}, {NULL}};
const Segment SGI384409[] = {{&S103543[23], 3}, {NULL}};
const Segment SGI384410[] = {{&S103555[7], 9}, {NULL}};
const Segment SGI384411[] = {{&S103555[69], 3}, {NULL}};
const Segment SGI384412[] = {{&S103577[3], 9}, {NULL}};
const Segment SGI384413[] = {{&S103577[28], 3}, {NULL}};
const Segment SGI384414[] = {{&S103579[7], 9}, {NULL}};
const Segment SGI384415[] = {{&S103584[7], 15}, {NULL}};
const Segment SGI384416[] = {{&S103601[16], 5}, {NULL}};
const Segment SGI384417[] = {{&S103603[3], 8}, {NULL}};
const Segment SGI384418[] = {{&S103610[7], 9}, {NULL}};
const Segment SGI384419[] = {{&S103610[41], 3}, {NULL}};
const Segment SGI384420[] = {{&S103615[7], 9}, {NULL}};
const Segment SGI384421[] = {{&S103615[32], 3}, {NULL}};
const Segment SGI384422[] = {{&S103616[7], 9}, {NULL}};
const Segment SGI384423[] = {{&S103620[7], 15}, {NULL}};
const Segment SGI384424[] = {{&S103622[3], 8}, {NULL}};
const Segment SGI384425[] = {{&S103637[69], 9}, {NULL}};
const Segment SGI384426[] = {{&S103639[7], 9}, {NULL}};
const Segment SGI384427[] = {{&S103639[72], 3}, {NULL}};
const Segment SGI384428[] = {{&S103650[7], 9}, {NULL}};
const Segment SGI384429[] = {{&S103736[44], 13}, {NULL}};
const Segment SGI384430[] = {{&S103736[64], 13}, {NULL}};
const Segment SGI384431[] = {{&S103797[41], 13}, {NULL}};
const Segment SGI384432[] = {{&S103797[61], 13}, {NULL}};
const Segment SGI384433[] = {{&S103802[44], 13}, {NULL}};
const Segment SGI384434[] = {{&S103802[64], 13}, {NULL}};
const Segment SGI384435[] = {{&S103807[44], 13}, {NULL}};
const Segment SGI384436[] = {{&S103807[64], 13}, {NULL}};
const Segment SGI384437[] = {{&S103813[44], 13}, {NULL}};
const Segment SGI384438[] = {{&S103813[64], 13}, {NULL}};
const Segment SGI384439[] = {{&S103833[7], 9}, {NULL}};
const Segment SGI384440[] = {{&S103833[11], 5}, {NULL}};
const Segment SGI384441[] = {{&S88385[112], 5}, {NULL}};
const Segment SGI384442[] = {{&S88385[180], 3}, {NULL}};
const Segment SGI384443[] = {{&S103846[7], 9}, {NULL}};
const Segment SGI384444[] = {{&S103846[101], 3}, {NULL}};
const Segment SGI384445[] = {{&S103847[7], 10}, {NULL}};
const Segment SGI384446[] = {{&S103857[7], 9}, {NULL}};
const Segment SGI384447[] = {{&S103857[72], 3}, {NULL}};
const Segment SGI384448[] = {{&S103858[3], 8}, {NULL}};
const Segment SGI384449[] = {{&S103859[7], 9}, {NULL}};
const Segment SGI384450[] = {{&S103861[3], 12}, {NULL}};
const Segment SGI384451[] = {{&S103861[33], 3}, {NULL}};
const Segment SGI384452[] = {{&S103869[3], 13}, {NULL}};
const Segment SGI384453[] = {{&S103870[3], 8}, {NULL}};
const Segment SGI384454[] = {{&S103875[7], 7}, {NULL}};
const Segment SGI384455[] = {{&S103876[7], 7}, {NULL}};
const Segment SGI384456[] = {{&S103876[38], 3}, {NULL}};
const Segment SGI384457[] = {{&S103886[7], 17}, {NULL}};
const Segment SGI384458[] = {{&S103886[68], 3}, {NULL}};
const Segment SGI384459[] = {{&S103888[7], 9}, {NULL}};
const Segment SGI384460[] = {{&S103889[7], 9}, {NULL}};
const Segment SGI384461[] = {{&S103889[32], 3}, {NULL}};
const Segment SGI384462[] = {{&S103892[16], 5}, {NULL}};
const Segment SGI384463[] = {{&S103896[3], 9}, {NULL}};
const Segment SGI384464[] = {{&S103900[7], 8}, {NULL}};
const Segment SGI384465[] = {{&S103905[16], 5}, {NULL}};
const Segment SGI384466[] = {{&S103907[7], 17}, {NULL}};
const Segment SGI384467[] = {{&S103911[3], 8}, {NULL}};
const Segment SGI384468[] = {{&S103911[25], 3}, {NULL}};
const Segment SGI384469[] = {{&S103924[3], 8}, {NULL}};
const Segment SGI384470[] = {{&S103942[7], 9}, {NULL}};
const Segment SGI384471[] = {{&S103951[7], 10}, {NULL}};
const Segment SGI384472[] = {{&S103951[73], 3}, {NULL}};
const Segment SGI384473[] = {{&S104025[43], 14}, {NULL}};
const Segment SGI384474[] = {{&S104025[64], 13}, {NULL}};
const Segment SGI384475[] = {{&S104116[44], 14}, {NULL}};
const Segment SGI384476[] = {{&S104116[65], 13}, {NULL}};
const Segment SGI384477[] = {{&S104121[43], 13}, {NULL}};
const Segment SGI384478[] = {{&S104121[63], 13}, {NULL}};
const Segment SGI384479[] = {{&S104122[44], 13}, {NULL}};
const Segment SGI384480[] = {{&S104122[64], 13}, {NULL}};
const Segment SGI384481[] = {{&S104123[44], 13}, {NULL}};
const Segment SGI384482[] = {{&S104123[64], 13}, {NULL}};
const Segment SGI384483[] = {{&S104139[44], 13}, {NULL}};
const Segment SGI384484[] = {{&S104139[64], 13}, {NULL}};
const Segment SGI384485[] = {{&S104141[44], 13}, {NULL}};
const Segment SGI384486[] = {{&S104141[64], 13}, {NULL}};
const Segment SGI384487[] = {{&S104149[40], 14}, {NULL}};
const Segment SGI384488[] = {{&S104149[61], 13}, {NULL}};
const Segment SGI384489[] = {{&S104151[7], 10}, {NULL}};
const Segment SGI384490[] = {{&S104151[74], 3}, {NULL}};
const Segment SGI384491[] = {{&S104153[7], 9}, {NULL}};
const Segment SGI384492[] = {{&S104153[41], 3}, {NULL}};
const Segment SGI384493[] = {{&S104157[7], 9}, {NULL}};
const Segment SGI384494[] = {{&S104166[7], 19}, {NULL}};
const Segment SGI384495[] = {{&S104166[51], 3}, {NULL}};
const Segment SGI384496[] = {{&S104172[3], 8}, {NULL}};
const Segment SGI384497[] = {{&S104173[3], 13}, {NULL}};
const Segment SGI384498[] = {{&S104181[7], 9}, {NULL}};
const Segment SGI384499[] = {{&S104181[53], 3}, {NULL}};
const Segment SGI384500[] = {{&S104198[7], 9}, {NULL}};
const Segment SGI384501[] = {{&S104199[7], 9}, {NULL}};
const Segment SGI384502[] = {{&S104199[72], 3}, {NULL}};
const Segment SGI384503[] = {{&S104208[7], 10}, {NULL}};
const Segment SGI384504[] = {{&S104210[3], 8}, {NULL}};
const Segment SGI384505[] = {{&S104210[25], 3}, {NULL}};
const Segment SGI384506[] = {{&S104216[7], 9}, {NULL}};
const Segment SGI384507[] = {{&S104216[11], 5}, {NULL}};
const Segment SGI384508[] = {{&S104216[73], 3}, {NULL}};
const Segment SGI384509[] = {{&S104221[3], 16}, {NULL}};
const Segment SGI384510[] = {{&S104221[20], 3}, {NULL}};
const Segment SGI384511[] = {{&S104229[7], 9}, {NULL}};
const Segment SGI384512[] = {{&S104234[7], 9}, {NULL}};
const Segment SGI384513[] = {{&S104234[47], 3}, {NULL}};
const Segment SGI384514[] = {{&S104246[7], 18}, {NULL}};
const Segment SGI384515[] = {{&S104246[56], 3}, {NULL}};
const Segment SGI384516[] = {{&S104249[7], 51}, {NULL}};
const Segment SGI384517[] = {{&S104249[11], 47}, {NULL}};
const Segment SGI384518[] = {{&S104256[7], 14}, {NULL}};
const Segment SGI384519[] = {{&S104261[7], 9}, {NULL}};
const Segment SGI384520[] = {{&S104261[41], 3}, {NULL}};
const Segment SGI384521[] = {{&S104263[7], 9}, {NULL}};
const Segment SGI384522[] = {{&S104266[7], 16}, {NULL}};
const Segment SGI384523[] = {{&S104266[85], 3}, {NULL}};
const Segment SGI384524[] = {{&S104267[7], 9}, {NULL}};
const Segment SGI384525[] = {{&S104267[69], 3}, {NULL}};
const Segment SGI384526[] = {{&S104268[7], 13}, {NULL}};
const Segment SGI384527[] = {{&S104268[47], 3}, {NULL}};
const Segment SGI384528[] = {{&S104277[3], 8}, {NULL}};
const Segment SGI384529[] = {{&S104291[7], 9}, {NULL}};
const Segment SGI384530[] = {{&S104292[7], 9}, {NULL}};
const Segment SGI384531[] = {{&S104303[20], 8}, {NULL}};
const Segment SGI384532[] = {{&S104333[7], 9}, {NULL}};
const Segment SGI384533[] = {{&S88967[11], 5}, {NULL}};
const Segment SGI384534[] = {{&S88967[41], 3}, {NULL}};
const Segment SGI384535[] = {{&S88977[11], 5}, {NULL}};
const Segment SGI384536[] = {{&S88977[41], 3}, {NULL}};
const Segment SGI384537[] = {{&S104399[43], 13}, {NULL}};
const Segment SGI384538[] = {{&S104399[63], 13}, {NULL}};
const Segment SGI384539[] = {{&S104402[43], 14}, {NULL}};
const Segment SGI384540[] = {{&S104402[64], 13}, {NULL}};
const Segment SGI384541[] = {{&S104405[43], 13}, {NULL}};
const Segment SGI384542[] = {{&S104405[63], 13}, {NULL}};
const Segment SGI384543[] = {{&S104526[44], 13}, {NULL}};
const Segment SGI384544[] = {{&S104526[64], 13}, {NULL}};
const Segment SGI384545[] = {{&S104527[43], 13}, {NULL}};
const Segment SGI384546[] = {{&S104527[63], 13}, {NULL}};
const Segment SGI384547[] = {{&S104529[36], 29}, {NULL}};
const Segment SGI384548[] = {{&S104531[43], 14}, {NULL}};
const Segment SGI384549[] = {{&S104531[64], 13}, {NULL}};
const Segment SGI384550[] = {{&S104537[44], 13}, {NULL}};
const Segment SGI384551[] = {{&S104537[64], 13}, {NULL}};
const Segment SGI384552[] = {{&S104538[44], 13}, {NULL}};
const Segment SGI384553[] = {{&S104538[64], 13}, {NULL}};
const Segment SGI384554[] = {{&S104566[7], 18}, {NULL}};
const Segment SGI384555[] = {{&S104570[7], 9}, {NULL}};
const Segment SGI384556[] = {{&S104570[101], 3}, {NULL}};
const Segment SGI384557[] = {{&S104572[7], 9}, {NULL}};
const Segment SGI384558[] = {{&S104581[7], 9}, {NULL}};
const Segment SGI384559[] = {{&S89229[11], 5}, {NULL}};
const Segment SGI384560[] = {{&S89229[40], 3}, {NULL}};
const Segment SGI384561[] = {{&S104583[7], 9}, {NULL}};
const Segment SGI384562[] = {{&S104586[7], 9}, {NULL}};
const Segment SGI384563[] = {{&S104586[41], 3}, {NULL}};
const Segment SGI384564[] = {{&S104587[7], 10}, {NULL}};
const Segment SGI384565[] = {{&S104590[7], 9}, {NULL}};
const Segment SGI384566[] = {{&S104590[78], 3}, {NULL}};
const Segment SGI384567[] = {{&S104597[7], 9}, {NULL}};
const Segment SGI384568[] = {{&S104597[72], 3}, {NULL}};
const Segment SGI384569[] = {{&S89246[11], 5}, {NULL}};
const Segment SGI384570[] = {{&S104601[7], 9}, {NULL}};
const Segment SGI384571[] = {{&S104609[3], 5}, {NULL}};
const Segment SGI384572[] = {{&S104609[21], 3}, {NULL}};
const Segment SGI384573[] = {{&S104626[7], 9}, {NULL}};
const Segment SGI384574[] = {{&S104632[7], 9}, {NULL}};
const Segment SGI384575[] = {{&S104632[73], 3}, {NULL}};
const Segment SGI384576[] = {{&S104634[7], 15}, {NULL}};
const Segment SGI384577[] = {{&S104643[3], 5}, {NULL}};
const Segment SGI384578[] = {{&S104648[16], 5}, {NULL}};
const Segment SGI384579[] = {{&S104652[7], 9}, {NULL}};
const Segment SGI384580[] = {{&S104652[38], 3}, {NULL}};
const Segment SGI384581[] = {{&S104653[7], 9}, {NULL}};
const Segment SGI384582[] = {{&S104665[7], 9}, {NULL}};
const Segment SGI384583[] = {{&S104665[72], 3}, {NULL}};
const Segment SGI384584[] = {{&S104670[7], 9}, {NULL}};
const Segment SGI384585[] = {{&S104670[73], 3}, {NULL}};
const Segment SGI384586[] = {{&S104676[16], 5}, {NULL}};
const Segment SGI384587[] = {{&S104677[7], 7}, {NULL}};
const Segment SGI384588[] = {{&S104677[39], 3}, {NULL}};
const Segment SGI384589[] = {{&S104684[7], 8}, {NULL}};
const Segment SGI384590[] = {{&S104684[40], 3}, {NULL}};
const Segment SGI384591[] = {{&S104687[3], 8}, {NULL}};
const Segment SGI384592[] = {{&S104710[3], 8}, {NULL}};
const Segment SGI384593[] = {{&S104711[3], 26}, {NULL}};
const Segment SGI384594[] = {{&S104712[3], 26}, {NULL}};
const Segment SGI384595[] = {{&S104722[3], 16}, {NULL}};
const Segment SGI384596[] = {{&S104722[20], 3}, {NULL}};
const Segment SGI384597[] = {{&S104729[7], 9}, {NULL}};
const Segment SGI384598[] = {{&S104730[3], 18}, {NULL}};
const Segment SGI384599[] = {{&S104731[3], 8}, {NULL}};
const Segment SGI384600[] = {{&S104747[7], 18}, {NULL}};
const Segment SGI384601[] = {{&S104749[3], 5}, {NULL}};
const Segment SGI384602[] = {{&S89386[11], 5}, {NULL}};
const Segment SGI384603[] = {{&S89386[79], 3}, {NULL}};
const Segment SGI384604[] = {{&S104755[3], 5}, {NULL}};
const Segment SGI384605[] = {{&S104755[24], 3}, {NULL}};
const Segment SGI384606[] = {{&S104760[7], 15}, {NULL}};
const Segment SGI384607[] = {{&S104761[20], 8}, {NULL}};
const Segment SGI384608[] = {{&S104761[56], 3}, {NULL}};
const Segment SGI384609[] = {{&S104762[7], 10}, {NULL}};
const Segment SGI384610[] = {{&S104762[73], 3}, {NULL}};
const Segment SGI384611[] = {{&S89467[27], 5}, {NULL}};
const Segment SGI384612[] = {{&S104848[43], 14}, {NULL}};
const Segment SGI384613[] = {{&S104848[64], 13}, {NULL}};
const Segment SGI384614[] = {{&S105027[43], 13}, {NULL}};
const Segment SGI384615[] = {{&S105027[63], 13}, {NULL}};
const Segment SGI384616[] = {{&S105033[44], 13}, {NULL}};
const Segment SGI384617[] = {{&S105033[64], 13}, {NULL}};
const Segment SGI384618[] = {{&S105036[44], 14}, {NULL}};
const Segment SGI384619[] = {{&S105036[65], 13}, {NULL}};
const Segment SGI384620[] = {{&S105038[43], 13}, {NULL}};
const Segment SGI384621[] = {{&S105038[63], 13}, {NULL}};
const Segment SGI384622[] = {{&S105041[44], 13}, {NULL}};
const Segment SGI384623[] = {{&S105041[64], 13}, {NULL}};
const Segment SGI384624[] = {{&S105045[44], 13}, {NULL}};
const Segment SGI384625[] = {{&S105045[64], 13}, {NULL}};
const Segment SGI384626[] = {{&S105046[44], 14}, {NULL}};
const Segment SGI384627[] = {{&S105046[65], 13}, {NULL}};
const Segment SGI384628[] = {{&S105065[44], 13}, {NULL}};
const Segment SGI384629[] = {{&S105065[64], 13}, {NULL}};
const Segment SGI384630[] = {{&S105100[7], 9}, {NULL}};
const Segment SGI384631[] = {{&S105100[76], 3}, {NULL}};
const Segment SGI384632[] = {{&S105103[7], 9}, {NULL}};
const Segment SGI384633[] = {{&S105103[73], 3}, {NULL}};
const Segment SGI384634[] = {{&S105104[7], 9}, {NULL}};
const Segment SGI384635[] = {{&S105106[7], 9}, {NULL}};
const Segment SGI384636[] = {{&S105110[7], 10}, {NULL}};
const Segment SGI384637[] = {{&S105110[74], 3}, {NULL}};
const Segment SGI384638[] = {{&S105118[77], 13}, {NULL}};
const Segment SGI384639[] = {{&S105118[81], 9}, {NULL}};
const Segment SGI384640[] = {{&S105119[7], 15}, {NULL}};
const Segment SGI384641[] = {{&S105119[23], 4}, {NULL}};
const Segment SGI384642[] = {{&S105122[7], 9}, {NULL}};
const Segment SGI384643[] = {{&S105124[7], 14}, {NULL}};
const Segment SGI384644[] = {{&S105126[7], 9}, {NULL}};
const Segment SGI384645[] = {{&S105126[73], 3}, {NULL}};
const Segment SGI384646[] = {{&S105127[3], 5}, {NULL}};
const Segment SGI384647[] = {{&S105131[7], 9}, {NULL}};
const Segment SGI384648[] = {{&S105131[72], 3}, {NULL}};
const Segment SGI384649[] = {{&S105140[7], 18}, {NULL}};
const Segment SGI384650[] = {{&S105145[7], 10}, {NULL}};
const Segment SGI384651[] = {{&S105145[32], 3}, {NULL}};
const Segment SGI384652[] = {{&S105154[3], 8}, {NULL}};
const Segment SGI384653[] = {{&S105161[7], 9}, {NULL}};
const Segment SGI384654[] = {{&S105161[73], 3}, {NULL}};
const Segment SGI384655[] = {{&S105181[7], 9}, {NULL}};
const Segment SGI384656[] = {{&S105184[7], 14}, {NULL}};
const Segment SGI384657[] = {{&S105189[7], 15}, {NULL}};
const Segment SGI384658[] = {{&S105189[79], 3}, {NULL}};
const Segment SGI384659[] = {{&S105191[7], 20}, {NULL}};
const Segment SGI384660[] = {{&S105193[3], 8}, {NULL}};
const Segment SGI384661[] = {{&S89741[11], 5}, {NULL}};
const Segment SGI384662[] = {{&S89760[11], 11}, {NULL}};
const Segment SGI384663[] = {{&S89760[23], 4}, {NULL}};
const Segment SGI384664[] = {{&S89762[11], 5}, {NULL}};
const Segment SGI384665[] = {{&S89762[37], 3}, {NULL}};
const Segment SGI384666[] = {{&S105222[7], 9}, {NULL}};
const Segment SGI384667[] = {{&S105225[3], 5}, {NULL}};
const Segment SGI384668[] = {{&S105226[16], 5}, {NULL}};
const Segment SGI384669[] = {{&S105236[7], 7}, {NULL}};
const Segment SGI384670[] = {{&S105236[71], 3}, {NULL}};
const Segment SGI384671[] = {{&S105238[7], 9}, {NULL}};
const Segment SGI384672[] = {{&S105238[31], 3}, {NULL}};
const Segment SGI384673[] = {{&S105239[3], 6}, {NULL}};
const Segment SGI384674[] = {{&S105239[25], 3}, {NULL}};
const Segment SGI384675[] = {{&S105241[7], 6}, {NULL}};
const Segment SGI384676[] = {{&S105241[45], 3}, {NULL}};
const Segment SGI384677[] = {{&S105242[7], 10}, {NULL}};
const Segment SGI384678[] = {{&S105244[3], 5}, {NULL}};
const Segment SGI384679[] = {{&S105247[16], 5}, {NULL}};
const Segment SGI384680[] = {{&S105249[7], 9}, {NULL}};
const Segment SGI384681[] = {{&S105256[7], 6}, {NULL}};
const Segment SGI384682[] = {{&S105258[7], 9}, {NULL}};
const Segment SGI384683[] = {{&S105263[7], 9}, {NULL}};
const Segment SGI384684[] = {{&S105264[7], 9}, {NULL}};
const Segment SGI384685[] = {{&S105269[7], 6}, {NULL}};
const Segment SGI384686[] = {{&S105275[7], 9}, {NULL}};
const Segment SGI384687[] = {{&S105276[7], 12}, {NULL}};
const Segment SGI384688[] = {{&S105278[7], 10}, {NULL}};
const Segment SGI384689[] = {{&S105284[3], 8}, {NULL}};
const Segment SGI384690[] = {{&S105292[7], 9}, {NULL}};
const Segment SGI384691[] = {{&S105292[73], 3}, {NULL}};
const Segment SGI384692[] = {{&S105296[7], 9}, {NULL}};
const Segment SGI384693[] = {{&S105296[11], 5}, {NULL}};
const Segment SGI384694[] = {{&S105305[3], 8}, {NULL}};
const Segment SGI384695[] = {{&S105305[25], 3}, {NULL}};
const Segment SGI384696[] = {{&S105306[7], 9}, {NULL}};
const Segment SGI384697[] = {{&S105306[11], 5}, {NULL}};
const Segment SGI384698[] = {{&S105308[3], 15}, {NULL}};
const Segment SGI384699[] = {{&S105310[3], 5}, {NULL}};
const Segment SGI384700[] = {{&S105310[22], 10}, {NULL}};
const Segment SGI384701[] = {{&S105322[7], 9}, {NULL}};
const Segment SGI384702[] = {{&S105332[7], 9}, {NULL}};
const Segment SGI384703[] = {{&S105335[7], 12}, {NULL}};
const Segment SGI384704[] = {{&S105336[7], 9}, {NULL}};
const Segment SGI384705[] = {{&S105336[34], 3}, {NULL}};
const Segment SGI384706[] = {{&S105340[7], 9}, {NULL}};
const Segment SGI384707[] = {{&S105348[7], 9}, {NULL}};
const Segment SGI384708[] = {{&S105364[3], 16}, {NULL}};
const Segment SGI384709[] = {{&S105364[20], 3}, {NULL}};
const Segment SGI384710[] = {{&S105365[7], 9}, {NULL}};
const Segment SGI384711[] = {{&S105366[3], 16}, {NULL}};
const Segment SGI384712[] = {{&S105366[20], 3}, {NULL}};
const Segment SGI384713[] = {{&S105367[7], 9}, {NULL}};
const Segment SGI384714[] = {{&S105368[3], 16}, {NULL}};
const Segment SGI384715[] = {{&S105368[20], 3}, {NULL}};
const Segment SGI384716[] = {{&S105369[7], 9}, {NULL}};
const Segment SGI384717[] = {{&S105369[72], 3}, {NULL}};
const Segment SGI384718[] = {{&S105370[7], 9}, {NULL}};
const Segment SGI384719[] = {{&S105370[32], 3}, {NULL}};
const Segment SGI384720[] = {{&S105371[7], 9}, {NULL}};
const Segment SGI384721[] = {{&S105378[3], 16}, {NULL}};
const Segment SGI384722[] = {{&S105378[20], 3}, {NULL}};
const Segment SGI384723[] = {{&S105398[7], 9}, {NULL}};
const Segment SGI384724[] = {{&S105399[7], 9}, {NULL}};
const Segment SGI384725[] = {{&S105402[20], 8}, {NULL}};
const Segment SGI384726[] = {{&S105407[3], 16}, {NULL}};
const Segment SGI384727[] = {{&S105407[20], 3}, {NULL}};
const Segment SGI384728[] = {{&S105420[7], 17}, {NULL}};
const Segment SGI384729[] = {{&S105425[7], 14}, {NULL}};
const Segment SGI384730[] = {{&S105429[3], 16}, {NULL}};
const Segment SGI384731[] = {{&S105429[20], 3}, {NULL}};
const Segment SGI384732[] = {{&S105431[7], 15}, {NULL}};
const Segment SGI384733[] = {{&S105443[7], 9}, {NULL}};
const Segment SGI384734[] = {{&S89861[11], 5}, {NULL}};
const Segment SGI384735[] = {{&S105466[7], 9}, {NULL}};
const Segment SGI384736[] = {{&S105479[7], 9}, {NULL}};
const Segment SGI384737[] = {{&S105507[7], 10}, {NULL}};
const Segment SGI384738[] = {{&S105566[43], 14}, {NULL}};
const Segment SGI384739[] = {{&S105566[64], 13}, {NULL}};
const Segment SGI384740[] = {{&S90012[11], 6}, {NULL}};
const Segment SGI384741[] = {{&S90012[41], 3}, {NULL}};
const Segment SGI384742[] = {{&S90024[11], 6}, {NULL}};
const Segment SGI384743[] = {{&S90065[11], 5}, {NULL}};
const Segment SGI384744[] = {{&S90088[11], 11}, {NULL}};
const Segment SGI384745[] = {{&S90088[23], 4}, {NULL}};
const Segment SGI384746[] = {{&S90176[11], 5}, {NULL}};
const Segment SGI384747[] = {{&S90176[46], 3}, {NULL}};
const Segment SGI384748[] = {{&S90218[11], 5}, {NULL}};
const Segment SGI384749[] = {{&S90355[11], 6}, {NULL}};
const Segment SGI384750[] = {{&S90355[81], 3}, {NULL}};
const Segment SGI384751[] = {{&S105931[44], 14}, {NULL}};
const Segment SGI384752[] = {{&S105931[65], 13}, {NULL}};
const Segment SGI384753[] = {{&S90531[11], 5}, {NULL}};
const Segment SGI384754[] = {{&S90578[11], 5}, {NULL}};
const Segment SGI384755[] = {{&S90662[11], 5}, {NULL}};
const Segment SGI384756[] = {{&S90662[37], 3}, {NULL}};
const Segment SGI384757[] = {{&S90839[111], 5}, {NULL}};
const Segment SGI384758[] = {{&S90839[179], 3}, {NULL}};
const Segment SGI384759[] = {{&S90984[27], 6}, {NULL}};
const Segment SGI384760[] = {{&S91275[11], 5}, {NULL}};
const Segment SGI384761[] = {{&S91275[39], 3}, {NULL}};
const Segment SGI384762[] = {{&S91733[27], 6}, {NULL}};
const Segment SGI384763[] = {{&S91875[11], 5}, {NULL}};
const Segment SGI384764[] = {{&S91875[30], 3}, {NULL}};
const Segment SGI384765[] = {{&S91876[11], 6}, {NULL}};
const Segment SGI384766[] = {{&S91967[27], 5}, {NULL}};
const Segment SGI384767[] = {{&S92069[11], 5}, {NULL}};
const Segment SGI384768[] = {{&S92069[38], 3}, {NULL}};
const Segment SGI384769[] = {{&S92254[11], 5}, {NULL}};
const Segment SGI384770[] = {{&S92254[37], 3}, {NULL}};
const Segment SGI384771[] = {{&S92468[11], 5}, {NULL}};
const Segment SGI384772[] = {{&S92468[46], 3}, {NULL}};
const Segment SGI384773[] = {{&S92692[11], 5}, {NULL}};
const Segment SGI384774[] = {{&S92692[26], 3}, {NULL}};
const Segment SGI384775[] = {{&S92831[11], 6}, {NULL}};
const Segment SGI384776[] = {{&S92831[56], 3}, {NULL}};
const Segment SGI384777[] = {{&S92833[11], 6}, {NULL}};
const Segment SGI384778[] = {{&S93049[11], 5}, {NULL}};
const Segment SGI384779[] = {{&S93049[35], 3}, {NULL}};
const Segment SGI384780[] = {{&S93060[11], 6}, {NULL}};
const Segment SGI384781[] = {{&S93060[42], 3}, {NULL}};
const Segment SGI384782[] = {{&S93099[99], 6}, {NULL}};
const Segment SGI384783[] = {{&S93099[161], 3}, {NULL}};
const Segment SGI384784[] = {{&S93100[99], 6}, {NULL}};
const Segment SGI384785[] = {{&S93100[161], 3}, {NULL}};
const Segment SGI384786[] = {{&S93178[11], 5}, {NULL}};
const Segment SGI384787[] = {{&S93184[11], 11}, {NULL}};
const Segment SGI384788[] = {{&S93184[23], 4}, {NULL}};
const Segment SGI384789[] = {{&S93184[51], 3}, {NULL}};
const Segment SGI384790[] = {{&S93307[11], 5}, {NULL}};
const Segment SGI384791[] = {{&S93307[79], 3}, {NULL}};
const Segment SGI384792[] = {{&S93554[11], 5}, {NULL}};
const Segment SGI384793[] = {{&S93554[39], 3}, {NULL}};
const Segment SGI384794[] = {{&S93582[11], 6}, {NULL}};
const Segment SGI384795[] = {{&S93582[81], 3}, {NULL}};
const Segment SGI384796[] = {{&S88830[52], 8}, {NULL}};
const Segment SGI384797[] = {{&S88830[40], 3}, {NULL}};
const Segment SGI384798[] = {{&S88830[23], 3}, {NULL}};
const Segment SGI384799[] = {{&S88830[13], 92}, {&S88830[124], 17}, {NULL}};
const Segment SGI384800[] = {{&S88831[51], 15}, {NULL}};
const Segment SGI384801[] = {{&S88831[39], 3}, {NULL}};
const Segment SGI384802[] = {{&S88831[22], 3}, {NULL}};
const Segment SGI384803[] = {{&S88831[129], 17}, {NULL}};
const Segment SGI384804[] = {{&S94396[11], 5}, {NULL}};
const Segment SGI384805[] = {{&S94396[46], 3}, {NULL}};
const Segment SGI384806[] = {{&S94482[11], 5}, {NULL}};
const Segment SGI384807[] = {{&S94647[11], 6}, {NULL}};
const Segment SGI384808[] = {{&S94651[11], 5}, {NULL}};
const Segment SGI384809[] = {{&S94652[11], 5}, {NULL}};
const Segment SGI384810[] = {{&S94652[128], 3}, {NULL}};
const Segment SGI384811[] = {{&S89549[52], 5}, {NULL}};
const Segment SGI384812[] = {{&S89549[40], 3}, {NULL}};
const Segment SGI384813[] = {{&S89549[23], 3}, {NULL}};
const Segment SGI384814[] = {{&S89549[13], 91}, {&S89549[123], 17}, {NULL}};
const Segment SGI384815[] = {{&S89550[52], 5}, {NULL}};
const Segment SGI384816[] = {{&S89550[40], 3}, {NULL}};
const Segment SGI384817[] = {{&S89550[23], 3}, {NULL}};
const Segment SGI384818[] = {{&S89550[13], 91}, {&S89550[123], 17}, {NULL}};
const Segment SGI384819[] = {{&S89551[52], 5}, {NULL}};
const Segment SGI384820[] = {{&S89551[40], 3}, {NULL}};
const Segment SGI384821[] = {{&S89551[23], 3}, {NULL}};
const Segment SGI384822[] = {{&S89551[13], 91}, {&S89551[123], 17}, {NULL}};
const Segment SGI384823[] = {{&S89552[52], 5}, {NULL}};
const Segment SGI384824[] = {{&S89552[40], 3}, {NULL}};
const Segment SGI384825[] = {{&S89552[23], 3}, {NULL}};
const Segment SGI384826[] = {{&S89552[13], 91}, {&S89552[123], 17}, {NULL}};
const Segment SGI384827[] = {{&S89553[52], 5}, {NULL}};
const Segment SGI384828[] = {{&S89553[40], 3}, {NULL}};
const Segment SGI384829[] = {{&S89553[23], 3}, {NULL}};
const Segment SGI384830[] = {{&S89553[13], 91}, {&S89553[123], 17}, {NULL}};
const Segment SGI384831[] = {{&S89554[52], 5}, {NULL}};
const Segment SGI384832[] = {{&S89554[40], 3}, {NULL}};
const Segment SGI384833[] = {{&S89554[23], 3}, {NULL}};
const Segment SGI384834[] = {{&S89554[13], 91}, {&S89554[123], 17}, {NULL}};
const Segment SGI384835[] = {{&S94777[11], 5}, {NULL}};
const Segment SGI384836[] = {{&S94777[37], 3}, {NULL}};
const Segment SGI384837[] = {{&S94953[27], 6}, {NULL}};
const Segment SGI384838[] = {{&S95378[11], 5}, {NULL}};
const Segment SGI384839[] = {{&S95478[27], 6}, {NULL}};
const Segment SGI384840[] = {{&S95679[11], 6}, {NULL}};
const Segment SGI384841[] = {{&S95679[42], 3}, {NULL}};
const Segment SGI384842[] = {{&S95693[11], 6}, {NULL}};
const Segment SGI384843[] = {{&S95693[40], 3}, {NULL}};
const Segment SGI384844[] = {{&S95742[11], 5}, {NULL}};
const Segment SGI384845[] = {{&S95744[29], 6}, {NULL}};
const Segment SGI384846[] = {{&S96110[11], 5}, {NULL}};
const Segment SGI384847[] = {{&S96452[110], 5}, {NULL}};
const Segment SGI384848[] = {{&S96452[178], 3}, {NULL}};
const Segment SGI384849[] = {{&S96503[11], 5}, {NULL}};
const Segment SGI384850[] = {{&S96503[79], 3}, {NULL}};
const Segment SGI384851[] = {{&S96602[11], 5}, {NULL}};
const Segment SGI384852[] = {{&S96602[32], 3}, {NULL}};
const Segment SGI384853[] = {{&S96603[11], 6}, {NULL}};
const Segment SGI384854[] = {{&S96603[41], 3}, {NULL}};
const Segment SGI384855[] = {{&S96915[11], 5}, {NULL}};
const Segment SGI384856[] = {{&S96915[73], 3}, {NULL}};
const Segment SGI384857[] = {{&S96930[11], 5}, {NULL}};
const Segment SGI384858[] = {{&S96930[79], 3}, {NULL}};
const Segment SGI384859[] = {{&S97102[11], 5}, {NULL}};
const Segment SGI384860[] = {{&S97262[11], 6}, {NULL}};
const Segment SGI384861[] = {{&S97262[31], 3}, {NULL}};
const Segment SGI384862[] = {{&S97266[11], 6}, {NULL}};
const Segment SGI384863[] = {{&S97266[18], 3}, {NULL}};
const Segment SGI384864[] = {{&S97322[11], 5}, {NULL}};
const Segment SGI384865[] = {{&S97477[11], 6}, {NULL}};
const Segment SGI384866[] = {{&S97477[31], 3}, {NULL}};
const Segment SGI384867[] = {{&S97552[11], 12}, {NULL}};
const Segment SGI384868[] = {{&S97552[24], 4}, {NULL}};
const Segment SGI384869[] = {{&S97631[11], 5}, {NULL}};
const Segment SGI384870[] = {{&S97631[30], 3}, {NULL}};
const Segment SGI384871[] = {{&S97647[11], 5}, {NULL}};
const Segment SGI384872[] = {{&S97651[11], 6}, {NULL}};
const Segment SGI384873[] = {{&S97651[42], 3}, {NULL}};
const Segment SGI384874[] = {{&S97655[11], 6}, {NULL}};
const Segment SGI384875[] = {{&S97814[11], 5}, {NULL}};
const Segment SGI384876[] = {{&S98072[11], 5}, {NULL}};
const Segment SGI384877[] = {{&S98074[11], 11}, {NULL}};
const Segment SGI384878[] = {{&S98247[11], 5}, {NULL}};
const Segment SGI384879[] = {{&S98254[27], 5}, {NULL}};
const Segment SGI384880[] = {{&S98536[11], 6}, {NULL}};
const Segment SGI384881[] = {{&S98536[42], 3}, {NULL}};
const Segment SGI384882[] = {{&S98537[11], 5}, {NULL}};
const Segment SGI384883[] = {{&S98537[38], 3}, {NULL}};
const Segment SGI384884[] = {{&S98876[11], 11}, {NULL}};
const Segment SGI384885[] = {{&S98876[23], 4}, {NULL}};
const Segment SGI384886[] = {{&S99053[11], 5}, {NULL}};
const Segment SGI384887[] = {{&S99204[11], 5}, {NULL}};
const Segment SGI384888[] = {{&S99204[79], 3}, {NULL}};
const Segment SGI384889[] = {{&S99549[11], 11}, {NULL}};
const Segment SGI384890[] = {{&S99551[11], 5}, {NULL}};
const Segment SGI384891[] = {{&S99551[128], 3}, {NULL}};
const Segment SGI384892[] = {{&S99552[11], 5}, {NULL}};
const Segment SGI384893[] = {{&S99738[111], 5}, {NULL}};
const Segment SGI384894[] = {{&S99738[179], 3}, {NULL}};
const Segment SGI384895[] = {{&S99772[27], 5}, {NULL}};
const Segment SGI384896[] = {{&S99857[27], 6}, {NULL}};
const Segment SGI384897[] = {{&S99906[11], 6}, {NULL}};
const Segment SGI384898[] = {{&S99964[11], 5}, {NULL}};
const Segment SGI384899[] = {{&S99964[74], 3}, {NULL}};
const Segment SGI384900[] = {{&S99983[11], 5}, {NULL}};
const Segment SGI384901[] = {{&S100300[29], 6}, {NULL}};
const Segment SGI384902[] = {{&S100605[27], 5}, {NULL}};
const Segment SGI384903[] = {{&S95133[52], 6}, {NULL}};
const Segment SGI384904[] = {{&S95133[40], 3}, {NULL}};
const Segment SGI384905[] = {{&S95133[23], 3}, {NULL}};
const Segment SGI384906[] = {{&S95133[13], 92}, {&S95133[124], 17}, {NULL}};
const Segment SGI384907[] = {{&S95134[52], 14}, {NULL}};
const Segment SGI384908[] = {{&S95134[40], 3}, {NULL}};
const Segment SGI384909[] = {{&S95134[23], 3}, {NULL}};
const Segment SGI384910[] = {{&S95134[13], 86}, {&S95134[118], 17}, {NULL}};
const Segment SGI384911[] = {{&S100831[11], 5}, {NULL}};
const Segment SGI384912[] = {{&S100889[11], 5}, {NULL}};
const Segment SGI384913[] = {{&S100951[11], 6}, {NULL}};
const Segment SGI384914[] = {{&S100951[63], 3}, {NULL}};
const Segment SGI384915[] = {{&S100959[11], 6}, {NULL}};
const Segment SGI384916[] = {{&S100981[11], 11}, {NULL}};
const Segment SGI384917[] = {{&S101149[11], 5}, {NULL}};
const Segment SGI384918[] = {{&S101149[30], 3}, {NULL}};
const Segment SGI384919[] = {{&S101163[11], 6}, {NULL}};
const Segment SGI384920[] = {{&S101163[40], 3}, {NULL}};
const Segment SGI384921[] = {{&S101178[11], 6}, {NULL}};
const Segment SGI384922[] = {{&S101190[11], 5}, {NULL}};
const Segment SGI384923[] = {{&S101222[29], 5}, {NULL}};
const Segment SGI384924[] = {{&S101311[11], 5}, {NULL}};
const Segment SGI384925[] = {{&S101311[112], 3}, {NULL}};
const Segment SGI384926[] = {{&S101427[11], 5}, {NULL}};
const Segment SGI384927[] = {{&S101432[11], 5}, {NULL}};
const Segment SGI384928[] = {{&S101432[82], 3}, {NULL}};
const Segment SGI384929[] = {{&S101437[62], 26}, {NULL}};
const Segment SGI384930[] = {{&S101438[23], 6}, {NULL}};
const Segment SGI384931[] = {{&S101441[11], 6}, {NULL}};
const Segment SGI384932[] = {{&S101441[73], 3}, {NULL}};
const Segment SGI384933[] = {{&S101455[11], 6}, {NULL}};
const Segment SGI384934[] = {{&S101455[42], 3}, {NULL}};
const Segment SGI384935[] = {{&S101458[11], 11}, {NULL}};
const Segment SGI384936[] = {{&S101590[11], 5}, {NULL}};
const Segment SGI384937[] = {{&S101594[29], 5}, {NULL}};
const Segment SGI384938[] = {{&S101681[11], 5}, {NULL}};
const Segment SGI384939[] = {{&S101816[11], 6}, {NULL}};
const Segment SGI384940[] = {{&S101816[42], 3}, {NULL}};
const Segment SGI384941[] = {{&S101820[11], 5}, {NULL}};
const Segment SGI384942[] = {{&S101912[29], 6}, {NULL}};
const Segment SGI384943[] = {{&S101977[11], 5}, {NULL}};
const Segment SGI384944[] = {{&S101977[30], 3}, {NULL}};
const Segment SGI384945[] = {{&S101984[11], 11}, {NULL}};
const Segment SGI384946[] = {{&S102006[11], 6}, {NULL}};
const Segment SGI384947[] = {{&S102006[18], 3}, {NULL}};
const Segment SGI384948[] = {{&S102019[11], 6}, {NULL}};
const Segment SGI384949[] = {{&S102024[11], 5}, {NULL}};
const Segment SGI384950[] = {{&S102158[11], 11}, {NULL}};
const Segment SGI384951[] = {{&S102191[11], 5}, {NULL}};
const Segment SGI384952[] = {{&S102191[72], 3}, {NULL}};
const Segment SGI384953[] = {{&S102203[11], 5}, {NULL}};
const Segment SGI384954[] = {{&S102203[39], 3}, {NULL}};
const Segment SGI384955[] = {{&S102223[31], 6}, {NULL}};
const Segment SGI384956[] = {{&S102339[11], 6}, {NULL}};
const Segment SGI384957[] = {{&S102339[42], 3}, {NULL}};
const Segment SGI384958[] = {{&S102438[11], 6}, {NULL}};
const Segment SGI384959[] = {{&S102533[29], 6}, {NULL}};
const Segment SGI384960[] = {{&S102553[11], 6}, {NULL}};
const Segment SGI384961[] = {{&S102553[114], 3}, {NULL}};
const Segment SGI384962[] = {{&S102556[11], 6}, {NULL}};
const Segment SGI384963[] = {{&S102556[102], 3}, {NULL}};
const Segment SGI384964[] = {{&S102572[11], 6}, {NULL}};
const Segment SGI384965[] = {{&S102572[18], 3}, {NULL}};
const Segment SGI384966[] = {{&S102638[31], 6}, {NULL}};
const Segment SGI384967[] = {{&S102800[11], 11}, {NULL}};
const Segment SGI384968[] = {{&S102956[48], 9}, {NULL}};
const Segment SGI384969[] = {{&S102997[11], 5}, {NULL}};
const Segment SGI384970[] = {{&S103009[11], 6}, {NULL}};
const Segment SGI384971[] = {{&S103010[11], 8}, {NULL}};
const Segment SGI384972[] = {{&S103011[11], 5}, {NULL}};
const Segment SGI384973[] = {{&S103038[11], 6}, {NULL}};
const Segment SGI384974[] = {{&S103038[42], 3}, {NULL}};
const Segment SGI384975[] = {{&S103069[11], 5}, {NULL}};
const Segment SGI384976[] = {{&S103223[48], 9}, {NULL}};
const Segment SGI384977[] = {{&S103235[11], 5}, {NULL}};
const Segment SGI384978[] = {{&S103277[11], 5}, {NULL}};
const Segment SGI384979[] = {{&S103277[82], 3}, {NULL}};
const Segment SGI384980[] = {{&S103281[11], 5}, {NULL}};
const Segment SGI384981[] = {{&S103292[11], 5}, {NULL}};
const Segment SGI384982[] = {{&S103293[11], 11}, {NULL}};
const Segment SGI384983[] = {{&S103294[11], 6}, {NULL}};
const Segment SGI384984[] = {{&S103298[26], 5}, {NULL}};
const Segment SGI384985[] = {{&S103508[48], 9}, {NULL}};
const Segment SGI384986[] = {{&S103586[11], 6}, {NULL}};
const Segment SGI384987[] = {{&S103590[26], 5}, {NULL}};
const Segment SGI384988[] = {{&S103632[11], 5}, {NULL}};
const Segment SGI384989[] = {{&S103632[51], 3}, {NULL}};
const Segment SGI384990[] = {{&S103667[31], 6}, {NULL}};
const Segment SGI384991[] = {{&S103831[48], 9}, {NULL}};
const Segment SGI384992[] = {{&S103862[11], 6}, {NULL}};
const Segment SGI384993[] = {{&S103862[38], 3}, {NULL}};
const Segment SGI384994[] = {{&S103868[41], 5}, {NULL}};
const Segment SGI384995[] = {{&S103868[6], 3}, {NULL}};
const Segment SGI384996[] = {{&S103887[11], 5}, {NULL}};
const Segment SGI384997[] = {{&S104119[11], 5}, {NULL}};
const Segment SGI384998[] = {{&S104160[11], 5}, {NULL}};
const Segment SGI384999[] = {{&S104160[79], 3}, {NULL}};
const Segment SGI385000[] = {{&S104162[11], 5}, {NULL}};
const Segment SGI385001[] = {{&S104162[79], 3}, {NULL}};
const Segment SGI385002[] = {{&S104218[11], 5}, {NULL}};
const Segment SGI385003[] = {{&S104218[47], 3}, {NULL}};
const Segment SGI385004[] = {{&S104225[11], 5}, {NULL}};
const Segment SGI385005[] = {{&S104251[11], 5}, {NULL}};
const Segment SGI385006[] = {{&S104255[11], 8}, {NULL}};
const Segment SGI385007[] = {{&S104265[11], 5}, {NULL}};
const Segment SGI385008[] = {{&S104265[79], 3}, {NULL}};
const Segment SGI385009[] = {{&S104270[11], 8}, {NULL}};
const Segment SGI385010[] = {{&S104403[27], 7}, {NULL}};
const Segment SGI385011[] = {{&S104613[11], 5}, {NULL}};
const Segment SGI385012[] = {{&S104613[39], 3}, {NULL}};
const Segment SGI385013[] = {{&S104636[11], 6}, {NULL}};
const Segment SGI385014[] = {{&S104636[42], 3}, {NULL}};
const Segment SGI385015[] = {{&S104637[11], 11}, {NULL}};
const Segment SGI385016[] = {{&S104637[33], 3}, {NULL}};
const Segment SGI385017[] = {{&S104642[11], 11}, {NULL}};
const Segment SGI385018[] = {{&S104642[148], 3}, {NULL}};
const Segment SGI385019[] = {{&S104664[11], 6}, {NULL}};
const Segment SGI385020[] = {{&S104781[31], 6}, {NULL}};
const Segment SGI385021[] = {{&S104847[11], 5}, {NULL}};
const Segment SGI385022[] = {{&S104851[11], 6}, {NULL}};
const Segment SGI385023[] = {{&S104852[27], 7}, {NULL}};
const Segment SGI385024[] = {{&S104855[11], 41}, {NULL}};
const Segment SGI385025[] = {{&S105088[48], 9}, {NULL}};
const Segment SGI385026[] = {{&S105142[11], 6}, {NULL}};
const Segment SGI385027[] = {{&S105142[42], 3}, {NULL}};
const Segment SGI385028[] = {{&S105178[33], 5}, {NULL}};
const Segment SGI385029[] = {{&S105178[39], 3}, {NULL}};
const Segment SGI385030[] = {{&S105186[11], 6}, {NULL}};
const Segment SGI385031[] = {{&S105186[61], 3}, {NULL}};
const Segment SGI385032[] = {{&S105202[11], 6}, {NULL}};
const Segment SGI385033[] = {{&S105277[11], 6}, {NULL}};
const Segment SGI385034[] = {{&S105277[56], 3}, {NULL}};
const Segment SGI385035[] = {{&S105327[33], 5}, {NULL}};
const Segment SGI385036[] = {{&S105327[52], 3}, {NULL}};
const Segment SGI385037[] = {{&S105473[29], 6}, {NULL}};
const Segment SGI385038[] = {{&S105570[11], 13}, {&S105570[11], 5}, {NULL}};
const Segment SGI385039[] = {{&S105571[11], 11}, {NULL}};
const Segment SGI385040[] = {{&S105571[23], 4}, {NULL}};
const Segment SGI385041[] = {{&S105571[51], 3}, {NULL}};
const Segment SGI385042[] = {{&S105997[48], 9}, {NULL}};
const Segment SGI385043[] = {{&S106000[48], 10}, {NULL}};
const Segment SGI385044[] = {{&S14586[57], 11}, {NULL}};
const Segment SGI385045[] = {{&S14588[57], 11}, {NULL}};
const Segment SGI385046[] = {{&S14589[57], 13}, {NULL}};
const Segment SGI385047[] = {{&S14590[57], 12}, {NULL}};
const Segment SGI385048[] = {{&S14591[57], 13}, {NULL}};
const Segment SGI385049[] = {{&S14592[57], 11}, {NULL}};
const Segment SGI385050[] = {{&S14593[57], 11}, {NULL}};
const Segment SGI385051[] = {{&S14594[57], 11}, {NULL}};
const Segment SGI385052[] = {{&S14602[57], 13}, {NULL}};
const Segment SGI385053[] = {{&S14603[57], 11}, {NULL}};
const Segment SGI385054[] = {{&S14604[57], 12}, {NULL}};
const Segment SGI385055[] = {{&S14605[57], 11}, {NULL}};
const Segment SGI385056[] = {{&S14606[57], 12}, {NULL}};
const Segment SGI385057[] = {{&S14607[57], 11}, {NULL}};
const Segment SGI385058[] = {{&S14609[57], 11}, {NULL}};
const Segment SGI385059[] = {{&S14610[57], 11}, {NULL}};
const Segment SGI385060[] = {{&S14616[55], 11}, {NULL}};
const Segment SGI385061[] = {{&S14617[57], 11}, {NULL}};
const Segment SGI385062[] = {{&S14618[55], 11}, {NULL}};
const Segment SGI385063[] = {{&S14619[57], 11}, {NULL}};
const Segment SGI385064[] = {{&S101173[52], 8}, {NULL}};
const Segment SGI385065[] = {{&S101173[40], 3}, {NULL}};
const Segment SGI385066[] = {{&S101173[23], 3}, {NULL}};
const Segment SGI385067[] = {{&S101173[13], 94}, {&S101173[126], 17}, {NULL}};
const Segment SGI385068[] = {{&S15558[57], 11}, {NULL}};
const Segment SGI385069[] = {{&S15559[57], 11}, {NULL}};
const Segment SGI385070[] = {{&S15560[57], 11}, {NULL}};
const Segment SGI385071[] = {{&S15561[57], 11}, {NULL}};
const Segment SGI385072[] = {{&S15562[57], 11}, {NULL}};
const Segment SGI385073[] = {{&S15563[57], 11}, {NULL}};
const Segment SGI385074[] = {{&S15565[57], 11}, {NULL}};
const Segment SGI385075[] = {{&S15566[57], 12}, {NULL}};
const Segment SGI385076[] = {{&S15567[57], 11}, {NULL}};
const Segment SGI385077[] = {{&S15569[57], 13}, {NULL}};
const Segment SGI385078[] = {{&S15570[57], 11}, {NULL}};
const Segment SGI385079[] = {{&S15571[57], 12}, {NULL}};
const Segment SGI385080[] = {{&S15572[57], 13}, {NULL}};
const Segment SGI385081[] = {{&S15574[57], 11}, {NULL}};
const Segment SGI385082[] = {{&S15575[57], 11}, {NULL}};
const Segment SGI385083[] = {{&S15576[57], 13}, {NULL}};
const Segment SGI385084[] = {{&S15577[57], 11}, {NULL}};
const Segment SGI385085[] = {{&S15579[57], 11}, {NULL}};
const Segment SGI385086[] = {{&S15580[57], 11}, {NULL}};
const Segment SGI385087[] = {{&S15581[57], 12}, {NULL}};
const Segment SGI385088[] = {{&S15585[57], 11}, {NULL}};
const Segment SGI385089[] = {{&S15589[57], 11}, {NULL}};
const Segment SGI385090[] = {{&S15590[57], 11}, {NULL}};
const Segment SGI385091[] = {{&S15591[57], 11}, {NULL}};
const Segment SGI385092[] = {{&S15599[55], 11}, {NULL}};
const Segment SGI385093[] = {{&S15600[55], 11}, {NULL}};
const Segment SGI385094[] = {{&S15605[57], 12}, {NULL}};
const Segment SGI385095[] = {{&S15607[57], 11}, {NULL}};
const Segment SGI385096[] = {{&S15787[57], 11}, {NULL}};
const Segment SGI385097[] = {{&S15795[57], 13}, {NULL}};
const Segment SGI385098[] = {{&S15798[57], 12}, {NULL}};
const Segment SGI385099[] = {{&S103017[52], 9}, {NULL}};
const Segment SGI385100[] = {{&S103017[40], 3}, {NULL}};
const Segment SGI385101[] = {{&S103017[23], 3}, {NULL}};
const Segment SGI385102[] = {{&S103017[13], 95}, {&S103017[127], 17}, {NULL}};
const Segment SGI385103[] = {{&S103018[52], 9}, {NULL}};
const Segment SGI385104[] = {{&S103018[40], 3}, {NULL}};
const Segment SGI385105[] = {{&S103018[23], 3}, {NULL}};
const Segment SGI385106[] = {{&S103018[13], 90}, {&S103018[122], 17}, {NULL}};
const Segment SGI385107[] = {{&S103557[52], 8}, {NULL}};
const Segment SGI385108[] = {{&S103557[40], 3}, {NULL}};
const Segment SGI385109[] = {{&S103557[23], 3}, {NULL}};
const Segment SGI385110[] = {{&S103557[13], 94}, {&S103557[126], 17}, {NULL}};
const Segment SGI385111[] = {{&S103558[52], 9}, {NULL}};
const Segment SGI385112[] = {{&S103558[40], 3}, {NULL}};
const Segment SGI385113[] = {{&S103558[23], 3}, {NULL}};
const Segment SGI385114[] = {{&S103558[13], 95}, {&S103558[127], 17}, {NULL}};
const Segment SGI385115[] = {{&S19001[57], 12}, {NULL}};
const Segment SGI385116[] = {{&S19002[57], 11}, {NULL}};
const Segment SGI385117[] = {{&S19003[57], 11}, {NULL}};
const Segment SGI385118[] = {{&S19004[57], 11}, {NULL}};
const Segment SGI385119[] = {{&S19005[57], 13}, {NULL}};
const Segment SGI385120[] = {{&S19006[57], 11}, {NULL}};
const Segment SGI385121[] = {{&S19007[57], 11}, {NULL}};
const Segment SGI385122[] = {{&S19008[57], 11}, {NULL}};
const Segment SGI385123[] = {{&S19010[57], 11}, {NULL}};
const Segment SGI385124[] = {{&S19011[57], 11}, {NULL}};
const Segment SGI385125[] = {{&S19013[57], 13}, {NULL}};
const Segment SGI385126[] = {{&S19014[57], 11}, {NULL}};
const Segment SGI385127[] = {{&S19016[57], 11}, {NULL}};
const Segment SGI385128[] = {{&S19018[57], 11}, {NULL}};
const Segment SGI385129[] = {{&S19019[57], 11}, {NULL}};
const Segment SGI385130[] = {{&S19020[57], 11}, {NULL}};
const Segment SGI385131[] = {{&S19021[57], 12}, {NULL}};
const Segment SGI385132[] = {{&S19022[57], 11}, {NULL}};
const Segment SGI385133[] = {{&S19023[55], 11}, {NULL}};
const Segment SGI385134[] = {{&S19024[57], 11}, {NULL}};
const Segment SGI385135[] = {{&S19025[57], 10}, {NULL}};
const Segment SGI385136[] = {{&S19026[57], 11}, {NULL}};
const Segment SGI385137[] = {{&S19027[57], 11}, {NULL}};
const Segment SGI385138[] = {{&S19028[57], 11}, {NULL}};
const Segment SGI385139[] = {{&S19029[57], 11}, {NULL}};
const Segment SGI385140[] = {{&S19030[57], 12}, {NULL}};
const Segment SGI385141[] = {{&S19031[57], 11}, {NULL}};
const Segment SGI385142[] = {{&S19032[57], 13}, {NULL}};
const Segment SGI385143[] = {{&S19035[57], 11}, {NULL}};
const Segment SGI385144[] = {{&S19036[57], 11}, {NULL}};
const Segment SGI385145[] = {{&S19037[57], 13}, {NULL}};
const Segment SGI385146[] = {{&S19038[57], 12}, {NULL}};
const Segment SGI385147[] = {{&S19039[57], 11}, {NULL}};
const Segment SGI385148[] = {{&S19040[57], 11}, {NULL}};
const Segment SGI385149[] = {{&S19041[57], 11}, {NULL}};
const Segment SGI385150[] = {{&S19042[57], 11}, {NULL}};
const Segment SGI385151[] = {{&S19043[57], 11}, {NULL}};
const Segment SGI385152[] = {{&S19044[57], 11}, {NULL}};
const Segment SGI385153[] = {{&S19045[57], 11}, {NULL}};
const Segment SGI385154[] = {{&S19046[57], 10}, {NULL}};
const Segment SGI385155[] = {{&S19047[57], 11}, {NULL}};
const Segment SGI385156[] = {{&S19048[57], 11}, {NULL}};
const Segment SGI385157[] = {{&S19049[57], 11}, {NULL}};
const Segment SGI385158[] = {{&S19050[57], 12}, {NULL}};
const Segment SGI385159[] = {{&S19052[57], 11}, {NULL}};
const Segment SGI385160[] = {{&S19053[57], 11}, {NULL}};
const Segment SGI385161[] = {{&S19054[57], 11}, {NULL}};
const Segment SGI385162[] = {{&S19055[57], 11}, {NULL}};
const Segment SGI385163[] = {{&S19056[57], 11}, {NULL}};
const Segment SGI385164[] = {{&S19057[57], 11}, {NULL}};
const Segment SGI385165[] = {{&S19058[57], 11}, {NULL}};
const Segment SGI385166[] = {{&S19059[57], 11}, {NULL}};
const Segment SGI385167[] = {{&S19063[57], 13}, {NULL}};
const Segment SGI385168[] = {{&S19064[57], 12}, {NULL}};
const Segment SGI385169[] = {{&S19065[57], 11}, {NULL}};
const Segment SGI385170[] = {{&S19066[57], 10}, {NULL}};
const Segment SGI385171[] = {{&S19067[57], 11}, {NULL}};
const Segment SGI385172[] = {{&S19069[57], 12}, {NULL}};
const Segment SGI385173[] = {{&S19070[57], 13}, {NULL}};
const Segment SGI385174[] = {{&S19082[55], 11}, {NULL}};
const Segment SGI385175[] = {{&S19086[57], 11}, {NULL}};
const Segment SGI385176[] = {{&S19088[57], 11}, {NULL}};
const Segment SGI385177[] = {{&S19089[57], 12}, {NULL}};
const Segment SGI385178[] = {{&S20209[57], 11}, {NULL}};
const Segment SGI385179[] = {{&S20210[57], 11}, {NULL}};
const Segment SGI385180[] = {{&S20211[57], 11}, {NULL}};
const Segment SGI385181[] = {{&S20212[57], 11}, {NULL}};
const Segment SGI385182[] = {{&S20213[57], 11}, {NULL}};
const Segment SGI385183[] = {{&S20214[57], 11}, {NULL}};
const Segment SGI385184[] = {{&S20216[57], 12}, {NULL}};
const Segment SGI385185[] = {{&S20217[57], 11}, {NULL}};
const Segment SGI385186[] = {{&S20218[57], 11}, {NULL}};
const Segment SGI385187[] = {{&S20219[57], 11}, {NULL}};
const Segment SGI385188[] = {{&S20220[57], 11}, {NULL}};
const Segment SGI385189[] = {{&S20223[57], 12}, {NULL}};
const Segment SGI385190[] = {{&S20224[57], 11}, {NULL}};
const Segment SGI385191[] = {{&S20225[57], 12}, {NULL}};
const Segment SGI385192[] = {{&S20226[57], 11}, {NULL}};
const Segment SGI385193[] = {{&S20227[57], 11}, {NULL}};
const Segment SGI385194[] = {{&S20228[57], 11}, {NULL}};
const Segment SGI385195[] = {{&S20229[57], 12}, {NULL}};
const Segment SGI385196[] = {{&S20230[57], 11}, {NULL}};
const Segment SGI385197[] = {{&S20231[57], 11}, {NULL}};
const Segment SGI385198[] = {{&S20233[57], 12}, {NULL}};
const Segment SGI385199[] = {{&S20235[57], 12}, {NULL}};
const Segment SGI385200[] = {{&S20236[57], 12}, {NULL}};
const Segment SGI385201[] = {{&S20237[57], 11}, {NULL}};
const Segment SGI385202[] = {{&S20239[57], 12}, {NULL}};
const Segment SGI385203[] = {{&S20243[57], 11}, {NULL}};
const Segment SGI385204[] = {{&S20247[57], 11}, {NULL}};
const Segment SGI385205[] = {{&S20255[57], 11}, {NULL}};
const Segment SGI385206[] = {{&S20256[57], 12}, {NULL}};
const Segment SGI385207[] = {{&S20258[55], 11}, {NULL}};
const Segment SGI385208[] = {{&S20259[57], 12}, {NULL}};
const Segment SGI385209[] = {{&S21247[57], 11}, {NULL}};
const Segment SGI385210[] = {{&S21258[57], 11}, {NULL}};
const Segment SGI385211[] = {{&S21259[57], 12}, {NULL}};
const Segment SGI385212[] = {{&S21263[57], 13}, {NULL}};
const Segment SGI385213[] = {{&S21964[57], 11}, {NULL}};
const Segment SGI385214[] = {{&S21965[57], 11}, {NULL}};
const Segment SGI385215[] = {{&S21966[57], 11}, {NULL}};
const Segment SGI385216[] = {{&S21967[57], 12}, {NULL}};
const Segment SGI385217[] = {{&S21969[57], 11}, {NULL}};
const Segment SGI385218[] = {{&S21970[57], 13}, {NULL}};
const Segment SGI385219[] = {{&S21972[57], 11}, {NULL}};
const Segment SGI385220[] = {{&S21973[57], 13}, {NULL}};
const Segment SGI385221[] = {{&S21974[57], 11}, {NULL}};
const Segment SGI385222[] = {{&S21978[57], 11}, {NULL}};
const Segment SGI385223[] = {{&S21979[57], 11}, {NULL}};
const Segment SGI385224[] = {{&S21980[57], 13}, {NULL}};
const Segment SGI385225[] = {{&S21981[57], 11}, {NULL}};
const Segment SGI385226[] = {{&S21983[57], 11}, {NULL}};
const Segment SGI385227[] = {{&S21985[57], 11}, {NULL}};
const Segment SGI385228[] = {{&S21986[57], 11}, {NULL}};
const Segment SGI385229[] = {{&S21987[57], 11}, {NULL}};
const Segment SGI385230[] = {{&S21988[57], 13}, {NULL}};
const Segment SGI385231[] = {{&S21989[57], 12}, {NULL}};
const Segment SGI385232[] = {{&S21991[57], 13}, {NULL}};
const Segment SGI385233[] = {{&S44448[0], 20}, {NULL}};
const Segment SGI385234[] = {{&S21992[57], 12}, {NULL}};
const Segment SGI385235[] = {{&S44449[0], 20}, {NULL}};
const Segment SGI385236[] = {{&S44450[0], 20}, {NULL}};
const Segment SGI385237[] = {{&S44451[0], 20}, {NULL}};
const Segment SGI385238[] = {{&S44452[0], 20}, {NULL}};
const Segment SGI385239[] = {{&S44453[0], 20}, {NULL}};
const Segment SGI385240[] = {{&S44454[0], 20}, {NULL}};
const Segment SGI385241[] = {{&S44456[0], 20}, {NULL}};
const Segment SGI385242[] = {{&S22000[57], 12}, {NULL}};
const Segment SGI385243[] = {{&S22692[63], 11}, {NULL}};
const Segment SGI385244[] = {{&S22693[63], 10}, {NULL}};
const Segment SGI385245[] = {{&S22694[63], 12}, {NULL}};
const Segment SGI385246[] = {{&S22695[63], 10}, {NULL}};
const Segment SGI385247[] = {{&S22699[63], 11}, {NULL}};
const Segment SGI385248[] = {{&S22706[63], 9}, {NULL}};
const Segment SGI385249[] = {{&S23626[57], 11}, {NULL}};
const Segment SGI385250[] = {{&S23627[57], 11}, {NULL}};
const Segment SGI385251[] = {{&S23628[57], 13}, {NULL}};
const Segment SGI385252[] = {{&S23629[57], 12}, {NULL}};
const Segment SGI385253[] = {{&S23630[57], 11}, {NULL}};
const Segment SGI385254[] = {{&S23631[57], 12}, {NULL}};
const Segment SGI385255[] = {{&S23634[57], 12}, {NULL}};
const Segment SGI385256[] = {{&S23636[57], 13}, {NULL}};
const Segment SGI385257[] = {{&S23637[57], 11}, {NULL}};
const Segment SGI385258[] = {{&S23638[57], 11}, {NULL}};
const Segment SGI385259[] = {{&S23639[57], 13}, {NULL}};
const Segment SGI385260[] = {{&S23640[57], 11}, {NULL}};
const Segment SGI385261[] = {{&S23643[57], 11}, {NULL}};
const Segment SGI385262[] = {{&S23647[57], 11}, {NULL}};
const Segment SGI385263[] = {{&S23648[57], 11}, {NULL}};
const Segment SGI385264[] = {{&S23651[57], 11}, {NULL}};
const Segment SGI385265[] = {{&S23658[55], 11}, {NULL}};
const Segment SGI385266[] = {{&S46770[0], 20}, {NULL}};
const Segment SGI385267[] = {{&S46771[0], 20}, {NULL}};
const Segment SGI385268[] = {{&S46772[0], 20}, {NULL}};
const Segment SGI385269[] = {{&S46773[0], 20}, {NULL}};
const Segment SGI385270[] = {{&S46774[0], 20}, {NULL}};
const Segment SGI385271[] = {{&S46776[0], 20}, {NULL}};
const Segment SGI385272[] = {{&S24223[57], 11}, {NULL}};
const Segment SGI385273[] = {{&S24227[57], 13}, {NULL}};
const Segment SGI385274[] = {{&S47851[0], 20}, {NULL}};
const Segment SGI385275[] = {{&S47852[0], 20}, {NULL}};
const Segment SGI385276[] = {{&S47853[0], 20}, {NULL}};
const Segment SGI385277[] = {{&S47854[0], 20}, {NULL}};
const Segment SGI385278[] = {{&S24758[57], 13}, {NULL}};
const Segment SGI385279[] = {{&S24759[57], 13}, {NULL}};
const Segment SGI385280[] = {{&S24760[57], 11}, {NULL}};
const Segment SGI385281[] = {{&S24762[57], 13}, {NULL}};
const Segment SGI385282[] = {{&S24764[57], 11}, {NULL}};
const Segment SGI385283[] = {{&S24766[57], 11}, {NULL}};
const Segment SGI385284[] = {{&S24768[57], 11}, {NULL}};
const Segment SGI385285[] = {{&S24770[57], 11}, {NULL}};
const Segment SGI385286[] = {{&S24772[57], 13}, {NULL}};
const Segment SGI385287[] = {{&S24775[57], 11}, {NULL}};
const Segment SGI385288[] = {{&S24777[57], 12}, {NULL}};
const Segment SGI385289[] = {{&S25852[57], 11}, {NULL}};
const Segment SGI385290[] = {{&S25854[57], 11}, {NULL}};
const Segment SGI385291[] = {{&S25855[57], 11}, {NULL}};
const Segment SGI385292[] = {{&S25856[57], 11}, {NULL}};
const Segment SGI385293[] = {{&S25857[57], 11}, {NULL}};
const Segment SGI385294[] = {{&S25858[57], 11}, {NULL}};
const Segment SGI385295[] = {{&S25859[57], 11}, {NULL}};
const Segment SGI385296[] = {{&S25862[57], 12}, {NULL}};
const Segment SGI385297[] = {{&S25863[57], 13}, {NULL}};
const Segment SGI385298[] = {{&S25864[57], 11}, {NULL}};
const Segment SGI385299[] = {{&S25866[57], 12}, {NULL}};
const Segment SGI385300[] = {{&S25867[57], 11}, {NULL}};
const Segment SGI385301[] = {{&S25868[57], 11}, {NULL}};
const Segment SGI385302[] = {{&S25869[57], 11}, {NULL}};
const Segment SGI385303[] = {{&S25870[57], 11}, {NULL}};
const Segment SGI385304[] = {{&S25871[57], 11}, {NULL}};
const Segment SGI385305[] = {{&S25872[57], 11}, {NULL}};
const Segment SGI385306[] = {{&S25873[57], 11}, {NULL}};
const Segment SGI385307[] = {{&S25874[57], 12}, {NULL}};
const Segment SGI385308[] = {{&S25875[57], 11}, {NULL}};
const Segment SGI385309[] = {{&S25876[57], 11}, {NULL}};
const Segment SGI385310[] = {{&S25877[57], 11}, {NULL}};
const Segment SGI385311[] = {{&S25878[57], 11}, {NULL}};
const Segment SGI385312[] = {{&S25879[57], 11}, {NULL}};
const Segment SGI385313[] = {{&S25881[57], 13}, {NULL}};
const Segment SGI385314[] = {{&S25882[57], 11}, {NULL}};
const Segment SGI385315[] = {{&S25884[57], 11}, {NULL}};
const Segment SGI385316[] = {{&S25885[57], 11}, {NULL}};
const Segment SGI385317[] = {{&S25886[57], 11}, {NULL}};
const Segment SGI385318[] = {{&S25888[57], 11}, {NULL}};
const Segment SGI385319[] = {{&S25889[57], 11}, {NULL}};
const Segment SGI385320[] = {{&S25890[57], 11}, {NULL}};
const Segment SGI385321[] = {{&S25891[57], 11}, {NULL}};
const Segment SGI385322[] = {{&S25892[57], 11}, {NULL}};
const Segment SGI385323[] = {{&S25893[57], 13}, {NULL}};
const Segment SGI385324[] = {{&S25894[57], 11}, {NULL}};
const Segment SGI385325[] = {{&S25896[57], 11}, {NULL}};
const Segment SGI385326[] = {{&S25900[57], 12}, {NULL}};
const Segment SGI385327[] = {{&S25902[57], 11}, {NULL}};
const Segment SGI385328[] = {{&S25904[57], 12}, {NULL}};
const Segment SGI385329[] = {{&S25905[57], 13}, {NULL}};
const Segment SGI385330[] = {{&S25906[57], 13}, {NULL}};
const Segment SGI385331[] = {{&S25907[57], 11}, {NULL}};
const Segment SGI385332[] = {{&S25910[57], 11}, {NULL}};
const Segment SGI385333[] = {{&S25911[57], 12}, {NULL}};
const Segment SGI385334[] = {{&S25915[57], 11}, {NULL}};
const Segment SGI385335[] = {{&S26458[57], 12}, {NULL}};
const Segment SGI385336[] = {{&S26459[57], 11}, {NULL}};
const Segment SGI385337[] = {{&S26461[57], 11}, {NULL}};
const Segment SGI385338[] = {{&S26733[57], 11}, {NULL}};
const Segment SGI385339[] = {{&S26735[57], 11}, {NULL}};
const Segment SGI385340[] = {{&S26737[57], 11}, {NULL}};
const Segment SGI385341[] = {{&S26738[57], 11}, {NULL}};
const Segment SGI385342[] = {{&S26739[55], 11}, {NULL}};
const Segment SGI385343[] = {{&S26740[57], 11}, {NULL}};
const Segment SGI385344[] = {{&S26741[57], 11}, {NULL}};
const Segment SGI385345[] = {{&S26743[57], 11}, {NULL}};
const Segment SGI385346[] = {{&S26744[57], 11}, {NULL}};
const Segment SGI385347[] = {{&S26745[57], 11}, {NULL}};
const Segment SGI385348[] = {{&S26746[57], 11}, {NULL}};
const Segment SGI385349[] = {{&S26748[57], 11}, {NULL}};
const Segment SGI385350[] = {{&S26749[57], 11}, {NULL}};
const Segment SGI385351[] = {{&S26750[57], 11}, {NULL}};
const Segment SGI385352[] = {{&S26751[57], 11}, {NULL}};
const Segment SGI385353[] = {{&S26752[57], 11}, {NULL}};
const Segment SGI385354[] = {{&S26754[57], 11}, {NULL}};
const Segment SGI385355[] = {{&S26755[57], 11}, {NULL}};
const Segment SGI385356[] = {{&S26757[57], 11}, {NULL}};
const Segment SGI385357[] = {{&S26758[57], 11}, {NULL}};
const Segment SGI385358[] = {{&S26759[57], 11}, {NULL}};
const Segment SGI385359[] = {{&S26760[57], 11}, {NULL}};
const Segment SGI385360[] = {{&S26761[57], 11}, {NULL}};
const Segment SGI385361[] = {{&S26762[57], 11}, {NULL}};
const Segment SGI385362[] = {{&S26763[57], 11}, {NULL}};
const Segment SGI385363[] = {{&S26765[57], 11}, {NULL}};
const Segment SGI385364[] = {{&S26767[57], 11}, {NULL}};
const Segment SGI385365[] = {{&S26768[57], 11}, {NULL}};
const Segment SGI385366[] = {{&S26771[57], 11}, {NULL}};
const Segment SGI385367[] = {{&S26772[55], 11}, {NULL}};
const Segment SGI385368[] = {{&S26776[57], 11}, {NULL}};
const Segment SGI385369[] = {{&S26778[57], 11}, {NULL}};
const Segment SGI385370[] = {{&S26782[57], 11}, {NULL}};
const Segment SGI385371[] = {{&S26787[57], 11}, {NULL}};
const Segment SGI385372[] = {{&S26948[57], 11}, {NULL}};
const Segment SGI385373[] = {{&S26949[57], 11}, {NULL}};
const Segment SGI385374[] = {{&S26950[57], 11}, {NULL}};
const Segment SGI385375[] = {{&S26952[57], 11}, {NULL}};
const Segment SGI385376[] = {{&S26953[57], 11}, {NULL}};
const Segment SGI385377[] = {{&S26956[55], 11}, {NULL}};
const Segment SGI385378[] = {{&S26957[57], 11}, {NULL}};
const Segment SGI385379[] = {{&S26958[57], 13}, {NULL}};
const Segment SGI385380[] = {{&S26961[57], 11}, {NULL}};
const Segment SGI385381[] = {{&S26963[57], 11}, {NULL}};
const Segment SGI385382[] = {{&S26964[57], 12}, {NULL}};
const Segment SGI385383[] = {{&S26965[57], 12}, {NULL}};
const Segment SGI385384[] = {{&S26967[57], 11}, {NULL}};
const Segment SGI385385[] = {{&S26968[57], 12}, {NULL}};
const Segment SGI385386[] = {{&S26969[57], 13}, {NULL}};
const Segment SGI385387[] = {{&S26971[57], 12}, {NULL}};
const Segment SGI385388[] = {{&S27017[57], 11}, {NULL}};
const Segment SGI385389[] = {{&S27019[57], 11}, {NULL}};
const Segment SGI385390[] = {{&S27022[57], 11}, {NULL}};
const Segment SGI385391[] = {{&S27023[57], 11}, {NULL}};
const Segment SGI385392[] = {{&S27025[57], 12}, {NULL}};
const Segment SGI385393[] = {{&S27029[55], 11}, {NULL}};
const Segment SGI385394[] = {{&S27915[57], 10}, {NULL}};
const Segment SGI385395[] = {{&S27916[57], 10}, {NULL}};
const Segment SGI385396[] = {{&S27917[57], 10}, {NULL}};
const Segment SGI385397[] = {{&S27918[57], 11}, {NULL}};
const Segment SGI385398[] = {{&S27919[57], 10}, {NULL}};
const Segment SGI385399[] = {{&S27920[57], 12}, {NULL}};
const Segment SGI385400[] = {{&S27921[57], 10}, {NULL}};
const Segment SGI385401[] = {{&S27923[57], 10}, {NULL}};
const Segment SGI385402[] = {{&S27924[57], 10}, {NULL}};
const Segment SGI385403[] = {{&S27925[57], 10}, {NULL}};
const Segment SGI385404[] = {{&S27926[57], 10}, {NULL}};
const Segment SGI385405[] = {{&S27929[71], 10}, {NULL}};
const Segment SGI385406[] = {{&S27933[57], 10}, {NULL}};
const Segment SGI385407[] = {{&S27937[57], 10}, {NULL}};
const Segment SGI385408[] = {{&S27939[55], 10}, {NULL}};
const Segment SGI385409[] = {{&S27943[57], 10}, {NULL}};
const Segment SGI385410[] = {{&S27947[57], 9}, {NULL}};
const Segment SGI385411[] = {{&S51429[0], 25}, {NULL}};
const Segment SGI385412[] = {{&S51430[0], 25}, {NULL}};
const Segment SGI385413[] = {{&S51431[0], 25}, {NULL}};
const Segment SGI385414[] = {{&S51432[0], 25}, {NULL}};
const Segment SGI385415[] = {{&S51433[0], 25}, {NULL}};
const Segment SGI385416[] = {{&S51434[0], 25}, {NULL}};
const Segment SGI385417[] = {{&S51435[0], 25}, {NULL}};
const Segment SGI385418[] = {{&S51436[0], 25}, {NULL}};
const Segment SGI385419[] = {{&S51437[0], 25}, {NULL}};
const Segment SGI385420[] = {{&S29866[57], 11}, {NULL}};
const Segment SGI385421[] = {{&S53285[0], 15}, {NULL}};
const Segment SGI385422[] = {{&S29868[57], 11}, {NULL}};
const Segment SGI385423[] = {{&S29872[57], 12}, {NULL}};
const Segment SGI385424[] = {{&S29873[57], 12}, {NULL}};
const Segment SGI385425[] = {{&S29878[57], 11}, {NULL}};
const Segment SGI385426[] = {{&S29880[57], 11}, {NULL}};
const Segment SGI385427[] = {{&S29881[57], 11}, {NULL}};
const Segment SGI385428[] = {{&S29882[57], 11}, {NULL}};
const Segment SGI385429[] = {{&S29883[57], 11}, {NULL}};
const Segment SGI385430[] = {{&S29884[57], 11}, {NULL}};
const Segment SGI385431[] = {{&S29885[57], 11}, {NULL}};
const Segment SGI385432[] = {{&S29886[57], 11}, {NULL}};
const Segment SGI385433[] = {{&S29887[57], 11}, {NULL}};
const Segment SGI385434[] = {{&S29888[57], 11}, {NULL}};
const Segment SGI385435[] = {{&S29890[57], 11}, {NULL}};
const Segment SGI385436[] = {{&S29892[57], 11}, {NULL}};
const Segment SGI385437[] = {{&S29896[57], 11}, {NULL}};
const Segment SGI385438[] = {{&S29897[57], 11}, {NULL}};
const Segment SGI385439[] = {{&S29899[57], 11}, {NULL}};
const Segment SGI385440[] = {{&S29900[57], 11}, {NULL}};
const Segment SGI385441[] = {{&S29904[57], 11}, {NULL}};
const Segment SGI385442[] = {{&S29905[57], 12}, {NULL}};
const Segment SGI385443[] = {{&S29906[57], 11}, {NULL}};
const Segment SGI385444[] = {{&S29908[57], 11}, {NULL}};
const Segment SGI385445[] = {{&S29909[57], 11}, {NULL}};
const Segment SGI385446[] = {{&S29910[57], 12}, {NULL}};
const Segment SGI385447[] = {{&S29914[57], 12}, {NULL}};
const Segment SGI385448[] = {{&S29917[55], 11}, {NULL}};
const Segment SGI385449[] = {{&S30262[57], 16}, {NULL}};
const Segment SGI385450[] = {{&S30264[57], 11}, {NULL}};
const Segment SGI385451[] = {{&S30266[57], 12}, {NULL}};
const Segment SGI385452[] = {{&S31538[57], 11}, {NULL}};
const Segment SGI385453[] = {{&S31540[57], 11}, {NULL}};
const Segment SGI385454[] = {{&S31541[57], 12}, {NULL}};
const Segment SGI385455[] = {{&S31542[57], 13}, {NULL}};
const Segment SGI385456[] = {{&S31945[57], 11}, {NULL}};
const Segment SGI385457[] = {{&S31947[57], 11}, {NULL}};
const Segment SGI385458[] = {{&S31948[57], 11}, {NULL}};
const Segment SGI385459[] = {{&S31949[57], 11}, {NULL}};
const Segment SGI385460[] = {{&S31954[57], 11}, {NULL}};
const Segment SGI385461[] = {{&S31956[55], 11}, {NULL}};
const Segment SGI385462[] = {{&S32002[63], 11}, {NULL}};
const Segment SGI385463[] = {{&S32008[63], 10}, {NULL}};
const Segment SGI385464[] = {{&S32050[57], 11}, {NULL}};
const Segment SGI385465[] = {{&S32051[57], 11}, {NULL}};
const Segment SGI385466[] = {{&S32054[57], 12}, {NULL}};
const Segment SGI385467[] = {{&S32106[57], 11}, {NULL}};
const Segment SGI385468[] = {{&S32108[57], 11}, {NULL}};
const Segment SGI385469[] = {{&S32109[57], 12}, {NULL}};
const Segment SGI385470[] = {{&S55467[0], 15}, {NULL}};
const Segment SGI385471[] = {{&S55468[0], 15}, {NULL}};
const Segment SGI385472[] = {{&S55469[0], 15}, {NULL}};
const Segment SGI385473[] = {{&S55470[0], 15}, {NULL}};
const Segment SGI385474[] = {{&S55471[0], 15}, {NULL}};
const Segment SGI385475[] = {{&S55472[0], 15}, {NULL}};
const Segment SGI385476[] = {{&S55473[0], 15}, {NULL}};
const Segment SGI385477[] = {{&S55474[0], 15}, {NULL}};
const Segment SGI385478[] = {{&S55507[0], 20}, {NULL}};
const Segment SGI385479[] = {{&S55508[0], 20}, {NULL}};
const Segment SGI385480[] = {{&S55509[0], 20}, {NULL}};
const Segment SGI385481[] = {{&S55510[0], 20}, {NULL}};
const Segment SGI385482[] = {{&S55511[0], 20}, {NULL}};
const Segment SGI385483[] = {{&S55512[0], 20}, {NULL}};
const Segment SGI385484[] = {{&S55513[0], 20}, {NULL}};
const Segment SGI385485[] = {{&S55514[0], 20}, {NULL}};
const Segment SGI385486[] = {{&S55515[0], 20}, {NULL}};
const Segment SGI385487[] = {{&S55516[0], 20}, {NULL}};
const Segment SGI385488[] = {{&S55517[0], 20}, {NULL}};
const Segment SGI385489[] = {{&S55518[0], 20}, {NULL}};
const Segment SGI385490[] = {{&S55519[0], 20}, {NULL}};
const Segment SGI385491[] = {{&S32885[57], 11}, {NULL}};
const Segment SGI385492[] = {{&S32886[57], 13}, {NULL}};
const Segment SGI385493[] = {{&S32887[57], 11}, {NULL}};
const Segment SGI385494[] = {{&S32891[57], 11}, {NULL}};
const Segment SGI385495[] = {{&S32892[57], 13}, {NULL}};
const Segment SGI385496[] = {{&S32893[57], 11}, {NULL}};
const Segment SGI385497[] = {{&S32895[57], 11}, {NULL}};
const Segment SGI385498[] = {{&S32896[57], 13}, {NULL}};
const Segment SGI385499[] = {{&S32897[57], 11}, {NULL}};
const Segment SGI385500[] = {{&S32899[57], 11}, {NULL}};
const Segment SGI385501[] = {{&S32900[57], 11}, {NULL}};
const Segment SGI385502[] = {{&S32901[57], 13}, {NULL}};
const Segment SGI385503[] = {{&S33217[63], 10}, {NULL}};
const Segment SGI385504[] = {{&S33218[63], 11}, {NULL}};
const Segment SGI385505[] = {{&S33327[57], 11}, {NULL}};
const Segment SGI385506[] = {{&S33485[57], 11}, {NULL}};
const Segment SGI385507[] = {{&S33486[57], 11}, {NULL}};
const Segment SGI385508[] = {{&S33488[57], 11}, {NULL}};
const Segment SGI385509[] = {{&S33490[57], 11}, {NULL}};
const Segment SGI385510[] = {{&S33492[57], 11}, {NULL}};
const Segment SGI385511[] = {{&S33493[57], 10}, {NULL}};
const Segment SGI385512[] = {{&S33498[57], 11}, {NULL}};
const Segment SGI385513[] = {{&S33500[57], 10}, {NULL}};
const Segment SGI385514[] = {{&S33501[57], 11}, {NULL}};
const Segment SGI385515[] = {{&S33502[57], 10}, {NULL}};
const Segment SGI385516[] = {{&S33503[56], 11}, {NULL}};
const Segment SGI385517[] = {{&S33504[56], 11}, {NULL}};
const Segment SGI385518[] = {{&S57851[0], 20}, {NULL}};
const Segment SGI385519[] = {{&S57852[0], 20}, {NULL}};
const Segment SGI385520[] = {{&S57853[0], 20}, {NULL}};
const Segment SGI385521[] = {{&S34975[57], 11}, {NULL}};
const Segment SGI385522[] = {{&S34977[57], 11}, {NULL}};
const Segment SGI385523[] = {{&S34980[57], 12}, {NULL}};
const Segment SGI385524[] = {{&S35013[63], 12}, {NULL}};
const Segment SGI385525[] = {{&S35014[63], 11}, {NULL}};
const Segment SGI385526[] = {{&S35023[63], 10}, {NULL}};
const Segment SGI385527[] = {{&S35993[57], 11}, {NULL}};
const Segment SGI385528[] = {{&S36565[57], 11}, {NULL}};
const Segment SGI385529[] = {{&S36566[57], 11}, {NULL}};
const Segment SGI385530[] = {{&S36567[57], 11}, {NULL}};
const Segment SGI385531[] = {{&S36568[57], 11}, {NULL}};
const Segment SGI385532[] = {{&S36569[57], 11}, {NULL}};
const Segment SGI385533[] = {{&S36570[57], 11}, {NULL}};
const Segment SGI385534[] = {{&S36571[57], 11}, {NULL}};
const Segment SGI385535[] = {{&S36572[57], 11}, {NULL}};
const Segment SGI385536[] = {{&S36573[57], 11}, {NULL}};
const Segment SGI385537[] = {{&S36574[57], 11}, {NULL}};
const Segment SGI385538[] = {{&S36575[57], 11}, {NULL}};
const Segment SGI385539[] = {{&S36576[57], 11}, {NULL}};
const Segment SGI385540[] = {{&S36577[57], 11}, {NULL}};
const Segment SGI385541[] = {{&S36578[57], 11}, {NULL}};
const Segment SGI385542[] = {{&S36581[57], 11}, {NULL}};
const Segment SGI385543[] = {{&S36582[57], 11}, {NULL}};
const Segment SGI385544[] = {{&S36583[57], 11}, {NULL}};
const Segment SGI385545[] = {{&S36585[57], 12}, {NULL}};
const Segment SGI385546[] = {{&S36586[57], 11}, {NULL}};
const Segment SGI385547[] = {{&S36587[57], 11}, {NULL}};
const Segment SGI385548[] = {{&S36589[57], 11}, {NULL}};
const Segment SGI385549[] = {{&S36593[57], 11}, {NULL}};
const Segment SGI385550[] = {{&S36594[57], 11}, {NULL}};
const Segment SGI385551[] = {{&S36669[57], 11}, {NULL}};
const Segment SGI385552[] = {{&S36670[57], 11}, {NULL}};
const Segment SGI385553[] = {{&S36671[57], 11}, {NULL}};
const Segment SGI385554[] = {{&S36672[57], 11}, {NULL}};
const Segment SGI385555[] = {{&S36673[57], 11}, {NULL}};
const Segment SGI385556[] = {{&S36675[57], 11}, {NULL}};
const Segment SGI385557[] = {{&S36676[57], 11}, {NULL}};
const Segment SGI385558[] = {{&S36677[57], 12}, {NULL}};
const Segment SGI385559[] = {{&S36679[57], 11}, {NULL}};
const Segment SGI385560[] = {{&S36681[57], 11}, {NULL}};
const Segment SGI385561[] = {{&S36682[57], 11}, {NULL}};
const Segment SGI385562[] = {{&S36683[57], 11}, {NULL}};
const Segment SGI385563[] = {{&S36684[57], 12}, {NULL}};
const Segment SGI385564[] = {{&S36687[57], 12}, {NULL}};
const Segment SGI385565[] = {{&S36690[57], 11}, {NULL}};
const Segment SGI385566[] = {{&S37446[57], 11}, {NULL}};
const Segment SGI385567[] = {{&S37447[57], 12}, {NULL}};
const Segment SGI385568[] = {{&S37618[57], 11}, {NULL}};
const Segment SGI385569[] = {{&S37623[57], 12}, {NULL}};
const Segment SGI385570[] = {{&S37664[57], 11}, {NULL}};
const Segment SGI385571[] = {{&S37665[57], 11}, {NULL}};
const Segment SGI385572[] = {{&S37666[57], 11}, {NULL}};
const Segment SGI385573[] = {{&S37668[57], 11}, {NULL}};
const Segment SGI385574[] = {{&S37670[57], 11}, {NULL}};
const Segment SGI385575[] = {{&S37671[57], 11}, {NULL}};
const Segment SGI385576[] = {{&S61204[0], 15}, {NULL}};
const Segment SGI385577[] = {{&S37673[57], 11}, {NULL}};
const Segment SGI385578[] = {{&S61205[0], 15}, {NULL}};
const Segment SGI385579[] = {{&S61206[0], 15}, {NULL}};
const Segment SGI385580[] = {{&S37675[57], 11}, {NULL}};
const Segment SGI385581[] = {{&S61207[0], 15}, {NULL}};
const Segment SGI385582[] = {{&S37677[57], 11}, {NULL}};
const Segment SGI385583[] = {{&S37680[57], 11}, {NULL}};
const Segment SGI385584[] = {{&S61262[0], 24}, {NULL}};
const Segment SGI385585[] = {{&S37978[57], 11}, {NULL}};
const Segment SGI385586[] = {{&S37979[57], 11}, {NULL}};
const Segment SGI385587[] = {{&S37980[57], 11}, {NULL}};
const Segment SGI385588[] = {{&S37981[57], 11}, {NULL}};
const Segment SGI385589[] = {{&S37982[57], 11}, {NULL}};
const Segment SGI385590[] = {{&S37983[57], 11}, {NULL}};
const Segment SGI385591[] = {{&S37984[57], 11}, {NULL}};
const Segment SGI385592[] = {{&S37987[57], 11}, {NULL}};
const Segment SGI385593[] = {{&S37989[57], 11}, {NULL}};
const Segment SGI385594[] = {{&S37990[57], 11}, {NULL}};
const Segment SGI385595[] = {{&S37993[57], 12}, {NULL}};
const Segment SGI385596[] = {{&S37994[57], 11}, {NULL}};
const Segment SGI385597[] = {{&S38433[57], 11}, {NULL}};
const Segment SGI385598[] = {{&S38436[57], 11}, {NULL}};
const Segment SGI385599[] = {{&S38437[57], 12}, {NULL}};
const Segment SGI385600[] = {{&S62145[0], 23}, {NULL}};
const Segment SGI385601[] = {{&S38610[57], 12}, {NULL}};
const Segment SGI385602[] = {{&S38611[55], 12}, {NULL}};
const Segment SGI385603[] = {{&S38614[57], 11}, {NULL}};
const Segment SGI385604[] = {{&S38615[57], 11}, {NULL}};
const Segment SGI385605[] = {{&S62485[0], 27}, {NULL}};
const Segment SGI385606[] = {{&S62487[0], 27}, {NULL}};
const Segment SGI385607[] = {{&S62488[0], 27}, {NULL}};
const Segment SGI385608[] = {{&S62491[0], 27}, {NULL}};
const Segment SGI385609[] = {{&S62513[0], 18}, {NULL}};
const Segment SGI385610[] = {{&S62514[0], 18}, {NULL}};
const Segment SGI385611[] = {{&S62774[0], 20}, {NULL}};
const Segment SGI385612[] = {{&S62775[0], 20}, {NULL}};
const Segment SGI385613[] = {{&S62776[0], 20}, {NULL}};
const Segment SGI385614[] = {{&S39708[57], 12}, {NULL}};
const Segment SGI385615[] = {{&S40307[57], 11}, {NULL}};
const Segment SGI385616[] = {{&S40395[57], 12}, {NULL}};
const Segment SGI385617[] = {{&S40397[57], 12}, {NULL}};
const Segment SGI385618[] = {{&S40398[57], 12}, {NULL}};
const Segment SGI385619[] = {{&S40399[57], 12}, {NULL}};
const Segment SGI385620[] = {{&S40401[57], 12}, {NULL}};
const Segment SGI385621[] = {{&S40584[63], 10}, {NULL}};
const Segment SGI385622[] = {{&S40734[55], 10}, {NULL}};
const Segment SGI385623[] = {{&S40735[55], 11}, {NULL}};
const Segment SGI385624[] = {{&S40905[57], 11}, {NULL}};
const Segment SGI385625[] = {{&S40906[57], 11}, {NULL}};
const Segment SGI385626[] = {{&S40907[57], 11}, {NULL}};
const Segment SGI385627[] = {{&S40910[57], 11}, {NULL}};
const Segment SGI385628[] = {{&S40911[57], 11}, {NULL}};
const Segment SGI385629[] = {{&S40914[57], 11}, {NULL}};
const Segment SGI385630[] = {{&S40974[57], 11}, {NULL}};
const Segment SGI385631[] = {{&S40975[57], 12}, {NULL}};
const Segment SGI385632[] = {{&S41770[55], 11}, {NULL}};
const Segment SGI385633[] = {{&S41771[55], 12}, {NULL}};
const Segment SGI385634[] = {{&S41777[55], 10}, {NULL}};
const Segment SGI385635[] = {{&S41778[12], 9}, {NULL}};
const Segment SGI385636[] = {{&S42345[57], 11}, {NULL}};
const Segment SGI385637[] = {{&S42347[57], 11}, {NULL}};
const Segment SGI385638[] = {{&S42348[57], 11}, {NULL}};
const Segment SGI385639[] = {{&S42350[57], 11}, {NULL}};
const Segment SGI385640[] = {{&S42353[57], 11}, {NULL}};
const Segment SGI385641[] = {{&S65861[0], 24}, {NULL}};
const Segment SGI385642[] = {{&S65873[0], 24}, {NULL}};
const Segment SGI385643[] = {{&S42553[55], 10}, {NULL}};
const Segment SGI385644[] = {{&S42554[55], 11}, {NULL}};
const Segment SGI385645[] = {{&S42556[55], 12}, {NULL}};
const Segment SGI385646[] = {{&S42557[55], 10}, {NULL}};
const Segment SGI385647[] = {{&S42560[21], 9}, {NULL}};
const Segment SGI385648[] = {{&S42827[55], 11}, {NULL}};
const Segment SGI385649[] = {{&S42830[55], 10}, {NULL}};
const Segment SGI385650[] = {{&S42876[63], 10}, {NULL}};
const Segment SGI385651[] = {{&S43171[12], 3}, {NULL}};
const Segment SGI385652[] = {{&S43171[30], 3}, {NULL}};
const Segment SGI385653[] = {{&S43172[12], 4}, {NULL}};
const Segment SGI385654[] = {{&S43172[31], 3}, {NULL}};
const Segment SGI385655[] = {{&S43173[12], 4}, {NULL}};
const Segment SGI385656[] = {{&S43173[31], 3}, {NULL}};
const Segment SGI385657[] = {{&S43174[12], 3}, {NULL}};
const Segment SGI385658[] = {{&S43174[30], 3}, {NULL}};
const Segment SGI385659[] = {{&S67280[0], 25}, {NULL}};
const Segment SGI385660[] = {{&S67281[0], 25}, {NULL}};
const Segment SGI385661[] = {{&S67282[0], 25}, {NULL}};
const Segment SGI385662[] = {{&S67283[0], 25}, {NULL}};
const Segment SGI385663[] = {{&S67518[0], 31}, {NULL}};
const Segment SGI385664[] = {{&S67523[0], 31}, {NULL}};
const Segment SGI385665[] = {{&S67527[0], 31}, {NULL}};
const Segment SGI385666[] = {{&S43897[57], 11}, {NULL}};
const Segment SGI385667[] = {{&S44213[57], 11}, {NULL}};
const Segment SGI385668[] = {{&S44214[57], 11}, {NULL}};
const Segment SGI385669[] = {{&S44811[57], 10}, {NULL}};
const Segment SGI385670[] = {{&S44813[57], 10}, {NULL}};
const Segment SGI385671[] = {{&S45842[12], 3}, {NULL}};
const Segment SGI385672[] = {{&S45842[30], 3}, {NULL}};
const Segment SGI385673[] = {{&S45843[12], 4}, {NULL}};
const Segment SGI385674[] = {{&S45843[31], 3}, {NULL}};
const Segment SGI385675[] = {{&S45844[12], 3}, {NULL}};
const Segment SGI385676[] = {{&S45844[30], 3}, {NULL}};
const Segment SGI385677[] = {{&S46217[57], 12}, {NULL}};
const Segment SGI385678[] = {{&S46219[57], 12}, {NULL}};
const Segment SGI385679[] = {{&S46221[57], 12}, {NULL}};
const Segment SGI385680[] = {{&S46223[57], 12}, {NULL}};
const Segment SGI385681[] = {{&S46226[57], 12}, {NULL}};
const Segment SGI385682[] = {{&S46227[56], 12}, {NULL}};
const Segment SGI385683[] = {{&S46228[57], 12}, {NULL}};
const Segment SGI385684[] = {{&S46230[57], 12}, {NULL}};
const Segment SGI385685[] = {{&S46233[57], 12}, {NULL}};
const Segment SGI385686[] = {{&S46236[57], 12}, {NULL}};
const Segment SGI385687[] = {{&S46578[57], 12}, {NULL}};
const Segment SGI385688[] = {{&S46579[57], 12}, {NULL}};
const Segment SGI385689[] = {{&S46581[57], 12}, {NULL}};
const Segment SGI385690[] = {{&S46582[57], 12}, {NULL}};
const Segment SGI385691[] = {{&S46583[57], 12}, {NULL}};
const Segment SGI385692[] = {{&S46588[57], 12}, {NULL}};
const Segment SGI385693[] = {{&S46589[57], 12}, {NULL}};
const Segment SGI385694[] = {{&S46647[55], 10}, {NULL}};
const Segment SGI385695[] = {{&S46777[12], 6}, {NULL}};
const Segment SGI385696[] = {{&S46777[41], 3}, {NULL}};
const Segment SGI385697[] = {{&S46778[12], 5}, {NULL}};
const Segment SGI385698[] = {{&S46778[40], 3}, {NULL}};
const Segment SGI385699[] = {{&S70258[0], 25}, {NULL}};
const Segment SGI385700[] = {{&S47855[57], 12}, {NULL}};
const Segment SGI385701[] = {{&S47856[57], 12}, {NULL}};
const Segment SGI385702[] = {{&S47857[57], 12}, {NULL}};
const Segment SGI385703[] = {{&S47858[57], 12}, {NULL}};
const Segment SGI385704[] = {{&S47859[57], 12}, {NULL}};
const Segment SGI385705[] = {{&S47860[57], 12}, {NULL}};
const Segment SGI385706[] = {{&S47861[57], 12}, {NULL}};
const Segment SGI385707[] = {{&S47862[57], 12}, {NULL}};
const Segment SGI385708[] = {{&S47863[57], 12}, {NULL}};
const Segment SGI385709[] = {{&S71210[0], 20}, {NULL}};
const Segment SGI385710[] = {{&S48198[57], 11}, {NULL}};
const Segment SGI385711[] = {{&S48201[57], 11}, {NULL}};
const Segment SGI385712[] = {{&S48203[57], 11}, {NULL}};
const Segment SGI385713[] = {{&S48205[57], 11}, {NULL}};
const Segment SGI385714[] = {{&S48206[57], 11}, {NULL}};
const Segment SGI385715[] = {{&S48208[57], 11}, {NULL}};
const Segment SGI385716[] = {{&S48211[57], 11}, {NULL}};
const Segment SGI385717[] = {{&S48212[57], 11}, {NULL}};
const Segment SGI385718[] = {{&S48310[63], 10}, {NULL}};
const Segment SGI385719[] = {{&S48392[57], 11}, {NULL}};
const Segment SGI385720[] = {{&S48393[57], 11}, {NULL}};
const Segment SGI385721[] = {{&S48394[57], 11}, {NULL}};
const Segment SGI385722[] = {{&S48395[57], 11}, {NULL}};
const Segment SGI385723[] = {{&S48397[57], 11}, {NULL}};
const Segment SGI385724[] = {{&S48398[57], 11}, {NULL}};
const Segment SGI385725[] = {{&S48400[57], 11}, {NULL}};
const Segment SGI385726[] = {{&S48404[57], 11}, {NULL}};
const Segment SGI385727[] = {{&S71638[0], 24}, {NULL}};
const Segment SGI385728[] = {{&S71639[0], 24}, {NULL}};
const Segment SGI385729[] = {{&S71640[0], 24}, {NULL}};
const Segment SGI385730[] = {{&S71641[0], 24}, {NULL}};
const Segment SGI385731[] = {{&S71642[0], 24}, {NULL}};
const Segment SGI385732[] = {{&S71643[0], 24}, {NULL}};
const Segment SGI385733[] = {{&S48530[12], 4}, {NULL}};
const Segment SGI385734[] = {{&S48530[39], 3}, {NULL}};
const Segment SGI385735[] = {{&S48815[57], 12}, {NULL}};
const Segment SGI385736[] = {{&S48816[57], 12}, {NULL}};
const Segment SGI385737[] = {{&S48818[57], 12}, {NULL}};
const Segment SGI385738[] = {{&S48820[57], 12}, {NULL}};
const Segment SGI385739[] = {{&S72683[0], 20}, {NULL}};
const Segment SGI385740[] = {{&S49586[12], 3}, {NULL}};
const Segment SGI385741[] = {{&S49586[38], 3}, {NULL}};
const Segment SGI385742[] = {{&S49587[12], 5}, {NULL}};
const Segment SGI385743[] = {{&S49587[40], 3}, {NULL}};
const Segment SGI385744[] = {{&S49588[12], 4}, {NULL}};
const Segment SGI385745[] = {{&S49588[39], 3}, {NULL}};
const Segment SGI385746[] = {{&S73395[0], 20}, {NULL}};
const Segment SGI385747[] = {{&S50202[63], 10}, {NULL}};
const Segment SGI385748[] = {{&S74376[0], 20}, {NULL}};
const Segment SGI385749[] = {{&S50960[57], 11}, {NULL}};
const Segment SGI385750[] = {{&S51353[12], 4}, {NULL}};
const Segment SGI385751[] = {{&S51353[39], 3}, {NULL}};
const Segment SGI385752[] = {{&S51354[12], 5}, {NULL}};
const Segment SGI385753[] = {{&S51354[40], 3}, {NULL}};
const Segment SGI385754[] = {{&S51355[24], 5}, {NULL}};
const Segment SGI385755[] = {{&S51355[52], 3}, {NULL}};
const Segment SGI385756[] = {{&S51356[24], 6}, {NULL}};
const Segment SGI385757[] = {{&S51356[53], 3}, {NULL}};
const Segment SGI385758[] = {{&S51679[57], 12}, {NULL}};
const Segment SGI385759[] = {{&S51785[12], 5}, {NULL}};
const Segment SGI385760[] = {{&S51785[40], 3}, {NULL}};
const Segment SGI385761[] = {{&S76316[0], 20}, {NULL}};
const Segment SGI385762[] = {{&S76317[0], 20}, {NULL}};
const Segment SGI385763[] = {{&S76318[0], 20}, {NULL}};
const Segment SGI385764[] = {{&S76319[0], 20}, {NULL}};
const Segment SGI385765[] = {{&S76320[0], 20}, {NULL}};
const Segment SGI385766[] = {{&S76321[0], 20}, {NULL}};
const Segment SGI385767[] = {{&S76322[0], 20}, {NULL}};
const Segment SGI385768[] = {{&S53312[12], 5}, {NULL}};
const Segment SGI385769[] = {{&S53312[40], 3}, {NULL}};
const Segment SGI385770[] = {{&S53313[12], 4}, {NULL}};
const Segment SGI385771[] = {{&S53313[39], 3}, {NULL}};
const Segment SGI385772[] = {{&S76682[0], 26}, {NULL}};
const Segment SGI385773[] = {{&S53460[12], 4}, {NULL}};
const Segment SGI385774[] = {{&S53460[39], 3}, {NULL}};
const Segment SGI385775[] = {{&S53461[12], 4}, {NULL}};
const Segment SGI385776[] = {{&S53461[39], 3}, {NULL}};
const Segment SGI385777[] = {{&S53526[12], 5}, {NULL}};
const Segment SGI385778[] = {{&S53526[40], 3}, {NULL}};
const Segment SGI385779[] = {{&S53527[12], 5}, {NULL}};
const Segment SGI385780[] = {{&S53527[40], 3}, {NULL}};
const Segment SGI385781[] = {{&S53528[12], 4}, {NULL}};
const Segment SGI385782[] = {{&S53528[39], 3}, {NULL}};
const Segment SGI385783[] = {{&S76902[0], 20}, {NULL}};
const Segment SGI385784[] = {{&S76903[0], 20}, {NULL}};
const Segment SGI385785[] = {{&S76904[0], 20}, {NULL}};
const Segment SGI385786[] = {{&S53596[12], 5}, {NULL}};
const Segment SGI385787[] = {{&S53596[40], 7}, {NULL}};
const Segment SGI385788[] = {{&S53597[12], 5}, {NULL}};
const Segment SGI385789[] = {{&S53597[40], 7}, {NULL}};
const Segment SGI385790[] = {{&S54112[12], 5}, {NULL}};
const Segment SGI385791[] = {{&S54112[40], 7}, {NULL}};
const Segment SGI385792[] = {{&S54254[57], 11}, {NULL}};
const Segment SGI385793[] = {{&S54529[12], 4}, {NULL}};
const Segment SGI385794[] = {{&S54529[39], 3}, {NULL}};
const Segment SGI385795[] = {{&S55123[12], 3}, {NULL}};
const Segment SGI385796[] = {{&S55123[38], 3}, {NULL}};
const Segment SGI385797[] = {{&S55124[12], 4}, {NULL}};
const Segment SGI385798[] = {{&S55124[39], 3}, {NULL}};
const Segment SGI385799[] = {{&S55216[12], 5}, {NULL}};
const Segment SGI385800[] = {{&S55216[40], 3}, {NULL}};
const Segment SGI385801[] = {{&S55217[12], 6}, {NULL}};
const Segment SGI385802[] = {{&S55217[41], 3}, {NULL}};
const Segment SGI385803[] = {{&S55218[12], 5}, {NULL}};
const Segment SGI385804[] = {{&S55218[40], 3}, {NULL}};
const Segment SGI385805[] = {{&S55584[12], 5}, {NULL}};
const Segment SGI385806[] = {{&S55584[40], 3}, {NULL}};
const Segment SGI385807[] = {{&S55585[12], 5}, {NULL}};
const Segment SGI385808[] = {{&S55585[40], 3}, {NULL}};
const Segment SGI385809[] = {{&S55615[63], 12}, {NULL}};
const Segment SGI385810[] = {{&S55709[57], 11}, {NULL}};
const Segment SGI385811[] = {{&S55952[12], 5}, {NULL}};
const Segment SGI385812[] = {{&S55952[40], 7}, {NULL}};
const Segment SGI385813[] = {{&S56296[24], 6}, {NULL}};
const Segment SGI385814[] = {{&S56296[53], 3}, {NULL}};
const Segment SGI385815[] = {{&S56297[12], 5}, {NULL}};
const Segment SGI385816[] = {{&S56297[40], 3}, {NULL}};
const Segment SGI385817[] = {{&S56800[12], 3}, {NULL}};
const Segment SGI385818[] = {{&S56800[30], 3}, {NULL}};
const Segment SGI385819[] = {{&S57406[12], 5}, {NULL}};
const Segment SGI385820[] = {{&S57406[40], 3}, {NULL}};
const Segment SGI385821[] = {{&S57407[12], 6}, {NULL}};
const Segment SGI385822[] = {{&S57407[41], 3}, {NULL}};
const Segment SGI385823[] = {{&S57706[12], 4}, {NULL}};
const Segment SGI385824[] = {{&S57706[39], 3}, {NULL}};
const Segment SGI385825[] = {{&S57917[57], 11}, {NULL}};
const Segment SGI385826[] = {{&S57918[57], 11}, {NULL}};
const Segment SGI385827[] = {{&S57919[57], 11}, {NULL}};
const Segment SGI385828[] = {{&S57922[57], 11}, {NULL}};
const Segment SGI385829[] = {{&S57924[57], 11}, {NULL}};
const Segment SGI385830[] = {{&S58501[55], 10}, {NULL}};
const Segment SGI385831[] = {{&S58615[12], 3}, {NULL}};
const Segment SGI385832[] = {{&S58615[79], 4}, {NULL}};
const Segment SGI385833[] = {{&S58884[12], 5}, {NULL}};
const Segment SGI385834[] = {{&S58884[40], 3}, {NULL}};
const Segment SGI385835[] = {{&S59007[12], 4}, {NULL}};
const Segment SGI385836[] = {{&S59007[39], 3}, {NULL}};
const Segment SGI385837[] = {{&S59008[12], 5}, {NULL}};
const Segment SGI385838[] = {{&S59008[40], 3}, {NULL}};
const Segment SGI385839[] = {{&S59014[12], 10}, {NULL}};
const Segment SGI385840[] = {{&S59014[77], 3}, {NULL}};
const Segment SGI385841[] = {{&S59015[12], 5}, {NULL}};
const Segment SGI385842[] = {{&S59015[40], 3}, {NULL}};
const Segment SGI385843[] = {{&S59016[12], 5}, {NULL}};
const Segment SGI385844[] = {{&S59016[40], 3}, {NULL}};
const Segment SGI385845[] = {{&S59192[12], 5}, {NULL}};
const Segment SGI385846[] = {{&S59192[36], 21}, {NULL}};
const Segment SGI385847[] = {{&S59193[12], 5}, {NULL}};
const Segment SGI385848[] = {{&S59193[35], 3}, {NULL}};
const Segment SGI385849[] = {{&S59194[12], 5}, {NULL}};
const Segment SGI385850[] = {{&S59194[35], 5}, {NULL}};
const Segment SGI385851[] = {{&S59195[12], 5}, {NULL}};
const Segment SGI385852[] = {{&S59195[36], 5}, {NULL}};
const Segment SGI385853[] = {{&S59196[12], 5}, {NULL}};
const Segment SGI385854[] = {{&S59196[36], 5}, {NULL}};
const Segment SGI385855[] = {{&S59321[57], 11}, {NULL}};
const Segment SGI385856[] = {{&S59541[12], 5}, {NULL}};
const Segment SGI385857[] = {{&S59541[40], 7}, {NULL}};
const Segment SGI385858[] = {{&S82818[0], 20}, {NULL}};
const Segment SGI385859[] = {{&S82819[0], 20}, {NULL}};
const Segment SGI385860[] = {{&S59918[55], 10}, {NULL}};
const Segment SGI385861[] = {{&S60215[12], 5}, {NULL}};
const Segment SGI385862[] = {{&S60215[40], 3}, {NULL}};
const Segment SGI385863[] = {{&S60216[12], 5}, {NULL}};
const Segment SGI385864[] = {{&S60216[40], 3}, {NULL}};
const Segment SGI385865[] = {{&S60223[12], 5}, {NULL}};
const Segment SGI385866[] = {{&S60223[40], 7}, {NULL}};
const Segment SGI385867[] = {{&S60234[23], 5}, {NULL}};
const Segment SGI385868[] = {{&S60234[6], 4}, {&S60234[92], 4}, {NULL}};
const Segment SGI385869[] = {{&S60235[22], 5}, {NULL}};
const Segment SGI385870[] = {{&S60235[6], 3}, {&S60235[91], 4}, {NULL}};
const Segment SGI385871[] = {{&S60236[12], 5}, {NULL}};
const Segment SGI385872[] = {{&S60236[81], 4}, {NULL}};
const Segment SGI385873[] = {{&S60237[12], 5}, {NULL}};
const Segment SGI385874[] = {{&S60237[81], 4}, {NULL}};
const Segment SGI385875[] = {{&S60309[12], 4}, {&S60309[12], 4}, {NULL}};
const Segment SGI385876[] = {{&S60310[12], 4}, {&S60310[12], 4}, {&S60310[12], 4}, {NULL}};
const Segment SGI385877[] = {{&S60311[12], 4}, {NULL}};
const Segment SGI385878[] = {{&S60578[55], 12}, {NULL}};
const Segment SGI385879[] = {{&S60736[12], 5}, {NULL}};
const Segment SGI385880[] = {{&S60736[40], 3}, {NULL}};
const Segment SGI385881[] = {{&S60737[12], 5}, {NULL}};
const Segment SGI385882[] = {{&S60737[40], 3}, {NULL}};
const Segment SGI385883[] = {{&S60738[12], 5}, {NULL}};
const Segment SGI385884[] = {{&S60738[40], 3}, {NULL}};
const Segment SGI385885[] = {{&S61062[12], 5}, {NULL}};
const Segment SGI385886[] = {{&S61062[43], 3}, {NULL}};
const Segment SGI385887[] = {{&S61122[12], 5}, {NULL}};
const Segment SGI385888[] = {{&S61122[40], 3}, {NULL}};
const Segment SGI385889[] = {{&S61123[12], 4}, {NULL}};
const Segment SGI385890[] = {{&S61123[39], 3}, {NULL}};
const Segment SGI385891[] = {{&S61525[12], 5}, {NULL}};
const Segment SGI385892[] = {{&S61525[41], 3}, {NULL}};
const Segment SGI385893[] = {{&S61526[12], 5}, {NULL}};
const Segment SGI385894[] = {{&S61526[60], 3}, {NULL}};
const Segment SGI385895[] = {{&S62075[12], 4}, {NULL}};
const Segment SGI385896[] = {{&S62075[39], 3}, {NULL}};
const Segment SGI385897[] = {{&S62129[12], 5}, {NULL}};
const Segment SGI385898[] = {{&S62129[40], 3}, {NULL}};
const Segment SGI385899[] = {{&S62130[12], 4}, {NULL}};
const Segment SGI385900[] = {{&S62130[39], 3}, {NULL}};
const Segment SGI385901[] = {{&S62156[12], 6}, {NULL}};
const Segment SGI385902[] = {{&S62156[42], 3}, {NULL}};
const Segment SGI385903[] = {{&S62157[12], 5}, {NULL}};
const Segment SGI385904[] = {{&S62157[41], 3}, {NULL}};
const Segment SGI385905[] = {{&S62507[12], 3}, {&S62507[12], 3}, {NULL}};
const Segment SGI385906[] = {{&S62512[74], 3}, {NULL}};
const Segment SGI385907[] = {{&S63246[12], 5}, {NULL}};
const Segment SGI385908[] = {{&S63246[40], 3}, {NULL}};
const Segment SGI385909[] = {{&S64218[12], 5}, {NULL}};
const Segment SGI385910[] = {{&S64218[40], 3}, {NULL}};
const Segment SGI385911[] = {{&S64219[12], 6}, {NULL}};
const Segment SGI385912[] = {{&S64219[41], 3}, {NULL}};
const Segment SGI385913[] = {{&S64228[12], 5}, {NULL}};
const Segment SGI385914[] = {{&S64228[40], 3}, {NULL}};
const Segment SGI385915[] = {{&S64253[12], 5}, {NULL}};
const Segment SGI385916[] = {{&S64253[40], 3}, {NULL}};
const Segment SGI385917[] = {{&S64817[12], 3}, {NULL}};
const Segment SGI385918[] = {{&S64817[38], 3}, {NULL}};
const Segment SGI385919[] = {{&S88573[0], 15}, {NULL}};
const Segment SGI385920[] = {{&S88574[0], 15}, {NULL}};
const Segment SGI385921[] = {{&S65665[12], 4}, {&S65665[12], 4}, {&S65665[12], 4}, {NULL}};
const Segment SGI385922[] = {{&S65666[12], 4}, {&S65666[12], 4}, {NULL}};
const Segment SGI385923[] = {{&S65667[12], 4}, {NULL}};
const Segment SGI385924[] = {{&S66026[12], 5}, {NULL}};
const Segment SGI385925[] = {{&S66026[40], 3}, {NULL}};
const Segment SGI385926[] = {{&S66201[12], 5}, {NULL}};
const Segment SGI385927[] = {{&S89399[0], 18}, {NULL}};
const Segment SGI385928[] = {{&S66201[43], 3}, {NULL}};
const Segment SGI385929[] = {{&S89400[0], 18}, {NULL}};
const Segment SGI385930[] = {{&S89401[0], 18}, {NULL}};
const Segment SGI385931[] = {{&S66265[12], 5}, {NULL}};
const Segment SGI385932[] = {{&S66265[40], 3}, {NULL}};
const Segment SGI385933[] = {{&S67050[12], 3}, {NULL}};
const Segment SGI385934[] = {{&S67219[12], 6}, {NULL}};
const Segment SGI385935[] = {{&S67219[41], 3}, {NULL}};
const Segment SGI385936[] = {{&S67220[12], 5}, {NULL}};
const Segment SGI385937[] = {{&S67220[40], 3}, {NULL}};
const Segment SGI385938[] = {{&S67830[12], 5}, {NULL}};
const Segment SGI385939[] = {{&S67830[40], 3}, {NULL}};
const Segment SGI385940[] = {{&S67894[12], 5}, {NULL}};
const Segment SGI385941[] = {{&S67894[137], 4}, {NULL}};
const Segment SGI385942[] = {{&S67894[64], 9}, {NULL}};
const Segment SGI385943[] = {{&S68101[12], 5}, {NULL}};
const Segment SGI385944[] = {{&S68101[138], 4}, {NULL}};
const Segment SGI385945[] = {{&S68101[64], 9}, {NULL}};
const Segment SGI385946[] = {{&S68109[12], 5}, {NULL}};
const Segment SGI385947[] = {{&S68109[40], 3}, {NULL}};
const Segment SGI385948[] = {{&S68164[12], 4}, {NULL}};
const Segment SGI385949[] = {{&S68164[39], 3}, {NULL}};
const Segment SGI385950[] = {{&S68305[62], 3}, {NULL}};
const Segment SGI385951[] = {{&S68305[41], 5}, {NULL}};
const Segment SGI385952[] = {{&S68349[12], 5}, {NULL}};
const Segment SGI385953[] = {{&S68349[40], 3}, {NULL}};
const Segment SGI385954[] = {{&S68674[12], 5}, {NULL}};
const Segment SGI385955[] = {{&S68674[40], 3}, {NULL}};
const Segment SGI385956[] = {{&S68802[57], 12}, {NULL}};
const Segment SGI385957[] = {{&S69064[12], 5}, {NULL}};
const Segment SGI385958[] = {{&S69064[40], 3}, {NULL}};
const Segment SGI385959[] = {{&S69080[12], 3}, {NULL}};
const Segment SGI385960[] = {{&S69080[30], 3}, {NULL}};
const Segment SGI385961[] = {{&S69123[12], 5}, {NULL}};
const Segment SGI385962[] = {{&S69123[39], 3}, {NULL}};
const Segment SGI385963[] = {{&S69124[12], 5}, {NULL}};
const Segment SGI385964[] = {{&S69124[40], 3}, {NULL}};
const Segment SGI385965[] = {{&S69324[12], 5}, {NULL}};
const Segment SGI385966[] = {{&S69324[40], 3}, {NULL}};
const Segment SGI385967[] = {{&S69433[12], 5}, {NULL}};
const Segment SGI385968[] = {{&S69433[81], 4}, {NULL}};
const Segment SGI385969[] = {{&S69537[24], 5}, {NULL}};
const Segment SGI385970[] = {{&S69537[52], 3}, {NULL}};
const Segment SGI385971[] = {{&S92476[0], 18}, {NULL}};
const Segment SGI385972[] = {{&S69848[12], 5}, {NULL}};
const Segment SGI385973[] = {{&S69848[35], 5}, {NULL}};
const Segment SGI385974[] = {{&S69849[12], 5}, {&S69849[12], 5}, {NULL}};
const Segment SGI385975[] = {{&S69849[36], 5}, {&S69849[36], 5}, {NULL}};
const Segment SGI385976[] = {{&S69850[12], 5}, {NULL}};
const Segment SGI385977[] = {{&S69850[36], 5}, {NULL}};
const Segment SGI385978[] = {{&S69851[12], 5}, {NULL}};
const Segment SGI385979[] = {{&S69851[36], 5}, {NULL}};
const Segment SGI385980[] = {{&S69852[12], 5}, {NULL}};
const Segment SGI385981[] = {{&S69852[35], 3}, {NULL}};
const Segment SGI385982[] = {{&S69876[12], 5}, {NULL}};
const Segment SGI385983[] = {{&S69876[40], 3}, {NULL}};
const Segment SGI385984[] = {{&S69877[12], 5}, {NULL}};
const Segment SGI385985[] = {{&S69877[40], 3}, {NULL}};
const Segment SGI385986[] = {{&S92839[0], 18}, {NULL}};
const Segment SGI385987[] = {{&S70329[24], 5}, {NULL}};
const Segment SGI385988[] = {{&S70329[52], 3}, {NULL}};
const Segment SGI385989[] = {{&S70424[57], 12}, {NULL}};
const Segment SGI385990[] = {{&S70425[57], 12}, {NULL}};
const Segment SGI385991[] = {{&S70428[57], 12}, {NULL}};
const Segment SGI385992[] = {{&S70431[57], 12}, {NULL}};
const Segment SGI385993[] = {{&S70607[12], 5}, {NULL}};
const Segment SGI385994[] = {{&S70607[40], 3}, {NULL}};
const Segment SGI385995[] = {{&S70692[12], 5}, {NULL}};
const Segment SGI385996[] = {{&S70692[40], 3}, {NULL}};
const Segment SGI385997[] = {{&S70710[12], 5}, {NULL}};
const Segment SGI385998[] = {{&S70710[40], 3}, {NULL}};
const Segment SGI385999[] = {{&S70711[12], 4}, {NULL}};
const Segment SGI386000[] = {{&S70711[39], 3}, {NULL}};
const Segment SGI386001[] = {{&S71192[12], 5}, {NULL}};
const Segment SGI386002[] = {{&S71192[43], 3}, {NULL}};
const Segment SGI386003[] = {{&S71656[42], 15}, {NULL}};
const Segment SGI386004[] = {{&S71656[94], 4}, {NULL}};
const Segment SGI386005[] = {{&S71760[12], 5}, {NULL}};
const Segment SGI386006[] = {{&S71760[134], 4}, {NULL}};
const Segment SGI386007[] = {{&S71760[64], 9}, {NULL}};
const Segment SGI386008[] = {{&S71761[12], 4}, {NULL}};
const Segment SGI386009[] = {{&S71761[132], 4}, {NULL}};
const Segment SGI386010[] = {{&S71761[63], 9}, {NULL}};
const Segment SGI386011[] = {{&S71909[12], 4}, {NULL}};
const Segment SGI386012[] = {{&S71909[80], 4}, {NULL}};
const Segment SGI386013[] = {{&S72045[12], 3}, {NULL}};
const Segment SGI386014[] = {{&S72046[12], 2}, {NULL}};
const Segment SGI386015[] = {{&S72175[12], 5}, {NULL}};
const Segment SGI386016[] = {{&S72175[40], 3}, {NULL}};
const Segment SGI386017[] = {{&S72206[82], 3}, {NULL}};
const Segment SGI386018[] = {{&S72732[12], 5}, {NULL}};
const Segment SGI386019[] = {{&S72732[41], 3}, {NULL}};
const Segment SGI386020[] = {{&S72811[12], 4}, {&S72811[12], 4}, {&S72811[12], 4}, {NULL}};
const Segment SGI386021[] = {{&S72812[12], 4}, {&S72812[12], 4}, {NULL}};
const Segment SGI386022[] = {{&S72813[12], 4}, {NULL}};
const Segment SGI386023[] = {{&S72813[102], 3}, {NULL}};
const Segment SGI386024[] = {{&S72814[12], 4}, {NULL}};
const Segment SGI386025[] = {{&S72944[12], 3}, {NULL}};
const Segment SGI386026[] = {{&S72985[12], 5}, {NULL}};
const Segment SGI386027[] = {{&S72985[40], 3}, {NULL}};
const Segment SGI386028[] = {{&S72986[12], 5}, {NULL}};
const Segment SGI386029[] = {{&S72986[40], 3}, {NULL}};
const Segment SGI386030[] = {{&S95987[0], 15}, {NULL}};
const Segment SGI386031[] = {{&S73136[23], 5}, {NULL}};
const Segment SGI386032[] = {{&S73136[6], 4}, {&S73136[92], 4}, {NULL}};
const Segment SGI386033[] = {{&S73137[22], 5}, {NULL}};
const Segment SGI386034[] = {{&S73137[6], 3}, {&S73137[91], 4}, {NULL}};
const Segment SGI386035[] = {{&S73138[12], 5}, {NULL}};
const Segment SGI386036[] = {{&S73138[81], 4}, {NULL}};
const Segment SGI386037[] = {{&S73139[12], 5}, {NULL}};
const Segment SGI386038[] = {{&S73139[81], 4}, {NULL}};
const Segment SGI386039[] = {{&S96657[0], 18}, {NULL}};
const Segment SGI386040[] = {{&S73634[12], 5}, {NULL}};
const Segment SGI386041[] = {{&S73634[40], 3}, {NULL}};
const Segment SGI386042[] = {{&S73635[12], 5}, {NULL}};
const Segment SGI386043[] = {{&S73635[40], 3}, {NULL}};
const Segment SGI386044[] = {{&S73706[24], 5}, {NULL}};
const Segment SGI386045[] = {{&S73706[52], 3}, {NULL}};
const Segment SGI386046[] = {{&S73720[49], 6}, {NULL}};
const Segment SGI386047[] = {{&S73720[71], 4}, {NULL}};
const Segment SGI386048[] = {{&S73720[0], 9}, {NULL}};
const Segment SGI386049[] = {{&S73811[12], 5}, {NULL}};
const Segment SGI386050[] = {{&S73811[40], 3}, {NULL}};
const Segment SGI386051[] = {{&S73908[12], 5}, {NULL}};
const Segment SGI386052[] = {{&S73908[40], 7}, {NULL}};
const Segment SGI386053[] = {{&S73971[24], 5}, {NULL}};
const Segment SGI386054[] = {{&S73971[52], 3}, {NULL}};
const Segment SGI386055[] = {{&S74342[55], 12}, {NULL}};
const Segment SGI386056[] = {{&S74403[12], 5}, {NULL}};
const Segment SGI386057[] = {{&S74403[40], 7}, {NULL}};
const Segment SGI386058[] = {{&S74404[12], 4}, {NULL}};
const Segment SGI386059[] = {{&S74404[39], 7}, {NULL}};
const Segment SGI386060[] = {{&S74458[12], 4}, {NULL}};
const Segment SGI386061[] = {{&S74458[39], 3}, {NULL}};
const Segment SGI386062[] = {{&S74459[24], 5}, {NULL}};
const Segment SGI386063[] = {{&S74459[52], 3}, {NULL}};
const Segment SGI386064[] = {{&S74641[12], 5}, {NULL}};
const Segment SGI386065[] = {{&S74641[138], 4}, {NULL}};
const Segment SGI386066[] = {{&S74641[64], 9}, {NULL}};
const Segment SGI386067[] = {{&S74642[12], 4}, {NULL}};
const Segment SGI386068[] = {{&S74642[133], 4}, {NULL}};
const Segment SGI386069[] = {{&S74642[63], 9}, {NULL}};
const Segment SGI386070[] = {{&S98061[0], 15}, {NULL}};
const Segment SGI386071[] = {{&S74916[12], 6}, {NULL}};
const Segment SGI386072[] = {{&S74916[41], 3}, {NULL}};
const Segment SGI386073[] = {{&S74966[12], 5}, {NULL}};
const Segment SGI386074[] = {{&S74966[138], 4}, {NULL}};
const Segment SGI386075[] = {{&S74966[64], 9}, {NULL}};
const Segment SGI386076[] = {{&S75008[12], 5}, {NULL}};
const Segment SGI386077[] = {{&S75008[40], 3}, {NULL}};
const Segment SGI386078[] = {{&S75146[12], 5}, {NULL}};
const Segment SGI386079[] = {{&S75146[137], 4}, {NULL}};
const Segment SGI386080[] = {{&S75146[64], 9}, {NULL}};
const Segment SGI386081[] = {{&S75227[12], 5}, {NULL}};
const Segment SGI386082[] = {{&S75227[138], 4}, {NULL}};
const Segment SGI386083[] = {{&S75227[64], 9}, {NULL}};
const Segment SGI386084[] = {{&S98548[0], 28}, {NULL}};
const Segment SGI386085[] = {{&S75390[22], 5}, {NULL}};
const Segment SGI386086[] = {{&S75390[6], 3}, {&S75390[91], 4}, {NULL}};
const Segment SGI386087[] = {{&S75391[12], 5}, {NULL}};
const Segment SGI386088[] = {{&S75391[81], 4}, {NULL}};
const Segment SGI386089[] = {{&S75392[12], 5}, {NULL}};
const Segment SGI386090[] = {{&S75392[81], 4}, {NULL}};
const Segment SGI386091[] = {{&S75393[23], 5}, {NULL}};
const Segment SGI386092[] = {{&S75393[6], 4}, {&S75393[92], 4}, {NULL}};
const Segment SGI386093[] = {{&S75452[12], 4}, {NULL}};
const Segment SGI386094[] = {{&S75452[135], 4}, {NULL}};
const Segment SGI386095[] = {{&S75452[63], 9}, {NULL}};
const Segment SGI386096[] = {{&S75818[12], 5}, {NULL}};
const Segment SGI386097[] = {{&S75818[138], 4}, {NULL}};
const Segment SGI386098[] = {{&S75818[64], 9}, {NULL}};
const Segment SGI386099[] = {{&S75899[12], 5}, {NULL}};
const Segment SGI386100[] = {{&S75899[40], 7}, {NULL}};
const Segment SGI386101[] = {{&S75900[12], 5}, {NULL}};
const Segment SGI386102[] = {{&S75900[40], 7}, {NULL}};
const Segment SGI386103[] = {{&S76429[12], 5}, {NULL}};
const Segment SGI386104[] = {{&S76429[60], 3}, {NULL}};
const Segment SGI386105[] = {{&S76636[12], 6}, {NULL}};
const Segment SGI386106[] = {{&S76636[41], 3}, {NULL}};
const Segment SGI386107[] = {{&S76637[12], 5}, {NULL}};
const Segment SGI386108[] = {{&S76637[40], 3}, {NULL}};
const Segment SGI386109[] = {{&S76638[12], 5}, {NULL}};
const Segment SGI386110[] = {{&S76638[40], 3}, {NULL}};
const Segment SGI386111[] = {{&S100132[0], 18}, {NULL}};
const Segment SGI386112[] = {{&S77322[12], 5}, {NULL}};
const Segment SGI386113[] = {{&S77322[40], 7}, {NULL}};
const Segment SGI386114[] = {{&S77492[49], 6}, {NULL}};
const Segment SGI386115[] = {{&S77492[71], 4}, {NULL}};
const Segment SGI386116[] = {{&S77492[0], 9}, {NULL}};
const Segment SGI386117[] = {{&S77501[12], 4}, {NULL}};
const Segment SGI386118[] = {{&S77501[136], 4}, {NULL}};
const Segment SGI386119[] = {{&S77501[63], 9}, {NULL}};
const Segment SGI386120[] = {{&S77646[12], 4}, {NULL}};
const Segment SGI386121[] = {{&S77646[132], 4}, {NULL}};
const Segment SGI386122[] = {{&S77646[63], 9}, {NULL}};
const Segment SGI386123[] = {{&S78007[24], 6}, {NULL}};
const Segment SGI386124[] = {{&S78007[53], 3}, {NULL}};
const Segment SGI386125[] = {{&S101330[0], 24}, {NULL}};
const Segment SGI386126[] = {{&S101333[0], 19}, {NULL}};
const Segment SGI386127[] = {{&S101334[0], 18}, {NULL}};
const Segment SGI386128[] = {{&S101335[0], 18}, {NULL}};
const Segment SGI386129[] = {{&S78081[42], 15}, {NULL}};
const Segment SGI386130[] = {{&S78081[94], 4}, {NULL}};
const Segment SGI386131[] = {{&S78094[50], 6}, {NULL}};
const Segment SGI386132[] = {{&S78094[72], 4}, {NULL}};
const Segment SGI386133[] = {{&S78094[0], 9}, {NULL}};
const Segment SGI386134[] = {{&S101465[0], 18}, {NULL}};
const Segment SGI386135[] = {{&S78270[12], 5}, {NULL}};
const Segment SGI386136[] = {{&S78270[41], 3}, {NULL}};
const Segment SGI386137[] = {{&S78271[12], 10}, {NULL}};
const Segment SGI386138[] = {{&S78271[79], 3}, {NULL}};
const Segment SGI386139[] = {{&S78272[12], 5}, {NULL}};
const Segment SGI386140[] = {{&S78272[41], 3}, {NULL}};
const Segment SGI386141[] = {{&S78426[57], 11}, {NULL}};
const Segment SGI386142[] = {{&S78427[57], 11}, {NULL}};
const Segment SGI386143[] = {{&S78428[57], 11}, {NULL}};
const Segment SGI386144[] = {{&S101826[0], 28}, {NULL}};
const Segment SGI386145[] = {{&S101827[0], 28}, {NULL}};
const Segment SGI386146[] = {{&S101828[0], 18}, {NULL}};
const Segment SGI386147[] = {{&S102211[0], 28}, {NULL}};
const Segment SGI386148[] = {{&S102212[0], 28}, {NULL}};
const Segment SGI386149[] = {{&S78821[42], 15}, {NULL}};
const Segment SGI386150[] = {{&S78821[94], 4}, {NULL}};
const Segment SGI386151[] = {{&S78862[12], 6}, {NULL}};
const Segment SGI386152[] = {{&S78863[12], 5}, {NULL}};
const Segment SGI386153[] = {{&S78870[57], 12}, {NULL}};
const Segment SGI386154[] = {{&S78871[57], 12}, {NULL}};
const Segment SGI386155[] = {{&S78873[57], 12}, {NULL}};
const Segment SGI386156[] = {{&S78929[12], 5}, {NULL}};
const Segment SGI386157[] = {{&S78929[40], 3}, {NULL}};
const Segment SGI386158[] = {{&S78930[12], 5}, {NULL}};
const Segment SGI386159[] = {{&S78930[40], 3}, {NULL}};
const Segment SGI386160[] = {{&S102464[0], 28}, {NULL}};
const Segment SGI386161[] = {{&S102622[0], 18}, {NULL}};
const Segment SGI386162[] = {{&S102623[0], 18}, {NULL}};
const Segment SGI386163[] = {{&S102624[0], 18}, {NULL}};
const Segment SGI386164[] = {{&S79237[12], 5}, {NULL}};
const Segment SGI386165[] = {{&S79237[160], 4}, {NULL}};
const Segment SGI386166[] = {{&S103055[0], 28}, {NULL}};
const Segment SGI386167[] = {{&S103056[0], 28}, {NULL}};
const Segment SGI386168[] = {{&S79468[12], 5}, {NULL}};
const Segment SGI386169[] = {{&S79468[35], 11}, {NULL}};
const Segment SGI386170[] = {{&S79469[12], 5}, {NULL}};
const Segment SGI386171[] = {{&S79469[35], 11}, {NULL}};
const Segment SGI386172[] = {{&S79470[12], 5}, {NULL}};
const Segment SGI386173[] = {{&S79470[35], 11}, {NULL}};
const Segment SGI386174[] = {{&S79647[12], 5}, {NULL}};
const Segment SGI386175[] = {{&S79647[137], 4}, {NULL}};
const Segment SGI386176[] = {{&S79647[64], 9}, {NULL}};
const Segment SGI386177[] = {{&S79671[50], 6}, {NULL}};
const Segment SGI386178[] = {{&S79671[72], 4}, {NULL}};
const Segment SGI386179[] = {{&S79671[0], 9}, {NULL}};
const Segment SGI386180[] = {{&S103254[0], 17}, {NULL}};
const Segment SGI386181[] = {{&S79699[12], 5}, {NULL}};
const Segment SGI386182[] = {{&S79699[159], 4}, {NULL}};
const Segment SGI386183[] = {{&S103329[0], 17}, {NULL}};
const Segment SGI386184[] = {{&S103330[0], 17}, {NULL}};
const Segment SGI386185[] = {{&S103598[0], 18}, {NULL}};
const Segment SGI386186[] = {{&S57529[14], 4}, {NULL}};
const Segment SGI386187[] = {{&S57529[0], 33}, {NULL}};
const Segment SGI386188[] = {{&S103640[0], 17}, {NULL}};
const Segment SGI386189[] = {{&S103641[0], 17}, {NULL}};
const Segment SGI386190[] = {{&S103642[0], 17}, {NULL}};
const Segment SGI386191[] = {{&S80032[63], 2}, {NULL}};
const Segment SGI386192[] = {{&S103879[0], 18}, {NULL}};
const Segment SGI386193[] = {{&S80075[12], 5}, {NULL}};
const Segment SGI386194[] = {{&S80075[160], 4}, {NULL}};
const Segment SGI386195[] = {{&S103946[0], 18}, {NULL}};
const Segment SGI386196[] = {{&S103948[0], 17}, {NULL}};
const Segment SGI386197[] = {{&S103949[0], 18}, {NULL}};
const Segment SGI386198[] = {{&S104175[0], 17}, {NULL}};
const Segment SGI386199[] = {{&S104203[0], 17}, {NULL}};
const Segment SGI386200[] = {{&S104204[0], 18}, {NULL}};
const Segment SGI386201[] = {{&S104213[0], 17}, {NULL}};
const Segment SGI386202[] = {{&S104222[0], 18}, {NULL}};
const Segment SGI386203[] = {{&S104231[0], 17}, {NULL}};
const Segment SGI386204[] = {{&S104236[0], 18}, {NULL}};
const Segment SGI386205[] = {{&S80344[57], 11}, {NULL}};
const Segment SGI386206[] = {{&S104303[0], 28}, {NULL}};
const Segment SGI386207[] = {{&S104312[0], 18}, {NULL}};
const Segment SGI386208[] = {{&S80477[55], 11}, {NULL}};
const Segment SGI386209[] = {{&S80510[12], 5}, {NULL}};
const Segment SGI386210[] = {{&S80510[40], 3}, {NULL}};
const Segment SGI386211[] = {{&S80669[12], 5}, {NULL}};
const Segment SGI386212[] = {{&S80669[121], 3}, {NULL}};
const Segment SGI386213[] = {{&S80688[12], 4}, {NULL}};
const Segment SGI386214[] = {{&S80688[136], 4}, {NULL}};
const Segment SGI386215[] = {{&S80688[63], 9}, {NULL}};
const Segment SGI386216[] = {{&S104641[0], 17}, {NULL}};
const Segment SGI386217[] = {{&S104657[0], 18}, {NULL}};
const Segment SGI386218[] = {{&S104688[0], 17}, {NULL}};
const Segment SGI386219[] = {{&S104745[0], 23}, {NULL}};
const Segment SGI386220[] = {{&S104750[0], 17}, {NULL}};
const Segment SGI386221[] = {{&S104752[0], 17}, {NULL}};
const Segment SGI386222[] = {{&S104753[0], 18}, {NULL}};
const Segment SGI386223[] = {{&S104754[0], 18}, {NULL}};
const Segment SGI386224[] = {{&S104761[0], 28}, {NULL}};
const Segment SGI386225[] = {{&S80838[12], 4}, {NULL}};
const Segment SGI386226[] = {{&S80838[136], 4}, {NULL}};
const Segment SGI386227[] = {{&S80838[63], 9}, {NULL}};
const Segment SGI386228[] = {{&S105136[0], 17}, {NULL}};
const Segment SGI386229[] = {{&S105144[0], 18}, {NULL}};
const Segment SGI386230[] = {{&S80988[12], 5}, {NULL}};
const Segment SGI386231[] = {{&S80988[36], 5}, {NULL}};
const Segment SGI386232[] = {{&S81120[12], 5}, {NULL}};
const Segment SGI386233[] = {{&S81120[40], 3}, {NULL}};
const Segment SGI386234[] = {{&S105402[0], 28}, {NULL}};
const Segment SGI386235[] = {{&S105417[0], 29}, {NULL}};
const Segment SGI386236[] = {{&S105423[0], 18}, {NULL}};
const Segment SGI386237[] = {{&S81378[12], 6}, {NULL}};
const Segment SGI386238[] = {{&S81378[41], 3}, {NULL}};
const Segment SGI386239[] = {{&S81588[12], 3}, {NULL}};
const Segment SGI386240[] = {{&S81588[112], 3}, {NULL}};
const Segment SGI386241[] = {{&S81668[12], 4}, {&S81668[12], 4}, {NULL}};
const Segment SGI386242[] = {{&S81668[39], 7}, {&S81668[39], 7}, {NULL}};
const Segment SGI386243[] = {{&S81669[12], 4}, {NULL}};
const Segment SGI386244[] = {{&S81669[39], 7}, {NULL}};
const Segment SGI386245[] = {{&S81670[12], 5}, {NULL}};
const Segment SGI386246[] = {{&S81670[40], 7}, {NULL}};
const Segment SGI386247[] = {{&S81828[50], 6}, {NULL}};
const Segment SGI386248[] = {{&S81828[72], 4}, {NULL}};
const Segment SGI386249[] = {{&S81828[0], 9}, {NULL}};
const Segment SGI386250[] = {{&S81879[12], 5}, {NULL}};
const Segment SGI386251[] = {{&S81879[92], 3}, {NULL}};
const Segment SGI386252[] = {{&S81949[24], 6}, {NULL}};
const Segment SGI386253[] = {{&S81949[53], 3}, {NULL}};
const Segment SGI386254[] = {{&S82487[55], 11}, {NULL}};
const Segment SGI386255[] = {{&S82510[12], 5}, {NULL}};
const Segment SGI386256[] = {{&S82510[40], 3}, {NULL}};
const Segment SGI386257[] = {{&S82733[12], 5}, {NULL}};
const Segment SGI386258[] = {{&S82733[44], 3}, {NULL}};
const Segment SGI386259[] = {{&S82734[12], 5}, {NULL}};
const Segment SGI386260[] = {{&S82734[43], 3}, {NULL}};
const Segment SGI386261[] = {{&S82737[12], 5}, {NULL}};
const Segment SGI386262[] = {{&S82737[40], 3}, {NULL}};
const Segment SGI386263[] = {{&S82829[12], 5}, {NULL}};
const Segment SGI386264[] = {{&S82829[92], 3}, {NULL}};
const Segment SGI386265[] = {{&S82869[12], 5}, {NULL}};
const Segment SGI386266[] = {{&S82869[138], 4}, {NULL}};
const Segment SGI386267[] = {{&S82869[64], 9}, {NULL}};
const Segment SGI386268[] = {{&S83040[12], 5}, {NULL}};
const Segment SGI386269[] = {{&S83040[40], 3}, {NULL}};
const Segment SGI386270[] = {{&S83042[12], 3}, {NULL}};
const Segment SGI386271[] = {{&S83042[92], 3}, {NULL}};
const Segment SGI386272[] = {{&S83085[12], 3}, {NULL}};
const Segment SGI386273[] = {{&S83085[134], 4}, {NULL}};
const Segment SGI386274[] = {{&S83085[62], 9}, {NULL}};
const Segment SGI386275[] = {{&S83150[12], 4}, {NULL}};
const Segment SGI386276[] = {{&S83150[158], 4}, {NULL}};
const Segment SGI386277[] = {{&S83186[12], 5}, {NULL}};
const Segment SGI386278[] = {{&S83186[138], 4}, {NULL}};
const Segment SGI386279[] = {{&S83186[64], 9}, {NULL}};
const Segment SGI386280[] = {{&S83423[12], 3}, {NULL}};
const Segment SGI386281[] = {{&S83423[132], 4}, {NULL}};
const Segment SGI386282[] = {{&S83423[61], 9}, {NULL}};
const Segment SGI386283[] = {{&S83486[12], 4}, {NULL}};
const Segment SGI386284[] = {{&S83486[39], 3}, {NULL}};
const Segment SGI386285[] = {{&S83490[63], 3}, {NULL}};
const Segment SGI386286[] = {{&S83661[12], 4}, {NULL}};
const Segment SGI386287[] = {{&S83661[136], 4}, {NULL}};
const Segment SGI386288[] = {{&S83661[63], 9}, {NULL}};
const Segment SGI386289[] = {{&S83929[12], 5}, {NULL}};
const Segment SGI386290[] = {{&S83929[121], 3}, {NULL}};
const Segment SGI386291[] = {{&S83997[42], 3}, {NULL}};
const Segment SGI386292[] = {{&S83997[56], 3}, {NULL}};
const Segment SGI386293[] = {{&S84002[12], 5}, {NULL}};
const Segment SGI386294[] = {{&S84002[121], 3}, {NULL}};
const Segment SGI386295[] = {{&S84143[12], 5}, {NULL}};
const Segment SGI386296[] = {{&S84143[138], 4}, {NULL}};
const Segment SGI386297[] = {{&S84143[64], 9}, {NULL}};
const Segment SGI386298[] = {{&S84394[12], 5}, {NULL}};
const Segment SGI386299[] = {{&S84394[121], 3}, {NULL}};
const Segment SGI386300[] = {{&S84582[12], 2}, {NULL}};
const Segment SGI386301[] = {{&S84767[12], 5}, {NULL}};
const Segment SGI386302[] = {{&S84767[43], 3}, {NULL}};
const Segment SGI386303[] = {{&S84768[12], 5}, {NULL}};
const Segment SGI386304[] = {{&S84768[43], 3}, {NULL}};
const Segment SGI386305[] = {{&S84769[12], 5}, {NULL}};
const Segment SGI386306[] = {{&S84769[43], 3}, {NULL}};
const Segment SGI386307[] = {{&S84775[12], 2}, {NULL}};
const Segment SGI386308[] = {{&S84775[91], 3}, {NULL}};
const Segment SGI386309[] = {{&S84817[12], 4}, {NULL}};
const Segment SGI386310[] = {{&S84817[133], 4}, {NULL}};
const Segment SGI386311[] = {{&S84817[63], 9}, {NULL}};
const Segment SGI386312[] = {{&S84947[74], 3}, {NULL}};
const Segment SGI386313[] = {{&S84987[12], 4}, {NULL}};
const Segment SGI386314[] = {{&S84987[135], 4}, {NULL}};
const Segment SGI386315[] = {{&S84987[63], 9}, {NULL}};
const Segment SGI386316[] = {{&S85081[12], 4}, {NULL}};
const Segment SGI386317[] = {{&S85081[135], 4}, {NULL}};
const Segment SGI386318[] = {{&S85081[63], 9}, {NULL}};
const Segment SGI386319[] = {{&S85135[12], 5}, {NULL}};
const Segment SGI386320[] = {{&S85151[12], 4}, {NULL}};
const Segment SGI386321[] = {{&S85151[90], 3}, {NULL}};
const Segment SGI386322[] = {{&S85152[12], 5}, {NULL}};
const Segment SGI386323[] = {{&S85152[92], 3}, {NULL}};
const Segment SGI386324[] = {{&S85258[12], 5}, {NULL}};
const Segment SGI386325[] = {{&S85258[121], 3}, {NULL}};
const Segment SGI386326[] = {{&S85259[12], 4}, {NULL}};
const Segment SGI386327[] = {{&S85259[119], 3}, {NULL}};
const Segment SGI386328[] = {{&S85426[12], 5}, {NULL}};
const Segment SGI386329[] = {{&S85426[40], 3}, {NULL}};
const Segment SGI386330[] = {{&S85470[12], 4}, {NULL}};
const Segment SGI386331[] = {{&S85470[135], 4}, {NULL}};
const Segment SGI386332[] = {{&S85470[63], 9}, {NULL}};
const Segment SGI386333[] = {{&S85568[12], 5}, {NULL}};
const Segment SGI386334[] = {{&S85568[92], 3}, {NULL}};
const Segment SGI386335[] = {{&S85643[12], 5}, {NULL}};
const Segment SGI386336[] = {{&S85643[40], 9}, {NULL}};
const Segment SGI386337[] = {{&S85821[12], 11}, {NULL}};
const Segment SGI386338[] = {{&S85821[23], 4}, {NULL}};
const Segment SGI386339[] = {{&S85955[12], 5}, {NULL}};
const Segment SGI386340[] = {{&S85955[137], 4}, {NULL}};
const Segment SGI386341[] = {{&S85955[64], 9}, {NULL}};
const Segment SGI386342[] = {{&S86077[12], 4}, {NULL}};
const Segment SGI386343[] = {{&S86077[136], 4}, {NULL}};
const Segment SGI386344[] = {{&S86077[63], 9}, {NULL}};
const Segment SGI386345[] = {{&S86143[12], 5}, {NULL}};
const Segment SGI386346[] = {{&S86143[136], 4}, {NULL}};
const Segment SGI386347[] = {{&S86143[63], 9}, {NULL}};
const Segment SGI386348[] = {{&S86147[94], 3}, {NULL}};
const Segment SGI386349[] = {{&S86147[54], 4}, {NULL}};
const Segment SGI386350[] = {{&S86147[0], 9}, {NULL}};
const Segment SGI386351[] = {{&S86263[12], 4}, {NULL}};
const Segment SGI386352[] = {{&S86263[135], 4}, {NULL}};
const Segment SGI386353[] = {{&S86263[63], 9}, {NULL}};
const Segment SGI386354[] = {{&S86366[12], 5}, {NULL}};
const Segment SGI386355[] = {{&S86366[138], 4}, {NULL}};
const Segment SGI386356[] = {{&S86366[64], 9}, {NULL}};
const Segment SGI386357[] = {{&S86414[24], 6}, {NULL}};
const Segment SGI386358[] = {{&S86414[53], 3}, {NULL}};
const Segment SGI386359[] = {{&S86440[12], 3}, {NULL}};
const Segment SGI386360[] = {{&S86440[92], 3}, {NULL}};
const Segment SGI386361[] = {{&S86449[42], 15}, {NULL}};
const Segment SGI386362[] = {{&S86449[94], 4}, {NULL}};
const Segment SGI386363[] = {{&S86516[12], 4}, {NULL}};
const Segment SGI386364[] = {{&S86516[136], 4}, {NULL}};
const Segment SGI386365[] = {{&S86516[63], 9}, {NULL}};
const Segment SGI386366[] = {{&S86530[12], 10}, {NULL}};
const Segment SGI386367[] = {{&S86530[77], 3}, {NULL}};
const Segment SGI386368[] = {{&S86531[12], 5}, {NULL}};
const Segment SGI386369[] = {{&S86531[40], 3}, {NULL}};
const Segment SGI386370[] = {{&S86612[12], 5}, {NULL}};
const Segment SGI386371[] = {{&S86612[138], 4}, {NULL}};
const Segment SGI386372[] = {{&S86612[64], 9}, {NULL}};
const Segment SGI386373[] = {{&S86770[12], 5}, {NULL}};
const Segment SGI386374[] = {{&S86770[35], 5}, {NULL}};
const Segment SGI386375[] = {{&S86771[12], 5}, {NULL}};
const Segment SGI386376[] = {{&S86771[36], 5}, {NULL}};
const Segment SGI386377[] = {{&S86772[12], 5}, {NULL}};
const Segment SGI386378[] = {{&S86772[36], 5}, {NULL}};
const Segment SGI386379[] = {{&S86773[12], 5}, {NULL}};
const Segment SGI386380[] = {{&S86773[36], 5}, {NULL}};
const Segment SGI386381[] = {{&S86774[12], 5}, {NULL}};
const Segment SGI386382[] = {{&S86774[35], 5}, {NULL}};
const Segment SGI386383[] = {{&S86775[12], 5}, {NULL}};
const Segment SGI386384[] = {{&S86775[36], 5}, {NULL}};
const Segment SGI386385[] = {{&S86878[61], 18}, {NULL}};
const Segment SGI386386[] = {{&S86878[29], 19}, {NULL}};
const Segment SGI386387[] = {{&S86993[50], 6}, {NULL}};
const Segment SGI386388[] = {{&S86993[72], 4}, {NULL}};
const Segment SGI386389[] = {{&S86993[0], 9}, {NULL}};
const Segment SGI386390[] = {{&S87120[12], 5}, {NULL}};
const Segment SGI386391[] = {{&S87120[160], 4}, {NULL}};
const Segment SGI386392[] = {{&S87181[62], 3}, {NULL}};
const Segment SGI386393[] = {{&S87181[41], 5}, {NULL}};
const Segment SGI386394[] = {{&S87280[12], 5}, {NULL}};
const Segment SGI386395[] = {{&S87280[138], 4}, {NULL}};
const Segment SGI386396[] = {{&S87280[64], 9}, {NULL}};
const Segment SGI386397[] = {{&S87298[12], 2}, {NULL}};
const Segment SGI386398[] = {{&S87298[25], 3}, {NULL}};
const Segment SGI386399[] = {{&S87325[12], 5}, {NULL}};
const Segment SGI386400[] = {{&S87325[137], 4}, {NULL}};
const Segment SGI386401[] = {{&S87325[64], 9}, {NULL}};
const Segment SGI386402[] = {{&S87326[12], 4}, {NULL}};
const Segment SGI386403[] = {{&S87326[135], 4}, {NULL}};
const Segment SGI386404[] = {{&S87326[63], 9}, {NULL}};
const Segment SGI386405[] = {{&S87522[12], 10}, {NULL}};
const Segment SGI386406[] = {{&S87522[45], 3}, {NULL}};
const Segment SGI386407[] = {{&S87526[12], 5}, {NULL}};
const Segment SGI386408[] = {{&S87526[40], 3}, {NULL}};
const Segment SGI386409[] = {{&S87620[28], 5}, {NULL}};
const Segment SGI386410[] = {{&S87835[12], 5}, {NULL}};
const Segment SGI386411[] = {{&S87835[157], 4}, {NULL}};
const Segment SGI386412[] = {{&S87836[12], 4}, {NULL}};
const Segment SGI386413[] = {{&S87836[155], 4}, {NULL}};
const Segment SGI386414[] = {{&S87897[12], 3}, {NULL}};
const Segment SGI386415[] = {{&S87897[134], 4}, {NULL}};
const Segment SGI386416[] = {{&S87897[62], 9}, {NULL}};
const Segment SGI386417[] = {{&S87903[12], 5}, {NULL}};
const Segment SGI386418[] = {{&S87929[12], 3}, {NULL}};
const Segment SGI386419[] = {{&S87929[112], 3}, {NULL}};
const Segment SGI386420[] = {{&S88233[12], 4}, {NULL}};
const Segment SGI386421[] = {{&S88233[136], 4}, {NULL}};
const Segment SGI386422[] = {{&S88233[63], 9}, {NULL}};
const Segment SGI386423[] = {{&S88408[12], 5}, {NULL}};
const Segment SGI386424[] = {{&S88476[12], 4}, {NULL}};
const Segment SGI386425[] = {{&S88476[136], 4}, {NULL}};
const Segment SGI386426[] = {{&S88476[63], 9}, {NULL}};
const Segment SGI386427[] = {{&S88543[12], 4}, {NULL}};
const Segment SGI386428[] = {{&S88543[133], 4}, {NULL}};
const Segment SGI386429[] = {{&S88543[63], 9}, {NULL}};
const Segment SGI386430[] = {{&S88690[12], 5}, {NULL}};
const Segment SGI386431[] = {{&S88690[44], 3}, {NULL}};
const Segment SGI386432[] = {{&S88702[12], 2}, {NULL}};
const Segment SGI386433[] = {{&S88713[42], 3}, {NULL}};
const Segment SGI386434[] = {{&S88713[56], 3}, {NULL}};
const Segment SGI386435[] = {{&S88821[12], 5}, {NULL}};
const Segment SGI386436[] = {{&S88821[44], 3}, {NULL}};
const Segment SGI386437[] = {{&S88822[12], 5}, {NULL}};
const Segment SGI386438[] = {{&S88825[12], 4}, {NULL}};
const Segment SGI386439[] = {{&S89082[12], 5}, {NULL}};
const Segment SGI386440[] = {{&S89082[138], 4}, {NULL}};
const Segment SGI386441[] = {{&S89082[64], 9}, {NULL}};
const Segment SGI386442[] = {{&S89083[12], 4}, {NULL}};
const Segment SGI386443[] = {{&S89083[133], 4}, {NULL}};
const Segment SGI386444[] = {{&S89083[63], 9}, {NULL}};
const Segment SGI386445[] = {{&S89167[12], 4}, {NULL}};
const Segment SGI386446[] = {{&S89167[136], 4}, {NULL}};
const Segment SGI386447[] = {{&S89167[63], 9}, {NULL}};
const Segment SGI386448[] = {{&S89226[12], 5}, {NULL}};
const Segment SGI386449[] = {{&S89226[41], 3}, {NULL}};
const Segment SGI386450[] = {{&S89277[42], 16}, {NULL}};
const Segment SGI386451[] = {{&S89277[95], 4}, {NULL}};
const Segment SGI386452[] = {{&S89389[12], 4}, {NULL}};
const Segment SGI386453[] = {{&S89409[12], 5}, {NULL}};
const Segment SGI386454[] = {{&S89409[92], 3}, {NULL}};
const Segment SGI386455[] = {{&S89757[12], 4}, {NULL}};
const Segment SGI386456[] = {{&S89757[157], 4}, {NULL}};
const Segment SGI386457[] = {{&S89785[12], 4}, {NULL}};
const Segment SGI386458[] = {{&S89785[136], 4}, {NULL}};
const Segment SGI386459[] = {{&S89785[63], 9}, {NULL}};
const Segment SGI386460[] = {{&S89791[50], 7}, {NULL}};
const Segment SGI386461[] = {{&S89791[73], 4}, {NULL}};
const Segment SGI386462[] = {{&S89791[0], 9}, {NULL}};
const Segment SGI386463[] = {{&S90014[12], 5}, {NULL}};
const Segment SGI386464[] = {{&S90014[35], 5}, {NULL}};
const Segment SGI386465[] = {{&S90015[12], 5}, {NULL}};
const Segment SGI386466[] = {{&S90015[35], 5}, {NULL}};
const Segment SGI386467[] = {{&S90016[12], 5}, {NULL}};
const Segment SGI386468[] = {{&S90016[35], 5}, {NULL}};
const Segment SGI386469[] = {{&S90018[12], 5}, {NULL}};
const Segment SGI386470[] = {{&S90019[12], 6}, {NULL}};
const Segment SGI386471[] = {{&S90020[12], 4}, {NULL}};
const Segment SGI386472[] = {{&S90139[12], 3}, {NULL}};
const Segment SGI386473[] = {{&S90139[134], 4}, {NULL}};
const Segment SGI386474[] = {{&S90139[62], 9}, {NULL}};
const Segment SGI386475[] = {{&S90221[12], 3}, {NULL}};
const Segment SGI386476[] = {{&S90221[156], 4}, {NULL}};
const Segment SGI386477[] = {{&S90344[12], 4}, {NULL}};
const Segment SGI386478[] = {{&S90344[136], 4}, {NULL}};
const Segment SGI386479[] = {{&S90344[63], 9}, {NULL}};
const Segment SGI386480[] = {{&S90419[12], 4}, {NULL}};
const Segment SGI386481[] = {{&S90419[135], 4}, {NULL}};
const Segment SGI386482[] = {{&S90419[63], 9}, {NULL}};
const Segment SGI386483[] = {{&S90507[12], 4}, {NULL}};
const Segment SGI386484[] = {{&S90507[136], 4}, {NULL}};
const Segment SGI386485[] = {{&S90507[63], 9}, {NULL}};
const Segment SGI386486[] = {{&S90540[12], 5}, {NULL}};
const Segment SGI386487[] = {{&S90568[12], 4}, {NULL}};
const Segment SGI386488[] = {{&S90568[119], 3}, {NULL}};
const Segment SGI386489[] = {{&S90780[12], 5}, {NULL}};
const Segment SGI386490[] = {{&S90780[121], 3}, {NULL}};
const Segment SGI386491[] = {{&S90828[12], 4}, {NULL}};
const Segment SGI386492[] = {{&S90828[135], 4}, {NULL}};
const Segment SGI386493[] = {{&S90828[63], 9}, {NULL}};
const Segment SGI386494[] = {{&S91292[12], 4}, {NULL}};
const Segment SGI386495[] = {{&S91292[90], 3}, {NULL}};
const Segment SGI386496[] = {{&S91470[12], 4}, {NULL}};
const Segment SGI386497[] = {{&S91668[12], 4}, {NULL}};
const Segment SGI386498[] = {{&S91668[39], 3}, {NULL}};
const Segment SGI386499[] = {{&S91685[12], 2}, {NULL}};
const Segment SGI386500[] = {{&S91686[12], 4}, {NULL}};
const Segment SGI386501[] = {{&S91795[95], 3}, {NULL}};
const Segment SGI386502[] = {{&S91795[55], 4}, {NULL}};
const Segment SGI386503[] = {{&S91795[0], 9}, {NULL}};
const Segment SGI386504[] = {{&S91879[12], 3}, {NULL}};
const Segment SGI386505[] = {{&S91916[42], 3}, {NULL}};
const Segment SGI386506[] = {{&S91916[56], 3}, {NULL}};
const Segment SGI386507[] = {{&S92166[12], 4}, {NULL}};
const Segment SGI386508[] = {{&S92166[135], 4}, {NULL}};
const Segment SGI386509[] = {{&S92166[63], 9}, {NULL}};
const Segment SGI386510[] = {{&S92174[49], 7}, {NULL}};
const Segment SGI386511[] = {{&S92174[72], 4}, {NULL}};
const Segment SGI386512[] = {{&S92174[0], 9}, {NULL}};
const Segment SGI386513[] = {{&S92287[12], 5}, {NULL}};
const Segment SGI386514[] = {{&S92323[12], 4}, {NULL}};
const Segment SGI386515[] = {{&S92323[119], 3}, {NULL}};
const Segment SGI386516[] = {{&S92338[12], 5}, {NULL}};
const Segment SGI386517[] = {{&S92338[121], 3}, {NULL}};
const Segment SGI386518[] = {{&S92489[12], 2}, {NULL}};
const Segment SGI386519[] = {{&S92489[91], 3}, {NULL}};
const Segment SGI386520[] = {{&S92498[12], 4}, {NULL}};
const Segment SGI386521[] = {{&S92498[90], 3}, {NULL}};
const Segment SGI386522[] = {{&S92502[42], 3}, {NULL}};
const Segment SGI386523[] = {{&S92502[56], 3}, {NULL}};
const Segment SGI386524[] = {{&S92503[12], 4}, {NULL}};
const Segment SGI386525[] = {{&S92503[90], 3}, {NULL}};
const Segment SGI386526[] = {{&S92827[12], 4}, {NULL}};
const Segment SGI386527[] = {{&S92827[42], 3}, {NULL}};
const Segment SGI386528[] = {{&S92828[12], 5}, {NULL}};
const Segment SGI386529[] = {{&S92828[43], 3}, {NULL}};
const Segment SGI386530[] = {{&S92835[12], 4}, {NULL}};
const Segment SGI386531[] = {{&S92862[12], 5}, {NULL}};
const Segment SGI386532[] = {{&S92862[92], 3}, {NULL}};
const Segment SGI386533[] = {{&S92863[12], 4}, {NULL}};
const Segment SGI386534[] = {{&S92863[90], 3}, {NULL}};
const Segment SGI386535[] = {{&S92972[12], 4}, {NULL}};
const Segment SGI386536[] = {{&S92972[136], 4}, {NULL}};
const Segment SGI386537[] = {{&S92972[63], 9}, {NULL}};
const Segment SGI386538[] = {{&S92987[12], 4}, {NULL}};
const Segment SGI386539[] = {{&S92987[136], 4}, {NULL}};
const Segment SGI386540[] = {{&S92987[63], 9}, {NULL}};
const Segment SGI386541[] = {{&S93039[50], 6}, {NULL}};
const Segment SGI386542[] = {{&S93039[72], 4}, {NULL}};
const Segment SGI386543[] = {{&S93039[0], 9}, {NULL}};
const Segment SGI386544[] = {{&S93054[12], 5}, {NULL}};
const Segment SGI386545[] = {{&S93054[36], 5}, {NULL}};
const Segment SGI386546[] = {{&S93059[12], 8}, {NULL}};
const Segment SGI386547[] = {{&S93073[12], 4}, {NULL}};
const Segment SGI386548[] = {{&S93073[90], 3}, {NULL}};
const Segment SGI386549[] = {{&S93212[12], 4}, {NULL}};
const Segment SGI386550[] = {{&S93212[133], 4}, {NULL}};
const Segment SGI386551[] = {{&S93212[63], 9}, {NULL}};
const Segment SGI386552[] = {{&S70715[14], 4}, {NULL}};
const Segment SGI386553[] = {{&S70715[0], 33}, {NULL}};
const Segment SGI386554[] = {{&S70716[14], 5}, {NULL}};
const Segment SGI386555[] = {{&S70716[0], 50}, {NULL}};
const Segment SGI386556[] = {{&S70717[14], 4}, {NULL}};
const Segment SGI386557[] = {{&S70717[0], 33}, {NULL}};
const Segment SGI386558[] = {{&S70718[14], 4}, {NULL}};
const Segment SGI386559[] = {{&S70718[0], 33}, {NULL}};
const Segment SGI386560[] = {{&S70719[14], 4}, {NULL}};
const Segment SGI386561[] = {{&S70719[0], 33}, {NULL}};
const Segment SGI386562[] = {{&S70720[14], 3}, {NULL}};
const Segment SGI386563[] = {{&S70720[0], 48}, {NULL}};
const Segment SGI386564[] = {{&S70721[14], 4}, {NULL}};
const Segment SGI386565[] = {{&S70721[0], 49}, {NULL}};
const Segment SGI386566[] = {{&S70722[14], 4}, {NULL}};
const Segment SGI386567[] = {{&S70722[0], 49}, {NULL}};
const Segment SGI386568[] = {{&S70723[14], 4}, {NULL}};
const Segment SGI386569[] = {{&S70723[0], 33}, {NULL}};
const Segment SGI386570[] = {{&S70724[14], 4}, {NULL}};
const Segment SGI386571[] = {{&S70724[0], 49}, {NULL}};
const Segment SGI386572[] = {{&S70725[14], 4}, {NULL}};
const Segment SGI386573[] = {{&S70725[0], 49}, {NULL}};
const Segment SGI386574[] = {{&S70726[14], 4}, {NULL}};
const Segment SGI386575[] = {{&S70726[0], 49}, {NULL}};
const Segment SGI386576[] = {{&S70727[14], 5}, {NULL}};
const Segment SGI386577[] = {{&S70727[0], 34}, {NULL}};
const Segment SGI386578[] = {{&S70728[14], 5}, {NULL}};
const Segment SGI386579[] = {{&S70728[0], 34}, {NULL}};
const Segment SGI386580[] = {{&S70729[14], 4}, {NULL}};
const Segment SGI386581[] = {{&S70729[0], 33}, {NULL}};
const Segment SGI386582[] = {{&S70730[14], 4}, {NULL}};
const Segment SGI386583[] = {{&S70730[0], 49}, {NULL}};
const Segment SGI386584[] = {{&S70731[14], 4}, {NULL}};
const Segment SGI386585[] = {{&S70731[0], 49}, {NULL}};
const Segment SGI386586[] = {{&S70732[14], 4}, {NULL}};
const Segment SGI386587[] = {{&S70732[0], 33}, {NULL}};
const Segment SGI386588[] = {{&S70733[14], 4}, {NULL}};
const Segment SGI386589[] = {{&S70733[0], 49}, {NULL}};
const Segment SGI386590[] = {{&S70734[14], 4}, {NULL}};
const Segment SGI386591[] = {{&S70734[0], 49}, {NULL}};
const Segment SGI386592[] = {{&S70735[14], 4}, {NULL}};
const Segment SGI386593[] = {{&S70735[0], 49}, {NULL}};
const Segment SGI386594[] = {{&S70736[14], 4}, {NULL}};
const Segment SGI386595[] = {{&S70736[0], 49}, {NULL}};
const Segment SGI386596[] = {{&S70737[14], 4}, {NULL}};
const Segment SGI386597[] = {{&S70737[0], 49}, {NULL}};
const Segment SGI386598[] = {{&S70738[14], 4}, {NULL}};
const Segment SGI386599[] = {{&S70738[0], 49}, {NULL}};
const Segment SGI386600[] = {{&S70739[14], 4}, {NULL}};
const Segment SGI386601[] = {{&S70739[0], 33}, {NULL}};
const Segment SGI386602[] = {{&S70740[14], 4}, {NULL}};
const Segment SGI386603[] = {{&S70740[0], 49}, {NULL}};
const Segment SGI386604[] = {{&S70741[14], 4}, {NULL}};
const Segment SGI386605[] = {{&S70741[0], 49}, {NULL}};
const Segment SGI386606[] = {{&S70742[14], 4}, {NULL}};
const Segment SGI386607[] = {{&S70742[0], 49}, {NULL}};
const Segment SGI386608[] = {{&S70743[14], 4}, {NULL}};
const Segment SGI386609[] = {{&S70743[0], 33}, {NULL}};
const Segment SGI386610[] = {{&S70744[14], 4}, {NULL}};
const Segment SGI386611[] = {{&S70744[0], 49}, {NULL}};
const Segment SGI386612[] = {{&S93324[12], 4}, {NULL}};
const Segment SGI386613[] = {{&S93324[90], 3}, {NULL}};
const Segment SGI386614[] = {{&S93337[42], 3}, {NULL}};
const Segment SGI386615[] = {{&S93337[56], 3}, {NULL}};
const Segment SGI386616[] = {{&S93404[12], 5}, {NULL}};
const Segment SGI386617[] = {{&S93404[159], 4}, {NULL}};
const Segment SGI386618[] = {{&S93405[12], 4}, {NULL}};
const Segment SGI386619[] = {{&S93405[155], 4}, {NULL}};
const Segment SGI386620[] = {{&S93496[12], 4}, {NULL}};
const Segment SGI386621[] = {{&S93496[39], 3}, {NULL}};
const Segment SGI386622[] = {{&S93497[12], 4}, {NULL}};
const Segment SGI386623[] = {{&S93497[39], 3}, {NULL}};
const Segment SGI386624[] = {{&S93506[12], 5}, {NULL}};
const Segment SGI386625[] = {{&S93508[12], 5}, {NULL}};
const Segment SGI386626[] = {{&S93518[12], 5}, {NULL}};
const Segment SGI386627[] = {{&S93518[40], 3}, {NULL}};
const Segment SGI386628[] = {{&S93519[12], 5}, {NULL}};
const Segment SGI386629[] = {{&S93519[40], 3}, {NULL}};
const Segment SGI386630[] = {{&S93520[12], 5}, {NULL}};
const Segment SGI386631[] = {{&S93520[40], 3}, {NULL}};
const Segment SGI386632[] = {{&S93521[12], 5}, {NULL}};
const Segment SGI386633[] = {{&S93521[40], 3}, {NULL}};
const Segment SGI386634[] = {{&S93556[12], 4}, {NULL}};
const Segment SGI386635[] = {{&S93556[90], 3}, {NULL}};
const Segment SGI386636[] = {{&S93563[57], 9}, {NULL}};
const Segment SGI386637[] = {{&S93583[12], 4}, {NULL}};
const Segment SGI386638[] = {{&S93583[119], 3}, {NULL}};
const Segment SGI386639[] = {{&S93744[12], 5}, {NULL}};
const Segment SGI386640[] = {{&S93745[12], 5}, {NULL}};
const Segment SGI386641[] = {{&S93815[12], 3}, {NULL}};
const Segment SGI386642[] = {{&S93815[117], 3}, {NULL}};
const Segment SGI386643[] = {{&S93960[12], 4}, {NULL}};
const Segment SGI386644[] = {{&S94064[12], 3}, {NULL}};
const Segment SGI386645[] = {{&S94064[155], 4}, {NULL}};
const Segment SGI386646[] = {{&S94169[50], 6}, {NULL}};
const Segment SGI386647[] = {{&S94169[72], 4}, {NULL}};
const Segment SGI386648[] = {{&S94169[0], 9}, {NULL}};
const Segment SGI386649[] = {{&S94195[12], 5}, {NULL}};
const Segment SGI386650[] = {{&S94195[40], 3}, {NULL}};
const Segment SGI386651[] = {{&S94196[12], 5}, {NULL}};
const Segment SGI386652[] = {{&S94196[40], 3}, {NULL}};
const Segment SGI386653[] = {{&S94217[12], 4}, {NULL}};
const Segment SGI386654[] = {{&S94217[90], 3}, {NULL}};
const Segment SGI386655[] = {{&S94389[12], 5}, {NULL}};
const Segment SGI386656[] = {{&S94394[12], 5}, {NULL}};
const Segment SGI386657[] = {{&S94394[40], 3}, {NULL}};
const Segment SGI386658[] = {{&S94397[12], 5}, {NULL}};
const Segment SGI386659[] = {{&S94645[12], 4}, {NULL}};
const Segment SGI386660[] = {{&S94967[26], 12}, {NULL}};
const Segment SGI386661[] = {{&S94967[39], 4}, {NULL}};
const Segment SGI386662[] = {{&S94967[52], 4}, {NULL}};
const Segment SGI386663[] = {{&S95002[28], 4}, {NULL}};
const Segment SGI386664[] = {{&S95014[28], 5}, {NULL}};
const Segment SGI386665[] = {{&S95128[12], 5}, {NULL}};
const Segment SGI386666[] = {{&S95128[36], 4}, {NULL}};
const Segment SGI386667[] = {{&S95129[12], 5}, {NULL}};
const Segment SGI386668[] = {{&S95129[36], 4}, {NULL}};
const Segment SGI386669[] = {{&S95199[42], 4}, {NULL}};
const Segment SGI386670[] = {{&S95199[57], 3}, {NULL}};
const Segment SGI386671[] = {{&S95218[28], 12}, {NULL}};
const Segment SGI386672[] = {{&S95218[41], 4}, {NULL}};
const Segment SGI386673[] = {{&S95218[54], 4}, {NULL}};
const Segment SGI386674[] = {{&S95365[12], 5}, {NULL}};
const Segment SGI386675[] = {{&S95365[40], 3}, {NULL}};
const Segment SGI386676[] = {{&S95366[12], 5}, {NULL}};
const Segment SGI386677[] = {{&S95366[40], 3}, {NULL}};
const Segment SGI386678[] = {{&S95540[12], 4}, {NULL}};
const Segment SGI386679[] = {{&S95540[136], 4}, {NULL}};
const Segment SGI386680[] = {{&S95540[63], 9}, {NULL}};
const Segment SGI386681[] = {{&S95669[12], 5}, {NULL}};
const Segment SGI386682[] = {{&S95670[12], 5}, {NULL}};
const Segment SGI386683[] = {{&S95673[12], 5}, {&S95673[12], 5}, {NULL}};
const Segment SGI386684[] = {{&S95674[12], 5}, {NULL}};
const Segment SGI386685[] = {{&S95705[12], 4}, {NULL}};
const Segment SGI386686[] = {{&S95705[90], 3}, {NULL}};
const Segment SGI386687[] = {{&S95836[28], 4}, {NULL}};
const Segment SGI386688[] = {{&S95909[50], 7}, {NULL}};
const Segment SGI386689[] = {{&S95909[73], 4}, {NULL}};
const Segment SGI386690[] = {{&S95909[0], 9}, {NULL}};
const Segment SGI386691[] = {{&S95956[12], 5}, {NULL}};
const Segment SGI386692[] = {{&S95956[40], 3}, {NULL}};
const Segment SGI386693[] = {{&S95957[12], 5}, {NULL}};
const Segment SGI386694[] = {{&S95957[40], 3}, {NULL}};
const Segment SGI386695[] = {{&S95961[12], 6}, {NULL}};
const Segment SGI386696[] = {{&S95967[12], 5}, {NULL}};
const Segment SGI386697[] = {{&S96136[26], 12}, {NULL}};
const Segment SGI386698[] = {{&S96136[39], 4}, {NULL}};
const Segment SGI386699[] = {{&S96136[52], 4}, {NULL}};
const Segment SGI386700[] = {{&S96287[12], 5}, {NULL}};
const Segment SGI386701[] = {{&S96293[12], 5}, {NULL}};
const Segment SGI386702[] = {{&S96294[12], 5}, {NULL}};
const Segment SGI386703[] = {{&S96384[12], 4}, {NULL}};
const Segment SGI386704[] = {{&S96384[135], 4}, {NULL}};
const Segment SGI386705[] = {{&S96384[63], 9}, {NULL}};
const Segment SGI386706[] = {{&S96598[12], 5}, {NULL}};
const Segment SGI386707[] = {{&S96598[36], 5}, {NULL}};
const Segment SGI386708[] = {{&S96599[12], 5}, {NULL}};
const Segment SGI386709[] = {{&S96599[36], 5}, {NULL}};
const Segment SGI386710[] = {{&S96600[12], 5}, {NULL}};
const Segment SGI386711[] = {{&S96600[36], 5}, {NULL}};
const Segment SGI386712[] = {{&S96914[12], 4}, {NULL}};
const Segment SGI386713[] = {{&S96916[12], 5}, {NULL}};
const Segment SGI386714[] = {{&S96916[40], 3}, {NULL}};
const Segment SGI386715[] = {{&S96917[12], 4}, {NULL}};
const Segment SGI386716[] = {{&S96917[58], 3}, {NULL}};
const Segment SGI386717[] = {{&S96918[12], 5}, {NULL}};
const Segment SGI386718[] = {{&S96918[44], 3}, {NULL}};
const Segment SGI386719[] = {{&S96942[12], 4}, {NULL}};
const Segment SGI386720[] = {{&S97132[12], 4}, {NULL}};
const Segment SGI386721[] = {{&S97132[136], 4}, {NULL}};
const Segment SGI386722[] = {{&S97132[63], 9}, {NULL}};
const Segment SGI386723[] = {{&S97252[12], 5}, {NULL}};
const Segment SGI386724[] = {{&S97252[35], 5}, {NULL}};
const Segment SGI386725[] = {{&S97253[12], 5}, {NULL}};
const Segment SGI386726[] = {{&S97253[35], 5}, {NULL}};
const Segment SGI386727[] = {{&S97254[12], 5}, {NULL}};
const Segment SGI386728[] = {{&S97254[35], 3}, {NULL}};
const Segment SGI386729[] = {{&S97261[12], 5}, {NULL}};
const Segment SGI386730[] = {{&S97281[12], 3}, {NULL}};
const Segment SGI386731[] = {{&S97281[88], 3}, {NULL}};
const Segment SGI386732[] = {{&S97284[12], 5}, {NULL}};
const Segment SGI386733[] = {{&S97284[92], 3}, {NULL}};
const Segment SGI386734[] = {{&S97286[12], 3}, {NULL}};
const Segment SGI386735[] = {{&S97286[88], 3}, {NULL}};
const Segment SGI386736[] = {{&S97361[12], 4}, {NULL}};
const Segment SGI386737[] = {{&S97361[135], 4}, {NULL}};
const Segment SGI386738[] = {{&S97361[63], 9}, {NULL}};
const Segment SGI386739[] = {{&S97635[12], 4}, {NULL}};
const Segment SGI386740[] = {{&S97636[12], 5}, {NULL}};
const Segment SGI386741[] = {{&S97666[12], 2}, {NULL}};
const Segment SGI386742[] = {{&S97666[25], 3}, {NULL}};
const Segment SGI386743[] = {{&S97768[12], 3}, {NULL}};
const Segment SGI386744[] = {{&S97768[133], 4}, {NULL}};
const Segment SGI386745[] = {{&S97768[62], 9}, {NULL}};
const Segment SGI386746[] = {{&S97811[12], 5}, {NULL}};
const Segment SGI386747[] = {{&S97811[137], 4}, {NULL}};
const Segment SGI386748[] = {{&S97811[63], 9}, {NULL}};
const Segment SGI386749[] = {{&S97869[12], 4}, {NULL}};
const Segment SGI386750[] = {{&S97869[155], 4}, {NULL}};
const Segment SGI386751[] = {{&S98050[12], 5}, {NULL}};
const Segment SGI386752[] = {{&S98051[12], 4}, {NULL}};
const Segment SGI386753[] = {{&S98054[12], 5}, {NULL}};
const Segment SGI386754[] = {{&S98054[35], 3}, {NULL}};
const Segment SGI386755[] = {{&S98055[12], 5}, {NULL}};
const Segment SGI386756[] = {{&S98055[36], 5}, {NULL}};
const Segment SGI386757[] = {{&S98056[12], 5}, {NULL}};
const Segment SGI386758[] = {{&S98056[35], 5}, {NULL}};
const Segment SGI386759[] = {{&S98057[12], 5}, {NULL}};
const Segment SGI386760[] = {{&S98057[36], 5}, {NULL}};
const Segment SGI386761[] = {{&S98088[12], 5}, {NULL}};
const Segment SGI386762[] = {{&S98088[92], 3}, {NULL}};
const Segment SGI386763[] = {{&S98116[12], 5}, {NULL}};
const Segment SGI386764[] = {{&S98116[92], 3}, {NULL}};
const Segment SGI386765[] = {{&S98138[12], 4}, {NULL}};
const Segment SGI386766[] = {{&S98138[119], 3}, {NULL}};
const Segment SGI386767[] = {{&S98230[94], 5}, {NULL}};
const Segment SGI386768[] = {{&S98230[54], 4}, {NULL}};
const Segment SGI386769[] = {{&S98230[0], 9}, {NULL}};
const Segment SGI386770[] = {{&S98519[12], 4}, {NULL}};
const Segment SGI386771[] = {{&S98539[12], 5}, {NULL}};
const Segment SGI386772[] = {{&S98550[56], 4}, {NULL}};
const Segment SGI386773[] = {{&S98550[8], 3}, {NULL}};
const Segment SGI386774[] = {{&S98565[12], 4}, {NULL}};
const Segment SGI386775[] = {{&S98565[90], 3}, {NULL}};
const Segment SGI386776[] = {{&S98619[42], 16}, {NULL}};
const Segment SGI386777[] = {{&S98619[95], 4}, {NULL}};
const Segment SGI386778[] = {{&S98734[68], 11}, {NULL}};
const Segment SGI386779[] = {{&S98843[12], 4}, {NULL}};
const Segment SGI386780[] = {{&S98843[136], 4}, {NULL}};
const Segment SGI386781[] = {{&S98843[63], 9}, {NULL}};
const Segment SGI386782[] = {{&S98895[28], 5}, {NULL}};
const Segment SGI386783[] = {{&S98945[12], 4}, {NULL}};
const Segment SGI386784[] = {{&S98945[136], 4}, {NULL}};
const Segment SGI386785[] = {{&S98945[63], 9}, {NULL}};
const Segment SGI386786[] = {{&S99031[12], 5}, {NULL}};
const Segment SGI386787[] = {{&S99039[12], 5}, {NULL}};
const Segment SGI386788[] = {{&S99075[12], 4}, {NULL}};
const Segment SGI386789[] = {{&S99075[90], 3}, {NULL}};
const Segment SGI386790[] = {{&S99161[12], 11}, {NULL}};
const Segment SGI386791[] = {{&S99161[23], 4}, {NULL}};
const Segment SGI386792[] = {{&S99281[12], 5}, {NULL}};
const Segment SGI386793[] = {{&S99281[121], 3}, {NULL}};
const Segment SGI386794[] = {{&S99339[12], 4}, {NULL}};
const Segment SGI386795[] = {{&S99339[136], 4}, {NULL}};
const Segment SGI386796[] = {{&S99339[63], 9}, {NULL}};
const Segment SGI386797[] = {{&S99371[12], 5}, {NULL}};
const Segment SGI386798[] = {{&S99371[138], 4}, {NULL}};
const Segment SGI386799[] = {{&S99371[64], 9}, {NULL}};
const Segment SGI386800[] = {{&S99426[28], 5}, {NULL}};
const Segment SGI386801[] = {{&S99580[12], 4}, {NULL}};
const Segment SGI386802[] = {{&S99580[90], 3}, {NULL}};
const Segment SGI386803[] = {{&S99689[12], 4}, {NULL}};
const Segment SGI386804[] = {{&S99689[134], 4}, {NULL}};
const Segment SGI386805[] = {{&S99689[62], 9}, {NULL}};
const Segment SGI386806[] = {{&S99725[12], 4}, {NULL}};
const Segment SGI386807[] = {{&S99725[134], 4}, {NULL}};
const Segment SGI386808[] = {{&S99725[62], 9}, {NULL}};
const Segment SGI386809[] = {{&S99743[12], 4}, {NULL}};
const Segment SGI386810[] = {{&S99743[132], 4}, {NULL}};
const Segment SGI386811[] = {{&S99743[63], 9}, {NULL}};
const Segment SGI386812[] = {{&S99840[12], 4}, {NULL}};
const Segment SGI386813[] = {{&S99840[135], 4}, {NULL}};
const Segment SGI386814[] = {{&S99840[63], 9}, {NULL}};
const Segment SGI386815[] = {{&S99880[12], 4}, {NULL}};
const Segment SGI386816[] = {{&S99880[135], 4}, {NULL}};
const Segment SGI386817[] = {{&S99880[62], 9}, {NULL}};
const Segment SGI386818[] = {{&S99900[12], 4}, {NULL}};
const Segment SGI386819[] = {{&S99900[136], 4}, {NULL}};
const Segment SGI386820[] = {{&S99900[63], 9}, {NULL}};
const Segment SGI386821[] = {{&S99910[95], 5}, {NULL}};
const Segment SGI386822[] = {{&S99910[55], 4}, {NULL}};
const Segment SGI386823[] = {{&S99910[0], 9}, {NULL}};
const Segment SGI386824[] = {{&S99935[28], 4}, {NULL}};
const Segment SGI386825[] = {{&S100105[24], 6}, {NULL}};
const Segment SGI386826[] = {{&S100106[12], 5}, {NULL}};
const Segment SGI386827[] = {{&S100111[90], 3}, {NULL}};
const Segment SGI386828[] = {{&S100160[12], 3}, {NULL}};
const Segment SGI386829[] = {{&S100174[12], 4}, {NULL}};
const Segment SGI386830[] = {{&S100174[90], 3}, {NULL}};
const Segment SGI386831[] = {{&S100176[12], 4}, {NULL}};
const Segment SGI386832[] = {{&S100176[90], 3}, {NULL}};
const Segment SGI386833[] = {{&S100240[12], 4}, {NULL}};
const Segment SGI386834[] = {{&S100240[119], 3}, {NULL}};
const Segment SGI386835[] = {{&S100243[12], 4}, {NULL}};
const Segment SGI386836[] = {{&S100243[90], 3}, {NULL}};
const Segment SGI386837[] = {{&S100246[12], 4}, {NULL}};
const Segment SGI386838[] = {{&S100246[119], 3}, {NULL}};
const Segment SGI386839[] = {{&S100298[12], 4}, {NULL}};
const Segment SGI386840[] = {{&S100298[119], 3}, {NULL}};
const Segment SGI386841[] = {{&S100299[81], 5}, {NULL}};
const Segment SGI386842[] = {{&S100299[38], 3}, {NULL}};
const Segment SGI386843[] = {{&S100319[12], 4}, {NULL}};
const Segment SGI386844[] = {{&S100319[90], 3}, {NULL}};
const Segment SGI386845[] = {{&S100329[12], 4}, {NULL}};
const Segment SGI386846[] = {{&S100329[154], 4}, {NULL}};
const Segment SGI386847[] = {{&S100343[12], 5}, {NULL}};
const Segment SGI386848[] = {{&S100343[136], 4}, {NULL}};
const Segment SGI386849[] = {{&S100343[63], 9}, {NULL}};
const Segment SGI386850[] = {{&S100606[28], 5}, {NULL}};
const Segment SGI386851[] = {{&S100658[12], 4}, {NULL}};
const Segment SGI386852[] = {{&S100658[136], 4}, {NULL}};
const Segment SGI386853[] = {{&S100658[63], 9}, {NULL}};
const Segment SGI386854[] = {{&S100721[12], 4}, {NULL}};
const Segment SGI386855[] = {{&S100787[12], 5}, {NULL}};
const Segment SGI386856[] = {{&S100787[92], 3}, {NULL}};
const Segment SGI386857[] = {{&S100802[12], 3}, {NULL}};
const Segment SGI386858[] = {{&S100802[117], 3}, {NULL}};
const Segment SGI386859[] = {{&S100810[12], 4}, {NULL}};
const Segment SGI386860[] = {{&S100810[90], 3}, {NULL}};
const Segment SGI386861[] = {{&S100879[28], 12}, {NULL}};
const Segment SGI386862[] = {{&S100879[41], 4}, {NULL}};
const Segment SGI386863[] = {{&S100879[54], 4}, {NULL}};
const Segment SGI386864[] = {{&S100963[12], 4}, {NULL}};
const Segment SGI386865[] = {{&S101006[12], 4}, {NULL}};
const Segment SGI386866[] = {{&S101006[90], 3}, {NULL}};
const Segment SGI386867[] = {{&S101045[12], 4}, {NULL}};
const Segment SGI386868[] = {{&S101045[90], 3}, {NULL}};
const Segment SGI386869[] = {{&S101049[12], 4}, {NULL}};
const Segment SGI386870[] = {{&S101049[90], 3}, {NULL}};
const Segment SGI386871[] = {{&S101105[94], 5}, {NULL}};
const Segment SGI386872[] = {{&S101105[54], 4}, {NULL}};
const Segment SGI386873[] = {{&S101105[0], 9}, {NULL}};
const Segment SGI386874[] = {{&S101106[12], 4}, {NULL}};
const Segment SGI386875[] = {{&S101106[134], 4}, {NULL}};
const Segment SGI386876[] = {{&S101106[62], 9}, {NULL}};
const Segment SGI386877[] = {{&S101140[12], 5}, {NULL}};
const Segment SGI386878[] = {{&S101140[40], 3}, {NULL}};
const Segment SGI386879[] = {{&S101145[21], 4}, {NULL}};
const Segment SGI386880[] = {{&S101145[54], 3}, {NULL}};
const Segment SGI386881[] = {{&S101160[12], 6}, {NULL}};
const Segment SGI386882[] = {{&S101161[12], 5}, {NULL}};
const Segment SGI386883[] = {{&S101275[120], 3}, {NULL}};
const Segment SGI386884[] = {{&S101275[77], 4}, {NULL}};
const Segment SGI386885[] = {{&S101298[12], 5}, {NULL}};
const Segment SGI386886[] = {{&S101303[12], 5}, {NULL}};
const Segment SGI386887[] = {{&S101303[44], 3}, {NULL}};
const Segment SGI386888[] = {{&S101314[57], 12}, {NULL}};
const Segment SGI386889[] = {{&S101420[12], 4}, {NULL}};
const Segment SGI386890[] = {{&S101421[12], 4}, {NULL}};
const Segment SGI386891[] = {{&S101422[12], 4}, {NULL}};
const Segment SGI386892[] = {{&S101423[12], 5}, {NULL}};
const Segment SGI386893[] = {{&S101425[12], 3}, {NULL}};
const Segment SGI386894[] = {{&S101447[12], 4}, {NULL}};
const Segment SGI386895[] = {{&S101457[12], 4}, {NULL}};
const Segment SGI386896[] = {{&S101457[39], 7}, {NULL}};
const Segment SGI386897[] = {{&S101483[12], 4}, {NULL}};
const Segment SGI386898[] = {{&S101608[12], 4}, {NULL}};
const Segment SGI386899[] = {{&S101616[12], 5}, {NULL}};
const Segment SGI386900[] = {{&S101616[40], 3}, {NULL}};
const Segment SGI386901[] = {{&S101628[12], 5}, {NULL}};
const Segment SGI386902[] = {{&S101628[36], 5}, {NULL}};
const Segment SGI386903[] = {{&S101662[12], 5}, {NULL}};
const Segment SGI386904[] = {{&S101662[40], 3}, {NULL}};
const Segment SGI386905[] = {{&S101784[12], 4}, {NULL}};
const Segment SGI386906[] = {{&S101784[157], 4}, {NULL}};
const Segment SGI386907[] = {{&S101887[12], 4}, {NULL}};
const Segment SGI386908[] = {{&S101887[118], 3}, {NULL}};
const Segment SGI386909[] = {{&S101986[12], 5}, {NULL}};
const Segment SGI386910[] = {{&S101986[40], 3}, {NULL}};
const Segment SGI386911[] = {{&S102017[12], 5}, {NULL}};
const Segment SGI386912[] = {{&S102017[36], 4}, {NULL}};
const Segment SGI386913[] = {{&S102031[12], 2}, {NULL}};
const Segment SGI386914[] = {{&S102031[25], 3}, {NULL}};
const Segment SGI386915[] = {{&S102150[12], 5}, {NULL}};
const Segment SGI386916[] = {{&S102160[12], 4}, {NULL}};
const Segment SGI386917[] = {{&S102184[12], 5}, {NULL}};
const Segment SGI386918[] = {{&S102328[12], 5}, {NULL}};
const Segment SGI386919[] = {{&S102328[40], 7}, {NULL}};
const Segment SGI386920[] = {{&S102353[12], 5}, {NULL}};
const Segment SGI386921[] = {{&S102353[43], 3}, {NULL}};
const Segment SGI386922[] = {{&S102365[12], 4}, {NULL}};
const Segment SGI386923[] = {{&S102416[12], 2}, {NULL}};
const Segment SGI386924[] = {{&S102416[25], 3}, {NULL}};
const Segment SGI386925[] = {{&S102439[12], 4}, {NULL}};
const Segment SGI386926[] = {{&S102439[90], 3}, {NULL}};
const Segment SGI386927[] = {{&S102441[12], 4}, {NULL}};
const Segment SGI386928[] = {{&S102441[90], 3}, {NULL}};
const Segment SGI386929[] = {{&S102550[12], 5}, {NULL}};
const Segment SGI386930[] = {{&S102551[25], 5}, {NULL}};
const Segment SGI386931[] = {{&S102557[12], 5}, {NULL}};
const Segment SGI386932[] = {{&S102566[12], 5}, {NULL}};
const Segment SGI386933[] = {{&S102566[35], 5}, {NULL}};
const Segment SGI386934[] = {{&S102567[12], 5}, {NULL}};
const Segment SGI386935[] = {{&S102567[35], 5}, {NULL}};
const Segment SGI386936[] = {{&S102587[12], 5}, {NULL}};
const Segment SGI386937[] = {{&S102587[40], 3}, {NULL}};
const Segment SGI386938[] = {{&S102590[12], 5}, {NULL}};
const Segment SGI386939[] = {{&S102595[12], 5}, {NULL}};
const Segment SGI386940[] = {{&S102595[36], 5}, {NULL}};
const Segment SGI386941[] = {{&S102596[12], 3}, {NULL}};
const Segment SGI386942[] = {{&S102731[12], 5}, {NULL}};
const Segment SGI386943[] = {{&S102731[40], 3}, {NULL}};
const Segment SGI386944[] = {{&S102766[12], 5}, {NULL}};
const Segment SGI386945[] = {{&S102766[81], 4}, {NULL}};
const Segment SGI386946[] = {{&S102767[12], 5}, {NULL}};
const Segment SGI386947[] = {{&S102767[81], 4}, {NULL}};
const Segment SGI386948[] = {{&S102799[12], 3}, {NULL}};
const Segment SGI386949[] = {{&S102802[12], 5}, {NULL}};
const Segment SGI386950[] = {{&S102987[12], 5}, {NULL}};
const Segment SGI386951[] = {{&S103015[12], 5}, {NULL}};
const Segment SGI386952[] = {{&S103021[12], 5}, {NULL}};
const Segment SGI386953[] = {{&S103026[12], 5}, {NULL}};
const Segment SGI386954[] = {{&S103026[36], 4}, {NULL}};
const Segment SGI386955[] = {{&S103040[12], 5}, {NULL}};
const Segment SGI386956[] = {{&S103040[35], 5}, {NULL}};
const Segment SGI386957[] = {{&S103121[42], 3}, {NULL}};
const Segment SGI386958[] = {{&S103121[56], 3}, {NULL}};
const Segment SGI386959[] = {{&S103198[12], 3}, {NULL}};
const Segment SGI386960[] = {{&S103198[116], 3}, {NULL}};
const Segment SGI386961[] = {{&S103255[12], 8}, {NULL}};
const Segment SGI386962[] = {{&S103255[43], 3}, {NULL}};
const Segment SGI386963[] = {{&S103269[12], 4}, {NULL}};
const Segment SGI386964[] = {{&S103273[12], 5}, {NULL}};
const Segment SGI386965[] = {{&S103273[41], 3}, {NULL}};
const Segment SGI386966[] = {{&S103325[12], 5}, {NULL}};
const Segment SGI386967[] = {{&S103525[65], 10}, {NULL}};
const Segment SGI386968[] = {{&S103534[12], 4}, {NULL}};
const Segment SGI386969[] = {{&S103553[12], 5}, {NULL}};
const Segment SGI386970[] = {{&S103570[24], 5}, {NULL}};
const Segment SGI386971[] = {{&S103576[12], 5}, {NULL}};
const Segment SGI386972[] = {{&S103578[12], 4}, {NULL}};
const Segment SGI386973[] = {{&S103604[12], 5}, {NULL}};
const Segment SGI386974[] = {{&S103604[40], 9}, {NULL}};
const Segment SGI386975[] = {{&S103651[12], 5}, {NULL}};
const Segment SGI386976[] = {{&S103652[12], 5}, {NULL}};
const Segment SGI386977[] = {{&S103652[36], 5}, {NULL}};
const Segment SGI386978[] = {{&S103654[12], 2}, {NULL}};
const Segment SGI386979[] = {{&S103654[25], 3}, {NULL}};
const Segment SGI386980[] = {{&S103852[12], 5}, {NULL}};
const Segment SGI386981[] = {{&S103878[12], 5}, {NULL}};
const Segment SGI386982[] = {{&S103878[40], 3}, {NULL}};
const Segment SGI386983[] = {{&S103898[12], 2}, {NULL}};
const Segment SGI386984[] = {{&S103898[37], 3}, {NULL}};
const Segment SGI386985[] = {{&S103901[12], 4}, {NULL}};
const Segment SGI386986[] = {{&S103901[39], 3}, {NULL}};
const Segment SGI386987[] = {{&S103903[12], 5}, {NULL}};
const Segment SGI386988[] = {{&S103913[12], 5}, {NULL}};
const Segment SGI386989[] = {{&S103913[36], 4}, {NULL}};
const Segment SGI386990[] = {{&S103941[59], 27}, {NULL}};
const Segment SGI386991[] = {{&S103941[77], 4}, {NULL}};
const Segment SGI386992[] = {{&S103953[12], 2}, {NULL}};
const Segment SGI386993[] = {{&S103953[25], 3}, {NULL}};
const Segment SGI386994[] = {{&S104168[12], 5}, {NULL}};
const Segment SGI386995[] = {{&S104209[12], 5}, {NULL}};
const Segment SGI386996[] = {{&S104215[12], 5}, {NULL}};
const Segment SGI386997[] = {{&S104243[12], 5}, {NULL}};
const Segment SGI386998[] = {{&S104276[12], 5}, {NULL}};
const Segment SGI386999[] = {{&S104276[91], 3}, {NULL}};
const Segment SGI387000[] = {{&S104294[12], 5}, {NULL}};
const Segment SGI387001[] = {{&S104294[35], 5}, {NULL}};
const Segment SGI387002[] = {{&S104295[12], 5}, {NULL}};
const Segment SGI387003[] = {{&S104295[40], 3}, {NULL}};
const Segment SGI387004[] = {{&S104296[12], 5}, {NULL}};
const Segment SGI387005[] = {{&S104297[12], 4}, {NULL}};
const Segment SGI387006[] = {{&S104582[12], 5}, {NULL}};
const Segment SGI387007[] = {{&S104582[40], 3}, {NULL}};
const Segment SGI387008[] = {{&S104589[12], 4}, {NULL}};
const Segment SGI387009[] = {{&S104589[39], 3}, {NULL}};
const Segment SGI387010[] = {{&S104600[12], 5}, {NULL}};
const Segment SGI387011[] = {{&S104600[40], 3}, {NULL}};
const Segment SGI387012[] = {{&S104615[24], 5}, {NULL}};
const Segment SGI387013[] = {{&S104631[12], 4}, {NULL}};
const Segment SGI387014[] = {{&S104631[39], 3}, {NULL}};
const Segment SGI387015[] = {{&S104640[12], 4}, {NULL}};
const Segment SGI387016[] = {{&S104655[12], 5}, {NULL}};
const Segment SGI387017[] = {{&S104671[12], 5}, {NULL}};
const Segment SGI387018[] = {{&S104697[12], 5}, {NULL}};
const Segment SGI387019[] = {{&S104697[91], 3}, {NULL}};
const Segment SGI387020[] = {{&S104721[12], 5}, {NULL}};
const Segment SGI387021[] = {{&S104723[12], 5}, {NULL}};
const Segment SGI387022[] = {{&S104732[12], 5}, {NULL}};
const Segment SGI387023[] = {{&S104733[12], 5}, {NULL}};
const Segment SGI387024[] = {{&S104734[12], 5}, {NULL}};
const Segment SGI387025[] = {{&S104735[12], 5}, {NULL}};
const Segment SGI387026[] = {{&S104765[12], 4}, {NULL}};
const Segment SGI387027[] = {{&S104766[12], 5}, {NULL}};
const Segment SGI387028[] = {{&S104767[12], 2}, {NULL}};
const Segment SGI387029[] = {{&S104767[25], 3}, {NULL}};
const Segment SGI387030[] = {{&S104811[12], 3}, {NULL}};
const Segment SGI387031[] = {{&S104811[26], 3}, {NULL}};
const Segment SGI387032[] = {{&S105105[12], 5}, {NULL}};
const Segment SGI387033[] = {{&S105105[81], 4}, {NULL}};
const Segment SGI387034[] = {{&S105129[12], 5}, {NULL}};
const Segment SGI387035[] = {{&S105129[40], 3}, {NULL}};
const Segment SGI387036[] = {{&S105150[12], 4}, {NULL}};
const Segment SGI387037[] = {{&S105150[34], 5}, {NULL}};
const Segment SGI387038[] = {{&S105151[12], 5}, {NULL}};
const Segment SGI387039[] = {{&S105151[60], 3}, {NULL}};
const Segment SGI387040[] = {{&S105153[12], 5}, {NULL}};
const Segment SGI387041[] = {{&S105153[40], 3}, {NULL}};
const Segment SGI387042[] = {{&S105171[12], 5}, {NULL}};
const Segment SGI387043[] = {{&S105182[12], 5}, {NULL}};
const Segment SGI387044[] = {{&S105182[40], 3}, {NULL}};
const Segment SGI387045[] = {{&S105198[12], 5}, {NULL}};
const Segment SGI387046[] = {{&S105204[12], 6}, {NULL}};
const Segment SGI387047[] = {{&S105205[12], 5}, {NULL}};
const Segment SGI387048[] = {{&S105229[12], 5}, {NULL}};
const Segment SGI387049[] = {{&S105250[12], 5}, {NULL}};
const Segment SGI387050[] = {{&S105253[12], 5}, {NULL}};
const Segment SGI387051[] = {{&S105253[39], 3}, {NULL}};
const Segment SGI387052[] = {{&S105259[12], 4}, {NULL}};
const Segment SGI387053[] = {{&S105261[24], 5}, {NULL}};
const Segment SGI387054[] = {{&S105265[12], 5}, {NULL}};
const Segment SGI387055[] = {{&S105267[25], 5}, {NULL}};
const Segment SGI387056[] = {{&S105280[12], 10}, {NULL}};
const Segment SGI387057[] = {{&S105281[24], 6}, {NULL}};
const Segment SGI387058[] = {{&S105290[78], 10}, {NULL}};
const Segment SGI387059[] = {{&S105295[12], 5}, {NULL}};
const Segment SGI387060[] = {{&S105341[12], 2}, {NULL}};
const Segment SGI387061[] = {{&S105341[25], 3}, {NULL}};
const Segment SGI387062[] = {{&S105344[12], 3}, {NULL}};
const Segment SGI387063[] = {{&S105344[87], 3}, {NULL}};
const Segment SGI387064[] = {{&S105379[12], 5}, {NULL}};
const Segment SGI387065[] = {{&S105396[59], 27}, {NULL}};
const Segment SGI387066[] = {{&S105396[77], 4}, {NULL}};
const Segment SGI387067[] = {{&S105397[59], 27}, {NULL}};
const Segment SGI387068[] = {{&S105397[77], 4}, {NULL}};
const Segment SGI387069[] = {{&S105410[12], 5}, {NULL}};
const Segment SGI387070[] = {{&S105410[81], 4}, {NULL}};
const Segment SGI387071[] = {{&S105433[12], 5}, {NULL}};
const Segment SGI387072[] = {{&S105433[40], 3}, {NULL}};
const Segment SGI387073[] = {{&S105445[12], 5}, {NULL}};
const Segment SGI387074[] = {{&S105446[12], 4}, {NULL}};
const Segment SGI387075[] = {{&S105446[89], 3}, {NULL}};
const Segment SGI387076[] = {{&S105448[12], 5}, {NULL}};
const Segment SGI387077[] = {{&S105448[91], 3}, {NULL}};
const Segment SGI387078[] = {{&S106007[13], 6}, {NULL}};
const Segment SGI387079[] = {{&S88414[14], 4}, {NULL}};
const Segment SGI387080[] = {{&S88414[0], 33}, {NULL}};
const Segment SGI387081[] = {{&S89402[14], 4}, {NULL}};
const Segment SGI387082[] = {{&S89402[0], 33}, {NULL}};
const Segment SGI387083[] = {{&S91488[14], 4}, {NULL}};
const Segment SGI387084[] = {{&S91488[0], 49}, {NULL}};
const Segment SGI387085[] = {{&S92842[14], 4}, {NULL}};
const Segment SGI387086[] = {{&S92842[0], 49}, {NULL}};
const Segment SGI387087[] = {{&S94208[14], 4}, {NULL}};
const Segment SGI387088[] = {{&S94208[0], 33}, {NULL}};
const Segment SGI387089[] = {{&S94416[14], 4}, {NULL}};
const Segment SGI387090[] = {{&S94416[0], 33}, {NULL}};
const Segment SGI387091[] = {{&S96616[14], 4}, {NULL}};
const Segment SGI387092[] = {{&S96616[0], 49}, {NULL}};
const Segment SGI387093[] = {{&S96617[14], 3}, {NULL}};
const Segment SGI387094[] = {{&S96617[0], 48}, {NULL}};
const Segment SGI387095[] = {{&S39338[13], 10}, {&S39338[24], 10}, {&S39338[35], 11}, {&S39338[47], 4}, {&S39338[52], 8}, {NULL}};
const Segment SGI387096[] = {{&S39339[13], 10}, {&S39339[24], 10}, {&S39339[35], 11}, {&S39339[47], 11}, {&S39339[59], 8}, {NULL}};
const Segment SGI387097[] = {{&S39340[13], 10}, {&S39340[24], 10}, {&S39340[35], 11}, {&S39340[47], 8}, {NULL}};
const Segment SGI387098[] = {{&S97664[14], 5}, {NULL}};
const Segment SGI387099[] = {{&S97664[0], 34}, {NULL}};
const Segment SGI387100[] = {{&S100108[14], 4}, {NULL}};
const Segment SGI387101[] = {{&S100108[0], 49}, {NULL}};
const Segment SGI387102[] = {{&S100158[14], 4}, {NULL}};
const Segment SGI387103[] = {{&S100158[0], 49}, {NULL}};
const Segment SGI387104[] = {{&S100758[14], 4}, {NULL}};
const Segment SGI387105[] = {{&S100758[0], 49}, {NULL}};
const Segment SGI387106[] = {{&S101487[14], 5}, {NULL}};
const Segment SGI387107[] = {{&S101487[0], 50}, {NULL}};
const Segment SGI387108[] = {{&S46007[13], 10}, {&S46007[24], 10}, {&S46007[35], 11}, {&S46007[47], 4}, {&S46007[52], 8}, {&S46007[61], 10}, {&S46007[72], 19}, {NULL}};
const Segment SGI387109[] = {{&S46008[13], 10}, {&S46008[24], 10}, {&S46008[35], 11}, {&S46008[47], 4}, {&S46008[52], 8}, {NULL}};
const Segment SGI387110[] = {{&S46009[13], 10}, {&S46009[24], 10}, {&S46009[35], 11}, {&S46009[47], 11}, {&S46009[59], 8}, {NULL}};
const Segment SGI387111[] = {{&S46010[13], 10}, {&S46010[24], 10}, {&S46010[35], 11}, {&S46010[47], 4}, {&S46010[52], 8}, {&S46010[61], 9}, {&S46010[71], 19}, {NULL}};
const Segment SGI387112[] = {{&S46011[13], 10}, {&S46011[24], 10}, {&S46011[35], 11}, {&S46011[47], 11}, {&S46011[59], 8}, {&S46011[68], 9}, {&S46011[78], 15}, {NULL}};
const Segment SGI387113[] = {{&S46013[13], 10}, {&S46013[24], 10}, {&S46013[35], 11}, {&S46013[47], 16}, {&S46013[64], 4}, {&S46013[69], 8}, {NULL}};
const Segment SGI387114[] = {{&S46014[13], 10}, {&S46014[24], 10}, {&S46014[35], 11}, {&S46014[47], 4}, {&S46014[52], 8}, {&S46014[61], 9}, {&S46014[71], 18}, {NULL}};
const Segment SGI387115[] = {{&S46015[13], 10}, {&S46015[24], 10}, {&S46015[35], 11}, {&S46015[47], 11}, {&S46015[59], 8}, {&S46015[68], 9}, {&S46015[78], 15}, {NULL}};
const Segment SGI387116[] = {{&S46016[13], 10}, {&S46016[24], 10}, {&S46016[35], 11}, {&S46016[47], 4}, {&S46016[52], 8}, {&S46016[61], 10}, {&S46016[72], 18}, {NULL}};
const Segment SGI387117[] = {{&S46017[13], 10}, {&S46017[24], 10}, {&S46017[35], 11}, {&S46017[47], 11}, {&S46017[59], 48}, {NULL}};
const Segment SGI387118[] = {{&S46018[13], 10}, {&S46018[24], 10}, {&S46018[35], 11}, {&S46018[47], 11}, {&S46018[59], 48}, {NULL}};
const Segment SGI387119[] = {{&S46019[13], 10}, {&S46019[24], 10}, {&S46019[35], 11}, {&S46019[47], 4}, {&S46019[52], 8}, {NULL}};
const Segment SGI387120[] = {{&S46020[13], 10}, {&S46020[24], 10}, {&S46020[35], 11}, {&S46020[47], 11}, {&S46020[59], 8}, {&S46020[47], 11}, {&S46020[80], 48}, {NULL}};
const Segment SGI387121[] = {{&S46021[13], 10}, {&S46021[24], 10}, {&S46021[35], 11}, {&S46021[47], 11}, {&S46021[59], 8}, {&S46021[68], 9}, {&S46021[78], 14}, {NULL}};
const Segment SGI387122[] = {{&S46022[13], 10}, {&S46022[24], 10}, {&S46022[35], 11}, {&S46022[47], 11}, {&S46022[59], 8}, {&S46022[68], 9}, {&S46022[78], 14}, {NULL}};
const Segment SGI387123[] = {{&S46023[13], 10}, {&S46023[24], 10}, {&S46023[35], 11}, {&S46023[47], 4}, {&S46023[52], 8}, {&S46023[61], 10}, {&S46023[72], 17}, {NULL}};
const Segment SGI387124[] = {{&S46024[13], 10}, {&S46024[24], 10}, {&S46024[35], 11}, {&S46024[47], 4}, {&S46024[52], 8}, {&S46024[61], 10}, {&S46024[72], 17}, {NULL}};
const Segment SGI387125[] = {{&S46025[13], 10}, {&S46025[24], 10}, {&S46025[35], 11}, {&S46025[47], 8}, {NULL}};
const Segment SGI387126[] = {{&S46026[13], 10}, {&S46026[24], 10}, {&S46026[35], 11}, {&S46026[47], 4}, {&S46026[52], 8}, {&S46026[61], 9}, {&S46026[71], 17}, {NULL}};
const Segment SGI387127[] = {{&S46027[13], 10}, {&S46027[24], 10}, {&S46027[35], 11}, {&S46027[47], 4}, {&S46027[52], 9}, {&S46027[62], 23}, {NULL}};
const Segment SGI387128[] = {{&S46028[13], 10}, {&S46028[24], 10}, {&S46028[35], 11}, {&S46028[47], 4}, {&S46028[52], 10}, {&S46028[63], 23}, {NULL}};
const Segment SGI387129[] = {{&S46029[13], 10}, {&S46029[24], 10}, {&S46029[35], 11}, {&S46029[47], 4}, {NULL}};
const Segment SGI387130[] = {{&S46030[13], 10}, {&S46030[24], 10}, {&S46030[35], 11}, {&S46030[47], 4}, {&S46030[52], 8}, {&S46030[61], 9}, {&S46030[71], 18}, {NULL}};
const Segment SGI387131[] = {{&S46031[13], 10}, {&S46031[24], 10}, {&S46031[35], 11}, {&S46031[47], 4}, {&S46031[52], 8}, {&S46031[61], 10}, {&S46031[72], 17}, {NULL}};
const Segment SGI387132[] = {{&S46032[13], 10}, {&S46032[24], 10}, {&S46032[35], 11}, {&S46032[47], 11}, {&S46032[59], 8}, {&S46032[68], 9}, {&S46032[78], 15}, {NULL}};
const Segment SGI387133[] = {{&S46033[13], 10}, {&S46033[24], 10}, {&S46033[35], 11}, {&S46033[47], 11}, {&S46033[59], 8}, {&S46033[68], 9}, {&S46033[78], 19}, {NULL}};
const Segment SGI387134[] = {{&S46034[13], 10}, {&S46034[24], 10}, {&S46034[35], 11}, {&S46034[47], 18}, {&S46034[66], 4}, {&S46034[71], 8}, {NULL}};
const Segment SGI387135[] = {{&S46035[13], 10}, {&S46035[24], 10}, {&S46035[35], 11}, {&S46035[47], 11}, {&S46035[59], 8}, {NULL}};
const Segment SGI387136[] = {{&S46036[13], 10}, {&S46036[24], 10}, {&S46036[35], 22}, {NULL}};
const Segment SGI387137[] = {{&S46037[13], 10}, {&S46037[24], 10}, {&S46037[35], 11}, {&S46037[47], 18}, {&S46037[66], 8}, {&S46037[75], 7}, {NULL}};
const Segment SGI387138[] = {{&S49568[13], 10}, {&S49568[24], 9}, {&S49568[34], 11}, {&S49568[46], 14}, {&S49568[61], 15}, {&S49568[77], 8}, {&S49568[86], 7}, {&S49568[94], 31}, {NULL}};
const Segment SGI387139[] = {{&S49569[13], 10}, {&S49569[24], 9}, {&S49569[34], 11}, {&S49569[46], 14}, {&S49569[61], 15}, {&S49569[77], 9}, {NULL}};
const Segment SGI387140[] = {{&S49571[13], 10}, {&S49571[24], 9}, {&S49571[34], 11}, {NULL}};
const Segment SGI387141[] = {{&S49572[13], 10}, {&S49572[24], 9}, {&S49572[34], 11}, {&S49572[46], 14}, {&S49572[61], 15}, {&S49572[77], 8}, {&S49572[86], 7}, {&S49572[94], 31}, {NULL}};
const Segment SGI387142[] = {{&S49573[13], 10}, {&S49573[24], 9}, {&S49573[34], 11}, {&S49573[46], 14}, {&S49573[61], 15}, {&S49573[77], 12}, {NULL}};
const Segment SGI387143[] = {{&S49574[13], 10}, {&S49574[24], 9}, {&S49574[34], 11}, {&S49574[46], 14}, {&S49574[61], 15}, {&S49574[77], 16}, {NULL}};
const Segment SGI387144[] = {{&S49575[13], 10}, {&S49575[24], 9}, {&S49575[34], 11}, {&S49575[46], 14}, {&S49575[61], 15}, {&S49575[77], 10}, {NULL}};
const Segment SGI387145[] = {{&S49576[13], 10}, {&S49576[24], 9}, {&S49576[34], 11}, {&S49576[46], 14}, {&S49576[61], 15}, {&S49576[77], 15}, {NULL}};
const Segment SGI387146[] = {{&S49577[13], 10}, {&S49577[24], 9}, {&S49577[34], 11}, {&S49577[46], 14}, {&S49577[61], 15}, {&S49577[77], 6}, {NULL}};
const Segment SGI387147[] = {{&S49578[13], 10}, {&S49578[24], 9}, {&S49578[34], 11}, {&S49578[46], 11}, {&S49578[58], 8}, {NULL}};
const Segment SGI387148[] = {{&S49579[13], 10}, {&S49579[24], 9}, {&S49579[34], 11}, {&S49579[46], 14}, {&S49579[61], 15}, {NULL}};
const Segment SGI387149[] = {{&S49580[13], 10}, {&S49580[24], 9}, {&S49580[34], 11}, {&S49580[46], 14}, {&S49580[61], 15}, {&S49580[77], 4}, {&S49580[82], 8}, {&S49580[91], 18}, {NULL}};
const Segment SGI387150[] = {{&S49581[13], 10}, {&S49581[24], 9}, {&S49581[34], 11}, {&S49581[46], 4}, {NULL}};
const Segment SGI387151[] = {{&S49582[13], 10}, {&S49582[24], 9}, {&S49582[34], 11}, {&S49582[46], 14}, {&S49582[61], 15}, {&S49582[77], 8}, {&S49582[86], 9}, {&S49582[96], 31}, {NULL}};
const Segment SGI387152[] = {{&S49583[13], 10}, {&S49583[24], 9}, {&S49583[34], 11}, {&S49583[46], 11}, {NULL}};
const Segment SGI387153[] = {{&S49584[13], 10}, {&S49584[24], 9}, {&S49584[34], 11}, {&S49584[46], 11}, {&S49584[58], 8}, {&S49584[46], 11}, {NULL}};
const Segment SGI387154[] = {{&S49585[13], 10}, {&S49585[24], 9}, {&S49585[34], 24}, {NULL}};
const Segment SGI387155[] = {{&S52590[13], 10}, {&S52590[24], 9}, {&S52590[34], 11}, {&S52590[46], 14}, {&S52590[61], 15}, {&S52590[77], 13}, {NULL}};
const Segment SGI387156[] = {{&S52593[13], 10}, {&S52593[24], 10}, {&S52593[35], 11}, {&S52593[47], 14}, {&S52593[62], 14}, {&S52593[77], 13}, {NULL}};
const Segment SGI387157[] = {{&S52601[13], 10}, {&S52601[24], 9}, {&S52601[34], 11}, {&S52601[46], 14}, {&S52601[61], 14}, {&S52601[76], 13}, {NULL}};
const Segment SGI387158[] = {{&S52690[13], 10}, {&S52690[24], 9}, {&S52690[34], 11}, {&S52690[46], 14}, {&S52690[61], 15}, {&S52690[77], 4}, {&S52690[82], 58}, {NULL}};
const Segment SGI387159[] = {{&S52691[13], 10}, {&S52691[24], 9}, {&S52691[34], 11}, {&S52691[46], 14}, {&S52691[61], 15}, {&S52691[77], 17}, {NULL}};
const Segment SGI387160[] = {{&S52692[13], 10}, {&S52692[24], 9}, {&S52692[34], 11}, {&S52692[46], 14}, {&S52692[61], 15}, {&S52692[77], 15}, {NULL}};
const Segment SGI387161[] = {{&S52693[13], 10}, {&S52693[24], 9}, {&S52693[34], 11}, {&S52693[46], 14}, {&S52693[61], 15}, {&S52693[77], 13}, {NULL}};
const Segment SGI387162[] = {{&S52695[13], 10}, {&S52695[24], 9}, {&S52695[34], 11}, {&S52695[46], 14}, {&S52695[61], 15}, {&S52695[77], 13}, {&S52695[91], 22}, {&S52695[114], 7}, {&S52695[122], 3}, {&S52695[126], 8}, {NULL}};
const Segment SGI387163[] = {{&S54151[13], 10}, {&S54151[24], 10}, {&S54151[35], 11}, {&S54151[47], 4}, {&S54151[52], 8}, {&S54151[61], 74}, {NULL}};
const Segment SGI387164[] = {{&S54152[13], 10}, {&S54152[24], 10}, {&S54152[35], 11}, {&S54152[47], 4}, {&S54152[52], 8}, {&S54152[61], 74}, {NULL}};
const Segment SGI387165[] = {{&S54153[13], 10}, {&S54153[24], 10}, {&S54153[35], 11}, {&S54153[47], 4}, {&S54153[52], 8}, {&S54153[61], 74}, {NULL}};
const Segment SGI387166[] = {{&S54201[13], 10}, {&S54201[24], 9}, {&S54201[34], 11}, {&S54201[46], 14}, {&S54201[61], 4}, {&S54201[66], 8}, {&S54201[75], 53}, {NULL}};
const Segment SGI387167[] = {{&S54202[13], 10}, {&S54202[24], 9}, {&S54202[34], 11}, {&S54202[46], 14}, {&S54202[61], 6}, {&S54202[68], 12}, {NULL}};
const Segment SGI387168[] = {{&S54203[13], 10}, {&S54203[24], 9}, {&S54203[34], 11}, {&S54203[46], 14}, {&S54203[61], 6}, {&S54203[68], 12}, {NULL}};
const Segment SGI387169[] = {{&S54204[13], 10}, {&S54204[24], 9}, {&S54204[34], 11}, {&S54204[46], 14}, {&S54204[61], 6}, {&S54204[68], 12}, {NULL}};
const Segment SGI387170[] = {{&S54205[13], 10}, {&S54205[24], 9}, {&S54205[34], 11}, {&S54205[46], 14}, {&S54205[61], 6}, {&S54205[68], 12}, {NULL}};
const Segment SGI387171[] = {{&S54206[13], 10}, {&S54206[24], 9}, {&S54206[34], 11}, {&S54206[46], 14}, {&S54206[61], 11}, {&S54206[73], 8}, {&S54206[82], 9}, {&S54206[92], 19}, {NULL}};
const Segment SGI387172[] = {{&S54207[13], 10}, {&S54207[24], 9}, {&S54207[34], 11}, {&S54207[46], 14}, {&S54207[61], 11}, {&S54207[73], 8}, {&S54207[82], 9}, {&S54207[92], 19}, {NULL}};
const Segment SGI387173[] = {{&S54208[13], 10}, {&S54208[24], 9}, {&S54208[34], 11}, {&S54208[46], 14}, {&S54208[61], 15}, {NULL}};
const Segment SGI387174[] = {{&S54209[13], 10}, {&S54209[24], 9}, {&S54209[34], 11}, {&S54209[46], 14}, {&S54209[61], 6}, {NULL}};
const Segment SGI387175[] = {{&S54210[13], 10}, {&S54210[24], 9}, {&S54210[34], 11}, {&S54210[46], 14}, {&S54210[61], 8}, {NULL}};
const Segment SGI387176[] = {{&S54213[13], 10}, {&S54213[24], 9}, {&S54213[34], 11}, {&S54213[46], 14}, {NULL}};
const Segment SGI387177[] = {{&S55456[13], 10}, {&S55456[24], 10}, {&S55456[35], 11}, {&S55456[47], 4}, {&S55456[52], 8}, {&S55456[61], 74}, {NULL}};
const Segment SGI387178[] = {{&S55458[13], 10}, {&S55458[24], 10}, {&S55458[35], 11}, {&S55458[47], 4}, {&S55458[52], 8}, {&S55458[61], 74}, {NULL}};
const Segment SGI387179[] = {{&S55715[13], 10}, {&S55715[24], 9}, {&S55715[34], 11}, {&S55715[46], 14}, {&S55715[61], 15}, {NULL}};
const Segment SGI387180[] = {{&S55716[13], 10}, {&S55716[24], 9}, {&S55716[34], 11}, {&S55716[46], 14}, {&S55716[61], 8}, {&S55716[70], 7}, {&S55716[78], 31}, {NULL}};
const Segment SGI387181[] = {{&S55717[13], 10}, {&S55717[24], 9}, {&S55717[34], 11}, {&S55717[46], 14}, {&S55717[61], 6}, {NULL}};
const Segment SGI387182[] = {{&S55719[13], 10}, {&S55719[24], 9}, {&S55719[34], 11}, {&S55719[46], 14}, {&S55719[61], 6}, {NULL}};
const Segment SGI387183[] = {{&S55720[13], 10}, {&S55720[24], 9}, {&S55720[34], 11}, {&S55720[46], 14}, {&S55720[61], 86}, {NULL}};
const Segment SGI387184[] = {{&S55722[13], 10}, {&S55722[24], 9}, {&S55722[34], 11}, {&S55722[46], 14}, {&S55722[61], 13}, {&S55722[75], 3}, {&S55722[79], 7}, {NULL}};
const Segment SGI387185[] = {{&S55723[13], 10}, {&S55723[24], 9}, {&S55723[34], 11}, {&S55723[46], 14}, {&S55723[61], 17}, {NULL}};
const Segment SGI387186[] = {{&S55724[13], 10}, {&S55724[24], 9}, {&S55724[34], 11}, {&S55724[46], 14}, {&S55724[61], 15}, {NULL}};
const Segment SGI387187[] = {{&S55725[13], 10}, {&S55725[24], 9}, {&S55725[34], 11}, {&S55725[46], 14}, {&S55725[61], 17}, {NULL}};
const Segment SGI387188[] = {{&S55726[13], 10}, {&S55726[24], 9}, {&S55726[34], 11}, {&S55726[46], 14}, {&S55726[61], 8}, {NULL}};
const Segment SGI387189[] = {{&S55727[13], 10}, {&S55727[24], 9}, {&S55727[34], 11}, {&S55727[46], 14}, {&S55727[61], 83}, {NULL}};
const Segment SGI387190[] = {{&S55728[13], 10}, {&S55728[24], 9}, {&S55728[34], 11}, {&S55728[46], 14}, {&S55728[61], 24}, {NULL}};
const Segment SGI387191[] = {{&S55729[13], 10}, {&S55729[24], 9}, {&S55729[34], 11}, {&S55729[46], 14}, {NULL}};
const Segment SGI387192[] = {{&S55730[13], 10}, {&S55730[24], 9}, {&S55730[34], 11}, {&S55730[46], 14}, {&S55730[61], 8}, {&S55730[70], 7}, {&S55730[78], 31}, {NULL}};
const Segment SGI387193[] = {{&S57799[13], 10}, {&S57799[24], 9}, {&S57799[34], 11}, {&S57799[46], 14}, {&S57799[61], 21}, {NULL}};
const Segment SGI387194[] = {{&S57801[13], 10}, {&S57801[24], 9}, {&S57801[34], 11}, {&S57801[46], 14}, {&S57801[61], 24}, {NULL}};
const Segment SGI387195[] = {{&S57802[13], 10}, {&S57802[24], 9}, {&S57802[34], 11}, {&S57802[46], 14}, {&S57802[61], 24}, {NULL}};
const Segment SGI387196[] = {{&S57803[13], 10}, {&S57803[24], 9}, {&S57803[34], 11}, {&S57803[46], 14}, {&S57803[61], 24}, {NULL}};
const Segment SGI387197[] = {{&S57804[13], 10}, {&S57804[24], 9}, {&S57804[34], 11}, {&S57804[46], 14}, {&S57804[61], 24}, {NULL}};
const Segment SGI387198[] = {{&S57805[13], 10}, {&S57805[24], 9}, {&S57805[34], 11}, {&S57805[46], 14}, {&S57805[61], 24}, {NULL}};
const Segment SGI387199[] = {{&S57806[13], 10}, {&S57806[24], 9}, {&S57806[34], 11}, {&S57806[46], 14}, {&S57806[61], 24}, {NULL}};
const Segment SGI387200[] = {{&S57807[13], 10}, {&S57807[24], 9}, {&S57807[34], 11}, {&S57807[46], 14}, {&S57807[61], 24}, {NULL}};
const Segment SGI387201[] = {{&S57808[13], 10}, {&S57808[24], 9}, {&S57808[34], 11}, {&S57808[46], 14}, {&S57808[61], 24}, {NULL}};
const Segment SGI387202[] = {{&S57809[13], 10}, {&S57809[24], 9}, {&S57809[34], 11}, {&S57809[46], 14}, {&S57809[61], 24}, {NULL}};
const Segment SGI387203[] = {{&S57810[13], 10}, {&S57810[24], 9}, {&S57810[34], 11}, {&S57810[46], 14}, {&S57810[61], 24}, {NULL}};
const Segment SGI387204[] = {{&S57811[13], 10}, {&S57811[24], 9}, {&S57811[34], 11}, {&S57811[46], 14}, {&S57811[61], 24}, {NULL}};
const Segment SGI387205[] = {{&S57812[13], 10}, {&S57812[24], 9}, {&S57812[34], 11}, {&S57812[46], 14}, {&S57812[61], 24}, {NULL}};
const Segment SGI387206[] = {{&S57813[13], 10}, {&S57813[24], 9}, {&S57813[34], 11}, {&S57813[46], 14}, {&S57813[61], 24}, {NULL}};
const Segment SGI387207[] = {{&S57814[13], 10}, {&S57814[24], 9}, {&S57814[34], 11}, {&S57814[46], 14}, {&S57814[61], 24}, {NULL}};
const Segment SGI387208[] = {{&S57815[13], 10}, {&S57815[24], 9}, {&S57815[34], 11}, {&S57815[46], 14}, {&S57815[61], 24}, {NULL}};
const Segment SGI387209[] = {{&S57816[13], 10}, {&S57816[24], 9}, {&S57816[34], 11}, {&S57816[46], 14}, {&S57816[61], 24}, {NULL}};
const Segment SGI387210[] = {{&S57817[13], 10}, {&S57817[24], 9}, {&S57817[34], 11}, {&S57817[46], 14}, {&S57817[61], 24}, {NULL}};
const Segment SGI387211[] = {{&S57819[13], 10}, {&S57819[24], 9}, {&S57819[34], 11}, {&S57819[46], 14}, {&S57819[61], 24}, {NULL}};
const Segment SGI387212[] = {{&S57820[13], 10}, {&S57820[24], 9}, {&S57820[34], 11}, {&S57820[46], 14}, {&S57820[61], 24}, {NULL}};
const Segment SGI387213[] = {{&S57821[13], 10}, {&S57821[24], 9}, {&S57821[34], 11}, {&S57821[46], 14}, {&S57821[61], 24}, {NULL}};
const Segment SGI387214[] = {{&S57822[13], 10}, {&S57822[24], 9}, {&S57822[34], 11}, {&S57822[46], 14}, {&S57822[61], 18}, {NULL}};
const Segment SGI387215[] = {{&S58170[13], 10}, {&S58170[24], 9}, {&S58170[34], 11}, {&S58170[46], 14}, {&S58170[61], 12}, {&S58170[74], 8}, {NULL}};
const Segment SGI387216[] = {{&S58171[13], 10}, {&S58171[24], 9}, {&S58171[34], 11}, {&S58171[46], 14}, {&S58171[61], 19}, {NULL}};
const Segment SGI387217[] = {{&S58172[13], 10}, {&S58172[24], 9}, {&S58172[34], 11}, {&S58172[46], 14}, {&S58172[61], 55}, {NULL}};
const Segment SGI387218[] = {{&S58174[13], 10}, {&S58174[24], 9}, {&S58174[34], 11}, {&S58174[46], 14}, {&S58174[61], 21}, {NULL}};
const Segment SGI387219[] = {{&S58175[13], 10}, {&S58175[24], 9}, {&S58175[34], 11}, {&S58175[46], 14}, {&S58175[61], 15}, {NULL}};
const Segment SGI387220[] = {{&S58176[13], 10}, {&S58176[24], 9}, {&S58176[34], 11}, {&S58176[46], 14}, {&S58176[61], 8}, {&S58176[70], 4}, {&S58176[75], 14}, {&S58176[90], 31}, {NULL}};
const Segment SGI387221[] = {{&S58177[13], 10}, {&S58177[24], 9}, {&S58177[34], 11}, {&S58177[46], 14}, {&S58177[61], 13}, {NULL}};
const Segment SGI387222[] = {{&S58179[13], 10}, {&S58179[24], 9}, {&S58179[34], 11}, {&S58179[46], 14}, {&S58179[61], 15}, {NULL}};
const Segment SGI387223[] = {{&S58180[13], 10}, {&S58180[24], 9}, {&S58180[34], 11}, {&S58180[46], 14}, {&S58180[61], 54}, {NULL}};
const Segment SGI387224[] = {{&S58181[13], 10}, {&S58181[24], 9}, {&S58181[34], 11}, {&S58181[46], 14}, {&S58181[61], 4}, {&S58181[66], 8}, {&S58181[75], 17}, {NULL}};
const Segment SGI387225[] = {{&S58183[13], 10}, {&S58183[24], 9}, {&S58183[34], 24}, {&S58183[59], 14}, {NULL}};
const Segment SGI387226[] = {{&S58952[13], 10}, {&S58952[24], 10}, {&S58952[35], 11}, {&S58952[47], 4}, {&S58952[52], 8}, {&S58952[61], 74}, {NULL}};
const Segment SGI387227[] = {{&S59063[13], 10}, {&S59063[24], 9}, {&S59063[34], 11}, {&S59063[46], 14}, {&S59063[61], 15}, {&S59063[77], 22}, {NULL}};
const Segment SGI387228[] = {{&S1559[8], 3}, {NULL}};
const Segment SGI387229[] = {{&S1559[13], 3}, {NULL}};
const Segment SGI387230[] = {{&S1559[76], 42}, {NULL}};
const Segment SGI387231[] = {{&S1827[8], 3}, {NULL}};
const Segment SGI387232[] = {{&S1827[13], 3}, {NULL}};
const Segment SGI387233[] = {{&S1827[76], 42}, {NULL}};
const Segment SGI387234[] = {{&S1999[8], 3}, {NULL}};
const Segment SGI387235[] = {{&S1999[13], 3}, {NULL}};
const Segment SGI387236[] = {{&S1999[76], 42}, {NULL}};
const Segment SGI387237[] = {{&S60503[13], 10}, {&S60503[24], 10}, {&S60503[35], 11}, {&S60503[47], 4}, {&S60503[52], 8}, {&S60503[61], 74}, {NULL}};
const Segment SGI387238[] = {{&S60505[13], 10}, {&S60505[24], 10}, {&S60505[35], 11}, {&S60505[47], 4}, {&S60505[52], 8}, {&S60505[61], 74}, {NULL}};
const Segment SGI387239[] = {{&S2101[8], 3}, {NULL}};
const Segment SGI387240[] = {{&S2101[13], 3}, {NULL}};
const Segment SGI387241[] = {{&S2101[76], 42}, {NULL}};
const Segment SGI387242[] = {{&S60969[13], 10}, {&S60969[24], 9}, {&S60969[34], 11}, {&S60969[46], 14}, {&S60969[61], 15}, {&S60969[77], 24}, {NULL}};
const Segment SGI387243[] = {{&S60970[13], 10}, {&S60970[24], 9}, {&S60970[34], 11}, {&S60970[46], 14}, {&S60970[61], 15}, {&S60970[77], 25}, {NULL}};
const Segment SGI387244[] = {{&S60972[13], 10}, {&S60972[24], 9}, {&S60972[34], 11}, {&S60972[46], 14}, {&S60972[61], 15}, {&S60972[77], 24}, {NULL}};
const Segment SGI387245[] = {{&S60974[13], 10}, {&S60974[24], 9}, {&S60974[34], 11}, {&S60974[46], 14}, {&S60974[61], 15}, {&S60974[77], 24}, {NULL}};
const Segment SGI387246[] = {{&S60975[13], 10}, {&S60975[24], 9}, {&S60975[34], 11}, {&S60975[46], 14}, {&S60975[61], 15}, {&S60975[77], 25}, {NULL}};
const Segment SGI387247[] = {{&S60978[13], 10}, {&S60978[24], 9}, {&S60978[34], 11}, {&S60978[46], 14}, {&S60978[61], 15}, {&S60978[77], 25}, {NULL}};
const Segment SGI387248[] = {{&S61198[13], 10}, {&S61198[24], 9}, {&S61198[34], 11}, {&S61198[46], 14}, {&S61198[61], 21}, {NULL}};
const Segment SGI387249[] = {{&S61199[13], 10}, {&S61199[24], 9}, {&S61199[34], 11}, {&S61199[46], 14}, {&S61199[61], 21}, {NULL}};
const Segment SGI387250[] = {{&S61200[13], 10}, {&S61200[24], 9}, {&S61200[34], 11}, {&S61200[46], 14}, {&S61200[61], 21}, {NULL}};
const Segment SGI387251[] = {{&S61240[13], 10}, {&S61240[24], 10}, {&S61240[35], 11}, {&S61240[47], 17}, {&S61240[65], 8}, {&S61240[60], 4}, {&S61240[65], 8}, {NULL}};
const Segment SGI387252[] = {{&S61242[13], 10}, {&S61242[24], 10}, {&S61242[35], 11}, {&S61242[47], 21}, {&S61242[69], 8}, {&S61242[78], 18}, {&S61242[69], 8}, {&S61242[64], 4}, {&S61242[69], 8}, {NULL}};
const Segment SGI387253[] = {{&S61243[13], 10}, {&S61243[24], 10}, {&S61243[35], 11}, {&S61243[47], 18}, {&S61243[66], 8}, {&S61243[61], 4}, {&S61243[80], 8}, {NULL}};
const Segment SGI387254[] = {{&S61244[13], 10}, {&S61244[24], 10}, {&S61244[35], 11}, {&S61244[47], 18}, {&S61244[66], 8}, {&S61244[61], 4}, {&S61244[66], 8}, {NULL}};
const Segment SGI387255[] = {{&S61245[13], 10}, {&S61245[24], 10}, {&S61245[35], 11}, {&S61245[47], 4}, {&S61245[52], 52}, {NULL}};
const Segment SGI387256[] = {{&S61246[13], 10}, {&S61246[24], 10}, {&S61246[35], 11}, {&S61246[47], 4}, {&S61246[52], 8}, {&S61246[61], 14}, {NULL}};
const Segment SGI387257[] = {{&S61247[26], 10}, {&S61247[37], 10}, {&S61247[48], 11}, {&S61247[60], 11}, {&S61247[72], 8}, {NULL}};
const Segment SGI387258[] = {{&S61249[13], 10}, {&S61249[24], 10}, {&S61249[35], 11}, {&S61249[47], 16}, {&S61249[64], 8}, {&S61249[47], 4}, {NULL}};
const Segment SGI387259[] = {{&S61250[13], 10}, {&S61250[24], 10}, {&S61250[35], 11}, {&S61250[47], 15}, {&S61250[63], 8}, {&S61250[58], 4}, {&S61250[63], 8}, {NULL}};
const Segment SGI387260[] = {{&S61251[13], 10}, {&S61251[24], 10}, {&S61251[35], 11}, {NULL}};
const Segment SGI387261[] = {{&S61252[13], 10}, {&S61252[24], 10}, {&S61252[35], 11}, {&S61252[47], 4}, {&S61252[52], 8}, {NULL}};
const Segment SGI387262[] = {{&S61253[13], 10}, {&S61253[24], 10}, {&S61253[35], 11}, {&S61253[47], 4}, {&S61253[52], 10}, {NULL}};
const Segment SGI387263[] = {{&S61254[13], 10}, {&S61254[24], 10}, {&S61254[35], 11}, {&S61254[47], 4}, {&S61254[52], 8}, {NULL}};
const Segment SGI387264[] = {{&S61255[13], 10}, {&S61255[24], 10}, {&S61255[35], 11}, {&S61255[47], 11}, {NULL}};
const Segment SGI387265[] = {{&S61256[17], 10}, {&S61256[28], 10}, {&S61256[39], 11}, {&S61256[11], 4}, {&S61256[56], 8}, {&S61256[65], 16}, {&S61256[11], 4}, {&S61256[56], 8}, {NULL}};
const Segment SGI387266[] = {{&S61257[13], 10}, {&S61257[24], 10}, {&S61257[35], 11}, {&S61257[47], 17}, {&S61257[65], 11}, {&S61257[77], 8}, {NULL}};
const Segment SGI387267[] = {{&S61258[13], 10}, {&S61258[24], 10}, {&S61258[35], 11}, {&S61258[47], 4}, {&S61258[52], 8}, {&S61258[47], 4}, {NULL}};
const Segment SGI387268[] = {{&S61259[12], 10}, {&S61259[23], 10}, {&S61259[34], 11}, {&S61259[46], 4}, {&S61259[51], 8}, {NULL}};
const Segment SGI387269[] = {{&S61260[13], 10}, {&S61260[24], 10}, {&S61260[35], 11}, {&S61260[47], 7}, {&S61260[55], 8}, {&S61260[64], 4}, {&S61260[69], 8}, {NULL}};
const Segment SGI387270[] = {{&S61544[13], 7}, {&S61544[21], 2}, {&S61544[24], 15}, {&S61544[40], 14}, {&S61544[55], 28}, {NULL}};
const Segment SGI387271[] = {{&S61545[13], 10}, {&S61545[24], 10}, {&S61545[35], 22}, {NULL}};
const Segment SGI387272[] = {{&S61546[13], 10}, {&S61546[24], 10}, {&S61546[35], 11}, {NULL}};
const Segment SGI387273[] = {{&S62361[13], 10}, {&S62361[24], 9}, {&S62361[34], 11}, {&S62361[46], 14}, {&S62361[61], 15}, {&S62361[77], 8}, {&S62361[86], 4}, {&S62361[91], 16}, {&S62361[108], 28}, {NULL}};
const Segment SGI387274[] = {{&S62362[13], 10}, {&S62362[24], 9}, {&S62362[34], 11}, {&S62362[46], 14}, {&S62362[61], 15}, {&S62362[77], 8}, {&S62362[86], 4}, {&S62362[91], 14}, {&S62362[106], 27}, {NULL}};
const Segment SGI387275[] = {{&S62505[13], 10}, {&S62505[24], 9}, {&S62505[34], 11}, {&S62505[46], 14}, {&S62505[61], 28}, {NULL}};
const Segment SGI387276[] = {{&S62506[13], 10}, {&S62506[24], 9}, {&S62506[34], 11}, {&S62506[46], 14}, {&S62506[61], 29}, {NULL}};
const Segment SGI387277[] = {{&S62508[13], 10}, {&S62508[24], 9}, {&S62508[34], 11}, {&S62508[46], 14}, {&S62508[61], 22}, {NULL}};
const Segment SGI387278[] = {{&S62509[13], 10}, {&S62509[24], 9}, {&S62509[34], 11}, {&S62509[46], 14}, {&S62509[61], 28}, {NULL}};
const Segment SGI387279[] = {{&S62511[13], 10}, {&S62511[24], 9}, {&S62511[34], 11}, {&S62511[46], 14}, {&S62511[61], 28}, {NULL}};
const Segment SGI387280[] = {{&S62690[13], 10}, {&S62690[24], 9}, {&S62690[34], 11}, {&S62690[46], 14}, {&S62690[61], 15}, {&S62690[77], 19}, {NULL}};
const Segment SGI387281[] = {{&S62877[13], 10}, {&S62877[24], 9}, {&S62877[34], 11}, {&S62877[46], 14}, {&S62877[61], 15}, {&S62877[77], 17}, {NULL}};
const Segment SGI387282[] = {{&S62878[13], 10}, {&S62878[24], 9}, {&S62878[34], 11}, {&S62878[46], 14}, {&S62878[61], 15}, {&S62878[77], 18}, {NULL}};
const Segment SGI387283[] = {{&S63153[13], 10}, {&S63153[24], 10}, {&S63153[35], 11}, {&S63153[47], 4}, {&S63153[52], 8}, {&S63153[61], 74}, {NULL}};
const Segment SGI387284[] = {{&S64185[13], 10}, {&S64185[24], 9}, {&S64185[34], 11}, {&S64185[46], 13}, {&S64185[60], 13}, {NULL}};
const Segment SGI387285[] = {{&S64186[13], 10}, {&S64186[24], 9}, {&S64186[34], 11}, {&S64186[46], 13}, {&S64186[60], 15}, {NULL}};
const Segment SGI387286[] = {{&S64187[13], 10}, {&S64187[24], 9}, {&S64187[34], 11}, {&S64187[46], 13}, {&S64187[60], 11}, {&S64187[72], 8}, {&S64187[81], 49}, {NULL}};
const Segment SGI387287[] = {{&S64188[13], 10}, {&S64188[24], 9}, {&S64188[34], 11}, {&S64188[46], 13}, {&S64188[60], 29}, {NULL}};
const Segment SGI387288[] = {{&S64189[13], 10}, {&S64189[24], 9}, {&S64189[34], 11}, {&S64189[46], 13}, {&S64189[60], 11}, {&S64189[72], 8}, {&S64189[81], 9}, {&S64189[91], 19}, {NULL}};
const Segment SGI387289[] = {{&S64191[13], 10}, {&S64191[24], 9}, {&S64191[34], 11}, {&S64191[46], 13}, {&S64191[60], 24}, {NULL}};
const Segment SGI387290[] = {{&S64193[13], 10}, {&S64193[24], 9}, {&S64193[34], 11}, {&S64193[46], 13}, {NULL}};
const Segment SGI387291[] = {{&S64582[13], 10}, {&S64582[24], 9}, {&S64582[34], 11}, {&S64582[46], 14}, {&S64582[61], 15}, {&S64582[77], 29}, {NULL}};
const Segment SGI387292[] = {{&S5468[8], 3}, {NULL}};
const Segment SGI387293[] = {{&S5468[13], 3}, {NULL}};
const Segment SGI387294[] = {{&S5468[77], 22}, {NULL}};
const Segment SGI387295[] = {{&S5469[8], 3}, {NULL}};
const Segment SGI387296[] = {{&S5469[13], 3}, {NULL}};
const Segment SGI387297[] = {{&S5469[81], 13}, {NULL}};
const Segment SGI387298[] = {{&S5481[8], 3}, {NULL}};
const Segment SGI387299[] = {{&S5481[13], 3}, {NULL}};
const Segment SGI387300[] = {{&S5481[77], 13}, {NULL}};
const Segment SGI387301[] = {{&S5494[8], 3}, {NULL}};
const Segment SGI387302[] = {{&S5494[13], 3}, {NULL}};
const Segment SGI387303[] = {{&S5494[75], 13}, {NULL}};
const Segment SGI387304[] = {{&S5499[8], 3}, {NULL}};
const Segment SGI387305[] = {{&S5499[13], 3}, {NULL}};
const Segment SGI387306[] = {{&S5499[81], 13}, {NULL}};
const Segment SGI387307[] = {{&S5511[8], 3}, {NULL}};
const Segment SGI387308[] = {{&S5511[13], 3}, {NULL}};
const Segment SGI387309[] = {{&S5551[8], 3}, {NULL}};
const Segment SGI387310[] = {{&S5551[13], 9}, {NULL}};
const Segment SGI387311[] = {{&S5553[8], 3}, {NULL}};
const Segment SGI387312[] = {{&S5553[13], 9}, {NULL}};
const Segment SGI387313[] = {{&S5553[84], 13}, {NULL}};
const Segment SGI387314[] = {{&S5554[8], 3}, {NULL}};
const Segment SGI387315[] = {{&S5554[13], 9}, {NULL}};
const Segment SGI387316[] = {{&S5554[80], 16}, {NULL}};
const Segment SGI387317[] = {{&S5555[8], 3}, {NULL}};
const Segment SGI387318[] = {{&S5555[13], 9}, {NULL}};
const Segment SGI387319[] = {{&S5555[84], 15}, {NULL}};
const Segment SGI387320[] = {{&S5557[8], 3}, {NULL}};
const Segment SGI387321[] = {{&S5557[13], 9}, {NULL}};
const Segment SGI387322[] = {{&S5557[87], 13}, {NULL}};
const Segment SGI387323[] = {{&S5559[8], 3}, {NULL}};
const Segment SGI387324[] = {{&S5559[13], 9}, {NULL}};
const Segment SGI387325[] = {{&S5559[83], 12}, {NULL}};
const Segment SGI387326[] = {{&S5563[8], 3}, {NULL}};
const Segment SGI387327[] = {{&S5563[13], 9}, {NULL}};
const Segment SGI387328[] = {{&S5563[84], 17}, {NULL}};
const Segment SGI387329[] = {{&S5565[8], 3}, {NULL}};
const Segment SGI387330[] = {{&S5565[13], 9}, {NULL}};
const Segment SGI387331[] = {{&S5565[84], 15}, {NULL}};
const Segment SGI387332[] = {{&S5566[8], 3}, {NULL}};
const Segment SGI387333[] = {{&S5566[13], 9}, {NULL}};
const Segment SGI387334[] = {{&S5566[84], 13}, {NULL}};
const Segment SGI387335[] = {{&S5572[8], 3}, {NULL}};
const Segment SGI387336[] = {{&S5572[13], 9}, {NULL}};
const Segment SGI387337[] = {{&S5572[85], 15}, {NULL}};
const Segment SGI387338[] = {{&S5574[8], 3}, {NULL}};
const Segment SGI387339[] = {{&S5574[13], 9}, {NULL}};
const Segment SGI387340[] = {{&S5574[84], 15}, {NULL}};
const Segment SGI387341[] = {{&S5576[8], 3}, {NULL}};
const Segment SGI387342[] = {{&S5576[13], 9}, {NULL}};
const Segment SGI387343[] = {{&S5576[87], 13}, {NULL}};
const Segment SGI387344[] = {{&S5578[8], 3}, {NULL}};
const Segment SGI387345[] = {{&S5578[13], 9}, {NULL}};
const Segment SGI387346[] = {{&S5578[87], 13}, {NULL}};
const Segment SGI387347[] = {{&S5580[8], 3}, {NULL}};
const Segment SGI387348[] = {{&S5580[13], 9}, {NULL}};
const Segment SGI387349[] = {{&S5580[83], 12}, {NULL}};
const Segment SGI387350[] = {{&S5586[8], 3}, {NULL}};
const Segment SGI387351[] = {{&S5586[13], 9}, {NULL}};
const Segment SGI387352[] = {{&S5586[83], 15}, {NULL}};
const Segment SGI387353[] = {{&S5589[8], 3}, {NULL}};
const Segment SGI387354[] = {{&S5589[13], 9}, {NULL}};
const Segment SGI387355[] = {{&S5589[87], 13}, {NULL}};
const Segment SGI387356[] = {{&S5592[8], 3}, {NULL}};
const Segment SGI387357[] = {{&S5592[13], 9}, {NULL}};
const Segment SGI387358[] = {{&S5592[87], 21}, {NULL}};
const Segment SGI387359[] = {{&S5593[8], 3}, {NULL}};
const Segment SGI387360[] = {{&S5593[13], 9}, {NULL}};
const Segment SGI387361[] = {{&S5593[87], 13}, {NULL}};
const Segment SGI387362[] = {{&S5596[8], 3}, {NULL}};
const Segment SGI387363[] = {{&S5596[13], 9}, {NULL}};
const Segment SGI387364[] = {{&S5596[82], 13}, {NULL}};
const Segment SGI387365[] = {{&S5597[8], 3}, {NULL}};
const Segment SGI387366[] = {{&S5597[13], 9}, {NULL}};
const Segment SGI387367[] = {{&S5597[82], 16}, {NULL}};
const Segment SGI387368[] = {{&S5601[8], 3}, {NULL}};
const Segment SGI387369[] = {{&S5601[13], 9}, {NULL}};
const Segment SGI387370[] = {{&S5601[82], 37}, {NULL}};
const Segment SGI387371[] = {{&S5602[8], 3}, {NULL}};
const Segment SGI387372[] = {{&S5602[13], 9}, {NULL}};
const Segment SGI387373[] = {{&S5602[87], 13}, {NULL}};
const Segment SGI387374[] = {{&S5609[8], 3}, {NULL}};
const Segment SGI387375[] = {{&S5609[13], 9}, {NULL}};
const Segment SGI387376[] = {{&S5609[87], 13}, {NULL}};
const Segment SGI387377[] = {{&S5613[8], 3}, {NULL}};
const Segment SGI387378[] = {{&S5613[13], 9}, {NULL}};
const Segment SGI387379[] = {{&S5613[87], 13}, {NULL}};
const Segment SGI387380[] = {{&S65550[13], 10}, {&S65550[24], 10}, {&S65550[35], 11}, {&S65550[47], 4}, {&S65550[52], 8}, {&S65550[61], 74}, {NULL}};
const Segment SGI387381[] = {{&S65650[13], 10}, {&S65650[24], 10}, {&S65650[35], 11}, {&S65650[47], 4}, {&S65650[52], 8}, {&S65650[61], 19}, {NULL}};
const Segment SGI387382[] = {{&S65655[13], 10}, {&S65655[24], 10}, {&S65655[35], 11}, {&S65655[47], 18}, {&S65655[66], 19}, {NULL}};
const Segment SGI387383[] = {{&S65657[13], 10}, {&S65657[24], 10}, {&S65657[35], 11}, {&S65657[47], 4}, {&S65657[52], 26}, {NULL}};
const Segment SGI387384[] = {{&S66520[13], 10}, {&S66520[24], 10}, {&S66520[35], 11}, {NULL}};
const Segment SGI387385[] = {{&S66757[13], 10}, {&S66757[24], 9}, {&S66757[34], 11}, {&S66757[46], 14}, {&S66757[61], 15}, {&S66757[77], 21}, {NULL}};
const Segment SGI387386[] = {{&S66758[13], 10}, {&S66758[24], 9}, {&S66758[34], 11}, {&S66758[46], 14}, {&S66758[61], 15}, {&S66758[77], 15}, {NULL}};
const Segment SGI387387[] = {{&S66759[13], 10}, {&S66759[24], 9}, {&S66759[34], 11}, {&S66759[46], 14}, {&S66759[61], 15}, {&S66759[77], 16}, {NULL}};
const Segment SGI387388[] = {{&S66760[13], 10}, {&S66760[24], 9}, {&S66760[34], 11}, {&S66760[46], 14}, {&S66760[61], 15}, {&S66760[77], 16}, {NULL}};
const Segment SGI387389[] = {{&S66762[13], 10}, {&S66762[24], 9}, {&S66762[34], 11}, {&S66762[46], 14}, {&S66762[61], 15}, {&S66762[77], 22}, {NULL}};
const Segment SGI387390[] = {{&S4813[8], 3}, {NULL}};
const Segment SGI387391[] = {{&S4813[13], 3}, {NULL}};
const Segment SGI387392[] = {{&S5016[8], 3}, {NULL}};
const Segment SGI387393[] = {{&S5016[13], 14}, {NULL}};
const Segment SGI387394[] = {{&S5270[8], 3}, {NULL}};
const Segment SGI387395[] = {{&S5270[13], 14}, {NULL}};
const Segment SGI387396[] = {{&S5493[8], 3}, {NULL}};
const Segment SGI387397[] = {{&S5493[26], 3}, {&S5493[31], 10}, {NULL}};
const Segment SGI387398[] = {{&S5507[8], 3}, {NULL}};
const Segment SGI387399[] = {{&S5507[13], 3}, {NULL}};
const Segment SGI387400[] = {{&S5508[8], 3}, {NULL}};
const Segment SGI387401[] = {{&S5508[13], 3}, {NULL}};
const Segment SGI387402[] = {{&S5525[8], 3}, {NULL}};
const Segment SGI387403[] = {{&S5525[13], 3}, {NULL}};
const Segment SGI387404[] = {{&S5525[13], 3}, {&S5525[21], 10}, {NULL}};
const Segment SGI387405[] = {{&S5582[8], 3}, {NULL}};
const Segment SGI387406[] = {{&S5582[13], 9}, {NULL}};
const Segment SGI387407[] = {{&S5583[8], 3}, {NULL}};
const Segment SGI387408[] = {{&S5583[13], 9}, {NULL}};
const Segment SGI387409[] = {{&S5583[31], 15}, {NULL}};
const Segment SGI387410[] = {{&S68256[13], 10}, {&S68256[24], 9}, {&S68256[34], 11}, {&S68256[46], 13}, {&S68256[60], 8}, {&S68256[69], 7}, {&S68256[77], 31}, {NULL}};
const Segment SGI387411[] = {{&S68257[13], 10}, {&S68257[24], 9}, {&S68257[34], 11}, {&S68257[46], 13}, {&S68257[60], 6}, {NULL}};
const Segment SGI387412[] = {{&S68259[13], 10}, {&S68259[24], 9}, {&S68259[34], 11}, {&S68259[46], 13}, {NULL}};
const Segment SGI387413[] = {{&S68281[13], 10}, {&S68281[24], 9}, {&S68281[34], 11}, {&S68281[46], 14}, {&S68281[61], 15}, {&S68281[77], 22}, {NULL}};
const Segment SGI387414[] = {{&S68283[13], 10}, {&S68283[24], 9}, {&S68283[34], 11}, {&S68283[46], 14}, {&S68283[61], 15}, {&S68283[77], 20}, {NULL}};
const Segment SGI387415[] = {{&S68285[13], 10}, {&S68285[24], 9}, {&S68285[34], 11}, {&S68285[46], 14}, {&S68285[61], 15}, {&S68285[77], 20}, {NULL}};
const Segment SGI387416[] = {{&S68286[13], 10}, {&S68286[24], 9}, {&S68286[34], 11}, {&S68286[46], 14}, {&S68286[61], 15}, {&S68286[77], 23}, {NULL}};
const Segment SGI387417[] = {{&S68487[13], 10}, {&S68487[24], 10}, {&S68487[35], 11}, {&S68487[47], 4}, {&S68487[52], 19}, {NULL}};
const Segment SGI387418[] = {{&S68491[13], 10}, {&S68491[24], 10}, {&S68491[35], 11}, {&S68491[47], 4}, {&S68491[52], 8}, {&S68491[61], 19}, {NULL}};
const Segment SGI387419[] = {{&S71214[13], 10}, {&S71214[24], 9}, {&S71214[34], 11}, {&S71214[46], 13}, {&S71214[60], 21}, {NULL}};
const Segment SGI387420[] = {{&S71616[13], 10}, {&S71616[24], 9}, {&S71616[34], 11}, {&S71616[46], 13}, {&S71616[60], 4}, {&S71616[65], 8}, {&S71616[74], 9}, {&S71616[84], 19}, {NULL}};
const Segment SGI387421[] = {{&S71617[13], 10}, {&S71617[24], 9}, {&S71617[34], 11}, {&S71617[46], 13}, {&S71617[60], 4}, {&S71617[65], 8}, {&S71617[74], 9}, {&S71617[84], 19}, {NULL}};
const Segment SGI387422[] = {{&S71618[13], 10}, {&S71618[24], 9}, {&S71618[34], 11}, {&S71618[46], 13}, {&S71618[60], 4}, {&S71618[65], 8}, {&S71618[74], 9}, {&S71618[84], 19}, {NULL}};
const Segment SGI387423[] = {{&S71619[13], 10}, {&S71619[24], 9}, {&S71619[34], 11}, {&S71619[46], 13}, {&S71619[60], 4}, {&S71619[65], 8}, {&S71619[74], 53}, {NULL}};
const Segment SGI387424[] = {{&S71620[13], 10}, {&S71620[24], 9}, {&S71620[34], 11}, {&S71620[46], 13}, {NULL}};
const Segment SGI387425[] = {{&S71621[13], 10}, {&S71621[24], 9}, {&S71621[34], 11}, {&S71621[46], 13}, {&S71621[60], 4}, {&S71621[65], 8}, {&S71621[74], 9}, {&S71621[84], 19}, {NULL}};
const Segment SGI387426[] = {{&S71622[13], 10}, {&S71622[24], 9}, {&S71622[34], 11}, {&S71622[46], 13}, {&S71622[60], 4}, {&S71622[65], 8}, {&S71622[74], 9}, {&S71622[84], 19}, {NULL}};
const Segment SGI387427[] = {{&S71623[13], 10}, {&S71623[24], 9}, {&S71623[34], 11}, {&S71623[46], 13}, {&S71623[60], 4}, {&S71623[65], 8}, {&S71623[74], 9}, {&S71623[84], 19}, {NULL}};
const Segment SGI387428[] = {{&S71624[13], 10}, {&S71624[24], 9}, {&S71624[34], 11}, {&S71624[46], 13}, {&S71624[60], 11}, {&S71624[72], 8}, {&S71624[81], 49}, {NULL}};
const Segment SGI387429[] = {{&S71625[13], 10}, {&S71625[24], 9}, {&S71625[34], 11}, {&S71625[46], 13}, {&S71625[60], 5}, {NULL}};
const Segment SGI387430[] = {{&S71626[13], 10}, {&S71626[24], 9}, {&S71626[34], 11}, {&S71626[46], 13}, {&S71626[60], 27}, {NULL}};
const Segment SGI387431[] = {{&S71668[13], 10}, {&S71668[24], 9}, {&S71668[34], 11}, {&S71668[46], 4}, {&S71668[51], 8}, {&S71668[60], 19}, {NULL}};
const Segment SGI387432[] = {{&S71669[13], 10}, {&S71669[24], 9}, {&S71669[34], 11}, {&S71669[46], 14}, {&S71669[61], 15}, {&S71669[77], 25}, {NULL}};
const Segment SGI387433[] = {{&S9192[8], 3}, {NULL}};
const Segment SGI387434[] = {{&S9192[13], 7}, {NULL}};
const Segment SGI387435[] = {{&S9192[25], 14}, {NULL}};
const Segment SGI387436[] = {{&S9192[66], 16}, {NULL}};
const Segment SGI387437[] = {{&S73903[13], 10}, {&S73903[24], 9}, {&S73903[34], 11}, {&S73903[46], 13}, {&S73903[60], 17}, {NULL}};
const Segment SGI387438[] = {{&S73904[13], 10}, {&S73904[24], 9}, {&S73904[34], 11}, {&S73904[46], 13}, {&S73904[60], 17}, {NULL}};
const Segment SGI387439[] = {{&S73905[13], 10}, {&S73905[24], 9}, {&S73905[34], 11}, {&S73905[46], 13}, {&S73905[60], 17}, {NULL}};
const Segment SGI387440[] = {{&S73906[13], 10}, {&S73906[24], 9}, {&S73906[34], 11}, {&S73906[46], 13}, {&S73906[60], 17}, {NULL}};
const Segment SGI387441[] = {{&S73907[13], 10}, {&S73907[24], 9}, {&S73907[34], 11}, {&S73907[46], 13}, {&S73907[60], 8}, {&S73907[69], 4}, {&S73907[74], 13}, {&S73907[88], 28}, {NULL}};
const Segment SGI387442[] = {{&S73970[13], 10}, {&S73970[24], 9}, {&S73970[34], 11}, {&S73970[46], 14}, {&S73970[61], 8}, {&S73970[70], 4}, {&S73970[75], 13}, {&S73970[89], 28}, {NULL}};
const Segment SGI387443[] = {{&S73974[13], 10}, {&S73974[24], 10}, {&S73974[35], 11}, {&S73974[47], 8}, {&S73974[56], 8}, {NULL}};
const Segment SGI387444[] = {{&S74266[13], 10}, {&S74266[24], 9}, {&S74266[34], 11}, {&S74266[46], 14}, {&S74266[61], 14}, {NULL}};
const Segment SGI387445[] = {{&S75269[13], 10}, {&S75269[24], 9}, {&S75269[34], 11}, {&S75269[46], 4}, {&S75269[51], 8}, {&S75269[60], 16}, {NULL}};
const Segment SGI387446[] = {{&S75270[13], 10}, {&S75270[24], 9}, {&S75270[34], 11}, {&S75270[46], 4}, {&S75270[51], 8}, {&S75270[60], 16}, {NULL}};
const Segment SGI387447[] = {{&S75271[13], 10}, {&S75271[24], 9}, {&S75271[34], 11}, {&S75271[46], 4}, {&S75271[51], 8}, {&S75271[60], 16}, {NULL}};
const Segment SGI387448[] = {{&S75272[13], 10}, {&S75272[24], 9}, {&S75272[34], 11}, {&S75272[46], 4}, {&S75272[51], 8}, {&S75272[60], 16}, {NULL}};
const Segment SGI387449[] = {{&S75273[13], 10}, {&S75273[24], 9}, {&S75273[34], 11}, {&S75273[46], 4}, {&S75273[51], 8}, {&S75273[60], 19}, {NULL}};
const Segment SGI387450[] = {{&S75274[13], 10}, {&S75274[24], 9}, {&S75274[34], 11}, {&S75274[46], 4}, {&S75274[51], 8}, {&S75274[60], 16}, {NULL}};
const Segment SGI387451[] = {{&S75275[13], 10}, {&S75275[24], 9}, {&S75275[34], 11}, {&S75275[46], 16}, {&S75275[63], 16}, {NULL}};
const Segment SGI387452[] = {{&S75276[13], 10}, {&S75276[24], 9}, {&S75276[34], 11}, {&S75276[46], 4}, {&S75276[51], 8}, {&S75276[60], 15}, {NULL}};
const Segment SGI387453[] = {{&S75322[13], 10}, {&S75322[24], 10}, {&S75322[35], 11}, {&S75322[47], 4}, {&S75322[52], 19}, {NULL}};
const Segment SGI387454[] = {{&S75394[127], 10}, {&S75394[138], 9}, {&S75394[148], 11}, {&S75394[160], 14}, {&S75394[175], 15}, {NULL}};
const Segment SGI387455[] = {{&S75395[13], 10}, {&S75395[24], 9}, {&S75395[34], 11}, {&S75395[46], 13}, {&S75395[60], 14}, {NULL}};
const Segment SGI387456[] = {{&S75396[13], 10}, {&S75396[24], 9}, {&S75396[34], 11}, {&S75396[46], 13}, {&S75396[60], 14}, {NULL}};
const Segment SGI387457[] = {{&S75397[13], 10}, {&S75397[24], 9}, {&S75397[34], 11}, {&S75397[46], 13}, {&S75397[60], 15}, {NULL}};
const Segment SGI387458[] = {{&S75398[13], 10}, {&S75398[24], 9}, {&S75398[34], 11}, {&S75398[46], 13}, {&S75398[60], 15}, {NULL}};
const Segment SGI387459[] = {{&S15671[8], 3}, {NULL}};
const Segment SGI387460[] = {{&S15671[13], 9}, {NULL}};
const Segment SGI387461[] = {{&S15671[83], 16}, {NULL}};
const Segment SGI387462[] = {{&S15672[8], 3}, {NULL}};
const Segment SGI387463[] = {{&S15672[13], 9}, {NULL}};
const Segment SGI387464[] = {{&S15672[88], 14}, {NULL}};
const Segment SGI387465[] = {{&S15673[8], 3}, {NULL}};
const Segment SGI387466[] = {{&S15673[13], 9}, {NULL}};
const Segment SGI387467[] = {{&S15673[83], 16}, {NULL}};
const Segment SGI387468[] = {{&S15675[8], 3}, {NULL}};
const Segment SGI387469[] = {{&S15675[13], 9}, {NULL}};
const Segment SGI387470[] = {{&S15675[88], 13}, {NULL}};
const Segment SGI387471[] = {{&S15676[8], 3}, {NULL}};
const Segment SGI387472[] = {{&S15676[13], 9}, {NULL}};
const Segment SGI387473[] = {{&S15676[88], 13}, {NULL}};
const Segment SGI387474[] = {{&S15677[8], 3}, {NULL}};
const Segment SGI387475[] = {{&S15677[13], 9}, {NULL}};
const Segment SGI387476[] = {{&S15677[88], 13}, {NULL}};
const Segment SGI387477[] = {{&S15678[8], 3}, {NULL}};
const Segment SGI387478[] = {{&S15678[13], 9}, {NULL}};
const Segment SGI387479[] = {{&S15678[88], 13}, {NULL}};
const Segment SGI387480[] = {{&S15679[8], 3}, {NULL}};
const Segment SGI387481[] = {{&S15679[13], 9}, {NULL}};
const Segment SGI387482[] = {{&S15679[88], 16}, {NULL}};
const Segment SGI387483[] = {{&S15681[8], 3}, {NULL}};
const Segment SGI387484[] = {{&S15681[13], 9}, {NULL}};
const Segment SGI387485[] = {{&S15681[88], 13}, {NULL}};
const Segment SGI387486[] = {{&S15682[8], 3}, {NULL}};
const Segment SGI387487[] = {{&S15682[13], 9}, {NULL}};
const Segment SGI387488[] = {{&S15682[88], 18}, {NULL}};
const Segment SGI387489[] = {{&S15683[8], 3}, {NULL}};
const Segment SGI387490[] = {{&S15683[13], 9}, {NULL}};
const Segment SGI387491[] = {{&S15683[88], 18}, {NULL}};
const Segment SGI387492[] = {{&S15684[8], 3}, {NULL}};
const Segment SGI387493[] = {{&S15684[13], 9}, {NULL}};
const Segment SGI387494[] = {{&S15684[88], 10}, {NULL}};
const Segment SGI387495[] = {{&S15685[8], 3}, {NULL}};
const Segment SGI387496[] = {{&S15685[13], 9}, {NULL}};
const Segment SGI387497[] = {{&S15685[88], 19}, {NULL}};
const Segment SGI387498[] = {{&S15687[8], 3}, {NULL}};
const Segment SGI387499[] = {{&S15687[13], 9}, {NULL}};
const Segment SGI387500[] = {{&S15687[88], 17}, {NULL}};
const Segment SGI387501[] = {{&S15688[8], 3}, {NULL}};
const Segment SGI387502[] = {{&S15688[13], 9}, {NULL}};
const Segment SGI387503[] = {{&S15688[88], 12}, {NULL}};
const Segment SGI387504[] = {{&S15689[8], 3}, {NULL}};
const Segment SGI387505[] = {{&S15689[13], 9}, {NULL}};
const Segment SGI387506[] = {{&S15689[88], 13}, {NULL}};
const Segment SGI387507[] = {{&S15690[8], 3}, {NULL}};
const Segment SGI387508[] = {{&S15690[13], 9}, {NULL}};
const Segment SGI387509[] = {{&S15690[88], 8}, {NULL}};
const Segment SGI387510[] = {{&S15692[8], 3}, {NULL}};
const Segment SGI387511[] = {{&S15692[13], 9}, {NULL}};
const Segment SGI387512[] = {{&S15692[88], 15}, {NULL}};
const Segment SGI387513[] = {{&S15693[8], 3}, {NULL}};
const Segment SGI387514[] = {{&S15693[13], 9}, {NULL}};
const Segment SGI387515[] = {{&S15693[88], 18}, {NULL}};
const Segment SGI387516[] = {{&S15694[8], 3}, {NULL}};
const Segment SGI387517[] = {{&S15694[13], 9}, {NULL}};
const Segment SGI387518[] = {{&S15694[88], 23}, {NULL}};
const Segment SGI387519[] = {{&S15696[8], 3}, {NULL}};
const Segment SGI387520[] = {{&S15696[13], 9}, {NULL}};
const Segment SGI387521[] = {{&S15696[88], 9}, {NULL}};
const Segment SGI387522[] = {{&S15697[8], 3}, {NULL}};
const Segment SGI387523[] = {{&S15697[13], 9}, {NULL}};
const Segment SGI387524[] = {{&S15697[88], 18}, {NULL}};
const Segment SGI387525[] = {{&S15698[8], 3}, {NULL}};
const Segment SGI387526[] = {{&S15698[13], 9}, {NULL}};
const Segment SGI387527[] = {{&S15698[88], 7}, {NULL}};
const Segment SGI387528[] = {{&S15699[8], 3}, {NULL}};
const Segment SGI387529[] = {{&S15699[13], 9}, {NULL}};
const Segment SGI387530[] = {{&S15699[88], 15}, {NULL}};
const Segment SGI387531[] = {{&S15700[8], 3}, {NULL}};
const Segment SGI387532[] = {{&S15700[13], 9}, {NULL}};
const Segment SGI387533[] = {{&S15700[88], 18}, {NULL}};
const Segment SGI387534[] = {{&S15702[8], 3}, {NULL}};
const Segment SGI387535[] = {{&S15702[13], 9}, {NULL}};
const Segment SGI387536[] = {{&S15702[88], 12}, {NULL}};
const Segment SGI387537[] = {{&S15703[8], 3}, {NULL}};
const Segment SGI387538[] = {{&S15703[13], 9}, {NULL}};
const Segment SGI387539[] = {{&S15703[88], 13}, {NULL}};
const Segment SGI387540[] = {{&S15704[8], 3}, {NULL}};
const Segment SGI387541[] = {{&S15704[13], 9}, {NULL}};
const Segment SGI387542[] = {{&S15704[88], 16}, {NULL}};
const Segment SGI387543[] = {{&S15705[8], 3}, {NULL}};
const Segment SGI387544[] = {{&S15705[13], 9}, {NULL}};
const Segment SGI387545[] = {{&S15705[88], 13}, {NULL}};
const Segment SGI387546[] = {{&S15706[8], 3}, {NULL}};
const Segment SGI387547[] = {{&S15706[13], 9}, {NULL}};
const Segment SGI387548[] = {{&S15708[8], 3}, {NULL}};
const Segment SGI387549[] = {{&S15708[13], 9}, {NULL}};
const Segment SGI387550[] = {{&S15708[88], 14}, {NULL}};
const Segment SGI387551[] = {{&S15717[8], 3}, {NULL}};
const Segment SGI387552[] = {{&S15717[13], 9}, {NULL}};
const Segment SGI387553[] = {{&S15717[88], 15}, {NULL}};
const Segment SGI387554[] = {{&S15722[8], 3}, {NULL}};
const Segment SGI387555[] = {{&S15722[13], 9}, {NULL}};
const Segment SGI387556[] = {{&S15722[87], 18}, {NULL}};
const Segment SGI387557[] = {{&S15725[8], 3}, {NULL}};
const Segment SGI387558[] = {{&S15725[13], 9}, {NULL}};
const Segment SGI387559[] = {{&S15725[88], 14}, {NULL}};
const Segment SGI387560[] = {{&S15726[8], 3}, {NULL}};
const Segment SGI387561[] = {{&S15726[13], 9}, {NULL}};
const Segment SGI387562[] = {{&S15726[87], 5}, {NULL}};
const Segment SGI387563[] = {{&S15727[8], 3}, {NULL}};
const Segment SGI387564[] = {{&S15727[13], 9}, {NULL}};
const Segment SGI387565[] = {{&S15727[87], 12}, {NULL}};
const Segment SGI387566[] = {{&S15728[8], 3}, {NULL}};
const Segment SGI387567[] = {{&S15728[13], 9}, {NULL}};
const Segment SGI387568[] = {{&S15728[87], 13}, {NULL}};
const Segment SGI387569[] = {{&S75854[13], 10}, {&S75854[24], 9}, {&S75854[34], 11}, {&S75854[46], 4}, {&S75854[51], 16}, {NULL}};
const Segment SGI387570[] = {{&S75855[13], 10}, {&S75855[24], 9}, {&S75855[34], 11}, {&S75855[46], 4}, {&S75855[51], 16}, {NULL}};
const Segment SGI387571[] = {{&S75856[13], 10}, {&S75856[24], 9}, {&S75856[34], 11}, {&S75856[46], 6}, {&S75856[54], 16}, {NULL}};
const Segment SGI387572[] = {{&S75857[13], 10}, {&S75857[24], 9}, {&S75857[34], 11}, {&S75857[46], 11}, {&S75857[58], 11}, {NULL}};
const Segment SGI387573[] = {{&S75858[13], 10}, {&S75858[24], 9}, {&S75858[34], 11}, {&S75858[46], 4}, {&S75858[51], 8}, {&S75858[60], 16}, {NULL}};
const Segment SGI387574[] = {{&S75859[13], 10}, {&S75859[24], 9}, {&S75859[34], 11}, {&S75859[46], 4}, {&S75859[51], 16}, {NULL}};
const Segment SGI387575[] = {{&S75860[13], 10}, {&S75860[24], 9}, {&S75860[34], 11}, {&S75860[46], 5}, {&S75860[52], 11}, {NULL}};
const Segment SGI387576[] = {{&S75861[13], 10}, {&S75861[24], 9}, {&S75861[34], 11}, {&S75861[46], 11}, {&S75861[58], 8}, {&S75861[67], 16}, {NULL}};
const Segment SGI387577[] = {{&S75862[13], 10}, {&S75862[24], 9}, {&S75862[34], 11}, {&S75862[46], 4}, {&S75862[51], 8}, {&S75862[60], 16}, {NULL}};
const Segment SGI387578[] = {{&S75863[13], 10}, {&S75863[24], 9}, {&S75863[34], 11}, {&S75863[46], 8}, {&S75863[55], 7}, {&S75863[63], 31}, {&S75863[96], 16}, {NULL}};
const Segment SGI387579[] = {{&S77251[13], 10}, {&S77251[24], 9}, {&S77251[34], 11}, {&S77251[46], 14}, {&S77251[61], 15}, {&S77251[77], 21}, {&S77251[99], 8}, {NULL}};
const Segment SGI387580[] = {{&S77252[13], 10}, {&S77252[24], 9}, {&S77252[34], 11}, {&S77252[46], 14}, {&S77252[61], 15}, {&S77252[77], 29}, {&S77252[107], 8}, {NULL}};
const Segment SGI387581[] = {{&S77253[13], 10}, {&S77253[24], 9}, {&S77253[34], 11}, {&S77253[46], 14}, {&S77253[61], 15}, {&S77253[77], 18}, {&S77253[96], 8}, {NULL}};
const Segment SGI387582[] = {{&S77254[13], 10}, {&S77254[24], 9}, {&S77254[34], 11}, {&S77254[46], 14}, {&S77254[61], 15}, {&S77254[77], 29}, {&S77254[107], 8}, {NULL}};
const Segment SGI387583[] = {{&S77255[13], 10}, {&S77255[24], 9}, {&S77255[34], 11}, {&S77255[46], 14}, {&S77255[61], 15}, {&S77255[77], 13}, {NULL}};
const Segment SGI387584[] = {{&S34567[24], 8}, {NULL}};
const Segment SGI387585[] = {{&S34568[22], 13}, {NULL}};
const Segment SGI387586[] = {{&S34569[24], 14}, {NULL}};
const Segment SGI387587[] = {{&S34570[24], 9}, {NULL}};
const Segment SGI387588[] = {{&S34571[24], 19}, {NULL}};
const Segment SGI387589[] = {{&S34572[24], 7}, {NULL}};
const Segment SGI387590[] = {{&S34575[24], 8}, {NULL}};
const Segment SGI387591[] = {{&S34576[24], 1}, {NULL}};
const Segment SGI387592[] = {{&S34577[24], 6}, {NULL}};
const Segment SGI387593[] = {{&S34578[21], 10}, {NULL}};
const Segment SGI387594[] = {{&S34579[24], 21}, {NULL}};
const Segment SGI387595[] = {{&S34580[22], 7}, {NULL}};
const Segment SGI387596[] = {{&S34581[24], 10}, {NULL}};
const Segment SGI387597[] = {{&S34582[24], 14}, {NULL}};
const Segment SGI387598[] = {{&S34583[24], 15}, {NULL}};
const Segment SGI387599[] = {{&S34584[22], 6}, {NULL}};
const Segment SGI387600[] = {{&S34585[24], 13}, {NULL}};
const Segment SGI387601[] = {{&S34586[22], 11}, {NULL}};
const Segment SGI387602[] = {{&S34587[34], 23}, {NULL}};
const Segment SGI387603[] = {{&S34588[34], 15}, {NULL}};
const Segment SGI387604[] = {{&S34589[34], 5}, {NULL}};
const Segment SGI387605[] = {{&S34590[34], 4}, {NULL}};
const Segment SGI387606[] = {{&S34591[24], 3}, {NULL}};
const Segment SGI387607[] = {{&S34593[34], 10}, {NULL}};
const Segment SGI387608[] = {{&S34594[34], 14}, {NULL}};
const Segment SGI387609[] = {{&S34595[34], 8}, {NULL}};
const Segment SGI387610[] = {{&S34596[34], 4}, {NULL}};
const Segment SGI387611[] = {{&S34597[34], 14}, {NULL}};
const Segment SGI387612[] = {{&S34598[34], 15}, {NULL}};
const Segment SGI387613[] = {{&S34599[34], 4}, {NULL}};
const Segment SGI387614[] = {{&S34600[34], 10}, {NULL}};
const Segment SGI387615[] = {{&S34601[24], 9}, {NULL}};
const Segment SGI387616[] = {{&S34604[34], 8}, {NULL}};
const Segment SGI387617[] = {{&S34605[34], 9}, {NULL}};
const Segment SGI387618[] = {{&S34607[24], 3}, {NULL}};
const Segment SGI387619[] = {{&S34608[24], 8}, {NULL}};
const Segment SGI387620[] = {{&S34609[24], 9}, {NULL}};
const Segment SGI387621[] = {{&S34610[24], 12}, {NULL}};
const Segment SGI387622[] = {{&S34611[24], 8}, {NULL}};
const Segment SGI387623[] = {{&S34612[22], 12}, {NULL}};
const Segment SGI387624[] = {{&S34613[22], 16}, {NULL}};
const Segment SGI387625[] = {{&S34614[22], 13}, {NULL}};
const Segment SGI387626[] = {{&S34615[24], 19}, {NULL}};
const Segment SGI387627[] = {{&S34616[24], 23}, {NULL}};
const Segment SGI387628[] = {{&S34617[22], 12}, {NULL}};
const Segment SGI387629[] = {{&S34618[22], 16}, {NULL}};
const Segment SGI387630[] = {{&S34619[34], 10}, {NULL}};
const Segment SGI387631[] = {{&S34620[22], 13}, {NULL}};
const Segment SGI387632[] = {{&S34622[22], 14}, {NULL}};
const Segment SGI387633[] = {{&S34623[24], 18}, {NULL}};
const Segment SGI387634[] = {{&S34624[22], 13}, {NULL}};
const Segment SGI387635[] = {{&S34625[24], 20}, {NULL}};
const Segment SGI387636[] = {{&S34626[22], 16}, {NULL}};
const Segment SGI387637[] = {{&S34627[22], 10}, {NULL}};
const Segment SGI387638[] = {{&S34628[24], 13}, {NULL}};
const Segment SGI387639[] = {{&S34629[24], 17}, {NULL}};
const Segment SGI387640[] = {{&S34630[24], 12}, {NULL}};
const Segment SGI387641[] = {{&S34631[22], 21}, {NULL}};
const Segment SGI387642[] = {{&S34632[24], 15}, {NULL}};
const Segment SGI387643[] = {{&S34633[24], 17}, {NULL}};
const Segment SGI387644[] = {{&S34634[22], 15}, {NULL}};
const Segment SGI387645[] = {{&S34635[22], 14}, {NULL}};
const Segment SGI387646[] = {{&S34636[22], 13}, {NULL}};
const Segment SGI387647[] = {{&S34637[22], 21}, {NULL}};
const Segment SGI387648[] = {{&S34638[22], 14}, {NULL}};
const Segment SGI387649[] = {{&S34639[24], 14}, {NULL}};
const Segment SGI387650[] = {{&S34640[6], 1}, {NULL}};
const Segment SGI387651[] = {{&S34645[22], 18}, {NULL}};
const Segment SGI387652[] = {{&S34646[24], 11}, {NULL}};
const Segment SGI387653[] = {{&S34647[22], 13}, {NULL}};
const Segment SGI387654[] = {{&S34649[24], 4}, {NULL}};
const Segment SGI387655[] = {{&S77359[13], 10}, {&S77359[24], 9}, {&S77359[34], 11}, {&S77359[46], 13}, {&S77359[60], 15}, {NULL}};
const Segment SGI387656[] = {{&S77360[13], 10}, {&S77360[24], 9}, {&S77360[34], 11}, {&S77360[46], 13}, {&S77360[60], 4}, {&S77360[65], 54}, {NULL}};
const Segment SGI387657[] = {{&S77361[13], 10}, {&S77361[24], 9}, {&S77361[34], 11}, {&S77361[46], 13}, {&S77361[60], 13}, {NULL}};
const Segment SGI387658[] = {{&S77362[13], 10}, {&S77362[24], 9}, {&S77362[34], 11}, {&S77362[46], 13}, {&S77362[60], 35}, {NULL}};
const Segment SGI387659[] = {{&S77363[13], 10}, {&S77363[24], 9}, {&S77363[34], 11}, {&S77363[46], 13}, {&S77363[60], 4}, {&S77363[65], 59}, {NULL}};
const Segment SGI387660[] = {{&S77364[13], 10}, {&S77364[24], 9}, {&S77364[34], 11}, {&S77364[46], 13}, {&S77364[60], 34}, {NULL}};
const Segment SGI387661[] = {{&S77365[13], 10}, {&S77365[24], 9}, {&S77365[34], 11}, {&S77365[46], 13}, {&S77365[60], 14}, {NULL}};
const Segment SGI387662[] = {{&S77366[13], 10}, {&S77366[24], 9}, {&S77366[34], 11}, {&S77366[46], 13}, {&S77366[60], 12}, {&S77366[73], 8}, {NULL}};
const Segment SGI387663[] = {{&S77367[13], 10}, {&S77367[24], 9}, {&S77367[34], 11}, {&S77367[46], 13}, {&S77367[60], 4}, {&S77367[65], 8}, {&S77367[74], 17}, {NULL}};
const Segment SGI387664[] = {{&S77518[13], 10}, {&S77518[24], 9}, {&S77518[34], 11}, {&S77518[46], 13}, {&S77518[60], 21}, {NULL}};
const Segment SGI387665[] = {{&S77520[13], 10}, {&S77520[24], 9}, {&S77520[34], 11}, {&S77520[46], 13}, {&S77520[60], 22}, {NULL}};
const Segment SGI387666[] = {{&S77521[13], 10}, {&S77521[24], 9}, {&S77521[34], 11}, {&S77521[46], 13}, {&S77521[60], 23}, {NULL}};
const Segment SGI387667[] = {{&S78159[13], 10}, {&S78159[24], 9}, {&S78159[34], 11}, {&S78159[46], 14}, {&S78159[61], 15}, {NULL}};
const Segment SGI387668[] = {{&S78160[13], 10}, {&S78160[24], 9}, {&S78160[34], 11}, {&S78160[46], 14}, {&S78160[61], 15}, {&S78160[77], 8}, {&S78160[86], 7}, {&S78160[94], 31}, {NULL}};
const Segment SGI387669[] = {{&S78161[13], 10}, {&S78161[24], 9}, {&S78161[34], 11}, {&S78161[46], 14}, {&S78161[61], 15}, {&S78161[77], 50}, {&S78161[95], 1}, {&S78161[130], 18}, {&S78161[149], 7}, {&S78161[157], 7}, {&S78161[165], 7}, {NULL}};
const Segment SGI387670[] = {{&S78451[13], 10}, {&S78451[24], 10}, {&S78451[35], 11}, {&S78451[47], 6}, {&S78451[54], 21}, {NULL}};
const Segment SGI387671[] = {{&S78452[13], 10}, {&S78452[24], 10}, {&S78452[35], 11}, {&S78452[47], 19}, {&S78452[67], 21}, {NULL}};
const Segment SGI387672[] = {{&S78453[13], 10}, {&S78453[24], 10}, {&S78453[35], 11}, {&S78453[47], 4}, {&S78453[52], 21}, {NULL}};
const Segment SGI387673[] = {{&S78454[13], 10}, {&S78454[24], 10}, {&S78454[35], 11}, {&S78454[47], 11}, {&S78454[59], 21}, {NULL}};
const Segment SGI387674[] = {{&S78455[13], 10}, {&S78455[24], 10}, {&S78455[35], 11}, {&S78455[47], 9}, {&S78455[57], 21}, {NULL}};
const Segment SGI387675[] = {{&S78456[13], 10}, {&S78456[24], 10}, {&S78456[35], 11}, {&S78456[47], 4}, {&S78456[52], 21}, {NULL}};
const Segment SGI387676[] = {{&S18183[8], 3}, {NULL}};
const Segment SGI387677[] = {{&S18183[13], 3}, {NULL}};
const Segment SGI387678[] = {{&S18183[70], 9}, {NULL}};
const Segment SGI387679[] = {{&S78567[13], 10}, {&S78567[24], 9}, {&S78567[34], 11}, {&S78567[46], 14}, {&S78567[61], 15}, {&S78567[77], 40}, {NULL}};
const Segment SGI387680[] = {{&S15674[8], 3}, {NULL}};
const Segment SGI387681[] = {{&S15674[13], 9}, {NULL}};
const Segment SGI387682[] = {{&S15674[33], 13}, {NULL}};
const Segment SGI387683[] = {{&S78759[13], 10}, {&S78759[24], 10}, {&S78759[35], 11}, {&S78759[47], 4}, {&S78759[52], 21}, {NULL}};
const Segment SGI387684[] = {{&S78760[13], 10}, {&S78760[24], 10}, {&S78760[35], 11}, {&S78760[47], 4}, {&S78760[52], 21}, {NULL}};
const Segment SGI387685[] = {{&S78761[13], 10}, {&S78761[24], 10}, {&S78761[35], 11}, {&S78761[47], 4}, {&S78761[52], 21}, {NULL}};
const Segment SGI387686[] = {{&S78762[13], 10}, {&S78762[24], 10}, {&S78762[35], 11}, {&S78762[47], 4}, {&S78762[52], 8}, {&S78762[61], 21}, {NULL}};
const Segment SGI387687[] = {{&S78763[13], 10}, {&S78763[24], 10}, {&S78763[35], 11}, {&S78763[47], 4}, {&S78763[52], 21}, {NULL}};
const Segment SGI387688[] = {{&S78920[13], 10}, {&S78920[24], 9}, {&S78920[34], 11}, {&S78920[46], 14}, {&S78920[61], 28}, {NULL}};
const Segment SGI387689[] = {{&S78921[13], 10}, {&S78921[24], 9}, {&S78921[34], 11}, {&S78921[46], 14}, {&S78921[61], 25}, {NULL}};
const Segment SGI387690[] = {{&S78922[13], 10}, {&S78922[24], 9}, {&S78922[34], 11}, {&S78922[46], 14}, {&S78922[61], 32}, {NULL}};
const Segment SGI387691[] = {{&S78924[13], 10}, {&S78924[24], 9}, {&S78924[34], 11}, {&S78924[46], 14}, {&S78924[61], 15}, {NULL}};
const Segment SGI387692[] = {{&S78925[13], 10}, {&S78925[24], 9}, {&S78925[34], 11}, {&S78925[46], 14}, {&S78925[61], 4}, {&S78925[66], 54}, {NULL}};
const Segment SGI387693[] = {{&S78927[13], 10}, {&S78927[24], 9}, {&S78927[34], 11}, {&S78927[46], 14}, {&S78927[61], 8}, {&S78927[70], 4}, {&S78927[75], 14}, {&S78927[90], 31}, {NULL}};
const Segment SGI387694[] = {{&S78928[13], 10}, {&S78928[24], 9}, {&S78928[34], 11}, {&S78928[46], 14}, {&S78928[61], 28}, {NULL}};
const Segment SGI387695[] = {{&S78975[13], 10}, {&S78975[24], 9}, {&S78975[34], 11}, {&S78975[46], 14}, {&S78975[61], 16}, {NULL}};
const Segment SGI387696[] = {{&S79177[13], 10}, {&S79177[24], 9}, {&S79177[34], 11}, {&S79177[46], 14}, {&S79177[61], 23}, {NULL}};
const Segment SGI387697[] = {{&S79179[13], 10}, {&S79179[24], 9}, {&S79179[34], 11}, {&S79179[46], 14}, {&S79179[61], 22}, {NULL}};
const Segment SGI387698[] = {{&S19161[8], 3}, {NULL}};
const Segment SGI387699[] = {{&S19161[13], 7}, {NULL}};
const Segment SGI387700[] = {{&S19161[25], 14}, {NULL}};
const Segment SGI387701[] = {{&S19163[8], 3}, {NULL}};
const Segment SGI387702[] = {{&S19163[13], 14}, {NULL}};
const Segment SGI387703[] = {{&S19165[8], 3}, {NULL}};
const Segment SGI387704[] = {{&S19165[13], 14}, {NULL}};
const Segment SGI387705[] = {{&S80129[13], 10}, {&S80129[24], 10}, {&S80129[35], 11}, {&S80129[47], 11}, {&S80129[59], 19}, {NULL}};
const Segment SGI387706[] = {{&S80130[13], 10}, {&S80130[24], 10}, {&S80130[35], 11}, {&S80130[47], 11}, {&S80130[59], 8}, {&S80130[68], 19}, {NULL}};
const Segment SGI387707[] = {{&S80133[13], 10}, {&S80133[24], 10}, {&S80133[35], 11}, {&S80133[47], 4}, {&S80133[52], 19}, {NULL}};
const Segment SGI387708[] = {{&S19875[8], 3}, {NULL}};
const Segment SGI387709[] = {{&S19875[13], 9}, {NULL}};
const Segment SGI387710[] = {{&S19875[33], 13}, {NULL}};
const Segment SGI387711[] = {{&S19881[8], 3}, {NULL}};
const Segment SGI387712[] = {{&S19881[13], 9}, {NULL}};
const Segment SGI387713[] = {{&S19881[88], 13}, {NULL}};
const Segment SGI387714[] = {{&S19883[8], 3}, {NULL}};
const Segment SGI387715[] = {{&S19883[13], 9}, {NULL}};
const Segment SGI387716[] = {{&S19883[87], 16}, {NULL}};
const Segment SGI387717[] = {{&S19884[8], 3}, {NULL}};
const Segment SGI387718[] = {{&S19884[13], 9}, {NULL}};
const Segment SGI387719[] = {{&S19884[87], 16}, {NULL}};
const Segment SGI387720[] = {{&S19885[8], 3}, {NULL}};
const Segment SGI387721[] = {{&S19885[13], 9}, {NULL}};
const Segment SGI387722[] = {{&S19885[30], 15}, {NULL}};
const Segment SGI387723[] = {{&S19886[8], 3}, {NULL}};
const Segment SGI387724[] = {{&S19886[13], 9}, {NULL}};
const Segment SGI387725[] = {{&S19886[87], 16}, {NULL}};
const Segment SGI387726[] = {{&S19888[8], 3}, {NULL}};
const Segment SGI387727[] = {{&S19888[13], 9}, {NULL}};
const Segment SGI387728[] = {{&S19888[87], 13}, {NULL}};
const Segment SGI387729[] = {{&S19889[8], 3}, {NULL}};
const Segment SGI387730[] = {{&S19889[13], 9}, {NULL}};
const Segment SGI387731[] = {{&S19889[87], 18}, {NULL}};
const Segment SGI387732[] = {{&S19890[8], 3}, {NULL}};
const Segment SGI387733[] = {{&S19890[13], 9}, {NULL}};
const Segment SGI387734[] = {{&S19890[87], 18}, {NULL}};
const Segment SGI387735[] = {{&S19891[8], 3}, {NULL}};
const Segment SGI387736[] = {{&S19891[13], 9}, {NULL}};
const Segment SGI387737[] = {{&S19892[8], 3}, {NULL}};
const Segment SGI387738[] = {{&S19892[13], 9}, {NULL}};
const Segment SGI387739[] = {{&S19892[87], 18}, {NULL}};
const Segment SGI387740[] = {{&S19893[8], 3}, {NULL}};
const Segment SGI387741[] = {{&S19893[13], 9}, {NULL}};
const Segment SGI387742[] = {{&S19893[87], 13}, {NULL}};
const Segment SGI387743[] = {{&S19894[8], 3}, {NULL}};
const Segment SGI387744[] = {{&S19894[13], 9}, {NULL}};
const Segment SGI387745[] = {{&S19894[87], 13}, {NULL}};
const Segment SGI387746[] = {{&S19897[8], 3}, {NULL}};
const Segment SGI387747[] = {{&S19897[13], 9}, {NULL}};
const Segment SGI387748[] = {{&S19897[87], 13}, {NULL}};
const Segment SGI387749[] = {{&S19898[8], 3}, {NULL}};
const Segment SGI387750[] = {{&S19898[13], 9}, {NULL}};
const Segment SGI387751[] = {{&S19898[87], 16}, {NULL}};
const Segment SGI387752[] = {{&S19899[8], 3}, {NULL}};
const Segment SGI387753[] = {{&S19899[13], 9}, {NULL}};
const Segment SGI387754[] = {{&S19899[87], 13}, {NULL}};
const Segment SGI387755[] = {{&S19900[8], 3}, {NULL}};
const Segment SGI387756[] = {{&S19900[13], 9}, {NULL}};
const Segment SGI387757[] = {{&S19900[87], 19}, {NULL}};
const Segment SGI387758[] = {{&S19901[8], 3}, {&S19901[8], 3}, {NULL}};
const Segment SGI387759[] = {{&S19901[13], 9}, {&S19901[13], 9}, {NULL}};
const Segment SGI387760[] = {{&S19901[30], 15}, {NULL}};
const Segment SGI387761[] = {{&S19902[8], 3}, {NULL}};
const Segment SGI387762[] = {{&S19902[13], 9}, {NULL}};
const Segment SGI387763[] = {{&S19902[87], 18}, {NULL}};
const Segment SGI387764[] = {{&S19904[8], 3}, {NULL}};
const Segment SGI387765[] = {{&S19904[13], 9}, {NULL}};
const Segment SGI387766[] = {{&S19904[87], 15}, {NULL}};
const Segment SGI387767[] = {{&S19905[8], 3}, {NULL}};
const Segment SGI387768[] = {{&S19905[13], 9}, {NULL}};
const Segment SGI387769[] = {{&S19905[87], 15}, {NULL}};
const Segment SGI387770[] = {{&S19906[8], 3}, {NULL}};
const Segment SGI387771[] = {{&S19906[13], 9}, {NULL}};
const Segment SGI387772[] = {{&S19906[87], 17}, {NULL}};
const Segment SGI387773[] = {{&S19907[8], 3}, {NULL}};
const Segment SGI387774[] = {{&S19907[13], 9}, {NULL}};
const Segment SGI387775[] = {{&S19907[87], 19}, {NULL}};
const Segment SGI387776[] = {{&S80336[13], 10}, {&S80336[24], 9}, {&S80336[34], 11}, {&S80336[46], 14}, {&S80336[61], 21}, {NULL}};
const Segment SGI387777[] = {{&S19911[8], 3}, {NULL}};
const Segment SGI387778[] = {{&S19911[13], 9}, {NULL}};
const Segment SGI387779[] = {{&S19911[87], 16}, {NULL}};
const Segment SGI387780[] = {{&S19912[8], 3}, {NULL}};
const Segment SGI387781[] = {{&S19912[13], 9}, {NULL}};
const Segment SGI387782[] = {{&S19912[87], 23}, {NULL}};
const Segment SGI387783[] = {{&S19914[8], 3}, {NULL}};
const Segment SGI387784[] = {{&S19914[13], 9}, {NULL}};
const Segment SGI387785[] = {{&S19914[87], 8}, {NULL}};
const Segment SGI387786[] = {{&S19916[8], 3}, {NULL}};
const Segment SGI387787[] = {{&S19916[13], 9}, {NULL}};
const Segment SGI387788[] = {{&S19916[87], 12}, {NULL}};
const Segment SGI387789[] = {{&S19917[8], 3}, {NULL}};
const Segment SGI387790[] = {{&S19917[13], 9}, {NULL}};
const Segment SGI387791[] = {{&S19917[87], 9}, {NULL}};
const Segment SGI387792[] = {{&S19918[8], 3}, {NULL}};
const Segment SGI387793[] = {{&S19918[13], 9}, {NULL}};
const Segment SGI387794[] = {{&S19918[87], 13}, {NULL}};
const Segment SGI387795[] = {{&S19919[8], 3}, {NULL}};
const Segment SGI387796[] = {{&S19919[13], 9}, {NULL}};
const Segment SGI387797[] = {{&S19919[87], 12}, {NULL}};
const Segment SGI387798[] = {{&S19920[8], 3}, {NULL}};
const Segment SGI387799[] = {{&S19920[13], 9}, {NULL}};
const Segment SGI387800[] = {{&S19920[87], 23}, {NULL}};
const Segment SGI387801[] = {{&S19921[8], 3}, {NULL}};
const Segment SGI387802[] = {{&S19921[13], 9}, {NULL}};
const Segment SGI387803[] = {{&S19921[87], 16}, {NULL}};
const Segment SGI387804[] = {{&S19922[8], 3}, {NULL}};
const Segment SGI387805[] = {{&S19922[13], 9}, {NULL}};
const Segment SGI387806[] = {{&S19922[88], 5}, {NULL}};
const Segment SGI387807[] = {{&S19925[8], 3}, {NULL}};
const Segment SGI387808[] = {{&S19925[13], 9}, {NULL}};
const Segment SGI387809[] = {{&S19925[87], 14}, {NULL}};
const Segment SGI387810[] = {{&S19926[8], 3}, {NULL}};
const Segment SGI387811[] = {{&S19926[13], 9}, {NULL}};
const Segment SGI387812[] = {{&S19926[87], 23}, {NULL}};
const Segment SGI387813[] = {{&S19927[8], 3}, {NULL}};
const Segment SGI387814[] = {{&S19927[13], 9}, {NULL}};
const Segment SGI387815[] = {{&S19927[87], 15}, {NULL}};
const Segment SGI387816[] = {{&S19931[8], 3}, {NULL}};
const Segment SGI387817[] = {{&S19931[13], 9}, {NULL}};
const Segment SGI387818[] = {{&S19931[87], 7}, {NULL}};
const Segment SGI387819[] = {{&S19937[8], 3}, {NULL}};
const Segment SGI387820[] = {{&S19937[13], 9}, {NULL}};
const Segment SGI387821[] = {{&S19937[87], 16}, {NULL}};
const Segment SGI387822[] = {{&S19938[8], 3}, {NULL}};
const Segment SGI387823[] = {{&S19938[13], 9}, {NULL}};
const Segment SGI387824[] = {{&S19938[87], 18}, {NULL}};
const Segment SGI387825[] = {{&S19939[8], 3}, {NULL}};
const Segment SGI387826[] = {{&S19939[13], 9}, {NULL}};
const Segment SGI387827[] = {{&S19939[87], 18}, {NULL}};
const Segment SGI387828[] = {{&S37430[24], 4}, {NULL}};
const Segment SGI387829[] = {{&S37431[24], 3}, {NULL}};
const Segment SGI387830[] = {{&S37432[24], 5}, {NULL}};
const Segment SGI387831[] = {{&S37433[24], 9}, {NULL}};
const Segment SGI387832[] = {{&S37435[32], 15}, {NULL}};
const Segment SGI387833[] = {{&S37441[24], 21}, {NULL}};
const Segment SGI387834[] = {{&S81313[13], 10}, {&S81313[24], 9}, {&S81313[34], 11}, {&S81313[46], 14}, {&S81313[61], 15}, {&S81313[77], 10}, {NULL}};
const Segment SGI387835[] = {{&S20990[8], 3}, {NULL}};
const Segment SGI387836[] = {{&S20990[13], 7}, {NULL}};
const Segment SGI387837[] = {{&S20990[69], 13}, {NULL}};
const Segment SGI387838[] = {{&S20991[8], 3}, {NULL}};
const Segment SGI387839[] = {{&S20991[13], 7}, {NULL}};
const Segment SGI387840[] = {{&S20991[69], 13}, {NULL}};
const Segment SGI387841[] = {{&S20992[8], 3}, {NULL}};
const Segment SGI387842[] = {{&S20992[13], 7}, {NULL}};
const Segment SGI387843[] = {{&S20992[69], 15}, {NULL}};
const Segment SGI387844[] = {{&S20993[8], 3}, {NULL}};
const Segment SGI387845[] = {{&S20993[13], 7}, {NULL}};
const Segment SGI387846[] = {{&S20993[72], 13}, {NULL}};
const Segment SGI387847[] = {{&S20994[8], 3}, {NULL}};
const Segment SGI387848[] = {{&S20994[13], 7}, {NULL}};
const Segment SGI387849[] = {{&S20994[70], 15}, {NULL}};
const Segment SGI387850[] = {{&S20995[8], 3}, {NULL}};
const Segment SGI387851[] = {{&S20995[13], 7}, {NULL}};
const Segment SGI387852[] = {{&S20995[69], 15}, {NULL}};
const Segment SGI387853[] = {{&S20997[8], 3}, {NULL}};
const Segment SGI387854[] = {{&S20997[13], 7}, {NULL}};
const Segment SGI387855[] = {{&S20997[69], 15}, {NULL}};
const Segment SGI387856[] = {{&S20999[8], 3}, {NULL}};
const Segment SGI387857[] = {{&S20999[13], 7}, {NULL}};
const Segment SGI387858[] = {{&S20999[69], 15}, {NULL}};
const Segment SGI387859[] = {{&S21000[8], 3}, {NULL}};
const Segment SGI387860[] = {{&S21000[13], 7}, {NULL}};
const Segment SGI387861[] = {{&S21000[69], 13}, {NULL}};
const Segment SGI387862[] = {{&S21004[8], 3}, {NULL}};
const Segment SGI387863[] = {{&S21004[13], 7}, {NULL}};
const Segment SGI387864[] = {{&S21004[68], 39}, {NULL}};
const Segment SGI387865[] = {{&S21005[8], 3}, {NULL}};
const Segment SGI387866[] = {{&S21005[13], 7}, {NULL}};
const Segment SGI387867[] = {{&S21005[72], 13}, {NULL}};
const Segment SGI387868[] = {{&S21006[8], 3}, {NULL}};
const Segment SGI387869[] = {{&S21006[13], 7}, {NULL}};
const Segment SGI387870[] = {{&S21006[72], 13}, {NULL}};
const Segment SGI387871[] = {{&S21010[8], 3}, {NULL}};
const Segment SGI387872[] = {{&S21010[13], 7}, {NULL}};
const Segment SGI387873[] = {{&S21010[70], 13}, {NULL}};
const Segment SGI387874[] = {{&S21012[8], 3}, {NULL}};
const Segment SGI387875[] = {{&S21012[13], 7}, {NULL}};
const Segment SGI387876[] = {{&S21012[72], 22}, {NULL}};
const Segment SGI387877[] = {{&S21013[8], 3}, {NULL}};
const Segment SGI387878[] = {{&S21013[13], 7}, {NULL}};
const Segment SGI387879[] = {{&S21013[69], 15}, {NULL}};
const Segment SGI387880[] = {{&S21014[8], 3}, {NULL}};
const Segment SGI387881[] = {{&S21014[13], 7}, {NULL}};
const Segment SGI387882[] = {{&S21014[69], 13}, {NULL}};
const Segment SGI387883[] = {{&S21015[8], 3}, {NULL}};
const Segment SGI387884[] = {{&S21015[13], 7}, {NULL}};
const Segment SGI387885[] = {{&S21015[72], 13}, {NULL}};
const Segment SGI387886[] = {{&S21018[8], 3}, {NULL}};
const Segment SGI387887[] = {{&S21018[13], 7}, {NULL}};
const Segment SGI387888[] = {{&S21018[68], 13}, {NULL}};
const Segment SGI387889[] = {{&S21019[8], 3}, {NULL}};
const Segment SGI387890[] = {{&S21019[13], 7}, {NULL}};
const Segment SGI387891[] = {{&S21019[70], 15}, {NULL}};
const Segment SGI387892[] = {{&S21022[8], 3}, {NULL}};
const Segment SGI387893[] = {{&S21022[13], 7}, {NULL}};
const Segment SGI387894[] = {{&S21022[72], 13}, {NULL}};
const Segment SGI387895[] = {{&S21023[8], 3}, {NULL}};
const Segment SGI387896[] = {{&S21023[13], 7}, {NULL}};
const Segment SGI387897[] = {{&S21023[70], 15}, {NULL}};
const Segment SGI387898[] = {{&S21025[8], 3}, {NULL}};
const Segment SGI387899[] = {{&S21025[13], 7}, {NULL}};
const Segment SGI387900[] = {{&S21025[72], 13}, {NULL}};
const Segment SGI387901[] = {{&S21027[8], 3}, {NULL}};
const Segment SGI387902[] = {{&S21027[13], 7}, {NULL}};
const Segment SGI387903[] = {{&S21027[72], 13}, {NULL}};
const Segment SGI387904[] = {{&S21029[8], 3}, {NULL}};
const Segment SGI387905[] = {{&S21029[13], 7}, {NULL}};
const Segment SGI387906[] = {{&S21029[63], 5}, {NULL}};
const Segment SGI387907[] = {{&S21030[8], 3}, {NULL}};
const Segment SGI387908[] = {{&S21030[13], 7}, {NULL}};
const Segment SGI387909[] = {{&S21030[72], 13}, {NULL}};
const Segment SGI387910[] = {{&S21032[8], 3}, {NULL}};
const Segment SGI387911[] = {{&S21032[13], 7}, {NULL}};
const Segment SGI387912[] = {{&S21032[72], 13}, {NULL}};
const Segment SGI387913[] = {{&S81583[13], 10}, {&S81583[24], 9}, {&S81583[34], 11}, {&S81583[46], 14}, {&S81583[61], 38}, {&S81583[24], 9}, {&S81583[34], 11}, {&S81583[122], 4}, {&S81583[127], 8}, {NULL}};
const Segment SGI387914[] = {{&S81734[13], 10}, {&S81734[24], 9}, {&S81734[34], 11}, {&S81734[46], 14}, {&S81734[61], 15}, {&S81734[77], 25}, {NULL}};
const Segment SGI387915[] = {{&S81844[13], 10}, {&S81844[24], 9}, {&S81844[34], 11}, {&S81844[46], 13}, {&S81844[60], 114}, {NULL}};
const Segment SGI387916[] = {{&S81845[13], 10}, {&S81845[24], 9}, {&S81845[34], 11}, {&S81845[46], 13}, {&S81845[60], 118}, {NULL}};
const Segment SGI387917[] = {{&S81846[13], 10}, {&S81846[24], 9}, {&S81846[34], 11}, {&S81846[46], 13}, {&S81846[60], 127}, {NULL}};
const Segment SGI387918[] = {{&S21498[8], 3}, {NULL}};
const Segment SGI387919[] = {{&S21498[13], 9}, {NULL}};
const Segment SGI387920[] = {{&S21498[33], 15}, {NULL}};
const Segment SGI387921[] = {{&S21499[8], 3}, {NULL}};
const Segment SGI387922[] = {{&S21499[13], 9}, {NULL}};
const Segment SGI387923[] = {{&S21499[33], 13}, {NULL}};
const Segment SGI387924[] = {{&S21500[8], 3}, {NULL}};
const Segment SGI387925[] = {{&S21500[13], 9}, {NULL}};
const Segment SGI387926[] = {{&S21500[33], 15}, {NULL}};
const Segment SGI387927[] = {{&S21501[8], 3}, {NULL}};
const Segment SGI387928[] = {{&S21501[13], 9}, {NULL}};
const Segment SGI387929[] = {{&S21501[33], 15}, {NULL}};
const Segment SGI387930[] = {{&S21502[8], 3}, {NULL}};
const Segment SGI387931[] = {{&S21502[13], 9}, {NULL}};
const Segment SGI387932[] = {{&S21502[33], 15}, {NULL}};
const Segment SGI387933[] = {{&S21504[8], 3}, {NULL}};
const Segment SGI387934[] = {{&S21504[13], 9}, {NULL}};
const Segment SGI387935[] = {{&S21504[33], 15}, {NULL}};
const Segment SGI387936[] = {{&S21505[8], 3}, {NULL}};
const Segment SGI387937[] = {{&S21505[13], 9}, {NULL}};
const Segment SGI387938[] = {{&S21505[33], 15}, {NULL}};
const Segment SGI387939[] = {{&S21506[8], 3}, {NULL}};
const Segment SGI387940[] = {{&S21506[13], 9}, {NULL}};
const Segment SGI387941[] = {{&S21506[33], 15}, {NULL}};
const Segment SGI387942[] = {{&S21507[8], 3}, {NULL}};
const Segment SGI387943[] = {{&S21507[13], 9}, {NULL}};
const Segment SGI387944[] = {{&S21507[33], 15}, {NULL}};
const Segment SGI387945[] = {{&S21508[8], 3}, {NULL}};
const Segment SGI387946[] = {{&S21508[13], 9}, {NULL}};
const Segment SGI387947[] = {{&S21509[8], 3}, {NULL}};
const Segment SGI387948[] = {{&S21509[13], 9}, {NULL}};
const Segment SGI387949[] = {{&S21509[87], 13}, {NULL}};
const Segment SGI387950[] = {{&S21510[8], 3}, {&S21510[8], 3}, {NULL}};
const Segment SGI387951[] = {{&S21510[13], 9}, {&S21510[13], 9}, {NULL}};
const Segment SGI387952[] = {{&S21510[33], 15}, {&S21510[33], 15}, {NULL}};
const Segment SGI387953[] = {{&S21510[97], 13}, {&S21510[192], 17}, {NULL}};
const Segment SGI387954[] = {{&S21512[8], 3}, {&S21512[8], 3}, {NULL}};
const Segment SGI387955[] = {{&S21512[13], 9}, {&S21512[13], 9}, {NULL}};
const Segment SGI387956[] = {{&S21512[33], 15}, {&S21512[33], 15}, {NULL}};
const Segment SGI387957[] = {{&S21512[98], 13}, {&S21512[194], 17}, {NULL}};
const Segment SGI387958[] = {{&S21517[8], 3}, {&S21517[8], 3}, {NULL}};
const Segment SGI387959[] = {{&S21517[13], 9}, {&S21517[13], 9}, {NULL}};
const Segment SGI387960[] = {{&S21517[33], 15}, {&S21517[33], 15}, {NULL}};
const Segment SGI387961[] = {{&S21517[97], 13}, {&S21517[192], 17}, {NULL}};
const Segment SGI387962[] = {{&S21518[8], 3}, {NULL}};
const Segment SGI387963[] = {{&S21518[13], 9}, {NULL}};
const Segment SGI387964[] = {{&S21518[88], 14}, {NULL}};
const Segment SGI387965[] = {{&S21520[8], 3}, {NULL}};
const Segment SGI387966[] = {{&S21520[13], 9}, {NULL}};
const Segment SGI387967[] = {{&S21520[33], 15}, {NULL}};
const Segment SGI387968[] = {{&S21520[97], 19}, {NULL}};
const Segment SGI387969[] = {{&S21521[8], 3}, {NULL}};
const Segment SGI387970[] = {{&S21521[13], 9}, {NULL}};
const Segment SGI387971[] = {{&S21521[87], 15}, {NULL}};
const Segment SGI387972[] = {{&S21522[8], 3}, {NULL}};
const Segment SGI387973[] = {{&S21522[13], 9}, {NULL}};
const Segment SGI387974[] = {{&S21522[33], 15}, {NULL}};
const Segment SGI387975[] = {{&S21522[97], 18}, {NULL}};
const Segment SGI387976[] = {{&S21523[8], 3}, {&S21523[8], 3}, {NULL}};
const Segment SGI387977[] = {{&S21523[13], 9}, {&S21523[13], 9}, {NULL}};
const Segment SGI387978[] = {{&S21523[33], 15}, {&S21523[33], 15}, {NULL}};
const Segment SGI387979[] = {{&S21523[97], 13}, {&S21523[192], 17}, {NULL}};
const Segment SGI387980[] = {{&S21524[8], 3}, {NULL}};
const Segment SGI387981[] = {{&S21524[13], 9}, {NULL}};
const Segment SGI387982[] = {{&S21524[87], 19}, {NULL}};
const Segment SGI387983[] = {{&S21525[8], 3}, {NULL}};
const Segment SGI387984[] = {{&S21525[13], 9}, {NULL}};
const Segment SGI387985[] = {{&S21525[87], 16}, {NULL}};
const Segment SGI387986[] = {{&S21527[8], 3}, {NULL}};
const Segment SGI387987[] = {{&S21527[13], 9}, {NULL}};
const Segment SGI387988[] = {{&S21527[87], 18}, {NULL}};
const Segment SGI387989[] = {{&S21528[8], 3}, {NULL}};
const Segment SGI387990[] = {{&S21528[13], 9}, {NULL}};
const Segment SGI387991[] = {{&S21528[33], 15}, {NULL}};
const Segment SGI387992[] = {{&S21528[97], 19}, {NULL}};
const Segment SGI387993[] = {{&S21531[8], 3}, {NULL}};
const Segment SGI387994[] = {{&S21531[13], 9}, {NULL}};
const Segment SGI387995[] = {{&S21531[87], 16}, {NULL}};
const Segment SGI387996[] = {{&S21532[8], 3}, {NULL}};
const Segment SGI387997[] = {{&S21532[13], 9}, {NULL}};
const Segment SGI387998[] = {{&S21532[87], 16}, {NULL}};
const Segment SGI387999[] = {{&S21533[8], 3}, {NULL}};
const Segment SGI388000[] = {{&S21533[13], 9}, {NULL}};
const Segment SGI388001[] = {{&S21533[87], 16}, {NULL}};
const Segment SGI388002[] = {{&S21534[8], 3}, {NULL}};
const Segment SGI388003[] = {{&S21534[13], 9}, {NULL}};
const Segment SGI388004[] = {{&S21534[87], 13}, {NULL}};
const Segment SGI388005[] = {{&S21535[8], 3}, {NULL}};
const Segment SGI388006[] = {{&S21535[13], 9}, {NULL}};
const Segment SGI388007[] = {{&S21535[33], 15}, {NULL}};
const Segment SGI388008[] = {{&S21535[97], 18}, {NULL}};
const Segment SGI388009[] = {{&S21539[8], 3}, {NULL}};
const Segment SGI388010[] = {{&S21539[13], 9}, {NULL}};
const Segment SGI388011[] = {{&S21539[87], 15}, {NULL}};
const Segment SGI388012[] = {{&S21540[8], 3}, {NULL}};
const Segment SGI388013[] = {{&S21540[13], 9}, {NULL}};
const Segment SGI388014[] = {{&S21540[87], 18}, {NULL}};
const Segment SGI388015[] = {{&S21541[8], 3}, {NULL}};
const Segment SGI388016[] = {{&S21541[13], 9}, {NULL}};
const Segment SGI388017[] = {{&S21541[87], 8}, {NULL}};
const Segment SGI388018[] = {{&S21542[8], 3}, {NULL}};
const Segment SGI388019[] = {{&S21542[13], 9}, {NULL}};
const Segment SGI388020[] = {{&S21542[87], 15}, {NULL}};
const Segment SGI388021[] = {{&S21543[8], 3}, {NULL}};
const Segment SGI388022[] = {{&S21543[13], 9}, {NULL}};
const Segment SGI388023[] = {{&S21543[87], 16}, {NULL}};
const Segment SGI388024[] = {{&S21545[8], 3}, {NULL}};
const Segment SGI388025[] = {{&S21545[13], 9}, {NULL}};
const Segment SGI388026[] = {{&S21545[87], 13}, {NULL}};
const Segment SGI388027[] = {{&S21546[8], 3}, {NULL}};
const Segment SGI388028[] = {{&S21546[13], 9}, {NULL}};
const Segment SGI388029[] = {{&S21546[87], 18}, {NULL}};
const Segment SGI388030[] = {{&S21547[8], 3}, {NULL}};
const Segment SGI388031[] = {{&S21547[13], 9}, {NULL}};
const Segment SGI388032[] = {{&S21547[87], 13}, {NULL}};
const Segment SGI388033[] = {{&S21549[8], 3}, {NULL}};
const Segment SGI388034[] = {{&S21549[13], 9}, {NULL}};
const Segment SGI388035[] = {{&S21549[33], 15}, {NULL}};
const Segment SGI388036[] = {{&S21549[97], 15}, {NULL}};
const Segment SGI388037[] = {{&S21550[8], 3}, {NULL}};
const Segment SGI388038[] = {{&S21550[13], 9}, {NULL}};
const Segment SGI388039[] = {{&S21550[87], 16}, {NULL}};
const Segment SGI388040[] = {{&S21552[8], 3}, {NULL}};
const Segment SGI388041[] = {{&S21552[13], 9}, {NULL}};
const Segment SGI388042[] = {{&S21552[87], 23}, {NULL}};
const Segment SGI388043[] = {{&S21553[8], 3}, {NULL}};
const Segment SGI388044[] = {{&S21553[13], 9}, {NULL}};
const Segment SGI388045[] = {{&S21553[87], 18}, {NULL}};
const Segment SGI388046[] = {{&S21554[8], 3}, {NULL}};
const Segment SGI388047[] = {{&S21554[13], 9}, {NULL}};
const Segment SGI388048[] = {{&S21554[87], 13}, {NULL}};
const Segment SGI388049[] = {{&S21555[8], 3}, {NULL}};
const Segment SGI388050[] = {{&S21555[13], 9}, {NULL}};
const Segment SGI388051[] = {{&S21555[87], 13}, {NULL}};
const Segment SGI388052[] = {{&S21556[8], 3}, {NULL}};
const Segment SGI388053[] = {{&S21556[13], 9}, {NULL}};
const Segment SGI388054[] = {{&S21556[87], 12}, {NULL}};
const Segment SGI388055[] = {{&S21557[8], 3}, {NULL}};
const Segment SGI388056[] = {{&S21557[13], 9}, {NULL}};
const Segment SGI388057[] = {{&S21557[87], 18}, {NULL}};
const Segment SGI388058[] = {{&S21558[8], 3}, {NULL}};
const Segment SGI388059[] = {{&S21558[13], 9}, {NULL}};
const Segment SGI388060[] = {{&S21558[87], 13}, {NULL}};
const Segment SGI388061[] = {{&S21560[8], 3}, {NULL}};
const Segment SGI388062[] = {{&S21560[13], 9}, {NULL}};
const Segment SGI388063[] = {{&S21560[33], 15}, {NULL}};
const Segment SGI388064[] = {{&S21560[11], 1}, {NULL}};
const Segment SGI388065[] = {{&S21562[8], 3}, {&S21562[8], 3}, {NULL}};
const Segment SGI388066[] = {{&S21562[13], 9}, {&S21562[13], 9}, {NULL}};
const Segment SGI388067[] = {{&S21562[30], 15}, {NULL}};
const Segment SGI388068[] = {{&S21562[88], 13}, {&S21562[183], 17}, {NULL}};
const Segment SGI388069[] = {{&S21563[8], 3}, {NULL}};
const Segment SGI388070[] = {{&S21563[13], 9}, {NULL}};
const Segment SGI388071[] = {{&S21563[87], 19}, {NULL}};
const Segment SGI388072[] = {{&S21564[8], 3}, {NULL}};
const Segment SGI388073[] = {{&S21564[13], 9}, {NULL}};
const Segment SGI388074[] = {{&S21564[87], 9}, {NULL}};
const Segment SGI388075[] = {{&S21567[8], 3}, {NULL}};
const Segment SGI388076[] = {{&S21567[13], 9}, {NULL}};
const Segment SGI388077[] = {{&S21567[87], 23}, {NULL}};
const Segment SGI388078[] = {{&S21568[8], 3}, {NULL}};
const Segment SGI388079[] = {{&S21568[13], 9}, {NULL}};
const Segment SGI388080[] = {{&S21568[87], 18}, {NULL}};
const Segment SGI388081[] = {{&S21569[8], 3}, {NULL}};
const Segment SGI388082[] = {{&S21569[13], 9}, {NULL}};
const Segment SGI388083[] = {{&S21569[87], 7}, {NULL}};
const Segment SGI388084[] = {{&S21573[8], 3}, {NULL}};
const Segment SGI388085[] = {{&S21573[13], 9}, {NULL}};
const Segment SGI388086[] = {{&S21573[87], 10}, {NULL}};
const Segment SGI388087[] = {{&S21575[8], 3}, {NULL}};
const Segment SGI388088[] = {{&S21575[13], 9}, {NULL}};
const Segment SGI388089[] = {{&S21575[87], 23}, {NULL}};
const Segment SGI388090[] = {{&S21576[8], 3}, {NULL}};
const Segment SGI388091[] = {{&S21576[13], 9}, {NULL}};
const Segment SGI388092[] = {{&S21576[87], 23}, {NULL}};
const Segment SGI388093[] = {{&S21577[8], 3}, {NULL}};
const Segment SGI388094[] = {{&S21577[13], 9}, {NULL}};
const Segment SGI388095[] = {{&S21577[11], 1}, {NULL}};
const Segment SGI388096[] = {{&S21579[8], 3}, {NULL}};
const Segment SGI388097[] = {{&S21579[13], 9}, {NULL}};
const Segment SGI388098[] = {{&S21579[85], 35}, {NULL}};
const Segment SGI388099[] = {{&S21581[8], 3}, {NULL}};
const Segment SGI388100[] = {{&S21581[13], 9}, {NULL}};
const Segment SGI388101[] = {{&S21581[87], 18}, {NULL}};
const Segment SGI388102[] = {{&S21583[8], 3}, {NULL}};
const Segment SGI388103[] = {{&S21583[13], 9}, {NULL}};
const Segment SGI388104[] = {{&S21583[33], 15}, {NULL}};
const Segment SGI388105[] = {{&S21583[97], 13}, {NULL}};
const Segment SGI388106[] = {{&S21586[8], 3}, {NULL}};
const Segment SGI388107[] = {{&S21586[13], 9}, {NULL}};
const Segment SGI388108[] = {{&S21586[87], 14}, {NULL}};
const Segment SGI388109[] = {{&S21592[8], 3}, {NULL}};
const Segment SGI388110[] = {{&S21592[13], 9}, {NULL}};
const Segment SGI388111[] = {{&S21592[87], 26}, {NULL}};
const Segment SGI388112[] = {{&S82667[13], 10}, {&S82667[24], 9}, {&S82667[34], 11}, {&S82667[46], 13}, {&S82667[60], 19}, {&S82667[80], 8}, {NULL}};
const Segment SGI388113[] = {{&S82668[13], 10}, {&S82668[24], 9}, {&S82668[34], 11}, {&S82668[46], 13}, {&S82668[60], 38}, {&S82668[24], 9}, {&S82668[34], 11}, {&S82668[121], 4}, {&S82668[126], 8}, {NULL}};
const Segment SGI388114[] = {{&S22306[8], 3}, {NULL}};
const Segment SGI388115[] = {{&S22306[13], 14}, {NULL}};
const Segment SGI388116[] = {{&S22306[72], 17}, {NULL}};
const Segment SGI388117[] = {{&S22312[8], 3}, {NULL}};
const Segment SGI388118[] = {{&S22312[13], 7}, {NULL}};
const Segment SGI388119[] = {{&S22312[25], 14}, {NULL}};
const Segment SGI388120[] = {{&S22317[8], 3}, {NULL}};
const Segment SGI388121[] = {{&S22317[13], 14}, {NULL}};
const Segment SGI388122[] = {{&S22318[8], 3}, {NULL}};
const Segment SGI388123[] = {{&S22318[13], 14}, {NULL}};
const Segment SGI388124[] = {{&S22319[8], 3}, {NULL}};
const Segment SGI388125[] = {{&S22319[13], 14}, {NULL}};
const Segment SGI388126[] = {{&S22319[72], 22}, {NULL}};
const Segment SGI388127[] = {{&S22322[8], 3}, {NULL}};
const Segment SGI388128[] = {{&S22322[13], 7}, {NULL}};
const Segment SGI388129[] = {{&S22322[25], 14}, {NULL}};
const Segment SGI388130[] = {{&S22322[85], 22}, {NULL}};
const Segment SGI388131[] = {{&S22325[8], 3}, {NULL}};
const Segment SGI388132[] = {{&S22325[13], 14}, {NULL}};
const Segment SGI388133[] = {{&S22325[72], 16}, {NULL}};
const Segment SGI388134[] = {{&S82957[13], 10}, {&S82957[24], 9}, {&S82957[34], 11}, {&S82957[46], 13}, {NULL}};
const Segment SGI388135[] = {{&S19153[8], 3}, {NULL}};
const Segment SGI388136[] = {{&S19153[13], 7}, {NULL}};
const Segment SGI388137[] = {{&S19153[25], 14}, {NULL}};
const Segment SGI388138[] = {{&S19168[8], 3}, {NULL}};
const Segment SGI388139[] = {{&S19168[13], 14}, {NULL}};
const Segment SGI388140[] = {{&S19171[8], 3}, {NULL}};
const Segment SGI388141[] = {{&S19171[13], 7}, {NULL}};
const Segment SGI388142[] = {{&S19171[25], 14}, {NULL}};
const Segment SGI388143[] = {{&S83294[13], 10}, {&S83294[24], 9}, {&S83294[34], 11}, {&S83294[46], 14}, {&S83294[61], 104}, {NULL}};
const Segment SGI388144[] = {{&S83297[13], 10}, {&S83297[24], 9}, {&S83297[34], 11}, {&S83297[46], 14}, {&S83297[61], 15}, {&S83297[77], 29}, {NULL}};
const Segment SGI388145[] = {{&S83487[13], 10}, {&S83487[24], 9}, {&S83487[34], 11}, {&S83487[46], 14}, {&S83487[61], 15}, {&S83487[77], 29}, {NULL}};
const Segment SGI388146[] = {{&S83845[13], 10}, {&S83845[24], 9}, {&S83845[34], 11}, {&S83845[46], 14}, {&S83845[61], 34}, {&S83845[96], 9}, {NULL}};
const Segment SGI388147[] = {{&S83846[13], 10}, {&S83846[24], 9}, {&S83846[34], 11}, {&S83846[46], 14}, {&S83846[61], 34}, {&S83846[96], 9}, {NULL}};
const Segment SGI388148[] = {{&S19879[8], 3}, {NULL}};
const Segment SGI388149[] = {{&S19879[13], 9}, {NULL}};
const Segment SGI388150[] = {{&S19879[33], 13}, {NULL}};
const Segment SGI388151[] = {{&S19943[8], 3}, {&S19943[83], 3}, {NULL}};
const Segment SGI388152[] = {{&S19943[13], 9}, {NULL}};
const Segment SGI388153[] = {{&S19943[33], 13}, {NULL}};
const Segment SGI388154[] = {{&S84110[13], 10}, {&S84110[24], 9}, {&S84110[34], 11}, {&S84110[46], 14}, {&S84110[61], 15}, {&S84110[77], 20}, {NULL}};
const Segment SGI388155[] = {{&S84111[13], 10}, {&S84111[24], 9}, {&S84111[34], 11}, {&S84111[46], 14}, {&S84111[61], 15}, {&S84111[77], 22}, {NULL}};
const Segment SGI388156[] = {{&S20285[8], 3}, {NULL}};
const Segment SGI388157[] = {{&S20285[13], 7}, {NULL}};
const Segment SGI388158[] = {{&S20285[25], 14}, {&S20285[25], 14}, {NULL}};
const Segment SGI388159[] = {{&S21009[8], 3}, {NULL}};
const Segment SGI388160[] = {{&S85237[13], 10}, {&S85237[24], 9}, {&S85237[34], 11}, {&S85237[46], 13}, {&S85237[60], 4}, {&S85237[65], 54}, {NULL}};
const Segment SGI388161[] = {{&S85238[13], 10}, {&S85238[24], 9}, {&S85238[34], 11}, {&S85238[46], 13}, {&S85238[60], 4}, {&S85238[65], 8}, {&S85238[74], 54}, {NULL}};
const Segment SGI388162[] = {{&S21526[8], 3}, {NULL}};
const Segment SGI388163[] = {{&S21526[13], 9}, {NULL}};
const Segment SGI388164[] = {{&S21526[33], 13}, {NULL}};
const Segment SGI388165[] = {{&S85545[13], 10}, {&S85545[24], 9}, {&S85545[34], 11}, {&S85545[46], 14}, {&S85545[61], 85}, {NULL}};
const Segment SGI388166[] = {{&S85546[13], 10}, {&S85546[24], 9}, {&S85546[34], 11}, {&S85546[46], 14}, {&S85546[61], 68}, {NULL}};
const Segment SGI388167[] = {{&S85547[13], 10}, {&S85547[24], 9}, {&S85547[34], 11}, {&S85547[46], 14}, {&S85547[61], 75}, {NULL}};
const Segment SGI388168[] = {{&S85548[13], 10}, {&S85548[24], 9}, {&S85548[34], 11}, {&S85548[46], 14}, {&S85548[61], 85}, {NULL}};
const Segment SGI388169[] = {{&S85549[13], 10}, {&S85549[24], 9}, {&S85549[34], 11}, {&S85549[46], 14}, {&S85549[61], 68}, {NULL}};
const Segment SGI388170[] = {{&S85550[13], 10}, {&S85550[24], 9}, {&S85550[34], 11}, {&S85550[46], 14}, {&S85550[61], 85}, {NULL}};
const Segment SGI388171[] = {{&S85551[13], 10}, {&S85551[24], 9}, {&S85551[34], 11}, {&S85551[46], 14}, {&S85551[61], 85}, {NULL}};
const Segment SGI388172[] = {{&S85769[13], 10}, {&S85769[24], 9}, {&S85769[34], 11}, {&S85769[46], 14}, {&S85769[61], 25}, {NULL}};
const Segment SGI388173[] = {{&S21957[8], 3}, {NULL}};
const Segment SGI388174[] = {{&S21957[13], 3}, {NULL}};
const Segment SGI388175[] = {{&S21957[13], 3}, {&S21957[21], 10}, {NULL}};
const Segment SGI388176[] = {{&S21957[59], 16}, {NULL}};
const Segment SGI388177[] = {{&S21959[8], 3}, {NULL}};
const Segment SGI388178[] = {{&S21959[13], 3}, {NULL}};
const Segment SGI388179[] = {{&S21959[57], 16}, {NULL}};
const Segment SGI388180[] = {{&S21960[8], 3}, {NULL}};
const Segment SGI388181[] = {{&S21960[13], 3}, {NULL}};
const Segment SGI388182[] = {{&S21960[62], 16}, {NULL}};
const Segment SGI388183[] = {{&S86107[13], 10}, {&S86107[24], 9}, {&S86107[34], 11}, {&S86107[46], 14}, {&S86107[61], 16}, {NULL}};
const Segment SGI388184[] = {{&S86151[13], 7}, {&S86151[21], 2}, {&S86151[24], 15}, {&S86151[40], 6}, {&S86151[47], 14}, {&S86151[62], 27}, {NULL}};
const Segment SGI388185[] = {{&S22298[8], 3}, {NULL}};
const Segment SGI388186[] = {{&S22298[13], 7}, {NULL}};
const Segment SGI388187[] = {{&S22298[25], 14}, {NULL}};
const Segment SGI388188[] = {{&S22320[8], 3}, {NULL}};
const Segment SGI388189[] = {{&S22320[13], 14}, {NULL}};
const Segment SGI388190[] = {{&S22323[8], 3}, {NULL}};
const Segment SGI388191[] = {{&S22323[13], 7}, {NULL}};
const Segment SGI388192[] = {{&S86526[13], 10}, {&S86526[24], 9}, {&S86526[34], 11}, {&S86526[46], 4}, {&S86526[51], 8}, {NULL}};
const Segment SGI388193[] = {{&S86527[13], 10}, {&S86527[24], 9}, {&S86527[34], 11}, {NULL}};
const Segment SGI388194[] = {{&S87409[13], 10}, {&S87409[24], 10}, {&S87409[35], 11}, {&S87409[47], 4}, {&S87409[52], 54}, {NULL}};
const Segment SGI388195[] = {{&S87410[13], 10}, {&S87410[24], 10}, {&S87410[35], 11}, {&S87410[47], 4}, {&S87410[52], 54}, {NULL}};
const Segment SGI388196[] = {{&S87411[13], 10}, {&S87411[24], 10}, {&S87411[35], 11}, {&S87411[47], 4}, {&S87411[52], 54}, {NULL}};
const Segment SGI388197[] = {{&S87667[13], 10}, {&S87667[24], 9}, {&S87667[34], 11}, {&S87667[46], 14}, {&S87667[61], 25}, {NULL}};
const Segment SGI388198[] = {{&S26850[8], 3}, {NULL}};
const Segment SGI388199[] = {{&S26850[13], 7}, {NULL}};
const Segment SGI388200[] = {{&S26850[25], 14}, {NULL}};
const Segment SGI388201[] = {{&S26850[95], 11}, {NULL}};
const Segment SGI388202[] = {{&S87909[13], 10}, {&S87909[24], 9}, {&S87909[34], 11}, {&S87909[46], 14}, {&S87909[61], 77}, {NULL}};
const Segment SGI388203[] = {{&S88286[13], 10}, {&S88286[24], 9}, {&S88286[34], 11}, {&S88286[46], 14}, {&S88286[61], 15}, {&S88286[77], 10}, {NULL}};
const Segment SGI388204[] = {{&S88699[13], 10}, {&S88699[24], 9}, {&S88699[34], 11}, {&S88699[46], 13}, {&S88699[60], 10}, {NULL}};
const Segment SGI388205[] = {{&S88966[13], 10}, {&S88966[24], 9}, {&S88966[34], 11}, {&S88966[46], 14}, {&S88966[61], 15}, {&S88966[77], 39}, {NULL}};
const Segment SGI388206[] = {{&S90033[13], 10}, {&S90033[24], 10}, {&S90033[35], 11}, {&S90033[47], 4}, {&S90033[52], 19}, {NULL}};
const Segment SGI388207[] = {{&S90174[13], 10}, {&S90174[24], 9}, {&S90174[34], 11}, {&S90174[46], 13}, {&S90174[60], 8}, {&S90174[69], 4}, {&S90174[74], 13}, {&S90174[88], 28}, {NULL}};
const Segment SGI388208[] = {{&S90356[13], 10}, {&S90356[24], 10}, {&S90356[35], 11}, {&S90356[47], 4}, {&S90356[52], 8}, {&S90356[61], 19}, {NULL}};
const Segment SGI388209[] = {{&S90358[13], 10}, {&S90358[24], 10}, {&S90358[35], 11}, {&S90358[47], 4}, {&S90358[52], 8}, {&S90358[61], 15}, {NULL}};
const Segment SGI388210[] = {{&S91093[13], 10}, {&S91093[24], 9}, {&S91093[34], 11}, {&S91093[46], 13}, {&S91093[60], 11}, {&S91093[72], 8}, {&S91093[81], 14}, {NULL}};
const Segment SGI388211[] = {{&S91278[13], 10}, {&S91278[24], 9}, {&S91278[34], 11}, {&S91278[46], 14}, {&S91278[61], 60}, {NULL}};
const Segment SGI388212[] = {{&S26857[8], 3}, {NULL}};
const Segment SGI388213[] = {{&S26857[13], 7}, {NULL}};
const Segment SGI388214[] = {{&S26857[25], 14}, {NULL}};
const Segment SGI388215[] = {{&S91451[13], 10}, {&S91451[24], 9}, {&S91451[34], 11}, {&S91451[46], 14}, {&S91451[61], 5}, {&S91451[67], 8}, {&S91451[76], 35}, {NULL}};
const Segment SGI388216[] = {{&S91452[13], 10}, {&S91452[24], 9}, {&S91452[34], 11}, {&S91452[46], 14}, {&S91452[61], 5}, {&S91452[67], 8}, {&S91452[76], 13}, {NULL}};
const Segment SGI388217[] = {{&S91453[13], 10}, {&S91453[24], 9}, {&S91453[34], 11}, {&S91453[46], 14}, {&S91453[61], 5}, {&S91453[67], 8}, {&S91453[76], 35}, {NULL}};
const Segment SGI388218[] = {{&S91454[13], 10}, {&S91454[24], 9}, {&S91454[34], 11}, {&S91454[46], 14}, {&S91454[61], 5}, {&S91454[67], 8}, {&S91454[76], 14}, {NULL}};
const Segment SGI388219[] = {{&S91455[13], 10}, {&S91455[24], 9}, {&S91455[34], 11}, {&S91455[46], 14}, {&S91455[61], 5}, {&S91455[67], 8}, {&S91455[76], 53}, {&S91455[130], 15}, {&S91455[146], 12}, {NULL}};
const Segment SGI388220[] = {{&S91471[13], 10}, {&S91471[24], 9}, {&S91471[34], 11}, {&S91471[46], 14}, {&S91471[61], 12}, {NULL}};
const Segment SGI388221[] = {{&S26942[8], 3}, {NULL}};
const Segment SGI388222[] = {{&S26942[13], 14}, {NULL}};
const Segment SGI388223[] = {{&S26942[29], 5}, {NULL}};
const Segment SGI388224[] = {{&S26943[8], 3}, {NULL}};
const Segment SGI388225[] = {{&S26943[13], 14}, {NULL}};
const Segment SGI388226[] = {{&S91669[13], 10}, {&S91669[24], 9}, {&S91669[34], 11}, {&S91669[46], 14}, {&S91669[61], 11}, {&S91669[73], 8}, {&S91669[82], 14}, {NULL}};
const Segment SGI388227[] = {{&S91670[13], 10}, {&S91670[24], 10}, {&S91670[35], 11}, {&S91670[47], 17}, {&S91670[65], 8}, {&S91670[74], 17}, {&S91670[65], 8}, {&S91670[60], 4}, {&S91670[65], 8}, {NULL}};
const Segment SGI388228[] = {{&S91671[13], 10}, {&S91671[24], 10}, {&S91671[35], 11}, {&S91671[47], 17}, {&S91671[65], 8}, {&S91671[60], 4}, {&S91671[65], 8}, {NULL}};
const Segment SGI388229[] = {{&S91684[13], 10}, {&S91684[24], 9}, {&S91684[34], 11}, {&S91684[46], 14}, {&S91684[61], 8}, {&S91684[70], 4}, {&S91684[75], 15}, {&S91684[91], 27}, {NULL}};
const Segment SGI388230[] = {{&S91713[13], 10}, {&S91713[24], 9}, {&S91713[34], 11}, {&S91713[46], 15}, {NULL}};
const Segment SGI388231[] = {{&S92075[13], 10}, {&S92075[24], 9}, {&S92075[34], 11}, {&S92075[46], 14}, {&S92075[61], 15}, {&S92075[77], 10}, {NULL}};
const Segment SGI388232[] = {{&S92485[13], 10}, {&S92485[24], 9}, {&S92485[34], 11}, {&S92485[46], 13}, {&S92485[60], 25}, {NULL}};
const Segment SGI388233[] = {{&S92491[13], 10}, {&S92491[24], 10}, {&S92491[35], 11}, {&S92491[47], 4}, {&S92491[52], 8}, {&S92491[61], 19}, {NULL}};
const Segment SGI388234[] = {{&S92823[13], 10}, {&S92823[24], 9}, {&S92823[34], 11}, {&S92823[46], 14}, {&S92823[61], 24}, {NULL}};
const Segment SGI388235[] = {{&S92824[13], 10}, {&S92824[24], 9}, {&S92824[34], 11}, {&S92824[46], 14}, {&S92824[61], 24}, {NULL}};
const Segment SGI388236[] = {{&S92825[13], 10}, {&S92825[24], 9}, {&S92825[34], 11}, {&S92825[46], 14}, {&S92825[61], 24}, {NULL}};
const Segment SGI388237[] = {{&S92826[13], 10}, {&S92826[24], 9}, {&S92826[34], 11}, {&S92826[46], 14}, {&S92826[61], 4}, {&S92826[66], 59}, {NULL}};
const Segment SGI388238[] = {{&S32230[8], 3}, {NULL}};
const Segment SGI388239[] = {{&S32230[13], 9}, {NULL}};
const Segment SGI388240[] = {{&S32230[33], 15}, {NULL}};
const Segment SGI388241[] = {{&S32231[8], 3}, {NULL}};
const Segment SGI388242[] = {{&S32231[13], 9}, {NULL}};
const Segment SGI388243[] = {{&S32231[33], 15}, {NULL}};
const Segment SGI388244[] = {{&S32232[8], 3}, {NULL}};
const Segment SGI388245[] = {{&S32232[13], 9}, {NULL}};
const Segment SGI388246[] = {{&S32232[33], 15}, {NULL}};
const Segment SGI388247[] = {{&S32233[8], 3}, {NULL}};
const Segment SGI388248[] = {{&S32233[13], 9}, {NULL}};
const Segment SGI388249[] = {{&S32233[33], 15}, {NULL}};
const Segment SGI388250[] = {{&S32234[8], 3}, {NULL}};
const Segment SGI388251[] = {{&S32234[13], 9}, {NULL}};
const Segment SGI388252[] = {{&S32234[33], 15}, {NULL}};
const Segment SGI388253[] = {{&S32236[8], 3}, {NULL}};
const Segment SGI388254[] = {{&S32236[13], 9}, {NULL}};
const Segment SGI388255[] = {{&S32236[33], 15}, {NULL}};
const Segment SGI388256[] = {{&S32237[8], 3}, {NULL}};
const Segment SGI388257[] = {{&S32237[13], 9}, {NULL}};
const Segment SGI388258[] = {{&S32238[8], 3}, {NULL}};
const Segment SGI388259[] = {{&S32238[13], 9}, {NULL}};
const Segment SGI388260[] = {{&S32238[87], 19}, {NULL}};
const Segment SGI388261[] = {{&S32243[8], 3}, {NULL}};
const Segment SGI388262[] = {{&S32243[13], 9}, {NULL}};
const Segment SGI388263[] = {{&S32243[33], 15}, {NULL}};
const Segment SGI388264[] = {{&S32243[11], 1}, {NULL}};
const Segment SGI388265[] = {{&S32245[8], 3}, {NULL}};
const Segment SGI388266[] = {{&S32245[13], 9}, {NULL}};
const Segment SGI388267[] = {{&S32245[87], 19}, {NULL}};
const Segment SGI388268[] = {{&S32247[8], 3}, {NULL}};
const Segment SGI388269[] = {{&S32247[13], 9}, {NULL}};
const Segment SGI388270[] = {{&S32247[87], 18}, {NULL}};
const Segment SGI388271[] = {{&S32249[8], 3}, {NULL}};
const Segment SGI388272[] = {{&S32249[13], 9}, {NULL}};
const Segment SGI388273[] = {{&S32249[33], 15}, {NULL}};
const Segment SGI388274[] = {{&S32250[8], 3}, {NULL}};
const Segment SGI388275[] = {{&S32250[13], 9}, {NULL}};
const Segment SGI388276[] = {{&S32250[11], 1}, {NULL}};
const Segment SGI388277[] = {{&S32253[8], 3}, {NULL}};
const Segment SGI388278[] = {{&S32253[13], 9}, {NULL}};
const Segment SGI388279[] = {{&S32253[87], 22}, {NULL}};
const Segment SGI388280[] = {{&S32258[8], 3}, {NULL}};
const Segment SGI388281[] = {{&S32258[13], 9}, {NULL}};
const Segment SGI388282[] = {{&S32258[87], 8}, {NULL}};
const Segment SGI388283[] = {{&S32259[8], 3}, {NULL}};
const Segment SGI388284[] = {{&S32259[13], 9}, {NULL}};
const Segment SGI388285[] = {{&S32259[87], 22}, {NULL}};
const Segment SGI388286[] = {{&S32262[8], 3}, {NULL}};
const Segment SGI388287[] = {{&S32262[13], 9}, {NULL}};
const Segment SGI388288[] = {{&S32262[87], 18}, {NULL}};
const Segment SGI388289[] = {{&S32265[8], 3}, {NULL}};
const Segment SGI388290[] = {{&S32265[13], 9}, {NULL}};
const Segment SGI388291[] = {{&S32265[33], 15}, {NULL}};
const Segment SGI388292[] = {{&S32265[95], 7}, {NULL}};
const Segment SGI388293[] = {{&S32267[8], 3}, {NULL}};
const Segment SGI388294[] = {{&S32267[13], 9}, {NULL}};
const Segment SGI388295[] = {{&S32267[87], 18}, {NULL}};
const Segment SGI388296[] = {{&S32268[8], 3}, {NULL}};
const Segment SGI388297[] = {{&S32268[13], 9}, {NULL}};
const Segment SGI388298[] = {{&S32268[87], 10}, {NULL}};
const Segment SGI388299[] = {{&S32269[8], 3}, {NULL}};
const Segment SGI388300[] = {{&S32269[13], 9}, {NULL}};
const Segment SGI388301[] = {{&S32269[87], 16}, {NULL}};
const Segment SGI388302[] = {{&S32272[8], 3}, {NULL}};
const Segment SGI388303[] = {{&S32272[13], 9}, {NULL}};
const Segment SGI388304[] = {{&S32272[87], 23}, {NULL}};
const Segment SGI388305[] = {{&S32274[8], 3}, {NULL}};
const Segment SGI388306[] = {{&S32274[13], 9}, {NULL}};
const Segment SGI388307[] = {{&S32274[87], 23}, {NULL}};
const Segment SGI388308[] = {{&S32275[8], 3}, {NULL}};
const Segment SGI388309[] = {{&S32275[13], 9}, {NULL}};
const Segment SGI388310[] = {{&S32275[87], 22}, {NULL}};
const Segment SGI388311[] = {{&S32277[8], 3}, {NULL}};
const Segment SGI388312[] = {{&S32277[13], 9}, {NULL}};
const Segment SGI388313[] = {{&S32277[87], 7}, {NULL}};
const Segment SGI388314[] = {{&S32278[8], 3}, {NULL}};
const Segment SGI388315[] = {{&S32278[13], 9}, {NULL}};
const Segment SGI388316[] = {{&S32278[87], 23}, {NULL}};
const Segment SGI388317[] = {{&S32279[8], 3}, {NULL}};
const Segment SGI388318[] = {{&S32279[13], 9}, {NULL}};
const Segment SGI388319[] = {{&S32279[87], 23}, {NULL}};
const Segment SGI388320[] = {{&S93273[13], 10}, {&S93273[24], 9}, {&S93273[34], 11}, {&S93273[46], 14}, {&S93273[61], 15}, {&S93273[77], 17}, {NULL}};
const Segment SGI388321[] = {{&S93536[13], 10}, {&S93536[24], 10}, {&S93536[35], 11}, {&S93536[47], 4}, {&S93536[52], 19}, {NULL}};
const Segment SGI388322[] = {{&S93962[13], 10}, {&S93962[24], 9}, {&S93962[34], 11}, {&S93962[46], 14}, {&S93962[61], 15}, {&S93962[77], 10}, {NULL}};
const Segment SGI388323[] = {{&S93963[13], 10}, {&S93963[24], 9}, {&S93963[34], 11}, {&S93963[46], 14}, {&S93963[61], 15}, {&S93963[77], 20}, {NULL}};
const Segment SGI388324[] = {{&S93964[13], 10}, {&S93964[24], 9}, {&S93964[34], 11}, {&S93964[46], 14}, {&S93964[61], 15}, {&S93964[77], 19}, {NULL}};
const Segment SGI388325[] = {{&S93965[13], 10}, {&S93965[24], 9}, {&S93965[34], 11}, {&S93965[46], 14}, {&S93965[61], 15}, {&S93965[77], 19}, {NULL}};
const Segment SGI388326[] = {{&S93966[13], 10}, {&S93966[24], 9}, {&S93966[34], 11}, {&S93966[46], 14}, {&S93966[61], 15}, {&S93966[77], 19}, {NULL}};
const Segment SGI388327[] = {{&S94185[13], 10}, {&S94185[24], 9}, {&S94185[34], 11}, {&S94185[46], 14}, {&S94185[61], 18}, {NULL}};
const Segment SGI388328[] = {{&S94194[13], 10}, {&S94194[24], 9}, {&S94194[34], 11}, {&S94194[46], 13}, {&S94194[60], 16}, {NULL}};
const Segment SGI388329[] = {{&S94235[13], 10}, {&S94235[24], 10}, {&S94235[35], 11}, {&S94235[47], 4}, {&S94235[52], 8}, {&S94235[61], 19}, {NULL}};
const Segment SGI388330[] = {{&S94390[13], 10}, {&S94390[24], 9}, {&S94390[34], 11}, {&S94390[46], 14}, {&S94390[61], 15}, {&S94390[77], 10}, {NULL}};
const Segment SGI388331[] = {{&S94391[13], 10}, {&S94391[24], 9}, {&S94391[34], 11}, {&S94391[46], 14}, {&S94391[61], 15}, {&S94391[77], 33}, {NULL}};
const Segment SGI388332[] = {{&S94408[13], 10}, {&S94408[24], 10}, {&S94408[35], 11}, {&S94408[47], 4}, {&S94408[52], 56}, {NULL}};
const Segment SGI388333[] = {{&S94662[13], 10}, {&S94662[24], 10}, {&S94662[35], 11}, {&S94662[47], 10}, {&S94662[58], 3}, {&S94662[62], 7}, {NULL}};
const Segment SGI388334[] = {{&S94663[13], 10}, {&S94663[24], 10}, {&S94663[35], 11}, {&S94663[47], 13}, {&S94663[61], 4}, {&S94663[66], 8}, {NULL}};
const Segment SGI388335[] = {{&S94664[13], 10}, {&S94664[24], 10}, {&S94664[35], 11}, {&S94664[47], 10}, {&S94664[58], 3}, {&S94664[62], 7}, {&S94664[70], 4}, {&S94664[75], 8}, {NULL}};
const Segment SGI388336[] = {{&S94665[13], 10}, {&S94665[24], 10}, {&S94665[35], 11}, {&S94665[47], 10}, {&S94665[58], 4}, {&S94665[63], 8}, {NULL}};
const Segment SGI388337[] = {{&S94877[13], 10}, {&S94877[24], 9}, {&S94877[34], 11}, {&S94877[46], 14}, {&S94877[61], 4}, {&S94877[66], 54}, {NULL}};
const Segment SGI388338[] = {{&S94914[13], 10}, {&S94914[24], 10}, {&S94914[35], 11}, {&S94914[47], 4}, {&S94914[52], 22}, {NULL}};
const Segment SGI388339[] = {{&S94915[13], 10}, {&S94915[24], 10}, {&S94915[35], 11}, {&S94915[47], 4}, {&S94915[52], 22}, {NULL}};
const Segment SGI388340[] = {{&S94916[13], 10}, {&S94916[24], 10}, {&S94916[35], 11}, {&S94916[47], 4}, {&S94916[52], 22}, {NULL}};
const Segment SGI388341[] = {{&S94917[13], 10}, {&S94917[24], 10}, {&S94917[35], 11}, {&S94917[47], 4}, {&S94917[52], 22}, {NULL}};
const Segment SGI388342[] = {{&S95710[13], 10}, {&S95710[24], 10}, {&S95710[35], 11}, {&S95710[47], 16}, {&S95710[64], 4}, {&S95710[69], 8}, {NULL}};
const Segment SGI388343[] = {{&S95711[13], 10}, {&S95711[24], 10}, {&S95711[35], 11}, {&S95711[47], 7}, {&S95711[55], 9}, {&S95711[65], 4}, {&S95711[70], 8}, {NULL}};
const Segment SGI388344[] = {{&S95950[13], 10}, {&S95950[24], 9}, {&S95950[34], 11}, {&S95950[46], 14}, {&S95950[61], 18}, {NULL}};
const Segment SGI388345[] = {{&S95960[13], 10}, {&S95960[24], 9}, {&S95960[34], 11}, {&S95960[46], 14}, {&S95960[61], 15}, {&S95960[77], 10}, {NULL}};
const Segment SGI388346[] = {{&S95968[13], 10}, {&S95968[24], 9}, {&S95968[34], 11}, {&S95968[46], 14}, {&S95968[61], 15}, {&S95968[77], 13}, {NULL}};
const Segment SGI388347[] = {{&S95969[13], 10}, {&S95969[24], 9}, {&S95969[34], 11}, {&S95969[46], 14}, {&S95969[61], 15}, {&S95969[77], 10}, {NULL}};
const Segment SGI388348[] = {{&S96132[13], 10}, {&S96132[24], 9}, {&S96132[34], 11}, {&S96132[46], 14}, {&S96132[61], 34}, {&S96132[96], 9}, {NULL}};
const Segment SGI388349[] = {{&S96318[13], 10}, {&S96318[24], 10}, {&S96318[35], 11}, {&S96318[47], 11}, {&S96318[59], 8}, {NULL}};
const Segment SGI388350[] = {{&S31969[8], 3}, {NULL}};
const Segment SGI388351[] = {{&S31969[13], 14}, {NULL}};
const Segment SGI388352[] = {{&S31969[49], 5}, {NULL}};
const Segment SGI388353[] = {{&S31970[8], 3}, {NULL}};
const Segment SGI388354[] = {{&S31970[13], 14}, {NULL}};
const Segment SGI388355[] = {{&S31970[29], 5}, {NULL}};
const Segment SGI388356[] = {{&S31970[54], 5}, {NULL}};
const Segment SGI388357[] = {{&S32271[8], 3}, {NULL}};
const Segment SGI388358[] = {{&S32271[13], 9}, {NULL}};
const Segment SGI388359[] = {{&S32271[33], 13}, {NULL}};
const Segment SGI388360[] = {{&S96913[13], 10}, {&S96913[24], 9}, {&S96913[34], 11}, {&S96913[46], 14}, {&S96913[61], 4}, {&S96913[66], 8}, {&S96913[75], 11}, {&S96913[87], 15}, {NULL}};
const Segment SGI388361[] = {{&S96931[13], 10}, {&S96931[24], 10}, {&S96931[35], 11}, {&S96931[47], 10}, {&S96931[58], 8}, {NULL}};
const Segment SGI388362[] = {{&S97652[13], 10}, {&S97652[24], 10}, {&S97652[35], 11}, {&S97652[47], 4}, {&S97652[52], 9}, {&S97652[62], 8}, {NULL}};
const Segment SGI388363[] = {{&S97653[13], 10}, {&S97653[24], 10}, {&S97653[35], 11}, {&S97653[47], 4}, {&S97653[52], 6}, {&S97653[59], 7}, {&S97653[47], 4}, {&S97653[72], 9}, {&S97653[82], 8}, {NULL}};
const Segment SGI388364[] = {{&S33190[8], 3}, {NULL}};
const Segment SGI388365[] = {{&S33190[13], 7}, {NULL}};
const Segment SGI388366[] = {{&S33190[25], 14}, {NULL}};
const Segment SGI388367[] = {{&S33190[66], 16}, {NULL}};
const Segment SGI388368[] = {{&S33210[8], 3}, {NULL}};
const Segment SGI388369[] = {{&S33210[22], 5}, {NULL}};
const Segment SGI388370[] = {{&S33211[8], 3}, {NULL}};
const Segment SGI388371[] = {{&S33211[13], 7}, {NULL}};
const Segment SGI388372[] = {{&S33211[25], 5}, {NULL}};
const Segment SGI388373[] = {{&S33212[8], 3}, {NULL}};
const Segment SGI388374[] = {{&S33212[13], 7}, {NULL}};
const Segment SGI388375[] = {{&S33212[25], 5}, {NULL}};
const Segment SGI388376[] = {{&S33213[8], 3}, {NULL}};
const Segment SGI388377[] = {{&S33213[13], 7}, {NULL}};
const Segment SGI388378[] = {{&S98045[13], 10}, {&S98045[24], 9}, {&S98045[34], 11}, {&S98045[46], 14}, {&S98045[61], 12}, {&S98045[74], 8}, {NULL}};
const Segment SGI388379[] = {{&S98046[13], 10}, {&S98046[24], 9}, {&S98046[34], 11}, {&S98046[46], 14}, {&S98046[61], 21}, {NULL}};
const Segment SGI388380[] = {{&S98510[13], 10}, {&S98510[24], 9}, {&S98510[34], 11}, {&S98510[46], 14}, {&S98510[61], 17}, {NULL}};
const Segment SGI388381[] = {{&S98511[13], 10}, {&S98511[24], 9}, {&S98511[34], 11}, {&S98511[46], 14}, {&S98511[61], 17}, {NULL}};
const Segment SGI388382[] = {{&S30188[39], 4}, {NULL}};
const Segment SGI388383[] = {{&S30188[8], 3}, {NULL}};
const Segment SGI388384[] = {{&S30188[13], 14}, {NULL}};
const Segment SGI388385[] = {{&S33982[8], 3}, {NULL}};
const Segment SGI388386[] = {{&S33982[13], 7}, {NULL}};
const Segment SGI388387[] = {{&S33982[25], 14}, {NULL}};
const Segment SGI388388[] = {{&S33983[8], 3}, {NULL}};
const Segment SGI388389[] = {{&S33983[13], 14}, {NULL}};
const Segment SGI388390[] = {{&S33984[8], 3}, {NULL}};
const Segment SGI388391[] = {{&S33984[13], 14}, {NULL}};
const Segment SGI388392[] = {{&S33984[29], 5}, {NULL}};
const Segment SGI388393[] = {{&S33986[8], 3}, {NULL}};
const Segment SGI388394[] = {{&S33986[13], 14}, {NULL}};
const Segment SGI388395[] = {{&S33986[49], 5}, {NULL}};
const Segment SGI388396[] = {{&S99021[13], 10}, {&S99021[24], 10}, {&S99021[35], 11}, {&S99021[47], 4}, {&S99021[52], 56}, {NULL}};
const Segment SGI388397[] = {{&S99077[13], 10}, {&S99077[24], 10}, {&S99077[35], 11}, {&S99077[47], 4}, {&S99077[52], 22}, {NULL}};
const Segment SGI388398[] = {{&S99553[13], 10}, {&S99553[24], 9}, {&S99553[34], 11}, {&S99553[46], 11}, {&S99553[58], 8}, {&S99553[67], 17}, {&S99553[85], 16}, {NULL}};
const Segment SGI388399[] = {{&S99554[13], 10}, {&S99554[24], 9}, {&S99554[34], 11}, {&S99554[46], 11}, {&S99554[58], 8}, {&S99554[67], 21}, {&S99554[89], 16}, {NULL}};
const Segment SGI388400[] = {{&S99555[13], 10}, {&S99555[24], 9}, {&S99555[34], 11}, {&S99555[46], 11}, {&S99555[58], 8}, {&S99555[67], 21}, {&S99555[89], 16}, {NULL}};
const Segment SGI388401[] = {{&S100142[13], 10}, {&S100142[24], 9}, {&S100142[34], 11}, {&S100142[46], 14}, {&S100142[61], 15}, {&S100142[77], 14}, {NULL}};
const Segment SGI388402[] = {{&S100962[13], 10}, {&S100962[24], 9}, {&S100962[34], 11}, {&S100962[46], 14}, {&S100962[61], 54}, {NULL}};
const Segment SGI388403[] = {{&S100971[13], 10}, {&S100971[24], 9}, {&S100971[34], 11}, {&S100971[46], 13}, {&S100971[60], 4}, {&S100971[65], 56}, {NULL}};
const Segment SGI388404[] = {{&S100979[13], 10}, {&S100979[24], 9}, {&S100979[34], 11}, {&S100979[46], 14}, {&S100979[61], 15}, {&S100979[77], 20}, {NULL}};
const Segment SGI388405[] = {{&S100980[13], 10}, {&S100980[24], 9}, {&S100980[34], 11}, {&S100980[46], 14}, {&S100980[61], 15}, {&S100980[77], 20}, {NULL}};
const Segment SGI388406[] = {{&S101148[13], 10}, {&S101148[24], 9}, {&S101148[34], 11}, {&S101148[46], 14}, {&S101148[61], 54}, {NULL}};
const Segment SGI388407[] = {{&S101172[13], 10}, {&S101172[24], 9}, {&S101172[34], 11}, {&S101172[46], 13}, {&S101172[60], 12}, {&S101172[73], 8}, {NULL}};
const Segment SGI388408[] = {{&S101184[13], 10}, {&S101184[24], 9}, {&S101184[34], 11}, {&S101184[46], 14}, {&S101184[61], 21}, {NULL}};
const Segment SGI388409[] = {{&S101285[13], 10}, {&S101285[24], 9}, {&S101285[34], 11}, {&S101285[46], 14}, {&S101285[61], 4}, {&S101285[66], 8}, {&S101285[75], 54}, {NULL}};
const Segment SGI388410[] = {{&S101286[13], 10}, {&S101286[24], 9}, {&S101286[34], 11}, {&S101286[46], 14}, {&S101286[61], 4}, {&S101286[66], 8}, {&S101286[75], 54}, {NULL}};
const Segment SGI388411[] = {{&S101336[13], 10}, {&S101336[24], 9}, {&S101336[34], 11}, {&S101336[46], 14}, {&S101336[61], 15}, {&S101336[77], 10}, {NULL}};
const Segment SGI388412[] = {{&S101426[13], 10}, {&S101426[24], 9}, {&S101426[34], 11}, {&S101426[46], 13}, {&S101426[60], 13}, {NULL}};
const Segment SGI388413[] = {{&S101429[13], 10}, {&S101429[24], 9}, {&S101429[34], 11}, {&S101429[46], 14}, {&S101429[61], 12}, {NULL}};
const Segment SGI388414[] = {{&S101436[13], 10}, {&S101436[24], 10}, {&S101436[35], 11}, {&S101436[47], 12}, {&S101436[60], 4}, {&S101436[65], 8}, {NULL}};
const Segment SGI388415[] = {{&S101440[13], 10}, {&S101440[24], 10}, {&S101440[35], 11}, {&S101440[47], 16}, {&S101440[64], 8}, {&S101440[47], 4}, {NULL}};
const Segment SGI388416[] = {{&S101469[13], 10}, {&S101469[24], 9}, {&S101469[34], 11}, {&S101469[46], 13}, {&S101469[60], 18}, {NULL}};
const Segment SGI388417[] = {{&S101501[13], 10}, {&S101501[24], 9}, {&S101501[34], 11}, {&S101501[46], 14}, {&S101501[61], 15}, {&S101501[77], 8}, {NULL}};
const Segment SGI388418[] = {{&S101624[13], 10}, {&S101624[24], 9}, {&S101624[34], 11}, {&S101624[46], 13}, {&S101624[60], 17}, {NULL}};
const Segment SGI388419[] = {{&S101625[13], 10}, {&S101625[24], 9}, {&S101625[34], 11}, {&S101625[46], 13}, {&S101625[60], 14}, {NULL}};
const Segment SGI388420[] = {{&S101633[13], 10}, {&S101633[24], 9}, {&S101633[34], 11}, {&S101633[46], 13}, {&S101633[60], 4}, {&S101633[65], 8}, {&S101633[74], 10}, {NULL}};
const Segment SGI388421[] = {{&S101693[13], 7}, {&S101693[21], 2}, {&S101693[24], 15}, {&S101693[40], 6}, {&S101693[47], 14}, {&S101693[62], 21}, {NULL}};
const Segment SGI388422[] = {{&S40523[8], 3}, {NULL}};
const Segment SGI388423[] = {{&S40523[13], 14}, {NULL}};
const Segment SGI388424[] = {{&S40525[8], 3}, {NULL}};
const Segment SGI388425[] = {{&S40525[13], 14}, {NULL}};
const Segment SGI388426[] = {{&S101790[13], 10}, {&S101790[24], 9}, {&S101790[34], 11}, {&S101790[46], 13}, {&S101790[60], 24}, {NULL}};
const Segment SGI388427[] = {{&S101798[13], 10}, {&S101798[24], 9}, {&S101798[34], 11}, {&S101798[46], 14}, {&S101798[61], 39}, {NULL}};
const Segment SGI388428[] = {{&S101799[13], 10}, {&S101799[24], 9}, {&S101799[34], 11}, {&S101799[46], 14}, {&S101799[61], 16}, {NULL}};
const Segment SGI388429[] = {{&S101800[13], 10}, {&S101800[24], 9}, {&S101800[34], 11}, {&S101800[46], 14}, {&S101800[61], 39}, {NULL}};
const Segment SGI388430[] = {{&S101806[13], 10}, {&S101806[24], 9}, {&S101806[34], 11}, {&S101806[46], 14}, {&S101806[61], 13}, {NULL}};
const Segment SGI388431[] = {{&S101808[13], 22}, {&S101808[36], 1}, {&S101808[38], 18}, {&S101808[57], 39}, {&S101808[97], 7}, {&S101808[105], 7}, {&S101808[113], 19}, {&S101808[133], 9}, {&S101808[143], 11}, {&S101808[155], 14}, {&S101808[170], 15}, {NULL}};
const Segment SGI388432[] = {{&S101831[13], 10}, {&S101831[24], 9}, {&S101831[34], 11}, {&S101831[46], 11}, {&S101831[58], 11}, {&S101831[70], 8}, {NULL}};
const Segment SGI388433[] = {{&S101976[13], 10}, {&S101976[24], 9}, {&S101976[34], 11}, {&S101976[46], 13}, {&S101976[60], 23}, {NULL}};
const Segment SGI388434[] = {{&S102012[13], 10}, {&S102012[24], 9}, {&S102012[34], 11}, {&S102012[46], 13}, {&S102012[60], 54}, {NULL}};
const Segment SGI388435[] = {{&S36667[8], 3}, {NULL}};
const Segment SGI388436[] = {{&S36667[13], 3}, {NULL}};
const Segment SGI388437[] = {{&S36667[13], 3}, {&S36667[18], 10}, {NULL}};
const Segment SGI388438[] = {{&S36667[47], 13}, {NULL}};
const Segment SGI388439[] = {{&S36668[8], 3}, {NULL}};
const Segment SGI388440[] = {{&S36668[13], 3}, {NULL}};
const Segment SGI388441[] = {{&S36668[49], 13}, {NULL}};
const Segment SGI388442[] = {{&S102151[13], 10}, {&S102151[24], 9}, {&S102151[34], 11}, {&S102151[46], 13}, {&S102151[60], 4}, {&S102151[65], 8}, {&S102151[74], 15}, {NULL}};
const Segment SGI388443[] = {{&S102153[13], 10}, {&S102153[24], 10}, {&S102153[35], 11}, {&S102153[47], 17}, {&S102153[65], 4}, {&S102153[70], 8}, {NULL}};
const Segment SGI388444[] = {{&S102181[13], 10}, {&S102181[24], 9}, {&S102181[34], 11}, {&S102181[46], 13}, {&S102181[60], 10}, {NULL}};
const Segment SGI388445[] = {{&S102218[13], 7}, {&S102218[21], 2}, {&S102218[24], 15}, {&S102218[40], 6}, {&S102218[47], 14}, {&S102218[62], 28}, {NULL}};
const Segment SGI388446[] = {{&S102338[13], 10}, {&S102338[24], 9}, {&S102338[34], 11}, {&S102338[46], 14}, {&S102338[61], 14}, {NULL}};
const Segment SGI388447[] = {{&S102355[13], 10}, {&S102355[24], 9}, {&S102355[34], 11}, {&S102355[46], 14}, {&S102355[61], 15}, {&S102355[77], 7}, {NULL}};
const Segment SGI388448[] = {{&S102370[13], 10}, {&S102370[24], 9}, {&S102370[34], 11}, {&S102370[46], 14}, {&S102370[61], 18}, {NULL}};
const Segment SGI388449[] = {{&S102372[13], 10}, {&S102372[24], 9}, {&S102372[34], 11}, {&S102372[46], 4}, {&S102372[51], 8}, {&S102372[60], 49}, {&S102372[110], 16}, {NULL}};
const Segment SGI388450[] = {{&S102373[13], 10}, {&S102373[24], 9}, {&S102373[34], 11}, {&S102373[46], 4}, {&S102373[51], 8}, {&S102373[60], 49}, {&S102373[110], 16}, {NULL}};
const Segment SGI388451[] = {{&S102374[13], 10}, {&S102374[24], 9}, {&S102374[34], 11}, {&S102374[46], 4}, {&S102374[51], 8}, {&S102374[60], 49}, {&S102374[110], 16}, {NULL}};
const Segment SGI388452[] = {{&S102397[13], 10}, {&S102397[24], 9}, {&S102397[34], 11}, {&S102397[46], 13}, {&S102397[60], 13}, {NULL}};
const Segment SGI388453[] = {{&S102403[13], 10}, {&S102403[24], 9}, {&S102403[34], 11}, {&S102403[46], 13}, {&S102403[60], 4}, {&S102403[65], 8}, {&S102403[74], 49}, {NULL}};
const Segment SGI388454[] = {{&S102404[13], 10}, {&S102404[24], 9}, {&S102404[34], 11}, {&S102404[46], 13}, {&S102404[60], 10}, {&S102404[71], 8}, {NULL}};
const Segment SGI388455[] = {{&S102548[13], 10}, {&S102548[24], 9}, {&S102548[34], 11}, {&S102548[46], 14}, {&S102548[61], 15}, {&S102548[77], 10}, {NULL}};
const Segment SGI388456[] = {{&S41406[8], 3}, {NULL}};
const Segment SGI388457[] = {{&S41406[13], 9}, {NULL}};
const Segment SGI388458[] = {{&S41406[31], 16}, {NULL}};
const Segment SGI388459[] = {{&S41413[8], 3}, {NULL}};
const Segment SGI388460[] = {{&S41413[13], 9}, {NULL}};
const Segment SGI388461[] = {{&S41413[31], 16}, {NULL}};
const Segment SGI388462[] = {{&S41413[93], 23}, {NULL}};
const Segment SGI388463[] = {{&S41414[8], 3}, {NULL}};
const Segment SGI388464[] = {{&S41414[13], 9}, {NULL}};
const Segment SGI388465[] = {{&S41414[33], 16}, {NULL}};
const Segment SGI388466[] = {{&S41414[95], 22}, {NULL}};
const Segment SGI388467[] = {{&S41415[8], 3}, {NULL}};
const Segment SGI388468[] = {{&S41415[13], 9}, {NULL}};
const Segment SGI388469[] = {{&S41415[31], 16}, {NULL}};
const Segment SGI388470[] = {{&S41415[93], 22}, {NULL}};
const Segment SGI388471[] = {{&S41416[8], 3}, {NULL}};
const Segment SGI388472[] = {{&S41416[13], 9}, {NULL}};
const Segment SGI388473[] = {{&S41416[31], 16}, {NULL}};
const Segment SGI388474[] = {{&S41416[93], 22}, {NULL}};
const Segment SGI388475[] = {{&S102747[13], 10}, {&S102747[24], 9}, {&S102747[34], 11}, {&S102747[46], 14}, {&S102747[61], 17}, {NULL}};
const Segment SGI388476[] = {{&S102773[13], 10}, {&S102773[24], 9}, {&S102773[34], 11}, {&S102773[46], 14}, {&S102773[61], 15}, {&S102773[77], 41}, {NULL}};
const Segment SGI388477[] = {{&S102803[13], 10}, {&S102803[24], 9}, {&S102803[34], 11}, {&S102803[46], 14}, {&S102803[61], 8}, {&S102803[70], 4}, {&S102803[75], 15}, {&S102803[91], 27}, {NULL}};
const Segment SGI388478[] = {{&S102807[13], 10}, {&S102807[24], 9}, {&S102807[34], 11}, {&S102807[46], 14}, {&S102807[61], 15}, {&S102807[77], 9}, {NULL}};
const Segment SGI388479[] = {{&S41578[8], 3}, {NULL}};
const Segment SGI388480[] = {{&S41578[13], 9}, {NULL}};
const Segment SGI388481[] = {{&S41585[8], 3}, {NULL}};
const Segment SGI388482[] = {{&S41585[13], 9}, {NULL}};
const Segment SGI388483[] = {{&S41585[85], 14}, {NULL}};
const Segment SGI388484[] = {{&S41592[8], 3}, {NULL}};
const Segment SGI388485[] = {{&S41592[13], 9}, {NULL}};
const Segment SGI388486[] = {{&S41592[83], 13}, {NULL}};
const Segment SGI388487[] = {{&S102990[13], 10}, {&S102990[24], 9}, {&S102990[34], 11}, {&S102990[46], 14}, {&S102990[61], 12}, {NULL}};
const Segment SGI388488[] = {{&S103022[13], 10}, {&S103022[24], 9}, {&S103022[34], 11}, {&S103022[46], 14}, {&S103022[61], 15}, {&S103022[77], 4}, {&S103022[82], 56}, {NULL}};
const Segment SGI388489[] = {{&S37733[8], 3}, {NULL}};
const Segment SGI388490[] = {{&S37733[13], 7}, {NULL}};
const Segment SGI388491[] = {{&S37733[25], 14}, {NULL}};
const Segment SGI388492[] = {{&S37733[66], 10}, {NULL}};
const Segment SGI388493[] = {{&S37736[8], 3}, {NULL}};
const Segment SGI388494[] = {{&S37736[13], 7}, {NULL}};
const Segment SGI388495[] = {{&S37736[25], 14}, {NULL}};
const Segment SGI388496[] = {{&S37736[66], 10}, {NULL}};
const Segment SGI388497[] = {{&S103231[13], 10}, {&S103231[24], 10}, {&S103231[35], 11}, {&S103231[47], 12}, {&S103231[60], 4}, {&S103231[65], 8}, {NULL}};
const Segment SGI388498[] = {{&S103251[13], 10}, {&S103251[24], 9}, {&S103251[34], 11}, {&S103251[46], 13}, {&S103251[60], 17}, {NULL}};
const Segment SGI388499[] = {{&S103320[13], 10}, {&S103320[24], 9}, {&S103320[34], 11}, {&S103320[46], 13}, {&S103320[60], 15}, {NULL}};
const Segment SGI388500[] = {{&S103326[13], 10}, {&S103326[24], 9}, {&S103326[34], 11}, {&S103326[46], 13}, {&S103326[60], 10}, {&S103326[71], 8}, {NULL}};
const Segment SGI388501[] = {{&S103327[13], 10}, {&S103327[24], 10}, {&S103327[35], 11}, {&S103327[47], 12}, {&S103327[60], 4}, {&S103327[65], 9}, {NULL}};
const Segment SGI388502[] = {{&S103350[13], 10}, {&S103350[24], 10}, {&S103350[35], 11}, {&S103350[47], 4}, {&S103350[52], 21}, {NULL}};
const Segment SGI388503[] = {{&S103565[13], 10}, {&S103565[24], 9}, {&S103565[34], 11}, {&S103565[46], 14}, {&S103565[61], 54}, {NULL}};
const Segment SGI388504[] = {{&S103594[13], 10}, {&S103594[24], 9}, {&S103594[34], 11}, {&S103594[46], 14}, {&S103594[61], 19}, {&S103594[81], 8}, {NULL}};
const Segment SGI388505[] = {{&S103595[13], 10}, {&S103595[24], 9}, {&S103595[34], 11}, {&S103595[46], 13}, {&S103595[60], 68}, {NULL}};
const Segment SGI388506[] = {{&S103602[13], 10}, {&S103602[24], 10}, {&S103602[35], 11}, {&S103602[47], 15}, {&S103602[63], 4}, {&S103602[68], 8}, {NULL}};
const Segment SGI388507[] = {{&S103631[13], 10}, {&S103631[24], 9}, {&S103631[34], 11}, {&S103631[46], 13}, {&S103631[60], 4}, {&S103631[65], 8}, {&S103631[74], 49}, {NULL}};
const Segment SGI388508[] = {{&S103637[13], 10}, {&S103637[24], 9}, {&S103637[34], 11}, {&S103637[46], 14}, {&S103637[61], 23}, {NULL}};
const Segment SGI388509[] = {{&S103864[13], 10}, {&S103864[24], 9}, {&S103864[34], 11}, {&S103864[46], 14}, {&S103864[61], 114}, {NULL}};
const Segment SGI388510[] = {{&S103865[13], 10}, {&S103865[24], 9}, {&S103865[34], 11}, {&S103865[46], 14}, {&S103865[61], 127}, {NULL}};
const Segment SGI388511[] = {{&S103899[13], 10}, {&S103899[24], 9}, {&S103899[34], 11}, {&S103899[46], 14}, {&S103899[61], 13}, {NULL}};
const Segment SGI388512[] = {{&S103904[13], 10}, {&S103904[24], 9}, {&S103904[34], 11}, {&S103904[46], 14}, {&S103904[61], 17}, {NULL}};
const Segment SGI388513[] = {{&S103944[13], 10}, {&S103944[24], 9}, {&S103944[34], 11}, {&S103944[46], 14}, {&S103944[61], 21}, {NULL}};
const Segment SGI388514[] = {{&S103945[13], 10}, {&S103945[24], 10}, {&S103945[35], 11}, {&S103945[47], 4}, {&S103945[52], 52}, {NULL}};
const Segment SGI388515[] = {{&S104159[13], 10}, {&S104159[24], 9}, {&S104159[34], 11}, {&S104159[46], 14}, {&S104159[61], 8}, {&S104159[70], 4}, {&S104159[75], 16}, {&S104159[92], 28}, {NULL}};
const Segment SGI388516[] = {{&S104176[13], 10}, {&S104176[24], 9}, {&S104176[34], 11}, {&S104176[46], 14}, {&S104176[61], 15}, {&S104176[77], 19}, {&S104176[97], 8}, {NULL}};
const Segment SGI388517[] = {{&S104205[13], 10}, {&S104205[24], 9}, {&S104205[34], 11}, {&S104205[46], 14}, {&S104205[61], 13}, {NULL}};
const Segment SGI388518[] = {{&S104206[13], 10}, {&S104206[24], 9}, {&S104206[34], 11}, {&S104206[46], 14}, {&S104206[61], 54}, {NULL}};
const Segment SGI388519[] = {{&S104227[13], 10}, {&S104227[24], 9}, {&S104227[34], 11}, {&S104227[46], 13}, {&S104227[60], 10}, {NULL}};
const Segment SGI388520[] = {{&S104240[13], 10}, {&S104240[24], 9}, {&S104240[34], 11}, {&S104240[46], 13}, {&S104240[60], 42}, {NULL}};
const Segment SGI388521[] = {{&S104260[13], 10}, {&S104260[24], 10}, {&S104260[35], 11}, {&S104260[47], 12}, {&S104260[60], 11}, {&S104260[72], 8}, {NULL}};
const Segment SGI388522[] = {{&S104304[13], 10}, {&S104304[24], 10}, {&S104304[35], 11}, {&S104304[47], 10}, {&S104304[58], 8}, {NULL}};
const Segment SGI388523[] = {{&S104573[13], 10}, {&S104573[24], 9}, {&S104573[34], 11}, {&S104573[46], 14}, {&S104573[61], 15}, {&S104573[77], 12}, {NULL}};
const Segment SGI388524[] = {{&S104575[13], 10}, {&S104575[24], 9}, {&S104575[34], 11}, {&S104575[46], 13}, {&S104575[60], 11}, {&S104575[72], 16}, {NULL}};
const Segment SGI388525[] = {{&S104594[13], 10}, {&S104594[24], 9}, {&S104594[34], 11}, {&S104594[46], 13}, {&S104594[60], 4}, {&S104594[65], 8}, {&S104594[74], 61}, {NULL}};
const Segment SGI388526[] = {{&S104606[13], 10}, {&S104606[24], 9}, {&S104606[34], 11}, {&S104606[46], 13}, {&S104606[60], 42}, {NULL}};
const Segment SGI388527[] = {{&S104625[13], 10}, {&S104625[24], 9}, {&S104625[34], 11}, {&S104625[46], 13}, {&S104625[60], 4}, {&S104625[65], 8}, {&S104625[74], 54}, {NULL}};
const Segment SGI388528[] = {{&S104630[13], 10}, {&S104630[24], 9}, {&S104630[34], 11}, {&S104630[46], 13}, {&S104630[60], 12}, {NULL}};
const Segment SGI388529[] = {{&S104662[13], 10}, {&S104662[24], 9}, {&S104662[34], 11}, {&S104662[46], 14}, {&S104662[61], 16}, {NULL}};
const Segment SGI388530[] = {{&S104673[13], 10}, {&S104673[24], 9}, {&S104673[34], 11}, {&S104673[46], 13}, {&S104673[60], 11}, {NULL}};
const Segment SGI388531[] = {{&S104675[13], 10}, {&S104675[24], 9}, {&S104675[34], 11}, {&S104675[46], 4}, {&S104675[51], 34}, {NULL}};
const Segment SGI388532[] = {{&S104746[13], 10}, {&S104746[24], 9}, {&S104746[34], 11}, {&S104746[46], 13}, {&S104746[60], 17}, {NULL}};
const Segment SGI388533[] = {{&S104756[13], 10}, {&S104756[24], 9}, {&S104756[34], 11}, {&S104756[46], 14}, {&S104756[61], 12}, {NULL}};
const Segment SGI388534[] = {{&S39166[8], 3}, {NULL}};
const Segment SGI388535[] = {{&S39166[13], 3}, {NULL}};
const Segment SGI388536[] = {{&S39166[13], 3}, {&S39166[21], 10}, {NULL}};
const Segment SGI388537[] = {{&S39166[89], 13}, {NULL}};
const Segment SGI388538[] = {{&S39167[8], 3}, {NULL}};
const Segment SGI388539[] = {{&S39167[13], 3}, {NULL}};
const Segment SGI388540[] = {{&S39167[63], 13}, {NULL}};
const Segment SGI388541[] = {{&S105091[13], 10}, {&S105091[24], 9}, {&S105091[34], 11}, {&S105091[46], 14}, {&S105091[61], 15}, {&S105091[77], 12}, {NULL}};
const Segment SGI388542[] = {{&S32950[62], 3}, {NULL}};
const Segment SGI388543[] = {{&S32950[158], 4}, {NULL}};
const Segment SGI388544[] = {{&S32951[62], 3}, {NULL}};
const Segment SGI388545[] = {{&S32951[154], 4}, {NULL}};
const Segment SGI388546[] = {{&S105114[13], 10}, {&S105114[24], 9}, {&S105114[34], 11}, {&S105114[46], 14}, {&S105114[61], 12}, {NULL}};
const Segment SGI388547[] = {{&S105115[13], 10}, {&S105115[24], 9}, {&S105115[34], 11}, {&S105115[46], 14}, {&S105115[61], 15}, {NULL}};
const Segment SGI388548[] = {{&S105116[13], 10}, {&S105116[24], 9}, {&S105116[34], 11}, {&S105116[46], 14}, {&S105116[61], 15}, {&S105116[77], 17}, {NULL}};
const Segment SGI388549[] = {{&S105118[13], 10}, {&S105118[24], 9}, {&S105118[34], 11}, {&S105118[46], 14}, {&S105118[61], 29}, {NULL}};
const Segment SGI388550[] = {{&S105147[13], 10}, {&S105147[24], 9}, {&S105147[34], 11}, {&S105147[46], 13}, {&S105147[60], 11}, {NULL}};
const Segment SGI388551[] = {{&S105162[13], 23}, {&S105162[37], 1}, {&S105162[39], 18}, {&S105162[58], 39}, {&S105162[98], 7}, {&S105162[106], 7}, {&S105162[114], 21}, {&S105162[136], 9}, {&S105162[146], 11}, {&S105162[158], 14}, {&S105162[173], 15}, {NULL}};
const Segment SGI388552[] = {{&S105168[13], 10}, {&S105168[24], 9}, {&S105168[34], 11}, {&S105168[46], 14}, {&S105168[61], 15}, {&S105168[77], 4}, {&S105168[82], 8}, {&S105168[91], 13}, {NULL}};
const Segment SGI388553[] = {{&S105169[13], 10}, {&S105169[24], 9}, {&S105169[34], 11}, {&S105169[46], 14}, {&S105169[61], 4}, {&S105169[66], 8}, {&S105169[75], 13}, {NULL}};
const Segment SGI388554[] = {{&S105170[13], 10}, {&S105170[24], 9}, {&S105170[34], 11}, {&S105170[46], 14}, {&S105170[61], 15}, {&S105170[77], 114}, {NULL}};
const Segment SGI388555[] = {{&S105179[13], 10}, {&S105179[24], 9}, {&S105179[34], 11}, {&S105179[46], 14}, {&S105179[61], 15}, {&S105179[77], 15}, {NULL}};
const Segment SGI388556[] = {{&S105203[13], 10}, {&S105203[24], 9}, {&S105203[34], 11}, {&S105203[46], 16}, {&S105203[63], 8}, {&S105203[72], 25}, {&S105203[46], 16}, {&S105203[63], 8}, {&S105203[46], 4}, {NULL}};
const Segment SGI388557[] = {{&S105220[13], 10}, {&S105220[24], 9}, {&S105220[34], 11}, {&S105220[46], 13}, {&S105220[60], 14}, {NULL}};
const Segment SGI388558[] = {{&S105223[13], 10}, {&S105223[24], 9}, {&S105223[34], 11}, {&S105223[46], 13}, {&S105223[60], 13}, {NULL}};
const Segment SGI388559[] = {{&S105266[13], 10}, {&S105266[24], 9}, {&S105266[34], 11}, {&S105266[46], 14}, {&S105266[61], 12}, {NULL}};
const Segment SGI388560[] = {{&S105270[13], 10}, {&S105270[24], 9}, {&S105270[34], 11}, {&S105270[46], 14}, {&S105270[61], 17}, {NULL}};
const Segment SGI388561[] = {{&S105299[13], 10}, {&S105299[24], 9}, {&S105299[34], 11}, {&S105299[46], 14}, {&S105299[61], 15}, {&S105299[77], 22}, {NULL}};
const Segment SGI388562[] = {{&S105415[13], 10}, {&S105415[24], 9}, {&S105415[34], 11}, {&S105415[46], 13}, {&S105415[60], 57}, {NULL}};
const Segment SGI388563[] = {{&S105422[13], 10}, {&S105422[24], 9}, {&S105422[34], 11}, {&S105422[46], 14}, {&S105422[61], 8}, {&S105422[70], 4}, {&S105422[75], 15}, {&S105422[91], 28}, {NULL}};
const Segment SGI388564[] = {{&S40277[8], 3}, {NULL}};
const Segment SGI388565[] = {{&S40277[13], 14}, {NULL}};
const Segment SGI388566[] = {{&S40277[46], 14}, {NULL}};
const Segment SGI388567[] = {{&S40301[8], 3}, {NULL}};
const Segment SGI388568[] = {{&S40301[13], 3}, {NULL}};
const Segment SGI388569[] = {{&S40301[56], 13}, {NULL}};
const Segment SGI388570[] = {{&S40302[8], 3}, {NULL}};
const Segment SGI388571[] = {{&S40302[13], 3}, {NULL}};
const Segment SGI388572[] = {{&S40302[13], 3}, {&S40302[21], 10}, {NULL}};
const Segment SGI388573[] = {{&S40302[60], 13}, {NULL}};
const Segment SGI388574[] = {{&S40303[8], 3}, {NULL}};
const Segment SGI388575[] = {{&S40303[12], 3}, {NULL}};
const Segment SGI388576[] = {{&S40303[50], 13}, {NULL}};
const Segment SGI388577[] = {{&S40527[8], 3}, {NULL}};
const Segment SGI388578[] = {{&S40527[13], 14}, {NULL}};
const Segment SGI388579[] = {{&S40531[8], 3}, {NULL}};
const Segment SGI388580[] = {{&S40531[13], 7}, {NULL}};
const Segment SGI388581[] = {{&S40531[25], 14}, {NULL}};
const Segment SGI388582[] = {{&S40948[8], 3}, {NULL}};
const Segment SGI388583[] = {{&S40948[13], 14}, {NULL}};
const Segment SGI388584[] = {{&S40948[53], 14}, {NULL}};
const Segment SGI388585[] = {{&S38090[39], 4}, {NULL}};
const Segment SGI388586[] = {{&S38090[8], 3}, {NULL}};
const Segment SGI388587[] = {{&S38090[13], 14}, {NULL}};
const Segment SGI388588[] = {{&S38091[32], 4}, {NULL}};
const Segment SGI388589[] = {{&S38091[8], 3}, {NULL}};
const Segment SGI388590[] = {{&S38091[13], 14}, {NULL}};
const Segment SGI388591[] = {{&S41670[8], 3}, {NULL}};
const Segment SGI388592[] = {{&S41670[13], 14}, {NULL}};
const Segment SGI388593[] = {{&S41670[53], 14}, {NULL}};
const Segment SGI388594[] = {{&S35512[50], 3}, {NULL}};
const Segment SGI388595[] = {{&S35512[151], 4}, {NULL}};
const Segment SGI388596[] = {{&S35513[50], 3}, {NULL}};
const Segment SGI388597[] = {{&S35513[150], 4}, {NULL}};
const Segment SGI388598[] = {{&S35514[50], 3}, {NULL}};
const Segment SGI388599[] = {{&S35514[147], 4}, {NULL}};
const Segment SGI388600[] = {{&S35515[50], 3}, {NULL}};
const Segment SGI388601[] = {{&S35515[147], 4}, {NULL}};
const Segment SGI388602[] = {{&S35516[50], 3}, {NULL}};
const Segment SGI388603[] = {{&S35516[150], 4}, {NULL}};
const Segment SGI388604[] = {{&S35517[50], 3}, {NULL}};
const Segment SGI388605[] = {{&S35517[151], 4}, {NULL}};
const Segment SGI388606[] = {{&S42206[8], 3}, {NULL}};
const Segment SGI388607[] = {{&S42206[13], 14}, {NULL}};
const Segment SGI388608[] = {{&S42206[53], 14}, {NULL}};
const Segment SGI388609[] = {{&S36138[93], 3}, {NULL}};
const Segment SGI388610[] = {{&S36138[29], 6}, {NULL}};
const Segment SGI388611[] = {{&S36138[128], 4}, {NULL}};
const Segment SGI388612[] = {{&S47160[8], 3}, {NULL}};
const Segment SGI388613[] = {{&S47160[13], 9}, {NULL}};
const Segment SGI388614[] = {{&S47160[85], 28}, {NULL}};
const Segment SGI388615[] = {{&S36612[93], 3}, {NULL}};
const Segment SGI388616[] = {{&S36612[29], 6}, {NULL}};
const Segment SGI388617[] = {{&S36612[128], 4}, {NULL}};
const Segment SGI388618[] = {{&S43935[8], 3}, {NULL}};
const Segment SGI388619[] = {{&S43935[13], 14}, {NULL}};
const Segment SGI388620[] = {{&S43935[53], 14}, {NULL}};
const Segment SGI388621[] = {{&S48185[8], 3}, {NULL}};
const Segment SGI388622[] = {{&S48185[13], 9}, {NULL}};
const Segment SGI388623[] = {{&S48185[86], 13}, {NULL}};
const Segment SGI388624[] = {{&S37706[93], 3}, {NULL}};
const Segment SGI388625[] = {{&S37706[29], 6}, {NULL}};
const Segment SGI388626[] = {{&S37706[128], 4}, {NULL}};
const Segment SGI388627[] = {{&S44326[8], 3}, {NULL}};
const Segment SGI388628[] = {{&S44326[13], 3}, {NULL}};
const Segment SGI388629[] = {{&S44341[8], 3}, {NULL}};
const Segment SGI388630[] = {{&S44341[13], 14}, {NULL}};
const Segment SGI388631[] = {{&S44341[53], 14}, {NULL}};
const Segment SGI388632[] = {{&S37838[93], 3}, {NULL}};
const Segment SGI388633[] = {{&S37838[29], 6}, {NULL}};
const Segment SGI388634[] = {{&S37838[128], 4}, {NULL}};
const Segment SGI388635[] = {{&S44542[8], 3}, {NULL}};
const Segment SGI388636[] = {{&S44542[13], 14}, {NULL}};
const Segment SGI388637[] = {{&S44542[46], 14}, {NULL}};
const Segment SGI388638[] = {{&S41385[39], 4}, {NULL}};
const Segment SGI388639[] = {{&S41385[8], 3}, {NULL}};
const Segment SGI388640[] = {{&S41385[13], 14}, {NULL}};
const Segment SGI388641[] = {{&S41622[32], 4}, {NULL}};
const Segment SGI388642[] = {{&S41622[8], 3}, {NULL}};
const Segment SGI388643[] = {{&S41622[13], 14}, {NULL}};
const Segment SGI388644[] = {{&S45400[8], 3}, {NULL}};
const Segment SGI388645[] = {{&S45400[13], 14}, {NULL}};
const Segment SGI388646[] = {{&S45400[46], 14}, {NULL}};
const Segment SGI388647[] = {{&S45454[8], 3}, {NULL}};
const Segment SGI388648[] = {{&S45454[13], 14}, {NULL}};
const Segment SGI388649[] = {{&S45454[47], 5}, {NULL}};
const Segment SGI388650[] = {{&S45455[8], 3}, {NULL}};
const Segment SGI388651[] = {{&S45455[13], 14}, {NULL}};
const Segment SGI388652[] = {{&S45561[8], 3}, {NULL}};
const Segment SGI388653[] = {{&S45561[13], 14}, {NULL}};
const Segment SGI388654[] = {{&S45561[53], 14}, {NULL}};
const Segment SGI388655[] = {{&S45828[8], 3}, {NULL}};
const Segment SGI388656[] = {{&S45828[13], 14}, {NULL}};
const Segment SGI388657[] = {{&S45828[46], 14}, {NULL}};
const Segment SGI388658[] = {{&S39684[62], 3}, {NULL}};
const Segment SGI388659[] = {{&S39684[154], 5}, {NULL}};
const Segment SGI388660[] = {{&S39685[62], 3}, {NULL}};
const Segment SGI388661[] = {{&S39685[158], 5}, {NULL}};
const Segment SGI388662[] = {{&S46649[8], 3}, {NULL}};
const Segment SGI388663[] = {{&S46649[13], 14}, {NULL}};
const Segment SGI388664[] = {{&S46649[53], 14}, {NULL}};
const Segment SGI388665[] = {{&S40018[93], 3}, {NULL}};
const Segment SGI388666[] = {{&S40018[29], 6}, {NULL}};
const Segment SGI388667[] = {{&S40018[128], 4}, {NULL}};
const Segment SGI388668[] = {{&S46861[8], 3}, {NULL}};
const Segment SGI388669[] = {{&S46861[13], 14}, {NULL}};
const Segment SGI388670[] = {{&S46861[46], 14}, {NULL}};
const Segment SGI388671[] = {{&S40280[93], 3}, {NULL}};
const Segment SGI388672[] = {{&S40280[29], 6}, {NULL}};
const Segment SGI388673[] = {{&S40280[128], 4}, {NULL}};
const Segment SGI388674[] = {{&S40468[93], 3}, {NULL}};
const Segment SGI388675[] = {{&S40468[29], 6}, {NULL}};
const Segment SGI388676[] = {{&S40468[128], 4}, {NULL}};
const Segment SGI388677[] = {{&S47391[8], 3}, {NULL}};
const Segment SGI388678[] = {{&S47391[13], 7}, {NULL}};
const Segment SGI388679[] = {{&S47391[25], 14}, {NULL}};
const Segment SGI388680[] = {{&S47391[69], 13}, {NULL}};
const Segment SGI388681[] = {{&S47392[8], 3}, {NULL}};
const Segment SGI388682[] = {{&S47392[13], 7}, {NULL}};
const Segment SGI388683[] = {{&S47392[25], 14}, {NULL}};
const Segment SGI388684[] = {{&S47392[85], 10}, {NULL}};
const Segment SGI388685[] = {{&S47392[71], 13}, {NULL}};
const Segment SGI388686[] = {{&S47394[8], 3}, {NULL}};
const Segment SGI388687[] = {{&S47394[13], 7}, {NULL}};
const Segment SGI388688[] = {{&S47394[25], 14}, {NULL}};
const Segment SGI388689[] = {{&S47394[69], 13}, {NULL}};
const Segment SGI388690[] = {{&S47524[8], 3}, {NULL}};
const Segment SGI388691[] = {{&S47524[13], 3}, {NULL}};
const Segment SGI388692[] = {{&S41028[93], 3}, {NULL}};
const Segment SGI388693[] = {{&S41028[29], 6}, {NULL}};
const Segment SGI388694[] = {{&S41028[128], 4}, {NULL}};
const Segment SGI388695[] = {{&S47833[8], 3}, {NULL}};
const Segment SGI388696[] = {{&S47833[13], 7}, {NULL}};
const Segment SGI388697[] = {{&S47833[25], 14}, {NULL}};
const Segment SGI388698[] = {{&S47833[66], 16}, {NULL}};
const Segment SGI388699[] = {{&S47834[8], 3}, {NULL}};
const Segment SGI388700[] = {{&S47834[13], 7}, {NULL}};
const Segment SGI388701[] = {{&S47834[25], 14}, {NULL}};
const Segment SGI388702[] = {{&S47834[66], 16}, {NULL}};
const Segment SGI388703[] = {{&S41271[93], 3}, {NULL}};
const Segment SGI388704[] = {{&S41271[29], 6}, {NULL}};
const Segment SGI388705[] = {{&S41271[128], 4}, {NULL}};
const Segment SGI388706[] = {{&S48108[8], 3}, {NULL}};
const Segment SGI388707[] = {{&S48108[13], 3}, {NULL}};
const Segment SGI388708[] = {{&S48108[13], 3}, {&S48108[18], 10}, {NULL}};
const Segment SGI388709[] = {{&S48108[55], 13}, {NULL}};
const Segment SGI388710[] = {{&S48109[8], 3}, {NULL}};
const Segment SGI388711[] = {{&S48109[13], 3}, {NULL}};
const Segment SGI388712[] = {{&S48109[50], 13}, {NULL}};
const Segment SGI388713[] = {{&S41392[93], 3}, {NULL}};
const Segment SGI388714[] = {{&S41392[29], 6}, {NULL}};
const Segment SGI388715[] = {{&S41392[128], 4}, {NULL}};
const Segment SGI388716[] = {{&S41467[50], 3}, {NULL}};
const Segment SGI388717[] = {{&S41467[147], 5}, {NULL}};
const Segment SGI388718[] = {{&S41468[50], 3}, {NULL}};
const Segment SGI388719[] = {{&S41468[150], 5}, {NULL}};
const Segment SGI388720[] = {{&S41469[50], 3}, {NULL}};
const Segment SGI388721[] = {{&S41469[147], 5}, {NULL}};
const Segment SGI388722[] = {{&S41470[50], 3}, {NULL}};
const Segment SGI388723[] = {{&S41470[151], 5}, {NULL}};
const Segment SGI388724[] = {{&S41471[50], 3}, {NULL}};
const Segment SGI388725[] = {{&S41471[151], 5}, {NULL}};
const Segment SGI388726[] = {{&S41714[93], 3}, {NULL}};
const Segment SGI388727[] = {{&S41714[29], 6}, {NULL}};
const Segment SGI388728[] = {{&S41714[128], 4}, {NULL}};
const Segment SGI388729[] = {{&S42293[94], 3}, {NULL}};
const Segment SGI388730[] = {{&S42293[29], 7}, {NULL}};
const Segment SGI388731[] = {{&S42293[129], 4}, {NULL}};
const Segment SGI388732[] = {{&S52871[8], 3}, {NULL}};
const Segment SGI388733[] = {{&S52871[13], 3}, {NULL}};
const Segment SGI388734[] = {{&S45078[35], 4}, {NULL}};
const Segment SGI388735[] = {{&S45078[8], 3}, {NULL}};
const Segment SGI388736[] = {{&S45078[13], 3}, {NULL}};
const Segment SGI388737[] = {{&S49001[8], 3}, {NULL}};
const Segment SGI388738[] = {{&S49001[13], 3}, {NULL}};
const Segment SGI388739[] = {{&S49001[57], 13}, {NULL}};
const Segment SGI388740[] = {{&S49002[8], 3}, {NULL}};
const Segment SGI388741[] = {{&S49002[13], 3}, {NULL}};
const Segment SGI388742[] = {{&S49002[13], 3}, {&S49002[18], 10}, {NULL}};
const Segment SGI388743[] = {{&S49002[55], 13}, {NULL}};
const Segment SGI388744[] = {{&S49358[8], 3}, {NULL}};
const Segment SGI388745[] = {{&S49358[13], 3}, {NULL}};
const Segment SGI388746[] = {{&S49358[13], 3}, {&S49358[18], 10}, {NULL}};
const Segment SGI388747[] = {{&S49358[47], 14}, {NULL}};
const Segment SGI388748[] = {{&S49359[8], 3}, {NULL}};
const Segment SGI388749[] = {{&S49359[13], 3}, {NULL}};
const Segment SGI388750[] = {{&S49359[49], 14}, {NULL}};
const Segment SGI388751[] = {{&S46193[39], 4}, {NULL}};
const Segment SGI388752[] = {{&S46193[8], 3}, {NULL}};
const Segment SGI388753[] = {{&S46193[13], 14}, {NULL}};
const Segment SGI388754[] = {{&S50354[8], 3}, {NULL}};
const Segment SGI388755[] = {{&S50354[13], 3}, {NULL}};
const Segment SGI388756[] = {{&S50354[58], 13}, {NULL}};
const Segment SGI388757[] = {{&S50358[8], 3}, {NULL}};
const Segment SGI388758[] = {{&S50358[13], 3}, {NULL}};
const Segment SGI388759[] = {{&S50358[13], 3}, {&S50358[18], 10}, {NULL}};
const Segment SGI388760[] = {{&S50358[56], 13}, {NULL}};
const Segment SGI388761[] = {{&S43852[94], 3}, {NULL}};
const Segment SGI388762[] = {{&S43852[29], 7}, {NULL}};
const Segment SGI388763[] = {{&S43852[129], 4}, {NULL}};
const Segment SGI388764[] = {{&S50925[8], 3}, {NULL}};
const Segment SGI388765[] = {{&S50925[13], 14}, {NULL}};
const Segment SGI388766[] = {{&S50925[46], 14}, {NULL}};
const Segment SGI388767[] = {{&S50980[8], 3}, {NULL}};
const Segment SGI388768[] = {{&S50980[13], 14}, {NULL}};
const Segment SGI388769[] = {{&S50980[53], 14}, {NULL}};
const Segment SGI388770[] = {{&S47299[48], 4}, {NULL}};
const Segment SGI388771[] = {{&S47299[8], 3}, {NULL}};
const Segment SGI388772[] = {{&S47299[13], 9}, {NULL}};
const Segment SGI388773[] = {{&S47299[30], 13}, {NULL}};
const Segment SGI388774[] = {{&S44569[93], 3}, {NULL}};
const Segment SGI388775[] = {{&S44569[29], 6}, {NULL}};
const Segment SGI388776[] = {{&S44569[128], 4}, {NULL}};
const Segment SGI388777[] = {{&S44672[93], 3}, {NULL}};
const Segment SGI388778[] = {{&S44672[29], 6}, {NULL}};
const Segment SGI388779[] = {{&S44672[128], 4}, {NULL}};
const Segment SGI388780[] = {{&S51380[8], 3}, {NULL}};
const Segment SGI388781[] = {{&S51380[13], 7}, {NULL}};
const Segment SGI388782[] = {{&S51380[25], 14}, {NULL}};
const Segment SGI388783[] = {{&S51380[66], 12}, {NULL}};
const Segment SGI388784[] = {{&S51595[8], 3}, {NULL}};
const Segment SGI388785[] = {{&S51595[13], 3}, {NULL}};
const Segment SGI388786[] = {{&S51595[13], 3}, {&S51595[18], 10}, {NULL}};
const Segment SGI388787[] = {{&S51595[48], 12}, {NULL}};
const Segment SGI388788[] = {{&S51599[8], 3}, {NULL}};
const Segment SGI388789[] = {{&S51599[13], 14}, {NULL}};
const Segment SGI388790[] = {{&S51599[46], 14}, {NULL}};
const Segment SGI388791[] = {{&S45075[93], 3}, {NULL}};
const Segment SGI388792[] = {{&S45075[29], 6}, {NULL}};
const Segment SGI388793[] = {{&S45075[128], 4}, {NULL}};
const Segment SGI388794[] = {{&S48633[48], 4}, {NULL}};
const Segment SGI388795[] = {{&S48633[8], 3}, {NULL}};
const Segment SGI388796[] = {{&S48633[13], 9}, {NULL}};
const Segment SGI388797[] = {{&S48633[30], 13}, {NULL}};
const Segment SGI388798[] = {{&S52199[8], 3}, {NULL}};
const Segment SGI388799[] = {{&S52199[13], 14}, {NULL}};
const Segment SGI388800[] = {{&S52200[8], 3}, {NULL}};
const Segment SGI388801[] = {{&S52200[13], 14}, {NULL}};
const Segment SGI388802[] = {{&S52200[29], 5}, {NULL}};
const Segment SGI388803[] = {{&S45563[94], 3}, {NULL}};
const Segment SGI388804[] = {{&S45563[29], 7}, {NULL}};
const Segment SGI388805[] = {{&S45563[129], 4}, {NULL}};
const Segment SGI388806[] = {{&S52342[8], 3}, {NULL}};
const Segment SGI388807[] = {{&S52342[13], 3}, {NULL}};
const Segment SGI388808[] = {{&S52342[13], 3}, {&S52342[18], 10}, {NULL}};
const Segment SGI388809[] = {{&S52342[47], 14}, {NULL}};
const Segment SGI388810[] = {{&S45731[94], 3}, {NULL}};
const Segment SGI388811[] = {{&S45731[29], 7}, {NULL}};
const Segment SGI388812[] = {{&S45731[129], 4}, {NULL}};
const Segment SGI388813[] = {{&S52449[8], 3}, {NULL}};
const Segment SGI388814[] = {{&S52449[13], 7}, {NULL}};
const Segment SGI388815[] = {{&S52449[25], 14}, {NULL}};
const Segment SGI388816[] = {{&S52449[71], 13}, {NULL}};
const Segment SGI388817[] = {{&S52450[8], 3}, {NULL}};
const Segment SGI388818[] = {{&S52450[13], 7}, {NULL}};
const Segment SGI388819[] = {{&S52450[25], 14}, {NULL}};
const Segment SGI388820[] = {{&S52450[69], 10}, {NULL}};
const Segment SGI388821[] = {{&S52450[82], 13}, {NULL}};
const Segment SGI388822[] = {{&S52451[8], 3}, {NULL}};
const Segment SGI388823[] = {{&S52451[13], 7}, {NULL}};
const Segment SGI388824[] = {{&S52451[25], 14}, {NULL}};
const Segment SGI388825[] = {{&S52451[72], 13}, {NULL}};
const Segment SGI388826[] = {{&S53185[8], 3}, {NULL}};
const Segment SGI388827[] = {{&S53185[13], 3}, {NULL}};
const Segment SGI388828[] = {{&S53185[61], 13}, {NULL}};
const Segment SGI388829[] = {{&S56726[8], 3}, {NULL}};
const Segment SGI388830[] = {{&S56726[13], 3}, {NULL}};
const Segment SGI388831[] = {{&S57004[8], 3}, {NULL}};
const Segment SGI388832[] = {{&S57004[13], 14}, {NULL}};
const Segment SGI388833[] = {{&S57006[8], 3}, {NULL}};
const Segment SGI388834[] = {{&S57006[13], 7}, {NULL}};
const Segment SGI388835[] = {{&S57006[25], 14}, {NULL}};
const Segment SGI388836[] = {{&S57007[8], 3}, {NULL}};
const Segment SGI388837[] = {{&S57007[13], 14}, {NULL}};
const Segment SGI388838[] = {{&S11721[0], 12}, {NULL}};
const Segment SGI388839[] = {{&S11721[40], 4}, {NULL}};
const Segment SGI388840[] = {{&S11722[0], 5}, {NULL}};
const Segment SGI388841[] = {{&S11722[33], 4}, {NULL}};
const Segment SGI388842[] = {{&S11723[0], 5}, {NULL}};
const Segment SGI388843[] = {{&S11723[33], 4}, {NULL}};
const Segment SGI388844[] = {{&S11724[0], 14}, {NULL}};
const Segment SGI388845[] = {{&S11724[43], 4}, {NULL}};
const Segment SGI388846[] = {{&S11725[0], 5}, {NULL}};
const Segment SGI388847[] = {{&S11725[33], 4}, {NULL}};
const Segment SGI388848[] = {{&S53491[8], 3}, {NULL}};
const Segment SGI388849[] = {{&S53491[13], 3}, {NULL}};
const Segment SGI388850[] = {{&S53491[13], 3}, {&S53491[18], 6}, {&S53491[26], 10}, {NULL}};
const Segment SGI388851[] = {{&S11726[0], 9}, {NULL}};
const Segment SGI388852[] = {{&S11726[38], 4}, {NULL}};
const Segment SGI388853[] = {{&S11727[0], 10}, {NULL}};
const Segment SGI388854[] = {{&S11727[39], 4}, {NULL}};
const Segment SGI388855[] = {{&S11728[0], 13}, {NULL}};
const Segment SGI388856[] = {{&S11728[40], 4}, {NULL}};
const Segment SGI388857[] = {{&S11729[0], 6}, {NULL}};
const Segment SGI388858[] = {{&S11729[32], 4}, {NULL}};
const Segment SGI388859[] = {{&S11730[0], 8}, {NULL}};
const Segment SGI388860[] = {{&S11730[36], 4}, {NULL}};
const Segment SGI388861[] = {{&S11731[0], 10}, {NULL}};
const Segment SGI388862[] = {{&S11731[36], 4}, {NULL}};
const Segment SGI388863[] = {{&S11732[0], 11}, {NULL}};
const Segment SGI388864[] = {{&S11732[37], 4}, {NULL}};
const Segment SGI388865[] = {{&S11733[0], 14}, {NULL}};
const Segment SGI388866[] = {{&S11733[40], 4}, {NULL}};
const Segment SGI388867[] = {{&S11734[0], 9}, {NULL}};
const Segment SGI388868[] = {{&S11734[37], 4}, {NULL}};
const Segment SGI388869[] = {{&S11735[0], 13}, {NULL}};
const Segment SGI388870[] = {{&S11735[41], 4}, {NULL}};
const Segment SGI388871[] = {{&S11736[0], 4}, {NULL}};
const Segment SGI388872[] = {{&S11736[34], 4}, {NULL}};
const Segment SGI388873[] = {{&S11737[0], 9}, {NULL}};
const Segment SGI388874[] = {{&S11737[37], 4}, {NULL}};
const Segment SGI388875[] = {{&S11738[0], 3}, {NULL}};
const Segment SGI388876[] = {{&S11738[27], 4}, {NULL}};
const Segment SGI388877[] = {{&S11739[0], 5}, {NULL}};
const Segment SGI388878[] = {{&S11739[31], 4}, {NULL}};
const Segment SGI388879[] = {{&S11740[0], 11}, {NULL}};
const Segment SGI388880[] = {{&S11740[38], 4}, {NULL}};
const Segment SGI388881[] = {{&S11741[0], 13}, {NULL}};
const Segment SGI388882[] = {{&S11741[41], 4}, {NULL}};
const Segment SGI388883[] = {{&S11742[0], 6}, {NULL}};
const Segment SGI388884[] = {{&S11742[32], 4}, {NULL}};
const Segment SGI388885[] = {{&S11743[0], 6}, {NULL}};
const Segment SGI388886[] = {{&S11743[32], 4}, {NULL}};
const Segment SGI388887[] = {{&S11744[0], 8}, {NULL}};
const Segment SGI388888[] = {{&S11744[36], 4}, {NULL}};
const Segment SGI388889[] = {{&S11745[0], 4}, {NULL}};
const Segment SGI388890[] = {{&S11745[32], 4}, {NULL}};
const Segment SGI388891[] = {{&S11746[0], 13}, {NULL}};
const Segment SGI388892[] = {{&S11746[39], 4}, {NULL}};
const Segment SGI388893[] = {{&S11747[0], 9}, {NULL}};
const Segment SGI388894[] = {{&S11747[35], 4}, {NULL}};
const Segment SGI388895[] = {{&S11748[0], 6}, {NULL}};
const Segment SGI388896[] = {{&S11748[34], 4}, {NULL}};
const Segment SGI388897[] = {{&S11749[0], 10}, {NULL}};
const Segment SGI388898[] = {{&S11749[36], 4}, {NULL}};
const Segment SGI388899[] = {{&S11751[0], 8}, {NULL}};
const Segment SGI388900[] = {{&S11751[34], 4}, {NULL}};
const Segment SGI388901[] = {{&S11752[0], 12}, {NULL}};
const Segment SGI388902[] = {{&S11752[38], 4}, {NULL}};
const Segment SGI388903[] = {{&S11753[0], 6}, {NULL}};
const Segment SGI388904[] = {{&S11753[32], 4}, {NULL}};
const Segment SGI388905[] = {{&S11754[0], 5}, {NULL}};
const Segment SGI388906[] = {{&S11754[33], 4}, {NULL}};
const Segment SGI388907[] = {{&S11755[0], 13}, {NULL}};
const Segment SGI388908[] = {{&S11755[39], 4}, {NULL}};
const Segment SGI388909[] = {{&S11756[0], 11}, {NULL}};
const Segment SGI388910[] = {{&S11756[39], 4}, {NULL}};
const Segment SGI388911[] = {{&S11757[0], 13}, {NULL}};
const Segment SGI388912[] = {{&S11757[39], 4}, {NULL}};
const Segment SGI388913[] = {{&S11758[0], 11}, {NULL}};
const Segment SGI388914[] = {{&S11758[36], 4}, {NULL}};
const Segment SGI388915[] = {{&S11759[0], 14}, {NULL}};
const Segment SGI388916[] = {{&S11759[39], 4}, {NULL}};
const Segment SGI388917[] = {{&S11760[0], 5}, {NULL}};
const Segment SGI388918[] = {{&S11760[32], 4}, {NULL}};
const Segment SGI388919[] = {{&S11761[0], 7}, {NULL}};
const Segment SGI388920[] = {{&S11761[35], 4}, {NULL}};
const Segment SGI388921[] = {{&S11762[0], 17}, {NULL}};
const Segment SGI388922[] = {{&S11762[45], 4}, {NULL}};
const Segment SGI388923[] = {{&S11763[0], 9}, {NULL}};
const Segment SGI388924[] = {{&S11763[35], 4}, {NULL}};
const Segment SGI388925[] = {{&S11764[0], 12}, {NULL}};
const Segment SGI388926[] = {{&S11764[40], 4}, {NULL}};
const Segment SGI388927[] = {{&S11765[0], 3}, {NULL}};
const Segment SGI388928[] = {{&S11765[33], 4}, {NULL}};
const Segment SGI388929[] = {{&S11766[0], 13}, {NULL}};
const Segment SGI388930[] = {{&S11766[45], 4}, {NULL}};
const Segment SGI388931[] = {{&S11767[0], 6}, {NULL}};
const Segment SGI388932[] = {{&S11767[44], 4}, {NULL}};
const Segment SGI388933[] = {{&S11768[0], 10}, {NULL}};
const Segment SGI388934[] = {{&S11768[42], 4}, {NULL}};
const Segment SGI388935[] = {{&S11769[0], 7}, {NULL}};
const Segment SGI388936[] = {{&S11769[40], 4}, {NULL}};
const Segment SGI388937[] = {{&S11770[0], 7}, {NULL}};
const Segment SGI388938[] = {{&S11770[39], 4}, {NULL}};
const Segment SGI388939[] = {{&S11771[0], 7}, {NULL}};
const Segment SGI388940[] = {{&S11771[37], 4}, {NULL}};
const Segment SGI388941[] = {{&S11772[0], 12}, {NULL}};
const Segment SGI388942[] = {{&S11772[40], 4}, {NULL}};
const Segment SGI388943[] = {{&S11773[0], 3}, {NULL}};
const Segment SGI388944[] = {{&S11773[34], 4}, {NULL}};
const Segment SGI388945[] = {{&S11774[0], 8}, {NULL}};
const Segment SGI388946[] = {{&S11774[36], 4}, {NULL}};
const Segment SGI388947[] = {{&S11775[0], 10}, {NULL}};
const Segment SGI388948[] = {{&S11775[42], 4}, {NULL}};
const Segment SGI388949[] = {{&S11776[0], 18}, {NULL}};
const Segment SGI388950[] = {{&S11776[51], 4}, {NULL}};
const Segment SGI388951[] = {{&S11777[0], 11}, {NULL}};
const Segment SGI388952[] = {{&S11777[40], 4}, {NULL}};
const Segment SGI388953[] = {{&S11778[0], 18}, {NULL}};
const Segment SGI388954[] = {{&S11778[48], 4}, {NULL}};
const Segment SGI388955[] = {{&S11779[0], 11}, {NULL}};
const Segment SGI388956[] = {{&S11779[40], 4}, {NULL}};
const Segment SGI388957[] = {{&S11780[0], 8}, {NULL}};
const Segment SGI388958[] = {{&S11780[38], 4}, {NULL}};
const Segment SGI388959[] = {{&S11781[0], 13}, {NULL}};
const Segment SGI388960[] = {{&S11781[45], 4}, {NULL}};
const Segment SGI388961[] = {{&S11782[0], 11}, {NULL}};
const Segment SGI388962[] = {{&S11782[43], 4}, {NULL}};
const Segment SGI388963[] = {{&S11783[0], 8}, {NULL}};
const Segment SGI388964[] = {{&S11783[40], 4}, {NULL}};
const Segment SGI388965[] = {{&S11784[0], 9}, {NULL}};
const Segment SGI388966[] = {{&S11784[41], 4}, {NULL}};
const Segment SGI388967[] = {{&S11785[0], 16}, {NULL}};
const Segment SGI388968[] = {{&S11785[48], 4}, {NULL}};
const Segment SGI388969[] = {{&S11786[0], 14}, {NULL}};
const Segment SGI388970[] = {{&S11786[44], 4}, {NULL}};
const Segment SGI388971[] = {{&S11787[0], 10}, {NULL}};
const Segment SGI388972[] = {{&S11787[43], 4}, {NULL}};
const Segment SGI388973[] = {{&S11788[0], 11}, {NULL}};
const Segment SGI388974[] = {{&S11788[44], 4}, {NULL}};
const Segment SGI388975[] = {{&S11789[0], 17}, {NULL}};
const Segment SGI388976[] = {{&S11789[49], 4}, {NULL}};
const Segment SGI388977[] = {{&S11790[0], 18}, {NULL}};
const Segment SGI388978[] = {{&S11790[51], 4}, {NULL}};
const Segment SGI388979[] = {{&S11791[0], 25}, {NULL}};
const Segment SGI388980[] = {{&S11791[55], 4}, {NULL}};
const Segment SGI388981[] = {{&S11792[0], 15}, {NULL}};
const Segment SGI388982[] = {{&S11792[47], 4}, {NULL}};
const Segment SGI388983[] = {{&S11793[0], 15}, {NULL}};
const Segment SGI388984[] = {{&S11793[45], 4}, {NULL}};
const Segment SGI388985[] = {{&S11794[0], 17}, {NULL}};
const Segment SGI388986[] = {{&S11794[49], 4}, {NULL}};
const Segment SGI388987[] = {{&S11795[0], 17}, {NULL}};
const Segment SGI388988[] = {{&S11795[45], 4}, {NULL}};
const Segment SGI388989[] = {{&S11796[0], 13}, {NULL}};
const Segment SGI388990[] = {{&S11796[45], 4}, {NULL}};
const Segment SGI388991[] = {{&S11797[0], 11}, {NULL}};
const Segment SGI388992[] = {{&S11797[42], 4}, {NULL}};
const Segment SGI388993[] = {{&S11798[0], 9}, {NULL}};
const Segment SGI388994[] = {{&S11798[39], 4}, {NULL}};
const Segment SGI388995[] = {{&S11799[0], 14}, {NULL}};
const Segment SGI388996[] = {{&S11799[46], 4}, {NULL}};
const Segment SGI388997[] = {{&S11800[0], 37}, {NULL}};
const Segment SGI388998[] = {{&S11800[53], 4}, {NULL}};
const Segment SGI388999[] = {{&S11801[0], 12}, {NULL}};
const Segment SGI389000[] = {{&S11801[45], 4}, {NULL}};
const Segment SGI389001[] = {{&S11802[0], 14}, {NULL}};
const Segment SGI389002[] = {{&S11802[44], 4}, {NULL}};
const Segment SGI389003[] = {{&S11803[0], 28}, {NULL}};
const Segment SGI389004[] = {{&S11803[58], 4}, {NULL}};
const Segment SGI389005[] = {{&S11804[0], 15}, {NULL}};
const Segment SGI389006[] = {{&S11804[47], 4}, {NULL}};
const Segment SGI389007[] = {{&S11805[0], 13}, {NULL}};
const Segment SGI389008[] = {{&S11805[45], 4}, {NULL}};
const Segment SGI389009[] = {{&S11806[0], 8}, {NULL}};
const Segment SGI389010[] = {{&S11806[40], 4}, {NULL}};
const Segment SGI389011[] = {{&S11807[0], 16}, {NULL}};
const Segment SGI389012[] = {{&S11807[48], 4}, {NULL}};
const Segment SGI389013[] = {{&S11808[0], 8}, {NULL}};
const Segment SGI389014[] = {{&S11808[38], 4}, {NULL}};
const Segment SGI389015[] = {{&S11809[0], 11}, {NULL}};
const Segment SGI389016[] = {{&S11809[43], 4}, {NULL}};
const Segment SGI389017[] = {{&S11810[0], 17}, {NULL}};
const Segment SGI389018[] = {{&S11810[47], 4}, {NULL}};
const Segment SGI389019[] = {{&S11811[0], 14}, {NULL}};
const Segment SGI389020[] = {{&S11811[44], 4}, {NULL}};
const Segment SGI389021[] = {{&S11812[0], 25}, {NULL}};
const Segment SGI389022[] = {{&S11812[57], 4}, {NULL}};
const Segment SGI389023[] = {{&S11813[0], 5}, {NULL}};
const Segment SGI389024[] = {{&S11813[37], 4}, {NULL}};
const Segment SGI389025[] = {{&S11814[0], 15}, {NULL}};
const Segment SGI389026[] = {{&S11814[45], 4}, {NULL}};
const Segment SGI389027[] = {{&S11815[0], 41}, {NULL}};
const Segment SGI389028[] = {{&S11815[57], 4}, {NULL}};
const Segment SGI389029[] = {{&S11816[0], 3}, {NULL}};
const Segment SGI389030[] = {{&S11816[38], 4}, {NULL}};
const Segment SGI389031[] = {{&S11817[0], 11}, {NULL}};
const Segment SGI389032[] = {{&S11817[41], 4}, {NULL}};
const Segment SGI389033[] = {{&S11818[0], 7}, {NULL}};
const Segment SGI389034[] = {{&S11818[37], 4}, {NULL}};
const Segment SGI389035[] = {{&S11819[0], 4}, {NULL}};
const Segment SGI389036[] = {{&S11819[34], 4}, {NULL}};
const Segment SGI389037[] = {{&S11820[0], 10}, {NULL}};
const Segment SGI389038[] = {{&S11820[40], 4}, {NULL}};
const Segment SGI389039[] = {{&S11821[0], 12}, {NULL}};
const Segment SGI389040[] = {{&S11821[43], 4}, {NULL}};
const Segment SGI389041[] = {{&S11822[0], 16}, {NULL}};
const Segment SGI389042[] = {{&S11822[49], 4}, {NULL}};
const Segment SGI389043[] = {{&S11823[0], 7}, {NULL}};
const Segment SGI389044[] = {{&S11823[37], 4}, {NULL}};
const Segment SGI389045[] = {{&S11824[0], 8}, {NULL}};
const Segment SGI389046[] = {{&S11824[39], 4}, {NULL}};
const Segment SGI389047[] = {{&S11825[0], 8}, {NULL}};
const Segment SGI389048[] = {{&S11825[36], 4}, {NULL}};
const Segment SGI389049[] = {{&S11826[0], 5}, {NULL}};
const Segment SGI389050[] = {{&S11826[35], 4}, {NULL}};
const Segment SGI389051[] = {{&S11827[0], 15}, {NULL}};
const Segment SGI389052[] = {{&S11827[45], 4}, {NULL}};
const Segment SGI389053[] = {{&S11828[0], 12}, {NULL}};
const Segment SGI389054[] = {{&S11828[40], 4}, {NULL}};
const Segment SGI389055[] = {{&S11829[0], 5}, {NULL}};
const Segment SGI389056[] = {{&S11829[33], 4}, {NULL}};
const Segment SGI389057[] = {{&S11830[0], 7}, {NULL}};
const Segment SGI389058[] = {{&S11830[36], 4}, {NULL}};
const Segment SGI389059[] = {{&S11831[0], 14}, {NULL}};
const Segment SGI389060[] = {{&S11831[47], 4}, {NULL}};
const Segment SGI389061[] = {{&S11832[0], 12}, {NULL}};
const Segment SGI389062[] = {{&S11832[42], 4}, {NULL}};
const Segment SGI389063[] = {{&S11833[0], 4}, {NULL}};
const Segment SGI389064[] = {{&S11833[30], 4}, {NULL}};
const Segment SGI389065[] = {{&S11834[0], 26}, {NULL}};
const Segment SGI389066[] = {{&S11834[56], 4}, {NULL}};
const Segment SGI389067[] = {{&S11835[0], 17}, {NULL}};
const Segment SGI389068[] = {{&S11835[47], 4}, {NULL}};
const Segment SGI389069[] = {{&S11836[0], 11}, {NULL}};
const Segment SGI389070[] = {{&S11836[43], 4}, {NULL}};
const Segment SGI389071[] = {{&S11837[0], 17}, {NULL}};
const Segment SGI389072[] = {{&S11837[49], 4}, {NULL}};
const Segment SGI389073[] = {{&S11838[0], 26}, {NULL}};
const Segment SGI389074[] = {{&S11838[56], 4}, {NULL}};
const Segment SGI389075[] = {{&S11839[0], 39}, {NULL}};
const Segment SGI389076[] = {{&S11839[55], 4}, {NULL}};
const Segment SGI389077[] = {{&S11840[0], 40}, {NULL}};
const Segment SGI389078[] = {{&S11840[56], 4}, {NULL}};
const Segment SGI389079[] = {{&S11841[0], 11}, {NULL}};
const Segment SGI389080[] = {{&S11841[43], 4}, {NULL}};
const Segment SGI389081[] = {{&S11842[0], 9}, {NULL}};
const Segment SGI389082[] = {{&S11842[43], 4}, {NULL}};
const Segment SGI389083[] = {{&S11843[0], 10}, {NULL}};
const Segment SGI389084[] = {{&S11843[40], 4}, {NULL}};
const Segment SGI389085[] = {{&S11844[0], 5}, {NULL}};
const Segment SGI389086[] = {{&S11844[35], 4}, {NULL}};
const Segment SGI389087[] = {{&S11845[0], 40}, {NULL}};
const Segment SGI389088[] = {{&S11845[56], 4}, {NULL}};
const Segment SGI389089[] = {{&S11846[0], 4}, {NULL}};
const Segment SGI389090[] = {{&S11846[34], 4}, {NULL}};
const Segment SGI389091[] = {{&S11847[0], 12}, {NULL}};
const Segment SGI389092[] = {{&S11847[43], 4}, {NULL}};
const Segment SGI389093[] = {{&S11848[0], 13}, {NULL}};
const Segment SGI389094[] = {{&S11848[45], 4}, {NULL}};
const Segment SGI389095[] = {{&S11849[0], 14}, {NULL}};
const Segment SGI389096[] = {{&S11849[46], 4}, {NULL}};
const Segment SGI389097[] = {{&S11850[0], 11}, {NULL}};
const Segment SGI389098[] = {{&S11850[40], 4}, {NULL}};
const Segment SGI389099[] = {{&S11851[0], 14}, {NULL}};
const Segment SGI389100[] = {{&S11851[44], 4}, {NULL}};
const Segment SGI389101[] = {{&S11852[0], 12}, {NULL}};
const Segment SGI389102[] = {{&S11852[42], 4}, {NULL}};
const Segment SGI389103[] = {{&S11853[0], 17}, {NULL}};
const Segment SGI389104[] = {{&S11853[47], 4}, {NULL}};
const Segment SGI389105[] = {{&S11854[0], 17}, {NULL}};
const Segment SGI389106[] = {{&S11854[54], 4}, {NULL}};
const Segment SGI389107[] = {{&S11855[0], 10}, {NULL}};
const Segment SGI389108[] = {{&S11855[42], 4}, {NULL}};
const Segment SGI389109[] = {{&S11856[0], 19}, {NULL}};
const Segment SGI389110[] = {{&S11856[49], 4}, {NULL}};
const Segment SGI389111[] = {{&S11857[0], 11}, {NULL}};
const Segment SGI389112[] = {{&S11857[42], 4}, {NULL}};
const Segment SGI389113[] = {{&S11858[0], 32}, {NULL}};
const Segment SGI389114[] = {{&S11858[48], 4}, {NULL}};
const Segment SGI389115[] = {{&S11859[0], 10}, {NULL}};
const Segment SGI389116[] = {{&S11859[40], 4}, {NULL}};
const Segment SGI389117[] = {{&S11860[0], 17}, {NULL}};
const Segment SGI389118[] = {{&S11860[47], 4}, {NULL}};
const Segment SGI389119[] = {{&S11861[0], 8}, {NULL}};
const Segment SGI389120[] = {{&S11861[38], 4}, {NULL}};
const Segment SGI389121[] = {{&S11862[0], 15}, {NULL}};
const Segment SGI389122[] = {{&S11862[45], 4}, {NULL}};
const Segment SGI389123[] = {{&S11863[0], 14}, {NULL}};
const Segment SGI389124[] = {{&S11863[44], 4}, {NULL}};
const Segment SGI389125[] = {{&S11864[0], 22}, {NULL}};
const Segment SGI389126[] = {{&S11864[54], 4}, {NULL}};
const Segment SGI389127[] = {{&S11865[0], 11}, {NULL}};
const Segment SGI389128[] = {{&S11865[39], 4}, {NULL}};
const Segment SGI389129[] = {{&S11866[0], 9}, {NULL}};
const Segment SGI389130[] = {{&S11866[37], 4}, {NULL}};
const Segment SGI389131[] = {{&S11867[0], 8}, {NULL}};
const Segment SGI389132[] = {{&S11867[38], 4}, {NULL}};
const Segment SGI389133[] = {{&S11868[0], 13}, {NULL}};
const Segment SGI389134[] = {{&S11868[44], 4}, {NULL}};
const Segment SGI389135[] = {{&S11869[0], 9}, {NULL}};
const Segment SGI389136[] = {{&S11869[42], 4}, {NULL}};
const Segment SGI389137[] = {{&S11870[0], 10}, {NULL}};
const Segment SGI389138[] = {{&S11870[40], 4}, {NULL}};
const Segment SGI389139[] = {{&S11871[0], 22}, {NULL}};
const Segment SGI389140[] = {{&S11871[52], 4}, {NULL}};
const Segment SGI389141[] = {{&S11872[0], 10}, {NULL}};
const Segment SGI389142[] = {{&S11872[40], 4}, {NULL}};
const Segment SGI389143[] = {{&S11873[0], 7}, {NULL}};
const Segment SGI389144[] = {{&S11873[37], 4}, {NULL}};
const Segment SGI389145[] = {{&S11874[0], 8}, {NULL}};
const Segment SGI389146[] = {{&S11874[40], 4}, {NULL}};
const Segment SGI389147[] = {{&S11875[0], 6}, {NULL}};
const Segment SGI389148[] = {{&S11875[36], 4}, {NULL}};
const Segment SGI389149[] = {{&S11876[0], 7}, {NULL}};
const Segment SGI389150[] = {{&S11876[37], 4}, {NULL}};
const Segment SGI389151[] = {{&S11877[0], 8}, {NULL}};
const Segment SGI389152[] = {{&S11877[40], 4}, {NULL}};
const Segment SGI389153[] = {{&S11878[0], 6}, {NULL}};
const Segment SGI389154[] = {{&S11878[36], 4}, {NULL}};
const Segment SGI389155[] = {{&S11879[0], 11}, {NULL}};
const Segment SGI389156[] = {{&S11879[43], 4}, {NULL}};
const Segment SGI389157[] = {{&S11880[0], 4}, {NULL}};
const Segment SGI389158[] = {{&S11880[34], 4}, {NULL}};
const Segment SGI389159[] = {{&S11881[0], 10}, {NULL}};
const Segment SGI389160[] = {{&S11881[40], 4}, {NULL}};
const Segment SGI389161[] = {{&S11882[0], 8}, {NULL}};
const Segment SGI389162[] = {{&S11882[40], 4}, {NULL}};
const Segment SGI389163[] = {{&S11883[0], 10}, {NULL}};
const Segment SGI389164[] = {{&S11883[67], 4}, {NULL}};
const Segment SGI389165[] = {{&S11884[0], 40}, {NULL}};
const Segment SGI389166[] = {{&S11884[56], 4}, {NULL}};
const Segment SGI389167[] = {{&S11885[0], 35}, {NULL}};
const Segment SGI389168[] = {{&S11885[51], 4}, {NULL}};
const Segment SGI389169[] = {{&S11886[0], 9}, {NULL}};
const Segment SGI389170[] = {{&S11886[41], 4}, {NULL}};
const Segment SGI389171[] = {{&S11887[0], 33}, {NULL}};
const Segment SGI389172[] = {{&S11887[49], 4}, {NULL}};
const Segment SGI389173[] = {{&S11888[0], 8}, {NULL}};
const Segment SGI389174[] = {{&S11888[38], 4}, {NULL}};
const Segment SGI389175[] = {{&S11889[0], 5}, {NULL}};
const Segment SGI389176[] = {{&S11889[35], 4}, {NULL}};
const Segment SGI389177[] = {{&S11890[0], 15}, {NULL}};
const Segment SGI389178[] = {{&S11890[45], 4}, {NULL}};
const Segment SGI389179[] = {{&S11891[0], 6}, {NULL}};
const Segment SGI389180[] = {{&S11891[36], 4}, {NULL}};
const Segment SGI389181[] = {{&S11892[0], 7}, {NULL}};
const Segment SGI389182[] = {{&S11892[37], 4}, {NULL}};
const Segment SGI389183[] = {{&S11893[0], 10}, {NULL}};
const Segment SGI389184[] = {{&S11893[43], 4}, {NULL}};
const Segment SGI389185[] = {{&S11894[0], 11}, {NULL}};
const Segment SGI389186[] = {{&S11894[41], 4}, {NULL}};
const Segment SGI389187[] = {{&S11895[0], 22}, {NULL}};
const Segment SGI389188[] = {{&S11895[55], 4}, {NULL}};
const Segment SGI389189[] = {{&S11896[0], 12}, {NULL}};
const Segment SGI389190[] = {{&S11896[45], 4}, {NULL}};
const Segment SGI389191[] = {{&S11897[0], 17}, {NULL}};
const Segment SGI389192[] = {{&S11897[47], 4}, {NULL}};
const Segment SGI389193[] = {{&S49962[48], 4}, {NULL}};
const Segment SGI389194[] = {{&S49962[8], 3}, {NULL}};
const Segment SGI389195[] = {{&S49962[13], 9}, {NULL}};
const Segment SGI389196[] = {{&S49962[30], 13}, {NULL}};
const Segment SGI389197[] = {{&S11898[0], 13}, {NULL}};
const Segment SGI389198[] = {{&S11898[45], 4}, {NULL}};
const Segment SGI389199[] = {{&S11899[0], 8}, {NULL}};
const Segment SGI389200[] = {{&S11899[40], 4}, {NULL}};
const Segment SGI389201[] = {{&S11900[0], 11}, {NULL}};
const Segment SGI389202[] = {{&S11900[41], 4}, {NULL}};
const Segment SGI389203[] = {{&S11901[0], 10}, {NULL}};
const Segment SGI389204[] = {{&S11901[42], 4}, {NULL}};
const Segment SGI389205[] = {{&S11902[0], 8}, {NULL}};
const Segment SGI389206[] = {{&S11902[38], 4}, {NULL}};
const Segment SGI389207[] = {{&S11903[0], 8}, {NULL}};
const Segment SGI389208[] = {{&S11903[36], 4}, {NULL}};
const Segment SGI389209[] = {{&S11904[0], 12}, {NULL}};
const Segment SGI389210[] = {{&S11904[44], 4}, {NULL}};
const Segment SGI389211[] = {{&S11905[0], 8}, {NULL}};
const Segment SGI389212[] = {{&S11905[38], 4}, {NULL}};
const Segment SGI389213[] = {{&S11906[0], 9}, {NULL}};
const Segment SGI389214[] = {{&S11906[43], 4}, {NULL}};
const Segment SGI389215[] = {{&S11907[0], 18}, {NULL}};
const Segment SGI389216[] = {{&S11907[48], 4}, {NULL}};
const Segment SGI389217[] = {{&S11908[0], 23}, {NULL}};
const Segment SGI389218[] = {{&S11908[55], 4}, {NULL}};
const Segment SGI389219[] = {{&S11909[0], 12}, {NULL}};
const Segment SGI389220[] = {{&S11909[42], 4}, {NULL}};
const Segment SGI389221[] = {{&S11910[0], 14}, {NULL}};
const Segment SGI389222[] = {{&S11910[44], 4}, {NULL}};
const Segment SGI389223[] = {{&S11911[0], 49}, {NULL}};
const Segment SGI389224[] = {{&S11911[77], 4}, {NULL}};
const Segment SGI389225[] = {{&S11912[0], 11}, {NULL}};
const Segment SGI389226[] = {{&S11912[41], 4}, {NULL}};
const Segment SGI389227[] = {{&S11913[0], 8}, {NULL}};
const Segment SGI389228[] = {{&S11913[38], 4}, {NULL}};
const Segment SGI389229[] = {{&S11914[0], 14}, {NULL}};
const Segment SGI389230[] = {{&S11914[46], 4}, {NULL}};
const Segment SGI389231[] = {{&S11915[0], 8}, {NULL}};
const Segment SGI389232[] = {{&S11915[36], 4}, {NULL}};
const Segment SGI389233[] = {{&S11916[0], 8}, {NULL}};
const Segment SGI389234[] = {{&S11916[38], 4}, {NULL}};
const Segment SGI389235[] = {{&S11917[0], 16}, {NULL}};
const Segment SGI389236[] = {{&S11917[48], 4}, {NULL}};
const Segment SGI389237[] = {{&S11918[0], 42}, {NULL}};
const Segment SGI389238[] = {{&S11918[58], 4}, {NULL}};
const Segment SGI389239[] = {{&S11919[0], 18}, {NULL}};
const Segment SGI389240[] = {{&S11919[48], 4}, {NULL}};
const Segment SGI389241[] = {{&S11920[0], 17}, {NULL}};
const Segment SGI389242[] = {{&S11920[49], 4}, {NULL}};
const Segment SGI389243[] = {{&S11921[0], 6}, {NULL}};
const Segment SGI389244[] = {{&S11921[37], 4}, {NULL}};
const Segment SGI389245[] = {{&S11922[0], 8}, {NULL}};
const Segment SGI389246[] = {{&S11922[38], 4}, {NULL}};
const Segment SGI389247[] = {{&S11923[0], 14}, {NULL}};
const Segment SGI389248[] = {{&S11923[44], 4}, {NULL}};
const Segment SGI389249[] = {{&S11924[0], 9}, {NULL}};
const Segment SGI389250[] = {{&S11924[41], 4}, {NULL}};
const Segment SGI389251[] = {{&S11925[0], 14}, {NULL}};
const Segment SGI389252[] = {{&S11925[46], 4}, {NULL}};
const Segment SGI389253[] = {{&S11926[0], 8}, {NULL}};
const Segment SGI389254[] = {{&S11926[38], 4}, {NULL}};
const Segment SGI389255[] = {{&S11927[0], 4}, {NULL}};
const Segment SGI389256[] = {{&S11927[35], 4}, {NULL}};
const Segment SGI389257[] = {{&S11928[0], 4}, {NULL}};
const Segment SGI389258[] = {{&S11928[34], 4}, {NULL}};
const Segment SGI389259[] = {{&S11929[0], 13}, {NULL}};
const Segment SGI389260[] = {{&S11929[45], 4}, {NULL}};
const Segment SGI389261[] = {{&S11930[0], 18}, {NULL}};
const Segment SGI389262[] = {{&S11930[50], 4}, {NULL}};
const Segment SGI389263[] = {{&S11931[0], 7}, {NULL}};
const Segment SGI389264[] = {{&S11931[37], 4}, {NULL}};
const Segment SGI389265[] = {{&S11932[0], 21}, {NULL}};
const Segment SGI389266[] = {{&S11932[53], 4}, {NULL}};
const Segment SGI389267[] = {{&S11933[0], 9}, {NULL}};
const Segment SGI389268[] = {{&S11933[37], 4}, {NULL}};
const Segment SGI389269[] = {{&S11935[0], 5}, {NULL}};
const Segment SGI389270[] = {{&S11935[35], 4}, {NULL}};
const Segment SGI389271[] = {{&S11936[0], 19}, {NULL}};
const Segment SGI389272[] = {{&S11936[49], 4}, {NULL}};
const Segment SGI389273[] = {{&S11937[0], 11}, {NULL}};
const Segment SGI389274[] = {{&S11937[41], 4}, {NULL}};
const Segment SGI389275[] = {{&S11938[0], 7}, {NULL}};
const Segment SGI389276[] = {{&S11938[38], 4}, {NULL}};
const Segment SGI389277[] = {{&S11939[0], 7}, {NULL}};
const Segment SGI389278[] = {{&S11939[37], 4}, {NULL}};
const Segment SGI389279[] = {{&S11940[0], 7}, {NULL}};
const Segment SGI389280[] = {{&S11940[37], 4}, {NULL}};
const Segment SGI389281[] = {{&S11941[0], 6}, {NULL}};
const Segment SGI389282[] = {{&S11941[35], 4}, {NULL}};
const Segment SGI389283[] = {{&S11942[0], 9}, {NULL}};
const Segment SGI389284[] = {{&S11942[41], 4}, {NULL}};
const Segment SGI389285[] = {{&S11943[0], 6}, {NULL}};
const Segment SGI389286[] = {{&S11943[34], 4}, {NULL}};
const Segment SGI389287[] = {{&S11944[0], 6}, {NULL}};
const Segment SGI389288[] = {{&S11944[36], 4}, {NULL}};
const Segment SGI389289[] = {{&S11945[0], 12}, {NULL}};
const Segment SGI389290[] = {{&S11945[44], 4}, {NULL}};
const Segment SGI389291[] = {{&S11946[0], 16}, {NULL}};
const Segment SGI389292[] = {{&S11946[49], 4}, {NULL}};
const Segment SGI389293[] = {{&S11947[0], 8}, {NULL}};
const Segment SGI389294[] = {{&S11947[38], 4}, {NULL}};
const Segment SGI389295[] = {{&S11948[0], 11}, {NULL}};
const Segment SGI389296[] = {{&S11948[44], 4}, {NULL}};
const Segment SGI389297[] = {{&S11949[0], 15}, {NULL}};
const Segment SGI389298[] = {{&S11949[45], 4}, {NULL}};
const Segment SGI389299[] = {{&S11950[0], 7}, {NULL}};
const Segment SGI389300[] = {{&S11950[37], 4}, {NULL}};
const Segment SGI389301[] = {{&S11951[0], 13}, {NULL}};
const Segment SGI389302[] = {{&S11951[45], 4}, {NULL}};
const Segment SGI389303[] = {{&S11952[0], 20}, {NULL}};
const Segment SGI389304[] = {{&S11952[51], 4}, {NULL}};
const Segment SGI389305[] = {{&S11953[0], 10}, {NULL}};
const Segment SGI389306[] = {{&S11953[40], 4}, {NULL}};
const Segment SGI389307[] = {{&S11954[0], 16}, {NULL}};
const Segment SGI389308[] = {{&S11954[46], 4}, {NULL}};
const Segment SGI389309[] = {{&S11955[0], 9}, {NULL}};
const Segment SGI389310[] = {{&S11955[38], 4}, {NULL}};
const Segment SGI389311[] = {{&S11956[0], 13}, {NULL}};
const Segment SGI389312[] = {{&S11956[46], 4}, {NULL}};
const Segment SGI389313[] = {{&S11957[0], 19}, {NULL}};
const Segment SGI389314[] = {{&S11957[49], 4}, {NULL}};
const Segment SGI389315[] = {{&S11958[0], 8}, {NULL}};
const Segment SGI389316[] = {{&S11958[38], 4}, {NULL}};
const Segment SGI389317[] = {{&S11959[0], 12}, {NULL}};
const Segment SGI389318[] = {{&S11959[43], 4}, {NULL}};
const Segment SGI389319[] = {{&S11960[0], 15}, {NULL}};
const Segment SGI389320[] = {{&S11960[45], 4}, {NULL}};
const Segment SGI389321[] = {{&S11961[0], 24}, {NULL}};
const Segment SGI389322[] = {{&S11961[54], 4}, {NULL}};
const Segment SGI389323[] = {{&S11962[0], 6}, {NULL}};
const Segment SGI389324[] = {{&S11962[39], 4}, {NULL}};
const Segment SGI389325[] = {{&S11964[0], 7}, {NULL}};
const Segment SGI389326[] = {{&S11964[39], 4}, {NULL}};
const Segment SGI389327[] = {{&S11966[0], 35}, {NULL}};
const Segment SGI389328[] = {{&S11966[51], 4}, {NULL}};
const Segment SGI389329[] = {{&S11967[0], 8}, {NULL}};
const Segment SGI389330[] = {{&S11967[38], 4}, {NULL}};
const Segment SGI389331[] = {{&S11968[0], 14}, {NULL}};
const Segment SGI389332[] = {{&S11968[45], 4}, {NULL}};
const Segment SGI389333[] = {{&S11969[0], 45}, {NULL}};
const Segment SGI389334[] = {{&S11969[76], 4}, {NULL}};
const Segment SGI389335[] = {{&S11970[0], 14}, {NULL}};
const Segment SGI389336[] = {{&S11970[46], 4}, {NULL}};
const Segment SGI389337[] = {{&S11971[0], 15}, {NULL}};
const Segment SGI389338[] = {{&S11971[54], 4}, {NULL}};
const Segment SGI389339[] = {{&S11972[0], 13}, {NULL}};
const Segment SGI389340[] = {{&S11972[43], 4}, {NULL}};
const Segment SGI389341[] = {{&S11973[0], 54}, {NULL}};
const Segment SGI389342[] = {{&S11973[85], 4}, {NULL}};
const Segment SGI389343[] = {{&S11974[0], 10}, {NULL}};
const Segment SGI389344[] = {{&S11974[42], 4}, {NULL}};
const Segment SGI389345[] = {{&S11975[0], 7}, {NULL}};
const Segment SGI389346[] = {{&S11975[39], 4}, {NULL}};
const Segment SGI389347[] = {{&S11976[0], 11}, {NULL}};
const Segment SGI389348[] = {{&S11976[43], 4}, {NULL}};
const Segment SGI389349[] = {{&S11977[0], 10}, {NULL}};
const Segment SGI389350[] = {{&S11977[42], 4}, {NULL}};
const Segment SGI389351[] = {{&S11978[0], 16}, {NULL}};
const Segment SGI389352[] = {{&S11978[46], 4}, {NULL}};
const Segment SGI389353[] = {{&S11979[0], 9}, {NULL}};
const Segment SGI389354[] = {{&S11979[39], 4}, {NULL}};
const Segment SGI389355[] = {{&S11980[0], 7}, {NULL}};
const Segment SGI389356[] = {{&S11980[36], 4}, {NULL}};
const Segment SGI389357[] = {{&S11981[0], 5}, {NULL}};
const Segment SGI389358[] = {{&S11981[42], 4}, {NULL}};
const Segment SGI389359[] = {{&S11982[0], 5}, {NULL}};
const Segment SGI389360[] = {{&S11982[37], 4}, {NULL}};
const Segment SGI389361[] = {{&S11983[0], 12}, {NULL}};
const Segment SGI389362[] = {{&S11983[43], 4}, {NULL}};
const Segment SGI389363[] = {{&S11984[0], 21}, {NULL}};
const Segment SGI389364[] = {{&S11984[53], 4}, {NULL}};
const Segment SGI389365[] = {{&S11985[0], 12}, {NULL}};
const Segment SGI389366[] = {{&S11985[43], 4}, {NULL}};
const Segment SGI389367[] = {{&S11986[0], 7}, {NULL}};
const Segment SGI389368[] = {{&S11986[37], 4}, {NULL}};
const Segment SGI389369[] = {{&S11987[0], 10}, {NULL}};
const Segment SGI389370[] = {{&S11987[40], 4}, {NULL}};
const Segment SGI389371[] = {{&S11988[0], 7}, {NULL}};
const Segment SGI389372[] = {{&S11988[39], 4}, {NULL}};
const Segment SGI389373[] = {{&S11989[0], 13}, {NULL}};
const Segment SGI389374[] = {{&S11989[43], 4}, {NULL}};
const Segment SGI389375[] = {{&S11990[0], 6}, {NULL}};
const Segment SGI389376[] = {{&S11990[36], 4}, {NULL}};
const Segment SGI389377[] = {{&S11991[0], 9}, {NULL}};
const Segment SGI389378[] = {{&S11991[41], 4}, {NULL}};
const Segment SGI389379[] = {{&S11992[0], 11}, {NULL}};
const Segment SGI389380[] = {{&S11992[41], 4}, {NULL}};
const Segment SGI389381[] = {{&S11993[0], 13}, {NULL}};
const Segment SGI389382[] = {{&S11993[45], 4}, {NULL}};
const Segment SGI389383[] = {{&S11994[0], 6}, {NULL}};
const Segment SGI389384[] = {{&S11994[38], 4}, {NULL}};
const Segment SGI389385[] = {{&S11995[0], 10}, {NULL}};
const Segment SGI389386[] = {{&S11995[40], 4}, {NULL}};
const Segment SGI389387[] = {{&S11996[0], 4}, {NULL}};
const Segment SGI389388[] = {{&S11996[34], 4}, {NULL}};
const Segment SGI389389[] = {{&S11997[0], 21}, {NULL}};
const Segment SGI389390[] = {{&S11997[53], 4}, {NULL}};
const Segment SGI389391[] = {{&S11998[0], 8}, {NULL}};
const Segment SGI389392[] = {{&S11998[38], 4}, {NULL}};
const Segment SGI389393[] = {{&S11999[0], 13}, {NULL}};
const Segment SGI389394[] = {{&S11999[43], 4}, {NULL}};
const Segment SGI389395[] = {{&S12000[0], 11}, {NULL}};
const Segment SGI389396[] = {{&S12000[44], 4}, {NULL}};
const Segment SGI389397[] = {{&S12001[0], 9}, {NULL}};
const Segment SGI389398[] = {{&S12001[39], 4}, {NULL}};
const Segment SGI389399[] = {{&S12002[0], 9}, {NULL}};
const Segment SGI389400[] = {{&S12002[39], 4}, {NULL}};
const Segment SGI389401[] = {{&S12003[0], 9}, {NULL}};
const Segment SGI389402[] = {{&S12003[40], 4}, {NULL}};
const Segment SGI389403[] = {{&S12004[0], 11}, {NULL}};
const Segment SGI389404[] = {{&S12004[43], 4}, {NULL}};
const Segment SGI389405[] = {{&S12005[0], 12}, {NULL}};
const Segment SGI389406[] = {{&S12005[43], 4}, {NULL}};
const Segment SGI389407[] = {{&S12006[0], 5}, {NULL}};
const Segment SGI389408[] = {{&S12006[35], 4}, {NULL}};
const Segment SGI389409[] = {{&S12007[0], 21}, {NULL}};
const Segment SGI389410[] = {{&S12007[53], 4}, {NULL}};
const Segment SGI389411[] = {{&S12008[0], 16}, {NULL}};
const Segment SGI389412[] = {{&S12008[46], 4}, {NULL}};
const Segment SGI389413[] = {{&S12009[0], 16}, {NULL}};
const Segment SGI389414[] = {{&S12009[48], 4}, {NULL}};
const Segment SGI389415[] = {{&S12010[0], 27}, {NULL}};
const Segment SGI389416[] = {{&S12010[57], 4}, {NULL}};
const Segment SGI389417[] = {{&S12011[0], 9}, {NULL}};
const Segment SGI389418[] = {{&S12011[40], 4}, {NULL}};
const Segment SGI389419[] = {{&S12012[0], 10}, {NULL}};
const Segment SGI389420[] = {{&S12012[43], 4}, {NULL}};
const Segment SGI389421[] = {{&S12013[0], 11}, {NULL}};
const Segment SGI389422[] = {{&S12013[41], 4}, {NULL}};
const Segment SGI389423[] = {{&S12014[0], 6}, {NULL}};
const Segment SGI389424[] = {{&S12014[36], 4}, {NULL}};
const Segment SGI389425[] = {{&S46760[93], 3}, {NULL}};
const Segment SGI389426[] = {{&S46760[29], 6}, {NULL}};
const Segment SGI389427[] = {{&S46760[128], 5}, {NULL}};
const Segment SGI389428[] = {{&S12015[0], 15}, {NULL}};
const Segment SGI389429[] = {{&S12015[47], 4}, {NULL}};
const Segment SGI389430[] = {{&S12016[0], 6}, {NULL}};
const Segment SGI389431[] = {{&S12016[37], 4}, {NULL}};
const Segment SGI389432[] = {{&S12017[0], 11}, {NULL}};
const Segment SGI389433[] = {{&S12017[43], 4}, {NULL}};
const Segment SGI389434[] = {{&S12018[0], 14}, {NULL}};
const Segment SGI389435[] = {{&S12018[49], 4}, {NULL}};
const Segment SGI389436[] = {{&S12019[0], 12}, {NULL}};
const Segment SGI389437[] = {{&S12019[43], 4}, {NULL}};
const Segment SGI389438[] = {{&S12020[0], 8}, {NULL}};
const Segment SGI389439[] = {{&S12020[38], 4}, {NULL}};
const Segment SGI389440[] = {{&S12021[0], 9}, {NULL}};
const Segment SGI389441[] = {{&S12021[39], 4}, {NULL}};
const Segment SGI389442[] = {{&S12022[0], 21}, {NULL}};
const Segment SGI389443[] = {{&S12022[51], 4}, {NULL}};
const Segment SGI389444[] = {{&S12023[0], 6}, {NULL}};
const Segment SGI389445[] = {{&S12023[36], 4}, {NULL}};
const Segment SGI389446[] = {{&S12024[0], 12}, {NULL}};
const Segment SGI389447[] = {{&S12024[44], 4}, {NULL}};
const Segment SGI389448[] = {{&S12025[0], 11}, {NULL}};
const Segment SGI389449[] = {{&S12025[41], 4}, {NULL}};
const Segment SGI389450[] = {{&S12026[0], 11}, {NULL}};
const Segment SGI389451[] = {{&S12026[42], 4}, {NULL}};
const Segment SGI389452[] = {{&S12027[0], 12}, {NULL}};
const Segment SGI389453[] = {{&S12027[43], 4}, {NULL}};
const Segment SGI389454[] = {{&S12028[0], 8}, {NULL}};
const Segment SGI389455[] = {{&S12028[40], 4}, {NULL}};
const Segment SGI389456[] = {{&S12029[0], 9}, {NULL}};
const Segment SGI389457[] = {{&S12029[39], 4}, {NULL}};
const Segment SGI389458[] = {{&S12030[0], 7}, {NULL}};
const Segment SGI389459[] = {{&S12030[37], 4}, {NULL}};
const Segment SGI389460[] = {{&S12031[0], 11}, {NULL}};
const Segment SGI389461[] = {{&S12031[41], 4}, {NULL}};
const Segment SGI389462[] = {{&S12032[0], 8}, {NULL}};
const Segment SGI389463[] = {{&S12032[38], 4}, {NULL}};
const Segment SGI389464[] = {{&S12033[0], 5}, {NULL}};
const Segment SGI389465[] = {{&S12033[34], 4}, {NULL}};
const Segment SGI389466[] = {{&S12034[0], 6}, {NULL}};
const Segment SGI389467[] = {{&S12034[37], 4}, {NULL}};
const Segment SGI389468[] = {{&S12035[0], 11}, {NULL}};
const Segment SGI389469[] = {{&S12035[43], 4}, {NULL}};
const Segment SGI389470[] = {{&S12036[0], 8}, {NULL}};
const Segment SGI389471[] = {{&S12036[38], 4}, {NULL}};
const Segment SGI389472[] = {{&S12037[0], 4}, {NULL}};
const Segment SGI389473[] = {{&S12037[36], 4}, {NULL}};
const Segment SGI389474[] = {{&S12038[0], 7}, {NULL}};
const Segment SGI389475[] = {{&S12038[36], 4}, {NULL}};
const Segment SGI389476[] = {{&S12039[0], 9}, {NULL}};
const Segment SGI389477[] = {{&S12039[41], 4}, {NULL}};
const Segment SGI389478[] = {{&S12040[0], 4}, {NULL}};
const Segment SGI389479[] = {{&S12040[36], 4}, {NULL}};
const Segment SGI389480[] = {{&S12041[0], 7}, {NULL}};
const Segment SGI389481[] = {{&S12041[37], 4}, {NULL}};
const Segment SGI389482[] = {{&S12042[0], 9}, {NULL}};
const Segment SGI389483[] = {{&S12042[39], 4}, {NULL}};
const Segment SGI389484[] = {{&S12043[0], 13}, {NULL}};
const Segment SGI389485[] = {{&S12043[43], 4}, {NULL}};
const Segment SGI389486[] = {{&S12044[0], 18}, {NULL}};
const Segment SGI389487[] = {{&S12044[48], 4}, {NULL}};
const Segment SGI389488[] = {{&S12045[0], 11}, {NULL}};
const Segment SGI389489[] = {{&S12045[42], 4}, {NULL}};
const Segment SGI389490[] = {{&S12047[0], 32}, {NULL}};
const Segment SGI389491[] = {{&S12047[62], 4}, {NULL}};
const Segment SGI389492[] = {{&S12048[0], 9}, {NULL}};
const Segment SGI389493[] = {{&S12048[41], 4}, {NULL}};
const Segment SGI389494[] = {{&S12049[0], 5}, {NULL}};
const Segment SGI389495[] = {{&S12049[35], 4}, {NULL}};
const Segment SGI389496[] = {{&S12050[0], 9}, {NULL}};
const Segment SGI389497[] = {{&S12050[46], 4}, {NULL}};
const Segment SGI389498[] = {{&S12051[0], 15}, {NULL}};
const Segment SGI389499[] = {{&S12051[44], 4}, {NULL}};
const Segment SGI389500[] = {{&S12052[0], 10}, {NULL}};
const Segment SGI389501[] = {{&S12052[41], 4}, {NULL}};
const Segment SGI389502[] = {{&S12053[0], 7}, {NULL}};
const Segment SGI389503[] = {{&S12053[39], 4}, {NULL}};
const Segment SGI389504[] = {{&S12054[0], 16}, {NULL}};
const Segment SGI389505[] = {{&S12054[46], 4}, {NULL}};
const Segment SGI389506[] = {{&S12055[0], 19}, {NULL}};
const Segment SGI389507[] = {{&S12055[49], 4}, {NULL}};
const Segment SGI389508[] = {{&S12056[0], 8}, {NULL}};
const Segment SGI389509[] = {{&S12056[38], 4}, {NULL}};
const Segment SGI389510[] = {{&S12057[0], 6}, {NULL}};
const Segment SGI389511[] = {{&S12057[36], 4}, {NULL}};
const Segment SGI389512[] = {{&S12058[0], 10}, {NULL}};
const Segment SGI389513[] = {{&S12058[42], 4}, {NULL}};
const Segment SGI389514[] = {{&S12059[0], 14}, {NULL}};
const Segment SGI389515[] = {{&S12059[44], 4}, {NULL}};
const Segment SGI389516[] = {{&S12060[0], 10}, {NULL}};
const Segment SGI389517[] = {{&S12060[42], 4}, {NULL}};
const Segment SGI389518[] = {{&S12061[0], 4}, {NULL}};
const Segment SGI389519[] = {{&S12061[34], 4}, {NULL}};
const Segment SGI389520[] = {{&S12062[0], 4}, {NULL}};
const Segment SGI389521[] = {{&S12062[33], 4}, {NULL}};
const Segment SGI389522[] = {{&S12063[0], 10}, {NULL}};
const Segment SGI389523[] = {{&S12063[40], 4}, {NULL}};
const Segment SGI389524[] = {{&S12064[0], 12}, {NULL}};
const Segment SGI389525[] = {{&S12064[44], 4}, {NULL}};
const Segment SGI389526[] = {{&S12065[0], 26}, {NULL}};
const Segment SGI389527[] = {{&S12065[56], 4}, {NULL}};
const Segment SGI389528[] = {{&S12066[0], 11}, {NULL}};
const Segment SGI389529[] = {{&S12066[41], 4}, {NULL}};
const Segment SGI389530[] = {{&S12067[0], 10}, {NULL}};
const Segment SGI389531[] = {{&S12067[40], 4}, {NULL}};
const Segment SGI389532[] = {{&S12068[0], 7}, {NULL}};
const Segment SGI389533[] = {{&S12068[39], 4}, {NULL}};
const Segment SGI389534[] = {{&S12069[0], 6}, {NULL}};
const Segment SGI389535[] = {{&S12069[35], 4}, {NULL}};
const Segment SGI389536[] = {{&S12070[0], 10}, {NULL}};
const Segment SGI389537[] = {{&S12070[40], 4}, {NULL}};
const Segment SGI389538[] = {{&S12071[0], 13}, {NULL}};
const Segment SGI389539[] = {{&S12071[43], 4}, {NULL}};
const Segment SGI389540[] = {{&S12072[0], 6}, {NULL}};
const Segment SGI389541[] = {{&S12072[38], 4}, {NULL}};
const Segment SGI389542[] = {{&S12073[0], 11}, {NULL}};
const Segment SGI389543[] = {{&S12073[40], 4}, {NULL}};
const Segment SGI389544[] = {{&S12074[0], 23}, {NULL}};
const Segment SGI389545[] = {{&S12074[55], 4}, {NULL}};
const Segment SGI389546[] = {{&S12075[0], 22}, {NULL}};
const Segment SGI389547[] = {{&S12075[51], 4}, {NULL}};
const Segment SGI389548[] = {{&S12076[0], 9}, {NULL}};
const Segment SGI389549[] = {{&S12076[41], 4}, {NULL}};
const Segment SGI389550[] = {{&S12077[0], 4}, {NULL}};
const Segment SGI389551[] = {{&S12077[36], 4}, {NULL}};
const Segment SGI389552[] = {{&S12078[0], 8}, {NULL}};
const Segment SGI389553[] = {{&S12078[40], 4}, {NULL}};
const Segment SGI389554[] = {{&S12079[0], 5}, {NULL}};
const Segment SGI389555[] = {{&S12079[43], 4}, {NULL}};
const Segment SGI389556[] = {{&S12080[0], 8}, {NULL}};
const Segment SGI389557[] = {{&S12080[40], 4}, {NULL}};
const Segment SGI389558[] = {{&S12081[0], 8}, {NULL}};
const Segment SGI389559[] = {{&S12081[38], 4}, {NULL}};
const Segment SGI389560[] = {{&S12082[0], 8}, {NULL}};
const Segment SGI389561[] = {{&S12082[38], 4}, {NULL}};
const Segment SGI389562[] = {{&S12083[0], 7}, {NULL}};
const Segment SGI389563[] = {{&S12083[37], 4}, {NULL}};
const Segment SGI389564[] = {{&S12084[0], 6}, {NULL}};
const Segment SGI389565[] = {{&S12084[36], 4}, {NULL}};
const Segment SGI389566[] = {{&S12085[0], 16}, {NULL}};
const Segment SGI389567[] = {{&S12085[47], 4}, {NULL}};
const Segment SGI389568[] = {{&S12086[0], 4}, {NULL}};
const Segment SGI389569[] = {{&S12086[36], 4}, {NULL}};
const Segment SGI389570[] = {{&S12087[0], 9}, {NULL}};
const Segment SGI389571[] = {{&S12087[41], 4}, {NULL}};
const Segment SGI389572[] = {{&S12088[0], 10}, {NULL}};
const Segment SGI389573[] = {{&S12088[40], 4}, {NULL}};
const Segment SGI389574[] = {{&S12089[0], 22}, {NULL}};
const Segment SGI389575[] = {{&S12089[57], 4}, {NULL}};
const Segment SGI389576[] = {{&S12090[0], 11}, {NULL}};
const Segment SGI389577[] = {{&S12090[40], 4}, {NULL}};
const Segment SGI389578[] = {{&S12091[0], 22}, {NULL}};
const Segment SGI389579[] = {{&S12091[52], 4}, {NULL}};
const Segment SGI389580[] = {{&S12092[0], 8}, {NULL}};
const Segment SGI389581[] = {{&S12092[38], 4}, {NULL}};
const Segment SGI389582[] = {{&S12093[0], 4}, {NULL}};
const Segment SGI389583[] = {{&S12093[33], 4}, {NULL}};
const Segment SGI389584[] = {{&S12094[0], 22}, {NULL}};
const Segment SGI389585[] = {{&S12094[52], 4}, {NULL}};
const Segment SGI389586[] = {{&S12095[0], 4}, {NULL}};
const Segment SGI389587[] = {{&S12095[34], 4}, {NULL}};
const Segment SGI389588[] = {{&S12096[0], 7}, {NULL}};
const Segment SGI389589[] = {{&S12096[37], 4}, {NULL}};
const Segment SGI389590[] = {{&S12097[0], 8}, {NULL}};
const Segment SGI389591[] = {{&S12097[38], 4}, {NULL}};
const Segment SGI389592[] = {{&S12098[0], 4}, {NULL}};
const Segment SGI389593[] = {{&S12098[36], 4}, {NULL}};
const Segment SGI389594[] = {{&S53882[8], 3}, {NULL}};
const Segment SGI389595[] = {{&S53882[13], 7}, {NULL}};
const Segment SGI389596[] = {{&S53882[72], 13}, {NULL}};
const Segment SGI389597[] = {{&S53883[8], 3}, {NULL}};
const Segment SGI389598[] = {{&S53883[13], 7}, {NULL}};
const Segment SGI389599[] = {{&S53883[25], 14}, {NULL}};
const Segment SGI389600[] = {{&S53883[69], 13}, {NULL}};
const Segment SGI389601[] = {{&S53884[8], 3}, {NULL}};
const Segment SGI389602[] = {{&S53884[13], 7}, {NULL}};
const Segment SGI389603[] = {{&S53884[25], 14}, {NULL}};
const Segment SGI389604[] = {{&S53884[66], 13}, {NULL}};
const Segment SGI389605[] = {{&S54056[8], 3}, {NULL}};
const Segment SGI389606[] = {{&S54056[13], 3}, {NULL}};
const Segment SGI389607[] = {{&S54056[13], 3}, {&S54056[18], 10}, {NULL}};
const Segment SGI389608[] = {{&S54056[47], 14}, {NULL}};
const Segment SGI389609[] = {{&S54217[8], 3}, {NULL}};
const Segment SGI389610[] = {{&S54217[13], 3}, {NULL}};
const Segment SGI389611[] = {{&S54217[13], 3}, {&S54217[18], 6}, {&S54217[26], 10}, {NULL}};
const Segment SGI389612[] = {{&S54217[55], 13}, {NULL}};
const Segment SGI389613[] = {{&S47098[108], 3}, {NULL}};
const Segment SGI389614[] = {{&S47098[44], 6}, {NULL}};
const Segment SGI389615[] = {{&S47098[30], 6}, {NULL}};
const Segment SGI389616[] = {{&S47098[150], 4}, {NULL}};
const Segment SGI389617[] = {{&S47628[93], 3}, {NULL}};
const Segment SGI389618[] = {{&S47628[29], 6}, {NULL}};
const Segment SGI389619[] = {{&S47628[128], 5}, {NULL}};
const Segment SGI389620[] = {{&S47830[93], 3}, {NULL}};
const Segment SGI389621[] = {{&S47830[29], 6}, {NULL}};
const Segment SGI389622[] = {{&S47830[128], 4}, {NULL}};
const Segment SGI389623[] = {{&S55144[8], 3}, {NULL}};
const Segment SGI389624[] = {{&S55144[13], 7}, {NULL}};
const Segment SGI389625[] = {{&S55144[25], 14}, {NULL}};
const Segment SGI389626[] = {{&S55144[71], 13}, {NULL}};
const Segment SGI389627[] = {{&S55265[8], 3}, {NULL}};
const Segment SGI389628[] = {{&S55265[13], 7}, {NULL}};
const Segment SGI389629[] = {{&S52431[39], 4}, {NULL}};
const Segment SGI389630[] = {{&S52431[8], 3}, {NULL}};
const Segment SGI389631[] = {{&S52431[13], 14}, {NULL}};
const Segment SGI389632[] = {{&S49367[93], 3}, {NULL}};
const Segment SGI389633[] = {{&S49367[29], 6}, {NULL}};
const Segment SGI389634[] = {{&S49367[128], 5}, {NULL}};
const Segment SGI389635[] = {{&S49390[93], 3}, {NULL}};
const Segment SGI389636[] = {{&S49390[29], 6}, {NULL}};
const Segment SGI389637[] = {{&S49390[128], 4}, {NULL}};
const Segment SGI389638[] = {{&S56737[8], 3}, {NULL}};
const Segment SGI389639[] = {{&S56737[13], 7}, {NULL}};
const Segment SGI389640[] = {{&S56737[66], 13}, {NULL}};
const Segment SGI389641[] = {{&S56738[8], 3}, {NULL}};
const Segment SGI389642[] = {{&S56738[13], 7}, {NULL}};
const Segment SGI389643[] = {{&S56738[25], 14}, {NULL}};
const Segment SGI389644[] = {{&S56738[68], 13}, {NULL}};
const Segment SGI389645[] = {{&S56906[8], 3}, {NULL}};
const Segment SGI389646[] = {{&S56906[13], 14}, {NULL}};
const Segment SGI389647[] = {{&S56906[53], 14}, {NULL}};
const Segment SGI389648[] = {{&S56915[8], 3}, {NULL}};
const Segment SGI389649[] = {{&S56915[13], 9}, {NULL}};
const Segment SGI389650[] = {{&S56915[31], 15}, {NULL}};
const Segment SGI389651[] = {{&S56915[75], 10}, {NULL}};
const Segment SGI389652[] = {{&S56916[8], 3}, {NULL}};
const Segment SGI389653[] = {{&S56916[13], 9}, {NULL}};
const Segment SGI389654[] = {{&S56916[64], 10}, {NULL}};
const Segment SGI389655[] = {{&S56997[8], 3}, {NULL}};
const Segment SGI389656[] = {{&S56997[13], 3}, {NULL}};
const Segment SGI389657[] = {{&S56997[13], 3}, {&S56997[18], 10}, {NULL}};
const Segment SGI389658[] = {{&S56997[48], 12}, {NULL}};
const Segment SGI389659[] = {{&S57003[8], 3}, {NULL}};
const Segment SGI389660[] = {{&S57003[13], 7}, {NULL}};
const Segment SGI389661[] = {{&S57003[25], 14}, {NULL}};
const Segment SGI389662[] = {{&S57005[8], 3}, {NULL}};
const Segment SGI389663[] = {{&S57005[13], 14}, {NULL}};
const Segment SGI389664[] = {{&S49726[93], 3}, {NULL}};
const Segment SGI389665[] = {{&S49726[29], 6}, {NULL}};
const Segment SGI389666[] = {{&S49726[128], 5}, {NULL}};
const Segment SGI389667[] = {{&S49727[93], 3}, {NULL}};
const Segment SGI389668[] = {{&S49727[29], 6}, {NULL}};
const Segment SGI389669[] = {{&S49727[128], 4}, {NULL}};
const Segment SGI389670[] = {{&S53332[81], 3}, {NULL}};
const Segment SGI389671[] = {{&S53332[8], 3}, {NULL}};
const Segment SGI389672[] = {{&S53332[13], 7}, {NULL}};
const Segment SGI389673[] = {{&S53332[25], 14}, {NULL}};
const Segment SGI389674[] = {{&S53333[81], 3}, {NULL}};
const Segment SGI389675[] = {{&S53333[8], 3}, {NULL}};
const Segment SGI389676[] = {{&S53333[13], 7}, {NULL}};
const Segment SGI389677[] = {{&S53333[25], 14}, {NULL}};
const Segment SGI389678[] = {{&S53578[32], 4}, {NULL}};
const Segment SGI389679[] = {{&S53578[8], 3}, {NULL}};
const Segment SGI389680[] = {{&S53578[13], 14}, {NULL}};
const Segment SGI389681[] = {{&S53579[39], 4}, {NULL}};
const Segment SGI389682[] = {{&S53579[8], 3}, {NULL}};
const Segment SGI389683[] = {{&S53579[13], 14}, {NULL}};
const Segment SGI389684[] = {{&S53642[81], 3}, {NULL}};
const Segment SGI389685[] = {{&S53642[8], 3}, {NULL}};
const Segment SGI389686[] = {{&S53642[13], 7}, {NULL}};
const Segment SGI389687[] = {{&S53642[25], 14}, {NULL}};
const Segment SGI389688[] = {{&S58162[8], 3}, {NULL}};
const Segment SGI389689[] = {{&S58162[13], 3}, {NULL}};
const Segment SGI389690[] = {{&S58162[13], 3}, {&S58162[21], 10}, {&S58162[63], 6}, {NULL}};
const Segment SGI389691[] = {{&S58162[63], 10}, {NULL}};
const Segment SGI389692[] = {{&S50749[108], 3}, {NULL}};
const Segment SGI389693[] = {{&S50749[44], 6}, {NULL}};
const Segment SGI389694[] = {{&S50749[30], 6}, {NULL}};
const Segment SGI389695[] = {{&S50749[150], 4}, {NULL}};
const Segment SGI389696[] = {{&S58368[8], 3}, {NULL}};
const Segment SGI389697[] = {{&S58368[13], 3}, {NULL}};
const Segment SGI389698[] = {{&S58368[49], 14}, {NULL}};
const Segment SGI389699[] = {{&S50848[108], 3}, {NULL}};
const Segment SGI389700[] = {{&S50848[44], 6}, {NULL}};
const Segment SGI389701[] = {{&S50848[30], 6}, {NULL}};
const Segment SGI389702[] = {{&S50848[150], 4}, {NULL}};
const Segment SGI389703[] = {{&S58664[8], 3}, {NULL}};
const Segment SGI389704[] = {{&S58664[13], 14}, {NULL}};
const Segment SGI389705[] = {{&S58664[54], 11}, {NULL}};
const Segment SGI389706[] = {{&S58823[8], 3}, {NULL}};
const Segment SGI389707[] = {{&S58823[13], 7}, {NULL}};
const Segment SGI389708[] = {{&S58823[25], 14}, {NULL}};
const Segment SGI389709[] = {{&S58823[69], 13}, {NULL}};
const Segment SGI389710[] = {{&S58824[8], 3}, {NULL}};
const Segment SGI389711[] = {{&S58824[13], 7}, {NULL}};
const Segment SGI389712[] = {{&S58824[25], 14}, {NULL}};
const Segment SGI389713[] = {{&S58824[86], 10}, {NULL}};
const Segment SGI389714[] = {{&S58824[69], 13}, {NULL}};
const Segment SGI389715[] = {{&S55043[81], 3}, {NULL}};
const Segment SGI389716[] = {{&S55043[8], 3}, {NULL}};
const Segment SGI389717[] = {{&S55043[13], 7}, {NULL}};
const Segment SGI389718[] = {{&S55043[25], 14}, {NULL}};
const Segment SGI389719[] = {{&S51583[108], 3}, {NULL}};
const Segment SGI389720[] = {{&S51583[44], 6}, {NULL}};
const Segment SGI389721[] = {{&S51583[30], 6}, {NULL}};
const Segment SGI389722[] = {{&S51583[150], 4}, {NULL}};
const Segment SGI389723[] = {{&S62922[8], 3}, {NULL}};
const Segment SGI389724[] = {{&S62922[13], 7}, {NULL}};
const Segment SGI389725[] = {{&S62922[25], 14}, {NULL}};
const Segment SGI389726[] = {{&S62922[85], 49}, {NULL}};
const Segment SGI389727[] = {{&S62923[8], 3}, {NULL}};
const Segment SGI389728[] = {{&S62923[13], 7}, {NULL}};
const Segment SGI389729[] = {{&S62923[25], 14}, {NULL}};
const Segment SGI389730[] = {{&S62923[85], 51}, {NULL}};
const Segment SGI389731[] = {{&S62924[8], 3}, {NULL}};
const Segment SGI389732[] = {{&S62924[13], 7}, {NULL}};
const Segment SGI389733[] = {{&S62924[25], 14}, {NULL}};
const Segment SGI389734[] = {{&S62924[85], 51}, {NULL}};
const Segment SGI389735[] = {{&S62925[8], 3}, {NULL}};
const Segment SGI389736[] = {{&S62925[13], 7}, {NULL}};
const Segment SGI389737[] = {{&S62925[25], 14}, {NULL}};
const Segment SGI389738[] = {{&S62925[85], 51}, {NULL}};
const Segment SGI389739[] = {{&S62926[8], 3}, {NULL}};
const Segment SGI389740[] = {{&S62926[13], 7}, {NULL}};
const Segment SGI389741[] = {{&S62926[25], 14}, {NULL}};
const Segment SGI389742[] = {{&S62926[85], 51}, {NULL}};
const Segment SGI389743[] = {{&S62927[8], 3}, {NULL}};
const Segment SGI389744[] = {{&S62927[13], 7}, {NULL}};
const Segment SGI389745[] = {{&S62927[25], 14}, {NULL}};
const Segment SGI389746[] = {{&S62927[85], 51}, {NULL}};
const Segment SGI389747[] = {{&S51724[108], 3}, {NULL}};
const Segment SGI389748[] = {{&S51724[44], 6}, {NULL}};
const Segment SGI389749[] = {{&S51724[30], 6}, {NULL}};
const Segment SGI389750[] = {{&S51724[150], 4}, {NULL}};
const Segment SGI389751[] = {{&S51756[108], 3}, {NULL}};
const Segment SGI389752[] = {{&S51756[44], 6}, {NULL}};
const Segment SGI389753[] = {{&S51756[30], 6}, {NULL}};
const Segment SGI389754[] = {{&S51756[150], 4}, {NULL}};
const Segment SGI389755[] = {{&S51767[93], 3}, {NULL}};
const Segment SGI389756[] = {{&S51767[29], 6}, {NULL}};
const Segment SGI389757[] = {{&S51767[128], 5}, {NULL}};
const Segment SGI389758[] = {{&S59553[8], 3}, {NULL}};
const Segment SGI389759[] = {{&S59553[13], 14}, {NULL}};
const Segment SGI389760[] = {{&S59553[46], 14}, {NULL}};
const Segment SGI389761[] = {{&S59769[8], 3}, {NULL}};
const Segment SGI389762[] = {{&S59769[13], 14}, {NULL}};
const Segment SGI389763[] = {{&S59769[53], 13}, {NULL}};
const Segment SGI389764[] = {{&S60002[8], 3}, {NULL}};
const Segment SGI389765[] = {{&S60002[13], 9}, {NULL}};
const Segment SGI389766[] = {{&S60002[30], 13}, {NULL}};
const Segment SGI389767[] = {{&S60002[63], 11}, {NULL}};
const Segment SGI389768[] = {{&S52203[94], 3}, {NULL}};
const Segment SGI389769[] = {{&S52203[29], 7}, {NULL}};
const Segment SGI389770[] = {{&S52203[129], 4}, {NULL}};
const Segment SGI389771[] = {{&S52256[94], 3}, {NULL}};
const Segment SGI389772[] = {{&S52256[29], 7}, {NULL}};
const Segment SGI389773[] = {{&S52256[129], 4}, {NULL}};
const Segment SGI389774[] = {{&S52334[93], 3}, {NULL}};
const Segment SGI389775[] = {{&S52334[29], 6}, {NULL}};
const Segment SGI389776[] = {{&S52334[128], 5}, {NULL}};
const Segment SGI389777[] = {{&S52415[94], 3}, {NULL}};
const Segment SGI389778[] = {{&S52415[29], 7}, {NULL}};
const Segment SGI389779[] = {{&S52415[129], 4}, {NULL}};
const Segment SGI389780[] = {{&S60272[8], 3}, {NULL}};
const Segment SGI389781[] = {{&S60272[13], 14}, {NULL}};
const Segment SGI389782[] = {{&S60272[53], 14}, {NULL}};
const Segment SGI389783[] = {{&S52537[108], 3}, {NULL}};
const Segment SGI389784[] = {{&S52537[44], 6}, {NULL}};
const Segment SGI389785[] = {{&S52537[30], 6}, {NULL}};
const Segment SGI389786[] = {{&S52537[150], 4}, {NULL}};
const Segment SGI389787[] = {{&S60486[8], 3}, {NULL}};
const Segment SGI389788[] = {{&S60486[13], 9}, {NULL}};
const Segment SGI389789[] = {{&S60486[30], 13}, {NULL}};
const Segment SGI389790[] = {{&S60486[62], 14}, {NULL}};
const Segment SGI389791[] = {{&S52935[108], 3}, {NULL}};
const Segment SGI389792[] = {{&S52935[44], 6}, {NULL}};
const Segment SGI389793[] = {{&S52935[30], 6}, {NULL}};
const Segment SGI389794[] = {{&S52935[150], 4}, {NULL}};
const Segment SGI389795[] = {{&S52977[93], 3}, {NULL}};
const Segment SGI389796[] = {{&S52977[29], 6}, {NULL}};
const Segment SGI389797[] = {{&S52977[128], 5}, {NULL}};
const Segment SGI389798[] = {{&S60987[8], 3}, {NULL}};
const Segment SGI389799[] = {{&S60987[13], 14}, {NULL}};
const Segment SGI389800[] = {{&S60987[46], 14}, {NULL}};
const Segment SGI389801[] = {{&S56961[32], 4}, {NULL}};
const Segment SGI389802[] = {{&S56961[8], 3}, {NULL}};
const Segment SGI389803[] = {{&S56961[13], 14}, {NULL}};
const Segment SGI389804[] = {{&S64249[8], 3}, {NULL}};
const Segment SGI389805[] = {{&S64249[13], 7}, {NULL}};
const Segment SGI389806[] = {{&S64249[25], 14}, {NULL}};
const Segment SGI389807[] = {{&S64249[95], 20}, {NULL}};
const Segment SGI389808[] = {{&S64250[8], 3}, {NULL}};
const Segment SGI389809[] = {{&S64250[13], 7}, {NULL}};
const Segment SGI389810[] = {{&S64250[25], 14}, {NULL}};
const Segment SGI389811[] = {{&S64250[95], 20}, {NULL}};
const Segment SGI389812[] = {{&S64251[8], 3}, {NULL}};
const Segment SGI389813[] = {{&S64251[13], 7}, {NULL}};
const Segment SGI389814[] = {{&S64251[25], 14}, {NULL}};
const Segment SGI389815[] = {{&S64251[95], 20}, {NULL}};
const Segment SGI389816[] = {{&S64252[8], 3}, {NULL}};
const Segment SGI389817[] = {{&S64252[13], 7}, {NULL}};
const Segment SGI389818[] = {{&S64252[25], 14}, {NULL}};
const Segment SGI389819[] = {{&S64252[95], 20}, {NULL}};
const Segment SGI389820[] = {{&S53835[93], 3}, {NULL}};
const Segment SGI389821[] = {{&S53835[29], 6}, {NULL}};
const Segment SGI389822[] = {{&S53835[128], 5}, {NULL}};
const Segment SGI389823[] = {{&S53855[93], 3}, {NULL}};
const Segment SGI389824[] = {{&S53855[29], 6}, {NULL}};
const Segment SGI389825[] = {{&S53855[128], 5}, {NULL}};
const Segment SGI389826[] = {{&S61544[8], 3}, {NULL}};
const Segment SGI389827[] = {{&S61544[13], 7}, {NULL}};
const Segment SGI389828[] = {{&S54042[93], 3}, {NULL}};
const Segment SGI389829[] = {{&S54042[29], 6}, {NULL}};
const Segment SGI389830[] = {{&S54042[128], 5}, {NULL}};
const Segment SGI389831[] = {{&S57847[78], 3}, {NULL}};
const Segment SGI389832[] = {{&S57847[8], 3}, {NULL}};
const Segment SGI389833[] = {{&S57847[13], 7}, {NULL}};
const Segment SGI389834[] = {{&S57847[25], 14}, {NULL}};
const Segment SGI389835[] = {{&S54473[93], 3}, {NULL}};
const Segment SGI389836[] = {{&S54473[29], 6}, {NULL}};
const Segment SGI389837[] = {{&S54473[128], 4}, {NULL}};
const Segment SGI389838[] = {{&S54544[93], 3}, {NULL}};
const Segment SGI389839[] = {{&S54544[29], 6}, {NULL}};
const Segment SGI389840[] = {{&S54544[128], 5}, {NULL}};
const Segment SGI389841[] = {{&S62431[8], 3}, {NULL}};
const Segment SGI389842[] = {{&S62431[13], 3}, {NULL}};
const Segment SGI389843[] = {{&S62431[61], 6}, {NULL}};
const Segment SGI389844[] = {{&S62432[8], 3}, {NULL}};
const Segment SGI389845[] = {{&S62432[13], 3}, {NULL}};
const Segment SGI389846[] = {{&S62432[13], 3}, {&S62432[21], 10}, {NULL}};
const Segment SGI389847[] = {{&S62432[68], 6}, {NULL}};
const Segment SGI389848[] = {{&S62466[8], 3}, {NULL}};
const Segment SGI389849[] = {{&S62466[13], 14}, {NULL}};
const Segment SGI389850[] = {{&S62466[46], 13}, {NULL}};
const Segment SGI389851[] = {{&S20670[0], 3}, {NULL}};
const Segment SGI389852[] = {{&S20670[33], 4}, {NULL}};
const Segment SGI389853[] = {{&S20671[0], 18}, {NULL}};
const Segment SGI389854[] = {{&S20671[51], 4}, {NULL}};
const Segment SGI389855[] = {{&S20672[0], 17}, {NULL}};
const Segment SGI389856[] = {{&S20672[50], 4}, {NULL}};
const Segment SGI389857[] = {{&S20673[0], 5}, {NULL}};
const Segment SGI389858[] = {{&S20673[37], 4}, {NULL}};
const Segment SGI389859[] = {{&S20674[0], 9}, {NULL}};
const Segment SGI389860[] = {{&S20674[42], 4}, {NULL}};
const Segment SGI389861[] = {{&S20675[0], 7}, {NULL}};
const Segment SGI389862[] = {{&S20675[38], 4}, {NULL}};
const Segment SGI389863[] = {{&S20676[0], 37}, {NULL}};
const Segment SGI389864[] = {{&S20676[54], 4}, {NULL}};
const Segment SGI389865[] = {{&S20677[0], 17}, {NULL}};
const Segment SGI389866[] = {{&S20677[48], 4}, {NULL}};
const Segment SGI389867[] = {{&S20678[0], 11}, {NULL}};
const Segment SGI389868[] = {{&S20678[45], 4}, {NULL}};
const Segment SGI389869[] = {{&S20679[0], 9}, {NULL}};
const Segment SGI389870[] = {{&S20679[41], 4}, {NULL}};
const Segment SGI389871[] = {{&S20680[0], 7}, {NULL}};
const Segment SGI389872[] = {{&S20680[40], 4}, {NULL}};
const Segment SGI389873[] = {{&S20681[0], 12}, {NULL}};
const Segment SGI389874[] = {{&S20681[45], 4}, {NULL}};
const Segment SGI389875[] = {{&S20682[0], 4}, {NULL}};
const Segment SGI389876[] = {{&S20682[34], 4}, {NULL}};
const Segment SGI389877[] = {{&S20683[0], 10}, {NULL}};
const Segment SGI389878[] = {{&S20683[40], 4}, {NULL}};
const Segment SGI389879[] = {{&S20684[0], 17}, {NULL}};
const Segment SGI389880[] = {{&S20684[47], 4}, {NULL}};
const Segment SGI389881[] = {{&S20685[0], 14}, {NULL}};
const Segment SGI389882[] = {{&S20685[47], 4}, {NULL}};
const Segment SGI389883[] = {{&S20686[0], 10}, {NULL}};
const Segment SGI389884[] = {{&S20686[43], 4}, {NULL}};
const Segment SGI389885[] = {{&S20687[0], 11}, {NULL}};
const Segment SGI389886[] = {{&S20687[43], 4}, {NULL}};
const Segment SGI389887[] = {{&S20688[0], 12}, {NULL}};
const Segment SGI389888[] = {{&S20688[43], 4}, {NULL}};
const Segment SGI389889[] = {{&S20689[0], 11}, {NULL}};
const Segment SGI389890[] = {{&S20689[41], 4}, {NULL}};
const Segment SGI389891[] = {{&S20690[0], 11}, {NULL}};
const Segment SGI389892[] = {{&S20690[40], 4}, {NULL}};
const Segment SGI389893[] = {{&S20691[0], 12}, {NULL}};
const Segment SGI389894[] = {{&S20691[44], 4}, {NULL}};
const Segment SGI389895[] = {{&S20692[0], 18}, {NULL}};
const Segment SGI389896[] = {{&S20692[51], 4}, {NULL}};
const Segment SGI389897[] = {{&S20693[0], 13}, {NULL}};
const Segment SGI389898[] = {{&S20693[44], 4}, {NULL}};
const Segment SGI389899[] = {{&S20694[0], 8}, {NULL}};
const Segment SGI389900[] = {{&S20694[38], 4}, {NULL}};
const Segment SGI389901[] = {{&S20695[0], 8}, {NULL}};
const Segment SGI389902[] = {{&S20695[38], 4}, {NULL}};
const Segment SGI389903[] = {{&S58616[39], 4}, {NULL}};
const Segment SGI389904[] = {{&S58616[8], 3}, {NULL}};
const Segment SGI389905[] = {{&S58616[13], 14}, {NULL}};
const Segment SGI389906[] = {{&S20696[0], 11}, {NULL}};
const Segment SGI389907[] = {{&S20696[43], 4}, {NULL}};
const Segment SGI389908[] = {{&S20697[0], 14}, {NULL}};
const Segment SGI389909[] = {{&S20697[46], 4}, {NULL}};
const Segment SGI389910[] = {{&S20698[0], 10}, {NULL}};
const Segment SGI389911[] = {{&S20698[40], 4}, {NULL}};
const Segment SGI389912[] = {{&S20699[0], 15}, {NULL}};
const Segment SGI389913[] = {{&S20699[45], 4}, {NULL}};
const Segment SGI389914[] = {{&S20700[0], 5}, {NULL}};
const Segment SGI389915[] = {{&S20700[37], 4}, {NULL}};
const Segment SGI389916[] = {{&S20701[0], 16}, {NULL}};
const Segment SGI389917[] = {{&S20701[49], 4}, {NULL}};
const Segment SGI389918[] = {{&S20702[0], 14}, {NULL}};
const Segment SGI389919[] = {{&S20702[44], 4}, {NULL}};
const Segment SGI389920[] = {{&S20703[0], 9}, {NULL}};
const Segment SGI389921[] = {{&S20703[41], 4}, {NULL}};
const Segment SGI389922[] = {{&S20704[0], 10}, {NULL}};
const Segment SGI389923[] = {{&S20704[40], 4}, {NULL}};
const Segment SGI389924[] = {{&S20705[0], 4}, {NULL}};
const Segment SGI389925[] = {{&S20705[34], 4}, {NULL}};
const Segment SGI389926[] = {{&S20706[0], 14}, {NULL}};
const Segment SGI389927[] = {{&S20706[46], 4}, {NULL}};
const Segment SGI389928[] = {{&S20707[0], 7}, {NULL}};
const Segment SGI389929[] = {{&S20707[39], 4}, {NULL}};
const Segment SGI389930[] = {{&S20708[0], 11}, {NULL}};
const Segment SGI389931[] = {{&S20708[40], 4}, {NULL}};
const Segment SGI389932[] = {{&S20709[0], 8}, {NULL}};
const Segment SGI389933[] = {{&S20709[36], 4}, {NULL}};
const Segment SGI389934[] = {{&S20710[0], 11}, {NULL}};
const Segment SGI389935[] = {{&S20710[44], 4}, {NULL}};
const Segment SGI389936[] = {{&S20711[0], 15}, {NULL}};
const Segment SGI389937[] = {{&S20711[45], 4}, {NULL}};
const Segment SGI389938[] = {{&S20712[0], 11}, {NULL}};
const Segment SGI389939[] = {{&S20712[41], 4}, {NULL}};
const Segment SGI389940[] = {{&S20713[0], 9}, {NULL}};
const Segment SGI389941[] = {{&S20713[42], 4}, {NULL}};
const Segment SGI389942[] = {{&S20714[0], 11}, {NULL}};
const Segment SGI389943[] = {{&S20714[43], 4}, {NULL}};
const Segment SGI389944[] = {{&S20715[0], 11}, {NULL}};
const Segment SGI389945[] = {{&S20715[43], 4}, {NULL}};
const Segment SGI389946[] = {{&S20716[0], 11}, {NULL}};
const Segment SGI389947[] = {{&S20716[44], 4}, {NULL}};
const Segment SGI389948[] = {{&S20717[0], 17}, {NULL}};
const Segment SGI389949[] = {{&S20717[47], 4}, {NULL}};
const Segment SGI389950[] = {{&S20718[0], 17}, {NULL}};
const Segment SGI389951[] = {{&S20718[49], 4}, {NULL}};
const Segment SGI389952[] = {{&S20719[0], 17}, {NULL}};
const Segment SGI389953[] = {{&S20719[49], 4}, {NULL}};
const Segment SGI389954[] = {{&S20720[0], 16}, {NULL}};
const Segment SGI389955[] = {{&S20720[48], 4}, {NULL}};
const Segment SGI389956[] = {{&S20721[0], 6}, {NULL}};
const Segment SGI389957[] = {{&S20721[36], 4}, {NULL}};
const Segment SGI389958[] = {{&S20722[0], 13}, {NULL}};
const Segment SGI389959[] = {{&S20722[45], 4}, {NULL}};
const Segment SGI389960[] = {{&S20723[0], 8}, {NULL}};
const Segment SGI389961[] = {{&S20723[40], 4}, {NULL}};
const Segment SGI389962[] = {{&S20724[0], 18}, {NULL}};
const Segment SGI389963[] = {{&S20724[50], 4}, {NULL}};
const Segment SGI389964[] = {{&S20725[0], 12}, {NULL}};
const Segment SGI389965[] = {{&S20725[44], 4}, {NULL}};
const Segment SGI389966[] = {{&S20726[0], 13}, {NULL}};
const Segment SGI389967[] = {{&S20726[43], 4}, {NULL}};
const Segment SGI389968[] = {{&S20727[0], 17}, {NULL}};
const Segment SGI389969[] = {{&S20727[46], 4}, {NULL}};
const Segment SGI389970[] = {{&S20728[0], 18}, {NULL}};
const Segment SGI389971[] = {{&S20728[47], 4}, {NULL}};
const Segment SGI389972[] = {{&S20729[0], 13}, {NULL}};
const Segment SGI389973[] = {{&S20729[42], 4}, {NULL}};
const Segment SGI389974[] = {{&S20730[0], 7}, {NULL}};
const Segment SGI389975[] = {{&S20730[39], 4}, {NULL}};
const Segment SGI389976[] = {{&S20731[0], 13}, {NULL}};
const Segment SGI389977[] = {{&S20731[45], 4}, {NULL}};
const Segment SGI389978[] = {{&S20732[0], 8}, {NULL}};
const Segment SGI389979[] = {{&S20732[40], 4}, {NULL}};
const Segment SGI389980[] = {{&S20733[0], 11}, {NULL}};
const Segment SGI389981[] = {{&S20733[43], 4}, {NULL}};
const Segment SGI389982[] = {{&S20734[0], 6}, {NULL}};
const Segment SGI389983[] = {{&S20734[36], 4}, {NULL}};
const Segment SGI389984[] = {{&S20735[0], 11}, {NULL}};
const Segment SGI389985[] = {{&S20735[40], 4}, {NULL}};
const Segment SGI389986[] = {{&S20736[0], 11}, {NULL}};
const Segment SGI389987[] = {{&S20736[41], 4}, {NULL}};
const Segment SGI389988[] = {{&S20737[0], 6}, {NULL}};
const Segment SGI389989[] = {{&S20737[38], 4}, {NULL}};
const Segment SGI389990[] = {{&S20738[0], 5}, {NULL}};
const Segment SGI389991[] = {{&S20738[37], 4}, {NULL}};
const Segment SGI389992[] = {{&S20739[0], 19}, {NULL}};
const Segment SGI389993[] = {{&S20739[51], 4}, {NULL}};
const Segment SGI389994[] = {{&S20740[0], 8}, {NULL}};
const Segment SGI389995[] = {{&S20740[38], 4}, {NULL}};
const Segment SGI389996[] = {{&S20741[0], 12}, {NULL}};
const Segment SGI389997[] = {{&S20741[44], 4}, {NULL}};
const Segment SGI389998[] = {{&S20742[0], 10}, {NULL}};
const Segment SGI389999[] = {{&S20742[42], 4}, {NULL}};
const Segment SGI390000[] = {{&S20743[0], 7}, {NULL}};
const Segment SGI390001[] = {{&S20743[38], 4}, {NULL}};
const Segment SGI390002[] = {{&S20744[0], 11}, {NULL}};
const Segment SGI390003[] = {{&S20744[43], 4}, {NULL}};
const Segment SGI390004[] = {{&S20745[0], 10}, {NULL}};
const Segment SGI390005[] = {{&S20745[40], 4}, {NULL}};
const Segment SGI390006[] = {{&S20746[0], 14}, {NULL}};
const Segment SGI390007[] = {{&S20746[44], 4}, {NULL}};
const Segment SGI390008[] = {{&S20747[0], 5}, {NULL}};
const Segment SGI390009[] = {{&S20747[35], 4}, {NULL}};
const Segment SGI390010[] = {{&S20748[0], 5}, {NULL}};
const Segment SGI390011[] = {{&S20748[40], 4}, {NULL}};
const Segment SGI390012[] = {{&S20749[0], 7}, {NULL}};
const Segment SGI390013[] = {{&S20749[39], 4}, {NULL}};
const Segment SGI390014[] = {{&S20750[0], 12}, {NULL}};
const Segment SGI390015[] = {{&S20750[42], 4}, {NULL}};
const Segment SGI390016[] = {{&S20751[0], 9}, {NULL}};
const Segment SGI390017[] = {{&S20751[39], 4}, {NULL}};
const Segment SGI390018[] = {{&S20752[0], 12}, {NULL}};
const Segment SGI390019[] = {{&S20752[44], 4}, {NULL}};
const Segment SGI390020[] = {{&S20753[0], 5}, {NULL}};
const Segment SGI390021[] = {{&S20753[37], 4}, {NULL}};
const Segment SGI390022[] = {{&S20754[0], 16}, {NULL}};
const Segment SGI390023[] = {{&S20754[51], 4}, {NULL}};
const Segment SGI390024[] = {{&S20755[0], 5}, {NULL}};
const Segment SGI390025[] = {{&S20755[37], 4}, {NULL}};
const Segment SGI390026[] = {{&S20756[0], 11}, {NULL}};
const Segment SGI390027[] = {{&S20756[43], 4}, {NULL}};
const Segment SGI390028[] = {{&S20757[0], 19}, {NULL}};
const Segment SGI390029[] = {{&S20757[47], 4}, {NULL}};
const Segment SGI390030[] = {{&S20758[0], 3}, {NULL}};
const Segment SGI390031[] = {{&S20758[35], 4}, {NULL}};
const Segment SGI390032[] = {{&S20759[0], 13}, {NULL}};
const Segment SGI390033[] = {{&S20759[43], 4}, {NULL}};
const Segment SGI390034[] = {{&S20760[0], 12}, {NULL}};
const Segment SGI390035[] = {{&S20760[44], 4}, {NULL}};
const Segment SGI390036[] = {{&S20763[0], 10}, {NULL}};
const Segment SGI390037[] = {{&S20763[40], 4}, {NULL}};
const Segment SGI390038[] = {{&S20764[0], 14}, {NULL}};
const Segment SGI390039[] = {{&S20764[44], 4}, {NULL}};
const Segment SGI390040[] = {{&S20765[0], 12}, {NULL}};
const Segment SGI390041[] = {{&S20765[44], 4}, {NULL}};
const Segment SGI390042[] = {{&S20766[0], 5}, {NULL}};
const Segment SGI390043[] = {{&S20766[35], 4}, {NULL}};
const Segment SGI390044[] = {{&S20767[0], 16}, {NULL}};
const Segment SGI390045[] = {{&S20767[47], 4}, {NULL}};
const Segment SGI390046[] = {{&S20768[0], 9}, {NULL}};
const Segment SGI390047[] = {{&S20768[39], 4}, {NULL}};
const Segment SGI390048[] = {{&S20769[0], 4}, {NULL}};
const Segment SGI390049[] = {{&S20769[33], 4}, {NULL}};
const Segment SGI390050[] = {{&S20770[0], 4}, {NULL}};
const Segment SGI390051[] = {{&S20770[34], 4}, {NULL}};
const Segment SGI390052[] = {{&S20771[0], 15}, {NULL}};
const Segment SGI390053[] = {{&S20771[47], 4}, {NULL}};
const Segment SGI390054[] = {{&S20772[0], 22}, {NULL}};
const Segment SGI390055[] = {{&S20772[54], 4}, {NULL}};
const Segment SGI390056[] = {{&S20773[0], 6}, {NULL}};
const Segment SGI390057[] = {{&S20773[36], 4}, {NULL}};
const Segment SGI390058[] = {{&S20774[0], 21}, {NULL}};
const Segment SGI390059[] = {{&S20774[51], 4}, {NULL}};
const Segment SGI390060[] = {{&S20775[0], 11}, {NULL}};
const Segment SGI390061[] = {{&S20775[42], 4}, {NULL}};
const Segment SGI390062[] = {{&S20776[0], 15}, {NULL}};
const Segment SGI390063[] = {{&S20776[45], 4}, {NULL}};
const Segment SGI390064[] = {{&S20777[0], 13}, {NULL}};
const Segment SGI390065[] = {{&S20777[46], 4}, {NULL}};
const Segment SGI390066[] = {{&S20778[0], 4}, {NULL}};
const Segment SGI390067[] = {{&S20778[38], 4}, {NULL}};
const Segment SGI390068[] = {{&S20779[0], 13}, {NULL}};
const Segment SGI390069[] = {{&S20779[45], 4}, {NULL}};
const Segment SGI390070[] = {{&S20780[0], 13}, {NULL}};
const Segment SGI390071[] = {{&S20780[45], 4}, {NULL}};
const Segment SGI390072[] = {{&S20781[0], 9}, {NULL}};
const Segment SGI390073[] = {{&S20781[41], 4}, {NULL}};
const Segment SGI390074[] = {{&S20782[0], 7}, {NULL}};
const Segment SGI390075[] = {{&S20782[37], 4}, {NULL}};
const Segment SGI390076[] = {{&S20783[0], 9}, {NULL}};
const Segment SGI390077[] = {{&S20783[41], 4}, {NULL}};
const Segment SGI390078[] = {{&S20784[0], 11}, {NULL}};
const Segment SGI390079[] = {{&S20784[46], 4}, {NULL}};
const Segment SGI390080[] = {{&S20785[0], 8}, {NULL}};
const Segment SGI390081[] = {{&S20785[38], 4}, {NULL}};
const Segment SGI390082[] = {{&S20786[0], 8}, {NULL}};
const Segment SGI390083[] = {{&S20786[38], 4}, {NULL}};
const Segment SGI390084[] = {{&S20787[0], 10}, {NULL}};
const Segment SGI390085[] = {{&S20787[43], 4}, {NULL}};
const Segment SGI390086[] = {{&S20788[0], 14}, {NULL}};
const Segment SGI390087[] = {{&S20788[44], 4}, {NULL}};
const Segment SGI390088[] = {{&S20789[0], 14}, {NULL}};
const Segment SGI390089[] = {{&S20789[44], 4}, {NULL}};
const Segment SGI390090[] = {{&S20790[0], 11}, {NULL}};
const Segment SGI390091[] = {{&S20790[43], 4}, {NULL}};
const Segment SGI390092[] = {{&S20791[0], 12}, {NULL}};
const Segment SGI390093[] = {{&S20791[42], 4}, {NULL}};
const Segment SGI390094[] = {{&S20792[0], 13}, {NULL}};
const Segment SGI390095[] = {{&S20792[47], 4}, {NULL}};
const Segment SGI390096[] = {{&S20793[0], 13}, {NULL}};
const Segment SGI390097[] = {{&S20793[45], 4}, {NULL}};
const Segment SGI390098[] = {{&S20794[0], 14}, {NULL}};
const Segment SGI390099[] = {{&S20794[47], 4}, {NULL}};
const Segment SGI390100[] = {{&S20795[0], 4}, {NULL}};
const Segment SGI390101[] = {{&S20795[34], 4}, {NULL}};
const Segment SGI390102[] = {{&S20796[0], 7}, {NULL}};
const Segment SGI390103[] = {{&S20796[35], 4}, {NULL}};
const Segment SGI390104[] = {{&S20797[0], 15}, {NULL}};
const Segment SGI390105[] = {{&S20797[45], 4}, {NULL}};
const Segment SGI390106[] = {{&S20798[0], 20}, {NULL}};
const Segment SGI390107[] = {{&S20798[51], 4}, {NULL}};
const Segment SGI390108[] = {{&S20799[0], 17}, {NULL}};
const Segment SGI390109[] = {{&S20799[47], 4}, {NULL}};
const Segment SGI390110[] = {{&S20800[0], 7}, {NULL}};
const Segment SGI390111[] = {{&S20800[36], 4}, {NULL}};
const Segment SGI390112[] = {{&S20801[0], 9}, {NULL}};
const Segment SGI390113[] = {{&S20801[41], 4}, {NULL}};
const Segment SGI390114[] = {{&S20802[0], 14}, {NULL}};
const Segment SGI390115[] = {{&S20802[44], 4}, {NULL}};
const Segment SGI390116[] = {{&S20803[0], 11}, {NULL}};
const Segment SGI390117[] = {{&S20803[42], 4}, {NULL}};
const Segment SGI390118[] = {{&S20804[0], 12}, {NULL}};
const Segment SGI390119[] = {{&S20804[42], 4}, {NULL}};
const Segment SGI390120[] = {{&S20805[0], 9}, {NULL}};
const Segment SGI390121[] = {{&S20805[41], 4}, {NULL}};
const Segment SGI390122[] = {{&S20806[0], 8}, {NULL}};
const Segment SGI390123[] = {{&S20806[38], 4}, {NULL}};
const Segment SGI390124[] = {{&S20807[0], 16}, {NULL}};
const Segment SGI390125[] = {{&S20807[49], 4}, {NULL}};
const Segment SGI390126[] = {{&S20808[0], 10}, {NULL}};
const Segment SGI390127[] = {{&S20808[43], 4}, {NULL}};
const Segment SGI390128[] = {{&S20809[0], 7}, {NULL}};
const Segment SGI390129[] = {{&S20809[39], 4}, {NULL}};
const Segment SGI390130[] = {{&S20810[0], 10}, {NULL}};
const Segment SGI390131[] = {{&S20810[40], 4}, {NULL}};
const Segment SGI390132[] = {{&S20811[0], 16}, {NULL}};
const Segment SGI390133[] = {{&S20811[49], 4}, {NULL}};
const Segment SGI390134[] = {{&S20812[0], 7}, {NULL}};
const Segment SGI390135[] = {{&S20812[37], 4}, {NULL}};
const Segment SGI390136[] = {{&S20813[0], 22}, {NULL}};
const Segment SGI390137[] = {{&S20813[56], 4}, {NULL}};
const Segment SGI390138[] = {{&S20814[0], 8}, {NULL}};
const Segment SGI390139[] = {{&S20814[40], 4}, {NULL}};
const Segment SGI390140[] = {{&S20815[0], 10}, {NULL}};
const Segment SGI390141[] = {{&S20815[40], 4}, {NULL}};
const Segment SGI390142[] = {{&S20816[0], 6}, {NULL}};
const Segment SGI390143[] = {{&S20816[38], 4}, {NULL}};
const Segment SGI390144[] = {{&S20817[0], 8}, {NULL}};
const Segment SGI390145[] = {{&S20817[38], 4}, {NULL}};
const Segment SGI390146[] = {{&S20818[0], 14}, {NULL}};
const Segment SGI390147[] = {{&S20818[44], 4}, {NULL}};
const Segment SGI390148[] = {{&S20819[0], 11}, {NULL}};
const Segment SGI390149[] = {{&S20819[42], 4}, {NULL}};
const Segment SGI390150[] = {{&S20820[0], 11}, {NULL}};
const Segment SGI390151[] = {{&S20820[43], 4}, {NULL}};
const Segment SGI390152[] = {{&S20821[0], 19}, {NULL}};
const Segment SGI390153[] = {{&S20821[51], 4}, {NULL}};
const Segment SGI390154[] = {{&S20822[0], 11}, {NULL}};
const Segment SGI390155[] = {{&S20822[44], 4}, {NULL}};
const Segment SGI390156[] = {{&S20823[0], 6}, {NULL}};
const Segment SGI390157[] = {{&S20823[44], 4}, {NULL}};
const Segment SGI390158[] = {{&S20824[0], 10}, {NULL}};
const Segment SGI390159[] = {{&S20824[41], 4}, {NULL}};
const Segment SGI390160[] = {{&S20825[0], 14}, {NULL}};
const Segment SGI390161[] = {{&S20825[46], 4}, {NULL}};
const Segment SGI390162[] = {{&S20826[0], 8}, {NULL}};
const Segment SGI390163[] = {{&S20826[40], 4}, {NULL}};
const Segment SGI390164[] = {{&S20827[0], 9}, {NULL}};
const Segment SGI390165[] = {{&S20827[41], 4}, {NULL}};
const Segment SGI390166[] = {{&S20828[0], 34}, {NULL}};
const Segment SGI390167[] = {{&S20828[51], 4}, {NULL}};
const Segment SGI390168[] = {{&S20829[0], 7}, {NULL}};
const Segment SGI390169[] = {{&S20829[39], 4}, {NULL}};
const Segment SGI390170[] = {{&S20830[0], 6}, {NULL}};
const Segment SGI390171[] = {{&S20830[36], 4}, {NULL}};
const Segment SGI390172[] = {{&S20831[0], 10}, {NULL}};
const Segment SGI390173[] = {{&S20831[40], 4}, {NULL}};
const Segment SGI390174[] = {{&S20832[0], 13}, {NULL}};
const Segment SGI390175[] = {{&S20832[45], 4}, {NULL}};
const Segment SGI390176[] = {{&S20833[0], 7}, {NULL}};
const Segment SGI390177[] = {{&S20833[38], 4}, {NULL}};
const Segment SGI390178[] = {{&S20834[0], 10}, {NULL}};
const Segment SGI390179[] = {{&S20834[42], 4}, {NULL}};
const Segment SGI390180[] = {{&S20835[0], 14}, {NULL}};
const Segment SGI390181[] = {{&S20835[47], 4}, {NULL}};
const Segment SGI390182[] = {{&S20836[0], 6}, {NULL}};
const Segment SGI390183[] = {{&S20836[36], 4}, {NULL}};
const Segment SGI390184[] = {{&S20837[0], 8}, {NULL}};
const Segment SGI390185[] = {{&S20837[40], 4}, {NULL}};
const Segment SGI390186[] = {{&S20838[0], 13}, {NULL}};
const Segment SGI390187[] = {{&S20838[43], 4}, {NULL}};
const Segment SGI390188[] = {{&S20839[0], 11}, {NULL}};
const Segment SGI390189[] = {{&S20839[41], 4}, {NULL}};
const Segment SGI390190[] = {{&S20840[0], 41}, {NULL}};
const Segment SGI390191[] = {{&S20840[57], 4}, {NULL}};
const Segment SGI390192[] = {{&S20841[0], 7}, {NULL}};
const Segment SGI390193[] = {{&S20841[39], 4}, {NULL}};
const Segment SGI390194[] = {{&S20842[0], 25}, {NULL}};
const Segment SGI390195[] = {{&S20842[55], 4}, {NULL}};
const Segment SGI390196[] = {{&S20843[0], 10}, {NULL}};
const Segment SGI390197[] = {{&S20843[42], 4}, {NULL}};
const Segment SGI390198[] = {{&S20844[0], 4}, {NULL}};
const Segment SGI390199[] = {{&S20844[36], 4}, {NULL}};
const Segment SGI390200[] = {{&S20845[0], 8}, {NULL}};
const Segment SGI390201[] = {{&S20845[39], 4}, {NULL}};
const Segment SGI390202[] = {{&S20846[0], 9}, {NULL}};
const Segment SGI390203[] = {{&S20846[44], 4}, {NULL}};
const Segment SGI390204[] = {{&S20848[0], 10}, {NULL}};
const Segment SGI390205[] = {{&S20848[43], 4}, {NULL}};
const Segment SGI390206[] = {{&S20849[0], 3}, {NULL}};
const Segment SGI390207[] = {{&S20849[35], 4}, {NULL}};
const Segment SGI390208[] = {{&S20850[0], 11}, {NULL}};
const Segment SGI390209[] = {{&S20850[43], 4}, {NULL}};
const Segment SGI390210[] = {{&S20851[0], 37}, {NULL}};
const Segment SGI390211[] = {{&S20851[53], 4}, {NULL}};
const Segment SGI390212[] = {{&S20852[0], 7}, {NULL}};
const Segment SGI390213[] = {{&S20852[39], 4}, {NULL}};
const Segment SGI390214[] = {{&S20853[0], 40}, {NULL}};
const Segment SGI390215[] = {{&S20853[56], 4}, {NULL}};
const Segment SGI390216[] = {{&S20854[0], 11}, {NULL}};
const Segment SGI390217[] = {{&S20854[41], 4}, {NULL}};
const Segment SGI390218[] = {{&S20855[0], 5}, {NULL}};
const Segment SGI390219[] = {{&S20855[37], 4}, {NULL}};
const Segment SGI390220[] = {{&S20856[0], 10}, {NULL}};
const Segment SGI390221[] = {{&S20856[42], 4}, {NULL}};
const Segment SGI390222[] = {{&S20858[0], 10}, {NULL}};
const Segment SGI390223[] = {{&S20858[38], 4}, {NULL}};
const Segment SGI390224[] = {{&S20859[0], 12}, {NULL}};
const Segment SGI390225[] = {{&S20859[44], 4}, {NULL}};
const Segment SGI390226[] = {{&S20860[0], 9}, {NULL}};
const Segment SGI390227[] = {{&S20860[39], 4}, {NULL}};
const Segment SGI390228[] = {{&S20861[0], 12}, {NULL}};
const Segment SGI390229[] = {{&S20861[43], 4}, {NULL}};
const Segment SGI390230[] = {{&S20862[0], 8}, {NULL}};
const Segment SGI390231[] = {{&S20862[40], 4}, {NULL}};
const Segment SGI390232[] = {{&S20863[0], 7}, {NULL}};
const Segment SGI390233[] = {{&S20863[37], 4}, {NULL}};
const Segment SGI390234[] = {{&S20864[0], 16}, {NULL}};
const Segment SGI390235[] = {{&S20864[48], 4}, {NULL}};
const Segment SGI390236[] = {{&S20865[0], 8}, {NULL}};
const Segment SGI390237[] = {{&S20865[40], 4}, {NULL}};
const Segment SGI390238[] = {{&S20866[0], 23}, {NULL}};
const Segment SGI390239[] = {{&S20866[53], 4}, {NULL}};
const Segment SGI390240[] = {{&S20867[0], 23}, {NULL}};
const Segment SGI390241[] = {{&S20867[54], 4}, {NULL}};
const Segment SGI390242[] = {{&S20868[0], 10}, {NULL}};
const Segment SGI390243[] = {{&S20868[40], 4}, {NULL}};
const Segment SGI390244[] = {{&S20869[0], 13}, {NULL}};
const Segment SGI390245[] = {{&S20869[43], 4}, {NULL}};
const Segment SGI390246[] = {{&S20870[0], 16}, {NULL}};
const Segment SGI390247[] = {{&S20870[46], 4}, {NULL}};
const Segment SGI390248[] = {{&S20871[0], 11}, {NULL}};
const Segment SGI390249[] = {{&S20871[43], 4}, {NULL}};
const Segment SGI390250[] = {{&S20872[0], 9}, {NULL}};
const Segment SGI390251[] = {{&S20872[41], 4}, {NULL}};
const Segment SGI390252[] = {{&S20873[0], 25}, {NULL}};
const Segment SGI390253[] = {{&S20873[57], 4}, {NULL}};
const Segment SGI390254[] = {{&S20874[0], 9}, {NULL}};
const Segment SGI390255[] = {{&S20874[44], 4}, {NULL}};
const Segment SGI390256[] = {{&S20875[0], 5}, {NULL}};
const Segment SGI390257[] = {{&S20875[35], 4}, {NULL}};
const Segment SGI390258[] = {{&S20876[0], 11}, {NULL}};
const Segment SGI390259[] = {{&S20876[41], 4}, {NULL}};
const Segment SGI390260[] = {{&S20877[0], 5}, {NULL}};
const Segment SGI390261[] = {{&S20877[33], 4}, {NULL}};
const Segment SGI390262[] = {{&S20878[0], 5}, {NULL}};
const Segment SGI390263[] = {{&S20878[35], 4}, {NULL}};
const Segment SGI390264[] = {{&S20879[0], 28}, {NULL}};
const Segment SGI390265[] = {{&S20879[58], 4}, {NULL}};
const Segment SGI390266[] = {{&S20880[0], 17}, {NULL}};
const Segment SGI390267[] = {{&S20880[50], 4}, {NULL}};
const Segment SGI390268[] = {{&S20881[0], 18}, {NULL}};
const Segment SGI390269[] = {{&S20881[48], 4}, {NULL}};
const Segment SGI390270[] = {{&S20882[0], 7}, {NULL}};
const Segment SGI390271[] = {{&S20882[40], 4}, {NULL}};
const Segment SGI390272[] = {{&S20883[0], 10}, {NULL}};
const Segment SGI390273[] = {{&S20883[40], 4}, {NULL}};
const Segment SGI390274[] = {{&S20884[0], 4}, {NULL}};
const Segment SGI390275[] = {{&S20884[36], 4}, {NULL}};
const Segment SGI390276[] = {{&S20885[0], 23}, {NULL}};
const Segment SGI390277[] = {{&S20885[54], 4}, {NULL}};
const Segment SGI390278[] = {{&S20886[0], 10}, {NULL}};
const Segment SGI390279[] = {{&S20886[46], 4}, {NULL}};
const Segment SGI390280[] = {{&S20887[0], 15}, {NULL}};
const Segment SGI390281[] = {{&S20887[47], 4}, {NULL}};
const Segment SGI390282[] = {{&S20888[0], 8}, {NULL}};
const Segment SGI390283[] = {{&S20888[38], 4}, {NULL}};
const Segment SGI390284[] = {{&S20889[0], 8}, {NULL}};
const Segment SGI390285[] = {{&S20889[38], 4}, {NULL}};
const Segment SGI390286[] = {{&S20890[0], 7}, {NULL}};
const Segment SGI390287[] = {{&S20890[36], 4}, {NULL}};
const Segment SGI390288[] = {{&S20891[0], 7}, {NULL}};
const Segment SGI390289[] = {{&S20891[39], 4}, {NULL}};
const Segment SGI390290[] = {{&S20892[0], 14}, {NULL}};
const Segment SGI390291[] = {{&S20892[46], 4}, {NULL}};
const Segment SGI390292[] = {{&S20893[0], 9}, {NULL}};
const Segment SGI390293[] = {{&S20893[41], 4}, {NULL}};
const Segment SGI390294[] = {{&S20894[0], 7}, {NULL}};
const Segment SGI390295[] = {{&S20894[40], 4}, {NULL}};
const Segment SGI390296[] = {{&S20895[0], 2}, {NULL}};
const Segment SGI390297[] = {{&S20895[32], 4}, {NULL}};
const Segment SGI390298[] = {{&S20896[0], 9}, {NULL}};
const Segment SGI390299[] = {{&S20896[41], 4}, {NULL}};
const Segment SGI390300[] = {{&S20897[0], 13}, {NULL}};
const Segment SGI390301[] = {{&S20897[47], 4}, {NULL}};
const Segment SGI390302[] = {{&S20898[0], 23}, {NULL}};
const Segment SGI390303[] = {{&S20898[54], 4}, {NULL}};
const Segment SGI390304[] = {{&S20899[0], 6}, {NULL}};
const Segment SGI390305[] = {{&S20899[38], 4}, {NULL}};
const Segment SGI390306[] = {{&S20900[0], 13}, {NULL}};
const Segment SGI390307[] = {{&S20900[43], 4}, {NULL}};
const Segment SGI390308[] = {{&S20901[0], 8}, {NULL}};
const Segment SGI390309[] = {{&S20901[40], 4}, {NULL}};
const Segment SGI390310[] = {{&S20902[0], 5}, {NULL}};
const Segment SGI390311[] = {{&S20902[39], 4}, {NULL}};
const Segment SGI390312[] = {{&S20903[0], 13}, {NULL}};
const Segment SGI390313[] = {{&S20903[52], 4}, {NULL}};
const Segment SGI390314[] = {{&S20904[0], 15}, {NULL}};
const Segment SGI390315[] = {{&S20904[47], 4}, {NULL}};
const Segment SGI390316[] = {{&S20905[0], 7}, {NULL}};
const Segment SGI390317[] = {{&S20905[37], 4}, {NULL}};
const Segment SGI390318[] = {{&S20906[0], 18}, {NULL}};
const Segment SGI390319[] = {{&S20906[48], 4}, {NULL}};
const Segment SGI390320[] = {{&S20907[0], 7}, {NULL}};
const Segment SGI390321[] = {{&S20907[37], 4}, {NULL}};
const Segment SGI390322[] = {{&S20908[0], 13}, {NULL}};
const Segment SGI390323[] = {{&S20908[45], 4}, {NULL}};
const Segment SGI390324[] = {{&S20909[0], 23}, {NULL}};
const Segment SGI390325[] = {{&S20909[55], 4}, {NULL}};
const Segment SGI390326[] = {{&S20910[0], 13}, {NULL}};
const Segment SGI390327[] = {{&S20910[45], 4}, {NULL}};
const Segment SGI390328[] = {{&S20911[0], 14}, {NULL}};
const Segment SGI390329[] = {{&S20911[46], 4}, {NULL}};
const Segment SGI390330[] = {{&S20912[0], 14}, {NULL}};
const Segment SGI390331[] = {{&S20912[44], 4}, {NULL}};
const Segment SGI390332[] = {{&S20913[0], 11}, {NULL}};
const Segment SGI390333[] = {{&S20913[42], 4}, {NULL}};
const Segment SGI390334[] = {{&S20914[0], 26}, {NULL}};
const Segment SGI390335[] = {{&S20914[56], 4}, {NULL}};
const Segment SGI390336[] = {{&S20915[0], 8}, {NULL}};
const Segment SGI390337[] = {{&S20915[40], 4}, {NULL}};
const Segment SGI390338[] = {{&S20916[0], 15}, {NULL}};
const Segment SGI390339[] = {{&S20916[45], 4}, {NULL}};
const Segment SGI390340[] = {{&S20917[0], 14}, {NULL}};
const Segment SGI390341[] = {{&S20917[46], 4}, {NULL}};
const Segment SGI390342[] = {{&S20918[0], 22}, {NULL}};
const Segment SGI390343[] = {{&S20918[52], 4}, {NULL}};
const Segment SGI390344[] = {{&S20919[0], 19}, {NULL}};
const Segment SGI390345[] = {{&S20919[49], 4}, {NULL}};
const Segment SGI390346[] = {{&S20920[0], 17}, {NULL}};
const Segment SGI390347[] = {{&S20920[54], 4}, {NULL}};
const Segment SGI390348[] = {{&S20921[0], 5}, {NULL}};
const Segment SGI390349[] = {{&S20921[37], 4}, {NULL}};
const Segment SGI390350[] = {{&S20922[0], 8}, {NULL}};
const Segment SGI390351[] = {{&S20922[42], 4}, {NULL}};
const Segment SGI390352[] = {{&S20923[0], 9}, {NULL}};
const Segment SGI390353[] = {{&S20923[41], 4}, {NULL}};
const Segment SGI390354[] = {{&S20924[0], 10}, {NULL}};
const Segment SGI390355[] = {{&S20924[42], 4}, {NULL}};
const Segment SGI390356[] = {{&S20925[0], 6}, {NULL}};
const Segment SGI390357[] = {{&S20925[35], 4}, {NULL}};
const Segment SGI390358[] = {{&S20926[0], 4}, {NULL}};
const Segment SGI390359[] = {{&S20926[34], 4}, {NULL}};
const Segment SGI390360[] = {{&S20927[0], 10}, {NULL}};
const Segment SGI390361[] = {{&S20927[40], 4}, {NULL}};
const Segment SGI390362[] = {{&S20928[0], 11}, {NULL}};
const Segment SGI390363[] = {{&S20928[43], 4}, {NULL}};
const Segment SGI390364[] = {{&S20929[0], 16}, {NULL}};
const Segment SGI390365[] = {{&S20929[46], 4}, {NULL}};
const Segment SGI390366[] = {{&S20931[0], 45}, {NULL}};
const Segment SGI390367[] = {{&S20931[76], 4}, {NULL}};
const Segment SGI390368[] = {{&S20932[0], 7}, {NULL}};
const Segment SGI390369[] = {{&S20932[39], 4}, {NULL}};
const Segment SGI390370[] = {{&S20933[0], 14}, {NULL}};
const Segment SGI390371[] = {{&S20933[48], 4}, {NULL}};
const Segment SGI390372[] = {{&S20934[0], 22}, {NULL}};
const Segment SGI390373[] = {{&S20934[54], 4}, {NULL}};
const Segment SGI390374[] = {{&S20935[0], 12}, {NULL}};
const Segment SGI390375[] = {{&S20935[45], 4}, {NULL}};
const Segment SGI390376[] = {{&S20936[0], 11}, {NULL}};
const Segment SGI390377[] = {{&S20936[43], 4}, {NULL}};
const Segment SGI390378[] = {{&S20937[0], 8}, {NULL}};
const Segment SGI390379[] = {{&S20937[40], 4}, {NULL}};
const Segment SGI390380[] = {{&S20938[0], 10}, {NULL}};
const Segment SGI390381[] = {{&S20938[40], 4}, {NULL}};
const Segment SGI390382[] = {{&S20939[0], 11}, {NULL}};
const Segment SGI390383[] = {{&S20939[43], 4}, {NULL}};
const Segment SGI390384[] = {{&S20940[0], 12}, {NULL}};
const Segment SGI390385[] = {{&S20940[40], 4}, {NULL}};
const Segment SGI390386[] = {{&S20941[0], 7}, {NULL}};
const Segment SGI390387[] = {{&S20941[37], 4}, {NULL}};
const Segment SGI390388[] = {{&S20943[0], 54}, {NULL}};
const Segment SGI390389[] = {{&S20943[85], 4}, {NULL}};
const Segment SGI390390[] = {{&S20944[0], 9}, {NULL}};
const Segment SGI390391[] = {{&S20944[41], 4}, {NULL}};
const Segment SGI390392[] = {{&S20945[0], 10}, {NULL}};
const Segment SGI390393[] = {{&S20945[40], 4}, {NULL}};
const Segment SGI390394[] = {{&S20946[0], 12}, {NULL}};
const Segment SGI390395[] = {{&S20946[43], 4}, {NULL}};
const Segment SGI390396[] = {{&S20947[0], 9}, {NULL}};
const Segment SGI390397[] = {{&S20947[41], 4}, {NULL}};
const Segment SGI390398[] = {{&S20948[0], 13}, {NULL}};
const Segment SGI390399[] = {{&S20948[45], 4}, {NULL}};
const Segment SGI390400[] = {{&S20949[0], 7}, {NULL}};
const Segment SGI390401[] = {{&S20949[37], 4}, {NULL}};
const Segment SGI390402[] = {{&S20950[0], 9}, {NULL}};
const Segment SGI390403[] = {{&S20950[42], 4}, {NULL}};
const Segment SGI390404[] = {{&S20951[0], 13}, {NULL}};
const Segment SGI390405[] = {{&S20951[43], 4}, {NULL}};
const Segment SGI390406[] = {{&S20952[0], 11}, {NULL}};
const Segment SGI390407[] = {{&S20952[41], 4}, {NULL}};
const Segment SGI390408[] = {{&S20954[0], 5}, {NULL}};
const Segment SGI390409[] = {{&S20954[43], 4}, {NULL}};
const Segment SGI390410[] = {{&S63088[8], 3}, {NULL}};
const Segment SGI390411[] = {{&S63088[13], 7}, {NULL}};
const Segment SGI390412[] = {{&S63088[25], 14}, {NULL}};
const Segment SGI390413[] = {{&S63088[69], 13}, {NULL}};
const Segment SGI390414[] = {{&S63089[8], 3}, {NULL}};
const Segment SGI390415[] = {{&S63089[13], 7}, {NULL}};
const Segment SGI390416[] = {{&S63089[75], 13}, {NULL}};
const Segment SGI390417[] = {{&S63139[8], 3}, {NULL}};
const Segment SGI390418[] = {{&S63139[13], 9}, {NULL}};
const Segment SGI390419[] = {{&S63139[31], 15}, {NULL}};
const Segment SGI390420[] = {{&S63139[75], 10}, {NULL}};
const Segment SGI390421[] = {{&S55266[93], 3}, {NULL}};
const Segment SGI390422[] = {{&S55266[29], 6}, {NULL}};
const Segment SGI390423[] = {{&S55266[128], 4}, {NULL}};
const Segment SGI390424[] = {{&S55455[108], 3}, {NULL}};
const Segment SGI390425[] = {{&S55455[44], 6}, {NULL}};
const Segment SGI390426[] = {{&S55455[30], 6}, {NULL}};
const Segment SGI390427[] = {{&S55455[150], 4}, {NULL}};
const Segment SGI390428[] = {{&S55786[94], 3}, {NULL}};
const Segment SGI390429[] = {{&S55786[29], 7}, {NULL}};
const Segment SGI390430[] = {{&S55786[129], 4}, {NULL}};
const Segment SGI390431[] = {{&S55864[108], 3}, {NULL}};
const Segment SGI390432[] = {{&S55864[44], 6}, {NULL}};
const Segment SGI390433[] = {{&S55864[30], 6}, {NULL}};
const Segment SGI390434[] = {{&S55864[150], 4}, {NULL}};
const Segment SGI390435[] = {{&S59771[44], 4}, {NULL}};
const Segment SGI390436[] = {{&S59771[8], 3}, {NULL}};
const Segment SGI390437[] = {{&S59771[13], 14}, {NULL}};
const Segment SGI390438[] = {{&S56027[108], 3}, {NULL}};
const Segment SGI390439[] = {{&S56027[44], 6}, {NULL}};
const Segment SGI390440[] = {{&S56027[30], 6}, {NULL}};
const Segment SGI390441[] = {{&S56027[150], 4}, {NULL}};
const Segment SGI390442[] = {{&S59947[35], 4}, {NULL}};
const Segment SGI390443[] = {{&S59947[8], 3}, {NULL}};
const Segment SGI390444[] = {{&S59947[13], 3}, {NULL}};
const Segment SGI390445[] = {{&S56223[94], 3}, {NULL}};
const Segment SGI390446[] = {{&S56223[29], 7}, {NULL}};
const Segment SGI390447[] = {{&S56223[129], 4}, {NULL}};
const Segment SGI390448[] = {{&S64201[8], 3}, {NULL}};
const Segment SGI390449[] = {{&S64201[13], 7}, {NULL}};
const Segment SGI390450[] = {{&S64201[25], 14}, {NULL}};
const Segment SGI390451[] = {{&S64201[65], 13}, {NULL}};
const Segment SGI390452[] = {{&S64223[8], 3}, {NULL}};
const Segment SGI390453[] = {{&S64223[13], 7}, {NULL}};
const Segment SGI390454[] = {{&S64223[25], 5}, {NULL}};
const Segment SGI390455[] = {{&S64223[53], 12}, {NULL}};
const Segment SGI390456[] = {{&S64224[8], 3}, {NULL}};
const Segment SGI390457[] = {{&S64224[13], 7}, {NULL}};
const Segment SGI390458[] = {{&S64224[61], 12}, {NULL}};
const Segment SGI390459[] = {{&S64297[8], 3}, {NULL}};
const Segment SGI390460[] = {{&S64297[13], 7}, {NULL}};
const Segment SGI390461[] = {{&S64297[25], 14}, {NULL}};
const Segment SGI390462[] = {{&S64297[66], 12}, {NULL}};
const Segment SGI390463[] = {{&S64305[8], 3}, {NULL}};
const Segment SGI390464[] = {{&S64305[13], 3}, {NULL}};
const Segment SGI390465[] = {{&S64305[87], 13}, {NULL}};
const Segment SGI390466[] = {{&S64306[8], 3}, {NULL}};
const Segment SGI390467[] = {{&S64306[13], 3}, {NULL}};
const Segment SGI390468[] = {{&S64306[13], 3}, {&S64306[21], 10}, {NULL}};
const Segment SGI390469[] = {{&S64306[60], 13}, {NULL}};
const Segment SGI390470[] = {{&S64506[8], 3}, {NULL}};
const Segment SGI390471[] = {{&S64506[13], 7}, {NULL}};
const Segment SGI390472[] = {{&S64506[25], 5}, {NULL}};
const Segment SGI390473[] = {{&S64506[60], 13}, {NULL}};
const Segment SGI390474[] = {{&S64507[8], 3}, {NULL}};
const Segment SGI390475[] = {{&S64507[13], 7}, {NULL}};
const Segment SGI390476[] = {{&S64507[65], 13}, {NULL}};
const Segment SGI390477[] = {{&S64606[8], 3}, {NULL}};
const Segment SGI390478[] = {{&S64606[13], 3}, {NULL}};
const Segment SGI390479[] = {{&S64606[13], 3}, {&S64606[21], 10}, {NULL}};
const Segment SGI390480[] = {{&S64606[63], 13}, {NULL}};
const Segment SGI390481[] = {{&S64607[8], 3}, {NULL}};
const Segment SGI390482[] = {{&S64607[13], 3}, {NULL}};
const Segment SGI390483[] = {{&S64607[90], 13}, {NULL}};
const Segment SGI390484[] = {{&S56712[108], 3}, {NULL}};
const Segment SGI390485[] = {{&S56712[44], 6}, {NULL}};
const Segment SGI390486[] = {{&S56712[30], 6}, {NULL}};
const Segment SGI390487[] = {{&S56712[150], 4}, {NULL}};
const Segment SGI390488[] = {{&S56802[94], 3}, {NULL}};
const Segment SGI390489[] = {{&S56802[29], 7}, {NULL}};
const Segment SGI390490[] = {{&S56802[129], 5}, {NULL}};
const Segment SGI390491[] = {{&S60884[32], 4}, {NULL}};
const Segment SGI390492[] = {{&S60884[8], 3}, {NULL}};
const Segment SGI390493[] = {{&S60884[13], 14}, {NULL}};
const Segment SGI390494[] = {{&S60885[32], 4}, {NULL}};
const Segment SGI390495[] = {{&S60885[8], 3}, {NULL}};
const Segment SGI390496[] = {{&S60885[13], 14}, {NULL}};
const Segment SGI390497[] = {{&S64933[8], 3}, {NULL}};
const Segment SGI390498[] = {{&S64933[13], 3}, {NULL}};
const Segment SGI390499[] = {{&S64933[13], 3}, {&S64933[21], 10}, {&S64933[60], 6}, {NULL}};
const Segment SGI390500[] = {{&S64933[60], 10}, {NULL}};
const Segment SGI390501[] = {{&S64934[8], 3}, {NULL}};
const Segment SGI390502[] = {{&S64934[13], 3}, {NULL}};
const Segment SGI390503[] = {{&S64934[13], 3}, {&S64934[62], 6}, {NULL}};
const Segment SGI390504[] = {{&S64934[62], 10}, {NULL}};
const Segment SGI390505[] = {{&S64998[8], 3}, {NULL}};
const Segment SGI390506[] = {{&S64998[13], 7}, {NULL}};
const Segment SGI390507[] = {{&S64998[25], 14}, {NULL}};
const Segment SGI390508[] = {{&S64998[78], 12}, {NULL}};
const Segment SGI390509[] = {{&S61297[78], 3}, {NULL}};
const Segment SGI390510[] = {{&S61297[8], 3}, {NULL}};
const Segment SGI390511[] = {{&S61297[13], 7}, {NULL}};
const Segment SGI390512[] = {{&S61297[25], 14}, {NULL}};
const Segment SGI390513[] = {{&S61457[39], 4}, {NULL}};
const Segment SGI390514[] = {{&S61457[8], 3}, {NULL}};
const Segment SGI390515[] = {{&S61457[13], 14}, {NULL}};
const Segment SGI390516[] = {{&S57594[108], 3}, {NULL}};
const Segment SGI390517[] = {{&S57594[44], 6}, {NULL}};
const Segment SGI390518[] = {{&S57594[30], 6}, {NULL}};
const Segment SGI390519[] = {{&S57594[150], 4}, {NULL}};
const Segment SGI390520[] = {{&S57648[93], 3}, {NULL}};
const Segment SGI390521[] = {{&S57648[29], 6}, {NULL}};
const Segment SGI390522[] = {{&S57648[128], 4}, {NULL}};
const Segment SGI390523[] = {{&S57725[94], 3}, {NULL}};
const Segment SGI390524[] = {{&S57725[29], 7}, {NULL}};
const Segment SGI390525[] = {{&S57725[129], 4}, {NULL}};
const Segment SGI390526[] = {{&S57752[108], 3}, {NULL}};
const Segment SGI390527[] = {{&S57752[44], 6}, {NULL}};
const Segment SGI390528[] = {{&S57752[30], 6}, {NULL}};
const Segment SGI390529[] = {{&S57752[150], 4}, {NULL}};
const Segment SGI390530[] = {{&S57779[93], 3}, {NULL}};
const Segment SGI390531[] = {{&S57779[29], 6}, {NULL}};
const Segment SGI390532[] = {{&S57779[128], 4}, {NULL}};
const Segment SGI390533[] = {{&S57932[94], 3}, {NULL}};
const Segment SGI390534[] = {{&S57932[29], 7}, {NULL}};
const Segment SGI390535[] = {{&S57932[129], 5}, {NULL}};
const Segment SGI390536[] = {{&S57951[94], 3}, {NULL}};
const Segment SGI390537[] = {{&S57951[29], 7}, {NULL}};
const Segment SGI390538[] = {{&S57951[129], 4}, {NULL}};
const Segment SGI390539[] = {{&S61909[39], 4}, {NULL}};
const Segment SGI390540[] = {{&S61909[8], 3}, {NULL}};
const Segment SGI390541[] = {{&S61909[13], 14}, {NULL}};
const Segment SGI390542[] = {{&S66182[8], 3}, {NULL}};
const Segment SGI390543[] = {{&S66182[13], 7}, {NULL}};
const Segment SGI390544[] = {{&S66182[25], 14}, {NULL}};
const Segment SGI390545[] = {{&S66182[71], 13}, {NULL}};
const Segment SGI390546[] = {{&S66184[8], 3}, {NULL}};
const Segment SGI390547[] = {{&S66184[13], 7}, {NULL}};
const Segment SGI390548[] = {{&S66184[25], 14}, {NULL}};
const Segment SGI390549[] = {{&S66184[71], 13}, {NULL}};
const Segment SGI390550[] = {{&S58465[109], 3}, {NULL}};
const Segment SGI390551[] = {{&S58465[44], 7}, {NULL}};
const Segment SGI390552[] = {{&S58465[30], 6}, {NULL}};
const Segment SGI390553[] = {{&S58465[151], 4}, {NULL}};
const Segment SGI390554[] = {{&S70397[8], 3}, {NULL}};
const Segment SGI390555[] = {{&S70397[13], 7}, {NULL}};
const Segment SGI390556[] = {{&S70397[25], 14}, {NULL}};
const Segment SGI390557[] = {{&S70397[84], 48}, {NULL}};
const Segment SGI390558[] = {{&S70482[8], 3}, {NULL}};
const Segment SGI390559[] = {{&S70482[13], 7}, {NULL}};
const Segment SGI390560[] = {{&S70482[87], 51}, {NULL}};
const Segment SGI390561[] = {{&S70483[8], 3}, {NULL}};
const Segment SGI390562[] = {{&S70483[13], 7}, {NULL}};
const Segment SGI390563[] = {{&S70483[87], 51}, {NULL}};
const Segment SGI390564[] = {{&S59401[93], 3}, {NULL}};
const Segment SGI390565[] = {{&S59401[29], 6}, {NULL}};
const Segment SGI390566[] = {{&S59401[128], 5}, {NULL}};
const Segment SGI390567[] = {{&S67153[8], 3}, {NULL}};
const Segment SGI390568[] = {{&S67153[13], 9}, {NULL}};
const Segment SGI390569[] = {{&S67153[30], 13}, {NULL}};
const Segment SGI390570[] = {{&S67153[63], 11}, {NULL}};
const Segment SGI390571[] = {{&S67224[8], 3}, {NULL}};
const Segment SGI390572[] = {{&S67224[13], 7}, {NULL}};
const Segment SGI390573[] = {{&S67224[75], 13}, {NULL}};
const Segment SGI390574[] = {{&S67225[8], 3}, {NULL}};
const Segment SGI390575[] = {{&S67225[13], 7}, {NULL}};
const Segment SGI390576[] = {{&S67225[25], 14}, {NULL}};
const Segment SGI390577[] = {{&S67225[68], 13}, {NULL}};
const Segment SGI390578[] = {{&S67473[8], 3}, {NULL}};
const Segment SGI390579[] = {{&S67473[13], 3}, {NULL}};
const Segment SGI390580[] = {{&S67474[8], 3}, {NULL}};
const Segment SGI390581[] = {{&S67474[13], 3}, {NULL}};
const Segment SGI390582[] = {{&S67474[13], 3}, {&S67474[18], 10}, {NULL}};
const Segment SGI390583[] = {{&S67512[8], 3}, {NULL}};
const Segment SGI390584[] = {{&S67512[13], 17}, {NULL}};
const Segment SGI390585[] = {{&S67512[74], 5}, {NULL}};
const Segment SGI390586[] = {{&S59886[93], 3}, {NULL}};
const Segment SGI390587[] = {{&S59886[29], 6}, {NULL}};
const Segment SGI390588[] = {{&S59886[128], 5}, {NULL}};
const Segment SGI390589[] = {{&S59950[93], 3}, {NULL}};
const Segment SGI390590[] = {{&S59950[29], 6}, {NULL}};
const Segment SGI390591[] = {{&S59950[128], 5}, {NULL}};
const Segment SGI390592[] = {{&S59957[108], 3}, {NULL}};
const Segment SGI390593[] = {{&S59957[44], 6}, {NULL}};
const Segment SGI390594[] = {{&S59957[30], 6}, {NULL}};
const Segment SGI390595[] = {{&S59957[150], 5}, {NULL}};
const Segment SGI390596[] = {{&S67783[8], 3}, {NULL}};
const Segment SGI390597[] = {{&S67783[13], 9}, {NULL}};
const Segment SGI390598[] = {{&S67783[71], 13}, {NULL}};
const Segment SGI390599[] = {{&S67784[8], 3}, {NULL}};
const Segment SGI390600[] = {{&S67784[13], 9}, {NULL}};
const Segment SGI390601[] = {{&S67784[31], 15}, {NULL}};
const Segment SGI390602[] = {{&S67784[76], 13}, {NULL}};
const Segment SGI390603[] = {{&S67870[8], 3}, {NULL}};
const Segment SGI390604[] = {{&S67870[13], 7}, {NULL}};
const Segment SGI390605[] = {{&S67870[25], 14}, {NULL}};
const Segment SGI390606[] = {{&S67870[69], 13}, {NULL}};
const Segment SGI390607[] = {{&S60166[108], 3}, {NULL}};
const Segment SGI390608[] = {{&S60166[44], 6}, {NULL}};
const Segment SGI390609[] = {{&S60166[30], 6}, {NULL}};
const Segment SGI390610[] = {{&S60166[150], 5}, {NULL}};
const Segment SGI390611[] = {{&S63739[32], 4}, {NULL}};
const Segment SGI390612[] = {{&S63739[8], 3}, {NULL}};
const Segment SGI390613[] = {{&S63739[13], 14}, {NULL}};
const Segment SGI390614[] = {{&S68000[8], 3}, {NULL}};
const Segment SGI390615[] = {{&S68000[13], 3}, {NULL}};
const Segment SGI390616[] = {{&S68000[13], 3}, {&S68000[18], 10}, {NULL}};
const Segment SGI390617[] = {{&S68000[50], 12}, {NULL}};
const Segment SGI390618[] = {{&S68001[8], 3}, {NULL}};
const Segment SGI390619[] = {{&S68001[13], 3}, {NULL}};
const Segment SGI390620[] = {{&S68001[52], 12}, {NULL}};
const Segment SGI390621[] = {{&S64002[73], 3}, {NULL}};
const Segment SGI390622[] = {{&S64002[8], 3}, {NULL}};
const Segment SGI390623[] = {{&S64002[13], 3}, {NULL}};
const Segment SGI390624[] = {{&S64090[32], 4}, {NULL}};
const Segment SGI390625[] = {{&S64090[8], 3}, {NULL}};
const Segment SGI390626[] = {{&S64090[13], 14}, {NULL}};
const Segment SGI390627[] = {{&S64091[39], 4}, {NULL}};
const Segment SGI390628[] = {{&S64091[8], 3}, {NULL}};
const Segment SGI390629[] = {{&S64091[13], 14}, {NULL}};
const Segment SGI390630[] = {{&S68260[8], 3}, {NULL}};
const Segment SGI390631[] = {{&S68260[13], 3}, {NULL}};
const Segment SGI390632[] = {{&S68260[64], 13}, {NULL}};
const Segment SGI390633[] = {{&S68296[8], 3}, {NULL}};
const Segment SGI390634[] = {{&S68296[13], 17}, {NULL}};
const Segment SGI390635[] = {{&S68296[74], 5}, {NULL}};
const Segment SGI390636[] = {{&S64247[48], 4}, {NULL}};
const Segment SGI390637[] = {{&S64247[8], 3}, {NULL}};
const Segment SGI390638[] = {{&S64247[13], 9}, {NULL}};
const Segment SGI390639[] = {{&S64247[30], 13}, {NULL}};
const Segment SGI390640[] = {{&S68485[8], 3}, {NULL}};
const Segment SGI390641[] = {{&S68485[13], 9}, {NULL}};
const Segment SGI390642[] = {{&S68485[33], 13}, {NULL}};
const Segment SGI390643[] = {{&S68485[75], 10}, {NULL}};
const Segment SGI390644[] = {{&S68602[8], 3}, {NULL}};
const Segment SGI390645[] = {{&S68602[13], 7}, {NULL}};
const Segment SGI390646[] = {{&S68602[25], 14}, {NULL}};
const Segment SGI390647[] = {{&S68602[66], 13}, {NULL}};
const Segment SGI390648[] = {{&S68691[8], 3}, {NULL}};
const Segment SGI390649[] = {{&S68691[13], 7}, {NULL}};
const Segment SGI390650[] = {{&S68691[25], 14}, {NULL}};
const Segment SGI390651[] = {{&S68691[66], 13}, {NULL}};
const Segment SGI390652[] = {{&S64612[35], 4}, {NULL}};
const Segment SGI390653[] = {{&S64612[8], 3}, {NULL}};
const Segment SGI390654[] = {{&S64612[13], 3}, {NULL}};
const Segment SGI390655[] = {{&S60900[94], 3}, {NULL}};
const Segment SGI390656[] = {{&S60900[29], 7}, {NULL}};
const Segment SGI390657[] = {{&S60900[129], 4}, {NULL}};
const Segment SGI390658[] = {{&S68938[8], 3}, {NULL}};
const Segment SGI390659[] = {{&S68938[13], 14}, {NULL}};
const Segment SGI390660[] = {{&S68938[47], 5}, {NULL}};
const Segment SGI390661[] = {{&S69011[8], 3}, {NULL}};
const Segment SGI390662[] = {{&S69011[13], 7}, {NULL}};
const Segment SGI390663[] = {{&S69011[25], 14}, {NULL}};
const Segment SGI390664[] = {{&S69011[66], 13}, {NULL}};
const Segment SGI390665[] = {{&S61102[94], 3}, {NULL}};
const Segment SGI390666[] = {{&S61102[29], 7}, {NULL}};
const Segment SGI390667[] = {{&S61102[129], 5}, {NULL}};
const Segment SGI390668[] = {{&S61143[94], 3}, {NULL}};
const Segment SGI390669[] = {{&S61143[29], 7}, {NULL}};
const Segment SGI390670[] = {{&S61143[129], 4}, {NULL}};
const Segment SGI390671[] = {{&S69326[8], 3}, {NULL}};
const Segment SGI390672[] = {{&S69326[13], 9}, {NULL}};
const Segment SGI390673[] = {{&S69326[66], 12}, {NULL}};
const Segment SGI390674[] = {{&S69348[8], 3}, {NULL}};
const Segment SGI390675[] = {{&S69348[13], 7}, {NULL}};
const Segment SGI390676[] = {{&S69348[25], 14}, {NULL}};
const Segment SGI390677[] = {{&S69348[66], 12}, {NULL}};
const Segment SGI390678[] = {{&S69349[8], 3}, {NULL}};
const Segment SGI390679[] = {{&S69349[13], 7}, {NULL}};
const Segment SGI390680[] = {{&S69349[25], 14}, {NULL}};
const Segment SGI390681[] = {{&S69349[66], 12}, {NULL}};
const Segment SGI390682[] = {{&S65342[33], 4}, {NULL}};
const Segment SGI390683[] = {{&S65342[8], 3}, {NULL}};
const Segment SGI390684[] = {{&S65342[13], 3}, {NULL}};
const Segment SGI390685[] = {{&S61549[110], 3}, {NULL}};
const Segment SGI390686[] = {{&S61549[45], 7}, {NULL}};
const Segment SGI390687[] = {{&S61549[30], 6}, {NULL}};
const Segment SGI390688[] = {{&S61549[152], 5}, {NULL}};
const Segment SGI390689[] = {{&S73151[8], 3}, {NULL}};
const Segment SGI390690[] = {{&S73151[13], 7}, {NULL}};
const Segment SGI390691[] = {{&S73151[25], 14}, {NULL}};
const Segment SGI390692[] = {{&S73151[84], 48}, {NULL}};
const Segment SGI390693[] = {{&S62013[93], 3}, {NULL}};
const Segment SGI390694[] = {{&S62013[29], 6}, {NULL}};
const Segment SGI390695[] = {{&S62013[128], 4}, {NULL}};
const Segment SGI390696[] = {{&S65972[48], 4}, {NULL}};
const Segment SGI390697[] = {{&S65972[8], 3}, {NULL}};
const Segment SGI390698[] = {{&S65972[13], 9}, {NULL}};
const Segment SGI390699[] = {{&S65972[30], 13}, {NULL}};
const Segment SGI390700[] = {{&S70338[8], 3}, {NULL}};
const Segment SGI390701[] = {{&S70338[13], 7}, {NULL}};
const Segment SGI390702[] = {{&S70338[25], 5}, {NULL}};
const Segment SGI390703[] = {{&S70338[57], 12}, {NULL}};
const Segment SGI390704[] = {{&S70339[8], 3}, {NULL}};
const Segment SGI390705[] = {{&S70339[13], 7}, {NULL}};
const Segment SGI390706[] = {{&S70339[63], 12}, {NULL}};
const Segment SGI390707[] = {{&S66032[73], 3}, {NULL}};
const Segment SGI390708[] = {{&S66032[8], 3}, {NULL}};
const Segment SGI390709[] = {{&S66032[13], 3}, {NULL}};
const Segment SGI390710[] = {{&S70395[8], 3}, {NULL}};
const Segment SGI390711[] = {{&S70395[13], 7}, {NULL}};
const Segment SGI390712[] = {{&S70395[25], 14}, {NULL}};
const Segment SGI390713[] = {{&S70395[41], 5}, {NULL}};
const Segment SGI390714[] = {{&S70395[83], 13}, {NULL}};
const Segment SGI390715[] = {{&S70396[8], 3}, {NULL}};
const Segment SGI390716[] = {{&S70396[13], 7}, {NULL}};
const Segment SGI390717[] = {{&S70396[25], 14}, {NULL}};
const Segment SGI390718[] = {{&S70396[71], 13}, {NULL}};
const Segment SGI390719[] = {{&S70473[8], 3}, {NULL}};
const Segment SGI390720[] = {{&S70473[13], 9}, {NULL}};
const Segment SGI390721[] = {{&S70473[67], 10}, {NULL}};
const Segment SGI390722[] = {{&S70474[8], 3}, {NULL}};
const Segment SGI390723[] = {{&S70474[13], 9}, {NULL}};
const Segment SGI390724[] = {{&S70474[33], 15}, {NULL}};
const Segment SGI390725[] = {{&S70474[75], 10}, {NULL}};
const Segment SGI390726[] = {{&S62328[93], 3}, {NULL}};
const Segment SGI390727[] = {{&S62328[29], 6}, {NULL}};
const Segment SGI390728[] = {{&S62328[128], 5}, {NULL}};
const Segment SGI390729[] = {{&S70789[8], 3}, {NULL}};
const Segment SGI390730[] = {{&S70789[13], 9}, {NULL}};
const Segment SGI390731[] = {{&S70789[31], 15}, {NULL}};
const Segment SGI390732[] = {{&S70789[73], 10}, {NULL}};
const Segment SGI390733[] = {{&S62424[94], 3}, {NULL}};
const Segment SGI390734[] = {{&S62424[29], 7}, {NULL}};
const Segment SGI390735[] = {{&S62424[129], 5}, {NULL}};
const Segment SGI390736[] = {{&S66506[39], 4}, {NULL}};
const Segment SGI390737[] = {{&S66506[8], 3}, {NULL}};
const Segment SGI390738[] = {{&S66506[13], 14}, {NULL}};
const Segment SGI390739[] = {{&S70857[8], 3}, {NULL}};
const Segment SGI390740[] = {{&S70857[13], 7}, {NULL}};
const Segment SGI390741[] = {{&S70857[25], 14}, {NULL}};
const Segment SGI390742[] = {{&S70857[68], 13}, {NULL}};
const Segment SGI390743[] = {{&S66577[48], 4}, {NULL}};
const Segment SGI390744[] = {{&S66577[8], 3}, {NULL}};
const Segment SGI390745[] = {{&S66577[13], 9}, {NULL}};
const Segment SGI390746[] = {{&S66577[30], 13}, {NULL}};
const Segment SGI390747[] = {{&S70899[8], 3}, {NULL}};
const Segment SGI390748[] = {{&S70899[13], 3}, {NULL}};
const Segment SGI390749[] = {{&S70899[13], 3}, {&S70899[21], 10}, {NULL}};
const Segment SGI390750[] = {{&S70899[63], 13}, {NULL}};
const Segment SGI390751[] = {{&S70921[8], 3}, {NULL}};
const Segment SGI390752[] = {{&S70921[13], 14}, {NULL}};
const Segment SGI390753[] = {{&S66630[78], 3}, {NULL}};
const Segment SGI390754[] = {{&S66630[8], 3}, {NULL}};
const Segment SGI390755[] = {{&S66630[13], 7}, {NULL}};
const Segment SGI390756[] = {{&S66630[25], 14}, {NULL}};
const Segment SGI390757[] = {{&S66750[48], 4}, {NULL}};
const Segment SGI390758[] = {{&S66750[8], 3}, {NULL}};
const Segment SGI390759[] = {{&S66750[13], 9}, {NULL}};
const Segment SGI390760[] = {{&S66750[30], 13}, {NULL}};
const Segment SGI390761[] = {{&S62817[94], 3}, {NULL}};
const Segment SGI390762[] = {{&S62817[29], 7}, {NULL}};
const Segment SGI390763[] = {{&S62817[129], 4}, {NULL}};
const Segment SGI390764[] = {{&S62844[108], 3}, {NULL}};
const Segment SGI390765[] = {{&S62844[44], 6}, {NULL}};
const Segment SGI390766[] = {{&S62844[30], 6}, {NULL}};
const Segment SGI390767[] = {{&S62844[150], 4}, {NULL}};
const Segment SGI390768[] = {{&S62848[109], 3}, {NULL}};
const Segment SGI390769[] = {{&S62848[44], 7}, {NULL}};
const Segment SGI390770[] = {{&S62848[30], 6}, {NULL}};
const Segment SGI390771[] = {{&S62848[151], 4}, {NULL}};
const Segment SGI390772[] = {{&S62933[109], 3}, {NULL}};
const Segment SGI390773[] = {{&S62933[44], 7}, {NULL}};
const Segment SGI390774[] = {{&S62933[30], 6}, {NULL}};
const Segment SGI390775[] = {{&S62933[151], 4}, {NULL}};
const Segment SGI390776[] = {{&S71530[8], 3}, {NULL}};
const Segment SGI390777[] = {{&S71530[13], 7}, {NULL}};
const Segment SGI390778[] = {{&S71530[25], 14}, {NULL}};
const Segment SGI390779[] = {{&S71530[71], 13}, {NULL}};
const Segment SGI390780[] = {{&S67433[88], 3}, {NULL}};
const Segment SGI390781[] = {{&S67433[8], 3}, {NULL}};
const Segment SGI390782[] = {{&S67433[13], 9}, {NULL}};
const Segment SGI390783[] = {{&S67433[33], 13}, {NULL}};
const Segment SGI390784[] = {{&S63243[108], 3}, {NULL}};
const Segment SGI390785[] = {{&S63243[44], 6}, {NULL}};
const Segment SGI390786[] = {{&S63243[30], 6}, {NULL}};
const Segment SGI390787[] = {{&S63243[150], 4}, {NULL}};
const Segment SGI390788[] = {{&S71764[8], 3}, {NULL}};
const Segment SGI390789[] = {{&S71764[13], 14}, {NULL}};
const Segment SGI390790[] = {{&S71764[49], 5}, {NULL}};
const Segment SGI390791[] = {{&S71765[8], 3}, {NULL}};
const Segment SGI390792[] = {{&S71765[13], 14}, {NULL}};
const Segment SGI390793[] = {{&S71765[29], 5}, {NULL}};
const Segment SGI390794[] = {{&S71765[54], 5}, {NULL}};
const Segment SGI390795[] = {{&S63452[108], 3}, {NULL}};
const Segment SGI390796[] = {{&S63452[44], 6}, {NULL}};
const Segment SGI390797[] = {{&S63452[30], 6}, {NULL}};
const Segment SGI390798[] = {{&S63452[150], 4}, {NULL}};
const Segment SGI390799[] = {{&S63541[94], 3}, {NULL}};
const Segment SGI390800[] = {{&S63541[29], 7}, {NULL}};
const Segment SGI390801[] = {{&S63541[129], 4}, {NULL}};
const Segment SGI390802[] = {{&S72095[8], 3}, {NULL}};
const Segment SGI390803[] = {{&S72095[13], 7}, {NULL}};
const Segment SGI390804[] = {{&S72095[75], 13}, {NULL}};
const Segment SGI390805[] = {{&S72096[8], 3}, {NULL}};
const Segment SGI390806[] = {{&S72096[13], 7}, {NULL}};
const Segment SGI390807[] = {{&S72096[25], 14}, {NULL}};
const Segment SGI390808[] = {{&S72096[71], 13}, {NULL}};
const Segment SGI390809[] = {{&S72274[8], 3}, {NULL}};
const Segment SGI390810[] = {{&S72274[13], 9}, {NULL}};
const Segment SGI390811[] = {{&S72274[33], 13}, {NULL}};
const Segment SGI390812[] = {{&S72274[78], 13}, {NULL}};
const Segment SGI390813[] = {{&S72303[8], 3}, {NULL}};
const Segment SGI390814[] = {{&S72303[13], 14}, {NULL}};
const Segment SGI390815[] = {{&S72303[54], 11}, {NULL}};
const Segment SGI390816[] = {{&S67998[32], 4}, {NULL}};
const Segment SGI390817[] = {{&S67998[8], 3}, {NULL}};
const Segment SGI390818[] = {{&S67998[13], 14}, {NULL}};
const Segment SGI390819[] = {{&S67999[39], 4}, {NULL}};
const Segment SGI390820[] = {{&S67999[8], 3}, {NULL}};
const Segment SGI390821[] = {{&S67999[13], 14}, {NULL}};
const Segment SGI390822[] = {{&S75683[8], 3}, {NULL}};
const Segment SGI390823[] = {{&S75683[13], 7}, {NULL}};
const Segment SGI390824[] = {{&S75683[25], 14}, {NULL}};
const Segment SGI390825[] = {{&S75683[95], 20}, {NULL}};
const Segment SGI390826[] = {{&S75684[8], 3}, {NULL}};
const Segment SGI390827[] = {{&S75684[13], 7}, {NULL}};
const Segment SGI390828[] = {{&S75684[25], 14}, {NULL}};
const Segment SGI390829[] = {{&S75684[95], 20}, {NULL}};
const Segment SGI390830[] = {{&S68147[83], 3}, {NULL}};
const Segment SGI390831[] = {{&S68147[8], 3}, {NULL}};
const Segment SGI390832[] = {{&S68147[13], 9}, {NULL}};
const Segment SGI390833[] = {{&S68320[32], 4}, {NULL}};
const Segment SGI390834[] = {{&S68320[8], 3}, {NULL}};
const Segment SGI390835[] = {{&S68320[13], 14}, {NULL}};
const Segment SGI390836[] = {{&S68321[32], 4}, {NULL}};
const Segment SGI390837[] = {{&S68321[8], 3}, {NULL}};
const Segment SGI390838[] = {{&S68321[13], 14}, {NULL}};
const Segment SGI390839[] = {{&S72481[8], 3}, {NULL}};
const Segment SGI390840[] = {{&S72481[13], 3}, {NULL}};
const Segment SGI390841[] = {{&S72481[13], 3}, {&S72481[18], 10}, {NULL}};
const Segment SGI390842[] = {{&S72481[69], 6}, {NULL}};
const Segment SGI390843[] = {{&S72481[55], 13}, {NULL}};
const Segment SGI390844[] = {{&S72482[8], 3}, {NULL}};
const Segment SGI390845[] = {{&S72482[13], 3}, {NULL}};
const Segment SGI390846[] = {{&S72482[71], 6}, {NULL}};
const Segment SGI390847[] = {{&S72482[57], 13}, {NULL}};
const Segment SGI390848[] = {{&S68653[81], 3}, {NULL}};
const Segment SGI390849[] = {{&S68653[8], 3}, {NULL}};
const Segment SGI390850[] = {{&S68653[13], 7}, {NULL}};
const Segment SGI390851[] = {{&S68653[25], 14}, {NULL}};
const Segment SGI390852[] = {{&S64772[93], 3}, {NULL}};
const Segment SGI390853[] = {{&S64772[29], 6}, {NULL}};
const Segment SGI390854[] = {{&S64772[128], 4}, {NULL}};
const Segment SGI390855[] = {{&S64840[108], 3}, {NULL}};
const Segment SGI390856[] = {{&S64840[44], 6}, {NULL}};
const Segment SGI390857[] = {{&S64840[30], 6}, {NULL}};
const Segment SGI390858[] = {{&S64840[150], 5}, {NULL}};
const Segment SGI390859[] = {{&S68999[32], 4}, {NULL}};
const Segment SGI390860[] = {{&S68999[8], 3}, {NULL}};
const Segment SGI390861[] = {{&S68999[13], 14}, {NULL}};
const Segment SGI390862[] = {{&S65013[109], 3}, {NULL}};
const Segment SGI390863[] = {{&S65013[44], 7}, {NULL}};
const Segment SGI390864[] = {{&S65013[30], 6}, {NULL}};
const Segment SGI390865[] = {{&S65013[151], 4}, {NULL}};
const Segment SGI390866[] = {{&S69104[88], 3}, {NULL}};
const Segment SGI390867[] = {{&S69104[8], 3}, {NULL}};
const Segment SGI390868[] = {{&S69104[13], 9}, {NULL}};
const Segment SGI390869[] = {{&S69104[33], 13}, {NULL}};
const Segment SGI390870[] = {{&S65107[109], 3}, {NULL}};
const Segment SGI390871[] = {{&S65107[44], 7}, {NULL}};
const Segment SGI390872[] = {{&S65107[30], 6}, {NULL}};
const Segment SGI390873[] = {{&S65107[151], 4}, {NULL}};
const Segment SGI390874[] = {{&S73227[8], 3}, {NULL}};
const Segment SGI390875[] = {{&S73227[13], 9}, {NULL}};
const Segment SGI390876[] = {{&S73227[30], 13}, {NULL}};
const Segment SGI390877[] = {{&S73227[63], 11}, {NULL}};
const Segment SGI390878[] = {{&S65199[109], 3}, {NULL}};
const Segment SGI390879[] = {{&S65199[44], 7}, {NULL}};
const Segment SGI390880[] = {{&S65199[30], 6}, {NULL}};
const Segment SGI390881[] = {{&S65199[151], 4}, {NULL}};
const Segment SGI390882[] = {{&S69168[32], 4}, {NULL}};
const Segment SGI390883[] = {{&S69168[8], 3}, {NULL}};
const Segment SGI390884[] = {{&S69168[13], 14}, {NULL}};
const Segment SGI390885[] = {{&S73572[8], 3}, {NULL}};
const Segment SGI390886[] = {{&S73572[13], 9}, {NULL}};
const Segment SGI390887[] = {{&S73572[31], 13}, {NULL}};
const Segment SGI390888[] = {{&S73572[71], 13}, {NULL}};
const Segment SGI390889[] = {{&S73612[8], 3}, {NULL}};
const Segment SGI390890[] = {{&S73612[13], 3}, {NULL}};
const Segment SGI390891[] = {{&S73612[71], 6}, {NULL}};
const Segment SGI390892[] = {{&S73612[57], 13}, {NULL}};
const Segment SGI390893[] = {{&S73613[8], 3}, {NULL}};
const Segment SGI390894[] = {{&S73613[13], 3}, {NULL}};
const Segment SGI390895[] = {{&S73613[13], 3}, {&S73613[18], 10}, {NULL}};
const Segment SGI390896[] = {{&S73613[69], 6}, {NULL}};
const Segment SGI390897[] = {{&S73613[55], 13}, {NULL}};
const Segment SGI390898[] = {{&S65759[93], 3}, {NULL}};
const Segment SGI390899[] = {{&S65759[29], 6}, {NULL}};
const Segment SGI390900[] = {{&S65759[128], 5}, {NULL}};
const Segment SGI390901[] = {{&S73760[8], 3}, {NULL}};
const Segment SGI390902[] = {{&S73760[13], 7}, {NULL}};
const Segment SGI390903[] = {{&S73760[25], 14}, {NULL}};
const Segment SGI390904[] = {{&S73760[66], 12}, {NULL}};
const Segment SGI390905[] = {{&S73911[8], 3}, {NULL}};
const Segment SGI390906[] = {{&S73911[13], 14}, {NULL}};
const Segment SGI390907[] = {{&S73911[53], 13}, {NULL}};
const Segment SGI390908[] = {{&S78010[8], 3}, {NULL}};
const Segment SGI390909[] = {{&S78010[13], 3}, {NULL}};
const Segment SGI390910[] = {{&S65923[93], 3}, {NULL}};
const Segment SGI390911[] = {{&S65923[29], 6}, {NULL}};
const Segment SGI390912[] = {{&S65923[128], 5}, {NULL}};
const Segment SGI390913[] = {{&S70050[86], 3}, {NULL}};
const Segment SGI390914[] = {{&S70050[8], 3}, {NULL}};
const Segment SGI390915[] = {{&S70050[13], 9}, {NULL}};
const Segment SGI390916[] = {{&S70050[31], 13}, {NULL}};
const Segment SGI390917[] = {{&S74196[8], 3}, {NULL}};
const Segment SGI390918[] = {{&S74196[13], 3}, {NULL}};
const Segment SGI390919[] = {{&S74196[59], 12}, {NULL}};
const Segment SGI390920[] = {{&S74241[8], 3}, {NULL}};
const Segment SGI390921[] = {{&S74241[13], 3}, {NULL}};
const Segment SGI390922[] = {{&S74241[63], 11}, {NULL}};
const Segment SGI390923[] = {{&S74242[8], 3}, {NULL}};
const Segment SGI390924[] = {{&S74242[13], 3}, {NULL}};
const Segment SGI390925[] = {{&S74242[13], 3}, {&S74242[21], 10}, {NULL}};
const Segment SGI390926[] = {{&S74242[61], 11}, {NULL}};
const Segment SGI390927[] = {{&S74664[8], 3}, {NULL}};
const Segment SGI390928[] = {{&S74664[13], 3}, {NULL}};
const Segment SGI390929[] = {{&S74664[13], 3}, {&S74664[21], 10}, {&S74664[79], 6}, {NULL}};
const Segment SGI390930[] = {{&S74664[61], 13}, {NULL}};
const Segment SGI390931[] = {{&S34042[0], 12}, {NULL}};
const Segment SGI390932[] = {{&S34042[46], 4}, {NULL}};
const Segment SGI390933[] = {{&S74694[8], 3}, {NULL}};
const Segment SGI390934[] = {{&S74694[13], 7}, {NULL}};
const Segment SGI390935[] = {{&S74694[25], 14}, {NULL}};
const Segment SGI390936[] = {{&S74694[71], 13}, {NULL}};
const Segment SGI390937[] = {{&S66783[108], 3}, {NULL}};
const Segment SGI390938[] = {{&S66783[44], 6}, {NULL}};
const Segment SGI390939[] = {{&S66783[30], 6}, {NULL}};
const Segment SGI390940[] = {{&S66783[150], 5}, {NULL}};
const Segment SGI390941[] = {{&S66808[108], 3}, {NULL}};
const Segment SGI390942[] = {{&S66808[44], 6}, {NULL}};
const Segment SGI390943[] = {{&S66808[30], 6}, {NULL}};
const Segment SGI390944[] = {{&S66808[150], 5}, {NULL}};
const Segment SGI390945[] = {{&S66832[108], 3}, {NULL}};
const Segment SGI390946[] = {{&S66832[44], 6}, {NULL}};
const Segment SGI390947[] = {{&S66832[30], 6}, {NULL}};
const Segment SGI390948[] = {{&S66832[150], 5}, {NULL}};
const Segment SGI390949[] = {{&S75133[8], 3}, {NULL}};
const Segment SGI390950[] = {{&S75133[13], 7}, {NULL}};
const Segment SGI390951[] = {{&S75133[25], 14}, {NULL}};
const Segment SGI390952[] = {{&S75133[68], 13}, {NULL}};
const Segment SGI390953[] = {{&S75135[8], 3}, {NULL}};
const Segment SGI390954[] = {{&S75135[13], 7}, {NULL}};
const Segment SGI390955[] = {{&S75135[25], 14}, {NULL}};
const Segment SGI390956[] = {{&S75135[71], 13}, {NULL}};
const Segment SGI390957[] = {{&S71416[48], 4}, {NULL}};
const Segment SGI390958[] = {{&S71416[8], 3}, {NULL}};
const Segment SGI390959[] = {{&S71416[13], 9}, {NULL}};
const Segment SGI390960[] = {{&S71416[30], 13}, {NULL}};
const Segment SGI390961[] = {{&S67113[94], 3}, {NULL}};
const Segment SGI390962[] = {{&S67113[29], 7}, {NULL}};
const Segment SGI390963[] = {{&S67113[129], 4}, {NULL}};
const Segment SGI390964[] = {{&S72152[81], 3}, {NULL}};
const Segment SGI390965[] = {{&S72152[8], 3}, {NULL}};
const Segment SGI390966[] = {{&S72152[13], 7}, {NULL}};
const Segment SGI390967[] = {{&S72152[25], 14}, {NULL}};
const Segment SGI390968[] = {{&S72153[80], 3}, {NULL}};
const Segment SGI390969[] = {{&S72153[8], 3}, {NULL}};
const Segment SGI390970[] = {{&S72153[13], 7}, {NULL}};
const Segment SGI390971[] = {{&S72153[25], 14}, {NULL}};
const Segment SGI390972[] = {{&S79788[8], 3}, {NULL}};
const Segment SGI390973[] = {{&S79788[13], 7}, {NULL}};
const Segment SGI390974[] = {{&S79788[25], 14}, {NULL}};
const Segment SGI390975[] = {{&S79788[84], 48}, {NULL}};
const Segment SGI390976[] = {{&S76032[8], 3}, {NULL}};
const Segment SGI390977[] = {{&S76032[13], 3}, {NULL}};
const Segment SGI390978[] = {{&S76032[57], 13}, {NULL}};
const Segment SGI390979[] = {{&S76033[8], 3}, {NULL}};
const Segment SGI390980[] = {{&S76033[13], 3}, {NULL}};
const Segment SGI390981[] = {{&S76033[13], 3}, {&S76033[18], 10}, {NULL}};
const Segment SGI390982[] = {{&S76033[55], 13}, {NULL}};
const Segment SGI390983[] = {{&S72419[35], 4}, {NULL}};
const Segment SGI390984[] = {{&S72419[8], 3}, {NULL}};
const Segment SGI390985[] = {{&S72419[13], 3}, {NULL}};
const Segment SGI390986[] = {{&S76553[8], 3}, {NULL}};
const Segment SGI390987[] = {{&S76553[13], 3}, {NULL}};
const Segment SGI390988[] = {{&S76553[13], 3}, {&S76553[18], 10}, {NULL}};
const Segment SGI390989[] = {{&S76553[49], 13}, {NULL}};
const Segment SGI390990[] = {{&S76654[8], 3}, {NULL}};
const Segment SGI390991[] = {{&S76654[13], 7}, {NULL}};
const Segment SGI390992[] = {{&S76654[25], 14}, {NULL}};
const Segment SGI390993[] = {{&S76654[66], 12}, {NULL}};
const Segment SGI390994[] = {{&S76690[8], 3}, {NULL}};
const Segment SGI390995[] = {{&S76690[13], 7}, {NULL}};
const Segment SGI390996[] = {{&S76690[25], 14}, {NULL}};
const Segment SGI390997[] = {{&S76690[89], 3}, {NULL}};
const Segment SGI390998[] = {{&S76691[8], 3}, {NULL}};
const Segment SGI390999[] = {{&S76691[13], 7}, {NULL}};
const Segment SGI391000[] = {{&S76691[25], 14}, {NULL}};
const Segment SGI391001[] = {{&S76691[99], 3}, {NULL}};
const Segment SGI391002[] = {{&S76908[8], 3}, {NULL}};
const Segment SGI391003[] = {{&S76908[13], 14}, {NULL}};
const Segment SGI391004[] = {{&S76908[48], 13}, {NULL}};
const Segment SGI391005[] = {{&S77328[8], 3}, {NULL}};
const Segment SGI391006[] = {{&S77328[13], 9}, {NULL}};
const Segment SGI391007[] = {{&S77328[31], 15}, {NULL}};
const Segment SGI391008[] = {{&S77328[72], 10}, {NULL}};
const Segment SGI391009[] = {{&S73571[81], 3}, {NULL}};
const Segment SGI391010[] = {{&S73571[8], 3}, {NULL}};
const Segment SGI391011[] = {{&S73571[13], 7}, {NULL}};
const Segment SGI391012[] = {{&S73571[25], 14}, {NULL}};
const Segment SGI391013[] = {{&S73611[89], 3}, {NULL}};
const Segment SGI391014[] = {{&S73611[8], 3}, {NULL}};
const Segment SGI391015[] = {{&S73611[13], 3}, {NULL}};
const Segment SGI391016[] = {{&S73716[32], 4}, {NULL}};
const Segment SGI391017[] = {{&S73716[8], 3}, {NULL}};
const Segment SGI391018[] = {{&S73716[13], 14}, {NULL}};
const Segment SGI391019[] = {{&S77473[8], 3}, {NULL}};
const Segment SGI391020[] = {{&S77473[13], 7}, {NULL}};
const Segment SGI391021[] = {{&S77473[25], 14}, {NULL}};
const Segment SGI391022[] = {{&S77473[69], 13}, {NULL}};
const Segment SGI391023[] = {{&S77474[8], 3}, {NULL}};
const Segment SGI391024[] = {{&S77474[13], 14}, {NULL}};
const Segment SGI391025[] = {{&S77474[57], 13}, {NULL}};
const Segment SGI391026[] = {{&S73838[39], 4}, {NULL}};
const Segment SGI391027[] = {{&S73838[8], 3}, {NULL}};
const Segment SGI391028[] = {{&S73838[13], 14}, {NULL}};
const Segment SGI391029[] = {{&S68908[94], 3}, {NULL}};
const Segment SGI391030[] = {{&S68908[29], 7}, {NULL}};
const Segment SGI391031[] = {{&S68908[129], 5}, {NULL}};
const Segment SGI391032[] = {{&S73888[48], 4}, {NULL}};
const Segment SGI391033[] = {{&S73888[8], 3}, {NULL}};
const Segment SGI391034[] = {{&S73888[13], 9}, {NULL}};
const Segment SGI391035[] = {{&S73888[30], 13}, {NULL}};
const Segment SGI391036[] = {{&S73996[48], 4}, {NULL}};
const Segment SGI391037[] = {{&S73996[8], 3}, {NULL}};
const Segment SGI391038[] = {{&S73996[13], 9}, {NULL}};
const Segment SGI391039[] = {{&S73996[30], 13}, {NULL}};
const Segment SGI391040[] = {{&S69059[108], 3}, {NULL}};
const Segment SGI391041[] = {{&S69059[44], 6}, {NULL}};
const Segment SGI391042[] = {{&S69059[30], 6}, {NULL}};
const Segment SGI391043[] = {{&S69059[150], 5}, {NULL}};
const Segment SGI391044[] = {{&S74106[78], 3}, {NULL}};
const Segment SGI391045[] = {{&S74106[8], 3}, {NULL}};
const Segment SGI391046[] = {{&S74106[13], 7}, {NULL}};
const Segment SGI391047[] = {{&S74106[25], 14}, {NULL}};
const Segment SGI391048[] = {{&S69159[108], 3}, {NULL}};
const Segment SGI391049[] = {{&S69159[44], 6}, {NULL}};
const Segment SGI391050[] = {{&S69159[30], 6}, {NULL}};
const Segment SGI391051[] = {{&S69159[150], 5}, {NULL}};
const Segment SGI391052[] = {{&S74253[39], 4}, {NULL}};
const Segment SGI391053[] = {{&S74253[8], 3}, {NULL}};
const Segment SGI391054[] = {{&S74253[13], 14}, {NULL}};
const Segment SGI391055[] = {{&S69399[94], 3}, {NULL}};
const Segment SGI391056[] = {{&S69399[29], 7}, {NULL}};
const Segment SGI391057[] = {{&S69399[129], 4}, {NULL}};
const Segment SGI391058[] = {{&S82048[8], 3}, {NULL}};
const Segment SGI391059[] = {{&S82048[13], 7}, {NULL}};
const Segment SGI391060[] = {{&S82048[25], 14}, {NULL}};
const Segment SGI391061[] = {{&S82048[84], 49}, {NULL}};
const Segment SGI391062[] = {{&S82049[8], 3}, {NULL}};
const Segment SGI391063[] = {{&S82049[13], 7}, {NULL}};
const Segment SGI391064[] = {{&S82049[25], 14}, {NULL}};
const Segment SGI391065[] = {{&S82049[84], 48}, {NULL}};
const Segment SGI391066[] = {{&S74471[24], 3}, {NULL}};
const Segment SGI391067[] = {{&S74471[8], 3}, {NULL}};
const Segment SGI391068[] = {{&S74471[13], 14}, {NULL}};
const Segment SGI391069[] = {{&S78146[8], 3}, {NULL}};
const Segment SGI391070[] = {{&S78146[13], 3}, {NULL}};
const Segment SGI391071[] = {{&S78146[49], 12}, {NULL}};
const Segment SGI391072[] = {{&S78147[8], 3}, {NULL}};
const Segment SGI391073[] = {{&S78147[13], 3}, {NULL}};
const Segment SGI391074[] = {{&S78147[13], 3}, {&S78147[18], 10}, {NULL}};
const Segment SGI391075[] = {{&S78147[47], 12}, {NULL}};
const Segment SGI391076[] = {{&S78227[8], 3}, {NULL}};
const Segment SGI391077[] = {{&S78227[13], 7}, {NULL}};
const Segment SGI391078[] = {{&S78227[25], 14}, {NULL}};
const Segment SGI391079[] = {{&S78227[66], 12}, {NULL}};
const Segment SGI391080[] = {{&S69729[94], 3}, {NULL}};
const Segment SGI391081[] = {{&S69729[29], 7}, {NULL}};
const Segment SGI391082[] = {{&S69729[129], 5}, {NULL}};
const Segment SGI391083[] = {{&S74884[39], 4}, {NULL}};
const Segment SGI391084[] = {{&S74884[8], 3}, {NULL}};
const Segment SGI391085[] = {{&S74884[13], 14}, {NULL}};
const Segment SGI391086[] = {{&S38455[0], 6}, {NULL}};
const Segment SGI391087[] = {{&S38455[58], 10}, {NULL}};
const Segment SGI391088[] = {{&S38455[43], 4}, {NULL}};
const Segment SGI391089[] = {{&S38456[0], 6}, {NULL}};
const Segment SGI391090[] = {{&S38456[58], 4}, {NULL}};
const Segment SGI391091[] = {{&S38456[43], 4}, {NULL}};
const Segment SGI391092[] = {{&S38457[0], 6}, {NULL}};
const Segment SGI391093[] = {{&S38457[58], 7}, {NULL}};
const Segment SGI391094[] = {{&S38457[43], 4}, {NULL}};
const Segment SGI391095[] = {{&S38458[0], 6}, {NULL}};
const Segment SGI391096[] = {{&S38458[58], 7}, {NULL}};
const Segment SGI391097[] = {{&S38458[43], 4}, {NULL}};
const Segment SGI391098[] = {{&S38459[0], 6}, {NULL}};
const Segment SGI391099[] = {{&S38459[58], 6}, {NULL}};
const Segment SGI391100[] = {{&S38459[43], 4}, {NULL}};
const Segment SGI391101[] = {{&S38460[0], 6}, {NULL}};
const Segment SGI391102[] = {{&S38460[58], 10}, {NULL}};
const Segment SGI391103[] = {{&S38460[43], 4}, {NULL}};
const Segment SGI391104[] = {{&S78689[8], 3}, {NULL}};
const Segment SGI391105[] = {{&S78689[13], 7}, {NULL}};
const Segment SGI391106[] = {{&S78689[25], 14}, {NULL}};
const Segment SGI391107[] = {{&S78689[71], 13}, {NULL}};
const Segment SGI391108[] = {{&S75016[33], 4}, {NULL}};
const Segment SGI391109[] = {{&S75016[8], 3}, {NULL}};
const Segment SGI391110[] = {{&S75016[13], 3}, {NULL}};
const Segment SGI391111[] = {{&S78810[8], 3}, {NULL}};
const Segment SGI391112[] = {{&S78810[13], 14}, {NULL}};
const Segment SGI391113[] = {{&S78810[46], 13}, {NULL}};
const Segment SGI391114[] = {{&S70414[94], 3}, {NULL}};
const Segment SGI391115[] = {{&S70414[29], 7}, {NULL}};
const Segment SGI391116[] = {{&S70414[129], 5}, {NULL}};
const Segment SGI391117[] = {{&S78986[8], 3}, {NULL}};
const Segment SGI391118[] = {{&S78986[13], 3}, {NULL}};
const Segment SGI391119[] = {{&S78986[63], 6}, {NULL}};
const Segment SGI391120[] = {{&S78986[84], 13}, {NULL}};
const Segment SGI391121[] = {{&S78987[8], 3}, {NULL}};
const Segment SGI391122[] = {{&S78987[13], 3}, {NULL}};
const Segment SGI391123[] = {{&S78987[13], 3}, {&S78987[21], 10}, {NULL}};
const Segment SGI391124[] = {{&S78987[61], 6}, {NULL}};
const Segment SGI391125[] = {{&S78987[82], 13}, {NULL}};
const Segment SGI391126[] = {{&S75453[39], 4}, {NULL}};
const Segment SGI391127[] = {{&S75453[8], 3}, {NULL}};
const Segment SGI391128[] = {{&S75453[13], 14}, {NULL}};
const Segment SGI391129[] = {{&S38967[0], 13}, {NULL}};
const Segment SGI391130[] = {{&S38967[44], 4}, {NULL}};
const Segment SGI391131[] = {{&S38968[0], 6}, {NULL}};
const Segment SGI391132[] = {{&S38968[37], 4}, {NULL}};
const Segment SGI391133[] = {{&S70815[93], 3}, {NULL}};
const Segment SGI391134[] = {{&S70815[29], 6}, {NULL}};
const Segment SGI391135[] = {{&S70815[128], 4}, {NULL}};
const Segment SGI391136[] = {{&S79479[8], 3}, {NULL}};
const Segment SGI391137[] = {{&S79479[13], 7}, {NULL}};
const Segment SGI391138[] = {{&S79479[25], 14}, {NULL}};
const Segment SGI391139[] = {{&S79479[68], 13}, {NULL}};
const Segment SGI391140[] = {{&S70953[108], 3}, {NULL}};
const Segment SGI391141[] = {{&S70953[44], 6}, {NULL}};
const Segment SGI391142[] = {{&S70953[30], 6}, {NULL}};
const Segment SGI391143[] = {{&S70953[150], 5}, {NULL}};
const Segment SGI391144[] = {{&S71045[108], 3}, {NULL}};
const Segment SGI391145[] = {{&S71045[44], 6}, {NULL}};
const Segment SGI391146[] = {{&S71045[30], 6}, {NULL}};
const Segment SGI391147[] = {{&S71045[150], 4}, {NULL}};
const Segment SGI391148[] = {{&S79590[8], 3}, {NULL}};
const Segment SGI391149[] = {{&S79590[13], 9}, {NULL}};
const Segment SGI391150[] = {{&S79590[31], 15}, {NULL}};
const Segment SGI391151[] = {{&S79590[75], 9}, {NULL}};
const Segment SGI391152[] = {{&S79646[8], 3}, {NULL}};
const Segment SGI391153[] = {{&S79646[13], 7}, {NULL}};
const Segment SGI391154[] = {{&S79646[25], 14}, {NULL}};
const Segment SGI391155[] = {{&S79646[69], 13}, {NULL}};
const Segment SGI391156[] = {{&S75831[32], 4}, {NULL}};
const Segment SGI391157[] = {{&S75831[8], 3}, {NULL}};
const Segment SGI391158[] = {{&S75831[13], 14}, {NULL}};
const Segment SGI391159[] = {{&S39544[0], 40}, {NULL}};
const Segment SGI391160[] = {{&S39544[73], 10}, {NULL}};
const Segment SGI391161[] = {{&S39544[56], 4}, {NULL}};
const Segment SGI391162[] = {{&S39545[0], 16}, {NULL}};
const Segment SGI391163[] = {{&S39545[69], 10}, {NULL}};
const Segment SGI391164[] = {{&S39545[52], 4}, {NULL}};
const Segment SGI391165[] = {{&S39546[0], 40}, {NULL}};
const Segment SGI391166[] = {{&S39546[73], 10}, {NULL}};
const Segment SGI391167[] = {{&S39546[56], 4}, {NULL}};
const Segment SGI391168[] = {{&S39547[0], 40}, {NULL}};
const Segment SGI391169[] = {{&S39547[73], 4}, {NULL}};
const Segment SGI391170[] = {{&S39547[56], 4}, {NULL}};
const Segment SGI391171[] = {{&S39548[0], 10}, {NULL}};
const Segment SGI391172[] = {{&S39548[63], 10}, {NULL}};
const Segment SGI391173[] = {{&S39548[46], 4}, {NULL}};
const Segment SGI391174[] = {{&S39549[0], 40}, {NULL}};
const Segment SGI391175[] = {{&S39549[73], 7}, {NULL}};
const Segment SGI391176[] = {{&S39549[56], 4}, {NULL}};
const Segment SGI391177[] = {{&S39550[0], 16}, {NULL}};
const Segment SGI391178[] = {{&S39550[69], 10}, {NULL}};
const Segment SGI391179[] = {{&S39550[52], 4}, {NULL}};
const Segment SGI391180[] = {{&S39551[0], 10}, {NULL}};
const Segment SGI391181[] = {{&S39551[63], 10}, {NULL}};
const Segment SGI391182[] = {{&S39551[46], 4}, {NULL}};
const Segment SGI391183[] = {{&S39552[0], 24}, {NULL}};
const Segment SGI391184[] = {{&S39552[72], 10}, {NULL}};
const Segment SGI391185[] = {{&S39552[55], 4}, {NULL}};
const Segment SGI391186[] = {{&S39553[0], 16}, {NULL}};
const Segment SGI391187[] = {{&S39553[69], 4}, {NULL}};
const Segment SGI391188[] = {{&S39553[52], 4}, {NULL}};
const Segment SGI391189[] = {{&S39554[0], 5}, {NULL}};
const Segment SGI391190[] = {{&S39554[52], 10}, {NULL}};
const Segment SGI391191[] = {{&S39554[35], 4}, {NULL}};
const Segment SGI391192[] = {{&S39555[0], 10}, {NULL}};
const Segment SGI391193[] = {{&S39555[63], 4}, {NULL}};
const Segment SGI391194[] = {{&S39555[46], 4}, {NULL}};
const Segment SGI391195[] = {{&S39557[0], 40}, {NULL}};
const Segment SGI391196[] = {{&S39557[73], 7}, {NULL}};
const Segment SGI391197[] = {{&S39557[56], 4}, {NULL}};
const Segment SGI391198[] = {{&S39558[0], 14}, {NULL}};
const Segment SGI391199[] = {{&S39558[63], 4}, {NULL}};
const Segment SGI391200[] = {{&S39558[46], 4}, {NULL}};
const Segment SGI391201[] = {{&S39559[0], 43}, {NULL}};
const Segment SGI391202[] = {{&S39559[76], 10}, {NULL}};
const Segment SGI391203[] = {{&S39559[59], 4}, {NULL}};
const Segment SGI391204[] = {{&S39560[0], 9}, {NULL}};
const Segment SGI391205[] = {{&S39560[57], 10}, {NULL}};
const Segment SGI391206[] = {{&S39560[40], 4}, {NULL}};
const Segment SGI391207[] = {{&S39561[0], 40}, {NULL}};
const Segment SGI391208[] = {{&S39561[73], 6}, {NULL}};
const Segment SGI391209[] = {{&S39561[56], 4}, {NULL}};
const Segment SGI391210[] = {{&S39562[0], 16}, {NULL}};
const Segment SGI391211[] = {{&S39562[69], 7}, {NULL}};
const Segment SGI391212[] = {{&S39562[52], 4}, {NULL}};
const Segment SGI391213[] = {{&S39563[0], 16}, {NULL}};
const Segment SGI391214[] = {{&S39563[69], 7}, {NULL}};
const Segment SGI391215[] = {{&S39563[52], 4}, {NULL}};
const Segment SGI391216[] = {{&S39564[0], 14}, {NULL}};
const Segment SGI391217[] = {{&S39564[63], 10}, {NULL}};
const Segment SGI391218[] = {{&S39564[46], 4}, {NULL}};
const Segment SGI391219[] = {{&S39565[0], 3}, {NULL}};
const Segment SGI391220[] = {{&S39565[51], 10}, {NULL}};
const Segment SGI391221[] = {{&S39565[34], 4}, {NULL}};
const Segment SGI391222[] = {{&S39566[0], 14}, {NULL}};
const Segment SGI391223[] = {{&S39566[59], 10}, {NULL}};
const Segment SGI391224[] = {{&S39566[42], 4}, {NULL}};
const Segment SGI391225[] = {{&S39567[0], 3}, {NULL}};
const Segment SGI391226[] = {{&S39567[51], 10}, {NULL}};
const Segment SGI391227[] = {{&S39567[34], 4}, {NULL}};
const Segment SGI391228[] = {{&S39568[0], 10}, {NULL}};
const Segment SGI391229[] = {{&S39568[63], 7}, {NULL}};
const Segment SGI391230[] = {{&S39568[46], 4}, {NULL}};
const Segment SGI391231[] = {{&S39569[0], 3}, {NULL}};
const Segment SGI391232[] = {{&S39569[51], 4}, {NULL}};
const Segment SGI391233[] = {{&S39569[34], 4}, {NULL}};
const Segment SGI391234[] = {{&S39593[0], 23}, {NULL}};
const Segment SGI391235[] = {{&S39593[70], 4}, {NULL}};
const Segment SGI391236[] = {{&S39593[55], 4}, {NULL}};
const Segment SGI391237[] = {{&S39594[0], 13}, {NULL}};
const Segment SGI391238[] = {{&S39594[60], 4}, {NULL}};
const Segment SGI391239[] = {{&S39594[45], 4}, {NULL}};
const Segment SGI391240[] = {{&S39595[0], 14}, {NULL}};
const Segment SGI391241[] = {{&S39595[62], 10}, {NULL}};
const Segment SGI391242[] = {{&S39595[47], 4}, {NULL}};
const Segment SGI391243[] = {{&S39596[0], 23}, {NULL}};
const Segment SGI391244[] = {{&S39596[70], 10}, {NULL}};
const Segment SGI391245[] = {{&S39596[55], 4}, {NULL}};
const Segment SGI391246[] = {{&S39597[0], 23}, {NULL}};
const Segment SGI391247[] = {{&S39597[70], 7}, {NULL}};
const Segment SGI391248[] = {{&S39597[55], 4}, {NULL}};
const Segment SGI391249[] = {{&S39598[0], 23}, {NULL}};
const Segment SGI391250[] = {{&S39598[70], 10}, {NULL}};
const Segment SGI391251[] = {{&S39598[55], 4}, {NULL}};
const Segment SGI391252[] = {{&S39599[0], 8}, {NULL}};
const Segment SGI391253[] = {{&S39599[53], 10}, {NULL}};
const Segment SGI391254[] = {{&S39599[38], 4}, {NULL}};
const Segment SGI391255[] = {{&S39600[0], 10}, {NULL}};
const Segment SGI391256[] = {{&S39600[62], 10}, {NULL}};
const Segment SGI391257[] = {{&S39600[47], 4}, {NULL}};
const Segment SGI391258[] = {{&S39601[0], 10}, {NULL}};
const Segment SGI391259[] = {{&S39601[62], 7}, {NULL}};
const Segment SGI391260[] = {{&S39601[47], 4}, {NULL}};
const Segment SGI391261[] = {{&S39602[0], 13}, {NULL}};
const Segment SGI391262[] = {{&S39602[60], 10}, {NULL}};
const Segment SGI391263[] = {{&S39602[45], 4}, {NULL}};
const Segment SGI391264[] = {{&S39603[0], 10}, {NULL}};
const Segment SGI391265[] = {{&S39603[62], 4}, {NULL}};
const Segment SGI391266[] = {{&S39603[47], 4}, {NULL}};
const Segment SGI391267[] = {{&S39604[0], 29}, {NULL}};
const Segment SGI391268[] = {{&S39604[82], 10}, {NULL}};
const Segment SGI391269[] = {{&S39604[67], 4}, {NULL}};
const Segment SGI391270[] = {{&S39605[0], 10}, {NULL}};
const Segment SGI391271[] = {{&S39605[62], 7}, {NULL}};
const Segment SGI391272[] = {{&S39605[47], 4}, {NULL}};
const Segment SGI391273[] = {{&S39606[0], 43}, {NULL}};
const Segment SGI391274[] = {{&S39606[75], 10}, {NULL}};
const Segment SGI391275[] = {{&S39606[60], 4}, {NULL}};
const Segment SGI391276[] = {{&S39607[0], 29}, {NULL}};
const Segment SGI391277[] = {{&S39607[82], 4}, {NULL}};
const Segment SGI391278[] = {{&S39607[67], 4}, {NULL}};
const Segment SGI391279[] = {{&S39608[0], 8}, {NULL}};
const Segment SGI391280[] = {{&S39608[53], 7}, {NULL}};
const Segment SGI391281[] = {{&S39608[38], 4}, {NULL}};
const Segment SGI391282[] = {{&S39609[0], 10}, {NULL}};
const Segment SGI391283[] = {{&S39609[62], 10}, {NULL}};
const Segment SGI391284[] = {{&S39609[47], 4}, {NULL}};
const Segment SGI391285[] = {{&S39610[0], 15}, {NULL}};
const Segment SGI391286[] = {{&S39610[63], 10}, {NULL}};
const Segment SGI391287[] = {{&S39610[48], 4}, {NULL}};
const Segment SGI391288[] = {{&S39611[0], 40}, {NULL}};
const Segment SGI391289[] = {{&S39611[74], 10}, {NULL}};
const Segment SGI391290[] = {{&S39611[57], 4}, {NULL}};
const Segment SGI391291[] = {{&S39612[0], 40}, {NULL}};
const Segment SGI391292[] = {{&S39612[72], 4}, {NULL}};
const Segment SGI391293[] = {{&S39612[57], 4}, {NULL}};
const Segment SGI391294[] = {{&S39613[0], 10}, {NULL}};
const Segment SGI391295[] = {{&S39613[62], 6}, {NULL}};
const Segment SGI391296[] = {{&S39613[47], 4}, {NULL}};
const Segment SGI391297[] = {{&S39614[0], 17}, {NULL}};
const Segment SGI391298[] = {{&S39614[65], 10}, {NULL}};
const Segment SGI391299[] = {{&S39614[50], 4}, {NULL}};
const Segment SGI391300[] = {{&S39615[0], 40}, {NULL}};
const Segment SGI391301[] = {{&S39615[72], 7}, {NULL}};
const Segment SGI391302[] = {{&S39615[57], 4}, {NULL}};
const Segment SGI391303[] = {{&S39616[0], 45}, {NULL}};
const Segment SGI391304[] = {{&S39616[77], 10}, {NULL}};
const Segment SGI391305[] = {{&S39616[62], 4}, {NULL}};
const Segment SGI391306[] = {{&S39617[0], 40}, {NULL}};
const Segment SGI391307[] = {{&S39617[72], 10}, {NULL}};
const Segment SGI391308[] = {{&S39617[57], 4}, {NULL}};
const Segment SGI391309[] = {{&S39618[0], 43}, {NULL}};
const Segment SGI391310[] = {{&S39618[75], 7}, {NULL}};
const Segment SGI391311[] = {{&S39618[60], 4}, {NULL}};
const Segment SGI391312[] = {{&S39619[0], 43}, {NULL}};
const Segment SGI391313[] = {{&S39619[75], 10}, {NULL}};
const Segment SGI391314[] = {{&S39619[60], 4}, {NULL}};
const Segment SGI391315[] = {{&S39620[0], 8}, {NULL}};
const Segment SGI391316[] = {{&S39620[54], 10}, {NULL}};
const Segment SGI391317[] = {{&S39620[39], 4}, {NULL}};
const Segment SGI391318[] = {{&S39621[0], 17}, {NULL}};
const Segment SGI391319[] = {{&S39621[63], 10}, {NULL}};
const Segment SGI391320[] = {{&S39621[48], 4}, {NULL}};
const Segment SGI391321[] = {{&S39622[0], 33}, {NULL}};
const Segment SGI391322[] = {{&S39622[65], 10}, {NULL}};
const Segment SGI391323[] = {{&S39622[50], 4}, {NULL}};
const Segment SGI391324[] = {{&S39623[0], 8}, {NULL}};
const Segment SGI391325[] = {{&S39623[54], 4}, {NULL}};
const Segment SGI391326[] = {{&S39623[39], 4}, {NULL}};
const Segment SGI391327[] = {{&S39624[0], 45}, {NULL}};
const Segment SGI391328[] = {{&S39624[77], 7}, {NULL}};
const Segment SGI391329[] = {{&S39624[62], 4}, {NULL}};
const Segment SGI391330[] = {{&S39625[0], 40}, {NULL}};
const Segment SGI391331[] = {{&S39625[72], 7}, {NULL}};
const Segment SGI391332[] = {{&S39625[57], 4}, {NULL}};
const Segment SGI391333[] = {{&S79912[8], 3}, {NULL}};
const Segment SGI391334[] = {{&S79912[13], 3}, {NULL}};
const Segment SGI391335[] = {{&S79912[73], 6}, {NULL}};
const Segment SGI391336[] = {{&S79912[58], 13}, {NULL}};
const Segment SGI391337[] = {{&S76156[39], 4}, {NULL}};
const Segment SGI391338[] = {{&S76156[8], 3}, {NULL}};
const Segment SGI391339[] = {{&S76156[13], 14}, {NULL}};
const Segment SGI391340[] = {{&S80214[8], 3}, {NULL}};
const Segment SGI391341[] = {{&S80214[13], 7}, {NULL}};
const Segment SGI391342[] = {{&S80214[63], 13}, {NULL}};
const Segment SGI391343[] = {{&S71684[93], 3}, {NULL}};
const Segment SGI391344[] = {{&S71684[29], 6}, {NULL}};
const Segment SGI391345[] = {{&S71684[128], 5}, {NULL}};
const Segment SGI391346[] = {{&S40441[0], 12}, {NULL}};
const Segment SGI391347[] = {{&S40441[59], 10}, {NULL}};
const Segment SGI391348[] = {{&S40441[42], 4}, {NULL}};
const Segment SGI391349[] = {{&S40442[0], 8}, {NULL}};
const Segment SGI391350[] = {{&S40442[54], 7}, {NULL}};
const Segment SGI391351[] = {{&S40442[37], 4}, {NULL}};
const Segment SGI391352[] = {{&S40443[0], 40}, {NULL}};
const Segment SGI391353[] = {{&S40443[73], 10}, {NULL}};
const Segment SGI391354[] = {{&S40443[56], 4}, {NULL}};
const Segment SGI391355[] = {{&S40444[0], 29}, {NULL}};
const Segment SGI391356[] = {{&S40444[83], 10}, {NULL}};
const Segment SGI391357[] = {{&S40444[66], 4}, {NULL}};
const Segment SGI391358[] = {{&S40446[0], 40}, {NULL}};
const Segment SGI391359[] = {{&S40446[73], 10}, {NULL}};
const Segment SGI391360[] = {{&S40446[56], 4}, {NULL}};
const Segment SGI391361[] = {{&S40447[0], 40}, {NULL}};
const Segment SGI391362[] = {{&S40447[73], 4}, {NULL}};
const Segment SGI391363[] = {{&S40447[56], 4}, {NULL}};
const Segment SGI391364[] = {{&S40448[0], 23}, {NULL}};
const Segment SGI391365[] = {{&S40448[71], 10}, {NULL}};
const Segment SGI391366[] = {{&S40448[54], 4}, {NULL}};
const Segment SGI391367[] = {{&S40449[0], 10}, {NULL}};
const Segment SGI391368[] = {{&S40449[64], 10}, {NULL}};
const Segment SGI391369[] = {{&S40449[47], 4}, {NULL}};
const Segment SGI391370[] = {{&S40450[0], 10}, {NULL}};
const Segment SGI391371[] = {{&S40450[64], 4}, {NULL}};
const Segment SGI391372[] = {{&S40450[47], 4}, {NULL}};
const Segment SGI391373[] = {{&S40451[0], 10}, {NULL}};
const Segment SGI391374[] = {{&S40451[64], 10}, {NULL}};
const Segment SGI391375[] = {{&S40451[47], 4}, {NULL}};
const Segment SGI391376[] = {{&S40452[0], 16}, {NULL}};
const Segment SGI391377[] = {{&S40452[69], 10}, {NULL}};
const Segment SGI391378[] = {{&S40452[52], 4}, {NULL}};
const Segment SGI391379[] = {{&S40453[0], 16}, {NULL}};
const Segment SGI391380[] = {{&S40453[69], 10}, {NULL}};
const Segment SGI391381[] = {{&S40453[52], 4}, {NULL}};
const Segment SGI391382[] = {{&S40454[0], 33}, {NULL}};
const Segment SGI391383[] = {{&S40454[66], 10}, {NULL}};
const Segment SGI391384[] = {{&S40454[49], 4}, {NULL}};
const Segment SGI391385[] = {{&S40455[0], 43}, {NULL}};
const Segment SGI391386[] = {{&S40455[76], 10}, {NULL}};
const Segment SGI391387[] = {{&S40455[59], 4}, {NULL}};
const Segment SGI391388[] = {{&S40456[0], 33}, {NULL}};
const Segment SGI391389[] = {{&S40456[66], 10}, {NULL}};
const Segment SGI391390[] = {{&S40456[49], 4}, {NULL}};
const Segment SGI391391[] = {{&S40457[0], 40}, {NULL}};
const Segment SGI391392[] = {{&S40457[73], 7}, {NULL}};
const Segment SGI391393[] = {{&S40457[56], 4}, {NULL}};
const Segment SGI391394[] = {{&S40458[0], 43}, {NULL}};
const Segment SGI391395[] = {{&S40458[76], 10}, {NULL}};
const Segment SGI391396[] = {{&S40458[59], 4}, {NULL}};
const Segment SGI391397[] = {{&S40459[0], 23}, {NULL}};
const Segment SGI391398[] = {{&S40459[71], 4}, {NULL}};
const Segment SGI391399[] = {{&S40459[54], 4}, {NULL}};
const Segment SGI391400[] = {{&S40460[0], 23}, {NULL}};
const Segment SGI391401[] = {{&S40460[71], 10}, {NULL}};
const Segment SGI391402[] = {{&S40460[54], 4}, {NULL}};
const Segment SGI391403[] = {{&S40461[0], 16}, {NULL}};
const Segment SGI391404[] = {{&S40461[69], 4}, {NULL}};
const Segment SGI391405[] = {{&S40461[52], 4}, {NULL}};
const Segment SGI391406[] = {{&S40462[0], 10}, {NULL}};
const Segment SGI391407[] = {{&S40462[64], 7}, {NULL}};
const Segment SGI391408[] = {{&S40462[47], 4}, {NULL}};
const Segment SGI391409[] = {{&S40463[0], 13}, {NULL}};
const Segment SGI391410[] = {{&S40463[61], 10}, {NULL}};
const Segment SGI391411[] = {{&S40463[44], 4}, {NULL}};
const Segment SGI391412[] = {{&S40464[0], 13}, {NULL}};
const Segment SGI391413[] = {{&S40464[61], 4}, {NULL}};
const Segment SGI391414[] = {{&S40464[44], 4}, {NULL}};
const Segment SGI391415[] = {{&S40465[0], 45}, {NULL}};
const Segment SGI391416[] = {{&S40465[78], 10}, {NULL}};
const Segment SGI391417[] = {{&S40465[61], 4}, {NULL}};
const Segment SGI391418[] = {{&S40466[0], 40}, {NULL}};
const Segment SGI391419[] = {{&S40466[73], 7}, {NULL}};
const Segment SGI391420[] = {{&S40466[56], 4}, {NULL}};
const Segment SGI391421[] = {{&S40467[0], 16}, {NULL}};
const Segment SGI391422[] = {{&S40467[67], 6}, {NULL}};
const Segment SGI391423[] = {{&S40467[52], 4}, {NULL}};
const Segment SGI391424[] = {{&S80742[8], 3}, {NULL}};
const Segment SGI391425[] = {{&S80742[13], 7}, {NULL}};
const Segment SGI391426[] = {{&S80742[25], 14}, {NULL}};
const Segment SGI391427[] = {{&S80742[64], 12}, {NULL}};
const Segment SGI391428[] = {{&S80758[8], 3}, {NULL}};
const Segment SGI391429[] = {{&S80758[13], 7}, {NULL}};
const Segment SGI391430[] = {{&S80758[25], 14}, {NULL}};
const Segment SGI391431[] = {{&S80758[78], 12}, {NULL}};
const Segment SGI391432[] = {{&S76749[39], 4}, {NULL}};
const Segment SGI391433[] = {{&S76749[8], 3}, {NULL}};
const Segment SGI391434[] = {{&S76749[13], 14}, {NULL}};
const Segment SGI391435[] = {{&S40624[0], 8}, {NULL}};
const Segment SGI391436[] = {{&S40624[36], 4}, {NULL}};
const Segment SGI391437[] = {{&S40625[0], 15}, {NULL}};
const Segment SGI391438[] = {{&S40625[45], 4}, {NULL}};
const Segment SGI391439[] = {{&S40626[0], 3}, {NULL}};
const Segment SGI391440[] = {{&S40626[33], 4}, {NULL}};
const Segment SGI391441[] = {{&S40627[0], 13}, {NULL}};
const Segment SGI391442[] = {{&S40627[45], 4}, {NULL}};
const Segment SGI391443[] = {{&S40628[0], 5}, {NULL}};
const Segment SGI391444[] = {{&S40628[43], 4}, {NULL}};
const Segment SGI391445[] = {{&S40629[0], 10}, {NULL}};
const Segment SGI391446[] = {{&S40629[40], 4}, {NULL}};
const Segment SGI391447[] = {{&S40630[0], 10}, {NULL}};
const Segment SGI391448[] = {{&S40630[40], 4}, {NULL}};
const Segment SGI391449[] = {{&S40631[0], 7}, {NULL}};
const Segment SGI391450[] = {{&S40631[37], 4}, {NULL}};
const Segment SGI391451[] = {{&S40632[0], 6}, {NULL}};
const Segment SGI391452[] = {{&S40632[44], 4}, {NULL}};
const Segment SGI391453[] = {{&S40633[0], 11}, {NULL}};
const Segment SGI391454[] = {{&S40633[41], 4}, {NULL}};
const Segment SGI391455[] = {{&S40634[0], 11}, {NULL}};
const Segment SGI391456[] = {{&S40634[43], 4}, {NULL}};
const Segment SGI391457[] = {{&S40635[0], 9}, {NULL}};
const Segment SGI391458[] = {{&S40635[41], 4}, {NULL}};
const Segment SGI391459[] = {{&S40636[0], 40}, {NULL}};
const Segment SGI391460[] = {{&S40636[56], 4}, {NULL}};
const Segment SGI391461[] = {{&S40637[0], 9}, {NULL}};
const Segment SGI391462[] = {{&S40637[40], 4}, {NULL}};
const Segment SGI391463[] = {{&S40638[0], 19}, {NULL}};
const Segment SGI391464[] = {{&S40638[49], 4}, {NULL}};
const Segment SGI391465[] = {{&S40639[0], 6}, {NULL}};
const Segment SGI391466[] = {{&S40639[35], 4}, {NULL}};
const Segment SGI391467[] = {{&S40640[0], 10}, {NULL}};
const Segment SGI391468[] = {{&S40640[42], 4}, {NULL}};
const Segment SGI391469[] = {{&S40641[0], 26}, {NULL}};
const Segment SGI391470[] = {{&S40641[57], 4}, {NULL}};
const Segment SGI391471[] = {{&S40642[0], 7}, {NULL}};
const Segment SGI391472[] = {{&S40642[39], 4}, {NULL}};
const Segment SGI391473[] = {{&S40643[0], 17}, {NULL}};
const Segment SGI391474[] = {{&S40643[47], 4}, {NULL}};
const Segment SGI391475[] = {{&S40644[0], 17}, {NULL}};
const Segment SGI391476[] = {{&S40644[54], 4}, {NULL}};
const Segment SGI391477[] = {{&S40645[0], 40}, {NULL}};
const Segment SGI391478[] = {{&S40645[56], 4}, {NULL}};
const Segment SGI391479[] = {{&S40646[0], 6}, {NULL}};
const Segment SGI391480[] = {{&S40646[38], 4}, {NULL}};
const Segment SGI391481[] = {{&S40647[0], 10}, {NULL}};
const Segment SGI391482[] = {{&S40647[42], 4}, {NULL}};
const Segment SGI391483[] = {{&S40648[0], 35}, {NULL}};
const Segment SGI391484[] = {{&S40648[51], 4}, {NULL}};
const Segment SGI391485[] = {{&S40649[0], 16}, {NULL}};
const Segment SGI391486[] = {{&S40649[48], 4}, {NULL}};
const Segment SGI391487[] = {{&S40650[0], 11}, {NULL}};
const Segment SGI391488[] = {{&S40650[44], 4}, {NULL}};
const Segment SGI391489[] = {{&S40651[0], 8}, {NULL}};
const Segment SGI391490[] = {{&S40651[40], 4}, {NULL}};
const Segment SGI391491[] = {{&S40652[0], 7}, {NULL}};
const Segment SGI391492[] = {{&S40652[39], 4}, {NULL}};
const Segment SGI391493[] = {{&S40653[0], 8}, {NULL}};
const Segment SGI391494[] = {{&S40653[38], 4}, {NULL}};
const Segment SGI391495[] = {{&S40654[0], 9}, {NULL}};
const Segment SGI391496[] = {{&S40654[41], 4}, {NULL}};
const Segment SGI391497[] = {{&S40655[0], 18}, {NULL}};
const Segment SGI391498[] = {{&S40655[50], 4}, {NULL}};
const Segment SGI391499[] = {{&S40656[0], 11}, {NULL}};
const Segment SGI391500[] = {{&S40656[43], 4}, {NULL}};
const Segment SGI391501[] = {{&S40657[0], 17}, {NULL}};
const Segment SGI391502[] = {{&S40657[49], 4}, {NULL}};
const Segment SGI391503[] = {{&S40658[0], 41}, {NULL}};
const Segment SGI391504[] = {{&S40658[57], 4}, {NULL}};
const Segment SGI391505[] = {{&S40659[0], 39}, {NULL}};
const Segment SGI391506[] = {{&S40659[55], 4}, {NULL}};
const Segment SGI391507[] = {{&S40660[0], 11}, {NULL}};
const Segment SGI391508[] = {{&S40660[40], 4}, {NULL}};
const Segment SGI391509[] = {{&S40661[0], 37}, {NULL}};
const Segment SGI391510[] = {{&S40661[53], 4}, {NULL}};
const Segment SGI391511[] = {{&S40662[0], 17}, {NULL}};
const Segment SGI391512[] = {{&S40662[45], 4}, {NULL}};
const Segment SGI391513[] = {{&S40663[0], 7}, {NULL}};
const Segment SGI391514[] = {{&S40663[37], 4}, {NULL}};
const Segment SGI391515[] = {{&S40664[0], 6}, {NULL}};
const Segment SGI391516[] = {{&S40664[36], 4}, {NULL}};
const Segment SGI391517[] = {{&S40665[0], 15}, {NULL}};
const Segment SGI391518[] = {{&S40665[46], 4}, {NULL}};
const Segment SGI391519[] = {{&S40666[0], 14}, {NULL}};
const Segment SGI391520[] = {{&S40666[46], 4}, {NULL}};
const Segment SGI391521[] = {{&S40667[0], 7}, {NULL}};
const Segment SGI391522[] = {{&S40667[40], 4}, {NULL}};
const Segment SGI391523[] = {{&S40668[0], 12}, {NULL}};
const Segment SGI391524[] = {{&S40668[43], 4}, {NULL}};
const Segment SGI391525[] = {{&S40669[0], 11}, {NULL}};
const Segment SGI391526[] = {{&S40669[42], 4}, {NULL}};
const Segment SGI391527[] = {{&S40670[0], 15}, {NULL}};
const Segment SGI391528[] = {{&S40670[45], 4}, {NULL}};
const Segment SGI391529[] = {{&S40671[0], 54}, {NULL}};
const Segment SGI391530[] = {{&S40671[85], 4}, {NULL}};
const Segment SGI391531[] = {{&S40672[0], 7}, {NULL}};
const Segment SGI391532[] = {{&S40672[39], 4}, {NULL}};
const Segment SGI391533[] = {{&S40673[0], 14}, {NULL}};
const Segment SGI391534[] = {{&S40673[44], 4}, {NULL}};
const Segment SGI391535[] = {{&S40674[0], 9}, {NULL}};
const Segment SGI391536[] = {{&S40674[41], 4}, {NULL}};
const Segment SGI391537[] = {{&S40675[0], 9}, {NULL}};
const Segment SGI391538[] = {{&S40675[41], 4}, {NULL}};
const Segment SGI391539[] = {{&S40676[0], 8}, {NULL}};
const Segment SGI391540[] = {{&S40676[36], 4}, {NULL}};
const Segment SGI391541[] = {{&S40677[0], 8}, {NULL}};
const Segment SGI391542[] = {{&S40677[38], 4}, {NULL}};
const Segment SGI391543[] = {{&S40678[0], 13}, {NULL}};
const Segment SGI391544[] = {{&S40678[46], 4}, {NULL}};
const Segment SGI391545[] = {{&S40679[0], 11}, {NULL}};
const Segment SGI391546[] = {{&S40679[43], 4}, {NULL}};
const Segment SGI391547[] = {{&S40680[0], 14}, {NULL}};
const Segment SGI391548[] = {{&S40680[47], 4}, {NULL}};
const Segment SGI391549[] = {{&S40681[0], 7}, {NULL}};
const Segment SGI391550[] = {{&S40681[37], 4}, {NULL}};
const Segment SGI391551[] = {{&S40682[0], 11}, {NULL}};
const Segment SGI391552[] = {{&S40682[43], 4}, {NULL}};
const Segment SGI391553[] = {{&S40683[0], 3}, {NULL}};
const Segment SGI391554[] = {{&S40683[35], 4}, {NULL}};
const Segment SGI391555[] = {{&S40684[0], 18}, {NULL}};
const Segment SGI391556[] = {{&S40684[48], 4}, {NULL}};
const Segment SGI391557[] = {{&S40685[0], 8}, {NULL}};
const Segment SGI391558[] = {{&S40685[40], 4}, {NULL}};
const Segment SGI391559[] = {{&S40686[0], 18}, {NULL}};
const Segment SGI391560[] = {{&S40686[51], 4}, {NULL}};
const Segment SGI391561[] = {{&S40687[0], 9}, {NULL}};
const Segment SGI391562[] = {{&S40687[39], 4}, {NULL}};
const Segment SGI391563[] = {{&S40688[0], 10}, {NULL}};
const Segment SGI391564[] = {{&S40688[42], 4}, {NULL}};
const Segment SGI391565[] = {{&S40689[0], 13}, {NULL}};
const Segment SGI391566[] = {{&S40689[43], 4}, {NULL}};
const Segment SGI391567[] = {{&S40690[0], 17}, {NULL}};
const Segment SGI391568[] = {{&S40690[49], 4}, {NULL}};
const Segment SGI391569[] = {{&S40691[0], 7}, {NULL}};
const Segment SGI391570[] = {{&S40691[39], 4}, {NULL}};
const Segment SGI391571[] = {{&S40692[0], 7}, {NULL}};
const Segment SGI391572[] = {{&S40692[37], 4}, {NULL}};
const Segment SGI391573[] = {{&S40693[0], 5}, {NULL}};
const Segment SGI391574[] = {{&S40693[37], 4}, {NULL}};
const Segment SGI391575[] = {{&S40694[0], 22}, {NULL}};
const Segment SGI391576[] = {{&S40694[52], 4}, {NULL}};
const Segment SGI391577[] = {{&S40695[0], 19}, {NULL}};
const Segment SGI391578[] = {{&S40695[49], 4}, {NULL}};
const Segment SGI391579[] = {{&S40696[0], 7}, {NULL}};
const Segment SGI391580[] = {{&S40696[37], 4}, {NULL}};
const Segment SGI391581[] = {{&S40697[0], 15}, {NULL}};
const Segment SGI391582[] = {{&S40697[45], 4}, {NULL}};
const Segment SGI391583[] = {{&S40698[0], 13}, {NULL}};
const Segment SGI391584[] = {{&S40698[45], 4}, {NULL}};
const Segment SGI391585[] = {{&S40699[0], 11}, {NULL}};
const Segment SGI391586[] = {{&S40699[44], 4}, {NULL}};
const Segment SGI391587[] = {{&S40700[0], 11}, {NULL}};
const Segment SGI391588[] = {{&S40700[43], 4}, {NULL}};
const Segment SGI391589[] = {{&S40701[0], 25}, {NULL}};
const Segment SGI391590[] = {{&S40701[55], 4}, {NULL}};
const Segment SGI391591[] = {{&S40702[0], 11}, {NULL}};
const Segment SGI391592[] = {{&S40702[43], 4}, {NULL}};
const Segment SGI391593[] = {{&S40703[0], 7}, {NULL}};
const Segment SGI391594[] = {{&S40703[40], 4}, {NULL}};
const Segment SGI391595[] = {{&S40704[0], 5}, {NULL}};
const Segment SGI391596[] = {{&S40704[37], 4}, {NULL}};
const Segment SGI391597[] = {{&S76906[32], 4}, {NULL}};
const Segment SGI391598[] = {{&S76906[8], 3}, {NULL}};
const Segment SGI391599[] = {{&S76906[13], 14}, {NULL}};
const Segment SGI391600[] = {{&S81135[8], 3}, {NULL}};
const Segment SGI391601[] = {{&S81135[13], 9}, {NULL}};
const Segment SGI391602[] = {{&S81135[33], 13}, {NULL}};
const Segment SGI391603[] = {{&S81135[85], 12}, {NULL}};
const Segment SGI391604[] = {{&S72350[108], 3}, {NULL}};
const Segment SGI391605[] = {{&S72350[44], 6}, {NULL}};
const Segment SGI391606[] = {{&S72350[30], 6}, {NULL}};
const Segment SGI391607[] = {{&S72350[150], 5}, {NULL}};
const Segment SGI391608[] = {{&S77135[33], 4}, {NULL}};
const Segment SGI391609[] = {{&S77135[8], 3}, {NULL}};
const Segment SGI391610[] = {{&S77135[13], 3}, {NULL}};
const Segment SGI391611[] = {{&S77269[48], 4}, {NULL}};
const Segment SGI391612[] = {{&S77269[8], 3}, {NULL}};
const Segment SGI391613[] = {{&S77269[13], 9}, {NULL}};
const Segment SGI391614[] = {{&S77269[30], 13}, {NULL}};
const Segment SGI391615[] = {{&S81483[8], 3}, {NULL}};
const Segment SGI391616[] = {{&S81483[13], 7}, {NULL}};
const Segment SGI391617[] = {{&S81483[25], 14}, {NULL}};
const Segment SGI391618[] = {{&S81483[71], 13}, {NULL}};
const Segment SGI391619[] = {{&S72782[108], 3}, {NULL}};
const Segment SGI391620[] = {{&S72782[44], 6}, {NULL}};
const Segment SGI391621[] = {{&S72782[30], 6}, {NULL}};
const Segment SGI391622[] = {{&S72782[150], 4}, {NULL}};
const Segment SGI391623[] = {{&S77593[32], 4}, {NULL}};
const Segment SGI391624[] = {{&S77593[8], 3}, {NULL}};
const Segment SGI391625[] = {{&S77593[13], 14}, {NULL}};
const Segment SGI391626[] = {{&S72866[108], 3}, {NULL}};
const Segment SGI391627[] = {{&S72866[44], 6}, {NULL}};
const Segment SGI391628[] = {{&S72866[30], 6}, {NULL}};
const Segment SGI391629[] = {{&S72866[150], 5}, {NULL}};
const Segment SGI391630[] = {{&S81740[8], 3}, {NULL}};
const Segment SGI391631[] = {{&S81740[13], 9}, {NULL}};
const Segment SGI391632[] = {{&S81740[33], 13}, {NULL}};
const Segment SGI391633[] = {{&S81740[76], 13}, {NULL}};
const Segment SGI391634[] = {{&S41318[0], 6}, {NULL}};
const Segment SGI391635[] = {{&S41318[59], 10}, {NULL}};
const Segment SGI391636[] = {{&S41318[42], 4}, {NULL}};
const Segment SGI391637[] = {{&S41319[0], 6}, {NULL}};
const Segment SGI391638[] = {{&S41319[59], 4}, {NULL}};
const Segment SGI391639[] = {{&S41319[42], 4}, {NULL}};
const Segment SGI391640[] = {{&S41320[0], 6}, {NULL}};
const Segment SGI391641[] = {{&S41320[59], 10}, {NULL}};
const Segment SGI391642[] = {{&S41320[42], 4}, {NULL}};
const Segment SGI391643[] = {{&S41321[0], 6}, {NULL}};
const Segment SGI391644[] = {{&S41321[59], 6}, {NULL}};
const Segment SGI391645[] = {{&S41321[42], 4}, {NULL}};
const Segment SGI391646[] = {{&S41322[0], 6}, {NULL}};
const Segment SGI391647[] = {{&S41322[59], 7}, {NULL}};
const Segment SGI391648[] = {{&S41322[42], 4}, {NULL}};
const Segment SGI391649[] = {{&S41323[0], 6}, {NULL}};
const Segment SGI391650[] = {{&S41323[59], 7}, {NULL}};
const Segment SGI391651[] = {{&S41323[42], 4}, {NULL}};
const Segment SGI391652[] = {{&S81801[8], 3}, {NULL}};
const Segment SGI391653[] = {{&S81801[13], 3}, {NULL}};
const Segment SGI391654[] = {{&S81801[13], 3}, {&S81801[18], 10}, {NULL}};
const Segment SGI391655[] = {{&S81801[51], 13}, {NULL}};
const Segment SGI391656[] = {{&S81811[8], 3}, {NULL}};
const Segment SGI391657[] = {{&S81811[13], 7}, {NULL}};
const Segment SGI391658[] = {{&S81811[25], 14}, {NULL}};
const Segment SGI391659[] = {{&S81811[78], 12}, {NULL}};
const Segment SGI391660[] = {{&S81888[8], 3}, {NULL}};
const Segment SGI391661[] = {{&S81888[13], 7}, {NULL}};
const Segment SGI391662[] = {{&S81888[25], 5}, {NULL}};
const Segment SGI391663[] = {{&S81888[52], 13}, {NULL}};
const Segment SGI391664[] = {{&S73202[109], 3}, {NULL}};
const Segment SGI391665[] = {{&S73202[44], 7}, {NULL}};
const Segment SGI391666[] = {{&S73202[30], 6}, {NULL}};
const Segment SGI391667[] = {{&S73202[151], 4}, {NULL}};
const Segment SGI391668[] = {{&S82047[8], 3}, {NULL}};
const Segment SGI391669[] = {{&S82047[13], 7}, {NULL}};
const Segment SGI391670[] = {{&S82047[25], 14}, {NULL}};
const Segment SGI391671[] = {{&S82047[68], 9}, {NULL}};
const Segment SGI391672[] = {{&S73317[108], 3}, {NULL}};
const Segment SGI391673[] = {{&S73317[44], 6}, {NULL}};
const Segment SGI391674[] = {{&S73317[30], 6}, {NULL}};
const Segment SGI391675[] = {{&S73317[150], 5}, {NULL}};
const Segment SGI391676[] = {{&S41594[0], 16}, {NULL}};
const Segment SGI391677[] = {{&S41594[69], 10}, {NULL}};
const Segment SGI391678[] = {{&S41594[52], 4}, {NULL}};
const Segment SGI391679[] = {{&S41595[0], 16}, {NULL}};
const Segment SGI391680[] = {{&S41595[69], 4}, {NULL}};
const Segment SGI391681[] = {{&S41595[52], 4}, {NULL}};
const Segment SGI391682[] = {{&S41596[0], 8}, {NULL}};
const Segment SGI391683[] = {{&S41596[54], 10}, {NULL}};
const Segment SGI391684[] = {{&S41596[37], 4}, {NULL}};
const Segment SGI391685[] = {{&S41597[0], 9}, {NULL}};
const Segment SGI391686[] = {{&S41597[57], 10}, {NULL}};
const Segment SGI391687[] = {{&S41597[40], 4}, {NULL}};
const Segment SGI391688[] = {{&S41598[0], 40}, {NULL}};
const Segment SGI391689[] = {{&S41598[73], 10}, {NULL}};
const Segment SGI391690[] = {{&S41598[56], 4}, {NULL}};
const Segment SGI391691[] = {{&S41599[0], 14}, {NULL}};
const Segment SGI391692[] = {{&S41599[63], 10}, {NULL}};
const Segment SGI391693[] = {{&S41599[46], 4}, {NULL}};
const Segment SGI391694[] = {{&S41600[0], 16}, {NULL}};
const Segment SGI391695[] = {{&S41600[69], 10}, {NULL}};
const Segment SGI391696[] = {{&S41600[52], 4}, {NULL}};
const Segment SGI391697[] = {{&S41601[0], 8}, {NULL}};
const Segment SGI391698[] = {{&S41601[54], 10}, {NULL}};
const Segment SGI391699[] = {{&S41601[37], 4}, {NULL}};
const Segment SGI391700[] = {{&S41602[0], 14}, {NULL}};
const Segment SGI391701[] = {{&S41602[63], 4}, {NULL}};
const Segment SGI391702[] = {{&S41602[46], 4}, {NULL}};
const Segment SGI391703[] = {{&S41603[0], 10}, {NULL}};
const Segment SGI391704[] = {{&S41603[64], 10}, {NULL}};
const Segment SGI391705[] = {{&S41603[47], 4}, {NULL}};
const Segment SGI391706[] = {{&S41604[0], 40}, {NULL}};
const Segment SGI391707[] = {{&S41604[73], 10}, {NULL}};
const Segment SGI391708[] = {{&S41604[56], 4}, {NULL}};
const Segment SGI391709[] = {{&S41605[0], 10}, {NULL}};
const Segment SGI391710[] = {{&S41605[64], 10}, {NULL}};
const Segment SGI391711[] = {{&S41605[47], 4}, {NULL}};
const Segment SGI391712[] = {{&S41606[0], 10}, {NULL}};
const Segment SGI391713[] = {{&S41606[64], 4}, {NULL}};
const Segment SGI391714[] = {{&S41606[47], 4}, {NULL}};
const Segment SGI391715[] = {{&S41607[0], 10}, {NULL}};
const Segment SGI391716[] = {{&S41607[64], 7}, {NULL}};
const Segment SGI391717[] = {{&S41607[47], 4}, {NULL}};
const Segment SGI391718[] = {{&S41608[0], 23}, {NULL}};
const Segment SGI391719[] = {{&S41608[71], 10}, {NULL}};
const Segment SGI391720[] = {{&S41608[54], 4}, {NULL}};
const Segment SGI391721[] = {{&S41609[0], 16}, {NULL}};
const Segment SGI391722[] = {{&S41609[69], 7}, {NULL}};
const Segment SGI391723[] = {{&S41609[52], 4}, {NULL}};
const Segment SGI391724[] = {{&S41610[0], 17}, {NULL}};
const Segment SGI391725[] = {{&S41610[64], 4}, {NULL}};
const Segment SGI391726[] = {{&S41610[47], 4}, {NULL}};
const Segment SGI391727[] = {{&S41611[0], 40}, {NULL}};
const Segment SGI391728[] = {{&S41611[73], 4}, {NULL}};
const Segment SGI391729[] = {{&S41611[56], 4}, {NULL}};
const Segment SGI391730[] = {{&S41612[0], 16}, {NULL}};
const Segment SGI391731[] = {{&S41612[69], 6}, {NULL}};
const Segment SGI391732[] = {{&S41612[52], 4}, {NULL}};
const Segment SGI391733[] = {{&S41613[0], 11}, {NULL}};
const Segment SGI391734[] = {{&S41613[60], 10}, {NULL}};
const Segment SGI391735[] = {{&S41613[43], 4}, {NULL}};
const Segment SGI391736[] = {{&S41614[0], 16}, {NULL}};
const Segment SGI391737[] = {{&S41614[69], 7}, {NULL}};
const Segment SGI391738[] = {{&S41614[52], 4}, {NULL}};
const Segment SGI391739[] = {{&S41615[0], 17}, {NULL}};
const Segment SGI391740[] = {{&S41615[65], 10}, {NULL}};
const Segment SGI391741[] = {{&S41615[48], 4}, {NULL}};
const Segment SGI391742[] = {{&S41616[0], 43}, {NULL}};
const Segment SGI391743[] = {{&S41616[76], 10}, {NULL}};
const Segment SGI391744[] = {{&S41616[59], 4}, {NULL}};
const Segment SGI391745[] = {{&S41617[0], 40}, {NULL}};
const Segment SGI391746[] = {{&S41617[73], 7}, {NULL}};
const Segment SGI391747[] = {{&S41617[56], 4}, {NULL}};
const Segment SGI391748[] = {{&S41618[0], 11}, {NULL}};
const Segment SGI391749[] = {{&S41618[60], 10}, {NULL}};
const Segment SGI391750[] = {{&S41618[43], 4}, {NULL}};
const Segment SGI391751[] = {{&S41619[0], 40}, {NULL}};
const Segment SGI391752[] = {{&S41619[73], 7}, {NULL}};
const Segment SGI391753[] = {{&S41619[56], 4}, {NULL}};
const Segment SGI391754[] = {{&S82102[8], 3}, {NULL}};
const Segment SGI391755[] = {{&S82102[13], 7}, {NULL}};
const Segment SGI391756[] = {{&S82102[25], 14}, {NULL}};
const Segment SGI391757[] = {{&S82102[66], 12}, {NULL}};
const Segment SGI391758[] = {{&S82271[8], 3}, {NULL}};
const Segment SGI391759[] = {{&S82271[13], 7}, {NULL}};
const Segment SGI391760[] = {{&S82271[25], 14}, {NULL}};
const Segment SGI391761[] = {{&S82271[69], 11}, {NULL}};
const Segment SGI391762[] = {{&S82277[8], 3}, {NULL}};
const Segment SGI391763[] = {{&S82277[13], 14}, {NULL}};
const Segment SGI391764[] = {{&S82277[47], 13}, {NULL}};
const Segment SGI391765[] = {{&S73761[93], 3}, {NULL}};
const Segment SGI391766[] = {{&S73761[29], 6}, {NULL}};
const Segment SGI391767[] = {{&S73761[128], 5}, {NULL}};
const Segment SGI391768[] = {{&S73801[93], 3}, {NULL}};
const Segment SGI391769[] = {{&S73801[29], 6}, {NULL}};
const Segment SGI391770[] = {{&S73801[128], 5}, {NULL}};
const Segment SGI391771[] = {{&S78430[79], 3}, {NULL}};
const Segment SGI391772[] = {{&S78430[8], 3}, {NULL}};
const Segment SGI391773[] = {{&S78430[13], 7}, {NULL}};
const Segment SGI391774[] = {{&S78430[25], 14}, {NULL}};
const Segment SGI391775[] = {{&S78492[33], 4}, {NULL}};
const Segment SGI391776[] = {{&S78492[8], 3}, {NULL}};
const Segment SGI391777[] = {{&S78492[13], 3}, {NULL}};
const Segment SGI391778[] = {{&S42434[0], 12}, {NULL}};
const Segment SGI391779[] = {{&S42434[48], 4}, {NULL}};
const Segment SGI391780[] = {{&S74116[94], 3}, {NULL}};
const Segment SGI391781[] = {{&S74116[29], 7}, {NULL}};
const Segment SGI391782[] = {{&S74116[129], 4}, {NULL}};
const Segment SGI391783[] = {{&S74151[108], 3}, {NULL}};
const Segment SGI391784[] = {{&S74151[44], 6}, {NULL}};
const Segment SGI391785[] = {{&S74151[30], 6}, {NULL}};
const Segment SGI391786[] = {{&S74151[150], 4}, {NULL}};
const Segment SGI391787[] = {{&S74174[108], 3}, {NULL}};
const Segment SGI391788[] = {{&S74174[44], 6}, {NULL}};
const Segment SGI391789[] = {{&S74174[30], 6}, {NULL}};
const Segment SGI391790[] = {{&S74174[150], 4}, {NULL}};
const Segment SGI391791[] = {{&S86624[8], 3}, {NULL}};
const Segment SGI391792[] = {{&S86624[13], 7}, {NULL}};
const Segment SGI391793[] = {{&S86624[88], 51}, {NULL}};
const Segment SGI391794[] = {{&S74323[93], 3}, {NULL}};
const Segment SGI391795[] = {{&S74323[29], 6}, {NULL}};
const Segment SGI391796[] = {{&S74323[128], 5}, {NULL}};
const Segment SGI391797[] = {{&S86702[8], 3}, {NULL}};
const Segment SGI391798[] = {{&S86702[13], 7}, {NULL}};
const Segment SGI391799[] = {{&S86702[25], 14}, {NULL}};
const Segment SGI391800[] = {{&S86702[84], 48}, {NULL}};
const Segment SGI391801[] = {{&S83393[8], 3}, {NULL}};
const Segment SGI391802[] = {{&S83393[13], 7}, {NULL}};
const Segment SGI391803[] = {{&S83393[25], 14}, {NULL}};
const Segment SGI391804[] = {{&S83393[68], 13}, {NULL}};
const Segment SGI391805[] = {{&S83396[8], 3}, {NULL}};
const Segment SGI391806[] = {{&S83396[13], 14}, {NULL}};
const Segment SGI391807[] = {{&S83396[55], 13}, {NULL}};
const Segment SGI391808[] = {{&S83421[8], 3}, {NULL}};
const Segment SGI391809[] = {{&S83421[13], 7}, {NULL}};
const Segment SGI391810[] = {{&S83421[25], 5}, {NULL}};
const Segment SGI391811[] = {{&S83421[57], 12}, {NULL}};
const Segment SGI391812[] = {{&S42939[0], 6}, {NULL}};
const Segment SGI391813[] = {{&S42939[60], 10}, {NULL}};
const Segment SGI391814[] = {{&S42939[43], 4}, {NULL}};
const Segment SGI391815[] = {{&S42940[0], 6}, {NULL}};
const Segment SGI391816[] = {{&S42940[60], 4}, {NULL}};
const Segment SGI391817[] = {{&S42940[43], 4}, {NULL}};
const Segment SGI391818[] = {{&S42941[0], 6}, {NULL}};
const Segment SGI391819[] = {{&S42941[60], 7}, {NULL}};
const Segment SGI391820[] = {{&S42941[43], 4}, {NULL}};
const Segment SGI391821[] = {{&S42942[0], 6}, {NULL}};
const Segment SGI391822[] = {{&S42942[60], 7}, {NULL}};
const Segment SGI391823[] = {{&S42942[43], 4}, {NULL}};
const Segment SGI391824[] = {{&S42943[0], 6}, {NULL}};
const Segment SGI391825[] = {{&S42943[60], 10}, {NULL}};
const Segment SGI391826[] = {{&S42943[43], 4}, {NULL}};
const Segment SGI391827[] = {{&S42944[0], 6}, {NULL}};
const Segment SGI391828[] = {{&S42944[60], 6}, {NULL}};
const Segment SGI391829[] = {{&S42944[43], 4}, {NULL}};
const Segment SGI391830[] = {{&S83600[8], 3}, {NULL}};
const Segment SGI391831[] = {{&S83600[13], 14}, {NULL}};
const Segment SGI391832[] = {{&S83600[47], 11}, {NULL}};
const Segment SGI391833[] = {{&S83937[8], 3}, {NULL}};
const Segment SGI391834[] = {{&S83937[13], 9}, {NULL}};
const Segment SGI391835[] = {{&S83937[33], 13}, {NULL}};
const Segment SGI391836[] = {{&S83937[85], 12}, {NULL}};
const Segment SGI391837[] = {{&S43397[0], 6}, {NULL}};
const Segment SGI391838[] = {{&S43397[60], 7}, {NULL}};
const Segment SGI391839[] = {{&S43397[43], 4}, {NULL}};
const Segment SGI391840[] = {{&S43398[0], 6}, {NULL}};
const Segment SGI391841[] = {{&S43398[59], 10}, {NULL}};
const Segment SGI391842[] = {{&S43398[42], 4}, {NULL}};
const Segment SGI391843[] = {{&S43399[0], 6}, {NULL}};
const Segment SGI391844[] = {{&S43399[59], 10}, {NULL}};
const Segment SGI391845[] = {{&S43399[42], 4}, {NULL}};
const Segment SGI391846[] = {{&S43400[0], 6}, {NULL}};
const Segment SGI391847[] = {{&S43400[59], 4}, {NULL}};
const Segment SGI391848[] = {{&S43400[42], 4}, {NULL}};
const Segment SGI391849[] = {{&S43401[0], 6}, {NULL}};
const Segment SGI391850[] = {{&S43401[57], 6}, {NULL}};
const Segment SGI391851[] = {{&S43401[42], 4}, {NULL}};
const Segment SGI391852[] = {{&S43402[0], 6}, {NULL}};
const Segment SGI391853[] = {{&S43402[57], 7}, {NULL}};
const Segment SGI391854[] = {{&S43402[42], 4}, {NULL}};
const Segment SGI391855[] = {{&S43446[0], 6}, {NULL}};
const Segment SGI391856[] = {{&S43446[60], 4}, {NULL}};
const Segment SGI391857[] = {{&S43446[43], 4}, {NULL}};
const Segment SGI391858[] = {{&S43447[0], 6}, {NULL}};
const Segment SGI391859[] = {{&S43447[60], 10}, {NULL}};
const Segment SGI391860[] = {{&S43447[43], 4}, {NULL}};
const Segment SGI391861[] = {{&S43448[0], 6}, {NULL}};
const Segment SGI391862[] = {{&S43448[60], 6}, {NULL}};
const Segment SGI391863[] = {{&S43448[43], 4}, {NULL}};
const Segment SGI391864[] = {{&S43449[0], 6}, {NULL}};
const Segment SGI391865[] = {{&S43449[60], 10}, {NULL}};
const Segment SGI391866[] = {{&S43449[43], 4}, {NULL}};
const Segment SGI391867[] = {{&S43450[0], 6}, {NULL}};
const Segment SGI391868[] = {{&S43450[60], 7}, {NULL}};
const Segment SGI391869[] = {{&S43450[43], 4}, {NULL}};
const Segment SGI391870[] = {{&S43451[0], 6}, {NULL}};
const Segment SGI391871[] = {{&S43451[60], 7}, {NULL}};
const Segment SGI391872[] = {{&S43451[43], 4}, {NULL}};
const Segment SGI391873[] = {{&S74817[94], 3}, {NULL}};
const Segment SGI391874[] = {{&S74817[29], 7}, {NULL}};
const Segment SGI391875[] = {{&S74817[129], 5}, {NULL}};
const Segment SGI391876[] = {{&S43562[0], 12}, {NULL}};
const Segment SGI391877[] = {{&S43562[46], 4}, {NULL}};
const Segment SGI391878[] = {{&S43711[0], 40}, {NULL}};
const Segment SGI391879[] = {{&S43711[73], 10}, {NULL}};
const Segment SGI391880[] = {{&S43711[56], 4}, {NULL}};
const Segment SGI391881[] = {{&S43712[0], 10}, {NULL}};
const Segment SGI391882[] = {{&S43712[64], 10}, {NULL}};
const Segment SGI391883[] = {{&S43712[47], 4}, {NULL}};
const Segment SGI391884[] = {{&S43713[0], 40}, {NULL}};
const Segment SGI391885[] = {{&S43713[73], 10}, {NULL}};
const Segment SGI391886[] = {{&S43713[56], 4}, {NULL}};
const Segment SGI391887[] = {{&S43714[0], 16}, {NULL}};
const Segment SGI391888[] = {{&S43714[69], 10}, {NULL}};
const Segment SGI391889[] = {{&S43714[52], 4}, {NULL}};
const Segment SGI391890[] = {{&S43715[0], 40}, {NULL}};
const Segment SGI391891[] = {{&S43715[73], 4}, {NULL}};
const Segment SGI391892[] = {{&S43715[56], 4}, {NULL}};
const Segment SGI391893[] = {{&S43716[0], 16}, {NULL}};
const Segment SGI391894[] = {{&S43716[69], 10}, {NULL}};
const Segment SGI391895[] = {{&S43716[52], 4}, {NULL}};
const Segment SGI391896[] = {{&S43717[0], 16}, {NULL}};
const Segment SGI391897[] = {{&S43717[69], 4}, {NULL}};
const Segment SGI391898[] = {{&S43717[52], 4}, {NULL}};
const Segment SGI391899[] = {{&S43718[0], 40}, {NULL}};
const Segment SGI391900[] = {{&S43718[73], 7}, {NULL}};
const Segment SGI391901[] = {{&S43718[56], 4}, {NULL}};
const Segment SGI391902[] = {{&S43719[0], 9}, {NULL}};
const Segment SGI391903[] = {{&S43719[57], 10}, {NULL}};
const Segment SGI391904[] = {{&S43719[40], 4}, {NULL}};
const Segment SGI391905[] = {{&S43721[0], 10}, {NULL}};
const Segment SGI391906[] = {{&S43721[64], 4}, {NULL}};
const Segment SGI391907[] = {{&S43721[47], 4}, {NULL}};
const Segment SGI391908[] = {{&S43722[0], 7}, {NULL}};
const Segment SGI391909[] = {{&S43722[54], 10}, {NULL}};
const Segment SGI391910[] = {{&S43722[37], 4}, {NULL}};
const Segment SGI391911[] = {{&S43723[0], 43}, {NULL}};
const Segment SGI391912[] = {{&S43723[76], 10}, {NULL}};
const Segment SGI391913[] = {{&S43723[59], 4}, {NULL}};
const Segment SGI391914[] = {{&S43724[0], 24}, {NULL}};
const Segment SGI391915[] = {{&S43724[72], 10}, {NULL}};
const Segment SGI391916[] = {{&S43724[55], 4}, {NULL}};
const Segment SGI391917[] = {{&S43725[0], 10}, {NULL}};
const Segment SGI391918[] = {{&S43725[64], 10}, {NULL}};
const Segment SGI391919[] = {{&S43725[47], 4}, {NULL}};
const Segment SGI391920[] = {{&S43726[0], 17}, {NULL}};
const Segment SGI391921[] = {{&S43726[62], 10}, {NULL}};
const Segment SGI391922[] = {{&S43726[45], 4}, {NULL}};
const Segment SGI391923[] = {{&S43727[0], 14}, {NULL}};
const Segment SGI391924[] = {{&S43727[63], 10}, {NULL}};
const Segment SGI391925[] = {{&S43727[46], 4}, {NULL}};
const Segment SGI391926[] = {{&S43728[0], 14}, {NULL}};
const Segment SGI391927[] = {{&S43728[63], 4}, {NULL}};
const Segment SGI391928[] = {{&S43728[46], 4}, {NULL}};
const Segment SGI391929[] = {{&S75003[108], 3}, {NULL}};
const Segment SGI391930[] = {{&S75003[44], 6}, {NULL}};
const Segment SGI391931[] = {{&S75003[30], 6}, {NULL}};
const Segment SGI391932[] = {{&S75003[150], 5}, {NULL}};
const Segment SGI391933[] = {{&S79761[48], 4}, {NULL}};
const Segment SGI391934[] = {{&S79761[8], 3}, {NULL}};
const Segment SGI391935[] = {{&S79761[13], 9}, {NULL}};
const Segment SGI391936[] = {{&S79761[30], 13}, {NULL}};
const Segment SGI391937[] = {{&S84276[8], 3}, {NULL}};
const Segment SGI391938[] = {{&S84276[13], 7}, {NULL}};
const Segment SGI391939[] = {{&S84276[73], 13}, {NULL}};
const Segment SGI391940[] = {{&S84277[8], 3}, {NULL}};
const Segment SGI391941[] = {{&S84277[13], 7}, {NULL}};
const Segment SGI391942[] = {{&S84277[25], 14}, {NULL}};
const Segment SGI391943[] = {{&S84277[69], 13}, {NULL}};
const Segment SGI391944[] = {{&S75235[94], 3}, {NULL}};
const Segment SGI391945[] = {{&S75235[29], 7}, {NULL}};
const Segment SGI391946[] = {{&S75235[129], 5}, {NULL}};
const Segment SGI391947[] = {{&S75585[62], 3}, {NULL}};
const Segment SGI391948[] = {{&S75585[164], 4}, {NULL}};
const Segment SGI391949[] = {{&S75585[73], 8}, {NULL}};
const Segment SGI391950[] = {{&S84713[8], 3}, {NULL}};
const Segment SGI391951[] = {{&S84713[13], 3}, {NULL}};
const Segment SGI391952[] = {{&S84713[60], 13}, {NULL}};
const Segment SGI391953[] = {{&S84714[8], 3}, {NULL}};
const Segment SGI391954[] = {{&S84714[13], 3}, {NULL}};
const Segment SGI391955[] = {{&S84714[13], 3}, {&S84714[18], 10}, {NULL}};
const Segment SGI391956[] = {{&S84714[60], 13}, {NULL}};
const Segment SGI391957[] = {{&S80469[32], 4}, {NULL}};
const Segment SGI391958[] = {{&S80469[8], 3}, {NULL}};
const Segment SGI391959[] = {{&S80469[13], 14}, {NULL}};
const Segment SGI391960[] = {{&S80470[39], 4}, {NULL}};
const Segment SGI391961[] = {{&S80470[8], 3}, {NULL}};
const Segment SGI391962[] = {{&S80470[13], 14}, {NULL}};
const Segment SGI391963[] = {{&S84888[8], 3}, {NULL}};
const Segment SGI391964[] = {{&S84888[13], 7}, {NULL}};
const Segment SGI391965[] = {{&S84888[25], 14}, {NULL}};
const Segment SGI391966[] = {{&S84888[71], 13}, {NULL}};
const Segment SGI391967[] = {{&S76159[108], 3}, {NULL}};
const Segment SGI391968[] = {{&S76159[44], 6}, {NULL}};
const Segment SGI391969[] = {{&S76159[30], 6}, {NULL}};
const Segment SGI391970[] = {{&S76159[150], 5}, {NULL}};
const Segment SGI391971[] = {{&S76305[108], 3}, {NULL}};
const Segment SGI391972[] = {{&S76305[44], 6}, {NULL}};
const Segment SGI391973[] = {{&S76305[30], 6}, {NULL}};
const Segment SGI391974[] = {{&S76305[150], 5}, {NULL}};
const Segment SGI391975[] = {{&S85373[8], 3}, {NULL}};
const Segment SGI391976[] = {{&S85373[13], 7}, {NULL}};
const Segment SGI391977[] = {{&S85373[25], 14}, {NULL}};
const Segment SGI391978[] = {{&S85373[69], 13}, {NULL}};
const Segment SGI391979[] = {{&S80941[32], 4}, {NULL}};
const Segment SGI391980[] = {{&S80941[8], 3}, {NULL}};
const Segment SGI391981[] = {{&S80941[13], 14}, {NULL}};
const Segment SGI391982[] = {{&S85459[8], 3}, {NULL}};
const Segment SGI391983[] = {{&S85459[13], 7}, {NULL}};
const Segment SGI391984[] = {{&S85459[25], 14}, {NULL}};
const Segment SGI391985[] = {{&S85459[71], 13}, {NULL}};
const Segment SGI391986[] = {{&S89320[8], 3}, {NULL}};
const Segment SGI391987[] = {{&S89320[13], 7}, {NULL}};
const Segment SGI391988[] = {{&S89320[25], 14}, {NULL}};
const Segment SGI391989[] = {{&S89320[95], 11}, {NULL}};
const Segment SGI391990[] = {{&S85793[8], 3}, {NULL}};
const Segment SGI391991[] = {{&S85793[13], 14}, {NULL}};
const Segment SGI391992[] = {{&S85793[47], 13}, {NULL}};
const Segment SGI391993[] = {{&S85819[8], 3}, {NULL}};
const Segment SGI391994[] = {{&S85819[13], 7}, {NULL}};
const Segment SGI391995[] = {{&S85819[25], 14}, {NULL}};
const Segment SGI391996[] = {{&S85819[71], 13}, {NULL}};
const Segment SGI391997[] = {{&S85913[8], 3}, {NULL}};
const Segment SGI391998[] = {{&S85913[13], 7}, {NULL}};
const Segment SGI391999[] = {{&S85913[25], 14}, {NULL}};
const Segment SGI392000[] = {{&S85913[78], 12}, {NULL}};
const Segment SGI392001[] = {{&S85943[8], 3}, {NULL}};
const Segment SGI392002[] = {{&S85943[13], 7}, {NULL}};
const Segment SGI392003[] = {{&S85943[65], 13}, {NULL}};
const Segment SGI392004[] = {{&S45850[0], 7}, {NULL}};
const Segment SGI392005[] = {{&S45850[37], 4}, {NULL}};
const Segment SGI392006[] = {{&S86056[8], 3}, {NULL}};
const Segment SGI392007[] = {{&S86056[13], 7}, {NULL}};
const Segment SGI392008[] = {{&S86056[63], 13}, {NULL}};
const Segment SGI392009[] = {{&S81610[48], 4}, {NULL}};
const Segment SGI392010[] = {{&S81610[8], 3}, {NULL}};
const Segment SGI392011[] = {{&S81610[13], 9}, {NULL}};
const Segment SGI392012[] = {{&S81610[30], 13}, {NULL}};
const Segment SGI392013[] = {{&S86151[8], 3}, {NULL}};
const Segment SGI392014[] = {{&S86151[13], 7}, {NULL}};
const Segment SGI392015[] = {{&S86151[71], 13}, {NULL}};
const Segment SGI392016[] = {{&S77233[109], 3}, {NULL}};
const Segment SGI392017[] = {{&S77233[44], 7}, {NULL}};
const Segment SGI392018[] = {{&S77233[30], 6}, {NULL}};
const Segment SGI392019[] = {{&S77233[151], 5}, {NULL}};
const Segment SGI392020[] = {{&S81956[80], 3}, {NULL}};
const Segment SGI392021[] = {{&S81956[8], 3}, {NULL}};
const Segment SGI392022[] = {{&S81956[13], 7}, {NULL}};
const Segment SGI392023[] = {{&S81956[25], 14}, {NULL}};
const Segment SGI392024[] = {{&S86428[8], 3}, {NULL}};
const Segment SGI392025[] = {{&S86428[13], 3}, {NULL}};
const Segment SGI392026[] = {{&S86428[13], 3}, {&S86428[21], 10}, {&S86428[63], 6}, {NULL}};
const Segment SGI392027[] = {{&S86428[63], 11}, {NULL}};
const Segment SGI392028[] = {{&S77443[94], 3}, {NULL}};
const Segment SGI392029[] = {{&S77443[29], 7}, {NULL}};
const Segment SGI392030[] = {{&S77443[129], 5}, {NULL}};
const Segment SGI392031[] = {{&S86724[8], 3}, {NULL}};
const Segment SGI392032[] = {{&S86724[13], 7}, {NULL}};
const Segment SGI392033[] = {{&S86724[25], 14}, {NULL}};
const Segment SGI392034[] = {{&S86724[66], 13}, {NULL}};
const Segment SGI392035[] = {{&S86787[8], 3}, {NULL}};
const Segment SGI392036[] = {{&S86787[13], 14}, {NULL}};
const Segment SGI392037[] = {{&S86787[50], 13}, {NULL}};
const Segment SGI392038[] = {{&S86788[8], 3}, {NULL}};
const Segment SGI392039[] = {{&S86788[13], 7}, {NULL}};
const Segment SGI392040[] = {{&S86788[22], 14}, {NULL}};
const Segment SGI392041[] = {{&S86788[66], 13}, {NULL}};
const Segment SGI392042[] = {{&S86809[8], 3}, {NULL}};
const Segment SGI392043[] = {{&S86809[13], 7}, {NULL}};
const Segment SGI392044[] = {{&S86809[25], 14}, {NULL}};
const Segment SGI392045[] = {{&S86809[71], 13}, {NULL}};
const Segment SGI392046[] = {{&S82458[68], 3}, {NULL}};
const Segment SGI392047[] = {{&S82458[8], 3}, {NULL}};
const Segment SGI392048[] = {{&S82458[13], 3}, {NULL}};
const Segment SGI392049[] = {{&S77964[93], 3}, {NULL}};
const Segment SGI392050[] = {{&S77964[29], 6}, {NULL}};
const Segment SGI392051[] = {{&S77964[128], 5}, {NULL}};
const Segment SGI392052[] = {{&S86917[8], 3}, {NULL}};
const Segment SGI392053[] = {{&S86917[13], 14}, {NULL}};
const Segment SGI392054[] = {{&S86917[47], 13}, {NULL}};
const Segment SGI392055[] = {{&S87049[8], 3}, {NULL}};
const Segment SGI392056[] = {{&S87049[13], 14}, {NULL}};
const Segment SGI392057[] = {{&S87049[47], 13}, {NULL}};
const Segment SGI392058[] = {{&S87119[8], 3}, {NULL}};
const Segment SGI392059[] = {{&S87119[13], 7}, {NULL}};
const Segment SGI392060[] = {{&S87119[25], 14}, {NULL}};
const Segment SGI392061[] = {{&S87119[65], 9}, {NULL}};
const Segment SGI392062[] = {{&S82746[48], 4}, {NULL}};
const Segment SGI392063[] = {{&S82746[8], 3}, {NULL}};
const Segment SGI392064[] = {{&S82746[13], 9}, {NULL}};
const Segment SGI392065[] = {{&S82746[30], 13}, {NULL}};
const Segment SGI392066[] = {{&S82825[73], 3}, {NULL}};
const Segment SGI392067[] = {{&S82825[8], 3}, {NULL}};
const Segment SGI392068[] = {{&S82825[13], 3}, {NULL}};
const Segment SGI392069[] = {{&S78345[108], 3}, {NULL}};
const Segment SGI392070[] = {{&S78345[44], 6}, {NULL}};
const Segment SGI392071[] = {{&S78345[30], 6}, {NULL}};
const Segment SGI392072[] = {{&S78345[150], 5}, {NULL}};
const Segment SGI392073[] = {{&S87433[8], 3}, {NULL}};
const Segment SGI392074[] = {{&S87433[13], 7}, {NULL}};
const Segment SGI392075[] = {{&S87433[25], 14}, {NULL}};
const Segment SGI392076[] = {{&S87433[64], 12}, {NULL}};
const Segment SGI392077[] = {{&S87468[8], 3}, {NULL}};
const Segment SGI392078[] = {{&S87468[13], 7}, {NULL}};
const Segment SGI392079[] = {{&S87468[25], 5}, {NULL}};
const Segment SGI392080[] = {{&S87468[57], 13}, {NULL}};
const Segment SGI392081[] = {{&S47260[0], 11}, {NULL}};
const Segment SGI392082[] = {{&S47260[45], 4}, {NULL}};
const Segment SGI392083[] = {{&S47261[0], 13}, {NULL}};
const Segment SGI392084[] = {{&S47261[46], 4}, {NULL}};
const Segment SGI392085[] = {{&S47262[0], 10}, {NULL}};
const Segment SGI392086[] = {{&S47262[41], 4}, {NULL}};
const Segment SGI392087[] = {{&S47263[0], 13}, {NULL}};
const Segment SGI392088[] = {{&S47263[47], 4}, {NULL}};
const Segment SGI392089[] = {{&S47264[0], 14}, {NULL}};
const Segment SGI392090[] = {{&S47264[48], 4}, {NULL}};
const Segment SGI392091[] = {{&S47265[0], 15}, {NULL}};
const Segment SGI392092[] = {{&S47265[48], 4}, {NULL}};
const Segment SGI392093[] = {{&S47266[0], 5}, {NULL}};
const Segment SGI392094[] = {{&S47266[38], 4}, {NULL}};
const Segment SGI392095[] = {{&S47267[0], 18}, {NULL}};
const Segment SGI392096[] = {{&S47267[52], 4}, {NULL}};
const Segment SGI392097[] = {{&S47268[0], 7}, {NULL}};
const Segment SGI392098[] = {{&S47268[41], 4}, {NULL}};
const Segment SGI392099[] = {{&S47269[0], 17}, {NULL}};
const Segment SGI392100[] = {{&S47269[52], 4}, {NULL}};
const Segment SGI392101[] = {{&S47270[0], 17}, {NULL}};
const Segment SGI392102[] = {{&S47270[48], 4}, {NULL}};
const Segment SGI392103[] = {{&S47271[0], 13}, {NULL}};
const Segment SGI392104[] = {{&S47271[46], 4}, {NULL}};
const Segment SGI392105[] = {{&S47272[0], 9}, {NULL}};
const Segment SGI392106[] = {{&S47272[43], 4}, {NULL}};
const Segment SGI392107[] = {{&S47273[0], 6}, {NULL}};
const Segment SGI392108[] = {{&S47273[45], 4}, {NULL}};
const Segment SGI392109[] = {{&S47274[0], 18}, {NULL}};
const Segment SGI392110[] = {{&S47274[51], 4}, {NULL}};
const Segment SGI392111[] = {{&S47275[0], 13}, {NULL}};
const Segment SGI392112[] = {{&S47275[44], 4}, {NULL}};
const Segment SGI392113[] = {{&S47276[0], 10}, {NULL}};
const Segment SGI392114[] = {{&S47276[44], 4}, {NULL}};
const Segment SGI392115[] = {{&S47277[0], 14}, {NULL}};
const Segment SGI392116[] = {{&S47277[47], 4}, {NULL}};
const Segment SGI392117[] = {{&S47278[0], 17}, {NULL}};
const Segment SGI392118[] = {{&S47278[48], 4}, {NULL}};
const Segment SGI392119[] = {{&S47279[0], 12}, {NULL}};
const Segment SGI392120[] = {{&S47279[45], 4}, {NULL}};
const Segment SGI392121[] = {{&S47280[0], 7}, {NULL}};
const Segment SGI392122[] = {{&S47280[38], 4}, {NULL}};
const Segment SGI392123[] = {{&S47281[0], 16}, {NULL}};
const Segment SGI392124[] = {{&S47281[50], 4}, {NULL}};
const Segment SGI392125[] = {{&S47282[0], 17}, {NULL}};
const Segment SGI392126[] = {{&S47282[48], 4}, {NULL}};
const Segment SGI392127[] = {{&S47283[0], 8}, {NULL}};
const Segment SGI392128[] = {{&S47283[41], 4}, {NULL}};
const Segment SGI392129[] = {{&S47284[0], 15}, {NULL}};
const Segment SGI392130[] = {{&S47284[46], 4}, {NULL}};
const Segment SGI392131[] = {{&S47285[0], 12}, {NULL}};
const Segment SGI392132[] = {{&S47285[44], 4}, {NULL}};
const Segment SGI392133[] = {{&S47286[0], 11}, {NULL}};
const Segment SGI392134[] = {{&S47286[42], 4}, {NULL}};
const Segment SGI392135[] = {{&S47287[0], 8}, {NULL}};
const Segment SGI392136[] = {{&S47287[41], 4}, {NULL}};
const Segment SGI392137[] = {{&S47288[0], 6}, {NULL}};
const Segment SGI392138[] = {{&S47288[37], 4}, {NULL}};
const Segment SGI392139[] = {{&S47289[0], 7}, {NULL}};
const Segment SGI392140[] = {{&S47289[40], 4}, {NULL}};
const Segment SGI392141[] = {{&S47290[0], 7}, {NULL}};
const Segment SGI392142[] = {{&S47290[40], 4}, {NULL}};
const Segment SGI392143[] = {{&S47291[0], 8}, {NULL}};
const Segment SGI392144[] = {{&S47291[42], 4}, {NULL}};
const Segment SGI392145[] = {{&S47292[0], 8}, {NULL}};
const Segment SGI392146[] = {{&S47292[39], 4}, {NULL}};
const Segment SGI392147[] = {{&S47294[0], 14}, {NULL}};
const Segment SGI392148[] = {{&S47294[45], 4}, {NULL}};
const Segment SGI392149[] = {{&S47295[0], 10}, {NULL}};
const Segment SGI392150[] = {{&S47295[41], 4}, {NULL}};
const Segment SGI392151[] = {{&S47296[0], 13}, {NULL}};
const Segment SGI392152[] = {{&S47296[48], 4}, {NULL}};
const Segment SGI392153[] = {{&S47297[0], 9}, {NULL}};
const Segment SGI392154[] = {{&S47297[42], 4}, {NULL}};
const Segment SGI392155[] = {{&S47298[0], 17}, {NULL}};
const Segment SGI392156[] = {{&S47298[50], 4}, {NULL}};
const Segment SGI392157[] = {{&S47348[0], 6}, {NULL}};
const Segment SGI392158[] = {{&S47348[37], 4}, {NULL}};
const Segment SGI392159[] = {{&S47349[0], 13}, {NULL}};
const Segment SGI392160[] = {{&S47349[44], 4}, {NULL}};
const Segment SGI392161[] = {{&S47507[0], 18}, {NULL}};
const Segment SGI392162[] = {{&S47507[46], 4}, {NULL}};
const Segment SGI392163[] = {{&S47508[0], 30}, {NULL}};
const Segment SGI392164[] = {{&S47508[58], 4}, {NULL}};
const Segment SGI392165[] = {{&S47509[0], 14}, {NULL}};
const Segment SGI392166[] = {{&S47509[46], 4}, {NULL}};
const Segment SGI392167[] = {{&S47510[0], 22}, {NULL}};
const Segment SGI392168[] = {{&S47510[52], 4}, {NULL}};
const Segment SGI392169[] = {{&S47511[0], 20}, {NULL}};
const Segment SGI392170[] = {{&S47511[52], 4}, {NULL}};
const Segment SGI392171[] = {{&S47512[0], 17}, {NULL}};
const Segment SGI392172[] = {{&S47512[47], 4}, {NULL}};
const Segment SGI392173[] = {{&S47513[0], 11}, {NULL}};
const Segment SGI392174[] = {{&S47513[43], 4}, {NULL}};
const Segment SGI392175[] = {{&S47514[0], 19}, {NULL}};
const Segment SGI392176[] = {{&S47514[49], 4}, {NULL}};
const Segment SGI392177[] = {{&S91623[8], 3}, {NULL}};
const Segment SGI392178[] = {{&S91623[13], 7}, {NULL}};
const Segment SGI392179[] = {{&S91623[25], 14}, {NULL}};
const Segment SGI392180[] = {{&S91623[86], 51}, {NULL}};
const Segment SGI392181[] = {{&S83316[39], 4}, {NULL}};
const Segment SGI392182[] = {{&S83316[8], 3}, {NULL}};
const Segment SGI392183[] = {{&S83316[13], 14}, {NULL}};
const Segment SGI392184[] = {{&S83394[33], 3}, {NULL}};
const Segment SGI392185[] = {{&S83394[8], 3}, {NULL}};
const Segment SGI392186[] = {{&S83394[13], 3}, {NULL}};
const Segment SGI392187[] = {{&S47873[0], 6}, {NULL}};
const Segment SGI392188[] = {{&S47873[60], 10}, {NULL}};
const Segment SGI392189[] = {{&S47873[43], 4}, {NULL}};
const Segment SGI392190[] = {{&S47874[0], 6}, {NULL}};
const Segment SGI392191[] = {{&S47874[60], 6}, {NULL}};
const Segment SGI392192[] = {{&S47874[43], 4}, {NULL}};
const Segment SGI392193[] = {{&S47875[0], 6}, {NULL}};
const Segment SGI392194[] = {{&S47875[60], 4}, {NULL}};
const Segment SGI392195[] = {{&S47875[43], 4}, {NULL}};
const Segment SGI392196[] = {{&S47876[0], 6}, {NULL}};
const Segment SGI392197[] = {{&S47876[60], 10}, {NULL}};
const Segment SGI392198[] = {{&S47876[43], 4}, {NULL}};
const Segment SGI392199[] = {{&S47877[0], 6}, {NULL}};
const Segment SGI392200[] = {{&S47877[60], 7}, {NULL}};
const Segment SGI392201[] = {{&S47877[43], 4}, {NULL}};
const Segment SGI392202[] = {{&S47878[0], 6}, {NULL}};
const Segment SGI392203[] = {{&S47878[60], 7}, {NULL}};
const Segment SGI392204[] = {{&S47878[43], 4}, {NULL}};
const Segment SGI392205[] = {{&S91905[8], 3}, {NULL}};
const Segment SGI392206[] = {{&S91905[13], 7}, {NULL}};
const Segment SGI392207[] = {{&S91905[25], 14}, {NULL}};
const Segment SGI392208[] = {{&S91905[85], 49}, {NULL}};
const Segment SGI392209[] = {{&S79095[109], 3}, {NULL}};
const Segment SGI392210[] = {{&S79095[44], 7}, {NULL}};
const Segment SGI392211[] = {{&S79095[30], 6}, {NULL}};
const Segment SGI392212[] = {{&S79095[151], 4}, {NULL}};
const Segment SGI392213[] = {{&S79148[108], 3}, {NULL}};
const Segment SGI392214[] = {{&S79148[44], 6}, {NULL}};
const Segment SGI392215[] = {{&S79148[30], 6}, {NULL}};
const Segment SGI392216[] = {{&S79148[150], 5}, {NULL}};
const Segment SGI392217[] = {{&S79545[93], 3}, {NULL}};
const Segment SGI392218[] = {{&S79545[29], 6}, {NULL}};
const Segment SGI392219[] = {{&S79545[128], 5}, {NULL}};
const Segment SGI392220[] = {{&S84102[39], 4}, {NULL}};
const Segment SGI392221[] = {{&S84102[8], 3}, {NULL}};
const Segment SGI392222[] = {{&S84102[13], 14}, {NULL}};
const Segment SGI392223[] = {{&S79594[108], 3}, {NULL}};
const Segment SGI392224[] = {{&S79594[44], 6}, {NULL}};
const Segment SGI392225[] = {{&S79594[30], 6}, {NULL}};
const Segment SGI392226[] = {{&S79594[150], 4}, {NULL}};
const Segment SGI392227[] = {{&S88526[8], 3}, {NULL}};
const Segment SGI392228[] = {{&S88526[13], 3}, {NULL}};
const Segment SGI392229[] = {{&S88526[49], 14}, {NULL}};
const Segment SGI392230[] = {{&S79717[94], 3}, {NULL}};
const Segment SGI392231[] = {{&S79717[29], 7}, {NULL}};
const Segment SGI392232[] = {{&S79717[129], 5}, {NULL}};
const Segment SGI392233[] = {{&S88612[8], 3}, {NULL}};
const Segment SGI392234[] = {{&S88612[13], 9}, {NULL}};
const Segment SGI392235[] = {{&S88612[33], 13}, {NULL}};
const Segment SGI392236[] = {{&S88612[85], 12}, {NULL}};
const Segment SGI392237[] = {{&S79807[109], 3}, {NULL}};
const Segment SGI392238[] = {{&S79807[44], 7}, {NULL}};
const Segment SGI392239[] = {{&S79807[30], 6}, {NULL}};
const Segment SGI392240[] = {{&S79807[151], 4}, {NULL}};
const Segment SGI392241[] = {{&S79946[109], 3}, {NULL}};
const Segment SGI392242[] = {{&S79946[44], 7}, {NULL}};
const Segment SGI392243[] = {{&S79946[30], 6}, {NULL}};
const Segment SGI392244[] = {{&S79946[151], 4}, {NULL}};
const Segment SGI392245[] = {{&S48826[0], 9}, {NULL}};
const Segment SGI392246[] = {{&S48826[41], 4}, {NULL}};
const Segment SGI392247[] = {{&S84381[8], 3}, {NULL}};
const Segment SGI392248[] = {{&S84381[13], 14}, {NULL}};
const Segment SGI392249[] = {{&S49033[0], 15}, {NULL}};
const Segment SGI392250[] = {{&S49033[60], 4}, {NULL}};
const Segment SGI392251[] = {{&S49033[43], 4}, {NULL}};
const Segment SGI392252[] = {{&S49034[0], 15}, {NULL}};
const Segment SGI392253[] = {{&S49034[60], 10}, {NULL}};
const Segment SGI392254[] = {{&S49034[43], 4}, {NULL}};
const Segment SGI392255[] = {{&S49035[0], 15}, {NULL}};
const Segment SGI392256[] = {{&S49035[60], 6}, {NULL}};
const Segment SGI392257[] = {{&S49035[43], 4}, {NULL}};
const Segment SGI392258[] = {{&S49036[0], 15}, {NULL}};
const Segment SGI392259[] = {{&S49036[60], 7}, {NULL}};
const Segment SGI392260[] = {{&S49036[43], 4}, {NULL}};
const Segment SGI392261[] = {{&S49037[0], 15}, {NULL}};
const Segment SGI392262[] = {{&S49037[60], 10}, {NULL}};
const Segment SGI392263[] = {{&S49037[43], 4}, {NULL}};
const Segment SGI392264[] = {{&S49038[0], 15}, {NULL}};
const Segment SGI392265[] = {{&S49038[60], 7}, {NULL}};
const Segment SGI392266[] = {{&S49038[43], 4}, {NULL}};
const Segment SGI392267[] = {{&S84596[48], 4}, {NULL}};
const Segment SGI392268[] = {{&S84596[8], 3}, {NULL}};
const Segment SGI392269[] = {{&S84596[13], 9}, {NULL}};
const Segment SGI392270[] = {{&S84596[30], 13}, {NULL}};
const Segment SGI392271[] = {{&S89119[8], 3}, {NULL}};
const Segment SGI392272[] = {{&S89119[13], 7}, {NULL}};
const Segment SGI392273[] = {{&S89119[25], 14}, {NULL}};
const Segment SGI392274[] = {{&S89119[66], 12}, {NULL}};
const Segment SGI392275[] = {{&S89120[8], 3}, {NULL}};
const Segment SGI392276[] = {{&S89120[13], 7}, {NULL}};
const Segment SGI392277[] = {{&S89120[25], 14}, {NULL}};
const Segment SGI392278[] = {{&S89120[66], 12}, {NULL}};
const Segment SGI392279[] = {{&S84784[48], 4}, {NULL}};
const Segment SGI392280[] = {{&S84784[8], 3}, {NULL}};
const Segment SGI392281[] = {{&S84784[13], 9}, {NULL}};
const Segment SGI392282[] = {{&S84784[30], 13}, {NULL}};
const Segment SGI392283[] = {{&S84859[48], 3}, {NULL}};
const Segment SGI392284[] = {{&S84859[8], 3}, {NULL}};
const Segment SGI392285[] = {{&S84859[13], 9}, {NULL}};
const Segment SGI392286[] = {{&S84859[30], 13}, {NULL}};
const Segment SGI392287[] = {{&S89416[8], 3}, {NULL}};
const Segment SGI392288[] = {{&S89416[13], 14}, {NULL}};
const Segment SGI392289[] = {{&S89579[8], 3}, {NULL}};
const Segment SGI392290[] = {{&S89579[13], 3}, {NULL}};
const Segment SGI392291[] = {{&S89579[13], 3}, {&S89579[21], 10}, {&S89579[61], 6}, {NULL}};
const Segment SGI392292[] = {{&S89579[61], 11}, {NULL}};
const Segment SGI392293[] = {{&S85149[39], 4}, {NULL}};
const Segment SGI392294[] = {{&S85149[8], 3}, {NULL}};
const Segment SGI392295[] = {{&S85149[13], 14}, {NULL}};
const Segment SGI392296[] = {{&S85155[73], 3}, {NULL}};
const Segment SGI392297[] = {{&S85155[8], 3}, {NULL}};
const Segment SGI392298[] = {{&S85155[13], 3}, {NULL}};
const Segment SGI392299[] = {{&S85158[48], 4}, {NULL}};
const Segment SGI392300[] = {{&S85158[8], 3}, {NULL}};
const Segment SGI392301[] = {{&S85158[13], 9}, {NULL}};
const Segment SGI392302[] = {{&S85158[30], 13}, {NULL}};
const Segment SGI392303[] = {{&S85167[48], 4}, {NULL}};
const Segment SGI392304[] = {{&S85167[8], 3}, {NULL}};
const Segment SGI392305[] = {{&S85167[13], 9}, {NULL}};
const Segment SGI392306[] = {{&S85167[30], 13}, {NULL}};
const Segment SGI392307[] = {{&S80834[93], 3}, {NULL}};
const Segment SGI392308[] = {{&S80834[29], 6}, {NULL}};
const Segment SGI392309[] = {{&S80834[128], 5}, {NULL}};
const Segment SGI392310[] = {{&S89752[8], 3}, {NULL}};
const Segment SGI392311[] = {{&S89752[13], 3}, {NULL}};
const Segment SGI392312[] = {{&S89752[65], 13}, {NULL}};
const Segment SGI392313[] = {{&S89753[8], 3}, {NULL}};
const Segment SGI392314[] = {{&S89753[13], 3}, {NULL}};
const Segment SGI392315[] = {{&S89753[13], 3}, {&S89753[21], 10}, {NULL}};
const Segment SGI392316[] = {{&S89753[63], 13}, {NULL}};
const Segment SGI392317[] = {{&S80924[109], 3}, {NULL}};
const Segment SGI392318[] = {{&S80924[44], 7}, {NULL}};
const Segment SGI392319[] = {{&S80924[30], 6}, {NULL}};
const Segment SGI392320[] = {{&S80924[151], 4}, {NULL}};
const Segment SGI392321[] = {{&S80957[109], 3}, {NULL}};
const Segment SGI392322[] = {{&S80957[44], 7}, {NULL}};
const Segment SGI392323[] = {{&S80957[30], 6}, {NULL}};
const Segment SGI392324[] = {{&S80957[151], 4}, {NULL}};
const Segment SGI392325[] = {{&S89865[8], 3}, {NULL}};
const Segment SGI392326[] = {{&S89865[13], 14}, {NULL}};
const Segment SGI392327[] = {{&S89865[45], 13}, {NULL}};
const Segment SGI392328[] = {{&S81009[94], 3}, {NULL}};
const Segment SGI392329[] = {{&S81009[29], 7}, {NULL}};
const Segment SGI392330[] = {{&S81009[129], 5}, {NULL}};
const Segment SGI392331[] = {{&S85345[32], 3}, {NULL}};
const Segment SGI392332[] = {{&S85345[8], 3}, {NULL}};
const Segment SGI392333[] = {{&S85345[13], 14}, {NULL}};
const Segment SGI392334[] = {{&S85351[48], 4}, {NULL}};
const Segment SGI392335[] = {{&S85351[8], 3}, {NULL}};
const Segment SGI392336[] = {{&S85351[13], 9}, {NULL}};
const Segment SGI392337[] = {{&S85351[30], 13}, {NULL}};
const Segment SGI392338[] = {{&S89935[8], 3}, {NULL}};
const Segment SGI392339[] = {{&S89935[13], 7}, {NULL}};
const Segment SGI392340[] = {{&S89935[25], 14}, {NULL}};
const Segment SGI392341[] = {{&S89935[66], 12}, {NULL}};
const Segment SGI392342[] = {{&S90036[8], 3}, {NULL}};
const Segment SGI392343[] = {{&S90036[13], 7}, {NULL}};
const Segment SGI392344[] = {{&S90036[25], 14}, {NULL}};
const Segment SGI392345[] = {{&S90036[64], 12}, {NULL}};
const Segment SGI392346[] = {{&S85578[32], 4}, {NULL}};
const Segment SGI392347[] = {{&S85578[8], 3}, {NULL}};
const Segment SGI392348[] = {{&S85578[13], 14}, {NULL}};
const Segment SGI392349[] = {{&S85800[48], 4}, {NULL}};
const Segment SGI392350[] = {{&S85800[8], 3}, {NULL}};
const Segment SGI392351[] = {{&S85800[13], 9}, {NULL}};
const Segment SGI392352[] = {{&S85800[30], 13}, {NULL}};
const Segment SGI392353[] = {{&S86037[48], 4}, {NULL}};
const Segment SGI392354[] = {{&S86037[8], 3}, {NULL}};
const Segment SGI392355[] = {{&S86037[13], 9}, {NULL}};
const Segment SGI392356[] = {{&S86037[30], 13}, {NULL}};
const Segment SGI392357[] = {{&S86118[48], 4}, {NULL}};
const Segment SGI392358[] = {{&S86118[8], 3}, {NULL}};
const Segment SGI392359[] = {{&S86118[13], 9}, {NULL}};
const Segment SGI392360[] = {{&S86118[30], 13}, {NULL}};
const Segment SGI392361[] = {{&S86177[32], 4}, {NULL}};
const Segment SGI392362[] = {{&S86177[8], 3}, {NULL}};
const Segment SGI392363[] = {{&S86177[13], 14}, {NULL}};
const Segment SGI392364[] = {{&S90768[8], 3}, {NULL}};
const Segment SGI392365[] = {{&S90768[13], 3}, {NULL}};
const Segment SGI392366[] = {{&S90768[13], 3}, {&S90768[21], 10}, {NULL}};
const Segment SGI392367[] = {{&S90768[58], 13}, {NULL}};
const Segment SGI392368[] = {{&S81992[109], 3}, {NULL}};
const Segment SGI392369[] = {{&S81992[44], 7}, {NULL}};
const Segment SGI392370[] = {{&S81992[30], 6}, {NULL}};
const Segment SGI392371[] = {{&S81992[151], 4}, {NULL}};
const Segment SGI392372[] = {{&S86441[34], 3}, {NULL}};
const Segment SGI392373[] = {{&S86441[8], 3}, {NULL}};
const Segment SGI392374[] = {{&S86441[13], 3}, {NULL}};
const Segment SGI392375[] = {{&S90999[8], 3}, {NULL}};
const Segment SGI392376[] = {{&S90999[13], 3}, {NULL}};
const Segment SGI392377[] = {{&S90999[13], 3}, {&S90999[21], 10}, {NULL}};
const Segment SGI392378[] = {{&S90999[79], 12}, {NULL}};
const Segment SGI392379[] = {{&S50967[0], 18}, {NULL}};
const Segment SGI392380[] = {{&S50967[50], 4}, {NULL}};
const Segment SGI392381[] = {{&S50968[0], 14}, {NULL}};
const Segment SGI392382[] = {{&S50968[46], 4}, {NULL}};
const Segment SGI392383[] = {{&S50969[0], 11}, {NULL}};
const Segment SGI392384[] = {{&S50969[43], 4}, {NULL}};
const Segment SGI392385[] = {{&S50970[0], 19}, {NULL}};
const Segment SGI392386[] = {{&S50970[49], 4}, {NULL}};
const Segment SGI392387[] = {{&S50971[0], 22}, {NULL}};
const Segment SGI392388[] = {{&S50971[54], 4}, {NULL}};
const Segment SGI392389[] = {{&S50972[0], 30}, {NULL}};
const Segment SGI392390[] = {{&S50972[62], 4}, {NULL}};
const Segment SGI392391[] = {{&S50973[0], 16}, {NULL}};
const Segment SGI392392[] = {{&S50973[48], 4}, {NULL}};
const Segment SGI392393[] = {{&S50974[0], 17}, {NULL}};
const Segment SGI392394[] = {{&S50974[47], 4}, {NULL}};
const Segment SGI392395[] = {{&S50975[0], 24}, {NULL}};
const Segment SGI392396[] = {{&S50975[56], 4}, {NULL}};
const Segment SGI392397[] = {{&S86556[88], 3}, {NULL}};
const Segment SGI392398[] = {{&S86556[8], 3}, {NULL}};
const Segment SGI392399[] = {{&S86556[13], 9}, {NULL}};
const Segment SGI392400[] = {{&S86556[33], 13}, {NULL}};
const Segment SGI392401[] = {{&S86671[44], 3}, {NULL}};
const Segment SGI392402[] = {{&S86671[8], 3}, {NULL}};
const Segment SGI392403[] = {{&S86671[13], 14}, {NULL}};
const Segment SGI392404[] = {{&S91295[8], 3}, {NULL}};
const Segment SGI392405[] = {{&S91295[13], 3}, {NULL}};
const Segment SGI392406[] = {{&S91295[13], 3}, {&S91295[18], 10}, {NULL}};
const Segment SGI392407[] = {{&S91295[69], 6}, {NULL}};
const Segment SGI392408[] = {{&S91295[55], 13}, {NULL}};
const Segment SGI392409[] = {{&S91296[8], 3}, {NULL}};
const Segment SGI392410[] = {{&S91296[13], 3}, {NULL}};
const Segment SGI392411[] = {{&S91296[71], 6}, {NULL}};
const Segment SGI392412[] = {{&S91296[57], 13}, {NULL}};
const Segment SGI392413[] = {{&S91317[8], 3}, {NULL}};
const Segment SGI392414[] = {{&S91317[13], 3}, {NULL}};
const Segment SGI392415[] = {{&S91317[13], 3}, {&S91317[21], 10}, {NULL}};
const Segment SGI392416[] = {{&S91317[63], 13}, {NULL}};
const Segment SGI392417[] = {{&S86835[32], 4}, {NULL}};
const Segment SGI392418[] = {{&S86835[8], 3}, {NULL}};
const Segment SGI392419[] = {{&S86835[13], 14}, {NULL}};
const Segment SGI392420[] = {{&S91421[8], 3}, {NULL}};
const Segment SGI392421[] = {{&S91421[13], 17}, {NULL}};
const Segment SGI392422[] = {{&S91421[50], 5}, {NULL}};
const Segment SGI392423[] = {{&S82571[109], 3}, {NULL}};
const Segment SGI392424[] = {{&S82571[44], 7}, {NULL}};
const Segment SGI392425[] = {{&S82571[30], 6}, {NULL}};
const Segment SGI392426[] = {{&S82571[151], 4}, {NULL}};
const Segment SGI392427[] = {{&S51476[0], 8}, {NULL}};
const Segment SGI392428[] = {{&S51476[37], 4}, {NULL}};
const Segment SGI392429[] = {{&S51477[0], 6}, {NULL}};
const Segment SGI392430[] = {{&S51477[36], 4}, {NULL}};
const Segment SGI392431[] = {{&S91526[8], 3}, {NULL}};
const Segment SGI392432[] = {{&S91526[13], 3}, {NULL}};
const Segment SGI392433[] = {{&S91526[63], 11}, {NULL}};
const Segment SGI392434[] = {{&S91527[8], 3}, {NULL}};
const Segment SGI392435[] = {{&S91527[13], 3}, {NULL}};
const Segment SGI392436[] = {{&S91527[13], 3}, {&S91527[21], 10}, {NULL}};
const Segment SGI392437[] = {{&S91527[61], 11}, {NULL}};
const Segment SGI392438[] = {{&S91553[8], 3}, {NULL}};
const Segment SGI392439[] = {{&S91553[13], 7}, {NULL}};
const Segment SGI392440[] = {{&S91553[25], 14}, {NULL}};
const Segment SGI392441[] = {{&S91553[78], 12}, {NULL}};
const Segment SGI392442[] = {{&S51535[0], 15}, {NULL}};
const Segment SGI392443[] = {{&S51535[60], 10}, {NULL}};
const Segment SGI392444[] = {{&S51535[43], 4}, {NULL}};
const Segment SGI392445[] = {{&S51536[0], 15}, {NULL}};
const Segment SGI392446[] = {{&S51536[60], 10}, {NULL}};
const Segment SGI392447[] = {{&S51536[43], 4}, {NULL}};
const Segment SGI392448[] = {{&S51537[0], 15}, {NULL}};
const Segment SGI392449[] = {{&S51537[60], 4}, {NULL}};
const Segment SGI392450[] = {{&S51537[43], 4}, {NULL}};
const Segment SGI392451[] = {{&S51538[0], 15}, {NULL}};
const Segment SGI392452[] = {{&S51538[60], 7}, {NULL}};
const Segment SGI392453[] = {{&S51538[43], 4}, {NULL}};
const Segment SGI392454[] = {{&S51539[0], 15}, {NULL}};
const Segment SGI392455[] = {{&S51539[60], 6}, {NULL}};
const Segment SGI392456[] = {{&S51539[43], 4}, {NULL}};
const Segment SGI392457[] = {{&S51540[0], 15}, {NULL}};
const Segment SGI392458[] = {{&S51540[60], 7}, {NULL}};
const Segment SGI392459[] = {{&S51540[43], 4}, {NULL}};
const Segment SGI392460[] = {{&S82659[109], 3}, {NULL}};
const Segment SGI392461[] = {{&S82659[44], 7}, {NULL}};
const Segment SGI392462[] = {{&S82659[30], 6}, {NULL}};
const Segment SGI392463[] = {{&S82659[151], 5}, {NULL}};
const Segment SGI392464[] = {{&S87200[48], 4}, {NULL}};
const Segment SGI392465[] = {{&S87200[8], 3}, {NULL}};
const Segment SGI392466[] = {{&S87200[13], 9}, {NULL}};
const Segment SGI392467[] = {{&S87200[30], 13}, {NULL}};
const Segment SGI392468[] = {{&S82764[94], 3}, {NULL}};
const Segment SGI392469[] = {{&S82764[29], 7}, {NULL}};
const Segment SGI392470[] = {{&S82764[129], 5}, {NULL}};
const Segment SGI392471[] = {{&S87303[39], 4}, {NULL}};
const Segment SGI392472[] = {{&S87303[8], 3}, {NULL}};
const Segment SGI392473[] = {{&S87303[13], 14}, {NULL}};
const Segment SGI392474[] = {{&S87322[32], 4}, {NULL}};
const Segment SGI392475[] = {{&S87322[8], 3}, {NULL}};
const Segment SGI392476[] = {{&S87322[13], 14}, {NULL}};
const Segment SGI392477[] = {{&S92118[8], 3}, {NULL}};
const Segment SGI392478[] = {{&S92118[13], 9}, {NULL}};
const Segment SGI392479[] = {{&S92118[33], 13}, {NULL}};
const Segment SGI392480[] = {{&S92118[85], 12}, {NULL}};
const Segment SGI392481[] = {{&S83219[108], 3}, {NULL}};
const Segment SGI392482[] = {{&S83219[44], 6}, {NULL}};
const Segment SGI392483[] = {{&S83219[30], 6}, {NULL}};
const Segment SGI392484[] = {{&S83219[150], 5}, {NULL}};
const Segment SGI392485[] = {{&S92320[8], 3}, {NULL}};
const Segment SGI392486[] = {{&S92320[13], 7}, {NULL}};
const Segment SGI392487[] = {{&S92320[25], 14}, {NULL}};
const Segment SGI392488[] = {{&S92320[75], 12}, {NULL}};
const Segment SGI392489[] = {{&S92350[15], 3}, {NULL}};
const Segment SGI392490[] = {{&S92350[20], 7}, {NULL}};
const Segment SGI392491[] = {{&S92350[32], 14}, {NULL}};
const Segment SGI392492[] = {{&S92374[8], 3}, {NULL}};
const Segment SGI392493[] = {{&S92374[13], 3}, {NULL}};
const Segment SGI392494[] = {{&S92374[58], 13}, {NULL}};
const Segment SGI392495[] = {{&S95565[8], 3}, {NULL}};
const Segment SGI392496[] = {{&S95565[13], 7}, {NULL}};
const Segment SGI392497[] = {{&S95565[25], 14}, {NULL}};
const Segment SGI392498[] = {{&S95565[95], 9}, {NULL}};
const Segment SGI392499[] = {{&S92497[8], 3}, {NULL}};
const Segment SGI392500[] = {{&S92497[13], 9}, {NULL}};
const Segment SGI392501[] = {{&S92497[33], 13}, {NULL}};
const Segment SGI392502[] = {{&S92497[85], 12}, {NULL}};
const Segment SGI392503[] = {{&S88065[48], 3}, {NULL}};
const Segment SGI392504[] = {{&S88065[8], 3}, {NULL}};
const Segment SGI392505[] = {{&S88065[13], 9}, {NULL}};
const Segment SGI392506[] = {{&S88065[30], 13}, {NULL}};
const Segment SGI392507[] = {{&S88186[48], 4}, {NULL}};
const Segment SGI392508[] = {{&S88186[8], 3}, {NULL}};
const Segment SGI392509[] = {{&S88186[13], 9}, {NULL}};
const Segment SGI392510[] = {{&S88186[30], 13}, {NULL}};
const Segment SGI392511[] = {{&S83638[109], 3}, {NULL}};
const Segment SGI392512[] = {{&S83638[44], 7}, {NULL}};
const Segment SGI392513[] = {{&S83638[30], 6}, {NULL}};
const Segment SGI392514[] = {{&S83638[151], 5}, {NULL}};
const Segment SGI392515[] = {{&S92676[8], 3}, {NULL}};
const Segment SGI392516[] = {{&S92676[13], 7}, {NULL}};
const Segment SGI392517[] = {{&S92676[25], 14}, {NULL}};
const Segment SGI392518[] = {{&S92676[66], 12}, {NULL}};
const Segment SGI392519[] = {{&S92705[8], 3}, {NULL}};
const Segment SGI392520[] = {{&S92705[13], 3}, {NULL}};
const Segment SGI392521[] = {{&S92705[13], 3}, {&S92705[21], 10}, {NULL}};
const Segment SGI392522[] = {{&S92705[63], 13}, {NULL}};
const Segment SGI392523[] = {{&S88309[48], 4}, {NULL}};
const Segment SGI392524[] = {{&S88309[8], 3}, {NULL}};
const Segment SGI392525[] = {{&S88309[13], 9}, {NULL}};
const Segment SGI392526[] = {{&S88309[30], 13}, {NULL}};
const Segment SGI392527[] = {{&S88427[48], 4}, {NULL}};
const Segment SGI392528[] = {{&S88427[8], 3}, {NULL}};
const Segment SGI392529[] = {{&S88427[13], 9}, {NULL}};
const Segment SGI392530[] = {{&S88427[30], 13}, {NULL}};
const Segment SGI392531[] = {{&S88461[32], 4}, {NULL}};
const Segment SGI392532[] = {{&S88461[8], 3}, {NULL}};
const Segment SGI392533[] = {{&S88461[13], 14}, {NULL}};
const Segment SGI392534[] = {{&S53055[0], 7}, {NULL}};
const Segment SGI392535[] = {{&S53055[37], 4}, {NULL}};
const Segment SGI392536[] = {{&S88839[44], 4}, {NULL}};
const Segment SGI392537[] = {{&S88839[8], 3}, {NULL}};
const Segment SGI392538[] = {{&S88839[13], 14}, {NULL}};
const Segment SGI392539[] = {{&S84201[109], 3}, {NULL}};
const Segment SGI392540[] = {{&S84201[44], 7}, {NULL}};
const Segment SGI392541[] = {{&S84201[30], 6}, {NULL}};
const Segment SGI392542[] = {{&S84201[151], 5}, {NULL}};
const Segment SGI392543[] = {{&S93316[8], 3}, {NULL}};
const Segment SGI392544[] = {{&S93316[13], 7}, {NULL}};
const Segment SGI392545[] = {{&S93316[25], 14}, {NULL}};
const Segment SGI392546[] = {{&S93316[66], 12}, {NULL}};
const Segment SGI392547[] = {{&S53349[0], 6}, {NULL}};
const Segment SGI392548[] = {{&S53349[60], 10}, {NULL}};
const Segment SGI392549[] = {{&S53349[43], 4}, {NULL}};
const Segment SGI392550[] = {{&S53350[0], 6}, {NULL}};
const Segment SGI392551[] = {{&S53350[60], 4}, {NULL}};
const Segment SGI392552[] = {{&S53350[43], 4}, {NULL}};
const Segment SGI392553[] = {{&S53351[0], 6}, {NULL}};
const Segment SGI392554[] = {{&S53351[60], 10}, {NULL}};
const Segment SGI392555[] = {{&S53351[43], 4}, {NULL}};
const Segment SGI392556[] = {{&S53352[0], 6}, {NULL}};
const Segment SGI392557[] = {{&S53352[60], 7}, {NULL}};
const Segment SGI392558[] = {{&S53352[43], 4}, {NULL}};
const Segment SGI392559[] = {{&S53353[0], 6}, {NULL}};
const Segment SGI392560[] = {{&S53353[60], 6}, {NULL}};
const Segment SGI392561[] = {{&S53353[43], 4}, {NULL}};
const Segment SGI392562[] = {{&S53354[0], 6}, {NULL}};
const Segment SGI392563[] = {{&S53354[60], 7}, {NULL}};
const Segment SGI392564[] = {{&S53354[43], 4}, {NULL}};
const Segment SGI392565[] = {{&S88987[86], 3}, {NULL}};
const Segment SGI392566[] = {{&S88987[8], 3}, {NULL}};
const Segment SGI392567[] = {{&S88987[13], 9}, {NULL}};
const Segment SGI392568[] = {{&S88987[31], 15}, {NULL}};
const Segment SGI392569[] = {{&S84291[109], 3}, {NULL}};
const Segment SGI392570[] = {{&S84291[44], 7}, {NULL}};
const Segment SGI392571[] = {{&S84291[30], 6}, {NULL}};
const Segment SGI392572[] = {{&S84291[151], 5}, {NULL}};
const Segment SGI392573[] = {{&S89239[87], 3}, {NULL}};
const Segment SGI392574[] = {{&S89239[8], 3}, {NULL}};
const Segment SGI392575[] = {{&S89239[13], 9}, {NULL}};
const Segment SGI392576[] = {{&S89239[33], 13}, {NULL}};
const Segment SGI392577[] = {{&S89240[48], 3}, {NULL}};
const Segment SGI392578[] = {{&S89240[8], 3}, {NULL}};
const Segment SGI392579[] = {{&S89240[13], 9}, {NULL}};
const Segment SGI392580[] = {{&S89240[30], 13}, {NULL}};
const Segment SGI392581[] = {{&S93571[8], 3}, {&S93571[90], 3}, {NULL}};
const Segment SGI392582[] = {{&S93571[13], 3}, {&S93571[95], 10}, {NULL}};
const Segment SGI392583[] = {{&S93571[13], 3}, {&S93571[59], 6}, {NULL}};
const Segment SGI392584[] = {{&S93571[59], 10}, {NULL}};
const Segment SGI392585[] = {{&S53673[0], 6}, {NULL}};
const Segment SGI392586[] = {{&S53673[36], 4}, {NULL}};
const Segment SGI392587[] = {{&S93593[8], 3}, {NULL}};
const Segment SGI392588[] = {{&S93593[13], 3}, {NULL}};
const Segment SGI392589[] = {{&S93593[13], 3}, {&S93593[21], 10}, {&S93593[63], 6}, {NULL}};
const Segment SGI392590[] = {{&S93593[63], 10}, {NULL}};
const Segment SGI392591[] = {{&S89321[39], 4}, {NULL}};
const Segment SGI392592[] = {{&S89321[8], 3}, {NULL}};
const Segment SGI392593[] = {{&S89321[13], 14}, {NULL}};
const Segment SGI392594[] = {{&S93603[8], 3}, {NULL}};
const Segment SGI392595[] = {{&S93603[13], 3}, {NULL}};
const Segment SGI392596[] = {{&S93603[62], 13}, {NULL}};
const Segment SGI392597[] = {{&S53827[0], 15}, {NULL}};
const Segment SGI392598[] = {{&S53827[60], 10}, {NULL}};
const Segment SGI392599[] = {{&S53827[43], 4}, {NULL}};
const Segment SGI392600[] = {{&S53828[0], 15}, {NULL}};
const Segment SGI392601[] = {{&S53828[60], 4}, {NULL}};
const Segment SGI392602[] = {{&S53828[43], 4}, {NULL}};
const Segment SGI392603[] = {{&S53829[0], 15}, {NULL}};
const Segment SGI392604[] = {{&S53829[60], 10}, {NULL}};
const Segment SGI392605[] = {{&S53829[43], 4}, {NULL}};
const Segment SGI392606[] = {{&S53830[0], 15}, {NULL}};
const Segment SGI392607[] = {{&S53830[60], 6}, {NULL}};
const Segment SGI392608[] = {{&S53830[43], 4}, {NULL}};
const Segment SGI392609[] = {{&S53831[0], 15}, {NULL}};
const Segment SGI392610[] = {{&S53831[60], 7}, {NULL}};
const Segment SGI392611[] = {{&S53831[43], 4}, {NULL}};
const Segment SGI392612[] = {{&S53832[0], 15}, {NULL}};
const Segment SGI392613[] = {{&S53832[60], 7}, {NULL}};
const Segment SGI392614[] = {{&S53832[43], 4}, {NULL}};
const Segment SGI392615[] = {{&S89574[48], 4}, {NULL}};
const Segment SGI392616[] = {{&S89574[8], 3}, {NULL}};
const Segment SGI392617[] = {{&S89574[13], 9}, {NULL}};
const Segment SGI392618[] = {{&S89574[30], 13}, {NULL}};
const Segment SGI392619[] = {{&S93886[8], 3}, {NULL}};
const Segment SGI392620[] = {{&S93886[13], 7}, {NULL}};
const Segment SGI392621[] = {{&S93886[25], 14}, {NULL}};
const Segment SGI392622[] = {{&S93886[69], 13}, {NULL}};
const Segment SGI392623[] = {{&S89719[81], 3}, {NULL}};
const Segment SGI392624[] = {{&S89719[8], 3}, {NULL}};
const Segment SGI392625[] = {{&S89719[13], 7}, {NULL}};
const Segment SGI392626[] = {{&S89719[25], 14}, {NULL}};
const Segment SGI392627[] = {{&S54095[0], 15}, {NULL}};
const Segment SGI392628[] = {{&S54095[60], 10}, {NULL}};
const Segment SGI392629[] = {{&S54095[43], 4}, {NULL}};
const Segment SGI392630[] = {{&S54096[0], 15}, {NULL}};
const Segment SGI392631[] = {{&S54096[60], 4}, {NULL}};
const Segment SGI392632[] = {{&S54096[43], 4}, {NULL}};
const Segment SGI392633[] = {{&S54097[0], 15}, {NULL}};
const Segment SGI392634[] = {{&S54097[60], 10}, {NULL}};
const Segment SGI392635[] = {{&S54097[43], 4}, {NULL}};
const Segment SGI392636[] = {{&S54098[0], 15}, {NULL}};
const Segment SGI392637[] = {{&S54098[58], 6}, {NULL}};
const Segment SGI392638[] = {{&S54098[43], 4}, {NULL}};
const Segment SGI392639[] = {{&S54099[0], 15}, {NULL}};
const Segment SGI392640[] = {{&S54099[60], 7}, {NULL}};
const Segment SGI392641[] = {{&S54099[43], 4}, {NULL}};
const Segment SGI392642[] = {{&S84820[109], 3}, {NULL}};
const Segment SGI392643[] = {{&S84820[44], 7}, {NULL}};
const Segment SGI392644[] = {{&S84820[30], 6}, {NULL}};
const Segment SGI392645[] = {{&S84820[151], 5}, {NULL}};
const Segment SGI392646[] = {{&S84884[94], 3}, {NULL}};
const Segment SGI392647[] = {{&S84884[29], 7}, {NULL}};
const Segment SGI392648[] = {{&S84884[129], 5}, {NULL}};
const Segment SGI392649[] = {{&S89867[39], 3}, {NULL}};
const Segment SGI392650[] = {{&S89867[8], 3}, {NULL}};
const Segment SGI392651[] = {{&S89867[13], 14}, {NULL}};
const Segment SGI392652[] = {{&S94216[8], 3}, {NULL}};
const Segment SGI392653[] = {{&S94216[13], 3}, {NULL}};
const Segment SGI392654[] = {{&S94216[58], 13}, {NULL}};
const Segment SGI392655[] = {{&S90042[48], 3}, {NULL}};
const Segment SGI392656[] = {{&S90042[8], 3}, {NULL}};
const Segment SGI392657[] = {{&S90042[13], 9}, {NULL}};
const Segment SGI392658[] = {{&S90042[30], 13}, {NULL}};
const Segment SGI392659[] = {{&S94239[8], 3}, {NULL}};
const Segment SGI392660[] = {{&S94239[13], 7}, {NULL}};
const Segment SGI392661[] = {{&S94239[25], 14}, {NULL}};
const Segment SGI392662[] = {{&S94239[66], 13}, {NULL}};
const Segment SGI392663[] = {{&S54537[0], 13}, {NULL}};
const Segment SGI392664[] = {{&S54537[41], 4}, {NULL}};
const Segment SGI392665[] = {{&S54538[0], 13}, {NULL}};
const Segment SGI392666[] = {{&S85262[108], 3}, {NULL}};
const Segment SGI392667[] = {{&S54538[39], 4}, {NULL}};
const Segment SGI392668[] = {{&S85262[44], 6}, {NULL}};
const Segment SGI392669[] = {{&S85262[30], 6}, {NULL}};
const Segment SGI392670[] = {{&S85262[150], 4}, {NULL}};
const Segment SGI392671[] = {{&S54539[0], 8}, {NULL}};
const Segment SGI392672[] = {{&S54539[35], 4}, {NULL}};
const Segment SGI392673[] = {{&S54540[0], 9}, {NULL}};
const Segment SGI392674[] = {{&S54540[37], 4}, {NULL}};
const Segment SGI392675[] = {{&S54541[0], 11}, {NULL}};
const Segment SGI392676[] = {{&S54541[44], 4}, {NULL}};
const Segment SGI392677[] = {{&S54542[0], 18}, {NULL}};
const Segment SGI392678[] = {{&S54542[50], 4}, {NULL}};
const Segment SGI392679[] = {{&S54543[0], 13}, {NULL}};
const Segment SGI392680[] = {{&S54543[46], 4}, {NULL}};
const Segment SGI392681[] = {{&S94426[8], 3}, {NULL}};
const Segment SGI392682[] = {{&S94426[13], 3}, {NULL}};
const Segment SGI392683[] = {{&S94426[13], 3}, {&S94426[21], 10}, {NULL}};
const Segment SGI392684[] = {{&S94426[70], 6}, {NULL}};
const Segment SGI392685[] = {{&S94426[89], 13}, {NULL}};
const Segment SGI392686[] = {{&S54688[0], 6}, {NULL}};
const Segment SGI392687[] = {{&S54688[34], 4}, {NULL}};
const Segment SGI392688[] = {{&S94676[8], 3}, {NULL}};
const Segment SGI392689[] = {{&S94676[13], 7}, {NULL}};
const Segment SGI392690[] = {{&S94676[25], 14}, {NULL}};
const Segment SGI392691[] = {{&S94676[78], 12}, {NULL}};
const Segment SGI392692[] = {{&S94704[8], 3}, {NULL}};
const Segment SGI392693[] = {{&S94704[13], 7}, {NULL}};
const Segment SGI392694[] = {{&S94704[25], 14}, {NULL}};
const Segment SGI392695[] = {{&S94704[66], 12}, {NULL}};
const Segment SGI392696[] = {{&S94721[8], 3}, {NULL}};
const Segment SGI392697[] = {{&S94721[13], 3}, {NULL}};
const Segment SGI392698[] = {{&S94721[58], 13}, {NULL}};
const Segment SGI392699[] = {{&S94740[8], 3}, {NULL}};
const Segment SGI392700[] = {{&S94740[13], 7}, {NULL}};
const Segment SGI392701[] = {{&S94740[25], 14}, {NULL}};
const Segment SGI392702[] = {{&S94740[66], 12}, {NULL}};
const Segment SGI392703[] = {{&S90554[33], 3}, {NULL}};
const Segment SGI392704[] = {{&S90554[8], 3}, {NULL}};
const Segment SGI392705[] = {{&S90554[13], 3}, {NULL}};
const Segment SGI392706[] = {{&S95202[8], 3}, {NULL}};
const Segment SGI392707[] = {{&S95202[13], 9}, {NULL}};
const Segment SGI392708[] = {{&S95202[31], 15}, {NULL}};
const Segment SGI392709[] = {{&S95202[71], 10}, {NULL}};
const Segment SGI392710[] = {{&S91316[48], 4}, {NULL}};
const Segment SGI392711[] = {{&S91316[8], 3}, {NULL}};
const Segment SGI392712[] = {{&S91316[13], 9}, {NULL}};
const Segment SGI392713[] = {{&S91316[30], 13}, {NULL}};
const Segment SGI392714[] = {{&S95408[8], 3}, {NULL}};
const Segment SGI392715[] = {{&S95408[13], 14}, {NULL}};
const Segment SGI392716[] = {{&S95408[54], 11}, {NULL}};
const Segment SGI392717[] = {{&S91372[88], 3}, {NULL}};
const Segment SGI392718[] = {{&S91372[8], 3}, {NULL}};
const Segment SGI392719[] = {{&S91372[13], 9}, {NULL}};
const Segment SGI392720[] = {{&S91372[33], 13}, {NULL}};
const Segment SGI392721[] = {{&S95702[8], 3}, {NULL}};
const Segment SGI392722[] = {{&S95702[13], 9}, {NULL}};
const Segment SGI392723[] = {{&S95702[33], 13}, {NULL}};
const Segment SGI392724[] = {{&S95702[76], 11}, {NULL}};
const Segment SGI392725[] = {{&S86576[108], 3}, {NULL}};
const Segment SGI392726[] = {{&S86576[44], 6}, {NULL}};
const Segment SGI392727[] = {{&S86576[30], 6}, {NULL}};
const Segment SGI392728[] = {{&S86576[150], 5}, {NULL}};
const Segment SGI392729[] = {{&S96012[8], 3}, {NULL}};
const Segment SGI392730[] = {{&S96012[13], 9}, {NULL}};
const Segment SGI392731[] = {{&S96012[31], 15}, {NULL}};
const Segment SGI392732[] = {{&S96012[66], 10}, {NULL}};
const Segment SGI392733[] = {{&S96070[8], 3}, {NULL}};
const Segment SGI392734[] = {{&S96070[13], 3}, {NULL}};
const Segment SGI392735[] = {{&S96070[65], 13}, {NULL}};
const Segment SGI392736[] = {{&S92122[39], 4}, {NULL}};
const Segment SGI392737[] = {{&S92122[8], 3}, {NULL}};
const Segment SGI392738[] = {{&S92122[13], 14}, {NULL}};
const Segment SGI392739[] = {{&S92157[39], 4}, {NULL}};
const Segment SGI392740[] = {{&S92157[8], 3}, {NULL}};
const Segment SGI392741[] = {{&S92157[13], 14}, {NULL}};
const Segment SGI392742[] = {{&S92317[48], 3}, {NULL}};
const Segment SGI392743[] = {{&S92317[8], 3}, {NULL}};
const Segment SGI392744[] = {{&S92317[13], 9}, {NULL}};
const Segment SGI392745[] = {{&S92317[30], 13}, {NULL}};
const Segment SGI392746[] = {{&S96338[8], 3}, {NULL}};
const Segment SGI392747[] = {{&S96338[13], 9}, {NULL}};
const Segment SGI392748[] = {{&S96338[33], 13}, {NULL}};
const Segment SGI392749[] = {{&S96338[76], 11}, {NULL}};
const Segment SGI392750[] = {{&S96351[8], 3}, {NULL}};
const Segment SGI392751[] = {{&S96351[13], 7}, {NULL}};
const Segment SGI392752[] = {{&S96351[25], 14}, {NULL}};
const Segment SGI392753[] = {{&S96351[68], 13}, {NULL}};
const Segment SGI392754[] = {{&S87115[109], 3}, {NULL}};
const Segment SGI392755[] = {{&S87115[44], 7}, {NULL}};
const Segment SGI392756[] = {{&S87115[30], 6}, {NULL}};
const Segment SGI392757[] = {{&S87115[151], 5}, {NULL}};
const Segment SGI392758[] = {{&S87133[109], 3}, {NULL}};
const Segment SGI392759[] = {{&S87133[44], 7}, {NULL}};
const Segment SGI392760[] = {{&S87133[30], 6}, {NULL}};
const Segment SGI392761[] = {{&S87133[151], 4}, {NULL}};
const Segment SGI392762[] = {{&S96650[8], 3}, {NULL}};
const Segment SGI392763[] = {{&S96650[13], 3}, {NULL}};
const Segment SGI392764[] = {{&S96650[13], 3}, {&S96650[21], 10}, {NULL}};
const Segment SGI392765[] = {{&S96650[58], 12}, {NULL}};
const Segment SGI392766[] = {{&S96714[8], 3}, {NULL}};
const Segment SGI392767[] = {{&S96714[13], 3}, {NULL}};
const Segment SGI392768[] = {{&S96714[13], 3}, {&S96714[21], 10}, {NULL}};
const Segment SGI392769[] = {{&S96714[58], 12}, {NULL}};
const Segment SGI392770[] = {{&S87707[94], 3}, {NULL}};
const Segment SGI392771[] = {{&S87707[29], 7}, {NULL}};
const Segment SGI392772[] = {{&S87707[129], 5}, {NULL}};
const Segment SGI392773[] = {{&S92858[39], 3}, {NULL}};
const Segment SGI392774[] = {{&S92858[8], 3}, {NULL}};
const Segment SGI392775[] = {{&S92858[13], 14}, {NULL}};
const Segment SGI392776[] = {{&S97048[8], 3}, {NULL}};
const Segment SGI392777[] = {{&S97048[13], 3}, {NULL}};
const Segment SGI392778[] = {{&S97048[13], 3}, {&S97048[62], 6}, {NULL}};
const Segment SGI392779[] = {{&S97048[85], 13}, {NULL}};
const Segment SGI392780[] = {{&S97048[62], 10}, {NULL}};
const Segment SGI392781[] = {{&S97056[8], 3}, {NULL}};
const Segment SGI392782[] = {{&S97056[13], 9}, {NULL}};
const Segment SGI392783[] = {{&S97056[30], 13}, {NULL}};
const Segment SGI392784[] = {{&S97056[63], 11}, {NULL}};
const Segment SGI392785[] = {{&S92917[39], 4}, {NULL}};
const Segment SGI392786[] = {{&S92917[8], 3}, {NULL}};
const Segment SGI392787[] = {{&S92917[13], 14}, {NULL}};
const Segment SGI392788[] = {{&S92940[39], 4}, {NULL}};
const Segment SGI392789[] = {{&S92940[8], 3}, {NULL}};
const Segment SGI392790[] = {{&S92940[13], 14}, {NULL}};
const Segment SGI392791[] = {{&S97328[8], 3}, {NULL}};
const Segment SGI392792[] = {{&S97328[13], 3}, {NULL}};
const Segment SGI392793[] = {{&S97328[13], 3}, {&S97328[21], 10}, {NULL}};
const Segment SGI392794[] = {{&S97328[60], 6}, {NULL}};
const Segment SGI392795[] = {{&S97328[86], 13}, {NULL}};
const Segment SGI392796[] = {{&S57587[0], 8}, {NULL}};
const Segment SGI392797[] = {{&S57587[41], 4}, {NULL}};
const Segment SGI392798[] = {{&S57588[0], 6}, {NULL}};
const Segment SGI392799[] = {{&S57588[36], 4}, {NULL}};
const Segment SGI392800[] = {{&S93105[48], 4}, {NULL}};
const Segment SGI392801[] = {{&S93105[8], 3}, {NULL}};
const Segment SGI392802[] = {{&S93105[13], 9}, {NULL}};
const Segment SGI392803[] = {{&S93105[30], 13}, {NULL}};
const Segment SGI392804[] = {{&S93161[81], 3}, {NULL}};
const Segment SGI392805[] = {{&S93161[8], 3}, {NULL}};
const Segment SGI392806[] = {{&S93161[13], 7}, {NULL}};
const Segment SGI392807[] = {{&S93161[25], 14}, {NULL}};
const Segment SGI392808[] = {{&S97705[8], 3}, {NULL}};
const Segment SGI392809[] = {{&S97705[13], 7}, {NULL}};
const Segment SGI392810[] = {{&S97705[58], 12}, {NULL}};
const Segment SGI392811[] = {{&S97706[8], 3}, {NULL}};
const Segment SGI392812[] = {{&S97706[13], 7}, {NULL}};
const Segment SGI392813[] = {{&S97706[25], 14}, {NULL}};
const Segment SGI392814[] = {{&S97706[66], 12}, {NULL}};
const Segment SGI392815[] = {{&S57836[0], 7}, {NULL}};
const Segment SGI392816[] = {{&S57836[33], 4}, {NULL}};
const Segment SGI392817[] = {{&S57959[0], 9}, {NULL}};
const Segment SGI392818[] = {{&S57959[37], 4}, {NULL}};
const Segment SGI392819[] = {{&S58023[0], 9}, {NULL}};
const Segment SGI392820[] = {{&S58023[41], 4}, {NULL}};
const Segment SGI392821[] = {{&S97932[8], 3}, {NULL}};
const Segment SGI392822[] = {{&S97932[13], 7}, {NULL}};
const Segment SGI392823[] = {{&S97932[62], 13}, {NULL}};
const Segment SGI392824[] = {{&S93308[39], 4}, {NULL}};
const Segment SGI392825[] = {{&S93308[8], 3}, {NULL}};
const Segment SGI392826[] = {{&S93308[13], 14}, {NULL}};
const Segment SGI392827[] = {{&S93310[32], 4}, {NULL}};
const Segment SGI392828[] = {{&S93310[8], 3}, {NULL}};
const Segment SGI392829[] = {{&S93310[13], 14}, {NULL}};
const Segment SGI392830[] = {{&S93320[8], 3}, {NULL}};
const Segment SGI392831[] = {{&S93320[13], 14}, {NULL}};
const Segment SGI392832[] = {{&S98099[8], 3}, {NULL}};
const Segment SGI392833[] = {{&S98099[13], 9}, {NULL}};
const Segment SGI392834[] = {{&S98099[68], 10}, {NULL}};
const Segment SGI392835[] = {{&S58426[0], 6}, {NULL}};
const Segment SGI392836[] = {{&S58426[38], 4}, {NULL}};
const Segment SGI392837[] = {{&S93543[48], 4}, {NULL}};
const Segment SGI392838[] = {{&S93543[8], 3}, {NULL}};
const Segment SGI392839[] = {{&S93543[13], 9}, {NULL}};
const Segment SGI392840[] = {{&S93543[30], 13}, {NULL}};
const Segment SGI392841[] = {{&S93774[32], 3}, {NULL}};
const Segment SGI392842[] = {{&S93774[8], 3}, {NULL}};
const Segment SGI392843[] = {{&S93774[13], 14}, {NULL}};
const Segment SGI392844[] = {{&S93775[39], 3}, {NULL}};
const Segment SGI392845[] = {{&S93775[8], 3}, {NULL}};
const Segment SGI392846[] = {{&S93775[13], 14}, {NULL}};
const Segment SGI392847[] = {{&S98601[8], 3}, {NULL}};
const Segment SGI392848[] = {{&S98601[13], 7}, {NULL}};
const Segment SGI392849[] = {{&S98601[25], 14}, {NULL}};
const Segment SGI392850[] = {{&S98601[66], 12}, {NULL}};
const Segment SGI392851[] = {{&S58754[0], 7}, {NULL}};
const Segment SGI392852[] = {{&S58754[37], 4}, {NULL}};
const Segment SGI392853[] = {{&S98811[8], 3}, {NULL}};
const Segment SGI392854[] = {{&S98811[13], 9}, {NULL}};
const Segment SGI392855[] = {{&S98811[30], 13}, {NULL}};
const Segment SGI392856[] = {{&S98811[63], 11}, {NULL}};
const Segment SGI392857[] = {{&S102264[8], 3}, {NULL}};
const Segment SGI392858[] = {{&S102264[13], 7}, {NULL}};
const Segment SGI392859[] = {{&S102264[25], 14}, {NULL}};
const Segment SGI392860[] = {{&S102264[85], 49}, {NULL}};
const Segment SGI392861[] = {{&S89607[94], 3}, {NULL}};
const Segment SGI392862[] = {{&S89607[29], 7}, {NULL}};
const Segment SGI392863[] = {{&S89607[129], 5}, {NULL}};
const Segment SGI392864[] = {{&S93995[39], 3}, {NULL}};
const Segment SGI392865[] = {{&S93995[8], 3}, {NULL}};
const Segment SGI392866[] = {{&S93995[13], 14}, {NULL}};
const Segment SGI392867[] = {{&S89748[109], 3}, {NULL}};
const Segment SGI392868[] = {{&S89748[44], 7}, {NULL}};
const Segment SGI392869[] = {{&S89748[30], 6}, {NULL}};
const Segment SGI392870[] = {{&S89748[151], 4}, {NULL}};
const Segment SGI392871[] = {{&S93996[32], 3}, {NULL}};
const Segment SGI392872[] = {{&S93996[8], 3}, {NULL}};
const Segment SGI392873[] = {{&S93996[13], 14}, {NULL}};
const Segment SGI392874[] = {{&S99064[8], 3}, {NULL}};
const Segment SGI392875[] = {{&S99064[13], 9}, {NULL}};
const Segment SGI392876[] = {{&S99064[31], 15}, {NULL}};
const Segment SGI392877[] = {{&S99064[73], 12}, {NULL}};
const Segment SGI392878[] = {{&S99067[8], 3}, {NULL}};
const Segment SGI392879[] = {{&S99067[13], 3}, {NULL}};
const Segment SGI392880[] = {{&S99067[13], 3}, {&S99067[21], 10}, {&S99067[64], 6}, {NULL}};
const Segment SGI392881[] = {{&S99067[64], 10}, {NULL}};
const Segment SGI392882[] = {{&S99100[8], 3}, {NULL}};
const Segment SGI392883[] = {{&S99100[13], 7}, {NULL}};
const Segment SGI392884[] = {{&S99100[25], 14}, {NULL}};
const Segment SGI392885[] = {{&S99107[8], 3}, {NULL}};
const Segment SGI392886[] = {{&S99107[13], 14}, {NULL}};
const Segment SGI392887[] = {{&S89900[108], 3}, {NULL}};
const Segment SGI392888[] = {{&S89900[44], 6}, {NULL}};
const Segment SGI392889[] = {{&S89900[30], 6}, {NULL}};
const Segment SGI392890[] = {{&S89900[150], 5}, {NULL}};
const Segment SGI392891[] = {{&S94428[88], 3}, {NULL}};
const Segment SGI392892[] = {{&S94428[8], 3}, {NULL}};
const Segment SGI392893[] = {{&S94428[13], 9}, {NULL}};
const Segment SGI392894[] = {{&S94428[33], 13}, {NULL}};
const Segment SGI392895[] = {{&S90231[109], 3}, {NULL}};
const Segment SGI392896[] = {{&S90231[44], 7}, {NULL}};
const Segment SGI392897[] = {{&S90231[30], 6}, {NULL}};
const Segment SGI392898[] = {{&S90231[151], 4}, {NULL}};
const Segment SGI392899[] = {{&S94481[48], 4}, {NULL}};
const Segment SGI392900[] = {{&S94481[8], 3}, {NULL}};
const Segment SGI392901[] = {{&S94481[13], 9}, {NULL}};
const Segment SGI392902[] = {{&S94481[30], 13}, {NULL}};
const Segment SGI392903[] = {{&S90260[94], 3}, {NULL}};
const Segment SGI392904[] = {{&S90260[29], 7}, {NULL}};
const Segment SGI392905[] = {{&S90260[129], 5}, {NULL}};
const Segment SGI392906[] = {{&S94516[32], 4}, {NULL}};
const Segment SGI392907[] = {{&S94516[8], 3}, {NULL}};
const Segment SGI392908[] = {{&S94516[13], 14}, {NULL}};
const Segment SGI392909[] = {{&S94735[39], 3}, {NULL}};
const Segment SGI392910[] = {{&S94735[8], 3}, {NULL}};
const Segment SGI392911[] = {{&S94735[13], 14}, {NULL}};
const Segment SGI392912[] = {{&S94776[32], 4}, {NULL}};
const Segment SGI392913[] = {{&S94776[8], 3}, {NULL}};
const Segment SGI392914[] = {{&S94776[13], 14}, {NULL}};
const Segment SGI392915[] = {{&S60028[0], 25}, {NULL}};
const Segment SGI392916[] = {{&S60028[55], 21}, {NULL}};
const Segment SGI392917[] = {{&S60028[40], 3}, {NULL}};
const Segment SGI392918[] = {{&S60029[0], 25}, {NULL}};
const Segment SGI392919[] = {{&S60029[54], 7}, {NULL}};
const Segment SGI392920[] = {{&S60029[40], 3}, {NULL}};
const Segment SGI392921[] = {{&S60030[0], 25}, {NULL}};
const Segment SGI392922[] = {{&S60030[54], 4}, {NULL}};
const Segment SGI392923[] = {{&S60030[40], 3}, {NULL}};
const Segment SGI392924[] = {{&S60031[0], 25}, {NULL}};
const Segment SGI392925[] = {{&S60031[54], 7}, {NULL}};
const Segment SGI392926[] = {{&S60031[40], 3}, {NULL}};
const Segment SGI392927[] = {{&S60032[0], 25}, {NULL}};
const Segment SGI392928[] = {{&S60032[54], 6}, {NULL}};
const Segment SGI392929[] = {{&S60032[40], 3}, {NULL}};
const Segment SGI392930[] = {{&S60033[0], 25}, {NULL}};
const Segment SGI392931[] = {{&S60033[54], 10}, {NULL}};
const Segment SGI392932[] = {{&S60033[40], 3}, {NULL}};
const Segment SGI392933[] = {{&S60034[0], 25}, {NULL}};
const Segment SGI392934[] = {{&S60034[54], 7}, {NULL}};
const Segment SGI392935[] = {{&S60034[40], 3}, {NULL}};
const Segment SGI392936[] = {{&S60035[0], 25}, {NULL}};
const Segment SGI392937[] = {{&S60035[54], 10}, {NULL}};
const Segment SGI392938[] = {{&S60035[40], 3}, {NULL}};
const Segment SGI392939[] = {{&S60036[0], 25}, {NULL}};
const Segment SGI392940[] = {{&S60036[55], 20}, {NULL}};
const Segment SGI392941[] = {{&S60036[40], 3}, {NULL}};
const Segment SGI392942[] = {{&S60037[0], 25}, {NULL}};
const Segment SGI392943[] = {{&S60037[55], 20}, {NULL}};
const Segment SGI392944[] = {{&S60037[40], 3}, {NULL}};
const Segment SGI392945[] = {{&S60038[0], 25}, {NULL}};
const Segment SGI392946[] = {{&S60038[54], 10}, {NULL}};
const Segment SGI392947[] = {{&S60038[40], 3}, {NULL}};
const Segment SGI392948[] = {{&S60039[0], 25}, {NULL}};
const Segment SGI392949[] = {{&S60039[54], 4}, {NULL}};
const Segment SGI392950[] = {{&S60039[40], 3}, {NULL}};
const Segment SGI392951[] = {{&S60040[0], 25}, {NULL}};
const Segment SGI392952[] = {{&S60040[54], 7}, {NULL}};
const Segment SGI392953[] = {{&S60040[40], 3}, {NULL}};
const Segment SGI392954[] = {{&S60041[0], 25}, {NULL}};
const Segment SGI392955[] = {{&S60041[54], 4}, {NULL}};
const Segment SGI392956[] = {{&S60041[40], 3}, {NULL}};
const Segment SGI392957[] = {{&S60042[0], 25}, {NULL}};
const Segment SGI392958[] = {{&S60042[54], 10}, {NULL}};
const Segment SGI392959[] = {{&S60042[40], 3}, {NULL}};
const Segment SGI392960[] = {{&S60043[0], 26}, {NULL}};
const Segment SGI392961[] = {{&S60043[55], 10}, {NULL}};
const Segment SGI392962[] = {{&S60043[41], 3}, {NULL}};
const Segment SGI392963[] = {{&S60044[0], 26}, {NULL}};
const Segment SGI392964[] = {{&S60044[55], 7}, {NULL}};
const Segment SGI392965[] = {{&S60044[41], 3}, {NULL}};
const Segment SGI392966[] = {{&S60045[0], 25}, {NULL}};
const Segment SGI392967[] = {{&S60045[54], 7}, {NULL}};
const Segment SGI392968[] = {{&S60045[40], 3}, {NULL}};
const Segment SGI392969[] = {{&S60046[0], 26}, {NULL}};
const Segment SGI392970[] = {{&S60046[55], 4}, {NULL}};
const Segment SGI392971[] = {{&S60046[41], 3}, {NULL}};
const Segment SGI392972[] = {{&S60047[0], 25}, {NULL}};
const Segment SGI392973[] = {{&S60047[54], 7}, {NULL}};
const Segment SGI392974[] = {{&S60047[40], 3}, {NULL}};
const Segment SGI392975[] = {{&S99599[8], 3}, {NULL}};
const Segment SGI392976[] = {{&S99599[13], 9}, {NULL}};
const Segment SGI392977[] = {{&S99599[30], 13}, {NULL}};
const Segment SGI392978[] = {{&S99619[8], 3}, {NULL}};
const Segment SGI392979[] = {{&S99619[13], 9}, {NULL}};
const Segment SGI392980[] = {{&S99619[33], 13}, {NULL}};
const Segment SGI392981[] = {{&S99619[76], 13}, {NULL}};
const Segment SGI392982[] = {{&S94908[48], 3}, {NULL}};
const Segment SGI392983[] = {{&S94908[8], 3}, {NULL}};
const Segment SGI392984[] = {{&S94908[13], 9}, {NULL}};
const Segment SGI392985[] = {{&S94908[30], 13}, {NULL}};
const Segment SGI392986[] = {{&S99638[8], 3}, {NULL}};
const Segment SGI392987[] = {{&S99638[13], 3}, {NULL}};
const Segment SGI392988[] = {{&S99638[13], 3}, {&S99638[21], 10}, {&S99638[79], 6}, {NULL}};
const Segment SGI392989[] = {{&S99638[61], 13}, {NULL}};
const Segment SGI392990[] = {{&S99648[8], 3}, {NULL}};
const Segment SGI392991[] = {{&S99648[13], 9}, {NULL}};
const Segment SGI392992[] = {{&S99648[33], 13}, {NULL}};
const Segment SGI392993[] = {{&S95166[78], 3}, {NULL}};
const Segment SGI392994[] = {{&S95166[8], 3}, {NULL}};
const Segment SGI392995[] = {{&S95166[13], 7}, {NULL}};
const Segment SGI392996[] = {{&S95166[25], 14}, {NULL}};
const Segment SGI392997[] = {{&S95184[72], 3}, {NULL}};
const Segment SGI392998[] = {{&S95184[8], 3}, {NULL}};
const Segment SGI392999[] = {{&S95184[13], 7}, {NULL}};
const Segment SGI393000[] = {{&S95287[48], 4}, {NULL}};
const Segment SGI393001[] = {{&S95287[8], 3}, {NULL}};
const Segment SGI393002[] = {{&S95287[13], 9}, {NULL}};
const Segment SGI393003[] = {{&S95287[30], 13}, {NULL}};
const Segment SGI393004[] = {{&S60581[0], 6}, {NULL}};
const Segment SGI393005[] = {{&S60581[60], 4}, {NULL}};
const Segment SGI393006[] = {{&S60581[43], 4}, {NULL}};
const Segment SGI393007[] = {{&S60582[0], 6}, {NULL}};
const Segment SGI393008[] = {{&S60582[60], 10}, {NULL}};
const Segment SGI393009[] = {{&S60582[43], 4}, {NULL}};
const Segment SGI393010[] = {{&S60583[0], 6}, {NULL}};
const Segment SGI393011[] = {{&S60583[60], 10}, {NULL}};
const Segment SGI393012[] = {{&S60583[43], 4}, {NULL}};
const Segment SGI393013[] = {{&S60584[0], 6}, {NULL}};
const Segment SGI393014[] = {{&S60584[60], 7}, {NULL}};
const Segment SGI393015[] = {{&S60584[43], 4}, {NULL}};
const Segment SGI393016[] = {{&S60585[0], 6}, {NULL}};
const Segment SGI393017[] = {{&S60585[60], 6}, {NULL}};
const Segment SGI393018[] = {{&S60585[43], 4}, {NULL}};
const Segment SGI393019[] = {{&S100164[8], 3}, {NULL}};
const Segment SGI393020[] = {{&S100164[13], 9}, {NULL}};
const Segment SGI393021[] = {{&S100164[30], 13}, {NULL}};
const Segment SGI393022[] = {{&S100164[63], 11}, {NULL}};
const Segment SGI393023[] = {{&S100185[8], 3}, {NULL}};
const Segment SGI393024[] = {{&S100185[13], 7}, {NULL}};
const Segment SGI393025[] = {{&S100185[25], 14}, {NULL}};
const Segment SGI393026[] = {{&S100185[78], 12}, {NULL}};
const Segment SGI393027[] = {{&S100193[8], 3}, {NULL}};
const Segment SGI393028[] = {{&S100193[13], 7}, {NULL}};
const Segment SGI393029[] = {{&S100193[25], 5}, {NULL}};
const Segment SGI393030[] = {{&S100193[57], 12}, {NULL}};
const Segment SGI393031[] = {{&S61015[0], 15}, {NULL}};
const Segment SGI393032[] = {{&S61015[59], 10}, {NULL}};
const Segment SGI393033[] = {{&S61015[44], 4}, {NULL}};
const Segment SGI393034[] = {{&S61016[0], 15}, {NULL}};
const Segment SGI393035[] = {{&S61016[59], 4}, {NULL}};
const Segment SGI393036[] = {{&S61016[44], 4}, {NULL}};
const Segment SGI393037[] = {{&S61017[0], 15}, {NULL}};
const Segment SGI393038[] = {{&S61017[59], 10}, {NULL}};
const Segment SGI393039[] = {{&S61017[44], 4}, {NULL}};
const Segment SGI393040[] = {{&S61018[0], 15}, {NULL}};
const Segment SGI393041[] = {{&S61018[59], 7}, {NULL}};
const Segment SGI393042[] = {{&S61018[44], 4}, {NULL}};
const Segment SGI393043[] = {{&S61019[0], 15}, {NULL}};
const Segment SGI393044[] = {{&S61019[59], 7}, {NULL}};
const Segment SGI393045[] = {{&S61019[44], 4}, {NULL}};
const Segment SGI393046[] = {{&S61020[0], 15}, {NULL}};
const Segment SGI393047[] = {{&S61020[59], 6}, {NULL}};
const Segment SGI393048[] = {{&S61020[44], 4}, {NULL}};
const Segment SGI393049[] = {{&S95749[39], 4}, {NULL}};
const Segment SGI393050[] = {{&S95749[8], 3}, {NULL}};
const Segment SGI393051[] = {{&S95749[13], 14}, {NULL}};
const Segment SGI393052[] = {{&S100317[8], 3}, {NULL}};
const Segment SGI393053[] = {{&S100317[13], 7}, {NULL}};
const Segment SGI393054[] = {{&S100317[25], 14}, {NULL}};
const Segment SGI393055[] = {{&S100317[68], 12}, {NULL}};
const Segment SGI393056[] = {{&S95777[81], 3}, {NULL}};
const Segment SGI393057[] = {{&S95777[8], 3}, {NULL}};
const Segment SGI393058[] = {{&S95777[13], 7}, {NULL}};
const Segment SGI393059[] = {{&S95777[25], 14}, {NULL}};
const Segment SGI393060[] = {{&S100400[8], 3}, {NULL}};
const Segment SGI393061[] = {{&S100400[13], 7}, {NULL}};
const Segment SGI393062[] = {{&S100400[25], 14}, {NULL}};
const Segment SGI393063[] = {{&S100400[69], 11}, {NULL}};
const Segment SGI393064[] = {{&S96011[32], 4}, {NULL}};
const Segment SGI393065[] = {{&S96011[8], 3}, {NULL}};
const Segment SGI393066[] = {{&S96011[13], 14}, {NULL}};
const Segment SGI393067[] = {{&S96060[32], 4}, {NULL}};
const Segment SGI393068[] = {{&S96060[8], 3}, {NULL}};
const Segment SGI393069[] = {{&S96060[13], 14}, {NULL}};
const Segment SGI393070[] = {{&S91941[94], 3}, {NULL}};
const Segment SGI393071[] = {{&S91941[29], 7}, {NULL}};
const Segment SGI393072[] = {{&S91941[129], 4}, {NULL}};
const Segment SGI393073[] = {{&S61566[0], 36}, {NULL}};
const Segment SGI393074[] = {{&S61566[65], 4}, {NULL}};
const Segment SGI393075[] = {{&S61566[51], 3}, {NULL}};
const Segment SGI393076[] = {{&S61567[0], 36}, {NULL}};
const Segment SGI393077[] = {{&S61567[65], 10}, {NULL}};
const Segment SGI393078[] = {{&S61567[51], 3}, {NULL}};
const Segment SGI393079[] = {{&S61568[0], 27}, {NULL}};
const Segment SGI393080[] = {{&S61568[56], 7}, {NULL}};
const Segment SGI393081[] = {{&S61568[42], 3}, {NULL}};
const Segment SGI393082[] = {{&S61569[0], 37}, {NULL}};
const Segment SGI393083[] = {{&S61569[66], 7}, {NULL}};
const Segment SGI393084[] = {{&S61569[52], 3}, {NULL}};
const Segment SGI393085[] = {{&S61570[0], 27}, {NULL}};
const Segment SGI393086[] = {{&S61570[56], 4}, {NULL}};
const Segment SGI393087[] = {{&S61570[42], 3}, {NULL}};
const Segment SGI393088[] = {{&S61571[0], 37}, {NULL}};
const Segment SGI393089[] = {{&S61571[66], 10}, {NULL}};
const Segment SGI393090[] = {{&S61571[52], 3}, {NULL}};
const Segment SGI393091[] = {{&S61572[0], 27}, {NULL}};
const Segment SGI393092[] = {{&S61572[56], 4}, {NULL}};
const Segment SGI393093[] = {{&S61572[42], 3}, {NULL}};
const Segment SGI393094[] = {{&S61573[0], 27}, {NULL}};
const Segment SGI393095[] = {{&S61573[56], 7}, {NULL}};
const Segment SGI393096[] = {{&S61573[42], 3}, {NULL}};
const Segment SGI393097[] = {{&S61574[0], 27}, {NULL}};
const Segment SGI393098[] = {{&S61574[56], 4}, {NULL}};
const Segment SGI393099[] = {{&S61574[42], 3}, {NULL}};
const Segment SGI393100[] = {{&S61575[0], 37}, {NULL}};
const Segment SGI393101[] = {{&S61575[66], 4}, {NULL}};
const Segment SGI393102[] = {{&S61575[52], 3}, {NULL}};
const Segment SGI393103[] = {{&S61576[0], 36}, {NULL}};
const Segment SGI393104[] = {{&S61576[65], 7}, {NULL}};
const Segment SGI393105[] = {{&S61576[51], 3}, {NULL}};
const Segment SGI393106[] = {{&S61577[0], 36}, {NULL}};
const Segment SGI393107[] = {{&S61577[65], 7}, {NULL}};
const Segment SGI393108[] = {{&S61577[51], 3}, {NULL}};
const Segment SGI393109[] = {{&S61578[0], 37}, {NULL}};
const Segment SGI393110[] = {{&S61578[66], 7}, {NULL}};
const Segment SGI393111[] = {{&S61578[52], 3}, {NULL}};
const Segment SGI393112[] = {{&S61579[0], 27}, {NULL}};
const Segment SGI393113[] = {{&S61579[56], 4}, {NULL}};
const Segment SGI393114[] = {{&S61579[42], 3}, {NULL}};
const Segment SGI393115[] = {{&S61580[0], 36}, {NULL}};
const Segment SGI393116[] = {{&S61580[65], 4}, {NULL}};
const Segment SGI393117[] = {{&S61580[51], 3}, {NULL}};
const Segment SGI393118[] = {{&S61581[0], 37}, {NULL}};
const Segment SGI393119[] = {{&S61581[66], 7}, {NULL}};
const Segment SGI393120[] = {{&S61581[52], 3}, {NULL}};
const Segment SGI393121[] = {{&S61582[0], 36}, {NULL}};
const Segment SGI393122[] = {{&S61582[65], 10}, {NULL}};
const Segment SGI393123[] = {{&S61582[51], 3}, {NULL}};
const Segment SGI393124[] = {{&S61583[0], 35}, {NULL}};
const Segment SGI393125[] = {{&S61583[64], 7}, {NULL}};
const Segment SGI393126[] = {{&S61583[50], 3}, {NULL}};
const Segment SGI393127[] = {{&S61584[0], 27}, {NULL}};
const Segment SGI393128[] = {{&S61584[56], 4}, {NULL}};
const Segment SGI393129[] = {{&S61584[42], 3}, {NULL}};
const Segment SGI393130[] = {{&S61585[0], 37}, {NULL}};
const Segment SGI393131[] = {{&S61585[66], 7}, {NULL}};
const Segment SGI393132[] = {{&S61585[52], 3}, {NULL}};
const Segment SGI393133[] = {{&S61586[0], 27}, {NULL}};
const Segment SGI393134[] = {{&S61586[56], 4}, {NULL}};
const Segment SGI393135[] = {{&S61586[42], 3}, {NULL}};
const Segment SGI393136[] = {{&S61587[0], 37}, {NULL}};
const Segment SGI393137[] = {{&S61587[66], 10}, {NULL}};
const Segment SGI393138[] = {{&S61587[52], 3}, {NULL}};
const Segment SGI393139[] = {{&S61588[0], 27}, {NULL}};
const Segment SGI393140[] = {{&S61588[56], 4}, {NULL}};
const Segment SGI393141[] = {{&S61588[42], 3}, {NULL}};
const Segment SGI393142[] = {{&S61589[0], 37}, {NULL}};
const Segment SGI393143[] = {{&S61589[66], 4}, {NULL}};
const Segment SGI393144[] = {{&S61589[52], 3}, {NULL}};
const Segment SGI393145[] = {{&S61590[0], 27}, {NULL}};
const Segment SGI393146[] = {{&S61590[56], 7}, {NULL}};
const Segment SGI393147[] = {{&S61590[42], 3}, {NULL}};
const Segment SGI393148[] = {{&S61591[0], 40}, {NULL}};
const Segment SGI393149[] = {{&S61591[69], 7}, {NULL}};
const Segment SGI393150[] = {{&S61591[55], 3}, {NULL}};
const Segment SGI393151[] = {{&S61592[0], 40}, {NULL}};
const Segment SGI393152[] = {{&S61592[69], 4}, {NULL}};
const Segment SGI393153[] = {{&S61592[55], 3}, {NULL}};
const Segment SGI393154[] = {{&S61593[0], 37}, {NULL}};
const Segment SGI393155[] = {{&S61593[66], 4}, {NULL}};
const Segment SGI393156[] = {{&S61593[52], 3}, {NULL}};
const Segment SGI393157[] = {{&S61594[0], 40}, {NULL}};
const Segment SGI393158[] = {{&S61594[69], 10}, {NULL}};
const Segment SGI393159[] = {{&S61594[55], 3}, {NULL}};
const Segment SGI393160[] = {{&S61696[0], 9}, {NULL}};
const Segment SGI393161[] = {{&S61696[41], 4}, {NULL}};
const Segment SGI393162[] = {{&S96377[48], 4}, {NULL}};
const Segment SGI393163[] = {{&S96377[8], 3}, {NULL}};
const Segment SGI393164[] = {{&S96377[13], 9}, {NULL}};
const Segment SGI393165[] = {{&S96377[30], 13}, {NULL}};
const Segment SGI393166[] = {{&S96713[48], 4}, {NULL}};
const Segment SGI393167[] = {{&S96713[8], 3}, {NULL}};
const Segment SGI393168[] = {{&S96713[13], 9}, {NULL}};
const Segment SGI393169[] = {{&S96713[30], 13}, {NULL}};
const Segment SGI393170[] = {{&S100812[8], 3}, {NULL}};
const Segment SGI393171[] = {{&S100812[13], 9}, {NULL}};
const Segment SGI393172[] = {{&S100812[30], 13}, {NULL}};
const Segment SGI393173[] = {{&S100812[63], 11}, {NULL}};
const Segment SGI393174[] = {{&S97066[48], 4}, {NULL}};
const Segment SGI393175[] = {{&S97066[8], 3}, {NULL}};
const Segment SGI393176[] = {{&S97066[13], 9}, {NULL}};
const Segment SGI393177[] = {{&S97066[30], 13}, {NULL}};
const Segment SGI393178[] = {{&S101085[8], 3}, {NULL}};
const Segment SGI393179[] = {{&S101085[13], 9}, {NULL}};
const Segment SGI393180[] = {{&S101085[33], 13}, {NULL}};
const Segment SGI393181[] = {{&S101085[85], 12}, {NULL}};
const Segment SGI393182[] = {{&S62365[0], 7}, {NULL}};
const Segment SGI393183[] = {{&S62365[35], 4}, {NULL}};
const Segment SGI393184[] = {{&S101132[8], 3}, {NULL}};
const Segment SGI393185[] = {{&S101132[13], 17}, {NULL}};
const Segment SGI393186[] = {{&S101132[74], 5}, {NULL}};
const Segment SGI393187[] = {{&S101133[8], 3}, {NULL}};
const Segment SGI393188[] = {{&S101133[13], 17}, {NULL}};
const Segment SGI393189[] = {{&S101133[50], 5}, {NULL}};
const Segment SGI393190[] = {{&S97280[39], 3}, {NULL}};
const Segment SGI393191[] = {{&S97280[8], 3}, {NULL}};
const Segment SGI393192[] = {{&S97280[13], 14}, {NULL}};
const Segment SGI393193[] = {{&S92892[109], 3}, {NULL}};
const Segment SGI393194[] = {{&S92892[44], 7}, {NULL}};
const Segment SGI393195[] = {{&S92892[30], 6}, {NULL}};
const Segment SGI393196[] = {{&S92892[151], 4}, {NULL}};
const Segment SGI393197[] = {{&S97349[32], 4}, {NULL}};
const Segment SGI393198[] = {{&S97349[8], 3}, {NULL}};
const Segment SGI393199[] = {{&S97349[13], 14}, {NULL}};
const Segment SGI393200[] = {{&S62696[0], 9}, {NULL}};
const Segment SGI393201[] = {{&S62696[41], 4}, {NULL}};
const Segment SGI393202[] = {{&S62879[0], 4}, {NULL}};
const Segment SGI393203[] = {{&S62879[52], 10}, {NULL}};
const Segment SGI393204[] = {{&S62879[35], 4}, {NULL}};
const Segment SGI393205[] = {{&S62880[0], 4}, {NULL}};
const Segment SGI393206[] = {{&S62880[52], 4}, {NULL}};
const Segment SGI393207[] = {{&S62880[35], 4}, {NULL}};
const Segment SGI393208[] = {{&S62881[0], 4}, {NULL}};
const Segment SGI393209[] = {{&S62881[52], 10}, {NULL}};
const Segment SGI393210[] = {{&S62881[35], 4}, {NULL}};
const Segment SGI393211[] = {{&S62882[0], 4}, {NULL}};
const Segment SGI393212[] = {{&S62882[52], 7}, {NULL}};
const Segment SGI393213[] = {{&S62882[35], 4}, {NULL}};
const Segment SGI393214[] = {{&S97685[32], 3}, {NULL}};
const Segment SGI393215[] = {{&S97685[8], 3}, {NULL}};
const Segment SGI393216[] = {{&S97685[13], 14}, {NULL}};
const Segment SGI393217[] = {{&S97729[32], 4}, {NULL}};
const Segment SGI393218[] = {{&S97729[8], 3}, {NULL}};
const Segment SGI393219[] = {{&S97729[13], 14}, {NULL}};
const Segment SGI393220[] = {{&S93103[108], 3}, {NULL}};
const Segment SGI393221[] = {{&S93103[44], 6}, {NULL}};
const Segment SGI393222[] = {{&S93103[30], 6}, {NULL}};
const Segment SGI393223[] = {{&S93103[150], 5}, {NULL}};
const Segment SGI393224[] = {{&S101844[8], 3}, {NULL}};
const Segment SGI393225[] = {{&S101844[13], 7}, {NULL}};
const Segment SGI393226[] = {{&S101844[25], 14}, {NULL}};
const Segment SGI393227[] = {{&S101844[77], 12}, {NULL}};
const Segment SGI393228[] = {{&S101845[8], 3}, {NULL}};
const Segment SGI393229[] = {{&S101845[13], 9}, {NULL}};
const Segment SGI393230[] = {{&S101845[33], 13}, {NULL}};
const Segment SGI393231[] = {{&S101845[82], 12}, {NULL}};
const Segment SGI393232[] = {{&S98111[32], 3}, {NULL}};
const Segment SGI393233[] = {{&S98111[8], 3}, {NULL}};
const Segment SGI393234[] = {{&S98111[13], 14}, {NULL}};
const Segment SGI393235[] = {{&S93578[108], 3}, {NULL}};
const Segment SGI393236[] = {{&S93578[44], 6}, {NULL}};
const Segment SGI393237[] = {{&S93578[30], 6}, {NULL}};
const Segment SGI393238[] = {{&S93578[150], 5}, {NULL}};
const Segment SGI393239[] = {{&S63523[0], 7}, {NULL}};
const Segment SGI393240[] = {{&S63523[37], 4}, {NULL}};
const Segment SGI393241[] = {{&S102222[8], 3}, {NULL}};
const Segment SGI393242[] = {{&S102222[13], 7}, {NULL}};
const Segment SGI393243[] = {{&S102222[25], 14}, {NULL}};
const Segment SGI393244[] = {{&S102222[77], 12}, {NULL}};
const Segment SGI393245[] = {{&S102226[8], 3}, {NULL}};
const Segment SGI393246[] = {{&S102226[13], 7}, {NULL}};
const Segment SGI393247[] = {{&S102226[25], 14}, {NULL}};
const Segment SGI393248[] = {{&S102237[8], 3}, {NULL}};
const Segment SGI393249[] = {{&S102237[13], 14}, {NULL}};
const Segment SGI393250[] = {{&S102237[47], 13}, {NULL}};
const Segment SGI393251[] = {{&S102319[8], 3}, {NULL}};
const Segment SGI393252[] = {{&S102319[13], 17}, {NULL}};
const Segment SGI393253[] = {{&S102319[50], 5}, {NULL}};
const Segment SGI393254[] = {{&S102428[8], 3}, {NULL}};
const Segment SGI393255[] = {{&S102428[13], 7}, {NULL}};
const Segment SGI393256[] = {{&S102428[25], 14}, {NULL}};
const Segment SGI393257[] = {{&S102428[64], 12}, {NULL}};
const Segment SGI393258[] = {{&S98711[8], 3}, {NULL}};
const Segment SGI393259[] = {{&S98711[13], 9}, {NULL}};
const Segment SGI393260[] = {{&S98711[30], 13}, {NULL}};
const Segment SGI393261[] = {{&S102545[8], 3}, {NULL}};
const Segment SGI393262[] = {{&S102545[13], 17}, {NULL}};
const Segment SGI393263[] = {{&S102545[74], 5}, {NULL}};
const Segment SGI393264[] = {{&S98792[79], 3}, {NULL}};
const Segment SGI393265[] = {{&S98792[8], 3}, {NULL}};
const Segment SGI393266[] = {{&S98792[13], 7}, {NULL}};
const Segment SGI393267[] = {{&S98792[25], 14}, {NULL}};
const Segment SGI393268[] = {{&S64439[0], 7}, {NULL}};
const Segment SGI393269[] = {{&S64439[33], 4}, {NULL}};
const Segment SGI393270[] = {{&S99069[33], 3}, {NULL}};
const Segment SGI393271[] = {{&S99069[8], 3}, {NULL}};
const Segment SGI393272[] = {{&S99069[13], 3}, {NULL}};
const Segment SGI393273[] = {{&S99078[32], 3}, {NULL}};
const Segment SGI393274[] = {{&S99078[8], 3}, {NULL}};
const Segment SGI393275[] = {{&S99078[13], 14}, {NULL}};
const Segment SGI393276[] = {{&S99081[48], 3}, {NULL}};
const Segment SGI393277[] = {{&S99081[8], 3}, {NULL}};
const Segment SGI393278[] = {{&S99081[13], 9}, {NULL}};
const Segment SGI393279[] = {{&S99081[30], 13}, {NULL}};
const Segment SGI393280[] = {{&S99098[81], 3}, {NULL}};
const Segment SGI393281[] = {{&S99098[8], 3}, {NULL}};
const Segment SGI393282[] = {{&S99098[13], 7}, {NULL}};
const Segment SGI393283[] = {{&S99098[25], 14}, {NULL}};
const Segment SGI393284[] = {{&S99182[68], 3}, {NULL}};
const Segment SGI393285[] = {{&S99182[8], 3}, {NULL}};
const Segment SGI393286[] = {{&S99182[13], 3}, {NULL}};
const Segment SGI393287[] = {{&S99218[81], 3}, {NULL}};
const Segment SGI393288[] = {{&S99218[8], 3}, {NULL}};
const Segment SGI393289[] = {{&S99218[13], 7}, {NULL}};
const Segment SGI393290[] = {{&S99218[25], 14}, {NULL}};
const Segment SGI393291[] = {{&S94759[109], 3}, {NULL}};
const Segment SGI393292[] = {{&S94759[44], 7}, {NULL}};
const Segment SGI393293[] = {{&S94759[30], 6}, {NULL}};
const Segment SGI393294[] = {{&S94759[151], 5}, {NULL}};
const Segment SGI393295[] = {{&S103218[8], 3}, {NULL}};
const Segment SGI393296[] = {{&S103218[13], 17}, {NULL}};
const Segment SGI393297[] = {{&S103218[74], 5}, {NULL}};
const Segment SGI393298[] = {{&S103219[8], 3}, {NULL}};
const Segment SGI393299[] = {{&S103219[13], 17}, {NULL}};
const Segment SGI393300[] = {{&S103219[74], 5}, {NULL}};
const Segment SGI393301[] = {{&S103220[8], 3}, {NULL}};
const Segment SGI393302[] = {{&S103220[13], 17}, {NULL}};
const Segment SGI393303[] = {{&S103220[50], 5}, {NULL}};
const Segment SGI393304[] = {{&S103221[8], 3}, {NULL}};
const Segment SGI393305[] = {{&S103221[13], 17}, {NULL}};
const Segment SGI393306[] = {{&S103221[50], 5}, {NULL}};
const Segment SGI393307[] = {{&S103504[8], 3}, {NULL}};
const Segment SGI393308[] = {{&S103504[13], 17}, {NULL}};
const Segment SGI393309[] = {{&S103504[74], 5}, {NULL}};
const Segment SGI393310[] = {{&S99600[48], 3}, {NULL}};
const Segment SGI393311[] = {{&S99600[8], 3}, {NULL}};
const Segment SGI393312[] = {{&S99600[13], 9}, {NULL}};
const Segment SGI393313[] = {{&S99600[30], 13}, {NULL}};
const Segment SGI393314[] = {{&S99629[8], 3}, {NULL}};
const Segment SGI393315[] = {{&S99629[13], 3}, {NULL}};
const Segment SGI393316[] = {{&S65129[0], 7}, {NULL}};
const Segment SGI393317[] = {{&S65129[39], 4}, {NULL}};
const Segment SGI393318[] = {{&S99825[48], 4}, {NULL}};
const Segment SGI393319[] = {{&S99825[8], 3}, {NULL}};
const Segment SGI393320[] = {{&S99825[13], 9}, {NULL}};
const Segment SGI393321[] = {{&S99825[30], 13}, {NULL}};
const Segment SGI393322[] = {{&S65560[0], 40}, {NULL}};
const Segment SGI393323[] = {{&S65560[73], 10}, {NULL}};
const Segment SGI393324[] = {{&S65560[56], 4}, {NULL}};
const Segment SGI393325[] = {{&S65561[0], 10}, {NULL}};
const Segment SGI393326[] = {{&S65561[61], 10}, {NULL}};
const Segment SGI393327[] = {{&S65561[44], 4}, {NULL}};
const Segment SGI393328[] = {{&S65562[0], 40}, {NULL}};
const Segment SGI393329[] = {{&S65562[73], 10}, {NULL}};
const Segment SGI393330[] = {{&S65562[56], 4}, {NULL}};
const Segment SGI393331[] = {{&S65563[0], 10}, {NULL}};
const Segment SGI393332[] = {{&S65563[61], 10}, {NULL}};
const Segment SGI393333[] = {{&S65563[44], 4}, {NULL}};
const Segment SGI393334[] = {{&S100198[48], 3}, {NULL}};
const Segment SGI393335[] = {{&S100198[8], 3}, {NULL}};
const Segment SGI393336[] = {{&S100198[13], 9}, {NULL}};
const Segment SGI393337[] = {{&S100198[30], 13}, {NULL}};
const Segment SGI393338[] = {{&S100225[39], 3}, {NULL}};
const Segment SGI393339[] = {{&S100225[8], 3}, {NULL}};
const Segment SGI393340[] = {{&S100225[13], 14}, {NULL}};
const Segment SGI393341[] = {{&S66422[0], 7}, {NULL}};
const Segment SGI393342[] = {{&S66422[37], 4}, {NULL}};
const Segment SGI393343[] = {{&S66653[0], 13}, {NULL}};
const Segment SGI393344[] = {{&S66653[45], 4}, {NULL}};
const Segment SGI393345[] = {{&S66654[0], 13}, {NULL}};
const Segment SGI393346[] = {{&S66654[43], 4}, {NULL}};
const Segment SGI393347[] = {{&S66655[0], 9}, {NULL}};
const Segment SGI393348[] = {{&S66655[39], 4}, {NULL}};
const Segment SGI393349[] = {{&S66656[0], 13}, {NULL}};
const Segment SGI393350[] = {{&S66656[46], 4}, {NULL}};
const Segment SGI393351[] = {{&S66657[0], 9}, {NULL}};
const Segment SGI393352[] = {{&S66657[40], 4}, {NULL}};
const Segment SGI393353[] = {{&S66658[0], 8}, {NULL}};
const Segment SGI393354[] = {{&S66658[39], 4}, {NULL}};
const Segment SGI393355[] = {{&S66659[0], 11}, {NULL}};
const Segment SGI393356[] = {{&S66659[44], 4}, {NULL}};
const Segment SGI393357[] = {{&S66679[0], 6}, {NULL}};
const Segment SGI393358[] = {{&S66679[60], 10}, {NULL}};
const Segment SGI393359[] = {{&S66679[43], 4}, {NULL}};
const Segment SGI393360[] = {{&S66680[0], 6}, {NULL}};
const Segment SGI393361[] = {{&S66680[60], 4}, {NULL}};
const Segment SGI393362[] = {{&S66680[43], 4}, {NULL}};
const Segment SGI393363[] = {{&S66681[0], 6}, {NULL}};
const Segment SGI393364[] = {{&S66681[58], 7}, {NULL}};
const Segment SGI393365[] = {{&S66681[43], 4}, {NULL}};
const Segment SGI393366[] = {{&S66682[0], 6}, {NULL}};
const Segment SGI393367[] = {{&S66682[60], 10}, {NULL}};
const Segment SGI393368[] = {{&S66682[43], 4}, {NULL}};
const Segment SGI393369[] = {{&S66683[0], 6}, {NULL}};
const Segment SGI393370[] = {{&S66683[58], 6}, {NULL}};
const Segment SGI393371[] = {{&S66683[43], 4}, {NULL}};
const Segment SGI393372[] = {{&S100811[8], 3}, {NULL}};
const Segment SGI393373[] = {{&S100811[13], 9}, {NULL}};
const Segment SGI393374[] = {{&S100811[30], 13}, {NULL}};
const Segment SGI393375[] = {{&S100813[8], 3}, {NULL}};
const Segment SGI393376[] = {{&S100813[13], 9}, {NULL}};
const Segment SGI393377[] = {{&S100813[30], 13}, {NULL}};
const Segment SGI393378[] = {{&S100814[48], 3}, {NULL}};
const Segment SGI393379[] = {{&S100814[8], 3}, {NULL}};
const Segment SGI393380[] = {{&S100814[13], 9}, {NULL}};
const Segment SGI393381[] = {{&S100814[30], 13}, {NULL}};
const Segment SGI393382[] = {{&S96361[108], 3}, {NULL}};
const Segment SGI393383[] = {{&S96361[44], 6}, {NULL}};
const Segment SGI393384[] = {{&S96361[30], 6}, {NULL}};
const Segment SGI393385[] = {{&S96361[150], 4}, {NULL}};
const Segment SGI393386[] = {{&S66729[0], 6}, {NULL}};
const Segment SGI393387[] = {{&S66729[56], 10}, {NULL}};
const Segment SGI393388[] = {{&S66729[41], 4}, {NULL}};
const Segment SGI393389[] = {{&S66730[0], 6}, {NULL}};
const Segment SGI393390[] = {{&S66730[52], 7}, {NULL}};
const Segment SGI393391[] = {{&S66730[37], 4}, {NULL}};
const Segment SGI393392[] = {{&S100998[32], 3}, {NULL}};
const Segment SGI393393[] = {{&S100998[8], 3}, {NULL}};
const Segment SGI393394[] = {{&S100998[13], 14}, {NULL}};
const Segment SGI393395[] = {{&S101041[8], 3}, {NULL}};
const Segment SGI393396[] = {{&S101041[13], 9}, {NULL}};
const Segment SGI393397[] = {{&S101041[30], 13}, {NULL}};
const Segment SGI393398[] = {{&S101111[35], 4}, {NULL}};
const Segment SGI393399[] = {{&S101111[8], 3}, {NULL}};
const Segment SGI393400[] = {{&S101111[13], 3}, {NULL}};
const Segment SGI393401[] = {{&S101119[48], 4}, {NULL}};
const Segment SGI393402[] = {{&S101119[8], 3}, {NULL}};
const Segment SGI393403[] = {{&S101119[13], 9}, {NULL}};
const Segment SGI393404[] = {{&S101119[30], 13}, {NULL}};
const Segment SGI393405[] = {{&S101343[8], 3}, {NULL}};
const Segment SGI393406[] = {{&S101343[13], 14}, {NULL}};
const Segment SGI393407[] = {{&S96865[109], 3}, {NULL}};
const Segment SGI393408[] = {{&S96865[44], 7}, {NULL}};
const Segment SGI393409[] = {{&S96865[30], 6}, {NULL}};
const Segment SGI393410[] = {{&S96865[151], 5}, {NULL}};
const Segment SGI393411[] = {{&S67716[0], 6}, {NULL}};
const Segment SGI393412[] = {{&S67716[57], 6}, {NULL}};
const Segment SGI393413[] = {{&S67716[42], 4}, {NULL}};
const Segment SGI393414[] = {{&S67717[0], 6}, {NULL}};
const Segment SGI393415[] = {{&S67717[59], 7}, {NULL}};
const Segment SGI393416[] = {{&S67717[42], 4}, {NULL}};
const Segment SGI393417[] = {{&S67718[0], 6}, {NULL}};
const Segment SGI393418[] = {{&S67718[59], 7}, {NULL}};
const Segment SGI393419[] = {{&S67718[42], 4}, {NULL}};
const Segment SGI393420[] = {{&S67719[0], 6}, {NULL}};
const Segment SGI393421[] = {{&S67719[59], 10}, {NULL}};
const Segment SGI393422[] = {{&S67719[42], 4}, {NULL}};
const Segment SGI393423[] = {{&S67720[0], 6}, {NULL}};
const Segment SGI393424[] = {{&S67720[59], 10}, {NULL}};
const Segment SGI393425[] = {{&S67720[42], 4}, {NULL}};
const Segment SGI393426[] = {{&S67721[0], 6}, {NULL}};
const Segment SGI393427[] = {{&S67721[59], 4}, {NULL}};
const Segment SGI393428[] = {{&S67721[42], 4}, {NULL}};
const Segment SGI393429[] = {{&S67804[0], 6}, {NULL}};
const Segment SGI393430[] = {{&S67804[36], 4}, {NULL}};
const Segment SGI393431[] = {{&S68024[0], 6}, {NULL}};
const Segment SGI393432[] = {{&S68024[60], 7}, {NULL}};
const Segment SGI393433[] = {{&S68024[43], 4}, {NULL}};
const Segment SGI393434[] = {{&S68025[0], 6}, {NULL}};
const Segment SGI393435[] = {{&S68025[60], 4}, {NULL}};
const Segment SGI393436[] = {{&S68025[43], 4}, {NULL}};
const Segment SGI393437[] = {{&S68026[0], 6}, {NULL}};
const Segment SGI393438[] = {{&S68026[60], 10}, {NULL}};
const Segment SGI393439[] = {{&S68026[43], 4}, {NULL}};
const Segment SGI393440[] = {{&S68027[0], 6}, {NULL}};
const Segment SGI393441[] = {{&S68027[60], 10}, {NULL}};
const Segment SGI393442[] = {{&S68027[43], 4}, {NULL}};
const Segment SGI393443[] = {{&S97733[108], 3}, {NULL}};
const Segment SGI393444[] = {{&S97733[44], 6}, {NULL}};
const Segment SGI393445[] = {{&S97733[30], 6}, {NULL}};
const Segment SGI393446[] = {{&S97733[150], 5}, {NULL}};
const Segment SGI393447[] = {{&S37042[13], 61}, {NULL}};
const Segment SGI393448[] = {{&S37043[15], 80}, {NULL}};
const Segment SGI393449[] = {{&S37044[15], 56}, {NULL}};
const Segment SGI393450[] = {{&S37045[13], 76}, {NULL}};
const Segment SGI393451[] = {{&S37046[13], 72}, {NULL}};
const Segment SGI393452[] = {{&S37047[13], 76}, {NULL}};
const Segment SGI393453[] = {{&S37048[13], 76}, {NULL}};
const Segment SGI393454[] = {{&S37049[13], 72}, {NULL}};
const Segment SGI393455[] = {{&S37050[13], 65}, {NULL}};
const Segment SGI393456[] = {{&S37051[13], 61}, {NULL}};
const Segment SGI393457[] = {{&S37052[13], 71}, {NULL}};
const Segment SGI393458[] = {{&S37053[13], 61}, {NULL}};
const Segment SGI393459[] = {{&S37054[13], 74}, {NULL}};
const Segment SGI393460[] = {{&S37055[13], 78}, {NULL}};
const Segment SGI393461[] = {{&S37056[13], 57}, {NULL}};
const Segment SGI393462[] = {{&S37057[13], 57}, {NULL}};
const Segment SGI393463[] = {{&S37058[13], 63}, {NULL}};
const Segment SGI393464[] = {{&S37059[13], 78}, {NULL}};
const Segment SGI393465[] = {{&S37060[13], 59}, {NULL}};
const Segment SGI393466[] = {{&S37061[13], 55}, {NULL}};
const Segment SGI393467[] = {{&S37062[13], 59}, {NULL}};
const Segment SGI393468[] = {{&S68881[0], 11}, {NULL}};
const Segment SGI393469[] = {{&S68881[42], 4}, {NULL}};
const Segment SGI393470[] = {{&S68882[0], 4}, {NULL}};
const Segment SGI393471[] = {{&S68882[35], 4}, {NULL}};
const Segment SGI393472[] = {{&S98745[109], 3}, {NULL}};
const Segment SGI393473[] = {{&S98745[44], 7}, {NULL}};
const Segment SGI393474[] = {{&S98745[30], 6}, {NULL}};
const Segment SGI393475[] = {{&S98745[151], 5}, {NULL}};
const Segment SGI393476[] = {{&S69498[0], 17}, {NULL}};
const Segment SGI393477[] = {{&S69498[49], 4}, {NULL}};
const Segment SGI393478[] = {{&S99120[108], 3}, {NULL}};
const Segment SGI393479[] = {{&S99120[44], 6}, {NULL}};
const Segment SGI393480[] = {{&S99120[30], 6}, {NULL}};
const Segment SGI393481[] = {{&S99120[150], 5}, {NULL}};
const Segment SGI393482[] = {{&S69612[0], 7}, {NULL}};
const Segment SGI393483[] = {{&S69612[37], 4}, {NULL}};
const Segment SGI393484[] = {{&S70122[0], 17}, {NULL}};
const Segment SGI393485[] = {{&S70122[45], 4}, {NULL}};
const Segment SGI393486[] = {{&S99923[109], 3}, {NULL}};
const Segment SGI393487[] = {{&S99923[44], 7}, {NULL}};
const Segment SGI393488[] = {{&S99923[30], 6}, {NULL}};
const Segment SGI393489[] = {{&S99923[151], 5}, {NULL}};
const Segment SGI393490[] = {{&S71467[0], 8}, {NULL}};
const Segment SGI393491[] = {{&S71467[41], 4}, {NULL}};
const Segment SGI393492[] = {{&S71468[0], 11}, {NULL}};
const Segment SGI393493[] = {{&S71468[38], 4}, {NULL}};
const Segment SGI393494[] = {{&S71469[0], 18}, {NULL}};
const Segment SGI393495[] = {{&S71469[52], 4}, {NULL}};
const Segment SGI393496[] = {{&S71470[0], 3}, {NULL}};
const Segment SGI393497[] = {{&S71470[32], 4}, {NULL}};
const Segment SGI393498[] = {{&S71471[0], 5}, {NULL}};
const Segment SGI393499[] = {{&S71471[36], 4}, {NULL}};
const Segment SGI393500[] = {{&S71472[0], 10}, {NULL}};
const Segment SGI393501[] = {{&S71472[41], 4}, {NULL}};
const Segment SGI393502[] = {{&S71473[0], 13}, {NULL}};
const Segment SGI393503[] = {{&S71473[44], 4}, {NULL}};
const Segment SGI393504[] = {{&S71474[0], 16}, {NULL}};
const Segment SGI393505[] = {{&S71474[50], 4}, {NULL}};
const Segment SGI393506[] = {{&S71475[0], 10}, {NULL}};
const Segment SGI393507[] = {{&S71475[43], 4}, {NULL}};
const Segment SGI393508[] = {{&S71476[0], 7}, {NULL}};
const Segment SGI393509[] = {{&S71476[41], 4}, {NULL}};
const Segment SGI393510[] = {{&S71477[0], 17}, {NULL}};
const Segment SGI393511[] = {{&S71477[46], 4}, {NULL}};
const Segment SGI393512[] = {{&S71478[0], 18}, {NULL}};
const Segment SGI393513[] = {{&S71478[52], 4}, {NULL}};
const Segment SGI393514[] = {{&S71479[0], 14}, {NULL}};
const Segment SGI393515[] = {{&S71479[48], 4}, {NULL}};
const Segment SGI393516[] = {{&S71480[0], 13}, {NULL}};
const Segment SGI393517[] = {{&S71480[46], 4}, {NULL}};
const Segment SGI393518[] = {{&S71481[0], 17}, {NULL}};
const Segment SGI393519[] = {{&S71481[48], 4}, {NULL}};
const Segment SGI393520[] = {{&S71482[0], 7}, {NULL}};
const Segment SGI393521[] = {{&S71482[40], 4}, {NULL}};
const Segment SGI393522[] = {{&S71483[0], 8}, {NULL}};
const Segment SGI393523[] = {{&S71483[41], 4}, {NULL}};
const Segment SGI393524[] = {{&S71484[0], 12}, {NULL}};
const Segment SGI393525[] = {{&S71484[45], 4}, {NULL}};
const Segment SGI393526[] = {{&S71485[0], 6}, {NULL}};
const Segment SGI393527[] = {{&S71485[37], 4}, {NULL}};
const Segment SGI393528[] = {{&S71486[0], 10}, {NULL}};
const Segment SGI393529[] = {{&S71486[41], 4}, {NULL}};
const Segment SGI393530[] = {{&S72533[0], 4}, {NULL}};
const Segment SGI393531[] = {{&S72533[51], 4}, {NULL}};
const Segment SGI393532[] = {{&S72533[36], 4}, {NULL}};
const Segment SGI393533[] = {{&S72534[0], 4}, {NULL}};
const Segment SGI393534[] = {{&S72534[51], 10}, {NULL}};
const Segment SGI393535[] = {{&S72534[36], 4}, {NULL}};
const Segment SGI393536[] = {{&S72535[0], 4}, {NULL}};
const Segment SGI393537[] = {{&S72535[51], 7}, {NULL}};
const Segment SGI393538[] = {{&S72535[36], 4}, {NULL}};
const Segment SGI393539[] = {{&S72537[0], 6}, {NULL}};
const Segment SGI393540[] = {{&S72537[38], 4}, {NULL}};
const Segment SGI393541[] = {{&S73277[0], 4}, {NULL}};
const Segment SGI393542[] = {{&S73277[33], 4}, {NULL}};
const Segment SGI393543[] = {{&S73278[0], 13}, {NULL}};
const Segment SGI393544[] = {{&S73278[45], 4}, {NULL}};
const Segment SGI393545[] = {{&S73540[0], 4}, {NULL}};
const Segment SGI393546[] = {{&S73540[31], 4}, {NULL}};
const Segment SGI393547[] = {{&S73541[0], 11}, {NULL}};
const Segment SGI393548[] = {{&S73541[38], 4}, {NULL}};
const Segment SGI393549[] = {{&S75328[0], 6}, {NULL}};
const Segment SGI393550[] = {{&S75328[58], 7}, {NULL}};
const Segment SGI393551[] = {{&S75328[43], 4}, {NULL}};
const Segment SGI393552[] = {{&S75329[0], 6}, {NULL}};
const Segment SGI393553[] = {{&S75329[58], 7}, {NULL}};
const Segment SGI393554[] = {{&S75329[43], 4}, {NULL}};
const Segment SGI393555[] = {{&S75330[0], 6}, {NULL}};
const Segment SGI393556[] = {{&S75330[58], 10}, {NULL}};
const Segment SGI393557[] = {{&S75330[43], 4}, {NULL}};
const Segment SGI393558[] = {{&S75331[0], 6}, {NULL}};
const Segment SGI393559[] = {{&S75331[58], 10}, {NULL}};
const Segment SGI393560[] = {{&S75331[43], 4}, {NULL}};
const Segment SGI393561[] = {{&S75332[0], 6}, {NULL}};
const Segment SGI393562[] = {{&S75332[60], 4}, {NULL}};
const Segment SGI393563[] = {{&S75332[43], 4}, {NULL}};
const Segment SGI393564[] = {{&S75333[0], 6}, {NULL}};
const Segment SGI393565[] = {{&S75333[58], 6}, {NULL}};
const Segment SGI393566[] = {{&S75333[43], 4}, {NULL}};
const Segment SGI393567[] = {{&S75769[0], 8}, {NULL}};
const Segment SGI393568[] = {{&S75769[42], 4}, {NULL}};
const Segment SGI393569[] = {{&S76177[0], 10}, {NULL}};
const Segment SGI393570[] = {{&S76177[64], 10}, {NULL}};
const Segment SGI393571[] = {{&S76177[47], 4}, {NULL}};
const Segment SGI393572[] = {{&S76178[0], 23}, {NULL}};
const Segment SGI393573[] = {{&S76178[71], 10}, {NULL}};
const Segment SGI393574[] = {{&S76178[54], 4}, {NULL}};
const Segment SGI393575[] = {{&S76179[0], 10}, {NULL}};
const Segment SGI393576[] = {{&S76179[64], 10}, {NULL}};
const Segment SGI393577[] = {{&S76179[47], 4}, {NULL}};
const Segment SGI393578[] = {{&S76180[0], 16}, {NULL}};
const Segment SGI393579[] = {{&S76180[69], 10}, {NULL}};
const Segment SGI393580[] = {{&S76180[52], 4}, {NULL}};
const Segment SGI393581[] = {{&S76181[0], 33}, {NULL}};
const Segment SGI393582[] = {{&S76181[66], 10}, {NULL}};
const Segment SGI393583[] = {{&S76181[49], 4}, {NULL}};
const Segment SGI393584[] = {{&S76182[0], 10}, {NULL}};
const Segment SGI393585[] = {{&S76182[64], 4}, {NULL}};
const Segment SGI393586[] = {{&S76182[47], 4}, {NULL}};
const Segment SGI393587[] = {{&S76183[0], 40}, {NULL}};
const Segment SGI393588[] = {{&S76183[73], 10}, {NULL}};
const Segment SGI393589[] = {{&S76183[56], 4}, {NULL}};
const Segment SGI393590[] = {{&S76467[0], 8}, {NULL}};
const Segment SGI393591[] = {{&S76467[44], 4}, {NULL}};
const Segment SGI393592[] = {{&S46725[15], 32}, {NULL}};
const Segment SGI393593[] = {{&S46726[15], 32}, {NULL}};
const Segment SGI393594[] = {{&S46727[15], 32}, {NULL}};
const Segment SGI393595[] = {{&S46728[15], 32}, {NULL}};
const Segment SGI393596[] = {{&S46729[13], 32}, {NULL}};
const Segment SGI393597[] = {{&S46730[13], 32}, {NULL}};
const Segment SGI393598[] = {{&S46731[14], 42}, {NULL}};
const Segment SGI393599[] = {{&S46732[15], 32}, {NULL}};
const Segment SGI393600[] = {{&S46733[15], 22}, {NULL}};
const Segment SGI393601[] = {{&S78518[0], 4}, {NULL}};
const Segment SGI393602[] = {{&S78518[52], 10}, {NULL}};
const Segment SGI393603[] = {{&S78518[35], 4}, {NULL}};
const Segment SGI393604[] = {{&S78519[0], 4}, {NULL}};
const Segment SGI393605[] = {{&S78519[52], 10}, {NULL}};
const Segment SGI393606[] = {{&S78519[35], 4}, {NULL}};
const Segment SGI393607[] = {{&S78520[0], 4}, {NULL}};
const Segment SGI393608[] = {{&S78520[52], 4}, {NULL}};
const Segment SGI393609[] = {{&S78520[35], 4}, {NULL}};
const Segment SGI393610[] = {{&S79240[0], 8}, {NULL}};
const Segment SGI393611[] = {{&S79240[41], 4}, {NULL}};
const Segment SGI393612[] = {{&S80377[0], 5}, {NULL}};
const Segment SGI393613[] = {{&S80377[41], 4}, {NULL}};
const Segment SGI393614[] = {{&S48446[15], 78}, {NULL}};
const Segment SGI393615[] = {{&S48447[15], 63}, {NULL}};
const Segment SGI393616[] = {{&S48448[15], 76}, {NULL}};
const Segment SGI393617[] = {{&S48449[13], 80}, {NULL}};
const Segment SGI393618[] = {{&S48450[13], 88}, {NULL}};
const Segment SGI393619[] = {{&S48451[13], 76}, {NULL}};
const Segment SGI393620[] = {{&S48452[13], 76}, {NULL}};
const Segment SGI393621[] = {{&S48453[13], 78}, {NULL}};
const Segment SGI393622[] = {{&S48454[13], 80}, {NULL}};
const Segment SGI393623[] = {{&S48455[13], 65}, {NULL}};
const Segment SGI393624[] = {{&S48456[13], 84}, {NULL}};
const Segment SGI393625[] = {{&S48457[13], 84}, {NULL}};
const Segment SGI393626[] = {{&S48458[15], 88}, {NULL}};
const Segment SGI393627[] = {{&S80462[0], 10}, {NULL}};
const Segment SGI393628[] = {{&S80462[64], 10}, {NULL}};
const Segment SGI393629[] = {{&S80462[47], 4}, {NULL}};
const Segment SGI393630[] = {{&S80463[0], 10}, {NULL}};
const Segment SGI393631[] = {{&S80463[64], 10}, {NULL}};
const Segment SGI393632[] = {{&S80463[47], 4}, {NULL}};
const Segment SGI393633[] = {{&S80464[0], 10}, {NULL}};
const Segment SGI393634[] = {{&S80464[63], 4}, {NULL}};
const Segment SGI393635[] = {{&S80464[46], 4}, {NULL}};
const Segment SGI393636[] = {{&S80465[0], 23}, {NULL}};
const Segment SGI393637[] = {{&S80465[71], 10}, {NULL}};
const Segment SGI393638[] = {{&S80465[54], 4}, {NULL}};
const Segment SGI393639[] = {{&S80466[0], 33}, {NULL}};
const Segment SGI393640[] = {{&S80466[66], 10}, {NULL}};
const Segment SGI393641[] = {{&S80466[49], 4}, {NULL}};
const Segment SGI393642[] = {{&S80467[0], 40}, {NULL}};
const Segment SGI393643[] = {{&S80467[73], 10}, {NULL}};
const Segment SGI393644[] = {{&S80467[56], 4}, {NULL}};
const Segment SGI393645[] = {{&S80468[0], 43}, {NULL}};
const Segment SGI393646[] = {{&S80468[76], 10}, {NULL}};
const Segment SGI393647[] = {{&S80468[59], 4}, {NULL}};
const Segment SGI393648[] = {{&S48802[15], 69}, {NULL}};
const Segment SGI393649[] = {{&S48803[15], 77}, {NULL}};
const Segment SGI393650[] = {{&S48804[13], 69}, {NULL}};
const Segment SGI393651[] = {{&S48805[13], 69}, {NULL}};
const Segment SGI393652[] = {{&S48806[15], 56}, {NULL}};
const Segment SGI393653[] = {{&S48807[15], 69}, {NULL}};
const Segment SGI393654[] = {{&S49406[15], 32}, {NULL}};
const Segment SGI393655[] = {{&S49407[15], 30}, {NULL}};
const Segment SGI393656[] = {{&S49408[15], 32}, {NULL}};
const Segment SGI393657[] = {{&S49409[15], 32}, {NULL}};
const Segment SGI393658[] = {{&S49410[15], 32}, {NULL}};
const Segment SGI393659[] = {{&S49411[13], 32}, {NULL}};
const Segment SGI393660[] = {{&S49412[13], 22}, {NULL}};
const Segment SGI393661[] = {{&S49413[15], 22}, {NULL}};
const Segment SGI393662[] = {{&S81384[0], 4}, {NULL}};
const Segment SGI393663[] = {{&S81384[30], 4}, {NULL}};
const Segment SGI393664[] = {{&S81895[0], 9}, {NULL}};
const Segment SGI393665[] = {{&S81895[47], 4}, {NULL}};
const Segment SGI393666[] = {{&S82178[0], 11}, {NULL}};
const Segment SGI393667[] = {{&S82178[43], 4}, {NULL}};
const Segment SGI393668[] = {{&S82179[0], 19}, {NULL}};
const Segment SGI393669[] = {{&S82179[49], 4}, {NULL}};
const Segment SGI393670[] = {{&S82180[0], 22}, {NULL}};
const Segment SGI393671[] = {{&S82180[53], 4}, {NULL}};
const Segment SGI393672[] = {{&S82382[0], 11}, {NULL}};
const Segment SGI393673[] = {{&S82382[39], 4}, {NULL}};
const Segment SGI393674[] = {{&S82530[0], 6}, {NULL}};
const Segment SGI393675[] = {{&S82530[60], 10}, {NULL}};
const Segment SGI393676[] = {{&S82530[43], 4}, {NULL}};
const Segment SGI393677[] = {{&S82531[0], 6}, {NULL}};
const Segment SGI393678[] = {{&S82531[60], 10}, {NULL}};
const Segment SGI393679[] = {{&S82531[43], 4}, {NULL}};
const Segment SGI393680[] = {{&S82532[0], 6}, {NULL}};
const Segment SGI393681[] = {{&S82532[60], 6}, {NULL}};
const Segment SGI393682[] = {{&S82532[43], 4}, {NULL}};
const Segment SGI393683[] = {{&S82533[0], 6}, {NULL}};
const Segment SGI393684[] = {{&S82533[59], 4}, {NULL}};
const Segment SGI393685[] = {{&S82533[42], 4}, {NULL}};
const Segment SGI393686[] = {{&S82534[0], 6}, {NULL}};
const Segment SGI393687[] = {{&S82534[57], 7}, {NULL}};
const Segment SGI393688[] = {{&S82534[42], 4}, {NULL}};
const Segment SGI393689[] = {{&S82535[0], 6}, {NULL}};
const Segment SGI393690[] = {{&S82535[60], 7}, {NULL}};
const Segment SGI393691[] = {{&S82535[43], 4}, {NULL}};
const Segment SGI393692[] = {{&S50612[15], 30}, {NULL}};
const Segment SGI393693[] = {{&S50613[13], 32}, {NULL}};
const Segment SGI393694[] = {{&S50614[13], 32}, {NULL}};
const Segment SGI393695[] = {{&S50615[13], 20}, {NULL}};
const Segment SGI393696[] = {{&S50616[15], 22}, {NULL}};
const Segment SGI393697[] = {{&S50617[15], 22}, {NULL}};
const Segment SGI393698[] = {{&S50618[15], 22}, {NULL}};
const Segment SGI393699[] = {{&S50619[13], 22}, {NULL}};
const Segment SGI393700[] = {{&S84271[0], 11}, {NULL}};
const Segment SGI393701[] = {{&S84271[43], 4}, {NULL}};
const Segment SGI393702[] = {{&S84272[0], 4}, {NULL}};
const Segment SGI393703[] = {{&S84272[36], 4}, {NULL}};
const Segment SGI393704[] = {{&S84284[0], 8}, {NULL}};
const Segment SGI393705[] = {{&S84284[38], 4}, {NULL}};
const Segment SGI393706[] = {{&S84482[0], 18}, {NULL}};
const Segment SGI393707[] = {{&S84482[47], 4}, {NULL}};
const Segment SGI393708[] = {{&S84483[0], 9}, {NULL}};
const Segment SGI393709[] = {{&S84483[38], 4}, {NULL}};
const Segment SGI393710[] = {{&S84484[0], 13}, {NULL}};
const Segment SGI393711[] = {{&S84484[41], 4}, {NULL}};
const Segment SGI393712[] = {{&S84485[0], 10}, {NULL}};
const Segment SGI393713[] = {{&S84485[36], 4}, {NULL}};
const Segment SGI393714[] = {{&S84486[0], 10}, {NULL}};
const Segment SGI393715[] = {{&S84486[38], 4}, {NULL}};
const Segment SGI393716[] = {{&S84487[0], 11}, {NULL}};
const Segment SGI393717[] = {{&S84487[39], 4}, {NULL}};
const Segment SGI393718[] = {{&S84678[0], 6}, {NULL}};
const Segment SGI393719[] = {{&S84678[60], 10}, {NULL}};
const Segment SGI393720[] = {{&S84678[43], 4}, {NULL}};
const Segment SGI393721[] = {{&S84679[0], 6}, {NULL}};
const Segment SGI393722[] = {{&S84679[60], 10}, {NULL}};
const Segment SGI393723[] = {{&S84679[43], 4}, {NULL}};
const Segment SGI393724[] = {{&S84680[0], 6}, {NULL}};
const Segment SGI393725[] = {{&S84680[60], 6}, {NULL}};
const Segment SGI393726[] = {{&S84680[43], 4}, {NULL}};
const Segment SGI393727[] = {{&S84681[0], 6}, {NULL}};
const Segment SGI393728[] = {{&S84681[60], 7}, {NULL}};
const Segment SGI393729[] = {{&S84681[43], 4}, {NULL}};
const Segment SGI393730[] = {{&S84682[0], 6}, {NULL}};
const Segment SGI393731[] = {{&S84682[60], 4}, {NULL}};
const Segment SGI393732[] = {{&S84682[43], 4}, {NULL}};
const Segment SGI393733[] = {{&S52820[21], 14}, {NULL}};
const Segment SGI393734[] = {{&S52821[21], 14}, {NULL}};
const Segment SGI393735[] = {{&S52822[21], 13}, {NULL}};
const Segment SGI393736[] = {{&S52823[21], 14}, {NULL}};
const Segment SGI393737[] = {{&S52824[21], 14}, {NULL}};
const Segment SGI393738[] = {{&S52825[21], 14}, {NULL}};
const Segment SGI393739[] = {{&S52826[21], 14}, {NULL}};
const Segment SGI393740[] = {{&S52827[21], 13}, {NULL}};
const Segment SGI393741[] = {{&S52828[21], 13}, {NULL}};
const Segment SGI393742[] = {{&S52829[21], 14}, {NULL}};
const Segment SGI393743[] = {{&S52830[21], 14}, {NULL}};
const Segment SGI393744[] = {{&S52831[21], 10}, {NULL}};
const Segment SGI393745[] = {{&S52832[21], 14}, {NULL}};
const Segment SGI393746[] = {{&S52833[21], 14}, {NULL}};
const Segment SGI393747[] = {{&S52834[21], 14}, {NULL}};
const Segment SGI393748[] = {{&S52835[21], 14}, {NULL}};
const Segment SGI393749[] = {{&S52836[21], 14}, {NULL}};
const Segment SGI393750[] = {{&S52837[21], 14}, {NULL}};
const Segment SGI393751[] = {{&S52838[21], 14}, {NULL}};
const Segment SGI393752[] = {{&S52877[21], 4}, {NULL}};
const Segment SGI393753[] = {{&S52878[21], 7}, {NULL}};
const Segment SGI393754[] = {{&S52879[21], 7}, {NULL}};
const Segment SGI393755[] = {{&S52880[21], 11}, {NULL}};
const Segment SGI393756[] = {{&S52881[21], 14}, {NULL}};
const Segment SGI393757[] = {{&S52882[21], 11}, {NULL}};
const Segment SGI393758[] = {{&S52883[21], 7}, {NULL}};
const Segment SGI393759[] = {{&S52884[21], 4}, {NULL}};
const Segment SGI393760[] = {{&S52885[21], 4}, {NULL}};
const Segment SGI393761[] = {{&S52886[21], 11}, {NULL}};
const Segment SGI393762[] = {{&S52887[21], 14}, {NULL}};
const Segment SGI393763[] = {{&S52888[21], 14}, {NULL}};
const Segment SGI393764[] = {{&S52889[21], 14}, {NULL}};
const Segment SGI393765[] = {{&S52890[21], 7}, {NULL}};
const Segment SGI393766[] = {{&S52891[21], 14}, {NULL}};
const Segment SGI393767[] = {{&S52892[21], 14}, {NULL}};
const Segment SGI393768[] = {{&S52893[21], 14}, {NULL}};
const Segment SGI393769[] = {{&S52894[21], 14}, {NULL}};
const Segment SGI393770[] = {{&S52895[21], 14}, {NULL}};
const Segment SGI393771[] = {{&S52896[21], 14}, {NULL}};
const Segment SGI393772[] = {{&S52897[21], 14}, {NULL}};
const Segment SGI393773[] = {{&S52898[21], 14}, {NULL}};
const Segment SGI393774[] = {{&S52899[21], 14}, {NULL}};
const Segment SGI393775[] = {{&S52900[21], 14}, {NULL}};
const Segment SGI393776[] = {{&S52901[21], 14}, {NULL}};
const Segment SGI393777[] = {{&S52902[21], 14}, {NULL}};
const Segment SGI393778[] = {{&S52903[21], 14}, {NULL}};
const Segment SGI393779[] = {{&S52904[21], 14}, {NULL}};
const Segment SGI393780[] = {{&S52905[21], 14}, {NULL}};
const Segment SGI393781[] = {{&S52906[21], 14}, {NULL}};
const Segment SGI393782[] = {{&S85245[0], 22}, {NULL}};
const Segment SGI393783[] = {{&S85245[53], 4}, {NULL}};
const Segment SGI393784[] = {{&S85246[0], 18}, {NULL}};
const Segment SGI393785[] = {{&S85246[51], 4}, {NULL}};
const Segment SGI393786[] = {{&S85247[0], 11}, {NULL}};
const Segment SGI393787[] = {{&S85247[44], 4}, {NULL}};
const Segment SGI393788[] = {{&S85248[0], 14}, {NULL}};
const Segment SGI393789[] = {{&S85248[47], 4}, {NULL}};
const Segment SGI393790[] = {{&S85283[0], 4}, {NULL}};
const Segment SGI393791[] = {{&S85283[52], 10}, {NULL}};
const Segment SGI393792[] = {{&S85283[35], 4}, {NULL}};
const Segment SGI393793[] = {{&S85284[0], 4}, {NULL}};
const Segment SGI393794[] = {{&S85284[52], 4}, {NULL}};
const Segment SGI393795[] = {{&S85284[35], 4}, {NULL}};
const Segment SGI393796[] = {{&S85391[0], 6}, {NULL}};
const Segment SGI393797[] = {{&S85391[34], 4}, {NULL}};
const Segment SGI393798[] = {{&S85392[0], 8}, {NULL}};
const Segment SGI393799[] = {{&S85392[36], 4}, {NULL}};
const Segment SGI393800[] = {{&S85656[0], 9}, {NULL}};
const Segment SGI393801[] = {{&S85656[41], 4}, {NULL}};
const Segment SGI393802[] = {{&S85783[0], 6}, {NULL}};
const Segment SGI393803[] = {{&S85783[59], 10}, {NULL}};
const Segment SGI393804[] = {{&S85783[44], 4}, {NULL}};
const Segment SGI393805[] = {{&S85784[0], 6}, {NULL}};
const Segment SGI393806[] = {{&S85784[60], 7}, {NULL}};
const Segment SGI393807[] = {{&S85784[45], 4}, {NULL}};
const Segment SGI393808[] = {{&S85785[0], 6}, {NULL}};
const Segment SGI393809[] = {{&S85785[59], 4}, {NULL}};
const Segment SGI393810[] = {{&S85785[44], 4}, {NULL}};
const Segment SGI393811[] = {{&S85786[0], 6}, {NULL}};
const Segment SGI393812[] = {{&S85786[59], 10}, {NULL}};
const Segment SGI393813[] = {{&S85786[44], 4}, {NULL}};
const Segment SGI393814[] = {{&S85787[0], 6}, {NULL}};
const Segment SGI393815[] = {{&S85787[59], 6}, {NULL}};
const Segment SGI393816[] = {{&S85787[44], 4}, {NULL}};
const Segment SGI393817[] = {{&S85788[0], 6}, {NULL}};
const Segment SGI393818[] = {{&S85788[59], 7}, {NULL}};
const Segment SGI393819[] = {{&S85788[44], 4}, {NULL}};
const Segment SGI393820[] = {{&S86145[0], 8}, {NULL}};
const Segment SGI393821[] = {{&S86145[44], 4}, {NULL}};
const Segment SGI393822[] = {{&S86462[0], 11}, {NULL}};
const Segment SGI393823[] = {{&S86462[48], 4}, {NULL}};
const Segment SGI393824[] = {{&S86463[0], 9}, {NULL}};
const Segment SGI393825[] = {{&S86463[37], 4}, {NULL}};
const Segment SGI393826[] = {{&S86581[0], 10}, {NULL}};
const Segment SGI393827[] = {{&S86581[59], 10}, {NULL}};
const Segment SGI393828[] = {{&S86581[42], 4}, {NULL}};
const Segment SGI393829[] = {{&S86582[0], 40}, {NULL}};
const Segment SGI393830[] = {{&S86582[71], 10}, {NULL}};
const Segment SGI393831[] = {{&S86582[54], 4}, {NULL}};
const Segment SGI393832[] = {{&S86740[0], 4}, {NULL}};
const Segment SGI393833[] = {{&S86740[52], 10}, {NULL}};
const Segment SGI393834[] = {{&S86740[35], 4}, {NULL}};
const Segment SGI393835[] = {{&S86786[0], 9}, {NULL}};
const Segment SGI393836[] = {{&S86786[41], 4}, {NULL}};
const Segment SGI393837[] = {{&S86836[0], 7}, {NULL}};
const Segment SGI393838[] = {{&S86836[37], 4}, {NULL}};
const Segment SGI393839[] = {{&S86836[0], 11}, {NULL}};
const Segment SGI393840[] = {{&S86914[0], 4}, {NULL}};
const Segment SGI393841[] = {{&S86914[33], 4}, {NULL}};
const Segment SGI393842[] = {{&S54821[13], 27}, {NULL}};
const Segment SGI393843[] = {{&S54822[13], 27}, {NULL}};
const Segment SGI393844[] = {{&S54823[13], 27}, {NULL}};
const Segment SGI393845[] = {{&S54824[13], 27}, {NULL}};
const Segment SGI393846[] = {{&S54825[13], 27}, {NULL}};
const Segment SGI393847[] = {{&S54826[13], 27}, {NULL}};
const Segment SGI393848[] = {{&S54827[13], 27}, {NULL}};
const Segment SGI393849[] = {{&S54828[13], 27}, {NULL}};
const Segment SGI393850[] = {{&S54829[13], 27}, {NULL}};
const Segment SGI393851[] = {{&S54830[15], 17}, {NULL}};
const Segment SGI393852[] = {{&S54831[13], 27}, {NULL}};
const Segment SGI393853[] = {{&S54832[13], 27}, {NULL}};
const Segment SGI393854[] = {{&S54833[13], 27}, {NULL}};
const Segment SGI393855[] = {{&S54834[13], 27}, {NULL}};
const Segment SGI393856[] = {{&S54835[13], 27}, {NULL}};
const Segment SGI393857[] = {{&S54836[13], 28}, {NULL}};
const Segment SGI393858[] = {{&S54837[13], 28}, {NULL}};
const Segment SGI393859[] = {{&S54838[15], 15}, {NULL}};
const Segment SGI393860[] = {{&S54839[13], 27}, {NULL}};
const Segment SGI393861[] = {{&S54963[19], 14}, {NULL}};
const Segment SGI393862[] = {{&S54964[19], 14}, {NULL}};
const Segment SGI393863[] = {{&S54965[21], 14}, {NULL}};
const Segment SGI393864[] = {{&S54966[21], 14}, {NULL}};
const Segment SGI393865[] = {{&S54967[21], 14}, {NULL}};
const Segment SGI393866[] = {{&S54968[21], 14}, {NULL}};
const Segment SGI393867[] = {{&S54969[21], 14}, {NULL}};
const Segment SGI393868[] = {{&S54970[21], 13}, {NULL}};
const Segment SGI393869[] = {{&S54971[21], 14}, {NULL}};
const Segment SGI393870[] = {{&S54972[21], 13}, {NULL}};
const Segment SGI393871[] = {{&S54973[21], 14}, {NULL}};
const Segment SGI393872[] = {{&S54974[21], 14}, {NULL}};
const Segment SGI393873[] = {{&S54975[21], 13}, {NULL}};
const Segment SGI393874[] = {{&S54976[21], 14}, {NULL}};
const Segment SGI393875[] = {{&S54977[19], 14}, {NULL}};
const Segment SGI393876[] = {{&S54978[21], 14}, {NULL}};
const Segment SGI393877[] = {{&S87065[0], 11}, {NULL}};
const Segment SGI393878[] = {{&S87065[43], 4}, {NULL}};
const Segment SGI393879[] = {{&S87300[0], 9}, {NULL}};
const Segment SGI393880[] = {{&S87300[39], 4}, {NULL}};
const Segment SGI393881[] = {{&S87473[0], 7}, {NULL}};
const Segment SGI393882[] = {{&S87473[37], 4}, {NULL}};
const Segment SGI393883[] = {{&S87473[0], 11}, {NULL}};
const Segment SGI393884[] = {{&S87938[0], 6}, {NULL}};
const Segment SGI393885[] = {{&S87938[57], 10}, {NULL}};
const Segment SGI393886[] = {{&S87938[40], 4}, {NULL}};
const Segment SGI393887[] = {{&S87939[0], 6}, {NULL}};
const Segment SGI393888[] = {{&S87939[57], 10}, {NULL}};
const Segment SGI393889[] = {{&S87939[40], 4}, {NULL}};
const Segment SGI393890[] = {{&S87940[0], 6}, {NULL}};
const Segment SGI393891[] = {{&S87940[57], 4}, {NULL}};
const Segment SGI393892[] = {{&S87940[40], 4}, {NULL}};
const Segment SGI393893[] = {{&S88724[0], 9}, {NULL}};
const Segment SGI393894[] = {{&S88724[42], 4}, {NULL}};
const Segment SGI393895[] = {{&S56923[19], 7}, {NULL}};
const Segment SGI393896[] = {{&S56923[7], 8}, {NULL}};
const Segment SGI393897[] = {{&S56924[19], 7}, {NULL}};
const Segment SGI393898[] = {{&S56924[7], 8}, {NULL}};
const Segment SGI393899[] = {{&S56925[19], 6}, {NULL}};
const Segment SGI393900[] = {{&S56925[7], 8}, {NULL}};
const Segment SGI393901[] = {{&S56926[19], 7}, {NULL}};
const Segment SGI393902[] = {{&S56926[7], 8}, {NULL}};
const Segment SGI393903[] = {{&S56927[19], 1}, {NULL}};
const Segment SGI393904[] = {{&S56927[7], 8}, {NULL}};
const Segment SGI393905[] = {{&S56928[19], 6}, {NULL}};
const Segment SGI393906[] = {{&S56928[7], 8}, {NULL}};
const Segment SGI393907[] = {{&S56929[17], 7}, {NULL}};
const Segment SGI393908[] = {{&S56929[7], 8}, {NULL}};
const Segment SGI393909[] = {{&S56930[19], 7}, {NULL}};
const Segment SGI393910[] = {{&S56930[7], 8}, {NULL}};
const Segment SGI393911[] = {{&S56931[19], 7}, {NULL}};
const Segment SGI393912[] = {{&S56931[7], 8}, {NULL}};
const Segment SGI393913[] = {{&S56932[19], 6}, {NULL}};
const Segment SGI393914[] = {{&S56932[7], 8}, {NULL}};
const Segment SGI393915[] = {{&S56933[19], 6}, {NULL}};
const Segment SGI393916[] = {{&S56933[7], 8}, {NULL}};
const Segment SGI393917[] = {{&S56934[19], 7}, {NULL}};
const Segment SGI393918[] = {{&S56934[7], 8}, {NULL}};
const Segment SGI393919[] = {{&S56935[17], 6}, {NULL}};
const Segment SGI393920[] = {{&S56935[7], 8}, {NULL}};
const Segment SGI393921[] = {{&S56936[7], 8}, {NULL}};
const Segment SGI393922[] = {{&S56937[19], 7}, {NULL}};
const Segment SGI393923[] = {{&S56937[7], 8}, {NULL}};
const Segment SGI393924[] = {{&S56938[19], 14}, {NULL}};
const Segment SGI393925[] = {{&S56938[7], 8}, {NULL}};
const Segment SGI393926[] = {{&S56939[19], 14}, {NULL}};
const Segment SGI393927[] = {{&S56939[7], 8}, {NULL}};
const Segment SGI393928[] = {{&S56940[19], 13}, {NULL}};
const Segment SGI393929[] = {{&S56940[7], 8}, {NULL}};
const Segment SGI393930[] = {{&S56941[19], 14}, {NULL}};
const Segment SGI393931[] = {{&S56941[7], 8}, {NULL}};
const Segment SGI393932[] = {{&S56942[19], 14}, {NULL}};
const Segment SGI393933[] = {{&S56942[7], 8}, {NULL}};
const Segment SGI393934[] = {{&S56943[17], 6}, {NULL}};
const Segment SGI393935[] = {{&S56943[7], 8}, {NULL}};
const Segment SGI393936[] = {{&S56944[17], 6}, {NULL}};
const Segment SGI393937[] = {{&S56944[7], 8}, {NULL}};
const Segment SGI393938[] = {{&S56945[17], 14}, {NULL}};
const Segment SGI393939[] = {{&S56945[7], 8}, {NULL}};
const Segment SGI393940[] = {{&S56946[17], 7}, {NULL}};
const Segment SGI393941[] = {{&S56946[7], 8}, {NULL}};
const Segment SGI393942[] = {{&S56947[17], 7}, {NULL}};
const Segment SGI393943[] = {{&S56947[7], 8}, {NULL}};
const Segment SGI393944[] = {{&S56948[19], 7}, {NULL}};
const Segment SGI393945[] = {{&S56948[7], 8}, {NULL}};
const Segment SGI393946[] = {{&S56949[19], 13}, {NULL}};
const Segment SGI393947[] = {{&S56949[7], 8}, {NULL}};
const Segment SGI393948[] = {{&S56950[19], 7}, {NULL}};
const Segment SGI393949[] = {{&S56950[7], 8}, {NULL}};
const Segment SGI393950[] = {{&S56951[19], 14}, {NULL}};
const Segment SGI393951[] = {{&S56951[7], 8}, {NULL}};
const Segment SGI393952[] = {{&S56952[19], 14}, {NULL}};
const Segment SGI393953[] = {{&S56952[7], 8}, {NULL}};
const Segment SGI393954[] = {{&S90215[0], 9}, {NULL}};
const Segment SGI393955[] = {{&S90215[47], 4}, {NULL}};
const Segment SGI393956[] = {{&S58665[19], 14}, {NULL}};
const Segment SGI393957[] = {{&S58666[19], 14}, {NULL}};
const Segment SGI393958[] = {{&S58667[19], 14}, {NULL}};
const Segment SGI393959[] = {{&S58668[19], 14}, {NULL}};
const Segment SGI393960[] = {{&S58669[19], 14}, {NULL}};
const Segment SGI393961[] = {{&S58670[19], 14}, {NULL}};
const Segment SGI393962[] = {{&S58671[19], 14}, {NULL}};
const Segment SGI393963[] = {{&S58672[19], 14}, {NULL}};
const Segment SGI393964[] = {{&S58673[19], 14}, {NULL}};
const Segment SGI393965[] = {{&S58674[19], 14}, {NULL}};
const Segment SGI393966[] = {{&S58675[19], 14}, {NULL}};
const Segment SGI393967[] = {{&S58676[19], 14}, {NULL}};
const Segment SGI393968[] = {{&S60193[15], 32}, {NULL}};
const Segment SGI393969[] = {{&S60194[15], 30}, {NULL}};
const Segment SGI393970[] = {{&S60401[13], 20}, {NULL}};
const Segment SGI393971[] = {{&S60402[13], 20}, {NULL}};
const Segment SGI393972[] = {{&S60403[13], 20}, {NULL}};
const Segment SGI393973[] = {{&S60404[15], 61}, {NULL}};
const Segment SGI393974[] = {{&S60405[15], 65}, {NULL}};
const Segment SGI393975[] = {{&S60406[15], 57}, {NULL}};
const Segment SGI393976[] = {{&S92850[0], 6}, {NULL}};
const Segment SGI393977[] = {{&S92850[58], 7}, {NULL}};
const Segment SGI393978[] = {{&S92850[43], 4}, {NULL}};
const Segment SGI393979[] = {{&S92851[0], 6}, {NULL}};
const Segment SGI393980[] = {{&S92851[58], 10}, {NULL}};
const Segment SGI393981[] = {{&S92851[43], 4}, {NULL}};
const Segment SGI393982[] = {{&S92852[0], 6}, {NULL}};
const Segment SGI393983[] = {{&S92852[58], 4}, {NULL}};
const Segment SGI393984[] = {{&S92852[43], 4}, {NULL}};
const Segment SGI393985[] = {{&S92853[0], 6}, {NULL}};
const Segment SGI393986[] = {{&S92853[58], 10}, {NULL}};
const Segment SGI393987[] = {{&S92853[43], 4}, {NULL}};
const Segment SGI393988[] = {{&S93361[0], 6}, {NULL}};
const Segment SGI393989[] = {{&S93361[59], 7}, {NULL}};
const Segment SGI393990[] = {{&S93361[44], 4}, {NULL}};
const Segment SGI393991[] = {{&S93362[0], 6}, {NULL}};
const Segment SGI393992[] = {{&S93362[59], 10}, {NULL}};
const Segment SGI393993[] = {{&S93362[44], 4}, {NULL}};
const Segment SGI393994[] = {{&S93772[0], 6}, {NULL}};
const Segment SGI393995[] = {{&S93772[58], 10}, {NULL}};
const Segment SGI393996[] = {{&S93772[43], 4}, {NULL}};
const Segment SGI393997[] = {{&S93773[0], 6}, {NULL}};
const Segment SGI393998[] = {{&S93773[57], 7}, {NULL}};
const Segment SGI393999[] = {{&S93773[42], 4}, {NULL}};
const Segment SGI394000[] = {{&S94128[0], 6}, {NULL}};
const Segment SGI394001[] = {{&S94128[34], 4}, {NULL}};
const Segment SGI394002[] = {{&S94444[0], 6}, {NULL}};
const Segment SGI394003[] = {{&S94444[58], 10}, {NULL}};
const Segment SGI394004[] = {{&S94444[41], 4}, {NULL}};
const Segment SGI394005[] = {{&S94445[0], 6}, {NULL}};
const Segment SGI394006[] = {{&S94445[58], 4}, {NULL}};
const Segment SGI394007[] = {{&S94445[41], 4}, {NULL}};
const Segment SGI394008[] = {{&S94446[0], 6}, {NULL}};
const Segment SGI394009[] = {{&S94446[58], 10}, {NULL}};
const Segment SGI394010[] = {{&S94446[41], 4}, {NULL}};
const Segment SGI394011[] = {{&S63034[19], 14}, {NULL}};
const Segment SGI394012[] = {{&S63035[19], 14}, {NULL}};
const Segment SGI394013[] = {{&S63036[19], 14}, {NULL}};
const Segment SGI394014[] = {{&S63037[19], 14}, {NULL}};
const Segment SGI394015[] = {{&S63038[19], 14}, {NULL}};
const Segment SGI394016[] = {{&S63039[19], 14}, {NULL}};
const Segment SGI394017[] = {{&S63040[19], 14}, {NULL}};
const Segment SGI394018[] = {{&S63041[19], 14}, {NULL}};
const Segment SGI394019[] = {{&S63042[19], 14}, {NULL}};
const Segment SGI394020[] = {{&S63043[19], 14}, {NULL}};
const Segment SGI394021[] = {{&S63044[19], 14}, {NULL}};
const Segment SGI394022[] = {{&S95119[0], 4}, {NULL}};
const Segment SGI394023[] = {{&S95119[36], 4}, {NULL}};
const Segment SGI394024[] = {{&S95178[0], 6}, {NULL}};
const Segment SGI394025[] = {{&S95178[60], 10}, {NULL}};
const Segment SGI394026[] = {{&S95178[43], 4}, {NULL}};
const Segment SGI394027[] = {{&S95179[0], 6}, {NULL}};
const Segment SGI394028[] = {{&S95179[60], 7}, {NULL}};
const Segment SGI394029[] = {{&S95179[43], 4}, {NULL}};
const Segment SGI394030[] = {{&S95180[0], 6}, {NULL}};
const Segment SGI394031[] = {{&S95180[60], 4}, {NULL}};
const Segment SGI394032[] = {{&S95180[43], 4}, {NULL}};
const Segment SGI394033[] = {{&S95181[0], 6}, {NULL}};
const Segment SGI394034[] = {{&S95181[60], 10}, {NULL}};
const Segment SGI394035[] = {{&S95181[43], 4}, {NULL}};
const Segment SGI394036[] = {{&S95182[0], 6}, {NULL}};
const Segment SGI394037[] = {{&S95182[60], 6}, {NULL}};
const Segment SGI394038[] = {{&S95182[43], 4}, {NULL}};
const Segment SGI394039[] = {{&S95381[0], 4}, {NULL}};
const Segment SGI394040[] = {{&S95381[37], 4}, {NULL}};
const Segment SGI394041[] = {{&S95382[0], 11}, {NULL}};
const Segment SGI394042[] = {{&S95382[41], 4}, {NULL}};
const Segment SGI394043[] = {{&S95536[0], 9}, {NULL}};
const Segment SGI394044[] = {{&S95536[48], 4}, {NULL}};
const Segment SGI394045[] = {{&S95696[0], 6}, {NULL}};
const Segment SGI394046[] = {{&S95696[58], 4}, {NULL}};
const Segment SGI394047[] = {{&S95696[43], 4}, {NULL}};
const Segment SGI394048[] = {{&S95697[0], 6}, {NULL}};
const Segment SGI394049[] = {{&S95697[58], 10}, {NULL}};
const Segment SGI394050[] = {{&S95697[43], 4}, {NULL}};
const Segment SGI394051[] = {{&S95698[0], 6}, {NULL}};
const Segment SGI394052[] = {{&S95698[58], 6}, {NULL}};
const Segment SGI394053[] = {{&S95698[43], 4}, {NULL}};
const Segment SGI394054[] = {{&S95699[0], 6}, {NULL}};
const Segment SGI394055[] = {{&S95699[58], 10}, {NULL}};
const Segment SGI394056[] = {{&S95699[43], 4}, {NULL}};
const Segment SGI394057[] = {{&S95700[0], 6}, {NULL}};
const Segment SGI394058[] = {{&S95700[58], 7}, {NULL}};
const Segment SGI394059[] = {{&S95700[43], 4}, {NULL}};
const Segment SGI394060[] = {{&S96329[0], 13}, {NULL}};
const Segment SGI394061[] = {{&S96329[44], 4}, {NULL}};
const Segment SGI394062[] = {{&S96330[0], 9}, {NULL}};
const Segment SGI394063[] = {{&S96330[40], 4}, {NULL}};
const Segment SGI394064[] = {{&S96331[0], 8}, {NULL}};
const Segment SGI394065[] = {{&S96331[37], 4}, {NULL}};
const Segment SGI394066[] = {{&S96960[0], 11}, {NULL}};
const Segment SGI394067[] = {{&S96960[42], 4}, {NULL}};
const Segment SGI394068[] = {{&S96961[0], 22}, {NULL}};
const Segment SGI394069[] = {{&S96961[52], 4}, {NULL}};
const Segment SGI394070[] = {{&S96974[0], 17}, {NULL}};
const Segment SGI394071[] = {{&S96974[50], 4}, {NULL}};
const Segment SGI394072[] = {{&S97004[0], 7}, {NULL}};
const Segment SGI394073[] = {{&S97004[33], 4}, {NULL}};
const Segment SGI394074[] = {{&S97004[0], 11}, {NULL}};
const Segment SGI394075[] = {{&S97282[0], 6}, {NULL}};
const Segment SGI394076[] = {{&S97282[39], 4}, {NULL}};
const Segment SGI394077[] = {{&S97285[0], 8}, {NULL}};
const Segment SGI394078[] = {{&S97285[42], 4}, {NULL}};
const Segment SGI394079[] = {{&S98087[0], 9}, {NULL}};
const Segment SGI394080[] = {{&S98087[40], 4}, {NULL}};
const Segment SGI394081[] = {{&S98093[0], 12}, {NULL}};
const Segment SGI394082[] = {{&S98093[43], 3}, {NULL}};
const Segment SGI394083[] = {{&S98133[0], 10}, {NULL}};
const Segment SGI394084[] = {{&S98133[64], 10}, {NULL}};
const Segment SGI394085[] = {{&S98133[47], 4}, {NULL}};
const Segment SGI394086[] = {{&S98598[0], 3}, {NULL}};
const Segment SGI394087[] = {{&S98598[34], 4}, {NULL}};
const Segment SGI394088[] = {{&S98626[0], 5}, {NULL}};
const Segment SGI394089[] = {{&S98626[42], 4}, {NULL}};
const Segment SGI394090[] = {{&S99441[0], 12}, {NULL}};
const Segment SGI394091[] = {{&S99441[46], 4}, {NULL}};
const Segment SGI394092[] = {{&S99585[0], 6}, {NULL}};
const Segment SGI394093[] = {{&S99585[57], 10}, {NULL}};
const Segment SGI394094[] = {{&S99585[42], 4}, {NULL}};
const Segment SGI394095[] = {{&S99586[0], 6}, {NULL}};
const Segment SGI394096[] = {{&S99586[57], 7}, {NULL}};
const Segment SGI394097[] = {{&S99586[42], 4}, {NULL}};
const Segment SGI394098[] = {{&S99613[0], 6}, {NULL}};
const Segment SGI394099[] = {{&S99613[59], 4}, {NULL}};
const Segment SGI394100[] = {{&S99613[42], 4}, {NULL}};
const Segment SGI394101[] = {{&S99614[0], 6}, {NULL}};
const Segment SGI394102[] = {{&S99614[59], 10}, {NULL}};
const Segment SGI394103[] = {{&S99614[42], 4}, {NULL}};
const Segment SGI394104[] = {{&S99615[0], 6}, {NULL}};
const Segment SGI394105[] = {{&S99615[59], 10}, {NULL}};
const Segment SGI394106[] = {{&S99615[42], 4}, {NULL}};
const Segment SGI394107[] = {{&S99616[0], 6}, {NULL}};
const Segment SGI394108[] = {{&S99616[57], 7}, {NULL}};
const Segment SGI394109[] = {{&S99616[42], 4}, {NULL}};
const Segment SGI394110[] = {{&S99617[0], 6}, {NULL}};
const Segment SGI394111[] = {{&S99617[57], 6}, {NULL}};
const Segment SGI394112[] = {{&S99617[42], 4}, {NULL}};
const Segment SGI394113[] = {{&S100324[0], 9}, {NULL}};
const Segment SGI394114[] = {{&S100324[37], 4}, {NULL}};
const Segment SGI394115[] = {{&S100475[0], 7}, {NULL}};
const Segment SGI394116[] = {{&S100475[37], 4}, {NULL}};
const Segment SGI394117[] = {{&S100475[0], 11}, {NULL}};
const Segment SGI394118[] = {{&S100547[0], 11}, {NULL}};
const Segment SGI394119[] = {{&S100547[42], 4}, {NULL}};
const Segment SGI394120[] = {{&S69220[19], 13}, {NULL}};
const Segment SGI394121[] = {{&S69220[7], 8}, {NULL}};
const Segment SGI394122[] = {{&S69221[19], 14}, {NULL}};
const Segment SGI394123[] = {{&S69221[7], 8}, {NULL}};
const Segment SGI394124[] = {{&S69222[19], 14}, {NULL}};
const Segment SGI394125[] = {{&S69222[7], 8}, {NULL}};
const Segment SGI394126[] = {{&S69223[19], 14}, {NULL}};
const Segment SGI394127[] = {{&S69223[7], 8}, {NULL}};
const Segment SGI394128[] = {{&S69224[19], 14}, {NULL}};
const Segment SGI394129[] = {{&S69224[7], 8}, {NULL}};
const Segment SGI394130[] = {{&S69225[19], 14}, {NULL}};
const Segment SGI394131[] = {{&S69225[7], 8}, {NULL}};
const Segment SGI394132[] = {{&S101001[0], 15}, {NULL}};
const Segment SGI394133[] = {{&S101001[60], 10}, {NULL}};
const Segment SGI394134[] = {{&S101001[43], 4}, {NULL}};
const Segment SGI394135[] = {{&S101002[0], 15}, {NULL}};
const Segment SGI394136[] = {{&S101002[60], 10}, {NULL}};
const Segment SGI394137[] = {{&S101002[43], 4}, {NULL}};
const Segment SGI394138[] = {{&S101003[0], 15}, {NULL}};
const Segment SGI394139[] = {{&S101003[60], 4}, {NULL}};
const Segment SGI394140[] = {{&S101003[43], 4}, {NULL}};
const Segment SGI394141[] = {{&S101004[0], 15}, {NULL}};
const Segment SGI394142[] = {{&S101004[60], 7}, {NULL}};
const Segment SGI394143[] = {{&S101004[43], 4}, {NULL}};
const Segment SGI394144[] = {{&S101005[0], 15}, {NULL}};
const Segment SGI394145[] = {{&S101005[60], 6}, {NULL}};
const Segment SGI394146[] = {{&S101005[43], 4}, {NULL}};
const Segment SGI394147[] = {{&S101034[0], 6}, {NULL}};
const Segment SGI394148[] = {{&S101034[52], 10}, {NULL}};
const Segment SGI394149[] = {{&S101034[37], 4}, {NULL}};
const Segment SGI394150[] = {{&S101196[0], 9}, {NULL}};
const Segment SGI394151[] = {{&S101196[42], 4}, {NULL}};
const Segment SGI394152[] = {{&S101223[0], 17}, {NULL}};
const Segment SGI394153[] = {{&S101223[46], 4}, {NULL}};
const Segment SGI394154[] = {{&S101223[32], 3}, {NULL}};
const Segment SGI394155[] = {{&S101340[0], 5}, {NULL}};
const Segment SGI394156[] = {{&S101340[35], 4}, {NULL}};
const Segment SGI394157[] = {{&S101344[0], 9}, {NULL}};
const Segment SGI394158[] = {{&S101344[40], 4}, {NULL}};
const Segment SGI394159[] = {{&S101345[0], 13}, {NULL}};
const Segment SGI394160[] = {{&S101345[46], 4}, {NULL}};
const Segment SGI394161[] = {{&S101346[0], 6}, {NULL}};
const Segment SGI394162[] = {{&S101346[38], 4}, {NULL}};
const Segment SGI394163[] = {{&S101545[0], 6}, {NULL}};
const Segment SGI394164[] = {{&S101545[59], 10}, {NULL}};
const Segment SGI394165[] = {{&S101545[44], 4}, {NULL}};
const Segment SGI394166[] = {{&S101857[0], 9}, {NULL}};
const Segment SGI394167[] = {{&S101857[41], 4}, {NULL}};
const Segment SGI394168[] = {{&S101876[0], 15}, {NULL}};
const Segment SGI394169[] = {{&S101876[58], 10}, {NULL}};
const Segment SGI394170[] = {{&S101876[41], 4}, {NULL}};
const Segment SGI394171[] = {{&S101877[0], 15}, {NULL}};
const Segment SGI394172[] = {{&S101877[58], 10}, {NULL}};
const Segment SGI394173[] = {{&S101877[41], 4}, {NULL}};
const Segment SGI394174[] = {{&S101878[0], 15}, {NULL}};
const Segment SGI394175[] = {{&S101878[58], 4}, {NULL}};
const Segment SGI394176[] = {{&S101878[41], 4}, {NULL}};
const Segment SGI394177[] = {{&S102080[0], 6}, {NULL}};
const Segment SGI394178[] = {{&S102080[55], 10}, {NULL}};
const Segment SGI394179[] = {{&S102080[38], 4}, {NULL}};
const Segment SGI394180[] = {{&S102081[0], 6}, {NULL}};
const Segment SGI394181[] = {{&S102081[55], 10}, {NULL}};
const Segment SGI394182[] = {{&S102081[38], 4}, {NULL}};
const Segment SGI394183[] = {{&S102417[0], 9}, {NULL}};
const Segment SGI394184[] = {{&S102417[42], 4}, {NULL}};
const Segment SGI394185[] = {{&S102429[0], 6}, {NULL}};
const Segment SGI394186[] = {{&S102429[58], 7}, {NULL}};
const Segment SGI394187[] = {{&S102429[43], 4}, {NULL}};
const Segment SGI394188[] = {{&S102430[0], 6}, {NULL}};
const Segment SGI394189[] = {{&S102430[60], 10}, {NULL}};
const Segment SGI394190[] = {{&S102430[43], 4}, {NULL}};
const Segment SGI394191[] = {{&S102455[0], 4}, {NULL}};
const Segment SGI394192[] = {{&S102455[52], 10}, {NULL}};
const Segment SGI394193[] = {{&S102455[35], 4}, {NULL}};
const Segment SGI394194[] = {{&S102456[0], 4}, {NULL}};
const Segment SGI394195[] = {{&S102456[52], 4}, {NULL}};
const Segment SGI394196[] = {{&S102456[35], 4}, {NULL}};
const Segment SGI394197[] = {{&S102467[0], 15}, {NULL}};
const Segment SGI394198[] = {{&S102467[60], 10}, {NULL}};
const Segment SGI394199[] = {{&S102467[43], 4}, {NULL}};
const Segment SGI394200[] = {{&S102468[0], 15}, {NULL}};
const Segment SGI394201[] = {{&S102468[60], 10}, {NULL}};
const Segment SGI394202[] = {{&S102468[43], 4}, {NULL}};
const Segment SGI394203[] = {{&S102847[0], 15}, {NULL}};
const Segment SGI394204[] = {{&S102847[60], 10}, {NULL}};
const Segment SGI394205[] = {{&S102847[43], 4}, {NULL}};
const Segment SGI394206[] = {{&S102848[0], 15}, {NULL}};
const Segment SGI394207[] = {{&S102848[60], 10}, {NULL}};
const Segment SGI394208[] = {{&S102848[43], 4}, {NULL}};
const Segment SGI394209[] = {{&S103340[0], 9}, {NULL}};
const Segment SGI394210[] = {{&S103340[42], 4}, {NULL}};
const Segment SGI394211[] = {{&S103363[0], 4}, {NULL}};
const Segment SGI394212[] = {{&S103363[52], 10}, {NULL}};
const Segment SGI394213[] = {{&S103363[35], 4}, {NULL}};
const Segment SGI394214[] = {{&S103366[0], 23}, {NULL}};
const Segment SGI394215[] = {{&S103366[52], 7}, {NULL}};
const Segment SGI394216[] = {{&S103366[38], 3}, {NULL}};
const Segment SGI394217[] = {{&S103655[0], 6}, {NULL}};
const Segment SGI394218[] = {{&S103655[60], 10}, {NULL}};
const Segment SGI394219[] = {{&S103655[43], 4}, {NULL}};
const Segment SGI394220[] = {{&S103656[0], 6}, {NULL}};
const Segment SGI394221[] = {{&S103656[60], 10}, {NULL}};
const Segment SGI394222[] = {{&S103656[43], 4}, {NULL}};
const Segment SGI394223[] = {{&S103657[0], 6}, {NULL}};
const Segment SGI394224[] = {{&S103657[60], 4}, {NULL}};
const Segment SGI394225[] = {{&S103657[43], 4}, {NULL}};
const Segment SGI394226[] = {{&S103675[0], 6}, {NULL}};
const Segment SGI394227[] = {{&S103675[60], 10}, {NULL}};
const Segment SGI394228[] = {{&S103675[43], 4}, {NULL}};
const Segment SGI394229[] = {{&S72508[17], 18}, {NULL}};
const Segment SGI394230[] = {{&S103970[0], 9}, {NULL}};
const Segment SGI394231[] = {{&S103970[42], 4}, {NULL}};
const Segment SGI394232[] = {{&S103985[0], 23}, {NULL}};
const Segment SGI394233[] = {{&S103985[71], 10}, {NULL}};
const Segment SGI394234[] = {{&S103985[56], 4}, {NULL}};
const Segment SGI394235[] = {{&S104322[0], 9}, {NULL}};
const Segment SGI394236[] = {{&S104322[42], 4}, {NULL}};
const Segment SGI394237[] = {{&S104349[0], 6}, {NULL}};
const Segment SGI394238[] = {{&S104349[37], 4}, {NULL}};
const Segment SGI394239[] = {{&S104793[0], 7}, {NULL}};
const Segment SGI394240[] = {{&S104793[38], 4}, {NULL}};
const Segment SGI394241[] = {{&S104793[0], 11}, {NULL}};
const Segment SGI394242[] = {{&S104796[0], 6}, {NULL}};
const Segment SGI394243[] = {{&S104796[38], 4}, {NULL}};
const Segment SGI394244[] = {{&S104840[0], 6}, {NULL}};
const Segment SGI394245[] = {{&S104840[37], 4}, {NULL}};
const Segment SGI394246[] = {{&S74066[8], 15}, {NULL}};
const Segment SGI394247[] = {{&S105468[0], 10}, {NULL}};
const Segment SGI394248[] = {{&S105468[64], 10}, {NULL}};
const Segment SGI394249[] = {{&S105468[47], 4}, {NULL}};
const Segment SGI394250[] = {{&S105482[0], 7}, {NULL}};
const Segment SGI394251[] = {{&S105482[39], 4}, {NULL}};
const Segment SGI394252[] = {{&S105482[0], 11}, {NULL}};
const Segment SGI394253[] = {{&S105495[0], 7}, {NULL}};
const Segment SGI394254[] = {{&S105495[37], 4}, {NULL}};
const Segment SGI394255[] = {{&S105495[0], 11}, {NULL}};
const Segment SGI394256[] = {{&S105496[0], 7}, {NULL}};
const Segment SGI394257[] = {{&S105496[38], 4}, {NULL}};
const Segment SGI394258[] = {{&S105496[0], 11}, {NULL}};
const Segment SGI394259[] = {{&S105499[0], 7}, {NULL}};
const Segment SGI394260[] = {{&S105499[40], 4}, {NULL}};
const Segment SGI394261[] = {{&S105499[0], 11}, {NULL}};
const Segment SGI394262[] = {{&S105501[0], 7}, {NULL}};
const Segment SGI394263[] = {{&S105501[37], 4}, {NULL}};
const Segment SGI394264[] = {{&S105501[0], 11}, {NULL}};
const Segment SGI394265[] = {{&S105518[0], 6}, {NULL}};
const Segment SGI394266[] = {{&S105518[60], 10}, {NULL}};
const Segment SGI394267[] = {{&S105518[43], 4}, {NULL}};
const Segment SGI394268[] = {{&S105555[0], 26}, {NULL}};
const Segment SGI394269[] = {{&S105555[55], 4}, {NULL}};
const Segment SGI394270[] = {{&S105555[41], 3}, {NULL}};
const Segment SGI394271[] = {{&S106034[0], 7}, {NULL}};
const Segment SGI394272[] = {{&S106034[37], 4}, {NULL}};
const Segment SGI394273[] = {{&S106034[0], 11}, {NULL}};
const Segment SGI394274[] = {{&S79529[14], 10}, {NULL}};
const Segment SGI394275[] = {{&S80816[12], 31}, {NULL}};
const Segment SGI394276[] = {{&S82536[14], 31}, {NULL}};
const Segment SGI394277[] = {{&S82537[14], 33}, {NULL}};
const Segment SGI394278[] = {{&S92688[15], 18}, {NULL}};
const Segment SGI394279[] = {{&S93584[17], 18}, {NULL}};
const Segment SGI394280[] = {{&S104802[17], 18}, {NULL}};
const Segment SGI394281[] = {{&S1933[0], 11}, {NULL}};
const Segment SGI394282[] = {{&S4845[0], 36}, {NULL}};
const Segment SGI394283[] = {{&S2407[57], 9}, {NULL}};
const Segment SGI394284[] = {{&S2408[57], 9}, {NULL}};
const Segment SGI394285[] = {{&S2409[57], 9}, {NULL}};
const Segment SGI394286[] = {{&S2413[57], 9}, {NULL}};
const Segment SGI394287[] = {{&S2420[57], 9}, {NULL}};
const Segment SGI394288[] = {{&S2445[57], 9}, {NULL}};
const Segment SGI394289[] = {{&S2473[57], 9}, {NULL}};
const Segment SGI394290[] = {{&S2481[57], 9}, {NULL}};
const Segment SGI394291[] = {{&S2503[57], 9}, {NULL}};
const Segment SGI394292[] = {{&S2589[57], 9}, {NULL}};
const Segment SGI394293[] = {{&S2616[57], 9}, {NULL}};
const Segment SGI394294[] = {{&S11145[0], 18}, {NULL}};
const Segment SGI394295[] = {{&S11146[0], 18}, {NULL}};
const Segment SGI394296[] = {{&S11148[0], 18}, {NULL}};
const Segment SGI394297[] = {{&S11149[0], 11}, {NULL}};
const Segment SGI394298[] = {{&S11150[0], 18}, {NULL}};
const Segment SGI394299[] = {{&S11151[0], 18}, {NULL}};
const Segment SGI394300[] = {{&S11396[0], 41}, {NULL}};
const Segment SGI394301[] = {{&S12099[57], 9}, {NULL}};
const Segment SGI394302[] = {{&S12100[57], 9}, {NULL}};
const Segment SGI394303[] = {{&S12102[57], 9}, {NULL}};
const Segment SGI394304[] = {{&S12103[57], 9}, {NULL}};
const Segment SGI394305[] = {{&S12107[57], 9}, {NULL}};
const Segment SGI394306[] = {{&S12117[57], 9}, {NULL}};
const Segment SGI394307[] = {{&S12119[57], 9}, {NULL}};
const Segment SGI394308[] = {{&S12125[57], 9}, {NULL}};
const Segment SGI394309[] = {{&S12136[57], 9}, {NULL}};
const Segment SGI394310[] = {{&S12140[57], 9}, {NULL}};
const Segment SGI394311[] = {{&S18199[0], 15}, {NULL}};
const Segment SGI394312[] = {{&S18206[0], 34}, {NULL}};
const Segment SGI394313[] = {{&S18208[0], 34}, {NULL}};
const Segment SGI394314[] = {{&S18213[0], 31}, {NULL}};
const Segment SGI394315[] = {{&S18474[0], 36}, {NULL}};
const Segment SGI394316[] = {{&S18475[0], 36}, {NULL}};
const Segment SGI394317[] = {{&S18480[0], 37}, {NULL}};
const Segment SGI394318[] = {{&S18481[0], 14}, {NULL}};
const Segment SGI394319[] = {{&S18484[0], 30}, {NULL}};
const Segment SGI394320[] = {{&S17498[57], 9}, {NULL}};
const Segment SGI394321[] = {{&S17513[57], 9}, {NULL}};
const Segment SGI394322[] = {{&S17526[57], 9}, {NULL}};
const Segment SGI394323[] = {{&S17532[57], 9}, {NULL}};
const Segment SGI394324[] = {{&S17553[57], 9}, {NULL}};
const Segment SGI394325[] = {{&S17598[57], 9}, {NULL}};
const Segment SGI394326[] = {{&S17637[57], 9}, {NULL}};
const Segment SGI394327[] = {{&S17682[57], 9}, {NULL}};
const Segment SGI394328[] = {{&S18733[57], 9}, {NULL}};
const Segment SGI394329[] = {{&S18739[57], 9}, {NULL}};
const Segment SGI394330[] = {{&S18745[57], 9}, {NULL}};
const Segment SGI394331[] = {{&S18753[57], 9}, {NULL}};
const Segment SGI394332[] = {{&S18760[57], 9}, {NULL}};
const Segment SGI394333[] = {{&S18784[57], 9}, {NULL}};
const Segment SGI394334[] = {{&S18801[57], 9}, {NULL}};
const Segment SGI394335[] = {{&S18831[57], 9}, {NULL}};
const Segment SGI394336[] = {{&S23689[0], 24}, {NULL}};
const Segment SGI394337[] = {{&S19392[57], 9}, {NULL}};
const Segment SGI394338[] = {{&S19400[57], 9}, {NULL}};
const Segment SGI394339[] = {{&S19401[57], 9}, {NULL}};
const Segment SGI394340[] = {{&S19414[57], 9}, {NULL}};
const Segment SGI394341[] = {{&S19420[57], 9}, {NULL}};
const Segment SGI394342[] = {{&S19421[57], 9}, {NULL}};
const Segment SGI394343[] = {{&S19450[57], 9}, {NULL}};
const Segment SGI394344[] = {{&S19461[57], 9}, {NULL}};
const Segment SGI394345[] = {{&S19468[57], 9}, {NULL}};
const Segment SGI394346[] = {{&S31806[0], 20}, {NULL}};
const Segment SGI394347[] = {{&S26134[6], 4}, {NULL}};
const Segment SGI394348[] = {{&S26134[118], 5}, {NULL}};
const Segment SGI394349[] = {{&S26134[12], 14}, {&S26134[51], 10}, {NULL}};
const Segment SGI394350[] = {{&S26154[6], 4}, {NULL}};
const Segment SGI394351[] = {{&S26154[97], 5}, {NULL}};
const Segment SGI394352[] = {{&S26154[12], 14}, {NULL}};
const Segment SGI394353[] = {{&S34707[57], 9}, {NULL}};
const Segment SGI394354[] = {{&S34708[57], 9}, {NULL}};
const Segment SGI394355[] = {{&S34709[57], 9}, {NULL}};
const Segment SGI394356[] = {{&S34711[57], 9}, {NULL}};
const Segment SGI394357[] = {{&S34713[57], 9}, {NULL}};
const Segment SGI394358[] = {{&S34721[57], 9}, {NULL}};
const Segment SGI394359[] = {{&S34729[57], 9}, {NULL}};
const Segment SGI394360[] = {{&S34747[57], 9}, {NULL}};
const Segment SGI394361[] = {{&S31035[6], 4}, {NULL}};
const Segment SGI394362[] = {{&S31035[111], 5}, {NULL}};
const Segment SGI394363[] = {{&S31035[12], 14}, {&S31035[48], 10}, {NULL}};
const Segment SGI394364[] = {{&S31057[6], 4}, {NULL}};
const Segment SGI394365[] = {{&S31057[90], 5}, {NULL}};
const Segment SGI394366[] = {{&S31057[12], 14}, {NULL}};
const Segment SGI394367[] = {{&S31480[6], 4}, {NULL}};
const Segment SGI394368[] = {{&S31480[111], 5}, {NULL}};
const Segment SGI394369[] = {{&S31480[12], 14}, {&S31480[48], 10}, {NULL}};
const Segment SGI394370[] = {{&S31510[6], 4}, {NULL}};
const Segment SGI394371[] = {{&S31510[90], 5}, {NULL}};
const Segment SGI394372[] = {{&S31510[12], 14}, {NULL}};
const Segment SGI394373[] = {{&S32773[6], 4}, {NULL}};
const Segment SGI394374[] = {{&S32773[118], 5}, {NULL}};
const Segment SGI394375[] = {{&S32773[12], 14}, {&S32773[51], 10}, {NULL}};
const Segment SGI394376[] = {{&S32777[6], 4}, {NULL}};
const Segment SGI394377[] = {{&S32777[97], 5}, {NULL}};
const Segment SGI394378[] = {{&S32777[12], 14}, {NULL}};
const Segment SGI394379[] = {{&S32787[6], 4}, {NULL}};
const Segment SGI394380[] = {{&S32787[111], 5}, {NULL}};
const Segment SGI394381[] = {{&S32787[12], 14}, {&S32787[44], 10}, {NULL}};
const Segment SGI394382[] = {{&S42289[0], 36}, {NULL}};
const Segment SGI394383[] = {{&S33904[6], 4}, {NULL}};
const Segment SGI394384[] = {{&S33904[111], 5}, {NULL}};
const Segment SGI394385[] = {{&S33904[12], 14}, {&S33904[48], 10}, {NULL}};
const Segment SGI394386[] = {{&S35490[6], 4}, {NULL}};
const Segment SGI394387[] = {{&S35490[111], 5}, {NULL}};
const Segment SGI394388[] = {{&S35490[12], 14}, {&S35490[48], 10}, {NULL}};
const Segment SGI394389[] = {{&S35491[6], 4}, {NULL}};
const Segment SGI394390[] = {{&S35491[97], 5}, {NULL}};
const Segment SGI394391[] = {{&S35491[12], 14}, {NULL}};
const Segment SGI394392[] = {{&S36799[6], 4}, {NULL}};
const Segment SGI394393[] = {{&S36799[111], 5}, {NULL}};
const Segment SGI394394[] = {{&S36799[12], 14}, {&S36799[48], 10}, {NULL}};
const Segment SGI394395[] = {{&S37186[6], 4}, {NULL}};
const Segment SGI394396[] = {{&S37186[111], 5}, {NULL}};
const Segment SGI394397[] = {{&S37186[12], 14}, {&S37186[48], 10}, {NULL}};
const Segment SGI394398[] = {{&S37196[6], 4}, {NULL}};
const Segment SGI394399[] = {{&S37196[97], 5}, {NULL}};
const Segment SGI394400[] = {{&S37196[12], 14}, {NULL}};
const Segment SGI394401[] = {{&S40290[6], 4}, {NULL}};
const Segment SGI394402[] = {{&S40290[111], 5}, {NULL}};
const Segment SGI394403[] = {{&S40290[12], 14}, {&S40290[44], 10}, {NULL}};
const Segment SGI394404[] = {{&S43366[6], 4}, {NULL}};
const Segment SGI394405[] = {{&S43366[111], 5}, {NULL}};
const Segment SGI394406[] = {{&S43366[12], 14}, {&S43366[44], 10}, {NULL}};
const Segment SGI394407[] = {{&S44633[6], 4}, {NULL}};
const Segment SGI394408[] = {{&S44633[117], 5}, {NULL}};
const Segment SGI394409[] = {{&S44633[12], 14}, {&S44633[50], 10}, {NULL}};
const Segment SGI394410[] = {{&S49006[0], 3}, {NULL}};
const Segment SGI394411[] = {{&S49006[32], 8}, {NULL}};
const Segment SGI394412[] = {{&S49006[22], 7}, {NULL}};
const Segment SGI394413[] = {{&S49006[6], 7}, {NULL}};
const Segment SGI394414[] = {{&S49007[0], 3}, {NULL}};
const Segment SGI394415[] = {{&S49007[32], 8}, {NULL}};
const Segment SGI394416[] = {{&S49007[22], 7}, {NULL}};
const Segment SGI394417[] = {{&S49007[6], 7}, {NULL}};
const Segment SGI394418[] = {{&S49008[0], 3}, {NULL}};
const Segment SGI394419[] = {{&S49008[30], 8}, {NULL}};
const Segment SGI394420[] = {{&S49008[20], 7}, {NULL}};
const Segment SGI394421[] = {{&S49008[6], 7}, {NULL}};
const Segment SGI394422[] = {{&S49009[0], 3}, {NULL}};
const Segment SGI394423[] = {{&S49009[32], 8}, {NULL}};
const Segment SGI394424[] = {{&S49009[22], 7}, {NULL}};
const Segment SGI394425[] = {{&S49009[6], 7}, {NULL}};
const Segment SGI394426[] = {{&S49018[0], 3}, {NULL}};
const Segment SGI394427[] = {{&S49018[32], 8}, {NULL}};
const Segment SGI394428[] = {{&S49018[22], 7}, {NULL}};
const Segment SGI394429[] = {{&S49018[6], 7}, {NULL}};
const Segment SGI394430[] = {{&S50541[0], 3}, {NULL}};
const Segment SGI394431[] = {{&S50541[32], 8}, {NULL}};
const Segment SGI394432[] = {{&S50541[22], 7}, {NULL}};
const Segment SGI394433[] = {{&S50541[6], 7}, {NULL}};
const Segment SGI394434[] = {{&S50542[0], 3}, {NULL}};
const Segment SGI394435[] = {{&S50542[32], 8}, {NULL}};
const Segment SGI394436[] = {{&S50542[22], 7}, {NULL}};
const Segment SGI394437[] = {{&S50542[6], 7}, {NULL}};
const Segment SGI394438[] = {{&S50544[0], 3}, {NULL}};
const Segment SGI394439[] = {{&S50544[32], 8}, {NULL}};
const Segment SGI394440[] = {{&S50544[22], 7}, {NULL}};
const Segment SGI394441[] = {{&S50544[6], 7}, {NULL}};
const Segment SGI394442[] = {{&S50545[0], 3}, {NULL}};
const Segment SGI394443[] = {{&S50545[30], 8}, {NULL}};
const Segment SGI394444[] = {{&S50545[20], 7}, {NULL}};
const Segment SGI394445[] = {{&S50545[6], 7}, {NULL}};
const Segment SGI394446[] = {{&S50550[0], 3}, {NULL}};
const Segment SGI394447[] = {{&S50550[30], 8}, {NULL}};
const Segment SGI394448[] = {{&S51606[0], 3}, {NULL}};
const Segment SGI394449[] = {{&S51606[48], 8}, {NULL}};
const Segment SGI394450[] = {{&S51606[28], 9}, {NULL}};
const Segment SGI394451[] = {{&S51606[5], 13}, {NULL}};
const Segment SGI394452[] = {{&S51609[0], 3}, {NULL}};
const Segment SGI394453[] = {{&S51609[49], 8}, {NULL}};
const Segment SGI394454[] = {{&S51609[28], 10}, {NULL}};
const Segment SGI394455[] = {{&S51609[5], 13}, {NULL}};
const Segment SGI394456[] = {{&S51610[0], 3}, {NULL}};
const Segment SGI394457[] = {{&S51610[49], 8}, {NULL}};
const Segment SGI394458[] = {{&S51610[28], 10}, {NULL}};
const Segment SGI394459[] = {{&S51610[5], 13}, {NULL}};
const Segment SGI394460[] = {{&S51613[0], 3}, {NULL}};
const Segment SGI394461[] = {{&S51613[32], 8}, {NULL}};
const Segment SGI394462[] = {{&S51613[22], 7}, {NULL}};
const Segment SGI394463[] = {{&S51613[6], 7}, {NULL}};
const Segment SGI394464[] = {{&S51620[0], 3}, {NULL}};
const Segment SGI394465[] = {{&S51620[32], 8}, {NULL}};
const Segment SGI394466[] = {{&S51620[22], 7}, {NULL}};
const Segment SGI394467[] = {{&S51620[6], 7}, {NULL}};
const Segment SGI394468[] = {{&S51621[0], 3}, {NULL}};
const Segment SGI394469[] = {{&S51621[32], 8}, {NULL}};
const Segment SGI394470[] = {{&S51625[0], 3}, {NULL}};
const Segment SGI394471[] = {{&S51625[32], 8}, {NULL}};
const Segment SGI394472[] = {{&S51625[22], 7}, {NULL}};
const Segment SGI394473[] = {{&S51625[6], 7}, {NULL}};
const Segment SGI394474[] = {{&S54581[0], 3}, {NULL}};
const Segment SGI394475[] = {{&S54581[32], 8}, {NULL}};
const Segment SGI394476[] = {{&S54581[22], 7}, {NULL}};
const Segment SGI394477[] = {{&S54581[6], 7}, {NULL}};
const Segment SGI394478[] = {{&S54582[0], 3}, {NULL}};
const Segment SGI394479[] = {{&S54582[32], 8}, {NULL}};
const Segment SGI394480[] = {{&S54582[22], 7}, {NULL}};
const Segment SGI394481[] = {{&S54582[6], 7}, {NULL}};
const Segment SGI394482[] = {{&S54584[0], 3}, {NULL}};
const Segment SGI394483[] = {{&S54584[32], 8}, {NULL}};
const Segment SGI394484[] = {{&S54597[0], 3}, {NULL}};
const Segment SGI394485[] = {{&S54597[32], 8}, {NULL}};
const Segment SGI394486[] = {{&S54597[22], 7}, {NULL}};
const Segment SGI394487[] = {{&S54597[6], 7}, {NULL}};
const Segment SGI394488[] = {{&S54980[0], 3}, {NULL}};
const Segment SGI394489[] = {{&S54980[48], 8}, {NULL}};
const Segment SGI394490[] = {{&S54980[28], 9}, {NULL}};
const Segment SGI394491[] = {{&S54980[5], 13}, {NULL}};
const Segment SGI394492[] = {{&S54981[0], 3}, {NULL}};
const Segment SGI394493[] = {{&S54981[48], 8}, {NULL}};
const Segment SGI394494[] = {{&S54981[28], 9}, {NULL}};
const Segment SGI394495[] = {{&S54981[5], 13}, {NULL}};
const Segment SGI394496[] = {{&S54982[0], 3}, {NULL}};
const Segment SGI394497[] = {{&S54982[49], 8}, {NULL}};
const Segment SGI394498[] = {{&S54982[28], 10}, {NULL}};
const Segment SGI394499[] = {{&S54982[5], 13}, {NULL}};
const Segment SGI394500[] = {{&S55350[0], 3}, {NULL}};
const Segment SGI394501[] = {{&S55350[49], 8}, {NULL}};
const Segment SGI394502[] = {{&S55350[28], 10}, {NULL}};
const Segment SGI394503[] = {{&S55350[5], 13}, {NULL}};
const Segment SGI394504[] = {{&S55351[0], 3}, {NULL}};
const Segment SGI394505[] = {{&S55351[48], 8}, {NULL}};
const Segment SGI394506[] = {{&S55351[28], 9}, {NULL}};
const Segment SGI394507[] = {{&S55351[5], 13}, {NULL}};
const Segment SGI394508[] = {{&S55352[0], 3}, {NULL}};
const Segment SGI394509[] = {{&S55352[48], 8}, {NULL}};
const Segment SGI394510[] = {{&S55352[28], 9}, {NULL}};
const Segment SGI394511[] = {{&S55352[5], 13}, {NULL}};
const Segment SGI394512[] = {{&S55353[0], 3}, {NULL}};
const Segment SGI394513[] = {{&S55353[48], 8}, {NULL}};
const Segment SGI394514[] = {{&S55353[28], 9}, {NULL}};
const Segment SGI394515[] = {{&S55353[5], 13}, {NULL}};
const Segment SGI394516[] = {{&S55354[0], 3}, {NULL}};
const Segment SGI394517[] = {{&S55354[49], 8}, {NULL}};
const Segment SGI394518[] = {{&S55354[28], 10}, {NULL}};
const Segment SGI394519[] = {{&S55354[5], 13}, {NULL}};
const Segment SGI394520[] = {{&S55355[0], 3}, {NULL}};
const Segment SGI394521[] = {{&S55355[49], 8}, {NULL}};
const Segment SGI394522[] = {{&S55355[28], 10}, {NULL}};
const Segment SGI394523[] = {{&S55355[5], 13}, {NULL}};
const Segment SGI394524[] = {{&S55809[0], 3}, {NULL}};
const Segment SGI394525[] = {{&S55809[32], 8}, {NULL}};
const Segment SGI394526[] = {{&S55809[22], 7}, {NULL}};
const Segment SGI394527[] = {{&S55809[6], 7}, {NULL}};
const Segment SGI394528[] = {{&S55810[0], 3}, {NULL}};
const Segment SGI394529[] = {{&S55810[32], 8}, {NULL}};
const Segment SGI394530[] = {{&S55810[22], 7}, {NULL}};
const Segment SGI394531[] = {{&S55810[6], 7}, {NULL}};
const Segment SGI394532[] = {{&S55811[0], 3}, {NULL}};
const Segment SGI394533[] = {{&S55811[32], 8}, {NULL}};
const Segment SGI394534[] = {{&S55811[22], 7}, {NULL}};
const Segment SGI394535[] = {{&S55811[6], 7}, {NULL}};
const Segment SGI394536[] = {{&S55812[0], 3}, {NULL}};
const Segment SGI394537[] = {{&S55812[30], 8}, {NULL}};
const Segment SGI394538[] = {{&S55812[20], 7}, {NULL}};
const Segment SGI394539[] = {{&S55812[6], 7}, {NULL}};
const Segment SGI394540[] = {{&S56141[0], 3}, {NULL}};
const Segment SGI394541[] = {{&S56141[32], 8}, {NULL}};
const Segment SGI394542[] = {{&S56141[22], 7}, {NULL}};
const Segment SGI394543[] = {{&S56141[6], 7}, {NULL}};
const Segment SGI394544[] = {{&S56142[0], 3}, {NULL}};
const Segment SGI394545[] = {{&S56142[32], 8}, {NULL}};
const Segment SGI394546[] = {{&S56142[22], 7}, {NULL}};
const Segment SGI394547[] = {{&S56142[6], 7}, {NULL}};
const Segment SGI394548[] = {{&S56143[0], 3}, {NULL}};
const Segment SGI394549[] = {{&S56143[32], 8}, {NULL}};
const Segment SGI394550[] = {{&S56143[22], 7}, {NULL}};
const Segment SGI394551[] = {{&S56143[6], 7}, {NULL}};
const Segment SGI394552[] = {{&S56144[0], 3}, {NULL}};
const Segment SGI394553[] = {{&S56144[32], 8}, {NULL}};
const Segment SGI394554[] = {{&S56145[0], 3}, {NULL}};
const Segment SGI394555[] = {{&S56145[32], 8}, {NULL}};
const Segment SGI394556[] = {{&S56145[22], 7}, {NULL}};
const Segment SGI394557[] = {{&S56145[6], 7}, {NULL}};
const Segment SGI394558[] = {{&S56153[0], 3}, {NULL}};
const Segment SGI394559[] = {{&S56153[30], 8}, {NULL}};
const Segment SGI394560[] = {{&S56153[20], 7}, {NULL}};
const Segment SGI394561[] = {{&S56153[6], 7}, {NULL}};
const Segment SGI394562[] = {{&S57720[0], 3}, {NULL}};
const Segment SGI394563[] = {{&S57720[48], 8}, {NULL}};
const Segment SGI394564[] = {{&S57720[28], 9}, {NULL}};
const Segment SGI394565[] = {{&S57720[5], 13}, {NULL}};
const Segment SGI394566[] = {{&S57721[0], 3}, {NULL}};
const Segment SGI394567[] = {{&S57721[48], 8}, {NULL}};
const Segment SGI394568[] = {{&S57721[28], 9}, {NULL}};
const Segment SGI394569[] = {{&S57721[5], 13}, {NULL}};
const Segment SGI394570[] = {{&S57722[0], 3}, {NULL}};
const Segment SGI394571[] = {{&S57722[49], 8}, {NULL}};
const Segment SGI394572[] = {{&S57722[28], 10}, {NULL}};
const Segment SGI394573[] = {{&S57722[5], 13}, {NULL}};
const Segment SGI394574[] = {{&S57723[0], 3}, {NULL}};
const Segment SGI394575[] = {{&S57723[49], 8}, {NULL}};
const Segment SGI394576[] = {{&S57723[28], 10}, {NULL}};
const Segment SGI394577[] = {{&S57723[5], 13}, {NULL}};
const Segment SGI394578[] = {{&S58595[0], 3}, {NULL}};
const Segment SGI394579[] = {{&S58595[30], 8}, {NULL}};
const Segment SGI394580[] = {{&S58595[20], 7}, {NULL}};
const Segment SGI394581[] = {{&S58595[6], 7}, {NULL}};
const Segment SGI394582[] = {{&S58596[0], 3}, {NULL}};
const Segment SGI394583[] = {{&S58596[32], 8}, {NULL}};
const Segment SGI394584[] = {{&S58596[22], 7}, {NULL}};
const Segment SGI394585[] = {{&S58596[6], 7}, {NULL}};
const Segment SGI394586[] = {{&S59262[0], 3}, {NULL}};
const Segment SGI394587[] = {{&S59262[51], 8}, {NULL}};
const Segment SGI394588[] = {{&S59262[34], 6}, {NULL}};
const Segment SGI394589[] = {{&S59262[5], 13}, {NULL}};
const Segment SGI394590[] = {{&S59263[0], 3}, {NULL}};
const Segment SGI394591[] = {{&S59263[48], 8}, {NULL}};
const Segment SGI394592[] = {{&S59263[28], 9}, {NULL}};
const Segment SGI394593[] = {{&S59263[5], 13}, {NULL}};
const Segment SGI394594[] = {{&S59264[0], 3}, {NULL}};
const Segment SGI394595[] = {{&S59264[47], 8}, {NULL}};
const Segment SGI394596[] = {{&S59264[28], 8}, {NULL}};
const Segment SGI394597[] = {{&S59264[5], 13}, {NULL}};
const Segment SGI394598[] = {{&S59265[0], 3}, {NULL}};
const Segment SGI394599[] = {{&S59265[44], 8}, {NULL}};
const Segment SGI394600[] = {{&S59265[28], 5}, {NULL}};
const Segment SGI394601[] = {{&S59265[5], 13}, {NULL}};
const Segment SGI394602[] = {{&S59266[0], 3}, {NULL}};
const Segment SGI394603[] = {{&S59266[42], 8}, {NULL}};
const Segment SGI394604[] = {{&S59266[28], 3}, {NULL}};
const Segment SGI394605[] = {{&S59266[5], 13}, {NULL}};
const Segment SGI394606[] = {{&S59267[0], 3}, {NULL}};
const Segment SGI394607[] = {{&S59267[45], 8}, {NULL}};
const Segment SGI394608[] = {{&S59267[28], 6}, {NULL}};
const Segment SGI394609[] = {{&S59267[5], 13}, {NULL}};
const Segment SGI394610[] = {{&S59268[0], 3}, {NULL}};
const Segment SGI394611[] = {{&S59268[45], 8}, {NULL}};
const Segment SGI394612[] = {{&S59268[28], 6}, {NULL}};
const Segment SGI394613[] = {{&S59268[5], 13}, {NULL}};
const Segment SGI394614[] = {{&S59269[0], 3}, {NULL}};
const Segment SGI394615[] = {{&S59269[45], 8}, {NULL}};
const Segment SGI394616[] = {{&S59269[28], 6}, {NULL}};
const Segment SGI394617[] = {{&S59269[5], 13}, {NULL}};
const Segment SGI394618[] = {{&S59270[0], 3}, {NULL}};
const Segment SGI394619[] = {{&S59270[51], 8}, {NULL}};
const Segment SGI394620[] = {{&S59270[28], 12}, {NULL}};
const Segment SGI394621[] = {{&S59270[5], 13}, {NULL}};
const Segment SGI394622[] = {{&S59271[0], 3}, {NULL}};
const Segment SGI394623[] = {{&S59271[53], 8}, {NULL}};
const Segment SGI394624[] = {{&S59271[34], 8}, {NULL}};
const Segment SGI394625[] = {{&S59271[5], 13}, {NULL}};
const Segment SGI394626[] = {{&S59272[0], 3}, {NULL}};
const Segment SGI394627[] = {{&S59272[42], 8}, {NULL}};
const Segment SGI394628[] = {{&S59272[26], 5}, {NULL}};
const Segment SGI394629[] = {{&S59272[5], 13}, {NULL}};
const Segment SGI394630[] = {{&S59273[0], 3}, {NULL}};
const Segment SGI394631[] = {{&S59273[51], 8}, {NULL}};
const Segment SGI394632[] = {{&S59273[34], 6}, {NULL}};
const Segment SGI394633[] = {{&S59273[5], 13}, {NULL}};
const Segment SGI394634[] = {{&S59274[0], 3}, {NULL}};
const Segment SGI394635[] = {{&S59274[43], 8}, {NULL}};
const Segment SGI394636[] = {{&S59274[26], 6}, {NULL}};
const Segment SGI394637[] = {{&S59274[5], 13}, {NULL}};
const Segment SGI394638[] = {{&S59275[0], 3}, {NULL}};
const Segment SGI394639[] = {{&S59275[48], 8}, {NULL}};
const Segment SGI394640[] = {{&S59275[28], 9}, {NULL}};
const Segment SGI394641[] = {{&S59275[5], 13}, {NULL}};
const Segment SGI394642[] = {{&S59276[0], 3}, {NULL}};
const Segment SGI394643[] = {{&S59276[48], 8}, {NULL}};
const Segment SGI394644[] = {{&S59276[28], 9}, {NULL}};
const Segment SGI394645[] = {{&S59276[5], 13}, {NULL}};
const Segment SGI394646[] = {{&S59277[0], 3}, {NULL}};
const Segment SGI394647[] = {{&S59277[46], 8}, {NULL}};
const Segment SGI394648[] = {{&S59277[28], 7}, {NULL}};
const Segment SGI394649[] = {{&S59277[5], 13}, {NULL}};
const Segment SGI394650[] = {{&S59278[0], 3}, {NULL}};
const Segment SGI394651[] = {{&S59278[42], 8}, {NULL}};
const Segment SGI394652[] = {{&S59278[28], 3}, {NULL}};
const Segment SGI394653[] = {{&S59278[5], 13}, {NULL}};
const Segment SGI394654[] = {{&S59279[0], 3}, {NULL}};
const Segment SGI394655[] = {{&S59279[46], 8}, {NULL}};
const Segment SGI394656[] = {{&S59279[28], 7}, {NULL}};
const Segment SGI394657[] = {{&S59279[5], 13}, {NULL}};
const Segment SGI394658[] = {{&S59280[0], 3}, {NULL}};
const Segment SGI394659[] = {{&S59280[42], 8}, {NULL}};
const Segment SGI394660[] = {{&S59280[28], 3}, {NULL}};
const Segment SGI394661[] = {{&S59280[5], 13}, {NULL}};
const Segment SGI394662[] = {{&S59281[0], 3}, {NULL}};
const Segment SGI394663[] = {{&S59281[53], 8}, {NULL}};
const Segment SGI394664[] = {{&S59281[28], 14}, {NULL}};
const Segment SGI394665[] = {{&S59281[5], 13}, {NULL}};
const Segment SGI394666[] = {{&S59282[0], 3}, {NULL}};
const Segment SGI394667[] = {{&S59282[48], 8}, {NULL}};
const Segment SGI394668[] = {{&S59282[28], 9}, {NULL}};
const Segment SGI394669[] = {{&S59282[5], 13}, {NULL}};
const Segment SGI394670[] = {{&S59283[0], 3}, {NULL}};
const Segment SGI394671[] = {{&S59283[52], 8}, {NULL}};
const Segment SGI394672[] = {{&S59283[28], 13}, {NULL}};
const Segment SGI394673[] = {{&S59283[5], 13}, {NULL}};
const Segment SGI394674[] = {{&S59284[0], 3}, {NULL}};
const Segment SGI394675[] = {{&S59284[44], 8}, {NULL}};
const Segment SGI394676[] = {{&S59284[28], 5}, {NULL}};
const Segment SGI394677[] = {{&S59284[5], 13}, {NULL}};
const Segment SGI394678[] = {{&S59285[0], 3}, {NULL}};
const Segment SGI394679[] = {{&S59285[52], 8}, {NULL}};
const Segment SGI394680[] = {{&S59285[28], 13}, {NULL}};
const Segment SGI394681[] = {{&S59285[5], 13}, {NULL}};
const Segment SGI394682[] = {{&S59286[0], 3}, {NULL}};
const Segment SGI394683[] = {{&S59286[53], 8}, {NULL}};
const Segment SGI394684[] = {{&S59286[28], 14}, {NULL}};
const Segment SGI394685[] = {{&S59286[5], 13}, {NULL}};
const Segment SGI394686[] = {{&S59287[0], 3}, {NULL}};
const Segment SGI394687[] = {{&S59287[44], 8}, {NULL}};
const Segment SGI394688[] = {{&S59287[28], 5}, {NULL}};
const Segment SGI394689[] = {{&S59287[5], 13}, {NULL}};
const Segment SGI394690[] = {{&S59288[0], 3}, {NULL}};
const Segment SGI394691[] = {{&S59288[45], 8}, {NULL}};
const Segment SGI394692[] = {{&S59288[28], 6}, {NULL}};
const Segment SGI394693[] = {{&S59288[5], 13}, {NULL}};
const Segment SGI394694[] = {{&S59289[0], 3}, {NULL}};
const Segment SGI394695[] = {{&S59289[45], 8}, {NULL}};
const Segment SGI394696[] = {{&S59289[28], 6}, {NULL}};
const Segment SGI394697[] = {{&S59289[5], 13}, {NULL}};
const Segment SGI394698[] = {{&S59290[0], 3}, {NULL}};
const Segment SGI394699[] = {{&S59290[42], 8}, {NULL}};
const Segment SGI394700[] = {{&S59290[28], 3}, {NULL}};
const Segment SGI394701[] = {{&S59290[5], 13}, {NULL}};
const Segment SGI394702[] = {{&S59291[0], 3}, {NULL}};
const Segment SGI394703[] = {{&S59291[44], 8}, {NULL}};
const Segment SGI394704[] = {{&S59291[28], 4}, {NULL}};
const Segment SGI394705[] = {{&S59291[5], 13}, {NULL}};
const Segment SGI394706[] = {{&S59292[0], 3}, {NULL}};
const Segment SGI394707[] = {{&S59292[45], 8}, {NULL}};
const Segment SGI394708[] = {{&S59292[28], 6}, {NULL}};
const Segment SGI394709[] = {{&S59292[5], 13}, {NULL}};
const Segment SGI394710[] = {{&S59293[0], 3}, {NULL}};
const Segment SGI394711[] = {{&S59293[48], 8}, {NULL}};
const Segment SGI394712[] = {{&S59293[28], 9}, {NULL}};
const Segment SGI394713[] = {{&S59293[5], 13}, {NULL}};
const Segment SGI394714[] = {{&S59294[0], 3}, {NULL}};
const Segment SGI394715[] = {{&S59294[51], 8}, {NULL}};
const Segment SGI394716[] = {{&S59294[28], 12}, {NULL}};
const Segment SGI394717[] = {{&S59294[5], 13}, {NULL}};
const Segment SGI394718[] = {{&S59295[0], 3}, {NULL}};
const Segment SGI394719[] = {{&S59295[48], 8}, {NULL}};
const Segment SGI394720[] = {{&S59295[28], 9}, {NULL}};
const Segment SGI394721[] = {{&S59295[5], 13}, {NULL}};
const Segment SGI394722[] = {{&S59296[0], 3}, {NULL}};
const Segment SGI394723[] = {{&S59296[55], 8}, {NULL}};
const Segment SGI394724[] = {{&S59296[28], 16}, {NULL}};
const Segment SGI394725[] = {{&S59296[5], 13}, {NULL}};
const Segment SGI394726[] = {{&S59297[0], 3}, {NULL}};
const Segment SGI394727[] = {{&S59297[43], 8}, {NULL}};
const Segment SGI394728[] = {{&S59297[28], 4}, {NULL}};
const Segment SGI394729[] = {{&S59297[5], 13}, {NULL}};
const Segment SGI394730[] = {{&S60292[0], 3}, {NULL}};
const Segment SGI394731[] = {{&S60292[49], 8}, {NULL}};
const Segment SGI394732[] = {{&S60292[28], 10}, {NULL}};
const Segment SGI394733[] = {{&S60292[5], 13}, {NULL}};
const Segment SGI394734[] = {{&S60836[0], 3}, {NULL}};
const Segment SGI394735[] = {{&S60836[32], 8}, {NULL}};
const Segment SGI394736[] = {{&S60836[22], 7}, {NULL}};
const Segment SGI394737[] = {{&S60836[6], 7}, {NULL}};
const Segment SGI394738[] = {{&S60837[0], 3}, {NULL}};
const Segment SGI394739[] = {{&S60837[32], 8}, {NULL}};
const Segment SGI394740[] = {{&S60837[22], 7}, {NULL}};
const Segment SGI394741[] = {{&S60837[6], 7}, {NULL}};
const Segment SGI394742[] = {{&S61311[0], 3}, {NULL}};
const Segment SGI394743[] = {{&S61311[32], 8}, {NULL}};
const Segment SGI394744[] = {{&S61311[22], 7}, {NULL}};
const Segment SGI394745[] = {{&S61311[6], 7}, {NULL}};
const Segment SGI394746[] = {{&S61312[0], 3}, {NULL}};
const Segment SGI394747[] = {{&S61312[32], 8}, {NULL}};
const Segment SGI394748[] = {{&S61312[22], 7}, {NULL}};
const Segment SGI394749[] = {{&S61312[6], 7}, {NULL}};
const Segment SGI394750[] = {{&S61313[0], 3}, {NULL}};
const Segment SGI394751[] = {{&S61313[30], 8}, {NULL}};
const Segment SGI394752[] = {{&S61313[20], 7}, {NULL}};
const Segment SGI394753[] = {{&S61313[6], 7}, {NULL}};
const Segment SGI394754[] = {{&S61316[0], 3}, {NULL}};
const Segment SGI394755[] = {{&S61316[32], 8}, {NULL}};
const Segment SGI394756[] = {{&S61316[22], 7}, {NULL}};
const Segment SGI394757[] = {{&S61316[6], 7}, {NULL}};
const Segment SGI394758[] = {{&S61319[0], 3}, {NULL}};
const Segment SGI394759[] = {{&S61319[32], 8}, {NULL}};
const Segment SGI394760[] = {{&S61319[22], 7}, {NULL}};
const Segment SGI394761[] = {{&S61319[6], 7}, {NULL}};
const Segment SGI394762[] = {{&S61322[0], 3}, {NULL}};
const Segment SGI394763[] = {{&S61322[32], 8}, {NULL}};
const Segment SGI394764[] = {{&S61373[0], 3}, {NULL}};
const Segment SGI394765[] = {{&S61373[50], 8}, {NULL}};
const Segment SGI394766[] = {{&S61373[34], 5}, {NULL}};
const Segment SGI394767[] = {{&S61373[5], 13}, {NULL}};
const Segment SGI394768[] = {{&S61374[0], 3}, {NULL}};
const Segment SGI394769[] = {{&S61374[50], 8}, {NULL}};
const Segment SGI394770[] = {{&S61374[34], 5}, {NULL}};
const Segment SGI394771[] = {{&S61374[5], 13}, {NULL}};
const Segment SGI394772[] = {{&S61375[0], 3}, {NULL}};
const Segment SGI394773[] = {{&S61375[44], 8}, {NULL}};
const Segment SGI394774[] = {{&S61375[28], 5}, {NULL}};
const Segment SGI394775[] = {{&S61375[5], 13}, {NULL}};
const Segment SGI394776[] = {{&S61421[0], 3}, {NULL}};
const Segment SGI394777[] = {{&S61421[48], 8}, {NULL}};
const Segment SGI394778[] = {{&S61421[28], 9}, {NULL}};
const Segment SGI394779[] = {{&S61421[5], 13}, {NULL}};
const Segment SGI394780[] = {{&S63457[0], 3}, {NULL}};
const Segment SGI394781[] = {{&S63457[48], 8}, {NULL}};
const Segment SGI394782[] = {{&S63457[28], 9}, {NULL}};
const Segment SGI394783[] = {{&S63457[5], 13}, {NULL}};
const Segment SGI394784[] = {{&S63458[0], 3}, {NULL}};
const Segment SGI394785[] = {{&S63458[48], 8}, {NULL}};
const Segment SGI394786[] = {{&S63458[28], 9}, {NULL}};
const Segment SGI394787[] = {{&S63458[5], 13}, {NULL}};
const Segment SGI394788[] = {{&S64086[0], 3}, {NULL}};
const Segment SGI394789[] = {{&S64086[48], 8}, {NULL}};
const Segment SGI394790[] = {{&S64086[28], 9}, {NULL}};
const Segment SGI394791[] = {{&S64086[5], 13}, {NULL}};
const Segment SGI394792[] = {{&S64087[0], 3}, {NULL}};
const Segment SGI394793[] = {{&S64087[48], 8}, {NULL}};
const Segment SGI394794[] = {{&S64087[28], 9}, {NULL}};
const Segment SGI394795[] = {{&S64087[5], 13}, {NULL}};
const Segment SGI394796[] = {{&S64283[0], 3}, {NULL}};
const Segment SGI394797[] = {{&S64283[45], 8}, {NULL}};
const Segment SGI394798[] = {{&S64283[28], 6}, {NULL}};
const Segment SGI394799[] = {{&S64283[5], 13}, {NULL}};
const Segment SGI394800[] = {{&S64284[0], 3}, {NULL}};
const Segment SGI394801[] = {{&S64284[45], 8}, {NULL}};
const Segment SGI394802[] = {{&S64284[28], 6}, {NULL}};
const Segment SGI394803[] = {{&S64284[5], 13}, {NULL}};
const Segment SGI394804[] = {{&S65402[0], 3}, {NULL}};
const Segment SGI394805[] = {{&S65402[32], 8}, {NULL}};
const Segment SGI394806[] = {{&S65402[22], 7}, {NULL}};
const Segment SGI394807[] = {{&S65402[6], 7}, {NULL}};
const Segment SGI394808[] = {{&S65403[0], 3}, {NULL}};
const Segment SGI394809[] = {{&S65403[32], 8}, {NULL}};
const Segment SGI394810[] = {{&S65403[22], 7}, {NULL}};
const Segment SGI394811[] = {{&S65403[6], 7}, {NULL}};
const Segment SGI394812[] = {{&S65404[0], 3}, {NULL}};
const Segment SGI394813[] = {{&S65404[30], 8}, {NULL}};
const Segment SGI394814[] = {{&S65404[20], 7}, {NULL}};
const Segment SGI394815[] = {{&S65404[6], 7}, {NULL}};
const Segment SGI394816[] = {{&S65407[0], 3}, {NULL}};
const Segment SGI394817[] = {{&S65407[32], 8}, {NULL}};
const Segment SGI394818[] = {{&S65407[22], 7}, {NULL}};
const Segment SGI394819[] = {{&S65407[6], 7}, {NULL}};
const Segment SGI394820[] = {{&S65411[0], 3}, {NULL}};
const Segment SGI394821[] = {{&S65411[32], 8}, {NULL}};
const Segment SGI394822[] = {{&S65411[22], 7}, {NULL}};
const Segment SGI394823[] = {{&S65411[6], 7}, {NULL}};
const Segment SGI394824[] = {{&S65551[0], 3}, {NULL}};
const Segment SGI394825[] = {{&S65551[48], 8}, {NULL}};
const Segment SGI394826[] = {{&S65551[28], 9}, {NULL}};
const Segment SGI394827[] = {{&S65551[5], 13}, {NULL}};
const Segment SGI394828[] = {{&S65803[0], 3}, {NULL}};
const Segment SGI394829[] = {{&S65803[48], 8}, {NULL}};
const Segment SGI394830[] = {{&S65803[28], 9}, {NULL}};
const Segment SGI394831[] = {{&S65803[5], 13}, {NULL}};
const Segment SGI394832[] = {{&S66489[0], 3}, {NULL}};
const Segment SGI394833[] = {{&S66489[49], 8}, {NULL}};
const Segment SGI394834[] = {{&S66489[28], 9}, {NULL}};
const Segment SGI394835[] = {{&S66489[5], 13}, {NULL}};
const Segment SGI394836[] = {{&S66490[0], 3}, {NULL}};
const Segment SGI394837[] = {{&S66490[49], 8}, {NULL}};
const Segment SGI394838[] = {{&S66490[28], 9}, {NULL}};
const Segment SGI394839[] = {{&S66490[5], 13}, {NULL}};
const Segment SGI394840[] = {{&S67653[0], 3}, {NULL}};
const Segment SGI394841[] = {{&S67653[32], 8}, {NULL}};
const Segment SGI394842[] = {{&S67653[22], 7}, {NULL}};
const Segment SGI394843[] = {{&S67653[6], 7}, {NULL}};
const Segment SGI394844[] = {{&S67654[0], 3}, {NULL}};
const Segment SGI394845[] = {{&S67654[32], 8}, {NULL}};
const Segment SGI394846[] = {{&S67654[22], 7}, {NULL}};
const Segment SGI394847[] = {{&S67654[6], 7}, {NULL}};
const Segment SGI394848[] = {{&S67732[0], 3}, {NULL}};
const Segment SGI394849[] = {{&S67732[45], 8}, {NULL}};
const Segment SGI394850[] = {{&S67732[28], 6}, {NULL}};
const Segment SGI394851[] = {{&S67732[5], 13}, {NULL}};
const Segment SGI394852[] = {{&S67733[0], 3}, {NULL}};
const Segment SGI394853[] = {{&S67733[45], 8}, {NULL}};
const Segment SGI394854[] = {{&S67733[28], 6}, {NULL}};
const Segment SGI394855[] = {{&S67733[5], 13}, {NULL}};
const Segment SGI394856[] = {{&S67852[0], 3}, {NULL}};
const Segment SGI394857[] = {{&S67852[48], 8}, {NULL}};
const Segment SGI394858[] = {{&S67852[28], 9}, {NULL}};
const Segment SGI394859[] = {{&S67852[5], 13}, {NULL}};
const Segment SGI394860[] = {{&S67853[0], 3}, {NULL}};
const Segment SGI394861[] = {{&S67853[48], 8}, {NULL}};
const Segment SGI394862[] = {{&S67853[28], 9}, {NULL}};
const Segment SGI394863[] = {{&S67853[5], 13}, {NULL}};
const Segment SGI394864[] = {{&S97734[0], 37}, {NULL}};
const Segment SGI394865[] = {{&S69727[0], 3}, {NULL}};
const Segment SGI394866[] = {{&S69727[45], 8}, {NULL}};
const Segment SGI394867[] = {{&S69727[28], 6}, {NULL}};
const Segment SGI394868[] = {{&S69727[5], 13}, {NULL}};
const Segment SGI394869[] = {{&S69728[0], 3}, {NULL}};
const Segment SGI394870[] = {{&S69728[45], 8}, {NULL}};
const Segment SGI394871[] = {{&S69728[28], 6}, {NULL}};
const Segment SGI394872[] = {{&S69728[5], 13}, {NULL}};
const Segment SGI394873[] = {{&S69983[0], 3}, {NULL}};
const Segment SGI394874[] = {{&S69983[49], 8}, {NULL}};
const Segment SGI394875[] = {{&S69983[28], 9}, {NULL}};
const Segment SGI394876[] = {{&S69983[5], 13}, {NULL}};
const Segment SGI394877[] = {{&S70079[0], 3}, {NULL}};
const Segment SGI394878[] = {{&S70079[47], 8}, {NULL}};
const Segment SGI394879[] = {{&S70079[26], 10}, {NULL}};
const Segment SGI394880[] = {{&S70079[5], 13}, {NULL}};
const Segment SGI394881[] = {{&S70080[0], 3}, {NULL}};
const Segment SGI394882[] = {{&S70080[49], 8}, {NULL}};
const Segment SGI394883[] = {{&S70080[28], 10}, {NULL}};
const Segment SGI394884[] = {{&S70080[5], 13}, {NULL}};
const Segment SGI394885[] = {{&S70162[0], 3}, {NULL}};
const Segment SGI394886[] = {{&S70162[44], 8}, {NULL}};
const Segment SGI394887[] = {{&S70162[28], 5}, {NULL}};
const Segment SGI394888[] = {{&S70162[5], 13}, {NULL}};
const Segment SGI394889[] = {{&S70163[0], 3}, {NULL}};
const Segment SGI394890[] = {{&S70163[50], 8}, {NULL}};
const Segment SGI394891[] = {{&S70163[34], 5}, {NULL}};
const Segment SGI394892[] = {{&S70163[5], 13}, {NULL}};
const Segment SGI394893[] = {{&S70224[0], 3}, {NULL}};
const Segment SGI394894[] = {{&S70224[48], 8}, {NULL}};
const Segment SGI394895[] = {{&S70224[28], 9}, {NULL}};
const Segment SGI394896[] = {{&S70224[5], 13}, {NULL}};
const Segment SGI394897[] = {{&S70225[0], 3}, {NULL}};
const Segment SGI394898[] = {{&S70225[46], 8}, {NULL}};
const Segment SGI394899[] = {{&S70225[26], 9}, {NULL}};
const Segment SGI394900[] = {{&S70225[5], 13}, {NULL}};
const Segment SGI394901[] = {{&S70621[0], 3}, {NULL}};
const Segment SGI394902[] = {{&S70621[45], 8}, {NULL}};
const Segment SGI394903[] = {{&S70621[28], 6}, {NULL}};
const Segment SGI394904[] = {{&S70621[5], 13}, {NULL}};
const Segment SGI394905[] = {{&S70622[0], 3}, {NULL}};
const Segment SGI394906[] = {{&S70622[45], 8}, {NULL}};
const Segment SGI394907[] = {{&S70622[28], 6}, {NULL}};
const Segment SGI394908[] = {{&S70622[5], 13}, {NULL}};
const Segment SGI394909[] = {{&S70816[0], 3}, {NULL}};
const Segment SGI394910[] = {{&S70816[49], 8}, {NULL}};
const Segment SGI394911[] = {{&S70816[28], 9}, {NULL}};
const Segment SGI394912[] = {{&S70816[5], 13}, {NULL}};
const Segment SGI394913[] = {{&S70876[0], 3}, {NULL}};
const Segment SGI394914[] = {{&S70876[48], 8}, {NULL}};
const Segment SGI394915[] = {{&S70876[28], 9}, {NULL}};
const Segment SGI394916[] = {{&S70876[5], 13}, {NULL}};
const Segment SGI394917[] = {{&S72219[0], 3}, {NULL}};
const Segment SGI394918[] = {{&S72219[46], 8}, {NULL}};
const Segment SGI394919[] = {{&S72219[28], 7}, {NULL}};
const Segment SGI394920[] = {{&S72219[5], 13}, {NULL}};
const Segment SGI394921[] = {{&S72220[0], 3}, {NULL}};
const Segment SGI394922[] = {{&S72220[45], 8}, {NULL}};
const Segment SGI394923[] = {{&S72220[28], 6}, {NULL}};
const Segment SGI394924[] = {{&S72220[5], 13}, {NULL}};
const Segment SGI394925[] = {{&S72260[0], 3}, {NULL}};
const Segment SGI394926[] = {{&S72260[48], 8}, {NULL}};
const Segment SGI394927[] = {{&S72260[28], 9}, {NULL}};
const Segment SGI394928[] = {{&S72260[5], 13}, {NULL}};
const Segment SGI394929[] = {{&S72347[0], 3}, {NULL}};
const Segment SGI394930[] = {{&S72347[43], 8}, {NULL}};
const Segment SGI394931[] = {{&S72347[28], 4}, {NULL}};
const Segment SGI394932[] = {{&S72347[5], 13}, {NULL}};
const Segment SGI394933[] = {{&S72410[0], 3}, {NULL}};
const Segment SGI394934[] = {{&S72410[48], 8}, {NULL}};
const Segment SGI394935[] = {{&S72410[28], 9}, {NULL}};
const Segment SGI394936[] = {{&S72410[5], 13}, {NULL}};
const Segment SGI394937[] = {{&S72783[0], 3}, {NULL}};
const Segment SGI394938[] = {{&S72783[50], 8}, {NULL}};
const Segment SGI394939[] = {{&S72783[34], 5}, {NULL}};
const Segment SGI394940[] = {{&S72783[5], 13}, {NULL}};
const Segment SGI394941[] = {{&S73200[0], 3}, {NULL}};
const Segment SGI394942[] = {{&S73200[49], 8}, {NULL}};
const Segment SGI394943[] = {{&S73200[28], 9}, {NULL}};
const Segment SGI394944[] = {{&S73200[5], 13}, {NULL}};
const Segment SGI394945[] = {{&S73469[0], 3}, {NULL}};
const Segment SGI394946[] = {{&S73469[48], 8}, {NULL}};
const Segment SGI394947[] = {{&S73469[28], 9}, {NULL}};
const Segment SGI394948[] = {{&S73469[5], 13}, {NULL}};
const Segment SGI394949[] = {{&S73653[0], 3}, {NULL}};
const Segment SGI394950[] = {{&S73653[32], 8}, {NULL}};
const Segment SGI394951[] = {{&S73653[22], 7}, {NULL}};
const Segment SGI394952[] = {{&S73653[6], 7}, {NULL}};
const Segment SGI394953[] = {{&S73821[0], 3}, {NULL}};
const Segment SGI394954[] = {{&S73821[48], 8}, {NULL}};
const Segment SGI394955[] = {{&S73821[28], 9}, {NULL}};
const Segment SGI394956[] = {{&S73821[5], 13}, {NULL}};
const Segment SGI394957[] = {{&S73889[0], 3}, {NULL}};
const Segment SGI394958[] = {{&S73889[50], 8}, {NULL}};
const Segment SGI394959[] = {{&S73889[34], 5}, {NULL}};
const Segment SGI394960[] = {{&S73889[5], 13}, {NULL}};
const Segment SGI394961[] = {{&S73949[0], 3}, {NULL}};
const Segment SGI394962[] = {{&S73949[48], 8}, {NULL}};
const Segment SGI394963[] = {{&S73949[28], 9}, {NULL}};
const Segment SGI394964[] = {{&S73949[5], 13}, {NULL}};
const Segment SGI394965[] = {{&S74110[0], 3}, {NULL}};
const Segment SGI394966[] = {{&S74110[47], 8}, {NULL}};
const Segment SGI394967[] = {{&S74110[26], 9}, {NULL}};
const Segment SGI394968[] = {{&S74110[5], 13}, {NULL}};
const Segment SGI394969[] = {{&S74111[0], 3}, {NULL}};
const Segment SGI394970[] = {{&S74111[49], 8}, {NULL}};
const Segment SGI394971[] = {{&S74111[28], 9}, {NULL}};
const Segment SGI394972[] = {{&S74111[5], 13}, {NULL}};
const Segment SGI394973[] = {{&S74171[0], 3}, {NULL}};
const Segment SGI394974[] = {{&S74171[45], 8}, {NULL}};
const Segment SGI394975[] = {{&S74171[28], 6}, {NULL}};
const Segment SGI394976[] = {{&S74171[5], 13}, {NULL}};
const Segment SGI394977[] = {{&S74172[0], 3}, {NULL}};
const Segment SGI394978[] = {{&S74172[45], 8}, {NULL}};
const Segment SGI394979[] = {{&S74172[28], 6}, {NULL}};
const Segment SGI394980[] = {{&S74172[5], 13}, {NULL}};
const Segment SGI394981[] = {{&S74302[0], 3}, {NULL}};
const Segment SGI394982[] = {{&S74302[52], 8}, {NULL}};
const Segment SGI394983[] = {{&S74302[28], 13}, {NULL}};
const Segment SGI394984[] = {{&S74302[5], 13}, {NULL}};
const Segment SGI394985[] = {{&S74412[0], 3}, {NULL}};
const Segment SGI394986[] = {{&S74412[30], 8}, {NULL}};
const Segment SGI394987[] = {{&S74412[20], 7}, {NULL}};
const Segment SGI394988[] = {{&S74412[6], 7}, {NULL}};
const Segment SGI394989[] = {{&S75145[0], 3}, {NULL}};
const Segment SGI394990[] = {{&S75145[52], 8}, {NULL}};
const Segment SGI394991[] = {{&S75145[28], 13}, {NULL}};
const Segment SGI394992[] = {{&S75145[5], 13}, {NULL}};
const Segment SGI394993[] = {{&S75279[0], 3}, {NULL}};
const Segment SGI394994[] = {{&S75279[32], 8}, {NULL}};
const Segment SGI394995[] = {{&S75279[22], 7}, {NULL}};
const Segment SGI394996[] = {{&S75279[6], 7}, {NULL}};
const Segment SGI394997[] = {{&S75280[0], 3}, {NULL}};
const Segment SGI394998[] = {{&S75280[32], 8}, {NULL}};
const Segment SGI394999[] = {{&S75280[22], 7}, {NULL}};
const Segment SGI395000[] = {{&S75280[6], 7}, {NULL}};
const Segment SGI395001[] = {{&S75281[0], 3}, {NULL}};
const Segment SGI395002[] = {{&S75281[30], 8}, {NULL}};
const Segment SGI395003[] = {{&S75281[20], 7}, {NULL}};
const Segment SGI395004[] = {{&S75281[6], 7}, {NULL}};
const Segment SGI395005[] = {{&S75282[0], 3}, {NULL}};
const Segment SGI395006[] = {{&S75282[32], 8}, {NULL}};
const Segment SGI395007[] = {{&S75282[22], 7}, {NULL}};
const Segment SGI395008[] = {{&S75282[6], 7}, {NULL}};
const Segment SGI395009[] = {{&S75283[0], 3}, {NULL}};
const Segment SGI395010[] = {{&S75283[32], 8}, {NULL}};
const Segment SGI395011[] = {{&S75283[22], 7}, {NULL}};
const Segment SGI395012[] = {{&S75283[6], 7}, {NULL}};
const Segment SGI395013[] = {{&S75523[0], 3}, {NULL}};
const Segment SGI395014[] = {{&S75523[50], 8}, {NULL}};
const Segment SGI395015[] = {{&S75523[34], 5}, {NULL}};
const Segment SGI395016[] = {{&S75523[5], 13}, {NULL}};
const Segment SGI395017[] = {{&S75664[0], 3}, {NULL}};
const Segment SGI395018[] = {{&S75664[45], 8}, {NULL}};
const Segment SGI395019[] = {{&S75664[28], 6}, {NULL}};
const Segment SGI395020[] = {{&S75664[5], 13}, {NULL}};
const Segment SGI395021[] = {{&S75665[0], 3}, {NULL}};
const Segment SGI395022[] = {{&S75665[45], 8}, {NULL}};
const Segment SGI395023[] = {{&S75665[28], 6}, {NULL}};
const Segment SGI395024[] = {{&S75665[5], 13}, {NULL}};
const Segment SGI395025[] = {{&S75987[0], 3}, {NULL}};
const Segment SGI395026[] = {{&S75987[43], 8}, {NULL}};
const Segment SGI395027[] = {{&S75987[28], 4}, {NULL}};
const Segment SGI395028[] = {{&S75987[5], 13}, {NULL}};
const Segment SGI395029[] = {{&S76231[0], 3}, {NULL}};
const Segment SGI395030[] = {{&S76231[48], 8}, {NULL}};
const Segment SGI395031[] = {{&S76231[25], 13}, {NULL}};
const Segment SGI395032[] = {{&S76232[0], 3}, {NULL}};
const Segment SGI395033[] = {{&S76232[45], 8}, {NULL}};
const Segment SGI395034[] = {{&S76232[25], 10}, {NULL}};
const Segment SGI395035[] = {{&S76233[0], 3}, {NULL}};
const Segment SGI395036[] = {{&S76233[47], 8}, {NULL}};
const Segment SGI395037[] = {{&S76233[25], 12}, {NULL}};
const Segment SGI395038[] = {{&S76234[0], 3}, {NULL}};
const Segment SGI395039[] = {{&S76234[36], 8}, {NULL}};
const Segment SGI395040[] = {{&S76234[22], 4}, {NULL}};
const Segment SGI395041[] = {{&S76235[0], 3}, {NULL}};
const Segment SGI395042[] = {{&S76235[36], 8}, {NULL}};
const Segment SGI395043[] = {{&S76235[22], 4}, {NULL}};
const Segment SGI395044[] = {{&S76236[0], 3}, {NULL}};
const Segment SGI395045[] = {{&S76236[39], 8}, {NULL}};
const Segment SGI395046[] = {{&S76236[25], 4}, {NULL}};
const Segment SGI395047[] = {{&S76282[0], 3}, {NULL}};
const Segment SGI395048[] = {{&S76282[46], 8}, {NULL}};
const Segment SGI395049[] = {{&S76282[28], 6}, {NULL}};
const Segment SGI395050[] = {{&S76282[5], 13}, {NULL}};
const Segment SGI395051[] = {{&S76283[0], 3}, {NULL}};
const Segment SGI395052[] = {{&S76283[46], 8}, {NULL}};
const Segment SGI395053[] = {{&S76283[28], 6}, {NULL}};
const Segment SGI395054[] = {{&S76283[5], 13}, {NULL}};
const Segment SGI395055[] = {{&S76350[0], 3}, {NULL}};
const Segment SGI395056[] = {{&S76350[32], 8}, {NULL}};
const Segment SGI395057[] = {{&S76350[22], 7}, {NULL}};
const Segment SGI395058[] = {{&S76350[6], 7}, {NULL}};
const Segment SGI395059[] = {{&S76464[0], 3}, {NULL}};
const Segment SGI395060[] = {{&S76464[45], 8}, {NULL}};
const Segment SGI395061[] = {{&S76464[28], 6}, {NULL}};
const Segment SGI395062[] = {{&S76464[5], 13}, {NULL}};
const Segment SGI395063[] = {{&S76465[0], 3}, {NULL}};
const Segment SGI395064[] = {{&S76465[45], 8}, {NULL}};
const Segment SGI395065[] = {{&S76465[28], 6}, {NULL}};
const Segment SGI395066[] = {{&S76465[5], 13}, {NULL}};
const Segment SGI395067[] = {{&S76658[0], 3}, {NULL}};
const Segment SGI395068[] = {{&S76658[45], 8}, {NULL}};
const Segment SGI395069[] = {{&S76658[28], 6}, {NULL}};
const Segment SGI395070[] = {{&S76658[5], 13}, {NULL}};
const Segment SGI395071[] = {{&S76659[0], 3}, {NULL}};
const Segment SGI395072[] = {{&S76659[45], 8}, {NULL}};
const Segment SGI395073[] = {{&S76659[28], 6}, {NULL}};
const Segment SGI395074[] = {{&S76659[5], 13}, {NULL}};
const Segment SGI395075[] = {{&S77104[0], 3}, {NULL}};
const Segment SGI395076[] = {{&S77104[45], 8}, {NULL}};
const Segment SGI395077[] = {{&S77104[28], 6}, {NULL}};
const Segment SGI395078[] = {{&S77104[5], 13}, {NULL}};
const Segment SGI395079[] = {{&S77279[0], 3}, {NULL}};
const Segment SGI395080[] = {{&S77279[49], 8}, {NULL}};
const Segment SGI395081[] = {{&S77279[28], 10}, {NULL}};
const Segment SGI395082[] = {{&S77279[5], 13}, {NULL}};
const Segment SGI395083[] = {{&S77280[0], 3}, {NULL}};
const Segment SGI395084[] = {{&S77280[49], 8}, {NULL}};
const Segment SGI395085[] = {{&S77280[28], 10}, {NULL}};
const Segment SGI395086[] = {{&S77280[5], 13}, {NULL}};
const Segment SGI395087[] = {{&S77426[0], 3}, {NULL}};
const Segment SGI395088[] = {{&S77426[45], 8}, {NULL}};
const Segment SGI395089[] = {{&S77426[28], 6}, {NULL}};
const Segment SGI395090[] = {{&S77426[5], 13}, {NULL}};
const Segment SGI395091[] = {{&S77427[0], 3}, {NULL}};
const Segment SGI395092[] = {{&S77427[45], 8}, {NULL}};
const Segment SGI395093[] = {{&S77427[28], 6}, {NULL}};
const Segment SGI395094[] = {{&S77427[5], 13}, {NULL}};
const Segment SGI395095[] = {{&S77692[0], 3}, {NULL}};
const Segment SGI395096[] = {{&S77692[45], 8}, {NULL}};
const Segment SGI395097[] = {{&S77692[28], 6}, {NULL}};
const Segment SGI395098[] = {{&S77692[5], 13}, {NULL}};
const Segment SGI395099[] = {{&S77693[0], 3}, {NULL}};
const Segment SGI395100[] = {{&S77693[45], 8}, {NULL}};
const Segment SGI395101[] = {{&S77693[28], 6}, {NULL}};
const Segment SGI395102[] = {{&S77693[5], 13}, {NULL}};
const Segment SGI395103[] = {{&S78290[0], 3}, {NULL}};
const Segment SGI395104[] = {{&S78290[45], 8}, {NULL}};
const Segment SGI395105[] = {{&S78290[28], 6}, {NULL}};
const Segment SGI395106[] = {{&S78290[5], 13}, {NULL}};
const Segment SGI395107[] = {{&S78291[0], 3}, {NULL}};
const Segment SGI395108[] = {{&S78291[45], 8}, {NULL}};
const Segment SGI395109[] = {{&S78291[28], 6}, {NULL}};
const Segment SGI395110[] = {{&S78291[5], 13}, {NULL}};
const Segment SGI395111[] = {{&S78292[0], 3}, {NULL}};
const Segment SGI395112[] = {{&S78292[45], 8}, {NULL}};
const Segment SGI395113[] = {{&S78292[28], 6}, {NULL}};
const Segment SGI395114[] = {{&S78292[5], 13}, {NULL}};
const Segment SGI395115[] = {{&S78293[0], 3}, {NULL}};
const Segment SGI395116[] = {{&S78293[46], 8}, {NULL}};
const Segment SGI395117[] = {{&S78293[28], 7}, {NULL}};
const Segment SGI395118[] = {{&S78293[5], 13}, {NULL}};
const Segment SGI395119[] = {{&S78737[0], 3}, {NULL}};
const Segment SGI395120[] = {{&S78737[52], 8}, {NULL}};
const Segment SGI395121[] = {{&S78737[28], 13}, {NULL}};
const Segment SGI395122[] = {{&S78737[5], 13}, {NULL}};
const Segment SGI395123[] = {{&S79092[0], 3}, {NULL}};
const Segment SGI395124[] = {{&S79092[47], 8}, {NULL}};
const Segment SGI395125[] = {{&S79092[28], 8}, {NULL}};
const Segment SGI395126[] = {{&S79092[5], 13}, {NULL}};
const Segment SGI395127[] = {{&S79137[0], 3}, {NULL}};
const Segment SGI395128[] = {{&S79137[32], 8}, {NULL}};
const Segment SGI395129[] = {{&S79137[22], 7}, {NULL}};
const Segment SGI395130[] = {{&S79137[6], 7}, {NULL}};
const Segment SGI395131[] = {{&S79138[0], 3}, {NULL}};
const Segment SGI395132[] = {{&S79138[32], 8}, {NULL}};
const Segment SGI395133[] = {{&S79138[22], 7}, {NULL}};
const Segment SGI395134[] = {{&S79138[6], 7}, {NULL}};
const Segment SGI395135[] = {{&S79139[0], 3}, {NULL}};
const Segment SGI395136[] = {{&S79139[32], 8}, {NULL}};
const Segment SGI395137[] = {{&S79139[22], 7}, {NULL}};
const Segment SGI395138[] = {{&S79139[6], 7}, {NULL}};
const Segment SGI395139[] = {{&S79484[0], 3}, {NULL}};
const Segment SGI395140[] = {{&S79484[48], 8}, {NULL}};
const Segment SGI395141[] = {{&S79484[28], 9}, {NULL}};
const Segment SGI395142[] = {{&S79484[5], 13}, {NULL}};
const Segment SGI395143[] = {{&S4735[30], 13}, {NULL}};
const Segment SGI395144[] = {{&S4736[30], 13}, {NULL}};
const Segment SGI395145[] = {{&S4737[30], 13}, {NULL}};
const Segment SGI395146[] = {{&S4738[30], 12}, {NULL}};
const Segment SGI395147[] = {{&S4739[30], 12}, {NULL}};
const Segment SGI395148[] = {{&S4743[30], 12}, {NULL}};
const Segment SGI395149[] = {{&S4745[30], 13}, {NULL}};
const Segment SGI395150[] = {{&S4746[30], 13}, {NULL}};
const Segment SGI395151[] = {{&S4750[30], 13}, {NULL}};
const Segment SGI395152[] = {{&S4752[30], 13}, {NULL}};
const Segment SGI395153[] = {{&S4753[30], 13}, {NULL}};
const Segment SGI395154[] = {{&S4754[30], 13}, {NULL}};
const Segment SGI395155[] = {{&S4755[30], 13}, {NULL}};
const Segment SGI395156[] = {{&S4756[30], 12}, {NULL}};
const Segment SGI395157[] = {{&S4757[30], 12}, {NULL}};
const Segment SGI395158[] = {{&S4758[30], 12}, {NULL}};
const Segment SGI395159[] = {{&S4759[30], 12}, {NULL}};
const Segment SGI395160[] = {{&S4760[30], 13}, {NULL}};
const Segment SGI395161[] = {{&S4761[30], 12}, {NULL}};
const Segment SGI395162[] = {{&S4762[30], 13}, {NULL}};
const Segment SGI395163[] = {{&S4763[30], 12}, {NULL}};
const Segment SGI395164[] = {{&S4764[30], 13}, {NULL}};
const Segment SGI395165[] = {{&S4765[30], 13}, {NULL}};
const Segment SGI395166[] = {{&S4766[30], 12}, {NULL}};
const Segment SGI395167[] = {{&S4767[30], 13}, {NULL}};
const Segment SGI395168[] = {{&S4768[30], 12}, {NULL}};
const Segment SGI395169[] = {{&S4769[30], 13}, {NULL}};
const Segment SGI395170[] = {{&S4770[30], 13}, {NULL}};
const Segment SGI395171[] = {{&S4771[30], 13}, {NULL}};
const Segment SGI395172[] = {{&S4772[30], 13}, {NULL}};
const Segment SGI395173[] = {{&S4773[30], 13}, {NULL}};
const Segment SGI395174[] = {{&S4774[30], 13}, {NULL}};
const Segment SGI395175[] = {{&S4775[30], 12}, {NULL}};
const Segment SGI395176[] = {{&S4784[30], 13}, {NULL}};
const Segment SGI395177[] = {{&S4785[30], 13}, {NULL}};
const Segment SGI395178[] = {{&S4787[30], 12}, {NULL}};
const Segment SGI395179[] = {{&S4788[30], 13}, {NULL}};
const Segment SGI395180[] = {{&S4789[30], 16}, {NULL}};
const Segment SGI395181[] = {{&S4790[30], 12}, {NULL}};
const Segment SGI395182[] = {{&S4792[30], 12}, {NULL}};
const Segment SGI395183[] = {{&S4793[30], 12}, {NULL}};
const Segment SGI395184[] = {{&S4794[30], 12}, {NULL}};
const Segment SGI395185[] = {{&S4795[30], 13}, {NULL}};
const Segment SGI395186[] = {{&S4796[30], 14}, {NULL}};
const Segment SGI395187[] = {{&S4797[30], 13}, {NULL}};
const Segment SGI395188[] = {{&S4798[30], 13}, {NULL}};
const Segment SGI395189[] = {{&S4799[30], 12}, {NULL}};
const Segment SGI395190[] = {{&S4800[30], 12}, {NULL}};
const Segment SGI395191[] = {{&S4801[30], 12}, {NULL}};
const Segment SGI395192[] = {{&S4802[30], 12}, {NULL}};
const Segment SGI395193[] = {{&S4803[30], 13}, {NULL}};
const Segment SGI395194[] = {{&S4804[30], 13}, {NULL}};
const Segment SGI395195[] = {{&S4805[30], 13}, {NULL}};
const Segment SGI395196[] = {{&S4806[30], 13}, {NULL}};
const Segment SGI395197[] = {{&S4807[30], 13}, {NULL}};
const Segment SGI395198[] = {{&S4808[30], 13}, {NULL}};
const Segment SGI395199[] = {{&S4809[30], 12}, {NULL}};
const Segment SGI395200[] = {{&S4810[30], 13}, {NULL}};
const Segment SGI395201[] = {{&S4811[30], 12}, {NULL}};
const Segment SGI395202[] = {{&S4812[30], 13}, {NULL}};
const Segment SGI395203[] = {{&S4814[30], 12}, {NULL}};
const Segment SGI395204[] = {{&S4826[30], 12}, {NULL}};
const Segment SGI395205[] = {{&S4827[30], 13}, {NULL}};
const Segment SGI395206[] = {{&S4828[30], 13}, {NULL}};
const Segment SGI395207[] = {{&S4829[30], 13}, {NULL}};
const Segment SGI395208[] = {{&S4830[30], 13}, {NULL}};
const Segment SGI395209[] = {{&S4831[30], 12}, {NULL}};
const Segment SGI395210[] = {{&S4832[30], 14}, {NULL}};
const Segment SGI395211[] = {{&S4833[30], 12}, {NULL}};
const Segment SGI395212[] = {{&S4834[30], 13}, {NULL}};
const Segment SGI395213[] = {{&S4835[30], 13}, {NULL}};
const Segment SGI395214[] = {{&S4836[30], 14}, {NULL}};
const Segment SGI395215[] = {{&S4837[30], 12}, {NULL}};
const Segment SGI395216[] = {{&S4838[30], 12}, {NULL}};
const Segment SGI395217[] = {{&S4839[30], 12}, {NULL}};
const Segment SGI395218[] = {{&S4840[30], 13}, {NULL}};
const Segment SGI395219[] = {{&S80461[0], 3}, {NULL}};
const Segment SGI395220[] = {{&S80461[39], 8}, {NULL}};
const Segment SGI395221[] = {{&S80461[24], 5}, {NULL}};
const Segment SGI395222[] = {{&S80532[0], 3}, {NULL}};
const Segment SGI395223[] = {{&S80532[43], 8}, {NULL}};
const Segment SGI395224[] = {{&S80532[28], 4}, {NULL}};
const Segment SGI395225[] = {{&S80532[5], 13}, {NULL}};
const Segment SGI395226[] = {{&S80535[0], 3}, {NULL}};
const Segment SGI395227[] = {{&S80535[53], 8}, {NULL}};
const Segment SGI395228[] = {{&S80535[28], 14}, {NULL}};
const Segment SGI395229[] = {{&S80535[5], 13}, {NULL}};
const Segment SGI395230[] = {{&S80593[0], 3}, {NULL}};
const Segment SGI395231[] = {{&S80593[45], 8}, {NULL}};
const Segment SGI395232[] = {{&S80593[28], 6}, {NULL}};
const Segment SGI395233[] = {{&S80593[5], 13}, {NULL}};
const Segment SGI395234[] = {{&S80594[0], 3}, {NULL}};
const Segment SGI395235[] = {{&S80594[45], 8}, {NULL}};
const Segment SGI395236[] = {{&S80594[28], 6}, {NULL}};
const Segment SGI395237[] = {{&S80594[5], 13}, {NULL}};
const Segment SGI395238[] = {{&S80835[0], 3}, {NULL}};
const Segment SGI395239[] = {{&S80835[48], 8}, {NULL}};
const Segment SGI395240[] = {{&S80835[28], 9}, {NULL}};
const Segment SGI395241[] = {{&S80835[5], 13}, {NULL}};
const Segment SGI395242[] = {{&S81339[0], 3}, {NULL}};
const Segment SGI395243[] = {{&S81339[45], 8}, {NULL}};
const Segment SGI395244[] = {{&S81339[28], 6}, {NULL}};
const Segment SGI395245[] = {{&S81339[5], 13}, {NULL}};
const Segment SGI395246[] = {{&S81901[0], 3}, {NULL}};
const Segment SGI395247[] = {{&S81901[45], 8}, {NULL}};
const Segment SGI395248[] = {{&S81901[28], 6}, {NULL}};
const Segment SGI395249[] = {{&S81901[5], 13}, {NULL}};
const Segment SGI395250[] = {{&S81961[0], 3}, {NULL}};
const Segment SGI395251[] = {{&S81961[39], 8}, {NULL}};
const Segment SGI395252[] = {{&S81961[25], 4}, {NULL}};
const Segment SGI395253[] = {{&S81975[0], 3}, {NULL}};
const Segment SGI395254[] = {{&S81975[47], 8}, {NULL}};
const Segment SGI395255[] = {{&S81975[28], 8}, {NULL}};
const Segment SGI395256[] = {{&S81975[5], 13}, {NULL}};
const Segment SGI395257[] = {{&S82402[0], 3}, {NULL}};
const Segment SGI395258[] = {{&S82402[32], 8}, {NULL}};
const Segment SGI395259[] = {{&S82402[22], 7}, {NULL}};
const Segment SGI395260[] = {{&S82402[6], 7}, {NULL}};
const Segment SGI395261[] = {{&S82567[0], 3}, {NULL}};
const Segment SGI395262[] = {{&S82567[49], 8}, {NULL}};
const Segment SGI395263[] = {{&S82567[28], 9}, {NULL}};
const Segment SGI395264[] = {{&S82567[5], 13}, {NULL}};
const Segment SGI395265[] = {{&S82844[0], 3}, {NULL}};
const Segment SGI395266[] = {{&S82844[49], 8}, {NULL}};
const Segment SGI395267[] = {{&S82844[28], 9}, {NULL}};
const Segment SGI395268[] = {{&S82844[5], 13}, {NULL}};
const Segment SGI395269[] = {{&S82915[0], 3}, {NULL}};
const Segment SGI395270[] = {{&S82915[48], 8}, {NULL}};
const Segment SGI395271[] = {{&S82915[28], 9}, {NULL}};
const Segment SGI395272[] = {{&S82915[5], 13}, {NULL}};
const Segment SGI395273[] = {{&S83071[0], 3}, {NULL}};
const Segment SGI395274[] = {{&S83071[48], 8}, {NULL}};
const Segment SGI395275[] = {{&S83071[28], 9}, {NULL}};
const Segment SGI395276[] = {{&S83071[5], 13}, {NULL}};
const Segment SGI395277[] = {{&S83157[0], 3}, {NULL}};
const Segment SGI395278[] = {{&S83157[49], 8}, {NULL}};
const Segment SGI395279[] = {{&S83157[28], 10}, {NULL}};
const Segment SGI395280[] = {{&S83157[5], 13}, {NULL}};
const Segment SGI395281[] = {{&S83413[0], 3}, {NULL}};
const Segment SGI395282[] = {{&S83413[43], 8}, {NULL}};
const Segment SGI395283[] = {{&S83413[28], 4}, {NULL}};
const Segment SGI395284[] = {{&S83413[5], 13}, {NULL}};
const Segment SGI395285[] = {{&S83414[0], 3}, {NULL}};
const Segment SGI395286[] = {{&S83414[50], 8}, {NULL}};
const Segment SGI395287[] = {{&S83414[28], 11}, {NULL}};
const Segment SGI395288[] = {{&S83414[5], 13}, {NULL}};
const Segment SGI395289[] = {{&S83415[0], 3}, {NULL}};
const Segment SGI395290[] = {{&S83415[46], 8}, {NULL}};
const Segment SGI395291[] = {{&S83415[28], 7}, {NULL}};
const Segment SGI395292[] = {{&S83415[5], 13}, {NULL}};
const Segment SGI395293[] = {{&S5464[25], 13}, {NULL}};
const Segment SGI395294[] = {{&S5466[25], 13}, {NULL}};
const Segment SGI395295[] = {{&S5477[25], 13}, {NULL}};
const Segment SGI395296[] = {{&S46750[12], 13}, {NULL}};
const Segment SGI395297[] = {{&S46750[33], 5}, {NULL}};
const Segment SGI395298[] = {{&S46751[12], 13}, {NULL}};
const Segment SGI395299[] = {{&S46751[33], 5}, {NULL}};
const Segment SGI395300[] = {{&S46753[12], 13}, {NULL}};
const Segment SGI395301[] = {{&S46753[31], 5}, {NULL}};
const Segment SGI395302[] = {{&S84206[0], 3}, {NULL}};
const Segment SGI395303[] = {{&S84206[48], 8}, {NULL}};
const Segment SGI395304[] = {{&S84206[28], 9}, {NULL}};
const Segment SGI395305[] = {{&S84206[5], 13}, {NULL}};
const Segment SGI395306[] = {{&S84514[0], 3}, {NULL}};
const Segment SGI395307[] = {{&S84514[47], 8}, {NULL}};
const Segment SGI395308[] = {{&S84514[28], 8}, {NULL}};
const Segment SGI395309[] = {{&S84514[5], 13}, {NULL}};
const Segment SGI395310[] = {{&S84515[0], 3}, {NULL}};
const Segment SGI395311[] = {{&S84515[47], 8}, {NULL}};
const Segment SGI395312[] = {{&S84515[28], 8}, {NULL}};
const Segment SGI395313[] = {{&S84515[5], 13}, {NULL}};
const Segment SGI395314[] = {{&S84723[0], 3}, {NULL}};
const Segment SGI395315[] = {{&S84723[43], 8}, {NULL}};
const Segment SGI395316[] = {{&S84723[26], 6}, {NULL}};
const Segment SGI395317[] = {{&S84723[5], 13}, {NULL}};
const Segment SGI395318[] = {{&S84724[0], 3}, {NULL}};
const Segment SGI395319[] = {{&S84724[51], 8}, {NULL}};
const Segment SGI395320[] = {{&S84724[34], 6}, {NULL}};
const Segment SGI395321[] = {{&S84724[5], 13}, {NULL}};
const Segment SGI395322[] = {{&S84883[0], 3}, {NULL}};
const Segment SGI395323[] = {{&S84883[53], 8}, {NULL}};
const Segment SGI395324[] = {{&S84883[28], 14}, {NULL}};
const Segment SGI395325[] = {{&S84883[5], 13}, {NULL}};
const Segment SGI395326[] = {{&S85146[0], 3}, {NULL}};
const Segment SGI395327[] = {{&S85146[44], 8}, {NULL}};
const Segment SGI395328[] = {{&S85146[24], 10}, {NULL}};
const Segment SGI395329[] = {{&S85177[0], 3}, {NULL}};
const Segment SGI395330[] = {{&S85177[49], 8}, {NULL}};
const Segment SGI395331[] = {{&S85177[28], 9}, {NULL}};
const Segment SGI395332[] = {{&S85177[5], 13}, {NULL}};
const Segment SGI395333[] = {{&S85255[0], 3}, {NULL}};
const Segment SGI395334[] = {{&S85255[45], 8}, {NULL}};
const Segment SGI395335[] = {{&S85255[28], 6}, {NULL}};
const Segment SGI395336[] = {{&S85255[5], 13}, {NULL}};
const Segment SGI395337[] = {{&S85569[0], 3}, {NULL}};
const Segment SGI395338[] = {{&S85569[51], 8}, {NULL}};
const Segment SGI395339[] = {{&S85569[28], 12}, {NULL}};
const Segment SGI395340[] = {{&S85569[5], 13}, {NULL}};
const Segment SGI395341[] = {{&S85579[0], 3}, {NULL}};
const Segment SGI395342[] = {{&S85579[46], 8}, {NULL}};
const Segment SGI395343[] = {{&S85579[26], 9}, {NULL}};
const Segment SGI395344[] = {{&S85579[5], 13}, {NULL}};
const Segment SGI395345[] = {{&S85580[0], 3}, {NULL}};
const Segment SGI395346[] = {{&S85580[43], 8}, {NULL}};
const Segment SGI395347[] = {{&S85580[28], 4}, {NULL}};
const Segment SGI395348[] = {{&S85580[5], 13}, {NULL}};
const Segment SGI395349[] = {{&S85596[0], 3}, {NULL}};
const Segment SGI395350[] = {{&S85596[52], 8}, {NULL}};
const Segment SGI395351[] = {{&S85596[28], 13}, {NULL}};
const Segment SGI395352[] = {{&S85596[5], 13}, {NULL}};
const Segment SGI395353[] = {{&S85687[0], 3}, {NULL}};
const Segment SGI395354[] = {{&S85687[50], 8}, {NULL}};
const Segment SGI395355[] = {{&S85687[28], 11}, {NULL}};
const Segment SGI395356[] = {{&S85687[5], 13}, {NULL}};
const Segment SGI395357[] = {{&S85947[0], 3}, {NULL}};
const Segment SGI395358[] = {{&S85947[43], 8}, {NULL}};
const Segment SGI395359[] = {{&S85947[28], 4}, {NULL}};
const Segment SGI395360[] = {{&S85947[5], 13}, {NULL}};
const Segment SGI395361[] = {{&S86120[0], 3}, {NULL}};
const Segment SGI395362[] = {{&S86120[50], 8}, {NULL}};
const Segment SGI395363[] = {{&S86120[28], 11}, {NULL}};
const Segment SGI395364[] = {{&S86120[5], 13}, {NULL}};
const Segment SGI395365[] = {{&S86123[0], 3}, {NULL}};
const Segment SGI395366[] = {{&S86123[45], 8}, {NULL}};
const Segment SGI395367[] = {{&S86123[25], 10}, {NULL}};
const Segment SGI395368[] = {{&S86359[0], 3}, {NULL}};
const Segment SGI395369[] = {{&S86359[54], 8}, {NULL}};
const Segment SGI395370[] = {{&S86359[28], 15}, {NULL}};
const Segment SGI395371[] = {{&S86359[5], 13}, {NULL}};
const Segment SGI395372[] = {{&S86442[0], 3}, {NULL}};
const Segment SGI395373[] = {{&S86442[43], 8}, {NULL}};
const Segment SGI395374[] = {{&S86442[25], 8}, {NULL}};
const Segment SGI395375[] = {{&S86698[0], 3}, {NULL}};
const Segment SGI395376[] = {{&S86698[48], 8}, {NULL}};
const Segment SGI395377[] = {{&S86698[28], 9}, {NULL}};
const Segment SGI395378[] = {{&S86698[5], 13}, {NULL}};
const Segment SGI395379[] = {{&S86805[0], 3}, {NULL}};
const Segment SGI395380[] = {{&S86805[58], 8}, {NULL}};
const Segment SGI395381[] = {{&S86805[34], 13}, {NULL}};
const Segment SGI395382[] = {{&S86805[5], 13}, {NULL}};
const Segment SGI395383[] = {{&S87549[0], 3}, {NULL}};
const Segment SGI395384[] = {{&S87549[32], 8}, {NULL}};
const Segment SGI395385[] = {{&S87549[22], 7}, {NULL}};
const Segment SGI395386[] = {{&S87549[6], 7}, {NULL}};
const Segment SGI395387[] = {{&S87550[0], 3}, {NULL}};
const Segment SGI395388[] = {{&S87550[32], 8}, {NULL}};
const Segment SGI395389[] = {{&S87550[22], 7}, {NULL}};
const Segment SGI395390[] = {{&S87550[6], 7}, {NULL}};
const Segment SGI395391[] = {{&S87551[0], 3}, {NULL}};
const Segment SGI395392[] = {{&S87551[32], 8}, {NULL}};
const Segment SGI395393[] = {{&S87551[22], 7}, {NULL}};
const Segment SGI395394[] = {{&S87551[6], 7}, {NULL}};
const Segment SGI395395[] = {{&S87552[0], 3}, {NULL}};
const Segment SGI395396[] = {{&S87552[30], 8}, {NULL}};
const Segment SGI395397[] = {{&S87552[20], 7}, {NULL}};
const Segment SGI395398[] = {{&S87552[6], 7}, {NULL}};
const Segment SGI395399[] = {{&S87961[0], 3}, {NULL}};
const Segment SGI395400[] = {{&S87961[51], 8}, {NULL}};
const Segment SGI395401[] = {{&S87961[28], 12}, {NULL}};
const Segment SGI395402[] = {{&S87961[5], 13}, {NULL}};
const Segment SGI395403[] = {{&S87964[0], 3}, {NULL}};
const Segment SGI395404[] = {{&S87964[47], 8}, {NULL}};
const Segment SGI395405[] = {{&S87964[28], 8}, {NULL}};
const Segment SGI395406[] = {{&S87964[5], 13}, {NULL}};
const Segment SGI395407[] = {{&S87965[0], 3}, {NULL}};
const Segment SGI395408[] = {{&S87965[47], 8}, {NULL}};
const Segment SGI395409[] = {{&S87965[28], 8}, {NULL}};
const Segment SGI395410[] = {{&S87965[5], 13}, {NULL}};
const Segment SGI395411[] = {{&S87989[0], 3}, {NULL}};
const Segment SGI395412[] = {{&S87989[43], 8}, {NULL}};
const Segment SGI395413[] = {{&S87989[28], 4}, {NULL}};
const Segment SGI395414[] = {{&S87989[5], 13}, {NULL}};
const Segment SGI395415[] = {{&S87990[0], 3}, {NULL}};
const Segment SGI395416[] = {{&S87990[43], 8}, {NULL}};
const Segment SGI395417[] = {{&S87990[28], 4}, {NULL}};
const Segment SGI395418[] = {{&S87990[5], 13}, {NULL}};
const Segment SGI395419[] = {{&S88066[0], 3}, {NULL}};
const Segment SGI395420[] = {{&S88066[40], 8}, {NULL}};
const Segment SGI395421[] = {{&S88066[25], 5}, {NULL}};
const Segment SGI395422[] = {{&S88075[0], 3}, {NULL}};
const Segment SGI395423[] = {{&S88075[43], 8}, {NULL}};
const Segment SGI395424[] = {{&S88075[28], 4}, {NULL}};
const Segment SGI395425[] = {{&S88075[5], 13}, {NULL}};
const Segment SGI395426[] = {{&S88087[0], 3}, {NULL}};
const Segment SGI395427[] = {{&S88087[48], 8}, {NULL}};
const Segment SGI395428[] = {{&S88087[28], 9}, {NULL}};
const Segment SGI395429[] = {{&S88087[5], 13}, {NULL}};
const Segment SGI395430[] = {{&S88208[0], 3}, {NULL}};
const Segment SGI395431[] = {{&S88208[28], 13}, {NULL}};
const Segment SGI395432[] = {{&S88208[5], 13}, {NULL}};
const Segment SGI395433[] = {{&S88311[0], 3}, {NULL}};
const Segment SGI395434[] = {{&S88311[47], 8}, {NULL}};
const Segment SGI395435[] = {{&S88311[28], 8}, {NULL}};
const Segment SGI395436[] = {{&S88311[5], 13}, {NULL}};
const Segment SGI395437[] = {{&S88312[0], 3}, {NULL}};
const Segment SGI395438[] = {{&S88312[47], 8}, {NULL}};
const Segment SGI395439[] = {{&S88312[28], 8}, {NULL}};
const Segment SGI395440[] = {{&S88312[5], 13}, {NULL}};
const Segment SGI395441[] = {{&S88441[0], 3}, {NULL}};
const Segment SGI395442[] = {{&S88441[48], 8}, {NULL}};
const Segment SGI395443[] = {{&S88441[28], 9}, {NULL}};
const Segment SGI395444[] = {{&S88441[5], 13}, {NULL}};
const Segment SGI395445[] = {{&S88456[0], 3}, {NULL}};
const Segment SGI395446[] = {{&S88456[42], 8}, {NULL}};
const Segment SGI395447[] = {{&S88456[28], 3}, {NULL}};
const Segment SGI395448[] = {{&S88456[5], 13}, {NULL}};
const Segment SGI395449[] = {{&S88493[0], 3}, {NULL}};
const Segment SGI395450[] = {{&S88493[52], 8}, {NULL}};
const Segment SGI395451[] = {{&S88493[28], 13}, {NULL}};
const Segment SGI395452[] = {{&S88493[5], 13}, {NULL}};
const Segment SGI395453[] = {{&S88617[0], 3}, {NULL}};
const Segment SGI395454[] = {{&S88617[47], 8}, {NULL}};
const Segment SGI395455[] = {{&S88617[28], 8}, {NULL}};
const Segment SGI395456[] = {{&S88617[5], 13}, {NULL}};
const Segment SGI395457[] = {{&S88621[0], 3}, {NULL}};
const Segment SGI395458[] = {{&S88621[48], 8}, {NULL}};
const Segment SGI395459[] = {{&S88621[28], 9}, {NULL}};
const Segment SGI395460[] = {{&S88621[5], 13}, {NULL}};
const Segment SGI395461[] = {{&S88747[0], 3}, {NULL}};
const Segment SGI395462[] = {{&S88747[48], 8}, {NULL}};
const Segment SGI395463[] = {{&S88747[28], 9}, {NULL}};
const Segment SGI395464[] = {{&S88747[5], 13}, {NULL}};
const Segment SGI395465[] = {{&S88997[0], 3}, {NULL}};
const Segment SGI395466[] = {{&S88997[45], 8}, {NULL}};
const Segment SGI395467[] = {{&S88997[28], 6}, {NULL}};
const Segment SGI395468[] = {{&S88997[5], 13}, {NULL}};
const Segment SGI395469[] = {{&S89245[0], 3}, {NULL}};
const Segment SGI395470[] = {{&S89245[50], 8}, {NULL}};
const Segment SGI395471[] = {{&S89245[28], 11}, {NULL}};
const Segment SGI395472[] = {{&S89245[5], 13}, {NULL}};
const Segment SGI395473[] = {{&S89298[0], 3}, {NULL}};
const Segment SGI395474[] = {{&S89298[46], 8}, {NULL}};
const Segment SGI395475[] = {{&S89298[28], 6}, {NULL}};
const Segment SGI395476[] = {{&S89298[5], 13}, {NULL}};
const Segment SGI395477[] = {{&S89304[0], 3}, {NULL}};
const Segment SGI395478[] = {{&S89304[43], 8}, {NULL}};
const Segment SGI395479[] = {{&S89304[28], 4}, {NULL}};
const Segment SGI395480[] = {{&S89304[5], 13}, {NULL}};
const Segment SGI395481[] = {{&S89445[0], 3}, {NULL}};
const Segment SGI395482[] = {{&S89445[44], 8}, {NULL}};
const Segment SGI395483[] = {{&S89445[28], 5}, {NULL}};
const Segment SGI395484[] = {{&S89445[5], 13}, {NULL}};
const Segment SGI395485[] = {{&S89582[0], 3}, {NULL}};
const Segment SGI395486[] = {{&S89582[46], 8}, {NULL}};
const Segment SGI395487[] = {{&S89582[28], 7}, {NULL}};
const Segment SGI395488[] = {{&S89582[5], 13}, {NULL}};
const Segment SGI395489[] = {{&S89583[0], 3}, {NULL}};
const Segment SGI395490[] = {{&S89583[42], 8}, {NULL}};
const Segment SGI395491[] = {{&S89583[28], 3}, {NULL}};
const Segment SGI395492[] = {{&S89583[5], 13}, {NULL}};
const Segment SGI395493[] = {{&S89588[0], 3}, {NULL}};
const Segment SGI395494[] = {{&S89588[48], 8}, {NULL}};
const Segment SGI395495[] = {{&S89588[28], 9}, {NULL}};
const Segment SGI395496[] = {{&S89588[5], 13}, {NULL}};
const Segment SGI395497[] = {{&S89749[0], 3}, {NULL}};
const Segment SGI395498[] = {{&S89749[54], 8}, {NULL}};
const Segment SGI395499[] = {{&S89749[34], 9}, {NULL}};
const Segment SGI395500[] = {{&S89749[5], 13}, {NULL}};
const Segment SGI395501[] = {{&S89755[0], 3}, {NULL}};
const Segment SGI395502[] = {{&S89755[45], 8}, {NULL}};
const Segment SGI395503[] = {{&S89755[28], 6}, {NULL}};
const Segment SGI395504[] = {{&S89755[5], 13}, {NULL}};
const Segment SGI395505[] = {{&S89784[0], 3}, {NULL}};
const Segment SGI395506[] = {{&S89784[44], 8}, {NULL}};
const Segment SGI395507[] = {{&S89784[28], 5}, {NULL}};
const Segment SGI395508[] = {{&S89784[5], 13}, {NULL}};
const Segment SGI395509[] = {{&S89870[0], 3}, {NULL}};
const Segment SGI395510[] = {{&S89870[50], 8}, {NULL}};
const Segment SGI395511[] = {{&S89870[28], 11}, {NULL}};
const Segment SGI395512[] = {{&S89870[5], 13}, {NULL}};
const Segment SGI395513[] = {{&S89872[0], 3}, {NULL}};
const Segment SGI395514[] = {{&S89872[50], 8}, {NULL}};
const Segment SGI395515[] = {{&S89872[28], 11}, {NULL}};
const Segment SGI395516[] = {{&S89872[5], 13}, {NULL}};
const Segment SGI395517[] = {{&S89898[0], 3}, {NULL}};
const Segment SGI395518[] = {{&S89898[52], 8}, {NULL}};
const Segment SGI395519[] = {{&S89898[28], 13}, {NULL}};
const Segment SGI395520[] = {{&S89898[5], 13}, {NULL}};
const Segment SGI395521[] = {{&S90234[0], 3}, {NULL}};
const Segment SGI395522[] = {{&S90234[54], 8}, {NULL}};
const Segment SGI395523[] = {{&S90234[28], 15}, {NULL}};
const Segment SGI395524[] = {{&S90234[5], 13}, {NULL}};
const Segment SGI395525[] = {{&S90274[0], 3}, {NULL}};
const Segment SGI395526[] = {{&S90274[47], 8}, {NULL}};
const Segment SGI395527[] = {{&S90274[28], 8}, {NULL}};
const Segment SGI395528[] = {{&S90274[5], 13}, {NULL}};
const Segment SGI395529[] = {{&S90585[0], 3}, {NULL}};
const Segment SGI395530[] = {{&S90585[49], 8}, {NULL}};
const Segment SGI395531[] = {{&S90585[28], 10}, {NULL}};
const Segment SGI395532[] = {{&S90585[5], 13}, {NULL}};
const Segment SGI395533[] = {{&S90602[0], 3}, {NULL}};
const Segment SGI395534[] = {{&S90602[47], 8}, {NULL}};
const Segment SGI395535[] = {{&S90602[28], 7}, {NULL}};
const Segment SGI395536[] = {{&S90602[5], 13}, {NULL}};
const Segment SGI395537[] = {{&S90623[0], 3}, {NULL}};
const Segment SGI395538[] = {{&S90623[48], 8}, {NULL}};
const Segment SGI395539[] = {{&S90623[28], 9}, {NULL}};
const Segment SGI395540[] = {{&S90623[5], 13}, {NULL}};
const Segment SGI395541[] = {{&S90776[0], 3}, {NULL}};
const Segment SGI395542[] = {{&S90776[45], 8}, {NULL}};
const Segment SGI395543[] = {{&S90776[25], 10}, {NULL}};
const Segment SGI395544[] = {{&S90957[0], 3}, {NULL}};
const Segment SGI395545[] = {{&S90957[45], 8}, {NULL}};
const Segment SGI395546[] = {{&S90957[28], 6}, {NULL}};
const Segment SGI395547[] = {{&S90957[5], 13}, {NULL}};
const Segment SGI395548[] = {{&S90982[0], 3}, {NULL}};
const Segment SGI395549[] = {{&S90982[49], 8}, {NULL}};
const Segment SGI395550[] = {{&S90982[28], 9}, {NULL}};
const Segment SGI395551[] = {{&S90982[5], 13}, {NULL}};
const Segment SGI395552[] = {{&S54066[12], 13}, {NULL}};
const Segment SGI395553[] = {{&S54066[33], 7}, {NULL}};
const Segment SGI395554[] = {{&S54069[12], 13}, {NULL}};
const Segment SGI395555[] = {{&S54069[33], 7}, {NULL}};
const Segment SGI395556[] = {{&S91182[0], 3}, {NULL}};
const Segment SGI395557[] = {{&S91182[48], 8}, {NULL}};
const Segment SGI395558[] = {{&S91182[28], 9}, {NULL}};
const Segment SGI395559[] = {{&S91182[5], 13}, {NULL}};
const Segment SGI395560[] = {{&S91335[0], 3}, {NULL}};
const Segment SGI395561[] = {{&S91335[50], 8}, {NULL}};
const Segment SGI395562[] = {{&S91335[28], 10}, {NULL}};
const Segment SGI395563[] = {{&S91335[5], 13}, {NULL}};
const Segment SGI395564[] = {{&S91353[0], 3}, {NULL}};
const Segment SGI395565[] = {{&S91353[45], 8}, {NULL}};
const Segment SGI395566[] = {{&S91353[28], 6}, {NULL}};
const Segment SGI395567[] = {{&S91353[5], 13}, {NULL}};
const Segment SGI395568[] = {{&S91386[0], 3}, {NULL}};
const Segment SGI395569[] = {{&S91386[50], 8}, {NULL}};
const Segment SGI395570[] = {{&S91386[28], 11}, {NULL}};
const Segment SGI395571[] = {{&S91386[5], 13}, {NULL}};
const Segment SGI395572[] = {{&S91387[0], 3}, {NULL}};
const Segment SGI395573[] = {{&S91387[49], 8}, {NULL}};
const Segment SGI395574[] = {{&S91387[28], 10}, {NULL}};
const Segment SGI395575[] = {{&S91387[5], 13}, {NULL}};
const Segment SGI395576[] = {{&S91528[0], 3}, {NULL}};
const Segment SGI395577[] = {{&S91528[47], 8}, {NULL}};
const Segment SGI395578[] = {{&S91528[28], 8}, {NULL}};
const Segment SGI395579[] = {{&S91528[5], 13}, {NULL}};
const Segment SGI395580[] = {{&S91592[0], 3}, {NULL}};
const Segment SGI395581[] = {{&S91592[48], 8}, {NULL}};
const Segment SGI395582[] = {{&S91592[28], 9}, {NULL}};
const Segment SGI395583[] = {{&S91592[5], 13}, {NULL}};
const Segment SGI395584[] = {{&S91599[0], 3}, {NULL}};
const Segment SGI395585[] = {{&S91599[48], 8}, {NULL}};
const Segment SGI395586[] = {{&S91599[28], 9}, {NULL}};
const Segment SGI395587[] = {{&S91599[5], 13}, {NULL}};
const Segment SGI395588[] = {{&S91937[0], 3}, {NULL}};
const Segment SGI395589[] = {{&S91937[45], 8}, {NULL}};
const Segment SGI395590[] = {{&S91937[28], 6}, {NULL}};
const Segment SGI395591[] = {{&S91937[5], 13}, {NULL}};
const Segment SGI395592[] = {{&S92154[0], 3}, {NULL}};
const Segment SGI395593[] = {{&S92154[42], 8}, {NULL}};
const Segment SGI395594[] = {{&S92154[28], 3}, {NULL}};
const Segment SGI395595[] = {{&S92154[5], 13}, {NULL}};
const Segment SGI395596[] = {{&S92321[0], 3}, {NULL}};
const Segment SGI395597[] = {{&S92321[45], 8}, {NULL}};
const Segment SGI395598[] = {{&S92321[24], 11}, {NULL}};
const Segment SGI395599[] = {{&S92353[0], 3}, {NULL}};
const Segment SGI395600[] = {{&S92353[47], 8}, {NULL}};
const Segment SGI395601[] = {{&S92353[26], 9}, {NULL}};
const Segment SGI395602[] = {{&S92353[5], 13}, {NULL}};
const Segment SGI395603[] = {{&S55227[12], 13}, {NULL}};
const Segment SGI395604[] = {{&S55227[34], 5}, {NULL}};
const Segment SGI395605[] = {{&S92402[0], 3}, {NULL}};
const Segment SGI395606[] = {{&S92402[46], 8}, {NULL}};
const Segment SGI395607[] = {{&S92402[28], 6}, {NULL}};
const Segment SGI395608[] = {{&S92402[5], 13}, {NULL}};
const Segment SGI395609[] = {{&S92560[0], 3}, {NULL}};
const Segment SGI395610[] = {{&S92560[50], 8}, {NULL}};
const Segment SGI395611[] = {{&S92560[28], 11}, {NULL}};
const Segment SGI395612[] = {{&S92560[5], 13}, {NULL}};
const Segment SGI395613[] = {{&S92718[0], 3}, {NULL}};
const Segment SGI395614[] = {{&S92718[46], 8}, {NULL}};
const Segment SGI395615[] = {{&S92718[26], 9}, {NULL}};
const Segment SGI395616[] = {{&S92718[5], 13}, {NULL}};
const Segment SGI395617[] = {{&S93101[0], 3}, {NULL}};
const Segment SGI395618[] = {{&S93101[39], 8}, {NULL}};
const Segment SGI395619[] = {{&S93101[24], 5}, {NULL}};
const Segment SGI395620[] = {{&S93114[0], 3}, {NULL}};
const Segment SGI395621[] = {{&S93114[51], 8}, {NULL}};
const Segment SGI395622[] = {{&S93114[28], 12}, {NULL}};
const Segment SGI395623[] = {{&S93114[5], 13}, {NULL}};
const Segment SGI395624[] = {{&S93549[0], 3}, {NULL}};
const Segment SGI395625[] = {{&S93549[51], 8}, {NULL}};
const Segment SGI395626[] = {{&S93549[28], 12}, {NULL}};
const Segment SGI395627[] = {{&S93549[5], 13}, {NULL}};
const Segment SGI395628[] = {{&S93604[0], 3}, {NULL}};
const Segment SGI395629[] = {{&S93604[44], 8}, {NULL}};
const Segment SGI395630[] = {{&S93604[28], 5}, {NULL}};
const Segment SGI395631[] = {{&S93604[5], 13}, {NULL}};
const Segment SGI395632[] = {{&S93850[0], 3}, {NULL}};
const Segment SGI395633[] = {{&S93850[48], 8}, {NULL}};
const Segment SGI395634[] = {{&S93850[28], 9}, {NULL}};
const Segment SGI395635[] = {{&S93850[5], 13}, {NULL}};
const Segment SGI395636[] = {{&S93894[0], 3}, {NULL}};
const Segment SGI395637[] = {{&S93894[46], 8}, {NULL}};
const Segment SGI395638[] = {{&S93894[26], 9}, {NULL}};
const Segment SGI395639[] = {{&S93894[5], 13}, {NULL}};
const Segment SGI395640[] = {{&S93997[0], 3}, {NULL}};
const Segment SGI395641[] = {{&S93997[32], 8}, {NULL}};
const Segment SGI395642[] = {{&S93997[22], 7}, {NULL}};
const Segment SGI395643[] = {{&S93997[6], 7}, {NULL}};
const Segment SGI395644[] = {{&S93998[0], 3}, {NULL}};
const Segment SGI395645[] = {{&S93998[32], 8}, {NULL}};
const Segment SGI395646[] = {{&S93998[22], 7}, {NULL}};
const Segment SGI395647[] = {{&S93998[6], 7}, {NULL}};
const Segment SGI395648[] = {{&S93999[0], 3}, {NULL}};
const Segment SGI395649[] = {{&S93999[30], 8}, {NULL}};
const Segment SGI395650[] = {{&S93999[20], 7}, {NULL}};
const Segment SGI395651[] = {{&S93999[6], 7}, {NULL}};
const Segment SGI395652[] = {{&S4748[5], 3}, {NULL}};
const Segment SGI395653[] = {{&S4751[5], 3}, {NULL}};
const Segment SGI395654[] = {{&S4777[5], 3}, {NULL}};
const Segment SGI395655[] = {{&S4846[5], 3}, {NULL}};
const Segment SGI395656[] = {{&S4862[5], 3}, {NULL}};
const Segment SGI395657[] = {{&S94257[0], 3}, {NULL}};
const Segment SGI395658[] = {{&S94257[44], 8}, {NULL}};
const Segment SGI395659[] = {{&S94257[28], 5}, {NULL}};
const Segment SGI395660[] = {{&S94257[5], 13}, {NULL}};
const Segment SGI395661[] = {{&S94270[0], 3}, {NULL}};
const Segment SGI395662[] = {{&S94270[45], 8}, {NULL}};
const Segment SGI395663[] = {{&S94270[28], 6}, {NULL}};
const Segment SGI395664[] = {{&S94270[5], 13}, {NULL}};
const Segment SGI395665[] = {{&S94488[0], 3}, {NULL}};
const Segment SGI395666[] = {{&S94488[47], 8}, {NULL}};
const Segment SGI395667[] = {{&S94488[28], 8}, {NULL}};
const Segment SGI395668[] = {{&S94488[5], 13}, {NULL}};
const Segment SGI395669[] = {{&S94489[0], 3}, {NULL}};
const Segment SGI395670[] = {{&S94489[47], 8}, {NULL}};
const Segment SGI395671[] = {{&S94489[28], 8}, {NULL}};
const Segment SGI395672[] = {{&S94489[5], 13}, {NULL}};
const Segment SGI395673[] = {{&S94553[0], 3}, {NULL}};
const Segment SGI395674[] = {{&S94553[52], 8}, {NULL}};
const Segment SGI395675[] = {{&S94553[28], 13}, {NULL}};
const Segment SGI395676[] = {{&S94553[5], 13}, {NULL}};
const Segment SGI395677[] = {{&S94692[0], 3}, {NULL}};
const Segment SGI395678[] = {{&S94692[50], 8}, {NULL}};
const Segment SGI395679[] = {{&S94692[28], 11}, {NULL}};
const Segment SGI395680[] = {{&S94692[5], 13}, {NULL}};
const Segment SGI395681[] = {{&S94710[0], 3}, {NULL}};
const Segment SGI395682[] = {{&S94710[47], 8}, {NULL}};
const Segment SGI395683[] = {{&S94710[28], 7}, {NULL}};
const Segment SGI395684[] = {{&S94710[5], 13}, {NULL}};
const Segment SGI395685[] = {{&S94726[0], 3}, {NULL}};
const Segment SGI395686[] = {{&S94726[43], 8}, {NULL}};
const Segment SGI395687[] = {{&S94726[28], 4}, {NULL}};
const Segment SGI395688[] = {{&S94726[5], 13}, {NULL}};
const Segment SGI395689[] = {{&S94935[0], 3}, {NULL}};
const Segment SGI395690[] = {{&S94935[39], 8}, {NULL}};
const Segment SGI395691[] = {{&S94935[24], 5}, {NULL}};
const Segment SGI395692[] = {{&S95016[0], 3}, {NULL}};
const Segment SGI395693[] = {{&S95016[48], 8}, {NULL}};
const Segment SGI395694[] = {{&S95016[28], 9}, {NULL}};
const Segment SGI395695[] = {{&S95016[5], 13}, {NULL}};
const Segment SGI395696[] = {{&S95177[0], 3}, {NULL}};
const Segment SGI395697[] = {{&S95177[42], 8}, {NULL}};
const Segment SGI395698[] = {{&S95177[24], 8}, {NULL}};
const Segment SGI395699[] = {{&S95403[0], 3}, {NULL}};
const Segment SGI395700[] = {{&S95403[44], 8}, {NULL}};
const Segment SGI395701[] = {{&S95403[30], 4}, {NULL}};
const Segment SGI395702[] = {{&S95404[0], 3}, {NULL}};
const Segment SGI395703[] = {{&S95404[41], 8}, {NULL}};
const Segment SGI395704[] = {{&S95471[0], 3}, {NULL}};
const Segment SGI395705[] = {{&S95471[45], 8}, {NULL}};
const Segment SGI395706[] = {{&S95471[28], 5}, {NULL}};
const Segment SGI395707[] = {{&S95471[5], 13}, {NULL}};
const Segment SGI395708[] = {{&S95489[0], 3}, {NULL}};
const Segment SGI395709[] = {{&S95489[49], 8}, {NULL}};
const Segment SGI395710[] = {{&S95489[28], 10}, {NULL}};
const Segment SGI395711[] = {{&S95489[5], 13}, {NULL}};
const Segment SGI395712[] = {{&S95720[0], 3}, {NULL}};
const Segment SGI395713[] = {{&S95720[25], 11}, {NULL}};
const Segment SGI395714[] = {{&S95864[0], 3}, {NULL}};
const Segment SGI395715[] = {{&S95864[52], 8}, {NULL}};
const Segment SGI395716[] = {{&S95864[28], 13}, {NULL}};
const Segment SGI395717[] = {{&S95864[5], 13}, {NULL}};
const Segment SGI395718[] = {{&S96005[0], 3}, {NULL}};
const Segment SGI395719[] = {{&S96005[51], 8}, {NULL}};
const Segment SGI395720[] = {{&S96005[34], 6}, {NULL}};
const Segment SGI395721[] = {{&S96005[5], 13}, {NULL}};
const Segment SGI395722[] = {{&S96027[0], 3}, {NULL}};
const Segment SGI395723[] = {{&S96027[47], 8}, {NULL}};
const Segment SGI395724[] = {{&S96027[25], 12}, {NULL}};
const Segment SGI395725[] = {{&S58558[12], 13}, {NULL}};
const Segment SGI395726[] = {{&S58558[34], 7}, {NULL}};
const Segment SGI395727[] = {{&S96052[0], 3}, {NULL}};
const Segment SGI395728[] = {{&S96052[54], 8}, {NULL}};
const Segment SGI395729[] = {{&S96052[28], 15}, {NULL}};
const Segment SGI395730[] = {{&S96052[5], 13}, {NULL}};
const Segment SGI395731[] = {{&S96099[0], 3}, {NULL}};
const Segment SGI395732[] = {{&S96099[51], 8}, {NULL}};
const Segment SGI395733[] = {{&S96099[28], 12}, {NULL}};
const Segment SGI395734[] = {{&S96099[5], 13}, {NULL}};
const Segment SGI395735[] = {{&S96133[0], 3}, {NULL}};
const Segment SGI395736[] = {{&S96133[49], 8}, {NULL}};
const Segment SGI395737[] = {{&S96133[28], 10}, {NULL}};
const Segment SGI395738[] = {{&S96133[5], 13}, {NULL}};
const Segment SGI395739[] = {{&S96258[0], 3}, {NULL}};
const Segment SGI395740[] = {{&S96258[48], 8}, {NULL}};
const Segment SGI395741[] = {{&S96258[28], 9}, {NULL}};
const Segment SGI395742[] = {{&S96258[5], 13}, {NULL}};
const Segment SGI395743[] = {{&S96335[0], 3}, {NULL}};
const Segment SGI395744[] = {{&S96335[42], 8}, {NULL}};
const Segment SGI395745[] = {{&S96335[24], 8}, {NULL}};
const Segment SGI395746[] = {{&S96352[0], 3}, {NULL}};
const Segment SGI395747[] = {{&S96352[40], 8}, {NULL}};
const Segment SGI395748[] = {{&S96352[24], 6}, {NULL}};
const Segment SGI395749[] = {{&S96353[0], 3}, {NULL}};
const Segment SGI395750[] = {{&S96353[47], 8}, {NULL}};
const Segment SGI395751[] = {{&S96353[30], 7}, {NULL}};
const Segment SGI395752[] = {{&S96382[0], 3}, {NULL}};
const Segment SGI395753[] = {{&S96382[49], 8}, {NULL}};
const Segment SGI395754[] = {{&S96382[28], 9}, {NULL}};
const Segment SGI395755[] = {{&S96382[5], 13}, {NULL}};
const Segment SGI395756[] = {{&S96446[0], 3}, {NULL}};
const Segment SGI395757[] = {{&S96446[48], 8}, {NULL}};
const Segment SGI395758[] = {{&S96446[28], 9}, {NULL}};
const Segment SGI395759[] = {{&S96446[5], 13}, {NULL}};
const Segment SGI395760[] = {{&S96451[0], 3}, {NULL}};
const Segment SGI395761[] = {{&S96451[45], 8}, {NULL}};
const Segment SGI395762[] = {{&S96451[28], 6}, {NULL}};
const Segment SGI395763[] = {{&S96451[5], 13}, {NULL}};
const Segment SGI395764[] = {{&S96508[0], 3}, {NULL}};
const Segment SGI395765[] = {{&S96508[45], 8}, {NULL}};
const Segment SGI395766[] = {{&S96508[28], 6}, {NULL}};
const Segment SGI395767[] = {{&S96508[5], 13}, {NULL}};
const Segment SGI395768[] = {{&S96708[0], 3}, {NULL}};
const Segment SGI395769[] = {{&S96708[45], 8}, {NULL}};
const Segment SGI395770[] = {{&S96708[28], 6}, {NULL}};
const Segment SGI395771[] = {{&S96708[5], 13}, {NULL}};
const Segment SGI395772[] = {{&S96795[0], 3}, {NULL}};
const Segment SGI395773[] = {{&S96795[32], 8}, {NULL}};
const Segment SGI395774[] = {{&S96795[22], 7}, {NULL}};
const Segment SGI395775[] = {{&S96795[6], 7}, {NULL}};
const Segment SGI395776[] = {{&S96995[0], 3}, {NULL}};
const Segment SGI395777[] = {{&S96995[28], 13}, {NULL}};
const Segment SGI395778[] = {{&S96995[5], 13}, {NULL}};
const Segment SGI395779[] = {{&S97076[0], 3}, {NULL}};
const Segment SGI395780[] = {{&S97076[49], 8}, {NULL}};
const Segment SGI395781[] = {{&S97076[28], 10}, {NULL}};
const Segment SGI395782[] = {{&S97076[5], 13}, {NULL}};
const Segment SGI395783[] = {{&S97093[0], 3}, {NULL}};
const Segment SGI395784[] = {{&S97093[52], 8}, {NULL}};
const Segment SGI395785[] = {{&S97093[28], 13}, {NULL}};
const Segment SGI395786[] = {{&S97093[5], 13}, {NULL}};
const Segment SGI395787[] = {{&S97100[0], 3}, {NULL}};
const Segment SGI395788[] = {{&S97100[47], 8}, {NULL}};
const Segment SGI395789[] = {{&S97100[28], 8}, {NULL}};
const Segment SGI395790[] = {{&S97100[5], 13}, {NULL}};
const Segment SGI395791[] = {{&S97276[0], 3}, {NULL}};
const Segment SGI395792[] = {{&S97276[32], 8}, {NULL}};
const Segment SGI395793[] = {{&S97276[22], 7}, {NULL}};
const Segment SGI395794[] = {{&S97276[6], 7}, {NULL}};
const Segment SGI395795[] = {{&S97277[0], 3}, {NULL}};
const Segment SGI395796[] = {{&S97277[32], 8}, {NULL}};
const Segment SGI395797[] = {{&S97277[22], 7}, {NULL}};
const Segment SGI395798[] = {{&S97277[6], 7}, {NULL}};
const Segment SGI395799[] = {{&S97336[0], 3}, {NULL}};
const Segment SGI395800[] = {{&S97336[45], 8}, {NULL}};
const Segment SGI395801[] = {{&S97336[28], 6}, {NULL}};
const Segment SGI395802[] = {{&S97336[5], 13}, {NULL}};
const Segment SGI395803[] = {{&S97342[0], 3}, {NULL}};
const Segment SGI395804[] = {{&S97342[51], 8}, {NULL}};
const Segment SGI395805[] = {{&S97342[28], 12}, {NULL}};
const Segment SGI395806[] = {{&S97342[5], 13}, {NULL}};
const Segment SGI395807[] = {{&S97408[0], 3}, {NULL}};
const Segment SGI395808[] = {{&S97408[40], 8}, {NULL}};
const Segment SGI395809[] = {{&S97410[0], 3}, {NULL}};
const Segment SGI395810[] = {{&S97410[49], 8}, {NULL}};
const Segment SGI395811[] = {{&S97410[28], 9}, {NULL}};
const Segment SGI395812[] = {{&S97410[5], 13}, {NULL}};
const Segment SGI395813[] = {{&S97429[0], 3}, {NULL}};
const Segment SGI395814[] = {{&S97429[49], 8}, {NULL}};
const Segment SGI395815[] = {{&S97429[28], 9}, {NULL}};
const Segment SGI395816[] = {{&S97429[5], 13}, {NULL}};
const Segment SGI395817[] = {{&S97711[0], 3}, {NULL}};
const Segment SGI395818[] = {{&S97711[40], 8}, {NULL}};
const Segment SGI395819[] = {{&S97711[25], 5}, {NULL}};
const Segment SGI395820[] = {{&S97728[0], 3}, {NULL}};
const Segment SGI395821[] = {{&S97728[32], 8}, {NULL}};
const Segment SGI395822[] = {{&S97728[22], 7}, {NULL}};
const Segment SGI395823[] = {{&S97728[6], 7}, {NULL}};
const Segment SGI395824[] = {{&S97818[0], 3}, {NULL}};
const Segment SGI395825[] = {{&S97818[43], 8}, {NULL}};
const Segment SGI395826[] = {{&S97818[28], 4}, {NULL}};
const Segment SGI395827[] = {{&S97818[5], 13}, {NULL}};
const Segment SGI395828[] = {{&S97935[0], 3}, {NULL}};
const Segment SGI395829[] = {{&S97935[32], 8}, {NULL}};
const Segment SGI395830[] = {{&S97935[22], 7}, {NULL}};
const Segment SGI395831[] = {{&S97935[6], 7}, {NULL}};
const Segment SGI395832[] = {{&S98159[0], 3}, {NULL}};
const Segment SGI395833[] = {{&S98159[49], 8}, {NULL}};
const Segment SGI395834[] = {{&S98159[28], 9}, {NULL}};
const Segment SGI395835[] = {{&S98159[5], 13}, {NULL}};
const Segment SGI395836[] = {{&S98188[0], 3}, {NULL}};
const Segment SGI395837[] = {{&S98188[44], 8}, {NULL}};
const Segment SGI395838[] = {{&S98188[28], 5}, {NULL}};
const Segment SGI395839[] = {{&S98188[5], 13}, {NULL}};
const Segment SGI395840[] = {{&S98238[0], 3}, {NULL}};
const Segment SGI395841[] = {{&S98238[45], 8}, {NULL}};
const Segment SGI395842[] = {{&S98238[28], 6}, {NULL}};
const Segment SGI395843[] = {{&S98238[5], 13}, {NULL}};
const Segment SGI395844[] = {{&S98239[0], 3}, {NULL}};
const Segment SGI395845[] = {{&S98239[46], 8}, {NULL}};
const Segment SGI395846[] = {{&S98239[26], 9}, {NULL}};
const Segment SGI395847[] = {{&S98239[5], 13}, {NULL}};
const Segment SGI395848[] = {{&S98240[0], 3}, {NULL}};
const Segment SGI395849[] = {{&S98240[43], 8}, {NULL}};
const Segment SGI395850[] = {{&S98240[28], 4}, {NULL}};
const Segment SGI395851[] = {{&S98240[5], 13}, {NULL}};
const Segment SGI395852[] = {{&S98340[0], 3}, {NULL}};
const Segment SGI395853[] = {{&S98340[44], 8}, {NULL}};
const Segment SGI395854[] = {{&S98340[28], 5}, {NULL}};
const Segment SGI395855[] = {{&S98340[5], 13}, {NULL}};
const Segment SGI395856[] = {{&S98353[0], 3}, {NULL}};
const Segment SGI395857[] = {{&S98353[49], 8}, {NULL}};
const Segment SGI395858[] = {{&S98353[28], 9}, {NULL}};
const Segment SGI395859[] = {{&S98353[5], 13}, {NULL}};
const Segment SGI395860[] = {{&S98562[0], 3}, {NULL}};
const Segment SGI395861[] = {{&S98562[32], 8}, {NULL}};
const Segment SGI395862[] = {{&S98562[22], 7}, {NULL}};
const Segment SGI395863[] = {{&S98562[6], 7}, {NULL}};
const Segment SGI395864[] = {{&S98563[0], 3}, {NULL}};
const Segment SGI395865[] = {{&S98563[32], 8}, {NULL}};
const Segment SGI395866[] = {{&S98563[22], 7}, {NULL}};
const Segment SGI395867[] = {{&S98563[6], 7}, {NULL}};
const Segment SGI395868[] = {{&S98599[0], 3}, {NULL}};
const Segment SGI395869[] = {{&S98599[49], 8}, {NULL}};
const Segment SGI395870[] = {{&S98599[28], 10}, {NULL}};
const Segment SGI395871[] = {{&S98599[5], 13}, {NULL}};
const Segment SGI395872[] = {{&S98651[0], 3}, {NULL}};
const Segment SGI395873[] = {{&S98651[36], 8}, {NULL}};
const Segment SGI395874[] = {{&S98666[0], 3}, {NULL}};
const Segment SGI395875[] = {{&S98666[45], 8}, {NULL}};
const Segment SGI395876[] = {{&S98666[26], 8}, {NULL}};
const Segment SGI395877[] = {{&S98666[5], 13}, {NULL}};
const Segment SGI395878[] = {{&S98773[0], 3}, {NULL}};
const Segment SGI395879[] = {{&S98773[45], 8}, {NULL}};
const Segment SGI395880[] = {{&S98773[28], 6}, {NULL}};
const Segment SGI395881[] = {{&S98773[5], 13}, {NULL}};
const Segment SGI395882[] = {{&S99184[0], 3}, {NULL}};
const Segment SGI395883[] = {{&S99184[51], 8}, {NULL}};
const Segment SGI395884[] = {{&S99184[28], 12}, {NULL}};
const Segment SGI395885[] = {{&S99184[5], 13}, {NULL}};
const Segment SGI395886[] = {{&S99190[0], 3}, {NULL}};
const Segment SGI395887[] = {{&S99190[47], 8}, {NULL}};
const Segment SGI395888[] = {{&S99190[28], 8}, {NULL}};
const Segment SGI395889[] = {{&S99190[5], 13}, {NULL}};
const Segment SGI395890[] = {{&S99217[0], 3}, {NULL}};
const Segment SGI395891[] = {{&S99217[32], 8}, {NULL}};
const Segment SGI395892[] = {{&S99217[22], 7}, {NULL}};
const Segment SGI395893[] = {{&S99217[6], 7}, {NULL}};
const Segment SGI395894[] = {{&S99223[0], 3}, {NULL}};
const Segment SGI395895[] = {{&S99223[43], 8}, {NULL}};
const Segment SGI395896[] = {{&S99223[28], 4}, {NULL}};
const Segment SGI395897[] = {{&S99223[5], 13}, {NULL}};
const Segment SGI395898[] = {{&S99282[0], 3}, {NULL}};
const Segment SGI395899[] = {{&S99282[28], 13}, {NULL}};
const Segment SGI395900[] = {{&S99282[5], 13}, {NULL}};
const Segment SGI395901[] = {{&S99439[0], 3}, {NULL}};
const Segment SGI395902[] = {{&S99439[50], 8}, {NULL}};
const Segment SGI395903[] = {{&S99439[28], 9}, {NULL}};
const Segment SGI395904[] = {{&S99439[5], 13}, {NULL}};
const Segment SGI395905[] = {{&S99587[0], 3}, {NULL}};
const Segment SGI395906[] = {{&S99587[30], 8}, {NULL}};
const Segment SGI395907[] = {{&S99587[20], 7}, {NULL}};
const Segment SGI395908[] = {{&S99587[6], 7}, {NULL}};
const Segment SGI395909[] = {{&S99588[0], 3}, {NULL}};
const Segment SGI395910[] = {{&S99588[32], 8}, {NULL}};
const Segment SGI395911[] = {{&S99588[22], 7}, {NULL}};
const Segment SGI395912[] = {{&S99588[6], 7}, {NULL}};
const Segment SGI395913[] = {{&S99589[0], 3}, {NULL}};
const Segment SGI395914[] = {{&S99589[32], 8}, {NULL}};
const Segment SGI395915[] = {{&S99589[22], 7}, {NULL}};
const Segment SGI395916[] = {{&S99589[6], 7}, {NULL}};
const Segment SGI395917[] = {{&S99594[0], 3}, {NULL}};
const Segment SGI395918[] = {{&S99594[43], 8}, {NULL}};
const Segment SGI395919[] = {{&S99594[24], 9}, {NULL}};
const Segment SGI395920[] = {{&S99639[0], 3}, {NULL}};
const Segment SGI395921[] = {{&S99639[47], 8}, {NULL}};
const Segment SGI395922[] = {{&S99639[25], 12}, {NULL}};
const Segment SGI395923[] = {{&S99773[0], 3}, {NULL}};
const Segment SGI395924[] = {{&S99773[43], 8}, {NULL}};
const Segment SGI395925[] = {{&S99773[28], 4}, {NULL}};
const Segment SGI395926[] = {{&S99773[5], 13}, {NULL}};
const Segment SGI395927[] = {{&S99774[0], 3}, {NULL}};
const Segment SGI395928[] = {{&S99774[45], 8}, {NULL}};
const Segment SGI395929[] = {{&S99774[28], 6}, {NULL}};
const Segment SGI395930[] = {{&S99774[5], 13}, {NULL}};
const Segment SGI395931[] = {{&S99839[0], 3}, {NULL}};
const Segment SGI395932[] = {{&S99839[46], 8}, {NULL}};
const Segment SGI395933[] = {{&S99839[25], 11}, {NULL}};
const Segment SGI395934[] = {{&S99859[0], 3}, {NULL}};
const Segment SGI395935[] = {{&S99859[45], 8}, {NULL}};
const Segment SGI395936[] = {{&S99859[28], 6}, {NULL}};
const Segment SGI395937[] = {{&S99859[5], 13}, {NULL}};
const Segment SGI395938[] = {{&S100257[0], 3}, {NULL}};
const Segment SGI395939[] = {{&S100257[52], 8}, {NULL}};
const Segment SGI395940[] = {{&S100257[28], 13}, {NULL}};
const Segment SGI395941[] = {{&S100257[5], 13}, {NULL}};
const Segment SGI395942[] = {{&S100294[0], 3}, {NULL}};
const Segment SGI395943[] = {{&S100294[48], 8}, {NULL}};
const Segment SGI395944[] = {{&S100294[28], 8}, {NULL}};
const Segment SGI395945[] = {{&S100294[5], 13}, {NULL}};
const Segment SGI395946[] = {{&S100296[0], 3}, {NULL}};
const Segment SGI395947[] = {{&S100296[46], 8}, {NULL}};
const Segment SGI395948[] = {{&S100296[28], 7}, {NULL}};
const Segment SGI395949[] = {{&S100296[5], 13}, {NULL}};
const Segment SGI395950[] = {{&S100336[0], 3}, {NULL}};
const Segment SGI395951[] = {{&S100336[49], 8}, {NULL}};
const Segment SGI395952[] = {{&S100336[28], 10}, {NULL}};
const Segment SGI395953[] = {{&S100336[5], 13}, {NULL}};
const Segment SGI395954[] = {{&S100459[0], 3}, {NULL}};
const Segment SGI395955[] = {{&S100459[47], 8}, {NULL}};
const Segment SGI395956[] = {{&S100459[28], 8}, {NULL}};
const Segment SGI395957[] = {{&S100459[5], 13}, {NULL}};
const Segment SGI395958[] = {{&S100603[0], 3}, {NULL}};
const Segment SGI395959[] = {{&S100603[47], 8}, {NULL}};
const Segment SGI395960[] = {{&S100603[28], 8}, {NULL}};
const Segment SGI395961[] = {{&S100603[5], 13}, {NULL}};
const Segment SGI395962[] = {{&S100617[0], 3}, {NULL}};
const Segment SGI395963[] = {{&S100617[48], 8}, {NULL}};
const Segment SGI395964[] = {{&S100617[28], 9}, {NULL}};
const Segment SGI395965[] = {{&S100617[5], 13}, {NULL}};
const Segment SGI395966[] = {{&S100619[0], 3}, {NULL}};
const Segment SGI395967[] = {{&S100619[53], 8}, {NULL}};
const Segment SGI395968[] = {{&S100619[28], 14}, {NULL}};
const Segment SGI395969[] = {{&S100619[5], 13}, {NULL}};
const Segment SGI395970[] = {{&S100818[0], 3}, {NULL}};
const Segment SGI395971[] = {{&S100818[48], 8}, {NULL}};
const Segment SGI395972[] = {{&S100818[28], 9}, {NULL}};
const Segment SGI395973[] = {{&S100818[5], 13}, {NULL}};
const Segment SGI395974[] = {{&S100900[0], 3}, {NULL}};
const Segment SGI395975[] = {{&S100900[45], 8}, {NULL}};
const Segment SGI395976[] = {{&S100900[28], 6}, {NULL}};
const Segment SGI395977[] = {{&S100900[5], 13}, {NULL}};
const Segment SGI395978[] = {{&S100902[0], 3}, {NULL}};
const Segment SGI395979[] = {{&S100902[47], 8}, {NULL}};
const Segment SGI395980[] = {{&S100902[26], 9}, {NULL}};
const Segment SGI395981[] = {{&S100902[5], 13}, {NULL}};
const Segment SGI395982[] = {{&S100905[0], 3}, {NULL}};
const Segment SGI395983[] = {{&S100905[46], 8}, {NULL}};
const Segment SGI395984[] = {{&S100905[28], 7}, {NULL}};
const Segment SGI395985[] = {{&S100905[5], 13}, {NULL}};
const Segment SGI395986[] = {{&S101100[0], 3}, {NULL}};
const Segment SGI395987[] = {{&S101100[32], 8}, {NULL}};
const Segment SGI395988[] = {{&S101100[22], 7}, {NULL}};
const Segment SGI395989[] = {{&S101100[6], 7}, {NULL}};
const Segment SGI395990[] = {{&S101107[0], 3}, {NULL}};
const Segment SGI395991[] = {{&S101107[49], 8}, {NULL}};
const Segment SGI395992[] = {{&S101107[28], 9}, {NULL}};
const Segment SGI395993[] = {{&S101107[5], 13}, {NULL}};
const Segment SGI395994[] = {{&S101267[0], 3}, {NULL}};
const Segment SGI395995[] = {{&S101267[51], 8}, {NULL}};
const Segment SGI395996[] = {{&S101267[28], 12}, {NULL}};
const Segment SGI395997[] = {{&S101267[5], 13}, {NULL}};
const Segment SGI395998[] = {{&S101269[0], 3}, {NULL}};
const Segment SGI395999[] = {{&S101269[39], 8}, {NULL}};
const Segment SGI396000[] = {{&S101269[24], 5}, {NULL}};
const Segment SGI396001[] = {{&S101355[0], 3}, {NULL}};
const Segment SGI396002[] = {{&S101355[42], 8}, {NULL}};
const Segment SGI396003[] = {{&S101355[24], 8}, {NULL}};
const Segment SGI396004[] = {{&S101382[0], 3}, {NULL}};
const Segment SGI396005[] = {{&S101382[49], 8}, {NULL}};
const Segment SGI396006[] = {{&S101382[28], 10}, {NULL}};
const Segment SGI396007[] = {{&S101382[5], 13}, {NULL}};
const Segment SGI396008[] = {{&S101408[0], 3}, {NULL}};
const Segment SGI396009[] = {{&S101408[49], 8}, {NULL}};
const Segment SGI396010[] = {{&S101408[28], 10}, {NULL}};
const Segment SGI396011[] = {{&S101408[5], 13}, {NULL}};
const Segment SGI396012[] = {{&S101498[0], 3}, {NULL}};
const Segment SGI396013[] = {{&S101498[37], 8}, {NULL}};
const Segment SGI396014[] = {{&S101498[22], 5}, {NULL}};
const Segment SGI396015[] = {{&S101520[0], 3}, {NULL}};
const Segment SGI396016[] = {{&S101520[41], 8}, {NULL}};
const Segment SGI396017[] = {{&S101520[22], 9}, {NULL}};
const Segment SGI396018[] = {{&S101536[0], 3}, {NULL}};
const Segment SGI396019[] = {{&S101536[42], 8}, {NULL}};
const Segment SGI396020[] = {{&S101536[24], 8}, {NULL}};
const Segment SGI396021[] = {{&S101586[0], 3}, {NULL}};
const Segment SGI396022[] = {{&S101586[45], 8}, {NULL}};
const Segment SGI396023[] = {{&S101586[28], 6}, {NULL}};
const Segment SGI396024[] = {{&S101586[5], 13}, {NULL}};
const Segment SGI396025[] = {{&S101588[0], 3}, {NULL}};
const Segment SGI396026[] = {{&S101588[45], 8}, {NULL}};
const Segment SGI396027[] = {{&S101588[28], 6}, {NULL}};
const Segment SGI396028[] = {{&S101588[5], 13}, {NULL}};
const Segment SGI396029[] = {{&S101732[0], 3}, {NULL}};
const Segment SGI396030[] = {{&S101732[46], 8}, {NULL}};
const Segment SGI396031[] = {{&S101732[26], 9}, {NULL}};
const Segment SGI396032[] = {{&S101732[5], 13}, {NULL}};
const Segment SGI396033[] = {{&S101846[0], 3}, {NULL}};
const Segment SGI396034[] = {{&S101846[39], 8}, {NULL}};
const Segment SGI396035[] = {{&S101846[24], 5}, {NULL}};
const Segment SGI396036[] = {{&S101847[0], 3}, {NULL}};
const Segment SGI396037[] = {{&S101847[39], 8}, {NULL}};
const Segment SGI396038[] = {{&S101847[24], 5}, {NULL}};
const Segment SGI396039[] = {{&S102084[0], 3}, {NULL}};
const Segment SGI396040[] = {{&S102084[25], 10}, {NULL}};
const Segment SGI396041[] = {{&S102489[0], 3}, {NULL}};
const Segment SGI396042[] = {{&S102489[44], 8}, {NULL}};
const Segment SGI396043[] = {{&S102489[28], 5}, {NULL}};
const Segment SGI396044[] = {{&S102489[5], 13}, {NULL}};
const Segment SGI396045[] = {{&S102656[0], 3}, {NULL}};
const Segment SGI396046[] = {{&S102656[42], 8}, {NULL}};
const Segment SGI396047[] = {{&S102656[24], 8}, {NULL}};
const Segment SGI396048[] = {{&S65265[12], 13}, {NULL}};
const Segment SGI396049[] = {{&S65265[34], 7}, {NULL}};
const Segment SGI396050[] = {{&S103079[0], 3}, {NULL}};
const Segment SGI396051[] = {{&S103079[50], 8}, {NULL}};
const Segment SGI396052[] = {{&S103079[28], 11}, {NULL}};
const Segment SGI396053[] = {{&S103079[5], 13}, {NULL}};
const Segment SGI396054[] = {{&S103088[0], 3}, {NULL}};
const Segment SGI396055[] = {{&S103088[39], 8}, {NULL}};
const Segment SGI396056[] = {{&S103088[24], 5}, {NULL}};
const Segment SGI396057[] = {{&S103193[0], 3}, {NULL}};
const Segment SGI396058[] = {{&S103193[49], 8}, {NULL}};
const Segment SGI396059[] = {{&S103193[28], 10}, {NULL}};
const Segment SGI396060[] = {{&S103193[5], 13}, {NULL}};
const Segment SGI396061[] = {{&S103345[0], 3}, {NULL}};
const Segment SGI396062[] = {{&S103345[48], 8}, {NULL}};
const Segment SGI396063[] = {{&S103345[28], 9}, {NULL}};
const Segment SGI396064[] = {{&S103345[5], 13}, {NULL}};
const Segment SGI396065[] = {{&S103409[0], 3}, {NULL}};
const Segment SGI396066[] = {{&S103409[56], 8}, {NULL}};
const Segment SGI396067[] = {{&S103409[34], 11}, {NULL}};
const Segment SGI396068[] = {{&S103409[5], 13}, {NULL}};
const Segment SGI396069[] = {{&S103477[0], 3}, {NULL}};
const Segment SGI396070[] = {{&S103477[51], 8}, {NULL}};
const Segment SGI396071[] = {{&S103477[28], 12}, {NULL}};
const Segment SGI396072[] = {{&S103477[5], 13}, {NULL}};
const Segment SGI396073[] = {{&S103695[0], 3}, {NULL}};
const Segment SGI396074[] = {{&S103695[46], 8}, {NULL}};
const Segment SGI396075[] = {{&S103695[26], 9}, {NULL}};
const Segment SGI396076[] = {{&S103695[5], 13}, {NULL}};
const Segment SGI396077[] = {{&S104780[0], 3}, {NULL}};
const Segment SGI396078[] = {{&S104780[48], 8}, {NULL}};
const Segment SGI396079[] = {{&S104780[28], 9}, {NULL}};
const Segment SGI396080[] = {{&S104780[5], 13}, {NULL}};
const Segment SGI396081[] = {{&S105471[0], 3}, {NULL}};
const Segment SGI396082[] = {{&S105471[48], 8}, {NULL}};
const Segment SGI396083[] = {{&S105471[28], 9}, {NULL}};
const Segment SGI396084[] = {{&S105471[5], 13}, {NULL}};
const Segment SGI396085[] = {{&S105472[0], 3}, {NULL}};
const Segment SGI396086[] = {{&S105472[48], 8}, {NULL}};
const Segment SGI396087[] = {{&S105472[28], 9}, {NULL}};
const Segment SGI396088[] = {{&S105472[5], 13}, {NULL}};
const Segment SGI396089[] = {{&S15691[35], 3}, {NULL}};
const Segment SGI396090[] = {{&S105550[0], 3}, {NULL}};
const Segment SGI396091[] = {{&S105550[48], 8}, {NULL}};
const Segment SGI396092[] = {{&S105550[28], 8}, {NULL}};
const Segment SGI396093[] = {{&S105550[5], 13}, {NULL}};
const Segment SGI396094[] = {{&S70422[12], 13}, {NULL}};
const Segment SGI396095[] = {{&S70422[33], 5}, {NULL}};
const Segment SGI396096[] = {{&S70798[12], 13}, {NULL}};
const Segment SGI396097[] = {{&S70798[32], 5}, {NULL}};
const Segment SGI396098[] = {{&S19154[34], 3}, {NULL}};
const Segment SGI396099[] = {{&S19154[13], 15}, {NULL}};
const Segment SGI396100[] = {{&S19167[47], 3}, {NULL}};
const Segment SGI396101[] = {{&S19169[34], 3}, {NULL}};
const Segment SGI396102[] = {{&S19169[13], 15}, {NULL}};
const Segment SGI396103[] = {{&S19887[35], 3}, {NULL}};
const Segment SGI396104[] = {{&S19930[35], 3}, {NULL}};
const Segment SGI396105[] = {{&S21590[35], 3}, {NULL}};
const Segment SGI396106[] = {{&S22308[33], 3}, {NULL}};
const Segment SGI396107[] = {{&S22308[13], 14}, {NULL}};
const Segment SGI396108[] = {{&S22309[18], 3}, {NULL}};
const Segment SGI396109[] = {{&S22309[28], 27}, {NULL}};
const Segment SGI396110[] = {{&S22316[46], 3}, {NULL}};
const Segment SGI396111[] = {{&S22321[33], 3}, {NULL}};
const Segment SGI396112[] = {{&S22321[13], 14}, {NULL}};
const Segment SGI396113[] = {{&S76520[12], 13}, {NULL}};
const Segment SGI396114[] = {{&S76520[34], 5}, {NULL}};
const Segment SGI396115[] = {{&S81488[12], 13}, {NULL}};
const Segment SGI396116[] = {{&S81488[34], 5}, {NULL}};
const Segment SGI396117[] = {{&S39527[25], 13}, {NULL}};
const Segment SGI396118[] = {{&S32246[35], 3}, {NULL}};
const Segment SGI396119[] = {{&S42831[25], 13}, {NULL}};
const Segment SGI396120[] = {{&S35628[25], 12}, {NULL}};
const Segment SGI396121[] = {{&S36790[5], 3}, {NULL}};
const Segment SGI396122[] = {{&S47523[25], 13}, {NULL}};
const Segment SGI396123[] = {{&S48110[43], 13}, {NULL}};
const Segment SGI396124[] = {{&S49005[43], 13}, {NULL}};
const Segment SGI396125[] = {{&S40528[37], 3}, {NULL}};
const Segment SGI396126[] = {{&S40528[13], 18}, {NULL}};
const Segment SGI396127[] = {{&S40530[37], 3}, {NULL}};
const Segment SGI396128[] = {{&S40530[13], 18}, {NULL}};
const Segment SGI396129[] = {{&S50357[44], 13}, {NULL}};
const Segment SGI396130[] = {{&S41412[36], 3}, {NULL}};
const Segment SGI396131[] = {{&S47711[25], 12}, {NULL}};
const Segment SGI396132[] = {{&S47712[25], 12}, {NULL}};
const Segment SGI396133[] = {{&S47713[25], 12}, {NULL}};
const Segment SGI396134[] = {{&S47714[25], 12}, {NULL}};
const Segment SGI396135[] = {{&S47715[25], 12}, {NULL}};
const Segment SGI396136[] = {{&S47716[25], 12}, {NULL}};
const Segment SGI396137[] = {{&S47717[25], 12}, {NULL}};
const Segment SGI396138[] = {{&S47718[25], 12}, {NULL}};
const Segment SGI396139[] = {{&S47719[25], 12}, {NULL}};
const Segment SGI396140[] = {{&S56914[25], 13}, {NULL}};
const Segment SGI396141[] = {{&S57823[25], 13}, {NULL}};
const Segment SGI396142[] = {{&S57987[25], 13}, {NULL}};
const Segment SGI396143[] = {{&S60277[25], 13}, {NULL}};
const Segment SGI396144[] = {{&S60985[25], 13}, {NULL}};
const Segment SGI396145[] = {{&S60985[72], 6}, {NULL}};
const Segment SGI396146[] = {{&S62200[25], 13}, {NULL}};
const Segment SGI396147[] = {{&S62347[25], 13}, {NULL}};
const Segment SGI396148[] = {{&S62494[25], 13}, {NULL}};
const Segment SGI396149[] = {{&S63644[25], 13}, {NULL}};
const Segment SGI396150[] = {{&S63902[25], 13}, {NULL}};
const Segment SGI396151[] = {{&S64062[25], 13}, {NULL}};
const Segment SGI396152[] = {{&S65242[25], 13}, {NULL}};
const Segment SGI396153[] = {{&S57008[33], 3}, {NULL}};
const Segment SGI396154[] = {{&S57008[13], 14}, {NULL}};
const Segment SGI396155[] = {{&S69886[0], 57}, {NULL}};
const Segment SGI396156[] = {{&S69887[0], 58}, {NULL}};
const Segment SGI396157[] = {{&S70150[0], 61}, {NULL}};
const Segment SGI396158[] = {{&S70151[0], 31}, {NULL}};
const Segment SGI396159[] = {{&S66967[25], 13}, {NULL}};
const Segment SGI396160[] = {{&S67432[25], 13}, {NULL}};
const Segment SGI396161[] = {{&S70803[0], 57}, {NULL}};
const Segment SGI396162[] = {{&S59199[25], 12}, {NULL}};
const Segment SGI396163[] = {{&S71786[0], 60}, {NULL}};
const Segment SGI396164[] = {{&S72221[0], 57}, {NULL}};
const Segment SGI396165[] = {{&S72222[0], 61}, {NULL}};
const Segment SGI396166[] = {{&S70015[25], 13}, {NULL}};
const Segment SGI396167[] = {{&S71068[25], 13}, {NULL}};
const Segment SGI396168[] = {{&S48082[66], 4}, {NULL}};
const Segment SGI396169[] = {{&S48082[39], 7}, {NULL}};
const Segment SGI396170[] = {{&S48083[66], 4}, {NULL}};
const Segment SGI396171[] = {{&S48083[39], 7}, {NULL}};
const Segment SGI396172[] = {{&S48084[66], 4}, {NULL}};
const Segment SGI396173[] = {{&S48084[39], 7}, {NULL}};
const Segment SGI396174[] = {{&S48085[66], 4}, {NULL}};
const Segment SGI396175[] = {{&S48085[39], 7}, {NULL}};
const Segment SGI396176[] = {{&S48086[66], 4}, {NULL}};
const Segment SGI396177[] = {{&S48086[39], 7}, {NULL}};
const Segment SGI396178[] = {{&S48087[66], 4}, {NULL}};
const Segment SGI396179[] = {{&S48087[39], 7}, {NULL}};
const Segment SGI396180[] = {{&S48088[66], 4}, {NULL}};
const Segment SGI396181[] = {{&S48088[39], 7}, {NULL}};
const Segment SGI396182[] = {{&S48089[63], 4}, {NULL}};
const Segment SGI396183[] = {{&S72268[25], 13}, {NULL}};
const Segment SGI396184[] = {{&S76227[0], 61}, {NULL}};
const Segment SGI396185[] = {{&S76228[0], 62}, {NULL}};
const Segment SGI396186[] = {{&S49010[61], 4}, {NULL}};
const Segment SGI396187[] = {{&S49010[35], 6}, {NULL}};
const Segment SGI396188[] = {{&S49011[61], 4}, {NULL}};
const Segment SGI396189[] = {{&S49011[35], 6}, {NULL}};
const Segment SGI396190[] = {{&S49012[60], 4}, {NULL}};
const Segment SGI396191[] = {{&S49012[35], 5}, {NULL}};
const Segment SGI396192[] = {{&S49013[60], 4}, {NULL}};
const Segment SGI396193[] = {{&S49013[35], 5}, {NULL}};
const Segment SGI396194[] = {{&S49014[61], 4}, {NULL}};
const Segment SGI396195[] = {{&S49014[35], 6}, {NULL}};
const Segment SGI396196[] = {{&S49015[63], 4}, {NULL}};
const Segment SGI396197[] = {{&S49015[35], 8}, {NULL}};
const Segment SGI396198[] = {{&S49016[61], 4}, {NULL}};
const Segment SGI396199[] = {{&S49016[35], 6}, {NULL}};
const Segment SGI396200[] = {{&S49017[61], 4}, {NULL}};
const Segment SGI396201[] = {{&S49017[35], 6}, {NULL}};
const Segment SGI396202[] = {{&S49019[63], 4}, {NULL}};
const Segment SGI396203[] = {{&S49019[35], 8}, {NULL}};
const Segment SGI396204[] = {{&S49020[70], 4}, {NULL}};
const Segment SGI396205[] = {{&S49020[35], 15}, {NULL}};
const Segment SGI396206[] = {{&S49021[64], 4}, {NULL}};
const Segment SGI396207[] = {{&S49021[35], 9}, {NULL}};
const Segment SGI396208[] = {{&S49022[64], 4}, {NULL}};
const Segment SGI396209[] = {{&S49022[35], 9}, {NULL}};
const Segment SGI396210[] = {{&S49023[63], 4}, {NULL}};
const Segment SGI396211[] = {{&S49023[35], 8}, {NULL}};
const Segment SGI396212[] = {{&S49024[63], 4}, {NULL}};
const Segment SGI396213[] = {{&S49024[35], 8}, {NULL}};
const Segment SGI396214[] = {{&S49025[57], 4}, {NULL}};
const Segment SGI396215[] = {{&S49026[60], 4}, {NULL}};
const Segment SGI396216[] = {{&S49026[35], 5}, {NULL}};
const Segment SGI396217[] = {{&S49027[68], 4}, {NULL}};
const Segment SGI396218[] = {{&S49027[35], 13}, {NULL}};
const Segment SGI396219[] = {{&S49028[57], 4}, {NULL}};
const Segment SGI396220[] = {{&S49030[73], 4}, {NULL}};
const Segment SGI396221[] = {{&S49030[35], 18}, {NULL}};
const Segment SGI396222[] = {{&S49031[68], 4}, {NULL}};
const Segment SGI396223[] = {{&S49031[35], 13}, {NULL}};
const Segment SGI396224[] = {{&S49032[60], 4}, {NULL}};
const Segment SGI396225[] = {{&S49032[35], 5}, {NULL}};
const Segment SGI396226[] = {{&S76962[0], 61}, {NULL}};
const Segment SGI396227[] = {{&S73713[25], 13}, {NULL}};
const Segment SGI396228[] = {{&S77285[0], 57}, {NULL}};
const Segment SGI396229[] = {{&S77382[0], 61}, {NULL}};
const Segment SGI396230[] = {{&S77387[0], 57}, {NULL}};
const Segment SGI396231[] = {{&S50543[84], 4}, {NULL}};
const Segment SGI396232[] = {{&S50543[28], 6}, {NULL}};
const Segment SGI396233[] = {{&S50546[83], 4}, {NULL}};
const Segment SGI396234[] = {{&S50546[28], 5}, {NULL}};
const Segment SGI396235[] = {{&S50547[84], 4}, {NULL}};
const Segment SGI396236[] = {{&S50547[28], 6}, {NULL}};
const Segment SGI396237[] = {{&S50548[83], 4}, {NULL}};
const Segment SGI396238[] = {{&S50548[28], 5}, {NULL}};
const Segment SGI396239[] = {{&S50549[84], 4}, {NULL}};
const Segment SGI396240[] = {{&S50549[28], 6}, {NULL}};
const Segment SGI396241[] = {{&S50551[86], 4}, {NULL}};
const Segment SGI396242[] = {{&S50551[28], 8}, {NULL}};
const Segment SGI396243[] = {{&S50552[80], 4}, {NULL}};
const Segment SGI396244[] = {{&S50553[86], 4}, {NULL}};
const Segment SGI396245[] = {{&S50553[28], 8}, {NULL}};
const Segment SGI396246[] = {{&S50554[91], 4}, {NULL}};
const Segment SGI396247[] = {{&S50554[28], 13}, {NULL}};
const Segment SGI396248[] = {{&S50555[93], 4}, {NULL}};
const Segment SGI396249[] = {{&S50555[28], 15}, {NULL}};
const Segment SGI396250[] = {{&S50556[91], 4}, {NULL}};
const Segment SGI396251[] = {{&S50556[28], 13}, {NULL}};
const Segment SGI396252[] = {{&S74918[25], 13}, {NULL}};
const Segment SGI396253[] = {{&S75323[25], 13}, {NULL}};
const Segment SGI396254[] = {{&S51607[33], 18}, {NULL}};
const Segment SGI396255[] = {{&S51614[61], 4}, {NULL}};
const Segment SGI396256[] = {{&S51614[36], 5}, {NULL}};
const Segment SGI396257[] = {{&S51615[62], 4}, {NULL}};
const Segment SGI396258[] = {{&S51615[36], 6}, {NULL}};
const Segment SGI396259[] = {{&S51616[64], 4}, {NULL}};
const Segment SGI396260[] = {{&S51616[36], 8}, {NULL}};
const Segment SGI396261[] = {{&S51617[69], 4}, {NULL}};
const Segment SGI396262[] = {{&S51617[36], 13}, {NULL}};
const Segment SGI396263[] = {{&S51618[71], 4}, {NULL}};
const Segment SGI396264[] = {{&S51618[36], 15}, {NULL}};
const Segment SGI396265[] = {{&S51619[69], 4}, {NULL}};
const Segment SGI396266[] = {{&S51619[36], 13}, {NULL}};
const Segment SGI396267[] = {{&S51622[58], 4}, {NULL}};
const Segment SGI396268[] = {{&S51623[65], 4}, {NULL}};
const Segment SGI396269[] = {{&S51623[36], 9}, {NULL}};
const Segment SGI396270[] = {{&S51624[58], 4}, {NULL}};
const Segment SGI396271[] = {{&S51626[59], 4}, {NULL}};
const Segment SGI396272[] = {{&S51627[61], 4}, {NULL}};
const Segment SGI396273[] = {{&S51627[36], 5}, {NULL}};
const Segment SGI396274[] = {{&S51628[64], 4}, {NULL}};
const Segment SGI396275[] = {{&S51628[36], 8}, {NULL}};
const Segment SGI396276[] = {{&S51629[61], 4}, {NULL}};
const Segment SGI396277[] = {{&S51629[36], 5}, {NULL}};
const Segment SGI396278[] = {{&S51630[62], 4}, {NULL}};
const Segment SGI396279[] = {{&S51630[36], 6}, {NULL}};
const Segment SGI396280[] = {{&S51631[62], 4}, {NULL}};
const Segment SGI396281[] = {{&S51631[36], 6}, {NULL}};
const Segment SGI396282[] = {{&S51632[61], 4}, {NULL}};
const Segment SGI396283[] = {{&S51632[36], 5}, {NULL}};
const Segment SGI396284[] = {{&S51633[61], 4}, {NULL}};
const Segment SGI396285[] = {{&S51633[36], 5}, {NULL}};
const Segment SGI396286[] = {{&S51634[64], 4}, {NULL}};
const Segment SGI396287[] = {{&S51634[36], 8}, {NULL}};
const Segment SGI396288[] = {{&S51635[61], 4}, {NULL}};
const Segment SGI396289[] = {{&S51635[36], 5}, {NULL}};
const Segment SGI396290[] = {{&S51636[73], 4}, {NULL}};
const Segment SGI396291[] = {{&S51636[36], 17}, {NULL}};
const Segment SGI396292[] = {{&S51637[59], 4}, {NULL}};
const Segment SGI396293[] = {{&S78637[0], 63}, {NULL}};
const Segment SGI396294[] = {{&S78730[0], 31}, {NULL}};
const Segment SGI396295[] = {{&S78884[0], 57}, {NULL}};
const Segment SGI396296[] = {{&S76373[25], 13}, {NULL}};
const Segment SGI396297[] = {{&S80203[0], 11}, {NULL}};
const Segment SGI396298[] = {{&S80512[0], 12}, {NULL}};
const Segment SGI396299[] = {{&S80667[0], 39}, {NULL}};
const Segment SGI396300[] = {{&S80989[0], 61}, {NULL}};
const Segment SGI396301[] = {{&S54144[170], 4}, {NULL}};
const Segment SGI396302[] = {{&S81244[0], 43}, {NULL}};
const Segment SGI396303[] = {{&S54583[60], 4}, {NULL}};
const Segment SGI396304[] = {{&S54583[35], 5}, {NULL}};
const Segment SGI396305[] = {{&S54585[61], 4}, {NULL}};
const Segment SGI396306[] = {{&S54585[35], 6}, {NULL}};
const Segment SGI396307[] = {{&S54586[60], 4}, {NULL}};
const Segment SGI396308[] = {{&S54586[35], 5}, {NULL}};
const Segment SGI396309[] = {{&S54587[61], 4}, {NULL}};
const Segment SGI396310[] = {{&S54587[35], 6}, {NULL}};
const Segment SGI396311[] = {{&S54588[61], 4}, {NULL}};
const Segment SGI396312[] = {{&S54588[35], 6}, {NULL}};
const Segment SGI396313[] = {{&S54589[60], 4}, {NULL}};
const Segment SGI396314[] = {{&S54589[35], 5}, {NULL}};
const Segment SGI396315[] = {{&S54590[63], 4}, {NULL}};
const Segment SGI396316[] = {{&S54590[35], 8}, {NULL}};
const Segment SGI396317[] = {{&S54591[63], 4}, {NULL}};
const Segment SGI396318[] = {{&S54591[35], 8}, {NULL}};
const Segment SGI396319[] = {{&S54592[68], 4}, {NULL}};
const Segment SGI396320[] = {{&S54592[35], 13}, {NULL}};
const Segment SGI396321[] = {{&S54593[60], 4}, {NULL}};
const Segment SGI396322[] = {{&S54593[35], 5}, {NULL}};
const Segment SGI396323[] = {{&S54594[70], 4}, {NULL}};
const Segment SGI396324[] = {{&S54594[35], 15}, {NULL}};
const Segment SGI396325[] = {{&S54595[64], 4}, {NULL}};
const Segment SGI396326[] = {{&S54595[35], 9}, {NULL}};
const Segment SGI396327[] = {{&S54596[68], 4}, {NULL}};
const Segment SGI396328[] = {{&S54596[35], 13}, {NULL}};
const Segment SGI396329[] = {{&S54598[61], 4}, {NULL}};
const Segment SGI396330[] = {{&S54598[35], 6}, {NULL}};
const Segment SGI396331[] = {{&S81678[0], 60}, {NULL}};
const Segment SGI396332[] = {{&S82456[0], 12}, {NULL}};
const Segment SGI396333[] = {{&S55813[83], 4}, {NULL}};
const Segment SGI396334[] = {{&S55813[28], 5}, {NULL}};
const Segment SGI396335[] = {{&S55814[83], 4}, {NULL}};
const Segment SGI396336[] = {{&S55814[28], 5}, {NULL}};
const Segment SGI396337[] = {{&S55815[84], 4}, {NULL}};
const Segment SGI396338[] = {{&S55815[28], 6}, {NULL}};
const Segment SGI396339[] = {{&S55816[84], 4}, {NULL}};
const Segment SGI396340[] = {{&S55816[28], 6}, {NULL}};
const Segment SGI396341[] = {{&S55817[86], 4}, {NULL}};
const Segment SGI396342[] = {{&S55817[28], 8}, {NULL}};
const Segment SGI396343[] = {{&S55818[93], 4}, {NULL}};
const Segment SGI396344[] = {{&S55818[28], 15}, {NULL}};
const Segment SGI396345[] = {{&S55819[84], 4}, {NULL}};
const Segment SGI396346[] = {{&S55819[28], 6}, {NULL}};
const Segment SGI396347[] = {{&S55820[87], 4}, {NULL}};
const Segment SGI396348[] = {{&S55820[28], 9}, {NULL}};
const Segment SGI396349[] = {{&S55821[91], 4}, {NULL}};
const Segment SGI396350[] = {{&S55821[28], 13}, {NULL}};
const Segment SGI396351[] = {{&S55822[80], 4}, {NULL}};
const Segment SGI396352[] = {{&S55823[86], 4}, {NULL}};
const Segment SGI396353[] = {{&S55823[28], 8}, {NULL}};
const Segment SGI396354[] = {{&S55824[60], 4}, {NULL}};
const Segment SGI396355[] = {{&S55824[35], 5}, {NULL}};
const Segment SGI396356[] = {{&S55825[86], 4}, {NULL}};
const Segment SGI396357[] = {{&S55825[28], 8}, {NULL}};
const Segment SGI396358[] = {{&S82928[0], 69}, {NULL}};
const Segment SGI396359[] = {{&S56146[45], 3}, {NULL}};
const Segment SGI396360[] = {{&S56146[28], 6}, {NULL}};
const Segment SGI396361[] = {{&S56147[45], 3}, {NULL}};
const Segment SGI396362[] = {{&S56147[28], 6}, {NULL}};
const Segment SGI396363[] = {{&S56148[44], 3}, {NULL}};
const Segment SGI396364[] = {{&S56148[28], 5}, {NULL}};
const Segment SGI396365[] = {{&S79232[25], 13}, {NULL}};
const Segment SGI396366[] = {{&S79232[72], 6}, {NULL}};
const Segment SGI396367[] = {{&S56149[44], 3}, {NULL}};
const Segment SGI396368[] = {{&S56149[28], 5}, {NULL}};
const Segment SGI396369[] = {{&S56150[45], 3}, {NULL}};
const Segment SGI396370[] = {{&S56150[28], 6}, {NULL}};
const Segment SGI396371[] = {{&S56151[54], 3}, {NULL}};
const Segment SGI396372[] = {{&S56151[28], 15}, {NULL}};
const Segment SGI396373[] = {{&S56152[47], 3}, {NULL}};
const Segment SGI396374[] = {{&S56152[28], 8}, {NULL}};
const Segment SGI396375[] = {{&S56154[47], 3}, {NULL}};
const Segment SGI396376[] = {{&S56154[28], 8}, {NULL}};
const Segment SGI396377[] = {{&S56155[42], 3}, {NULL}};
const Segment SGI396378[] = {{&S83208[0], 61}, {NULL}};
const Segment SGI396379[] = {{&S79620[25], 13}, {NULL}};
const Segment SGI396380[] = {{&S83411[0], 61}, {NULL}};
const Segment SGI396381[] = {{&S83428[0], 70}, {NULL}};
const Segment SGI396382[] = {{&S83493[0], 41}, {NULL}};
const Segment SGI396383[] = {{&S83672[0], 62}, {NULL}};
const Segment SGI396384[] = {{&S84077[0], 13}, {NULL}};
const Segment SGI396385[] = {{&S80871[25], 13}, {NULL}};
const Segment SGI396386[] = {{&S80871[72], 6}, {NULL}};
const Segment SGI396387[] = {{&S80938[25], 13}, {NULL}};
const Segment SGI396388[] = {{&S85145[0], 62}, {NULL}};
const Segment SGI396389[] = {{&S85650[0], 43}, {NULL}};
const Segment SGI396390[] = {{&S85657[0], 60}, {NULL}};
const Segment SGI396391[] = {{&S85669[0], 31}, {NULL}};
const Segment SGI396392[] = {{&S85782[0], 12}, {NULL}};
const Segment SGI396393[] = {{&S86660[0], 61}, {NULL}};
const Segment SGI396394[] = {{&S86927[0], 61}, {NULL}};
const Segment SGI396395[] = {{&S87040[0], 59}, {NULL}};
const Segment SGI396396[] = {{&S87048[0], 61}, {NULL}};
const Segment SGI396397[] = {{&S60640[67], 4}, {NULL}};
const Segment SGI396398[] = {{&S60640[38], 9}, {NULL}};
const Segment SGI396399[] = {{&S60642[68], 4}, {NULL}};
const Segment SGI396400[] = {{&S60642[38], 10}, {NULL}};
const Segment SGI396401[] = {{&S87548[0], 38}, {NULL}};
const Segment SGI396402[] = {{&S87567[0], 31}, {NULL}};
const Segment SGI396403[] = {{&S61314[44], 3}, {NULL}};
const Segment SGI396404[] = {{&S61314[28], 5}, {NULL}};
const Segment SGI396405[] = {{&S61315[45], 3}, {NULL}};
const Segment SGI396406[] = {{&S61315[28], 6}, {NULL}};
const Segment SGI396407[] = {{&S61317[54], 3}, {NULL}};
const Segment SGI396408[] = {{&S61317[28], 15}, {NULL}};
const Segment SGI396409[] = {{&S61318[45], 3}, {NULL}};
const Segment SGI396410[] = {{&S61318[28], 6}, {NULL}};
const Segment SGI396411[] = {{&S61320[45], 3}, {NULL}};
const Segment SGI396412[] = {{&S61320[28], 6}, {NULL}};
const Segment SGI396413[] = {{&S61321[44], 3}, {NULL}};
const Segment SGI396414[] = {{&S61321[28], 5}, {NULL}};
const Segment SGI396415[] = {{&S61323[47], 3}, {NULL}};
const Segment SGI396416[] = {{&S61323[28], 8}, {NULL}};
const Segment SGI396417[] = {{&S61324[45], 3}, {NULL}};
const Segment SGI396418[] = {{&S84400[43], 13}, {NULL}};
const Segment SGI396419[] = {{&S89005[0], 61}, {NULL}};
const Segment SGI396420[] = {{&S62632[67], 4}, {NULL}};
const Segment SGI396421[] = {{&S62632[38], 9}, {NULL}};
const Segment SGI396422[] = {{&S62633[64], 4}, {NULL}};
const Segment SGI396423[] = {{&S89056[0], 60}, {NULL}};
const Segment SGI396424[] = {{&S89057[0], 60}, {NULL}};
const Segment SGI396425[] = {{&S89058[0], 61}, {NULL}};
const Segment SGI396426[] = {{&S89110[0], 11}, {NULL}};
const Segment SGI396427[] = {{&S62691[170], 4}, {NULL}};
const Segment SGI396428[] = {{&S63017[88], 4}, {NULL}};
const Segment SGI396429[] = {{&S63017[28], 10}, {NULL}};
const Segment SGI396430[] = {{&S63018[85], 4}, {NULL}};
const Segment SGI396431[] = {{&S85780[25], 13}, {NULL}};
const Segment SGI396432[] = {{&S63357[67], 4}, {NULL}};
const Segment SGI396433[] = {{&S63357[38], 9}, {NULL}};
const Segment SGI396434[] = {{&S63358[64], 4}, {NULL}};
const Segment SGI396435[] = {{&S89735[0], 28}, {NULL}};
const Segment SGI396436[] = {{&S90034[0], 11}, {NULL}};
const Segment SGI396437[] = {{&S86325[25], 13}, {NULL}};
const Segment SGI396438[] = {{&S24005[32], 9}, {NULL}};
const Segment SGI396439[] = {{&S24005[27], 4}, {NULL}};
const Segment SGI396440[] = {{&S24005[94], 9}, {NULL}};
const Segment SGI396441[] = {{&S64640[67], 4}, {NULL}};
const Segment SGI396442[] = {{&S64640[38], 9}, {NULL}};
const Segment SGI396443[] = {{&S64641[64], 4}, {NULL}};
const Segment SGI396444[] = {{&S87082[25], 13}, {NULL}};
const Segment SGI396445[] = {{&S64986[49], 3}, {NULL}};
const Segment SGI396446[] = {{&S64986[28], 10}, {NULL}};
const Segment SGI396447[] = {{&S64987[46], 3}, {NULL}};
const Segment SGI396448[] = {{&S65108[61], 4}, {NULL}};
const Segment SGI396449[] = {{&S65108[36], 5}, {NULL}};
const Segment SGI396450[] = {{&S65109[61], 4}, {NULL}};
const Segment SGI396451[] = {{&S65109[36], 5}, {NULL}};
const Segment SGI396452[] = {{&S65110[62], 4}, {NULL}};
const Segment SGI396453[] = {{&S65110[36], 6}, {NULL}};
const Segment SGI396454[] = {{&S65111[64], 4}, {NULL}};
const Segment SGI396455[] = {{&S65111[36], 8}, {NULL}};
const Segment SGI396456[] = {{&S65112[62], 4}, {NULL}};
const Segment SGI396457[] = {{&S65112[36], 6}, {NULL}};
const Segment SGI396458[] = {{&S65113[62], 4}, {NULL}};
const Segment SGI396459[] = {{&S65113[36], 6}, {NULL}};
const Segment SGI396460[] = {{&S65114[64], 4}, {NULL}};
const Segment SGI396461[] = {{&S65114[36], 8}, {NULL}};
const Segment SGI396462[] = {{&S65115[61], 4}, {NULL}};
const Segment SGI396463[] = {{&S65115[36], 5}, {NULL}};
const Segment SGI396464[] = {{&S65116[61], 4}, {NULL}};
const Segment SGI396465[] = {{&S65116[36], 5}, {NULL}};
const Segment SGI396466[] = {{&S65117[62], 4}, {NULL}};
const Segment SGI396467[] = {{&S65117[36], 6}, {NULL}};
const Segment SGI396468[] = {{&S65118[61], 4}, {NULL}};
const Segment SGI396469[] = {{&S65118[36], 5}, {NULL}};
const Segment SGI396470[] = {{&S65119[64], 4}, {NULL}};
const Segment SGI396471[] = {{&S65119[36], 8}, {NULL}};
const Segment SGI396472[] = {{&S65120[58], 4}, {NULL}};
const Segment SGI396473[] = {{&S65121[69], 4}, {NULL}};
const Segment SGI396474[] = {{&S65121[36], 13}, {NULL}};
const Segment SGI396475[] = {{&S65122[69], 4}, {NULL}};
const Segment SGI396476[] = {{&S65122[36], 13}, {NULL}};
const Segment SGI396477[] = {{&S65123[58], 4}, {NULL}};
const Segment SGI396478[] = {{&S65405[45], 3}, {NULL}};
const Segment SGI396479[] = {{&S65405[28], 6}, {NULL}};
const Segment SGI396480[] = {{&S65406[45], 3}, {NULL}};
const Segment SGI396481[] = {{&S65406[28], 6}, {NULL}};
const Segment SGI396482[] = {{&S65408[54], 3}, {NULL}};
const Segment SGI396483[] = {{&S65408[28], 15}, {NULL}};
const Segment SGI396484[] = {{&S65409[44], 3}, {NULL}};
const Segment SGI396485[] = {{&S65409[28], 5}, {NULL}};
const Segment SGI396486[] = {{&S65410[44], 3}, {NULL}};
const Segment SGI396487[] = {{&S65410[28], 5}, {NULL}};
const Segment SGI396488[] = {{&S65412[42], 3}, {NULL}};
const Segment SGI396489[] = {{&S65714[66], 4}, {NULL}};
const Segment SGI396490[] = {{&S65714[39], 7}, {NULL}};
const Segment SGI396491[] = {{&S65715[66], 4}, {NULL}};
const Segment SGI396492[] = {{&S65715[39], 7}, {NULL}};
const Segment SGI396493[] = {{&S65716[66], 4}, {NULL}};
const Segment SGI396494[] = {{&S65716[39], 7}, {NULL}};
const Segment SGI396495[] = {{&S65905[64], 4}, {NULL}};
const Segment SGI396496[] = {{&S65905[35], 9}, {NULL}};
const Segment SGI396497[] = {{&S65906[64], 4}, {NULL}};
const Segment SGI396498[] = {{&S65906[35], 9}, {NULL}};
const Segment SGI396499[] = {{&S65907[61], 4}, {NULL}};
const Segment SGI396500[] = {{&S65908[61], 4}, {NULL}};
const Segment SGI396501[] = {{&S65909[61], 4}, {NULL}};
const Segment SGI396502[] = {{&S65910[64], 4}, {NULL}};
const Segment SGI396503[] = {{&S65910[35], 9}, {NULL}};
const Segment SGI396504[] = {{&S92493[0], 61}, {NULL}};
const Segment SGI396505[] = {{&S66376[66], 4}, {NULL}};
const Segment SGI396506[] = {{&S66376[39], 7}, {NULL}};
const Segment SGI396507[] = {{&S66377[66], 4}, {NULL}};
const Segment SGI396508[] = {{&S66377[39], 7}, {NULL}};
const Segment SGI396509[] = {{&S66378[66], 4}, {NULL}};
const Segment SGI396510[] = {{&S66378[39], 7}, {NULL}};
const Segment SGI396511[] = {{&S66419[49], 3}, {NULL}};
const Segment SGI396512[] = {{&S66419[28], 10}, {NULL}};
const Segment SGI396513[] = {{&S66420[46], 3}, {NULL}};
const Segment SGI396514[] = {{&S66446[67], 4}, {NULL}};
const Segment SGI396515[] = {{&S66446[38], 9}, {NULL}};
const Segment SGI396516[] = {{&S66447[64], 4}, {NULL}};
const Segment SGI396517[] = {{&S66474[61], 4}, {NULL}};
const Segment SGI396518[] = {{&S66474[36], 5}, {NULL}};
const Segment SGI396519[] = {{&S66475[69], 4}, {NULL}};
const Segment SGI396520[] = {{&S66475[36], 13}, {NULL}};
const Segment SGI396521[] = {{&S66476[64], 4}, {NULL}};
const Segment SGI396522[] = {{&S66476[36], 8}, {NULL}};
const Segment SGI396523[] = {{&S66477[62], 4}, {NULL}};
const Segment SGI396524[] = {{&S66477[36], 6}, {NULL}};
const Segment SGI396525[] = {{&S66478[64], 4}, {NULL}};
const Segment SGI396526[] = {{&S66478[36], 8}, {NULL}};
const Segment SGI396527[] = {{&S66479[65], 4}, {NULL}};
const Segment SGI396528[] = {{&S66479[36], 9}, {NULL}};
const Segment SGI396529[] = {{&S66480[64], 4}, {NULL}};
const Segment SGI396530[] = {{&S66480[36], 8}, {NULL}};
const Segment SGI396531[] = {{&S66481[62], 4}, {NULL}};
const Segment SGI396532[] = {{&S66481[36], 6}, {NULL}};
const Segment SGI396533[] = {{&S66482[64], 4}, {NULL}};
const Segment SGI396534[] = {{&S66482[36], 8}, {NULL}};
const Segment SGI396535[] = {{&S66483[62], 4}, {NULL}};
const Segment SGI396536[] = {{&S66483[36], 6}, {NULL}};
const Segment SGI396537[] = {{&S66484[62], 4}, {NULL}};
const Segment SGI396538[] = {{&S66484[36], 6}, {NULL}};
const Segment SGI396539[] = {{&S66485[61], 4}, {NULL}};
const Segment SGI396540[] = {{&S66485[36], 5}, {NULL}};
const Segment SGI396541[] = {{&S66486[71], 4}, {NULL}};
const Segment SGI396542[] = {{&S66486[36], 15}, {NULL}};
const Segment SGI396543[] = {{&S66487[70], 4}, {NULL}};
const Segment SGI396544[] = {{&S66487[36], 14}, {NULL}};
const Segment SGI396545[] = {{&S66526[68], 4}, {NULL}};
const Segment SGI396546[] = {{&S66526[38], 10}, {NULL}};
const Segment SGI396547[] = {{&S95708[30], 14}, {NULL}};
const Segment SGI396548[] = {{&S67036[67], 4}, {NULL}};
const Segment SGI396549[] = {{&S67036[38], 9}, {NULL}};
const Segment SGI396550[] = {{&S93289[0], 43}, {NULL}};
const Segment SGI396551[] = {{&S93304[0], 60}, {NULL}};
const Segment SGI396552[] = {{&S93340[0], 62}, {NULL}};
const Segment SGI396553[] = {{&S67582[67], 4}, {NULL}};
const Segment SGI396554[] = {{&S67582[38], 9}, {NULL}};
const Segment SGI396555[] = {{&S67588[67], 4}, {NULL}};
const Segment SGI396556[] = {{&S67588[38], 9}, {NULL}};
const Segment SGI396557[] = {{&S67636[67], 4}, {NULL}};
const Segment SGI396558[] = {{&S67636[38], 9}, {NULL}};
const Segment SGI396559[] = {{&S67637[68], 4}, {NULL}};
const Segment SGI396560[] = {{&S67637[38], 10}, {NULL}};
const Segment SGI396561[] = {{&S67638[68], 4}, {NULL}};
const Segment SGI396562[] = {{&S67638[38], 10}, {NULL}};
const Segment SGI396563[] = {{&S89712[25], 13}, {NULL}};
const Segment SGI396564[] = {{&S89712[72], 6}, {NULL}};
const Segment SGI396565[] = {{&S67735[67], 4}, {NULL}};
const Segment SGI396566[] = {{&S67735[38], 9}, {NULL}};
const Segment SGI396567[] = {{&S94014[0], 61}, {NULL}};
const Segment SGI396568[] = {{&S94063[0], 60}, {NULL}};
const Segment SGI396569[] = {{&S90238[25], 13}, {NULL}};
const Segment SGI396570[] = {{&S68736[61], 4}, {NULL}};
const Segment SGI396571[] = {{&S68736[36], 5}, {NULL}};
const Segment SGI396572[] = {{&S68737[62], 4}, {NULL}};
const Segment SGI396573[] = {{&S68737[36], 6}, {NULL}};
const Segment SGI396574[] = {{&S68738[61], 4}, {NULL}};
const Segment SGI396575[] = {{&S68738[36], 5}, {NULL}};
const Segment SGI396576[] = {{&S68739[62], 4}, {NULL}};
const Segment SGI396577[] = {{&S68739[36], 6}, {NULL}};
const Segment SGI396578[] = {{&S68740[69], 4}, {NULL}};
const Segment SGI396579[] = {{&S68740[36], 13}, {NULL}};
const Segment SGI396580[] = {{&S68741[71], 4}, {NULL}};
const Segment SGI396581[] = {{&S68741[36], 15}, {NULL}};
const Segment SGI396582[] = {{&S68742[58], 4}, {NULL}};
const Segment SGI396583[] = {{&S68743[58], 4}, {NULL}};
const Segment SGI396584[] = {{&S68768[65], 4}, {NULL}};
const Segment SGI396585[] = {{&S68768[38], 7}, {NULL}};
const Segment SGI396586[] = {{&S68769[70], 4}, {NULL}};
const Segment SGI396587[] = {{&S68769[38], 12}, {NULL}};
const Segment SGI396588[] = {{&S68770[65], 4}, {NULL}};
const Segment SGI396589[] = {{&S68770[38], 7}, {NULL}};
const Segment SGI396590[] = {{&S68771[62], 4}, {NULL}};
const Segment SGI396591[] = {{&S68771[38], 4}, {NULL}};
const Segment SGI396592[] = {{&S68772[68], 4}, {NULL}};
const Segment SGI396593[] = {{&S68772[38], 10}, {NULL}};
const Segment SGI396594[] = {{&S68773[63], 4}, {NULL}};
const Segment SGI396595[] = {{&S68773[38], 5}, {NULL}};
const Segment SGI396596[] = {{&S68774[61], 4}, {NULL}};
const Segment SGI396597[] = {{&S68774[38], 3}, {NULL}};
const Segment SGI396598[] = {{&S68775[67], 4}, {NULL}};
const Segment SGI396599[] = {{&S68775[38], 9}, {NULL}};
const Segment SGI396600[] = {{&S68776[62], 4}, {NULL}};
const Segment SGI396601[] = {{&S68776[38], 4}, {NULL}};
const Segment SGI396602[] = {{&S68777[67], 4}, {NULL}};
const Segment SGI396603[] = {{&S68777[38], 9}, {NULL}};
const Segment SGI396604[] = {{&S68778[62], 4}, {NULL}};
const Segment SGI396605[] = {{&S68778[38], 4}, {NULL}};
const Segment SGI396606[] = {{&S68781[71], 4}, {NULL}};
const Segment SGI396607[] = {{&S68781[38], 13}, {NULL}};
const Segment SGI396608[] = {{&S68783[66], 4}, {NULL}};
const Segment SGI396609[] = {{&S68783[38], 8}, {NULL}};
const Segment SGI396610[] = {{&S68784[67], 4}, {NULL}};
const Segment SGI396611[] = {{&S68784[38], 9}, {NULL}};
const Segment SGI396612[] = {{&S68785[62], 4}, {NULL}};
const Segment SGI396613[] = {{&S68785[38], 4}, {NULL}};
const Segment SGI396614[] = {{&S68786[67], 4}, {NULL}};
const Segment SGI396615[] = {{&S68786[38], 9}, {NULL}};
const Segment SGI396616[] = {{&S68893[69], 4}, {NULL}};
const Segment SGI396617[] = {{&S68893[38], 11}, {NULL}};
const Segment SGI396618[] = {{&S69089[68], 4}, {NULL}};
const Segment SGI396619[] = {{&S69089[38], 10}, {NULL}};
const Segment SGI396620[] = {{&S69090[67], 4}, {NULL}};
const Segment SGI396621[] = {{&S69090[38], 9}, {NULL}};
const Segment SGI396622[] = {{&S69304[65], 4}, {NULL}};
const Segment SGI396623[] = {{&S69304[36], 9}, {NULL}};
const Segment SGI396624[] = {{&S69305[62], 4}, {NULL}};
const Segment SGI396625[] = {{&S69306[62], 4}, {NULL}};
const Segment SGI396626[] = {{&S69307[65], 4}, {NULL}};
const Segment SGI396627[] = {{&S69307[36], 9}, {NULL}};
const Segment SGI396628[] = {{&S69308[65], 4}, {NULL}};
const Segment SGI396629[] = {{&S69308[36], 9}, {NULL}};
const Segment SGI396630[] = {{&S95758[0], 31}, {NULL}};
const Segment SGI396631[] = {{&S69739[67], 4}, {NULL}};
const Segment SGI396632[] = {{&S69739[38], 9}, {NULL}};
const Segment SGI396633[] = {{&S69740[64], 4}, {NULL}};
const Segment SGI396634[] = {{&S69823[67], 4}, {NULL}};
const Segment SGI396635[] = {{&S69823[38], 9}, {NULL}};
const Segment SGI396636[] = {{&S91890[25], 13}, {NULL}};
const Segment SGI396637[] = {{&S96376[0], 57}, {NULL}};
const Segment SGI396638[] = {{&S69987[67], 4}, {NULL}};
const Segment SGI396639[] = {{&S69987[38], 9}, {NULL}};
const Segment SGI396640[] = {{&S29794[32], 9}, {NULL}};
const Segment SGI396641[] = {{&S29794[27], 4}, {NULL}};
const Segment SGI396642[] = {{&S29794[94], 9}, {NULL}};
const Segment SGI396643[] = {{&S70138[69], 4}, {NULL}};
const Segment SGI396644[] = {{&S70138[38], 11}, {NULL}};
const Segment SGI396645[] = {{&S70139[66], 4}, {NULL}};
const Segment SGI396646[] = {{&S70238[67], 4}, {NULL}};
const Segment SGI396647[] = {{&S70238[38], 9}, {NULL}};
const Segment SGI396648[] = {{&S98580[30], 12}, {NULL}};
const Segment SGI396649[] = {{&S70322[69], 4}, {NULL}};
const Segment SGI396650[] = {{&S70322[38], 11}, {NULL}};
const Segment SGI396651[] = {{&S70441[62], 4}, {NULL}};
const Segment SGI396652[] = {{&S70441[38], 4}, {NULL}};
const Segment SGI396653[] = {{&S70442[73], 4}, {NULL}};
const Segment SGI396654[] = {{&S70442[38], 15}, {NULL}};
const Segment SGI396655[] = {{&S70443[71], 4}, {NULL}};
const Segment SGI396656[] = {{&S70443[38], 13}, {NULL}};
const Segment SGI396657[] = {{&S70444[61], 4}, {NULL}};
const Segment SGI396658[] = {{&S70444[38], 3}, {NULL}};
const Segment SGI396659[] = {{&S70445[72], 4}, {NULL}};
const Segment SGI396660[] = {{&S70445[38], 14}, {NULL}};
const Segment SGI396661[] = {{&S70446[62], 4}, {NULL}};
const Segment SGI396662[] = {{&S70446[38], 4}, {NULL}};
const Segment SGI396663[] = {{&S70447[71], 4}, {NULL}};
const Segment SGI396664[] = {{&S70447[38], 13}, {NULL}};
const Segment SGI396665[] = {{&S70448[61], 4}, {NULL}};
const Segment SGI396666[] = {{&S70448[38], 3}, {NULL}};
const Segment SGI396667[] = {{&S70449[61], 4}, {NULL}};
const Segment SGI396668[] = {{&S70449[38], 3}, {NULL}};
const Segment SGI396669[] = {{&S70450[63], 4}, {NULL}};
const Segment SGI396670[] = {{&S70450[38], 5}, {NULL}};
const Segment SGI396671[] = {{&S70451[69], 4}, {NULL}};
const Segment SGI396672[] = {{&S70451[38], 11}, {NULL}};
const Segment SGI396673[] = {{&S70452[62], 4}, {NULL}};
const Segment SGI396674[] = {{&S70452[38], 4}, {NULL}};
const Segment SGI396675[] = {{&S70453[67], 4}, {NULL}};
const Segment SGI396676[] = {{&S70453[38], 9}, {NULL}};
const Segment SGI396677[] = {{&S70454[63], 4}, {NULL}};
const Segment SGI396678[] = {{&S70454[38], 5}, {NULL}};
const Segment SGI396679[] = {{&S70455[62], 4}, {NULL}};
const Segment SGI396680[] = {{&S70455[38], 4}, {NULL}};
const Segment SGI396681[] = {{&S70456[65], 4}, {NULL}};
const Segment SGI396682[] = {{&S70456[38], 7}, {NULL}};
const Segment SGI396683[] = {{&S70570[69], 4}, {NULL}};
const Segment SGI396684[] = {{&S70570[44], 5}, {NULL}};
const Segment SGI396685[] = {{&S70571[69], 4}, {NULL}};
const Segment SGI396686[] = {{&S70571[44], 5}, {NULL}};
const Segment SGI396687[] = {{&S70589[71], 4}, {NULL}};
const Segment SGI396688[] = {{&S70589[38], 13}, {NULL}};
const Segment SGI396689[] = {{&S70642[49], 3}, {NULL}};
const Segment SGI396690[] = {{&S70642[28], 10}, {NULL}};
const Segment SGI396691[] = {{&S70907[67], 4}, {NULL}};
const Segment SGI396692[] = {{&S70907[38], 9}, {NULL}};
const Segment SGI396693[] = {{&S99066[30], 13}, {NULL}};
const Segment SGI396694[] = {{&S71052[62], 4}, {NULL}};
const Segment SGI396695[] = {{&S71052[36], 6}, {NULL}};
const Segment SGI396696[] = {{&S71053[64], 4}, {NULL}};
const Segment SGI396697[] = {{&S71053[36], 8}, {NULL}};
const Segment SGI396698[] = {{&S71054[64], 4}, {NULL}};
const Segment SGI396699[] = {{&S71054[36], 8}, {NULL}};
const Segment SGI396700[] = {{&S71055[65], 4}, {NULL}};
const Segment SGI396701[] = {{&S71055[36], 9}, {NULL}};
const Segment SGI396702[] = {{&S71056[69], 4}, {NULL}};
const Segment SGI396703[] = {{&S71056[36], 13}, {NULL}};
const Segment SGI396704[] = {{&S71057[64], 4}, {NULL}};
const Segment SGI396705[] = {{&S71057[36], 8}, {NULL}};
const Segment SGI396706[] = {{&S71058[64], 4}, {NULL}};
const Segment SGI396707[] = {{&S71058[36], 8}, {NULL}};
const Segment SGI396708[] = {{&S71059[61], 4}, {NULL}};
const Segment SGI396709[] = {{&S71059[36], 5}, {NULL}};
const Segment SGI396710[] = {{&S71060[70], 4}, {NULL}};
const Segment SGI396711[] = {{&S71060[36], 14}, {NULL}};
const Segment SGI396712[] = {{&S71061[61], 4}, {NULL}};
const Segment SGI396713[] = {{&S71061[36], 5}, {NULL}};
const Segment SGI396714[] = {{&S71107[87], 4}, {NULL}};
const Segment SGI396715[] = {{&S71107[28], 9}, {NULL}};
const Segment SGI396716[] = {{&S71108[84], 4}, {NULL}};
const Segment SGI396717[] = {{&S71298[67], 4}, {NULL}};
const Segment SGI396718[] = {{&S71298[38], 9}, {NULL}};
const Segment SGI396719[] = {{&S71313[62], 4}, {NULL}};
const Segment SGI396720[] = {{&S71313[38], 4}, {NULL}};
const Segment SGI396721[] = {{&S71317[71], 4}, {NULL}};
const Segment SGI396722[] = {{&S71317[38], 13}, {NULL}};
const Segment SGI396723[] = {{&S71423[68], 4}, {NULL}};
const Segment SGI396724[] = {{&S71423[38], 10}, {NULL}};
const Segment SGI396725[] = {{&S93638[25], 13}, {NULL}};
const Segment SGI396726[] = {{&S93638[110], 6}, {NULL}};
const Segment SGI396727[] = {{&S71759[88], 4}, {NULL}};
const Segment SGI396728[] = {{&S71759[28], 10}, {NULL}};
const Segment SGI396729[] = {{&S71815[71], 4}, {NULL}};
const Segment SGI396730[] = {{&S71815[38], 13}, {NULL}};
const Segment SGI396731[] = {{&S71823[67], 4}, {NULL}};
const Segment SGI396732[] = {{&S71823[38], 9}, {NULL}};
const Segment SGI396733[] = {{&S71824[64], 4}, {NULL}};
const Segment SGI396734[] = {{&S72010[67], 4}, {NULL}};
const Segment SGI396735[] = {{&S72010[38], 9}, {NULL}};
const Segment SGI396736[] = {{&S72011[68], 4}, {NULL}};
const Segment SGI396737[] = {{&S72011[38], 10}, {NULL}};
const Segment SGI396738[] = {{&S72108[67], 4}, {NULL}};
const Segment SGI396739[] = {{&S72108[38], 9}, {NULL}};
const Segment SGI396740[] = {{&S72202[67], 4}, {NULL}};
const Segment SGI396741[] = {{&S72202[38], 9}, {NULL}};
const Segment SGI396742[] = {{&S72224[67], 4}, {NULL}};
const Segment SGI396743[] = {{&S72224[38], 9}, {NULL}};
const Segment SGI396744[] = {{&S72287[67], 4}, {NULL}};
const Segment SGI396745[] = {{&S72287[38], 9}, {NULL}};
const Segment SGI396746[] = {{&S94222[25], 13}, {NULL}};
const Segment SGI396747[] = {{&S94222[72], 6}, {NULL}};
const Segment SGI396748[] = {{&S72409[67], 4}, {NULL}};
const Segment SGI396749[] = {{&S72409[38], 9}, {NULL}};
const Segment SGI396750[] = {{&S72641[64], 4}, {NULL}};
const Segment SGI396751[] = {{&S72641[37], 7}, {NULL}};
const Segment SGI396752[] = {{&S72642[64], 4}, {NULL}};
const Segment SGI396753[] = {{&S72642[37], 7}, {NULL}};
const Segment SGI396754[] = {{&S72643[64], 4}, {NULL}};
const Segment SGI396755[] = {{&S72643[37], 7}, {NULL}};
const Segment SGI396756[] = {{&S72644[64], 4}, {NULL}};
const Segment SGI396757[] = {{&S72644[37], 7}, {NULL}};
const Segment SGI396758[] = {{&S72727[67], 4}, {NULL}};
const Segment SGI396759[] = {{&S72727[38], 9}, {NULL}};
const Segment SGI396760[] = {{&S72830[65], 4}, {NULL}};
const Segment SGI396761[] = {{&S72830[38], 7}, {NULL}};
const Segment SGI396762[] = {{&S72831[64], 4}, {NULL}};
const Segment SGI396763[] = {{&S72831[38], 6}, {NULL}};
const Segment SGI396764[] = {{&S72878[67], 4}, {NULL}};
const Segment SGI396765[] = {{&S72878[38], 9}, {NULL}};
const Segment SGI396766[] = {{&S72908[69], 4}, {NULL}};
const Segment SGI396767[] = {{&S72908[38], 11}, {NULL}};
const Segment SGI396768[] = {{&S100785[30], 13}, {NULL}};
const Segment SGI396769[] = {{&S73045[71], 4}, {NULL}};
const Segment SGI396770[] = {{&S73045[38], 13}, {NULL}};
const Segment SGI396771[] = {{&S73046[68], 4}, {NULL}};
const Segment SGI396772[] = {{&S73046[38], 10}, {NULL}};
const Segment SGI396773[] = {{&S33187[32], 9}, {NULL}};
const Segment SGI396774[] = {{&S33187[27], 4}, {NULL}};
const Segment SGI396775[] = {{&S33187[94], 9}, {NULL}};
const Segment SGI396776[] = {{&S33188[32], 16}, {NULL}};
const Segment SGI396777[] = {{&S33188[27], 4}, {NULL}};
const Segment SGI396778[] = {{&S33188[101], 9}, {NULL}};
const Segment SGI396779[] = {{&S73157[67], 4}, {NULL}};
const Segment SGI396780[] = {{&S73157[38], 9}, {NULL}};
const Segment SGI396781[] = {{&S33313[42], 14}, {NULL}};
const Segment SGI396782[] = {{&S33313[42], 26}, {NULL}};
const Segment SGI396783[] = {{&S33313[30], 4}, {NULL}};
const Segment SGI396784[] = {{&S33313[117], 3}, {NULL}};
const Segment SGI396785[] = {{&S33314[42], 14}, {NULL}};
const Segment SGI396786[] = {{&S33314[42], 26}, {NULL}};
const Segment SGI396787[] = {{&S33314[30], 4}, {NULL}};
const Segment SGI396788[] = {{&S33314[117], 3}, {NULL}};
const Segment SGI396789[] = {{&S33315[42], 14}, {NULL}};
const Segment SGI396790[] = {{&S33315[42], 26}, {NULL}};
const Segment SGI396791[] = {{&S33315[30], 4}, {NULL}};
const Segment SGI396792[] = {{&S33315[117], 3}, {NULL}};
const Segment SGI396793[] = {{&S33316[42], 14}, {NULL}};
const Segment SGI396794[] = {{&S33316[42], 26}, {NULL}};
const Segment SGI396795[] = {{&S33316[30], 4}, {NULL}};
const Segment SGI396796[] = {{&S33316[117], 3}, {NULL}};
const Segment SGI396797[] = {{&S33317[42], 14}, {NULL}};
const Segment SGI396798[] = {{&S33317[42], 26}, {NULL}};
const Segment SGI396799[] = {{&S33317[30], 4}, {NULL}};
const Segment SGI396800[] = {{&S33317[117], 3}, {NULL}};
const Segment SGI396801[] = {{&S33318[36], 14}, {NULL}};
const Segment SGI396802[] = {{&S33318[31], 4}, {NULL}};
const Segment SGI396803[] = {{&S33319[42], 14}, {NULL}};
const Segment SGI396804[] = {{&S33319[42], 26}, {NULL}};
const Segment SGI396805[] = {{&S33319[30], 4}, {NULL}};
const Segment SGI396806[] = {{&S33319[117], 3}, {NULL}};
const Segment SGI396807[] = {{&S33320[40], 14}, {NULL}};
const Segment SGI396808[] = {{&S33320[30], 4}, {NULL}};
const Segment SGI396809[] = {{&S33320[115], 3}, {NULL}};
const Segment SGI396810[] = {{&S33321[42], 14}, {NULL}};
const Segment SGI396811[] = {{&S33321[42], 26}, {NULL}};
const Segment SGI396812[] = {{&S33321[30], 4}, {NULL}};
const Segment SGI396813[] = {{&S33321[117], 3}, {NULL}};
const Segment SGI396814[] = {{&S33322[42], 14}, {NULL}};
const Segment SGI396815[] = {{&S33322[42], 26}, {NULL}};
const Segment SGI396816[] = {{&S33322[30], 4}, {NULL}};
const Segment SGI396817[] = {{&S33322[117], 3}, {NULL}};
const Segment SGI396818[] = {{&S33407[42], 9}, {NULL}};
const Segment SGI396819[] = {{&S33407[42], 21}, {NULL}};
const Segment SGI396820[] = {{&S33407[30], 4}, {NULL}};
const Segment SGI396821[] = {{&S33407[112], 3}, {NULL}};
const Segment SGI396822[] = {{&S33408[42], 9}, {NULL}};
const Segment SGI396823[] = {{&S33408[42], 21}, {NULL}};
const Segment SGI396824[] = {{&S33408[30], 4}, {NULL}};
const Segment SGI396825[] = {{&S33408[112], 3}, {NULL}};
const Segment SGI396826[] = {{&S33409[42], 9}, {NULL}};
const Segment SGI396827[] = {{&S33409[42], 21}, {NULL}};
const Segment SGI396828[] = {{&S33409[30], 4}, {NULL}};
const Segment SGI396829[] = {{&S33409[112], 3}, {NULL}};
const Segment SGI396830[] = {{&S33410[36], 9}, {NULL}};
const Segment SGI396831[] = {{&S33410[31], 4}, {NULL}};
const Segment SGI396832[] = {{&S33411[42], 9}, {NULL}};
const Segment SGI396833[] = {{&S33411[42], 21}, {NULL}};
const Segment SGI396834[] = {{&S33411[30], 4}, {NULL}};
const Segment SGI396835[] = {{&S33411[112], 3}, {NULL}};
const Segment SGI396836[] = {{&S33412[42], 9}, {NULL}};
const Segment SGI396837[] = {{&S33412[42], 21}, {NULL}};
const Segment SGI396838[] = {{&S33412[30], 4}, {NULL}};
const Segment SGI396839[] = {{&S33412[112], 3}, {NULL}};
const Segment SGI396840[] = {{&S33413[40], 9}, {NULL}};
const Segment SGI396841[] = {{&S33413[30], 4}, {NULL}};
const Segment SGI396842[] = {{&S33413[110], 3}, {NULL}};
const Segment SGI396843[] = {{&S33414[42], 9}, {NULL}};
const Segment SGI396844[] = {{&S33414[42], 21}, {NULL}};
const Segment SGI396845[] = {{&S33414[30], 4}, {NULL}};
const Segment SGI396846[] = {{&S33414[112], 3}, {NULL}};
const Segment SGI396847[] = {{&S33415[42], 9}, {NULL}};
const Segment SGI396848[] = {{&S33415[42], 21}, {NULL}};
const Segment SGI396849[] = {{&S33415[30], 4}, {NULL}};
const Segment SGI396850[] = {{&S33415[112], 3}, {NULL}};
const Segment SGI396851[] = {{&S33416[42], 9}, {NULL}};
const Segment SGI396852[] = {{&S33416[42], 21}, {NULL}};
const Segment SGI396853[] = {{&S33416[30], 4}, {NULL}};
const Segment SGI396854[] = {{&S33416[112], 3}, {NULL}};
const Segment SGI396855[] = {{&S73493[66], 4}, {NULL}};
const Segment SGI396856[] = {{&S73493[36], 10}, {NULL}};
const Segment SGI396857[] = {{&S73494[63], 4}, {NULL}};
const Segment SGI396858[] = {{&S73495[66], 4}, {NULL}};
const Segment SGI396859[] = {{&S73495[36], 10}, {NULL}};
const Segment SGI396860[] = {{&S73496[67], 4}, {NULL}};
const Segment SGI396861[] = {{&S73496[37], 10}, {NULL}};
const Segment SGI396862[] = {{&S73553[67], 4}, {NULL}};
const Segment SGI396863[] = {{&S73553[38], 9}, {NULL}};
const Segment SGI396864[] = {{&S101521[30], 12}, {NULL}};
const Segment SGI396865[] = {{&S73867[67], 4}, {NULL}};
const Segment SGI396866[] = {{&S73867[38], 9}, {NULL}};
const Segment SGI396867[] = {{&S74199[67], 4}, {NULL}};
const Segment SGI396868[] = {{&S74199[38], 9}, {NULL}};
const Segment SGI396869[] = {{&S102231[30], 13}, {NULL}};
const Segment SGI396870[] = {{&S102248[30], 13}, {NULL}};
const Segment SGI396871[] = {{&S74502[67], 4}, {NULL}};
const Segment SGI396872[] = {{&S74502[38], 9}, {NULL}};
const Segment SGI396873[] = {{&S96355[25], 13}, {NULL}};
const Segment SGI396874[] = {{&S74630[88], 4}, {NULL}};
const Segment SGI396875[] = {{&S74630[28], 10}, {NULL}};
const Segment SGI396876[] = {{&S74631[85], 4}, {NULL}};
const Segment SGI396877[] = {{&S74686[67], 4}, {NULL}};
const Segment SGI396878[] = {{&S74686[38], 9}, {NULL}};
const Segment SGI396879[] = {{&S74687[68], 4}, {NULL}};
const Segment SGI396880[] = {{&S74687[38], 10}, {NULL}};
const Segment SGI396881[] = {{&S74810[72], 4}, {NULL}};
const Segment SGI396882[] = {{&S74810[38], 14}, {NULL}};
const Segment SGI396883[] = {{&S74928[61], 4}, {NULL}};
const Segment SGI396884[] = {{&S74928[36], 5}, {NULL}};
const Segment SGI396885[] = {{&S74929[70], 4}, {NULL}};
const Segment SGI396886[] = {{&S74929[36], 14}, {NULL}};
const Segment SGI396887[] = {{&S74930[60], 4}, {NULL}};
const Segment SGI396888[] = {{&S74930[36], 4}, {NULL}};
const Segment SGI396889[] = {{&S74931[64], 4}, {NULL}};
const Segment SGI396890[] = {{&S74931[36], 8}, {NULL}};
const Segment SGI396891[] = {{&S74932[61], 4}, {NULL}};
const Segment SGI396892[] = {{&S74933[65], 4}, {NULL}};
const Segment SGI396893[] = {{&S74933[36], 9}, {NULL}};
const Segment SGI396894[] = {{&S74934[62], 4}, {NULL}};
const Segment SGI396895[] = {{&S74934[36], 6}, {NULL}};
const Segment SGI396896[] = {{&S74935[60], 4}, {NULL}};
const Segment SGI396897[] = {{&S74935[36], 4}, {NULL}};
const Segment SGI396898[] = {{&S74936[69], 4}, {NULL}};
const Segment SGI396899[] = {{&S74936[36], 13}, {NULL}};
const Segment SGI396900[] = {{&S74937[66], 4}, {NULL}};
const Segment SGI396901[] = {{&S74937[36], 10}, {NULL}};
const Segment SGI396902[] = {{&S74938[64], 4}, {NULL}};
const Segment SGI396903[] = {{&S74938[36], 8}, {NULL}};
const Segment SGI396904[] = {{&S74939[64], 4}, {NULL}};
const Segment SGI396905[] = {{&S74939[36], 8}, {NULL}};
const Segment SGI396906[] = {{&S74940[61], 4}, {NULL}};
const Segment SGI396907[] = {{&S74940[36], 5}, {NULL}};
const Segment SGI396908[] = {{&S74941[64], 4}, {NULL}};
const Segment SGI396909[] = {{&S74941[36], 8}, {NULL}};
const Segment SGI396910[] = {{&S74942[61], 4}, {NULL}};
const Segment SGI396911[] = {{&S74943[60], 4}, {NULL}};
const Segment SGI396912[] = {{&S74943[36], 4}, {NULL}};
const Segment SGI396913[] = {{&S74944[62], 4}, {NULL}};
const Segment SGI396914[] = {{&S74944[36], 6}, {NULL}};
const Segment SGI396915[] = {{&S74945[61], 4}, {NULL}};
const Segment SGI396916[] = {{&S74945[36], 5}, {NULL}};
const Segment SGI396917[] = {{&S100509[0], 61}, {NULL}};
const Segment SGI396918[] = {{&S75035[65], 4}, {NULL}};
const Segment SGI396919[] = {{&S75035[38], 7}, {NULL}};
const Segment SGI396920[] = {{&S75036[64], 4}, {NULL}};
const Segment SGI396921[] = {{&S75036[38], 6}, {NULL}};
const Segment SGI396922[] = {{&S97015[25], 13}, {NULL}};
const Segment SGI396923[] = {{&S75263[67], 4}, {NULL}};
const Segment SGI396924[] = {{&S75263[38], 9}, {NULL}};
const Segment SGI396925[] = {{&S75353[67], 4}, {NULL}};
const Segment SGI396926[] = {{&S75353[38], 9}, {NULL}};
const Segment SGI396927[] = {{&S75373[64], 4}, {NULL}};
const Segment SGI396928[] = {{&S75373[38], 6}, {NULL}};
const Segment SGI396929[] = {{&S75374[62], 4}, {NULL}};
const Segment SGI396930[] = {{&S75374[38], 4}, {NULL}};
const Segment SGI396931[] = {{&S75375[67], 4}, {NULL}};
const Segment SGI396932[] = {{&S75375[38], 9}, {NULL}};
const Segment SGI396933[] = {{&S75376[64], 4}, {NULL}};
const Segment SGI396934[] = {{&S75376[38], 6}, {NULL}};
const Segment SGI396935[] = {{&S75377[66], 4}, {NULL}};
const Segment SGI396936[] = {{&S75377[38], 8}, {NULL}};
const Segment SGI396937[] = {{&S75378[66], 4}, {NULL}};
const Segment SGI396938[] = {{&S75378[38], 8}, {NULL}};
const Segment SGI396939[] = {{&S75379[73], 4}, {NULL}};
const Segment SGI396940[] = {{&S75379[38], 15}, {NULL}};
const Segment SGI396941[] = {{&S75380[79], 4}, {NULL}};
const Segment SGI396942[] = {{&S75380[38], 21}, {NULL}};
const Segment SGI396943[] = {{&S75381[69], 4}, {NULL}};
const Segment SGI396944[] = {{&S75381[38], 11}, {NULL}};
const Segment SGI396945[] = {{&S75428[68], 4}, {NULL}};
const Segment SGI396946[] = {{&S75428[38], 10}, {NULL}};
const Segment SGI396947[] = {{&S75502[71], 4}, {NULL}};
const Segment SGI396948[] = {{&S75502[38], 13}, {NULL}};
const Segment SGI396949[] = {{&S75625[62], 4}, {NULL}};
const Segment SGI396950[] = {{&S75625[36], 6}, {NULL}};
const Segment SGI396951[] = {{&S75626[64], 4}, {NULL}};
const Segment SGI396952[] = {{&S75626[36], 8}, {NULL}};
const Segment SGI396953[] = {{&S75659[87], 4}, {NULL}};
const Segment SGI396954[] = {{&S75659[28], 9}, {NULL}};
const Segment SGI396955[] = {{&S75897[67], 4}, {NULL}};
const Segment SGI396956[] = {{&S75897[38], 9}, {NULL}};
const Segment SGI396957[] = {{&S75992[67], 4}, {NULL}};
const Segment SGI396958[] = {{&S75992[38], 9}, {NULL}};
const Segment SGI396959[] = {{&S76301[72], 4}, {NULL}};
const Segment SGI396960[] = {{&S76301[38], 14}, {NULL}};
const Segment SGI396961[] = {{&S76347[64], 4}, {NULL}};
const Segment SGI396962[] = {{&S76347[38], 6}, {NULL}};
const Segment SGI396963[] = {{&S76348[64], 4}, {NULL}};
const Segment SGI396964[] = {{&S76348[38], 6}, {NULL}};
const Segment SGI396965[] = {{&S76409[67], 4}, {NULL}};
const Segment SGI396966[] = {{&S76409[38], 9}, {NULL}};
const Segment SGI396967[] = {{&S76504[88], 4}, {NULL}};
const Segment SGI396968[] = {{&S76504[28], 10}, {NULL}};
const Segment SGI396969[] = {{&S76505[85], 4}, {NULL}};
const Segment SGI396970[] = {{&S76612[48], 3}, {NULL}};
const Segment SGI396971[] = {{&S76612[28], 9}, {NULL}};
const Segment SGI396972[] = {{&S76628[69], 4}, {NULL}};
const Segment SGI396973[] = {{&S76628[38], 11}, {NULL}};
const Segment SGI396974[] = {{&S76629[61], 4}, {NULL}};
const Segment SGI396975[] = {{&S76629[38], 3}, {NULL}};
const Segment SGI396976[] = {{&S76792[69], 4}, {NULL}};
const Segment SGI396977[] = {{&S76792[38], 11}, {NULL}};
const Segment SGI396978[] = {{&S36931[32], 9}, {NULL}};
const Segment SGI396979[] = {{&S36931[27], 4}, {NULL}};
const Segment SGI396980[] = {{&S36931[94], 9}, {NULL}};
const Segment SGI396981[] = {{&S76985[67], 4}, {NULL}};
const Segment SGI396982[] = {{&S76985[38], 9}, {NULL}};
const Segment SGI396983[] = {{&S77109[69], 4}, {NULL}};
const Segment SGI396984[] = {{&S77109[44], 5}, {NULL}};
const Segment SGI396985[] = {{&S77110[67], 4}, {NULL}};
const Segment SGI396986[] = {{&S77110[38], 9}, {NULL}};
const Segment SGI396987[] = {{&S77111[68], 4}, {NULL}};
const Segment SGI396988[] = {{&S77111[38], 10}, {NULL}};
const Segment SGI396989[] = {{&S37252[42], 9}, {NULL}};
const Segment SGI396990[] = {{&S37252[42], 21}, {NULL}};
const Segment SGI396991[] = {{&S37252[30], 4}, {NULL}};
const Segment SGI396992[] = {{&S37252[112], 3}, {NULL}};
const Segment SGI396993[] = {{&S37253[42], 9}, {NULL}};
const Segment SGI396994[] = {{&S37253[42], 21}, {NULL}};
const Segment SGI396995[] = {{&S37253[30], 4}, {NULL}};
const Segment SGI396996[] = {{&S37253[112], 3}, {NULL}};
const Segment SGI396997[] = {{&S37254[42], 9}, {NULL}};
const Segment SGI396998[] = {{&S37254[42], 21}, {NULL}};
const Segment SGI396999[] = {{&S37254[30], 4}, {NULL}};
const Segment SGI397000[] = {{&S37254[112], 3}, {NULL}};
const Segment SGI397001[] = {{&S37255[42], 9}, {NULL}};
const Segment SGI397002[] = {{&S37255[42], 21}, {NULL}};
const Segment SGI397003[] = {{&S37255[30], 4}, {NULL}};
const Segment SGI397004[] = {{&S37255[112], 3}, {NULL}};
const Segment SGI397005[] = {{&S37256[42], 9}, {NULL}};
const Segment SGI397006[] = {{&S37256[42], 21}, {NULL}};
const Segment SGI397007[] = {{&S37256[30], 4}, {NULL}};
const Segment SGI397008[] = {{&S37256[112], 3}, {NULL}};
const Segment SGI397009[] = {{&S77206[71], 4}, {NULL}};
const Segment SGI397010[] = {{&S37257[36], 9}, {NULL}};
const Segment SGI397011[] = {{&S77206[38], 13}, {NULL}};
const Segment SGI397012[] = {{&S37257[31], 4}, {NULL}};
const Segment SGI397013[] = {{&S37258[40], 9}, {NULL}};
const Segment SGI397014[] = {{&S37258[30], 4}, {NULL}};
const Segment SGI397015[] = {{&S37258[110], 3}, {NULL}};
const Segment SGI397016[] = {{&S37259[42], 9}, {NULL}};
const Segment SGI397017[] = {{&S77209[67], 4}, {NULL}};
const Segment SGI397018[] = {{&S37259[42], 21}, {NULL}};
const Segment SGI397019[] = {{&S77209[38], 9}, {NULL}};
const Segment SGI397020[] = {{&S37259[30], 4}, {NULL}};
const Segment SGI397021[] = {{&S37259[112], 3}, {NULL}};
const Segment SGI397022[] = {{&S37260[42], 9}, {NULL}};
const Segment SGI397023[] = {{&S37260[42], 21}, {NULL}};
const Segment SGI397024[] = {{&S37260[30], 4}, {NULL}};
const Segment SGI397025[] = {{&S37260[112], 3}, {NULL}};
const Segment SGI397026[] = {{&S37261[42], 9}, {NULL}};
const Segment SGI397027[] = {{&S37261[42], 21}, {NULL}};
const Segment SGI397028[] = {{&S37261[30], 4}, {NULL}};
const Segment SGI397029[] = {{&S37261[112], 3}, {NULL}};
const Segment SGI397030[] = {{&S77249[67], 4}, {NULL}};
const Segment SGI397031[] = {{&S77249[38], 9}, {NULL}};
const Segment SGI397032[] = {{&S77493[72], 4}, {NULL}};
const Segment SGI397033[] = {{&S77493[36], 16}, {NULL}};
const Segment SGI397034[] = {{&S77506[68], 4}, {NULL}};
const Segment SGI397035[] = {{&S77506[38], 10}, {NULL}};
const Segment SGI397036[] = {{&S77507[67], 4}, {NULL}};
const Segment SGI397037[] = {{&S77507[38], 9}, {NULL}};
const Segment SGI397038[] = {{&S77508[67], 4}, {NULL}};
const Segment SGI397039[] = {{&S77508[38], 9}, {NULL}};
const Segment SGI397040[] = {{&S77561[68], 4}, {NULL}};
const Segment SGI397041[] = {{&S77561[38], 10}, {NULL}};
const Segment SGI397042[] = {{&S77562[67], 4}, {NULL}};
const Segment SGI397043[] = {{&S77562[38], 9}, {NULL}};
const Segment SGI397044[] = {{&S77563[64], 4}, {NULL}};
const Segment SGI397045[] = {{&S77563[38], 6}, {NULL}};
const Segment SGI397046[] = {{&S77564[65], 4}, {NULL}};
const Segment SGI397047[] = {{&S77564[38], 7}, {NULL}};
const Segment SGI397048[] = {{&S77565[61], 4}, {NULL}};
const Segment SGI397049[] = {{&S77569[71], 4}, {NULL}};
const Segment SGI397050[] = {{&S77569[38], 13}, {NULL}};
const Segment SGI397051[] = {{&S77609[71], 4}, {NULL}};
const Segment SGI397052[] = {{&S77609[38], 13}, {NULL}};
const Segment SGI397053[] = {{&S77617[67], 4}, {NULL}};
const Segment SGI397054[] = {{&S77617[38], 9}, {NULL}};
const Segment SGI397055[] = {{&S77673[67], 4}, {NULL}};
const Segment SGI397056[] = {{&S77673[38], 9}, {NULL}};
const Segment SGI397057[] = {{&S77717[59], 4}, {NULL}};
const Segment SGI397058[] = {{&S77718[62], 4}, {NULL}};
const Segment SGI397059[] = {{&S77718[38], 4}, {NULL}};
const Segment SGI397060[] = {{&S77722[67], 4}, {NULL}};
const Segment SGI397061[] = {{&S77722[38], 9}, {NULL}};
const Segment SGI397062[] = {{&S77744[66], 4}, {NULL}};
const Segment SGI397063[] = {{&S77744[38], 8}, {NULL}};
const Segment SGI397064[] = {{&S77750[67], 4}, {NULL}};
const Segment SGI397065[] = {{&S77750[38], 9}, {NULL}};
const Segment SGI397066[] = {{&S77791[62], 4}, {NULL}};
const Segment SGI397067[] = {{&S77791[38], 4}, {NULL}};
const Segment SGI397068[] = {{&S77852[67], 4}, {NULL}};
const Segment SGI397069[] = {{&S77852[38], 9}, {NULL}};
const Segment SGI397070[] = {{&S99590[25], 13}, {NULL}};
const Segment SGI397071[] = {{&S77987[66], 4}, {NULL}};
const Segment SGI397072[] = {{&S77987[39], 7}, {NULL}};
const Segment SGI397073[] = {{&S77988[66], 4}, {NULL}};
const Segment SGI397074[] = {{&S77988[39], 7}, {NULL}};
const Segment SGI397075[] = {{&S78006[71], 4}, {NULL}};
const Segment SGI397076[] = {{&S78006[38], 13}, {NULL}};
const Segment SGI397077[] = {{&S78235[49], 3}, {NULL}};
const Segment SGI397078[] = {{&S78235[28], 10}, {NULL}};
const Segment SGI397079[] = {{&S78237[48], 3}, {NULL}};
const Segment SGI397080[] = {{&S78237[28], 9}, {NULL}};
const Segment SGI397081[] = {{&S78243[61], 4}, {NULL}};
const Segment SGI397082[] = {{&S78243[35], 6}, {NULL}};
const Segment SGI397083[] = {{&S78250[73], 4}, {NULL}};
const Segment SGI397084[] = {{&S78250[38], 15}, {NULL}};
const Segment SGI397085[] = {{&S78346[87], 4}, {NULL}};
const Segment SGI397086[] = {{&S78346[28], 9}, {NULL}};
const Segment SGI397087[] = {{&S78347[84], 4}, {NULL}};
const Segment SGI397088[] = {{&S38175[32], 10}, {NULL}};
const Segment SGI397089[] = {{&S38175[27], 4}, {NULL}};
const Segment SGI397090[] = {{&S38175[95], 9}, {NULL}};
const Segment SGI397091[] = {{&S78561[67], 4}, {NULL}};
const Segment SGI397092[] = {{&S78561[38], 9}, {NULL}};
const Segment SGI397093[] = {{&S78705[67], 4}, {NULL}};
const Segment SGI397094[] = {{&S78705[38], 9}, {NULL}};
const Segment SGI397095[] = {{&S78718[67], 4}, {NULL}};
const Segment SGI397096[] = {{&S78718[38], 9}, {NULL}};
const Segment SGI397097[] = {{&S78788[69], 4}, {NULL}};
const Segment SGI397098[] = {{&S78788[38], 11}, {NULL}};
const Segment SGI397099[] = {{&S78791[72], 4}, {NULL}};
const Segment SGI397100[] = {{&S78791[38], 14}, {NULL}};
const Segment SGI397101[] = {{&S78801[67], 4}, {NULL}};
const Segment SGI397102[] = {{&S78801[38], 9}, {NULL}};
const Segment SGI397103[] = {{&S78830[62], 4}, {NULL}};
const Segment SGI397104[] = {{&S78830[38], 4}, {NULL}};
const Segment SGI397105[] = {{&S78838[67], 4}, {NULL}};
const Segment SGI397106[] = {{&S78838[38], 9}, {NULL}};
const Segment SGI397107[] = {{&S78850[66], 4}, {NULL}};
const Segment SGI397108[] = {{&S78850[39], 7}, {NULL}};
const Segment SGI397109[] = {{&S78851[66], 4}, {NULL}};
const Segment SGI397110[] = {{&S78851[39], 7}, {NULL}};
const Segment SGI397111[] = {{&S78852[66], 4}, {NULL}};
const Segment SGI397112[] = {{&S78852[39], 7}, {NULL}};
const Segment SGI397113[] = {{&S78855[65], 4}, {NULL}};
const Segment SGI397114[] = {{&S78855[38], 7}, {NULL}};
const Segment SGI397115[] = {{&S78856[64], 4}, {NULL}};
const Segment SGI397116[] = {{&S78856[38], 6}, {NULL}};
const Segment SGI397117[] = {{&S78954[67], 4}, {NULL}};
const Segment SGI397118[] = {{&S78954[38], 9}, {NULL}};
const Segment SGI397119[] = {{&S79038[65], 4}, {NULL}};
const Segment SGI397120[] = {{&S79038[36], 9}, {NULL}};
const Segment SGI397121[] = {{&S79039[64], 4}, {NULL}};
const Segment SGI397122[] = {{&S79039[36], 8}, {NULL}};
const Segment SGI397123[] = {{&S79058[65], 4}, {NULL}};
const Segment SGI397124[] = {{&S79058[36], 9}, {NULL}};
const Segment SGI397125[] = {{&S79163[67], 4}, {NULL}};
const Segment SGI397126[] = {{&S79163[38], 9}, {NULL}};
const Segment SGI397127[] = {{&S100759[25], 13}, {NULL}};
const Segment SGI397128[] = {{&S79297[67], 4}, {NULL}};
const Segment SGI397129[] = {{&S79297[38], 9}, {NULL}};
const Segment SGI397130[] = {{&S79347[69], 4}, {NULL}};
const Segment SGI397131[] = {{&S79347[44], 5}, {NULL}};
const Segment SGI397132[] = {{&S79387[62], 4}, {NULL}};
const Segment SGI397133[] = {{&S79387[38], 4}, {NULL}};
const Segment SGI397134[] = {{&S79401[67], 4}, {NULL}};
const Segment SGI397135[] = {{&S79401[38], 9}, {NULL}};
const Segment SGI397136[] = {{&S79431[65], 4}, {NULL}};
const Segment SGI397137[] = {{&S79431[35], 10}, {NULL}};
const Segment SGI397138[] = {{&S79432[65], 4}, {NULL}};
const Segment SGI397139[] = {{&S79432[35], 10}, {NULL}};
const Segment SGI397140[] = {{&S79433[62], 4}, {NULL}};
const Segment SGI397141[] = {{&S79434[62], 4}, {NULL}};
const Segment SGI397142[] = {{&S79435[65], 4}, {NULL}};
const Segment SGI397143[] = {{&S79435[35], 10}, {NULL}};
const Segment SGI397144[] = {{&S79441[67], 4}, {NULL}};
const Segment SGI397145[] = {{&S79441[38], 9}, {NULL}};
const Segment SGI397146[] = {{&S79499[71], 4}, {NULL}};
const Segment SGI397147[] = {{&S79499[38], 13}, {NULL}};
const Segment SGI397148[] = {{&S79648[71], 4}, {NULL}};
const Segment SGI397149[] = {{&S79648[38], 13}, {NULL}};
const Segment SGI397150[] = {{&S79650[68], 4}, {NULL}};
const Segment SGI397151[] = {{&S79650[38], 10}, {NULL}};
const Segment SGI397152[] = {{&S79674[67], 4}, {NULL}};
const Segment SGI397153[] = {{&S79674[38], 9}, {NULL}};
const Segment SGI397154[] = {{&S79700[64], 4}, {NULL}};
const Segment SGI397155[] = {{&S79700[35], 9}, {NULL}};
const Segment SGI397156[] = {{&S79701[63], 4}, {NULL}};
const Segment SGI397157[] = {{&S79701[35], 8}, {NULL}};
const Segment SGI397158[] = {{&S79702[64], 4}, {NULL}};
const Segment SGI397159[] = {{&S79702[35], 9}, {NULL}};
const Segment SGI397160[] = {{&S79718[62], 4}, {NULL}};
const Segment SGI397161[] = {{&S79718[38], 4}, {NULL}};
const Segment SGI397162[] = {{&S79719[70], 4}, {NULL}};
const Segment SGI397163[] = {{&S79719[38], 12}, {NULL}};
const Segment SGI397164[] = {{&S79740[67], 4}, {NULL}};
const Segment SGI397165[] = {{&S79740[38], 9}, {NULL}};
const Segment SGI397166[] = {{&S79850[64], 4}, {NULL}};
const Segment SGI397167[] = {{&S79850[38], 6}, {NULL}};
const Segment SGI397168[] = {{&S79853[67], 4}, {NULL}};
const Segment SGI397169[] = {{&S79853[38], 9}, {NULL}};
const Segment SGI397170[] = {{&S79861[67], 4}, {NULL}};
const Segment SGI397171[] = {{&S79861[38], 9}, {NULL}};
const Segment SGI397172[] = {{&S79863[67], 4}, {NULL}};
const Segment SGI397173[] = {{&S79863[38], 9}, {NULL}};
const Segment SGI397174[] = {{&S79901[73], 4}, {NULL}};
const Segment SGI397175[] = {{&S79901[35], 18}, {NULL}};
const Segment SGI397176[] = {{&S79902[73], 4}, {NULL}};
const Segment SGI397177[] = {{&S79902[35], 18}, {NULL}};
const Segment SGI397178[] = {{&S79903[70], 4}, {NULL}};
const Segment SGI397179[] = {{&S79904[70], 4}, {NULL}};
const Segment SGI397180[] = {{&S79908[64], 4}, {NULL}};
const Segment SGI397181[] = {{&S79908[38], 6}, {NULL}};
const Segment SGI397182[] = {{&S79929[67], 4}, {NULL}};
const Segment SGI397183[] = {{&S79929[38], 9}, {NULL}};
const Segment SGI397184[] = {{&S79932[67], 4}, {NULL}};
const Segment SGI397185[] = {{&S79932[38], 9}, {NULL}};
const Segment SGI397186[] = {{&S101522[25], 13}, {NULL}};
const Segment SGI397187[] = {{&S80012[69], 4}, {NULL}};
const Segment SGI397188[] = {{&S80012[44], 5}, {NULL}};
const Segment SGI397189[] = {{&S80013[67], 4}, {NULL}};
const Segment SGI397190[] = {{&S80013[38], 9}, {NULL}};
const Segment SGI397191[] = {{&S80091[62], 4}, {NULL}};
const Segment SGI397192[] = {{&S80091[38], 4}, {NULL}};
const Segment SGI397193[] = {{&S80092[70], 4}, {NULL}};
const Segment SGI397194[] = {{&S80092[38], 12}, {NULL}};
const Segment SGI397195[] = {{&S80093[69], 4}, {NULL}};
const Segment SGI397196[] = {{&S80093[38], 11}, {NULL}};
const Segment SGI397197[] = {{&S80101[67], 4}, {NULL}};
const Segment SGI397198[] = {{&S80101[38], 9}, {NULL}};
const Segment SGI397199[] = {{&S80154[65], 4}, {NULL}};
const Segment SGI397200[] = {{&S80154[35], 10}, {NULL}};
const Segment SGI397201[] = {{&S80155[65], 4}, {NULL}};
const Segment SGI397202[] = {{&S80155[35], 10}, {NULL}};
const Segment SGI397203[] = {{&S80156[65], 4}, {NULL}};
const Segment SGI397204[] = {{&S80156[35], 10}, {NULL}};
const Segment SGI397205[] = {{&S80235[67], 4}, {NULL}};
const Segment SGI397206[] = {{&S80235[38], 9}, {NULL}};
const Segment SGI397207[] = {{&S80315[65], 4}, {NULL}};
const Segment SGI397208[] = {{&S80315[36], 9}, {NULL}};
const Segment SGI397209[] = {{&S80316[65], 4}, {NULL}};
const Segment SGI397210[] = {{&S80316[36], 9}, {NULL}};
const Segment SGI397211[] = {{&S80448[67], 4}, {NULL}};
const Segment SGI397212[] = {{&S80448[38], 9}, {NULL}};
const Segment SGI397213[] = {{&S80449[68], 4}, {NULL}};
const Segment SGI397214[] = {{&S80449[38], 10}, {NULL}};
const Segment SGI397215[] = {{&S80487[66], 4}, {NULL}};
const Segment SGI397216[] = {{&S80487[38], 8}, {NULL}};
const Segment SGI397217[] = {{&S80627[72], 4}, {NULL}};
const Segment SGI397218[] = {{&S80627[38], 14}, {NULL}};
const Segment SGI397219[] = {{&S40321[32], 14}, {NULL}};
const Segment SGI397220[] = {{&S40321[27], 4}, {NULL}};
const Segment SGI397221[] = {{&S40321[99], 9}, {NULL}};
const Segment SGI397222[] = {{&S80675[67], 4}, {NULL}};
const Segment SGI397223[] = {{&S80675[38], 9}, {NULL}};
const Segment SGI397224[] = {{&S80689[67], 4}, {NULL}};
const Segment SGI397225[] = {{&S80689[38], 9}, {NULL}};
const Segment SGI397226[] = {{&S80901[67], 4}, {NULL}};
const Segment SGI397227[] = {{&S80901[38], 9}, {NULL}};
const Segment SGI397228[] = {{&S80915[72], 4}, {NULL}};
const Segment SGI397229[] = {{&S80915[38], 14}, {NULL}};
const Segment SGI397230[] = {{&S80925[64], 4}, {NULL}};
const Segment SGI397231[] = {{&S80925[38], 6}, {NULL}};
const Segment SGI397232[] = {{&S80926[65], 4}, {NULL}};
const Segment SGI397233[] = {{&S80926[38], 7}, {NULL}};
const Segment SGI397234[] = {{&S80931[67], 4}, {NULL}};
const Segment SGI397235[] = {{&S80931[38], 9}, {NULL}};
const Segment SGI397236[] = {{&S81043[71], 4}, {NULL}};
const Segment SGI397237[] = {{&S81043[38], 13}, {NULL}};
const Segment SGI397238[] = {{&S81083[69], 4}, {NULL}};
const Segment SGI397239[] = {{&S81083[44], 5}, {NULL}};
const Segment SGI397240[] = {{&S81097[72], 4}, {NULL}};
const Segment SGI397241[] = {{&S81097[38], 14}, {NULL}};
const Segment SGI397242[] = {{&S81128[49], 3}, {NULL}};
const Segment SGI397243[] = {{&S81128[28], 10}, {NULL}};
const Segment SGI397244[] = {{&S81197[64], 4}, {NULL}};
const Segment SGI397245[] = {{&S81197[38], 6}, {NULL}};
const Segment SGI397246[] = {{&S81206[67], 4}, {NULL}};
const Segment SGI397247[] = {{&S81206[38], 9}, {NULL}};
const Segment SGI397248[] = {{&S81221[66], 4}, {NULL}};
const Segment SGI397249[] = {{&S81221[36], 10}, {NULL}};
const Segment SGI397250[] = {{&S81222[63], 4}, {NULL}};
const Segment SGI397251[] = {{&S81284[76], 4}, {NULL}};
const Segment SGI397252[] = {{&S81284[38], 18}, {NULL}};
const Segment SGI397253[] = {{&S81285[77], 4}, {NULL}};
const Segment SGI397254[] = {{&S81285[38], 19}, {NULL}};
const Segment SGI397255[] = {{&S81287[71], 4}, {NULL}};
const Segment SGI397256[] = {{&S81287[38], 13}, {NULL}};
const Segment SGI397257[] = {{&S81290[67], 4}, {NULL}};
const Segment SGI397258[] = {{&S81290[38], 9}, {NULL}};
const Segment SGI397259[] = {{&S81291[68], 4}, {NULL}};
const Segment SGI397260[] = {{&S81291[38], 10}, {NULL}};
const Segment SGI397261[] = {{&S81305[67], 4}, {NULL}};
const Segment SGI397262[] = {{&S81305[38], 9}, {NULL}};
const Segment SGI397263[] = {{&S81321[48], 3}, {NULL}};
const Segment SGI397264[] = {{&S81321[28], 9}, {NULL}};
const Segment SGI397265[] = {{&S40946[32], 9}, {NULL}};
const Segment SGI397266[] = {{&S40946[27], 4}, {NULL}};
const Segment SGI397267[] = {{&S40946[94], 9}, {NULL}};
const Segment SGI397268[] = {{&S81358[62], 4}, {NULL}};
const Segment SGI397269[] = {{&S81358[38], 4}, {NULL}};
const Segment SGI397270[] = {{&S81415[67], 4}, {NULL}};
const Segment SGI397271[] = {{&S81415[38], 9}, {NULL}};
const Segment SGI397272[] = {{&S81423[62], 4}, {NULL}};
const Segment SGI397273[] = {{&S81423[38], 4}, {NULL}};
const Segment SGI397274[] = {{&S81434[67], 4}, {NULL}};
const Segment SGI397275[] = {{&S81434[38], 9}, {NULL}};
const Segment SGI397276[] = {{&S81546[63], 4}, {NULL}};
const Segment SGI397277[] = {{&S81546[38], 5}, {NULL}};
const Segment SGI397278[] = {{&S81547[62], 4}, {NULL}};
const Segment SGI397279[] = {{&S81547[38], 4}, {NULL}};
const Segment SGI397280[] = {{&S81575[62], 4}, {NULL}};
const Segment SGI397281[] = {{&S81575[38], 4}, {NULL}};
const Segment SGI397282[] = {{&S81628[62], 4}, {NULL}};
const Segment SGI397283[] = {{&S81628[38], 4}, {NULL}};
const Segment SGI397284[] = {{&S81647[72], 4}, {NULL}};
const Segment SGI397285[] = {{&S81647[38], 14}, {NULL}};
const Segment SGI397286[] = {{&S81650[69], 4}, {NULL}};
const Segment SGI397287[] = {{&S81650[38], 11}, {NULL}};
const Segment SGI397288[] = {{&S81804[49], 3}, {NULL}};
const Segment SGI397289[] = {{&S81804[28], 10}, {NULL}};
const Segment SGI397290[] = {{&S81829[73], 4}, {NULL}};
const Segment SGI397291[] = {{&S81829[37], 16}, {NULL}};
const Segment SGI397292[] = {{&S41504[32], 9}, {NULL}};
const Segment SGI397293[] = {{&S41504[27], 4}, {NULL}};
const Segment SGI397294[] = {{&S41504[94], 9}, {NULL}};
const Segment SGI397295[] = {{&S41505[34], 9}, {NULL}};
const Segment SGI397296[] = {{&S41505[27], 6}, {NULL}};
const Segment SGI397297[] = {{&S41505[96], 9}, {NULL}};
const Segment SGI397298[] = {{&S81920[67], 4}, {NULL}};
const Segment SGI397299[] = {{&S81920[38], 9}, {NULL}};
const Segment SGI397300[] = {{&S81921[67], 4}, {NULL}};
const Segment SGI397301[] = {{&S81921[38], 9}, {NULL}};
const Segment SGI397302[] = {{&S81922[67], 4}, {NULL}};
const Segment SGI397303[] = {{&S81922[38], 9}, {NULL}};
const Segment SGI397304[] = {{&S81926[67], 4}, {NULL}};
const Segment SGI397305[] = {{&S81926[38], 9}, {NULL}};
const Segment SGI397306[] = {{&S81980[67], 4}, {NULL}};
const Segment SGI397307[] = {{&S81980[38], 9}, {NULL}};
const Segment SGI397308[] = {{&S82004[67], 4}, {NULL}};
const Segment SGI397309[] = {{&S82004[38], 9}, {NULL}};
const Segment SGI397310[] = {{&S82069[66], 4}, {NULL}};
const Segment SGI397311[] = {{&S82069[38], 8}, {NULL}};
const Segment SGI397312[] = {{&S82074[64], 4}, {NULL}};
const Segment SGI397313[] = {{&S82074[38], 6}, {NULL}};
const Segment SGI397314[] = {{&S103684[43], 13}, {NULL}};
const Segment SGI397315[] = {{&S82141[62], 4}, {NULL}};
const Segment SGI397316[] = {{&S82141[38], 4}, {NULL}};
const Segment SGI397317[] = {{&S82142[73], 4}, {NULL}};
const Segment SGI397318[] = {{&S82142[38], 15}, {NULL}};
const Segment SGI397319[] = {{&S82171[67], 4}, {NULL}};
const Segment SGI397320[] = {{&S82171[38], 9}, {NULL}};
const Segment SGI397321[] = {{&S82227[73], 4}, {NULL}};
const Segment SGI397322[] = {{&S82227[44], 9}, {NULL}};
const Segment SGI397323[] = {{&S82248[72], 4}, {NULL}};
const Segment SGI397324[] = {{&S82248[38], 14}, {NULL}};
const Segment SGI397325[] = {{&S82300[64], 4}, {NULL}};
const Segment SGI397326[] = {{&S82300[35], 9}, {NULL}};
const Segment SGI397327[] = {{&S82301[64], 4}, {NULL}};
const Segment SGI397328[] = {{&S82301[35], 9}, {NULL}};
const Segment SGI397329[] = {{&S82302[69], 4}, {NULL}};
const Segment SGI397330[] = {{&S82302[35], 14}, {NULL}};
const Segment SGI397331[] = {{&S82303[61], 4}, {NULL}};
const Segment SGI397332[] = {{&S82303[35], 6}, {NULL}};
const Segment SGI397333[] = {{&S82304[68], 4}, {NULL}};
const Segment SGI397334[] = {{&S82304[35], 13}, {NULL}};
const Segment SGI397335[] = {{&S82305[65], 4}, {NULL}};
const Segment SGI397336[] = {{&S82305[35], 10}, {NULL}};
const Segment SGI397337[] = {{&S41910[32], 9}, {NULL}};
const Segment SGI397338[] = {{&S41910[27], 4}, {NULL}};
const Segment SGI397339[] = {{&S82306[73], 4}, {NULL}};
const Segment SGI397340[] = {{&S82306[35], 18}, {NULL}};
const Segment SGI397341[] = {{&S41910[94], 9}, {NULL}};
const Segment SGI397342[] = {{&S82307[69], 4}, {NULL}};
const Segment SGI397343[] = {{&S82307[35], 14}, {NULL}};
const Segment SGI397344[] = {{&S82308[63], 4}, {NULL}};
const Segment SGI397345[] = {{&S82308[35], 8}, {NULL}};
const Segment SGI397346[] = {{&S82334[66], 4}, {NULL}};
const Segment SGI397347[] = {{&S82334[36], 10}, {NULL}};
const Segment SGI397348[] = {{&S82335[66], 4}, {NULL}};
const Segment SGI397349[] = {{&S82335[36], 10}, {NULL}};
const Segment SGI397350[] = {{&S82336[66], 4}, {NULL}};
const Segment SGI397351[] = {{&S82336[36], 10}, {NULL}};
const Segment SGI397352[] = {{&S82337[63], 4}, {NULL}};
const Segment SGI397353[] = {{&S82338[62], 4}, {NULL}};
const Segment SGI397354[] = {{&S82338[38], 4}, {NULL}};
const Segment SGI397355[] = {{&S82387[46], 3}, {NULL}};
const Segment SGI397356[] = {{&S82388[48], 3}, {NULL}};
const Segment SGI397357[] = {{&S82388[28], 9}, {NULL}};
const Segment SGI397358[] = {{&S82505[72], 4}, {NULL}};
const Segment SGI397359[] = {{&S82505[38], 14}, {NULL}};
const Segment SGI397360[] = {{&S82558[88], 4}, {NULL}};
const Segment SGI397361[] = {{&S82558[28], 10}, {NULL}};
const Segment SGI397362[] = {{&S82559[85], 4}, {NULL}};
const Segment SGI397363[] = {{&S82568[67], 4}, {NULL}};
const Segment SGI397364[] = {{&S82568[38], 9}, {NULL}};
const Segment SGI397365[] = {{&S82590[67], 4}, {NULL}};
const Segment SGI397366[] = {{&S82590[38], 9}, {NULL}};
const Segment SGI397367[] = {{&S42208[32], 9}, {NULL}};
const Segment SGI397368[] = {{&S42208[27], 4}, {NULL}};
const Segment SGI397369[] = {{&S42208[94], 9}, {NULL}};
const Segment SGI397370[] = {{&S82722[62], 4}, {NULL}};
const Segment SGI397371[] = {{&S82722[38], 4}, {NULL}};
const Segment SGI397372[] = {{&S82725[66], 4}, {NULL}};
const Segment SGI397373[] = {{&S82725[38], 8}, {NULL}};
const Segment SGI397374[] = {{&S82727[71], 4}, {NULL}};
const Segment SGI397375[] = {{&S82727[38], 13}, {NULL}};
const Segment SGI397376[] = {{&S82780[61], 4}, {NULL}};
const Segment SGI397377[] = {{&S82780[38], 3}, {NULL}};
const Segment SGI397378[] = {{&S82782[67], 4}, {NULL}};
const Segment SGI397379[] = {{&S82782[38], 9}, {NULL}};
const Segment SGI397380[] = {{&S82793[64], 4}, {NULL}};
const Segment SGI397381[] = {{&S82793[38], 6}, {NULL}};
const Segment SGI397382[] = {{&S82871[67], 4}, {NULL}};
const Segment SGI397383[] = {{&S82871[38], 9}, {NULL}};
const Segment SGI397384[] = {{&S82874[67], 4}, {NULL}};
const Segment SGI397385[] = {{&S82874[38], 9}, {NULL}};
const Segment SGI397386[] = {{&S82921[67], 4}, {NULL}};
const Segment SGI397387[] = {{&S82921[38], 9}, {NULL}};
const Segment SGI397388[] = {{&S82936[70], 4}, {NULL}};
const Segment SGI397389[] = {{&S82936[38], 12}, {NULL}};
const Segment SGI397390[] = {{&S83026[66], 4}, {NULL}};
const Segment SGI397391[] = {{&S83026[36], 10}, {NULL}};
const Segment SGI397392[] = {{&S83027[63], 4}, {NULL}};
const Segment SGI397393[] = {{&S83155[66], 4}, {NULL}};
const Segment SGI397394[] = {{&S83155[35], 11}, {NULL}};
const Segment SGI397395[] = {{&S83156[66], 4}, {NULL}};
const Segment SGI397396[] = {{&S83156[35], 11}, {NULL}};
const Segment SGI397397[] = {{&S83160[64], 4}, {NULL}};
const Segment SGI397398[] = {{&S83160[38], 6}, {NULL}};
const Segment SGI397399[] = {{&S83172[62], 4}, {NULL}};
const Segment SGI397400[] = {{&S83172[38], 4}, {NULL}};
const Segment SGI397401[] = {{&S83195[62], 4}, {NULL}};
const Segment SGI397402[] = {{&S83195[38], 4}, {NULL}};
const Segment SGI397403[] = {{&S83229[67], 4}, {NULL}};
const Segment SGI397404[] = {{&S83229[38], 9}, {NULL}};
const Segment SGI397405[] = {{&S83253[70], 4}, {NULL}};
const Segment SGI397406[] = {{&S83253[38], 12}, {NULL}};
const Segment SGI397407[] = {{&S83289[71], 4}, {NULL}};
const Segment SGI397408[] = {{&S83289[38], 13}, {NULL}};
const Segment SGI397409[] = {{&S83326[64], 4}, {NULL}};
const Segment SGI397410[] = {{&S83326[38], 6}, {NULL}};
const Segment SGI397411[] = {{&S83331[72], 4}, {NULL}};
const Segment SGI397412[] = {{&S83331[38], 14}, {NULL}};
const Segment SGI397413[] = {{&S83339[67], 4}, {NULL}};
const Segment SGI397414[] = {{&S83339[38], 9}, {NULL}};
const Segment SGI397415[] = {{&S83340[68], 4}, {NULL}};
const Segment SGI397416[] = {{&S83340[38], 10}, {NULL}};
const Segment SGI397417[] = {{&S83356[67], 4}, {NULL}};
const Segment SGI397418[] = {{&S83356[38], 9}, {NULL}};
const Segment SGI397419[] = {{&S83362[64], 4}, {NULL}};
const Segment SGI397420[] = {{&S83362[38], 6}, {NULL}};
const Segment SGI397421[] = {{&S83363[65], 4}, {NULL}};
const Segment SGI397422[] = {{&S83363[38], 7}, {NULL}};
const Segment SGI397423[] = {{&S83425[64], 4}, {NULL}};
const Segment SGI397424[] = {{&S83425[35], 9}, {NULL}};
const Segment SGI397425[] = {{&S83426[64], 4}, {NULL}};
const Segment SGI397426[] = {{&S83426[35], 9}, {NULL}};
const Segment SGI397427[] = {{&S83458[71], 4}, {NULL}};
const Segment SGI397428[] = {{&S83458[38], 13}, {NULL}};
const Segment SGI397429[] = {{&S83465[72], 4}, {NULL}};
const Segment SGI397430[] = {{&S83465[38], 14}, {NULL}};
const Segment SGI397431[] = {{&S83466[67], 4}, {NULL}};
const Segment SGI397432[] = {{&S83466[38], 9}, {NULL}};
const Segment SGI397433[] = {{&S83606[64], 4}, {NULL}};
const Segment SGI397434[] = {{&S83606[38], 6}, {NULL}};
const Segment SGI397435[] = {{&S83627[64], 4}, {NULL}};
const Segment SGI397436[] = {{&S83627[38], 6}, {NULL}};
const Segment SGI397437[] = {{&S83640[67], 4}, {NULL}};
const Segment SGI397438[] = {{&S83640[38], 9}, {NULL}};
const Segment SGI397439[] = {{&S83654[72], 4}, {NULL}};
const Segment SGI397440[] = {{&S83654[38], 14}, {NULL}};
const Segment SGI397441[] = {{&S83724[67], 4}, {NULL}};
const Segment SGI397442[] = {{&S83724[38], 9}, {NULL}};
const Segment SGI397443[] = {{&S105444[25], 13}, {NULL}};
const Segment SGI397444[] = {{&S83770[96], 4}, {NULL}};
const Segment SGI397445[] = {{&S83770[28], 18}, {NULL}};
const Segment SGI397446[] = {{&S83771[93], 4}, {NULL}};
const Segment SGI397447[] = {{&S83776[64], 4}, {NULL}};
const Segment SGI397448[] = {{&S83776[38], 6}, {NULL}};
const Segment SGI397449[] = {{&S83786[67], 4}, {NULL}};
const Segment SGI397450[] = {{&S83786[38], 9}, {NULL}};
const Segment SGI397451[] = {{&S83800[67], 4}, {NULL}};
const Segment SGI397452[] = {{&S83800[38], 9}, {NULL}};
const Segment SGI397453[] = {{&S83868[71], 4}, {NULL}};
const Segment SGI397454[] = {{&S83868[38], 13}, {NULL}};
const Segment SGI397455[] = {{&S83875[66], 4}, {NULL}};
const Segment SGI397456[] = {{&S83875[39], 7}, {NULL}};
const Segment SGI397457[] = {{&S83876[66], 4}, {NULL}};
const Segment SGI397458[] = {{&S83876[39], 7}, {NULL}};
const Segment SGI397459[] = {{&S83885[67], 4}, {NULL}};
const Segment SGI397460[] = {{&S83885[38], 9}, {NULL}};
const Segment SGI397461[] = {{&S83945[68], 4}, {NULL}};
const Segment SGI397462[] = {{&S83945[36], 12}, {NULL}};
const Segment SGI397463[] = {{&S83982[67], 4}, {NULL}};
const Segment SGI397464[] = {{&S83982[38], 9}, {NULL}};
const Segment SGI397465[] = {{&S84127[67], 4}, {NULL}};
const Segment SGI397466[] = {{&S84127[36], 11}, {NULL}};
const Segment SGI397467[] = {{&S84135[67], 4}, {NULL}};
const Segment SGI397468[] = {{&S84135[38], 9}, {NULL}};
const Segment SGI397469[] = {{&S84208[64], 4}, {NULL}};
const Segment SGI397470[] = {{&S84208[38], 6}, {NULL}};
const Segment SGI397471[] = {{&S84218[62], 4}, {NULL}};
const Segment SGI397472[] = {{&S84218[38], 4}, {NULL}};
const Segment SGI397473[] = {{&S84228[70], 4}, {NULL}};
const Segment SGI397474[] = {{&S84228[38], 12}, {NULL}};
const Segment SGI397475[] = {{&S84302[63], 4}, {NULL}};
const Segment SGI397476[] = {{&S84302[38], 5}, {NULL}};
const Segment SGI397477[] = {{&S84316[67], 4}, {NULL}};
const Segment SGI397478[] = {{&S84316[38], 9}, {NULL}};
const Segment SGI397479[] = {{&S84333[69], 4}, {NULL}};
const Segment SGI397480[] = {{&S84333[44], 5}, {NULL}};
const Segment SGI397481[] = {{&S84464[67], 4}, {NULL}};
const Segment SGI397482[] = {{&S84464[38], 9}, {NULL}};
const Segment SGI397483[] = {{&S84519[64], 4}, {NULL}};
const Segment SGI397484[] = {{&S84519[38], 6}, {NULL}};
const Segment SGI397485[] = {{&S84527[65], 4}, {NULL}};
const Segment SGI397486[] = {{&S84527[36], 9}, {NULL}};
const Segment SGI397487[] = {{&S84548[67], 4}, {NULL}};
const Segment SGI397488[] = {{&S84548[38], 9}, {NULL}};
const Segment SGI397489[] = {{&S84627[64], 4}, {NULL}};
const Segment SGI397490[] = {{&S84627[38], 6}, {NULL}};
const Segment SGI397491[] = {{&S84632[67], 4}, {NULL}};
const Segment SGI397492[] = {{&S84632[38], 9}, {NULL}};
const Segment SGI397493[] = {{&S84647[67], 4}, {NULL}};
const Segment SGI397494[] = {{&S84647[38], 9}, {NULL}};
const Segment SGI397495[] = {{&S84651[62], 4}, {NULL}};
const Segment SGI397496[] = {{&S84651[38], 4}, {NULL}};
const Segment SGI397497[] = {{&S84653[71], 4}, {NULL}};
const Segment SGI397498[] = {{&S84653[38], 13}, {NULL}};
const Segment SGI397499[] = {{&S84733[66], 4}, {NULL}};
const Segment SGI397500[] = {{&S84733[38], 8}, {NULL}};
const Segment SGI397501[] = {{&S84741[64], 4}, {NULL}};
const Segment SGI397502[] = {{&S84741[38], 6}, {NULL}};
const Segment SGI397503[] = {{&S84804[67], 4}, {NULL}};
const Segment SGI397504[] = {{&S84804[38], 9}, {NULL}};
const Segment SGI397505[] = {{&S84813[71], 4}, {NULL}};
const Segment SGI397506[] = {{&S84813[38], 13}, {NULL}};
const Segment SGI397507[] = {{&S84818[67], 4}, {NULL}};
const Segment SGI397508[] = {{&S84818[38], 9}, {NULL}};
const Segment SGI397509[] = {{&S84822[71], 4}, {NULL}};
const Segment SGI397510[] = {{&S84822[38], 13}, {NULL}};
const Segment SGI397511[] = {{&S84900[67], 4}, {NULL}};
const Segment SGI397512[] = {{&S84900[38], 9}, {NULL}};
const Segment SGI397513[] = {{&S84903[77], 4}, {NULL}};
const Segment SGI397514[] = {{&S84903[44], 13}, {NULL}};
const Segment SGI397515[] = {{&S85023[71], 4}, {NULL}};
const Segment SGI397516[] = {{&S85023[38], 13}, {NULL}};
const Segment SGI397517[] = {{&S85024[68], 4}, {NULL}};
const Segment SGI397518[] = {{&S85025[72], 4}, {NULL}};
const Segment SGI397519[] = {{&S85025[38], 14}, {NULL}};
const Segment SGI397520[] = {{&S85123[62], 4}, {NULL}};
const Segment SGI397521[] = {{&S85123[38], 4}, {NULL}};
const Segment SGI397522[] = {{&S85124[69], 4}, {NULL}};
const Segment SGI397523[] = {{&S85124[38], 11}, {NULL}};
const Segment SGI397524[] = {{&S85170[49], 3}, {NULL}};
const Segment SGI397525[] = {{&S85170[28], 10}, {NULL}};
const Segment SGI397526[] = {{&S85171[46], 3}, {NULL}};
const Segment SGI397527[] = {{&S85178[67], 4}, {NULL}};
const Segment SGI397528[] = {{&S85178[38], 9}, {NULL}};
const Segment SGI397529[] = {{&S85221[67], 4}, {NULL}};
const Segment SGI397530[] = {{&S85221[38], 9}, {NULL}};
const Segment SGI397531[] = {{&S85229[72], 4}, {NULL}};
const Segment SGI397532[] = {{&S85229[38], 14}, {NULL}};
const Segment SGI397533[] = {{&S85231[69], 4}, {NULL}};
const Segment SGI397534[] = {{&S85231[44], 5}, {NULL}};
const Segment SGI397535[] = {{&S85233[67], 4}, {NULL}};
const Segment SGI397536[] = {{&S85233[38], 9}, {NULL}};
const Segment SGI397537[] = {{&S85275[62], 4}, {NULL}};
const Segment SGI397538[] = {{&S85275[35], 7}, {NULL}};
const Segment SGI397539[] = {{&S85276[62], 4}, {NULL}};
const Segment SGI397540[] = {{&S85276[35], 7}, {NULL}};
const Segment SGI397541[] = {{&S85277[62], 4}, {NULL}};
const Segment SGI397542[] = {{&S85277[35], 7}, {NULL}};
const Segment SGI397543[] = {{&S85286[64], 4}, {NULL}};
const Segment SGI397544[] = {{&S85286[38], 6}, {NULL}};
const Segment SGI397545[] = {{&S85290[70], 4}, {NULL}};
const Segment SGI397546[] = {{&S85290[38], 12}, {NULL}};
const Segment SGI397547[] = {{&S85321[62], 4}, {NULL}};
const Segment SGI397548[] = {{&S85321[38], 4}, {NULL}};
const Segment SGI397549[] = {{&S85376[63], 4}, {NULL}};
const Segment SGI397550[] = {{&S85376[38], 5}, {NULL}};
const Segment SGI397551[] = {{&S85380[65], 4}, {NULL}};
const Segment SGI397552[] = {{&S85380[36], 9}, {NULL}};
const Segment SGI397553[] = {{&S85402[67], 4}, {NULL}};
const Segment SGI397554[] = {{&S85402[38], 9}, {NULL}};
const Segment SGI397555[] = {{&S85403[62], 4}, {NULL}};
const Segment SGI397556[] = {{&S85403[38], 4}, {NULL}};
const Segment SGI397557[] = {{&S85405[71], 4}, {NULL}};
const Segment SGI397558[] = {{&S85405[38], 13}, {NULL}};
const Segment SGI397559[] = {{&S85472[65], 4}, {NULL}};
const Segment SGI397560[] = {{&S85472[36], 9}, {NULL}};
const Segment SGI397561[] = {{&S85481[66], 4}, {NULL}};
const Segment SGI397562[] = {{&S85481[38], 8}, {NULL}};
const Segment SGI397563[] = {{&S85482[66], 4}, {NULL}};
const Segment SGI397564[] = {{&S85482[38], 8}, {NULL}};
const Segment SGI397565[] = {{&S85623[67], 4}, {NULL}};
const Segment SGI397566[] = {{&S85623[38], 9}, {NULL}};
const Segment SGI397567[] = {{&S85631[67], 4}, {NULL}};
const Segment SGI397568[] = {{&S85631[38], 9}, {NULL}};
const Segment SGI397569[] = {{&S45379[32], 10}, {NULL}};
const Segment SGI397570[] = {{&S45379[27], 4}, {NULL}};
const Segment SGI397571[] = {{&S45379[95], 9}, {NULL}};
const Segment SGI397572[] = {{&S85738[72], 4}, {NULL}};
const Segment SGI397573[] = {{&S85738[38], 14}, {NULL}};
const Segment SGI397574[] = {{&S85748[66], 4}, {NULL}};
const Segment SGI397575[] = {{&S85748[38], 8}, {NULL}};
const Segment SGI397576[] = {{&S85820[68], 4}, {NULL}};
const Segment SGI397577[] = {{&S85822[68], 4}, {NULL}};
const Segment SGI397578[] = {{&S85823[71], 4}, {NULL}};
const Segment SGI397579[] = {{&S85823[35], 16}, {NULL}};
const Segment SGI397580[] = {{&S85835[64], 4}, {NULL}};
const Segment SGI397581[] = {{&S85835[38], 6}, {NULL}};
const Segment SGI397582[] = {{&S85845[66], 4}, {NULL}};
const Segment SGI397583[] = {{&S85845[38], 8}, {NULL}};
const Segment SGI397584[] = {{&S85862[83], 4}, {NULL}};
const Segment SGI397585[] = {{&S85862[28], 5}, {NULL}};
const Segment SGI397586[] = {{&S85863[87], 4}, {NULL}};
const Segment SGI397587[] = {{&S85863[28], 9}, {NULL}};
const Segment SGI397588[] = {{&S85864[86], 4}, {NULL}};
const Segment SGI397589[] = {{&S85864[28], 8}, {NULL}};
const Segment SGI397590[] = {{&S85865[86], 4}, {NULL}};
const Segment SGI397591[] = {{&S85865[28], 8}, {NULL}};
const Segment SGI397592[] = {{&S45562[32], 14}, {NULL}};
const Segment SGI397593[] = {{&S45562[27], 4}, {NULL}};
const Segment SGI397594[] = {{&S45562[99], 9}, {NULL}};
const Segment SGI397595[] = {{&S85877[64], 4}, {NULL}};
const Segment SGI397596[] = {{&S85877[38], 6}, {NULL}};
const Segment SGI397597[] = {{&S85878[71], 4}, {NULL}};
const Segment SGI397598[] = {{&S85878[38], 13}, {NULL}};
const Segment SGI397599[] = {{&S85881[63], 4}, {NULL}};
const Segment SGI397600[] = {{&S85881[38], 5}, {NULL}};
const Segment SGI397601[] = {{&S85882[64], 4}, {NULL}};
const Segment SGI397602[] = {{&S85882[38], 6}, {NULL}};
const Segment SGI397603[] = {{&S85950[66], 4}, {NULL}};
const Segment SGI397604[] = {{&S85950[35], 11}, {NULL}};
const Segment SGI397605[] = {{&S85951[66], 4}, {NULL}};
const Segment SGI397606[] = {{&S85951[35], 11}, {NULL}};
const Segment SGI397607[] = {{&S85978[67], 4}, {NULL}};
const Segment SGI397608[] = {{&S85978[38], 9}, {NULL}};
const Segment SGI397609[] = {{&S85994[67], 4}, {NULL}};
const Segment SGI397610[] = {{&S85994[38], 9}, {NULL}};
const Segment SGI397611[] = {{&S86002[71], 4}, {NULL}};
const Segment SGI397612[] = {{&S86002[38], 13}, {NULL}};
const Segment SGI397613[] = {{&S86060[87], 4}, {NULL}};
const Segment SGI397614[] = {{&S86060[28], 9}, {NULL}};
const Segment SGI397615[] = {{&S45762[32], 9}, {NULL}};
const Segment SGI397616[] = {{&S45762[27], 4}, {NULL}};
const Segment SGI397617[] = {{&S45762[94], 9}, {NULL}};
const Segment SGI397618[] = {{&S45763[32], 5}, {NULL}};
const Segment SGI397619[] = {{&S45763[27], 4}, {NULL}};
const Segment SGI397620[] = {{&S45763[94], 9}, {NULL}};
const Segment SGI397621[] = {{&S45764[32], 14}, {NULL}};
const Segment SGI397622[] = {{&S45764[27], 4}, {NULL}};
const Segment SGI397623[] = {{&S45764[99], 9}, {NULL}};
const Segment SGI397624[] = {{&S45765[32], 36}, {NULL}};
const Segment SGI397625[] = {{&S45765[27], 4}, {NULL}};
const Segment SGI397626[] = {{&S45765[125], 9}, {NULL}};
const Segment SGI397627[] = {{&S45766[32], 8}, {NULL}};
const Segment SGI397628[] = {{&S45766[27], 4}, {NULL}};
const Segment SGI397629[] = {{&S45766[93], 9}, {NULL}};
const Segment SGI397630[] = {{&S86075[61], 4}, {NULL}};
const Segment SGI397631[] = {{&S86075[38], 3}, {NULL}};
const Segment SGI397632[] = {{&S86079[67], 4}, {NULL}};
const Segment SGI397633[] = {{&S86079[38], 9}, {NULL}};
const Segment SGI397634[] = {{&S86150[67], 4}, {NULL}};
const Segment SGI397635[] = {{&S86150[38], 9}, {NULL}};
const Segment SGI397636[] = {{&S86154[68], 4}, {NULL}};
const Segment SGI397637[] = {{&S86154[38], 10}, {NULL}};
const Segment SGI397638[] = {{&S86155[64], 4}, {NULL}};
const Segment SGI397639[] = {{&S86155[38], 6}, {NULL}};
const Segment SGI397640[] = {{&S86159[67], 4}, {NULL}};
const Segment SGI397641[] = {{&S86159[38], 9}, {NULL}};
const Segment SGI397642[] = {{&S86202[67], 4}, {NULL}};
const Segment SGI397643[] = {{&S86202[38], 9}, {NULL}};
const Segment SGI397644[] = {{&S86277[64], 4}, {NULL}};
const Segment SGI397645[] = {{&S86277[38], 6}, {NULL}};
const Segment SGI397646[] = {{&S86278[64], 4}, {NULL}};
const Segment SGI397647[] = {{&S86278[38], 6}, {NULL}};
const Segment SGI397648[] = {{&S86279[67], 4}, {NULL}};
const Segment SGI397649[] = {{&S86279[38], 9}, {NULL}};
const Segment SGI397650[] = {{&S86290[71], 4}, {NULL}};
const Segment SGI397651[] = {{&S86290[38], 13}, {NULL}};
const Segment SGI397652[] = {{&S86291[69], 4}, {NULL}};
const Segment SGI397653[] = {{&S86291[36], 13}, {NULL}};
const Segment SGI397654[] = {{&S86292[62], 4}, {NULL}};
const Segment SGI397655[] = {{&S86292[36], 6}, {NULL}};
const Segment SGI397656[] = {{&S86293[65], 4}, {NULL}};
const Segment SGI397657[] = {{&S86293[36], 9}, {NULL}};
const Segment SGI397658[] = {{&S86353[73], 4}, {NULL}};
const Segment SGI397659[] = {{&S86353[35], 18}, {NULL}};
const Segment SGI397660[] = {{&S86354[73], 4}, {NULL}};
const Segment SGI397661[] = {{&S86354[35], 18}, {NULL}};
const Segment SGI397662[] = {{&S86363[64], 4}, {NULL}};
const Segment SGI397663[] = {{&S86363[38], 6}, {NULL}};
const Segment SGI397664[] = {{&S86364[64], 4}, {NULL}};
const Segment SGI397665[] = {{&S86364[38], 6}, {NULL}};
const Segment SGI397666[] = {{&S86393[62], 4}, {NULL}};
const Segment SGI397667[] = {{&S86393[38], 4}, {NULL}};
const Segment SGI397668[] = {{&S86400[67], 4}, {NULL}};
const Segment SGI397669[] = {{&S86400[38], 9}, {NULL}};
const Segment SGI397670[] = {{&S86403[71], 4}, {NULL}};
const Segment SGI397671[] = {{&S86403[38], 13}, {NULL}};
const Segment SGI397672[] = {{&S86491[62], 4}, {NULL}};
const Segment SGI397673[] = {{&S86491[38], 4}, {NULL}};
const Segment SGI397674[] = {{&S86502[67], 4}, {NULL}};
const Segment SGI397675[] = {{&S86502[38], 9}, {NULL}};
const Segment SGI397676[] = {{&S86508[64], 4}, {NULL}};
const Segment SGI397677[] = {{&S86508[38], 6}, {NULL}};
const Segment SGI397678[] = {{&S86513[67], 4}, {NULL}};
const Segment SGI397679[] = {{&S86513[38], 9}, {NULL}};
const Segment SGI397680[] = {{&S86595[64], 4}, {NULL}};
const Segment SGI397681[] = {{&S86595[38], 6}, {NULL}};
const Segment SGI397682[] = {{&S86602[69], 4}, {NULL}};
const Segment SGI397683[] = {{&S86602[44], 5}, {NULL}};
const Segment SGI397684[] = {{&S86605[67], 4}, {NULL}};
const Segment SGI397685[] = {{&S86605[38], 9}, {NULL}};
const Segment SGI397686[] = {{&S86610[62], 4}, {NULL}};
const Segment SGI397687[] = {{&S86610[38], 4}, {NULL}};
const Segment SGI397688[] = {{&S86622[66], 4}, {NULL}};
const Segment SGI397689[] = {{&S86622[39], 7}, {NULL}};
const Segment SGI397690[] = {{&S86638[73], 4}, {NULL}};
const Segment SGI397691[] = {{&S86638[38], 15}, {NULL}};
const Segment SGI397692[] = {{&S86651[64], 4}, {NULL}};
const Segment SGI397693[] = {{&S86651[38], 6}, {NULL}};
const Segment SGI397694[] = {{&S86652[64], 4}, {NULL}};
const Segment SGI397695[] = {{&S86652[38], 6}, {NULL}};
const Segment SGI397696[] = {{&S86653[73], 4}, {NULL}};
const Segment SGI397697[] = {{&S86653[38], 15}, {NULL}};
const Segment SGI397698[] = {{&S86704[65], 4}, {NULL}};
const Segment SGI397699[] = {{&S86704[36], 9}, {NULL}};
const Segment SGI397700[] = {{&S86706[62], 4}, {NULL}};
const Segment SGI397701[] = {{&S86707[62], 4}, {NULL}};
const Segment SGI397702[] = {{&S86708[65], 4}, {NULL}};
const Segment SGI397703[] = {{&S86708[36], 9}, {NULL}};
const Segment SGI397704[] = {{&S86709[62], 4}, {NULL}};
const Segment SGI397705[] = {{&S86712[65], 4}, {NULL}};
const Segment SGI397706[] = {{&S86712[36], 9}, {NULL}};
const Segment SGI397707[] = {{&S86725[73], 4}, {NULL}};
const Segment SGI397708[] = {{&S86725[44], 9}, {NULL}};
const Segment SGI397709[] = {{&S86733[70], 4}, {NULL}};
const Segment SGI397710[] = {{&S86733[38], 12}, {NULL}};
const Segment SGI397711[] = {{&S86747[67], 4}, {NULL}};
const Segment SGI397712[] = {{&S86747[38], 9}, {NULL}};
const Segment SGI397713[] = {{&S86750[67], 4}, {NULL}};
const Segment SGI397714[] = {{&S86750[38], 9}, {NULL}};
const Segment SGI397715[] = {{&S86759[67], 4}, {NULL}};
const Segment SGI397716[] = {{&S86759[38], 9}, {NULL}};
const Segment SGI397717[] = {{&S86819[74], 4}, {NULL}};
const Segment SGI397718[] = {{&S86819[36], 18}, {NULL}};
const Segment SGI397719[] = {{&S86843[64], 4}, {NULL}};
const Segment SGI397720[] = {{&S86843[38], 6}, {NULL}};
const Segment SGI397721[] = {{&S86867[70], 4}, {NULL}};
const Segment SGI397722[] = {{&S86867[38], 12}, {NULL}};
const Segment SGI397723[] = {{&S86868[62], 4}, {NULL}};
const Segment SGI397724[] = {{&S86868[38], 4}, {NULL}};
const Segment SGI397725[] = {{&S86869[71], 4}, {NULL}};
const Segment SGI397726[] = {{&S86869[38], 13}, {NULL}};
const Segment SGI397727[] = {{&S86879[69], 4}, {NULL}};
const Segment SGI397728[] = {{&S86879[38], 11}, {NULL}};
const Segment SGI397729[] = {{&S86948[68], 4}, {NULL}};
const Segment SGI397730[] = {{&S86948[35], 13}, {NULL}};
const Segment SGI397731[] = {{&S86949[68], 4}, {NULL}};
const Segment SGI397732[] = {{&S86949[35], 13}, {NULL}};
const Segment SGI397733[] = {{&S86967[61], 4}, {NULL}};
const Segment SGI397734[] = {{&S86967[38], 3}, {NULL}};
const Segment SGI397735[] = {{&S86969[67], 4}, {NULL}};
const Segment SGI397736[] = {{&S86969[38], 9}, {NULL}};
const Segment SGI397737[] = {{&S86998[67], 4}, {NULL}};
const Segment SGI397738[] = {{&S86998[38], 9}, {NULL}};
const Segment SGI397739[] = {{&S87004[66], 4}, {NULL}};
const Segment SGI397740[] = {{&S87004[39], 7}, {NULL}};
const Segment SGI397741[] = {{&S87005[66], 4}, {NULL}};
const Segment SGI397742[] = {{&S87005[39], 7}, {NULL}};
const Segment SGI397743[] = {{&S87124[62], 4}, {NULL}};
const Segment SGI397744[] = {{&S87124[38], 4}, {NULL}};
const Segment SGI397745[] = {{&S87125[63], 4}, {NULL}};
const Segment SGI397746[] = {{&S87125[38], 5}, {NULL}};
const Segment SGI397747[] = {{&S87128[67], 4}, {NULL}};
const Segment SGI397748[] = {{&S87128[38], 9}, {NULL}};
const Segment SGI397749[] = {{&S87134[71], 4}, {NULL}};
const Segment SGI397750[] = {{&S87134[38], 13}, {NULL}};
const Segment SGI397751[] = {{&S87147[67], 4}, {NULL}};
const Segment SGI397752[] = {{&S87147[38], 9}, {NULL}};
const Segment SGI397753[] = {{&S87151[72], 4}, {NULL}};
const Segment SGI397754[] = {{&S87151[38], 14}, {NULL}};
const Segment SGI397755[] = {{&S87159[69], 4}, {NULL}};
const Segment SGI397756[] = {{&S87159[38], 11}, {NULL}};
const Segment SGI397757[] = {{&S87167[67], 4}, {NULL}};
const Segment SGI397758[] = {{&S87167[38], 9}, {NULL}};
const Segment SGI397759[] = {{&S87245[63], 4}, {NULL}};
const Segment SGI397760[] = {{&S87245[38], 5}, {NULL}};
const Segment SGI397761[] = {{&S87265[66], 4}, {NULL}};
const Segment SGI397762[] = {{&S87265[38], 8}, {NULL}};
const Segment SGI397763[] = {{&S87266[67], 4}, {NULL}};
const Segment SGI397764[] = {{&S87266[38], 9}, {NULL}};
const Segment SGI397765[] = {{&S87333[68], 4}, {NULL}};
const Segment SGI397766[] = {{&S87333[36], 12}, {NULL}};
const Segment SGI397767[] = {{&S87334[68], 4}, {NULL}};
const Segment SGI397768[] = {{&S87334[36], 12}, {NULL}};
const Segment SGI397769[] = {{&S87386[62], 4}, {NULL}};
const Segment SGI397770[] = {{&S87386[38], 4}, {NULL}};
const Segment SGI397771[] = {{&S87387[69], 4}, {NULL}};
const Segment SGI397772[] = {{&S87387[38], 11}, {NULL}};
const Segment SGI397773[] = {{&S47184[42], 10}, {NULL}};
const Segment SGI397774[] = {{&S47184[42], 22}, {NULL}};
const Segment SGI397775[] = {{&S47184[30], 4}, {NULL}};
const Segment SGI397776[] = {{&S47184[113], 3}, {NULL}};
const Segment SGI397777[] = {{&S47185[36], 10}, {NULL}};
const Segment SGI397778[] = {{&S47185[31], 4}, {NULL}};
const Segment SGI397779[] = {{&S47186[42], 10}, {NULL}};
const Segment SGI397780[] = {{&S47186[42], 22}, {NULL}};
const Segment SGI397781[] = {{&S47186[30], 4}, {NULL}};
const Segment SGI397782[] = {{&S47186[113], 3}, {NULL}};
const Segment SGI397783[] = {{&S47187[42], 10}, {NULL}};
const Segment SGI397784[] = {{&S47187[42], 22}, {NULL}};
const Segment SGI397785[] = {{&S47187[30], 4}, {NULL}};
const Segment SGI397786[] = {{&S47187[113], 3}, {NULL}};
const Segment SGI397787[] = {{&S47188[42], 10}, {NULL}};
const Segment SGI397788[] = {{&S47188[42], 22}, {NULL}};
const Segment SGI397789[] = {{&S47188[30], 4}, {NULL}};
const Segment SGI397790[] = {{&S47188[113], 3}, {NULL}};
const Segment SGI397791[] = {{&S87494[62], 4}, {NULL}};
const Segment SGI397792[] = {{&S87494[38], 4}, {NULL}};
const Segment SGI397793[] = {{&S87512[73], 4}, {NULL}};
const Segment SGI397794[] = {{&S87512[44], 9}, {NULL}};
const Segment SGI397795[] = {{&S87519[71], 4}, {NULL}};
const Segment SGI397796[] = {{&S87519[38], 13}, {NULL}};
const Segment SGI397797[] = {{&S87520[71], 4}, {NULL}};
const Segment SGI397798[] = {{&S87520[38], 13}, {NULL}};
const Segment SGI397799[] = {{&S87568[48], 3}, {NULL}};
const Segment SGI397800[] = {{&S87568[28], 9}, {NULL}};
const Segment SGI397801[] = {{&S87569[45], 3}, {NULL}};
const Segment SGI397802[] = {{&S87604[66], 4}, {NULL}};
const Segment SGI397803[] = {{&S87604[38], 8}, {NULL}};
const Segment SGI397804[] = {{&S87611[62], 4}, {NULL}};
const Segment SGI397805[] = {{&S87611[38], 4}, {NULL}};
const Segment SGI397806[] = {{&S87614[67], 4}, {NULL}};
const Segment SGI397807[] = {{&S87614[38], 9}, {NULL}};
const Segment SGI397808[] = {{&S87625[67], 4}, {NULL}};
const Segment SGI397809[] = {{&S87625[38], 9}, {NULL}};
const Segment SGI397810[] = {{&S87701[67], 4}, {NULL}};
const Segment SGI397811[] = {{&S87701[35], 12}, {NULL}};
const Segment SGI397812[] = {{&S87731[67], 4}, {NULL}};
const Segment SGI397813[] = {{&S87731[38], 9}, {NULL}};
const Segment SGI397814[] = {{&S87738[73], 4}, {NULL}};
const Segment SGI397815[] = {{&S87738[38], 15}, {NULL}};
const Segment SGI397816[] = {{&S87752[71], 4}, {NULL}};
const Segment SGI397817[] = {{&S87752[38], 13}, {NULL}};
const Segment SGI397818[] = {{&S87762[62], 4}, {NULL}};
const Segment SGI397819[] = {{&S87762[38], 4}, {NULL}};
const Segment SGI397820[] = {{&S87769[65], 4}, {NULL}};
const Segment SGI397821[] = {{&S87769[36], 9}, {NULL}};
const Segment SGI397822[] = {{&S87770[64], 4}, {NULL}};
const Segment SGI397823[] = {{&S87770[36], 8}, {NULL}};
const Segment SGI397824[] = {{&S87877[71], 4}, {NULL}};
const Segment SGI397825[] = {{&S87877[38], 13}, {NULL}};
const Segment SGI397826[] = {{&S87878[71], 4}, {NULL}};
const Segment SGI397827[] = {{&S87878[38], 13}, {NULL}};
const Segment SGI397828[] = {{&S87883[71], 4}, {NULL}};
const Segment SGI397829[] = {{&S87883[38], 13}, {NULL}};
const Segment SGI397830[] = {{&S87899[62], 4}, {NULL}};
const Segment SGI397831[] = {{&S87899[38], 4}, {NULL}};
const Segment SGI397832[] = {{&S87900[69], 4}, {NULL}};
const Segment SGI397833[] = {{&S87900[38], 11}, {NULL}};
const Segment SGI397834[] = {{&S47707[32], 10}, {NULL}};
const Segment SGI397835[] = {{&S47707[27], 4}, {NULL}};
const Segment SGI397836[] = {{&S47707[95], 9}, {NULL}};
const Segment SGI397837[] = {{&S47708[32], 10}, {NULL}};
const Segment SGI397838[] = {{&S47708[27], 4}, {NULL}};
const Segment SGI397839[] = {{&S47708[95], 9}, {NULL}};
const Segment SGI397840[] = {{&S47709[32], 10}, {NULL}};
const Segment SGI397841[] = {{&S47709[27], 4}, {NULL}};
const Segment SGI397842[] = {{&S47709[95], 9}, {NULL}};
const Segment SGI397843[] = {{&S47710[32], 18}, {NULL}};
const Segment SGI397844[] = {{&S47710[27], 4}, {NULL}};
const Segment SGI397845[] = {{&S47710[103], 9}, {NULL}};
const Segment SGI397846[] = {{&S87959[73], 4}, {NULL}};
const Segment SGI397847[] = {{&S87959[35], 18}, {NULL}};
const Segment SGI397848[] = {{&S87960[73], 4}, {NULL}};
const Segment SGI397849[] = {{&S87960[35], 18}, {NULL}};
const Segment SGI397850[] = {{&S87969[67], 4}, {NULL}};
const Segment SGI397851[] = {{&S87969[36], 11}, {NULL}};
const Segment SGI397852[] = {{&S87991[63], 4}, {NULL}};
const Segment SGI397853[] = {{&S87991[36], 7}, {NULL}};
const Segment SGI397854[] = {{&S87992[63], 4}, {NULL}};
const Segment SGI397855[] = {{&S87992[36], 7}, {NULL}};
const Segment SGI397856[] = {{&S87994[62], 4}, {NULL}};
const Segment SGI397857[] = {{&S87994[38], 4}, {NULL}};
const Segment SGI397858[] = {{&S87996[67], 4}, {NULL}};
const Segment SGI397859[] = {{&S87996[38], 9}, {NULL}};
const Segment SGI397860[] = {{&S88019[62], 4}, {NULL}};
const Segment SGI397861[] = {{&S88019[38], 4}, {NULL}};
const Segment SGI397862[] = {{&S88089[67], 4}, {NULL}};
const Segment SGI397863[] = {{&S88089[38], 9}, {NULL}};
const Segment SGI397864[] = {{&S88090[66], 4}, {NULL}};
const Segment SGI397865[] = {{&S88090[38], 8}, {NULL}};
const Segment SGI397866[] = {{&S88108[67], 4}, {NULL}};
const Segment SGI397867[] = {{&S88108[38], 9}, {NULL}};
const Segment SGI397868[] = {{&S88112[64], 4}, {NULL}};
const Segment SGI397869[] = {{&S88112[38], 6}, {NULL}};
const Segment SGI397870[] = {{&S88126[71], 4}, {NULL}};
const Segment SGI397871[] = {{&S88126[38], 13}, {NULL}};
const Segment SGI397872[] = {{&S88140[67], 4}, {NULL}};
const Segment SGI397873[] = {{&S88140[38], 9}, {NULL}};
const Segment SGI397874[] = {{&S88159[66], 4}, {NULL}};
const Segment SGI397875[] = {{&S88159[39], 7}, {NULL}};
const Segment SGI397876[] = {{&S88160[66], 4}, {NULL}};
const Segment SGI397877[] = {{&S88160[39], 7}, {NULL}};
const Segment SGI397878[] = {{&S101819[54], 3}, {NULL}};
const Segment SGI397879[] = {{&S88214[73], 4}, {NULL}};
const Segment SGI397880[] = {{&S88214[35], 18}, {NULL}};
const Segment SGI397881[] = {{&S88232[65], 4}, {NULL}};
const Segment SGI397882[] = {{&S88232[36], 9}, {NULL}};
const Segment SGI397883[] = {{&S88234[62], 4}, {NULL}};
const Segment SGI397884[] = {{&S88234[38], 4}, {NULL}};
const Segment SGI397885[] = {{&S88237[67], 4}, {NULL}};
const Segment SGI397886[] = {{&S88237[38], 9}, {NULL}};
const Segment SGI397887[] = {{&S88238[67], 4}, {NULL}};
const Segment SGI397888[] = {{&S88238[38], 9}, {NULL}};
const Segment SGI397889[] = {{&S88267[69], 4}, {NULL}};
const Segment SGI397890[] = {{&S88267[44], 5}, {NULL}};
const Segment SGI397891[] = {{&S88268[64], 4}, {NULL}};
const Segment SGI397892[] = {{&S88268[38], 6}, {NULL}};
const Segment SGI397893[] = {{&S88313[49], 3}, {NULL}};
const Segment SGI397894[] = {{&S88313[28], 10}, {NULL}};
const Segment SGI397895[] = {{&S88314[46], 3}, {NULL}};
const Segment SGI397896[] = {{&S88334[66], 4}, {NULL}};
const Segment SGI397897[] = {{&S88334[36], 10}, {NULL}};
const Segment SGI397898[] = {{&S88335[63], 4}, {NULL}};
const Segment SGI397899[] = {{&S88336[66], 4}, {NULL}};
const Segment SGI397900[] = {{&S88336[36], 10}, {NULL}};
const Segment SGI397901[] = {{&S88350[67], 4}, {NULL}};
const Segment SGI397902[] = {{&S88350[38], 9}, {NULL}};
const Segment SGI397903[] = {{&S88364[69], 4}, {NULL}};
const Segment SGI397904[] = {{&S88364[44], 5}, {NULL}};
const Segment SGI397905[] = {{&S88370[62], 4}, {NULL}};
const Segment SGI397906[] = {{&S88370[38], 4}, {NULL}};
const Segment SGI397907[] = {{&S88383[65], 4}, {NULL}};
const Segment SGI397908[] = {{&S88383[36], 9}, {NULL}};
const Segment SGI397909[] = {{&S88395[73], 4}, {NULL}};
const Segment SGI397910[] = {{&S88395[38], 15}, {NULL}};
const Segment SGI397911[] = {{&S88451[73], 4}, {NULL}};
const Segment SGI397912[] = {{&S88451[35], 18}, {NULL}};
const Segment SGI397913[] = {{&S88452[73], 4}, {NULL}};
const Segment SGI397914[] = {{&S88452[35], 18}, {NULL}};
const Segment SGI397915[] = {{&S88466[74], 4}, {NULL}};
const Segment SGI397916[] = {{&S88466[36], 18}, {NULL}};
const Segment SGI397917[] = {{&S88478[69], 4}, {NULL}};
const Segment SGI397918[] = {{&S88478[38], 11}, {NULL}};
const Segment SGI397919[] = {{&S88494[62], 4}, {NULL}};
const Segment SGI397920[] = {{&S88494[38], 4}, {NULL}};
const Segment SGI397921[] = {{&S88496[68], 4}, {NULL}};
const Segment SGI397922[] = {{&S88496[38], 10}, {NULL}};
const Segment SGI397923[] = {{&S88502[71], 4}, {NULL}};
const Segment SGI397924[] = {{&S88502[38], 13}, {NULL}};
const Segment SGI397925[] = {{&S88529[63], 4}, {NULL}};
const Segment SGI397926[] = {{&S88529[38], 5}, {NULL}};
const Segment SGI397927[] = {{&S88540[70], 4}, {NULL}};
const Segment SGI397928[] = {{&S88540[38], 12}, {NULL}};
const Segment SGI397929[] = {{&S88541[62], 4}, {NULL}};
const Segment SGI397930[] = {{&S88541[38], 4}, {NULL}};
const Segment SGI397931[] = {{&S88542[67], 4}, {NULL}};
const Segment SGI397932[] = {{&S88542[38], 9}, {NULL}};
const Segment SGI397933[] = {{&S88549[62], 4}, {NULL}};
const Segment SGI397934[] = {{&S88549[38], 4}, {NULL}};
const Segment SGI397935[] = {{&S88563[70], 4}, {NULL}};
const Segment SGI397936[] = {{&S88563[38], 12}, {NULL}};
const Segment SGI397937[] = {{&S88565[71], 4}, {NULL}};
const Segment SGI397938[] = {{&S88565[38], 13}, {NULL}};
const Segment SGI397939[] = {{&S88651[69], 4}, {NULL}};
const Segment SGI397940[] = {{&S88651[38], 11}, {NULL}};
const Segment SGI397941[] = {{&S88654[64], 4}, {NULL}};
const Segment SGI397942[] = {{&S88654[38], 6}, {NULL}};
const Segment SGI397943[] = {{&S88661[69], 4}, {NULL}};
const Segment SGI397944[] = {{&S88661[44], 5}, {NULL}};
const Segment SGI397945[] = {{&S88662[70], 4}, {NULL}};
const Segment SGI397946[] = {{&S88662[44], 6}, {NULL}};
const Segment SGI397947[] = {{&S88663[73], 4}, {NULL}};
const Segment SGI397948[] = {{&S88663[44], 9}, {NULL}};
const Segment SGI397949[] = {{&S88678[72], 4}, {NULL}};
const Segment SGI397950[] = {{&S88678[38], 14}, {NULL}};
const Segment SGI397951[] = {{&S88682[67], 4}, {NULL}};
const Segment SGI397952[] = {{&S88682[38], 9}, {NULL}};
const Segment SGI397953[] = {{&S88686[67], 4}, {NULL}};
const Segment SGI397954[] = {{&S88686[38], 9}, {NULL}};
const Segment SGI397955[] = {{&S88751[70], 4}, {NULL}};
const Segment SGI397956[] = {{&S88751[44], 6}, {NULL}};
const Segment SGI397957[] = {{&S88771[70], 4}, {NULL}};
const Segment SGI397958[] = {{&S88771[38], 12}, {NULL}};
const Segment SGI397959[] = {{&S88784[67], 4}, {NULL}};
const Segment SGI397960[] = {{&S88784[38], 9}, {NULL}};
const Segment SGI397961[] = {{&S88792[69], 4}, {NULL}};
const Segment SGI397962[] = {{&S88792[44], 5}, {NULL}};
const Segment SGI397963[] = {{&S88808[71], 4}, {NULL}};
const Segment SGI397964[] = {{&S88808[38], 13}, {NULL}};
const Segment SGI397965[] = {{&S88858[46], 3}, {NULL}};
const Segment SGI397966[] = {{&S88859[48], 3}, {NULL}};
const Segment SGI397967[] = {{&S88859[28], 9}, {NULL}};
const Segment SGI397968[] = {{&S88884[64], 4}, {NULL}};
const Segment SGI397969[] = {{&S88884[36], 8}, {NULL}};
const Segment SGI397970[] = {{&S88885[61], 4}, {NULL}};
const Segment SGI397971[] = {{&S88910[64], 4}, {NULL}};
const Segment SGI397972[] = {{&S88910[38], 6}, {NULL}};
const Segment SGI397973[] = {{&S88939[67], 4}, {NULL}};
const Segment SGI397974[] = {{&S88939[38], 9}, {NULL}};
const Segment SGI397975[] = {{&S88948[72], 4}, {NULL}};
const Segment SGI397976[] = {{&S88948[38], 14}, {NULL}};
const Segment SGI397977[] = {{&S88951[67], 4}, {NULL}};
const Segment SGI397978[] = {{&S88951[38], 9}, {NULL}};
const Segment SGI397979[] = {{&S88952[68], 4}, {NULL}};
const Segment SGI397980[] = {{&S88952[38], 10}, {NULL}};
const Segment SGI397981[] = {{&S88998[46], 3}, {NULL}};
const Segment SGI397982[] = {{&S88999[48], 3}, {NULL}};
const Segment SGI397983[] = {{&S88999[28], 9}, {NULL}};
const Segment SGI397984[] = {{&S89011[65], 4}, {NULL}};
const Segment SGI397985[] = {{&S89011[35], 10}, {NULL}};
const Segment SGI397986[] = {{&S89012[65], 4}, {NULL}};
const Segment SGI397987[] = {{&S89012[35], 10}, {NULL}};
const Segment SGI397988[] = {{&S89028[64], 4}, {NULL}};
const Segment SGI397989[] = {{&S89028[38], 6}, {NULL}};
const Segment SGI397990[] = {{&S89038[64], 4}, {NULL}};
const Segment SGI397991[] = {{&S89038[38], 6}, {NULL}};
const Segment SGI397992[] = {{&S89048[66], 4}, {NULL}};
const Segment SGI397993[] = {{&S89048[38], 8}, {NULL}};
const Segment SGI397994[] = {{&S89070[64], 4}, {NULL}};
const Segment SGI397995[] = {{&S89070[38], 6}, {NULL}};
const Segment SGI397996[] = {{&S89075[70], 4}, {NULL}};
const Segment SGI397997[] = {{&S89075[38], 12}, {NULL}};
const Segment SGI397998[] = {{&S89084[63], 4}, {NULL}};
const Segment SGI397999[] = {{&S89084[38], 5}, {NULL}};
const Segment SGI398000[] = {{&S89089[72], 4}, {NULL}};
const Segment SGI398001[] = {{&S89089[38], 14}, {NULL}};
const Segment SGI398002[] = {{&S89091[67], 4}, {NULL}};
const Segment SGI398003[] = {{&S89091[36], 11}, {NULL}};
const Segment SGI398004[] = {{&S89101[64], 4}, {NULL}};
const Segment SGI398005[] = {{&S89101[38], 6}, {NULL}};
const Segment SGI398006[] = {{&S89140[55], 3}, {NULL}};
const Segment SGI398007[] = {{&S89140[28], 16}, {NULL}};
const Segment SGI398008[] = {{&S89176[66], 4}, {NULL}};
const Segment SGI398009[] = {{&S89176[38], 8}, {NULL}};
const Segment SGI398010[] = {{&S89180[69], 4}, {NULL}};
const Segment SGI398011[] = {{&S89180[38], 11}, {NULL}};
const Segment SGI398012[] = {{&S89215[68], 4}, {NULL}};
const Segment SGI398013[] = {{&S89215[36], 12}, {NULL}};
const Segment SGI398014[] = {{&S89306[61], 4}, {NULL}};
const Segment SGI398015[] = {{&S89306[38], 3}, {NULL}};
const Segment SGI398016[] = {{&S89314[66], 4}, {NULL}};
const Segment SGI398017[] = {{&S89314[38], 8}, {NULL}};
const Segment SGI398018[] = {{&S89317[66], 4}, {NULL}};
const Segment SGI398019[] = {{&S89317[38], 8}, {NULL}};
const Segment SGI398020[] = {{&S89327[71], 4}, {NULL}};
const Segment SGI398021[] = {{&S89327[38], 13}, {NULL}};
const Segment SGI398022[] = {{&S89342[69], 4}, {NULL}};
const Segment SGI398023[] = {{&S89342[38], 11}, {NULL}};
const Segment SGI398024[] = {{&S89352[67], 4}, {NULL}};
const Segment SGI398025[] = {{&S89352[38], 9}, {NULL}};
const Segment SGI398026[] = {{&S89354[65], 4}, {NULL}};
const Segment SGI398027[] = {{&S89354[36], 9}, {NULL}};
const Segment SGI398028[] = {{&S89357[71], 4}, {NULL}};
const Segment SGI398029[] = {{&S89357[38], 13}, {NULL}};
const Segment SGI398030[] = {{&S89360[67], 4}, {NULL}};
const Segment SGI398031[] = {{&S89360[38], 9}, {NULL}};
const Segment SGI398032[] = {{&S89362[70], 4}, {NULL}};
const Segment SGI398033[] = {{&S89362[38], 12}, {NULL}};
const Segment SGI398034[] = {{&S89475[74], 4}, {NULL}};
const Segment SGI398035[] = {{&S89475[36], 18}, {NULL}};
const Segment SGI398036[] = {{&S89528[67], 4}, {NULL}};
const Segment SGI398037[] = {{&S89528[36], 11}, {NULL}};
const Segment SGI398038[] = {{&S89531[66], 4}, {NULL}};
const Segment SGI398039[] = {{&S89531[39], 7}, {NULL}};
const Segment SGI398040[] = {{&S89532[66], 4}, {NULL}};
const Segment SGI398041[] = {{&S89532[39], 7}, {NULL}};
const Segment SGI398042[] = {{&S89533[67], 4}, {NULL}};
const Segment SGI398043[] = {{&S89533[38], 9}, {NULL}};
const Segment SGI398044[] = {{&S89535[73], 4}, {NULL}};
const Segment SGI398045[] = {{&S89535[38], 15}, {NULL}};
const Segment SGI398046[] = {{&S89606[69], 4}, {NULL}};
const Segment SGI398047[] = {{&S89606[36], 13}, {NULL}};
const Segment SGI398048[] = {{&S89614[64], 4}, {NULL}};
const Segment SGI398049[] = {{&S89614[38], 6}, {NULL}};
const Segment SGI398050[] = {{&S89615[65], 4}, {NULL}};
const Segment SGI398051[] = {{&S89615[38], 7}, {NULL}};
const Segment SGI398052[] = {{&S89623[61], 4}, {NULL}};
const Segment SGI398053[] = {{&S89623[38], 3}, {NULL}};
const Segment SGI398054[] = {{&S89624[65], 4}, {NULL}};
const Segment SGI398055[] = {{&S89624[36], 9}, {NULL}};
const Segment SGI398056[] = {{&S89635[65], 4}, {NULL}};
const Segment SGI398057[] = {{&S89635[36], 9}, {NULL}};
const Segment SGI398058[] = {{&S89636[65], 4}, {NULL}};
const Segment SGI398059[] = {{&S89636[36], 9}, {NULL}};
const Segment SGI398060[] = {{&S89645[70], 4}, {NULL}};
const Segment SGI398061[] = {{&S89645[38], 12}, {NULL}};
const Segment SGI398062[] = {{&S89658[67], 4}, {NULL}};
const Segment SGI398063[] = {{&S89658[38], 9}, {NULL}};
const Segment SGI398064[] = {{&S89673[66], 4}, {NULL}};
const Segment SGI398065[] = {{&S89673[38], 8}, {NULL}};
const Segment SGI398066[] = {{&S89674[69], 4}, {NULL}};
const Segment SGI398067[] = {{&S89674[38], 11}, {NULL}};
const Segment SGI398068[] = {{&S89743[73], 4}, {NULL}};
const Segment SGI398069[] = {{&S89743[35], 18}, {NULL}};
const Segment SGI398070[] = {{&S89744[73], 4}, {NULL}};
const Segment SGI398071[] = {{&S89744[35], 18}, {NULL}};
const Segment SGI398072[] = {{&S89745[70], 4}, {NULL}};
const Segment SGI398073[] = {{&S89746[70], 4}, {NULL}};
const Segment SGI398074[] = {{&S89768[74], 4}, {NULL}};
const Segment SGI398075[] = {{&S89768[36], 18}, {NULL}};
const Segment SGI398076[] = {{&S49627[32], 10}, {NULL}};
const Segment SGI398077[] = {{&S49627[27], 4}, {NULL}};
const Segment SGI398078[] = {{&S49627[95], 9}, {NULL}};
const Segment SGI398079[] = {{&S49628[32], 11}, {NULL}};
const Segment SGI398080[] = {{&S49628[27], 4}, {NULL}};
const Segment SGI398081[] = {{&S49628[101], 9}, {NULL}};
const Segment SGI398082[] = {{&S89800[65], 4}, {NULL}};
const Segment SGI398083[] = {{&S89800[36], 9}, {NULL}};
const Segment SGI398084[] = {{&S89803[69], 4}, {NULL}};
const Segment SGI398085[] = {{&S89803[38], 11}, {NULL}};
const Segment SGI398086[] = {{&S89816[67], 4}, {NULL}};
const Segment SGI398087[] = {{&S89816[38], 9}, {NULL}};
const Segment SGI398088[] = {{&S89817[67], 4}, {NULL}};
const Segment SGI398089[] = {{&S89817[38], 9}, {NULL}};
const Segment SGI398090[] = {{&S89819[62], 4}, {NULL}};
const Segment SGI398091[] = {{&S89819[38], 4}, {NULL}};
const Segment SGI398092[] = {{&S89827[63], 4}, {NULL}};
const Segment SGI398093[] = {{&S89827[38], 5}, {NULL}};
const Segment SGI398094[] = {{&S89832[64], 4}, {NULL}};
const Segment SGI398095[] = {{&S89832[38], 6}, {NULL}};
const Segment SGI398096[] = {{&S89834[69], 4}, {NULL}};
const Segment SGI398097[] = {{&S89834[44], 5}, {NULL}};
const Segment SGI398098[] = {{&S89835[73], 4}, {NULL}};
const Segment SGI398099[] = {{&S89835[44], 9}, {NULL}};
const Segment SGI398100[] = {{&S89839[67], 4}, {NULL}};
const Segment SGI398101[] = {{&S89839[38], 9}, {NULL}};
const Segment SGI398102[] = {{&S89936[67], 4}, {NULL}};
const Segment SGI398103[] = {{&S89936[38], 9}, {NULL}};
const Segment SGI398104[] = {{&S89939[72], 4}, {NULL}};
const Segment SGI398105[] = {{&S89939[38], 14}, {NULL}};
const Segment SGI398106[] = {{&S89963[69], 4}, {NULL}};
const Segment SGI398107[] = {{&S89963[38], 11}, {NULL}};
const Segment SGI398108[] = {{&S89967[72], 4}, {NULL}};
const Segment SGI398109[] = {{&S89967[38], 14}, {NULL}};
const Segment SGI398110[] = {{&S89970[67], 4}, {NULL}};
const Segment SGI398111[] = {{&S89970[38], 9}, {NULL}};
const Segment SGI398112[] = {{&S89977[71], 4}, {NULL}};
const Segment SGI398113[] = {{&S89977[38], 13}, {NULL}};
const Segment SGI398114[] = {{&S89981[71], 4}, {NULL}};
const Segment SGI398115[] = {{&S89981[38], 13}, {NULL}};
const Segment SGI398116[] = {{&S89996[71], 4}, {NULL}};
const Segment SGI398117[] = {{&S89996[38], 13}, {NULL}};
const Segment SGI398118[] = {{&S89997[72], 4}, {NULL}};
const Segment SGI398119[] = {{&S89997[38], 14}, {NULL}};
const Segment SGI398120[] = {{&S89999[71], 4}, {NULL}};
const Segment SGI398121[] = {{&S89999[38], 13}, {NULL}};
const Segment SGI398122[] = {{&S90001[67], 4}, {NULL}};
const Segment SGI398123[] = {{&S90001[38], 9}, {NULL}};
const Segment SGI398124[] = {{&S90058[96], 4}, {NULL}};
const Segment SGI398125[] = {{&S90058[28], 18}, {NULL}};
const Segment SGI398126[] = {{&S90059[93], 4}, {NULL}};
const Segment SGI398127[] = {{&S90076[66], 4}, {NULL}};
const Segment SGI398128[] = {{&S90076[38], 8}, {NULL}};
const Segment SGI398129[] = {{&S90096[64], 4}, {NULL}};
const Segment SGI398130[] = {{&S90096[38], 6}, {NULL}};
const Segment SGI398131[] = {{&S90107[61], 4}, {NULL}};
const Segment SGI398132[] = {{&S90107[38], 3}, {NULL}};
const Segment SGI398133[] = {{&S90108[63], 4}, {NULL}};
const Segment SGI398134[] = {{&S90108[38], 5}, {NULL}};
const Segment SGI398135[] = {{&S90111[67], 4}, {NULL}};
const Segment SGI398136[] = {{&S90111[38], 9}, {NULL}};
const Segment SGI398137[] = {{&S90150[64], 4}, {NULL}};
const Segment SGI398138[] = {{&S90150[38], 6}, {NULL}};
const Segment SGI398139[] = {{&S90154[67], 4}, {NULL}};
const Segment SGI398140[] = {{&S90154[38], 9}, {NULL}};
const Segment SGI398141[] = {{&S90159[67], 4}, {NULL}};
const Segment SGI398142[] = {{&S90159[38], 9}, {NULL}};
const Segment SGI398143[] = {{&S90226[94], 4}, {NULL}};
const Segment SGI398144[] = {{&S90226[28], 16}, {NULL}};
const Segment SGI398145[] = {{&S90227[91], 4}, {NULL}};
const Segment SGI398146[] = {{&S90236[73], 4}, {NULL}};
const Segment SGI398147[] = {{&S90236[35], 18}, {NULL}};
const Segment SGI398148[] = {{&S90237[73], 4}, {NULL}};
const Segment SGI398149[] = {{&S90237[35], 18}, {NULL}};
const Segment SGI398150[] = {{&S90278[62], 4}, {NULL}};
const Segment SGI398151[] = {{&S90278[38], 4}, {NULL}};
const Segment SGI398152[] = {{&S90300[73], 4}, {NULL}};
const Segment SGI398153[] = {{&S90300[38], 15}, {NULL}};
const Segment SGI398154[] = {{&S90308[66], 4}, {NULL}};
const Segment SGI398155[] = {{&S90308[38], 8}, {NULL}};
const Segment SGI398156[] = {{&S90337[67], 4}, {NULL}};
const Segment SGI398157[] = {{&S90337[38], 9}, {NULL}};
const Segment SGI398158[] = {{&S90338[67], 4}, {NULL}};
const Segment SGI398159[] = {{&S90338[36], 11}, {NULL}};
const Segment SGI398160[] = {{&S90345[67], 4}, {NULL}};
const Segment SGI398161[] = {{&S90345[38], 9}, {NULL}};
const Segment SGI398162[] = {{&S90441[74], 4}, {NULL}};
const Segment SGI398163[] = {{&S90441[36], 18}, {NULL}};
const Segment SGI398164[] = {{&S90465[67], 4}, {NULL}};
const Segment SGI398165[] = {{&S90465[38], 9}, {NULL}};
const Segment SGI398166[] = {{&S90472[69], 4}, {NULL}};
const Segment SGI398167[] = {{&S90472[44], 5}, {NULL}};
const Segment SGI398168[] = {{&S90501[67], 4}, {NULL}};
const Segment SGI398169[] = {{&S90501[38], 9}, {NULL}};
const Segment SGI398170[] = {{&S90512[67], 4}, {NULL}};
const Segment SGI398171[] = {{&S90512[38], 9}, {NULL}};
const Segment SGI398172[] = {{&S90513[62], 4}, {NULL}};
const Segment SGI398173[] = {{&S90513[38], 4}, {NULL}};
const Segment SGI398174[] = {{&S90517[74], 4}, {NULL}};
const Segment SGI398175[] = {{&S90517[36], 18}, {NULL}};
const Segment SGI398176[] = {{&S90521[69], 4}, {NULL}};
const Segment SGI398177[] = {{&S90521[38], 11}, {NULL}};
const Segment SGI398178[] = {{&S90523[69], 4}, {NULL}};
const Segment SGI398179[] = {{&S90523[44], 5}, {NULL}};
const Segment SGI398180[] = {{&S90622[62], 4}, {NULL}};
const Segment SGI398181[] = {{&S90622[38], 4}, {NULL}};
const Segment SGI398182[] = {{&S90645[63], 4}, {NULL}};
const Segment SGI398183[] = {{&S90645[38], 5}, {NULL}};
const Segment SGI398184[] = {{&S90652[65], 4}, {NULL}};
const Segment SGI398185[] = {{&S90652[36], 9}, {NULL}};
const Segment SGI398186[] = {{&S90655[69], 4}, {NULL}};
const Segment SGI398187[] = {{&S90655[44], 5}, {NULL}};
const Segment SGI398188[] = {{&S90668[71], 4}, {NULL}};
const Segment SGI398189[] = {{&S90668[38], 13}, {NULL}};
const Segment SGI398190[] = {{&S90669[67], 4}, {NULL}};
const Segment SGI398191[] = {{&S90669[38], 9}, {NULL}};
const Segment SGI398192[] = {{&S90724[67], 4}, {NULL}};
const Segment SGI398193[] = {{&S90724[38], 9}, {NULL}};
const Segment SGI398194[] = {{&S90725[67], 4}, {NULL}};
const Segment SGI398195[] = {{&S90725[38], 9}, {NULL}};
const Segment SGI398196[] = {{&S90726[61], 4}, {NULL}};
const Segment SGI398197[] = {{&S90726[38], 3}, {NULL}};
const Segment SGI398198[] = {{&S90728[67], 4}, {NULL}};
const Segment SGI398199[] = {{&S90728[38], 9}, {NULL}};
const Segment SGI398200[] = {{&S90801[89], 4}, {NULL}};
const Segment SGI398201[] = {{&S90801[28], 11}, {NULL}};
const Segment SGI398202[] = {{&S90803[55], 3}, {NULL}};
const Segment SGI398203[] = {{&S90803[28], 16}, {NULL}};
const Segment SGI398204[] = {{&S90810[65], 4}, {NULL}};
const Segment SGI398205[] = {{&S90810[35], 10}, {NULL}};
const Segment SGI398206[] = {{&S90848[64], 4}, {NULL}};
const Segment SGI398207[] = {{&S90848[38], 6}, {NULL}};
const Segment SGI398208[] = {{&S90853[64], 4}, {NULL}};
const Segment SGI398209[] = {{&S90853[38], 6}, {NULL}};
const Segment SGI398210[] = {{&S90867[70], 4}, {NULL}};
const Segment SGI398211[] = {{&S90867[38], 12}, {NULL}};
const Segment SGI398212[] = {{&S90893[67], 4}, {NULL}};
const Segment SGI398213[] = {{&S90893[38], 9}, {NULL}};
const Segment SGI398214[] = {{&S90894[71], 4}, {NULL}};
const Segment SGI398215[] = {{&S90894[38], 13}, {NULL}};
const Segment SGI398216[] = {{&S90897[67], 4}, {NULL}};
const Segment SGI398217[] = {{&S90897[38], 9}, {NULL}};
const Segment SGI398218[] = {{&S90909[63], 4}, {NULL}};
const Segment SGI398219[] = {{&S90909[38], 5}, {NULL}};
const Segment SGI398220[] = {{&S90923[66], 4}, {NULL}};
const Segment SGI398221[] = {{&S90923[38], 8}, {NULL}};
const Segment SGI398222[] = {{&S90929[67], 4}, {NULL}};
const Segment SGI398223[] = {{&S90929[38], 9}, {NULL}};
const Segment SGI398224[] = {{&S90930[68], 4}, {NULL}};
const Segment SGI398225[] = {{&S90930[38], 10}, {NULL}};
const Segment SGI398226[] = {{&S90966[45], 3}, {NULL}};
const Segment SGI398227[] = {{&S90966[28], 6}, {NULL}};
const Segment SGI398228[] = {{&S90967[44], 3}, {NULL}};
const Segment SGI398229[] = {{&S90967[28], 5}, {NULL}};
const Segment SGI398230[] = {{&S91010[63], 4}, {NULL}};
const Segment SGI398231[] = {{&S91010[38], 5}, {NULL}};
const Segment SGI398232[] = {{&S91011[74], 4}, {NULL}};
const Segment SGI398233[] = {{&S91011[36], 18}, {NULL}};
const Segment SGI398234[] = {{&S91012[65], 4}, {NULL}};
const Segment SGI398235[] = {{&S91012[38], 7}, {NULL}};
const Segment SGI398236[] = {{&S91022[71], 4}, {NULL}};
const Segment SGI398237[] = {{&S91022[38], 13}, {NULL}};
const Segment SGI398238[] = {{&S91033[67], 4}, {NULL}};
const Segment SGI398239[] = {{&S91033[38], 9}, {NULL}};
const Segment SGI398240[] = {{&S91038[71], 4}, {NULL}};
const Segment SGI398241[] = {{&S91038[38], 13}, {NULL}};
const Segment SGI398242[] = {{&S91047[72], 4}, {NULL}};
const Segment SGI398243[] = {{&S91047[38], 14}, {NULL}};
const Segment SGI398244[] = {{&S91048[71], 4}, {NULL}};
const Segment SGI398245[] = {{&S91048[38], 13}, {NULL}};
const Segment SGI398246[] = {{&S91049[64], 4}, {NULL}};
const Segment SGI398247[] = {{&S91049[38], 6}, {NULL}};
const Segment SGI398248[] = {{&S91076[67], 4}, {NULL}};
const Segment SGI398249[] = {{&S91076[38], 9}, {NULL}};
const Segment SGI398250[] = {{&S91081[64], 4}, {NULL}};
const Segment SGI398251[] = {{&S91081[38], 6}, {NULL}};
const Segment SGI398252[] = {{&S91082[68], 4}, {NULL}};
const Segment SGI398253[] = {{&S91082[38], 10}, {NULL}};
const Segment SGI398254[] = {{&S91162[63], 4}, {NULL}};
const Segment SGI398255[] = {{&S91162[35], 8}, {NULL}};
const Segment SGI398256[] = {{&S91163[63], 4}, {NULL}};
const Segment SGI398257[] = {{&S91163[35], 8}, {NULL}};
const Segment SGI398258[] = {{&S91164[60], 4}, {NULL}};
const Segment SGI398259[] = {{&S51065[42], 10}, {NULL}};
const Segment SGI398260[] = {{&S51065[42], 22}, {NULL}};
const Segment SGI398261[] = {{&S51065[30], 4}, {NULL}};
const Segment SGI398262[] = {{&S51065[113], 3}, {NULL}};
const Segment SGI398263[] = {{&S51065[138], 8}, {NULL}};
const Segment SGI398264[] = {{&S51066[42], 10}, {NULL}};
const Segment SGI398265[] = {{&S51066[42], 22}, {NULL}};
const Segment SGI398266[] = {{&S51066[30], 4}, {NULL}};
const Segment SGI398267[] = {{&S51066[113], 3}, {NULL}};
const Segment SGI398268[] = {{&S51066[138], 8}, {NULL}};
const Segment SGI398269[] = {{&S51067[42], 10}, {NULL}};
const Segment SGI398270[] = {{&S51067[42], 22}, {NULL}};
const Segment SGI398271[] = {{&S51067[30], 4}, {NULL}};
const Segment SGI398272[] = {{&S51067[113], 3}, {NULL}};
const Segment SGI398273[] = {{&S51067[138], 8}, {NULL}};
const Segment SGI398274[] = {{&S51068[42], 10}, {NULL}};
const Segment SGI398275[] = {{&S51068[42], 22}, {NULL}};
const Segment SGI398276[] = {{&S51068[30], 4}, {NULL}};
const Segment SGI398277[] = {{&S51068[113], 3}, {NULL}};
const Segment SGI398278[] = {{&S51068[138], 8}, {NULL}};
const Segment SGI398279[] = {{&S91177[63], 4}, {NULL}};
const Segment SGI398280[] = {{&S91178[66], 4}, {NULL}};
const Segment SGI398281[] = {{&S51069[42], 10}, {NULL}};
const Segment SGI398282[] = {{&S91178[36], 10}, {NULL}};
const Segment SGI398283[] = {{&S51069[42], 22}, {NULL}};
const Segment SGI398284[] = {{&S51069[30], 4}, {NULL}};
const Segment SGI398285[] = {{&S51069[113], 3}, {NULL}};
const Segment SGI398286[] = {{&S51069[138], 8}, {NULL}};
const Segment SGI398287[] = {{&S91179[66], 4}, {NULL}};
const Segment SGI398288[] = {{&S91179[36], 10}, {NULL}};
const Segment SGI398289[] = {{&S91180[65], 4}, {NULL}};
const Segment SGI398290[] = {{&S51070[42], 10}, {NULL}};
const Segment SGI398291[] = {{&S91180[36], 9}, {NULL}};
const Segment SGI398292[] = {{&S51070[42], 22}, {NULL}};
const Segment SGI398293[] = {{&S51070[30], 4}, {NULL}};
const Segment SGI398294[] = {{&S51070[113], 3}, {NULL}};
const Segment SGI398295[] = {{&S51070[138], 8}, {NULL}};
const Segment SGI398296[] = {{&S51071[42], 10}, {NULL}};
const Segment SGI398297[] = {{&S51071[42], 22}, {NULL}};
const Segment SGI398298[] = {{&S51071[30], 4}, {NULL}};
const Segment SGI398299[] = {{&S51071[113], 3}, {NULL}};
const Segment SGI398300[] = {{&S51071[138], 8}, {NULL}};
const Segment SGI398301[] = {{&S51072[42], 10}, {NULL}};
const Segment SGI398302[] = {{&S51072[42], 22}, {NULL}};
const Segment SGI398303[] = {{&S51072[30], 4}, {NULL}};
const Segment SGI398304[] = {{&S51072[113], 3}, {NULL}};
const Segment SGI398305[] = {{&S51072[138], 8}, {NULL}};
const Segment SGI398306[] = {{&S51073[42], 10}, {NULL}};
const Segment SGI398307[] = {{&S51073[42], 22}, {NULL}};
const Segment SGI398308[] = {{&S51073[30], 4}, {NULL}};
const Segment SGI398309[] = {{&S51073[113], 3}, {NULL}};
const Segment SGI398310[] = {{&S51073[138], 8}, {NULL}};
const Segment SGI398311[] = {{&S51074[42], 10}, {NULL}};
const Segment SGI398312[] = {{&S51074[42], 22}, {NULL}};
const Segment SGI398313[] = {{&S51074[30], 4}, {NULL}};
const Segment SGI398314[] = {{&S51074[113], 3}, {NULL}};
const Segment SGI398315[] = {{&S51074[138], 8}, {NULL}};
const Segment SGI398316[] = {{&S51075[42], 10}, {NULL}};
const Segment SGI398317[] = {{&S51075[42], 22}, {NULL}};
const Segment SGI398318[] = {{&S51075[30], 4}, {NULL}};
const Segment SGI398319[] = {{&S51075[113], 3}, {NULL}};
const Segment SGI398320[] = {{&S51075[138], 8}, {NULL}};
const Segment SGI398321[] = {{&S51076[42], 10}, {NULL}};
const Segment SGI398322[] = {{&S51076[42], 22}, {NULL}};
const Segment SGI398323[] = {{&S51076[30], 4}, {NULL}};
const Segment SGI398324[] = {{&S51076[113], 3}, {NULL}};
const Segment SGI398325[] = {{&S51076[138], 8}, {NULL}};
const Segment SGI398326[] = {{&S91199[67], 4}, {NULL}};
const Segment SGI398327[] = {{&S91199[38], 9}, {NULL}};
const Segment SGI398328[] = {{&S91208[66], 4}, {NULL}};
const Segment SGI398329[] = {{&S91208[38], 8}, {NULL}};
const Segment SGI398330[] = {{&S91209[70], 4}, {NULL}};
const Segment SGI398331[] = {{&S91209[38], 12}, {NULL}};
const Segment SGI398332[] = {{&S91224[71], 4}, {NULL}};
const Segment SGI398333[] = {{&S91224[38], 13}, {NULL}};
const Segment SGI398334[] = {{&S91230[71], 4}, {NULL}};
const Segment SGI398335[] = {{&S91230[38], 13}, {NULL}};
const Segment SGI398336[] = {{&S91244[64], 4}, {NULL}};
const Segment SGI398337[] = {{&S91244[38], 6}, {NULL}};
const Segment SGI398338[] = {{&S91245[63], 4}, {NULL}};
const Segment SGI398339[] = {{&S91245[38], 5}, {NULL}};
const Segment SGI398340[] = {{&S91254[64], 4}, {NULL}};
const Segment SGI398341[] = {{&S91254[38], 6}, {NULL}};
const Segment SGI398342[] = {{&S91256[62], 4}, {NULL}};
const Segment SGI398343[] = {{&S91256[38], 4}, {NULL}};
const Segment SGI398344[] = {{&S91330[65], 4}, {NULL}};
const Segment SGI398345[] = {{&S91330[35], 10}, {NULL}};
const Segment SGI398346[] = {{&S91331[65], 4}, {NULL}};
const Segment SGI398347[] = {{&S91331[35], 10}, {NULL}};
const Segment SGI398348[] = {{&S91332[62], 4}, {NULL}};
const Segment SGI398349[] = {{&S91333[62], 4}, {NULL}};
const Segment SGI398350[] = {{&S91334[65], 4}, {NULL}};
const Segment SGI398351[] = {{&S91334[35], 10}, {NULL}};
const Segment SGI398352[] = {{&S91339[67], 4}, {NULL}};
const Segment SGI398353[] = {{&S91339[35], 12}, {NULL}};
const Segment SGI398354[] = {{&S91354[74], 4}, {NULL}};
const Segment SGI398355[] = {{&S91354[36], 18}, {NULL}};
const Segment SGI398356[] = {{&S91355[71], 4}, {NULL}};
const Segment SGI398357[] = {{&S91370[69], 4}, {NULL}};
const Segment SGI398358[] = {{&S91370[36], 13}, {NULL}};
const Segment SGI398359[] = {{&S91380[64], 4}, {NULL}};
const Segment SGI398360[] = {{&S91380[38], 6}, {NULL}};
const Segment SGI398361[] = {{&S91393[66], 4}, {NULL}};
const Segment SGI398362[] = {{&S91393[38], 8}, {NULL}};
const Segment SGI398363[] = {{&S91405[66], 4}, {NULL}};
const Segment SGI398364[] = {{&S91405[38], 8}, {NULL}};
const Segment SGI398365[] = {{&S91413[67], 4}, {NULL}};
const Segment SGI398366[] = {{&S91413[38], 9}, {NULL}};
const Segment SGI398367[] = {{&S91432[62], 4}, {NULL}};
const Segment SGI398368[] = {{&S91432[38], 4}, {NULL}};
const Segment SGI398369[] = {{&S91435[70], 4}, {NULL}};
const Segment SGI398370[] = {{&S91435[38], 12}, {NULL}};
const Segment SGI398371[] = {{&S91443[67], 4}, {NULL}};
const Segment SGI398372[] = {{&S91443[38], 9}, {NULL}};
const Segment SGI398373[] = {{&S91548[87], 4}, {NULL}};
const Segment SGI398374[] = {{&S91548[28], 9}, {NULL}};
const Segment SGI398375[] = {{&S91581[61], 4}, {NULL}};
const Segment SGI398376[] = {{&S91581[38], 3}, {NULL}};
const Segment SGI398377[] = {{&S91584[64], 4}, {NULL}};
const Segment SGI398378[] = {{&S91584[38], 6}, {NULL}};
const Segment SGI398379[] = {{&S91612[69], 4}, {NULL}};
const Segment SGI398380[] = {{&S91612[38], 11}, {NULL}};
const Segment SGI398381[] = {{&S91615[74], 4}, {NULL}};
const Segment SGI398382[] = {{&S91615[36], 18}, {NULL}};
const Segment SGI398383[] = {{&S91616[62], 4}, {NULL}};
const Segment SGI398384[] = {{&S91616[38], 4}, {NULL}};
const Segment SGI398385[] = {{&S91617[68], 4}, {NULL}};
const Segment SGI398386[] = {{&S91617[36], 12}, {NULL}};
const Segment SGI398387[] = {{&S91624[73], 4}, {NULL}};
const Segment SGI398388[] = {{&S91624[38], 15}, {NULL}};
const Segment SGI398389[] = {{&S91628[73], 4}, {NULL}};
const Segment SGI398390[] = {{&S91628[38], 15}, {NULL}};
const Segment SGI398391[] = {{&S91630[67], 4}, {NULL}};
const Segment SGI398392[] = {{&S91630[38], 9}, {NULL}};
const Segment SGI398393[] = {{&S91644[67], 4}, {NULL}};
const Segment SGI398394[] = {{&S91644[38], 9}, {NULL}};
const Segment SGI398395[] = {{&S91651[67], 4}, {NULL}};
const Segment SGI398396[] = {{&S91651[38], 9}, {NULL}};
const Segment SGI398397[] = {{&S91652[64], 4}, {NULL}};
const Segment SGI398398[] = {{&S91652[38], 6}, {NULL}};
const Segment SGI398399[] = {{&S91662[62], 4}, {NULL}};
const Segment SGI398400[] = {{&S91662[38], 4}, {NULL}};
const Segment SGI398401[] = {{&S91727[48], 3}, {NULL}};
const Segment SGI398402[] = {{&S91727[28], 9}, {NULL}};
const Segment SGI398403[] = {{&S91731[87], 4}, {NULL}};
const Segment SGI398404[] = {{&S91731[28], 9}, {NULL}};
const Segment SGI398405[] = {{&S91743[64], 4}, {NULL}};
const Segment SGI398406[] = {{&S91743[35], 9}, {NULL}};
const Segment SGI398407[] = {{&S91770[70], 4}, {NULL}};
const Segment SGI398408[] = {{&S91770[38], 12}, {NULL}};
const Segment SGI398409[] = {{&S91776[69], 4}, {NULL}};
const Segment SGI398410[] = {{&S91776[38], 11}, {NULL}};
const Segment SGI398411[] = {{&S91780[63], 4}, {NULL}};
const Segment SGI398412[] = {{&S91780[38], 5}, {NULL}};
const Segment SGI398413[] = {{&S91820[67], 4}, {NULL}};
const Segment SGI398414[] = {{&S91820[38], 9}, {NULL}};
const Segment SGI398415[] = {{&S91829[67], 4}, {NULL}};
const Segment SGI398416[] = {{&S91829[38], 9}, {NULL}};
const Segment SGI398417[] = {{&S91832[67], 4}, {NULL}};
const Segment SGI398418[] = {{&S91832[38], 9}, {NULL}};
const Segment SGI398419[] = {{&S91855[73], 4}, {NULL}};
const Segment SGI398420[] = {{&S91855[38], 15}, {NULL}};
const Segment SGI398421[] = {{&S91859[71], 4}, {NULL}};
const Segment SGI398422[] = {{&S91859[38], 13}, {NULL}};
const Segment SGI398423[] = {{&S91921[48], 3}, {NULL}};
const Segment SGI398424[] = {{&S91921[28], 9}, {NULL}};
const Segment SGI398425[] = {{&S91926[96], 4}, {NULL}};
const Segment SGI398426[] = {{&S91926[28], 18}, {NULL}};
const Segment SGI398427[] = {{&S91930[86], 4}, {NULL}};
const Segment SGI398428[] = {{&S91930[28], 8}, {NULL}};
const Segment SGI398429[] = {{&S91978[67], 4}, {NULL}};
const Segment SGI398430[] = {{&S91978[38], 9}, {NULL}};
const Segment SGI398431[] = {{&S91979[69], 4}, {NULL}};
const Segment SGI398432[] = {{&S91979[38], 11}, {NULL}};
const Segment SGI398433[] = {{&S92005[63], 4}, {NULL}};
const Segment SGI398434[] = {{&S92005[38], 5}, {NULL}};
const Segment SGI398435[] = {{&S92011[67], 4}, {NULL}};
const Segment SGI398436[] = {{&S92011[38], 9}, {NULL}};
const Segment SGI398437[] = {{&S92012[62], 4}, {NULL}};
const Segment SGI398438[] = {{&S92012[38], 4}, {NULL}};
const Segment SGI398439[] = {{&S92014[67], 4}, {NULL}};
const Segment SGI398440[] = {{&S92014[38], 9}, {NULL}};
const Segment SGI398441[] = {{&S92026[67], 4}, {NULL}};
const Segment SGI398442[] = {{&S92026[38], 9}, {NULL}};
const Segment SGI398443[] = {{&S92033[64], 4}, {NULL}};
const Segment SGI398444[] = {{&S92033[38], 6}, {NULL}};
const Segment SGI398445[] = {{&S92034[62], 4}, {NULL}};
const Segment SGI398446[] = {{&S92034[38], 4}, {NULL}};
const Segment SGI398447[] = {{&S92128[64], 4}, {NULL}};
const Segment SGI398448[] = {{&S92128[35], 9}, {NULL}};
const Segment SGI398449[] = {{&S92130[64], 4}, {NULL}};
const Segment SGI398450[] = {{&S92130[35], 9}, {NULL}};
const Segment SGI398451[] = {{&S92131[61], 4}, {NULL}};
const Segment SGI398452[] = {{&S92132[61], 4}, {NULL}};
const Segment SGI398453[] = {{&S92133[64], 4}, {NULL}};
const Segment SGI398454[] = {{&S92133[35], 9}, {NULL}};
const Segment SGI398455[] = {{&S92134[61], 4}, {NULL}};
const Segment SGI398456[] = {{&S92135[61], 4}, {NULL}};
const Segment SGI398457[] = {{&S92139[84], 4}, {NULL}};
const Segment SGI398458[] = {{&S92185[64], 4}, {NULL}};
const Segment SGI398459[] = {{&S92185[38], 6}, {NULL}};
const Segment SGI398460[] = {{&S92191[64], 4}, {NULL}};
const Segment SGI398461[] = {{&S92191[38], 6}, {NULL}};
const Segment SGI398462[] = {{&S92192[64], 4}, {NULL}};
const Segment SGI398463[] = {{&S92192[38], 6}, {NULL}};
const Segment SGI398464[] = {{&S92194[69], 4}, {NULL}};
const Segment SGI398465[] = {{&S92194[38], 11}, {NULL}};
const Segment SGI398466[] = {{&S92195[67], 4}, {NULL}};
const Segment SGI398467[] = {{&S92195[38], 9}, {NULL}};
const Segment SGI398468[] = {{&S92199[64], 4}, {NULL}};
const Segment SGI398469[] = {{&S92199[38], 6}, {NULL}};
const Segment SGI398470[] = {{&S92229[67], 4}, {NULL}};
const Segment SGI398471[] = {{&S92229[38], 9}, {NULL}};
const Segment SGI398472[] = {{&S92234[71], 4}, {NULL}};
const Segment SGI398473[] = {{&S92234[38], 13}, {NULL}};
const Segment SGI398474[] = {{&S92239[62], 4}, {NULL}};
const Segment SGI398475[] = {{&S92239[38], 4}, {NULL}};
const Segment SGI398476[] = {{&S92240[64], 4}, {NULL}};
const Segment SGI398477[] = {{&S92240[38], 6}, {NULL}};
const Segment SGI398478[] = {{&S92244[67], 4}, {NULL}};
const Segment SGI398479[] = {{&S92244[38], 9}, {NULL}};
const Segment SGI398480[] = {{&S92249[67], 4}, {NULL}};
const Segment SGI398481[] = {{&S92249[38], 9}, {NULL}};
const Segment SGI398482[] = {{&S92253[67], 4}, {NULL}};
const Segment SGI398483[] = {{&S92253[38], 9}, {NULL}};
const Segment SGI398484[] = {{&S92262[67], 4}, {NULL}};
const Segment SGI398485[] = {{&S92262[38], 9}, {NULL}};
const Segment SGI398486[] = {{&S92265[62], 4}, {NULL}};
const Segment SGI398487[] = {{&S92265[38], 4}, {NULL}};
const Segment SGI398488[] = {{&S92269[67], 4}, {NULL}};
const Segment SGI398489[] = {{&S92269[38], 9}, {NULL}};
const Segment SGI398490[] = {{&S92377[65], 4}, {NULL}};
const Segment SGI398491[] = {{&S92377[36], 9}, {NULL}};
const Segment SGI398492[] = {{&S92394[72], 4}, {NULL}};
const Segment SGI398493[] = {{&S92394[38], 14}, {NULL}};
const Segment SGI398494[] = {{&S92409[67], 4}, {NULL}};
const Segment SGI398495[] = {{&S92409[38], 9}, {NULL}};
const Segment SGI398496[] = {{&S92410[63], 4}, {NULL}};
const Segment SGI398497[] = {{&S92410[38], 5}, {NULL}};
const Segment SGI398498[] = {{&S92419[74], 4}, {NULL}};
const Segment SGI398499[] = {{&S92419[36], 18}, {NULL}};
const Segment SGI398500[] = {{&S92420[63], 4}, {NULL}};
const Segment SGI398501[] = {{&S92420[38], 5}, {NULL}};
const Segment SGI398502[] = {{&S92448[67], 4}, {NULL}};
const Segment SGI398503[] = {{&S92448[38], 9}, {NULL}};
const Segment SGI398504[] = {{&S92449[64], 4}, {NULL}};
const Segment SGI398505[] = {{&S92449[38], 6}, {NULL}};
const Segment SGI398506[] = {{&S92454[66], 4}, {NULL}};
const Segment SGI398507[] = {{&S92454[38], 8}, {NULL}};
const Segment SGI398508[] = {{&S52444[32], 18}, {NULL}};
const Segment SGI398509[] = {{&S52444[27], 4}, {NULL}};
const Segment SGI398510[] = {{&S52444[103], 9}, {NULL}};
const Segment SGI398511[] = {{&S52445[32], 10}, {NULL}};
const Segment SGI398512[] = {{&S52445[27], 4}, {NULL}};
const Segment SGI398513[] = {{&S52445[95], 9}, {NULL}};
const Segment SGI398514[] = {{&S52446[32], 10}, {NULL}};
const Segment SGI398515[] = {{&S52446[27], 4}, {NULL}};
const Segment SGI398516[] = {{&S52446[95], 9}, {NULL}};
const Segment SGI398517[] = {{&S52447[32], 11}, {NULL}};
const Segment SGI398518[] = {{&S52447[27], 4}, {NULL}};
const Segment SGI398519[] = {{&S52447[96], 9}, {NULL}};
const Segment SGI398520[] = {{&S52448[32], 10}, {NULL}};
const Segment SGI398521[] = {{&S52448[27], 4}, {NULL}};
const Segment SGI398522[] = {{&S52448[95], 9}, {NULL}};
const Segment SGI398523[] = {{&S92580[66], 4}, {NULL}};
const Segment SGI398524[] = {{&S92580[39], 7}, {NULL}};
const Segment SGI398525[] = {{&S92601[75], 4}, {NULL}};
const Segment SGI398526[] = {{&S92601[44], 11}, {NULL}};
const Segment SGI398527[] = {{&S92612[64], 4}, {NULL}};
const Segment SGI398528[] = {{&S92612[38], 6}, {NULL}};
const Segment SGI398529[] = {{&S92625[62], 4}, {NULL}};
const Segment SGI398530[] = {{&S92625[38], 4}, {NULL}};
const Segment SGI398531[] = {{&S92651[67], 4}, {NULL}};
const Segment SGI398532[] = {{&S92651[38], 9}, {NULL}};
const Segment SGI398533[] = {{&S92714[66], 4}, {NULL}};
const Segment SGI398534[] = {{&S92714[35], 11}, {NULL}};
const Segment SGI398535[] = {{&S92715[64], 4}, {NULL}};
const Segment SGI398536[] = {{&S92715[35], 9}, {NULL}};
const Segment SGI398537[] = {{&S92752[63], 4}, {NULL}};
const Segment SGI398538[] = {{&S92752[38], 5}, {NULL}};
const Segment SGI398539[] = {{&S92758[70], 4}, {NULL}};
const Segment SGI398540[] = {{&S92758[38], 12}, {NULL}};
const Segment SGI398541[] = {{&S92767[64], 4}, {NULL}};
const Segment SGI398542[] = {{&S92767[38], 6}, {NULL}};
const Segment SGI398543[] = {{&S92790[62], 4}, {NULL}};
const Segment SGI398544[] = {{&S92790[38], 4}, {NULL}};
const Segment SGI398545[] = {{&S52676[42], 9}, {NULL}};
const Segment SGI398546[] = {{&S52676[42], 25}, {NULL}};
const Segment SGI398547[] = {{&S52676[30], 4}, {NULL}};
const Segment SGI398548[] = {{&S52676[116], 3}, {NULL}};
const Segment SGI398549[] = {{&S52677[36], 9}, {NULL}};
const Segment SGI398550[] = {{&S52677[31], 4}, {NULL}};
const Segment SGI398551[] = {{&S52678[42], 9}, {NULL}};
const Segment SGI398552[] = {{&S52678[42], 21}, {NULL}};
const Segment SGI398553[] = {{&S52678[30], 4}, {NULL}};
const Segment SGI398554[] = {{&S52678[112], 3}, {NULL}};
const Segment SGI398555[] = {{&S52679[42], 9}, {NULL}};
const Segment SGI398556[] = {{&S52679[42], 21}, {NULL}};
const Segment SGI398557[] = {{&S52679[30], 4}, {NULL}};
const Segment SGI398558[] = {{&S52679[112], 3}, {NULL}};
const Segment SGI398559[] = {{&S52680[42], 5}, {NULL}};
const Segment SGI398560[] = {{&S52680[42], 17}, {NULL}};
const Segment SGI398561[] = {{&S52680[30], 4}, {NULL}};
const Segment SGI398562[] = {{&S52680[108], 3}, {NULL}};
const Segment SGI398563[] = {{&S92818[72], 4}, {NULL}};
const Segment SGI398564[] = {{&S92818[38], 14}, {NULL}};
const Segment SGI398565[] = {{&S52681[42], 9}, {NULL}};
const Segment SGI398566[] = {{&S52681[42], 40}, {NULL}};
const Segment SGI398567[] = {{&S52681[30], 4}, {NULL}};
const Segment SGI398568[] = {{&S52681[131], 3}, {NULL}};
const Segment SGI398569[] = {{&S52682[42], 9}, {NULL}};
const Segment SGI398570[] = {{&S52682[42], 24}, {NULL}};
const Segment SGI398571[] = {{&S52682[30], 4}, {NULL}};
const Segment SGI398572[] = {{&S52682[115], 3}, {NULL}};
const Segment SGI398573[] = {{&S52683[42], 11}, {NULL}};
const Segment SGI398574[] = {{&S52683[42], 23}, {NULL}};
const Segment SGI398575[] = {{&S52683[30], 4}, {NULL}};
const Segment SGI398576[] = {{&S52683[114], 3}, {NULL}};
const Segment SGI398577[] = {{&S52684[42], 13}, {NULL}};
const Segment SGI398578[] = {{&S52684[42], 25}, {NULL}};
const Segment SGI398579[] = {{&S52684[30], 4}, {NULL}};
const Segment SGI398580[] = {{&S52684[116], 3}, {NULL}};
const Segment SGI398581[] = {{&S52685[42], 14}, {NULL}};
const Segment SGI398582[] = {{&S52685[42], 26}, {NULL}};
const Segment SGI398583[] = {{&S52685[30], 4}, {NULL}};
const Segment SGI398584[] = {{&S52685[117], 3}, {NULL}};
const Segment SGI398585[] = {{&S52686[42], 8}, {NULL}};
const Segment SGI398586[] = {{&S52686[42], 20}, {NULL}};
const Segment SGI398587[] = {{&S52686[30], 4}, {NULL}};
const Segment SGI398588[] = {{&S52686[111], 3}, {NULL}};
const Segment SGI398589[] = {{&S52687[36], 14}, {NULL}};
const Segment SGI398590[] = {{&S52687[31], 4}, {NULL}};
const Segment SGI398591[] = {{&S52688[36], 8}, {NULL}};
const Segment SGI398592[] = {{&S52688[31], 4}, {NULL}};
const Segment SGI398593[] = {{&S92887[65], 4}, {NULL}};
const Segment SGI398594[] = {{&S92887[35], 10}, {NULL}};
const Segment SGI398595[] = {{&S92888[65], 4}, {NULL}};
const Segment SGI398596[] = {{&S92888[35], 10}, {NULL}};
const Segment SGI398597[] = {{&S92889[62], 4}, {NULL}};
const Segment SGI398598[] = {{&S92890[62], 4}, {NULL}};
const Segment SGI398599[] = {{&S92903[68], 4}, {NULL}};
const Segment SGI398600[] = {{&S92903[35], 13}, {NULL}};
const Segment SGI398601[] = {{&S92946[64], 4}, {NULL}};
const Segment SGI398602[] = {{&S92946[38], 6}, {NULL}};
const Segment SGI398603[] = {{&S92948[70], 4}, {NULL}};
const Segment SGI398604[] = {{&S92948[38], 12}, {NULL}};
const Segment SGI398605[] = {{&S92958[68], 4}, {NULL}};
const Segment SGI398606[] = {{&S92958[38], 10}, {NULL}};
const Segment SGI398607[] = {{&S92966[83], 4}, {NULL}};
const Segment SGI398608[] = {{&S92966[28], 5}, {NULL}};
const Segment SGI398609[] = {{&S92967[84], 4}, {NULL}};
const Segment SGI398610[] = {{&S92967[28], 6}, {NULL}};
const Segment SGI398611[] = {{&S92986[74], 4}, {NULL}};
const Segment SGI398612[] = {{&S92986[36], 18}, {NULL}};
const Segment SGI398613[] = {{&S92991[67], 4}, {NULL}};
const Segment SGI398614[] = {{&S92991[38], 9}, {NULL}};
const Segment SGI398615[] = {{&S93001[64], 4}, {NULL}};
const Segment SGI398616[] = {{&S93001[38], 6}, {NULL}};
const Segment SGI398617[] = {{&S93008[72], 4}, {NULL}};
const Segment SGI398618[] = {{&S93008[38], 14}, {NULL}};
const Segment SGI398619[] = {{&S93018[73], 4}, {NULL}};
const Segment SGI398620[] = {{&S93018[38], 15}, {NULL}};
const Segment SGI398621[] = {{&S93031[63], 4}, {NULL}};
const Segment SGI398622[] = {{&S93031[38], 5}, {NULL}};
const Segment SGI398623[] = {{&S93037[71], 4}, {NULL}};
const Segment SGI398624[] = {{&S93037[38], 13}, {NULL}};
const Segment SGI398625[] = {{&S93038[67], 4}, {NULL}};
const Segment SGI398626[] = {{&S93038[38], 9}, {NULL}};
const Segment SGI398627[] = {{&S93113[71], 4}, {NULL}};
const Segment SGI398628[] = {{&S93113[35], 16}, {NULL}};
const Segment SGI398629[] = {{&S93136[74], 4}, {NULL}};
const Segment SGI398630[] = {{&S93136[36], 18}, {NULL}};
const Segment SGI398631[] = {{&S93154[69], 4}, {NULL}};
const Segment SGI398632[] = {{&S93154[36], 13}, {NULL}};
const Segment SGI398633[] = {{&S93156[64], 4}, {NULL}};
const Segment SGI398634[] = {{&S93156[38], 6}, {NULL}};
const Segment SGI398635[] = {{&S93157[61], 4}, {NULL}};
const Segment SGI398636[] = {{&S93157[38], 3}, {NULL}};
const Segment SGI398637[] = {{&S93165[68], 4}, {NULL}};
const Segment SGI398638[] = {{&S93165[38], 10}, {NULL}};
const Segment SGI398639[] = {{&S93197[62], 4}, {NULL}};
const Segment SGI398640[] = {{&S93197[38], 4}, {NULL}};
const Segment SGI398641[] = {{&S93200[71], 4}, {NULL}};
const Segment SGI398642[] = {{&S93200[38], 13}, {NULL}};
const Segment SGI398643[] = {{&S93229[67], 4}, {NULL}};
const Segment SGI398644[] = {{&S93229[38], 9}, {NULL}};
const Segment SGI398645[] = {{&S93230[67], 4}, {NULL}};
const Segment SGI398646[] = {{&S93230[38], 9}, {NULL}};
const Segment SGI398647[] = {{&S93236[73], 4}, {NULL}};
const Segment SGI398648[] = {{&S93236[38], 15}, {NULL}};
const Segment SGI398649[] = {{&S93248[64], 4}, {NULL}};
const Segment SGI398650[] = {{&S93248[38], 6}, {NULL}};
const Segment SGI398651[] = {{&S93249[65], 4}, {NULL}};
const Segment SGI398652[] = {{&S93249[38], 7}, {NULL}};
const Segment SGI398653[] = {{&S93317[48], 3}, {NULL}};
const Segment SGI398654[] = {{&S93317[28], 9}, {NULL}};
const Segment SGI398655[] = {{&S93342[50], 3}, {NULL}};
const Segment SGI398656[] = {{&S93342[28], 11}, {NULL}};
const Segment SGI398657[] = {{&S93397[64], 4}, {NULL}};
const Segment SGI398658[] = {{&S93397[38], 6}, {NULL}};
const Segment SGI398659[] = {{&S93401[62], 4}, {NULL}};
const Segment SGI398660[] = {{&S93401[38], 4}, {NULL}};
const Segment SGI398661[] = {{&S93406[96], 4}, {NULL}};
const Segment SGI398662[] = {{&S93406[28], 18}, {NULL}};
const Segment SGI398663[] = {{&S93411[67], 4}, {NULL}};
const Segment SGI398664[] = {{&S93411[38], 9}, {NULL}};
const Segment SGI398665[] = {{&S93478[62], 4}, {NULL}};
const Segment SGI398666[] = {{&S93478[38], 4}, {NULL}};
const Segment SGI398667[] = {{&S93487[71], 4}, {NULL}};
const Segment SGI398668[] = {{&S93487[38], 13}, {NULL}};
const Segment SGI398669[] = {{&S93491[62], 4}, {NULL}};
const Segment SGI398670[] = {{&S93491[38], 4}, {NULL}};
const Segment SGI398671[] = {{&S93494[71], 4}, {NULL}};
const Segment SGI398672[] = {{&S93494[38], 13}, {NULL}};
const Segment SGI398673[] = {{&S93558[48], 3}, {NULL}};
const Segment SGI398674[] = {{&S93558[28], 9}, {NULL}};
const Segment SGI398675[] = {{&S53547[32], 10}, {NULL}};
const Segment SGI398676[] = {{&S53547[27], 4}, {NULL}};
const Segment SGI398677[] = {{&S53547[95], 9}, {NULL}};
const Segment SGI398678[] = {{&S93576[48], 3}, {NULL}};
const Segment SGI398679[] = {{&S93576[28], 9}, {NULL}};
const Segment SGI398680[] = {{&S93602[73], 4}, {NULL}};
const Segment SGI398681[] = {{&S93602[35], 18}, {NULL}};
const Segment SGI398682[] = {{&S93628[68], 4}, {NULL}};
const Segment SGI398683[] = {{&S93628[38], 10}, {NULL}};
const Segment SGI398684[] = {{&S93644[62], 4}, {NULL}};
const Segment SGI398685[] = {{&S93644[36], 6}, {NULL}};
const Segment SGI398686[] = {{&S93645[67], 4}, {NULL}};
const Segment SGI398687[] = {{&S93645[38], 9}, {NULL}};
const Segment SGI398688[] = {{&S93646[77], 4}, {NULL}};
const Segment SGI398689[] = {{&S93646[38], 19}, {NULL}};
const Segment SGI398690[] = {{&S93655[62], 4}, {NULL}};
const Segment SGI398691[] = {{&S93655[38], 4}, {NULL}};
const Segment SGI398692[] = {{&S93676[68], 4}, {NULL}};
const Segment SGI398693[] = {{&S93676[38], 10}, {NULL}};
const Segment SGI398694[] = {{&S93679[67], 4}, {NULL}};
const Segment SGI398695[] = {{&S93679[38], 9}, {NULL}};
const Segment SGI398696[] = {{&S93681[71], 4}, {NULL}};
const Segment SGI398697[] = {{&S93681[38], 13}, {NULL}};
const Segment SGI398698[] = {{&S93695[62], 4}, {NULL}};
const Segment SGI398699[] = {{&S93695[38], 4}, {NULL}};
const Segment SGI398700[] = {{&S93697[73], 4}, {NULL}};
const Segment SGI398701[] = {{&S93697[38], 15}, {NULL}};
const Segment SGI398702[] = {{&S93704[63], 4}, {NULL}};
const Segment SGI398703[] = {{&S93704[38], 5}, {NULL}};
const Segment SGI398704[] = {{&S93705[62], 4}, {NULL}};
const Segment SGI398705[] = {{&S93705[38], 4}, {NULL}};
const Segment SGI398706[] = {{&S93820[49], 3}, {NULL}};
const Segment SGI398707[] = {{&S93820[28], 10}, {NULL}};
const Segment SGI398708[] = {{&S93833[74], 4}, {NULL}};
const Segment SGI398709[] = {{&S93833[36], 18}, {NULL}};
const Segment SGI398710[] = {{&S93834[71], 4}, {NULL}};
const Segment SGI398711[] = {{&S93880[64], 4}, {NULL}};
const Segment SGI398712[] = {{&S93880[38], 6}, {NULL}};
const Segment SGI398713[] = {{&S93883[84], 4}, {NULL}};
const Segment SGI398714[] = {{&S93883[38], 26}, {NULL}};
const Segment SGI398715[] = {{&S53822[32], 14}, {NULL}};
const Segment SGI398716[] = {{&S53822[27], 4}, {NULL}};
const Segment SGI398717[] = {{&S53822[99], 9}, {NULL}};
const Segment SGI398718[] = {{&S93898[68], 4}, {NULL}};
const Segment SGI398719[] = {{&S93898[38], 10}, {NULL}};
const Segment SGI398720[] = {{&S93917[69], 4}, {NULL}};
const Segment SGI398721[] = {{&S93917[38], 11}, {NULL}};
const Segment SGI398722[] = {{&S93923[63], 4}, {NULL}};
const Segment SGI398723[] = {{&S93923[38], 5}, {NULL}};
const Segment SGI398724[] = {{&S93941[63], 4}, {NULL}};
const Segment SGI398725[] = {{&S93941[38], 5}, {NULL}};
const Segment SGI398726[] = {{&S93945[68], 4}, {NULL}};
const Segment SGI398727[] = {{&S93945[36], 12}, {NULL}};
const Segment SGI398728[] = {{&S93946[73], 4}, {NULL}};
const Segment SGI398729[] = {{&S93946[38], 15}, {NULL}};
const Segment SGI398730[] = {{&S93949[73], 4}, {NULL}};
const Segment SGI398731[] = {{&S93949[44], 9}, {NULL}};
const Segment SGI398732[] = {{&S94039[57], 3}, {NULL}};
const Segment SGI398733[] = {{&S94039[28], 18}, {NULL}};
const Segment SGI398734[] = {{&S94047[64], 4}, {NULL}};
const Segment SGI398735[] = {{&S94047[35], 9}, {NULL}};
const Segment SGI398736[] = {{&S94048[64], 4}, {NULL}};
const Segment SGI398737[] = {{&S94048[35], 9}, {NULL}};
const Segment SGI398738[] = {{&S94100[66], 4}, {NULL}};
const Segment SGI398739[] = {{&S94100[38], 8}, {NULL}};
const Segment SGI398740[] = {{&S94129[69], 4}, {NULL}};
const Segment SGI398741[] = {{&S94129[38], 11}, {NULL}};
const Segment SGI398742[] = {{&S94130[72], 4}, {NULL}};
const Segment SGI398743[] = {{&S94130[38], 14}, {NULL}};
const Segment SGI398744[] = {{&S94134[64], 4}, {NULL}};
const Segment SGI398745[] = {{&S94134[38], 6}, {NULL}};
const Segment SGI398746[] = {{&S94150[64], 4}, {NULL}};
const Segment SGI398747[] = {{&S94150[38], 6}, {NULL}};
const Segment SGI398748[] = {{&S94163[63], 4}, {NULL}};
const Segment SGI398749[] = {{&S94163[38], 5}, {NULL}};
const Segment SGI398750[] = {{&S94166[62], 4}, {NULL}};
const Segment SGI398751[] = {{&S94166[38], 4}, {NULL}};
const Segment SGI398752[] = {{&S94170[62], 4}, {NULL}};
const Segment SGI398753[] = {{&S94170[38], 4}, {NULL}};
const Segment SGI398754[] = {{&S94175[67], 4}, {NULL}};
const Segment SGI398755[] = {{&S94175[38], 9}, {NULL}};
const Segment SGI398756[] = {{&S94247[64], 4}, {NULL}};
const Segment SGI398757[] = {{&S94247[35], 9}, {NULL}};
const Segment SGI398758[] = {{&S94284[74], 4}, {NULL}};
const Segment SGI398759[] = {{&S94284[36], 18}, {NULL}};
const Segment SGI398760[] = {{&S94331[69], 4}, {NULL}};
const Segment SGI398761[] = {{&S94331[38], 11}, {NULL}};
const Segment SGI398762[] = {{&S94337[71], 4}, {NULL}};
const Segment SGI398763[] = {{&S94337[38], 13}, {NULL}};
const Segment SGI398764[] = {{&S94338[62], 4}, {NULL}};
const Segment SGI398765[] = {{&S94338[38], 4}, {NULL}};
const Segment SGI398766[] = {{&S94348[73], 4}, {NULL}};
const Segment SGI398767[] = {{&S94348[38], 15}, {NULL}};
const Segment SGI398768[] = {{&S94360[62], 4}, {NULL}};
const Segment SGI398769[] = {{&S94360[38], 4}, {NULL}};
const Segment SGI398770[] = {{&S94361[74], 4}, {NULL}};
const Segment SGI398771[] = {{&S94361[36], 18}, {NULL}};
const Segment SGI398772[] = {{&S94362[65], 4}, {NULL}};
const Segment SGI398773[] = {{&S94362[36], 9}, {NULL}};
const Segment SGI398774[] = {{&S94376[77], 4}, {NULL}};
const Segment SGI398775[] = {{&S94376[44], 13}, {NULL}};
const Segment SGI398776[] = {{&S94379[67], 4}, {NULL}};
const Segment SGI398777[] = {{&S94379[38], 9}, {NULL}};
const Segment SGI398778[] = {{&S94386[72], 4}, {NULL}};
const Segment SGI398779[] = {{&S94386[38], 14}, {NULL}};
const Segment SGI398780[] = {{&S54247[32], 18}, {NULL}};
const Segment SGI398781[] = {{&S54247[27], 4}, {NULL}};
const Segment SGI398782[] = {{&S54247[103], 9}, {NULL}};
const Segment SGI398783[] = {{&S54248[32], 10}, {NULL}};
const Segment SGI398784[] = {{&S54248[27], 4}, {NULL}};
const Segment SGI398785[] = {{&S54248[95], 9}, {NULL}};
const Segment SGI398786[] = {{&S54249[32], 10}, {NULL}};
const Segment SGI398787[] = {{&S54249[27], 4}, {NULL}};
const Segment SGI398788[] = {{&S54249[95], 9}, {NULL}};
const Segment SGI398789[] = {{&S54250[32], 10}, {NULL}};
const Segment SGI398790[] = {{&S54250[27], 4}, {NULL}};
const Segment SGI398791[] = {{&S54250[95], 9}, {NULL}};
const Segment SGI398792[] = {{&S54251[32], 10}, {NULL}};
const Segment SGI398793[] = {{&S54251[27], 4}, {NULL}};
const Segment SGI398794[] = {{&S54251[95], 9}, {NULL}};
const Segment SGI398795[] = {{&S54252[32], 16}, {NULL}};
const Segment SGI398796[] = {{&S54252[27], 4}, {NULL}};
const Segment SGI398797[] = {{&S54252[101], 9}, {NULL}};
const Segment SGI398798[] = {{&S54253[32], 10}, {NULL}};
const Segment SGI398799[] = {{&S54253[27], 4}, {NULL}};
const Segment SGI398800[] = {{&S54253[95], 9}, {NULL}};
const Segment SGI398801[] = {{&S94526[73], 4}, {NULL}};
const Segment SGI398802[] = {{&S94526[44], 9}, {NULL}};
const Segment SGI398803[] = {{&S94531[74], 4}, {NULL}};
const Segment SGI398804[] = {{&S94531[36], 18}, {NULL}};
const Segment SGI398805[] = {{&S94535[67], 4}, {NULL}};
const Segment SGI398806[] = {{&S94535[38], 9}, {NULL}};
const Segment SGI398807[] = {{&S94548[70], 4}, {NULL}};
const Segment SGI398808[] = {{&S94548[38], 12}, {NULL}};
const Segment SGI398809[] = {{&S94549[64], 4}, {NULL}};
const Segment SGI398810[] = {{&S94549[38], 6}, {NULL}};
const Segment SGI398811[] = {{&S94567[67], 4}, {NULL}};
const Segment SGI398812[] = {{&S94567[38], 9}, {NULL}};
const Segment SGI398813[] = {{&S94577[70], 4}, {NULL}};
const Segment SGI398814[] = {{&S94577[38], 12}, {NULL}};
const Segment SGI398815[] = {{&S94595[64], 4}, {NULL}};
const Segment SGI398816[] = {{&S94595[38], 6}, {NULL}};
const Segment SGI398817[] = {{&S94599[70], 4}, {NULL}};
const Segment SGI398818[] = {{&S94599[38], 12}, {NULL}};
const Segment SGI398819[] = {{&S94603[69], 4}, {NULL}};
const Segment SGI398820[] = {{&S94603[36], 13}, {NULL}};
const Segment SGI398821[] = {{&S94608[71], 4}, {NULL}};
const Segment SGI398822[] = {{&S94608[38], 13}, {NULL}};
const Segment SGI398823[] = {{&S94610[64], 4}, {NULL}};
const Segment SGI398824[] = {{&S94610[38], 6}, {NULL}};
const Segment SGI398825[] = {{&S94611[69], 4}, {NULL}};
const Segment SGI398826[] = {{&S94611[44], 5}, {NULL}};
const Segment SGI398827[] = {{&S94612[65], 4}, {NULL}};
const Segment SGI398828[] = {{&S94612[36], 9}, {NULL}};
const Segment SGI398829[] = {{&S94614[71], 4}, {NULL}};
const Segment SGI398830[] = {{&S94614[38], 13}, {NULL}};
const Segment SGI398831[] = {{&S54524[32], 14}, {NULL}};
const Segment SGI398832[] = {{&S54524[27], 4}, {NULL}};
const Segment SGI398833[] = {{&S54524[99], 9}, {NULL}};
const Segment SGI398834[] = {{&S94713[90], 4}, {NULL}};
const Segment SGI398835[] = {{&S94713[28], 12}, {NULL}};
const Segment SGI398836[] = {{&S94752[65], 4}, {NULL}};
const Segment SGI398837[] = {{&S94752[36], 9}, {NULL}};
const Segment SGI398838[] = {{&S94753[62], 4}, {NULL}};
const Segment SGI398839[] = {{&S94754[68], 4}, {NULL}};
const Segment SGI398840[] = {{&S94754[36], 12}, {NULL}};
const Segment SGI398841[] = {{&S94764[64], 4}, {NULL}};
const Segment SGI398842[] = {{&S94764[38], 6}, {NULL}};
const Segment SGI398843[] = {{&S94768[63], 4}, {NULL}};
const Segment SGI398844[] = {{&S94768[38], 5}, {NULL}};
const Segment SGI398845[] = {{&S94773[63], 4}, {NULL}};
const Segment SGI398846[] = {{&S94773[38], 5}, {NULL}};
const Segment SGI398847[] = {{&S94783[69], 4}, {NULL}};
const Segment SGI398848[] = {{&S94783[38], 11}, {NULL}};
const Segment SGI398849[] = {{&S94800[67], 4}, {NULL}};
const Segment SGI398850[] = {{&S94800[38], 9}, {NULL}};
const Segment SGI398851[] = {{&S94802[71], 4}, {NULL}};
const Segment SGI398852[] = {{&S94802[38], 13}, {NULL}};
const Segment SGI398853[] = {{&S94803[65], 4}, {NULL}};
const Segment SGI398854[] = {{&S94803[36], 9}, {NULL}};
const Segment SGI398855[] = {{&S94805[65], 4}, {NULL}};
const Segment SGI398856[] = {{&S94805[36], 9}, {NULL}};
const Segment SGI398857[] = {{&S94806[62], 4}, {NULL}};
const Segment SGI398858[] = {{&S54664[32], 10}, {NULL}};
const Segment SGI398859[] = {{&S54664[27], 4}, {NULL}};
const Segment SGI398860[] = {{&S54664[95], 9}, {NULL}};
const Segment SGI398861[] = {{&S94814[67], 4}, {NULL}};
const Segment SGI398862[] = {{&S94814[38], 9}, {NULL}};
const Segment SGI398863[] = {{&S94824[67], 4}, {NULL}};
const Segment SGI398864[] = {{&S94824[36], 11}, {NULL}};
const Segment SGI398865[] = {{&S94826[69], 4}, {NULL}};
const Segment SGI398866[] = {{&S94826[44], 5}, {NULL}};
const Segment SGI398867[] = {{&S94827[70], 4}, {NULL}};
const Segment SGI398868[] = {{&S94827[38], 12}, {NULL}};
const Segment SGI398869[] = {{&S94830[63], 4}, {NULL}};
const Segment SGI398870[] = {{&S94830[38], 5}, {NULL}};
const Segment SGI398871[] = {{&S94831[62], 4}, {NULL}};
const Segment SGI398872[] = {{&S94831[38], 4}, {NULL}};
const Segment SGI398873[] = {{&S94834[70], 4}, {NULL}};
const Segment SGI398874[] = {{&S94834[38], 12}, {NULL}};
const Segment SGI398875[] = {{&S94847[66], 4}, {NULL}};
const Segment SGI398876[] = {{&S94847[38], 8}, {NULL}};
const Segment SGI398877[] = {{&S94866[67], 4}, {NULL}};
const Segment SGI398878[] = {{&S94866[38], 9}, {NULL}};
const Segment SGI398879[] = {{&S94867[72], 4}, {NULL}};
const Segment SGI398880[] = {{&S94867[38], 14}, {NULL}};
const Segment SGI398881[] = {{&S94872[67], 4}, {NULL}};
const Segment SGI398882[] = {{&S94872[38], 9}, {NULL}};
const Segment SGI398883[] = {{&S94944[64], 4}, {NULL}};
const Segment SGI398884[] = {{&S94944[35], 9}, {NULL}};
const Segment SGI398885[] = {{&S94945[64], 4}, {NULL}};
const Segment SGI398886[] = {{&S94945[35], 9}, {NULL}};
const Segment SGI398887[] = {{&S95000[68], 4}, {NULL}};
const Segment SGI398888[] = {{&S95000[38], 10}, {NULL}};
const Segment SGI398889[] = {{&S95001[65], 4}, {NULL}};
const Segment SGI398890[] = {{&S95001[36], 9}, {NULL}};
const Segment SGI398891[] = {{&S95004[65], 4}, {NULL}};
const Segment SGI398892[] = {{&S95004[38], 7}, {NULL}};
const Segment SGI398893[] = {{&S95005[67], 4}, {NULL}};
const Segment SGI398894[] = {{&S95005[36], 11}, {NULL}};
const Segment SGI398895[] = {{&S95045[68], 4}, {NULL}};
const Segment SGI398896[] = {{&S95045[38], 10}, {NULL}};
const Segment SGI398897[] = {{&S95046[66], 4}, {NULL}};
const Segment SGI398898[] = {{&S95046[38], 8}, {NULL}};
const Segment SGI398899[] = {{&S95073[68], 4}, {NULL}};
const Segment SGI398900[] = {{&S95073[38], 10}, {NULL}};
const Segment SGI398901[] = {{&S95077[62], 4}, {NULL}};
const Segment SGI398902[] = {{&S95077[38], 4}, {NULL}};
const Segment SGI398903[] = {{&S95094[68], 4}, {NULL}};
const Segment SGI398904[] = {{&S95094[36], 12}, {NULL}};
const Segment SGI398905[] = {{&S95098[67], 4}, {NULL}};
const Segment SGI398906[] = {{&S95098[38], 9}, {NULL}};
const Segment SGI398907[] = {{&S95106[62], 4}, {NULL}};
const Segment SGI398908[] = {{&S95106[38], 4}, {NULL}};
const Segment SGI398909[] = {{&S95110[67], 4}, {NULL}};
const Segment SGI398910[] = {{&S95110[38], 9}, {NULL}};
const Segment SGI398911[] = {{&S95111[67], 4}, {NULL}};
const Segment SGI398912[] = {{&S95111[38], 9}, {NULL}};
const Segment SGI398913[] = {{&S95114[67], 4}, {NULL}};
const Segment SGI398914[] = {{&S95114[38], 9}, {NULL}};
const Segment SGI398915[] = {{&S95117[73], 4}, {NULL}};
const Segment SGI398916[] = {{&S95117[38], 15}, {NULL}};
const Segment SGI398917[] = {{&S95208[73], 4}, {NULL}};
const Segment SGI398918[] = {{&S95208[35], 18}, {NULL}};
const Segment SGI398919[] = {{&S95210[63], 4}, {NULL}};
const Segment SGI398920[] = {{&S95210[35], 8}, {NULL}};
const Segment SGI398921[] = {{&S95266[64], 4}, {NULL}};
const Segment SGI398922[] = {{&S95266[38], 6}, {NULL}};
const Segment SGI398923[] = {{&S95274[67], 4}, {NULL}};
const Segment SGI398924[] = {{&S95274[38], 9}, {NULL}};
const Segment SGI398925[] = {{&S95280[68], 4}, {NULL}};
const Segment SGI398926[] = {{&S95280[38], 10}, {NULL}};
const Segment SGI398927[] = {{&S95282[69], 4}, {NULL}};
const Segment SGI398928[] = {{&S95282[38], 11}, {NULL}};
const Segment SGI398929[] = {{&S95290[68], 4}, {NULL}};
const Segment SGI398930[] = {{&S95290[38], 10}, {NULL}};
const Segment SGI398931[] = {{&S95323[69], 4}, {NULL}};
const Segment SGI398932[] = {{&S95323[36], 13}, {NULL}};
const Segment SGI398933[] = {{&S95336[66], 4}, {NULL}};
const Segment SGI398934[] = {{&S95336[38], 8}, {NULL}};
const Segment SGI398935[] = {{&S95347[64], 4}, {NULL}};
const Segment SGI398936[] = {{&S95347[38], 6}, {NULL}};
const Segment SGI398937[] = {{&S95355[70], 4}, {NULL}};
const Segment SGI398938[] = {{&S95355[38], 12}, {NULL}};
const Segment SGI398939[] = {{&S95476[68], 4}, {NULL}};
const Segment SGI398940[] = {{&S95476[35], 13}, {NULL}};
const Segment SGI398941[] = {{&S95520[71], 4}, {NULL}};
const Segment SGI398942[] = {{&S95520[38], 13}, {NULL}};
const Segment SGI398943[] = {{&S95541[64], 4}, {NULL}};
const Segment SGI398944[] = {{&S95541[38], 6}, {NULL}};
const Segment SGI398945[] = {{&S95577[63], 4}, {NULL}};
const Segment SGI398946[] = {{&S95577[38], 5}, {NULL}};
const Segment SGI398947[] = {{&S95582[63], 4}, {NULL}};
const Segment SGI398948[] = {{&S95582[38], 5}, {NULL}};
const Segment SGI398949[] = {{&S95588[70], 4}, {NULL}};
const Segment SGI398950[] = {{&S95588[38], 12}, {NULL}};
const Segment SGI398951[] = {{&S95597[67], 4}, {NULL}};
const Segment SGI398952[] = {{&S95597[38], 9}, {NULL}};
const Segment SGI398953[] = {{&S95600[67], 4}, {NULL}};
const Segment SGI398954[] = {{&S95600[38], 9}, {NULL}};
const Segment SGI398955[] = {{&S95629[69], 4}, {NULL}};
const Segment SGI398956[] = {{&S95629[38], 11}, {NULL}};
const Segment SGI398957[] = {{&S95630[64], 4}, {NULL}};
const Segment SGI398958[] = {{&S95630[38], 6}, {NULL}};
const Segment SGI398959[] = {{&S95632[69], 4}, {NULL}};
const Segment SGI398960[] = {{&S95632[38], 11}, {NULL}};
const Segment SGI398961[] = {{&S95638[74], 4}, {NULL}};
const Segment SGI398962[] = {{&S95638[36], 18}, {NULL}};
const Segment SGI398963[] = {{&S95643[67], 4}, {NULL}};
const Segment SGI398964[] = {{&S95643[38], 9}, {NULL}};
const Segment SGI398965[] = {{&S95644[62], 4}, {NULL}};
const Segment SGI398966[] = {{&S95644[38], 4}, {NULL}};
const Segment SGI398967[] = {{&S95645[63], 4}, {NULL}};
const Segment SGI398968[] = {{&S95645[38], 5}, {NULL}};
const Segment SGI398969[] = {{&S95648[62], 4}, {NULL}};
const Segment SGI398970[] = {{&S95648[38], 4}, {NULL}};
const Segment SGI398971[] = {{&S95750[64], 4}, {NULL}};
const Segment SGI398972[] = {{&S95750[35], 9}, {NULL}};
const Segment SGI398973[] = {{&S95752[64], 4}, {NULL}};
const Segment SGI398974[] = {{&S95752[35], 9}, {NULL}};
const Segment SGI398975[] = {{&S95753[61], 4}, {NULL}};
const Segment SGI398976[] = {{&S95764[67], 4}, {NULL}};
const Segment SGI398977[] = {{&S95764[35], 12}, {NULL}};
const Segment SGI398978[] = {{&S95772[68], 4}, {NULL}};
const Segment SGI398979[] = {{&S95772[35], 13}, {NULL}};
const Segment SGI398980[] = {{&S95787[65], 4}, {NULL}};
const Segment SGI398981[] = {{&S95787[36], 9}, {NULL}};
const Segment SGI398982[] = {{&S95788[66], 4}, {NULL}};
const Segment SGI398983[] = {{&S95788[36], 10}, {NULL}};
const Segment SGI398984[] = {{&S95818[62], 4}, {NULL}};
const Segment SGI398985[] = {{&S95818[38], 4}, {NULL}};
const Segment SGI398986[] = {{&S95828[71], 4}, {NULL}};
const Segment SGI398987[] = {{&S95828[44], 7}, {NULL}};
const Segment SGI398988[] = {{&S95831[66], 4}, {NULL}};
const Segment SGI398989[] = {{&S95831[38], 8}, {NULL}};
const Segment SGI398990[] = {{&S95834[48], 3}, {NULL}};
const Segment SGI398991[] = {{&S95834[28], 9}, {NULL}};
const Segment SGI398992[] = {{&S95835[47], 3}, {NULL}};
const Segment SGI398993[] = {{&S95835[28], 8}, {NULL}};
const Segment SGI398994[] = {{&S95838[69], 4}, {NULL}};
const Segment SGI398995[] = {{&S95838[36], 13}, {NULL}};
const Segment SGI398996[] = {{&S95860[67], 4}, {NULL}};
const Segment SGI398997[] = {{&S95860[38], 9}, {NULL}};
const Segment SGI398998[] = {{&S95865[65], 4}, {NULL}};
const Segment SGI398999[] = {{&S95865[36], 9}, {NULL}};
const Segment SGI399000[] = {{&S95872[66], 4}, {NULL}};
const Segment SGI399001[] = {{&S95872[38], 8}, {NULL}};
const Segment SGI399002[] = {{&S95875[67], 4}, {NULL}};
const Segment SGI399003[] = {{&S95875[38], 9}, {NULL}};
const Segment SGI399004[] = {{&S95880[67], 4}, {NULL}};
const Segment SGI399005[] = {{&S95880[38], 9}, {NULL}};
const Segment SGI399006[] = {{&S95881[66], 4}, {NULL}};
const Segment SGI399007[] = {{&S95881[39], 7}, {NULL}};
const Segment SGI399008[] = {{&S95892[62], 4}, {NULL}};
const Segment SGI399009[] = {{&S95892[38], 4}, {NULL}};
const Segment SGI399010[] = {{&S95893[67], 4}, {NULL}};
const Segment SGI399011[] = {{&S95893[38], 9}, {NULL}};
const Segment SGI399012[] = {{&S95894[64], 4}, {NULL}};
const Segment SGI399013[] = {{&S95894[38], 6}, {NULL}};
const Segment SGI399014[] = {{&S95896[65], 4}, {NULL}};
const Segment SGI399015[] = {{&S95896[36], 9}, {NULL}};
const Segment SGI399016[] = {{&S95904[74], 4}, {NULL}};
const Segment SGI399017[] = {{&S95904[36], 18}, {NULL}};
const Segment SGI399018[] = {{&S95912[68], 4}, {NULL}};
const Segment SGI399019[] = {{&S95912[38], 10}, {NULL}};
const Segment SGI399020[] = {{&S95914[67], 4}, {NULL}};
const Segment SGI399021[] = {{&S95914[38], 9}, {NULL}};
const Segment SGI399022[] = {{&S95916[74], 4}, {NULL}};
const Segment SGI399023[] = {{&S95916[36], 18}, {NULL}};
const Segment SGI399024[] = {{&S95923[64], 4}, {NULL}};
const Segment SGI399025[] = {{&S95923[38], 6}, {NULL}};
const Segment SGI399026[] = {{&S95925[67], 4}, {NULL}};
const Segment SGI399027[] = {{&S95925[38], 9}, {NULL}};
const Segment SGI399028[] = {{&S95929[67], 4}, {NULL}};
const Segment SGI399029[] = {{&S95929[38], 9}, {NULL}};
const Segment SGI399030[] = {{&S95930[71], 4}, {NULL}};
const Segment SGI399031[] = {{&S95930[38], 13}, {NULL}};
const Segment SGI399032[] = {{&S95935[64], 4}, {NULL}};
const Segment SGI399033[] = {{&S95935[38], 6}, {NULL}};
const Segment SGI399034[] = {{&S96043[87], 4}, {NULL}};
const Segment SGI399035[] = {{&S96043[28], 9}, {NULL}};
const Segment SGI399036[] = {{&S96046[73], 4}, {NULL}};
const Segment SGI399037[] = {{&S96046[35], 18}, {NULL}};
const Segment SGI399038[] = {{&S96047[65], 4}, {NULL}};
const Segment SGI399039[] = {{&S96047[35], 10}, {NULL}};
const Segment SGI399040[] = {{&S96048[65], 4}, {NULL}};
const Segment SGI399041[] = {{&S96048[35], 10}, {NULL}};
const Segment SGI399042[] = {{&S96050[73], 4}, {NULL}};
const Segment SGI399043[] = {{&S96050[35], 18}, {NULL}};
const Segment SGI399044[] = {{&S96054[89], 4}, {NULL}};
const Segment SGI399045[] = {{&S96054[28], 11}, {NULL}};
const Segment SGI399046[] = {{&S96057[67], 4}, {NULL}};
const Segment SGI399047[] = {{&S96057[35], 12}, {NULL}};
const Segment SGI399048[] = {{&S96121[66], 4}, {NULL}};
const Segment SGI399049[] = {{&S96121[38], 8}, {NULL}};
const Segment SGI399050[] = {{&S96140[71], 4}, {NULL}};
const Segment SGI399051[] = {{&S96140[38], 13}, {NULL}};
const Segment SGI399052[] = {{&S96143[69], 4}, {NULL}};
const Segment SGI399053[] = {{&S96143[38], 11}, {NULL}};
const Segment SGI399054[] = {{&S96175[68], 4}, {NULL}};
const Segment SGI399055[] = {{&S96175[38], 10}, {NULL}};
const Segment SGI399056[] = {{&S96190[67], 4}, {NULL}};
const Segment SGI399057[] = {{&S96190[38], 9}, {NULL}};
const Segment SGI399058[] = {{&S96202[71], 4}, {NULL}};
const Segment SGI399059[] = {{&S96202[38], 13}, {NULL}};
const Segment SGI399060[] = {{&S96203[68], 4}, {NULL}};
const Segment SGI399061[] = {{&S96203[38], 10}, {NULL}};
const Segment SGI399062[] = {{&S96205[67], 4}, {NULL}};
const Segment SGI399063[] = {{&S96205[38], 9}, {NULL}};
const Segment SGI399064[] = {{&S96215[62], 4}, {NULL}};
const Segment SGI399065[] = {{&S96215[38], 4}, {NULL}};
const Segment SGI399066[] = {{&S96222[64], 4}, {NULL}};
const Segment SGI399067[] = {{&S96222[38], 6}, {NULL}};
const Segment SGI399068[] = {{&S96231[67], 4}, {NULL}};
const Segment SGI399069[] = {{&S96231[38], 9}, {NULL}};
const Segment SGI399070[] = {{&S96232[69], 4}, {NULL}};
const Segment SGI399071[] = {{&S96232[44], 5}, {NULL}};
const Segment SGI399072[] = {{&S96263[73], 4}, {NULL}};
const Segment SGI399073[] = {{&S96263[38], 15}, {NULL}};
const Segment SGI399074[] = {{&S96265[63], 4}, {NULL}};
const Segment SGI399075[] = {{&S96265[38], 5}, {NULL}};
const Segment SGI399076[] = {{&S96266[70], 4}, {NULL}};
const Segment SGI399077[] = {{&S96266[38], 12}, {NULL}};
const Segment SGI399078[] = {{&S96269[64], 4}, {NULL}};
const Segment SGI399079[] = {{&S96269[38], 6}, {NULL}};
const Segment SGI399080[] = {{&S96273[67], 4}, {NULL}};
const Segment SGI399081[] = {{&S96273[38], 9}, {NULL}};
const Segment SGI399082[] = {{&S96274[61], 4}, {NULL}};
const Segment SGI399083[] = {{&S96274[38], 3}, {NULL}};
const Segment SGI399084[] = {{&S96372[73], 4}, {NULL}};
const Segment SGI399085[] = {{&S96372[35], 18}, {NULL}};
const Segment SGI399086[] = {{&S96374[73], 4}, {NULL}};
const Segment SGI399087[] = {{&S96374[35], 18}, {NULL}};
const Segment SGI399088[] = {{&S96400[73], 4}, {NULL}};
const Segment SGI399089[] = {{&S96400[35], 18}, {NULL}};
const Segment SGI399090[] = {{&S96415[65], 4}, {NULL}};
const Segment SGI399091[] = {{&S96415[35], 10}, {NULL}};
const Segment SGI399092[] = {{&S96457[65], 4}, {NULL}};
const Segment SGI399093[] = {{&S96457[38], 7}, {NULL}};
const Segment SGI399094[] = {{&S96460[73], 4}, {NULL}};
const Segment SGI399095[] = {{&S96460[44], 9}, {NULL}};
const Segment SGI399096[] = {{&S96470[73], 4}, {NULL}};
const Segment SGI399097[] = {{&S96470[44], 9}, {NULL}};
const Segment SGI399098[] = {{&S96471[71], 4}, {NULL}};
const Segment SGI399099[] = {{&S96471[38], 13}, {NULL}};
const Segment SGI399100[] = {{&S96476[65], 4}, {NULL}};
const Segment SGI399101[] = {{&S96476[36], 9}, {NULL}};
const Segment SGI399102[] = {{&S96489[68], 4}, {NULL}};
const Segment SGI399103[] = {{&S96489[38], 10}, {NULL}};
const Segment SGI399104[] = {{&S96490[66], 4}, {NULL}};
const Segment SGI399105[] = {{&S96490[38], 8}, {NULL}};
const Segment SGI399106[] = {{&S96500[72], 4}, {NULL}};
const Segment SGI399107[] = {{&S96500[38], 14}, {NULL}};
const Segment SGI399108[] = {{&S96523[67], 4}, {NULL}};
const Segment SGI399109[] = {{&S96523[38], 9}, {NULL}};
const Segment SGI399110[] = {{&S96526[67], 4}, {NULL}};
const Segment SGI399111[] = {{&S96526[38], 9}, {NULL}};
const Segment SGI399112[] = {{&S96548[67], 4}, {NULL}};
const Segment SGI399113[] = {{&S96548[38], 9}, {NULL}};
const Segment SGI399114[] = {{&S96550[76], 4}, {NULL}};
const Segment SGI399115[] = {{&S96550[38], 18}, {NULL}};
const Segment SGI399116[] = {{&S96551[62], 4}, {NULL}};
const Segment SGI399117[] = {{&S96551[36], 6}, {NULL}};
const Segment SGI399118[] = {{&S96554[62], 4}, {NULL}};
const Segment SGI399119[] = {{&S96554[38], 4}, {NULL}};
const Segment SGI399120[] = {{&S96555[69], 4}, {NULL}};
const Segment SGI399121[] = {{&S96555[36], 13}, {NULL}};
const Segment SGI399122[] = {{&S96557[62], 4}, {NULL}};
const Segment SGI399123[] = {{&S96557[36], 6}, {NULL}};
const Segment SGI399124[] = {{&S96570[67], 4}, {NULL}};
const Segment SGI399125[] = {{&S96570[38], 9}, {NULL}};
const Segment SGI399126[] = {{&S96571[64], 4}, {NULL}};
const Segment SGI399127[] = {{&S96571[38], 6}, {NULL}};
const Segment SGI399128[] = {{&S96574[67], 4}, {NULL}};
const Segment SGI399129[] = {{&S96574[38], 9}, {NULL}};
const Segment SGI399130[] = {{&S56112[32], 11}, {NULL}};
const Segment SGI399131[] = {{&S56112[27], 4}, {NULL}};
const Segment SGI399132[] = {{&S56112[101], 9}, {NULL}};
const Segment SGI399133[] = {{&S56113[32], 10}, {NULL}};
const Segment SGI399134[] = {{&S56113[27], 4}, {NULL}};
const Segment SGI399135[] = {{&S56113[95], 9}, {NULL}};
const Segment SGI399136[] = {{&S56114[39], 18}, {NULL}};
const Segment SGI399137[] = {{&S56114[39], 30}, {NULL}};
const Segment SGI399138[] = {{&S56114[27], 4}, {NULL}};
const Segment SGI399139[] = {{&S56114[118], 3}, {NULL}};
const Segment SGI399140[] = {{&S56114[122], 9}, {NULL}};
const Segment SGI399141[] = {{&S96653[57], 3}, {NULL}};
const Segment SGI399142[] = {{&S96653[28], 18}, {NULL}};
const Segment SGI399143[] = {{&S96656[45], 3}, {NULL}};
const Segment SGI399144[] = {{&S96656[28], 6}, {NULL}};
const Segment SGI399145[] = {{&S96666[87], 4}, {NULL}};
const Segment SGI399146[] = {{&S96666[28], 9}, {NULL}};
const Segment SGI399147[] = {{&S96719[73], 4}, {NULL}};
const Segment SGI399148[] = {{&S96719[35], 18}, {NULL}};
const Segment SGI399149[] = {{&S96739[68], 4}, {NULL}};
const Segment SGI399150[] = {{&S96739[36], 12}, {NULL}};
const Segment SGI399151[] = {{&S96754[67], 4}, {NULL}};
const Segment SGI399152[] = {{&S96754[38], 9}, {NULL}};
const Segment SGI399153[] = {{&S96758[69], 4}, {NULL}};
const Segment SGI399154[] = {{&S96758[38], 11}, {NULL}};
const Segment SGI399155[] = {{&S96785[63], 4}, {NULL}};
const Segment SGI399156[] = {{&S96785[38], 5}, {NULL}};
const Segment SGI399157[] = {{&S96788[66], 4}, {NULL}};
const Segment SGI399158[] = {{&S96788[36], 10}, {NULL}};
const Segment SGI399159[] = {{&S96789[63], 4}, {NULL}};
const Segment SGI399160[] = {{&S96793[67], 4}, {NULL}};
const Segment SGI399161[] = {{&S96793[38], 9}, {NULL}};
const Segment SGI399162[] = {{&S96794[62], 4}, {NULL}};
const Segment SGI399163[] = {{&S96794[38], 4}, {NULL}};
const Segment SGI399164[] = {{&S96796[67], 4}, {NULL}};
const Segment SGI399165[] = {{&S96796[38], 9}, {NULL}};
const Segment SGI399166[] = {{&S96797[72], 4}, {NULL}};
const Segment SGI399167[] = {{&S96797[38], 14}, {NULL}};
const Segment SGI399168[] = {{&S96810[64], 4}, {NULL}};
const Segment SGI399169[] = {{&S96810[38], 6}, {NULL}};
const Segment SGI399170[] = {{&S96812[64], 4}, {NULL}};
const Segment SGI399171[] = {{&S96812[38], 6}, {NULL}};
const Segment SGI399172[] = {{&S96814[68], 4}, {NULL}};
const Segment SGI399173[] = {{&S96814[36], 12}, {NULL}};
const Segment SGI399174[] = {{&S96816[76], 4}, {NULL}};
const Segment SGI399175[] = {{&S96816[38], 18}, {NULL}};
const Segment SGI399176[] = {{&S96823[69], 4}, {NULL}};
const Segment SGI399177[] = {{&S96823[36], 13}, {NULL}};
const Segment SGI399178[] = {{&S96825[66], 4}, {NULL}};
const Segment SGI399179[] = {{&S96825[38], 8}, {NULL}};
const Segment SGI399180[] = {{&S96847[77], 4}, {NULL}};
const Segment SGI399181[] = {{&S96847[38], 19}, {NULL}};
const Segment SGI399182[] = {{&S96853[67], 4}, {NULL}};
const Segment SGI399183[] = {{&S96853[38], 9}, {NULL}};
const Segment SGI399184[] = {{&S96856[61], 4}, {NULL}};
const Segment SGI399185[] = {{&S96856[38], 3}, {NULL}};
const Segment SGI399186[] = {{&S96877[64], 4}, {NULL}};
const Segment SGI399187[] = {{&S96877[38], 6}, {NULL}};
const Segment SGI399188[] = {{&S96879[67], 4}, {NULL}};
const Segment SGI399189[] = {{&S96879[38], 9}, {NULL}};
const Segment SGI399190[] = {{&S96882[67], 4}, {NULL}};
const Segment SGI399191[] = {{&S96882[38], 9}, {NULL}};
const Segment SGI399192[] = {{&S97006[55], 3}, {NULL}};
const Segment SGI399193[] = {{&S97006[28], 16}, {NULL}};
const Segment SGI399194[] = {{&S97018[73], 4}, {NULL}};
const Segment SGI399195[] = {{&S97018[35], 18}, {NULL}};
const Segment SGI399196[] = {{&S97029[62], 4}, {NULL}};
const Segment SGI399197[] = {{&S97030[65], 4}, {NULL}};
const Segment SGI399198[] = {{&S97030[36], 9}, {NULL}};
const Segment SGI399199[] = {{&S97096[74], 4}, {NULL}};
const Segment SGI399200[] = {{&S97096[36], 18}, {NULL}};
const Segment SGI399201[] = {{&S97106[72], 4}, {NULL}};
const Segment SGI399202[] = {{&S97106[38], 13}, {NULL}};
const Segment SGI399203[] = {{&S97110[64], 4}, {NULL}};
const Segment SGI399204[] = {{&S97110[38], 6}, {NULL}};
const Segment SGI399205[] = {{&S97113[67], 4}, {NULL}};
const Segment SGI399206[] = {{&S97113[38], 9}, {NULL}};
const Segment SGI399207[] = {{&S97143[67], 4}, {NULL}};
const Segment SGI399208[] = {{&S97143[38], 9}, {NULL}};
const Segment SGI399209[] = {{&S97144[66], 4}, {NULL}};
const Segment SGI399210[] = {{&S97144[38], 8}, {NULL}};
const Segment SGI399211[] = {{&S97161[63], 4}, {NULL}};
const Segment SGI399212[] = {{&S97161[38], 5}, {NULL}};
const Segment SGI399213[] = {{&S97196[66], 4}, {NULL}};
const Segment SGI399214[] = {{&S97196[39], 7}, {NULL}};
const Segment SGI399215[] = {{&S97210[67], 4}, {NULL}};
const Segment SGI399216[] = {{&S97210[38], 9}, {NULL}};
const Segment SGI399217[] = {{&S97213[68], 4}, {NULL}};
const Segment SGI399218[] = {{&S97213[38], 10}, {NULL}};
const Segment SGI399219[] = {{&S97214[69], 4}, {NULL}};
const Segment SGI399220[] = {{&S97214[44], 5}, {NULL}};
const Segment SGI399221[] = {{&S97326[86], 4}, {NULL}};
const Segment SGI399222[] = {{&S97326[28], 8}, {NULL}};
const Segment SGI399223[] = {{&S97340[67], 4}, {NULL}};
const Segment SGI399224[] = {{&S97340[35], 12}, {NULL}};
const Segment SGI399225[] = {{&S97384[64], 4}, {NULL}};
const Segment SGI399226[] = {{&S97384[35], 9}, {NULL}};
const Segment SGI399227[] = {{&S97436[66], 4}, {NULL}};
const Segment SGI399228[] = {{&S97436[38], 8}, {NULL}};
const Segment SGI399229[] = {{&S97469[64], 4}, {NULL}};
const Segment SGI399230[] = {{&S97469[38], 6}, {NULL}};
const Segment SGI399231[] = {{&S97490[67], 4}, {NULL}};
const Segment SGI399232[] = {{&S97490[38], 9}, {NULL}};
const Segment SGI399233[] = {{&S97491[64], 4}, {NULL}};
const Segment SGI399234[] = {{&S97491[38], 6}, {NULL}};
const Segment SGI399235[] = {{&S97500[64], 4}, {NULL}};
const Segment SGI399236[] = {{&S97500[38], 6}, {NULL}};
const Segment SGI399237[] = {{&S97505[68], 4}, {NULL}};
const Segment SGI399238[] = {{&S97505[38], 10}, {NULL}};
const Segment SGI399239[] = {{&S97507[74], 4}, {NULL}};
const Segment SGI399240[] = {{&S97507[36], 18}, {NULL}};
const Segment SGI399241[] = {{&S97513[75], 4}, {NULL}};
const Segment SGI399242[] = {{&S97513[44], 11}, {NULL}};
const Segment SGI399243[] = {{&S97516[71], 4}, {NULL}};
const Segment SGI399244[] = {{&S97516[38], 13}, {NULL}};
const Segment SGI399245[] = {{&S97518[64], 4}, {NULL}};
const Segment SGI399246[] = {{&S97518[38], 6}, {NULL}};
const Segment SGI399247[] = {{&S97527[66], 4}, {NULL}};
const Segment SGI399248[] = {{&S97527[36], 10}, {NULL}};
const Segment SGI399249[] = {{&S97528[63], 4}, {NULL}};
const Segment SGI399250[] = {{&S97537[86], 4}, {NULL}};
const Segment SGI399251[] = {{&S97537[28], 8}, {NULL}};
const Segment SGI399252[] = {{&S97538[68], 4}, {NULL}};
const Segment SGI399253[] = {{&S97538[38], 10}, {NULL}};
const Segment SGI399254[] = {{&S97548[69], 4}, {NULL}};
const Segment SGI399255[] = {{&S97548[38], 11}, {NULL}};
const Segment SGI399256[] = {{&S97568[64], 4}, {NULL}};
const Segment SGI399257[] = {{&S97568[38], 6}, {NULL}};
const Segment SGI399258[] = {{&S97574[73], 4}, {NULL}};
const Segment SGI399259[] = {{&S97574[44], 9}, {NULL}};
const Segment SGI399260[] = {{&S97577[68], 4}, {NULL}};
const Segment SGI399261[] = {{&S97577[38], 10}, {NULL}};
const Segment SGI399262[] = {{&S57222[32], 10}, {NULL}};
const Segment SGI399263[] = {{&S57222[27], 4}, {NULL}};
const Segment SGI399264[] = {{&S57222[95], 9}, {NULL}};
const Segment SGI399265[] = {{&S57223[32], 11}, {NULL}};
const Segment SGI399266[] = {{&S57223[27], 4}, {NULL}};
const Segment SGI399267[] = {{&S57223[96], 9}, {NULL}};
const Segment SGI399268[] = {{&S97587[70], 4}, {NULL}};
const Segment SGI399269[] = {{&S97587[38], 12}, {NULL}};
const Segment SGI399270[] = {{&S97601[67], 4}, {NULL}};
const Segment SGI399271[] = {{&S97601[38], 9}, {NULL}};
const Segment SGI399272[] = {{&S97603[70], 4}, {NULL}};
const Segment SGI399273[] = {{&S97603[38], 12}, {NULL}};
const Segment SGI399274[] = {{&S57286[42], 14}, {NULL}};
const Segment SGI399275[] = {{&S57286[42], 26}, {NULL}};
const Segment SGI399276[] = {{&S57286[30], 4}, {NULL}};
const Segment SGI399277[] = {{&S57286[117], 3}, {NULL}};
const Segment SGI399278[] = {{&S57286[142], 8}, {NULL}};
const Segment SGI399279[] = {{&S57287[42], 14}, {NULL}};
const Segment SGI399280[] = {{&S57287[42], 26}, {NULL}};
const Segment SGI399281[] = {{&S57287[30], 4}, {NULL}};
const Segment SGI399282[] = {{&S57287[117], 3}, {NULL}};
const Segment SGI399283[] = {{&S57287[142], 8}, {NULL}};
const Segment SGI399284[] = {{&S57288[42], 14}, {NULL}};
const Segment SGI399285[] = {{&S57288[42], 26}, {NULL}};
const Segment SGI399286[] = {{&S57288[30], 4}, {NULL}};
const Segment SGI399287[] = {{&S57288[117], 3}, {NULL}};
const Segment SGI399288[] = {{&S57288[142], 8}, {NULL}};
const Segment SGI399289[] = {{&S57289[42], 14}, {NULL}};
const Segment SGI399290[] = {{&S57289[42], 26}, {NULL}};
const Segment SGI399291[] = {{&S57289[30], 4}, {NULL}};
const Segment SGI399292[] = {{&S57289[117], 3}, {NULL}};
const Segment SGI399293[] = {{&S57289[142], 8}, {NULL}};
const Segment SGI399294[] = {{&S97759[73], 4}, {NULL}};
const Segment SGI399295[] = {{&S97759[36], 17}, {NULL}};
const Segment SGI399296[] = {{&S57386[42], 9}, {NULL}};
const Segment SGI399297[] = {{&S57386[42], 21}, {NULL}};
const Segment SGI399298[] = {{&S57386[30], 4}, {NULL}};
const Segment SGI399299[] = {{&S57386[112], 3}, {NULL}};
const Segment SGI399300[] = {{&S57386[137], 8}, {NULL}};
const Segment SGI399301[] = {{&S57387[42], 9}, {NULL}};
const Segment SGI399302[] = {{&S57387[42], 21}, {NULL}};
const Segment SGI399303[] = {{&S57387[30], 4}, {NULL}};
const Segment SGI399304[] = {{&S57387[112], 3}, {NULL}};
const Segment SGI399305[] = {{&S57387[137], 8}, {NULL}};
const Segment SGI399306[] = {{&S57388[42], 9}, {NULL}};
const Segment SGI399307[] = {{&S57388[42], 21}, {NULL}};
const Segment SGI399308[] = {{&S57388[30], 4}, {NULL}};
const Segment SGI399309[] = {{&S57388[112], 3}, {NULL}};
const Segment SGI399310[] = {{&S57388[137], 8}, {NULL}};
const Segment SGI399311[] = {{&S57389[42], 9}, {NULL}};
const Segment SGI399312[] = {{&S57389[42], 21}, {NULL}};
const Segment SGI399313[] = {{&S57389[30], 4}, {NULL}};
const Segment SGI399314[] = {{&S57389[112], 3}, {NULL}};
const Segment SGI399315[] = {{&S57389[137], 8}, {NULL}};
const Segment SGI399316[] = {{&S97825[67], 4}, {NULL}};
const Segment SGI399317[] = {{&S97825[38], 9}, {NULL}};
const Segment SGI399318[] = {{&S97844[66], 4}, {NULL}};
const Segment SGI399319[] = {{&S97844[38], 8}, {NULL}};
const Segment SGI399320[] = {{&S97845[64], 4}, {NULL}};
const Segment SGI399321[] = {{&S97845[38], 6}, {NULL}};
const Segment SGI399322[] = {{&S97846[67], 4}, {NULL}};
const Segment SGI399323[] = {{&S97846[38], 9}, {NULL}};
const Segment SGI399324[] = {{&S97860[64], 4}, {NULL}};
const Segment SGI399325[] = {{&S97860[38], 6}, {NULL}};
const Segment SGI399326[] = {{&S97881[67], 4}, {NULL}};
const Segment SGI399327[] = {{&S97881[38], 9}, {NULL}};
const Segment SGI399328[] = {{&S97882[69], 4}, {NULL}};
const Segment SGI399329[] = {{&S97882[38], 11}, {NULL}};
const Segment SGI399330[] = {{&S97883[66], 4}, {NULL}};
const Segment SGI399331[] = {{&S97883[38], 8}, {NULL}};
const Segment SGI399332[] = {{&S97889[66], 4}, {NULL}};
const Segment SGI399333[] = {{&S97889[38], 8}, {NULL}};
const Segment SGI399334[] = {{&S97909[64], 4}, {NULL}};
const Segment SGI399335[] = {{&S97909[38], 6}, {NULL}};
const Segment SGI399336[] = {{&S97911[66], 4}, {NULL}};
const Segment SGI399337[] = {{&S97911[38], 8}, {NULL}};
const Segment SGI399338[] = {{&S97915[67], 4}, {NULL}};
const Segment SGI399339[] = {{&S97915[38], 9}, {NULL}};
const Segment SGI399340[] = {{&S97923[77], 4}, {NULL}};
const Segment SGI399341[] = {{&S97923[44], 13}, {NULL}};
const Segment SGI399342[] = {{&S97967[69], 4}, {NULL}};
const Segment SGI399343[] = {{&S97967[38], 11}, {NULL}};
const Segment SGI399344[] = {{&S97984[65], 4}, {NULL}};
const Segment SGI399345[] = {{&S97984[36], 9}, {NULL}};
const Segment SGI399346[] = {{&S97987[70], 4}, {NULL}};
const Segment SGI399347[] = {{&S97987[38], 12}, {NULL}};
const Segment SGI399348[] = {{&S97989[74], 4}, {NULL}};
const Segment SGI399349[] = {{&S97989[36], 18}, {NULL}};
const Segment SGI399350[] = {{&S98001[63], 4}, {NULL}};
const Segment SGI399351[] = {{&S98001[38], 5}, {NULL}};
const Segment SGI399352[] = {{&S98008[71], 4}, {NULL}};
const Segment SGI399353[] = {{&S98008[38], 13}, {NULL}};
const Segment SGI399354[] = {{&S98009[63], 4}, {NULL}};
const Segment SGI399355[] = {{&S98009[38], 5}, {NULL}};
const Segment SGI399356[] = {{&S98010[63], 4}, {NULL}};
const Segment SGI399357[] = {{&S98010[38], 5}, {NULL}};
const Segment SGI399358[] = {{&S98013[72], 4}, {NULL}};
const Segment SGI399359[] = {{&S98013[38], 14}, {NULL}};
const Segment SGI399360[] = {{&S98134[49], 3}, {NULL}};
const Segment SGI399361[] = {{&S98134[28], 10}, {NULL}};
const Segment SGI399362[] = {{&S98150[64], 4}, {NULL}};
const Segment SGI399363[] = {{&S98150[35], 9}, {NULL}};
const Segment SGI399364[] = {{&S98152[73], 4}, {NULL}};
const Segment SGI399365[] = {{&S98152[35], 18}, {NULL}};
const Segment SGI399366[] = {{&S98154[68], 4}, {NULL}};
const Segment SGI399367[] = {{&S98154[35], 13}, {NULL}};
const Segment SGI399368[] = {{&S98162[65], 4}, {NULL}};
const Segment SGI399369[] = {{&S98162[36], 9}, {NULL}};
const Segment SGI399370[] = {{&S98171[73], 4}, {NULL}};
const Segment SGI399371[] = {{&S98171[35], 18}, {NULL}};
const Segment SGI399372[] = {{&S98197[86], 4}, {NULL}};
const Segment SGI399373[] = {{&S98197[28], 8}, {NULL}};
const Segment SGI399374[] = {{&S98218[74], 4}, {NULL}};
const Segment SGI399375[] = {{&S98218[36], 18}, {NULL}};
const Segment SGI399376[] = {{&S98219[74], 4}, {NULL}};
const Segment SGI399377[] = {{&S98219[36], 18}, {NULL}};
const Segment SGI399378[] = {{&S98232[69], 4}, {NULL}};
const Segment SGI399379[] = {{&S98232[36], 13}, {NULL}};
const Segment SGI399380[] = {{&S98259[61], 4}, {NULL}};
const Segment SGI399381[] = {{&S57886[32], 9}, {NULL}};
const Segment SGI399382[] = {{&S57886[27], 4}, {NULL}};
const Segment SGI399383[] = {{&S57886[94], 9}, {NULL}};
const Segment SGI399384[] = {{&S57886[128], 8}, {NULL}};
const Segment SGI399385[] = {{&S98279[63], 4}, {NULL}};
const Segment SGI399386[] = {{&S98279[38], 5}, {NULL}};
const Segment SGI399387[] = {{&S98329[74], 4}, {NULL}};
const Segment SGI399388[] = {{&S98329[36], 18}, {NULL}};
const Segment SGI399389[] = {{&S98343[66], 4}, {NULL}};
const Segment SGI399390[] = {{&S98343[38], 8}, {NULL}};
const Segment SGI399391[] = {{&S98346[64], 4}, {NULL}};
const Segment SGI399392[] = {{&S98346[38], 6}, {NULL}};
const Segment SGI399393[] = {{&S98352[69], 4}, {NULL}};
const Segment SGI399394[] = {{&S98352[38], 11}, {NULL}};
const Segment SGI399395[] = {{&S98355[66], 4}, {NULL}};
const Segment SGI399396[] = {{&S98355[38], 8}, {NULL}};
const Segment SGI399397[] = {{&S98376[64], 4}, {NULL}};
const Segment SGI399398[] = {{&S98376[38], 6}, {NULL}};
const Segment SGI399399[] = {{&S98390[68], 4}, {NULL}};
const Segment SGI399400[] = {{&S98390[36], 12}, {NULL}};
const Segment SGI399401[] = {{&S98391[64], 4}, {NULL}};
const Segment SGI399402[] = {{&S98391[38], 6}, {NULL}};
const Segment SGI399403[] = {{&S98392[62], 4}, {NULL}};
const Segment SGI399404[] = {{&S98392[38], 4}, {NULL}};
const Segment SGI399405[] = {{&S98405[64], 4}, {NULL}};
const Segment SGI399406[] = {{&S98405[38], 6}, {NULL}};
const Segment SGI399407[] = {{&S98407[67], 4}, {NULL}};
const Segment SGI399408[] = {{&S98407[38], 9}, {NULL}};
const Segment SGI399409[] = {{&S98425[64], 4}, {NULL}};
const Segment SGI399410[] = {{&S98425[38], 6}, {NULL}};
const Segment SGI399411[] = {{&S98428[64], 4}, {NULL}};
const Segment SGI399412[] = {{&S98428[36], 8}, {NULL}};
const Segment SGI399413[] = {{&S98438[64], 4}, {NULL}};
const Segment SGI399414[] = {{&S98438[38], 6}, {NULL}};
const Segment SGI399415[] = {{&S98440[66], 4}, {NULL}};
const Segment SGI399416[] = {{&S98440[38], 8}, {NULL}};
const Segment SGI399417[] = {{&S98443[70], 4}, {NULL}};
const Segment SGI399418[] = {{&S98443[38], 12}, {NULL}};
const Segment SGI399419[] = {{&S58058[42], 11}, {NULL}};
const Segment SGI399420[] = {{&S58058[42], 23}, {NULL}};
const Segment SGI399421[] = {{&S58058[30], 4}, {NULL}};
const Segment SGI399422[] = {{&S58058[114], 3}, {NULL}};
const Segment SGI399423[] = {{&S58058[139], 8}, {NULL}};
const Segment SGI399424[] = {{&S58059[42], 10}, {NULL}};
const Segment SGI399425[] = {{&S58059[42], 22}, {NULL}};
const Segment SGI399426[] = {{&S58059[30], 4}, {NULL}};
const Segment SGI399427[] = {{&S58059[113], 3}, {NULL}};
const Segment SGI399428[] = {{&S58059[138], 8}, {NULL}};
const Segment SGI399429[] = {{&S58060[42], 10}, {NULL}};
const Segment SGI399430[] = {{&S58060[42], 22}, {NULL}};
const Segment SGI399431[] = {{&S58060[30], 4}, {NULL}};
const Segment SGI399432[] = {{&S58060[113], 3}, {NULL}};
const Segment SGI399433[] = {{&S58060[138], 8}, {NULL}};
const Segment SGI399434[] = {{&S98450[65], 4}, {NULL}};
const Segment SGI399435[] = {{&S58061[42], 11}, {NULL}};
const Segment SGI399436[] = {{&S98450[36], 9}, {NULL}};
const Segment SGI399437[] = {{&S58061[42], 23}, {NULL}};
const Segment SGI399438[] = {{&S58061[30], 4}, {NULL}};
const Segment SGI399439[] = {{&S58061[114], 3}, {NULL}};
const Segment SGI399440[] = {{&S58061[139], 8}, {NULL}};
const Segment SGI399441[] = {{&S58062[42], 10}, {NULL}};
const Segment SGI399442[] = {{&S58062[42], 22}, {NULL}};
const Segment SGI399443[] = {{&S58062[30], 4}, {NULL}};
const Segment SGI399444[] = {{&S98452[69], 4}, {NULL}};
const Segment SGI399445[] = {{&S98452[44], 5}, {NULL}};
const Segment SGI399446[] = {{&S58062[113], 3}, {NULL}};
const Segment SGI399447[] = {{&S58062[138], 8}, {NULL}};
const Segment SGI399448[] = {{&S58063[42], 10}, {NULL}};
const Segment SGI399449[] = {{&S58063[42], 22}, {NULL}};
const Segment SGI399450[] = {{&S58063[30], 4}, {NULL}};
const Segment SGI399451[] = {{&S58063[113], 3}, {NULL}};
const Segment SGI399452[] = {{&S58063[138], 8}, {NULL}};
const Segment SGI399453[] = {{&S58064[42], 10}, {NULL}};
const Segment SGI399454[] = {{&S58064[42], 22}, {NULL}};
const Segment SGI399455[] = {{&S58064[30], 4}, {NULL}};
const Segment SGI399456[] = {{&S58064[113], 3}, {NULL}};
const Segment SGI399457[] = {{&S58064[138], 8}, {NULL}};
const Segment SGI399458[] = {{&S98468[62], 4}, {NULL}};
const Segment SGI399459[] = {{&S98468[38], 4}, {NULL}};
const Segment SGI399460[] = {{&S98475[67], 4}, {NULL}};
const Segment SGI399461[] = {{&S98475[38], 9}, {NULL}};
const Segment SGI399462[] = {{&S98476[68], 4}, {NULL}};
const Segment SGI399463[] = {{&S98476[38], 10}, {NULL}};
const Segment SGI399464[] = {{&S98486[66], 4}, {NULL}};
const Segment SGI399465[] = {{&S98486[38], 8}, {NULL}};
const Segment SGI399466[] = {{&S98487[67], 4}, {NULL}};
const Segment SGI399467[] = {{&S98487[38], 9}, {NULL}};
const Segment SGI399468[] = {{&S98488[64], 4}, {NULL}};
const Segment SGI399469[] = {{&S98488[38], 6}, {NULL}};
const Segment SGI399470[] = {{&S98571[54], 3}, {NULL}};
const Segment SGI399471[] = {{&S98571[28], 15}, {NULL}};
const Segment SGI399472[] = {{&S98572[45], 3}, {NULL}};
const Segment SGI399473[] = {{&S98572[28], 6}, {NULL}};
const Segment SGI399474[] = {{&S98612[47], 3}, {NULL}};
const Segment SGI399475[] = {{&S98612[28], 8}, {NULL}};
const Segment SGI399476[] = {{&S98615[87], 4}, {NULL}};
const Segment SGI399477[] = {{&S98615[28], 9}, {NULL}};
const Segment SGI399478[] = {{&S98620[73], 4}, {NULL}};
const Segment SGI399479[] = {{&S98620[35], 18}, {NULL}};
const Segment SGI399480[] = {{&S98621[73], 4}, {NULL}};
const Segment SGI399481[] = {{&S98621[35], 18}, {NULL}};
const Segment SGI399482[] = {{&S98622[70], 4}, {NULL}};
const Segment SGI399483[] = {{&S98632[65], 4}, {NULL}};
const Segment SGI399484[] = {{&S98632[35], 10}, {NULL}};
const Segment SGI399485[] = {{&S58262[42], 10}, {NULL}};
const Segment SGI399486[] = {{&S58262[42], 22}, {NULL}};
const Segment SGI399487[] = {{&S58262[30], 4}, {NULL}};
const Segment SGI399488[] = {{&S58262[113], 3}, {NULL}};
const Segment SGI399489[] = {{&S58262[138], 8}, {NULL}};
const Segment SGI399490[] = {{&S58263[42], 18}, {NULL}};
const Segment SGI399491[] = {{&S58263[42], 30}, {NULL}};
const Segment SGI399492[] = {{&S58263[30], 4}, {NULL}};
const Segment SGI399493[] = {{&S58263[121], 3}, {NULL}};
const Segment SGI399494[] = {{&S58263[146], 8}, {NULL}};
const Segment SGI399495[] = {{&S58264[42], 18}, {NULL}};
const Segment SGI399496[] = {{&S58264[42], 30}, {NULL}};
const Segment SGI399497[] = {{&S58264[30], 4}, {NULL}};
const Segment SGI399498[] = {{&S58264[121], 3}, {NULL}};
const Segment SGI399499[] = {{&S58264[146], 8}, {NULL}};
const Segment SGI399500[] = {{&S58265[42], 10}, {NULL}};
const Segment SGI399501[] = {{&S58265[42], 22}, {NULL}};
const Segment SGI399502[] = {{&S58265[30], 4}, {NULL}};
const Segment SGI399503[] = {{&S58265[113], 3}, {NULL}};
const Segment SGI399504[] = {{&S58265[138], 8}, {NULL}};
const Segment SGI399505[] = {{&S58266[42], 10}, {NULL}};
const Segment SGI399506[] = {{&S58266[42], 22}, {NULL}};
const Segment SGI399507[] = {{&S58266[30], 4}, {NULL}};
const Segment SGI399508[] = {{&S58266[113], 3}, {NULL}};
const Segment SGI399509[] = {{&S58266[138], 8}, {NULL}};
const Segment SGI399510[] = {{&S58267[42], 18}, {NULL}};
const Segment SGI399511[] = {{&S58267[42], 30}, {NULL}};
const Segment SGI399512[] = {{&S58267[30], 4}, {NULL}};
const Segment SGI399513[] = {{&S58267[121], 3}, {NULL}};
const Segment SGI399514[] = {{&S58267[146], 8}, {NULL}};
const Segment SGI399515[] = {{&S58268[42], 18}, {NULL}};
const Segment SGI399516[] = {{&S58268[42], 30}, {NULL}};
const Segment SGI399517[] = {{&S58268[30], 4}, {NULL}};
const Segment SGI399518[] = {{&S58268[121], 3}, {NULL}};
const Segment SGI399519[] = {{&S58268[146], 8}, {NULL}};
const Segment SGI399520[] = {{&S58269[42], 10}, {NULL}};
const Segment SGI399521[] = {{&S58269[42], 22}, {NULL}};
const Segment SGI399522[] = {{&S58269[30], 4}, {NULL}};
const Segment SGI399523[] = {{&S58269[113], 3}, {NULL}};
const Segment SGI399524[] = {{&S58269[138], 8}, {NULL}};
const Segment SGI399525[] = {{&S58270[42], 10}, {NULL}};
const Segment SGI399526[] = {{&S58270[42], 22}, {NULL}};
const Segment SGI399527[] = {{&S58270[30], 4}, {NULL}};
const Segment SGI399528[] = {{&S58270[113], 3}, {NULL}};
const Segment SGI399529[] = {{&S58270[138], 8}, {NULL}};
const Segment SGI399530[] = {{&S58271[42], 18}, {NULL}};
const Segment SGI399531[] = {{&S58271[42], 30}, {NULL}};
const Segment SGI399532[] = {{&S58271[30], 4}, {NULL}};
const Segment SGI399533[] = {{&S58271[121], 3}, {NULL}};
const Segment SGI399534[] = {{&S58271[146], 8}, {NULL}};
const Segment SGI399535[] = {{&S98669[62], 4}, {NULL}};
const Segment SGI399536[] = {{&S98669[36], 6}, {NULL}};
const Segment SGI399537[] = {{&S98686[66], 4}, {NULL}};
const Segment SGI399538[] = {{&S98686[36], 10}, {NULL}};
const Segment SGI399539[] = {{&S98694[65], 4}, {NULL}};
const Segment SGI399540[] = {{&S98694[36], 9}, {NULL}};
const Segment SGI399541[] = {{&S98702[49], 3}, {NULL}};
const Segment SGI399542[] = {{&S98702[28], 10}, {NULL}};
const Segment SGI399543[] = {{&S98727[64], 4}, {NULL}};
const Segment SGI399544[] = {{&S98727[38], 6}, {NULL}};
const Segment SGI399545[] = {{&S98730[90], 4}, {NULL}};
const Segment SGI399546[] = {{&S98730[44], 26}, {NULL}};
const Segment SGI399547[] = {{&S98761[64], 4}, {NULL}};
const Segment SGI399548[] = {{&S98761[38], 6}, {NULL}};
const Segment SGI399549[] = {{&S98776[69], 4}, {NULL}};
const Segment SGI399550[] = {{&S98776[44], 5}, {NULL}};
const Segment SGI399551[] = {{&S98790[64], 4}, {NULL}};
const Segment SGI399552[] = {{&S98790[36], 8}, {NULL}};
const Segment SGI399553[] = {{&S98797[71], 4}, {NULL}};
const Segment SGI399554[] = {{&S98797[38], 13}, {NULL}};
const Segment SGI399555[] = {{&S98800[67], 4}, {NULL}};
const Segment SGI399556[] = {{&S98800[38], 9}, {NULL}};
const Segment SGI399557[] = {{&S98809[71], 4}, {NULL}};
const Segment SGI399558[] = {{&S98809[38], 13}, {NULL}};
const Segment SGI399559[] = {{&S98814[64], 4}, {NULL}};
const Segment SGI399560[] = {{&S98814[38], 6}, {NULL}};
const Segment SGI399561[] = {{&S98832[73], 4}, {NULL}};
const Segment SGI399562[] = {{&S98832[38], 15}, {NULL}};
const Segment SGI399563[] = {{&S98836[74], 4}, {NULL}};
const Segment SGI399564[] = {{&S98836[36], 18}, {NULL}};
const Segment SGI399565[] = {{&S98846[62], 4}, {NULL}};
const Segment SGI399566[] = {{&S98846[38], 4}, {NULL}};
const Segment SGI399567[] = {{&S98849[66], 4}, {NULL}};
const Segment SGI399568[] = {{&S98849[38], 8}, {NULL}};
const Segment SGI399569[] = {{&S98850[67], 4}, {NULL}};
const Segment SGI399570[] = {{&S98850[38], 9}, {NULL}};
const Segment SGI399571[] = {{&S98851[74], 4}, {NULL}};
const Segment SGI399572[] = {{&S98851[36], 18}, {NULL}};
const Segment SGI399573[] = {{&S98854[64], 4}, {NULL}};
const Segment SGI399574[] = {{&S98854[38], 6}, {NULL}};
const Segment SGI399575[] = {{&S98856[69], 4}, {NULL}};
const Segment SGI399576[] = {{&S98856[44], 5}, {NULL}};
const Segment SGI399577[] = {{&S98869[67], 4}, {NULL}};
const Segment SGI399578[] = {{&S98869[38], 9}, {NULL}};
const Segment SGI399579[] = {{&S98880[64], 4}, {NULL}};
const Segment SGI399580[] = {{&S98880[38], 6}, {NULL}};
const Segment SGI399581[] = {{&S98882[72], 4}, {NULL}};
const Segment SGI399582[] = {{&S98882[38], 14}, {NULL}};
const Segment SGI399583[] = {{&S98887[63], 4}, {NULL}};
const Segment SGI399584[] = {{&S98887[38], 5}, {NULL}};
const Segment SGI399585[] = {{&S98899[66], 4}, {NULL}};
const Segment SGI399586[] = {{&S98899[38], 8}, {NULL}};
const Segment SGI399587[] = {{&S98936[67], 4}, {NULL}};
const Segment SGI399588[] = {{&S98936[38], 9}, {NULL}};
const Segment SGI399589[] = {{&S98940[64], 4}, {NULL}};
const Segment SGI399590[] = {{&S98940[38], 6}, {NULL}};
const Segment SGI399591[] = {{&S98950[70], 4}, {NULL}};
const Segment SGI399592[] = {{&S98950[38], 12}, {NULL}};
const Segment SGI399593[] = {{&S98970[66], 4}, {NULL}};
const Segment SGI399594[] = {{&S98970[38], 8}, {NULL}};
const Segment SGI399595[] = {{&S98972[62], 4}, {NULL}};
const Segment SGI399596[] = {{&S98972[38], 4}, {NULL}};
const Segment SGI399597[] = {{&S98975[64], 4}, {NULL}};
const Segment SGI399598[] = {{&S98975[38], 6}, {NULL}};
const Segment SGI399599[] = {{&S98976[64], 4}, {NULL}};
const Segment SGI399600[] = {{&S98976[38], 6}, {NULL}};
const Segment SGI399601[] = {{&S98981[69], 4}, {NULL}};
const Segment SGI399602[] = {{&S98981[38], 11}, {NULL}};
const Segment SGI399603[] = {{&S99127[64], 4}, {NULL}};
const Segment SGI399604[] = {{&S99127[35], 9}, {NULL}};
const Segment SGI399605[] = {{&S99138[87], 4}, {NULL}};
const Segment SGI399606[] = {{&S99138[28], 9}, {NULL}};
const Segment SGI399607[] = {{&S99163[45], 3}, {NULL}};
const Segment SGI399608[] = {{&S99166[86], 4}, {NULL}};
const Segment SGI399609[] = {{&S99166[28], 8}, {NULL}};
const Segment SGI399610[] = {{&S99168[65], 4}, {NULL}};
const Segment SGI399611[] = {{&S99168[35], 10}, {NULL}};
const Segment SGI399612[] = {{&S99206[91], 4}, {NULL}};
const Segment SGI399613[] = {{&S99206[28], 13}, {NULL}};
const Segment SGI399614[] = {{&S99221[63], 4}, {NULL}};
const Segment SGI399615[] = {{&S99221[35], 8}, {NULL}};
const Segment SGI399616[] = {{&S99240[69], 4}, {NULL}};
const Segment SGI399617[] = {{&S99240[36], 13}, {NULL}};
const Segment SGI399618[] = {{&S99242[64], 4}, {NULL}};
const Segment SGI399619[] = {{&S99242[38], 6}, {NULL}};
const Segment SGI399620[] = {{&S99286[89], 4}, {NULL}};
const Segment SGI399621[] = {{&S99286[28], 11}, {NULL}};
const Segment SGI399622[] = {{&S99294[64], 4}, {NULL}};
const Segment SGI399623[] = {{&S99294[38], 6}, {NULL}};
const Segment SGI399624[] = {{&S99295[66], 4}, {NULL}};
const Segment SGI399625[] = {{&S99295[38], 8}, {NULL}};
const Segment SGI399626[] = {{&S99297[71], 4}, {NULL}};
const Segment SGI399627[] = {{&S99297[38], 13}, {NULL}};
const Segment SGI399628[] = {{&S99302[67], 4}, {NULL}};
const Segment SGI399629[] = {{&S99302[38], 9}, {NULL}};
const Segment SGI399630[] = {{&S99308[81], 4}, {NULL}};
const Segment SGI399631[] = {{&S99308[38], 22}, {NULL}};
const Segment SGI399632[] = {{&S99313[68], 4}, {NULL}};
const Segment SGI399633[] = {{&S99313[36], 12}, {NULL}};
const Segment SGI399634[] = {{&S99314[70], 4}, {NULL}};
const Segment SGI399635[] = {{&S99314[38], 12}, {NULL}};
const Segment SGI399636[] = {{&S99335[67], 4}, {NULL}};
const Segment SGI399637[] = {{&S99335[38], 9}, {NULL}};
const Segment SGI399638[] = {{&S99350[65], 4}, {NULL}};
const Segment SGI399639[] = {{&S99350[36], 9}, {NULL}};
const Segment SGI399640[] = {{&S99367[63], 4}, {NULL}};
const Segment SGI399641[] = {{&S99367[38], 5}, {NULL}};
const Segment SGI399642[] = {{&S99372[68], 4}, {NULL}};
const Segment SGI399643[] = {{&S99372[38], 10}, {NULL}};
const Segment SGI399644[] = {{&S99373[64], 4}, {NULL}};
const Segment SGI399645[] = {{&S99373[38], 6}, {NULL}};
const Segment SGI399646[] = {{&S99378[70], 4}, {NULL}};
const Segment SGI399647[] = {{&S99378[38], 12}, {NULL}};
const Segment SGI399648[] = {{&S99380[64], 4}, {NULL}};
const Segment SGI399649[] = {{&S99380[38], 6}, {NULL}};
const Segment SGI399650[] = {{&S99388[66], 4}, {NULL}};
const Segment SGI399651[] = {{&S99388[38], 8}, {NULL}};
const Segment SGI399652[] = {{&S99393[66], 4}, {NULL}};
const Segment SGI399653[] = {{&S99393[38], 8}, {NULL}};
const Segment SGI399654[] = {{&S99417[63], 4}, {NULL}};
const Segment SGI399655[] = {{&S99417[38], 5}, {NULL}};
const Segment SGI399656[] = {{&S99434[64], 4}, {NULL}};
const Segment SGI399657[] = {{&S99434[38], 6}, {NULL}};
const Segment SGI399658[] = {{&S99436[67], 4}, {NULL}};
const Segment SGI399659[] = {{&S99436[38], 9}, {NULL}};
const Segment SGI399660[] = {{&S99444[63], 4}, {NULL}};
const Segment SGI399661[] = {{&S99444[38], 5}, {NULL}};
const Segment SGI399662[] = {{&S99457[63], 4}, {NULL}};
const Segment SGI399663[] = {{&S99457[38], 5}, {NULL}};
const Segment SGI399664[] = {{&S99460[63], 4}, {NULL}};
const Segment SGI399665[] = {{&S99460[38], 5}, {NULL}};
const Segment SGI399666[] = {{&S99462[73], 4}, {NULL}};
const Segment SGI399667[] = {{&S99462[38], 15}, {NULL}};
const Segment SGI399668[] = {{&S99473[70], 4}, {NULL}};
const Segment SGI399669[] = {{&S99473[38], 12}, {NULL}};
const Segment SGI399670[] = {{&S99481[66], 4}, {NULL}};
const Segment SGI399671[] = {{&S99481[36], 10}, {NULL}};
const Segment SGI399672[] = {{&S99484[67], 4}, {NULL}};
const Segment SGI399673[] = {{&S99484[38], 9}, {NULL}};
const Segment SGI399674[] = {{&S99488[67], 4}, {NULL}};
const Segment SGI399675[] = {{&S99488[38], 9}, {NULL}};
const Segment SGI399676[] = {{&S99490[62], 4}, {NULL}};
const Segment SGI399677[] = {{&S99490[38], 4}, {NULL}};
const Segment SGI399678[] = {{&S99493[66], 4}, {NULL}};
const Segment SGI399679[] = {{&S99493[38], 8}, {NULL}};
const Segment SGI399680[] = {{&S99498[63], 4}, {NULL}};
const Segment SGI399681[] = {{&S99498[38], 5}, {NULL}};
const Segment SGI399682[] = {{&S99502[62], 4}, {NULL}};
const Segment SGI399683[] = {{&S99502[38], 4}, {NULL}};
const Segment SGI399684[] = {{&S99505[64], 4}, {NULL}};
const Segment SGI399685[] = {{&S99505[38], 6}, {NULL}};
const Segment SGI399686[] = {{&S99507[70], 4}, {NULL}};
const Segment SGI399687[] = {{&S99507[38], 12}, {NULL}};
const Segment SGI399688[] = {{&S99516[67], 4}, {NULL}};
const Segment SGI399689[] = {{&S99516[38], 9}, {NULL}};
const Segment SGI399690[] = {{&S99661[50], 3}, {NULL}};
const Segment SGI399691[] = {{&S99661[28], 11}, {NULL}};
const Segment SGI399692[] = {{&S99666[96], 4}, {NULL}};
const Segment SGI399693[] = {{&S99666[28], 18}, {NULL}};
const Segment SGI399694[] = {{&S99667[47], 3}, {NULL}};
const Segment SGI399695[] = {{&S99667[28], 8}, {NULL}};
const Segment SGI399696[] = {{&S99668[48], 3}, {NULL}};
const Segment SGI399697[] = {{&S99668[28], 9}, {NULL}};
const Segment SGI399698[] = {{&S99674[52], 3}, {NULL}};
const Segment SGI399699[] = {{&S99674[28], 13}, {NULL}};
const Segment SGI399700[] = {{&S99682[52], 3}, {NULL}};
const Segment SGI399701[] = {{&S99682[28], 13}, {NULL}};
const Segment SGI399702[] = {{&S99716[73], 4}, {NULL}};
const Segment SGI399703[] = {{&S99716[35], 18}, {NULL}};
const Segment SGI399704[] = {{&S99717[67], 4}, {NULL}};
const Segment SGI399705[] = {{&S99717[35], 12}, {NULL}};
const Segment SGI399706[] = {{&S99737[66], 4}, {NULL}};
const Segment SGI399707[] = {{&S99737[35], 11}, {NULL}};
const Segment SGI399708[] = {{&S99751[64], 4}, {NULL}};
const Segment SGI399709[] = {{&S99751[36], 8}, {NULL}};
const Segment SGI399710[] = {{&S99752[64], 4}, {NULL}};
const Segment SGI399711[] = {{&S99752[36], 8}, {NULL}};
const Segment SGI399712[] = {{&S99778[87], 4}, {NULL}};
const Segment SGI399713[] = {{&S99778[28], 9}, {NULL}};
const Segment SGI399714[] = {{&S99829[67], 4}, {NULL}};
const Segment SGI399715[] = {{&S99829[36], 11}, {NULL}};
const Segment SGI399716[] = {{&S99860[73], 4}, {NULL}};
const Segment SGI399717[] = {{&S99860[44], 9}, {NULL}};
const Segment SGI399718[] = {{&S99871[67], 4}, {NULL}};
const Segment SGI399719[] = {{&S99871[38], 9}, {NULL}};
const Segment SGI399720[] = {{&S99872[62], 4}, {NULL}};
const Segment SGI399721[] = {{&S99872[38], 4}, {NULL}};
const Segment SGI399722[] = {{&S99876[61], 4}, {NULL}};
const Segment SGI399723[] = {{&S99876[38], 3}, {NULL}};
const Segment SGI399724[] = {{&S99886[67], 4}, {NULL}};
const Segment SGI399725[] = {{&S99886[38], 9}, {NULL}};
const Segment SGI399726[] = {{&S99890[72], 4}, {NULL}};
const Segment SGI399727[] = {{&S99890[38], 14}, {NULL}};
const Segment SGI399728[] = {{&S99896[62], 4}, {NULL}};
const Segment SGI399729[] = {{&S99896[38], 4}, {NULL}};
const Segment SGI399730[] = {{&S99901[67], 4}, {NULL}};
const Segment SGI399731[] = {{&S99901[38], 9}, {NULL}};
const Segment SGI399732[] = {{&S99909[64], 4}, {NULL}};
const Segment SGI399733[] = {{&S99909[38], 6}, {NULL}};
const Segment SGI399734[] = {{&S99911[67], 4}, {NULL}};
const Segment SGI399735[] = {{&S99911[38], 9}, {NULL}};
const Segment SGI399736[] = {{&S99917[61], 4}, {NULL}};
const Segment SGI399737[] = {{&S99917[38], 3}, {NULL}};
const Segment SGI399738[] = {{&S99924[61], 4}, {NULL}};
const Segment SGI399739[] = {{&S99924[38], 3}, {NULL}};
const Segment SGI399740[] = {{&S99928[73], 4}, {NULL}};
const Segment SGI399741[] = {{&S99928[44], 9}, {NULL}};
const Segment SGI399742[] = {{&S99929[67], 4}, {NULL}};
const Segment SGI399743[] = {{&S99929[38], 9}, {NULL}};
const Segment SGI399744[] = {{&S99940[66], 4}, {NULL}};
const Segment SGI399745[] = {{&S99940[38], 8}, {NULL}};
const Segment SGI399746[] = {{&S99954[64], 4}, {NULL}};
const Segment SGI399747[] = {{&S99954[38], 6}, {NULL}};
const Segment SGI399748[] = {{&S99959[69], 4}, {NULL}};
const Segment SGI399749[] = {{&S99959[44], 5}, {NULL}};
const Segment SGI399750[] = {{&S99977[67], 4}, {NULL}};
const Segment SGI399751[] = {{&S99977[38], 9}, {NULL}};
const Segment SGI399752[] = {{&S99979[66], 4}, {NULL}};
const Segment SGI399753[] = {{&S99979[38], 8}, {NULL}};
const Segment SGI399754[] = {{&S99980[67], 4}, {NULL}};
const Segment SGI399755[] = {{&S99980[38], 9}, {NULL}};
const Segment SGI399756[] = {{&S99997[66], 4}, {NULL}};
const Segment SGI399757[] = {{&S99997[38], 8}, {NULL}};
const Segment SGI399758[] = {{&S100008[63], 4}, {NULL}};
const Segment SGI399759[] = {{&S100008[38], 5}, {NULL}};
const Segment SGI399760[] = {{&S100019[67], 4}, {NULL}};
const Segment SGI399761[] = {{&S100019[38], 9}, {NULL}};
const Segment SGI399762[] = {{&S100048[66], 4}, {NULL}};
const Segment SGI399763[] = {{&S100048[38], 8}, {NULL}};
const Segment SGI399764[] = {{&S100058[64], 4}, {NULL}};
const Segment SGI399765[] = {{&S100058[38], 6}, {NULL}};
const Segment SGI399766[] = {{&S100061[64], 4}, {NULL}};
const Segment SGI399767[] = {{&S100061[38], 6}, {NULL}};
const Segment SGI399768[] = {{&S100064[64], 4}, {NULL}};
const Segment SGI399769[] = {{&S100064[38], 6}, {NULL}};
const Segment SGI399770[] = {{&S100066[62], 4}, {NULL}};
const Segment SGI399771[] = {{&S100066[38], 4}, {NULL}};
const Segment SGI399772[] = {{&S100068[63], 4}, {NULL}};
const Segment SGI399773[] = {{&S100068[38], 5}, {NULL}};
const Segment SGI399774[] = {{&S100238[50], 3}, {NULL}};
const Segment SGI399775[] = {{&S100238[28], 11}, {NULL}};
const Segment SGI399776[] = {{&S100245[86], 4}, {NULL}};
const Segment SGI399777[] = {{&S100245[28], 8}, {NULL}};
const Segment SGI399778[] = {{&S100254[90], 4}, {NULL}};
const Segment SGI399779[] = {{&S100254[28], 12}, {NULL}};
const Segment SGI399780[] = {{&S100272[89], 4}, {NULL}};
const Segment SGI399781[] = {{&S100272[28], 11}, {NULL}};
const Segment SGI399782[] = {{&S100293[46], 3}, {NULL}};
const Segment SGI399783[] = {{&S100293[28], 7}, {NULL}};
const Segment SGI399784[] = {{&S100326[90], 4}, {NULL}};
const Segment SGI399785[] = {{&S100326[28], 12}, {NULL}};
const Segment SGI399786[] = {{&S100328[88], 4}, {NULL}};
const Segment SGI399787[] = {{&S100328[28], 10}, {NULL}};
const Segment SGI399788[] = {{&S60115[32], 9}, {NULL}};
const Segment SGI399789[] = {{&S60115[27], 4}, {NULL}};
const Segment SGI399790[] = {{&S60115[98], 9}, {NULL}};
const Segment SGI399791[] = {{&S60116[32], 5}, {NULL}};
const Segment SGI399792[] = {{&S60116[27], 4}, {NULL}};
const Segment SGI399793[] = {{&S60116[94], 9}, {NULL}};
const Segment SGI399794[] = {{&S100351[69], 4}, {NULL}};
const Segment SGI399795[] = {{&S100351[44], 5}, {NULL}};
const Segment SGI399796[] = {{&S100365[70], 4}, {NULL}};
const Segment SGI399797[] = {{&S100365[38], 12}, {NULL}};
const Segment SGI399798[] = {{&S100416[64], 4}, {NULL}};
const Segment SGI399799[] = {{&S100416[38], 6}, {NULL}};
const Segment SGI399800[] = {{&S100428[74], 4}, {NULL}};
const Segment SGI399801[] = {{&S100428[36], 18}, {NULL}};
const Segment SGI399802[] = {{&S100430[64], 4}, {NULL}};
const Segment SGI399803[] = {{&S100430[38], 6}, {NULL}};
const Segment SGI399804[] = {{&S100435[62], 4}, {NULL}};
const Segment SGI399805[] = {{&S100435[38], 4}, {NULL}};
const Segment SGI399806[] = {{&S100440[63], 4}, {NULL}};
const Segment SGI399807[] = {{&S100440[38], 5}, {NULL}};
const Segment SGI399808[] = {{&S100450[64], 4}, {NULL}};
const Segment SGI399809[] = {{&S100450[38], 6}, {NULL}};
const Segment SGI399810[] = {{&S100477[66], 4}, {NULL}};
const Segment SGI399811[] = {{&S100477[38], 8}, {NULL}};
const Segment SGI399812[] = {{&S100480[68], 4}, {NULL}};
const Segment SGI399813[] = {{&S100480[38], 10}, {NULL}};
const Segment SGI399814[] = {{&S100486[70], 4}, {NULL}};
const Segment SGI399815[] = {{&S100486[38], 12}, {NULL}};
const Segment SGI399816[] = {{&S100490[67], 4}, {NULL}};
const Segment SGI399817[] = {{&S100490[38], 9}, {NULL}};
const Segment SGI399818[] = {{&S100505[63], 4}, {NULL}};
const Segment SGI399819[] = {{&S100505[38], 5}, {NULL}};
const Segment SGI399820[] = {{&S100506[74], 4}, {NULL}};
const Segment SGI399821[] = {{&S100506[36], 18}, {NULL}};
const Segment SGI399822[] = {{&S100520[67], 4}, {NULL}};
const Segment SGI399823[] = {{&S100520[38], 9}, {NULL}};
const Segment SGI399824[] = {{&S100532[64], 4}, {NULL}};
const Segment SGI399825[] = {{&S100532[38], 6}, {NULL}};
const Segment SGI399826[] = {{&S100557[69], 4}, {NULL}};
const Segment SGI399827[] = {{&S100557[38], 11}, {NULL}};
const Segment SGI399828[] = {{&S100559[64], 4}, {NULL}};
const Segment SGI399829[] = {{&S100559[38], 6}, {NULL}};
const Segment SGI399830[] = {{&S100564[64], 4}, {NULL}};
const Segment SGI399831[] = {{&S100564[38], 6}, {NULL}};
const Segment SGI399832[] = {{&S100568[67], 4}, {NULL}};
const Segment SGI399833[] = {{&S100568[38], 9}, {NULL}};
const Segment SGI399834[] = {{&S100581[64], 4}, {NULL}};
const Segment SGI399835[] = {{&S100581[38], 6}, {NULL}};
const Segment SGI399836[] = {{&S100616[73], 4}, {NULL}};
const Segment SGI399837[] = {{&S100616[38], 15}, {NULL}};
const Segment SGI399838[] = {{&S100625[70], 4}, {NULL}};
const Segment SGI399839[] = {{&S100625[38], 12}, {NULL}};
const Segment SGI399840[] = {{&S100627[72], 4}, {NULL}};
const Segment SGI399841[] = {{&S100627[38], 14}, {NULL}};
const Segment SGI399842[] = {{&S60332[44], 7}, {NULL}};
const Segment SGI399843[] = {{&S60332[44], 19}, {NULL}};
const Segment SGI399844[] = {{&S60332[30], 6}, {NULL}};
const Segment SGI399845[] = {{&S60332[111], 3}, {NULL}};
const Segment SGI399846[] = {{&S100632[72], 4}, {NULL}};
const Segment SGI399847[] = {{&S100632[38], 14}, {NULL}};
const Segment SGI399848[] = {{&S60333[42], 10}, {NULL}};
const Segment SGI399849[] = {{&S60333[42], 22}, {NULL}};
const Segment SGI399850[] = {{&S60333[30], 4}, {NULL}};
const Segment SGI399851[] = {{&S60333[113], 3}, {NULL}};
const Segment SGI399852[] = {{&S60334[42], 10}, {NULL}};
const Segment SGI399853[] = {{&S60334[42], 22}, {NULL}};
const Segment SGI399854[] = {{&S60334[30], 4}, {NULL}};
const Segment SGI399855[] = {{&S100636[67], 4}, {NULL}};
const Segment SGI399856[] = {{&S100636[38], 9}, {NULL}};
const Segment SGI399857[] = {{&S60334[113], 3}, {NULL}};
const Segment SGI399858[] = {{&S100638[65], 4}, {NULL}};
const Segment SGI399859[] = {{&S60335[42], 10}, {NULL}};
const Segment SGI399860[] = {{&S100638[36], 9}, {NULL}};
const Segment SGI399861[] = {{&S60335[42], 22}, {NULL}};
const Segment SGI399862[] = {{&S60335[30], 4}, {NULL}};
const Segment SGI399863[] = {{&S60335[113], 3}, {NULL}};
const Segment SGI399864[] = {{&S60336[36], 10}, {NULL}};
const Segment SGI399865[] = {{&S60336[31], 4}, {NULL}};
const Segment SGI399866[] = {{&S60337[42], 10}, {NULL}};
const Segment SGI399867[] = {{&S60337[42], 22}, {NULL}};
const Segment SGI399868[] = {{&S60337[30], 4}, {NULL}};
const Segment SGI399869[] = {{&S60337[113], 3}, {NULL}};
const Segment SGI399870[] = {{&S60338[42], 18}, {NULL}};
const Segment SGI399871[] = {{&S60338[42], 30}, {NULL}};
const Segment SGI399872[] = {{&S60338[30], 4}, {NULL}};
const Segment SGI399873[] = {{&S60338[121], 3}, {NULL}};
const Segment SGI399874[] = {{&S100642[64], 4}, {NULL}};
const Segment SGI399875[] = {{&S100642[38], 6}, {NULL}};
const Segment SGI399876[] = {{&S60339[42], 10}, {NULL}};
const Segment SGI399877[] = {{&S60339[42], 22}, {NULL}};
const Segment SGI399878[] = {{&S60339[30], 4}, {NULL}};
const Segment SGI399879[] = {{&S60339[113], 3}, {NULL}};
const Segment SGI399880[] = {{&S60340[36], 10}, {NULL}};
const Segment SGI399881[] = {{&S60340[31], 4}, {NULL}};
const Segment SGI399882[] = {{&S100644[64], 4}, {NULL}};
const Segment SGI399883[] = {{&S100644[38], 6}, {NULL}};
const Segment SGI399884[] = {{&S100645[67], 4}, {NULL}};
const Segment SGI399885[] = {{&S100645[38], 9}, {NULL}};
const Segment SGI399886[] = {{&S100646[67], 4}, {NULL}};
const Segment SGI399887[] = {{&S100646[38], 9}, {NULL}};
const Segment SGI399888[] = {{&S100651[70], 4}, {NULL}};
const Segment SGI399889[] = {{&S100651[38], 12}, {NULL}};
const Segment SGI399890[] = {{&S100653[67], 4}, {NULL}};
const Segment SGI399891[] = {{&S100653[38], 9}, {NULL}};
const Segment SGI399892[] = {{&S100661[66], 4}, {NULL}};
const Segment SGI399893[] = {{&S100661[38], 8}, {NULL}};
const Segment SGI399894[] = {{&S100664[67], 4}, {NULL}};
const Segment SGI399895[] = {{&S100664[38], 9}, {NULL}};
const Segment SGI399896[] = {{&S100667[74], 4}, {NULL}};
const Segment SGI399897[] = {{&S100667[36], 18}, {NULL}};
const Segment SGI399898[] = {{&S100672[67], 4}, {NULL}};
const Segment SGI399899[] = {{&S100672[38], 9}, {NULL}};
const Segment SGI399900[] = {{&S100674[69], 4}, {NULL}};
const Segment SGI399901[] = {{&S100674[38], 11}, {NULL}};
const Segment SGI399902[] = {{&S100677[67], 4}, {NULL}};
const Segment SGI399903[] = {{&S100677[38], 9}, {NULL}};
const Segment SGI399904[] = {{&S100678[67], 4}, {NULL}};
const Segment SGI399905[] = {{&S100678[38], 9}, {NULL}};
const Segment SGI399906[] = {{&S100680[68], 4}, {NULL}};
const Segment SGI399907[] = {{&S100680[38], 10}, {NULL}};
const Segment SGI399908[] = {{&S100682[67], 4}, {NULL}};
const Segment SGI399909[] = {{&S100682[38], 9}, {NULL}};
const Segment SGI399910[] = {{&S100685[67], 4}, {NULL}};
const Segment SGI399911[] = {{&S100685[38], 9}, {NULL}};
const Segment SGI399912[] = {{&S21519[11], 8}, {NULL}};
const Segment SGI399913[] = {{&S21519[37], 6}, {NULL}};
const Segment SGI399914[] = {{&S100820[48], 3}, {NULL}};
const Segment SGI399915[] = {{&S100820[28], 9}, {NULL}};
const Segment SGI399916[] = {{&S100833[48], 3}, {NULL}};
const Segment SGI399917[] = {{&S100833[28], 9}, {NULL}};
const Segment SGI399918[] = {{&S100864[64], 4}, {NULL}};
const Segment SGI399919[] = {{&S100864[36], 8}, {NULL}};
const Segment SGI399920[] = {{&S100865[64], 4}, {NULL}};
const Segment SGI399921[] = {{&S100865[36], 8}, {NULL}};
const Segment SGI399922[] = {{&S100869[65], 4}, {NULL}};
const Segment SGI399923[] = {{&S100869[36], 9}, {NULL}};
const Segment SGI399924[] = {{&S100871[65], 4}, {NULL}};
const Segment SGI399925[] = {{&S100871[36], 9}, {NULL}};
const Segment SGI399926[] = {{&S100901[85], 4}, {NULL}};
const Segment SGI399927[] = {{&S100901[28], 7}, {NULL}};
const Segment SGI399928[] = {{&S100907[68], 4}, {NULL}};
const Segment SGI399929[] = {{&S100907[35], 13}, {NULL}};
const Segment SGI399930[] = {{&S100941[74], 4}, {NULL}};
const Segment SGI399931[] = {{&S100941[36], 18}, {NULL}};
const Segment SGI399932[] = {{&S101061[61], 4}, {NULL}};
const Segment SGI399933[] = {{&S101061[35], 6}, {NULL}};
const Segment SGI399934[] = {{&S101079[74], 4}, {NULL}};
const Segment SGI399935[] = {{&S101079[36], 18}, {NULL}};
const Segment SGI399936[] = {{&S101080[74], 4}, {NULL}};
const Segment SGI399937[] = {{&S101080[36], 18}, {NULL}};
const Segment SGI399938[] = {{&S101087[57], 3}, {NULL}};
const Segment SGI399939[] = {{&S101087[28], 18}, {NULL}};
const Segment SGI399940[] = {{&S101102[72], 4}, {NULL}};
const Segment SGI399941[] = {{&S101102[35], 17}, {NULL}};
const Segment SGI399942[] = {{&S101122[69], 4}, {NULL}};
const Segment SGI399943[] = {{&S101122[36], 13}, {NULL}};
const Segment SGI399944[] = {{&S101125[64], 4}, {NULL}};
const Segment SGI399945[] = {{&S101125[36], 8}, {NULL}};
const Segment SGI399946[] = {{&S101262[64], 4}, {NULL}};
const Segment SGI399947[] = {{&S101262[36], 8}, {NULL}};
const Segment SGI399948[] = {{&S101277[65], 4}, {NULL}};
const Segment SGI399949[] = {{&S101277[36], 9}, {NULL}};
const Segment SGI399950[] = {{&S101385[64], 4}, {NULL}};
const Segment SGI399951[] = {{&S101385[35], 9}, {NULL}};
const Segment SGI399952[] = {{&S101410[74], 4}, {NULL}};
const Segment SGI399953[] = {{&S101410[36], 18}, {NULL}};
const Segment SGI399954[] = {{&S101597[66], 4}, {NULL}};
const Segment SGI399955[] = {{&S101597[36], 10}, {NULL}};
const Segment SGI399956[] = {{&S101743[65], 4}, {NULL}};
const Segment SGI399957[] = {{&S101743[36], 9}, {NULL}};
const Segment SGI399958[] = {{&S101780[57], 3}, {NULL}};
const Segment SGI399959[] = {{&S101780[28], 18}, {NULL}};
const Segment SGI399960[] = {{&S101786[64], 4}, {NULL}};
const Segment SGI399961[] = {{&S101786[35], 9}, {NULL}};
const Segment SGI399962[] = {{&S101907[69], 4}, {NULL}};
const Segment SGI399963[] = {{&S101908[72], 4}, {NULL}};
const Segment SGI399964[] = {{&S101908[36], 16}, {NULL}};
const Segment SGI399965[] = {{&S101909[66], 4}, {NULL}};
const Segment SGI399966[] = {{&S101909[36], 10}, {NULL}};
const Segment SGI399967[] = {{&S101910[66], 4}, {NULL}};
const Segment SGI399968[] = {{&S101910[36], 10}, {NULL}};
const Segment SGI399969[] = {{&S61725[32], 14}, {NULL}};
const Segment SGI399970[] = {{&S61725[27], 4}, {NULL}};
const Segment SGI399971[] = {{&S61725[99], 9}, {NULL}};
const Segment SGI399972[] = {{&S101953[64], 4}, {NULL}};
const Segment SGI399973[] = {{&S101953[36], 8}, {NULL}};
const Segment SGI399974[] = {{&S61752[32], 9}, {NULL}};
const Segment SGI399975[] = {{&S61752[27], 4}, {NULL}};
const Segment SGI399976[] = {{&S61752[94], 9}, {NULL}};
const Segment SGI399977[] = {{&S61752[135], 8}, {NULL}};
const Segment SGI399978[] = {{&S102085[73], 4}, {NULL}};
const Segment SGI399979[] = {{&S102085[35], 18}, {NULL}};
const Segment SGI399980[] = {{&S102090[63], 4}, {NULL}};
const Segment SGI399981[] = {{&S102090[35], 8}, {NULL}};
const Segment SGI399982[] = {{&S102093[57], 3}, {NULL}};
const Segment SGI399983[] = {{&S102093[28], 18}, {NULL}};
const Segment SGI399984[] = {{&S102094[64], 4}, {NULL}};
const Segment SGI399985[] = {{&S102094[36], 8}, {NULL}};
const Segment SGI399986[] = {{&S61916[42], 11}, {NULL}};
const Segment SGI399987[] = {{&S61916[42], 28}, {NULL}};
const Segment SGI399988[] = {{&S61916[30], 4}, {NULL}};
const Segment SGI399989[] = {{&S61916[119], 3}, {NULL}};
const Segment SGI399990[] = {{&S61917[42], 11}, {NULL}};
const Segment SGI399991[] = {{&S61917[42], 28}, {NULL}};
const Segment SGI399992[] = {{&S61917[30], 4}, {NULL}};
const Segment SGI399993[] = {{&S61917[119], 3}, {NULL}};
const Segment SGI399994[] = {{&S61918[42], 11}, {NULL}};
const Segment SGI399995[] = {{&S61918[42], 28}, {NULL}};
const Segment SGI399996[] = {{&S61918[30], 4}, {NULL}};
const Segment SGI399997[] = {{&S61918[119], 3}, {NULL}};
const Segment SGI399998[] = {{&S61919[36], 11}, {NULL}};
const Segment SGI399999[] = {{&S61919[31], 4}, {NULL}};
const Segment SGI400000[] = {{&S61920[42], 11}, {NULL}};
const Segment SGI400001[] = {{&S61920[42], 28}, {NULL}};
const Segment SGI400002[] = {{&S61920[30], 4}, {NULL}};
const Segment SGI400003[] = {{&S61920[119], 3}, {NULL}};
const Segment SGI400004[] = {{&S102272[64], 4}, {NULL}};
const Segment SGI400005[] = {{&S102272[36], 8}, {NULL}};
const Segment SGI400006[] = {{&S102309[65], 4}, {NULL}};
const Segment SGI400007[] = {{&S102309[36], 9}, {NULL}};
const Segment SGI400008[] = {{&S102310[65], 4}, {NULL}};
const Segment SGI400009[] = {{&S102310[36], 9}, {NULL}};
const Segment SGI400010[] = {{&S102323[64], 4}, {NULL}};
const Segment SGI400011[] = {{&S102323[36], 8}, {NULL}};
const Segment SGI400012[] = {{&S102530[76], 4}, {NULL}};
const Segment SGI400013[] = {{&S102530[36], 19}, {NULL}};
const Segment SGI400014[] = {{&S102534[64], 4}, {NULL}};
const Segment SGI400015[] = {{&S102534[36], 8}, {NULL}};
const Segment SGI400016[] = {{&S102535[64], 4}, {NULL}};
const Segment SGI400017[] = {{&S102535[36], 8}, {NULL}};
const Segment SGI400018[] = {{&S102726[65], 4}, {NULL}};
const Segment SGI400019[] = {{&S102726[36], 9}, {NULL}};
const Segment SGI400020[] = {{&S102730[65], 4}, {NULL}};
const Segment SGI400021[] = {{&S102730[36], 9}, {NULL}};
const Segment SGI400022[] = {{&S102866[25], 8}, {NULL}};
const Segment SGI400023[] = {{&S102889[61], 4}, {NULL}};
const Segment SGI400024[] = {{&S102890[61], 4}, {NULL}};
const Segment SGI400025[] = {{&S102891[57], 3}, {NULL}};
const Segment SGI400026[] = {{&S102891[28], 18}, {NULL}};
const Segment SGI400027[] = {{&S102892[63], 4}, {NULL}};
const Segment SGI400028[] = {{&S102892[35], 8}, {NULL}};
const Segment SGI400029[] = {{&S102894[66], 4}, {NULL}};
const Segment SGI400030[] = {{&S102894[36], 10}, {NULL}};
const Segment SGI400031[] = {{&S102942[65], 4}, {NULL}};
const Segment SGI400032[] = {{&S102942[36], 9}, {NULL}};
const Segment SGI400033[] = {{&S103190[64], 4}, {NULL}};
const Segment SGI400034[] = {{&S103190[35], 9}, {NULL}};
const Segment SGI400035[] = {{&S103192[72], 4}, {NULL}};
const Segment SGI400036[] = {{&S103192[36], 16}, {NULL}};
const Segment SGI400037[] = {{&S103201[73], 4}, {NULL}};
const Segment SGI400038[] = {{&S103201[36], 17}, {NULL}};
const Segment SGI400039[] = {{&S103202[64], 4}, {NULL}};
const Segment SGI400040[] = {{&S103202[36], 8}, {NULL}};
const Segment SGI400041[] = {{&S103203[64], 4}, {NULL}};
const Segment SGI400042[] = {{&S103203[36], 8}, {NULL}};
const Segment SGI400043[] = {{&S103205[62], 4}, {NULL}};
const Segment SGI400044[] = {{&S103205[36], 6}, {NULL}};
const Segment SGI400045[] = {{&S103206[72], 4}, {NULL}};
const Segment SGI400046[] = {{&S103206[36], 16}, {NULL}};
const Segment SGI400047[] = {{&S103207[64], 4}, {NULL}};
const Segment SGI400048[] = {{&S103207[36], 8}, {NULL}};
const Segment SGI400049[] = {{&S103217[65], 4}, {NULL}};
const Segment SGI400050[] = {{&S103217[36], 9}, {NULL}};
const Segment SGI400051[] = {{&S103222[64], 4}, {NULL}};
const Segment SGI400052[] = {{&S103222[36], 8}, {NULL}};
const Segment SGI400053[] = {{&S103415[63], 4}, {NULL}};
const Segment SGI400054[] = {{&S103415[35], 8}, {NULL}};
const Segment SGI400055[] = {{&S103417[63], 4}, {NULL}};
const Segment SGI400056[] = {{&S103417[35], 8}, {NULL}};
const Segment SGI400057[] = {{&S103419[63], 4}, {NULL}};
const Segment SGI400058[] = {{&S103419[35], 8}, {NULL}};
const Segment SGI400059[] = {{&S103467[59], 4}, {NULL}};
const Segment SGI400060[] = {{&S103468[71], 4}, {NULL}};
const Segment SGI400061[] = {{&S103468[35], 16}, {NULL}};
const Segment SGI400062[] = {{&S103470[63], 4}, {NULL}};
const Segment SGI400063[] = {{&S103470[35], 8}, {NULL}};
const Segment SGI400064[] = {{&S103471[63], 4}, {NULL}};
const Segment SGI400065[] = {{&S103471[35], 8}, {NULL}};
const Segment SGI400066[] = {{&S103472[63], 4}, {NULL}};
const Segment SGI400067[] = {{&S103472[35], 8}, {NULL}};
const Segment SGI400068[] = {{&S103473[60], 4}, {NULL}};
const Segment SGI400069[] = {{&S103474[65], 4}, {NULL}};
const Segment SGI400070[] = {{&S103474[36], 9}, {NULL}};
const Segment SGI400071[] = {{&S103475[65], 4}, {NULL}};
const Segment SGI400072[] = {{&S103475[36], 9}, {NULL}};
const Segment SGI400073[] = {{&S103480[64], 4}, {NULL}};
const Segment SGI400074[] = {{&S103480[36], 8}, {NULL}};
const Segment SGI400075[] = {{&S103484[65], 4}, {NULL}};
const Segment SGI400076[] = {{&S103484[36], 9}, {NULL}};
const Segment SGI400077[] = {{&S103485[64], 4}, {NULL}};
const Segment SGI400078[] = {{&S103485[36], 8}, {NULL}};
const Segment SGI400079[] = {{&S103503[64], 4}, {NULL}};
const Segment SGI400080[] = {{&S103503[36], 8}, {NULL}};
const Segment SGI400081[] = {{&S103507[66], 4}, {NULL}};
const Segment SGI400082[] = {{&S103507[36], 10}, {NULL}};
const Segment SGI400083[] = {{&S103731[63], 4}, {NULL}};
const Segment SGI400084[] = {{&S103731[35], 8}, {NULL}};
const Segment SGI400085[] = {{&S103733[73], 4}, {NULL}};
const Segment SGI400086[] = {{&S103733[35], 18}, {NULL}};
const Segment SGI400087[] = {{&S103735[63], 4}, {NULL}};
const Segment SGI400088[] = {{&S103735[35], 8}, {NULL}};
const Segment SGI400089[] = {{&S63629[42], 9}, {NULL}};
const Segment SGI400090[] = {{&S63629[42], 21}, {NULL}};
const Segment SGI400091[] = {{&S63629[30], 4}, {NULL}};
const Segment SGI400092[] = {{&S63629[112], 3}, {NULL}};
const Segment SGI400093[] = {{&S63629[140], 3}, {NULL}};
const Segment SGI400094[] = {{&S63630[42], 9}, {NULL}};
const Segment SGI400095[] = {{&S63630[42], 21}, {NULL}};
const Segment SGI400096[] = {{&S63630[30], 4}, {NULL}};
const Segment SGI400097[] = {{&S63630[112], 3}, {NULL}};
const Segment SGI400098[] = {{&S63630[140], 3}, {NULL}};
const Segment SGI400099[] = {{&S63631[42], 9}, {NULL}};
const Segment SGI400100[] = {{&S63631[42], 21}, {NULL}};
const Segment SGI400101[] = {{&S63631[30], 4}, {NULL}};
const Segment SGI400102[] = {{&S63631[112], 3}, {NULL}};
const Segment SGI400103[] = {{&S63631[140], 3}, {NULL}};
const Segment SGI400104[] = {{&S63632[42], 9}, {NULL}};
const Segment SGI400105[] = {{&S63632[42], 21}, {NULL}};
const Segment SGI400106[] = {{&S63632[30], 4}, {NULL}};
const Segment SGI400107[] = {{&S63632[112], 3}, {NULL}};
const Segment SGI400108[] = {{&S63632[140], 3}, {NULL}};
const Segment SGI400109[] = {{&S63633[32], 5}, {NULL}};
const Segment SGI400110[] = {{&S63633[27], 4}, {NULL}};
const Segment SGI400111[] = {{&S63633[94], 9}, {NULL}};
const Segment SGI400112[] = {{&S63634[32], 9}, {NULL}};
const Segment SGI400113[] = {{&S63634[27], 4}, {NULL}};
const Segment SGI400114[] = {{&S63634[94], 9}, {NULL}};
const Segment SGI400115[] = {{&S63635[32], 8}, {NULL}};
const Segment SGI400116[] = {{&S63635[27], 4}, {NULL}};
const Segment SGI400117[] = {{&S63635[93], 9}, {NULL}};
const Segment SGI400118[] = {{&S103796[65], 4}, {NULL}};
const Segment SGI400119[] = {{&S103796[35], 10}, {NULL}};
const Segment SGI400120[] = {{&S103798[64], 4}, {NULL}};
const Segment SGI400121[] = {{&S103798[36], 8}, {NULL}};
const Segment SGI400122[] = {{&S103801[65], 4}, {NULL}};
const Segment SGI400123[] = {{&S103801[35], 10}, {NULL}};
const Segment SGI400124[] = {{&S103803[64], 4}, {NULL}};
const Segment SGI400125[] = {{&S103803[35], 9}, {NULL}};
const Segment SGI400126[] = {{&S103804[66], 4}, {NULL}};
const Segment SGI400127[] = {{&S103804[36], 10}, {NULL}};
const Segment SGI400128[] = {{&S103805[66], 4}, {NULL}};
const Segment SGI400129[] = {{&S103805[36], 10}, {NULL}};
const Segment SGI400130[] = {{&S103806[63], 4}, {NULL}};
const Segment SGI400131[] = {{&S103808[65], 4}, {NULL}};
const Segment SGI400132[] = {{&S103808[36], 9}, {NULL}};
const Segment SGI400133[] = {{&S103811[61], 4}, {NULL}};
const Segment SGI400134[] = {{&S103811[35], 6}, {NULL}};
const Segment SGI400135[] = {{&S103812[69], 4}, {NULL}};
const Segment SGI400136[] = {{&S103812[35], 14}, {NULL}};
const Segment SGI400137[] = {{&S103816[65], 4}, {NULL}};
const Segment SGI400138[] = {{&S103816[36], 9}, {NULL}};
const Segment SGI400139[] = {{&S103817[65], 4}, {NULL}};
const Segment SGI400140[] = {{&S103817[36], 9}, {NULL}};
const Segment SGI400141[] = {{&S103829[64], 4}, {NULL}};
const Segment SGI400142[] = {{&S103829[36], 8}, {NULL}};
const Segment SGI400143[] = {{&S103832[61], 4}, {NULL}};
const Segment SGI400144[] = {{&S103832[36], 5}, {NULL}};
const Segment SGI400145[] = {{&S104021[64], 4}, {NULL}};
const Segment SGI400146[] = {{&S104021[35], 9}, {NULL}};
const Segment SGI400147[] = {{&S104024[65], 4}, {NULL}};
const Segment SGI400148[] = {{&S104024[35], 10}, {NULL}};
const Segment SGI400149[] = {{&S104026[63], 4}, {NULL}};
const Segment SGI400150[] = {{&S104026[35], 8}, {NULL}};
const Segment SGI400151[] = {{&S104117[64], 4}, {NULL}};
const Segment SGI400152[] = {{&S104117[36], 8}, {NULL}};
const Segment SGI400153[] = {{&S104120[71], 4}, {NULL}};
const Segment SGI400154[] = {{&S104120[35], 16}, {NULL}};
const Segment SGI400155[] = {{&S104124[64], 4}, {NULL}};
const Segment SGI400156[] = {{&S104124[36], 8}, {NULL}};
const Segment SGI400157[] = {{&S104125[64], 4}, {NULL}};
const Segment SGI400158[] = {{&S104125[36], 8}, {NULL}};
const Segment SGI400159[] = {{&S104126[64], 4}, {NULL}};
const Segment SGI400160[] = {{&S104126[36], 8}, {NULL}};
const Segment SGI400161[] = {{&S104128[66], 4}, {NULL}};
const Segment SGI400162[] = {{&S104128[36], 10}, {NULL}};
const Segment SGI400163[] = {{&S104134[64], 4}, {NULL}};
const Segment SGI400164[] = {{&S104134[36], 8}, {NULL}};
const Segment SGI400165[] = {{&S104140[72], 4}, {NULL}};
const Segment SGI400166[] = {{&S104140[36], 16}, {NULL}};
const Segment SGI400167[] = {{&S104364[57], 3}, {NULL}};
const Segment SGI400168[] = {{&S104364[28], 18}, {NULL}};
const Segment SGI400169[] = {{&S104398[64], 4}, {NULL}};
const Segment SGI400170[] = {{&S104398[35], 9}, {NULL}};
const Segment SGI400171[] = {{&S104400[64], 4}, {NULL}};
const Segment SGI400172[] = {{&S104400[35], 9}, {NULL}};
const Segment SGI400173[] = {{&S104525[66], 4}, {NULL}};
const Segment SGI400174[] = {{&S104525[36], 10}, {NULL}};
const Segment SGI400175[] = {{&S104530[62], 4}, {NULL}};
const Segment SGI400176[] = {{&S104530[36], 6}, {NULL}};
const Segment SGI400177[] = {{&S104850[61], 4}, {NULL}};
const Segment SGI400178[] = {{&S104850[35], 6}, {NULL}};
const Segment SGI400179[] = {{&S104859[57], 3}, {NULL}};
const Segment SGI400180[] = {{&S104859[28], 18}, {NULL}};
const Segment SGI400181[] = {{&S104861[62], 4}, {NULL}};
const Segment SGI400182[] = {{&S104861[36], 6}, {NULL}};
const Segment SGI400183[] = {{&S64880[32], 10}, {NULL}};
const Segment SGI400184[] = {{&S64880[27], 4}, {NULL}};
const Segment SGI400185[] = {{&S64880[95], 9}, {NULL}};
const Segment SGI400186[] = {{&S64881[32], 10}, {NULL}};
const Segment SGI400187[] = {{&S64881[27], 4}, {NULL}};
const Segment SGI400188[] = {{&S64881[95], 9}, {NULL}};
const Segment SGI400189[] = {{&S64882[32], 10}, {NULL}};
const Segment SGI400190[] = {{&S64882[27], 4}, {NULL}};
const Segment SGI400191[] = {{&S64882[95], 9}, {NULL}};
const Segment SGI400192[] = {{&S105026[87], 4}, {NULL}};
const Segment SGI400193[] = {{&S105026[28], 9}, {NULL}};
const Segment SGI400194[] = {{&S105029[62], 4}, {NULL}};
const Segment SGI400195[] = {{&S105029[36], 6}, {NULL}};
const Segment SGI400196[] = {{&S105030[66], 4}, {NULL}};
const Segment SGI400197[] = {{&S105030[36], 10}, {NULL}};
const Segment SGI400198[] = {{&S105034[64], 4}, {NULL}};
const Segment SGI400199[] = {{&S105034[35], 9}, {NULL}};
const Segment SGI400200[] = {{&S105035[65], 4}, {NULL}};
const Segment SGI400201[] = {{&S105035[35], 10}, {NULL}};
const Segment SGI400202[] = {{&S105037[47], 3}, {NULL}};
const Segment SGI400203[] = {{&S105037[28], 8}, {NULL}};
const Segment SGI400204[] = {{&S105039[62], 4}, {NULL}};
const Segment SGI400205[] = {{&S105040[72], 4}, {NULL}};
const Segment SGI400206[] = {{&S105040[35], 17}, {NULL}};
const Segment SGI400207[] = {{&S105043[62], 4}, {NULL}};
const Segment SGI400208[] = {{&S105043[36], 6}, {NULL}};
const Segment SGI400209[] = {{&S105044[66], 4}, {NULL}};
const Segment SGI400210[] = {{&S105044[36], 10}, {NULL}};
const Segment SGI400211[] = {{&S105066[66], 4}, {NULL}};
const Segment SGI400212[] = {{&S105066[36], 10}, {NULL}};
const Segment SGI400213[] = {{&S105089[66], 4}, {NULL}};
const Segment SGI400214[] = {{&S105089[36], 10}, {NULL}};
const Segment SGI400215[] = {{&S65138[42], 10}, {NULL}};
const Segment SGI400216[] = {{&S65138[42], 22}, {NULL}};
const Segment SGI400217[] = {{&S65138[30], 4}, {NULL}};
const Segment SGI400218[] = {{&S65138[113], 3}, {NULL}};
const Segment SGI400219[] = {{&S65139[42], 18}, {NULL}};
const Segment SGI400220[] = {{&S65139[42], 30}, {NULL}};
const Segment SGI400221[] = {{&S65139[30], 4}, {NULL}};
const Segment SGI400222[] = {{&S65139[121], 3}, {NULL}};
const Segment SGI400223[] = {{&S65140[42], 10}, {NULL}};
const Segment SGI400224[] = {{&S65140[42], 22}, {NULL}};
const Segment SGI400225[] = {{&S65140[30], 4}, {NULL}};
const Segment SGI400226[] = {{&S65140[113], 3}, {NULL}};
const Segment SGI400227[] = {{&S65141[42], 10}, {NULL}};
const Segment SGI400228[] = {{&S65141[42], 22}, {NULL}};
const Segment SGI400229[] = {{&S65141[30], 4}, {NULL}};
const Segment SGI400230[] = {{&S65141[113], 3}, {NULL}};
const Segment SGI400231[] = {{&S65142[42], 18}, {NULL}};
const Segment SGI400232[] = {{&S65142[42], 30}, {NULL}};
const Segment SGI400233[] = {{&S65142[30], 4}, {NULL}};
const Segment SGI400234[] = {{&S65142[121], 3}, {NULL}};
const Segment SGI400235[] = {{&S65143[42], 10}, {NULL}};
const Segment SGI400236[] = {{&S65143[42], 22}, {NULL}};
const Segment SGI400237[] = {{&S65143[30], 4}, {NULL}};
const Segment SGI400238[] = {{&S65143[113], 3}, {NULL}};
const Segment SGI400239[] = {{&S65144[36], 18}, {NULL}};
const Segment SGI400240[] = {{&S65144[31], 4}, {NULL}};
const Segment SGI400241[] = {{&S65145[42], 10}, {NULL}};
const Segment SGI400242[] = {{&S65145[42], 22}, {NULL}};
const Segment SGI400243[] = {{&S65145[30], 4}, {NULL}};
const Segment SGI400244[] = {{&S65145[113], 3}, {NULL}};
const Segment SGI400245[] = {{&S105561[87], 4}, {NULL}};
const Segment SGI400246[] = {{&S105561[28], 9}, {NULL}};
const Segment SGI400247[] = {{&S105562[61], 4}, {NULL}};
const Segment SGI400248[] = {{&S105562[35], 6}, {NULL}};
const Segment SGI400249[] = {{&S105564[88], 4}, {NULL}};
const Segment SGI400250[] = {{&S105564[28], 10}, {NULL}};
const Segment SGI400251[] = {{&S105578[75], 4}, {NULL}};
const Segment SGI400252[] = {{&S105578[35], 19}, {NULL}};
const Segment SGI400253[] = {{&S65645[32], 9}, {NULL}};
const Segment SGI400254[] = {{&S65645[27], 4}, {NULL}};
const Segment SGI400255[] = {{&S65645[94], 9}, {NULL}};
const Segment SGI400256[] = {{&S65645[128], 8}, {NULL}};
const Segment SGI400257[] = {{&S65800[32], 10}, {NULL}};
const Segment SGI400258[] = {{&S65800[27], 4}, {NULL}};
const Segment SGI400259[] = {{&S65800[95], 9}, {NULL}};
const Segment SGI400260[] = {{&S65801[32], 10}, {NULL}};
const Segment SGI400261[] = {{&S65801[27], 4}, {NULL}};
const Segment SGI400262[] = {{&S65801[95], 9}, {NULL}};
const Segment SGI400263[] = {{&S105904[68], 4}, {NULL}};
const Segment SGI400264[] = {{&S105904[38], 10}, {NULL}};
const Segment SGI400265[] = {{&S105906[63], 4}, {NULL}};
const Segment SGI400266[] = {{&S105906[35], 8}, {NULL}};
const Segment SGI400267[] = {{&S105917[65], 4}, {NULL}};
const Segment SGI400268[] = {{&S105917[35], 10}, {NULL}};
const Segment SGI400269[] = {{&S105919[66], 4}, {NULL}};
const Segment SGI400270[] = {{&S105919[36], 10}, {NULL}};
const Segment SGI400271[] = {{&S105928[65], 4}, {NULL}};
const Segment SGI400272[] = {{&S105928[36], 9}, {NULL}};
const Segment SGI400273[] = {{&S105992[65], 4}, {NULL}};
const Segment SGI400274[] = {{&S105992[36], 9}, {NULL}};
const Segment SGI400275[] = {{&S105996[65], 4}, {NULL}};
const Segment SGI400276[] = {{&S105996[36], 9}, {NULL}};
const Segment SGI400277[] = {{&S105999[66], 4}, {NULL}};
const Segment SGI400278[] = {{&S105999[36], 10}, {NULL}};
const Segment SGI400279[] = {{&S106035[45], 3}, {NULL}};
const Segment SGI400280[] = {{&S106037[48], 3}, {NULL}};
const Segment SGI400281[] = {{&S106037[28], 9}, {NULL}};
const Segment SGI400282[] = {{&S106038[45], 3}, {NULL}};
const Segment SGI400283[] = {{&S66120[42], 10}, {NULL}};
const Segment SGI400284[] = {{&S66120[42], 22}, {NULL}};
const Segment SGI400285[] = {{&S66120[30], 4}, {NULL}};
const Segment SGI400286[] = {{&S66120[113], 3}, {NULL}};
const Segment SGI400287[] = {{&S66120[138], 8}, {NULL}};
const Segment SGI400288[] = {{&S66121[42], 10}, {NULL}};
const Segment SGI400289[] = {{&S66121[42], 22}, {NULL}};
const Segment SGI400290[] = {{&S66121[30], 4}, {NULL}};
const Segment SGI400291[] = {{&S66121[113], 3}, {NULL}};
const Segment SGI400292[] = {{&S66121[138], 8}, {NULL}};
const Segment SGI400293[] = {{&S66122[42], 10}, {NULL}};
const Segment SGI400294[] = {{&S66122[42], 22}, {NULL}};
const Segment SGI400295[] = {{&S66122[30], 4}, {NULL}};
const Segment SGI400296[] = {{&S66122[113], 3}, {NULL}};
const Segment SGI400297[] = {{&S66122[138], 8}, {NULL}};
const Segment SGI400298[] = {{&S66123[42], 10}, {NULL}};
const Segment SGI400299[] = {{&S66123[42], 22}, {NULL}};
const Segment SGI400300[] = {{&S66123[30], 4}, {NULL}};
const Segment SGI400301[] = {{&S66123[113], 3}, {NULL}};
const Segment SGI400302[] = {{&S66123[138], 8}, {NULL}};
const Segment SGI400303[] = {{&S66124[42], 18}, {NULL}};
const Segment SGI400304[] = {{&S66124[42], 30}, {NULL}};
const Segment SGI400305[] = {{&S66124[30], 4}, {NULL}};
const Segment SGI400306[] = {{&S66124[121], 3}, {NULL}};
const Segment SGI400307[] = {{&S66124[146], 8}, {NULL}};
const Segment SGI400308[] = {{&S66125[42], 10}, {NULL}};
const Segment SGI400309[] = {{&S66125[42], 22}, {NULL}};
const Segment SGI400310[] = {{&S66125[30], 4}, {NULL}};
const Segment SGI400311[] = {{&S66125[113], 3}, {NULL}};
const Segment SGI400312[] = {{&S66125[138], 8}, {NULL}};
const Segment SGI400313[] = {{&S66126[42], 10}, {NULL}};
const Segment SGI400314[] = {{&S66126[42], 22}, {NULL}};
const Segment SGI400315[] = {{&S66126[30], 4}, {NULL}};
const Segment SGI400316[] = {{&S66126[113], 3}, {NULL}};
const Segment SGI400317[] = {{&S66126[138], 8}, {NULL}};
const Segment SGI400318[] = {{&S66127[42], 10}, {NULL}};
const Segment SGI400319[] = {{&S66127[42], 22}, {NULL}};
const Segment SGI400320[] = {{&S66127[30], 4}, {NULL}};
const Segment SGI400321[] = {{&S66127[113], 3}, {NULL}};
const Segment SGI400322[] = {{&S66127[138], 8}, {NULL}};
const Segment SGI400323[] = {{&S66128[42], 10}, {NULL}};
const Segment SGI400324[] = {{&S66128[42], 22}, {NULL}};
const Segment SGI400325[] = {{&S66128[30], 4}, {NULL}};
const Segment SGI400326[] = {{&S66128[113], 3}, {NULL}};
const Segment SGI400327[] = {{&S66128[138], 8}, {NULL}};
const Segment SGI400328[] = {{&S66129[42], 10}, {NULL}};
const Segment SGI400329[] = {{&S66129[42], 22}, {NULL}};
const Segment SGI400330[] = {{&S66129[30], 4}, {NULL}};
const Segment SGI400331[] = {{&S66129[113], 3}, {NULL}};
const Segment SGI400332[] = {{&S66129[138], 8}, {NULL}};
const Segment SGI400333[] = {{&S66130[42], 10}, {NULL}};
const Segment SGI400334[] = {{&S66130[42], 22}, {NULL}};
const Segment SGI400335[] = {{&S66130[30], 4}, {NULL}};
const Segment SGI400336[] = {{&S66130[113], 3}, {NULL}};
const Segment SGI400337[] = {{&S66130[138], 8}, {NULL}};
const Segment SGI400338[] = {{&S66131[42], 10}, {NULL}};
const Segment SGI400339[] = {{&S66131[42], 22}, {NULL}};
const Segment SGI400340[] = {{&S66131[30], 4}, {NULL}};
const Segment SGI400341[] = {{&S66131[113], 3}, {NULL}};
const Segment SGI400342[] = {{&S66131[138], 8}, {NULL}};
const Segment SGI400343[] = {{&S66132[42], 18}, {NULL}};
const Segment SGI400344[] = {{&S66132[42], 30}, {NULL}};
const Segment SGI400345[] = {{&S66132[30], 4}, {NULL}};
const Segment SGI400346[] = {{&S66132[121], 3}, {NULL}};
const Segment SGI400347[] = {{&S66132[146], 8}, {NULL}};
const Segment SGI400348[] = {{&S66253[42], 9}, {NULL}};
const Segment SGI400349[] = {{&S66253[42], 21}, {NULL}};
const Segment SGI400350[] = {{&S66253[30], 4}, {NULL}};
const Segment SGI400351[] = {{&S66253[112], 3}, {NULL}};
const Segment SGI400352[] = {{&S66253[137], 8}, {NULL}};
const Segment SGI400353[] = {{&S66254[42], 9}, {NULL}};
const Segment SGI400354[] = {{&S66254[42], 21}, {NULL}};
const Segment SGI400355[] = {{&S66254[30], 4}, {NULL}};
const Segment SGI400356[] = {{&S66254[112], 3}, {NULL}};
const Segment SGI400357[] = {{&S66254[137], 8}, {NULL}};
const Segment SGI400358[] = {{&S66255[42], 9}, {NULL}};
const Segment SGI400359[] = {{&S66255[42], 21}, {NULL}};
const Segment SGI400360[] = {{&S66255[30], 4}, {NULL}};
const Segment SGI400361[] = {{&S66255[112], 3}, {NULL}};
const Segment SGI400362[] = {{&S66255[130], 8}, {NULL}};
const Segment SGI400363[] = {{&S66256[42], 9}, {NULL}};
const Segment SGI400364[] = {{&S66256[42], 21}, {NULL}};
const Segment SGI400365[] = {{&S66256[30], 4}, {NULL}};
const Segment SGI400366[] = {{&S66256[112], 3}, {NULL}};
const Segment SGI400367[] = {{&S66256[137], 8}, {NULL}};
const Segment SGI400368[] = {{&S66471[32], 14}, {NULL}};
const Segment SGI400369[] = {{&S66471[27], 4}, {NULL}};
const Segment SGI400370[] = {{&S66471[99], 9}, {NULL}};
const Segment SGI400371[] = {{&S67859[32], 10}, {NULL}};
const Segment SGI400372[] = {{&S67859[27], 4}, {NULL}};
const Segment SGI400373[] = {{&S67859[95], 9}, {NULL}};
const Segment SGI400374[] = {{&S68689[32], 10}, {NULL}};
const Segment SGI400375[] = {{&S68689[27], 4}, {NULL}};
const Segment SGI400376[] = {{&S68689[95], 9}, {NULL}};
const Segment SGI400377[] = {{&S69122[32], 10}, {NULL}};
const Segment SGI400378[] = {{&S69122[27], 4}, {NULL}};
const Segment SGI400379[] = {{&S69122[95], 9}, {NULL}};
const Segment SGI400380[] = {{&S69203[32], 7}, {NULL}};
const Segment SGI400381[] = {{&S69203[27], 4}, {NULL}};
const Segment SGI400382[] = {{&S69203[92], 9}, {NULL}};
const Segment SGI400383[] = {{&S69204[32], 11}, {NULL}};
const Segment SGI400384[] = {{&S69204[27], 4}, {NULL}};
const Segment SGI400385[] = {{&S69204[96], 9}, {NULL}};
const Segment SGI400386[] = {{&S69429[42], 15}, {NULL}};
const Segment SGI400387[] = {{&S69429[42], 27}, {NULL}};
const Segment SGI400388[] = {{&S69429[30], 4}, {NULL}};
const Segment SGI400389[] = {{&S69429[118], 3}, {NULL}};
const Segment SGI400390[] = {{&S69429[143], 8}, {NULL}};
const Segment SGI400391[] = {{&S69430[42], 15}, {NULL}};
const Segment SGI400392[] = {{&S69430[42], 27}, {NULL}};
const Segment SGI400393[] = {{&S69430[30], 4}, {NULL}};
const Segment SGI400394[] = {{&S69430[118], 3}, {NULL}};
const Segment SGI400395[] = {{&S69430[143], 8}, {NULL}};
const Segment SGI400396[] = {{&S69431[42], 15}, {NULL}};
const Segment SGI400397[] = {{&S69431[42], 27}, {NULL}};
const Segment SGI400398[] = {{&S69431[30], 4}, {NULL}};
const Segment SGI400399[] = {{&S69431[118], 3}, {NULL}};
const Segment SGI400400[] = {{&S69431[143], 8}, {NULL}};
const Segment SGI400401[] = {{&S69432[42], 15}, {NULL}};
const Segment SGI400402[] = {{&S69432[42], 27}, {NULL}};
const Segment SGI400403[] = {{&S69432[30], 4}, {NULL}};
const Segment SGI400404[] = {{&S69432[118], 3}, {NULL}};
const Segment SGI400405[] = {{&S69432[143], 8}, {NULL}};
const Segment SGI400406[] = {{&S69631[32], 19}, {NULL}};
const Segment SGI400407[] = {{&S69631[27], 4}, {NULL}};
const Segment SGI400408[] = {{&S69631[104], 9}, {NULL}};
const Segment SGI400409[] = {{&S69762[32], 14}, {NULL}};
const Segment SGI400410[] = {{&S69762[27], 4}, {NULL}};
const Segment SGI400411[] = {{&S69762[99], 9}, {NULL}};
const Segment SGI400412[] = {{&S69762[140], 8}, {NULL}};
const Segment SGI400413[] = {{&S70240[11], 1}, {NULL}};
const Segment SGI400414[] = {{&S70240[42], 13}, {NULL}};
const Segment SGI400415[] = {{&S70240[30], 4}, {NULL}};
const Segment SGI400416[] = {{&S70240[104], 3}, {NULL}};
const Segment SGI400417[] = {{&S70241[11], 1}, {NULL}};
const Segment SGI400418[] = {{&S70241[42], 13}, {NULL}};
const Segment SGI400419[] = {{&S70241[30], 4}, {NULL}};
const Segment SGI400420[] = {{&S70241[104], 3}, {NULL}};
const Segment SGI400421[] = {{&S70242[11], 1}, {NULL}};
const Segment SGI400422[] = {{&S70242[42], 13}, {NULL}};
const Segment SGI400423[] = {{&S70242[30], 4}, {NULL}};
const Segment SGI400424[] = {{&S70242[104], 3}, {NULL}};
const Segment SGI400425[] = {{&S70327[42], 5}, {NULL}};
const Segment SGI400426[] = {{&S70327[42], 17}, {NULL}};
const Segment SGI400427[] = {{&S70327[30], 4}, {NULL}};
const Segment SGI400428[] = {{&S70327[108], 3}, {NULL}};
const Segment SGI400429[] = {{&S70327[133], 8}, {NULL}};
const Segment SGI400430[] = {{&S70328[42], 5}, {NULL}};
const Segment SGI400431[] = {{&S70328[42], 21}, {NULL}};
const Segment SGI400432[] = {{&S70328[30], 4}, {NULL}};
const Segment SGI400433[] = {{&S70328[112], 3}, {NULL}};
const Segment SGI400434[] = {{&S70328[137], 8}, {NULL}};
const Segment SGI400435[] = {{&S71782[32], 10}, {NULL}};
const Segment SGI400436[] = {{&S71782[27], 4}, {NULL}};
const Segment SGI400437[] = {{&S71782[95], 9}, {NULL}};
const Segment SGI400438[] = {{&S72728[32], 10}, {NULL}};
const Segment SGI400439[] = {{&S72728[27], 4}, {NULL}};
const Segment SGI400440[] = {{&S72728[95], 9}, {NULL}};
const Segment SGI400441[] = {{&S72917[39], 18}, {NULL}};
const Segment SGI400442[] = {{&S72917[39], 30}, {NULL}};
const Segment SGI400443[] = {{&S72917[27], 4}, {NULL}};
const Segment SGI400444[] = {{&S72917[118], 3}, {NULL}};
const Segment SGI400445[] = {{&S72917[122], 9}, {NULL}};
const Segment SGI400446[] = {{&S72918[39], 11}, {NULL}};
const Segment SGI400447[] = {{&S72918[39], 23}, {NULL}};
const Segment SGI400448[] = {{&S72918[27], 4}, {NULL}};
const Segment SGI400449[] = {{&S72918[111], 3}, {NULL}};
const Segment SGI400450[] = {{&S72918[115], 9}, {NULL}};
const Segment SGI400451[] = {{&S72919[39], 18}, {NULL}};
const Segment SGI400452[] = {{&S72919[39], 30}, {NULL}};
const Segment SGI400453[] = {{&S72919[27], 4}, {NULL}};
const Segment SGI400454[] = {{&S72919[118], 3}, {NULL}};
const Segment SGI400455[] = {{&S72919[122], 9}, {NULL}};
const Segment SGI400456[] = {{&S72920[39], 26}, {NULL}};
const Segment SGI400457[] = {{&S72920[39], 38}, {NULL}};
const Segment SGI400458[] = {{&S72920[27], 4}, {NULL}};
const Segment SGI400459[] = {{&S72920[126], 3}, {NULL}};
const Segment SGI400460[] = {{&S72920[130], 9}, {NULL}};
const Segment SGI400461[] = {{&S72921[39], 18}, {NULL}};
const Segment SGI400462[] = {{&S72921[39], 30}, {NULL}};
const Segment SGI400463[] = {{&S72921[27], 4}, {NULL}};
const Segment SGI400464[] = {{&S72921[118], 3}, {NULL}};
const Segment SGI400465[] = {{&S72921[122], 9}, {NULL}};
const Segment SGI400466[] = {{&S72922[39], 31}, {NULL}};
const Segment SGI400467[] = {{&S72922[39], 43}, {NULL}};
const Segment SGI400468[] = {{&S72922[27], 4}, {NULL}};
const Segment SGI400469[] = {{&S72922[131], 3}, {NULL}};
const Segment SGI400470[] = {{&S72922[135], 9}, {NULL}};
const Segment SGI400471[] = {{&S72979[32], 10}, {NULL}};
const Segment SGI400472[] = {{&S72979[27], 4}, {NULL}};
const Segment SGI400473[] = {{&S72979[95], 9}, {NULL}};
const Segment SGI400474[] = {{&S73331[32], 9}, {NULL}};
const Segment SGI400475[] = {{&S73331[27], 4}, {NULL}};
const Segment SGI400476[] = {{&S73331[94], 9}, {NULL}};
const Segment SGI400477[] = {{&S73331[135], 8}, {NULL}};
const Segment SGI400478[] = {{&S73602[32], 10}, {NULL}};
const Segment SGI400479[] = {{&S73602[27], 4}, {NULL}};
const Segment SGI400480[] = {{&S73602[95], 9}, {NULL}};
const Segment SGI400481[] = {{&S73603[32], 18}, {NULL}};
const Segment SGI400482[] = {{&S73603[27], 4}, {NULL}};
const Segment SGI400483[] = {{&S73603[103], 9}, {NULL}};
const Segment SGI400484[] = {{&S73604[32], 10}, {NULL}};
const Segment SGI400485[] = {{&S73604[27], 4}, {NULL}};
const Segment SGI400486[] = {{&S73604[95], 9}, {NULL}};
const Segment SGI400487[] = {{&S73627[42], 19}, {NULL}};
const Segment SGI400488[] = {{&S73627[42], 31}, {NULL}};
const Segment SGI400489[] = {{&S73627[30], 4}, {NULL}};
const Segment SGI400490[] = {{&S73627[122], 3}, {NULL}};
const Segment SGI400491[] = {{&S73628[42], 19}, {NULL}};
const Segment SGI400492[] = {{&S73628[42], 31}, {NULL}};
const Segment SGI400493[] = {{&S73628[30], 4}, {NULL}};
const Segment SGI400494[] = {{&S73628[122], 3}, {NULL}};
const Segment SGI400495[] = {{&S74511[42], 10}, {NULL}};
const Segment SGI400496[] = {{&S74511[42], 22}, {NULL}};
const Segment SGI400497[] = {{&S74511[30], 4}, {NULL}};
const Segment SGI400498[] = {{&S74511[113], 3}, {NULL}};
const Segment SGI400499[] = {{&S74511[138], 8}, {NULL}};
const Segment SGI400500[] = {{&S74512[42], 10}, {NULL}};
const Segment SGI400501[] = {{&S74512[42], 22}, {NULL}};
const Segment SGI400502[] = {{&S74512[30], 4}, {NULL}};
const Segment SGI400503[] = {{&S74512[113], 3}, {NULL}};
const Segment SGI400504[] = {{&S74512[138], 8}, {NULL}};
const Segment SGI400505[] = {{&S74513[42], 10}, {NULL}};
const Segment SGI400506[] = {{&S74513[42], 22}, {NULL}};
const Segment SGI400507[] = {{&S74513[30], 4}, {NULL}};
const Segment SGI400508[] = {{&S74513[113], 3}, {NULL}};
const Segment SGI400509[] = {{&S74513[138], 8}, {NULL}};
const Segment SGI400510[] = {{&S75161[42], 10}, {NULL}};
const Segment SGI400511[] = {{&S75161[42], 22}, {NULL}};
const Segment SGI400512[] = {{&S75161[30], 4}, {NULL}};
const Segment SGI400513[] = {{&S75161[113], 3}, {NULL}};
const Segment SGI400514[] = {{&S75161[138], 8}, {NULL}};
const Segment SGI400515[] = {{&S75162[42], 10}, {NULL}};
const Segment SGI400516[] = {{&S75162[42], 22}, {NULL}};
const Segment SGI400517[] = {{&S75162[30], 4}, {NULL}};
const Segment SGI400518[] = {{&S75162[113], 3}, {NULL}};
const Segment SGI400519[] = {{&S75162[138], 8}, {NULL}};
const Segment SGI400520[] = {{&S75163[42], 10}, {NULL}};
const Segment SGI400521[] = {{&S75163[42], 22}, {NULL}};
const Segment SGI400522[] = {{&S75163[30], 4}, {NULL}};
const Segment SGI400523[] = {{&S75163[113], 3}, {NULL}};
const Segment SGI400524[] = {{&S75163[138], 8}, {NULL}};
const Segment SGI400525[] = {{&S75460[32], 10}, {NULL}};
const Segment SGI400526[] = {{&S75460[27], 4}, {NULL}};
const Segment SGI400527[] = {{&S75460[95], 9}, {NULL}};
const Segment SGI400528[] = {{&S75461[32], 10}, {NULL}};
const Segment SGI400529[] = {{&S75461[27], 4}, {NULL}};
const Segment SGI400530[] = {{&S75461[95], 9}, {NULL}};
const Segment SGI400531[] = {{&S75462[32], 10}, {NULL}};
const Segment SGI400532[] = {{&S75462[27], 4}, {NULL}};
const Segment SGI400533[] = {{&S75462[95], 9}, {NULL}};
const Segment SGI400534[] = {{&S75463[32], 18}, {NULL}};
const Segment SGI400535[] = {{&S75463[27], 4}, {NULL}};
const Segment SGI400536[] = {{&S75463[103], 9}, {NULL}};
const Segment SGI400537[] = {{&S75464[32], 16}, {NULL}};
const Segment SGI400538[] = {{&S75464[27], 4}, {NULL}};
const Segment SGI400539[] = {{&S75464[101], 9}, {NULL}};
const Segment SGI400540[] = {{&S75721[32], 10}, {NULL}};
const Segment SGI400541[] = {{&S75721[27], 4}, {NULL}};
const Segment SGI400542[] = {{&S75721[95], 9}, {NULL}};
const Segment SGI400543[] = {{&S75722[32], 11}, {NULL}};
const Segment SGI400544[] = {{&S75722[27], 4}, {NULL}};
const Segment SGI400545[] = {{&S75722[96], 9}, {NULL}};
const Segment SGI400546[] = {{&S75958[32], 10}, {NULL}};
const Segment SGI400547[] = {{&S75958[27], 4}, {NULL}};
const Segment SGI400548[] = {{&S75958[95], 9}, {NULL}};
const Segment SGI400549[] = {{&S75959[32], 11}, {NULL}};
const Segment SGI400550[] = {{&S75959[27], 4}, {NULL}};
const Segment SGI400551[] = {{&S75959[101], 9}, {NULL}};
const Segment SGI400552[] = {{&S76849[42], 10}, {NULL}};
const Segment SGI400553[] = {{&S76849[42], 22}, {NULL}};
const Segment SGI400554[] = {{&S76849[30], 4}, {NULL}};
const Segment SGI400555[] = {{&S76849[113], 3}, {NULL}};
const Segment SGI400556[] = {{&S76849[138], 8}, {NULL}};
const Segment SGI400557[] = {{&S76850[42], 10}, {NULL}};
const Segment SGI400558[] = {{&S76850[42], 22}, {NULL}};
const Segment SGI400559[] = {{&S76850[30], 4}, {NULL}};
const Segment SGI400560[] = {{&S76850[113], 3}, {NULL}};
const Segment SGI400561[] = {{&S76850[138], 8}, {NULL}};
const Segment SGI400562[] = {{&S76851[42], 10}, {NULL}};
const Segment SGI400563[] = {{&S76851[42], 22}, {NULL}};
const Segment SGI400564[] = {{&S76851[30], 4}, {NULL}};
const Segment SGI400565[] = {{&S76851[113], 3}, {NULL}};
const Segment SGI400566[] = {{&S76851[138], 8}, {NULL}};
const Segment SGI400567[] = {{&S76989[42], 10}, {NULL}};
const Segment SGI400568[] = {{&S76989[42], 22}, {NULL}};
const Segment SGI400569[] = {{&S76989[30], 4}, {NULL}};
const Segment SGI400570[] = {{&S76989[113], 3}, {NULL}};
const Segment SGI400571[] = {{&S76989[138], 8}, {NULL}};
const Segment SGI400572[] = {{&S76990[42], 10}, {NULL}};
const Segment SGI400573[] = {{&S76990[42], 22}, {NULL}};
const Segment SGI400574[] = {{&S76990[30], 4}, {NULL}};
const Segment SGI400575[] = {{&S76990[113], 3}, {NULL}};
const Segment SGI400576[] = {{&S76990[138], 8}, {NULL}};
const Segment SGI400577[] = {{&S76991[42], 10}, {NULL}};
const Segment SGI400578[] = {{&S76991[42], 22}, {NULL}};
const Segment SGI400579[] = {{&S76991[30], 4}, {NULL}};
const Segment SGI400580[] = {{&S76991[113], 3}, {NULL}};
const Segment SGI400581[] = {{&S76991[138], 8}, {NULL}};
const Segment SGI400582[] = {{&S77128[32], 15}, {NULL}};
const Segment SGI400583[] = {{&S77128[27], 4}, {NULL}};
const Segment SGI400584[] = {{&S77128[100], 9}, {NULL}};
const Segment SGI400585[] = {{&S77410[42], 7}, {NULL}};
const Segment SGI400586[] = {{&S77410[42], 19}, {NULL}};
const Segment SGI400587[] = {{&S77410[30], 4}, {NULL}};
const Segment SGI400588[] = {{&S77410[110], 3}, {NULL}};
const Segment SGI400589[] = {{&S77410[135], 8}, {NULL}};
const Segment SGI400590[] = {{&S77411[42], 7}, {NULL}};
const Segment SGI400591[] = {{&S77411[42], 19}, {NULL}};
const Segment SGI400592[] = {{&S77411[30], 4}, {NULL}};
const Segment SGI400593[] = {{&S77411[110], 3}, {NULL}};
const Segment SGI400594[] = {{&S77411[135], 8}, {NULL}};
const Segment SGI400595[] = {{&S77412[42], 11}, {NULL}};
const Segment SGI400596[] = {{&S77412[42], 23}, {NULL}};
const Segment SGI400597[] = {{&S77412[30], 4}, {NULL}};
const Segment SGI400598[] = {{&S77412[114], 3}, {NULL}};
const Segment SGI400599[] = {{&S77412[139], 8}, {NULL}};
const Segment SGI400600[] = {{&S77413[42], 11}, {NULL}};
const Segment SGI400601[] = {{&S77413[42], 23}, {NULL}};
const Segment SGI400602[] = {{&S77413[30], 4}, {NULL}};
const Segment SGI400603[] = {{&S77413[114], 3}, {NULL}};
const Segment SGI400604[] = {{&S77413[139], 8}, {NULL}};
const Segment SGI400605[] = {{&S77414[42], 7}, {NULL}};
const Segment SGI400606[] = {{&S77414[42], 19}, {NULL}};
const Segment SGI400607[] = {{&S77414[30], 4}, {NULL}};
const Segment SGI400608[] = {{&S77414[110], 3}, {NULL}};
const Segment SGI400609[] = {{&S77414[135], 8}, {NULL}};
const Segment SGI400610[] = {{&S77510[42], 10}, {NULL}};
const Segment SGI400611[] = {{&S77510[42], 22}, {NULL}};
const Segment SGI400612[] = {{&S77510[30], 4}, {NULL}};
const Segment SGI400613[] = {{&S77510[113], 3}, {NULL}};
const Segment SGI400614[] = {{&S77511[36], 10}, {NULL}};
const Segment SGI400615[] = {{&S77511[31], 4}, {NULL}};
const Segment SGI400616[] = {{&S77512[42], 10}, {NULL}};
const Segment SGI400617[] = {{&S77512[42], 22}, {NULL}};
const Segment SGI400618[] = {{&S77512[30], 4}, {NULL}};
const Segment SGI400619[] = {{&S77512[113], 3}, {NULL}};
const Segment SGI400620[] = {{&S77513[42], 10}, {NULL}};
const Segment SGI400621[] = {{&S77513[42], 22}, {NULL}};
const Segment SGI400622[] = {{&S77513[30], 4}, {NULL}};
const Segment SGI400623[] = {{&S77513[113], 3}, {NULL}};
const Segment SGI400624[] = {{&S77566[42], 10}, {NULL}};
const Segment SGI400625[] = {{&S77566[42], 22}, {NULL}};
const Segment SGI400626[] = {{&S77566[30], 4}, {NULL}};
const Segment SGI400627[] = {{&S77566[113], 3}, {NULL}};
const Segment SGI400628[] = {{&S77566[138], 8}, {NULL}};
const Segment SGI400629[] = {{&S77567[42], 10}, {NULL}};
const Segment SGI400630[] = {{&S77567[42], 22}, {NULL}};
const Segment SGI400631[] = {{&S77567[30], 4}, {NULL}};
const Segment SGI400632[] = {{&S77567[113], 3}, {NULL}};
const Segment SGI400633[] = {{&S77567[138], 8}, {NULL}};
const Segment SGI400634[] = {{&S77568[42], 10}, {NULL}};
const Segment SGI400635[] = {{&S77568[42], 22}, {NULL}};
const Segment SGI400636[] = {{&S77568[30], 4}, {NULL}};
const Segment SGI400637[] = {{&S77568[113], 3}, {NULL}};
const Segment SGI400638[] = {{&S77568[138], 8}, {NULL}};
const Segment SGI400639[] = {{&S77860[32], 5}, {NULL}};
const Segment SGI400640[] = {{&S77860[27], 4}, {NULL}};
const Segment SGI400641[] = {{&S77860[90], 9}, {NULL}};
const Segment SGI400642[] = {{&S78055[42], 10}, {NULL}};
const Segment SGI400643[] = {{&S78055[42], 22}, {NULL}};
const Segment SGI400644[] = {{&S78055[30], 4}, {NULL}};
const Segment SGI400645[] = {{&S78055[113], 3}, {NULL}};
const Segment SGI400646[] = {{&S78056[42], 10}, {NULL}};
const Segment SGI400647[] = {{&S78056[42], 22}, {NULL}};
const Segment SGI400648[] = {{&S78056[30], 4}, {NULL}};
const Segment SGI400649[] = {{&S78056[113], 3}, {NULL}};
const Segment SGI400650[] = {{&S78057[42], 10}, {NULL}};
const Segment SGI400651[] = {{&S78057[42], 22}, {NULL}};
const Segment SGI400652[] = {{&S78057[30], 4}, {NULL}};
const Segment SGI400653[] = {{&S78057[113], 3}, {NULL}};
const Segment SGI400654[] = {{&S78058[42], 10}, {NULL}};
const Segment SGI400655[] = {{&S78058[42], 22}, {NULL}};
const Segment SGI400656[] = {{&S78058[30], 4}, {NULL}};
const Segment SGI400657[] = {{&S78058[113], 3}, {NULL}};
const Segment SGI400658[] = {{&S78059[42], 10}, {NULL}};
const Segment SGI400659[] = {{&S78059[42], 22}, {NULL}};
const Segment SGI400660[] = {{&S78059[30], 4}, {NULL}};
const Segment SGI400661[] = {{&S78059[113], 3}, {NULL}};
const Segment SGI400662[] = {{&S78321[32], 12}, {NULL}};
const Segment SGI400663[] = {{&S78321[27], 4}, {NULL}};
const Segment SGI400664[] = {{&S78321[97], 9}, {NULL}};
const Segment SGI400665[] = {{&S78415[42], 9}, {NULL}};
const Segment SGI400666[] = {{&S78415[42], 21}, {NULL}};
const Segment SGI400667[] = {{&S78415[30], 4}, {NULL}};
const Segment SGI400668[] = {{&S78416[42], 9}, {NULL}};
const Segment SGI400669[] = {{&S78416[42], 21}, {NULL}};
const Segment SGI400670[] = {{&S78416[30], 4}, {NULL}};
const Segment SGI400671[] = {{&S78661[32], 11}, {NULL}};
const Segment SGI400672[] = {{&S78661[27], 4}, {NULL}};
const Segment SGI400673[] = {{&S78661[101], 9}, {NULL}};
const Segment SGI400674[] = {{&S78965[42], 5}, {NULL}};
const Segment SGI400675[] = {{&S78965[42], 17}, {NULL}};
const Segment SGI400676[] = {{&S78965[30], 4}, {NULL}};
const Segment SGI400677[] = {{&S78965[108], 3}, {NULL}};
const Segment SGI400678[] = {{&S78965[133], 8}, {NULL}};
const Segment SGI400679[] = {{&S79572[42], 10}, {NULL}};
const Segment SGI400680[] = {{&S79572[42], 22}, {NULL}};
const Segment SGI400681[] = {{&S79572[30], 4}, {NULL}};
const Segment SGI400682[] = {{&S79572[113], 3}, {NULL}};
const Segment SGI400683[] = {{&S79572[138], 8}, {NULL}};
const Segment SGI400684[] = {{&S79573[42], 10}, {NULL}};
const Segment SGI400685[] = {{&S79573[42], 22}, {NULL}};
const Segment SGI400686[] = {{&S79573[30], 4}, {NULL}};
const Segment SGI400687[] = {{&S79573[113], 3}, {NULL}};
const Segment SGI400688[] = {{&S79573[138], 8}, {NULL}};
const Segment SGI400689[] = {{&S79675[32], 10}, {NULL}};
const Segment SGI400690[] = {{&S79675[27], 4}, {NULL}};
const Segment SGI400691[] = {{&S79675[95], 9}, {NULL}};
const Segment SGI400692[] = {{&S79854[32], 14}, {NULL}};
const Segment SGI400693[] = {{&S79854[27], 4}, {NULL}};
const Segment SGI400694[] = {{&S79854[99], 9}, {NULL}};
const Segment SGI400695[] = {{&S79872[42], 12}, {NULL}};
const Segment SGI400696[] = {{&S79872[42], 24}, {NULL}};
const Segment SGI400697[] = {{&S79872[30], 4}, {NULL}};
const Segment SGI400698[] = {{&S79872[115], 3}, {NULL}};
const Segment SGI400699[] = {{&S79872[133], 8}, {NULL}};
const Segment SGI400700[] = {{&S79873[42], 12}, {NULL}};
const Segment SGI400701[] = {{&S79873[42], 24}, {NULL}};
const Segment SGI400702[] = {{&S79873[30], 4}, {NULL}};
const Segment SGI400703[] = {{&S79873[115], 3}, {NULL}};
const Segment SGI400704[] = {{&S79873[133], 8}, {NULL}};
const Segment SGI400705[] = {{&S79874[42], 12}, {NULL}};
const Segment SGI400706[] = {{&S79874[42], 24}, {NULL}};
const Segment SGI400707[] = {{&S79874[30], 4}, {NULL}};
const Segment SGI400708[] = {{&S79874[115], 3}, {NULL}};
const Segment SGI400709[] = {{&S79874[133], 8}, {NULL}};
const Segment SGI400710[] = {{&S79983[32], 9}, {NULL}};
const Segment SGI400711[] = {{&S79983[27], 4}, {NULL}};
const Segment SGI400712[] = {{&S79983[94], 9}, {NULL}};
const Segment SGI400713[] = {{&S80185[42], 7}, {NULL}};
const Segment SGI400714[] = {{&S80185[42], 19}, {NULL}};
const Segment SGI400715[] = {{&S80185[30], 4}, {NULL}};
const Segment SGI400716[] = {{&S80185[110], 3}, {NULL}};
const Segment SGI400717[] = {{&S80185[135], 8}, {NULL}};
const Segment SGI400718[] = {{&S80186[42], 7}, {NULL}};
const Segment SGI400719[] = {{&S80186[42], 19}, {NULL}};
const Segment SGI400720[] = {{&S80186[30], 4}, {NULL}};
const Segment SGI400721[] = {{&S80186[110], 3}, {NULL}};
const Segment SGI400722[] = {{&S80186[135], 8}, {NULL}};
const Segment SGI400723[] = {{&S80187[42], 7}, {NULL}};
const Segment SGI400724[] = {{&S80187[42], 19}, {NULL}};
const Segment SGI400725[] = {{&S80187[30], 4}, {NULL}};
const Segment SGI400726[] = {{&S80187[110], 3}, {NULL}};
const Segment SGI400727[] = {{&S80187[135], 8}, {NULL}};
const Segment SGI400728[] = {{&S80569[42], 9}, {NULL}};
const Segment SGI400729[] = {{&S80569[42], 21}, {NULL}};
const Segment SGI400730[] = {{&S80569[30], 4}, {NULL}};
const Segment SGI400731[] = {{&S80569[112], 3}, {NULL}};
const Segment SGI400732[] = {{&S80570[42], 9}, {NULL}};
const Segment SGI400733[] = {{&S80570[42], 21}, {NULL}};
const Segment SGI400734[] = {{&S80570[30], 4}, {NULL}};
const Segment SGI400735[] = {{&S80570[112], 3}, {NULL}};
const Segment SGI400736[] = {{&S80646[42], 9}, {NULL}};
const Segment SGI400737[] = {{&S80646[42], 21}, {NULL}};
const Segment SGI400738[] = {{&S80646[30], 4}, {NULL}};
const Segment SGI400739[] = {{&S80713[42], 10}, {NULL}};
const Segment SGI400740[] = {{&S80713[42], 22}, {NULL}};
const Segment SGI400741[] = {{&S80713[30], 4}, {NULL}};
const Segment SGI400742[] = {{&S80713[113], 3}, {NULL}};
const Segment SGI400743[] = {{&S80713[138], 8}, {NULL}};
const Segment SGI400744[] = {{&S80714[42], 10}, {NULL}};
const Segment SGI400745[] = {{&S80714[42], 22}, {NULL}};
const Segment SGI400746[] = {{&S80714[30], 4}, {NULL}};
const Segment SGI400747[] = {{&S80714[113], 3}, {NULL}};
const Segment SGI400748[] = {{&S80714[138], 8}, {NULL}};
const Segment SGI400749[] = {{&S80715[42], 10}, {NULL}};
const Segment SGI400750[] = {{&S80715[42], 22}, {NULL}};
const Segment SGI400751[] = {{&S80715[30], 4}, {NULL}};
const Segment SGI400752[] = {{&S80715[113], 3}, {NULL}};
const Segment SGI400753[] = {{&S80715[138], 8}, {NULL}};
const Segment SGI400754[] = {{&S80716[42], 10}, {NULL}};
const Segment SGI400755[] = {{&S80716[42], 22}, {NULL}};
const Segment SGI400756[] = {{&S80716[30], 4}, {NULL}};
const Segment SGI400757[] = {{&S80716[113], 3}, {NULL}};
const Segment SGI400758[] = {{&S80716[138], 8}, {NULL}};
const Segment SGI400759[] = {{&S80717[42], 10}, {NULL}};
const Segment SGI400760[] = {{&S80717[42], 22}, {NULL}};
const Segment SGI400761[] = {{&S80717[30], 4}, {NULL}};
const Segment SGI400762[] = {{&S80717[113], 3}, {NULL}};
const Segment SGI400763[] = {{&S80717[138], 8}, {NULL}};
const Segment SGI400764[] = {{&S80718[42], 10}, {NULL}};
const Segment SGI400765[] = {{&S80718[42], 22}, {NULL}};
const Segment SGI400766[] = {{&S80718[30], 4}, {NULL}};
const Segment SGI400767[] = {{&S80718[113], 3}, {NULL}};
const Segment SGI400768[] = {{&S80718[138], 8}, {NULL}};
const Segment SGI400769[] = {{&S80791[32], 7}, {NULL}};
const Segment SGI400770[] = {{&S80791[27], 4}, {NULL}};
const Segment SGI400771[] = {{&S80791[92], 9}, {NULL}};
const Segment SGI400772[] = {{&S80792[32], 18}, {NULL}};
const Segment SGI400773[] = {{&S80792[27], 4}, {NULL}};
const Segment SGI400774[] = {{&S80792[103], 9}, {NULL}};
const Segment SGI400775[] = {{&S81653[42], 18}, {NULL}};
const Segment SGI400776[] = {{&S81653[42], 30}, {NULL}};
const Segment SGI400777[] = {{&S81653[30], 4}, {NULL}};
const Segment SGI400778[] = {{&S81653[121], 3}, {NULL}};
const Segment SGI400779[] = {{&S81653[146], 8}, {NULL}};
const Segment SGI400780[] = {{&S81654[42], 18}, {NULL}};
const Segment SGI400781[] = {{&S81654[42], 30}, {NULL}};
const Segment SGI400782[] = {{&S81654[30], 4}, {NULL}};
const Segment SGI400783[] = {{&S81654[121], 3}, {NULL}};
const Segment SGI400784[] = {{&S81654[146], 8}, {NULL}};
const Segment SGI400785[] = {{&S81655[42], 18}, {NULL}};
const Segment SGI400786[] = {{&S81655[42], 30}, {NULL}};
const Segment SGI400787[] = {{&S81655[30], 4}, {NULL}};
const Segment SGI400788[] = {{&S81655[121], 3}, {NULL}};
const Segment SGI400789[] = {{&S81655[146], 8}, {NULL}};
const Segment SGI400790[] = {{&S81840[32], 18}, {NULL}};
const Segment SGI400791[] = {{&S81840[27], 4}, {NULL}};
const Segment SGI400792[] = {{&S81840[103], 9}, {NULL}};
const Segment SGI400793[] = {{&S82172[42], 20}, {NULL}};
const Segment SGI400794[] = {{&S82172[42], 32}, {NULL}};
const Segment SGI400795[] = {{&S82172[30], 4}, {NULL}};
const Segment SGI400796[] = {{&S82172[123], 3}, {NULL}};
const Segment SGI400797[] = {{&S82172[148], 8}, {NULL}};
const Segment SGI400798[] = {{&S82173[42], 20}, {NULL}};
const Segment SGI400799[] = {{&S82173[42], 32}, {NULL}};
const Segment SGI400800[] = {{&S82173[30], 4}, {NULL}};
const Segment SGI400801[] = {{&S82173[123], 3}, {NULL}};
const Segment SGI400802[] = {{&S82173[148], 8}, {NULL}};
const Segment SGI400803[] = {{&S82728[32], 10}, {NULL}};
const Segment SGI400804[] = {{&S82728[27], 4}, {NULL}};
const Segment SGI400805[] = {{&S82728[100], 9}, {NULL}};
const Segment SGI400806[] = {{&S82811[32], 10}, {NULL}};
const Segment SGI400807[] = {{&S82811[27], 4}, {NULL}};
const Segment SGI400808[] = {{&S82811[95], 9}, {NULL}};
const Segment SGI400809[] = {{&S82812[32], 11}, {NULL}};
const Segment SGI400810[] = {{&S82812[27], 4}, {NULL}};
const Segment SGI400811[] = {{&S82812[96], 9}, {NULL}};
const Segment SGI400812[] = {{&S82813[32], 10}, {NULL}};
const Segment SGI400813[] = {{&S82813[27], 4}, {NULL}};
const Segment SGI400814[] = {{&S82813[95], 9}, {NULL}};
const Segment SGI400815[] = {{&S82814[32], 21}, {NULL}};
const Segment SGI400816[] = {{&S82814[27], 4}, {NULL}};
const Segment SGI400817[] = {{&S82814[106], 9}, {NULL}};
const Segment SGI400818[] = {{&S83656[42], 12}, {NULL}};
const Segment SGI400819[] = {{&S83656[42], 24}, {NULL}};
const Segment SGI400820[] = {{&S83656[30], 4}, {NULL}};
const Segment SGI400821[] = {{&S83656[115], 3}, {NULL}};
const Segment SGI400822[] = {{&S83656[140], 8}, {NULL}};
const Segment SGI400823[] = {{&S83657[42], 12}, {NULL}};
const Segment SGI400824[] = {{&S83657[42], 24}, {NULL}};
const Segment SGI400825[] = {{&S83657[30], 4}, {NULL}};
const Segment SGI400826[] = {{&S83657[115], 3}, {NULL}};
const Segment SGI400827[] = {{&S83657[140], 8}, {NULL}};
const Segment SGI400828[] = {{&S83658[42], 12}, {NULL}};
const Segment SGI400829[] = {{&S83658[42], 24}, {NULL}};
const Segment SGI400830[] = {{&S83658[30], 4}, {NULL}};
const Segment SGI400831[] = {{&S83658[115], 3}, {NULL}};
const Segment SGI400832[] = {{&S83658[140], 8}, {NULL}};
const Segment SGI400833[] = {{&S83659[42], 10}, {NULL}};
const Segment SGI400834[] = {{&S83659[42], 22}, {NULL}};
const Segment SGI400835[] = {{&S83659[30], 4}, {NULL}};
const Segment SGI400836[] = {{&S83659[113], 3}, {NULL}};
const Segment SGI400837[] = {{&S83659[138], 8}, {NULL}};
const Segment SGI400838[] = {{&S83660[42], 10}, {NULL}};
const Segment SGI400839[] = {{&S83660[42], 22}, {NULL}};
const Segment SGI400840[] = {{&S83660[30], 4}, {NULL}};
const Segment SGI400841[] = {{&S83660[113], 3}, {NULL}};
const Segment SGI400842[] = {{&S83660[138], 8}, {NULL}};
const Segment SGI400843[] = {{&S83665[32], 12}, {NULL}};
const Segment SGI400844[] = {{&S83665[27], 4}, {NULL}};
const Segment SGI400845[] = {{&S83665[97], 9}, {NULL}};
const Segment SGI400846[] = {{&S83888[42], 10}, {NULL}};
const Segment SGI400847[] = {{&S83888[42], 22}, {NULL}};
const Segment SGI400848[] = {{&S83888[30], 4}, {NULL}};
const Segment SGI400849[] = {{&S83888[113], 3}, {NULL}};
const Segment SGI400850[] = {{&S83889[42], 10}, {NULL}};
const Segment SGI400851[] = {{&S83889[42], 22}, {NULL}};
const Segment SGI400852[] = {{&S83889[30], 4}, {NULL}};
const Segment SGI400853[] = {{&S83889[113], 3}, {NULL}};
const Segment SGI400854[] = {{&S83890[42], 10}, {NULL}};
const Segment SGI400855[] = {{&S83890[42], 22}, {NULL}};
const Segment SGI400856[] = {{&S83890[30], 4}, {NULL}};
const Segment SGI400857[] = {{&S83890[113], 3}, {NULL}};
const Segment SGI400858[] = {{&S84462[32], 10}, {NULL}};
const Segment SGI400859[] = {{&S84462[27], 4}, {NULL}};
const Segment SGI400860[] = {{&S84462[95], 9}, {NULL}};
const Segment SGI400861[] = {{&S84942[32], 14}, {NULL}};
const Segment SGI400862[] = {{&S84942[27], 4}, {NULL}};
const Segment SGI400863[] = {{&S84942[99], 9}, {NULL}};
const Segment SGI400864[] = {{&S84942[140], 8}, {NULL}};
const Segment SGI400865[] = {{&S85317[42], 10}, {NULL}};
const Segment SGI400866[] = {{&S85317[42], 22}, {NULL}};
const Segment SGI400867[] = {{&S85317[30], 4}, {NULL}};
const Segment SGI400868[] = {{&S85317[113], 3}, {NULL}};
const Segment SGI400869[] = {{&S85317[141], 3}, {NULL}};
const Segment SGI400870[] = {{&S85318[42], 10}, {NULL}};
const Segment SGI400871[] = {{&S85318[42], 22}, {NULL}};
const Segment SGI400872[] = {{&S85318[30], 4}, {NULL}};
const Segment SGI400873[] = {{&S85318[113], 3}, {NULL}};
const Segment SGI400874[] = {{&S85318[141], 3}, {NULL}};
const Segment SGI400875[] = {{&S85407[42], 18}, {NULL}};
const Segment SGI400876[] = {{&S85407[42], 30}, {NULL}};
const Segment SGI400877[] = {{&S85407[30], 4}, {NULL}};
const Segment SGI400878[] = {{&S85407[121], 3}, {NULL}};
const Segment SGI400879[] = {{&S85407[146], 8}, {NULL}};
const Segment SGI400880[] = {{&S85412[32], 11}, {NULL}};
const Segment SGI400881[] = {{&S85412[27], 4}, {NULL}};
const Segment SGI400882[] = {{&S85412[96], 9}, {NULL}};
const Segment SGI400883[] = {{&S85533[42], 10}, {NULL}};
const Segment SGI400884[] = {{&S85533[42], 22}, {NULL}};
const Segment SGI400885[] = {{&S85533[30], 4}, {NULL}};
const Segment SGI400886[] = {{&S85533[113], 3}, {NULL}};
const Segment SGI400887[] = {{&S85533[138], 8}, {NULL}};
const Segment SGI400888[] = {{&S85534[42], 10}, {NULL}};
const Segment SGI400889[] = {{&S85534[42], 22}, {NULL}};
const Segment SGI400890[] = {{&S85534[30], 4}, {NULL}};
const Segment SGI400891[] = {{&S85534[113], 3}, {NULL}};
const Segment SGI400892[] = {{&S85534[138], 8}, {NULL}};
const Segment SGI400893[] = {{&S85535[42], 10}, {NULL}};
const Segment SGI400894[] = {{&S85535[42], 22}, {NULL}};
const Segment SGI400895[] = {{&S85535[30], 4}, {NULL}};
const Segment SGI400896[] = {{&S85535[113], 3}, {NULL}};
const Segment SGI400897[] = {{&S85535[138], 8}, {NULL}};
const Segment SGI400898[] = {{&S85536[32], 10}, {NULL}};
const Segment SGI400899[] = {{&S85536[27], 4}, {NULL}};
const Segment SGI400900[] = {{&S85536[95], 9}, {NULL}};
const Segment SGI400901[] = {{&S85752[32], 7}, {NULL}};
const Segment SGI400902[] = {{&S85752[27], 4}, {NULL}};
const Segment SGI400903[] = {{&S85752[92], 9}, {NULL}};
const Segment SGI400904[] = {{&S85753[42], 10}, {NULL}};
const Segment SGI400905[] = {{&S85753[42], 22}, {NULL}};
const Segment SGI400906[] = {{&S85753[30], 4}, {NULL}};
const Segment SGI400907[] = {{&S85753[113], 3}, {NULL}};
const Segment SGI400908[] = {{&S85754[42], 10}, {NULL}};
const Segment SGI400909[] = {{&S85754[42], 22}, {NULL}};
const Segment SGI400910[] = {{&S85754[30], 4}, {NULL}};
const Segment SGI400911[] = {{&S85754[113], 3}, {NULL}};
const Segment SGI400912[] = {{&S85879[32], 9}, {NULL}};
const Segment SGI400913[] = {{&S85879[27], 4}, {NULL}};
const Segment SGI400914[] = {{&S85879[94], 9}, {NULL}};
const Segment SGI400915[] = {{&S85880[32], 10}, {NULL}};
const Segment SGI400916[] = {{&S85880[27], 4}, {NULL}};
const Segment SGI400917[] = {{&S85880[95], 9}, {NULL}};
const Segment SGI400918[] = {{&S86015[42], 6}, {NULL}};
const Segment SGI400919[] = {{&S86015[42], 18}, {NULL}};
const Segment SGI400920[] = {{&S86015[30], 4}, {NULL}};
const Segment SGI400921[] = {{&S86015[109], 3}, {NULL}};
const Segment SGI400922[] = {{&S86201[32], 11}, {NULL}};
const Segment SGI400923[] = {{&S86201[27], 4}, {NULL}};
const Segment SGI400924[] = {{&S86201[96], 9}, {NULL}};
const Segment SGI400925[] = {{&S86407[42], 10}, {NULL}};
const Segment SGI400926[] = {{&S86407[42], 22}, {NULL}};
const Segment SGI400927[] = {{&S86407[30], 4}, {NULL}};
const Segment SGI400928[] = {{&S86407[113], 3}, {NULL}};
const Segment SGI400929[] = {{&S86407[131], 8}, {NULL}};
const Segment SGI400930[] = {{&S86408[42], 10}, {NULL}};
const Segment SGI400931[] = {{&S86408[42], 22}, {NULL}};
const Segment SGI400932[] = {{&S86408[30], 4}, {NULL}};
const Segment SGI400933[] = {{&S86408[113], 3}, {NULL}};
const Segment SGI400934[] = {{&S86408[131], 8}, {NULL}};
const Segment SGI400935[] = {{&S86410[32], 9}, {NULL}};
const Segment SGI400936[] = {{&S86410[27], 4}, {NULL}};
const Segment SGI400937[] = {{&S86410[94], 9}, {NULL}};
const Segment SGI400938[] = {{&S86517[42], 10}, {NULL}};
const Segment SGI400939[] = {{&S86517[42], 22}, {NULL}};
const Segment SGI400940[] = {{&S86517[30], 4}, {NULL}};
const Segment SGI400941[] = {{&S86517[113], 3}, {NULL}};
const Segment SGI400942[] = {{&S86518[42], 10}, {NULL}};
const Segment SGI400943[] = {{&S86518[42], 22}, {NULL}};
const Segment SGI400944[] = {{&S86518[30], 4}, {NULL}};
const Segment SGI400945[] = {{&S86518[113], 3}, {NULL}};
const Segment SGI400946[] = {{&S86646[32], 9}, {NULL}};
const Segment SGI400947[] = {{&S86646[27], 4}, {NULL}};
const Segment SGI400948[] = {{&S86646[94], 9}, {NULL}};
const Segment SGI400949[] = {{&S86650[32], 18}, {NULL}};
const Segment SGI400950[] = {{&S86650[27], 4}, {NULL}};
const Segment SGI400951[] = {{&S86650[103], 9}, {NULL}};
const Segment SGI400952[] = {{&S86760[42], 10}, {NULL}};
const Segment SGI400953[] = {{&S86760[42], 22}, {NULL}};
const Segment SGI400954[] = {{&S86760[30], 4}, {NULL}};
const Segment SGI400955[] = {{&S86760[113], 3}, {NULL}};
const Segment SGI400956[] = {{&S86761[42], 10}, {NULL}};
const Segment SGI400957[] = {{&S86761[42], 22}, {NULL}};
const Segment SGI400958[] = {{&S86761[30], 4}, {NULL}};
const Segment SGI400959[] = {{&S86761[113], 3}, {NULL}};
const Segment SGI400960[] = {{&S87164[42], 14}, {NULL}};
const Segment SGI400961[] = {{&S87164[42], 26}, {NULL}};
const Segment SGI400962[] = {{&S87164[30], 4}, {NULL}};
const Segment SGI400963[] = {{&S87164[117], 3}, {NULL}};
const Segment SGI400964[] = {{&S87165[42], 21}, {NULL}};
const Segment SGI400965[] = {{&S87165[42], 33}, {NULL}};
const Segment SGI400966[] = {{&S87165[30], 4}, {NULL}};
const Segment SGI400967[] = {{&S87165[124], 3}, {NULL}};
const Segment SGI400968[] = {{&S87166[42], 12}, {NULL}};
const Segment SGI400969[] = {{&S87166[42], 24}, {NULL}};
const Segment SGI400970[] = {{&S87166[30], 4}, {NULL}};
const Segment SGI400971[] = {{&S87166[115], 3}, {NULL}};
const Segment SGI400972[] = {{&S87642[42], 20}, {NULL}};
const Segment SGI400973[] = {{&S87642[42], 32}, {NULL}};
const Segment SGI400974[] = {{&S87642[30], 4}, {NULL}};
const Segment SGI400975[] = {{&S87642[123], 3}, {NULL}};
const Segment SGI400976[] = {{&S87893[42], 9}, {NULL}};
const Segment SGI400977[] = {{&S87893[42], 21}, {NULL}};
const Segment SGI400978[] = {{&S87893[30], 4}, {NULL}};
const Segment SGI400979[] = {{&S87893[112], 3}, {NULL}};
const Segment SGI400980[] = {{&S87894[42], 9}, {NULL}};
const Segment SGI400981[] = {{&S87894[42], 21}, {NULL}};
const Segment SGI400982[] = {{&S87894[30], 4}, {NULL}};
const Segment SGI400983[] = {{&S87894[112], 3}, {NULL}};
const Segment SGI400984[] = {{&S88032[32], 23}, {NULL}};
const Segment SGI400985[] = {{&S88032[27], 4}, {NULL}};
const Segment SGI400986[] = {{&S88032[108], 9}, {NULL}};
const Segment SGI400987[] = {{&S88141[32], 10}, {NULL}};
const Segment SGI400988[] = {{&S88141[27], 4}, {NULL}};
const Segment SGI400989[] = {{&S88141[95], 9}, {NULL}};
const Segment SGI400990[] = {{&S88158[32], 10}, {NULL}};
const Segment SGI400991[] = {{&S88158[27], 4}, {NULL}};
const Segment SGI400992[] = {{&S88158[95], 9}, {NULL}};
const Segment SGI400993[] = {{&S88399[32], 10}, {NULL}};
const Segment SGI400994[] = {{&S88399[27], 4}, {NULL}};
const Segment SGI400995[] = {{&S88399[95], 9}, {NULL}};
const Segment SGI400996[] = {{&S88802[32], 10}, {NULL}};
const Segment SGI400997[] = {{&S88802[27], 4}, {NULL}};
const Segment SGI400998[] = {{&S88802[95], 9}, {NULL}};
const Segment SGI400999[] = {{&S89095[42], 9}, {NULL}};
const Segment SGI401000[] = {{&S89095[42], 21}, {NULL}};
const Segment SGI401001[] = {{&S89095[30], 4}, {NULL}};
const Segment SGI401002[] = {{&S89095[112], 3}, {NULL}};
const Segment SGI401003[] = {{&S89096[42], 9}, {NULL}};
const Segment SGI401004[] = {{&S89096[42], 21}, {NULL}};
const Segment SGI401005[] = {{&S89096[30], 4}, {NULL}};
const Segment SGI401006[] = {{&S89096[112], 3}, {NULL}};
const Segment SGI401007[] = {{&S89210[42], 9}, {NULL}};
const Segment SGI401008[] = {{&S89210[42], 21}, {NULL}};
const Segment SGI401009[] = {{&S89210[30], 4}, {NULL}};
const Segment SGI401010[] = {{&S89211[42], 9}, {NULL}};
const Segment SGI401011[] = {{&S89211[42], 21}, {NULL}};
const Segment SGI401012[] = {{&S89211[30], 4}, {NULL}};
const Segment SGI401013[] = {{&S89221[42], 18}, {NULL}};
const Segment SGI401014[] = {{&S89221[42], 30}, {NULL}};
const Segment SGI401015[] = {{&S89221[30], 4}, {NULL}};
const Segment SGI401016[] = {{&S89221[121], 3}, {NULL}};
const Segment SGI401017[] = {{&S89221[146], 8}, {NULL}};
const Segment SGI401018[] = {{&S89222[42], 18}, {NULL}};
const Segment SGI401019[] = {{&S89222[42], 30}, {NULL}};
const Segment SGI401020[] = {{&S89222[30], 4}, {NULL}};
const Segment SGI401021[] = {{&S89222[121], 3}, {NULL}};
const Segment SGI401022[] = {{&S89222[146], 8}, {NULL}};
const Segment SGI401023[] = {{&S89372[32], 10}, {NULL}};
const Segment SGI401024[] = {{&S89372[27], 4}, {NULL}};
const Segment SGI401025[] = {{&S89372[95], 9}, {NULL}};
const Segment SGI401026[] = {{&S89685[32], 7}, {NULL}};
const Segment SGI401027[] = {{&S89685[27], 4}, {NULL}};
const Segment SGI401028[] = {{&S89685[92], 9}, {NULL}};
const Segment SGI401029[] = {{&S90006[42], 18}, {NULL}};
const Segment SGI401030[] = {{&S90006[42], 30}, {NULL}};
const Segment SGI401031[] = {{&S90006[30], 4}, {NULL}};
const Segment SGI401032[] = {{&S90006[121], 3}, {NULL}};
const Segment SGI401033[] = {{&S90007[42], 18}, {NULL}};
const Segment SGI401034[] = {{&S90007[42], 30}, {NULL}};
const Segment SGI401035[] = {{&S90007[30], 4}, {NULL}};
const Segment SGI401036[] = {{&S90007[121], 3}, {NULL}};
const Segment SGI401037[] = {{&S90160[32], 18}, {NULL}};
const Segment SGI401038[] = {{&S90160[27], 4}, {NULL}};
const Segment SGI401039[] = {{&S90160[103], 9}, {NULL}};
const Segment SGI401040[] = {{&S90350[32], 10}, {NULL}};
const Segment SGI401041[] = {{&S90350[27], 4}, {NULL}};
const Segment SGI401042[] = {{&S90350[95], 9}, {NULL}};
const Segment SGI401043[] = {{&S90351[32], 10}, {NULL}};
const Segment SGI401044[] = {{&S90351[27], 4}, {NULL}};
const Segment SGI401045[] = {{&S90351[95], 9}, {NULL}};
const Segment SGI401046[] = {{&S90727[32], 10}, {NULL}};
const Segment SGI401047[] = {{&S90727[27], 4}, {NULL}};
const Segment SGI401048[] = {{&S90727[95], 9}, {NULL}};
const Segment SGI401049[] = {{&S90913[42], 7}, {NULL}};
const Segment SGI401050[] = {{&S90913[42], 19}, {NULL}};
const Segment SGI401051[] = {{&S90913[30], 4}, {NULL}};
const Segment SGI401052[] = {{&S90913[110], 3}, {NULL}};
const Segment SGI401053[] = {{&S90913[135], 8}, {NULL}};
const Segment SGI401054[] = {{&S90914[42], 7}, {NULL}};
const Segment SGI401055[] = {{&S90914[42], 19}, {NULL}};
const Segment SGI401056[] = {{&S90914[30], 4}, {NULL}};
const Segment SGI401057[] = {{&S90914[110], 3}, {NULL}};
const Segment SGI401058[] = {{&S90914[135], 8}, {NULL}};
const Segment SGI401059[] = {{&S90918[32], 10}, {NULL}};
const Segment SGI401060[] = {{&S90918[27], 4}, {NULL}};
const Segment SGI401061[] = {{&S90918[95], 9}, {NULL}};
const Segment SGI401062[] = {{&S90918[136], 8}, {NULL}};
const Segment SGI401063[] = {{&S91259[42], 7}, {NULL}};
const Segment SGI401064[] = {{&S91259[42], 19}, {NULL}};
const Segment SGI401065[] = {{&S91259[30], 4}, {NULL}};
const Segment SGI401066[] = {{&S91259[110], 3}, {NULL}};
const Segment SGI401067[] = {{&S91259[135], 8}, {NULL}};
const Segment SGI401068[] = {{&S91263[42], 10}, {NULL}};
const Segment SGI401069[] = {{&S91263[42], 22}, {NULL}};
const Segment SGI401070[] = {{&S91263[30], 4}, {NULL}};
const Segment SGI401071[] = {{&S91263[113], 3}, {NULL}};
const Segment SGI401072[] = {{&S91263[138], 8}, {NULL}};
const Segment SGI401073[] = {{&S91264[42], 10}, {NULL}};
const Segment SGI401074[] = {{&S91264[42], 22}, {NULL}};
const Segment SGI401075[] = {{&S91264[30], 4}, {NULL}};
const Segment SGI401076[] = {{&S91264[113], 3}, {NULL}};
const Segment SGI401077[] = {{&S91264[138], 8}, {NULL}};
const Segment SGI401078[] = {{&S91266[42], 9}, {NULL}};
const Segment SGI401079[] = {{&S91266[42], 21}, {NULL}};
const Segment SGI401080[] = {{&S91266[30], 4}, {NULL}};
const Segment SGI401081[] = {{&S91436[42], 9}, {NULL}};
const Segment SGI401082[] = {{&S91436[42], 21}, {NULL}};
const Segment SGI401083[] = {{&S91436[30], 4}, {NULL}};
const Segment SGI401084[] = {{&S91436[112], 3}, {NULL}};
const Segment SGI401085[] = {{&S91437[42], 9}, {NULL}};
const Segment SGI401086[] = {{&S91437[42], 21}, {NULL}};
const Segment SGI401087[] = {{&S91437[30], 4}, {NULL}};
const Segment SGI401088[] = {{&S91437[112], 3}, {NULL}};
const Segment SGI401089[] = {{&S91840[42], 9}, {NULL}};
const Segment SGI401090[] = {{&S91840[42], 21}, {NULL}};
const Segment SGI401091[] = {{&S91840[30], 4}, {NULL}};
const Segment SGI401092[] = {{&S91840[112], 3}, {NULL}};
const Segment SGI401093[] = {{&S91840[147], 3}, {NULL}};
const Segment SGI401094[] = {{&S91851[32], 10}, {NULL}};
const Segment SGI401095[] = {{&S91851[27], 4}, {NULL}};
const Segment SGI401096[] = {{&S91851[95], 9}, {NULL}};
const Segment SGI401097[] = {{&S91852[32], 10}, {NULL}};
const Segment SGI401098[] = {{&S91852[27], 4}, {NULL}};
const Segment SGI401099[] = {{&S91852[95], 9}, {NULL}};
const Segment SGI401100[] = {{&S92050[42], 7}, {NULL}};
const Segment SGI401101[] = {{&S92050[42], 19}, {NULL}};
const Segment SGI401102[] = {{&S92050[30], 4}, {NULL}};
const Segment SGI401103[] = {{&S92050[110], 3}, {NULL}};
const Segment SGI401104[] = {{&S92050[135], 8}, {NULL}};
const Segment SGI401105[] = {{&S92051[42], 7}, {NULL}};
const Segment SGI401106[] = {{&S92051[42], 19}, {NULL}};
const Segment SGI401107[] = {{&S92051[30], 4}, {NULL}};
const Segment SGI401108[] = {{&S92051[110], 3}, {NULL}};
const Segment SGI401109[] = {{&S92051[135], 8}, {NULL}};
const Segment SGI401110[] = {{&S92057[32], 7}, {NULL}};
const Segment SGI401111[] = {{&S92057[27], 4}, {NULL}};
const Segment SGI401112[] = {{&S92057[92], 9}, {NULL}};
const Segment SGI401113[] = {{&S92251[32], 12}, {NULL}};
const Segment SGI401114[] = {{&S92251[27], 4}, {NULL}};
const Segment SGI401115[] = {{&S92251[97], 9}, {NULL}};
const Segment SGI401116[] = {{&S92646[42], 11}, {NULL}};
const Segment SGI401117[] = {{&S92646[42], 23}, {NULL}};
const Segment SGI401118[] = {{&S92646[30], 4}, {NULL}};
const Segment SGI401119[] = {{&S92646[114], 3}, {NULL}};
const Segment SGI401120[] = {{&S92646[139], 8}, {NULL}};
const Segment SGI401121[] = {{&S92647[42], 11}, {NULL}};
const Segment SGI401122[] = {{&S92647[42], 23}, {NULL}};
const Segment SGI401123[] = {{&S92647[30], 4}, {NULL}};
const Segment SGI401124[] = {{&S92647[114], 3}, {NULL}};
const Segment SGI401125[] = {{&S92647[139], 8}, {NULL}};
const Segment SGI401126[] = {{&S92802[32], 10}, {NULL}};
const Segment SGI401127[] = {{&S92802[27], 4}, {NULL}};
const Segment SGI401128[] = {{&S92802[95], 9}, {NULL}};
const Segment SGI401129[] = {{&S93035[32], 10}, {NULL}};
const Segment SGI401130[] = {{&S93035[27], 4}, {NULL}};
const Segment SGI401131[] = {{&S93035[95], 9}, {NULL}};
const Segment SGI401132[] = {{&S93221[32], 19}, {NULL}};
const Segment SGI401133[] = {{&S93221[27], 4}, {NULL}};
const Segment SGI401134[] = {{&S93221[104], 9}, {NULL}};
const Segment SGI401135[] = {{&S93463[42], 9}, {NULL}};
const Segment SGI401136[] = {{&S93463[42], 21}, {NULL}};
const Segment SGI401137[] = {{&S93463[30], 4}, {NULL}};
const Segment SGI401138[] = {{&S93463[112], 3}, {NULL}};
const Segment SGI401139[] = {{&S93464[42], 9}, {NULL}};
const Segment SGI401140[] = {{&S93464[42], 21}, {NULL}};
const Segment SGI401141[] = {{&S93464[30], 4}, {NULL}};
const Segment SGI401142[] = {{&S93464[112], 3}, {NULL}};
const Segment SGI401143[] = {{&S93476[32], 14}, {NULL}};
const Segment SGI401144[] = {{&S93476[27], 4}, {NULL}};
const Segment SGI401145[] = {{&S93476[99], 9}, {NULL}};
const Segment SGI401146[] = {{&S93694[32], 11}, {NULL}};
const Segment SGI401147[] = {{&S93694[27], 4}, {NULL}};
const Segment SGI401148[] = {{&S93694[96], 9}, {NULL}};
const Segment SGI401149[] = {{&S93939[32], 20}, {NULL}};
const Segment SGI401150[] = {{&S93939[27], 4}, {NULL}};
const Segment SGI401151[] = {{&S93939[105], 9}, {NULL}};
const Segment SGI401152[] = {{&S93943[42], 23}, {NULL}};
const Segment SGI401153[] = {{&S93943[42], 35}, {NULL}};
const Segment SGI401154[] = {{&S93943[30], 4}, {NULL}};
const Segment SGI401155[] = {{&S93943[126], 3}, {NULL}};
const Segment SGI401156[] = {{&S93943[144], 8}, {NULL}};
const Segment SGI401157[] = {{&S94374[42], 12}, {NULL}};
const Segment SGI401158[] = {{&S94374[42], 24}, {NULL}};
const Segment SGI401159[] = {{&S94374[30], 4}, {NULL}};
const Segment SGI401160[] = {{&S94374[115], 3}, {NULL}};
const Segment SGI401161[] = {{&S94374[140], 8}, {NULL}};
const Segment SGI401162[] = {{&S94375[32], 9}, {NULL}};
const Segment SGI401163[] = {{&S94375[27], 4}, {NULL}};
const Segment SGI401164[] = {{&S94375[94], 9}, {NULL}};
const Segment SGI401165[] = {{&S94848[42], 11}, {NULL}};
const Segment SGI401166[] = {{&S94848[42], 23}, {NULL}};
const Segment SGI401167[] = {{&S94848[30], 4}, {NULL}};
const Segment SGI401168[] = {{&S94848[114], 3}, {NULL}};
const Segment SGI401169[] = {{&S94848[139], 8}, {NULL}};
const Segment SGI401170[] = {{&S95091[32], 11}, {NULL}};
const Segment SGI401171[] = {{&S95091[27], 4}, {NULL}};
const Segment SGI401172[] = {{&S95091[96], 9}, {NULL}};
const Segment SGI401173[] = {{&S95622[32], 12}, {NULL}};
const Segment SGI401174[] = {{&S95622[27], 4}, {NULL}};
const Segment SGI401175[] = {{&S95622[97], 9}, {NULL}};
const Segment SGI401176[] = {{&S95623[32], 23}, {NULL}};
const Segment SGI401177[] = {{&S95623[27], 4}, {NULL}};
const Segment SGI401178[] = {{&S95623[108], 9}, {NULL}};
const Segment SGI401179[] = {{&S95654[42], 11}, {NULL}};
const Segment SGI401180[] = {{&S95654[42], 23}, {NULL}};
const Segment SGI401181[] = {{&S95654[30], 4}, {NULL}};
const Segment SGI401182[] = {{&S95654[114], 3}, {NULL}};
const Segment SGI401183[] = {{&S95654[132], 8}, {NULL}};
const Segment SGI401184[] = {{&S95664[39], 19}, {NULL}};
const Segment SGI401185[] = {{&S95664[39], 31}, {NULL}};
const Segment SGI401186[] = {{&S95664[27], 4}, {NULL}};
const Segment SGI401187[] = {{&S95664[119], 3}, {NULL}};
const Segment SGI401188[] = {{&S95664[123], 9}, {NULL}};
const Segment SGI401189[] = {{&S96240[32], 15}, {NULL}};
const Segment SGI401190[] = {{&S96240[27], 4}, {NULL}};
const Segment SGI401191[] = {{&S96240[100], 9}, {NULL}};
const Segment SGI401192[] = {{&S96250[32], 10}, {NULL}};
const Segment SGI401193[] = {{&S96250[27], 4}, {NULL}};
const Segment SGI401194[] = {{&S96250[95], 9}, {NULL}};
const Segment SGI401195[] = {{&S96250[136], 8}, {NULL}};
const Segment SGI401196[] = {{&S96268[32], 12}, {NULL}};
const Segment SGI401197[] = {{&S96268[27], 4}, {NULL}};
const Segment SGI401198[] = {{&S96268[97], 9}, {NULL}};
const Segment SGI401199[] = {{&S96549[32], 18}, {NULL}};
const Segment SGI401200[] = {{&S96549[27], 4}, {NULL}};
const Segment SGI401201[] = {{&S96549[103], 9}, {NULL}};
const Segment SGI401202[] = {{&S96565[32], 10}, {NULL}};
const Segment SGI401203[] = {{&S96565[27], 4}, {NULL}};
const Segment SGI401204[] = {{&S96565[95], 9}, {NULL}};
const Segment SGI401205[] = {{&S96868[42], 10}, {NULL}};
const Segment SGI401206[] = {{&S96868[42], 22}, {NULL}};
const Segment SGI401207[] = {{&S96868[30], 4}, {NULL}};
const Segment SGI401208[] = {{&S96868[113], 3}, {NULL}};
const Segment SGI401209[] = {{&S80700[42], 82}, {NULL}};
const Segment SGI401210[] = {{&S80700[42], 103}, {NULL}};
const Segment SGI401211[] = {{&S80700[30], 4}, {NULL}};
const Segment SGI401212[] = {{&S80700[192], 3}, {NULL}};
const Segment SGI401213[] = {{&S80700[118], 5}, {NULL}};
const Segment SGI401214[] = {{&S80701[42], 82}, {NULL}};
const Segment SGI401215[] = {{&S80701[42], 103}, {NULL}};
const Segment SGI401216[] = {{&S80701[30], 4}, {NULL}};
const Segment SGI401217[] = {{&S80701[192], 3}, {NULL}};
const Segment SGI401218[] = {{&S80701[118], 5}, {NULL}};
const Segment SGI401219[] = {{&S80702[42], 82}, {NULL}};
const Segment SGI401220[] = {{&S80702[42], 103}, {NULL}};
const Segment SGI401221[] = {{&S80702[30], 4}, {NULL}};
const Segment SGI401222[] = {{&S80702[89], 3}, {NULL}};
const Segment SGI401223[] = {{&S80702[118], 5}, {NULL}};
const Segment SGI401224[] = {{&S80703[42], 8}, {NULL}};
const Segment SGI401225[] = {{&S80703[30], 4}, {NULL}};
const Segment SGI401226[] = {{&S80704[42], 82}, {NULL}};
const Segment SGI401227[] = {{&S80704[42], 103}, {NULL}};
const Segment SGI401228[] = {{&S80704[30], 4}, {NULL}};
const Segment SGI401229[] = {{&S80704[192], 3}, {NULL}};
const Segment SGI401230[] = {{&S80704[118], 5}, {NULL}};
const Segment SGI401231[] = {{&S80705[42], 82}, {NULL}};
const Segment SGI401232[] = {{&S80705[42], 103}, {NULL}};
const Segment SGI401233[] = {{&S80705[30], 4}, {NULL}};
const Segment SGI401234[] = {{&S80705[192], 3}, {NULL}};
const Segment SGI401235[] = {{&S80705[118], 5}, {NULL}};
const Segment SGI401236[] = {{&S80706[42], 82}, {NULL}};
const Segment SGI401237[] = {{&S80706[42], 103}, {NULL}};
const Segment SGI401238[] = {{&S80706[30], 4}, {NULL}};
const Segment SGI401239[] = {{&S80706[192], 3}, {NULL}};
const Segment SGI401240[] = {{&S80706[118], 5}, {NULL}};
const Segment SGI401241[] = {{&S80707[42], 82}, {NULL}};
const Segment SGI401242[] = {{&S80707[42], 103}, {NULL}};
const Segment SGI401243[] = {{&S80707[30], 4}, {NULL}};
const Segment SGI401244[] = {{&S80707[192], 3}, {NULL}};
const Segment SGI401245[] = {{&S80707[118], 5}, {NULL}};
const Segment SGI401246[] = {{&S80708[42], 72}, {NULL}};
const Segment SGI401247[] = {{&S80708[42], 93}, {NULL}};
const Segment SGI401248[] = {{&S80708[30], 4}, {NULL}};
const Segment SGI401249[] = {{&S80708[182], 3}, {NULL}};
const Segment SGI401250[] = {{&S80708[108], 5}, {NULL}};
const Segment SGI401251[] = {{&S97230[42], 10}, {NULL}};
const Segment SGI401252[] = {{&S97230[42], 22}, {NULL}};
const Segment SGI401253[] = {{&S97230[30], 4}, {NULL}};
const Segment SGI401254[] = {{&S97230[113], 3}, {NULL}};
const Segment SGI401255[] = {{&S97230[131], 8}, {NULL}};
const Segment SGI401256[] = {{&S97234[42], 10}, {NULL}};
const Segment SGI401257[] = {{&S97234[42], 22}, {NULL}};
const Segment SGI401258[] = {{&S97234[30], 4}, {NULL}};
const Segment SGI401259[] = {{&S97234[113], 3}, {NULL}};
const Segment SGI401260[] = {{&S97234[131], 8}, {NULL}};
const Segment SGI401261[] = {{&S97531[32], 7}, {NULL}};
const Segment SGI401262[] = {{&S97531[27], 4}, {NULL}};
const Segment SGI401263[] = {{&S97531[92], 9}, {NULL}};
const Segment SGI401264[] = {{&S97532[32], 11}, {NULL}};
const Segment SGI401265[] = {{&S97532[27], 4}, {NULL}};
const Segment SGI401266[] = {{&S97532[96], 9}, {NULL}};
const Segment SGI401267[] = {{&S97575[42], 10}, {NULL}};
const Segment SGI401268[] = {{&S97575[42], 22}, {NULL}};
const Segment SGI401269[] = {{&S97575[30], 4}, {NULL}};
const Segment SGI401270[] = {{&S97575[113], 3}, {NULL}};
const Segment SGI401271[] = {{&S97586[32], 7}, {NULL}};
const Segment SGI401272[] = {{&S97586[27], 4}, {NULL}};
const Segment SGI401273[] = {{&S97586[92], 9}, {NULL}};
const Segment SGI401274[] = {{&S97593[32], 10}, {NULL}};
const Segment SGI401275[] = {{&S97593[27], 4}, {NULL}};
const Segment SGI401276[] = {{&S97593[95], 9}, {NULL}};
const Segment SGI401277[] = {{&S97600[42], 18}, {NULL}};
const Segment SGI401278[] = {{&S97600[42], 30}, {NULL}};
const Segment SGI401279[] = {{&S97600[30], 4}, {NULL}};
const Segment SGI401280[] = {{&S97600[121], 3}, {NULL}};
const Segment SGI401281[] = {{&S98017[42], 10}, {NULL}};
const Segment SGI401282[] = {{&S98017[42], 22}, {NULL}};
const Segment SGI401283[] = {{&S98017[30], 4}, {NULL}};
const Segment SGI401284[] = {{&S98017[113], 3}, {NULL}};
const Segment SGI401285[] = {{&S98978[42], 18}, {NULL}};
const Segment SGI401286[] = {{&S98978[42], 30}, {NULL}};
const Segment SGI401287[] = {{&S98978[30], 4}, {NULL}};
const Segment SGI401288[] = {{&S98978[121], 3}, {NULL}};
const Segment SGI401289[] = {{&S98988[42], 10}, {NULL}};
const Segment SGI401290[] = {{&S98988[42], 22}, {NULL}};
const Segment SGI401291[] = {{&S98988[30], 4}, {NULL}};
const Segment SGI401292[] = {{&S98988[113], 3}, {NULL}};
const Segment SGI401293[] = {{&S99379[32], 9}, {NULL}};
const Segment SGI401294[] = {{&S99379[27], 4}, {NULL}};
const Segment SGI401295[] = {{&S99379[94], 9}, {NULL}};
const Segment SGI401296[] = {{&S99379[128], 8}, {NULL}};
const Segment SGI401297[] = {{&S99461[42], 9}, {NULL}};
const Segment SGI401298[] = {{&S99461[42], 21}, {NULL}};
const Segment SGI401299[] = {{&S99461[30], 4}, {NULL}};
const Segment SGI401300[] = {{&S99461[112], 3}, {NULL}};
const Segment SGI401301[] = {{&S99461[137], 8}, {NULL}};
const Segment SGI401302[] = {{&S99467[42], 9}, {NULL}};
const Segment SGI401303[] = {{&S99467[42], 21}, {NULL}};
const Segment SGI401304[] = {{&S99467[30], 4}, {NULL}};
const Segment SGI401305[] = {{&S99467[110], 3}, {NULL}};
const Segment SGI401306[] = {{&S99468[42], 10}, {NULL}};
const Segment SGI401307[] = {{&S99468[42], 22}, {NULL}};
const Segment SGI401308[] = {{&S99468[30], 4}, {NULL}};
const Segment SGI401309[] = {{&S99468[113], 3}, {NULL}};
const Segment SGI401310[] = {{&S99468[138], 8}, {NULL}};
const Segment SGI401311[] = {{&S99476[42], 10}, {NULL}};
const Segment SGI401312[] = {{&S99476[42], 27}, {NULL}};
const Segment SGI401313[] = {{&S99476[30], 4}, {NULL}};
const Segment SGI401314[] = {{&S99476[118], 3}, {NULL}};
const Segment SGI401315[] = {{&S99483[42], 10}, {NULL}};
const Segment SGI401316[] = {{&S99483[42], 22}, {NULL}};
const Segment SGI401317[] = {{&S99483[30], 4}, {NULL}};
const Segment SGI401318[] = {{&S99483[113], 3}, {NULL}};
const Segment SGI401319[] = {{&S99494[42], 14}, {NULL}};
const Segment SGI401320[] = {{&S99494[42], 26}, {NULL}};
const Segment SGI401321[] = {{&S99494[30], 4}, {NULL}};
const Segment SGI401322[] = {{&S99957[32], 9}, {NULL}};
const Segment SGI401323[] = {{&S99957[27], 4}, {NULL}};
const Segment SGI401324[] = {{&S99957[94], 9}, {NULL}};
const Segment SGI401325[] = {{&S100028[32], 10}, {NULL}};
const Segment SGI401326[] = {{&S100028[27], 4}, {NULL}};
const Segment SGI401327[] = {{&S100028[95], 9}, {NULL}};
const Segment SGI401328[] = {{&S100055[42], 9}, {NULL}};
const Segment SGI401329[] = {{&S100055[42], 21}, {NULL}};
const Segment SGI401330[] = {{&S100055[30], 4}, {NULL}};
const Segment SGI401331[] = {{&S100055[112], 3}, {NULL}};
const Segment SGI401332[] = {{&S100055[137], 8}, {NULL}};
const Segment SGI401333[] = {{&S100076[32], 11}, {NULL}};
const Segment SGI401334[] = {{&S100076[27], 4}, {NULL}};
const Segment SGI401335[] = {{&S100076[96], 9}, {NULL}};
const Segment SGI401336[] = {{&S100079[42], 7}, {NULL}};
const Segment SGI401337[] = {{&S100079[42], 19}, {NULL}};
const Segment SGI401338[] = {{&S100079[30], 4}, {NULL}};
const Segment SGI401339[] = {{&S100079[110], 3}, {NULL}};
const Segment SGI401340[] = {{&S100455[32], 7}, {NULL}};
const Segment SGI401341[] = {{&S100455[27], 4}, {NULL}};
const Segment SGI401342[] = {{&S100455[92], 9}, {NULL}};
const Segment SGI401343[] = {{&S100457[32], 9}, {NULL}};
const Segment SGI401344[] = {{&S100457[27], 4}, {NULL}};
const Segment SGI401345[] = {{&S100457[94], 9}, {NULL}};
const Segment SGI401346[] = {{&S100493[42], 9}, {NULL}};
const Segment SGI401347[] = {{&S100493[42], 21}, {NULL}};
const Segment SGI401348[] = {{&S100493[30], 4}, {NULL}};
const Segment SGI401349[] = {{&S100493[112], 3}, {NULL}};
const Segment SGI401350[] = {{&S100508[32], 10}, {NULL}};
const Segment SGI401351[] = {{&S100508[27], 4}, {NULL}};
const Segment SGI401352[] = {{&S100508[95], 9}, {NULL}};
const Segment SGI401353[] = {{&S100659[42], 12}, {NULL}};
const Segment SGI401354[] = {{&S100659[42], 24}, {NULL}};
const Segment SGI401355[] = {{&S100659[30], 4}, {NULL}};
const Segment SGI401356[] = {{&S100659[115], 3}, {NULL}};
const Segment SGI401357[] = {{&S100684[32], 20}, {NULL}};
const Segment SGI401358[] = {{&S100684[27], 4}, {NULL}};
const Segment SGI401359[] = {{&S100684[105], 9}, {NULL}};
const Segment SGI401360[] = {{&S64554[11], 7}, {NULL}};
const Segment SGI401361[] = {{&S64554[36], 6}, {NULL}};
const Segment SGI401362[] = {{&S64555[11], 7}, {NULL}};
const Segment SGI401363[] = {{&S64555[36], 6}, {NULL}};
const Segment SGI401364[] = {{&S64556[11], 7}, {NULL}};
const Segment SGI401365[] = {{&S64556[36], 6}, {NULL}};
const Segment SGI401366[] = {{&S64557[11], 7}, {NULL}};
const Segment SGI401367[] = {{&S64557[36], 6}, {NULL}};
const Segment SGI401368[] = {{&S64558[11], 7}, {NULL}};
const Segment SGI401369[] = {{&S64558[36], 6}, {NULL}};
const Segment SGI401370[] = {{&S64559[11], 7}, {NULL}};
const Segment SGI401371[] = {{&S64559[36], 6}, {NULL}};
const Segment SGI401372[] = {{&S64560[11], 7}, {NULL}};
const Segment SGI401373[] = {{&S64560[36], 6}, {NULL}};
const Segment SGI401374[] = {{&S64561[11], 7}, {NULL}};
const Segment SGI401375[] = {{&S64561[36], 6}, {NULL}};
const Segment SGI401376[] = {{&S64562[11], 7}, {NULL}};
const Segment SGI401377[] = {{&S64562[36], 6}, {NULL}};
const Segment SGI401378[] = {{&S64563[11], 7}, {NULL}};
const Segment SGI401379[] = {{&S64563[36], 6}, {NULL}};
const Segment SGI401380[] = {{&S64564[11], 7}, {NULL}};
const Segment SGI401381[] = {{&S64564[36], 6}, {NULL}};
const Segment SGI401382[] = {{&S64565[11], 7}, {NULL}};
const Segment SGI401383[] = {{&S64565[36], 6}, {NULL}};
const Segment SGI401384[] = {{&S64566[11], 7}, {NULL}};
const Segment SGI401385[] = {{&S64566[36], 6}, {NULL}};
const Segment SGI401386[] = {{&S64567[11], 7}, {NULL}};
const Segment SGI401387[] = {{&S64567[36], 6}, {NULL}};
const Segment SGI401388[] = {{&S103828[32], 11}, {NULL}};
const Segment SGI401389[] = {{&S103828[27], 4}, {NULL}};
const Segment SGI401390[] = {{&S103828[101], 9}, {NULL}};
const Segment SGI401391[] = {{&S104555[32], 10}, {NULL}};
const Segment SGI401392[] = {{&S104555[27], 4}, {NULL}};
const Segment SGI401393[] = {{&S104555[95], 9}, {NULL}};
const Segment SGI401394[] = {{&S105067[32], 9}, {NULL}};
const Segment SGI401395[] = {{&S105067[27], 4}, {NULL}};
const Segment SGI401396[] = {{&S105067[97], 9}, {NULL}};
const Segment SGI401397[] = {{&S73344[11], 7}, {NULL}};
const Segment SGI401398[] = {{&S73344[36], 6}, {NULL}};
const Segment SGI401399[] = {{&S73345[11], 7}, {NULL}};
const Segment SGI401400[] = {{&S73345[36], 6}, {NULL}};
const Segment SGI401401[] = {{&S73346[11], 7}, {NULL}};
const Segment SGI401402[] = {{&S73346[36], 6}, {NULL}};
const Segment SGI401403[] = {{&S73347[11], 7}, {NULL}};
const Segment SGI401404[] = {{&S73347[36], 6}, {NULL}};
const Segment SGI401405[] = {{&S73348[11], 7}, {NULL}};
const Segment SGI401406[] = {{&S73348[36], 6}, {NULL}};
const Segment SGI401407[] = {{&S73349[11], 7}, {NULL}};
const Segment SGI401408[] = {{&S73349[36], 6}, {NULL}};
const Segment SGI401409[] = {{&S73350[11], 7}, {NULL}};
const Segment SGI401410[] = {{&S73350[36], 6}, {NULL}};
const Segment SGI401411[] = {{&S74283[11], 7}, {NULL}};
const Segment SGI401412[] = {{&S74283[36], 6}, {NULL}};
const Segment SGI401413[] = {{&S74284[11], 7}, {NULL}};
const Segment SGI401414[] = {{&S74284[36], 6}, {NULL}};
const Segment SGI401415[] = {{&S74285[11], 7}, {NULL}};
const Segment SGI401416[] = {{&S74285[36], 6}, {NULL}};
const Segment SGI401417[] = {{&S74286[11], 7}, {NULL}};
const Segment SGI401418[] = {{&S74286[36], 6}, {NULL}};
const Segment SGI401419[] = {{&S74287[11], 7}, {NULL}};
const Segment SGI401420[] = {{&S74287[36], 6}, {NULL}};
const Segment SGI401421[] = {{&S74288[11], 7}, {NULL}};
const Segment SGI401422[] = {{&S74288[36], 6}, {NULL}};
const Segment SGI401423[] = {{&S74289[11], 7}, {NULL}};
const Segment SGI401424[] = {{&S74289[36], 6}, {NULL}};
const Segment SGI401425[] = {{&S74290[11], 7}, {NULL}};
const Segment SGI401426[] = {{&S74290[36], 6}, {NULL}};
const Segment SGI401427[] = {{&S74291[11], 7}, {NULL}};
const Segment SGI401428[] = {{&S74291[36], 6}, {NULL}};
const Segment SGI401429[] = {{&S74292[11], 7}, {NULL}};
const Segment SGI401430[] = {{&S74292[36], 6}, {NULL}};
const Segment SGI401431[] = {{&S99934[43], 17}, {NULL}};
const Segment SGI401432[] = {{&S99934[30], 4}, {NULL}};
const Segment SGI401433[] = {{&S99934[129], 3}, {NULL}};
const Segment SGI401434[] = {{&S99934[95], 5}, {NULL}};
const Segment SGI401435[] = {{&S99934[153], 13}, {NULL}};
const Segment SGI401436[] = {{&S79632[11], 7}, {NULL}};
const Segment SGI401437[] = {{&S79632[36], 6}, {NULL}};
const Segment SGI401438[] = {{&S79633[11], 7}, {NULL}};
const Segment SGI401439[] = {{&S79633[36], 6}, {NULL}};
const Segment SGI401440[] = {{&S79634[11], 7}, {NULL}};
const Segment SGI401441[] = {{&S79634[36], 6}, {NULL}};
const Segment SGI401442[] = {{&S79635[11], 7}, {NULL}};
const Segment SGI401443[] = {{&S79635[36], 6}, {NULL}};
const Segment SGI401444[] = {{&S79636[11], 7}, {NULL}};
const Segment SGI401445[] = {{&S79636[36], 6}, {NULL}};
const Segment SGI401446[] = {{&S80094[11], 7}, {NULL}};
const Segment SGI401447[] = {{&S80094[36], 6}, {NULL}};
const Segment SGI401448[] = {{&S80095[11], 7}, {NULL}};
const Segment SGI401449[] = {{&S80095[36], 6}, {NULL}};
const Segment SGI401450[] = {{&S80096[11], 7}, {NULL}};
const Segment SGI401451[] = {{&S80096[36], 6}, {NULL}};
const Segment SGI401452[] = {{&S80097[11], 7}, {NULL}};
const Segment SGI401453[] = {{&S80097[36], 6}, {NULL}};
const Segment SGI401454[] = {{&S80098[11], 7}, {NULL}};
const Segment SGI401455[] = {{&S80098[36], 6}, {NULL}};
const Segment SGI401456[] = {{&S80099[11], 7}, {NULL}};
const Segment SGI401457[] = {{&S80099[36], 6}, {NULL}};
const Segment SGI401458[] = {{&S102542[36], 17}, {NULL}};
const Segment SGI401459[] = {{&S102542[31], 4}, {NULL}};
const Segment SGI401460[] = {{&S102543[42], 17}, {NULL}};
const Segment SGI401461[] = {{&S102543[42], 38}, {NULL}};
const Segment SGI401462[] = {{&S102543[30], 4}, {NULL}};
const Segment SGI401463[] = {{&S102543[128], 3}, {NULL}};
const Segment SGI401464[] = {{&S102543[94], 5}, {NULL}};
const Segment SGI401465[] = {{&S102544[63], 17}, {NULL}};
const Segment SGI401466[] = {{&S102544[63], 38}, {NULL}};
const Segment SGI401467[] = {{&S102544[51], 4}, {NULL}};
const Segment SGI401468[] = {{&S103824[44], 7}, {NULL}};
const Segment SGI401469[] = {{&S103824[44], 18}, {NULL}};
const Segment SGI401470[] = {{&S103824[30], 6}, {NULL}};
const Segment SGI401471[] = {{&S103824[111], 3}, {NULL}};
const Segment SGI401472[] = {{&S103824[76], 6}, {NULL}};
const Segment SGI401473[] = {{&S84674[11], 8}, {NULL}};
const Segment SGI401474[] = {{&S84674[37], 6}, {NULL}};
const Segment SGI401475[] = {{&S84675[11], 8}, {NULL}};
const Segment SGI401476[] = {{&S84675[37], 6}, {NULL}};
const Segment SGI401477[] = {{&S84676[11], 8}, {NULL}};
const Segment SGI401478[] = {{&S84676[37], 6}, {NULL}};
const Segment SGI401479[] = {{&S84677[11], 8}, {NULL}};
const Segment SGI401480[] = {{&S84677[37], 6}, {NULL}};
const Segment SGI401481[] = {{&S85366[11], 7}, {NULL}};
const Segment SGI401482[] = {{&S85366[36], 6}, {NULL}};
const Segment SGI401483[] = {{&S85367[11], 7}, {NULL}};
const Segment SGI401484[] = {{&S85367[36], 6}, {NULL}};
const Segment SGI401485[] = {{&S85368[11], 7}, {NULL}};
const Segment SGI401486[] = {{&S85368[36], 6}, {NULL}};
const Segment SGI401487[] = {{&S85794[11], 7}, {NULL}};
const Segment SGI401488[] = {{&S85794[36], 6}, {NULL}};
const Segment SGI401489[] = {{&S85795[11], 7}, {NULL}};
const Segment SGI401490[] = {{&S85795[36], 6}, {NULL}};
const Segment SGI401491[] = {{&S85796[11], 7}, {NULL}};
const Segment SGI401492[] = {{&S85796[36], 6}, {NULL}};
const Segment SGI401493[] = {{&S85797[11], 7}, {NULL}};
const Segment SGI401494[] = {{&S85797[36], 6}, {NULL}};
const Segment SGI401495[] = {{&S87094[11], 8}, {NULL}};
const Segment SGI401496[] = {{&S87094[37], 6}, {NULL}};
const Segment SGI401497[] = {{&S87095[11], 8}, {NULL}};
const Segment SGI401498[] = {{&S87095[37], 6}, {NULL}};
const Segment SGI401499[] = {{&S87096[11], 8}, {NULL}};
const Segment SGI401500[] = {{&S87096[37], 6}, {NULL}};
const Segment SGI401501[] = {{&S88709[11], 8}, {NULL}};
const Segment SGI401502[] = {{&S88709[37], 6}, {NULL}};
const Segment SGI401503[] = {{&S88710[11], 8}, {NULL}};
const Segment SGI401504[] = {{&S88710[37], 6}, {NULL}};
const Segment SGI401505[] = {{&S88711[11], 8}, {NULL}};
const Segment SGI401506[] = {{&S88711[37], 6}, {NULL}};
const Segment SGI401507[] = {{&S92495[11], 7}, {NULL}};
const Segment SGI401508[] = {{&S92495[36], 6}, {NULL}};
const Segment SGI401509[] = {{&S95168[11], 8}, {NULL}};
const Segment SGI401510[] = {{&S95168[37], 6}, {NULL}};
const Segment SGI401511[] = {{&S95224[11], 7}, {NULL}};
const Segment SGI401512[] = {{&S95224[36], 6}, {NULL}};
const Segment SGI401513[] = {{&S97674[11], 8}, {NULL}};
const Segment SGI401514[] = {{&S97674[37], 6}, {NULL}};
const Segment SGI401515[] = {{&S98577[11], 8}, {NULL}};
const Segment SGI401516[] = {{&S98577[37], 6}, {NULL}};
const Segment SGI401517[] = {{&S100172[11], 8}, {NULL}};
const Segment SGI401518[] = {{&S100172[37], 6}, {NULL}};
const Segment SGI401519[] = {{&S100175[11], 8}, {NULL}};
const Segment SGI401520[] = {{&S100175[37], 6}, {NULL}};
const Segment * const SG0[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1, SGI2, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3, SGI4, SGI5, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2[] = {SGI6, SGI7, SGI8, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9, SGI0, SGI0, SGI10, SGI10, SGI11, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13, NULL};
const Segment * const SG4[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5[] = {SGI15, SGI16, SGI17, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18, SGI0, SGI0, SGI19, SGI19, SGI20, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6[] = {SGI21, SGI22, SGI23, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24, SGI0, SGI0, SGI25, SGI25, SGI26, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27, SGI28, SGI29, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30, SGI31, SGI32, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33, SGI34, SGI35, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36, SGI37, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11[] = {SGI38, SGI39, SGI40, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41, SGI0, SGI0, SGI0, SGI42, SGI42, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12[] = {SGI43, SGI44, SGI45, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46, SGI0, SGI0, SGI0, SGI47, SGI47, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13[] = {SGI48, SGI49, SGI50, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51, SGI0, SGI0, SGI0, SGI52, SGI52, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14[] = {SGI53, SGI54, SGI55, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56, SGI0, SGI0, SGI0, SGI57, SGI57, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15[] = {SGI58, SGI59, SGI60, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61, SGI0, SGI0, SGI0, SGI62, SGI62, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16[] = {SGI63, SGI64, SGI65, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66, SGI0, SGI0, SGI0, SGI67, SGI67, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17[] = {SGI68, SGI69, SGI70, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71, SGI0, SGI0, SGI0, SGI72, SGI72, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18[] = {SGI73, SGI0, SGI74, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75, SGI0, SGI0, SGI0, SGI76, SGI76, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19[] = {SGI77, SGI78, SGI79, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83, SGI0, SGI0, SGI0, SGI85, SGI85, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20[] = {SGI80, SGI81, SGI82, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84, SGI0, SGI0, SGI0, SGI86, SGI86, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21[] = {SGI87, SGI88, SGI89, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90, SGI0, SGI0, SGI0, SGI91, SGI91, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22[] = {SGI92, SGI93, SGI94, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95, SGI0, SGI0, SGI0, SGI97, SGI97, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23[] = {SGI96, SGI98, SGI99, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100, SGI0, SGI0, SGI0, SGI101, SGI101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24[] = {SGI102, SGI103, SGI104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105, SGI0, SGI0, SGI0, SGI106, SGI106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25[] = {SGI107, SGI108, SGI109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110, SGI0, SGI0, SGI0, SGI111, SGI111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26[] = {SGI112, SGI113, SGI115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118, SGI0, SGI0, SGI0, SGI120, SGI120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27[] = {SGI114, SGI116, SGI117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119, SGI0, SGI0, SGI0, SGI121, SGI121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28[] = {SGI122, SGI123, SGI125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128, SGI0, SGI0, SGI0, SGI130, SGI130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29[] = {SGI124, SGI126, SGI127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129, SGI0, SGI0, SGI0, SGI131, SGI131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30[] = {SGI132, SGI133, SGI136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138, SGI0, SGI0, SGI0, SGI140, SGI140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31[] = {SGI134, SGI135, SGI137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139, SGI0, SGI0, SGI0, SGI141, SGI141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32[] = {SGI142, SGI144, SGI145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148, SGI0, SGI0, SGI0, SGI150, SGI150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33[] = {SGI143, SGI146, SGI147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149, SGI0, SGI0, SGI0, SGI151, SGI151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34[] = {SGI152, SGI153, SGI154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155, SGI0, SGI0, SGI0, SGI156, SGI156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35[] = {SGI157, SGI158, SGI159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160, SGI0, SGI0, SGI0, SGI161, SGI161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36[] = {SGI162, SGI163, SGI164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165, SGI0, SGI0, SGI0, SGI166, SGI166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37[] = {SGI167, SGI168, SGI169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170, SGI0, SGI0, SGI0, SGI171, SGI171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38[] = {SGI172, SGI0, SGI173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39[] = {SGI174, SGI175, SGI176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177, SGI0, SGI0, SGI0, SGI178, SGI178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40[] = {SGI179, SGI180, SGI181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182, SGI0, SGI0, SGI0, SGI185, SGI185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41[] = {SGI183, SGI184, SGI186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187, SGI0, SGI0, SGI0, SGI188, SGI188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42[] = {SGI189, SGI190, SGI191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192, SGI0, SGI0, SGI0, SGI193, SGI193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43[] = {SGI194, SGI195, SGI196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197, SGI0, SGI0, SGI0, SGI198, SGI199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44[] = {SGI200, SGI0, SGI201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45[] = {SGI202, SGI0, SGI203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204, SGI0, SGI0, SGI0, SGI205, SGI205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46[] = {SGI206, SGI207, SGI208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209, SGI0, SGI0, SGI0, SGI210, SGI210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47[] = {SGI211, SGI212, SGI213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214, SGI215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48[] = {SGI216, SGI217, SGI218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219, SGI0, SGI0, SGI0, SGI0, SGI220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49[] = {SGI221, SGI222, SGI223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224, SGI224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50[] = {SGI225, SGI0, SGI226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51[] = {SGI227, SGI228, SGI229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230, SGI0, SGI0, SGI0, SGI231, SGI231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52[] = {SGI0, SGI232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53[] = {SGI233, SGI234, SGI235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236, SGI0, SGI0, SGI237, SGI237, SGI238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54[] = {SGI239, SGI240, SGI241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242, SGI0, SGI0, SGI243, SGI243, SGI244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55[] = {SGI245, SGI246, SGI247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248, SGI0, SGI0, SGI249, SGI249, SGI250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56[] = {SGI251, SGI252, SGI253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI254, SGI0, SGI0, SGI255, SGI255, SGI256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57[] = {SGI257, SGI258, SGI259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI260, SGI0, SGI0, SGI261, SGI261, SGI262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58[] = {SGI263, SGI264, SGI265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266, SGI0, SGI0, SGI267, SGI267, SGI268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59[] = {SGI269, SGI270, SGI271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI272, SGI0, SGI0, SGI273, SGI273, SGI274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60[] = {SGI275, SGI276, SGI277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278, SGI0, SGI0, SGI279, SGI279, SGI280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283, SGI0, SGI0, SGI284, SGI284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64[] = {SGI285, SGI0, SGI286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287, SGI0, SGI0, SGI288, SGI288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289, SGI290, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66[] = {SGI291, SGI292, SGI293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294, SGI0, SGI0, SGI0, SGI295, SGI295, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67[] = {SGI296, SGI297, SGI298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299, SGI0, SGI0, SGI0, SGI300, SGI300, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68[] = {SGI0, SGI301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302, SGI0, SGI0, SGI0, SGI303, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69[] = {SGI304, SGI305, SGI306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307, SGI0, SGI0, SGI0, SGI308, SGI308, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70[] = {SGI309, SGI310, SGI311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312, SGI0, SGI0, SGI0, SGI313, SGI313, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71[] = {SGI314, SGI315, SGI316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317, SGI0, SGI0, SGI0, SGI318, SGI318, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72[] = {SGI319, SGI320, SGI321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322, SGI0, SGI0, SGI0, SGI323, SGI323, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73[] = {SGI324, SGI325, SGI326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327, SGI0, SGI0, SGI0, SGI328, SGI328, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74[] = {SGI329, SGI0, SGI330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331, SGI0, SGI0, SGI0, SGI332, SGI332, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75[] = {SGI333, SGI334, SGI335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336, SGI0, SGI0, SGI0, SGI337, SGI337, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76[] = {SGI0, SGI338, SGI339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340, SGI0, SGI0, SGI0, SGI341, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77[] = {SGI342, SGI343, SGI344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345, SGI0, SGI0, SGI0, SGI346, SGI346, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78[] = {SGI347, SGI0, SGI348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79[] = {SGI349, SGI350, SGI351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352, SGI0, SGI0, SGI0, SGI353, SGI353, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80[] = {SGI354, SGI355, SGI356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357, SGI358, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81[] = {SGI359, SGI0, SGI360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82[] = {SGI361, SGI362, SGI363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI364, SGI365, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83[] = {SGI366, SGI367, SGI368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI369, SGI370, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84[] = {SGI371, SGI372, SGI373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI374, SGI375, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85[] = {SGI376, SGI377, SGI378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379, SGI379, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86[] = {SGI380, SGI381, SGI382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383, SGI383, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87[] = {SGI384, SGI385, SGI386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI387, SGI0, SGI0, SGI0, SGI388, SGI388, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88[] = {SGI389, SGI390, SGI391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392, SGI0, SGI0, SGI393, SGI393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI394, SGI395, SGI396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397, SGI398, SGI399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91[] = {SGI400, SGI401, SGI402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI403, SGI0, SGI0, SGI0, SGI404, SGI404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92[] = {SGI405, SGI0, SGI406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI407, SGI0, SGI0, SGI0, SGI408, SGI408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93[] = {SGI409, SGI410, SGI411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI412, SGI0, SGI0, SGI0, SGI413, SGI413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94[] = {SGI414, SGI415, SGI416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI417, SGI0, SGI0, SGI0, SGI418, SGI418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95[] = {SGI419, SGI420, SGI421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI422, SGI0, SGI0, SGI0, SGI423, SGI423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96[] = {SGI424, SGI0, SGI425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97[] = {SGI426, SGI427, SGI428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI429, SGI0, SGI0, SGI0, SGI430, SGI430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG98[] = {SGI431, SGI432, SGI433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI434, SGI435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99[] = {SGI436, SGI437, SGI438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI439, SGI439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100[] = {SGI440, SGI441, SGI442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI443, SGI0, SGI0, SGI0, SGI444, SGI444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101[] = {SGI445, SGI446, SGI447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI448, SGI0, SGI0, SGI0, SGI449, SGI449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG102[] = {SGI450, SGI451, SGI452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI453, SGI0, SGI0, SGI0, SGI454, SGI454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG103[] = {SGI455, SGI0, SGI456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG104[] = {SGI457, SGI458, SGI459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI460, SGI461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG105[] = {SGI462, SGI0, SGI463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG106[] = {SGI464, SGI465, SGI466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI467, SGI467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG107[] = {SGI468, SGI0, SGI469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG108[] = {SGI0, SGI470, SGI471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG109[] = {SGI472, SGI473, SGI474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI475, SGI0, SGI0, SGI0, SGI476, SGI477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG110[] = {SGI478, SGI479, SGI480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI481, SGI0, SGI0, SGI0, SGI482, SGI483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG111[] = {SGI484, SGI0, SGI485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI486, SGI0, SGI0, SGI0, SGI487, SGI487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG112[] = {SGI488, SGI489, SGI490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG113[] = {SGI491, SGI0, SGI492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI493, SGI0, SGI0, SGI0, SGI494, SGI494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG114[] = {SGI0, SGI495, SGI496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG115[] = {SGI497, SGI498, SGI499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI500, SGI0, SGI0, SGI0, SGI501, SGI501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG116[] = {SGI502, SGI503, SGI504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI505, SGI0, SGI0, SGI0, SGI506, SGI507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG117[] = {SGI508, SGI509, SGI510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG118[] = {SGI511, SGI512, SGI513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI514, SGI0, SGI0, SGI0, SGI515, SGI515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG119[] = {SGI0, SGI516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI517, SGI0, SGI0, SGI0, SGI518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG120[] = {SGI519, SGI520, SGI521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI522, SGI0, SGI0, SGI0, SGI523, SGI523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG121[] = {SGI524, SGI525, SGI526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI527, SGI0, SGI0, SGI0, SGI528, SGI528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG122[] = {SGI529, SGI530, SGI531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI532, SGI0, SGI0, SGI0, SGI533, SGI533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG123[] = {SGI534, SGI535, SGI536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI537, SGI0, SGI0, SGI0, SGI538, SGI538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG124[] = {SGI539, SGI0, SGI540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI541, SGI0, SGI0, SGI0, SGI542, SGI542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG125[] = {SGI543, SGI544, SGI545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI546, SGI0, SGI0, SGI0, SGI547, SGI547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG126[] = {SGI548, SGI549, SGI550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI551, SGI0, SGI0, SGI0, SGI552, SGI552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG127[] = {SGI553, SGI0, SGI554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG128[] = {SGI555, SGI556, SGI557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI558, SGI0, SGI0, SGI0, SGI559, SGI559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG129[] = {SGI560, SGI561, SGI562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI563, SGI0, SGI0, SGI0, SGI564, SGI564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG130[] = {SGI565, SGI0, SGI566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG131[] = {SGI567, SGI568, SGI569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI570, SGI571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG132[] = {SGI572, SGI0, SGI573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG133[] = {SGI574, SGI575, SGI576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI577, SGI577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG134[] = {SGI0, SGI578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI579, SGI0, SGI0, SGI0, SGI580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG135[] = {SGI581, SGI582, SGI583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI584, SGI0, SGI0, SGI0, SGI585, SGI585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG136[] = {SGI586, SGI587, SGI588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI589, SGI0, SGI0, SGI0, SGI590, SGI590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG137[] = {SGI591, SGI592, SGI593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI594, SGI0, SGI0, SGI0, SGI595, SGI595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG138[] = {SGI596, SGI597, SGI598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI599, SGI0, SGI0, SGI0, SGI600, SGI600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG139[] = {SGI601, SGI602, SGI603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI604, SGI0, SGI0, SGI0, SGI605, SGI605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG140[] = {SGI606, SGI607, SGI608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI609, SGI0, SGI0, SGI0, SGI610, SGI610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG141[] = {SGI611, SGI612, SGI613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI614, SGI0, SGI0, SGI0, SGI615, SGI615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG142[] = {SGI616, SGI617, SGI618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI619, SGI0, SGI0, SGI0, SGI620, SGI620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG143[] = {SGI621, SGI622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI623, SGI0, SGI0, SGI0, SGI624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG144[] = {SGI625, SGI0, SGI626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG145[] = {SGI627, SGI0, SGI628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG146[] = {SGI629, SGI630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI631, SGI0, SGI0, SGI0, SGI632, SGI632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG147[] = {SGI633, SGI634, SGI635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG148[] = {SGI636, SGI0, SGI637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG149[] = {SGI638, SGI0, SGI639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG150[] = {SGI640, SGI0, SGI641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG151[] = {SGI642, SGI0, SGI643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI644, SGI0, SGI0, SGI0, SGI645, SGI645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG152[] = {SGI646, SGI0, SGI647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG153[] = {SGI648, SGI649, SGI650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI651, SGI652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG154[] = {SGI653, SGI654, SGI655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI656, SGI656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG155[] = {SGI657, SGI0, SGI658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI659, SGI0, SGI0, SGI0, SGI660, SGI660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG156[] = {SGI661, SGI662, SGI663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI664, SGI0, SGI0, SGI0, SGI665, SGI665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG157[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI667, NULL};
const Segment * const SG158[] = {SGI0, SGI668, SGI669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI670, SGI0, SGI0, SGI0, SGI671, SGI672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG159[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI673, SGI0, SGI0, SGI0, SGI674, SGI674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG160[] = {SGI675, SGI0, SGI676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI677, SGI0, SGI0, SGI0, SGI678, SGI678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG161[] = {SGI679, SGI680, SGI681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI682, SGI0, SGI0, SGI0, SGI683, SGI683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG162[] = {SGI684, SGI685, SGI686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI687, SGI0, SGI0, SGI0, SGI688, SGI688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG163[] = {SGI689, SGI690, SGI691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG164[] = {SGI692, SGI693, SGI694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI695, SGI0, SGI0, SGI0, SGI696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG165[] = {SGI697, SGI0, SGI698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI699, SGI0, SGI0, SGI0, SGI700, SGI700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG166[] = {SGI701, SGI702, SGI703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG167[] = {SGI704, SGI705, SGI706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI707, SGI0, SGI0, SGI0, SGI708, SGI709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG168[] = {SGI710, SGI711, SGI712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI713, SGI0, SGI0, SGI0, SGI714, SGI714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG169[] = {SGI715, SGI716, SGI717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI718, SGI0, SGI0, SGI0, SGI719, SGI719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG170[] = {SGI0, SGI720, SGI721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG171[] = {SGI722, SGI723, SGI724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI725, SGI0, SGI0, SGI0, SGI726, SGI726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG172[] = {SGI727, SGI728, SGI729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI730, SGI0, SGI0, SGI0, SGI731, SGI731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG173[] = {SGI0, SGI732, SGI733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG174[] = {SGI734, SGI735, SGI736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI737, SGI0, SGI0, SGI0, SGI738, SGI738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG175[] = {SGI742, SGI743, SGI744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI745, SGI0, SGI0, SGI746, SGI746, SGI747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG176[] = {SGI739, SGI740, SGI741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG177[] = {SGI749, SGI750, SGI751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI752, SGI0, SGI0, SGI0, SGI753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG178[] = {SGI754, SGI755, SGI756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI757, SGI0, SGI0, SGI0, SGI758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG179[] = {SGI759, SGI760, SGI761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG180[] = {SGI762, SGI763, SGI764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI765, SGI765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG181[] = {SGI766, SGI767, SGI768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG182[] = {SGI769, SGI770, SGI771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG183[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG184[] = {SGI772, SGI773, SGI774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG185[] = {SGI776, SGI777, SGI778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG186[] = {SGI779, SGI780, SGI781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG187[] = {SGI782, SGI783, SGI784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG188[] = {SGI785, SGI786, SGI787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG189[] = {SGI788, SGI789, SGI790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI791, SGI0, SGI0, SGI0, SGI792, SGI792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG190[] = {SGI793, SGI794, SGI795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG191[] = {SGI796, SGI797, SGI798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG192[] = {SGI799, SGI800, SGI801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG193[] = {SGI802, SGI0, SGI803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI804, SGI0, SGI0, SGI0, SGI805, SGI805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG194[] = {SGI806, SGI807, SGI808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI809, SGI0, SGI0, SGI0, SGI810, SGI810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG195[] = {SGI811, SGI0, SGI812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI813, SGI0, SGI0, SGI0, SGI814, SGI814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG196[] = {SGI815, SGI816, SGI817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG197[] = {SGI818, SGI819, SGI820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG198[] = {SGI821, SGI822, SGI823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI824, SGI0, SGI0, SGI0, SGI825, SGI825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG199[] = {SGI826, SGI827, SGI828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI829, SGI0, SGI0, SGI0, SGI830, SGI831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG200[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG201[] = {SGI833, SGI834, SGI835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG202[] = {SGI836, SGI837, SGI838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG203[] = {SGI839, SGI840, SGI841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG204[] = {SGI842, SGI843, SGI844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG205[] = {SGI0, SGI845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG206[] = {SGI846, SGI847, SGI848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG207[] = {SGI849, SGI850, SGI851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI852, SGI0, SGI0, SGI0, SGI853, SGI853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG208[] = {SGI854, SGI855, SGI856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI857, SGI0, SGI0, SGI0, SGI858, SGI858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG209[] = {SGI859, SGI860, SGI861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI862, SGI0, SGI0, SGI0, SGI863, SGI863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG210[] = {SGI864, SGI865, SGI866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI867, SGI0, SGI0, SGI0, SGI868, SGI869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG211[] = {SGI870, SGI871, SGI872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI873, SGI0, SGI0, SGI0, SGI874, SGI874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG212[] = {SGI875, SGI876, SGI877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI878, SGI0, SGI0, SGI0, SGI879, SGI879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG213[] = {SGI880, SGI881, SGI882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG214[] = {SGI883, SGI884, SGI885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG215[] = {SGI0, SGI886, SGI887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG216[] = {SGI888, SGI889, SGI890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI891, SGI0, SGI0, SGI0, SGI892, SGI892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG217[] = {SGI893, SGI894, SGI895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI896, SGI0, SGI0, SGI0, SGI897, SGI897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG218[] = {SGI0, SGI0, SGI898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG219[] = {SGI899, SGI900, SGI901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI902, SGI0, SGI0, SGI0, SGI903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG220[] = {SGI0, SGI904, SGI905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG221[] = {SGI906, SGI907, SGI908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI909, SGI0, SGI0, SGI0, SGI910, SGI911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG222[] = {SGI912, SGI913, SGI914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI915, SGI0, SGI0, SGI0, SGI916, SGI917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG223[] = {SGI918, SGI919, SGI920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG224[] = {SGI921, SGI922, SGI923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG225[] = {SGI925, SGI926, SGI927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI928, SGI0, SGI0, SGI0, SGI929, SGI930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG226[] = {SGI931, SGI932, SGI933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI934, SGI0, SGI0, SGI0, SGI935, SGI936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG227[] = {SGI937, SGI938, SGI939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI940, SGI0, SGI0, SGI0, SGI941, SGI941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG228[] = {SGI942, SGI943, SGI944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI945, SGI0, SGI0, SGI0, SGI946, SGI946, SGI0, SGI0, SGI0, SGI0, SGI947, SGI0, SGI0, SGI0, NULL};
const Segment * const SG229[] = {SGI948, SGI949, SGI950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI951, SGI0, SGI0, SGI0, SGI952, SGI953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG230[] = {SGI954, SGI955, SGI956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI957, SGI0, SGI0, SGI0, SGI958, SGI958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG231[] = {SGI959, SGI960, SGI961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI962, SGI0, SGI0, SGI0, SGI0, SGI963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG232[] = {SGI964, SGI0, SGI965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI966, SGI0, SGI0, SGI0, SGI967, SGI968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG233[] = {SGI969, SGI970, SGI971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI972, SGI0, SGI0, SGI0, SGI973, SGI973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG234[] = {SGI0, SGI974, SGI975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG235[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI976, SGI977, SGI978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG236[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI979, SGI980, SGI981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG237[] = {SGI982, SGI983, SGI984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI985, SGI986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG238[] = {SGI987, SGI988, SGI989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI990, SGI0, SGI0, SGI0, SGI991, SGI991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG239[] = {SGI992, SGI993, SGI994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI995, SGI0, SGI0, SGI0, SGI996, SGI996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG240[] = {SGI997, SGI998, SGI999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1000, SGI0, SGI0, SGI0, SGI1001, SGI1001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG241[] = {SGI1002, SGI0, SGI1003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1004, SGI0, SGI0, SGI0, SGI1005, SGI1005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG242[] = {SGI1006, SGI1007, SGI1008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1009, SGI0, SGI0, SGI0, SGI1010, SGI1010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG243[] = {SGI1011, SGI1012, SGI1013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1014, SGI0, SGI0, SGI0, SGI1015, SGI1016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG244[] = {SGI1017, SGI0, SGI1018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1019, SGI0, SGI0, SGI0, SGI1020, SGI1020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG245[] = {SGI1021, SGI0, SGI1022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1023, SGI0, SGI0, SGI0, SGI1024, SGI1024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG246[] = {SGI1025, SGI1026, SGI1027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1028, SGI0, SGI0, SGI0, SGI1029, SGI1029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG247[] = {SGI1030, SGI1031, SGI1032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1033, SGI0, SGI0, SGI0, SGI1034, SGI1035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG248[] = {SGI1036, SGI0, SGI1037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG249[] = {SGI1038, SGI0, SGI1039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG250[] = {SGI1040, SGI1041, SGI1042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1043, SGI1044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG251[] = {SGI1045, SGI1046, SGI1047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1048, SGI0, SGI0, SGI0, SGI1049, SGI1049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG252[] = {SGI1050, SGI1051, SGI1052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1053, SGI0, SGI0, SGI0, SGI1054, SGI1055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG253[] = {SGI1056, SGI1057, SGI1058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1059, SGI0, SGI0, SGI0, SGI1060, SGI1060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG254[] = {SGI1061, SGI1062, SGI1063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1064, SGI0, SGI0, SGI0, SGI1065, SGI1065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG255[] = {SGI0, SGI1066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1067, SGI0, SGI0, SGI0, SGI1068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG256[] = {SGI1069, SGI1070, SGI1071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1072, SGI0, SGI0, SGI0, SGI1073, SGI1073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG257[] = {SGI1074, SGI0, SGI1075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1076, SGI0, SGI0, SGI0, SGI1077, SGI1077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG258[] = {SGI1078, SGI1079, SGI1080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1081, SGI0, SGI0, SGI0, SGI1082, SGI1082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG259[] = {SGI1083, SGI1084, SGI1085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1086, SGI0, SGI0, SGI0, SGI1087, SGI1087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG260[] = {SGI1088, SGI1089, SGI1090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1091, SGI0, SGI0, SGI0, SGI1092, SGI1092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG261[] = {SGI1093, SGI0, SGI1094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG262[] = {SGI1095, SGI0, SGI1096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG263[] = {SGI1097, SGI1098, SGI1099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1101, SGI0, SGI0, SGI1103, SGI1103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG264[] = {SGI1100, SGI1102, SGI1104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1105, SGI0, SGI0, SGI0, SGI1106, SGI1107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG265[] = {SGI1108, SGI1109, SGI1110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1111, SGI0, SGI0, SGI1112, SGI1112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG266[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1113, NULL};
const Segment * const SG267[] = {SGI1114, SGI1115, SGI1116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1117, SGI0, SGI0, SGI0, SGI1118, SGI1118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG268[] = {SGI1119, SGI1120, SGI1121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1122, SGI0, SGI0, SGI0, SGI1123, SGI1123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG269[] = {SGI1124, SGI1125, SGI1126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1127, SGI0, SGI0, SGI0, SGI1128, SGI1128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG270[] = {SGI1129, SGI1130, SGI1131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1132, SGI0, SGI0, SGI0, SGI1133, SGI1133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG271[] = {SGI1134, SGI1135, SGI1136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1137, SGI0, SGI0, SGI0, SGI1138, SGI1138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG272[] = {SGI1139, SGI1140, SGI1141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1142, SGI0, SGI0, SGI0, SGI1143, SGI1143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG273[] = {SGI1144, SGI1145, SGI1146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1147, SGI0, SGI0, SGI0, SGI1148, SGI1148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG274[] = {SGI1149, SGI0, SGI1150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG275[] = {SGI1151, SGI1152, SGI1153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1154, SGI1155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG276[] = {SGI1156, SGI0, SGI1157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG277[] = {SGI1158, SGI0, SGI1159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG278[] = {SGI0, SGI1160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1161, SGI0, SGI0, SGI0, SGI1162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG279[] = {SGI1163, SGI0, SGI1164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1165, SGI0, SGI0, SGI0, SGI1166, SGI1166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG280[] = {SGI1167, SGI1168, SGI1169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1170, SGI1170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG281[] = {SGI1171, SGI1172, SGI1173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1174, SGI0, SGI0, SGI1175, SGI1175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG282[] = {SGI1176, SGI1177, SGI1178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1179, SGI0, SGI0, SGI0, SGI1180, SGI1180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG283[] = {SGI1181, SGI1182, SGI1183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1184, SGI0, SGI0, SGI0, SGI1185, SGI1185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG284[] = {SGI1186, SGI1187, SGI1188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1189, SGI0, SGI0, SGI0, SGI1190, SGI1190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG285[] = {SGI1191, SGI1192, SGI1193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1194, SGI0, SGI0, SGI0, SGI1195, SGI1195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG286[] = {SGI1196, SGI1197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1198, SGI0, SGI0, SGI0, SGI1199, SGI1199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG287[] = {SGI1200, SGI1201, SGI1202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1203, SGI0, SGI0, SGI0, SGI1204, SGI1204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG288[] = {SGI1205, SGI1206, SGI1207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1208, SGI0, SGI0, SGI0, SGI1209, SGI1209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG289[] = {SGI1210, SGI1211, SGI1212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1213, SGI0, SGI0, SGI0, SGI1214, SGI1214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG290[] = {SGI1215, SGI1216, SGI1217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1218, SGI0, SGI0, SGI0, SGI1219, SGI1219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG291[] = {SGI1220, SGI0, SGI1221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1222, SGI0, SGI0, SGI0, SGI1223, SGI1223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG292[] = {SGI1224, SGI1225, SGI1226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1227, SGI0, SGI0, SGI0, SGI1228, SGI1228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG293[] = {SGI1229, SGI1230, SGI1231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1232, SGI0, SGI0, SGI0, SGI1233, SGI1233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG294[] = {SGI1234, SGI1235, SGI1236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1237, SGI0, SGI0, SGI0, SGI1238, SGI1238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG295[] = {SGI1239, SGI1240, SGI1241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1242, SGI0, SGI0, SGI0, SGI1243, SGI1243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG296[] = {SGI1244, SGI1245, SGI1246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1247, SGI0, SGI0, SGI0, SGI1248, SGI1248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG297[] = {SGI1249, SGI1250, SGI1251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1252, SGI0, SGI0, SGI0, SGI1253, SGI1253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG298[] = {SGI1254, SGI1255, SGI1256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1257, SGI0, SGI0, SGI0, SGI1258, SGI1258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG299[] = {SGI1259, SGI1260, SGI1261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1262, SGI0, SGI0, SGI0, SGI1263, SGI1263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG300[] = {SGI1264, SGI1265, SGI1266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1267, SGI0, SGI0, SGI0, SGI1268, SGI1268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG301[] = {SGI1269, SGI1270, SGI1271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1272, SGI0, SGI0, SGI0, SGI1273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG302[] = {SGI1274, SGI1275, SGI1276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1277, SGI0, SGI0, SGI0, SGI1278, SGI1278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG303[] = {SGI1279, SGI1280, SGI1281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1282, SGI0, SGI0, SGI0, SGI1283, SGI1283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG304[] = {SGI1284, SGI1285, SGI1286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1287, SGI0, SGI0, SGI0, SGI1288, SGI1288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG305[] = {SGI1289, SGI1290, SGI1291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1292, SGI0, SGI0, SGI0, SGI1293, SGI1293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG306[] = {SGI1294, SGI0, SGI1295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG307[] = {SGI1296, SGI1297, SGI1298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1299, SGI1300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG308[] = {SGI1301, SGI1302, SGI1303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1304, SGI1305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG309[] = {SGI1306, SGI1307, SGI1308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1309, SGI1310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG310[] = {SGI1311, SGI1312, SGI1313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1314, SGI1315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG311[] = {SGI1316, SGI1317, SGI1318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1319, SGI1319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG312[] = {SGI1320, SGI1321, SGI1322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1323, SGI1323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG313[] = {SGI1324, SGI1325, SGI1326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1327, SGI0, SGI0, SGI0, SGI1328, SGI1328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG314[] = {SGI1329, SGI1330, SGI1331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1332, SGI0, SGI0, SGI0, SGI1333, SGI1333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG315[] = {SGI1334, SGI1335, SGI1336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1337, SGI0, SGI0, SGI0, SGI1338, SGI1338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG316[] = {SGI1339, SGI1340, SGI1341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1342, SGI0, SGI0, SGI0, SGI1343, SGI1343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG317[] = {SGI1344, SGI1345, SGI1346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1347, SGI0, SGI0, SGI0, SGI1348, SGI1348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG318[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1349, SGI0, SGI0, SGI0, SGI1350, SGI1350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG319[] = {SGI1351, SGI0, SGI1352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG320[] = {SGI1353, SGI0, SGI1354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG321[] = {SGI1355, SGI1356, SGI1357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1358, SGI1359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG322[] = {SGI1360, SGI0, SGI1361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG323[] = {SGI1362, SGI1363, SGI1364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1365, SGI1365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG324[] = {SGI1366, SGI0, SGI1367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1368, SGI0, SGI0, SGI0, SGI1369, SGI1369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG325[] = {SGI1370, SGI0, SGI1371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG326[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1372, NULL};
const Segment * const SG327[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1373, NULL};
const Segment * const SG328[] = {SGI1374, SGI1375, SGI1376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1377, SGI0, SGI0, SGI0, SGI1378, SGI1378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG329[] = {SGI1379, SGI1380, SGI1381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1382, SGI0, SGI0, SGI0, SGI1383, SGI1383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG330[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1384, NULL};
const Segment * const SG331[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1385, SGI1386, SGI1387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG332[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1388, NULL};
const Segment * const SG333[] = {SGI1389, SGI1390, SGI1391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1392, SGI0, SGI0, SGI0, SGI1393, SGI1393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG334[] = {SGI1394, SGI1395, SGI1396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1397, SGI0, SGI0, SGI0, SGI1398, SGI1398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG335[] = {SGI1399, SGI1400, SGI1401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1402, SGI0, SGI0, SGI0, SGI1403, SGI1403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG336[] = {SGI1404, SGI1405, SGI1406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1407, SGI0, SGI0, SGI0, SGI1408, SGI1408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG337[] = {SGI1409, SGI1410, SGI1411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1412, SGI0, SGI0, SGI0, SGI1413, SGI1413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG338[] = {SGI1414, SGI1415, SGI1416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1417, SGI0, SGI0, SGI0, SGI1418, SGI1418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG339[] = {SGI1419, SGI1420, SGI1421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1422, SGI0, SGI0, SGI0, SGI1423, SGI1423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG340[] = {SGI1424, SGI1425, SGI1426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1427, SGI0, SGI0, SGI0, SGI1428, SGI1428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG341[] = {SGI1429, SGI1430, SGI1431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1432, SGI0, SGI0, SGI0, SGI1433, SGI1433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG342[] = {SGI1434, SGI1435, SGI1436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1437, SGI0, SGI0, SGI0, SGI1438, SGI1438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG343[] = {SGI1439, SGI1440, SGI1441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1442, SGI0, SGI0, SGI0, SGI1443, SGI1443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG344[] = {SGI1444, SGI1445, SGI1446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1447, SGI0, SGI0, SGI0, SGI1448, SGI1448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG345[] = {SGI1449, SGI1450, SGI1451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1452, SGI0, SGI0, SGI0, SGI1453, SGI1453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG346[] = {SGI1454, SGI1455, SGI1456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1457, SGI0, SGI0, SGI0, SGI1458, SGI1458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG347[] = {SGI1459, SGI1460, SGI1461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1462, SGI0, SGI0, SGI0, SGI1463, SGI1463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG348[] = {SGI1464, SGI1465, SGI1466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1467, SGI0, SGI0, SGI0, SGI1468, SGI1468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG349[] = {SGI1469, SGI1470, SGI1471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1472, SGI0, SGI0, SGI0, SGI1473, SGI1473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG350[] = {SGI1474, SGI1475, SGI1476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1477, SGI0, SGI0, SGI0, SGI1478, SGI1478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG351[] = {SGI1479, SGI1480, SGI1481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1482, SGI0, SGI0, SGI0, SGI1483, SGI1483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG352[] = {SGI1484, SGI1485, SGI1486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1487, SGI0, SGI0, SGI0, SGI1488, SGI1488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG353[] = {SGI1489, SGI1490, SGI1491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1492, SGI0, SGI0, SGI0, SGI1493, SGI1493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG354[] = {SGI1494, SGI1495, SGI1496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1497, SGI0, SGI0, SGI0, SGI1498, SGI1498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG355[] = {SGI1499, SGI1500, SGI1501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1502, SGI0, SGI0, SGI0, SGI1503, SGI1503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG356[] = {SGI1504, SGI1505, SGI1506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1507, SGI0, SGI0, SGI0, SGI1508, SGI1508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG357[] = {SGI1509, SGI1510, SGI1511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1512, SGI0, SGI0, SGI0, SGI1513, SGI1513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG358[] = {SGI1514, SGI1515, SGI1516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1517, SGI0, SGI0, SGI0, SGI1518, SGI1518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG359[] = {SGI1519, SGI1520, SGI1521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1522, SGI0, SGI0, SGI0, SGI1523, SGI1523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG360[] = {SGI1524, SGI1525, SGI1526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1527, SGI0, SGI0, SGI0, SGI1528, SGI1528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG361[] = {SGI1529, SGI1530, SGI1531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1532, SGI0, SGI0, SGI0, SGI1533, SGI1533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG362[] = {SGI1534, SGI1535, SGI1536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1537, SGI0, SGI0, SGI0, SGI1538, SGI1538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG363[] = {SGI0, SGI1539, SGI1540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1541, SGI0, SGI0, SGI0, SGI1542, SGI1542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG364[] = {SGI1543, SGI1544, SGI1545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1546, SGI0, SGI0, SGI0, SGI1547, SGI1547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG365[] = {SGI1548, SGI1549, SGI1550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1551, SGI0, SGI0, SGI0, SGI1552, SGI1552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG366[] = {SGI1553, SGI1554, SGI1555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1556, SGI0, SGI0, SGI0, SGI1557, SGI1557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG367[] = {SGI1558, SGI1559, SGI1560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1561, SGI0, SGI0, SGI0, SGI1562, SGI1562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG368[] = {SGI1563, SGI1564, SGI1565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1566, SGI0, SGI0, SGI0, SGI1567, SGI1567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG369[] = {SGI1568, SGI1569, SGI1570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1571, SGI0, SGI0, SGI0, SGI1572, SGI1572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG370[] = {SGI1573, SGI1574, SGI1575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1576, SGI0, SGI0, SGI0, SGI1577, SGI1577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG371[] = {SGI1578, SGI1579, SGI1580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1581, SGI0, SGI0, SGI0, SGI1582, SGI1582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG372[] = {SGI1583, SGI1584, SGI1585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1586, SGI0, SGI0, SGI0, SGI1587, SGI1587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG373[] = {SGI1588, SGI1589, SGI1590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1591, SGI0, SGI0, SGI0, SGI1592, SGI1592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG374[] = {SGI1593, SGI1594, SGI1595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1596, SGI0, SGI0, SGI0, SGI1597, SGI1597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG375[] = {SGI1598, SGI1599, SGI1600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1601, SGI0, SGI0, SGI0, SGI1602, SGI1602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG376[] = {SGI1603, SGI1604, SGI1605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1606, SGI0, SGI0, SGI0, SGI1607, SGI1607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG377[] = {SGI1608, SGI1609, SGI1610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1611, SGI0, SGI0, SGI0, SGI1612, SGI1612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG378[] = {SGI1613, SGI1614, SGI1615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1616, SGI0, SGI0, SGI0, SGI1617, SGI1617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG379[] = {SGI0, SGI1618, SGI1619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1620, SGI0, SGI0, SGI0, SGI1621, SGI1621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG380[] = {SGI1622, SGI1623, SGI1624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1625, SGI0, SGI0, SGI0, SGI1626, SGI1626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG381[] = {SGI1627, SGI1628, SGI1629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1630, SGI0, SGI0, SGI0, SGI1631, SGI1631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG382[] = {SGI1632, SGI1633, SGI1634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1635, SGI0, SGI0, SGI0, SGI1636, SGI1636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG383[] = {SGI1637, SGI1638, SGI1639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1640, SGI0, SGI0, SGI0, SGI1641, SGI1641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG384[] = {SGI1642, SGI0, SGI1643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1644, SGI0, SGI0, SGI0, SGI1645, SGI1646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG385[] = {SGI1647, SGI0, SGI1648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1649, SGI0, SGI0, SGI0, SGI1650, SGI1651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG386[] = {SGI1652, SGI1653, SGI1654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1655, SGI0, SGI0, SGI0, SGI1656, SGI1656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG387[] = {SGI1657, SGI1658, SGI1659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1660, SGI0, SGI0, SGI0, SGI1661, SGI1661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG388[] = {SGI0, SGI1662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG389[] = {SGI1663, SGI1664, SGI1665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1666, SGI0, SGI0, SGI0, SGI1667, SGI1667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG390[] = {SGI1668, SGI1669, SGI1670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1671, SGI0, SGI0, SGI0, SGI1672, SGI1672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG391[] = {SGI1673, SGI1674, SGI1675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1676, SGI0, SGI0, SGI0, SGI1677, SGI1677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG392[] = {SGI1678, SGI1679, SGI1680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1681, SGI0, SGI0, SGI0, SGI1682, SGI1682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG393[] = {SGI1683, SGI1684, SGI1685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1686, SGI0, SGI0, SGI0, SGI1687, SGI1687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG394[] = {SGI1688, SGI1689, SGI1690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1691, SGI0, SGI0, SGI0, SGI1692, SGI1692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG395[] = {SGI1693, SGI1694, SGI1695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1696, SGI0, SGI0, SGI0, SGI1697, SGI1697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG396[] = {SGI1698, SGI1699, SGI1700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1701, SGI0, SGI0, SGI0, SGI1702, SGI1702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG397[] = {SGI1703, SGI1704, SGI1705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1706, SGI0, SGI0, SGI0, SGI1707, SGI1707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG398[] = {SGI1708, SGI1709, SGI1710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1711, SGI0, SGI0, SGI0, SGI1712, SGI1712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG399[] = {SGI1713, SGI1714, SGI1715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1716, SGI0, SGI0, SGI0, SGI1717, SGI1717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG400[] = {SGI1718, SGI1719, SGI1720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1721, SGI0, SGI0, SGI0, SGI1722, SGI1722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG401[] = {SGI1723, SGI1724, SGI1725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1726, SGI0, SGI0, SGI0, SGI1727, SGI1727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG402[] = {SGI1728, SGI1729, SGI1730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1731, SGI0, SGI0, SGI0, SGI1732, SGI1732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG403[] = {SGI1733, SGI1734, SGI1735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1736, SGI0, SGI0, SGI0, SGI1737, SGI1737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG404[] = {SGI1738, SGI1739, SGI1740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1741, SGI0, SGI0, SGI0, SGI1742, SGI1742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG405[] = {SGI1743, SGI1744, SGI1745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1746, SGI0, SGI0, SGI0, SGI1747, SGI1747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG406[] = {SGI1748, SGI1749, SGI1750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1751, SGI0, SGI0, SGI0, SGI1752, SGI1752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG407[] = {SGI1753, SGI1754, SGI1755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1756, SGI0, SGI0, SGI0, SGI1757, SGI1757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG408[] = {SGI1758, SGI1759, SGI1760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1761, SGI0, SGI0, SGI0, SGI1762, SGI1762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG409[] = {SGI1763, SGI1764, SGI1765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1766, SGI0, SGI0, SGI0, SGI1767, SGI1767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG410[] = {SGI1768, SGI1769, SGI1770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1771, SGI0, SGI0, SGI0, SGI1772, SGI1772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG411[] = {SGI1773, SGI1774, SGI1775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1776, SGI0, SGI0, SGI0, SGI1777, SGI1777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG412[] = {SGI1778, SGI1779, SGI1780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1781, SGI0, SGI0, SGI0, SGI1782, SGI1782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG413[] = {SGI1783, SGI1784, SGI1785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1786, SGI0, SGI0, SGI0, SGI1787, SGI1787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG414[] = {SGI1788, SGI1789, SGI1790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1791, SGI0, SGI0, SGI0, SGI1792, SGI1792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG415[] = {SGI1793, SGI1794, SGI1795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1796, SGI0, SGI0, SGI0, SGI1797, SGI1797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG416[] = {SGI1798, SGI1799, SGI1800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1801, SGI0, SGI0, SGI0, SGI1802, SGI1802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG417[] = {SGI1803, SGI1804, SGI1805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1806, SGI0, SGI0, SGI0, SGI1807, SGI1807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG418[] = {SGI1808, SGI1809, SGI1810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1811, SGI0, SGI0, SGI0, SGI1812, SGI1812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG419[] = {SGI1813, SGI1814, SGI1815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1816, SGI0, SGI0, SGI0, SGI1817, SGI1817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG420[] = {SGI1818, SGI1819, SGI1820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1821, SGI0, SGI0, SGI0, SGI1822, SGI1822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG421[] = {SGI1823, SGI1824, SGI1825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1826, SGI0, SGI0, SGI0, SGI1827, SGI1827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG422[] = {SGI1828, SGI1829, SGI1830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1831, SGI0, SGI0, SGI0, SGI1832, SGI1832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG423[] = {SGI1833, SGI1834, SGI1835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1836, SGI0, SGI0, SGI0, SGI1837, SGI1837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG424[] = {SGI1838, SGI1839, SGI1840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1841, SGI0, SGI0, SGI0, SGI1842, SGI1842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG425[] = {SGI1843, SGI1844, SGI1845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1846, SGI0, SGI0, SGI0, SGI1847, SGI1847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG426[] = {SGI1848, SGI1849, SGI1850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1851, SGI0, SGI0, SGI0, SGI1852, SGI1852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG427[] = {SGI1853, SGI1854, SGI1855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1856, SGI1857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG428[] = {SGI1858, SGI1859, SGI1860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1861, SGI0, SGI0, SGI0, SGI1862, SGI1862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG429[] = {SGI1863, SGI1864, SGI1865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1866, SGI1867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG430[] = {SGI1868, SGI1869, SGI1870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1871, SGI0, SGI0, SGI0, SGI1872, SGI1872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG431[] = {SGI1873, SGI1874, SGI1875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1876, SGI0, SGI0, SGI0, SGI1877, SGI1877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG432[] = {SGI1878, SGI1879, SGI1880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1881, SGI0, SGI0, SGI0, SGI1882, SGI1882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG433[] = {SGI1883, SGI1884, SGI1885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1886, SGI0, SGI0, SGI0, SGI1887, SGI1887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG434[] = {SGI1888, SGI1889, SGI1890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1891, SGI0, SGI0, SGI0, SGI1892, SGI1892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG435[] = {SGI1893, SGI1894, SGI1895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1896, SGI0, SGI0, SGI0, SGI1897, SGI1897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG436[] = {SGI1898, SGI1899, SGI1900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1901, SGI0, SGI0, SGI0, SGI1902, SGI1902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG437[] = {SGI1903, SGI1904, SGI1905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1906, SGI0, SGI0, SGI0, SGI1907, SGI1907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG438[] = {SGI1908, SGI1909, SGI1910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1911, SGI0, SGI0, SGI0, SGI1912, SGI1912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG439[] = {SGI1913, SGI1914, SGI1915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1916, SGI0, SGI0, SGI0, SGI1917, SGI1917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG440[] = {SGI1918, SGI1919, SGI1920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1921, SGI0, SGI0, SGI0, SGI1922, SGI1922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG441[] = {SGI1923, SGI1924, SGI1925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1926, SGI0, SGI0, SGI0, SGI1927, SGI1927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG442[] = {SGI1928, SGI1929, SGI1930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1931, SGI1932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG443[] = {SGI1933, SGI1934, SGI1935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1936, SGI0, SGI0, SGI0, SGI1937, SGI1937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG444[] = {SGI1938, SGI1939, SGI1940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1941, SGI1942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG445[] = {SGI1943, SGI1944, SGI1945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1946, SGI0, SGI0, SGI0, SGI1947, SGI1947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG446[] = {SGI1948, SGI1949, SGI1950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1951, SGI0, SGI0, SGI0, SGI1952, SGI1952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG447[] = {SGI1953, SGI1954, SGI1955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1956, SGI0, SGI0, SGI0, SGI1957, SGI1957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG448[] = {SGI1958, SGI1959, SGI1960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1961, SGI0, SGI0, SGI0, SGI1962, SGI1962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG449[] = {SGI1963, SGI1964, SGI1965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1966, SGI0, SGI0, SGI0, SGI1967, SGI1967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG450[] = {SGI1968, SGI1969, SGI1970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1971, SGI0, SGI0, SGI0, SGI1972, SGI1972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG451[] = {SGI1973, SGI0, SGI1974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1975, SGI0, SGI0, SGI0, SGI1976, SGI1977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG452[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1979, NULL};
const Segment * const SG453[] = {SGI1978, SGI1980, SGI1981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1982, SGI0, SGI0, SGI0, SGI1983, SGI1983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG454[] = {SGI1984, SGI1985, SGI1986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1987, SGI0, SGI0, SGI0, SGI1988, SGI1988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG455[] = {SGI1989, SGI1990, SGI1991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1992, SGI1992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG456[] = {SGI1993, SGI1994, SGI1995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI1996, SGI0, SGI0, SGI0, SGI1997, SGI1997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG457[] = {SGI1998, SGI1999, SGI2000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2001, SGI0, SGI0, SGI0, SGI2002, SGI2002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG458[] = {SGI0, SGI2003, SGI2004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG459[] = {SGI0, SGI2005, SGI2006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2007, SGI0, SGI0, SGI0, SGI0, SGI2008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG460[] = {SGI2009, SGI2010, SGI2011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2012, SGI0, SGI0, SGI0, SGI2013, SGI2013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG461[] = {SGI2014, SGI2016, SGI2018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2019, SGI0, SGI0, SGI0, SGI2021, SGI2021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG462[] = {SGI0, SGI2015, SGI2017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2020, SGI0, SGI0, SGI0, SGI0, SGI2022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG463[] = {SGI2023, SGI2024, SGI2025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2026, SGI0, SGI0, SGI0, SGI2027, SGI2027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG464[] = {SGI2028, SGI2029, SGI2030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2031, SGI0, SGI0, SGI0, SGI2032, SGI2032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG465[] = {SGI2033, SGI2034, SGI2035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2036, SGI0, SGI0, SGI0, SGI2038, SGI2038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG466[] = {SGI2037, SGI2039, SGI2040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2041, SGI0, SGI0, SGI0, SGI2042, SGI2042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG467[] = {SGI2043, SGI2044, SGI2045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2046, SGI0, SGI0, SGI0, SGI2047, SGI2047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG468[] = {SGI2048, SGI2049, SGI2050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2051, SGI0, SGI0, SGI0, SGI2052, SGI2052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG469[] = {SGI2053, SGI2054, SGI2055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2056, SGI0, SGI0, SGI0, SGI2057, SGI2057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG470[] = {SGI2058, SGI2059, SGI2060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2061, SGI0, SGI0, SGI0, SGI2062, SGI2062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG471[] = {SGI2063, SGI2064, SGI2065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2069, SGI0, SGI0, SGI0, SGI2070, SGI2070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG472[] = {SGI2066, SGI2067, SGI2068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2071, SGI0, SGI0, SGI0, SGI2072, SGI2072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG473[] = {SGI2073, SGI2074, SGI2075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2076, SGI0, SGI0, SGI0, SGI2078, SGI2078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG474[] = {SGI2077, SGI2079, SGI2080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2081, SGI0, SGI0, SGI0, SGI2082, SGI2082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG475[] = {SGI2083, SGI2084, SGI2085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2086, SGI0, SGI0, SGI0, SGI2087, SGI2087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG476[] = {SGI0, SGI2088, SGI2089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2090, SGI0, SGI0, SGI0, SGI2091, SGI2091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG477[] = {SGI2092, SGI2093, SGI2094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2095, SGI0, SGI0, SGI0, SGI2096, SGI2096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG478[] = {SGI2097, SGI2098, SGI2099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2101, SGI0, SGI0, SGI0, SGI2104, SGI2104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG479[] = {SGI2100, SGI2102, SGI2103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2105, SGI0, SGI0, SGI0, SGI2106, SGI2106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG480[] = {SGI2107, SGI2108, SGI2109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2110, SGI0, SGI0, SGI0, SGI2111, SGI2111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG481[] = {SGI2112, SGI2113, SGI2114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2115, SGI0, SGI0, SGI0, SGI2116, SGI2116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG482[] = {SGI2117, SGI2118, SGI2119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2120, SGI0, SGI0, SGI0, SGI2121, SGI2121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG483[] = {SGI2122, SGI0, SGI2123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2126, SGI0, SGI0, SGI0, SGI2128, SGI2128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG484[] = {SGI2124, SGI2125, SGI2127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2129, SGI0, SGI0, SGI0, SGI2130, SGI2130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG485[] = {SGI2131, SGI2132, SGI2133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2134, SGI0, SGI0, SGI0, SGI2135, SGI2135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG486[] = {SGI2136, SGI2137, SGI2138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2139, SGI0, SGI0, SGI0, SGI2140, SGI2140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG487[] = {SGI2141, SGI2142, SGI2143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2147, SGI2147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG488[] = {SGI2144, SGI2145, SGI2146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2148, SGI0, SGI0, SGI0, SGI2149, SGI2149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG489[] = {SGI2150, SGI2151, SGI2152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2153, SGI0, SGI0, SGI0, SGI2155, SGI2155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG490[] = {SGI2154, SGI2156, SGI2157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2158, SGI0, SGI0, SGI0, SGI2159, SGI2159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG491[] = {SGI2160, SGI2161, SGI2163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2166, SGI0, SGI0, SGI0, SGI2168, SGI2168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG492[] = {SGI2162, SGI2164, SGI2165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2167, SGI0, SGI0, SGI0, SGI2169, SGI2169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG493[] = {SGI2170, SGI2171, SGI2172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2173, SGI0, SGI0, SGI0, SGI2174, SGI2174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG494[] = {SGI2175, SGI2176, SGI2177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2178, SGI0, SGI0, SGI0, SGI2179, SGI2179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG495[] = {SGI2180, SGI2181, SGI2182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2183, SGI0, SGI0, SGI0, SGI2184, SGI2184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG496[] = {SGI2185, SGI2186, SGI2188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2191, SGI0, SGI0, SGI0, SGI2193, SGI2193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG497[] = {SGI2187, SGI2189, SGI2190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2192, SGI0, SGI0, SGI0, SGI2194, SGI2194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG498[] = {SGI2195, SGI2196, SGI2197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2198, SGI0, SGI0, SGI0, SGI2199, SGI2199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG499[] = {SGI2200, SGI2201, SGI2202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2203, SGI0, SGI0, SGI0, SGI2204, SGI2204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG500[] = {SGI2205, SGI2206, SGI2207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2208, SGI0, SGI0, SGI0, SGI2209, SGI2209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG501[] = {SGI2210, SGI2211, SGI2212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2213, SGI0, SGI0, SGI0, SGI2214, SGI2214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG502[] = {SGI2215, SGI2216, SGI2217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2221, SGI0, SGI0, SGI0, SGI2222, SGI2222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG503[] = {SGI2218, SGI2219, SGI2220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2223, SGI0, SGI0, SGI0, SGI2224, SGI2224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG504[] = {SGI2225, SGI2226, SGI2227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2228, SGI0, SGI0, SGI0, SGI2229, SGI2229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG505[] = {SGI2230, SGI2231, SGI2232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2233, SGI0, SGI0, SGI0, SGI2234, SGI2234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG506[] = {SGI2235, SGI2236, SGI2237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2238, SGI0, SGI0, SGI0, SGI2239, SGI2239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG507[] = {SGI2240, SGI2241, SGI2242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2243, SGI0, SGI0, SGI0, SGI2244, SGI2244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG508[] = {SGI2245, SGI2246, SGI2247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2248, SGI0, SGI0, SGI0, SGI2249, SGI2249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG509[] = {SGI2250, SGI2251, SGI2252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2253, SGI0, SGI0, SGI0, SGI2254, SGI2254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG510[] = {SGI2255, SGI2256, SGI2257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2258, SGI0, SGI0, SGI0, SGI2259, SGI2259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG511[] = {SGI2260, SGI2261, SGI2262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2263, SGI0, SGI0, SGI0, SGI2264, SGI2264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG512[] = {SGI2265, SGI2266, SGI2267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2268, SGI0, SGI0, SGI0, SGI2269, SGI2269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG513[] = {SGI2270, SGI2271, SGI2272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2273, SGI0, SGI0, SGI0, SGI2274, SGI2274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG514[] = {SGI2275, SGI2276, SGI2277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2278, SGI0, SGI0, SGI0, SGI2279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG515[] = {SGI2280, SGI2281, SGI2282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2283, SGI0, SGI0, SGI0, SGI2284, SGI2284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG516[] = {SGI2285, SGI0, SGI2286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG517[] = {SGI2287, SGI2288, SGI2289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2290, SGI0, SGI0, SGI0, SGI2291, SGI2291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG518[] = {SGI2292, SGI2293, SGI2294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2295, SGI0, SGI0, SGI0, SGI2296, SGI2296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG519[] = {SGI2297, SGI2298, SGI2299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2300, SGI0, SGI0, SGI0, SGI2303, SGI2303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG520[] = {SGI2301, SGI2302, SGI2304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2305, SGI0, SGI0, SGI0, SGI2306, SGI2306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG521[] = {SGI2307, SGI2308, SGI2309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2311, SGI0, SGI0, SGI0, SGI2314, SGI2314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG522[] = {SGI2310, SGI2312, SGI2313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2315, SGI0, SGI0, SGI0, SGI2316, SGI2316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG523[] = {SGI2317, SGI2318, SGI2319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2320, SGI0, SGI0, SGI0, SGI2321, SGI2321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG524[] = {SGI2322, SGI2323, SGI2324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2325, SGI0, SGI0, SGI0, SGI2326, SGI2326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG525[] = {SGI2327, SGI2328, SGI2329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2330, SGI0, SGI0, SGI0, SGI2331, SGI2331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG526[] = {SGI0, SGI2332, SGI2333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG527[] = {SGI2334, SGI2335, SGI2336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2337, SGI0, SGI0, SGI0, SGI2338, SGI2338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG528[] = {SGI2339, SGI2340, SGI2341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2342, SGI0, SGI0, SGI0, SGI2343, SGI2343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG529[] = {SGI2344, SGI2345, SGI2346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2347, SGI0, SGI0, SGI0, SGI2348, SGI2348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG530[] = {SGI2349, SGI2350, SGI2351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2352, SGI0, SGI0, SGI0, SGI2353, SGI2353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG531[] = {SGI2354, SGI2355, SGI2356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2357, SGI0, SGI0, SGI0, SGI2358, SGI2358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG532[] = {SGI2359, SGI2360, SGI2361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2362, SGI0, SGI0, SGI0, SGI2363, SGI2363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG533[] = {SGI2364, SGI2365, SGI2366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2367, SGI0, SGI0, SGI0, SGI2368, SGI2368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG534[] = {SGI2369, SGI2370, SGI2371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2372, SGI0, SGI0, SGI0, SGI2373, SGI2373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG535[] = {SGI2374, SGI2375, SGI2376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2377, SGI0, SGI0, SGI0, SGI2378, SGI2378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG536[] = {SGI2379, SGI2380, SGI2381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2382, SGI0, SGI0, SGI0, SGI2383, SGI2383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG537[] = {SGI2384, SGI2385, SGI2386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2387, SGI0, SGI0, SGI0, SGI2388, SGI2388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG538[] = {SGI2389, SGI2390, SGI2391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2392, SGI0, SGI0, SGI0, SGI2393, SGI2393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG539[] = {SGI2394, SGI2395, SGI2396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2397, SGI0, SGI0, SGI0, SGI2398, SGI2398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG540[] = {SGI0, SGI2399, SGI2400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG541[] = {SGI2401, SGI0, SGI2402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2403, SGI0, SGI0, SGI0, SGI2404, SGI2404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG542[] = {SGI2405, SGI2406, SGI2407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2408, SGI0, SGI0, SGI0, SGI2409, SGI2409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG543[] = {SGI2410, SGI2411, SGI2412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2413, SGI0, SGI0, SGI0, SGI2414, SGI2414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG544[] = {SGI2415, SGI2416, SGI2417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2418, SGI0, SGI0, SGI0, SGI2419, SGI2419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG545[] = {SGI2420, SGI2421, SGI2422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2423, SGI0, SGI0, SGI0, SGI2424, SGI2424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG546[] = {SGI2425, SGI2426, SGI2427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2428, SGI0, SGI0, SGI0, SGI2429, SGI2429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG547[] = {SGI2430, SGI2431, SGI2432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2433, SGI0, SGI0, SGI0, SGI2434, SGI2434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG548[] = {SGI2435, SGI2436, SGI2437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2438, SGI0, SGI0, SGI0, SGI2439, SGI2439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG549[] = {SGI2440, SGI2441, SGI2442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2443, SGI0, SGI0, SGI0, SGI2444, SGI2444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG550[] = {SGI2445, SGI2446, SGI2447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2448, SGI0, SGI0, SGI0, SGI2449, SGI2449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG551[] = {SGI2450, SGI2451, SGI2452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2453, SGI0, SGI0, SGI0, SGI2454, SGI2454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG552[] = {SGI2455, SGI2456, SGI2457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2458, SGI0, SGI0, SGI0, SGI2459, SGI2459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG553[] = {SGI2460, SGI2461, SGI2462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2463, SGI0, SGI0, SGI0, SGI2464, SGI2464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG554[] = {SGI2465, SGI0, SGI2466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG555[] = {SGI2467, SGI2468, SGI2469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2470, SGI0, SGI0, SGI0, SGI2471, SGI2471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG556[] = {SGI2472, SGI2473, SGI2474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2475, SGI0, SGI0, SGI0, SGI2476, SGI2476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG557[] = {SGI2477, SGI0, SGI2478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG558[] = {SGI2479, SGI2480, SGI2481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2482, SGI0, SGI0, SGI0, SGI2483, SGI2483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG559[] = {SGI2484, SGI2485, SGI2486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2487, SGI0, SGI0, SGI0, SGI2488, SGI2488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG560[] = {SGI2489, SGI2490, SGI2491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2492, SGI0, SGI0, SGI0, SGI2493, SGI2493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG561[] = {SGI2494, SGI2495, SGI2496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2497, SGI0, SGI0, SGI0, SGI2498, SGI2498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG562[] = {SGI2499, SGI2500, SGI2501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG563[] = {SGI2503, SGI0, SGI2504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG564[] = {SGI0, SGI2505, SGI2506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG565[] = {SGI2507, SGI2508, SGI2509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2510, SGI0, SGI0, SGI0, SGI2511, SGI2511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG566[] = {SGI0, SGI0, SGI2512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG567[] = {SGI2514, SGI2515, SGI2516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2517, SGI0, SGI0, SGI0, SGI2518, SGI2518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG568[] = {SGI0, SGI2519, SGI2520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2521, SGI0, SGI0, SGI0, SGI2522, SGI2522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG569[] = {SGI2523, SGI2524, SGI2525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2526, SGI0, SGI0, SGI0, SGI2527, SGI2527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG570[] = {SGI2528, SGI2529, SGI2530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2531, SGI2532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG571[] = {SGI2533, SGI2534, SGI2535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2536, SGI0, SGI0, SGI0, SGI2537, SGI2537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG572[] = {SGI2538, SGI2539, SGI2540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2541, SGI0, SGI0, SGI0, SGI2542, SGI2542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG573[] = {SGI2543, SGI2544, SGI2545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2546, SGI0, SGI0, SGI0, SGI2547, SGI2547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG574[] = {SGI2548, SGI2549, SGI2550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2551, SGI0, SGI0, SGI0, SGI2552, SGI2552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG575[] = {SGI0, SGI2553, SGI2554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG576[] = {SGI2555, SGI2556, SGI2557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2558, SGI0, SGI0, SGI0, SGI2559, SGI2559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG577[] = {SGI2560, SGI2561, SGI2562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2563, SGI0, SGI0, SGI0, SGI2564, SGI2564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG578[] = {SGI2565, SGI2566, SGI2567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2568, SGI0, SGI0, SGI0, SGI2569, SGI2569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG579[] = {SGI2570, SGI2571, SGI2572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2573, SGI0, SGI0, SGI0, SGI2574, SGI2574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG580[] = {SGI2575, SGI2576, SGI2577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2578, SGI0, SGI0, SGI0, SGI2579, SGI2579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG581[] = {SGI2580, SGI2581, SGI2582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2583, SGI0, SGI0, SGI0, SGI2584, SGI2584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG582[] = {SGI2585, SGI2586, SGI2587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2588, SGI0, SGI0, SGI0, SGI2589, SGI2589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG583[] = {SGI2590, SGI2591, SGI2592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2593, SGI0, SGI0, SGI0, SGI2594, SGI2594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG584[] = {SGI2595, SGI2596, SGI2597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2598, SGI0, SGI0, SGI0, SGI2599, SGI2599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG585[] = {SGI2600, SGI2601, SGI2602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2603, SGI0, SGI0, SGI0, SGI2604, SGI2604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG586[] = {SGI2605, SGI2606, SGI2607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2608, SGI0, SGI0, SGI0, SGI2609, SGI2609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG587[] = {SGI2610, SGI2611, SGI2612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2613, SGI0, SGI0, SGI0, SGI2614, SGI2614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG588[] = {SGI2615, SGI2616, SGI2617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2618, SGI0, SGI0, SGI0, SGI2619, SGI2619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG589[] = {SGI2620, SGI2621, SGI2622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2623, SGI0, SGI0, SGI0, SGI2624, SGI2624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG590[] = {SGI0, SGI0, SGI2625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2626, SGI0, SGI0, SGI0, SGI2627, SGI2627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG591[] = {SGI2628, SGI2629, SGI2630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2631, SGI0, SGI0, SGI0, SGI2632, SGI2632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG592[] = {SGI2633, SGI2634, SGI2635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2636, SGI0, SGI0, SGI0, SGI2637, SGI2637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG593[] = {SGI2638, SGI2639, SGI2640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2641, SGI0, SGI0, SGI0, SGI2642, SGI2642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG594[] = {SGI2643, SGI2644, SGI2645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2646, SGI0, SGI0, SGI0, SGI2647, SGI2647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG595[] = {SGI2648, SGI2649, SGI2650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2651, SGI0, SGI0, SGI0, SGI2652, SGI2652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG596[] = {SGI2653, SGI2654, SGI2655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2656, SGI0, SGI0, SGI0, SGI2657, SGI2657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG597[] = {SGI0, SGI0, SGI2658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG598[] = {SGI2659, SGI2660, SGI2661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2662, SGI0, SGI0, SGI0, SGI2663, SGI2663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG599[] = {SGI2664, SGI2665, SGI2666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2667, SGI0, SGI0, SGI0, SGI2668, SGI2668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG600[] = {SGI2669, SGI2670, SGI2671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2672, SGI0, SGI0, SGI0, SGI2673, SGI2673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG601[] = {SGI2674, SGI2675, SGI2676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2677, SGI0, SGI0, SGI0, SGI2678, SGI2678, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG602[] = {SGI2679, SGI2680, SGI2681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2682, SGI0, SGI0, SGI0, SGI2683, SGI2683, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG603[] = {SGI2684, SGI2685, SGI2686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2687, SGI0, SGI0, SGI0, SGI2688, SGI2688, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG604[] = {SGI2689, SGI2690, SGI2691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2692, SGI0, SGI0, SGI0, SGI2693, SGI2693, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG605[] = {SGI2694, SGI2695, SGI2696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2697, SGI0, SGI0, SGI0, SGI2698, SGI2698, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG606[] = {SGI2699, SGI2700, SGI2701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2702, SGI0, SGI0, SGI0, SGI2703, SGI2703, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG607[] = {SGI2704, SGI2705, SGI2706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2707, SGI0, SGI0, SGI0, SGI2708, SGI2708, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG608[] = {SGI2709, SGI2710, SGI2711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2712, SGI0, SGI0, SGI0, SGI2713, SGI2713, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG609[] = {SGI2714, SGI0, SGI2715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG610[] = {SGI2716, SGI2717, SGI2718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2719, SGI2720, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG611[] = {SGI2721, SGI2722, SGI2723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2724, SGI2725, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG612[] = {SGI2726, SGI2727, SGI2728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2729, SGI0, SGI0, SGI0, SGI2730, SGI2730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG613[] = {SGI2731, SGI0, SGI2732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2733, SGI0, SGI0, SGI0, SGI2734, SGI2734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG614[] = {SGI2735, SGI2736, SGI2737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2738, SGI0, SGI0, SGI0, SGI2739, SGI2739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG615[] = {SGI2740, SGI2741, SGI2742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2743, SGI0, SGI0, SGI0, SGI2744, SGI2744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG616[] = {SGI2745, SGI2746, SGI2747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2748, SGI0, SGI0, SGI0, SGI2749, SGI2749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG617[] = {SGI0, SGI2750, SGI2751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2752, SGI0, SGI0, SGI0, SGI2753, SGI2753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG618[] = {SGI2754, SGI0, SGI2755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG619[] = {SGI2756, SGI2757, SGI2758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2759, SGI2760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG620[] = {SGI2761, SGI2762, SGI2763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2764, SGI0, SGI0, SGI0, SGI2765, SGI2765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG621[] = {SGI2766, SGI2767, SGI2768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2769, SGI0, SGI0, SGI0, SGI2770, SGI2770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG622[] = {SGI2771, SGI2772, SGI2773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2774, SGI0, SGI0, SGI0, SGI2775, SGI2775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG623[] = {SGI2776, SGI0, SGI2777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG624[] = {SGI2778, SGI2779, SGI2780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2781, SGI0, SGI0, SGI0, SGI2782, SGI2782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG625[] = {SGI0, SGI2783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2784, SGI0, SGI0, SGI0, SGI2785, SGI2785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG626[] = {SGI2786, SGI2787, SGI2788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2789, SGI0, SGI0, SGI0, SGI2790, SGI2790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG627[] = {SGI2791, SGI2792, SGI2793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2794, SGI0, SGI0, SGI0, SGI2795, SGI2795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG628[] = {SGI2796, SGI2797, SGI2798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2799, SGI0, SGI0, SGI0, SGI2800, SGI2800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG629[] = {SGI2801, SGI2802, SGI2803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2804, SGI0, SGI0, SGI0, SGI2805, SGI2805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG630[] = {SGI2806, SGI0, SGI2807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG631[] = {SGI0, SGI0, SGI2808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2809, SGI0, SGI0, SGI0, SGI2810, SGI2810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG632[] = {SGI2811, SGI2812, SGI2813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2814, SGI0, SGI0, SGI0, SGI2815, SGI2815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG633[] = {SGI2816, SGI2817, SGI2818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2819, SGI0, SGI0, SGI0, SGI2820, SGI2820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG634[] = {SGI2821, SGI2822, SGI2823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2824, SGI0, SGI0, SGI0, SGI2825, SGI2825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG635[] = {SGI2826, SGI2827, SGI2828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2829, SGI0, SGI0, SGI0, SGI2830, SGI2830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG636[] = {SGI2831, SGI2832, SGI2833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2834, SGI0, SGI0, SGI0, SGI2835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG637[] = {SGI0, SGI0, SGI2836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2837, SGI0, SGI0, SGI0, SGI2838, SGI2838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG638[] = {SGI2839, SGI2840, SGI2841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2842, SGI0, SGI0, SGI0, SGI2843, SGI2843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG639[] = {SGI2844, SGI0, SGI2845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2846, SGI0, SGI0, SGI0, SGI2847, SGI2847, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG640[] = {SGI2848, SGI2849, SGI2850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2851, SGI0, SGI0, SGI0, SGI2852, SGI2852, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG641[] = {SGI2853, SGI2854, SGI2855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2856, SGI0, SGI0, SGI0, SGI2857, SGI2857, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG642[] = {SGI0, SGI2858, SGI2859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2860, SGI0, SGI0, SGI0, SGI2861, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG643[] = {SGI2862, SGI2863, SGI2864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2865, SGI0, SGI0, SGI0, SGI2866, SGI2866, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG644[] = {SGI2867, SGI2868, SGI2869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2870, SGI0, SGI0, SGI0, SGI2871, SGI2871, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG645[] = {SGI2872, SGI2873, SGI2874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2875, SGI0, SGI0, SGI0, SGI2876, SGI2876, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG646[] = {SGI0, SGI2877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2878, SGI0, SGI0, SGI0, SGI2879, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG647[] = {SGI2880, SGI2881, SGI2882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2883, SGI0, SGI0, SGI0, SGI2884, SGI2884, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG648[] = {SGI2885, SGI2886, SGI2887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2888, SGI0, SGI0, SGI0, SGI2889, SGI2889, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG649[] = {SGI2890, SGI0, SGI2891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG650[] = {SGI2892, SGI2893, SGI2894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2895, SGI2896, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG651[] = {SGI2897, SGI2898, SGI2899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2900, SGI0, SGI0, SGI0, SGI2901, SGI2901, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG652[] = {SGI2902, SGI2903, SGI2904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2905, SGI2906, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG653[] = {SGI2907, SGI2908, SGI2909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2910, SGI0, SGI0, SGI0, SGI2911, SGI2911, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG654[] = {SGI2912, SGI2913, SGI2914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2915, SGI2915, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG655[] = {SGI2916, SGI0, SGI2917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2918, SGI0, SGI0, SGI0, SGI2919, SGI2919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG656[] = {SGI2920, SGI2921, SGI2922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2923, SGI0, SGI0, SGI0, SGI2924, SGI2924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG657[] = {SGI0, SGI0, SGI2925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2926, SGI0, SGI0, SGI0, SGI2927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG658[] = {SGI2928, SGI2929, SGI2930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2931, SGI0, SGI0, SGI0, SGI2932, SGI2932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG659[] = {SGI0, SGI2933, SGI2934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2935, SGI0, SGI0, SGI0, SGI2936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG660[] = {SGI0, SGI0, SGI2937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2938, SGI0, SGI0, SGI0, SGI2939, SGI2939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG661[] = {SGI2940, SGI2941, SGI2942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2943, SGI0, SGI0, SGI0, SGI2944, SGI2944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG662[] = {SGI0, SGI2945, SGI2946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2947, SGI0, SGI0, SGI0, SGI2948, SGI2948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG663[] = {SGI0, SGI2949, SGI2950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2951, SGI0, SGI0, SGI0, SGI2952, SGI2952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG664[] = {SGI2953, SGI2954, SGI2955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2956, SGI0, SGI0, SGI0, SGI2957, SGI2957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG665[] = {SGI0, SGI2958, SGI2959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG666[] = {SGI2961, SGI2962, SGI2963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2964, SGI0, SGI0, SGI0, SGI2965, SGI2965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG667[] = {SGI2966, SGI2967, SGI2968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2969, SGI0, SGI0, SGI0, SGI2970, SGI2970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG668[] = {SGI2971, SGI2972, SGI2973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2974, SGI0, SGI0, SGI0, SGI2975, SGI2975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG669[] = {SGI2976, SGI0, SGI2977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG670[] = {SGI0, SGI0, SGI2978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG671[] = {SGI2980, SGI2981, SGI2982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2983, SGI0, SGI0, SGI0, SGI2984, SGI2984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG672[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2985, SGI0, SGI0, SGI0, SGI2986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG673[] = {SGI2987, SGI0, SGI2988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG674[] = {SGI2989, SGI0, SGI2990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2991, SGI0, SGI0, SGI0, SGI2992, SGI2992, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG675[] = {SGI2993, SGI2994, SGI2995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI2996, SGI2998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG676[] = {SGI2997, SGI2999, SGI3000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3001, SGI0, SGI0, SGI0, SGI3002, SGI3002, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG677[] = {SGI3003, SGI3004, SGI3005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3006, SGI0, SGI0, SGI0, SGI3007, SGI3007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG678[] = {SGI3008, SGI3009, SGI3010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3011, SGI0, SGI0, SGI0, SGI3012, SGI3012, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG679[] = {SGI3013, SGI0, SGI3014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG680[] = {SGI3015, SGI3016, SGI3017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3018, SGI0, SGI0, SGI0, SGI3019, SGI3019, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG681[] = {SGI3020, SGI3021, SGI3022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3023, SGI0, SGI0, SGI0, SGI3024, SGI3024, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG682[] = {SGI3025, SGI3026, SGI3027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3028, SGI0, SGI0, SGI0, SGI3029, SGI3030, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG683[] = {SGI3031, SGI0, SGI3032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG684[] = {SGI0, SGI3033, SGI3034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3035, SGI0, SGI0, SGI0, SGI3036, SGI3036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG685[] = {SGI3037, SGI0, SGI3038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG686[] = {SGI3039, SGI3040, SGI3041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3042, SGI3043, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG687[] = {SGI3045, SGI3047, SGI3049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3050, SGI0, SGI0, SGI0, SGI3052, SGI3052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG688[] = {SGI3044, SGI3046, SGI3048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3051, SGI0, SGI0, SGI0, SGI3053, SGI3053, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG689[] = {SGI3054, SGI3055, SGI3056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3057, SGI0, SGI0, SGI0, SGI3058, SGI3059, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG690[] = {SGI3060, SGI3061, SGI3062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3063, SGI0, SGI0, SGI0, SGI3064, SGI3064, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG691[] = {SGI3065, SGI0, SGI3066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3067, SGI0, SGI0, SGI0, SGI3068, SGI3068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG692[] = {SGI3069, SGI3070, SGI3071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3072, SGI0, SGI0, SGI0, SGI3073, SGI3073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG693[] = {SGI3074, SGI3075, SGI3076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3077, SGI0, SGI0, SGI0, SGI3078, SGI3078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG694[] = {SGI3079, SGI3080, SGI3081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3082, SGI0, SGI0, SGI0, SGI3083, SGI3083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG695[] = {SGI3084, SGI3085, SGI3086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3087, SGI0, SGI0, SGI0, SGI3088, SGI3088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG696[] = {SGI3089, SGI3090, SGI3091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3092, SGI0, SGI0, SGI0, SGI3093, SGI3093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG697[] = {SGI3094, SGI3095, SGI3096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3097, SGI0, SGI0, SGI0, SGI3098, SGI3098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG698[] = {SGI3099, SGI3100, SGI3101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3102, SGI0, SGI0, SGI0, SGI3103, SGI3103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG699[] = {SGI3104, SGI3105, SGI3106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3107, SGI0, SGI0, SGI0, SGI3108, SGI3108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG700[] = {SGI3109, SGI3110, SGI3111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3112, SGI0, SGI0, SGI0, SGI3113, SGI3113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG701[] = {SGI3114, SGI3115, SGI3116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3117, SGI0, SGI0, SGI0, SGI3118, SGI3119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG702[] = {SGI3120, SGI3121, SGI3122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3123, SGI0, SGI0, SGI0, SGI3124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG703[] = {SGI3125, SGI3126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3127, SGI0, SGI0, SGI0, SGI3128, SGI3128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG704[] = {SGI3129, SGI0, SGI3130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3131, SGI0, SGI0, SGI0, SGI3132, SGI3132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG705[] = {SGI3133, SGI3134, SGI3135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3136, SGI0, SGI0, SGI0, SGI3137, SGI3137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG706[] = {SGI3138, SGI0, SGI3139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG707[] = {SGI3140, SGI3141, SGI3142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3143, SGI0, SGI0, SGI0, SGI3144, SGI3144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG708[] = {SGI3145, SGI3146, SGI3147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3148, SGI0, SGI0, SGI0, SGI3149, SGI3149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG709[] = {SGI3150, SGI3151, SGI3152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3153, SGI0, SGI0, SGI0, SGI3154, SGI3154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG710[] = {SGI3155, SGI3156, SGI3157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3158, SGI0, SGI0, SGI0, SGI3159, SGI3159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG711[] = {SGI3160, SGI0, SGI3161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG712[] = {SGI3162, SGI0, SGI3163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG713[] = {SGI3164, SGI3165, SGI3166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3167, SGI0, SGI0, SGI0, SGI3168, SGI3168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG714[] = {SGI3169, SGI3170, SGI3171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3172, SGI0, SGI0, SGI0, SGI3173, SGI3173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG715[] = {SGI3174, SGI3175, SGI3176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3177, SGI0, SGI0, SGI0, SGI3178, SGI3178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG716[] = {SGI3179, SGI3180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3181, SGI0, SGI0, SGI0, SGI3182, SGI3182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG717[] = {SGI3183, SGI3184, SGI3185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3186, SGI0, SGI0, SGI0, SGI3187, SGI3187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG718[] = {SGI3188, SGI3189, SGI3190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG719[] = {SGI3191, SGI3192, SGI3193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3194, SGI0, SGI0, SGI0, SGI3195, SGI3195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG720[] = {SGI3196, SGI0, SGI3197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG721[] = {SGI3198, SGI3199, SGI3200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3201, SGI0, SGI0, SGI0, SGI3202, SGI3202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG722[] = {SGI3203, SGI3204, SGI3205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3206, SGI0, SGI0, SGI0, SGI3207, SGI3207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG723[] = {SGI3208, SGI3209, SGI3210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3211, SGI0, SGI0, SGI0, SGI3212, SGI3212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG724[] = {SGI3213, SGI3214, SGI3215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3216, SGI0, SGI0, SGI0, SGI3217, SGI3217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG725[] = {SGI3218, SGI3219, SGI3220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3221, SGI0, SGI0, SGI0, SGI3222, SGI3222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG726[] = {SGI3223, SGI3224, SGI3225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3226, SGI0, SGI0, SGI0, SGI3227, SGI3227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG727[] = {SGI3228, SGI3229, SGI3230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3231, SGI0, SGI0, SGI0, SGI3232, SGI3232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG728[] = {SGI3233, SGI3234, SGI3235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3236, SGI0, SGI0, SGI0, SGI3237, SGI3237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG729[] = {SGI3238, SGI3239, SGI3240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3241, SGI0, SGI0, SGI0, SGI3242, SGI3242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG730[] = {SGI3243, SGI3244, SGI3245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3246, SGI0, SGI0, SGI0, SGI3247, SGI3247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG731[] = {SGI3248, SGI3249, SGI3250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3251, SGI0, SGI0, SGI0, SGI3252, SGI3252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG732[] = {SGI3253, SGI3254, SGI3255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3256, SGI0, SGI0, SGI0, SGI3257, SGI3257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG733[] = {SGI3258, SGI3259, SGI3260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3261, SGI0, SGI0, SGI0, SGI3262, SGI3262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG734[] = {SGI3263, SGI3264, SGI3265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3266, SGI0, SGI0, SGI0, SGI3267, SGI3267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG735[] = {SGI3268, SGI3269, SGI3270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3271, SGI0, SGI0, SGI0, SGI3272, SGI3272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG736[] = {SGI3273, SGI3274, SGI3275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3276, SGI0, SGI0, SGI0, SGI3277, SGI3277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG737[] = {SGI3278, SGI3279, SGI3280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3281, SGI0, SGI0, SGI0, SGI3282, SGI3282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG738[] = {SGI3283, SGI3284, SGI3285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3286, SGI0, SGI0, SGI0, SGI3287, SGI3287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG739[] = {SGI3288, SGI0, SGI3289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3290, SGI0, SGI0, SGI0, SGI3291, SGI3291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG740[] = {SGI3292, SGI3293, SGI3294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3295, SGI0, SGI0, SGI0, SGI3296, SGI3296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG741[] = {SGI3297, SGI3298, SGI3299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3300, SGI0, SGI0, SGI0, SGI3301, SGI3301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG742[] = {SGI3302, SGI3303, SGI3304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3305, SGI0, SGI0, SGI0, SGI3306, SGI3306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG743[] = {SGI3307, SGI0, SGI3308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG744[] = {SGI3309, SGI3310, SGI3311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3312, SGI3313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG745[] = {SGI3314, SGI3315, SGI3316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3317, SGI0, SGI0, SGI0, SGI3318, SGI3318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG746[] = {SGI3319, SGI3320, SGI3321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3322, SGI0, SGI0, SGI0, SGI3323, SGI3323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG747[] = {SGI3324, SGI3325, SGI3326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3327, SGI3327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG748[] = {SGI3328, SGI3329, SGI3330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3331, SGI0, SGI0, SGI0, SGI3332, SGI3332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG749[] = {SGI3333, SGI3334, SGI3335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3336, SGI0, SGI0, SGI0, SGI3337, SGI3337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG750[] = {SGI3338, SGI3339, SGI3340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3341, SGI0, SGI0, SGI0, SGI3342, SGI3342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG751[] = {SGI3343, SGI3344, SGI3345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3346, SGI0, SGI0, SGI0, SGI3347, SGI3347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG752[] = {SGI3353, SGI3354, SGI3355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3356, SGI0, SGI0, SGI3357, SGI3357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG753[] = {SGI3348, SGI3349, SGI3350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3351, SGI0, SGI0, SGI0, SGI3352, SGI3352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG754[] = {SGI0, SGI3358, SGI3359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3360, SGI0, SGI0, SGI0, SGI3361, SGI3361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG755[] = {SGI3362, SGI3363, SGI3364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3365, SGI0, SGI0, SGI0, SGI3366, SGI3366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG756[] = {SGI3367, SGI3368, SGI3369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3370, SGI3371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG757[] = {SGI3372, SGI3373, SGI3374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3375, SGI3375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG758[] = {SGI3376, SGI3377, SGI3378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3379, SGI0, SGI0, SGI0, SGI3380, SGI3380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG759[] = {SGI3381, SGI3382, SGI3383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3384, SGI0, SGI0, SGI0, SGI3385, SGI3385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG760[] = {SGI3386, SGI0, SGI3387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3388, SGI0, SGI0, SGI0, SGI3389, SGI3389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG761[] = {SGI3390, SGI0, SGI3391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3392, SGI0, SGI0, SGI0, SGI3393, SGI3394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG762[] = {SGI3395, SGI0, SGI3396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG763[] = {SGI3397, SGI3398, SGI3399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3400, SGI0, SGI0, SGI0, SGI3401, SGI3402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG764[] = {SGI3403, SGI3404, SGI3405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3406, SGI0, SGI0, SGI0, SGI3407, SGI3407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG765[] = {SGI3408, SGI3409, SGI3410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3411, SGI0, SGI0, SGI0, SGI3412, SGI3412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG766[] = {SGI3413, SGI3414, SGI3415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3416, SGI0, SGI0, SGI0, SGI3417, SGI3417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG767[] = {SGI3418, SGI3419, SGI3420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3421, SGI0, SGI0, SGI0, SGI3422, SGI3422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG768[] = {SGI3423, SGI3424, SGI3425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3426, SGI0, SGI0, SGI0, SGI3427, SGI3427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG769[] = {SGI3428, SGI3429, SGI3430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3431, SGI0, SGI0, SGI0, SGI3432, SGI3433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG770[] = {SGI3434, SGI0, SGI3435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG771[] = {SGI3436, SGI0, SGI3437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3438, SGI0, SGI0, SGI0, SGI3439, SGI3439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG772[] = {SGI3440, SGI3441, SGI3442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3443, SGI0, SGI0, SGI0, SGI3444, SGI3444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG773[] = {SGI3445, SGI3446, SGI3447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3448, SGI0, SGI0, SGI0, SGI3449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG774[] = {SGI3450, SGI3451, SGI3452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3453, SGI0, SGI0, SGI0, SGI3454, SGI3454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG775[] = {SGI3455, SGI0, SGI3456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3457, SGI0, SGI0, SGI0, SGI3458, SGI3458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG776[] = {SGI3459, SGI0, SGI3460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG777[] = {SGI3461, SGI3462, SGI3463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3464, SGI0, SGI0, SGI3465, SGI3465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG778[] = {SGI3466, SGI3467, SGI3468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3469, SGI0, SGI0, SGI3470, SGI3470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG779[] = {SGI3471, SGI3472, SGI3473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3474, SGI0, SGI0, SGI3475, SGI3475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG780[] = {SGI3476, SGI3477, SGI3478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3479, SGI0, SGI0, SGI3480, SGI3480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG781[] = {SGI3481, SGI3482, SGI3483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3484, SGI0, SGI0, SGI3485, SGI3485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG782[] = {SGI3486, SGI3487, SGI3488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3489, SGI0, SGI0, SGI3490, SGI3490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG783[] = {SGI3491, SGI3492, SGI3493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3494, SGI0, SGI0, SGI3495, SGI3495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG784[] = {SGI3496, SGI3497, SGI3498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3499, SGI0, SGI0, SGI0, SGI3500, SGI3500, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG785[] = {SGI3501, SGI3502, SGI3503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3504, SGI0, SGI0, SGI0, SGI3505, SGI3505, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG786[] = {SGI3506, SGI3507, SGI3508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3509, SGI0, SGI0, SGI0, SGI3510, SGI3510, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG787[] = {SGI0, SGI3511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3512, SGI0, SGI0, SGI0, SGI3513, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG788[] = {SGI3514, SGI3515, SGI3516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3517, SGI0, SGI0, SGI0, SGI3518, SGI3518, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG789[] = {SGI0, SGI3519, SGI3520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3521, SGI0, SGI0, SGI0, SGI3522, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG790[] = {SGI3523, SGI3524, SGI3525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3526, SGI0, SGI0, SGI0, SGI3527, SGI3527, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG791[] = {SGI3528, SGI3529, SGI3530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3531, SGI0, SGI0, SGI0, SGI3532, SGI3532, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG792[] = {SGI3533, SGI3534, SGI3535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3536, SGI0, SGI0, SGI0, SGI3537, SGI3537, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG793[] = {SGI3538, SGI0, SGI3539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG794[] = {SGI3540, SGI3541, SGI3542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3543, SGI0, SGI0, SGI0, SGI3544, SGI3544, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG795[] = {SGI3545, SGI3546, SGI3547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3548, SGI3549, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG796[] = {SGI3550, SGI3551, SGI3552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3553, SGI3554, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG797[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3555, SGI3556, SGI3557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG798[] = {SGI3558, SGI3559, SGI3560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3561, SGI0, SGI0, SGI0, SGI3562, SGI3562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG799[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3563, SGI3564, SGI3565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG800[] = {SGI3566, SGI3567, SGI3568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3569, SGI0, SGI0, SGI0, SGI3570, SGI3570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG801[] = {SGI3571, SGI3572, SGI3573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3574, SGI0, SGI0, SGI0, SGI3575, SGI3575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG802[] = {SGI3576, SGI3577, SGI3578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3579, SGI0, SGI0, SGI0, SGI3580, SGI3580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG803[] = {SGI3581, SGI3582, SGI3583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3584, SGI0, SGI0, SGI0, SGI3585, SGI3585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG804[] = {SGI3586, SGI3587, SGI3588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3589, SGI0, SGI0, SGI0, SGI3590, SGI3590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG805[] = {SGI3591, SGI3592, SGI3593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3594, SGI0, SGI0, SGI0, SGI3595, SGI3595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG806[] = {SGI3596, SGI0, SGI3597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG807[] = {SGI3598, SGI3599, SGI3600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3601, SGI0, SGI0, SGI0, SGI3602, SGI3602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG808[] = {SGI3603, SGI3604, SGI3605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3606, SGI0, SGI0, SGI0, SGI3607, SGI3607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG809[] = {SGI3608, SGI3609, SGI3610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3611, SGI0, SGI0, SGI0, SGI3612, SGI3612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG810[] = {SGI3613, SGI3614, SGI3615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3616, SGI0, SGI0, SGI0, SGI3617, SGI3617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG811[] = {SGI3618, SGI3619, SGI3620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3621, SGI0, SGI0, SGI0, SGI3622, SGI3622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG812[] = {SGI3623, SGI3624, SGI3625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3626, SGI0, SGI0, SGI0, SGI3627, SGI3627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG813[] = {SGI3628, SGI3629, SGI3630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3631, SGI0, SGI0, SGI0, SGI3632, SGI3632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG814[] = {SGI3633, SGI3634, SGI3635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3636, SGI0, SGI0, SGI0, SGI3637, SGI3637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG815[] = {SGI3638, SGI3639, SGI3640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3641, SGI0, SGI0, SGI0, SGI3642, SGI3642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG816[] = {SGI3643, SGI3644, SGI3645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3646, SGI0, SGI0, SGI0, SGI3647, SGI3647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG817[] = {SGI3648, SGI3649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3650, SGI0, SGI0, SGI0, SGI3651, SGI3651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG818[] = {SGI3652, SGI3653, SGI3654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3655, SGI0, SGI0, SGI0, SGI3656, SGI3656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG819[] = {SGI3657, SGI3658, SGI3659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3660, SGI0, SGI0, SGI0, SGI3661, SGI3661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG820[] = {SGI3662, SGI3663, SGI3664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3665, SGI0, SGI0, SGI0, SGI3666, SGI3666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG821[] = {SGI3667, SGI0, SGI3668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3669, SGI0, SGI0, SGI0, SGI3670, SGI3670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG822[] = {SGI3671, SGI3672, SGI3673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG823[] = {SGI3674, SGI3675, SGI3676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3677, SGI0, SGI0, SGI0, SGI3678, SGI3678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG824[] = {SGI3679, SGI3680, SGI3681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3682, SGI0, SGI0, SGI0, SGI3683, SGI3683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG825[] = {SGI3684, SGI0, SGI3685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG826[] = {SGI3686, SGI3687, SGI3688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3689, SGI0, SGI0, SGI0, SGI3690, SGI3690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG827[] = {SGI3691, SGI3692, SGI3693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3694, SGI0, SGI0, SGI0, SGI3695, SGI3696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG828[] = {SGI3697, SGI0, SGI3698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3699, SGI0, SGI0, SGI0, SGI3700, SGI3700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG829[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3701, NULL};
const Segment * const SG830[] = {SGI3702, SGI3703, SGI3704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3705, SGI0, SGI0, SGI0, SGI3706, SGI3706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG831[] = {SGI3707, SGI3708, SGI3709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3710, SGI0, SGI0, SGI0, SGI3711, SGI3711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG832[] = {SGI3712, SGI3713, SGI3714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3715, SGI0, SGI0, SGI0, SGI3716, SGI3716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG833[] = {SGI3717, SGI3718, SGI3719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3720, SGI0, SGI0, SGI0, SGI3721, SGI3721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG834[] = {SGI3722, SGI3723, SGI3724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3725, SGI0, SGI0, SGI0, SGI3726, SGI3726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG835[] = {SGI3727, SGI3728, SGI3729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3730, SGI0, SGI0, SGI0, SGI3731, SGI3731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG836[] = {SGI3732, SGI3733, SGI3734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3735, SGI0, SGI0, SGI0, SGI3736, SGI3736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG837[] = {SGI3737, SGI3738, SGI3739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3740, SGI0, SGI0, SGI0, SGI3741, SGI3741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG838[] = {SGI3742, SGI3743, SGI3744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3745, SGI0, SGI0, SGI0, SGI3746, SGI3746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG839[] = {SGI3747, SGI3748, SGI3749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3750, SGI0, SGI0, SGI0, SGI3751, SGI3751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG840[] = {SGI3752, SGI3753, SGI3754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3755, SGI0, SGI0, SGI0, SGI3756, SGI3756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG841[] = {SGI3757, SGI3758, SGI3759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3760, SGI0, SGI0, SGI0, SGI3761, SGI3761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG842[] = {SGI3762, SGI3763, SGI3764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3765, SGI0, SGI0, SGI0, SGI3766, SGI3766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG843[] = {SGI3767, SGI3768, SGI3769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3770, SGI0, SGI0, SGI0, SGI3771, SGI3771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG844[] = {SGI3772, SGI3773, SGI3774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3775, SGI0, SGI0, SGI0, SGI3776, SGI3776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG845[] = {SGI3777, SGI3778, SGI3779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3780, SGI0, SGI0, SGI0, SGI3781, SGI3781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG846[] = {SGI3782, SGI0, SGI3783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG847[] = {SGI3784, SGI3785, SGI3786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3787, SGI3788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG848[] = {SGI3789, SGI3790, SGI3791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3792, SGI0, SGI0, SGI0, SGI3793, SGI3794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG849[] = {SGI3795, SGI3796, SGI3797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3798, SGI3798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG850[] = {SGI3799, SGI3800, SGI3801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3802, SGI0, SGI0, SGI0, SGI3803, SGI3803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG851[] = {SGI3804, SGI0, SGI3805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG852[] = {SGI3806, SGI3807, SGI3808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3809, SGI0, SGI0, SGI0, SGI3810, SGI3810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG853[] = {SGI3811, SGI3812, SGI3813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3814, SGI0, SGI0, SGI0, SGI3815, SGI3815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG854[] = {SGI3816, SGI0, SGI3817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG855[] = {SGI3818, SGI3819, SGI3820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3821, SGI3822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG856[] = {SGI3823, SGI0, SGI3824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG857[] = {SGI3825, SGI3826, SGI3827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3828, SGI3828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG858[] = {SGI3829, SGI0, SGI3830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3831, SGI0, SGI0, SGI0, SGI3832, SGI3832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG859[] = {SGI3833, SGI0, SGI3834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3835, SGI0, SGI0, SGI0, SGI3836, SGI3836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG860[] = {SGI3837, SGI3838, SGI3839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3840, SGI0, SGI0, SGI0, SGI3841, SGI3841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG861[] = {SGI3842, SGI3843, SGI3844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3845, SGI0, SGI0, SGI0, SGI3846, SGI3846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG862[] = {SGI3847, SGI3848, SGI3849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3850, SGI0, SGI0, SGI0, SGI3851, SGI3851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG863[] = {SGI3852, SGI3853, SGI3854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3855, SGI0, SGI0, SGI0, SGI3856, SGI3856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG864[] = {SGI3857, SGI3858, SGI3859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3860, SGI0, SGI0, SGI0, SGI3861, SGI3861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG865[] = {SGI3862, SGI3863, SGI3864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3865, SGI0, SGI0, SGI0, SGI3866, SGI3866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG866[] = {SGI3867, SGI3868, SGI3869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3870, SGI0, SGI0, SGI0, SGI3871, SGI3871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG867[] = {SGI3872, SGI3873, SGI3874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3875, SGI0, SGI0, SGI0, SGI3876, SGI3876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG868[] = {SGI3877, SGI0, SGI3878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG869[] = {SGI3879, SGI3880, SGI3881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3882, SGI3883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG870[] = {SGI3884, SGI0, SGI3885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG871[] = {SGI3886, SGI3887, SGI3888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3889, SGI3889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG872[] = {SGI3890, SGI3891, SGI3892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3893, SGI0, SGI0, SGI0, SGI3894, SGI3894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG873[] = {SGI3895, SGI3896, SGI3897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3898, SGI0, SGI0, SGI0, SGI3899, SGI3899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG874[] = {SGI3900, SGI3901, SGI3902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3903, SGI0, SGI0, SGI0, SGI3904, SGI3904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG875[] = {SGI3905, SGI0, SGI3906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG876[] = {SGI3907, SGI0, SGI3908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG877[] = {SGI3909, SGI3910, SGI3911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3912, SGI0, SGI0, SGI0, SGI3913, SGI3913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG878[] = {SGI3914, SGI0, SGI3915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG879[] = {SGI3916, SGI3917, SGI3918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3919, SGI0, SGI0, SGI0, SGI3920, SGI3920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG880[] = {SGI3921, SGI3922, SGI3923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3924, SGI0, SGI0, SGI0, SGI3925, SGI3925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG881[] = {SGI3926, SGI3927, SGI3928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3929, SGI0, SGI0, SGI0, SGI3930, SGI3930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG882[] = {SGI3931, SGI0, SGI3932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3933, SGI0, SGI0, SGI0, SGI3934, SGI3934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG883[] = {SGI3935, SGI3936, SGI3937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3938, SGI0, SGI0, SGI0, SGI3939, SGI3939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG884[] = {SGI3940, SGI0, SGI3941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG885[] = {SGI3942, SGI3943, SGI3944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3945, SGI0, SGI0, SGI0, SGI3946, SGI3947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG886[] = {SGI3948, SGI0, SGI3949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3950, SGI0, SGI0, SGI0, SGI3951, SGI3951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG887[] = {SGI3952, SGI3953, SGI3954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3955, SGI0, SGI0, SGI0, SGI3956, SGI3956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG888[] = {SGI3957, SGI3958, SGI3959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3960, SGI0, SGI0, SGI0, SGI3961, SGI3961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG889[] = {SGI3962, SGI3963, SGI3964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3965, SGI0, SGI0, SGI0, SGI3966, SGI3966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG890[] = {SGI3967, SGI3968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3969, SGI0, SGI0, SGI0, SGI3970, SGI3970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG891[] = {SGI3971, SGI3972, SGI3973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3974, SGI0, SGI0, SGI0, SGI3975, SGI3975, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG892[] = {SGI3976, SGI3977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3978, SGI0, SGI0, SGI0, SGI3979, SGI3979, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG893[] = {SGI3980, SGI3981, SGI3982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3983, SGI0, SGI0, SGI0, SGI3984, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG894[] = {SGI3985, SGI3986, SGI3987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3988, SGI0, SGI0, SGI0, SGI3989, SGI3989, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG895[] = {SGI3990, SGI3991, SGI3992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3993, SGI0, SGI0, SGI0, SGI3994, SGI3994, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG896[] = {SGI3995, SGI3996, SGI3997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI3998, SGI0, SGI0, SGI0, SGI3999, SGI3999, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG897[] = {SGI4000, SGI4001, SGI4002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4003, SGI0, SGI0, SGI0, SGI4004, SGI4004, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG898[] = {SGI4005, SGI4006, SGI4007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4008, SGI0, SGI0, SGI0, SGI4009, SGI4009, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG899[] = {SGI4010, SGI4011, SGI4012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4013, SGI0, SGI0, SGI0, SGI4014, SGI4014, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG900[] = {SGI4015, SGI4016, SGI4017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4018, SGI0, SGI0, SGI0, SGI4019, SGI4019, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG901[] = {SGI4020, SGI4021, SGI4022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4023, SGI0, SGI0, SGI0, SGI4024, SGI4024, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG902[] = {SGI4025, SGI0, SGI4026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4027, SGI0, SGI0, SGI0, SGI4028, SGI4028, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG903[] = {SGI4029, SGI4030, SGI4031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4032, SGI0, SGI0, SGI0, SGI4033, SGI4033, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG904[] = {SGI4034, SGI4035, SGI4036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4037, SGI0, SGI0, SGI0, SGI4038, SGI4038, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG905[] = {SGI4039, SGI4040, SGI4041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4042, SGI0, SGI0, SGI0, SGI4043, SGI4043, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG906[] = {SGI4044, SGI4045, SGI4046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4047, SGI0, SGI0, SGI0, SGI4048, SGI4048, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG907[] = {SGI4050, SGI4051, SGI4052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4053, SGI0, SGI0, SGI0, SGI4054, SGI4054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG908[] = {SGI4055, SGI4056, SGI4057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4058, SGI0, SGI0, SGI0, SGI4059, SGI4059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG909[] = {SGI4060, SGI4061, SGI4062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4063, SGI0, SGI0, SGI0, SGI4064, SGI4064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG910[] = {SGI4065, SGI0, SGI4066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG911[] = {SGI4067, SGI4068, SGI4069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4070, SGI0, SGI0, SGI0, SGI4071, SGI4071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG912[] = {SGI4049, SGI4072, SGI4073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4074, SGI0, SGI0, SGI0, SGI4075, SGI4075, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG913[] = {SGI4076, SGI4077, SGI4078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4079, SGI0, SGI0, SGI0, SGI4080, SGI4080, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG914[] = {SGI0, SGI4082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4085, SGI0, SGI0, SGI0, SGI4086, SGI4086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG915[] = {SGI4081, SGI4083, SGI4084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4087, SGI0, SGI0, SGI0, SGI4088, SGI4088, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG916[] = {SGI4089, SGI4090, SGI4091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4092, SGI0, SGI0, SGI0, SGI4093, SGI4093, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG917[] = {SGI4094, SGI4095, SGI4096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4097, SGI0, SGI0, SGI0, SGI4098, SGI4098, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG918[] = {SGI4099, SGI4100, SGI4101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4102, SGI0, SGI0, SGI0, SGI4103, SGI4103, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG919[] = {SGI4104, SGI4105, SGI4106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4107, SGI0, SGI0, SGI0, SGI4108, SGI4108, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG920[] = {SGI4109, SGI4110, SGI4111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4112, SGI0, SGI0, SGI0, SGI4113, SGI4113, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG921[] = {SGI4114, SGI4115, SGI4116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4117, SGI0, SGI0, SGI0, SGI4118, SGI4118, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG922[] = {SGI4119, SGI4120, SGI4121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4122, SGI0, SGI0, SGI0, SGI4123, SGI4123, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG923[] = {SGI4124, SGI4125, SGI4126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4127, SGI0, SGI0, SGI0, SGI4128, SGI4128, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG924[] = {SGI4129, SGI4130, SGI4131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4132, SGI0, SGI0, SGI0, SGI4133, SGI4133, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG925[] = {SGI4134, SGI4135, SGI4136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4137, SGI0, SGI0, SGI0, SGI4138, SGI4138, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG926[] = {SGI4139, SGI4140, SGI4141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4142, SGI0, SGI0, SGI0, SGI4143, SGI4143, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG927[] = {SGI4144, SGI4145, SGI4146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4147, SGI0, SGI0, SGI0, SGI4148, SGI4148, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG928[] = {SGI4149, SGI4150, SGI4151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4152, SGI0, SGI0, SGI0, SGI4153, SGI4153, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG929[] = {SGI4154, SGI4155, SGI4156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4157, SGI0, SGI0, SGI0, SGI4158, SGI4158, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG930[] = {SGI4159, SGI0, SGI4160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG931[] = {SGI4161, SGI4162, SGI4163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4164, SGI4165, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG932[] = {SGI4166, SGI4167, SGI4168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4169, SGI0, SGI0, SGI0, SGI4170, SGI4170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG933[] = {SGI4171, SGI4172, SGI4173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4174, SGI0, SGI0, SGI0, SGI4175, SGI4175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG934[] = {SGI4176, SGI4177, SGI4178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4179, SGI0, SGI0, SGI0, SGI4180, SGI4180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG935[] = {SGI4181, SGI4182, SGI4183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4184, SGI0, SGI0, SGI0, SGI4185, SGI4185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG936[] = {SGI4186, SGI4187, SGI4188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4189, SGI0, SGI0, SGI0, SGI4190, SGI4190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG937[] = {SGI4191, SGI4192, SGI4193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4194, SGI0, SGI0, SGI0, SGI4195, SGI4195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG938[] = {SGI4196, SGI4197, SGI4198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4199, SGI0, SGI0, SGI0, SGI4200, SGI4200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG939[] = {SGI4201, SGI4202, SGI4203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4204, SGI0, SGI0, SGI0, SGI4205, SGI4205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG940[] = {SGI4206, SGI4207, SGI4208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4209, SGI0, SGI0, SGI0, SGI4210, SGI4210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG941[] = {SGI4211, SGI4212, SGI4213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4214, SGI0, SGI0, SGI0, SGI4215, SGI4215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG942[] = {SGI4216, SGI4217, SGI4218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4219, SGI0, SGI0, SGI0, SGI4220, SGI4220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG943[] = {SGI4221, SGI4222, SGI4223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4224, SGI0, SGI0, SGI0, SGI4225, SGI4225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG944[] = {SGI4226, SGI4227, SGI4228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4229, SGI0, SGI0, SGI0, SGI4230, SGI4230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG945[] = {SGI4231, SGI4232, SGI4233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4234, SGI0, SGI0, SGI0, SGI4235, SGI4235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG946[] = {SGI4236, SGI4237, SGI4238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4239, SGI0, SGI0, SGI0, SGI4240, SGI4240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG947[] = {SGI4241, SGI4242, SGI4243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4244, SGI0, SGI0, SGI0, SGI4245, SGI4245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG948[] = {SGI4246, SGI4247, SGI4248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4249, SGI0, SGI0, SGI0, SGI4250, SGI4250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG949[] = {SGI4251, SGI4252, SGI4253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4254, SGI0, SGI0, SGI0, SGI4255, SGI4255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG950[] = {SGI4256, SGI4257, SGI4258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4259, SGI0, SGI0, SGI0, SGI4260, SGI4260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG951[] = {SGI4261, SGI4262, SGI4263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4264, SGI0, SGI0, SGI0, SGI4265, SGI4265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG952[] = {SGI4266, SGI4267, SGI4268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4269, SGI0, SGI0, SGI0, SGI4270, SGI4270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG953[] = {SGI4271, SGI0, SGI4272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG954[] = {SGI4273, SGI4274, SGI4275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4276, SGI0, SGI0, SGI0, SGI4277, SGI4277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG955[] = {SGI4278, SGI4279, SGI4280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4281, SGI0, SGI0, SGI0, SGI4282, SGI4282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG956[] = {SGI4283, SGI4284, SGI4285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4286, SGI0, SGI0, SGI0, SGI4287, SGI4287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG957[] = {SGI4288, SGI4289, SGI4290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4291, SGI0, SGI0, SGI0, SGI4292, SGI4292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG958[] = {SGI4293, SGI4294, SGI4295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4296, SGI0, SGI0, SGI0, SGI4297, SGI4297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG959[] = {SGI4298, SGI4299, SGI4300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4301, SGI0, SGI0, SGI0, SGI4302, SGI4302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG960[] = {SGI4303, SGI4304, SGI4305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4306, SGI0, SGI0, SGI0, SGI4307, SGI4307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG961[] = {SGI4308, SGI4309, SGI4310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4311, SGI0, SGI0, SGI0, SGI4312, SGI4312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG962[] = {SGI4313, SGI4314, SGI4315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4316, SGI0, SGI0, SGI0, SGI4317, SGI4317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG963[] = {SGI4318, SGI4319, SGI4320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4321, SGI0, SGI0, SGI0, SGI4322, SGI4322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG964[] = {SGI4323, SGI0, SGI4324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG965[] = {SGI4325, SGI4326, SGI4327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4328, SGI0, SGI0, SGI0, SGI4329, SGI4329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG966[] = {SGI4330, SGI4331, SGI4332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4333, SGI0, SGI0, SGI0, SGI4334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG967[] = {SGI4335, SGI4336, SGI4337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4338, SGI0, SGI0, SGI0, SGI4339, SGI4339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG968[] = {SGI4340, SGI4341, SGI4342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4343, SGI0, SGI0, SGI0, SGI4344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG969[] = {SGI4345, SGI4346, SGI4347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4348, SGI0, SGI0, SGI0, SGI4349, SGI4349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG970[] = {SGI4350, SGI4351, SGI4352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG971[] = {SGI4353, SGI4354, SGI4355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4356, SGI0, SGI0, SGI0, SGI4357, SGI4357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG972[] = {SGI4358, SGI4359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4360, SGI0, SGI0, SGI0, SGI4361, SGI4361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG973[] = {SGI4362, SGI0, SGI4363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG974[] = {SGI4364, SGI4365, SGI4366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG975[] = {SGI4367, SGI4368, SGI4369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4370, SGI4371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG976[] = {SGI4372, SGI0, SGI4373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4374, SGI0, SGI0, SGI0, SGI4375, SGI4375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG977[] = {SGI4376, SGI4377, SGI4378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4379, SGI0, SGI0, SGI0, SGI4380, SGI4380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG978[] = {SGI4381, SGI4382, SGI4383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4384, SGI0, SGI0, SGI0, SGI4385, SGI4386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG979[] = {SGI4387, SGI4388, SGI4389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4390, SGI0, SGI0, SGI0, SGI4391, SGI4391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG980[] = {SGI4392, SGI4393, SGI4394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4395, SGI0, SGI0, SGI0, SGI4396, SGI4396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG981[] = {SGI4397, SGI4398, SGI4399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4400, SGI0, SGI0, SGI0, SGI4401, SGI4401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG982[] = {SGI4402, SGI4403, SGI4404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4405, SGI0, SGI0, SGI0, SGI4406, SGI4406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG983[] = {SGI4407, SGI4408, SGI4409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4410, SGI0, SGI0, SGI0, SGI4411, SGI4411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG984[] = {SGI4412, SGI0, SGI4413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG985[] = {SGI4414, SGI0, SGI4415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG986[] = {SGI4416, SGI4417, SGI4418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4419, SGI0, SGI0, SGI0, SGI4420, SGI4420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG987[] = {SGI4421, SGI4422, SGI4423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4424, SGI0, SGI0, SGI0, SGI4425, SGI4426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG988[] = {SGI4427, SGI4428, SGI4429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4430, SGI0, SGI0, SGI0, SGI4431, SGI4431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG989[] = {SGI4432, SGI0, SGI4433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG990[] = {SGI4434, SGI4435, SGI4436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4437, SGI0, SGI0, SGI0, SGI4438, SGI4438, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG991[] = {SGI4439, SGI4440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4441, SGI0, SGI0, SGI0, SGI4442, SGI4442, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG992[] = {SGI4443, SGI4444, SGI4445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4446, SGI0, SGI0, SGI0, SGI4447, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG993[] = {SGI4448, SGI4449, SGI4450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4451, SGI0, SGI0, SGI0, SGI4452, SGI4452, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG994[] = {SGI4453, SGI4454, SGI4455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4456, SGI0, SGI0, SGI0, SGI4457, SGI4457, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG995[] = {SGI4458, SGI4459, SGI4460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4461, SGI0, SGI0, SGI0, SGI4462, SGI4462, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG996[] = {SGI4463, SGI4464, SGI4465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4466, SGI0, SGI0, SGI0, SGI4467, SGI4467, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG997[] = {SGI4468, SGI0, SGI4469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4470, SGI0, SGI0, SGI0, SGI4471, SGI4471, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG998[] = {SGI4472, SGI4473, SGI4474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4475, SGI0, SGI0, SGI0, SGI4476, SGI4477, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG999[] = {SGI4478, SGI0, SGI4479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1000[] = {SGI4480, SGI4481, SGI4482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4483, SGI0, SGI0, SGI0, SGI4484, SGI4484, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1001[] = {SGI4485, SGI0, SGI4486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1002[] = {SGI4487, SGI4488, SGI4489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4490, SGI0, SGI0, SGI0, SGI4491, SGI4491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1003[] = {SGI4492, SGI4493, SGI4494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4495, SGI0, SGI0, SGI0, SGI4496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1004[] = {SGI4497, SGI4498, SGI4499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4500, SGI0, SGI0, SGI0, SGI4501, SGI4501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1005[] = {SGI4502, SGI0, SGI4503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4504, SGI0, SGI0, SGI0, SGI4505, SGI4505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1006[] = {SGI4506, SGI4507, SGI4508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4509, SGI0, SGI0, SGI0, SGI4510, SGI4510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1007[] = {SGI4511, SGI4512, SGI4513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4514, SGI0, SGI0, SGI0, SGI4515, SGI4515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1008[] = {SGI4516, SGI4517, SGI4518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4519, SGI0, SGI0, SGI0, SGI4520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1009[] = {SGI4521, SGI4522, SGI4523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4524, SGI0, SGI0, SGI0, SGI4525, SGI4525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1010[] = {SGI4526, SGI4527, SGI4528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4529, SGI0, SGI0, SGI0, SGI4530, SGI4530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1011[] = {SGI4531, SGI4532, SGI4533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4534, SGI0, SGI0, SGI0, SGI4535, SGI4535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1012[] = {SGI4536, SGI4537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4538, SGI0, SGI0, SGI0, SGI4539, SGI4539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1013[] = {SGI4540, SGI4541, SGI4542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4543, SGI0, SGI0, SGI0, SGI4544, SGI4544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1014[] = {SGI4545, SGI4546, SGI4547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4548, SGI0, SGI0, SGI0, SGI4549, SGI4549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1015[] = {SGI4550, SGI4551, SGI4552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4553, SGI0, SGI0, SGI0, SGI4554, SGI4554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1016[] = {SGI4555, SGI0, SGI4556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1017[] = {SGI4557, SGI4558, SGI4559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4560, SGI0, SGI0, SGI0, SGI4561, SGI4561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1018[] = {SGI4562, SGI0, SGI4563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1019[] = {SGI4564, SGI4565, SGI4566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4567, SGI4568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1020[] = {SGI4569, SGI4570, SGI4571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4572, SGI0, SGI0, SGI0, SGI4573, SGI4574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1021[] = {SGI4575, SGI4576, SGI4577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4578, SGI0, SGI0, SGI0, SGI4580, SGI4580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1022[] = {SGI4579, SGI4581, SGI4582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4583, SGI0, SGI0, SGI0, SGI4584, SGI4584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1023[] = {SGI4585, SGI4586, SGI4587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4588, SGI0, SGI0, SGI0, SGI4589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1024[] = {SGI4590, SGI4591, SGI4592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4593, SGI0, SGI0, SGI0, SGI4594, SGI4594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1025[] = {SGI4595, SGI0, SGI4596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1026[] = {SGI4597, SGI4598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4599, SGI0, SGI0, SGI0, SGI4600, SGI4600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1027[] = {SGI4601, SGI4602, SGI4603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4604, SGI0, SGI0, SGI0, SGI4605, SGI4605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1028[] = {SGI4606, SGI0, SGI4607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4608, SGI0, SGI0, SGI0, SGI4609, SGI4609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1029[] = {SGI4610, SGI4611, SGI4612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4613, SGI0, SGI0, SGI0, SGI4614, SGI4614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1030[] = {SGI4615, SGI4616, SGI4617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4618, SGI0, SGI0, SGI0, SGI4619, SGI4619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1031[] = {SGI4620, SGI4621, SGI4622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4623, SGI0, SGI0, SGI0, SGI4624, SGI4624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1032[] = {SGI4625, SGI0, SGI4626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1033[] = {SGI4627, SGI4628, SGI4629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1034[] = {SGI4630, SGI4631, SGI4632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4633, SGI0, SGI0, SGI0, SGI4634, SGI4635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1035[] = {SGI4636, SGI4637, SGI4638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4639, SGI0, SGI0, SGI0, SGI4640, SGI4640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1036[] = {SGI4641, SGI4642, SGI4643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4644, SGI0, SGI0, SGI4645, SGI4645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1037[] = {SGI4646, SGI4647, SGI4648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4649, SGI0, SGI0, SGI0, SGI4650, SGI4650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1038[] = {SGI4651, SGI4652, SGI4653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4654, SGI0, SGI0, SGI0, SGI4655, SGI4655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1039[] = {SGI4656, SGI4657, SGI4658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4659, SGI0, SGI0, SGI0, SGI4660, SGI4660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1040[] = {SGI4661, SGI0, SGI4662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4663, SGI0, SGI0, SGI0, SGI4664, SGI4664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1041[] = {SGI4665, SGI4666, SGI4667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4668, SGI0, SGI0, SGI0, SGI4669, SGI4669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1042[] = {SGI4670, SGI4671, SGI4672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4673, SGI0, SGI0, SGI0, SGI4674, SGI4674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1043[] = {SGI4675, SGI4676, SGI4677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4678, SGI0, SGI0, SGI0, SGI4679, SGI4679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1044[] = {SGI4680, SGI4681, SGI4682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4683, SGI0, SGI0, SGI0, SGI4684, SGI4684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1045[] = {SGI4685, SGI4686, SGI4687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4688, SGI0, SGI0, SGI0, SGI4689, SGI4689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1046[] = {SGI4690, SGI4691, SGI4692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4693, SGI0, SGI0, SGI0, SGI4694, SGI4694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1047[] = {SGI4695, SGI4696, SGI4697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4698, SGI0, SGI0, SGI0, SGI4699, SGI4699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1048[] = {SGI4700, SGI0, SGI4701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1049[] = {SGI4702, SGI4703, SGI4704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4705, SGI4706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1050[] = {SGI4707, SGI4708, SGI4709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4710, SGI0, SGI0, SGI0, SGI4711, SGI4711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1051[] = {SGI4712, SGI4713, SGI4714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4715, SGI0, SGI0, SGI0, SGI4716, SGI4716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1052[] = {SGI4717, SGI4718, SGI4719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4720, SGI4720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1053[] = {SGI4721, SGI4722, SGI4723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4724, SGI0, SGI0, SGI0, SGI4725, SGI4725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1054[] = {SGI4726, SGI0, SGI4727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1055[] = {SGI4728, SGI0, SGI4729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4730, SGI0, SGI0, SGI0, SGI4731, SGI4731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1056[] = {SGI4732, SGI4733, SGI4734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4735, SGI4736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1057[] = {SGI4737, SGI4738, SGI4739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4740, SGI4740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1058[] = {SGI4741, SGI4742, SGI4743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1059[] = {SGI4744, SGI4745, SGI4746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1060[] = {SGI0, SGI4747, SGI4748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1061[] = {SGI4749, SGI4750, SGI4751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1062[] = {SGI4752, SGI4753, SGI4754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1063[] = {SGI4755, SGI4756, SGI4757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1064[] = {SGI4758, SGI4759, SGI4760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1065[] = {SGI4761, SGI4762, SGI4763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4764, SGI0, SGI0, SGI0, SGI4765, SGI4765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1066[] = {SGI4766, SGI4767, SGI4768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4769, SGI0, SGI0, SGI0, SGI4770, SGI4770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1067[] = {SGI4771, SGI4772, SGI4773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1068[] = {SGI4774, SGI4775, SGI4776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4777, SGI0, SGI0, SGI0, SGI4778, SGI4778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1069[] = {SGI4779, SGI4780, SGI4781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4782, SGI0, SGI0, SGI0, SGI4783, SGI4784, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1070[] = {SGI4785, SGI4786, SGI4787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4788, SGI0, SGI0, SGI0, SGI4789, SGI4789, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1071[] = {SGI4790, SGI4791, SGI4792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4793, SGI0, SGI0, SGI0, SGI4794, SGI4794, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1072[] = {SGI4795, SGI0, SGI4796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4797, SGI0, SGI0, SGI0, SGI4798, SGI4798, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1073[] = {SGI4799, SGI4800, SGI4801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4802, SGI0, SGI0, SGI0, SGI4803, SGI4803, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1074[] = {SGI4804, SGI4805, SGI4806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4807, SGI0, SGI0, SGI0, SGI4808, SGI4808, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1075[] = {SGI4809, SGI0, SGI4810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1076[] = {SGI4811, SGI0, SGI4812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1077[] = {SGI4813, SGI4814, SGI4815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4816, SGI0, SGI0, SGI0, SGI4817, SGI4818, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1078[] = {SGI4819, SGI4820, SGI4821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4822, SGI0, SGI0, SGI0, SGI4823, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1079[] = {SGI4824, SGI4825, SGI4826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4827, SGI0, SGI0, SGI0, SGI4828, SGI4828, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1080[] = {SGI4829, SGI4830, SGI4831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4832, SGI0, SGI0, SGI0, SGI4833, SGI4833, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1081[] = {SGI4834, SGI4835, SGI4836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4837, SGI0, SGI0, SGI0, SGI4838, SGI4838, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1082[] = {SGI4839, SGI4840, SGI4841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4842, SGI0, SGI0, SGI0, SGI4843, SGI4843, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1083[] = {SGI4844, SGI4845, SGI4846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4847, SGI0, SGI0, SGI0, SGI4848, SGI4848, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1084[] = {SGI4849, SGI4850, SGI4851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4852, SGI0, SGI0, SGI0, SGI4853, SGI4853, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1085[] = {SGI4854, SGI4855, SGI4856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4857, SGI0, SGI0, SGI0, SGI4858, SGI4858, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1086[] = {SGI4859, SGI4860, SGI4861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4862, SGI0, SGI0, SGI0, SGI4863, SGI4863, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1087[] = {SGI4864, SGI4865, SGI4866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4867, SGI0, SGI0, SGI0, SGI4868, SGI4868, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1088[] = {SGI4869, SGI4870, SGI4871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4872, SGI0, SGI0, SGI0, SGI4873, SGI4873, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1089[] = {SGI4874, SGI4875, SGI4876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4877, SGI0, SGI0, SGI0, SGI4878, SGI4878, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1090[] = {SGI4879, SGI4880, SGI4881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4882, SGI0, SGI0, SGI0, SGI4883, SGI4883, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1091[] = {SGI4884, SGI4885, SGI4886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4887, SGI0, SGI0, SGI0, SGI4888, SGI4888, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1092[] = {SGI4889, SGI4890, SGI4891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4892, SGI0, SGI0, SGI0, SGI4893, SGI4893, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1093[] = {SGI4894, SGI4895, SGI4896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4897, SGI0, SGI0, SGI0, SGI4898, SGI4898, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1094[] = {SGI4899, SGI4900, SGI4901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4902, SGI0, SGI0, SGI0, SGI4903, SGI4903, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1095[] = {SGI4904, SGI4905, SGI4906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4907, SGI0, SGI0, SGI0, SGI4908, SGI4908, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1096[] = {SGI4909, SGI4910, SGI4911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4912, SGI0, SGI0, SGI0, SGI4913, SGI4913, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1097[] = {SGI4914, SGI4915, SGI4916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4917, SGI0, SGI0, SGI0, SGI4918, SGI4918, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1098[] = {SGI4919, SGI4920, SGI4921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4922, SGI0, SGI0, SGI0, SGI4923, SGI4923, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1099[] = {SGI4924, SGI4925, SGI4926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4927, SGI0, SGI0, SGI0, SGI4928, SGI4928, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1100[] = {SGI4929, SGI4930, SGI4931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4932, SGI0, SGI0, SGI0, SGI4933, SGI4933, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1101[] = {SGI4934, SGI4935, SGI4936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4937, SGI0, SGI0, SGI0, SGI4938, SGI4938, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1102[] = {SGI4939, SGI4940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4941, SGI0, SGI0, SGI0, SGI4942, SGI4942, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1103[] = {SGI4943, SGI0, SGI4944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4945, SGI0, SGI0, SGI0, SGI4946, SGI4946, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1104[] = {SGI4947, SGI4948, SGI4949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4950, SGI0, SGI0, SGI0, SGI4951, SGI4951, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1105[] = {SGI4952, SGI0, SGI4953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1106[] = {SGI4954, SGI4955, SGI4956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4957, SGI0, SGI0, SGI0, SGI4958, SGI4958, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1107[] = {SGI4959, SGI4960, SGI4961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4962, SGI0, SGI0, SGI0, SGI4963, SGI4964, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1108[] = {SGI4965, SGI4966, SGI4967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4968, SGI0, SGI0, SGI0, SGI4969, SGI4969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1109[] = {SGI0, SGI4970, SGI4971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4972, SGI0, SGI0, SGI0, SGI4973, SGI4973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1110[] = {SGI4974, SGI4975, SGI4976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4977, SGI0, SGI0, SGI0, SGI0, SGI4978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1111[] = {SGI4979, SGI0, SGI4980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1112[] = {SGI4981, SGI4982, SGI4983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4984, SGI0, SGI0, SGI0, SGI0, SGI4985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1113[] = {SGI4986, SGI0, SGI4987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1114[] = {SGI4988, SGI4989, SGI4990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4991, SGI0, SGI0, SGI0, SGI0, SGI4992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1115[] = {SGI4993, SGI4994, SGI4995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI4996, SGI0, SGI0, SGI0, SGI0, SGI4997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1116[] = {SGI4998, SGI4999, SGI5000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5001, SGI0, SGI0, SGI0, SGI5002, SGI5003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1117[] = {SGI5004, SGI5005, SGI5006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1118[] = {SGI5008, SGI5009, SGI5010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1119[] = {SGI5012, SGI5013, SGI5014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1120[] = {SGI5015, SGI5016, SGI5017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5018, SGI0, SGI0, SGI0, SGI0, SGI5019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1121[] = {SGI5020, SGI5021, SGI5022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5023, SGI0, SGI0, SGI0, SGI5024, SGI5024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1122[] = {SGI5025, SGI5026, SGI5027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5028, SGI0, SGI0, SGI0, SGI5029, SGI5029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1123[] = {SGI5030, SGI5031, SGI5032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5033, SGI0, SGI0, SGI0, SGI5034, SGI5034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1124[] = {SGI5035, SGI0, SGI5036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1125[] = {SGI5037, SGI5038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5039, SGI0, SGI0, SGI0, SGI5040, SGI5040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1126[] = {SGI5041, SGI5042, SGI5043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5044, SGI0, SGI0, SGI0, SGI5045, SGI5045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1127[] = {SGI5046, SGI5047, SGI5048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5049, SGI0, SGI0, SGI0, SGI5050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1128[] = {SGI5051, SGI5052, SGI5053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5054, SGI0, SGI0, SGI0, SGI5055, SGI5055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1129[] = {SGI5056, SGI5057, SGI5058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5059, SGI0, SGI0, SGI0, SGI5060, SGI5060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1130[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5061, SGI0, SGI0, SGI5062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1131[] = {SGI5063, SGI5064, SGI5065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5066, SGI0, SGI0, SGI0, SGI5067, SGI5067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1132[] = {SGI5068, SGI5069, SGI5070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5071, SGI0, SGI0, SGI0, SGI5072, SGI5072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1133[] = {SGI5073, SGI5074, SGI5075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5076, SGI0, SGI0, SGI0, SGI5077, SGI5077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1134[] = {SGI5078, SGI5079, SGI5080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5081, SGI0, SGI0, SGI0, SGI5082, SGI5082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1135[] = {SGI5083, SGI5084, SGI5085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5086, SGI0, SGI0, SGI0, SGI5087, SGI5087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1136[] = {SGI5088, SGI0, SGI5089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1137[] = {SGI5090, SGI5091, SGI5092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5093, SGI0, SGI0, SGI0, SGI5094, SGI5094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1138[] = {SGI5095, SGI0, SGI5096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1139[] = {SGI5097, SGI5098, SGI5099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5100, SGI5101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1140[] = {SGI5102, SGI5103, SGI5104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5105, SGI0, SGI0, SGI0, SGI5106, SGI5107, NULL};
const Segment * const SG1141[] = {SGI5108, SGI5109, SGI5110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5111, SGI0, SGI0, SGI0, SGI5112, SGI5113, NULL};
const Segment * const SG1142[] = {SGI5114, SGI5115, SGI5116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5117, SGI0, SGI0, SGI0, SGI5118, SGI5119, NULL};
const Segment * const SG1143[] = {SGI5120, SGI5121, SGI5122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5123, SGI0, SGI0, SGI0, SGI5124, SGI5125, NULL};
const Segment * const SG1144[] = {SGI5126, SGI5127, SGI5128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5129, SGI0, SGI0, SGI0, SGI5130, SGI5131, NULL};
const Segment * const SG1145[] = {SGI5132, SGI5133, SGI5134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5135, SGI0, SGI0, SGI0, SGI5136, SGI5137, NULL};
const Segment * const SG1146[] = {SGI5138, SGI5139, SGI5140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5141, SGI0, SGI0, SGI0, SGI5142, SGI5143, NULL};
const Segment * const SG1147[] = {SGI5144, SGI5145, SGI5146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5147, SGI0, SGI0, SGI0, SGI5148, SGI5148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1148[] = {SGI5149, SGI5150, SGI5151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5152, SGI0, SGI0, SGI0, SGI5153, SGI5153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1149[] = {SGI5154, SGI5155, SGI5156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5157, SGI0, SGI0, SGI0, SGI5158, SGI5158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1150[] = {SGI5159, SGI0, SGI5160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1151[] = {SGI5161, SGI5162, SGI5163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5164, SGI0, SGI0, SGI0, SGI5165, SGI5165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1152[] = {SGI5166, SGI5167, SGI5168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5169, SGI0, SGI0, SGI0, SGI5170, SGI5170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1153[] = {SGI0, SGI5171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5172, SGI0, SGI0, SGI0, SGI5173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1154[] = {SGI5174, SGI0, SGI5175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1155[] = {SGI5176, SGI5177, SGI5178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5179, SGI5180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1156[] = {SGI5181, SGI5182, SGI5183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5184, SGI5184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1157[] = {SGI5185, SGI0, SGI5186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5187, SGI0, SGI0, SGI0, SGI5188, SGI5188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1158[] = {SGI5189, SGI0, SGI5190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5191, SGI0, SGI0, SGI0, SGI5192, SGI5192, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1159[] = {SGI5193, SGI5194, SGI5195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5196, SGI0, SGI0, SGI0, SGI5197, SGI5197, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1160[] = {SGI5198, SGI5199, SGI5200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5201, SGI0, SGI0, SGI0, SGI5202, SGI5202, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1161[] = {SGI5203, SGI0, SGI5204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5205, SGI0, SGI0, SGI0, SGI5206, SGI5206, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1162[] = {SGI5207, SGI5208, SGI5209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5210, SGI0, SGI0, SGI0, SGI5211, SGI5211, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1163[] = {SGI5212, SGI5213, SGI5214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5215, SGI0, SGI0, SGI0, SGI5216, SGI5216, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1164[] = {SGI5217, SGI5218, SGI5219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5220, SGI0, SGI0, SGI0, SGI5221, SGI5221, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1165[] = {SGI5222, SGI5223, SGI5224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5225, SGI0, SGI0, SGI0, SGI5226, SGI5226, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1166[] = {SGI5227, SGI5228, SGI5229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5230, SGI0, SGI0, SGI0, SGI5231, SGI5231, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1167[] = {SGI5232, SGI5233, SGI5234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5235, SGI0, SGI0, SGI0, SGI5236, SGI5236, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1168[] = {SGI5237, SGI5238, SGI5239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5240, SGI0, SGI0, SGI0, SGI5241, SGI5241, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1169[] = {SGI5242, SGI5243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5244, SGI0, SGI0, SGI0, SGI5245, SGI5245, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1170[] = {SGI5246, SGI5247, SGI5248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5249, SGI0, SGI0, SGI0, SGI5250, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1171[] = {SGI5251, SGI5252, SGI5253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5254, SGI0, SGI0, SGI0, SGI5255, SGI5255, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1172[] = {SGI5256, SGI5257, SGI5258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1173[] = {SGI5259, SGI5260, SGI5261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5262, SGI0, SGI0, SGI0, SGI5263, SGI5264, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1174[] = {SGI5265, SGI0, SGI5266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1175[] = {SGI5267, SGI5268, SGI5269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5270, SGI0, SGI0, SGI0, SGI5271, SGI5272, NULL};
const Segment * const SG1176[] = {SGI5273, SGI5274, SGI5275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5276, SGI0, SGI0, SGI0, SGI5277, SGI5278, NULL};
const Segment * const SG1177[] = {SGI5279, SGI5280, SGI5281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5282, SGI0, SGI0, SGI0, SGI5283, SGI5284, NULL};
const Segment * const SG1178[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1179[] = {SGI5285, SGI5286, SGI5287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5289, SGI0, SGI0, SGI0, SGI5290, SGI5290, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1180[] = {SGI5291, SGI5292, SGI5293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5294, SGI0, SGI0, SGI0, SGI5295, SGI5295, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1181[] = {SGI5296, SGI5297, SGI5298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5299, SGI0, SGI0, SGI0, SGI5300, SGI5300, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1182[] = {SGI5301, SGI5302, SGI5303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5304, SGI0, SGI0, SGI0, SGI5305, SGI5305, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1183[] = {SGI5306, SGI0, SGI5307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5308, SGI0, SGI0, SGI0, SGI5309, SGI5309, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1184[] = {SGI5310, SGI0, SGI5311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1185[] = {SGI5312, SGI0, SGI5313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1186[] = {SGI5314, SGI0, SGI5315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1187[] = {SGI5316, SGI0, SGI5317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1188[] = {SGI5318, SGI5319, SGI5320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5321, SGI0, SGI0, SGI0, SGI5322, SGI5323, NULL};
const Segment * const SG1189[] = {SGI5324, SGI5325, SGI5326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5327, SGI0, SGI0, SGI0, SGI5328, SGI5329, NULL};
const Segment * const SG1190[] = {SGI5330, SGI5331, SGI5332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5333, SGI0, SGI0, SGI0, SGI5334, SGI5335, NULL};
const Segment * const SG1191[] = {SGI5336, SGI5337, SGI5338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5339, SGI0, SGI0, SGI0, SGI5340, SGI5341, NULL};
const Segment * const SG1192[] = {SGI5342, SGI5343, SGI5344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5345, SGI0, SGI0, SGI0, SGI5346, SGI5347, NULL};
const Segment * const SG1193[] = {SGI5348, SGI5349, SGI5350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5351, SGI0, SGI0, SGI0, SGI5352, SGI5352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1194[] = {SGI5353, SGI5354, SGI5355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5356, SGI0, SGI0, SGI0, SGI5357, SGI5357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1195[] = {SGI5358, SGI5359, SGI5360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5361, SGI0, SGI0, SGI0, SGI5362, SGI5362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1196[] = {SGI5363, SGI0, SGI5364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1197[] = {SGI5365, SGI5366, SGI5367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5368, SGI0, SGI0, SGI0, SGI5369, SGI5369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1198[] = {SGI5370, SGI5371, SGI5372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5373, SGI0, SGI0, SGI0, SGI5374, SGI5374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1199[] = {SGI5375, SGI0, SGI5376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5377, SGI0, SGI0, SGI0, SGI5378, SGI5378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1200[] = {SGI0, SGI5379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5380, SGI0, SGI0, SGI0, SGI5381, SGI5382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1201[] = {SGI5383, SGI5384, SGI5385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5386, SGI0, SGI0, SGI0, SGI5387, SGI5387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1202[] = {SGI5388, SGI5389, SGI5390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5391, SGI0, SGI0, SGI0, SGI5392, SGI5392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1203[] = {SGI5393, SGI5394, SGI5395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5396, SGI0, SGI0, SGI0, SGI5397, SGI5397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1204[] = {SGI5398, SGI5399, SGI5400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5401, SGI0, SGI0, SGI0, SGI5402, SGI5402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1205[] = {SGI5403, SGI5404, SGI5405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5406, SGI0, SGI0, SGI0, SGI5407, SGI5407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1206[] = {SGI5408, SGI5409, SGI5410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5411, SGI0, SGI0, SGI0, SGI5412, SGI5412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1207[] = {SGI5413, SGI5414, SGI5415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5416, SGI0, SGI0, SGI0, SGI5417, SGI5417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1208[] = {SGI5418, SGI5419, SGI5420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5421, SGI0, SGI0, SGI0, SGI5422, SGI5422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1209[] = {SGI5423, SGI5424, SGI5425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5426, SGI0, SGI0, SGI0, SGI5427, SGI5428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1210[] = {SGI5429, SGI0, SGI5430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1211[] = {SGI5431, SGI5432, SGI5433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5434, SGI0, SGI0, SGI0, SGI5435, SGI5435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1212[] = {SGI5436, SGI5437, SGI5438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5439, SGI0, SGI0, SGI0, SGI5440, SGI5440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1213[] = {SGI5441, SGI5442, SGI5443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5444, SGI0, SGI0, SGI0, SGI5445, SGI5446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1214[] = {SGI5447, SGI5448, SGI5449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5450, SGI0, SGI0, SGI0, SGI5451, SGI5451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1215[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1216[] = {SGI5453, SGI5454, SGI5455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5456, SGI0, SGI0, SGI0, SGI5457, SGI5457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1217[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1218[] = {SGI5458, SGI5459, SGI5460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5461, SGI0, SGI0, SGI0, SGI5463, SGI5463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1219[] = {SGI5464, SGI5465, SGI5466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5467, SGI0, SGI0, SGI0, SGI5468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1220[] = {SGI5469, SGI5470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5471, SGI0, SGI0, SGI0, SGI5472, SGI5472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1221[] = {SGI5473, SGI5474, SGI5475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5476, SGI0, SGI0, SGI0, SGI5477, SGI5477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1222[] = {SGI5478, SGI5479, SGI5480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5481, SGI0, SGI0, SGI0, SGI5482, SGI5483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1223[] = {SGI5484, SGI0, SGI5485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1224[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1225[] = {SGI5487, SGI5488, SGI5489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5490, SGI0, SGI0, SGI0, SGI5491, SGI5491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1226[] = {SGI0, SGI5492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5493, SGI0, SGI0, SGI0, SGI5494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1227[] = {SGI5495, SGI5496, SGI5497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5498, SGI0, SGI0, SGI0, SGI5499, SGI5499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1228[] = {SGI5500, SGI5501, SGI5502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5503, SGI0, SGI0, SGI0, SGI5504, SGI5504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1229[] = {SGI5505, SGI5506, SGI5507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5508, SGI0, SGI0, SGI0, SGI5509, SGI5510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1230[] = {SGI5511, SGI0, SGI5512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1231[] = {SGI5513, SGI5514, SGI5515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5516, SGI5517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1232[] = {SGI5518, SGI0, SGI5519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1233[] = {SGI5520, SGI0, SGI5521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5522, SGI0, SGI0, SGI0, SGI5523, SGI5523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1234[] = {SGI5524, SGI5525, SGI5526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5527, SGI5527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1235[] = {SGI5528, SGI5529, SGI5530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5531, SGI0, SGI0, SGI0, SGI5532, SGI5532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1236[] = {SGI5533, SGI5534, SGI5535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5536, SGI0, SGI0, SGI0, SGI5537, SGI5537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1237[] = {SGI5538, SGI0, SGI5539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1238[] = {SGI5540, SGI0, SGI5541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5542, SGI0, SGI0, SGI0, SGI5543, SGI5543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1239[] = {SGI5544, SGI5545, SGI5546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5547, SGI0, SGI0, SGI0, SGI5548, SGI5548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1240[] = {SGI5549, SGI5550, SGI5551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5552, SGI0, SGI0, SGI0, SGI5553, SGI5554, NULL};
const Segment * const SG1241[] = {SGI5555, SGI5556, SGI5557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5558, SGI0, SGI0, SGI0, SGI5559, SGI5561, NULL};
const Segment * const SG1242[] = {SGI5560, SGI0, SGI5562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1243[] = {SGI5563, SGI5564, SGI5565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5566, SGI0, SGI0, SGI0, SGI5567, SGI5568, NULL};
const Segment * const SG1244[] = {SGI5569, SGI5570, SGI5571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5572, SGI0, SGI0, SGI0, SGI5573, SGI5574, NULL};
const Segment * const SG1245[] = {SGI5575, SGI5576, SGI5577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5578, SGI0, SGI0, SGI0, SGI5579, SGI5579, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1246[] = {SGI5580, SGI5581, SGI5582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5583, SGI0, SGI0, SGI0, SGI5584, SGI5584, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1247[] = {SGI5585, SGI5586, SGI5587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5588, SGI0, SGI0, SGI0, SGI5589, SGI5589, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1248[] = {SGI0, SGI5590, SGI5591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5592, SGI0, SGI0, SGI0, SGI5593, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1249[] = {SGI5594, SGI5595, SGI5596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5597, SGI0, SGI0, SGI0, SGI5598, SGI5598, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1250[] = {SGI5599, SGI5600, SGI5601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5602, SGI0, SGI0, SGI0, SGI5603, SGI5603, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1251[] = {SGI5604, SGI5605, SGI5606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5607, SGI0, SGI0, SGI0, SGI5608, SGI5608, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1252[] = {SGI5609, SGI0, SGI5610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1253[] = {SGI5611, SGI5612, SGI5613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5614, SGI0, SGI0, SGI0, SGI5615, SGI5615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1254[] = {SGI5616, SGI5617, SGI5618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5619, SGI0, SGI0, SGI0, SGI5620, SGI5620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1255[] = {SGI5621, SGI0, SGI5622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1256[] = {SGI5623, SGI0, SGI5624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5625, SGI0, SGI0, SGI0, SGI5626, SGI5626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1257[] = {SGI5627, SGI5628, SGI5629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5630, SGI5631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1258[] = {SGI5632, SGI5633, SGI5634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5635, SGI0, SGI0, SGI0, SGI5636, SGI5636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1259[] = {SGI5637, SGI0, SGI5638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1260[] = {SGI5639, SGI5640, SGI5641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5642, SGI0, SGI0, SGI0, SGI5643, SGI5643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1261[] = {SGI5644, SGI0, SGI5645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1262[] = {SGI5646, SGI5647, SGI5648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5649, SGI0, SGI0, SGI0, SGI5650, SGI5650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1263[] = {SGI5651, SGI5652, SGI5653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5654, SGI0, SGI0, SGI0, SGI5655, SGI5656, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1264[] = {SGI5657, SGI5658, SGI5659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5660, SGI0, SGI0, SGI0, SGI5661, SGI5661, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1265[] = {SGI5662, SGI0, SGI5663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1266[] = {SGI5664, SGI0, SGI5665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5666, SGI0, SGI0, SGI0, SGI5667, SGI5667, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1267[] = {SGI5668, SGI5669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5670, SGI0, SGI0, SGI0, SGI5671, SGI5671, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1268[] = {SGI5672, SGI5673, SGI5674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5675, SGI0, SGI0, SGI0, SGI5676, SGI5676, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1269[] = {SGI5677, SGI5678, SGI5679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5680, SGI0, SGI0, SGI0, SGI5681, SGI5681, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1270[] = {SGI5682, SGI5683, SGI5684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5685, SGI0, SGI0, SGI0, SGI5686, SGI5686, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1271[] = {SGI5687, SGI5688, SGI5689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5690, SGI0, SGI0, SGI0, SGI5691, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1272[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5692, SGI5693, SGI5694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1273[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5695, SGI5696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1274[] = {SGI5697, SGI5698, SGI5699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5700, SGI0, SGI0, SGI0, SGI5701, SGI5701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1275[] = {SGI5702, SGI5703, SGI5704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5705, SGI0, SGI0, SGI5706, SGI5706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1276[] = {SGI5707, SGI5708, SGI5709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5710, SGI0, SGI0, SGI0, SGI5711, SGI5711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1277[] = {SGI5712, SGI5713, SGI5714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5715, SGI0, SGI0, SGI0, SGI5716, SGI5716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1278[] = {SGI5717, SGI0, SGI5718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1279[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5719, SGI5720, SGI5721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5722, SGI0, SGI0, NULL};
const Segment * const SG1280[] = {SGI5723, SGI5724, SGI5725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5726, SGI0, SGI0, SGI0, SGI5727, SGI5727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1281[] = {SGI5728, SGI0, SGI5729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1282[] = {SGI5730, SGI5731, SGI5732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1283[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5733, SGI5734, SGI5735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5736, SGI0, SGI0, NULL};
const Segment * const SG1284[] = {SGI5737, SGI0, SGI5738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5739, SGI0, SGI0, SGI0, SGI5740, SGI5740, SGI0, NULL};
const Segment * const SG1285[] = {SGI5741, SGI5742, SGI5743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5744, SGI0, SGI0, SGI0, SGI5745, SGI5745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1286[] = {SGI5746, SGI5747, SGI5748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5749, SGI0, SGI0, SGI0, SGI5750, SGI5750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1287[] = {SGI5751, SGI5752, SGI5753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5754, SGI0, SGI0, SGI0, SGI5755, SGI5755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1288[] = {SGI5756, SGI5757, SGI5758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5759, SGI0, SGI0, SGI0, SGI5760, SGI5760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1289[] = {SGI5761, SGI5762, SGI5763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5764, SGI0, SGI0, SGI0, SGI5765, SGI5765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1290[] = {SGI5766, SGI0, SGI5767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5768, SGI0, SGI0, SGI0, SGI5769, SGI5769, SGI0, NULL};
const Segment * const SG1291[] = {SGI5770, SGI0, SGI5771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5772, SGI0, SGI0, SGI0, SGI5773, SGI5773, SGI0, NULL};
const Segment * const SG1292[] = {SGI5774, SGI5775, SGI5776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5777, SGI0, SGI0, SGI0, SGI5778, SGI5778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1293[] = {SGI5779, SGI5780, SGI5781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5782, SGI0, SGI0, SGI0, SGI5783, SGI5783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1294[] = {SGI5784, SGI0, SGI5785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5786, SGI0, SGI0, SGI0, SGI5787, SGI5787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1295[] = {SGI5788, SGI5789, SGI5790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5791, SGI0, SGI0, SGI0, SGI5792, SGI5792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1296[] = {SGI5793, SGI5794, SGI5795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5796, SGI0, SGI0, SGI0, SGI5797, SGI5797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1297[] = {SGI5798, SGI0, SGI5799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1298[] = {SGI5800, SGI5801, SGI5802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5803, SGI0, SGI0, SGI0, SGI5804, SGI5804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1299[] = {SGI5805, SGI5806, SGI5807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5808, SGI5809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1300[] = {SGI5810, SGI5811, SGI5812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5813, SGI5813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1301[] = {SGI5814, SGI5815, SGI5816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5817, SGI0, SGI0, SGI5818, SGI5818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1302[] = {SGI5819, SGI5820, SGI5821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5822, SGI0, SGI0, SGI5823, SGI5823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1303[] = {SGI5824, SGI5825, SGI5826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5827, SGI0, SGI0, SGI5828, SGI5828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1304[] = {SGI5829, SGI5830, SGI5831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5832, SGI0, SGI0, SGI5833, SGI5833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1305[] = {SGI5834, SGI0, SGI5835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5836, SGI0, SGI0, SGI5837, SGI5837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1306[] = {SGI5838, SGI0, SGI5839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5840, SGI0, SGI0, SGI5841, SGI5841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1307[] = {SGI5842, SGI0, SGI5843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5844, SGI0, SGI0, SGI5845, SGI5845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1308[] = {SGI5846, SGI0, SGI5847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5848, SGI0, SGI0, SGI5849, SGI5849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1309[] = {SGI5850, SGI0, SGI5851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5852, SGI0, SGI0, SGI5853, SGI5853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1310[] = {SGI5854, SGI0, SGI5855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5856, SGI0, SGI0, SGI5857, SGI5857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1311[] = {SGI5858, SGI5859, SGI5860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5861, SGI0, SGI0, SGI5862, SGI5862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1312[] = {SGI5863, SGI5864, SGI5865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5866, SGI0, SGI0, SGI5867, SGI5867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1313[] = {SGI5868, SGI0, SGI5869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5870, SGI0, SGI0, SGI5871, SGI5871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1314[] = {SGI5872, SGI0, SGI5873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5874, SGI0, SGI0, SGI5875, SGI5875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1315[] = {SGI5876, SGI0, SGI5877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5878, SGI0, SGI0, SGI5879, SGI5879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1316[] = {SGI5880, SGI0, SGI5881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5882, SGI0, SGI0, SGI5883, SGI5883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1317[] = {SGI5884, SGI5885, SGI5886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5887, SGI0, SGI0, SGI5888, SGI5888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1318[] = {SGI5889, SGI0, SGI5890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5891, SGI0, SGI0, SGI5892, SGI5892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1319[] = {SGI5893, SGI0, SGI5894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5895, SGI0, SGI0, SGI5896, SGI5896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1320[] = {SGI5897, SGI0, SGI5898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5899, SGI0, SGI0, SGI5900, SGI5900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1321[] = {SGI5901, SGI0, SGI5902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5903, SGI0, SGI0, SGI5904, SGI5904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1322[] = {SGI5905, SGI0, SGI5906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5907, SGI0, SGI0, SGI5908, SGI5908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1323[] = {SGI5909, SGI5910, SGI5911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5912, SGI0, SGI0, SGI0, SGI5913, SGI5913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1324[] = {SGI5914, SGI5915, SGI5916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5917, SGI0, SGI0, SGI0, SGI5918, SGI5918, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1325[] = {SGI5919, SGI5920, SGI5921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5922, SGI0, SGI0, SGI0, SGI5923, SGI5923, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1326[] = {SGI5924, SGI5925, SGI5926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5927, SGI0, SGI0, SGI0, SGI5928, SGI5928, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1327[] = {SGI5929, SGI5930, SGI5931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5932, SGI0, SGI0, SGI0, SGI5933, SGI5933, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1328[] = {SGI5934, SGI5935, SGI5936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5937, SGI0, SGI0, SGI0, SGI5938, SGI5938, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1329[] = {SGI5939, SGI5940, SGI5941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5942, SGI0, SGI0, SGI0, SGI5943, SGI5943, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1330[] = {SGI5944, SGI0, SGI5945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5946, SGI0, SGI0, SGI0, SGI5947, SGI5947, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1331[] = {SGI5948, SGI5949, SGI5950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5951, SGI0, SGI0, SGI0, SGI5952, SGI5952, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1332[] = {SGI5953, SGI5954, SGI5955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5956, SGI0, SGI0, SGI0, SGI5957, SGI5957, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1333[] = {SGI5958, SGI5959, SGI5960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5961, SGI0, SGI0, SGI0, SGI5962, SGI5962, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1334[] = {SGI5963, SGI5964, SGI5965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5966, SGI0, SGI0, SGI0, SGI5967, SGI5967, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1335[] = {SGI5968, SGI5969, SGI5970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5971, SGI5972, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1336[] = {SGI5973, SGI5974, SGI5975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5976, SGI0, SGI0, SGI0, SGI5977, SGI5977, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1337[] = {SGI5978, SGI5979, SGI5980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5981, SGI0, SGI0, SGI0, SGI5982, SGI5982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1338[] = {SGI5983, SGI5984, SGI5985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5986, SGI0, SGI0, SGI0, SGI5987, SGI5987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1339[] = {SGI5988, SGI5989, SGI5990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5991, SGI0, SGI0, SGI0, SGI5992, SGI5992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1340[] = {SGI5993, SGI5994, SGI5995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI5996, SGI0, SGI0, SGI0, SGI5997, SGI5997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1341[] = {SGI5998, SGI5999, SGI6000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6001, SGI0, SGI0, SGI0, SGI6002, SGI6003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1342[] = {SGI6004, SGI6005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6006, SGI0, SGI0, SGI0, SGI6007, SGI6007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1343[] = {SGI6008, SGI6009, SGI6010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6011, SGI0, SGI0, SGI0, SGI6012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1344[] = {SGI6013, SGI0, SGI6014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1345[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6015, SGI6016, SGI6017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6018, SGI0, SGI0, NULL};
const Segment * const SG1346[] = {SGI6019, SGI6020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6021, SGI0, SGI0, SGI0, SGI6022, SGI6023, NULL};
const Segment * const SG1347[] = {SGI6024, SGI6025, SGI6026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6027, SGI0, SGI0, SGI0, SGI6028, SGI0, NULL};
const Segment * const SG1348[] = {SGI6029, SGI6030, SGI6031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6032, SGI0, SGI0, SGI0, SGI6033, SGI6034, NULL};
const Segment * const SG1349[] = {SGI6035, SGI6036, SGI6037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6038, SGI0, SGI0, SGI0, SGI6039, SGI6040, NULL};
const Segment * const SG1350[] = {SGI6041, SGI6042, SGI6043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6044, SGI0, SGI0, SGI0, SGI6045, SGI6046, NULL};
const Segment * const SG1351[] = {SGI6047, SGI0, SGI6048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6049, SGI0, SGI0, SGI0, SGI6050, SGI6051, NULL};
const Segment * const SG1352[] = {SGI6052, SGI6053, SGI6054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6055, SGI0, SGI0, SGI0, SGI6056, SGI6057, NULL};
const Segment * const SG1353[] = {SGI6058, SGI6059, SGI6060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6061, SGI0, SGI0, SGI0, SGI6062, SGI6063, NULL};
const Segment * const SG1354[] = {SGI6064, SGI6065, SGI6066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6067, SGI0, SGI0, SGI0, SGI6068, SGI6068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1355[] = {SGI6069, SGI6070, SGI6071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6072, SGI0, SGI0, SGI0, SGI6073, SGI6073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1356[] = {SGI6074, SGI0, SGI6075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1357[] = {SGI6076, SGI6077, SGI6078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6079, SGI0, SGI0, SGI0, SGI6080, SGI6080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1358[] = {SGI6081, SGI6082, SGI6083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6084, SGI0, SGI0, SGI0, SGI6085, SGI6085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1359[] = {SGI6086, SGI6087, SGI6088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6089, SGI0, SGI0, SGI0, SGI6090, SGI6090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1360[] = {SGI6091, SGI6092, SGI6093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6094, SGI0, SGI0, SGI0, SGI6095, SGI6095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1361[] = {SGI6096, SGI6097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6098, SGI0, SGI0, SGI0, SGI6099, SGI6099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1362[] = {SGI6100, SGI0, SGI6101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1363[] = {SGI6102, SGI6103, SGI6104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6105, SGI0, SGI0, SGI0, SGI6106, SGI6107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1364[] = {SGI6108, SGI6109, SGI6110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6111, SGI0, SGI0, SGI0, SGI6112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1365[] = {SGI6113, SGI6114, SGI6115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6116, SGI0, SGI0, SGI0, SGI6117, SGI6117, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1366[] = {SGI6118, SGI0, SGI6119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1367[] = {SGI6120, SGI6121, SGI6122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6123, SGI0, SGI0, SGI0, SGI6124, SGI6124, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1368[] = {SGI6125, SGI6126, SGI6127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6128, SGI0, SGI0, SGI0, SGI6129, SGI6129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1369[] = {SGI6130, SGI6131, SGI6132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6133, SGI0, SGI0, SGI0, SGI6134, SGI6134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1370[] = {SGI6135, SGI6136, SGI6137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6138, SGI0, SGI0, SGI0, SGI6139, SGI6139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1371[] = {SGI6140, SGI6141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6142, SGI0, SGI0, SGI0, SGI6143, SGI6143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1372[] = {SGI6144, SGI0, SGI6145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6146, SGI0, SGI0, SGI0, SGI6147, SGI6147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1373[] = {SGI6148, SGI6149, SGI6150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6151, SGI0, SGI0, SGI0, SGI6152, SGI6152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1374[] = {SGI6153, SGI6154, SGI6155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6156, SGI0, SGI0, SGI0, SGI6157, SGI6157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1375[] = {SGI6158, SGI6159, SGI6160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6161, SGI0, SGI0, SGI0, SGI6162, SGI6162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1376[] = {SGI6163, SGI6164, SGI6165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6166, SGI0, SGI0, SGI0, SGI6167, SGI6167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1377[] = {SGI6168, SGI6169, SGI6170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6171, SGI0, SGI0, SGI0, SGI6172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1378[] = {SGI6173, SGI6174, SGI6175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6176, SGI0, SGI0, SGI0, SGI6177, SGI6177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1379[] = {SGI6178, SGI6179, SGI6180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6181, SGI0, SGI0, SGI0, SGI6182, SGI6183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1380[] = {SGI6184, SGI0, SGI6185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1381[] = {SGI6186, SGI6187, SGI6188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6189, SGI0, SGI0, SGI0, SGI6190, SGI6190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1382[] = {SGI6191, SGI6192, SGI6193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6194, SGI0, SGI0, SGI0, SGI6195, SGI6195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1383[] = {SGI6196, SGI6197, SGI6198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6199, SGI0, SGI0, SGI0, SGI6200, SGI6200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1384[] = {SGI6201, SGI6202, SGI6203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6204, SGI0, SGI0, SGI0, SGI6205, SGI6205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1385[] = {SGI6206, SGI6207, SGI6208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6209, SGI0, SGI0, SGI0, SGI6210, SGI6210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1386[] = {SGI6211, SGI6212, SGI6213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6214, SGI0, SGI0, SGI0, SGI6215, SGI6215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1387[] = {SGI6216, SGI6217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6218, SGI0, SGI0, SGI0, SGI6219, SGI6219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1388[] = {SGI6220, SGI6221, SGI6222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6223, SGI0, SGI0, SGI0, SGI6224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1389[] = {SGI6225, SGI6226, SGI6227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6228, SGI0, SGI0, SGI0, SGI6229, SGI6229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1390[] = {SGI6230, SGI0, SGI6231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1391[] = {SGI6232, SGI6233, SGI6234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6235, SGI0, SGI0, SGI0, SGI6236, SGI6236, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1392[] = {SGI6237, SGI6238, SGI6239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6240, SGI0, SGI0, SGI0, SGI6241, SGI6241, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1393[] = {SGI6242, SGI6243, SGI6244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6245, SGI0, SGI0, SGI0, SGI6246, SGI6246, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1394[] = {SGI6247, SGI6248, SGI6249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6250, SGI0, SGI0, SGI0, SGI6251, SGI6251, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1395[] = {SGI6252, SGI6253, SGI6254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6255, SGI0, SGI0, SGI0, SGI6256, SGI6256, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1396[] = {SGI6257, SGI6258, SGI6259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6260, SGI0, SGI0, SGI0, SGI6261, SGI6261, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1397[] = {SGI6262, SGI6263, SGI6264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6265, SGI0, SGI0, SGI0, SGI6266, SGI6266, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1398[] = {SGI6267, SGI6268, SGI6269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6270, SGI0, SGI0, SGI0, SGI6271, SGI6271, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1399[] = {SGI6272, SGI6273, SGI6274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6275, SGI0, SGI0, SGI0, SGI6276, SGI6276, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1400[] = {SGI6277, SGI6278, SGI6279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6280, SGI0, SGI0, SGI0, SGI6281, SGI6281, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1401[] = {SGI6282, SGI6283, SGI6284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6285, SGI0, SGI0, SGI0, SGI6286, SGI6286, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1402[] = {SGI6287, SGI6288, SGI6289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6290, SGI0, SGI0, SGI0, SGI6291, SGI6291, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1403[] = {SGI6292, SGI6293, SGI6294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6295, SGI0, SGI0, SGI0, SGI6296, SGI6296, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1404[] = {SGI6297, SGI6298, SGI6299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6300, SGI0, SGI0, SGI0, SGI6301, SGI6301, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1405[] = {SGI6302, SGI6303, SGI6304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6305, SGI0, SGI0, SGI0, SGI6306, SGI6306, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1406[] = {SGI6307, SGI6308, SGI6309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6310, SGI0, SGI0, SGI0, SGI6311, SGI6311, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1407[] = {SGI6312, SGI6313, SGI6314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6315, SGI0, SGI0, SGI0, SGI6316, SGI6316, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1408[] = {SGI6317, SGI6318, SGI6319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6320, SGI0, SGI0, SGI0, SGI6321, SGI6321, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1409[] = {SGI6322, SGI6323, SGI6324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6325, SGI0, SGI0, SGI0, SGI6326, SGI6326, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1410[] = {SGI6327, SGI6328, SGI6329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6330, SGI0, SGI0, SGI0, SGI6331, SGI6331, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1411[] = {SGI6332, SGI6333, SGI6334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6335, SGI0, SGI0, SGI0, SGI6336, SGI6336, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1412[] = {SGI6337, SGI6338, SGI6339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6340, SGI0, SGI0, SGI0, SGI6341, SGI6341, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1413[] = {SGI6342, SGI6343, SGI6344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6345, SGI0, SGI0, SGI0, SGI6346, SGI6346, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1414[] = {SGI6347, SGI6348, SGI6349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6350, SGI0, SGI0, SGI0, SGI6351, SGI6351, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1415[] = {SGI6352, SGI6353, SGI6354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6355, SGI0, SGI0, SGI0, SGI6356, SGI6356, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1416[] = {SGI6357, SGI6358, SGI6359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6360, SGI0, SGI0, SGI0, SGI6361, SGI6361, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1417[] = {SGI6362, SGI6363, SGI6364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6365, SGI0, SGI0, SGI0, SGI6366, SGI6366, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1418[] = {SGI6367, SGI6368, SGI6369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6370, SGI0, SGI0, SGI0, SGI6371, SGI6371, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1419[] = {SGI6372, SGI6373, SGI6374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6375, SGI0, SGI0, SGI0, SGI6376, SGI6376, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1420[] = {SGI6377, SGI6378, SGI6379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6380, SGI0, SGI0, SGI0, SGI6381, SGI6381, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1421[] = {SGI6382, SGI6383, SGI6384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6385, SGI0, SGI0, SGI0, SGI6386, SGI6387, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1422[] = {SGI6388, SGI6389, SGI6390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6391, SGI0, SGI0, SGI0, SGI6392, SGI6392, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1423[] = {SGI6393, SGI6394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6395, SGI0, SGI0, SGI0, SGI6396, SGI6397, NULL};
const Segment * const SG1424[] = {SGI6398, SGI6399, SGI6400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6401, SGI0, SGI0, SGI0, SGI6402, SGI6403, NULL};
const Segment * const SG1425[] = {SGI6404, SGI6405, SGI6406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6407, SGI0, SGI0, SGI0, SGI6408, SGI6409, NULL};
const Segment * const SG1426[] = {SGI6410, SGI6411, SGI6412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6413, SGI0, SGI0, SGI0, SGI6414, SGI6415, NULL};
const Segment * const SG1427[] = {SGI6416, SGI6417, SGI6418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6419, SGI0, SGI0, SGI0, SGI6420, SGI6421, NULL};
const Segment * const SG1428[] = {SGI6422, SGI0, SGI6423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6424, SGI0, SGI0, SGI0, SGI6425, SGI6426, NULL};
const Segment * const SG1429[] = {SGI6427, SGI6428, SGI6429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6430, SGI0, SGI0, SGI0, SGI6431, SGI6432, NULL};
const Segment * const SG1430[] = {SGI6433, SGI6434, SGI6435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6436, SGI0, SGI0, SGI0, SGI6437, SGI6438, NULL};
const Segment * const SG1431[] = {SGI6439, SGI6440, SGI6441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6442, SGI0, SGI0, SGI0, SGI6443, SGI0, NULL};
const Segment * const SG1432[] = {SGI6444, SGI0, SGI6445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6446, SGI0, SGI0, SGI0, SGI6447, SGI6447, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1433[] = {SGI6448, SGI6449, SGI6450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6451, SGI0, SGI0, SGI0, SGI6452, SGI6453, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1434[] = {SGI6454, SGI6455, SGI6456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6457, SGI0, SGI0, SGI0, SGI6458, SGI6458, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1435[] = {SGI6459, SGI6460, SGI6461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6462, SGI0, SGI0, SGI0, SGI6463, SGI6463, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1436[] = {SGI6464, SGI6465, SGI6466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6467, SGI0, SGI0, SGI0, SGI6468, SGI6469, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1437[] = {SGI6470, SGI6471, SGI6472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6473, SGI0, SGI0, SGI0, SGI6474, SGI6474, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1438[] = {SGI6475, SGI6476, SGI6477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6478, SGI0, SGI0, SGI0, SGI6479, SGI6479, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1439[] = {SGI6480, SGI6481, SGI6482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6483, SGI0, SGI0, SGI0, SGI6484, SGI6485, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1440[] = {SGI6486, SGI6487, SGI6488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6491, SGI0, SGI0, SGI0, SGI6493, SGI6493, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1441[] = {SGI6489, SGI6490, SGI6492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6494, SGI0, SGI0, SGI0, SGI6495, SGI6496, NULL};
const Segment * const SG1442[] = {SGI6497, SGI6498, SGI6499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6503, SGI6504, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1443[] = {SGI6500, SGI6501, SGI6502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6505, SGI0, SGI0, SGI0, SGI6506, SGI6507, NULL};
const Segment * const SG1444[] = {SGI6508, SGI6509, SGI6510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6513, SGI0, SGI0, SGI0, SGI6515, SGI6515, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1445[] = {SGI6511, SGI6512, SGI6514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6516, SGI0, SGI0, SGI0, SGI6517, SGI6518, NULL};
const Segment * const SG1446[] = {SGI6519, SGI0, SGI6520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6521, SGI0, SGI0, SGI0, SGI6523, SGI6523, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1447[] = {SGI6522, SGI6524, SGI6525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6526, SGI0, SGI0, SGI0, SGI6527, SGI6528, NULL};
const Segment * const SG1448[] = {SGI6529, SGI0, SGI6530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1449[] = {SGI6531, SGI6532, SGI6533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6535, SGI0, SGI0, SGI0, SGI6538, SGI6539, NULL};
const Segment * const SG1450[] = {SGI6534, SGI6536, SGI6537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6540, SGI6541, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1451[] = {SGI6542, SGI6543, SGI6544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6548, SGI0, SGI0, SGI0, SGI6549, SGI6551, NULL};
const Segment * const SG1452[] = {SGI6545, SGI6546, SGI6547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6550, SGI6552, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1453[] = {SGI6553, SGI6554, SGI6555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6556, SGI0, SGI0, SGI0, SGI6557, SGI6558, NULL};
const Segment * const SG1454[] = {SGI6559, SGI6560, SGI6561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6562, SGI0, SGI0, SGI0, SGI6563, SGI6564, NULL};
const Segment * const SG1455[] = {SGI6565, SGI6566, SGI6567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6568, SGI0, SGI0, SGI0, SGI6569, SGI6570, NULL};
const Segment * const SG1456[] = {SGI6571, SGI6572, SGI6573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6574, SGI0, SGI0, SGI0, SGI6575, SGI6575, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1457[] = {SGI0, SGI6576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6577, SGI0, SGI0, SGI0, SGI6578, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1458[] = {SGI6579, SGI6580, SGI6581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6582, SGI0, SGI0, SGI0, SGI6583, SGI6583, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1459[] = {SGI6584, SGI6585, SGI6586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6587, SGI0, SGI0, SGI0, SGI6588, SGI6589, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1460[] = {SGI6590, SGI0, SGI6591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1461[] = {SGI6592, SGI6593, SGI6594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6595, SGI6596, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1462[] = {SGI6597, SGI6598, SGI6599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6600, SGI6601, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1463[] = {SGI6602, SGI6603, SGI6604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6605, SGI0, SGI0, SGI0, SGI6606, SGI6606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1464[] = {SGI6607, SGI6608, SGI6609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6610, SGI0, SGI0, SGI0, SGI6611, SGI6611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1465[] = {SGI6612, SGI6613, SGI6614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6615, SGI0, SGI0, SGI0, SGI6616, SGI6616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1466[] = {SGI6617, SGI6618, SGI6619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6620, SGI0, SGI0, SGI0, SGI6621, SGI6621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1467[] = {SGI6622, SGI0, SGI6623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6624, SGI0, SGI0, SGI0, SGI6625, SGI6625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1468[] = {SGI6626, SGI6627, SGI6628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6629, SGI0, SGI0, SGI0, SGI6630, SGI6630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1469[] = {SGI6631, SGI0, SGI6632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1470[] = {SGI6633, SGI6634, SGI6635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6636, SGI6637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1471[] = {SGI6638, SGI0, SGI6639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6640, SGI0, SGI0, SGI0, SGI6641, SGI6641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1472[] = {SGI6642, SGI6643, SGI6644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6645, SGI0, SGI0, SGI0, SGI6646, SGI6646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1473[] = {SGI6647, SGI6648, SGI6649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6650, SGI0, SGI0, SGI0, SGI6651, SGI6651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1474[] = {SGI6652, SGI0, SGI6653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1475[] = {SGI6654, SGI6655, SGI6656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6657, SGI0, SGI0, SGI6658, SGI6658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1476[] = {SGI6659, SGI6660, SGI6661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6662, SGI0, SGI0, SGI0, SGI6663, SGI6663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1477[] = {SGI6664, SGI6665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6666, SGI0, SGI0, SGI0, SGI6667, SGI6668, NULL};
const Segment * const SG1478[] = {SGI6669, SGI6670, SGI6671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6672, SGI0, SGI0, SGI0, SGI6673, SGI6674, NULL};
const Segment * const SG1479[] = {SGI6675, SGI6676, SGI6677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6678, SGI0, SGI0, SGI0, SGI6679, SGI6680, NULL};
const Segment * const SG1480[] = {SGI6681, SGI6682, SGI6683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6684, SGI0, SGI0, SGI0, SGI6685, SGI6686, NULL};
const Segment * const SG1481[] = {SGI6687, SGI6688, SGI6689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6690, SGI0, SGI0, SGI0, SGI6691, SGI6692, NULL};
const Segment * const SG1482[] = {SGI6693, SGI6694, SGI6695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6696, SGI0, SGI0, SGI0, SGI6697, SGI6698, NULL};
const Segment * const SG1483[] = {SGI6699, SGI6700, SGI6701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6702, SGI0, SGI0, SGI0, SGI6703, SGI6704, NULL};
const Segment * const SG1484[] = {SGI6705, SGI6706, SGI6707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6708, SGI0, SGI0, SGI0, SGI6709, SGI6710, NULL};
const Segment * const SG1485[] = {SGI6711, SGI6712, SGI6713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6714, SGI0, SGI0, SGI0, SGI6715, SGI6716, NULL};
const Segment * const SG1486[] = {SGI6717, SGI6718, SGI6719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6720, SGI0, SGI0, SGI0, SGI6721, SGI6722, NULL};
const Segment * const SG1487[] = {SGI6723, SGI6724, SGI6725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6726, SGI0, SGI0, SGI0, SGI6727, SGI6727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1488[] = {SGI6728, SGI6729, SGI6730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6731, SGI0, SGI0, SGI0, SGI6732, SGI6732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1489[] = {SGI6733, SGI0, SGI6734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1490[] = {SGI6735, SGI6736, SGI6737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6738, SGI0, SGI0, SGI0, SGI6739, SGI6739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1491[] = {SGI6740, SGI6741, SGI6742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6743, SGI0, SGI0, SGI0, SGI6744, SGI6744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1492[] = {SGI6745, SGI0, SGI6746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1493[] = {SGI6747, SGI6748, SGI6749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6750, SGI0, SGI0, SGI0, SGI6751, SGI6751, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1494[] = {SGI6752, SGI6753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6754, SGI0, SGI0, SGI0, SGI6755, SGI6755, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1495[] = {SGI6756, SGI6757, SGI6758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6759, SGI0, SGI0, SGI0, SGI6760, SGI6760, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1496[] = {SGI6761, SGI6762, SGI6763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6764, SGI0, SGI0, SGI0, SGI6765, SGI6765, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1497[] = {SGI6766, SGI6767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6768, SGI0, SGI0, SGI0, SGI6769, SGI6769, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1498[] = {SGI6770, SGI6771, SGI6772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6773, SGI0, SGI0, SGI0, SGI6774, SGI6774, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1499[] = {SGI6775, SGI6776, SGI6777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6778, SGI0, SGI0, SGI0, SGI6779, SGI6779, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1500[] = {SGI6780, SGI6781, SGI6782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6783, SGI0, SGI0, SGI0, SGI6784, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1501[] = {SGI6785, SGI6786, SGI6787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6788, SGI0, SGI0, SGI0, SGI6789, SGI6789, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1502[] = {SGI6790, SGI6791, SGI6792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6793, SGI0, SGI0, SGI0, SGI6794, SGI6794, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1503[] = {SGI6795, SGI6796, SGI6797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6798, SGI0, SGI0, SGI0, SGI6799, SGI6799, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1504[] = {SGI6800, SGI6801, SGI6802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6803, SGI0, SGI0, SGI0, SGI6804, SGI6804, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1505[] = {SGI6805, SGI6806, SGI6807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6808, SGI0, SGI0, SGI0, SGI6809, SGI6809, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1506[] = {SGI6810, SGI6811, SGI6812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6813, SGI0, SGI0, SGI0, SGI6814, SGI6814, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1507[] = {SGI6815, SGI6816, SGI6817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6818, SGI0, SGI0, SGI0, SGI6819, SGI6819, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1508[] = {SGI6820, SGI6821, SGI6822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6823, SGI0, SGI0, SGI0, SGI6824, SGI6824, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1509[] = {SGI6825, SGI6826, SGI6827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6828, SGI0, SGI0, SGI0, SGI6829, SGI6829, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1510[] = {SGI6830, SGI6831, SGI6832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6833, SGI0, SGI0, SGI0, SGI6834, SGI6834, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1511[] = {SGI6835, SGI6836, SGI6837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6838, SGI0, SGI0, SGI0, SGI6839, SGI6839, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1512[] = {SGI6840, SGI6841, SGI6842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6843, SGI0, SGI0, SGI0, SGI6844, SGI6844, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1513[] = {SGI6845, SGI0, SGI6846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6847, SGI0, SGI0, SGI0, SGI6848, SGI6848, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1514[] = {SGI6849, SGI6850, SGI6851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6852, SGI0, SGI0, SGI0, SGI6853, SGI6853, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1515[] = {SGI6854, SGI6855, SGI6856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6857, SGI0, SGI0, SGI0, SGI6858, SGI6858, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1516[] = {SGI6859, SGI6860, SGI6861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6862, SGI0, SGI0, SGI0, SGI6863, SGI6863, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1517[] = {SGI6864, SGI6865, SGI6866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6867, SGI0, SGI0, SGI0, SGI6868, SGI6868, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1518[] = {SGI6869, SGI6870, SGI6871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6872, SGI0, SGI0, SGI0, SGI6873, SGI6873, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1519[] = {SGI6874, SGI6875, SGI6876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6877, SGI0, SGI0, SGI0, SGI6878, SGI6878, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1520[] = {SGI6879, SGI0, SGI6880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1521[] = {SGI6881, SGI6882, SGI6883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6884, SGI6885, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1522[] = {SGI6886, SGI6887, SGI6888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6889, SGI0, SGI0, SGI0, SGI6890, SGI6890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1523[] = {SGI6891, SGI6892, SGI6893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6894, SGI0, SGI0, SGI0, SGI6895, SGI6895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1524[] = {SGI6896, SGI6897, SGI6898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6899, SGI0, SGI0, SGI0, SGI6900, SGI6900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1525[] = {SGI6901, SGI6902, SGI6903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6904, SGI0, SGI0, SGI0, SGI6905, SGI6905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1526[] = {SGI6906, SGI6907, SGI6908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6909, SGI0, SGI0, SGI0, SGI6910, SGI6910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1527[] = {SGI6911, SGI6912, SGI6913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6914, SGI0, SGI0, SGI0, SGI6915, SGI6915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1528[] = {SGI6916, SGI6917, SGI6918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6919, SGI0, SGI0, SGI0, SGI6920, SGI6920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1529[] = {SGI6921, SGI6922, SGI6923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6924, SGI0, SGI0, SGI0, SGI6925, SGI6925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1530[] = {SGI6926, SGI6927, SGI6928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6929, SGI0, SGI0, SGI0, SGI6930, SGI6930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1531[] = {SGI6931, SGI0, SGI6932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1532[] = {SGI6933, SGI6934, SGI6935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6936, SGI0, SGI0, SGI0, SGI6937, SGI6937, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1533[] = {SGI6938, SGI0, SGI6939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1534[] = {SGI6940, SGI6941, SGI6942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6943, SGI0, SGI0, SGI0, SGI6944, SGI6944, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1535[] = {SGI6945, SGI6946, SGI6947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6948, SGI0, SGI0, SGI0, SGI6949, SGI6950, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1536[] = {SGI6951, SGI6952, SGI6953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6954, SGI0, SGI0, SGI0, SGI6955, SGI6955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1537[] = {SGI6956, SGI6957, SGI6958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6959, SGI0, SGI0, SGI0, SGI6960, SGI6960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1538[] = {SGI6961, SGI6962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6963, SGI0, SGI0, SGI0, SGI6964, SGI6964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1539[] = {SGI6965, SGI6966, SGI6967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6968, SGI0, SGI0, SGI0, SGI6969, SGI6969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1540[] = {SGI6970, SGI6971, SGI6972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6973, SGI0, SGI0, SGI0, SGI6974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1541[] = {SGI6975, SGI6976, SGI6977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6978, SGI0, SGI0, SGI0, SGI6979, SGI6979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1542[] = {SGI6980, SGI6981, SGI6982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6983, SGI0, SGI0, SGI0, SGI6984, SGI6985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1543[] = {SGI6986, SGI0, SGI6987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1544[] = {SGI6988, SGI6989, SGI6990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6991, SGI6992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1545[] = {SGI6993, SGI6994, SGI6995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI6996, SGI0, SGI0, SGI0, SGI6997, SGI6997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1546[] = {SGI6998, SGI6999, SGI7000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7001, SGI0, SGI0, SGI0, SGI7002, SGI7002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1547[] = {SGI7003, SGI7004, SGI7005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7006, SGI0, SGI0, SGI0, SGI7007, SGI7007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1548[] = {SGI7008, SGI7009, SGI7010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7011, SGI0, SGI0, SGI0, SGI7012, SGI7012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1549[] = {SGI7013, SGI7014, SGI7015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7016, SGI0, SGI0, SGI0, SGI7017, SGI7017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1550[] = {SGI7018, SGI0, SGI7019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1551[] = {SGI7020, SGI7021, SGI7022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7023, SGI0, SGI0, SGI0, SGI7024, SGI7024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1552[] = {SGI7025, SGI7026, SGI7027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7028, SGI0, SGI0, SGI0, SGI7029, SGI7030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1553[] = {SGI7031, SGI0, SGI7032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1554[] = {SGI7033, SGI7034, SGI7035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7036, SGI0, SGI0, SGI0, SGI7037, SGI7037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1555[] = {SGI7038, SGI7039, SGI7040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7041, SGI0, SGI0, SGI0, SGI7042, SGI7042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1556[] = {SGI7043, SGI0, SGI7044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1557[] = {SGI7045, SGI7046, SGI7047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7048, SGI0, SGI0, SGI0, SGI7049, SGI7049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1558[] = {SGI7050, SGI0, SGI7051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1559[] = {SGI7052, SGI7053, SGI7054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7055, SGI0, SGI0, SGI0, SGI7056, SGI7056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1560[] = {SGI7057, SGI7058, SGI7059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7060, SGI0, SGI0, SGI0, SGI7061, SGI7061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1561[] = {SGI7062, SGI7063, SGI7064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7065, SGI0, SGI0, SGI0, SGI7066, SGI7066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1562[] = {SGI7067, SGI7068, SGI7069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7070, SGI0, SGI0, SGI0, SGI7071, SGI7071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1563[] = {SGI7072, SGI7073, SGI7074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7075, SGI0, SGI0, SGI0, SGI7076, SGI7076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1564[] = {SGI7077, SGI7078, SGI7079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7080, SGI0, SGI0, SGI0, SGI7081, SGI7081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1565[] = {SGI7082, SGI7083, SGI7084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7085, SGI0, SGI0, SGI0, SGI7086, SGI7086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1566[] = {SGI7087, SGI7088, SGI7089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7090, SGI0, SGI0, SGI0, SGI7091, SGI7091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1567[] = {SGI7092, SGI7093, SGI7094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7095, SGI0, SGI0, SGI0, SGI7096, SGI7096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1568[] = {SGI7097, SGI7098, SGI7099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7100, SGI0, SGI0, SGI0, SGI7101, SGI7101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1569[] = {SGI7102, SGI7103, SGI7104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7105, SGI0, SGI0, SGI0, SGI7106, SGI7106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1570[] = {SGI7107, SGI7108, SGI7109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7110, SGI0, SGI0, SGI0, SGI7111, SGI7111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1571[] = {SGI7112, SGI7113, SGI7114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7115, SGI0, SGI0, SGI0, SGI7116, SGI7116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1572[] = {SGI7117, SGI7118, SGI7119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7120, SGI0, SGI0, SGI0, SGI7121, SGI7121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1573[] = {SGI7122, SGI7123, SGI7124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7125, SGI0, SGI0, SGI0, SGI7126, SGI7126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1574[] = {SGI7127, SGI0, SGI7128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1575[] = {SGI7129, SGI0, SGI7130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1576[] = {SGI7131, SGI7132, SGI7133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7134, SGI7135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1577[] = {SGI7136, SGI7137, SGI7138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7139, SGI7139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1578[] = {SGI7140, SGI7141, SGI7142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7143, SGI0, SGI0, SGI0, SGI7144, SGI7144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1579[] = {SGI7145, SGI7146, SGI7147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7148, SGI0, SGI0, SGI0, SGI7149, SGI7149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1580[] = {SGI7150, SGI7151, SGI7152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7153, SGI0, SGI0, SGI0, SGI7154, SGI7154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1581[] = {SGI7155, SGI7156, SGI7157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7158, SGI0, SGI0, SGI0, SGI7159, SGI7159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1582[] = {SGI7160, SGI0, SGI7161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1583[] = {SGI7162, SGI0, SGI7163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1584[] = {SGI7164, SGI7165, SGI7166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7167, SGI7168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1585[] = {SGI7169, SGI7170, SGI7171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7172, SGI7172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1586[] = {SGI7173, SGI7174, SGI7175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7176, SGI0, SGI0, SGI0, SGI7177, SGI7177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1587[] = {SGI7178, SGI7179, SGI7180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7181, SGI0, SGI0, SGI0, SGI7182, SGI7182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1588[] = {SGI7183, SGI7184, SGI7185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7186, SGI0, SGI0, SGI0, SGI7187, SGI7187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1589[] = {SGI7188, SGI7189, SGI7190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7191, SGI0, SGI0, SGI0, SGI7192, SGI7192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1590[] = {SGI7193, SGI7194, SGI7195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7196, SGI0, SGI0, SGI0, SGI7197, SGI7197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1591[] = {SGI7198, SGI7199, SGI7200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7201, SGI0, SGI0, SGI0, SGI7202, SGI7202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1592[] = {SGI7203, SGI7204, SGI7205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7206, SGI0, SGI0, SGI0, SGI7207, SGI7207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1593[] = {SGI7208, SGI7209, SGI7210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7211, SGI0, SGI0, SGI0, SGI7212, SGI7212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1594[] = {SGI7213, SGI7214, SGI7215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7216, SGI0, SGI0, SGI0, SGI7217, SGI7217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1595[] = {SGI7218, SGI7219, SGI7220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7221, SGI0, SGI0, SGI0, SGI7222, SGI7222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1596[] = {SGI7223, SGI7224, SGI7225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7226, SGI0, SGI0, SGI0, SGI7227, SGI7227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1597[] = {SGI7228, SGI7229, SGI7230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7231, SGI0, SGI0, SGI0, SGI7232, SGI7232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1598[] = {SGI7233, SGI7234, SGI7235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7236, SGI0, SGI0, SGI0, SGI7237, SGI7238, NULL};
const Segment * const SG1599[] = {SGI7239, SGI7240, SGI7241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7242, SGI0, SGI0, SGI0, SGI7243, SGI7244, NULL};
const Segment * const SG1600[] = {SGI7245, SGI7246, SGI7247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7248, SGI0, SGI0, SGI0, SGI7252, SGI7252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1601[] = {SGI7249, SGI7250, SGI7251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7253, SGI0, SGI0, SGI0, SGI7254, SGI0, NULL};
const Segment * const SG1602[] = {SGI7257, SGI0, SGI7259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7261, SGI0, SGI0, SGI0, SGI7263, SGI7264, NULL};
const Segment * const SG1603[] = {SGI7255, SGI7256, SGI7258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7260, SGI0, SGI0, SGI0, SGI7262, SGI7262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1604[] = {SGI7265, SGI7266, SGI7267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7271, SGI0, SGI0, SGI0, SGI7272, SGI7274, NULL};
const Segment * const SG1605[] = {SGI7268, SGI7269, SGI7270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7273, SGI0, SGI0, SGI0, SGI7275, SGI7275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1606[] = {SGI7276, SGI7277, SGI7278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7279, SGI0, SGI0, SGI0, SGI7282, SGI7284, NULL};
const Segment * const SG1607[] = {SGI7280, SGI7281, SGI7283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7285, SGI0, SGI0, SGI0, SGI7286, SGI7286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1608[] = {SGI7287, SGI7288, SGI7289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7290, SGI0, SGI0, SGI0, SGI7292, SGI7294, NULL};
const Segment * const SG1609[] = {SGI7291, SGI7293, SGI7295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7296, SGI0, SGI0, SGI0, SGI7297, SGI7297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1610[] = {SGI7298, SGI7299, SGI7300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7301, SGI0, SGI0, SGI0, SGI7302, SGI7303, NULL};
const Segment * const SG1611[] = {SGI7304, SGI7305, SGI7306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7307, SGI0, SGI0, SGI0, SGI7309, SGI7309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1612[] = {SGI7308, SGI7310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7311, SGI0, SGI0, SGI0, SGI7312, SGI7313, NULL};
const Segment * const SG1613[] = {SGI7314, SGI7315, SGI7316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7320, SGI0, SGI0, SGI0, SGI7321, SGI7321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1614[] = {SGI7317, SGI7318, SGI7319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7322, SGI0, SGI0, SGI0, SGI7323, SGI7324, NULL};
const Segment * const SG1615[] = {SGI7325, SGI7326, SGI7327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7328, SGI0, SGI0, SGI0, SGI7329, SGI0, NULL};
const Segment * const SG1616[] = {SGI7330, SGI7331, SGI7333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7336, SGI0, SGI0, SGI0, SGI7337, SGI7339, NULL};
const Segment * const SG1617[] = {SGI7332, SGI7334, SGI7335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7338, SGI0, SGI0, SGI0, SGI7340, SGI7340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1618[] = {SGI7341, SGI7342, SGI7343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7344, SGI0, SGI0, SGI0, SGI7345, SGI7345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1619[] = {SGI7346, SGI7347, SGI7348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7349, SGI0, SGI0, SGI0, SGI7350, SGI7350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1620[] = {SGI7351, SGI7352, SGI7353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7354, SGI0, SGI0, SGI0, SGI7355, SGI7355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1621[] = {SGI7356, SGI7357, SGI7358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7359, SGI0, SGI0, SGI0, SGI7360, SGI7360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1622[] = {SGI7361, SGI7362, SGI7363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7364, SGI0, SGI0, SGI0, SGI7365, SGI7365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1623[] = {SGI7366, SGI7367, SGI7368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7369, SGI0, SGI0, SGI0, SGI7370, SGI7370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1624[] = {SGI7371, SGI7372, SGI7373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7374, SGI0, SGI0, SGI0, SGI7375, SGI7375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1625[] = {SGI7376, SGI7377, SGI7378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7379, SGI0, SGI0, SGI0, SGI7380, SGI7380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1626[] = {SGI7381, SGI7382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7383, SGI0, SGI0, SGI0, SGI7384, SGI7384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1627[] = {SGI7385, SGI7386, SGI7387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7388, SGI0, SGI0, SGI0, SGI7389, SGI7389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1628[] = {SGI7390, SGI7391, SGI7392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7393, SGI0, SGI0, SGI0, SGI7394, SGI7394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1629[] = {SGI7395, SGI7396, SGI7397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7398, SGI0, SGI0, SGI0, SGI7399, SGI7400, NULL};
const Segment * const SG1630[] = {SGI7401, SGI7402, SGI7403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7404, SGI0, SGI0, SGI0, SGI7405, SGI7406, NULL};
const Segment * const SG1631[] = {SGI7407, SGI7408, SGI7409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7410, SGI0, SGI0, SGI0, SGI7411, SGI7412, NULL};
const Segment * const SG1632[] = {SGI7413, SGI7414, SGI7415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7416, SGI0, SGI0, SGI0, SGI7417, SGI7417, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1633[] = {SGI7418, SGI7419, SGI7420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7421, SGI0, SGI0, SGI0, SGI7422, SGI7422, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1634[] = {SGI7423, SGI7424, SGI7425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7426, SGI0, SGI0, SGI0, SGI7427, SGI7427, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1635[] = {SGI7428, SGI7429, SGI7430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7431, SGI0, SGI0, SGI0, SGI7432, SGI7432, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1636[] = {SGI7433, SGI7434, SGI7435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7436, SGI0, SGI0, SGI0, SGI7437, SGI7437, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1637[] = {SGI7438, SGI7439, SGI7440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7441, SGI0, SGI0, SGI0, SGI7442, SGI7442, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1638[] = {SGI7443, SGI7444, SGI7445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7446, SGI0, SGI0, SGI0, SGI7447, SGI7447, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1639[] = {SGI7448, SGI7449, SGI7450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7451, SGI0, SGI0, SGI0, SGI7452, SGI7452, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1640[] = {SGI7453, SGI7454, SGI7455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7456, SGI0, SGI0, SGI0, SGI7457, SGI7457, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1641[] = {SGI7458, SGI7459, SGI7460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7461, SGI0, SGI0, SGI0, SGI7462, SGI7462, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1642[] = {SGI7463, SGI7464, SGI7465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7466, SGI0, SGI0, SGI0, SGI7467, SGI7467, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1643[] = {SGI7468, SGI7469, SGI7470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7471, SGI0, SGI0, SGI0, SGI7472, SGI7472, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1644[] = {SGI0, SGI7473, SGI7474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7475, SGI0, SGI0, SGI0, SGI7479, SGI7479, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1645[] = {SGI7476, SGI7477, SGI7478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7480, SGI0, SGI0, SGI0, SGI7481, SGI7481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1646[] = {SGI7482, SGI7483, SGI7484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7485, SGI0, SGI0, SGI0, SGI7486, SGI7486, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1647[] = {SGI7487, SGI0, SGI7488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1648[] = {SGI7489, SGI7490, SGI7491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7492, SGI0, SGI0, SGI0, SGI7493, SGI7493, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1649[] = {SGI7494, SGI7495, SGI7496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7497, SGI0, SGI0, SGI0, SGI7498, SGI7498, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1650[] = {SGI7499, SGI7500, SGI7501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7502, SGI0, SGI0, SGI0, SGI7503, SGI7503, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1651[] = {SGI7504, SGI0, SGI7505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1652[] = {SGI7506, SGI7507, SGI7508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7509, SGI0, SGI0, SGI0, SGI7510, SGI7510, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1653[] = {SGI7511, SGI7512, SGI7513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7514, SGI0, SGI0, SGI0, SGI7515, SGI7515, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1654[] = {SGI7516, SGI7517, SGI7518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7519, SGI7520, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1655[] = {SGI7521, SGI7522, SGI7523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7524, SGI0, SGI0, SGI0, SGI7525, SGI7525, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1656[] = {SGI7526, SGI7527, SGI7528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7529, SGI0, SGI0, SGI0, SGI7530, SGI7530, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1657[] = {SGI7531, SGI7532, SGI7533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7534, SGI0, SGI0, SGI0, SGI7535, SGI7535, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1658[] = {SGI7536, SGI7537, SGI7538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7539, SGI0, SGI0, SGI0, SGI7540, SGI7540, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1659[] = {SGI7541, SGI7542, SGI7543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7544, SGI0, SGI0, SGI0, SGI7545, SGI7545, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1660[] = {SGI7546, SGI7547, SGI7548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7549, SGI0, SGI0, SGI0, SGI7550, SGI7550, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1661[] = {SGI7551, SGI7552, SGI7553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7554, SGI0, SGI0, SGI0, SGI7555, SGI7555, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1662[] = {SGI7556, SGI0, SGI7557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7558, SGI0, SGI0, SGI0, SGI7559, SGI7559, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1663[] = {SGI7560, SGI7561, SGI7562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7563, SGI0, SGI0, SGI0, SGI7564, SGI7564, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1664[] = {SGI7565, SGI7566, SGI7567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7568, SGI0, SGI0, SGI0, SGI7569, SGI7569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1665[] = {SGI7570, SGI7571, SGI7572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7573, SGI0, SGI0, SGI0, SGI7574, SGI7574, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1666[] = {SGI7575, SGI0, SGI7576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1667[] = {SGI7577, SGI7578, SGI7579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7582, SGI0, SGI0, SGI0, SGI7583, SGI7583, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1668[] = {SGI7580, SGI7581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7584, SGI0, SGI0, SGI0, SGI7585, SGI7585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1669[] = {SGI7586, SGI7587, SGI7589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7592, SGI0, SGI0, SGI0, SGI7594, SGI7594, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1670[] = {SGI7588, SGI7590, SGI7591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7593, SGI0, SGI0, SGI0, SGI7595, SGI7595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1671[] = {SGI7597, SGI7599, SGI7600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7602, SGI0, SGI0, SGI0, SGI7604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1672[] = {SGI7596, SGI7598, SGI7601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7603, SGI0, SGI0, SGI0, SGI7605, SGI7605, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1673[] = {SGI7606, SGI7607, SGI7609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7612, SGI0, SGI0, SGI0, SGI7614, SGI7614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1674[] = {SGI7608, SGI7610, SGI7611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7613, SGI0, SGI0, SGI0, SGI7615, SGI7615, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1675[] = {SGI7616, SGI7617, SGI7618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7619, SGI0, SGI0, SGI0, SGI7620, SGI7620, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1676[] = {SGI0, SGI7621, SGI7622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1677[] = {SGI7623, SGI7624, SGI7625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7626, SGI0, SGI0, SGI0, SGI7627, SGI7627, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1678[] = {SGI7628, SGI7629, SGI7630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7631, SGI0, SGI0, SGI0, SGI7632, SGI7632, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1679[] = {SGI7633, SGI7634, SGI7635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7636, SGI0, SGI0, SGI0, SGI7637, SGI7637, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1680[] = {SGI7638, SGI7639, SGI7640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7641, SGI7642, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1681[] = {SGI7643, SGI7644, SGI7645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7646, SGI0, SGI0, SGI0, SGI7647, SGI7647, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1682[] = {SGI7648, SGI7649, SGI7650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7651, SGI0, SGI0, SGI0, SGI7652, SGI7652, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1683[] = {SGI7653, SGI7654, SGI7655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7656, SGI0, SGI0, SGI0, SGI7657, SGI7657, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1684[] = {SGI7658, SGI7659, SGI7660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7661, SGI0, SGI0, SGI0, SGI7662, SGI7662, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1685[] = {SGI7663, SGI7664, SGI7665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7666, SGI0, SGI0, SGI0, SGI7667, SGI7667, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1686[] = {SGI7668, SGI7669, SGI7670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7671, SGI0, SGI0, SGI0, SGI7672, SGI7672, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1687[] = {SGI7673, SGI7674, SGI7675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7676, SGI0, SGI0, SGI0, SGI7677, SGI7677, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1688[] = {SGI7678, SGI7679, SGI7680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7681, SGI0, SGI0, SGI0, SGI7682, SGI7682, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1689[] = {SGI7683, SGI7684, SGI7685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7686, SGI0, SGI0, SGI0, SGI7687, SGI7687, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1690[] = {SGI7688, SGI7689, SGI7690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7691, SGI0, SGI0, SGI0, SGI7692, SGI7692, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1691[] = {SGI7693, SGI7694, SGI7695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7696, SGI0, SGI0, SGI0, SGI7697, SGI7697, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1692[] = {SGI7698, SGI7699, SGI7700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7701, SGI0, SGI0, SGI0, SGI7702, SGI7702, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1693[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7703, SGI0, SGI0, SGI0, SGI7704, SGI7704, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1694[] = {SGI7705, SGI7706, SGI7707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7708, SGI0, SGI0, SGI0, SGI7709, SGI7709, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1695[] = {SGI7710, SGI7711, SGI7712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7713, SGI0, SGI0, SGI0, SGI7714, SGI7714, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1696[] = {SGI7715, SGI7716, SGI7717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7718, SGI0, SGI0, SGI0, SGI7719, SGI7719, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1697[] = {SGI7720, SGI7721, SGI7722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7723, SGI0, SGI0, SGI0, SGI7724, SGI7724, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1698[] = {SGI7725, SGI7726, SGI7727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7728, SGI0, SGI0, SGI0, SGI7729, SGI7729, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1699[] = {SGI7730, SGI7731, SGI7732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7733, SGI0, SGI0, SGI0, SGI7734, SGI7734, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1700[] = {SGI0, SGI7735, SGI7736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7737, SGI0, SGI0, SGI0, SGI7738, SGI7738, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1701[] = {SGI7739, SGI7740, SGI7741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7742, SGI0, SGI0, SGI0, SGI7743, SGI7743, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1702[] = {SGI7744, SGI7745, SGI7746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7747, SGI0, SGI0, SGI0, SGI7748, SGI7748, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1703[] = {SGI7749, SGI7750, SGI7751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7752, SGI0, SGI0, SGI0, SGI7753, SGI7753, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1704[] = {SGI7754, SGI7755, SGI7756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7757, SGI0, SGI0, SGI0, SGI7758, SGI7758, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1705[] = {SGI7759, SGI7760, SGI7761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7762, SGI0, SGI0, SGI0, SGI7763, SGI7763, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1706[] = {SGI7764, SGI7765, SGI7766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7767, SGI0, SGI0, SGI0, SGI7768, SGI7768, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1707[] = {SGI0, SGI7769, SGI7770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1708[] = {SGI7771, SGI7772, SGI7773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7774, SGI0, SGI0, SGI0, SGI7775, SGI7775, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1709[] = {SGI7776, SGI7777, SGI7778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7779, SGI7780, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1710[] = {SGI7781, SGI7782, SGI7783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7784, SGI0, SGI0, SGI0, SGI7785, SGI7785, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1711[] = {SGI7786, SGI7787, SGI7788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7789, SGI0, SGI0, SGI0, SGI7790, SGI7790, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1712[] = {SGI7791, SGI7792, SGI7793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7794, SGI0, SGI0, SGI0, SGI7795, SGI7795, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1713[] = {SGI7796, SGI7797, SGI7798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7799, SGI0, SGI0, SGI0, SGI7800, SGI7800, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1714[] = {SGI7801, SGI7802, SGI7803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7804, SGI0, SGI0, SGI0, SGI7805, SGI7805, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1715[] = {SGI7806, SGI7807, SGI7808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7809, SGI0, SGI0, SGI0, SGI7810, SGI7810, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1716[] = {SGI0, SGI7811, SGI7812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7813, SGI0, SGI0, SGI0, SGI0, SGI7814, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1717[] = {SGI7815, SGI7816, SGI7817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7818, SGI0, SGI0, SGI0, SGI7819, SGI7819, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1718[] = {SGI7820, SGI7821, SGI7822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7823, SGI0, SGI0, SGI0, SGI7824, SGI7824, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1719[] = {SGI0, SGI7825, SGI7826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1720[] = {SGI7827, SGI7828, SGI7829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7830, SGI0, SGI0, SGI0, SGI7831, SGI7831, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1721[] = {SGI7832, SGI7833, SGI7834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7835, SGI0, SGI0, SGI0, SGI7836, SGI7836, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1722[] = {SGI7837, SGI7838, SGI7839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7840, SGI0, SGI0, SGI0, SGI7841, SGI7841, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1723[] = {SGI7842, SGI7843, SGI7844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7845, SGI0, SGI0, SGI0, SGI7846, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1724[] = {SGI7847, SGI7848, SGI7849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7850, SGI0, SGI0, SGI0, SGI7851, SGI7851, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1725[] = {SGI7852, SGI7853, SGI7854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7855, SGI0, SGI0, SGI0, SGI7856, SGI7856, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1726[] = {SGI7857, SGI7858, SGI7859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7860, SGI0, SGI0, SGI0, SGI7861, SGI7861, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1727[] = {SGI7862, SGI7863, SGI7864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7865, SGI0, SGI0, SGI0, SGI7866, SGI7866, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1728[] = {SGI0, SGI7867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7868, SGI0, SGI0, SGI0, SGI7869, SGI7869, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1729[] = {SGI7870, SGI7871, SGI7872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7873, SGI0, SGI0, SGI0, SGI7874, SGI7874, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1730[] = {SGI0, SGI7875, SGI7876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7877, SGI0, SGI0, SGI0, SGI7878, SGI7878, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1731[] = {SGI7879, SGI7880, SGI7881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7882, SGI0, SGI0, SGI0, SGI7883, SGI7883, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1732[] = {SGI7884, SGI7885, SGI7886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7887, SGI0, SGI0, SGI0, SGI7888, SGI7888, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1733[] = {SGI0, SGI7889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7890, SGI0, SGI0, SGI0, SGI7891, SGI7891, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1734[] = {SGI7892, SGI7893, SGI7894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7895, SGI0, SGI0, SGI0, SGI7896, SGI7896, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1735[] = {SGI7897, SGI7898, SGI7899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7900, SGI0, SGI0, SGI0, SGI7901, SGI7901, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1736[] = {SGI7902, SGI7903, SGI7904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7905, SGI0, SGI0, SGI0, SGI7906, SGI7906, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1737[] = {SGI7907, SGI7908, SGI7909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7910, SGI0, SGI0, SGI0, SGI7911, SGI7911, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1738[] = {SGI0, SGI7912, SGI7913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7914, SGI0, SGI0, SGI0, SGI7915, SGI7915, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1739[] = {SGI7916, SGI7917, SGI7918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7919, SGI0, SGI0, SGI0, SGI7920, SGI7920, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1740[] = {SGI7921, SGI7922, SGI7923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7924, SGI0, SGI0, SGI0, SGI7925, SGI7925, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1741[] = {SGI7926, SGI7927, SGI7928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7929, SGI0, SGI0, SGI0, SGI7930, SGI7930, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1742[] = {SGI0, SGI7931, SGI7932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7933, SGI0, SGI0, SGI0, SGI7934, SGI7934, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1743[] = {SGI7935, SGI7936, SGI7937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7938, SGI0, SGI0, SGI0, SGI7939, SGI7939, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1744[] = {SGI7940, SGI7941, SGI7942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7943, SGI7944, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1745[] = {SGI7945, SGI7946, SGI7947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7948, SGI0, SGI0, SGI0, SGI7949, SGI7949, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1746[] = {SGI7950, SGI7951, SGI7952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7953, SGI0, SGI0, SGI0, SGI7954, SGI7954, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1747[] = {SGI7955, SGI7956, SGI7957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7958, SGI0, SGI0, SGI0, SGI7959, SGI7959, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1748[] = {SGI7960, SGI7961, SGI7962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7963, SGI0, SGI0, SGI0, SGI7964, SGI7964, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1749[] = {SGI7965, SGI7966, SGI7967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7968, SGI0, SGI0, SGI0, SGI7969, SGI7969, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1750[] = {SGI7970, SGI7971, SGI7972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7973, SGI0, SGI0, SGI0, SGI7974, SGI7974, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1751[] = {SGI7975, SGI7976, SGI7977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7980, SGI0, SGI0, SGI0, SGI7981, SGI7981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1752[] = {SGI0, SGI7978, SGI7979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7982, SGI0, SGI0, SGI0, SGI7983, SGI7983, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1753[] = {SGI7984, SGI7985, SGI7986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7990, SGI0, SGI0, SGI0, SGI7991, SGI7991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1754[] = {SGI7987, SGI7988, SGI7989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI7992, SGI0, SGI0, SGI0, SGI7993, SGI7993, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1755[] = {SGI7994, SGI7995, SGI7996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8000, SGI0, SGI0, SGI0, SGI8001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1756[] = {SGI7997, SGI7998, SGI7999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8002, SGI0, SGI0, SGI0, SGI8003, SGI8003, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1757[] = {SGI8004, SGI8005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8009, SGI0, SGI0, SGI0, SGI8010, SGI8010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1758[] = {SGI8006, SGI8007, SGI8008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8011, SGI0, SGI0, SGI0, SGI8012, SGI8012, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1759[] = {SGI8013, SGI8014, SGI8016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8019, SGI0, SGI0, SGI0, SGI8020, SGI8020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1760[] = {SGI8015, SGI8017, SGI8018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8021, SGI8022, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1761[] = {SGI8023, SGI8026, SGI8028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8030, SGI0, SGI0, SGI0, SGI8032, SGI8032, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1762[] = {SGI8024, SGI8025, SGI8027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8029, SGI0, SGI0, SGI0, SGI8031, SGI8031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1763[] = {SGI8033, SGI8034, SGI8035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8036, SGI0, SGI0, SGI0, SGI8037, SGI8037, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1764[] = {SGI8038, SGI8039, SGI8040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8041, SGI0, SGI0, SGI0, SGI8042, SGI8042, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1765[] = {SGI8043, SGI8044, SGI8045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8046, SGI0, SGI0, SGI0, SGI8047, SGI8047, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1766[] = {SGI8048, SGI8049, SGI8050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8051, SGI0, SGI0, SGI0, SGI8052, SGI8052, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1767[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8053, SGI0, SGI0, SGI0, SGI8054, SGI8055, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1768[] = {SGI8056, SGI8057, SGI8058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8059, SGI0, SGI0, SGI0, SGI8060, SGI8060, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1769[] = {SGI8061, SGI8062, SGI8063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8064, SGI0, SGI0, SGI0, SGI8065, SGI8065, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1770[] = {SGI8066, SGI8067, SGI8068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8069, SGI0, SGI0, SGI0, SGI8070, SGI8070, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1771[] = {SGI0, SGI8071, SGI8072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8073, SGI0, SGI0, SGI0, SGI8074, SGI8074, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1772[] = {SGI8075, SGI8076, SGI8077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8078, SGI0, SGI0, SGI0, SGI8079, SGI8079, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1773[] = {SGI8080, SGI8081, SGI8082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8083, SGI0, SGI0, SGI0, SGI8084, SGI8084, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1774[] = {SGI8085, SGI8086, SGI8087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8088, SGI0, SGI0, SGI0, SGI8089, SGI8089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1775[] = {SGI8090, SGI8091, SGI8092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8093, SGI0, SGI0, SGI0, SGI8094, SGI8094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1776[] = {SGI8095, SGI8096, SGI8097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8098, SGI0, SGI0, SGI0, SGI8099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1777[] = {SGI8100, SGI8101, SGI8102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8103, SGI0, SGI0, SGI0, SGI8104, SGI8104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1778[] = {SGI8105, SGI8106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8107, SGI0, SGI0, SGI0, SGI8108, SGI8108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1779[] = {SGI8109, SGI8110, SGI8111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8112, SGI0, SGI0, SGI0, SGI8113, SGI8113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1780[] = {SGI8114, SGI8115, SGI8116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8117, SGI0, SGI0, SGI0, SGI8118, SGI8118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1781[] = {SGI8119, SGI8120, SGI8121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8122, SGI0, SGI0, SGI0, SGI8123, SGI8123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1782[] = {SGI8124, SGI8125, SGI8126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8127, SGI0, SGI0, SGI0, SGI8128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1783[] = {SGI8129, SGI8130, SGI8131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8132, SGI0, SGI0, SGI0, SGI8133, SGI8133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1784[] = {SGI8134, SGI8135, SGI8136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8137, SGI0, SGI0, SGI0, SGI8138, SGI8138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1785[] = {SGI8139, SGI8140, SGI8141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8142, SGI0, SGI0, SGI0, SGI8143, SGI8143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1786[] = {SGI8144, SGI8145, SGI8146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8147, SGI0, SGI0, SGI0, SGI8148, SGI8148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1787[] = {SGI8149, SGI8150, SGI8151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8152, SGI0, SGI0, SGI0, SGI8153, SGI8153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1788[] = {SGI8154, SGI8155, SGI8156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8157, SGI0, SGI0, SGI0, SGI8158, SGI8158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1789[] = {SGI8159, SGI8160, SGI8161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8162, SGI0, SGI0, SGI0, SGI8163, SGI8163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1790[] = {SGI8164, SGI8165, SGI8166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8167, SGI0, SGI0, SGI0, SGI8168, SGI8168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1791[] = {SGI8169, SGI8170, SGI8171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8172, SGI0, SGI0, SGI0, SGI8173, SGI8173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1792[] = {SGI8174, SGI8175, SGI8176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8177, SGI0, SGI0, SGI0, SGI8178, SGI8178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1793[] = {SGI8179, SGI8180, SGI8181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8182, SGI0, SGI0, SGI0, SGI8183, SGI8183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1794[] = {SGI8184, SGI8185, SGI8186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8187, SGI0, SGI0, SGI0, SGI8188, SGI8188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1795[] = {SGI8189, SGI8190, SGI8191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8192, SGI0, SGI0, SGI0, SGI8193, SGI8193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1796[] = {SGI8194, SGI8195, SGI8196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8197, SGI0, SGI0, SGI0, SGI8198, SGI8198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1797[] = {SGI8199, SGI8200, SGI8201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8202, SGI0, SGI0, SGI0, SGI8203, SGI8203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1798[] = {SGI8204, SGI0, SGI8205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1799[] = {SGI8206, SGI8207, SGI8208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8209, SGI0, SGI0, SGI0, SGI8210, SGI8210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1800[] = {SGI8211, SGI8212, SGI8213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8214, SGI0, SGI0, SGI0, SGI8215, SGI8215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1801[] = {SGI8216, SGI0, SGI8217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8218, SGI0, SGI0, SGI0, SGI8219, SGI8220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1802[] = {SGI0, SGI8221, NULL};
const Segment * const SG1803[] = {SGI0, SGI8222, NULL};
const Segment * const SG1804[] = {SGI8223, SGI8224, SGI8225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8226, SGI0, SGI0, SGI0, SGI8227, SGI8227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1805[] = {SGI8228, SGI8229, SGI8230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8231, SGI0, SGI0, SGI0, SGI8232, SGI8232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1806[] = {SGI8233, SGI8234, SGI8235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8236, SGI0, SGI0, SGI0, SGI8237, SGI8237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1807[] = {SGI8238, SGI0, SGI8239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1808[] = {SGI8240, SGI0, SGI8241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1809[] = {SGI8242, SGI8243, SGI8244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8245, SGI0, SGI0, SGI0, SGI8246, SGI8246, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1810[] = {SGI8247, SGI8248, SGI8249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8250, SGI0, SGI0, SGI0, SGI8251, SGI8251, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1811[] = {SGI8252, SGI8253, SGI8254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8255, SGI0, SGI0, SGI0, SGI8256, SGI8256, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1812[] = {SGI8257, SGI8258, SGI8259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8260, SGI0, SGI0, SGI0, SGI8261, SGI8261, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1813[] = {SGI8262, SGI8263, SGI8264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8265, SGI0, SGI0, SGI0, SGI8266, SGI8266, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1814[] = {SGI8267, SGI8268, SGI8269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8270, SGI0, SGI0, SGI0, SGI8271, SGI8271, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1815[] = {SGI8272, SGI8273, SGI8274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8275, SGI0, SGI0, SGI0, SGI8276, SGI8276, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1816[] = {SGI8277, SGI8278, SGI8279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8280, SGI0, SGI0, SGI0, SGI8281, SGI8281, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1817[] = {SGI8282, SGI8283, SGI8284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8285, SGI0, SGI0, SGI0, SGI8286, SGI8286, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1818[] = {SGI8287, SGI8288, SGI8289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8290, SGI0, SGI0, SGI0, SGI8291, SGI8291, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1819[] = {SGI8292, SGI8293, SGI8294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8295, SGI0, SGI0, SGI0, SGI8296, SGI8296, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1820[] = {SGI8297, SGI8298, SGI8299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8300, SGI0, SGI0, SGI0, SGI8301, SGI8301, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1821[] = {SGI8302, SGI8303, SGI8304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8305, SGI0, SGI0, SGI0, SGI8306, SGI8306, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1822[] = {SGI8307, SGI8308, SGI8309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8310, SGI0, SGI0, SGI0, SGI8311, SGI8311, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1823[] = {SGI8312, SGI8313, SGI8314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8315, SGI0, SGI0, SGI0, SGI8316, SGI8316, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1824[] = {SGI8317, SGI8318, SGI8319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8320, SGI0, SGI0, SGI0, SGI8321, SGI8321, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1825[] = {SGI8322, SGI8323, SGI8324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8325, SGI0, SGI0, SGI0, SGI8326, SGI8326, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1826[] = {SGI8327, SGI8328, SGI8329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8330, SGI0, SGI0, SGI0, SGI8331, SGI8331, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1827[] = {SGI8332, SGI8333, SGI8334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8335, SGI0, SGI0, SGI0, SGI8336, SGI8336, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1828[] = {SGI8337, SGI8338, SGI8339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8340, SGI0, SGI0, SGI0, SGI8341, SGI8341, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1829[] = {SGI8342, SGI0, SGI8343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1830[] = {SGI8344, SGI8345, SGI8346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8347, SGI0, SGI0, SGI0, SGI8348, SGI8348, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1831[] = {SGI8349, SGI8350, SGI8351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8352, SGI0, SGI0, SGI0, SGI8353, SGI8353, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1832[] = {SGI8354, SGI8355, SGI8356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8357, SGI0, SGI0, SGI0, SGI8358, SGI8358, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1833[] = {SGI8359, SGI8360, SGI8361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8362, SGI0, SGI0, SGI0, SGI8363, SGI8363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1834[] = {SGI8364, SGI8365, SGI8366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8367, SGI0, SGI0, SGI0, SGI8368, SGI8368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1835[] = {SGI8369, SGI8370, SGI8371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8372, SGI0, SGI0, SGI0, SGI8373, SGI8373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1836[] = {SGI8374, SGI8375, SGI8376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8377, SGI0, SGI0, SGI0, SGI8378, SGI8378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1837[] = {SGI8379, SGI8380, SGI8381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8382, SGI0, SGI0, SGI0, SGI8383, SGI8383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1838[] = {SGI8384, SGI8385, SGI8386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8387, SGI0, SGI0, SGI0, SGI8388, SGI8388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1839[] = {SGI8389, SGI8390, SGI8391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8392, SGI0, SGI0, SGI0, SGI8393, SGI8394, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1840[] = {SGI8395, SGI8396, SGI8397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8398, SGI0, SGI0, SGI0, SGI8399, SGI8399, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1841[] = {SGI8400, SGI8401, SGI8402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8403, SGI0, SGI0, SGI0, SGI8404, SGI8404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1842[] = {SGI8405, SGI8406, SGI8407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8408, SGI0, SGI0, SGI0, SGI8409, SGI8409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1843[] = {SGI8410, SGI8411, SGI8412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8413, SGI0, SGI0, SGI0, SGI8414, SGI8414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1844[] = {SGI8415, SGI8416, SGI8417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8418, SGI0, SGI0, SGI0, SGI8419, SGI8420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1845[] = {SGI8421, SGI8422, SGI8423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8424, SGI0, SGI0, SGI0, SGI8425, SGI8425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1846[] = {SGI8426, SGI8427, SGI8428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8429, SGI0, SGI0, SGI8430, SGI8430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1847[] = {SGI8431, SGI8432, SGI8433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8434, SGI0, SGI0, SGI0, SGI8435, SGI8436, NULL};
const Segment * const SG1848[] = {SGI8437, SGI8438, SGI8439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8440, SGI0, SGI0, SGI0, SGI8441, SGI8442, NULL};
const Segment * const SG1849[] = {SGI8443, SGI8444, SGI8445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8446, SGI0, SGI0, SGI0, SGI8447, SGI8448, NULL};
const Segment * const SG1850[] = {SGI8449, SGI8450, SGI8451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8452, SGI0, SGI0, SGI0, SGI8453, SGI8454, NULL};
const Segment * const SG1851[] = {SGI8455, SGI8456, SGI8457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8458, SGI0, SGI0, SGI0, SGI8459, SGI8460, NULL};
const Segment * const SG1852[] = {SGI8461, SGI8462, SGI8463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8464, SGI0, SGI0, SGI0, SGI8465, SGI8466, NULL};
const Segment * const SG1853[] = {SGI8467, SGI8468, SGI8469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8470, SGI0, SGI0, SGI0, SGI8471, SGI8472, NULL};
const Segment * const SG1854[] = {SGI8473, SGI8474, SGI8475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8476, SGI0, SGI0, SGI0, SGI8477, SGI8477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1855[] = {SGI8478, SGI8479, SGI8480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8481, SGI0, SGI0, SGI0, SGI8482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1856[] = {SGI8483, SGI8484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8485, SGI0, SGI0, SGI0, SGI8486, SGI8486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1857[] = {SGI8487, SGI8488, SGI8489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8490, SGI0, SGI0, SGI0, SGI8491, SGI8491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1858[] = {SGI8492, SGI8493, SGI8494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8495, SGI0, SGI0, SGI0, SGI8496, SGI8496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1859[] = {SGI8497, SGI8498, SGI8499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8500, SGI0, SGI0, SGI0, SGI8501, SGI8502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1860[] = {SGI8503, SGI0, SGI8504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1861[] = {SGI8505, SGI8506, SGI8507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8508, SGI0, SGI0, SGI0, SGI8509, SGI8509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1862[] = {SGI8510, SGI8511, SGI8512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8513, SGI0, SGI0, SGI0, SGI8514, SGI8514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1863[] = {SGI8515, SGI8516, SGI8517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8518, SGI0, SGI0, SGI0, SGI8519, SGI8519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1864[] = {SGI8520, SGI8521, SGI8522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8523, SGI0, SGI0, SGI0, SGI8524, SGI8524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1865[] = {SGI8525, SGI8526, SGI8527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8528, SGI0, SGI0, SGI0, SGI8529, SGI8529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1866[] = {SGI8530, SGI8531, SGI8532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8533, SGI0, SGI0, SGI0, SGI8534, SGI8534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1867[] = {SGI8535, SGI0, SGI8536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1868[] = {SGI8537, SGI8538, SGI8539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8540, SGI0, SGI0, SGI0, SGI8541, SGI8541, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1869[] = {SGI8542, SGI8543, SGI8544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8545, SGI0, SGI0, SGI0, SGI8546, SGI8546, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1870[] = {SGI8547, SGI8548, SGI8550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8553, SGI0, SGI0, SGI0, SGI8555, SGI8555, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1871[] = {SGI8549, SGI8551, SGI8552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8554, SGI0, SGI0, SGI0, SGI8556, SGI8556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1872[] = {SGI8557, SGI8558, SGI8559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8560, SGI0, SGI0, SGI0, SGI8561, SGI8561, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1873[] = {SGI8562, SGI8563, SGI8564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8565, SGI0, SGI0, SGI0, SGI8566, SGI8566, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1874[] = {SGI8567, SGI8568, SGI8569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8570, SGI0, SGI0, SGI0, SGI8571, SGI8571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1875[] = {SGI8572, SGI8573, SGI8574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8575, SGI0, SGI0, SGI0, SGI8576, SGI8576, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1876[] = {SGI8577, SGI8578, SGI8579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8580, SGI0, SGI0, SGI0, SGI8581, SGI8581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1877[] = {SGI8582, SGI8583, SGI8584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8585, SGI0, SGI0, SGI0, SGI8586, SGI8586, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1878[] = {SGI8587, SGI0, SGI8588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1879[] = {SGI8589, SGI0, SGI8590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1880[] = {SGI8591, SGI8592, SGI8593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8594, SGI0, SGI0, SGI0, SGI8597, SGI8597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1881[] = {SGI8595, SGI8596, SGI8598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8599, SGI0, SGI0, SGI0, SGI8600, SGI8600, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1882[] = {SGI8604, SGI0, SGI8605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1883[] = {SGI8601, SGI8602, SGI8603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8606, SGI0, SGI0, SGI0, SGI8607, SGI8607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1884[] = {SGI8608, SGI8609, SGI8610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8611, SGI0, SGI0, SGI0, SGI8612, SGI8612, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1885[] = {SGI8618, SGI8619, SGI8620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8621, SGI0, SGI0, SGI0, SGI8622, SGI8622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1886[] = {SGI8613, SGI8614, SGI8615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8616, SGI0, SGI0, SGI0, SGI8617, SGI8617, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1887[] = {SGI8623, SGI8624, SGI8625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8626, SGI0, SGI0, SGI0, SGI8627, SGI8627, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1888[] = {SGI8628, SGI8629, SGI8630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8631, SGI0, SGI0, SGI0, SGI8632, SGI8632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1889[] = {SGI8633, SGI8634, SGI8635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8636, SGI0, SGI0, SGI0, SGI8637, SGI8637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1890[] = {SGI8638, SGI8639, SGI8640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8641, SGI0, SGI0, SGI0, SGI8642, SGI8642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1891[] = {SGI8643, SGI8644, SGI8645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8646, SGI0, SGI0, SGI0, SGI8647, SGI8647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1892[] = {SGI8648, SGI8649, SGI8650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8651, SGI0, SGI0, SGI0, SGI8652, SGI8652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1893[] = {SGI8653, SGI0, SGI8654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1894[] = {SGI8655, SGI8656, SGI8657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8658, SGI0, SGI0, SGI0, SGI8659, SGI8659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1895[] = {SGI8660, SGI8661, SGI8662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8663, SGI0, SGI0, SGI0, SGI8664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1896[] = {SGI8665, SGI8666, SGI8667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8668, SGI0, SGI0, SGI0, SGI8669, SGI8669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1897[] = {SGI8670, SGI8671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8672, SGI0, SGI0, SGI0, SGI8673, SGI8673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1898[] = {SGI8674, SGI8675, SGI8676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8677, SGI0, SGI0, SGI0, SGI8678, SGI8678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1899[] = {SGI8679, SGI8680, SGI8681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8682, SGI0, SGI0, SGI0, SGI8683, SGI8683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1900[] = {SGI8684, SGI8685, SGI8686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8687, SGI0, SGI0, SGI0, SGI8688, SGI8688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1901[] = {SGI8689, SGI8690, SGI8691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1902[] = {SGI8692, SGI0, SGI8693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1903[] = {SGI8694, SGI8695, SGI8696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8697, SGI0, SGI0, SGI0, SGI8698, SGI8698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1904[] = {SGI8699, SGI8700, SGI8701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8702, SGI0, SGI0, SGI0, SGI8703, SGI8703, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1905[] = {SGI8704, SGI8705, SGI8706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8707, SGI0, SGI0, SGI0, SGI8708, SGI8708, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1906[] = {SGI8709, SGI8710, SGI8711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8712, SGI0, SGI0, SGI0, SGI8713, SGI8713, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1907[] = {SGI8714, SGI8715, SGI8716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8717, SGI0, SGI0, SGI0, SGI8718, SGI8718, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1908[] = {SGI8719, SGI8720, SGI8721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8722, SGI0, SGI0, SGI0, SGI8723, SGI8723, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1909[] = {SGI8724, SGI8725, SGI8726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8727, SGI0, SGI0, SGI0, SGI8728, SGI8728, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1910[] = {SGI8729, SGI8730, SGI8731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8732, SGI0, SGI0, SGI0, SGI8733, SGI8733, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1911[] = {SGI8734, SGI8735, SGI8736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8737, SGI0, SGI0, SGI0, SGI8738, SGI8738, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1912[] = {SGI8739, SGI8740, SGI8741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8742, SGI0, SGI0, SGI0, SGI8743, SGI8743, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1913[] = {SGI8744, SGI8745, SGI8746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8747, SGI0, SGI0, SGI0, SGI8748, SGI8748, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1914[] = {SGI8749, SGI8750, SGI8751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8752, SGI0, SGI0, SGI0, SGI8753, SGI8753, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1915[] = {SGI8754, SGI8755, SGI8756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8757, SGI0, SGI0, SGI0, SGI8758, SGI8758, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1916[] = {SGI8759, SGI8760, SGI8761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8762, SGI0, SGI0, SGI0, SGI8763, SGI8763, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1917[] = {SGI8764, SGI8765, SGI8766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8767, SGI0, SGI0, SGI0, SGI8768, SGI8768, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1918[] = {SGI8769, SGI0, SGI8770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1919[] = {SGI8771, SGI8772, SGI8773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8774, SGI0, SGI0, SGI0, SGI8775, SGI8775, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1920[] = {SGI8776, SGI8777, SGI8778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8779, SGI0, SGI0, SGI0, SGI8780, SGI8780, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1921[] = {SGI8781, SGI8782, SGI8783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8784, SGI0, SGI0, SGI0, SGI8785, SGI8785, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1922[] = {SGI8786, SGI8787, SGI8788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8789, SGI0, SGI0, SGI0, SGI8790, SGI8790, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1923[] = {SGI8791, SGI8792, SGI8793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8794, SGI0, SGI0, SGI0, SGI8795, SGI8795, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1924[] = {SGI8796, SGI8797, SGI8798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8799, SGI8800, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1925[] = {SGI8801, SGI8802, SGI8803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8804, SGI0, SGI0, SGI0, SGI8805, SGI8805, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1926[] = {SGI8806, SGI8807, SGI8808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8809, SGI0, SGI0, SGI0, SGI8810, SGI8811, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1927[] = {SGI8812, SGI8813, SGI8814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8815, SGI0, SGI0, SGI0, SGI8816, SGI8816, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1928[] = {SGI8817, SGI8818, SGI8819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8820, SGI0, SGI0, SGI0, SGI8821, SGI8821, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1929[] = {SGI8822, SGI8823, SGI8824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8825, SGI0, SGI0, SGI0, SGI8826, SGI8826, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1930[] = {SGI8827, SGI0, SGI8828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1931[] = {SGI8829, SGI8830, SGI8831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8832, SGI0, SGI0, SGI0, SGI8833, SGI8834, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1932[] = {SGI8835, SGI8836, SGI8837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8838, SGI0, SGI0, SGI0, SGI8839, SGI8840, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1933[] = {SGI8841, SGI8842, SGI8843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8844, SGI8845, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1934[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1935[] = {SGI8847, SGI8848, SGI8849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8850, SGI0, SGI0, SGI0, SGI8851, SGI8851, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1936[] = {SGI8852, SGI8853, SGI8854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8855, SGI0, SGI0, SGI0, SGI8856, SGI8856, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1937[] = {SGI8857, SGI8858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8859, SGI0, SGI0, SGI0, SGI8860, SGI8860, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1938[] = {SGI8861, SGI8862, SGI8863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8864, SGI0, SGI0, SGI0, SGI8865, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1939[] = {SGI8866, SGI8867, SGI8868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8869, SGI0, SGI0, SGI0, SGI8870, SGI8870, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1940[] = {SGI8871, SGI8872, SGI8873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8874, SGI0, SGI0, SGI0, SGI8875, SGI8875, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1941[] = {SGI8876, SGI8877, SGI8878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8879, SGI0, SGI0, SGI0, SGI8880, SGI8881, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1942[] = {SGI8882, SGI8883, SGI8884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8885, SGI0, SGI0, SGI0, SGI8886, SGI8886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1943[] = {SGI8887, SGI8888, SGI8889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8890, SGI0, SGI0, SGI0, SGI8891, SGI8891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1944[] = {SGI8892, SGI8893, SGI8894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8895, SGI0, SGI0, SGI0, SGI8896, SGI8896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1945[] = {SGI8897, SGI8898, SGI8899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8900, SGI0, SGI0, SGI0, SGI8901, SGI8901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1946[] = {SGI8902, SGI0, SGI8903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1947[] = {SGI8904, SGI8905, SGI8906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8907, SGI0, SGI0, SGI0, SGI8908, SGI8908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1948[] = {SGI8909, SGI0, SGI8910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1949[] = {SGI8911, SGI8912, SGI8913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8914, SGI0, SGI0, SGI0, SGI8915, SGI8916, NULL};
const Segment * const SG1950[] = {SGI8917, SGI8918, SGI8919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8920, SGI0, SGI0, SGI0, SGI8921, SGI8922, NULL};
const Segment * const SG1951[] = {SGI8923, SGI8924, SGI8925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8926, SGI0, SGI0, SGI0, SGI8927, SGI8927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1952[] = {SGI8928, SGI0, SGI8929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1953[] = {SGI8930, SGI8931, SGI8932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8933, SGI0, SGI0, SGI0, SGI8934, SGI8934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1954[] = {SGI8935, SGI8936, SGI8937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8938, SGI0, SGI0, SGI0, SGI8939, SGI8939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1955[] = {SGI8940, SGI8941, SGI8942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8943, SGI0, SGI0, SGI0, SGI8944, SGI8944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1956[] = {SGI8945, SGI0, SGI8946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1957[] = {SGI8948, SGI8950, SGI8952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8953, SGI0, SGI0, SGI0, SGI8955, SGI8955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1958[] = {SGI8947, SGI8949, SGI8951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8954, SGI0, SGI0, SGI0, SGI8956, SGI8956, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1959[] = {SGI8957, SGI8958, SGI8959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8960, SGI0, SGI0, SGI0, SGI8961, SGI8961, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1960[] = {SGI8962, SGI8963, SGI8964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8965, SGI0, SGI0, SGI0, SGI8966, SGI8966, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1961[] = {SGI8967, SGI8968, SGI8969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8970, SGI0, SGI0, SGI0, SGI8971, SGI8971, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1962[] = {SGI8972, SGI8973, SGI8974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8975, SGI0, SGI0, SGI0, SGI8976, SGI8976, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1963[] = {SGI8977, SGI8978, SGI8979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8980, SGI0, SGI0, SGI0, SGI8981, SGI8981, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1964[] = {SGI8982, SGI0, SGI8983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1965[] = {SGI8984, SGI8985, SGI8986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8987, SGI0, SGI0, SGI0, SGI8988, SGI8988, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1966[] = {SGI8989, SGI8990, SGI8991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8992, SGI0, SGI0, SGI0, SGI8993, SGI8993, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1967[] = {SGI8994, SGI0, SGI8995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1968[] = {SGI8996, SGI8997, SGI8998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI8999, SGI9000, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1969[] = {SGI9001, SGI0, SGI9002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1970[] = {SGI9003, SGI0, SGI9004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9005, SGI0, SGI0, SGI0, SGI9006, SGI9006, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1971[] = {SGI9007, SGI9008, SGI9009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9010, SGI0, SGI0, SGI0, SGI9011, SGI9011, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1972[] = {SGI9012, SGI9013, SGI9014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9015, SGI0, SGI0, SGI0, SGI9016, SGI9016, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1973[] = {SGI9017, SGI9018, SGI9019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9020, SGI0, SGI0, SGI0, SGI9021, SGI9021, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1974[] = {SGI9022, SGI9023, SGI9024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9025, SGI0, SGI0, SGI0, SGI9026, SGI9026, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1975[] = {SGI9027, SGI9028, SGI9029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9030, SGI0, SGI0, SGI0, SGI9031, SGI9031, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1976[] = {SGI9032, SGI9033, SGI9034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9035, SGI0, SGI0, SGI0, SGI9036, SGI9036, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1977[] = {SGI9037, SGI0, SGI9038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1978[] = {SGI9039, SGI9040, SGI9041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9042, SGI0, SGI0, SGI0, SGI9043, SGI9044, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1979[] = {SGI9045, SGI9046, SGI9047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9048, SGI0, SGI0, SGI0, SGI9049, SGI9049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1980[] = {SGI9050, SGI9051, SGI9052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9053, SGI0, SGI0, SGI0, SGI9054, SGI9054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1981[] = {SGI9055, SGI9056, SGI9057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9058, SGI0, SGI0, SGI0, SGI9059, SGI9059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1982[] = {SGI9060, SGI9061, SGI9062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9063, SGI0, SGI0, SGI0, SGI9064, SGI9064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1983[] = {SGI9065, SGI9066, SGI9067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9068, SGI0, SGI0, SGI0, SGI9069, SGI9069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1984[] = {SGI9070, SGI9071, SGI9072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9073, SGI0, SGI0, SGI0, SGI9074, SGI9074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1985[] = {SGI9075, SGI9076, SGI9077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9078, SGI0, SGI0, SGI0, SGI9079, SGI9079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1986[] = {SGI9080, SGI9081, SGI9082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9083, SGI0, SGI0, SGI0, SGI9084, SGI9084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1987[] = {SGI9085, SGI9086, SGI9087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9088, SGI0, SGI0, SGI0, SGI9089, SGI9089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1988[] = {SGI9090, SGI9091, SGI9092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9093, SGI0, SGI0, SGI0, SGI9094, SGI9094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1989[] = {SGI9095, SGI9096, SGI9097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9098, SGI0, SGI0, SGI0, SGI9099, SGI9099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1990[] = {SGI9100, SGI9101, SGI9102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9103, SGI0, SGI0, SGI0, SGI9104, SGI9104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1991[] = {SGI9105, SGI9106, SGI9107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9108, SGI0, SGI0, SGI0, SGI9109, SGI9109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1992[] = {SGI9110, SGI0, SGI9111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1993[] = {SGI9112, SGI9113, SGI9114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9115, SGI9116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1994[] = {SGI9117, SGI9118, SGI9119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9120, SGI0, SGI0, SGI0, SGI9121, SGI9121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1995[] = {SGI9122, SGI9123, SGI9124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9128, SGI0, SGI0, SGI0, SGI9129, SGI9129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1996[] = {SGI9125, SGI9126, SGI9127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9130, SGI0, SGI0, SGI0, SGI9131, SGI9131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1997[] = {SGI9132, SGI9133, SGI9134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9135, SGI0, SGI0, SGI0, SGI9136, SGI9136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1998[] = {SGI9137, SGI9138, SGI9139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9140, SGI0, SGI0, SGI0, SGI9141, SGI9141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG1999[] = {SGI9142, SGI0, SGI9143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2000[] = {SGI9144, SGI9145, SGI9146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9147, SGI0, SGI0, SGI0, SGI9148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2001[] = {SGI9149, SGI9150, SGI9151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9152, SGI0, SGI0, SGI0, SGI9153, SGI9153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2002[] = {SGI9154, SGI9155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9156, SGI0, SGI0, SGI0, SGI9157, SGI9157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2003[] = {SGI9158, SGI9159, SGI9160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9161, SGI0, SGI0, SGI0, SGI9162, SGI9162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2004[] = {SGI9163, SGI9164, SGI9165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9166, SGI0, SGI0, SGI0, SGI9167, SGI9167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2005[] = {SGI9168, SGI9169, SGI9170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9171, SGI0, SGI0, SGI0, SGI9172, SGI9172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2006[] = {SGI9173, SGI9174, SGI9175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9176, SGI0, SGI0, SGI0, SGI9177, SGI9177, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2007[] = {SGI9178, SGI0, SGI9179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2008[] = {SGI9180, SGI9181, SGI9182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9183, SGI0, SGI0, SGI0, SGI9184, SGI9184, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2009[] = {SGI9185, SGI0, SGI9186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2010[] = {SGI9187, SGI9188, SGI9189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9190, SGI0, SGI0, SGI0, SGI9191, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2011[] = {SGI9192, SGI9193, SGI9194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9195, SGI0, SGI0, SGI0, SGI9196, SGI9196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2012[] = {SGI9197, SGI0, SGI9198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9199, SGI0, SGI0, SGI0, SGI9200, SGI9200, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2013[] = {SGI9201, SGI9202, SGI9204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9207, SGI0, SGI0, SGI0, SGI9208, SGI9208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2014[] = {SGI9203, SGI9205, SGI9206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9209, SGI0, SGI0, SGI0, SGI9210, SGI9210, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2015[] = {SGI9211, SGI9212, SGI9213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9214, SGI0, SGI0, SGI0, SGI9215, SGI9215, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2016[] = {SGI9216, SGI9217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9218, SGI0, SGI0, SGI0, SGI9219, SGI9219, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2017[] = {SGI9220, SGI9221, SGI9222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9223, SGI0, SGI0, SGI0, SGI9224, SGI9224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2018[] = {SGI9225, SGI0, SGI9226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2019[] = {SGI9227, SGI9228, SGI9229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9230, SGI0, SGI0, SGI0, SGI9231, SGI9231, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2020[] = {SGI9232, SGI9233, SGI9234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9235, SGI0, SGI0, SGI0, SGI9236, SGI9236, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2021[] = {SGI9237, SGI9238, SGI9239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9240, SGI0, SGI0, SGI0, SGI9241, SGI9241, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2022[] = {SGI9242, SGI0, SGI9243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2023[] = {SGI9244, SGI9245, SGI9246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9247, SGI0, SGI0, SGI0, SGI9248, SGI9248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2024[] = {SGI9249, SGI9250, SGI9251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9252, SGI0, SGI0, SGI0, SGI9253, SGI9253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2025[] = {SGI9254, SGI9255, SGI9256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9257, SGI0, SGI0, SGI0, SGI9258, SGI9259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2026[] = {SGI9260, SGI0, SGI9261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2027[] = {SGI9262, SGI9263, SGI9264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9265, SGI0, SGI0, SGI0, SGI9266, SGI9266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2028[] = {SGI9267, SGI0, SGI9268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2029[] = {SGI9269, SGI9270, SGI9271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9272, SGI0, SGI0, SGI0, SGI9273, SGI9273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2030[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2031[] = {SGI9275, SGI9276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9277, SGI0, SGI0, SGI9278, SGI9278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2032[] = {SGI9279, SGI9280, SGI9281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9282, SGI0, SGI0, SGI9283, SGI9283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2033[] = {SGI9284, SGI9285, SGI9286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9287, SGI0, SGI0, SGI0, SGI9288, SGI9289, NULL};
const Segment * const SG2034[] = {SGI9290, SGI9291, SGI9292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9293, SGI0, SGI0, SGI0, SGI9294, SGI9295, NULL};
const Segment * const SG2035[] = {SGI9296, SGI9297, SGI9298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9299, SGI0, SGI0, SGI0, SGI9300, SGI9301, NULL};
const Segment * const SG2036[] = {SGI9302, SGI9303, SGI9304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9305, SGI0, SGI0, SGI0, SGI9306, SGI9307, NULL};
const Segment * const SG2037[] = {SGI9308, SGI9309, SGI9310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9311, SGI0, SGI0, SGI0, SGI9312, SGI9313, NULL};
const Segment * const SG2038[] = {SGI9314, SGI9315, SGI9316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9317, SGI0, SGI0, SGI0, SGI9318, SGI9319, NULL};
const Segment * const SG2039[] = {SGI9320, SGI9321, SGI9322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9323, SGI0, SGI0, SGI0, SGI9324, SGI9325, NULL};
const Segment * const SG2040[] = {SGI9326, SGI9327, SGI9328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9330, SGI0, SGI0, SGI0, SGI9333, SGI9334, NULL};
const Segment * const SG2041[] = {SGI9329, SGI9331, SGI9332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9335, SGI0, SGI0, SGI0, SGI9336, SGI9336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2042[] = {SGI9337, SGI0, SGI9338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9339, SGI0, SGI0, SGI0, SGI9340, SGI9341, NULL};
const Segment * const SG2043[] = {SGI9342, SGI0, SGI9343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2044[] = {SGI0, SGI9344, SGI9345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9346, SGI0, SGI0, SGI0, SGI9347, SGI9348, NULL};
const Segment * const SG2045[] = {SGI9349, SGI9350, SGI9351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9352, SGI0, SGI0, SGI0, SGI9353, SGI9354, NULL};
const Segment * const SG2046[] = {SGI9355, SGI9356, SGI9357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9358, SGI0, SGI0, SGI0, SGI9359, SGI9360, NULL};
const Segment * const SG2047[] = {SGI9361, SGI9362, SGI9363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9364, SGI0, SGI0, SGI0, SGI9365, SGI9366, NULL};
const Segment * const SG2048[] = {SGI9367, SGI9368, SGI9369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9370, SGI0, SGI0, SGI0, SGI9371, SGI9372, NULL};
const Segment * const SG2049[] = {SGI9373, SGI9374, SGI9375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9376, SGI0, SGI0, SGI0, SGI9377, SGI9378, NULL};
const Segment * const SG2050[] = {SGI9379, SGI9380, SGI9381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9382, SGI0, SGI0, SGI0, SGI9383, SGI9384, NULL};
const Segment * const SG2051[] = {SGI9385, SGI9386, SGI9387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9388, SGI0, SGI0, SGI0, SGI9389, SGI9390, NULL};
const Segment * const SG2052[] = {SGI9391, SGI9392, SGI9393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9394, SGI0, SGI0, SGI0, SGI9395, SGI9396, NULL};
const Segment * const SG2053[] = {SGI9397, SGI9398, SGI9399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9400, SGI0, SGI0, SGI0, SGI9401, SGI9402, NULL};
const Segment * const SG2054[] = {SGI9403, SGI9404, SGI9405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9406, SGI0, SGI0, SGI0, SGI9407, SGI9408, NULL};
const Segment * const SG2055[] = {SGI9409, SGI0, SGI9410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9411, SGI0, SGI0, SGI0, SGI9412, SGI9413, NULL};
const Segment * const SG2056[] = {SGI9414, SGI0, SGI9415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9416, SGI0, SGI0, SGI0, SGI9417, SGI9418, NULL};
const Segment * const SG2057[] = {SGI9419, SGI9420, SGI9421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9422, SGI0, SGI0, SGI0, SGI9423, SGI9424, NULL};
const Segment * const SG2058[] = {SGI9425, SGI9426, SGI9427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9428, SGI0, SGI0, SGI0, SGI9429, SGI9429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2059[] = {SGI9430, SGI9431, SGI9432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9433, SGI0, SGI0, SGI0, SGI9434, SGI9434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2060[] = {SGI9435, SGI9436, SGI9437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9438, SGI0, SGI0, SGI0, SGI9439, SGI9439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2061[] = {SGI9440, SGI0, SGI9441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2062[] = {SGI9442, SGI0, SGI9443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2063[] = {SGI9444, SGI9445, SGI9446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9447, SGI0, SGI0, SGI0, SGI9448, SGI9448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2064[] = {SGI9449, SGI0, SGI9450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9451, SGI0, SGI0, SGI0, SGI9452, SGI9452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2065[] = {SGI9453, SGI0, SGI9454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2066[] = {SGI9455, SGI9456, SGI9457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9458, SGI0, SGI0, SGI0, SGI9459, SGI9459, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2067[] = {SGI9460, SGI0, SGI9461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2068[] = {SGI9462, SGI0, SGI9463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9464, SGI0, SGI0, SGI0, SGI9465, SGI9465, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2069[] = {SGI9466, SGI9467, SGI9468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9469, SGI0, SGI0, SGI0, SGI9470, SGI0, NULL};
const Segment * const SG2070[] = {SGI9471, SGI9472, SGI9473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9474, SGI0, SGI0, SGI0, SGI9475, SGI9476, NULL};
const Segment * const SG2071[] = {SGI9477, SGI9478, SGI9479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9480, SGI0, SGI0, SGI0, SGI9481, SGI9482, NULL};
const Segment * const SG2072[] = {SGI9483, SGI9484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9485, SGI0, SGI0, SGI0, SGI9486, SGI9487, NULL};
const Segment * const SG2073[] = {SGI9488, SGI9489, SGI9490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9491, SGI0, SGI0, SGI0, SGI9492, SGI9493, NULL};
const Segment * const SG2074[] = {SGI9494, SGI9495, SGI9496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9497, SGI0, SGI0, SGI0, SGI9498, SGI9499, NULL};
const Segment * const SG2075[] = {SGI9500, SGI9501, SGI9502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9503, SGI0, SGI0, SGI0, SGI9504, SGI9505, NULL};
const Segment * const SG2076[] = {SGI9506, SGI9507, SGI9508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9509, SGI0, SGI0, SGI0, SGI9510, SGI9510, SGI0, NULL};
const Segment * const SG2077[] = {SGI9511, SGI9512, SGI9513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9514, SGI0, SGI0, SGI0, SGI9515, SGI9515, SGI0, NULL};
const Segment * const SG2078[] = {SGI9516, SGI9517, SGI9518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9519, SGI0, SGI0, SGI0, SGI9520, SGI9520, SGI0, NULL};
const Segment * const SG2079[] = {SGI9521, SGI9522, SGI9523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9524, SGI0, SGI0, SGI0, SGI9525, SGI9526, SGI0, NULL};
const Segment * const SG2080[] = {SGI9527, SGI9528, SGI9529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9530, SGI0, SGI0, SGI0, SGI9531, SGI0, SGI0, NULL};
const Segment * const SG2081[] = {SGI9532, SGI9533, SGI9534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9535, SGI0, SGI0, SGI0, SGI9536, SGI9536, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2082[] = {SGI9537, SGI9538, SGI9539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9540, SGI0, SGI0, SGI0, SGI9541, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2083[] = {SGI9542, SGI9543, SGI9544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9545, SGI0, SGI0, SGI0, SGI9546, SGI9546, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2084[] = {SGI9547, SGI0, SGI9548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2085[] = {SGI9549, SGI9550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9551, SGI0, SGI0, SGI0, SGI9552, SGI9552, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2086[] = {SGI9553, SGI9554, SGI9555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9556, SGI0, SGI0, SGI0, SGI9557, SGI9557, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2087[] = {SGI9558, SGI9559, SGI9560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9561, SGI0, SGI0, SGI0, SGI9562, SGI9562, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2088[] = {SGI9563, SGI0, SGI9564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9565, SGI0, SGI0, SGI0, SGI9566, SGI9566, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2089[] = {SGI9567, SGI9568, SGI9569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9570, SGI0, SGI0, SGI0, SGI9571, SGI9571, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2090[] = {SGI9572, SGI9573, SGI9574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9575, SGI0, SGI0, SGI0, SGI9576, SGI9576, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2091[] = {SGI9577, SGI9578, SGI9579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9580, SGI0, SGI0, SGI0, SGI9581, SGI9581, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2092[] = {SGI9582, SGI9583, SGI9584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9585, SGI0, SGI0, SGI0, SGI9586, SGI9587, NULL};
const Segment * const SG2093[] = {SGI9588, SGI9589, SGI9590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9591, SGI0, SGI0, SGI0, SGI9592, SGI9593, NULL};
const Segment * const SG2094[] = {SGI9594, SGI9595, SGI9596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9597, SGI0, SGI0, SGI0, SGI9598, SGI9599, NULL};
const Segment * const SG2095[] = {SGI9600, SGI9601, SGI9602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9603, SGI0, SGI0, SGI0, SGI9604, SGI9605, NULL};
const Segment * const SG2096[] = {SGI9606, SGI9607, SGI9608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9609, SGI0, SGI0, SGI0, SGI9610, SGI9611, NULL};
const Segment * const SG2097[] = {SGI9612, SGI9613, SGI9614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9615, SGI0, SGI0, SGI0, SGI9616, SGI9617, NULL};
const Segment * const SG2098[] = {SGI9618, SGI9619, SGI9620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9621, SGI0, SGI0, SGI0, SGI9622, SGI9623, NULL};
const Segment * const SG2099[] = {SGI9624, SGI9625, SGI9626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9627, SGI0, SGI0, SGI0, SGI9628, SGI9629, NULL};
const Segment * const SG2100[] = {SGI9630, SGI9631, SGI9632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9633, SGI0, SGI0, SGI0, SGI9634, SGI9635, NULL};
const Segment * const SG2101[] = {SGI9636, SGI9637, SGI9638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9639, SGI0, SGI0, SGI0, SGI9640, SGI9641, NULL};
const Segment * const SG2102[] = {SGI9642, SGI9643, SGI9644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9645, SGI0, SGI0, SGI0, SGI9646, SGI9647, NULL};
const Segment * const SG2103[] = {SGI9648, SGI9649, SGI9650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9651, SGI0, SGI0, SGI0, SGI9652, SGI9653, NULL};
const Segment * const SG2104[] = {SGI9654, SGI9655, SGI9656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9657, SGI0, SGI0, SGI0, SGI9658, SGI9658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2105[] = {SGI9659, SGI9660, SGI9661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9664, SGI0, SGI0, SGI0, SGI9666, SGI9666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2106[] = {SGI9662, SGI9663, SGI9665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9667, SGI0, SGI0, SGI0, SGI9668, SGI9668, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2107[] = {SGI9669, SGI9670, SGI9671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9672, SGI0, SGI0, SGI0, SGI9673, SGI9673, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2108[] = {SGI9674, SGI9675, SGI9676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9677, SGI0, SGI0, SGI0, SGI9678, SGI9678, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2109[] = {SGI9679, SGI9680, SGI9681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9682, SGI0, SGI0, SGI0, SGI9683, SGI9683, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2110[] = {SGI9684, SGI0, SGI9685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2111[] = {SGI9686, SGI9687, SGI9688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9689, SGI0, SGI0, SGI0, SGI9690, SGI9690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2112[] = {SGI9691, SGI9692, SGI9693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9694, SGI0, SGI0, SGI0, SGI9695, SGI9695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2113[] = {SGI9696, SGI0, SGI9697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2114[] = {SGI9698, SGI9699, SGI9700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9701, SGI0, SGI0, SGI0, SGI9702, SGI9702, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2115[] = {SGI9703, SGI0, SGI9704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9705, SGI0, SGI0, SGI0, SGI9706, SGI9706, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2116[] = {SGI9707, SGI0, SGI9708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2117[] = {SGI9709, SGI9710, SGI9711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9712, SGI0, SGI0, SGI0, SGI9713, SGI9713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2118[] = {SGI9714, SGI9715, SGI9716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9717, SGI0, SGI0, SGI0, SGI9718, SGI9718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2119[] = {SGI0, SGI9719, SGI9720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9721, SGI0, SGI0, SGI9722, SGI9723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2120[] = {SGI9724, SGI9725, SGI9726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9727, SGI0, SGI0, SGI0, SGI9728, SGI9729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2121[] = {SGI9730, SGI9731, SGI9732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9733, SGI0, SGI0, SGI0, SGI9734, SGI9734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2122[] = {SGI9735, SGI0, SGI9736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2123[] = {SGI9737, SGI9738, SGI9739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9740, SGI0, SGI0, SGI0, SGI9741, SGI9741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2124[] = {SGI9742, SGI9743, SGI9744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9745, SGI0, SGI0, SGI0, SGI9746, SGI9746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2125[] = {SGI9747, SGI9748, SGI9749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9750, SGI0, SGI0, SGI0, SGI9751, SGI9751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2126[] = {SGI9752, SGI9753, SGI9754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9755, SGI0, SGI0, SGI0, SGI9756, SGI9756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2127[] = {SGI9757, SGI9758, SGI9759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9760, SGI0, SGI0, SGI0, SGI9761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2128[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9762, SGI9763, SGI9764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2129[] = {SGI9765, SGI9766, SGI9767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9768, SGI0, SGI0, SGI0, SGI9769, SGI9769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2130[] = {SGI9770, SGI9771, SGI9772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9773, SGI0, SGI0, SGI0, SGI9774, SGI9774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2131[] = {SGI9775, SGI9776, SGI9777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9778, SGI0, SGI0, SGI0, SGI9779, SGI9779, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2132[] = {SGI9780, SGI0, SGI9781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2133[] = {SGI9782, SGI9783, SGI9784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9785, SGI0, SGI0, SGI0, SGI9786, SGI9786, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2134[] = {SGI9787, SGI9788, SGI9789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9790, SGI0, SGI0, SGI0, SGI9791, SGI9791, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2135[] = {SGI9792, SGI9793, SGI9794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9795, SGI0, SGI0, SGI0, SGI9796, SGI9796, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2136[] = {SGI9797, SGI9798, SGI9799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9800, SGI0, SGI0, SGI0, SGI9801, SGI9801, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2137[] = {SGI9802, SGI9803, SGI9804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9805, SGI0, SGI0, SGI0, SGI9806, SGI9806, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2138[] = {SGI9807, SGI9808, SGI9809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9810, SGI0, SGI0, SGI0, SGI9811, SGI9811, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2139[] = {SGI9812, SGI9813, SGI9814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9815, SGI0, SGI0, SGI0, SGI9816, SGI9816, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2140[] = {SGI9817, SGI9818, SGI9819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9820, SGI0, SGI0, SGI0, SGI9821, SGI9821, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2141[] = {SGI9822, SGI9823, SGI9824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9825, SGI0, SGI0, SGI0, SGI9826, SGI9826, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2142[] = {SGI9827, SGI9828, SGI9829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9830, SGI0, SGI0, SGI0, SGI9831, SGI9831, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2143[] = {SGI9832, SGI9833, SGI9834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9835, SGI0, SGI0, SGI0, SGI9836, SGI9836, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2144[] = {SGI9837, SGI9838, SGI9839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9840, SGI0, SGI0, SGI0, SGI9841, SGI9841, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2145[] = {SGI9842, SGI9843, SGI9844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9845, SGI0, SGI0, SGI0, SGI9846, SGI9846, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2146[] = {SGI9847, SGI9848, SGI9849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9850, SGI0, SGI0, SGI0, SGI9851, SGI9851, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2147[] = {SGI9852, SGI9853, SGI9854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9855, SGI0, SGI0, SGI0, SGI9856, SGI9856, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2148[] = {SGI9857, SGI0, SGI9858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2149[] = {SGI9859, SGI9860, SGI9861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9862, SGI0, SGI0, SGI0, SGI9863, SGI9863, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2150[] = {SGI9864, SGI9865, SGI9866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9867, SGI9868, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2151[] = {SGI9869, SGI9870, SGI9871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9872, SGI0, SGI0, SGI0, SGI9873, SGI9873, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2152[] = {SGI9874, SGI0, SGI9875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2153[] = {SGI9876, SGI9877, SGI9878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9879, SGI9880, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2154[] = {SGI9881, SGI0, SGI9882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9883, SGI0, SGI0, SGI0, SGI9884, SGI9884, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2155[] = {SGI9885, SGI9886, SGI9887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9888, SGI0, SGI0, SGI0, SGI9889, SGI9889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2156[] = {SGI9890, SGI0, SGI9891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2157[] = {SGI9892, SGI9893, SGI9894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9895, SGI0, SGI0, SGI0, SGI9896, SGI9896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2158[] = {SGI9897, SGI0, SGI9898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2159[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9899, SGI9900, SGI9901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2160[] = {SGI9902, SGI9903, SGI9904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9905, SGI0, SGI0, SGI0, SGI9906, SGI9906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2161[] = {SGI9907, SGI9908, SGI9909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9910, SGI0, SGI0, SGI0, SGI9911, SGI9911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2162[] = {SGI9912, SGI9913, SGI9914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9915, SGI0, SGI0, SGI0, SGI9916, SGI9916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2163[] = {SGI9917, SGI0, SGI9918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2164[] = {SGI9919, SGI9920, SGI9921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9922, SGI0, SGI0, SGI0, SGI9923, SGI9923, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2165[] = {SGI9924, SGI9925, SGI9926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9927, SGI0, SGI0, SGI0, SGI9928, SGI9928, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2166[] = {SGI9929, SGI9930, SGI9931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9932, SGI0, SGI0, SGI0, SGI9933, SGI9933, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2167[] = {SGI9934, SGI9935, SGI9936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9937, SGI0, SGI0, SGI0, SGI9938, SGI9938, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2168[] = {SGI9939, SGI0, SGI9940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2169[] = {SGI9941, SGI9942, SGI9943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9944, SGI0, SGI0, SGI0, SGI9945, SGI9945, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2170[] = {SGI9946, SGI9947, SGI9948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9949, SGI0, SGI0, SGI0, SGI9950, SGI9951, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2171[] = {SGI9952, SGI0, SGI9953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2172[] = {SGI9954, SGI9955, SGI9956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9957, SGI0, SGI0, SGI0, SGI9958, SGI9958, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2173[] = {SGI9959, SGI9960, SGI9961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9962, SGI0, SGI0, SGI0, SGI9963, SGI9963, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2174[] = {SGI9964, SGI9965, SGI9966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9967, SGI0, SGI0, SGI0, SGI9968, SGI9968, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2175[] = {SGI9969, SGI9970, SGI9971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9972, SGI0, SGI0, SGI0, SGI9973, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2176[] = {SGI9974, SGI9975, SGI9976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9977, SGI0, SGI0, SGI0, SGI9978, SGI9979, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2177[] = {SGI9980, SGI9981, SGI9982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9983, SGI0, SGI0, SGI0, SGI9984, SGI9985, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2178[] = {SGI9986, SGI9987, SGI9988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9989, SGI0, SGI0, SGI0, SGI9990, SGI9990, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2179[] = {SGI9991, SGI9992, SGI9993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9994, SGI0, SGI0, SGI0, SGI9995, SGI9995, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2180[] = {SGI9996, SGI9997, SGI9998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI9999, SGI0, SGI0, SGI0, SGI10000, SGI10000, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2181[] = {SGI10001, SGI10002, SGI10003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10004, SGI0, SGI0, SGI0, SGI10005, SGI10005, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2182[] = {SGI10006, SGI10007, SGI10008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10009, SGI0, SGI0, SGI0, SGI10010, SGI10010, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2183[] = {SGI10011, SGI0, SGI10012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10013, SGI0, SGI0, SGI0, SGI10014, SGI10014, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2184[] = {SGI10015, SGI10016, SGI10017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10018, SGI0, SGI0, SGI0, SGI10019, SGI10019, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2185[] = {SGI10020, SGI10021, SGI10022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10023, SGI0, SGI0, SGI0, SGI10024, SGI10025, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2186[] = {SGI10026, SGI10027, SGI10028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10029, SGI0, SGI0, SGI0, SGI10030, SGI10030, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2187[] = {SGI10031, SGI0, SGI10032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2188[] = {SGI10033, SGI0, SGI10034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2189[] = {SGI10035, SGI10036, SGI10037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10038, SGI0, SGI0, SGI0, SGI10039, SGI10039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2190[] = {SGI10040, SGI10041, SGI10042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10043, SGI0, SGI0, SGI0, SGI10044, SGI10044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2191[] = {SGI10045, SGI0, SGI10046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10047, SGI0, SGI0, SGI0, SGI10048, SGI10048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2192[] = {SGI10049, SGI0, SGI10050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2193[] = {SGI10051, SGI10052, SGI10053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10054, SGI0, SGI0, SGI0, SGI10055, SGI10055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2194[] = {SGI10056, SGI0, SGI10057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2195[] = {SGI10058, SGI10059, SGI10060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10061, SGI0, SGI0, SGI0, SGI10062, SGI10062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2196[] = {SGI10063, SGI10064, SGI10065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10066, SGI0, SGI0, SGI0, SGI10067, SGI10067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2197[] = {SGI10068, SGI10069, SGI10070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10071, SGI0, SGI0, SGI0, SGI10072, SGI10072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2198[] = {SGI10073, SGI0, SGI10074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2199[] = {SGI10075, SGI10076, SGI10077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10078, SGI0, SGI0, SGI0, SGI10079, SGI10079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2200[] = {SGI10080, SGI10081, SGI10082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10083, SGI10083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2201[] = {SGI10084, SGI10085, SGI10086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10087, SGI0, SGI0, SGI0, SGI10088, SGI10088, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2202[] = {SGI10089, SGI10090, SGI10091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10092, SGI0, SGI0, SGI0, SGI10093, SGI10093, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2203[] = {SGI10094, SGI10095, SGI10096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10097, SGI0, SGI0, SGI0, SGI10098, SGI10098, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2204[] = {SGI10099, SGI10100, SGI10101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10102, SGI0, SGI0, SGI0, SGI10103, SGI10103, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2205[] = {SGI10104, SGI10105, SGI10106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10107, SGI0, SGI0, SGI0, SGI10108, SGI10108, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2206[] = {SGI10109, SGI10110, SGI10111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10112, SGI0, SGI0, SGI0, SGI10113, SGI10113, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2207[] = {SGI10114, SGI10115, SGI10116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10117, SGI0, SGI0, SGI0, SGI10118, SGI10118, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2208[] = {SGI10119, SGI10120, SGI10121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10122, SGI0, SGI0, SGI0, SGI10123, SGI10123, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2209[] = {SGI10124, SGI10125, SGI10126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10127, SGI0, SGI0, SGI0, SGI10128, SGI10128, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2210[] = {SGI10129, SGI10130, SGI10131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10132, SGI0, SGI0, SGI0, SGI10133, SGI10133, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2211[] = {SGI10134, SGI10135, SGI10136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10137, SGI0, SGI0, SGI0, SGI10138, SGI10138, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2212[] = {SGI10139, SGI10140, SGI10141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10142, SGI0, SGI0, SGI0, SGI10143, SGI10143, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2213[] = {SGI10144, SGI10145, SGI10146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10147, SGI0, SGI0, SGI0, SGI10148, SGI10148, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2214[] = {SGI10149, SGI10150, SGI10151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10152, SGI0, SGI0, SGI0, SGI10153, SGI10153, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2215[] = {SGI10154, SGI10155, SGI10156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10157, SGI0, SGI0, SGI0, SGI10158, SGI10158, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2216[] = {SGI10159, SGI10160, SGI10161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10162, SGI0, SGI0, SGI0, SGI10163, SGI10163, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2217[] = {SGI10164, SGI10165, SGI10166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10167, SGI0, SGI0, SGI0, SGI10168, SGI10168, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2218[] = {SGI10169, SGI10170, SGI10171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10172, SGI0, SGI0, SGI0, SGI10173, SGI10173, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2219[] = {SGI10174, SGI10175, SGI10176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10177, SGI0, SGI0, SGI0, SGI10178, SGI10178, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2220[] = {SGI10179, SGI0, SGI10180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2221[] = {SGI10181, SGI0, SGI10182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2222[] = {SGI10183, SGI10184, SGI10185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10186, SGI10187, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2223[] = {SGI10188, SGI10189, SGI10190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10191, SGI0, SGI0, SGI0, SGI10192, SGI10192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2224[] = {SGI10193, SGI10194, SGI10195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10196, SGI0, SGI0, SGI0, SGI10197, SGI10197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2225[] = {SGI10198, SGI10199, SGI10200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10201, SGI0, SGI0, SGI0, SGI10202, SGI10202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2226[] = {SGI10203, SGI10204, SGI10205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10206, SGI0, SGI0, SGI0, SGI10207, SGI10207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2227[] = {SGI10208, SGI10209, SGI10210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10211, SGI0, SGI0, SGI0, SGI10212, SGI10212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2228[] = {SGI10213, SGI10214, SGI10215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10216, SGI0, SGI0, SGI0, SGI10217, SGI10217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2229[] = {SGI10218, SGI10219, SGI10220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10221, SGI0, SGI0, SGI0, SGI10222, SGI10222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2230[] = {SGI10223, SGI0, SGI10224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2231[] = {SGI10225, SGI10226, SGI10227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10228, SGI0, SGI0, SGI0, SGI10229, SGI10230, NULL};
const Segment * const SG2232[] = {SGI10231, SGI10232, SGI10233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10234, SGI0, SGI0, SGI0, SGI10235, SGI10236, NULL};
const Segment * const SG2233[] = {SGI10237, SGI10238, SGI10239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10240, SGI0, SGI0, SGI0, SGI10241, SGI10241, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2234[] = {SGI10242, SGI10243, SGI10244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10245, SGI0, SGI0, SGI0, SGI10246, SGI10246, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2235[] = {SGI10247, SGI10248, SGI10249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10250, SGI0, SGI0, SGI0, SGI10251, SGI10251, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2236[] = {SGI10252, SGI10253, SGI10254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10255, SGI0, SGI0, SGI0, SGI10256, SGI10256, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2237[] = {SGI10257, SGI10258, SGI10259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10260, SGI0, SGI0, SGI0, SGI10261, SGI10261, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2238[] = {SGI10262, SGI10263, SGI10264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10265, SGI0, SGI0, SGI0, SGI10266, SGI10266, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2239[] = {SGI10267, SGI10268, SGI10269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10270, SGI0, SGI0, SGI0, SGI10271, SGI10271, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2240[] = {SGI10272, SGI10273, SGI10274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10275, SGI0, SGI0, SGI0, SGI10276, SGI10276, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2241[] = {SGI10277, SGI10278, SGI10279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10280, SGI0, SGI0, SGI0, SGI10281, SGI10281, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2242[] = {SGI10282, SGI10283, SGI10284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10285, SGI0, SGI0, SGI0, SGI10286, SGI10286, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2243[] = {SGI10287, SGI10288, SGI10289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10290, SGI0, SGI0, SGI0, SGI10291, SGI10291, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2244[] = {SGI10292, SGI10293, SGI10294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10295, SGI0, SGI0, SGI0, SGI10296, SGI10296, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2245[] = {SGI10297, SGI10298, SGI10299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10300, SGI0, SGI0, SGI0, SGI10301, SGI10301, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2246[] = {SGI10302, SGI0, SGI10303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2247[] = {SGI10304, SGI10305, SGI10306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10307, SGI0, SGI0, SGI0, SGI10308, SGI10308, NULL};
const Segment * const SG2248[] = {SGI10309, SGI10310, SGI10311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10312, SGI0, SGI0, SGI0, SGI10313, SGI10313, NULL};
const Segment * const SG2249[] = {SGI10314, SGI10315, SGI10316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10317, SGI0, SGI0, SGI0, SGI10318, SGI10318, NULL};
const Segment * const SG2250[] = {SGI10319, SGI10320, SGI10321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10322, SGI0, SGI0, SGI0, SGI10323, SGI10323, NULL};
const Segment * const SG2251[] = {SGI10324, SGI10325, SGI10326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10327, SGI0, SGI0, SGI0, SGI10328, SGI10328, NULL};
const Segment * const SG2252[] = {SGI10329, SGI10330, SGI10331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10332, SGI0, SGI0, SGI0, SGI10333, SGI10333, NULL};
const Segment * const SG2253[] = {SGI10334, SGI10335, SGI10336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10337, SGI0, SGI0, SGI0, SGI10338, SGI10338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2254[] = {SGI10339, SGI10340, SGI10341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10342, SGI0, SGI0, SGI0, SGI10343, SGI10343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2255[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10344, SGI10345, SGI10346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2256[] = {SGI10347, SGI10348, SGI10349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10350, SGI0, SGI0, SGI0, SGI10351, SGI10351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2257[] = {SGI10352, SGI10353, SGI10354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10355, SGI0, SGI0, SGI0, SGI10356, SGI10356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2258[] = {SGI10357, SGI10358, SGI10359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10360, SGI0, SGI0, SGI0, SGI10361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2259[] = {SGI10362, SGI10363, SGI10364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10365, SGI0, SGI0, SGI0, SGI10366, SGI10366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2260[] = {SGI10367, SGI0, SGI10368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10369, SGI0, SGI0, SGI0, SGI10370, SGI10370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2261[] = {SGI10371, SGI10372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10376, SGI0, SGI0, SGI0, SGI10377, SGI10377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2262[] = {SGI10373, SGI10374, SGI10375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10378, SGI0, SGI0, SGI0, SGI10379, SGI10379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2263[] = {SGI10380, SGI10381, SGI10382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10385, SGI0, SGI0, SGI0, SGI10387, SGI10387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2264[] = {SGI10383, SGI10384, SGI10386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10388, SGI0, SGI0, SGI0, SGI10389, SGI10389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2265[] = {SGI10392, SGI0, SGI10394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2266[] = {SGI10390, SGI10391, SGI10393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10395, SGI0, SGI0, SGI0, SGI10396, SGI10396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2267[] = {SGI10397, SGI0, SGI10398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2268[] = {SGI10399, SGI10400, SGI10401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10402, SGI0, SGI0, SGI0, SGI10403, SGI10403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2269[] = {SGI10404, SGI0, SGI10405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2270[] = {SGI10406, SGI10407, SGI10408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10409, SGI0, SGI0, SGI0, SGI10410, SGI10410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2271[] = {SGI10411, SGI0, SGI10412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2272[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10413, SGI10414, SGI10415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2273[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10416, SGI10417, SGI10418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2274[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10419, SGI10420, SGI10421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2275[] = {SGI10422, SGI10423, SGI10424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10425, SGI0, SGI0, SGI0, SGI10426, SGI10426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2276[] = {SGI10427, SGI10428, SGI10429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10430, SGI0, SGI0, SGI0, SGI10431, SGI10431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2277[] = {SGI10432, SGI10433, SGI10434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10435, SGI0, SGI0, SGI0, SGI10436, SGI10436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2278[] = {SGI10437, SGI10438, SGI10439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10440, SGI0, SGI0, SGI0, SGI10441, SGI10441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2279[] = {SGI10442, SGI0, SGI10443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2280[] = {SGI10444, SGI10445, SGI10446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10447, SGI0, SGI0, SGI0, SGI10448, SGI10448, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2281[] = {SGI10449, SGI10450, SGI10451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10452, SGI0, SGI0, SGI0, SGI10453, SGI10453, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2282[] = {SGI10454, SGI0, SGI10455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2283[] = {SGI10456, SGI10457, SGI10458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10459, SGI0, SGI0, SGI0, SGI10460, SGI10460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2284[] = {SGI10461, SGI10462, SGI10463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10464, SGI0, SGI0, SGI0, SGI10465, SGI10465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2285[] = {SGI10466, SGI0, SGI10467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2286[] = {SGI10468, SGI0, SGI10469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2287[] = {SGI10470, SGI10471, SGI10472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10473, SGI0, SGI0, SGI0, SGI10474, SGI10474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2288[] = {SGI10475, SGI10476, SGI10477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10478, SGI0, SGI0, SGI0, SGI10479, SGI10479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2289[] = {SGI10480, SGI10481, SGI10482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10483, SGI0, SGI0, SGI0, SGI10484, SGI10484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2290[] = {SGI0, SGI10485, SGI10486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10487, SGI0, SGI0, SGI0, SGI10488, SGI10488, SGI0, NULL};
const Segment * const SG2291[] = {SGI0, SGI10489, SGI10490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10491, SGI0, SGI0, SGI0, SGI10492, SGI10492, SGI0, NULL};
const Segment * const SG2292[] = {SGI10493, SGI10494, NULL};
const Segment * const SG2293[] = {SGI10495, SGI10496, NULL};
const Segment * const SG2294[] = {SGI10497, SGI10498, NULL};
const Segment * const SG2295[] = {SGI10499, SGI10500, NULL};
const Segment * const SG2296[] = {SGI10501, SGI10502, NULL};
const Segment * const SG2297[] = {SGI0, SGI0, SGI10503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10504, SGI0, SGI0, SGI0, SGI10505, SGI10506, NULL};
const Segment * const SG2298[] = {SGI10507, SGI0, SGI10508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10509, SGI0, SGI0, SGI0, SGI10510, SGI10511, NULL};
const Segment * const SG2299[] = {SGI10512, SGI0, SGI10513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10514, SGI0, SGI0, SGI0, SGI10515, SGI10516, NULL};
const Segment * const SG2300[] = {SGI10517, SGI10518, SGI10519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10520, SGI0, SGI0, SGI0, SGI10521, SGI10522, NULL};
const Segment * const SG2301[] = {SGI10523, SGI10524, SGI10525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10526, SGI0, SGI0, SGI0, SGI10527, SGI10527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2302[] = {SGI10528, SGI0, SGI10529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2303[] = {SGI10530, SGI10531, SGI10532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10533, SGI0, SGI0, SGI0, SGI10534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2304[] = {SGI10535, SGI10536, SGI10537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10538, SGI0, SGI0, SGI0, SGI10539, SGI10539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2305[] = {SGI0, SGI10540, SGI10542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10545, SGI0, SGI0, SGI0, SGI10546, SGI10548, NULL};
const Segment * const SG2306[] = {SGI10541, SGI10543, SGI10544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10547, SGI0, SGI0, SGI0, SGI10549, SGI10549, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2307[] = {SGI10550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10551, SGI10551, SGI10552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2308[] = {SGI10553, SGI10554, SGI10555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10556, SGI0, SGI0, SGI0, SGI10557, SGI10558, NULL};
const Segment * const SG2309[] = {SGI10559, SGI10560, SGI10561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10562, SGI0, SGI0, SGI0, SGI10563, SGI10564, NULL};
const Segment * const SG2310[] = {SGI10565, SGI10566, SGI10567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10568, SGI0, SGI0, SGI0, SGI10569, SGI10569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2311[] = {SGI10570, SGI10571, SGI10572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10573, SGI0, SGI0, SGI0, SGI10574, SGI10574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2312[] = {SGI10575, SGI0, SGI10576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2313[] = {SGI10577, SGI0, SGI10578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2314[] = {SGI10579, SGI10580, SGI10581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10582, SGI0, SGI0, SGI0, SGI10583, SGI10584, NULL};
const Segment * const SG2315[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2316[] = {SGI10586, SGI10587, SGI10588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10589, SGI0, SGI0, SGI0, SGI10590, SGI10590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2317[] = {SGI10591, SGI10592, SGI10593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10594, SGI0, SGI0, SGI0, SGI10595, SGI10595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2318[] = {SGI10596, SGI10597, SGI10598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10599, SGI0, SGI0, SGI0, SGI10600, SGI10600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2319[] = {SGI10601, SGI0, SGI10602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2320[] = {SGI10603, SGI10604, SGI10605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10606, SGI0, SGI0, SGI0, SGI10607, SGI10607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2321[] = {SGI10608, SGI10609, SGI10610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10611, SGI0, SGI0, SGI0, SGI10612, SGI10612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2322[] = {SGI10613, SGI0, SGI10614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2323[] = {SGI10615, SGI10616, SGI10617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10618, SGI0, SGI0, SGI0, SGI10619, SGI10619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2324[] = {SGI10620, SGI10621, SGI10622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10623, SGI0, SGI0, SGI0, SGI10624, SGI10624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2325[] = {SGI10625, SGI0, SGI10626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2326[] = {SGI10627, SGI10628, SGI10629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10630, SGI10631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2327[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2328[] = {SGI10633, SGI10634, SGI10635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10636, SGI0, SGI0, SGI0, SGI10637, SGI10637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2329[] = {SGI10638, SGI10639, SGI10640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10641, SGI0, SGI0, SGI0, SGI10642, SGI10642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2330[] = {SGI10643, SGI0, SGI10644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2331[] = {SGI10645, SGI10646, SGI10647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10648, SGI0, SGI0, SGI0, SGI10649, SGI10649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2332[] = {SGI10650, SGI10651, SGI10652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10653, SGI0, SGI0, SGI0, SGI10654, SGI10655, NULL};
const Segment * const SG2333[] = {SGI10656, SGI10657, SGI10658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10659, SGI0, SGI0, SGI0, SGI10660, SGI10661, NULL};
const Segment * const SG2334[] = {SGI0, SGI10662, SGI10663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10664, SGI0, SGI0, SGI0, SGI10665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2335[] = {SGI10666, SGI10667, SGI10668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10671, SGI0, SGI0, SGI0, SGI10673, SGI10674, NULL};
const Segment * const SG2336[] = {SGI0, SGI10669, SGI10670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10672, SGI0, SGI0, SGI0, SGI10675, SGI10675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2337[] = {SGI10676, SGI10677, SGI10678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10679, SGI0, SGI0, SGI0, SGI10680, SGI10681, NULL};
const Segment * const SG2338[] = {SGI10682, SGI10683, SGI10684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10685, SGI0, SGI0, SGI0, SGI10686, SGI10687, NULL};
const Segment * const SG2339[] = {SGI10688, SGI10689, SGI10690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10691, SGI0, SGI0, SGI0, SGI10692, SGI10693, NULL};
const Segment * const SG2340[] = {SGI10694, SGI10696, SGI10698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10700, SGI0, SGI0, SGI0, SGI10702, SGI10702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2341[] = {SGI10695, SGI10697, SGI10699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10701, SGI0, SGI0, SGI0, SGI10703, SGI10703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2342[] = {SGI10704, SGI0, SGI10705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2343[] = {SGI10706, SGI10707, SGI10708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10709, SGI0, SGI0, SGI0, SGI10710, SGI10710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2344[] = {SGI10711, SGI10712, SGI10713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10714, SGI0, SGI0, SGI0, SGI10715, SGI10715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2345[] = {SGI10716, SGI0, SGI10717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2346[] = {SGI10718, SGI10719, SGI10720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10721, SGI0, SGI0, SGI0, SGI10722, SGI10722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2347[] = {SGI10723, SGI10724, SGI10725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10726, SGI0, SGI0, SGI0, SGI10727, SGI10728, NULL};
const Segment * const SG2348[] = {SGI10729, SGI10730, SGI10731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10732, SGI0, SGI0, SGI0, SGI10733, SGI10734, NULL};
const Segment * const SG2349[] = {SGI10735, SGI0, SGI10736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10737, SGI0, SGI0, SGI0, SGI10738, SGI10739, NULL};
const Segment * const SG2350[] = {SGI10740, SGI10741, SGI10742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10743, SGI0, SGI0, SGI0, SGI10744, SGI10745, NULL};
const Segment * const SG2351[] = {SGI10746, SGI10747, SGI10748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10749, SGI0, SGI0, SGI0, SGI10750, SGI10750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2352[] = {SGI10751, SGI0, SGI10752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2353[] = {SGI10753, SGI10754, SGI10755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10756, SGI0, SGI0, SGI0, SGI10757, SGI10757, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2354[] = {SGI10758, SGI10759, SGI10760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10761, SGI0, SGI0, SGI0, SGI10762, SGI10762, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2355[] = {SGI10763, SGI10764, SGI10765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10766, SGI0, SGI0, SGI0, SGI10767, SGI10767, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2356[] = {SGI10768, SGI10769, SGI10770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10771, SGI0, SGI0, SGI0, SGI10772, SGI10772, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2357[] = {SGI10773, SGI10774, SGI10775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10776, SGI0, SGI0, SGI0, SGI10777, SGI10777, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2358[] = {SGI10778, SGI10779, SGI10780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10781, SGI0, SGI0, SGI0, SGI10782, SGI10782, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2359[] = {SGI10783, SGI0, SGI10784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2360[] = {SGI10785, SGI10786, SGI10787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10788, SGI0, SGI0, SGI0, SGI10789, SGI10789, NULL};
const Segment * const SG2361[] = {SGI10790, SGI10791, SGI10792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10793, SGI0, SGI0, SGI0, SGI10794, SGI10794, NULL};
const Segment * const SG2362[] = {SGI10795, SGI10796, SGI10797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10798, SGI0, SGI0, SGI0, SGI10799, SGI10799, NULL};
const Segment * const SG2363[] = {SGI10800, SGI10801, SGI10802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10803, SGI0, SGI0, SGI0, SGI10804, SGI10804, NULL};
const Segment * const SG2364[] = {SGI10805, SGI10806, SGI10807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10808, SGI0, SGI0, SGI0, SGI10809, SGI10809, NULL};
const Segment * const SG2365[] = {SGI10810, SGI10811, SGI10812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10813, SGI0, SGI0, SGI0, SGI10814, SGI10814, NULL};
const Segment * const SG2366[] = {SGI10815, SGI10816, SGI10817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10818, SGI0, SGI0, SGI0, SGI10819, SGI10819, NULL};
const Segment * const SG2367[] = {SGI10820, SGI10821, SGI10822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10823, SGI0, SGI0, SGI0, SGI10824, SGI10824, NULL};
const Segment * const SG2368[] = {SGI10825, SGI10826, SGI10827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10828, SGI0, SGI0, SGI0, SGI10829, SGI10829, NULL};
const Segment * const SG2369[] = {SGI10830, SGI10831, SGI10832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10833, SGI0, SGI0, SGI0, SGI10834, SGI10834, NULL};
const Segment * const SG2370[] = {SGI10835, SGI10836, SGI10837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10838, SGI0, SGI0, SGI0, SGI10839, SGI10839, NULL};
const Segment * const SG2371[] = {SGI10840, SGI10841, SGI10842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10843, SGI0, SGI0, SGI0, SGI10844, SGI10844, NULL};
const Segment * const SG2372[] = {SGI10845, SGI10846, SGI10847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10848, SGI0, SGI0, SGI0, SGI10849, SGI10849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2373[] = {SGI10850, SGI0, SGI10851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2374[] = {SGI10852, SGI10853, SGI10854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2375[] = {SGI10855, SGI10856, SGI10857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2376[] = {SGI10858, SGI10859, SGI10860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10861, SGI0, SGI0, SGI0, SGI10862, SGI10863, NULL};
const Segment * const SG2377[] = {SGI10864, SGI10865, SGI10866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10867, SGI0, SGI0, SGI0, SGI10868, SGI10869, NULL};
const Segment * const SG2378[] = {SGI0, SGI10870, SGI10871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10872, SGI0, SGI0, SGI0, SGI10873, SGI10874, NULL};
const Segment * const SG2379[] = {SGI10875, SGI0, SGI10876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10877, SGI0, SGI0, SGI0, SGI10878, SGI10878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2380[] = {SGI10879, SGI10880, SGI10881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10882, SGI0, SGI0, SGI0, SGI10883, SGI10883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2381[] = {SGI10884, SGI0, SGI10885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2382[] = {SGI10886, SGI10887, SGI10888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10889, SGI0, SGI0, SGI0, SGI10890, SGI10890, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2383[] = {SGI10891, SGI10892, SGI10893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10894, SGI0, SGI0, SGI0, SGI10895, SGI10895, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2384[] = {SGI10896, SGI10897, SGI10898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10899, SGI0, SGI0, SGI0, SGI10900, SGI10900, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2385[] = {SGI10901, SGI10902, SGI10903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10904, SGI0, SGI0, SGI0, SGI10905, SGI10905, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2386[] = {SGI10906, SGI0, SGI10907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2387[] = {SGI10908, SGI0, SGI10909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2388[] = {SGI10910, SGI10911, SGI10912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10913, SGI0, SGI0, SGI0, SGI10914, SGI10914, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2389[] = {SGI10915, SGI10916, SGI10917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10918, SGI0, SGI0, SGI0, SGI10919, SGI10919, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2390[] = {SGI10920, SGI10921, SGI10922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10923, SGI0, SGI0, SGI0, SGI10924, SGI10924, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2391[] = {SGI10925, SGI10926, SGI10927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10928, SGI0, SGI0, SGI0, SGI10929, SGI10929, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2392[] = {SGI10930, SGI10931, SGI10932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10933, SGI0, SGI0, SGI0, SGI10934, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2393[] = {SGI10935, SGI10936, SGI10937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10938, SGI0, SGI0, SGI0, SGI10939, SGI10939, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2394[] = {SGI10940, SGI0, SGI10941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2395[] = {SGI10942, SGI10943, SGI10944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10945, SGI0, SGI0, SGI0, SGI10946, SGI10946, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2396[] = {SGI10947, SGI0, SGI10948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10949, SGI0, SGI0, SGI0, SGI10950, SGI10950, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2397[] = {SGI10951, SGI0, SGI10952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2398[] = {SGI0, SGI10953, SGI10954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10955, SGI0, SGI0, SGI0, SGI10956, SGI10957, NULL};
const Segment * const SG2399[] = {SGI0, SGI10958, SGI10959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10960, SGI0, SGI0, SGI0, SGI10961, SGI10962, NULL};
const Segment * const SG2400[] = {SGI0, SGI10963, SGI10964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10965, SGI0, SGI0, SGI0, SGI10966, SGI10967, NULL};
const Segment * const SG2401[] = {SGI10968, SGI10969, SGI10970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10971, SGI0, SGI0, SGI0, SGI10972, SGI10973, NULL};
const Segment * const SG2402[] = {SGI0, SGI0, SGI10974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10975, SGI0, SGI0, SGI0, SGI10976, SGI10977, NULL};
const Segment * const SG2403[] = {SGI10978, SGI10979, SGI10980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10981, SGI0, SGI0, SGI0, SGI0, SGI10982, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2404[] = {SGI10983, SGI10984, SGI10985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10986, SGI0, SGI0, SGI0, SGI10987, SGI10987, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2405[] = {SGI10988, SGI10989, SGI10990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10991, SGI0, SGI0, SGI0, SGI10992, SGI10992, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2406[] = {SGI10993, SGI10994, SGI10995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI10996, SGI0, SGI0, SGI0, SGI10997, SGI10997, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2407[] = {SGI10998, SGI10999, SGI11000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11001, SGI0, SGI0, SGI0, SGI11002, SGI11002, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2408[] = {SGI11003, SGI11004, SGI11005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11006, SGI0, SGI0, SGI0, SGI11007, SGI11007, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2409[] = {SGI11008, SGI11009, SGI11010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11011, SGI0, SGI0, SGI0, SGI11012, SGI11012, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2410[] = {SGI11013, SGI11014, SGI11015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11016, SGI0, SGI0, SGI0, SGI11017, SGI11017, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2411[] = {SGI11018, SGI11019, SGI11020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11021, SGI0, SGI0, SGI0, SGI11022, SGI11022, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2412[] = {SGI11023, SGI11024, SGI11025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11026, SGI0, SGI0, SGI0, SGI11027, SGI11027, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2413[] = {SGI11028, SGI11029, SGI11030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11031, SGI0, SGI0, SGI0, SGI11032, SGI11032, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2414[] = {SGI11033, SGI11034, SGI11035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11036, SGI0, SGI0, SGI0, SGI11037, SGI11037, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2415[] = {SGI11038, SGI11039, SGI11040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11041, SGI0, SGI0, SGI0, SGI11044, SGI11044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2416[] = {SGI11042, SGI11043, SGI11045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11046, SGI0, SGI0, SGI0, SGI11047, SGI11047, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2417[] = {SGI11048, SGI11049, SGI11050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11052, SGI0, SGI0, SGI0, SGI11055, SGI11055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2418[] = {SGI11051, SGI11053, SGI11054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11056, SGI0, SGI0, SGI0, SGI11057, SGI11057, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2419[] = {SGI11058, SGI11059, SGI11060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11062, SGI0, SGI0, SGI0, SGI11064, SGI11064, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2420[] = {SGI11061, SGI0, SGI11063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2421[] = {SGI11065, SGI0, SGI11066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2422[] = {SGI11067, SGI11068, SGI11069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11070, SGI11071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2423[] = {SGI11072, SGI11073, SGI11074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11076, SGI11078, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2424[] = {SGI11075, SGI11077, SGI11079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11080, SGI11080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2425[] = {SGI11081, SGI11082, SGI11083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11084, SGI0, SGI0, SGI0, SGI11085, SGI11085, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2426[] = {SGI11086, SGI0, SGI11087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2427[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11088, SGI0, SGI0, SGI11089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2428[] = {SGI11090, SGI11091, SGI11092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11093, SGI0, SGI0, SGI0, SGI11094, SGI11095, NULL};
const Segment * const SG2429[] = {SGI11096, SGI11097, SGI11098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11099, SGI0, SGI0, SGI0, SGI11100, SGI11100, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2430[] = {SGI11101, SGI0, SGI11102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11103, SGI0, SGI0, SGI0, SGI11104, SGI11104, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2431[] = {SGI11105, SGI11106, SGI11107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11108, SGI0, SGI0, SGI0, SGI11109, SGI11109, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2432[] = {SGI11110, SGI11111, SGI11112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11113, SGI0, SGI0, SGI0, SGI11114, SGI11114, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2433[] = {SGI11115, SGI11116, SGI11117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11118, SGI0, SGI0, SGI0, SGI11119, SGI11119, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2434[] = {SGI11120, SGI11121, SGI11122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11123, SGI0, SGI0, SGI0, SGI11124, SGI11124, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2435[] = {SGI11125, SGI11126, SGI11127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11128, SGI0, SGI0, SGI0, SGI11129, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2436[] = {SGI11130, SGI11131, SGI11132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11133, SGI0, SGI0, SGI0, SGI11134, SGI11134, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2437[] = {SGI11135, SGI11136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11137, SGI0, SGI0, SGI0, SGI11138, SGI11138, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2438[] = {SGI11139, SGI11140, SGI11141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11142, SGI0, SGI0, SGI0, SGI11143, SGI11143, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2439[] = {SGI11144, SGI11145, SGI11146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11147, SGI0, SGI0, SGI0, SGI11148, SGI11148, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2440[] = {SGI11149, SGI0, SGI11150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2441[] = {SGI11151, SGI11152, SGI11153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11154, SGI0, SGI0, SGI0, SGI11155, SGI11156, NULL};
const Segment * const SG2442[] = {SGI11157, SGI11158, SGI11159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11160, SGI0, SGI0, SGI0, SGI11161, SGI11162, NULL};
const Segment * const SG2443[] = {SGI11163, SGI11164, SGI11165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11166, SGI0, SGI0, SGI0, SGI11167, SGI11168, NULL};
const Segment * const SG2444[] = {SGI11169, SGI11170, SGI11171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11172, SGI0, SGI0, SGI0, SGI11173, SGI11174, NULL};
const Segment * const SG2445[] = {SGI11175, SGI11176, SGI11177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11178, SGI0, SGI0, SGI0, SGI11179, SGI11180, NULL};
const Segment * const SG2446[] = {SGI11181, SGI11182, SGI11183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11184, SGI0, SGI0, SGI0, SGI11185, SGI11186, NULL};
const Segment * const SG2447[] = {SGI11187, SGI11188, SGI11189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11190, SGI0, SGI0, SGI0, SGI11191, SGI11192, NULL};
const Segment * const SG2448[] = {SGI11193, SGI11194, SGI11195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2449[] = {SGI0, SGI11196, SGI11197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11198, SGI0, SGI0, SGI0, SGI11199, SGI11199, SGI0, NULL};
const Segment * const SG2450[] = {SGI0, SGI0, SGI11200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11201, SGI0, SGI0, SGI0, SGI11202, SGI11202, SGI0, NULL};
const Segment * const SG2451[] = {SGI11203, SGI11204, SGI11205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11206, SGI0, SGI0, SGI0, SGI11207, SGI11207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2452[] = {SGI11208, SGI11209, SGI11210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11211, SGI0, SGI0, SGI0, SGI11212, SGI11212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2453[] = {SGI11213, SGI0, SGI11214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2454[] = {SGI11215, SGI11216, SGI11217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11218, SGI0, SGI0, SGI0, SGI11219, SGI11219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2455[] = {SGI11220, SGI11221, SGI11222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11223, SGI0, SGI0, SGI0, SGI11224, SGI11224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2456[] = {SGI11225, SGI11226, SGI11227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11228, SGI0, SGI0, SGI0, SGI11229, SGI11229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2457[] = {SGI11230, SGI11231, SGI11232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11233, SGI0, SGI0, SGI0, SGI11234, SGI11234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2458[] = {SGI11235, SGI11236, SGI11237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11238, SGI0, SGI0, SGI0, SGI11239, SGI11239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2459[] = {SGI11240, SGI11241, SGI11242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11243, SGI0, SGI0, SGI0, SGI11244, SGI11244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2460[] = {SGI11245, SGI11246, SGI11247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11248, SGI0, SGI0, SGI0, SGI11249, SGI11249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2461[] = {SGI11250, SGI11251, SGI11252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11253, SGI0, SGI0, SGI0, SGI11254, SGI11254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2462[] = {SGI11255, SGI11256, SGI11257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11258, SGI0, SGI0, SGI0, SGI11259, SGI11259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2463[] = {SGI11260, SGI0, SGI11261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2464[] = {SGI11262, SGI11263, SGI11264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11265, SGI0, SGI0, SGI0, SGI11266, SGI11266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2465[] = {SGI11267, SGI11268, SGI11269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11270, SGI0, SGI0, SGI0, SGI11271, SGI11271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2466[] = {SGI11272, SGI0, SGI11273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2467[] = {SGI11274, SGI11275, SGI11276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2468[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11277, SGI0, SGI0, SGI11278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2469[] = {SGI11279, SGI11280, SGI11281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11282, SGI0, SGI0, SGI0, SGI11283, SGI11283, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2470[] = {SGI11284, SGI11285, SGI11286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11287, SGI0, SGI0, SGI0, SGI11288, SGI11288, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2471[] = {SGI11289, SGI0, SGI11290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2472[] = {SGI11291, SGI0, SGI11292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2473[] = {SGI11293, SGI11294, SGI11295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11296, SGI0, SGI0, SGI0, SGI11297, SGI11297, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2474[] = {SGI11298, SGI11299, SGI11300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2475[] = {SGI0, SGI11301, SGI11302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11303, SGI0, SGI0, SGI0, SGI11304, SGI11304, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2476[] = {SGI11305, SGI11306, SGI11307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11308, SGI0, SGI0, SGI0, SGI11309, SGI11310, NULL};
const Segment * const SG2477[] = {SGI11311, SGI11312, SGI11313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11314, SGI0, SGI0, SGI0, SGI11315, SGI11316, NULL};
const Segment * const SG2478[] = {SGI11317, SGI11318, SGI11319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11320, SGI0, SGI0, SGI0, SGI11321, SGI11322, NULL};
const Segment * const SG2479[] = {SGI11323, SGI11324, SGI11325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2480[] = {SGI11326, SGI11327, SGI11328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11329, SGI0, SGI0, SGI0, SGI11330, SGI11330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2481[] = {SGI11331, SGI0, SGI11332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2482[] = {SGI11333, SGI11334, SGI11335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11336, SGI0, SGI0, SGI0, SGI11337, SGI11337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2483[] = {SGI11338, SGI0, SGI11339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11340, SGI0, SGI0, SGI0, SGI11341, SGI11341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2484[] = {SGI11342, SGI11343, SGI11344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11345, SGI0, SGI0, SGI0, SGI11346, SGI11346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2485[] = {SGI11347, SGI11348, SGI11349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11350, SGI0, SGI0, SGI0, SGI11351, SGI11351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2486[] = {SGI11352, SGI11353, SGI11354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11355, SGI0, SGI0, SGI0, SGI11356, SGI11356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2487[] = {SGI11357, SGI0, SGI11358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2488[] = {SGI11359, SGI11360, SGI11361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11362, SGI11363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2489[] = {SGI11364, SGI11365, SGI11366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11367, SGI0, SGI0, SGI0, SGI11368, SGI11369, NULL};
const Segment * const SG2490[] = {SGI11370, SGI11371, SGI11372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11373, SGI0, SGI0, SGI0, SGI11374, SGI11374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2491[] = {SGI11375, SGI0, SGI11376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2492[] = {SGI11377, SGI11378, SGI11379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11380, SGI0, SGI0, SGI0, SGI11381, SGI11381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2493[] = {SGI11382, SGI0, SGI11383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2494[] = {SGI11384, SGI11385, SGI11386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11387, SGI11388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2495[] = {SGI11389, SGI11390, SGI11391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11392, SGI0, SGI0, SGI0, SGI11393, SGI11393, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2496[] = {SGI11394, SGI11395, SGI11396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11397, SGI0, SGI0, SGI0, SGI11398, SGI11398, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2497[] = {SGI11399, SGI11400, SGI11401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11402, SGI0, SGI0, SGI0, SGI11403, SGI11403, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2498[] = {SGI11404, SGI0, SGI11405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2499[] = {SGI11406, SGI11407, SGI11408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11409, SGI0, SGI0, SGI11410, SGI11410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2500[] = {SGI11411, SGI11412, SGI11413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11414, SGI0, SGI0, SGI11415, SGI11415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2501[] = {SGI11416, SGI11417, SGI11418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11419, SGI0, SGI0, SGI11420, SGI11420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2502[] = {SGI11421, SGI11422, SGI11423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11424, SGI0, SGI0, SGI11425, SGI11425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2503[] = {SGI11426, SGI11427, SGI11428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11429, SGI0, SGI0, SGI11430, SGI11430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2504[] = {SGI11431, SGI11432, SGI11433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11434, SGI0, SGI0, SGI11435, SGI11435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2505[] = {SGI11436, SGI11437, SGI11438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11439, SGI0, SGI0, SGI11440, SGI11440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2506[] = {SGI11441, SGI11442, SGI11443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11444, SGI0, SGI0, SGI11445, SGI11445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2507[] = {SGI11446, SGI11447, SGI11448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11449, SGI0, SGI0, SGI11450, SGI11450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2508[] = {SGI11451, SGI11452, SGI11453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11454, SGI0, SGI0, SGI11455, SGI11455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2509[] = {SGI11456, SGI11457, SGI11458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11459, SGI0, SGI0, SGI11460, SGI11460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2510[] = {SGI11461, SGI11462, SGI11463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11464, SGI0, SGI0, SGI11465, SGI11465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2511[] = {SGI11466, SGI0, SGI11467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2512[] = {SGI11468, SGI11469, SGI11470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11471, SGI0, SGI0, SGI11472, SGI11472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2513[] = {SGI11473, SGI11474, SGI11475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11476, SGI0, SGI0, SGI11477, SGI11477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2514[] = {SGI11478, SGI11479, SGI11480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11481, SGI0, SGI0, SGI0, SGI11482, SGI11482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2515[] = {SGI11483, SGI11484, SGI11485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11486, SGI0, SGI0, SGI11487, SGI11487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2516[] = {SGI11488, SGI11489, SGI11490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11491, SGI0, SGI0, SGI11492, SGI11492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2517[] = {SGI11493, SGI11494, SGI11495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11496, SGI0, SGI0, SGI11497, SGI11497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2518[] = {SGI11498, SGI11499, SGI11500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11501, SGI0, SGI0, SGI11502, SGI11502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2519[] = {SGI11503, SGI11504, SGI11505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11506, SGI0, SGI0, SGI11507, SGI11507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2520[] = {SGI11508, SGI11509, SGI11510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11511, SGI0, SGI0, SGI11512, SGI11512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2521[] = {SGI0, SGI0, SGI11513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11514, SGI0, SGI0, SGI0, SGI11515, SGI11515, SGI0, NULL};
const Segment * const SG2522[] = {SGI11516, SGI11517, SGI11518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11519, SGI0, SGI0, SGI0, SGI11520, SGI11520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2523[] = {SGI11521, SGI0, SGI11522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2524[] = {SGI11523, SGI11524, SGI11525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2525[] = {SGI11526, SGI11527, SGI11528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11529, SGI0, SGI0, SGI0, SGI11530, SGI11530, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2526[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11531, SGI0, SGI0, SGI11532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2527[] = {SGI11533, SGI11534, SGI11535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11536, SGI0, SGI0, SGI11537, SGI11537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2528[] = {SGI11538, SGI11539, SGI11540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11541, SGI0, SGI0, SGI11542, SGI11543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2529[] = {SGI11544, SGI11545, SGI11546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11547, SGI0, SGI0, SGI11548, SGI11548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2530[] = {SGI11549, SGI11550, SGI11551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11552, SGI0, SGI0, SGI11553, SGI11553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2531[] = {SGI11554, SGI0, SGI11555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2532[] = {SGI11556, SGI11557, SGI11558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11559, SGI0, SGI0, SGI11560, SGI11560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2533[] = {SGI11561, SGI11562, SGI11563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11564, SGI0, SGI0, SGI11565, SGI11565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2534[] = {SGI11566, SGI11567, SGI11568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11569, SGI0, SGI0, SGI11570, SGI11570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2535[] = {SGI11571, SGI11572, SGI11573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11574, SGI0, SGI0, SGI11575, SGI11575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2536[] = {SGI11576, SGI11577, SGI11578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11579, SGI0, SGI0, SGI11580, SGI11580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2537[] = {SGI11581, SGI11582, SGI11583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11584, SGI0, SGI0, SGI0, SGI11587, SGI11587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2538[] = {SGI11585, SGI11586, SGI11588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11589, SGI0, SGI0, SGI11590, SGI11590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2539[] = {SGI11591, SGI11592, SGI11593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11594, SGI0, SGI0, SGI11595, SGI11595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2540[] = {SGI11596, SGI11597, SGI11598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11599, SGI0, SGI0, SGI11600, SGI11600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2541[] = {SGI11604, SGI11605, SGI11606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11607, SGI0, SGI0, SGI0, SGI11608, SGI11608, NULL};
const Segment * const SG2542[] = {SGI11609, SGI11610, SGI11611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11612, SGI0, SGI0, SGI0, SGI11613, SGI11613, NULL};
const Segment * const SG2543[] = {SGI11614, SGI11615, SGI11616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11617, SGI0, SGI0, SGI0, SGI11618, SGI11618, NULL};
const Segment * const SG2544[] = {SGI11619, SGI11620, SGI11621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11622, SGI0, SGI0, SGI0, SGI11623, SGI11623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2545[] = {SGI11624, SGI11625, SGI11626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11627, SGI0, SGI0, SGI0, SGI11628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2546[] = {SGI11629, SGI11630, SGI11631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11632, SGI0, SGI0, SGI0, SGI11633, SGI11633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2547[] = {SGI11634, SGI11635, SGI11636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11637, SGI0, SGI0, SGI0, SGI11638, SGI11638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2548[] = {SGI11639, SGI11640, SGI11641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11642, SGI0, SGI0, SGI0, SGI11643, SGI11643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2549[] = {SGI11644, SGI11645, SGI11646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11647, SGI0, SGI0, SGI0, SGI11648, SGI11649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2550[] = {SGI11650, SGI0, SGI11651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2551[] = {SGI11601, SGI11602, SGI11603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11652, SGI0, SGI0, SGI11653, SGI11653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2552[] = {SGI0, SGI11654, SGI11655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11656, SGI0, SGI0, SGI11657, SGI11657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2553[] = {SGI11658, SGI11659, SGI11660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11661, SGI0, SGI0, SGI11662, SGI11662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2554[] = {SGI0, SGI11663, SGI11664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11665, SGI0, SGI0, SGI11666, SGI11666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2555[] = {SGI11667, SGI0, SGI11668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11669, SGI0, SGI0, SGI11670, SGI11670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2556[] = {SGI0, SGI11671, SGI11672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11673, SGI0, SGI0, SGI11674, SGI11674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2557[] = {SGI11675, SGI11676, SGI11677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11678, SGI0, SGI0, SGI11679, SGI11679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2558[] = {SGI11680, SGI11681, SGI11682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11683, SGI0, SGI0, SGI0, SGI11684, SGI11684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2559[] = {SGI11685, SGI0, SGI11686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11687, SGI0, SGI0, SGI0, SGI11688, SGI11688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2560[] = {SGI11689, SGI11690, SGI11691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11692, SGI0, SGI0, SGI0, SGI11693, SGI11694, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2561[] = {SGI11695, SGI11696, SGI11697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11698, SGI0, SGI0, SGI0, SGI11699, SGI11699, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2562[] = {SGI11700, SGI0, SGI11701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2563[] = {SGI11702, SGI11703, SGI11704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11705, SGI0, SGI0, SGI0, SGI11706, SGI11706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2564[] = {SGI11707, SGI11708, SGI11709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11710, SGI0, SGI0, SGI0, SGI11711, SGI11711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2565[] = {SGI11712, SGI0, SGI11713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2566[] = {SGI11714, SGI0, SGI11715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2567[] = {SGI11716, SGI11717, SGI11718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11719, SGI0, SGI0, SGI0, SGI11720, SGI11720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2568[] = {SGI11721, SGI11722, SGI11723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11724, SGI0, SGI0, SGI0, SGI11725, SGI11725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2569[] = {SGI11726, SGI0, SGI11727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2570[] = {SGI11728, SGI11729, SGI11730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11731, SGI0, SGI0, SGI0, SGI11734, SGI11734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2571[] = {SGI11732, SGI11733, SGI11735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11736, SGI0, SGI0, SGI0, SGI11737, SGI11737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2572[] = {SGI11738, SGI11739, SGI11740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11741, SGI0, SGI0, SGI0, SGI11742, SGI11742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2573[] = {SGI11743, SGI11744, SGI11745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11746, SGI0, SGI0, SGI0, SGI11747, SGI11747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2574[] = {SGI11748, SGI0, SGI11749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11750, SGI0, SGI0, SGI11751, SGI11751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2575[] = {SGI11752, SGI0, SGI11753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11754, SGI0, SGI0, SGI11755, SGI11755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2576[] = {SGI11756, SGI0, SGI11757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11758, SGI0, SGI0, SGI11759, SGI11759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2577[] = {SGI11760, SGI0, SGI11761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11762, SGI0, SGI0, SGI11763, SGI11763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2578[] = {SGI11764, SGI0, SGI11765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11766, SGI0, SGI0, SGI11767, SGI11767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2579[] = {SGI11768, SGI0, SGI11769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11770, SGI0, SGI0, SGI11771, SGI11771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2580[] = {SGI11772, SGI0, SGI11773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11774, SGI0, SGI0, SGI11775, SGI11775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2581[] = {SGI11776, SGI0, SGI11777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11778, SGI0, SGI0, SGI11779, SGI11779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2582[] = {SGI11780, SGI0, SGI11781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11782, SGI0, SGI0, SGI11783, SGI11783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2583[] = {SGI11784, SGI0, SGI11785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11786, SGI0, SGI0, SGI11787, SGI11787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2584[] = {SGI11788, SGI0, SGI11789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11790, SGI0, SGI0, SGI11791, SGI11791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2585[] = {SGI11792, SGI0, SGI11793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11794, SGI0, SGI0, SGI11795, SGI11795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2586[] = {SGI11796, SGI0, SGI11797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2587[] = {SGI11798, SGI0, SGI11799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11800, SGI0, SGI0, SGI11801, SGI11801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2588[] = {SGI0, SGI11802, NULL};
const Segment * const SG2589[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2590[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2591[] = {SGI11805, SGI11807, SGI11809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11811, SGI0, SGI0, SGI0, SGI11813, SGI11813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2592[] = {SGI11806, SGI11808, SGI11810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11812, SGI0, SGI0, SGI0, SGI11814, SGI11814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2593[] = {SGI11815, SGI11816, SGI11817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11818, SGI0, SGI0, SGI0, SGI11819, SGI11819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2594[] = {SGI11820, SGI11821, SGI11822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11823, SGI0, SGI0, SGI0, SGI11824, SGI11824, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2595[] = {SGI11825, SGI11826, SGI11827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11828, SGI0, SGI0, SGI0, SGI11829, SGI11829, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2596[] = {SGI11830, SGI11831, SGI11832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11833, SGI0, SGI0, SGI0, SGI11834, SGI11834, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2597[] = {SGI11835, SGI11836, SGI11837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11838, SGI0, SGI0, SGI0, SGI11839, SGI11839, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2598[] = {SGI11840, SGI11841, SGI11842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11843, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2599[] = {SGI11844, SGI11845, SGI11846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11847, SGI0, SGI0, SGI0, SGI11848, SGI11849, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2600[] = {SGI11850, SGI11851, SGI11852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11853, SGI0, SGI0, SGI0, SGI11854, SGI11854, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2601[] = {SGI11855, SGI11856, SGI11857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11858, SGI0, SGI0, SGI0, SGI11859, SGI11859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2602[] = {SGI11860, SGI0, SGI11861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2603[] = {SGI11862, SGI11863, SGI11864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11865, SGI0, SGI0, SGI0, SGI11866, SGI11866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2604[] = {SGI11867, SGI11868, SGI11869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11870, SGI0, SGI0, SGI0, SGI11871, SGI11871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2605[] = {SGI11872, SGI0, SGI11873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2606[] = {SGI11874, SGI11875, SGI11876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11877, SGI11878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2607[] = {SGI11879, SGI11880, SGI11881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11882, SGI11882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2608[] = {SGI11883, SGI0, SGI11884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11885, SGI0, SGI0, SGI0, SGI11886, SGI11887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2609[] = {SGI11888, SGI11889, SGI11890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11891, SGI0, SGI0, SGI0, SGI11892, SGI11892, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2610[] = {SGI11893, SGI11894, SGI11895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11896, SGI0, SGI0, SGI0, SGI11897, SGI11897, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2611[] = {SGI11898, SGI11899, SGI11900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11901, SGI0, SGI0, SGI0, SGI11902, SGI11902, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2612[] = {SGI11903, SGI11904, SGI11905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11906, SGI0, SGI0, SGI0, SGI11907, SGI11907, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2613[] = {SGI11908, SGI11909, SGI11910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11911, SGI0, SGI0, SGI0, SGI11912, SGI11912, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2614[] = {SGI0, SGI11913, SGI11914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11915, SGI0, SGI0, SGI0, SGI11916, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2615[] = {SGI11917, SGI11918, SGI11919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11920, SGI0, SGI0, SGI0, SGI11921, SGI11921, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2616[] = {SGI0, SGI11922, SGI11923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11924, SGI0, SGI0, SGI0, SGI11925, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2617[] = {SGI11926, SGI11927, SGI11928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11929, SGI0, SGI0, SGI0, SGI11931, SGI11931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2618[] = {SGI0, SGI0, SGI11930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11932, SGI0, SGI0, SGI0, SGI11933, SGI0, SGI0, SGI0, SGI0, SGI11934, NULL};
const Segment * const SG2619[] = {SGI0, SGI11935, SGI11936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11937, SGI0, SGI0, SGI0, SGI11938, SGI11938, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2620[] = {SGI0, SGI11939, SGI11940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11941, SGI0, SGI0, SGI0, SGI11942, SGI11942, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2621[] = {SGI11943, SGI11944, SGI11945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11946, SGI0, SGI0, SGI0, SGI11947, SGI11947, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2622[] = {SGI0, SGI11948, SGI11949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11950, SGI0, SGI0, SGI0, SGI11951, SGI11951, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2623[] = {SGI0, SGI11952, SGI11953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11954, SGI0, SGI0, SGI0, SGI11955, SGI11955, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2624[] = {SGI0, SGI11956, SGI11957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11958, SGI0, SGI0, SGI0, SGI11959, SGI11959, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2625[] = {SGI11960, SGI11961, SGI11962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11963, SGI0, SGI0, SGI0, SGI11964, SGI11964, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2626[] = {SGI11965, SGI11966, SGI11967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11968, SGI0, SGI0, SGI0, SGI11969, SGI11969, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2627[] = {SGI11970, SGI0, SGI11971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11972, SGI0, SGI0, SGI0, SGI11973, SGI11973, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2628[] = {SGI11974, SGI0, SGI11975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11976, SGI0, SGI0, SGI0, SGI11977, SGI11977, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2629[] = {SGI11978, SGI11979, SGI11980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11981, SGI11981, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2630[] = {SGI11982, SGI11983, SGI11984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11985, SGI0, SGI0, SGI0, SGI11986, SGI11986, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2631[] = {SGI11987, SGI11988, SGI11989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11990, SGI0, SGI0, SGI0, SGI11991, SGI11991, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2632[] = {SGI0, SGI0, SGI11992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2633[] = {SGI11993, SGI0, SGI11994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI11995, SGI0, SGI0, SGI0, SGI11996, SGI11996, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2634[] = {SGI11997, SGI11998, SGI11999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12000, SGI0, SGI0, SGI0, SGI12001, SGI12001, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2635[] = {SGI12002, SGI12003, SGI12004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12005, SGI0, SGI0, SGI0, SGI12006, SGI12006, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2636[] = {SGI12007, SGI12008, SGI12009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12010, SGI0, SGI0, SGI0, SGI12011, SGI12011, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2637[] = {SGI12012, SGI0, SGI12013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2638[] = {SGI12014, SGI12015, NULL};
const Segment * const SG2639[] = {SGI12016, SGI12017, NULL};
const Segment * const SG2640[] = {SGI12018, SGI12019, NULL};
const Segment * const SG2641[] = {SGI12020, SGI12021, NULL};
const Segment * const SG2642[] = {SGI12022, SGI12023, NULL};
const Segment * const SG2643[] = {SGI12024, SGI12025, SGI12026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12027, SGI0, SGI0, SGI0, SGI12028, SGI12028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2644[] = {SGI12029, SGI0, SGI12030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2645[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12031, SGI12032, SGI12033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2646[] = {SGI12034, SGI12035, SGI12036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12037, SGI0, SGI0, SGI0, SGI12038, SGI12039, NULL};
const Segment * const SG2647[] = {SGI12040, SGI12041, SGI12042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12043, SGI0, SGI0, SGI0, SGI12044, SGI12045, NULL};
const Segment * const SG2648[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2649[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2650[] = {SGI12048, SGI12049, SGI12050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12051, SGI0, SGI0, SGI0, SGI12052, SGI12052, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2651[] = {SGI12053, SGI12054, SGI12055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12056, SGI0, SGI0, SGI0, SGI12057, SGI12057, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2652[] = {SGI12058, SGI12059, SGI12060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12061, SGI0, SGI0, SGI0, SGI12062, SGI12062, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2653[] = {SGI12063, SGI12064, SGI12065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12066, SGI0, SGI0, SGI0, SGI12067, SGI12067, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2654[] = {SGI12068, SGI12069, SGI12070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12071, SGI0, SGI0, SGI0, SGI12072, SGI12072, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2655[] = {SGI12073, SGI12074, SGI12075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12076, SGI0, SGI0, SGI0, SGI12077, SGI12077, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2656[] = {SGI12078, SGI12079, SGI12080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12081, SGI0, SGI0, SGI0, SGI12082, SGI12083, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2657[] = {SGI12084, SGI12085, SGI12086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12087, SGI0, SGI0, SGI0, SGI12088, SGI12088, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2658[] = {SGI12089, SGI12090, SGI12091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12092, SGI0, SGI0, SGI0, SGI12093, SGI12093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2659[] = {SGI12094, SGI0, SGI12095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2660[] = {SGI12096, SGI12097, SGI12098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12099, SGI0, SGI0, SGI0, SGI12100, SGI12100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2661[] = {SGI12101, SGI12102, NULL};
const Segment * const SG2662[] = {SGI12103, SGI12104, NULL};
const Segment * const SG2663[] = {SGI12105, SGI12106, NULL};
const Segment * const SG2664[] = {SGI12107, SGI12108, SGI12109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12110, SGI0, SGI0, SGI0, SGI12111, SGI12111, NULL};
const Segment * const SG2665[] = {SGI12112, SGI12113, SGI12114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12115, SGI0, SGI0, SGI0, SGI12116, SGI12116, NULL};
const Segment * const SG2666[] = {SGI12117, SGI12118, SGI12119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12120, SGI0, SGI0, SGI0, SGI12121, SGI12121, NULL};
const Segment * const SG2667[] = {SGI12122, SGI12123, SGI12124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12125, SGI0, SGI0, SGI0, SGI12126, SGI12126, NULL};
const Segment * const SG2668[] = {SGI12127, SGI12128, SGI12129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12130, SGI0, SGI0, SGI0, SGI12131, SGI12131, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2669[] = {SGI12132, SGI12133, SGI12134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12135, SGI0, SGI0, SGI0, SGI12136, SGI12136, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2670[] = {SGI12137, SGI12138, SGI12139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12140, SGI0, SGI0, SGI0, SGI12141, SGI12141, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2671[] = {SGI12142, SGI12143, SGI12144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12145, SGI0, SGI0, SGI0, SGI12146, SGI12146, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2672[] = {SGI12147, SGI12148, SGI12149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12150, SGI0, SGI0, SGI0, SGI12151, SGI12151, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2673[] = {SGI12152, SGI12153, SGI12154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12155, SGI0, SGI0, SGI0, SGI12156, SGI12156, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2674[] = {SGI12157, SGI12158, SGI12159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12160, SGI0, SGI0, SGI0, SGI12161, SGI12161, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2675[] = {SGI12162, SGI12163, SGI12164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12165, SGI0, SGI0, SGI0, SGI12166, SGI12166, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2676[] = {SGI12167, SGI12168, SGI12169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12170, SGI0, SGI0, SGI0, SGI12171, SGI12171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2677[] = {SGI12172, SGI0, SGI12173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2678[] = {SGI12174, SGI12175, SGI12176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12177, SGI0, SGI0, SGI0, SGI12178, SGI12178, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2679[] = {SGI12179, SGI12180, SGI12181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12182, SGI0, SGI0, SGI0, SGI12183, SGI12183, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2680[] = {SGI12184, SGI12185, SGI12186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12187, SGI0, SGI0, SGI0, SGI12188, SGI12188, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2681[] = {SGI12189, SGI12190, SGI12191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12192, SGI0, SGI0, SGI0, SGI12193, SGI12193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2682[] = {SGI12194, SGI12195, SGI12196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12197, SGI0, SGI0, SGI0, SGI12198, SGI12198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2683[] = {SGI12199, SGI12200, SGI12201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12202, SGI0, SGI0, SGI0, SGI12203, SGI12203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2684[] = {SGI12204, SGI0, SGI12205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2685[] = {SGI12206, SGI12207, SGI12208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12209, SGI0, SGI0, SGI0, SGI12210, SGI12210, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2686[] = {SGI12211, SGI12212, SGI12213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12214, SGI0, SGI0, SGI0, SGI12215, SGI12215, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2687[] = {SGI12216, SGI12217, SGI12218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12219, SGI0, SGI0, SGI0, SGI12220, SGI12220, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2688[] = {SGI12221, SGI0, SGI12222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2689[] = {SGI12223, SGI12224, SGI12225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12226, SGI0, SGI0, SGI0, SGI12227, SGI12227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2690[] = {SGI12228, SGI12229, SGI12230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12231, SGI12232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2691[] = {SGI12233, SGI12234, SGI12235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12236, SGI0, SGI0, SGI0, SGI12237, SGI12237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2692[] = {SGI12238, SGI12239, SGI12240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12241, SGI12241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2693[] = {SGI12242, SGI12243, SGI12244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12245, SGI12245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2694[] = {SGI0, SGI12246, SGI12247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12248, SGI0, SGI0, SGI0, SGI12249, SGI12250, NULL};
const Segment * const SG2695[] = {SGI12251, SGI12252, SGI12253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12254, SGI0, SGI0, SGI0, SGI12255, SGI12255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2696[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2697[] = {SGI12257, SGI0, SGI12258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12259, SGI0, SGI0, SGI0, SGI12260, SGI12261, NULL};
const Segment * const SG2698[] = {SGI12262, SGI12263, SGI12264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12265, SGI0, SGI0, SGI0, SGI12266, SGI12267, NULL};
const Segment * const SG2699[] = {SGI12268, SGI12269, SGI12270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12271, SGI0, SGI0, SGI0, SGI12272, SGI12273, NULL};
const Segment * const SG2700[] = {SGI12274, SGI12275, SGI12276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12277, SGI0, SGI0, SGI0, SGI12278, SGI12279, NULL};
const Segment * const SG2701[] = {SGI0, SGI12280, SGI12281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12282, SGI0, SGI0, SGI0, SGI12283, SGI12284, NULL};
const Segment * const SG2702[] = {SGI12285, SGI12286, SGI12287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12288, SGI0, SGI0, SGI0, SGI12289, SGI12290, NULL};
const Segment * const SG2703[] = {SGI12291, SGI12292, SGI12293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12294, SGI0, SGI0, SGI0, SGI12295, SGI12296, NULL};
const Segment * const SG2704[] = {SGI12297, SGI12298, SGI12299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12300, SGI0, SGI0, SGI0, SGI12301, SGI12302, NULL};
const Segment * const SG2705[] = {SGI12303, SGI12304, SGI12305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12306, SGI0, SGI0, SGI0, SGI12307, SGI12308, NULL};
const Segment * const SG2706[] = {SGI12309, SGI12310, SGI12311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12312, SGI0, SGI0, SGI0, SGI12313, SGI12314, NULL};
const Segment * const SG2707[] = {SGI12315, SGI12316, SGI12317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12318, SGI0, SGI0, SGI0, SGI12319, SGI12320, NULL};
const Segment * const SG2708[] = {SGI12321, SGI12322, SGI12323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12324, SGI0, SGI0, SGI0, SGI12325, SGI12326, NULL};
const Segment * const SG2709[] = {SGI0, SGI0, SGI12327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12328, SGI0, NULL};
const Segment * const SG2710[] = {SGI12329, SGI0, SGI12330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12331, SGI0, SGI0, SGI0, SGI12332, SGI12333, NULL};
const Segment * const SG2711[] = {SGI12334, SGI12335, SGI12336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12337, SGI0, SGI0, SGI0, SGI12338, SGI12339, NULL};
const Segment * const SG2712[] = {SGI12340, SGI12341, SGI12342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12343, SGI12343, NULL};
const Segment * const SG2713[] = {SGI12344, SGI12345, SGI12346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12347, SGI0, SGI0, SGI0, SGI12348, SGI12349, NULL};
const Segment * const SG2714[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12350, SGI12351, SGI12352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2715[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12353, SGI0, SGI12354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2716[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12355, SGI12356, SGI12357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2717[] = {SGI12358, SGI12359, NULL};
const Segment * const SG2718[] = {SGI12360, SGI12361, NULL};
const Segment * const SG2719[] = {SGI12362, SGI12363, NULL};
const Segment * const SG2720[] = {SGI12364, SGI12365, NULL};
const Segment * const SG2721[] = {SGI12366, SGI12367, NULL};
const Segment * const SG2722[] = {SGI12368, SGI12369, SGI12370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12371, SGI0, SGI0, SGI0, SGI12372, SGI12373, NULL};
const Segment * const SG2723[] = {SGI0, SGI12374, NULL};
const Segment * const SG2724[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2725[] = {SGI12376, SGI12377, SGI12378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12379, SGI0, SGI0, SGI0, SGI12380, SGI12381, NULL};
const Segment * const SG2726[] = {SGI12382, SGI12383, SGI12384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12385, SGI0, SGI0, SGI0, SGI12386, SGI12387, NULL};
const Segment * const SG2727[] = {SGI12388, SGI0, SGI12389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12390, SGI0, SGI0, SGI0, SGI12391, SGI12392, NULL};
const Segment * const SG2728[] = {SGI12393, SGI12394, SGI12395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12396, SGI0, SGI0, SGI0, SGI12397, SGI12398, NULL};
const Segment * const SG2729[] = {SGI12399, SGI12400, SGI12401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12402, SGI0, SGI0, SGI0, SGI12403, SGI12403, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2730[] = {SGI12404, SGI12405, SGI12406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12407, SGI0, SGI0, SGI0, SGI12408, SGI12408, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2731[] = {SGI12409, SGI12410, SGI12411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12412, SGI0, SGI0, SGI0, SGI12413, SGI12413, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2732[] = {SGI12414, SGI12415, SGI12416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12417, SGI0, SGI0, SGI0, SGI12418, SGI12418, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2733[] = {SGI12419, SGI12420, SGI12421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12422, SGI0, SGI0, SGI0, SGI12423, SGI12423, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2734[] = {SGI12424, SGI12425, SGI12426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12427, SGI0, SGI0, SGI0, SGI12428, SGI12428, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2735[] = {SGI12429, SGI0, SGI12430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2736[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2737[] = {SGI12432, SGI12433, SGI12434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12435, SGI0, SGI0, SGI0, SGI12436, SGI12436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2738[] = {SGI12437, SGI12438, SGI12439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12440, SGI0, SGI0, SGI0, SGI12441, SGI12441, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2739[] = {SGI12442, SGI12443, SGI12444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12445, SGI0, SGI0, SGI0, SGI12446, SGI12446, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2740[] = {SGI12447, SGI12448, SGI12449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12450, SGI0, SGI0, SGI0, SGI12451, SGI12451, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2741[] = {SGI12452, SGI0, SGI12453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2742[] = {SGI12454, SGI12455, SGI12456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12457, SGI12458, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2743[] = {SGI12459, SGI12460, SGI12461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12462, SGI0, SGI0, SGI0, SGI12463, SGI12463, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2744[] = {SGI12464, SGI0, SGI12465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2745[] = {SGI12466, SGI0, SGI12467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12468, SGI0, SGI0, SGI0, SGI12469, SGI12469, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2746[] = {SGI12470, SGI12471, SGI12472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12473, SGI0, SGI0, SGI0, SGI12474, SGI12474, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2747[] = {SGI12475, SGI0, SGI12476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2748[] = {SGI12477, SGI12478, SGI12479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12480, SGI0, SGI0, SGI0, SGI12481, SGI12481, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2749[] = {SGI12482, SGI0, SGI12483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12484, SGI0, SGI0, SGI0, SGI12485, SGI12485, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2750[] = {SGI12486, SGI0, SGI12487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2751[] = {SGI12488, SGI0, SGI12489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12490, SGI0, SGI0, SGI0, SGI12491, SGI12492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2752[] = {SGI12493, SGI12494, SGI12495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12496, SGI0, SGI0, SGI0, SGI12497, SGI12498, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2753[] = {SGI12499, SGI12500, SGI12501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12502, SGI0, SGI0, SGI0, SGI12503, SGI12503, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2754[] = {SGI12504, SGI0, SGI12505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2755[] = {SGI12506, SGI12507, SGI12508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12509, SGI0, SGI0, SGI0, SGI12510, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2756[] = {SGI12511, SGI12512, SGI12513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12514, SGI0, SGI0, SGI0, SGI12515, SGI12515, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2757[] = {SGI12516, SGI0, SGI12517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2758[] = {SGI12518, SGI12519, SGI12520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12521, SGI0, SGI0, SGI0, SGI12522, SGI12522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2759[] = {SGI12523, SGI12524, SGI12525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12526, SGI0, SGI0, SGI0, SGI12527, SGI12527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2760[] = {SGI12528, SGI12529, SGI12530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12531, SGI0, SGI0, SGI0, SGI12532, SGI12532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2761[] = {SGI12533, SGI12534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12535, SGI0, SGI0, SGI0, SGI12536, SGI12536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2762[] = {SGI12537, SGI12538, SGI12539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12540, SGI0, SGI0, SGI0, SGI12541, SGI12541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2763[] = {SGI12542, SGI0, SGI12543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2764[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12544, SGI12545, SGI12546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2765[] = {SGI12547, SGI12548, SGI12549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12550, SGI0, SGI0, SGI0, SGI12551, SGI12551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2766[] = {SGI12552, SGI0, SGI12553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2767[] = {SGI12554, SGI12555, SGI12556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12557, SGI0, SGI0, SGI0, SGI12558, SGI12558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2768[] = {SGI12559, SGI12560, SGI12561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12562, SGI0, SGI0, SGI0, SGI12563, SGI12563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2769[] = {SGI12564, SGI12565, SGI12566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12567, SGI0, SGI0, SGI0, SGI12568, SGI12568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2770[] = {SGI12569, SGI12570, SGI12571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12572, SGI0, SGI0, SGI0, SGI12573, SGI12573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2771[] = {SGI12574, SGI0, SGI12575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2772[] = {SGI12576, SGI12577, SGI12578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12579, SGI0, SGI0, SGI0, SGI12580, SGI12580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2773[] = {SGI0, SGI12581, NULL};
const Segment * const SG2774[] = {SGI0, SGI12582, NULL};
const Segment * const SG2775[] = {SGI12583, SGI12584, SGI12585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12586, SGI0, SGI0, SGI0, SGI12587, SGI12587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2776[] = {SGI12588, SGI12589, SGI12590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12591, SGI0, SGI0, SGI0, SGI12592, SGI12592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2777[] = {SGI12593, SGI12594, SGI12595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12596, SGI0, SGI0, SGI0, SGI12597, SGI12597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2778[] = {SGI12598, SGI12599, SGI12600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12601, SGI0, SGI0, SGI0, SGI12602, SGI12602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2779[] = {SGI12603, SGI0, SGI12604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12605, SGI0, SGI0, SGI0, SGI12606, SGI12606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2780[] = {SGI12607, SGI0, SGI12608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2781[] = {SGI12609, SGI12610, SGI12611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12612, SGI0, SGI0, SGI0, SGI12613, SGI12613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2782[] = {SGI12614, SGI12615, SGI12616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12617, SGI0, SGI0, SGI0, SGI12618, SGI12619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2783[] = {SGI12620, SGI0, SGI12621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2784[] = {SGI12622, SGI12623, SGI12624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12625, SGI0, SGI0, SGI0, SGI12626, SGI12626, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2785[] = {SGI12627, SGI12628, SGI12629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12630, SGI0, SGI0, SGI0, SGI12631, SGI12631, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2786[] = {SGI12632, SGI0, SGI12633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2787[] = {SGI12634, SGI12635, SGI12636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12637, SGI0, SGI0, SGI0, SGI12638, SGI12638, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2788[] = {SGI12639, SGI12640, SGI12641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12642, SGI0, SGI0, SGI0, SGI12643, SGI12644, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2789[] = {SGI12645, SGI12646, SGI12647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12648, SGI0, SGI0, SGI0, SGI12649, SGI12649, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2790[] = {SGI12650, SGI12651, SGI12652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12653, SGI0, SGI0, SGI0, SGI12654, SGI12654, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2791[] = {SGI12655, SGI12656, SGI12657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12658, SGI0, SGI0, SGI0, SGI12659, SGI12659, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2792[] = {SGI12660, SGI12661, SGI12662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12663, SGI0, SGI0, SGI0, SGI12664, SGI12664, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2793[] = {SGI12665, SGI12666, SGI12667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12668, SGI0, SGI0, SGI0, SGI12669, SGI12669, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2794[] = {SGI12670, SGI12671, SGI12672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12673, SGI0, SGI0, SGI0, SGI12674, SGI12674, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2795[] = {SGI12675, SGI0, SGI12676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2796[] = {SGI0, SGI12677, SGI12678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12679, SGI0, SGI0, SGI0, SGI12680, SGI12680, SGI0, NULL};
const Segment * const SG2797[] = {SGI12681, SGI12682, SGI12683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12684, SGI0, SGI0, SGI0, SGI12685, SGI12685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2798[] = {SGI12686, SGI12687, SGI12688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12689, SGI0, SGI0, SGI0, SGI12690, SGI12690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2799[] = {SGI12691, SGI0, SGI12692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2800[] = {SGI12693, SGI12694, SGI12695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12696, SGI0, SGI0, SGI12697, SGI12698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2801[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12699, SGI0, SGI12700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2802[] = {SGI0, SGI12701, SGI0, SGI0, SGI0, SGI0, SGI12703, SGI0, SGI12705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2803[] = {SGI12702, SGI12704, SGI12706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12707, SGI0, SGI0, SGI0, SGI12708, SGI12708, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2804[] = {SGI12709, SGI12710, SGI12711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12712, SGI0, SGI0, SGI0, SGI12713, SGI12713, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2805[] = {SGI12714, SGI0, SGI12715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2806[] = {SGI12716, SGI12717, SGI12718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12719, SGI0, SGI0, SGI0, SGI12720, SGI12720, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2807[] = {SGI12721, SGI12722, SGI12723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12724, SGI0, SGI0, SGI0, SGI12725, SGI12725, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2808[] = {SGI12726, SGI12727, SGI12728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12729, SGI0, SGI0, SGI0, SGI12730, SGI12730, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2809[] = {SGI12731, SGI0, SGI12732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2810[] = {SGI12733, SGI12734, SGI12735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12736, SGI12737, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2811[] = {SGI12738, SGI12739, SGI12740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12741, SGI0, SGI0, SGI0, SGI12742, SGI12742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2812[] = {SGI12743, SGI0, SGI12744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2813[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12745, SGI0, SGI0, SGI12746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2814[] = {SGI12747, SGI12748, SGI12749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12750, SGI0, SGI0, SGI0, SGI12751, SGI12751, NULL};
const Segment * const SG2815[] = {SGI12752, SGI12753, NULL};
const Segment * const SG2816[] = {SGI12754, SGI12755, NULL};
const Segment * const SG2817[] = {SGI12756, SGI12757, NULL};
const Segment * const SG2818[] = {SGI12758, SGI12759, NULL};
const Segment * const SG2819[] = {SGI12760, SGI12761, SGI12762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12763, SGI0, SGI0, SGI0, SGI12764, SGI12764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2820[] = {SGI12765, SGI12766, SGI12767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12768, SGI0, SGI0, SGI0, SGI12769, SGI12770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2821[] = {SGI12771, SGI0, SGI12772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2822[] = {SGI0, SGI12773, SGI12774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2823[] = {SGI0, SGI12775, SGI12776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12777, SGI0, SGI0, SGI0, SGI12778, SGI12779, NULL};
const Segment * const SG2824[] = {SGI0, SGI12780, SGI12781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12782, SGI0, SGI0, SGI0, SGI12783, SGI12784, NULL};
const Segment * const SG2825[] = {SGI12785, SGI12786, SGI12787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12788, SGI0, SGI0, SGI0, SGI12789, SGI12789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2826[] = {SGI12790, SGI12791, SGI12792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12793, SGI0, SGI0, SGI0, SGI12794, SGI12794, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2827[] = {SGI12795, SGI12796, SGI12797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12798, SGI0, SGI0, SGI0, SGI12799, SGI12799, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2828[] = {SGI12800, SGI12801, SGI12802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12803, SGI0, SGI0, SGI0, SGI12804, SGI12804, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2829[] = {SGI12805, SGI0, SGI12806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2830[] = {SGI12807, SGI12808, SGI12809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12810, SGI0, SGI0, SGI0, SGI12811, SGI12811, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2831[] = {SGI12812, SGI0, SGI12813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2832[] = {SGI12814, SGI12815, SGI12816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12817, SGI0, SGI0, SGI0, SGI12818, SGI12818, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2833[] = {SGI12819, SGI12820, SGI12821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12822, SGI0, SGI0, SGI0, SGI12823, SGI12823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2834[] = {SGI12824, SGI12825, SGI12826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12827, SGI0, SGI0, SGI0, SGI12828, SGI12828, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2835[] = {SGI12829, SGI12830, SGI12831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12832, SGI0, SGI0, SGI0, SGI12833, SGI12833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2836[] = {SGI12834, SGI12835, SGI12836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12837, SGI0, SGI0, SGI0, SGI12838, SGI12838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2837[] = {SGI12839, SGI12840, SGI12841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12842, SGI0, SGI0, SGI0, SGI12843, SGI12843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2838[] = {SGI12844, SGI12845, SGI12846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12847, SGI0, SGI0, SGI0, SGI12848, SGI12848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2839[] = {SGI12849, SGI12850, SGI12851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12852, SGI0, SGI0, SGI0, SGI12853, SGI12853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2840[] = {SGI12854, SGI12855, SGI12856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12857, SGI0, SGI0, SGI0, SGI12858, SGI12858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2841[] = {SGI12859, SGI12860, SGI12861, SGI0, SGI0, SGI0, SGI12862, SGI12863, SGI0, SGI12864, SGI0, SGI0, SGI0, SGI12865, SGI12865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2842[] = {SGI12866, SGI12867, SGI12868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12869, SGI0, SGI0, SGI0, SGI12870, SGI12870, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2843[] = {SGI12871, SGI12872, SGI12873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12874, SGI0, SGI0, SGI0, SGI12875, SGI12875, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2844[] = {SGI12876, SGI0, SGI12877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12878, SGI0, SGI0, SGI0, SGI12879, SGI12879, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2845[] = {SGI12880, SGI12881, SGI12882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12883, SGI0, SGI0, SGI0, SGI12884, SGI12885, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2846[] = {SGI12886, SGI0, SGI12887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2847[] = {SGI12888, SGI12889, SGI12890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12891, SGI0, SGI0, SGI0, SGI12892, SGI12892, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2848[] = {SGI12893, SGI12894, SGI12895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12896, SGI0, SGI0, SGI0, SGI12897, SGI12897, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2849[] = {SGI12898, SGI0, SGI12899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2850[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12900, SGI0, SGI0, SGI12901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2851[] = {SGI12902, SGI12903, SGI12904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12905, SGI0, SGI0, SGI0, SGI12906, SGI12906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2852[] = {SGI12907, SGI12908, SGI12909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12910, SGI0, SGI0, SGI0, SGI12911, SGI12911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2853[] = {SGI12912, SGI12913, SGI12914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12915, SGI0, SGI0, SGI0, SGI12916, SGI12916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2854[] = {SGI12917, SGI12918, SGI12919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12920, SGI0, SGI0, SGI0, SGI12921, SGI12921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2855[] = {SGI12922, SGI12923, SGI12924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12925, SGI0, SGI0, SGI0, SGI12926, SGI12926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2856[] = {SGI12927, SGI12928, SGI12929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12930, SGI0, SGI0, SGI0, SGI12931, SGI12931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2857[] = {SGI12932, SGI12933, SGI12934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12935, SGI0, SGI0, SGI0, SGI12936, SGI12936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2858[] = {SGI12937, SGI12938, SGI12939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12940, SGI0, SGI0, SGI0, SGI12941, SGI12941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2859[] = {SGI12942, SGI12943, SGI12944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12945, SGI0, SGI0, SGI0, SGI12946, SGI12946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2860[] = {SGI12947, SGI12948, SGI12949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12950, SGI0, SGI0, SGI0, SGI12951, SGI12951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2861[] = {SGI12952, SGI12953, SGI12954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12955, SGI0, SGI0, SGI0, SGI12956, SGI12956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2862[] = {SGI12957, SGI12958, SGI12959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12960, SGI0, SGI0, SGI0, SGI12961, SGI12961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2863[] = {SGI12962, SGI0, SGI12963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2864[] = {SGI12964, SGI0, SGI12965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12966, SGI0, SGI0, SGI0, SGI12967, SGI12967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2865[] = {SGI12968, SGI0, SGI12969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2866[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2867[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2868[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2869[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12973, SGI12974, SGI12975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2870[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12976, SGI12977, SGI12978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2871[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12979, SGI12980, SGI12981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2872[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12982, SGI12983, SGI12984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2873[] = {SGI12985, SGI12986, SGI12987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12988, SGI0, SGI0, SGI0, SGI12989, SGI12990, NULL};
const Segment * const SG2874[] = {SGI12991, SGI12992, SGI12993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12994, SGI0, SGI0, SGI0, SGI0, SGI12995, NULL};
const Segment * const SG2875[] = {SGI12996, SGI12997, SGI12998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI12999, SGI0, SGI0, SGI0, SGI13000, SGI13000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2876[] = {SGI13001, SGI0, SGI13002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2877[] = {SGI13003, SGI13004, SGI13005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13006, SGI0, SGI0, SGI0, SGI13007, SGI13007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2878[] = {SGI13008, SGI13009, SGI13010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13011, SGI13012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2879[] = {SGI13013, SGI0, SGI13014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2880[] = {SGI13015, SGI13016, SGI13017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13018, SGI0, SGI0, SGI0, SGI13019, SGI13019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2881[] = {SGI13020, SGI0, SGI13021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13022, SGI0, SGI0, SGI0, SGI13023, SGI13023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2882[] = {SGI13024, SGI0, SGI13025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2883[] = {SGI13026, SGI13027, SGI13028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13029, SGI0, SGI0, SGI0, SGI13030, SGI13030, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2884[] = {SGI13031, SGI13032, SGI13033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13034, SGI0, SGI0, SGI0, SGI13035, SGI13035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2885[] = {SGI13036, SGI0, SGI13037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2886[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13038, SGI0, SGI0, SGI13039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2887[] = {SGI13040, SGI13041, SGI13042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13043, SGI0, SGI0, SGI0, SGI13044, SGI13044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2888[] = {SGI13045, SGI13046, SGI13047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13048, SGI0, SGI0, SGI0, SGI13049, SGI13049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2889[] = {SGI13050, SGI13051, SGI13052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13053, SGI0, SGI0, SGI0, SGI13054, SGI13055, NULL};
const Segment * const SG2890[] = {SGI13056, SGI13057, SGI13058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13059, SGI0, SGI0, SGI0, SGI13060, SGI13060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2891[] = {SGI13061, SGI13062, SGI13063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13064, SGI0, SGI0, SGI0, SGI13065, SGI13065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2892[] = {SGI13066, SGI0, SGI13067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2893[] = {SGI13068, SGI0, SGI13069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13070, SGI0, SGI0, SGI0, SGI13071, SGI13071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2894[] = {SGI13072, SGI13073, SGI13074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13075, SGI0, SGI0, SGI0, SGI13076, SGI13076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2895[] = {SGI13077, SGI13078, SGI13079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13080, SGI0, SGI0, SGI0, SGI13081, SGI13081, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2896[] = {SGI13082, SGI13083, SGI13084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13085, SGI0, SGI0, SGI0, SGI13086, SGI13086, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2897[] = {SGI13087, SGI13088, SGI13089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13090, SGI0, SGI0, SGI0, SGI13091, SGI13092, NULL};
const Segment * const SG2898[] = {SGI13093, SGI13094, SGI13095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13096, SGI0, SGI0, SGI0, SGI13097, SGI13098, NULL};
const Segment * const SG2899[] = {SGI13099, SGI13100, SGI13101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13102, SGI0, SGI0, SGI0, SGI13103, SGI13104, NULL};
const Segment * const SG2900[] = {SGI13105, SGI13106, SGI13107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13108, SGI0, SGI0, SGI0, SGI13109, SGI13110, NULL};
const Segment * const SG2901[] = {SGI13111, SGI0, SGI13112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13113, SGI0, SGI0, SGI0, SGI13114, SGI13115, NULL};
const Segment * const SG2902[] = {SGI13116, SGI13117, SGI13118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13119, SGI0, SGI0, SGI0, SGI13120, SGI0, NULL};
const Segment * const SG2903[] = {SGI13121, SGI13122, SGI13123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13124, SGI0, SGI0, SGI0, SGI13125, SGI13125, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2904[] = {SGI13126, SGI13127, SGI13128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13129, SGI0, SGI0, SGI0, SGI13130, SGI13131, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2905[] = {SGI13132, SGI0, SGI13133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2906[] = {SGI13134, SGI13135, SGI13136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13137, SGI0, SGI0, SGI0, SGI13138, SGI13138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2907[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13139, SGI0, SGI0, SGI13140, SGI13140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2908[] = {SGI13141, SGI13142, SGI13143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13144, SGI0, SGI0, SGI0, SGI13145, SGI13145, NULL};
const Segment * const SG2909[] = {SGI13146, SGI13147, SGI13148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13149, SGI0, SGI0, SGI0, SGI13150, SGI13150, NULL};
const Segment * const SG2910[] = {SGI13151, SGI13152, SGI13153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13154, SGI0, SGI0, SGI0, SGI13155, SGI13155, SGI0, NULL};
const Segment * const SG2911[] = {SGI13156, SGI13157, SGI13158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13159, SGI0, SGI0, SGI0, SGI13160, SGI13160, SGI0, NULL};
const Segment * const SG2912[] = {SGI13161, SGI13162, SGI13163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13164, SGI0, SGI0, SGI0, SGI13165, SGI13165, SGI0, NULL};
const Segment * const SG2913[] = {SGI13166, SGI13167, SGI13168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13169, SGI0, SGI0, SGI0, SGI13170, SGI13170, SGI0, NULL};
const Segment * const SG2914[] = {SGI13171, SGI13172, SGI13173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13174, SGI0, SGI0, SGI0, SGI13175, SGI13175, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2915[] = {SGI0, SGI13176, SGI13177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13178, SGI0, SGI0, SGI0, SGI13179, SGI13179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2916[] = {SGI13180, SGI13181, SGI13182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13183, SGI0, SGI0, SGI0, SGI13184, SGI13184, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2917[] = {SGI13185, SGI0, SGI13186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13187, SGI0, SGI0, SGI13188, SGI13188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2918[] = {SGI13189, SGI13190, SGI13191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13192, SGI13193, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2919[] = {SGI13194, SGI0, SGI13195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13196, SGI0, SGI0, SGI13197, SGI13197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2920[] = {SGI13198, SGI0, SGI13199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2921[] = {SGI13200, SGI0, SGI13201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13202, SGI0, SGI0, SGI13203, SGI13203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2922[] = {SGI13204, SGI0, SGI13205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13206, SGI0, SGI0, SGI13207, SGI13207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2923[] = {SGI13208, SGI0, SGI13209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13210, SGI0, SGI0, SGI13211, SGI13211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2924[] = {SGI13212, SGI0, SGI13213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13214, SGI0, SGI0, SGI13215, SGI13215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2925[] = {SGI13216, SGI0, SGI13217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13218, SGI0, SGI0, SGI13219, SGI13219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2926[] = {SGI13220, SGI0, SGI13221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13222, SGI0, SGI0, SGI13223, SGI13223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2927[] = {SGI13224, SGI0, SGI13225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13226, SGI0, SGI0, SGI13227, SGI13227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2928[] = {SGI13228, SGI0, SGI13229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13230, SGI0, SGI0, SGI13231, SGI13231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2929[] = {SGI13232, SGI13233, SGI13234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13235, SGI0, SGI0, SGI0, SGI13236, SGI13236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2930[] = {SGI13237, SGI13238, SGI13239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13240, SGI0, SGI0, SGI0, SGI13241, SGI13241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2931[] = {SGI13242, SGI13243, SGI13244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13245, SGI13246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2932[] = {SGI13247, SGI13248, SGI13249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13250, SGI0, SGI0, SGI0, SGI0, SGI13251, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2933[] = {SGI13252, SGI13253, SGI13254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13255, SGI0, SGI0, SGI0, SGI13256, SGI13256, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2934[] = {SGI13257, SGI13258, SGI13259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13260, SGI0, SGI0, SGI0, SGI13261, SGI13261, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2935[] = {SGI13262, SGI13263, SGI13264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13265, SGI0, SGI0, SGI0, SGI13266, SGI13266, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2936[] = {SGI13267, SGI13268, SGI13269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13270, SGI0, SGI0, SGI0, SGI13271, SGI13271, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2937[] = {SGI13272, SGI13273, SGI13274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13275, SGI0, SGI0, SGI0, SGI13276, SGI13277, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2938[] = {SGI13278, SGI0, SGI13279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2939[] = {SGI13280, SGI13281, SGI13282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13283, SGI13284, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2940[] = {SGI13285, SGI13286, SGI13287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13288, SGI0, SGI0, SGI0, SGI13289, SGI13289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2941[] = {SGI13290, SGI0, SGI13291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2942[] = {SGI13292, SGI13293, NULL};
const Segment * const SG2943[] = {SGI13294, SGI13295, NULL};
const Segment * const SG2944[] = {SGI13296, SGI13297, NULL};
const Segment * const SG2945[] = {SGI13298, SGI13299, NULL};
const Segment * const SG2946[] = {SGI13300, SGI13301, NULL};
const Segment * const SG2947[] = {SGI0, SGI13302, SGI0, SGI13303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13304, SGI0, SGI0, SGI0, SGI13305, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2948[] = {SGI0, SGI13306, SGI0, SGI13307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2949[] = {SGI13308, SGI13309, SGI13310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13311, SGI0, SGI0, SGI0, SGI13312, SGI13312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2950[] = {SGI13313, SGI13314, SGI13315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13316, SGI0, SGI0, SGI0, SGI13317, SGI13317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2951[] = {SGI13318, SGI13319, SGI13320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13321, SGI0, SGI0, SGI0, SGI13322, SGI13322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2952[] = {SGI13323, SGI13324, SGI13325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13326, SGI0, SGI0, SGI0, SGI13327, SGI13327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2953[] = {SGI13328, SGI13329, SGI13330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13331, SGI0, SGI0, SGI0, SGI13332, SGI13332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2954[] = {SGI13333, SGI13334, SGI13335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13336, SGI0, SGI0, SGI0, SGI13337, SGI13337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2955[] = {SGI13338, SGI13339, SGI13340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13341, SGI0, SGI0, SGI0, SGI13342, SGI13342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2956[] = {SGI13343, SGI0, SGI13344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2957[] = {SGI13345, SGI13346, SGI13347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13348, SGI0, SGI0, SGI0, SGI13349, SGI13349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2958[] = {SGI13350, SGI0, SGI13351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2959[] = {SGI13352, SGI13353, SGI13354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13355, SGI0, SGI0, SGI0, SGI13356, SGI13356, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2960[] = {SGI13358, SGI0, SGI13360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2961[] = {SGI13357, SGI13359, SGI13361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13362, SGI0, SGI0, SGI0, SGI13363, SGI13363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2962[] = {SGI13364, SGI13365, SGI13366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13367, SGI0, SGI0, SGI0, SGI13368, SGI13368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2963[] = {SGI13369, SGI13370, SGI13371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13372, SGI0, SGI0, SGI0, SGI13373, SGI13373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2964[] = {SGI13374, SGI13375, SGI13376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13377, SGI0, SGI0, SGI0, SGI13378, SGI13378, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2965[] = {SGI13379, SGI13380, SGI13381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13382, SGI0, SGI0, SGI0, SGI13383, SGI13383, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2966[] = {SGI13384, SGI0, SGI13385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2967[] = {SGI13386, SGI13387, SGI13388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13389, SGI0, SGI0, SGI0, SGI13390, SGI13391, NULL};
const Segment * const SG2968[] = {SGI13392, SGI13393, SGI13394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13395, SGI0, SGI0, SGI0, SGI13396, SGI13397, NULL};
const Segment * const SG2969[] = {SGI13398, SGI13399, SGI13400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13401, SGI0, SGI0, SGI0, SGI13402, SGI13403, NULL};
const Segment * const SG2970[] = {SGI13404, SGI13405, SGI13406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13407, SGI0, SGI0, SGI0, SGI13408, SGI13409, NULL};
const Segment * const SG2971[] = {SGI13410, SGI13411, SGI13412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13413, SGI0, SGI0, SGI0, SGI13414, SGI13415, NULL};
const Segment * const SG2972[] = {SGI13416, SGI13417, SGI13418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13419, SGI0, SGI0, SGI0, SGI13420, SGI13421, NULL};
const Segment * const SG2973[] = {SGI13422, SGI13423, SGI13424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13425, SGI0, SGI0, SGI0, SGI13426, SGI13427, NULL};
const Segment * const SG2974[] = {SGI13428, SGI13429, SGI13430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13431, SGI0, SGI0, SGI0, SGI13432, SGI13432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2975[] = {SGI13433, SGI13434, SGI13435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13436, SGI0, SGI0, SGI0, SGI13437, SGI13437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2976[] = {SGI13438, SGI0, SGI13439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2977[] = {SGI13440, SGI13441, SGI13442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13443, SGI0, SGI0, SGI0, SGI13444, SGI13444, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2978[] = {SGI13445, SGI13446, SGI13447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13448, SGI0, SGI0, SGI0, SGI13449, SGI13449, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2979[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2980[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13454, SGI13455, SGI13457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2981[] = {SGI13451, SGI13452, SGI13453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13456, SGI0, SGI0, SGI0, SGI13458, SGI13458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2982[] = {SGI13459, SGI13460, SGI13461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13462, SGI0, SGI0, SGI0, SGI13463, SGI13463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2983[] = {SGI13464, SGI13465, SGI13466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13467, SGI0, SGI0, SGI0, SGI13468, SGI13468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2984[] = {SGI0, SGI13469, SGI0, SGI13470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2985[] = {SGI0, SGI13471, SGI0, SGI13472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13473, SGI0, SGI0, SGI0, SGI13474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2986[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2987[] = {SGI0, SGI0, SGI13476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13477, SGI0, SGI0, SGI0, SGI13478, SGI13479, NULL};
const Segment * const SG2988[] = {SGI13480, SGI0, SGI13481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13482, SGI0, SGI0, SGI0, SGI13483, SGI13484, NULL};
const Segment * const SG2989[] = {SGI13485, SGI13486, SGI13487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13488, SGI0, SGI0, SGI0, SGI13489, SGI13490, NULL};
const Segment * const SG2990[] = {SGI13491, SGI13492, SGI13493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13494, SGI0, SGI0, SGI0, SGI13495, SGI13495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2991[] = {SGI13496, SGI13497, SGI13498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13499, SGI0, SGI0, SGI0, SGI13500, SGI13500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2992[] = {SGI13501, SGI13502, NULL};
const Segment * const SG2993[] = {SGI13503, SGI13504, NULL};
const Segment * const SG2994[] = {SGI13505, SGI13506, NULL};
const Segment * const SG2995[] = {SGI13507, SGI13508, NULL};
const Segment * const SG2996[] = {SGI13509, SGI13510, SGI13511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13512, SGI0, SGI0, SGI0, SGI13513, SGI13513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2997[] = {SGI13514, SGI0, SGI13515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2998[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG2999[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13517, SGI0, SGI0, SGI13518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3000[] = {SGI13519, SGI13520, SGI13521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13522, SGI0, SGI0, SGI0, SGI13523, SGI13523, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3001[] = {SGI13524, SGI13525, SGI13526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13527, SGI0, SGI0, SGI0, SGI13528, SGI13528, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3002[] = {SGI13529, SGI13530, SGI13531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13532, SGI0, SGI0, SGI0, SGI13533, SGI13533, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3003[] = {SGI13534, SGI13535, SGI13536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13537, SGI0, SGI0, SGI0, SGI13538, SGI13538, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3004[] = {SGI13539, SGI0, SGI13540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3005[] = {SGI13541, SGI13542, SGI13543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13544, SGI0, SGI0, SGI0, SGI13545, SGI13545, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3006[] = {SGI13546, SGI13547, SGI13548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13549, SGI0, SGI0, SGI0, SGI13550, SGI13551, NULL};
const Segment * const SG3007[] = {SGI13552, SGI13553, SGI13554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13555, SGI0, SGI0, SGI0, SGI13556, SGI0, NULL};
const Segment * const SG3008[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3009[] = {SGI13558, SGI13559, SGI13560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13561, SGI0, SGI0, SGI0, SGI13562, SGI13562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3010[] = {SGI13563, SGI0, SGI13564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3011[] = {SGI13565, SGI13566, SGI13567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13568, SGI0, SGI0, SGI0, SGI13569, SGI13569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3012[] = {SGI13570, SGI13571, SGI13572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13573, SGI0, SGI0, SGI0, SGI13574, SGI13574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3013[] = {SGI13575, SGI0, SGI13576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3014[] = {SGI13577, SGI13578, SGI13579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13580, SGI0, SGI0, SGI0, SGI13581, SGI13582, NULL};
const Segment * const SG3015[] = {SGI13583, SGI13584, SGI13585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13586, SGI0, SGI0, SGI0, SGI13587, SGI13587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3016[] = {SGI13588, SGI0, SGI13589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3017[] = {SGI13590, SGI13591, SGI13592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13593, SGI0, SGI0, SGI0, SGI13594, SGI13594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3018[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13595, SGI0, SGI0, SGI13596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3019[] = {SGI13597, SGI13598, SGI13599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13600, SGI0, SGI0, SGI0, SGI13601, SGI13601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3020[] = {SGI13602, SGI13603, SGI13604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13605, SGI0, SGI0, SGI0, SGI13606, SGI13606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3021[] = {SGI13607, SGI13608, SGI13609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13610, SGI0, SGI0, SGI0, SGI13611, SGI13611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3022[] = {SGI13612, SGI13613, SGI13614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13615, SGI0, SGI0, SGI0, SGI13616, SGI13617, NULL};
const Segment * const SG3023[] = {SGI13618, SGI13619, SGI13620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13621, SGI0, SGI0, SGI0, SGI13622, SGI13623, NULL};
const Segment * const SG3024[] = {SGI13624, SGI13625, SGI13626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13627, SGI0, SGI0, SGI0, SGI13628, SGI13629, NULL};
const Segment * const SG3025[] = {SGI13630, SGI13631, SGI13632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13633, SGI0, SGI0, SGI0, SGI13634, SGI13635, NULL};
const Segment * const SG3026[] = {SGI13636, SGI13637, SGI13638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13639, SGI0, SGI0, SGI0, SGI13640, SGI13640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3027[] = {SGI13641, SGI13642, SGI13643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13644, SGI0, SGI0, SGI0, SGI13645, SGI13645, NULL};
const Segment * const SG3028[] = {SGI13646, SGI13647, SGI13648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13649, SGI0, SGI0, SGI0, SGI13650, SGI13650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3029[] = {SGI13651, SGI0, SGI13652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3030[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3031[] = {SGI13654, SGI13655, SGI13656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13657, SGI0, SGI0, SGI0, SGI13658, SGI13658, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3032[] = {SGI0, SGI13659, SGI13660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13661, SGI0, SGI0, SGI0, SGI13662, SGI13662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3033[] = {SGI13663, SGI13664, SGI13665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13666, SGI0, SGI0, SGI0, SGI13667, SGI13668, NULL};
const Segment * const SG3034[] = {SGI13669, SGI13670, SGI13671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13672, SGI0, SGI0, SGI0, SGI13673, SGI13674, NULL};
const Segment * const SG3035[] = {SGI13675, SGI13676, SGI13677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13678, SGI0, SGI0, SGI0, SGI13679, SGI13680, NULL};
const Segment * const SG3036[] = {SGI13681, SGI13682, SGI13683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13684, SGI0, SGI0, SGI0, SGI13685, SGI13686, NULL};
const Segment * const SG3037[] = {SGI0, SGI13687, SGI13688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13689, SGI0, SGI0, SGI0, SGI13690, SGI13691, NULL};
const Segment * const SG3038[] = {SGI13692, SGI13693, SGI13694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13695, SGI0, SGI0, SGI0, SGI13696, SGI13697, NULL};
const Segment * const SG3039[] = {SGI13698, SGI13699, SGI13700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13701, SGI0, SGI0, SGI0, SGI13702, SGI13702, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3040[] = {SGI13703, SGI13704, SGI13705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13706, SGI0, SGI0, SGI0, SGI13707, SGI13707, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3041[] = {SGI13708, SGI13709, SGI13710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13711, SGI0, SGI0, SGI0, SGI13712, SGI13712, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3042[] = {SGI13713, SGI13714, SGI13715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13716, SGI0, SGI0, SGI0, SGI13717, SGI13717, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3043[] = {SGI13718, SGI0, SGI13719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3044[] = {SGI13720, SGI13721, SGI13722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13723, SGI13724, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3045[] = {SGI13725, SGI13726, SGI13727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13728, SGI0, SGI0, SGI0, SGI13729, SGI13729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3046[] = {SGI13730, SGI13731, SGI13732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13733, SGI0, SGI0, SGI0, SGI13734, SGI13734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3047[] = {SGI13735, SGI13736, SGI13737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13738, SGI0, SGI0, SGI0, SGI13739, SGI13739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3048[] = {SGI13740, SGI13741, SGI13742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13743, SGI0, SGI0, SGI0, SGI13744, SGI13744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3049[] = {SGI13745, SGI13746, SGI13747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13748, SGI0, SGI0, SGI0, SGI13749, SGI13749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3050[] = {SGI13750, SGI13751, SGI13752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13753, SGI0, SGI0, SGI0, SGI13754, SGI13754, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3051[] = {SGI13755, SGI13756, SGI13757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13758, SGI0, SGI0, SGI0, SGI13759, SGI13759, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3052[] = {SGI13760, SGI0, SGI13761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3053[] = {SGI13762, SGI13763, SGI13764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13765, SGI0, SGI0, SGI0, SGI13766, SGI13766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3054[] = {SGI13767, SGI13768, SGI13769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13770, SGI0, SGI0, SGI0, SGI13771, SGI13771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3055[] = {SGI13772, SGI0, SGI13773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3056[] = {SGI13774, SGI13775, SGI13776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13777, SGI0, SGI0, SGI0, SGI13778, SGI13778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3057[] = {SGI13779, SGI0, SGI13780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3058[] = {SGI13781, SGI0, SGI13782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13783, SGI0, SGI0, SGI0, SGI13784, SGI13784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3059[] = {SGI13785, SGI13786, SGI13787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13788, SGI0, SGI0, SGI0, SGI13789, SGI13789, SGI0, NULL};
const Segment * const SG3060[] = {SGI13790, SGI13791, SGI13792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13793, SGI0, SGI0, SGI0, SGI13794, SGI13794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3061[] = {SGI13795, SGI0, SGI13796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3062[] = {SGI13797, SGI13798, SGI13799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13800, SGI0, SGI0, SGI0, SGI13801, SGI13801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3063[] = {SGI13802, SGI13803, SGI13804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13805, SGI13806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3064[] = {SGI13807, SGI13808, SGI13809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13810, SGI13810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3065[] = {SGI13811, SGI13812, SGI13813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13814, SGI0, SGI0, SGI0, SGI13815, SGI13815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3066[] = {SGI13816, SGI0, SGI13817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3067[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3068[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3069[] = {SGI13820, SGI13821, SGI13822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13823, SGI0, SGI0, SGI0, SGI13824, SGI13824, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3070[] = {SGI13825, SGI0, SGI13826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3071[] = {SGI13827, SGI13828, SGI13829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13830, SGI0, SGI0, SGI0, SGI13831, SGI13831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3072[] = {SGI13832, SGI0, SGI13833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3073[] = {SGI13834, SGI0, SGI13835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13836, SGI0, SGI0, SGI0, SGI13837, SGI13837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3074[] = {SGI13838, SGI13839, SGI13840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13841, SGI0, SGI0, SGI0, SGI13842, SGI13842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3075[] = {SGI13843, SGI0, SGI13844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3076[] = {SGI13845, SGI13846, SGI13847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13848, SGI0, SGI0, SGI0, SGI13849, SGI13849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3077[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3078[] = {SGI13851, SGI13852, SGI13853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13854, SGI0, SGI0, SGI0, SGI13855, SGI0, NULL};
const Segment * const SG3079[] = {SGI13856, SGI13857, SGI13858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13859, SGI0, SGI0, SGI0, SGI13860, SGI13860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3080[] = {SGI13861, SGI0, SGI13862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13863, SGI0, SGI0, SGI0, SGI13864, SGI13864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3081[] = {SGI13865, SGI0, SGI13866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3082[] = {SGI13867, SGI13868, NULL};
const Segment * const SG3083[] = {SGI13869, SGI13870, SGI13871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13872, SGI0, SGI0, SGI0, SGI13873, SGI13874, NULL};
const Segment * const SG3084[] = {SGI13875, SGI13876, SGI13877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13878, SGI0, SGI0, SGI0, SGI13879, SGI13880, NULL};
const Segment * const SG3085[] = {SGI13881, SGI13882, SGI13883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13884, SGI0, SGI0, SGI0, SGI13885, SGI13885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3086[] = {SGI13886, SGI13887, SGI13888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13889, SGI13890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3087[] = {SGI13891, SGI13892, SGI13893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13894, SGI0, SGI0, SGI0, SGI13895, SGI13895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3088[] = {SGI13896, SGI13897, NULL};
const Segment * const SG3089[] = {SGI13898, SGI13899, NULL};
const Segment * const SG3090[] = {SGI13900, SGI13901, NULL};
const Segment * const SG3091[] = {SGI13902, SGI13903, NULL};
const Segment * const SG3092[] = {SGI13904, SGI13905, SGI13906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13907, SGI0, SGI0, SGI13908, SGI13908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3093[] = {SGI13909, SGI13910, SGI13911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13912, SGI0, SGI0, SGI0, SGI13913, SGI13913, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3094[] = {SGI13914, SGI13915, SGI13916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13917, SGI0, SGI0, SGI0, SGI13918, SGI13918, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3095[] = {SGI13919, SGI13920, SGI13921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13922, SGI0, SGI0, SGI0, SGI13923, SGI13923, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3096[] = {SGI13924, SGI0, SGI13925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3097[] = {SGI13926, SGI13927, NULL};
const Segment * const SG3098[] = {SGI13928, SGI13929, NULL};
const Segment * const SG3099[] = {SGI13930, SGI13931, SGI13932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13933, SGI0, SGI0, SGI0, SGI13934, SGI13935, NULL};
const Segment * const SG3100[] = {SGI13936, SGI13937, SGI13938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13939, SGI0, SGI0, SGI0, SGI13940, SGI13941, NULL};
const Segment * const SG3101[] = {SGI13942, SGI13943, SGI13944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13945, SGI0, SGI0, SGI0, SGI13946, SGI13947, NULL};
const Segment * const SG3102[] = {SGI13948, SGI13949, SGI13950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13951, SGI0, SGI0, SGI0, SGI13952, SGI13953, NULL};
const Segment * const SG3103[] = {SGI13954, SGI13955, SGI13956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13957, SGI0, SGI0, SGI0, SGI13958, SGI13959, NULL};
const Segment * const SG3104[] = {SGI13960, SGI0, SGI13961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13962, SGI0, SGI0, SGI0, SGI13963, SGI13964, NULL};
const Segment * const SG3105[] = {SGI13965, SGI0, SGI13966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13967, SGI0, SGI0, SGI0, SGI0, SGI13968, NULL};
const Segment * const SG3106[] = {SGI13969, SGI13970, SGI13971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13972, SGI0, SGI0, SGI0, SGI13973, SGI13974, NULL};
const Segment * const SG3107[] = {SGI13975, SGI13976, SGI13977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13978, SGI0, SGI0, SGI0, SGI0, SGI13979, NULL};
const Segment * const SG3108[] = {SGI13980, SGI13981, SGI13982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13983, SGI0, SGI0, SGI0, SGI13984, SGI13985, NULL};
const Segment * const SG3109[] = {SGI13986, SGI13987, SGI13988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13989, SGI0, SGI0, SGI0, SGI13990, SGI13990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3110[] = {SGI0, SGI13991, NULL};
const Segment * const SG3111[] = {SGI13992, SGI13993, SGI13994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI13995, SGI0, SGI0, SGI0, SGI13996, SGI13996, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3112[] = {SGI13997, SGI0, SGI13998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3113[] = {SGI13999, SGI14000, SGI14001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14002, SGI0, SGI0, SGI0, SGI14003, SGI14003, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3114[] = {SGI14004, SGI14005, SGI14006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14007, SGI0, SGI0, SGI0, SGI14008, SGI14008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3115[] = {SGI14009, SGI14010, SGI14011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14012, SGI0, SGI0, SGI0, SGI14013, SGI14013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3116[] = {SGI14014, SGI14015, SGI14016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14017, SGI0, SGI0, SGI0, SGI14018, SGI14018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3117[] = {SGI14019, SGI14020, SGI14021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14022, SGI0, SGI0, SGI0, SGI14023, SGI14023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3118[] = {SGI14024, SGI14025, SGI14026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14027, SGI0, SGI0, SGI0, SGI14028, SGI14028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3119[] = {SGI14029, SGI14030, SGI14031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14032, SGI0, SGI0, SGI0, SGI14033, SGI14033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3120[] = {SGI14034, SGI14035, SGI14036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14037, SGI0, SGI0, SGI0, SGI14038, SGI14038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3121[] = {SGI14039, SGI14040, SGI14041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14042, SGI0, SGI0, SGI0, SGI14043, SGI14043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3122[] = {SGI14044, SGI0, SGI14045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3123[] = {SGI14046, SGI14047, SGI14048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14049, SGI0, SGI0, SGI0, SGI14050, SGI14050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3124[] = {SGI14051, SGI14052, SGI14053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14054, SGI0, SGI0, SGI0, SGI14055, SGI14055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3125[] = {SGI14056, SGI14057, SGI14058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14059, SGI0, SGI0, SGI0, SGI14060, SGI14061, NULL};
const Segment * const SG3126[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3127[] = {SGI14063, SGI14064, SGI14065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14066, SGI0, SGI0, SGI0, SGI14067, SGI14067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3128[] = {SGI14068, SGI14069, SGI14070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14071, SGI0, SGI0, SGI0, SGI14072, SGI14072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3129[] = {SGI14073, SGI14074, SGI14075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14076, SGI0, SGI0, SGI0, SGI14077, SGI14077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3130[] = {SGI14078, SGI14079, SGI14080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14081, SGI0, SGI0, SGI0, SGI14082, SGI14082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3131[] = {SGI14083, SGI14084, SGI14085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14086, SGI0, SGI0, SGI0, SGI14087, SGI14087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3132[] = {SGI14088, SGI0, SGI14089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3133[] = {SGI14090, SGI14091, SGI14092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14093, SGI0, SGI0, SGI0, SGI14094, SGI14094, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3134[] = {SGI14095, SGI14096, SGI14097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14098, SGI0, SGI0, SGI0, SGI14099, SGI14099, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3135[] = {SGI14100, SGI14101, SGI14102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14103, SGI0, SGI0, SGI0, SGI14104, SGI14104, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3136[] = {SGI14105, SGI14106, SGI14107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14108, SGI0, SGI0, SGI0, SGI14109, SGI14109, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3137[] = {SGI14110, SGI14111, SGI14112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14113, SGI0, SGI0, SGI0, SGI14114, SGI14114, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3138[] = {SGI14115, SGI14116, SGI14117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14118, SGI0, SGI0, SGI0, SGI14119, SGI14119, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3139[] = {SGI14120, SGI14121, SGI14122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14123, SGI0, SGI0, SGI0, SGI14124, SGI14124, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3140[] = {SGI14125, SGI14126, SGI14127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14128, SGI0, SGI0, SGI0, SGI14129, SGI14129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3141[] = {SGI14130, SGI14131, SGI14132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14133, SGI0, SGI0, SGI0, SGI14134, SGI14134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3142[] = {SGI14135, SGI0, SGI14136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3143[] = {SGI14137, SGI14138, SGI14139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14140, SGI0, SGI0, SGI0, SGI14141, SGI14141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3144[] = {SGI14142, SGI0, SGI14143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3145[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14144, SGI0, SGI0, SGI14145, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3146[] = {SGI14146, SGI0, SGI14147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14148, SGI0, SGI0, SGI0, SGI14149, SGI14149, SGI0, SGI0, SGI0, SGI14150, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3147[] = {SGI14151, SGI14152, SGI14153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14154, SGI0, SGI0, SGI0, SGI14155, SGI14156, NULL};
const Segment * const SG3148[] = {SGI14157, SGI14158, SGI14159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14160, SGI0, SGI0, SGI0, SGI14161, SGI14162, NULL};
const Segment * const SG3149[] = {SGI14163, SGI14164, SGI14165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14166, SGI0, SGI0, SGI0, SGI14167, SGI14168, NULL};
const Segment * const SG3150[] = {SGI14169, SGI14170, SGI14171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14172, SGI0, SGI0, SGI0, SGI14173, SGI14174, NULL};
const Segment * const SG3151[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14175, SGI0, SGI0, SGI14176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3152[] = {SGI14177, SGI14178, SGI14179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14180, SGI0, SGI0, SGI0, SGI14181, SGI14181, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3153[] = {SGI14182, SGI0, SGI14183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3154[] = {SGI14184, SGI14185, SGI14186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14187, SGI0, SGI0, SGI0, SGI14188, SGI14189, NULL};
const Segment * const SG3155[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3156[] = {SGI14191, SGI14192, SGI14193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14194, SGI0, SGI0, SGI0, SGI14195, SGI14195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3157[] = {SGI14196, SGI14197, SGI14198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14199, SGI0, SGI0, SGI0, SGI14200, SGI14200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3158[] = {SGI14201, SGI14202, NULL};
const Segment * const SG3159[] = {SGI14203, SGI14204, NULL};
const Segment * const SG3160[] = {SGI14205, SGI14206, NULL};
const Segment * const SG3161[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14207, SGI0, SGI0, SGI14208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3162[] = {SGI14209, SGI0, SGI14210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14211, SGI0, SGI0, SGI0, SGI14212, SGI14212, SGI0, SGI0, SGI0, SGI14213, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3163[] = {SGI14214, SGI14215, SGI14216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14217, SGI0, SGI0, SGI0, SGI14218, SGI14219, SGI0, SGI0, SGI0, SGI14220, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3164[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14221, SGI0, SGI0, SGI14222, SGI14222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3165[] = {SGI14223, SGI14224, SGI14225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14226, SGI0, SGI0, SGI0, SGI14227, SGI14227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3166[] = {SGI14228, SGI14229, SGI14230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14231, SGI0, SGI0, SGI0, SGI14232, SGI14232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3167[] = {SGI14233, SGI14234, SGI14235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14236, SGI0, SGI0, SGI0, SGI14237, SGI14237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3168[] = {SGI14238, SGI0, SGI14239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3169[] = {SGI0, SGI14240, SGI14241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14242, SGI0, SGI0, SGI0, SGI14243, SGI14243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3170[] = {SGI14244, SGI14245, SGI14246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14247, SGI0, SGI0, SGI0, SGI14248, SGI14248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3171[] = {SGI14249, SGI14250, SGI14251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14252, SGI0, SGI0, SGI0, SGI14253, SGI14253, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3172[] = {SGI14254, SGI0, SGI14255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3173[] = {SGI14256, SGI14257, SGI14258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14259, SGI0, SGI0, SGI0, SGI14260, SGI14260, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3174[] = {SGI14261, SGI14262, SGI14263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14264, SGI0, SGI0, SGI0, SGI14265, SGI14265, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3175[] = {SGI14266, SGI14267, SGI14268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14269, SGI0, SGI0, SGI0, SGI14270, SGI14270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3176[] = {SGI14271, SGI14272, SGI14273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14274, SGI0, SGI0, SGI0, SGI14275, SGI14275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3177[] = {SGI14276, SGI14277, NULL};
const Segment * const SG3178[] = {SGI14278, SGI14279, NULL};
const Segment * const SG3179[] = {SGI14280, SGI14281, NULL};
const Segment * const SG3180[] = {SGI14282, SGI14283, NULL};
const Segment * const SG3181[] = {SGI14284, SGI14285, NULL};
const Segment * const SG3182[] = {SGI14286, SGI14287, SGI14288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14289, SGI0, SGI0, SGI0, SGI14290, SGI14290, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3183[] = {SGI14291, SGI14292, SGI14293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14294, SGI0, SGI0, SGI0, SGI14295, SGI14295, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3184[] = {SGI14296, SGI14297, SGI14298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14299, SGI0, SGI0, SGI0, SGI14300, SGI14300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3185[] = {SGI14301, SGI14302, SGI14303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14304, SGI0, SGI0, SGI0, SGI14305, SGI14305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3186[] = {SGI14306, SGI0, SGI14307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3187[] = {SGI14308, SGI14309, SGI14310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14311, SGI0, SGI0, SGI0, SGI14312, SGI14312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3188[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14313, SGI0, SGI0, SGI14314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3189[] = {SGI14315, SGI14316, SGI14317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14318, SGI0, SGI0, SGI0, SGI14319, SGI14319, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3190[] = {SGI14320, SGI14321, SGI14322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14323, SGI0, SGI0, SGI0, SGI14324, SGI14324, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3191[] = {SGI14325, SGI14326, SGI14327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14328, SGI0, SGI0, SGI0, SGI14329, SGI14329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3192[] = {SGI14330, SGI14331, SGI14332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14333, SGI0, SGI0, SGI0, SGI14334, SGI14334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3193[] = {SGI14335, SGI14336, SGI14337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14338, SGI0, SGI0, SGI0, SGI14339, SGI14340, NULL};
const Segment * const SG3194[] = {SGI14341, SGI0, SGI14342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14343, SGI0, SGI0, SGI0, SGI14344, SGI14344, SGI0, SGI0, SGI14345, SGI0, NULL};
const Segment * const SG3195[] = {SGI14346, SGI0, SGI14347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14348, SGI0, SGI0, SGI0, SGI14349, SGI14349, SGI0, SGI0, SGI14350, SGI0, NULL};
const Segment * const SG3196[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3197[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14352, SGI0, SGI0, SGI14353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3198[] = {SGI14354, SGI14355, SGI14356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14357, SGI0, SGI0, SGI0, SGI14358, SGI14358, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3199[] = {SGI14359, SGI0, SGI14360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3200[] = {SGI14361, SGI14362, SGI14363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14364, SGI0, SGI0, SGI0, SGI14365, SGI14365, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3201[] = {SGI14366, SGI0, SGI14367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3202[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3203[] = {SGI14369, SGI14370, SGI14371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14372, SGI0, SGI0, SGI0, SGI14373, SGI14373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3204[] = {SGI14374, SGI14375, SGI14376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14377, SGI0, SGI0, SGI0, SGI14378, SGI14378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3205[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14379, SGI0, SGI0, SGI14380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3206[] = {SGI14381, SGI14382, SGI14383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14384, SGI0, SGI0, SGI14385, SGI14385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3207[] = {SGI14386, SGI14387, SGI14388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14389, SGI0, SGI0, SGI14390, SGI14390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3208[] = {SGI14391, SGI14392, SGI14393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14394, SGI0, SGI0, SGI14395, SGI14395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3209[] = {SGI14396, SGI14397, SGI14398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14399, SGI0, SGI0, SGI14400, SGI14400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3210[] = {SGI14401, SGI14402, SGI14403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14404, SGI0, SGI0, SGI14405, SGI14405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3211[] = {SGI14406, SGI14407, SGI14408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14409, SGI0, SGI0, SGI14410, SGI14410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3212[] = {SGI14411, SGI0, SGI14412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3213[] = {SGI14413, SGI14414, SGI14415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14416, SGI14417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3214[] = {SGI14418, SGI0, SGI14419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14420, SGI0, SGI0, SGI0, SGI14421, SGI14422, NULL};
const Segment * const SG3215[] = {SGI14423, SGI14424, SGI14425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14426, SGI0, SGI0, SGI0, SGI14427, SGI14428, NULL};
const Segment * const SG3216[] = {SGI14429, SGI14430, SGI14431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14432, SGI0, SGI0, SGI0, SGI14433, SGI14433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3217[] = {SGI14434, SGI14435, NULL};
const Segment * const SG3218[] = {SGI14436, SGI14437, NULL};
const Segment * const SG3219[] = {SGI14438, SGI14439, NULL};
const Segment * const SG3220[] = {SGI14440, SGI14441, SGI14442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14443, SGI0, SGI0, SGI0, SGI14444, SGI14444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3221[] = {SGI14445, SGI0, SGI14446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3222[] = {SGI14447, SGI14448, SGI14449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14450, SGI0, SGI0, SGI0, SGI14451, SGI14452, NULL};
const Segment * const SG3223[] = {SGI0, SGI14453, NULL};
const Segment * const SG3224[] = {SGI0, SGI14454, NULL};
const Segment * const SG3225[] = {SGI0, SGI14455, NULL};
const Segment * const SG3226[] = {SGI14456, SGI14457, SGI14458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14459, SGI0, SGI0, SGI0, SGI14460, SGI14460, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3227[] = {SGI14461, SGI14462, SGI14463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14464, SGI0, SGI0, SGI0, SGI14465, SGI14465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3228[] = {SGI14466, SGI0, SGI14467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3229[] = {SGI14468, SGI0, SGI14469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14470, SGI0, SGI0, SGI0, SGI14471, SGI14471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3230[] = {SGI14472, SGI14473, SGI14474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14475, SGI0, SGI0, SGI0, SGI14476, SGI14476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3231[] = {SGI14477, SGI14478, SGI14479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14480, SGI0, SGI0, SGI0, SGI14481, SGI14482, NULL};
const Segment * const SG3232[] = {SGI14483, SGI14484, SGI14485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14486, SGI0, SGI0, SGI0, SGI14487, SGI14488, NULL};
const Segment * const SG3233[] = {SGI14489, SGI14490, SGI14491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14492, SGI0, SGI0, SGI0, SGI14493, SGI14494, NULL};
const Segment * const SG3234[] = {SGI14495, SGI14496, SGI14497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14498, SGI0, SGI0, SGI0, SGI14499, SGI14500, NULL};
const Segment * const SG3235[] = {SGI14501, SGI14502, SGI14503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14504, SGI0, SGI0, SGI0, SGI14505, SGI14506, NULL};
const Segment * const SG3236[] = {SGI14507, SGI14508, SGI14509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14510, SGI0, SGI0, SGI0, SGI14511, SGI14512, NULL};
const Segment * const SG3237[] = {SGI14513, SGI14514, SGI14515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14518, SGI0, SGI0, SGI0, SGI14520, SGI14521, NULL};
const Segment * const SG3238[] = {SGI14516, SGI14517, SGI14519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14522, SGI14523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3239[] = {SGI14524, SGI14525, SGI14526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14529, SGI0, SGI0, SGI0, SGI14531, SGI14532, NULL};
const Segment * const SG3240[] = {SGI14527, SGI14528, SGI14530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14533, SGI0, SGI0, SGI0, SGI14534, SGI14534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3241[] = {SGI14538, SGI0, SGI14539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3242[] = {SGI14535, SGI14536, SGI14537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14540, SGI0, SGI0, SGI0, SGI14541, SGI14542, NULL};
const Segment * const SG3243[] = {SGI14543, SGI14544, SGI14545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14546, SGI14546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3244[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14547, SGI0, SGI0, SGI14548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3245[] = {SGI14549, SGI0, SGI14550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14551, SGI0, SGI0, SGI0, SGI14552, SGI14552, SGI0, SGI0, SGI0, SGI14553, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3246[] = {SGI14554, SGI14555, SGI14556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14557, SGI0, SGI0, SGI0, SGI14558, SGI14558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3247[] = {SGI14559, SGI0, SGI14560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3248[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3249[] = {SGI0, SGI14562, NULL};
const Segment * const SG3250[] = {SGI14563, SGI14564, SGI14565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14566, SGI0, SGI0, SGI0, SGI14567, SGI14567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3251[] = {SGI14568, SGI14569, SGI14570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14571, SGI0, SGI0, SGI0, SGI14572, SGI14573, NULL};
const Segment * const SG3252[] = {SGI14574, SGI14575, SGI14576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14577, SGI0, SGI0, SGI0, SGI14578, SGI14579, NULL};
const Segment * const SG3253[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3254[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3255[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3256[] = {SGI14583, SGI14584, SGI14585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14586, SGI0, SGI0, SGI0, SGI14587, SGI14588, NULL};
const Segment * const SG3257[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3258[] = {SGI14590, SGI14591, SGI14592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14593, SGI0, SGI0, SGI0, SGI14594, SGI14594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3259[] = {SGI14595, SGI14596, SGI14597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14598, SGI0, SGI0, SGI0, SGI14599, SGI14599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3260[] = {SGI14600, SGI14601, SGI14602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14603, SGI0, SGI0, SGI0, SGI14604, SGI14604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3261[] = {SGI14605, SGI14606, SGI14607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14608, SGI0, SGI0, SGI0, SGI14609, SGI14609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3262[] = {SGI14610, SGI0, SGI14611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14612, SGI0, SGI0, SGI0, SGI14613, SGI14613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3263[] = {SGI14614, SGI0, SGI14615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3264[] = {SGI0, SGI14616, SGI14617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14618, SGI0, SGI0, SGI0, SGI14619, SGI14619, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3265[] = {SGI14620, SGI0, SGI14621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14622, SGI0, SGI0, SGI0, SGI14623, SGI14623, SGI0, SGI0, SGI14624, SGI0, NULL};
const Segment * const SG3266[] = {SGI14625, SGI14626, SGI14627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14628, SGI0, SGI0, SGI0, SGI14629, SGI14630, SGI0, SGI0, SGI14631, SGI0, NULL};
const Segment * const SG3267[] = {SGI14632, SGI14633, SGI14634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14635, SGI0, SGI0, SGI0, SGI14636, SGI14636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3268[] = {SGI0, SGI14637, SGI0, SGI14638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3269[] = {SGI0, SGI14639, SGI0, SGI14640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14641, SGI0, SGI0, SGI0, SGI14642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3270[] = {SGI14643, SGI14644, NULL};
const Segment * const SG3271[] = {SGI14645, SGI14646, NULL};
const Segment * const SG3272[] = {SGI14647, SGI14648, NULL};
const Segment * const SG3273[] = {SGI14649, SGI14650, NULL};
const Segment * const SG3274[] = {SGI14651, SGI14652, SGI14653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14654, SGI0, SGI0, SGI0, SGI14655, SGI14655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3275[] = {SGI14656, SGI14657, SGI14658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14659, SGI0, SGI0, SGI0, SGI14660, SGI14660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3276[] = {SGI14661, SGI14662, SGI14663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14664, SGI14664, SGI0, SGI0, SGI0, SGI14665, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3277[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3278[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3279[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3280[] = {SGI14669, SGI14670, SGI14671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14672, SGI0, SGI0, SGI0, SGI14673, SGI14674, NULL};
const Segment * const SG3281[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3282[] = {SGI14676, SGI14677, SGI14678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14679, SGI0, SGI0, SGI0, SGI14680, SGI14680, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3283[] = {SGI14681, SGI14682, SGI14683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14684, SGI0, SGI0, SGI0, SGI14685, SGI14685, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3284[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14686, SGI0, SGI0, SGI14687, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3285[] = {SGI14688, SGI14689, SGI14690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14691, SGI0, SGI0, SGI0, SGI14692, SGI14692, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3286[] = {SGI14693, SGI14694, SGI14695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14696, SGI0, SGI0, SGI0, SGI14697, SGI14697, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3287[] = {SGI14698, SGI14699, SGI14700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14701, SGI14702, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3288[] = {SGI14703, SGI14704, SGI14705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14706, SGI0, SGI0, SGI0, SGI14707, SGI14707, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3289[] = {SGI14708, SGI14709, SGI14710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14711, SGI14712, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3290[] = {SGI14713, SGI14714, SGI14715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14716, SGI0, SGI0, SGI0, SGI14717, SGI14717, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3291[] = {SGI14718, SGI14719, SGI14720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14721, SGI0, SGI0, SGI0, SGI14722, SGI14722, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3292[] = {SGI14723, SGI0, SGI14724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3293[] = {SGI14725, SGI14726, SGI14727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14728, SGI0, SGI0, SGI0, SGI14729, SGI14730, NULL};
const Segment * const SG3294[] = {SGI14731, SGI14732, SGI14733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14734, SGI14735, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3295[] = {SGI14736, SGI14737, SGI14738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14739, SGI0, SGI0, SGI0, SGI14740, SGI14740, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3296[] = {SGI14741, SGI14742, SGI14743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14744, SGI14744, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3297[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14745, SGI14745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3298[] = {SGI14746, SGI14747, SGI14748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14749, SGI0, SGI0, SGI0, SGI14750, SGI14751, NULL};
const Segment * const SG3299[] = {SGI14752, SGI14753, SGI14754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14755, SGI0, SGI0, SGI0, SGI14756, SGI14756, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3300[] = {SGI14757, SGI14758, SGI14759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14760, SGI0, SGI0, SGI0, SGI14761, SGI14761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3301[] = {SGI14762, SGI14763, SGI14764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14765, SGI0, SGI0, SGI0, SGI14766, SGI14766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3302[] = {SGI0, SGI0, SGI14767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14768, SGI0, SGI0, SGI0, SGI14770, SGI0, NULL};
const Segment * const SG3303[] = {SGI14769, SGI14771, SGI14772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14773, SGI0, SGI0, SGI0, SGI14774, SGI14774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3304[] = {SGI0, SGI14775, SGI0, SGI14776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14777, SGI0, SGI0, SGI0, SGI14778, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3305[] = {SGI0, SGI14779, SGI0, SGI14780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3306[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3307[] = {SGI14782, SGI14783, SGI14784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14785, SGI0, SGI0, SGI0, SGI14786, SGI14786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3308[] = {SGI14787, SGI0, SGI14788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3309[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3310[] = {SGI14790, SGI14791, SGI14792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14793, SGI0, SGI0, SGI0, SGI14794, SGI14794, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3311[] = {SGI14795, SGI0, SGI14796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3312[] = {SGI14797, SGI14798, SGI14799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14800, SGI0, SGI0, SGI0, SGI14801, SGI14801, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3313[] = {SGI14802, SGI14803, SGI14804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14805, SGI0, SGI0, SGI0, SGI14806, SGI14806, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3314[] = {SGI14807, SGI14808, SGI14809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14810, SGI0, SGI0, SGI0, SGI14811, SGI14811, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3315[] = {SGI14812, SGI14813, SGI14814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14815, SGI0, SGI0, SGI0, SGI14816, SGI14816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3316[] = {SGI14817, SGI14818, SGI14819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14820, SGI0, SGI0, SGI0, SGI14821, SGI14821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3317[] = {SGI14822, SGI14823, SGI14824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14825, SGI0, SGI0, SGI0, SGI14826, SGI14826, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3318[] = {SGI14827, SGI14828, SGI14829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14830, SGI0, SGI0, SGI0, SGI14831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3319[] = {SGI0, SGI14832, SGI14833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14834, SGI0, SGI0, SGI0, SGI0, SGI14835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3320[] = {SGI0, SGI0, SGI14836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14837, SGI0, SGI0, SGI0, SGI14838, SGI14838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3321[] = {SGI14839, SGI14840, SGI14841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14842, SGI0, SGI0, SGI0, SGI14843, SGI14843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3322[] = {SGI14844, SGI0, SGI14845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3323[] = {SGI14846, SGI14847, SGI14848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14849, SGI0, SGI0, SGI0, SGI14850, SGI14850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3324[] = {SGI14851, SGI14852, SGI14853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14854, SGI0, SGI0, SGI0, SGI14855, SGI14855, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3325[] = {SGI14856, SGI14857, SGI14858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14859, SGI0, SGI0, SGI0, SGI14860, SGI14860, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3326[] = {SGI14861, SGI0, SGI14862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3327[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14863, SGI0, SGI0, SGI14864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3328[] = {SGI14865, SGI14866, SGI14867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14868, SGI0, SGI0, SGI0, SGI14869, SGI14869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3329[] = {SGI14870, SGI14871, SGI14872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14873, SGI0, SGI0, SGI0, SGI14874, SGI14874, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3330[] = {SGI14875, SGI0, SGI14876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3331[] = {SGI14877, SGI14878, SGI14879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14880, SGI0, SGI0, SGI0, SGI14881, SGI14881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3332[] = {SGI14882, SGI14883, SGI14884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14885, SGI0, SGI0, SGI0, SGI14886, SGI14886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3333[] = {SGI14887, SGI14888, SGI14889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14890, SGI0, SGI0, SGI0, SGI14891, SGI14891, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3334[] = {SGI14892, SGI14893, SGI14894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3335[] = {SGI14895, SGI0, SGI14896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3336[] = {SGI14897, SGI14898, SGI14899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14900, SGI0, SGI0, SGI0, SGI14901, SGI14901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3337[] = {SGI14902, SGI0, SGI14903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3338[] = {SGI0, SGI14904, NULL};
const Segment * const SG3339[] = {SGI0, SGI14905, NULL};
const Segment * const SG3340[] = {SGI14906, SGI14907, SGI14908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14909, SGI0, SGI0, SGI0, SGI14910, SGI14911, NULL};
const Segment * const SG3341[] = {SGI14912, SGI14913, SGI14914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14915, SGI0, SGI0, SGI0, SGI14916, SGI14917, NULL};
const Segment * const SG3342[] = {SGI14918, SGI14919, SGI14920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14921, SGI0, SGI0, SGI0, SGI14922, SGI14923, NULL};
const Segment * const SG3343[] = {SGI14924, SGI14925, SGI14926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14927, SGI0, SGI0, SGI0, SGI14928, SGI14928, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3344[] = {SGI14929, SGI14930, SGI14931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14932, SGI14933, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3345[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3346[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14935, SGI0, SGI0, SGI14936, SGI14936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3347[] = {SGI14937, SGI0, SGI14938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14939, SGI0, SGI0, SGI0, SGI14940, SGI14940, SGI0, SGI0, SGI14941, SGI0, NULL};
const Segment * const SG3348[] = {SGI14942, SGI14943, SGI14944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14945, SGI0, SGI0, SGI0, SGI14946, SGI14946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3349[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14947, SGI0, SGI0, SGI14948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3350[] = {SGI14949, SGI14950, SGI14951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14952, SGI0, SGI0, SGI0, SGI14953, SGI14953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3351[] = {SGI14954, SGI0, SGI14955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14956, SGI0, SGI0, SGI0, SGI14957, SGI14957, SGI0, SGI0, SGI0, SGI14958, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3352[] = {SGI14959, SGI14960, SGI14961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14962, SGI0, SGI0, SGI0, SGI14963, SGI14964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3353[] = {SGI14965, SGI14966, SGI14967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14968, SGI0, SGI0, SGI0, SGI14969, SGI14969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3354[] = {SGI0, SGI0, SGI14970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14971, SGI14972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3355[] = {SGI0, SGI14973, SGI0, SGI14974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14975, SGI0, SGI0, SGI0, SGI14976, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3356[] = {SGI0, SGI14977, SGI0, SGI14978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3357[] = {SGI0, SGI14979, SGI14980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14981, SGI0, SGI0, SGI0, SGI14982, SGI14982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3358[] = {SGI0, SGI14983, SGI14984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14985, SGI0, SGI0, SGI0, SGI14986, SGI14986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3359[] = {SGI0, SGI14987, SGI14988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14989, SGI0, SGI0, SGI0, SGI14990, SGI14990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3360[] = {SGI14991, SGI14992, SGI14993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI14994, SGI0, SGI0, SGI0, SGI14995, SGI14995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3361[] = {SGI14999, SGI15000, SGI15001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15002, SGI15002, SGI0, SGI0, SGI15003, SGI0, NULL};
const Segment * const SG3362[] = {SGI15004, SGI15005, SGI15006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15007, SGI15007, SGI0, SGI0, SGI15008, SGI0, NULL};
const Segment * const SG3363[] = {SGI14996, SGI14997, SGI14998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15009, SGI0, SGI0, SGI0, SGI15010, SGI15010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3364[] = {SGI15011, SGI15012, SGI15013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15014, SGI15015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3365[] = {SGI15016, SGI0, SGI15017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15018, SGI0, SGI0, SGI0, SGI15019, SGI15019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15020, SGI0, SGI0, NULL};
const Segment * const SG3366[] = {SGI15021, SGI0, SGI15022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15023, SGI0, SGI0, SGI0, SGI15024, SGI15024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15025, SGI0, SGI0, NULL};
const Segment * const SG3367[] = {SGI15026, SGI0, SGI15027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15028, SGI0, SGI0, SGI0, SGI15029, SGI15029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15030, SGI0, SGI0, NULL};
const Segment * const SG3368[] = {SGI15031, SGI0, SGI15032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15033, SGI0, SGI0, SGI0, SGI15034, SGI15034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15035, SGI0, SGI0, NULL};
const Segment * const SG3369[] = {SGI15036, SGI15037, SGI15038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15039, SGI0, SGI0, SGI0, SGI15040, SGI15040, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3370[] = {SGI15041, SGI15042, SGI15043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15044, SGI0, SGI0, SGI0, SGI15045, SGI15045, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3371[] = {SGI15046, SGI15047, SGI15048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15049, SGI15050, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3372[] = {SGI15051, SGI15052, SGI15053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15054, SGI15054, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3373[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3374[] = {SGI0, SGI15056, SGI15057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15058, SGI0, SGI0, SGI0, SGI15059, SGI15059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3375[] = {SGI0, SGI15060, SGI15061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15062, SGI0, SGI0, SGI0, SGI15063, SGI15063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3376[] = {SGI15064, SGI0, SGI15065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15066, SGI0, SGI0, SGI0, SGI15067, SGI15067, SGI0, SGI0, SGI15068, SGI0, NULL};
const Segment * const SG3377[] = {SGI15069, SGI15070, SGI15071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15072, SGI0, SGI0, SGI0, SGI15073, SGI15073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3378[] = {SGI15074, SGI15075, SGI15076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15077, SGI0, SGI0, SGI0, SGI15078, SGI15078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3379[] = {SGI15079, SGI15080, SGI15081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15082, SGI15083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3380[] = {SGI0, SGI15084, NULL};
const Segment * const SG3381[] = {SGI15085, SGI15086, SGI15087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15088, SGI0, SGI0, SGI0, SGI15089, SGI15089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3382[] = {SGI15090, SGI0, SGI15091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15092, SGI0, SGI0, SGI0, SGI15093, SGI15093, SGI0, SGI0, SGI0, SGI15094, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3383[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3384[] = {SGI15096, SGI15097, SGI15098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15099, SGI0, SGI0, SGI0, SGI15100, SGI15100, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3385[] = {SGI15101, SGI15102, SGI15103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15104, SGI0, SGI0, SGI0, SGI15105, SGI15105, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3386[] = {SGI15106, SGI15107, SGI15108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15109, SGI0, SGI0, SGI0, SGI15110, SGI15110, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3387[] = {SGI15111, SGI0, SGI15112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15113, SGI0, SGI0, SGI0, SGI15114, SGI15114, SGI0, SGI0, SGI15115, SGI0, NULL};
const Segment * const SG3388[] = {SGI15116, SGI0, SGI15117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15118, SGI0, SGI0, SGI0, SGI15119, SGI15119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15120, SGI0, SGI0, NULL};
const Segment * const SG3389[] = {SGI15121, SGI0, SGI15122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15123, SGI0, SGI0, SGI0, SGI15124, SGI15124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15125, SGI0, SGI0, NULL};
const Segment * const SG3390[] = {SGI15126, SGI0, SGI15127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15128, SGI0, SGI0, SGI0, SGI15129, SGI15129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15130, SGI0, SGI0, NULL};
const Segment * const SG3391[] = {SGI15131, SGI15132, SGI15133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15134, SGI0, SGI0, SGI0, SGI15135, SGI15135, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3392[] = {SGI15136, SGI15137, SGI15138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15139, SGI0, SGI0, SGI0, SGI15140, SGI15140, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3393[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3394[] = {SGI15142, SGI15143, SGI15144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15145, SGI0, SGI0, SGI0, SGI15146, SGI15146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3395[] = {SGI15147, SGI15148, SGI15149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15150, SGI0, SGI0, SGI0, SGI15151, SGI15151, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3396[] = {SGI15152, SGI15153, SGI15154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15155, SGI0, SGI0, SGI0, SGI15156, SGI15156, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3397[] = {SGI15157, SGI15158, SGI15159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15160, SGI0, SGI0, SGI0, SGI15161, SGI15161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3398[] = {SGI0, SGI15162, SGI0, SGI15163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3399[] = {SGI0, SGI15164, SGI0, SGI15165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15166, SGI0, SGI0, SGI0, SGI15167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3400[] = {SGI15168, SGI0, SGI15169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15170, SGI0, SGI0, SGI0, SGI15171, SGI15171, SGI0, SGI0, SGI0, SGI15172, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3401[] = {SGI15173, SGI0, SGI15174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15175, SGI0, SGI0, SGI0, SGI15176, SGI15177, NULL};
const Segment * const SG3402[] = {SGI15178, SGI15179, SGI15180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15181, SGI0, SGI0, SGI0, SGI15182, SGI15183, NULL};
const Segment * const SG3403[] = {SGI15184, SGI15185, SGI15186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15187, SGI0, SGI0, SGI0, SGI15188, SGI15188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3404[] = {SGI0, SGI15189, NULL};
const Segment * const SG3405[] = {SGI0, SGI15190, NULL};
const Segment * const SG3406[] = {SGI15191, SGI0, SGI15192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15193, SGI0, SGI0, SGI0, SGI15194, SGI15194, SGI0, SGI0, SGI0, SGI15195, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3407[] = {SGI15196, SGI0, SGI15197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15198, SGI0, SGI0, SGI0, SGI15199, SGI15199, SGI0, SGI0, SGI15200, SGI0, NULL};
const Segment * const SG3408[] = {SGI15201, SGI15202, SGI15203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15204, SGI0, SGI0, SGI0, SGI15205, SGI15206, SGI0, SGI0, SGI15207, SGI0, NULL};
const Segment * const SG3409[] = {SGI0, SGI15208, SGI0, SGI15209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15210, SGI0, SGI0, SGI0, SGI15211, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3410[] = {SGI0, SGI15212, SGI0, SGI15213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3411[] = {SGI15214, SGI15215, SGI15216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15217, SGI0, SGI0, SGI0, SGI15218, SGI15218, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3412[] = {SGI15219, SGI15220, SGI15221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15222, SGI0, SGI0, SGI0, SGI15223, SGI15223, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3413[] = {SGI15224, SGI15225, SGI15226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15227, SGI0, SGI0, SGI0, SGI15228, SGI15228, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3414[] = {SGI15229, SGI15230, SGI15231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15232, SGI15233, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3415[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3416[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3417[] = {SGI15236, SGI0, SGI15237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15238, SGI0, SGI0, SGI0, SGI15239, SGI15239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15240, SGI0, SGI0, NULL};
const Segment * const SG3418[] = {SGI15241, SGI0, SGI15242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15243, SGI0, SGI0, SGI0, SGI15244, SGI15244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15245, SGI0, SGI0, NULL};
const Segment * const SG3419[] = {SGI15246, SGI0, SGI15247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15248, SGI0, SGI0, SGI0, SGI15249, SGI15249, SGI0, SGI0, SGI0, SGI15250, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3420[] = {SGI15251, SGI0, SGI15252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15253, SGI0, SGI0, SGI0, SGI15254, SGI15254, SGI0, SGI0, SGI0, SGI15255, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3421[] = {SGI15256, SGI15257, SGI15258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15259, SGI0, SGI0, SGI0, SGI15260, SGI15260, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3422[] = {SGI15261, SGI0, SGI15262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3423[] = {SGI15263, SGI0, SGI15264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15265, SGI0, SGI0, SGI0, SGI15266, SGI15266, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3424[] = {SGI15267, SGI15268, SGI15269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15270, SGI0, SGI0, SGI0, SGI15271, SGI15271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3425[] = {SGI15272, SGI15273, SGI15274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15275, SGI0, SGI0, SGI0, SGI15276, SGI15276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3426[] = {SGI15277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3427[] = {SGI0, SGI15278, SGI0, SGI15279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15280, SGI0, SGI0, SGI0, SGI15281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3428[] = {SGI0, SGI15282, SGI0, SGI15283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3429[] = {SGI0, SGI15284, SGI0, SGI15285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15286, SGI0, SGI0, SGI0, SGI15287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3430[] = {SGI15288, SGI15289, SGI15290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15291, SGI0, SGI0, SGI0, SGI15292, SGI15292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3431[] = {SGI15293, SGI15294, SGI15295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15296, SGI0, SGI0, SGI0, SGI15297, SGI15297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3432[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3433[] = {SGI15299, SGI15300, SGI15301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15302, SGI0, SGI0, SGI0, SGI15303, SGI15303, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3434[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15304, SGI15305, SGI15306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3435[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3436[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15308, SGI0, SGI0, SGI15309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3437[] = {SGI15310, SGI15311, SGI15312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15313, SGI0, SGI0, SGI15314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3438[] = {SGI15315, SGI15316, SGI15317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15318, SGI0, SGI0, SGI0, SGI15319, SGI15319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3439[] = {SGI15320, SGI15321, SGI15322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15323, SGI0, SGI0, SGI0, SGI15324, SGI15324, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3440[] = {SGI15325, SGI15326, SGI15327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15328, SGI0, SGI0, SGI0, SGI15329, SGI15329, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3441[] = {SGI15330, SGI15331, SGI15332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15333, SGI0, SGI0, SGI0, SGI15334, SGI15334, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3442[] = {SGI15335, SGI15336, SGI15337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15338, SGI0, SGI0, SGI0, SGI15339, SGI15339, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3443[] = {SGI0, SGI15340, NULL};
const Segment * const SG3444[] = {SGI0, SGI15341, SGI0, SGI15342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15343, SGI0, SGI0, SGI0, SGI15344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3445[] = {SGI0, SGI15345, SGI0, SGI15346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3446[] = {SGI0, SGI15347, SGI0, SGI15348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15349, SGI0, SGI0, SGI0, SGI15350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3447[] = {SGI0, SGI15351, SGI0, SGI15352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3448[] = {SGI15353, SGI15354, SGI15355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15356, SGI0, SGI0, SGI0, SGI15357, SGI15357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3449[] = {SGI15358, SGI15359, SGI15360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15361, SGI0, SGI0, SGI0, SGI15362, SGI15362, SGI0, NULL};
const Segment * const SG3450[] = {SGI15363, SGI15364, SGI15365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15366, SGI0, SGI0, SGI0, SGI15367, SGI15367, SGI0, NULL};
const Segment * const SG3451[] = {SGI15368, SGI15369, SGI15370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15371, SGI0, SGI0, SGI0, SGI15372, SGI15372, SGI0, NULL};
const Segment * const SG3452[] = {SGI15373, SGI15374, SGI15375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15376, SGI0, SGI0, SGI0, SGI15377, SGI15377, SGI0, NULL};
const Segment * const SG3453[] = {SGI15378, SGI15379, SGI15380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15381, SGI0, SGI0, SGI0, SGI15382, SGI15382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3454[] = {SGI15383, SGI15384, SGI15385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15386, SGI15387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3455[] = {SGI15388, SGI15389, SGI15390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15391, SGI0, SGI0, SGI0, SGI15392, SGI15392, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3456[] = {SGI15393, SGI15394, SGI15395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15396, SGI0, SGI0, SGI0, SGI15397, SGI15398, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3457[] = {SGI15399, SGI0, SGI15400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3458[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15401, SGI0, SGI0, SGI15402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3459[] = {SGI15403, SGI15404, SGI15405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15406, SGI0, SGI0, SGI0, SGI15407, SGI15407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3460[] = {SGI15408, SGI15409, SGI15410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15411, SGI0, SGI0, SGI0, SGI15412, SGI15412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3461[] = {SGI15413, SGI0, SGI15414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3462[] = {SGI15415, SGI15416, SGI15417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15418, SGI0, SGI0, SGI0, SGI15419, SGI15420, NULL};
const Segment * const SG3463[] = {SGI15421, SGI15422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15423, SGI15424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3464[] = {SGI15425, SGI15426, SGI15427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15428, SGI0, SGI0, SGI0, SGI15429, SGI15429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3465[] = {SGI15430, SGI15431, SGI15432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15433, SGI0, SGI0, SGI0, SGI15434, SGI15434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3466[] = {SGI15435, SGI15436, SGI15437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15438, SGI0, SGI0, SGI0, SGI15439, SGI15439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3467[] = {SGI15440, SGI15441, SGI15442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15443, SGI0, SGI0, SGI0, SGI15444, SGI15444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3468[] = {SGI15445, SGI15446, SGI15447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15448, SGI0, SGI0, SGI0, SGI15449, SGI15449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3469[] = {SGI15450, SGI15451, SGI15452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15453, SGI0, SGI0, SGI0, SGI15454, SGI15454, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3470[] = {SGI0, SGI15455, NULL};
const Segment * const SG3471[] = {SGI0, SGI15456, NULL};
const Segment * const SG3472[] = {SGI15457, SGI0, SGI15458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15459, SGI0, SGI0, SGI0, SGI15460, SGI15460, SGI0, SGI0, SGI0, SGI15461, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3473[] = {SGI15462, SGI15463, SGI15464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15465, SGI0, SGI0, SGI0, SGI15466, SGI15467, SGI0, SGI0, SGI0, SGI15468, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3474[] = {SGI15469, SGI15470, SGI15471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15472, SGI0, SGI0, SGI0, SGI15473, SGI15473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3475[] = {SGI15474, SGI15475, SGI15476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15477, SGI0, SGI0, SGI0, SGI15478, SGI15478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3476[] = {SGI15479, SGI15480, SGI15481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15482, SGI0, SGI0, SGI0, SGI15483, SGI15483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3477[] = {SGI15484, SGI15485, SGI15486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15487, SGI0, SGI0, SGI0, SGI15488, SGI15488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3478[] = {SGI15489, SGI15490, SGI15491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15492, SGI0, SGI0, SGI0, SGI15493, SGI15494, NULL};
const Segment * const SG3479[] = {SGI15495, SGI0, SGI15496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15497, SGI0, SGI0, SGI0, SGI15498, SGI15498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3480[] = {SGI15499, SGI15500, SGI15501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15502, SGI0, SGI0, SGI0, SGI15503, SGI15503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3481[] = {SGI15504, SGI15505, SGI15506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15507, SGI0, SGI0, SGI0, SGI15508, SGI15509, NULL};
const Segment * const SG3482[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15510, SGI0, SGI0, SGI15511, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3483[] = {SGI15512, SGI15513, SGI15514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15515, SGI0, SGI0, SGI0, SGI15516, SGI0, SGI0, SGI15517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3484[] = {SGI15518, SGI0, SGI15519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15520, SGI0, SGI0, SGI0, SGI15521, SGI15521, SGI0, SGI0, SGI0, SGI15522, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3485[] = {SGI15523, SGI15524, SGI15525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15526, SGI0, SGI0, SGI0, SGI15527, SGI15528, SGI0, SGI0, SGI0, SGI15529, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3486[] = {SGI0, SGI0, SGI15530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3487[] = {SGI15532, SGI15533, SGI15534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15535, SGI0, SGI0, SGI0, SGI15536, SGI15536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3488[] = {SGI15537, SGI15538, SGI15539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15540, SGI0, SGI0, SGI0, SGI15541, SGI15541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3489[] = {SGI15542, SGI15543, SGI15544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15545, SGI0, SGI0, SGI0, SGI15546, SGI15546, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3490[] = {SGI15547, SGI0, SGI15548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3491[] = {SGI15549, SGI15550, SGI15551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15552, SGI0, SGI0, SGI0, SGI15553, SGI15553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3492[] = {SGI15554, SGI15555, SGI15556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15557, SGI0, SGI0, SGI0, SGI15558, SGI15558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3493[] = {SGI15559, SGI15560, SGI15561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15562, SGI0, SGI0, SGI0, SGI15563, SGI15563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3494[] = {SGI15564, SGI15565, SGI15566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15567, SGI0, SGI0, SGI0, SGI15568, SGI15568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3495[] = {SGI15569, SGI15570, SGI15571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15572, SGI0, SGI0, SGI0, SGI15573, SGI15573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3496[] = {SGI15574, SGI15575, SGI15576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15577, SGI0, SGI0, SGI0, SGI15578, SGI15578, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3497[] = {SGI15579, SGI15580, SGI15581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15582, SGI0, SGI0, SGI0, SGI15583, SGI15583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3498[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15584, SGI0, SGI0, SGI0, SGI15585, SGI15585, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3499[] = {SGI15586, SGI15587, SGI15588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15589, SGI0, SGI0, SGI0, SGI15590, SGI15590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3500[] = {SGI15591, SGI15592, SGI15593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15594, SGI0, SGI0, SGI0, SGI15595, SGI15595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3501[] = {SGI15596, SGI15597, SGI15598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15599, SGI0, SGI0, SGI0, SGI15600, SGI15600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3502[] = {SGI15601, SGI15602, SGI15603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15604, SGI15604, SGI0, SGI0, SGI0, SGI15605, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3503[] = {SGI15606, SGI15607, SGI15608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15609, SGI0, SGI0, SGI0, SGI15610, SGI15611, NULL};
const Segment * const SG3504[] = {SGI0, SGI15612, SGI15613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15614, SGI0, SGI0, SGI0, SGI15615, SGI15616, NULL};
const Segment * const SG3505[] = {SGI0, SGI15617, SGI15618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15619, SGI0, SGI0, SGI0, SGI15620, SGI15621, NULL};
const Segment * const SG3506[] = {SGI15622, SGI15623, SGI15624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15625, SGI15625, SGI0, SGI0, SGI0, SGI15626, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3507[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3508[] = {SGI15628, SGI15629, SGI15630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15631, SGI0, SGI0, SGI0, SGI15632, SGI15632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3509[] = {SGI15633, SGI15634, SGI15635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15636, SGI0, SGI0, SGI0, SGI15637, SGI15637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3510[] = {SGI15638, SGI15639, SGI15640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15641, SGI0, SGI0, SGI0, SGI15642, SGI15642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3511[] = {SGI15643, SGI15644, SGI15645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15646, SGI0, SGI0, SGI0, SGI15647, SGI15647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3512[] = {SGI15648, SGI15649, SGI15650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15651, SGI0, SGI0, SGI0, SGI15652, SGI15652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3513[] = {SGI15653, SGI15654, SGI15655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15656, SGI0, SGI0, SGI0, SGI15657, SGI15657, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3514[] = {SGI15658, SGI15659, SGI15660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15661, SGI0, SGI0, SGI0, SGI15662, SGI15662, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3515[] = {SGI15663, SGI15664, SGI15665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15666, SGI0, SGI0, SGI0, SGI15667, SGI15668, NULL};
const Segment * const SG3516[] = {SGI15669, SGI15670, SGI15671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15672, SGI15673, NULL};
const Segment * const SG3517[] = {SGI15674, SGI15675, SGI15676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15677, SGI0, SGI0, SGI0, SGI15678, SGI15679, NULL};
const Segment * const SG3518[] = {SGI15680, SGI15681, SGI15682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15683, SGI0, SGI0, SGI0, SGI15684, SGI15684, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3519[] = {SGI15685, SGI15686, SGI15687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15688, SGI0, SGI0, SGI0, SGI15689, SGI15689, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3520[] = {SGI15690, SGI15691, SGI15692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15693, SGI0, SGI0, SGI0, SGI15694, SGI15694, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3521[] = {SGI15695, SGI15696, SGI15697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15698, SGI0, SGI0, SGI0, SGI15699, SGI15699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3522[] = {SGI15700, SGI15701, SGI15702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15703, SGI0, SGI0, SGI0, SGI15704, SGI15704, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3523[] = {SGI15705, SGI0, SGI15706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3524[] = {SGI15707, SGI0, SGI15708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15709, SGI0, SGI0, SGI0, SGI15710, SGI15710, SGI0, SGI0, SGI0, SGI15711, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3525[] = {SGI15712, SGI15713, SGI15714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15715, SGI15715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15716, SGI0, SGI0, NULL};
const Segment * const SG3526[] = {SGI15717, SGI15718, SGI15719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15720, SGI0, SGI0, SGI0, SGI15721, SGI15721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3527[] = {SGI15722, SGI15723, SGI15724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15725, SGI15726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3528[] = {SGI15727, SGI15728, SGI15729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15730, SGI15730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3529[] = {SGI15731, SGI15732, SGI15733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15734, SGI0, SGI0, SGI0, SGI15735, SGI15735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3530[] = {SGI0, SGI15736, NULL};
const Segment * const SG3531[] = {SGI15737, SGI15738, SGI15739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15740, SGI0, SGI0, SGI15741, SGI15741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3532[] = {SGI15742, SGI15743, SGI15744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15745, SGI0, SGI0, SGI15746, SGI15746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3533[] = {SGI15747, SGI15748, SGI15749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15750, SGI0, SGI0, SGI15751, SGI15751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3534[] = {SGI15752, SGI15753, SGI15754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15755, SGI0, SGI0, SGI15756, SGI15756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3535[] = {SGI15757, SGI15758, SGI15759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15760, SGI0, SGI0, SGI15761, SGI15761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3536[] = {SGI15762, SGI15763, SGI15764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15765, SGI0, SGI0, SGI15766, SGI15766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3537[] = {SGI15767, SGI0, SGI15768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15769, SGI0, SGI0, SGI0, SGI15770, SGI15770, SGI0, SGI0, SGI0, SGI15771, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3538[] = {SGI15772, SGI15773, SGI15774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15775, SGI0, SGI0, SGI0, SGI15776, SGI15777, SGI0, SGI0, SGI0, SGI15778, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3539[] = {SGI15779, SGI0, SGI15780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15781, SGI0, SGI0, SGI0, SGI15782, SGI15782, SGI0, SGI0, SGI15783, SGI0, NULL};
const Segment * const SG3540[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15784, SGI0, SGI0, SGI15785, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3541[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15786, SGI0, SGI0, SGI15787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3542[] = {SGI15788, SGI0, SGI15789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15790, SGI0, SGI0, SGI0, SGI15791, SGI15791, SGI0, SGI0, SGI0, SGI15792, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3543[] = {SGI15793, SGI15794, SGI15795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15796, SGI0, SGI0, SGI0, SGI15797, SGI15797, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3544[] = {SGI15798, SGI15799, NULL};
const Segment * const SG3545[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15800, SGI0, SGI0, SGI15801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3546[] = {SGI15802, SGI15803, NULL};
const Segment * const SG3547[] = {SGI15804, SGI0, SGI15805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15806, SGI0, SGI0, SGI0, SGI15807, SGI15807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15808, SGI0, SGI0, NULL};
const Segment * const SG3548[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15809, SGI0, SGI0, SGI15810, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3549[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15811, SGI0, SGI0, SGI15812, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3550[] = {SGI15813, SGI15814, SGI15815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15816, SGI15816, SGI0, SGI0, SGI0, SGI15817, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3551[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3552[] = {SGI15819, SGI15820, SGI15821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15822, SGI0, SGI0, SGI0, SGI15823, SGI15823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3553[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3554[] = {SGI15825, SGI15826, SGI15827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15828, SGI0, SGI0, SGI0, SGI15829, SGI15829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3555[] = {SGI0, SGI0, SGI15830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15831, SGI0, SGI0, SGI0, SGI15832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3556[] = {SGI15833, SGI0, SGI15834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15835, SGI0, SGI0, SGI0, SGI15836, SGI15836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3557[] = {SGI0, SGI15837, SGI15838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15839, SGI0, SGI0, SGI0, SGI15840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3558[] = {SGI15841, SGI15842, SGI15843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15844, SGI0, SGI0, SGI15845, SGI15845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3559[] = {SGI15846, SGI15847, SGI15848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15849, SGI0, SGI0, SGI15850, SGI15850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3560[] = {SGI15851, SGI15852, SGI15853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15854, SGI0, SGI0, SGI0, SGI15855, SGI15855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3561[] = {SGI15856, SGI15857, SGI15858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15859, SGI0, SGI0, SGI15860, SGI15860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3562[] = {SGI15861, SGI15862, SGI15863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15864, SGI0, SGI0, SGI15865, SGI15865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3563[] = {SGI15866, SGI15867, SGI15868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15869, SGI0, SGI0, SGI15870, SGI15870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3564[] = {SGI15871, SGI15872, SGI15873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15874, SGI0, SGI0, SGI15875, SGI15875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3565[] = {SGI15876, SGI15877, SGI15878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15879, SGI0, SGI0, SGI15880, SGI15880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3566[] = {SGI15881, SGI15882, SGI15883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15884, SGI0, SGI0, SGI15885, SGI15885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3567[] = {SGI15886, SGI15887, SGI15888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15889, SGI0, SGI0, SGI15890, SGI15890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3568[] = {SGI15893, SGI15894, NULL};
const Segment * const SG3569[] = {SGI15891, SGI15892, SGI15895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15896, SGI0, SGI0, SGI15897, SGI15897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3570[] = {SGI15898, SGI15899, SGI15900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15901, SGI0, SGI0, SGI15902, SGI15902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3571[] = {SGI15903, SGI15904, SGI15905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15906, SGI0, SGI0, SGI15907, SGI15907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3572[] = {SGI15908, SGI15909, SGI15910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15911, SGI0, SGI0, SGI15912, SGI15912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3573[] = {SGI15913, SGI15914, SGI15915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15916, SGI0, SGI0, SGI15917, SGI15917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3574[] = {SGI15918, SGI15919, SGI15920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15921, SGI0, SGI0, SGI15922, SGI15922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3575[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15923, SGI0, SGI0, SGI15924, SGI15924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3576[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3577[] = {SGI15926, SGI15927, SGI15928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15929, SGI0, SGI0, SGI0, SGI15930, SGI15930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3578[] = {SGI15931, SGI15932, SGI15933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15934, SGI0, SGI0, SGI0, SGI15935, SGI15935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3579[] = {SGI15937, SGI15938, SGI15940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15941, SGI0, SGI0, SGI0, SGI15943, SGI15943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3580[] = {SGI15936, SGI0, SGI15939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15942, SGI0, SGI0, SGI0, SGI15944, SGI15944, SGI0, SGI0, SGI0, SGI15945, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3581[] = {SGI15946, SGI15947, SGI15948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15949, SGI0, SGI0, SGI0, SGI15950, SGI15950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3582[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3583[] = {SGI15952, SGI0, SGI15953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15954, SGI0, SGI0, SGI0, SGI15955, SGI15955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15956, SGI0, SGI0, NULL};
const Segment * const SG3584[] = {SGI15957, SGI15958, SGI15959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15960, SGI15960, SGI0, SGI0, SGI15961, SGI0, NULL};
const Segment * const SG3585[] = {SGI0, SGI0, SGI15962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15963, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3586[] = {SGI15964, SGI15965, SGI15966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15967, SGI0, SGI0, SGI15968, SGI15968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3587[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15969, SGI15970, SGI0, SGI0, SGI0, SGI0, SGI15971, NULL};
const Segment * const SG3588[] = {SGI15972, SGI15973, SGI15974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15975, SGI0, SGI0, SGI0, SGI15976, SGI15976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3589[] = {SGI0, SGI0, SGI15977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3590[] = {SGI0, SGI15979, SGI0, SGI15980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15981, SGI0, SGI0, SGI0, SGI15982, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3591[] = {SGI0, SGI15983, SGI0, SGI15984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3592[] = {SGI15985, SGI15986, SGI15987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15988, SGI0, SGI0, SGI0, SGI15989, SGI15989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3593[] = {SGI15990, SGI15991, SGI15992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15993, SGI15994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3594[] = {SGI15995, SGI15996, SGI15997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI15998, SGI15998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3595[] = {SGI15999, SGI0, SGI16000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16001, SGI0, SGI0, SGI0, SGI16002, SGI16002, SGI0, SGI0, SGI0, SGI16003, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3596[] = {SGI0, SGI0, SGI16004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16005, SGI0, SGI0, SGI0, SGI16006, SGI16007, NULL};
const Segment * const SG3597[] = {SGI16008, SGI0, SGI16009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16010, SGI0, SGI0, SGI0, SGI16011, SGI16011, SGI0, SGI0, SGI0, SGI16012, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3598[] = {SGI16013, SGI16014, SGI16015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16016, SGI0, SGI0, SGI0, SGI16017, SGI16017, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3599[] = {SGI16018, SGI16019, SGI16020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16021, SGI0, SGI0, SGI0, SGI16022, SGI16022, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3600[] = {SGI16023, SGI0, SGI16024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16025, SGI0, SGI0, SGI0, SGI16026, SGI16026, SGI0, SGI0, SGI0, SGI16027, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3601[] = {SGI16028, SGI16029, SGI16030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16031, SGI0, SGI0, SGI0, SGI16032, SGI16032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3602[] = {SGI16033, SGI0, SGI16034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3603[] = {SGI16035, SGI16036, SGI16037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16038, SGI0, SGI0, SGI0, SGI16039, SGI16039, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3604[] = {SGI16040, SGI16041, SGI16042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16043, SGI0, SGI0, SGI0, SGI16044, SGI16044, SGI16045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3605[] = {SGI16046, SGI16047, SGI16048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16049, SGI0, SGI0, SGI0, SGI16050, SGI16050, SGI16051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3606[] = {SGI16052, SGI16053, SGI16054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16055, SGI0, SGI0, SGI0, SGI16056, SGI16056, SGI16057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3607[] = {SGI16058, SGI16059, NULL};
const Segment * const SG3608[] = {SGI16060, SGI16061, NULL};
const Segment * const SG3609[] = {SGI16062, SGI16063, NULL};
const Segment * const SG3610[] = {SGI16064, SGI16065, SGI16066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16067, SGI16067, NULL};
const Segment * const SG3611[] = {SGI16068, SGI16069, SGI16070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16071, SGI16071, NULL};
const Segment * const SG3612[] = {SGI16072, SGI16073, SGI16074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16075, SGI0, SGI0, SGI0, SGI16076, SGI16076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3613[] = {SGI0, SGI16077, SGI0, SGI16078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3614[] = {SGI16079, SGI16080, SGI16081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16082, SGI0, SGI0, SGI0, SGI16083, SGI16083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3615[] = {SGI16084, SGI0, SGI16085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3616[] = {SGI16086, SGI16087, SGI16088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16089, SGI16090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3617[] = {SGI0, SGI16091, NULL};
const Segment * const SG3618[] = {SGI0, SGI16092, NULL};
const Segment * const SG3619[] = {SGI16093, SGI16094, SGI16095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16096, SGI0, SGI0, SGI16097, SGI16097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3620[] = {SGI16098, SGI16099, SGI16100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16101, SGI0, SGI0, SGI16102, SGI16102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3621[] = {SGI16103, SGI16104, SGI16105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16106, SGI16106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16107, SGI0, SGI0, NULL};
const Segment * const SG3622[] = {SGI0, SGI16108, NULL};
const Segment * const SG3623[] = {SGI16109, SGI16110, SGI16111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16112, SGI0, SGI0, SGI0, SGI16113, SGI16113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3624[] = {SGI16114, SGI16115, SGI16116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16117, SGI0, SGI0, SGI0, SGI16118, SGI16118, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3625[] = {SGI16119, SGI16120, SGI16121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16122, SGI0, SGI0, SGI0, SGI16123, SGI16123, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3626[] = {SGI16124, SGI16125, SGI16126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16127, SGI0, SGI0, SGI0, SGI16128, SGI16128, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3627[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3628[] = {SGI16130, SGI16131, SGI16132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16133, SGI0, SGI0, SGI0, SGI16134, SGI16134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3629[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3630[] = {SGI16136, SGI0, SGI16137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16138, SGI0, SGI0, SGI0, SGI16139, SGI16139, SGI0, SGI0, SGI16140, SGI0, NULL};
const Segment * const SG3631[] = {SGI16141, SGI16142, SGI16143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16144, SGI0, SGI0, SGI0, SGI16145, SGI16145, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3632[] = {SGI16146, SGI16147, SGI16148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16149, SGI0, SGI0, SGI0, SGI16150, SGI16150, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3633[] = {SGI0, SGI16151, NULL};
const Segment * const SG3634[] = {SGI16152, SGI16153, SGI16154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16155, SGI0, SGI0, SGI16156, SGI16156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3635[] = {SGI16157, SGI16158, SGI16159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16160, SGI0, SGI0, SGI0, SGI16161, SGI16161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3636[] = {SGI16162, SGI16163, SGI16164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16165, SGI0, SGI0, SGI0, SGI16166, SGI16166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3637[] = {SGI16167, SGI0, SGI16168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16169, SGI0, SGI0, SGI0, SGI16170, SGI16170, SGI0, SGI0, SGI0, SGI16171, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3638[] = {SGI0, SGI16172, NULL};
const Segment * const SG3639[] = {SGI0, SGI16173, NULL};
const Segment * const SG3640[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3641[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3642[] = {SGI16176, SGI16177, SGI16178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16179, SGI0, SGI0, SGI0, SGI16180, SGI16180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3643[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3644[] = {SGI16182, SGI16183, SGI16184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16185, SGI0, SGI0, SGI16186, SGI16186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3645[] = {SGI16187, SGI16188, SGI16189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16190, SGI0, SGI0, SGI16191, SGI16191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3646[] = {SGI16192, SGI16193, SGI16194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16195, SGI0, SGI0, SGI16196, SGI16196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3647[] = {SGI16197, SGI16199, SGI16200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16202, SGI0, SGI0, SGI16203, SGI16203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3648[] = {SGI16198, SGI0, SGI16201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16204, SGI0, SGI0, SGI0, SGI16205, SGI16205, SGI0, SGI0, SGI0, SGI16206, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3649[] = {SGI16208, SGI16211, SGI16212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16213, SGI0, SGI0, SGI16215, SGI16215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3650[] = {SGI16207, SGI16209, SGI16210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16214, SGI0, SGI0, SGI0, SGI16216, SGI16216, SGI16217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3651[] = {SGI16218, SGI16219, SGI16220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16221, SGI0, SGI0, SGI16222, SGI16222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3652[] = {SGI16226, SGI16227, SGI16228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16230, SGI0, SGI0, SGI16232, SGI16232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3653[] = {SGI16223, SGI16224, SGI16225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16229, SGI0, SGI0, SGI0, SGI16231, SGI16231, SGI16233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3654[] = {SGI16234, SGI16235, SGI16237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16240, SGI0, SGI0, SGI16241, SGI16241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3655[] = {SGI16236, SGI16238, SGI16239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16242, SGI0, SGI0, SGI0, SGI16243, SGI16243, SGI16244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3656[] = {SGI16245, SGI16246, SGI16247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16248, SGI0, SGI0, SGI16249, SGI16249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3657[] = {SGI16250, SGI16251, SGI16252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16255, SGI0, SGI0, SGI0, SGI16257, SGI16257, SGI16260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3658[] = {SGI16253, SGI16254, SGI16256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16258, SGI0, SGI0, SGI16259, SGI16259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3659[] = {SGI16261, SGI16262, SGI16263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16264, SGI0, SGI0, SGI0, SGI16265, SGI16265, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3660[] = {SGI16266, SGI16267, SGI16268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16269, SGI16269, SGI0, SGI0, SGI0, SGI16270, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3661[] = {SGI16271, SGI16272, SGI16273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16274, SGI0, SGI0, SGI0, SGI16275, SGI16275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3662[] = {SGI16276, SGI16277, SGI16278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16279, SGI0, SGI0, SGI0, SGI16280, SGI16280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3663[] = {SGI0, SGI16281, NULL};
const Segment * const SG3664[] = {SGI16282, SGI16283, SGI16284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16285, SGI0, SGI0, SGI0, SGI16286, SGI16286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3665[] = {SGI16287, SGI0, SGI16288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16289, SGI0, SGI0, SGI0, SGI16290, SGI16290, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3666[] = {SGI16291, SGI16292, SGI16293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16294, SGI0, SGI0, SGI0, SGI16295, SGI16295, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3667[] = {SGI16296, SGI16297, SGI16298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16299, SGI0, SGI0, SGI0, SGI16300, SGI16301, NULL};
const Segment * const SG3668[] = {SGI0, SGI16302, SGI0, SGI16303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16304, SGI0, SGI0, SGI0, SGI16305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3669[] = {SGI0, SGI16306, SGI0, SGI16307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3670[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16309, NULL};
const Segment * const SG3671[] = {SGI0, SGI16310, SGI0, SGI16311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3672[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3673[] = {SGI16313, SGI16314, NULL};
const Segment * const SG3674[] = {SGI16315, SGI16316, SGI16317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16318, SGI0, SGI0, SGI0, SGI16319, SGI16319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3675[] = {SGI16320, SGI16321, SGI16322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16323, SGI0, SGI0, SGI0, SGI16324, SGI16324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3676[] = {SGI0, SGI16325, SGI16326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3677[] = {SGI0, SGI16327, SGI16328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3678[] = {SGI16329, SGI16330, SGI16331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16332, SGI16333, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3679[] = {SGI16334, SGI16335, SGI16336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16337, SGI0, SGI0, SGI0, SGI16338, SGI16338, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3680[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3681[] = {SGI16340, SGI16341, NULL};
const Segment * const SG3682[] = {SGI16342, SGI0, SGI16343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16344, SGI0, SGI0, SGI0, SGI16345, SGI16345, SGI0, SGI0, SGI0, SGI16346, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3683[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3684[] = {SGI0, SGI16348, NULL};
const Segment * const SG3685[] = {SGI16349, SGI0, SGI16350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16351, SGI0, SGI0, SGI0, SGI16352, SGI16352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16353, SGI0, SGI0, NULL};
const Segment * const SG3686[] = {SGI0, SGI16354, NULL};
const Segment * const SG3687[] = {SGI0, SGI16355, NULL};
const Segment * const SG3688[] = {SGI16356, SGI16357, SGI16358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16359, SGI0, SGI0, SGI0, SGI16360, SGI16360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3689[] = {SGI0, SGI16361, SGI0, SGI16362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16363, SGI0, SGI0, SGI0, SGI16364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3690[] = {SGI0, SGI16365, SGI0, SGI16366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3691[] = {SGI16367, SGI16368, SGI16369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16370, SGI0, SGI0, SGI0, SGI16371, SGI16371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3692[] = {SGI16372, SGI0, SGI16373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3693[] = {SGI16374, SGI0, SGI16375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16376, SGI0, SGI0, SGI0, SGI16377, SGI16377, SGI0, SGI0, SGI0, SGI16378, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3694[] = {SGI16379, SGI16380, SGI16381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16382, SGI0, SGI0, SGI0, SGI16383, SGI16384, NULL};
const Segment * const SG3695[] = {SGI16385, SGI16386, SGI16387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16388, SGI0, SGI0, SGI0, SGI16389, SGI16390, NULL};
const Segment * const SG3696[] = {SGI16391, SGI16392, SGI16393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16394, SGI0, SGI0, SGI0, SGI16395, SGI16396, NULL};
const Segment * const SG3697[] = {SGI16397, SGI16398, SGI16399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16400, SGI0, SGI0, SGI0, SGI16401, SGI16402, NULL};
const Segment * const SG3698[] = {SGI16403, SGI16404, SGI16405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16406, SGI0, SGI0, SGI0, SGI16407, SGI16407, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3699[] = {SGI16408, SGI0, SGI16409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3700[] = {SGI16410, SGI16411, SGI16412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16413, SGI0, SGI0, SGI0, SGI16414, SGI16414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3701[] = {SGI16415, SGI0, SGI16416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3702[] = {SGI0, SGI0, SGI16417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3703[] = {SGI16419, SGI16420, NULL};
const Segment * const SG3704[] = {SGI16421, SGI16422, NULL};
const Segment * const SG3705[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3706[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3707[] = {SGI16425, SGI16426, SGI16427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16428, SGI16428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16429, SGI0, SGI0, NULL};
const Segment * const SG3708[] = {SGI16430, SGI16431, SGI16432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16433, SGI0, SGI0, SGI0, SGI16434, SGI16434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3709[] = {SGI0, SGI16435, SGI0, SGI16436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16437, SGI0, SGI0, SGI0, SGI16438, SGI0, NULL};
const Segment * const SG3710[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3711[] = {SGI16441, SGI16442, SGI16443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16444, SGI0, SGI0, SGI0, SGI16445, SGI16445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3712[] = {SGI16446, SGI16447, SGI16448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16449, SGI0, SGI0, SGI0, SGI16450, SGI16450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3713[] = {SGI0, SGI16439, SGI0, SGI16451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3714[] = {SGI16452, SGI16453, SGI16454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16455, SGI16455, SGI0, SGI0, SGI0, SGI16456, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3715[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16457, SGI0, SGI0, SGI16458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3716[] = {SGI16459, SGI16460, SGI16461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16462, SGI0, SGI0, SGI0, SGI16463, SGI16464, NULL};
const Segment * const SG3717[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3718[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3719[] = {SGI0, SGI16467, NULL};
const Segment * const SG3720[] = {SGI16468, SGI16469, SGI16470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16471, SGI0, SGI0, SGI16472, SGI16472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3721[] = {SGI16473, SGI16474, SGI16475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16476, SGI0, SGI0, SGI16477, SGI16477, SGI16478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3722[] = {SGI16479, SGI16480, SGI16481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16482, SGI0, SGI0, SGI16483, SGI16483, SGI16484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3723[] = {SGI16485, SGI16486, SGI16487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16488, SGI0, SGI0, SGI0, SGI16489, SGI16489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3724[] = {SGI16490, SGI16491, SGI16492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16493, SGI0, SGI0, SGI0, SGI16494, SGI16494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3725[] = {SGI16495, SGI16496, SGI16497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16498, SGI0, SGI0, SGI0, SGI16499, SGI16499, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3726[] = {SGI0, SGI16500, SGI0, SGI16501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16502, SGI0, SGI0, SGI0, SGI16503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3727[] = {SGI16504, SGI16505, SGI16506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16507, SGI0, SGI0, SGI0, SGI16508, SGI16508, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3728[] = {SGI0, SGI16509, SGI0, SGI16510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3729[] = {SGI0, SGI16511, NULL};
const Segment * const SG3730[] = {SGI16512, SGI0, SGI16513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16514, SGI0, SGI0, SGI0, SGI16515, SGI16515, SGI0, SGI0, SGI16516, SGI0, NULL};
const Segment * const SG3731[] = {SGI16517, SGI16518, SGI16519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16520, SGI0, SGI0, SGI0, SGI16521, SGI16522, SGI0, SGI0, SGI16523, SGI0, NULL};
const Segment * const SG3732[] = {SGI16524, SGI0, SGI16525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16526, SGI0, SGI0, SGI0, SGI16527, SGI16527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16528, SGI0, SGI0, NULL};
const Segment * const SG3733[] = {SGI16529, SGI16530, NULL};
const Segment * const SG3734[] = {SGI16531, SGI16532, NULL};
const Segment * const SG3735[] = {SGI16533, SGI16534, NULL};
const Segment * const SG3736[] = {SGI16535, SGI16536, NULL};
const Segment * const SG3737[] = {SGI16537, SGI16538, NULL};
const Segment * const SG3738[] = {SGI16539, SGI16540, NULL};
const Segment * const SG3739[] = {SGI16541, SGI16542, SGI16543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16544, SGI0, SGI0, SGI0, SGI16545, SGI16545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3740[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3741[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16547, SGI0, SGI0, SGI16548, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3742[] = {SGI16549, SGI16550, SGI16551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16552, SGI0, SGI0, SGI0, SGI16553, SGI16553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3743[] = {SGI16554, SGI16555, SGI16556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16557, SGI16558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3744[] = {SGI16559, SGI16560, SGI16561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16562, SGI16562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3745[] = {SGI16563, SGI16564, SGI16565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16566, SGI0, SGI0, SGI0, SGI16567, SGI16567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3746[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3747[] = {SGI16569, SGI16570, SGI16571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16572, SGI16572, SGI0, SGI0, SGI0, SGI16573, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3748[] = {SGI16574, SGI16575, SGI16576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16577, SGI16577, SGI0, SGI0, SGI0, SGI16578, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3749[] = {SGI16579, SGI16580, SGI16581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16582, SGI0, SGI0, SGI16583, SGI16583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3750[] = {SGI16584, SGI16585, SGI16586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16587, SGI0, SGI0, SGI0, SGI16588, SGI16588, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3751[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3752[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3753[] = {SGI0, SGI16591, SGI16592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16593, SGI0, SGI0, SGI16594, SGI16594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3754[] = {SGI16595, SGI0, SGI16596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16597, SGI0, SGI0, SGI0, SGI16598, SGI16598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16599, SGI0, SGI0, NULL};
const Segment * const SG3755[] = {SGI16600, SGI16601, SGI16602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16603, SGI0, SGI0, SGI0, SGI16604, SGI16604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3756[] = {SGI16605, SGI16606, SGI16607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16608, SGI0, SGI0, SGI0, SGI16609, SGI16609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3757[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16610, SGI0, SGI0, SGI16611, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3758[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16612, SGI0, SGI0, SGI16613, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3759[] = {SGI16614, SGI16615, SGI16616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16617, SGI0, SGI0, SGI0, SGI16618, SGI16618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3760[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16619, SGI0, SGI0, SGI16620, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3761[] = {SGI16621, SGI0, SGI16622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16623, SGI0, SGI0, SGI0, SGI16624, SGI16624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16625, SGI0, SGI0, NULL};
const Segment * const SG3762[] = {SGI16626, SGI16627, SGI16628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16629, SGI0, SGI0, SGI0, SGI16630, SGI16630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3763[] = {SGI16631, SGI16632, SGI16633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16634, SGI0, SGI0, SGI0, SGI16635, SGI16635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3764[] = {SGI16636, SGI16637, SGI16638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16639, SGI16639, SGI0, SGI0, SGI0, SGI16640, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3765[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16641, SGI0, SGI0, SGI16642, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3766[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3767[] = {SGI16644, SGI16645, NULL};
const Segment * const SG3768[] = {SGI16646, SGI16647, NULL};
const Segment * const SG3769[] = {SGI16648, SGI16649, NULL};
const Segment * const SG3770[] = {SGI16650, SGI16651, NULL};
const Segment * const SG3771[] = {SGI16652, SGI16653, SGI16654, SGI0, SGI0, SGI0, SGI16655, SGI16656, SGI0, SGI16657, SGI0, SGI0, SGI0, SGI16658, SGI16658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3772[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16659, SGI0, SGI0, SGI16660, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3773[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16661, SGI0, SGI0, SGI16662, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3774[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16667, SGI0, SGI0, SGI16669, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3775[] = {SGI16663, SGI16664, SGI16665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16666, SGI0, SGI0, SGI0, SGI16668, SGI16668, SGI0, SGI0, SGI0, SGI0, SGI16670, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3776[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16671, SGI0, SGI0, SGI16672, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3777[] = {SGI16673, SGI16674, SGI16675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16676, SGI0, SGI0, SGI0, SGI16677, SGI16677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3778[] = {SGI16678, SGI0, SGI16679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16680, SGI0, SGI0, SGI0, SGI16681, SGI16681, SGI0, SGI0, SGI0, SGI16682, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3779[] = {SGI16683, SGI16684, SGI16685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16686, SGI0, SGI0, SGI0, SGI16687, SGI16688, SGI0, SGI0, SGI0, SGI16689, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3780[] = {SGI16690, SGI16691, SGI16692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16693, SGI0, SGI0, SGI0, SGI16694, SGI16694, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3781[] = {SGI16695, SGI16696, SGI16697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16698, SGI16699, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3782[] = {SGI16700, SGI16701, SGI16702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16703, SGI0, SGI0, SGI0, SGI16704, SGI16704, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3783[] = {SGI16705, SGI16706, SGI16707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16708, SGI0, SGI0, SGI0, SGI16709, SGI16709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3784[] = {SGI16710, SGI16711, NULL};
const Segment * const SG3785[] = {SGI16712, SGI16713, NULL};
const Segment * const SG3786[] = {SGI16714, SGI16715, SGI16716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16717, SGI0, SGI0, SGI0, SGI16718, SGI16718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3787[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3788[] = {SGI16720, SGI16721, SGI16722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16723, SGI0, SGI0, SGI0, SGI16724, SGI16724, SGI0, SGI16725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3789[] = {SGI16726, SGI16727, SGI16728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16729, SGI16729, SGI0, SGI0, SGI0, SGI16730, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3790[] = {SGI16731, SGI16732, SGI16733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16734, SGI0, SGI0, SGI0, SGI16735, SGI16735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3791[] = {SGI16736, SGI16737, SGI16738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16739, SGI0, SGI0, SGI0, SGI16740, SGI16740, NULL};
const Segment * const SG3792[] = {SGI16741, SGI0, SGI16742, SGI0, SGI0, SGI0, SGI16743, SGI16744, SGI0, SGI16745, SGI0, SGI0, SGI0, SGI16746, SGI16746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3793[] = {SGI16747, SGI16748, SGI16749, SGI0, SGI0, SGI0, SGI16750, SGI16751, SGI0, SGI16752, SGI0, SGI0, SGI0, SGI16753, SGI16753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3794[] = {SGI16754, SGI0, SGI16755, SGI0, SGI0, SGI0, SGI16756, SGI16757, SGI0, SGI16758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3795[] = {SGI16759, SGI0, SGI16760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16761, SGI0, SGI0, SGI0, SGI16762, SGI16762, SGI0, SGI0, SGI0, SGI16763, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3796[] = {SGI16764, SGI16765, SGI16766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16767, SGI0, SGI0, SGI0, SGI16768, SGI16768, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3797[] = {SGI16769, SGI0, SGI16770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3798[] = {SGI16771, SGI16772, SGI16773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16774, SGI16774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16775, SGI0, SGI0, NULL};
const Segment * const SG3799[] = {SGI0, SGI16776, SGI0, SGI16777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3800[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3801[] = {SGI16779, SGI16780, SGI16781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16782, SGI0, SGI0, SGI0, SGI16783, SGI16783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3802[] = {SGI16784, SGI16785, SGI16786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16787, SGI0, SGI0, SGI0, SGI16788, SGI16788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3803[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16789, SGI0, SGI0, SGI0, SGI0, SGI16790, SGI0, SGI0, NULL};
const Segment * const SG3804[] = {SGI16791, SGI16792, SGI16793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16794, SGI0, SGI0, SGI0, SGI16795, SGI16795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3805[] = {SGI16796, SGI16797, SGI16798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16799, SGI0, SGI0, SGI16800, SGI16800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3806[] = {SGI16801, SGI16802, SGI16803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16804, SGI0, SGI0, SGI0, SGI16805, SGI16805, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3807[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16806, SGI0, SGI0, SGI16807, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3808[] = {SGI16808, SGI16809, SGI16810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16811, SGI0, SGI0, SGI0, SGI16812, SGI16813, NULL};
const Segment * const SG3809[] = {SGI16814, SGI16815, SGI16816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16817, SGI0, SGI0, SGI0, SGI16818, SGI16819, NULL};
const Segment * const SG3810[] = {SGI16820, SGI16821, SGI16822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16823, SGI0, SGI0, SGI0, SGI16824, SGI16824, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3811[] = {SGI16825, SGI16826, SGI16827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16828, SGI0, SGI0, SGI0, SGI16829, SGI16829, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3812[] = {SGI16830, SGI16831, SGI16832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16833, SGI0, SGI0, SGI0, SGI16834, SGI16834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3813[] = {SGI0, SGI16835, SGI0, SGI16836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3814[] = {SGI16837, SGI16838, SGI16839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16840, SGI0, SGI0, SGI16841, SGI16841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3815[] = {SGI16842, SGI16843, SGI16844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16845, SGI0, SGI0, SGI16846, SGI16846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3816[] = {SGI16847, SGI16848, SGI16849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16850, SGI0, SGI0, SGI16851, SGI16851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3817[] = {SGI16852, SGI16853, SGI16854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16855, SGI0, SGI0, SGI16856, SGI16856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3818[] = {SGI16857, SGI16858, SGI16859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16860, SGI0, SGI0, SGI16861, SGI16861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3819[] = {SGI16862, SGI16863, SGI16864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16865, SGI0, SGI0, SGI16866, SGI16866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3820[] = {SGI16867, SGI16868, SGI16869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16870, SGI0, SGI0, SGI0, SGI16871, SGI16871, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3821[] = {SGI16872, SGI16873, SGI16874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16875, SGI16876, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3822[] = {SGI16877, SGI16878, SGI16879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16880, SGI16880, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3823[] = {SGI16881, SGI16882, SGI16883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16884, SGI0, SGI0, SGI0, SGI16885, SGI16885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3824[] = {SGI16886, SGI16887, SGI16888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16889, SGI16889, SGI0, SGI0, SGI16890, SGI0, NULL};
const Segment * const SG3825[] = {SGI16891, SGI0, SGI16892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16893, SGI0, SGI0, SGI16894, SGI16894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3826[] = {SGI16895, SGI16896, SGI16897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16898, SGI0, SGI0, SGI16899, SGI16899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3827[] = {SGI16900, SGI16901, SGI16902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16903, SGI0, SGI0, SGI0, SGI16904, SGI16905, NULL};
const Segment * const SG3828[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3829[] = {SGI16907, SGI16908, SGI16909, SGI0, SGI0, SGI0, SGI16910, SGI16911, SGI0, SGI16912, SGI0, SGI0, SGI0, SGI16913, SGI16913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3830[] = {SGI16914, SGI16915, SGI16916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16917, SGI0, SGI0, SGI0, SGI16918, SGI16918, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3831[] = {SGI16919, SGI16920, SGI16921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16922, SGI0, SGI0, SGI0, SGI16923, SGI16923, SGI0, SGI0, SGI0, SGI0, SGI16924, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3832[] = {SGI16925, SGI0, SGI16926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16927, SGI0, SGI0, SGI0, SGI16928, SGI16928, SGI0, SGI0, SGI16929, SGI0, NULL};
const Segment * const SG3833[] = {SGI16930, SGI16931, SGI16932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16933, SGI0, SGI0, SGI0, SGI16934, SGI16935, SGI0, SGI0, SGI16936, SGI0, NULL};
const Segment * const SG3834[] = {SGI16937, SGI16938, SGI16939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16940, SGI0, SGI0, SGI0, SGI16941, SGI16941, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3835[] = {SGI16942, SGI16943, SGI16944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16945, SGI0, SGI0, SGI0, SGI16946, SGI16946, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3836[] = {SGI16947, SGI16948, SGI16949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16950, SGI0, SGI0, SGI0, SGI16951, SGI16951, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3837[] = {SGI0, SGI16952, SGI0, SGI16953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3838[] = {SGI16954, SGI16955, SGI16956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16957, SGI0, SGI0, SGI16958, SGI16958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3839[] = {SGI16959, SGI16960, SGI16961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16962, SGI0, SGI0, SGI16963, SGI16963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3840[] = {SGI16964, SGI16965, SGI16966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16967, SGI0, SGI0, SGI16968, SGI16968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3841[] = {SGI16969, SGI0, SGI16970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16971, SGI0, SGI0, SGI0, SGI16972, SGI16972, SGI0, SGI0, SGI0, SGI16973, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3842[] = {SGI16974, SGI0, SGI16975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16976, SGI0, SGI0, SGI0, SGI16977, SGI16977, SGI0, SGI0, SGI16978, SGI0, NULL};
const Segment * const SG3843[] = {SGI16979, SGI16980, SGI16981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16982, SGI0, SGI0, SGI16983, SGI16983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3844[] = {SGI16984, SGI16985, SGI16986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16987, SGI0, SGI0, SGI16988, SGI16988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3845[] = {SGI16989, SGI16990, SGI16991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16992, SGI0, SGI0, SGI16993, SGI16993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3846[] = {SGI16994, SGI16995, SGI16996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI16997, SGI0, SGI0, SGI16998, SGI16998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3847[] = {SGI16999, SGI17000, SGI17001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17002, SGI0, SGI0, SGI17003, SGI17003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3848[] = {SGI17004, SGI17005, SGI17006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17007, SGI0, SGI0, SGI17008, SGI17008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3849[] = {SGI17009, SGI17010, SGI17011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17012, SGI0, SGI0, SGI17013, SGI17013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3850[] = {SGI17014, SGI17015, SGI17016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17017, SGI0, SGI0, SGI17018, SGI17018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3851[] = {SGI17019, SGI17020, SGI17021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17022, SGI0, SGI0, SGI17023, SGI17023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3852[] = {SGI17024, SGI17025, SGI17026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17027, SGI0, SGI0, SGI17028, SGI17028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3853[] = {SGI17029, SGI17030, SGI17031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17032, SGI0, SGI0, SGI17033, SGI17033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3854[] = {SGI17034, SGI17035, SGI17036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17037, SGI0, SGI0, SGI17038, SGI17038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3855[] = {SGI17039, SGI17040, SGI17041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17042, SGI0, SGI0, SGI17043, SGI17043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3856[] = {SGI17044, SGI17045, SGI17046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17047, SGI0, SGI0, SGI17048, SGI17048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3857[] = {SGI17049, SGI17050, SGI17051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17052, SGI0, SGI0, SGI17053, SGI17054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3858[] = {SGI17055, SGI0, SGI17056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17057, SGI0, SGI0, SGI0, SGI17058, SGI17058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17059, SGI0, SGI0, NULL};
const Segment * const SG3859[] = {SGI17060, SGI17061, SGI17062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17063, SGI0, SGI0, SGI0, SGI17064, SGI17064, SGI0, SGI0, SGI0, SGI17065, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3860[] = {SGI17066, SGI17067, SGI17068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17069, SGI0, SGI0, SGI0, SGI17070, SGI17070, SGI0, SGI0, SGI0, SGI17071, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3861[] = {SGI17072, SGI17073, SGI17074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17075, SGI0, SGI0, SGI0, SGI17076, SGI17076, SGI0, SGI0, SGI0, SGI17077, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3862[] = {SGI17078, SGI0, SGI17079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17080, SGI0, SGI0, SGI0, SGI17081, SGI17081, SGI0, SGI0, SGI0, SGI17082, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3863[] = {SGI17083, SGI17084, SGI17085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17086, SGI0, SGI0, SGI0, SGI17087, SGI17087, SGI0, SGI0, SGI0, SGI17088, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3864[] = {SGI17089, SGI17090, SGI17091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17092, SGI0, SGI0, SGI0, SGI17093, SGI17094, NULL};
const Segment * const SG3865[] = {SGI17095, SGI0, SGI17096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17097, SGI0, SGI0, SGI0, SGI17098, SGI17098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17099, SGI0, SGI0, NULL};
const Segment * const SG3866[] = {SGI17100, SGI17101, NULL};
const Segment * const SG3867[] = {SGI17102, SGI17103, NULL};
const Segment * const SG3868[] = {SGI0, SGI17104, NULL};
const Segment * const SG3869[] = {SGI17105, SGI17106, SGI17107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17108, SGI0, SGI0, SGI0, SGI17109, SGI17110, NULL};
const Segment * const SG3870[] = {SGI0, SGI17111, SGI17112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17113, SGI0, SGI0, SGI0, SGI17114, SGI17114, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3871[] = {SGI0, SGI17115, SGI17116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17117, SGI0, SGI0, SGI0, SGI17118, SGI17118, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3872[] = {SGI17119, SGI0, SGI17120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17123, SGI0, SGI0, SGI0, SGI17124, SGI17124, SGI0, SGI0, SGI0, SGI17126, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3873[] = {SGI0, SGI17121, SGI17122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17125, SGI0, SGI0, SGI0, SGI17127, SGI17127, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3874[] = {SGI0, SGI17128, NULL};
const Segment * const SG3875[] = {SGI17129, SGI17130, SGI17131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17132, SGI0, SGI0, SGI0, SGI17133, SGI17133, NULL};
const Segment * const SG3876[] = {SGI17134, SGI17135, SGI17136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17137, SGI0, SGI0, SGI0, SGI17138, SGI17138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3877[] = {SGI17139, SGI17140, SGI17141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17142, SGI0, SGI0, SGI0, SGI17143, SGI17143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3878[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17144, SGI0, SGI17145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3879[] = {SGI17146, SGI0, SGI17147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17148, SGI0, SGI0, SGI0, SGI17149, SGI17149, SGI0, SGI0, SGI17150, SGI0, NULL};
const Segment * const SG3880[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3881[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3882[] = {SGI17153, SGI17154, SGI17155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17156, SGI0, SGI0, SGI0, SGI17157, SGI17157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3883[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17158, SGI0, SGI0, SGI17159, SGI17159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3884[] = {SGI17160, SGI17161, SGI17162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17163, SGI0, SGI0, SGI0, SGI17164, SGI17164, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3885[] = {SGI17165, SGI17166, SGI17167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17168, SGI0, SGI0, SGI0, SGI17169, SGI17170, NULL};
const Segment * const SG3886[] = {SGI17171, SGI17172, SGI17173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17174, SGI0, SGI0, SGI0, SGI17175, SGI17175, NULL};
const Segment * const SG3887[] = {SGI17176, SGI0, SGI17177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17178, SGI0, SGI0, SGI0, SGI17179, SGI17179, SGI0, SGI0, SGI17180, SGI0, NULL};
const Segment * const SG3888[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3889[] = {SGI17182, SGI17183, SGI17184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17185, SGI0, SGI0, SGI0, SGI17186, SGI17186, SGI0, SGI17187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3890[] = {SGI17188, SGI17189, SGI17190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17191, SGI0, SGI0, SGI0, SGI17192, SGI17192, SGI0, SGI17193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3891[] = {SGI17194, SGI17195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3892[] = {SGI17197, SGI17198, SGI17199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17200, SGI17200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17201, SGI0, SGI0, NULL};
const Segment * const SG3893[] = {SGI17202, SGI17203, SGI17204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17205, SGI0, SGI0, SGI0, SGI17206, SGI17207, SGI0, SGI0, SGI0, SGI17208, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3894[] = {SGI17209, SGI17210, SGI17211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17212, SGI17212, SGI0, SGI0, SGI17213, SGI0, NULL};
const Segment * const SG3895[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3896[] = {SGI17214, SGI17215, SGI17216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17218, SGI0, SGI0, SGI0, SGI17219, SGI17219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3897[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17220, SGI0, SGI0, SGI17221, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3898[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17222, SGI0, SGI0, SGI17223, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3899[] = {SGI17224, SGI0, SGI17225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17226, SGI0, SGI0, SGI0, SGI17227, SGI17227, SGI0, SGI0, SGI0, SGI17228, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3900[] = {SGI17229, SGI17230, SGI17231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17232, SGI0, SGI0, SGI0, SGI17233, SGI17234, SGI0, SGI0, SGI0, SGI17235, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3901[] = {SGI0, SGI17236, NULL};
const Segment * const SG3902[] = {SGI17237, SGI17238, SGI17239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17240, SGI0, SGI0, SGI0, SGI17241, SGI17241, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3903[] = {SGI17242, SGI17243, SGI17244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17245, SGI0, SGI0, SGI0, SGI17246, SGI17246, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3904[] = {SGI17247, SGI17248, SGI17249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17250, SGI0, SGI0, SGI0, SGI17251, SGI17252, NULL};
const Segment * const SG3905[] = {SGI17253, SGI0, SGI17254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17255, SGI0, SGI0, SGI0, SGI17256, SGI17256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17257, SGI0, SGI0, NULL};
const Segment * const SG3906[] = {SGI17258, SGI17259, SGI17260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17261, SGI17261, SGI0, SGI0, SGI0, SGI17262, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3907[] = {SGI17263, SGI17264, SGI17265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17266, SGI17266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3908[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17267, SGI17268, SGI0, SGI0, SGI0, SGI0, SGI17269, NULL};
const Segment * const SG3909[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17270, SGI0, SGI0, SGI17271, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3910[] = {SGI17272, SGI0, SGI17273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17274, SGI0, SGI0, SGI0, SGI17275, SGI17275, SGI0, SGI0, SGI17276, SGI0, NULL};
const Segment * const SG3911[] = {SGI17277, SGI17278, SGI17279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17280, SGI0, SGI0, SGI0, SGI17281, SGI17281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3912[] = {SGI17282, SGI17283, SGI17284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17285, SGI17285, SGI0, SGI0, SGI17286, SGI0, NULL};
const Segment * const SG3913[] = {SGI17287, SGI17288, SGI17289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17290, SGI17290, SGI0, SGI0, SGI17291, SGI0, NULL};
const Segment * const SG3914[] = {SGI17292, SGI17293, SGI17294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17295, SGI0, SGI0, SGI0, SGI17296, SGI17296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3915[] = {SGI17297, SGI17298, SGI17299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17300, SGI0, SGI0, SGI0, SGI17301, SGI17301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17302, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3916[] = {SGI17303, SGI0, SGI17304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17305, SGI0, SGI0, SGI0, SGI17306, SGI17306, SGI0, SGI0, SGI17307, SGI0, NULL};
const Segment * const SG3917[] = {SGI17308, SGI17309, SGI17310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17311, SGI0, SGI0, SGI0, SGI17312, SGI17313, SGI0, SGI0, SGI17314, SGI0, NULL};
const Segment * const SG3918[] = {SGI0, SGI17317, NULL};
const Segment * const SG3919[] = {SGI17315, SGI17316, SGI17318, SGI17319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17320, SGI0, SGI0, SGI0, SGI17321, SGI17322, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3920[] = {SGI17323, SGI17324, SGI17325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17326, SGI0, SGI0, SGI0, SGI17327, SGI17327, NULL};
const Segment * const SG3921[] = {SGI17328, SGI17329, SGI17330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17331, SGI0, SGI0, SGI0, SGI17332, SGI17332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3922[] = {SGI17333, SGI17334, SGI17335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17336, SGI0, SGI0, SGI0, SGI17337, SGI17337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3923[] = {SGI17338, SGI17339, SGI17340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17341, SGI0, SGI0, SGI0, SGI17342, SGI17342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3924[] = {SGI17343, SGI17344, SGI17345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17346, SGI17347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3925[] = {SGI17348, SGI17349, SGI17350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17351, SGI0, SGI0, SGI0, SGI17352, SGI17352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3926[] = {SGI17353, SGI17354, SGI17355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17356, SGI0, SGI0, SGI0, SGI17357, SGI17357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3927[] = {SGI17358, SGI17359, SGI17360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17361, SGI17361, SGI0, SGI0, SGI17362, SGI0, NULL};
const Segment * const SG3928[] = {SGI17363, SGI17364, NULL};
const Segment * const SG3929[] = {SGI17365, SGI17366, NULL};
const Segment * const SG3930[] = {SGI17367, SGI0, SGI17368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17369, SGI0, SGI0, SGI0, SGI17370, SGI17370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17371, SGI0, SGI0, NULL};
const Segment * const SG3931[] = {SGI17372, SGI17373, SGI17374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17375, SGI0, SGI0, SGI0, SGI17376, SGI17377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17378, SGI0, SGI0, NULL};
const Segment * const SG3932[] = {SGI17379, SGI17380, SGI17381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17382, SGI0, SGI0, SGI0, SGI17383, SGI17383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3933[] = {SGI0, SGI17384, NULL};
const Segment * const SG3934[] = {SGI0, SGI17385, SGI17386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17387, SGI0, SGI0, SGI17388, SGI17388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3935[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17389, SGI0, SGI0, SGI17390, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3936[] = {SGI17391, SGI17392, SGI17393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17394, SGI0, SGI0, SGI0, SGI17395, SGI17395, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3937[] = {SGI17396, SGI17397, SGI17398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17399, SGI17399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17400, SGI0, SGI0, NULL};
const Segment * const SG3938[] = {SGI17401, SGI17402, SGI17403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17404, SGI17404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17405, SGI0, SGI0, NULL};
const Segment * const SG3939[] = {SGI17406, SGI0, SGI17407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17408, SGI0, SGI0, SGI0, SGI17409, SGI17409, SGI0, SGI0, SGI0, SGI17410, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3940[] = {SGI17411, SGI17412, SGI17413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17414, SGI0, SGI0, SGI17415, SGI17415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3941[] = {SGI17416, SGI17417, SGI17418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17419, SGI0, SGI0, SGI17420, SGI17420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3942[] = {SGI17421, SGI17422, SGI17423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17424, SGI0, SGI0, SGI17425, SGI17425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3943[] = {SGI17426, SGI17427, SGI17428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17429, SGI0, SGI0, SGI17430, SGI17430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3944[] = {SGI17431, SGI17432, SGI17433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17434, SGI0, SGI0, SGI17435, SGI17435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3945[] = {SGI17436, SGI17437, SGI17438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17439, SGI0, SGI0, SGI17440, SGI17440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3946[] = {SGI17441, SGI17442, SGI17443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17444, SGI0, SGI0, SGI17445, SGI17445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3947[] = {SGI17446, SGI17447, SGI17448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17449, SGI0, SGI0, SGI17450, SGI17450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3948[] = {SGI17451, SGI17452, NULL};
const Segment * const SG3949[] = {SGI17453, SGI0, SGI17454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17455, SGI0, SGI0, SGI0, SGI17456, SGI17456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17457, SGI0, SGI0, NULL};
const Segment * const SG3950[] = {SGI0, SGI17458, NULL};
const Segment * const SG3951[] = {SGI17459, SGI0, SGI17460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17461, SGI0, SGI0, SGI0, SGI17462, SGI17462, SGI0, SGI0, SGI17463, SGI0, NULL};
const Segment * const SG3952[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3953[] = {SGI17465, SGI17466, SGI17467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17470, SGI0, SGI0, SGI0, SGI17471, SGI17471, SGI0, SGI0, SGI17472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3954[] = {SGI0, SGI17468, SGI17469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17473, SGI0, SGI0, SGI0, SGI0, SGI17474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3955[] = {SGI17475, SGI17476, SGI17477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17478, SGI17479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3956[] = {SGI17480, SGI17481, SGI17482, SGI17483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17484, SGI0, SGI0, SGI0, SGI17485, SGI17486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3957[] = {SGI17487, SGI17488, SGI17489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17490, SGI0, SGI0, SGI0, SGI17491, SGI17491, SGI0, SGI0, SGI17492, SGI0, SGI0, SGI0, SGI17493, SGI0, SGI0, NULL};
const Segment * const SG3958[] = {SGI17494, SGI17495, SGI17496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17497, SGI0, SGI0, SGI17498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3959[] = {SGI17499, SGI17500, SGI17501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17502, SGI0, SGI0, SGI0, SGI17503, SGI17503, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3960[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3961[] = {SGI17505, SGI17506, SGI17507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17508, SGI0, SGI0, SGI0, SGI17509, SGI17509, SGI0, SGI0, SGI17510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3962[] = {SGI17511, SGI17512, SGI17513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17514, SGI0, SGI0, SGI0, SGI17515, SGI17515, SGI0, SGI0, SGI0, SGI17516, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3963[] = {SGI17517, SGI17518, SGI17519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17520, SGI0, SGI0, SGI0, SGI17521, SGI17521, SGI0, SGI0, SGI0, SGI17522, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3964[] = {SGI17523, SGI17524, NULL};
const Segment * const SG3965[] = {SGI17525, SGI17526, NULL};
const Segment * const SG3966[] = {SGI17527, SGI17528, NULL};
const Segment * const SG3967[] = {SGI17529, SGI0, SGI17530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17531, SGI0, SGI0, SGI0, SGI17532, SGI17532, SGI0, SGI0, SGI17533, SGI0, NULL};
const Segment * const SG3968[] = {SGI17534, SGI17535, SGI17536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17537, SGI17537, SGI0, SGI0, SGI17538, SGI0, NULL};
const Segment * const SG3969[] = {SGI17539, SGI17540, SGI17541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17542, SGI0, SGI0, SGI0, SGI17543, SGI17543, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3970[] = {SGI17544, SGI17545, SGI17546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17547, SGI17547, SGI0, SGI0, SGI0, SGI17548, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3971[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3972[] = {SGI17550, SGI17551, SGI17552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17553, SGI0, SGI0, SGI0, SGI17554, SGI17554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3973[] = {SGI17555, SGI17556, SGI17557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17558, SGI0, SGI0, SGI0, SGI17559, SGI17559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3974[] = {SGI17560, SGI17561, SGI17562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17563, SGI0, SGI0, SGI0, SGI17564, SGI17565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3975[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3976[] = {SGI17567, SGI0, SGI17568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17569, SGI0, SGI0, SGI0, SGI17570, SGI17570, SGI0, SGI0, SGI0, SGI17571, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3977[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3978[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3979[] = {SGI17574, SGI17575, SGI17576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17577, SGI0, SGI0, SGI0, SGI17578, SGI17578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3980[] = {SGI17579, SGI0, SGI17580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17581, SGI0, SGI0, SGI0, SGI17582, SGI17582, SGI0, SGI0, SGI0, SGI17583, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3981[] = {SGI17584, SGI17585, SGI17586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17587, SGI0, SGI0, SGI0, SGI17588, SGI17588, SGI0, SGI17590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3982[] = {SGI0, SGI17589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17591, SGI0, SGI0, SGI0, SGI0, SGI17592, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3983[] = {SGI0, SGI17593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17594, SGI0, SGI0, SGI0, SGI0, SGI17595, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3984[] = {SGI0, SGI17596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17597, SGI0, SGI0, SGI0, SGI0, SGI17598, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3985[] = {SGI0, SGI17599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17600, SGI0, SGI0, SGI0, SGI0, SGI17601, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3986[] = {SGI17602, SGI17603, SGI17604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17605, SGI17606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3987[] = {SGI0, SGI0, SGI17607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3988[] = {SGI17609, SGI0, SGI17610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17611, SGI0, SGI0, SGI0, SGI17612, SGI17612, SGI0, SGI0, SGI17613, SGI0, NULL};
const Segment * const SG3989[] = {SGI17614, SGI0, SGI17615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17616, SGI0, SGI0, SGI0, SGI17617, SGI17617, SGI0, SGI0, SGI17618, SGI0, NULL};
const Segment * const SG3990[] = {SGI17619, SGI17620, NULL};
const Segment * const SG3991[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3992[] = {SGI0, SGI17622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17623, SGI0, SGI0, SGI0, SGI0, SGI17624, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3993[] = {SGI17625, SGI17626, NULL};
const Segment * const SG3994[] = {SGI0, SGI17627, SGI17628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17629, SGI0, SGI0, SGI0, SGI0, SGI17630, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3995[] = {SGI0, SGI17631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17632, SGI0, SGI0, SGI0, SGI0, SGI17633, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3996[] = {SGI17634, SGI17635, SGI17636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17637, SGI17638, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3997[] = {SGI17639, SGI17640, SGI17641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17642, SGI0, SGI0, SGI0, SGI17643, SGI17643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3998[] = {SGI17644, SGI17645, SGI17646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17647, SGI17648, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG3999[] = {SGI17649, SGI17650, SGI17651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17652, SGI17652, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4000[] = {SGI0, SGI17653, SGI17654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4001[] = {SGI17655, SGI17656, SGI17657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17658, SGI0, SGI0, SGI17659, SGI17659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4002[] = {SGI17660, SGI17661, SGI17662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17663, SGI0, SGI0, SGI17664, SGI17664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4003[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17665, SGI0, SGI0, SGI17666, SGI17666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4004[] = {SGI17667, SGI17668, SGI17669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17670, SGI0, SGI0, SGI17671, SGI17671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4005[] = {SGI0, SGI17672, SGI0, SGI17673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4006[] = {SGI0, SGI17674, SGI17675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17676, SGI0, SGI0, SGI0, SGI0, SGI17677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4007[] = {SGI17678, SGI17679, NULL};
const Segment * const SG4008[] = {SGI17680, SGI17681, NULL};
const Segment * const SG4009[] = {SGI17682, SGI17683, NULL};
const Segment * const SG4010[] = {SGI17684, SGI17685, NULL};
const Segment * const SG4011[] = {SGI17689, SGI17690, NULL};
const Segment * const SG4012[] = {SGI17686, SGI17687, SGI17688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17691, SGI0, SGI0, SGI0, SGI17692, SGI17693, NULL};
const Segment * const SG4013[] = {SGI17694, SGI17695, SGI17696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17697, SGI0, SGI0, SGI0, SGI17698, SGI17699, NULL};
const Segment * const SG4014[] = {SGI17700, SGI17701, SGI17702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17703, SGI0, SGI0, SGI0, SGI17704, SGI17705, NULL};
const Segment * const SG4015[] = {SGI17706, SGI17707, SGI17708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17709, SGI0, SGI0, SGI0, SGI17710, SGI17711, NULL};
const Segment * const SG4016[] = {SGI0, SGI17712, SGI0, SGI17713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17714, SGI0, SGI0, SGI0, SGI17715, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4017[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17717, SGI0, SGI0, SGI17719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4018[] = {SGI0, SGI17716, SGI0, SGI17718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4019[] = {SGI17720, SGI17721, SGI17722, SGI17723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17724, SGI0, SGI0, SGI0, SGI17725, SGI17726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4020[] = {SGI17727, SGI0, SGI17728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17729, SGI0, SGI0, SGI0, SGI17730, SGI17730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17731, SGI0, SGI0, NULL};
const Segment * const SG4021[] = {SGI17732, SGI0, SGI17733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17734, SGI0, SGI0, SGI0, SGI17735, SGI17735, SGI0, SGI0, SGI17736, SGI0, NULL};
const Segment * const SG4022[] = {SGI17737, SGI17738, NULL};
const Segment * const SG4023[] = {SGI17739, SGI17740, NULL};
const Segment * const SG4024[] = {SGI17741, SGI17742, NULL};
const Segment * const SG4025[] = {SGI17743, SGI17744, NULL};
const Segment * const SG4026[] = {SGI0, SGI17745, SGI0, SGI17746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4027[] = {SGI17747, SGI17748, SGI17749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17750, SGI0, SGI0, SGI17751, SGI17751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4028[] = {SGI17752, SGI17753, SGI17754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17755, SGI0, SGI0, SGI17756, SGI17757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4029[] = {SGI17758, SGI17759, NULL};
const Segment * const SG4030[] = {SGI17760, SGI17761, NULL};
const Segment * const SG4031[] = {SGI17762, SGI17763, NULL};
const Segment * const SG4032[] = {SGI17764, SGI17765, SGI17766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17767, SGI0, SGI0, SGI0, SGI17768, SGI17768, SGI0, SGI0, SGI0, SGI0, SGI17769, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4033[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4034[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17771, SGI0, SGI0, SGI17772, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4035[] = {SGI17773, SGI17774, SGI17775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17776, SGI0, SGI0, SGI0, SGI17777, SGI17777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4036[] = {SGI17778, SGI0, SGI17779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17780, SGI0, SGI0, SGI0, SGI17781, SGI17781, SGI0, SGI0, SGI17782, SGI0, NULL};
const Segment * const SG4037[] = {SGI17783, SGI17784, SGI17785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17786, SGI0, SGI0, SGI0, SGI17787, SGI17787, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4038[] = {SGI17788, SGI17789, SGI17790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17791, SGI17791, SGI0, SGI0, SGI0, SGI17792, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4039[] = {SGI17793, SGI17794, SGI17795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17796, SGI17796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17797, SGI0, SGI0, NULL};
const Segment * const SG4040[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17798, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4041[] = {SGI17799, SGI17800, SGI17801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17802, SGI17802, SGI0, SGI0, SGI0, SGI17803, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4042[] = {SGI17804, SGI17805, SGI17806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17807, SGI0, SGI0, SGI0, SGI17808, SGI17809, NULL};
const Segment * const SG4043[] = {SGI17810, SGI17811, SGI17812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17813, SGI17813, SGI0, SGI0, SGI17814, SGI0, NULL};
const Segment * const SG4044[] = {SGI17815, SGI17816, NULL};
const Segment * const SG4045[] = {SGI17817, SGI17818, SGI17819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17820, SGI17820, SGI0, SGI0, SGI0, SGI17821, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4046[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4047[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17824, NULL};
const Segment * const SG4048[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17825, SGI0, SGI0, SGI17826, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4049[] = {SGI17827, SGI17828, SGI17829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17830, SGI0, SGI0, SGI0, SGI17831, SGI17831, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4050[] = {SGI17832, SGI17833, NULL};
const Segment * const SG4051[] = {SGI17834, SGI17835, SGI17836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17837, SGI0, SGI0, SGI0, SGI17838, SGI17838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4052[] = {SGI17839, SGI17840, SGI17841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17842, SGI0, SGI0, SGI0, SGI17843, SGI17843, SGI0, SGI0, SGI0, SGI0, SGI17844, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4053[] = {SGI17845, SGI17846, SGI17847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17848, SGI17848, SGI0, SGI0, SGI17849, SGI0, NULL};
const Segment * const SG4054[] = {SGI17850, SGI17851, SGI17852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17853, SGI0, SGI0, SGI0, SGI17854, SGI17854, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4055[] = {SGI17855, SGI17856, SGI17857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17858, SGI17859, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4056[] = {SGI17860, SGI0, SGI17861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17862, SGI0, SGI0, SGI0, SGI17863, SGI17863, SGI0, SGI0, SGI0, SGI17864, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4057[] = {SGI17865, SGI17866, NULL};
const Segment * const SG4058[] = {SGI17867, SGI17868, NULL};
const Segment * const SG4059[] = {SGI17869, SGI17870, NULL};
const Segment * const SG4060[] = {SGI17871, SGI17872, SGI17873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17874, SGI17874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17875, SGI0, SGI0, NULL};
const Segment * const SG4061[] = {SGI17876, SGI17877, SGI17878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17879, SGI0, SGI0, SGI0, SGI17880, SGI17881, NULL};
const Segment * const SG4062[] = {SGI17882, SGI17883, NULL};
const Segment * const SG4063[] = {SGI17884, SGI17885, NULL};
const Segment * const SG4064[] = {SGI17886, SGI17887, NULL};
const Segment * const SG4065[] = {SGI17888, SGI17889, NULL};
const Segment * const SG4066[] = {SGI17890, SGI17891, SGI17892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17893, SGI17893, SGI0, SGI0, SGI0, SGI17894, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4067[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17896, NULL};
const Segment * const SG4068[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4069[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4070[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4071[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17900, SGI0, SGI0, SGI17901, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4072[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17902, SGI0, SGI0, SGI17903, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4073[] = {SGI17904, SGI17905, SGI17906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17907, SGI0, SGI0, SGI0, SGI17908, SGI17908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4074[] = {SGI17909, SGI17910, SGI17911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17912, SGI0, SGI0, SGI0, SGI17913, SGI17913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4075[] = {SGI17914, SGI17915, NULL};
const Segment * const SG4076[] = {SGI17916, SGI17917, NULL};
const Segment * const SG4077[] = {SGI17918, SGI17919, NULL};
const Segment * const SG4078[] = {SGI17920, SGI17921, NULL};
const Segment * const SG4079[] = {SGI17922, SGI17923, NULL};
const Segment * const SG4080[] = {SGI17924, SGI17925, NULL};
const Segment * const SG4081[] = {SGI17926, SGI17927, SGI17928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17929, SGI17929, SGI0, SGI0, SGI0, SGI17930, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4082[] = {SGI17931, SGI17932, SGI17933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17934, SGI0, SGI0, SGI0, SGI17935, SGI17935, SGI0, SGI17936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4083[] = {SGI17937, SGI17938, SGI17939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17940, SGI17940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17941, SGI0, SGI0, NULL};
const Segment * const SG4084[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4085[] = {SGI17943, SGI17944, SGI17945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17946, SGI0, SGI0, SGI0, SGI17947, SGI17947, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4086[] = {SGI17948, SGI17949, SGI17950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17951, SGI0, SGI0, SGI0, SGI17952, SGI17952, SGI0, SGI0, SGI0, SGI0, SGI17953, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4087[] = {SGI17954, SGI17955, NULL};
const Segment * const SG4088[] = {SGI17956, SGI17957, NULL};
const Segment * const SG4089[] = {SGI17958, SGI17959, SGI17960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17961, SGI0, SGI0, SGI0, SGI17962, SGI17962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4090[] = {SGI0, SGI17963, SGI0, SGI17964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4091[] = {SGI17965, SGI17966, NULL};
const Segment * const SG4092[] = {SGI17967, SGI17968, NULL};
const Segment * const SG4093[] = {SGI17969, SGI17970, SGI17971, SGI17972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17973, SGI0, SGI0, SGI0, SGI17974, SGI17975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4094[] = {SGI17976, SGI17977, NULL};
const Segment * const SG4095[] = {SGI17978, SGI17979, SGI17980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17981, SGI17981, NULL};
const Segment * const SG4096[] = {SGI17982, SGI17983, SGI17984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17985, SGI17985, SGI0, SGI0, SGI0, SGI17986, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4097[] = {SGI17987, SGI17988, SGI17989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17990, SGI17990, SGI0, SGI0, SGI0, SGI17991, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4098[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17993, NULL};
const Segment * const SG4099[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4100[] = {SGI17995, SGI17996, SGI17997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17998, SGI17998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI17999, SGI0, SGI0, NULL};
const Segment * const SG4101[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4102[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18001, SGI0, SGI0, SGI18002, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4103[] = {SGI18003, SGI18004, SGI18005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18006, SGI0, SGI0, SGI0, SGI18007, SGI18008, NULL};
const Segment * const SG4104[] = {SGI18009, SGI18010, SGI18011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18012, SGI0, SGI0, SGI0, SGI18013, SGI18013, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4105[] = {SGI18014, SGI18015, SGI18016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18017, SGI0, SGI0, SGI0, SGI18018, SGI18019, SGI0, SGI0, SGI0, SGI18020, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4106[] = {SGI18021, SGI0, SGI18022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18023, SGI0, SGI0, SGI0, SGI18024, SGI18024, SGI0, SGI0, SGI18025, SGI0, NULL};
const Segment * const SG4107[] = {SGI18026, SGI18027, SGI18028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18029, SGI0, SGI0, SGI0, SGI18030, SGI18031, NULL};
const Segment * const SG4108[] = {SGI18032, SGI0, SGI18033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18034, SGI0, SGI0, SGI0, SGI18035, SGI18035, SGI0, SGI0, SGI18036, SGI0, NULL};
const Segment * const SG4109[] = {SGI18037, SGI18038, SGI18039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18040, SGI18040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18041, SGI0, SGI0, NULL};
const Segment * const SG4110[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18042, SGI0, SGI0, SGI0, SGI0, SGI18043, SGI0, NULL};
const Segment * const SG4111[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18044, SGI0, SGI0, SGI18045, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4112[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18046, SGI0, SGI0, SGI18047, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4113[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18048, SGI0, SGI0, SGI18049, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4114[] = {SGI18050, SGI18051, SGI18052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18053, SGI0, SGI0, SGI0, SGI18054, SGI18054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4115[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4116[] = {SGI18056, SGI18057, SGI18058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18059, SGI0, SGI0, SGI0, SGI18060, SGI18060, SGI0, SGI0, SGI18061, SGI0, SGI0, SGI0, SGI18062, SGI0, SGI0, NULL};
const Segment * const SG4117[] = {SGI18063, SGI0, SGI18064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18065, SGI0, SGI0, SGI0, SGI18066, SGI18066, SGI0, SGI0, SGI18067, SGI0, NULL};
const Segment * const SG4118[] = {SGI18068, SGI18069, SGI18070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18071, SGI0, SGI0, SGI0, SGI18072, SGI18073, SGI0, SGI0, SGI18074, SGI0, NULL};
const Segment * const SG4119[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4120[] = {SGI18076, SGI0, SGI18077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18078, SGI0, SGI0, SGI0, SGI18079, SGI18079, SGI0, SGI0, SGI0, SGI18080, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4121[] = {SGI18081, SGI0, SGI18082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18083, SGI0, SGI0, SGI0, SGI18084, SGI18084, SGI0, SGI0, SGI18085, SGI0, NULL};
const Segment * const SG4122[] = {SGI18086, SGI18087, SGI18088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18089, SGI0, SGI0, SGI0, SGI18090, SGI18090, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4123[] = {SGI0, SGI18091, SGI0, SGI18092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18093, SGI0, SGI0, SGI0, SGI18094, SGI0, NULL};
const Segment * const SG4124[] = {SGI0, SGI18095, SGI0, SGI18096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4125[] = {SGI18097, SGI18098, SGI18099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18100, SGI0, SGI0, SGI0, SGI18101, SGI18102, NULL};
const Segment * const SG4126[] = {SGI18103, SGI18104, SGI18105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18106, SGI0, SGI0, SGI0, SGI18107, SGI18107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4127[] = {SGI18108, SGI0, SGI18109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18110, SGI0, SGI0, SGI0, SGI18111, SGI18111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18112, SGI0, SGI0, NULL};
const Segment * const SG4128[] = {SGI18113, SGI18114, SGI18115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18116, SGI0, SGI0, SGI0, SGI18118, SGI18118, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4129[] = {SGI18117, SGI18119, SGI18120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18121, SGI0, SGI0, SGI0, SGI18122, SGI18123, SGI0, SGI0, SGI0, SGI18124, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4130[] = {SGI18125, SGI18126, SGI18127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18128, SGI0, SGI0, SGI0, SGI18129, SGI18129, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4131[] = {SGI18130, SGI0, SGI18131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18132, SGI0, SGI0, SGI0, SGI18133, SGI18133, SGI0, SGI0, SGI18134, SGI0, NULL};
const Segment * const SG4132[] = {SGI18135, SGI18136, SGI18137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18138, SGI18138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18139, SGI0, SGI0, NULL};
const Segment * const SG4133[] = {SGI18140, SGI18141, SGI18142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18143, SGI0, SGI0, SGI18144, SGI18145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4134[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18146, SGI0, SGI0, SGI18147, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4135[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18148, SGI0, SGI0, SGI18149, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4136[] = {SGI18150, SGI18151, SGI18152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18153, SGI0, SGI0, SGI0, SGI18154, SGI18154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4137[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18155, SGI0, SGI0, SGI18156, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4138[] = {SGI18157, SGI18158, SGI18159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18160, SGI0, SGI0, SGI0, SGI18161, SGI18161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4139[] = {SGI18162, SGI18163, SGI18164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18165, SGI0, SGI0, SGI0, SGI18166, SGI18166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4140[] = {SGI18167, SGI18168, SGI18169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18170, SGI0, SGI0, SGI0, SGI18171, SGI18171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4141[] = {SGI18172, SGI18173, SGI18174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18175, SGI0, SGI0, SGI0, SGI18176, SGI18176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4142[] = {SGI18177, SGI18178, SGI18179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18180, SGI0, SGI0, SGI0, SGI18181, SGI18181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4143[] = {SGI18182, SGI18183, SGI18184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18185, SGI0, SGI0, SGI0, SGI18186, SGI18187, NULL};
const Segment * const SG4144[] = {SGI18188, SGI18189, SGI18190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18191, SGI0, SGI0, SGI0, SGI18192, SGI18193, NULL};
const Segment * const SG4145[] = {SGI18194, SGI18195, SGI18196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18197, SGI0, SGI0, SGI0, SGI18198, SGI18198, SGI0, SGI0, SGI0, SGI18199, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4146[] = {SGI18200, SGI18201, SGI18202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18203, SGI0, SGI0, SGI0, SGI18204, SGI18205, NULL};
const Segment * const SG4147[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18206, SGI0, SGI0, SGI18207, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4148[] = {SGI18208, SGI18209, SGI18210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18211, SGI0, SGI0, SGI0, SGI18212, SGI18213, SGI0, SGI0, SGI18214, SGI0, NULL};
const Segment * const SG4149[] = {SGI18215, SGI18216, SGI18217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18218, SGI0, SGI0, SGI0, SGI18219, SGI18220, NULL};
const Segment * const SG4150[] = {SGI18221, SGI18222, SGI18223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18224, SGI0, SGI0, SGI0, SGI18225, SGI18225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4151[] = {SGI18226, SGI18227, SGI18228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18229, SGI0, SGI0, SGI18230, SGI18230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4152[] = {SGI18231, SGI18232, SGI18233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18234, SGI0, SGI0, SGI18235, SGI18235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4153[] = {SGI18236, SGI18237, SGI18238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18239, SGI0, SGI0, SGI18240, SGI18240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4154[] = {SGI18241, SGI18242, SGI18243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18244, SGI0, SGI0, SGI18245, SGI18245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4155[] = {SGI18246, SGI18247, SGI18248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18249, SGI18249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18250, SGI0, SGI0, NULL};
const Segment * const SG4156[] = {SGI18251, SGI18252, SGI18253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18254, SGI0, SGI0, SGI18255, SGI18255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4157[] = {SGI18256, SGI18257, SGI18258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18259, SGI0, SGI0, SGI18260, SGI18260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4158[] = {SGI18261, SGI18262, SGI18263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18264, SGI0, SGI0, SGI18265, SGI18265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4159[] = {SGI18266, SGI18267, SGI18268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18269, SGI18269, SGI0, SGI0, SGI0, SGI18270, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4160[] = {SGI18271, SGI0, SGI18272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18273, SGI0, SGI0, SGI0, SGI18274, SGI18274, SGI0, SGI0, SGI18275, SGI0, NULL};
const Segment * const SG4161[] = {SGI18276, SGI18277, SGI18278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18279, SGI18279, SGI0, SGI0, SGI0, SGI0, SGI18280, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4162[] = {SGI0, SGI18281, SGI0, SGI18282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4163[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4164[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18285, NULL};
const Segment * const SG4165[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4166[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18287, SGI0, SGI0, SGI18288, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4167[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18289, SGI0, SGI0, SGI18290, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4168[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18291, SGI0, SGI0, SGI18292, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4169[] = {SGI18293, SGI18294, SGI18295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18296, SGI18296, SGI0, SGI0, SGI0, SGI18297, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4170[] = {SGI18298, SGI18299, SGI18300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18301, SGI18301, SGI0, SGI0, SGI18302, SGI0, NULL};
const Segment * const SG4171[] = {SGI0, SGI18303, SGI18304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18305, SGI0, SGI0, SGI0, SGI0, SGI18306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4172[] = {SGI18307, SGI18308, SGI18309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18310, SGI18311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4173[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4174[] = {SGI18313, SGI18314, SGI18315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18316, SGI0, SGI0, SGI0, SGI18317, SGI18317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4175[] = {SGI18318, SGI0, SGI18319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4176[] = {SGI18320, SGI18321, SGI18322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18323, SGI0, SGI0, SGI0, SGI18324, SGI18324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4177[] = {SGI18325, SGI0, SGI18326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4178[] = {SGI18328, SGI0, SGI18331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4179[] = {SGI18327, SGI18329, SGI18330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18332, SGI0, SGI0, SGI0, SGI18333, SGI18333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4180[] = {SGI18334, SGI18335, SGI18336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18337, SGI0, SGI0, SGI0, SGI18338, SGI18338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4181[] = {SGI18339, SGI18340, SGI18341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18342, SGI0, SGI0, SGI0, SGI18343, SGI18344, NULL};
const Segment * const SG4182[] = {SGI18345, SGI0, SGI18346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18347, SGI0, SGI0, SGI0, SGI18348, SGI18348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18349, SGI0, SGI0, NULL};
const Segment * const SG4183[] = {SGI18350, SGI18351, SGI18352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18353, SGI0, SGI0, SGI0, SGI18354, SGI18354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4184[] = {SGI18355, SGI18356, SGI18357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18358, SGI0, SGI0, SGI0, SGI18359, SGI18360, SGI0, SGI0, SGI18361, SGI0, NULL};
const Segment * const SG4185[] = {SGI18362, SGI18363, SGI18364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18365, SGI0, SGI0, SGI0, SGI18366, SGI18366, SGI0, SGI18367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4186[] = {SGI18368, SGI18369, SGI18370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18371, SGI0, SGI0, SGI0, SGI18372, SGI18373, NULL};
const Segment * const SG4187[] = {SGI18374, SGI18375, SGI18376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18377, SGI0, SGI0, SGI18378, SGI18378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4188[] = {SGI18379, SGI18380, SGI18381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18382, SGI0, SGI0, SGI18383, SGI18383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4189[] = {SGI18384, SGI18385, SGI18386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18387, SGI0, SGI0, SGI0, SGI18388, SGI18388, SGI0, SGI0, SGI18389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4190[] = {SGI18390, SGI18391, SGI18392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18393, SGI0, SGI0, SGI0, SGI18394, SGI18395, NULL};
const Segment * const SG4191[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4192[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4193[] = {SGI18398, SGI18399, SGI18400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18401, SGI0, SGI0, SGI0, SGI18402, SGI18403, NULL};
const Segment * const SG4194[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4195[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4196[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18406, SGI0, SGI0, SGI0, SGI0, SGI18407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4197[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4198[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18409, SGI0, SGI0, SGI18410, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4199[] = {SGI18411, SGI18412, SGI18413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18414, SGI0, SGI0, SGI0, SGI18415, SGI18415, NULL};
const Segment * const SG4200[] = {SGI18416, SGI18417, SGI18418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18419, SGI18419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18420, SGI0, SGI0, NULL};
const Segment * const SG4201[] = {SGI18421, SGI18422, SGI18423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18424, SGI0, SGI0, SGI0, SGI18425, SGI18425, SGI0, SGI18426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4202[] = {SGI0, SGI18427, SGI0, SGI18428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4203[] = {SGI18429, SGI18430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18431, SGI18431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4204[] = {SGI18432, SGI18433, SGI18434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18435, SGI0, SGI0, SGI0, SGI18436, SGI18437, NULL};
const Segment * const SG4205[] = {SGI18438, SGI18439, SGI18440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18441, SGI0, SGI0, SGI0, SGI18442, SGI18443, NULL};
const Segment * const SG4206[] = {SGI18444, SGI18445, SGI18446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18447, SGI0, SGI0, SGI0, SGI18448, SGI18449, NULL};
const Segment * const SG4207[] = {SGI18450, SGI18451, SGI18452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18453, SGI0, SGI0, SGI0, SGI18454, SGI18454, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4208[] = {SGI18455, SGI18456, SGI18457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18458, SGI0, SGI0, SGI0, SGI18459, SGI18459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4209[] = {SGI18460, SGI0, SGI18461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18462, SGI0, SGI0, SGI0, SGI18463, SGI18463, SGI0, SGI0, SGI18464, SGI0, NULL};
const Segment * const SG4210[] = {SGI18465, SGI18466, SGI18467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18468, SGI0, SGI0, SGI0, SGI18469, SGI18469, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4211[] = {SGI18470, SGI0, SGI18471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18472, SGI0, SGI0, SGI0, SGI18473, SGI18473, SGI0, SGI0, SGI18474, SGI0, NULL};
const Segment * const SG4212[] = {SGI18475, SGI18476, SGI18477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18478, SGI0, SGI0, SGI0, SGI18479, SGI18480, NULL};
const Segment * const SG4213[] = {SGI18481, SGI0, SGI18482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18483, SGI0, SGI0, SGI0, SGI18484, SGI18484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18485, SGI0, SGI0, NULL};
const Segment * const SG4214[] = {SGI18486, SGI18487, SGI18488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18489, SGI0, SGI0, SGI0, SGI18490, SGI18490, SGI0, SGI0, SGI0, SGI0, SGI18491, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4215[] = {SGI18492, SGI0, SGI18493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18494, SGI0, SGI0, SGI0, SGI18495, SGI18495, SGI0, SGI0, SGI0, SGI18496, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4216[] = {SGI18497, SGI18498, SGI18499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18500, SGI0, SGI0, SGI0, SGI18501, SGI18501, SGI0, SGI0, SGI0, SGI0, SGI18502, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4217[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18503, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4218[] = {SGI18504, SGI0, SGI18505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18506, SGI0, SGI0, SGI0, SGI18507, SGI18507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18508, SGI0, SGI0, NULL};
const Segment * const SG4219[] = {SGI18509, SGI18510, SGI18511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18512, SGI0, SGI0, SGI0, SGI18513, SGI18513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4220[] = {SGI18514, SGI18515, SGI18516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18517, SGI0, SGI0, SGI0, SGI18518, SGI18519, NULL};
const Segment * const SG4221[] = {SGI18520, SGI18521, SGI18522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18523, SGI0, SGI0, SGI0, SGI18524, SGI18524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4222[] = {SGI18525, SGI18526, SGI18527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18528, SGI0, SGI0, SGI18529, SGI18529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4223[] = {SGI18530, SGI18531, SGI18532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18533, SGI0, SGI0, SGI0, SGI18534, SGI18535, NULL};
const Segment * const SG4224[] = {SGI18536, SGI18537, SGI18538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18539, SGI0, SGI0, SGI18540, SGI18540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4225[] = {SGI18541, SGI0, SGI18542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18543, SGI0, SGI0, SGI0, SGI18544, SGI18544, SGI0, SGI0, SGI0, SGI18545, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4226[] = {SGI18546, SGI18547, SGI18548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18549, SGI0, SGI0, SGI0, SGI18550, SGI18550, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4227[] = {SGI18551, SGI18552, SGI18553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18554, SGI0, SGI0, SGI0, SGI18555, SGI18555, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4228[] = {SGI18556, SGI18557, SGI18558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18559, SGI0, SGI0, SGI0, SGI18560, SGI18560, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4229[] = {SGI18561, SGI18562, SGI18563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18564, SGI0, SGI0, SGI0, SGI18565, SGI18565, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4230[] = {SGI18566, SGI18567, SGI18568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18569, SGI0, SGI0, SGI0, SGI18570, SGI18570, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4231[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4232[] = {SGI18572, SGI18573, SGI18574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18575, SGI0, SGI0, SGI0, SGI18576, SGI18576, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4233[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4234[] = {SGI0, SGI18578, SGI0, SGI18580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4235[] = {SGI0, SGI18579, SGI0, SGI18581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4236[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18582, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4237[] = {SGI18583, SGI18584, SGI18585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18586, SGI0, SGI0, SGI0, SGI18587, SGI18587, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4238[] = {SGI18588, SGI18589, SGI18590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18591, SGI0, SGI0, SGI0, SGI18592, SGI18592, SGI0, SGI0, SGI0, SGI0, SGI18593, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4239[] = {SGI18594, SGI18595, SGI18596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18597, SGI0, SGI0, SGI0, SGI18598, SGI18598, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4240[] = {SGI18599, SGI0, SGI18600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18601, SGI0, SGI0, SGI0, SGI18602, SGI18602, SGI0, SGI0, SGI0, SGI18603, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4241[] = {SGI18604, SGI0, SGI18605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18606, SGI0, SGI0, SGI0, SGI18607, SGI18607, SGI0, SGI0, SGI0, SGI18608, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4242[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18609, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4243[] = {SGI18610, SGI18611, SGI18612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18613, SGI0, SGI0, SGI0, SGI18614, SGI18615, NULL};
const Segment * const SG4244[] = {SGI18616, SGI0, SGI18617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18618, SGI0, SGI0, SGI0, SGI18619, SGI18619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18620, SGI0, SGI0, NULL};
const Segment * const SG4245[] = {SGI18621, SGI18622, SGI18623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18624, SGI0, SGI0, SGI18625, SGI18625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4246[] = {SGI18626, SGI18627, SGI18628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18629, SGI0, SGI0, SGI18630, SGI18630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4247[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4248[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18632, SGI0, SGI0, SGI18633, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4249[] = {SGI18634, SGI18635, SGI18636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18637, SGI0, SGI0, SGI0, SGI18638, SGI18639, NULL};
const Segment * const SG4250[] = {SGI18640, SGI18641, SGI18642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18643, SGI0, SGI0, SGI0, SGI18644, SGI18644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4251[] = {SGI18645, SGI0, SGI18646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18647, SGI0, SGI0, SGI0, SGI18648, SGI18648, SGI0, SGI0, SGI0, SGI18649, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4252[] = {SGI18650, SGI0, SGI18651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18652, SGI0, SGI0, SGI0, SGI18653, SGI18653, SGI0, SGI0, SGI0, SGI18654, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4253[] = {SGI18655, SGI18656, SGI18657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18658, SGI0, SGI0, SGI0, SGI18659, SGI18660, NULL};
const Segment * const SG4254[] = {SGI18661, SGI18662, SGI18663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18664, SGI0, SGI0, SGI0, SGI18665, SGI18665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4255[] = {SGI18666, SGI0, SGI18667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4256[] = {SGI18668, SGI18669, SGI18670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18671, SGI0, SGI0, SGI0, SGI18672, SGI18672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4257[] = {SGI18673, SGI18674, SGI18675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18676, SGI0, SGI0, SGI0, SGI18677, SGI18677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4258[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18678, SGI0, SGI0, SGI18679, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4259[] = {SGI0, SGI18680, SGI0, SGI18681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4260[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18682, SGI0, SGI0, SGI18683, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4261[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4262[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4263[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18686, SGI0, SGI0, SGI18687, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4264[] = {SGI18688, SGI0, SGI18689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18690, SGI0, SGI0, SGI0, SGI18691, SGI18691, SGI0, SGI0, SGI18692, SGI0, NULL};
const Segment * const SG4265[] = {SGI18693, SGI18694, SGI18695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18696, SGI0, SGI0, SGI0, SGI18697, SGI18697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4266[] = {SGI18698, SGI18699, SGI18700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18701, SGI0, SGI0, SGI0, SGI18702, SGI18702, SGI0, SGI18703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4267[] = {SGI18704, SGI0, SGI18705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18706, SGI0, SGI0, SGI0, SGI18707, SGI18707, SGI0, SGI0, SGI0, SGI18708, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4268[] = {SGI18709, SGI18710, SGI18711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18712, SGI0, SGI0, SGI0, SGI18713, SGI18713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18714, SGI0, NULL};
const Segment * const SG4269[] = {SGI18715, SGI18716, SGI18717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18718, SGI0, SGI0, SGI0, SGI18719, SGI18719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18720, SGI0, NULL};
const Segment * const SG4270[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18721, SGI0, SGI0, SGI18722, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4271[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4272[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4273[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4274[] = {SGI18726, SGI0, SGI18727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4275[] = {SGI18729, SGI0, SGI18730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4276[] = {SGI18732, SGI0, SGI18733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4277[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4278[] = {SGI18736, SGI18737, SGI18738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18739, SGI0, SGI0, SGI0, SGI18740, SGI18741, NULL};
const Segment * const SG4279[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4280[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4281[] = {SGI18743, SGI0, SGI18744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18745, SGI0, SGI0, SGI0, SGI18747, SGI18747, SGI0, SGI0, SGI0, SGI18748, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4282[] = {SGI18749, SGI18750, SGI18751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18752, SGI0, SGI0, SGI0, SGI18753, SGI18753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4283[] = {SGI18754, SGI18755, SGI18756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18757, SGI18757, SGI0, SGI18758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4284[] = {SGI18759, SGI18760, SGI18761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18762, SGI0, SGI0, SGI0, SGI18763, SGI0, SGI0, SGI18764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4285[] = {SGI18765, SGI18766, SGI18767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18768, SGI0, SGI0, SGI0, SGI18769, SGI18769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18770, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4286[] = {SGI18771, SGI18772, SGI18773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18774, SGI0, SGI0, SGI0, SGI18775, SGI18775, SGI0, SGI0, SGI18776, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4287[] = {SGI18777, SGI18778, SGI18779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18780, SGI0, SGI0, SGI0, SGI18781, SGI18781, SGI0, SGI0, SGI18782, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4288[] = {SGI18783, SGI18784, SGI18785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18786, SGI0, SGI0, SGI0, SGI18787, SGI18787, SGI0, SGI0, SGI18788, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4289[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18789, SGI0, SGI0, SGI18790, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4290[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18791, SGI0, SGI0, SGI18792, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4291[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4292[] = {SGI18794, SGI0, SGI18795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18796, SGI0, SGI0, SGI0, SGI18797, SGI18797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18798, SGI0, SGI0, NULL};
const Segment * const SG4293[] = {SGI18799, SGI18800, SGI18801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18802, SGI0, SGI0, SGI0, SGI18803, SGI18803, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4294[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4295[] = {SGI18804, SGI18805, SGI18806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18808, SGI0, SGI0, SGI0, SGI18809, SGI18809, SGI0, SGI18810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4296[] = {SGI18811, SGI18812, SGI18813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18814, SGI0, SGI0, SGI0, SGI18815, SGI18815, SGI0, SGI18816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4297[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18817, SGI0, SGI0, SGI18818, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4298[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4299[] = {SGI0, SGI18820, SGI18821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4300[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4301[] = {SGI18824, SGI18825, SGI18826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18827, SGI0, SGI0, SGI0, SGI18828, SGI18828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4302[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18829, SGI0, SGI18830, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4303[] = {SGI18831, SGI18832, SGI18833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18834, SGI0, SGI0, SGI0, SGI18835, SGI18835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4304[] = {SGI18836, SGI0, SGI18837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18838, SGI0, SGI0, SGI0, SGI18839, SGI18839, SGI0, SGI0, SGI0, SGI18840, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4305[] = {SGI18841, SGI18842, SGI18843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18844, SGI0, SGI0, SGI0, SGI18845, SGI18845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4306[] = {SGI18846, SGI18847, SGI18848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18849, SGI0, SGI0, SGI0, SGI18850, SGI18850, SGI0, SGI18851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4307[] = {SGI18852, SGI18853, SGI18854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18855, SGI0, SGI0, SGI0, SGI18856, SGI18856, SGI18857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4308[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18858, NULL};
const Segment * const SG4309[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4310[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18860, SGI0, SGI0, SGI18861, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4311[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4312[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4313[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18864, SGI0, SGI18865, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4314[] = {SGI18866, SGI18867, SGI18868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18869, SGI0, SGI0, SGI0, SGI18870, SGI18870, SGI0, SGI0, SGI0, SGI18871, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4315[] = {SGI18872, SGI18873, SGI18874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18875, SGI0, SGI0, SGI0, SGI18876, SGI18876, SGI0, SGI0, SGI0, SGI18877, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4316[] = {SGI18878, SGI18879, SGI18880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18881, SGI0, SGI0, SGI0, SGI18882, SGI18883, NULL};
const Segment * const SG4317[] = {SGI18884, SGI0, SGI18885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18886, SGI0, SGI0, SGI0, SGI18887, SGI18887, SGI0, SGI0, SGI0, SGI18888, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4318[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18889, SGI0, SGI18890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4319[] = {SGI18891, SGI18892, SGI18893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18894, SGI0, SGI0, SGI18895, SGI18895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4320[] = {SGI18896, SGI18897, SGI18898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18899, SGI0, SGI0, SGI0, SGI18900, SGI18900, NULL};
const Segment * const SG4321[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4322[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18902, SGI0, SGI0, SGI18903, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4323[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18904, SGI0, SGI0, SGI18905, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4324[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18906, SGI0, SGI0, SGI18907, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4325[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18908, SGI0, SGI0, SGI18909, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4326[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18910, SGI0, SGI18911, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4327[] = {SGI18912, SGI0, SGI18913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18914, SGI0, SGI0, SGI0, SGI18915, SGI18915, SGI0, SGI0, SGI18916, SGI0, NULL};
const Segment * const SG4328[] = {SGI0, SGI18917, SGI0, SGI18918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4329[] = {SGI18919, SGI18920, SGI18921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18922, SGI0, SGI0, SGI0, SGI18923, SGI18923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4330[] = {SGI18924, SGI0, SGI18925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18926, SGI0, SGI0, SGI0, SGI18927, SGI18927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18928, SGI0, SGI0, NULL};
const Segment * const SG4331[] = {SGI18929, SGI18930, SGI18931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18932, SGI0, SGI0, SGI0, SGI18933, SGI18933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4332[] = {SGI18934, SGI18935, SGI18936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18937, SGI0, SGI0, SGI0, SGI18938, SGI18938, SGI0, SGI0, SGI18939, SGI0, SGI0, SGI0, SGI18940, SGI0, SGI0, NULL};
const Segment * const SG4333[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4334[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4335[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4336[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4337[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4338[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4339[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4340[] = {SGI18948, SGI18949, SGI18950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18951, SGI0, SGI0, SGI0, SGI18952, SGI18952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4341[] = {SGI18953, SGI18954, SGI18955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18956, SGI0, SGI0, SGI0, SGI18957, SGI18957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4342[] = {SGI18958, SGI18959, SGI18960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18961, SGI0, SGI0, SGI0, SGI18962, SGI18962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4343[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4344[] = {SGI18964, SGI18965, SGI18966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18967, SGI0, SGI0, SGI18968, SGI18968, SGI18969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4345[] = {SGI0, SGI0, SGI18970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4346[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18972, SGI0, SGI0, SGI0, SGI0, SGI18973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4347[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4348[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4349[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18976, SGI0, SGI0, SGI18977, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4350[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18978, SGI0, SGI0, SGI18979, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4351[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18982, SGI0, SGI0, SGI18983, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4352[] = {SGI0, SGI0, SGI18980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18981, SGI0, SGI0, SGI0, SGI18984, SGI18985, NULL};
const Segment * const SG4353[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4354[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4355[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18988, SGI0, SGI0, SGI18989, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4356[] = {SGI0, SGI0, SGI18990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4357[] = {SGI0, SGI0, SGI18992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4358[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18994, SGI0, SGI0, SGI18995, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4359[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI18996, SGI18997, SGI18998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4360[] = {SGI18999, SGI19000, SGI19001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19002, SGI0, SGI0, SGI0, SGI19003, SGI19004, NULL};
const Segment * const SG4361[] = {SGI19005, SGI19006, SGI19007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19008, SGI0, SGI0, SGI0, SGI19009, SGI19009, SGI0, SGI19010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4362[] = {SGI19011, SGI19012, SGI19013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19014, SGI0, SGI0, SGI0, SGI19015, SGI19015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4363[] = {SGI19016, SGI19017, SGI19018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19019, SGI0, SGI0, SGI0, SGI19020, SGI19020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4364[] = {SGI0, SGI0, SGI19021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19022, SGI0, SGI0, SGI0, SGI19023, SGI19024, NULL};
const Segment * const SG4365[] = {SGI19025, SGI19026, SGI19027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19028, SGI0, SGI0, SGI0, SGI19029, SGI19029, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4366[] = {SGI19030, SGI19031, SGI19033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19036, SGI19037, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4367[] = {SGI19032, SGI19034, SGI19035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19038, SGI0, SGI0, SGI0, SGI19039, SGI19040, NULL};
const Segment * const SG4368[] = {SGI19041, SGI19042, SGI19043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19044, SGI0, SGI0, SGI0, SGI19045, SGI19046, NULL};
const Segment * const SG4369[] = {SGI19047, SGI19048, SGI19049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19050, SGI0, SGI0, SGI0, SGI19051, SGI19052, NULL};
const Segment * const SG4370[] = {SGI19053, SGI19054, SGI19055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19056, SGI0, SGI0, SGI0, SGI19057, SGI19057, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4371[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4372[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19059, SGI0, SGI0, SGI19060, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4373[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19061, SGI0, SGI0, SGI19062, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4374[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4375[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4376[] = {SGI19065, SGI0, SGI19066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4377[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4378[] = {SGI19069, SGI19070, SGI19071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19072, SGI0, SGI0, SGI0, SGI19073, SGI19073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4379[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19074, SGI0, SGI19075, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4380[] = {SGI0, SGI19076, SGI0, SGI19077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4381[] = {SGI19078, SGI19079, SGI19080, SGI19081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19082, SGI0, SGI0, SGI0, SGI19083, SGI19084, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4382[] = {SGI19085, SGI19086, SGI19087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19088, SGI0, SGI0, SGI19089, SGI19089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4383[] = {SGI19090, SGI0, SGI19091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19092, SGI0, SGI0, SGI0, SGI19093, SGI19093, SGI0, SGI0, SGI19094, SGI0, NULL};
const Segment * const SG4384[] = {SGI19095, SGI0, SGI19096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19097, SGI0, SGI0, SGI0, SGI19098, SGI19098, SGI0, SGI0, SGI19099, SGI0, NULL};
const Segment * const SG4385[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19100, NULL};
const Segment * const SG4386[] = {SGI0, SGI19101, SGI0, SGI19102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4387[] = {SGI19103, SGI19104, SGI19105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19106, SGI0, SGI0, SGI0, SGI19107, SGI19107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4388[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19108, SGI0, SGI0, SGI19109, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4389[] = {SGI0, SGI0, SGI19110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4390[] = {SGI0, SGI0, SGI19112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4391[] = {SGI19113, SGI19115, SGI19116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19117, SGI0, SGI0, SGI0, SGI19118, SGI19118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4392[] = {SGI0, SGI0, SGI19119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4393[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19121, SGI0, SGI0, SGI19122, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4394[] = {SGI19123, SGI19124, SGI19125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19126, SGI0, SGI0, SGI0, SGI19127, SGI19127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4395[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4396[] = {SGI0, SGI0, SGI19129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4397[] = {SGI19131, SGI19132, SGI19133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19134, SGI0, SGI0, SGI0, SGI19135, SGI19135, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4398[] = {SGI19136, SGI19138, SGI19140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19143, SGI19144, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4399[] = {SGI19137, SGI19139, SGI19141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19142, SGI0, SGI0, SGI0, SGI19145, SGI19145, SGI0, SGI19146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4400[] = {SGI19147, SGI0, SGI19148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19151, SGI0, SGI0, SGI0, SGI19152, SGI19152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19154, SGI0, SGI0, NULL};
const Segment * const SG4401[] = {SGI19149, SGI0, SGI19150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19153, SGI0, SGI0, SGI0, SGI19155, SGI19155, SGI0, SGI0, SGI0, SGI19156, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4402[] = {SGI19157, SGI19158, SGI19159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19160, SGI0, SGI0, SGI0, SGI19161, SGI19161, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4403[] = {SGI0, SGI19162, SGI0, SGI19163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4404[] = {SGI19164, SGI19165, SGI19166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19167, SGI0, SGI0, SGI0, SGI19168, SGI19168, SGI0, SGI19169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4405[] = {SGI19170, SGI0, SGI19171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19172, SGI19173, SGI0, NULL};
const Segment * const SG4406[] = {SGI19174, SGI0, SGI19175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19176, SGI0, SGI0, SGI0, SGI19177, SGI19177, SGI0, SGI0, SGI19178, SGI0, NULL};
const Segment * const SG4407[] = {SGI19179, SGI19180, SGI19181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19182, SGI0, SGI0, SGI19183, SGI19183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4408[] = {SGI19184, SGI19185, SGI19186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19187, SGI0, SGI0, SGI19188, SGI19188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4409[] = {SGI19189, SGI19190, SGI19191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19192, SGI0, SGI0, SGI0, SGI19193, SGI19193, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4410[] = {SGI19194, SGI19195, SGI19196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19197, SGI0, SGI0, SGI0, SGI19198, SGI19198, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4411[] = {SGI19199, SGI19200, SGI19201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19202, SGI0, SGI0, SGI0, SGI19203, SGI19203, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4412[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4413[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4414[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4415[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19207, SGI0, SGI0, SGI19208, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4416[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4417[] = {SGI19210, SGI19211, SGI19212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19213, SGI0, SGI0, SGI0, SGI19214, SGI19214, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4418[] = {SGI19215, SGI19216, SGI19217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19218, SGI0, SGI0, SGI0, SGI19219, SGI19219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19220, SGI0, NULL};
const Segment * const SG4419[] = {SGI19221, SGI19222, SGI19223, SGI19224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19225, SGI0, SGI0, SGI0, SGI19226, SGI19227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4420[] = {SGI19228, SGI19229, SGI19230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19231, SGI0, SGI0, SGI0, SGI19232, SGI19232, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4421[] = {SGI19233, SGI0, SGI19234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19235, SGI0, SGI0, SGI0, SGI19236, SGI19236, SGI0, SGI0, SGI19237, SGI0, NULL};
const Segment * const SG4422[] = {SGI19238, SGI19239, SGI19240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19241, SGI0, SGI0, SGI0, SGI19242, SGI19242, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4423[] = {SGI0, SGI19243, SGI0, SGI19244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4424[] = {SGI19245, SGI0, SGI19246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19247, SGI0, SGI0, SGI0, SGI19248, SGI19248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19249, SGI0, SGI0, NULL};
const Segment * const SG4425[] = {SGI0, SGI19250, SGI0, SGI19251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4426[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4427[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19253, SGI0, SGI0, SGI19254, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4428[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4429[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4430[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19257, SGI0, SGI0, SGI19258, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4431[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4432[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4433[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19261, SGI0, SGI0, SGI19262, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4434[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19263, SGI0, SGI0, SGI19264, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4435[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19265, SGI0, SGI19266, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4436[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19267, SGI0, SGI19268, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4437[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19269, SGI0, SGI19270, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4438[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19271, SGI0, SGI19272, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4439[] = {SGI19273, SGI0, SGI19274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19275, SGI0, SGI0, SGI0, SGI19276, SGI19276, SGI0, SGI0, SGI19277, SGI0, NULL};
const Segment * const SG4440[] = {SGI19278, SGI0, SGI19279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19280, SGI0, SGI0, SGI0, SGI19281, SGI19281, SGI0, SGI0, SGI19282, SGI0, NULL};
const Segment * const SG4441[] = {SGI0, SGI0, SGI19283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19284, SGI19285, SGI0, SGI0, SGI19286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4442[] = {SGI19287, SGI19288, SGI19289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19290, SGI0, SGI0, SGI0, SGI19291, SGI19291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4443[] = {SGI19292, SGI19293, SGI19294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19295, SGI0, SGI0, SGI0, SGI19296, SGI19296, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4444[] = {SGI19297, SGI19298, SGI19299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19300, SGI0, SGI0, SGI0, SGI19301, SGI19301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4445[] = {SGI19302, SGI19303, SGI19304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19305, SGI0, SGI0, SGI0, SGI19306, SGI19306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4446[] = {SGI0, SGI19307, SGI19308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19309, SGI0, SGI0, SGI0, SGI0, SGI19310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4447[] = {SGI19311, SGI19312, SGI19313, SGI19314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19315, SGI0, SGI0, SGI0, SGI19316, SGI19317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4448[] = {SGI19318, SGI0, SGI19319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19320, SGI0, SGI0, SGI0, SGI19321, SGI19321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19322, SGI0, SGI0, NULL};
const Segment * const SG4449[] = {SGI19323, SGI19324, SGI19325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19326, SGI0, SGI0, SGI0, SGI19327, SGI19327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4450[] = {SGI0, SGI0, SGI19328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4451[] = {SGI0, SGI0, SGI19329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4452[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4453[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19334, NULL};
const Segment * const SG4454[] = {SGI19331, SGI0, SGI19332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19335, SGI0, SGI0, SGI0, SGI19336, SGI19336, SGI0, SGI0, SGI0, SGI19337, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4455[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4456[] = {SGI0, SGI0, SGI19339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4457[] = {SGI0, SGI0, SGI19341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4458[] = {SGI0, SGI0, SGI19343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4459[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19345, SGI0, SGI0, SGI19346, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4460[] = {SGI19347, SGI19348, SGI19349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19350, SGI0, SGI0, SGI0, SGI19351, SGI19351, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4461[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4462[] = {SGI19353, SGI19354, SGI19355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19356, SGI0, SGI0, SGI0, SGI19357, SGI19357, SGI0, SGI19358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4463[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4464[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19360, SGI19360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4465[] = {SGI19361, SGI19362, SGI19363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19364, SGI0, SGI0, SGI19365, SGI19365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4466[] = {SGI19366, SGI19367, SGI19368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19369, SGI0, SGI0, SGI0, SGI19370, SGI19370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4467[] = {SGI0, SGI19371, SGI19372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19373, SGI0, SGI0, SGI0, SGI19374, SGI19374, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4468[] = {SGI0, SGI19375, SGI19376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19377, SGI0, SGI0, SGI0, SGI19378, SGI19378, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4469[] = {SGI19379, SGI0, SGI19380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19381, SGI0, SGI0, SGI0, SGI19382, SGI19382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19383, SGI0, SGI0, NULL};
const Segment * const SG4470[] = {SGI19384, SGI19385, SGI19386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19387, SGI19387, SGI0, SGI0, SGI0, SGI19388, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4471[] = {SGI19389, SGI19390, SGI19391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19392, SGI0, SGI0, SGI0, SGI19393, SGI0, SGI0, SGI0, SGI0, SGI19394, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4472[] = {SGI19395, SGI19396, SGI19397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19398, SGI0, SGI0, SGI0, SGI19399, SGI19399, SGI0, SGI19400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4473[] = {SGI19401, SGI19402, SGI19403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19404, SGI0, SGI0, SGI19405, SGI19405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4474[] = {SGI19406, SGI19407, SGI19408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19409, SGI0, SGI0, SGI0, SGI19410, SGI19410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4475[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4476[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19412, SGI0, SGI0, SGI19413, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4477[] = {SGI19414, SGI19415, SGI19416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19417, SGI0, SGI0, SGI0, SGI19418, SGI19418, SGI0, SGI0, SGI19419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4478[] = {SGI19420, SGI19421, SGI19422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19423, SGI0, SGI0, SGI0, SGI19424, SGI19424, SGI0, SGI0, SGI0, SGI0, SGI19425, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4479[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19426, SGI0, SGI0, SGI19427, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4480[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19428, SGI0, SGI0, SGI19429, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4481[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19430, SGI0, SGI0, SGI19431, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4482[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4483[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4484[] = {SGI19434, SGI19435, SGI19436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19437, SGI0, SGI0, SGI0, SGI19438, SGI19438, SGI0, SGI19439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4485[] = {SGI19440, SGI19441, SGI19442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19443, SGI0, SGI0, SGI0, SGI19444, SGI19444, SGI0, SGI19445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4486[] = {SGI19446, SGI0, SGI19447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19448, SGI0, SGI0, SGI0, SGI19449, SGI19449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19450, SGI0, SGI0, NULL};
const Segment * const SG4487[] = {SGI19451, SGI19452, SGI19453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19454, SGI0, SGI0, SGI0, SGI19455, SGI19455, SGI0, SGI0, SGI0, SGI19456, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4488[] = {SGI19457, SGI19458, SGI19459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19460, SGI0, SGI0, SGI19461, SGI19461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4489[] = {SGI19462, SGI19463, SGI19464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19465, SGI0, SGI0, SGI0, SGI19466, SGI19466, SGI0, SGI0, SGI0, SGI19467, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4490[] = {SGI19468, SGI19469, SGI19470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19471, SGI0, SGI0, SGI0, SGI19472, SGI19472, SGI0, SGI19473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4491[] = {SGI19474, SGI19475, SGI19476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19477, SGI0, SGI0, SGI0, SGI19478, SGI19478, SGI0, SGI19479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4492[] = {SGI0, SGI19480, SGI0, SGI19481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4493[] = {SGI19482, SGI19483, SGI19484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19485, SGI0, SGI0, SGI19486, SGI19486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4494[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4495[] = {SGI0, SGI19488, SGI0, SGI19489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19490, SGI0, SGI0, SGI0, SGI19491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4496[] = {SGI0, SGI19492, SGI0, SGI19493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4497[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4498[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4499[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4500[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19497, SGI0, SGI0, SGI19498, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4501[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4502[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19500, SGI0, SGI0, SGI19501, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4503[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19502, SGI0, SGI0, SGI19503, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4504[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4505[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4506[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19506, SGI0, SGI0, SGI19507, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4507[] = {SGI0, SGI19508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4508[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4509[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4510[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4511[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19513, SGI0, SGI0, SGI19514, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4512[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19515, SGI0, SGI0, SGI19516, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4513[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4514[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19518, SGI0, SGI0, SGI19519, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4515[] = {SGI19520, SGI0, SGI19521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19522, SGI0, SGI0, SGI0, SGI19523, SGI19523, SGI0, SGI0, SGI19524, SGI0, NULL};
const Segment * const SG4516[] = {SGI19525, SGI0, SGI19526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19527, SGI0, SGI0, SGI0, SGI19528, SGI19528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19529, SGI0, SGI0, NULL};
const Segment * const SG4517[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4518[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19531, SGI0, SGI19532, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4519[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19533, SGI0, SGI19534, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4520[] = {SGI19535, SGI19536, SGI19537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19538, SGI0, SGI0, SGI19539, SGI19539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4521[] = {SGI19543, SGI19544, SGI19545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4522[] = {SGI19546, SGI19547, SGI19548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19549, SGI0, SGI0, SGI0, SGI19550, SGI19550, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4523[] = {SGI19551, SGI19552, SGI19553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19554, SGI0, SGI0, SGI0, SGI19555, SGI19555, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4524[] = {SGI19556, SGI19557, SGI19558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19559, SGI0, SGI0, SGI0, SGI19560, SGI19560, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4525[] = {SGI19561, SGI0, SGI19562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19563, SGI0, SGI0, SGI0, SGI19564, SGI19564, SGI0, SGI0, SGI0, SGI19565, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4526[] = {SGI19566, SGI0, SGI19567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19568, SGI0, SGI0, SGI0, SGI19569, SGI19569, SGI0, SGI0, SGI0, SGI19570, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4527[] = {SGI19540, SGI19541, SGI19542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19571, SGI0, SGI0, SGI19572, SGI19572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4528[] = {SGI19573, SGI19574, SGI19575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19576, SGI0, SGI0, SGI19577, SGI19577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4529[] = {SGI19578, SGI19579, SGI19580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19581, SGI0, SGI0, SGI0, SGI19582, SGI19582, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4530[] = {SGI19583, SGI19584, SGI19585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19586, SGI0, SGI0, SGI0, SGI19587, SGI19587, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4531[] = {SGI19588, SGI0, SGI19589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19590, SGI0, SGI0, SGI0, SGI19591, SGI19591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19592, SGI0, SGI0, NULL};
const Segment * const SG4532[] = {SGI19593, SGI19594, SGI19595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19596, SGI0, SGI0, SGI0, SGI19597, SGI19597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19598, SGI0, NULL};
const Segment * const SG4533[] = {SGI19599, SGI0, SGI19600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19601, SGI0, SGI0, SGI0, SGI19602, SGI19602, SGI0, SGI0, SGI19603, SGI0, NULL};
const Segment * const SG4534[] = {SGI0, SGI19604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4535[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4536[] = {SGI0, SGI0, SGI19606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4537[] = {SGI19608, SGI19610, SGI19612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19614, SGI0, SGI0, SGI0, SGI19616, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4538[] = {SGI19609, SGI19611, SGI19613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19615, SGI0, SGI0, SGI0, SGI19617, SGI19617, SGI0, SGI19618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4539[] = {SGI19619, SGI19620, SGI19621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19625, SGI0, SGI0, SGI0, SGI19626, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4540[] = {SGI19622, SGI19623, SGI19624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19627, SGI19627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4541[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4542[] = {SGI19629, SGI19630, SGI19631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19632, SGI0, SGI0, SGI0, SGI19633, SGI19633, SGI0, SGI19634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4543[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19635, SGI0, SGI0, SGI0, SGI0, SGI19636, SGI0, NULL};
const Segment * const SG4544[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4545[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19638, SGI0, SGI0, SGI0, SGI0, SGI19639, SGI0, NULL};
const Segment * const SG4546[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4547[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4548[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19642, SGI0, SGI0, SGI19643, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4549[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4550[] = {SGI0, SGI19645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4551[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19647, SGI0, SGI0, SGI19648, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4552[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4553[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19650, SGI0, SGI0, SGI19651, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4554[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19652, SGI0, SGI0, SGI19653, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4555[] = {SGI19654, SGI19655, SGI19656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19657, SGI0, SGI0, SGI0, SGI19658, SGI19658, SGI0, SGI19659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4556[] = {SGI0, SGI19660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4557[] = {SGI19662, SGI19663, SGI19664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19665, SGI0, SGI0, SGI0, SGI19666, SGI19666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4558[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4559[] = {SGI0, SGI19668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4560[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4561[] = {SGI0, SGI19671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4562[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19673, SGI0, SGI19674, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4563[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19675, SGI0, SGI19676, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4564[] = {SGI19677, SGI19678, SGI19679, SGI19680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19681, SGI0, SGI0, SGI0, SGI19682, SGI19683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4565[] = {SGI19684, SGI19685, SGI19686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19687, SGI0, SGI0, SGI19688, SGI19688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4566[] = {SGI19689, SGI0, SGI19690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4567[] = {SGI19691, SGI19692, SGI19693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19694, SGI0, SGI0, SGI0, SGI19695, SGI19695, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4568[] = {SGI19696, SGI19697, SGI19698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19699, SGI0, SGI0, SGI0, SGI19700, SGI19700, SGI0, SGI0, SGI0, SGI19701, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4569[] = {SGI19702, SGI19703, SGI19704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19705, SGI0, SGI0, SGI0, SGI19706, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4570[] = {SGI19707, SGI19709, SGI19711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19713, SGI0, SGI0, SGI0, SGI19715, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4571[] = {SGI19708, SGI19710, SGI19712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19714, SGI0, SGI0, SGI0, SGI19716, SGI19716, SGI0, SGI0, SGI0, SGI19717, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4572[] = {SGI19718, SGI19719, SGI19720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19721, SGI0, SGI0, SGI0, SGI19722, SGI19722, SGI0, SGI0, SGI0, SGI0, SGI19723, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4573[] = {SGI19724, SGI19725, SGI19726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19727, SGI19727, SGI0, SGI0, SGI0, SGI19728, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4574[] = {SGI19729, SGI19730, SGI19731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19732, SGI0, SGI0, SGI0, SGI19733, SGI19733, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4575[] = {SGI19734, SGI19735, SGI19736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19737, SGI0, SGI0, SGI19738, SGI19738, SGI19739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4576[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19740, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4577[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4578[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4579[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19743, SGI0, SGI0, SGI19744, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4580[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19745, SGI0, SGI0, SGI19746, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4581[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19747, SGI0, SGI0, SGI19748, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4582[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19749, SGI0, SGI0, SGI0, SGI0, SGI19750, SGI0, NULL};
const Segment * const SG4583[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19751, SGI0, SGI0, SGI0, SGI0, SGI19752, SGI0, NULL};
const Segment * const SG4584[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19754, NULL};
const Segment * const SG4585[] = {SGI19755, SGI19756, SGI19757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19758, SGI0, SGI0, SGI0, SGI19759, SGI19759, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4586[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4587[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19761, SGI0, SGI0, SGI19762, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4588[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19763, SGI0, SGI0, SGI0, SGI0, SGI19764, SGI0, NULL};
const Segment * const SG4589[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19765, SGI0, SGI0, SGI19766, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4590[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4591[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19768, SGI0, SGI0, SGI19769, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4592[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19770, SGI0, SGI0, SGI19771, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4593[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4594[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4595[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4596[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4597[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4598[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19777, SGI0, SGI0, SGI19778, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4599[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4600[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4601[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19781, SGI0, SGI19782, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4602[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4603[] = {SGI0, SGI19784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4604[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19786, SGI0, SGI19787, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4605[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19788, SGI0, SGI19789, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4606[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4607[] = {SGI19791, SGI19792, SGI19793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19794, SGI0, SGI0, SGI0, SGI19795, SGI19795, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4608[] = {SGI19796, SGI19797, SGI19798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19799, SGI0, SGI0, SGI0, SGI19800, SGI19800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4609[] = {SGI19801, SGI19802, SGI19803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19804, SGI0, SGI0, SGI19805, SGI19805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4610[] = {SGI19806, SGI19807, SGI19808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19809, SGI0, SGI0, SGI19810, SGI19810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4611[] = {SGI19811, SGI19812, SGI19813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19814, SGI0, SGI0, SGI19815, SGI19815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4612[] = {SGI19816, SGI19817, SGI19818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19819, SGI0, SGI0, SGI19820, SGI19820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4613[] = {SGI19821, SGI19822, SGI19823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19824, SGI0, SGI0, SGI19825, SGI19825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4614[] = {SGI19826, SGI19827, SGI19828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19829, SGI0, SGI0, SGI0, SGI19830, SGI19830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4615[] = {SGI19831, SGI19832, SGI19833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19834, SGI0, SGI0, SGI0, SGI19835, SGI19835, SGI0, SGI0, SGI0, SGI19836, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4616[] = {SGI19837, SGI19838, SGI19839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19840, SGI0, SGI0, SGI0, SGI19841, SGI19841, SGI0, SGI0, SGI0, SGI19844, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4617[] = {SGI19842, SGI19843, SGI19845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19846, SGI0, SGI0, SGI19847, SGI19847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4618[] = {SGI19848, SGI19849, SGI19850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19851, SGI0, SGI0, SGI0, SGI19852, SGI19852, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4619[] = {SGI19853, SGI19854, SGI19855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19856, SGI0, SGI0, SGI0, SGI19857, SGI19857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19858, SGI0, NULL};
const Segment * const SG4620[] = {SGI19859, SGI19860, SGI19861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19862, SGI0, SGI0, SGI0, SGI19863, SGI19863, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4621[] = {SGI19864, SGI19865, SGI19866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19867, SGI0, SGI0, SGI0, SGI19868, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4622[] = {SGI19869, SGI19870, SGI19871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19872, SGI0, SGI0, SGI0, SGI19873, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4623[] = {SGI19874, SGI19875, SGI19876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19877, SGI0, SGI0, SGI0, SGI19878, SGI19878, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4624[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4625[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19881, NULL};
const Segment * const SG4626[] = {SGI19882, SGI19883, SGI19884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19885, SGI0, SGI0, SGI0, SGI19886, SGI19886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4627[] = {SGI19887, SGI19888, SGI19889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19890, SGI0, SGI0, SGI0, SGI19891, SGI19891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4628[] = {SGI19892, SGI19893, SGI19894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19895, SGI0, SGI0, SGI0, SGI19896, SGI19896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19897, SGI0, NULL};
const Segment * const SG4629[] = {SGI19898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4630[] = {SGI19900, SGI19901, SGI19902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19903, SGI0, SGI0, SGI0, SGI19904, SGI19904, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4631[] = {SGI0, SGI19905, SGI0, SGI19906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4632[] = {SGI19907, SGI19908, SGI19909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19910, SGI0, SGI0, SGI0, SGI19911, SGI19911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4633[] = {SGI19912, SGI19913, SGI19914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19915, SGI0, SGI0, SGI0, SGI19916, SGI19916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19917, SGI0, NULL};
const Segment * const SG4634[] = {SGI19918, SGI19919, SGI19920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19921, SGI0, SGI0, SGI0, SGI19922, SGI19922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4635[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19923, SGI0, SGI0, SGI19924, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4636[] = {SGI0, SGI19925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4637[] = {SGI19927, SGI19928, SGI19929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19930, SGI0, SGI0, SGI0, SGI19931, SGI19931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4638[] = {SGI0, SGI19932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4639[] = {SGI19934, SGI19935, SGI19936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19937, SGI0, SGI0, SGI0, SGI19938, SGI19938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19939, SGI0, NULL};
const Segment * const SG4640[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19940, SGI0, SGI0, SGI19941, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4641[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19942, SGI0, SGI0, SGI19943, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4642[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4643[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4644[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19947, NULL};
const Segment * const SG4645[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4646[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19949, SGI0, SGI0, SGI19950, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4647[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19951, SGI19952, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4648[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4649[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19954, SGI0, SGI0, SGI19955, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4650[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4651[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19957, SGI0, SGI0, SGI19958, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4652[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4653[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4654[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4655[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19962, SGI0, SGI0, SGI19963, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4656[] = {SGI0, SGI19964, SGI19965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4657[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4658[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4659[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4660[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19970, SGI0, SGI0, SGI0, SGI0, SGI19971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4661[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4662[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4663[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19974, SGI0, SGI0, SGI19975, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4664[] = {SGI0, SGI19976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4665[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19978, SGI0, SGI0, SGI19979, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4666[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19980, SGI0, SGI19981, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4667[] = {SGI19982, SGI19983, SGI19984, SGI19985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19986, SGI0, SGI0, SGI0, SGI19987, SGI19988, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4668[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19989, SGI0, SGI19990, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4669[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4670[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4671[] = {SGI19993, SGI19994, SGI19995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI19996, SGI19997, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4672[] = {SGI19998, SGI19999, SGI20000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20001, SGI0, SGI0, SGI0, SGI20002, SGI20002, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4673[] = {SGI0, SGI20003, SGI0, SGI20004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20005, SGI0, SGI0, SGI0, SGI20006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4674[] = {SGI20007, SGI20008, SGI20009, SGI20010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20011, SGI0, SGI0, SGI0, SGI20012, SGI20013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4675[] = {SGI20014, SGI20015, SGI20016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20017, SGI0, SGI0, SGI0, SGI20018, SGI20018, SGI0, SGI0, SGI0, SGI20019, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4676[] = {SGI20020, SGI20021, SGI20022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20023, SGI0, SGI0, SGI0, SGI20024, SGI20024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20025, SGI0, NULL};
const Segment * const SG4677[] = {SGI20026, SGI20027, SGI20028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20029, SGI0, SGI0, SGI0, SGI20030, SGI20030, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4678[] = {SGI20031, SGI20032, SGI20033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20034, SGI0, SGI0, SGI0, SGI20035, SGI20035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4679[] = {SGI20036, SGI0, SGI20037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20038, SGI0, SGI0, SGI0, SGI20039, SGI20039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20040, SGI0, SGI0, NULL};
const Segment * const SG4680[] = {SGI0, SGI20041, SGI0, SGI20042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4681[] = {SGI20043, SGI20044, SGI20045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20046, SGI0, SGI0, SGI0, SGI20047, SGI20047, SGI0, SGI0, SGI0, SGI20048, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4682[] = {SGI20049, SGI20050, SGI20051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20052, SGI0, SGI0, SGI0, SGI20053, SGI20053, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4683[] = {SGI20054, SGI0, SGI20055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4684[] = {SGI0, SGI0, SGI20056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20057, SGI0, SGI20058, SGI0, SGI20059, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4685[] = {SGI20060, SGI20061, SGI20062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20063, SGI0, SGI0, SGI0, SGI20064, SGI20064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4686[] = {SGI20065, SGI20066, SGI20067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20068, SGI0, SGI0, SGI0, SGI20069, SGI20069, SGI0, SGI0, SGI0, SGI20070, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4687[] = {SGI20071, SGI20072, SGI20073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20074, SGI0, SGI0, SGI0, SGI20075, SGI20075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20076, SGI0, NULL};
const Segment * const SG4688[] = {SGI20077, SGI20078, SGI20079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20080, SGI0, SGI0, SGI0, SGI20081, SGI20081, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4689[] = {SGI20082, SGI20083, SGI20084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20085, SGI0, SGI0, SGI0, SGI20086, SGI20086, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4690[] = {SGI20087, SGI20088, SGI20089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20090, SGI0, SGI0, SGI0, SGI20091, SGI20091, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4691[] = {SGI20092, SGI20093, SGI20094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20095, SGI0, SGI0, SGI0, SGI20096, SGI20096, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4692[] = {SGI20097, SGI20098, SGI20099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20100, SGI0, SGI0, SGI0, SGI0, SGI20101, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4693[] = {SGI20102, SGI20103, SGI20104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20105, SGI0, SGI0, SGI0, SGI20106, SGI20106, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4694[] = {SGI20107, SGI20108, SGI20109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20110, SGI0, SGI0, SGI0, SGI20111, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4695[] = {SGI20112, SGI20113, SGI20114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20115, SGI0, SGI0, SGI0, SGI20116, SGI20116, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4696[] = {SGI20117, SGI20118, SGI20119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20120, SGI0, SGI0, SGI0, SGI20121, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4697[] = {SGI20122, SGI20123, SGI20124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20125, SGI0, SGI0, SGI0, SGI20126, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4698[] = {SGI20127, SGI20128, SGI20129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20130, SGI0, SGI0, SGI0, SGI20131, SGI20131, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4699[] = {SGI20132, SGI20133, SGI20134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20135, SGI0, SGI0, SGI0, SGI20136, SGI20136, SGI0, SGI0, NULL};
const Segment * const SG4700[] = {SGI20137, SGI20138, SGI20139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20140, SGI0, SGI0, SGI0, SGI20141, SGI20141, SGI0, SGI0, NULL};
const Segment * const SG4701[] = {SGI20142, SGI20143, SGI20144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20145, SGI0, SGI0, SGI0, SGI20146, SGI20146, SGI0, SGI0, NULL};
const Segment * const SG4702[] = {SGI20147, SGI20148, SGI20149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20150, SGI0, SGI0, SGI0, SGI20151, SGI20151, SGI0, SGI0, NULL};
const Segment * const SG4703[] = {SGI20152, SGI20153, SGI20154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20155, SGI0, SGI0, SGI0, SGI20156, SGI20156, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4704[] = {SGI20157, SGI20158, SGI20159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20160, SGI0, SGI0, SGI0, SGI20161, SGI20161, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4705[] = {SGI20162, SGI20163, SGI20164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20165, SGI0, SGI0, SGI0, SGI20166, SGI20166, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4706[] = {SGI20167, SGI20168, SGI20169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20170, SGI0, SGI0, SGI0, SGI20171, SGI20171, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4707[] = {SGI20172, SGI0, SGI20173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4708[] = {SGI20174, SGI20175, SGI20176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20177, SGI0, SGI0, SGI0, SGI20178, SGI20178, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4709[] = {SGI20179, SGI20180, SGI20181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20182, SGI0, SGI0, SGI0, SGI20183, SGI20183, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4710[] = {SGI20184, SGI20185, SGI20186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20187, SGI0, SGI0, SGI0, SGI20188, SGI20188, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4711[] = {SGI20189, SGI20190, SGI20191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20192, SGI0, SGI0, SGI0, SGI20193, SGI20193, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4712[] = {SGI20194, SGI20195, SGI20196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20197, SGI0, SGI0, SGI0, SGI20198, SGI20198, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4713[] = {SGI20199, SGI0, SGI20200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20201, SGI0, SGI0, SGI0, SGI20202, SGI20202, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4714[] = {SGI20203, SGI20204, SGI20205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20206, SGI20207, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4715[] = {SGI20208, SGI20209, SGI20210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20211, SGI0, SGI0, SGI0, SGI20212, SGI20212, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4716[] = {SGI20213, SGI0, SGI20214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20215, SGI0, SGI0, SGI0, SGI20216, SGI20216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4717[] = {SGI20217, SGI20218, SGI20219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20220, SGI0, SGI0, SGI0, SGI20221, SGI20221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4718[] = {SGI20222, SGI20223, SGI20224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20225, SGI0, SGI0, SGI0, SGI20226, SGI20226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4719[] = {SGI20227, SGI20228, SGI20229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20230, SGI0, SGI0, SGI0, SGI20231, SGI20231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4720[] = {SGI20232, SGI20233, SGI20234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20235, SGI0, SGI0, SGI0, SGI20236, SGI20236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4721[] = {SGI20237, SGI20238, SGI20239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20240, SGI0, SGI0, SGI0, SGI20241, SGI20241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4722[] = {SGI20242, SGI20243, SGI20244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20245, SGI0, SGI0, SGI0, SGI20246, SGI20246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4723[] = {SGI20247, SGI20248, SGI20249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20250, SGI0, SGI0, SGI0, SGI20251, SGI20251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4724[] = {SGI20252, SGI20253, SGI20254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20255, SGI0, SGI0, SGI0, SGI20256, SGI20256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4725[] = {SGI20257, SGI20258, SGI20259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20260, SGI0, SGI0, SGI0, SGI20261, SGI20261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4726[] = {SGI20262, SGI20263, SGI20264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20265, SGI0, SGI0, SGI0, SGI20266, SGI20266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4727[] = {SGI20267, SGI20268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20269, SGI0, SGI0, SGI0, SGI20270, SGI20270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4728[] = {SGI20271, SGI20272, SGI20273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20274, SGI0, SGI0, SGI0, SGI20275, SGI20275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4729[] = {SGI20276, SGI20277, SGI20278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20279, SGI0, SGI0, SGI0, SGI20280, SGI20280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4730[] = {SGI20281, SGI20282, SGI20283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20284, SGI0, SGI0, SGI0, SGI20285, SGI20285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4731[] = {SGI20286, SGI20287, SGI20288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20289, SGI0, SGI0, SGI0, SGI20290, SGI20290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4732[] = {SGI20291, SGI20292, SGI20293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20294, SGI0, SGI0, SGI0, SGI20295, SGI20295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4733[] = {SGI20296, SGI20297, SGI20298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20299, SGI0, SGI0, SGI0, SGI20300, SGI20300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4734[] = {SGI20301, SGI20302, SGI20303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20304, SGI0, SGI0, SGI0, SGI20305, SGI20305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4735[] = {SGI20306, SGI0, SGI20307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20308, SGI0, SGI0, SGI0, SGI20309, SGI20309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4736[] = {SGI20310, SGI20311, SGI20312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20313, SGI0, SGI0, SGI0, SGI20314, SGI20314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4737[] = {SGI20315, SGI20316, SGI20317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20318, SGI0, SGI0, SGI0, SGI20319, SGI20319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4738[] = {SGI20320, SGI0, SGI20321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4739[] = {SGI20322, SGI0, SGI20323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4740[] = {SGI20324, SGI20325, SGI20326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20327, SGI0, SGI0, SGI0, SGI20328, SGI20329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4741[] = {SGI20330, SGI0, SGI20331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4742[] = {SGI20332, SGI20333, SGI20334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20335, SGI20336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4743[] = {SGI20337, SGI20338, SGI20339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20340, SGI0, SGI0, SGI0, SGI20341, SGI20342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4744[] = {SGI20343, SGI20344, SGI20345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20346, SGI0, SGI0, SGI0, SGI20347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4745[] = {SGI20348, SGI0, SGI20349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20350, SGI0, SGI0, SGI0, SGI20351, SGI20351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4746[] = {SGI20352, SGI20353, SGI20354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20355, SGI20355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4747[] = {SGI20356, SGI0, SGI20357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20358, SGI0, SGI0, SGI0, SGI20359, SGI20359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4748[] = {SGI20360, SGI20361, SGI20362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20363, SGI20364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4749[] = {SGI20365, SGI20366, SGI20367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20368, SGI0, SGI0, SGI0, SGI20369, SGI20370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4750[] = {SGI20371, SGI20372, SGI20373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20374, SGI20374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4751[] = {SGI20375, SGI20376, SGI20377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20378, SGI0, SGI0, SGI0, SGI20379, SGI20379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4752[] = {SGI20380, SGI20381, SGI20382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20383, SGI0, SGI0, SGI0, SGI20384, SGI20384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4753[] = {SGI20385, SGI20386, SGI20387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20388, SGI0, SGI0, SGI0, SGI20389, SGI20389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4754[] = {SGI20390, SGI0, SGI20391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4755[] = {SGI20392, SGI20393, SGI20394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4756[] = {SGI20395, SGI0, SGI20396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4757[] = {SGI20397, SGI20398, SGI20399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20400, SGI20401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4758[] = {SGI20402, SGI20403, SGI20404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20405, SGI0, SGI0, SGI0, SGI20406, SGI20406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4759[] = {SGI20407, SGI20408, SGI20409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20410, SGI20411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4760[] = {SGI0, SGI20412, SGI20413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20414, SGI0, SGI0, SGI0, SGI20415, SGI20415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4761[] = {SGI20416, SGI20417, SGI20418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20419, SGI20419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4762[] = {SGI20420, SGI20421, SGI20422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20423, SGI20424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4763[] = {SGI20425, SGI0, SGI20426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20427, SGI0, SGI0, SGI0, SGI20428, SGI20428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4764[] = {SGI20429, SGI20430, SGI20431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20432, SGI20432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4765[] = {SGI20433, SGI20434, SGI20435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20436, SGI0, SGI0, SGI0, SGI20437, SGI20437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4766[] = {SGI20438, SGI20439, SGI20440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20441, SGI20441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4767[] = {SGI20442, SGI0, SGI20443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20444, SGI0, SGI0, SGI0, SGI20445, SGI20445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4768[] = {SGI20446, SGI20447, SGI20448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20449, SGI0, SGI0, SGI0, SGI20450, SGI20450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4769[] = {SGI20451, SGI20452, SGI20453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20454, SGI20454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4770[] = {SGI20455, SGI20456, SGI20457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20458, SGI0, SGI0, SGI0, SGI20459, SGI20459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4771[] = {SGI20460, SGI20461, SGI20462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20463, SGI0, SGI0, SGI0, SGI20464, SGI20464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4772[] = {SGI20465, SGI20466, SGI20467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20468, SGI0, SGI0, SGI0, SGI20469, SGI20469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4773[] = {SGI20470, SGI20471, SGI20472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20473, SGI0, SGI0, SGI0, SGI20474, SGI20474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4774[] = {SGI20475, SGI0, SGI20476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4775[] = {SGI20477, SGI20478, SGI20479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20480, SGI0, SGI0, SGI0, SGI20481, SGI20481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4776[] = {SGI20482, SGI20483, SGI20484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20485, SGI0, SGI0, SGI0, SGI20486, SGI20486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4777[] = {SGI20487, SGI20488, SGI20489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20490, SGI20491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4778[] = {SGI20492, SGI20493, SGI20494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20495, SGI0, SGI0, SGI0, SGI20496, SGI20496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4779[] = {SGI20497, SGI20498, SGI20499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20500, SGI0, SGI0, SGI0, SGI20501, SGI20501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4780[] = {SGI20502, SGI20503, SGI20504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20505, SGI20506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4781[] = {SGI20507, SGI20508, SGI20509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20510, SGI20511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4782[] = {SGI20512, SGI0, SGI20513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4783[] = {SGI20514, SGI20515, SGI20516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20517, SGI20518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4784[] = {SGI20519, SGI20520, SGI20521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20522, SGI0, SGI0, SGI0, SGI20523, SGI20523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4785[] = {SGI20524, SGI20525, SGI20526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20527, SGI20527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4786[] = {SGI20528, SGI0, SGI20529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20530, SGI0, SGI0, SGI0, SGI20531, SGI20531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4787[] = {SGI20532, SGI20533, SGI20534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20535, SGI20535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4788[] = {SGI20536, SGI0, SGI20537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4789[] = {SGI20538, SGI20539, SGI20540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20541, SGI20541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4790[] = {SGI20542, SGI20543, SGI20544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20545, SGI20545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4791[] = {SGI20546, SGI20547, SGI20548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20549, SGI20549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4792[] = {SGI0, SGI20550, SGI20551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4793[] = {SGI20552, SGI0, SGI20553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20554, SGI0, SGI0, SGI0, SGI20555, SGI20555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4794[] = {SGI20556, SGI20557, SGI20558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20559, SGI0, SGI0, SGI0, SGI20560, SGI20560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4795[] = {SGI20561, SGI20562, SGI20563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20564, SGI0, SGI0, SGI0, SGI20565, SGI20565, SGI0, SGI0, NULL};
const Segment * const SG4796[] = {SGI20566, SGI20567, SGI20568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20569, SGI0, SGI0, SGI0, SGI20570, SGI20570, SGI0, SGI0, NULL};
const Segment * const SG4797[] = {SGI20571, SGI20572, SGI20573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20574, SGI0, SGI0, SGI0, SGI20575, SGI20575, SGI0, SGI0, NULL};
const Segment * const SG4798[] = {SGI20576, SGI20577, SGI20578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20579, SGI0, SGI0, SGI0, SGI20580, SGI20580, SGI0, SGI0, NULL};
const Segment * const SG4799[] = {SGI20581, SGI20582, SGI20583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20584, SGI0, SGI0, SGI0, SGI20585, SGI20585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4800[] = {SGI20586, SGI20587, SGI20588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20589, SGI0, SGI0, SGI0, SGI20590, SGI20590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4801[] = {SGI20591, SGI20592, SGI20593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20594, SGI0, SGI0, SGI0, SGI20595, SGI20595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4802[] = {SGI20596, SGI20597, SGI20598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20599, SGI0, SGI0, SGI0, SGI20600, SGI20600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4803[] = {SGI20601, SGI20602, SGI20603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20604, SGI0, SGI0, SGI0, SGI20605, SGI20605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4804[] = {SGI20606, SGI20607, SGI20608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20609, SGI0, SGI0, SGI0, SGI20610, SGI20610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4805[] = {SGI20611, SGI0, SGI20612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4806[] = {SGI20613, SGI20614, SGI20615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20616, SGI20617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4807[] = {SGI20618, SGI20619, SGI20620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20621, SGI0, SGI0, SGI0, SGI20622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4808[] = {SGI20623, SGI0, SGI20624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20625, SGI0, SGI0, SGI0, SGI20626, SGI20626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4809[] = {SGI20627, SGI20628, SGI20629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20630, SGI0, SGI0, SGI0, SGI20631, SGI20631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4810[] = {SGI20632, SGI20633, SGI20634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20635, SGI0, SGI0, SGI0, SGI20636, SGI20636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4811[] = {SGI20637, SGI20638, SGI20639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20640, SGI0, SGI0, SGI0, SGI20641, SGI20641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4812[] = {SGI20642, SGI20643, SGI20644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20645, SGI0, SGI0, SGI0, SGI20646, SGI20646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4813[] = {SGI20647, SGI20648, SGI20649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20650, SGI0, SGI0, SGI0, SGI20651, SGI20651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4814[] = {SGI20652, SGI20653, SGI20654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20655, SGI0, SGI0, SGI0, SGI20656, SGI20656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4815[] = {SGI20657, SGI20658, SGI20659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20660, SGI0, SGI0, SGI0, SGI20661, SGI20661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4816[] = {SGI20662, SGI0, SGI20663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4817[] = {SGI20664, SGI20665, SGI20666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20667, SGI0, SGI0, SGI0, SGI20668, SGI20668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4818[] = {SGI20669, SGI20670, SGI20671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20672, SGI0, SGI0, SGI0, SGI20673, SGI20673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4819[] = {SGI20674, SGI20676, SGI20677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20680, SGI0, SGI0, SGI0, SGI20682, SGI20682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4820[] = {SGI20675, SGI20678, SGI20679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20681, SGI0, SGI0, SGI0, SGI0, SGI20683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4821[] = {SGI20684, SGI20685, SGI20686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20687, SGI0, SGI0, SGI0, SGI20688, SGI20688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4822[] = {SGI20689, SGI20690, SGI20691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20692, SGI0, SGI0, SGI0, SGI20694, SGI20694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4823[] = {SGI20693, SGI20695, SGI20696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20697, SGI0, SGI0, SGI0, SGI20698, SGI20698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4824[] = {SGI20699, SGI20700, SGI20701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20702, SGI0, SGI0, SGI0, SGI20703, SGI20703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4825[] = {SGI20704, SGI0, SGI20705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4826[] = {SGI20706, SGI20707, SGI20708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20709, SGI0, SGI0, SGI0, SGI20710, SGI20710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4827[] = {SGI20711, SGI0, SGI20713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4828[] = {SGI20712, SGI20714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20715, SGI0, SGI0, SGI0, SGI20716, SGI20716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4829[] = {SGI20717, SGI20718, SGI20719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20720, SGI0, SGI0, SGI0, SGI20721, SGI20721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4830[] = {SGI20723, SGI20725, SGI20727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20730, SGI20731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4831[] = {SGI20722, SGI20724, SGI20726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20728, SGI0, SGI0, SGI0, SGI20729, SGI20729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4832[] = {SGI20732, SGI20733, SGI20734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20735, SGI0, SGI0, SGI0, SGI20736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4833[] = {SGI20737, SGI20738, SGI20739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20740, SGI0, SGI0, SGI0, SGI20741, SGI20741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4834[] = {SGI20742, SGI20743, SGI20744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20745, SGI0, SGI0, SGI0, SGI20746, SGI20746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4835[] = {SGI20747, SGI20748, SGI20749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20750, SGI0, SGI0, SGI0, SGI20751, SGI20751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4836[] = {SGI20752, SGI20753, SGI20754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20755, SGI0, SGI0, SGI0, SGI20756, SGI20756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4837[] = {SGI20757, SGI20758, SGI20759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20760, SGI0, SGI0, SGI0, SGI20761, SGI20761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4838[] = {SGI20762, SGI20763, SGI20764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20765, SGI0, SGI0, SGI0, SGI20766, SGI20766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4839[] = {SGI20767, SGI20768, SGI20769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20770, SGI0, SGI0, SGI0, SGI20771, SGI20771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4840[] = {SGI20772, SGI20773, SGI20774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20775, SGI20775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4841[] = {SGI20776, SGI20777, SGI20778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20779, SGI0, SGI0, SGI0, SGI20780, SGI20780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4842[] = {SGI20781, SGI0, SGI20782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4843[] = {SGI20783, SGI20784, SGI20785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20786, SGI0, SGI0, SGI0, SGI20787, SGI20787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4844[] = {SGI20788, SGI0, SGI20789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4845[] = {SGI20790, SGI20791, SGI20792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20793, SGI0, SGI0, SGI0, SGI20794, SGI20794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4846[] = {SGI20795, SGI20796, SGI20797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20798, SGI0, SGI0, SGI0, SGI20799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4847[] = {SGI20800, SGI20801, SGI20802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20803, SGI0, SGI0, SGI0, SGI20804, SGI20804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4848[] = {SGI20805, SGI0, SGI20806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4849[] = {SGI20807, SGI20808, SGI20809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4850[] = {SGI20810, SGI20811, SGI20812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20813, SGI0, SGI0, SGI0, SGI20814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4851[] = {SGI20815, SGI0, SGI20816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20817, SGI0, SGI0, SGI0, SGI20818, SGI20818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4852[] = {SGI20819, SGI0, SGI20820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4853[] = {SGI0, SGI20821, SGI20822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4854[] = {SGI20823, SGI20824, SGI20825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20826, SGI20827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4855[] = {SGI20828, SGI0, SGI20829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20830, SGI0, SGI0, SGI0, SGI20831, SGI20831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4856[] = {SGI20832, SGI20833, SGI20834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20835, SGI20836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4857[] = {SGI20837, SGI0, SGI20840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20842, SGI0, SGI0, SGI0, SGI20844, SGI20844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4858[] = {SGI20838, SGI20839, SGI20841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20843, SGI0, SGI0, SGI0, SGI20845, SGI20845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4859[] = {SGI20846, SGI20847, SGI20848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20849, SGI20849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4860[] = {SGI20850, SGI0, SGI20851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4861[] = {SGI20852, SGI20853, SGI20854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20855, SGI20856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4862[] = {SGI20857, SGI20858, SGI20859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20860, SGI20860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4863[] = {SGI0, SGI20861, SGI20862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4864[] = {SGI20863, SGI20864, SGI20865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20866, SGI0, SGI0, SGI0, SGI20867, SGI20867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4865[] = {SGI20868, SGI20869, SGI20870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20871, SGI0, SGI0, SGI0, SGI20872, SGI20872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4866[] = {SGI20873, SGI20874, SGI20875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20876, SGI0, SGI0, SGI0, SGI20877, SGI20877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4867[] = {SGI20878, SGI20879, SGI20880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20881, SGI0, SGI0, SGI0, SGI20882, SGI20882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4868[] = {SGI20883, SGI0, SGI20884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4869[] = {SGI20885, SGI20886, SGI20887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20888, SGI0, SGI0, SGI0, SGI20889, SGI20889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4870[] = {SGI20890, SGI20891, SGI20892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20893, SGI0, SGI0, SGI0, SGI20894, SGI20894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4871[] = {SGI20895, SGI20896, SGI20897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20898, SGI0, SGI0, SGI0, SGI20899, SGI20899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4872[] = {SGI20900, SGI0, SGI20901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4873[] = {SGI20902, SGI20903, SGI20904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20905, SGI0, SGI0, SGI0, SGI20906, SGI20906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4874[] = {SGI20907, SGI0, SGI20908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4875[] = {SGI20909, SGI20910, SGI20911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20912, SGI0, SGI0, SGI0, SGI20913, SGI20913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4876[] = {SGI20914, SGI20915, SGI20916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20917, SGI0, SGI0, SGI0, SGI20918, SGI20918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4877[] = {SGI20919, SGI20920, SGI20921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20922, SGI0, SGI0, SGI0, SGI20923, SGI20923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4878[] = {SGI20924, SGI0, SGI20925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4879[] = {SGI20926, SGI0, SGI20927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20928, SGI0, SGI0, SGI0, SGI20929, SGI20929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4880[] = {SGI20930, SGI20931, SGI20932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20933, SGI0, SGI0, SGI0, SGI20934, SGI20934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4881[] = {SGI20935, SGI20936, SGI20937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20938, SGI0, SGI0, SGI0, SGI20939, SGI20939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4882[] = {SGI20940, SGI20941, SGI20942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20943, SGI0, SGI0, SGI0, SGI20944, SGI20944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4883[] = {SGI20945, SGI20946, SGI20947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20948, SGI0, SGI0, SGI0, SGI20949, SGI20949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4884[] = {SGI20950, SGI20951, SGI20952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20953, SGI0, SGI0, SGI0, SGI20954, SGI20954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4885[] = {SGI20955, SGI20956, SGI20957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20958, SGI0, SGI0, SGI0, SGI20959, SGI20959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4886[] = {SGI20960, SGI20961, SGI20962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20963, SGI0, SGI0, SGI0, SGI20964, SGI20964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4887[] = {SGI20965, SGI0, SGI20966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20967, SGI0, SGI0, SGI0, SGI20968, SGI20968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4888[] = {SGI20969, SGI20970, SGI20971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20972, SGI0, SGI0, SGI0, SGI20973, SGI20973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4889[] = {SGI20974, SGI20975, SGI20976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20977, SGI0, SGI0, SGI0, SGI20978, SGI20978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4890[] = {SGI20979, SGI20980, SGI20981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20982, SGI0, SGI0, SGI0, SGI20983, SGI20983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4891[] = {SGI20984, SGI20985, SGI20986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20987, SGI0, SGI0, SGI0, SGI20988, SGI20988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4892[] = {SGI20989, SGI20990, SGI20991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20992, SGI0, SGI0, SGI0, SGI20993, SGI20993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4893[] = {SGI20994, SGI20995, SGI20996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI20997, SGI0, SGI0, SGI0, SGI20998, SGI20998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4894[] = {SGI20999, SGI21000, SGI21001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21004, SGI0, SGI0, SGI0, SGI21006, SGI21006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4895[] = {SGI21002, SGI21003, SGI21005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21007, SGI0, SGI0, SGI0, SGI21008, SGI21008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4896[] = {SGI21009, SGI21010, SGI21011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21013, SGI0, SGI0, SGI0, SGI21016, SGI21016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4897[] = {SGI21012, SGI21014, SGI21015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21017, SGI0, SGI0, SGI0, SGI21018, SGI21018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4898[] = {SGI21019, SGI21020, SGI21021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21022, SGI0, SGI0, SGI0, SGI21023, SGI21023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4899[] = {SGI21024, SGI21025, SGI21026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21027, SGI0, SGI0, SGI0, SGI21028, SGI21028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4900[] = {SGI21029, SGI21030, SGI21031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21032, SGI0, SGI0, SGI0, SGI21033, SGI21033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4901[] = {SGI21034, SGI21035, SGI21036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21037, SGI0, SGI0, SGI0, SGI21038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4902[] = {SGI21039, SGI21040, SGI21041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21042, SGI0, SGI0, SGI0, SGI21043, SGI21043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4903[] = {SGI21044, SGI21045, SGI21046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21047, SGI0, SGI0, SGI0, SGI21048, SGI21048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4904[] = {SGI21049, SGI0, SGI21050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21051, SGI0, SGI0, SGI0, SGI21052, SGI21052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4905[] = {SGI21053, SGI21054, SGI21055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21056, SGI21057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4906[] = {SGI21058, SGI0, SGI21059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21061, SGI0, SGI0, SGI0, SGI21064, SGI21064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4907[] = {SGI21060, SGI21062, SGI21063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21068, SGI0, SGI0, SGI0, SGI21069, SGI21069, SGI0, SGI0, NULL};
const Segment * const SG4908[] = {SGI21065, SGI21066, SGI21067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21070, SGI0, SGI0, SGI0, SGI21071, SGI21071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4909[] = {SGI21072, SGI21073, SGI21074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21075, SGI0, SGI0, SGI0, SGI21076, SGI21076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4910[] = {SGI21077, SGI21078, SGI21079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21080, SGI0, SGI0, SGI0, SGI21081, SGI21081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4911[] = {SGI21082, SGI0, SGI21083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4912[] = {SGI21084, SGI21085, SGI21086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21087, SGI0, SGI0, SGI0, SGI21088, SGI21088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4913[] = {SGI21089, SGI21090, SGI21091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21092, SGI0, SGI0, SGI0, SGI21093, SGI21093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4914[] = {SGI21094, SGI0, SGI21095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4915[] = {SGI21096, SGI21097, SGI21098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21101, SGI0, SGI0, SGI0, SGI21103, SGI21103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4916[] = {SGI21099, SGI21100, SGI21102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4917[] = {SGI21104, SGI0, SGI21105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4918[] = {SGI21106, SGI0, SGI21107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21110, SGI0, SGI0, SGI0, SGI21112, SGI21112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4919[] = {SGI21108, SGI21109, SGI21111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21113, SGI0, SGI0, SGI0, SGI21114, SGI21114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4920[] = {SGI21115, SGI0, SGI21116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4921[] = {SGI21117, SGI21118, SGI21119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21120, SGI21121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4922[] = {SGI21122, SGI21123, SGI21124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21125, SGI0, SGI0, SGI0, SGI21126, SGI21126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4923[] = {SGI0, SGI21127, SGI21128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21129, SGI0, SGI0, SGI0, SGI21130, SGI21130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4924[] = {SGI21131, SGI21132, SGI21133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21134, SGI0, SGI0, SGI0, SGI21135, SGI21135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4925[] = {SGI21136, SGI21137, SGI21138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21139, SGI0, SGI0, SGI0, SGI21140, SGI21140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4926[] = {SGI21141, SGI21142, SGI21143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21144, SGI0, SGI0, SGI0, SGI21145, SGI21146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4927[] = {SGI21147, SGI21148, SGI21149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21150, SGI0, SGI0, SGI0, SGI21151, SGI21151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4928[] = {SGI21152, SGI21153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21154, SGI0, SGI0, SGI0, SGI21155, SGI21155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4929[] = {SGI21156, SGI21157, SGI21158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21159, SGI21159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4930[] = {SGI21160, SGI21161, SGI21162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21163, SGI0, SGI0, SGI0, SGI21164, SGI21164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4931[] = {SGI21165, SGI21166, SGI21167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21168, SGI0, SGI0, SGI0, SGI21169, SGI21169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4932[] = {SGI21170, SGI21171, SGI21172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21173, SGI0, SGI0, SGI0, SGI21174, SGI21174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4933[] = {SGI21180, SGI21181, SGI21182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21183, SGI21183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4934[] = {SGI21184, SGI21185, SGI21186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21187, SGI0, SGI0, SGI0, SGI21188, SGI21188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4935[] = {SGI21189, SGI21190, SGI21191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21192, SGI0, SGI0, SGI0, SGI21193, SGI21193, SGI0, SGI0, NULL};
const Segment * const SG4936[] = {SGI21194, SGI21195, SGI21196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21197, SGI0, SGI0, SGI0, SGI21198, SGI21198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4937[] = {SGI21199, SGI21200, SGI21201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21202, SGI0, SGI0, SGI0, SGI21203, SGI21203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4938[] = {SGI21204, SGI21205, SGI21206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21207, SGI0, SGI0, SGI0, SGI21208, SGI21208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4939[] = {SGI21209, SGI21210, SGI21211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21212, SGI0, SGI0, SGI0, SGI21213, SGI21213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4940[] = {SGI21214, SGI21215, SGI21216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21217, SGI0, SGI0, SGI0, SGI21218, SGI21218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4941[] = {SGI21175, SGI21176, SGI21177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21178, SGI21179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4942[] = {SGI21219, SGI21220, SGI21221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21222, SGI0, SGI0, SGI0, SGI21223, SGI21223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4943[] = {SGI21224, SGI21225, SGI21226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21230, SGI0, SGI0, SGI0, SGI21231, SGI21231, SGI0, SGI0, NULL};
const Segment * const SG4944[] = {SGI21227, SGI21228, SGI21229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21233, SGI0, SGI0, SGI0, SGI21235, SGI21235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4945[] = {SGI21232, SGI0, SGI21234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4946[] = {SGI21236, SGI21237, SGI21238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21239, SGI0, SGI0, SGI0, SGI21240, SGI21240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4947[] = {SGI21241, SGI21242, SGI21243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21244, SGI0, SGI0, SGI0, SGI21245, SGI21245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4948[] = {SGI21246, SGI21247, SGI21248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21249, SGI0, SGI0, SGI0, SGI21250, SGI21250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4949[] = {SGI21251, SGI21252, SGI21253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21254, SGI0, SGI0, SGI0, SGI21255, SGI21255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4950[] = {SGI21256, SGI21257, SGI21258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21259, SGI0, SGI0, SGI0, SGI21260, SGI21260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4951[] = {SGI21261, SGI21262, SGI21263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21264, SGI21264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4952[] = {SGI21265, SGI21266, SGI21267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21268, SGI0, SGI0, SGI0, SGI21269, SGI21269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4953[] = {SGI21270, SGI21271, SGI21272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21273, SGI0, SGI0, SGI0, SGI21274, SGI21274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4954[] = {SGI21275, SGI21276, SGI21277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21278, SGI0, SGI0, SGI0, SGI21279, SGI21279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4955[] = {SGI21280, SGI21281, SGI21282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21283, SGI0, SGI0, SGI0, SGI21284, SGI21284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4956[] = {SGI21285, SGI21286, SGI21287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21288, SGI0, SGI0, SGI0, SGI21289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4957[] = {SGI21290, SGI21291, SGI21292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21293, SGI21293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4958[] = {SGI21294, SGI21295, SGI21296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21297, SGI0, SGI0, SGI0, SGI21298, SGI21298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4959[] = {SGI21299, SGI21300, SGI21301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21302, SGI0, SGI0, SGI0, SGI21303, SGI21303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4960[] = {SGI21304, SGI21305, SGI21306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21307, SGI0, SGI0, SGI0, SGI21308, SGI21308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4961[] = {SGI21309, SGI21310, SGI21311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21312, SGI0, SGI0, SGI0, SGI21313, SGI21313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4962[] = {SGI21314, SGI21315, SGI21316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21317, SGI0, SGI0, SGI0, SGI21318, SGI21318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4963[] = {SGI21319, SGI21320, SGI21321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21322, SGI0, SGI0, SGI0, SGI21323, SGI21323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4964[] = {SGI21324, SGI21325, SGI21326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21327, SGI0, SGI0, SGI0, SGI21328, SGI21328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4965[] = {SGI21329, SGI21330, SGI21331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21332, SGI0, SGI0, SGI0, SGI21333, SGI21333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4966[] = {SGI21334, SGI21335, SGI21336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21337, SGI0, SGI0, SGI0, SGI21338, SGI21338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4967[] = {SGI21339, SGI21340, SGI21341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21342, SGI0, SGI0, SGI0, SGI21343, SGI21343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4968[] = {SGI21344, SGI21345, SGI21346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21347, SGI0, SGI0, SGI0, SGI21348, SGI21348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4969[] = {SGI21349, SGI21350, SGI21351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21352, SGI0, SGI0, SGI0, SGI21353, SGI21353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4970[] = {SGI21354, SGI0, SGI21355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4971[] = {SGI21356, SGI0, SGI21357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4972[] = {SGI21358, SGI0, SGI21359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4973[] = {SGI21360, SGI21361, SGI21362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21363, SGI0, SGI0, SGI0, SGI21364, SGI21364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4974[] = {SGI21365, SGI21366, SGI21367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21368, SGI0, SGI0, SGI0, SGI21369, SGI21369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4975[] = {SGI21370, SGI21371, SGI21372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21373, SGI0, SGI0, SGI0, SGI21374, SGI21374, SGI0, SGI0, NULL};
const Segment * const SG4976[] = {SGI21375, SGI21376, SGI21377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21378, SGI0, SGI0, SGI0, SGI21379, SGI21379, SGI0, SGI0, NULL};
const Segment * const SG4977[] = {SGI21380, SGI0, SGI21381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4978[] = {SGI21382, SGI21383, SGI21384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21385, SGI0, SGI0, SGI0, SGI21386, SGI21386, SGI0, SGI0, NULL};
const Segment * const SG4979[] = {SGI21387, SGI21388, SGI21389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21390, SGI0, SGI0, SGI0, SGI21391, SGI21391, SGI0, SGI0, NULL};
const Segment * const SG4980[] = {SGI21392, SGI21393, SGI21394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21395, SGI0, SGI0, SGI0, SGI21396, SGI21396, SGI0, SGI0, NULL};
const Segment * const SG4981[] = {SGI21397, SGI21398, SGI21399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21400, SGI0, SGI0, SGI0, SGI21401, SGI21401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4982[] = {SGI21402, SGI21403, SGI21404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21405, SGI0, SGI0, SGI0, SGI21406, SGI21406, SGI0, SGI0, NULL};
const Segment * const SG4983[] = {SGI21407, SGI21408, SGI21409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21410, SGI0, SGI0, SGI0, SGI21411, SGI21411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4984[] = {SGI21412, SGI21413, SGI21414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21415, SGI21416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4985[] = {SGI21417, SGI21418, SGI21419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21423, SGI0, SGI0, SGI0, SGI21424, SGI21424, SGI0, SGI0, NULL};
const Segment * const SG4986[] = {SGI21420, SGI21421, SGI21422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21425, SGI0, SGI0, SGI0, SGI21426, SGI21426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4987[] = {SGI21427, SGI21428, SGI21429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21430, SGI21431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4988[] = {SGI21432, SGI21433, SGI21434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21435, SGI0, SGI0, SGI0, SGI21436, SGI21436, SGI0, SGI0, NULL};
const Segment * const SG4989[] = {SGI21437, SGI21438, SGI21439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21440, SGI21440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4990[] = {SGI21441, SGI21442, SGI21443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21444, SGI0, SGI0, SGI0, SGI21445, SGI21445, SGI0, SGI0, NULL};
const Segment * const SG4991[] = {SGI21446, SGI21448, SGI21449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21452, SGI0, SGI0, SGI0, SGI21454, SGI21454, SGI0, SGI0, NULL};
const Segment * const SG4992[] = {SGI21447, SGI21450, SGI21451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21453, SGI0, SGI0, SGI0, SGI21455, SGI21455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4993[] = {SGI21456, SGI21457, SGI21458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21459, SGI0, SGI0, SGI0, SGI21460, SGI21460, SGI0, SGI0, NULL};
const Segment * const SG4994[] = {SGI21461, SGI0, SGI21462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21463, SGI0, SGI0, SGI0, SGI21464, SGI21464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4995[] = {SGI21465, SGI21466, SGI21467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21468, SGI0, SGI0, SGI0, SGI21469, SGI21469, SGI0, SGI0, NULL};
const Segment * const SG4996[] = {SGI21470, SGI21471, SGI21472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21473, SGI21473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4997[] = {SGI21474, SGI21475, SGI21476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21477, SGI0, SGI0, SGI0, SGI21478, SGI21478, SGI0, SGI0, NULL};
const Segment * const SG4998[] = {SGI21479, SGI21480, SGI21481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21482, SGI21482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG4999[] = {SGI21483, SGI21484, SGI21485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21486, SGI0, SGI0, SGI0, SGI21487, SGI21487, SGI0, SGI0, NULL};
const Segment * const SG5000[] = {SGI21488, SGI21489, SGI21490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21491, SGI0, SGI0, SGI0, SGI21492, SGI21492, SGI0, SGI0, NULL};
const Segment * const SG5001[] = {SGI21493, SGI21494, SGI21495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21496, SGI0, SGI0, SGI0, SGI21497, SGI21497, SGI0, SGI0, NULL};
const Segment * const SG5002[] = {SGI21498, SGI21499, SGI21500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21501, SGI0, SGI0, SGI0, SGI21502, SGI21502, SGI0, SGI0, NULL};
const Segment * const SG5003[] = {SGI21503, SGI21504, SGI21505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21506, SGI0, SGI0, SGI0, SGI21507, SGI21507, SGI0, SGI0, NULL};
const Segment * const SG5004[] = {SGI21508, SGI21509, SGI21510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21511, SGI0, SGI0, SGI0, SGI21512, SGI21512, SGI0, SGI0, NULL};
const Segment * const SG5005[] = {SGI21513, SGI21514, SGI21515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21516, SGI0, SGI0, SGI0, SGI21517, SGI21517, SGI0, SGI0, NULL};
const Segment * const SG5006[] = {SGI21518, SGI21519, SGI21520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21521, SGI0, SGI0, SGI0, SGI21522, SGI21522, SGI0, SGI0, NULL};
const Segment * const SG5007[] = {SGI21523, SGI21524, SGI21525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21526, SGI0, SGI0, SGI0, SGI21527, SGI21527, SGI0, SGI0, NULL};
const Segment * const SG5008[] = {SGI21528, SGI21529, SGI21530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21531, SGI0, SGI0, SGI0, SGI21532, SGI21532, SGI0, SGI0, NULL};
const Segment * const SG5009[] = {SGI21533, SGI21534, SGI21535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21536, SGI0, SGI0, SGI0, SGI21537, SGI21537, SGI0, SGI0, NULL};
const Segment * const SG5010[] = {SGI21538, SGI21539, SGI21540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21541, SGI0, SGI0, SGI0, SGI21542, SGI21542, SGI0, SGI0, NULL};
const Segment * const SG5011[] = {SGI21543, SGI21544, SGI21545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21546, SGI0, SGI0, SGI0, SGI21547, SGI21547, SGI0, SGI0, NULL};
const Segment * const SG5012[] = {SGI21548, SGI21549, SGI21550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21551, SGI0, SGI0, SGI0, SGI21552, SGI21552, SGI0, SGI0, NULL};
const Segment * const SG5013[] = {SGI21553, SGI21554, SGI21555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21556, SGI0, SGI0, SGI0, SGI21557, SGI21557, SGI0, SGI0, NULL};
const Segment * const SG5014[] = {SGI21558, SGI21559, SGI21560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21561, SGI0, SGI0, SGI0, SGI21562, SGI21562, SGI0, SGI0, NULL};
const Segment * const SG5015[] = {SGI21563, SGI21564, SGI21565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21566, SGI0, SGI0, SGI0, SGI21567, SGI21567, SGI0, SGI0, NULL};
const Segment * const SG5016[] = {SGI21568, SGI21569, SGI21570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21571, SGI0, SGI0, SGI0, SGI21572, SGI21572, SGI0, SGI0, NULL};
const Segment * const SG5017[] = {SGI21573, SGI21574, SGI21575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21576, SGI0, SGI0, SGI0, SGI21577, SGI21577, SGI0, SGI0, NULL};
const Segment * const SG5018[] = {SGI21578, SGI21579, SGI21580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21581, SGI0, SGI0, SGI0, SGI21582, SGI21582, SGI0, SGI0, NULL};
const Segment * const SG5019[] = {SGI21583, SGI21584, SGI21585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21586, SGI0, SGI0, SGI0, SGI21587, SGI21587, SGI0, SGI0, NULL};
const Segment * const SG5020[] = {SGI21588, SGI21589, SGI21590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21591, SGI0, SGI0, SGI0, SGI21592, SGI21592, SGI0, SGI0, NULL};
const Segment * const SG5021[] = {SGI21593, SGI21594, SGI21595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21596, SGI0, SGI0, SGI0, SGI21597, SGI21597, SGI0, SGI0, NULL};
const Segment * const SG5022[] = {SGI21598, SGI21599, SGI21600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21601, SGI0, SGI0, SGI0, SGI21602, SGI21602, SGI0, SGI0, NULL};
const Segment * const SG5023[] = {SGI21603, SGI21604, SGI21605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21606, SGI0, SGI0, SGI0, SGI21607, SGI21607, SGI0, SGI0, NULL};
const Segment * const SG5024[] = {SGI21608, SGI21609, SGI21610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21611, SGI0, SGI0, SGI0, SGI21612, SGI21612, SGI0, SGI0, NULL};
const Segment * const SG5025[] = {SGI21613, SGI21614, SGI21615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21616, SGI0, SGI0, SGI0, SGI21617, SGI21617, SGI0, SGI0, NULL};
const Segment * const SG5026[] = {SGI21618, SGI21619, SGI21620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21621, SGI0, SGI0, SGI0, SGI21622, SGI21622, SGI0, SGI0, NULL};
const Segment * const SG5027[] = {SGI21623, SGI21624, SGI21625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21626, SGI0, SGI0, SGI0, SGI21627, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5028[] = {SGI21628, SGI21629, SGI21630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21631, SGI0, SGI0, SGI0, SGI21632, SGI21632, SGI0, SGI0, NULL};
const Segment * const SG5029[] = {SGI21633, SGI21634, SGI21635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21636, SGI0, SGI0, SGI0, SGI21637, SGI21637, SGI0, SGI0, NULL};
const Segment * const SG5030[] = {SGI21638, SGI21639, SGI21640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21641, SGI0, SGI0, SGI0, SGI21642, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5031[] = {SGI21643, SGI21644, SGI21645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21646, SGI0, SGI0, SGI0, SGI21647, SGI21647, SGI0, SGI0, NULL};
const Segment * const SG5032[] = {SGI21648, SGI21649, SGI21650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21651, SGI0, SGI0, SGI0, SGI21652, SGI21652, SGI0, SGI0, NULL};
const Segment * const SG5033[] = {SGI21653, SGI21654, SGI21655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21656, SGI0, SGI0, SGI0, SGI21657, SGI21657, SGI0, SGI0, NULL};
const Segment * const SG5034[] = {SGI21658, SGI21659, SGI21660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21661, SGI0, SGI0, SGI0, SGI21662, SGI21662, SGI0, SGI0, NULL};
const Segment * const SG5035[] = {SGI21663, SGI21664, SGI21665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21666, SGI0, SGI0, SGI0, SGI21667, SGI21667, SGI0, SGI0, NULL};
const Segment * const SG5036[] = {SGI21668, SGI21669, SGI21670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21671, SGI0, SGI0, SGI0, SGI21672, SGI21672, SGI0, SGI0, NULL};
const Segment * const SG5037[] = {SGI21673, SGI21674, SGI21675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21676, SGI0, SGI0, SGI0, SGI21677, SGI21677, SGI0, SGI0, NULL};
const Segment * const SG5038[] = {SGI21678, SGI21679, SGI21680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21681, SGI0, SGI0, SGI0, SGI21682, SGI21682, SGI0, SGI0, NULL};
const Segment * const SG5039[] = {SGI21683, SGI21684, SGI21685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21686, SGI0, SGI0, SGI0, SGI21687, SGI21687, SGI0, SGI0, NULL};
const Segment * const SG5040[] = {SGI21688, SGI21689, SGI21690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21691, SGI0, SGI0, SGI0, SGI21692, SGI21692, SGI0, SGI0, NULL};
const Segment * const SG5041[] = {SGI21693, SGI21694, SGI21695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21696, SGI0, SGI0, SGI0, SGI21697, SGI21697, SGI0, SGI0, NULL};
const Segment * const SG5042[] = {SGI21698, SGI21699, SGI21700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21701, SGI0, SGI0, SGI0, SGI21702, SGI21702, SGI0, SGI0, NULL};
const Segment * const SG5043[] = {SGI21703, SGI21704, SGI21705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21706, SGI0, SGI0, SGI0, SGI21707, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5044[] = {SGI21708, SGI21709, SGI21710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21711, SGI0, SGI0, SGI0, SGI21712, SGI21712, SGI0, SGI0, NULL};
const Segment * const SG5045[] = {SGI21713, SGI21714, SGI21715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21716, SGI0, SGI0, SGI0, SGI21717, SGI21717, SGI0, SGI0, NULL};
const Segment * const SG5046[] = {SGI21718, SGI21719, SGI21720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21721, SGI0, SGI0, SGI0, SGI21722, SGI21722, SGI0, SGI0, NULL};
const Segment * const SG5047[] = {SGI21723, SGI21724, SGI21725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21726, SGI21727, SGI0, SGI0, NULL};
const Segment * const SG5048[] = {SGI21728, SGI21729, SGI21730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21731, SGI0, SGI0, SGI0, SGI21732, SGI21732, SGI0, SGI0, NULL};
const Segment * const SG5049[] = {SGI21733, SGI21734, SGI21735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21736, SGI0, SGI0, SGI0, SGI21737, SGI21737, SGI0, SGI0, NULL};
const Segment * const SG5050[] = {SGI21738, SGI21739, SGI21740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21741, SGI0, SGI0, SGI0, SGI21742, SGI21742, SGI0, SGI0, NULL};
const Segment * const SG5051[] = {SGI21743, SGI21744, SGI21745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21746, SGI0, SGI0, SGI0, SGI21747, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5052[] = {SGI21748, SGI21749, SGI21750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21751, SGI0, SGI0, SGI0, SGI21752, SGI21752, SGI0, SGI0, NULL};
const Segment * const SG5053[] = {SGI21753, SGI0, SGI21754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5054[] = {SGI21755, SGI21756, SGI21757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21758, SGI0, SGI0, SGI0, SGI21759, SGI21759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5055[] = {SGI0, SGI21760, SGI21761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5056[] = {SGI21762, SGI21763, SGI21764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21765, SGI21765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5057[] = {SGI21766, SGI21767, SGI21768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21769, SGI0, SGI0, SGI0, SGI21770, SGI21770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5058[] = {SGI21771, SGI21772, SGI21773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21774, SGI21774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5059[] = {SGI21775, SGI0, SGI21776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21777, SGI0, SGI0, SGI0, SGI21778, SGI21778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5060[] = {SGI21779, SGI21780, SGI21781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21782, SGI0, SGI0, SGI0, SGI21783, SGI21783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5061[] = {SGI21784, SGI21785, SGI21786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21787, SGI0, SGI0, SGI0, SGI21788, SGI21788, SGI0, SGI0, NULL};
const Segment * const SG5062[] = {SGI21789, SGI21790, SGI21791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21792, SGI0, SGI0, SGI0, SGI21793, SGI21793, SGI0, SGI0, NULL};
const Segment * const SG5063[] = {SGI21794, SGI21795, SGI21796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21797, SGI0, SGI0, SGI0, SGI21798, SGI21798, SGI0, SGI0, NULL};
const Segment * const SG5064[] = {SGI21799, SGI21800, SGI21801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21802, SGI0, SGI0, SGI0, SGI21803, SGI21803, SGI0, SGI0, NULL};
const Segment * const SG5065[] = {SGI21804, SGI21805, SGI21806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21807, SGI0, SGI0, SGI0, SGI21808, SGI21808, SGI0, SGI0, NULL};
const Segment * const SG5066[] = {SGI21809, SGI21810, SGI21811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21812, SGI0, SGI0, SGI0, SGI21813, SGI21813, SGI0, SGI0, NULL};
const Segment * const SG5067[] = {SGI21814, SGI21815, SGI21816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21817, SGI0, SGI0, SGI0, SGI21818, SGI21818, SGI0, SGI0, NULL};
const Segment * const SG5068[] = {SGI21819, SGI21820, SGI21821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21822, SGI0, SGI0, SGI0, SGI21823, SGI21823, SGI0, SGI0, NULL};
const Segment * const SG5069[] = {SGI21824, SGI21825, SGI21826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21827, SGI0, SGI0, SGI0, SGI21828, SGI21828, SGI0, SGI0, NULL};
const Segment * const SG5070[] = {SGI21829, SGI21830, SGI21831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21832, SGI0, SGI0, SGI0, SGI21833, SGI21833, SGI0, SGI0, NULL};
const Segment * const SG5071[] = {SGI21834, SGI21835, SGI21836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21837, SGI0, SGI0, SGI0, SGI21838, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5072[] = {SGI21839, SGI21840, SGI21841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21842, SGI0, SGI0, SGI0, SGI21843, SGI21843, SGI0, SGI0, NULL};
const Segment * const SG5073[] = {SGI21844, SGI21845, SGI21846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21847, SGI0, SGI0, SGI0, SGI21848, SGI21848, SGI0, SGI0, NULL};
const Segment * const SG5074[] = {SGI21849, SGI21850, SGI21851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21852, SGI0, SGI0, SGI0, SGI21853, SGI21853, SGI0, SGI0, NULL};
const Segment * const SG5075[] = {SGI21854, SGI21855, SGI21856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21857, SGI0, SGI0, SGI0, SGI21858, SGI21858, SGI0, SGI0, NULL};
const Segment * const SG5076[] = {SGI21859, SGI21860, SGI21861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21862, SGI0, SGI0, SGI0, SGI21863, SGI21863, SGI0, SGI0, NULL};
const Segment * const SG5077[] = {SGI21864, SGI21865, SGI21866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21867, SGI0, SGI0, SGI0, SGI21868, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5078[] = {SGI21869, SGI21870, SGI21871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21872, SGI0, SGI0, SGI0, SGI21873, SGI21873, SGI0, SGI0, NULL};
const Segment * const SG5079[] = {SGI21874, SGI21875, SGI21876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21877, SGI0, SGI0, SGI0, SGI21878, SGI21878, SGI0, SGI0, NULL};
const Segment * const SG5080[] = {SGI21879, SGI21880, SGI21881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21882, SGI0, SGI0, SGI0, SGI21883, SGI21883, SGI0, SGI0, NULL};
const Segment * const SG5081[] = {SGI21884, SGI21885, SGI21886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21887, SGI0, SGI0, SGI0, SGI21888, SGI21888, SGI0, SGI0, NULL};
const Segment * const SG5082[] = {SGI21889, SGI21890, SGI21891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21892, SGI0, SGI0, SGI0, SGI21893, SGI21893, SGI0, SGI0, NULL};
const Segment * const SG5083[] = {SGI21894, SGI21895, SGI21896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21897, SGI0, SGI0, SGI0, SGI21898, SGI21898, SGI0, SGI0, NULL};
const Segment * const SG5084[] = {SGI21899, SGI21900, SGI21901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21902, SGI0, SGI0, SGI0, SGI21903, SGI21903, SGI0, SGI0, NULL};
const Segment * const SG5085[] = {SGI21904, SGI21905, SGI21906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21907, SGI0, SGI0, SGI0, SGI21908, SGI21908, SGI0, SGI0, NULL};
const Segment * const SG5086[] = {SGI21909, SGI21910, SGI21911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21912, SGI0, SGI0, SGI0, SGI21913, SGI21913, SGI0, SGI0, NULL};
const Segment * const SG5087[] = {SGI21914, SGI21915, SGI21916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21917, SGI0, SGI0, SGI0, SGI21918, SGI21918, SGI0, SGI0, NULL};
const Segment * const SG5088[] = {SGI21919, SGI21920, SGI21921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21922, SGI0, SGI0, SGI0, SGI21923, SGI21923, SGI0, SGI0, NULL};
const Segment * const SG5089[] = {SGI21924, SGI21925, SGI21926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21927, SGI0, SGI0, SGI0, SGI21928, SGI21928, SGI0, SGI0, NULL};
const Segment * const SG5090[] = {SGI21929, SGI21930, SGI21931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21932, SGI0, SGI0, SGI0, SGI21933, SGI21933, SGI0, SGI0, NULL};
const Segment * const SG5091[] = {SGI21934, SGI21935, SGI21936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21937, SGI0, SGI0, SGI0, SGI21938, SGI21938, SGI0, SGI0, NULL};
const Segment * const SG5092[] = {SGI21939, SGI21940, SGI21941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21942, SGI0, SGI0, SGI0, SGI21943, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5093[] = {SGI21944, SGI21945, SGI21946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21947, SGI0, SGI0, SGI0, SGI21948, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5094[] = {SGI21949, SGI21950, SGI21951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21952, SGI0, SGI0, SGI0, SGI21953, SGI21953, SGI0, SGI0, NULL};
const Segment * const SG5095[] = {SGI21954, SGI21955, SGI21956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21957, SGI0, SGI0, SGI0, SGI21958, SGI21958, SGI0, SGI0, NULL};
const Segment * const SG5096[] = {SGI21959, SGI21960, SGI21961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21962, SGI0, SGI0, SGI0, SGI21963, SGI21963, SGI0, SGI0, NULL};
const Segment * const SG5097[] = {SGI21964, SGI21965, SGI21966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21967, SGI0, SGI0, SGI0, SGI21968, SGI21968, SGI0, SGI0, NULL};
const Segment * const SG5098[] = {SGI21969, SGI21970, SGI21971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21972, SGI0, SGI0, SGI0, SGI21973, SGI21973, SGI0, SGI0, NULL};
const Segment * const SG5099[] = {SGI21974, SGI21975, SGI21976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21977, SGI0, SGI0, SGI0, SGI21978, SGI21978, SGI0, SGI0, NULL};
const Segment * const SG5100[] = {SGI21979, SGI21980, SGI21981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21982, SGI0, SGI0, SGI0, SGI21983, SGI21983, SGI0, SGI0, NULL};
const Segment * const SG5101[] = {SGI21984, SGI21985, SGI21986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21987, SGI0, SGI0, SGI0, SGI21988, SGI21988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5102[] = {SGI21989, SGI0, SGI21990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5103[] = {SGI21991, SGI21992, SGI21993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21994, SGI0, SGI0, SGI0, SGI21995, SGI21995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5104[] = {SGI21996, SGI21997, SGI21998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI21999, SGI22000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5105[] = {SGI22001, SGI22002, SGI22004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22007, SGI0, SGI0, SGI0, SGI22009, SGI22009, SGI0, SGI0, NULL};
const Segment * const SG5106[] = {SGI22003, SGI22005, SGI22006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22008, SGI0, SGI0, SGI0, SGI22010, SGI22010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5107[] = {SGI22011, SGI22012, SGI22013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22014, SGI0, SGI0, SGI0, SGI22015, SGI22015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5108[] = {SGI22016, SGI22017, SGI22018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22019, SGI0, SGI0, SGI0, SGI22020, SGI22020, SGI0, SGI0, NULL};
const Segment * const SG5109[] = {SGI22021, SGI0, SGI22022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5110[] = {SGI22023, SGI22024, SGI22025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22026, SGI0, SGI0, SGI0, SGI22027, SGI22027, SGI0, SGI0, NULL};
const Segment * const SG5111[] = {SGI22028, SGI22029, SGI22030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22031, SGI22032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5112[] = {SGI22033, SGI0, SGI22034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22035, SGI0, SGI0, SGI0, SGI22036, SGI22036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5113[] = {SGI22037, SGI22038, SGI22039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22040, SGI0, SGI0, SGI0, SGI22041, SGI22041, SGI0, SGI0, NULL};
const Segment * const SG5114[] = {SGI22042, SGI22043, SGI22044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22045, SGI22046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5115[] = {SGI22047, SGI0, SGI22048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5116[] = {SGI22049, SGI22050, SGI22051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22052, SGI0, SGI0, SGI0, SGI22053, SGI22053, SGI0, SGI0, NULL};
const Segment * const SG5117[] = {SGI22054, SGI22055, SGI22056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22057, SGI0, SGI0, SGI0, SGI22058, SGI22058, SGI0, SGI0, NULL};
const Segment * const SG5118[] = {SGI22059, SGI22060, SGI22061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22062, SGI0, SGI0, SGI0, SGI22063, SGI22063, SGI0, SGI0, NULL};
const Segment * const SG5119[] = {SGI22064, SGI22065, SGI22066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22067, SGI0, SGI0, SGI0, SGI22068, SGI22068, SGI0, SGI0, NULL};
const Segment * const SG5120[] = {SGI22069, SGI22070, SGI22071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22072, SGI0, SGI0, SGI0, SGI22073, SGI22073, SGI0, SGI0, NULL};
const Segment * const SG5121[] = {SGI22074, SGI22075, SGI22076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22080, SGI0, SGI0, SGI0, SGI22082, SGI22082, SGI0, SGI0, NULL};
const Segment * const SG5122[] = {SGI22077, SGI22078, SGI22079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22081, SGI0, SGI0, SGI0, SGI22083, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5123[] = {SGI22084, SGI22085, SGI22086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22087, SGI0, SGI0, SGI0, SGI22088, SGI22088, SGI0, SGI0, NULL};
const Segment * const SG5124[] = {SGI22089, SGI22090, SGI22091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22092, SGI0, SGI0, SGI0, SGI22093, SGI22093, SGI0, SGI0, NULL};
const Segment * const SG5125[] = {SGI22094, SGI22095, SGI22096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22097, SGI0, SGI0, SGI0, SGI22098, SGI22098, SGI0, SGI0, NULL};
const Segment * const SG5126[] = {SGI22099, SGI22100, SGI22101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22102, SGI0, SGI0, SGI0, SGI22103, SGI22103, SGI0, SGI0, NULL};
const Segment * const SG5127[] = {SGI0, SGI0, SGI22104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22105, SGI0, SGI0, SGI0, SGI22106, SGI22106, SGI0, SGI0, NULL};
const Segment * const SG5128[] = {SGI22107, SGI22108, SGI22109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22110, SGI0, SGI0, SGI0, SGI22111, SGI22111, SGI0, SGI0, NULL};
const Segment * const SG5129[] = {SGI22112, SGI22113, SGI22114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22115, SGI0, SGI0, SGI0, SGI22116, SGI22116, SGI0, SGI0, NULL};
const Segment * const SG5130[] = {SGI22117, SGI22118, SGI22119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22120, SGI0, SGI0, SGI0, SGI22121, SGI22121, SGI0, SGI0, NULL};
const Segment * const SG5131[] = {SGI22122, SGI22123, SGI22124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22127, SGI0, SGI0, SGI0, SGI22129, SGI22129, SGI0, SGI0, NULL};
const Segment * const SG5132[] = {SGI22125, SGI22126, SGI22128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22130, SGI0, SGI0, SGI0, SGI22131, SGI22131, SGI0, SGI0, NULL};
const Segment * const SG5133[] = {SGI22132, SGI22133, SGI22134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22135, SGI0, SGI0, SGI0, SGI22136, SGI22136, SGI0, SGI0, NULL};
const Segment * const SG5134[] = {SGI22137, SGI22138, SGI22139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22140, SGI0, SGI0, SGI0, SGI22141, SGI22141, SGI0, SGI0, NULL};
const Segment * const SG5135[] = {SGI22142, SGI22143, SGI22144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22145, SGI0, SGI0, SGI0, SGI22146, SGI22146, SGI0, SGI0, NULL};
const Segment * const SG5136[] = {SGI22147, SGI22148, SGI22149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22150, SGI0, SGI0, SGI0, SGI22151, SGI22151, SGI0, SGI0, NULL};
const Segment * const SG5137[] = {SGI22152, SGI22153, SGI22154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22155, SGI0, SGI0, SGI0, SGI22156, SGI22156, SGI0, SGI0, NULL};
const Segment * const SG5138[] = {SGI22157, SGI22158, SGI22159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22160, SGI0, SGI0, SGI0, SGI22161, SGI22161, SGI0, SGI0, NULL};
const Segment * const SG5139[] = {SGI0, SGI0, SGI22162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22163, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5140[] = {SGI22164, SGI22165, SGI22166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22167, SGI0, SGI0, SGI0, SGI22168, SGI22168, SGI0, SGI0, NULL};
const Segment * const SG5141[] = {SGI22169, SGI22170, SGI22171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22172, SGI0, SGI0, SGI0, SGI22173, SGI22173, SGI0, SGI0, NULL};
const Segment * const SG5142[] = {SGI22174, SGI22175, SGI22176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22177, SGI0, SGI0, SGI0, SGI22178, SGI22178, SGI0, SGI0, NULL};
const Segment * const SG5143[] = {SGI0, SGI0, SGI22179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22180, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5144[] = {SGI22181, SGI22182, SGI22183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22184, SGI0, SGI0, SGI0, SGI22185, SGI22185, SGI0, SGI0, NULL};
const Segment * const SG5145[] = {SGI22186, SGI22187, SGI22188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22189, SGI0, SGI0, SGI0, SGI22190, SGI22190, SGI0, SGI0, NULL};
const Segment * const SG5146[] = {SGI22191, SGI22192, SGI22193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22194, SGI0, SGI0, SGI0, SGI22195, SGI22195, SGI0, SGI0, NULL};
const Segment * const SG5147[] = {SGI22196, SGI22197, SGI22198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22199, SGI0, SGI0, SGI0, SGI22200, SGI22200, SGI0, SGI0, NULL};
const Segment * const SG5148[] = {SGI22201, SGI22202, SGI22203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22204, SGI0, SGI0, SGI0, SGI22205, SGI22205, SGI0, SGI0, NULL};
const Segment * const SG5149[] = {SGI22206, SGI22207, SGI22208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22209, SGI0, SGI0, SGI0, SGI22210, SGI22210, SGI0, SGI0, NULL};
const Segment * const SG5150[] = {SGI22211, SGI22212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22213, SGI0, SGI0, SGI0, SGI22214, SGI22214, SGI0, SGI0, NULL};
const Segment * const SG5151[] = {SGI22215, SGI22216, SGI22217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22218, SGI0, SGI0, SGI0, SGI0, SGI22219, SGI0, SGI0, NULL};
const Segment * const SG5152[] = {SGI22220, SGI22221, SGI22222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22223, SGI0, SGI0, SGI0, SGI22224, SGI22224, SGI0, SGI0, NULL};
const Segment * const SG5153[] = {SGI22225, SGI22226, SGI22227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22228, SGI0, SGI0, SGI0, SGI22229, SGI22229, SGI0, SGI0, NULL};
const Segment * const SG5154[] = {SGI22230, SGI22231, SGI22232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22233, SGI0, SGI0, SGI0, SGI22234, SGI22234, SGI0, SGI0, NULL};
const Segment * const SG5155[] = {SGI22235, SGI22236, SGI22237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22238, SGI0, SGI0, SGI0, SGI22239, SGI22239, SGI0, SGI0, NULL};
const Segment * const SG5156[] = {SGI22240, SGI22241, SGI22242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22243, SGI0, SGI0, SGI0, SGI22244, SGI22244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5157[] = {SGI22245, SGI0, SGI22246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5158[] = {SGI22247, SGI22248, SGI22249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22250, SGI0, SGI0, SGI0, SGI0, SGI22251, SGI0, SGI0, NULL};
const Segment * const SG5159[] = {SGI22252, SGI22253, SGI22254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22255, SGI0, SGI0, SGI0, SGI22256, SGI22256, SGI0, SGI0, NULL};
const Segment * const SG5160[] = {SGI22257, SGI22258, SGI22259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22260, SGI0, SGI0, SGI0, SGI22261, SGI22261, SGI0, SGI0, NULL};
const Segment * const SG5161[] = {SGI0, SGI0, SGI22262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22263, SGI0, SGI0, SGI0, SGI22264, SGI22265, SGI0, SGI0, NULL};
const Segment * const SG5162[] = {SGI22266, SGI22267, SGI22268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22269, SGI0, SGI0, SGI0, SGI22270, SGI22270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5163[] = {SGI22271, SGI22272, SGI22273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22274, SGI0, SGI0, SGI0, SGI22275, SGI22275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5164[] = {SGI22276, SGI0, SGI22277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5165[] = {SGI22278, SGI22279, SGI22280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22281, SGI0, SGI0, SGI0, SGI22282, SGI22282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5166[] = {SGI22283, SGI22284, SGI22285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22286, SGI0, SGI0, SGI0, SGI22287, SGI22287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5167[] = {SGI22288, SGI22289, SGI22290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22291, SGI22292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5168[] = {SGI22293, SGI22294, SGI22295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22296, SGI0, SGI0, SGI0, SGI22297, SGI22297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5169[] = {SGI22298, SGI0, SGI22299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5170[] = {SGI22300, SGI22301, SGI22302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22303, SGI22303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5171[] = {SGI22304, SGI0, SGI22305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22306, SGI0, SGI0, SGI0, SGI22307, SGI22307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5172[] = {SGI22308, SGI0, SGI22309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5173[] = {SGI22310, SGI22311, SGI22312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22313, SGI0, SGI0, SGI0, SGI22314, SGI22314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5174[] = {SGI22315, SGI22316, SGI22317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22318, SGI0, SGI0, SGI0, SGI22319, SGI22319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5175[] = {SGI22320, SGI22321, SGI22322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22323, SGI0, SGI0, SGI0, SGI22324, SGI22324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5176[] = {SGI22325, SGI0, SGI22326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5177[] = {SGI22327, SGI22328, SGI22329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22330, SGI0, SGI0, SGI0, SGI22331, SGI22331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5178[] = {SGI22332, SGI22333, SGI22334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22335, SGI0, SGI0, SGI0, SGI22336, SGI22336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5179[] = {SGI22337, SGI22338, SGI22339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22340, SGI0, SGI0, SGI0, SGI22341, SGI22341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5180[] = {SGI22342, SGI22343, SGI22344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22345, SGI0, SGI0, SGI0, SGI22346, SGI22346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5181[] = {SGI22347, SGI0, SGI22348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5182[] = {SGI22349, SGI0, SGI22350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22351, SGI0, SGI0, SGI0, SGI22352, SGI22352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5183[] = {SGI22353, SGI22354, SGI22355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22356, SGI0, SGI0, SGI0, SGI22357, SGI22357, SGI0, SGI0, NULL};
const Segment * const SG5184[] = {SGI22358, SGI22359, SGI22360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22361, SGI0, SGI0, SGI0, SGI22362, SGI22362, SGI0, SGI0, NULL};
const Segment * const SG5185[] = {SGI22363, SGI22364, SGI22365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22366, SGI0, SGI0, SGI0, SGI22367, SGI22367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5186[] = {SGI22368, SGI22369, SGI22370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22371, SGI0, SGI0, SGI0, SGI22372, SGI22372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5187[] = {SGI22373, SGI22374, SGI22375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22376, SGI0, SGI0, SGI0, SGI22377, SGI22377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5188[] = {SGI22378, SGI0, SGI22379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5189[] = {SGI22380, SGI22381, SGI22382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22383, SGI0, SGI0, SGI0, SGI22384, SGI22384, SGI0, SGI0, NULL};
const Segment * const SG5190[] = {SGI22385, SGI22386, SGI22387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22388, SGI22389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5191[] = {SGI22390, SGI22391, SGI22392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22393, SGI22393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5192[] = {SGI22394, SGI0, SGI22395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22396, SGI0, SGI0, SGI0, SGI22397, SGI22397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5193[] = {SGI22398, SGI22399, SGI22400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22401, SGI0, SGI0, SGI0, SGI22402, SGI22402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5194[] = {SGI22403, SGI0, SGI22404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22405, SGI0, SGI0, SGI0, SGI22406, SGI22406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5195[] = {SGI22407, SGI22408, SGI22409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22410, SGI0, SGI0, SGI0, SGI22411, SGI22411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5196[] = {SGI22412, SGI22413, SGI22414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22415, SGI0, SGI0, SGI0, SGI22416, SGI22416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5197[] = {SGI22417, SGI22418, SGI22419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22420, SGI0, SGI0, SGI0, SGI22421, SGI22421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5198[] = {SGI22422, SGI22423, SGI22424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22425, SGI0, SGI0, SGI0, SGI22426, SGI22426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5199[] = {SGI22427, SGI22428, SGI22429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22430, SGI0, SGI0, SGI0, SGI22431, SGI22431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5200[] = {SGI22432, SGI0, SGI22433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5201[] = {SGI22434, SGI0, SGI22435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5202[] = {SGI22436, SGI0, SGI22437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5203[] = {SGI22438, SGI22439, SGI22440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22441, SGI0, SGI0, SGI0, SGI22442, SGI22443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5204[] = {SGI22444, SGI22445, SGI22446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22447, SGI0, SGI0, SGI0, SGI22448, SGI22448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5205[] = {SGI22449, SGI22450, SGI22451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22452, SGI0, SGI0, SGI0, SGI22453, SGI22453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5206[] = {SGI22454, SGI22455, SGI22456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22457, SGI22458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5207[] = {SGI22459, SGI22460, SGI22461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22462, SGI0, SGI0, SGI0, SGI22463, SGI22463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5208[] = {SGI22464, SGI22465, SGI22466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22467, SGI0, SGI0, SGI0, SGI22468, SGI22468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5209[] = {SGI22469, SGI22470, SGI22471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22472, SGI22472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5210[] = {SGI22473, SGI0, SGI22474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5211[] = {SGI22475, SGI0, SGI22476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22477, SGI0, SGI0, SGI0, SGI22478, SGI22478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5212[] = {SGI22479, SGI22480, SGI22481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22482, SGI22483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5213[] = {SGI22484, SGI0, SGI22485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22486, SGI0, SGI0, SGI0, SGI22487, SGI22487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5214[] = {SGI22488, SGI22489, SGI22490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22491, SGI0, SGI0, SGI0, SGI22492, SGI22492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5215[] = {SGI22493, SGI22494, SGI22495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22496, SGI0, SGI0, SGI0, SGI22497, SGI22497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5216[] = {SGI22498, SGI22499, SGI22500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22501, SGI0, SGI0, SGI0, SGI22502, SGI22502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5217[] = {SGI22503, SGI22504, SGI22505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22506, SGI0, SGI0, SGI0, SGI22507, SGI22507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5218[] = {SGI22508, SGI22509, SGI22510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22511, SGI0, SGI0, SGI0, SGI22512, SGI22512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5219[] = {SGI22513, SGI22514, SGI22515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22516, SGI0, SGI0, SGI0, SGI22517, SGI22517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5220[] = {SGI22518, SGI22519, SGI22520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22521, SGI0, SGI0, SGI0, SGI22522, SGI22522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5221[] = {SGI22523, SGI22524, SGI22525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22526, SGI0, SGI0, SGI0, SGI22527, SGI22527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5222[] = {SGI22528, SGI0, SGI22529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22530, SGI0, SGI0, SGI0, SGI22531, SGI22531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5223[] = {SGI22532, SGI22533, SGI22534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22535, SGI0, SGI0, SGI0, SGI22536, SGI22536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5224[] = {SGI22537, SGI22538, SGI22539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22540, SGI0, SGI0, SGI0, SGI22541, SGI22541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5225[] = {SGI22542, SGI22543, SGI22544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22545, SGI0, SGI0, SGI0, SGI22546, SGI22546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5226[] = {SGI22547, SGI22548, SGI22549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22550, SGI0, SGI0, SGI0, SGI22551, SGI22551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5227[] = {SGI22552, SGI22553, SGI22554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22555, SGI0, SGI0, SGI0, SGI22556, SGI22556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5228[] = {SGI22557, SGI22558, SGI22559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22560, SGI0, SGI0, SGI0, SGI22561, SGI22561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5229[] = {SGI22562, SGI0, SGI22563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5230[] = {SGI22564, SGI22565, SGI22566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22567, SGI0, SGI0, SGI0, SGI22568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5231[] = {SGI22569, SGI22570, SGI22571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22572, SGI22573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5232[] = {SGI22574, SGI22575, SGI22576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22577, SGI22577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5233[] = {SGI22578, SGI22579, SGI22580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22581, SGI0, SGI0, SGI0, SGI22582, SGI22582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5234[] = {SGI22583, SGI22584, SGI22585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22586, SGI0, SGI0, SGI0, SGI22587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5235[] = {SGI0, SGI22588, SGI22589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5236[] = {SGI22590, SGI0, SGI22591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22592, SGI0, SGI0, SGI0, SGI22593, SGI22593, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5237[] = {SGI22594, SGI22595, SGI22596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22597, SGI0, SGI0, SGI0, SGI22598, SGI22598, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5238[] = {SGI22599, SGI22600, SGI22601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22602, SGI0, SGI0, SGI0, SGI22603, SGI22603, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5239[] = {SGI22604, SGI22605, SGI22606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22607, SGI0, SGI0, SGI0, SGI22608, SGI22608, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5240[] = {SGI0, SGI22609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22610, SGI0, SGI0, SGI0, SGI22611, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5241[] = {SGI22612, SGI22613, SGI22614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22615, SGI0, SGI0, SGI0, SGI22616, SGI22616, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5242[] = {SGI22617, SGI22618, SGI22619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22620, SGI0, SGI0, SGI0, SGI22621, SGI22621, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5243[] = {SGI22622, SGI22623, SGI22624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22625, SGI0, SGI0, SGI0, SGI22626, SGI22626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5244[] = {SGI22627, SGI0, SGI22628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5245[] = {SGI22629, SGI22630, SGI22631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22632, SGI0, SGI0, SGI0, SGI22633, SGI22633, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5246[] = {SGI22634, SGI0, SGI22635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5247[] = {SGI22636, SGI22637, SGI22638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22639, SGI0, SGI0, SGI0, SGI22640, SGI22640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5248[] = {SGI22641, SGI22642, SGI22643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22644, SGI22645, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5249[] = {SGI22646, SGI22647, SGI22648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22649, SGI22650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5250[] = {SGI22651, SGI22652, SGI22653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22654, SGI22654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5251[] = {SGI22655, SGI0, SGI22656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22657, SGI0, SGI0, SGI0, SGI22658, SGI22658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5252[] = {SGI22659, SGI22660, SGI22661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22662, SGI0, SGI0, SGI0, SGI22663, SGI22663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5253[] = {SGI22664, SGI22665, SGI22666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22667, SGI0, SGI0, SGI0, SGI22668, SGI22668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5254[] = {SGI22669, SGI22670, SGI22671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22672, SGI0, SGI0, SGI0, SGI22673, SGI22673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5255[] = {SGI22674, SGI22675, SGI22676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22677, SGI0, SGI0, SGI0, SGI22678, SGI22678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5256[] = {SGI22679, SGI0, SGI22680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5257[] = {SGI22681, SGI22682, SGI22683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22684, SGI0, SGI0, SGI0, SGI22688, SGI22688, SGI0, SGI0, NULL};
const Segment * const SG5258[] = {SGI22685, SGI22686, SGI22687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22689, SGI0, SGI0, SGI0, SGI22690, SGI22690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5259[] = {SGI22691, SGI0, SGI22692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22693, SGI0, SGI0, SGI0, SGI22694, SGI22694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5260[] = {SGI22695, SGI22696, SGI22697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22698, SGI22699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5261[] = {SGI22700, SGI0, SGI22701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22702, SGI0, SGI0, SGI0, SGI22703, SGI22703, SGI0, SGI0, NULL};
const Segment * const SG5262[] = {SGI22704, SGI22705, SGI22706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22707, SGI0, SGI0, SGI0, SGI22708, SGI22708, SGI0, SGI0, NULL};
const Segment * const SG5263[] = {SGI22709, SGI0, SGI22710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5264[] = {SGI22711, SGI22712, SGI22713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22714, SGI0, SGI0, SGI0, SGI22715, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5265[] = {SGI22716, SGI22717, SGI22718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22719, SGI0, SGI0, SGI0, SGI22720, SGI22720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5266[] = {SGI22721, SGI22722, SGI22723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22724, SGI0, SGI0, SGI0, SGI22725, SGI22725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5267[] = {SGI22726, SGI22727, SGI22728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22729, SGI0, SGI0, SGI0, SGI22730, SGI22730, SGI0, SGI0, NULL};
const Segment * const SG5268[] = {SGI22731, SGI22732, SGI22733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22734, SGI0, SGI0, SGI0, SGI22735, SGI22735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5269[] = {SGI22736, SGI22737, SGI22738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22739, SGI0, SGI0, SGI0, SGI22740, SGI22740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5270[] = {SGI22741, SGI22742, SGI22743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22744, SGI0, SGI0, SGI0, SGI22745, SGI22745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5271[] = {SGI22746, SGI22747, SGI22748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22749, SGI0, SGI0, SGI0, SGI22752, SGI22752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5272[] = {SGI22750, SGI22751, SGI22753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22754, SGI0, SGI0, SGI0, SGI22755, SGI22755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5273[] = {SGI22756, SGI22757, SGI22758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22759, SGI0, SGI0, SGI0, SGI22760, SGI22760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5274[] = {SGI22761, SGI22762, SGI22763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22764, SGI0, SGI0, SGI0, SGI22765, SGI22765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5275[] = {SGI22766, SGI22767, SGI22768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22769, SGI0, SGI0, SGI0, SGI22770, SGI22770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5276[] = {SGI22771, SGI22772, SGI22773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22774, SGI0, SGI0, SGI0, SGI22775, SGI22775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5277[] = {SGI22776, SGI22777, SGI22778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22779, SGI0, SGI0, SGI0, SGI22780, SGI22780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5278[] = {SGI22781, SGI22782, SGI22783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22784, SGI0, SGI0, SGI0, SGI22787, SGI22787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5279[] = {SGI22785, SGI22786, SGI22788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22789, SGI0, SGI0, SGI0, SGI22790, SGI22790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5280[] = {SGI22791, SGI22792, SGI22793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22794, SGI0, SGI0, SGI0, SGI22795, SGI22795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5281[] = {SGI22796, SGI22797, SGI22798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22799, SGI0, SGI0, SGI0, SGI22802, SGI22802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5282[] = {SGI22800, SGI22801, SGI22803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22804, SGI0, SGI0, SGI0, SGI22805, SGI22805, SGI0, SGI0, NULL};
const Segment * const SG5283[] = {SGI22806, SGI22807, SGI22808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22809, SGI0, SGI0, SGI0, SGI22810, SGI22810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5284[] = {SGI22811, SGI22812, SGI22813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22814, SGI0, SGI0, SGI0, SGI22816, SGI22816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5285[] = {SGI22815, SGI22817, SGI22818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22819, SGI0, SGI0, SGI0, SGI22820, SGI22820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5286[] = {SGI22821, SGI22822, SGI22823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22824, SGI0, SGI0, SGI0, SGI22825, SGI22825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5287[] = {SGI22826, SGI22827, SGI22829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22832, SGI0, SGI0, SGI0, SGI22834, SGI22834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5288[] = {SGI22828, SGI22830, SGI22831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22833, SGI0, SGI0, SGI0, SGI22835, SGI22835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5289[] = {SGI22836, SGI22837, SGI22838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22839, SGI0, SGI0, SGI0, SGI22840, SGI22840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5290[] = {SGI22841, SGI22842, SGI22843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22844, SGI0, SGI0, SGI0, SGI22845, SGI22845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5291[] = {SGI22846, SGI0, SGI22847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5292[] = {SGI22848, SGI22849, SGI22850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22851, SGI0, SGI0, SGI0, SGI22853, SGI22853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5293[] = {SGI22852, SGI22854, SGI22855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22856, SGI0, SGI0, SGI0, SGI22857, SGI22857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5294[] = {SGI22858, SGI22859, SGI22860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22861, SGI0, SGI0, SGI0, SGI22862, SGI22862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5295[] = {SGI22863, SGI0, SGI22864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5296[] = {SGI22865, SGI22866, SGI22867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22868, SGI0, SGI0, SGI0, SGI22869, SGI22869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5297[] = {SGI22870, SGI22871, SGI22872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22876, SGI0, SGI0, SGI0, SGI22877, SGI22877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5298[] = {SGI22873, SGI22874, SGI22875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22878, SGI0, SGI0, SGI0, SGI22879, SGI22879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5299[] = {SGI22880, SGI22881, SGI22882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22883, SGI0, SGI0, SGI0, SGI22884, SGI22884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5300[] = {SGI22885, SGI22886, SGI22887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22888, SGI0, SGI0, SGI0, SGI22889, SGI22889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5301[] = {SGI22890, SGI22891, SGI22892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22893, SGI0, SGI0, SGI0, SGI22895, SGI22895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5302[] = {SGI22894, SGI22896, SGI22897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22898, SGI0, SGI0, SGI0, SGI22899, SGI22899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5303[] = {SGI22900, SGI22901, SGI22902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22903, SGI0, SGI0, SGI0, SGI22904, SGI22904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5304[] = {SGI22905, SGI22906, SGI22907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22908, SGI0, SGI0, SGI0, SGI22909, SGI22909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5305[] = {SGI22910, SGI22911, SGI22912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22913, SGI0, SGI0, SGI0, SGI22914, SGI22914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5306[] = {SGI22915, SGI22916, SGI22917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22918, SGI22919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5307[] = {SGI22920, SGI22921, SGI22922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22923, SGI22924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5308[] = {SGI22925, SGI22926, SGI22927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22931, SGI0, SGI0, SGI0, SGI22932, SGI22932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5309[] = {SGI22928, SGI22929, SGI22930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22933, SGI0, SGI0, SGI0, SGI22934, SGI22934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5310[] = {SGI22935, SGI22936, SGI22937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22941, SGI22942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5311[] = {SGI22938, SGI22939, SGI22940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22943, SGI0, SGI0, SGI0, SGI22944, SGI22944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5312[] = {SGI22945, SGI22946, SGI22947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22948, SGI0, SGI0, SGI0, SGI22949, SGI22949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5313[] = {SGI22950, SGI22951, SGI22952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22953, SGI0, SGI0, SGI0, SGI22954, SGI22954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5314[] = {SGI22955, SGI22956, SGI22957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22958, SGI0, SGI0, SGI0, SGI22959, SGI22959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5315[] = {SGI22960, SGI22961, SGI22962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22963, SGI0, SGI0, SGI0, SGI22964, SGI22964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5316[] = {SGI22965, SGI22966, SGI22967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22968, SGI0, SGI0, SGI0, SGI22971, SGI22971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5317[] = {SGI22969, SGI22970, SGI22972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22973, SGI0, SGI0, SGI0, SGI22974, SGI22974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5318[] = {SGI22975, SGI22976, SGI22977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22978, SGI0, SGI0, SGI0, SGI22979, SGI22979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5319[] = {SGI22980, SGI22981, SGI22982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22983, SGI0, SGI0, SGI0, SGI22984, SGI22984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5320[] = {SGI22986, SGI22988, SGI22990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22992, SGI0, SGI0, SGI0, SGI22993, SGI22993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5321[] = {SGI22994, SGI22995, SGI22996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22997, SGI22998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5322[] = {SGI22999, SGI23000, SGI23001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23002, SGI23002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5323[] = {SGI23003, SGI23004, SGI23005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23006, SGI0, SGI0, SGI0, SGI23007, SGI23007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5324[] = {SGI23008, SGI0, SGI23009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5325[] = {SGI23010, SGI23011, SGI23012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23013, SGI0, SGI0, SGI0, SGI23014, SGI23014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5326[] = {SGI23015, SGI0, SGI23016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23017, SGI0, SGI0, SGI0, SGI23018, SGI23018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5327[] = {SGI23019, SGI0, SGI23020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23021, SGI0, SGI0, SGI0, SGI23022, SGI23022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5328[] = {SGI23023, SGI23024, SGI23025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23026, SGI0, SGI0, SGI0, SGI23027, SGI23027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5329[] = {SGI23028, SGI23029, SGI23030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23031, SGI23031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5330[] = {SGI23032, SGI23033, SGI23034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23035, SGI23035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5331[] = {SGI23036, SGI23037, SGI23038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23039, SGI23039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5332[] = {SGI23040, SGI23041, SGI23042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23043, SGI0, SGI0, SGI0, SGI23044, SGI23044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5333[] = {SGI23045, SGI23046, SGI23047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23048, SGI23048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5334[] = {SGI23049, SGI23050, SGI23051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23052, SGI0, SGI0, SGI0, SGI23053, SGI23053, SGI0, SGI0, NULL};
const Segment * const SG5335[] = {SGI0, SGI23054, SGI23055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5336[] = {SGI22985, SGI22987, SGI22989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI22991, SGI0, SGI0, SGI0, SGI23056, SGI23056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5337[] = {SGI23057, SGI23058, SGI23059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23060, SGI0, SGI0, SGI0, SGI23061, SGI23061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5338[] = {SGI23062, SGI23063, SGI23064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23065, SGI0, SGI0, SGI0, SGI23066, SGI23066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5339[] = {SGI23070, SGI0, SGI23071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5340[] = {SGI23067, SGI23068, SGI23069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23072, SGI0, SGI0, SGI0, SGI23073, SGI23073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5341[] = {SGI23074, SGI23075, SGI23076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23080, SGI0, SGI0, SGI0, SGI23081, SGI23081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5342[] = {SGI23077, SGI23078, SGI23079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23082, SGI0, SGI0, SGI0, SGI23083, SGI23083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5343[] = {SGI23084, SGI23085, SGI23086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23087, SGI0, SGI0, SGI0, SGI23088, SGI23088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5344[] = {SGI23089, SGI23090, SGI23091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23092, SGI0, SGI0, SGI0, SGI23093, SGI23093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5345[] = {SGI23094, SGI23095, SGI23096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23097, SGI23098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5346[] = {SGI23099, SGI23100, SGI23101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23102, SGI0, SGI0, SGI0, SGI23103, SGI23103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5347[] = {SGI23104, SGI23105, SGI23106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23107, SGI0, SGI0, SGI0, SGI23108, SGI23108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5348[] = {SGI23109, SGI23110, SGI23111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23112, SGI0, SGI0, SGI0, SGI23113, SGI23113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5349[] = {SGI23114, SGI23115, SGI23116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23117, SGI0, SGI0, SGI0, SGI23118, SGI23118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5350[] = {SGI23119, SGI23120, SGI23121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23122, SGI0, SGI0, SGI0, SGI23123, SGI23123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5351[] = {SGI23124, SGI23125, SGI23126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23127, SGI0, SGI0, SGI0, SGI23129, SGI23129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5352[] = {SGI23128, SGI0, SGI23130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23131, SGI0, SGI0, SGI0, SGI23132, SGI23132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5353[] = {SGI23133, SGI23134, SGI23135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23136, SGI0, SGI0, SGI0, SGI23137, SGI23137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5354[] = {SGI23138, SGI23139, SGI23140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23141, SGI23141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5355[] = {SGI23142, SGI23143, SGI23144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23145, SGI0, SGI0, SGI0, SGI23146, SGI23146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5356[] = {SGI23147, SGI23148, SGI23149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23150, SGI0, SGI0, SGI0, SGI23151, SGI23151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5357[] = {SGI23152, SGI23153, SGI23154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23155, SGI0, SGI0, SGI0, SGI23156, SGI23156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5358[] = {SGI23157, SGI23158, SGI23159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23160, SGI0, SGI0, SGI0, SGI23161, SGI23161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5359[] = {SGI23162, SGI23163, SGI23164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23165, SGI0, SGI0, SGI0, SGI23166, SGI23166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5360[] = {SGI23167, SGI23168, SGI23169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23170, SGI0, SGI0, SGI0, SGI23171, SGI23171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5361[] = {SGI0, SGI0, SGI23172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23173, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5362[] = {SGI23174, SGI23175, SGI23176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23177, SGI0, SGI0, SGI0, SGI23178, SGI23178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5363[] = {SGI23179, SGI23180, SGI23181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23182, SGI0, SGI0, SGI0, SGI23183, SGI23183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5364[] = {SGI0, SGI23184, SGI23185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23186, SGI0, SGI0, SGI0, SGI23187, SGI23188, SGI0, SGI0, NULL};
const Segment * const SG5365[] = {SGI23189, SGI23190, SGI23191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23192, SGI0, SGI0, SGI0, SGI23193, SGI23193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5366[] = {SGI23194, SGI0, SGI23195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5367[] = {SGI23196, SGI23197, SGI23198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23199, SGI0, SGI0, SGI0, SGI23200, SGI23200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5368[] = {SGI23201, SGI23202, SGI23203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23204, SGI0, SGI0, SGI0, SGI23205, SGI23205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5369[] = {SGI23207, SGI23209, SGI23211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23213, SGI0, SGI0, SGI0, SGI23214, SGI23214, SGI0, SGI0, NULL};
const Segment * const SG5370[] = {SGI23206, SGI23208, SGI23210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23212, SGI0, SGI0, SGI0, SGI23215, SGI23215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5371[] = {SGI23216, SGI23217, SGI23218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23219, SGI0, SGI0, SGI0, SGI23220, SGI23220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5372[] = {SGI23221, SGI23222, SGI23223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23224, SGI0, SGI0, SGI0, SGI23226, SGI23226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5373[] = {SGI23225, SGI23227, SGI23228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23229, SGI0, SGI0, SGI0, SGI23230, SGI23230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5374[] = {SGI23231, SGI23232, SGI23233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23234, SGI0, SGI0, SGI0, SGI23237, SGI23237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5375[] = {SGI23235, SGI23236, SGI23238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23239, SGI0, SGI0, SGI0, SGI23240, SGI23240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5376[] = {SGI23241, SGI23242, SGI23243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23246, SGI0, SGI0, SGI0, SGI23248, SGI23248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5377[] = {SGI23244, SGI23245, SGI23247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23249, SGI0, SGI0, SGI0, SGI23250, SGI23250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5378[] = {SGI23251, SGI0, SGI23252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5379[] = {SGI23253, SGI23254, SGI23255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23259, SGI0, SGI0, SGI0, SGI23260, SGI23260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5380[] = {SGI23256, SGI23257, SGI23258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23261, SGI0, SGI0, SGI0, SGI23262, SGI23262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5381[] = {SGI23263, SGI23264, SGI23265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23269, SGI0, SGI0, SGI0, SGI23270, SGI23270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5382[] = {SGI23266, SGI23267, SGI23268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23271, SGI0, SGI0, SGI0, SGI23272, SGI23272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5383[] = {SGI23273, SGI23274, SGI23275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23276, SGI0, SGI0, SGI0, SGI23277, SGI23277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5384[] = {SGI23278, SGI23279, SGI23280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23281, SGI23282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5385[] = {SGI23283, SGI23284, SGI23285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23286, SGI0, SGI0, SGI0, SGI23287, SGI23287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5386[] = {SGI23288, SGI23289, SGI23290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23291, SGI0, SGI0, SGI0, SGI23292, SGI23292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5387[] = {SGI23293, SGI23294, SGI23295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23296, SGI0, SGI0, SGI0, SGI23297, SGI23297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5388[] = {SGI23298, SGI23299, SGI23300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23301, SGI0, SGI0, SGI0, SGI23302, SGI23302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5389[] = {SGI23303, SGI23304, SGI23305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23306, SGI0, SGI0, SGI0, SGI23307, SGI23307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5390[] = {SGI23308, SGI23309, SGI23310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23314, SGI0, SGI0, SGI0, SGI23315, SGI23315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5391[] = {SGI23311, SGI23312, SGI23313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23316, SGI0, SGI0, SGI0, SGI23317, SGI23317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5392[] = {SGI23318, SGI23319, SGI23320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23323, SGI23323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5393[] = {SGI23321, SGI23322, SGI23324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23325, SGI0, SGI0, SGI0, SGI23326, SGI23326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5394[] = {SGI23327, SGI0, SGI23328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23329, SGI0, SGI0, SGI0, SGI23330, SGI23330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5395[] = {SGI23331, SGI23332, SGI23333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23334, SGI0, SGI0, SGI0, SGI23335, SGI23335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5396[] = {SGI23336, SGI0, SGI23337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5397[] = {SGI23338, SGI23339, SGI23340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23341, SGI0, SGI0, SGI0, SGI23342, SGI23342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5398[] = {SGI23343, SGI23344, SGI23345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23346, SGI0, SGI0, SGI0, SGI23347, SGI23347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5399[] = {SGI23348, SGI23349, SGI23350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23351, SGI0, SGI0, SGI0, SGI23352, SGI23352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5400[] = {SGI23353, SGI23354, SGI23355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23356, SGI0, SGI0, SGI0, SGI23357, SGI23357, SGI0, SGI0, NULL};
const Segment * const SG5401[] = {SGI23358, SGI23359, SGI23360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23361, SGI0, SGI0, SGI0, SGI23362, SGI23362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5402[] = {SGI0, SGI23363, SGI23364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5403[] = {SGI23365, SGI0, SGI23366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23367, SGI0, SGI0, SGI0, SGI23368, SGI23368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5404[] = {SGI23369, SGI23370, SGI23371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23372, SGI0, SGI0, SGI0, SGI23373, SGI23373, SGI0, SGI0, NULL};
const Segment * const SG5405[] = {SGI23374, SGI23375, SGI23376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23377, SGI0, SGI0, SGI0, SGI23378, SGI23378, SGI0, SGI0, NULL};
const Segment * const SG5406[] = {SGI23379, SGI0, SGI23380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5407[] = {SGI23381, SGI23382, SGI23383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23384, SGI0, SGI0, SGI0, SGI23385, SGI23385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5408[] = {SGI23386, SGI0, SGI23387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5409[] = {SGI23388, SGI23389, SGI23390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23391, SGI0, SGI0, SGI0, SGI23392, SGI23392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5410[] = {SGI23393, SGI23394, SGI23395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23396, SGI0, SGI0, SGI0, SGI23397, SGI23397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5411[] = {SGI23398, SGI23399, SGI23400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23401, SGI0, SGI0, SGI0, SGI23402, SGI23402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5412[] = {SGI23403, SGI23404, SGI23405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23406, SGI0, SGI0, SGI0, SGI23407, SGI23407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5413[] = {SGI23408, SGI23409, SGI23410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23411, SGI0, SGI0, SGI0, SGI23412, SGI23412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5414[] = {SGI23413, SGI23414, SGI23415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23416, SGI0, SGI0, SGI0, SGI23417, SGI23417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5415[] = {SGI23418, SGI23419, SGI23420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23421, SGI0, SGI0, SGI0, SGI23422, SGI23422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5416[] = {SGI23423, SGI23424, SGI23425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23426, SGI0, SGI0, SGI0, SGI23427, SGI23427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5417[] = {SGI23428, SGI23429, SGI23430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23431, SGI0, SGI0, SGI0, SGI23432, SGI23432, SGI0, SGI0, NULL};
const Segment * const SG5418[] = {SGI23433, SGI23434, SGI23435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23436, SGI0, SGI0, SGI0, SGI23437, SGI23437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5419[] = {SGI23438, SGI23439, SGI23440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23441, SGI0, SGI0, SGI0, SGI23442, SGI23442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5420[] = {SGI23443, SGI23444, SGI23445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23446, SGI0, SGI0, SGI0, SGI23447, SGI23447, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5421[] = {SGI23448, SGI23449, SGI23450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23451, SGI0, SGI0, SGI0, SGI23452, SGI23452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5422[] = {SGI23453, SGI23454, SGI23455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23456, SGI0, SGI0, SGI0, SGI23457, SGI23457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5423[] = {SGI23458, SGI23459, SGI23460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23461, SGI0, SGI0, SGI0, SGI23462, SGI23462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5424[] = {SGI23463, SGI23464, SGI23465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23466, SGI0, SGI0, SGI0, SGI23467, SGI23467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5425[] = {SGI23468, SGI23469, SGI23470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23471, SGI0, SGI0, SGI0, SGI23472, SGI23472, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5426[] = {SGI23473, SGI23474, SGI23475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23476, SGI0, SGI0, SGI0, SGI23477, SGI23477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5427[] = {SGI23478, SGI23479, SGI23480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23481, SGI0, SGI0, SGI0, SGI23482, SGI23482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5428[] = {SGI0, SGI23483, SGI23484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23485, SGI0, SGI0, SGI0, SGI0, SGI23486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5429[] = {SGI23487, SGI23488, SGI23489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23490, SGI23491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5430[] = {SGI23492, SGI23493, SGI23494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23495, SGI0, SGI0, SGI0, SGI23496, SGI23496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5431[] = {SGI23497, SGI0, SGI23498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5432[] = {SGI23499, SGI23500, SGI23501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23502, SGI0, SGI0, SGI0, SGI23503, SGI23503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5433[] = {SGI23504, SGI23505, SGI23506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23507, SGI0, SGI0, SGI0, SGI23508, SGI23508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5434[] = {SGI23509, SGI23510, SGI23511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23512, SGI0, SGI0, SGI0, SGI23513, SGI23513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5435[] = {SGI23514, SGI23515, SGI23516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23517, SGI0, SGI0, SGI0, SGI23518, SGI23518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5436[] = {SGI23519, SGI23520, SGI23521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23522, SGI0, SGI0, SGI0, SGI23523, SGI23523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5437[] = {SGI23524, SGI23525, SGI23526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23527, SGI0, SGI0, SGI0, SGI23528, SGI23528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5438[] = {SGI23529, SGI23530, SGI23531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23532, SGI0, SGI0, SGI0, SGI23533, SGI23533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5439[] = {SGI23534, SGI23535, SGI23536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23537, SGI0, SGI0, SGI0, SGI23538, SGI23538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5440[] = {SGI23539, SGI23540, SGI23541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23542, SGI0, SGI0, SGI0, SGI23543, SGI23543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5441[] = {SGI23544, SGI23545, SGI23546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23547, SGI0, SGI0, SGI0, SGI23548, SGI23548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5442[] = {SGI23549, SGI23550, SGI23551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23552, SGI0, SGI0, SGI0, SGI23553, SGI23553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5443[] = {SGI0, SGI23554, SGI23556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23559, SGI0, SGI0, SGI0, SGI23560, SGI23560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5444[] = {SGI23555, SGI23557, SGI23558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23561, SGI0, SGI0, SGI0, SGI23562, SGI23562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5445[] = {SGI23563, SGI23564, SGI23565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23566, SGI0, SGI0, SGI0, SGI23567, SGI23567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5446[] = {SGI23568, SGI23569, SGI23570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23571, SGI0, SGI0, SGI0, SGI23572, SGI23572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5447[] = {SGI23573, SGI23574, SGI23575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23576, SGI0, SGI0, SGI0, SGI23577, SGI23577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5448[] = {SGI23578, SGI23579, SGI23580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23581, SGI0, SGI0, SGI0, SGI23582, SGI23582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5449[] = {SGI23583, SGI23584, SGI23585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23586, SGI0, SGI0, SGI0, SGI23587, SGI23587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5450[] = {SGI23588, SGI23589, SGI23590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23591, SGI0, SGI0, SGI0, SGI23592, SGI23592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5451[] = {SGI23593, SGI23594, SGI23595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23596, SGI0, SGI0, SGI0, SGI23597, SGI23597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5452[] = {SGI23598, SGI23599, SGI23600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23601, SGI0, SGI0, SGI0, SGI23602, SGI23602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5453[] = {SGI23603, SGI23604, SGI23605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23606, SGI0, SGI0, SGI0, SGI23607, SGI23607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5454[] = {SGI23608, SGI23609, SGI23610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23611, SGI0, SGI0, SGI0, SGI23612, SGI23612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5455[] = {SGI23613, SGI23614, SGI23615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23616, SGI0, SGI0, SGI0, SGI23617, SGI23617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5456[] = {SGI0, SGI23618, SGI23619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23620, SGI0, SGI0, SGI0, SGI23621, SGI23621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5457[] = {SGI23622, SGI23623, SGI23624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23625, SGI0, SGI0, SGI0, SGI23626, SGI23626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5458[] = {SGI23627, SGI23628, SGI23629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23630, SGI0, SGI0, SGI0, SGI23631, SGI23631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5459[] = {SGI23632, SGI0, SGI23633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5460[] = {SGI23634, SGI23635, SGI23636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23637, SGI0, SGI0, SGI0, SGI23638, SGI23638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5461[] = {SGI23639, SGI23640, SGI23641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23642, SGI0, SGI0, SGI0, SGI23643, SGI23643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5462[] = {SGI23644, SGI23645, SGI23646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23647, SGI0, SGI0, SGI0, SGI23648, SGI23648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5463[] = {SGI23649, SGI23650, SGI23651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23652, SGI0, SGI0, SGI0, SGI23653, SGI23653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5464[] = {SGI23654, SGI23655, SGI23656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23657, SGI0, SGI0, SGI0, SGI23658, SGI23658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5465[] = {SGI23659, SGI23660, SGI23661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23662, SGI0, SGI0, SGI0, SGI23663, SGI23663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5466[] = {SGI23664, SGI23665, SGI23666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23667, SGI0, SGI0, SGI0, SGI23668, SGI23668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5467[] = {SGI23669, SGI23670, SGI23671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23672, SGI0, SGI0, SGI0, SGI23673, SGI23673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5468[] = {SGI23674, SGI23675, SGI23676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23677, SGI0, SGI0, SGI0, SGI23678, SGI23678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5469[] = {SGI23679, SGI23680, SGI23681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23682, SGI0, SGI0, SGI0, SGI23683, SGI23683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5470[] = {SGI23684, SGI23685, SGI23686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23687, SGI0, SGI0, SGI0, SGI23688, SGI23688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5471[] = {SGI23689, SGI23690, SGI23691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23693, SGI0, SGI0, SGI0, SGI23696, SGI23696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5472[] = {SGI23692, SGI23694, SGI23695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23697, SGI0, SGI0, SGI0, SGI23698, SGI23698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5473[] = {SGI23699, SGI23700, SGI23701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23702, SGI0, SGI0, SGI0, SGI23703, SGI23703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5474[] = {SGI23704, SGI23705, SGI23706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23710, SGI0, SGI0, SGI0, SGI23712, SGI23712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5475[] = {SGI23707, SGI23708, SGI23709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23711, SGI0, SGI0, SGI0, SGI23713, SGI23713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5476[] = {SGI23714, SGI23715, SGI23716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23717, SGI0, SGI0, SGI0, SGI23718, SGI23718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5477[] = {SGI23719, SGI23720, SGI23721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23722, SGI0, SGI0, SGI0, SGI23723, SGI23723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5478[] = {SGI23724, SGI23725, SGI23726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23727, SGI0, SGI0, SGI0, SGI23728, SGI23728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5479[] = {SGI23729, SGI23730, SGI23731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23732, SGI0, SGI0, SGI0, SGI23733, SGI23733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5480[] = {SGI23734, SGI23735, SGI23736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23737, SGI0, SGI0, SGI0, SGI23738, SGI23738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5481[] = {SGI23739, SGI23740, SGI23741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23742, SGI0, SGI0, SGI0, SGI23743, SGI23743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5482[] = {SGI23744, SGI23745, SGI23746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23747, SGI0, SGI0, SGI0, SGI23748, SGI23748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5483[] = {SGI23749, SGI23750, SGI23751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23752, SGI0, SGI0, SGI0, SGI23753, SGI23753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5484[] = {SGI23754, SGI23755, SGI23756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23757, SGI0, SGI0, SGI0, SGI23758, SGI23758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5485[] = {SGI23759, SGI23760, SGI23761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23762, SGI0, SGI0, SGI0, SGI23763, SGI23763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5486[] = {SGI23764, SGI23765, SGI23766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23767, SGI0, SGI0, SGI0, SGI23768, SGI23768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5487[] = {SGI23769, SGI0, SGI23770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5488[] = {SGI23771, SGI23772, SGI23773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23774, SGI0, SGI0, SGI0, SGI23775, SGI23775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5489[] = {SGI23776, SGI23777, SGI23778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23779, SGI0, SGI0, SGI0, SGI23780, SGI23780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5490[] = {SGI23781, SGI23782, SGI23783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23784, SGI0, SGI0, SGI0, SGI23785, SGI23785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5491[] = {SGI23786, SGI23787, SGI23788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23789, SGI0, SGI0, SGI0, SGI23790, SGI23790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5492[] = {SGI23791, SGI23792, SGI23793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23794, SGI0, SGI0, SGI0, SGI23795, SGI23795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5493[] = {SGI23796, SGI23797, SGI23798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23799, SGI0, SGI0, SGI0, SGI23800, SGI23800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5494[] = {SGI23801, SGI23802, SGI23803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23804, SGI0, SGI0, SGI0, SGI23805, SGI23805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5495[] = {SGI23806, SGI23807, SGI23808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23809, SGI0, SGI0, SGI0, SGI23810, SGI23810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5496[] = {SGI0, SGI23811, SGI23812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23813, SGI0, SGI0, SGI0, SGI0, SGI23814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5497[] = {SGI23815, SGI23816, SGI23817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23818, SGI23819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5498[] = {SGI23820, SGI23821, SGI23822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23823, SGI0, SGI0, SGI0, SGI23824, SGI23824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5499[] = {SGI23825, SGI23826, SGI23827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23828, SGI23829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5500[] = {SGI23830, SGI23831, SGI23832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23833, SGI0, SGI0, SGI0, SGI23834, SGI23834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5501[] = {SGI23835, SGI23836, SGI23837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23838, SGI0, SGI0, SGI0, SGI23839, SGI23839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5502[] = {SGI23840, SGI23841, SGI23842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23843, SGI0, SGI0, SGI0, SGI23844, SGI23844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5503[] = {SGI23845, SGI0, SGI23846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5504[] = {SGI23847, SGI23848, SGI23849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23850, SGI0, SGI0, SGI0, SGI23851, SGI23851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5505[] = {SGI23852, SGI23853, SGI23854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23855, SGI0, SGI0, SGI0, SGI23856, SGI23856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5506[] = {SGI23857, SGI23858, SGI23859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23860, SGI0, SGI0, SGI0, SGI23861, SGI23861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5507[] = {SGI23862, SGI23863, SGI23864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23865, SGI0, SGI0, SGI0, SGI23866, SGI23866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5508[] = {SGI23867, SGI23868, SGI23869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23870, SGI0, SGI0, SGI0, SGI23871, SGI23871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5509[] = {SGI23872, SGI23873, SGI23874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23875, SGI0, SGI0, SGI0, SGI23876, SGI23876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5510[] = {SGI23877, SGI23878, SGI23879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23880, SGI0, SGI0, SGI0, SGI23881, SGI23881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5511[] = {SGI23882, SGI23883, SGI23884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23885, SGI0, SGI0, SGI0, SGI23886, SGI23886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5512[] = {SGI23887, SGI23888, SGI23889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23890, SGI0, SGI0, SGI0, SGI23891, SGI23891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5513[] = {SGI23892, SGI23893, SGI23894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23895, SGI0, SGI0, SGI0, SGI23896, SGI23896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5514[] = {SGI0, SGI23897, SGI23898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23899, SGI0, SGI0, SGI0, SGI23900, SGI23900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5515[] = {SGI23901, SGI0, SGI23902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5516[] = {SGI23903, SGI23904, SGI23905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23906, SGI0, SGI0, SGI0, SGI23907, SGI23907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5517[] = {SGI23908, SGI23909, SGI23910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23911, SGI0, SGI0, SGI0, SGI23912, SGI23912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5518[] = {SGI23913, SGI23914, SGI23915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23916, SGI0, SGI0, SGI0, SGI23917, SGI23917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5519[] = {SGI0, SGI23918, SGI23919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23920, SGI0, SGI0, SGI0, SGI0, SGI23921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5520[] = {SGI23922, SGI23923, SGI23924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23925, SGI0, SGI0, SGI0, SGI23926, SGI23926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5521[] = {SGI23927, SGI23928, SGI23929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23930, SGI0, SGI0, SGI0, SGI23931, SGI23931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5522[] = {SGI23932, SGI23933, SGI23934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23935, SGI0, SGI0, SGI0, SGI23936, SGI23936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5523[] = {SGI23937, SGI23938, SGI23939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23940, SGI23941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5524[] = {SGI23942, SGI23943, SGI23944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23945, SGI0, SGI0, SGI0, SGI23946, SGI23946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5525[] = {SGI23947, SGI23948, SGI23949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23950, SGI0, SGI0, SGI0, SGI23951, SGI23951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5526[] = {SGI23952, SGI23953, SGI23954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23955, SGI0, SGI0, SGI0, SGI23956, SGI23956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5527[] = {SGI23957, SGI23958, SGI23959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23960, SGI0, SGI0, SGI0, SGI23961, SGI23961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5528[] = {SGI23962, SGI23963, SGI23964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23965, SGI0, SGI0, SGI0, SGI23966, SGI23966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5529[] = {SGI23967, SGI23968, SGI23969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23970, SGI0, SGI0, SGI0, SGI23971, SGI23971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5530[] = {SGI23972, SGI23973, SGI23974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23975, SGI0, SGI0, SGI0, SGI23976, SGI23976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5531[] = {SGI23977, SGI23978, SGI23979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23980, SGI0, SGI0, SGI0, SGI23981, SGI23981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5532[] = {SGI23982, SGI23983, SGI23984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23985, SGI0, SGI0, SGI0, SGI23986, SGI23986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5533[] = {SGI23987, SGI23988, SGI23989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23990, SGI0, SGI0, SGI0, SGI23991, SGI23991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5534[] = {SGI23992, SGI23993, SGI23994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI23995, SGI0, SGI0, SGI0, SGI23996, SGI23996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5535[] = {SGI23997, SGI23998, SGI23999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24000, SGI0, SGI0, SGI0, SGI24001, SGI24001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5536[] = {SGI24002, SGI24003, SGI24004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24005, SGI0, SGI0, SGI0, SGI24006, SGI24006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5537[] = {SGI24007, SGI24008, SGI24009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24010, SGI0, SGI0, SGI0, SGI24011, SGI24011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5538[] = {SGI24012, SGI24013, SGI24014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24015, SGI0, SGI0, SGI0, SGI24016, SGI24016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5539[] = {SGI24017, SGI24018, SGI24019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24020, SGI0, SGI0, SGI0, SGI24021, SGI24021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5540[] = {SGI24022, SGI24023, SGI24024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24025, SGI0, SGI0, SGI0, SGI24026, SGI24026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5541[] = {SGI24027, SGI24028, SGI24029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24030, SGI24030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5542[] = {SGI24031, SGI24032, SGI24033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24034, SGI0, SGI0, SGI0, SGI24035, SGI24035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5543[] = {SGI24036, SGI24037, SGI24038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24039, SGI0, SGI0, SGI0, SGI24040, SGI24040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5544[] = {SGI24041, SGI24042, SGI24043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24044, SGI0, SGI0, SGI0, SGI24045, SGI24045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5545[] = {SGI24046, SGI24047, SGI24048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24049, SGI0, SGI0, SGI0, SGI24050, SGI24050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5546[] = {SGI24051, SGI24052, SGI24053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24054, SGI0, SGI0, SGI0, SGI24055, SGI24055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5547[] = {SGI24056, SGI24057, SGI24058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24059, SGI0, SGI0, SGI0, SGI24060, SGI24060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5548[] = {SGI24061, SGI24062, SGI24063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24064, SGI0, SGI0, SGI0, SGI24065, SGI24065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5549[] = {SGI24066, SGI24067, SGI24068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24069, SGI0, SGI0, SGI0, SGI24070, SGI24070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5550[] = {SGI24071, SGI24072, SGI24073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24074, SGI0, SGI0, SGI0, SGI24075, SGI24075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5551[] = {SGI24076, SGI24077, SGI24078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24079, SGI0, SGI0, SGI0, SGI24080, SGI24080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5552[] = {SGI24081, SGI24082, SGI24083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24084, SGI24085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5553[] = {SGI24086, SGI24087, SGI24088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24090, SGI0, SGI0, SGI0, SGI24093, SGI24093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5554[] = {SGI24089, SGI24091, SGI24092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24094, SGI0, SGI0, SGI0, SGI24095, SGI24095, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5555[] = {SGI24096, SGI24097, SGI24098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24099, SGI0, SGI0, SGI0, SGI24100, SGI24100, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5556[] = {SGI24101, SGI24103, SGI24105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24107, SGI0, SGI0, SGI0, SGI24109, SGI24109, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5557[] = {SGI24102, SGI24104, SGI24106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24108, SGI0, SGI0, SGI0, SGI24110, SGI24110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5558[] = {SGI24111, SGI24112, SGI24113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24114, SGI0, SGI0, SGI0, SGI24115, SGI24115, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5559[] = {SGI24116, SGI24117, SGI24118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24119, SGI0, SGI0, SGI0, SGI24120, SGI24120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5560[] = {SGI24121, SGI24122, SGI24123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24124, SGI0, SGI0, SGI0, SGI24125, SGI24125, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5561[] = {SGI24126, SGI24127, SGI24128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24129, SGI0, SGI0, SGI0, SGI24130, SGI24130, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5562[] = {SGI24131, SGI24132, SGI24133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24135, SGI0, SGI0, SGI0, SGI24138, SGI24138, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5563[] = {SGI24134, SGI24136, SGI24137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24139, SGI0, SGI0, SGI0, SGI24140, SGI24140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5564[] = {SGI24141, SGI24142, SGI24143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24144, SGI0, SGI0, SGI0, SGI24145, SGI24145, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5565[] = {SGI24146, SGI24147, SGI24148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24149, SGI24149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5566[] = {SGI24150, SGI24151, SGI24152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24156, SGI0, SGI0, SGI0, SGI24158, SGI24158, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5567[] = {SGI24153, SGI24154, SGI24155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24157, SGI0, SGI0, SGI0, SGI24159, SGI24159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5568[] = {SGI24160, SGI24161, SGI24162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24167, SGI0, SGI0, SGI0, SGI24169, SGI24169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5569[] = {SGI24163, SGI0, SGI24165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24170, SGI0, SGI0, SGI0, SGI24172, SGI24172, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5570[] = {SGI24164, SGI24166, SGI24168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24171, SGI0, SGI0, SGI0, SGI24173, SGI24173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5571[] = {SGI24174, SGI24175, SGI24176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24178, SGI0, SGI0, SGI0, SGI24181, SGI24181, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5572[] = {SGI24177, SGI24179, SGI24180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24182, SGI0, SGI0, SGI0, SGI24183, SGI24183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5573[] = {SGI24184, SGI24185, SGI24186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24187, SGI0, SGI0, SGI0, SGI24188, SGI24188, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5574[] = {SGI24189, SGI24190, SGI24192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24195, SGI0, SGI0, SGI0, SGI24198, SGI24198, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5575[] = {SGI24191, SGI24193, SGI24194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24196, SGI0, SGI0, SGI0, SGI24197, SGI24197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5576[] = {SGI24199, SGI24200, SGI24201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24202, SGI0, SGI0, SGI0, SGI24203, SGI24203, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5577[] = {SGI24204, SGI0, SGI24205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24206, SGI0, SGI0, SGI0, SGI24209, SGI24209, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5578[] = {SGI24207, SGI24208, SGI24210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24211, SGI0, SGI0, SGI0, SGI24212, SGI24212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5579[] = {SGI24213, SGI0, SGI24214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24215, SGI0, SGI0, SGI0, SGI24216, SGI24216, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5580[] = {SGI24217, SGI24218, SGI24220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24223, SGI0, SGI0, SGI0, SGI24225, SGI24225, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5581[] = {SGI24219, SGI24221, SGI24222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24224, SGI0, SGI0, SGI0, SGI24226, SGI24226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5582[] = {SGI24227, SGI24228, SGI24229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24230, SGI0, SGI0, SGI0, SGI24231, SGI24231, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5583[] = {SGI24232, SGI0, SGI24233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5584[] = {SGI24234, SGI0, SGI24235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5585[] = {SGI24236, SGI24237, SGI24238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24239, SGI24240, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5586[] = {SGI24241, SGI0, SGI24242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24243, SGI0, SGI0, SGI0, SGI24244, SGI24244, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5587[] = {SGI24245, SGI24246, SGI24247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24248, SGI24248, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5588[] = {SGI24249, SGI0, SGI24250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5589[] = {SGI24251, SGI0, SGI24252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24253, SGI0, SGI0, SGI0, SGI24254, SGI24254, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5590[] = {SGI24255, SGI24256, SGI24257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24258, SGI24258, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5591[] = {SGI24259, SGI24260, SGI24261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24262, SGI24262, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5592[] = {SGI24263, SGI24264, SGI24265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24266, SGI0, SGI0, SGI0, SGI24267, SGI24267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5593[] = {SGI24268, SGI24269, SGI24270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24271, SGI0, SGI0, SGI0, SGI24272, SGI24272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5594[] = {SGI24273, SGI24274, SGI24275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24276, SGI0, SGI0, SGI0, SGI24277, SGI24277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5595[] = {SGI24278, SGI24279, SGI24280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24281, SGI0, SGI0, SGI0, SGI24282, SGI24282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5596[] = {SGI24288, SGI24289, SGI24290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24291, SGI0, SGI0, SGI0, SGI24292, SGI24292, SGI0, SGI0, NULL};
const Segment * const SG5597[] = {SGI24293, SGI24294, SGI24295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24296, SGI0, SGI0, SGI0, SGI24297, SGI24297, SGI0, SGI0, NULL};
const Segment * const SG5598[] = {SGI24298, SGI24299, SGI24300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24301, SGI0, SGI0, SGI0, SGI24302, SGI24302, SGI0, SGI0, NULL};
const Segment * const SG5599[] = {SGI24303, SGI24304, SGI24305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24307, SGI0, SGI0, SGI0, SGI24310, SGI24310, SGI0, SGI0, NULL};
const Segment * const SG5600[] = {SGI24306, SGI24308, SGI24309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24311, SGI0, SGI0, SGI0, SGI24312, SGI24312, SGI0, SGI0, NULL};
const Segment * const SG5601[] = {SGI24313, SGI24314, SGI24315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24316, SGI0, SGI0, SGI0, SGI24317, SGI24317, SGI0, SGI0, NULL};
const Segment * const SG5602[] = {SGI24318, SGI24319, SGI24320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24321, SGI0, SGI0, SGI0, SGI24322, SGI24322, SGI0, SGI0, NULL};
const Segment * const SG5603[] = {SGI24323, SGI24324, SGI24325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24326, SGI0, SGI0, SGI0, SGI24327, SGI24327, SGI0, SGI0, NULL};
const Segment * const SG5604[] = {SGI24328, SGI24329, SGI24330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24331, SGI0, SGI0, SGI0, SGI24332, SGI24332, SGI0, SGI0, NULL};
const Segment * const SG5605[] = {SGI24333, SGI24334, SGI24335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24336, SGI0, SGI0, SGI0, SGI24337, SGI24337, SGI0, SGI0, NULL};
const Segment * const SG5606[] = {SGI24338, SGI24339, SGI24340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24341, SGI0, SGI0, SGI0, SGI24342, SGI24342, SGI0, SGI0, NULL};
const Segment * const SG5607[] = {SGI24343, SGI24344, SGI24345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24346, SGI0, SGI0, SGI0, SGI24347, SGI24347, SGI0, SGI0, NULL};
const Segment * const SG5608[] = {SGI24348, SGI24349, SGI24350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24351, SGI0, SGI0, SGI0, SGI24352, SGI24352, SGI0, SGI0, NULL};
const Segment * const SG5609[] = {SGI24353, SGI24354, SGI24355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24356, SGI0, SGI0, SGI0, SGI24357, SGI24357, SGI0, SGI0, NULL};
const Segment * const SG5610[] = {SGI24358, SGI24359, SGI24360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24361, SGI0, SGI0, SGI0, SGI24362, SGI24362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5611[] = {SGI24363, SGI24364, SGI24365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24366, SGI0, SGI0, SGI0, SGI24369, SGI24369, SGI0, SGI0, NULL};
const Segment * const SG5612[] = {SGI24367, SGI24368, SGI24370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24371, SGI0, SGI0, SGI0, SGI24372, SGI24372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5613[] = {SGI24373, SGI24374, SGI24375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24376, SGI0, SGI0, SGI0, SGI24377, SGI24377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5614[] = {SGI24378, SGI24379, SGI24380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24381, SGI0, SGI0, SGI0, SGI24382, SGI24382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5615[] = {SGI24383, SGI24384, SGI24385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24386, SGI0, SGI0, SGI0, SGI24387, SGI24387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5616[] = {SGI24388, SGI24389, SGI24390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24391, SGI0, SGI0, SGI0, SGI24392, SGI24392, SGI0, SGI0, NULL};
const Segment * const SG5617[] = {SGI24283, SGI24284, SGI24285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24286, SGI0, SGI0, SGI0, SGI24287, SGI24287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5618[] = {SGI24394, SGI24395, SGI24396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24397, SGI24397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5619[] = {SGI24398, SGI24399, SGI24400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24401, SGI24401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5620[] = {SGI24402, SGI0, SGI24403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5621[] = {SGI24404, SGI24405, SGI24406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24407, SGI0, SGI0, SGI0, SGI24408, SGI24408, SGI0, SGI0, NULL};
const Segment * const SG5622[] = {SGI0, SGI24409, SGI24410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5623[] = {SGI24411, SGI24412, SGI24413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24414, SGI0, SGI0, SGI0, SGI24415, SGI24415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5624[] = {SGI24416, SGI24417, SGI24418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24419, SGI0, SGI0, SGI0, SGI24420, SGI24420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5625[] = {SGI24421, SGI24422, SGI24423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24424, SGI0, SGI0, SGI0, SGI24425, SGI24425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5626[] = {SGI24393, SGI24426, SGI24427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24428, SGI0, SGI0, SGI0, SGI24429, SGI24429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5627[] = {SGI24430, SGI0, SGI24431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5628[] = {SGI24432, SGI24433, SGI24434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24435, SGI0, SGI0, SGI0, SGI24436, SGI24436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5629[] = {SGI24437, SGI24438, SGI24439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24440, SGI0, SGI0, SGI0, SGI24441, SGI24441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5630[] = {SGI24442, SGI24443, SGI24444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24445, SGI0, SGI0, SGI0, SGI24446, SGI24446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5631[] = {SGI24447, SGI24448, SGI24449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24450, SGI0, SGI0, SGI0, SGI24451, SGI24451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5632[] = {SGI24452, SGI24453, SGI24454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24455, SGI0, SGI0, SGI0, SGI24456, SGI24456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5633[] = {SGI0, SGI24457, SGI24458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24459, SGI0, SGI0, SGI0, SGI24460, SGI24460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5634[] = {SGI24461, SGI24462, SGI24463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24464, SGI0, SGI0, SGI0, SGI24465, SGI24465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5635[] = {SGI24466, SGI24467, SGI24468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24469, SGI0, SGI0, SGI0, SGI24470, SGI24470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5636[] = {SGI24471, SGI24472, SGI24473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24474, SGI0, SGI0, SGI0, SGI24475, SGI24475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5637[] = {SGI24476, SGI24477, SGI24478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24479, SGI0, SGI0, SGI0, SGI24480, SGI24480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5638[] = {SGI0, SGI24481, SGI24482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24483, SGI0, SGI0, SGI0, SGI24484, SGI24484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5639[] = {SGI24485, SGI24486, SGI24487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24488, SGI0, SGI0, SGI0, SGI24489, SGI24489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5640[] = {SGI24490, SGI24491, SGI24492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24493, SGI0, SGI0, SGI0, SGI24494, SGI24494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5641[] = {SGI24495, SGI24496, SGI24497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24498, SGI0, SGI0, SGI0, SGI24499, SGI24499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5642[] = {SGI24500, SGI24501, SGI24502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24503, SGI0, SGI0, SGI0, SGI24506, SGI24506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5643[] = {SGI24504, SGI24505, SGI24507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24508, SGI0, SGI0, SGI0, SGI24509, SGI24509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5644[] = {SGI24510, SGI24511, SGI24512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24513, SGI0, SGI0, SGI0, SGI24514, SGI24514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5645[] = {SGI24515, SGI24516, SGI24517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24518, SGI0, SGI0, SGI0, SGI24519, SGI24519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5646[] = {SGI24520, SGI24521, SGI24522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24523, SGI0, SGI0, SGI0, SGI24524, SGI24524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5647[] = {SGI24525, SGI24526, SGI24527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24528, SGI0, SGI0, SGI0, SGI24530, SGI24530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5648[] = {SGI24529, SGI24531, SGI24532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24533, SGI0, SGI0, SGI0, SGI24534, SGI24534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5649[] = {SGI24535, SGI24536, SGI24537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24538, SGI0, SGI0, SGI0, SGI24539, SGI24539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5650[] = {SGI24540, SGI24541, SGI24542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24543, SGI0, SGI0, SGI0, SGI24544, SGI24544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5651[] = {SGI24545, SGI24546, SGI24547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24548, SGI24549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5652[] = {SGI24550, SGI24551, SGI24552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24556, SGI0, SGI0, SGI0, SGI24557, SGI24557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5653[] = {SGI24553, SGI24554, SGI24555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24558, SGI0, SGI0, SGI0, SGI24560, SGI24560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5654[] = {SGI24559, SGI24561, SGI24562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24563, SGI0, SGI0, SGI0, SGI24564, SGI24564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5655[] = {SGI24565, SGI24566, SGI24567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24569, SGI0, SGI0, SGI0, SGI24572, SGI24572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5656[] = {SGI24568, SGI24570, SGI24571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24573, SGI0, SGI0, SGI0, SGI24574, SGI24574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5657[] = {SGI0, SGI24575, SGI24576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24577, SGI0, SGI0, SGI0, SGI24578, SGI24578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5658[] = {SGI24579, SGI24580, SGI24581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24582, SGI0, SGI0, SGI0, SGI24583, SGI24583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5659[] = {SGI24584, SGI24585, SGI24586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24587, SGI0, SGI0, SGI0, SGI24588, SGI24588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5660[] = {SGI24589, SGI24590, SGI24591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24592, SGI0, SGI0, SGI0, SGI24593, SGI24593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5661[] = {SGI24594, SGI24595, SGI24596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24599, SGI0, SGI0, SGI0, SGI24601, SGI24601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5662[] = {SGI24597, SGI24598, SGI24600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24602, SGI0, SGI0, SGI0, SGI24603, SGI24603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5663[] = {SGI0, SGI24604, SGI24605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24606, SGI0, SGI0, SGI0, SGI24607, SGI24607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5664[] = {SGI24608, SGI24609, SGI24610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24611, SGI0, SGI0, SGI0, SGI24612, SGI24612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5665[] = {SGI24613, SGI24614, SGI24615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24616, SGI24616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5666[] = {SGI24617, SGI24618, SGI24619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24620, SGI0, SGI0, SGI0, SGI24623, SGI24623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5667[] = {SGI24621, SGI24622, SGI24624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24625, SGI0, SGI0, SGI0, SGI24626, SGI24626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5668[] = {SGI0, SGI24627, SGI24628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24629, SGI0, SGI0, SGI0, SGI24630, SGI24630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5669[] = {SGI24631, SGI24632, SGI24633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24634, SGI0, SGI0, SGI0, SGI24635, SGI24635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5670[] = {SGI24636, SGI24637, SGI24638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24641, SGI0, SGI0, SGI0, SGI24643, SGI24643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5671[] = {SGI24639, SGI24640, SGI24642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24644, SGI24644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5672[] = {SGI24645, SGI24646, SGI24647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24648, SGI0, SGI0, SGI0, SGI24649, SGI24649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5673[] = {SGI24650, SGI24651, SGI24652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24653, SGI0, SGI0, SGI0, SGI24654, SGI24654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5674[] = {SGI24655, SGI24656, SGI24657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24658, SGI0, SGI0, SGI0, SGI24659, SGI24659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5675[] = {SGI24660, SGI24661, SGI24662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24666, SGI0, SGI0, SGI0, SGI24667, SGI24667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5676[] = {SGI24663, SGI24664, SGI24665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24668, SGI0, SGI0, SGI0, SGI24669, SGI24669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5677[] = {SGI24670, SGI24671, SGI24672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24673, SGI0, SGI0, SGI0, SGI24674, SGI24674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5678[] = {SGI24675, SGI24676, SGI24677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24678, SGI0, SGI0, SGI0, SGI24679, SGI24679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5679[] = {SGI24680, SGI24681, SGI24682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24683, SGI0, SGI0, SGI0, SGI24684, SGI24684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5680[] = {SGI24685, SGI24686, SGI24687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24688, SGI0, SGI0, SGI0, SGI24689, SGI24689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5681[] = {SGI24690, SGI24691, SGI24692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24693, SGI0, SGI0, SGI0, SGI24696, SGI24696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5682[] = {SGI24694, SGI24695, SGI24697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24698, SGI0, SGI0, SGI0, SGI24699, SGI24699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5683[] = {SGI24700, SGI24701, SGI24702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24703, SGI0, SGI0, SGI0, SGI24704, SGI24704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5684[] = {SGI24705, SGI24706, SGI24707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24708, SGI0, SGI0, SGI0, SGI24709, SGI24709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5685[] = {SGI24710, SGI24711, SGI24712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24713, SGI0, SGI0, SGI0, SGI24714, SGI24714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5686[] = {SGI24715, SGI24716, SGI24717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24718, SGI0, SGI0, SGI0, SGI24719, SGI24719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5687[] = {SGI24720, SGI24721, SGI24722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24723, SGI0, SGI0, SGI0, SGI24724, SGI24724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5688[] = {SGI24725, SGI24726, SGI24727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24728, SGI0, SGI0, SGI0, SGI24729, SGI24729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5689[] = {SGI24730, SGI24731, SGI24732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24733, SGI0, SGI0, SGI0, SGI24734, SGI24734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5690[] = {SGI24735, SGI24736, SGI24737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24738, SGI0, SGI0, SGI0, SGI24739, SGI24739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5691[] = {SGI24740, SGI24741, SGI24742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24743, SGI0, SGI0, SGI0, SGI24744, SGI24744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5692[] = {SGI24745, SGI24746, SGI24747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24748, SGI0, SGI0, SGI0, SGI24749, SGI24749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5693[] = {SGI24750, SGI24751, SGI24752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24753, SGI0, SGI0, SGI0, SGI24754, SGI24754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5694[] = {SGI24755, SGI24756, SGI24757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24758, SGI0, SGI0, SGI0, SGI24759, SGI24759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5695[] = {SGI24760, SGI24761, SGI24762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24763, SGI24764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5696[] = {SGI0, SGI24765, SGI24766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24767, SGI0, SGI0, SGI0, SGI24768, SGI24768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5697[] = {SGI24769, SGI24770, SGI24771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24772, SGI0, SGI0, SGI0, SGI24773, SGI24773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5698[] = {SGI24774, SGI24775, SGI24776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24777, SGI0, SGI0, SGI0, SGI24778, SGI24778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5699[] = {SGI24779, SGI24780, SGI24781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24782, SGI0, SGI0, SGI0, SGI24783, SGI24783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5700[] = {SGI24784, SGI24785, SGI24786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24787, SGI0, SGI0, SGI0, SGI24788, SGI24788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5701[] = {SGI24789, SGI24790, SGI24791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24792, SGI0, SGI0, SGI0, SGI24793, SGI24793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5702[] = {SGI24794, SGI24795, SGI24796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24797, SGI0, SGI0, SGI0, SGI24798, SGI24798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5703[] = {SGI0, SGI0, SGI24799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5704[] = {SGI0, SGI24800, SGI24801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24802, SGI0, SGI0, SGI0, SGI24803, SGI24803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5705[] = {SGI24804, SGI24805, SGI24806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24807, SGI0, SGI0, SGI0, SGI24808, SGI24808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5706[] = {SGI24809, SGI24810, SGI24811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24812, SGI0, SGI0, SGI0, SGI24813, SGI24813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5707[] = {SGI24814, SGI24815, SGI24816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24817, SGI0, SGI0, SGI0, SGI24818, SGI24818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5708[] = {SGI24819, SGI24820, SGI24821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24822, SGI0, SGI0, SGI0, SGI24823, SGI24823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5709[] = {SGI24824, SGI24825, SGI24826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24827, SGI0, SGI0, SGI0, SGI24828, SGI24828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5710[] = {SGI24829, SGI24830, SGI24831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24832, SGI0, SGI0, SGI0, SGI24833, SGI24833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5711[] = {SGI24834, SGI24835, SGI24836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24837, SGI0, SGI0, SGI0, SGI24838, SGI24838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5712[] = {SGI24839, SGI24840, SGI24841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24842, SGI24842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5713[] = {SGI24843, SGI24844, SGI24845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24846, SGI24847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5714[] = {SGI24848, SGI24849, SGI24850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24851, SGI0, SGI0, SGI0, SGI24852, SGI24852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5715[] = {SGI24853, SGI24854, SGI24855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24856, SGI0, SGI0, SGI0, SGI24857, SGI24857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5716[] = {SGI24858, SGI24859, SGI24860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24861, SGI0, SGI0, SGI0, SGI24862, SGI24862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5717[] = {SGI24863, SGI24864, SGI24865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24866, SGI0, SGI0, SGI0, SGI24867, SGI24867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5718[] = {SGI24868, SGI24869, SGI24870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24871, SGI0, SGI0, SGI0, SGI24872, SGI24872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5719[] = {SGI24873, SGI24874, SGI24875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24876, SGI0, SGI0, SGI0, SGI24877, SGI24877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5720[] = {SGI24878, SGI24879, SGI24880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24884, SGI0, SGI0, SGI0, SGI24885, SGI24885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5721[] = {SGI24881, SGI24882, SGI24883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24886, SGI24886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5722[] = {SGI24887, SGI24888, SGI24889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24890, SGI0, SGI0, SGI0, SGI24891, SGI24891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5723[] = {SGI24892, SGI24893, SGI24894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24895, SGI0, SGI0, SGI0, SGI24896, SGI24896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5724[] = {SGI24897, SGI24898, SGI24899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24900, SGI0, SGI0, SGI0, SGI24901, SGI24901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5725[] = {SGI24902, SGI24903, SGI24904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24905, SGI0, SGI0, SGI0, SGI24906, SGI24906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5726[] = {SGI24907, SGI24908, SGI24909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24910, SGI0, SGI0, SGI0, SGI24911, SGI24911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5727[] = {SGI0, SGI24912, SGI24913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24914, SGI0, SGI0, SGI0, SGI24915, SGI24915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5728[] = {SGI24916, SGI24917, SGI24918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24919, SGI0, SGI0, SGI0, SGI24920, SGI24920, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5729[] = {SGI24921, SGI24922, SGI24923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24924, SGI0, SGI0, SGI0, SGI24925, SGI24925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5730[] = {SGI24926, SGI24927, SGI24928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24929, SGI0, SGI0, SGI0, SGI24930, SGI24930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5731[] = {SGI24931, SGI24932, SGI24933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24934, SGI0, SGI0, SGI0, SGI24935, SGI24935, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5732[] = {SGI24936, SGI24937, SGI24938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24939, SGI0, SGI0, SGI0, SGI24940, SGI24940, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5733[] = {SGI24942, SGI24944, SGI24946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24948, SGI0, SGI0, SGI0, SGI24950, SGI24950, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5734[] = {SGI24941, SGI24943, SGI24945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24947, SGI0, SGI0, SGI0, SGI24949, SGI24949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5735[] = {SGI24951, SGI24952, SGI24953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24954, SGI0, SGI0, SGI0, SGI24955, SGI24955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5736[] = {SGI24956, SGI24957, SGI24958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24959, SGI0, SGI0, SGI0, SGI24960, SGI24960, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5737[] = {SGI24961, SGI0, SGI24962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24963, SGI0, SGI0, SGI0, SGI24964, SGI24964, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5738[] = {SGI24965, SGI24966, SGI24967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24968, SGI0, SGI0, SGI0, SGI24969, SGI24969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5739[] = {SGI24970, SGI24971, SGI24972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24973, SGI0, SGI0, SGI0, SGI24977, SGI24977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5740[] = {SGI24974, SGI24975, SGI24976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24978, SGI0, SGI0, SGI0, SGI24979, SGI24979, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5741[] = {SGI24980, SGI24981, SGI24983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24986, SGI0, SGI0, SGI0, SGI24988, SGI24988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5742[] = {SGI24982, SGI24984, SGI24985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24987, SGI0, SGI0, SGI0, SGI24989, SGI24989, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5743[] = {SGI24990, SGI0, SGI24991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5744[] = {SGI24992, SGI24993, SGI24994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI24995, SGI0, SGI0, SGI0, SGI24996, SGI24996, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5745[] = {SGI24997, SGI24998, SGI24999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25000, SGI0, SGI0, SGI0, SGI25001, SGI25001, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5746[] = {SGI25002, SGI25004, SGI25006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25008, SGI0, SGI0, SGI0, SGI25010, SGI25010, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5747[] = {SGI25003, SGI25005, SGI25007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25009, SGI0, SGI0, SGI0, SGI25011, SGI25011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5748[] = {SGI25012, SGI25013, SGI25014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25020, SGI0, SGI0, SGI0, SGI25022, SGI25022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5749[] = {SGI25015, SGI25016, SGI25018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25023, SGI0, SGI0, SGI0, SGI25025, SGI25025, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5750[] = {SGI25017, SGI25019, SGI25021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25024, SGI0, SGI0, SGI0, SGI25026, SGI25026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5751[] = {SGI25027, SGI25028, SGI25029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25030, SGI0, SGI0, SGI0, SGI25031, SGI25031, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5752[] = {SGI25032, SGI25033, SGI25034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25035, SGI0, SGI0, SGI0, SGI25036, SGI25036, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5753[] = {SGI25037, SGI25038, SGI25039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25040, SGI0, SGI0, SGI0, SGI25041, SGI25041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5754[] = {SGI25042, SGI25043, SGI25044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25045, SGI0, SGI0, SGI0, SGI25046, SGI25046, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5755[] = {SGI25047, SGI25048, SGI25049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25050, SGI0, SGI0, SGI0, SGI25051, SGI25051, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5756[] = {SGI25052, SGI25053, SGI25054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25055, SGI0, SGI0, SGI0, SGI25056, SGI25056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5757[] = {SGI25057, SGI25058, SGI25059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25060, SGI0, SGI0, SGI0, SGI25061, SGI25061, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5758[] = {SGI0, SGI25062, SGI25063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25064, SGI0, SGI0, SGI0, SGI25066, SGI25066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5759[] = {SGI25065, SGI25067, SGI25068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25069, SGI0, SGI0, SGI0, SGI25070, SGI25070, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5760[] = {SGI25071, SGI25072, SGI25073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25074, SGI0, SGI0, SGI0, SGI25075, SGI25075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5761[] = {SGI25076, SGI25077, SGI25078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25082, SGI0, SGI0, SGI0, SGI25084, SGI25084, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5762[] = {SGI25079, SGI25080, SGI25081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25083, SGI0, SGI0, SGI0, SGI25085, SGI25085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5763[] = {SGI25087, SGI25089, SGI25091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25092, SGI0, SGI0, SGI0, SGI25094, SGI25094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5764[] = {SGI25086, SGI25088, SGI25090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25093, SGI0, SGI0, SGI0, SGI25095, SGI25095, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5765[] = {SGI25096, SGI25097, SGI25098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25099, SGI0, SGI0, SGI0, SGI25103, SGI25103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5766[] = {SGI25104, SGI0, SGI25105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5767[] = {SGI25100, SGI25101, SGI25102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25106, SGI0, SGI0, SGI0, SGI25107, SGI25107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5768[] = {SGI25108, SGI25109, SGI25110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25112, SGI0, SGI0, SGI0, SGI25115, SGI25115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5769[] = {SGI25111, SGI25113, SGI25114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25116, SGI25117, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5770[] = {SGI0, SGI25118, SGI25119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25120, SGI0, SGI0, SGI0, SGI25121, SGI25121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5771[] = {SGI25122, SGI25123, SGI25124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25125, SGI25126, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5772[] = {SGI25127, SGI25128, SGI25129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25130, SGI25131, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5773[] = {SGI25132, SGI25133, SGI25134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25135, SGI0, SGI0, SGI0, SGI25136, SGI25136, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5774[] = {SGI25137, SGI25138, SGI25139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25142, SGI0, SGI0, SGI0, SGI25143, SGI25143, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5775[] = {SGI0, SGI25140, SGI25141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5776[] = {SGI25144, SGI25145, SGI25146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25148, SGI0, SGI0, SGI0, SGI25153, SGI25153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5777[] = {SGI25147, SGI25149, SGI25152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25155, SGI0, SGI0, SGI0, SGI25157, SGI25157, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5778[] = {SGI0, SGI25150, SGI25151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25154, SGI0, SGI0, SGI0, SGI25156, SGI25156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5779[] = {SGI25158, SGI25160, SGI25162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25165, SGI25166, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5780[] = {SGI25159, SGI25161, SGI25163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25164, SGI0, SGI0, SGI0, SGI25167, SGI25167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5781[] = {SGI25168, SGI25169, SGI25170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25171, SGI25171, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5782[] = {SGI25172, SGI25173, SGI25174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25175, SGI0, SGI0, SGI0, SGI25176, SGI25176, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5783[] = {SGI25177, SGI25178, SGI25179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25180, SGI25182, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5784[] = {SGI25181, SGI25183, SGI25184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25185, SGI0, SGI0, SGI0, SGI25186, SGI25186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5785[] = {SGI25187, SGI25188, SGI25189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25190, SGI25191, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5786[] = {SGI25192, SGI25193, SGI25194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25195, SGI0, SGI0, SGI0, SGI25196, SGI25196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5787[] = {SGI25197, SGI25198, SGI25199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25200, SGI0, SGI0, SGI0, SGI25201, SGI25201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5788[] = {SGI25202, SGI25203, SGI25204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25205, SGI25205, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5789[] = {SGI25206, SGI25207, SGI25208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25209, SGI0, SGI0, SGI0, SGI25210, SGI25210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5790[] = {SGI25211, SGI25212, SGI25213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25214, SGI25214, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5791[] = {SGI25215, SGI25216, SGI25217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25218, SGI0, SGI0, SGI0, SGI25219, SGI25219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5792[] = {SGI25220, SGI25221, SGI25223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25227, SGI25227, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5793[] = {SGI25222, SGI25224, SGI25225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25226, SGI0, SGI0, SGI0, SGI25228, SGI25228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5794[] = {SGI25229, SGI25230, SGI25231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25232, SGI25232, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5795[] = {SGI25233, SGI25234, SGI25235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25236, SGI0, SGI0, SGI0, SGI25237, SGI25237, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5796[] = {SGI25238, SGI25239, SGI25240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25241, SGI25241, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5797[] = {SGI25242, SGI25243, SGI25244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25245, SGI0, SGI0, SGI0, SGI25246, SGI25246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5798[] = {SGI25247, SGI25248, SGI25249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25250, SGI0, SGI0, SGI0, SGI25251, SGI25251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5799[] = {SGI25252, SGI25253, SGI25254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25255, SGI0, SGI0, SGI0, SGI25256, SGI25256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5800[] = {SGI25257, SGI25258, SGI25259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25260, SGI0, SGI0, SGI0, SGI25261, SGI25261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5801[] = {SGI25262, SGI25263, SGI25264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25265, SGI0, SGI0, SGI0, SGI25266, SGI25266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5802[] = {SGI25267, SGI25268, SGI25269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25270, SGI0, SGI0, SGI0, SGI25271, SGI25271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5803[] = {SGI25272, SGI25273, SGI25274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25275, SGI0, SGI0, SGI0, SGI25276, SGI25276, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5804[] = {SGI25277, SGI25278, SGI25279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25280, SGI0, SGI0, SGI0, SGI25281, SGI25281, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5805[] = {SGI25282, SGI0, SGI25283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5806[] = {SGI25284, SGI0, SGI25286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5807[] = {SGI25285, SGI25287, SGI25288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25289, SGI0, SGI0, SGI0, SGI25290, SGI25290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5808[] = {SGI25291, SGI25292, SGI25293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25294, SGI0, SGI0, SGI0, SGI25295, SGI25295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5809[] = {SGI25296, SGI25297, SGI25298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25299, SGI0, SGI0, SGI0, SGI25300, SGI25300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5810[] = {SGI25301, SGI0, SGI25302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25303, SGI0, SGI0, SGI0, SGI25304, SGI25304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5811[] = {SGI25305, SGI25306, SGI25307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25308, SGI0, SGI0, SGI0, SGI25309, SGI25309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5812[] = {SGI25310, SGI25311, SGI25312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25313, SGI0, SGI0, SGI0, SGI25314, SGI25314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5813[] = {SGI25315, SGI25316, SGI25317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25318, SGI0, SGI0, SGI0, SGI25319, SGI25319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5814[] = {SGI25320, SGI25321, SGI25322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25323, SGI0, SGI0, SGI0, SGI25324, SGI25324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5815[] = {SGI25325, SGI25326, SGI25327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25328, SGI0, SGI0, SGI0, SGI25329, SGI25329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5816[] = {SGI0, SGI25330, SGI25331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25332, SGI0, SGI0, SGI0, SGI25333, SGI25333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5817[] = {SGI25334, SGI25335, SGI25336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25337, SGI0, SGI0, SGI0, SGI25338, SGI25338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5818[] = {SGI25339, SGI25340, SGI25341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25342, SGI0, SGI0, SGI0, SGI25343, SGI25343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5819[] = {SGI25344, SGI25345, SGI25346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25347, SGI0, SGI0, SGI0, SGI25348, SGI25348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5820[] = {SGI25349, SGI25350, SGI25351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25352, SGI0, SGI0, SGI0, SGI25353, SGI25353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5821[] = {SGI25354, SGI25355, SGI25356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25357, SGI0, SGI0, SGI0, SGI25358, SGI25358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5822[] = {SGI25359, SGI25360, SGI25361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25362, SGI0, SGI0, SGI0, SGI25363, SGI25363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5823[] = {SGI25364, SGI25365, SGI25366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25367, SGI0, SGI0, SGI0, SGI25368, SGI25368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5824[] = {SGI25369, SGI25370, SGI25371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25372, SGI0, SGI0, SGI0, SGI25373, SGI25373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5825[] = {SGI25374, SGI25375, SGI25376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25377, SGI0, SGI0, SGI0, SGI25378, SGI25378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5826[] = {SGI25379, SGI25380, SGI25381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25382, SGI0, SGI0, SGI0, SGI25383, SGI25383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5827[] = {SGI25384, SGI25385, SGI25386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25387, SGI0, SGI0, SGI0, SGI25388, SGI25388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5828[] = {SGI25389, SGI25390, SGI25391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25392, SGI0, SGI0, SGI0, SGI25393, SGI25393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5829[] = {SGI25394, SGI25395, SGI25396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25397, SGI0, SGI0, SGI0, SGI25398, SGI25398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5830[] = {SGI25399, SGI25400, SGI25401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25402, SGI0, SGI0, SGI0, SGI25403, SGI25403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5831[] = {SGI25404, SGI25405, SGI25406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25407, SGI0, SGI0, SGI0, SGI25408, SGI25408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5832[] = {SGI25409, SGI25410, SGI25411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25412, SGI0, SGI0, SGI0, SGI25415, SGI25415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5833[] = {SGI25413, SGI25414, SGI25416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25417, SGI0, SGI0, SGI0, SGI25418, SGI25418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5834[] = {SGI25419, SGI25420, SGI25421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25422, SGI0, SGI0, SGI0, SGI25423, SGI25423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5835[] = {SGI25424, SGI25425, SGI25426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25427, SGI0, SGI0, SGI0, SGI25428, SGI25428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5836[] = {SGI25429, SGI25430, SGI25431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25432, SGI0, SGI0, SGI0, SGI25433, SGI25433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5837[] = {SGI25434, SGI25435, SGI25436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25437, SGI0, SGI0, SGI0, SGI25438, SGI25438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5838[] = {SGI0, SGI25439, SGI25440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25441, SGI0, SGI0, SGI0, SGI25442, SGI25442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5839[] = {SGI25443, SGI25444, SGI25445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25446, SGI0, SGI0, SGI0, SGI25447, SGI25447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5840[] = {SGI25448, SGI25449, SGI25450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25451, SGI0, SGI0, SGI0, SGI25452, SGI25452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5841[] = {SGI25453, SGI25454, SGI25455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25456, SGI0, SGI0, SGI0, SGI25457, SGI25457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5842[] = {SGI25458, SGI25459, SGI25460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25461, SGI0, SGI0, SGI0, SGI25462, SGI25462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5843[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5844[] = {SGI25464, SGI25465, SGI25466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25467, SGI0, SGI0, SGI0, SGI25468, SGI25468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5845[] = {SGI25469, SGI25470, SGI25471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25472, SGI0, SGI0, SGI0, SGI25473, SGI25473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5846[] = {SGI25474, SGI25475, SGI25476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25477, SGI0, SGI0, SGI0, SGI25478, SGI25478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5847[] = {SGI25479, SGI25480, SGI25481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25482, SGI0, SGI0, SGI0, SGI25483, SGI25483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5848[] = {SGI0, SGI25484, SGI25485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5849[] = {SGI25486, SGI25487, SGI25488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25489, SGI25489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5850[] = {SGI25490, SGI25491, SGI25492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25493, SGI0, SGI0, SGI0, SGI25494, SGI25494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5851[] = {SGI25495, SGI25496, SGI25497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25498, SGI0, SGI0, SGI0, SGI25499, SGI25499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5852[] = {SGI25500, SGI25501, SGI25502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25503, SGI0, SGI0, SGI0, SGI25504, SGI25504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5853[] = {SGI25505, SGI25506, SGI25507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25508, SGI0, SGI0, SGI0, SGI25509, SGI25509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5854[] = {SGI25510, SGI25511, SGI25512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25513, SGI0, SGI0, SGI0, SGI25514, SGI25514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5855[] = {SGI25515, SGI25516, SGI25517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25518, SGI0, SGI0, SGI0, SGI25519, SGI25519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5856[] = {SGI25520, SGI25521, SGI25522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25523, SGI0, SGI0, SGI0, SGI25524, SGI25524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5857[] = {SGI25525, SGI25526, SGI25527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25528, SGI0, SGI0, SGI0, SGI25529, SGI25529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5858[] = {SGI25530, SGI25531, SGI25532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25533, SGI0, SGI0, SGI0, SGI25534, SGI25534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5859[] = {SGI25535, SGI25536, SGI25537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25538, SGI0, SGI0, SGI0, SGI25539, SGI25539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5860[] = {SGI0, SGI25540, SGI25541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25542, SGI0, SGI0, SGI0, SGI25543, SGI25543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5861[] = {SGI25544, SGI25545, SGI25546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25547, SGI0, SGI0, SGI0, SGI25548, SGI25548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5862[] = {SGI25549, SGI25550, SGI25551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25552, SGI0, SGI0, SGI0, SGI25553, SGI25553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5863[] = {SGI25554, SGI25555, SGI25556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25557, SGI0, SGI0, SGI0, SGI25558, SGI25558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5864[] = {SGI25559, SGI25560, SGI25561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25562, SGI0, SGI0, SGI0, SGI25563, SGI25563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5865[] = {SGI25564, SGI25565, SGI25566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25567, SGI0, SGI0, SGI0, SGI25568, SGI25568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5866[] = {SGI25569, SGI25570, SGI25571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25572, SGI0, SGI0, SGI0, SGI25573, SGI25573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5867[] = {SGI25574, SGI25575, SGI25576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25577, SGI0, SGI0, SGI0, SGI25578, SGI25578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5868[] = {SGI0, SGI25579, SGI25580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25581, SGI0, SGI0, SGI0, SGI25582, SGI25582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5869[] = {SGI25583, SGI25584, SGI25585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25586, SGI0, SGI0, SGI0, SGI25587, SGI25587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5870[] = {SGI25588, SGI25589, SGI25590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25591, SGI0, SGI0, SGI0, SGI25592, SGI25592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5871[] = {SGI25593, SGI25594, SGI25595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25596, SGI0, SGI0, SGI0, SGI25597, SGI25597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5872[] = {SGI25598, SGI25599, SGI25600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25601, SGI0, SGI0, SGI0, SGI25602, SGI25602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5873[] = {SGI25603, SGI25604, SGI25605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25606, SGI0, SGI0, SGI0, SGI25607, SGI25607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5874[] = {SGI25608, SGI25609, SGI25610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25611, SGI0, SGI0, SGI0, SGI25612, SGI25612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5875[] = {SGI25613, SGI25614, SGI25615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25616, SGI0, SGI0, SGI0, SGI25617, SGI25617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5876[] = {SGI25618, SGI25619, SGI25620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25621, SGI0, SGI0, SGI0, SGI25622, SGI25622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5877[] = {SGI25623, SGI25624, SGI25625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25626, SGI0, SGI0, SGI0, SGI25627, SGI25627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5878[] = {SGI25628, SGI25629, SGI25630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25631, SGI0, SGI0, SGI0, SGI25632, SGI25632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5879[] = {SGI25633, SGI25634, SGI25635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25636, SGI0, SGI0, SGI0, SGI25637, SGI25637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5880[] = {SGI25638, SGI25639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25640, SGI0, SGI0, SGI0, SGI25641, SGI25641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5881[] = {SGI25642, SGI25643, SGI25644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25645, SGI0, SGI0, SGI0, SGI25646, SGI25646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5882[] = {SGI25647, SGI25648, SGI25649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25650, SGI25650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5883[] = {SGI25651, SGI25652, SGI25653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25654, SGI25654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5884[] = {SGI25655, SGI25656, SGI25657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25658, SGI0, SGI0, SGI0, SGI25659, SGI25659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5885[] = {SGI25660, SGI25661, SGI25662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25663, SGI0, SGI0, SGI0, SGI25664, SGI25664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5886[] = {SGI25665, SGI25666, SGI25667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25668, SGI0, SGI0, SGI0, SGI25669, SGI25669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5887[] = {SGI0, SGI25670, SGI25671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25672, SGI0, SGI0, SGI0, SGI25673, SGI25673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5888[] = {SGI25674, SGI25675, SGI25676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25677, SGI0, SGI0, SGI0, SGI25678, SGI25678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5889[] = {SGI0, SGI25679, SGI25680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25681, SGI0, SGI0, SGI0, SGI25682, SGI25682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5890[] = {SGI25683, SGI25684, SGI25685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25686, SGI0, SGI0, SGI0, SGI25687, SGI25687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5891[] = {SGI25688, SGI25689, SGI25690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25691, SGI0, SGI0, SGI0, SGI25692, SGI25692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5892[] = {SGI25693, SGI25694, SGI25695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25696, SGI0, SGI0, SGI0, SGI25697, SGI25697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5893[] = {SGI0, SGI0, SGI25698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25699, SGI0, SGI0, SGI0, SGI25700, SGI25700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5894[] = {SGI25701, SGI25702, SGI25703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25704, SGI0, SGI0, SGI0, SGI25705, SGI25705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5895[] = {SGI25706, SGI25707, SGI25708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25709, SGI0, SGI0, SGI0, SGI25710, SGI25710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5896[] = {SGI25711, SGI25712, SGI25713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25714, SGI0, SGI0, SGI0, SGI25715, SGI25715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5897[] = {SGI25716, SGI25717, SGI25718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25719, SGI0, SGI0, SGI0, SGI25720, SGI25720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5898[] = {SGI25721, SGI25722, SGI25723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25724, SGI0, SGI0, SGI0, SGI25725, SGI25725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5899[] = {SGI25726, SGI25727, SGI25728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25729, SGI0, SGI0, SGI0, SGI25730, SGI25730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5900[] = {SGI25731, SGI25732, SGI25733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25734, SGI0, SGI0, SGI0, SGI25735, SGI25735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5901[] = {SGI0, SGI25736, SGI25737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5902[] = {SGI25738, SGI25739, SGI25740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25741, SGI0, SGI0, SGI0, SGI25742, SGI25742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5903[] = {SGI0, SGI25743, SGI25744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25745, SGI0, SGI0, SGI0, SGI25746, SGI25746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5904[] = {SGI25747, SGI25748, SGI25749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25750, SGI0, SGI0, SGI0, SGI25751, SGI25751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5905[] = {SGI25752, SGI25753, SGI25754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25755, SGI0, SGI0, SGI0, SGI25756, SGI25756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5906[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25757, SGI0, SGI0, SGI0, SGI25758, SGI25758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5907[] = {SGI25759, SGI25760, SGI25761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25762, SGI0, SGI0, SGI0, SGI25763, SGI25763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5908[] = {SGI25764, SGI25765, SGI25766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25767, SGI0, SGI0, SGI0, SGI25768, SGI25768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5909[] = {SGI0, SGI25769, SGI25770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25771, SGI0, SGI0, SGI0, SGI25772, SGI25772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5910[] = {SGI25773, SGI25774, SGI25775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25776, SGI0, SGI0, SGI0, SGI25777, SGI25777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5911[] = {SGI25778, SGI25779, SGI25780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25781, SGI0, SGI0, SGI0, SGI25782, SGI25782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5912[] = {SGI25783, SGI25784, SGI25785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25786, SGI0, SGI0, SGI0, SGI25787, SGI25787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5913[] = {SGI25788, SGI25789, SGI25790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25791, SGI0, SGI0, SGI0, SGI25792, SGI25792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5914[] = {SGI25793, SGI25794, SGI25795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25796, SGI0, SGI0, SGI0, SGI25797, SGI25797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5915[] = {SGI25798, SGI25799, SGI25800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25801, SGI0, SGI0, SGI0, SGI25802, SGI25802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5916[] = {SGI25803, SGI25804, SGI25805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25806, SGI0, SGI0, SGI0, SGI25807, SGI25807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5917[] = {SGI25808, SGI25809, SGI25810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25811, SGI0, SGI0, SGI0, SGI25812, SGI25812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5918[] = {SGI0, SGI25813, SGI25814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25815, SGI0, SGI0, SGI0, SGI25816, SGI25816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5919[] = {SGI25817, SGI25818, SGI25819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25820, SGI0, SGI0, SGI0, SGI25821, SGI25821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5920[] = {SGI25822, SGI25823, SGI25824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25825, SGI0, SGI0, SGI0, SGI25826, SGI25826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5921[] = {SGI25827, SGI25828, SGI25829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25830, SGI0, SGI0, SGI0, SGI25831, SGI25831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5922[] = {SGI25832, SGI25833, SGI25834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25835, SGI0, SGI0, SGI0, SGI25836, SGI25836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5923[] = {SGI25837, SGI25838, SGI25839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25840, SGI0, SGI0, SGI0, SGI25841, SGI25841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5924[] = {SGI0, SGI25842, SGI25843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25844, SGI0, SGI0, SGI0, SGI25845, SGI25845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5925[] = {SGI25846, SGI25847, SGI25848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25849, SGI0, SGI0, SGI0, SGI25850, SGI25850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5926[] = {SGI0, SGI25851, SGI25852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5927[] = {SGI0, SGI25853, SGI25854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25855, SGI0, SGI0, SGI0, SGI25856, SGI25856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5928[] = {SGI25857, SGI25858, SGI25859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25860, SGI0, SGI0, SGI0, SGI25861, SGI25861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5929[] = {SGI25862, SGI25863, SGI25864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25865, SGI0, SGI0, SGI0, SGI25866, SGI25866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5930[] = {SGI25867, SGI25868, SGI25869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25870, SGI0, SGI0, SGI0, SGI25871, SGI25871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5931[] = {SGI25872, SGI25873, SGI25874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25875, SGI0, SGI0, SGI0, SGI25876, SGI25876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5932[] = {SGI25877, SGI25878, SGI25879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25880, SGI25880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5933[] = {SGI25881, SGI25882, SGI25883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25884, SGI0, SGI0, SGI0, SGI25885, SGI25885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5934[] = {SGI25886, SGI25887, SGI25888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25889, SGI0, SGI0, SGI0, SGI25890, SGI25890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5935[] = {SGI25891, SGI25892, SGI25893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25894, SGI0, SGI0, SGI0, SGI25895, SGI25895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5936[] = {SGI25896, SGI25897, SGI25898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25899, SGI0, SGI0, SGI0, SGI25900, SGI25900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5937[] = {SGI25901, SGI25902, SGI25903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25904, SGI0, SGI0, SGI0, SGI25905, SGI25905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5938[] = {SGI25906, SGI25907, SGI25908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25909, SGI0, SGI0, SGI0, SGI25910, SGI25910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5939[] = {SGI25911, SGI25912, SGI25913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25914, SGI0, SGI0, SGI0, SGI25915, SGI25915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5940[] = {SGI25916, SGI0, SGI25917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25918, SGI0, SGI0, SGI0, SGI25919, SGI25919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5941[] = {SGI25920, SGI25921, SGI25922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25923, SGI0, SGI0, SGI0, SGI25924, SGI25924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5942[] = {SGI25925, SGI25926, SGI25927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25928, SGI0, SGI0, SGI0, SGI25929, SGI25929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5943[] = {SGI25930, SGI25931, SGI25932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25933, SGI25934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5944[] = {SGI25935, SGI25936, SGI25937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25938, SGI25938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5945[] = {SGI25939, SGI25940, SGI25941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25942, SGI25942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5946[] = {SGI25943, SGI25944, SGI25945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25946, SGI0, SGI0, SGI0, SGI25947, SGI25947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5947[] = {SGI25948, SGI25949, SGI25950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25951, SGI0, SGI0, SGI0, SGI25952, SGI25952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5948[] = {SGI25953, SGI25954, SGI25955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25956, SGI0, SGI0, SGI0, SGI25957, SGI25957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5949[] = {SGI25958, SGI25959, SGI25960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25961, SGI25961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5950[] = {SGI25962, SGI25963, SGI25964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25965, SGI0, SGI0, SGI0, SGI25966, SGI25966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5951[] = {SGI25967, SGI25968, SGI25969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25970, SGI0, SGI0, SGI0, SGI25971, SGI25971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5952[] = {SGI25972, SGI25973, SGI25974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25975, SGI0, SGI0, SGI0, SGI25976, SGI25976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5953[] = {SGI25977, SGI25978, SGI25979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25980, SGI25980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5954[] = {SGI25981, SGI25982, SGI25983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25984, SGI0, SGI0, SGI0, SGI25985, SGI25985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5955[] = {SGI25986, SGI25987, SGI25988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25989, SGI0, SGI0, SGI0, SGI25992, SGI25992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5956[] = {SGI25990, SGI25991, SGI25993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25994, SGI0, SGI0, SGI0, SGI25995, SGI25995, SGI0, SGI0, NULL};
const Segment * const SG5957[] = {SGI25996, SGI25997, SGI25998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI25999, SGI0, SGI0, SGI0, SGI26000, SGI26000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5958[] = {SGI26001, SGI26002, SGI26003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26004, SGI26005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5959[] = {SGI26006, SGI26007, SGI26008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26009, SGI26010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5960[] = {SGI26011, SGI26012, SGI26013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26014, SGI0, SGI0, SGI0, SGI26015, SGI26015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5961[] = {SGI26016, SGI26017, SGI26018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26019, SGI0, SGI0, SGI0, SGI26020, SGI26020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5962[] = {SGI26021, SGI26022, SGI26023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26024, SGI0, SGI0, SGI0, SGI26025, SGI26025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5963[] = {SGI26026, SGI26027, SGI26028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26029, SGI0, SGI0, SGI0, SGI26030, SGI26030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5964[] = {SGI26031, SGI26032, SGI26033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26034, SGI0, SGI0, SGI0, SGI26035, SGI26035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5965[] = {SGI26036, SGI0, SGI26037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26038, SGI0, SGI0, SGI0, SGI26039, SGI26039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5966[] = {SGI26040, SGI26041, SGI26042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26043, SGI26044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5967[] = {SGI26045, SGI26046, SGI26047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26048, SGI0, SGI0, SGI0, SGI26049, SGI26049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5968[] = {SGI0, SGI0, SGI26050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5969[] = {SGI26051, SGI26052, SGI26053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26054, SGI0, SGI0, SGI0, SGI26055, SGI26055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5970[] = {SGI26056, SGI26057, SGI26058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26059, SGI0, SGI0, SGI0, SGI26060, SGI26060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5971[] = {SGI26061, SGI26062, SGI26063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26064, SGI0, SGI0, SGI0, SGI26065, SGI26065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5972[] = {SGI26066, SGI26067, SGI26068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26069, SGI0, SGI0, SGI0, SGI26070, SGI26070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5973[] = {SGI26071, SGI26072, SGI26073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26074, SGI0, SGI0, SGI0, SGI26075, SGI26075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5974[] = {SGI26076, SGI26077, SGI26078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26079, SGI0, SGI0, SGI0, SGI26080, SGI26080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5975[] = {SGI0, SGI26081, SGI26082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26083, SGI0, SGI0, SGI0, SGI26084, SGI26084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5976[] = {SGI26085, SGI26086, SGI26087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26088, SGI0, SGI0, SGI0, SGI26089, SGI26089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5977[] = {SGI26090, SGI26091, SGI26092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26093, SGI0, SGI0, SGI0, SGI26094, SGI26094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5978[] = {SGI26095, SGI26096, SGI26097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26098, SGI26098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5979[] = {SGI26099, SGI26100, SGI26101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26102, SGI0, SGI0, SGI0, SGI26103, SGI26103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5980[] = {SGI26104, SGI26105, SGI26106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26107, SGI0, SGI0, SGI0, SGI26108, SGI26108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5981[] = {SGI26109, SGI26110, SGI26111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26112, SGI0, SGI0, SGI0, SGI26113, SGI26113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5982[] = {SGI26114, SGI26115, SGI26116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26117, SGI0, SGI0, SGI0, SGI26118, SGI26118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5983[] = {SGI0, SGI26119, SGI26120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26121, SGI0, SGI0, SGI0, SGI26122, SGI26122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5984[] = {SGI26123, SGI26124, SGI26125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26126, SGI0, SGI0, SGI0, SGI26127, SGI26127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5985[] = {SGI26128, SGI26129, SGI26130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26131, SGI0, SGI0, SGI0, SGI26132, SGI26132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5986[] = {SGI26133, SGI26134, SGI26135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26136, SGI0, SGI0, SGI0, SGI26137, SGI26137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5987[] = {SGI26138, SGI26139, SGI26140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26141, SGI0, SGI0, SGI0, SGI26142, SGI26142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5988[] = {SGI26143, SGI26144, SGI26145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26146, SGI0, SGI0, SGI0, SGI26147, SGI26147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5989[] = {SGI0, SGI26148, SGI26149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26150, SGI0, SGI0, SGI0, SGI26151, SGI26151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5990[] = {SGI26152, SGI26153, SGI26154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26155, SGI0, SGI0, SGI0, SGI26156, SGI26156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5991[] = {SGI26157, SGI26158, SGI26159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26160, SGI0, SGI0, SGI0, SGI26161, SGI26161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5992[] = {SGI26162, SGI26163, SGI26164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26165, SGI0, SGI0, SGI0, SGI26166, SGI26166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5993[] = {SGI26167, SGI26168, SGI26169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26170, SGI0, SGI0, SGI0, SGI26171, SGI26171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5994[] = {SGI26172, SGI26173, SGI26174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26175, SGI0, SGI0, SGI0, SGI26176, SGI26176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5995[] = {SGI26177, SGI26178, SGI26179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26180, SGI0, SGI0, SGI0, SGI26181, SGI26181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5996[] = {SGI26182, SGI26183, SGI26184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26185, SGI0, SGI0, SGI0, SGI26186, SGI26186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5997[] = {SGI26187, SGI26188, SGI26189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26190, SGI0, SGI0, SGI0, SGI26191, SGI26191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5998[] = {SGI26192, SGI26193, SGI26194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26195, SGI0, SGI0, SGI0, SGI26196, SGI26196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG5999[] = {SGI26197, SGI26198, SGI26199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26200, SGI0, SGI0, SGI0, SGI26201, SGI26201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6000[] = {SGI26202, SGI26203, SGI26204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26205, SGI0, SGI0, SGI0, SGI26206, SGI26206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6001[] = {SGI26207, SGI26208, SGI26209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26210, SGI26211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6002[] = {SGI26212, SGI26213, SGI26214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26215, SGI0, SGI0, SGI0, SGI26216, SGI26216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6003[] = {SGI26217, SGI26218, SGI26219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26220, SGI26221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6004[] = {SGI26222, SGI26223, SGI26224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26225, SGI0, SGI0, SGI0, SGI26226, SGI26226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6005[] = {SGI26227, SGI26228, SGI26229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26230, SGI0, SGI0, SGI0, SGI26231, SGI26231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6006[] = {SGI26232, SGI26233, SGI26234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26235, SGI0, SGI0, SGI0, SGI26236, SGI26236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6007[] = {SGI0, SGI26237, SGI26238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26239, SGI0, SGI0, SGI0, SGI26240, SGI26240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6008[] = {SGI26241, SGI26242, SGI26243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26244, SGI0, SGI0, SGI0, SGI26245, SGI26245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6009[] = {SGI0, SGI26246, SGI26247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26248, SGI0, SGI0, SGI0, SGI26249, SGI26249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6010[] = {SGI0, SGI0, SGI26250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26251, SGI0, SGI0, SGI0, SGI26252, SGI26252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6011[] = {SGI26253, SGI26254, SGI26255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26256, SGI0, SGI0, SGI0, SGI26257, SGI26257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6012[] = {SGI26258, SGI26259, SGI26261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26264, SGI0, SGI0, SGI0, SGI26266, SGI26266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6013[] = {SGI26260, SGI26262, SGI26263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26265, SGI0, SGI0, SGI0, SGI26267, SGI26267, SGI0, SGI0, NULL};
const Segment * const SG6014[] = {SGI26268, SGI26269, SGI26270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26271, SGI0, SGI0, SGI0, SGI26272, SGI26272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6015[] = {SGI26273, SGI26274, SGI26275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26276, SGI0, SGI0, SGI0, SGI26277, SGI26277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6016[] = {SGI26278, SGI26279, SGI26280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26281, SGI0, SGI0, SGI0, SGI26282, SGI26282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6017[] = {SGI26283, SGI26284, SGI26285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26289, SGI0, SGI0, SGI0, SGI0, SGI26291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6018[] = {SGI26286, SGI26287, SGI26288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26290, SGI0, SGI0, SGI0, SGI26292, SGI26292, SGI0, SGI0, NULL};
const Segment * const SG6019[] = {SGI26293, SGI26294, SGI26295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26296, SGI0, SGI0, SGI0, SGI26297, SGI26297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6020[] = {SGI26298, SGI26299, SGI26300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26301, SGI0, SGI0, SGI0, SGI26302, SGI26302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6021[] = {SGI26303, SGI26304, SGI26305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26306, SGI0, SGI0, SGI0, SGI0, SGI26307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6022[] = {SGI26308, SGI26309, SGI26310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26311, SGI0, SGI0, SGI0, SGI0, SGI26312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6023[] = {SGI26313, SGI26314, SGI26315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26316, SGI0, SGI0, SGI0, SGI26317, SGI26317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6024[] = {SGI26318, SGI26319, SGI26320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26321, SGI0, SGI0, SGI0, SGI26322, SGI26322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6025[] = {SGI26323, SGI26324, SGI26325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26326, SGI0, SGI0, SGI0, SGI26327, SGI26327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6026[] = {SGI26328, SGI26329, SGI26330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26331, SGI0, SGI0, SGI0, SGI26332, SGI26332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6027[] = {SGI26333, SGI26334, SGI26335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26336, SGI0, SGI0, SGI0, SGI26337, SGI26337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6028[] = {SGI26338, SGI26339, SGI26340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26341, SGI0, SGI0, SGI0, SGI26342, SGI26342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6029[] = {SGI26343, SGI26344, SGI26345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26346, SGI0, SGI0, SGI0, SGI26347, SGI26347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6030[] = {SGI26348, SGI26349, SGI26350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26351, SGI0, SGI0, SGI0, SGI26352, SGI26352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6031[] = {SGI26353, SGI26354, SGI26355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26356, SGI0, SGI0, SGI0, SGI26357, SGI26357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6032[] = {SGI26358, SGI26359, SGI26360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26361, SGI0, SGI0, SGI0, SGI26362, SGI26362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6033[] = {SGI26363, SGI26364, SGI26365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26366, SGI0, SGI0, SGI0, SGI26367, SGI26367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6034[] = {SGI26368, SGI26369, SGI26370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26371, SGI26372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6035[] = {SGI26373, SGI26374, SGI26375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26376, SGI0, SGI0, SGI0, SGI26377, SGI26377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6036[] = {SGI26378, SGI26379, SGI26380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26381, SGI26381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6037[] = {SGI26382, SGI26383, SGI26384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26385, SGI0, SGI0, SGI0, SGI26386, SGI26386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6038[] = {SGI0, SGI26387, SGI26388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26389, SGI0, SGI0, SGI0, SGI26390, SGI26390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6039[] = {SGI0, SGI26391, SGI26392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26393, SGI0, SGI0, SGI0, SGI26394, SGI26394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6040[] = {SGI26395, SGI26396, SGI26397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26398, SGI0, SGI0, SGI0, SGI26399, SGI26399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6041[] = {SGI26400, SGI26401, SGI26402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26403, SGI0, SGI0, SGI0, SGI0, SGI26404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6042[] = {SGI26405, SGI26406, SGI26407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26408, SGI0, SGI0, SGI0, SGI26409, SGI26409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6043[] = {SGI26410, SGI26411, SGI26412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26413, SGI0, SGI0, SGI0, SGI26414, SGI26414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6044[] = {SGI0, SGI26415, SGI26416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6045[] = {SGI26417, SGI26418, SGI26419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26420, SGI0, SGI0, SGI0, SGI0, SGI26421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6046[] = {SGI26422, SGI26423, SGI26424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26425, SGI0, SGI0, SGI0, SGI26426, SGI26426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6047[] = {SGI26427, SGI26428, SGI26429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26430, SGI0, SGI0, SGI0, SGI26431, SGI26431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6048[] = {SGI26432, SGI26433, SGI26434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26435, SGI0, SGI0, SGI0, SGI0, SGI26436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6049[] = {SGI26437, SGI26438, SGI26439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26440, SGI0, SGI0, SGI0, SGI26441, SGI26441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6050[] = {SGI26442, SGI26443, SGI26444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26445, SGI0, SGI0, SGI0, SGI26446, SGI26446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6051[] = {SGI26447, SGI26448, SGI26449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26450, SGI0, SGI0, SGI0, SGI0, SGI26451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6052[] = {SGI26452, SGI26453, SGI26454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26455, SGI0, SGI0, SGI0, SGI26456, SGI26456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6053[] = {SGI26457, SGI26458, SGI26459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26460, SGI26461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6054[] = {SGI0, SGI26462, SGI26463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26464, SGI0, SGI0, SGI0, SGI26465, SGI26465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6055[] = {SGI26466, SGI0, SGI26467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6056[] = {SGI26468, SGI26469, SGI26470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26471, SGI0, SGI0, SGI0, SGI26472, SGI26472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6057[] = {SGI26473, SGI26474, SGI26475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26476, SGI0, SGI0, SGI0, SGI26477, SGI26477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6058[] = {SGI26478, SGI26479, SGI26480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26481, SGI0, SGI0, SGI0, SGI26482, SGI26482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6059[] = {SGI26483, SGI26484, SGI26485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26486, SGI0, SGI0, SGI0, SGI26487, SGI26487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6060[] = {SGI26488, SGI26489, SGI26490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26491, SGI26491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6061[] = {SGI26492, SGI0, SGI26493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26495, SGI0, SGI0, SGI0, SGI26498, SGI26498, SGI0, SGI0, NULL};
const Segment * const SG6062[] = {SGI26494, SGI26496, SGI26497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26499, SGI0, SGI0, SGI0, SGI26500, SGI26500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6063[] = {SGI26501, SGI26502, SGI26503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26504, SGI0, SGI0, SGI0, SGI26505, SGI26505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6064[] = {SGI26506, SGI26507, SGI26508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26509, SGI0, SGI0, SGI0, SGI0, SGI26510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6065[] = {SGI26511, SGI0, SGI26512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26515, SGI0, SGI0, SGI0, SGI26517, SGI26517, SGI0, SGI0, NULL};
const Segment * const SG6066[] = {SGI26513, SGI26514, SGI26516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26518, SGI0, SGI0, SGI0, SGI26519, SGI26519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6067[] = {SGI26520, SGI0, SGI26521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26522, SGI0, SGI0, SGI0, SGI26524, SGI26524, SGI0, SGI0, NULL};
const Segment * const SG6068[] = {SGI26523, SGI26525, SGI26526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26527, SGI0, SGI0, SGI0, SGI26528, SGI26528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6069[] = {SGI0, SGI26529, SGI26530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26531, SGI0, SGI0, SGI0, SGI26532, SGI26532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6070[] = {SGI26533, SGI0, SGI26534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6071[] = {SGI26535, SGI26536, SGI26537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26538, SGI0, SGI0, SGI0, SGI26539, SGI26539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6072[] = {SGI26540, SGI26541, SGI26542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26543, SGI0, SGI0, SGI0, SGI0, SGI26544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6073[] = {SGI26545, SGI26546, SGI26547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26548, SGI0, SGI0, SGI0, SGI26549, SGI26549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6074[] = {SGI26550, SGI26551, SGI26552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26553, SGI0, SGI0, SGI0, SGI26554, SGI26554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6075[] = {SGI0, SGI26555, SGI26556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26557, SGI0, SGI0, SGI0, SGI26558, SGI26558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6076[] = {SGI26559, SGI26560, SGI26561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26562, SGI0, SGI0, SGI0, SGI0, SGI26563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6077[] = {SGI26564, SGI0, SGI26565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26566, SGI0, SGI0, SGI0, SGI0, SGI26567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6078[] = {SGI26568, SGI26569, SGI26570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26571, SGI0, SGI0, SGI0, SGI26572, SGI26572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6079[] = {SGI26573, SGI0, SGI26574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26575, SGI0, SGI0, SGI0, SGI26576, SGI26576, SGI0, SGI0, NULL};
const Segment * const SG6080[] = {SGI26577, SGI0, SGI26578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26579, SGI0, SGI0, SGI0, SGI26580, SGI26580, SGI0, SGI0, NULL};
const Segment * const SG6081[] = {SGI26581, SGI26582, SGI26583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26584, SGI0, SGI0, SGI0, SGI26585, SGI26585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6082[] = {SGI0, SGI26586, SGI26587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26588, SGI0, SGI0, SGI0, SGI26589, SGI26589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6083[] = {SGI0, SGI26590, SGI26591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26592, SGI0, SGI0, SGI0, SGI26593, SGI26593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6084[] = {SGI26594, SGI26595, SGI26596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26597, SGI26598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6085[] = {SGI26599, SGI26600, SGI26601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26602, SGI0, SGI0, SGI0, SGI26603, SGI26603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6086[] = {SGI26604, SGI0, SGI26605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26606, SGI0, SGI0, SGI0, SGI26607, SGI26607, SGI0, SGI0, NULL};
const Segment * const SG6087[] = {SGI26608, SGI26609, SGI26610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26611, SGI0, SGI0, SGI0, SGI26612, SGI26612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6088[] = {SGI26613, SGI26614, SGI26615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26616, SGI0, SGI0, SGI0, SGI26617, SGI26617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6089[] = {SGI26621, SGI0, SGI26622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26623, SGI0, SGI0, SGI0, SGI26625, SGI26625, SGI0, SGI0, NULL};
const Segment * const SG6090[] = {SGI26618, SGI26619, SGI26620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26624, SGI0, SGI0, SGI0, SGI26626, SGI26626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6091[] = {SGI26627, SGI26628, SGI26629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26630, SGI0, SGI0, SGI0, SGI0, SGI26631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6092[] = {SGI26632, SGI26633, SGI26634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26635, SGI0, SGI0, SGI0, SGI26636, SGI26636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6093[] = {SGI26637, SGI26638, SGI26639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26640, SGI0, SGI0, SGI0, SGI26641, SGI26641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6094[] = {SGI26642, SGI26643, SGI26644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26645, SGI0, SGI0, SGI0, SGI26646, SGI26646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6095[] = {SGI26647, SGI26648, SGI26649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26650, SGI0, SGI0, SGI0, SGI26651, SGI26651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6096[] = {SGI26652, SGI26653, SGI26654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26655, SGI0, SGI0, SGI0, SGI26656, SGI26656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6097[] = {SGI26657, SGI26658, SGI26659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26660, SGI0, SGI0, SGI0, SGI0, SGI26661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6098[] = {SGI26662, SGI26663, SGI26664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26665, SGI0, SGI0, SGI0, SGI26666, SGI26666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6099[] = {SGI26667, SGI26668, SGI26669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26670, SGI0, SGI0, SGI0, SGI26671, SGI26671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6100[] = {SGI26672, SGI26673, SGI26674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26675, SGI0, SGI0, SGI0, SGI0, SGI26676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6101[] = {SGI26677, SGI0, SGI26678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26679, SGI0, SGI0, SGI0, SGI26680, SGI26680, SGI0, SGI0, NULL};
const Segment * const SG6102[] = {SGI26681, SGI26682, SGI26683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26684, SGI0, SGI0, SGI0, SGI26685, SGI26685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6103[] = {SGI26686, SGI0, SGI26687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6104[] = {SGI26688, SGI26689, SGI26690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26691, SGI0, SGI0, SGI0, SGI26692, SGI26692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6105[] = {SGI26693, SGI26694, SGI26695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26696, SGI0, SGI0, SGI0, SGI26697, SGI26697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6106[] = {SGI26698, SGI26699, SGI26700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26701, SGI0, SGI0, SGI0, SGI26702, SGI26702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6107[] = {SGI26703, SGI26704, SGI26705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26706, SGI0, SGI0, SGI0, SGI26707, SGI26707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6108[] = {SGI26708, SGI26709, SGI26710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26711, SGI0, SGI0, SGI0, SGI26712, SGI26712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6109[] = {SGI26713, SGI26714, SGI26715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26716, SGI0, SGI0, SGI0, SGI26717, SGI26717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6110[] = {SGI26718, SGI26719, SGI26720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26721, SGI0, SGI0, SGI0, SGI26722, SGI26722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6111[] = {SGI26723, SGI26724, SGI26725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26726, SGI0, SGI0, SGI0, SGI26727, SGI26727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6112[] = {SGI26728, SGI26729, SGI26730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26731, SGI0, SGI0, SGI0, SGI26732, SGI26732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6113[] = {SGI26733, SGI26734, SGI26735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26736, SGI0, SGI0, SGI0, SGI26737, SGI26737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6114[] = {SGI26738, SGI0, SGI26739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26740, SGI0, SGI0, SGI0, SGI26741, SGI26741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6115[] = {SGI26742, SGI26743, SGI26744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26745, SGI0, SGI0, SGI0, SGI26746, SGI26746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6116[] = {SGI26747, SGI26748, SGI26749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26750, SGI0, SGI0, SGI0, SGI26751, SGI26751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6117[] = {SGI26752, SGI26753, SGI26754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26755, SGI0, SGI0, SGI0, SGI26756, SGI26756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6118[] = {SGI26757, SGI26758, SGI26759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26760, SGI0, SGI0, SGI0, SGI26761, SGI26761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6119[] = {SGI26762, SGI26763, SGI26764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26765, SGI0, SGI0, SGI0, SGI26766, SGI26766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6120[] = {SGI26767, SGI26768, SGI26769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26770, SGI0, SGI0, SGI0, SGI26771, SGI26771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6121[] = {SGI26772, SGI26773, SGI26774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26775, SGI0, SGI0, SGI0, SGI26776, SGI26776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6122[] = {SGI26777, SGI26778, SGI26779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26780, SGI0, SGI0, SGI0, SGI26781, SGI26781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6123[] = {SGI26782, SGI26783, SGI26784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26785, SGI0, SGI0, SGI0, SGI26786, SGI26786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6124[] = {SGI26787, SGI26788, SGI26789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26790, SGI0, SGI0, SGI0, SGI26791, SGI26791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6125[] = {SGI26792, SGI26793, SGI26794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26795, SGI0, SGI0, SGI0, SGI26796, SGI26796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6126[] = {SGI26797, SGI26798, SGI26799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26800, SGI0, SGI0, SGI0, SGI26801, SGI26801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6127[] = {SGI26802, SGI26803, SGI26804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26805, SGI0, SGI0, SGI0, SGI26806, SGI26806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6128[] = {SGI26807, SGI26808, SGI26809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26810, SGI0, SGI0, SGI0, SGI26811, SGI26811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6129[] = {SGI26812, SGI26813, SGI26814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26815, SGI0, SGI0, SGI0, SGI26816, SGI26816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6130[] = {SGI26817, SGI26818, SGI26819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26820, SGI0, SGI0, SGI0, SGI26821, SGI26821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6131[] = {SGI26822, SGI0, SGI26823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6132[] = {SGI26824, SGI26825, SGI26826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26827, SGI26828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6133[] = {SGI26829, SGI26830, SGI26831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26832, SGI26833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6134[] = {SGI26834, SGI26835, SGI26836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26837, SGI26837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6135[] = {SGI26838, SGI26839, SGI26840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26841, SGI0, SGI0, SGI0, SGI26842, SGI26842, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6136[] = {SGI26843, SGI26844, SGI26845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26846, SGI0, SGI0, SGI0, SGI26847, SGI26847, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6137[] = {SGI26848, SGI26849, SGI26850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26851, SGI0, SGI0, SGI0, SGI26852, SGI26852, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6138[] = {SGI26853, SGI26854, SGI26855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26856, SGI0, SGI0, SGI0, SGI26857, SGI26857, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6139[] = {SGI26858, SGI26859, SGI26860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26861, SGI0, SGI0, SGI0, SGI26862, SGI26862, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6140[] = {SGI26863, SGI26864, SGI26865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26866, SGI0, SGI0, SGI0, SGI26867, SGI26867, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6141[] = {SGI26868, SGI26869, SGI26870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26871, SGI0, SGI0, SGI0, SGI26872, SGI26872, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6142[] = {SGI26873, SGI26874, SGI26875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26876, SGI0, SGI0, SGI0, SGI26877, SGI26877, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6143[] = {SGI0, SGI26878, SGI26879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26880, SGI0, SGI0, SGI0, SGI0, SGI26881, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6144[] = {SGI26882, SGI26883, SGI26884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26885, SGI0, SGI0, SGI0, SGI26886, SGI26886, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6145[] = {SGI26887, SGI26888, SGI26889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26890, SGI0, SGI0, SGI0, SGI26891, SGI26891, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6146[] = {SGI26892, SGI26893, SGI26894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26895, SGI0, SGI0, SGI0, SGI26896, SGI26897, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6147[] = {SGI26898, SGI26899, SGI26900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26901, SGI0, SGI0, SGI0, SGI26902, SGI26902, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6148[] = {SGI26903, SGI26904, SGI26905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26906, SGI0, SGI0, SGI0, SGI26907, SGI26907, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6149[] = {SGI26908, SGI0, SGI26909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26910, SGI0, SGI0, SGI0, SGI26911, SGI26911, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6150[] = {SGI26912, SGI26913, SGI26914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26915, SGI0, SGI0, SGI0, SGI26916, SGI26916, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6151[] = {SGI26917, SGI26918, SGI26919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26920, SGI26921, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6152[] = {SGI26922, SGI26923, SGI26924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26925, SGI0, SGI0, SGI0, SGI26926, SGI26926, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6153[] = {SGI26927, SGI26928, SGI26929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26930, SGI0, SGI0, SGI0, SGI26931, SGI26931, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6154[] = {SGI26932, SGI26933, SGI26934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26935, SGI0, SGI0, SGI0, SGI26936, SGI26936, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6155[] = {SGI26937, SGI26938, SGI26939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26940, SGI0, SGI0, SGI0, SGI26941, SGI26941, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6156[] = {SGI26942, SGI26943, SGI26944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26945, SGI0, SGI0, SGI0, SGI26946, SGI26946, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6157[] = {SGI26947, SGI26948, SGI26949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26950, SGI0, SGI0, SGI0, SGI26951, SGI26951, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6158[] = {SGI0, SGI26952, SGI26953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26954, SGI0, SGI0, SGI0, SGI0, SGI26955, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6159[] = {SGI26956, SGI26957, SGI26958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26959, SGI0, SGI0, SGI0, SGI26960, SGI26960, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6160[] = {SGI26961, SGI26962, SGI26963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26964, SGI0, SGI0, SGI0, SGI26965, SGI26965, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6161[] = {SGI26966, SGI26967, SGI26968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26969, SGI0, SGI0, SGI0, SGI26970, SGI26970, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6162[] = {SGI26971, SGI26972, SGI26973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26974, SGI0, SGI0, SGI0, SGI26975, SGI26975, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6163[] = {SGI26976, SGI26977, SGI26978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26979, SGI0, SGI0, SGI0, SGI26980, SGI26980, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6164[] = {SGI26981, SGI26982, SGI26983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26984, SGI0, SGI0, SGI0, SGI26985, SGI26985, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6165[] = {SGI26986, SGI26987, SGI26988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26989, SGI0, SGI0, SGI0, SGI26990, SGI26990, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6166[] = {SGI26991, SGI26992, SGI26993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26994, SGI0, SGI0, SGI0, SGI26995, SGI26995, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6167[] = {SGI26996, SGI26997, SGI26998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI26999, SGI0, SGI0, SGI0, SGI27000, SGI27000, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6168[] = {SGI27001, SGI27002, SGI27003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27004, SGI0, SGI0, SGI0, SGI27005, SGI27005, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6169[] = {SGI27006, SGI27007, SGI27008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27009, SGI0, SGI0, SGI0, SGI27010, SGI27010, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6170[] = {SGI27011, SGI27012, SGI27013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27014, SGI0, SGI0, SGI0, SGI27015, SGI27015, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6171[] = {SGI27016, SGI27017, SGI27018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27019, SGI0, SGI0, SGI0, SGI27020, SGI27020, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6172[] = {SGI27021, SGI27022, SGI27023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27024, SGI0, SGI0, SGI0, SGI27025, SGI27025, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6173[] = {SGI27026, SGI27027, SGI27028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27029, SGI0, SGI0, SGI0, SGI27030, SGI27030, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6174[] = {SGI27031, SGI27032, SGI27033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27034, SGI0, SGI0, SGI0, SGI27035, SGI27035, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6175[] = {SGI27036, SGI27037, SGI27038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27039, SGI0, SGI0, SGI0, SGI27040, SGI27040, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6176[] = {SGI27041, SGI27042, SGI27043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27044, SGI0, SGI0, SGI0, SGI27045, SGI27045, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6177[] = {SGI27046, SGI27047, SGI27048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27049, SGI0, SGI0, SGI0, SGI27050, SGI27050, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6178[] = {SGI27051, SGI27052, SGI27053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27054, SGI0, SGI0, SGI0, SGI27055, SGI27055, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6179[] = {SGI27056, SGI27057, SGI27058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27059, SGI0, SGI0, SGI0, SGI27060, SGI27060, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6180[] = {SGI27061, SGI27062, SGI27063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27064, SGI0, SGI0, SGI0, SGI27065, SGI27065, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6181[] = {SGI27066, SGI27067, SGI27068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27069, SGI0, SGI0, SGI0, SGI27070, SGI27070, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6182[] = {SGI27071, SGI27072, SGI27073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27074, SGI0, SGI0, SGI0, SGI27075, SGI27075, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6183[] = {SGI0, SGI27076, SGI27077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27078, SGI0, SGI0, SGI0, SGI0, SGI27079, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6184[] = {SGI27080, SGI27081, SGI27082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27083, SGI0, SGI0, SGI0, SGI27084, SGI27084, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6185[] = {SGI27085, SGI27086, SGI27087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27088, SGI0, SGI0, SGI0, SGI27089, SGI27089, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6186[] = {SGI27090, SGI27091, SGI27092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27093, SGI0, SGI0, SGI0, SGI27094, SGI27094, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6187[] = {SGI27095, SGI27096, SGI27097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27098, SGI0, SGI0, SGI0, SGI27099, SGI27099, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6188[] = {SGI27100, SGI27101, SGI27102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27103, SGI0, SGI0, SGI0, SGI27104, SGI27104, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6189[] = {SGI27105, SGI27106, SGI27107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27108, SGI0, SGI0, SGI0, SGI27109, SGI27109, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6190[] = {SGI27110, SGI27111, SGI27112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27113, SGI0, SGI0, SGI0, SGI27114, SGI27114, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6191[] = {SGI27115, SGI27116, SGI27117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27118, SGI0, SGI0, SGI0, SGI27119, SGI27119, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6192[] = {SGI27120, SGI27121, SGI27122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27123, SGI0, SGI0, SGI0, SGI27124, SGI27124, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6193[] = {SGI27125, SGI27126, SGI27127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27128, SGI27129, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6194[] = {SGI27130, SGI27131, SGI27132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27133, SGI0, SGI0, SGI0, SGI27134, SGI27134, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6195[] = {SGI27135, SGI27136, SGI27137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27138, SGI0, SGI0, SGI0, SGI27139, SGI27139, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6196[] = {SGI27140, SGI27141, SGI27142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27143, SGI0, SGI0, SGI0, SGI27144, SGI27144, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6197[] = {SGI27145, SGI27146, SGI27147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27148, SGI27149, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6198[] = {SGI27150, SGI27151, SGI27152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27153, SGI0, SGI0, SGI0, SGI27154, SGI27154, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6199[] = {SGI27155, SGI27156, SGI27157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27158, SGI0, SGI0, SGI0, SGI27159, SGI27159, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6200[] = {SGI27160, SGI27161, SGI27162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27163, SGI0, SGI0, SGI0, SGI27164, SGI27164, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6201[] = {SGI27165, SGI27166, SGI27167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27168, SGI0, SGI0, SGI0, SGI27169, SGI27169, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6202[] = {SGI27170, SGI27171, SGI27172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27173, SGI0, SGI0, SGI0, SGI27174, SGI27174, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6203[] = {SGI27175, SGI27176, SGI27177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27178, SGI0, SGI0, SGI0, SGI27179, SGI27179, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6204[] = {SGI27180, SGI27181, SGI27182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27183, SGI0, SGI0, SGI0, SGI27184, SGI27184, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6205[] = {SGI27185, SGI27186, SGI27187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27188, SGI0, SGI0, SGI0, SGI27189, SGI27189, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6206[] = {SGI27190, SGI27191, SGI27192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27193, SGI0, SGI0, SGI0, SGI27194, SGI27194, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6207[] = {SGI27195, SGI27196, SGI27197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27198, SGI0, SGI0, SGI0, SGI27199, SGI27199, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6208[] = {SGI27200, SGI27201, SGI27202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27205, SGI0, SGI0, SGI0, SGI27207, SGI27207, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6209[] = {SGI27203, SGI27204, SGI27206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27208, SGI0, SGI0, SGI0, SGI27209, SGI27209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6210[] = {SGI27210, SGI27211, SGI27212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27213, SGI0, SGI0, SGI0, SGI27214, SGI27214, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6211[] = {SGI27215, SGI27216, SGI27217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27218, SGI0, SGI0, SGI0, SGI27219, SGI27219, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6212[] = {SGI27220, SGI27221, SGI27222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27223, SGI27224, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6213[] = {SGI27225, SGI27226, SGI27227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27228, SGI0, SGI0, SGI0, SGI27229, SGI27229, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6214[] = {SGI27230, SGI27231, SGI27232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27233, SGI0, SGI0, SGI0, SGI27234, SGI27234, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6215[] = {SGI27235, SGI27236, SGI27237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27238, SGI0, SGI0, SGI0, SGI27239, SGI27239, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6216[] = {SGI27240, SGI27241, SGI27242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27243, SGI0, SGI0, SGI0, SGI27244, SGI27244, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6217[] = {SGI0, SGI27245, SGI27246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27247, SGI0, SGI0, SGI0, SGI0, SGI27248, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6218[] = {SGI27249, SGI27250, SGI27251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27252, SGI0, SGI0, SGI0, SGI27253, SGI27253, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6219[] = {SGI0, SGI27254, SGI27255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27256, SGI0, SGI0, SGI0, SGI0, SGI27257, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6220[] = {SGI27258, SGI27259, SGI27260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27261, SGI27261, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6221[] = {SGI27262, SGI27263, SGI27264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27265, SGI0, SGI0, SGI0, SGI27266, SGI27266, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6222[] = {SGI27267, SGI27268, SGI27269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27270, SGI0, SGI0, SGI0, SGI27271, SGI27271, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6223[] = {SGI27272, SGI27273, SGI27274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27275, SGI0, SGI0, SGI0, SGI27276, SGI27276, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6224[] = {SGI27277, SGI27278, SGI27279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27280, SGI27280, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6225[] = {SGI27281, SGI27282, SGI27283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27284, SGI0, SGI0, SGI0, SGI27285, SGI27285, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6226[] = {SGI27286, SGI27287, SGI27288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27289, SGI0, SGI0, SGI0, SGI27290, SGI27290, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6227[] = {SGI27291, SGI27292, SGI27293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27294, SGI27294, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6228[] = {SGI27295, SGI27296, SGI27297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27298, SGI0, SGI0, SGI0, SGI27299, SGI27299, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6229[] = {SGI27300, SGI27301, SGI27302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27303, SGI0, SGI0, SGI0, SGI27304, SGI27304, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6230[] = {SGI27305, SGI27306, SGI27307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27308, SGI0, SGI0, SGI0, SGI27309, SGI27309, SGI0, SGI0, NULL};
const Segment * const SG6231[] = {SGI27310, SGI27311, SGI27312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27313, SGI27314, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6232[] = {SGI27315, SGI0, SGI27316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6233[] = {SGI27317, SGI27318, SGI27319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27320, SGI27321, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6234[] = {SGI0, SGI27322, SGI27323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27324, SGI0, SGI0, SGI0, SGI0, SGI27325, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6235[] = {SGI27326, SGI0, SGI27328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6236[] = {SGI27327, SGI27329, SGI27330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27331, SGI0, SGI0, SGI0, SGI27332, SGI27332, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6237[] = {SGI27333, SGI27334, SGI27335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27336, SGI27337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6238[] = {SGI0, SGI27338, SGI27339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27340, SGI27340, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6239[] = {SGI27341, SGI27343, SGI27344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27348, SGI27348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6240[] = {SGI27342, SGI27345, SGI27346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27347, SGI0, SGI0, SGI0, SGI27349, SGI27349, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6241[] = {SGI27350, SGI27351, SGI27352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27353, SGI0, SGI0, SGI0, SGI27354, SGI27354, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6242[] = {SGI27355, SGI27356, SGI27357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27358, SGI0, SGI0, SGI0, SGI27359, SGI27359, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6243[] = {SGI27360, SGI27361, SGI27362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27363, SGI0, SGI0, SGI0, SGI27364, SGI27364, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6244[] = {SGI0, SGI27365, SGI27366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6245[] = {SGI27367, SGI27368, SGI27369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27370, SGI27370, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6246[] = {SGI27371, SGI0, SGI27372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27373, SGI0, SGI0, SGI0, SGI27374, SGI27374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6247[] = {SGI27375, SGI27376, SGI27377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27378, SGI0, SGI0, SGI0, SGI27379, SGI27379, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6248[] = {SGI27380, SGI27381, SGI27382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27383, SGI0, SGI0, SGI0, SGI27384, SGI27384, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6249[] = {SGI27385, SGI27386, SGI27387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27388, SGI0, SGI0, SGI0, SGI27389, SGI27389, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6250[] = {SGI27390, SGI27391, SGI27392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27393, SGI0, SGI0, SGI0, SGI27394, SGI27394, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6251[] = {SGI27395, SGI27396, SGI27397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27398, SGI0, SGI0, SGI0, SGI27399, SGI27399, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6252[] = {SGI27400, SGI27401, SGI27402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27403, SGI0, SGI0, SGI0, SGI27404, SGI27404, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6253[] = {SGI27405, SGI27406, SGI27407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27408, SGI0, SGI0, SGI0, SGI27409, SGI27409, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6254[] = {SGI27410, SGI27411, SGI27412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27413, SGI0, SGI0, SGI0, SGI27414, SGI27414, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6255[] = {SGI27415, SGI27416, SGI27417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27418, SGI0, SGI0, SGI0, SGI27419, SGI27419, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6256[] = {SGI27420, SGI27421, SGI27422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27423, SGI0, SGI0, SGI0, SGI27424, SGI27424, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6257[] = {SGI27425, SGI27426, SGI27427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27428, SGI0, SGI0, SGI0, SGI27429, SGI27429, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6258[] = {SGI27430, SGI27431, SGI27432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27433, SGI0, SGI0, SGI0, SGI27434, SGI27434, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6259[] = {SGI27435, SGI27436, SGI27437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27438, SGI0, SGI0, SGI0, SGI27439, SGI27439, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6260[] = {SGI27440, SGI27441, SGI27442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27443, SGI0, SGI0, SGI0, SGI27444, SGI27444, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6261[] = {SGI27445, SGI27446, SGI27447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27448, SGI0, SGI0, SGI0, SGI27449, SGI27449, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6262[] = {SGI27450, SGI27451, SGI27452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27453, SGI0, SGI0, SGI0, SGI27454, SGI27454, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6263[] = {SGI27455, SGI27456, SGI27457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27458, SGI0, SGI0, SGI0, SGI27459, SGI27459, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6264[] = {SGI0, SGI27460, SGI27461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27462, SGI0, SGI0, SGI0, SGI0, SGI27463, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6265[] = {SGI27464, SGI27465, SGI27466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27467, SGI0, SGI0, SGI0, SGI27468, SGI27468, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6266[] = {SGI27469, SGI27470, SGI27471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27472, SGI0, SGI0, SGI0, SGI27473, SGI27473, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6267[] = {SGI27474, SGI27475, SGI27476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27477, SGI0, SGI0, SGI0, SGI27478, SGI27478, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6268[] = {SGI0, SGI27479, SGI27480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27481, SGI27481, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6269[] = {SGI0, SGI27482, SGI27483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27484, SGI0, SGI0, SGI0, SGI0, SGI27485, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6270[] = {SGI27486, SGI27487, SGI27488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27489, SGI0, SGI0, SGI0, SGI27490, SGI27490, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6271[] = {SGI27491, SGI27492, SGI27493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27494, SGI0, SGI0, SGI0, SGI27495, SGI27495, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6272[] = {SGI27496, SGI27497, SGI27498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27499, SGI0, SGI0, SGI0, SGI27500, SGI27500, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6273[] = {SGI27501, SGI27502, SGI27503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27504, SGI0, SGI0, SGI0, SGI27505, SGI27505, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6274[] = {SGI27506, SGI27507, SGI27508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27509, SGI0, SGI0, SGI0, SGI27510, SGI27510, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6275[] = {SGI27511, SGI27512, SGI27513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27514, SGI0, SGI0, SGI0, SGI27515, SGI27515, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6276[] = {SGI27516, SGI27517, SGI27518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27519, SGI0, SGI0, SGI0, SGI27520, SGI27520, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6277[] = {SGI27521, SGI27522, SGI27523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27524, SGI27524, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6278[] = {SGI27525, SGI0, SGI27526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6279[] = {SGI27527, SGI27528, SGI27529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27530, SGI0, SGI0, SGI0, SGI27531, SGI27531, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6280[] = {SGI27532, SGI27533, SGI27534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27535, SGI0, SGI0, SGI0, SGI27536, SGI27536, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6281[] = {SGI27537, SGI27538, SGI27539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27540, SGI0, SGI0, SGI0, SGI27541, SGI27541, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6282[] = {SGI27542, SGI27543, SGI27544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27545, SGI0, SGI0, SGI0, SGI27546, SGI27546, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6283[] = {SGI27547, SGI27548, SGI27549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27550, SGI0, SGI0, SGI0, SGI27551, SGI27551, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6284[] = {SGI27552, SGI27553, SGI27554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27555, SGI0, SGI0, SGI0, SGI27556, SGI27556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6285[] = {SGI27557, SGI27558, SGI27559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27560, SGI0, SGI0, SGI0, SGI27561, SGI27561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6286[] = {SGI27562, SGI27563, SGI27564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27565, SGI0, SGI0, SGI0, SGI27566, SGI27566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6287[] = {SGI27567, SGI27568, SGI27569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27570, SGI0, SGI0, SGI0, SGI27571, SGI27571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6288[] = {SGI27572, SGI27573, SGI27574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27575, SGI0, SGI0, SGI0, SGI27576, SGI27576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6289[] = {SGI27577, SGI27578, SGI27579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27580, SGI0, SGI0, SGI0, SGI27581, SGI27581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6290[] = {SGI27582, SGI27583, SGI27584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27585, SGI0, SGI0, SGI0, SGI27586, SGI27586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6291[] = {SGI27587, SGI27588, SGI27589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27590, SGI0, SGI0, SGI0, SGI27591, SGI27591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6292[] = {SGI27592, SGI27593, SGI27594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27595, SGI0, SGI0, SGI0, SGI27596, SGI27596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6293[] = {SGI27597, SGI27598, SGI27599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27600, SGI0, SGI0, SGI0, SGI27601, SGI27601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6294[] = {SGI27602, SGI27603, SGI27604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27605, SGI0, SGI0, SGI0, SGI27606, SGI27606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6295[] = {SGI27607, SGI27608, SGI27609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27610, SGI0, SGI0, SGI0, SGI27611, SGI27611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6296[] = {SGI27612, SGI27613, SGI27614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27615, SGI0, SGI0, SGI0, SGI27616, SGI27616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6297[] = {SGI27617, SGI27618, SGI27619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27620, SGI0, SGI0, SGI0, SGI27621, SGI27621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6298[] = {SGI27622, SGI27623, SGI27624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27625, SGI0, SGI0, SGI0, SGI27626, SGI27626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6299[] = {SGI27627, SGI27628, SGI27629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27630, SGI0, SGI0, SGI0, SGI27631, SGI27631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6300[] = {SGI27632, SGI27633, SGI27634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27635, SGI0, SGI0, SGI0, SGI27636, SGI27636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6301[] = {SGI27637, SGI27638, SGI27639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27640, SGI0, SGI0, SGI0, SGI27641, SGI27641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6302[] = {SGI27642, SGI27643, SGI27644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27645, SGI0, SGI0, SGI0, SGI27646, SGI27646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6303[] = {SGI27647, SGI27648, SGI27649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27650, SGI0, SGI0, SGI0, SGI27651, SGI27651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6304[] = {SGI27652, SGI27653, SGI27654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27655, SGI0, SGI0, SGI0, SGI27656, SGI27656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6305[] = {SGI27657, SGI27658, SGI27659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27660, SGI27661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6306[] = {SGI27662, SGI27663, SGI27664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27665, SGI0, SGI0, SGI0, SGI27666, SGI27666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6307[] = {SGI27667, SGI27668, SGI27669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27670, SGI0, SGI0, SGI0, SGI27671, SGI27671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6308[] = {SGI27672, SGI27673, SGI27674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27675, SGI0, SGI0, SGI0, SGI27676, SGI27676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6309[] = {SGI27677, SGI0, SGI27678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6310[] = {SGI27679, SGI27680, SGI27681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27682, SGI0, SGI0, SGI0, SGI27683, SGI27683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6311[] = {SGI27684, SGI27685, SGI27686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27687, SGI27688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6312[] = {SGI27689, SGI27690, SGI27691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27692, SGI27692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6313[] = {SGI27693, SGI27694, SGI27695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27696, SGI0, SGI0, SGI0, SGI27697, SGI27697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6314[] = {SGI27698, SGI27699, SGI27700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27701, SGI27702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6315[] = {SGI27703, SGI27704, SGI27705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27706, SGI27706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6316[] = {SGI27707, SGI0, SGI27708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27709, SGI0, SGI0, SGI0, SGI27710, SGI27710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6317[] = {SGI27711, SGI27712, SGI27713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27714, SGI0, SGI0, SGI0, SGI27715, SGI27715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6318[] = {SGI27716, SGI27717, SGI27718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27719, SGI0, SGI0, SGI0, SGI27720, SGI27720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6319[] = {SGI27721, SGI27722, SGI27723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27724, SGI0, SGI0, SGI0, SGI27725, SGI27725, SGI0, SGI0, NULL};
const Segment * const SG6320[] = {SGI27726, SGI27727, SGI27728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27729, SGI0, SGI0, SGI0, SGI27730, SGI27730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6321[] = {SGI27731, SGI27732, SGI27733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27734, SGI0, SGI0, SGI0, SGI27735, SGI27735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6322[] = {SGI27736, SGI27737, SGI27738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27739, SGI0, SGI0, SGI0, SGI27740, SGI27740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6323[] = {SGI27741, SGI27742, SGI27743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27744, SGI0, SGI0, SGI0, SGI27745, SGI27745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6324[] = {SGI27746, SGI27747, SGI27748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27749, SGI0, SGI0, SGI0, SGI27750, SGI27750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6325[] = {SGI27751, SGI27752, SGI27753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27754, SGI0, SGI0, SGI0, SGI27755, SGI27755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6326[] = {SGI27756, SGI27757, SGI27758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27759, SGI0, SGI0, SGI0, SGI27760, SGI27760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6327[] = {SGI27761, SGI27762, SGI27763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27764, SGI0, SGI0, SGI0, SGI27765, SGI27765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6328[] = {SGI27766, SGI27767, SGI27768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27769, SGI0, SGI0, SGI0, SGI27770, SGI27770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6329[] = {SGI27771, SGI27772, SGI27773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27774, SGI0, SGI0, SGI0, SGI27775, SGI27775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6330[] = {SGI27776, SGI0, SGI27777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6331[] = {SGI27778, SGI27779, SGI27780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27781, SGI0, SGI0, SGI0, SGI27782, SGI27782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6332[] = {SGI27783, SGI27784, SGI27785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27786, SGI0, SGI0, SGI0, SGI27787, SGI27787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6333[] = {SGI27788, SGI0, SGI27789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6334[] = {SGI27790, SGI27791, SGI27792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27793, SGI27794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6335[] = {SGI0, SGI0, SGI27795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27796, SGI0, SGI0, SGI0, SGI27797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27798, SGI0, SGI0, NULL};
const Segment * const SG6336[] = {SGI27799, SGI0, SGI27800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27801, SGI0, SGI0, SGI0, SGI27802, SGI27802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6337[] = {SGI27803, SGI27804, SGI27805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27806, SGI0, SGI0, SGI0, SGI27807, SGI27807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6338[] = {SGI27808, SGI27809, SGI27810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27811, SGI0, SGI0, SGI0, SGI27812, SGI27812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6339[] = {SGI27813, SGI27814, SGI27815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27816, SGI0, SGI0, SGI0, SGI27817, SGI27817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6340[] = {SGI27818, SGI27819, SGI27820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27821, SGI0, SGI0, SGI0, SGI27822, SGI27822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6341[] = {SGI27823, SGI27824, SGI27825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27826, SGI0, SGI0, SGI0, SGI27827, SGI27827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6342[] = {SGI27828, SGI0, SGI27829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6343[] = {SGI27830, SGI27831, SGI27832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27833, SGI0, SGI0, SGI0, SGI27834, SGI27834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6344[] = {SGI0, SGI0, SGI27835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27836, SGI0, SGI0, SGI0, SGI27837, SGI27837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6345[] = {SGI27838, SGI0, SGI27839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6346[] = {SGI27840, SGI27841, SGI27842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27843, SGI0, SGI0, SGI0, SGI27844, SGI27844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6347[] = {SGI27845, SGI27846, SGI27847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27848, SGI0, SGI0, SGI0, SGI27849, SGI27849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6348[] = {SGI27850, SGI27851, SGI27852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27853, SGI0, SGI0, SGI0, SGI27854, SGI27854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6349[] = {SGI27855, SGI0, SGI27856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6350[] = {SGI27857, SGI27858, SGI27859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27860, SGI0, SGI0, SGI0, SGI27861, SGI27861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6351[] = {SGI27862, SGI27863, SGI27864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27865, SGI0, SGI0, SGI0, SGI27866, SGI27866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6352[] = {SGI27867, SGI27868, SGI27869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27870, SGI0, SGI0, SGI0, SGI27871, SGI27871, SGI0, SGI0, NULL};
const Segment * const SG6353[] = {SGI27872, SGI27873, SGI27874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27875, SGI0, SGI0, SGI0, SGI27876, SGI27876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6354[] = {SGI27877, SGI27878, SGI27879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27880, SGI0, SGI0, SGI0, SGI27881, SGI27881, SGI0, SGI0, NULL};
const Segment * const SG6355[] = {SGI27882, SGI27883, SGI27884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27885, SGI0, SGI0, SGI0, SGI27886, SGI27886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6356[] = {SGI27887, SGI27888, SGI27889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27890, SGI0, SGI0, SGI0, SGI27891, SGI27891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6357[] = {SGI27892, SGI27893, SGI27894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27895, SGI0, SGI0, SGI0, SGI27896, SGI27896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6358[] = {SGI27897, SGI27898, SGI27899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27900, SGI0, SGI0, SGI0, SGI27901, SGI27901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6359[] = {SGI27902, SGI27903, SGI27904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27905, SGI0, SGI0, SGI0, SGI27906, SGI27906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6360[] = {SGI27907, SGI27908, SGI27909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27910, SGI0, SGI0, SGI0, SGI27911, SGI27911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6361[] = {SGI27912, SGI27913, SGI27914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27915, SGI0, SGI0, SGI0, SGI27916, SGI27916, SGI0, SGI0, NULL};
const Segment * const SG6362[] = {SGI27917, SGI27918, SGI27919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27920, SGI0, SGI0, SGI0, SGI27921, SGI27921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6363[] = {SGI27922, SGI27923, SGI27924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27925, SGI0, SGI0, SGI0, SGI27926, SGI27926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6364[] = {SGI27927, SGI27928, SGI27929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27931, SGI0, SGI0, SGI0, SGI27934, SGI27934, SGI0, SGI0, NULL};
const Segment * const SG6365[] = {SGI27930, SGI27932, SGI27933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27935, SGI0, SGI0, SGI0, SGI27936, SGI27936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6366[] = {SGI27937, SGI27938, SGI27939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27940, SGI0, SGI0, SGI0, SGI27941, SGI27941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6367[] = {SGI27942, SGI27943, SGI27944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27945, SGI0, SGI0, SGI0, SGI27946, SGI27946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6368[] = {SGI27947, SGI27948, SGI27949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27950, SGI0, SGI0, SGI0, SGI27951, SGI27951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6369[] = {SGI27952, SGI0, SGI27953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6370[] = {SGI27954, SGI27955, SGI27956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27957, SGI0, SGI0, SGI0, SGI27958, SGI27958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6371[] = {SGI27959, SGI27960, SGI27961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27962, SGI0, SGI0, SGI0, SGI27963, SGI27963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6372[] = {SGI27964, SGI27965, SGI27966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27967, SGI0, SGI0, SGI0, SGI27968, SGI27968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6373[] = {SGI27969, SGI27970, SGI27971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27972, SGI0, SGI0, SGI0, SGI27973, SGI27973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6374[] = {SGI27974, SGI27975, SGI27976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27977, SGI0, SGI0, SGI0, SGI27978, SGI27978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6375[] = {SGI27979, SGI27980, SGI27981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27982, SGI0, SGI0, SGI0, SGI27983, SGI27983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6376[] = {SGI27984, SGI27985, SGI27986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27987, SGI27988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6377[] = {SGI27989, SGI27990, SGI27991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27992, SGI0, SGI0, SGI0, SGI27993, SGI27993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6378[] = {SGI27994, SGI27995, SGI27996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI27997, SGI0, SGI0, SGI0, SGI27998, SGI27998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6379[] = {SGI27999, SGI28000, SGI28001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28002, SGI0, SGI0, SGI0, SGI28003, SGI28003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6380[] = {SGI28004, SGI28005, SGI28006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28007, SGI28007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6381[] = {SGI28008, SGI28009, SGI28010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28011, SGI0, SGI0, SGI0, SGI28012, SGI28012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6382[] = {SGI28013, SGI28014, SGI28015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28016, SGI28017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6383[] = {SGI28018, SGI28019, SGI28020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28021, SGI28022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6384[] = {SGI28023, SGI28024, SGI28025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28026, SGI0, SGI0, SGI0, SGI28027, SGI28027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6385[] = {SGI28028, SGI28029, SGI28030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28031, SGI0, SGI0, SGI0, SGI28032, SGI28032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6386[] = {SGI28033, SGI28034, SGI28035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28036, SGI28036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6387[] = {SGI28037, SGI0, SGI28038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28039, SGI0, SGI0, SGI0, SGI28040, SGI28040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6388[] = {SGI28041, SGI28042, SGI28043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28044, SGI0, SGI0, SGI0, SGI28045, SGI28045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6389[] = {SGI28046, SGI0, SGI28047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6390[] = {SGI0, SGI0, SGI28048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28049, SGI0, SGI0, SGI0, SGI28050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28051, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6391[] = {SGI0, SGI0, SGI28052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28053, SGI0, SGI0, SGI0, SGI28054, SGI28054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6392[] = {SGI28055, SGI28056, SGI28057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28058, SGI0, SGI0, SGI0, SGI28059, SGI28059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6393[] = {SGI28060, SGI28061, SGI28062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28063, SGI0, SGI0, SGI0, SGI28064, SGI28064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6394[] = {SGI28065, SGI28066, SGI28067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28068, SGI0, SGI0, SGI0, SGI28069, SGI28069, SGI0, SGI0, NULL};
const Segment * const SG6395[] = {SGI0, SGI0, SGI28070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28071, SGI0, SGI0, SGI0, SGI28072, SGI28072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6396[] = {SGI28073, SGI0, SGI28074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28075, SGI0, SGI0, SGI0, SGI28076, SGI28076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6397[] = {SGI28077, SGI0, SGI28078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28079, SGI0, SGI0, SGI0, SGI28080, SGI28080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6398[] = {SGI0, SGI0, SGI28081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28082, SGI0, SGI0, SGI0, SGI28083, SGI28083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6399[] = {SGI0, SGI0, SGI28084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28085, SGI0, SGI0, SGI0, SGI28086, SGI28086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6400[] = {SGI0, SGI0, SGI28087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28088, SGI0, SGI0, SGI0, SGI28089, SGI28089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6401[] = {SGI28090, SGI28091, SGI28092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28093, SGI0, SGI0, SGI0, SGI28094, SGI28094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6402[] = {SGI28095, SGI28096, SGI28097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28098, SGI0, SGI0, SGI0, SGI28099, SGI28099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6403[] = {SGI28100, SGI28101, SGI28102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28103, SGI0, SGI0, SGI0, SGI28104, SGI28104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6404[] = {SGI28105, SGI28106, SGI28107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28108, SGI0, SGI0, SGI0, SGI28109, SGI28109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6405[] = {SGI28110, SGI28111, SGI28112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28113, SGI0, SGI0, SGI0, SGI28114, SGI28114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6406[] = {SGI28115, SGI28116, SGI28117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28118, SGI0, SGI0, SGI0, SGI28119, SGI28119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6407[] = {SGI28120, SGI28121, SGI28122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28123, SGI0, SGI0, SGI0, SGI28124, SGI28124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6408[] = {SGI28125, SGI28126, SGI28127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28128, SGI0, SGI0, SGI0, SGI28129, SGI28129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6409[] = {SGI28130, SGI28131, SGI28132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28133, SGI0, SGI0, SGI0, SGI28134, SGI28134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6410[] = {SGI28135, SGI28136, SGI28137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28138, SGI0, SGI0, SGI0, SGI28139, SGI28139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6411[] = {SGI28140, SGI28141, SGI28142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28143, SGI0, SGI0, SGI0, SGI28144, SGI28144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6412[] = {SGI28145, SGI28146, SGI28147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28148, SGI0, SGI0, SGI0, SGI28149, SGI28149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6413[] = {SGI28150, SGI28151, SGI28152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28153, SGI0, SGI0, SGI0, SGI28154, SGI28154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6414[] = {SGI28155, SGI28156, SGI28157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28158, SGI0, SGI0, SGI0, SGI28159, SGI28159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6415[] = {SGI28160, SGI28161, SGI28162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28163, SGI0, SGI0, SGI0, SGI28164, SGI28164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6416[] = {SGI28165, SGI28166, SGI28167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28168, SGI0, SGI0, SGI0, SGI28169, SGI28169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6417[] = {SGI28170, SGI28171, SGI28172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28173, SGI28174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6418[] = {SGI28175, SGI0, SGI28176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6419[] = {SGI28177, SGI0, SGI28178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6420[] = {SGI0, SGI0, SGI28179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6421[] = {SGI28180, SGI28181, SGI28182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28183, SGI0, SGI0, SGI0, SGI28184, SGI28184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6422[] = {SGI28185, SGI0, SGI28186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6423[] = {SGI28187, SGI28188, SGI28189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28190, SGI0, SGI0, SGI0, SGI28191, SGI28191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6424[] = {SGI28192, SGI28193, SGI28194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28195, SGI0, SGI0, SGI0, SGI28196, SGI28196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6425[] = {SGI28197, SGI28198, SGI28199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28200, SGI0, SGI0, SGI0, SGI28201, SGI28201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6426[] = {SGI28202, SGI28203, SGI28204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28205, SGI0, SGI0, SGI0, SGI28206, SGI28206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6427[] = {SGI28207, SGI28208, SGI28209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28210, SGI0, SGI0, SGI0, SGI28211, SGI28211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6428[] = {SGI28212, SGI28213, SGI28214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28215, SGI0, SGI0, SGI0, SGI28216, SGI28216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6429[] = {SGI28217, SGI28218, SGI28219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28220, SGI0, SGI0, SGI0, SGI28221, SGI28221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6430[] = {SGI28222, SGI28223, SGI28224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28225, SGI0, SGI0, SGI0, SGI28226, SGI28226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6431[] = {SGI0, SGI0, SGI28227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28228, SGI0, SGI0, SGI0, SGI28229, SGI28229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6432[] = {SGI28230, SGI0, SGI28231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6433[] = {SGI0, SGI0, SGI28232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28233, SGI0, SGI0, SGI0, SGI28234, SGI28234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6434[] = {SGI28235, SGI28236, SGI28237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28238, SGI0, SGI0, SGI0, SGI28239, SGI28239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6435[] = {SGI28240, SGI28241, SGI28242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28243, SGI0, SGI0, SGI0, SGI28244, SGI28244, SGI0, SGI0, NULL};
const Segment * const SG6436[] = {SGI28245, SGI28246, SGI28247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28248, SGI0, SGI0, SGI0, SGI28249, SGI28249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6437[] = {SGI28250, SGI28251, SGI28252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28253, SGI0, SGI0, SGI0, SGI28254, SGI28254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6438[] = {SGI28255, SGI28256, SGI28257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28258, SGI0, SGI0, SGI0, SGI28259, SGI28259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6439[] = {SGI0, SGI28260, SGI28261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28262, SGI0, SGI0, SGI0, SGI28263, SGI28263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6440[] = {SGI28264, SGI28265, SGI28266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28267, SGI0, SGI0, SGI0, SGI28268, SGI28268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6441[] = {SGI28269, SGI28270, SGI28271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28272, SGI0, SGI0, SGI0, SGI28273, SGI28273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6442[] = {SGI28274, SGI28275, SGI28276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28277, SGI0, SGI0, SGI0, SGI28278, SGI28278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6443[] = {SGI28279, SGI0, SGI28280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6444[] = {SGI28281, SGI28282, SGI28283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28284, SGI0, SGI0, SGI0, SGI28285, SGI28285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6445[] = {SGI28286, SGI28287, SGI28288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28289, SGI0, SGI0, SGI0, SGI28290, SGI28290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6446[] = {SGI28291, SGI28292, SGI28293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28294, SGI0, SGI0, SGI0, SGI28295, SGI28295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6447[] = {SGI28296, SGI28297, SGI28298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28299, SGI0, SGI0, SGI0, SGI28300, SGI28300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6448[] = {SGI28301, SGI28302, SGI28303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28304, SGI0, SGI0, SGI0, SGI28305, SGI28305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6449[] = {SGI28306, SGI28307, SGI28308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28309, SGI0, SGI0, SGI0, SGI28310, SGI28310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6450[] = {SGI28311, SGI28312, SGI28313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28314, SGI0, SGI0, SGI0, SGI28315, SGI28315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6451[] = {SGI28316, SGI28317, SGI28318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28319, SGI0, SGI0, SGI0, SGI28320, SGI28320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6452[] = {SGI28321, SGI28322, SGI28323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28324, SGI0, SGI0, SGI0, SGI28325, SGI28325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6453[] = {SGI28326, SGI28327, SGI28328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28329, SGI0, SGI0, SGI0, SGI28330, SGI28330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6454[] = {SGI28331, SGI28332, SGI28333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28334, SGI0, SGI0, SGI0, SGI28335, SGI28335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6455[] = {SGI28336, SGI28337, SGI28338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28339, SGI0, SGI0, SGI0, SGI28340, SGI28340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6456[] = {SGI28341, SGI28342, SGI28343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28344, SGI0, SGI0, SGI0, SGI28345, SGI28345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6457[] = {SGI28346, SGI28347, SGI28348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28349, SGI0, SGI0, SGI0, SGI28350, SGI28350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6458[] = {SGI28351, SGI0, SGI28352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6459[] = {SGI28353, SGI28354, SGI28355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28356, SGI0, SGI0, SGI0, SGI28357, SGI28357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6460[] = {SGI28358, SGI0, SGI28359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6461[] = {SGI28360, SGI28361, SGI28362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28363, SGI0, SGI0, SGI0, SGI28364, SGI28364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6462[] = {SGI28365, SGI28366, SGI28367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28368, SGI0, SGI0, SGI0, SGI28369, SGI28369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6463[] = {SGI28370, SGI28371, SGI28372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28373, SGI0, SGI0, SGI0, SGI28374, SGI28374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6464[] = {SGI28375, SGI28376, SGI28377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28378, SGI0, SGI0, SGI0, SGI28379, SGI28379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6465[] = {SGI28380, SGI28381, SGI28382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28383, SGI0, SGI0, SGI0, SGI28384, SGI28384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6466[] = {SGI28385, SGI28386, SGI28387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28388, SGI28388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6467[] = {SGI28389, SGI28390, SGI28391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28392, SGI0, SGI0, SGI0, SGI28393, SGI28393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6468[] = {SGI28394, SGI0, SGI28395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6469[] = {SGI28396, SGI28397, SGI28398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28399, SGI0, SGI0, SGI0, SGI28400, SGI28400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6470[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6471[] = {SGI0, SGI0, SGI28402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28403, SGI0, SGI0, SGI0, SGI28404, SGI28404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6472[] = {SGI28405, SGI28406, SGI28407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28408, SGI0, SGI0, SGI0, SGI28409, SGI28409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6473[] = {SGI0, SGI28410, SGI28411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28412, SGI0, SGI0, SGI0, SGI0, SGI28413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6474[] = {SGI28414, SGI28415, SGI28416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28417, SGI0, SGI0, SGI0, SGI28418, SGI28418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6475[] = {SGI28419, SGI28420, SGI28421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28422, SGI0, SGI0, SGI0, SGI28423, SGI28423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6476[] = {SGI28424, SGI28425, SGI28426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28427, SGI0, SGI0, SGI0, SGI28428, SGI28428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6477[] = {SGI28429, SGI28430, SGI28431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28432, SGI0, SGI0, SGI0, SGI28433, SGI28433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6478[] = {SGI0, SGI28434, SGI28435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28436, SGI0, SGI0, SGI0, SGI28437, SGI28437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6479[] = {SGI28438, SGI28439, SGI28440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28441, SGI0, SGI0, SGI0, SGI28442, SGI28442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6480[] = {SGI28443, SGI28444, SGI28445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28446, SGI0, SGI0, SGI0, SGI28447, SGI28447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6481[] = {SGI28448, SGI28449, SGI28450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28451, SGI0, SGI0, SGI0, SGI28452, SGI28452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6482[] = {SGI28453, SGI28454, SGI28455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28456, SGI0, SGI0, SGI0, SGI28457, SGI28457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6483[] = {SGI28458, SGI28459, SGI28460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28461, SGI0, SGI0, SGI0, SGI28462, SGI28462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6484[] = {SGI28463, SGI28464, SGI28465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28466, SGI0, SGI0, SGI0, SGI28467, SGI28467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6485[] = {SGI28468, SGI28469, SGI28470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28471, SGI0, SGI0, SGI0, SGI28472, SGI28472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6486[] = {SGI0, SGI0, SGI28473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6487[] = {SGI28474, SGI28475, SGI28476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28477, SGI0, SGI0, SGI0, SGI28478, SGI28478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6488[] = {SGI28479, SGI0, SGI28480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6489[] = {SGI28481, SGI28482, SGI28483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28484, SGI28484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6490[] = {SGI28485, SGI28486, SGI28487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28488, SGI0, SGI0, SGI0, SGI28489, SGI28489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6491[] = {SGI28490, SGI0, SGI28491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6492[] = {SGI28492, SGI28493, SGI28494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28495, SGI28496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6493[] = {SGI28497, SGI0, SGI28498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28499, SGI0, SGI0, SGI0, SGI28500, SGI28500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6494[] = {SGI28501, SGI28502, SGI28503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28504, SGI0, SGI0, SGI0, SGI28505, SGI28505, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6495[] = {SGI0, SGI28506, SGI28507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28508, SGI0, SGI0, SGI0, SGI28509, SGI28509, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6496[] = {SGI28510, SGI28511, SGI28512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28513, SGI0, SGI0, SGI0, SGI28514, SGI28514, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6497[] = {SGI28515, SGI28516, SGI28517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28518, SGI0, SGI0, SGI0, SGI28519, SGI28519, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6498[] = {SGI28520, SGI28521, SGI28522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28523, SGI0, SGI0, SGI0, SGI28524, SGI28524, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6499[] = {SGI0, SGI28525, SGI28526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28527, SGI0, SGI0, SGI0, SGI28528, SGI28528, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6500[] = {SGI28529, SGI28530, SGI28531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28532, SGI0, SGI0, SGI0, SGI28533, SGI28533, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6501[] = {SGI28534, SGI28535, SGI28536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28537, SGI0, SGI0, SGI0, SGI28538, SGI28538, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6502[] = {SGI28539, SGI28540, SGI28541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28542, SGI0, SGI0, SGI0, SGI28543, SGI28543, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6503[] = {SGI28544, SGI0, SGI28545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28546, SGI0, SGI0, SGI0, SGI28547, SGI28547, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6504[] = {SGI28548, SGI0, SGI28549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6505[] = {SGI28550, SGI28551, SGI28552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28553, SGI0, SGI0, SGI0, SGI28554, SGI28554, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6506[] = {SGI28555, SGI28556, SGI28557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28558, SGI28559, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6507[] = {SGI28560, SGI0, SGI28561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6508[] = {SGI28562, SGI0, SGI28563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6509[] = {SGI28564, SGI28565, SGI28566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28567, SGI0, SGI0, SGI0, SGI28568, SGI28568, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6510[] = {SGI28569, SGI28570, SGI28571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28572, SGI0, SGI0, SGI0, SGI28573, SGI28573, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6511[] = {SGI28574, SGI28575, SGI28576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28577, SGI0, SGI0, SGI0, SGI28578, SGI28578, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6512[] = {SGI28579, SGI28580, SGI28581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28582, SGI0, SGI0, SGI0, SGI28583, SGI28583, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6513[] = {SGI28584, SGI28585, SGI28586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28587, SGI0, SGI0, SGI0, SGI28588, SGI28588, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6514[] = {SGI28589, SGI28590, SGI28591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28592, SGI0, SGI0, SGI0, SGI28593, SGI28593, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6515[] = {SGI28594, SGI28595, SGI28596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28597, SGI0, SGI0, SGI0, SGI28598, SGI28598, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6516[] = {SGI28599, SGI28600, SGI28601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28602, SGI0, SGI0, SGI0, SGI28603, SGI28603, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6517[] = {SGI28604, SGI28605, SGI28606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28607, SGI0, SGI0, SGI0, SGI28608, SGI28608, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6518[] = {SGI28609, SGI28610, SGI28611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28612, SGI0, SGI0, SGI0, SGI28613, SGI28613, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6519[] = {SGI28614, SGI28615, SGI28616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28617, SGI0, SGI0, SGI0, SGI28618, SGI28618, SGI0, SGI0, NULL};
const Segment * const SG6520[] = {SGI28619, SGI28620, SGI28621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28622, SGI0, SGI0, SGI0, SGI28623, SGI28623, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6521[] = {SGI28624, SGI28625, SGI28626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28627, SGI0, SGI0, SGI0, SGI28628, SGI28628, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6522[] = {SGI28629, SGI28630, SGI28631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28632, SGI0, SGI0, SGI0, SGI28633, SGI28633, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6523[] = {SGI28634, SGI28635, SGI28636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28637, SGI0, SGI0, SGI0, SGI28638, SGI28638, SGI0, SGI0, NULL};
const Segment * const SG6524[] = {SGI28639, SGI28640, SGI28641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28642, SGI0, SGI0, SGI0, SGI28643, SGI28643, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6525[] = {SGI28644, SGI28645, SGI28646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28647, SGI0, SGI0, SGI0, SGI28648, SGI28648, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6526[] = {SGI28649, SGI28650, SGI28651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28652, SGI0, SGI0, SGI0, SGI28653, SGI28653, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6527[] = {SGI28654, SGI28655, SGI28656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28657, SGI0, SGI0, SGI0, SGI28658, SGI28658, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6528[] = {SGI28659, SGI28660, SGI28661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28662, SGI0, SGI0, SGI0, SGI28663, SGI28663, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6529[] = {SGI28664, SGI28665, SGI28666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28667, SGI0, SGI0, SGI0, SGI28668, SGI28669, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6530[] = {SGI28670, SGI28671, SGI28672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28673, SGI0, SGI0, SGI0, SGI28674, SGI28674, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6531[] = {SGI0, SGI0, SGI28675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28676, SGI0, SGI0, SGI0, SGI28677, SGI28677, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6532[] = {SGI28678, SGI28679, SGI28680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28681, SGI0, SGI0, SGI0, SGI28682, SGI28682, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6533[] = {SGI28683, SGI28684, SGI28685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28686, SGI0, SGI0, SGI0, SGI28687, SGI28687, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6534[] = {SGI28688, SGI28689, SGI28690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28691, SGI0, SGI0, SGI0, SGI28692, SGI28692, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6535[] = {SGI28693, SGI28694, SGI28695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28696, SGI0, SGI0, SGI0, SGI28697, SGI28697, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6536[] = {SGI28698, SGI28699, SGI28700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28701, SGI0, SGI0, SGI0, SGI28702, SGI28702, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6537[] = {SGI28703, SGI28704, SGI28705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28706, SGI0, SGI0, SGI0, SGI28707, SGI28707, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6538[] = {SGI0, SGI0, SGI28708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28709, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6539[] = {SGI28710, SGI28711, SGI28712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28713, SGI0, SGI0, SGI0, SGI28714, SGI28714, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6540[] = {SGI28715, SGI28716, SGI28717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28718, SGI0, SGI0, SGI0, SGI28719, SGI28719, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6541[] = {SGI28720, SGI28721, SGI28722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28723, SGI0, SGI0, SGI0, SGI28724, SGI28724, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6542[] = {SGI28725, SGI28726, SGI28727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28728, SGI0, SGI0, SGI0, SGI28729, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6543[] = {SGI28730, SGI28731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28732, SGI0, SGI0, SGI0, SGI28733, SGI28733, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6544[] = {SGI28734, SGI28735, SGI28736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28737, SGI0, SGI0, SGI0, SGI28738, SGI28738, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6545[] = {SGI28739, SGI28740, SGI28741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28742, SGI0, SGI0, SGI0, SGI28743, SGI28743, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6546[] = {SGI28744, SGI0, SGI28745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6547[] = {SGI28746, SGI28747, SGI28748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28749, SGI0, SGI0, SGI0, SGI28750, SGI28750, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6548[] = {SGI28751, SGI28752, SGI28753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28754, SGI0, SGI0, SGI0, SGI28755, SGI28755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6549[] = {SGI28756, SGI28757, SGI28758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28759, SGI0, SGI0, SGI0, SGI28760, SGI28760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6550[] = {SGI28761, SGI28762, SGI28763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28764, SGI0, SGI0, SGI0, SGI28765, SGI28765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6551[] = {SGI28766, SGI28767, SGI28768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28769, SGI0, SGI0, SGI0, SGI28770, SGI28770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6552[] = {SGI28771, SGI28772, SGI28773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28774, SGI0, SGI0, SGI0, SGI28775, SGI28775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6553[] = {SGI28776, SGI28777, SGI28778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28779, SGI0, SGI0, SGI0, SGI28780, SGI28780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6554[] = {SGI28781, SGI28782, SGI28783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28784, SGI0, SGI0, SGI0, SGI28785, SGI28785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6555[] = {SGI28786, SGI0, SGI28787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6556[] = {SGI28788, SGI28789, SGI28790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28791, SGI0, SGI0, SGI0, SGI28792, SGI28792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6557[] = {SGI28793, SGI28794, SGI28795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28796, SGI0, SGI0, SGI0, SGI28797, SGI28797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6558[] = {SGI28798, SGI28799, SGI28800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28801, SGI0, SGI0, SGI0, SGI28802, SGI28802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6559[] = {SGI28803, SGI28804, SGI28805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28806, SGI0, SGI0, SGI0, SGI28807, SGI28807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6560[] = {SGI28808, SGI28809, SGI28810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28811, SGI0, SGI0, SGI0, SGI28812, SGI28812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6561[] = {SGI28813, SGI28814, SGI28815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28816, SGI0, SGI0, SGI0, SGI28817, SGI28817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6562[] = {SGI28818, SGI28819, SGI28820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28821, SGI0, SGI0, SGI0, SGI28822, SGI28822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6563[] = {SGI28823, SGI28824, SGI28825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28826, SGI0, SGI0, SGI0, SGI28827, SGI28827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6564[] = {SGI28828, SGI28829, SGI28830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28831, SGI0, SGI0, SGI0, SGI28832, SGI28832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6565[] = {SGI28833, SGI0, SGI28834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6566[] = {SGI28835, SGI28836, SGI28837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28838, SGI0, SGI0, SGI0, SGI28839, SGI28839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6567[] = {SGI28840, SGI28841, SGI28842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28843, SGI28844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6568[] = {SGI28845, SGI28846, SGI28847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28848, SGI0, SGI0, SGI0, SGI28849, SGI28849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6569[] = {SGI28850, SGI28851, SGI28852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28853, SGI28854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6570[] = {SGI28855, SGI28856, SGI28857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28858, SGI0, SGI0, SGI0, SGI28859, SGI28859, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6571[] = {SGI28860, SGI28861, SGI28862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28863, SGI0, SGI0, SGI0, SGI28864, SGI28864, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6572[] = {SGI28865, SGI28866, SGI28867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28868, SGI0, SGI0, SGI0, SGI28869, SGI28869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6573[] = {SGI28870, SGI28871, SGI28872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28873, SGI0, SGI0, SGI0, SGI28874, SGI28874, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6574[] = {SGI28875, SGI28876, SGI28877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28878, SGI0, SGI0, SGI0, SGI28879, SGI28879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6575[] = {SGI28880, SGI28881, SGI28882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28883, SGI0, SGI0, SGI0, SGI28884, SGI28884, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6576[] = {SGI28885, SGI0, SGI28886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28887, SGI0, SGI0, SGI0, SGI28888, SGI28888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6577[] = {SGI28889, SGI0, SGI28890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28891, SGI0, SGI0, SGI0, SGI28892, SGI28892, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6578[] = {SGI28893, SGI28894, SGI28895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28896, SGI0, SGI0, SGI0, SGI28897, SGI28897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6579[] = {SGI28898, SGI0, SGI28899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6580[] = {SGI28900, SGI0, SGI28901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6581[] = {SGI28902, SGI28903, SGI28904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28905, SGI0, SGI0, SGI0, SGI28906, SGI28906, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6582[] = {SGI28907, SGI28908, SGI28909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28910, SGI0, SGI0, SGI0, SGI28911, SGI28911, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6583[] = {SGI28912, SGI28913, SGI28914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28915, SGI0, SGI0, SGI0, SGI28916, SGI28916, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6584[] = {SGI28917, SGI28918, SGI28919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28920, SGI0, SGI0, SGI0, SGI28921, SGI28921, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6585[] = {SGI28922, SGI28923, SGI28924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28925, SGI0, SGI0, SGI0, SGI28926, SGI28926, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6586[] = {SGI28927, SGI28928, SGI28929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28930, SGI0, SGI0, SGI0, SGI28931, SGI28931, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6587[] = {SGI28932, SGI28933, SGI28934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28935, SGI0, SGI0, SGI0, SGI28936, SGI28937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6588[] = {SGI28938, SGI0, SGI28939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28940, SGI0, SGI0, SGI0, SGI28941, SGI28941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6589[] = {SGI28942, SGI28943, SGI28944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28945, SGI28946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6590[] = {SGI28947, SGI28948, SGI28949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28950, SGI0, SGI0, SGI0, SGI28951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6591[] = {SGI28952, SGI28953, SGI28954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28955, SGI28955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6592[] = {SGI28956, SGI28957, SGI28958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28959, SGI0, SGI0, SGI0, SGI28960, SGI28960, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6593[] = {SGI28961, SGI28962, SGI28963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28964, SGI0, SGI0, SGI0, SGI28965, SGI28965, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6594[] = {SGI28966, SGI28967, SGI28968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28969, SGI0, SGI0, SGI0, SGI28970, SGI28970, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6595[] = {SGI28971, SGI28972, SGI28973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28974, SGI0, SGI0, SGI0, SGI28975, SGI28975, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6596[] = {SGI28976, SGI28977, SGI28978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28979, SGI0, SGI0, SGI0, SGI28980, SGI28980, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6597[] = {SGI28981, SGI0, SGI28982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6598[] = {SGI28983, SGI0, SGI28984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6599[] = {SGI28985, SGI0, SGI28986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6600[] = {SGI28987, SGI28988, SGI28989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28990, SGI0, SGI0, SGI0, SGI28991, SGI28991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6601[] = {SGI28992, SGI28993, SGI28994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI28995, SGI0, SGI0, SGI0, SGI28996, SGI28996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6602[] = {SGI28997, SGI28998, SGI28999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29000, SGI0, SGI0, SGI0, SGI29001, SGI29001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6603[] = {SGI29002, SGI29003, SGI29004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29005, SGI0, SGI0, SGI0, SGI29006, SGI29006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6604[] = {SGI29007, SGI29008, SGI29009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29010, SGI0, SGI0, SGI0, SGI29011, SGI29011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6605[] = {SGI29012, SGI29013, SGI29014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29015, SGI0, SGI0, SGI0, SGI29016, SGI29016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6606[] = {SGI29017, SGI29018, SGI29019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29020, SGI0, SGI0, SGI0, SGI29021, SGI29021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6607[] = {SGI29022, SGI29023, SGI29024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29025, SGI0, SGI0, SGI0, SGI29026, SGI29026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6608[] = {SGI29027, SGI29028, SGI29029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29030, SGI0, SGI0, SGI0, SGI29031, SGI29031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6609[] = {SGI29032, SGI0, SGI29033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6610[] = {SGI29034, SGI29035, SGI29036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29037, SGI0, SGI0, SGI0, SGI29038, SGI29038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6611[] = {SGI29039, SGI29040, SGI29041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29042, SGI29043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6612[] = {SGI29044, SGI29045, SGI29046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29047, SGI0, SGI0, SGI0, SGI29048, SGI29048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6613[] = {SGI29049, SGI29050, SGI29051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29052, SGI0, SGI0, SGI0, SGI29053, SGI29053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6614[] = {SGI29054, SGI29055, SGI29056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29057, SGI0, SGI0, SGI0, SGI29058, SGI29058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6615[] = {SGI29059, SGI29060, SGI29061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29062, SGI0, SGI0, SGI0, SGI29063, SGI29063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6616[] = {SGI29064, SGI0, SGI29065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6617[] = {SGI29066, SGI29067, SGI29068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29069, SGI0, SGI0, SGI0, SGI29070, SGI29070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6618[] = {SGI29071, SGI29072, SGI29073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29074, SGI0, SGI0, SGI0, SGI29075, SGI29075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6619[] = {SGI29076, SGI29077, SGI29078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29079, SGI29080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6620[] = {SGI29081, SGI0, SGI29082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6621[] = {SGI29083, SGI29084, SGI29085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29086, SGI0, SGI0, SGI0, SGI29087, SGI29087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6622[] = {SGI29088, SGI29089, SGI29090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29091, SGI29091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6623[] = {SGI29092, SGI29093, SGI29094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29095, SGI0, SGI0, SGI0, SGI29096, SGI29096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6624[] = {SGI29097, SGI29098, SGI29099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29100, SGI29100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6625[] = {SGI29101, SGI29102, SGI29103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29104, SGI0, SGI0, SGI0, SGI29105, SGI29105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6626[] = {SGI29106, SGI29107, SGI29108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29109, SGI0, SGI0, SGI0, SGI29110, SGI29110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6627[] = {SGI29111, SGI29112, SGI29113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29114, SGI0, SGI0, SGI0, SGI29115, SGI29115, SGI0, SGI0, NULL};
const Segment * const SG6628[] = {SGI29116, SGI29117, SGI29118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29119, SGI0, SGI0, SGI0, SGI29120, SGI29120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6629[] = {SGI29121, SGI29122, SGI29123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29124, SGI0, SGI0, SGI0, SGI29125, SGI29125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6630[] = {SGI29126, SGI29127, SGI29128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29129, SGI0, SGI0, SGI0, SGI29130, SGI29130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6631[] = {SGI29131, SGI29132, SGI29133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29134, SGI0, SGI0, SGI0, SGI29137, SGI29137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6632[] = {SGI29135, SGI0, SGI29136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6633[] = {SGI29138, SGI29139, SGI29140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29141, SGI0, SGI0, SGI0, SGI29142, SGI29142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6634[] = {SGI29143, SGI29144, SGI29145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29146, SGI0, SGI0, SGI0, SGI29147, SGI29147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6635[] = {SGI29148, SGI29149, SGI29150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29153, SGI29155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6636[] = {SGI29151, SGI29152, SGI29154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29156, SGI0, SGI0, SGI0, SGI29157, SGI29157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6637[] = {SGI29158, SGI0, SGI29159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6638[] = {SGI29160, SGI29161, SGI29162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29163, SGI29163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6639[] = {SGI29164, SGI0, SGI29165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6640[] = {SGI29166, SGI29167, SGI29168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29169, SGI0, SGI0, SGI0, SGI29170, SGI29170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6641[] = {SGI29171, SGI29172, SGI29173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29174, SGI0, SGI0, SGI0, SGI29175, SGI29175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6642[] = {SGI29176, SGI29177, SGI29178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29179, SGI0, SGI0, SGI0, SGI29180, SGI29180, SGI0, SGI0, NULL};
const Segment * const SG6643[] = {SGI29181, SGI29182, SGI29183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29184, SGI0, SGI0, SGI0, SGI29185, SGI29185, SGI0, SGI0, NULL};
const Segment * const SG6644[] = {SGI29186, SGI29187, SGI29188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29189, SGI0, SGI0, SGI0, SGI29190, SGI29190, SGI0, SGI0, NULL};
const Segment * const SG6645[] = {SGI29191, SGI29192, SGI29193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29194, SGI0, SGI0, SGI0, SGI29195, SGI29195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6646[] = {SGI29196, SGI29197, SGI29198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29199, SGI0, SGI0, SGI0, SGI29200, SGI29200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6647[] = {SGI29201, SGI29202, SGI29203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29204, SGI0, SGI0, SGI0, SGI29206, SGI29206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6648[] = {SGI29205, SGI29207, SGI29208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29209, SGI0, SGI0, SGI0, SGI29210, SGI29210, SGI0, SGI0, NULL};
const Segment * const SG6649[] = {SGI29211, SGI29212, SGI29213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29214, SGI0, SGI0, SGI0, SGI29215, SGI29215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6650[] = {SGI29216, SGI29217, SGI29218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29219, SGI0, SGI0, SGI0, SGI29220, SGI29220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6651[] = {SGI29221, SGI29222, SGI29223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29224, SGI0, SGI0, SGI0, SGI29225, SGI29225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6652[] = {SGI29226, SGI29227, SGI29228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29229, SGI0, SGI0, SGI0, SGI29230, SGI29230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6653[] = {SGI29231, SGI29232, SGI29233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29234, SGI0, SGI0, SGI0, SGI29235, SGI29235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6654[] = {SGI29236, SGI29237, SGI29238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29239, SGI0, SGI0, SGI0, SGI29240, SGI29240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6655[] = {SGI29241, SGI29242, SGI29243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29244, SGI0, SGI0, SGI0, SGI29245, SGI29245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6656[] = {SGI29246, SGI29247, SGI29248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29250, SGI0, SGI0, SGI0, SGI29252, SGI29252, SGI0, SGI0, NULL};
const Segment * const SG6657[] = {SGI29249, SGI0, SGI29251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29253, SGI0, SGI0, SGI0, SGI29254, SGI29254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6658[] = {SGI29255, SGI29256, SGI29257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29258, SGI0, SGI0, SGI0, SGI29259, SGI29259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6659[] = {SGI29260, SGI29261, SGI29262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29263, SGI0, SGI0, SGI0, SGI0, SGI29264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6660[] = {SGI29265, SGI29266, SGI29267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29268, SGI0, SGI0, SGI0, SGI29269, SGI29269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6661[] = {SGI29270, SGI0, SGI29271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6662[] = {SGI29272, SGI29273, SGI29274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29275, SGI0, SGI0, SGI0, SGI29276, SGI29276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6663[] = {SGI29277, SGI29278, SGI29279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29280, SGI0, SGI0, SGI0, SGI29281, SGI29281, SGI0, SGI0, NULL};
const Segment * const SG6664[] = {SGI29282, SGI29283, SGI29284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29285, SGI29286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6665[] = {SGI0, SGI29287, SGI29288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29289, SGI29289, SGI0, SGI0, NULL};
const Segment * const SG6666[] = {SGI29290, SGI29291, SGI29292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29293, SGI29293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6667[] = {SGI29294, SGI29295, SGI29296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29297, SGI0, SGI0, SGI0, SGI29298, SGI29298, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6668[] = {SGI29299, SGI0, SGI29300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6669[] = {SGI29301, SGI29302, SGI29303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29304, SGI0, SGI0, SGI0, SGI29305, SGI29305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6670[] = {SGI29306, SGI29307, SGI29308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29309, SGI0, SGI0, SGI0, SGI29310, SGI29311, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6671[] = {SGI29312, SGI0, SGI29313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29314, SGI0, SGI0, SGI0, SGI29315, SGI29315, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6672[] = {SGI29316, SGI29317, SGI29318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29319, SGI0, SGI0, SGI0, SGI29320, SGI29320, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6673[] = {SGI29321, SGI29322, SGI29323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29324, SGI0, SGI0, SGI0, SGI29325, SGI29325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6674[] = {SGI29326, SGI29327, SGI29328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29329, SGI0, SGI0, SGI0, SGI29330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6675[] = {SGI29331, SGI29332, SGI29333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29334, SGI0, SGI0, SGI0, SGI29335, SGI29335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6676[] = {SGI29336, SGI29337, SGI29338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29339, SGI0, SGI0, SGI0, SGI29340, SGI29340, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6677[] = {SGI29341, SGI29342, SGI29343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29344, SGI0, SGI0, SGI0, SGI29345, SGI29345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6678[] = {SGI29346, SGI29347, SGI29348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29349, SGI0, SGI0, SGI0, SGI29350, SGI29350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6679[] = {SGI29351, SGI29352, SGI29353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29354, SGI0, SGI0, SGI0, SGI29355, SGI29355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6680[] = {SGI29356, SGI0, SGI29357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6681[] = {SGI29358, SGI29359, SGI29360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29361, SGI0, SGI0, SGI0, SGI29362, SGI29362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6682[] = {SGI29363, SGI0, SGI29364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29365, SGI0, SGI0, SGI0, SGI29366, SGI29366, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6683[] = {SGI29367, SGI29368, SGI29369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29370, SGI0, SGI0, SGI0, SGI29371, SGI29371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6684[] = {SGI29372, SGI29373, SGI29374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29375, SGI0, SGI0, SGI0, SGI29376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6685[] = {SGI29377, SGI29378, SGI29379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29380, SGI0, SGI0, SGI0, SGI29381, SGI29381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6686[] = {SGI29382, SGI29383, SGI29384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29385, SGI0, SGI0, SGI0, SGI29386, SGI29386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6687[] = {SGI29387, SGI29388, SGI29389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29390, SGI0, SGI0, SGI0, SGI29391, SGI29391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6688[] = {SGI29392, SGI29393, SGI29394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29395, SGI0, SGI0, SGI0, SGI29396, SGI29396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6689[] = {SGI29397, SGI29398, SGI29399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29400, SGI0, SGI0, SGI0, SGI29401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6690[] = {SGI29402, SGI0, SGI29403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6691[] = {SGI29404, SGI29405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29406, SGI0, SGI0, SGI0, SGI29407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6692[] = {SGI29408, SGI29409, SGI29410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29411, SGI0, SGI0, SGI0, SGI29412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6693[] = {SGI29413, SGI29414, SGI29415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29416, SGI0, SGI0, SGI0, SGI29417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6694[] = {SGI29418, SGI29419, SGI29420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29424, SGI0, SGI0, SGI0, SGI29425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6695[] = {SGI29421, SGI29422, SGI29423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29426, SGI0, SGI0, SGI0, SGI29427, SGI29427, SGI0, SGI0, NULL};
const Segment * const SG6696[] = {SGI29428, SGI0, SGI29429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6697[] = {SGI29430, SGI29431, SGI29432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29433, SGI0, SGI0, SGI0, SGI29434, SGI29434, SGI0, SGI0, NULL};
const Segment * const SG6698[] = {SGI29435, SGI29436, SGI29437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29438, SGI0, SGI0, SGI0, SGI29439, SGI29439, SGI0, SGI0, NULL};
const Segment * const SG6699[] = {SGI29440, SGI29441, SGI29442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29443, SGI29444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6700[] = {SGI29445, SGI29446, SGI29447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29448, SGI0, SGI0, SGI0, SGI29449, SGI29449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6701[] = {SGI29450, SGI29451, SGI29452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29453, SGI0, SGI0, SGI0, SGI29454, SGI29454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6702[] = {SGI29455, SGI29456, SGI29457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29458, SGI29458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6703[] = {SGI29459, SGI29460, SGI29461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29462, SGI0, SGI0, SGI0, SGI29463, SGI29463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6704[] = {SGI29464, SGI29465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29466, SGI0, SGI0, SGI0, SGI29467, SGI29467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6705[] = {SGI29468, SGI29470, SGI29472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29474, SGI0, SGI0, SGI0, SGI29477, SGI29477, SGI0, SGI0, NULL};
const Segment * const SG6706[] = {SGI29469, SGI29471, SGI29473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29475, SGI0, SGI0, SGI0, SGI29476, SGI29476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6707[] = {SGI29478, SGI29479, SGI29480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29481, SGI0, SGI0, SGI0, SGI29482, SGI29482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6708[] = {SGI29483, SGI29484, SGI29485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29486, SGI0, SGI0, SGI0, SGI29487, SGI29487, SGI0, SGI0, NULL};
const Segment * const SG6709[] = {SGI29488, SGI29489, SGI29490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29492, SGI0, SGI0, SGI0, SGI29494, SGI29494, SGI0, SGI0, NULL};
const Segment * const SG6710[] = {SGI29491, SGI0, SGI29493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6711[] = {SGI29495, SGI29496, SGI29497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29498, SGI0, SGI0, SGI0, SGI29499, SGI29499, SGI0, SGI0, NULL};
const Segment * const SG6712[] = {SGI29500, SGI29501, SGI29502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29503, SGI0, SGI0, SGI0, SGI29504, SGI29504, SGI0, SGI0, NULL};
const Segment * const SG6713[] = {SGI29505, SGI29506, SGI29507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29508, SGI0, SGI0, SGI0, SGI29509, SGI29509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6714[] = {SGI29510, SGI0, SGI29511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6715[] = {SGI29512, SGI29513, SGI29514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29515, SGI0, SGI0, SGI0, SGI29516, SGI29516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6716[] = {SGI29517, SGI29518, SGI29519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29520, SGI0, SGI0, SGI0, SGI29521, SGI29521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6717[] = {SGI0, SGI29522, SGI29523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29524, SGI0, SGI0, SGI0, SGI29525, SGI29525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6718[] = {SGI29526, SGI29527, SGI29528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29529, SGI0, SGI0, SGI0, SGI29530, SGI29530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6719[] = {SGI29531, SGI29532, SGI29533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29534, SGI0, SGI0, SGI0, SGI29535, SGI29535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6720[] = {SGI29536, SGI29537, SGI29538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29539, SGI0, SGI0, SGI0, SGI29540, SGI29540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6721[] = {SGI29541, SGI29542, SGI29543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29544, SGI0, SGI0, SGI0, SGI29545, SGI29545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6722[] = {SGI29546, SGI0, SGI29547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6723[] = {SGI29548, SGI29549, SGI29550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29551, SGI0, SGI0, SGI0, SGI29552, SGI29552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6724[] = {SGI29553, SGI29554, SGI29555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29556, SGI0, SGI0, SGI0, SGI29557, SGI29557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6725[] = {SGI29558, SGI0, SGI29559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6726[] = {SGI29560, SGI29561, SGI29562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29563, SGI29564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6727[] = {SGI29565, SGI0, SGI29566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6728[] = {SGI29567, SGI0, SGI29568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6729[] = {SGI29569, SGI0, SGI29570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6730[] = {SGI29571, SGI0, SGI29572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29573, SGI0, SGI0, SGI0, SGI29574, SGI29574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6731[] = {SGI29575, SGI29576, SGI29577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29578, SGI29578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6732[] = {SGI29579, SGI0, SGI29580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6733[] = {SGI0, SGI29581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29582, SGI0, SGI0, SGI0, SGI29583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6734[] = {SGI29584, SGI29585, SGI29586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29587, SGI0, SGI0, SGI0, SGI29588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6735[] = {SGI29589, SGI29590, SGI29591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29592, SGI0, SGI0, SGI0, SGI29593, SGI29593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6736[] = {SGI29594, SGI0, SGI29595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29596, SGI0, SGI0, SGI0, SGI29597, SGI29597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6737[] = {SGI29598, SGI29599, SGI29600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29601, SGI0, SGI0, SGI0, SGI29602, SGI29602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6738[] = {SGI29603, SGI29604, SGI29605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6739[] = {SGI29606, SGI29607, SGI29608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29609, SGI0, SGI0, SGI0, SGI29610, SGI29610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6740[] = {SGI0, SGI29611, SGI29612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6741[] = {SGI29613, SGI29614, SGI29615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29616, SGI0, SGI0, SGI0, SGI29617, SGI29617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6742[] = {SGI29618, SGI29619, SGI29620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29621, SGI0, SGI0, SGI0, SGI29622, SGI29622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6743[] = {SGI29623, SGI29624, SGI29625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29626, SGI0, SGI0, SGI0, SGI29627, SGI29627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6744[] = {SGI29628, SGI29629, SGI29630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29631, SGI0, SGI0, SGI0, SGI29632, SGI29633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6745[] = {SGI29634, SGI29635, SGI29636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29637, SGI29637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6746[] = {SGI29638, SGI29639, SGI29640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6747[] = {SGI29641, SGI29642, SGI29643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6748[] = {SGI29644, SGI29645, SGI29646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29647, SGI29647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6749[] = {SGI29648, SGI29649, SGI29650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6750[] = {SGI29651, SGI29652, SGI29653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6751[] = {SGI29654, SGI29655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29656, SGI0, SGI0, SGI0, SGI29657, SGI29657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6752[] = {SGI29658, SGI29659, SGI29660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6753[] = {SGI29661, SGI29662, SGI29663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6754[] = {SGI29664, SGI29665, SGI29666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29667, SGI0, SGI0, SGI0, SGI29668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6755[] = {SGI0, SGI29669, SGI29670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6756[] = {SGI29671, SGI29672, SGI29673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6757[] = {SGI29674, SGI29675, SGI29676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6758[] = {SGI29677, SGI29678, SGI29679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6759[] = {SGI29680, SGI0, SGI29681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29682, SGI0, SGI0, SGI0, SGI29683, SGI29683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6760[] = {SGI29684, SGI29685, SGI29686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6761[] = {SGI29687, SGI29688, SGI29689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6762[] = {SGI29690, SGI29691, SGI29692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29693, SGI29693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6763[] = {SGI29694, SGI29695, SGI29696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6764[] = {SGI29697, SGI29698, SGI29699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6765[] = {SGI29700, SGI29701, SGI29702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29703, SGI0, SGI0, SGI0, SGI29704, SGI29705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6766[] = {SGI29706, SGI29707, SGI29708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29709, SGI0, SGI0, SGI0, SGI29710, SGI29710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6767[] = {SGI29711, SGI29712, SGI29713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6768[] = {SGI29714, SGI29715, SGI29716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6769[] = {SGI29717, SGI29718, SGI29719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29720, SGI0, SGI0, SGI0, SGI29721, SGI29721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6770[] = {SGI29722, SGI29723, SGI29724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6771[] = {SGI29725, SGI29726, SGI29727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6772[] = {SGI29728, SGI29729, SGI29730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6773[] = {SGI0, SGI29731, SGI29732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6774[] = {SGI29733, SGI29734, SGI29735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29736, SGI0, SGI0, SGI0, SGI0, SGI29737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6775[] = {SGI29738, SGI29739, SGI29740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6776[] = {SGI29741, SGI0, SGI29742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29743, SGI0, SGI0, SGI0, SGI29744, SGI29744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6777[] = {SGI0, SGI29745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6778[] = {SGI29746, SGI29747, SGI29748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6779[] = {SGI29749, SGI29750, SGI29751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29752, SGI0, SGI0, SGI0, SGI29753, SGI29753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6780[] = {SGI29754, SGI29755, SGI29756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29757, SGI0, SGI0, SGI0, SGI29758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6781[] = {SGI29759, SGI29760, SGI29761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6782[] = {SGI29762, SGI29763, SGI29764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29765, SGI0, SGI0, SGI0, SGI29766, SGI29766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6783[] = {SGI0, SGI29767, SGI29768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29769, SGI0, SGI0, SGI0, SGI29770, SGI29771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6784[] = {SGI0, SGI29772, SGI29773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6785[] = {SGI29774, SGI29775, SGI29776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6786[] = {SGI29777, SGI29778, SGI29779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29780, SGI0, SGI0, SGI0, SGI29781, SGI29781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6787[] = {SGI0, SGI29782, SGI29783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6788[] = {SGI29784, SGI29785, SGI29786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29787, SGI0, SGI0, SGI0, SGI29788, SGI29789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6789[] = {SGI29790, SGI29791, SGI29792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29793, SGI29793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6790[] = {SGI0, SGI0, SGI29794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6791[] = {SGI29795, SGI0, SGI29796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29797, SGI0, SGI0, SGI0, SGI29798, SGI29798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6792[] = {SGI29799, SGI0, SGI29800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29801, SGI0, SGI0, SGI0, SGI29802, SGI29802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6793[] = {SGI29803, SGI29804, SGI29805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6794[] = {SGI29806, SGI29807, SGI29808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29809, SGI0, SGI0, SGI0, SGI29810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6795[] = {SGI29811, SGI29812, SGI29813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29814, SGI0, SGI0, SGI0, SGI29815, SGI29815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6796[] = {SGI29816, SGI29817, SGI29818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29819, SGI0, SGI0, SGI0, SGI29820, SGI29820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6797[] = {SGI29821, SGI29822, SGI29823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29824, SGI0, SGI0, SGI0, SGI29825, SGI29825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6798[] = {SGI29826, SGI29827, SGI29828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29829, SGI0, SGI0, SGI0, SGI29830, SGI29830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6799[] = {SGI29831, SGI29832, SGI29833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29834, SGI0, SGI0, SGI0, SGI29835, SGI29835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6800[] = {SGI29836, SGI29837, SGI29838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29839, SGI0, SGI0, SGI0, SGI29840, SGI29840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6801[] = {SGI29841, SGI29842, SGI29843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29844, SGI0, SGI0, SGI0, SGI29845, SGI29845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6802[] = {SGI29846, SGI29847, SGI29848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29849, SGI0, SGI0, SGI0, SGI29850, SGI29850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6803[] = {SGI29851, SGI29852, SGI29853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29854, SGI0, SGI0, SGI0, SGI29855, SGI29855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6804[] = {SGI29856, SGI29857, SGI29858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29859, SGI0, SGI0, SGI0, SGI29860, SGI29860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6805[] = {SGI29861, SGI29862, SGI29863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29864, SGI0, SGI0, SGI0, SGI29865, SGI29865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6806[] = {SGI29866, SGI29867, SGI29868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29869, SGI0, SGI0, SGI0, SGI29870, SGI29870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6807[] = {SGI29871, SGI29872, SGI29873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29874, SGI0, SGI0, SGI0, SGI29875, SGI29875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6808[] = {SGI29876, SGI29877, SGI29878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29879, SGI0, SGI0, SGI0, SGI29880, SGI29880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6809[] = {SGI29881, SGI29882, SGI29883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29884, SGI0, SGI0, SGI0, SGI29885, SGI29885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6810[] = {SGI29886, SGI29887, SGI29888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29889, SGI0, SGI0, SGI0, SGI29890, SGI29890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6811[] = {SGI29891, SGI29892, SGI29893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29894, SGI0, SGI0, SGI0, SGI29895, SGI29895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6812[] = {SGI29896, SGI29897, SGI29898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29899, SGI0, SGI0, SGI0, SGI29900, SGI29900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6813[] = {SGI29901, SGI29902, SGI29903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29904, SGI0, SGI0, SGI0, SGI29905, SGI29905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6814[] = {SGI29906, SGI29907, SGI29908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29909, SGI0, SGI0, SGI0, SGI29910, SGI29910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6815[] = {SGI29911, SGI29912, SGI29913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29914, SGI0, SGI0, SGI0, SGI29915, SGI29915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6816[] = {SGI29916, SGI29917, SGI29918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29919, SGI0, SGI0, SGI0, SGI29920, SGI29920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6817[] = {SGI29921, SGI29922, SGI29923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29924, SGI0, SGI0, SGI0, SGI29925, SGI29925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6818[] = {SGI29926, SGI29927, SGI29928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29929, SGI0, SGI0, SGI0, SGI29930, SGI29930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6819[] = {SGI29931, SGI29932, SGI29933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29934, SGI29934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6820[] = {SGI29935, SGI29936, SGI29937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29938, SGI0, SGI0, SGI0, SGI29939, SGI29939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6821[] = {SGI29940, SGI29941, SGI29942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29943, SGI0, SGI0, SGI0, SGI29944, SGI29944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6822[] = {SGI29945, SGI29946, SGI29947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29948, SGI0, SGI0, SGI0, SGI29949, SGI29949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6823[] = {SGI29950, SGI29951, SGI29952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29953, SGI0, SGI0, SGI0, SGI29954, SGI29954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6824[] = {SGI29955, SGI29956, SGI29957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29958, SGI0, SGI0, SGI0, SGI29959, SGI29959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6825[] = {SGI29960, SGI29961, SGI29962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29963, SGI0, SGI0, SGI0, SGI29964, SGI29964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6826[] = {SGI29965, SGI29966, SGI29967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29968, SGI0, SGI0, SGI0, SGI29969, SGI29969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6827[] = {SGI29970, SGI0, SGI29971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6828[] = {SGI29972, SGI29973, SGI29974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29975, SGI0, SGI0, SGI0, SGI29976, SGI29976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6829[] = {SGI29977, SGI29978, SGI29979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29980, SGI0, SGI0, SGI0, SGI29981, SGI29981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6830[] = {SGI29982, SGI29983, SGI29984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29985, SGI0, SGI0, SGI0, SGI29986, SGI29986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6831[] = {SGI29987, SGI0, SGI29988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29989, SGI0, SGI0, SGI0, SGI29990, SGI29990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6832[] = {SGI29991, SGI29992, SGI29993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29994, SGI0, SGI0, SGI0, SGI29995, SGI29995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6833[] = {SGI29996, SGI29997, SGI29998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI29999, SGI0, SGI0, SGI0, SGI30000, SGI30000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6834[] = {SGI30001, SGI30002, SGI30003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30004, SGI0, SGI0, SGI0, SGI30005, SGI30005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6835[] = {SGI30006, SGI30007, SGI30008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30009, SGI0, SGI0, SGI0, SGI30010, SGI30010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6836[] = {SGI30011, SGI30012, SGI30013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30014, SGI0, SGI0, SGI0, SGI30015, SGI30015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6837[] = {SGI30016, SGI30017, SGI30018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30019, SGI0, SGI0, SGI0, SGI30020, SGI30020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6838[] = {SGI30021, SGI30022, SGI30023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30024, SGI0, SGI0, SGI0, SGI30025, SGI30025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6839[] = {SGI30026, SGI30027, SGI30028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30029, SGI0, SGI0, SGI0, SGI30030, SGI30030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6840[] = {SGI30031, SGI0, SGI30032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6841[] = {SGI30033, SGI30034, SGI30035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30036, SGI0, SGI0, SGI0, SGI30037, SGI30037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6842[] = {SGI30038, SGI30039, SGI30040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30041, SGI0, SGI0, SGI0, SGI30042, SGI30042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6843[] = {SGI30043, SGI30044, SGI30045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30046, SGI0, SGI0, SGI0, SGI30047, SGI30047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6844[] = {SGI30048, SGI30049, SGI30050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30051, SGI0, SGI0, SGI0, SGI30052, SGI30052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6845[] = {SGI30053, SGI30054, SGI30055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30056, SGI0, SGI0, SGI0, SGI30057, SGI30057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6846[] = {SGI30058, SGI30059, SGI30060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30061, SGI0, SGI0, SGI0, SGI30062, SGI30062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6847[] = {SGI30063, SGI30064, SGI30065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30066, SGI0, SGI0, SGI0, SGI30067, SGI30067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6848[] = {SGI30068, SGI30069, SGI30070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30071, SGI0, SGI0, SGI0, SGI30072, SGI30072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6849[] = {SGI30073, SGI30074, SGI30075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30076, SGI0, SGI0, SGI0, SGI30077, SGI30077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6850[] = {SGI30078, SGI30079, SGI30080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30081, SGI30082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6851[] = {SGI30083, SGI30084, SGI30085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30086, SGI0, SGI0, SGI0, SGI30087, SGI30087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6852[] = {SGI30088, SGI30089, SGI30090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30091, SGI0, SGI0, SGI0, SGI30092, SGI30092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6853[] = {SGI30093, SGI30094, SGI30095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30096, SGI0, SGI0, SGI0, SGI30097, SGI30097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6854[] = {SGI30098, SGI30099, SGI30100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30101, SGI0, SGI0, SGI0, SGI30102, SGI30102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6855[] = {SGI30103, SGI30104, SGI30105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30106, SGI0, SGI0, SGI0, SGI30107, SGI30107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6856[] = {SGI30108, SGI30109, SGI30110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30111, SGI0, SGI0, SGI0, SGI30112, SGI30112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6857[] = {SGI30113, SGI30114, SGI30115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30116, SGI30117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6858[] = {SGI30118, SGI30119, SGI30120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30121, SGI0, SGI0, SGI0, SGI30122, SGI30122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6859[] = {SGI30123, SGI30124, SGI30125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30126, SGI0, SGI0, SGI0, SGI30127, SGI30127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6860[] = {SGI30128, SGI30129, SGI30130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30131, SGI0, SGI0, SGI0, SGI30132, SGI30132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6861[] = {SGI30133, SGI30134, SGI30135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30136, SGI30136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6862[] = {SGI30137, SGI0, SGI30138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6863[] = {SGI30139, SGI0, SGI30140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30141, SGI0, SGI0, SGI0, SGI30142, SGI30142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6864[] = {SGI30143, SGI30144, SGI30145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30146, SGI0, SGI0, SGI0, SGI30147, SGI30147, SGI0, SGI0, NULL};
const Segment * const SG6865[] = {SGI30148, SGI30149, SGI30150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30151, SGI0, SGI0, SGI0, SGI30152, SGI30152, SGI0, SGI0, NULL};
const Segment * const SG6866[] = {SGI30153, SGI30154, SGI30155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30156, SGI0, SGI0, SGI0, SGI30157, SGI30157, SGI0, SGI0, NULL};
const Segment * const SG6867[] = {SGI30158, SGI30159, SGI30160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30161, SGI0, SGI0, SGI0, SGI30162, SGI30162, SGI0, SGI0, NULL};
const Segment * const SG6868[] = {SGI30163, SGI30164, SGI30165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30166, SGI0, SGI0, SGI0, SGI30167, SGI30167, SGI0, SGI0, NULL};
const Segment * const SG6869[] = {SGI30168, SGI30169, SGI30170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30171, SGI0, SGI0, SGI0, SGI30172, SGI30172, SGI0, SGI0, NULL};
const Segment * const SG6870[] = {SGI30173, SGI0, SGI30174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6871[] = {SGI30175, SGI30176, SGI30177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30178, SGI0, SGI0, SGI0, SGI30179, SGI30179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6872[] = {SGI30180, SGI30181, SGI30182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30183, SGI0, SGI0, SGI0, SGI30184, SGI30184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6873[] = {SGI30185, SGI0, SGI30186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30187, SGI0, SGI0, SGI0, SGI30188, SGI30188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6874[] = {SGI30189, SGI30190, SGI30191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30192, SGI0, SGI0, SGI0, SGI30193, SGI30194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6875[] = {SGI30195, SGI30196, SGI30197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6876[] = {SGI30198, SGI30199, SGI30200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30201, SGI0, SGI0, SGI0, SGI30202, SGI30202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6877[] = {SGI30203, SGI30204, SGI30205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30206, SGI0, SGI0, SGI0, SGI30207, SGI30207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6878[] = {SGI30208, SGI0, SGI30209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6879[] = {SGI30210, SGI30211, SGI30212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30213, SGI0, SGI0, SGI0, SGI30214, SGI30214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6880[] = {SGI30215, SGI0, SGI30216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30217, SGI0, SGI0, SGI0, SGI30218, SGI30218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6881[] = {SGI30219, SGI30220, SGI30221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30222, SGI0, SGI0, SGI0, SGI30223, SGI30223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6882[] = {SGI30224, SGI30225, SGI30226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30227, SGI30227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6883[] = {SGI30228, SGI30229, SGI30230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30231, SGI0, SGI0, SGI0, SGI30232, SGI30232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6884[] = {SGI30233, SGI30234, SGI30235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30236, SGI30237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6885[] = {SGI30238, SGI30239, SGI30240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30241, SGI0, SGI0, SGI0, SGI30242, SGI30242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6886[] = {SGI30243, SGI30244, SGI30245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30246, SGI30246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6887[] = {SGI30247, SGI30248, SGI30249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30250, SGI0, SGI0, SGI0, SGI30251, SGI30251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6888[] = {SGI30252, SGI30253, SGI30254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30255, SGI0, SGI0, SGI0, SGI30256, SGI30256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6889[] = {SGI30257, SGI30258, SGI30259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30260, SGI0, SGI0, SGI0, SGI30261, SGI30261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6890[] = {SGI30262, SGI30263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30264, SGI0, SGI0, SGI0, SGI30265, SGI30265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6891[] = {SGI30266, SGI30267, SGI30268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30269, SGI0, SGI0, SGI0, SGI30270, SGI30270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6892[] = {SGI30271, SGI30272, SGI30273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30274, SGI0, SGI0, SGI0, SGI30275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6893[] = {SGI30276, SGI30277, SGI30278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30279, SGI0, SGI0, SGI0, SGI30280, SGI30280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6894[] = {SGI30281, SGI30282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30283, SGI0, SGI0, SGI0, SGI30284, SGI30284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6895[] = {SGI30285, SGI30286, SGI30287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30288, SGI0, SGI0, SGI0, SGI30289, SGI30289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6896[] = {SGI30290, SGI30291, SGI30292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30293, SGI0, SGI0, SGI0, SGI30294, SGI30294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6897[] = {SGI30295, SGI30296, SGI30297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30298, SGI0, SGI0, SGI0, SGI30299, SGI30299, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6898[] = {SGI0, SGI30300, SGI30301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30302, SGI0, SGI0, SGI0, SGI30303, SGI30303, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6899[] = {SGI30304, SGI30305, SGI30306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30307, SGI0, SGI0, SGI0, SGI30308, SGI30308, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6900[] = {SGI30309, SGI30310, SGI30311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30312, SGI0, SGI0, SGI0, SGI30313, SGI30313, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6901[] = {SGI0, SGI30314, SGI30315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30316, SGI0, SGI0, SGI0, SGI30317, SGI30317, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6902[] = {SGI30318, SGI30319, SGI30320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30321, SGI0, SGI0, SGI0, SGI30323, SGI30323, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6903[] = {SGI30322, SGI30324, SGI30325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30326, SGI0, SGI0, SGI0, SGI30327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6904[] = {SGI30328, SGI30329, SGI30330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30331, SGI0, SGI0, SGI0, SGI30333, SGI30333, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6905[] = {SGI30332, SGI30334, SGI30335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30336, SGI0, SGI0, SGI0, SGI30337, SGI30337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6906[] = {SGI30338, SGI30339, SGI30340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30341, SGI0, SGI0, SGI0, SGI30343, SGI30343, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6907[] = {SGI30342, SGI30344, SGI30345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30346, SGI0, SGI0, SGI0, SGI30347, SGI30347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6908[] = {SGI30348, SGI30349, SGI30350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30351, SGI0, SGI0, SGI0, SGI30353, SGI30353, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6909[] = {SGI30352, SGI30354, SGI30355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30356, SGI0, SGI0, SGI0, SGI30357, SGI30357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6910[] = {SGI30358, SGI30359, SGI30360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30361, SGI0, SGI0, SGI0, SGI30362, SGI30362, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6911[] = {SGI30363, SGI0, SGI30364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6912[] = {SGI30365, SGI30366, SGI30367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30368, SGI0, SGI0, SGI0, SGI30369, SGI30369, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6913[] = {SGI30370, SGI0, SGI30371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30372, SGI0, SGI0, SGI0, SGI30373, SGI30373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6914[] = {SGI0, SGI30374, SGI30375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30376, SGI0, SGI0, SGI0, SGI30377, SGI30377, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6915[] = {SGI30378, SGI30379, SGI30380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30381, SGI0, SGI0, SGI0, SGI30382, SGI30382, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6916[] = {SGI30383, SGI0, SGI30384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6917[] = {SGI30385, SGI30386, SGI30387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30388, SGI0, SGI0, SGI0, SGI30390, SGI30390, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6918[] = {SGI30389, SGI0, SGI30391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6919[] = {SGI30393, SGI0, SGI30396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6920[] = {SGI30392, SGI30394, SGI30395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30397, SGI30397, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6921[] = {SGI0, SGI30398, SGI30399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30402, SGI0, SGI0, SGI0, SGI30404, SGI30404, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6922[] = {SGI30400, SGI30401, SGI30403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6923[] = {SGI30405, SGI30406, SGI30407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30408, SGI0, SGI0, SGI0, SGI30409, SGI30409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6924[] = {SGI30410, SGI30411, SGI30412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30413, SGI0, SGI0, SGI0, SGI30414, SGI30414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6925[] = {SGI30415, SGI30416, SGI30417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30418, SGI0, SGI0, SGI0, SGI30419, SGI30419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6926[] = {SGI30420, SGI30421, SGI30422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30423, SGI30424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6927[] = {SGI30425, SGI30426, SGI30427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30428, SGI30429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6928[] = {SGI30430, SGI30431, SGI30432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30433, SGI0, SGI0, SGI0, SGI30434, SGI30434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6929[] = {SGI30435, SGI30436, SGI30437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30438, SGI30438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6930[] = {SGI30439, SGI30440, SGI30441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30442, SGI0, SGI0, SGI0, SGI30443, SGI30444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6931[] = {SGI30445, SGI0, SGI30446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30447, SGI0, SGI0, SGI0, SGI30448, SGI30448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6932[] = {SGI30449, SGI30450, SGI30451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30452, SGI30453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6933[] = {SGI30454, SGI0, SGI30455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30456, SGI0, SGI0, SGI0, SGI30457, SGI30457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6934[] = {SGI30458, SGI30459, SGI30460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30461, SGI30461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6935[] = {SGI30462, SGI30463, SGI30464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30465, SGI0, SGI0, SGI0, SGI30466, SGI30466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6936[] = {SGI30467, SGI30468, SGI30469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30470, SGI0, SGI0, SGI0, SGI30471, SGI30471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6937[] = {SGI30472, SGI30473, SGI30474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30475, SGI0, SGI0, SGI0, SGI30476, SGI30476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6938[] = {SGI30477, SGI30478, SGI30479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30480, SGI0, SGI0, SGI0, SGI30481, SGI30481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6939[] = {SGI30482, SGI30483, SGI30484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30485, SGI0, SGI0, SGI0, SGI30486, SGI30486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6940[] = {SGI30487, SGI30488, SGI30489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30490, SGI0, SGI0, SGI0, SGI30491, SGI30491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6941[] = {SGI30492, SGI30493, SGI30494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30495, SGI0, SGI0, SGI0, SGI30496, SGI30496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6942[] = {SGI30497, SGI0, SGI30498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6943[] = {SGI30499, SGI30500, SGI30501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30502, SGI0, SGI0, SGI0, SGI30503, SGI30503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6944[] = {SGI30504, SGI30505, SGI30506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30507, SGI30508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6945[] = {SGI30509, SGI30510, SGI30511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30512, SGI0, SGI0, SGI0, SGI30513, SGI30513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6946[] = {SGI30514, SGI30515, SGI30516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30517, SGI0, SGI0, SGI0, SGI30518, SGI30518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6947[] = {SGI30519, SGI30520, SGI30521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30522, SGI0, SGI0, SGI0, SGI30523, SGI30523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6948[] = {SGI30524, SGI30525, SGI30526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30527, SGI30527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6949[] = {SGI30528, SGI30529, SGI30530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30531, SGI0, SGI0, SGI0, SGI30532, SGI30532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6950[] = {SGI30533, SGI0, SGI30534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6951[] = {SGI30535, SGI30536, SGI30537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30538, SGI0, SGI0, SGI0, SGI30539, SGI30539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6952[] = {SGI30540, SGI0, SGI30541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30542, SGI0, SGI0, SGI0, SGI30543, SGI30543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6953[] = {SGI30544, SGI30545, SGI30546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30547, SGI0, SGI0, SGI0, SGI30548, SGI30548, SGI0, SGI0, NULL};
const Segment * const SG6954[] = {SGI30549, SGI30550, SGI30551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30552, SGI0, SGI0, SGI0, SGI30553, SGI30553, SGI0, SGI0, NULL};
const Segment * const SG6955[] = {SGI30554, SGI30555, SGI30556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30557, SGI0, SGI0, SGI0, SGI30558, SGI30558, SGI0, SGI0, NULL};
const Segment * const SG6956[] = {SGI30559, SGI30560, SGI30561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30562, SGI0, SGI0, SGI0, SGI30563, SGI30563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6957[] = {SGI30564, SGI30565, SGI30566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30567, SGI0, SGI0, SGI0, SGI30568, SGI30568, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6958[] = {SGI30569, SGI30570, SGI30571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30572, SGI0, SGI0, SGI0, SGI30573, SGI30573, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6959[] = {SGI30574, SGI30575, SGI30576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30577, SGI0, SGI0, SGI0, SGI30578, SGI30578, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6960[] = {SGI30579, SGI30580, SGI30581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30582, SGI0, SGI0, SGI0, SGI30583, SGI30583, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6961[] = {SGI30584, SGI30585, SGI30586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30587, SGI0, SGI0, SGI0, SGI30588, SGI30588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6962[] = {SGI30589, SGI30590, SGI30591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30592, SGI0, SGI0, SGI0, SGI30593, SGI30593, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6963[] = {SGI30594, SGI0, SGI30595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30596, SGI0, SGI0, SGI0, SGI30597, SGI30597, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6964[] = {SGI30598, SGI30599, SGI30600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30604, SGI0, SGI0, SGI0, SGI30605, SGI30605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6965[] = {SGI30601, SGI30602, SGI30603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30606, SGI0, SGI0, SGI0, SGI30607, SGI30607, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6966[] = {SGI30608, SGI30609, SGI30610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30613, SGI0, SGI0, SGI0, SGI30615, SGI30615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6967[] = {SGI30611, SGI30612, SGI30614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30616, SGI0, SGI0, SGI0, SGI30617, SGI30617, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6968[] = {SGI30618, SGI30619, SGI30620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30623, SGI0, SGI0, SGI0, SGI30624, SGI30624, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6969[] = {SGI30621, SGI0, SGI30622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6970[] = {SGI30626, SGI30628, SGI30630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30632, SGI0, SGI0, SGI0, SGI30633, SGI30633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6971[] = {SGI30625, SGI30627, SGI30629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30631, SGI0, SGI0, SGI0, SGI30634, SGI30634, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6972[] = {SGI30635, SGI30636, SGI30637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30638, SGI0, SGI0, SGI0, SGI30639, SGI30639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6973[] = {SGI30640, SGI30641, SGI30642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30643, SGI0, SGI0, SGI0, SGI30647, SGI30647, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6974[] = {SGI30644, SGI30645, SGI30646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30648, SGI0, SGI0, SGI0, SGI30649, SGI30649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6975[] = {SGI30650, SGI0, SGI30651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6976[] = {SGI30652, SGI30653, SGI30654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30655, SGI0, SGI0, SGI0, SGI30656, SGI30656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6977[] = {SGI30657, SGI30658, SGI30659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30660, SGI0, SGI0, SGI0, SGI30661, SGI30661, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6978[] = {SGI30662, SGI30663, SGI30664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30665, SGI0, SGI0, SGI0, SGI30668, SGI30668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6979[] = {SGI30666, SGI30667, SGI30669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30670, SGI30671, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6980[] = {SGI30675, SGI0, SGI30676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6981[] = {SGI30672, SGI30673, SGI30674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30677, SGI0, SGI0, SGI0, SGI30678, SGI30678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6982[] = {SGI30679, SGI0, SGI30680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6983[] = {SGI30681, SGI30682, SGI30683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30684, SGI0, SGI0, SGI0, SGI30685, SGI30685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6984[] = {SGI30686, SGI30687, SGI30688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30689, SGI30689, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6985[] = {SGI30690, SGI30691, SGI30692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30693, SGI0, SGI0, SGI0, SGI30694, SGI30694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6986[] = {SGI30695, SGI30696, SGI30697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30698, SGI0, SGI0, SGI0, SGI30699, SGI30699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6987[] = {SGI30700, SGI30701, SGI30702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30703, SGI0, SGI0, SGI0, SGI30704, SGI30704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6988[] = {SGI30705, SGI30706, SGI30707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30708, SGI0, SGI0, SGI0, SGI0, SGI30709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6989[] = {SGI30710, SGI30711, SGI30712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30713, SGI0, SGI0, SGI0, SGI30714, SGI30714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6990[] = {SGI30715, SGI30716, SGI30717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30718, SGI0, SGI0, SGI0, SGI30719, SGI30719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6991[] = {SGI30720, SGI30721, SGI30722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30723, SGI0, SGI0, SGI0, SGI30724, SGI30724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6992[] = {SGI30725, SGI30726, SGI30727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30728, SGI30729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6993[] = {SGI30730, SGI30731, SGI30732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30733, SGI0, SGI0, SGI0, SGI30734, SGI30734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6994[] = {SGI30735, SGI30736, SGI30737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30738, SGI0, SGI0, SGI0, SGI30739, SGI30739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6995[] = {SGI30740, SGI30741, SGI30742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30743, SGI30743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6996[] = {SGI30744, SGI30745, SGI30746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30747, SGI0, SGI0, SGI0, SGI30748, SGI30748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6997[] = {SGI30749, SGI30750, SGI30751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30752, SGI0, SGI0, SGI0, SGI30753, SGI30753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6998[] = {SGI30754, SGI30755, SGI30756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30757, SGI0, SGI0, SGI0, SGI30758, SGI30758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG6999[] = {SGI30759, SGI30760, SGI30761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30762, SGI0, SGI0, SGI0, SGI30763, SGI30763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7000[] = {SGI30764, SGI30765, SGI30766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30767, SGI0, SGI0, SGI0, SGI30768, SGI30768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7001[] = {SGI30769, SGI30770, SGI30771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30772, SGI0, SGI0, SGI0, SGI30773, SGI30773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7002[] = {SGI30774, SGI30775, SGI30776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30777, SGI0, SGI0, SGI0, SGI30778, SGI30778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7003[] = {SGI30779, SGI30780, SGI30781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30782, SGI0, SGI0, SGI0, SGI30783, SGI30783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7004[] = {SGI30784, SGI30785, SGI30786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30787, SGI0, SGI0, SGI0, SGI30788, SGI30788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7005[] = {SGI30789, SGI30790, SGI30791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30792, SGI0, SGI0, SGI0, SGI30793, SGI30793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7006[] = {SGI30794, SGI30795, SGI30796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30797, SGI0, SGI0, SGI0, SGI30798, SGI30798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7007[] = {SGI30799, SGI30800, SGI30801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30802, SGI0, SGI0, SGI0, SGI30803, SGI30803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7008[] = {SGI30804, SGI30805, SGI30806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30807, SGI0, SGI0, SGI0, SGI30808, SGI30808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7009[] = {SGI30809, SGI30810, SGI30811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30812, SGI0, SGI0, SGI0, SGI30813, SGI30813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7010[] = {SGI30814, SGI30815, SGI30816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30817, SGI0, SGI0, SGI0, SGI30818, SGI30818, SGI0, SGI0, NULL};
const Segment * const SG7011[] = {SGI30819, SGI30820, SGI30821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30822, SGI0, SGI0, SGI0, SGI30823, SGI30823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7012[] = {SGI30824, SGI30825, SGI30826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30827, SGI0, SGI0, SGI0, SGI30828, SGI30828, SGI0, SGI0, NULL};
const Segment * const SG7013[] = {SGI30829, SGI30830, SGI30831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30832, SGI0, SGI0, SGI0, SGI30833, SGI30833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7014[] = {SGI30834, SGI30835, SGI30836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30837, SGI0, SGI0, SGI0, SGI30838, SGI30838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7015[] = {SGI30839, SGI30840, SGI30841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30842, SGI0, SGI0, SGI0, SGI30843, SGI30843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7016[] = {SGI30844, SGI30845, SGI30846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30847, SGI0, SGI0, SGI0, SGI30848, SGI30848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7017[] = {SGI30849, SGI30850, SGI30851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30852, SGI0, SGI0, SGI0, SGI30853, SGI30853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7018[] = {SGI30854, SGI30855, SGI30856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30857, SGI0, SGI0, SGI0, SGI30858, SGI30858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7019[] = {SGI30859, SGI30860, SGI30861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30862, SGI0, SGI0, SGI0, SGI30863, SGI30863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7020[] = {SGI30864, SGI30865, SGI30866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30867, SGI0, SGI0, SGI0, SGI30868, SGI30868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7021[] = {SGI30869, SGI30870, SGI30871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30872, SGI0, SGI0, SGI0, SGI30873, SGI30873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7022[] = {SGI30874, SGI30875, SGI30876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30877, SGI0, SGI0, SGI0, SGI30878, SGI30878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7023[] = {SGI30879, SGI30880, SGI30881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30882, SGI0, SGI0, SGI0, SGI30883, SGI30883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7024[] = {SGI30884, SGI30885, SGI30886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30887, SGI0, SGI0, SGI0, SGI30888, SGI30888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7025[] = {SGI30889, SGI30890, SGI30891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30892, SGI0, SGI0, SGI0, SGI30893, SGI30893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7026[] = {SGI30894, SGI30895, SGI30896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30897, SGI0, SGI0, SGI0, SGI30898, SGI30898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7027[] = {SGI30899, SGI30900, SGI30901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30902, SGI0, SGI0, SGI0, SGI30903, SGI30903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7028[] = {SGI30904, SGI30905, SGI30906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30907, SGI0, SGI0, SGI0, SGI30908, SGI30908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7029[] = {SGI30909, SGI30910, SGI30911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30912, SGI0, SGI0, SGI0, SGI30913, SGI30913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7030[] = {SGI0, SGI30914, SGI30915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30916, SGI0, SGI0, SGI0, SGI30917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7031[] = {SGI30918, SGI0, SGI30919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7032[] = {SGI30920, SGI30921, SGI30922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30923, SGI30924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7033[] = {SGI30925, SGI30926, SGI30927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30928, SGI30929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7034[] = {SGI30930, SGI30931, SGI30932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30933, SGI30933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7035[] = {SGI30934, SGI30935, SGI30936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30937, SGI0, SGI0, SGI0, SGI30938, SGI30938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7036[] = {SGI30939, SGI0, SGI30940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7037[] = {SGI30941, SGI30942, SGI30943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30944, SGI0, SGI0, SGI0, SGI30945, SGI30945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7038[] = {SGI30946, SGI30947, SGI30948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30949, SGI0, SGI0, SGI0, SGI30950, SGI30950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7039[] = {SGI30951, SGI30952, SGI30953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30954, SGI0, SGI0, SGI0, SGI30955, SGI30955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7040[] = {SGI30956, SGI30957, SGI30958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30959, SGI0, SGI0, SGI0, SGI30960, SGI30960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7041[] = {SGI30961, SGI30962, SGI30963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30964, SGI0, SGI0, SGI0, SGI30965, SGI30965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7042[] = {SGI30966, SGI30967, SGI30968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30969, SGI0, SGI0, SGI0, SGI30970, SGI30970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7043[] = {SGI30971, SGI30972, SGI30973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30974, SGI0, SGI0, SGI0, SGI30975, SGI30975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7044[] = {SGI30976, SGI0, SGI30977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7045[] = {SGI30978, SGI30979, SGI30980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30981, SGI0, SGI0, SGI0, SGI30982, SGI30982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7046[] = {SGI30983, SGI30984, SGI30985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30986, SGI30987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7047[] = {SGI30988, SGI30989, SGI30990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30991, SGI0, SGI0, SGI0, SGI30992, SGI30992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7048[] = {SGI30993, SGI30994, SGI30995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI30996, SGI0, SGI0, SGI0, SGI30997, SGI30997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7049[] = {SGI30998, SGI30999, SGI31000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31001, SGI0, SGI0, SGI0, SGI31002, SGI31002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7050[] = {SGI31003, SGI31004, SGI31005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31006, SGI31006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7051[] = {SGI31007, SGI31008, SGI31009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31010, SGI31010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7052[] = {SGI31011, SGI31012, SGI31013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31014, SGI0, SGI0, SGI0, SGI31015, SGI31015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7053[] = {SGI31016, SGI31017, SGI31018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31019, SGI0, SGI0, SGI0, SGI31020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7054[] = {SGI31021, SGI31022, SGI31023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31024, SGI0, SGI0, SGI0, SGI31025, SGI31025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7055[] = {SGI31026, SGI31027, SGI31028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7056[] = {SGI31029, SGI31030, SGI31031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31032, SGI0, SGI0, SGI0, SGI31033, SGI31033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7057[] = {SGI31034, SGI31035, SGI31036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31037, SGI0, SGI0, SGI0, SGI31038, SGI31038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7058[] = {SGI31039, SGI31040, SGI31041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31042, SGI0, SGI0, SGI0, SGI31043, SGI31043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7059[] = {SGI31044, SGI31045, SGI31046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31047, SGI0, SGI0, SGI0, SGI31048, SGI31048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7060[] = {SGI31049, SGI31050, SGI31051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31052, SGI0, SGI0, SGI0, SGI31053, SGI31054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7061[] = {SGI31055, SGI31056, SGI31057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31058, SGI0, SGI0, SGI0, SGI31059, SGI31059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7062[] = {SGI31060, SGI31061, SGI31062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31063, SGI0, SGI0, SGI0, SGI31064, SGI31064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7063[] = {SGI31065, SGI31066, SGI31067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31068, SGI0, SGI0, SGI0, SGI31069, SGI31069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7064[] = {SGI31070, SGI31071, SGI31072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31073, SGI0, SGI0, SGI0, SGI31074, SGI31074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7065[] = {SGI31075, SGI31076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31077, SGI0, SGI0, SGI0, SGI31078, SGI31078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7066[] = {SGI31079, SGI31080, SGI31081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31082, SGI0, SGI0, SGI0, SGI31083, SGI31083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7067[] = {SGI31084, SGI31085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31086, SGI0, SGI0, SGI0, SGI31087, SGI31087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7068[] = {SGI31088, SGI31089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31090, SGI0, SGI0, SGI0, SGI31091, SGI31091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7069[] = {SGI31092, SGI31093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31094, SGI0, SGI0, SGI0, SGI31095, SGI31095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7070[] = {SGI31096, SGI31097, SGI31098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31099, SGI0, SGI0, SGI0, SGI31100, SGI31100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7071[] = {SGI31101, SGI31102, SGI31103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31104, SGI0, SGI0, SGI0, SGI31105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7072[] = {SGI31106, SGI31107, SGI31108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31109, SGI0, SGI0, SGI0, SGI31110, SGI31111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7073[] = {SGI31112, SGI31113, SGI31114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31115, SGI0, SGI0, SGI0, SGI31116, SGI31116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7074[] = {SGI31117, SGI31118, SGI31119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31120, SGI0, SGI0, SGI0, SGI31121, SGI31121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7075[] = {SGI31122, SGI31123, SGI31124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31125, SGI0, SGI0, SGI0, SGI31126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7076[] = {SGI31127, SGI31128, SGI31129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31130, SGI0, SGI0, SGI0, SGI31131, SGI31131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7077[] = {SGI31132, SGI31133, SGI31134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31135, SGI0, SGI0, SGI0, SGI31136, SGI31136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7078[] = {SGI31137, SGI31138, SGI31139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31140, SGI0, SGI0, SGI0, SGI31141, SGI31141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7079[] = {SGI31142, SGI0, SGI31143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7080[] = {SGI31144, SGI31145, SGI31146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31147, SGI0, SGI0, SGI0, SGI31148, SGI31148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7081[] = {SGI31149, SGI31150, SGI31151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31152, SGI0, SGI0, SGI0, SGI31153, SGI31153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7082[] = {SGI31154, SGI31155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31156, SGI0, SGI0, SGI0, SGI31157, SGI31157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7083[] = {SGI31158, SGI31159, SGI31160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31161, SGI0, SGI0, SGI0, SGI31162, SGI31162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7084[] = {SGI31163, SGI31164, SGI31165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31166, SGI0, SGI0, SGI0, SGI31167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7085[] = {SGI31168, SGI31169, SGI31170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31171, SGI0, SGI0, SGI0, SGI31172, SGI31172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7086[] = {SGI31173, SGI31174, SGI31175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31176, SGI0, SGI0, SGI0, SGI31177, SGI31177, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7087[] = {SGI31178, SGI31179, SGI31180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31181, SGI0, SGI0, SGI0, SGI31182, SGI31182, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7088[] = {SGI31183, SGI0, SGI31184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7089[] = {SGI31185, SGI0, SGI31186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7090[] = {SGI31187, SGI31188, SGI31189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31190, SGI0, SGI0, SGI0, SGI31191, SGI31191, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7091[] = {SGI31192, SGI31193, SGI31194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31195, SGI0, SGI0, SGI0, SGI31196, SGI31196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7092[] = {SGI31197, SGI0, SGI31198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7093[] = {SGI31199, SGI0, SGI31200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31201, SGI0, SGI0, SGI0, SGI31202, SGI31202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7094[] = {SGI31203, SGI31204, SGI31205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31206, SGI0, SGI0, SGI0, SGI31207, SGI31207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7095[] = {SGI31208, SGI31209, SGI31210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31211, SGI0, SGI0, SGI0, SGI31212, SGI31212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7096[] = {SGI31213, SGI31214, SGI31215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31216, SGI0, SGI0, SGI0, SGI31217, SGI31217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7097[] = {SGI31218, SGI31219, SGI31220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31221, SGI0, SGI0, SGI0, SGI31222, SGI31222, SGI0, SGI0, NULL};
const Segment * const SG7098[] = {SGI31223, SGI31224, SGI31225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31226, SGI0, SGI0, SGI0, SGI31227, SGI31227, SGI0, SGI0, NULL};
const Segment * const SG7099[] = {SGI31228, SGI31229, SGI31230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31231, SGI0, SGI0, SGI0, SGI31232, SGI31232, SGI0, SGI0, NULL};
const Segment * const SG7100[] = {SGI31233, SGI31234, SGI31235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31239, SGI0, SGI0, SGI0, SGI31240, SGI31240, SGI0, SGI0, NULL};
const Segment * const SG7101[] = {SGI31236, SGI31237, SGI31238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31241, SGI0, SGI0, SGI0, SGI31242, SGI31242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7102[] = {SGI31243, SGI31244, SGI31245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31246, SGI0, SGI0, SGI0, SGI31247, SGI31247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7103[] = {SGI31248, SGI0, SGI31249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31250, SGI0, SGI0, SGI0, SGI31251, SGI31251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7104[] = {SGI31252, SGI31253, SGI31254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31255, SGI0, SGI0, SGI0, SGI31256, SGI31256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7105[] = {SGI31257, SGI31258, SGI31259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31260, SGI0, SGI0, SGI0, SGI31261, SGI31261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7106[] = {SGI31262, SGI31263, SGI31264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31265, SGI0, SGI0, SGI0, SGI31266, SGI31266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7107[] = {SGI31267, SGI31268, SGI31269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31270, SGI0, SGI0, SGI0, SGI31271, SGI31271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7108[] = {SGI31272, SGI31273, SGI31274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31275, SGI0, SGI0, SGI0, SGI31276, SGI31276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7109[] = {SGI31277, SGI31278, SGI31279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31280, SGI0, SGI0, SGI0, SGI31281, SGI31281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7110[] = {SGI31282, SGI31283, SGI31284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31285, SGI0, SGI0, SGI0, SGI31286, SGI31286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7111[] = {SGI31287, SGI31288, SGI31290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31293, SGI0, SGI0, SGI0, SGI31295, SGI31295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7112[] = {SGI31289, SGI31291, SGI31292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31294, SGI0, SGI0, SGI0, SGI31296, SGI31296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7113[] = {SGI31297, SGI31298, SGI31300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31303, SGI0, SGI0, SGI0, SGI31305, SGI31305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7114[] = {SGI31299, SGI31301, SGI31302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31304, SGI0, SGI0, SGI0, SGI31306, SGI31306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7115[] = {SGI31307, SGI31308, SGI31309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31310, SGI0, SGI0, SGI0, SGI31311, SGI31311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7116[] = {SGI31312, SGI31313, SGI31314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31315, SGI0, SGI0, SGI0, SGI31316, SGI31316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7117[] = {SGI31317, SGI31318, SGI31319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31320, SGI0, SGI0, SGI0, SGI31321, SGI31321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7118[] = {SGI31322, SGI31323, SGI31324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31325, SGI0, SGI0, SGI0, SGI31326, SGI31326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7119[] = {SGI31327, SGI31328, SGI31329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31330, SGI0, SGI0, SGI0, SGI31331, SGI31331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7120[] = {SGI31332, SGI31333, SGI31334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31335, SGI0, SGI0, SGI0, SGI31337, SGI31337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7121[] = {SGI31336, SGI31338, SGI31339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31340, SGI0, SGI0, SGI0, SGI31341, SGI31341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7122[] = {SGI31342, SGI31343, SGI31344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31346, SGI0, SGI0, SGI0, SGI31349, SGI31349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7123[] = {SGI31345, SGI31347, SGI31348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31350, SGI0, SGI0, SGI0, SGI31351, SGI31351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7124[] = {SGI31352, SGI31353, SGI31354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31358, SGI0, SGI0, SGI0, SGI31359, SGI31359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7125[] = {SGI31355, SGI31356, SGI31357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31360, SGI0, SGI0, SGI0, SGI31361, SGI31361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7126[] = {SGI31362, SGI31364, SGI31366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31368, SGI0, SGI0, SGI0, SGI31370, SGI31370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7127[] = {SGI31363, SGI31365, SGI31367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31369, SGI0, SGI0, SGI0, SGI31371, SGI31371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7128[] = {SGI31372, SGI31373, SGI31374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31375, SGI0, SGI0, SGI0, SGI31376, SGI31376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7129[] = {SGI31377, SGI31378, SGI31379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31380, SGI0, SGI0, SGI0, SGI31381, SGI31381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7130[] = {SGI31382, SGI0, SGI31383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7131[] = {SGI31384, SGI31385, SGI31386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31387, SGI0, SGI0, SGI0, SGI31388, SGI31388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7132[] = {SGI31389, SGI31390, SGI31391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31392, SGI0, SGI0, SGI0, SGI31393, SGI31393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7133[] = {SGI31394, SGI31395, SGI31396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31397, SGI0, SGI0, SGI0, SGI31398, SGI31398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7134[] = {SGI31399, SGI31400, SGI31401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31402, SGI0, SGI0, SGI0, SGI31403, SGI31403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7135[] = {SGI31404, SGI31405, SGI31406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31407, SGI0, SGI0, SGI0, SGI31408, SGI31408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7136[] = {SGI31409, SGI0, SGI31410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7137[] = {SGI31411, SGI31412, SGI31413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31414, SGI0, SGI0, SGI0, SGI31415, SGI31415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7138[] = {SGI31416, SGI31417, SGI31418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31422, SGI0, SGI0, SGI0, SGI31423, SGI31423, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7139[] = {SGI31419, SGI31420, SGI31421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7140[] = {SGI31424, SGI0, SGI31425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7141[] = {SGI31426, SGI31427, SGI31428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31430, SGI0, SGI0, SGI0, SGI31433, SGI31433, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7142[] = {SGI31429, SGI31431, SGI31432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31434, SGI0, SGI0, SGI0, SGI31435, SGI31435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7143[] = {SGI31436, SGI0, SGI31437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7144[] = {SGI31438, SGI31439, SGI31440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31441, SGI0, SGI0, SGI0, SGI31442, SGI31442, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7145[] = {SGI31443, SGI31444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31445, SGI0, SGI0, SGI0, SGI31446, SGI31446, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7146[] = {SGI31447, SGI31448, SGI31449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31450, SGI0, SGI0, SGI0, SGI31451, SGI31451, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7147[] = {SGI31452, SGI0, SGI31453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31454, SGI0, SGI0, SGI0, SGI31455, SGI31455, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7148[] = {SGI31456, SGI31457, SGI31458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31459, SGI0, SGI0, SGI0, SGI31460, SGI31460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7149[] = {SGI31461, SGI31462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31463, SGI0, SGI0, SGI0, SGI31464, SGI31464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7150[] = {SGI31465, SGI31466, SGI31467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31468, SGI0, SGI0, SGI0, SGI31469, SGI31469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7151[] = {SGI31470, SGI31471, SGI31472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31474, SGI0, SGI0, SGI0, SGI31477, SGI31477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7152[] = {SGI31473, SGI31475, SGI31476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31478, SGI0, SGI0, SGI0, SGI31479, SGI31479, SGI0, SGI0, NULL};
const Segment * const SG7153[] = {SGI31480, SGI31481, SGI31482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31483, SGI0, SGI0, SGI0, SGI31484, SGI31484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7154[] = {SGI31485, SGI31486, SGI31487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31488, SGI0, SGI0, SGI0, SGI31489, SGI31489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7155[] = {SGI31490, SGI31491, SGI31492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31493, SGI0, SGI0, SGI0, SGI31494, SGI31494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7156[] = {SGI31495, SGI31496, SGI31497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31498, SGI0, SGI0, SGI0, SGI31499, SGI31499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7157[] = {SGI31500, SGI31501, SGI31502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31503, SGI0, SGI0, SGI0, SGI31504, SGI31504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7158[] = {SGI31505, SGI31506, SGI31507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31508, SGI0, SGI0, SGI0, SGI31509, SGI31509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7159[] = {SGI31510, SGI31511, SGI31512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31513, SGI0, SGI0, SGI0, SGI31514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7160[] = {SGI31515, SGI31516, SGI31517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31518, SGI0, SGI0, SGI0, SGI31519, SGI31519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7161[] = {SGI31520, SGI31521, SGI31522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31523, SGI0, SGI0, SGI0, SGI31524, SGI31524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7162[] = {SGI31525, SGI31526, SGI31527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31528, SGI0, SGI0, SGI0, SGI31529, SGI31529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7163[] = {SGI31530, SGI31531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31532, SGI0, SGI0, SGI0, SGI31533, SGI31533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7164[] = {SGI31534, SGI31535, SGI31536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31537, SGI0, SGI0, SGI0, SGI31538, SGI31538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7165[] = {SGI31539, SGI31540, SGI31541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31542, SGI0, SGI0, SGI0, SGI31543, SGI31543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7166[] = {SGI31544, SGI31545, SGI31546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31547, SGI0, SGI0, SGI0, SGI31548, SGI31548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7167[] = {SGI31549, SGI0, SGI31550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7168[] = {SGI31551, SGI31552, SGI31553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31554, SGI0, SGI0, SGI0, SGI31555, SGI31555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7169[] = {SGI31556, SGI31557, SGI31558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31559, SGI0, SGI0, SGI0, SGI31560, SGI31561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7170[] = {SGI31562, SGI31563, SGI31564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31565, SGI0, SGI0, SGI0, SGI31566, SGI31566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7171[] = {SGI31567, SGI31568, SGI31569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31570, SGI0, SGI0, SGI0, SGI31571, SGI31571, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7172[] = {SGI31572, SGI0, SGI31573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7173[] = {SGI31574, SGI31575, SGI31576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31577, SGI0, SGI0, SGI0, SGI31578, SGI31578, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7174[] = {SGI31579, SGI31580, SGI31581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31582, SGI0, SGI0, SGI0, SGI31583, SGI31583, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7175[] = {SGI31584, SGI31585, SGI31586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31587, SGI0, SGI0, SGI0, SGI31588, SGI31588, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7176[] = {SGI31589, SGI31590, SGI31591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31592, SGI0, SGI0, SGI0, SGI31593, SGI31593, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7177[] = {SGI31594, SGI31595, SGI31596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31597, SGI0, SGI0, SGI0, SGI31598, SGI31598, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7178[] = {SGI31599, SGI31600, SGI31601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31602, SGI0, SGI0, SGI0, SGI31603, SGI31603, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7179[] = {SGI31604, SGI31605, SGI31606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31607, SGI0, SGI0, SGI0, SGI31608, SGI31608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7180[] = {SGI31609, SGI31610, SGI31611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31612, SGI0, SGI0, SGI0, SGI31613, SGI31613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7181[] = {SGI31614, SGI31615, SGI31616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31617, SGI0, SGI0, SGI0, SGI31618, SGI31618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7182[] = {SGI31619, SGI31620, SGI31621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31622, SGI0, SGI0, SGI0, SGI31623, SGI31623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7183[] = {SGI31624, SGI31625, SGI31626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31627, SGI0, SGI0, SGI0, SGI31628, SGI31628, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7184[] = {SGI31629, SGI31630, SGI31631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31632, SGI0, SGI0, SGI0, SGI31633, SGI31633, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7185[] = {SGI31634, SGI31635, SGI31636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31637, SGI0, SGI0, SGI0, SGI31638, SGI31638, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7186[] = {SGI31639, SGI31640, SGI31641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31642, SGI0, SGI0, SGI0, SGI31643, SGI31643, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7187[] = {SGI31644, SGI31645, SGI31646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31647, SGI0, SGI0, SGI0, SGI31648, SGI31648, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7188[] = {SGI31649, SGI31650, SGI31651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31652, SGI0, SGI0, SGI0, SGI31653, SGI31653, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7189[] = {SGI31654, SGI31655, SGI31656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31657, SGI0, SGI0, SGI0, SGI31658, SGI31658, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7190[] = {SGI31659, SGI31660, SGI31661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31663, SGI0, SGI0, SGI0, SGI31666, SGI31666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7191[] = {SGI31662, SGI31664, SGI31665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31667, SGI0, SGI0, SGI0, SGI31668, SGI31668, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7192[] = {SGI31669, SGI31670, SGI31671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31672, SGI0, SGI0, SGI0, SGI31673, SGI31673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7193[] = {SGI31674, SGI31675, SGI31676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31677, SGI0, SGI0, SGI0, SGI31679, SGI31679, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7194[] = {SGI31678, SGI0, SGI31680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7195[] = {SGI31681, SGI31682, SGI31684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31687, SGI0, SGI0, SGI0, SGI31688, SGI31688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7196[] = {SGI31683, SGI31685, SGI31686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31689, SGI0, SGI0, SGI0, SGI31690, SGI31690, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7197[] = {SGI31691, SGI31692, SGI31693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31696, SGI0, SGI0, SGI0, SGI31697, SGI31697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7198[] = {SGI31694, SGI0, SGI31695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31698, SGI0, SGI0, SGI0, SGI31700, SGI31700, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7199[] = {SGI31699, SGI31701, SGI31702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31703, SGI0, SGI0, SGI0, SGI31704, SGI31704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7200[] = {SGI31705, SGI31707, SGI31709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31711, SGI0, SGI0, SGI0, SGI31713, SGI31713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7201[] = {SGI31706, SGI31708, SGI31710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31712, SGI0, SGI0, SGI0, SGI31714, SGI31714, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7202[] = {SGI31715, SGI31716, SGI31717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31718, SGI0, SGI0, SGI0, SGI31719, SGI31719, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7203[] = {SGI31720, SGI31721, SGI31723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31726, SGI0, SGI0, SGI0, SGI31728, SGI31728, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7204[] = {SGI31722, SGI31724, SGI31725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31727, SGI31727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7205[] = {SGI31729, SGI31730, SGI31731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31732, SGI0, SGI0, SGI0, SGI31735, SGI31735, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7206[] = {SGI31733, SGI31734, SGI31736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31737, SGI0, SGI0, SGI0, SGI31738, SGI31738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7207[] = {SGI31739, SGI31740, SGI31741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31743, SGI0, SGI0, SGI0, SGI31746, SGI31746, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7208[] = {SGI31742, SGI31744, SGI31745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31747, SGI0, SGI0, SGI0, SGI31748, SGI31748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7209[] = {SGI31749, SGI31750, SGI31751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31753, SGI0, SGI0, SGI0, SGI31756, SGI31756, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7210[] = {SGI31752, SGI31754, SGI31755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31757, SGI0, SGI0, SGI0, SGI31758, SGI31758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7211[] = {SGI31759, SGI31760, SGI31761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31762, SGI0, SGI0, SGI0, SGI31764, SGI31764, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7212[] = {SGI31763, SGI31765, SGI31766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31767, SGI0, SGI0, SGI0, SGI31768, SGI31768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7213[] = {SGI31769, SGI31770, SGI31771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31772, SGI0, SGI0, SGI0, SGI31774, SGI31774, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7214[] = {SGI31776, SGI31777, SGI31778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31779, SGI0, SGI0, SGI0, SGI31780, SGI31780, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7215[] = {SGI31781, SGI31782, SGI31783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31784, SGI0, SGI0, SGI0, SGI31785, SGI31785, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7216[] = {SGI31786, SGI31787, SGI31788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31789, SGI0, SGI0, SGI0, SGI31790, SGI31790, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7217[] = {SGI31791, SGI31792, SGI31793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31794, SGI0, SGI0, SGI0, SGI31795, SGI31795, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7218[] = {SGI31796, SGI31797, SGI31798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31799, SGI0, SGI0, SGI0, SGI31800, SGI31800, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7219[] = {SGI31801, SGI31802, SGI31803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31804, SGI0, SGI0, SGI0, SGI31805, SGI31805, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7220[] = {SGI31806, SGI31807, SGI31808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31809, SGI0, SGI0, SGI0, SGI31810, SGI31810, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7221[] = {SGI31811, SGI31812, SGI31813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31814, SGI0, SGI0, SGI0, SGI31815, SGI31815, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7222[] = {SGI31816, SGI31817, SGI31818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31819, SGI0, SGI0, SGI0, SGI31820, SGI31820, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7223[] = {SGI31821, SGI31822, SGI31823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31824, SGI0, SGI0, SGI0, SGI31825, SGI31825, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7224[] = {SGI31826, SGI31827, SGI31828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31829, SGI0, SGI0, SGI0, SGI31830, SGI31830, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7225[] = {SGI31831, SGI31832, SGI31833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31834, SGI0, SGI0, SGI0, SGI31835, SGI31835, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7226[] = {SGI31836, SGI31837, SGI31838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31839, SGI0, SGI0, SGI0, SGI31840, SGI31840, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7227[] = {SGI31841, SGI31842, SGI31843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31844, SGI0, SGI0, SGI0, SGI31845, SGI31845, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7228[] = {SGI31846, SGI31847, SGI31848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31849, SGI0, SGI0, SGI0, SGI31850, SGI31850, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7229[] = {SGI31851, SGI31852, SGI31853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31854, SGI0, SGI0, SGI0, SGI31855, SGI31855, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7230[] = {SGI31856, SGI31857, SGI31858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31859, SGI0, SGI0, SGI0, SGI31860, SGI31860, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7231[] = {SGI31861, SGI31862, SGI31863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31864, SGI0, SGI0, SGI0, SGI31865, SGI31865, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7232[] = {SGI31866, SGI31867, SGI31868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31869, SGI0, SGI0, SGI0, SGI31870, SGI31870, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7233[] = {SGI31871, SGI31872, SGI31873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31874, SGI0, SGI0, SGI0, SGI31875, SGI31875, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7234[] = {SGI31876, SGI31877, SGI31878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31879, SGI0, SGI0, SGI0, SGI31880, SGI31880, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7235[] = {SGI31881, SGI31882, SGI31883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31884, SGI0, SGI0, SGI0, SGI31885, SGI31885, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7236[] = {SGI31886, SGI31887, SGI31888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31889, SGI0, SGI0, SGI0, SGI31890, SGI31890, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7237[] = {SGI31891, SGI31892, SGI31893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31894, SGI0, SGI0, SGI0, SGI31895, SGI31895, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7238[] = {SGI31896, SGI31897, SGI31898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31899, SGI0, SGI0, SGI0, SGI31900, SGI31900, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7239[] = {SGI31901, SGI31902, SGI31903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31906, SGI0, SGI0, SGI0, SGI31908, SGI31908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7240[] = {SGI31904, SGI31905, SGI31907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31909, SGI0, SGI0, SGI0, SGI31910, SGI31910, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7241[] = {SGI31911, SGI31912, SGI31913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31914, SGI0, SGI0, SGI0, SGI31915, SGI31915, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7242[] = {SGI31916, SGI31917, SGI31918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31919, SGI0, SGI0, SGI0, SGI31920, SGI31920, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7243[] = {SGI31921, SGI31922, SGI31923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31924, SGI0, SGI0, SGI0, SGI31925, SGI31925, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7244[] = {SGI31926, SGI31927, SGI31928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31929, SGI0, SGI0, SGI0, SGI31930, SGI31930, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7245[] = {SGI31931, SGI31932, SGI31933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31934, SGI0, SGI0, SGI0, SGI31935, SGI31935, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7246[] = {SGI31936, SGI0, SGI31937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7247[] = {SGI31938, SGI31939, SGI31940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31941, SGI0, SGI0, SGI0, SGI31942, SGI31942, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7248[] = {SGI31943, SGI31944, SGI31945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31946, SGI31947, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7249[] = {SGI31948, SGI31949, SGI31950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31951, SGI0, SGI0, SGI0, SGI31952, SGI31952, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7250[] = {SGI31953, SGI31954, SGI31955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31956, SGI0, SGI0, SGI0, SGI31957, SGI31957, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7251[] = {SGI31958, SGI31959, SGI31960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31961, SGI31961, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7252[] = {SGI31962, SGI31963, SGI31964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31965, SGI0, SGI0, SGI0, SGI31966, SGI31966, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7253[] = {SGI31967, SGI31968, SGI31969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31970, SGI0, SGI0, SGI0, SGI31971, SGI31971, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7254[] = {SGI31972, SGI31973, SGI31974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31975, SGI31975, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7255[] = {SGI31773, SGI31775, SGI31976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31977, SGI0, SGI0, SGI0, SGI31978, SGI31978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7256[] = {SGI31979, SGI31980, SGI31981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31982, SGI0, SGI0, SGI0, SGI31983, SGI31983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7257[] = {SGI31984, SGI31985, SGI31986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31987, SGI0, SGI0, SGI0, SGI31988, SGI31988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7258[] = {SGI31989, SGI0, SGI31990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7259[] = {SGI31991, SGI31992, SGI31993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31994, SGI0, SGI0, SGI0, SGI31995, SGI31995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7260[] = {SGI31996, SGI31997, SGI31998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI31999, SGI0, SGI0, SGI0, SGI32000, SGI32000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7261[] = {SGI32001, SGI32002, SGI32003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32004, SGI32005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7262[] = {SGI32006, SGI32007, SGI32008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32009, SGI0, SGI0, SGI0, SGI32010, SGI32010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7263[] = {SGI32011, SGI32012, SGI32013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32014, SGI32015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7264[] = {SGI32016, SGI32017, SGI32018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32019, SGI0, SGI0, SGI0, SGI32020, SGI32020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7265[] = {SGI32021, SGI32022, SGI32023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32024, SGI32024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7266[] = {SGI32025, SGI32026, SGI32027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32028, SGI32028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7267[] = {SGI32029, SGI32030, SGI32031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32032, SGI0, SGI0, SGI0, SGI32033, SGI32033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7268[] = {SGI32034, SGI32035, SGI32036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32037, SGI0, SGI0, SGI0, SGI32038, SGI32038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7269[] = {SGI32039, SGI32040, SGI32041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32042, SGI0, SGI0, SGI0, SGI32043, SGI32043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7270[] = {SGI32044, SGI0, SGI32045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7271[] = {SGI32046, SGI0, SGI32047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7272[] = {SGI32048, SGI0, SGI32049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7273[] = {SGI32050, SGI32051, SGI32052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32053, SGI0, SGI0, SGI0, SGI32054, SGI32054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7274[] = {SGI32055, SGI32056, SGI32057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32058, SGI0, SGI0, SGI0, SGI32059, SGI32059, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7275[] = {SGI32060, SGI0, SGI32061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7276[] = {SGI32062, SGI32063, SGI32064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7277[] = {SGI32065, SGI32066, SGI32067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32068, SGI0, SGI0, SGI0, SGI32069, SGI32069, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7278[] = {SGI32070, SGI32071, SGI32072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32073, SGI0, SGI0, SGI0, SGI32074, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7279[] = {SGI32075, SGI32076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32077, SGI0, SGI0, SGI0, SGI32078, SGI32078, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7280[] = {SGI32079, SGI32080, SGI32081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32082, SGI0, SGI0, SGI0, SGI32083, SGI32083, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7281[] = {SGI32084, SGI32085, SGI32086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32087, SGI0, SGI0, SGI0, SGI32088, SGI32088, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7282[] = {SGI32089, SGI32090, SGI32091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32092, SGI0, SGI0, SGI0, SGI32093, SGI32094, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7283[] = {SGI32095, SGI0, SGI32096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32097, SGI0, SGI0, SGI0, SGI32098, SGI32098, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7284[] = {SGI32099, SGI32100, SGI32101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32102, SGI0, SGI0, SGI0, SGI32103, SGI32103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7285[] = {SGI32104, SGI32105, SGI32106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32107, SGI0, SGI0, SGI0, SGI32108, SGI32108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7286[] = {SGI32109, SGI32110, SGI32111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32112, SGI0, SGI0, SGI0, SGI32113, SGI32113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7287[] = {SGI32114, SGI32115, SGI32116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32117, SGI0, SGI0, SGI0, SGI32118, SGI32118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7288[] = {SGI32119, SGI32120, SGI32121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32122, SGI0, SGI0, SGI0, SGI32123, SGI32123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7289[] = {SGI32124, SGI32125, SGI32126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32127, SGI0, SGI0, SGI0, SGI32128, SGI32128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7290[] = {SGI32129, SGI32130, SGI32131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32132, SGI0, SGI0, SGI0, SGI32133, SGI32133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7291[] = {SGI32134, SGI0, SGI32135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32136, SGI0, SGI0, SGI0, SGI32137, SGI32137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7292[] = {SGI32138, SGI32139, SGI32140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32141, SGI0, SGI0, SGI0, SGI32142, SGI32142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7293[] = {SGI32143, SGI32144, SGI32145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32146, SGI0, SGI0, SGI0, SGI32147, SGI32147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7294[] = {SGI32148, SGI0, SGI32149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7295[] = {SGI32150, SGI32151, SGI32152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32153, SGI0, SGI0, SGI0, SGI32154, SGI32154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7296[] = {SGI32155, SGI32156, SGI32157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32158, SGI0, SGI0, SGI0, SGI32159, SGI32159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7297[] = {SGI32160, SGI32161, SGI32162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32163, SGI0, SGI0, SGI0, SGI32164, SGI32164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7298[] = {SGI32165, SGI0, SGI32166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7299[] = {SGI32167, SGI32168, SGI32169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32172, SGI32174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7300[] = {SGI32170, SGI32171, SGI32173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32175, SGI0, SGI0, SGI0, SGI32176, SGI32176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7301[] = {SGI32177, SGI32178, SGI32179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32180, SGI0, SGI0, SGI0, SGI32181, SGI32181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7302[] = {SGI32182, SGI0, SGI32183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7303[] = {SGI32184, SGI32185, SGI32186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32187, SGI0, SGI0, SGI0, SGI32188, SGI32188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7304[] = {SGI32189, SGI32190, SGI32191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32192, SGI0, SGI0, SGI0, SGI32193, SGI32193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7305[] = {SGI32194, SGI32195, SGI32196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32197, SGI0, SGI0, SGI0, SGI32198, SGI32198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7306[] = {SGI32199, SGI32200, SGI32201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32202, SGI0, SGI0, SGI0, SGI32203, SGI32204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7307[] = {SGI32205, SGI32206, SGI32207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32208, SGI0, SGI0, SGI0, SGI32209, SGI32209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7308[] = {SGI32210, SGI32211, SGI32212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32213, SGI32213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7309[] = {SGI32214, SGI32215, SGI32216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32217, SGI0, SGI0, SGI0, SGI32218, SGI32218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7310[] = {SGI32219, SGI32220, SGI32221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32222, SGI0, SGI0, SGI0, SGI32223, SGI32223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7311[] = {SGI32224, SGI32225, SGI32226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32227, SGI32227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7312[] = {SGI32228, SGI32229, SGI32230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32231, SGI0, SGI0, SGI0, SGI32232, SGI32232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7313[] = {SGI32233, SGI0, SGI32234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32235, SGI0, SGI0, SGI0, SGI32236, SGI32236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7314[] = {SGI32237, SGI32238, SGI32239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32240, SGI0, SGI0, SGI0, SGI32241, SGI32241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7315[] = {SGI32242, SGI32243, SGI32244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32245, SGI32246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7316[] = {SGI32247, SGI32248, SGI32249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32250, SGI0, SGI0, SGI0, SGI32251, SGI32251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7317[] = {SGI32252, SGI32253, SGI32254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32255, SGI32255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7318[] = {SGI32256, SGI32257, SGI32258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32259, SGI0, SGI0, SGI0, SGI32260, SGI32260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7319[] = {SGI32261, SGI32262, SGI32263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32264, SGI0, SGI0, SGI0, SGI32265, SGI32265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7320[] = {SGI32266, SGI32267, SGI32268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32269, SGI0, SGI0, SGI0, SGI32270, SGI32270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7321[] = {SGI32271, SGI32272, SGI32273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32274, SGI0, SGI0, SGI0, SGI32275, SGI32275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7322[] = {SGI32276, SGI32277, SGI32278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32279, SGI0, SGI0, SGI0, SGI32280, SGI32280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7323[] = {SGI32281, SGI32282, SGI32283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32284, SGI0, SGI0, SGI0, SGI32285, SGI32285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7324[] = {SGI32286, SGI32287, SGI32288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32289, SGI0, SGI0, SGI0, SGI32290, SGI32290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7325[] = {SGI32291, SGI32292, SGI32293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32294, SGI0, SGI0, SGI0, SGI32295, SGI32295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7326[] = {SGI32296, SGI32297, SGI32298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32299, SGI0, SGI0, SGI0, SGI32300, SGI32300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7327[] = {SGI32301, SGI32302, SGI32303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32304, SGI0, SGI0, SGI0, SGI32305, SGI32305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7328[] = {SGI32306, SGI32307, SGI32308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32309, SGI0, SGI0, SGI0, SGI32310, SGI32310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7329[] = {SGI32311, SGI32312, SGI32313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32314, SGI0, SGI0, SGI0, SGI32315, SGI32315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7330[] = {SGI32316, SGI32317, SGI32318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32319, SGI0, SGI0, SGI0, SGI32320, SGI32320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7331[] = {SGI0, SGI32321, SGI32322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32323, SGI0, SGI0, SGI0, SGI32324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7332[] = {SGI0, SGI32325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32326, SGI0, SGI0, SGI0, SGI32327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7333[] = {SGI32328, SGI0, SGI32329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7334[] = {SGI32330, SGI32331, SGI32332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32333, SGI32334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7335[] = {SGI32335, SGI32336, SGI32337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32338, SGI32338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7336[] = {SGI32339, SGI0, SGI32340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32341, SGI0, SGI0, SGI0, SGI32342, SGI32342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7337[] = {SGI32343, SGI32344, SGI32345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32346, SGI0, SGI0, SGI0, SGI32347, SGI32347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7338[] = {SGI32348, SGI32349, SGI32350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32351, SGI0, SGI0, SGI0, SGI32352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7339[] = {SGI32353, SGI32354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32355, SGI0, SGI0, SGI0, SGI32356, SGI32356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7340[] = {SGI32357, SGI32358, SGI32359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32360, SGI0, SGI0, SGI0, SGI32361, SGI32361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7341[] = {SGI32362, SGI32363, SGI32364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32365, SGI0, SGI0, SGI0, SGI32366, SGI32366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7342[] = {SGI32367, SGI32368, SGI32369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32370, SGI0, SGI0, SGI0, SGI32371, SGI32371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7343[] = {SGI32372, SGI32373, SGI32374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32375, SGI0, SGI0, SGI0, SGI32376, SGI32376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7344[] = {SGI32377, SGI32378, SGI32379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32380, SGI0, SGI0, SGI0, SGI32381, SGI32381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7345[] = {SGI32382, SGI32383, SGI32384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32385, SGI0, SGI0, SGI0, SGI32386, SGI32386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7346[] = {SGI32387, SGI32388, SGI32389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32390, SGI0, SGI0, SGI0, SGI32391, SGI32391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7347[] = {SGI32392, SGI32393, SGI32394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32395, SGI0, SGI0, SGI0, SGI32396, SGI32396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7348[] = {SGI32397, SGI32398, SGI32399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32400, SGI0, SGI0, SGI0, SGI32401, SGI32401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7349[] = {SGI32402, SGI0, SGI32403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7350[] = {SGI32404, SGI32405, SGI32406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32407, SGI0, SGI0, SGI0, SGI32408, SGI32408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7351[] = {SGI32409, SGI32410, SGI32411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32412, SGI0, SGI0, SGI0, SGI32413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7352[] = {SGI32414, SGI32415, SGI32416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32417, SGI0, SGI0, SGI0, SGI32418, SGI32418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7353[] = {SGI32419, SGI0, SGI32420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32421, SGI0, SGI0, SGI0, SGI32422, SGI32422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7354[] = {SGI32423, SGI32424, SGI32425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32426, SGI0, SGI0, SGI0, SGI32427, SGI32427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7355[] = {SGI32428, SGI32429, SGI32430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32431, SGI0, SGI0, SGI0, SGI32432, SGI32432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7356[] = {SGI32433, SGI32434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32435, SGI0, SGI0, SGI0, SGI32436, SGI32436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7357[] = {SGI32437, SGI32438, SGI32439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32443, SGI0, SGI0, SGI0, SGI32444, SGI32444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7358[] = {SGI32440, SGI32441, SGI32442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32445, SGI0, SGI0, SGI0, SGI32446, SGI32446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7359[] = {SGI32447, SGI32448, SGI32449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32451, SGI0, SGI0, SGI0, SGI32454, SGI32454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7360[] = {SGI32450, SGI32452, SGI32453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32455, SGI0, SGI0, SGI0, SGI32456, SGI32456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7361[] = {SGI32457, SGI32458, SGI32459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32460, SGI0, SGI0, SGI0, SGI32461, SGI32461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7362[] = {SGI32462, SGI32463, SGI32464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32465, SGI0, SGI0, SGI0, SGI32466, SGI32466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7363[] = {SGI32467, SGI32468, SGI32469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32470, SGI0, SGI0, SGI0, SGI32472, SGI32472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7364[] = {SGI32471, SGI0, SGI32473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7365[] = {SGI32474, SGI32475, SGI32476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32477, SGI32478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7366[] = {SGI32479, SGI32480, SGI32481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32482, SGI0, SGI0, SGI0, SGI32483, SGI32483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7367[] = {SGI32484, SGI32485, SGI32486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32489, SGI32489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7368[] = {SGI32487, SGI32488, SGI32490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32491, SGI0, SGI0, SGI0, SGI32492, SGI32492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7369[] = {SGI32493, SGI32494, SGI32495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32496, SGI0, SGI0, SGI0, SGI32497, SGI32497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7370[] = {SGI32498, SGI32499, SGI32500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32501, SGI0, SGI0, SGI0, SGI32502, SGI32502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7371[] = {SGI32503, SGI32504, SGI32505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32506, SGI0, SGI0, SGI0, SGI32507, SGI32507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7372[] = {SGI32508, SGI32509, SGI32510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32511, SGI0, SGI0, SGI0, SGI32512, SGI32512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7373[] = {SGI32513, SGI0, SGI32514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7374[] = {SGI32515, SGI32516, SGI32517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32518, SGI0, SGI0, SGI0, SGI32519, SGI32519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7375[] = {SGI32520, SGI0, SGI32521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32522, SGI0, SGI0, SGI0, SGI32523, SGI32523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7376[] = {SGI32524, SGI32525, SGI32526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32527, SGI0, SGI0, SGI0, SGI32528, SGI32528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7377[] = {SGI32529, SGI32530, SGI32531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32532, SGI0, SGI0, SGI0, SGI32533, SGI32533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7378[] = {SGI32534, SGI32535, SGI32536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32537, SGI0, SGI0, SGI0, SGI32538, SGI32538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7379[] = {SGI32539, SGI32540, SGI32541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32542, SGI32542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7380[] = {SGI32543, SGI32544, SGI32545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32546, SGI32546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7381[] = {SGI32547, SGI32548, SGI32549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32550, SGI32551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7382[] = {SGI32552, SGI32553, SGI32554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32555, SGI0, SGI0, SGI0, SGI32556, SGI32556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7383[] = {SGI32557, SGI32558, SGI32559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32560, SGI0, SGI0, SGI0, SGI32561, SGI32561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7384[] = {SGI32562, SGI32563, SGI32564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32565, SGI0, SGI0, SGI0, SGI32566, SGI32566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7385[] = {SGI32567, SGI0, SGI32568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7386[] = {SGI32569, SGI32570, SGI32571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32572, SGI0, SGI0, SGI0, SGI32573, SGI32573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7387[] = {SGI32574, SGI32575, SGI32576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32577, SGI0, SGI0, SGI0, SGI32578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7388[] = {SGI32579, SGI32580, SGI32581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32582, SGI0, SGI0, SGI0, SGI32583, SGI32583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7389[] = {SGI32584, SGI32585, SGI32586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32589, SGI0, SGI0, SGI0, SGI32590, SGI32590, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7390[] = {SGI32587, SGI32588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32591, SGI0, SGI0, SGI0, SGI32592, SGI32592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7391[] = {SGI32593, SGI32594, SGI32595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32596, SGI0, SGI0, SGI0, SGI32599, SGI32599, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7392[] = {SGI32597, SGI32598, SGI32600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32601, SGI0, SGI0, SGI0, SGI32602, SGI32602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7393[] = {SGI32603, SGI32604, SGI32605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32606, SGI0, SGI0, SGI0, SGI32607, SGI32607, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7394[] = {SGI32608, SGI32609, SGI32610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32611, SGI0, SGI0, SGI0, SGI32612, SGI32612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7395[] = {SGI32613, SGI32614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32615, SGI0, SGI0, SGI0, SGI32616, SGI32616, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7396[] = {SGI32617, SGI32618, SGI32619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32620, SGI0, SGI0, SGI0, SGI32621, SGI32621, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7397[] = {SGI32622, SGI32623, SGI32624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32625, SGI0, SGI0, SGI0, SGI32626, SGI32626, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7398[] = {SGI32627, SGI32628, SGI32629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32632, SGI0, SGI0, SGI0, SGI32634, SGI32635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7399[] = {SGI32630, SGI32631, SGI32633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32636, SGI0, SGI0, SGI0, SGI32637, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7400[] = {SGI32638, SGI0, SGI32639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7401[] = {SGI32640, SGI0, SGI32642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7402[] = {SGI32641, SGI32643, SGI32644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32645, SGI0, SGI0, SGI0, SGI32646, SGI32646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7403[] = {SGI32647, SGI0, SGI32648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32649, SGI0, SGI0, SGI0, SGI32650, SGI32650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7404[] = {SGI32651, SGI32652, SGI32653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32654, SGI32654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7405[] = {SGI32655, SGI32656, SGI32657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32658, SGI0, SGI0, SGI0, SGI32659, SGI32659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7406[] = {SGI32660, SGI32661, SGI32662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32663, SGI0, SGI0, SGI0, SGI32664, SGI32664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7407[] = {SGI32665, SGI32666, SGI32667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32668, SGI0, SGI0, SGI0, SGI32669, SGI32669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7408[] = {SGI32670, SGI32671, SGI32672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32673, SGI0, SGI0, SGI0, SGI32674, SGI32674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7409[] = {SGI32675, SGI32676, SGI32677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32678, SGI0, SGI0, SGI0, SGI32679, SGI32679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7410[] = {SGI32680, SGI32681, SGI32682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32683, SGI0, SGI0, SGI0, SGI32684, SGI32684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7411[] = {SGI32685, SGI32686, SGI32687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32688, SGI0, SGI0, SGI0, SGI32689, SGI32689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7412[] = {SGI32690, SGI32691, SGI32692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32693, SGI0, SGI0, SGI0, SGI32694, SGI32694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7413[] = {SGI32695, SGI32696, SGI32697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32698, SGI32699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7414[] = {SGI32700, SGI0, SGI32701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32702, SGI0, SGI0, SGI0, SGI32703, SGI32703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7415[] = {SGI32704, SGI32705, SGI32706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32707, SGI0, SGI0, SGI0, SGI32708, SGI32708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7416[] = {SGI32709, SGI32710, SGI32711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32712, SGI0, SGI0, SGI0, SGI32713, SGI32713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7417[] = {SGI32714, SGI32715, SGI32716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32717, SGI0, SGI0, SGI0, SGI32718, SGI32718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7418[] = {SGI32719, SGI0, SGI32720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7419[] = {SGI32721, SGI32722, SGI32723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32724, SGI0, SGI0, SGI0, SGI32725, SGI32725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7420[] = {SGI32726, SGI32727, SGI32728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32729, SGI0, SGI0, SGI0, SGI32730, SGI32730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7421[] = {SGI32731, SGI32732, SGI32733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32734, SGI0, SGI0, SGI0, SGI32735, SGI32735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7422[] = {SGI32736, SGI32737, SGI32738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32739, SGI0, SGI0, SGI0, SGI32740, SGI32740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7423[] = {SGI32741, SGI32742, SGI32743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32744, SGI0, SGI0, SGI0, SGI32745, SGI32745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7424[] = {SGI32746, SGI32747, SGI32748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32749, SGI32750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7425[] = {SGI32751, SGI32752, SGI32753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32754, SGI0, SGI0, SGI0, SGI32755, SGI32755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7426[] = {SGI32756, SGI32757, SGI32758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32759, SGI0, SGI0, SGI0, SGI32760, SGI32760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7427[] = {SGI32761, SGI32762, SGI32763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32764, SGI0, SGI0, SGI0, SGI32765, SGI32765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7428[] = {SGI32766, SGI32767, SGI32768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32769, SGI0, SGI0, SGI0, SGI32770, SGI32770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7429[] = {SGI32771, SGI32772, SGI32773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32774, SGI0, SGI0, SGI0, SGI32775, SGI32775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7430[] = {SGI32776, SGI32777, SGI32778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32779, SGI32779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7431[] = {SGI32780, SGI32781, SGI32782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32783, SGI32783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7432[] = {SGI32784, SGI32785, SGI32786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32787, SGI0, SGI0, SGI0, SGI32788, SGI32788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7433[] = {SGI32789, SGI32790, SGI32791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32792, SGI0, SGI0, SGI0, SGI32793, SGI32793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7434[] = {SGI32794, SGI32795, SGI32796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32797, SGI0, SGI0, SGI0, SGI32798, SGI32799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7435[] = {SGI32800, SGI32801, SGI32802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32803, SGI0, SGI0, SGI0, SGI32804, SGI32804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7436[] = {SGI32805, SGI0, SGI32806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7437[] = {SGI32807, SGI0, SGI32808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7438[] = {SGI32809, SGI32810, SGI32811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32812, SGI0, SGI0, SGI0, SGI32813, SGI32813, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7439[] = {SGI32814, SGI0, SGI32815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7440[] = {SGI32816, SGI32817, SGI32818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32819, SGI32820, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7441[] = {SGI32821, SGI32822, SGI32823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32824, SGI32824, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7442[] = {SGI32825, SGI32826, SGI32827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7443[] = {SGI32828, SGI32829, SGI32830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32831, SGI0, SGI0, SGI0, SGI32832, SGI32832, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7444[] = {SGI32833, SGI32834, SGI32835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32836, SGI0, SGI0, SGI0, SGI32837, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7445[] = {SGI32838, SGI32839, SGI32840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32841, SGI0, SGI0, SGI0, SGI32843, SGI32843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7446[] = {SGI32842, SGI32844, SGI32845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32846, SGI0, SGI0, SGI0, SGI32847, SGI32847, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7447[] = {SGI32848, SGI32849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32850, SGI0, SGI0, SGI0, SGI32851, SGI32851, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7448[] = {SGI32852, SGI32853, SGI32855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32858, SGI0, SGI0, SGI0, SGI32860, SGI32860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7449[] = {SGI32854, SGI32856, SGI32857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32859, SGI0, SGI0, SGI0, SGI32861, SGI32861, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7450[] = {SGI32862, SGI32863, SGI32864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32865, SGI0, SGI0, SGI0, SGI32866, SGI32866, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7451[] = {SGI32867, SGI32868, SGI32869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32870, SGI0, SGI0, SGI0, SGI32871, SGI32871, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7452[] = {SGI32872, SGI32873, SGI32874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32877, SGI0, SGI0, SGI0, SGI32878, SGI32878, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7453[] = {SGI32875, SGI0, SGI32876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7454[] = {SGI32879, SGI32880, SGI32881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32882, SGI0, SGI0, SGI0, SGI32883, SGI32884, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7455[] = {SGI32885, SGI0, SGI32886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7456[] = {SGI32887, SGI32888, SGI32889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32890, SGI32891, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7457[] = {SGI32892, SGI0, SGI32893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7458[] = {SGI32894, SGI32895, SGI32896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32897, SGI0, SGI0, SGI0, SGI32898, SGI32898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7459[] = {SGI32899, SGI32900, SGI32901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32902, SGI0, SGI0, SGI0, SGI32903, SGI32903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7460[] = {SGI32904, SGI32905, SGI32906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32907, SGI0, SGI0, SGI0, SGI32908, SGI32908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7461[] = {SGI32909, SGI32910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32911, SGI0, SGI0, SGI0, SGI32912, SGI32912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7462[] = {SGI32913, SGI32914, SGI32915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32916, SGI0, SGI0, SGI0, SGI32917, SGI32917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7463[] = {SGI32918, SGI32919, SGI32920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32921, SGI0, SGI0, SGI0, SGI32922, SGI32922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7464[] = {SGI32923, SGI32924, SGI32925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32926, SGI0, SGI0, SGI0, SGI32927, SGI32927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7465[] = {SGI32928, SGI32929, SGI32930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32931, SGI0, SGI0, SGI0, SGI32932, SGI32932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7466[] = {SGI32933, SGI32934, SGI32935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32936, SGI0, SGI0, SGI0, SGI32937, SGI32937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7467[] = {SGI32938, SGI32939, SGI32940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32941, SGI0, SGI0, SGI0, SGI32942, SGI32942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7468[] = {SGI32943, SGI32944, SGI32945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32946, SGI0, SGI0, SGI0, SGI32949, SGI32949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7469[] = {SGI32947, SGI0, SGI32948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7470[] = {SGI32950, SGI32951, SGI32952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32953, SGI0, SGI0, SGI0, SGI32954, SGI32954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7471[] = {SGI32955, SGI32956, SGI32957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32958, SGI0, SGI0, SGI0, SGI32959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7472[] = {SGI32960, SGI32961, SGI32962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32963, SGI0, SGI0, SGI0, SGI32964, SGI32964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7473[] = {SGI32965, SGI0, SGI32966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32967, SGI0, SGI0, SGI0, SGI32968, SGI32968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7474[] = {SGI32969, SGI0, SGI32970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7475[] = {SGI32971, SGI32972, SGI32973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32974, SGI0, SGI0, SGI0, SGI32975, SGI32975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7476[] = {SGI32976, SGI0, SGI32977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7477[] = {SGI32978, SGI0, SGI32979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7478[] = {SGI32980, SGI0, SGI32981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7479[] = {SGI32982, SGI32983, SGI32984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32985, SGI0, SGI0, SGI0, SGI32986, SGI32986, SGI0, SGI0, NULL};
const Segment * const SG7480[] = {SGI32987, SGI32988, SGI32989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32990, SGI0, SGI0, SGI0, SGI32991, SGI32991, SGI0, SGI0, NULL};
const Segment * const SG7481[] = {SGI32992, SGI32993, SGI32994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI32995, SGI0, SGI0, SGI0, SGI32996, SGI32996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7482[] = {SGI32997, SGI32998, SGI32999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33000, SGI0, SGI0, SGI0, SGI33001, SGI33001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7483[] = {SGI33002, SGI33003, SGI33004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33005, SGI0, SGI0, SGI0, SGI33006, SGI33006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7484[] = {SGI33007, SGI33008, SGI33009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33010, SGI0, SGI0, SGI0, SGI33011, SGI33011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7485[] = {SGI33012, SGI33013, SGI33014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33015, SGI0, SGI0, SGI0, SGI33016, SGI33016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7486[] = {SGI33017, SGI33018, SGI33019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33020, SGI0, SGI0, SGI0, SGI33021, SGI33021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7487[] = {SGI33022, SGI33023, SGI33024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33025, SGI0, SGI0, SGI0, SGI33026, SGI33026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7488[] = {SGI33027, SGI33028, SGI33029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33030, SGI0, SGI0, SGI0, SGI33031, SGI33031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7489[] = {SGI33032, SGI0, SGI33033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7490[] = {SGI33034, SGI33035, SGI33036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33037, SGI33038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7491[] = {SGI33039, SGI33040, SGI33041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33042, SGI0, SGI0, SGI0, SGI33043, SGI33043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7492[] = {SGI33044, SGI33045, SGI33046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33047, SGI0, SGI0, SGI0, SGI33048, SGI33048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7493[] = {SGI33049, SGI33050, SGI33051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33052, SGI0, SGI0, SGI0, SGI33053, SGI33053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7494[] = {SGI33054, SGI33055, SGI33056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33057, SGI0, SGI0, SGI0, SGI33058, SGI33058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7495[] = {SGI33059, SGI0, SGI33060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7496[] = {SGI33061, SGI33062, SGI33063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33064, SGI0, SGI0, SGI0, SGI33065, SGI33065, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7497[] = {SGI33066, SGI33067, SGI33068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33069, SGI0, SGI0, SGI0, SGI33070, SGI33070, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7498[] = {SGI33071, SGI33072, SGI33073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33074, SGI0, SGI0, SGI0, SGI33075, SGI33075, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7499[] = {SGI33076, SGI33077, SGI33078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33079, SGI0, SGI0, SGI0, SGI33083, SGI33083, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7500[] = {SGI33080, SGI33081, SGI33082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33084, SGI0, SGI0, SGI0, SGI33085, SGI33085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7501[] = {SGI33086, SGI33087, SGI33088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33089, SGI0, SGI0, SGI0, SGI33090, SGI33090, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7502[] = {SGI33091, SGI0, SGI33092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7503[] = {SGI33093, SGI33094, SGI33095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33096, SGI0, SGI0, SGI0, SGI33097, SGI33097, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7504[] = {SGI33098, SGI33099, SGI33100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33101, SGI0, SGI0, SGI0, SGI33102, SGI33102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7505[] = {SGI33103, SGI0, SGI33104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33105, SGI0, SGI0, SGI0, SGI33106, SGI33106, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7506[] = {SGI33107, SGI33108, SGI33109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33110, SGI0, SGI0, SGI0, SGI33111, SGI33111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7507[] = {SGI33112, SGI33113, SGI33114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33115, SGI0, SGI0, SGI0, SGI33116, SGI33116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7508[] = {SGI33117, SGI33118, SGI33119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33120, SGI0, SGI0, SGI0, SGI33121, SGI33121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7509[] = {SGI33122, SGI33123, SGI33124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33125, SGI0, SGI0, SGI0, SGI33126, SGI33126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7510[] = {SGI33127, SGI33128, SGI33129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33130, SGI0, SGI0, SGI0, SGI33131, SGI33131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7511[] = {SGI33132, SGI33133, SGI33134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33135, SGI0, SGI0, SGI0, SGI33136, SGI33136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7512[] = {SGI33137, SGI33138, SGI33139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33140, SGI0, SGI0, SGI0, SGI33141, SGI33141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7513[] = {SGI33142, SGI33143, SGI33144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33145, SGI0, SGI0, SGI0, SGI33146, SGI33146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7514[] = {SGI33147, SGI33148, SGI33149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33150, SGI0, SGI0, SGI0, SGI33151, SGI33151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7515[] = {SGI33152, SGI33153, SGI33154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33155, SGI0, SGI0, SGI0, SGI33156, SGI33156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7516[] = {SGI33157, SGI33158, SGI33159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33160, SGI0, SGI0, SGI0, SGI33161, SGI33161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7517[] = {SGI33162, SGI33163, SGI33164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33165, SGI0, SGI0, SGI0, SGI33166, SGI33166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7518[] = {SGI33167, SGI33168, SGI33169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33170, SGI0, SGI0, SGI0, SGI33171, SGI33171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7519[] = {SGI33172, SGI33173, SGI33174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33175, SGI0, SGI0, SGI0, SGI33176, SGI33176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7520[] = {SGI33177, SGI33178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33179, SGI0, SGI0, SGI0, SGI33180, SGI33180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7521[] = {SGI33181, SGI33182, SGI33183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33184, SGI0, SGI0, SGI0, SGI33185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7522[] = {SGI33186, SGI33187, SGI33188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33189, SGI0, SGI0, SGI0, SGI33190, SGI33190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7523[] = {SGI33191, SGI0, SGI33192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7524[] = {SGI33193, SGI33194, SGI33195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33196, SGI0, SGI0, SGI0, SGI33197, SGI33197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7525[] = {SGI33198, SGI33199, SGI33200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33201, SGI33201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7526[] = {SGI33202, SGI33203, SGI33204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33205, SGI0, SGI0, SGI0, SGI33206, SGI33206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7527[] = {SGI33207, SGI33208, SGI33209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33211, SGI0, SGI0, SGI0, SGI33214, SGI33214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7528[] = {SGI33210, SGI33212, SGI33213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33215, SGI0, SGI0, SGI0, SGI33216, SGI33216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7529[] = {SGI33217, SGI33218, SGI33219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33220, SGI0, SGI0, SGI0, SGI33221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7530[] = {SGI33222, SGI33223, SGI33224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33225, SGI0, SGI0, SGI0, SGI33226, SGI33226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7531[] = {SGI33227, SGI33228, SGI33229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33230, SGI0, SGI0, SGI0, SGI33231, SGI33231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7532[] = {SGI33232, SGI33233, SGI33234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33235, SGI0, SGI0, SGI0, SGI33236, SGI33236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7533[] = {SGI33237, SGI33238, SGI33239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33240, SGI0, SGI0, SGI0, SGI33241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7534[] = {SGI33242, SGI33243, SGI33244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33245, SGI0, SGI0, SGI0, SGI33246, SGI33246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7535[] = {SGI33247, SGI33248, SGI33249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33250, SGI0, SGI0, SGI0, SGI33251, SGI33251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7536[] = {SGI33252, SGI0, SGI33253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33254, SGI0, SGI0, SGI0, SGI33255, SGI33255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7537[] = {SGI33256, SGI33257, SGI33258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33259, SGI0, SGI0, SGI0, SGI33260, SGI33260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7538[] = {SGI33261, SGI33262, SGI33263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7539[] = {SGI33264, SGI33265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33266, SGI0, SGI0, SGI0, SGI33267, SGI33267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7540[] = {SGI33268, SGI33269, SGI33270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33271, SGI0, SGI0, SGI0, SGI33272, SGI33272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7541[] = {SGI33273, SGI0, SGI33274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7542[] = {SGI33275, SGI0, SGI33276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33278, SGI0, SGI0, SGI0, SGI33280, SGI33280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7543[] = {SGI33277, SGI33279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33281, SGI0, SGI0, SGI0, SGI33282, SGI33282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7544[] = {SGI33283, SGI33284, SGI33285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33288, SGI0, SGI0, SGI0, SGI33289, SGI33289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7545[] = {SGI33286, SGI0, SGI33287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7546[] = {SGI33291, SGI33292, SGI33294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7547[] = {SGI33290, SGI33293, SGI33295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33296, SGI0, SGI0, SGI0, SGI33297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7548[] = {SGI33298, SGI33299, SGI33300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33301, SGI0, SGI0, SGI0, SGI33302, SGI33302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7549[] = {SGI33303, SGI33304, SGI33305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33306, SGI0, SGI0, SGI0, SGI33307, SGI33308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7550[] = {SGI33309, SGI33310, SGI33311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33312, SGI0, SGI0, SGI0, SGI33313, SGI33313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7551[] = {SGI33314, SGI33315, SGI33316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33317, SGI0, SGI0, SGI0, SGI33318, SGI33319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7552[] = {SGI33320, SGI33321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33322, SGI0, SGI0, SGI0, SGI33323, SGI33323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7553[] = {SGI33324, SGI0, SGI33325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33326, SGI0, SGI0, SGI0, SGI33327, SGI33327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7554[] = {SGI33328, SGI33329, SGI33330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33331, SGI33331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7555[] = {SGI33332, SGI33333, SGI33334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33335, SGI0, SGI0, SGI0, SGI33336, SGI33336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7556[] = {SGI33337, SGI33338, SGI33339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33340, SGI0, SGI0, SGI0, SGI33341, SGI33341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7557[] = {SGI33342, SGI33343, SGI33344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33345, SGI0, SGI0, SGI0, SGI33346, SGI33346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7558[] = {SGI33347, SGI33348, SGI33349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33350, SGI0, SGI0, SGI0, SGI33351, SGI33351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7559[] = {SGI33352, SGI33353, SGI33354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33355, SGI0, SGI0, SGI0, SGI33356, SGI33356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7560[] = {SGI33357, SGI0, SGI33358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7561[] = {SGI33359, SGI0, SGI33360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7562[] = {SGI33361, SGI33362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33363, SGI0, SGI0, SGI0, SGI33364, SGI33364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7563[] = {SGI33365, SGI0, SGI33366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33367, SGI0, SGI0, SGI0, SGI33368, SGI33368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7564[] = {SGI33369, SGI33370, SGI33371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33372, SGI0, SGI0, SGI0, SGI33373, SGI33373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7565[] = {SGI33374, SGI33375, SGI33376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33377, SGI0, SGI0, SGI0, SGI33378, SGI33378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7566[] = {SGI33379, SGI33380, SGI33381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33382, SGI0, SGI0, SGI0, SGI33383, SGI33383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7567[] = {SGI33384, SGI0, SGI33385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7568[] = {SGI33386, SGI33387, SGI33388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33389, SGI0, SGI0, SGI0, SGI33390, SGI33390, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7569[] = {SGI33391, SGI0, SGI33392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7570[] = {SGI33393, SGI33394, SGI33395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33396, SGI0, SGI0, SGI0, SGI33397, SGI33397, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7571[] = {SGI33398, SGI0, SGI33399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7572[] = {SGI33400, SGI33401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33402, SGI0, SGI0, SGI0, SGI33403, SGI33403, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7573[] = {SGI33404, SGI33405, SGI33406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33407, SGI0, SGI0, SGI0, SGI33408, SGI33408, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7574[] = {SGI33409, SGI33410, SGI33411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33412, SGI0, SGI0, SGI0, SGI33413, SGI33413, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7575[] = {SGI33414, SGI0, SGI33415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7576[] = {SGI33416, SGI33417, SGI33418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33419, SGI0, SGI0, SGI0, SGI33420, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7577[] = {SGI33421, SGI33422, SGI33423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33424, SGI0, SGI0, SGI0, SGI33425, SGI33425, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7578[] = {SGI33426, SGI33427, SGI33428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33429, SGI0, SGI0, SGI0, SGI33430, SGI33431, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7579[] = {SGI33432, SGI33433, SGI33434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33435, SGI0, SGI0, SGI0, SGI33436, SGI33436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7580[] = {SGI33437, SGI33438, SGI33439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33440, SGI0, SGI0, SGI0, SGI33441, SGI33441, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7581[] = {SGI33442, SGI33443, SGI33444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33445, SGI0, SGI0, SGI0, SGI33446, SGI33446, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7582[] = {SGI33447, SGI33448, SGI33449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33450, SGI0, SGI0, SGI0, SGI33451, SGI33451, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7583[] = {SGI33452, SGI33453, SGI33454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33455, SGI0, SGI0, SGI0, SGI33456, SGI33456, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7584[] = {SGI33457, SGI33458, SGI33459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33460, SGI0, SGI0, SGI0, SGI33461, SGI33461, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7585[] = {SGI33462, SGI0, SGI33463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7586[] = {SGI33464, SGI33465, SGI33466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33467, SGI0, SGI0, SGI0, SGI33468, SGI33468, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7587[] = {SGI33469, SGI33470, SGI33471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33472, SGI0, SGI0, SGI0, SGI33473, SGI33473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7588[] = {SGI33474, SGI33475, SGI33476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33477, SGI0, SGI0, SGI0, SGI33478, SGI33478, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7589[] = {SGI33479, SGI33480, SGI33481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33482, SGI0, SGI0, SGI0, SGI33483, SGI33483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7590[] = {SGI33484, SGI33485, SGI33486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33487, SGI0, SGI0, SGI0, SGI33488, SGI33488, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7591[] = {SGI33489, SGI33490, SGI33491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33492, SGI0, SGI0, SGI0, SGI33493, SGI33493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7592[] = {SGI33494, SGI33495, SGI33496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33497, SGI0, SGI0, SGI0, SGI33498, SGI33498, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7593[] = {SGI33499, SGI33500, SGI33501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33502, SGI0, SGI0, SGI0, SGI33503, SGI33503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7594[] = {SGI33504, SGI33505, SGI33506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33507, SGI0, SGI0, SGI0, SGI33508, SGI33508, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7595[] = {SGI33509, SGI33510, SGI33511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33512, SGI0, SGI0, SGI0, SGI33515, SGI33515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7596[] = {SGI33513, SGI33514, SGI33516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33517, SGI0, SGI0, SGI0, SGI33518, SGI33518, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7597[] = {SGI33519, SGI33520, SGI33521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33522, SGI0, SGI0, SGI0, SGI33525, SGI33525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7598[] = {SGI33523, SGI33524, SGI33526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33527, SGI0, SGI0, SGI0, SGI33528, SGI33528, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7599[] = {SGI33529, SGI33530, SGI33531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33533, SGI0, SGI0, SGI0, SGI33536, SGI33536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7600[] = {SGI33532, SGI33534, SGI33535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33537, SGI0, SGI0, SGI0, SGI33538, SGI33538, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7601[] = {SGI33539, SGI33540, SGI33541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33545, SGI0, SGI0, SGI0, SGI33546, SGI33546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7602[] = {SGI33542, SGI33543, SGI33544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33547, SGI0, SGI0, SGI0, SGI33548, SGI33548, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7603[] = {SGI33549, SGI33550, SGI33551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33552, SGI0, SGI0, SGI0, SGI33554, SGI33554, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7604[] = {SGI33553, SGI33555, SGI33556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33557, SGI0, SGI0, SGI0, SGI33558, SGI33558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7605[] = {SGI33559, SGI33560, SGI33561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33562, SGI0, SGI0, SGI0, SGI33563, SGI33563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7606[] = {SGI33564, SGI33565, SGI33566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33567, SGI0, SGI0, SGI0, SGI33568, SGI33568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7607[] = {SGI33569, SGI33570, SGI33571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33572, SGI0, SGI0, SGI0, SGI33573, SGI33573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7608[] = {SGI33574, SGI33575, SGI33576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33577, SGI0, SGI0, SGI0, SGI33578, SGI33578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7609[] = {SGI33579, SGI33580, SGI33581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33582, SGI0, SGI0, SGI0, SGI33583, SGI33583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7610[] = {SGI33584, SGI33585, SGI33586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33587, SGI0, SGI0, SGI0, SGI33588, SGI33588, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7611[] = {SGI33589, SGI33590, SGI33591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33592, SGI0, SGI0, SGI0, SGI33593, SGI33593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7612[] = {SGI33594, SGI33595, SGI33596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33597, SGI0, SGI0, SGI0, SGI33598, SGI33598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7613[] = {SGI33599, SGI33600, SGI33601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33602, SGI0, SGI0, SGI0, SGI33603, SGI33603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7614[] = {SGI33604, SGI33605, SGI33606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33607, SGI0, SGI0, SGI0, SGI33608, SGI33608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7615[] = {SGI33609, SGI33610, SGI33611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33612, SGI0, SGI0, SGI0, SGI33613, SGI33613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7616[] = {SGI33614, SGI33615, SGI33616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33617, SGI0, SGI0, SGI0, SGI33618, SGI33618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7617[] = {SGI33619, SGI33620, SGI33621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33622, SGI0, SGI0, SGI0, SGI33623, SGI33623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7618[] = {SGI33624, SGI33625, SGI33626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33627, SGI0, SGI0, SGI0, SGI33628, SGI33628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7619[] = {SGI33629, SGI33630, SGI33631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33632, SGI0, SGI0, SGI0, SGI33633, SGI33633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7620[] = {SGI33634, SGI33635, SGI33636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33637, SGI0, SGI0, SGI0, SGI33638, SGI33638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7621[] = {SGI33639, SGI0, SGI33640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7622[] = {SGI33641, SGI33642, SGI33643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33644, SGI0, SGI0, SGI0, SGI33645, SGI33645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7623[] = {SGI33646, SGI33647, SGI33648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33649, SGI0, SGI0, SGI0, SGI33650, SGI33650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7624[] = {SGI33651, SGI0, SGI33652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7625[] = {SGI33653, SGI33654, SGI33655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33656, SGI0, SGI0, SGI0, SGI33657, SGI33657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7626[] = {SGI33658, SGI33659, SGI33660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33661, SGI0, SGI0, SGI0, SGI33662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7627[] = {SGI33663, SGI33664, SGI33665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33666, SGI0, SGI0, SGI0, SGI33667, SGI33667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7628[] = {SGI33668, SGI33669, SGI33670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33671, SGI0, SGI0, SGI0, SGI33672, SGI33672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7629[] = {SGI33673, SGI33674, SGI33675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33676, SGI0, SGI0, SGI0, SGI33677, SGI33677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7630[] = {SGI33678, SGI33679, SGI33681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33684, SGI0, SGI0, SGI0, SGI33685, SGI33685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7631[] = {SGI33680, SGI33682, SGI33683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33686, SGI0, SGI0, SGI0, SGI33687, SGI33687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7632[] = {SGI33688, SGI33689, SGI33690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33691, SGI0, SGI0, SGI0, SGI33692, SGI33692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7633[] = {SGI33693, SGI33694, SGI33695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33696, SGI0, SGI0, SGI0, SGI33697, SGI33697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7634[] = {SGI33698, SGI33699, SGI33700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33701, SGI0, SGI0, SGI0, SGI33702, SGI33702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7635[] = {SGI33703, SGI33704, SGI33705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33706, SGI0, SGI0, SGI0, SGI33707, SGI33707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7636[] = {SGI33708, SGI33709, SGI33710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33711, SGI0, SGI0, SGI0, SGI33712, SGI33712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7637[] = {SGI33713, SGI33714, SGI33715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33716, SGI0, SGI0, SGI0, SGI33717, SGI33717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7638[] = {SGI33718, SGI33719, SGI33720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33721, SGI0, SGI0, SGI0, SGI33722, SGI33722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7639[] = {SGI33723, SGI33724, SGI33725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33726, SGI0, SGI0, SGI0, SGI33727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7640[] = {SGI33728, SGI33729, SGI33730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33731, SGI0, SGI0, SGI0, SGI33732, SGI33732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7641[] = {SGI33733, SGI33734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33735, SGI0, SGI0, SGI0, SGI33736, SGI33736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7642[] = {SGI33737, SGI33738, SGI33739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33740, SGI0, SGI0, SGI0, SGI33741, SGI33741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7643[] = {SGI33742, SGI33743, SGI33744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33745, SGI0, SGI0, SGI0, SGI33746, SGI33746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7644[] = {SGI33747, SGI33748, SGI33749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33750, SGI0, SGI0, SGI0, SGI33751, SGI33751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7645[] = {SGI33752, SGI0, SGI33753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7646[] = {SGI33754, SGI33755, SGI33756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33757, SGI0, SGI0, SGI0, SGI33758, SGI33758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7647[] = {SGI33759, SGI33760, SGI33761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33762, SGI0, SGI0, SGI0, SGI33763, SGI33764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7648[] = {SGI33765, SGI33766, SGI33767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33768, SGI0, SGI0, SGI0, SGI33769, SGI33769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7649[] = {SGI33770, SGI0, SGI33771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7650[] = {SGI33772, SGI33773, SGI33774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33775, SGI0, SGI0, SGI0, SGI33776, SGI33776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7651[] = {SGI33777, SGI0, SGI33778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33779, SGI0, SGI0, SGI0, SGI33780, SGI33780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7652[] = {SGI33781, SGI33782, SGI33783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33784, SGI0, SGI0, SGI0, SGI33785, SGI33785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7653[] = {SGI33786, SGI33787, SGI33788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33789, SGI0, SGI0, SGI0, SGI33790, SGI33790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7654[] = {SGI33791, SGI33792, SGI33793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33794, SGI0, SGI0, SGI0, SGI33795, SGI33795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7655[] = {SGI33796, SGI33797, SGI33798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33799, SGI0, SGI0, SGI0, SGI33800, SGI33800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7656[] = {SGI33801, SGI33802, SGI33803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33804, SGI0, SGI0, SGI0, SGI33805, SGI33805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7657[] = {SGI33806, SGI33807, SGI33808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33809, SGI0, SGI0, SGI0, SGI33810, SGI33810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7658[] = {SGI33811, SGI33812, SGI33813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33814, SGI0, SGI0, SGI0, SGI33815, SGI33815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7659[] = {SGI33816, SGI33817, SGI33818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33819, SGI0, SGI0, SGI0, SGI33820, SGI33820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7660[] = {SGI33821, SGI33822, SGI33823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33824, SGI0, SGI0, SGI0, SGI33825, SGI33825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7661[] = {SGI33826, SGI0, SGI33827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7662[] = {SGI33828, SGI33829, SGI33830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33831, SGI0, SGI0, SGI0, SGI33832, SGI33832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7663[] = {SGI33833, SGI33834, SGI33835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33836, SGI0, SGI0, SGI0, SGI33837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7664[] = {SGI33838, SGI33839, SGI33840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33841, SGI0, SGI0, SGI0, SGI33842, SGI33842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7665[] = {SGI33843, SGI33844, SGI33845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33846, SGI0, SGI0, SGI0, SGI33847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7666[] = {SGI33848, SGI0, SGI33849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7667[] = {SGI33850, SGI0, SGI33851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7668[] = {SGI33852, SGI33853, SGI33854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33855, SGI0, SGI0, SGI0, SGI33856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7669[] = {SGI33857, SGI33858, SGI33859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33860, SGI0, SGI0, SGI0, SGI33861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7670[] = {SGI33862, SGI33863, SGI33864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33865, SGI0, SGI0, SGI0, SGI33866, SGI33866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7671[] = {SGI33867, SGI33868, SGI33869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33870, SGI0, SGI0, SGI0, SGI33871, SGI33871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7672[] = {SGI33872, SGI33873, SGI33874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33875, SGI0, SGI0, SGI0, SGI33876, SGI33876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7673[] = {SGI33877, SGI33878, SGI33879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33880, SGI0, SGI0, SGI0, SGI33881, SGI33881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7674[] = {SGI33882, SGI33883, SGI33884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33885, SGI0, SGI0, SGI0, SGI33886, SGI33886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7675[] = {SGI33887, SGI33888, SGI33889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33893, SGI0, SGI0, SGI0, SGI33894, SGI33894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7676[] = {SGI33890, SGI33891, SGI33892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33895, SGI0, SGI0, SGI0, SGI33896, SGI33896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7677[] = {SGI33897, SGI33898, SGI33899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33900, SGI0, SGI0, SGI0, SGI33901, SGI33901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7678[] = {SGI33902, SGI0, SGI33903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7679[] = {SGI33904, SGI0, SGI33905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7680[] = {SGI33906, SGI33907, SGI33908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33909, SGI0, SGI0, SGI0, SGI33910, SGI33910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7681[] = {SGI33911, SGI33912, SGI33913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33914, SGI0, SGI0, SGI0, SGI33915, SGI33915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7682[] = {SGI33916, SGI33917, SGI33918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33919, SGI0, SGI0, SGI0, SGI33920, SGI33920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7683[] = {SGI33921, SGI33922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33923, SGI0, SGI0, SGI0, SGI33924, SGI33924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7684[] = {SGI33925, SGI33926, SGI33927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33928, SGI0, SGI0, SGI0, SGI33929, SGI33929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7685[] = {SGI33930, SGI33931, SGI33932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33933, SGI0, SGI0, SGI0, SGI33934, SGI33934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7686[] = {SGI33935, SGI33936, SGI33937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33938, SGI0, SGI0, SGI0, SGI33939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7687[] = {SGI33940, SGI33941, SGI33942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33943, SGI0, SGI0, SGI0, SGI33944, SGI33944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7688[] = {SGI33945, SGI0, SGI33946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7689[] = {SGI33947, SGI0, SGI33948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33949, SGI0, SGI0, SGI0, SGI33950, SGI33950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7690[] = {SGI33951, SGI33952, SGI33953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33954, SGI33954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7691[] = {SGI33955, SGI33956, SGI33957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33958, SGI0, SGI0, SGI0, SGI33959, SGI33959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7692[] = {SGI33960, SGI33961, SGI33962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33963, SGI0, SGI0, SGI0, SGI33964, SGI33964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7693[] = {SGI33965, SGI33966, SGI33967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33968, SGI0, SGI0, SGI0, SGI33969, SGI33969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7694[] = {SGI33970, SGI33971, SGI33972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33973, SGI0, SGI0, SGI0, SGI33974, SGI33974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7695[] = {SGI33975, SGI0, SGI33976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7696[] = {SGI33977, SGI33978, SGI33979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33980, SGI0, SGI0, SGI0, SGI33981, SGI33981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7697[] = {SGI33982, SGI33983, SGI33984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33985, SGI0, SGI0, SGI0, SGI33986, SGI33986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7698[] = {SGI33987, SGI33988, SGI33989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33990, SGI0, SGI0, SGI0, SGI33991, SGI33991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7699[] = {SGI33992, SGI33993, SGI33994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI33995, SGI0, SGI0, SGI0, SGI33996, SGI33996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7700[] = {SGI33997, SGI0, SGI33998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7701[] = {SGI33999, SGI34000, SGI34001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34002, SGI0, SGI0, SGI0, SGI34003, SGI34003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7702[] = {SGI34004, SGI34005, SGI34006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34007, SGI0, SGI0, SGI0, SGI34008, SGI34008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7703[] = {SGI34009, SGI34010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34011, SGI0, SGI0, SGI0, SGI34012, SGI34012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7704[] = {SGI34013, SGI34014, SGI34015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34016, SGI0, SGI0, SGI0, SGI34017, SGI34017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7705[] = {SGI34018, SGI34019, SGI34020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34021, SGI0, SGI0, SGI0, SGI34022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7706[] = {SGI34023, SGI34024, SGI34025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34026, SGI0, SGI0, SGI0, SGI34027, SGI34027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7707[] = {SGI34028, SGI0, SGI34029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7708[] = {SGI34030, SGI34031, SGI34032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34033, SGI0, SGI0, SGI0, SGI34034, SGI34034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7709[] = {SGI34035, SGI0, SGI34036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34037, SGI0, SGI0, SGI0, SGI34038, SGI34038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7710[] = {SGI34039, SGI34040, SGI34041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34042, SGI34042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7711[] = {SGI34043, SGI34044, SGI34045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34046, SGI0, SGI0, SGI0, SGI34047, SGI34047, SGI0, SGI0, NULL};
const Segment * const SG7712[] = {SGI34048, SGI34049, SGI34050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34051, SGI0, SGI0, SGI0, SGI34052, SGI34052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7713[] = {SGI34053, SGI0, SGI34054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7714[] = {SGI34055, SGI34056, SGI34057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34058, SGI0, SGI0, SGI0, SGI34059, SGI34059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7715[] = {SGI34060, SGI34061, SGI34062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34063, SGI0, SGI0, SGI0, SGI34064, SGI34064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7716[] = {SGI34065, SGI34066, SGI34067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34068, SGI0, SGI0, SGI0, SGI34069, SGI34069, SGI0, SGI0, NULL};
const Segment * const SG7717[] = {SGI34070, SGI0, SGI34071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7718[] = {SGI34072, SGI34073, SGI34074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34075, SGI0, SGI0, SGI0, SGI34076, SGI34076, SGI0, SGI0, NULL};
const Segment * const SG7719[] = {SGI34077, SGI0, SGI34078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34079, SGI0, SGI0, SGI0, SGI34080, SGI34080, SGI0, SGI0, NULL};
const Segment * const SG7720[] = {SGI34081, SGI34082, SGI34083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34084, SGI0, SGI0, SGI0, SGI34085, SGI34085, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7721[] = {SGI34086, SGI34087, SGI34088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34089, SGI0, SGI0, SGI0, SGI34090, SGI34090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7722[] = {SGI34091, SGI34092, SGI34093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34094, SGI0, SGI0, SGI0, SGI34095, SGI34095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7723[] = {SGI34096, SGI34097, SGI34098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34102, SGI0, SGI0, SGI0, SGI34103, SGI34103, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7724[] = {SGI34099, SGI34100, SGI34101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34104, SGI0, SGI0, SGI0, SGI34105, SGI34105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7725[] = {SGI34106, SGI0, SGI34107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7726[] = {SGI34108, SGI34109, SGI34110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34111, SGI0, SGI0, SGI0, SGI34112, SGI34112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7727[] = {SGI34113, SGI34114, SGI34115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34116, SGI0, SGI0, SGI0, SGI34117, SGI34117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7728[] = {SGI34118, SGI34119, SGI34120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34121, SGI0, SGI0, SGI0, SGI34122, SGI34122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7729[] = {SGI34123, SGI34124, SGI34125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34126, SGI0, SGI0, SGI0, SGI34127, SGI34127, SGI0, SGI0, NULL};
const Segment * const SG7730[] = {SGI34128, SGI34129, SGI34130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34131, SGI0, SGI0, SGI0, SGI34132, SGI34132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7731[] = {SGI34133, SGI0, SGI34134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7732[] = {SGI34135, SGI34136, SGI34137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34138, SGI0, SGI0, SGI0, SGI34139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7733[] = {SGI34140, SGI34141, SGI34142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34143, SGI0, SGI0, SGI0, SGI34144, SGI34144, SGI0, SGI0, NULL};
const Segment * const SG7734[] = {SGI34145, SGI34146, SGI34147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34148, SGI0, SGI0, SGI0, SGI34149, SGI34149, SGI0, SGI0, NULL};
const Segment * const SG7735[] = {SGI34150, SGI34151, SGI34152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34153, SGI0, SGI0, SGI0, SGI34154, SGI34154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7736[] = {SGI34155, SGI34156, SGI34157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34158, SGI0, SGI0, SGI0, SGI34159, SGI34159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7737[] = {SGI34160, SGI34161, SGI34162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34163, SGI0, SGI0, SGI0, SGI34164, SGI34164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7738[] = {SGI34165, SGI34166, SGI34167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34168, SGI0, SGI0, SGI0, SGI34169, SGI34169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7739[] = {SGI34170, SGI34171, SGI34172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34173, SGI0, SGI0, SGI0, SGI34174, SGI34174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7740[] = {SGI34175, SGI34176, SGI34177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34178, SGI34179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7741[] = {SGI34180, SGI34181, SGI34182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34183, SGI0, SGI0, SGI0, SGI34184, SGI34184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7742[] = {SGI34185, SGI34186, SGI34187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34188, SGI0, SGI0, SGI0, SGI34189, SGI34189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7743[] = {SGI34190, SGI34191, SGI34192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34193, SGI0, SGI0, SGI0, SGI34194, SGI34194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7744[] = {SGI34195, SGI34196, SGI34197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34198, SGI0, SGI0, SGI0, SGI34199, SGI34199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7745[] = {SGI34200, SGI34201, SGI34202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34203, SGI0, SGI0, SGI0, SGI34204, SGI34204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7746[] = {SGI34205, SGI34206, SGI34207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34208, SGI0, SGI0, SGI0, SGI34209, SGI34209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7747[] = {SGI34210, SGI34211, SGI34212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34213, SGI0, SGI0, SGI0, SGI34214, SGI34214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7748[] = {SGI34215, SGI34216, SGI34217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34218, SGI0, SGI0, SGI0, SGI34219, SGI34219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7749[] = {SGI34220, SGI34221, SGI34222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34223, SGI0, SGI0, SGI0, SGI34224, SGI34224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7750[] = {SGI34225, SGI34226, SGI34227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34228, SGI0, SGI0, SGI0, SGI34229, SGI34229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7751[] = {SGI34230, SGI34231, SGI34232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34233, SGI0, SGI0, SGI0, SGI34234, SGI34234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7752[] = {SGI34235, SGI34236, SGI34237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34238, SGI0, SGI0, SGI0, SGI34239, SGI34239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7753[] = {SGI34240, SGI34241, SGI34242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34243, SGI0, SGI0, SGI0, SGI34244, SGI34244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7754[] = {SGI34245, SGI34246, SGI34247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34248, SGI0, SGI0, SGI0, SGI34249, SGI34249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7755[] = {SGI34250, SGI34251, SGI34252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34253, SGI0, SGI0, SGI0, SGI34254, SGI34254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7756[] = {SGI34255, SGI34256, SGI34257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34258, SGI0, SGI0, SGI0, SGI34259, SGI34259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7757[] = {SGI34260, SGI34261, SGI34262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34263, SGI0, SGI0, SGI0, SGI34264, SGI34264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7758[] = {SGI34265, SGI34266, SGI34267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34268, SGI0, SGI0, SGI0, SGI34269, SGI34269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7759[] = {SGI34270, SGI34271, SGI34272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34273, SGI0, SGI0, SGI0, SGI34274, SGI34274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7760[] = {SGI34275, SGI34276, SGI34277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34278, SGI0, SGI0, SGI0, SGI34279, SGI34279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7761[] = {SGI34280, SGI34281, SGI34282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34283, SGI0, SGI0, SGI0, SGI34284, SGI34284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7762[] = {SGI34285, SGI34286, SGI34287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34288, SGI0, SGI0, SGI0, SGI34289, SGI34289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7763[] = {SGI34290, SGI34291, SGI34292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34293, SGI0, SGI0, SGI0, SGI34294, SGI34294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7764[] = {SGI34295, SGI34296, SGI34297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34298, SGI0, SGI0, SGI0, SGI34299, SGI34299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7765[] = {SGI34300, SGI34301, SGI34302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34303, SGI0, SGI0, SGI0, SGI34304, SGI34304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7766[] = {SGI34305, SGI34306, SGI34307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34308, SGI0, SGI0, SGI0, SGI34309, SGI34309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7767[] = {SGI34310, SGI34311, SGI34312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34313, SGI0, SGI0, SGI0, SGI34314, SGI34314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7768[] = {SGI34315, SGI34316, SGI34317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34318, SGI0, SGI0, SGI0, SGI34319, SGI34319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7769[] = {SGI34320, SGI34321, SGI34322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34323, SGI0, SGI0, SGI0, SGI34324, SGI34324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7770[] = {SGI34325, SGI0, SGI34326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7771[] = {SGI34327, SGI34328, SGI34329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34330, SGI0, SGI0, SGI0, SGI34331, SGI34331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7772[] = {SGI34332, SGI34333, SGI34334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34335, SGI0, SGI0, SGI0, SGI34336, SGI34336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7773[] = {SGI34337, SGI34338, SGI34339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34340, SGI0, SGI0, SGI0, SGI34341, SGI34341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7774[] = {SGI34342, SGI34343, SGI34344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34347, SGI0, SGI0, SGI0, SGI34349, SGI34349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7775[] = {SGI34345, SGI34346, SGI34348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7776[] = {SGI34350, SGI34351, SGI34352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34353, SGI0, SGI0, SGI0, SGI34354, SGI34354, SGI0, SGI0, NULL};
const Segment * const SG7777[] = {SGI34358, SGI0, SGI34359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7778[] = {SGI34355, SGI34356, SGI34357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34360, SGI0, SGI0, SGI0, SGI34361, SGI34361, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7779[] = {SGI34362, SGI34363, SGI34364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34365, SGI0, SGI0, SGI0, SGI34366, SGI34366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7780[] = {SGI34367, SGI34368, SGI34369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34370, SGI0, SGI0, SGI0, SGI34373, SGI34373, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7781[] = {SGI34371, SGI34372, SGI34374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34375, SGI0, SGI0, SGI0, SGI34376, SGI34376, SGI0, SGI0, NULL};
const Segment * const SG7782[] = {SGI34377, SGI34378, SGI34379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34380, SGI0, SGI0, SGI0, SGI34381, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7783[] = {SGI34382, SGI34383, SGI34384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34388, SGI34389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7784[] = {SGI34385, SGI34386, SGI34387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34390, SGI0, SGI0, SGI0, SGI34391, SGI34391, SGI0, SGI0, NULL};
const Segment * const SG7785[] = {SGI34392, SGI34393, SGI34395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34398, SGI34398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7786[] = {SGI34394, SGI34396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34397, SGI0, SGI0, SGI0, SGI34399, SGI34399, SGI0, SGI0, NULL};
const Segment * const SG7787[] = {SGI34400, SGI0, SGI34401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34402, SGI0, SGI0, SGI0, SGI34405, SGI34405, SGI0, SGI0, NULL};
const Segment * const SG7788[] = {SGI34403, SGI34404, SGI34406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34407, SGI0, SGI0, SGI0, SGI34408, SGI34408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7789[] = {SGI34409, SGI34410, SGI34411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34412, SGI0, SGI0, SGI0, SGI34413, SGI34414, SGI0, SGI0, NULL};
const Segment * const SG7790[] = {SGI34415, SGI34416, SGI34417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34418, SGI0, SGI0, SGI0, SGI34419, SGI34419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7791[] = {SGI34420, SGI34421, SGI34422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34423, SGI0, SGI0, SGI0, SGI34424, SGI34424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7792[] = {SGI34425, SGI0, SGI34426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7793[] = {SGI34427, SGI34428, SGI34429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34430, SGI0, SGI0, SGI0, SGI34431, SGI34432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7794[] = {SGI34433, SGI34434, SGI34435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34436, SGI0, SGI0, SGI0, SGI34437, SGI34437, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7795[] = {SGI34438, SGI0, SGI34439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34440, SGI0, SGI0, SGI0, SGI34441, SGI34441, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7796[] = {SGI34442, SGI34443, SGI34444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34445, SGI0, SGI0, SGI0, SGI34446, SGI34446, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7797[] = {SGI34447, SGI0, SGI34448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7798[] = {SGI34449, SGI0, SGI34450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7799[] = {SGI34451, SGI34452, SGI34453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34454, SGI34455, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7800[] = {SGI34456, SGI34457, SGI34458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34459, SGI34459, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7801[] = {SGI34460, SGI34461, SGI34462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34463, SGI0, SGI0, SGI0, SGI34464, SGI34464, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7802[] = {SGI34465, SGI0, SGI34466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34467, SGI0, SGI0, SGI0, SGI34468, SGI34468, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7803[] = {SGI34469, SGI34470, SGI34471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34472, SGI0, SGI0, SGI0, SGI34473, SGI34473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7804[] = {SGI34474, SGI0, SGI34475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7805[] = {SGI34476, SGI34477, SGI34478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34479, SGI0, SGI0, SGI0, SGI34480, SGI34480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7806[] = {SGI34481, SGI34482, SGI34483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7807[] = {SGI34484, SGI34485, SGI34486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34487, SGI0, SGI0, SGI0, SGI34488, SGI34488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7808[] = {SGI34489, SGI34490, SGI34491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34492, SGI0, SGI0, SGI0, SGI34493, SGI34493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7809[] = {SGI34494, SGI34495, SGI34496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34497, SGI0, SGI0, SGI0, SGI34498, SGI34498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7810[] = {SGI34499, SGI34500, SGI34501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34502, SGI0, SGI0, SGI0, SGI34503, SGI34503, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7811[] = {SGI34504, SGI34505, SGI34506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34507, SGI0, SGI0, SGI0, SGI34508, SGI34508, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7812[] = {SGI34509, SGI34510, SGI34511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34512, SGI0, SGI0, SGI0, SGI34513, SGI34513, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7813[] = {SGI34514, SGI34515, SGI34516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34517, SGI0, SGI0, SGI0, SGI34518, SGI34518, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7814[] = {SGI34519, SGI34520, SGI34521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34522, SGI0, SGI0, SGI0, SGI34523, SGI34523, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7815[] = {SGI34524, SGI34525, SGI34526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34527, SGI0, SGI0, SGI0, SGI34528, SGI34528, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7816[] = {SGI34529, SGI34530, SGI34531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34532, SGI0, SGI0, SGI0, SGI34533, SGI34533, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7817[] = {SGI34534, SGI34535, SGI34536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34537, SGI34538, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7818[] = {SGI34539, SGI0, SGI34540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7819[] = {SGI34541, SGI34542, SGI34543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34544, SGI0, SGI0, SGI0, SGI34546, SGI34546, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7820[] = {SGI34545, SGI34547, SGI34548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34549, SGI0, SGI0, SGI0, SGI34550, SGI34550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7821[] = {SGI34551, SGI34552, SGI34553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34554, SGI0, SGI0, SGI0, SGI34555, SGI34555, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7822[] = {SGI34556, SGI34557, SGI34558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34559, SGI0, SGI0, SGI0, SGI34560, SGI34560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7823[] = {SGI34561, SGI0, SGI34562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7824[] = {SGI34563, SGI0, SGI34564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34565, SGI0, SGI0, SGI0, SGI34566, SGI34566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7825[] = {SGI34567, SGI34568, SGI34569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34570, SGI0, SGI0, SGI0, SGI34571, SGI34571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7826[] = {SGI34572, SGI34573, SGI34574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34575, SGI0, SGI0, SGI0, SGI34576, SGI34576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7827[] = {SGI34577, SGI34578, SGI34579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34580, SGI0, SGI0, SGI0, SGI34581, SGI34581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7828[] = {SGI34582, SGI34583, SGI34584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34585, SGI0, SGI0, SGI0, SGI34586, SGI34586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7829[] = {SGI34587, SGI34588, SGI34589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34590, SGI0, SGI0, SGI0, SGI34591, SGI34591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7830[] = {SGI34592, SGI34593, SGI34594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34595, SGI0, SGI0, SGI0, SGI34596, SGI34596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7831[] = {SGI34597, SGI34598, SGI34599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34600, SGI0, SGI0, SGI0, SGI34603, SGI34603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7832[] = {SGI34601, SGI34602, SGI34604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34605, SGI34606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7833[] = {SGI34608, SGI0, SGI34611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7834[] = {SGI34607, SGI34609, SGI34610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34612, SGI0, SGI0, SGI0, SGI34613, SGI34614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7835[] = {SGI34615, SGI34616, SGI34617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34619, SGI0, SGI0, SGI0, SGI34622, SGI34622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7836[] = {SGI34618, SGI34620, SGI34621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34623, SGI0, SGI0, SGI0, SGI34624, SGI34624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7837[] = {SGI34625, SGI34626, SGI34627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34628, SGI0, SGI0, SGI0, SGI34630, SGI34630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7838[] = {SGI34629, SGI34631, SGI34632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34633, SGI0, SGI0, SGI0, SGI34634, SGI34634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7839[] = {SGI34635, SGI34636, SGI34637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34638, SGI0, SGI0, SGI0, SGI34639, SGI34639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7840[] = {SGI34640, SGI34641, SGI34642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34643, SGI0, SGI0, SGI0, SGI34644, SGI34644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7841[] = {SGI34645, SGI34646, SGI34647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34648, SGI0, SGI0, SGI0, SGI34649, SGI34649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7842[] = {SGI34650, SGI34651, SGI34652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34653, SGI0, SGI0, SGI0, SGI34654, SGI34654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7843[] = {SGI34655, SGI34656, SGI34657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34658, SGI0, SGI0, SGI0, SGI34659, SGI34659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7844[] = {SGI34660, SGI34661, SGI34662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34663, SGI0, SGI0, SGI0, SGI34664, SGI34664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7845[] = {SGI34666, SGI34668, SGI34670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34671, SGI0, SGI0, SGI0, SGI34673, SGI34673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7846[] = {SGI34665, SGI34667, SGI34669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34672, SGI0, SGI0, SGI0, SGI34674, SGI34674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7847[] = {SGI34675, SGI34676, SGI34677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34681, SGI0, SGI0, SGI0, SGI34682, SGI34682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7848[] = {SGI34678, SGI34679, SGI34680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34683, SGI0, SGI0, SGI0, SGI34684, SGI34684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7849[] = {SGI34685, SGI34686, SGI34687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34688, SGI0, SGI0, SGI0, SGI34689, SGI34689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7850[] = {SGI34690, SGI34691, SGI34692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34693, SGI0, SGI0, SGI0, SGI34694, SGI34694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7851[] = {SGI34695, SGI34696, SGI34697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34698, SGI0, SGI0, SGI0, SGI34699, SGI34699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7852[] = {SGI34701, SGI0, SGI34703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7853[] = {SGI34706, SGI34707, SGI34708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34709, SGI0, SGI0, SGI0, SGI34710, SGI34710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7854[] = {SGI34711, SGI34712, SGI34713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34714, SGI0, SGI0, SGI0, SGI34715, SGI34715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7855[] = {SGI34716, SGI0, SGI34717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7856[] = {SGI34718, SGI0, SGI34719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34720, SGI0, SGI0, SGI0, SGI34721, SGI34721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7857[] = {SGI34722, SGI34723, SGI34724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34725, SGI0, SGI0, SGI0, SGI34726, SGI34726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7858[] = {SGI34727, SGI34728, SGI34729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34730, SGI34731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7859[] = {SGI34732, SGI34733, SGI34734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34735, SGI0, SGI0, SGI0, SGI34736, SGI34736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7860[] = {SGI34737, SGI0, SGI34738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34739, SGI0, SGI0, SGI0, SGI34740, SGI34740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7861[] = {SGI34741, SGI34742, SGI34743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34744, SGI0, SGI0, SGI0, SGI34745, SGI34745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7862[] = {SGI34746, SGI34747, SGI34748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34749, SGI0, SGI0, SGI0, SGI34750, SGI34750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7863[] = {SGI34751, SGI34752, SGI34753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34754, SGI0, SGI0, SGI0, SGI34755, SGI34755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7864[] = {SGI34756, SGI0, SGI34757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7865[] = {SGI34758, SGI34759, SGI34760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34764, SGI34765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7866[] = {SGI34761, SGI34762, SGI34763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34766, SGI34766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7867[] = {SGI34767, SGI34768, SGI34769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34770, SGI34770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7868[] = {SGI34771, SGI0, SGI34772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34773, SGI0, SGI0, SGI0, SGI34774, SGI34774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7869[] = {SGI34700, SGI34702, SGI34704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34705, SGI0, SGI0, SGI0, SGI34775, SGI34775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7870[] = {SGI34776, SGI34777, SGI34778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34779, SGI0, SGI0, SGI0, SGI34780, SGI34780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7871[] = {SGI34781, SGI34782, SGI34783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34784, SGI0, SGI0, SGI0, SGI34785, SGI34785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7872[] = {SGI34786, SGI34787, SGI34788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34789, SGI0, SGI0, SGI0, SGI34790, SGI34790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7873[] = {SGI34791, SGI34792, SGI34793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34794, SGI0, SGI0, SGI0, SGI34795, SGI34795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7874[] = {SGI34796, SGI34797, SGI34798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34799, SGI0, SGI0, SGI0, SGI34800, SGI34800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7875[] = {SGI34801, SGI34802, SGI34803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34804, SGI0, SGI0, SGI0, SGI34805, SGI34805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7876[] = {SGI34806, SGI34807, SGI34808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34809, SGI0, SGI0, SGI0, SGI34810, SGI34810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7877[] = {SGI34811, SGI0, SGI34812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34813, SGI0, SGI0, SGI0, SGI34814, SGI34814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7878[] = {SGI34815, SGI34816, SGI34817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34818, SGI0, SGI0, SGI0, SGI34819, SGI34819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7879[] = {SGI34820, SGI0, SGI34821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7880[] = {SGI34822, SGI34823, SGI34824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7881[] = {SGI34825, SGI34826, SGI34827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34828, SGI0, SGI0, SGI0, SGI34829, SGI34830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7882[] = {SGI34831, SGI0, SGI34832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34833, SGI0, SGI0, SGI0, SGI34834, SGI34834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7883[] = {SGI34835, SGI34836, SGI34837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34838, SGI34839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7884[] = {SGI34840, SGI34841, SGI34842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34843, SGI0, SGI0, SGI0, SGI34844, SGI34844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7885[] = {SGI34845, SGI34846, SGI34847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34848, SGI0, SGI0, SGI0, SGI34849, SGI34849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7886[] = {SGI34850, SGI34851, SGI34852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34853, SGI0, SGI0, SGI0, SGI34854, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7887[] = {SGI34855, SGI34856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34857, SGI0, SGI0, SGI0, SGI34858, SGI34858, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7888[] = {SGI34859, SGI0, SGI34860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34861, SGI0, SGI0, SGI0, SGI34862, SGI34862, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7889[] = {SGI34863, SGI34864, SGI34865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34866, SGI0, SGI0, SGI0, SGI34867, SGI34867, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7890[] = {SGI34868, SGI34869, SGI34870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34871, SGI0, SGI0, SGI0, SGI34872, SGI34872, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7891[] = {SGI34873, SGI34874, SGI34875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34876, SGI0, SGI0, SGI0, SGI34877, SGI34877, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7892[] = {SGI34878, SGI34879, SGI34880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34881, SGI0, SGI0, SGI0, SGI34882, SGI34882, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7893[] = {SGI34883, SGI0, SGI34884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7894[] = {SGI34885, SGI34886, SGI34887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34888, SGI0, SGI0, SGI0, SGI34889, SGI34889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7895[] = {SGI34890, SGI34891, SGI34892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34893, SGI0, SGI0, SGI0, SGI34894, SGI34894, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7896[] = {SGI34895, SGI0, SGI34896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7897[] = {SGI34897, SGI0, SGI34898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34899, SGI0, SGI0, SGI0, SGI34900, SGI34900, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7898[] = {SGI34901, SGI34902, SGI34903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34904, SGI0, SGI0, SGI0, SGI34905, SGI34905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7899[] = {SGI34906, SGI0, SGI34907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7900[] = {SGI34908, SGI34909, SGI34910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34911, SGI0, SGI0, SGI0, SGI34912, SGI34912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7901[] = {SGI34913, SGI34914, SGI34915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34916, SGI0, SGI0, SGI0, SGI34917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7902[] = {SGI34918, SGI34919, SGI34920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34921, SGI0, SGI0, SGI0, SGI34922, SGI34922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7903[] = {SGI34923, SGI34924, SGI34925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34926, SGI0, SGI0, SGI0, SGI34927, SGI34927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7904[] = {SGI34928, SGI34929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34930, SGI0, SGI0, SGI0, SGI34931, SGI34931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7905[] = {SGI34932, SGI34933, SGI34934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34935, SGI0, SGI0, SGI0, SGI34936, SGI34936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7906[] = {SGI34937, SGI34938, SGI34939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34940, SGI0, SGI0, SGI0, SGI34941, SGI34941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7907[] = {SGI34942, SGI34943, SGI34944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7908[] = {SGI34945, SGI34946, SGI34947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34948, SGI0, SGI0, SGI0, SGI34949, SGI34949, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7909[] = {SGI34950, SGI0, SGI34951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7910[] = {SGI34952, SGI34953, SGI34954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34955, SGI0, SGI0, SGI0, SGI34956, SGI34956, SGI0, SGI0, NULL};
const Segment * const SG7911[] = {SGI34957, SGI34958, SGI34959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34960, SGI0, SGI0, SGI0, SGI34961, SGI34961, SGI0, SGI0, NULL};
const Segment * const SG7912[] = {SGI34962, SGI34963, SGI34964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34965, SGI0, SGI0, SGI0, SGI34966, SGI34966, SGI0, SGI0, NULL};
const Segment * const SG7913[] = {SGI34967, SGI0, SGI34968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7914[] = {SGI34969, SGI34970, SGI34971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34972, SGI0, SGI0, SGI0, SGI34973, SGI34973, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7915[] = {SGI34974, SGI0, SGI34975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34976, SGI0, SGI0, SGI0, SGI34977, SGI34977, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7916[] = {SGI34978, SGI34979, SGI34980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34981, SGI0, SGI0, SGI0, SGI34982, SGI34982, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7917[] = {SGI34983, SGI0, SGI34984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7918[] = {SGI34985, SGI34986, SGI34987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34988, SGI0, SGI0, SGI0, SGI34989, SGI34990, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7919[] = {SGI34991, SGI34992, SGI34993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34994, SGI0, SGI0, SGI0, SGI34995, SGI34995, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7920[] = {SGI34996, SGI34997, SGI34998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI34999, SGI0, SGI0, SGI0, SGI35000, SGI35000, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7921[] = {SGI35001, SGI0, SGI35002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7922[] = {SGI35003, SGI35004, SGI35005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35006, SGI0, SGI0, SGI0, SGI35007, SGI35007, SGI0, SGI0, NULL};
const Segment * const SG7923[] = {SGI35008, SGI35009, SGI35010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35011, SGI0, SGI0, SGI0, SGI35012, SGI35012, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7924[] = {SGI35013, SGI35014, SGI35015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35016, SGI0, SGI0, SGI0, SGI35017, SGI35017, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7925[] = {SGI35018, SGI35019, SGI35020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35021, SGI0, SGI0, SGI0, SGI35022, SGI35022, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7926[] = {SGI35023, SGI35024, SGI35025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35026, SGI0, SGI0, SGI0, SGI35027, SGI35027, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7927[] = {SGI35028, SGI35029, SGI35030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35031, SGI0, SGI0, SGI0, SGI35032, SGI35032, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7928[] = {SGI35033, SGI35034, SGI35035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35036, SGI0, SGI0, SGI0, SGI35037, SGI35037, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7929[] = {SGI35041, SGI0, SGI35043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7930[] = {SGI35038, SGI35039, SGI35040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35042, SGI0, SGI0, SGI0, SGI35044, SGI35044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7931[] = {SGI35045, SGI0, SGI35046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7932[] = {SGI35047, SGI35048, SGI35049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35050, SGI0, SGI0, SGI0, SGI35051, SGI35051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7933[] = {SGI35052, SGI0, SGI35053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7934[] = {SGI35054, SGI0, SGI35055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7935[] = {SGI35056, SGI35057, SGI35058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35059, SGI0, SGI0, SGI0, SGI35060, SGI35060, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7936[] = {SGI35061, SGI0, SGI35062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7937[] = {SGI35063, SGI35064, SGI35065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35066, SGI0, SGI0, SGI0, SGI35067, SGI35067, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7938[] = {SGI35068, SGI35069, SGI35070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35071, SGI0, SGI0, SGI0, SGI35072, SGI35072, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7939[] = {SGI35073, SGI35074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35075, SGI0, SGI0, SGI0, SGI35076, SGI35076, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7940[] = {SGI35077, SGI35078, SGI35079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35080, SGI0, SGI0, SGI0, SGI35081, SGI35081, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7941[] = {SGI35082, SGI35083, SGI35084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35085, SGI0, SGI0, SGI0, SGI35086, SGI35086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7942[] = {SGI35087, SGI0, SGI35088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7943[] = {SGI35089, SGI35090, SGI35091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35092, SGI0, SGI0, SGI0, SGI35093, SGI35093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7944[] = {SGI35094, SGI35095, SGI35096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35097, SGI0, SGI0, SGI0, SGI35098, SGI35098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7945[] = {SGI35099, SGI35100, SGI35101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35102, SGI0, SGI0, SGI0, SGI35103, SGI35103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7946[] = {SGI35104, SGI35105, SGI35106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35107, SGI0, SGI0, SGI0, SGI35108, SGI35108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7947[] = {SGI35109, SGI35110, SGI35111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35112, SGI0, SGI0, SGI0, SGI35113, SGI35113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7948[] = {SGI35114, SGI35115, SGI35116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35117, SGI0, SGI0, SGI0, SGI35118, SGI35118, SGI0, SGI0, NULL};
const Segment * const SG7949[] = {SGI35119, SGI35120, SGI35121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35122, SGI0, SGI0, SGI0, SGI35123, SGI35123, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7950[] = {SGI35124, SGI0, SGI35125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7951[] = {SGI35126, SGI35127, SGI35128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35129, SGI0, SGI0, SGI0, SGI35130, SGI35130, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7952[] = {SGI35131, SGI0, SGI35132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35133, SGI0, SGI0, SGI0, SGI35134, SGI35134, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7953[] = {SGI35135, SGI0, SGI35136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7954[] = {SGI35137, SGI35138, SGI35139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35140, SGI0, SGI0, SGI0, SGI35141, SGI35141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7955[] = {SGI35142, SGI35143, SGI35144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35145, SGI0, SGI0, SGI0, SGI35146, SGI35146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7956[] = {SGI35147, SGI35148, SGI35149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35150, SGI0, SGI0, SGI0, SGI35151, SGI35151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7957[] = {SGI35152, SGI35153, SGI35154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35155, SGI0, SGI0, SGI0, SGI35156, SGI35156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7958[] = {SGI35157, SGI35158, SGI35159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35160, SGI0, SGI0, SGI0, SGI35161, SGI35162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7959[] = {SGI35163, SGI0, SGI35164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7960[] = {SGI35165, SGI35166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35167, SGI0, SGI0, SGI0, SGI35168, SGI35168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7961[] = {SGI35169, SGI35170, SGI35171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35172, SGI0, SGI0, SGI0, SGI35173, SGI35173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7962[] = {SGI35174, SGI35175, SGI35176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35177, SGI0, SGI0, SGI0, SGI35178, SGI35178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7963[] = {SGI35179, SGI35180, SGI35181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35182, SGI0, SGI0, SGI0, SGI35183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7964[] = {SGI35184, SGI35185, SGI35186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35187, SGI0, SGI0, SGI0, SGI35188, SGI35188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7965[] = {SGI35189, SGI0, SGI35190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7966[] = {SGI35191, SGI35192, SGI35193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35194, SGI0, SGI0, SGI0, SGI35195, SGI35195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7967[] = {SGI35196, SGI35197, SGI35198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35199, SGI0, SGI0, SGI0, SGI35200, SGI35200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7968[] = {SGI35201, SGI35202, SGI35203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35204, SGI0, SGI0, SGI0, SGI35205, SGI35205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7969[] = {SGI35206, SGI35207, SGI35208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35209, SGI0, SGI0, SGI0, SGI35210, SGI35210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7970[] = {SGI35211, SGI35212, SGI35213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35214, SGI0, SGI0, SGI0, SGI35215, SGI35215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7971[] = {SGI35216, SGI35217, SGI35218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35219, SGI0, SGI0, SGI0, SGI35220, SGI35220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7972[] = {SGI35221, SGI35222, SGI35223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35224, SGI0, SGI0, SGI0, SGI35225, SGI35225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7973[] = {SGI35226, SGI35227, SGI35228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35229, SGI0, SGI0, SGI0, SGI35230, SGI35230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7974[] = {SGI35231, SGI35232, SGI35233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35234, SGI0, SGI0, SGI0, SGI35235, SGI35235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7975[] = {SGI35236, SGI35237, SGI35238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35239, SGI0, SGI0, SGI0, SGI35240, SGI35240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7976[] = {SGI35241, SGI35242, SGI35243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35244, SGI0, SGI0, SGI0, SGI35245, SGI35245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7977[] = {SGI35246, SGI35247, SGI35248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35249, SGI0, SGI0, SGI0, SGI35250, SGI35250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7978[] = {SGI35251, SGI0, SGI35252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7979[] = {SGI35253, SGI35254, SGI35255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35256, SGI0, SGI0, SGI0, SGI35257, SGI35257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7980[] = {SGI35258, SGI35259, SGI35260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35261, SGI0, SGI0, SGI0, SGI35262, SGI35262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7981[] = {SGI35263, SGI35264, SGI35265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35266, SGI0, SGI0, SGI0, SGI35267, SGI35267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7982[] = {SGI35268, SGI35269, SGI35270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35271, SGI0, SGI0, SGI0, SGI35272, SGI35272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7983[] = {SGI35273, SGI35274, SGI35275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35276, SGI0, SGI0, SGI0, SGI35277, SGI35277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7984[] = {SGI35278, SGI0, SGI35279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7985[] = {SGI35280, SGI35281, SGI35282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35283, SGI0, SGI0, SGI0, SGI35284, SGI35284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7986[] = {SGI35285, SGI35286, SGI35287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35288, SGI0, SGI0, SGI0, SGI35289, SGI35289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7987[] = {SGI35290, SGI35291, SGI35292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35293, SGI0, SGI0, SGI0, SGI35294, SGI35294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7988[] = {SGI35295, SGI0, SGI35296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7989[] = {SGI35297, SGI35298, SGI35299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35300, SGI0, SGI0, SGI0, SGI35301, SGI35301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7990[] = {SGI35302, SGI35303, SGI35304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35305, SGI0, SGI0, SGI0, SGI35306, SGI35306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7991[] = {SGI35307, SGI35308, SGI35309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35310, SGI0, SGI0, SGI0, SGI35311, SGI35311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7992[] = {SGI35312, SGI35313, SGI35314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35315, SGI0, SGI0, SGI0, SGI35316, SGI35316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7993[] = {SGI35317, SGI0, SGI35318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7994[] = {SGI35319, SGI35320, SGI35321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35322, SGI0, SGI0, SGI0, SGI35323, SGI35323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7995[] = {SGI35324, SGI35325, SGI35326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35327, SGI0, SGI0, SGI0, SGI35328, SGI35328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7996[] = {SGI35329, SGI35330, SGI35331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35332, SGI0, SGI0, SGI0, SGI35333, SGI35334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7997[] = {SGI35335, SGI35336, SGI35337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35338, SGI0, SGI0, SGI0, SGI35339, SGI35339, SGI0, SGI0, NULL};
const Segment * const SG7998[] = {SGI35340, SGI35341, SGI35342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35343, SGI0, SGI0, SGI0, SGI35344, SGI35344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG7999[] = {SGI35345, SGI35346, SGI35347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35348, SGI0, SGI0, SGI0, SGI35349, SGI35349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8000[] = {SGI35350, SGI0, SGI35351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8001[] = {SGI35352, SGI35353, SGI35354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35355, SGI0, SGI0, SGI0, SGI35357, SGI35357, SGI0, SGI0, NULL};
const Segment * const SG8002[] = {SGI35356, SGI35358, SGI35359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35360, SGI0, SGI0, SGI0, SGI35361, SGI35361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8003[] = {SGI35362, SGI0, SGI35363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8004[] = {SGI35364, SGI35365, SGI35366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35369, SGI0, SGI0, SGI0, SGI35371, SGI35371, SGI0, SGI0, NULL};
const Segment * const SG8005[] = {SGI35367, SGI35368, SGI35370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35372, SGI0, SGI0, SGI0, SGI35373, SGI35373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8006[] = {SGI35374, SGI35375, SGI35376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35377, SGI35378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8007[] = {SGI35379, SGI0, SGI35380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8008[] = {SGI35381, SGI35382, SGI35383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35384, SGI35384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8009[] = {SGI35385, SGI0, SGI35386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35387, SGI0, SGI0, SGI0, SGI35388, SGI35388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8010[] = {SGI35389, SGI35390, SGI35391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35392, SGI0, SGI0, SGI0, SGI35393, SGI35393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8011[] = {SGI35394, SGI35395, SGI35396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35397, SGI0, SGI0, SGI0, SGI35398, SGI35398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8012[] = {SGI35399, SGI35400, SGI35401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35402, SGI0, SGI0, SGI0, SGI35403, SGI35403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8013[] = {SGI35404, SGI35405, SGI35406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35407, SGI0, SGI0, SGI0, SGI35408, SGI35408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8014[] = {SGI35409, SGI0, SGI35410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35411, SGI0, SGI0, SGI0, SGI35412, SGI35412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8015[] = {SGI35413, SGI0, SGI35414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8016[] = {SGI35415, SGI35416, SGI35417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35418, SGI35419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8017[] = {SGI35420, SGI35421, SGI35422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35423, SGI0, SGI0, SGI0, SGI35424, SGI35424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8018[] = {SGI35425, SGI35426, SGI35427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35428, SGI35428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8019[] = {SGI0, SGI35429, SGI35430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8020[] = {SGI35431, SGI0, SGI35432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8021[] = {SGI35433, SGI0, SGI35434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35435, SGI0, SGI0, SGI0, SGI35436, SGI35436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8022[] = {SGI35437, SGI35438, SGI35439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35440, SGI0, SGI0, SGI0, SGI35441, SGI35441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8023[] = {SGI35442, SGI35443, SGI35444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35445, SGI0, SGI0, SGI0, SGI35446, SGI35446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8024[] = {SGI35447, SGI35448, SGI35449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35450, SGI0, SGI0, SGI0, SGI35451, SGI35451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8025[] = {SGI35452, SGI35453, SGI35454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35455, SGI0, SGI0, SGI0, SGI35456, SGI35456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8026[] = {SGI35457, SGI35458, SGI35459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35460, SGI0, SGI0, SGI0, SGI35461, SGI35461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8027[] = {SGI35462, SGI35463, SGI35464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35465, SGI0, SGI0, SGI0, SGI35466, SGI35466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8028[] = {SGI35467, SGI35468, SGI35469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35470, SGI0, SGI0, SGI0, SGI35471, SGI35471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8029[] = {SGI35472, SGI35473, SGI35474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35475, SGI0, SGI0, SGI0, SGI35476, SGI35476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8030[] = {SGI35477, SGI35478, SGI35479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35480, SGI0, SGI0, SGI0, SGI35481, SGI35481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8031[] = {SGI35482, SGI35483, SGI35484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35485, SGI35486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8032[] = {SGI35487, SGI35488, SGI35489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35490, SGI0, SGI0, SGI0, SGI35492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8033[] = {SGI35491, SGI35493, SGI35494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35495, SGI0, SGI0, SGI0, SGI35496, SGI35496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8034[] = {SGI35497, SGI35498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35499, SGI0, SGI0, SGI0, SGI35501, SGI35501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8035[] = {SGI35500, SGI35502, SGI35503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35504, SGI0, SGI0, SGI0, SGI35505, SGI35505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8036[] = {SGI35506, SGI35507, SGI35508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35509, SGI0, SGI0, SGI0, SGI35510, SGI35510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8037[] = {SGI35511, SGI35512, SGI35513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35514, SGI0, SGI0, SGI0, SGI35515, SGI35515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8038[] = {SGI35516, SGI35517, SGI35518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35519, SGI0, SGI0, SGI0, SGI35520, SGI35520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8039[] = {SGI35521, SGI35522, SGI35523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35524, SGI35524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8040[] = {SGI35525, SGI35526, SGI35527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35528, SGI0, SGI0, SGI0, SGI35529, SGI35529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8041[] = {SGI0, SGI35530, SGI35531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35532, SGI0, SGI0, SGI0, SGI35533, SGI35533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8042[] = {SGI35534, SGI35535, SGI35536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35537, SGI0, SGI0, SGI0, SGI35538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8043[] = {SGI35539, SGI35540, SGI35541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35542, SGI0, SGI0, SGI0, SGI35543, SGI35543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8044[] = {SGI35544, SGI35545, SGI35546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35547, SGI0, SGI0, SGI0, SGI35548, SGI35548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8045[] = {SGI35549, SGI0, SGI35550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8046[] = {SGI35551, SGI0, SGI35552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8047[] = {SGI35553, SGI0, SGI35554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8048[] = {SGI35555, SGI35556, SGI35557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35558, SGI0, SGI0, SGI0, SGI35559, SGI35559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8049[] = {SGI35560, SGI35561, SGI35562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35563, SGI0, SGI0, SGI0, SGI35564, SGI35564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8050[] = {SGI35565, SGI35566, SGI35567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35568, SGI0, SGI0, SGI0, SGI35569, SGI35571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8051[] = {SGI35570, SGI35572, SGI35573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35574, SGI0, SGI0, SGI0, SGI35575, SGI35575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8052[] = {SGI35576, SGI35577, SGI35578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35579, SGI35579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8053[] = {SGI35580, SGI35581, SGI35582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35583, SGI0, SGI0, SGI0, SGI35584, SGI35584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8054[] = {SGI35585, SGI35586, SGI35587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35588, SGI0, SGI0, SGI0, SGI35589, SGI35589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8055[] = {SGI35590, SGI35591, SGI35592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35593, SGI0, SGI0, SGI0, SGI35594, SGI35594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8056[] = {SGI35595, SGI35596, SGI35597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35598, SGI0, SGI0, SGI0, SGI35599, SGI35599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8057[] = {SGI35600, SGI35601, SGI35602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35603, SGI0, SGI0, SGI0, SGI35604, SGI35604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8058[] = {SGI35605, SGI0, SGI35606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8059[] = {SGI35607, SGI35608, SGI35609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35610, SGI0, SGI0, SGI0, SGI35611, SGI35611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8060[] = {SGI35612, SGI35613, SGI35614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35615, SGI0, SGI0, SGI0, SGI35616, SGI35616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8061[] = {SGI35617, SGI35618, SGI35619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35620, SGI0, SGI0, SGI0, SGI35621, SGI35621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8062[] = {SGI35622, SGI0, SGI35623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8063[] = {SGI35624, SGI35625, SGI35626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35627, SGI0, SGI0, SGI0, SGI35628, SGI35628, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8064[] = {SGI35629, SGI35630, SGI35631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35632, SGI0, SGI0, SGI0, SGI35633, SGI35633, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8065[] = {SGI35634, SGI35635, SGI35636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35637, SGI0, SGI0, SGI0, SGI35638, SGI35638, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8066[] = {SGI35639, SGI35640, SGI35641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35642, SGI0, SGI0, SGI0, SGI35643, SGI35643, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8067[] = {SGI35644, SGI35645, SGI35646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35647, SGI0, SGI0, SGI0, SGI35648, SGI35648, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8068[] = {SGI35649, SGI0, SGI35650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8069[] = {SGI35651, SGI35652, SGI35653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35654, SGI0, SGI0, SGI0, SGI35655, SGI35655, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8070[] = {SGI35656, SGI35657, SGI35658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35659, SGI0, SGI0, SGI0, SGI35660, SGI35660, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8071[] = {SGI35661, SGI35662, SGI35663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35664, SGI0, SGI0, SGI0, SGI35665, SGI35665, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8072[] = {SGI35666, SGI0, SGI35667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8073[] = {SGI35668, SGI35669, SGI35670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35671, SGI35672, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8074[] = {SGI35673, SGI0, SGI35674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8075[] = {SGI35675, SGI0, SGI35676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35677, SGI0, SGI0, SGI0, SGI35678, SGI35678, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8076[] = {SGI35679, SGI35680, SGI35681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35682, SGI0, SGI0, SGI0, SGI35683, SGI35683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8077[] = {SGI35684, SGI0, SGI35685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8078[] = {SGI35686, SGI0, SGI35687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35688, SGI0, SGI0, SGI0, SGI35689, SGI35689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8079[] = {SGI35690, SGI35691, SGI35692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35693, SGI0, SGI0, SGI0, SGI35694, SGI35694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8080[] = {SGI35695, SGI35696, SGI35697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35698, SGI0, SGI0, SGI0, SGI35699, SGI35699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8081[] = {SGI35700, SGI35701, SGI35702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35703, SGI0, SGI0, SGI0, SGI35704, SGI35704, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8082[] = {SGI35705, SGI0, SGI35706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8083[] = {SGI35707, SGI35708, SGI35709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35710, SGI0, SGI0, SGI0, SGI35711, SGI35711, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8084[] = {SGI35712, SGI0, SGI35713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8085[] = {SGI35714, SGI35715, SGI35716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35717, SGI0, SGI0, SGI0, SGI35718, SGI35718, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8086[] = {SGI35719, SGI35720, SGI35721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35722, SGI0, SGI0, SGI0, SGI35723, SGI35723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8087[] = {SGI35724, SGI35725, SGI35726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35727, SGI0, SGI0, SGI0, SGI35728, SGI35728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8088[] = {SGI35729, SGI35730, SGI35731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35732, SGI0, SGI0, SGI0, SGI35733, SGI35733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8089[] = {SGI35734, SGI35735, SGI35736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35737, SGI0, SGI0, SGI0, SGI35738, SGI35738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8090[] = {SGI35739, SGI35740, SGI35741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35742, SGI0, SGI0, SGI0, SGI35743, SGI35743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8091[] = {SGI35744, SGI35745, SGI35746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35747, SGI0, SGI0, SGI0, SGI35748, SGI35748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8092[] = {SGI35749, SGI35750, SGI35751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35752, SGI0, SGI0, SGI0, SGI35753, SGI35753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8093[] = {SGI35754, SGI35755, SGI35756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35757, SGI0, SGI0, SGI0, SGI35758, SGI35758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8094[] = {SGI35759, SGI35760, SGI35761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35762, SGI0, SGI0, SGI0, SGI35763, SGI35763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8095[] = {SGI35764, SGI35765, SGI35766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35767, SGI0, SGI0, SGI0, SGI35768, SGI35768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8096[] = {SGI35769, SGI35770, SGI35771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35772, SGI0, SGI0, SGI0, SGI35773, SGI35773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8097[] = {SGI35774, SGI35775, SGI35776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35777, SGI0, SGI0, SGI0, SGI35778, SGI35778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8098[] = {SGI35779, SGI35780, SGI35781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35782, SGI0, SGI0, SGI0, SGI35783, SGI35783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8099[] = {SGI35784, SGI35785, SGI35786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35787, SGI0, SGI0, SGI0, SGI35788, SGI35788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8100[] = {SGI35789, SGI35790, SGI35791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35792, SGI0, SGI0, SGI0, SGI35793, SGI35793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8101[] = {SGI35794, SGI35795, SGI35796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35797, SGI0, SGI0, SGI0, SGI35798, SGI35798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8102[] = {SGI35799, SGI35800, SGI35801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35802, SGI0, SGI0, SGI0, SGI35803, SGI35803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8103[] = {SGI35804, SGI35805, SGI35806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35807, SGI0, SGI0, SGI0, SGI35808, SGI35808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8104[] = {SGI35809, SGI35810, SGI35811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35812, SGI0, SGI0, SGI0, SGI35813, SGI35813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8105[] = {SGI35814, SGI35815, SGI35816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35817, SGI0, SGI0, SGI0, SGI35818, SGI35818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8106[] = {SGI35819, SGI35820, SGI35821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35822, SGI0, SGI0, SGI0, SGI35823, SGI35823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8107[] = {SGI35824, SGI35825, SGI35826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35827, SGI0, SGI0, SGI0, SGI35828, SGI35828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8108[] = {SGI35829, SGI35830, SGI35831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35832, SGI0, SGI0, SGI0, SGI35833, SGI35833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8109[] = {SGI35834, SGI35835, SGI35836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35837, SGI0, SGI0, SGI0, SGI35838, SGI35838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8110[] = {SGI35839, SGI35840, SGI35841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35842, SGI0, SGI0, SGI0, SGI35843, SGI35843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8111[] = {SGI35844, SGI35845, SGI35846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35847, SGI0, SGI0, SGI0, SGI35848, SGI35848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8112[] = {SGI35849, SGI35850, SGI35851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35852, SGI0, SGI0, SGI0, SGI35853, SGI35853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8113[] = {SGI35854, SGI35855, SGI35856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35857, SGI0, SGI0, SGI0, SGI35858, SGI35858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8114[] = {SGI35859, SGI35860, SGI35861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35862, SGI0, SGI0, SGI0, SGI35863, SGI35863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8115[] = {SGI35864, SGI35865, SGI35866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35867, SGI0, SGI0, SGI0, SGI35868, SGI35868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8116[] = {SGI35869, SGI35870, SGI35871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35872, SGI0, SGI0, SGI0, SGI35873, SGI35873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8117[] = {SGI35874, SGI35875, SGI35876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35877, SGI0, SGI0, SGI0, SGI35878, SGI35878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8118[] = {SGI35879, SGI0, SGI35880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8119[] = {SGI35881, SGI35882, SGI35883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35884, SGI0, SGI0, SGI0, SGI35885, SGI35885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8120[] = {SGI35886, SGI35887, SGI35888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35889, SGI0, SGI0, SGI0, SGI35890, SGI35890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8121[] = {SGI35891, SGI35892, SGI35893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35894, SGI0, SGI0, SGI0, SGI35895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8122[] = {SGI35896, SGI35897, SGI35898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35899, SGI35899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8123[] = {SGI35900, SGI35901, SGI35902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35903, SGI0, SGI0, SGI0, SGI35904, SGI35904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8124[] = {SGI35905, SGI35906, SGI35907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35908, SGI0, SGI0, SGI0, SGI35909, SGI35909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8125[] = {SGI35910, SGI0, SGI35911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8126[] = {SGI35912, SGI0, SGI35913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8127[] = {SGI35914, SGI35915, SGI35916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35917, SGI0, SGI0, SGI0, SGI35918, SGI35919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8128[] = {SGI35920, SGI35921, SGI35922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35923, SGI0, SGI0, SGI0, SGI35924, SGI35924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8129[] = {SGI35925, SGI0, SGI35926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8130[] = {SGI35927, SGI35928, SGI35929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35930, SGI0, SGI0, SGI0, SGI35931, SGI35931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8131[] = {SGI35932, SGI35933, SGI35934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35935, SGI0, SGI0, SGI0, SGI35936, SGI35936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8132[] = {SGI35937, SGI35938, SGI35939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35940, SGI0, SGI0, SGI0, SGI35941, SGI35941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8133[] = {SGI35942, SGI35943, SGI35944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35945, SGI0, SGI0, SGI0, SGI35946, SGI35946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8134[] = {SGI35947, SGI35948, SGI35949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35950, SGI0, SGI0, SGI0, SGI35951, SGI35951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8135[] = {SGI35952, SGI35953, SGI35954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35955, SGI0, SGI0, SGI0, SGI35956, SGI35956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8136[] = {SGI35957, SGI35958, SGI35959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35960, SGI0, SGI0, SGI0, SGI35961, SGI35961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8137[] = {SGI35962, SGI0, SGI35963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8138[] = {SGI35964, SGI0, SGI35965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8139[] = {SGI0, SGI35966, SGI35967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35968, SGI0, SGI0, SGI0, SGI0, SGI35969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8140[] = {SGI0, SGI35970, SGI35971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35972, SGI0, SGI0, SGI0, SGI0, SGI35973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8141[] = {SGI35974, SGI35975, SGI35976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35977, SGI35978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8142[] = {SGI35979, SGI35980, SGI35981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35982, SGI35982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8143[] = {SGI35983, SGI35984, SGI35985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35986, SGI0, SGI0, SGI0, SGI35987, SGI35987, SGI0, SGI0, NULL};
const Segment * const SG8144[] = {SGI35988, SGI0, SGI35989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8145[] = {SGI35990, SGI0, SGI35991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35992, SGI0, SGI0, SGI0, SGI35993, SGI35993, SGI0, SGI0, NULL};
const Segment * const SG8146[] = {SGI35994, SGI35995, SGI35996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI35997, SGI0, SGI0, SGI0, SGI35998, SGI35998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8147[] = {SGI35999, SGI0, SGI36000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8148[] = {SGI36001, SGI36002, SGI36003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36004, SGI0, SGI0, SGI0, SGI36005, SGI36005, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8149[] = {SGI36006, SGI36007, SGI36008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36009, SGI0, SGI0, SGI0, SGI36010, SGI36010, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8150[] = {SGI36011, SGI36012, SGI36013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36014, SGI0, SGI0, SGI0, SGI36015, SGI36015, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8151[] = {SGI36016, SGI0, SGI36017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8152[] = {SGI36018, SGI36019, SGI36020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36021, SGI0, SGI0, SGI0, SGI36022, SGI36022, SGI0, SGI0, NULL};
const Segment * const SG8153[] = {SGI36023, SGI36024, SGI36025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36026, SGI0, SGI0, SGI0, SGI36027, SGI36027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8154[] = {SGI36028, SGI36029, SGI36030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36031, SGI0, SGI0, SGI0, SGI36032, SGI36032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8155[] = {SGI36033, SGI36034, SGI36035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36036, SGI0, SGI0, SGI0, SGI36037, SGI36037, SGI0, SGI0, NULL};
const Segment * const SG8156[] = {SGI36038, SGI0, SGI36039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8157[] = {SGI36040, SGI36041, SGI36042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36043, SGI0, SGI0, SGI0, SGI36044, SGI36044, SGI0, SGI0, NULL};
const Segment * const SG8158[] = {SGI36045, SGI36046, SGI36047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36048, SGI0, SGI0, SGI0, SGI36049, SGI36049, SGI0, SGI0, NULL};
const Segment * const SG8159[] = {SGI36050, SGI36051, SGI36052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36053, SGI0, SGI0, SGI0, SGI36054, SGI36054, SGI0, SGI0, NULL};
const Segment * const SG8160[] = {SGI36055, SGI36056, SGI36057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36058, SGI0, SGI0, SGI0, SGI36060, SGI36060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8161[] = {SGI36059, SGI36061, SGI36062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36063, SGI0, SGI0, SGI0, SGI36064, SGI36064, SGI0, SGI0, NULL};
const Segment * const SG8162[] = {SGI36065, SGI36066, SGI36067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36068, SGI0, SGI0, SGI0, SGI36069, SGI36069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8163[] = {SGI36070, SGI36071, SGI36072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36073, SGI0, SGI0, SGI0, SGI36074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8164[] = {SGI36075, SGI36076, SGI36077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36078, SGI0, SGI0, SGI0, SGI36079, SGI36079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8165[] = {SGI36080, SGI36081, SGI36082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36083, SGI0, SGI0, SGI0, SGI36084, SGI36084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8166[] = {SGI36085, SGI36086, SGI36087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36088, SGI0, SGI0, SGI0, SGI36089, SGI36089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8167[] = {SGI36090, SGI36091, SGI36092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36093, SGI0, SGI0, SGI0, SGI36095, SGI36095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8168[] = {SGI36094, SGI36096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36097, SGI0, SGI0, SGI0, SGI36098, SGI36098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8169[] = {SGI36099, SGI36100, SGI36101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36102, SGI0, SGI0, SGI0, SGI36103, SGI36103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8170[] = {SGI36104, SGI36105, SGI36106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36107, SGI0, SGI0, SGI0, SGI36108, SGI36108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8171[] = {SGI36109, SGI36110, SGI36111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36112, SGI0, SGI0, SGI0, SGI36113, SGI36113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8172[] = {SGI36114, SGI36115, SGI36116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36117, SGI0, SGI0, SGI0, SGI36118, SGI36118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8173[] = {SGI36119, SGI36120, SGI36121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36122, SGI0, SGI0, SGI0, SGI36123, SGI36123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8174[] = {SGI36124, SGI36125, SGI36126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36127, SGI0, SGI0, SGI0, SGI36128, SGI36128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8175[] = {SGI36129, SGI36130, SGI36131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36132, SGI0, SGI0, SGI0, SGI36133, SGI36133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8176[] = {SGI36137, SGI0, SGI36138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8177[] = {SGI36134, SGI36135, SGI36136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36139, SGI0, SGI0, SGI0, SGI36140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8178[] = {SGI36141, SGI0, SGI36142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8179[] = {SGI36143, SGI36144, SGI36145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36146, SGI0, SGI0, SGI0, SGI36147, SGI36147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8180[] = {SGI36148, SGI36149, SGI36150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36151, SGI0, SGI0, SGI0, SGI36152, SGI36152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8181[] = {SGI36153, SGI36154, SGI36155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36156, SGI0, SGI0, SGI0, SGI36157, SGI36157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8182[] = {SGI36158, SGI36159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36160, SGI0, SGI0, SGI0, SGI36161, SGI36161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8183[] = {SGI36162, SGI36163, SGI36164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36165, SGI0, SGI0, SGI0, SGI36166, SGI36166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8184[] = {SGI36167, SGI36168, SGI36169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36170, SGI0, SGI0, SGI0, SGI36171, SGI36171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8185[] = {SGI36172, SGI36173, SGI36174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36175, SGI0, SGI0, SGI0, SGI36176, SGI36176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8186[] = {SGI36177, SGI36178, SGI36179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36180, SGI0, SGI0, SGI0, SGI36181, SGI36181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8187[] = {SGI36182, SGI0, SGI36183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8188[] = {SGI36184, SGI36185, SGI36186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36187, SGI0, SGI0, SGI0, SGI36188, SGI36188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8189[] = {SGI36189, SGI36190, SGI36191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36192, SGI0, SGI0, SGI0, SGI36193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8190[] = {SGI36194, SGI36195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36196, SGI0, SGI0, SGI0, SGI36197, SGI36197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8191[] = {SGI36198, SGI36199, SGI36200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36201, SGI0, SGI0, SGI0, SGI36202, SGI36202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8192[] = {SGI36203, SGI36204, SGI36205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36206, SGI0, SGI0, SGI0, SGI36207, SGI36207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8193[] = {SGI36208, SGI36209, SGI36210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36211, SGI0, SGI0, SGI0, SGI36212, SGI36212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8194[] = {SGI36213, SGI36214, SGI36215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36216, SGI0, SGI0, SGI0, SGI36217, SGI36217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8195[] = {SGI36218, SGI36219, SGI36220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36221, SGI0, SGI0, SGI0, SGI36222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8196[] = {SGI36223, SGI36224, SGI36225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36226, SGI0, SGI0, SGI0, SGI36227, SGI36227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8197[] = {SGI36228, SGI36229, SGI36230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36231, SGI0, SGI0, SGI0, SGI36232, SGI36232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8198[] = {SGI36233, SGI36234, SGI36235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36236, SGI0, SGI0, SGI0, SGI36237, SGI36237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8199[] = {SGI36238, SGI36239, SGI36240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36241, SGI0, SGI0, SGI0, SGI36242, SGI36242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8200[] = {SGI36243, SGI36244, SGI36245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36246, SGI0, SGI0, SGI0, SGI36247, SGI36247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8201[] = {SGI36248, SGI0, SGI36249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8202[] = {SGI36250, SGI36251, SGI36252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36253, SGI0, SGI0, SGI0, SGI36254, SGI36254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8203[] = {SGI36255, SGI36256, SGI36257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36258, SGI0, SGI0, SGI0, SGI36259, SGI36259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8204[] = {SGI36260, SGI36261, SGI36262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36263, SGI0, SGI0, SGI0, SGI36264, SGI36264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8205[] = {SGI36265, SGI36266, SGI36267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36268, SGI0, SGI0, SGI0, SGI36269, SGI36269, SGI0, SGI0, NULL};
const Segment * const SG8206[] = {SGI36270, SGI36271, SGI36272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36273, SGI0, SGI0, SGI0, SGI36274, SGI36274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8207[] = {SGI36275, SGI0, SGI36276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36277, SGI0, SGI0, SGI0, SGI36278, SGI36278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8208[] = {SGI36279, SGI0, SGI36280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8209[] = {SGI36281, SGI36282, SGI36283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36284, SGI0, SGI0, SGI0, SGI36285, SGI36285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8210[] = {SGI36286, SGI0, SGI36287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36288, SGI0, SGI0, SGI0, SGI36289, SGI36289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8211[] = {SGI36290, SGI36291, SGI36292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36293, SGI0, SGI0, SGI0, SGI36294, SGI36294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8212[] = {SGI36295, SGI36296, SGI36297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36298, SGI0, SGI0, SGI0, SGI36299, SGI36299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8213[] = {SGI36300, SGI36301, SGI36302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36303, SGI0, SGI0, SGI0, SGI36304, SGI36304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8214[] = {SGI36305, SGI36306, SGI36307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36308, SGI0, SGI0, SGI0, SGI36309, SGI36309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8215[] = {SGI36310, SGI36311, SGI36312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36313, SGI0, SGI0, SGI0, SGI36314, SGI36314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8216[] = {SGI36315, SGI36316, SGI36317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36318, SGI0, SGI0, SGI0, SGI36319, SGI36319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8217[] = {SGI36320, SGI36321, SGI36322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36323, SGI0, SGI0, SGI0, SGI36324, SGI36324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8218[] = {SGI36325, SGI36326, SGI36327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36328, SGI36329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8219[] = {SGI36330, SGI0, SGI36331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8220[] = {SGI36332, SGI36333, SGI36334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36335, SGI36335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8221[] = {SGI36336, SGI0, SGI36337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36338, SGI0, SGI0, SGI0, SGI36339, SGI36339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8222[] = {SGI36340, SGI36341, SGI36342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36343, SGI0, SGI0, SGI0, SGI36344, SGI36344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8223[] = {SGI36345, SGI0, SGI36346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36347, SGI0, SGI0, SGI0, SGI36348, SGI36348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8224[] = {SGI36349, SGI36350, SGI36351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36352, SGI0, SGI0, SGI0, SGI36353, SGI36353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8225[] = {SGI36354, SGI36355, SGI36356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36357, SGI0, SGI0, SGI0, SGI36358, SGI36358, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8226[] = {SGI36359, SGI36360, SGI36361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36362, SGI36363, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8227[] = {SGI36364, SGI0, SGI36365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8228[] = {SGI36366, SGI36367, SGI36368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36369, SGI0, SGI0, SGI0, SGI36370, SGI36370, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8229[] = {SGI36371, SGI0, SGI36372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8230[] = {SGI36373, SGI36374, SGI36375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36376, SGI0, SGI0, SGI0, SGI36377, SGI36377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8231[] = {SGI36378, SGI36379, SGI36380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36381, SGI0, SGI0, SGI0, SGI36382, SGI36382, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8232[] = {SGI36383, SGI36384, SGI36385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36386, SGI0, SGI0, SGI0, SGI36387, SGI36387, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8233[] = {SGI36388, SGI36389, SGI36390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36391, SGI0, SGI0, SGI0, SGI36392, SGI36392, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8234[] = {SGI36393, SGI36394, SGI36395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36396, SGI0, SGI0, SGI0, SGI36397, SGI36397, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8235[] = {SGI36398, SGI36399, SGI36400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36401, SGI0, SGI0, SGI0, SGI36402, SGI36402, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8236[] = {SGI36403, SGI36404, SGI36405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36406, SGI0, SGI0, SGI0, SGI36407, SGI36407, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8237[] = {SGI36408, SGI36409, SGI36410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36411, SGI0, SGI0, SGI0, SGI36412, SGI36412, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8238[] = {SGI36413, SGI0, SGI36414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8239[] = {SGI36415, SGI36416, SGI36417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36418, SGI0, SGI0, SGI0, SGI36419, SGI36419, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8240[] = {SGI36420, SGI36421, SGI36422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36423, SGI0, SGI0, SGI0, SGI36424, SGI36424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8241[] = {SGI36425, SGI36426, SGI36427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36428, SGI0, SGI0, SGI0, SGI36429, SGI36429, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8242[] = {SGI36430, SGI36431, SGI36432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36436, SGI0, SGI0, SGI0, SGI36437, SGI36437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8243[] = {SGI36433, SGI36434, SGI36435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36438, SGI0, SGI0, SGI0, SGI36439, SGI36440, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8244[] = {SGI36441, SGI0, SGI36442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8245[] = {SGI36443, SGI0, SGI36444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8246[] = {SGI36445, SGI36446, SGI36447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36448, SGI0, SGI0, SGI0, SGI36449, SGI36449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8247[] = {SGI36450, SGI36451, SGI36452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36453, SGI0, SGI0, SGI0, SGI36454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8248[] = {SGI36455, SGI36456, SGI36457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36458, SGI0, SGI0, SGI0, SGI36459, SGI36459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8249[] = {SGI36460, SGI36461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36462, SGI0, SGI0, SGI0, SGI36463, SGI36463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8250[] = {SGI36464, SGI36465, SGI36466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36467, SGI0, SGI0, SGI0, SGI36468, SGI36468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8251[] = {SGI36469, SGI36470, SGI36471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36472, SGI0, SGI0, SGI0, SGI36473, SGI36473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8252[] = {SGI36474, SGI36475, SGI36476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36477, SGI0, SGI0, SGI0, SGI36478, SGI36478, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8253[] = {SGI36479, SGI0, SGI36480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8254[] = {SGI36481, SGI36482, SGI36483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36484, SGI36485, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8255[] = {SGI36486, SGI36487, SGI36488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36489, SGI0, SGI0, SGI0, SGI36490, SGI36490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8256[] = {SGI36491, SGI0, SGI36492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8257[] = {SGI36493, SGI36494, SGI36495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36496, SGI0, SGI0, SGI0, SGI36497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8258[] = {SGI36498, SGI36499, SGI36500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36501, SGI0, SGI0, SGI0, SGI36502, SGI36502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8259[] = {SGI36503, SGI36504, SGI36505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36506, SGI0, SGI0, SGI0, SGI36507, SGI36507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8260[] = {SGI36508, SGI36509, SGI36510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36511, SGI0, SGI0, SGI0, SGI36512, SGI36512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8261[] = {SGI36513, SGI0, SGI36514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8262[] = {SGI36515, SGI36516, SGI36517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36518, SGI0, SGI0, SGI0, SGI36519, SGI36519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8263[] = {SGI36520, SGI0, SGI36521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8264[] = {SGI36522, SGI36523, SGI36524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36525, SGI0, SGI0, SGI0, SGI36526, SGI36526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8265[] = {SGI36527, SGI36528, SGI36529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36530, SGI0, SGI0, SGI0, SGI36531, SGI36531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8266[] = {SGI36532, SGI36533, SGI36534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36535, SGI0, SGI0, SGI0, SGI36536, SGI36536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8267[] = {SGI36537, SGI36538, SGI36539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36540, SGI0, SGI0, SGI0, SGI36541, SGI36541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8268[] = {SGI36542, SGI36543, SGI36544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36545, SGI0, SGI0, SGI0, SGI36546, SGI36546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8269[] = {SGI36547, SGI36548, SGI36549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36550, SGI0, SGI0, SGI0, SGI36551, SGI36551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8270[] = {SGI36552, SGI36553, SGI36554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36555, SGI0, SGI0, SGI0, SGI36558, SGI36558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8271[] = {SGI36556, SGI0, SGI36557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8272[] = {SGI36559, SGI36560, SGI36561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36562, SGI0, SGI0, SGI0, SGI36563, SGI36563, SGI0, SGI0, NULL};
const Segment * const SG8273[] = {SGI36564, SGI36565, SGI36566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36567, SGI0, SGI0, SGI0, SGI36568, SGI36568, SGI0, SGI0, NULL};
const Segment * const SG8274[] = {SGI36569, SGI36570, SGI36571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36572, SGI0, SGI0, SGI0, SGI36573, SGI36573, SGI0, SGI0, NULL};
const Segment * const SG8275[] = {SGI36574, SGI36575, SGI36576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36577, SGI0, SGI0, SGI0, SGI36578, SGI36578, SGI0, SGI0, NULL};
const Segment * const SG8276[] = {SGI36579, SGI0, SGI36580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8277[] = {SGI36581, SGI36582, SGI36583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36586, SGI0, SGI0, SGI0, SGI36588, SGI36588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8278[] = {SGI36584, SGI36585, SGI36587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36589, SGI0, SGI0, SGI0, SGI36590, SGI36590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8279[] = {SGI36591, SGI36592, SGI36593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36594, SGI0, SGI0, SGI0, SGI36595, SGI36595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8280[] = {SGI36596, SGI0, SGI36597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8281[] = {SGI36598, SGI36599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36600, SGI0, SGI0, SGI0, SGI36601, SGI36601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8282[] = {SGI36602, SGI36603, SGI36604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36605, SGI0, SGI0, SGI0, SGI36606, SGI36606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8283[] = {SGI36607, SGI36608, SGI36609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36610, SGI0, SGI0, SGI0, SGI36611, SGI36611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8284[] = {SGI36612, SGI36613, SGI36614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36615, SGI0, SGI0, SGI0, SGI36616, SGI36616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8285[] = {SGI36617, SGI36618, SGI36619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36620, SGI0, SGI0, SGI0, SGI36621, SGI36621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8286[] = {SGI36622, SGI36623, SGI36624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36625, SGI0, SGI0, SGI0, SGI36626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8287[] = {SGI36627, SGI36628, SGI36629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36630, SGI0, SGI0, SGI0, SGI36631, SGI36631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8288[] = {SGI36632, SGI36633, SGI36634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36635, SGI0, SGI0, SGI0, SGI36636, SGI36636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8289[] = {SGI36637, SGI36638, SGI36639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36640, SGI0, SGI0, SGI0, SGI36641, SGI36641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8290[] = {SGI36642, SGI36643, SGI36644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36645, SGI0, SGI0, SGI0, SGI36646, SGI36646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8291[] = {SGI36647, SGI36648, SGI36649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36650, SGI0, SGI0, SGI0, SGI36651, SGI36651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8292[] = {SGI36652, SGI36653, SGI36654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36655, SGI0, SGI0, SGI0, SGI36656, SGI36656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8293[] = {SGI36657, SGI36658, SGI36659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36660, SGI0, SGI0, SGI0, SGI36661, SGI36661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8294[] = {SGI36662, SGI0, SGI36663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8295[] = {SGI36664, SGI36665, SGI36666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36667, SGI36668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8296[] = {SGI36669, SGI36670, SGI36671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36672, SGI0, SGI0, SGI0, SGI36673, SGI36673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8297[] = {SGI36674, SGI36675, SGI36676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36677, SGI0, SGI0, SGI0, SGI36678, SGI36678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8298[] = {SGI36679, SGI36680, SGI36681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36682, SGI0, SGI0, SGI0, SGI36683, SGI36683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8299[] = {SGI36684, SGI36685, SGI36686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36687, SGI0, SGI0, SGI0, SGI36688, SGI36688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8300[] = {SGI36689, SGI36690, SGI36691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36692, SGI36693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8301[] = {SGI36694, SGI36695, SGI36696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36697, SGI36697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8302[] = {SGI36698, SGI36699, SGI36700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36701, SGI36701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8303[] = {SGI36702, SGI0, SGI36703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36704, SGI0, SGI0, SGI0, SGI36705, SGI36705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8304[] = {SGI36706, SGI36707, SGI36708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36709, SGI0, SGI0, SGI0, SGI36710, SGI36710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8305[] = {SGI36711, SGI0, SGI36712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8306[] = {SGI36713, SGI36714, SGI36715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36716, SGI0, SGI0, SGI0, SGI36717, SGI36717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8307[] = {SGI36718, SGI36719, SGI36720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36721, SGI0, SGI0, SGI0, SGI36722, SGI36722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8308[] = {SGI36723, SGI36724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36725, SGI0, SGI0, SGI0, SGI36726, SGI36726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8309[] = {SGI36727, SGI36728, SGI36729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36730, SGI0, SGI0, SGI0, SGI36731, SGI36731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8310[] = {SGI36732, SGI36733, SGI36734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36735, SGI0, SGI0, SGI0, SGI36736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8311[] = {SGI36737, SGI36738, SGI36739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36740, SGI36740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8312[] = {SGI36741, SGI36742, SGI36743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36744, SGI0, SGI0, SGI0, SGI36745, SGI36745, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8313[] = {SGI36746, SGI0, SGI36747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36748, SGI0, SGI0, SGI0, SGI36749, SGI36749, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8314[] = {SGI36750, SGI0, SGI36751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8315[] = {SGI36752, SGI36753, SGI36754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36755, SGI0, SGI0, SGI0, SGI36756, SGI36757, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8316[] = {SGI36758, SGI36759, SGI36760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36761, SGI0, SGI0, SGI0, SGI36762, SGI36762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8317[] = {SGI36764, SGI36766, SGI36768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36770, SGI0, SGI0, SGI0, SGI36772, SGI36772, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8318[] = {SGI36763, SGI36765, SGI36767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36769, SGI0, SGI0, SGI0, SGI36771, SGI36771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8319[] = {SGI36773, SGI36774, SGI36775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36776, SGI0, SGI0, SGI0, SGI36777, SGI36777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8320[] = {SGI36782, SGI0, SGI36783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8321[] = {SGI36778, SGI36779, SGI36780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36781, SGI0, SGI0, SGI0, SGI36784, SGI36784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8322[] = {SGI36785, SGI36786, SGI36787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36788, SGI0, SGI0, SGI0, SGI36789, SGI36789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8323[] = {SGI36790, SGI36791, SGI36792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36793, SGI0, SGI0, SGI0, SGI36794, SGI36794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8324[] = {SGI36795, SGI36796, SGI36797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36798, SGI0, SGI0, SGI0, SGI36799, SGI36799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8325[] = {SGI36800, SGI36801, SGI36802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36803, SGI0, SGI0, SGI0, SGI36804, SGI36804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8326[] = {SGI36805, SGI36806, SGI36807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36808, SGI0, SGI0, SGI0, SGI36809, SGI36809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8327[] = {SGI36810, SGI36811, SGI36812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36813, SGI0, SGI0, SGI0, SGI36814, SGI36814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8328[] = {SGI36815, SGI36816, SGI36817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36818, SGI0, SGI0, SGI0, SGI36819, SGI36819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8329[] = {SGI36820, SGI36821, SGI36822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36823, SGI0, SGI0, SGI0, SGI36824, SGI36824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8330[] = {SGI36825, SGI36826, SGI36827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36828, SGI0, SGI0, SGI0, SGI36829, SGI36829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8331[] = {SGI36830, SGI36831, SGI36832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36833, SGI0, SGI0, SGI0, SGI36834, SGI36834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8332[] = {SGI36835, SGI36836, SGI36837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36838, SGI0, SGI0, SGI0, SGI36839, SGI36839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8333[] = {SGI36840, SGI36841, SGI36842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36843, SGI0, SGI0, SGI0, SGI36844, SGI36844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8334[] = {SGI36845, SGI36846, SGI36847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36848, SGI0, SGI0, SGI0, SGI36849, SGI36849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8335[] = {SGI36850, SGI36851, SGI36852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36853, SGI0, SGI0, SGI0, SGI36854, SGI36854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8336[] = {SGI36855, SGI36856, SGI36857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36858, SGI0, SGI0, SGI0, SGI36859, SGI36859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8337[] = {SGI36860, SGI36861, SGI36862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36863, SGI0, SGI0, SGI0, SGI36864, SGI36864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8338[] = {SGI36865, SGI0, SGI36866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8339[] = {SGI36867, SGI36868, SGI36869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36870, SGI0, SGI0, SGI0, SGI36871, SGI36871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8340[] = {SGI36872, SGI0, SGI36873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8341[] = {SGI36874, SGI0, SGI36875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8342[] = {SGI36876, SGI36877, SGI36878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36879, SGI0, SGI0, SGI0, SGI36880, SGI36880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8343[] = {SGI36881, SGI36882, SGI36883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36884, SGI0, SGI0, SGI0, SGI36885, SGI36885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8344[] = {SGI36886, SGI36887, SGI36888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36889, SGI0, SGI0, SGI0, SGI36890, SGI36890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8345[] = {SGI36891, SGI36892, SGI36893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36894, SGI0, SGI0, SGI0, SGI36895, SGI36895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8346[] = {SGI36896, SGI36897, SGI36898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36899, SGI0, SGI0, SGI0, SGI36900, SGI36900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8347[] = {SGI36901, SGI36902, SGI36903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36904, SGI0, SGI0, SGI0, SGI36905, SGI36905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8348[] = {SGI36906, SGI36907, SGI36908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36909, SGI0, SGI0, SGI0, SGI36910, SGI36910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8349[] = {SGI36911, SGI0, SGI36912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8350[] = {SGI36913, SGI36914, SGI36915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36916, SGI0, SGI0, SGI0, SGI36917, SGI36917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8351[] = {SGI36918, SGI36919, SGI36920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36921, SGI0, SGI0, SGI0, SGI36922, SGI36922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8352[] = {SGI36923, SGI36924, SGI36925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36926, SGI0, SGI0, SGI0, SGI36927, SGI36927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8353[] = {SGI36928, SGI36929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36930, SGI0, SGI0, SGI0, SGI36931, SGI36931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8354[] = {SGI36932, SGI36933, SGI36934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36935, SGI0, SGI0, SGI0, SGI36936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8355[] = {SGI36937, SGI36938, SGI36939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36940, SGI36941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8356[] = {SGI36942, SGI36943, SGI36944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36945, SGI36945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8357[] = {SGI36946, SGI36947, SGI36948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36949, SGI0, SGI0, SGI0, SGI36950, SGI36950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8358[] = {SGI36951, SGI36952, SGI36953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36954, SGI0, SGI0, SGI0, SGI36955, SGI36955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8359[] = {SGI36956, SGI36957, SGI36958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36959, SGI0, SGI0, SGI0, SGI36960, SGI36960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8360[] = {SGI36961, SGI0, SGI36962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8361[] = {SGI36963, SGI36964, SGI36965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36966, SGI0, SGI0, SGI0, SGI36967, SGI36967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8362[] = {SGI36968, SGI36969, SGI36970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36971, SGI36972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8363[] = {SGI36973, SGI36974, SGI36975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36976, SGI0, SGI0, SGI0, SGI36977, SGI36977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8364[] = {SGI36978, SGI0, SGI36979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36980, SGI0, SGI0, SGI0, SGI36981, SGI36981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8365[] = {SGI36982, SGI36983, SGI36984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36985, SGI36985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8366[] = {SGI36986, SGI36987, SGI36988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36989, SGI0, SGI0, SGI0, SGI36990, SGI36990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8367[] = {SGI0, SGI36991, SGI36992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8368[] = {SGI36993, SGI36994, SGI36995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI36996, SGI0, SGI0, SGI0, SGI36997, SGI36997, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8369[] = {SGI36998, SGI36999, SGI37000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37001, SGI0, SGI0, SGI0, SGI37002, SGI37002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8370[] = {SGI37003, SGI37004, SGI37005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37006, SGI0, SGI0, SGI0, SGI37007, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8371[] = {SGI37008, SGI37009, SGI37010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37011, SGI0, SGI0, SGI0, SGI37012, SGI37012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8372[] = {SGI37013, SGI37014, SGI37015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37016, SGI0, SGI0, SGI0, SGI37017, SGI37017, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8373[] = {SGI37018, SGI37019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37020, SGI0, SGI0, SGI0, SGI37021, SGI37021, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8374[] = {SGI37022, SGI37023, SGI37024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37025, SGI0, SGI0, SGI0, SGI37026, SGI37026, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8375[] = {SGI37027, SGI37028, SGI37029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37030, SGI0, SGI0, SGI0, SGI37031, SGI37031, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8376[] = {SGI37032, SGI0, SGI37033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8377[] = {SGI37034, SGI37035, SGI37036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37037, SGI0, SGI0, SGI0, SGI37038, SGI37038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8378[] = {SGI37039, SGI37040, SGI37041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37042, SGI0, SGI0, SGI0, SGI37043, SGI37043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8379[] = {SGI37044, SGI0, SGI37045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8380[] = {SGI37046, SGI0, SGI37047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8381[] = {SGI37048, SGI0, SGI37049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37050, SGI0, SGI0, SGI0, SGI37051, SGI37051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8382[] = {SGI37052, SGI37053, SGI37054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37055, SGI37055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8383[] = {SGI37056, SGI0, SGI37057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8384[] = {SGI37058, SGI37059, SGI37060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37061, SGI0, SGI0, SGI0, SGI37062, SGI37062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8385[] = {SGI37063, SGI37064, SGI37065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37066, SGI0, SGI0, SGI0, SGI37067, SGI37067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8386[] = {SGI37068, SGI0, SGI37069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37070, SGI0, SGI0, SGI0, SGI37071, SGI37071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8387[] = {SGI37072, SGI37073, SGI37074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37075, SGI0, SGI0, SGI0, SGI37076, SGI37076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8388[] = {SGI37077, SGI37078, SGI37079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37080, SGI0, SGI0, SGI0, SGI37081, SGI37081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8389[] = {SGI37082, SGI37083, SGI37084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37085, SGI0, SGI0, SGI0, SGI37086, SGI37086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8390[] = {SGI37087, SGI37088, SGI37089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37090, SGI0, SGI0, SGI0, SGI37091, SGI37091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8391[] = {SGI37092, SGI37093, SGI37094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37095, SGI0, SGI0, SGI0, SGI37096, SGI37096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8392[] = {SGI37097, SGI0, SGI37098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8393[] = {SGI37099, SGI37100, SGI37101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37102, SGI0, SGI0, SGI0, SGI37103, SGI37103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8394[] = {SGI37104, SGI37105, SGI37106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37107, SGI0, SGI0, SGI0, SGI37108, SGI37108, SGI0, SGI0, NULL};
const Segment * const SG8395[] = {SGI37109, SGI37110, SGI37111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37112, SGI0, SGI0, SGI0, SGI37113, SGI37113, SGI0, SGI0, NULL};
const Segment * const SG8396[] = {SGI37114, SGI37115, SGI37116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37117, SGI0, SGI0, SGI0, SGI37118, SGI37118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8397[] = {SGI37119, SGI37120, SGI37121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37122, SGI0, SGI0, SGI0, SGI37123, SGI37123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8398[] = {SGI37124, SGI0, SGI37125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8399[] = {SGI37126, SGI37127, SGI37128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37129, SGI0, SGI0, SGI0, SGI37130, SGI37130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8400[] = {SGI37131, SGI37132, SGI37133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37134, SGI0, SGI0, SGI0, SGI37135, SGI37135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8401[] = {SGI37136, SGI0, SGI37137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37138, SGI0, SGI0, SGI0, SGI37139, SGI37139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8402[] = {SGI37140, SGI37141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37142, SGI0, SGI0, SGI0, SGI37143, SGI37143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8403[] = {SGI37144, SGI37145, SGI37146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37147, SGI0, SGI0, SGI0, SGI37148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8404[] = {SGI37149, SGI37150, SGI37151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37152, SGI0, SGI0, SGI0, SGI37153, SGI37153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8405[] = {SGI37154, SGI37155, SGI37156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37157, SGI0, SGI0, SGI0, SGI37158, SGI37158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8406[] = {SGI37159, SGI37160, SGI37161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37162, SGI0, SGI0, SGI0, SGI37163, SGI37163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8407[] = {SGI37164, SGI37165, SGI37166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37167, SGI0, SGI0, SGI0, SGI37168, SGI37168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8408[] = {SGI37169, SGI37170, SGI37171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37172, SGI0, SGI0, SGI0, SGI37173, SGI37173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8409[] = {SGI37174, SGI37175, SGI37176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37177, SGI0, SGI0, SGI0, SGI37178, SGI37178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8410[] = {SGI37179, SGI0, SGI37180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8411[] = {SGI37181, SGI37182, SGI37183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37184, SGI0, SGI0, SGI0, SGI37185, SGI37185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8412[] = {SGI37186, SGI37187, SGI37188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37189, SGI0, SGI0, SGI0, SGI37190, SGI37190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8413[] = {SGI37191, SGI0, SGI37192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8414[] = {SGI37193, SGI37194, SGI37195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37196, SGI0, SGI0, SGI0, SGI37197, SGI37197, SGI0, SGI0, NULL};
const Segment * const SG8415[] = {SGI37198, SGI0, SGI37199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8416[] = {SGI37200, SGI37201, SGI37202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37203, SGI0, SGI0, SGI0, SGI37204, SGI37204, SGI0, SGI0, NULL};
const Segment * const SG8417[] = {SGI37205, SGI37206, SGI37207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37208, SGI0, SGI0, SGI0, SGI37209, SGI37209, SGI0, SGI0, NULL};
const Segment * const SG8418[] = {SGI37210, SGI37211, SGI37212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37213, SGI0, SGI0, SGI0, SGI37214, SGI37214, SGI0, SGI0, NULL};
const Segment * const SG8419[] = {SGI37218, SGI0, SGI37219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37221, SGI0, SGI0, SGI0, SGI37223, SGI37223, SGI0, SGI0, NULL};
const Segment * const SG8420[] = {SGI37215, SGI37216, SGI37217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37220, SGI0, SGI0, SGI0, SGI37222, SGI37222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8421[] = {SGI37224, SGI37225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37226, SGI0, SGI0, SGI0, SGI37227, SGI37227, SGI0, SGI0, NULL};
const Segment * const SG8422[] = {SGI37228, SGI37229, SGI37230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37231, SGI0, SGI0, SGI0, SGI37232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8423[] = {SGI37233, SGI37234, SGI37235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37236, SGI0, SGI0, SGI0, SGI37237, SGI37237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8424[] = {SGI37238, SGI37239, SGI37240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37241, SGI0, SGI0, SGI0, SGI37242, SGI37242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8425[] = {SGI37243, SGI37244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37245, SGI0, SGI0, SGI0, SGI37246, SGI37246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8426[] = {SGI37247, SGI37248, SGI37249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37250, SGI0, SGI0, SGI0, SGI37251, SGI37251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8427[] = {SGI37252, SGI37253, SGI37254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37255, SGI0, SGI0, SGI0, SGI37256, SGI37256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8428[] = {SGI37257, SGI37258, SGI37259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37260, SGI0, SGI0, SGI0, SGI37261, SGI37261, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8429[] = {SGI37262, SGI37263, SGI37264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37268, SGI0, SGI0, SGI0, SGI37269, SGI37269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8430[] = {SGI37265, SGI37266, SGI37267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37270, SGI0, SGI0, SGI0, SGI37271, SGI37271, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8431[] = {SGI37272, SGI0, SGI37273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8432[] = {SGI37274, SGI0, SGI37275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8433[] = {SGI37276, SGI37277, SGI37278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37281, SGI0, SGI0, SGI0, SGI37283, SGI37283, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8434[] = {SGI37279, SGI37280, SGI37282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8435[] = {SGI37284, SGI37285, SGI37286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37287, SGI0, SGI0, SGI0, SGI37288, SGI37288, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8436[] = {SGI37290, SGI37293, SGI37294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37296, SGI0, SGI0, SGI0, SGI37299, SGI37299, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8437[] = {SGI37289, SGI37291, SGI37292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37295, SGI0, SGI0, SGI0, SGI37297, SGI37298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8438[] = {SGI37300, SGI0, SGI37301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37302, SGI0, SGI0, SGI0, SGI37303, SGI37303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8439[] = {SGI37304, SGI37305, SGI37306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37307, SGI0, SGI0, SGI0, SGI37308, SGI37308, SGI0, SGI0, NULL};
const Segment * const SG8440[] = {SGI37309, SGI37310, SGI37311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37312, SGI0, SGI0, SGI0, SGI37313, SGI37313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8441[] = {SGI37314, SGI37315, SGI37316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37317, SGI0, SGI0, SGI0, SGI37318, SGI37318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8442[] = {SGI37319, SGI37320, SGI37321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37322, SGI0, SGI0, SGI0, SGI37323, SGI37323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8443[] = {SGI37324, SGI37325, SGI37326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37327, SGI0, SGI0, SGI0, SGI37328, SGI37328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8444[] = {SGI37329, SGI37330, SGI37331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37332, SGI0, SGI0, SGI0, SGI37333, SGI37333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8445[] = {SGI37334, SGI0, SGI37335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8446[] = {SGI37336, SGI37337, SGI37338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37339, SGI0, SGI0, SGI0, SGI37340, SGI37340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8447[] = {SGI37341, SGI37342, SGI37343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37344, SGI0, SGI0, SGI0, SGI37345, SGI37345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8448[] = {SGI37346, SGI37347, SGI37348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37349, SGI0, SGI0, SGI0, SGI37350, SGI37350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8449[] = {SGI37351, SGI0, SGI37352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8450[] = {SGI37353, SGI37354, SGI37355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37356, SGI0, SGI0, SGI0, SGI37357, SGI37357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8451[] = {SGI37358, SGI37359, SGI37360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37361, SGI0, SGI0, SGI0, SGI37362, SGI37362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8452[] = {SGI37363, SGI37364, SGI37365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37366, SGI0, SGI0, SGI0, SGI37367, SGI37367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8453[] = {SGI37368, SGI37369, SGI37370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37371, SGI37372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8454[] = {SGI37373, SGI37374, SGI37375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37376, SGI0, SGI0, SGI0, SGI37377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8455[] = {SGI37378, SGI37379, SGI37380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37381, SGI0, SGI0, SGI0, SGI37382, SGI37382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8456[] = {SGI37383, SGI37384, SGI37385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37386, SGI0, SGI0, SGI0, SGI37387, SGI37387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8457[] = {SGI37388, SGI37389, SGI37390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37391, SGI0, SGI0, SGI0, SGI37396, SGI37396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8458[] = {SGI37392, SGI0, SGI37394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8459[] = {SGI37393, SGI37395, SGI37397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37398, SGI0, SGI0, SGI0, SGI37399, SGI37399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8460[] = {SGI37400, SGI37401, SGI37402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37403, SGI0, SGI0, SGI0, SGI37404, SGI37404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8461[] = {SGI37405, SGI37406, SGI37407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37408, SGI0, SGI0, SGI0, SGI37409, SGI37409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8462[] = {SGI37410, SGI37411, SGI37412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37413, SGI0, SGI0, SGI0, SGI37414, SGI37414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8463[] = {SGI37415, SGI0, SGI37416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8464[] = {SGI37417, SGI37418, SGI37419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8465[] = {SGI37421, SGI0, SGI37423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8466[] = {SGI37420, SGI37422, SGI37424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37425, SGI0, SGI0, SGI0, SGI37426, SGI37427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8467[] = {SGI37428, SGI0, SGI37429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37432, SGI0, SGI0, SGI0, SGI37434, SGI37434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8468[] = {SGI37430, SGI37431, SGI37433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37435, SGI0, SGI0, SGI0, SGI37436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8469[] = {SGI37437, SGI37438, SGI37439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37440, SGI0, SGI0, SGI0, SGI37441, SGI37441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8470[] = {SGI37442, SGI37443, SGI37444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37445, SGI0, SGI0, SGI0, SGI37446, SGI37446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8471[] = {SGI37447, SGI37448, SGI37449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37450, SGI0, SGI0, SGI0, SGI37451, SGI37451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8472[] = {SGI37452, SGI37453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37454, SGI0, SGI0, SGI0, SGI37455, SGI37455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8473[] = {SGI37456, SGI37457, SGI37458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37459, SGI0, SGI0, SGI0, SGI37460, SGI37460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8474[] = {SGI37461, SGI0, SGI37462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37463, SGI0, SGI0, SGI0, SGI37464, SGI37464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8475[] = {SGI37465, SGI37466, SGI37467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37468, SGI0, SGI0, SGI0, SGI37469, SGI37470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8476[] = {SGI37471, SGI37472, SGI37473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37474, SGI0, SGI0, SGI0, SGI37475, SGI37475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8477[] = {SGI37476, SGI0, SGI37477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37478, SGI0, SGI0, SGI0, SGI37479, SGI37479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8478[] = {SGI37480, SGI37481, SGI37482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37483, SGI37484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8479[] = {SGI37485, SGI37486, SGI37487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37488, SGI37488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8480[] = {SGI37489, SGI37490, SGI37491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37492, SGI37492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8481[] = {SGI37493, SGI37494, SGI37495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37496, SGI0, SGI0, SGI0, SGI37497, SGI37497, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8482[] = {SGI37498, SGI37499, SGI37500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37501, SGI0, SGI0, SGI0, SGI37502, SGI37502, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8483[] = {SGI37503, SGI37504, SGI37505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37506, SGI0, SGI0, SGI0, SGI37507, SGI37507, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8484[] = {SGI37508, SGI37509, SGI37510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37511, SGI0, SGI0, SGI0, SGI37512, SGI37512, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8485[] = {SGI37513, SGI37514, SGI37515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37516, SGI0, SGI0, SGI0, SGI37517, SGI37517, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8486[] = {SGI37518, SGI37519, SGI37520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37521, SGI0, SGI0, SGI0, SGI37522, SGI37522, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8487[] = {SGI37523, SGI37524, SGI37525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37526, SGI0, SGI0, SGI0, SGI37527, SGI37527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8488[] = {SGI37528, SGI37529, SGI37530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37531, SGI0, SGI0, SGI0, SGI37532, SGI37532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8489[] = {SGI37533, SGI37534, SGI37535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37536, SGI0, SGI0, SGI0, SGI37537, SGI37537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8490[] = {SGI37543, SGI37544, SGI37545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37546, SGI0, SGI0, SGI0, SGI37547, SGI37547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8491[] = {SGI37548, SGI37549, SGI37550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37551, SGI0, SGI0, SGI0, SGI37552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8492[] = {SGI37553, SGI37554, SGI37555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37556, SGI0, SGI0, SGI0, SGI37557, SGI37557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8493[] = {SGI37558, SGI37559, SGI37560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37561, SGI0, SGI0, SGI0, SGI37562, SGI37562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8494[] = {SGI37563, SGI37564, SGI37565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37566, SGI0, SGI0, SGI0, SGI37567, SGI37567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8495[] = {SGI37568, SGI37569, SGI37570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37571, SGI0, SGI0, SGI0, SGI37572, SGI37572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8496[] = {SGI37573, SGI37574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37575, SGI0, SGI0, SGI0, SGI37576, SGI37576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8497[] = {SGI37577, SGI0, SGI37578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8498[] = {SGI37579, SGI37580, SGI37581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37582, SGI0, SGI0, SGI0, SGI37583, SGI37583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8499[] = {SGI37584, SGI37585, SGI37586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37587, SGI0, SGI0, SGI0, SGI37588, SGI37588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8500[] = {SGI37538, SGI37539, SGI37540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37541, SGI0, SGI0, SGI0, SGI37542, SGI37542, SGI0, SGI0, NULL};
const Segment * const SG8501[] = {SGI37589, SGI37590, SGI37591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37592, SGI0, SGI0, SGI0, SGI37593, SGI37593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8502[] = {SGI37594, SGI37595, SGI37596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37597, SGI0, SGI0, SGI0, SGI37598, SGI37598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8503[] = {SGI37599, SGI37600, SGI37601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37602, SGI0, SGI0, SGI0, SGI37603, SGI37603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8504[] = {SGI37604, SGI37605, SGI37606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37607, SGI0, SGI0, SGI0, SGI37608, SGI37608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8505[] = {SGI37609, SGI37610, SGI37611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37612, SGI0, SGI0, SGI0, SGI37613, SGI37613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8506[] = {SGI37614, SGI37615, SGI37616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37617, SGI0, SGI0, SGI0, SGI37618, SGI37618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8507[] = {SGI37619, SGI37620, SGI37621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37622, SGI0, SGI0, SGI0, SGI37623, SGI37623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8508[] = {SGI37624, SGI37625, SGI37626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37627, SGI0, SGI0, SGI0, SGI37628, SGI37628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8509[] = {SGI37629, SGI37630, SGI37631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37632, SGI0, SGI0, SGI0, SGI37633, SGI37633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8510[] = {SGI37634, SGI37635, SGI37636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37637, SGI0, SGI0, SGI0, SGI37638, SGI37638, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8511[] = {SGI37639, SGI37640, SGI37641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37642, SGI0, SGI0, SGI0, SGI37643, SGI37643, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8512[] = {SGI37644, SGI37645, SGI37646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37647, SGI0, SGI0, SGI0, SGI37648, SGI37648, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8513[] = {SGI37649, SGI0, SGI37650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8514[] = {SGI0, SGI37651, SGI37652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37653, SGI0, SGI0, SGI0, SGI37654, SGI37654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8515[] = {SGI0, SGI37655, SGI37656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37657, SGI0, SGI0, SGI0, SGI37658, SGI37658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8516[] = {SGI0, SGI37659, SGI37660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37661, SGI0, SGI0, SGI0, SGI37662, SGI37662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8517[] = {SGI0, SGI37663, SGI37664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37665, SGI0, SGI0, SGI0, SGI37666, SGI37666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8518[] = {SGI0, SGI37667, SGI37668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37669, SGI0, SGI0, SGI0, SGI37670, SGI37670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8519[] = {SGI0, SGI37671, SGI37672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37673, SGI0, SGI0, SGI0, SGI37674, SGI37674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8520[] = {SGI37675, SGI37676, SGI37677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37678, SGI0, SGI0, SGI0, SGI37679, SGI37679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8521[] = {SGI37680, SGI37681, SGI37682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37683, SGI0, SGI0, SGI0, SGI37684, SGI37684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8522[] = {SGI37685, SGI37686, SGI37687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37688, SGI0, SGI0, SGI0, SGI37689, SGI37689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8523[] = {SGI37690, SGI37691, SGI37692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37693, SGI0, SGI0, SGI0, SGI37694, SGI37694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8524[] = {SGI37695, SGI0, SGI37696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8525[] = {SGI37697, SGI37698, SGI37699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37700, SGI0, SGI0, SGI0, SGI37701, SGI37701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8526[] = {SGI37702, SGI37703, SGI37704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37705, SGI0, SGI0, SGI0, SGI37706, SGI37706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8527[] = {SGI37707, SGI37708, SGI37709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37710, SGI0, SGI0, SGI0, SGI37711, SGI37711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8528[] = {SGI37712, SGI37713, SGI37714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37715, SGI0, SGI0, SGI0, SGI37716, SGI37716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8529[] = {SGI37717, SGI0, SGI37718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8530[] = {SGI37719, SGI37720, SGI37721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37722, SGI0, SGI0, SGI0, SGI37723, SGI37723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8531[] = {SGI37724, SGI37725, SGI37726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37727, SGI0, SGI0, SGI0, SGI37728, SGI37728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8532[] = {SGI37729, SGI37730, SGI37731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37732, SGI0, SGI0, SGI0, SGI37733, SGI37733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8533[] = {SGI37734, SGI37735, SGI37736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37737, SGI0, SGI0, SGI0, SGI37738, SGI37738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8534[] = {SGI37739, SGI37740, SGI37741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37742, SGI0, SGI0, SGI0, SGI37743, SGI37743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8535[] = {SGI37744, SGI37745, SGI37746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37747, SGI0, SGI0, SGI0, SGI37748, SGI37748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8536[] = {SGI37749, SGI37750, SGI37751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37752, SGI0, SGI0, SGI0, SGI37753, SGI37753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8537[] = {SGI37754, SGI37755, SGI37756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37757, SGI0, SGI0, SGI0, SGI37758, SGI37758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8538[] = {SGI37759, SGI37760, SGI37761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37762, SGI0, SGI0, SGI0, SGI37763, SGI37763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8539[] = {SGI37764, SGI37765, SGI37766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37767, SGI0, SGI0, SGI0, SGI37768, SGI37768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8540[] = {SGI37769, SGI37770, SGI37771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37772, SGI0, SGI0, SGI0, SGI37773, SGI37773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8541[] = {SGI37774, SGI37775, SGI37776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37777, SGI0, SGI0, SGI0, SGI37778, SGI37778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8542[] = {SGI37779, SGI37780, SGI37781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37782, SGI0, SGI0, SGI0, SGI37783, SGI37783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8543[] = {SGI37784, SGI37785, SGI37786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37787, SGI0, SGI0, SGI0, SGI37788, SGI37788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8544[] = {SGI37789, SGI37790, SGI37791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37792, SGI0, SGI0, SGI0, SGI37793, SGI37793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8545[] = {SGI37794, SGI37795, SGI37796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37797, SGI0, SGI0, SGI0, SGI37798, SGI37798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8546[] = {SGI37799, SGI37800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37801, SGI0, SGI0, SGI0, SGI37802, SGI37802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8547[] = {SGI37803, SGI37804, SGI37805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37806, SGI0, SGI0, SGI0, SGI37807, SGI37807, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8548[] = {SGI37808, SGI37809, SGI37810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37811, SGI0, SGI0, SGI0, SGI37812, SGI37812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8549[] = {SGI37813, SGI0, SGI37814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8550[] = {SGI37815, SGI37816, SGI37817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37818, SGI0, SGI0, SGI0, SGI37819, SGI37819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8551[] = {SGI37820, SGI37821, SGI37822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37823, SGI0, SGI0, SGI0, SGI37824, SGI37824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8552[] = {SGI37825, SGI37826, SGI37827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37828, SGI0, SGI0, SGI0, SGI37829, SGI37829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8553[] = {SGI37830, SGI37831, SGI37832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37833, SGI0, SGI0, SGI0, SGI37834, SGI37834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8554[] = {SGI37835, SGI37836, SGI37837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37838, SGI0, SGI0, SGI0, SGI37839, SGI37839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8555[] = {SGI37840, SGI37841, SGI37842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37843, SGI0, SGI0, SGI0, SGI37844, SGI37844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8556[] = {SGI37845, SGI0, SGI37846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8557[] = {SGI37847, SGI0, SGI37848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8558[] = {SGI37849, SGI37850, SGI37851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37852, SGI0, SGI0, SGI0, SGI37853, SGI37853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8559[] = {SGI37854, SGI37855, SGI37856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37857, SGI0, SGI0, SGI0, SGI37858, SGI37858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8560[] = {SGI37859, SGI37860, SGI37861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37862, SGI0, SGI0, SGI0, SGI37863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8561[] = {SGI37864, SGI37865, SGI37866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37867, SGI0, SGI0, SGI0, SGI37868, SGI37868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8562[] = {SGI37869, SGI37870, SGI37871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37872, SGI0, SGI0, SGI0, SGI37873, SGI37873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8563[] = {SGI37874, SGI37875, SGI37876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37877, SGI0, SGI0, SGI0, SGI37878, SGI37878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8564[] = {SGI37879, SGI37880, SGI37881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37882, SGI0, SGI0, SGI0, SGI37883, SGI37883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8565[] = {SGI37884, SGI37885, SGI37886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37887, SGI0, SGI0, SGI0, SGI37888, SGI37888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8566[] = {SGI37889, SGI37890, SGI37891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37892, SGI0, SGI0, SGI0, SGI37893, SGI37893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8567[] = {SGI37894, SGI37895, SGI37896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37897, SGI0, SGI0, SGI0, SGI37898, SGI37898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8568[] = {SGI37899, SGI0, SGI37900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8569[] = {SGI37901, SGI37902, SGI37903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37904, SGI0, SGI0, SGI0, SGI37905, SGI37905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8570[] = {SGI37906, SGI37907, SGI37908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37909, SGI0, SGI0, SGI0, SGI37910, SGI37910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8571[] = {SGI37911, SGI37912, SGI37913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37914, SGI0, SGI0, SGI0, SGI37915, SGI37915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8572[] = {SGI37916, SGI37917, SGI37918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8573[] = {SGI37919, SGI0, SGI37920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8574[] = {SGI37921, SGI0, SGI37922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37923, SGI0, SGI0, SGI0, SGI37924, SGI37924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8575[] = {SGI37925, SGI37926, SGI37927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37928, SGI0, SGI0, SGI0, SGI37929, SGI37929, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8576[] = {SGI37930, SGI37931, SGI37932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37933, SGI0, SGI0, SGI0, SGI37934, SGI37934, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8577[] = {SGI37935, SGI0, SGI37936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8578[] = {SGI37937, SGI37938, SGI37939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37940, SGI0, SGI0, SGI0, SGI37941, SGI37941, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8579[] = {SGI37942, SGI37943, SGI37944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37945, SGI0, SGI0, SGI0, SGI37946, SGI37946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8580[] = {SGI37947, SGI37948, SGI37949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37950, SGI0, SGI0, SGI0, SGI37951, SGI37951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8581[] = {SGI37952, SGI37953, SGI37954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37955, SGI0, SGI0, SGI0, SGI37956, SGI37956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8582[] = {SGI37957, SGI37958, SGI37959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37960, SGI0, SGI0, SGI0, SGI37961, SGI37961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8583[] = {SGI37962, SGI37963, SGI37964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37965, SGI0, SGI0, SGI0, SGI37966, SGI37966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8584[] = {SGI37967, SGI37968, SGI37969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37970, SGI0, SGI0, SGI0, SGI37971, SGI37971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8585[] = {SGI37972, SGI37973, SGI37974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37975, SGI0, SGI0, SGI0, SGI37976, SGI37976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8586[] = {SGI37977, SGI0, SGI37978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8587[] = {SGI37979, SGI37980, SGI37981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37982, SGI0, SGI0, SGI0, SGI37983, SGI37983, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8588[] = {SGI37984, SGI0, SGI37985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8589[] = {SGI37986, SGI37987, SGI37988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37989, SGI0, SGI0, SGI0, SGI37990, SGI37990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8590[] = {SGI37991, SGI37992, SGI37993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI37994, SGI0, SGI0, SGI0, SGI37995, SGI37995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8591[] = {SGI37996, SGI0, SGI37997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8592[] = {SGI37998, SGI37999, SGI38000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38001, SGI0, SGI0, SGI0, SGI38002, SGI38002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8593[] = {SGI38003, SGI38004, SGI38005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38006, SGI0, SGI0, SGI0, SGI38007, SGI38007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8594[] = {SGI38008, SGI38009, SGI38010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38011, SGI0, SGI0, SGI0, SGI38012, SGI38012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8595[] = {SGI38013, SGI38014, SGI38015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38016, SGI0, SGI0, SGI0, SGI38017, SGI38017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8596[] = {SGI38018, SGI38019, SGI38020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38021, SGI0, SGI0, SGI0, SGI38022, SGI38022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8597[] = {SGI38023, SGI38024, SGI38025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38026, SGI0, SGI0, SGI0, SGI38027, SGI38027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8598[] = {SGI38028, SGI0, SGI38029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8599[] = {SGI38030, SGI38031, SGI38032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8600[] = {SGI38033, SGI38034, SGI38035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38036, SGI0, SGI0, SGI0, SGI38037, SGI38037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8601[] = {SGI38038, SGI38039, SGI38040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38041, SGI0, SGI0, SGI0, SGI38042, SGI38042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8602[] = {SGI38043, SGI38044, SGI38045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38046, SGI38046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8603[] = {SGI38047, SGI38048, SGI38049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38050, SGI0, SGI0, SGI0, SGI38051, SGI38051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8604[] = {SGI38052, SGI38053, SGI38054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38055, SGI0, SGI0, SGI0, SGI38056, SGI38056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8605[] = {SGI38057, SGI38058, SGI38059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38060, SGI0, SGI0, SGI0, SGI38061, SGI38061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8606[] = {SGI38062, SGI38063, SGI38064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38065, SGI0, SGI0, SGI0, SGI38066, SGI38066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8607[] = {SGI38067, SGI38068, SGI38069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38070, SGI0, SGI0, SGI0, SGI38074, SGI38074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8608[] = {SGI38071, SGI38072, SGI38073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38075, SGI0, SGI0, SGI0, SGI38076, SGI38076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8609[] = {SGI38077, SGI38078, SGI38079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38083, SGI0, SGI0, SGI0, SGI38084, SGI38084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8610[] = {SGI38080, SGI38081, SGI38082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38085, SGI0, SGI0, SGI0, SGI38086, SGI38087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8611[] = {SGI38088, SGI0, SGI38089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38090, SGI0, SGI0, SGI0, SGI38091, SGI38091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8612[] = {SGI38092, SGI0, SGI38093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8613[] = {SGI38094, SGI38095, SGI38096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38097, SGI0, SGI0, SGI0, SGI38098, SGI38098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8614[] = {SGI38099, SGI38100, SGI38101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38102, SGI0, SGI0, SGI0, SGI38103, SGI38103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8615[] = {SGI38104, SGI38105, SGI38106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38107, SGI38107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8616[] = {SGI38108, SGI0, SGI38109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8617[] = {SGI38110, SGI38111, SGI38112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38113, SGI0, SGI0, SGI0, SGI38114, SGI38114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8618[] = {SGI38115, SGI38116, SGI38118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38121, SGI0, SGI0, SGI0, SGI38123, SGI38123, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8619[] = {SGI38117, SGI38119, SGI38120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38122, SGI0, SGI0, SGI0, SGI38124, SGI38124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8620[] = {SGI38125, SGI38127, SGI38129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38132, SGI38133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8621[] = {SGI38126, SGI38128, SGI38130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38131, SGI0, SGI0, SGI0, SGI38134, SGI38134, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8622[] = {SGI38135, SGI38136, SGI38137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38138, SGI0, SGI0, SGI0, SGI38139, SGI38139, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8623[] = {SGI38140, SGI0, SGI38141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8624[] = {SGI38142, SGI38143, SGI38144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38145, SGI0, SGI0, SGI0, SGI38146, SGI38146, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8625[] = {SGI38147, SGI38148, SGI38149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38150, SGI0, SGI0, SGI0, SGI38151, SGI38151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8626[] = {SGI38152, SGI38153, SGI38154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38155, SGI0, SGI0, SGI0, SGI38156, SGI38156, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8627[] = {SGI38157, SGI38158, SGI38159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38160, SGI0, SGI0, SGI0, SGI38161, SGI38161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8628[] = {SGI38162, SGI38163, SGI38164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38165, SGI0, SGI0, SGI0, SGI38166, SGI38166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8629[] = {SGI38167, SGI38168, SGI38169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38170, SGI0, SGI0, SGI0, SGI38171, SGI38171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8630[] = {SGI38172, SGI38173, SGI38174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38175, SGI0, SGI0, SGI0, SGI38176, SGI38176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8631[] = {SGI38177, SGI0, SGI38178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8632[] = {SGI38179, SGI38180, SGI38181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38182, SGI38183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8633[] = {SGI38184, SGI38185, SGI38186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38187, SGI38187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8634[] = {SGI38188, SGI0, SGI38189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8635[] = {SGI38190, SGI38191, SGI38192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38193, SGI0, SGI0, SGI0, SGI38194, SGI38194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8636[] = {SGI38195, SGI38196, SGI38197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38198, SGI38198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8637[] = {SGI38199, SGI38200, SGI38201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38202, SGI0, SGI0, SGI0, SGI38203, SGI38203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8638[] = {SGI38204, SGI38205, SGI38206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38207, SGI0, SGI0, SGI0, SGI38208, SGI38208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8639[] = {SGI38209, SGI38210, SGI38211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38212, SGI0, SGI0, SGI0, SGI38213, SGI38213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8640[] = {SGI38214, SGI38215, SGI38216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38217, SGI0, SGI0, SGI0, SGI38218, SGI38218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8641[] = {SGI38219, SGI38220, SGI38221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38222, SGI0, SGI0, SGI0, SGI38223, SGI38223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8642[] = {SGI38224, SGI38225, SGI38226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38227, SGI0, SGI0, SGI0, SGI38228, SGI38228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8643[] = {SGI38229, SGI38230, SGI38231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38232, SGI0, SGI0, SGI0, SGI38233, SGI38233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8644[] = {SGI38234, SGI0, SGI38235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8645[] = {SGI38236, SGI38237, SGI38238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38239, SGI0, SGI0, SGI0, SGI38240, SGI38240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8646[] = {SGI38241, SGI0, SGI38242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8647[] = {SGI38243, SGI38244, SGI38245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38246, SGI0, SGI0, SGI0, SGI38247, SGI38247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8648[] = {SGI38248, SGI38249, SGI38250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38251, SGI0, SGI0, SGI0, SGI38252, SGI38252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8649[] = {SGI38253, SGI38254, SGI38255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38257, SGI0, SGI0, SGI0, SGI38259, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8650[] = {SGI38256, SGI0, SGI38258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8651[] = {SGI38260, SGI0, SGI38261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8652[] = {SGI38262, SGI38263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38264, SGI0, SGI0, SGI0, SGI38265, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8653[] = {SGI38266, SGI38267, SGI38268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38272, SGI0, SGI0, SGI0, SGI38273, SGI38274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8654[] = {SGI38269, SGI38270, SGI38271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38275, SGI0, SGI0, SGI0, SGI38276, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8655[] = {SGI38277, SGI38278, SGI38279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38282, SGI0, SGI0, SGI0, SGI38284, SGI38284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8656[] = {SGI38280, SGI38281, SGI38283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38285, SGI0, SGI0, SGI0, SGI38286, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8657[] = {SGI38287, SGI0, SGI38288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8658[] = {SGI38289, SGI38290, SGI38291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38292, SGI0, SGI0, SGI0, SGI38293, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8659[] = {SGI38294, SGI38295, SGI38296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38297, SGI0, SGI0, SGI0, SGI38298, SGI38298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8660[] = {SGI38299, SGI38300, SGI38301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38302, SGI0, SGI0, SGI0, SGI38303, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8661[] = {SGI38304, SGI38305, SGI38306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38307, SGI0, SGI0, SGI0, SGI38308, SGI38308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8662[] = {SGI38309, SGI0, SGI38310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38311, SGI0, SGI0, SGI0, SGI38312, SGI38312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8663[] = {SGI38313, SGI38314, SGI38315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38316, SGI0, SGI0, SGI0, SGI38317, SGI38317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8664[] = {SGI38318, SGI38319, SGI38320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38321, SGI0, SGI0, SGI0, SGI38322, SGI38322, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8665[] = {SGI38323, SGI0, SGI38324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8666[] = {SGI38325, SGI38326, SGI38327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38328, SGI0, SGI0, SGI0, SGI38329, SGI38329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8667[] = {SGI38330, SGI38331, SGI38332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38333, SGI0, SGI0, SGI0, SGI38334, SGI38334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8668[] = {SGI38335, SGI38336, SGI38337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38338, SGI0, SGI0, SGI0, SGI38339, SGI38339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8669[] = {SGI38340, SGI38341, SGI38342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38343, SGI0, SGI0, SGI0, SGI38344, SGI38344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8670[] = {SGI38345, SGI38346, SGI38347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38348, SGI0, SGI0, SGI0, SGI38349, SGI38349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8671[] = {SGI38350, SGI0, SGI38351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8672[] = {SGI38352, SGI38353, SGI38354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38355, SGI0, SGI0, SGI0, SGI38356, SGI38356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8673[] = {SGI38357, SGI0, SGI38358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8674[] = {SGI38359, SGI38360, SGI38361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38362, SGI0, SGI0, SGI0, SGI38363, SGI38363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8675[] = {SGI38364, SGI38365, SGI38366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38367, SGI38367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8676[] = {SGI38368, SGI38369, SGI38370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38371, SGI0, SGI0, SGI0, SGI38374, SGI38374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8677[] = {SGI38372, SGI38373, SGI38375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38376, SGI0, SGI0, SGI0, SGI38377, SGI38377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8678[] = {SGI38378, SGI38379, SGI38380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38381, SGI0, SGI0, SGI0, SGI38382, SGI38382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8679[] = {SGI38383, SGI38384, SGI38385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38386, SGI0, SGI0, SGI0, SGI38387, SGI38387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8680[] = {SGI38388, SGI38389, SGI38390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38391, SGI0, SGI0, SGI0, SGI38392, SGI38392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8681[] = {SGI38393, SGI38394, SGI38395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38396, SGI0, SGI0, SGI0, SGI38397, SGI38397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8682[] = {SGI38398, SGI38399, SGI38400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38401, SGI0, SGI0, SGI0, SGI38402, SGI38402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8683[] = {SGI38403, SGI38404, SGI38405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38406, SGI0, SGI0, SGI0, SGI38407, SGI38407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8684[] = {SGI38408, SGI38409, SGI38410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38411, SGI0, SGI0, SGI0, SGI38412, SGI38412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8685[] = {SGI38413, SGI38414, SGI38415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38416, SGI0, SGI0, SGI0, SGI38417, SGI38417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8686[] = {SGI38418, SGI38419, SGI38420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38421, SGI0, SGI0, SGI0, SGI38422, SGI38422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8687[] = {SGI38423, SGI38424, SGI38425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38426, SGI0, SGI0, SGI0, SGI38427, SGI38427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8688[] = {SGI38428, SGI38429, SGI38430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38431, SGI0, SGI0, SGI0, SGI38432, SGI38432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8689[] = {SGI38433, SGI38434, SGI38435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38436, SGI0, SGI0, SGI0, SGI38437, SGI38437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8690[] = {SGI38438, SGI38439, SGI38440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38441, SGI0, SGI0, SGI0, SGI38442, SGI38442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8691[] = {SGI38443, SGI38444, SGI38445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38446, SGI0, SGI0, SGI0, SGI38447, SGI38447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8692[] = {SGI38448, SGI38449, SGI38450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38451, SGI0, SGI0, SGI0, SGI38452, SGI38452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8693[] = {SGI38453, SGI38454, SGI38455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38456, SGI0, SGI0, SGI0, SGI38457, SGI38457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8694[] = {SGI38458, SGI38459, SGI38460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38461, SGI0, SGI0, SGI0, SGI38462, SGI38462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8695[] = {SGI38463, SGI38464, SGI38465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38466, SGI0, SGI0, SGI0, SGI38467, SGI38467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8696[] = {SGI38468, SGI38469, SGI38470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38471, SGI0, SGI0, SGI0, SGI38472, SGI38472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8697[] = {SGI38473, SGI38474, SGI38475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38476, SGI0, SGI0, SGI0, SGI38477, SGI38477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8698[] = {SGI38478, SGI38479, SGI38480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38481, SGI0, SGI0, SGI0, SGI38482, SGI38482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8699[] = {SGI38483, SGI38484, SGI38485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38486, SGI0, SGI0, SGI0, SGI38487, SGI38487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8700[] = {SGI38488, SGI38489, SGI38490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38491, SGI0, SGI0, SGI0, SGI38494, SGI38494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8701[] = {SGI38492, SGI38493, SGI38495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38496, SGI0, SGI0, SGI0, SGI38497, SGI38497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8702[] = {SGI38498, SGI0, SGI38499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8703[] = {SGI38500, SGI38501, SGI38502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38503, SGI0, SGI0, SGI0, SGI38504, SGI38504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8704[] = {SGI38505, SGI38506, SGI38507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38508, SGI0, SGI0, SGI0, SGI38509, SGI38509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8705[] = {SGI38510, SGI38511, SGI38512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38513, SGI0, SGI0, SGI0, SGI38514, SGI38514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8706[] = {SGI38515, SGI38516, SGI38517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38518, SGI0, SGI0, SGI0, SGI38519, SGI38519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8707[] = {SGI38520, SGI0, SGI38521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8708[] = {SGI38522, SGI38523, SGI38524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38525, SGI0, SGI0, SGI0, SGI38526, SGI38526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8709[] = {SGI38527, SGI38528, SGI38529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38530, SGI0, SGI0, SGI0, SGI38531, SGI38531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8710[] = {SGI38532, SGI38533, SGI38534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38535, SGI38536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8711[] = {SGI38537, SGI0, SGI38538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8712[] = {SGI38539, SGI38540, SGI38541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38542, SGI0, SGI0, SGI0, SGI38543, SGI38543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8713[] = {SGI38544, SGI38545, SGI38546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38547, SGI0, SGI0, SGI0, SGI38548, SGI38548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8714[] = {SGI38549, SGI38550, SGI38551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38552, SGI0, SGI0, SGI0, SGI38553, SGI38553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8715[] = {SGI38554, SGI0, SGI38555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8716[] = {SGI38556, SGI38557, SGI38558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38559, SGI0, SGI0, SGI0, SGI38560, SGI38560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8717[] = {SGI38561, SGI38562, SGI38563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38564, SGI0, SGI0, SGI0, SGI38565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8718[] = {SGI38566, SGI38567, SGI38568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38569, SGI0, SGI0, SGI0, SGI38570, SGI38570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8719[] = {SGI38571, SGI38572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38573, SGI0, SGI0, SGI0, SGI38574, SGI38574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8720[] = {SGI38575, SGI38576, SGI38577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38578, SGI0, SGI0, SGI0, SGI38579, SGI38579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8721[] = {SGI38580, SGI38581, SGI38582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38583, SGI0, SGI0, SGI0, SGI38584, SGI38584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8722[] = {SGI38585, SGI38586, SGI38587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38588, SGI0, SGI0, SGI0, SGI38589, SGI38589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8723[] = {SGI38590, SGI38591, SGI38592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38593, SGI0, SGI0, SGI0, SGI38594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8724[] = {SGI38595, SGI38596, SGI38597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38598, SGI0, SGI0, SGI0, SGI38599, SGI38599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8725[] = {SGI38600, SGI38601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38602, SGI0, SGI0, SGI0, SGI38603, SGI38603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8726[] = {SGI38604, SGI38605, SGI38606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38607, SGI0, SGI0, SGI0, SGI38608, SGI38608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8727[] = {SGI38609, SGI38610, SGI38611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38612, SGI0, SGI0, SGI0, SGI38613, SGI38613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8728[] = {SGI38614, SGI0, SGI38615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8729[] = {SGI38616, SGI38617, SGI38618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8730[] = {SGI38619, SGI38620, SGI38621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38622, SGI0, SGI0, SGI0, SGI38623, SGI38624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8731[] = {SGI38625, SGI38626, SGI38627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38628, SGI0, SGI0, SGI0, SGI38629, SGI38629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8732[] = {SGI38630, SGI38631, SGI38632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38633, SGI0, SGI0, SGI0, SGI38634, SGI38634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8733[] = {SGI38635, SGI38636, SGI38637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38638, SGI0, SGI0, SGI0, SGI38639, SGI38639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8734[] = {SGI38640, SGI0, SGI38641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8735[] = {SGI38642, SGI0, SGI38643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8736[] = {SGI38644, SGI38645, SGI38646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38647, SGI0, SGI0, SGI0, SGI38648, SGI38648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8737[] = {SGI38649, SGI38650, SGI38651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38652, SGI0, SGI0, SGI0, SGI38653, SGI38653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8738[] = {SGI38654, SGI38655, SGI38656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38657, SGI0, SGI0, SGI0, SGI38658, SGI38658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8739[] = {SGI38659, SGI38660, SGI38661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38662, SGI0, SGI0, SGI0, SGI38663, SGI38663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8740[] = {SGI38664, SGI38665, SGI38666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38667, SGI0, SGI0, SGI0, SGI38668, SGI38668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8741[] = {SGI38669, SGI38670, SGI38671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38672, SGI0, SGI0, SGI0, SGI38673, SGI38673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8742[] = {SGI38674, SGI38675, SGI38676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38677, SGI0, SGI0, SGI0, SGI38678, SGI38678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8743[] = {SGI38679, SGI38680, SGI38681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38682, SGI0, SGI0, SGI0, SGI38683, SGI38683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8744[] = {SGI38684, SGI38685, SGI38686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38687, SGI0, SGI0, SGI0, SGI38688, SGI38688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8745[] = {SGI38689, SGI38690, SGI38691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38692, SGI0, SGI0, SGI0, SGI38693, SGI38693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8746[] = {SGI38694, SGI38695, SGI38696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38697, SGI0, SGI0, SGI0, SGI38699, SGI38699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8747[] = {SGI38698, SGI38700, SGI38701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38702, SGI0, SGI0, SGI0, SGI38703, SGI38703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8748[] = {SGI38704, SGI38705, SGI38706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38707, SGI0, SGI0, SGI0, SGI38708, SGI38708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8749[] = {SGI38709, SGI38710, SGI38711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38712, SGI0, SGI0, SGI0, SGI38713, SGI38713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8750[] = {SGI38714, SGI38715, SGI38716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38717, SGI0, SGI0, SGI0, SGI38718, SGI38718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8751[] = {SGI38719, SGI38720, SGI38721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38722, SGI0, SGI0, SGI0, SGI38723, SGI38723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8752[] = {SGI38724, SGI38725, SGI38726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38727, SGI0, SGI0, SGI0, SGI38728, SGI38728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8753[] = {SGI38729, SGI38730, SGI38731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38732, SGI0, SGI0, SGI0, SGI38733, SGI38733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8754[] = {SGI38734, SGI38735, SGI38736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38737, SGI0, SGI0, SGI0, SGI38738, SGI38738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8755[] = {SGI38739, SGI38740, SGI38741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38742, SGI0, SGI0, SGI0, SGI38743, SGI38743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8756[] = {SGI38744, SGI0, SGI38745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8757[] = {SGI38746, SGI38747, SGI38748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38752, SGI0, SGI0, SGI0, SGI38754, SGI38754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8758[] = {SGI38749, SGI38750, SGI38751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38756, SGI0, SGI0, SGI0, SGI38758, SGI38758, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8759[] = {SGI38753, SGI38755, SGI38757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38759, SGI0, SGI0, SGI0, SGI38760, SGI38760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8760[] = {SGI38761, SGI38762, SGI38763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38764, SGI0, SGI0, SGI0, SGI38765, SGI38765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8761[] = {SGI38766, SGI38767, SGI38768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38769, SGI0, SGI0, SGI0, SGI38770, SGI38770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8762[] = {SGI38771, SGI38772, SGI38773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38774, SGI0, SGI0, SGI0, SGI38775, SGI38775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8763[] = {SGI38776, SGI38777, SGI38778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38779, SGI0, SGI0, SGI0, SGI38780, SGI38780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8764[] = {SGI38781, SGI38782, SGI38783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38784, SGI0, SGI0, SGI0, SGI38785, SGI38785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8765[] = {SGI38786, SGI0, SGI38787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8766[] = {SGI38788, SGI38789, SGI38790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38791, SGI0, SGI0, SGI0, SGI38792, SGI38792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8767[] = {SGI38793, SGI38794, SGI38795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38796, SGI0, SGI0, SGI0, SGI38797, SGI38797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8768[] = {SGI38798, SGI38799, SGI38800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38801, SGI0, SGI0, SGI0, SGI38803, SGI38803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8769[] = {SGI38802, SGI38804, SGI38805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38806, SGI0, SGI0, SGI0, SGI38807, SGI38807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8770[] = {SGI38808, SGI38809, SGI38810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38811, SGI0, SGI0, SGI0, SGI38812, SGI38812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8771[] = {SGI38813, SGI38814, SGI38815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38816, SGI0, SGI0, SGI0, SGI38818, SGI38818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8772[] = {SGI38817, SGI38819, SGI38820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38821, SGI0, SGI0, SGI0, SGI38822, SGI38822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8773[] = {SGI38823, SGI0, SGI38824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8774[] = {SGI38825, SGI38826, SGI38827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38828, SGI0, SGI0, SGI0, SGI38829, SGI38829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8775[] = {SGI38830, SGI38831, SGI38832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38833, SGI0, SGI0, SGI0, SGI38834, SGI38834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8776[] = {SGI38835, SGI38836, SGI38837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38838, SGI0, SGI0, SGI0, SGI38839, SGI38839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8777[] = {SGI38840, SGI38841, SGI38842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38843, SGI0, SGI0, SGI0, SGI38844, SGI38844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8778[] = {SGI38845, SGI38846, SGI38847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38848, SGI0, SGI0, SGI0, SGI38849, SGI38849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8779[] = {SGI38850, SGI38851, SGI38852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38853, SGI0, SGI0, SGI0, SGI38854, SGI38854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8780[] = {SGI38855, SGI38856, SGI38857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38858, SGI0, SGI0, SGI0, SGI38859, SGI38859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8781[] = {SGI38860, SGI0, SGI38861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8782[] = {SGI38862, SGI38863, SGI38864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38865, SGI0, SGI0, SGI0, SGI38866, SGI38866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8783[] = {SGI38867, SGI38868, SGI38869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38871, SGI0, SGI0, SGI0, SGI38874, SGI38874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8784[] = {SGI38870, SGI38872, SGI38873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38875, SGI0, SGI0, SGI0, SGI38876, SGI38876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8785[] = {SGI38877, SGI38878, SGI38879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38883, SGI0, SGI0, SGI0, SGI38885, SGI38885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8786[] = {SGI38880, SGI38881, SGI38882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38884, SGI0, SGI0, SGI0, SGI38886, SGI38886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8787[] = {SGI38887, SGI0, SGI38888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8788[] = {SGI38889, SGI38890, SGI38891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38892, SGI0, SGI0, SGI0, SGI38894, SGI38894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8789[] = {SGI38893, SGI38895, SGI38896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38897, SGI38897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8790[] = {SGI38898, SGI0, SGI38899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8791[] = {SGI38900, SGI0, SGI38901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38902, SGI0, SGI0, SGI0, SGI38903, SGI38903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8792[] = {SGI38904, SGI38905, SGI38906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38907, SGI0, SGI0, SGI0, SGI38908, SGI38908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8793[] = {SGI38909, SGI38910, SGI38911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38912, SGI0, SGI0, SGI0, SGI38913, SGI38913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8794[] = {SGI38914, SGI0, SGI38915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8795[] = {SGI38916, SGI38917, SGI38918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38919, SGI38920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8796[] = {SGI38921, SGI38922, SGI38923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38924, SGI38924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8797[] = {SGI38925, SGI38926, SGI38927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38928, SGI0, SGI0, SGI0, SGI38929, SGI38929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8798[] = {SGI38930, SGI38931, SGI38932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38933, SGI0, SGI0, SGI0, SGI38934, SGI38934, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8799[] = {SGI38935, SGI38936, SGI38937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38938, SGI0, SGI0, SGI0, SGI38939, SGI38939, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8800[] = {SGI38940, SGI0, SGI38941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8801[] = {SGI38942, SGI38943, SGI38944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38945, SGI0, SGI0, SGI0, SGI38946, SGI38946, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8802[] = {SGI38947, SGI38948, SGI38949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38950, SGI0, SGI0, SGI0, SGI38951, SGI38951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8803[] = {SGI38952, SGI38953, SGI38954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38955, SGI0, SGI0, SGI0, SGI38956, SGI38956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8804[] = {SGI38957, SGI38958, SGI38959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38960, SGI0, SGI0, SGI0, SGI38961, SGI38961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8805[] = {SGI38962, SGI0, SGI38963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8806[] = {SGI38964, SGI38965, SGI38966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38967, SGI0, SGI0, SGI0, SGI38968, SGI38968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8807[] = {SGI38969, SGI0, SGI38970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8808[] = {SGI38971, SGI38972, SGI38973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38974, SGI38974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8809[] = {SGI38975, SGI38976, SGI38977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38978, SGI0, SGI0, SGI0, SGI38979, SGI38979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8810[] = {SGI38980, SGI38981, SGI38982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38983, SGI0, SGI0, SGI0, SGI38984, SGI38984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8811[] = {SGI38985, SGI38986, SGI38987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38988, SGI0, SGI0, SGI0, SGI38989, SGI38989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8812[] = {SGI38990, SGI38991, SGI38992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38993, SGI0, SGI0, SGI0, SGI38994, SGI38994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8813[] = {SGI38995, SGI38996, SGI38997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI38998, SGI0, SGI0, SGI0, SGI38999, SGI38999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8814[] = {SGI39000, SGI39001, SGI39002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39003, SGI0, SGI0, SGI0, SGI39004, SGI39004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8815[] = {SGI39005, SGI39006, SGI39007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39008, SGI0, SGI0, SGI0, SGI39009, SGI39009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8816[] = {SGI39010, SGI0, SGI39011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39012, SGI0, SGI0, SGI0, SGI39013, SGI39013, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8817[] = {SGI39014, SGI39015, SGI39016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39017, SGI0, SGI0, SGI0, SGI39018, SGI39018, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8818[] = {SGI39019, SGI0, SGI39020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8819[] = {SGI39021, SGI39022, SGI39023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39024, SGI0, SGI0, SGI0, SGI39025, SGI39026, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8820[] = {SGI39027, SGI39028, SGI39029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39030, SGI0, SGI0, SGI0, SGI39031, SGI39031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8821[] = {SGI39032, SGI39033, SGI39034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39035, SGI0, SGI0, SGI0, SGI39036, SGI39036, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8822[] = {SGI39037, SGI39038, SGI39039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39040, SGI0, SGI0, SGI0, SGI39041, SGI39041, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8823[] = {SGI39042, SGI0, SGI39043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8824[] = {SGI39044, SGI0, SGI39045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8825[] = {SGI39046, SGI39047, SGI39048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39049, SGI0, SGI0, SGI0, SGI39050, SGI39050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8826[] = {SGI39051, SGI39052, SGI39053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39054, SGI0, SGI0, SGI0, SGI39055, SGI39055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8827[] = {SGI39056, SGI39057, SGI39058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39059, SGI0, SGI0, SGI0, SGI39060, SGI39060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8828[] = {SGI39061, SGI39062, SGI39063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39064, SGI0, SGI0, SGI0, SGI39065, SGI39065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8829[] = {SGI39066, SGI39067, SGI39068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39069, SGI0, SGI0, SGI0, SGI39070, SGI39070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8830[] = {SGI39071, SGI39072, SGI39073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39074, SGI0, SGI0, SGI0, SGI39075, SGI39075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8831[] = {SGI39076, SGI39077, SGI39078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39079, SGI0, SGI0, SGI0, SGI39080, SGI39080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8832[] = {SGI39081, SGI39082, SGI39083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39084, SGI0, SGI0, SGI0, SGI39085, SGI39085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8833[] = {SGI39086, SGI39087, SGI39088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39089, SGI0, SGI0, SGI0, SGI39090, SGI39090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8834[] = {SGI39091, SGI39092, SGI39093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39094, SGI0, SGI0, SGI0, SGI39095, SGI39095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8835[] = {SGI39096, SGI39097, SGI39098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39099, SGI0, SGI0, SGI0, SGI39100, SGI39100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8836[] = {SGI39101, SGI39102, SGI39103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39104, SGI0, SGI0, SGI0, SGI39105, SGI39105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8837[] = {SGI39106, SGI39107, SGI39108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39109, SGI0, SGI0, SGI0, SGI39110, SGI39110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8838[] = {SGI39111, SGI39112, SGI39113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39114, SGI0, SGI0, SGI0, SGI39115, SGI39115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8839[] = {SGI39116, SGI0, SGI39117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8840[] = {SGI39118, SGI39119, SGI39120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39121, SGI0, SGI0, SGI0, SGI39122, SGI39122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8841[] = {SGI39123, SGI39124, SGI39125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39126, SGI0, SGI0, SGI0, SGI39127, SGI39127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8842[] = {SGI39128, SGI0, SGI39129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8843[] = {SGI39130, SGI39131, SGI39132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39133, SGI0, SGI0, SGI0, SGI39134, SGI39134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8844[] = {SGI39135, SGI39136, SGI39137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39138, SGI0, SGI0, SGI0, SGI39139, SGI39139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8845[] = {SGI39140, SGI39141, SGI39142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39143, SGI0, SGI0, SGI0, SGI39144, SGI39144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8846[] = {SGI39145, SGI39146, SGI39147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39148, SGI0, SGI0, SGI0, SGI39149, SGI39149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8847[] = {SGI39150, SGI39151, SGI39152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39153, SGI0, SGI0, SGI0, SGI39154, SGI39154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8848[] = {SGI39155, SGI39156, SGI39157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39158, SGI0, SGI0, SGI0, SGI39159, SGI39159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8849[] = {SGI39160, SGI39161, SGI39162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39163, SGI0, SGI0, SGI0, SGI39164, SGI39164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8850[] = {SGI39165, SGI39166, SGI39167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39168, SGI0, SGI0, SGI0, SGI39170, SGI39170, SGI0, SGI0, NULL};
const Segment * const SG8851[] = {SGI39169, SGI39171, SGI39172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39173, SGI0, SGI0, SGI0, SGI39174, SGI39174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8852[] = {SGI39175, SGI39176, SGI39177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39178, SGI0, SGI0, SGI0, SGI39179, SGI39179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8853[] = {SGI39180, SGI39181, SGI39182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39183, SGI0, SGI0, SGI0, SGI39184, SGI39184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8854[] = {SGI39185, SGI39186, SGI39187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39188, SGI0, SGI0, SGI0, SGI39189, SGI39189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8855[] = {SGI39190, SGI39191, SGI39192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39193, SGI0, SGI0, SGI0, SGI39194, SGI39194, SGI0, SGI0, NULL};
const Segment * const SG8856[] = {SGI39195, SGI0, SGI39196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8857[] = {SGI39197, SGI39198, SGI39200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39203, SGI0, SGI0, SGI0, SGI39205, SGI39205, SGI0, SGI0, NULL};
const Segment * const SG8858[] = {SGI39199, SGI39201, SGI39202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39204, SGI0, SGI0, SGI0, SGI39206, SGI39206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8859[] = {SGI39207, SGI39208, SGI39209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39210, SGI0, SGI0, SGI0, SGI39211, SGI39211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8860[] = {SGI39215, SGI0, SGI39216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8861[] = {SGI39212, SGI39213, SGI39214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39217, SGI0, SGI0, SGI0, SGI39218, SGI39218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8862[] = {SGI39222, SGI0, SGI39223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8863[] = {SGI39219, SGI39220, SGI39221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39224, SGI0, SGI0, SGI0, SGI39225, SGI39225, SGI0, SGI0, NULL};
const Segment * const SG8864[] = {SGI39226, SGI39227, SGI39228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39229, SGI0, SGI0, SGI0, SGI39230, SGI39230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8865[] = {SGI39231, SGI39232, SGI39233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39234, SGI0, SGI0, SGI0, SGI39235, SGI39235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8866[] = {SGI39236, SGI0, SGI39237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8867[] = {SGI39238, SGI39239, SGI39240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39241, SGI0, SGI0, SGI0, SGI39242, SGI39242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8868[] = {SGI39243, SGI39244, SGI39245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39246, SGI0, SGI0, SGI0, SGI39247, SGI39247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8869[] = {SGI39248, SGI39249, SGI39250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39251, SGI0, SGI0, SGI0, SGI39252, SGI39252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8870[] = {SGI39253, SGI39254, SGI39255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39256, SGI0, SGI0, SGI0, SGI39257, SGI39257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8871[] = {SGI39258, SGI39259, SGI39260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39261, SGI0, SGI0, SGI0, SGI39262, SGI39262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8872[] = {SGI39263, SGI39264, SGI39265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39266, SGI0, SGI0, SGI0, SGI39267, SGI39267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8873[] = {SGI39268, SGI39269, SGI39270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39271, SGI0, SGI0, SGI0, SGI39272, SGI39272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8874[] = {SGI39273, SGI0, SGI39274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8875[] = {SGI39275, SGI39276, SGI39277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39278, SGI0, SGI0, SGI0, SGI39279, SGI39279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8876[] = {SGI39280, SGI39281, SGI39282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39285, SGI0, SGI0, SGI0, SGI39287, SGI39287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8877[] = {SGI39283, SGI39284, SGI39286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39288, SGI0, SGI0, SGI0, SGI39289, SGI39289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8878[] = {SGI39291, SGI39293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39295, SGI0, SGI0, SGI0, SGI39297, SGI39297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8879[] = {SGI39290, SGI39292, SGI39294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39296, SGI0, SGI0, SGI0, SGI39298, SGI39298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8880[] = {SGI39299, SGI39300, SGI39301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39302, SGI0, SGI0, SGI0, SGI39305, SGI39305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8881[] = {SGI39303, SGI39304, SGI39306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39307, SGI0, SGI0, SGI0, SGI39308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8882[] = {SGI39309, SGI0, SGI39310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8883[] = {SGI39311, SGI39312, SGI39313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39314, SGI0, SGI0, SGI0, SGI39315, SGI39315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8884[] = {SGI39316, SGI39317, SGI39318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39319, SGI0, SGI0, SGI0, SGI39320, SGI39320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8885[] = {SGI39321, SGI39322, SGI39323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39324, SGI0, SGI0, SGI0, SGI39325, SGI39325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8886[] = {SGI39326, SGI39327, SGI39328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39329, SGI0, SGI0, SGI0, SGI39331, SGI39331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8887[] = {SGI39330, SGI39332, SGI39333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39334, SGI0, SGI0, SGI0, SGI39335, SGI39335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8888[] = {SGI39336, SGI39337, SGI39338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39339, SGI0, SGI0, SGI0, SGI39340, SGI39340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8889[] = {SGI39341, SGI39342, SGI39343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39347, SGI0, SGI0, SGI0, SGI39349, SGI39349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8890[] = {SGI39344, SGI39345, SGI39346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39348, SGI0, SGI0, SGI0, SGI39350, SGI39350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8891[] = {SGI39352, SGI0, SGI39355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39356, SGI0, SGI0, SGI0, SGI39358, SGI39358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8892[] = {SGI39351, SGI39353, SGI39354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39357, SGI0, SGI0, SGI0, SGI39359, SGI39359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8893[] = {SGI39360, SGI0, SGI39361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8894[] = {SGI39362, SGI39363, SGI39364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39368, SGI0, SGI0, SGI0, SGI39370, SGI39370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8895[] = {SGI39365, SGI39366, SGI39367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39369, SGI0, SGI0, SGI0, SGI39371, SGI39372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8896[] = {SGI39374, SGI39376, SGI39378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39380, SGI39380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8897[] = {SGI39373, SGI39375, SGI39377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39379, SGI0, SGI0, SGI0, SGI39381, SGI39381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8898[] = {SGI39382, SGI39383, SGI39384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39385, SGI0, SGI0, SGI0, SGI39386, SGI39387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8899[] = {SGI39388, SGI39389, SGI39390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39391, SGI0, SGI0, SGI0, SGI39392, SGI39392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8900[] = {SGI39393, SGI39394, SGI39395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39396, SGI0, SGI0, SGI0, SGI39397, SGI39397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8901[] = {SGI39398, SGI39399, SGI39400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39401, SGI39401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8902[] = {SGI39402, SGI39403, SGI39404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39405, SGI0, SGI0, SGI0, SGI39406, SGI39406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8903[] = {SGI39407, SGI39408, SGI39409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39410, SGI0, SGI0, SGI0, SGI39411, SGI39411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8904[] = {SGI39412, SGI39413, SGI39414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39415, SGI0, SGI0, SGI0, SGI39416, SGI39416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8905[] = {SGI39417, SGI39418, SGI39419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39420, SGI0, SGI0, SGI0, SGI39421, SGI39421, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8906[] = {SGI39422, SGI0, SGI39423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8907[] = {SGI39424, SGI39425, SGI39426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39427, SGI0, SGI0, SGI0, SGI39428, SGI39428, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8908[] = {SGI39429, SGI0, SGI39430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8909[] = {SGI39431, SGI39432, SGI39433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39434, SGI0, SGI0, SGI0, SGI39435, SGI39435, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8910[] = {SGI39436, SGI39437, SGI39438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39439, SGI0, SGI0, SGI0, SGI39440, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8911[] = {SGI39441, SGI39442, SGI39443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39444, SGI0, SGI0, SGI0, SGI39445, SGI39445, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8912[] = {SGI39446, SGI39447, SGI39448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39449, SGI0, SGI0, SGI0, SGI39450, SGI39450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8913[] = {SGI39451, SGI39452, SGI39453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39454, SGI0, SGI0, SGI0, SGI39455, SGI39455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8914[] = {SGI39456, SGI39457, SGI39458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39459, SGI0, SGI0, SGI0, SGI39460, SGI39460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8915[] = {SGI39461, SGI39462, SGI39463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39464, SGI0, SGI0, SGI0, SGI39465, SGI39465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8916[] = {SGI39466, SGI39467, SGI39468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39469, SGI0, SGI0, SGI0, SGI39470, SGI39470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8917[] = {SGI39471, SGI39472, SGI39473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39474, SGI0, SGI0, SGI0, SGI39475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8918[] = {SGI39476, SGI39477, SGI39478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39479, SGI0, SGI0, SGI0, SGI39480, SGI39480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8919[] = {SGI39481, SGI39482, SGI39483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39484, SGI0, SGI0, SGI0, SGI39485, SGI39485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8920[] = {SGI39486, SGI39487, SGI39488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39489, SGI0, SGI0, SGI0, SGI39490, SGI39490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8921[] = {SGI39491, SGI39492, SGI39494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39497, SGI0, SGI0, SGI0, SGI39499, SGI39499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8922[] = {SGI39493, SGI39495, SGI39496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39498, SGI0, SGI0, SGI0, SGI39500, SGI39500, SGI0, SGI0, NULL};
const Segment * const SG8923[] = {SGI39501, SGI39502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39503, SGI0, SGI0, SGI0, SGI39504, SGI39504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8924[] = {SGI39505, SGI0, SGI39506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8925[] = {SGI39507, SGI0, SGI39508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8926[] = {SGI39509, SGI39510, SGI39511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39512, SGI0, SGI0, SGI0, SGI39513, SGI39513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8927[] = {SGI39514, SGI39515, SGI39516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39517, SGI0, SGI0, SGI0, SGI39518, SGI39518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8928[] = {SGI39519, SGI39520, SGI39521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39522, SGI0, SGI0, SGI0, SGI39523, SGI39523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8929[] = {SGI39524, SGI39525, SGI39526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39527, SGI0, SGI0, SGI0, SGI39528, SGI39528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8930[] = {SGI39529, SGI39530, SGI39531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39532, SGI0, SGI0, SGI0, SGI39533, SGI39533, SGI0, SGI0, NULL};
const Segment * const SG8931[] = {SGI39534, SGI39535, SGI39536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39537, SGI0, SGI0, SGI0, SGI39538, SGI39538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8932[] = {SGI39539, SGI39540, SGI39541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39542, SGI0, SGI0, SGI0, SGI39543, SGI39543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8933[] = {SGI39544, SGI39545, SGI39546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39547, SGI0, SGI0, SGI0, SGI39548, SGI39548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8934[] = {SGI39549, SGI39550, SGI39551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39552, SGI0, SGI0, SGI0, SGI39553, SGI39553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8935[] = {SGI39554, SGI39555, SGI39556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39557, SGI0, SGI0, SGI0, SGI39558, SGI39558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8936[] = {SGI39559, SGI39560, SGI39561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39562, SGI0, SGI0, SGI0, SGI39563, SGI39563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8937[] = {SGI39564, SGI0, SGI39565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8938[] = {SGI39566, SGI39567, SGI39568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39569, SGI0, SGI0, SGI0, SGI39570, SGI39570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8939[] = {SGI39571, SGI39572, SGI39573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39574, SGI0, SGI0, SGI0, SGI39575, SGI39575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8940[] = {SGI39576, SGI39577, SGI39578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39579, SGI0, SGI0, SGI0, SGI39580, SGI39580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8941[] = {SGI39581, SGI39582, SGI39583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39584, SGI0, SGI0, SGI0, SGI39585, SGI39585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8942[] = {SGI39586, SGI39587, SGI39588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39589, SGI0, SGI0, SGI0, SGI39590, SGI39590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8943[] = {SGI39591, SGI39592, SGI39593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39594, SGI0, SGI0, SGI0, SGI39595, SGI39595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8944[] = {SGI39596, SGI39597, SGI39598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39599, SGI39600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8945[] = {SGI39601, SGI39602, SGI39603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39604, SGI0, SGI0, SGI0, SGI39605, SGI39605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8946[] = {SGI39606, SGI39607, SGI39608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39609, SGI0, SGI0, SGI0, SGI39610, SGI39610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8947[] = {SGI39611, SGI39612, SGI39613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39614, SGI0, SGI0, SGI0, SGI39615, SGI39615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8948[] = {SGI39616, SGI0, SGI39617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39618, SGI0, SGI0, SGI0, SGI39619, SGI39619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8949[] = {SGI39620, SGI39621, SGI39622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39623, SGI0, SGI0, SGI0, SGI39624, SGI39624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8950[] = {SGI39625, SGI39626, SGI39627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39628, SGI0, SGI0, SGI0, SGI39629, SGI39629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8951[] = {SGI39630, SGI39631, SGI39632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39633, SGI0, SGI0, SGI0, SGI39634, SGI39634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8952[] = {SGI39635, SGI39636, SGI39637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39638, SGI0, SGI0, SGI0, SGI39639, SGI39639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8953[] = {SGI39640, SGI39641, SGI39642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39643, SGI0, SGI0, SGI0, SGI39644, SGI39644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8954[] = {SGI39645, SGI39646, SGI39647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39648, SGI0, SGI0, SGI0, SGI39649, SGI39649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8955[] = {SGI39650, SGI0, SGI39651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8956[] = {SGI39652, SGI39653, SGI39654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39655, SGI0, SGI0, SGI0, SGI39656, SGI39656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8957[] = {SGI39657, SGI39658, SGI39659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39660, SGI0, SGI0, SGI0, SGI39661, SGI39661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8958[] = {SGI39662, SGI39663, SGI39664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39665, SGI39666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8959[] = {SGI39667, SGI39668, SGI39669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39670, SGI0, SGI0, SGI0, SGI39671, SGI39671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8960[] = {SGI39672, SGI39673, SGI39674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39675, SGI0, SGI0, SGI0, SGI39676, SGI39676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8961[] = {SGI39677, SGI0, SGI39678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8962[] = {SGI39679, SGI39680, SGI39681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39682, SGI39682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8963[] = {SGI39683, SGI0, SGI39684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8964[] = {SGI39685, SGI39686, SGI39687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39688, SGI39689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8965[] = {SGI39690, SGI39691, SGI39692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39693, SGI0, SGI0, SGI0, SGI39694, SGI39694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8966[] = {SGI39695, SGI39696, SGI39697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39698, SGI39698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8967[] = {SGI39699, SGI0, SGI39700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8968[] = {SGI39701, SGI39702, SGI39703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39704, SGI39704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8969[] = {SGI39705, SGI39706, SGI39707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39708, SGI0, SGI0, SGI0, SGI39709, SGI39709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8970[] = {SGI39710, SGI39711, SGI39712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39713, SGI0, SGI0, SGI0, SGI39714, SGI39714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8971[] = {SGI39715, SGI0, SGI39716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8972[] = {SGI39717, SGI39718, SGI39719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39720, SGI0, SGI0, SGI0, SGI39721, SGI39721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8973[] = {SGI39722, SGI39723, SGI39724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39725, SGI0, SGI0, SGI0, SGI39726, SGI39726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8974[] = {SGI39727, SGI0, SGI39728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8975[] = {SGI39729, SGI39730, SGI39731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39732, SGI0, SGI0, SGI0, SGI39733, SGI39733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8976[] = {SGI39734, SGI39735, SGI39736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39737, SGI0, SGI0, SGI0, SGI39738, SGI39738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8977[] = {SGI39739, SGI39740, SGI39741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39742, SGI0, SGI0, SGI0, SGI39743, SGI39743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8978[] = {SGI39744, SGI39745, SGI39746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39747, SGI0, SGI0, SGI0, SGI39748, SGI39748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8979[] = {SGI39749, SGI39750, SGI39751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39752, SGI0, SGI0, SGI0, SGI39753, SGI39753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8980[] = {SGI39754, SGI39755, SGI39756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39757, SGI0, SGI0, SGI0, SGI39758, SGI39758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8981[] = {SGI39759, SGI39760, SGI39761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39762, SGI0, SGI0, SGI0, SGI39763, SGI39763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8982[] = {SGI39764, SGI39765, SGI39766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39767, SGI0, SGI0, SGI0, SGI39768, SGI39768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8983[] = {SGI39769, SGI0, SGI39770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8984[] = {SGI39771, SGI0, SGI39772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8985[] = {SGI39773, SGI39774, SGI39775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39776, SGI39777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8986[] = {SGI39778, SGI39779, SGI39780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39781, SGI39781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8987[] = {SGI39782, SGI0, SGI39783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39784, SGI0, SGI0, SGI0, SGI39785, SGI39785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8988[] = {SGI39786, SGI39787, SGI39788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39789, SGI0, SGI0, SGI0, SGI39790, SGI39790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8989[] = {SGI39791, SGI0, SGI39792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8990[] = {SGI39793, SGI39794, SGI39795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39796, SGI0, SGI0, SGI0, SGI39797, SGI39797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8991[] = {SGI39798, SGI39799, SGI39800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39801, SGI0, SGI0, SGI0, SGI39802, SGI39802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8992[] = {SGI39803, SGI39804, SGI39805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39806, SGI39807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8993[] = {SGI39808, SGI0, SGI39809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39810, SGI0, SGI0, SGI0, SGI39811, SGI39811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8994[] = {SGI39812, SGI39813, SGI39814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39815, SGI39815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8995[] = {SGI39816, SGI39817, SGI39818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39819, SGI0, SGI0, SGI0, SGI39820, SGI39820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8996[] = {SGI39821, SGI0, SGI39822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8997[] = {SGI39823, SGI39824, SGI39825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39826, SGI0, SGI0, SGI0, SGI39827, SGI39827, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8998[] = {SGI39828, SGI39829, SGI39830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39831, SGI0, SGI0, SGI0, SGI39832, SGI39832, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG8999[] = {SGI39833, SGI0, SGI39834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9000[] = {SGI39835, SGI39836, SGI39837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39838, SGI39838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9001[] = {SGI39839, SGI39840, SGI39841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39842, SGI0, SGI0, SGI0, SGI39843, SGI39843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9002[] = {SGI39844, SGI0, SGI39845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9003[] = {SGI39846, SGI0, SGI39847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39848, SGI0, SGI0, SGI0, SGI39849, SGI39849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9004[] = {SGI39850, SGI39851, SGI39852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39853, SGI0, SGI0, SGI0, SGI39854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9005[] = {SGI39855, SGI39856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39857, SGI0, SGI0, SGI0, SGI39858, SGI39858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9006[] = {SGI39859, SGI39860, SGI39861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39862, SGI0, SGI0, SGI0, SGI39863, SGI39863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9007[] = {SGI39864, SGI39865, SGI39866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39867, SGI0, SGI0, SGI0, SGI39868, SGI39868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9008[] = {SGI39869, SGI39870, SGI39871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39872, SGI0, SGI0, SGI0, SGI39873, SGI39873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9009[] = {SGI39874, SGI39875, SGI39876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39877, SGI0, SGI0, SGI0, SGI39878, SGI39878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9010[] = {SGI39879, SGI39880, SGI39881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9011[] = {SGI39882, SGI0, SGI39883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9012[] = {SGI39884, SGI39885, SGI39886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39887, SGI0, SGI0, SGI0, SGI39888, SGI39889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9013[] = {SGI39890, SGI0, SGI39891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39892, SGI0, SGI0, SGI0, SGI39893, SGI39893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9014[] = {SGI39894, SGI39895, SGI39896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39897, SGI0, SGI0, SGI0, SGI39898, SGI39898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9015[] = {SGI39899, SGI39900, SGI39901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39902, SGI0, SGI0, SGI0, SGI39903, SGI39903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9016[] = {SGI39904, SGI39905, SGI39906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39907, SGI0, SGI0, SGI0, SGI39908, SGI39908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9017[] = {SGI39909, SGI39910, SGI39911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39912, SGI0, SGI0, SGI0, SGI39913, SGI39913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9018[] = {SGI39914, SGI0, SGI39915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9019[] = {SGI39916, SGI39917, SGI39918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39919, SGI0, SGI0, SGI0, SGI39920, SGI39920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9020[] = {SGI39921, SGI39922, SGI39923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39924, SGI0, SGI0, SGI0, SGI39925, SGI39925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9021[] = {SGI39926, SGI0, SGI39927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9022[] = {SGI39928, SGI39929, SGI39930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39931, SGI0, SGI0, SGI0, SGI39932, SGI39932, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9023[] = {SGI39933, SGI39934, SGI39935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39936, SGI0, SGI0, SGI0, SGI39937, SGI39937, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9024[] = {SGI39938, SGI0, SGI39939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9025[] = {SGI39940, SGI39941, SGI39942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39943, SGI0, SGI0, SGI0, SGI39944, SGI39944, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9026[] = {SGI39945, SGI39946, SGI39947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39948, SGI0, SGI0, SGI0, SGI39949, SGI39949, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9027[] = {SGI39950, SGI39951, SGI39952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39953, SGI0, SGI0, SGI0, SGI39954, SGI39954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9028[] = {SGI39955, SGI39956, SGI39957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39958, SGI0, SGI0, SGI0, SGI39959, SGI39959, SGI0, SGI0, NULL};
const Segment * const SG9029[] = {SGI39960, SGI39961, SGI39962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39963, SGI0, SGI0, SGI0, SGI39964, SGI39964, SGI0, SGI0, NULL};
const Segment * const SG9030[] = {SGI39965, SGI39966, SGI39967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39968, SGI0, SGI0, SGI0, SGI39969, SGI39969, SGI0, SGI0, NULL};
const Segment * const SG9031[] = {SGI39970, SGI0, SGI39971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9032[] = {SGI39972, SGI0, SGI39973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39974, SGI0, SGI0, SGI0, SGI39975, SGI39975, SGI0, SGI0, NULL};
const Segment * const SG9033[] = {SGI39976, SGI39977, SGI39978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39979, SGI0, SGI0, SGI0, SGI39980, SGI39980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9034[] = {SGI39981, SGI39982, SGI39983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39984, SGI0, SGI0, SGI0, SGI39985, SGI39986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9035[] = {SGI39987, SGI39988, SGI39989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39990, SGI39990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9036[] = {SGI39991, SGI0, SGI39992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9037[] = {SGI39993, SGI0, SGI39994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI39995, SGI0, SGI0, SGI0, SGI39996, SGI39996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9038[] = {SGI39997, SGI39998, SGI39999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40000, SGI0, SGI0, SGI0, SGI40001, SGI40001, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9039[] = {SGI40002, SGI0, SGI40003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9040[] = {SGI40004, SGI40005, SGI40006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40007, SGI0, SGI0, SGI0, SGI40008, SGI40008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9041[] = {SGI40009, SGI40010, SGI40011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40012, SGI0, SGI0, SGI0, SGI40013, SGI40013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9042[] = {SGI40014, SGI40015, SGI40016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40017, SGI0, SGI0, SGI0, SGI40018, SGI40018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9043[] = {SGI40019, SGI0, SGI40020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9044[] = {SGI40021, SGI0, SGI40022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9045[] = {SGI40023, SGI40024, SGI40025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40026, SGI0, SGI0, SGI0, SGI40027, SGI40027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9046[] = {SGI40028, SGI40029, SGI40030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40031, SGI0, SGI0, SGI0, SGI40032, SGI40032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9047[] = {SGI40033, SGI40034, SGI40035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40036, SGI0, SGI0, SGI0, SGI40037, SGI40037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9048[] = {SGI40038, SGI0, SGI40039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9049[] = {SGI40040, SGI40041, SGI40042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40043, SGI0, SGI0, SGI0, SGI40044, SGI40044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9050[] = {SGI40045, SGI40046, SGI40047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40048, SGI0, SGI0, SGI0, SGI40049, SGI40049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9051[] = {SGI40050, SGI40051, SGI40052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40053, SGI0, SGI0, SGI0, SGI40054, SGI40054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9052[] = {SGI40055, SGI40056, SGI40057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40058, SGI0, SGI0, SGI0, SGI40059, SGI40059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9053[] = {SGI40060, SGI0, SGI40061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9054[] = {SGI40062, SGI0, SGI40063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9055[] = {SGI40064, SGI40065, SGI40066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40067, SGI0, SGI0, SGI0, SGI40068, SGI40068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9056[] = {SGI40069, SGI40070, SGI40071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40072, SGI0, SGI0, SGI0, SGI40073, SGI40073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9057[] = {SGI40074, SGI40075, SGI40076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9058[] = {SGI40077, SGI0, SGI40078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40079, SGI0, SGI0, SGI0, SGI40080, SGI40080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9059[] = {SGI40081, SGI40082, SGI40083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9060[] = {SGI40084, SGI40085, SGI40086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40087, SGI40087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9061[] = {SGI40091, SGI0, SGI40092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9062[] = {SGI40088, SGI40089, SGI40090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40093, SGI0, SGI0, SGI0, SGI40094, SGI40094, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9063[] = {SGI40095, SGI40096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40100, SGI0, SGI0, SGI0, SGI40101, SGI40101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9064[] = {SGI40097, SGI40098, SGI40099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40102, SGI0, SGI0, SGI0, SGI40103, SGI40103, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9065[] = {SGI40104, SGI0, SGI40105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9066[] = {SGI40106, SGI40107, SGI40108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40109, SGI0, SGI0, SGI0, SGI40110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9067[] = {SGI40111, SGI40112, SGI40113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40114, SGI0, SGI0, SGI0, SGI40115, SGI40115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9068[] = {SGI40116, SGI40117, SGI40118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40119, SGI0, SGI0, SGI0, SGI40121, SGI40121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9069[] = {SGI40120, SGI40122, SGI40123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40124, SGI0, SGI0, SGI0, SGI40125, SGI40125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9070[] = {SGI40126, SGI40127, SGI40128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40131, SGI0, SGI0, SGI0, SGI40133, SGI40133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9071[] = {SGI40129, SGI40130, SGI40132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40134, SGI0, SGI0, SGI0, SGI40135, SGI40135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9072[] = {SGI40136, SGI40137, SGI40138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40142, SGI0, SGI0, SGI0, SGI40143, SGI40143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9073[] = {SGI40139, SGI40140, SGI40141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40144, SGI0, SGI0, SGI0, SGI40145, SGI40145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9074[] = {SGI40146, SGI40147, SGI40148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40149, SGI0, SGI0, SGI0, SGI40150, SGI40150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9075[] = {SGI40151, SGI40152, SGI40153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40154, SGI0, SGI0, SGI0, SGI40155, SGI40155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9076[] = {SGI40156, SGI40157, SGI40158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40159, SGI0, SGI0, SGI0, SGI40160, SGI40160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9077[] = {SGI40161, SGI40162, SGI40163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40164, SGI0, SGI0, SGI0, SGI40165, SGI40165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9078[] = {SGI40166, SGI0, SGI40167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9079[] = {SGI40168, SGI40169, SGI40170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40171, SGI0, SGI0, SGI0, SGI40172, SGI40172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9080[] = {SGI40173, SGI40174, SGI40175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40176, SGI0, SGI0, SGI0, SGI40177, SGI40177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9081[] = {SGI40178, SGI40179, SGI40180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40181, SGI0, SGI0, SGI0, SGI40182, SGI40182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9082[] = {SGI40183, SGI40184, SGI40185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40186, SGI0, SGI0, SGI0, SGI40187, SGI40187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9083[] = {SGI40188, SGI0, SGI40189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9084[] = {SGI40190, SGI40191, SGI40192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40193, SGI0, SGI0, SGI0, SGI40194, SGI40194, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9085[] = {SGI40195, SGI40196, SGI40197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40198, SGI0, SGI0, SGI0, SGI40199, SGI40199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9086[] = {SGI40200, SGI40201, SGI40202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40203, SGI0, SGI0, SGI0, SGI40204, SGI40204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9087[] = {SGI40205, SGI40206, SGI40207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40208, SGI0, SGI0, SGI0, SGI40209, SGI40209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9088[] = {SGI40210, SGI0, SGI40211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9089[] = {SGI40212, SGI0, SGI40213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9090[] = {SGI40214, SGI40215, SGI40217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40220, SGI0, SGI0, SGI0, SGI40222, SGI40222, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9091[] = {SGI40216, SGI40218, SGI40219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40221, SGI0, SGI0, SGI0, SGI40223, SGI40223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9092[] = {SGI40227, SGI0, SGI40229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9093[] = {SGI40224, SGI40225, SGI40226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40228, SGI0, SGI0, SGI0, SGI40230, SGI40230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9094[] = {SGI40231, SGI0, SGI40232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9095[] = {SGI40233, SGI40234, SGI40235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40236, SGI0, SGI0, SGI0, SGI40237, SGI40237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9096[] = {SGI40238, SGI40239, SGI40240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40241, SGI0, SGI0, SGI0, SGI40242, SGI40242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9097[] = {SGI40243, SGI0, SGI40244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9098[] = {SGI40245, SGI40246, SGI40247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40248, SGI0, SGI0, SGI0, SGI40249, SGI40249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9099[] = {SGI40250, SGI40251, SGI40252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40253, SGI0, SGI0, SGI0, SGI40254, SGI40254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9100[] = {SGI40255, SGI0, SGI40256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9101[] = {SGI40257, SGI40258, SGI40259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40260, SGI0, SGI0, SGI0, SGI40261, SGI40261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9102[] = {SGI40262, SGI40263, SGI40264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40265, SGI0, SGI0, SGI0, SGI40266, SGI40266, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9103[] = {SGI40267, SGI40268, SGI40269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40270, SGI0, SGI0, SGI0, SGI40271, SGI40271, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9104[] = {SGI40272, SGI40273, SGI40274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40275, SGI0, SGI0, SGI0, SGI40276, SGI40276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9105[] = {SGI40277, SGI40278, SGI40279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40280, SGI0, SGI0, SGI0, SGI40281, SGI40281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9106[] = {SGI40282, SGI40283, SGI40284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40285, SGI0, SGI0, SGI0, SGI40286, SGI40286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9107[] = {SGI40287, SGI40288, SGI40289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40290, SGI0, SGI0, SGI0, SGI40291, SGI40291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9108[] = {SGI40292, SGI40293, SGI40294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40295, SGI0, SGI0, SGI0, SGI40296, SGI40296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9109[] = {SGI40297, SGI0, SGI40298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9110[] = {SGI40299, SGI40300, SGI40301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40302, SGI0, SGI0, SGI0, SGI40303, SGI40303, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9111[] = {SGI40304, SGI40305, SGI40306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40307, SGI0, SGI0, SGI0, SGI40308, SGI40308, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9112[] = {SGI40309, SGI0, SGI40310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9113[] = {SGI40311, SGI40312, SGI40313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40314, SGI0, SGI0, SGI0, SGI40315, SGI40315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9114[] = {SGI40316, SGI40317, SGI40318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40319, SGI0, SGI0, SGI0, SGI40320, SGI40320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9115[] = {SGI40321, SGI40322, SGI40323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40324, SGI0, SGI0, SGI0, SGI40325, SGI40325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9116[] = {SGI40326, SGI40327, SGI40328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40329, SGI0, SGI0, SGI0, SGI40330, SGI40330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9117[] = {SGI40331, SGI40332, SGI40333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40334, SGI0, SGI0, SGI0, SGI40335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9118[] = {SGI40336, SGI40337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40338, SGI0, SGI0, SGI0, SGI40339, SGI40339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9119[] = {SGI40340, SGI40341, SGI40342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40343, SGI0, SGI0, SGI0, SGI40344, SGI40344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9120[] = {SGI40345, SGI0, SGI40346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9121[] = {SGI40347, SGI40348, SGI40349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9122[] = {SGI40350, SGI40351, SGI40352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40353, SGI0, SGI0, SGI0, SGI40354, SGI40354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9123[] = {SGI40355, SGI40356, SGI40357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40358, SGI0, SGI0, SGI0, SGI40359, SGI40360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9124[] = {SGI40361, SGI0, SGI40362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40363, SGI0, SGI0, SGI0, SGI40364, SGI40364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9125[] = {SGI40365, SGI40366, SGI40367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40368, SGI0, SGI0, SGI0, SGI40369, SGI40369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9126[] = {SGI40370, SGI40371, SGI40372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40373, SGI40373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9127[] = {SGI40374, SGI40375, SGI40376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40377, SGI0, SGI0, SGI0, SGI40378, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9128[] = {SGI40379, SGI40380, SGI40381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40382, SGI0, SGI0, SGI0, SGI40383, SGI40383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9129[] = {SGI40384, SGI40385, SGI40386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40387, SGI0, SGI0, SGI0, SGI40388, SGI40388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9130[] = {SGI40389, SGI40390, SGI40391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40392, SGI0, SGI0, SGI0, SGI40393, SGI40393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9131[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40397, SGI0, SGI0, SGI40398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9132[] = {SGI40394, SGI40395, SGI40396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40399, SGI0, SGI0, SGI0, SGI40400, SGI40400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9133[] = {SGI40401, SGI0, SGI40402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9134[] = {SGI40403, SGI40404, SGI40405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40406, SGI0, SGI0, SGI0, SGI40407, SGI40407, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9135[] = {SGI40408, SGI40409, SGI40410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40411, SGI0, SGI0, SGI0, SGI40412, SGI40412, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9136[] = {SGI40413, SGI0, SGI40414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9137[] = {SGI40415, SGI40416, SGI40417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40418, SGI0, SGI0, SGI0, SGI40419, SGI40419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9138[] = {SGI40420, SGI0, SGI40421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9139[] = {SGI40422, SGI40423, SGI40424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40425, SGI0, SGI0, SGI0, SGI40426, SGI40426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9140[] = {SGI40427, SGI0, SGI40428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40429, SGI0, SGI0, SGI0, SGI40430, SGI40430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9141[] = {SGI40431, SGI40432, SGI40433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40434, SGI40434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9142[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40435, SGI0, SGI0, SGI40436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9143[] = {SGI40437, SGI40438, SGI40439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40440, SGI0, SGI0, SGI0, SGI40441, SGI40441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9144[] = {SGI40442, SGI0, SGI40443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9145[] = {SGI40444, SGI40445, SGI40446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40447, SGI0, SGI0, SGI0, SGI40448, SGI40448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9146[] = {SGI40449, SGI40450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40451, SGI0, SGI0, SGI0, SGI40452, SGI40452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9147[] = {SGI40453, SGI40454, SGI40455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40456, SGI0, SGI0, SGI0, SGI40457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9148[] = {SGI40458, SGI40459, SGI40460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40461, SGI0, SGI0, SGI0, SGI40462, SGI40462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9149[] = {SGI40463, SGI40464, SGI40465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40466, SGI0, SGI0, SGI0, SGI40467, SGI40467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9150[] = {SGI40468, SGI40469, SGI40470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40471, SGI0, SGI0, SGI0, SGI40472, SGI40472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9151[] = {SGI40473, SGI40474, SGI40475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40476, SGI0, SGI0, SGI0, SGI40477, SGI40477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9152[] = {SGI40478, SGI40479, SGI40480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40481, SGI0, SGI0, SGI0, SGI40482, SGI40482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9153[] = {SGI40483, SGI40484, SGI40485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40486, SGI0, SGI0, SGI0, SGI40487, SGI40487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9154[] = {SGI40488, SGI40489, SGI40490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40491, SGI0, SGI0, SGI0, SGI40492, SGI40492, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9155[] = {SGI40493, SGI40494, SGI40495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40498, SGI40500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9156[] = {SGI40496, SGI40497, SGI40499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40501, SGI0, SGI0, SGI0, SGI40502, SGI40502, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9157[] = {SGI40503, SGI40504, SGI40505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40506, SGI40508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9158[] = {SGI40507, SGI40509, SGI40510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40511, SGI0, SGI0, SGI0, SGI40512, SGI40512, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9159[] = {SGI40513, SGI0, SGI40514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9160[] = {SGI40515, SGI40516, SGI40517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40518, SGI40518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9161[] = {SGI40519, SGI0, SGI40520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9162[] = {SGI40521, SGI40522, SGI40523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40524, SGI0, SGI0, SGI0, SGI40525, SGI40525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9163[] = {SGI40526, SGI40527, SGI40528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40529, SGI0, SGI0, SGI0, SGI40530, SGI40530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9164[] = {SGI40531, SGI40532, SGI40533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40534, SGI40535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9165[] = {SGI40536, SGI40537, SGI40538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40539, SGI0, SGI0, SGI0, SGI40540, SGI40540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9166[] = {SGI40541, SGI40542, SGI40543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40544, SGI0, SGI0, SGI0, SGI40545, SGI40545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9167[] = {SGI40546, SGI40547, SGI40548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40549, SGI0, SGI0, SGI0, SGI40550, SGI40550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9168[] = {SGI40551, SGI40552, SGI40553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40554, SGI0, SGI0, SGI0, SGI40555, SGI40555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9169[] = {SGI40556, SGI40557, SGI40558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40559, SGI0, SGI0, SGI0, SGI40560, SGI40560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9170[] = {SGI40561, SGI0, SGI40562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9171[] = {SGI40563, SGI0, SGI40564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9172[] = {SGI40565, SGI40566, SGI40567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40568, SGI0, SGI0, SGI0, SGI40569, SGI40569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9173[] = {SGI40570, SGI40571, SGI40572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40573, SGI0, SGI0, SGI0, SGI40574, SGI40574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9174[] = {SGI40575, SGI40576, SGI40577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40578, SGI40579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9175[] = {SGI40580, SGI40581, SGI40582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40583, SGI40584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9176[] = {SGI40585, SGI40586, SGI40587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40588, SGI40588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9177[] = {SGI40589, SGI40590, SGI40591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40592, SGI40592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9178[] = {SGI40593, SGI0, SGI40594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9179[] = {SGI40595, SGI40596, SGI40597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40598, SGI0, SGI0, SGI0, SGI40599, SGI40599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9180[] = {SGI40600, SGI40601, SGI40602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40603, SGI0, SGI0, SGI0, SGI40604, SGI40604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9181[] = {SGI40605, SGI40606, SGI40607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40608, SGI0, SGI0, SGI0, SGI40609, SGI40609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9182[] = {SGI40610, SGI40611, SGI40612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40613, SGI0, SGI0, SGI0, SGI40614, SGI40614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9183[] = {SGI40615, SGI40616, SGI40617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40618, SGI0, SGI0, SGI0, SGI40619, SGI40619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9184[] = {SGI40620, SGI40621, SGI40622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40623, SGI0, SGI0, SGI0, SGI40624, SGI40624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9185[] = {SGI40625, SGI40626, SGI40627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40628, SGI0, SGI0, SGI0, SGI40629, SGI40629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9186[] = {SGI40630, SGI40631, SGI40632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40633, SGI0, SGI0, SGI0, SGI40634, SGI40634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9187[] = {SGI40635, SGI40636, SGI40637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40638, SGI0, SGI0, SGI0, SGI40639, SGI40639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9188[] = {SGI40640, SGI40641, SGI40642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40643, SGI0, SGI0, SGI0, SGI40644, SGI40644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9189[] = {SGI40645, SGI40646, SGI40647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40648, SGI0, SGI0, SGI0, SGI40649, SGI40649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9190[] = {SGI40650, SGI0, SGI40651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9191[] = {SGI40652, SGI40653, SGI40654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40655, SGI40656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9192[] = {SGI40657, SGI40658, SGI40659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40660, SGI0, SGI0, SGI0, SGI40661, SGI40661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9193[] = {SGI40662, SGI40663, SGI40664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40665, SGI0, SGI0, SGI0, SGI40666, SGI40666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9194[] = {SGI40667, SGI40668, SGI40669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40670, SGI0, SGI0, SGI0, SGI40671, SGI40671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9195[] = {SGI40672, SGI40673, SGI40674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40677, SGI0, SGI0, SGI0, SGI40679, SGI40679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9196[] = {SGI40675, SGI40676, SGI40678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40680, SGI0, SGI0, SGI0, SGI40682, SGI40682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9197[] = {SGI40681, SGI40683, SGI40684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40685, SGI0, SGI0, SGI0, SGI40686, SGI40686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9198[] = {SGI40687, SGI0, SGI40688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9199[] = {SGI40689, SGI40690, SGI40691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40692, SGI0, SGI0, SGI0, SGI40693, SGI40693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9200[] = {SGI40694, SGI40695, SGI40696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40697, SGI40698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9201[] = {SGI40699, SGI40700, SGI40701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40702, SGI0, SGI0, SGI0, SGI40703, SGI40703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9202[] = {SGI40704, SGI40705, SGI40706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40707, SGI0, SGI0, SGI0, SGI40708, SGI40709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9203[] = {SGI40710, SGI40711, SGI40712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40713, SGI40713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9204[] = {SGI0, SGI40714, SGI40715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9205[] = {SGI40716, SGI40717, SGI40718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40719, SGI0, SGI0, SGI0, SGI40720, SGI40720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9206[] = {SGI40721, SGI0, SGI40722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9207[] = {SGI40723, SGI40724, SGI40725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40726, SGI0, SGI0, SGI0, SGI40727, SGI40727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9208[] = {SGI40728, SGI40729, SGI40730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40731, SGI0, SGI0, SGI0, SGI40732, SGI40732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9209[] = {SGI40733, SGI40734, SGI40735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40736, SGI0, SGI0, SGI0, SGI40737, SGI40737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9210[] = {SGI40738, SGI40739, SGI40740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40741, SGI0, SGI0, SGI0, SGI40742, SGI40742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9211[] = {SGI40743, SGI40744, SGI40745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40746, SGI0, SGI0, SGI0, SGI40747, SGI40747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9212[] = {SGI40748, SGI40749, SGI40750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40751, SGI0, SGI0, SGI0, SGI40752, SGI40752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9213[] = {SGI40753, SGI40754, SGI40755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40756, SGI0, SGI0, SGI0, SGI40757, SGI40757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9214[] = {SGI40758, SGI0, SGI40759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9215[] = {SGI40760, SGI40761, SGI40762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40763, SGI40763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9216[] = {SGI40764, SGI0, SGI40765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9217[] = {SGI40766, SGI0, SGI40767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9218[] = {SGI40768, SGI40769, SGI40770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40771, SGI40772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9219[] = {SGI40773, SGI40774, SGI40775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40776, SGI0, SGI0, SGI0, SGI40777, SGI40777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9220[] = {SGI40778, SGI40779, SGI40780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40781, SGI40781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9221[] = {SGI40782, SGI40783, SGI40784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40785, SGI40786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9222[] = {SGI40787, SGI40788, SGI40789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40790, SGI0, SGI0, SGI0, SGI40791, SGI40791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9223[] = {SGI40792, SGI40793, SGI40794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40795, SGI40795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9224[] = {SGI0, SGI40796, SGI40797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9225[] = {SGI40798, SGI0, SGI40799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9226[] = {SGI40800, SGI40801, SGI40802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40803, SGI0, SGI0, SGI0, SGI40804, SGI40804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9227[] = {SGI40805, SGI40806, SGI40807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40808, SGI0, SGI0, SGI0, SGI40809, SGI40809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9228[] = {SGI40810, SGI40811, SGI40812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40813, SGI40813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9229[] = {SGI40814, SGI40815, SGI40816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40817, SGI40818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9230[] = {SGI40819, SGI40820, SGI40821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40822, SGI0, SGI0, SGI0, SGI40823, SGI40823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9231[] = {SGI40824, SGI40825, SGI40826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40827, SGI0, SGI0, SGI0, SGI40828, SGI40828, SGI0, SGI0, NULL};
const Segment * const SG9232[] = {SGI40829, SGI40830, SGI40831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40832, SGI0, SGI0, SGI0, SGI40833, SGI40833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9233[] = {SGI40834, SGI0, SGI40835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9234[] = {SGI40836, SGI0, SGI40837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40838, SGI0, SGI0, SGI0, SGI40839, SGI40839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9235[] = {SGI40840, SGI40841, SGI40842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40843, SGI0, SGI0, SGI0, SGI40844, SGI40844, SGI0, SGI0, NULL};
const Segment * const SG9236[] = {SGI40845, SGI40846, SGI40847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40848, SGI0, SGI0, SGI0, SGI40849, SGI40849, SGI0, SGI0, NULL};
const Segment * const SG9237[] = {SGI40850, SGI40851, SGI40852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40853, SGI0, SGI0, SGI0, SGI40854, SGI40854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9238[] = {SGI40855, SGI40856, SGI40857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40858, SGI0, SGI0, SGI0, SGI40859, SGI40861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9239[] = {SGI40860, SGI40862, SGI40863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40864, SGI0, SGI0, SGI0, SGI40865, SGI40865, SGI0, SGI0, NULL};
const Segment * const SG9240[] = {SGI40866, SGI0, SGI40867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9241[] = {SGI40868, SGI40869, SGI40870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40871, SGI0, SGI0, SGI0, SGI40872, SGI40872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9242[] = {SGI40873, SGI40874, SGI40875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40876, SGI0, SGI0, SGI0, SGI40877, SGI40877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9243[] = {SGI40878, SGI40879, SGI40880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40881, SGI0, SGI0, SGI0, SGI40882, SGI40882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9244[] = {SGI40883, SGI40884, SGI40885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40886, SGI0, SGI0, SGI0, SGI40887, SGI40887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9245[] = {SGI40888, SGI40889, SGI40890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40891, SGI0, SGI0, SGI0, SGI40892, SGI40892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9246[] = {SGI40893, SGI40894, SGI40895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40896, SGI0, SGI0, SGI0, SGI40897, SGI40897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9247[] = {SGI40898, SGI40899, SGI40900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40901, SGI0, SGI0, SGI0, SGI40902, SGI40902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9248[] = {SGI40903, SGI0, SGI40904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9249[] = {SGI40905, SGI40906, SGI40907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40908, SGI0, SGI0, SGI0, SGI40909, SGI40909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9250[] = {SGI40910, SGI0, SGI40911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9251[] = {SGI40912, SGI40913, SGI40914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40915, SGI0, SGI0, SGI0, SGI40916, SGI40916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9252[] = {SGI40917, SGI40918, SGI40919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40920, SGI0, SGI0, SGI0, SGI40921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9253[] = {SGI40922, SGI40923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40924, SGI0, SGI0, SGI0, SGI40925, SGI40925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9254[] = {SGI40926, SGI40927, SGI40928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40929, SGI0, SGI0, SGI0, SGI40930, SGI40930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9255[] = {SGI40931, SGI40932, SGI40933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40934, SGI0, SGI0, SGI0, SGI40935, SGI40935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9256[] = {SGI40936, SGI40937, SGI40938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40939, SGI0, SGI0, SGI0, SGI40940, SGI40941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9257[] = {SGI40942, SGI0, SGI40943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40944, SGI0, SGI0, SGI0, SGI40945, SGI40945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9258[] = {SGI40946, SGI40947, SGI40948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40949, SGI0, SGI0, SGI0, SGI40950, SGI40950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9259[] = {SGI40951, SGI0, SGI40952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9260[] = {SGI40953, SGI40954, SGI40955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40956, SGI0, SGI0, SGI0, SGI40957, SGI40957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9261[] = {SGI40958, SGI40959, SGI40960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40961, SGI0, SGI0, SGI0, SGI40962, SGI40962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9262[] = {SGI40963, SGI40964, SGI40965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40966, SGI0, SGI0, SGI0, SGI40967, SGI40967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9263[] = {SGI40968, SGI40969, SGI40970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40971, SGI0, SGI0, SGI0, SGI40972, SGI40972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9264[] = {SGI40973, SGI40974, SGI40975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40976, SGI0, SGI0, SGI0, SGI40977, SGI40977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9265[] = {SGI40978, SGI40979, SGI40980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40981, SGI0, SGI0, SGI0, SGI40982, SGI40982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9266[] = {SGI40983, SGI40984, SGI40985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40986, SGI0, SGI0, SGI0, SGI40987, SGI40987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9267[] = {SGI40988, SGI40989, SGI40990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40991, SGI0, SGI0, SGI0, SGI40992, SGI40992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9268[] = {SGI40993, SGI0, SGI40994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9269[] = {SGI40995, SGI40996, SGI40997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI40998, SGI0, SGI0, SGI0, SGI40999, SGI40999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9270[] = {SGI41000, SGI41001, SGI41002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41003, SGI0, SGI0, SGI0, SGI41004, SGI41004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9271[] = {SGI41005, SGI41006, SGI41007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41008, SGI0, SGI0, SGI0, SGI41009, SGI41009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9272[] = {SGI41010, SGI41011, SGI41012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41013, SGI0, SGI0, SGI0, SGI41014, SGI41014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9273[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41015, SGI0, SGI0, SGI41016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9274[] = {SGI41017, SGI41018, SGI41019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41020, SGI0, SGI0, SGI0, SGI41021, SGI41021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9275[] = {SGI41022, SGI41023, SGI41024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41025, SGI0, SGI0, SGI0, SGI41026, SGI41026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9276[] = {SGI41027, SGI41028, SGI41029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41030, SGI0, SGI0, SGI0, SGI41031, SGI41031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9277[] = {SGI41032, SGI0, SGI41033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9278[] = {SGI41034, SGI41035, SGI41036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41037, SGI0, SGI0, SGI0, SGI41038, SGI41038, SGI0, SGI0, NULL};
const Segment * const SG9279[] = {SGI41039, SGI41040, SGI41041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41042, SGI0, SGI0, SGI0, SGI41043, SGI41043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9280[] = {SGI41044, SGI41045, SGI41046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41047, SGI0, SGI0, SGI0, SGI41048, SGI41048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9281[] = {SGI41049, SGI41050, SGI41051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41052, SGI0, SGI0, SGI0, SGI41053, SGI41053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9282[] = {SGI41054, SGI0, SGI41055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9283[] = {SGI41056, SGI41057, SGI41058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41059, SGI0, SGI0, SGI0, SGI41060, SGI41060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9284[] = {SGI41061, SGI0, SGI41062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9285[] = {SGI41063, SGI41064, SGI41065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41066, SGI0, SGI0, SGI0, SGI41067, SGI41067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9286[] = {SGI41068, SGI41069, SGI41070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41071, SGI0, SGI0, SGI0, SGI41072, SGI41072, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9287[] = {SGI41073, SGI41074, SGI41075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41076, SGI0, SGI0, SGI0, SGI41077, SGI41077, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9288[] = {SGI41078, SGI41079, SGI41080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41081, SGI0, SGI0, SGI0, SGI41082, SGI41082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9289[] = {SGI41083, SGI41084, SGI41085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41086, SGI0, SGI0, SGI0, SGI41087, SGI41087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9290[] = {SGI41088, SGI41089, SGI41090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41091, SGI0, SGI0, SGI0, SGI41092, SGI41092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9291[] = {SGI41093, SGI41094, SGI41095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41096, SGI0, SGI0, SGI0, SGI41097, SGI41097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9292[] = {SGI41098, SGI41099, SGI41100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41101, SGI0, SGI0, SGI0, SGI41102, SGI41102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9293[] = {SGI41103, SGI41104, SGI41105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41106, SGI0, SGI0, SGI0, SGI41107, SGI41107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9294[] = {SGI41108, SGI0, SGI41109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9295[] = {SGI41110, SGI41111, SGI41112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9296[] = {SGI41113, SGI41114, SGI41115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41116, SGI0, SGI0, SGI0, SGI41117, SGI41117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9297[] = {SGI41118, SGI41119, SGI41120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41121, SGI0, SGI0, SGI0, SGI41122, SGI41122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9298[] = {SGI41123, SGI41124, SGI41125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41126, SGI0, SGI0, SGI0, SGI41127, SGI41127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9299[] = {SGI41128, SGI0, SGI41129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9300[] = {SGI0, SGI41130, SGI41131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41132, SGI0, SGI0, SGI0, SGI41133, SGI41133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9301[] = {SGI0, SGI41134, SGI41135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41136, SGI0, SGI0, SGI0, SGI41137, SGI41137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9302[] = {SGI0, SGI41138, SGI41139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41140, SGI0, SGI0, SGI0, SGI41141, SGI41141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9303[] = {SGI0, SGI41142, SGI41143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41144, SGI0, SGI0, SGI0, SGI41145, SGI41145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9304[] = {SGI0, SGI41146, SGI41147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41148, SGI0, SGI0, SGI0, SGI41149, SGI41149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9305[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41150, SGI0, SGI0, SGI41151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9306[] = {SGI41152, SGI41153, SGI41154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41156, SGI0, SGI0, SGI0, SGI41159, SGI41159, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9307[] = {SGI41155, SGI41157, SGI41158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41160, SGI0, SGI0, SGI0, SGI41161, SGI41161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9308[] = {SGI41162, SGI0, SGI41163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9309[] = {SGI41164, SGI0, SGI41165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9310[] = {SGI41166, SGI41167, SGI41168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41169, SGI0, SGI0, SGI0, SGI41170, SGI41170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9311[] = {SGI41171, SGI41172, SGI41173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41174, SGI0, SGI0, SGI0, SGI41175, SGI41175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9312[] = {SGI41176, SGI41177, SGI41178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41179, SGI41180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9313[] = {SGI41181, SGI41182, SGI41183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41184, SGI41184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9314[] = {SGI41185, SGI41186, SGI41187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41188, SGI0, SGI0, SGI0, SGI41189, SGI41189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9315[] = {SGI41190, SGI41191, SGI41192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41193, SGI0, SGI0, SGI0, SGI41194, SGI41194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9316[] = {SGI41195, SGI0, SGI41196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9317[] = {SGI41197, SGI41198, SGI41199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41200, SGI0, SGI0, SGI0, SGI41201, SGI41201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9318[] = {SGI41202, SGI41203, SGI41204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41205, SGI0, SGI0, SGI0, SGI41206, SGI41206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9319[] = {SGI41207, SGI0, SGI41208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9320[] = {SGI41209, SGI41210, SGI41211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41212, SGI0, SGI0, SGI0, SGI41213, SGI41213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9321[] = {SGI41214, SGI41215, SGI41216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41217, SGI0, SGI0, SGI0, SGI41218, SGI41218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9322[] = {SGI41219, SGI41220, SGI41221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41222, SGI0, SGI0, SGI0, SGI41223, SGI41223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9323[] = {SGI41224, SGI41225, SGI41226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41227, SGI0, SGI0, SGI0, SGI41228, SGI41228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9324[] = {SGI41229, SGI41230, SGI41231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41232, SGI0, SGI0, SGI0, SGI41233, SGI41233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9325[] = {SGI41234, SGI41235, SGI41236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41237, SGI0, SGI0, SGI0, SGI41239, SGI41239, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9326[] = {SGI41238, SGI41240, SGI41241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41242, SGI0, SGI0, SGI0, SGI41243, SGI41243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9327[] = {SGI41244, SGI41245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41246, SGI0, SGI0, SGI0, SGI41247, SGI41247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9328[] = {SGI41248, SGI0, SGI41249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9329[] = {SGI41250, SGI41251, SGI41252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41253, SGI0, SGI0, SGI0, SGI41254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9330[] = {SGI41255, SGI41256, SGI41257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41258, SGI0, SGI0, SGI0, SGI41259, SGI41259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9331[] = {SGI41260, SGI41261, SGI41262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41263, SGI0, SGI0, SGI0, SGI41264, SGI41264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9332[] = {SGI41265, SGI41266, SGI41267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41268, SGI0, SGI0, SGI0, SGI41269, SGI41269, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9333[] = {SGI41270, SGI41271, SGI41272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41273, SGI0, SGI0, SGI0, SGI41274, SGI41274, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9334[] = {SGI41275, SGI41276, SGI41277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41278, SGI0, SGI0, SGI0, SGI41279, SGI41279, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9335[] = {SGI41280, SGI41281, SGI41282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41283, SGI0, SGI0, SGI0, SGI41284, SGI41284, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9336[] = {SGI41285, SGI41286, SGI41287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41288, SGI0, SGI0, SGI0, SGI41289, SGI41289, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9337[] = {SGI41290, SGI41291, SGI41292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41293, SGI0, SGI0, SGI0, SGI41294, SGI41294, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9338[] = {SGI41295, SGI41296, SGI41297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41298, SGI0, SGI0, SGI0, SGI41299, SGI41299, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9339[] = {SGI41300, SGI41301, SGI41302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41303, SGI0, SGI0, SGI0, SGI41304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9340[] = {SGI41305, SGI41306, SGI41307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41308, SGI0, SGI0, SGI0, SGI41309, SGI41309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9341[] = {SGI41310, SGI41311, SGI41312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41313, SGI0, SGI0, SGI0, SGI41314, SGI41314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9342[] = {SGI41315, SGI0, SGI41316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9343[] = {SGI41317, SGI41318, SGI41319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41320, SGI0, SGI0, SGI0, SGI41321, SGI41321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9344[] = {SGI41322, SGI41323, SGI41324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41325, SGI0, SGI0, SGI0, SGI41326, SGI41326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9345[] = {SGI41327, SGI41328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41329, SGI0, SGI0, SGI0, SGI41330, SGI41330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9346[] = {SGI41331, SGI0, SGI41332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9347[] = {SGI41333, SGI41334, SGI41335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41336, SGI0, SGI0, SGI0, SGI41337, SGI41337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9348[] = {SGI41338, SGI41339, SGI41340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41341, SGI0, SGI0, SGI0, SGI41342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9349[] = {SGI41343, SGI41344, SGI41345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41346, SGI0, SGI0, SGI0, SGI41347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9350[] = {SGI41348, SGI0, SGI41349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9351[] = {SGI41350, SGI41351, SGI41352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41353, SGI41353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9352[] = {SGI41354, SGI41355, SGI41356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41357, SGI0, SGI0, SGI0, SGI41358, SGI41358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9353[] = {SGI41359, SGI41360, SGI41361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41362, SGI0, SGI0, SGI0, SGI41363, SGI41363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9354[] = {SGI41364, SGI41365, SGI41366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41367, SGI0, SGI0, SGI0, SGI41368, SGI41368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9355[] = {SGI41369, SGI41370, SGI41371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41372, SGI0, SGI0, SGI0, SGI41373, SGI41373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9356[] = {SGI41374, SGI41375, SGI41376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41377, SGI0, SGI0, SGI0, SGI41378, SGI41378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9357[] = {SGI41379, SGI0, SGI41380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9358[] = {SGI41381, SGI41382, SGI41383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41384, SGI0, SGI0, SGI0, SGI41385, SGI41385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9359[] = {SGI41386, SGI41387, SGI41388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41389, SGI0, SGI0, SGI0, SGI41390, SGI41390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9360[] = {SGI41391, SGI41392, SGI41393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41394, SGI0, SGI0, SGI0, SGI41395, SGI41395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9361[] = {SGI41396, SGI41397, SGI41398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41399, SGI0, SGI0, SGI0, SGI41400, SGI41400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9362[] = {SGI41401, SGI41402, SGI41403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41404, SGI0, SGI0, SGI0, SGI41405, SGI41405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9363[] = {SGI41406, SGI41407, SGI41408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41409, SGI0, SGI0, SGI0, SGI41410, SGI41410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9364[] = {SGI41416, SGI41417, SGI41418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41419, SGI0, SGI0, SGI0, SGI41420, SGI41420, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9365[] = {SGI41421, SGI41422, SGI41423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41424, SGI0, SGI0, SGI0, SGI41425, SGI41425, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9366[] = {SGI41426, SGI0, SGI41427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9367[] = {SGI41428, SGI0, SGI41429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9368[] = {SGI41430, SGI41431, SGI41432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41433, SGI0, SGI0, SGI0, SGI41434, SGI41434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9369[] = {SGI41435, SGI41436, SGI41437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41438, SGI0, SGI0, SGI0, SGI41439, SGI41439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9370[] = {SGI41440, SGI41441, SGI41442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41443, SGI0, SGI0, SGI0, SGI41444, SGI41444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9371[] = {SGI41445, SGI41446, SGI41447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41448, SGI0, SGI0, SGI0, SGI41449, SGI41449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9372[] = {SGI41450, SGI41451, SGI41452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41453, SGI0, SGI0, SGI0, SGI41454, SGI41454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9373[] = {SGI41455, SGI41456, SGI41457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41458, SGI0, SGI0, SGI0, SGI41459, SGI41459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9374[] = {SGI41460, SGI0, SGI41461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9375[] = {SGI41411, SGI41412, SGI41413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41414, SGI0, SGI0, SGI0, SGI41415, SGI41415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9376[] = {SGI41462, SGI41463, SGI41464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41465, SGI0, SGI0, SGI0, SGI41466, SGI41466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9377[] = {SGI41467, SGI41468, SGI41469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41470, SGI0, SGI0, SGI0, SGI41471, SGI41471, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9378[] = {SGI41472, SGI41473, SGI41474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41475, SGI0, SGI0, SGI0, SGI41476, SGI41476, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9379[] = {SGI41477, SGI41478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41479, SGI0, SGI0, SGI0, SGI41481, SGI41481, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9380[] = {SGI41480, SGI41482, SGI41483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41484, SGI0, SGI0, SGI0, SGI41485, SGI41485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9381[] = {SGI41486, SGI41487, SGI41488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41489, SGI0, SGI0, SGI0, SGI41490, SGI41490, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9382[] = {SGI41491, SGI41492, SGI41493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41494, SGI0, SGI0, SGI0, SGI41495, SGI41495, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9383[] = {SGI41496, SGI41497, SGI41498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41499, SGI0, SGI0, SGI0, SGI41500, SGI41500, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9384[] = {SGI41501, SGI41502, SGI41503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41504, SGI0, SGI0, SGI0, SGI41505, SGI41505, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9385[] = {SGI41506, SGI41507, SGI41508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41509, SGI0, SGI0, SGI0, SGI41510, SGI41510, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9386[] = {SGI41511, SGI0, SGI41512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9387[] = {SGI41513, SGI41514, SGI41515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41516, SGI0, SGI0, SGI0, SGI41517, SGI41517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9388[] = {SGI41518, SGI0, SGI41519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9389[] = {SGI41520, SGI41521, SGI41522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41523, SGI0, SGI0, SGI0, SGI41524, SGI41524, SGI0, SGI0, NULL};
const Segment * const SG9390[] = {SGI41525, SGI41526, SGI41527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41528, SGI0, SGI0, SGI0, SGI41529, SGI41529, SGI0, SGI0, NULL};
const Segment * const SG9391[] = {SGI41530, SGI41531, SGI41532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41533, SGI0, SGI0, SGI0, SGI41534, SGI41534, SGI0, SGI0, NULL};
const Segment * const SG9392[] = {SGI41535, SGI41536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41537, SGI0, SGI0, SGI0, SGI41538, SGI41538, SGI0, SGI0, NULL};
const Segment * const SG9393[] = {SGI41539, SGI0, SGI41540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9394[] = {SGI41541, SGI41542, SGI41543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41544, SGI0, SGI0, SGI0, SGI41545, SGI41545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9395[] = {SGI41546, SGI41547, SGI41548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41549, SGI0, SGI0, SGI0, SGI41550, SGI41550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9396[] = {SGI41551, SGI0, SGI41552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9397[] = {SGI41553, SGI41554, SGI41555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41556, SGI0, SGI0, SGI0, SGI41557, SGI41557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9398[] = {SGI41558, SGI41559, SGI41560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41561, SGI0, SGI0, SGI0, SGI41562, SGI41562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9399[] = {SGI41563, SGI41564, SGI41565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41566, SGI0, SGI0, SGI0, SGI41567, SGI41567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9400[] = {SGI41568, SGI41569, SGI41570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41571, SGI0, SGI0, SGI0, SGI41572, SGI41572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9401[] = {SGI41573, SGI0, SGI41574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9402[] = {SGI41575, SGI41576, SGI41577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41578, SGI0, SGI0, SGI0, SGI41579, SGI41579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9403[] = {SGI41580, SGI41581, SGI41582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41583, SGI0, SGI0, SGI0, SGI41584, SGI41584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9404[] = {SGI41585, SGI41586, SGI41587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41588, SGI0, SGI0, SGI0, SGI41589, SGI41589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9405[] = {SGI41590, SGI0, SGI41591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9406[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41592, SGI0, SGI0, SGI41593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9407[] = {SGI41594, SGI41595, SGI41596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41597, SGI0, SGI0, SGI0, SGI41598, SGI41598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9408[] = {SGI41599, SGI41600, SGI41601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41602, SGI0, SGI0, SGI0, SGI41603, SGI41603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9409[] = {SGI41604, SGI0, SGI41605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9410[] = {SGI41606, SGI41607, SGI41608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41609, SGI41609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9411[] = {SGI41610, SGI41611, SGI41612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41613, SGI0, SGI0, SGI0, SGI41614, SGI41614, SGI0, SGI0, NULL};
const Segment * const SG9412[] = {SGI41615, SGI41616, SGI41617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41618, SGI0, SGI0, SGI0, SGI41619, SGI41619, SGI0, SGI0, NULL};
const Segment * const SG9413[] = {SGI41620, SGI0, SGI41621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9414[] = {SGI41622, SGI41623, SGI41624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41625, SGI0, SGI0, SGI0, SGI41626, SGI41626, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9415[] = {SGI41627, SGI41628, SGI41629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41630, SGI0, SGI0, SGI0, SGI41631, SGI41631, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9416[] = {SGI41632, SGI41633, SGI41634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41635, SGI0, SGI0, SGI0, SGI41636, SGI41636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9417[] = {SGI41640, SGI0, SGI41641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9418[] = {SGI41637, SGI41638, SGI41639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41642, SGI0, SGI0, SGI0, SGI41643, SGI41643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9419[] = {SGI41644, SGI0, SGI41645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9420[] = {SGI41646, SGI41647, SGI41648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41649, SGI0, SGI0, SGI0, SGI41650, SGI41650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9421[] = {SGI41651, SGI41652, SGI41653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41654, SGI0, SGI0, SGI0, SGI41655, SGI41655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9422[] = {SGI41656, SGI41657, SGI41658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41659, SGI0, SGI0, SGI0, SGI41660, SGI41660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9423[] = {SGI41661, SGI0, SGI41662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9424[] = {SGI41663, SGI41664, SGI41666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41669, SGI0, SGI0, SGI0, SGI41671, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9425[] = {SGI41665, SGI41667, SGI41668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41670, SGI0, SGI0, SGI0, SGI41672, SGI41672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9426[] = {SGI41673, SGI41674, SGI41675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41676, SGI41676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9427[] = {SGI41677, SGI0, SGI41678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41679, SGI0, SGI0, SGI0, SGI41680, SGI41680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9428[] = {SGI41681, SGI41682, SGI41683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41684, SGI0, SGI0, SGI0, SGI41685, SGI41685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9429[] = {SGI41686, SGI0, SGI41687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9430[] = {SGI41688, SGI41689, SGI41690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41691, SGI0, SGI0, SGI0, SGI41692, SGI41692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9431[] = {SGI41693, SGI0, SGI41694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9432[] = {SGI41695, SGI41696, SGI41697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41698, SGI0, SGI0, SGI0, SGI41699, SGI41699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9433[] = {SGI41700, SGI41701, SGI41702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41703, SGI0, SGI0, SGI0, SGI41704, SGI41704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9434[] = {SGI41705, SGI41706, SGI41707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41708, SGI0, SGI0, SGI0, SGI41709, SGI41709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9435[] = {SGI0, SGI41710, SGI41711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41712, SGI0, SGI0, SGI0, SGI41713, SGI41713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9436[] = {SGI41714, SGI41715, SGI41716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41717, SGI0, SGI0, SGI0, SGI41718, SGI41718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9437[] = {SGI41719, SGI41720, SGI41721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41722, SGI0, SGI0, SGI0, SGI41723, SGI41723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9438[] = {SGI0, SGI41724, SGI41725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41726, SGI0, SGI0, SGI0, SGI41727, SGI41727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9439[] = {SGI41728, SGI0, SGI41729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41730, SGI0, SGI0, SGI0, SGI41731, SGI41731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9440[] = {SGI41732, SGI0, SGI41733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9441[] = {SGI41734, SGI0, SGI41735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9442[] = {SGI41736, SGI0, SGI41737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9443[] = {SGI41738, SGI0, SGI41739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41740, SGI0, SGI0, SGI0, SGI41741, SGI41741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9444[] = {SGI41742, SGI41743, SGI41744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41745, SGI41745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9445[] = {SGI41746, SGI41747, SGI41748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41749, SGI0, SGI0, SGI0, SGI41750, SGI41750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9446[] = {SGI41751, SGI41752, SGI41753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41754, SGI0, SGI0, SGI0, SGI41755, SGI41755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9447[] = {SGI41756, SGI0, SGI41757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41758, SGI0, SGI0, SGI0, SGI41759, SGI41759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9448[] = {SGI41760, SGI41761, SGI41762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9449[] = {SGI41763, SGI0, SGI41764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9450[] = {SGI41765, SGI41766, SGI41767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41768, SGI0, SGI0, SGI0, SGI41769, SGI41769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9451[] = {SGI41770, SGI41771, SGI41772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41775, SGI0, SGI0, SGI0, SGI41777, SGI41777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9452[] = {SGI41773, SGI41774, SGI41776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41778, SGI0, SGI0, SGI0, SGI41779, SGI41779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9453[] = {SGI41780, SGI41781, SGI41782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41783, SGI0, SGI0, SGI0, SGI41784, SGI41784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9454[] = {SGI41785, SGI41786, SGI41787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41788, SGI0, SGI0, SGI0, SGI41789, SGI41789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9455[] = {SGI41790, SGI41791, SGI41792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41793, SGI0, SGI0, SGI0, SGI41794, SGI41794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9456[] = {SGI41795, SGI41796, SGI41797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41798, SGI0, SGI0, SGI0, SGI41799, SGI41799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9457[] = {SGI41800, SGI41801, SGI41802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41803, SGI41804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9458[] = {SGI41805, SGI41806, SGI41807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41808, SGI41808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9459[] = {SGI41809, SGI41810, SGI41811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41812, SGI0, SGI0, SGI0, SGI41813, SGI41813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9460[] = {SGI41814, SGI41815, SGI41816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41817, SGI0, SGI0, SGI0, SGI41818, SGI41818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9461[] = {SGI41819, SGI41820, SGI41821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41822, SGI0, SGI0, SGI0, SGI41823, SGI41823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9462[] = {SGI41824, SGI41825, SGI41826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41827, SGI0, SGI0, SGI0, SGI41828, SGI41828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9463[] = {SGI41829, SGI41830, SGI41831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41832, SGI0, SGI0, SGI0, SGI41833, SGI41833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9464[] = {SGI41834, SGI41835, SGI41836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41837, SGI0, SGI0, SGI0, SGI41838, SGI41838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9465[] = {SGI41839, SGI0, SGI41840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9466[] = {SGI41841, SGI41842, SGI41843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41844, SGI0, SGI0, SGI0, SGI41845, SGI41845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9467[] = {SGI41846, SGI0, SGI41847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9468[] = {SGI41848, SGI41849, SGI41850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41851, SGI0, SGI0, SGI0, SGI41852, SGI41852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9469[] = {SGI41853, SGI41854, SGI41855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41857, SGI0, SGI0, SGI0, SGI41860, SGI41860, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9470[] = {SGI41856, SGI41858, SGI41859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41861, SGI0, SGI0, SGI0, SGI41862, SGI41862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9471[] = {SGI41863, SGI0, SGI41864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9472[] = {SGI41865, SGI41866, SGI41867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41868, SGI0, SGI0, SGI0, SGI41869, SGI41869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9473[] = {SGI41870, SGI41871, SGI41872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41873, SGI0, SGI0, SGI0, SGI41874, SGI41874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9474[] = {SGI41875, SGI0, SGI41876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9475[] = {SGI41877, SGI41878, SGI41879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41880, SGI0, SGI0, SGI0, SGI41881, SGI41881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9476[] = {SGI41882, SGI41883, SGI41884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41885, SGI0, SGI0, SGI0, SGI41886, SGI41886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9477[] = {SGI41887, SGI41888, SGI41889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41890, SGI41891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9478[] = {SGI41892, SGI0, SGI41893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9479[] = {SGI41894, SGI41895, SGI41896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41897, SGI0, SGI0, SGI0, SGI41898, SGI41898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9480[] = {SGI0, SGI41899, SGI41900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41901, SGI0, SGI0, SGI0, SGI41902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9481[] = {SGI41903, SGI41904, SGI41905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41906, SGI41906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9482[] = {SGI41907, SGI41908, SGI41909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41910, SGI41911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9483[] = {SGI41912, SGI41913, SGI41914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41915, SGI0, SGI0, SGI0, SGI41916, SGI41916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9484[] = {SGI41917, SGI41918, SGI41919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41920, SGI0, SGI0, SGI0, SGI41921, SGI41921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9485[] = {SGI41922, SGI41923, SGI41924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41925, SGI41925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9486[] = {SGI41926, SGI41927, SGI41928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41929, SGI0, SGI0, SGI0, SGI41930, SGI41930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9487[] = {SGI41931, SGI41932, SGI41933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41934, SGI0, SGI0, SGI0, SGI41935, SGI41935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9488[] = {SGI41936, SGI41937, SGI41938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41939, SGI0, SGI0, SGI0, SGI41940, SGI41940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9489[] = {SGI41941, SGI41942, SGI41943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41944, SGI0, SGI0, SGI0, SGI41945, SGI41945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9490[] = {SGI41946, SGI41947, SGI41948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41949, SGI0, SGI0, SGI0, SGI41950, SGI41950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9491[] = {SGI41951, SGI41952, SGI41953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41954, SGI0, SGI0, SGI0, SGI41955, SGI41955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9492[] = {SGI41956, SGI41957, SGI41958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41959, SGI0, SGI0, SGI0, SGI41960, SGI41960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9493[] = {SGI41961, SGI41962, SGI41963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41964, SGI0, SGI0, SGI0, SGI41965, SGI41965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9494[] = {SGI41966, SGI41967, SGI41968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41969, SGI0, SGI0, SGI0, SGI41970, SGI41970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9495[] = {SGI41971, SGI41972, SGI41973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41974, SGI41975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9496[] = {SGI41976, SGI41977, SGI41978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41979, SGI0, SGI0, SGI0, SGI41980, SGI41980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9497[] = {SGI41981, SGI0, SGI41982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9498[] = {SGI41983, SGI41984, SGI41985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41986, SGI0, SGI0, SGI0, SGI41987, SGI41987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9499[] = {SGI41988, SGI41989, SGI41990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41991, SGI0, SGI0, SGI0, SGI41992, SGI41992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9500[] = {SGI41993, SGI0, SGI41994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9501[] = {SGI41995, SGI41996, SGI41997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI41998, SGI0, SGI0, SGI0, SGI41999, SGI41999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9502[] = {SGI42000, SGI42001, SGI42002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42003, SGI0, SGI0, SGI0, SGI42004, SGI42004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9503[] = {SGI42005, SGI42006, SGI42007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42008, SGI0, SGI0, SGI0, SGI42009, SGI42009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9504[] = {SGI42010, SGI42011, SGI42012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42013, SGI0, SGI0, SGI0, SGI42014, SGI42014, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9505[] = {SGI42015, SGI0, SGI42016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9506[] = {SGI42017, SGI42018, SGI42019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42020, SGI0, SGI0, SGI0, SGI42021, SGI42021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9507[] = {SGI42022, SGI0, SGI42023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9508[] = {SGI42024, SGI42025, SGI42026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42027, SGI0, SGI0, SGI0, SGI42028, SGI42028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9509[] = {SGI42029, SGI0, SGI42030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9510[] = {SGI42031, SGI0, SGI42032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42033, SGI0, SGI0, SGI0, SGI42034, SGI42034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9511[] = {SGI42035, SGI42036, SGI42037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42038, SGI0, SGI0, SGI0, SGI42039, SGI42039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9512[] = {SGI42040, SGI42041, SGI42042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42043, SGI0, SGI0, SGI0, SGI42044, SGI42044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9513[] = {SGI42045, SGI42046, SGI42047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42048, SGI0, SGI0, SGI0, SGI42049, SGI42049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9514[] = {SGI42050, SGI42051, SGI42052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42053, SGI0, SGI0, SGI0, SGI42054, SGI42054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9515[] = {SGI42055, SGI0, SGI42056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42057, SGI0, SGI0, SGI0, SGI42058, SGI42058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9516[] = {SGI42059, SGI0, SGI42060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9517[] = {SGI42061, SGI42062, SGI42063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42064, SGI0, SGI0, SGI0, SGI42065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9518[] = {SGI42066, SGI42067, SGI42068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42069, SGI0, SGI0, SGI0, SGI42070, SGI42070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9519[] = {SGI42071, SGI42072, SGI42073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42074, SGI0, SGI0, SGI0, SGI42075, SGI42075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9520[] = {SGI42076, SGI42077, SGI42078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42079, SGI0, SGI0, SGI0, SGI42080, SGI42080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9521[] = {SGI42081, SGI42082, SGI42083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42084, SGI0, SGI0, SGI0, SGI42085, SGI42085, SGI0, SGI0, NULL};
const Segment * const SG9522[] = {SGI42086, SGI42087, SGI42088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42089, SGI0, SGI0, SGI0, SGI42090, SGI42090, SGI0, SGI0, NULL};
const Segment * const SG9523[] = {SGI42091, SGI42092, SGI42093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42094, SGI0, SGI0, SGI0, SGI42095, SGI42095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9524[] = {SGI42096, SGI42097, SGI42098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42099, SGI0, SGI0, SGI0, SGI42100, SGI42100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9525[] = {SGI42101, SGI42102, SGI42103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42104, SGI0, SGI0, SGI0, SGI42105, SGI42105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9526[] = {SGI42106, SGI42107, SGI42108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42109, SGI0, SGI0, SGI0, SGI42110, SGI42110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9527[] = {SGI42111, SGI42112, SGI42113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42114, SGI0, SGI0, SGI0, SGI42115, SGI42115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9528[] = {SGI42116, SGI42117, SGI42118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42119, SGI0, SGI0, SGI0, SGI42120, SGI42120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9529[] = {SGI42121, SGI42122, SGI42123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42124, SGI0, SGI0, SGI0, SGI42125, SGI42125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9530[] = {SGI42126, SGI0, SGI42127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9531[] = {SGI42128, SGI42129, SGI42130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42131, SGI0, SGI0, SGI0, SGI42132, SGI42132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9532[] = {SGI42133, SGI0, SGI42134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9533[] = {SGI42135, SGI42136, SGI42137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42138, SGI0, SGI0, SGI0, SGI42139, SGI42139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9534[] = {SGI42140, SGI42141, SGI42142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42143, SGI0, SGI0, SGI0, SGI42144, SGI42144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9535[] = {SGI42145, SGI42146, SGI42147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42148, SGI0, SGI0, SGI0, SGI42149, SGI42149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9536[] = {SGI42150, SGI0, SGI42151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9537[] = {SGI42152, SGI42153, SGI42154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42155, SGI0, SGI0, SGI0, SGI42156, SGI42156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9538[] = {SGI42157, SGI42158, SGI42159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42160, SGI0, SGI0, SGI0, SGI42161, SGI42161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9539[] = {SGI42162, SGI42163, SGI42164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42165, SGI0, SGI0, SGI0, SGI42166, SGI42166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9540[] = {SGI42167, SGI0, SGI42168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9541[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42169, SGI0, SGI0, SGI42170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9542[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42171, SGI0, SGI0, SGI42172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9543[] = {SGI42173, SGI42174, SGI42175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42176, SGI0, SGI0, SGI0, SGI42177, SGI42177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9544[] = {SGI42178, SGI42179, SGI42180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42181, SGI0, SGI0, SGI0, SGI42182, SGI42182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9545[] = {SGI42183, SGI0, SGI42184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9546[] = {SGI42185, SGI42186, SGI42187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42188, SGI0, SGI0, SGI0, SGI42189, SGI42189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9547[] = {SGI42190, SGI42191, SGI42192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42193, SGI0, SGI0, SGI0, SGI42194, SGI42194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9548[] = {SGI42195, SGI42196, SGI42197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42198, SGI0, SGI0, SGI0, SGI42199, SGI42199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9549[] = {SGI42200, SGI42201, SGI42202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42203, SGI0, SGI0, SGI0, SGI42204, SGI42204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9550[] = {SGI42205, SGI42206, SGI42207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42208, SGI0, SGI0, SGI0, SGI42209, SGI42209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9551[] = {SGI42210, SGI42211, SGI42212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42213, SGI0, SGI0, SGI0, SGI42214, SGI42214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9552[] = {SGI42215, SGI42216, SGI42217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42218, SGI0, SGI0, SGI0, SGI42219, SGI42219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9553[] = {SGI42220, SGI0, SGI42221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9554[] = {SGI42222, SGI42223, SGI42224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9555[] = {SGI42225, SGI42226, SGI42227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42228, SGI0, SGI0, SGI0, SGI42229, SGI42229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9556[] = {SGI42230, SGI42231, SGI42232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42234, SGI0, SGI0, SGI0, SGI42237, SGI42237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9557[] = {SGI42233, SGI42235, SGI42236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42238, SGI0, SGI0, SGI0, SGI42239, SGI42239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9558[] = {SGI42240, SGI42241, SGI42242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42244, SGI0, SGI0, SGI0, SGI42247, SGI42247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9559[] = {SGI42243, SGI42245, SGI42246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42248, SGI0, SGI0, SGI0, SGI42249, SGI42249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9560[] = {SGI42252, SGI0, SGI42254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9561[] = {SGI42250, SGI42251, SGI42253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42255, SGI0, SGI0, SGI0, SGI42256, SGI42256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9562[] = {SGI42257, SGI42258, SGI42259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42260, SGI0, SGI0, SGI0, SGI42261, SGI42261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9563[] = {SGI42262, SGI0, SGI42264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9564[] = {SGI42263, SGI42265, SGI42266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42267, SGI42267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9565[] = {SGI42268, SGI0, SGI42269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9566[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42270, SGI0, SGI0, SGI42271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9567[] = {SGI42272, SGI42273, SGI42274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42275, SGI0, SGI0, SGI0, SGI42276, SGI42276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9568[] = {SGI42277, SGI42278, SGI42279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42280, SGI0, SGI0, SGI0, SGI42281, SGI42281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9569[] = {SGI42282, SGI42283, SGI42284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42285, SGI0, SGI0, SGI0, SGI42286, SGI42286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9570[] = {SGI42287, SGI42288, SGI42289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42290, SGI0, SGI0, SGI0, SGI42291, SGI42291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9571[] = {SGI42292, SGI42293, SGI42294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42295, SGI0, SGI0, SGI0, SGI42296, SGI42296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9572[] = {SGI42297, SGI42298, SGI42299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42300, SGI0, SGI0, SGI0, SGI42301, SGI42301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9573[] = {SGI42302, SGI42303, SGI42304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42305, SGI0, SGI0, SGI0, SGI42306, SGI42306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9574[] = {SGI42307, SGI0, SGI42308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9575[] = {SGI42309, SGI42310, SGI42311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42312, SGI0, SGI0, SGI0, SGI42313, SGI42313, SGI0, SGI0, NULL};
const Segment * const SG9576[] = {SGI42314, SGI42315, SGI42316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42317, SGI0, SGI0, SGI0, SGI42318, SGI42318, SGI0, SGI0, NULL};
const Segment * const SG9577[] = {SGI42319, SGI42320, SGI42321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42322, SGI0, SGI0, SGI0, SGI42323, SGI42323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9578[] = {SGI42324, SGI42325, SGI42326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42327, SGI0, SGI0, SGI0, SGI42328, SGI42328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9579[] = {SGI42329, SGI42330, SGI42331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42332, SGI0, SGI0, SGI0, SGI42333, SGI42333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9580[] = {SGI42334, SGI42335, SGI42336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42337, SGI0, SGI0, SGI0, SGI42338, SGI42338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9581[] = {SGI42339, SGI42340, SGI42341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42342, SGI0, SGI0, SGI0, SGI42344, SGI42344, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9582[] = {SGI42343, SGI42345, SGI42346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42347, SGI0, SGI0, SGI0, SGI42348, SGI42348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9583[] = {SGI42349, SGI0, SGI42350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9584[] = {SGI42351, SGI0, SGI42352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9585[] = {SGI42353, SGI42354, SGI42355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42356, SGI0, SGI0, SGI0, SGI42357, SGI42357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9586[] = {SGI42358, SGI42359, SGI42360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42361, SGI0, SGI0, SGI0, SGI42362, SGI42362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9587[] = {SGI42363, SGI42364, SGI42365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42366, SGI0, SGI0, SGI0, SGI42367, SGI42367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9588[] = {SGI42368, SGI42369, SGI42370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42371, SGI42371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9589[] = {SGI42372, SGI42373, SGI42374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42375, SGI0, SGI0, SGI0, SGI42376, SGI42376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9590[] = {SGI42377, SGI42378, SGI42379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42380, SGI0, SGI0, SGI0, SGI42381, SGI42381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9591[] = {SGI42383, SGI42385, SGI42387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42389, SGI0, SGI0, SGI0, SGI42391, SGI42391, SGI0, SGI0, NULL};
const Segment * const SG9592[] = {SGI42382, SGI42384, SGI42386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42388, SGI0, SGI0, SGI0, SGI42390, SGI42390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9593[] = {SGI42392, SGI0, SGI42393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9594[] = {SGI42394, SGI0, SGI42395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9595[] = {SGI42396, SGI42397, SGI42398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42399, SGI0, SGI0, SGI0, SGI42400, SGI42400, SGI0, SGI0, NULL};
const Segment * const SG9596[] = {SGI42401, SGI0, SGI42402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9597[] = {SGI42403, SGI42404, SGI42405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42406, SGI0, SGI0, SGI0, SGI42407, SGI42407, SGI0, SGI0, NULL};
const Segment * const SG9598[] = {SGI42411, SGI0, SGI42412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9599[] = {SGI42408, SGI42409, SGI42410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42413, SGI0, SGI0, SGI0, SGI42414, SGI42414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9600[] = {SGI42415, SGI42416, SGI42417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42418, SGI0, SGI0, SGI0, SGI42419, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9601[] = {SGI42420, SGI42421, SGI42422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42423, SGI42424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9602[] = {SGI42425, SGI42426, SGI42427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42428, SGI0, SGI0, SGI0, SGI42429, SGI42429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9603[] = {SGI42430, SGI0, SGI42431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42432, SGI0, SGI0, SGI0, SGI42433, SGI42433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9604[] = {SGI42434, SGI42435, SGI42436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42437, SGI0, SGI0, SGI0, SGI42438, SGI42438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9605[] = {SGI42439, SGI42440, SGI42441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42442, SGI0, SGI0, SGI0, SGI42443, SGI42443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9606[] = {SGI42444, SGI42445, SGI42446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42447, SGI0, SGI0, SGI0, SGI42448, SGI42448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9607[] = {SGI42449, SGI42450, SGI42451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42452, SGI0, SGI0, SGI0, SGI42453, SGI42453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9608[] = {SGI42454, SGI0, SGI42455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9609[] = {SGI42456, SGI42457, SGI42458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42459, SGI0, SGI0, SGI0, SGI42460, SGI42460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9610[] = {SGI42461, SGI42462, SGI42463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42464, SGI0, SGI0, SGI0, SGI42465, SGI42465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9611[] = {SGI42466, SGI42467, SGI42468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42469, SGI0, SGI0, SGI0, SGI42470, SGI42470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9612[] = {SGI42471, SGI0, SGI42472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9613[] = {SGI0, SGI42473, SGI42474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42475, SGI0, SGI0, SGI0, SGI42476, SGI42476, SGI0, SGI0, NULL};
const Segment * const SG9614[] = {SGI42477, SGI42478, SGI42479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42480, SGI0, SGI0, SGI0, SGI42481, SGI42481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9615[] = {SGI42482, SGI42483, SGI42484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42485, SGI0, SGI0, SGI0, SGI42486, SGI42486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9616[] = {SGI42487, SGI42488, SGI42489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42490, SGI0, SGI0, SGI0, SGI42491, SGI42491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9617[] = {SGI42492, SGI42493, SGI42494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42495, SGI0, SGI0, SGI0, SGI42496, SGI42496, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9618[] = {SGI42497, SGI42498, SGI42499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42500, SGI0, SGI0, SGI0, SGI42501, SGI42501, SGI0, SGI0, NULL};
const Segment * const SG9619[] = {SGI42502, SGI0, SGI42503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9620[] = {SGI42504, SGI42505, SGI42506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42507, SGI0, SGI0, SGI0, SGI42508, SGI42508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9621[] = {SGI42509, SGI42510, SGI42511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42512, SGI0, SGI0, SGI0, SGI42513, SGI42513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9622[] = {SGI42514, SGI0, SGI42515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9623[] = {SGI42516, SGI42517, SGI42518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42519, SGI0, SGI0, SGI0, SGI42520, SGI42520, SGI0, SGI0, NULL};
const Segment * const SG9624[] = {SGI42521, SGI42522, SGI42523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42524, SGI0, SGI0, SGI0, SGI42525, SGI42525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9625[] = {SGI42526, SGI42527, SGI42528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42529, SGI0, SGI0, SGI0, SGI42530, SGI42530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9626[] = {SGI42531, SGI42532, SGI42533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42534, SGI0, SGI0, SGI0, SGI42535, SGI42535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9627[] = {SGI0, SGI42536, SGI42537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42538, SGI0, SGI0, SGI0, SGI42539, SGI42539, SGI0, SGI0, NULL};
const Segment * const SG9628[] = {SGI42540, SGI42541, SGI42542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42543, SGI0, SGI0, SGI0, SGI42544, SGI42544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9629[] = {SGI42545, SGI42546, SGI42547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42548, SGI0, SGI0, SGI0, SGI42549, SGI42549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9630[] = {SGI42550, SGI0, SGI42551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9631[] = {SGI42552, SGI42553, SGI42554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42555, SGI0, SGI0, SGI0, SGI42556, SGI42556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9632[] = {SGI42557, SGI0, SGI42558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9633[] = {SGI42559, SGI42560, SGI42561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42562, SGI0, SGI0, SGI0, SGI42563, SGI42563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9634[] = {SGI42564, SGI42565, SGI42566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42567, SGI0, SGI0, SGI0, SGI42568, SGI42568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9635[] = {SGI42569, SGI42570, SGI42571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42572, SGI0, SGI0, SGI0, SGI42573, SGI42573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9636[] = {SGI42574, SGI42575, SGI42576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42577, SGI0, SGI0, SGI0, SGI42578, SGI42578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9637[] = {SGI42579, SGI42580, SGI42581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42582, SGI0, SGI0, SGI0, SGI42583, SGI42583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9638[] = {SGI42584, SGI42585, SGI42586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42587, SGI0, SGI0, SGI0, SGI42588, SGI42588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9639[] = {SGI42589, SGI42590, SGI42591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42592, SGI0, SGI0, SGI0, SGI42593, SGI42593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9640[] = {SGI42594, SGI42595, SGI42596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42597, SGI0, SGI0, SGI0, SGI42598, SGI42598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9641[] = {SGI42599, SGI42600, SGI42601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42602, SGI0, SGI0, SGI0, SGI42603, SGI42603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9642[] = {SGI42604, SGI42605, SGI42606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42607, SGI0, SGI0, SGI0, SGI42608, SGI42608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9643[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42609, SGI0, SGI0, SGI42610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9644[] = {SGI42611, SGI42612, SGI42613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42614, SGI0, SGI0, SGI0, SGI42615, SGI42615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9645[] = {SGI42616, SGI42617, SGI42618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42619, SGI0, SGI0, SGI0, SGI42620, SGI42620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9646[] = {SGI42621, SGI42622, SGI42623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42624, SGI0, SGI0, SGI0, SGI42625, SGI42625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9647[] = {SGI42626, SGI42627, SGI42628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42629, SGI0, SGI0, SGI0, SGI42630, SGI42630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9648[] = {SGI42631, SGI42632, SGI42633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42634, SGI0, SGI0, SGI0, SGI42635, SGI42635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9649[] = {SGI42636, SGI42637, SGI42638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42639, SGI0, SGI0, SGI0, SGI42640, SGI42640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9650[] = {SGI42641, SGI42642, SGI42643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42644, SGI0, SGI0, SGI0, SGI42645, SGI42645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9651[] = {SGI42646, SGI42647, SGI42648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42649, SGI0, SGI0, SGI0, SGI42650, SGI42650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9652[] = {SGI42651, SGI42652, SGI42653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42654, SGI0, SGI0, SGI0, SGI42655, SGI42655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9653[] = {SGI42656, SGI42657, SGI42658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42659, SGI0, SGI0, SGI0, SGI42660, SGI42660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9654[] = {SGI42661, SGI42662, SGI42663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42664, SGI0, SGI0, SGI0, SGI42665, SGI42665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9655[] = {SGI42666, SGI42667, SGI42668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42669, SGI0, SGI0, SGI0, SGI42670, SGI42670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9656[] = {SGI42671, SGI42672, SGI42673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42674, SGI0, SGI0, SGI0, SGI42675, SGI42675, SGI0, SGI0, NULL};
const Segment * const SG9657[] = {SGI42676, SGI42677, SGI42678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42679, SGI0, SGI0, SGI0, SGI42681, SGI42681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9658[] = {SGI42680, SGI42682, SGI42683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42684, SGI0, SGI0, SGI0, SGI42685, SGI42685, SGI0, SGI0, NULL};
const Segment * const SG9659[] = {SGI42686, SGI42687, SGI42688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42689, SGI0, SGI0, SGI0, SGI42690, SGI42690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9660[] = {SGI42691, SGI42692, SGI42693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42694, SGI0, SGI0, SGI0, SGI42695, SGI42695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9661[] = {SGI42696, SGI42697, SGI42698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42699, SGI0, SGI0, SGI0, SGI42700, SGI42700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9662[] = {SGI42701, SGI0, SGI42702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9663[] = {SGI42703, SGI42704, SGI42705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42706, SGI0, SGI0, SGI0, SGI42707, SGI42707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9664[] = {SGI42708, SGI42709, SGI42710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42711, SGI0, SGI0, SGI0, SGI42712, SGI42712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9665[] = {SGI42713, SGI42714, SGI42715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42716, SGI0, SGI0, SGI0, SGI42717, SGI42717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9666[] = {SGI42718, SGI0, SGI42719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9667[] = {SGI42720, SGI42721, SGI42722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42723, SGI0, SGI0, SGI0, SGI42724, SGI42724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9668[] = {SGI42725, SGI42726, SGI42727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42728, SGI0, SGI0, SGI0, SGI42729, SGI42729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9669[] = {SGI42730, SGI42731, SGI42732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42733, SGI0, SGI0, SGI0, SGI42734, SGI42734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9670[] = {SGI42735, SGI42736, SGI42737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42738, SGI0, SGI0, SGI0, SGI42739, SGI42739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9671[] = {SGI42740, SGI42741, SGI42742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42743, SGI0, SGI0, SGI0, SGI42744, SGI42744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9672[] = {SGI42745, SGI42746, SGI42747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42748, SGI0, SGI0, SGI0, SGI42749, SGI42749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9673[] = {SGI42750, SGI0, SGI42751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9674[] = {SGI42752, SGI42753, SGI42754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42755, SGI0, SGI0, SGI0, SGI42756, SGI42756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9675[] = {SGI42757, SGI42758, SGI42759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42760, SGI0, SGI0, SGI0, SGI42761, SGI42761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9676[] = {SGI42762, SGI42763, SGI42764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42765, SGI0, SGI0, SGI0, SGI42766, SGI42766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9677[] = {SGI42767, SGI42768, SGI42769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42770, SGI0, SGI0, SGI0, SGI42771, SGI42771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9678[] = {SGI42772, SGI42773, SGI42774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42775, SGI0, SGI0, SGI0, SGI42776, SGI42776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9679[] = {SGI42777, SGI42778, SGI42779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42780, SGI0, SGI0, SGI0, SGI42781, SGI42781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9680[] = {SGI42782, SGI0, SGI42783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9681[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42786, SGI0, SGI0, SGI42788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9682[] = {SGI42784, SGI42785, SGI42787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42789, SGI0, SGI0, SGI0, SGI42790, SGI42790, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9683[] = {SGI42791, SGI42792, SGI42793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42794, SGI0, SGI0, SGI0, SGI42795, SGI42795, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9684[] = {SGI42796, SGI0, SGI42797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9685[] = {SGI42798, SGI42799, SGI42800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42801, SGI0, SGI0, SGI0, SGI42802, SGI42802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9686[] = {SGI42803, SGI42804, SGI42805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42806, SGI0, SGI0, SGI0, SGI42807, SGI42807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9687[] = {SGI42808, SGI0, SGI42809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42810, SGI0, SGI0, SGI0, SGI42811, SGI42811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9688[] = {SGI42812, SGI0, SGI42813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9689[] = {SGI42814, SGI42815, SGI42816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42817, SGI0, SGI0, SGI0, SGI42818, SGI42818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9690[] = {SGI42819, SGI42820, SGI42821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42822, SGI0, SGI0, SGI0, SGI42824, SGI42824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9691[] = {SGI42823, SGI42825, SGI42826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42827, SGI0, SGI0, SGI0, SGI42828, SGI42828, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9692[] = {SGI42829, SGI42830, SGI42831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42832, SGI0, SGI0, SGI0, SGI42833, SGI42833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9693[] = {SGI42834, SGI42835, SGI42836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42837, SGI0, SGI0, SGI0, SGI42838, SGI42838, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9694[] = {SGI42839, SGI42840, SGI42841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42842, SGI0, SGI0, SGI0, SGI42844, SGI42844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9695[] = {SGI42843, SGI42845, SGI42846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42847, SGI42848, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9696[] = {SGI42849, SGI0, SGI42850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9697[] = {SGI42851, SGI42852, SGI42853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42854, SGI0, SGI0, SGI0, SGI42855, SGI42855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9698[] = {SGI42856, SGI42857, SGI42858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42859, SGI0, SGI0, SGI0, SGI42860, SGI42860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9699[] = {SGI42861, SGI0, SGI42862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9700[] = {SGI42863, SGI42864, SGI42865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42866, SGI0, SGI0, SGI0, SGI42867, SGI42867, SGI0, SGI0, NULL};
const Segment * const SG9701[] = {SGI42868, SGI42869, SGI42870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42871, SGI0, SGI0, SGI0, SGI42872, SGI42872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9702[] = {SGI42873, SGI42874, SGI42875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42876, SGI0, SGI0, SGI0, SGI42877, SGI42878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9703[] = {SGI42879, SGI0, SGI42880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9704[] = {SGI42881, SGI42882, SGI42883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9705[] = {SGI42884, SGI42885, SGI42886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42887, SGI0, SGI0, SGI0, SGI42888, SGI42888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9706[] = {SGI42889, SGI42890, SGI42891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42892, SGI0, SGI0, SGI0, SGI42893, SGI42893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9707[] = {SGI42894, SGI42895, SGI42896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42897, SGI0, SGI0, SGI0, SGI42898, SGI42898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9708[] = {SGI42899, SGI42900, SGI42901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42902, SGI0, SGI0, SGI0, SGI42903, SGI42903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9709[] = {SGI42904, SGI42905, SGI42906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42907, SGI0, SGI0, SGI0, SGI42908, SGI42908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9710[] = {SGI42909, SGI42910, SGI42911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42912, SGI0, SGI0, SGI0, SGI42913, SGI42913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9711[] = {SGI42914, SGI42915, SGI42916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42917, SGI0, SGI0, SGI0, SGI42918, SGI42918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9712[] = {SGI42919, SGI42920, SGI42921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42922, SGI0, SGI0, SGI0, SGI42925, SGI42925, SGI0, SGI0, NULL};
const Segment * const SG9713[] = {SGI42923, SGI42924, SGI42926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42927, SGI0, SGI0, SGI0, SGI42928, SGI42928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9714[] = {SGI42929, SGI42930, SGI42931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42932, SGI0, SGI0, SGI0, SGI42933, SGI42933, SGI0, SGI0, NULL};
const Segment * const SG9715[] = {SGI42934, SGI42935, SGI42936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42937, SGI0, SGI0, SGI0, SGI42938, SGI42938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9716[] = {SGI42939, SGI42940, SGI42941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42942, SGI0, SGI0, SGI0, SGI42943, SGI42943, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9717[] = {SGI42944, SGI42945, SGI42946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42947, SGI0, SGI0, SGI0, SGI42948, SGI42948, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9718[] = {SGI42949, SGI42950, SGI42951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42952, SGI0, SGI0, SGI0, SGI42953, SGI42953, SGI0, SGI0, NULL};
const Segment * const SG9719[] = {SGI42954, SGI42955, SGI42956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42957, SGI0, SGI0, SGI0, SGI42958, SGI42958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9720[] = {SGI42959, SGI0, SGI42960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9721[] = {SGI42961, SGI42962, SGI42963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9722[] = {SGI42964, SGI0, SGI42965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9723[] = {SGI42966, SGI42967, SGI42968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42969, SGI0, SGI0, SGI0, SGI42970, SGI42970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9724[] = {SGI42971, SGI0, SGI42972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9725[] = {SGI42973, SGI42974, SGI42975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42976, SGI0, SGI0, SGI0, SGI42977, SGI42977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9726[] = {SGI42978, SGI42979, SGI42980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42981, SGI0, SGI0, SGI0, SGI42982, SGI42982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9727[] = {SGI42983, SGI0, SGI42984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9728[] = {SGI42985, SGI42986, SGI42987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42988, SGI0, SGI0, SGI0, SGI42989, SGI42989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9729[] = {SGI42990, SGI42991, SGI42992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42993, SGI0, SGI0, SGI0, SGI42994, SGI42994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9730[] = {SGI42995, SGI42996, SGI42997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI42998, SGI0, SGI0, SGI0, SGI42999, SGI42999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9731[] = {SGI43000, SGI43001, SGI43002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43003, SGI0, SGI0, SGI0, SGI43004, SGI43004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9732[] = {SGI43005, SGI43006, SGI43007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43008, SGI0, SGI0, SGI0, SGI43009, SGI43009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9733[] = {SGI43010, SGI43011, SGI43012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43013, SGI0, SGI0, SGI0, SGI43014, SGI43014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9734[] = {SGI43015, SGI43016, SGI43017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43018, SGI0, SGI0, SGI0, SGI43019, SGI43019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9735[] = {SGI43020, SGI43021, SGI43022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43023, SGI43024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9736[] = {SGI43025, SGI0, SGI43026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9737[] = {SGI43027, SGI0, SGI43028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43029, SGI0, SGI0, SGI0, SGI43030, SGI43030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9738[] = {SGI43031, SGI43032, SGI43033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43034, SGI0, SGI0, SGI0, SGI43035, SGI43035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9739[] = {SGI43036, SGI43037, SGI43038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43039, SGI0, SGI0, SGI0, SGI43040, SGI43040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9740[] = {SGI43042, SGI43043, SGI43044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43045, SGI0, SGI0, SGI0, SGI43046, SGI43046, SGI0, SGI0, SGI0, SGI0, SGI43047, SGI0, SGI0, SGI0, SGI43048, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9741[] = {SGI43049, SGI43050, SGI43051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43052, SGI0, SGI0, SGI0, SGI43053, SGI43053, SGI0, SGI0, SGI0, SGI0, SGI43054, SGI0, SGI0, SGI0, SGI43055, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9742[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43056, SGI0, SGI0, SGI43057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9743[] = {SGI43058, SGI43059, SGI43060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43061, SGI0, SGI0, SGI0, SGI43062, SGI43062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9744[] = {SGI43063, SGI43064, SGI43065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43066, SGI0, SGI0, SGI0, SGI43067, SGI43067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9745[] = {SGI43068, SGI0, SGI43069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9746[] = {SGI43070, SGI43071, SGI43072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43073, SGI0, SGI0, SGI0, SGI43074, SGI43074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9747[] = {SGI43075, SGI43076, SGI43077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43078, SGI0, SGI0, SGI0, SGI43079, SGI43079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9748[] = {SGI43080, SGI43081, SGI43082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43083, SGI0, SGI0, SGI0, SGI43084, SGI43084, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9749[] = {SGI43085, SGI43086, SGI43087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43088, SGI0, SGI0, SGI0, SGI43089, SGI43089, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9750[] = {SGI43090, SGI0, SGI43091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9751[] = {SGI43092, SGI43093, SGI43094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43095, SGI0, SGI0, SGI0, SGI43096, SGI43096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9752[] = {SGI43097, SGI0, SGI43098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9753[] = {SGI43099, SGI43100, SGI43101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43102, SGI0, SGI0, SGI0, SGI43103, SGI43103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9754[] = {SGI43104, SGI0, SGI43105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9755[] = {SGI43106, SGI43107, SGI43108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43109, SGI0, SGI0, SGI0, SGI43110, SGI43110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9756[] = {SGI43111, SGI43112, SGI43113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43114, SGI0, SGI0, SGI0, SGI43115, SGI43115, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9757[] = {SGI43116, SGI43117, SGI43118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43119, SGI0, SGI0, SGI0, SGI43120, SGI43120, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9758[] = {SGI43121, SGI43122, SGI43123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43124, SGI0, SGI0, SGI0, SGI43125, SGI43125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43126, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9759[] = {SGI43127, SGI43128, SGI43129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43130, SGI0, SGI0, SGI0, SGI43131, SGI43131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9760[] = {SGI43132, SGI43133, SGI43134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9761[] = {SGI43135, SGI0, SGI43136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9762[] = {SGI43137, SGI43138, SGI43139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43140, SGI0, SGI0, SGI0, SGI0, SGI43141, SGI0, SGI0, NULL};
const Segment * const SG9763[] = {SGI43142, SGI43143, SGI43144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43145, SGI0, SGI0, SGI0, SGI43146, SGI43146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9764[] = {SGI43147, SGI43148, SGI43149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43150, SGI0, SGI0, SGI0, SGI43151, SGI43151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9765[] = {SGI43152, SGI0, SGI43153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9766[] = {SGI43154, SGI0, SGI43155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43156, SGI0, SGI0, SGI0, SGI43157, SGI43157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9767[] = {SGI43158, SGI0, SGI43159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9768[] = {SGI43160, SGI43161, SGI43162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43163, SGI0, SGI0, SGI0, SGI43164, SGI43164, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9769[] = {SGI43165, SGI0, SGI43166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9770[] = {SGI43167, SGI43168, SGI43169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43170, SGI0, SGI0, SGI0, SGI43171, SGI43171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9771[] = {SGI43172, SGI0, SGI43173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9772[] = {SGI43174, SGI43175, SGI43176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43177, SGI0, SGI0, SGI0, SGI43178, SGI43178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9773[] = {SGI43179, SGI43180, SGI43181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43182, SGI0, SGI0, SGI0, SGI43183, SGI43183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9774[] = {SGI43184, SGI43185, SGI43186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43187, SGI0, SGI0, SGI0, SGI43188, SGI43188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9775[] = {SGI43189, SGI43190, SGI43191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43192, SGI0, SGI0, SGI0, SGI43193, SGI43193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9776[] = {SGI43194, SGI43195, SGI43196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43197, SGI0, SGI0, SGI0, SGI43198, SGI43198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9777[] = {SGI43199, SGI43200, SGI43201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43202, SGI0, SGI0, SGI0, SGI43203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9778[] = {SGI43204, SGI43205, SGI43206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43207, SGI0, SGI0, SGI0, SGI43208, SGI43208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9779[] = {SGI43209, SGI43210, SGI43211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43212, SGI0, SGI0, SGI0, SGI43213, SGI43213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9780[] = {SGI43214, SGI43215, SGI43216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43217, SGI0, SGI0, SGI0, SGI43218, SGI43218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9781[] = {SGI43219, SGI43220, SGI43221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43222, SGI0, SGI0, SGI0, SGI43223, SGI43223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9782[] = {SGI43224, SGI43225, SGI43226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43227, SGI0, SGI0, SGI0, SGI43228, SGI43228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9783[] = {SGI43229, SGI43230, SGI43231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43232, SGI0, SGI0, SGI0, SGI43233, SGI43233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9784[] = {SGI43234, SGI43235, SGI43236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43237, SGI0, SGI0, SGI0, SGI43238, SGI43238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9785[] = {SGI43239, SGI43240, SGI43241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43242, SGI0, SGI0, SGI0, SGI43243, SGI43243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9786[] = {SGI43244, SGI43245, SGI43246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43247, SGI0, SGI0, SGI0, SGI43248, SGI43248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9787[] = {SGI43249, SGI43250, SGI43251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43252, SGI0, SGI0, SGI0, SGI43253, SGI43253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9788[] = {SGI43254, SGI43255, SGI43256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43257, SGI0, SGI0, SGI0, SGI43258, SGI43258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9789[] = {SGI43259, SGI43260, SGI43261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43262, SGI0, SGI0, SGI0, SGI43263, SGI43263, SGI0, SGI0, NULL};
const Segment * const SG9790[] = {SGI43264, SGI0, SGI43265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9791[] = {SGI43266, SGI43267, SGI43268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43269, SGI0, SGI0, SGI0, SGI43270, SGI43270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9792[] = {SGI43271, SGI43272, SGI43273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43274, SGI0, SGI0, SGI0, SGI43275, SGI43275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9793[] = {SGI43276, SGI0, SGI43277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9794[] = {SGI43278, SGI43279, SGI43280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43281, SGI0, SGI0, SGI0, SGI43282, SGI43282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9795[] = {SGI43283, SGI0, SGI43284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9796[] = {SGI43285, SGI43286, SGI43287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43288, SGI0, SGI0, SGI0, SGI43289, SGI43289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9797[] = {SGI43290, SGI43291, SGI43292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43293, SGI0, SGI0, SGI0, SGI43294, SGI43294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9798[] = {SGI43295, SGI43296, SGI43297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43298, SGI0, SGI0, SGI0, SGI43299, SGI43299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9799[] = {SGI43300, SGI43301, SGI43302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43303, SGI0, SGI0, SGI0, SGI43304, SGI43304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9800[] = {SGI43305, SGI43306, SGI43307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43308, SGI0, SGI0, SGI0, SGI43309, SGI43309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9801[] = {SGI43310, SGI43311, SGI43312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43313, SGI0, SGI0, SGI0, SGI43314, SGI43314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9802[] = {SGI43315, SGI43316, SGI43317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43318, SGI0, SGI0, SGI0, SGI43319, SGI43319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9803[] = {SGI43320, SGI0, SGI43321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9804[] = {SGI43322, SGI43323, SGI43324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43325, SGI0, SGI0, SGI0, SGI43326, SGI43326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9805[] = {SGI43327, SGI43328, SGI43329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43330, SGI0, SGI0, SGI0, SGI43331, SGI43331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9806[] = {SGI43332, SGI43333, SGI43334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43335, SGI0, SGI0, SGI0, SGI43336, SGI43336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9807[] = {SGI43337, SGI43338, SGI43339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43340, SGI0, SGI0, SGI0, SGI43341, SGI43341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9808[] = {SGI43342, SGI43343, SGI43344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43345, SGI0, SGI0, SGI0, SGI43346, SGI43346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9809[] = {SGI43347, SGI43348, SGI43349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43350, SGI43351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9810[] = {SGI43352, SGI0, SGI43353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9811[] = {SGI43354, SGI43355, SGI43356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43357, SGI43357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9812[] = {SGI43358, SGI43359, SGI43360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43361, SGI0, SGI0, SGI0, SGI43362, SGI43362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9813[] = {SGI43363, SGI43364, SGI43365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43366, SGI0, SGI0, SGI0, SGI43367, SGI43367, SGI0, SGI0, NULL};
const Segment * const SG9814[] = {SGI43368, SGI43369, SGI43370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43371, SGI0, SGI0, SGI0, SGI43372, SGI43372, SGI0, SGI0, NULL};
const Segment * const SG9815[] = {SGI43373, SGI43374, SGI43375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43376, SGI0, SGI0, SGI0, SGI43377, SGI43377, SGI0, SGI0, NULL};
const Segment * const SG9816[] = {SGI43378, SGI0, SGI43379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9817[] = {SGI43380, SGI43381, SGI43382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43383, SGI0, SGI0, SGI0, SGI43384, SGI43384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9818[] = {SGI43385, SGI43386, SGI43387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43388, SGI0, SGI0, SGI0, SGI43389, SGI43389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9819[] = {SGI43390, SGI43391, SGI43392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43393, SGI0, SGI0, SGI0, SGI43394, SGI43394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9820[] = {SGI43395, SGI43396, SGI43397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43398, SGI0, SGI0, SGI0, SGI43399, SGI43399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9821[] = {SGI43400, SGI43401, SGI43402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43403, SGI0, SGI0, SGI0, SGI43404, SGI43404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9822[] = {SGI43405, SGI43406, SGI43407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43408, SGI0, SGI0, SGI0, SGI43409, SGI43409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9823[] = {SGI43410, SGI0, SGI43411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9824[] = {SGI43412, SGI43413, SGI43414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43415, SGI0, SGI0, SGI0, SGI43416, SGI43416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9825[] = {SGI43417, SGI0, SGI43418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9826[] = {SGI43419, SGI43420, SGI43421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43422, SGI0, SGI0, SGI0, SGI43423, SGI43423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9827[] = {SGI43424, SGI0, SGI43425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9828[] = {SGI43426, SGI43427, SGI43428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43429, SGI0, SGI0, SGI0, SGI43430, SGI43430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9829[] = {SGI43431, SGI43432, SGI43433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43434, SGI0, SGI0, SGI0, SGI43435, SGI43435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9830[] = {SGI43436, SGI43437, SGI43438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43439, SGI0, SGI0, SGI0, SGI43440, SGI43440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9831[] = {SGI43441, SGI0, SGI43442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9832[] = {SGI43443, SGI43444, SGI43445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9833[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43446, SGI0, SGI0, SGI43447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9834[] = {SGI43448, SGI43449, SGI43450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43451, SGI0, SGI0, SGI0, SGI43452, SGI43452, SGI0, SGI0, NULL};
const Segment * const SG9835[] = {SGI43453, SGI43454, SGI43455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43456, SGI0, SGI0, SGI0, SGI43457, SGI43457, SGI0, SGI0, NULL};
const Segment * const SG9836[] = {SGI43458, SGI43459, SGI43460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43461, SGI0, SGI0, SGI0, SGI43462, SGI43462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9837[] = {SGI43463, SGI43464, SGI43465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43466, SGI0, SGI0, SGI0, SGI43467, SGI43467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9838[] = {SGI43468, SGI43469, SGI43470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43471, SGI0, SGI0, SGI0, SGI43472, SGI43472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9839[] = {SGI43473, SGI43474, SGI43475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43476, SGI0, SGI0, SGI0, SGI43477, SGI43477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9840[] = {SGI43478, SGI0, SGI43479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9841[] = {SGI43480, SGI43481, SGI43482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43483, SGI0, SGI0, SGI0, SGI43484, SGI43484, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9842[] = {SGI43485, SGI0, SGI43486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9843[] = {SGI43487, SGI43488, SGI43489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43490, SGI0, SGI0, SGI0, SGI43491, SGI43491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9844[] = {SGI43492, SGI43493, SGI43494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43495, SGI0, SGI0, SGI0, SGI43496, SGI43496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9845[] = {SGI43497, SGI43498, SGI43499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43500, SGI0, SGI0, SGI0, SGI43501, SGI43501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9846[] = {SGI43502, SGI43503, SGI43504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43505, SGI0, SGI0, SGI0, SGI43506, SGI43506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9847[] = {SGI43507, SGI43508, SGI43509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43510, SGI0, SGI0, SGI0, SGI43511, SGI43511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9848[] = {SGI43512, SGI43513, SGI43514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43515, SGI0, SGI0, SGI0, SGI43516, SGI43516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9849[] = {SGI43517, SGI43518, SGI43519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43520, SGI0, SGI0, SGI0, SGI43521, SGI43521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9850[] = {SGI43522, SGI43523, SGI43524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43525, SGI0, SGI0, SGI0, SGI43526, SGI43526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9851[] = {SGI43527, SGI43528, SGI43529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43530, SGI0, SGI0, SGI0, SGI43531, SGI43531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9852[] = {SGI43532, SGI43533, SGI43534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43535, SGI0, SGI0, SGI0, SGI43536, SGI43536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9853[] = {SGI43537, SGI43538, SGI43539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43540, SGI0, SGI0, SGI0, SGI43541, SGI43541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9854[] = {SGI43542, SGI43543, SGI43544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43545, SGI0, SGI0, SGI0, SGI43546, SGI43546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9855[] = {SGI43547, SGI43548, SGI43549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43550, SGI0, SGI0, SGI0, SGI43551, SGI43551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9856[] = {SGI43552, SGI43553, SGI43554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43555, SGI0, SGI0, SGI0, SGI43556, SGI43556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9857[] = {SGI43560, SGI0, SGI43561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9858[] = {SGI43557, SGI43558, SGI43559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43562, SGI0, SGI0, SGI0, SGI43563, SGI43563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9859[] = {SGI43564, SGI43565, SGI43566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43567, SGI0, SGI0, SGI0, SGI43569, SGI43569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9860[] = {SGI43568, SGI43570, SGI43571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43572, SGI0, SGI0, SGI0, SGI43573, SGI43573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9861[] = {SGI43574, SGI43575, SGI43576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9862[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43577, SGI0, SGI0, SGI43578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9863[] = {SGI43579, SGI43580, SGI43581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43582, SGI0, SGI0, SGI0, SGI43583, SGI43583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9864[] = {SGI43584, SGI43585, SGI43586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43587, SGI0, SGI0, SGI0, SGI43588, SGI43588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9865[] = {SGI43589, SGI43590, SGI43591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43592, SGI0, SGI0, SGI0, SGI43593, SGI43593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9866[] = {SGI43594, SGI0, SGI43595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9867[] = {SGI43596, SGI43597, SGI43598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43599, SGI0, SGI0, SGI0, SGI43600, SGI43600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9868[] = {SGI43601, SGI43602, SGI43603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43604, SGI43604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9869[] = {SGI43605, SGI43606, SGI43607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43608, SGI0, SGI0, SGI0, SGI43609, SGI43609, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9870[] = {SGI43610, SGI43611, SGI43612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43613, SGI43614, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9871[] = {SGI43615, SGI43616, SGI43617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43618, SGI43618, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9872[] = {SGI43619, SGI0, SGI43620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9873[] = {SGI43621, SGI43622, SGI43623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43624, SGI0, SGI0, SGI0, SGI43625, SGI43625, SGI0, SGI0, NULL};
const Segment * const SG9874[] = {SGI43626, SGI43627, SGI43628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43629, SGI0, SGI0, SGI0, SGI43630, SGI43630, SGI0, SGI0, NULL};
const Segment * const SG9875[] = {SGI43631, SGI43632, SGI43633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43634, SGI0, SGI0, SGI0, SGI43635, SGI43635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9876[] = {SGI43636, SGI43637, SGI43638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43639, SGI0, SGI0, SGI0, SGI43640, SGI43640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9877[] = {SGI43641, SGI43642, SGI43643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43644, SGI0, SGI0, SGI0, SGI43645, SGI43645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9878[] = {SGI43646, SGI43647, SGI43648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43649, SGI0, SGI0, SGI0, SGI43650, SGI43650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9879[] = {SGI43651, SGI43652, SGI43653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43654, SGI0, SGI0, SGI0, SGI43655, SGI43655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9880[] = {SGI43656, SGI43657, SGI43658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43659, SGI0, SGI0, SGI0, SGI43660, SGI43660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9881[] = {SGI43661, SGI0, SGI43662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9882[] = {SGI43663, SGI43664, SGI43665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43666, SGI0, SGI0, SGI0, SGI43667, SGI43667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9883[] = {SGI43668, SGI0, SGI43669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9884[] = {SGI43670, SGI43671, SGI43672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43673, SGI0, SGI0, SGI0, SGI43674, SGI43674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9885[] = {SGI43675, SGI43676, SGI43677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43678, SGI43679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9886[] = {SGI43680, SGI43681, SGI43682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43683, SGI0, SGI0, SGI0, SGI43684, SGI43684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9887[] = {SGI43685, SGI43686, SGI43687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9888[] = {SGI43688, SGI43689, SGI43690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43691, SGI0, SGI0, SGI0, SGI43692, SGI43692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9889[] = {SGI43693, SGI0, SGI43694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9890[] = {SGI43695, SGI43696, SGI43697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43698, SGI0, SGI0, SGI0, SGI43699, SGI43699, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9891[] = {SGI43700, SGI43701, SGI43702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43703, SGI0, SGI0, SGI0, SGI43704, SGI43704, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9892[] = {SGI43705, SGI0, SGI43706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9893[] = {SGI43707, SGI43708, SGI43709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43710, SGI0, SGI0, SGI0, SGI43711, SGI43711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9894[] = {SGI43712, SGI43713, SGI43714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43715, SGI0, SGI0, SGI0, SGI43716, SGI43716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9895[] = {SGI43717, SGI43718, SGI43719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43720, SGI0, SGI0, SGI0, SGI43721, SGI43721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9896[] = {SGI43722, SGI43723, SGI43724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43725, SGI0, SGI0, SGI0, SGI43726, SGI43726, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9897[] = {SGI43727, SGI43728, SGI43729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43730, SGI0, SGI0, SGI0, SGI43731, SGI43731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9898[] = {SGI43732, SGI43733, SGI43734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43735, SGI0, SGI0, SGI0, SGI43736, SGI43736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9899[] = {SGI43737, SGI43738, SGI43739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43740, SGI0, SGI0, SGI0, SGI43741, SGI43741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9900[] = {SGI43742, SGI43743, SGI43744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43745, SGI0, SGI0, SGI0, SGI43746, SGI43746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9901[] = {SGI43747, SGI43748, SGI43749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43750, SGI0, SGI0, SGI0, SGI43751, SGI43751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9902[] = {SGI43752, SGI43753, SGI43754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43755, SGI0, SGI0, SGI0, SGI43756, SGI43756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9903[] = {SGI43757, SGI0, SGI43758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9904[] = {SGI43759, SGI43760, SGI43761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43762, SGI0, SGI0, SGI0, SGI43763, SGI43763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9905[] = {SGI43764, SGI43765, SGI43766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43767, SGI0, SGI0, SGI0, SGI43768, SGI43768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9906[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43769, SGI0, SGI0, SGI43770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9907[] = {SGI43771, SGI43772, SGI43773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43774, SGI0, SGI0, SGI0, SGI43775, SGI43775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9908[] = {SGI43776, SGI43777, SGI43778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43779, SGI0, SGI0, SGI0, SGI43780, SGI43780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9909[] = {SGI43781, SGI43782, SGI43783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43784, SGI0, SGI0, SGI0, SGI43785, SGI43785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9910[] = {SGI43786, SGI43787, SGI43788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43789, SGI0, SGI0, SGI0, SGI43790, SGI43790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9911[] = {SGI43791, SGI43792, SGI43793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43797, SGI0, SGI0, SGI0, SGI43798, SGI43798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9912[] = {SGI43794, SGI43795, SGI43796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43799, SGI0, SGI0, SGI0, SGI43800, SGI43800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9913[] = {SGI43801, SGI43802, SGI43803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43804, SGI0, SGI0, SGI0, SGI43808, SGI43808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9914[] = {SGI43805, SGI43806, SGI43807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43809, SGI0, SGI0, SGI0, SGI43810, SGI43810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9915[] = {SGI43811, SGI43812, SGI43813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43814, SGI0, SGI0, SGI0, SGI43815, SGI43815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9916[] = {SGI43816, SGI43817, SGI43818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43819, SGI0, SGI0, SGI0, SGI43820, SGI43820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9917[] = {SGI43821, SGI43822, SGI43823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43824, SGI0, SGI0, SGI0, SGI43825, SGI43825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9918[] = {SGI43826, SGI43827, SGI43828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43829, SGI0, SGI0, SGI0, SGI43830, SGI43830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9919[] = {SGI43831, SGI43832, SGI43833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43834, SGI0, SGI0, SGI0, SGI43835, SGI43835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9920[] = {SGI43836, SGI43837, SGI43838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43839, SGI0, SGI0, SGI0, SGI43840, SGI43840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9921[] = {SGI43841, SGI43842, SGI43843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43844, SGI0, SGI0, SGI0, SGI43845, SGI43845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9922[] = {SGI43846, SGI0, SGI43847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43848, SGI0, SGI0, SGI0, SGI43849, SGI43849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9923[] = {SGI43850, SGI0, SGI43851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9924[] = {SGI43852, SGI43853, SGI43854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43855, SGI0, SGI0, SGI0, SGI43856, SGI43856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9925[] = {SGI43857, SGI0, SGI43858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9926[] = {SGI43859, SGI0, SGI43860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43861, SGI0, SGI0, SGI0, SGI43862, SGI43862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9927[] = {SGI43863, SGI43864, SGI43865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43866, SGI0, SGI0, SGI0, SGI43867, SGI43867, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9928[] = {SGI43868, SGI43869, SGI43870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43871, SGI0, SGI0, SGI0, SGI43872, SGI43872, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9929[] = {SGI43873, SGI43874, SGI43875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43876, SGI0, SGI0, SGI0, SGI43877, SGI43877, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9930[] = {SGI43878, SGI0, SGI43879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9931[] = {SGI43880, SGI43881, SGI43882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43883, SGI0, SGI0, SGI0, SGI43884, SGI43884, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9932[] = {SGI43885, SGI43886, SGI43887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43888, SGI0, SGI0, SGI0, SGI43889, SGI43889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9933[] = {SGI43890, SGI43891, SGI43892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43893, SGI0, SGI0, SGI0, SGI43894, SGI43894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9934[] = {SGI43895, SGI0, SGI43896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9935[] = {SGI43897, SGI43898, SGI43899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43900, SGI43900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9936[] = {SGI43901, SGI43902, SGI43903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43904, SGI0, SGI0, SGI0, SGI43905, SGI43905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9937[] = {SGI43906, SGI0, SGI43907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9938[] = {SGI43908, SGI43909, SGI43910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43911, SGI0, SGI0, SGI0, SGI43912, SGI43912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9939[] = {SGI43913, SGI43914, SGI43915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43916, SGI0, SGI0, SGI0, SGI43917, SGI43917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9940[] = {SGI43918, SGI43919, SGI43920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43921, SGI0, SGI0, SGI0, SGI43922, SGI43922, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9941[] = {SGI43923, SGI43924, SGI43925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43926, SGI0, SGI0, SGI0, SGI43927, SGI43927, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9942[] = {SGI43928, SGI43929, SGI43930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43931, SGI0, SGI0, SGI0, SGI43932, SGI43932, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9943[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43936, SGI0, SGI0, SGI43937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9944[] = {SGI43933, SGI43934, SGI43935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43938, SGI0, SGI0, SGI0, SGI43939, SGI43939, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9945[] = {SGI43945, SGI0, SGI43946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43947, SGI43948, SGI43948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43949, NULL};
const Segment * const SG9946[] = {SGI43950, SGI0, SGI43951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43952, SGI43953, SGI43953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43954, NULL};
const Segment * const SG9947[] = {SGI43955, SGI0, SGI43956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43957, SGI43958, SGI43958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43959, NULL};
const Segment * const SG9948[] = {SGI43960, SGI43961, SGI43962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43963, SGI0, SGI0, SGI0, SGI43964, SGI43964, SGI0, SGI0, NULL};
const Segment * const SG9949[] = {SGI43965, SGI43966, SGI43967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43968, SGI0, SGI0, SGI0, SGI43969, SGI43969, SGI0, SGI0, NULL};
const Segment * const SG9950[] = {SGI43970, SGI0, SGI43971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9951[] = {SGI43972, SGI0, SGI43973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9952[] = {SGI43974, SGI43975, SGI43976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43977, SGI0, SGI0, SGI0, SGI43978, SGI43978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9953[] = {SGI43979, SGI0, SGI43980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9954[] = {SGI43940, SGI43941, SGI43942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43943, SGI0, SGI0, SGI0, SGI43944, SGI43944, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9955[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43981, SGI0, SGI0, SGI43982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9956[] = {SGI43983, SGI43984, SGI43985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43986, SGI0, SGI0, SGI0, SGI43987, SGI43987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9957[] = {SGI43988, SGI43989, SGI43990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43991, SGI0, SGI0, SGI0, SGI43992, SGI43992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9958[] = {SGI43993, SGI0, SGI43994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9959[] = {SGI43995, SGI43996, SGI43997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI43998, SGI0, SGI0, SGI0, SGI43999, SGI43999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9960[] = {SGI44000, SGI44001, SGI44002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44003, SGI0, SGI0, SGI0, SGI44004, SGI44004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9961[] = {SGI44005, SGI0, SGI44006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9962[] = {SGI44007, SGI44008, SGI44009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44010, SGI0, SGI0, SGI0, SGI44011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9963[] = {SGI44012, SGI44013, SGI44014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44015, SGI0, SGI0, SGI0, SGI44016, SGI44016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9964[] = {SGI44017, SGI44018, SGI44019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44020, SGI0, SGI0, SGI0, SGI44021, SGI44021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9965[] = {SGI44022, SGI44023, SGI44024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44025, SGI0, SGI0, SGI0, SGI44026, SGI44026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9966[] = {SGI44027, SGI0, SGI44028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9967[] = {SGI44029, SGI44030, SGI44031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44032, SGI0, SGI0, SGI0, SGI44033, SGI44033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9968[] = {SGI44034, SGI44035, SGI44036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44037, SGI0, SGI0, SGI0, SGI44038, SGI44038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9969[] = {SGI44039, SGI44040, SGI44041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44042, SGI0, SGI0, SGI0, SGI44043, SGI44043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9970[] = {SGI44044, SGI44045, SGI44046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44047, SGI0, SGI0, SGI0, SGI44048, SGI44048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9971[] = {SGI44049, SGI44050, SGI44051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44052, SGI0, SGI0, SGI0, SGI44053, SGI44053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9972[] = {SGI44054, SGI44055, SGI44056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44057, SGI0, SGI0, SGI0, SGI44058, SGI44058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9973[] = {SGI44059, SGI44060, SGI44061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44062, SGI0, SGI0, SGI0, SGI44063, SGI44063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9974[] = {SGI44064, SGI0, SGI44065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9975[] = {SGI44066, SGI44067, SGI44068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44069, SGI0, SGI0, SGI0, SGI44070, SGI44070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9976[] = {SGI44071, SGI44072, SGI44073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44074, SGI44075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9977[] = {SGI44076, SGI44077, SGI44078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44079, SGI0, SGI0, SGI0, SGI44080, SGI44080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9978[] = {SGI44081, SGI44082, SGI44083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44084, SGI0, SGI0, SGI0, SGI44085, SGI44085, SGI0, SGI0, NULL};
const Segment * const SG9979[] = {SGI44086, SGI44087, SGI44088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44089, SGI0, SGI0, SGI0, SGI44090, SGI44090, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9980[] = {SGI44091, SGI44092, SGI44093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44094, SGI0, SGI0, SGI0, SGI44095, SGI44095, SGI0, SGI0, NULL};
const Segment * const SG9981[] = {SGI44096, SGI0, SGI44097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9982[] = {SGI44098, SGI44099, SGI44100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44101, SGI0, SGI0, SGI0, SGI44102, SGI44102, SGI0, SGI0, NULL};
const Segment * const SG9983[] = {SGI44103, SGI44104, SGI44105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44106, SGI0, SGI0, SGI0, SGI44107, SGI44107, SGI0, SGI0, NULL};
const Segment * const SG9984[] = {SGI44108, SGI44109, SGI44110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44111, SGI0, SGI0, SGI0, SGI44112, SGI44112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9985[] = {SGI44113, SGI0, SGI44114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9986[] = {SGI44115, SGI44116, SGI44117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44118, SGI0, SGI0, SGI0, SGI44119, SGI44119, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9987[] = {SGI44120, SGI0, SGI44121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9988[] = {SGI44122, SGI0, SGI44123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44124, SGI0, SGI0, SGI0, SGI44125, SGI44125, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9989[] = {SGI44126, SGI44127, SGI44128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44129, SGI0, SGI0, SGI0, SGI44130, SGI44130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9990[] = {SGI44131, SGI0, SGI44132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9991[] = {SGI44133, SGI44134, SGI44135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44136, SGI44137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9992[] = {SGI44138, SGI44139, SGI44140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44141, SGI44141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9993[] = {SGI44142, SGI44143, SGI44144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44145, SGI0, SGI0, SGI0, SGI44146, SGI44146, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9994[] = {SGI44147, SGI0, SGI44148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9995[] = {SGI44149, SGI44150, SGI44151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44152, SGI0, SGI0, SGI0, SGI44153, SGI44153, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9996[] = {SGI44154, SGI0, SGI44155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9997[] = {SGI0, SGI44156, SGI44157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44158, SGI0, SGI0, SGI0, SGI44159, SGI44159, SGI0, SGI0, NULL};
const Segment * const SG9998[] = {SGI44160, SGI44161, SGI44162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44163, SGI0, SGI0, SGI0, SGI44164, SGI44164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG9999[] = {SGI44165, SGI0, SGI44166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10000[] = {SGI44167, SGI44168, SGI44169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44170, SGI0, SGI0, SGI0, SGI44171, SGI44171, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10001[] = {SGI44172, SGI44173, SGI44174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44175, SGI0, SGI0, SGI0, SGI44176, SGI44176, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10002[] = {SGI44177, SGI44178, SGI44179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44180, SGI0, SGI0, SGI0, SGI44181, SGI44181, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10003[] = {SGI44183, SGI44184, SGI44185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44186, SGI0, SGI0, SGI0, SGI44187, SGI44187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10004[] = {SGI44188, SGI0, SGI44189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10005[] = {SGI44182, SGI0, SGI44190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10006[] = {SGI44191, SGI0, SGI44192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10007[] = {SGI44193, SGI44194, SGI44195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44196, SGI0, SGI0, SGI0, SGI44197, SGI44197, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10008[] = {SGI44198, SGI44199, SGI44200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44201, SGI0, SGI0, SGI0, SGI44202, SGI44202, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10009[] = {SGI44203, SGI44204, SGI44205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44206, SGI0, SGI0, SGI0, SGI44207, SGI44207, SGI0, SGI0, NULL};
const Segment * const SG10010[] = {SGI44208, SGI44209, SGI44210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44211, SGI0, SGI0, SGI0, SGI44212, SGI44212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10011[] = {SGI44213, SGI0, SGI44214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10012[] = {SGI44215, SGI44216, SGI44217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44218, SGI44219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10013[] = {SGI44220, SGI44221, SGI44222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44223, SGI44223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10014[] = {SGI44224, SGI44225, SGI44226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44227, SGI0, SGI0, SGI0, SGI44228, SGI44228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10015[] = {SGI44229, SGI44230, SGI44231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44232, SGI0, SGI0, SGI0, SGI44233, SGI44233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10016[] = {SGI44234, SGI0, SGI44235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10017[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44236, SGI0, SGI0, SGI44237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10018[] = {SGI44238, SGI44239, SGI44240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44241, SGI0, SGI0, SGI0, SGI44242, SGI44242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10019[] = {SGI44243, SGI44244, SGI44245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44246, SGI0, SGI0, SGI0, SGI44247, SGI44247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10020[] = {SGI44248, SGI0, SGI44249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10021[] = {SGI44250, SGI0, SGI44251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44252, SGI0, SGI0, SGI0, SGI44253, SGI44253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10022[] = {SGI44254, SGI44255, SGI44256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44257, SGI0, SGI0, SGI0, SGI44258, SGI44258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10023[] = {SGI44259, SGI0, SGI44260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10024[] = {SGI44261, SGI44262, SGI44263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44264, SGI0, SGI0, SGI0, SGI44265, SGI44265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10025[] = {SGI44266, SGI44267, SGI44268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44269, SGI0, SGI0, SGI0, SGI44270, SGI44270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10026[] = {SGI44271, SGI44272, SGI44273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44274, SGI0, SGI0, SGI0, SGI44275, SGI44275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10027[] = {SGI44276, SGI0, SGI44277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44278, SGI44279, SGI44279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44280, NULL};
const Segment * const SG10028[] = {SGI44281, SGI44282, SGI44283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44284, SGI0, SGI0, SGI0, SGI44285, SGI44285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10029[] = {SGI44286, SGI0, SGI44287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10030[] = {SGI44288, SGI0, SGI44289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44290, SGI0, SGI0, SGI0, SGI44291, SGI44291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10031[] = {SGI44292, SGI44293, SGI44294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44295, SGI0, SGI0, SGI0, SGI44296, SGI44296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10032[] = {SGI44297, SGI44298, SGI44299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44300, SGI0, SGI0, SGI0, SGI44301, SGI44301, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10033[] = {SGI44302, SGI44303, SGI44304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44305, SGI0, SGI0, SGI0, SGI44306, SGI44306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10034[] = {SGI44307, SGI44308, SGI44309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44310, SGI0, SGI0, SGI0, SGI44311, SGI44311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10035[] = {SGI44312, SGI0, SGI44313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10036[] = {SGI44314, SGI44315, SGI44316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44317, SGI0, SGI0, SGI0, SGI44318, SGI44318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10037[] = {SGI44319, SGI44320, SGI44321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44322, SGI0, SGI0, SGI0, SGI44323, SGI44323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10038[] = {SGI44324, SGI44325, SGI44326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44327, SGI0, SGI0, SGI0, SGI44328, SGI44328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10039[] = {SGI44329, SGI44330, SGI44331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44332, SGI0, SGI0, SGI0, SGI44333, SGI44333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10040[] = {SGI44334, SGI44335, SGI44336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44337, SGI0, SGI0, SGI0, SGI44338, SGI44338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10041[] = {SGI44339, SGI44340, SGI44341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44342, SGI0, SGI0, SGI0, SGI44343, SGI44343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10042[] = {SGI44344, SGI44345, SGI44346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44347, SGI0, SGI0, SGI0, SGI44348, SGI44348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10043[] = {SGI44349, SGI44350, SGI44351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44352, SGI0, SGI0, SGI0, SGI44353, SGI44353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10044[] = {SGI44354, SGI44355, SGI44356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44357, SGI0, SGI0, SGI0, SGI44358, SGI44358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10045[] = {SGI44359, SGI0, SGI44360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10046[] = {SGI44361, SGI44362, SGI44363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44364, SGI0, SGI0, SGI0, SGI44365, SGI44365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10047[] = {SGI44366, SGI44367, SGI44368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44369, SGI0, SGI0, SGI0, SGI44370, SGI44370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10048[] = {SGI44371, SGI0, SGI44372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10049[] = {SGI44373, SGI0, SGI44374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10050[] = {SGI44375, SGI44376, SGI44377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44378, SGI0, SGI0, SGI0, SGI44379, SGI44379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10051[] = {SGI44380, SGI44381, SGI44382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44383, SGI0, SGI0, SGI0, SGI44384, SGI44384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10052[] = {SGI44385, SGI44386, SGI44387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44388, SGI0, SGI0, SGI0, SGI44389, SGI44389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10053[] = {SGI44390, SGI0, SGI44391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10054[] = {SGI44392, SGI44393, SGI44394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44395, SGI0, SGI0, SGI0, SGI44396, SGI44396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10055[] = {SGI44397, SGI44398, SGI44399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44400, SGI0, SGI0, SGI0, SGI44401, SGI44401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10056[] = {SGI44402, SGI44403, SGI44404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44405, SGI0, SGI0, SGI0, SGI44406, SGI44406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10057[] = {SGI44407, SGI44408, SGI44409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44410, SGI0, SGI0, SGI0, SGI44411, SGI44411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10058[] = {SGI44412, SGI0, SGI44413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10059[] = {SGI0, SGI44414, SGI44415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44416, SGI44416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10060[] = {SGI44417, SGI44418, SGI44419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44420, SGI44421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10061[] = {SGI44422, SGI44423, SGI44424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44425, SGI44425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10062[] = {SGI44426, SGI0, SGI44427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10063[] = {SGI0, SGI44428, SGI44429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10064[] = {SGI44430, SGI44431, SGI44432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44433, SGI0, SGI0, SGI0, SGI44434, SGI44434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10065[] = {SGI44435, SGI44436, SGI44437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44438, SGI0, SGI0, SGI0, SGI44439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10066[] = {SGI44440, SGI44441, SGI44442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44443, SGI0, SGI0, SGI0, SGI44444, SGI44444, SGI0, SGI0, NULL};
const Segment * const SG10067[] = {SGI44445, SGI44446, SGI44447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44448, SGI0, SGI0, SGI0, SGI44449, SGI44449, SGI0, SGI0, NULL};
const Segment * const SG10068[] = {SGI44450, SGI44451, SGI44452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44453, SGI0, SGI0, SGI0, SGI44454, SGI44454, SGI0, SGI0, NULL};
const Segment * const SG10069[] = {SGI44455, SGI44456, SGI44457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44458, SGI0, SGI0, SGI0, SGI44459, SGI44459, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10070[] = {SGI44460, SGI44461, SGI44462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44463, SGI0, SGI0, SGI0, SGI44464, SGI44464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10071[] = {SGI44465, SGI44466, SGI44467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10072[] = {SGI44468, SGI0, SGI44469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10073[] = {SGI44470, SGI44471, SGI44472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44473, SGI0, SGI0, SGI0, SGI44474, SGI44474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10074[] = {SGI44475, SGI44476, SGI44477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44478, SGI0, SGI0, SGI0, SGI44479, SGI44479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10075[] = {SGI44480, SGI0, SGI44481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10076[] = {SGI44482, SGI44483, SGI44484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44485, SGI0, SGI0, SGI0, SGI44486, SGI44486, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10077[] = {SGI44487, SGI44488, SGI44489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44490, SGI0, SGI0, SGI0, SGI44491, SGI44491, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10078[] = {SGI44492, SGI44493, SGI44494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44495, SGI0, SGI0, SGI0, SGI44496, SGI44496, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10079[] = {SGI44497, SGI44498, SGI44499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44500, SGI0, SGI0, SGI0, SGI44501, SGI44501, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10080[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44502, SGI0, SGI0, SGI44503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10081[] = {SGI44504, SGI44505, SGI44506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44507, SGI0, SGI0, SGI0, SGI44508, SGI44508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10082[] = {SGI44509, SGI44510, SGI44511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44512, SGI0, SGI0, SGI0, SGI44513, SGI44513, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10083[] = {SGI44514, SGI0, SGI44515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10084[] = {SGI44516, SGI44517, SGI44518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44519, SGI0, SGI0, SGI0, SGI44520, SGI44520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10085[] = {SGI44521, SGI44522, SGI44523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44524, SGI0, SGI0, SGI0, SGI44525, SGI44525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10086[] = {SGI44526, SGI0, SGI44527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10087[] = {SGI44528, SGI44529, SGI44530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44531, SGI0, SGI0, SGI0, SGI44532, SGI44532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10088[] = {SGI44533, SGI44534, SGI44535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10089[] = {SGI44536, SGI0, SGI44537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10090[] = {SGI44538, SGI44539, SGI44540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44541, SGI0, SGI0, SGI0, SGI44542, SGI44542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10091[] = {SGI44543, SGI44544, SGI44545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44546, SGI0, SGI0, SGI0, SGI44547, SGI44547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10092[] = {SGI44548, SGI0, SGI44549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44550, SGI44551, SGI44551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44552, NULL};
const Segment * const SG10093[] = {SGI44553, SGI44554, SGI44555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44556, SGI0, SGI0, SGI0, SGI44557, SGI44557, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10094[] = {SGI44558, SGI0, SGI44559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44560, SGI44561, SGI44561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44562, NULL};
const Segment * const SG10095[] = {SGI44563, SGI44564, SGI44565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44566, SGI0, SGI0, SGI0, SGI44567, SGI44567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10096[] = {SGI44568, SGI0, SGI44569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10097[] = {SGI44570, SGI44571, SGI44572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44573, SGI0, SGI0, SGI0, SGI44574, SGI44574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10098[] = {SGI44575, SGI44576, SGI44577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44578, SGI0, SGI0, SGI0, SGI44579, SGI44579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10099[] = {SGI44580, SGI44581, SGI44582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44583, SGI0, SGI0, SGI0, SGI44584, SGI44584, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10100[] = {SGI44585, SGI44586, SGI44587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44588, SGI0, SGI0, SGI0, SGI44589, SGI44589, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10101[] = {SGI44590, SGI0, SGI44591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10102[] = {SGI44592, SGI44593, SGI44594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44595, SGI0, SGI0, SGI0, SGI44596, SGI44596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10103[] = {SGI44597, SGI0, SGI44598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10104[] = {SGI44599, SGI44600, SGI44601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44602, SGI0, SGI0, SGI0, SGI44603, SGI44603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10105[] = {SGI44604, SGI44605, SGI44606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44607, SGI0, SGI0, SGI0, SGI44608, SGI44608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10106[] = {SGI44609, SGI44610, SGI44611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44612, SGI0, SGI0, SGI0, SGI44613, SGI44613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10107[] = {SGI44614, SGI44615, SGI44616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44617, SGI0, SGI0, SGI0, SGI44618, SGI44618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10108[] = {SGI44619, SGI44620, SGI44621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44622, SGI0, SGI0, SGI0, SGI44623, SGI44623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10109[] = {SGI44624, SGI44625, SGI44626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44627, SGI0, SGI0, SGI0, SGI44628, SGI44628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10110[] = {SGI44629, SGI44630, SGI44631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44632, SGI0, SGI0, SGI0, SGI44633, SGI44633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10111[] = {SGI44634, SGI44635, SGI44636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44637, SGI0, SGI0, SGI0, SGI44638, SGI44638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10112[] = {SGI44639, SGI44640, SGI44641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44642, SGI0, SGI0, SGI0, SGI44643, SGI44643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10113[] = {SGI44644, SGI44645, SGI44646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44647, SGI0, SGI0, SGI0, SGI44648, SGI44648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10114[] = {SGI44649, SGI44650, SGI44651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44652, SGI0, SGI0, SGI0, SGI44653, SGI44653, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10115[] = {SGI44654, SGI44655, SGI44656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44657, SGI0, SGI0, SGI0, SGI44658, SGI44658, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10116[] = {SGI44659, SGI44660, SGI44661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44662, SGI44663, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10117[] = {SGI44664, SGI44665, SGI44666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44667, SGI44667, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10118[] = {SGI44668, SGI0, SGI44669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10119[] = {SGI44670, SGI44671, SGI44672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44673, SGI0, SGI0, SGI0, SGI44674, SGI44674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10120[] = {SGI44675, SGI44676, SGI44677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44678, SGI0, SGI0, SGI0, SGI44679, SGI44679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10121[] = {SGI44680, SGI44681, SGI44682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44683, SGI44684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10122[] = {SGI44685, SGI0, SGI44686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10123[] = {SGI44687, SGI44688, SGI44689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44690, SGI0, SGI0, SGI0, SGI44691, SGI44691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10124[] = {SGI44692, SGI44693, SGI44694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44695, SGI0, SGI0, SGI0, SGI44696, SGI44696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10125[] = {SGI44697, SGI44698, SGI44699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44700, SGI0, SGI0, SGI0, SGI44701, SGI44701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10126[] = {SGI44702, SGI44703, SGI44704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44705, SGI0, SGI0, SGI0, SGI44706, SGI44706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10127[] = {SGI44707, SGI0, SGI44708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10128[] = {SGI44709, SGI0, SGI44710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10129[] = {SGI44711, SGI44712, SGI44713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44714, SGI0, SGI0, SGI0, SGI44715, SGI44715, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10130[] = {SGI44716, SGI44717, SGI44718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44719, SGI0, SGI0, SGI0, SGI44720, SGI44720, SGI0, SGI0, NULL};
const Segment * const SG10131[] = {SGI44721, SGI44722, SGI44723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44724, SGI0, SGI0, SGI0, SGI44725, SGI44725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10132[] = {SGI44726, SGI0, SGI44727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10133[] = {SGI44728, SGI44729, SGI44730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44731, SGI0, SGI0, SGI0, SGI44732, SGI44732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10134[] = {SGI44733, SGI44734, SGI44735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44736, SGI44737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10135[] = {SGI44738, SGI44739, SGI44740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44741, SGI44742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10136[] = {SGI44743, SGI44744, SGI44745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44746, SGI0, SGI0, SGI0, SGI44747, SGI44747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10137[] = {SGI44748, SGI0, SGI44749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10138[] = {SGI44750, SGI44751, SGI44752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44753, SGI44753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10139[] = {SGI44754, SGI44755, SGI44756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44757, SGI0, SGI0, SGI0, SGI44758, SGI44758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10140[] = {SGI44759, SGI0, SGI44760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10141[] = {SGI44761, SGI44762, SGI44763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44764, SGI0, SGI0, SGI0, SGI44765, SGI44765, SGI0, SGI0, NULL};
const Segment * const SG10142[] = {SGI44766, SGI44767, SGI44768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44769, SGI0, SGI0, SGI0, SGI44770, SGI44770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10143[] = {SGI44771, SGI44772, SGI44773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44774, SGI44775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10144[] = {SGI44776, SGI44777, SGI44778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44779, SGI44779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10145[] = {SGI44780, SGI0, SGI44781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10146[] = {SGI0, SGI44782, SGI44783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10147[] = {SGI44784, SGI44785, SGI44786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44787, SGI0, SGI0, SGI0, SGI44788, SGI44788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10148[] = {SGI44789, SGI44790, SGI44791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44792, SGI0, SGI0, SGI0, SGI44793, SGI44793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10149[] = {SGI44794, SGI0, SGI44795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10150[] = {SGI44796, SGI44797, SGI44798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44799, SGI0, SGI0, SGI0, SGI44800, SGI44800, SGI0, SGI0, NULL};
const Segment * const SG10151[] = {SGI44801, SGI44802, SGI44803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44804, SGI0, SGI0, SGI0, SGI44805, SGI44805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10152[] = {SGI44806, SGI0, SGI44807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10153[] = {SGI44808, SGI0, SGI44809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44810, SGI0, SGI0, SGI0, SGI44811, SGI44811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10154[] = {SGI44812, SGI44813, SGI44814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44815, SGI44815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10155[] = {SGI44816, SGI44817, SGI44818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44819, SGI0, SGI0, SGI0, SGI44820, SGI44820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10156[] = {SGI44821, SGI44822, SGI44823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44824, SGI0, SGI0, SGI0, SGI44825, SGI44825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10157[] = {SGI44826, SGI44827, SGI44828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44829, SGI0, SGI0, SGI0, SGI44830, SGI44830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10158[] = {SGI44831, SGI44832, SGI44833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44834, SGI0, SGI0, SGI0, SGI44835, SGI44835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10159[] = {SGI44836, SGI44837, SGI44838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44839, SGI0, SGI0, SGI0, SGI44840, SGI44840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10160[] = {SGI44841, SGI44842, SGI44843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44844, SGI0, SGI0, SGI0, SGI44845, SGI44845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10161[] = {SGI44846, SGI44847, SGI44848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44849, SGI0, SGI0, SGI0, SGI44850, SGI44850, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10162[] = {SGI44851, SGI44852, SGI44853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44854, SGI0, SGI0, SGI0, SGI44855, SGI44855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10163[] = {SGI44856, SGI0, SGI44857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10164[] = {SGI44858, SGI44859, SGI44860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44862, SGI0, SGI0, SGI0, SGI44865, SGI44865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10165[] = {SGI44861, SGI44863, SGI44864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44866, SGI44867, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10166[] = {SGI44868, SGI44869, SGI44870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44871, SGI0, SGI0, SGI0, SGI44872, SGI44872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10167[] = {SGI44873, SGI44874, SGI44875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44876, SGI0, SGI0, SGI0, SGI44877, SGI44877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10168[] = {SGI44878, SGI44879, SGI44880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44881, SGI0, SGI0, SGI0, SGI44882, SGI44882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10169[] = {SGI44883, SGI44884, SGI44885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44886, SGI0, SGI0, SGI0, SGI44887, SGI44887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10170[] = {SGI44888, SGI44889, SGI44890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44891, SGI0, SGI0, SGI0, SGI44892, SGI44892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10171[] = {SGI44893, SGI44894, SGI44895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44896, SGI0, SGI0, SGI0, SGI44897, SGI44897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10172[] = {SGI44898, SGI44899, SGI44900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44901, SGI0, SGI0, SGI0, SGI44902, SGI44902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10173[] = {SGI44903, SGI0, SGI44904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10174[] = {SGI44905, SGI44906, SGI44907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44908, SGI0, SGI0, SGI0, SGI44909, SGI44909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10175[] = {SGI44910, SGI0, SGI44911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10176[] = {SGI44912, SGI44913, SGI44914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44915, SGI44916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10177[] = {SGI44917, SGI44918, SGI44919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44920, SGI0, SGI0, SGI0, SGI44921, SGI44921, SGI0, SGI0, NULL};
const Segment * const SG10178[] = {SGI44922, SGI44923, SGI44924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44925, SGI0, SGI0, SGI0, SGI44926, SGI44926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10179[] = {SGI44927, SGI44928, SGI44929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44930, SGI0, SGI0, SGI0, SGI44931, SGI44931, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10180[] = {SGI44932, SGI0, SGI44933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10181[] = {SGI44934, SGI44935, SGI44936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44937, SGI0, SGI0, SGI0, SGI44938, SGI44938, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10182[] = {SGI44939, SGI44940, SGI44941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44942, SGI0, SGI0, SGI0, SGI44943, SGI44943, SGI0, SGI0, NULL};
const Segment * const SG10183[] = {SGI44944, SGI44945, SGI44946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44947, SGI44947, SGI0, SGI0, NULL};
const Segment * const SG10184[] = {SGI44948, SGI44949, SGI44950, SGI0, SGI0, SGI0, SGI44951, SGI44952, SGI0, SGI44953, SGI0, SGI0, SGI0, SGI44954, SGI44954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10185[] = {SGI44955, SGI44956, SGI44957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44958, SGI0, SGI0, SGI0, SGI44959, SGI44959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10186[] = {SGI44960, SGI0, SGI44961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10187[] = {SGI44962, SGI44963, SGI44964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44965, SGI0, SGI0, SGI0, SGI44966, SGI44966, SGI0, SGI0, NULL};
const Segment * const SG10188[] = {SGI44967, SGI44968, SGI44969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44970, SGI0, SGI0, SGI0, SGI44971, SGI44971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10189[] = {SGI44972, SGI0, SGI44973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10190[] = {SGI44974, SGI44975, SGI44976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44977, SGI0, SGI0, SGI0, SGI44978, SGI44978, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10191[] = {SGI44979, SGI44980, SGI44981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44982, SGI0, SGI0, SGI0, SGI44983, SGI44983, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10192[] = {SGI44984, SGI44985, SGI44986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44987, SGI0, SGI0, SGI0, SGI44988, SGI44988, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10193[] = {SGI44989, SGI44990, SGI44991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44992, SGI0, SGI0, SGI0, SGI44993, SGI44993, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10194[] = {SGI44994, SGI44995, SGI44996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI44997, SGI0, SGI0, SGI0, SGI44998, SGI44998, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10195[] = {SGI44999, SGI45000, SGI45001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45002, SGI0, SGI0, SGI0, SGI45003, SGI45003, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10196[] = {SGI45004, SGI45005, SGI45006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45007, SGI0, SGI0, SGI0, SGI45008, SGI45008, SGI0, SGI0, NULL};
const Segment * const SG10197[] = {SGI45009, SGI45010, SGI45011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45012, SGI0, SGI0, SGI0, SGI45013, SGI45013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10198[] = {SGI45014, SGI45015, SGI45016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45017, SGI0, SGI0, SGI0, SGI45018, SGI45018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10199[] = {SGI45019, SGI0, SGI45020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10200[] = {SGI45021, SGI45022, SGI45024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45027, SGI0, SGI0, SGI0, SGI45028, SGI45028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10201[] = {SGI45023, SGI45025, SGI45026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45029, SGI0, SGI0, SGI0, SGI45030, SGI45030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10202[] = {SGI45032, SGI0, SGI45034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10203[] = {SGI45031, SGI45033, SGI45035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45036, SGI0, SGI0, SGI0, SGI45037, SGI45037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10204[] = {SGI45038, SGI45039, SGI45040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45041, SGI0, SGI0, SGI0, SGI45042, SGI45042, SGI0, SGI0, NULL};
const Segment * const SG10205[] = {SGI45043, SGI0, SGI45044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10206[] = {SGI45045, SGI45046, SGI45047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45048, SGI0, SGI0, SGI0, SGI45049, SGI45049, SGI0, SGI0, NULL};
const Segment * const SG10207[] = {SGI45050, SGI45051, SGI45052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45053, SGI0, SGI0, SGI0, SGI45054, SGI45054, SGI0, SGI0, NULL};
const Segment * const SG10208[] = {SGI45055, SGI45056, SGI45057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45058, SGI0, SGI0, SGI0, SGI45059, SGI45059, SGI0, SGI0, NULL};
const Segment * const SG10209[] = {SGI45060, SGI45061, SGI45062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45063, SGI0, SGI0, SGI0, SGI45064, SGI45064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10210[] = {SGI45065, SGI45066, SGI45067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45068, SGI0, SGI0, SGI0, SGI45069, SGI45069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10211[] = {SGI45070, SGI45071, SGI45072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45073, SGI0, SGI0, SGI0, SGI45074, SGI45074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10212[] = {SGI45075, SGI45076, SGI45077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45078, SGI0, SGI0, SGI0, SGI45079, SGI45079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10213[] = {SGI45080, SGI45081, SGI45082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45083, SGI0, SGI0, SGI0, SGI45084, SGI45084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10214[] = {SGI45085, SGI45086, SGI45087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45088, SGI0, SGI0, SGI0, SGI45089, SGI45089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10215[] = {SGI45090, SGI45091, SGI45092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45093, SGI0, SGI0, SGI0, SGI45094, SGI45094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10216[] = {SGI45095, SGI45096, SGI45097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45098, SGI0, SGI0, SGI0, SGI45099, SGI45099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10217[] = {SGI45100, SGI45101, SGI45102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45103, SGI0, SGI0, SGI0, SGI45104, SGI45104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10218[] = {SGI45105, SGI0, SGI45106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10219[] = {SGI45107, SGI45108, SGI45109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45110, SGI0, SGI0, SGI0, SGI45111, SGI45111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10220[] = {SGI45112, SGI45113, SGI45114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45115, SGI0, SGI0, SGI0, SGI45116, SGI45116, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10221[] = {SGI45117, SGI45118, SGI45119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45120, SGI0, SGI0, SGI0, SGI45121, SGI45121, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10222[] = {SGI45122, SGI45123, SGI45124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45125, SGI0, SGI0, SGI0, SGI45126, SGI45126, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10223[] = {SGI45127, SGI45128, SGI45129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45130, SGI0, SGI0, SGI0, SGI45131, SGI45131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10224[] = {SGI45132, SGI0, SGI45133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10225[] = {SGI45134, SGI45135, SGI45136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45137, SGI0, SGI0, SGI0, SGI45138, SGI45138, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10226[] = {SGI45139, SGI0, SGI45140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10227[] = {SGI45141, SGI45142, SGI45143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45144, SGI0, SGI0, SGI0, SGI45145, SGI45145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10228[] = {SGI45146, SGI0, SGI45147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10229[] = {SGI45148, SGI45149, SGI45150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45151, SGI0, SGI0, SGI0, SGI45152, SGI45152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10230[] = {SGI45153, SGI0, SGI45154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10231[] = {SGI45155, SGI0, SGI45156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45157, SGI0, SGI0, SGI0, SGI45158, SGI45158, SGI0, SGI0, SGI0, SGI45159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10232[] = {SGI0, SGI45160, SGI45161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45162, SGI0, SGI0, SGI0, SGI45163, SGI45163, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10233[] = {SGI45164, SGI45165, SGI45166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45167, SGI0, SGI0, SGI0, SGI45168, SGI45168, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10234[] = {SGI0, SGI45169, SGI45170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45171, SGI0, SGI0, SGI0, SGI45172, SGI45172, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10235[] = {SGI45173, SGI45174, SGI45175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45176, SGI0, SGI0, SGI0, SGI45177, SGI45177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10236[] = {SGI45178, SGI45179, SGI45180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45181, SGI0, SGI0, SGI0, SGI45182, SGI45182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10237[] = {SGI45183, SGI45184, SGI45185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45186, SGI0, SGI0, SGI0, SGI45187, SGI45187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10238[] = {SGI45188, SGI45189, SGI45190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45191, SGI0, SGI0, SGI0, SGI45192, SGI45192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10239[] = {SGI45193, SGI45194, SGI45195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45196, SGI0, SGI0, SGI0, SGI45197, SGI45197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10240[] = {SGI45198, SGI45199, SGI45200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45201, SGI0, SGI0, SGI0, SGI45202, SGI45202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10241[] = {SGI45203, SGI45204, SGI45205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45206, SGI0, SGI0, SGI0, SGI45207, SGI45207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10242[] = {SGI45208, SGI45209, SGI45210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45211, SGI0, SGI0, SGI0, SGI45212, SGI45212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10243[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45213, SGI0, SGI0, SGI45214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10244[] = {SGI45215, SGI45216, SGI45217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45218, SGI0, SGI0, SGI0, SGI45219, SGI45219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10245[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45220, SGI0, SGI0, SGI45221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10246[] = {SGI45222, SGI45223, SGI45224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45225, SGI0, SGI0, SGI0, SGI45226, SGI45226, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10247[] = {SGI45227, SGI45228, SGI45229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45230, SGI0, SGI0, SGI0, SGI45231, SGI45231, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10248[] = {SGI45232, SGI45233, SGI45234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45235, SGI0, SGI0, SGI0, SGI45236, SGI45236, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10249[] = {SGI45237, SGI45238, SGI45239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45240, SGI0, SGI0, SGI0, SGI45241, SGI45241, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10250[] = {SGI45242, SGI0, SGI45243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10251[] = {SGI45244, SGI45245, SGI45246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45247, SGI0, SGI0, SGI0, SGI45248, SGI45248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10252[] = {SGI45249, SGI0, SGI45250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10253[] = {SGI45251, SGI45252, SGI45253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10254[] = {SGI45254, SGI45255, SGI45256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45257, SGI0, SGI0, SGI0, SGI45258, SGI45258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10255[] = {SGI45259, SGI45260, SGI45261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45262, SGI0, SGI0, SGI0, SGI45263, SGI45263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10256[] = {SGI45264, SGI0, SGI45265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10257[] = {SGI45266, SGI0, SGI45267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45268, SGI0, SGI0, SGI0, SGI45269, SGI45269, SGI0, SGI0, SGI0, SGI45270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10258[] = {SGI45271, SGI45272, SGI45273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45274, SGI0, SGI0, SGI0, SGI45275, SGI45275, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10259[] = {SGI45276, SGI45277, SGI45278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45279, SGI0, SGI0, SGI0, SGI45280, SGI45280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10260[] = {SGI45281, SGI0, SGI45282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10261[] = {SGI45283, SGI45284, SGI45285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45286, SGI0, SGI0, SGI0, SGI45287, SGI45287, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10262[] = {SGI45288, SGI45289, SGI45290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45291, SGI0, SGI0, SGI0, SGI45292, SGI45292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10263[] = {SGI45293, SGI0, SGI45294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45295, SGI0, SGI0, SGI0, SGI45296, SGI45296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10264[] = {SGI45297, SGI0, SGI45298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10265[] = {SGI45299, SGI45300, SGI45301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45302, SGI0, SGI0, SGI0, SGI45303, SGI45303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10266[] = {SGI45304, SGI45305, SGI45306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45307, SGI0, SGI0, SGI0, SGI45308, SGI45308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10267[] = {SGI45309, SGI45310, SGI45311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45312, SGI0, SGI0, SGI0, SGI45313, SGI45313, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10268[] = {SGI45314, SGI45315, SGI45316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45317, SGI0, SGI0, SGI0, SGI45318, SGI45318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10269[] = {SGI45319, SGI45320, SGI45321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45322, SGI0, SGI0, SGI0, SGI45323, SGI45323, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10270[] = {SGI45324, SGI0, SGI45325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10271[] = {SGI45326, SGI45327, SGI45328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45329, SGI0, SGI0, SGI0, SGI45330, SGI45330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10272[] = {SGI45331, SGI45332, SGI45333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45334, SGI0, SGI0, SGI0, SGI45335, SGI45335, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10273[] = {SGI45336, SGI0, SGI45337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10274[] = {SGI45338, SGI0, SGI45339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45340, SGI0, SGI0, SGI0, SGI45341, SGI45341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10275[] = {SGI45342, SGI45343, SGI45344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45345, SGI0, SGI0, SGI0, SGI45346, SGI45346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10276[] = {SGI45347, SGI45348, SGI45349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45350, SGI0, SGI0, SGI0, SGI45351, SGI45351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10277[] = {SGI45352, SGI0, SGI45353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10278[] = {SGI45354, SGI0, SGI45355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10279[] = {SGI45356, SGI0, SGI45357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45358, SGI0, SGI0, SGI0, SGI45359, SGI45359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10280[] = {SGI45360, SGI45361, SGI45362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45363, SGI0, SGI0, SGI0, SGI45364, SGI45364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10281[] = {SGI45365, SGI45366, SGI45367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45368, SGI0, SGI0, SGI0, SGI45369, SGI45369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10282[] = {SGI45370, SGI45371, SGI45372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45373, SGI0, SGI0, SGI0, SGI45374, SGI45374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10283[] = {SGI45375, SGI45376, SGI45377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45378, SGI0, SGI0, SGI0, SGI45379, SGI45379, SGI0, SGI0, NULL};
const Segment * const SG10284[] = {SGI45380, SGI0, SGI45381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10285[] = {SGI45382, SGI0, SGI45383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10286[] = {SGI45384, SGI45385, SGI45386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45387, SGI45388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10287[] = {SGI45389, SGI45390, SGI45391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45392, SGI0, SGI0, SGI0, SGI45393, SGI45393, SGI0, SGI0, NULL};
const Segment * const SG10288[] = {SGI45394, SGI45395, SGI45396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45397, SGI0, SGI0, SGI0, SGI45398, SGI45398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10289[] = {SGI45399, SGI45400, SGI45401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45402, SGI45402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10290[] = {SGI45403, SGI45404, SGI45405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45406, SGI0, SGI0, SGI0, SGI45407, SGI45407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10291[] = {SGI0, SGI45408, SGI45409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10292[] = {SGI45410, SGI0, SGI45411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10293[] = {SGI45412, SGI45413, SGI45414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45415, SGI0, SGI0, SGI0, SGI45416, SGI45416, SGI0, SGI0, NULL};
const Segment * const SG10294[] = {SGI45417, SGI45418, SGI45419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45420, SGI0, SGI0, SGI0, SGI45421, SGI45421, SGI0, SGI0, NULL};
const Segment * const SG10295[] = {SGI45422, SGI45423, SGI45424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45425, SGI0, SGI0, SGI0, SGI45426, SGI45426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10296[] = {SGI45428, SGI0, SGI45430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10297[] = {SGI45427, SGI45429, SGI45431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45432, SGI45432, SGI0, SGI0, SGI0, SGI45433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10298[] = {SGI45434, SGI0, SGI45435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45436, SGI0, SGI0, SGI0, SGI45437, SGI45437, SGI0, SGI0, SGI0, SGI45438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10299[] = {SGI45439, SGI45440, SGI45441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45442, SGI0, SGI0, SGI0, SGI45443, SGI45443, SGI0, SGI0, NULL};
const Segment * const SG10300[] = {SGI45444, SGI45445, SGI45446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45447, SGI0, SGI0, SGI0, SGI45448, SGI45448, SGI0, SGI0, NULL};
const Segment * const SG10301[] = {SGI45449, SGI45450, SGI45451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45452, SGI0, SGI0, SGI0, SGI45453, SGI45453, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10302[] = {SGI45454, SGI0, SGI45455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45456, SGI0, SGI0, SGI0, SGI45457, SGI45457, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10303[] = {SGI45458, SGI45459, SGI45460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45461, SGI0, SGI0, SGI0, SGI45462, SGI45462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10304[] = {SGI45463, SGI0, SGI45464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10305[] = {SGI45465, SGI45466, SGI45467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45468, SGI0, SGI0, SGI0, SGI45469, SGI45469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10306[] = {SGI45470, SGI45471, SGI45472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45473, SGI0, SGI0, SGI0, SGI45474, SGI45474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10307[] = {SGI45475, SGI45476, SGI45477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45478, SGI0, SGI0, SGI0, SGI45479, SGI45479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10308[] = {SGI45480, SGI45481, SGI45482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45483, SGI0, SGI0, SGI0, SGI45484, SGI45484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10309[] = {SGI45485, SGI0, SGI45486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10310[] = {SGI45487, SGI45488, SGI45489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45490, SGI0, SGI0, SGI0, SGI45491, SGI45491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10311[] = {SGI45492, SGI45493, SGI45494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45495, SGI0, SGI0, SGI0, SGI45496, SGI45496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10312[] = {SGI45497, SGI45498, SGI45499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10313[] = {SGI45500, SGI45501, SGI45502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45503, SGI0, SGI0, SGI0, SGI45504, SGI45504, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10314[] = {SGI45505, SGI45506, SGI45507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45508, SGI45509, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10315[] = {SGI45510, SGI45511, SGI45512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45513, SGI45513, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10316[] = {SGI45514, SGI45515, SGI45516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45517, SGI45518, SGI0, SGI0, NULL};
const Segment * const SG10317[] = {SGI45519, SGI45520, SGI45521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45522, SGI45522, SGI0, SGI0, NULL};
const Segment * const SG10318[] = {SGI45523, SGI45524, SGI45525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45526, SGI0, SGI0, SGI0, SGI45527, SGI45527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10319[] = {SGI45528, SGI45529, SGI45530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45531, SGI0, SGI0, SGI0, SGI45532, SGI45532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10320[] = {SGI45533, SGI45534, SGI45535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45536, SGI0, SGI0, SGI0, SGI45537, SGI45537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10321[] = {SGI45538, SGI0, SGI45539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10322[] = {SGI45540, SGI45541, SGI45542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10323[] = {SGI45543, SGI45544, SGI45545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45546, SGI0, SGI0, SGI0, SGI45547, SGI45547, SGI0, SGI0, NULL};
const Segment * const SG10324[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45548, SGI0, SGI0, SGI45549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10325[] = {SGI45550, SGI45551, SGI45552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45553, SGI0, SGI0, SGI0, SGI45554, SGI45554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10326[] = {SGI45555, SGI0, SGI45556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10327[] = {SGI45557, SGI45558, SGI45559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45560, SGI0, SGI0, SGI0, SGI45561, SGI45561, SGI0, SGI0, NULL};
const Segment * const SG10328[] = {SGI45562, SGI0, SGI45563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10329[] = {SGI45564, SGI45565, SGI45566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45567, SGI0, SGI0, SGI0, SGI45568, SGI45568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10330[] = {SGI45569, SGI45570, SGI45571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45572, SGI0, SGI0, SGI0, SGI45573, SGI45573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10331[] = {SGI45574, SGI45575, SGI45576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45577, SGI0, SGI0, SGI0, SGI45578, SGI45578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10332[] = {SGI45579, SGI45580, SGI45581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45582, SGI0, SGI0, SGI0, SGI45583, SGI45583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10333[] = {SGI45584, SGI0, SGI45585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10334[] = {SGI45586, SGI0, SGI45587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45588, SGI0, SGI0, SGI0, SGI45589, SGI45589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10335[] = {SGI45590, SGI45591, SGI45592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45593, SGI0, SGI0, SGI0, SGI45594, SGI45594, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10336[] = {SGI45595, SGI45596, SGI45597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45598, SGI0, SGI0, SGI0, SGI45599, SGI45599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10337[] = {SGI45600, SGI45601, SGI45602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45603, SGI0, SGI0, SGI0, SGI45604, SGI45604, SGI0, SGI0, NULL};
const Segment * const SG10338[] = {SGI45606, SGI0, SGI45608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10339[] = {SGI45605, SGI45607, SGI45609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45610, SGI0, SGI0, SGI0, SGI45611, SGI45611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10340[] = {SGI45612, SGI45613, SGI45614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45615, SGI0, SGI0, SGI0, SGI45616, SGI45616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10341[] = {SGI45617, SGI45618, SGI45619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45620, SGI0, SGI0, SGI0, SGI45621, SGI45621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10342[] = {SGI45622, SGI0, SGI45623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10343[] = {SGI45624, SGI45625, SGI45626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45627, SGI0, SGI0, SGI0, SGI45628, SGI45628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10344[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45629, SGI0, SGI0, SGI45630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10345[] = {SGI45631, SGI45632, SGI45633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45634, SGI0, SGI0, SGI0, SGI45635, SGI45635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10346[] = {SGI45636, SGI45637, SGI45638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45639, SGI0, SGI0, SGI0, SGI45640, SGI45640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10347[] = {SGI45641, SGI45642, SGI45643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45647, SGI0, SGI0, SGI0, SGI45648, SGI45648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10348[] = {SGI45644, SGI45645, SGI45646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45649, SGI0, SGI0, SGI0, SGI45650, SGI45650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10349[] = {SGI45651, SGI45652, SGI45653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45656, SGI0, SGI0, SGI0, SGI45657, SGI45657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10350[] = {SGI45654, SGI0, SGI45655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10351[] = {SGI45658, SGI45659, SGI45660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45661, SGI0, SGI0, SGI0, SGI45662, SGI45662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10352[] = {SGI45663, SGI45664, SGI45665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45666, SGI0, SGI0, SGI0, SGI45667, SGI45667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10353[] = {SGI45668, SGI45669, SGI45670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45671, SGI0, SGI0, SGI0, SGI45672, SGI45672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10354[] = {SGI45673, SGI45674, SGI45675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45676, SGI0, SGI0, SGI0, SGI45677, SGI45677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10355[] = {SGI45678, SGI45679, SGI45680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45681, SGI0, SGI0, SGI0, SGI45682, SGI45682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10356[] = {SGI45683, SGI45684, SGI45685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45686, SGI0, SGI0, SGI0, SGI45687, SGI45687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10357[] = {SGI45688, SGI45689, SGI45690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45691, SGI0, SGI0, SGI0, SGI45692, SGI45692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10358[] = {SGI45693, SGI45694, SGI45695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45696, SGI0, SGI0, SGI0, SGI45697, SGI45697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10359[] = {SGI45698, SGI0, SGI45699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10360[] = {SGI45700, SGI45701, SGI45702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45703, SGI0, SGI0, SGI0, SGI45704, SGI45704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10361[] = {SGI45705, SGI45706, SGI45707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45708, SGI0, SGI0, SGI0, SGI45709, SGI45709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10362[] = {SGI45710, SGI45711, SGI45712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45713, SGI0, SGI0, SGI0, SGI45714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10363[] = {SGI45715, SGI45716, SGI45717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45718, SGI0, SGI0, SGI0, SGI45719, SGI45719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10364[] = {SGI45720, SGI0, SGI45721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10365[] = {SGI45722, SGI45723, SGI45724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45725, SGI0, SGI0, SGI0, SGI45726, SGI45726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10366[] = {SGI45727, SGI45728, SGI45729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45730, SGI0, SGI0, SGI0, SGI45731, SGI45731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10367[] = {SGI45732, SGI45733, SGI45734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45735, SGI0, SGI0, SGI0, SGI45736, SGI45736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10368[] = {SGI45737, SGI45738, SGI45739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45740, SGI0, SGI0, SGI0, SGI45741, SGI45741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10369[] = {SGI45742, SGI45743, SGI45744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45745, SGI0, SGI0, SGI0, SGI45746, SGI45746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10370[] = {SGI45747, SGI45748, SGI45749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45750, SGI45750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10371[] = {SGI45751, SGI45752, SGI45753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45754, SGI0, SGI0, SGI0, SGI45755, SGI45755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10372[] = {SGI45756, SGI0, SGI45757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10373[] = {SGI45758, SGI45759, SGI45760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45761, SGI0, SGI0, SGI0, SGI45762, SGI45762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10374[] = {SGI45763, SGI45764, SGI45765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45766, SGI0, SGI0, SGI0, SGI45767, SGI45767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10375[] = {SGI45768, SGI45769, SGI45770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45771, SGI0, SGI0, SGI0, SGI45772, SGI45772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10376[] = {SGI45773, SGI45774, SGI45775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45776, SGI0, SGI0, SGI0, SGI45777, SGI45777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10377[] = {SGI45778, SGI45779, SGI45780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45781, SGI0, SGI0, SGI0, SGI45782, SGI45782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10378[] = {SGI45783, SGI0, SGI45784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10379[] = {SGI45785, SGI45786, SGI45787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45788, SGI0, SGI0, SGI0, SGI45789, SGI45789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10380[] = {SGI45790, SGI45791, SGI45792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45793, SGI0, SGI0, SGI0, SGI45794, SGI45794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10381[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45795, SGI0, SGI0, SGI45796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10382[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45797, SGI0, SGI0, SGI45798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10383[] = {SGI45799, SGI45800, SGI45801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45802, SGI0, SGI0, SGI0, SGI45803, SGI45803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10384[] = {SGI45804, SGI45805, SGI45806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45807, SGI0, SGI0, SGI0, SGI45808, SGI45808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10385[] = {SGI45809, SGI45810, SGI45811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45812, SGI0, SGI0, SGI0, SGI45813, SGI45813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10386[] = {SGI45814, SGI45815, SGI45816, SGI0, SGI0, SGI0, SGI45817, SGI45818, SGI0, SGI45819, SGI0, SGI0, SGI0, SGI45820, SGI45820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10387[] = {SGI45821, SGI45822, SGI45823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45824, SGI0, SGI0, SGI0, SGI45825, SGI45825, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10388[] = {SGI45826, SGI45827, SGI45828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45829, SGI0, SGI0, SGI0, SGI45830, SGI45830, SGI0, SGI0, NULL};
const Segment * const SG10389[] = {SGI45831, SGI45832, SGI45833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45834, SGI0, SGI0, SGI0, SGI45835, SGI45835, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10390[] = {SGI45836, SGI45837, SGI45838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45839, SGI0, SGI0, SGI0, SGI45840, SGI45840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10391[] = {SGI45841, SGI0, SGI45842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10392[] = {SGI45843, SGI0, SGI45844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45845, SGI0, SGI0, SGI0, SGI45846, SGI45846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10393[] = {SGI45847, SGI45848, SGI45849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45850, SGI45850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10394[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45851, SGI0, SGI0, SGI45852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10395[] = {SGI45853, SGI45854, SGI45855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45856, SGI0, SGI0, SGI0, SGI45857, SGI45857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10396[] = {SGI45858, SGI45859, SGI45860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45861, SGI0, SGI0, SGI0, SGI45862, SGI45862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10397[] = {SGI45863, SGI45864, SGI45865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45866, SGI0, SGI0, SGI0, SGI45867, SGI45867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10398[] = {SGI45868, SGI45869, SGI45870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45871, SGI0, SGI0, SGI0, SGI45872, SGI45872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10399[] = {SGI45873, SGI45874, SGI45875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45876, SGI0, SGI0, SGI0, SGI45877, SGI45877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10400[] = {SGI45878, SGI0, SGI45879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10401[] = {SGI45880, SGI45881, SGI45882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45883, SGI0, SGI0, SGI0, SGI45884, SGI45884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10402[] = {SGI45885, SGI45886, SGI45887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45888, SGI0, SGI0, SGI0, SGI45889, SGI45889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10403[] = {SGI45890, SGI45891, SGI45892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45893, SGI0, SGI0, SGI0, SGI45894, SGI45894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10404[] = {SGI0, SGI45895, SGI45896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45897, SGI45897, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10405[] = {SGI0, SGI45898, SGI45899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45900, SGI45900, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10406[] = {SGI45901, SGI45902, SGI45903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45907, SGI0, SGI0, SGI0, SGI45908, SGI45908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10407[] = {SGI45904, SGI45905, SGI45906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45909, SGI0, SGI0, SGI0, SGI45910, SGI45910, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10408[] = {SGI45911, SGI0, SGI45912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10409[] = {SGI45913, SGI45914, SGI45915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45916, SGI0, SGI0, SGI0, SGI45917, SGI45917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10410[] = {SGI45918, SGI45919, SGI45920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45921, SGI0, SGI0, SGI0, SGI45922, SGI45922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10411[] = {SGI45923, SGI45924, SGI45925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45926, SGI0, SGI0, SGI0, SGI45927, SGI45927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10412[] = {SGI45928, SGI45929, SGI45930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45931, SGI0, SGI0, SGI0, SGI45932, SGI45932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10413[] = {SGI45933, SGI45934, SGI45935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45936, SGI0, SGI0, SGI0, SGI45937, SGI45937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10414[] = {SGI45938, SGI0, SGI45939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10415[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45940, SGI0, SGI0, SGI45941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10416[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45942, SGI0, SGI0, SGI45943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10417[] = {SGI45944, SGI45945, SGI45946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45947, SGI0, SGI0, SGI0, SGI45948, SGI45948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10418[] = {SGI45949, SGI0, SGI45950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10419[] = {SGI45951, SGI45952, SGI45953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45954, SGI0, SGI0, SGI0, SGI45955, SGI45955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10420[] = {SGI45956, SGI45957, SGI45958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45959, SGI0, SGI0, SGI0, SGI45960, SGI45960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10421[] = {SGI45961, SGI45962, SGI45963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45964, SGI0, SGI0, SGI0, SGI45965, SGI45965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10422[] = {SGI45966, SGI45967, SGI45968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45969, SGI0, SGI0, SGI0, SGI45970, SGI45970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10423[] = {SGI45971, SGI45972, SGI45973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45974, SGI0, SGI0, SGI0, SGI45975, SGI45975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10424[] = {SGI45976, SGI45977, SGI45978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45979, SGI0, SGI0, SGI0, SGI45980, SGI45980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10425[] = {SGI45981, SGI45982, SGI45983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45984, SGI0, SGI0, SGI0, SGI45985, SGI45985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10426[] = {SGI45986, SGI45987, SGI45988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45989, SGI0, SGI0, SGI0, SGI45990, SGI45990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10427[] = {SGI45991, SGI45992, SGI45993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45994, SGI0, SGI0, SGI0, SGI45995, SGI45995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10428[] = {SGI45996, SGI45997, SGI45998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI45999, SGI0, SGI0, SGI0, SGI46000, SGI46000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10429[] = {SGI46001, SGI46002, SGI46003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46004, SGI0, SGI0, SGI0, SGI46005, SGI46005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10430[] = {SGI46006, SGI46007, SGI46008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46009, SGI0, SGI0, SGI0, SGI46010, SGI46010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10431[] = {SGI46011, SGI0, SGI46012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10432[] = {SGI46013, SGI46014, SGI46015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46016, SGI0, SGI0, SGI0, SGI46017, SGI46017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10433[] = {SGI46018, SGI46019, SGI46020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46021, SGI0, SGI0, SGI0, SGI46022, SGI46022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10434[] = {SGI46023, SGI46024, SGI46025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46026, SGI0, SGI0, SGI0, SGI46027, SGI46027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10435[] = {SGI46028, SGI46029, SGI46030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46031, SGI0, SGI0, SGI0, SGI46032, SGI46032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10436[] = {SGI46033, SGI46034, SGI46035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46036, SGI46036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10437[] = {SGI46037, SGI46038, SGI46039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46040, SGI0, SGI0, SGI0, SGI46041, SGI46041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10438[] = {SGI46042, SGI0, SGI46043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10439[] = {SGI46044, SGI46045, SGI46046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46047, SGI0, SGI0, SGI0, SGI46048, SGI46048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10440[] = {SGI46049, SGI46050, SGI46051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46052, SGI0, SGI0, SGI0, SGI46053, SGI46053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10441[] = {SGI46054, SGI46055, SGI46056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46057, SGI0, SGI0, SGI0, SGI46058, SGI46058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10442[] = {SGI46059, SGI46060, SGI46061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46062, SGI0, SGI0, SGI0, SGI46063, SGI46063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10443[] = {SGI46064, SGI46065, SGI46066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46067, SGI0, SGI0, SGI0, SGI46068, SGI46068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10444[] = {SGI46069, SGI0, SGI46070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10445[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46071, SGI0, SGI0, SGI46072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10446[] = {SGI46073, SGI46074, SGI46075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46076, SGI0, SGI0, SGI0, SGI46077, SGI46077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10447[] = {SGI46078, SGI0, SGI46079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10448[] = {SGI46080, SGI46081, SGI46082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46083, SGI0, SGI0, SGI0, SGI46084, SGI46084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10449[] = {SGI46085, SGI46086, SGI46087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46088, SGI0, SGI0, SGI0, SGI46089, SGI46089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10450[] = {SGI46090, SGI46091, SGI46092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46093, SGI46094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10451[] = {SGI46095, SGI46096, SGI46097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46098, SGI0, SGI0, SGI0, SGI46099, SGI46099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10452[] = {SGI46100, SGI46101, SGI46102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46103, SGI46103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10453[] = {SGI46104, SGI46105, SGI46106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46107, SGI0, SGI0, SGI0, SGI46108, SGI46108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10454[] = {SGI46109, SGI46110, SGI46111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46112, SGI0, SGI0, SGI0, SGI46113, SGI46113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10455[] = {SGI46114, SGI46115, SGI46116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46117, SGI0, SGI0, SGI0, SGI46118, SGI46118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10456[] = {SGI46119, SGI46120, SGI46121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46122, SGI0, SGI0, SGI0, SGI46123, SGI46123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10457[] = {SGI46124, SGI46125, SGI46126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46127, SGI0, SGI0, SGI0, SGI46128, SGI46128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10458[] = {SGI46129, SGI0, SGI46130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46131, SGI0, SGI0, SGI0, SGI46132, SGI46132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10459[] = {SGI46133, SGI0, SGI46134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10460[] = {SGI46135, SGI0, SGI46136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46137, SGI0, SGI0, SGI0, SGI46138, SGI46138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10461[] = {SGI46139, SGI46140, SGI46141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46142, SGI0, SGI0, SGI0, SGI46143, SGI46143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10462[] = {SGI46144, SGI0, SGI46145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10463[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46146, SGI0, SGI0, SGI46147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10464[] = {SGI0, SGI46148, SGI0, SGI46149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10465[] = {SGI46150, SGI46151, SGI46152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46153, SGI0, SGI0, SGI0, SGI46154, SGI46154, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10466[] = {SGI46155, SGI46156, SGI46157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46158, SGI0, SGI0, SGI0, SGI46159, SGI46159, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10467[] = {SGI46160, SGI46161, SGI46162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46163, SGI0, SGI0, SGI0, SGI46164, SGI46164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10468[] = {SGI46165, SGI0, SGI46166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10469[] = {SGI46167, SGI46168, SGI46169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46170, SGI0, SGI0, SGI0, SGI46171, SGI46171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10470[] = {SGI46172, SGI46173, SGI46174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46175, SGI0, SGI0, SGI0, SGI46176, SGI46176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10471[] = {SGI46177, SGI46178, SGI46179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46180, SGI0, SGI0, SGI0, SGI46181, SGI46181, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10472[] = {SGI46182, SGI0, SGI46183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10473[] = {SGI46184, SGI46185, SGI46186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46187, SGI0, SGI0, SGI0, SGI46188, SGI46188, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10474[] = {SGI46189, SGI46190, SGI46191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46192, SGI0, SGI0, SGI0, SGI46193, SGI46193, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10475[] = {SGI46194, SGI46195, SGI46196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46197, SGI0, SGI0, SGI0, SGI46198, SGI46198, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10476[] = {SGI46199, SGI46200, SGI46201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46202, SGI0, SGI0, SGI0, SGI46203, SGI46203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10477[] = {SGI46204, SGI46205, SGI46206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46207, SGI0, SGI0, SGI0, SGI46208, SGI46208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10478[] = {SGI46209, SGI46210, SGI46211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46212, SGI0, SGI0, SGI0, SGI46213, SGI46213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10479[] = {SGI46214, SGI46215, SGI46216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46217, SGI0, SGI0, SGI0, SGI46218, SGI46218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10480[] = {SGI46219, SGI46220, SGI46221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46222, SGI0, SGI0, SGI0, SGI46223, SGI46223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10481[] = {SGI46224, SGI46225, SGI46226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46227, SGI0, SGI0, SGI0, SGI46228, SGI46228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10482[] = {SGI46229, SGI0, SGI46230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10483[] = {SGI46231, SGI46232, SGI46233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46234, SGI0, SGI0, SGI0, SGI46235, SGI46235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10484[] = {SGI46236, SGI0, SGI46237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10485[] = {SGI46238, SGI46239, SGI46240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46241, SGI0, SGI0, SGI0, SGI46242, SGI46242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10486[] = {SGI46243, SGI46244, SGI46245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46246, SGI46247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10487[] = {SGI46248, SGI46249, SGI46250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46251, SGI46251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10488[] = {SGI46252, SGI46253, SGI46254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46255, SGI0, SGI0, SGI0, SGI46256, SGI46256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10489[] = {SGI46257, SGI0, SGI46258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10490[] = {SGI46259, SGI46260, SGI46261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46262, SGI0, SGI0, SGI0, SGI46263, SGI46263, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10491[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46264, SGI0, SGI0, SGI46265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10492[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46266, SGI0, SGI0, SGI46267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10493[] = {SGI46268, SGI46269, SGI46270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46271, SGI0, SGI0, SGI0, SGI46272, SGI46272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10494[] = {SGI46273, SGI46274, SGI46275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46276, SGI0, SGI0, SGI0, SGI46277, SGI46277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10495[] = {SGI46278, SGI46279, SGI46280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46281, SGI0, SGI0, SGI0, SGI46282, SGI46282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10496[] = {SGI46283, SGI46284, SGI46285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46286, SGI0, SGI0, SGI0, SGI46287, SGI46287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10497[] = {SGI46288, SGI46289, SGI46290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46291, SGI0, SGI0, SGI0, SGI46292, SGI46292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10498[] = {SGI46293, SGI0, SGI46294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46295, SGI0, SGI0, SGI0, SGI46296, SGI46296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10499[] = {SGI46297, SGI46298, SGI46299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46300, SGI0, SGI0, SGI0, SGI46301, SGI46301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10500[] = {SGI46302, SGI46303, SGI46304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46308, SGI46308, SGI0, SGI46309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10501[] = {SGI46305, SGI46306, SGI46307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46310, SGI0, SGI0, SGI0, SGI46311, SGI46311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10502[] = {SGI46312, SGI0, SGI46313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10503[] = {SGI0, SGI0, SGI46314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46315, SGI46316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10504[] = {SGI46317, SGI46318, SGI46319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46320, SGI0, SGI0, SGI0, SGI46321, SGI46321, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10505[] = {SGI46322, SGI46323, SGI46324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46325, SGI46326, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10506[] = {SGI0, SGI46327, SGI46328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46329, SGI0, SGI0, SGI0, SGI46330, SGI46330, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10507[] = {SGI46331, SGI0, SGI46332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10508[] = {SGI46333, SGI46334, SGI46335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46336, SGI0, SGI0, SGI0, SGI46337, SGI46337, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10509[] = {SGI46338, SGI46339, SGI46340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46341, SGI0, SGI0, SGI0, SGI46342, SGI46342, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10510[] = {SGI46343, SGI46344, SGI46345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46346, SGI46347, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10511[] = {SGI46348, SGI46349, SGI46350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46351, SGI0, SGI0, SGI0, SGI46352, SGI46352, SGI0, SGI0, NULL};
const Segment * const SG10512[] = {SGI46353, SGI46354, SGI46355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46356, SGI46357, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10513[] = {SGI46358, SGI46359, SGI46360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46361, SGI0, SGI0, SGI0, SGI46362, SGI46362, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10514[] = {SGI46363, SGI46364, SGI46365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46366, SGI46366, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10515[] = {SGI46367, SGI0, SGI46368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10516[] = {SGI46369, SGI46370, SGI46371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46372, SGI0, SGI0, SGI0, SGI46373, SGI46373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10517[] = {SGI46374, SGI46375, SGI46376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46377, SGI0, SGI0, SGI0, SGI46378, SGI46378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10518[] = {SGI46379, SGI46380, SGI46381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46382, SGI0, SGI0, SGI0, SGI46383, SGI46383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10519[] = {SGI46384, SGI46385, SGI46386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46387, SGI0, SGI0, SGI0, SGI46388, SGI46388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10520[] = {SGI46389, SGI46390, SGI46391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46392, SGI0, SGI0, SGI0, SGI46393, SGI46393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10521[] = {SGI46394, SGI46395, SGI46396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46397, SGI0, SGI0, SGI0, SGI46398, SGI46398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10522[] = {SGI46399, SGI46400, SGI46401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46402, SGI0, SGI0, SGI0, SGI46403, SGI46403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10523[] = {SGI46404, SGI46405, SGI46406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46407, SGI0, SGI0, SGI0, SGI46408, SGI46408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10524[] = {SGI46409, SGI46410, SGI46411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46412, SGI0, SGI0, SGI0, SGI46413, SGI46413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10525[] = {SGI46414, SGI46415, SGI46416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46417, SGI0, SGI0, SGI0, SGI46418, SGI46418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10526[] = {SGI46419, SGI46420, SGI46421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46422, SGI0, SGI0, SGI0, SGI46423, SGI46423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10527[] = {SGI46424, SGI46425, SGI46426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46427, SGI0, SGI0, SGI0, SGI46428, SGI46428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10528[] = {SGI46429, SGI0, SGI46430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10529[] = {SGI46431, SGI46432, SGI46433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46434, SGI0, SGI0, SGI0, SGI46435, SGI46435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10530[] = {SGI46436, SGI46437, SGI46438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46439, SGI46439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10531[] = {SGI46440, SGI0, SGI46441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10532[] = {SGI46442, SGI46443, SGI46444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46445, SGI0, SGI0, SGI0, SGI46446, SGI46446, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10533[] = {SGI46447, SGI46448, SGI46449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46450, SGI0, SGI0, SGI0, SGI46451, SGI46451, SGI0, SGI0, NULL};
const Segment * const SG10534[] = {SGI46452, SGI46453, SGI46454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46455, SGI0, SGI0, SGI0, SGI46456, SGI46456, SGI0, SGI0, NULL};
const Segment * const SG10535[] = {SGI46457, SGI46458, SGI46459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46460, SGI0, SGI0, SGI0, SGI46461, SGI46461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10536[] = {SGI46462, SGI46463, SGI46464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46465, SGI0, SGI0, SGI0, SGI46466, SGI46466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10537[] = {SGI46467, SGI46468, SGI46469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46470, SGI46471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10538[] = {SGI46472, SGI46473, SGI46474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46475, SGI0, SGI0, SGI0, SGI46476, SGI46476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10539[] = {SGI46477, SGI46478, SGI46479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46480, SGI0, SGI0, SGI0, SGI46481, SGI46481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10540[] = {SGI46482, SGI46483, SGI46484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46485, SGI0, SGI0, SGI0, SGI46486, SGI46486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10541[] = {SGI46487, SGI46488, SGI46489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46490, SGI0, SGI0, SGI0, SGI46491, SGI46491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10542[] = {SGI46492, SGI46493, SGI46494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46495, SGI0, SGI0, SGI0, SGI46496, SGI46496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10543[] = {SGI46497, SGI46498, SGI46499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46500, SGI0, SGI0, SGI0, SGI46501, SGI46501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10544[] = {SGI46502, SGI46503, SGI46504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46505, SGI0, SGI0, SGI0, SGI46506, SGI46506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10545[] = {SGI46507, SGI46508, SGI46509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46510, SGI0, SGI0, SGI0, SGI46511, SGI46511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10546[] = {SGI46512, SGI46513, SGI46514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46515, SGI0, SGI0, SGI0, SGI46516, SGI46516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10547[] = {SGI46517, SGI46518, SGI46519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46520, SGI0, SGI0, SGI0, SGI46521, SGI46521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10548[] = {SGI46522, SGI46523, SGI46524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46525, SGI0, SGI0, SGI0, SGI46526, SGI46526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10549[] = {SGI46527, SGI0, SGI46528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10550[] = {SGI46529, SGI46530, SGI46531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46532, SGI0, SGI0, SGI0, SGI46533, SGI46533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10551[] = {SGI46534, SGI46535, SGI46536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46537, SGI0, SGI0, SGI0, SGI46538, SGI46538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10552[] = {SGI46539, SGI46540, SGI46541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46542, SGI0, SGI0, SGI0, SGI46543, SGI46543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10553[] = {SGI46544, SGI0, SGI46545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10554[] = {SGI46546, SGI0, SGI46547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10555[] = {SGI46548, SGI46549, SGI46550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46551, SGI0, SGI0, SGI0, SGI46552, SGI46552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10556[] = {SGI46553, SGI46554, SGI46555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46556, SGI0, SGI0, SGI0, SGI46557, SGI46557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10557[] = {SGI46558, SGI46559, SGI46560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46561, SGI0, SGI0, SGI0, SGI46562, SGI46562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10558[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46563, SGI0, SGI0, SGI46564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10559[] = {SGI46565, SGI46566, SGI46567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46568, SGI46569, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10560[] = {SGI46570, SGI46571, SGI46572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46573, SGI0, SGI0, SGI0, SGI46574, SGI46574, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10561[] = {SGI46575, SGI46576, SGI46577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46578, SGI46578, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10562[] = {SGI46579, SGI46580, SGI46581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46582, SGI0, SGI0, SGI0, SGI46583, SGI46583, SGI0, SGI0, NULL};
const Segment * const SG10563[] = {SGI46584, SGI46585, SGI46586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46587, SGI0, SGI0, SGI0, SGI46588, SGI46588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10564[] = {SGI46589, SGI0, SGI46590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10565[] = {SGI46591, SGI46592, SGI46593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46594, SGI0, SGI0, SGI0, SGI46595, SGI46595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10566[] = {SGI46596, SGI46597, SGI46598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46599, SGI0, SGI0, SGI0, SGI46600, SGI46600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10567[] = {SGI46601, SGI0, SGI46602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10568[] = {SGI46603, SGI46604, SGI46605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46606, SGI0, SGI0, SGI0, SGI46607, SGI46607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10569[] = {SGI46608, SGI46609, SGI46610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46611, SGI0, SGI0, SGI0, SGI46612, SGI46612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10570[] = {SGI46613, SGI0, SGI46614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46615, SGI0, SGI0, SGI0, SGI46616, SGI46616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10571[] = {SGI46618, SGI0, SGI46620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10572[] = {SGI46617, SGI46619, SGI46621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46622, SGI0, SGI0, SGI0, SGI46623, SGI46623, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10573[] = {SGI46624, SGI46625, SGI46626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46627, SGI0, SGI0, SGI0, SGI46628, SGI46628, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10574[] = {SGI46629, SGI46630, SGI46631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46632, SGI0, SGI0, SGI0, SGI46633, SGI46633, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10575[] = {SGI46634, SGI46635, SGI46636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46637, SGI0, SGI0, SGI0, SGI46638, SGI46638, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10576[] = {SGI46639, SGI46640, SGI46641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46642, SGI0, SGI0, SGI0, SGI46643, SGI46643, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10577[] = {SGI46644, SGI46645, SGI46646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46647, SGI0, SGI0, SGI0, SGI46648, SGI46648, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10578[] = {SGI46649, SGI46650, SGI46651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46652, SGI0, SGI0, SGI0, SGI46653, SGI46653, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10579[] = {SGI46654, SGI46655, SGI46656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46657, SGI0, SGI0, SGI0, SGI46658, SGI46658, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10580[] = {SGI46659, SGI46660, SGI46661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46662, SGI46663, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10581[] = {SGI46664, SGI46665, SGI46666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46667, SGI0, SGI0, SGI0, SGI46668, SGI46668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10582[] = {SGI46669, SGI46670, SGI46671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46672, SGI0, SGI0, SGI0, SGI46673, SGI46673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10583[] = {SGI46674, SGI46675, SGI46676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46677, SGI0, SGI0, SGI0, SGI46678, SGI46678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10584[] = {SGI46679, SGI46680, SGI46681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46682, SGI0, SGI0, SGI0, SGI46683, SGI46683, SGI0, SGI0, NULL};
const Segment * const SG10585[] = {SGI46684, SGI46685, SGI46686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46687, SGI0, SGI0, SGI0, SGI46688, SGI46688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10586[] = {SGI46689, SGI46690, SGI46691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46692, SGI0, SGI0, SGI0, SGI46693, SGI46693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10587[] = {SGI46694, SGI46695, SGI46696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46697, SGI0, SGI0, SGI0, SGI46698, SGI46698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10588[] = {SGI46699, SGI46700, SGI46701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46702, SGI0, SGI0, SGI0, SGI46703, SGI46703, SGI0, SGI0, NULL};
const Segment * const SG10589[] = {SGI46704, SGI46705, SGI46706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46707, SGI0, SGI0, SGI0, SGI46708, SGI46708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10590[] = {SGI46709, SGI46710, SGI46711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46712, SGI0, SGI0, SGI0, SGI46713, SGI46713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10591[] = {SGI46714, SGI46715, SGI46716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10592[] = {SGI46717, SGI46718, SGI46719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46720, SGI0, SGI0, SGI0, SGI46721, SGI46721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10593[] = {SGI46722, SGI46723, SGI46724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46725, SGI0, SGI0, SGI0, SGI46726, SGI46726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10594[] = {SGI46727, SGI46728, SGI46729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46730, SGI0, SGI0, SGI0, SGI46731, SGI46731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10595[] = {SGI46732, SGI46733, SGI46734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46735, SGI46736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10596[] = {SGI46737, SGI46738, SGI46739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46740, SGI0, SGI0, SGI0, SGI46741, SGI46741, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10597[] = {SGI46742, SGI0, SGI46743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10598[] = {SGI46744, SGI46745, SGI46746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46747, SGI0, SGI0, SGI0, SGI46748, SGI46748, SGI0, SGI0, NULL};
const Segment * const SG10599[] = {SGI46749, SGI46750, SGI46751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46752, SGI0, SGI0, SGI0, SGI46753, SGI46753, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10600[] = {SGI46754, SGI46755, SGI46756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46757, SGI0, SGI0, SGI0, SGI46758, SGI46758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10601[] = {SGI46759, SGI46760, SGI46761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46762, SGI0, SGI0, SGI0, SGI46763, SGI46763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10602[] = {SGI46764, SGI0, SGI46765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10603[] = {SGI46766, SGI46767, SGI46768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46769, SGI0, SGI0, SGI0, SGI46770, SGI46770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10604[] = {SGI46771, SGI0, SGI46772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10605[] = {SGI46773, SGI46774, SGI46775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46776, SGI0, SGI0, SGI0, SGI46777, SGI46777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10606[] = {SGI46778, SGI46779, SGI46780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46781, SGI0, SGI0, SGI0, SGI46782, SGI46782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10607[] = {SGI46783, SGI46784, SGI46785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46786, SGI0, SGI0, SGI0, SGI46787, SGI46787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10608[] = {SGI46788, SGI46789, SGI46790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46791, SGI0, SGI0, SGI0, SGI46792, SGI46792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10609[] = {SGI46793, SGI46794, SGI46795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46796, SGI0, SGI0, SGI0, SGI46797, SGI46797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10610[] = {SGI46798, SGI46799, SGI46800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46801, SGI0, SGI0, SGI0, SGI46802, SGI46802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10611[] = {SGI46803, SGI46804, SGI46805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46806, SGI0, SGI0, SGI0, SGI46807, SGI46807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10612[] = {SGI46808, SGI46809, SGI46810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46811, SGI0, SGI0, SGI0, SGI46812, SGI46812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10613[] = {SGI46813, SGI46814, SGI46815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46816, SGI0, SGI0, SGI0, SGI46817, SGI46817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10614[] = {SGI46818, SGI46819, SGI46820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46821, SGI0, SGI0, SGI0, SGI46822, SGI46822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10615[] = {SGI46823, SGI0, SGI46824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10616[] = {SGI46825, SGI46826, SGI46827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46828, SGI0, SGI0, SGI0, SGI46829, SGI46829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10617[] = {SGI46830, SGI46831, SGI46832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46833, SGI0, SGI0, SGI0, SGI46834, SGI46834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10618[] = {SGI46835, SGI46836, SGI46837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46838, SGI0, SGI0, SGI0, SGI46839, SGI46839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10619[] = {SGI46840, SGI46841, SGI46842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46843, SGI0, SGI0, SGI0, SGI46844, SGI46844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10620[] = {SGI46845, SGI46846, SGI46847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46848, SGI0, SGI0, SGI0, SGI46849, SGI46849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10621[] = {SGI46850, SGI46851, SGI46852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46853, SGI0, SGI0, SGI0, SGI46854, SGI46854, SGI0, SGI0, NULL};
const Segment * const SG10622[] = {SGI46855, SGI46856, SGI46857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46858, SGI0, SGI0, SGI0, SGI46859, SGI46859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10623[] = {SGI46860, SGI46861, SGI46862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46863, SGI0, SGI0, SGI0, SGI46864, SGI46864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10624[] = {SGI46865, SGI46866, SGI46867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46868, SGI0, SGI0, SGI0, SGI46869, SGI46869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10625[] = {SGI46870, SGI46871, SGI46872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46873, SGI0, SGI0, SGI0, SGI46874, SGI46874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10626[] = {SGI46875, SGI46876, SGI46877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46878, SGI0, SGI0, SGI0, SGI46879, SGI46879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10627[] = {SGI46880, SGI0, SGI46881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10628[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46882, SGI0, SGI0, SGI46883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10629[] = {SGI46884, SGI46885, SGI46886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46887, SGI0, SGI0, SGI0, SGI46888, SGI46888, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10630[] = {SGI46889, SGI46890, SGI46891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46892, SGI0, SGI0, SGI0, SGI46893, SGI46893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10631[] = {SGI46894, SGI46895, SGI46896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46897, SGI0, SGI0, SGI0, SGI46898, SGI46898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10632[] = {SGI46901, SGI0, SGI46903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10633[] = {SGI46899, SGI46900, SGI46902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46904, SGI0, SGI0, SGI0, SGI46905, SGI46905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10634[] = {SGI46907, SGI0, SGI46910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10635[] = {SGI46906, SGI46908, SGI46909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46911, SGI0, SGI0, SGI0, SGI46912, SGI46912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10636[] = {SGI46913, SGI46914, SGI46915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46916, SGI0, SGI0, SGI0, SGI46917, SGI46917, SGI0, SGI0, NULL};
const Segment * const SG10637[] = {SGI46918, SGI0, SGI46919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10638[] = {SGI46920, SGI46921, SGI46922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46923, SGI0, SGI0, SGI0, SGI46924, SGI46924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10639[] = {SGI46925, SGI46926, SGI46927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46928, SGI0, SGI0, SGI0, SGI46929, SGI46929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10640[] = {SGI46930, SGI46931, SGI46932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46933, SGI0, SGI0, SGI0, SGI46934, SGI46934, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10641[] = {SGI46935, SGI46936, SGI46937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46938, SGI0, SGI0, SGI0, SGI46939, SGI46939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10642[] = {SGI46940, SGI46941, SGI46942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46943, SGI0, SGI0, SGI0, SGI46944, SGI46944, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10643[] = {SGI46945, SGI46946, SGI46947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46948, SGI46949, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10644[] = {SGI46950, SGI46951, SGI46952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46953, SGI46953, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10645[] = {SGI46954, SGI46955, SGI46956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46957, SGI0, SGI0, SGI0, SGI46958, SGI46958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10646[] = {SGI46959, SGI46960, SGI46961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46962, SGI0, SGI0, SGI0, SGI46963, SGI46963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10647[] = {SGI46964, SGI46965, SGI46966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46967, SGI0, SGI0, SGI0, SGI46968, SGI46968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10648[] = {SGI46969, SGI46970, SGI46971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46972, SGI0, SGI0, SGI0, SGI46973, SGI46973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10649[] = {SGI46974, SGI46975, SGI46976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46977, SGI0, SGI0, SGI0, SGI46978, SGI46978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10650[] = {SGI46979, SGI46980, SGI46981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46982, SGI0, SGI0, SGI0, SGI46983, SGI46983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10651[] = {SGI46984, SGI0, SGI46985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10652[] = {SGI46986, SGI46987, SGI46988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46989, SGI0, SGI0, SGI0, SGI46990, SGI46990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10653[] = {SGI46991, SGI0, SGI46992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10654[] = {SGI46993, SGI46994, SGI46995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI46996, SGI0, SGI0, SGI0, SGI46997, SGI46997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10655[] = {SGI46998, SGI46999, SGI47000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47001, SGI0, SGI0, SGI0, SGI47002, SGI47002, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10656[] = {SGI47003, SGI47004, SGI47005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47006, SGI0, SGI0, SGI0, SGI47007, SGI47007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10657[] = {SGI47008, SGI47009, SGI47010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47012, SGI0, SGI0, SGI0, SGI47015, SGI47015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10658[] = {SGI47011, SGI47013, SGI47014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47016, SGI0, SGI0, SGI0, SGI47017, SGI47017, SGI0, SGI0, NULL};
const Segment * const SG10659[] = {SGI47018, SGI47019, SGI47020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47021, SGI0, SGI0, SGI0, SGI47022, SGI47022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10660[] = {SGI47023, SGI0, SGI47024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10661[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47025, SGI0, SGI0, SGI47026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10662[] = {SGI47027, SGI47028, SGI47029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47030, SGI0, SGI0, SGI0, SGI47031, SGI47031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10663[] = {SGI47032, SGI47033, SGI47034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47035, SGI0, SGI0, SGI0, SGI47036, SGI47036, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10664[] = {SGI47037, SGI47038, SGI47039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47040, SGI0, SGI0, SGI0, SGI47041, SGI47041, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10665[] = {SGI47042, SGI47043, SGI47044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47045, SGI0, SGI0, SGI0, SGI47046, SGI47046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10666[] = {SGI47047, SGI47048, SGI47049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47050, SGI0, SGI0, SGI0, SGI47051, SGI47051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10667[] = {SGI47052, SGI0, SGI47053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10668[] = {SGI47054, SGI47055, SGI47056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47057, SGI47057, SGI0, SGI0, SGI0, SGI47058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10669[] = {SGI47059, SGI47060, SGI47061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47062, SGI47062, SGI0, SGI0, SGI0, SGI47063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10670[] = {SGI47064, SGI47065, SGI47066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47067, SGI0, SGI0, SGI0, SGI47068, SGI47068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10671[] = {SGI47069, SGI47070, SGI47071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47072, SGI0, SGI0, SGI0, SGI47073, SGI47073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10672[] = {SGI47074, SGI0, SGI47075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10673[] = {SGI0, SGI0, SGI47076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47077, SGI47078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10674[] = {SGI47079, SGI47080, SGI47081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47082, SGI0, SGI0, SGI0, SGI47083, SGI47083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10675[] = {SGI47084, SGI0, SGI47085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10676[] = {SGI47086, SGI47087, SGI47088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47089, SGI0, SGI0, SGI0, SGI47090, SGI47091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10677[] = {SGI47092, SGI47093, SGI47094, SGI0, SGI0, SGI0, SGI47095, SGI47096, SGI0, SGI47097, SGI0, SGI0, SGI0, SGI47098, SGI47098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10678[] = {SGI47099, SGI47100, SGI47101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47102, SGI0, SGI0, SGI0, SGI47103, SGI47103, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10679[] = {SGI47104, SGI47105, SGI47106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47107, SGI0, SGI0, SGI0, SGI47108, SGI47108, SGI0, SGI0, NULL};
const Segment * const SG10680[] = {SGI47109, SGI47110, SGI47111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47112, SGI0, SGI0, SGI0, SGI47114, SGI47114, SGI0, SGI0, NULL};
const Segment * const SG10681[] = {SGI47113, SGI47115, SGI47116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47117, SGI0, SGI0, SGI0, SGI47118, SGI47118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10682[] = {SGI47119, SGI47120, SGI47121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47122, SGI0, SGI0, SGI0, SGI47123, SGI47123, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10683[] = {SGI47124, SGI47125, SGI47126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47127, SGI47127, SGI0, SGI47128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10684[] = {SGI47129, SGI0, SGI47130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10685[] = {SGI47131, SGI47132, SGI47133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47134, SGI0, SGI0, SGI0, SGI47135, SGI47135, SGI0, SGI0, NULL};
const Segment * const SG10686[] = {SGI47136, SGI47137, SGI47138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47139, SGI0, SGI0, SGI0, SGI47140, SGI47140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10687[] = {SGI47141, SGI47142, SGI47143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47144, SGI0, SGI0, SGI0, SGI47145, SGI47145, SGI0, SGI0, NULL};
const Segment * const SG10688[] = {SGI47146, SGI47147, SGI47148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47151, SGI0, SGI0, SGI0, SGI47153, SGI47153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10689[] = {SGI47149, SGI47150, SGI47152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47154, SGI47154, SGI0, SGI0, NULL};
const Segment * const SG10690[] = {SGI47155, SGI47156, SGI47157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47158, SGI47159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10691[] = {SGI47160, SGI47161, SGI47162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47163, SGI47163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10692[] = {SGI47164, SGI0, SGI47165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10693[] = {SGI0, SGI47166, SGI47167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10694[] = {SGI47168, SGI47169, SGI47170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47171, SGI0, SGI0, SGI0, SGI47172, SGI47172, SGI0, SGI0, NULL};
const Segment * const SG10695[] = {SGI47173, SGI0, SGI47174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10696[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47175, SGI0, SGI0, SGI47177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10697[] = {SGI47176, SGI0, SGI47178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47179, SGI47180, SGI47180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47181, NULL};
const Segment * const SG10698[] = {SGI47182, SGI47183, SGI47184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47185, SGI0, SGI0, SGI0, SGI47186, SGI47186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10699[] = {SGI47187, SGI0, SGI47188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10700[] = {SGI47189, SGI47190, SGI47191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47192, SGI47192, SGI0, SGI0, SGI0, SGI47193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10701[] = {SGI47194, SGI47195, SGI47196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47197, SGI0, SGI0, SGI0, SGI47198, SGI47198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10702[] = {SGI47199, SGI0, SGI47200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10703[] = {SGI0, SGI0, SGI47201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47202, SGI47203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10704[] = {SGI47204, SGI47205, SGI47206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47207, SGI47207, SGI0, SGI47208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10705[] = {SGI47209, SGI47210, SGI47211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47212, SGI0, SGI0, SGI0, SGI47213, SGI47213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10706[] = {SGI47214, SGI47215, SGI47216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47217, SGI0, SGI0, SGI0, SGI47218, SGI47218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10707[] = {SGI47219, SGI47220, SGI47221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47222, SGI0, SGI0, SGI0, SGI47223, SGI47223, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10708[] = {SGI47224, SGI47225, SGI47226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47227, SGI0, SGI0, SGI0, SGI47228, SGI47228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10709[] = {SGI47229, SGI47230, SGI47231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47232, SGI0, SGI0, SGI0, SGI47233, SGI47233, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10710[] = {SGI47234, SGI47235, SGI47236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47237, SGI0, SGI0, SGI0, SGI47238, SGI47238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10711[] = {SGI47239, SGI47240, SGI47241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47242, SGI0, SGI0, SGI0, SGI47243, SGI47243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10712[] = {SGI0, SGI47244, SGI47245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47246, SGI0, SGI0, SGI0, SGI0, SGI47247, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10713[] = {SGI47248, SGI47249, SGI47250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47251, SGI0, SGI0, SGI0, SGI47252, SGI47252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10714[] = {SGI0, SGI47253, SGI47254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47255, SGI0, SGI0, SGI0, SGI0, SGI47256, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10715[] = {SGI47257, SGI47258, SGI47259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47260, SGI0, SGI0, SGI0, SGI47261, SGI47261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10716[] = {SGI0, SGI47262, SGI47263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47264, SGI0, SGI0, SGI0, SGI0, SGI47265, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10717[] = {SGI47266, SGI47267, SGI47268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47269, SGI0, SGI0, SGI0, SGI47270, SGI47270, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10718[] = {SGI0, SGI47271, SGI47272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47273, SGI0, SGI0, SGI0, SGI0, SGI47274, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10719[] = {SGI0, SGI47275, SGI47276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47277, SGI0, SGI0, SGI0, SGI0, SGI47278, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10720[] = {SGI0, SGI47279, SGI47280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47281, SGI0, SGI0, SGI0, SGI0, SGI47282, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10721[] = {SGI47283, SGI47284, SGI47285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47286, SGI47286, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10722[] = {SGI47287, SGI47288, SGI47289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47290, SGI0, SGI0, SGI0, SGI47291, SGI47291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10723[] = {SGI47292, SGI47293, SGI47294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47295, SGI0, SGI0, SGI0, SGI47296, SGI47296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10724[] = {SGI47297, SGI47298, SGI47299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47300, SGI0, SGI0, SGI0, SGI47301, SGI47301, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10725[] = {SGI47302, SGI47303, SGI47304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47306, SGI0, SGI0, SGI0, SGI47309, SGI47309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10726[] = {SGI47305, SGI47307, SGI47308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47310, SGI0, SGI0, SGI0, SGI47311, SGI47311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10727[] = {SGI47312, SGI0, SGI47313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10728[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47314, SGI0, SGI0, SGI47315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10729[] = {SGI47316, SGI47317, SGI47318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47319, SGI0, SGI0, SGI0, SGI47320, SGI47320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10730[] = {SGI47321, SGI0, SGI47322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10731[] = {SGI47323, SGI47324, SGI47325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47326, SGI0, SGI0, SGI0, SGI47327, SGI47327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10732[] = {SGI47328, SGI47329, SGI47330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47331, SGI0, SGI0, SGI0, SGI47332, SGI47332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10733[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10734[] = {SGI47334, SGI47335, SGI47336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47337, SGI0, SGI0, SGI0, SGI47338, SGI47338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10735[] = {SGI47339, SGI47340, SGI47341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47342, SGI47342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10736[] = {SGI47343, SGI0, SGI47344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10737[] = {SGI47345, SGI47346, SGI47347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47348, SGI0, SGI0, SGI0, SGI47349, SGI47349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10738[] = {SGI47350, SGI0, SGI47351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10739[] = {SGI47352, SGI47353, SGI47354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47355, SGI0, SGI0, SGI0, SGI47356, SGI47356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10740[] = {SGI47357, SGI47358, SGI47359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47360, SGI0, SGI0, SGI0, SGI47361, SGI47361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10741[] = {SGI47362, SGI47363, SGI47364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47365, SGI0, SGI0, SGI0, SGI47366, SGI47366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10742[] = {SGI47367, SGI0, SGI47368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47369, SGI0, SGI0, SGI0, SGI47370, SGI47370, SGI0, SGI47371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10743[] = {SGI47372, SGI47373, SGI47374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47375, SGI0, SGI0, SGI0, SGI47376, SGI47376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10744[] = {SGI47377, SGI0, SGI47378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10745[] = {SGI47379, SGI47380, SGI47381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47382, SGI0, SGI0, SGI0, SGI47383, SGI47383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10746[] = {SGI47384, SGI47385, SGI47386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47387, SGI0, SGI0, SGI0, SGI47388, SGI47388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10747[] = {SGI47389, SGI47390, SGI47391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47392, SGI0, SGI0, SGI0, SGI47393, SGI47393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10748[] = {SGI47394, SGI47395, SGI47396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47397, SGI0, SGI0, SGI0, SGI47398, SGI47398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10749[] = {SGI47399, SGI47400, SGI47401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47402, SGI47403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10750[] = {SGI47404, SGI47405, SGI47406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47407, SGI0, SGI0, SGI0, SGI47408, SGI47408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10751[] = {SGI47409, SGI0, SGI47410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10752[] = {SGI47411, SGI47412, SGI47413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47414, SGI0, SGI0, SGI0, SGI47415, SGI47415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10753[] = {SGI47416, SGI0, SGI47417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10754[] = {SGI47418, SGI47419, SGI47420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47421, SGI47421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10755[] = {SGI0, SGI47422, SGI47423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10756[] = {SGI47424, SGI47425, SGI47426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47427, SGI0, SGI0, SGI0, SGI47428, SGI47428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10757[] = {SGI47429, SGI0, SGI47430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10758[] = {SGI47431, SGI47432, SGI47433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47434, SGI0, SGI0, SGI0, SGI47435, SGI47435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10759[] = {SGI47436, SGI47437, SGI47438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47439, SGI0, SGI0, SGI0, SGI47440, SGI47440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10760[] = {SGI47441, SGI47442, SGI47443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47444, SGI0, SGI0, SGI0, SGI47445, SGI47445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10761[] = {SGI47446, SGI47447, SGI47448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47449, SGI0, SGI0, SGI0, SGI47450, SGI47450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10762[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47451, SGI0, SGI0, SGI47452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10763[] = {SGI47453, SGI47454, SGI47455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47456, SGI0, SGI0, SGI0, SGI47457, SGI47457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10764[] = {SGI47458, SGI47459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47460, SGI0, SGI0, SGI0, SGI47461, SGI47461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10765[] = {SGI47462, SGI47463, SGI47464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47465, SGI0, SGI0, SGI0, SGI47466, SGI47466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10766[] = {SGI47467, SGI47468, SGI47469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47470, SGI0, SGI0, SGI0, SGI47471, SGI47471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10767[] = {SGI47472, SGI0, SGI47473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10768[] = {SGI47474, SGI47475, SGI47476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47477, SGI47478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10769[] = {SGI47479, SGI47480, SGI47481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47482, SGI0, SGI0, SGI0, SGI47483, SGI47483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10770[] = {SGI47484, SGI47485, SGI47486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47487, SGI0, SGI0, SGI0, SGI47488, SGI47488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10771[] = {SGI47489, SGI47490, SGI47491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47492, SGI0, SGI0, SGI0, SGI47493, SGI47493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10772[] = {SGI47494, SGI0, SGI47495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47496, SGI0, SGI0, SGI0, SGI47497, SGI47497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10773[] = {SGI47498, SGI0, SGI47499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10774[] = {SGI47500, SGI47501, SGI47502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47503, SGI0, SGI0, SGI0, SGI47504, SGI47504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10775[] = {SGI47505, SGI47506, SGI47507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47508, SGI0, SGI0, SGI0, SGI47509, SGI47509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10776[] = {SGI47510, SGI47511, SGI47512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47513, SGI0, SGI0, SGI0, SGI47514, SGI47514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10777[] = {SGI47515, SGI47516, SGI47517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47518, SGI0, SGI0, SGI0, SGI47519, SGI47519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10778[] = {SGI47520, SGI47521, SGI47522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47523, SGI0, SGI0, SGI0, SGI47524, SGI47524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10779[] = {SGI47525, SGI47526, SGI47527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47528, SGI0, SGI0, SGI0, SGI47529, SGI47529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10780[] = {SGI47530, SGI47531, SGI47532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47533, SGI0, SGI0, SGI0, SGI47534, SGI47534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10781[] = {SGI47535, SGI47536, SGI47537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47538, SGI0, SGI0, SGI0, SGI47539, SGI47539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10782[] = {SGI47540, SGI47541, SGI47542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47543, SGI0, SGI0, SGI0, SGI47544, SGI47544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10783[] = {SGI47545, SGI47546, SGI47547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47548, SGI0, SGI0, SGI0, SGI47549, SGI47549, SGI0, SGI0, NULL};
const Segment * const SG10784[] = {SGI47550, SGI47551, SGI47552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47553, SGI0, SGI0, SGI0, SGI47554, SGI47554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10785[] = {SGI47555, SGI47556, SGI47557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47558, SGI0, SGI0, SGI0, SGI47559, SGI47559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10786[] = {SGI47560, SGI0, SGI47561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47562, SGI0, SGI0, SGI0, SGI47563, SGI47563, SGI0, SGI47564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10787[] = {SGI47565, SGI47566, SGI47567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47568, SGI0, SGI0, SGI0, SGI47569, SGI47569, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10788[] = {SGI47570, SGI47571, SGI47572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47573, SGI47574, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10789[] = {SGI47575, SGI47576, SGI47577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47578, SGI47578, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10790[] = {SGI0, SGI47579, SGI47580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47581, SGI0, SGI0, SGI0, SGI47582, SGI47582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10791[] = {SGI47583, SGI47584, SGI47585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47586, SGI0, SGI0, SGI0, SGI47587, SGI47587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10792[] = {SGI47588, SGI47589, SGI47590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47591, SGI47592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10793[] = {SGI47593, SGI47594, SGI47595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47596, SGI0, SGI0, SGI0, SGI47597, SGI47597, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10794[] = {SGI47598, SGI47599, SGI47600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47601, SGI0, SGI0, SGI0, SGI47602, SGI47602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10795[] = {SGI47603, SGI47604, SGI47605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47606, SGI47607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10796[] = {SGI47608, SGI47609, SGI47610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47611, SGI47611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10797[] = {SGI47612, SGI0, SGI47613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10798[] = {SGI47614, SGI47615, SGI47616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47617, SGI0, SGI0, SGI0, SGI47618, SGI47618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10799[] = {SGI47619, SGI47620, SGI47621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47622, SGI0, SGI0, SGI0, SGI47623, SGI47623, SGI0, SGI0, NULL};
const Segment * const SG10800[] = {SGI47624, SGI47625, SGI47626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47627, SGI0, SGI0, SGI0, SGI47628, SGI47628, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10801[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47629, SGI0, SGI0, SGI47630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10802[] = {SGI47631, SGI47632, SGI47633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47634, SGI0, SGI0, SGI0, SGI47635, SGI47635, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10803[] = {SGI47636, SGI47637, SGI47638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47639, SGI0, SGI0, SGI0, SGI47640, SGI47640, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10804[] = {SGI47641, SGI47642, SGI47643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47644, SGI0, SGI0, SGI0, SGI47645, SGI47645, SGI0, SGI0, NULL};
const Segment * const SG10805[] = {SGI47646, SGI47647, SGI47648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47649, SGI0, SGI0, SGI0, SGI47650, SGI47650, SGI0, SGI0, NULL};
const Segment * const SG10806[] = {SGI0, SGI47651, SGI47652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47653, SGI0, SGI0, SGI0, SGI47654, SGI47654, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10807[] = {SGI47655, SGI47656, SGI47657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47658, SGI0, SGI0, SGI0, SGI47659, SGI47659, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10808[] = {SGI47660, SGI47661, SGI47662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47663, SGI0, SGI0, SGI0, SGI47664, SGI47664, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10809[] = {SGI47665, SGI47666, SGI47667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47668, SGI0, SGI0, SGI0, SGI47669, SGI47669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10810[] = {SGI47670, SGI47671, SGI47672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47673, SGI47673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10811[] = {SGI47674, SGI47675, SGI47676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47677, SGI0, SGI0, SGI0, SGI47678, SGI47678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10812[] = {SGI47679, SGI47680, SGI47681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47682, SGI0, SGI0, SGI0, SGI47683, SGI47683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10813[] = {SGI47684, SGI47685, SGI47686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47687, SGI0, SGI0, SGI0, SGI47688, SGI47688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10814[] = {SGI47689, SGI47690, SGI47691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47692, SGI0, SGI0, SGI0, SGI47693, SGI47693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10815[] = {SGI47694, SGI47695, SGI47696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47697, SGI0, SGI0, SGI0, SGI47698, SGI47698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10816[] = {SGI47699, SGI47700, SGI47701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47702, SGI0, SGI0, SGI0, SGI47703, SGI47703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10817[] = {SGI47704, SGI47705, SGI47706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47709, SGI47709, SGI0, SGI0, NULL};
const Segment * const SG10818[] = {SGI47707, SGI0, SGI47708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10819[] = {SGI47710, SGI47711, SGI47712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47714, SGI0, SGI0, SGI0, SGI47718, SGI47718, SGI0, SGI0, NULL};
const Segment * const SG10820[] = {SGI47713, SGI47715, SGI47716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47721, SGI0, SGI0, SGI0, SGI47722, SGI47722, SGI0, SGI0, NULL};
const Segment * const SG10821[] = {SGI47717, SGI47719, SGI47720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47723, SGI0, SGI0, SGI0, SGI47724, SGI47724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10822[] = {SGI47725, SGI47726, SGI47727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47728, SGI0, SGI0, SGI0, SGI47729, SGI47729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10823[] = {SGI47730, SGI47731, SGI47732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47733, SGI0, SGI0, SGI0, SGI47734, SGI47734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10824[] = {SGI47735, SGI47736, SGI47737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47738, SGI0, SGI0, SGI0, SGI47739, SGI47739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10825[] = {SGI47740, SGI47741, SGI47742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47743, SGI47744, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10826[] = {SGI47745, SGI47746, SGI47747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47748, SGI47748, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10827[] = {SGI0, SGI47749, SGI47750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10828[] = {SGI47751, SGI0, SGI47752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47753, SGI0, SGI0, SGI0, SGI47754, SGI47754, SGI0, SGI0, SGI0, SGI47755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10829[] = {SGI47756, SGI47757, SGI47758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47759, SGI0, SGI0, SGI0, SGI47760, SGI47760, SGI0, SGI0, NULL};
const Segment * const SG10830[] = {SGI47761, SGI47762, SGI47763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47764, SGI0, SGI0, SGI0, SGI47765, SGI47765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10831[] = {SGI47766, SGI47767, SGI47768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47769, SGI0, SGI0, SGI0, SGI47770, SGI47770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10832[] = {SGI47771, SGI47772, SGI47773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47774, SGI0, SGI0, SGI0, SGI47775, SGI47775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10833[] = {SGI47776, SGI47777, SGI47778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47779, SGI0, SGI0, SGI0, SGI47780, SGI47780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10834[] = {SGI47781, SGI0, SGI47782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10835[] = {SGI47783, SGI47784, SGI47785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47786, SGI0, SGI0, SGI0, SGI47787, SGI47787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10836[] = {SGI47788, SGI47789, SGI47790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47791, SGI0, SGI0, SGI0, SGI47792, SGI47792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10837[] = {SGI47793, SGI0, SGI47794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47795, SGI0, SGI0, SGI0, SGI47796, SGI47796, SGI0, SGI47797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10838[] = {SGI0, SGI47798, SGI47799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47800, SGI0, SGI0, SGI0, SGI47801, SGI47801, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10839[] = {SGI0, SGI47802, SGI47803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47804, SGI0, SGI0, SGI0, SGI47805, SGI47805, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10840[] = {SGI47806, SGI47807, SGI47808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47809, SGI0, SGI0, SGI0, SGI47810, SGI47810, SGI0, SGI0, NULL};
const Segment * const SG10841[] = {SGI47811, SGI47812, SGI47813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47814, SGI47814, SGI0, SGI0, SGI0, SGI47815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10842[] = {SGI47816, SGI47817, SGI47818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47819, SGI0, SGI0, SGI0, SGI47820, SGI47820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10843[] = {SGI47821, SGI47822, SGI47823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47824, SGI0, SGI0, SGI0, SGI47825, SGI47825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10844[] = {SGI47826, SGI47827, SGI47828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47829, SGI0, SGI0, SGI0, SGI47830, SGI47830, SGI0, SGI0, NULL};
const Segment * const SG10845[] = {SGI47831, SGI47832, SGI47833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47834, SGI0, SGI0, SGI0, SGI47835, SGI47835, SGI0, SGI0, NULL};
const Segment * const SG10846[] = {SGI47836, SGI47837, SGI47838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47839, SGI0, SGI0, SGI0, SGI47840, SGI47840, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10847[] = {SGI47841, SGI47842, SGI47843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47844, SGI0, SGI0, SGI0, SGI47845, SGI47845, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10848[] = {SGI47846, SGI47847, SGI47848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47849, SGI0, SGI0, SGI0, SGI47850, SGI47850, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10849[] = {SGI47851, SGI47852, SGI47853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47854, SGI47855, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10850[] = {SGI47856, SGI47857, SGI47858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47859, SGI47859, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10851[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47860, SGI0, SGI0, SGI47861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10852[] = {SGI47862, SGI47863, SGI47864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47865, SGI0, SGI0, SGI0, SGI47866, SGI47866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10853[] = {SGI47867, SGI47868, SGI47869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47870, SGI0, SGI0, SGI0, SGI47871, SGI47871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10854[] = {SGI47872, SGI47873, SGI47874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47875, SGI47875, SGI0, SGI47876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10855[] = {SGI47877, SGI47878, SGI47879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47880, SGI0, SGI0, SGI0, SGI47881, SGI47881, SGI0, SGI0, NULL};
const Segment * const SG10856[] = {SGI47882, SGI47883, SGI47884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47885, SGI0, SGI0, SGI0, SGI47886, SGI47886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10857[] = {SGI47887, SGI0, SGI47888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10858[] = {SGI47889, SGI47890, SGI47891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47892, SGI0, SGI0, SGI0, SGI47893, SGI47893, SGI0, SGI0, NULL};
const Segment * const SG10859[] = {SGI47894, SGI47895, SGI47896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47897, SGI47897, SGI0, SGI0, SGI0, SGI47898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10860[] = {SGI47899, SGI47900, SGI47901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47902, SGI0, SGI0, SGI0, SGI47903, SGI47903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10861[] = {SGI47904, SGI47905, SGI47906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47907, SGI0, SGI0, SGI0, SGI47908, SGI47908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10862[] = {SGI47909, SGI47910, SGI47911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47912, SGI47912, SGI0, SGI0, SGI0, SGI47913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10863[] = {SGI47914, SGI0, SGI47915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47916, SGI0, SGI0, SGI0, SGI47917, SGI47917, SGI0, SGI0, SGI0, SGI47918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10864[] = {SGI47919, SGI0, SGI47920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47921, SGI0, SGI0, SGI0, SGI47922, SGI47922, SGI0, SGI0, SGI0, SGI47923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10865[] = {SGI47924, SGI47925, SGI47926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47927, SGI0, SGI0, SGI0, SGI47928, SGI47928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10866[] = {SGI47929, SGI47930, SGI47931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47932, SGI0, SGI0, SGI0, SGI47933, SGI47933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10867[] = {SGI47934, SGI47935, SGI47936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47937, SGI0, SGI0, SGI0, SGI47938, SGI47938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10868[] = {SGI47939, SGI47940, SGI47941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47942, SGI0, SGI0, SGI0, SGI47943, SGI47943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10869[] = {SGI47944, SGI47945, SGI47946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47947, SGI0, SGI0, SGI0, SGI47948, SGI47948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10870[] = {SGI47949, SGI47950, SGI47951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47952, SGI0, SGI0, SGI0, SGI47953, SGI47953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10871[] = {SGI47954, SGI0, SGI47955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10872[] = {SGI47956, SGI47957, SGI47958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47959, SGI0, SGI0, SGI0, SGI47960, SGI47960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10873[] = {SGI47961, SGI0, SGI47962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47963, SGI0, SGI0, SGI0, SGI47964, SGI47964, SGI0, SGI0, SGI0, SGI47965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10874[] = {SGI47966, SGI0, SGI47967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47968, SGI0, SGI0, SGI0, SGI47969, SGI47969, SGI0, SGI47970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10875[] = {SGI47971, SGI47972, SGI47973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47974, SGI0, SGI0, SGI0, SGI47975, SGI47975, SGI0, SGI47976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10876[] = {SGI47977, SGI47978, SGI47979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47980, SGI0, SGI0, SGI0, SGI47981, SGI47981, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10877[] = {SGI47982, SGI47983, SGI47984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47985, SGI47986, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10878[] = {SGI47987, SGI47988, SGI47989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47990, SGI0, SGI0, SGI0, SGI47991, SGI47991, SGI0, SGI0, NULL};
const Segment * const SG10879[] = {SGI47992, SGI47993, SGI47994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI47995, SGI0, SGI0, SGI0, SGI47996, SGI47996, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10880[] = {SGI47997, SGI0, SGI47998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10881[] = {SGI47999, SGI0, SGI48000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48001, SGI48002, SGI48002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48003, NULL};
const Segment * const SG10882[] = {SGI48004, SGI0, SGI48005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48006, SGI48007, SGI48007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48008, NULL};
const Segment * const SG10883[] = {SGI48009, SGI48010, SGI48011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48012, SGI0, SGI0, SGI0, SGI48013, SGI48013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10884[] = {SGI48014, SGI48015, SGI48016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48017, SGI48018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10885[] = {SGI48019, SGI48020, SGI48021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48022, SGI0, SGI0, SGI0, SGI48023, SGI48023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10886[] = {SGI48024, SGI48025, SGI48026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48027, SGI48027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10887[] = {SGI48028, SGI0, SGI48029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10888[] = {SGI48030, SGI48031, SGI48032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48033, SGI0, SGI0, SGI0, SGI48034, SGI48034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10889[] = {SGI48035, SGI48036, SGI48037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48038, SGI0, SGI0, SGI0, SGI48039, SGI48039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10890[] = {SGI48040, SGI48041, SGI48042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48043, SGI48043, SGI0, SGI0, SGI0, SGI48044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10891[] = {SGI48045, SGI48046, SGI48047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48048, SGI0, SGI0, SGI0, SGI48049, SGI48049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10892[] = {SGI48050, SGI48051, SGI48052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48053, SGI0, SGI0, SGI0, SGI48054, SGI48054, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10893[] = {SGI48055, SGI0, SGI48056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48057, SGI0, SGI0, SGI0, SGI48058, SGI48058, SGI0, SGI0, SGI0, SGI48059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10894[] = {SGI48060, SGI48061, SGI48062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48063, SGI0, SGI0, SGI0, SGI48064, SGI48064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10895[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48065, SGI0, SGI0, SGI48066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10896[] = {SGI48067, SGI48068, SGI48069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48070, SGI0, SGI0, SGI0, SGI48071, SGI48071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10897[] = {SGI48072, SGI48073, SGI48074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48075, SGI0, SGI0, SGI0, SGI48076, SGI48076, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10898[] = {SGI48077, SGI48078, SGI48079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48080, SGI0, SGI0, SGI0, SGI48081, SGI48081, SGI0, SGI0, NULL};
const Segment * const SG10899[] = {SGI48082, SGI0, SGI48083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48084, SGI0, SGI0, SGI0, SGI48085, SGI48085, SGI0, SGI0, SGI0, SGI48086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10900[] = {SGI48087, SGI0, SGI48088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48089, SGI0, SGI0, SGI0, SGI48090, SGI48090, SGI0, SGI0, SGI0, SGI48091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10901[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48092, SGI0, SGI0, SGI48093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10902[] = {SGI48094, SGI48095, SGI48096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48097, SGI0, SGI0, SGI0, SGI48098, SGI48098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10903[] = {SGI48099, SGI48100, SGI48101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48102, SGI0, SGI0, SGI0, SGI48103, SGI48103, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10904[] = {SGI48104, SGI48105, SGI48106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48107, SGI0, SGI0, SGI0, SGI48108, SGI48108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10905[] = {SGI48109, SGI0, SGI48110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10906[] = {SGI48111, SGI48112, SGI48113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48114, SGI48114, SGI0, SGI0, SGI0, SGI48115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10907[] = {SGI0, SGI48116, SGI48117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48118, SGI0, SGI0, SGI0, SGI48119, SGI48119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10908[] = {SGI48120, SGI48121, SGI48122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48123, SGI0, SGI0, SGI0, SGI48124, SGI48124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10909[] = {SGI48125, SGI48126, SGI48127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48128, SGI48129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10910[] = {SGI48130, SGI48131, SGI48132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48133, SGI48133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10911[] = {SGI48134, SGI48135, SGI48136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48137, SGI0, SGI0, SGI0, SGI48138, SGI48138, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10912[] = {SGI48139, SGI48140, SGI48141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48142, SGI0, SGI0, SGI0, SGI48143, SGI48143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10913[] = {SGI48144, SGI0, SGI48145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48146, SGI0, SGI0, SGI0, SGI48147, SGI48147, SGI0, SGI0, SGI0, SGI48148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10914[] = {SGI0, SGI48149, SGI48150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48151, SGI0, SGI0, SGI0, SGI48152, SGI48152, SGI0, SGI0, NULL};
const Segment * const SG10915[] = {SGI0, SGI0, SGI48153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48154, SGI0, SGI0, SGI0, SGI48155, SGI48155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10916[] = {SGI48156, SGI48157, SGI48158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48159, SGI0, SGI0, SGI0, SGI48160, SGI48160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10917[] = {SGI48161, SGI0, SGI48162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48163, SGI0, SGI0, SGI0, SGI48164, SGI48164, SGI0, SGI0, SGI0, SGI48165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10918[] = {SGI48166, SGI48167, SGI48168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48169, SGI0, SGI0, SGI0, SGI48170, SGI48170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10919[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48171, SGI0, SGI0, SGI48172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10920[] = {SGI48173, SGI48174, SGI48175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48176, SGI0, SGI0, SGI0, SGI48177, SGI48177, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10921[] = {SGI48178, SGI48179, SGI48180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48181, SGI0, SGI0, SGI0, SGI48182, SGI48182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10922[] = {SGI48183, SGI48184, SGI48185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48186, SGI0, SGI0, SGI0, SGI48187, SGI48187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10923[] = {SGI48188, SGI48189, SGI48190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48191, SGI0, SGI0, SGI0, SGI48192, SGI48192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10924[] = {SGI48193, SGI48194, SGI48195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48196, SGI0, SGI0, SGI0, SGI48197, SGI48197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10925[] = {SGI48198, SGI0, SGI48199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48200, SGI0, SGI0, SGI0, SGI48201, SGI48201, SGI0, SGI48202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10926[] = {SGI48203, SGI48204, SGI48205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48206, SGI0, SGI0, SGI0, SGI48207, SGI48207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10927[] = {SGI48208, SGI48209, SGI48210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48211, SGI0, SGI0, SGI0, SGI48212, SGI48212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10928[] = {SGI48213, SGI0, SGI48214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10929[] = {SGI48215, SGI48216, SGI48217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48218, SGI0, SGI0, SGI0, SGI48219, SGI48219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10930[] = {SGI48221, SGI48223, SGI48225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48227, SGI0, SGI0, SGI0, SGI48229, SGI48229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10931[] = {SGI48220, SGI48222, SGI48224, SGI0, SGI0, SGI0, SGI48226, SGI48228, SGI0, SGI48230, SGI0, SGI0, SGI0, SGI48231, SGI48231, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10932[] = {SGI48232, SGI0, SGI48233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10933[] = {SGI48234, SGI48235, SGI48236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48237, SGI0, SGI0, SGI0, SGI48238, SGI48238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10934[] = {SGI48239, SGI0, SGI48240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48241, SGI0, SGI0, SGI0, SGI48242, SGI48242, SGI0, SGI0, SGI0, SGI48243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10935[] = {SGI48244, SGI0, SGI48245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48246, SGI0, SGI0, SGI0, SGI48247, SGI48247, SGI0, SGI0, SGI0, SGI48248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10936[] = {SGI48249, SGI48250, SGI48251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48252, SGI0, SGI0, SGI0, SGI48253, SGI48253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10937[] = {SGI48254, SGI48255, SGI48256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48257, SGI0, SGI0, SGI0, SGI48258, SGI48258, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10938[] = {SGI48259, SGI48260, SGI48261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48264, SGI0, SGI0, SGI0, SGI48265, SGI48265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10939[] = {SGI0, SGI48262, SGI48263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48266, SGI48266, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10940[] = {SGI48267, SGI48268, SGI48269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48270, SGI48271, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10941[] = {SGI48272, SGI0, SGI48273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48274, SGI0, SGI0, SGI0, SGI48275, SGI48275, SGI0, SGI0, SGI0, SGI48276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10942[] = {SGI0, SGI48277, SGI0, SGI48278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10943[] = {SGI48279, SGI48280, SGI48281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48282, SGI0, SGI0, SGI0, SGI48283, SGI48283, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10944[] = {SGI48284, SGI48285, SGI48286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48287, SGI48288, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10945[] = {SGI48289, SGI48290, SGI48291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48292, SGI0, SGI0, SGI0, SGI48293, SGI48293, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10946[] = {SGI48294, SGI48295, SGI48296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48297, SGI48297, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10947[] = {SGI0, SGI0, SGI48298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48299, SGI0, SGI0, SGI0, SGI48300, SGI48300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10948[] = {SGI48301, SGI48302, SGI48303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48304, SGI48304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10949[] = {SGI0, SGI48305, SGI48306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10950[] = {SGI48307, SGI0, SGI48308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10951[] = {SGI48309, SGI48310, SGI48311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48312, SGI0, SGI0, SGI0, SGI48313, SGI48313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10952[] = {SGI0, SGI48314, SGI0, SGI48315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10953[] = {SGI48316, SGI48317, SGI48318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48319, SGI0, SGI0, SGI0, SGI48320, SGI48320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10954[] = {SGI48321, SGI48322, SGI48323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48324, SGI0, SGI0, SGI0, SGI48325, SGI48325, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10955[] = {SGI48326, SGI48327, SGI48328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48329, SGI0, SGI0, SGI0, SGI48330, SGI48330, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10956[] = {SGI48331, SGI48332, SGI48333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48334, SGI0, SGI0, SGI0, SGI48335, SGI48335, SGI0, SGI0, NULL};
const Segment * const SG10957[] = {SGI48336, SGI48337, SGI48338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48339, SGI0, SGI0, SGI0, SGI48340, SGI48340, SGI0, SGI0, NULL};
const Segment * const SG10958[] = {SGI48341, SGI48342, SGI48343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48344, SGI0, SGI0, SGI0, SGI48345, SGI48345, SGI0, SGI0, NULL};
const Segment * const SG10959[] = {SGI48346, SGI0, SGI48347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48348, SGI0, SGI0, SGI0, SGI48349, SGI48349, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10960[] = {SGI48350, SGI48351, SGI48352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48353, SGI0, SGI0, SGI0, SGI48354, SGI48354, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10961[] = {SGI48355, SGI48356, SGI48357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48358, SGI48358, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10962[] = {SGI0, SGI48359, SGI0, SGI48360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48361, SGI0, SGI0, SGI0, SGI48362, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10963[] = {SGI0, SGI48363, SGI0, SGI48364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10964[] = {SGI48365, SGI48366, SGI48367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48368, SGI0, SGI0, SGI0, SGI48369, SGI48369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10965[] = {SGI48370, SGI48371, SGI48372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48373, SGI0, SGI0, SGI0, SGI48374, SGI48374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10966[] = {SGI48375, SGI48376, SGI48377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48378, SGI0, SGI0, SGI0, SGI48379, SGI48379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10967[] = {SGI48380, SGI48381, SGI48382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48383, SGI0, SGI0, SGI0, SGI48384, SGI48384, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10968[] = {SGI48385, SGI0, SGI48386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48387, SGI0, SGI0, SGI0, SGI48391, SGI48391, SGI0, SGI0, SGI0, SGI48392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10969[] = {SGI48388, SGI48389, SGI48390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48393, SGI0, SGI0, SGI0, SGI48394, SGI48394, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10970[] = {SGI48395, SGI48396, SGI48397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48398, SGI0, SGI0, SGI0, SGI48399, SGI48399, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10971[] = {SGI48400, SGI48401, SGI48402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48403, SGI0, SGI0, SGI0, SGI48404, SGI48404, SGI0, SGI0, NULL};
const Segment * const SG10972[] = {SGI48405, SGI48406, SGI48407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48408, SGI0, SGI0, SGI0, SGI48409, SGI48409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10973[] = {SGI48410, SGI48411, SGI48412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48413, SGI0, SGI0, SGI0, SGI48414, SGI48414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10974[] = {SGI48415, SGI48416, SGI48417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48418, SGI48419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10975[] = {SGI48420, SGI0, SGI48421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48422, SGI0, SGI0, SGI0, SGI48423, SGI48423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10976[] = {SGI48424, SGI0, SGI48425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10977[] = {SGI48428, SGI48430, SGI48431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48433, SGI48433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10978[] = {SGI48426, SGI48427, SGI48429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48432, SGI0, SGI0, SGI0, SGI48434, SGI48434, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10979[] = {SGI48435, SGI48436, SGI48437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48438, SGI0, SGI0, SGI0, SGI48439, SGI48439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10980[] = {SGI48440, SGI48441, SGI48442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48443, SGI0, SGI0, SGI0, SGI48444, SGI48444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10981[] = {SGI48445, SGI48446, SGI48447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48448, SGI0, SGI0, SGI0, SGI48449, SGI48449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10982[] = {SGI48450, SGI48451, SGI48452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48453, SGI0, SGI0, SGI0, SGI48454, SGI48454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10983[] = {SGI48455, SGI48456, SGI48457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48458, SGI0, SGI0, SGI0, SGI48459, SGI48459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10984[] = {SGI48460, SGI48461, SGI48462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48463, SGI0, SGI0, SGI0, SGI48464, SGI48464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10985[] = {SGI48465, SGI48466, SGI48467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48468, SGI0, SGI0, SGI0, SGI48469, SGI48469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10986[] = {SGI48470, SGI48471, SGI48472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10987[] = {SGI48473, SGI0, SGI48474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10988[] = {SGI48475, SGI0, SGI48476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48477, SGI0, SGI0, SGI0, SGI48478, SGI48478, SGI0, SGI0, SGI48480, SGI0, NULL};
const Segment * const SG10989[] = {SGI48479, SGI48481, SGI48482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48483, SGI0, SGI0, SGI0, SGI48484, SGI48484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10990[] = {SGI48485, SGI0, SGI48486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48487, SGI0, SGI0, SGI0, SGI48488, SGI48488, SGI0, SGI0, SGI0, SGI48489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10991[] = {SGI48490, SGI0, SGI48491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48492, SGI0, SGI0, SGI0, SGI48493, SGI48493, SGI0, SGI0, SGI0, SGI48494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10992[] = {SGI48495, SGI48496, SGI48497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48498, SGI0, SGI0, SGI0, SGI48499, SGI48499, SGI0, SGI0, NULL};
const Segment * const SG10993[] = {SGI48500, SGI48501, SGI48502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48503, SGI0, SGI0, SGI0, SGI48504, SGI48504, SGI0, SGI0, NULL};
const Segment * const SG10994[] = {SGI48505, SGI48506, SGI48507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48508, SGI0, SGI0, SGI0, SGI48509, SGI48509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10995[] = {SGI48510, SGI48511, SGI48512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48513, SGI0, SGI0, SGI0, SGI48514, SGI48514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10996[] = {SGI0, SGI48515, SGI48516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48517, SGI48517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10997[] = {SGI48518, SGI48519, SGI48520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48521, SGI48522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10998[] = {SGI48523, SGI48524, SGI48525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48526, SGI0, SGI0, SGI0, SGI48527, SGI48527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG10999[] = {SGI48528, SGI48529, SGI48530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48531, SGI0, SGI0, SGI0, SGI48532, SGI48532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11000[] = {SGI0, SGI48533, SGI0, SGI48534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11001[] = {SGI48535, SGI48536, SGI48537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48538, SGI0, SGI0, SGI0, SGI48539, SGI48539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11002[] = {SGI48540, SGI48541, SGI48542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48543, SGI0, SGI0, SGI0, SGI48544, SGI48544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11003[] = {SGI48545, SGI0, SGI48546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48547, SGI0, SGI0, SGI0, SGI48548, SGI48548, SGI0, SGI48549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11004[] = {SGI48550, SGI48551, SGI48552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48553, SGI0, SGI0, SGI0, SGI48554, SGI48554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11005[] = {SGI48555, SGI0, SGI48556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11006[] = {SGI48557, SGI48558, SGI48559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48560, SGI0, SGI0, SGI0, SGI48561, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11007[] = {SGI48562, SGI48563, SGI48564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48565, SGI0, SGI0, SGI0, SGI48566, SGI48566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11008[] = {SGI48567, SGI0, SGI48568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48569, SGI0, SGI0, SGI0, SGI48570, SGI48570, SGI0, SGI0, SGI0, SGI48571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11009[] = {SGI48572, SGI48573, SGI48574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48578, SGI0, SGI0, SGI0, SGI48579, SGI48579, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11010[] = {SGI48575, SGI48576, SGI48577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48580, SGI0, SGI0, SGI0, SGI48581, SGI48581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11011[] = {SGI48582, SGI48583, SGI48584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48585, SGI0, SGI0, SGI0, SGI48586, SGI48586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11012[] = {SGI48587, SGI48588, SGI48589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48590, SGI0, SGI0, SGI0, SGI48591, SGI48591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11013[] = {SGI48592, SGI48593, SGI48594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48595, SGI0, SGI0, SGI0, SGI48596, SGI48596, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11014[] = {SGI48597, SGI48598, SGI48599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48600, SGI0, SGI0, SGI0, SGI48601, SGI48601, SGI0, SGI0, NULL};
const Segment * const SG11015[] = {SGI48602, SGI48603, SGI48604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48605, SGI0, SGI0, SGI0, SGI48606, SGI48606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11016[] = {SGI48607, SGI0, SGI48608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11017[] = {SGI48609, SGI48610, SGI48611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48612, SGI0, SGI0, SGI0, SGI48613, SGI48613, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11018[] = {SGI48614, SGI48615, SGI48616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48617, SGI0, SGI0, SGI0, SGI48618, SGI48618, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11019[] = {SGI48619, SGI48620, SGI48621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48622, SGI0, SGI0, SGI0, SGI48623, SGI48623, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11020[] = {SGI48624, SGI48625, SGI48626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48627, SGI0, SGI0, SGI0, SGI48628, SGI48628, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11021[] = {SGI48629, SGI48630, SGI48631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48632, SGI0, SGI0, SGI0, SGI48633, SGI48633, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11022[] = {SGI48634, SGI48635, SGI48636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48637, SGI0, SGI0, SGI0, SGI48638, SGI48638, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11023[] = {SGI48639, SGI48640, SGI48641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48642, SGI0, SGI0, SGI0, SGI48643, SGI48643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11024[] = {SGI48644, SGI48645, SGI48646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48647, SGI48647, SGI0, SGI0, SGI0, SGI48648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11025[] = {SGI48649, SGI48650, SGI48651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48652, SGI0, SGI0, SGI0, SGI48653, SGI48653, SGI0, SGI0, NULL};
const Segment * const SG11026[] = {SGI48654, SGI48655, SGI48656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48657, SGI0, SGI0, SGI0, SGI48658, SGI48658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11027[] = {SGI48659, SGI48660, SGI48661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48662, SGI0, SGI0, SGI0, SGI48663, SGI48663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11028[] = {SGI48664, SGI48665, SGI48666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48667, SGI0, SGI0, SGI0, SGI48668, SGI48668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11029[] = {SGI48669, SGI48670, SGI48671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48672, SGI0, SGI0, SGI0, SGI48673, SGI48673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11030[] = {SGI48674, SGI48675, SGI48676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48677, SGI0, SGI0, SGI0, SGI48678, SGI48678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11031[] = {SGI48679, SGI0, SGI48680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11032[] = {SGI48681, SGI48682, SGI48683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48684, SGI0, SGI0, SGI0, SGI48685, SGI48685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11033[] = {SGI48686, SGI48687, SGI48688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48689, SGI0, SGI0, SGI0, SGI48690, SGI48690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11034[] = {SGI48691, SGI48692, SGI48693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48694, SGI0, SGI0, SGI0, SGI48695, SGI48695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11035[] = {SGI48696, SGI48697, SGI48698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48699, SGI0, SGI0, SGI0, SGI48700, SGI48700, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11036[] = {SGI48701, SGI48702, SGI48703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48704, SGI0, SGI0, SGI0, SGI48705, SGI48705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11037[] = {SGI48706, SGI48707, SGI48708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48709, SGI0, SGI0, SGI0, SGI48710, SGI48710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11038[] = {SGI48711, SGI0, SGI48712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11039[] = {SGI48713, SGI48714, SGI48715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48716, SGI0, SGI0, SGI0, SGI48717, SGI48717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11040[] = {SGI0, SGI48718, SGI48719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48720, SGI0, SGI0, SGI0, SGI0, SGI48721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11041[] = {SGI0, SGI48722, SGI48723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48724, SGI0, SGI0, SGI0, SGI48725, SGI48725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11042[] = {SGI48726, SGI48727, SGI48728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48729, SGI0, SGI0, SGI0, SGI48730, SGI48730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11043[] = {SGI48731, SGI48732, SGI48733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48734, SGI0, SGI0, SGI0, SGI48735, SGI48735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11044[] = {SGI48736, SGI48737, SGI48738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48739, SGI48739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11045[] = {SGI48740, SGI48741, SGI48742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48743, SGI0, SGI0, SGI0, SGI48744, SGI48744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11046[] = {SGI48745, SGI48746, SGI48747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48748, SGI0, SGI0, SGI0, SGI48749, SGI48749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11047[] = {SGI0, SGI48750, SGI0, SGI48751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11048[] = {SGI0, SGI48752, SGI0, SGI48753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48754, SGI0, SGI0, SGI0, SGI48755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11049[] = {SGI48756, SGI48757, SGI48758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48759, SGI0, SGI0, SGI0, SGI48760, SGI48760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11050[] = {SGI48761, SGI0, SGI48762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11051[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48763, SGI0, SGI0, SGI48764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11052[] = {SGI48765, SGI48766, SGI48767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48768, SGI0, SGI0, SGI0, SGI48769, SGI48769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11053[] = {SGI48770, SGI48771, SGI48772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48773, SGI0, SGI0, SGI0, SGI48774, SGI48774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11054[] = {SGI48775, SGI48776, SGI48777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48778, SGI0, SGI0, SGI0, SGI48779, SGI48779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11055[] = {SGI48780, SGI48781, SGI48782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48783, SGI0, SGI0, SGI0, SGI48784, SGI48784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11056[] = {SGI48785, SGI48786, SGI48787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48788, SGI0, SGI0, SGI0, SGI48789, SGI48789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11057[] = {SGI48790, SGI0, SGI48791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11058[] = {SGI48792, SGI48793, SGI48794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48795, SGI0, SGI0, SGI0, SGI48796, SGI48796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11059[] = {SGI48797, SGI48798, SGI48799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48800, SGI0, SGI0, SGI0, SGI48801, SGI48801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11060[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48802, SGI0, SGI0, SGI48803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11061[] = {SGI48804, SGI48805, SGI48806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48807, SGI0, SGI0, SGI0, SGI48808, SGI48808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11062[] = {SGI48809, SGI48810, SGI48811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48812, SGI0, SGI0, SGI0, SGI48813, SGI48813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11063[] = {SGI48814, SGI48815, SGI48816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48817, SGI0, SGI0, SGI0, SGI48818, SGI48818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11064[] = {SGI48819, SGI48820, SGI48821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48822, SGI0, SGI0, SGI0, SGI48823, SGI48823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11065[] = {SGI48824, SGI0, SGI48825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11066[] = {SGI48826, SGI48827, SGI48828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48829, SGI0, SGI0, SGI0, SGI48830, SGI48830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11067[] = {SGI48836, SGI48837, SGI48838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48839, SGI0, SGI0, SGI0, SGI48840, SGI48840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11068[] = {SGI48841, SGI48842, SGI48843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48844, SGI0, SGI0, SGI0, SGI48845, SGI48845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11069[] = {SGI0, SGI48847, SGI48848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48849, SGI0, SGI0, SGI0, SGI48850, SGI48850, SGI0, SGI0, NULL};
const Segment * const SG11070[] = {SGI48851, SGI48852, SGI48853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48854, SGI48854, SGI0, SGI48855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11071[] = {SGI48831, SGI48832, SGI48833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48834, SGI0, SGI0, SGI0, SGI48835, SGI48835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11072[] = {SGI48856, SGI0, SGI48857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48858, SGI0, SGI0, SGI0, SGI48859, SGI48859, SGI0, SGI0, SGI0, SGI48860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11073[] = {SGI48861, SGI48862, SGI48863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48864, SGI0, SGI0, SGI0, SGI48865, SGI48865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11074[] = {SGI48866, SGI48867, SGI48868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48869, SGI0, SGI0, SGI0, SGI48870, SGI48870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11075[] = {SGI48871, SGI48872, SGI48873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48874, SGI0, SGI0, SGI0, SGI48875, SGI48875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11076[] = {SGI48876, SGI48877, SGI48878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48879, SGI0, SGI0, SGI0, SGI48880, SGI48880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11077[] = {SGI48881, SGI48882, SGI48883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48884, SGI0, SGI0, SGI0, SGI48885, SGI48885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11078[] = {SGI48886, SGI48887, SGI48888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48889, SGI0, SGI0, SGI0, SGI48890, SGI48890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11079[] = {SGI48891, SGI48892, SGI48893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48894, SGI0, SGI0, SGI0, SGI48895, SGI48895, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11080[] = {SGI48896, SGI48897, SGI48898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48899, SGI0, SGI0, SGI0, SGI48900, SGI48900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11081[] = {SGI48901, SGI48902, SGI48903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48904, SGI0, SGI0, SGI0, SGI48905, SGI48905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11082[] = {SGI48906, SGI48907, SGI48908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48909, SGI0, SGI0, SGI0, SGI48910, SGI48910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11083[] = {SGI48911, SGI48912, SGI48913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48914, SGI0, SGI0, SGI0, SGI48915, SGI48915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11084[] = {SGI0, SGI48916, SGI48917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48918, SGI0, SGI0, SGI0, SGI48919, SGI48919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11085[] = {SGI48920, SGI48921, SGI48922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48923, SGI0, SGI0, SGI0, SGI48924, SGI48924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11086[] = {SGI0, SGI48925, SGI48926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48927, SGI0, SGI0, SGI0, SGI48928, SGI48928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11087[] = {SGI48929, SGI48930, SGI48931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48932, SGI48933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11088[] = {SGI48934, SGI48935, SGI48936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48937, SGI48937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11089[] = {SGI48938, SGI48939, SGI48940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48941, SGI0, SGI0, SGI0, SGI48942, SGI48942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11090[] = {SGI48943, SGI48944, SGI48945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48946, SGI0, SGI0, SGI0, SGI48947, SGI48947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11091[] = {SGI48948, SGI48949, SGI48950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48951, SGI0, SGI0, SGI0, SGI48952, SGI48952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11092[] = {SGI48953, SGI48954, SGI48955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48956, SGI0, SGI0, SGI0, SGI48957, SGI48957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11093[] = {SGI48958, SGI48959, SGI48960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48961, SGI0, SGI0, SGI0, SGI48962, SGI48962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11094[] = {SGI48963, SGI48964, SGI48965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48966, SGI0, SGI0, SGI0, SGI48967, SGI48967, SGI0, SGI0, NULL};
const Segment * const SG11095[] = {SGI48968, SGI48969, SGI48970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48971, SGI0, SGI0, SGI0, SGI48972, SGI48972, SGI0, SGI0, NULL};
const Segment * const SG11096[] = {SGI48973, SGI48974, SGI48975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48976, SGI48977, SGI0, SGI0, NULL};
const Segment * const SG11097[] = {SGI48978, SGI48979, SGI48980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48981, SGI48981, SGI0, SGI0, NULL};
const Segment * const SG11098[] = {SGI48982, SGI48983, SGI48984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48985, SGI0, SGI0, SGI0, SGI48986, SGI48986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11099[] = {SGI48987, SGI0, SGI48988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48989, SGI0, SGI0, SGI0, SGI48990, SGI48990, SGI0, SGI0, SGI0, SGI48991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11100[] = {SGI0, SGI0, SGI48992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48993, SGI48994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11101[] = {SGI48995, SGI48996, SGI48997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI48998, SGI0, SGI0, SGI0, SGI48999, SGI48999, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11102[] = {SGI49000, SGI49001, SGI49002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49003, SGI0, SGI0, SGI0, SGI49004, SGI49004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11103[] = {SGI49005, SGI49006, SGI49007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49008, SGI0, SGI0, SGI0, SGI49009, SGI49009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11104[] = {SGI49010, SGI49011, SGI49012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49013, SGI49013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11105[] = {SGI49014, SGI49015, SGI49016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49017, SGI49018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11106[] = {SGI49019, SGI0, SGI49020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11107[] = {SGI0, SGI49021, SGI49022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11108[] = {SGI49023, SGI49024, SGI49025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49026, SGI0, SGI0, SGI0, SGI49027, SGI49027, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11109[] = {SGI49028, SGI49029, SGI49030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49031, SGI0, SGI0, SGI0, SGI49032, SGI49032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11110[] = {SGI49033, SGI49034, SGI49035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49036, SGI0, SGI0, SGI0, SGI49037, SGI49037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11111[] = {SGI49038, SGI49039, SGI49040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49041, SGI0, SGI0, SGI0, SGI49042, SGI49042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11112[] = {SGI49043, SGI49044, SGI49045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49046, SGI0, SGI0, SGI0, SGI49047, SGI49047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11113[] = {SGI49048, SGI49049, SGI49050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49051, SGI0, SGI0, SGI0, SGI49052, SGI49052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11114[] = {SGI49053, SGI0, SGI49054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11115[] = {SGI49055, SGI49056, SGI49057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49058, SGI0, SGI0, SGI0, SGI49059, SGI49059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11116[] = {SGI49060, SGI49061, SGI49062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49063, SGI0, SGI0, SGI0, SGI49064, SGI49064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11117[] = {SGI49065, SGI49066, SGI49067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49068, SGI0, SGI0, SGI0, SGI49069, SGI49069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11118[] = {SGI49070, SGI49071, SGI49072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49073, SGI0, SGI0, SGI0, SGI49074, SGI49074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11119[] = {SGI49075, SGI49076, SGI49077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49078, SGI0, SGI0, SGI0, SGI49079, SGI49079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11120[] = {SGI49080, SGI49081, SGI49082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49083, SGI0, SGI0, SGI0, SGI49084, SGI49084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11121[] = {SGI49085, SGI49086, SGI49087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49088, SGI0, SGI0, SGI0, SGI49089, SGI49089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11122[] = {SGI49090, SGI49091, SGI49092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49093, SGI0, SGI0, SGI0, SGI49094, SGI49094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11123[] = {SGI49095, SGI49096, SGI49097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49098, SGI0, SGI0, SGI0, SGI49099, SGI49099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11124[] = {SGI49100, SGI49101, SGI49102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49103, SGI0, SGI0, SGI0, SGI49104, SGI49104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11125[] = {SGI49106, SGI49107, SGI49108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49109, SGI0, SGI0, SGI0, SGI49110, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11126[] = {SGI0, SGI49111, SGI0, SGI49112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11127[] = {SGI49113, SGI49114, SGI49115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49116, SGI0, SGI0, SGI0, SGI49117, SGI49117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11128[] = {SGI49118, SGI49119, SGI49120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49121, SGI0, SGI0, SGI0, SGI49122, SGI49122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11129[] = {SGI49123, SGI49124, SGI49125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49126, SGI0, SGI0, SGI0, SGI49127, SGI49127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11130[] = {SGI49128, SGI49129, SGI49130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49131, SGI0, SGI0, SGI0, SGI49132, SGI49132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11131[] = {SGI49133, SGI49134, SGI49135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49136, SGI0, SGI0, SGI0, SGI49137, SGI49137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11132[] = {SGI49138, SGI49139, SGI49140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49141, SGI0, SGI0, SGI0, SGI49142, SGI49142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11133[] = {SGI49143, SGI49144, SGI49145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49147, SGI0, SGI0, SGI0, SGI49150, SGI49150, SGI0, SGI0, NULL};
const Segment * const SG11134[] = {SGI49146, SGI49148, SGI49149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49151, SGI0, SGI0, SGI0, SGI49152, SGI49152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11135[] = {SGI49153, SGI49154, SGI49155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49159, SGI0, SGI0, SGI0, SGI49161, SGI49162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11136[] = {SGI49156, SGI49157, SGI49158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49160, SGI0, SGI0, SGI0, SGI49163, SGI49163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11137[] = {SGI49164, SGI49165, SGI49166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49167, SGI0, SGI0, SGI0, SGI49168, SGI49168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11138[] = {SGI49169, SGI49170, SGI49171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49172, SGI0, SGI0, SGI0, SGI49173, SGI49173, SGI0, SGI49174, SGI0, SGI0, NULL};
const Segment * const SG11139[] = {SGI49175, SGI49176, SGI49177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49178, SGI0, SGI0, SGI0, SGI49179, SGI49179, SGI0, SGI49180, SGI0, SGI0, NULL};
const Segment * const SG11140[] = {SGI49181, SGI49182, SGI49183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49184, SGI0, SGI0, SGI0, SGI49185, SGI49185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11141[] = {SGI49186, SGI49187, SGI49188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49189, SGI0, SGI0, SGI0, SGI49190, SGI49190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11142[] = {SGI49191, SGI0, SGI49192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49193, SGI0, SGI0, SGI0, SGI49194, SGI49194, SGI0, SGI49195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11143[] = {SGI49196, SGI49197, SGI49198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49199, SGI0, SGI0, SGI0, SGI49200, SGI49200, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11144[] = {SGI49201, SGI49202, SGI49203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49204, SGI0, SGI0, SGI0, SGI49205, SGI49205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11145[] = {SGI49206, SGI49207, SGI49208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49209, SGI0, SGI0, SGI0, SGI49211, SGI49211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11146[] = {SGI49210, SGI49212, SGI49213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49214, SGI0, SGI0, SGI0, SGI49215, SGI49215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11147[] = {SGI49216, SGI49217, SGI49218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49219, SGI49219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11148[] = {SGI49220, SGI49221, SGI49222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49223, SGI0, SGI0, SGI0, SGI49224, SGI49224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11149[] = {SGI0, SGI49225, SGI49226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49227, SGI0, SGI0, SGI0, SGI49228, SGI49228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11150[] = {SGI49229, SGI49230, SGI49231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49232, SGI0, SGI0, SGI0, SGI49233, SGI49233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11151[] = {SGI49234, SGI49235, SGI49236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49237, SGI49237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11152[] = {SGI49238, SGI0, SGI49239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11153[] = {SGI49240, SGI49241, SGI49242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49243, SGI0, SGI0, SGI0, SGI49244, SGI49244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11154[] = {SGI49245, SGI49246, SGI49247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49248, SGI49249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11155[] = {SGI49250, SGI0, SGI49251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11156[] = {SGI49252, SGI49253, SGI49254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49255, SGI0, SGI0, SGI0, SGI49256, SGI49256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11157[] = {SGI49257, SGI49258, SGI49259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49260, SGI0, SGI0, SGI0, SGI49261, SGI49261, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11158[] = {SGI49262, SGI49263, SGI49264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49265, SGI0, SGI0, SGI0, SGI49266, SGI49266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11159[] = {SGI49267, SGI49268, SGI49269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49270, SGI0, SGI0, SGI0, SGI49271, SGI49271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11160[] = {SGI49272, SGI49273, SGI49274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49275, SGI0, SGI0, SGI0, SGI49276, SGI49276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11161[] = {SGI49277, SGI0, SGI49278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49279, SGI0, SGI0, SGI0, SGI49280, SGI49280, SGI0, SGI0, SGI0, SGI49281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11162[] = {SGI0, SGI49282, SGI49283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49284, SGI0, SGI0, SGI0, SGI0, SGI49285, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11163[] = {SGI0, SGI49286, SGI49287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49288, SGI0, SGI0, SGI0, SGI0, SGI49289, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11164[] = {SGI0, SGI49290, SGI49291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49292, SGI0, SGI0, SGI0, SGI0, SGI49293, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11165[] = {SGI49294, SGI49295, SGI49296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49297, SGI49297, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11166[] = {SGI49298, SGI0, SGI49299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49300, SGI0, SGI0, SGI0, SGI49301, SGI49301, SGI0, SGI0, SGI49302, SGI0, NULL};
const Segment * const SG11167[] = {SGI49303, SGI49304, SGI49305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49306, SGI0, SGI0, SGI0, SGI49307, SGI49307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11168[] = {SGI49308, SGI49309, SGI49310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49311, SGI0, SGI0, SGI0, SGI49312, SGI49312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11169[] = {SGI0, SGI49313, SGI49314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11170[] = {SGI49315, SGI49316, SGI49317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49320, SGI0, SGI0, SGI0, SGI49321, SGI49321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11171[] = {SGI0, SGI49318, SGI49319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11172[] = {SGI49323, SGI49325, SGI49327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49328, SGI0, SGI0, SGI0, SGI49330, SGI49330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11173[] = {SGI49322, SGI49324, SGI49326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49329, SGI0, SGI0, SGI0, SGI49331, SGI49331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11174[] = {SGI49332, SGI49333, SGI49334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49335, SGI0, SGI0, SGI0, SGI49336, SGI49336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11175[] = {SGI49337, SGI49338, SGI49339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49340, SGI49340, SGI0, SGI0, SGI0, SGI49341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11176[] = {SGI49342, SGI49343, SGI49344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49345, SGI0, SGI0, SGI0, SGI49346, SGI49346, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11177[] = {SGI49347, SGI49348, SGI49349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49350, SGI0, SGI0, SGI0, SGI49351, SGI49351, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11178[] = {SGI49352, SGI49353, SGI49354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49355, SGI49356, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11179[] = {SGI49357, SGI49358, SGI49359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49360, SGI49360, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11180[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49361, SGI0, SGI0, SGI49362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11181[] = {SGI49363, SGI49364, SGI49365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49366, SGI0, SGI0, SGI0, SGI49367, SGI49367, SGI0, SGI0, NULL};
const Segment * const SG11182[] = {SGI49368, SGI49369, SGI49370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49371, SGI0, SGI0, SGI0, SGI49372, SGI49372, SGI0, SGI0, NULL};
const Segment * const SG11183[] = {SGI49373, SGI49374, SGI49375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49376, SGI0, SGI0, SGI0, SGI49377, SGI49377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11184[] = {SGI0, SGI0, SGI49378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49379, SGI0, SGI0, SGI0, SGI49380, SGI49380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11185[] = {SGI49381, SGI49382, SGI49383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49384, SGI0, SGI0, SGI0, SGI49385, SGI49385, SGI0, SGI0, NULL};
const Segment * const SG11186[] = {SGI49386, SGI49387, SGI49388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49389, SGI49389, SGI0, SGI49390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11187[] = {SGI49391, SGI49392, SGI49393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49394, SGI49394, SGI0, SGI49395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11188[] = {SGI0, SGI49396, SGI0, SGI49397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11189[] = {SGI0, SGI49398, SGI0, SGI49399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49400, SGI0, SGI0, SGI0, SGI49401, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11190[] = {SGI0, SGI49402, SGI0, SGI49403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11191[] = {SGI49404, SGI49405, SGI49406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49407, SGI0, SGI0, SGI0, SGI49408, SGI49408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11192[] = {SGI49409, SGI49410, SGI49411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49412, SGI0, SGI0, SGI0, SGI49413, SGI49413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11193[] = {SGI49414, SGI49415, SGI49416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49417, SGI49418, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11194[] = {SGI49419, SGI49420, SGI49421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49422, SGI49422, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11195[] = {SGI49423, SGI49424, SGI49425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49426, SGI0, SGI0, SGI0, SGI49427, SGI49427, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11196[] = {SGI0, SGI49428, SGI49429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11197[] = {SGI0, SGI49430, SGI49431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11198[] = {SGI49432, SGI49433, SGI49434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49435, SGI49436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11199[] = {SGI49437, SGI49438, SGI49439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49440, SGI0, SGI0, SGI0, SGI49441, SGI49441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11200[] = {SGI0, SGI49444, SGI49446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11201[] = {SGI49442, SGI49443, SGI49445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49447, SGI49447, SGI0, SGI49448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11202[] = {SGI49449, SGI49450, SGI49451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49452, SGI49452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11203[] = {SGI0, SGI49453, SGI49454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11204[] = {SGI0, SGI49455, SGI49456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11205[] = {SGI49457, SGI49458, SGI49459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49460, SGI0, SGI0, SGI0, SGI49461, SGI49461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11206[] = {SGI49462, SGI49463, SGI49464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49465, SGI0, SGI0, SGI0, SGI49466, SGI49466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11207[] = {SGI49467, SGI49468, SGI49469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49470, SGI49470, SGI0, SGI49471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11208[] = {SGI49472, SGI49473, SGI49474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49475, SGI0, SGI0, SGI0, SGI49476, SGI49476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11209[] = {SGI49477, SGI49478, SGI49479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49480, SGI0, SGI0, SGI0, SGI49481, SGI49481, SGI0, SGI0, NULL};
const Segment * const SG11210[] = {SGI49482, SGI49483, SGI49484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49485, SGI49486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11211[] = {SGI49487, SGI49488, SGI49489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49490, SGI49490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11212[] = {SGI49491, SGI49492, SGI49493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49494, SGI0, SGI0, SGI0, SGI49495, SGI49495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11213[] = {SGI49496, SGI49497, SGI49498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49499, SGI0, SGI0, SGI0, SGI49500, SGI49500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11214[] = {SGI49501, SGI0, SGI49502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11215[] = {SGI49503, SGI49504, SGI49505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49506, SGI0, SGI0, SGI0, SGI49507, SGI49507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11216[] = {SGI49508, SGI49509, SGI49510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49511, SGI0, SGI0, SGI0, SGI49512, SGI49512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11217[] = {SGI49513, SGI49514, SGI49515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49516, SGI0, SGI0, SGI0, SGI49517, SGI49517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11218[] = {SGI49518, SGI49519, SGI49520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49521, SGI0, SGI0, SGI0, SGI49522, SGI49522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11219[] = {SGI49523, SGI49524, SGI49525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49526, SGI0, SGI0, SGI0, SGI49527, SGI49527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11220[] = {SGI49528, SGI0, SGI49529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49530, SGI0, SGI0, SGI0, SGI49531, SGI49531, SGI0, SGI49532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11221[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49533, SGI0, SGI0, SGI49534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11222[] = {SGI49535, SGI49536, SGI49537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49538, SGI0, SGI0, SGI0, SGI49539, SGI49539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11223[] = {SGI49540, SGI49541, SGI49542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49543, SGI0, SGI0, SGI0, SGI49544, SGI49544, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11224[] = {SGI49545, SGI49546, SGI49547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49548, SGI0, SGI0, SGI0, SGI49549, SGI49549, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11225[] = {SGI49550, SGI49551, SGI49552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49553, SGI0, SGI0, SGI0, SGI49554, SGI49554, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11226[] = {SGI49555, SGI49556, SGI49557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49558, SGI49558, SGI0, SGI49561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11227[] = {SGI49559, SGI49560, SGI49562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49563, SGI0, SGI0, SGI0, SGI49564, SGI49564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11228[] = {SGI49565, SGI49566, SGI49567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49568, SGI0, SGI0, SGI0, SGI49569, SGI49569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11229[] = {SGI0, SGI0, SGI49571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49575, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11230[] = {SGI49570, SGI49572, SGI49573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49574, SGI0, SGI0, SGI0, SGI49576, SGI49576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11231[] = {SGI49577, SGI0, SGI49578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11232[] = {SGI49579, SGI49580, SGI49581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49582, SGI49582, SGI0, SGI0, SGI49583, SGI0, NULL};
const Segment * const SG11233[] = {SGI49584, SGI0, SGI49585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49586, SGI49587, SGI49587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49588, NULL};
const Segment * const SG11234[] = {SGI49589, SGI49590, SGI49591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49592, SGI0, SGI0, SGI0, SGI49593, SGI49593, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11235[] = {SGI49594, SGI49595, SGI49596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49597, SGI49598, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11236[] = {SGI49599, SGI49600, SGI49601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49602, SGI0, SGI0, SGI0, SGI49603, SGI49603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11237[] = {SGI0, SGI49604, SGI0, SGI49605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11238[] = {SGI49606, SGI49607, SGI49608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49609, SGI49610, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11239[] = {SGI49611, SGI49612, SGI49613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49614, SGI0, SGI0, SGI0, SGI49615, SGI49615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11240[] = {SGI49616, SGI49617, SGI49618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49619, SGI0, SGI0, SGI0, SGI49620, SGI49620, SGI0, SGI0, NULL};
const Segment * const SG11241[] = {SGI49621, SGI49622, SGI49623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49624, SGI49624, SGI0, SGI0, SGI0, SGI49625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11242[] = {SGI49626, SGI0, SGI49627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49628, SGI0, SGI0, SGI0, SGI49629, SGI49629, SGI0, SGI0, SGI0, SGI49630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11243[] = {SGI49631, SGI49632, SGI49633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49634, SGI0, SGI0, SGI0, SGI49635, SGI49635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11244[] = {SGI49636, SGI0, SGI49637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11245[] = {SGI49638, SGI49639, SGI49640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49641, SGI0, SGI0, SGI0, SGI49642, SGI49642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11246[] = {SGI49643, SGI0, SGI49644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11247[] = {SGI49645, SGI0, SGI49646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49647, SGI0, SGI0, SGI0, SGI49648, SGI49648, SGI0, SGI49649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11248[] = {SGI49650, SGI49651, SGI49652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49653, SGI0, SGI0, SGI0, SGI49654, SGI49654, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11249[] = {SGI49655, SGI49656, SGI49657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49658, SGI0, SGI0, SGI0, SGI49659, SGI49659, SGI0, SGI0, NULL};
const Segment * const SG11250[] = {SGI49660, SGI49661, SGI49662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49663, SGI0, SGI0, SGI0, SGI49664, SGI49664, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11251[] = {SGI49665, SGI49666, SGI49667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49668, SGI0, SGI0, SGI0, SGI49669, SGI49669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11252[] = {SGI49671, SGI49673, SGI49674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49678, SGI49678, SGI0, SGI0, SGI0, SGI49680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11253[] = {SGI49670, SGI49672, SGI49675, SGI0, SGI0, SGI0, SGI49676, SGI49677, SGI0, SGI49679, SGI0, SGI0, SGI0, SGI49681, SGI49681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11254[] = {SGI49682, SGI49683, SGI49684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49685, SGI0, SGI0, SGI0, SGI49686, SGI49686, SGI0, SGI0, NULL};
const Segment * const SG11255[] = {SGI49687, SGI49688, SGI49689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49690, SGI0, SGI0, SGI0, SGI49691, SGI49691, SGI0, SGI0, NULL};
const Segment * const SG11256[] = {SGI0, SGI49692, SGI0, SGI49693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11257[] = {SGI49694, SGI0, SGI49695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49696, SGI0, SGI0, SGI0, SGI49697, SGI49697, SGI0, SGI0, SGI49698, SGI0, NULL};
const Segment * const SG11258[] = {SGI49699, SGI49700, SGI49701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49702, SGI0, SGI0, SGI0, SGI49703, SGI49704, SGI0, SGI0, SGI49705, SGI0, NULL};
const Segment * const SG11259[] = {SGI49706, SGI49707, SGI49708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49709, SGI0, SGI0, SGI0, SGI49710, SGI49710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11260[] = {SGI49711, SGI49712, SGI49713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49714, SGI0, SGI0, SGI0, SGI49715, SGI49715, SGI0, SGI0, NULL};
const Segment * const SG11261[] = {SGI49716, SGI0, SGI49717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49718, SGI49719, SGI49719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49720, NULL};
const Segment * const SG11262[] = {SGI49721, SGI49722, SGI49723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49724, SGI0, SGI0, SGI0, SGI49725, SGI49725, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11263[] = {SGI49726, SGI49727, SGI49728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49729, SGI0, SGI0, SGI0, SGI49730, SGI49730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11264[] = {SGI49731, SGI0, SGI49732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49733, SGI0, SGI0, SGI0, SGI49734, SGI49734, SGI0, SGI0, SGI49735, SGI0, NULL};
const Segment * const SG11265[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49736, SGI0, SGI0, SGI49737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11266[] = {SGI49738, SGI49739, SGI49740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49741, SGI0, SGI0, SGI0, SGI49742, SGI49742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11267[] = {SGI49743, SGI0, SGI49744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49745, SGI49746, SGI49746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49747, NULL};
const Segment * const SG11268[] = {SGI49748, SGI49749, SGI49750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49751, SGI0, SGI0, SGI0, SGI49752, SGI49752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11269[] = {SGI49753, SGI49754, SGI49755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49756, SGI0, SGI0, SGI0, SGI49757, SGI49757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11270[] = {SGI49758, SGI49759, SGI49760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49761, SGI0, SGI0, SGI0, SGI49764, SGI49764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11271[] = {SGI49762, SGI49763, SGI49765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49766, SGI49767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11272[] = {SGI49768, SGI49769, SGI49770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49773, SGI49775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11273[] = {SGI49771, SGI49772, SGI49774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49776, SGI0, SGI0, SGI0, SGI49777, SGI49777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11274[] = {SGI49778, SGI49779, SGI49780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49781, SGI0, SGI0, SGI0, SGI49785, SGI49785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11275[] = {SGI49782, SGI49783, SGI49784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49786, SGI49786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11276[] = {SGI49787, SGI49788, SGI49789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49790, SGI0, SGI0, SGI0, SGI49791, SGI49791, SGI0, SGI49792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11277[] = {SGI49793, SGI49794, SGI49795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49796, SGI0, SGI0, SGI0, SGI49797, SGI49797, SGI0, SGI49798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11278[] = {SGI49799, SGI49800, SGI49801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49802, SGI0, SGI0, SGI0, SGI49803, SGI49803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11279[] = {SGI0, SGI49804, SGI0, SGI49805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11280[] = {SGI49806, SGI49807, SGI49808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49809, SGI49809, SGI0, SGI49810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11281[] = {SGI49811, SGI49812, SGI49813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49814, SGI0, SGI0, SGI0, SGI49815, SGI49815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11282[] = {SGI49816, SGI49817, SGI49818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49819, SGI0, SGI0, SGI0, SGI49820, SGI49820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11283[] = {SGI49821, SGI0, SGI49822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11284[] = {SGI49823, SGI49824, SGI49825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49826, SGI0, SGI0, SGI0, SGI49827, SGI49827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11285[] = {SGI49828, SGI49829, SGI49830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49831, SGI0, SGI0, SGI0, SGI49832, SGI49832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11286[] = {SGI49834, SGI0, SGI49835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49836, SGI49837, SGI49837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49838, NULL};
const Segment * const SG11287[] = {SGI49839, SGI49840, SGI49841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49842, SGI0, SGI0, SGI0, SGI49843, SGI49843, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11288[] = {SGI49844, SGI49845, SGI49846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49847, SGI0, SGI0, SGI0, SGI49849, SGI49849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11289[] = {SGI49848, SGI49850, SGI49851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49852, SGI0, SGI0, SGI0, SGI49853, SGI49853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11290[] = {SGI49854, SGI0, SGI49855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11291[] = {SGI49856, SGI49857, SGI49858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49859, SGI0, SGI0, SGI0, SGI49860, SGI49860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11292[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49861, SGI0, SGI0, SGI49862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11293[] = {SGI49863, SGI49864, SGI49865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49866, SGI0, SGI0, SGI0, SGI49867, SGI49867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11294[] = {SGI49868, SGI49869, SGI49870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49871, SGI0, SGI0, SGI0, SGI49872, SGI49872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11295[] = {SGI49873, SGI0, SGI49874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49875, SGI0, SGI0, SGI0, SGI49876, SGI49876, SGI0, SGI0, SGI0, SGI49877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11296[] = {SGI49878, SGI49879, SGI49880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49881, SGI0, SGI0, SGI0, SGI49882, SGI49883, SGI0, SGI0, SGI0, SGI49884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11297[] = {SGI49885, SGI49886, SGI49887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49888, SGI0, SGI0, SGI0, SGI49889, SGI49889, SGI49890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11298[] = {SGI49891, SGI49892, SGI49893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49894, SGI0, SGI0, SGI0, SGI49895, SGI49895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11299[] = {SGI49896, SGI49897, SGI49898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49899, SGI0, SGI0, SGI0, SGI49900, SGI49900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11300[] = {SGI49901, SGI49902, SGI49903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49904, SGI0, SGI0, SGI0, SGI49905, SGI49905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11301[] = {SGI49906, SGI49907, SGI49908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49909, SGI0, SGI0, SGI0, SGI49910, SGI49910, SGI0, SGI0, SGI0, SGI0, SGI49911, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11302[] = {SGI49912, SGI49913, SGI49914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49915, SGI0, SGI0, SGI0, SGI49916, SGI49916, SGI0, SGI0, SGI0, SGI0, SGI49917, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11303[] = {SGI49918, SGI49919, SGI49920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49921, SGI0, SGI0, SGI0, SGI49922, SGI49922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11304[] = {SGI49923, SGI49924, SGI49925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49926, SGI0, SGI0, SGI0, SGI49927, SGI49927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11305[] = {SGI49928, SGI49929, SGI49930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49931, SGI0, SGI0, SGI0, SGI49932, SGI49932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11306[] = {SGI49933, SGI49934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49935, SGI0, SGI0, SGI0, SGI49936, SGI49936, SGI0, SGI0, NULL};
const Segment * const SG11307[] = {SGI49937, SGI49938, SGI49939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49940, SGI49940, SGI0, SGI0, SGI0, SGI49941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11308[] = {SGI49942, SGI49943, SGI49944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49945, SGI0, SGI0, SGI0, SGI49946, SGI49946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11309[] = {SGI49947, SGI49948, SGI49949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49950, SGI0, SGI0, SGI0, SGI49951, SGI49951, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11310[] = {SGI49952, SGI49953, SGI49954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49955, SGI0, SGI0, SGI0, SGI49956, SGI49956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11311[] = {SGI49957, SGI49958, SGI49959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49960, SGI0, SGI0, SGI0, SGI49961, SGI49961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11312[] = {SGI49962, SGI0, SGI49963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11313[] = {SGI49964, SGI49965, SGI49966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49967, SGI0, SGI0, SGI0, SGI49968, SGI49968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11314[] = {SGI49969, SGI49970, SGI49971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49972, SGI0, SGI0, SGI0, SGI49973, SGI49973, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11315[] = {SGI49974, SGI49975, SGI49976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49977, SGI49978, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11316[] = {SGI49979, SGI49980, SGI49981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49982, SGI49982, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11317[] = {SGI49983, SGI0, SGI49984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49985, SGI0, SGI0, SGI0, SGI49986, SGI49986, SGI0, SGI0, SGI0, SGI49987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11318[] = {SGI49988, SGI0, SGI49989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49990, SGI0, SGI0, SGI0, SGI49991, SGI49991, SGI0, SGI0, SGI49992, SGI0, NULL};
const Segment * const SG11319[] = {SGI49993, SGI49994, SGI49995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI49996, SGI0, SGI0, SGI0, SGI49997, SGI49997, SGI0, SGI0, NULL};
const Segment * const SG11320[] = {SGI49998, SGI49999, SGI50000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50001, SGI0, SGI0, SGI0, SGI50002, SGI50002, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11321[] = {SGI50003, SGI50004, SGI50005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50006, SGI0, SGI0, SGI0, SGI50007, SGI50007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11322[] = {SGI50008, SGI0, SGI50009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11323[] = {SGI50011, SGI50012, SGI50013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50014, SGI0, SGI0, SGI0, SGI50015, SGI50015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11324[] = {SGI50016, SGI0, SGI50017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11325[] = {SGI50018, SGI50019, SGI50020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50021, SGI50022, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11326[] = {SGI50023, SGI50024, SGI50025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50026, SGI0, SGI0, SGI0, SGI50027, SGI50027, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11327[] = {SGI50028, SGI50029, SGI50030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50031, SGI50031, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11328[] = {SGI50010, SGI50032, SGI50033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50034, SGI0, SGI0, SGI0, SGI50035, SGI50035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11329[] = {SGI50036, SGI50037, SGI50038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50039, SGI0, SGI0, SGI0, SGI50040, SGI50040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11330[] = {SGI50041, SGI50042, SGI50043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50044, SGI0, SGI0, SGI0, SGI50045, SGI50045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11331[] = {SGI50046, SGI50047, SGI50048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50049, SGI0, SGI0, SGI0, SGI50050, SGI50050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11332[] = {SGI50051, SGI50052, SGI50053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50054, SGI0, SGI0, SGI0, SGI50055, SGI50055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11333[] = {SGI50056, SGI50057, SGI50058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50059, SGI50060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11334[] = {SGI50061, SGI50062, SGI50063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50064, SGI50064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11335[] = {SGI0, SGI50065, SGI0, SGI50066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11336[] = {SGI0, SGI50067, SGI0, SGI50068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50069, SGI0, SGI0, SGI0, SGI50070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11337[] = {SGI50071, SGI50072, SGI50073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50074, SGI0, SGI0, SGI0, SGI50075, SGI50075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11338[] = {SGI50076, SGI50077, SGI50078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50079, SGI0, SGI0, SGI0, SGI50080, SGI50080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11339[] = {SGI50081, SGI50082, SGI50083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50084, SGI0, SGI0, SGI0, SGI50085, SGI50085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11340[] = {SGI50086, SGI50087, SGI50088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50089, SGI0, SGI0, SGI0, SGI50090, SGI50090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11341[] = {SGI50091, SGI0, SGI50092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11342[] = {SGI50093, SGI50094, SGI50095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50096, SGI0, SGI0, SGI0, SGI50097, SGI50097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11343[] = {SGI50098, SGI50099, SGI50100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50101, SGI0, SGI0, SGI0, SGI50102, SGI50102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11344[] = {SGI50103, SGI50104, SGI50105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50106, SGI0, SGI0, SGI0, SGI50107, SGI50107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11345[] = {SGI50108, SGI0, SGI50109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50110, SGI0, SGI0, SGI0, SGI50111, SGI50111, SGI0, SGI0, SGI0, SGI50112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11346[] = {SGI50113, SGI50114, SGI50115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50116, SGI0, SGI0, SGI0, SGI50117, SGI50117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11347[] = {SGI50118, SGI0, SGI50119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50120, SGI50121, SGI50121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50122, NULL};
const Segment * const SG11348[] = {SGI50123, SGI0, SGI50124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50125, SGI50126, SGI50126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50127, NULL};
const Segment * const SG11349[] = {SGI50128, SGI50129, SGI50130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50131, SGI0, SGI0, SGI0, SGI50132, SGI50132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11350[] = {SGI0, SGI50133, SGI50134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50135, SGI0, SGI0, SGI0, SGI0, SGI50136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11351[] = {SGI0, SGI50137, SGI50138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50139, SGI0, SGI0, SGI0, SGI0, SGI50140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11352[] = {SGI0, SGI50141, SGI50142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50143, SGI0, SGI0, SGI0, SGI0, SGI50144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11353[] = {SGI50145, SGI50146, SGI50147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50148, SGI50149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11354[] = {SGI0, SGI50150, SGI50151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50152, SGI0, SGI0, SGI0, SGI0, SGI50153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11355[] = {SGI0, SGI50154, SGI50155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50156, SGI0, SGI0, SGI0, SGI0, SGI50158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11356[] = {SGI50157, SGI50159, SGI50160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50161, SGI0, SGI0, SGI0, SGI50162, SGI50162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11357[] = {SGI50163, SGI50164, SGI50165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50166, SGI50166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11358[] = {SGI50167, SGI50168, SGI50169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50170, SGI0, SGI0, SGI0, SGI50171, SGI50171, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11359[] = {SGI50172, SGI50173, SGI50174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50175, SGI0, SGI0, SGI0, SGI50176, SGI50176, SGI0, SGI50177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11360[] = {SGI50178, SGI50179, SGI50180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50181, SGI0, SGI0, SGI0, SGI50182, SGI50182, SGI0, SGI50183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11361[] = {SGI50184, SGI50185, SGI50186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50187, SGI0, SGI0, SGI0, SGI50188, SGI50188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11362[] = {SGI50189, SGI50190, SGI50191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50192, SGI0, SGI0, SGI0, SGI50193, SGI50193, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11363[] = {SGI50194, SGI50195, SGI50196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50197, SGI0, SGI0, SGI0, SGI50198, SGI50198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11364[] = {SGI50199, SGI50200, SGI50201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50202, SGI0, SGI0, SGI0, SGI50203, SGI50203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11365[] = {SGI50204, SGI50205, SGI50206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50207, SGI0, SGI0, SGI0, SGI50208, SGI50208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11366[] = {SGI50209, SGI50210, SGI50211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50212, SGI0, SGI0, SGI0, SGI50213, SGI50213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11367[] = {SGI50214, SGI50215, SGI50216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50217, SGI0, SGI0, SGI0, SGI50218, SGI50218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11368[] = {SGI50219, SGI50220, SGI50221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50222, SGI0, SGI0, SGI0, SGI50223, SGI50223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11369[] = {SGI50224, SGI50225, SGI50226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50227, SGI0, SGI0, SGI0, SGI50228, SGI50228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11370[] = {SGI50229, SGI50230, SGI50231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50232, SGI50232, SGI0, SGI50233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11371[] = {SGI50234, SGI50235, SGI50236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50237, SGI0, SGI0, SGI0, SGI50238, SGI50238, SGI0, SGI0, NULL};
const Segment * const SG11372[] = {SGI50239, SGI50240, SGI50241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50242, SGI0, SGI0, SGI0, SGI50243, SGI50243, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11373[] = {SGI50244, SGI0, SGI50245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50246, SGI0, SGI0, SGI0, SGI50247, SGI50247, SGI0, SGI50248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11374[] = {SGI50249, SGI50250, SGI50251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50252, SGI0, SGI0, SGI0, SGI50253, SGI50253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11375[] = {SGI50254, SGI50255, SGI50256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50257, SGI0, SGI0, SGI0, SGI50258, SGI50258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11376[] = {SGI0, SGI50259, SGI0, SGI50260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11377[] = {SGI50261, SGI50262, SGI50263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50264, SGI0, SGI0, SGI0, SGI50265, SGI50265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11378[] = {SGI50266, SGI0, SGI50267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50268, SGI0, SGI0, SGI0, SGI50269, SGI50269, SGI0, SGI50270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11379[] = {SGI0, SGI50271, SGI50272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11380[] = {SGI50273, SGI50274, SGI50275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50276, SGI0, SGI0, SGI0, SGI50277, SGI50277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11381[] = {SGI50278, SGI50279, SGI50280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50281, SGI0, SGI0, SGI0, SGI50282, SGI50282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11382[] = {SGI50283, SGI50284, SGI50285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50286, SGI0, SGI0, SGI0, SGI50287, SGI50287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11383[] = {SGI50288, SGI50289, SGI50290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50291, SGI0, SGI0, SGI0, SGI50292, SGI50292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11384[] = {SGI50293, SGI50294, SGI50295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50296, SGI0, SGI0, SGI0, SGI50297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11385[] = {SGI50298, SGI50299, SGI50300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50301, SGI0, SGI0, SGI0, SGI50302, SGI50302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11386[] = {SGI50303, SGI50304, SGI50305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50306, SGI0, SGI0, SGI0, SGI50307, SGI50307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11387[] = {SGI50308, SGI50309, SGI50310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50311, SGI0, SGI0, SGI0, SGI50312, SGI50312, SGI0, SGI0, SGI0, SGI0, SGI50313, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11388[] = {SGI50314, SGI50315, SGI50316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50317, SGI0, SGI0, SGI0, SGI50318, SGI50318, SGI0, SGI0, SGI0, SGI0, SGI50319, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11389[] = {SGI50320, SGI50321, SGI50322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50323, SGI0, SGI0, SGI0, SGI50324, SGI50324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11390[] = {SGI0, SGI50325, SGI0, SGI50326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11391[] = {SGI50327, SGI50328, SGI50329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50330, SGI0, SGI0, SGI0, SGI50331, SGI50331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11392[] = {SGI50332, SGI50333, SGI50334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50335, SGI50335, SGI0, SGI50336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11393[] = {SGI50337, SGI0, SGI50338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50339, SGI0, SGI0, SGI0, SGI50340, SGI50340, SGI0, SGI50341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11394[] = {SGI50342, SGI50343, SGI50344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50345, SGI0, SGI0, SGI0, SGI50346, SGI50346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11395[] = {SGI50347, SGI50348, SGI50349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50350, SGI0, SGI0, SGI0, SGI50351, SGI50351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11396[] = {SGI50352, SGI50353, SGI50354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50355, SGI50355, SGI0, SGI50356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11397[] = {SGI50357, SGI0, SGI50358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50359, SGI50360, SGI50360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50361, NULL};
const Segment * const SG11398[] = {SGI50362, SGI50363, SGI50364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50365, SGI50366, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11399[] = {SGI0, SGI50367, SGI50368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50369, SGI0, SGI0, SGI0, SGI0, SGI50370, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11400[] = {SGI0, SGI50371, SGI50372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50373, SGI0, SGI0, SGI0, SGI0, SGI50374, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11401[] = {SGI50375, SGI0, SGI50376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50377, SGI0, SGI0, SGI0, SGI50378, SGI50378, SGI0, SGI0, SGI0, SGI50379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11402[] = {SGI50380, SGI50381, SGI50382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50383, SGI0, SGI0, SGI0, SGI50384, SGI50385, SGI0, SGI0, SGI0, SGI50386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11403[] = {SGI0, SGI50387, SGI0, SGI50388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11404[] = {SGI0, SGI50389, SGI0, SGI50390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50391, SGI0, SGI0, SGI0, SGI50392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11405[] = {SGI50393, SGI0, SGI50394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50395, SGI0, SGI0, SGI0, SGI50396, SGI50396, SGI0, SGI50397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11406[] = {SGI50398, SGI50399, SGI50400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50401, SGI0, SGI0, SGI0, SGI50402, SGI50402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11407[] = {SGI50403, SGI50404, SGI50405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50406, SGI50406, SGI0, SGI0, SGI0, SGI50407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11408[] = {SGI50408, SGI50409, SGI50410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50411, SGI50411, SGI0, SGI0, SGI0, SGI50412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11409[] = {SGI50413, SGI50414, SGI50415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50416, SGI0, SGI0, SGI0, SGI50417, SGI50417, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11410[] = {SGI50418, SGI50419, SGI50420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50421, SGI0, SGI0, SGI0, SGI50422, SGI50422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11411[] = {SGI50423, SGI50424, SGI50425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50426, SGI50427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11412[] = {SGI50428, SGI50429, SGI50430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50431, SGI50431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11413[] = {SGI50432, SGI50433, SGI50434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50435, SGI0, SGI0, SGI0, SGI50436, SGI50436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11414[] = {SGI50437, SGI50438, SGI50439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50440, SGI0, SGI0, SGI0, SGI50441, SGI50441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11415[] = {SGI50442, SGI50443, SGI50444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50445, SGI0, SGI0, SGI0, SGI50446, SGI50446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11416[] = {SGI50447, SGI50448, SGI50449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50450, SGI0, SGI0, SGI0, SGI50451, SGI50451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11417[] = {SGI50452, SGI50453, SGI50454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50455, SGI0, SGI0, SGI0, SGI50456, SGI50456, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11418[] = {SGI50457, SGI50458, SGI50459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50460, SGI0, SGI0, SGI0, SGI50461, SGI50461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11419[] = {SGI50462, SGI50463, SGI50464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50465, SGI0, SGI0, SGI0, SGI50466, SGI50466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11420[] = {SGI50467, SGI50468, SGI50469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50470, SGI0, SGI0, SGI0, SGI50471, SGI50471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11421[] = {SGI50472, SGI0, SGI50473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11422[] = {SGI50474, SGI50475, SGI50476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50477, SGI0, SGI0, SGI0, SGI50478, SGI50478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11423[] = {SGI50479, SGI50480, SGI50481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50482, SGI50483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11424[] = {SGI50484, SGI50485, SGI50486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50487, SGI0, SGI0, SGI0, SGI50488, SGI50488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11425[] = {SGI50489, SGI0, SGI50490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50491, SGI0, SGI0, SGI0, SGI50492, SGI50492, SGI0, SGI0, SGI0, SGI50493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11426[] = {SGI50494, SGI50495, SGI50496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50497, SGI50497, SGI0, SGI0, SGI50498, SGI0, NULL};
const Segment * const SG11427[] = {SGI50499, SGI50500, SGI50501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50502, SGI50502, SGI0, SGI0, SGI0, SGI50503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11428[] = {SGI50504, SGI50505, SGI50506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50507, SGI0, SGI0, SGI0, SGI50508, SGI50508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11429[] = {SGI50509, SGI0, SGI50510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50511, SGI50512, SGI50512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50513, NULL};
const Segment * const SG11430[] = {SGI50514, SGI50515, SGI50516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50517, SGI0, SGI0, SGI0, SGI50518, SGI50518, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11431[] = {SGI50519, SGI50520, SGI50521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50522, SGI0, SGI0, SGI0, SGI50523, SGI50523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11432[] = {SGI50524, SGI50525, SGI50526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50527, SGI50527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11433[] = {SGI50528, SGI50529, SGI50530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50531, SGI0, SGI0, SGI0, SGI50532, SGI50532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11434[] = {SGI50533, SGI50534, SGI50535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50536, SGI50536, SGI0, SGI0, SGI0, SGI50537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11435[] = {SGI50538, SGI50539, SGI50540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50541, SGI0, SGI0, SGI0, SGI50542, SGI50542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11436[] = {SGI50543, SGI50544, SGI50545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50546, SGI0, SGI0, SGI0, SGI50547, SGI50547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11437[] = {SGI50548, SGI50549, SGI50550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50551, SGI0, SGI0, SGI0, SGI50552, SGI50552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11438[] = {SGI50553, SGI50554, SGI50555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50556, SGI0, SGI0, SGI0, SGI50557, SGI50557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11439[] = {SGI50558, SGI50559, SGI50560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50561, SGI0, SGI0, SGI0, SGI50562, SGI50562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11440[] = {SGI50563, SGI50564, SGI50565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50566, SGI0, SGI0, SGI0, SGI50567, SGI50567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50568, SGI0, SGI0, NULL};
const Segment * const SG11441[] = {SGI50569, SGI50570, SGI50571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50572, SGI50573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11442[] = {SGI50574, SGI50575, SGI50576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50577, SGI50577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11443[] = {SGI50578, SGI50579, SGI50580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50581, SGI0, SGI0, SGI0, SGI50582, SGI50582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11444[] = {SGI50583, SGI50584, SGI50585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50586, SGI50586, SGI0, SGI0, SGI0, SGI50587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11445[] = {SGI50588, SGI50589, SGI50590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50591, SGI50591, SGI0, SGI0, SGI0, SGI50592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11446[] = {SGI50593, SGI50594, SGI50595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50596, SGI0, SGI0, SGI0, SGI50597, SGI50597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11447[] = {SGI50598, SGI50599, SGI50600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50601, SGI0, SGI0, SGI0, SGI50602, SGI50602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11448[] = {SGI50603, SGI50604, SGI50605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50606, SGI0, SGI0, SGI0, SGI50607, SGI50607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11449[] = {SGI50608, SGI0, SGI50609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50610, SGI0, SGI0, SGI0, SGI50611, SGI50611, SGI0, SGI0, SGI0, SGI50612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11450[] = {SGI50613, SGI50614, SGI50615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50616, SGI0, SGI0, SGI0, SGI50617, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11451[] = {SGI50618, SGI50619, SGI50620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50621, SGI0, SGI0, SGI0, SGI50622, SGI50622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11452[] = {SGI50623, SGI50624, SGI50625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50626, SGI0, SGI0, SGI0, SGI50627, SGI50627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11453[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50628, SGI0, SGI0, SGI50629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11454[] = {SGI50630, SGI50631, SGI50632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50633, SGI0, SGI0, SGI0, SGI50634, SGI50634, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11455[] = {SGI50635, SGI50636, SGI50637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50638, SGI0, SGI0, SGI0, SGI50639, SGI50639, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11456[] = {SGI50640, SGI50641, SGI50642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50643, SGI50643, SGI0, SGI0, SGI0, SGI50644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11457[] = {SGI50645, SGI50646, SGI50647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50648, SGI0, SGI0, SGI0, SGI50649, SGI50649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50650, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11458[] = {SGI50651, SGI50652, SGI50653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50654, SGI0, SGI0, SGI0, SGI50655, SGI50655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50656, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11459[] = {SGI50657, SGI50658, SGI50659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50660, SGI0, SGI0, SGI0, SGI50661, SGI50661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11460[] = {SGI50662, SGI50663, SGI50664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50665, SGI50665, SGI0, SGI0, SGI0, SGI50666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11461[] = {SGI50667, SGI50668, SGI50669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50670, SGI50670, SGI0, SGI0, SGI0, SGI50671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11462[] = {SGI50672, SGI50673, SGI50674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50675, SGI0, SGI0, SGI0, SGI50676, SGI50676, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11463[] = {SGI50677, SGI50678, SGI50679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50680, SGI0, SGI0, SGI0, SGI50681, SGI50681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11464[] = {SGI50682, SGI50683, SGI50684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50685, SGI50686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11465[] = {SGI0, SGI50687, SGI50688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50689, SGI50689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11466[] = {SGI50690, SGI50691, SGI50692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50693, SGI50693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11467[] = {SGI50694, SGI50695, SGI50696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50697, SGI0, SGI0, SGI0, SGI50698, SGI50698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11468[] = {SGI50699, SGI50700, SGI50701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50702, SGI0, SGI0, SGI0, SGI50703, SGI50703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11469[] = {SGI50704, SGI0, SGI50705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11470[] = {SGI50706, SGI50707, SGI50708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50709, SGI50709, SGI0, SGI0, SGI0, SGI50710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11471[] = {SGI50711, SGI0, SGI50712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50713, SGI0, SGI0, SGI0, SGI50714, SGI50714, SGI0, SGI0, SGI0, SGI50715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11472[] = {SGI50716, SGI0, SGI50717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50718, SGI0, SGI0, SGI0, SGI50719, SGI50719, SGI0, SGI0, SGI50720, SGI0, NULL};
const Segment * const SG11473[] = {SGI50721, SGI50722, SGI50723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50724, SGI0, SGI0, SGI0, SGI50725, SGI50725, SGI0, SGI50726, SGI0, SGI0, NULL};
const Segment * const SG11474[] = {SGI50727, SGI50728, SGI50729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50730, SGI0, SGI0, SGI0, SGI50731, SGI50731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11475[] = {SGI50733, SGI50734, SGI50735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50736, SGI0, SGI0, SGI0, SGI50737, SGI50737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11476[] = {SGI50738, SGI50739, SGI50740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50741, SGI0, SGI0, SGI0, SGI50742, SGI50742, SGI0, SGI0, NULL};
const Segment * const SG11477[] = {SGI50743, SGI0, SGI50744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50745, SGI0, SGI0, SGI0, SGI50746, SGI50746, SGI0, SGI50747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11478[] = {SGI50748, SGI50749, SGI50750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50751, SGI50751, SGI0, SGI0, SGI0, SGI50752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11479[] = {SGI50753, SGI50754, SGI50755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50756, SGI0, SGI0, SGI0, SGI50757, SGI50757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11480[] = {SGI50758, SGI0, SGI50759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50760, SGI0, SGI0, SGI0, SGI50761, SGI50761, SGI0, SGI0, SGI0, SGI50762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11481[] = {SGI50763, SGI0, SGI50764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50765, SGI0, SGI0, SGI0, SGI50766, SGI50766, SGI0, SGI0, SGI0, SGI50767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11482[] = {SGI50768, SGI50769, SGI50770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50771, SGI0, SGI0, SGI0, SGI50772, SGI50772, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11483[] = {SGI50773, SGI50774, SGI50775, SGI0, SGI0, SGI0, SGI50776, SGI50777, SGI0, SGI50778, SGI0, SGI0, SGI0, SGI50779, SGI50779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11484[] = {SGI50780, SGI50781, SGI50782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50783, SGI0, SGI0, SGI0, SGI50784, SGI50784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50785, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11485[] = {SGI50786, SGI50787, SGI50788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50789, SGI0, SGI0, SGI0, SGI50790, SGI50790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50791, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11486[] = {SGI50792, SGI50793, SGI50794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50795, SGI0, SGI0, SGI0, SGI50796, SGI50796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50797, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11487[] = {SGI50798, SGI50799, SGI50800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50802, SGI0, SGI0, SGI0, SGI50805, SGI50805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50807, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11488[] = {SGI50801, SGI50803, SGI50804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50806, SGI0, SGI0, SGI0, SGI50808, SGI50808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11489[] = {SGI50809, SGI50810, SGI50811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50812, SGI0, SGI0, SGI0, SGI50813, SGI50813, SGI0, SGI0, SGI0, SGI0, SGI50814, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11490[] = {SGI50815, SGI50816, SGI50817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50818, SGI0, SGI0, SGI0, SGI50819, SGI50819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11491[] = {SGI50820, SGI50821, SGI50822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50823, SGI0, SGI0, SGI0, SGI50824, SGI50824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11492[] = {SGI50825, SGI50826, SGI50827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50828, SGI0, SGI0, SGI0, SGI50829, SGI50829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11493[] = {SGI50830, SGI50831, SGI50832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50833, SGI0, SGI0, SGI0, SGI50834, SGI50834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11494[] = {SGI50835, SGI50836, SGI50837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50838, SGI0, SGI0, SGI0, SGI50839, SGI50839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11495[] = {SGI50840, SGI50841, SGI50842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50843, SGI0, SGI0, SGI0, SGI50844, SGI50844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11496[] = {SGI50845, SGI0, SGI50846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11497[] = {SGI50847, SGI50848, SGI50849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50850, SGI50851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11498[] = {SGI50852, SGI50853, SGI50854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50855, SGI0, SGI0, SGI0, SGI50856, SGI50856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11499[] = {SGI50857, SGI50858, SGI50859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50860, SGI50860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11500[] = {SGI50861, SGI0, SGI50862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11501[] = {SGI50863, SGI0, SGI50864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50865, SGI0, SGI0, SGI0, SGI50866, SGI50866, SGI0, SGI0, SGI0, SGI50867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11502[] = {SGI50868, SGI0, SGI50869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50870, SGI0, SGI0, SGI0, SGI50871, SGI50871, SGI0, SGI0, SGI50872, SGI0, NULL};
const Segment * const SG11503[] = {SGI50873, SGI50874, SGI50875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50876, SGI0, SGI0, SGI0, SGI50877, SGI50877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11504[] = {SGI50878, SGI50879, SGI50880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50881, SGI0, SGI0, SGI0, SGI50882, SGI50882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11505[] = {SGI50883, SGI50884, SGI50885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50886, SGI0, SGI0, SGI0, SGI50887, SGI50887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11506[] = {SGI50888, SGI50889, SGI50890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50891, SGI0, SGI0, SGI0, SGI50892, SGI50892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11507[] = {SGI50893, SGI50894, SGI50895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50896, SGI0, SGI0, SGI0, SGI50897, SGI50897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11508[] = {SGI50898, SGI50899, SGI50900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50901, SGI0, SGI0, SGI0, SGI50902, SGI50902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11509[] = {SGI50903, SGI50904, SGI50905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50906, SGI0, SGI0, SGI0, SGI50907, SGI50907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11510[] = {SGI50908, SGI50909, SGI50910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50911, SGI0, SGI0, SGI0, SGI50912, SGI50912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11511[] = {SGI50913, SGI50914, SGI50915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50916, SGI0, SGI0, SGI0, SGI50917, SGI50917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11512[] = {SGI50918, SGI50919, SGI50920, SGI0, SGI0, SGI0, SGI50921, SGI50922, SGI0, SGI50923, SGI0, SGI0, SGI0, SGI50924, SGI50924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11513[] = {SGI50925, SGI0, SGI50926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50927, SGI0, SGI0, SGI0, SGI50928, SGI50928, SGI0, SGI50929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11514[] = {SGI50930, SGI50931, SGI50932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50933, SGI50933, SGI0, SGI0, SGI0, SGI50934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11515[] = {SGI50935, SGI0, SGI50936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50937, SGI50938, SGI50938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50939, NULL};
const Segment * const SG11516[] = {SGI50940, SGI50941, SGI50942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50943, SGI0, SGI0, SGI0, SGI50944, SGI50944, SGI0, SGI50945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11517[] = {SGI50946, SGI50947, SGI50948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50949, SGI0, SGI0, SGI0, SGI50950, SGI50950, SGI0, SGI50951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11518[] = {SGI50952, SGI50953, SGI50954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50955, SGI0, SGI0, SGI0, SGI50956, SGI50956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11519[] = {SGI50957, SGI50958, SGI50959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50960, SGI0, SGI0, SGI0, SGI50961, SGI50961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11520[] = {SGI0, SGI50962, SGI50963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50964, SGI0, SGI0, SGI0, SGI0, SGI50965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11521[] = {SGI0, SGI50966, SGI50967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50968, SGI0, SGI0, SGI0, SGI0, SGI50969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11522[] = {SGI50970, SGI50971, SGI50972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50973, SGI50974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11523[] = {SGI50975, SGI50976, SGI50977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50978, SGI50978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11524[] = {SGI50979, SGI0, SGI50980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50981, SGI0, SGI0, SGI0, SGI50982, SGI50982, SGI0, SGI50983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11525[] = {SGI50984, SGI50985, SGI50986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50987, SGI0, SGI0, SGI0, SGI50988, SGI50988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11526[] = {SGI50989, SGI0, SGI50990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50991, SGI0, SGI0, SGI0, SGI50992, SGI50992, SGI0, SGI50993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11527[] = {SGI0, SGI50994, SGI50995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI50996, SGI0, SGI0, SGI0, SGI50997, SGI50997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11528[] = {SGI50998, SGI50999, SGI51000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51001, SGI0, SGI0, SGI0, SGI51002, SGI51002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51003, SGI0, NULL};
const Segment * const SG11529[] = {SGI51004, SGI51005, SGI51006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51007, SGI0, SGI0, SGI0, SGI51008, SGI51008, SGI0, SGI0, SGI51009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11530[] = {SGI51010, SGI51011, SGI51012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51013, SGI0, SGI0, SGI0, SGI51014, SGI51014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11531[] = {SGI0, SGI51015, SGI0, SGI51016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11532[] = {SGI51017, SGI51018, SGI51019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51020, SGI0, SGI0, SGI0, SGI51021, SGI51021, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11533[] = {SGI51022, SGI51023, SGI51024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51025, SGI0, SGI0, SGI0, SGI51026, SGI51026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11534[] = {SGI51027, SGI51028, SGI51029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51030, SGI0, SGI0, SGI0, SGI51031, SGI51031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11535[] = {SGI51032, SGI51033, SGI51034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51035, SGI0, SGI0, SGI0, SGI51036, SGI51036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11536[] = {SGI51037, SGI51038, SGI51039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51040, SGI51041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11537[] = {SGI51042, SGI51043, SGI51044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51045, SGI51045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11538[] = {SGI51046, SGI0, SGI51047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11539[] = {SGI51048, SGI51049, SGI51050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51051, SGI0, SGI0, SGI0, SGI51052, SGI51052, SGI0, SGI0, SGI0, SGI0, SGI51053, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11540[] = {SGI51054, SGI51055, SGI51056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51057, SGI0, SGI0, SGI0, SGI51058, SGI51058, SGI0, SGI0, SGI0, SGI0, SGI51059, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11541[] = {SGI51060, SGI51061, SGI51062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51063, SGI0, SGI0, SGI0, SGI51064, SGI51064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11542[] = {SGI51065, SGI51066, SGI51067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51068, SGI0, SGI0, SGI0, SGI51069, SGI51069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11543[] = {SGI51070, SGI51071, SGI51072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51073, SGI51073, SGI0, SGI51074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11544[] = {SGI51075, SGI51076, SGI51077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51078, SGI0, SGI0, SGI0, SGI51079, SGI51079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11545[] = {SGI51080, SGI51081, SGI51082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51083, SGI0, SGI0, SGI0, SGI51084, SGI51084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11546[] = {SGI51085, SGI51086, SGI51087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51088, SGI0, SGI0, SGI0, SGI51089, SGI51089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11547[] = {SGI51090, SGI51091, SGI51092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51093, SGI0, SGI0, SGI0, SGI51094, SGI51094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11548[] = {SGI51095, SGI51096, SGI51097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51098, SGI0, SGI0, SGI0, SGI51099, SGI51099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11549[] = {SGI51100, SGI51101, SGI51102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51103, SGI0, SGI0, SGI0, SGI51104, SGI51104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11550[] = {SGI51105, SGI0, SGI51106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11551[] = {SGI0, SGI51107, SGI51108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51109, SGI0, SGI0, SGI0, SGI0, SGI51110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11552[] = {SGI51111, SGI51112, SGI51113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51114, SGI51115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11553[] = {SGI0, SGI51116, SGI51117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51118, SGI0, SGI0, SGI0, SGI0, SGI51119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11554[] = {SGI51120, SGI51121, SGI51122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51123, SGI51123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11555[] = {SGI0, SGI51124, SGI51125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51126, SGI0, SGI0, SGI0, SGI0, SGI51127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11556[] = {SGI51128, SGI51129, SGI51130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51131, SGI0, SGI0, SGI0, SGI51132, SGI51132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51133, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11557[] = {SGI51134, SGI51135, SGI51136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51137, SGI51137, SGI0, SGI0, SGI0, SGI51138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11558[] = {SGI51139, SGI51140, SGI51141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51142, SGI51143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11559[] = {SGI51144, SGI51145, SGI51146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51147, SGI0, SGI0, SGI0, SGI51148, SGI51148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11560[] = {SGI51149, SGI51150, SGI51151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51152, SGI51152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11561[] = {SGI51153, SGI51154, SGI51155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51156, SGI51156, SGI0, SGI51157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11562[] = {SGI51158, SGI51159, SGI51160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51161, SGI0, SGI0, SGI0, SGI51162, SGI51162, SGI0, SGI0, SGI0, SGI0, SGI51163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11563[] = {SGI51164, SGI51165, SGI51166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51167, SGI0, SGI0, SGI0, SGI51168, SGI51168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11564[] = {SGI51169, SGI51170, SGI51171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51172, SGI51172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11565[] = {SGI51173, SGI51174, SGI51175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51176, SGI0, SGI0, SGI0, SGI51177, SGI51177, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11566[] = {SGI51178, SGI0, SGI51179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51180, SGI0, SGI0, SGI0, SGI51181, SGI51181, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11567[] = {SGI51182, SGI51183, SGI51184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51185, SGI51185, SGI0, SGI0, SGI0, SGI51186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11568[] = {SGI51187, SGI51188, SGI51189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51190, SGI0, SGI0, SGI0, SGI51191, SGI51191, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11569[] = {SGI51192, SGI0, SGI51193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51194, SGI0, SGI0, SGI0, SGI51195, SGI51195, SGI0, SGI0, SGI0, SGI51196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11570[] = {SGI0, SGI0, SGI51197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51198, SGI0, SGI0, SGI0, SGI51199, SGI51199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11571[] = {SGI51200, SGI51201, SGI51202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51203, SGI0, SGI0, SGI0, SGI51204, SGI51204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11572[] = {SGI0, SGI51205, SGI0, SGI51207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11573[] = {SGI51206, SGI51208, SGI51209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51210, SGI51210, SGI0, SGI0, SGI51211, SGI0, NULL};
const Segment * const SG11574[] = {SGI51212, SGI51213, SGI51214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51215, SGI0, SGI0, SGI0, SGI51216, SGI51216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11575[] = {SGI51217, SGI0, SGI51218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51219, SGI0, SGI0, SGI0, SGI51220, SGI51220, SGI0, SGI0, SGI0, SGI51221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11576[] = {SGI51222, SGI51223, SGI51224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51225, SGI0, SGI0, SGI0, SGI51226, SGI51226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11577[] = {SGI51227, SGI51228, SGI51229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51230, SGI51230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11578[] = {SGI51231, SGI51232, SGI51233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51234, SGI0, SGI0, SGI0, SGI51235, SGI51235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11579[] = {SGI51236, SGI51237, SGI51238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51239, SGI0, SGI0, SGI0, SGI51240, SGI51240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11580[] = {SGI51241, SGI0, SGI51242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51243, SGI0, SGI0, SGI0, SGI51244, SGI51244, SGI0, SGI0, SGI0, SGI51245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11581[] = {SGI51246, SGI0, SGI51247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51248, SGI0, SGI0, SGI0, SGI51249, SGI51249, SGI0, SGI0, SGI51250, SGI0, NULL};
const Segment * const SG11582[] = {SGI51251, SGI51252, SGI51253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51254, SGI0, SGI0, SGI0, SGI51255, SGI51255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11583[] = {SGI51256, SGI51257, SGI51258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51259, SGI51259, SGI0, SGI0, SGI0, SGI51260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11584[] = {SGI51261, SGI51262, SGI51263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51264, SGI51264, SGI0, SGI0, SGI0, SGI51265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11585[] = {SGI51266, SGI0, SGI51267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51268, SGI0, SGI0, SGI0, SGI51269, SGI51269, SGI0, SGI0, SGI0, SGI51270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11586[] = {SGI51271, SGI51272, SGI51273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51274, SGI0, SGI0, SGI0, SGI51275, SGI51275, SGI0, SGI0, NULL};
const Segment * const SG11587[] = {SGI51276, SGI0, SGI51277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51278, SGI0, SGI0, SGI0, SGI51279, SGI51279, SGI0, SGI0, SGI0, SGI51280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11588[] = {SGI51281, SGI51282, SGI51283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51284, SGI0, SGI0, SGI0, SGI51285, SGI51285, SGI0, SGI0, NULL};
const Segment * const SG11589[] = {SGI51286, SGI51287, SGI51288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51289, SGI0, SGI0, SGI0, SGI51290, SGI51290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11590[] = {SGI51291, SGI51292, SGI51293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51294, SGI0, SGI0, SGI0, SGI51295, SGI51295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11591[] = {SGI51296, SGI0, SGI51297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51298, SGI0, SGI0, SGI0, SGI51299, SGI51299, SGI0, SGI0, SGI0, SGI51300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11592[] = {SGI51301, SGI51302, SGI51303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51304, SGI0, SGI0, SGI0, SGI51305, SGI51305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11593[] = {SGI0, SGI51306, SGI0, SGI51307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51308, SGI0, SGI0, SGI0, SGI51309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11594[] = {SGI0, SGI51310, SGI0, SGI51311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11595[] = {SGI51312, SGI51313, SGI51314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51315, SGI0, SGI0, SGI0, SGI51316, SGI51316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11596[] = {SGI51317, SGI51318, SGI51319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51320, SGI0, SGI0, SGI0, SGI51321, SGI51321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11597[] = {SGI51322, SGI0, SGI51323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51324, SGI0, SGI0, SGI0, SGI51325, SGI51325, SGI0, SGI0, SGI0, SGI51326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11598[] = {SGI51327, SGI0, SGI51328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51329, SGI0, SGI0, SGI0, SGI51330, SGI51330, SGI0, SGI0, SGI0, SGI51331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11599[] = {SGI51332, SGI51333, SGI51334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51335, SGI0, SGI0, SGI0, SGI51336, SGI51336, SGI0, SGI0, NULL};
const Segment * const SG11600[] = {SGI0, SGI51337, SGI0, SGI51338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51339, SGI0, SGI0, SGI0, SGI51340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11601[] = {SGI0, SGI51341, SGI0, SGI51342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11602[] = {SGI51343, SGI51344, SGI51345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51346, SGI0, SGI0, SGI0, SGI51347, SGI51347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11603[] = {SGI51348, SGI51349, SGI51350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51351, SGI0, SGI0, SGI0, SGI51352, SGI51352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11604[] = {SGI51353, SGI51354, SGI51355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51356, SGI0, SGI0, SGI0, SGI51357, SGI51357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11605[] = {SGI51358, SGI51359, SGI51360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51361, SGI51361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11606[] = {SGI51362, SGI51363, SGI51364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51365, SGI0, SGI0, SGI0, SGI51366, SGI51366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11607[] = {SGI0, SGI51367, SGI51368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11608[] = {SGI51369, SGI0, SGI51370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11609[] = {SGI51371, SGI0, SGI51372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51373, SGI0, SGI0, SGI0, SGI51374, SGI51374, SGI0, SGI0, SGI0, SGI51375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11610[] = {SGI51376, SGI51377, SGI51378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51379, SGI0, SGI0, SGI0, SGI51380, SGI51380, SGI0, SGI0, NULL};
const Segment * const SG11611[] = {SGI0, SGI51381, SGI0, SGI51382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11612[] = {SGI51383, SGI51384, SGI51385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51386, SGI0, SGI0, SGI0, SGI51387, SGI51387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11613[] = {SGI51388, SGI51389, SGI51390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51391, SGI51391, SGI0, SGI0, SGI0, SGI51392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11614[] = {SGI51393, SGI51394, SGI51395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51396, SGI0, SGI0, SGI0, SGI51397, SGI51397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11615[] = {SGI51398, SGI51399, SGI51400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51401, SGI0, SGI0, SGI0, SGI51402, SGI51402, SGI0, SGI0, NULL};
const Segment * const SG11616[] = {SGI51403, SGI51404, SGI51405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51406, SGI51406, SGI0, SGI51407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11617[] = {SGI51408, SGI51409, SGI51410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51411, SGI51411, SGI0, SGI0, SGI0, SGI51412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11618[] = {SGI51416, SGI0, SGI51417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51418, SGI0, SGI0, SGI0, SGI51419, SGI51419, SGI0, SGI51420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11619[] = {SGI51421, SGI0, SGI51422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51423, SGI0, SGI0, SGI0, SGI51424, SGI51424, SGI0, SGI51426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11620[] = {SGI51425, SGI0, SGI51427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51428, SGI0, SGI0, SGI0, SGI51429, SGI51429, SGI0, SGI0, SGI51430, SGI0, NULL};
const Segment * const SG11621[] = {SGI51432, SGI51434, SGI51435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51437, SGI0, SGI0, SGI0, SGI51439, SGI51439, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11622[] = {SGI51431, SGI0, SGI51433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51436, SGI0, SGI0, SGI0, SGI51438, SGI51438, SGI0, SGI51440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11623[] = {SGI51413, SGI51414, SGI51415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51441, SGI0, SGI0, SGI0, SGI51442, SGI51442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11624[] = {SGI51443, SGI0, SGI51445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51447, SGI0, SGI0, SGI0, SGI51449, SGI51449, SGI0, SGI0, SGI0, SGI51451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11625[] = {SGI51444, SGI0, SGI51446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51448, SGI0, SGI0, SGI0, SGI51450, SGI51450, SGI0, SGI0, SGI51452, SGI0, NULL};
const Segment * const SG11626[] = {SGI51453, SGI0, SGI51454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51455, SGI0, SGI0, SGI0, SGI51456, SGI51456, SGI0, SGI0, SGI51457, SGI0, NULL};
const Segment * const SG11627[] = {SGI51458, SGI51459, SGI51460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51461, SGI0, SGI0, SGI0, SGI51462, SGI51462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51463, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11628[] = {SGI51464, SGI51465, SGI51466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51467, SGI0, SGI0, SGI0, SGI51468, SGI51468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51469, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11629[] = {SGI0, SGI51470, SGI0, SGI51471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11630[] = {SGI0, SGI51472, SGI0, SGI51473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51474, SGI0, SGI0, SGI0, SGI51475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11631[] = {SGI51476, SGI51477, SGI51478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51479, SGI0, SGI0, SGI0, SGI51480, SGI51480, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11632[] = {SGI51481, SGI0, SGI51482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51483, SGI0, SGI0, SGI0, SGI51484, SGI51484, SGI0, SGI0, SGI0, SGI51485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11633[] = {SGI51486, SGI51487, SGI51488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51489, SGI0, SGI0, SGI0, SGI51490, SGI51490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11634[] = {SGI51491, SGI51492, SGI51493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51494, SGI0, SGI0, SGI0, SGI51495, SGI51495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11635[] = {SGI51496, SGI0, SGI51497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51498, SGI0, SGI0, SGI0, SGI51499, SGI51499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11636[] = {SGI51500, SGI51501, SGI51502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51503, SGI0, SGI0, SGI0, SGI51504, SGI51504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11637[] = {SGI51505, SGI51506, SGI51507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51508, SGI0, SGI0, SGI0, SGI51509, SGI51509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11638[] = {SGI51510, SGI51511, SGI51512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51513, SGI51513, SGI0, SGI0, SGI51514, SGI0, NULL};
const Segment * const SG11639[] = {SGI51515, SGI51516, SGI51517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51518, SGI0, SGI0, SGI0, SGI51519, SGI51519, SGI0, SGI0, NULL};
const Segment * const SG11640[] = {SGI51520, SGI51521, SGI51522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51523, SGI51523, SGI0, SGI0, NULL};
const Segment * const SG11641[] = {SGI51524, SGI51525, SGI51526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51527, SGI0, SGI0, SGI0, SGI51528, SGI51528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11642[] = {SGI51529, SGI51530, SGI51531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51532, SGI0, SGI0, SGI0, SGI51533, SGI51533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11643[] = {SGI51534, SGI51535, SGI51536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51537, SGI0, SGI0, SGI0, SGI51538, SGI51538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11644[] = {SGI51539, SGI51540, SGI51541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51542, SGI0, SGI0, SGI0, SGI51543, SGI51543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11645[] = {SGI51544, SGI51545, SGI51546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51547, SGI0, SGI0, SGI0, SGI51548, SGI51548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11646[] = {SGI51549, SGI51550, SGI51551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51552, SGI0, SGI0, SGI0, SGI51553, SGI51553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51554, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11647[] = {SGI51555, SGI51556, SGI51557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51558, SGI51558, SGI0, SGI0, SGI0, SGI51559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11648[] = {SGI51560, SGI51561, SGI51562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51563, SGI0, SGI0, SGI0, SGI51564, SGI51564, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11649[] = {SGI51565, SGI51566, SGI51567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51568, SGI51568, SGI0, SGI51570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11650[] = {SGI0, SGI0, SGI51569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51571, SGI0, SGI0, SGI0, SGI51572, SGI51572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11651[] = {SGI51573, SGI0, SGI51574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51575, SGI0, SGI0, SGI0, SGI51576, SGI51576, SGI0, SGI0, SGI51577, SGI0, NULL};
const Segment * const SG11652[] = {SGI51578, SGI51579, SGI51580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51581, SGI0, SGI0, SGI0, SGI51582, SGI51582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11653[] = {SGI51583, SGI51584, SGI51585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51586, SGI0, SGI0, SGI0, SGI51587, SGI51587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11654[] = {SGI51588, SGI51589, SGI51590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51591, SGI0, SGI0, SGI0, SGI51592, SGI51592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11655[] = {SGI51593, SGI51594, SGI51595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51596, SGI0, SGI0, SGI0, SGI51597, SGI51597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51598, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11656[] = {SGI51599, SGI51600, SGI51601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51602, SGI0, SGI0, SGI0, SGI51603, SGI51603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11657[] = {SGI51604, SGI0, SGI51605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51606, SGI0, SGI0, SGI0, SGI51607, SGI51607, SGI0, SGI0, SGI0, SGI51608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11658[] = {SGI51609, SGI51610, SGI51611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51612, SGI0, SGI0, SGI0, SGI51613, SGI51613, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11659[] = {SGI51614, SGI51615, SGI51616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51617, SGI0, SGI0, SGI0, SGI51618, SGI51618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11660[] = {SGI51619, SGI0, SGI51620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11661[] = {SGI51621, SGI51622, SGI51623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51624, SGI0, SGI0, SGI0, SGI51625, SGI51625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11662[] = {SGI51626, SGI0, SGI51627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51628, SGI0, SGI0, SGI0, SGI51629, SGI51629, SGI0, SGI51630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11663[] = {SGI51631, SGI0, SGI51632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51633, SGI0, SGI0, SGI0, SGI51634, SGI51634, SGI0, SGI51635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11664[] = {SGI51636, SGI0, SGI51637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51638, SGI0, SGI0, SGI0, SGI51639, SGI51639, SGI0, SGI51640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11665[] = {SGI51641, SGI0, SGI51642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51643, SGI0, SGI0, SGI0, SGI51644, SGI51644, SGI0, SGI51645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11666[] = {SGI51646, SGI51647, SGI51648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51649, SGI0, SGI0, SGI0, SGI51650, SGI51650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51651, SGI0, NULL};
const Segment * const SG11667[] = {SGI51652, SGI51653, SGI51654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51655, SGI0, SGI0, SGI0, SGI51656, SGI51656, SGI0, SGI0, SGI51657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11668[] = {SGI51658, SGI0, SGI51659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51660, SGI0, SGI0, SGI0, SGI51661, SGI51661, SGI0, SGI0, SGI0, SGI51662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11669[] = {SGI51663, SGI51664, SGI51665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51666, SGI0, SGI0, SGI0, SGI51667, SGI51667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11670[] = {SGI51668, SGI51669, SGI51670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51671, SGI0, SGI0, SGI0, SGI51672, SGI51672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11671[] = {SGI51673, SGI51674, SGI51675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51676, SGI0, SGI0, SGI0, SGI51677, SGI51677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11672[] = {SGI51678, SGI51679, SGI51680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51681, SGI0, SGI0, SGI0, SGI51682, SGI51682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11673[] = {SGI51683, SGI51684, SGI51685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51686, SGI0, SGI0, SGI0, SGI51687, SGI51687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11674[] = {SGI51688, SGI0, SGI51689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51690, SGI0, SGI0, SGI0, SGI51691, SGI51691, SGI0, SGI0, SGI0, SGI51692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11675[] = {SGI51693, SGI51694, SGI51695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51696, SGI0, SGI0, SGI0, SGI51697, SGI51697, SGI0, SGI0, SGI51698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11676[] = {SGI51699, SGI51700, SGI51701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51702, SGI0, SGI0, SGI0, SGI51703, SGI51703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11677[] = {SGI51704, SGI51705, SGI51706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51707, SGI0, SGI0, SGI0, SGI51708, SGI51708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11678[] = {SGI51709, SGI51710, SGI51711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51712, SGI0, SGI0, SGI0, SGI51713, SGI51713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11679[] = {SGI51714, SGI51715, SGI51716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51717, SGI0, SGI0, SGI0, SGI51718, SGI51718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11680[] = {SGI51719, SGI51720, SGI51721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51722, SGI0, SGI0, SGI0, SGI51723, SGI51723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11681[] = {SGI51724, SGI51725, SGI51726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51727, SGI0, SGI0, SGI0, SGI51728, SGI51728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11682[] = {SGI51729, SGI51730, SGI51731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51732, SGI0, SGI0, SGI0, SGI51733, SGI51733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11683[] = {SGI51734, SGI51735, SGI51736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51737, SGI0, SGI0, SGI0, SGI51738, SGI51738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51739, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11684[] = {SGI51740, SGI51741, SGI51742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51743, SGI0, SGI0, SGI0, SGI51744, SGI51744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51745, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11685[] = {SGI51746, SGI51747, SGI51748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51749, SGI51749, SGI0, SGI0, SGI0, SGI51750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11686[] = {SGI51751, SGI51752, SGI51753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51754, SGI0, SGI0, SGI0, SGI51755, SGI51755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11687[] = {SGI51756, SGI0, SGI51757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51758, SGI0, SGI0, SGI0, SGI51759, SGI51759, SGI0, SGI0, SGI0, SGI51760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11688[] = {SGI51761, SGI51762, SGI51763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51764, SGI0, SGI0, SGI0, SGI51765, SGI51765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11689[] = {SGI51766, SGI0, SGI51767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11690[] = {SGI51768, SGI0, SGI51769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51770, SGI0, SGI0, SGI0, SGI51771, SGI51771, SGI0, SGI51772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11691[] = {SGI51773, SGI51774, SGI51775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51776, SGI0, SGI0, SGI0, SGI51777, SGI51777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11692[] = {SGI51778, SGI0, SGI51779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51780, SGI51781, SGI51781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51782, NULL};
const Segment * const SG11693[] = {SGI51783, SGI0, SGI51784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51785, SGI0, SGI0, SGI0, SGI51786, SGI51786, SGI0, SGI0, SGI0, SGI51787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11694[] = {SGI51788, SGI51789, SGI51790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51791, SGI0, SGI0, SGI0, SGI51792, SGI51792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11695[] = {SGI51793, SGI51794, SGI51795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51796, SGI0, SGI0, SGI0, SGI51797, SGI51797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11696[] = {SGI51798, SGI0, SGI51799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51800, SGI0, SGI0, SGI0, SGI51801, SGI51801, SGI0, SGI0, SGI51802, SGI0, NULL};
const Segment * const SG11697[] = {SGI51803, SGI51804, SGI51805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51806, SGI0, SGI0, SGI0, SGI51807, SGI51807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11698[] = {SGI51808, SGI51809, SGI51810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51811, SGI0, SGI0, SGI0, SGI51812, SGI51812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11699[] = {SGI51813, SGI0, SGI51814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51815, SGI0, SGI0, SGI0, SGI51816, SGI51816, SGI0, SGI0, SGI0, SGI51817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11700[] = {SGI51818, SGI0, SGI51819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51820, SGI0, SGI0, SGI0, SGI51821, SGI51821, SGI0, SGI0, SGI0, SGI51822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11701[] = {SGI51823, SGI0, SGI51824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51825, SGI0, SGI0, SGI0, SGI51826, SGI51826, SGI0, SGI51827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11702[] = {SGI51828, SGI0, SGI51829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51830, SGI51831, SGI51831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51832, NULL};
const Segment * const SG11703[] = {SGI51833, SGI0, SGI51834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51835, SGI51836, SGI51836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51837, NULL};
const Segment * const SG11704[] = {SGI51838, SGI51839, SGI51840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51841, SGI0, SGI0, SGI0, SGI51842, SGI51842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11705[] = {SGI51843, SGI51844, SGI51845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51846, SGI0, SGI0, SGI0, SGI51847, SGI51847, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11706[] = {SGI51848, SGI0, SGI51849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51850, SGI0, SGI0, SGI0, SGI51851, SGI51851, SGI0, SGI0, SGI0, SGI51852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11707[] = {SGI51853, SGI0, SGI51854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51855, SGI0, SGI0, SGI0, SGI51856, SGI51856, SGI0, SGI0, SGI0, SGI51857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11708[] = {SGI51858, SGI51859, SGI51860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51861, SGI0, SGI0, SGI0, SGI51862, SGI51862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11709[] = {SGI51864, SGI51865, SGI51866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51867, SGI51867, SGI0, SGI0, SGI0, SGI51868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11710[] = {SGI51869, SGI0, SGI51870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51871, SGI0, SGI0, SGI0, SGI51872, SGI51872, SGI0, SGI0, SGI0, SGI51873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11711[] = {SGI51874, SGI0, SGI51875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51876, SGI51877, SGI51877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51878, NULL};
const Segment * const SG11712[] = {SGI0, SGI51879, SGI0, SGI51880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11713[] = {SGI51881, SGI51882, SGI51883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51884, SGI51884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11714[] = {SGI0, SGI51885, SGI51886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11715[] = {SGI51887, SGI51888, SGI51889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51890, SGI0, SGI0, SGI0, SGI51891, SGI51891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11716[] = {SGI0, SGI51892, SGI0, SGI51893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11717[] = {SGI51894, SGI51895, SGI51896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51897, SGI0, SGI0, SGI0, SGI51898, SGI51898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11718[] = {SGI51899, SGI0, SGI51900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11719[] = {SGI51901, SGI51902, SGI51903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51904, SGI0, SGI0, SGI0, SGI51905, SGI51905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51906, SGI0, SGI0, NULL};
const Segment * const SG11720[] = {SGI51907, SGI0, SGI51908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51909, SGI0, SGI0, SGI0, SGI51910, SGI51910, SGI0, SGI51911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11721[] = {SGI51912, SGI51913, SGI51914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51915, SGI0, SGI0, SGI0, SGI51916, SGI51916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11722[] = {SGI0, SGI51917, SGI0, SGI51918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11723[] = {SGI51919, SGI51920, SGI51921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51922, SGI0, SGI0, SGI0, SGI51923, SGI51923, SGI0, SGI0, SGI51924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11724[] = {SGI51925, SGI51926, SGI51927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51928, SGI0, SGI0, SGI0, SGI51929, SGI51929, SGI0, SGI0, SGI51930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11725[] = {SGI51931, SGI51932, SGI51933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51934, SGI0, SGI0, SGI0, SGI51935, SGI51935, SGI0, SGI0, SGI51936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11726[] = {SGI51937, SGI51938, SGI51939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51940, SGI0, SGI0, SGI0, SGI51941, SGI51941, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11727[] = {SGI51942, SGI51943, SGI51944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51945, SGI0, SGI0, SGI0, SGI51946, SGI51946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11728[] = {SGI51947, SGI0, SGI51948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51949, SGI0, SGI0, SGI0, SGI51950, SGI51950, SGI0, SGI0, SGI0, SGI51951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11729[] = {SGI51952, SGI51953, SGI51954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51955, SGI0, SGI0, SGI0, SGI51956, SGI51956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51957, SGI0, NULL};
const Segment * const SG11730[] = {SGI51958, SGI51959, SGI51960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51961, SGI0, SGI0, SGI0, SGI51962, SGI51962, SGI0, SGI0, SGI51963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11731[] = {SGI51964, SGI51965, SGI51966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51967, SGI0, SGI0, SGI0, SGI51968, SGI51968, SGI0, SGI0, SGI51969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11732[] = {SGI51970, SGI51971, SGI51972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51973, SGI51973, SGI0, SGI0, SGI0, SGI51974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11733[] = {SGI51975, SGI51976, SGI51977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51978, SGI0, SGI0, SGI0, SGI51979, SGI51979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11734[] = {SGI51980, SGI51981, SGI51982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51983, SGI0, SGI0, SGI0, SGI51984, SGI51984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11735[] = {SGI51985, SGI51986, SGI51987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51988, SGI51988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11736[] = {SGI0, SGI51989, SGI0, SGI51990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11737[] = {SGI0, SGI51991, SGI0, SGI51992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11738[] = {SGI51993, SGI51994, SGI51995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI51996, SGI0, SGI0, SGI0, SGI51997, SGI51997, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11739[] = {SGI51998, SGI51999, SGI52000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52001, SGI0, SGI0, SGI0, SGI52002, SGI52002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11740[] = {SGI52003, SGI52004, SGI52005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52006, SGI0, SGI0, SGI0, SGI52007, SGI52007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11741[] = {SGI52008, SGI52009, SGI52010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52011, SGI0, SGI0, SGI0, SGI52012, SGI52012, SGI0, SGI0, NULL};
const Segment * const SG11742[] = {SGI52013, SGI0, SGI52014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52015, SGI0, SGI0, SGI0, SGI52016, SGI52016, SGI0, SGI0, SGI0, SGI52017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11743[] = {SGI52018, SGI52019, SGI52020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52021, SGI0, SGI0, SGI0, SGI52022, SGI52022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11744[] = {SGI52023, SGI52024, SGI52025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52026, SGI52026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11745[] = {SGI52027, SGI52028, SGI52029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52030, SGI52030, SGI0, SGI0, SGI0, SGI52031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11746[] = {SGI52032, SGI0, SGI52033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52034, SGI0, SGI0, SGI0, SGI52035, SGI52035, SGI0, SGI0, SGI0, SGI52036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11747[] = {SGI52038, SGI52040, SGI52042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52044, SGI0, SGI0, SGI0, SGI52046, SGI52046, SGI0, SGI0, NULL};
const Segment * const SG11748[] = {SGI52037, SGI52039, SGI52041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52043, SGI0, SGI0, SGI0, SGI52045, SGI52045, SGI0, SGI0, SGI52047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11749[] = {SGI52048, SGI52049, SGI52050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52051, SGI0, SGI0, SGI0, SGI52052, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11750[] = {SGI0, SGI52053, SGI52054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52055, SGI0, SGI0, SGI0, SGI52056, SGI52056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11751[] = {SGI52057, SGI52058, SGI52059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52060, SGI0, SGI0, SGI0, SGI52061, SGI52061, SGI0, SGI52062, SGI0, SGI0, NULL};
const Segment * const SG11752[] = {SGI52063, SGI52064, SGI52065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52066, SGI0, SGI0, SGI0, SGI52067, SGI52067, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11753[] = {SGI52068, SGI0, SGI52069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52070, SGI0, SGI0, SGI0, SGI52071, SGI52071, SGI0, SGI0, SGI52072, SGI0, NULL};
const Segment * const SG11754[] = {SGI0, SGI52073, SGI0, SGI52074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11755[] = {SGI52075, SGI52076, SGI52077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52078, SGI52078, SGI0, SGI52079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11756[] = {SGI52080, SGI0, SGI52081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52082, SGI0, SGI0, SGI0, SGI52083, SGI52083, SGI0, SGI0, SGI52084, SGI0, NULL};
const Segment * const SG11757[] = {SGI52085, SGI52086, SGI52087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52088, SGI52088, SGI0, SGI0, SGI0, SGI52089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11758[] = {SGI52090, SGI52091, SGI52092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52093, SGI52093, SGI0, SGI0, SGI0, SGI52094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11759[] = {SGI52095, SGI52096, SGI52097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52098, SGI0, SGI0, SGI0, SGI0, SGI52099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11760[] = {SGI52100, SGI0, SGI52101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52102, SGI0, SGI0, SGI0, SGI52103, SGI52103, SGI0, SGI0, SGI0, SGI52104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11761[] = {SGI52105, SGI0, SGI52106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52107, SGI0, SGI0, SGI0, SGI52108, SGI52108, SGI0, SGI52109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11762[] = {SGI52110, SGI52111, SGI52112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52113, SGI0, SGI0, SGI0, SGI52114, SGI52114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11763[] = {SGI52115, SGI52116, SGI52117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52118, SGI0, SGI0, SGI0, SGI52119, SGI52119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11764[] = {SGI52120, SGI52121, SGI52122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52123, SGI0, SGI0, SGI0, SGI52124, SGI52124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11765[] = {SGI0, SGI52125, SGI52126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52127, SGI0, SGI0, SGI0, SGI52128, SGI52128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11766[] = {SGI52129, SGI52130, SGI52131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52132, SGI0, SGI0, SGI0, SGI52133, SGI52133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11767[] = {SGI52134, SGI52135, SGI52136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52137, SGI0, SGI0, SGI0, SGI52138, SGI52138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11768[] = {SGI52139, SGI52140, SGI52141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52142, SGI0, SGI0, SGI0, SGI52143, SGI52143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11769[] = {SGI52144, SGI52145, SGI52146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52147, SGI52148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11770[] = {SGI52149, SGI52150, SGI52151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52152, SGI52152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11771[] = {SGI52153, SGI52154, SGI52155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52156, SGI52157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11772[] = {SGI52158, SGI52159, SGI52160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52161, SGI0, SGI0, SGI0, SGI52162, SGI52162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11773[] = {SGI52163, SGI52164, SGI52165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52166, SGI52167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11774[] = {SGI52168, SGI0, SGI52169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52170, SGI0, SGI0, SGI0, SGI52171, SGI52171, SGI0, SGI0, SGI0, SGI52172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11775[] = {SGI52173, SGI52174, SGI52175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52176, SGI0, SGI0, SGI0, SGI52177, SGI52177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11776[] = {SGI52178, SGI52179, SGI52180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52181, SGI0, SGI0, SGI0, SGI52182, SGI52182, SGI0, SGI0, NULL};
const Segment * const SG11777[] = {SGI0, SGI52183, SGI52184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52185, SGI0, SGI0, SGI0, SGI52186, SGI52186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11778[] = {SGI52187, SGI52188, SGI52189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52190, SGI52190, SGI0, SGI0, SGI52191, SGI0, NULL};
const Segment * const SG11779[] = {SGI52192, SGI0, SGI52193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52194, SGI0, SGI0, SGI0, SGI52195, SGI52195, SGI0, SGI52196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11780[] = {SGI52197, SGI52198, SGI52199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52200, SGI0, SGI0, SGI0, SGI52201, SGI52201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11781[] = {SGI0, SGI52202, SGI52203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52204, SGI0, SGI0, SGI0, SGI52205, SGI52205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11782[] = {SGI0, SGI52206, SGI0, SGI52207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11783[] = {SGI52208, SGI52209, SGI52210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52211, SGI0, SGI0, SGI0, SGI52212, SGI52212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11784[] = {SGI52213, SGI0, SGI52214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11785[] = {SGI52215, SGI52216, SGI52217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52218, SGI52218, SGI0, SGI0, SGI0, SGI52219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11786[] = {SGI52220, SGI52221, SGI52223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52226, SGI52226, SGI0, SGI0, SGI0, SGI52229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11787[] = {SGI52222, SGI52224, SGI52225, SGI0, SGI0, SGI0, SGI52227, SGI52228, SGI0, SGI52230, SGI0, SGI0, SGI0, SGI52231, SGI52231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11788[] = {SGI52232, SGI52233, SGI52234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52235, SGI52235, SGI0, SGI0, SGI0, SGI52236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11789[] = {SGI52237, SGI52238, SGI52239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52240, SGI52240, SGI0, SGI0, SGI0, SGI52241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11790[] = {SGI52244, SGI0, SGI52245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52246, SGI0, SGI0, SGI0, SGI52248, SGI52248, SGI0, SGI0, SGI0, SGI52251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11791[] = {SGI52247, SGI52249, SGI52250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52252, SGI52252, SGI0, SGI52253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11792[] = {SGI52242, SGI0, SGI52243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52254, SGI0, SGI0, SGI0, SGI52255, SGI52255, SGI0, SGI0, SGI52256, SGI0, NULL};
const Segment * const SG11793[] = {SGI52257, SGI0, SGI52258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52259, SGI0, SGI0, SGI0, SGI52260, SGI52260, SGI0, SGI0, SGI52261, SGI0, NULL};
const Segment * const SG11794[] = {SGI52262, SGI52263, SGI52264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52265, SGI0, SGI0, SGI0, SGI52266, SGI52266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11795[] = {SGI52267, SGI52268, SGI52269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52270, SGI0, SGI0, SGI0, SGI52271, SGI52271, SGI0, SGI0, SGI52272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11796[] = {SGI52273, SGI52274, SGI52275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52276, SGI0, SGI0, SGI0, SGI52277, SGI52277, SGI0, SGI0, SGI52278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11797[] = {SGI52279, SGI52280, SGI52281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52282, SGI0, SGI0, SGI0, SGI52283, SGI52283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11798[] = {SGI0, SGI52284, SGI0, SGI52285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11799[] = {SGI52286, SGI52287, SGI52288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52289, SGI52289, SGI0, SGI0, SGI0, SGI52290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11800[] = {SGI52291, SGI0, SGI52292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52293, SGI0, SGI0, SGI0, SGI52294, SGI52294, SGI0, SGI0, SGI0, SGI52295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11801[] = {SGI52296, SGI0, SGI52297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52298, SGI0, SGI0, SGI0, SGI52299, SGI52299, SGI0, SGI0, SGI0, SGI52300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11802[] = {SGI52301, SGI52302, SGI52303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52304, SGI0, SGI0, SGI0, SGI52305, SGI52306, SGI0, SGI0, SGI0, SGI52307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11803[] = {SGI52308, SGI52309, SGI52310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52311, SGI52311, SGI0, SGI52312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11804[] = {SGI52313, SGI0, SGI52314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52315, SGI0, SGI0, SGI0, SGI52316, SGI52316, SGI0, SGI0, SGI0, SGI52317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11805[] = {SGI52318, SGI52319, SGI52320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52321, SGI0, SGI0, SGI0, SGI52322, SGI52323, SGI0, SGI0, SGI0, SGI52324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11806[] = {SGI52325, SGI52326, SGI52327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52328, SGI0, SGI0, SGI0, SGI52329, SGI52329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11807[] = {SGI52330, SGI52331, SGI52332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52333, SGI52333, SGI0, SGI0, SGI0, SGI52334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11808[] = {SGI52335, SGI52336, SGI52337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52338, SGI0, SGI0, SGI0, SGI52339, SGI52339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11809[] = {SGI52340, SGI52341, SGI52342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52343, SGI0, SGI0, SGI0, SGI52344, SGI52344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11810[] = {SGI52345, SGI52346, SGI52347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52348, SGI0, SGI0, SGI0, SGI52349, SGI52349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11811[] = {SGI52350, SGI52351, SGI52352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52353, SGI0, SGI0, SGI0, SGI52354, SGI52354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11812[] = {SGI52355, SGI52356, SGI52357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52358, SGI0, SGI0, SGI0, SGI52359, SGI52359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11813[] = {SGI52360, SGI52361, SGI52362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52363, SGI0, SGI0, SGI0, SGI52364, SGI52364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11814[] = {SGI0, SGI52365, SGI0, SGI52366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11815[] = {SGI52367, SGI52368, SGI52369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52370, SGI0, SGI0, SGI0, SGI52371, SGI52371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11816[] = {SGI52372, SGI52373, SGI52374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52375, SGI52375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11817[] = {SGI52376, SGI0, SGI52377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52378, SGI0, SGI0, SGI0, SGI52379, SGI52379, SGI0, SGI0, SGI52380, SGI0, NULL};
const Segment * const SG11818[] = {SGI52381, SGI52382, SGI52383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52384, SGI0, SGI0, SGI0, SGI52385, SGI52385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11819[] = {SGI52386, SGI52387, SGI52388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52392, SGI0, SGI0, SGI0, SGI52393, SGI52393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11820[] = {SGI52389, SGI52390, SGI52391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52394, SGI0, SGI0, SGI0, SGI52395, SGI52395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11821[] = {SGI52396, SGI52397, SGI52398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52402, SGI52402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11822[] = {SGI52399, SGI52400, SGI52401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52403, SGI0, SGI0, SGI0, SGI52404, SGI52404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11823[] = {SGI52405, SGI52406, SGI52407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52408, SGI0, SGI0, SGI0, SGI52409, SGI52409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11824[] = {SGI52410, SGI52411, SGI52412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52413, SGI0, SGI0, SGI0, SGI52414, SGI52414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11825[] = {SGI52415, SGI52416, SGI52417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52418, SGI52418, SGI0, SGI0, SGI0, SGI52419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11826[] = {SGI52420, SGI52421, SGI52422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52423, SGI0, SGI0, SGI0, SGI52424, SGI52424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11827[] = {SGI52425, SGI52426, SGI52427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52428, SGI0, SGI0, SGI0, SGI52429, SGI52429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11828[] = {SGI52430, SGI52431, SGI52432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52433, SGI0, SGI0, SGI0, SGI52434, SGI52434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11829[] = {SGI52435, SGI0, SGI52436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52437, SGI0, SGI0, SGI0, SGI52438, SGI52438, SGI0, SGI0, SGI52439, SGI0, NULL};
const Segment * const SG11830[] = {SGI52440, SGI52441, SGI52442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52443, SGI0, SGI0, SGI0, SGI52444, SGI52444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11831[] = {SGI52445, SGI52446, SGI52447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52448, SGI0, SGI0, SGI0, SGI52449, SGI52449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11832[] = {SGI52450, SGI52451, SGI52452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52453, SGI0, SGI0, SGI0, SGI52454, SGI52454, SGI0, SGI52455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11833[] = {SGI52456, SGI0, SGI52457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52458, SGI0, SGI0, SGI0, SGI52459, SGI52459, SGI0, SGI0, SGI0, SGI52460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11834[] = {SGI52461, SGI52462, SGI52463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52464, SGI52464, SGI0, SGI0, SGI0, SGI52465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11835[] = {SGI52466, SGI52467, SGI52468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52469, SGI52469, SGI0, SGI52470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11836[] = {SGI52471, SGI0, SGI52472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52473, SGI0, SGI0, SGI0, SGI52474, SGI52474, SGI0, SGI0, SGI0, SGI52475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11837[] = {SGI52476, SGI0, SGI52477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52478, SGI0, SGI0, SGI0, SGI52479, SGI52479, SGI0, SGI0, SGI0, SGI52480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11838[] = {SGI52481, SGI52482, SGI52483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52484, SGI0, SGI0, SGI0, SGI52485, SGI52485, SGI0, SGI0, SGI52486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11839[] = {SGI52487, SGI52488, SGI52489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52490, SGI0, SGI0, SGI0, SGI52491, SGI52491, SGI0, SGI0, SGI52492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11840[] = {SGI52493, SGI52494, SGI52495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52496, SGI0, SGI0, SGI0, SGI52497, SGI52497, SGI0, SGI0, SGI52498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11841[] = {SGI52499, SGI0, SGI52500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52501, SGI0, SGI0, SGI0, SGI52502, SGI52502, SGI0, SGI0, SGI0, SGI52503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11842[] = {SGI0, SGI52504, SGI0, SGI52505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11843[] = {SGI52506, SGI52507, SGI52508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52509, SGI0, SGI0, SGI0, SGI52510, SGI52510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11844[] = {SGI52511, SGI0, SGI52512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11845[] = {SGI52513, SGI52514, SGI52515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52516, SGI0, SGI0, SGI0, SGI52517, SGI52517, SGI0, SGI0, NULL};
const Segment * const SG11846[] = {SGI52518, SGI52519, SGI52520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52521, SGI0, SGI0, SGI0, SGI52522, SGI52522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11847[] = {SGI0, SGI52523, SGI0, SGI52524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11848[] = {SGI52525, SGI52526, SGI52527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52528, SGI0, SGI0, SGI0, SGI52529, SGI52529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11849[] = {SGI52530, SGI0, SGI52531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52532, SGI0, SGI0, SGI0, SGI52533, SGI52533, SGI0, SGI0, SGI0, SGI52534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11850[] = {SGI52535, SGI0, SGI52536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52537, SGI0, SGI0, SGI0, SGI52538, SGI52538, SGI0, SGI52539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11851[] = {SGI52540, SGI52541, SGI52542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52543, SGI0, SGI0, SGI0, SGI52544, SGI52544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11852[] = {SGI52545, SGI0, SGI52546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11853[] = {SGI52547, SGI52548, SGI52549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52550, SGI0, SGI0, SGI0, SGI52551, SGI52551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11854[] = {SGI52552, SGI0, SGI52553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52554, SGI0, SGI0, SGI0, SGI52555, SGI52555, SGI0, SGI0, SGI0, SGI52556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11855[] = {SGI52557, SGI0, SGI52558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52559, SGI0, SGI0, SGI0, SGI52560, SGI52560, SGI0, SGI0, SGI52561, SGI0, NULL};
const Segment * const SG11856[] = {SGI52562, SGI52563, SGI52564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52565, SGI0, SGI0, SGI0, SGI52566, SGI52566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52567, SGI0, NULL};
const Segment * const SG11857[] = {SGI52568, SGI52569, SGI52570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52571, SGI0, SGI0, SGI0, SGI52572, SGI52572, SGI0, SGI0, SGI52573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11858[] = {SGI52574, SGI0, SGI52575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52576, SGI0, SGI0, SGI0, SGI52577, SGI52577, SGI0, SGI0, SGI0, SGI52578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11859[] = {SGI52579, SGI52580, SGI52581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52582, SGI52582, SGI0, SGI52583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11860[] = {SGI52584, SGI52585, SGI52586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52587, SGI0, SGI0, SGI0, SGI52588, SGI52588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11861[] = {SGI0, SGI52589, SGI52590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11862[] = {SGI52591, SGI52592, SGI52593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52594, SGI0, SGI0, SGI0, SGI52595, SGI52595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11863[] = {SGI52596, SGI52597, SGI52598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52601, SGI0, SGI0, SGI0, SGI52603, SGI52603, SGI0, SGI0, NULL};
const Segment * const SG11864[] = {SGI52599, SGI52600, SGI52602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52604, SGI0, SGI0, SGI0, SGI52605, SGI52605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11865[] = {SGI52606, SGI52607, SGI52608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52609, SGI0, SGI0, SGI0, SGI52610, SGI52610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11866[] = {SGI52612, SGI52613, SGI52615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52617, SGI0, SGI0, SGI0, SGI52618, SGI52618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11867[] = {SGI52611, SGI0, SGI52614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52616, SGI0, SGI0, SGI0, SGI52619, SGI52619, SGI0, SGI0, SGI0, SGI52620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11868[] = {SGI52621, SGI0, SGI52622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52623, SGI0, SGI0, SGI0, SGI52624, SGI52624, SGI0, SGI52625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11869[] = {SGI52626, SGI52627, SGI52628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52629, SGI52629, SGI0, SGI0, SGI0, SGI52630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11870[] = {SGI52631, SGI52632, SGI52633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52634, SGI0, SGI0, SGI0, SGI52635, SGI52635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11871[] = {SGI52636, SGI0, SGI52637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52638, SGI0, SGI0, SGI0, SGI52639, SGI52639, SGI0, SGI0, SGI0, SGI52640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11872[] = {SGI52641, SGI52642, SGI52643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52644, SGI0, SGI0, SGI0, SGI52645, SGI52645, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11873[] = {SGI52646, SGI52647, SGI52648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52649, SGI0, SGI0, SGI0, SGI52650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11874[] = {SGI52651, SGI52652, SGI52653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52654, SGI52655, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11875[] = {SGI52656, SGI52657, SGI52658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52659, SGI52660, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11876[] = {SGI52661, SGI52662, SGI52663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52664, SGI0, SGI0, SGI0, SGI52665, SGI52665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11877[] = {SGI52666, SGI52667, SGI52668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52669, SGI52669, SGI0, SGI0, SGI0, SGI52670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11878[] = {SGI52671, SGI0, SGI52672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52673, SGI0, SGI0, SGI0, SGI52674, SGI52674, SGI0, SGI0, SGI52675, SGI0, NULL};
const Segment * const SG11879[] = {SGI52676, SGI52677, SGI52678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52679, SGI0, SGI0, SGI0, SGI52680, SGI52680, SGI0, SGI0, SGI0, SGI0, SGI52681, SGI0, SGI0, SGI0, SGI52682, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11880[] = {SGI52683, SGI52684, SGI52685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52686, SGI0, SGI0, SGI0, SGI52687, SGI52687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11881[] = {SGI52689, SGI52690, SGI52691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52692, SGI0, SGI0, SGI0, SGI52693, SGI52693, SGI0, SGI0, SGI0, SGI0, SGI52694, SGI0, SGI0, SGI0, SGI52695, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11882[] = {SGI52696, SGI52697, SGI52698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52699, SGI0, SGI0, SGI0, SGI52700, SGI52700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11883[] = {SGI52701, SGI0, SGI52702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52703, SGI52704, SGI52704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52705, NULL};
const Segment * const SG11884[] = {SGI52706, SGI52707, SGI52708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52709, SGI52709, SGI0, SGI0, NULL};
const Segment * const SG11885[] = {SGI52710, SGI52711, SGI52712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52713, SGI52713, SGI0, SGI0, SGI0, SGI52714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11886[] = {SGI52715, SGI52716, SGI52717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52718, SGI52718, SGI0, SGI52719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11887[] = {SGI0, SGI52720, SGI52721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52722, SGI52722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11888[] = {SGI52724, SGI52725, SGI52726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52727, SGI52727, SGI0, SGI52728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11889[] = {SGI52729, SGI52730, SGI52731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52732, SGI52732, SGI0, SGI0, SGI52733, SGI0, NULL};
const Segment * const SG11890[] = {SGI52734, SGI0, SGI52735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52736, SGI0, SGI0, SGI0, SGI52737, SGI52737, SGI0, SGI0, SGI0, SGI52738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11891[] = {SGI52739, SGI52740, SGI52741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52742, SGI0, SGI0, SGI0, SGI52743, SGI52743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11892[] = {SGI52744, SGI52745, SGI52746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52747, SGI52747, SGI0, SGI0, SGI0, SGI52748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11893[] = {SGI52749, SGI52750, SGI52751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52752, SGI0, SGI0, SGI0, SGI52753, SGI52753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11894[] = {SGI52754, SGI52755, SGI52756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52757, SGI52757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11895[] = {SGI52758, SGI52759, SGI52760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52761, SGI0, SGI0, SGI0, SGI52762, SGI52762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11896[] = {SGI52763, SGI52764, SGI52765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52766, SGI52766, SGI0, SGI0, SGI0, SGI52767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11897[] = {SGI52768, SGI0, SGI52769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52770, SGI0, SGI0, SGI0, SGI52771, SGI52771, SGI0, SGI52772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11898[] = {SGI52773, SGI0, SGI52774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52775, SGI0, SGI0, SGI0, SGI52776, SGI52776, SGI0, SGI0, SGI52777, SGI0, NULL};
const Segment * const SG11899[] = {SGI52778, SGI52779, SGI52780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52781, SGI0, SGI0, SGI0, SGI52782, SGI52782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11900[] = {SGI0, SGI0, SGI52783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11901[] = {SGI52784, SGI52785, SGI52786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52787, SGI52787, SGI0, SGI0, SGI0, SGI52788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11902[] = {SGI52789, SGI52790, SGI52791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52792, SGI0, SGI0, SGI0, SGI52793, SGI52794, SGI0, SGI0, SGI0, SGI52795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11903[] = {SGI52796, SGI52797, SGI52798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52799, SGI0, SGI0, SGI0, SGI52800, SGI52800, SGI0, SGI0, SGI0, SGI0, SGI52801, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11904[] = {SGI52802, SGI52803, SGI52804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52805, SGI0, SGI0, SGI0, SGI52806, SGI52806, SGI0, SGI0, SGI0, SGI0, SGI52807, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11905[] = {SGI52808, SGI52809, SGI52810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52811, SGI52811, SGI0, SGI0, SGI52812, SGI0, NULL};
const Segment * const SG11906[] = {SGI52813, SGI52814, SGI52815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52816, SGI0, SGI0, SGI0, SGI52817, SGI52817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11907[] = {SGI52818, SGI0, SGI52819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52820, SGI0, SGI0, SGI0, SGI52821, SGI52821, SGI0, SGI52822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11908[] = {SGI52823, SGI52824, SGI52825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52826, SGI0, SGI0, SGI0, SGI52827, SGI52827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11909[] = {SGI52828, SGI0, SGI52829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52830, SGI0, SGI0, SGI0, SGI52831, SGI52831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11910[] = {SGI52832, SGI52833, SGI52834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52835, SGI0, SGI0, SGI0, SGI52836, SGI52836, SGI0, SGI0, NULL};
const Segment * const SG11911[] = {SGI52837, SGI52838, SGI52839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52840, SGI0, SGI0, SGI0, SGI52841, SGI52841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11912[] = {SGI52843, SGI52844, SGI52845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52846, SGI0, SGI0, SGI0, SGI52847, SGI52847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11913[] = {SGI52849, SGI52850, SGI52851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52852, SGI0, SGI0, SGI0, SGI52853, SGI52853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11914[] = {SGI52854, SGI52855, SGI52856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52857, SGI52857, SGI0, SGI0, SGI0, SGI52858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11915[] = {SGI52859, SGI52860, SGI52861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52862, SGI0, SGI0, SGI0, SGI52863, SGI52863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11916[] = {SGI52864, SGI52865, SGI52866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52867, SGI0, SGI0, SGI0, SGI52868, SGI52868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11917[] = {SGI52869, SGI52870, SGI52871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52872, SGI0, SGI0, SGI0, SGI52873, SGI52873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11918[] = {SGI52874, SGI0, SGI52875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52876, SGI0, SGI0, SGI0, SGI52877, SGI52877, SGI0, SGI0, SGI0, SGI52878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11919[] = {SGI52879, SGI52880, SGI52881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52882, SGI0, SGI0, SGI0, SGI52883, SGI52883, SGI0, SGI0, SGI52884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11920[] = {SGI52885, SGI0, SGI52886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52887, SGI0, SGI0, SGI0, SGI52888, SGI52888, SGI0, SGI0, SGI0, SGI52889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11921[] = {SGI52890, SGI0, SGI52891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52892, SGI0, SGI0, SGI0, SGI52893, SGI52893, SGI0, SGI0, SGI52894, SGI0, NULL};
const Segment * const SG11922[] = {SGI52895, SGI52896, SGI52897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52898, SGI0, SGI0, SGI0, SGI52899, SGI52899, SGI0, SGI52900, SGI0, SGI0, NULL};
const Segment * const SG11923[] = {SGI52901, SGI0, SGI52902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52903, SGI0, SGI0, SGI0, SGI52904, SGI52904, SGI0, SGI0, SGI0, SGI52905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11924[] = {SGI52906, SGI0, SGI52907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52908, SGI0, SGI0, SGI0, SGI52909, SGI52909, SGI0, SGI0, SGI0, SGI52910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11925[] = {SGI52911, SGI0, SGI52912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52913, SGI52914, SGI52914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52915, NULL};
const Segment * const SG11926[] = {SGI52916, SGI52917, SGI52918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52919, SGI0, SGI0, SGI0, SGI52920, SGI52920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11927[] = {SGI52921, SGI0, SGI52922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52923, SGI0, SGI0, SGI0, SGI52924, SGI52924, SGI0, SGI0, SGI0, SGI52925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11928[] = {SGI52926, SGI0, SGI52927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52928, SGI0, SGI0, SGI0, SGI52929, SGI52929, SGI0, SGI52930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11929[] = {SGI52931, SGI52932, SGI52933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52934, SGI52934, SGI0, SGI0, SGI52935, SGI0, NULL};
const Segment * const SG11930[] = {SGI52936, SGI52937, SGI52938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52939, SGI0, SGI0, SGI0, SGI52940, SGI52940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11931[] = {SGI52941, SGI0, SGI52942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52943, SGI0, SGI0, SGI0, SGI52944, SGI52944, SGI0, SGI0, SGI0, SGI52945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11932[] = {SGI52946, SGI0, SGI52947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52948, SGI0, SGI0, SGI0, SGI52949, SGI52949, SGI0, SGI0, SGI0, SGI52950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11933[] = {SGI52951, SGI52952, SGI52953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52954, SGI52954, SGI0, SGI0, SGI0, SGI52955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11934[] = {SGI52956, SGI52957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52958, SGI0, SGI0, SGI0, SGI52959, SGI52959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11935[] = {SGI52960, SGI52961, SGI52962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52963, SGI0, SGI0, SGI0, SGI52964, SGI52964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11936[] = {SGI52965, SGI52966, SGI52967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52968, SGI0, SGI0, SGI0, SGI0, SGI52969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11937[] = {SGI52970, SGI52971, SGI52972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52973, SGI52973, SGI0, SGI0, SGI0, SGI52974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11938[] = {SGI52975, SGI52976, SGI52977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52978, SGI0, SGI0, SGI0, SGI52979, SGI52979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11939[] = {SGI52980, SGI0, SGI52981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52982, SGI0, SGI0, SGI0, SGI52983, SGI52983, SGI0, SGI0, SGI0, SGI52984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11940[] = {SGI0, SGI52985, SGI52986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52987, SGI0, SGI0, SGI0, SGI0, SGI52988, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11941[] = {SGI0, SGI52989, SGI52990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52991, SGI0, SGI0, SGI0, SGI0, SGI52992, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11942[] = {SGI52993, SGI0, SGI52994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI52995, SGI0, SGI0, SGI0, SGI52996, SGI52996, SGI0, SGI52997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11943[] = {SGI52998, SGI52999, SGI53000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53001, SGI53001, SGI0, SGI0, SGI0, SGI53002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11944[] = {SGI53003, SGI53004, SGI53005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53006, SGI0, SGI0, SGI0, SGI53007, SGI53007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11945[] = {SGI53008, SGI53009, SGI53010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53011, SGI0, SGI0, SGI0, SGI53012, SGI53012, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11946[] = {SGI53013, SGI53014, SGI53015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53016, SGI0, SGI0, SGI0, SGI53017, SGI53017, SGI0, SGI53018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11947[] = {SGI53019, SGI53020, SGI53021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53022, SGI0, SGI0, SGI0, SGI53023, SGI53023, SGI0, SGI0, NULL};
const Segment * const SG11948[] = {SGI53024, SGI53025, SGI53026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53027, SGI0, SGI0, SGI0, SGI53028, SGI53028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11949[] = {SGI53029, SGI53030, SGI53031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53032, SGI0, SGI0, SGI0, SGI53033, SGI53033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11950[] = {SGI53034, SGI53035, SGI53036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53037, SGI53037, SGI0, SGI0, SGI53038, SGI0, NULL};
const Segment * const SG11951[] = {SGI53039, SGI53040, SGI53041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53042, SGI53042, SGI0, SGI0, SGI0, SGI53043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11952[] = {SGI53044, SGI53045, SGI53046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53047, SGI53047, SGI0, SGI0, SGI0, SGI53048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11953[] = {SGI0, SGI53049, SGI0, SGI53050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53051, SGI0, SGI0, SGI0, SGI53052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11954[] = {SGI0, SGI53053, SGI0, SGI53054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11955[] = {SGI53055, SGI53056, SGI53057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53058, SGI0, SGI0, SGI0, SGI53059, SGI53059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11956[] = {SGI53060, SGI0, SGI53061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53062, SGI0, SGI0, SGI0, SGI53063, SGI53063, SGI0, SGI0, SGI0, SGI53064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11957[] = {SGI53065, SGI53066, SGI53067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53068, SGI0, SGI0, SGI0, SGI53069, SGI53069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11958[] = {SGI0, SGI53070, SGI0, SGI53071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11959[] = {SGI53072, SGI53073, SGI53074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53075, SGI0, SGI0, SGI0, SGI53076, SGI53076, SGI0, SGI0, SGI0, SGI0, SGI53077, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11960[] = {SGI53078, SGI53079, SGI53080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53081, SGI0, SGI0, SGI0, SGI53082, SGI53082, SGI0, SGI0, SGI0, SGI0, SGI53083, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11961[] = {SGI53084, SGI53085, SGI53086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53087, SGI0, SGI0, SGI0, SGI53088, SGI53088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11962[] = {SGI53089, SGI53090, SGI53091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53092, SGI0, SGI0, SGI0, SGI53093, SGI53093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11963[] = {SGI53094, SGI0, SGI53095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11964[] = {SGI53096, SGI53097, SGI53098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53099, SGI53100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11965[] = {SGI53101, SGI53102, SGI53103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53104, SGI53104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11966[] = {SGI53105, SGI53106, SGI53107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53108, SGI0, SGI0, SGI0, SGI53109, SGI53109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11967[] = {SGI53110, SGI53111, SGI53112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53113, SGI0, SGI0, SGI0, SGI53114, SGI53114, SGI0, SGI53115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11968[] = {SGI53116, SGI53117, SGI53118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53119, SGI53119, SGI0, SGI0, SGI0, SGI53120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11969[] = {SGI0, SGI53121, SGI0, SGI53122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53123, SGI0, SGI0, SGI0, SGI53124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11970[] = {SGI0, SGI53125, SGI0, SGI53126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11971[] = {SGI53127, SGI53128, SGI53129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53130, SGI0, SGI0, SGI0, SGI53131, SGI53131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11972[] = {SGI53132, SGI0, SGI53133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53134, SGI0, SGI0, SGI0, SGI53135, SGI53135, SGI0, SGI0, SGI53136, SGI0, NULL};
const Segment * const SG11973[] = {SGI53137, SGI0, SGI53138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53139, SGI0, SGI0, SGI0, SGI53140, SGI53140, SGI0, SGI53141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11974[] = {SGI53142, SGI53143, SGI53144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53145, SGI0, SGI0, SGI0, SGI53146, SGI53146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11975[] = {SGI53147, SGI53148, SGI53149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53150, SGI53150, SGI0, SGI0, SGI53151, SGI0, NULL};
const Segment * const SG11976[] = {SGI53152, SGI0, SGI53153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53154, SGI0, SGI0, SGI0, SGI53155, SGI53155, SGI0, SGI53156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11977[] = {SGI53157, SGI53158, SGI53159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53160, SGI53160, SGI0, SGI0, SGI53161, SGI0, NULL};
const Segment * const SG11978[] = {SGI53162, SGI53163, SGI53164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53165, SGI0, SGI0, SGI0, SGI53166, SGI53166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11979[] = {SGI53167, SGI53168, SGI53169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53170, SGI53170, SGI0, SGI0, SGI0, SGI53171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11980[] = {SGI53172, SGI53173, SGI53174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53175, SGI0, SGI0, SGI0, SGI53176, SGI53176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11981[] = {SGI53177, SGI53178, SGI53179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53180, SGI0, SGI0, SGI0, SGI53181, SGI53181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11982[] = {SGI53182, SGI0, SGI53183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53184, SGI0, SGI0, SGI0, SGI53185, SGI53185, SGI0, SGI0, SGI0, SGI53186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11983[] = {SGI53187, SGI53188, SGI53189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53190, SGI0, SGI0, SGI0, SGI53191, SGI53191, SGI0, SGI0, SGI0, SGI0, SGI53192, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11984[] = {SGI53193, SGI53194, SGI53195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53196, SGI0, SGI0, SGI0, SGI53197, SGI53197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53198, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11985[] = {SGI53199, SGI53200, SGI53201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53202, SGI0, SGI0, SGI0, SGI53203, SGI53203, SGI0, SGI0, NULL};
const Segment * const SG11986[] = {SGI53204, SGI53205, SGI53206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53207, SGI53207, SGI0, SGI0, SGI0, SGI53208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11987[] = {SGI53209, SGI53210, SGI53211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53212, SGI0, SGI0, SGI0, SGI53213, SGI53213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11988[] = {SGI0, SGI53214, SGI0, SGI53215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11989[] = {SGI53216, SGI53217, SGI53218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53219, SGI0, SGI0, SGI0, SGI53220, SGI53220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11990[] = {SGI53221, SGI0, SGI53222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53223, SGI0, SGI0, SGI0, SGI53225, SGI53225, SGI0, SGI0, SGI0, SGI53227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11991[] = {SGI53224, SGI0, SGI53226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53228, SGI0, SGI0, SGI0, SGI53229, SGI53229, SGI0, SGI0, SGI53230, SGI0, NULL};
const Segment * const SG11992[] = {SGI53231, SGI53232, SGI53233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53234, SGI0, SGI0, SGI0, SGI53235, SGI53235, SGI0, SGI0, NULL};
const Segment * const SG11993[] = {SGI53236, SGI0, SGI53237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53238, SGI0, SGI0, SGI0, SGI53239, SGI53239, SGI0, SGI0, NULL};
const Segment * const SG11994[] = {SGI0, SGI53240, SGI0, SGI53241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53242, SGI0, SGI0, SGI0, SGI53243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11995[] = {SGI0, SGI53244, SGI0, SGI53245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11996[] = {SGI53246, SGI53247, SGI53248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53249, SGI0, SGI0, SGI0, SGI53250, SGI53250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11997[] = {SGI53251, SGI53252, SGI53253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53254, SGI53254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11998[] = {SGI53255, SGI0, SGI53256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53257, SGI0, SGI0, SGI0, SGI53258, SGI53258, SGI0, SGI0, SGI0, SGI53259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG11999[] = {SGI53260, SGI53261, SGI53262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53263, SGI0, SGI0, SGI0, SGI53264, SGI53264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12000[] = {SGI53265, SGI53266, SGI53267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53268, SGI53268, SGI0, SGI0, SGI0, SGI53270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12001[] = {SGI53269, SGI53271, SGI53272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53273, SGI53273, SGI0, SGI53274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12002[] = {SGI0, SGI53275, SGI53276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53277, SGI0, SGI0, SGI0, SGI53278, SGI53278, SGI0, SGI0, NULL};
const Segment * const SG12003[] = {SGI53279, SGI53280, SGI53281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53282, SGI0, SGI0, SGI0, SGI53283, SGI53283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12004[] = {SGI53284, SGI53285, SGI53286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53287, SGI0, SGI0, SGI0, SGI53288, SGI53288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12005[] = {SGI53290, SGI53291, SGI53292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53293, SGI0, SGI0, SGI0, SGI53294, SGI53294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12006[] = {SGI53295, SGI53296, SGI53297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53298, SGI53298, SGI0, SGI0, SGI0, SGI53299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12007[] = {SGI53300, SGI53301, SGI53302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53303, SGI53303, SGI0, SGI0, SGI0, SGI53304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12008[] = {SGI53305, SGI53306, SGI53307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53308, SGI0, SGI0, SGI0, SGI53309, SGI53309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12009[] = {SGI0, SGI53310, SGI0, SGI53311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12010[] = {SGI53312, SGI53313, SGI53314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53315, SGI0, SGI0, SGI0, SGI53316, SGI53316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53317, SGI0, SGI0, NULL};
const Segment * const SG12011[] = {SGI53318, SGI53319, SGI53320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53321, SGI53321, SGI0, SGI0, SGI0, SGI53322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12012[] = {SGI53323, SGI0, SGI53324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53325, SGI0, SGI0, SGI0, SGI53326, SGI53326, SGI0, SGI0, SGI0, SGI53327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12013[] = {SGI53328, SGI53329, SGI53330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53331, SGI0, SGI0, SGI0, SGI53332, SGI53332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12014[] = {SGI53333, SGI53334, SGI53335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53336, SGI0, SGI0, SGI0, SGI53337, SGI53337, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12015[] = {SGI53338, SGI53339, SGI53340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53341, SGI0, SGI0, SGI0, SGI53342, SGI53342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12016[] = {SGI53343, SGI53344, SGI53345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53346, SGI53346, SGI0, SGI0, SGI0, SGI53347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12017[] = {SGI53348, SGI53349, SGI53350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53351, SGI0, SGI0, SGI0, SGI53352, SGI53352, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12018[] = {SGI0, SGI53353, SGI53354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53355, SGI0, SGI0, SGI0, SGI0, SGI53356, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12019[] = {SGI53357, SGI53358, SGI53359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53360, SGI0, SGI0, SGI0, SGI53361, SGI53361, SGI0, SGI53362, SGI0, SGI0, NULL};
const Segment * const SG12020[] = {SGI53363, SGI53364, SGI53365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53366, SGI0, SGI0, SGI0, SGI53367, SGI53367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12021[] = {SGI53368, SGI53369, SGI53370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53371, SGI0, SGI0, SGI0, SGI53372, SGI53372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12022[] = {SGI53373, SGI53374, SGI53375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53376, SGI0, SGI0, SGI0, SGI53377, SGI53377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12023[] = {SGI53378, SGI53379, SGI53380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53381, SGI53382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12024[] = {SGI53383, SGI53384, SGI53385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53386, SGI0, SGI0, SGI0, SGI53387, SGI53387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12025[] = {SGI53388, SGI0, SGI53389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53390, SGI0, SGI0, SGI0, SGI53391, SGI53391, SGI0, SGI0, SGI0, SGI53392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12026[] = {SGI53393, SGI53394, SGI53395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53396, SGI0, SGI0, SGI0, SGI53397, SGI53398, SGI0, SGI0, SGI0, SGI53399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12027[] = {SGI53400, SGI53401, SGI53402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53403, SGI0, SGI0, SGI0, SGI53404, SGI53404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12028[] = {SGI53405, SGI53406, SGI53407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53408, SGI0, SGI0, SGI0, SGI53409, SGI53409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12029[] = {SGI53410, SGI53411, SGI53412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53413, SGI0, SGI0, SGI0, SGI53414, SGI53414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12030[] = {SGI53415, SGI0, SGI53416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53417, SGI53418, SGI53418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53419, NULL};
const Segment * const SG12031[] = {SGI0, SGI53420, SGI0, SGI53421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12032[] = {SGI53422, SGI53423, SGI53424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53425, SGI53425, SGI0, SGI53426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12033[] = {SGI53427, SGI53428, SGI53429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53430, SGI0, SGI0, SGI0, SGI53431, SGI53431, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12034[] = {SGI53432, SGI53433, SGI53434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53435, SGI0, SGI0, SGI0, SGI53436, SGI53436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12035[] = {SGI0, SGI53437, SGI0, SGI53438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53439, SGI0, SGI0, SGI0, SGI53440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12036[] = {SGI53441, SGI53442, SGI53443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53444, SGI0, SGI0, SGI0, SGI53445, SGI53445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12037[] = {SGI53446, SGI53447, SGI53448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53449, SGI0, SGI0, SGI0, SGI53450, SGI53450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12038[] = {SGI53451, SGI53452, SGI53453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53454, SGI0, SGI0, SGI0, SGI53455, SGI53455, SGI0, SGI0, SGI53456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12039[] = {SGI53457, SGI0, SGI53458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53459, SGI0, SGI0, SGI0, SGI53460, SGI53460, SGI0, SGI0, SGI0, SGI53461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12040[] = {SGI53462, SGI53463, SGI53464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53465, SGI0, SGI0, SGI0, SGI53466, SGI53466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12041[] = {SGI53467, SGI53468, SGI53469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53470, SGI53470, SGI0, SGI0, SGI0, SGI53471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12042[] = {SGI53472, SGI53473, SGI53474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53475, SGI53475, SGI0, SGI0, SGI0, SGI53476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12043[] = {SGI53477, SGI53478, SGI53479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53480, SGI0, SGI0, SGI0, SGI53481, SGI53481, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12044[] = {SGI53482, SGI0, SGI53483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53484, SGI0, SGI0, SGI0, SGI53485, SGI53485, SGI0, SGI0, SGI53486, SGI0, NULL};
const Segment * const SG12045[] = {SGI53487, SGI53488, SGI53489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53490, SGI0, SGI0, SGI0, SGI53491, SGI53491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53492, SGI0, NULL};
const Segment * const SG12046[] = {SGI53493, SGI53494, SGI53495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53496, SGI0, SGI0, SGI0, SGI53497, SGI53497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12047[] = {SGI53498, SGI53499, SGI53500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53501, SGI53502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12048[] = {SGI53503, SGI53504, SGI53505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53506, SGI0, SGI0, SGI0, SGI53507, SGI53507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12049[] = {SGI53508, SGI53509, SGI53510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53511, SGI53511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12050[] = {SGI53512, SGI0, SGI53513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53514, SGI0, SGI0, SGI0, SGI53515, SGI53515, SGI0, SGI0, SGI0, SGI53516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12051[] = {SGI53517, SGI53518, SGI53519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53520, SGI0, SGI0, SGI0, SGI53521, SGI53522, SGI0, SGI0, SGI0, SGI53523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12052[] = {SGI53524, SGI0, SGI53525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53526, SGI0, SGI0, SGI0, SGI53527, SGI53527, SGI0, SGI0, SGI53528, SGI0, NULL};
const Segment * const SG12053[] = {SGI53529, SGI0, SGI53530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53531, SGI0, SGI0, SGI0, SGI53532, SGI53532, SGI0, SGI0, SGI53533, SGI0, NULL};
const Segment * const SG12054[] = {SGI53534, SGI53535, SGI53536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53537, SGI53537, SGI0, SGI53538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12055[] = {SGI53539, SGI0, SGI53540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53541, SGI0, SGI0, SGI0, SGI53542, SGI53542, SGI0, SGI0, SGI0, SGI53543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12056[] = {SGI53544, SGI53545, SGI53546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53547, SGI0, SGI0, SGI0, SGI53548, SGI53548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12057[] = {SGI53549, SGI0, SGI53550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53551, SGI0, SGI0, SGI0, SGI53552, SGI53552, SGI0, SGI0, SGI53553, SGI0, NULL};
const Segment * const SG12058[] = {SGI0, SGI53554, SGI53555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53559, SGI53559, SGI0, SGI53560, SGI0, SGI0, NULL};
const Segment * const SG12059[] = {SGI53556, SGI53557, SGI53558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53561, SGI0, SGI0, SGI0, SGI53562, SGI53562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12060[] = {SGI53563, SGI53564, SGI53565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53566, SGI0, SGI0, SGI0, SGI53567, SGI53567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12061[] = {SGI53568, SGI53569, SGI53570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53571, SGI53571, SGI0, SGI0, SGI0, SGI53572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12062[] = {SGI53573, SGI53574, SGI53575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53576, SGI53576, SGI0, SGI0, SGI0, SGI53577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12063[] = {SGI53578, SGI53579, SGI53580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53581, SGI53581, SGI0, SGI53582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12064[] = {SGI53583, SGI0, SGI53584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53585, SGI0, SGI0, SGI0, SGI53586, SGI53586, SGI0, SGI53587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12065[] = {SGI53588, SGI53589, SGI53590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53591, SGI0, SGI0, SGI0, SGI53592, SGI53592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12066[] = {SGI0, SGI53593, SGI0, SGI53594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12067[] = {SGI53595, SGI0, SGI53596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53597, SGI0, SGI0, SGI0, SGI53598, SGI53598, SGI0, SGI0, SGI0, SGI53599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12068[] = {SGI53600, SGI0, SGI53601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53602, SGI0, SGI0, SGI0, SGI53603, SGI53603, SGI0, SGI0, SGI0, SGI53604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12069[] = {SGI53605, SGI53606, SGI53607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53608, SGI53608, SGI0, SGI53609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12070[] = {SGI53610, SGI53611, SGI53612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53613, SGI0, SGI0, SGI0, SGI53614, SGI53614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12071[] = {SGI53615, SGI53616, SGI53617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53618, SGI0, SGI0, SGI0, SGI53619, SGI53619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12072[] = {SGI53620, SGI53621, SGI53622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53623, SGI0, SGI0, SGI0, SGI53624, SGI53624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12073[] = {SGI53625, SGI53626, SGI53627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53628, SGI0, SGI0, SGI0, SGI53629, SGI53629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53630, SGI0, SGI0, NULL};
const Segment * const SG12074[] = {SGI53631, SGI53632, SGI53633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53634, SGI53634, SGI0, SGI0, SGI0, SGI53635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12075[] = {SGI53636, SGI53637, SGI53638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53639, SGI0, SGI0, SGI0, SGI53640, SGI53640, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12076[] = {SGI53641, SGI53642, SGI53643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53644, SGI0, SGI0, SGI0, SGI53645, SGI53645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12077[] = {SGI53646, SGI0, SGI53647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12078[] = {SGI0, SGI53648, SGI53649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53650, SGI53650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12079[] = {SGI53651, SGI53652, SGI53653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53655, SGI0, SGI0, SGI0, SGI53656, SGI53656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12080[] = {SGI53657, SGI53658, SGI53659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53660, SGI0, SGI0, SGI0, SGI53661, SGI53661, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12081[] = {SGI53662, SGI53663, SGI53664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53665, SGI0, SGI0, SGI0, SGI53666, SGI53666, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12082[] = {SGI53667, SGI0, SGI53668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53669, SGI0, SGI0, SGI0, SGI53670, SGI53670, SGI0, SGI0, SGI53671, SGI0, NULL};
const Segment * const SG12083[] = {SGI0, SGI53672, SGI0, SGI53673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12084[] = {SGI53674, SGI53675, SGI53676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53677, SGI0, SGI0, SGI0, SGI53678, SGI53678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12085[] = {SGI53679, SGI53680, SGI53681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53682, SGI53682, SGI0, SGI0, SGI0, SGI53683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12086[] = {SGI53684, SGI53685, SGI53686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53687, SGI0, SGI0, SGI0, SGI53688, SGI53688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12087[] = {SGI53690, SGI53691, SGI53692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53693, SGI0, SGI0, SGI0, SGI53694, SGI53694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53695, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12088[] = {SGI53696, SGI0, SGI53697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53698, SGI0, SGI0, SGI0, SGI53699, SGI53699, SGI0, SGI0, SGI0, SGI53700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12089[] = {SGI53701, SGI0, SGI53702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53703, SGI0, SGI0, SGI0, SGI53704, SGI53704, SGI0, SGI0, SGI0, SGI53705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12090[] = {SGI53706, SGI53707, SGI53708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53709, SGI0, SGI0, SGI0, SGI53710, SGI53710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12091[] = {SGI53712, SGI53713, SGI53714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53715, SGI0, SGI0, SGI0, SGI53716, SGI53716, SGI0, SGI0, NULL};
const Segment * const SG12092[] = {SGI53717, SGI53718, SGI53719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53720, SGI0, SGI0, SGI0, SGI53721, SGI53721, SGI0, SGI0, SGI0, SGI0, SGI53722, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12093[] = {SGI53723, SGI53724, SGI53725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53726, SGI53726, SGI0, SGI0, SGI0, SGI53727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12094[] = {SGI53728, SGI53729, SGI53730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53731, SGI0, SGI0, SGI0, SGI53732, SGI53732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12095[] = {SGI53733, SGI0, SGI53734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53735, SGI0, SGI0, SGI0, SGI53736, SGI53736, SGI0, SGI0, SGI53737, SGI0, NULL};
const Segment * const SG12096[] = {SGI53738, SGI0, SGI53739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53740, SGI0, SGI0, SGI0, SGI53741, SGI53741, SGI0, SGI0, SGI0, SGI53742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12097[] = {SGI0, SGI0, SGI53743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53744, SGI0, SGI0, SGI0, SGI53745, SGI53745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12098[] = {SGI53746, SGI53747, SGI53748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53749, SGI0, SGI0, SGI0, SGI53750, SGI53750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12099[] = {SGI53751, SGI53752, SGI53753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53754, SGI0, SGI0, SGI0, SGI53755, SGI53755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12100[] = {SGI53756, SGI53757, SGI53758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53759, SGI0, SGI0, SGI0, SGI53760, SGI53760, SGI0, SGI0, NULL};
const Segment * const SG12101[] = {SGI53761, SGI53762, SGI53763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53764, SGI53764, SGI0, SGI0, SGI0, SGI53765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12102[] = {SGI53766, SGI53767, SGI53768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53769, SGI0, SGI0, SGI0, SGI53770, SGI53770, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12103[] = {SGI53771, SGI53772, SGI53773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53774, SGI53774, SGI0, SGI53775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12104[] = {SGI53776, SGI53777, SGI53778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53779, SGI53779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53780, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12105[] = {SGI53781, SGI53782, SGI53783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53784, SGI0, SGI0, SGI0, SGI53785, SGI53785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12106[] = {SGI53786, SGI53787, SGI53788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53789, SGI0, SGI0, SGI0, SGI53790, SGI53790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12107[] = {SGI53791, SGI53792, SGI53793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53794, SGI0, SGI0, SGI0, SGI53795, SGI53795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12108[] = {SGI0, SGI53797, SGI53798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53799, SGI0, SGI0, SGI0, SGI0, SGI53800, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12109[] = {SGI53801, SGI53802, SGI53803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53804, SGI53804, SGI0, SGI0, SGI0, SGI53805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12110[] = {SGI53806, SGI53807, SGI53808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53809, SGI0, SGI0, SGI0, SGI53810, SGI53810, SGI0, SGI53811, SGI0, SGI0, NULL};
const Segment * const SG12111[] = {SGI53812, SGI0, SGI53813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53814, SGI53815, SGI53815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53816, NULL};
const Segment * const SG12112[] = {SGI53817, SGI53818, SGI53819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53820, SGI0, SGI0, SGI0, SGI53821, SGI53821, SGI0, SGI0, SGI53822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12113[] = {SGI53823, SGI53824, SGI53825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53826, SGI0, SGI0, SGI0, SGI53827, SGI53827, SGI53828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12114[] = {SGI0, SGI0, SGI53829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53830, SGI53831, SGI0, SGI0, SGI53832, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12115[] = {SGI53833, SGI53834, SGI53835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53836, SGI0, SGI0, SGI0, SGI53837, SGI53837, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12116[] = {SGI53838, SGI53839, SGI53840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53841, SGI0, SGI0, SGI0, SGI53842, SGI53842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12117[] = {SGI53843, SGI53844, SGI53845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53846, SGI0, SGI0, SGI0, SGI53847, SGI53847, SGI0, SGI0, NULL};
const Segment * const SG12118[] = {SGI53848, SGI53849, SGI53850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53851, SGI0, SGI0, SGI0, SGI53852, SGI53852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12119[] = {SGI53853, SGI53854, SGI53855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53856, SGI53856, SGI0, SGI53857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12120[] = {SGI53858, SGI53859, SGI53860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53861, SGI53861, SGI0, SGI53862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12121[] = {SGI0, SGI53863, SGI0, SGI53864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53865, SGI0, SGI0, SGI0, SGI53866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12122[] = {SGI0, SGI53867, SGI0, SGI53868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12123[] = {SGI53869, SGI53870, SGI53871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53872, SGI0, SGI0, SGI0, SGI53873, SGI53873, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12124[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53874, SGI0, SGI0, SGI53875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12125[] = {SGI53876, SGI0, SGI53877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53878, SGI0, SGI0, SGI0, SGI53879, SGI53879, SGI0, SGI53880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12126[] = {SGI53881, SGI0, SGI53882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53883, SGI0, SGI0, SGI0, SGI53884, SGI53884, SGI0, SGI53885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12127[] = {SGI53886, SGI53887, SGI53888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53889, SGI53889, SGI0, SGI0, SGI0, SGI53890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12128[] = {SGI53891, SGI53892, SGI53893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53894, SGI53894, SGI0, SGI0, SGI0, SGI53895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12129[] = {SGI53896, SGI53897, SGI53898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53899, SGI0, SGI0, SGI0, SGI53900, SGI53900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12130[] = {SGI53901, SGI53902, SGI53903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53904, SGI0, SGI0, SGI0, SGI53905, SGI53905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53906, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12131[] = {SGI53907, SGI0, SGI53908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53909, SGI0, SGI0, SGI0, SGI53910, SGI53910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53911, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12132[] = {SGI53914, SGI53916, SGI53917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53918, SGI0, SGI0, SGI0, SGI53920, SGI53920, SGI0, SGI0, NULL};
const Segment * const SG12133[] = {SGI53912, SGI53913, SGI53915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53919, SGI53919, SGI0, SGI0, SGI0, SGI53921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12134[] = {SGI53922, SGI53923, SGI53924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53925, SGI0, SGI0, SGI0, SGI53926, SGI53926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12135[] = {SGI53927, SGI53928, SGI53929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53930, SGI0, SGI0, SGI0, SGI53931, SGI53931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12136[] = {SGI53932, SGI53933, SGI53934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53935, SGI0, SGI0, SGI0, SGI53936, SGI53936, SGI0, SGI0, SGI53937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12137[] = {SGI53938, SGI53939, SGI53940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53941, SGI0, SGI0, SGI0, SGI53942, SGI53942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12138[] = {SGI53943, SGI53944, SGI53945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53946, SGI0, SGI0, SGI0, SGI53947, SGI53947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12139[] = {SGI53948, SGI53949, SGI53950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53954, SGI53954, SGI0, SGI0, SGI0, SGI53956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12140[] = {SGI53951, SGI53952, SGI53953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53955, SGI0, SGI0, SGI0, SGI53957, SGI53957, SGI0, SGI53958, SGI0, SGI0, NULL};
const Segment * const SG12141[] = {SGI53959, SGI53960, SGI53961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53962, SGI0, SGI0, SGI0, SGI53963, SGI53963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12142[] = {SGI53964, SGI53965, SGI53966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53967, SGI0, SGI0, SGI0, SGI53968, SGI53968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12143[] = {SGI53969, SGI53970, SGI53971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53972, SGI0, SGI0, SGI0, SGI53973, SGI53973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12144[] = {SGI53974, SGI53975, SGI53976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53977, SGI53977, SGI0, SGI0, SGI0, SGI53978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12145[] = {SGI53979, SGI53980, SGI53981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53982, SGI0, SGI0, SGI0, SGI53983, SGI53983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12146[] = {SGI53984, SGI53985, SGI53986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53987, SGI0, SGI0, SGI0, SGI53988, SGI53988, SGI0, SGI0, SGI53989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12147[] = {SGI53990, SGI53991, SGI53992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI53993, SGI53993, SGI0, SGI53994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12148[] = {SGI0, SGI53995, SGI0, SGI53996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12149[] = {SGI53997, SGI53998, SGI53999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54002, SGI0, SGI0, SGI0, SGI54006, SGI54006, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12150[] = {SGI54001, SGI54003, SGI54005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54007, SGI0, SGI0, SGI0, SGI54009, SGI54009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12151[] = {SGI54000, SGI0, SGI54004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54008, SGI0, SGI0, SGI0, SGI54010, SGI54010, SGI0, SGI0, NULL};
const Segment * const SG12152[] = {SGI54011, SGI54012, SGI54013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54014, SGI0, SGI0, SGI0, SGI54015, SGI54015, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12153[] = {SGI54016, SGI0, SGI54017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54018, SGI0, SGI0, SGI0, SGI54019, SGI54019, SGI0, SGI0, SGI0, SGI54020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12154[] = {SGI54021, SGI54022, SGI54023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54024, SGI0, SGI0, SGI0, SGI54025, SGI54025, SGI0, SGI0, SGI54026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12155[] = {SGI54027, SGI54028, SGI54029, SGI54030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54031, SGI0, SGI0, SGI0, SGI54032, SGI54033, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12156[] = {SGI54034, SGI54035, SGI54036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54037, SGI0, SGI0, SGI0, SGI54038, SGI54038, SGI0, SGI0, SGI0, SGI0, SGI54040, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12157[] = {SGI54039, SGI0, SGI54041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54042, SGI0, SGI0, SGI0, SGI54043, SGI54043, SGI0, SGI54044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12158[] = {SGI54045, SGI54046, SGI54047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54048, SGI54048, SGI0, SGI0, SGI0, SGI54049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12159[] = {SGI54050, SGI54051, SGI54052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54053, SGI0, SGI0, SGI0, SGI54054, SGI54054, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12160[] = {SGI54055, SGI0, SGI54056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54057, SGI54058, SGI54058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54059, NULL};
const Segment * const SG12161[] = {SGI54060, SGI54061, SGI54062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54063, SGI54063, SGI0, SGI0, SGI0, SGI54064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12162[] = {SGI54065, SGI0, SGI54066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54067, SGI0, SGI0, SGI0, SGI54068, SGI54068, SGI0, SGI0, SGI54069, SGI0, NULL};
const Segment * const SG12163[] = {SGI54070, SGI0, SGI54071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54072, SGI0, SGI0, SGI0, SGI54073, SGI54073, SGI0, SGI0, SGI54074, SGI0, NULL};
const Segment * const SG12164[] = {SGI54075, SGI0, SGI54076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54077, SGI0, SGI0, SGI0, SGI54078, SGI54078, SGI0, SGI0, SGI54079, SGI0, NULL};
const Segment * const SG12165[] = {SGI54080, SGI54081, SGI54082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54083, SGI0, SGI0, SGI0, SGI54084, SGI54085, SGI0, SGI0, SGI0, SGI54086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12166[] = {SGI54087, SGI54088, SGI54089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54090, SGI0, SGI0, SGI0, SGI54091, SGI54091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12167[] = {SGI54092, SGI54093, SGI54094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54095, SGI0, SGI0, SGI0, SGI54096, SGI54096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12168[] = {SGI54097, SGI54098, SGI54099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54100, SGI0, SGI0, SGI0, SGI54101, SGI54101, SGI0, SGI54102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12169[] = {SGI54103, SGI54104, SGI54105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54106, SGI0, SGI0, SGI0, SGI54107, SGI54107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12170[] = {SGI54108, SGI54109, SGI54110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54111, SGI0, SGI0, SGI0, SGI54112, SGI54112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12171[] = {SGI0, SGI54113, SGI0, SGI54114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12172[] = {SGI54115, SGI54116, SGI54117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54118, SGI54118, SGI0, SGI0, SGI0, SGI54119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12173[] = {SGI0, SGI54120, SGI0, SGI54121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12174[] = {SGI54122, SGI54123, SGI54124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54125, SGI0, SGI0, SGI0, SGI54126, SGI54126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12175[] = {SGI54127, SGI0, SGI54128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54129, SGI0, SGI0, SGI0, SGI54130, SGI54130, SGI0, SGI0, SGI0, SGI54131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12176[] = {SGI54132, SGI54133, SGI54134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54135, SGI54135, SGI0, SGI0, SGI0, SGI54136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12177[] = {SGI54137, SGI54138, SGI54139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54140, SGI54140, SGI0, SGI0, SGI0, SGI54141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12178[] = {SGI54142, SGI54143, SGI54144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54145, SGI0, SGI0, SGI0, SGI54146, SGI54146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12179[] = {SGI54147, SGI54148, SGI54149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54150, SGI54151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12180[] = {SGI54152, SGI0, SGI54153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12181[] = {SGI54154, SGI54155, SGI54156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54157, SGI0, SGI0, SGI0, SGI54158, SGI54158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12182[] = {SGI54159, SGI0, SGI54160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54161, SGI0, SGI0, SGI0, SGI54162, SGI54162, SGI0, SGI0, SGI0, SGI54163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12183[] = {SGI54164, SGI54165, SGI54166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54167, SGI0, SGI0, SGI0, SGI54168, SGI54168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12184[] = {SGI54169, SGI54170, SGI54171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54172, SGI0, SGI0, SGI0, SGI54173, SGI54173, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12185[] = {SGI54174, SGI54175, SGI54176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54177, SGI0, SGI0, SGI0, SGI54178, SGI54178, SGI0, SGI54179, SGI0, SGI0, NULL};
const Segment * const SG12186[] = {SGI54180, SGI54181, SGI54182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54183, SGI0, SGI0, SGI0, SGI54184, SGI54184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12187[] = {SGI54185, SGI54186, SGI54187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54188, SGI0, SGI0, SGI0, SGI54189, SGI54189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12188[] = {SGI54190, SGI54191, SGI54192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54193, SGI0, SGI0, SGI0, SGI54194, SGI54194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12189[] = {SGI54195, SGI0, SGI54196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54197, SGI0, SGI0, SGI0, SGI54198, SGI54198, SGI0, SGI0, SGI0, SGI54199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12190[] = {SGI54200, SGI54201, SGI54202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54203, SGI0, SGI0, SGI0, SGI54204, SGI54204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12191[] = {SGI54205, SGI54206, SGI54207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54208, SGI54208, SGI0, SGI0, SGI0, SGI54209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12192[] = {SGI54210, SGI54211, SGI54212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54215, SGI0, SGI0, SGI0, SGI54217, SGI54217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12193[] = {SGI54213, SGI54214, SGI54216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54219, SGI54219, SGI0, SGI0, SGI0, SGI54220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12194[] = {SGI54221, SGI54222, SGI54223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54224, SGI0, SGI0, SGI0, SGI54225, SGI54225, SGI0, SGI54226, SGI0, SGI0, NULL};
const Segment * const SG12195[] = {SGI54227, SGI54228, SGI54229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54230, SGI0, SGI0, SGI0, SGI54231, SGI54231, SGI0, SGI54232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12196[] = {SGI54233, SGI54234, SGI54235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54236, SGI54236, SGI0, SGI0, SGI0, SGI54237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12197[] = {SGI54238, SGI54239, SGI54240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54241, SGI0, SGI0, SGI0, SGI54242, SGI54242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12198[] = {SGI54243, SGI0, SGI54244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54245, SGI0, SGI0, SGI0, SGI54246, SGI54246, SGI0, SGI0, SGI0, SGI54247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12199[] = {SGI54248, SGI54249, SGI54250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54251, SGI0, SGI0, SGI0, SGI54252, SGI54252, SGI0, SGI54253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12200[] = {SGI54254, SGI54255, SGI54256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54257, SGI54257, SGI0, SGI0, SGI0, SGI54258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12201[] = {SGI54259, SGI54260, SGI54261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54262, SGI0, SGI0, SGI0, SGI54263, SGI54263, SGI0, SGI0, SGI54264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12202[] = {SGI54265, SGI54266, SGI54267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54268, SGI54268, SGI0, SGI0, SGI0, SGI54269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12203[] = {SGI54270, SGI54271, SGI54272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54273, SGI54273, SGI0, SGI0, SGI0, SGI54274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12204[] = {SGI54275, SGI54276, SGI54277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54278, SGI54278, SGI0, SGI0, SGI0, SGI54279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12205[] = {SGI0, SGI54280, SGI54281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54282, SGI0, SGI0, SGI0, SGI54283, SGI54283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12206[] = {SGI0, SGI54284, SGI54285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54286, SGI0, SGI0, SGI0, SGI54287, SGI54287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12207[] = {SGI54288, SGI54289, SGI54290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54291, SGI0, SGI0, SGI0, SGI54292, SGI54292, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12208[] = {SGI54293, SGI0, SGI54294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54295, SGI54296, SGI54296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54299, NULL};
const Segment * const SG12209[] = {SGI54297, SGI0, SGI54298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54300, SGI0, SGI0, SGI0, SGI54301, SGI54301, SGI0, SGI54302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12210[] = {SGI54303, SGI54304, SGI54305, SGI0, SGI0, SGI0, SGI54306, SGI54307, SGI0, SGI54308, SGI0, SGI0, SGI0, SGI54309, SGI54309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12211[] = {SGI54310, SGI54311, SGI54312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54313, SGI0, SGI0, SGI0, SGI54314, SGI54314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54315, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12212[] = {SGI54316, SGI54317, SGI54318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54319, SGI0, SGI0, SGI0, SGI54320, SGI54320, SGI0, SGI0, SGI0, SGI0, SGI54321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12213[] = {SGI54322, SGI54323, SGI54324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54325, SGI0, SGI0, SGI0, SGI54326, SGI54326, SGI0, SGI0, SGI0, SGI0, SGI54327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12214[] = {SGI54328, SGI54329, SGI54330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54331, SGI0, SGI0, SGI0, SGI54332, SGI54332, SGI0, SGI0, SGI0, SGI0, SGI54333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12215[] = {SGI54334, SGI54335, SGI54336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54337, SGI0, SGI0, SGI0, SGI54338, SGI54338, SGI0, SGI0, SGI0, SGI0, SGI54339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12216[] = {SGI54340, SGI54341, SGI54342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54343, SGI0, SGI0, SGI0, SGI54344, SGI54344, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12217[] = {SGI54345, SGI0, SGI54346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54347, SGI0, SGI0, SGI0, SGI54348, SGI54348, SGI0, SGI0, SGI0, SGI54349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12218[] = {SGI54350, SGI0, SGI54351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54352, SGI0, SGI0, SGI0, SGI54353, SGI54353, SGI0, SGI0, SGI0, SGI54354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12219[] = {SGI54355, SGI0, SGI54356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54357, SGI0, SGI0, SGI0, SGI54358, SGI54358, SGI0, SGI0, SGI0, SGI54359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12220[] = {SGI54360, SGI54361, SGI54362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54363, SGI0, SGI0, SGI0, SGI54364, SGI54364, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12221[] = {SGI54365, SGI0, SGI54366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54367, SGI54368, SGI54368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54369, NULL};
const Segment * const SG12222[] = {SGI54370, SGI0, SGI54372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54375, SGI0, SGI0, SGI0, SGI54377, SGI54377, SGI0, SGI0, SGI54379, SGI0, NULL};
const Segment * const SG12223[] = {SGI54371, SGI54373, SGI54374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54376, SGI0, SGI0, SGI0, SGI54378, SGI54378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54380, SGI0, NULL};
const Segment * const SG12224[] = {SGI54381, SGI54382, SGI54383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54384, SGI0, SGI0, SGI0, SGI54385, SGI54385, SGI0, SGI0, SGI54386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12225[] = {SGI54387, SGI54388, SGI54389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54390, SGI54390, SGI0, SGI54391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12226[] = {SGI54392, SGI54393, SGI54394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54395, SGI54395, SGI0, SGI54396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12227[] = {SGI54397, SGI54398, SGI54399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54400, SGI0, SGI0, SGI0, SGI54401, SGI54401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54402, SGI0, SGI0, NULL};
const Segment * const SG12228[] = {SGI54403, SGI54404, SGI54405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54406, SGI54406, SGI0, SGI0, SGI0, SGI54407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12229[] = {SGI54408, SGI54409, SGI54410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54411, SGI0, SGI0, SGI0, SGI54412, SGI54412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12230[] = {SGI54414, SGI54415, SGI54416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54417, SGI0, SGI0, SGI0, SGI54418, SGI54418, SGI0, SGI0, SGI0, SGI0, SGI54419, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12231[] = {SGI54420, SGI54421, SGI54422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54423, SGI0, SGI0, SGI0, SGI54424, SGI54425, SGI0, SGI0, SGI0, SGI54426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12232[] = {SGI54427, SGI0, SGI54428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54429, SGI0, SGI0, SGI0, SGI54430, SGI54430, SGI0, SGI54431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12233[] = {SGI54432, SGI54433, SGI54434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54435, SGI0, SGI0, SGI0, SGI54436, SGI54436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12234[] = {SGI54437, SGI54438, SGI54439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54440, SGI0, SGI0, SGI0, SGI54441, SGI54441, SGI0, SGI54442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12235[] = {SGI54443, SGI54444, SGI54445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54446, SGI0, SGI0, SGI0, SGI54447, SGI54447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12236[] = {SGI54449, SGI54450, SGI54451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54452, SGI0, SGI0, SGI0, SGI54453, SGI54453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12237[] = {SGI54454, SGI54455, SGI54456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54457, SGI0, SGI0, SGI0, SGI54458, SGI54458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54459, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12238[] = {SGI54460, SGI0, SGI54461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54462, SGI54463, SGI54463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54464, NULL};
const Segment * const SG12239[] = {SGI54465, SGI54466, SGI54467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54468, SGI0, SGI0, SGI0, SGI54469, SGI54469, SGI0, SGI54470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12240[] = {SGI54471, SGI0, SGI54472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54473, SGI0, SGI0, SGI0, SGI54474, SGI54474, SGI0, SGI0, SGI0, SGI54475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12241[] = {SGI54476, SGI0, SGI54477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54478, SGI0, SGI0, SGI0, SGI54479, SGI54479, SGI0, SGI0, SGI0, SGI54480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12242[] = {SGI54481, SGI54482, SGI54483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54484, SGI54484, SGI0, SGI54485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12243[] = {SGI54486, SGI54487, SGI54488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54489, SGI0, SGI0, SGI0, SGI54490, SGI54490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12244[] = {SGI54491, SGI54492, SGI54493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54494, SGI0, SGI0, SGI0, SGI54495, SGI54495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54496, SGI0, SGI0, NULL};
const Segment * const SG12245[] = {SGI54497, SGI0, SGI54498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54499, SGI0, SGI0, SGI0, SGI54500, SGI54500, SGI0, SGI0, SGI0, SGI54501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12246[] = {SGI54502, SGI54503, SGI54504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54505, SGI0, SGI0, SGI0, SGI54506, SGI54506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12247[] = {SGI54507, SGI54508, SGI54509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54510, SGI0, SGI0, SGI0, SGI54511, SGI54511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12248[] = {SGI54512, SGI54513, SGI54514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54515, SGI0, SGI0, SGI0, SGI54516, SGI54516, SGI0, SGI0, SGI0, SGI0, SGI54517, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12249[] = {SGI54518, SGI54519, SGI54520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54521, SGI54521, SGI0, SGI54522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12250[] = {SGI54523, SGI54524, SGI54525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54526, SGI0, SGI0, SGI0, SGI54527, SGI54527, SGI0, SGI0, SGI0, SGI0, SGI54528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12251[] = {SGI54529, SGI54530, SGI54531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54532, SGI54532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12252[] = {SGI54533, SGI0, SGI54534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12253[] = {SGI54535, SGI54536, SGI54537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54538, SGI54538, SGI0, SGI0, SGI0, SGI54539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12254[] = {SGI54540, SGI54541, SGI54542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54543, SGI54544, SGI0, SGI0, NULL};
const Segment * const SG12255[] = {SGI54545, SGI54546, SGI54547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54548, SGI0, SGI0, SGI0, SGI54549, SGI54549, SGI0, SGI0, NULL};
const Segment * const SG12256[] = {SGI54550, SGI54551, SGI54552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54553, SGI0, SGI0, SGI0, SGI54554, SGI54554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12257[] = {SGI0, SGI0, SGI54555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54556, SGI54557, SGI0, SGI0, SGI54558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12258[] = {SGI54559, SGI54560, SGI54561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54562, SGI0, SGI0, SGI0, SGI54563, SGI54563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12259[] = {SGI54564, SGI54565, SGI54566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54567, SGI0, SGI0, SGI0, SGI54568, SGI54568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12260[] = {SGI54569, SGI54570, SGI54571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54572, SGI54572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12261[] = {SGI54573, SGI54574, SGI54575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54576, SGI0, SGI0, SGI0, SGI54577, SGI54577, SGI0, SGI0, SGI0, SGI0, SGI54578, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12262[] = {SGI54579, SGI54580, SGI54581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54582, SGI54582, SGI0, SGI0, SGI0, SGI54583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12263[] = {SGI54584, SGI54585, SGI54586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54587, SGI0, SGI0, SGI0, SGI54588, SGI54588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12264[] = {SGI54589, SGI54590, SGI54591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54592, SGI54592, SGI0, SGI0, SGI0, SGI54593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12265[] = {SGI54594, SGI54595, SGI54596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54597, SGI54597, SGI0, SGI0, SGI0, SGI54598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12266[] = {SGI54599, SGI54600, SGI54601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54602, SGI54602, SGI0, SGI0, SGI0, SGI54603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12267[] = {SGI54604, SGI54605, SGI54606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54607, SGI54607, SGI0, SGI0, SGI0, SGI54608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12268[] = {SGI54609, SGI54610, SGI54611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54612, SGI54612, SGI0, SGI0, SGI0, SGI54613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12269[] = {SGI54614, SGI54615, SGI54616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54617, SGI0, SGI0, SGI0, SGI54618, SGI54618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12270[] = {SGI54619, SGI54620, SGI54621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54622, SGI0, SGI0, SGI0, SGI54623, SGI54623, SGI0, SGI0, NULL};
const Segment * const SG12271[] = {SGI54624, SGI54625, SGI54626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54627, SGI0, SGI0, SGI0, SGI54628, SGI54628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12272[] = {SGI54629, SGI54630, SGI54631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54632, SGI0, SGI0, SGI0, SGI54633, SGI54633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12273[] = {SGI54634, SGI0, SGI54635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54636, SGI54637, SGI54637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54638, NULL};
const Segment * const SG12274[] = {SGI54639, SGI54640, SGI54641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54642, SGI0, SGI0, SGI0, SGI54643, SGI54643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54644, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12275[] = {SGI54645, SGI0, SGI54646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54647, SGI54648, SGI54648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54649, NULL};
const Segment * const SG12276[] = {SGI54650, SGI54651, SGI54652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54653, SGI54653, SGI0, SGI0, SGI0, SGI54654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12277[] = {SGI54655, SGI0, SGI54656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54657, SGI0, SGI0, SGI0, SGI54658, SGI54658, SGI0, SGI54659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12278[] = {SGI54660, SGI0, SGI54661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54662, SGI0, SGI0, SGI0, SGI54663, SGI54663, SGI0, SGI54664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12279[] = {SGI54665, SGI0, SGI54666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54667, SGI0, SGI0, SGI0, SGI54668, SGI54668, SGI0, SGI54669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12280[] = {SGI54670, SGI54671, SGI54672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54673, SGI54673, SGI0, SGI0, SGI0, SGI54674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12281[] = {SGI54675, SGI54676, SGI54677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54678, SGI0, SGI0, SGI0, SGI54679, SGI54679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12282[] = {SGI54680, SGI0, SGI54681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54682, SGI0, SGI0, SGI0, SGI54683, SGI54683, SGI0, SGI0, SGI0, SGI54684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12283[] = {SGI54685, SGI0, SGI54686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54687, SGI0, SGI0, SGI0, SGI54688, SGI54688, SGI0, SGI0, SGI0, SGI54689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12284[] = {SGI54690, SGI54691, SGI54692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54693, SGI0, SGI0, SGI0, SGI54694, SGI54695, SGI0, SGI0, SGI0, SGI54696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12285[] = {SGI54697, SGI54698, SGI54699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54700, SGI0, SGI0, SGI0, SGI54701, SGI54701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12286[] = {SGI54702, SGI54703, SGI54704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54705, SGI0, SGI0, SGI0, SGI54706, SGI54706, SGI0, SGI0, SGI0, SGI54707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12287[] = {SGI54708, SGI54709, SGI54710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54711, SGI54711, SGI0, SGI0, SGI0, SGI54712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12288[] = {SGI54713, SGI54714, SGI54715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54716, SGI54716, SGI0, SGI0, SGI0, SGI54717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12289[] = {SGI54718, SGI54719, SGI54720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54721, SGI0, SGI0, SGI0, SGI54722, SGI54723, SGI0, SGI0, SGI0, SGI54724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12290[] = {SGI54725, SGI54726, SGI54727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54728, SGI0, SGI0, SGI0, SGI54729, SGI54729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12291[] = {SGI54731, SGI54732, SGI54733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54734, SGI0, SGI0, SGI0, SGI54735, SGI54735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54736, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12292[] = {SGI54737, SGI54738, SGI54739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54740, SGI0, SGI0, SGI0, SGI54741, SGI54741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12293[] = {SGI54742, SGI54743, SGI54744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54745, SGI0, SGI0, SGI0, SGI54746, SGI54746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12294[] = {SGI54747, SGI54748, SGI54749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54750, SGI0, SGI0, SGI0, SGI54751, SGI54751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12295[] = {SGI54753, SGI54754, SGI54755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54756, SGI0, SGI0, SGI0, SGI54757, SGI54757, SGI0, SGI54758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12296[] = {SGI54759, SGI54760, SGI54761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54762, SGI0, SGI0, SGI0, SGI54763, SGI54763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12297[] = {SGI54764, SGI0, SGI54765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54766, SGI0, SGI0, SGI0, SGI54767, SGI54767, SGI0, SGI0, SGI0, SGI54768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12298[] = {SGI54769, SGI54770, SGI54771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54772, SGI0, SGI0, SGI0, SGI54773, SGI54773, SGI0, SGI0, SGI0, SGI0, SGI54774, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12299[] = {SGI54775, SGI54776, SGI54777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54778, SGI0, SGI0, SGI0, SGI54779, SGI54779, SGI0, SGI0, SGI0, SGI0, SGI54780, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12300[] = {SGI54781, SGI54782, SGI54783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54784, SGI0, SGI0, SGI0, SGI54785, SGI54785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12301[] = {SGI54786, SGI54787, SGI54788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54789, SGI54789, SGI0, SGI0, SGI0, SGI54790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12302[] = {SGI54791, SGI54792, SGI54793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54794, SGI0, SGI0, SGI0, SGI54795, SGI54795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12303[] = {SGI0, SGI54797, SGI0, SGI54798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12304[] = {SGI54799, SGI54800, SGI54801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54802, SGI0, SGI0, SGI0, SGI54803, SGI54803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54804, SGI0, SGI0, NULL};
const Segment * const SG12305[] = {SGI54805, SGI54806, SGI54807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54808, SGI0, SGI0, SGI0, SGI54809, SGI54809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54810, SGI0, SGI0, NULL};
const Segment * const SG12306[] = {SGI0, SGI54811, SGI0, SGI54813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12307[] = {SGI54812, SGI54814, SGI54815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54816, SGI0, SGI0, SGI0, SGI54817, SGI54817, SGI0, SGI0, NULL};
const Segment * const SG12308[] = {SGI54818, SGI0, SGI54819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54820, SGI0, SGI0, SGI0, SGI54821, SGI54821, SGI0, SGI0, SGI0, SGI54822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12309[] = {SGI0, SGI0, SGI54823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54824, SGI54825, SGI0, SGI0, SGI54826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12310[] = {SGI54827, SGI54828, SGI54829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54830, SGI0, SGI0, SGI0, SGI54831, SGI54831, SGI0, SGI0, SGI0, SGI0, SGI54832, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12311[] = {SGI54833, SGI54834, SGI54835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54836, SGI0, SGI0, SGI0, SGI54837, SGI54837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12312[] = {SGI54838, SGI54839, SGI54840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54841, SGI0, SGI0, SGI0, SGI54842, SGI54842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12313[] = {SGI54843, SGI0, SGI54844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54845, SGI0, SGI0, SGI0, SGI54846, SGI54846, SGI0, SGI0, SGI0, SGI54847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12314[] = {SGI54848, SGI54849, SGI54850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54851, SGI0, SGI0, SGI0, SGI54852, SGI54852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12315[] = {SGI54853, SGI54854, SGI54855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54856, SGI0, SGI0, SGI0, SGI54857, SGI54857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12316[] = {SGI54858, SGI54859, SGI54860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54861, SGI0, SGI0, SGI0, SGI54862, SGI54862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12317[] = {SGI54863, SGI0, SGI54864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12318[] = {SGI54865, SGI54866, SGI54867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54868, SGI0, SGI0, SGI0, SGI54869, SGI54869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12319[] = {SGI0, SGI54870, SGI0, SGI54871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12320[] = {SGI54872, SGI54873, SGI54874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54875, SGI0, SGI0, SGI0, SGI54876, SGI54876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12321[] = {SGI54878, SGI54879, SGI54880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54881, SGI0, SGI0, SGI0, SGI54882, SGI54882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54883, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12322[] = {SGI54884, SGI54885, SGI54886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54887, SGI0, SGI0, SGI0, SGI54888, SGI54888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12323[] = {SGI54889, SGI54890, SGI54891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54892, SGI0, SGI0, SGI0, SGI54893, SGI54893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12324[] = {SGI54895, SGI54896, SGI54897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54898, SGI0, SGI0, SGI0, SGI54899, SGI54899, SGI0, SGI0, SGI0, SGI0, SGI54900, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12325[] = {SGI54901, SGI54902, SGI54903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54904, SGI54904, SGI0, SGI0, SGI0, SGI54905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12326[] = {SGI54906, SGI54907, SGI54908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54909, SGI54909, SGI0, SGI0, SGI0, SGI54910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12327[] = {SGI54911, SGI54912, SGI54913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54914, SGI0, SGI0, SGI0, SGI54915, SGI54916, SGI0, SGI0, SGI0, SGI54917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12328[] = {SGI54918, SGI54919, SGI54920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54921, SGI0, SGI0, SGI0, SGI54922, SGI54922, SGI0, SGI0, SGI0, SGI0, SGI54923, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12329[] = {SGI54924, SGI54925, SGI54926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54927, SGI0, SGI0, SGI0, SGI54928, SGI54928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12330[] = {SGI54929, SGI54930, SGI54931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54932, SGI0, SGI0, SGI0, SGI54933, SGI54933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12331[] = {SGI54934, SGI54935, SGI54936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54937, SGI0, SGI0, SGI0, SGI54938, SGI54938, SGI0, SGI0, NULL};
const Segment * const SG12332[] = {SGI54939, SGI0, SGI54940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54941, SGI0, SGI0, SGI0, SGI54942, SGI54942, SGI0, SGI0, NULL};
const Segment * const SG12333[] = {SGI54943, SGI54944, SGI54945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54946, SGI0, SGI0, SGI0, SGI54947, SGI54947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12334[] = {SGI54948, SGI0, SGI54949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54950, SGI0, SGI0, SGI0, SGI54951, SGI54951, SGI0, SGI0, SGI0, SGI54952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12335[] = {SGI54953, SGI54954, SGI54955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54956, SGI0, SGI0, SGI0, SGI54957, SGI54958, SGI0, SGI0, SGI0, SGI54959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12336[] = {SGI54960, SGI0, SGI54961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54962, SGI0, SGI0, SGI0, SGI54963, SGI54963, SGI0, SGI0, SGI0, SGI54964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12337[] = {SGI54965, SGI54966, SGI54967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54968, SGI54968, SGI0, SGI0, SGI0, SGI54969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12338[] = {SGI54970, SGI54971, SGI54972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54973, SGI54973, SGI0, SGI0, SGI0, SGI54974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12339[] = {SGI54975, SGI0, SGI54976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54977, SGI0, SGI0, SGI0, SGI54978, SGI54978, SGI0, SGI0, SGI0, SGI54979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12340[] = {SGI54980, SGI54981, SGI54982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54983, SGI0, SGI0, SGI0, SGI54984, SGI54984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12341[] = {SGI54985, SGI54986, SGI54987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54988, SGI54988, SGI0, SGI0, SGI0, SGI54989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12342[] = {SGI54990, SGI54991, SGI54992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54993, SGI0, SGI0, SGI0, SGI54994, SGI54994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12343[] = {SGI54995, SGI54996, SGI54997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI54998, SGI0, SGI0, SGI0, SGI54999, SGI54999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12344[] = {SGI55000, SGI55001, SGI55002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55003, SGI0, SGI0, SGI0, SGI55004, SGI55004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12345[] = {SGI55005, SGI0, SGI55006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55007, SGI0, SGI0, SGI0, SGI55008, SGI55008, SGI0, SGI0, SGI0, SGI55009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12346[] = {SGI0, SGI55010, SGI0, SGI55011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12347[] = {SGI55012, SGI55013, SGI55014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55015, SGI55015, SGI0, SGI0, SGI0, SGI55016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12348[] = {SGI55017, SGI55018, SGI55019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55020, SGI55020, SGI0, SGI0, SGI0, SGI55021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12349[] = {SGI55022, SGI55023, SGI55024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55025, SGI0, SGI0, SGI0, SGI55026, SGI55026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55027, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12350[] = {SGI55028, SGI55029, SGI55030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55031, SGI0, SGI0, SGI0, SGI55032, SGI55032, SGI0, SGI0, SGI0, SGI0, SGI55033, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12351[] = {SGI55034, SGI0, SGI55035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55036, SGI0, SGI0, SGI0, SGI55037, SGI55037, SGI0, SGI55038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12352[] = {SGI55039, SGI55040, SGI55041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55042, SGI55042, SGI0, SGI0, SGI0, SGI55043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12353[] = {SGI55044, SGI55045, SGI55046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55047, SGI0, SGI0, SGI0, SGI55048, SGI55048, SGI0, SGI0, SGI0, SGI0, SGI55049, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12354[] = {SGI55050, SGI55051, SGI55052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55053, SGI0, SGI0, SGI0, SGI55054, SGI55054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55055, SGI0, SGI0, NULL};
const Segment * const SG12355[] = {SGI55056, SGI0, SGI55057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55058, SGI55059, SGI55059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55060, NULL};
const Segment * const SG12356[] = {SGI55061, SGI55062, SGI55063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55064, SGI0, SGI0, SGI0, SGI55065, SGI55065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55066, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12357[] = {SGI55067, SGI55068, SGI55069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55070, SGI0, SGI0, SGI0, SGI55071, SGI55071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12358[] = {SGI55072, SGI55073, SGI55074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55075, SGI0, SGI0, SGI0, SGI55076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55077, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12359[] = {SGI55078, SGI55079, SGI55080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55081, SGI0, SGI0, SGI0, SGI55082, SGI55082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12360[] = {SGI55083, SGI55084, SGI55085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55086, SGI0, SGI0, SGI0, SGI55087, SGI55087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12361[] = {SGI55088, SGI55089, SGI55090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55091, SGI0, SGI0, SGI0, SGI55092, SGI55092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12362[] = {SGI55094, SGI55096, SGI55098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55099, SGI0, SGI0, SGI0, SGI55101, SGI55101, SGI0, SGI0, NULL};
const Segment * const SG12363[] = {SGI55093, SGI55095, SGI55097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55100, SGI55100, SGI0, SGI0, SGI0, SGI55102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12364[] = {SGI55103, SGI0, SGI55104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55105, SGI0, SGI0, SGI0, SGI55106, SGI55106, SGI0, SGI0, SGI0, SGI55107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12365[] = {SGI55108, SGI55109, SGI55110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55111, SGI0, SGI0, SGI0, SGI55112, SGI55112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12366[] = {SGI55113, SGI55114, SGI55115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55116, SGI0, SGI0, SGI0, SGI55117, SGI55117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12367[] = {SGI55118, SGI55119, SGI55120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55121, SGI0, SGI0, SGI0, SGI55122, SGI55122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12368[] = {SGI55123, SGI55124, SGI55125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55126, SGI0, SGI0, SGI0, SGI55127, SGI55127, SGI0, SGI55128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12369[] = {SGI55129, SGI55130, SGI55131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55132, SGI0, SGI0, SGI0, SGI55133, SGI55134, SGI0, SGI0, SGI0, SGI55135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12370[] = {SGI55136, SGI55137, SGI55138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55139, SGI0, SGI0, SGI0, SGI55140, SGI55140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12371[] = {SGI55141, SGI55142, SGI55143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55144, SGI0, SGI0, SGI0, SGI55145, SGI55146, SGI0, SGI0, SGI0, SGI55147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12372[] = {SGI55148, SGI55149, SGI55150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55151, SGI55151, SGI0, SGI0, SGI0, SGI55152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12373[] = {SGI0, SGI0, SGI55153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55154, SGI55155, SGI0, SGI0, SGI55156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12374[] = {SGI55157, SGI55158, SGI55159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55160, SGI0, SGI0, SGI0, SGI55161, SGI55161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12375[] = {SGI55163, SGI0, SGI55164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55165, SGI0, SGI0, SGI0, SGI55166, SGI55166, SGI0, SGI0, SGI0, SGI55167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12376[] = {SGI55168, SGI55169, SGI55170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55171, SGI0, SGI0, SGI0, SGI55172, SGI55172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12377[] = {SGI55173, SGI55174, SGI55175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55176, SGI55176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12378[] = {SGI55177, SGI0, SGI55178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55179, SGI0, SGI0, SGI0, SGI55180, SGI55180, SGI0, SGI0, SGI0, SGI55181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12379[] = {SGI55182, SGI55183, SGI55184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55185, SGI0, SGI0, SGI0, SGI55186, SGI55186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12380[] = {SGI55187, SGI55188, SGI55189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55190, SGI0, SGI0, SGI0, SGI55191, SGI55191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12381[] = {SGI55192, SGI55193, SGI55194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55195, SGI55195, SGI0, SGI55196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12382[] = {SGI55197, SGI55198, SGI55199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55200, SGI0, SGI0, SGI0, SGI55201, SGI55201, SGI0, SGI55202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12383[] = {SGI55203, SGI0, SGI55204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55205, SGI0, SGI0, SGI0, SGI55206, SGI55206, SGI0, SGI0, SGI0, SGI55207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12384[] = {SGI55208, SGI55209, SGI55210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55211, SGI0, SGI0, SGI0, SGI55212, SGI55212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55213, SGI0, SGI0, NULL};
const Segment * const SG12385[] = {SGI55214, SGI55215, SGI55216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55217, SGI0, SGI0, SGI0, SGI55218, SGI55219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12386[] = {SGI0, SGI0, SGI55220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55221, SGI55222, SGI0, SGI0, SGI55223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12387[] = {SGI55227, SGI55228, SGI55229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55230, SGI0, SGI0, SGI0, SGI55231, SGI55231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12388[] = {SGI55232, SGI55233, SGI55234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55235, SGI0, SGI0, SGI0, SGI55236, SGI55236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12389[] = {SGI55237, SGI55238, SGI55239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55240, SGI0, SGI0, SGI0, SGI55241, SGI55241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55242, SGI0, NULL};
const Segment * const SG12390[] = {SGI55224, SGI55225, SGI55226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55243, SGI0, SGI0, SGI0, SGI55244, SGI55244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55245, SGI0, NULL};
const Segment * const SG12391[] = {SGI55246, SGI55247, SGI55248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55249, SGI0, SGI0, SGI0, SGI55250, SGI55250, SGI0, SGI0, SGI55251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12392[] = {SGI55252, SGI0, SGI55253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55254, SGI0, SGI0, SGI0, SGI55255, SGI55255, SGI0, SGI0, SGI0, SGI55256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12393[] = {SGI55257, SGI55258, SGI55259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55260, SGI0, SGI0, SGI0, SGI55261, SGI55261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12394[] = {SGI0, SGI55262, SGI55263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55264, SGI0, SGI0, SGI0, SGI0, SGI55265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12395[] = {SGI55266, SGI55267, SGI55268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55269, SGI0, SGI0, SGI0, SGI55270, SGI55270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12396[] = {SGI55271, SGI55272, SGI55273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55274, SGI0, SGI0, SGI0, SGI55275, SGI55275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12397[] = {SGI55276, SGI55277, SGI55278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55279, SGI0, SGI0, SGI0, SGI55280, SGI55280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12398[] = {SGI55281, SGI55282, SGI55283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55284, SGI0, SGI0, SGI0, SGI55285, SGI55285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55286, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12399[] = {SGI55287, SGI0, SGI55288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55289, SGI0, SGI0, SGI0, SGI55290, SGI55290, SGI0, SGI0, SGI0, SGI55291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12400[] = {SGI55292, SGI55293, SGI55294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55295, SGI0, SGI0, SGI0, SGI55296, SGI55296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12401[] = {SGI55297, SGI55298, SGI55299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55300, SGI55300, SGI0, SGI0, SGI0, SGI55301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12402[] = {SGI55302, SGI0, SGI55303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55304, SGI0, SGI0, SGI0, SGI55305, SGI55305, SGI0, SGI0, SGI0, SGI55306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12403[] = {SGI55307, SGI55308, SGI55309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55310, SGI0, SGI0, SGI0, SGI55311, SGI55312, SGI0, SGI0, SGI0, SGI55313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12404[] = {SGI55314, SGI55315, SGI55316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55317, SGI0, SGI0, SGI0, SGI55318, SGI55318, SGI0, SGI0, SGI0, SGI0, SGI55319, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12405[] = {SGI55320, SGI0, SGI55321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55322, SGI55323, SGI55323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55324, NULL};
const Segment * const SG12406[] = {SGI55325, SGI55326, SGI55327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55328, SGI0, SGI0, SGI0, SGI55329, SGI55329, SGI0, SGI55330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12407[] = {SGI55331, SGI55332, SGI55333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55334, SGI55334, SGI0, SGI55335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12408[] = {SGI55336, SGI55337, SGI55338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55339, SGI0, SGI0, SGI0, SGI55340, SGI55340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12409[] = {SGI0, SGI55341, SGI0, SGI55342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12410[] = {SGI55343, SGI55344, SGI55345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55346, SGI0, SGI0, SGI0, SGI55347, SGI55347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12411[] = {SGI0, SGI0, SGI55348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55349, SGI0, SGI0, SGI0, SGI55350, SGI55350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12412[] = {SGI55351, SGI0, SGI55352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55353, SGI0, SGI0, SGI0, SGI55354, SGI55354, SGI0, SGI55355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12413[] = {SGI55356, SGI55357, SGI55358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55359, SGI0, SGI0, SGI0, SGI55360, SGI55360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12414[] = {SGI55361, SGI0, SGI55362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12415[] = {SGI55363, SGI55364, SGI55365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55366, SGI0, SGI0, SGI0, SGI55367, SGI55367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12416[] = {SGI0, SGI55368, SGI55369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55370, SGI0, SGI0, SGI0, SGI0, SGI55371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12417[] = {SGI55372, SGI55373, SGI55374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55375, SGI0, SGI0, SGI0, SGI55376, SGI55376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55377, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12418[] = {SGI55378, SGI55379, SGI55380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55381, SGI0, SGI0, SGI0, SGI55382, SGI55382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12419[] = {SGI55384, SGI55385, SGI55386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55387, SGI0, SGI0, SGI0, SGI55388, SGI55388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12420[] = {SGI55390, SGI55391, SGI55392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55393, SGI0, SGI0, SGI0, SGI55394, SGI55394, SGI0, SGI55395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12421[] = {SGI55396, SGI55397, SGI55398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55399, SGI55399, SGI0, SGI0, SGI0, SGI55400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12422[] = {SGI55401, SGI55402, SGI55403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55404, SGI0, SGI0, SGI0, SGI55405, SGI55405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12423[] = {SGI55406, SGI55407, SGI55408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55409, SGI0, SGI0, SGI0, SGI55410, SGI55410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55411, SGI0, SGI0, NULL};
const Segment * const SG12424[] = {SGI55412, SGI0, SGI55413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55414, SGI0, SGI0, SGI0, SGI55415, SGI55415, SGI0, SGI0, SGI0, SGI55418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12425[] = {SGI55416, SGI0, SGI55417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55419, SGI0, SGI0, SGI0, SGI55420, SGI55420, SGI0, SGI0, SGI0, SGI55421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12426[] = {SGI55422, SGI0, SGI55423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55424, SGI0, SGI0, SGI0, SGI55425, SGI55425, SGI0, SGI0, SGI0, SGI55426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12427[] = {SGI55427, SGI0, SGI55428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55429, SGI0, SGI0, SGI0, SGI55430, SGI55430, SGI0, SGI0, SGI0, SGI55431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12428[] = {SGI55432, SGI0, SGI55433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55434, SGI0, SGI0, SGI0, SGI55435, SGI55435, SGI0, SGI0, SGI0, SGI55436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12429[] = {SGI55437, SGI55438, SGI55439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55440, SGI0, SGI0, SGI0, SGI55441, SGI55441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55442, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12430[] = {SGI55443, SGI0, SGI55444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55445, SGI55446, SGI55446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55447, NULL};
const Segment * const SG12431[] = {SGI55448, SGI55449, SGI55450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55451, SGI0, SGI0, SGI0, SGI55452, SGI55452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12432[] = {SGI55453, SGI55454, SGI55455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55456, SGI0, SGI0, SGI0, SGI55457, SGI55457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12433[] = {SGI55458, SGI55459, SGI55460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55461, SGI0, SGI0, SGI0, SGI55462, SGI55462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12434[] = {SGI55463, SGI55464, SGI55465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55466, SGI55466, SGI0, SGI0, SGI0, SGI55467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12435[] = {SGI55468, SGI55469, SGI55470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55471, SGI55471, SGI0, SGI55472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12436[] = {SGI55473, SGI55475, SGI55477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55479, SGI0, SGI0, SGI0, SGI55480, SGI55482, SGI0, SGI0, SGI0, SGI55484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12437[] = {SGI55474, SGI55476, SGI55478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55481, SGI55481, SGI0, SGI0, SGI0, SGI55483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12438[] = {SGI55485, SGI55486, SGI55487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55488, SGI0, SGI0, SGI0, SGI55489, SGI55490, SGI0, SGI0, SGI0, SGI55491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12439[] = {SGI0, SGI0, SGI55492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55493, SGI55494, SGI0, SGI0, SGI55495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12440[] = {SGI55496, SGI55497, SGI55498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55499, SGI0, SGI0, SGI0, SGI55500, SGI55500, SGI0, SGI0, SGI0, SGI0, SGI55501, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12441[] = {SGI55502, SGI55503, SGI55504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55505, SGI0, SGI0, SGI0, SGI55506, SGI55506, SGI0, SGI0, SGI0, SGI0, SGI55507, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12442[] = {SGI0, SGI55508, SGI0, SGI55509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12443[] = {SGI55510, SGI0, SGI55511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55512, SGI0, SGI0, SGI0, SGI55513, SGI55513, SGI0, SGI0, SGI0, SGI55514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12444[] = {SGI55515, SGI55516, SGI55517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55518, SGI0, SGI0, SGI0, SGI55519, SGI55519, SGI0, SGI0, SGI55520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12445[] = {SGI55521, SGI0, SGI55522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55523, SGI0, SGI0, SGI0, SGI55524, SGI55524, SGI0, SGI55525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12446[] = {SGI55526, SGI55527, SGI55528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55529, SGI0, SGI0, SGI0, SGI55530, SGI55531, SGI0, SGI0, SGI0, SGI55532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12447[] = {SGI55533, SGI55534, SGI55535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55536, SGI55536, SGI0, SGI55537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12448[] = {SGI0, SGI0, SGI55538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55539, SGI55540, SGI0, SGI0, SGI55541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12449[] = {SGI55542, SGI55543, SGI55544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55545, SGI0, SGI0, SGI0, SGI55546, SGI55546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12450[] = {SGI55547, SGI55548, SGI55549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55550, SGI0, SGI0, SGI0, SGI55551, SGI55551, SGI0, SGI0, NULL};
const Segment * const SG12451[] = {SGI55552, SGI55553, SGI55554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55555, SGI55555, SGI0, SGI0, SGI0, SGI55556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12452[] = {SGI55557, SGI55558, SGI55559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55560, SGI0, SGI0, SGI0, SGI55561, SGI55561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55562, SGI0, SGI0, NULL};
const Segment * const SG12453[] = {SGI55563, SGI55564, SGI55565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55566, SGI0, SGI0, SGI0, SGI55567, SGI55567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12454[] = {SGI55568, SGI55569, SGI55570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55571, SGI0, SGI0, SGI0, SGI55572, SGI55572, SGI0, SGI0, SGI0, SGI0, SGI55573, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12455[] = {SGI55574, SGI55575, SGI55576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55577, SGI0, SGI0, SGI0, SGI55578, SGI55579, SGI0, SGI0, SGI0, SGI55580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12456[] = {SGI55581, SGI55582, SGI55583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55584, SGI0, SGI0, SGI0, SGI55585, SGI55585, SGI0, SGI0, SGI0, SGI0, SGI55586, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12457[] = {SGI55587, SGI55588, SGI55589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55590, SGI55590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55591, SGI0, SGI0, NULL};
const Segment * const SG12458[] = {SGI55592, SGI0, SGI55593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55594, SGI0, SGI0, SGI0, SGI55595, SGI55595, SGI0, SGI0, SGI0, SGI55596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12459[] = {SGI55597, SGI55598, SGI55599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55600, SGI0, SGI0, SGI0, SGI55601, SGI55601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12460[] = {SGI55603, SGI55604, SGI55605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55606, SGI0, SGI0, SGI0, SGI55607, SGI55607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55608, SGI0, SGI0, NULL};
const Segment * const SG12461[] = {SGI55609, SGI55610, SGI55611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55612, SGI0, SGI0, SGI0, SGI55613, SGI55613, SGI0, SGI0, SGI55614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12462[] = {SGI55615, SGI0, SGI55616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55617, SGI0, SGI0, SGI0, SGI55618, SGI55618, SGI0, SGI0, SGI0, SGI55619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12463[] = {SGI55620, SGI0, SGI55621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55622, SGI0, SGI0, SGI0, SGI55623, SGI55623, SGI0, SGI0, SGI0, SGI55624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12464[] = {SGI55625, SGI0, SGI55626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55627, SGI0, SGI0, SGI0, SGI55628, SGI55628, SGI0, SGI0, SGI0, SGI55629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12465[] = {SGI55630, SGI55631, SGI55632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55633, SGI55633, SGI0, SGI55634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12466[] = {SGI55635, SGI55636, SGI55637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55638, SGI0, SGI0, SGI0, SGI55639, SGI55639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12467[] = {SGI55640, SGI0, SGI55641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55642, SGI0, SGI0, SGI0, SGI55643, SGI55643, SGI0, SGI55644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12468[] = {SGI55645, SGI0, SGI55646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55647, SGI0, SGI0, SGI0, SGI55648, SGI55648, SGI0, SGI0, SGI0, SGI55649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12469[] = {SGI55650, SGI55651, SGI55652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55653, SGI0, SGI0, SGI0, SGI55654, SGI55654, SGI0, SGI0, NULL};
const Segment * const SG12470[] = {SGI55655, SGI55656, SGI55657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55658, SGI0, SGI0, SGI0, SGI55659, SGI55659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12471[] = {SGI55660, SGI55661, SGI55662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55663, SGI0, SGI0, SGI0, SGI55664, SGI55666, SGI0, SGI0, SGI0, SGI55669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12472[] = {SGI55665, SGI55667, SGI55668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55670, SGI0, SGI0, SGI0, SGI55671, SGI55671, SGI0, SGI55672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12473[] = {SGI55673, SGI55674, SGI55675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55676, SGI0, SGI0, SGI0, SGI55677, SGI55677, SGI0, SGI0, SGI55678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12474[] = {SGI55679, SGI55680, SGI55681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55682, SGI55682, SGI0, SGI0, SGI0, SGI55683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12475[] = {SGI55684, SGI55685, SGI55686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55687, SGI55687, SGI0, SGI0, SGI0, SGI55688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12476[] = {SGI55689, SGI55690, SGI55691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55692, SGI0, SGI0, SGI0, SGI55693, SGI55693, SGI0, SGI0, SGI0, SGI0, SGI55694, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12477[] = {SGI55695, SGI55696, SGI55697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55698, SGI0, SGI0, SGI0, SGI55699, SGI55699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12478[] = {SGI0, SGI55700, SGI55701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55702, SGI0, SGI0, SGI0, SGI55703, SGI55703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12479[] = {SGI0, SGI0, SGI55704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12480[] = {SGI0, SGI55705, SGI55706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55707, SGI55708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12481[] = {SGI55709, SGI0, SGI55710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55711, SGI0, SGI0, SGI0, SGI55712, SGI55712, SGI0, SGI0, SGI0, SGI55713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12482[] = {SGI55714, SGI0, SGI55715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55716, SGI0, SGI0, SGI0, SGI55717, SGI55717, SGI0, SGI0, SGI0, SGI55718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12483[] = {SGI55719, SGI55720, SGI55721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55722, SGI0, SGI0, SGI0, SGI55723, SGI55723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12484[] = {SGI55724, SGI55725, SGI55726, SGI0, SGI0, SGI0, SGI55727, SGI55728, SGI0, SGI55729, SGI0, SGI0, SGI0, SGI55730, SGI55730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12485[] = {SGI55731, SGI55732, SGI55733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55734, SGI0, SGI0, SGI0, SGI55735, SGI55735, SGI0, SGI0, SGI0, SGI0, SGI55736, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12486[] = {SGI55737, SGI0, SGI55738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55739, SGI0, SGI0, SGI0, SGI55740, SGI55740, SGI0, SGI0, SGI0, SGI55741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12487[] = {SGI55742, SGI55743, SGI55744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55745, SGI0, SGI0, SGI0, SGI55746, SGI55746, SGI0, SGI0, SGI0, SGI0, SGI55747, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12488[] = {SGI55748, SGI0, SGI55749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55750, SGI0, SGI0, SGI0, SGI55751, SGI55751, SGI0, SGI0, SGI0, SGI55752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12489[] = {SGI55753, SGI55754, SGI55755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55756, SGI0, SGI0, SGI0, SGI55757, SGI55757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55758, SGI0, SGI0, NULL};
const Segment * const SG12490[] = {SGI55759, SGI55760, SGI55761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55762, SGI55762, SGI0, SGI0, SGI0, SGI55763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12491[] = {SGI55764, SGI55765, SGI55766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55767, SGI0, SGI0, SGI0, SGI55768, SGI55768, SGI0, SGI0, NULL};
const Segment * const SG12492[] = {SGI55769, SGI55770, SGI55771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55772, SGI0, SGI0, SGI0, SGI55773, SGI55773, SGI0, SGI0, SGI55774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12493[] = {SGI55775, SGI55776, SGI55777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55778, SGI0, SGI0, SGI0, SGI55779, SGI55779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12494[] = {SGI55780, SGI0, SGI55781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55782, SGI55783, SGI55783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55784, NULL};
const Segment * const SG12495[] = {SGI55785, SGI55786, SGI55787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55788, SGI0, SGI0, SGI0, SGI55789, SGI55790, SGI0, SGI0, SGI0, SGI55791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12496[] = {SGI55792, SGI55793, SGI55794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55795, SGI0, SGI0, SGI0, SGI55796, SGI55797, SGI0, SGI0, SGI0, SGI55798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12497[] = {SGI55799, SGI55800, SGI55801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55802, SGI0, SGI0, SGI0, SGI55803, SGI55804, SGI0, SGI0, SGI0, SGI55805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12498[] = {SGI55806, SGI55807, SGI55808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55809, SGI0, SGI0, SGI0, SGI55810, SGI55810, SGI0, SGI55811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12499[] = {SGI55812, SGI55813, SGI55814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55815, SGI0, SGI0, SGI0, SGI55816, SGI55816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12500[] = {SGI55817, SGI55818, SGI55819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55820, SGI0, SGI0, SGI0, SGI55821, SGI55821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12501[] = {SGI55822, SGI55823, SGI55824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55825, SGI0, SGI0, SGI0, SGI55826, SGI55826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55827, SGI0, NULL};
const Segment * const SG12502[] = {SGI55828, SGI55829, SGI55830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55831, SGI0, SGI0, SGI0, SGI55832, SGI55832, SGI0, SGI0, SGI55833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12503[] = {SGI55834, SGI55835, SGI55836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55837, SGI0, SGI0, SGI0, SGI55838, SGI55838, SGI0, SGI0, SGI0, SGI0, SGI55839, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12504[] = {SGI55840, SGI55841, SGI55842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55843, SGI0, SGI0, SGI0, SGI55844, SGI55844, SGI0, SGI0, SGI0, SGI0, SGI55845, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12505[] = {SGI55846, SGI0, SGI55847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55848, SGI0, SGI0, SGI0, SGI55849, SGI55849, SGI0, SGI0, SGI0, SGI55850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12506[] = {SGI55851, SGI0, SGI55852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55853, SGI0, SGI0, SGI0, SGI55854, SGI55854, SGI0, SGI0, SGI0, SGI55855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12507[] = {SGI55856, SGI55857, SGI55858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55859, SGI0, SGI0, SGI0, SGI55860, SGI55860, SGI0, SGI0, SGI0, SGI0, SGI55861, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12508[] = {SGI55862, SGI55863, SGI55864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55865, SGI0, SGI0, SGI0, SGI55866, SGI55866, SGI0, SGI0, SGI0, SGI0, SGI55867, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12509[] = {SGI55868, SGI55869, SGI55870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55871, SGI0, SGI0, SGI0, SGI55872, SGI55873, SGI0, SGI0, SGI0, SGI55874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12510[] = {SGI55875, SGI55876, SGI55877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55878, SGI55878, SGI0, SGI0, SGI0, SGI55879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12511[] = {SGI55880, SGI55881, SGI55882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55883, SGI0, SGI0, SGI0, SGI55884, SGI55884, SGI55885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12512[] = {SGI55886, SGI55887, SGI55888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55889, SGI0, SGI0, SGI0, SGI55890, SGI55890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12513[] = {SGI55891, SGI55892, SGI55893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55894, SGI0, SGI0, SGI0, SGI55895, SGI55895, SGI0, SGI55896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12514[] = {SGI55897, SGI55898, SGI55899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55900, SGI0, SGI0, SGI0, SGI55901, SGI55901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55902, SGI0, SGI0, NULL};
const Segment * const SG12515[] = {SGI55903, SGI55904, SGI55905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55906, SGI0, SGI0, SGI0, SGI55907, SGI55907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55908, SGI0, SGI0, NULL};
const Segment * const SG12516[] = {SGI55909, SGI0, SGI55910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55911, SGI0, SGI0, SGI0, SGI55912, SGI55912, SGI0, SGI0, SGI0, SGI55913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12517[] = {SGI55914, SGI55915, SGI55916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55917, SGI0, SGI0, SGI0, SGI55918, SGI55918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55919, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12518[] = {SGI55920, SGI55921, SGI55922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55923, SGI0, SGI0, SGI0, SGI55924, SGI55924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12519[] = {SGI55925, SGI55926, SGI55927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55928, SGI0, SGI0, SGI0, SGI55929, SGI55929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12520[] = {SGI55930, SGI55931, SGI55932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55933, SGI0, SGI0, SGI0, SGI55934, SGI55934, SGI0, SGI0, NULL};
const Segment * const SG12521[] = {SGI55935, SGI0, SGI55936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55937, SGI0, SGI0, SGI0, SGI55938, SGI55938, SGI0, SGI0, SGI0, SGI55939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12522[] = {SGI55940, SGI0, SGI55941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55942, SGI0, SGI0, SGI0, SGI55943, SGI55943, SGI0, SGI0, SGI0, SGI55944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12523[] = {SGI55945, SGI55946, SGI55947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55948, SGI55948, SGI0, SGI55949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12524[] = {SGI55950, SGI55951, SGI55952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55953, SGI55953, SGI0, SGI55954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12525[] = {SGI55955, SGI55956, SGI55957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55958, SGI0, SGI0, SGI0, SGI55959, SGI55959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12526[] = {SGI55960, SGI55961, SGI55962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55963, SGI0, SGI0, SGI0, SGI55964, SGI55964, SGI0, SGI0, SGI0, SGI0, SGI55965, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12527[] = {SGI55966, SGI55967, SGI55968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55969, SGI55969, SGI0, SGI0, SGI0, SGI55970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12528[] = {SGI55971, SGI55972, SGI55973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55974, SGI0, SGI0, SGI0, SGI55975, SGI55975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12529[] = {SGI55976, SGI55977, SGI55978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55981, SGI0, SGI0, SGI0, SGI55983, SGI55983, SGI0, SGI0, SGI0, SGI0, SGI55984, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12530[] = {SGI55979, SGI55980, SGI55982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55985, SGI55985, SGI0, SGI0, SGI0, SGI55986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12531[] = {SGI55987, SGI55988, SGI55989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55990, SGI0, SGI0, SGI0, SGI55991, SGI55991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12532[] = {SGI55992, SGI0, SGI55993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12533[] = {SGI55994, SGI55995, SGI55996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI55997, SGI0, SGI0, SGI0, SGI55998, SGI55998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12534[] = {SGI55999, SGI0, SGI56000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12535[] = {SGI56001, SGI56002, SGI56003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56004, SGI0, SGI0, SGI0, SGI56005, SGI56005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12536[] = {SGI56006, SGI56007, SGI56008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56009, SGI0, SGI0, SGI0, SGI56010, SGI56010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12537[] = {SGI56011, SGI56012, SGI56013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56014, SGI0, SGI0, SGI0, SGI56015, SGI56015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12538[] = {SGI56016, SGI56017, SGI56018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56019, SGI56020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12539[] = {SGI56021, SGI56022, SGI56023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56024, SGI56025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12540[] = {SGI56026, SGI56027, SGI56028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56029, SGI56030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12541[] = {SGI56031, SGI56032, SGI56033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56034, SGI56034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12542[] = {SGI56035, SGI56036, SGI56037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56038, SGI0, SGI0, SGI0, SGI56039, SGI56039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12543[] = {SGI56040, SGI0, SGI56041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56042, SGI56044, SGI56044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56047, NULL};
const Segment * const SG12544[] = {SGI56043, SGI56045, SGI56046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56048, SGI56048, SGI0, SGI0, SGI0, SGI56049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12545[] = {SGI56050, SGI56051, SGI56052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56053, SGI56053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12546[] = {SGI56054, SGI56055, SGI56056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56059, SGI56059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12547[] = {SGI56057, SGI56058, SGI56060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56061, SGI56061, SGI0, SGI56062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12548[] = {SGI56063, SGI0, SGI56064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56065, SGI56066, SGI56066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56067, NULL};
const Segment * const SG12549[] = {SGI56068, SGI56069, SGI56070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56071, SGI0, SGI0, SGI0, SGI56072, SGI56072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12550[] = {SGI56073, SGI0, SGI56074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56075, SGI0, SGI0, SGI0, SGI56076, SGI56076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12551[] = {SGI56077, SGI56078, SGI56079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56080, SGI56081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12552[] = {SGI56082, SGI56083, SGI56084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56085, SGI0, SGI0, SGI0, SGI56086, SGI56086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12553[] = {SGI56087, SGI0, SGI56088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12554[] = {SGI56089, SGI0, SGI56090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12555[] = {SGI56091, SGI56092, SGI56093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56094, SGI56094, SGI0, SGI0, SGI0, SGI56095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12556[] = {SGI56096, SGI56097, SGI56098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56099, SGI56099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12557[] = {SGI56100, SGI56101, SGI56102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56103, SGI0, SGI0, SGI0, SGI56104, SGI56104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12558[] = {SGI56106, SGI0, SGI56108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56110, SGI0, SGI0, SGI0, SGI56111, SGI56111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12559[] = {SGI56105, SGI56107, SGI56109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56112, SGI56112, SGI0, SGI0, SGI0, SGI56113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12560[] = {SGI56114, SGI0, SGI56115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12561[] = {SGI56116, SGI56117, SGI56118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56119, SGI0, SGI0, SGI0, SGI56123, SGI56123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12562[] = {SGI56120, SGI56121, SGI56122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56124, SGI0, SGI0, SGI0, SGI56126, SGI56126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12563[] = {SGI56127, SGI0, SGI56128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56129, SGI0, SGI0, SGI0, SGI56130, SGI56130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12564[] = {SGI56131, SGI56132, SGI56133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56134, SGI0, SGI0, SGI0, SGI56135, SGI56135, SGI0, SGI0, SGI0, SGI0, SGI56136, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12565[] = {SGI56137, SGI56138, SGI56139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56140, SGI0, SGI0, SGI0, SGI56141, SGI56141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56142, SGI0, SGI0, NULL};
const Segment * const SG12566[] = {SGI56143, SGI0, SGI56144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56145, SGI0, SGI0, SGI0, SGI56146, SGI56146, SGI0, SGI0, SGI0, SGI56147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12567[] = {SGI56148, SGI56149, SGI56150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56151, SGI0, SGI0, SGI0, SGI56152, SGI56152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56153, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12568[] = {SGI56154, SGI56155, SGI56156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56157, SGI56157, SGI0, SGI0, SGI0, SGI56158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12569[] = {SGI56159, SGI0, SGI56160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56161, SGI0, SGI0, SGI0, SGI56162, SGI56162, SGI0, SGI0, NULL};
const Segment * const SG12570[] = {SGI56163, SGI56164, SGI56165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56166, SGI0, SGI0, SGI0, SGI56167, SGI56167, SGI0, SGI56168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12571[] = {SGI56169, SGI56170, SGI56171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56172, SGI56172, SGI0, SGI56173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12572[] = {SGI56174, SGI56175, SGI56176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56177, SGI0, SGI0, SGI0, SGI56178, SGI56178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12573[] = {SGI56179, SGI56180, SGI56181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56184, SGI0, SGI0, SGI0, SGI56186, SGI56186, SGI0, SGI0, SGI0, SGI0, SGI56187, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12574[] = {SGI56182, SGI56183, SGI56185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56188, SGI0, SGI0, SGI0, SGI56189, SGI56189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12575[] = {SGI56190, SGI56191, SGI56192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56193, SGI56193, SGI0, SGI56194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12576[] = {SGI56195, SGI56196, SGI56197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56198, SGI0, SGI0, SGI0, SGI56199, SGI56199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12577[] = {SGI56201, SGI56202, SGI56203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56204, SGI0, SGI0, SGI0, SGI56205, SGI56205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56206, SGI0, SGI0, NULL};
const Segment * const SG12578[] = {SGI56207, SGI56208, SGI56209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56210, SGI0, SGI0, SGI0, SGI56211, SGI56211, SGI0, SGI0, SGI56212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12579[] = {SGI56213, SGI56214, SGI56215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56216, SGI0, SGI0, SGI0, SGI56217, SGI56217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56218, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12580[] = {SGI0, SGI56219, SGI0, SGI56220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56221, SGI0, SGI0, SGI0, SGI56222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12581[] = {SGI0, SGI56223, SGI0, SGI56224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12582[] = {SGI56225, SGI56226, SGI56227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56228, SGI0, SGI0, SGI0, SGI56229, SGI56229, SGI0, SGI0, SGI0, SGI0, SGI56230, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12583[] = {SGI56231, SGI56232, SGI56233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56234, SGI0, SGI0, SGI0, SGI56235, SGI56235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56236, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12584[] = {SGI56237, SGI56238, SGI56239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56240, SGI0, SGI0, SGI0, SGI56241, SGI56241, SGI56242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12585[] = {SGI56243, SGI56244, SGI56245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56246, SGI0, SGI0, SGI0, SGI56247, SGI56247, SGI0, SGI56248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12586[] = {SGI56249, SGI56250, SGI56251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56252, SGI56252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56253, SGI0, SGI0, NULL};
const Segment * const SG12587[] = {SGI56254, SGI56255, SGI56256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56257, SGI0, SGI0, SGI0, SGI56258, SGI56258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12588[] = {SGI56259, SGI56260, SGI56261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56262, SGI56262, SGI0, SGI0, SGI0, SGI56263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12589[] = {SGI56264, SGI0, SGI56265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56266, SGI0, SGI0, SGI0, SGI56267, SGI56267, SGI0, SGI56268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12590[] = {SGI56269, SGI56270, SGI56271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56272, SGI0, SGI0, SGI0, SGI56273, SGI56273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12591[] = {SGI56274, SGI56275, SGI56276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56277, SGI0, SGI0, SGI0, SGI56278, SGI56278, SGI0, SGI0, SGI0, SGI0, SGI56279, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12592[] = {SGI56280, SGI56281, SGI56282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56283, SGI56283, SGI0, SGI56284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12593[] = {SGI56285, SGI56286, SGI56287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56288, SGI0, SGI0, SGI0, SGI56289, SGI56289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12594[] = {SGI56291, SGI56292, SGI56293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56294, SGI0, SGI0, SGI0, SGI56295, SGI56295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12595[] = {SGI56296, SGI56297, SGI56298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56299, SGI56299, SGI0, SGI56300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12596[] = {SGI56301, SGI56302, SGI56303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56304, SGI56304, SGI0, SGI56305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12597[] = {SGI56306, SGI0, SGI56307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56308, SGI56309, SGI56309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56310, NULL};
const Segment * const SG12598[] = {SGI56311, SGI56312, SGI56313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56314, SGI0, SGI0, SGI0, SGI56315, SGI56315, SGI0, SGI0, SGI0, SGI56316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12599[] = {SGI56317, SGI56318, SGI56319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56320, SGI56320, SGI0, SGI56321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12600[] = {SGI56322, SGI56323, SGI56324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56325, SGI56325, SGI0, SGI0, SGI0, SGI56326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12601[] = {SGI56327, SGI56328, SGI56329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56330, SGI56330, SGI0, SGI56331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12602[] = {SGI56332, SGI56333, SGI56334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56335, SGI56335, SGI0, SGI0, SGI0, SGI56336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12603[] = {SGI56337, SGI56338, SGI56339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56340, SGI0, SGI0, SGI0, SGI56341, SGI56341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56342, SGI0, SGI0, NULL};
const Segment * const SG12604[] = {SGI0, SGI56343, SGI0, SGI56344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12605[] = {SGI56345, SGI0, SGI56346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56347, SGI0, SGI0, SGI0, SGI56348, SGI56348, SGI0, SGI0, SGI0, SGI56349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12606[] = {SGI56350, SGI0, SGI56351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56352, SGI0, SGI0, SGI0, SGI56353, SGI56353, SGI0, SGI0, SGI0, SGI56354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12607[] = {SGI56355, SGI56356, SGI56357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56358, SGI0, SGI0, SGI0, SGI56359, SGI56359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12608[] = {SGI56360, SGI56361, SGI56362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56363, SGI0, SGI0, SGI0, SGI56364, SGI56364, SGI0, SGI56365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12609[] = {SGI56366, SGI0, SGI56367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56368, SGI0, SGI0, SGI0, SGI56369, SGI56369, SGI0, SGI0, SGI0, SGI56370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12610[] = {SGI56371, SGI56372, SGI56373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56374, SGI56374, SGI0, SGI0, SGI0, SGI56375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12611[] = {SGI56376, SGI0, SGI56377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56378, SGI0, SGI0, SGI0, SGI56379, SGI56379, SGI0, SGI0, SGI0, SGI56380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12612[] = {SGI56381, SGI56382, SGI56383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56384, SGI56384, SGI0, SGI56385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12613[] = {SGI56386, SGI0, SGI56387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56388, SGI0, SGI0, SGI0, SGI56389, SGI56389, SGI0, SGI0, SGI0, SGI56390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12614[] = {SGI56391, SGI56392, SGI56393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56394, SGI56394, SGI0, SGI0, SGI0, SGI56395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12615[] = {SGI56396, SGI56397, SGI56398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56399, SGI0, SGI0, SGI0, SGI56400, SGI56400, SGI0, SGI56401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12616[] = {SGI56402, SGI56403, SGI56404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56405, SGI0, SGI0, SGI0, SGI56406, SGI56406, SGI0, SGI0, SGI0, SGI0, SGI56407, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12617[] = {SGI56408, SGI56409, SGI56410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56411, SGI56411, SGI0, SGI0, SGI0, SGI56412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12618[] = {SGI0, SGI56413, SGI0, SGI56414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12619[] = {SGI56415, SGI56416, SGI56417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56418, SGI56418, SGI0, SGI0, SGI0, SGI56419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12620[] = {SGI56420, SGI56421, SGI56422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56423, SGI0, SGI0, SGI0, SGI56424, SGI56424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56425, SGI0, SGI0, NULL};
const Segment * const SG12621[] = {SGI56426, SGI0, SGI56427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56428, SGI56429, SGI56429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56430, NULL};
const Segment * const SG12622[] = {SGI56431, SGI56432, SGI56433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56434, SGI0, SGI0, SGI0, SGI56435, SGI56435, SGI0, SGI56436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12623[] = {SGI56437, SGI56438, SGI56439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56440, SGI56440, SGI0, SGI56441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12624[] = {SGI56442, SGI0, SGI56443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56444, SGI0, SGI0, SGI0, SGI56445, SGI56445, SGI0, SGI0, SGI0, SGI56446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12625[] = {SGI56447, SGI56448, SGI56449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56450, SGI0, SGI0, SGI0, SGI56451, SGI56451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12626[] = {SGI56453, SGI56454, SGI56455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56456, SGI56456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56457, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12627[] = {SGI56458, SGI56459, SGI56460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56461, SGI56461, SGI0, SGI0, SGI0, SGI56462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12628[] = {SGI56463, SGI56464, SGI56465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56466, SGI0, SGI0, SGI0, SGI56467, SGI56467, SGI0, SGI56468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12629[] = {SGI56469, SGI56470, SGI56471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56472, SGI0, SGI0, SGI0, SGI56473, SGI56473, SGI0, SGI56474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12630[] = {SGI56475, SGI0, SGI56476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56477, SGI56478, SGI56478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56479, NULL};
const Segment * const SG12631[] = {SGI56480, SGI56481, SGI56482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56483, SGI0, SGI0, SGI0, SGI56484, SGI56484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56485, SGI0, NULL};
const Segment * const SG12632[] = {SGI56486, SGI56487, SGI56488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56489, SGI0, SGI0, SGI0, SGI56490, SGI56490, SGI0, SGI0, SGI56491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12633[] = {SGI56492, SGI0, SGI56493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56494, SGI0, SGI0, SGI0, SGI56495, SGI56495, SGI0, SGI0, SGI0, SGI56496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12634[] = {SGI56497, SGI0, SGI56498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56499, SGI0, SGI0, SGI0, SGI56500, SGI56500, SGI0, SGI0, SGI0, SGI56501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12635[] = {SGI56502, SGI0, SGI56503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56504, SGI0, SGI0, SGI0, SGI56505, SGI56505, SGI0, SGI0, SGI0, SGI56506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12636[] = {SGI56507, SGI0, SGI56508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56509, SGI0, SGI0, SGI0, SGI56510, SGI56510, SGI0, SGI0, SGI0, SGI56511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12637[] = {SGI56512, SGI56513, SGI56514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56515, SGI56515, SGI0, SGI0, SGI0, SGI56516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12638[] = {SGI56517, SGI56518, SGI56519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56520, SGI0, SGI0, SGI0, SGI56521, SGI56521, SGI0, SGI0, SGI0, SGI0, SGI56522, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12639[] = {SGI56523, SGI0, SGI56524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56525, SGI0, SGI0, SGI0, SGI56526, SGI56526, SGI0, SGI0, SGI0, SGI56527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12640[] = {SGI56528, SGI56529, SGI56530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56531, SGI56531, SGI0, SGI0, SGI0, SGI56532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12641[] = {SGI56533, SGI0, SGI56534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56535, SGI56536, SGI56536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56537, NULL};
const Segment * const SG12642[] = {SGI56538, SGI0, SGI56539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56540, SGI56541, SGI56541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56542, NULL};
const Segment * const SG12643[] = {SGI56543, SGI56544, SGI56545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56546, SGI56546, SGI0, SGI0, SGI0, SGI56549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12644[] = {SGI56547, SGI0, SGI56548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56550, SGI0, SGI0, SGI0, SGI56551, SGI56551, SGI0, SGI0, SGI0, SGI56552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12645[] = {SGI56553, SGI56554, SGI56555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56556, SGI56556, SGI0, SGI0, SGI0, SGI56557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12646[] = {SGI56558, SGI56559, SGI56560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56561, SGI0, SGI0, SGI0, SGI56562, SGI56562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12647[] = {SGI56563, SGI56564, SGI56565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56566, SGI56566, SGI0, SGI0, SGI0, SGI56567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12648[] = {SGI56568, SGI56569, SGI56570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56571, SGI0, SGI0, SGI0, SGI56572, SGI56572, SGI0, SGI56573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12649[] = {SGI56574, SGI56575, SGI56576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56577, SGI56577, SGI0, SGI0, SGI0, SGI56578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12650[] = {SGI56579, SGI56580, SGI56581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56582, SGI0, SGI0, SGI0, SGI56583, SGI56583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56584, SGI0, NULL};
const Segment * const SG12651[] = {SGI56585, SGI56586, SGI56587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56588, SGI0, SGI0, SGI0, SGI56589, SGI56589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12652[] = {SGI56591, SGI56592, SGI56593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56594, SGI0, SGI0, SGI0, SGI56595, SGI56595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56596, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12653[] = {SGI56597, SGI56598, SGI56599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56600, SGI0, SGI0, SGI0, SGI56601, SGI56601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12654[] = {SGI0, SGI56602, SGI56603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56604, SGI56604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12655[] = {SGI56606, SGI56607, SGI56608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56609, SGI0, SGI0, SGI0, SGI56610, SGI56610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12656[] = {SGI56612, SGI56613, SGI56614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56615, SGI0, SGI0, SGI0, SGI56616, SGI56616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12657[] = {SGI56617, SGI56618, SGI56619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56620, SGI0, SGI0, SGI0, SGI56621, SGI56621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12658[] = {SGI56623, SGI56624, SGI56625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56626, SGI0, SGI0, SGI0, SGI56627, SGI56627, SGI0, SGI56628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12659[] = {SGI56629, SGI56630, SGI56631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56632, SGI0, SGI0, SGI0, SGI56633, SGI56633, SGI0, SGI0, SGI0, SGI0, SGI56634, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12660[] = {SGI56635, SGI56636, SGI56637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56638, SGI56638, SGI0, SGI0, SGI0, SGI56639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12661[] = {SGI56640, SGI56641, SGI56642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56643, SGI0, SGI0, SGI0, SGI56644, SGI56644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12662[] = {SGI56645, SGI56646, SGI56647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56648, SGI0, SGI0, SGI0, SGI56649, SGI56649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12663[] = {SGI56650, SGI56651, SGI56652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56653, SGI0, SGI0, SGI0, SGI56654, SGI56654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12664[] = {SGI56655, SGI56656, SGI56657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56658, SGI0, SGI0, SGI0, SGI56659, SGI56659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12665[] = {SGI56660, SGI0, SGI56661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56662, SGI0, NULL};
const Segment * const SG12666[] = {SGI56663, SGI0, SGI56664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56667, SGI0, NULL};
const Segment * const SG12667[] = {SGI56665, SGI56666, SGI56668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56669, SGI56669, SGI0, SGI0, SGI0, SGI56670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12668[] = {SGI56671, SGI0, SGI56672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56673, SGI0, NULL};
const Segment * const SG12669[] = {SGI56674, SGI0, SGI56675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56676, SGI56677, SGI56677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56678, NULL};
const Segment * const SG12670[] = {SGI56679, SGI56680, SGI56681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56682, SGI0, SGI0, SGI0, SGI56683, SGI56683, SGI0, SGI0, SGI0, SGI0, SGI56684, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12671[] = {SGI56685, SGI0, SGI56686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56687, SGI56688, SGI56688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56689, NULL};
const Segment * const SG12672[] = {SGI56690, SGI56691, SGI56692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56693, SGI0, SGI0, SGI0, SGI56694, SGI56694, SGI0, SGI0, SGI56695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12673[] = {SGI56696, SGI56697, SGI56698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56699, SGI56699, SGI0, SGI0, SGI0, SGI56700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12674[] = {SGI56701, SGI56702, SGI56703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56704, SGI0, SGI0, SGI0, SGI56705, SGI56705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56706, SGI0, SGI0, NULL};
const Segment * const SG12675[] = {SGI56707, SGI0, SGI56708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56709, SGI0, SGI0, SGI0, SGI56710, SGI56710, SGI0, SGI56711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12676[] = {SGI56712, SGI56713, SGI56714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56715, SGI0, SGI0, SGI0, SGI56716, SGI56717, SGI0, SGI56718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12677[] = {SGI56719, SGI0, SGI56720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56721, SGI0, SGI0, SGI0, SGI56722, SGI56722, SGI0, SGI0, SGI0, SGI56723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12678[] = {SGI56724, SGI56725, SGI56726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56727, SGI0, SGI0, SGI0, SGI56728, SGI56728, SGI0, SGI0, SGI0, SGI56729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12679[] = {SGI56730, SGI56731, SGI56732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56733, SGI0, SGI0, SGI0, SGI56734, SGI56734, SGI0, SGI0, SGI0, SGI56735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12680[] = {SGI56736, SGI56737, SGI56738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56739, SGI0, SGI0, SGI0, SGI56740, SGI56740, SGI0, SGI0, SGI0, SGI56741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12681[] = {SGI56742, SGI56743, SGI56744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56745, SGI0, SGI0, SGI0, SGI56746, SGI56746, SGI0, SGI0, SGI0, SGI56747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12682[] = {SGI56748, SGI0, SGI56749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56750, SGI56751, SGI56751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56752, NULL};
const Segment * const SG12683[] = {SGI56753, SGI56754, SGI56755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56756, SGI56756, SGI0, SGI0, SGI0, SGI56757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12684[] = {SGI56758, SGI56759, SGI56760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56761, SGI56761, SGI0, SGI0, SGI0, SGI56762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12685[] = {SGI0, SGI56763, SGI56764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56765, SGI56765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12686[] = {SGI56767, SGI56768, SGI56769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56770, SGI0, SGI0, SGI0, SGI56771, SGI56771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56772, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12687[] = {SGI56773, SGI0, SGI56774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56775, SGI0, SGI0, SGI0, SGI56776, SGI56776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56777, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12688[] = {SGI56778, SGI56779, SGI56780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56781, SGI0, SGI0, SGI0, SGI56782, SGI56782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12689[] = {SGI56784, SGI56785, SGI56786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56787, SGI0, SGI0, SGI0, SGI56788, SGI56788, SGI0, SGI56789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12690[] = {SGI56790, SGI0, SGI56791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56792, SGI0, SGI0, SGI0, SGI56793, SGI56793, SGI0, SGI0, SGI0, SGI56794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12691[] = {SGI56795, SGI0, SGI56796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56797, SGI0, SGI0, SGI0, SGI56798, SGI56798, SGI0, SGI0, SGI0, SGI56799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12692[] = {SGI56800, SGI0, SGI56801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56802, SGI0, SGI0, SGI0, SGI56803, SGI56803, SGI0, SGI0, SGI0, SGI56804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12693[] = {SGI56805, SGI56806, SGI56807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56808, SGI0, SGI0, SGI0, SGI56809, SGI56809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12694[] = {SGI56810, SGI56811, SGI56812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56813, SGI0, SGI0, SGI0, SGI56814, SGI56814, SGI0, SGI0, SGI56815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12695[] = {SGI56816, SGI56817, SGI56818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56819, SGI0, SGI0, SGI0, SGI56820, SGI56820, SGI0, SGI0, SGI0, SGI0, SGI56821, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12696[] = {SGI56822, SGI56823, SGI56824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56825, SGI56825, SGI0, SGI0, SGI0, SGI56826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12697[] = {SGI56827, SGI56828, SGI56829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56830, SGI0, SGI0, SGI0, SGI56831, SGI56831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12698[] = {SGI56832, SGI0, SGI56833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56834, SGI56835, SGI56835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56836, NULL};
const Segment * const SG12699[] = {SGI56837, SGI56838, SGI56839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56840, SGI56840, SGI0, SGI0, SGI0, SGI56841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12700[] = {SGI56842, SGI0, SGI56843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56844, SGI0, SGI0, SGI0, SGI56845, SGI56845, SGI0, SGI0, SGI0, SGI56846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12701[] = {SGI56847, SGI56848, SGI56849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56850, SGI0, SGI0, SGI0, SGI56851, SGI56851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12702[] = {SGI56852, SGI56853, SGI56855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56856, SGI56856, SGI0, SGI0, SGI0, SGI56857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12703[] = {SGI56858, SGI56859, SGI56860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56861, SGI0, SGI0, SGI0, SGI56862, SGI56862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56863, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12704[] = {SGI56864, SGI56865, SGI56866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56867, SGI56867, SGI0, SGI0, SGI0, SGI56868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12705[] = {SGI56869, SGI56870, SGI56871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56872, SGI56872, SGI0, SGI0, SGI0, SGI56873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12706[] = {SGI56874, SGI56875, SGI56878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56882, SGI56882, SGI0, SGI56884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12707[] = {SGI56876, SGI56877, SGI56879, SGI56880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56881, SGI0, SGI0, SGI0, SGI56883, SGI56885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12708[] = {SGI56886, SGI56887, SGI56888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56889, SGI56889, SGI0, SGI0, SGI0, SGI56890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12709[] = {SGI56891, SGI56892, SGI56893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56894, SGI0, SGI0, SGI0, SGI56895, SGI56895, SGI0, SGI56896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12710[] = {SGI56897, SGI56898, SGI56899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56900, SGI0, SGI0, SGI0, SGI56901, SGI56901, SGI0, SGI0, SGI0, SGI0, SGI56902, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12711[] = {SGI56903, SGI56904, SGI56905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56906, SGI0, SGI0, SGI0, SGI56907, SGI56907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12712[] = {SGI56908, SGI0, SGI56909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56910, SGI0, SGI0, SGI0, SGI56911, SGI56911, SGI0, SGI56912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12713[] = {SGI56913, SGI56914, SGI56915, SGI56916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56917, SGI0, SGI0, SGI0, SGI56918, SGI56919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12714[] = {SGI56920, SGI0, SGI56921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56922, SGI0, SGI0, SGI0, SGI56923, SGI56923, SGI0, SGI0, SGI0, SGI56924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12715[] = {SGI56925, SGI56926, SGI56927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56928, SGI0, SGI0, SGI0, SGI56929, SGI56929, SGI0, SGI56930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12716[] = {SGI56931, SGI56932, SGI56933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56934, SGI0, SGI0, SGI0, SGI56935, SGI56935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56936, SGI0, SGI0, NULL};
const Segment * const SG12717[] = {SGI56937, SGI56938, SGI56939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56942, SGI0, SGI0, SGI0, SGI56943, SGI56943, SGI0, SGI0, SGI0, SGI0, SGI56945, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12718[] = {SGI56940, SGI0, SGI56941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56944, SGI0, SGI0, SGI0, SGI56946, SGI56946, SGI0, SGI0, SGI0, SGI56947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12719[] = {SGI56948, SGI56949, SGI56950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56951, SGI0, SGI0, SGI0, SGI56952, SGI56952, SGI0, SGI0, SGI0, SGI0, SGI56953, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12720[] = {SGI56954, SGI56955, SGI56956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56957, SGI0, SGI0, SGI0, SGI56958, SGI56958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56959, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12721[] = {SGI56960, SGI56961, SGI56962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56963, SGI0, SGI0, SGI0, SGI56964, SGI56964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12722[] = {SGI56965, SGI0, SGI56967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56968, SGI0, SGI0, SGI0, SGI56969, SGI56969, SGI0, SGI0, SGI0, SGI56970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12723[] = {SGI56971, SGI0, SGI56972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56973, SGI0, SGI0, SGI0, SGI56974, SGI56974, SGI0, SGI0, SGI0, SGI56975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12724[] = {SGI56976, SGI56977, SGI56978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56982, SGI0, SGI0, SGI0, SGI56983, SGI56983, SGI0, SGI0, SGI0, SGI0, SGI56985, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12725[] = {SGI56979, SGI56980, SGI56981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56984, SGI0, SGI0, SGI0, SGI56986, SGI56986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56987, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12726[] = {SGI56988, SGI56989, SGI56990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56991, SGI0, SGI0, SGI0, SGI56992, SGI56992, SGI0, SGI56993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12727[] = {SGI56994, SGI56995, SGI56996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI56997, SGI56997, SGI0, SGI0, SGI0, SGI56998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12728[] = {SGI56999, SGI57000, SGI57001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57002, SGI0, SGI0, SGI0, SGI57003, SGI57003, SGI0, SGI0, SGI0, SGI0, SGI57004, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12729[] = {SGI57005, SGI57006, SGI57007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57008, SGI0, SGI0, SGI0, SGI57009, SGI57009, SGI0, SGI0, SGI57010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12730[] = {SGI57011, SGI57012, SGI57013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57014, SGI0, SGI0, SGI0, SGI57015, SGI57015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57016, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12731[] = {SGI57017, SGI57018, SGI57019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57020, SGI0, SGI0, SGI0, SGI57021, SGI57021, SGI0, SGI0, SGI0, SGI0, SGI57022, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12732[] = {SGI57023, SGI0, SGI57024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57025, SGI57026, SGI57026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57027, NULL};
const Segment * const SG12733[] = {SGI57028, SGI57029, SGI57030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57031, SGI57031, SGI0, SGI0, SGI0, SGI57033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12734[] = {SGI57032, SGI0, SGI57034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57035, SGI0, SGI0, SGI0, SGI57036, SGI57036, SGI0, SGI57037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12735[] = {SGI57038, SGI57039, SGI57040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57041, SGI57041, SGI0, SGI57042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12736[] = {SGI57043, SGI57044, SGI57045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57046, SGI0, SGI0, SGI0, SGI57047, SGI57047, SGI0, SGI57048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12737[] = {SGI57049, SGI57050, SGI57051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57052, SGI0, SGI0, SGI0, SGI57053, SGI57053, SGI0, SGI0, SGI0, SGI0, SGI57054, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12738[] = {SGI57055, SGI57056, SGI57057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57058, SGI0, SGI0, SGI0, SGI57059, SGI57059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57060, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12739[] = {SGI57061, SGI57062, SGI57063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57064, SGI0, SGI0, SGI0, SGI57065, SGI57065, SGI0, SGI0, NULL};
const Segment * const SG12740[] = {SGI57066, SGI57067, SGI57068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57069, SGI0, SGI0, SGI0, SGI57070, SGI57071, SGI0, SGI0, SGI0, SGI57072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12741[] = {SGI57073, SGI57074, SGI57075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57076, SGI57076, SGI0, SGI0, SGI0, SGI57077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12742[] = {SGI57078, SGI57079, SGI57080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57081, SGI57081, SGI0, SGI0, SGI0, SGI57082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12743[] = {SGI57083, SGI57084, SGI57085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57086, SGI57086, SGI0, SGI0, SGI0, SGI57087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12744[] = {SGI57088, SGI57089, SGI57090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57091, SGI0, SGI0, SGI0, SGI57092, SGI57092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57093, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12745[] = {SGI57094, SGI57095, SGI57096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57097, SGI0, SGI0, SGI0, SGI57098, SGI57098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12746[] = {SGI57100, SGI57101, SGI57102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57103, SGI0, SGI0, SGI0, SGI57104, SGI57104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12747[] = {SGI57105, SGI0, SGI57106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57107, SGI0, SGI0, SGI0, SGI57108, SGI57108, SGI0, SGI0, SGI0, SGI57109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12748[] = {SGI57110, SGI57111, SGI57112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57113, SGI0, SGI0, SGI0, SGI57114, SGI57114, SGI0, SGI0, SGI0, SGI0, SGI57115, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12749[] = {SGI57116, SGI0, SGI57117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57118, SGI0, SGI0, SGI0, SGI57119, SGI57119, SGI0, SGI0, SGI0, SGI57120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12750[] = {SGI57121, SGI0, SGI57122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57123, SGI0, SGI0, SGI0, SGI57124, SGI57124, SGI0, SGI0, SGI0, SGI57125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12751[] = {SGI57126, SGI0, SGI57127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57128, SGI0, SGI0, SGI0, SGI57129, SGI57129, SGI0, SGI0, SGI0, SGI57130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12752[] = {SGI57131, SGI57132, SGI57133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57134, SGI0, SGI0, SGI0, SGI57135, SGI57135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12753[] = {SGI57137, SGI0, SGI57138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57139, SGI0, SGI0, SGI0, SGI57140, SGI57140, SGI0, SGI0, SGI0, SGI57141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12754[] = {SGI57142, SGI57143, SGI57144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57145, SGI0, SGI0, SGI0, SGI57146, SGI57146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12755[] = {SGI57148, SGI57149, SGI57150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57151, SGI0, SGI0, SGI0, SGI57152, SGI57152, SGI0, SGI0, SGI0, SGI57153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12756[] = {SGI57154, SGI57155, SGI57156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57157, SGI0, SGI0, SGI0, SGI57158, SGI57158, SGI0, SGI57159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12757[] = {SGI57160, SGI57161, SGI57162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57163, SGI0, SGI0, SGI0, SGI57164, SGI57164, SGI0, SGI57165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12758[] = {SGI57166, SGI57167, SGI57168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57169, SGI57169, SGI0, SGI0, SGI0, SGI57170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12759[] = {SGI57171, SGI57172, SGI57173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57174, SGI57174, SGI0, SGI0, SGI0, SGI57175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12760[] = {SGI57176, SGI57177, SGI57178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57179, SGI0, SGI0, SGI0, SGI57180, SGI57181, SGI0, SGI0, SGI0, SGI57182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12761[] = {SGI57183, SGI57184, SGI57185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57186, SGI57186, SGI0, SGI0, SGI0, SGI57187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12762[] = {SGI57188, SGI57189, SGI57190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57191, SGI57191, SGI0, SGI0, SGI0, SGI57192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12763[] = {SGI57193, SGI57194, SGI57195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57196, SGI57196, SGI0, SGI0, SGI0, SGI57197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12764[] = {SGI57198, SGI0, SGI57199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57200, SGI57201, SGI57201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57202, NULL};
const Segment * const SG12765[] = {SGI57203, SGI57204, SGI57205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57206, SGI0, SGI0, SGI0, SGI57207, SGI57207, SGI0, SGI0, SGI0, SGI0, SGI57208, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12766[] = {SGI57209, SGI57210, SGI57211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57212, SGI57212, SGI0, SGI0, SGI0, SGI57213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12767[] = {SGI57214, SGI57215, SGI57216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57217, SGI57217, SGI0, SGI0, SGI0, SGI57218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12768[] = {SGI57219, SGI57220, SGI57221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57222, SGI0, SGI0, SGI0, SGI57223, SGI57223, SGI0, SGI0, SGI0, SGI0, SGI57224, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12769[] = {SGI57225, SGI57226, SGI57227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57228, SGI57228, SGI0, SGI0, SGI0, SGI57229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12770[] = {SGI57230, SGI57231, SGI57232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57233, SGI0, SGI0, SGI0, SGI57234, SGI57234, SGI0, SGI0, SGI0, SGI57235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12771[] = {SGI57236, SGI57237, SGI57238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57239, SGI57239, SGI0, SGI57240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12772[] = {SGI57241, SGI57242, SGI57243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57244, SGI0, SGI0, SGI0, SGI57245, SGI57246, SGI0, SGI0, SGI0, SGI57247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12773[] = {SGI57248, SGI57249, SGI57250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57251, SGI0, SGI0, SGI0, SGI57252, SGI57253, SGI0, SGI0, SGI0, SGI57254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12774[] = {SGI57255, SGI57256, SGI57257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57258, SGI0, SGI0, SGI0, SGI57259, SGI57260, SGI0, SGI0, SGI0, SGI57261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12775[] = {SGI57262, SGI57263, SGI57264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57265, SGI57265, SGI0, SGI0, SGI0, SGI57266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12776[] = {SGI57267, SGI57268, SGI57269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57270, SGI57270, SGI0, SGI0, SGI0, SGI57271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12777[] = {SGI57272, SGI57273, SGI57274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57275, SGI57275, SGI0, SGI0, SGI0, SGI57276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12778[] = {SGI57277, SGI57278, SGI57279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57280, SGI0, SGI0, SGI0, SGI57281, SGI57281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12779[] = {SGI57282, SGI57283, SGI57284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57285, SGI0, SGI0, SGI0, SGI57286, SGI57286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12780[] = {SGI57287, SGI0, SGI57288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57289, SGI0, SGI0, SGI0, SGI57290, SGI57290, SGI0, SGI0, SGI0, SGI57291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12781[] = {SGI57292, SGI57293, SGI57294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57295, SGI0, SGI0, SGI0, SGI57296, SGI57296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57297, SGI0, SGI0, NULL};
const Segment * const SG12782[] = {SGI57298, SGI57299, SGI57300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57301, SGI0, SGI0, SGI0, SGI57302, SGI57302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12783[] = {SGI57303, SGI0, SGI57304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57305, SGI0, SGI0, SGI0, SGI57306, SGI57306, SGI0, SGI0, SGI0, SGI57307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12784[] = {SGI57308, SGI0, SGI57309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57310, SGI0, SGI0, SGI0, SGI57311, SGI57311, SGI0, SGI0, SGI0, SGI57312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12785[] = {SGI57313, SGI0, SGI57314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57315, SGI0, SGI0, SGI0, SGI57316, SGI57316, SGI0, SGI0, SGI0, SGI57317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12786[] = {SGI57318, SGI0, SGI57319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57320, SGI0, SGI0, SGI0, SGI57321, SGI57321, SGI0, SGI0, SGI0, SGI57322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12787[] = {SGI57323, SGI0, SGI57324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57325, SGI0, SGI0, SGI0, SGI57326, SGI57326, SGI0, SGI0, SGI0, SGI57327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12788[] = {SGI57328, SGI57329, SGI57330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57331, SGI57331, SGI0, SGI0, SGI0, SGI57332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12789[] = {SGI57333, SGI57334, SGI57335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57336, SGI0, SGI0, SGI0, SGI57337, SGI57337, SGI0, SGI0, SGI0, SGI0, SGI57338, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12790[] = {SGI57339, SGI57340, SGI57341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57342, SGI57342, SGI0, SGI0, SGI0, SGI57343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12791[] = {SGI57344, SGI57345, SGI57346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57347, SGI0, SGI0, SGI0, SGI57348, SGI57348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12792[] = {SGI57349, SGI57350, SGI57351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57352, SGI0, SGI0, SGI0, SGI57353, SGI57353, SGI0, SGI0, SGI0, SGI0, SGI57354, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12793[] = {SGI57355, SGI57356, SGI57357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57358, SGI0, SGI0, SGI0, SGI57359, SGI57359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12794[] = {SGI57360, SGI57361, SGI57362, SGI57363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57364, SGI0, SGI0, SGI0, SGI57365, SGI57366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12795[] = {SGI57367, SGI0, SGI57368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57369, SGI0, SGI0, SGI0, SGI57370, SGI57370, SGI0, SGI0, SGI0, SGI57371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12796[] = {SGI57372, SGI57373, SGI57374, SGI57375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57376, SGI0, SGI0, SGI0, SGI57377, SGI57378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12797[] = {SGI57379, SGI57380, SGI57381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57382, SGI0, SGI0, SGI0, SGI57383, SGI57383, SGI0, SGI57384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12798[] = {SGI57385, SGI57386, SGI57387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57388, SGI0, SGI0, SGI0, SGI57389, SGI57389, SGI0, SGI0, SGI57390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12799[] = {SGI0, SGI57391, SGI57392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57393, SGI57393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12800[] = {SGI0, SGI57395, SGI57396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57397, SGI57397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12801[] = {SGI0, SGI57399, SGI0, SGI57400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12802[] = {SGI57401, SGI57402, SGI57403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57404, SGI0, SGI0, SGI0, SGI57405, SGI57405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57406, SGI0, SGI0, NULL};
const Segment * const SG12803[] = {SGI57407, SGI57408, SGI57409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57410, SGI0, SGI0, SGI0, SGI57411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12804[] = {SGI57413, SGI57414, SGI57415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57416, SGI0, SGI0, SGI0, SGI57417, SGI57417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12805[] = {SGI57418, SGI57419, SGI57420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57421, SGI0, SGI0, SGI0, SGI57422, SGI57422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12806[] = {SGI0, SGI57423, SGI0, SGI57424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12807[] = {SGI57425, SGI57426, SGI57428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57431, SGI0, SGI0, SGI0, SGI57433, SGI57433, SGI0, SGI0, SGI0, SGI0, SGI57435, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12808[] = {SGI57427, SGI57429, SGI57430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57432, SGI0, SGI0, SGI0, SGI57434, SGI57434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12809[] = {SGI57440, SGI57441, SGI57442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57443, SGI0, SGI0, SGI0, SGI57444, SGI57444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12810[] = {SGI57436, SGI57437, SGI57438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57439, SGI0, SGI0, SGI0, SGI57445, SGI57445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12811[] = {SGI57446, SGI57448, SGI57450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57453, SGI0, SGI0, SGI0, SGI57455, SGI57455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12812[] = {SGI57447, SGI57449, SGI57451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57452, SGI0, SGI0, SGI0, SGI57454, SGI57454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12813[] = {SGI57456, SGI57457, SGI57458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57459, SGI0, SGI0, SGI0, SGI57460, SGI57460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12814[] = {SGI57461, SGI57462, SGI57463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57464, SGI0, SGI0, SGI0, SGI57467, SGI57467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12815[] = {SGI0, SGI57465, SGI57466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57468, SGI57468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12816[] = {SGI57470, SGI0, SGI57471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12817[] = {SGI57472, SGI57473, SGI57474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57475, SGI0, SGI0, SGI0, SGI57476, SGI57476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12818[] = {SGI57477, SGI57478, SGI57479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57480, SGI0, SGI0, SGI0, SGI57481, SGI57481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12819[] = {SGI57482, SGI0, SGI57483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12820[] = {SGI57484, SGI57485, SGI57486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57487, SGI57487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12821[] = {SGI57488, SGI57489, SGI57490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57491, SGI57492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12822[] = {SGI57493, SGI57494, SGI57495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57496, SGI0, SGI0, SGI0, SGI57497, SGI57497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12823[] = {SGI57498, SGI57499, SGI57500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57501, SGI0, SGI0, SGI0, SGI57502, SGI57502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12824[] = {SGI57503, SGI0, SGI57504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12825[] = {SGI57507, SGI0, SGI57509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12826[] = {SGI57505, SGI57506, SGI57508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57510, SGI0, SGI0, SGI0, SGI57511, SGI57511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12827[] = {SGI57512, SGI57513, SGI57514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57515, SGI0, SGI0, SGI0, SGI57516, SGI57516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12828[] = {SGI57517, SGI57518, SGI57519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57521, SGI0, SGI0, SGI0, SGI57524, SGI57524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12829[] = {SGI57520, SGI57522, SGI57523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57525, SGI0, SGI0, SGI0, SGI57527, SGI57527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12830[] = {SGI57528, SGI57529, SGI57530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57531, SGI0, SGI0, SGI0, SGI57532, SGI57532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12831[] = {SGI57533, SGI0, SGI57534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57535, SGI0, SGI0, SGI0, SGI57536, SGI57536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12832[] = {SGI57537, SGI57538, SGI57539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57540, SGI0, SGI0, SGI0, SGI57541, SGI57541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12833[] = {SGI57542, SGI57544, SGI57546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57548, SGI0, SGI0, SGI0, SGI57550, SGI57550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12834[] = {SGI57543, SGI57545, SGI57547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57549, SGI0, SGI0, SGI0, SGI57551, SGI57551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12835[] = {SGI57553, SGI57554, SGI57555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57556, SGI0, SGI0, SGI0, SGI57557, SGI57557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12836[] = {SGI57558, SGI57559, SGI57560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57561, SGI0, SGI0, SGI0, SGI57562, SGI57562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12837[] = {SGI57563, SGI57564, SGI57565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57566, SGI0, SGI0, SGI0, SGI57567, SGI57567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57570, SGI0, SGI0, NULL};
const Segment * const SG12838[] = {SGI57568, SGI0, SGI57569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57571, SGI0, SGI0, SGI0, SGI57572, SGI57572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12839[] = {SGI57573, SGI0, SGI57574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12840[] = {SGI57575, SGI57576, SGI57577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57578, SGI0, SGI0, SGI0, SGI57579, SGI57579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12841[] = {SGI57580, SGI57581, SGI57582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57583, SGI0, SGI0, SGI0, SGI57584, SGI57584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12842[] = {SGI57585, SGI0, SGI57586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57587, SGI0, SGI0, SGI0, SGI57588, SGI57588, SGI0, SGI0, SGI0, SGI57589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12843[] = {SGI57590, SGI0, SGI57591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12844[] = {SGI57592, SGI57593, SGI57594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57595, SGI0, SGI0, SGI0, SGI57596, SGI57596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12845[] = {SGI57597, SGI57598, SGI57599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57600, SGI0, SGI0, SGI0, SGI57601, SGI57601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12846[] = {SGI57602, SGI0, SGI57603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12847[] = {SGI57604, SGI57605, SGI57606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57607, SGI0, SGI0, SGI0, SGI57608, SGI57608, SGI0, SGI0, SGI0, SGI0, SGI57609, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12848[] = {SGI57610, SGI0, SGI57611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57612, SGI0, SGI0, SGI0, SGI57613, SGI57613, SGI0, SGI0, SGI0, SGI57614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12849[] = {SGI57615, SGI57616, SGI57617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57618, SGI0, SGI0, SGI0, SGI57619, SGI57619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57620, SGI0, SGI0, NULL};
const Segment * const SG12850[] = {SGI57621, SGI57622, SGI57623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57624, SGI0, SGI0, SGI0, SGI57625, SGI57625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12851[] = {SGI57626, SGI57627, SGI57628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57629, SGI0, SGI0, SGI0, SGI57630, SGI57630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12852[] = {SGI57631, SGI57632, SGI57633, SGI57634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57635, SGI0, SGI0, SGI0, SGI57636, SGI57637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12853[] = {SGI57638, SGI57639, SGI57640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57641, SGI0, SGI0, SGI0, SGI57642, SGI57642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12854[] = {SGI57643, SGI57644, SGI57645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57646, SGI0, SGI0, SGI0, SGI57647, SGI57647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12855[] = {SGI57649, SGI57650, SGI57651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57652, SGI0, SGI0, SGI0, SGI57653, SGI57653, SGI0, SGI57654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12856[] = {SGI57655, SGI57656, SGI57657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57658, SGI0, SGI0, SGI0, SGI57659, SGI57659, SGI0, SGI57660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12857[] = {SGI57661, SGI57662, SGI57663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57664, SGI0, SGI0, SGI0, SGI57665, SGI57665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57666, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12858[] = {SGI57667, SGI57668, SGI57669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57670, SGI0, SGI0, SGI0, SGI57671, SGI57671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12859[] = {SGI57673, SGI0, SGI57674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57675, SGI0, SGI0, SGI0, SGI57676, SGI57676, SGI0, SGI0, SGI0, SGI57677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12860[] = {SGI57678, SGI0, SGI57679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57680, SGI0, SGI0, SGI0, SGI57681, SGI57681, SGI0, SGI0, SGI0, SGI57682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12861[] = {SGI57683, SGI57684, SGI57685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57686, SGI0, SGI0, SGI0, SGI57687, SGI57687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57688, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12862[] = {SGI57689, SGI57690, SGI57691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57692, SGI0, SGI0, SGI0, SGI57693, SGI57693, SGI0, SGI0, SGI0, SGI0, SGI57694, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12863[] = {SGI57695, SGI57696, SGI57697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57698, SGI0, SGI0, SGI0, SGI57699, SGI57699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12864[] = {SGI57700, SGI57701, SGI57702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57703, SGI0, SGI0, SGI0, SGI57704, SGI57704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12865[] = {SGI57705, SGI57706, SGI57707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57708, SGI0, SGI0, SGI0, SGI57709, SGI57709, SGI0, SGI57710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12866[] = {SGI57711, SGI57712, SGI57713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57714, SGI0, SGI0, SGI0, SGI57715, SGI57715, SGI0, SGI0, SGI0, SGI0, SGI57716, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12867[] = {SGI57717, SGI0, SGI57718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57719, SGI0, NULL};
const Segment * const SG12868[] = {SGI0, SGI57720, SGI57721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57722, SGI0, SGI0, SGI0, SGI0, SGI57723, SGI0, SGI0, SGI0, SGI0, SGI57724, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12869[] = {SGI57725, SGI57726, SGI57727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57728, SGI0, SGI0, SGI0, SGI57729, SGI57729, SGI0, SGI0, SGI0, SGI0, SGI57730, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12870[] = {SGI57731, SGI0, SGI57732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57733, SGI0, SGI0, SGI0, SGI57734, SGI57734, SGI0, SGI0, SGI0, SGI57735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12871[] = {SGI57736, SGI57737, SGI57738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57739, SGI0, SGI0, SGI0, SGI57740, SGI57740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12872[] = {SGI57741, SGI57742, SGI57743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57744, SGI0, SGI0, SGI0, SGI57745, SGI57745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57746, SGI0, SGI0, NULL};
const Segment * const SG12873[] = {SGI57747, SGI57748, SGI57749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57750, SGI0, SGI0, SGI0, SGI57751, SGI57751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12874[] = {SGI57753, SGI57754, SGI57755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57756, SGI0, SGI0, SGI0, SGI57757, SGI57757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12875[] = {SGI57758, SGI57759, SGI57760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57761, SGI0, SGI0, SGI0, SGI57762, SGI57762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12876[] = {SGI57763, SGI57764, SGI57765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57766, SGI0, SGI0, SGI0, SGI57767, SGI57767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12877[] = {SGI57768, SGI57769, SGI57770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57771, SGI0, SGI0, SGI0, SGI57772, SGI57772, SGI0, SGI57773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12878[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57774, SGI0, SGI0, SGI57775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12879[] = {SGI57776, SGI57777, SGI57778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57779, SGI0, SGI0, SGI0, SGI57780, SGI57780, SGI0, SGI0, SGI0, SGI0, SGI57781, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12880[] = {SGI57782, SGI57783, SGI57784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57785, SGI57785, SGI0, SGI57786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12881[] = {SGI57787, SGI57788, SGI57789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57790, SGI0, SGI0, SGI0, SGI57791, SGI57791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12882[] = {SGI57792, SGI57793, SGI57794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57795, SGI0, SGI0, SGI0, SGI57796, SGI57796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12883[] = {SGI57797, SGI57798, SGI57799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57800, SGI0, SGI0, SGI0, SGI57801, SGI57801, SGI0, SGI0, SGI0, SGI0, SGI57802, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12884[] = {SGI57803, SGI0, SGI57804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57805, SGI57806, SGI57806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57807, NULL};
const Segment * const SG12885[] = {SGI57808, SGI0, SGI57809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57810, SGI0, SGI0, SGI0, SGI57811, SGI57811, SGI0, SGI0, SGI0, SGI57812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12886[] = {SGI0, SGI57813, SGI0, SGI57814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12887[] = {SGI57815, SGI57816, SGI57817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57818, SGI57819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12888[] = {SGI57820, SGI57821, SGI57822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57823, SGI0, SGI0, SGI0, SGI57824, SGI57824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12889[] = {SGI57825, SGI57826, SGI57827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57828, SGI0, SGI0, SGI0, SGI57829, SGI57829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12890[] = {SGI57830, SGI0, SGI57831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57832, SGI0, SGI0, SGI0, SGI57833, SGI57833, SGI0, SGI0, SGI0, SGI57834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12891[] = {SGI57835, SGI57836, SGI57837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57838, SGI0, SGI0, SGI0, SGI57839, SGI57839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12892[] = {SGI57840, SGI57841, SGI57842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57843, SGI0, SGI0, SGI0, SGI57844, SGI57844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12893[] = {SGI57845, SGI57846, SGI57847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57848, SGI0, SGI0, SGI0, SGI57849, SGI57849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12894[] = {SGI57850, SGI57851, SGI57852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57853, SGI0, SGI0, SGI0, SGI57854, SGI57854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12895[] = {SGI57855, SGI0, SGI57856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12896[] = {SGI57857, SGI57858, SGI57859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57860, SGI0, SGI0, SGI0, SGI57861, SGI57861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12897[] = {SGI57862, SGI57863, SGI57864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57865, SGI0, SGI0, SGI0, SGI57866, SGI57866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12898[] = {SGI57867, SGI57868, SGI57869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57870, SGI0, SGI0, SGI0, SGI57871, SGI57871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12899[] = {SGI57872, SGI0, SGI57873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57874, SGI0, SGI0, SGI0, SGI57875, SGI57875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12900[] = {SGI57878, SGI0, SGI57879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12901[] = {SGI57876, SGI0, SGI57877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57880, SGI0, SGI0, SGI0, SGI57881, SGI57881, SGI0, SGI0, SGI0, SGI57882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12902[] = {SGI57883, SGI57884, SGI57885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57886, SGI0, SGI0, SGI0, SGI57887, SGI57887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12903[] = {SGI57888, SGI57889, SGI57890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57891, SGI0, SGI0, SGI0, SGI57892, SGI57892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12904[] = {SGI57893, SGI57894, SGI57895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57896, SGI0, SGI0, SGI0, SGI57897, SGI57897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12905[] = {SGI57898, SGI57899, SGI57900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57901, SGI57902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12906[] = {SGI57903, SGI57904, SGI57905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57906, SGI0, SGI0, SGI0, SGI57907, SGI57907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12907[] = {SGI57908, SGI57909, SGI57910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57911, SGI0, SGI0, SGI0, SGI57912, SGI57912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12908[] = {SGI57913, SGI0, SGI57914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12909[] = {SGI57915, SGI0, SGI57916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12910[] = {SGI57917, SGI0, SGI57918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12911[] = {SGI57919, SGI0, SGI57920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12912[] = {SGI57921, SGI57922, SGI57923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57924, SGI57924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12913[] = {SGI0, SGI57925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57926, SGI0, SGI0, SGI0, SGI57927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12914[] = {SGI57928, SGI57929, SGI57930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57931, SGI0, SGI0, SGI0, SGI57932, SGI57932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12915[] = {SGI57933, SGI57934, SGI57935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57936, SGI57936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12916[] = {SGI57937, SGI57938, SGI57939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57940, SGI0, SGI0, SGI0, SGI57941, SGI57941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12917[] = {SGI57942, SGI57943, SGI57944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57945, SGI0, SGI0, SGI0, SGI57946, SGI57946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12918[] = {SGI57947, SGI57948, SGI57949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57950, SGI0, SGI0, SGI0, SGI57951, SGI57951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12919[] = {SGI57952, SGI57953, SGI57954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57955, SGI0, SGI0, SGI0, SGI57956, SGI57956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12920[] = {SGI57957, SGI57958, SGI57959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57960, SGI0, SGI0, SGI0, SGI57961, SGI57961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12921[] = {SGI57962, SGI57963, SGI57964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57965, SGI0, SGI0, SGI0, SGI57966, SGI57966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12922[] = {SGI57967, SGI0, SGI57968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57969, SGI0, SGI0, SGI0, SGI57970, SGI57970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12923[] = {SGI57971, SGI0, SGI57972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57973, SGI0, SGI0, SGI0, SGI57974, SGI57974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12924[] = {SGI57975, SGI57976, SGI57977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57978, SGI0, SGI0, SGI0, SGI57979, SGI57979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12925[] = {SGI57980, SGI0, SGI57981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57982, SGI0, SGI0, SGI0, SGI57983, SGI57983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12926[] = {SGI57984, SGI57985, SGI57986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57987, SGI0, SGI0, SGI0, SGI57988, SGI57988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12927[] = {SGI57989, SGI57990, SGI57991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57992, SGI0, SGI0, SGI0, SGI57993, SGI57993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12928[] = {SGI57994, SGI57995, SGI57996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI57997, SGI0, SGI0, SGI0, SGI57998, SGI57998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12929[] = {SGI57999, SGI58000, SGI58001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58002, SGI0, SGI0, SGI0, SGI58003, SGI58003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12930[] = {SGI58004, SGI58005, SGI58006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58007, SGI0, SGI0, SGI0, SGI58008, SGI58008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12931[] = {SGI58009, SGI58010, SGI58011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58012, SGI0, SGI0, SGI0, SGI58013, SGI58013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58014, SGI0, SGI0, NULL};
const Segment * const SG12932[] = {SGI58015, SGI58016, SGI58017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58018, SGI0, SGI0, SGI0, SGI58019, SGI58019, SGI0, SGI58020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12933[] = {SGI58021, SGI58022, SGI58023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58024, SGI0, SGI0, SGI0, SGI58025, SGI58025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12934[] = {SGI58026, SGI0, SGI58027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58028, SGI0, SGI0, SGI0, SGI58029, SGI58029, SGI0, SGI0, SGI0, SGI58030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12935[] = {SGI58031, SGI0, SGI58032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58033, SGI0, SGI0, SGI0, SGI58034, SGI58034, SGI0, SGI0, SGI0, SGI58035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12936[] = {SGI58036, SGI58037, SGI58038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58039, SGI0, SGI0, SGI0, SGI58040, SGI58040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12937[] = {SGI58041, SGI0, SGI58042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58043, SGI58044, SGI58044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58045, NULL};
const Segment * const SG12938[] = {SGI0, SGI58046, SGI0, SGI58047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12939[] = {SGI58048, SGI0, SGI58049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58052, SGI0, SGI0, SGI0, SGI58054, SGI58054, SGI0, SGI0, SGI0, SGI58055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12940[] = {SGI58050, SGI0, SGI58051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58053, SGI0, SGI0, SGI0, SGI58056, SGI58056, SGI0, SGI58057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12941[] = {SGI0, SGI58058, SGI0, SGI58059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12942[] = {SGI58060, SGI0, SGI58061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58062, SGI0, SGI0, SGI0, SGI58063, SGI58063, SGI0, SGI0, SGI0, SGI58064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12943[] = {SGI58065, SGI58066, SGI58067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58068, SGI0, SGI0, SGI0, SGI58069, SGI58069, SGI0, SGI58070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12944[] = {SGI58071, SGI58072, SGI58073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58074, SGI0, SGI0, SGI0, SGI58075, SGI58075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12945[] = {SGI58076, SGI58077, SGI58078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58079, SGI0, SGI0, SGI0, SGI58080, SGI58080, SGI0, SGI0, NULL};
const Segment * const SG12946[] = {SGI58081, SGI58082, SGI58083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58084, SGI0, SGI0, SGI0, SGI58085, SGI58085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12947[] = {SGI58087, SGI58088, SGI58089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58090, SGI0, SGI0, SGI0, SGI58091, SGI58091, SGI0, SGI0, SGI0, SGI0, SGI58092, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12948[] = {SGI58093, SGI58094, SGI58095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58096, SGI0, SGI0, SGI0, SGI58097, SGI58097, SGI0, SGI0, SGI0, SGI0, SGI58098, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12949[] = {SGI58099, SGI0, SGI58100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58101, SGI0, SGI0, SGI0, SGI58102, SGI58102, SGI0, SGI0, SGI0, SGI58103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12950[] = {SGI58104, SGI58105, SGI58106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58107, SGI0, SGI0, SGI0, SGI58108, SGI58108, SGI0, SGI58109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12951[] = {SGI58110, SGI0, SGI58111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12952[] = {SGI58112, SGI58113, SGI58114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58115, SGI0, SGI0, SGI0, SGI58116, SGI58116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12953[] = {SGI0, SGI58117, SGI58118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58119, SGI0, SGI0, SGI0, SGI0, SGI58120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12954[] = {SGI58121, SGI58122, SGI58123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58124, SGI0, SGI0, SGI0, SGI58125, SGI58125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12955[] = {SGI58126, SGI58127, SGI58128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58129, SGI0, SGI0, SGI0, SGI58130, SGI58130, SGI0, SGI58131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12956[] = {SGI58132, SGI0, SGI58133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58134, SGI0, SGI0, SGI0, SGI58136, SGI58136, SGI0, SGI0, SGI0, SGI58139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12957[] = {SGI58135, SGI58137, SGI58138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58140, SGI0, SGI0, SGI0, SGI58141, SGI58141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12958[] = {SGI58142, SGI0, SGI58143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58144, SGI0, SGI0, SGI0, SGI58145, SGI58145, SGI0, SGI0, SGI0, SGI58146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12959[] = {SGI58147, SGI58148, SGI58149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58150, SGI0, SGI0, SGI0, SGI58151, SGI58151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12960[] = {SGI58152, SGI58153, SGI58154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58155, SGI0, SGI0, SGI0, SGI58156, SGI58156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12961[] = {SGI58157, SGI0, SGI58158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58159, SGI0, SGI0, SGI0, SGI58160, SGI58160, SGI0, SGI0, SGI0, SGI58161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12962[] = {SGI58162, SGI58163, SGI58164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58165, SGI0, SGI0, SGI0, SGI58166, SGI58166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58167, SGI0, SGI0, NULL};
const Segment * const SG12963[] = {SGI58168, SGI58169, SGI58170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58171, SGI0, SGI0, SGI0, SGI58172, SGI58172, SGI0, SGI0, SGI0, SGI0, SGI58173, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12964[] = {SGI58174, SGI0, SGI58175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58176, SGI0, SGI0, SGI0, SGI58177, SGI58177, SGI0, SGI0, SGI0, SGI58178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12965[] = {SGI58179, SGI58180, SGI58181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58182, SGI0, SGI0, SGI0, SGI58183, SGI58183, SGI0, SGI0, SGI0, SGI0, SGI58184, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12966[] = {SGI58185, SGI58186, SGI58187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58188, SGI0, SGI0, SGI0, SGI58189, SGI58189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12967[] = {SGI58191, SGI58192, SGI58193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58194, SGI0, SGI0, SGI0, SGI58195, SGI58195, SGI58196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12968[] = {SGI58197, SGI0, SGI58198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58199, SGI0, SGI0, SGI0, SGI58200, SGI58200, SGI0, SGI0, SGI0, SGI58201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12969[] = {SGI58202, SGI58203, SGI58204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58205, SGI0, SGI0, SGI0, SGI58206, SGI58206, SGI0, SGI58207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12970[] = {SGI58208, SGI58209, SGI58210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58211, SGI58211, SGI0, SGI58212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12971[] = {SGI58213, SGI58214, SGI58215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58216, SGI0, SGI0, SGI0, SGI58217, SGI0, SGI0, SGI58218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12972[] = {SGI0, SGI58219, SGI0, SGI58220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12973[] = {SGI58221, SGI58222, SGI58223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58224, SGI0, SGI0, SGI0, SGI58225, SGI58225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12974[] = {SGI58227, SGI58228, SGI58229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58230, SGI0, SGI0, SGI0, SGI58231, SGI58231, SGI58232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12975[] = {SGI58233, SGI0, SGI58234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58235, SGI0, SGI0, SGI0, SGI58236, SGI58236, SGI0, SGI0, SGI0, SGI58237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12976[] = {SGI58238, SGI58239, SGI58240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58241, SGI0, SGI0, SGI0, SGI58242, SGI58242, SGI0, SGI58243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12977[] = {SGI0, SGI58244, SGI0, SGI58245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12978[] = {SGI58246, SGI58247, SGI58248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58249, SGI0, SGI0, SGI0, SGI58250, SGI58250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58251, SGI0, SGI0, NULL};
const Segment * const SG12979[] = {SGI58252, SGI0, SGI58253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58254, SGI0, SGI0, SGI0, SGI58255, SGI58255, SGI0, SGI0, SGI0, SGI58256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12980[] = {SGI58257, SGI58258, SGI58259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58260, SGI0, SGI0, SGI0, SGI58261, SGI58261, SGI0, SGI58262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12981[] = {SGI58263, SGI0, SGI58264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12982[] = {SGI58265, SGI58266, SGI58267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58268, SGI0, SGI0, SGI0, SGI58269, SGI58269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12983[] = {SGI58270, SGI58271, SGI58272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58273, SGI58274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12984[] = {SGI58275, SGI58276, SGI58277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58278, SGI0, SGI0, SGI0, SGI58279, SGI58279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12985[] = {SGI58280, SGI58281, SGI58282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58283, SGI0, SGI0, SGI0, SGI58284, SGI58284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12986[] = {SGI58285, SGI58286, SGI58287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58288, SGI0, SGI0, SGI0, SGI58289, SGI58289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12987[] = {SGI58290, SGI58291, SGI58292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58293, SGI0, SGI0, SGI0, SGI58294, SGI58294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12988[] = {SGI58295, SGI58296, SGI58297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58298, SGI0, SGI0, SGI0, SGI58299, SGI58299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12989[] = {SGI58300, SGI58301, SGI58302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58303, SGI0, SGI0, SGI0, SGI58304, SGI58304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12990[] = {SGI58305, SGI58306, SGI58307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58308, SGI0, SGI0, SGI0, SGI58309, SGI58309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12991[] = {SGI58310, SGI0, SGI58311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12992[] = {SGI58312, SGI58313, SGI58314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58315, SGI0, SGI0, SGI0, SGI58316, SGI58316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12993[] = {SGI58317, SGI58318, SGI58319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58320, SGI0, SGI0, SGI0, SGI58321, SGI58321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12994[] = {SGI58322, SGI58323, SGI58324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58325, SGI58325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12995[] = {SGI58326, SGI58327, SGI58328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58329, SGI0, SGI0, SGI0, SGI58330, SGI58330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12996[] = {SGI58331, SGI0, SGI58332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58333, SGI0, SGI0, SGI0, SGI58334, SGI58334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12997[] = {SGI58335, SGI58336, SGI58337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58338, SGI0, SGI0, SGI0, SGI58339, SGI58339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12998[] = {SGI58340, SGI58341, SGI58342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58343, SGI0, SGI0, SGI0, SGI58344, SGI58344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG12999[] = {SGI58345, SGI58346, SGI58347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58348, SGI0, SGI0, SGI0, SGI58349, SGI58349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13000[] = {SGI58350, SGI58351, SGI58352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58353, SGI0, SGI0, SGI0, SGI58354, SGI58354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13001[] = {SGI58355, SGI0, SGI58356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58357, SGI0, SGI0, SGI0, SGI58358, SGI58358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13002[] = {SGI58359, SGI58360, SGI58361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58362, SGI0, SGI0, SGI0, SGI58363, SGI58363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13003[] = {SGI58364, SGI58365, SGI58366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58367, SGI0, SGI0, SGI0, SGI58368, SGI58368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13004[] = {SGI58369, SGI58370, SGI58371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58372, SGI0, SGI0, SGI0, SGI58373, SGI58373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13005[] = {SGI58374, SGI58375, SGI58376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58377, SGI0, SGI0, SGI0, SGI58378, SGI58378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13006[] = {SGI58379, SGI58380, SGI58381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58382, SGI0, SGI0, SGI0, SGI58383, SGI58383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13007[] = {SGI58384, SGI58385, SGI58387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58390, SGI0, SGI0, SGI0, SGI58392, SGI58392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13008[] = {SGI58386, SGI58388, SGI58389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58391, SGI0, SGI0, SGI0, SGI58393, SGI58393, SGI0, SGI0, SGI0, SGI58394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13009[] = {SGI58395, SGI58396, SGI58397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58399, SGI0, SGI0, SGI0, SGI58401, SGI58401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13010[] = {SGI58398, SGI0, SGI58400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58402, SGI0, SGI0, SGI0, SGI58403, SGI58403, SGI0, SGI0, SGI0, SGI58404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13011[] = {SGI58405, SGI58406, SGI58407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58408, SGI0, SGI0, SGI0, SGI58409, SGI58409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13012[] = {SGI58410, SGI58411, SGI58412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58413, SGI0, SGI0, SGI0, SGI58414, SGI58414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13013[] = {SGI58415, SGI58416, SGI58417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58418, SGI0, SGI0, SGI0, SGI58419, SGI58419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13014[] = {SGI58420, SGI58421, SGI58422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58423, SGI0, SGI0, SGI0, SGI58424, SGI58424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13015[] = {SGI0, SGI58425, SGI58426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13016[] = {SGI58427, SGI58428, SGI58429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58430, SGI0, SGI0, SGI0, SGI58431, SGI58431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13017[] = {SGI58432, SGI58433, SGI58434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58435, SGI0, SGI0, SGI0, SGI58436, SGI58436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13018[] = {SGI58437, SGI58438, SGI58439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58440, SGI0, SGI0, SGI0, SGI58441, SGI58441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13019[] = {SGI58442, SGI58443, SGI58444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58445, SGI0, SGI0, SGI0, SGI58446, SGI58446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13020[] = {SGI58447, SGI58448, SGI58449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58450, SGI0, SGI0, SGI0, SGI58451, SGI58451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13021[] = {SGI58452, SGI58453, SGI58454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58455, SGI0, SGI0, SGI0, SGI58456, SGI58456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13022[] = {SGI58457, SGI58458, SGI58459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58460, SGI0, SGI0, SGI0, SGI58461, SGI58461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13023[] = {SGI58462, SGI58463, SGI58464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58465, SGI0, SGI0, SGI0, SGI58466, SGI58466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13024[] = {SGI58467, SGI58468, SGI58469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58470, SGI0, SGI0, SGI0, SGI58471, SGI58471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13025[] = {SGI58472, SGI58473, SGI58474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58475, SGI0, SGI0, SGI0, SGI58476, SGI58476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13026[] = {SGI58477, SGI58478, SGI58479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58480, SGI0, SGI0, SGI0, SGI58481, SGI58481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13027[] = {SGI58482, SGI58483, SGI58484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58485, SGI0, SGI0, SGI0, SGI58486, SGI58486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13028[] = {SGI58487, SGI58488, SGI58489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58490, SGI0, SGI0, SGI0, SGI58491, SGI58491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13029[] = {SGI58492, SGI58493, SGI58494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58495, SGI0, SGI0, SGI0, SGI58496, SGI58496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13030[] = {SGI0, SGI0, SGI58497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58498, SGI58499, SGI0, SGI0, SGI58500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13031[] = {SGI58501, SGI58502, SGI58503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58504, SGI0, SGI0, SGI0, SGI58505, SGI58505, SGI0, SGI58506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13032[] = {SGI58507, SGI58508, SGI58509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58510, SGI0, SGI0, SGI0, SGI58511, SGI58511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13033[] = {SGI0, SGI0, SGI58512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58513, SGI58514, SGI0, SGI0, SGI58515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13034[] = {SGI58516, SGI0, SGI58517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58518, SGI0, SGI0, SGI0, SGI58519, SGI58519, SGI0, SGI0, SGI0, SGI58520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13035[] = {SGI0, SGI58521, SGI0, SGI58522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13036[] = {SGI58523, SGI0, SGI58524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58525, SGI0, SGI0, SGI0, SGI58526, SGI58526, SGI0, SGI0, SGI0, SGI58527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13037[] = {SGI58528, SGI58529, SGI58530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58531, SGI0, SGI0, SGI0, SGI58532, SGI58532, SGI58533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13038[] = {SGI58534, SGI58535, SGI58536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58537, SGI0, SGI0, SGI0, SGI58538, SGI58538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13039[] = {SGI58539, SGI58540, SGI58541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58542, SGI0, SGI0, SGI0, SGI58543, SGI58543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13040[] = {SGI58544, SGI58545, SGI58546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58547, SGI58548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13041[] = {SGI58549, SGI58550, SGI58551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58552, SGI0, SGI0, SGI0, SGI58553, SGI58553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13042[] = {SGI58554, SGI58555, SGI58556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58557, SGI0, SGI0, SGI0, SGI58558, SGI58558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13043[] = {SGI58559, SGI0, SGI58560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13044[] = {SGI58561, SGI0, SGI58562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13045[] = {SGI58563, SGI58564, SGI58565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58566, SGI0, SGI0, SGI0, SGI58567, SGI58567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13046[] = {SGI58568, SGI58569, SGI58570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58571, SGI58572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13047[] = {SGI58573, SGI0, SGI58574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13048[] = {SGI58575, SGI0, SGI58576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13049[] = {SGI58577, SGI58578, SGI58579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58580, SGI0, SGI0, SGI0, SGI58581, SGI58581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13050[] = {SGI58582, SGI0, SGI58583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58584, SGI0, SGI0, SGI0, SGI58585, SGI58585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13051[] = {SGI58586, SGI0, SGI58587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58588, SGI0, SGI0, SGI0, SGI58589, SGI58589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13052[] = {SGI58590, SGI58591, SGI58592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58593, SGI0, SGI0, SGI0, SGI58594, SGI58594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13053[] = {SGI58595, SGI58596, SGI58597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58598, SGI0, SGI0, SGI0, SGI58599, SGI58599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13054[] = {SGI58600, SGI58601, SGI58602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58603, SGI0, SGI0, SGI0, SGI58604, SGI58604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13055[] = {SGI58605, SGI58606, SGI58607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58608, SGI0, SGI0, SGI0, SGI58609, SGI58609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13056[] = {SGI58610, SGI58611, SGI58612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58613, SGI0, SGI0, SGI0, SGI58614, SGI58614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13057[] = {SGI58615, SGI0, SGI58616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13058[] = {SGI0, SGI58617, SGI0, SGI58618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13059[] = {SGI58619, SGI0, SGI58620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58621, SGI0, SGI0, SGI0, SGI58622, SGI58622, SGI0, SGI0, SGI0, SGI58623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13060[] = {SGI58624, SGI58625, SGI58626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58627, SGI0, SGI0, SGI0, SGI58628, SGI58628, SGI0, SGI58629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13061[] = {SGI58630, SGI58631, SGI58632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58633, SGI0, SGI0, SGI0, SGI58634, SGI58634, SGI0, SGI58635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13062[] = {SGI58636, SGI58637, SGI58638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58639, SGI0, SGI0, SGI0, SGI58640, SGI58640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13063[] = {SGI58641, SGI0, SGI58642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58643, SGI58644, SGI58644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58645, NULL};
const Segment * const SG13064[] = {SGI58646, SGI58647, SGI58648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58649, SGI0, SGI0, SGI0, SGI58650, SGI58650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13065[] = {SGI58651, SGI0, SGI58652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13066[] = {SGI58653, SGI58654, SGI58655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58656, SGI58656, SGI0, SGI0, SGI0, SGI58657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13067[] = {SGI58658, SGI58659, SGI58660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58661, SGI0, SGI0, SGI0, SGI58662, SGI58662, SGI58663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13068[] = {SGI58664, SGI58665, SGI58666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58667, SGI0, SGI0, SGI0, SGI58668, SGI58668, SGI0, SGI58669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13069[] = {SGI58670, SGI58671, SGI58672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58673, SGI0, SGI0, SGI0, SGI58674, SGI58674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13070[] = {SGI58675, SGI58676, SGI58677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58678, SGI0, SGI0, SGI0, SGI58679, SGI58679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13071[] = {SGI58681, SGI0, SGI58682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58683, SGI0, SGI0, SGI0, SGI58684, SGI58684, SGI0, SGI0, SGI0, SGI58685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13072[] = {SGI58686, SGI0, SGI58687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58688, SGI0, SGI0, SGI0, SGI58689, SGI58689, SGI0, SGI0, SGI0, SGI58690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13073[] = {SGI58691, SGI58692, SGI58693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58694, SGI0, SGI0, SGI0, SGI58695, SGI58695, SGI0, SGI0, SGI0, SGI0, SGI58696, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13074[] = {SGI58697, SGI58698, SGI58699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58700, SGI0, SGI0, SGI0, SGI58701, SGI58701, SGI0, SGI0, SGI58702, SGI0, SGI0, SGI0, SGI58703, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13075[] = {SGI0, SGI58704, SGI0, SGI58705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13076[] = {SGI58706, SGI58707, SGI58708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58709, SGI0, SGI0, SGI0, SGI58710, SGI58710, SGI58711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13077[] = {SGI58712, SGI0, SGI58713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58714, SGI0, SGI0, SGI0, SGI58715, SGI58715, SGI0, SGI0, SGI0, SGI58716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13078[] = {SGI58717, SGI58718, SGI58719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58720, SGI0, SGI0, SGI0, SGI58721, SGI58721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58722, SGI0, SGI0, NULL};
const Segment * const SG13079[] = {SGI58723, SGI0, SGI58724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58725, SGI0, SGI0, SGI0, SGI58726, SGI58726, SGI0, SGI0, SGI0, SGI58727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13080[] = {SGI0, SGI58728, SGI0, SGI58729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13081[] = {SGI58730, SGI58731, SGI58732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58733, SGI0, SGI0, SGI0, SGI58734, SGI58734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13082[] = {SGI58735, SGI58736, SGI58737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58738, SGI58738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13083[] = {SGI58739, SGI58740, SGI58741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58742, SGI0, SGI0, SGI0, SGI58743, SGI58743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13084[] = {SGI58744, SGI0, SGI58745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13085[] = {SGI58746, SGI58747, SGI58748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58749, SGI58750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13086[] = {SGI58751, SGI58752, SGI58753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58754, SGI0, SGI0, SGI0, SGI58755, SGI58755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13087[] = {SGI58756, SGI58757, SGI58758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58759, SGI0, SGI0, SGI0, SGI58760, SGI58760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13088[] = {SGI58761, SGI58762, SGI58763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58764, SGI58764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13089[] = {SGI58765, SGI58766, SGI58767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58768, SGI0, SGI0, SGI0, SGI58769, SGI58769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13090[] = {SGI58770, SGI0, SGI58771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58775, SGI58777, SGI58777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58779, NULL};
const Segment * const SG13091[] = {SGI58772, SGI58773, SGI58774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58776, SGI0, SGI0, SGI0, SGI58778, SGI58778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13092[] = {SGI58780, SGI58781, SGI58782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58783, SGI0, SGI0, SGI0, SGI58784, SGI58784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13093[] = {SGI58785, SGI0, SGI58786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58789, SGI58791, SGI58791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58792, NULL};
const Segment * const SG13094[] = {SGI58787, SGI58788, SGI58790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58793, SGI0, SGI0, SGI0, SGI58794, SGI58794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13095[] = {SGI58795, SGI0, SGI58796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58797, SGI0, SGI0, SGI0, SGI58798, SGI58798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13096[] = {SGI58799, SGI58800, SGI58801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58802, SGI0, SGI0, SGI0, SGI58803, SGI58803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13097[] = {SGI58804, SGI58805, SGI58806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58807, SGI0, SGI0, SGI0, SGI58808, SGI58808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13098[] = {SGI58809, SGI58810, SGI58811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58812, SGI0, SGI0, SGI0, SGI58813, SGI58813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13099[] = {SGI58814, SGI58815, SGI58816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58817, SGI0, SGI0, SGI0, SGI58818, SGI58818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13100[] = {SGI58819, SGI0, SGI58820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13101[] = {SGI58821, SGI58822, SGI58823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58824, SGI0, SGI0, SGI0, SGI58825, SGI58825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13102[] = {SGI58826, SGI58827, SGI58828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58829, SGI58830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13103[] = {SGI58831, SGI58832, SGI58833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58834, SGI0, SGI0, SGI0, SGI58835, SGI58835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13104[] = {SGI58836, SGI58837, SGI58838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58839, SGI58839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13105[] = {SGI58840, SGI58841, SGI58842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58843, SGI0, SGI0, SGI0, SGI58844, SGI58844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13106[] = {SGI58845, SGI58846, SGI58847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58848, SGI0, SGI0, SGI0, SGI58849, SGI58849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13107[] = {SGI58850, SGI58851, SGI58852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58853, SGI0, SGI0, SGI0, SGI58854, SGI58854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13108[] = {SGI58855, SGI58856, SGI58857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58858, SGI0, SGI0, SGI0, SGI58859, SGI58859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13109[] = {SGI58860, SGI58861, SGI58862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58863, SGI0, SGI0, SGI0, SGI58864, SGI58864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13110[] = {SGI58865, SGI0, SGI58866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58867, SGI0, SGI0, SGI0, SGI58868, SGI58868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13111[] = {SGI58869, SGI58870, SGI58871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58872, SGI0, SGI0, SGI0, SGI58873, SGI58873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13112[] = {SGI58874, SGI58875, SGI58876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58877, SGI0, SGI0, SGI0, SGI58878, SGI58878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13113[] = {SGI58879, SGI58880, SGI58881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58882, SGI0, SGI0, SGI0, SGI58883, SGI58883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13114[] = {SGI58884, SGI58885, SGI58886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58887, SGI0, SGI0, SGI0, SGI58888, SGI58888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13115[] = {SGI58889, SGI58890, SGI58891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58892, SGI0, SGI0, SGI0, SGI58893, SGI58893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13116[] = {SGI58894, SGI58895, SGI58896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58897, SGI0, SGI0, SGI0, SGI58898, SGI58898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13117[] = {SGI58899, SGI58900, SGI58901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58902, SGI0, SGI0, SGI0, SGI58903, SGI58903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13118[] = {SGI58904, SGI58905, SGI58906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58907, SGI0, SGI0, SGI0, SGI58908, SGI58908, SGI0, SGI0, NULL};
const Segment * const SG13119[] = {SGI58909, SGI58910, SGI58911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58912, SGI0, SGI0, SGI0, SGI58913, SGI58913, SGI0, SGI0, NULL};
const Segment * const SG13120[] = {SGI58914, SGI58915, SGI58916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58917, SGI0, SGI0, SGI0, SGI58918, SGI58918, SGI0, SGI0, NULL};
const Segment * const SG13121[] = {SGI58919, SGI58920, SGI58921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58922, SGI0, SGI0, SGI0, SGI58923, SGI58923, SGI0, SGI0, NULL};
const Segment * const SG13122[] = {SGI58924, SGI58925, SGI58926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58927, SGI0, SGI0, SGI0, SGI58928, SGI58928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13123[] = {SGI58929, SGI58930, SGI58931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58932, SGI0, SGI0, SGI0, SGI58933, SGI58933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13124[] = {SGI58934, SGI58935, SGI58936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58937, SGI58937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13125[] = {SGI58938, SGI58939, SGI58940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58941, SGI0, SGI0, SGI0, SGI58942, SGI58942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13126[] = {SGI58943, SGI58944, SGI58945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58946, SGI0, SGI0, SGI0, SGI58947, SGI58947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13127[] = {SGI58948, SGI58949, SGI58950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58951, SGI0, SGI0, SGI0, SGI58952, SGI58952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13128[] = {SGI58953, SGI58954, SGI58955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58956, SGI0, SGI0, SGI0, SGI58957, SGI58957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13129[] = {SGI58958, SGI58959, SGI58960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58961, SGI0, SGI0, SGI0, SGI58962, SGI58962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13130[] = {SGI58963, SGI58964, SGI58965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58966, SGI0, SGI0, SGI0, SGI58967, SGI58967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13131[] = {SGI58968, SGI58969, SGI58970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58971, SGI0, SGI0, SGI0, SGI58972, SGI58972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13132[] = {SGI58973, SGI58974, SGI58975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58976, SGI0, SGI0, SGI0, SGI58977, SGI58977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13133[] = {SGI58978, SGI58979, SGI58980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58981, SGI0, SGI0, SGI0, SGI58982, SGI58982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13134[] = {SGI58983, SGI58984, SGI58985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58986, SGI0, SGI0, SGI0, SGI58987, SGI58987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13135[] = {SGI58988, SGI58989, SGI58990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58991, SGI58992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13136[] = {SGI58993, SGI58994, SGI58995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI58996, SGI0, SGI0, SGI0, SGI58997, SGI58997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13137[] = {SGI58998, SGI58999, SGI59000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59001, SGI0, SGI0, SGI0, SGI59002, SGI59002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13138[] = {SGI59003, SGI59004, SGI59005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59006, SGI0, SGI0, SGI0, SGI59007, SGI59007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13139[] = {SGI59008, SGI59009, SGI59010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59011, SGI0, SGI0, SGI0, SGI59012, SGI59012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13140[] = {SGI59013, SGI59014, SGI59015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59016, SGI0, SGI0, SGI0, SGI59017, SGI59017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13141[] = {SGI59018, SGI59019, SGI59020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59021, SGI0, SGI0, SGI0, SGI59022, SGI59022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13142[] = {SGI59023, SGI59024, SGI59025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59026, SGI0, SGI0, SGI0, SGI59027, SGI59027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13143[] = {SGI59028, SGI59029, SGI59030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59031, SGI0, SGI0, SGI0, SGI59032, SGI59032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13144[] = {SGI59033, SGI59034, SGI59035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59036, SGI0, SGI0, SGI0, SGI59037, SGI59037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13145[] = {SGI59038, SGI59039, SGI59040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59041, SGI0, SGI0, SGI0, SGI59042, SGI59042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13146[] = {SGI59043, SGI59044, SGI59045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59046, SGI0, SGI0, SGI0, SGI59047, SGI59047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13147[] = {SGI59048, SGI59049, SGI59050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59051, SGI0, SGI0, SGI0, SGI59052, SGI59052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13148[] = {SGI59053, SGI59054, SGI59055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59056, SGI0, SGI0, SGI0, SGI59057, SGI59057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13149[] = {SGI59058, SGI59059, SGI59060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59061, SGI0, SGI0, SGI0, SGI59062, SGI59062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13150[] = {SGI59063, SGI59064, SGI59065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59066, SGI0, SGI0, SGI0, SGI59067, SGI59067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13151[] = {SGI59068, SGI59069, SGI59070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59071, SGI0, SGI0, SGI0, SGI59072, SGI59072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13152[] = {SGI59073, SGI59074, SGI59075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59076, SGI0, SGI0, SGI0, SGI59077, SGI59077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13153[] = {SGI59078, SGI59079, SGI59080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59081, SGI0, SGI0, SGI0, SGI59082, SGI59082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13154[] = {SGI59083, SGI59084, SGI59085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59086, SGI0, SGI0, SGI0, SGI59087, SGI59087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13155[] = {SGI59088, SGI59089, SGI59090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59091, SGI0, SGI0, SGI0, SGI59092, SGI59092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13156[] = {SGI59093, SGI59094, SGI59095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59096, SGI0, SGI0, SGI0, SGI59097, SGI59097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13157[] = {SGI59098, SGI59099, SGI59100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59101, SGI0, SGI0, SGI0, SGI59102, SGI59102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13158[] = {SGI59103, SGI59104, SGI59105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59106, SGI0, SGI0, SGI0, SGI59107, SGI59107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13159[] = {SGI59108, SGI59109, SGI59110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59111, SGI0, SGI0, SGI0, SGI59112, SGI59112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13160[] = {SGI59113, SGI59114, SGI59115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59116, SGI0, SGI0, SGI0, SGI59117, SGI59117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13161[] = {SGI59118, SGI59119, SGI59120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59121, SGI0, SGI0, SGI0, SGI59122, SGI59122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13162[] = {SGI59123, SGI59124, SGI59125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59126, SGI0, SGI0, SGI0, SGI59127, SGI59127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13163[] = {SGI59128, SGI59129, SGI59130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59131, SGI0, SGI0, SGI0, SGI59132, SGI59132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13164[] = {SGI59133, SGI59134, SGI59135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59136, SGI0, SGI0, SGI0, SGI59137, SGI59137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13165[] = {SGI59138, SGI59139, SGI59140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59141, SGI0, SGI0, SGI0, SGI59142, SGI59142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13166[] = {SGI59143, SGI59144, SGI59145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59146, SGI0, SGI0, SGI0, SGI59147, SGI59147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13167[] = {SGI59148, SGI59149, SGI59150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59151, SGI0, SGI0, SGI0, SGI59152, SGI59152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13168[] = {SGI59153, SGI59154, SGI59155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59156, SGI0, SGI0, SGI0, SGI59157, SGI59157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13169[] = {SGI59158, SGI59159, SGI59160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59161, SGI0, SGI0, SGI0, SGI59162, SGI59162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13170[] = {SGI59163, SGI59164, SGI59165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59166, SGI0, SGI0, SGI0, SGI59167, SGI59167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13171[] = {SGI59168, SGI59169, SGI59170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59171, SGI0, SGI0, SGI0, SGI59172, SGI59172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13172[] = {SGI59173, SGI59174, SGI59175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59176, SGI0, SGI0, SGI0, SGI59177, SGI59177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13173[] = {SGI59178, SGI59179, SGI59180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59181, SGI0, SGI0, SGI0, SGI59182, SGI59182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13174[] = {SGI59183, SGI59184, SGI59185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59186, SGI0, SGI0, SGI0, SGI59187, SGI59187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13175[] = {SGI59188, SGI59189, SGI59190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59191, SGI0, SGI0, SGI0, SGI59192, SGI59192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13176[] = {SGI59193, SGI59195, SGI59196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59200, SGI59201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13177[] = {SGI59194, SGI59197, SGI59198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59199, SGI0, SGI0, SGI0, SGI59202, SGI59202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13178[] = {SGI59203, SGI59204, SGI59205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59206, SGI0, SGI0, SGI0, SGI59207, SGI59207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13179[] = {SGI59208, SGI59209, SGI59210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59211, SGI0, SGI0, SGI0, SGI59212, SGI59212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13180[] = {SGI59213, SGI59214, SGI59215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59216, SGI0, SGI0, SGI0, SGI59217, SGI59217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13181[] = {SGI59218, SGI59219, SGI59220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59221, SGI0, SGI0, SGI0, SGI59222, SGI59222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13182[] = {SGI59223, SGI59224, SGI59225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59226, SGI0, SGI0, SGI0, SGI59227, SGI59227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13183[] = {SGI59228, SGI59229, SGI59230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59231, SGI0, SGI0, SGI0, SGI59232, SGI59232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13184[] = {SGI59233, SGI59234, SGI59235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59236, SGI0, SGI0, SGI0, SGI59237, SGI59237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13185[] = {SGI59238, SGI59239, SGI59240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59241, SGI0, SGI0, SGI0, SGI59242, SGI59242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13186[] = {SGI59243, SGI59244, SGI59245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59246, SGI0, SGI0, SGI0, SGI59247, SGI59247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13187[] = {SGI59248, SGI59249, SGI59250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59251, SGI0, SGI0, SGI0, SGI59252, SGI59252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13188[] = {SGI59253, SGI59254, SGI59255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59256, SGI59256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13189[] = {SGI59257, SGI59258, SGI59259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59260, SGI0, SGI0, SGI0, SGI59261, SGI59261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13190[] = {SGI59262, SGI59263, SGI59264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59265, SGI0, SGI0, SGI0, SGI59266, SGI59266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13191[] = {SGI59267, SGI59268, SGI59269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59270, SGI0, SGI0, SGI0, SGI59271, SGI59271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13192[] = {SGI59272, SGI59273, SGI59274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59275, SGI0, SGI0, SGI0, SGI59277, SGI59277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13193[] = {SGI59276, SGI59278, SGI59279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59280, SGI0, SGI0, SGI0, SGI59281, SGI59281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13194[] = {SGI59282, SGI59283, SGI59284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59285, SGI0, SGI0, SGI0, SGI59286, SGI59286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13195[] = {SGI59287, SGI59288, SGI59289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59290, SGI0, SGI0, SGI0, SGI59291, SGI59291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13196[] = {SGI59292, SGI0, SGI59293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13197[] = {SGI59294, SGI59295, SGI59296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59297, SGI0, SGI0, SGI0, SGI59298, SGI59298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13198[] = {SGI59299, SGI59300, SGI59301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59302, SGI0, SGI0, SGI0, SGI59303, SGI59303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13199[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13200[] = {SGI59304, SGI59305, SGI59306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59308, SGI0, SGI0, SGI0, SGI59309, SGI59309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13201[] = {SGI59310, SGI59311, SGI59312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59313, SGI0, SGI0, SGI0, SGI59314, SGI59314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13202[] = {SGI59315, SGI59316, SGI59317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59318, SGI0, SGI0, SGI0, SGI59319, SGI59319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13203[] = {SGI59320, SGI59321, SGI59322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59323, SGI0, SGI0, SGI0, SGI59324, SGI59324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13204[] = {SGI59325, SGI59326, SGI59327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59328, SGI59328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13205[] = {SGI59329, SGI59330, SGI59331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59332, SGI0, SGI0, SGI0, SGI59333, SGI59333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13206[] = {SGI59334, SGI59335, SGI59336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59337, SGI0, SGI0, SGI0, SGI59338, SGI59338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13207[] = {SGI59339, SGI59340, SGI59341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59342, SGI0, SGI0, SGI0, SGI59343, SGI59343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13208[] = {SGI59344, SGI59345, SGI59346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59347, SGI0, SGI0, SGI0, SGI59348, SGI59348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13209[] = {SGI59349, SGI59350, SGI59351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59352, SGI0, SGI0, SGI0, SGI59353, SGI59353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13210[] = {SGI59354, SGI59355, SGI59356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59357, SGI0, SGI0, SGI0, SGI59358, SGI59358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13211[] = {SGI59359, SGI59360, SGI59361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59362, SGI0, SGI0, SGI0, SGI59363, SGI59363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13212[] = {SGI59364, SGI59365, SGI59366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59367, SGI0, SGI0, SGI0, SGI59368, SGI59368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13213[] = {SGI59369, SGI59370, SGI59371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59372, SGI0, SGI0, SGI0, SGI59373, SGI59373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13214[] = {SGI59374, SGI59375, SGI59376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59377, SGI0, SGI0, SGI0, SGI59378, SGI59378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13215[] = {SGI59379, SGI59380, SGI59381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59382, SGI0, SGI0, SGI0, SGI59384, SGI59384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13216[] = {SGI59383, SGI59385, SGI59386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59387, SGI0, SGI0, SGI0, SGI59388, SGI59388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13217[] = {SGI59389, SGI59390, SGI59391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59392, SGI0, SGI0, SGI0, SGI59393, SGI59393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13218[] = {SGI59394, SGI59395, SGI59396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59397, SGI0, SGI0, SGI0, SGI59398, SGI59398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13219[] = {SGI59399, SGI59400, SGI59401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59402, SGI0, SGI0, SGI0, SGI59403, SGI59403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13220[] = {SGI59404, SGI59405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59406, SGI0, SGI0, SGI0, SGI0, SGI59407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13221[] = {SGI59408, SGI59409, SGI59410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59411, SGI0, SGI0, SGI0, SGI59412, SGI59412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13222[] = {SGI59413, SGI59414, SGI59415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59416, SGI0, SGI0, SGI0, SGI59417, SGI59417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13223[] = {SGI59418, SGI59419, SGI59420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59421, SGI0, SGI0, SGI0, SGI59422, SGI59422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13224[] = {SGI59423, SGI59424, SGI59425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59426, SGI0, SGI0, SGI0, SGI59427, SGI59427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13225[] = {SGI59428, SGI59429, SGI59430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59431, SGI0, SGI0, SGI0, SGI59432, SGI59432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13226[] = {SGI59433, SGI59434, SGI59435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59436, SGI0, SGI0, SGI0, SGI59437, SGI59437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13227[] = {SGI59438, SGI59439, SGI59440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59441, SGI59441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13228[] = {SGI59442, SGI59443, SGI59444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59445, SGI0, SGI0, SGI0, SGI59446, SGI59446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13229[] = {SGI59447, SGI59448, SGI59449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59450, SGI0, SGI0, SGI0, SGI59451, SGI59451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13230[] = {SGI59452, SGI59453, SGI59454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59455, SGI0, SGI0, SGI0, SGI59456, SGI59456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13231[] = {SGI59457, SGI59458, SGI59459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59460, SGI0, SGI0, SGI0, SGI59461, SGI59461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13232[] = {SGI59462, SGI59463, SGI59464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59465, SGI0, SGI0, SGI0, SGI59466, SGI59466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13233[] = {SGI59467, SGI59468, SGI59469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59470, SGI0, SGI0, SGI0, SGI59471, SGI59471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13234[] = {SGI59472, SGI59473, SGI59474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59475, SGI0, SGI0, SGI0, SGI59476, SGI59476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13235[] = {SGI59477, SGI59478, SGI59479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59480, SGI0, SGI0, SGI0, SGI59481, SGI59481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13236[] = {SGI59482, SGI59483, SGI59484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59485, SGI0, SGI0, SGI0, SGI59486, SGI59486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13237[] = {SGI59487, SGI59488, SGI59489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59490, SGI0, SGI0, SGI0, SGI59491, SGI59491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13238[] = {SGI59492, SGI59493, SGI59494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59495, SGI0, SGI0, SGI0, SGI59496, SGI59496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13239[] = {SGI59497, SGI59498, SGI59499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59500, SGI0, SGI0, SGI0, SGI59501, SGI59501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13240[] = {SGI59502, SGI59503, SGI59504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59505, SGI0, SGI0, SGI0, SGI59506, SGI59506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13241[] = {SGI59507, SGI59508, SGI59509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59510, SGI0, SGI0, SGI0, SGI59511, SGI59511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13242[] = {SGI59512, SGI59513, SGI59514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59515, SGI0, SGI0, SGI0, SGI59516, SGI59516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13243[] = {SGI59517, SGI59518, SGI59519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59520, SGI0, SGI0, SGI0, SGI59521, SGI59521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13244[] = {SGI59522, SGI59523, SGI59524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59525, SGI0, SGI0, SGI0, SGI59526, SGI59526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13245[] = {SGI59527, SGI59528, SGI59529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59530, SGI0, SGI0, SGI0, SGI59531, SGI59531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13246[] = {SGI59532, SGI59533, SGI59534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59535, SGI0, SGI0, SGI0, SGI59536, SGI59536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13247[] = {SGI59537, SGI59538, SGI59539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59540, SGI0, SGI0, SGI0, SGI59541, SGI59541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13248[] = {SGI59542, SGI59543, SGI59544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59545, SGI0, SGI0, SGI0, SGI59546, SGI59546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13249[] = {SGI59547, SGI59548, SGI59549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59550, SGI0, SGI0, SGI0, SGI59551, SGI59551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13250[] = {SGI59552, SGI59553, SGI59554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59555, SGI0, SGI0, SGI0, SGI59556, SGI59556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13251[] = {SGI0, SGI59557, SGI59558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59559, SGI0, SGI0, SGI0, SGI59560, SGI59560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13252[] = {SGI59561, SGI59562, SGI59563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59564, SGI0, SGI0, SGI0, SGI59565, SGI59565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13253[] = {SGI59566, SGI59567, SGI59568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59569, SGI0, SGI0, SGI0, SGI59570, SGI59570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13254[] = {SGI59571, SGI59572, SGI59573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59574, SGI0, SGI0, SGI0, SGI59575, SGI59575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13255[] = {SGI59576, SGI59577, SGI59578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59579, SGI0, SGI0, SGI0, SGI59580, SGI59580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13256[] = {SGI59581, SGI59582, SGI59583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59584, SGI0, SGI0, SGI0, SGI59585, SGI59585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13257[] = {SGI59586, SGI59587, SGI59588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59589, SGI0, SGI0, SGI0, SGI59590, SGI59590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13258[] = {SGI59591, SGI59592, SGI59593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59594, SGI0, SGI0, SGI0, SGI59595, SGI59595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13259[] = {SGI59596, SGI59597, SGI59598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59599, SGI59599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13260[] = {SGI59600, SGI59601, SGI59602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59603, SGI0, SGI0, SGI0, SGI59604, SGI59604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13261[] = {SGI59605, SGI59606, SGI59607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59608, SGI0, SGI0, SGI0, SGI59609, SGI59609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13262[] = {SGI59610, SGI59611, SGI59612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59613, SGI0, SGI0, SGI0, SGI59614, SGI59614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13263[] = {SGI59615, SGI59616, SGI59617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59618, SGI0, SGI0, SGI0, SGI59619, SGI59619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13264[] = {SGI59620, SGI59621, SGI59622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59623, SGI0, SGI0, SGI0, SGI59624, SGI59624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13265[] = {SGI59625, SGI59626, SGI59627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59628, SGI0, SGI0, SGI0, SGI59629, SGI59629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13266[] = {SGI59630, SGI59631, SGI59632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59633, SGI0, SGI0, SGI0, SGI59634, SGI59634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13267[] = {SGI59635, SGI59636, SGI59637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59638, SGI0, SGI0, SGI0, SGI59639, SGI59639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13268[] = {SGI59640, SGI59641, SGI59642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59643, SGI0, SGI0, SGI0, SGI59644, SGI59644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13269[] = {SGI59645, SGI59646, SGI59647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59648, SGI0, SGI0, SGI0, SGI59649, SGI59649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13270[] = {SGI59650, SGI59651, SGI59652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59653, SGI0, SGI0, SGI0, SGI59654, SGI59654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13271[] = {SGI59655, SGI59656, SGI59657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59658, SGI0, SGI0, SGI0, SGI59659, SGI59659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13272[] = {SGI59660, SGI59661, SGI59662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59663, SGI0, SGI0, SGI0, SGI59664, SGI59664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13273[] = {SGI59665, SGI59666, SGI59667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59668, SGI0, SGI0, SGI0, SGI59669, SGI59669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13274[] = {SGI59670, SGI59671, SGI59672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59673, SGI0, SGI0, SGI0, SGI59674, SGI59674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13275[] = {SGI59675, SGI59676, SGI59677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59678, SGI0, SGI0, SGI0, SGI59679, SGI59679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13276[] = {SGI59680, SGI59681, SGI59682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59683, SGI0, SGI0, SGI0, SGI59684, SGI59684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13277[] = {SGI59685, SGI59686, SGI59687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59688, SGI0, SGI0, SGI0, SGI59689, SGI59689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13278[] = {SGI59690, SGI59691, SGI59692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59693, SGI0, SGI0, SGI0, SGI59694, SGI59694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13279[] = {SGI59695, SGI59696, SGI59697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59698, SGI0, SGI0, SGI0, SGI59699, SGI59699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13280[] = {SGI59700, SGI59701, SGI59702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59703, SGI0, SGI0, SGI0, SGI59704, SGI59704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13281[] = {SGI59705, SGI59706, SGI59707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59708, SGI0, SGI0, SGI0, SGI59709, SGI59709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13282[] = {SGI59710, SGI59711, SGI59712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59713, SGI0, SGI0, SGI0, SGI59714, SGI59714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13283[] = {SGI59715, SGI59716, SGI59717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59718, SGI0, SGI0, SGI0, SGI59719, SGI59719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13284[] = {SGI59720, SGI59721, SGI59722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59723, SGI59723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13285[] = {SGI59724, SGI59725, SGI59726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59727, SGI0, SGI0, SGI0, SGI59728, SGI59728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13286[] = {SGI59729, SGI59730, SGI59731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59732, SGI0, SGI0, SGI0, SGI59733, SGI59733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13287[] = {SGI59734, SGI59735, SGI59736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59737, SGI0, SGI0, SGI0, SGI59738, SGI59738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13288[] = {SGI59739, SGI59740, SGI59741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59742, SGI0, SGI0, SGI0, SGI59743, SGI59743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13289[] = {SGI59744, SGI59745, SGI59746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59747, SGI0, SGI0, SGI0, SGI59748, SGI59748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13290[] = {SGI59749, SGI59750, SGI59751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59752, SGI0, SGI0, SGI0, SGI59753, SGI59753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13291[] = {SGI59754, SGI59755, SGI59756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59757, SGI0, SGI0, SGI0, SGI59758, SGI59758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13292[] = {SGI59759, SGI59760, SGI59761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59762, SGI0, SGI0, SGI0, SGI59763, SGI59763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13293[] = {SGI59764, SGI59765, SGI59766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59767, SGI0, SGI0, SGI0, SGI59768, SGI59768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13294[] = {SGI59769, SGI59770, SGI59771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59772, SGI0, SGI0, SGI0, SGI59773, SGI59773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13295[] = {SGI59774, SGI0, SGI59775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13296[] = {SGI59776, SGI59777, SGI59778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59779, SGI0, SGI0, SGI0, SGI59780, SGI59780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13297[] = {SGI59781, SGI59782, SGI59783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59784, SGI0, SGI0, SGI0, SGI59785, SGI59785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13298[] = {SGI59786, SGI59787, SGI59788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59789, SGI0, SGI0, SGI0, SGI59790, SGI59790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13299[] = {SGI59791, SGI59792, SGI59793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59794, SGI0, SGI0, SGI0, SGI59795, SGI59795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13300[] = {SGI59796, SGI59797, SGI59798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59799, SGI0, SGI0, SGI0, SGI59800, SGI59800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13301[] = {SGI59801, SGI59802, SGI59803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59804, SGI0, SGI0, SGI0, SGI59805, SGI59805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13302[] = {SGI59806, SGI59807, SGI59808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59809, SGI0, SGI0, SGI0, SGI59810, SGI59810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13303[] = {SGI59811, SGI59812, SGI59813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59814, SGI0, SGI0, SGI0, SGI59815, SGI59815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13304[] = {SGI59816, SGI59817, SGI59818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59819, SGI0, SGI0, SGI0, SGI59820, SGI59820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13305[] = {SGI59821, SGI59822, SGI59823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59824, SGI0, SGI0, SGI0, SGI59825, SGI59825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13306[] = {SGI59826, SGI59827, SGI59828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59829, SGI0, SGI0, SGI0, SGI59830, SGI59830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13307[] = {SGI59831, SGI59832, SGI59833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59834, SGI0, SGI0, SGI0, SGI59835, SGI59835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13308[] = {SGI59836, SGI59837, SGI59838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59839, SGI0, SGI0, SGI0, SGI59840, SGI59840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13309[] = {SGI59841, SGI59842, SGI59843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59844, SGI0, SGI0, SGI0, SGI59845, SGI59845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13310[] = {SGI59846, SGI59847, SGI59848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59849, SGI0, SGI0, SGI0, SGI59850, SGI59850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13311[] = {SGI59851, SGI59852, SGI59853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59854, SGI0, SGI0, SGI0, SGI59855, SGI59855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13312[] = {SGI59856, SGI59857, SGI59858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59859, SGI0, SGI0, SGI0, SGI59860, SGI59860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13313[] = {SGI59861, SGI59862, SGI59863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59864, SGI0, SGI0, SGI0, SGI59865, SGI59865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13314[] = {SGI59866, SGI59867, SGI59868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59869, SGI0, SGI0, SGI0, SGI59870, SGI59870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13315[] = {SGI59871, SGI59872, SGI59873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59874, SGI59874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13316[] = {SGI59875, SGI59876, SGI59877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59878, SGI0, SGI0, SGI0, SGI59879, SGI59879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13317[] = {SGI59880, SGI0, SGI59881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13318[] = {SGI59882, SGI59883, SGI59884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59885, SGI0, SGI0, SGI0, SGI59886, SGI59886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13319[] = {SGI59887, SGI59888, SGI59889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59890, SGI0, SGI0, SGI0, SGI59891, SGI59891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13320[] = {SGI59892, SGI59893, SGI59894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59895, SGI0, SGI0, SGI0, SGI59896, SGI59896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13321[] = {SGI59897, SGI59898, SGI59899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59900, SGI0, SGI0, SGI0, SGI59901, SGI59901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13322[] = {SGI59902, SGI59903, SGI59904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59905, SGI0, SGI0, SGI0, SGI59906, SGI59906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13323[] = {SGI59907, SGI59908, SGI59909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59910, SGI0, SGI0, SGI0, SGI59911, SGI59911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13324[] = {SGI59912, SGI59913, SGI59914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59915, SGI0, SGI0, SGI0, SGI59916, SGI59916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13325[] = {SGI59917, SGI59918, SGI59919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59920, SGI0, SGI0, SGI0, SGI59921, SGI59921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13326[] = {SGI59922, SGI59923, SGI59924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59925, SGI0, SGI0, SGI0, SGI59926, SGI59926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13327[] = {SGI59927, SGI59928, SGI59929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59930, SGI0, SGI0, SGI0, SGI59931, SGI59931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13328[] = {SGI59932, SGI59933, SGI59934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59935, SGI0, SGI0, SGI0, SGI59936, SGI59936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13329[] = {SGI59937, SGI59938, SGI59939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59940, SGI0, SGI0, SGI0, SGI59941, SGI59941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13330[] = {SGI59942, SGI59943, SGI59944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59945, SGI0, SGI0, SGI0, SGI59946, SGI59946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13331[] = {SGI59947, SGI59948, SGI59949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59950, SGI0, SGI0, SGI0, SGI59951, SGI59951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13332[] = {SGI59952, SGI59953, SGI59954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59955, SGI0, SGI0, SGI0, SGI59956, SGI59956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13333[] = {SGI59957, SGI59958, SGI59959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59960, SGI0, SGI0, SGI0, SGI59961, SGI59961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13334[] = {SGI59962, SGI59963, SGI59964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59965, SGI0, SGI0, SGI0, SGI59966, SGI59966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13335[] = {SGI59967, SGI59968, SGI59969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59970, SGI0, SGI0, SGI0, SGI59971, SGI59971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13336[] = {SGI59972, SGI59973, SGI59974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59975, SGI0, SGI0, SGI0, SGI59976, SGI59976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13337[] = {SGI59977, SGI59978, SGI59979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59980, SGI0, SGI0, SGI0, SGI59981, SGI59981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13338[] = {SGI59982, SGI59983, SGI59984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59985, SGI0, SGI0, SGI0, SGI59986, SGI59986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13339[] = {SGI59987, SGI59988, SGI59989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59990, SGI0, SGI0, SGI0, SGI59991, SGI59991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13340[] = {SGI59992, SGI59993, SGI59994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI59995, SGI0, SGI0, SGI0, SGI59996, SGI59996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13341[] = {SGI59997, SGI59998, SGI59999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60000, SGI0, SGI0, SGI0, SGI60001, SGI60001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13342[] = {SGI60002, SGI60003, SGI60004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60005, SGI0, SGI0, SGI0, SGI60006, SGI60006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13343[] = {SGI60007, SGI60008, SGI60009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60010, SGI0, SGI0, SGI0, SGI60011, SGI60011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13344[] = {SGI60012, SGI60013, SGI60014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60015, SGI0, SGI0, SGI0, SGI60016, SGI60016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13345[] = {SGI60017, SGI60018, SGI60019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60020, SGI0, SGI0, SGI0, SGI60021, SGI60021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13346[] = {SGI60022, SGI60023, SGI60024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60025, SGI0, SGI0, SGI0, SGI60026, SGI60026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13347[] = {SGI60027, SGI60028, SGI60029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60030, SGI0, SGI0, SGI0, SGI60031, SGI60031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13348[] = {SGI60032, SGI60033, SGI60034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60035, SGI0, SGI0, SGI0, SGI60036, SGI60036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13349[] = {SGI60037, SGI60038, SGI60039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60040, SGI0, SGI0, SGI0, SGI60041, SGI60041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13350[] = {SGI60042, SGI60043, SGI60044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60045, SGI0, SGI0, SGI0, SGI60046, SGI60046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13351[] = {SGI60047, SGI60048, SGI60049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60050, SGI0, SGI0, SGI0, SGI60051, SGI60051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13352[] = {SGI60052, SGI60053, SGI60054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60055, SGI0, SGI0, SGI0, SGI60056, SGI60056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13353[] = {SGI60057, SGI60058, SGI60059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60060, SGI0, SGI0, SGI0, SGI60061, SGI60061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13354[] = {SGI60062, SGI60063, SGI60064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60065, SGI0, SGI0, SGI0, SGI60066, SGI60066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13355[] = {SGI60067, SGI60068, SGI60069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60070, SGI60071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13356[] = {SGI60072, SGI60073, SGI60074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60075, SGI0, SGI0, SGI0, SGI60076, SGI60076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13357[] = {SGI60077, SGI60078, SGI60079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60080, SGI0, SGI0, SGI0, SGI60081, SGI60081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13358[] = {SGI60082, SGI60083, SGI60084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60085, SGI0, SGI0, SGI0, SGI60086, SGI60086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13359[] = {SGI60087, SGI60088, SGI60089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60090, SGI0, SGI0, SGI0, SGI60091, SGI60091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13360[] = {SGI60092, SGI60093, SGI60094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60095, SGI0, SGI0, SGI0, SGI60096, SGI60096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13361[] = {SGI60097, SGI60098, SGI60099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60100, SGI60100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13362[] = {SGI60101, SGI60102, SGI60103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60104, SGI60105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13363[] = {SGI60106, SGI60107, SGI60108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60109, SGI60109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13364[] = {SGI60110, SGI60111, SGI60112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60113, SGI0, SGI0, SGI0, SGI60114, SGI60114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13365[] = {SGI60115, SGI60116, SGI60117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60118, SGI60119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13366[] = {SGI60120, SGI60121, SGI60122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60123, SGI0, SGI0, SGI0, SGI60124, SGI60124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13367[] = {SGI60125, SGI60126, SGI60127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60128, SGI0, SGI0, SGI0, SGI60129, SGI60129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13368[] = {SGI60130, SGI60131, SGI60132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60133, SGI0, SGI0, SGI0, SGI60134, SGI60134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13369[] = {SGI60135, SGI60136, SGI60137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60138, SGI0, SGI0, SGI0, SGI60139, SGI60139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13370[] = {SGI60140, SGI60141, SGI60142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60143, SGI0, SGI0, SGI0, SGI60144, SGI60144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13371[] = {SGI60145, SGI60146, SGI60147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60148, SGI0, SGI0, SGI0, SGI60149, SGI60149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13372[] = {SGI60150, SGI60151, SGI60152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60153, SGI0, SGI0, SGI0, SGI60154, SGI60154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13373[] = {SGI60155, SGI60156, SGI60157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60158, SGI0, SGI0, SGI0, SGI60159, SGI60159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13374[] = {SGI60160, SGI60161, SGI60162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60163, SGI0, SGI0, SGI0, SGI60164, SGI60164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13375[] = {SGI60165, SGI60166, SGI60167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60168, SGI0, SGI0, SGI0, SGI60169, SGI60169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13376[] = {SGI60170, SGI60171, SGI60172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60173, SGI0, SGI0, SGI0, SGI60174, SGI60174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13377[] = {SGI60175, SGI60176, SGI60177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60178, SGI0, SGI0, SGI0, SGI60179, SGI60179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13378[] = {SGI60180, SGI60181, SGI60182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60183, SGI0, SGI0, SGI0, SGI60184, SGI60184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13379[] = {SGI60185, SGI60186, SGI60187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60188, SGI0, SGI0, SGI0, SGI60189, SGI60189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13380[] = {SGI60190, SGI60191, SGI60192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60193, SGI0, SGI0, SGI0, SGI60194, SGI60194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13381[] = {SGI60195, SGI60196, SGI60197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60198, SGI0, SGI0, SGI0, SGI60199, SGI60199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13382[] = {SGI60200, SGI60201, SGI60202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60203, SGI0, SGI0, SGI0, SGI60204, SGI60204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13383[] = {SGI60205, SGI60206, SGI60207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60208, SGI0, SGI0, SGI0, SGI60209, SGI60209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13384[] = {SGI60210, SGI60211, SGI60212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60213, SGI60214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13385[] = {SGI60215, SGI60216, SGI60217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60218, SGI0, SGI0, SGI0, SGI60219, SGI60219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13386[] = {SGI60220, SGI60221, SGI60222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60223, SGI0, SGI0, SGI0, SGI60224, SGI60224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13387[] = {SGI60225, SGI60226, SGI60227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60228, SGI0, SGI0, SGI0, SGI60229, SGI60229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13388[] = {SGI60230, SGI60231, SGI60232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60233, SGI0, SGI0, SGI0, SGI60234, SGI60234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13389[] = {SGI60235, SGI60236, SGI60237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60238, SGI0, SGI0, SGI0, SGI60239, SGI60239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13390[] = {SGI60240, SGI60241, SGI60242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60243, SGI60243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13391[] = {SGI60244, SGI60245, SGI60246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60247, SGI0, SGI0, SGI0, SGI60248, SGI60248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13392[] = {SGI60249, SGI0, SGI60250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13393[] = {SGI60251, SGI60252, SGI60253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60254, SGI0, SGI0, SGI0, SGI60255, SGI60255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13394[] = {SGI60256, SGI60257, SGI60258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60259, SGI0, SGI0, SGI0, SGI60260, SGI60260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13395[] = {SGI60261, SGI60262, SGI60263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60264, SGI0, SGI0, SGI0, SGI60265, SGI60265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13396[] = {SGI60266, SGI60267, SGI60268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60269, SGI0, SGI0, SGI0, SGI60270, SGI60270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13397[] = {SGI60271, SGI60272, SGI60273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60274, SGI0, SGI0, SGI0, SGI60275, SGI60275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13398[] = {SGI60276, SGI60277, SGI60278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60279, SGI0, SGI0, SGI0, SGI60280, SGI60280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13399[] = {SGI60281, SGI60282, SGI60283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60284, SGI0, SGI0, SGI0, SGI60285, SGI60285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13400[] = {SGI60286, SGI60287, SGI60288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60289, SGI0, SGI0, SGI0, SGI60290, SGI60290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13401[] = {SGI60291, SGI60292, SGI60293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60294, SGI0, SGI0, SGI0, SGI60295, SGI60295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13402[] = {SGI60296, SGI60297, SGI60298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60299, SGI0, SGI0, SGI0, SGI60300, SGI60300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13403[] = {SGI60301, SGI60302, SGI60303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60304, SGI0, SGI0, SGI0, SGI60305, SGI60305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13404[] = {SGI60306, SGI60307, SGI60308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60309, SGI0, SGI0, SGI0, SGI60310, SGI60310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13405[] = {SGI60311, SGI60312, SGI60313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60314, SGI0, SGI0, SGI0, SGI60315, SGI60315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13406[] = {SGI60316, SGI60317, SGI60318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60319, SGI0, SGI0, SGI0, SGI60320, SGI60320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13407[] = {SGI60321, SGI60322, SGI60323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60324, SGI0, SGI0, SGI0, SGI60325, SGI60325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13408[] = {SGI60326, SGI60327, SGI60328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60329, SGI0, SGI0, SGI0, SGI60330, SGI60330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13409[] = {SGI60331, SGI60332, SGI60333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60334, SGI0, SGI0, SGI0, SGI60335, SGI60335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13410[] = {SGI60336, SGI60337, SGI60338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60339, SGI0, SGI0, SGI0, SGI60340, SGI60340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13411[] = {SGI60341, SGI0, SGI60342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13412[] = {SGI60343, SGI60344, SGI60345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60346, SGI0, SGI0, SGI0, SGI60347, SGI60347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13413[] = {SGI60348, SGI60349, SGI60350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60351, SGI0, SGI0, SGI0, SGI0, SGI60352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13414[] = {SGI60353, SGI60354, SGI60355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60356, SGI0, SGI0, SGI0, SGI60357, SGI60357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13415[] = {SGI60358, SGI60359, SGI60360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60361, SGI0, SGI0, SGI0, SGI60362, SGI60362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13416[] = {SGI60363, SGI60364, SGI60365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60366, SGI0, SGI0, SGI0, SGI60367, SGI60367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13417[] = {SGI60368, SGI60369, SGI60370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60371, SGI0, SGI0, SGI0, SGI60372, SGI60372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13418[] = {SGI60373, SGI60374, SGI60375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60376, SGI0, SGI0, SGI0, SGI60377, SGI60377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13419[] = {SGI60378, SGI60379, SGI60380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60381, SGI0, SGI0, SGI0, SGI60382, SGI60382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13420[] = {SGI60383, SGI60384, SGI60385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60386, SGI0, SGI0, SGI0, SGI60387, SGI60387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13421[] = {SGI60388, SGI60389, SGI60390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60391, SGI0, SGI0, SGI0, SGI60392, SGI60392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13422[] = {SGI60393, SGI0, SGI60394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60395, SGI0, SGI0, SGI0, SGI60396, SGI60396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13423[] = {SGI60397, SGI60398, SGI60399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60400, SGI0, SGI0, SGI0, SGI60401, SGI60401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13424[] = {SGI60402, SGI60403, SGI60404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60407, SGI0, SGI0, SGI0, SGI60409, SGI60409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13425[] = {SGI60405, SGI60406, SGI60408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60410, SGI0, SGI0, SGI0, SGI60411, SGI60411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13426[] = {SGI60412, SGI60413, SGI60414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60415, SGI0, SGI0, SGI0, SGI60416, SGI60416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13427[] = {SGI60417, SGI60418, SGI60419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60422, SGI0, SGI0, SGI0, SGI60424, SGI60424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13428[] = {SGI60420, SGI60421, SGI60423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60425, SGI0, SGI0, SGI0, SGI60426, SGI60426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13429[] = {SGI60427, SGI60428, SGI60429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60430, SGI0, SGI0, SGI0, SGI60431, SGI60431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13430[] = {SGI60432, SGI0, SGI60433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13431[] = {SGI60434, SGI60435, SGI60436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60437, SGI0, SGI0, SGI0, SGI60438, SGI60438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13432[] = {SGI60439, SGI60440, SGI60441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60442, SGI0, SGI0, SGI0, SGI60443, SGI60443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13433[] = {SGI60444, SGI60445, SGI60446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60447, SGI0, SGI0, SGI0, SGI60448, SGI60448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13434[] = {SGI60449, SGI60450, SGI60451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60452, SGI0, SGI0, SGI0, SGI60453, SGI60453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13435[] = {SGI60454, SGI60455, SGI60456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60457, SGI0, SGI0, SGI0, SGI60458, SGI60458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13436[] = {SGI60459, SGI60460, SGI60461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60462, SGI0, SGI0, SGI0, SGI60463, SGI60463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13437[] = {SGI60464, SGI60465, SGI60466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60467, SGI0, SGI0, SGI0, SGI60468, SGI60468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13438[] = {SGI60469, SGI60470, SGI60471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60472, SGI0, SGI0, SGI0, SGI60473, SGI60473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13439[] = {SGI60474, SGI60475, SGI60476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60477, SGI0, SGI0, SGI0, SGI60478, SGI60478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13440[] = {SGI60479, SGI60480, SGI60481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60482, SGI0, SGI0, SGI0, SGI60483, SGI60483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13441[] = {SGI60484, SGI60485, SGI60486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60487, SGI0, SGI0, SGI0, SGI60488, SGI60488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13442[] = {SGI60489, SGI60490, SGI60491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60492, SGI0, SGI0, SGI0, SGI60493, SGI60493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13443[] = {SGI60494, SGI60495, SGI60496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60497, SGI0, SGI0, SGI0, SGI60498, SGI60498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13444[] = {SGI60499, SGI60500, SGI60501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60502, SGI0, SGI0, SGI0, SGI60504, SGI60504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13445[] = {SGI0, SGI60503, SGI60505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13446[] = {SGI60506, SGI60507, SGI60508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60512, SGI0, SGI0, SGI0, SGI60513, SGI60513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13447[] = {SGI60509, SGI60510, SGI60511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60514, SGI0, SGI0, SGI0, SGI60515, SGI60515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13448[] = {SGI60516, SGI60518, SGI60520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60522, SGI0, SGI0, SGI0, SGI60524, SGI60524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13449[] = {SGI60517, SGI60519, SGI60521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60523, SGI0, SGI0, SGI0, SGI60525, SGI60525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13450[] = {SGI60526, SGI60527, SGI60528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60529, SGI0, SGI0, SGI0, SGI60530, SGI60530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13451[] = {SGI60531, SGI60532, SGI60533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60534, SGI0, SGI0, SGI0, SGI60535, SGI60535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13452[] = {SGI60536, SGI60537, SGI60538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60539, SGI0, SGI0, SGI0, SGI60540, SGI60540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13453[] = {SGI60541, SGI60542, SGI60543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60544, SGI0, SGI0, SGI0, SGI60545, SGI60545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13454[] = {SGI60546, SGI60547, SGI60548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60549, SGI0, SGI0, SGI0, SGI60550, SGI60550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13455[] = {SGI60551, SGI60552, SGI60553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60554, SGI0, SGI0, SGI0, SGI60555, SGI60555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13456[] = {SGI60556, SGI60557, SGI60558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60559, SGI0, SGI0, SGI0, SGI60560, SGI60560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13457[] = {SGI60561, SGI60562, SGI60563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60564, SGI0, SGI0, SGI0, SGI60565, SGI60565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13458[] = {SGI60566, SGI60567, SGI60568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60569, SGI0, SGI0, SGI0, SGI60570, SGI60570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13459[] = {SGI60571, SGI60572, SGI60573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60574, SGI0, SGI0, SGI0, SGI60575, SGI60575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13460[] = {SGI60576, SGI60577, SGI60578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60579, SGI0, SGI0, SGI0, SGI60580, SGI60580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13461[] = {SGI60581, SGI60582, SGI60583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60584, SGI0, SGI0, SGI0, SGI60585, SGI60585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13462[] = {SGI60586, SGI60587, SGI60588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60589, SGI0, SGI0, SGI0, SGI60590, SGI60590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13463[] = {SGI60591, SGI0, SGI60592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60593, SGI0, SGI0, SGI0, SGI60594, SGI60594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13464[] = {SGI60595, SGI60596, SGI60597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60598, SGI0, SGI0, SGI0, SGI60599, SGI60599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13465[] = {SGI60600, SGI60601, SGI60602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60603, SGI0, SGI0, SGI0, SGI60604, SGI60604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13466[] = {SGI60605, SGI60606, SGI60607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60608, SGI0, SGI0, SGI0, SGI60609, SGI60609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13467[] = {SGI60610, SGI60611, SGI60612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60613, SGI0, SGI0, SGI0, SGI60614, SGI60614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13468[] = {SGI60615, SGI60616, SGI60617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60618, SGI0, SGI0, SGI0, SGI60619, SGI60619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13469[] = {SGI60620, SGI60621, SGI60622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60623, SGI0, SGI0, SGI0, SGI60624, SGI60624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13470[] = {SGI60625, SGI60626, SGI60627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60628, SGI0, SGI0, SGI0, SGI60629, SGI60629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13471[] = {SGI60630, SGI60631, SGI60632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60633, SGI0, SGI0, SGI0, SGI60634, SGI60634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13472[] = {SGI60635, SGI60636, SGI60637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60638, SGI0, SGI0, SGI0, SGI60639, SGI60639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13473[] = {SGI60640, SGI60641, SGI60642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60643, SGI0, SGI0, SGI0, SGI60644, SGI60644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13474[] = {SGI60645, SGI60646, SGI60647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60648, SGI0, SGI0, SGI0, SGI60649, SGI60649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13475[] = {SGI60650, SGI60651, SGI60652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60653, SGI0, SGI0, SGI0, SGI60654, SGI60654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13476[] = {SGI60655, SGI60656, SGI60657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60658, SGI0, SGI0, SGI0, SGI60659, SGI60659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13477[] = {SGI60660, SGI60661, SGI60662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60663, SGI0, SGI0, SGI0, SGI60664, SGI60664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13478[] = {SGI60665, SGI60666, SGI60667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60668, SGI0, SGI0, SGI0, SGI60669, SGI60669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13479[] = {SGI60670, SGI60671, SGI60672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60673, SGI0, SGI0, SGI0, SGI60674, SGI60674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13480[] = {SGI60675, SGI60676, SGI60677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60678, SGI0, SGI0, SGI0, SGI60679, SGI60679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13481[] = {SGI60680, SGI60681, SGI60682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60683, SGI0, SGI0, SGI0, SGI60684, SGI60684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13482[] = {SGI60685, SGI60686, SGI60687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60688, SGI0, SGI0, SGI0, SGI60689, SGI60689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13483[] = {SGI60690, SGI60691, SGI60692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60693, SGI0, SGI0, SGI0, SGI60694, SGI60694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13484[] = {SGI60695, SGI60696, SGI60697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60698, SGI0, SGI0, SGI0, SGI60699, SGI60699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13485[] = {SGI60700, SGI60701, SGI60702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60703, SGI0, SGI0, SGI0, SGI60704, SGI60704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13486[] = {SGI60705, SGI60706, SGI60707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60708, SGI60709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13487[] = {SGI60710, SGI60711, SGI60712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60713, SGI0, SGI0, SGI0, SGI60714, SGI60714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13488[] = {SGI60715, SGI60716, SGI60717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60718, SGI0, SGI0, SGI0, SGI60719, SGI60719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13489[] = {SGI60720, SGI60721, SGI60722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60723, SGI0, SGI0, SGI0, SGI60724, SGI60724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13490[] = {SGI60725, SGI60726, SGI60727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60728, SGI0, SGI0, SGI0, SGI60729, SGI60729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13491[] = {SGI60730, SGI60731, SGI60732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60733, SGI0, SGI0, SGI0, SGI60734, SGI60734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13492[] = {SGI60735, SGI60736, SGI60737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60738, SGI0, SGI0, SGI0, SGI60739, SGI60739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13493[] = {SGI60740, SGI60741, SGI60742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60743, SGI0, SGI0, SGI0, SGI60744, SGI60744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13494[] = {SGI60745, SGI60746, SGI60747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60748, SGI0, SGI0, SGI0, SGI60749, SGI60749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13495[] = {SGI60750, SGI60751, SGI60752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60753, SGI0, SGI0, SGI0, SGI60754, SGI60754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13496[] = {SGI60755, SGI60756, SGI60757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60758, SGI0, SGI0, SGI0, SGI60759, SGI60759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13497[] = {SGI60760, SGI60761, SGI60762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60763, SGI0, SGI0, SGI0, SGI60764, SGI60764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13498[] = {SGI60765, SGI60766, SGI60767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60768, SGI0, SGI0, SGI0, SGI0, SGI60769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13499[] = {SGI60770, SGI60771, SGI60772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60773, SGI0, SGI0, SGI0, SGI60774, SGI60774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13500[] = {SGI60775, SGI60776, SGI60777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60778, SGI0, SGI0, SGI0, SGI60779, SGI60779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13501[] = {SGI60780, SGI60781, SGI60782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60783, SGI0, SGI0, SGI0, SGI60784, SGI60784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13502[] = {SGI60785, SGI60786, SGI60787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60788, SGI0, SGI0, SGI0, SGI60789, SGI60789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13503[] = {SGI60790, SGI60791, SGI60792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60793, SGI0, SGI0, SGI0, SGI60794, SGI60794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13504[] = {SGI60795, SGI60796, SGI60797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60798, SGI0, SGI0, SGI0, SGI60799, SGI60799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13505[] = {SGI60800, SGI60801, SGI60802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60803, SGI0, SGI0, SGI0, SGI60804, SGI60804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13506[] = {SGI60805, SGI60806, SGI60807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60808, SGI0, SGI0, SGI0, SGI60809, SGI60809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13507[] = {SGI60810, SGI0, SGI60811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60812, SGI0, SGI0, SGI0, SGI60813, SGI60813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13508[] = {SGI60814, SGI60815, SGI60816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60817, SGI0, SGI0, SGI0, SGI60818, SGI60818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13509[] = {SGI60819, SGI60820, SGI60821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60822, SGI0, SGI0, SGI0, SGI60823, SGI60823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13510[] = {SGI60824, SGI60825, SGI60826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60827, SGI0, SGI0, SGI0, SGI60828, SGI60828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13511[] = {SGI60829, SGI60830, SGI60831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60832, SGI0, SGI0, SGI0, SGI60833, SGI60833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13512[] = {SGI60834, SGI60835, SGI60836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60837, SGI0, SGI0, SGI0, SGI60838, SGI60838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13513[] = {SGI60839, SGI60840, SGI60841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60842, SGI0, SGI0, SGI0, SGI60843, SGI60843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13514[] = {SGI60844, SGI60845, SGI60846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60847, SGI0, SGI0, SGI0, SGI60848, SGI60848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13515[] = {SGI60849, SGI60850, SGI60851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60852, SGI0, SGI0, SGI0, SGI60853, SGI60853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13516[] = {SGI60854, SGI60855, SGI60856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60857, SGI0, SGI0, SGI0, SGI60858, SGI60858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13517[] = {SGI60859, SGI60860, SGI60861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60862, SGI0, SGI0, SGI0, SGI60863, SGI60863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13518[] = {SGI60864, SGI60865, SGI60866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60867, SGI0, SGI0, SGI0, SGI60868, SGI60868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13519[] = {SGI60869, SGI60870, SGI60871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60872, SGI0, SGI0, SGI0, SGI60873, SGI60873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13520[] = {SGI60874, SGI60875, SGI60876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60877, SGI0, SGI0, SGI0, SGI60878, SGI60878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13521[] = {SGI60879, SGI0, SGI60880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13522[] = {SGI60881, SGI60882, SGI60883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60884, SGI0, SGI0, SGI0, SGI60885, SGI60885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13523[] = {SGI60886, SGI0, SGI60887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13524[] = {SGI60888, SGI60889, SGI60890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60891, SGI0, SGI0, SGI0, SGI60892, SGI60892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13525[] = {SGI60893, SGI60894, SGI60895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60896, SGI0, SGI0, SGI0, SGI60897, SGI60897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13526[] = {SGI60898, SGI60899, SGI60900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60901, SGI0, SGI0, SGI0, SGI60902, SGI60902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13527[] = {SGI60903, SGI60904, SGI60905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60906, SGI0, SGI0, SGI0, SGI60907, SGI60907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13528[] = {SGI60908, SGI60909, SGI60910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60911, SGI0, SGI0, SGI0, SGI60912, SGI60912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13529[] = {SGI60913, SGI60914, SGI60915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60916, SGI0, SGI0, SGI0, SGI60917, SGI60917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13530[] = {SGI60918, SGI60919, SGI60920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60921, SGI0, SGI0, SGI0, SGI0, SGI60922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13531[] = {SGI60923, SGI60924, SGI60925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60926, SGI0, SGI0, SGI0, SGI60927, SGI60927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13532[] = {SGI60928, SGI60929, SGI60930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60931, SGI0, SGI0, SGI0, SGI60932, SGI60932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13533[] = {SGI60933, SGI60934, SGI60935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60936, SGI0, SGI0, SGI0, SGI60937, SGI60937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13534[] = {SGI60938, SGI60939, SGI60940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60941, SGI0, SGI0, SGI0, SGI60942, SGI60942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13535[] = {SGI60943, SGI60944, SGI60945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60946, SGI0, SGI0, SGI0, SGI60947, SGI60947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13536[] = {SGI60948, SGI60949, SGI60950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60951, SGI0, SGI0, SGI0, SGI60952, SGI60952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13537[] = {SGI60953, SGI0, SGI60954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13538[] = {SGI60955, SGI60956, SGI60957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60958, SGI0, SGI0, SGI0, SGI60959, SGI60959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13539[] = {SGI60960, SGI60961, SGI60962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60963, SGI0, SGI0, SGI0, SGI60964, SGI60964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13540[] = {SGI60965, SGI60966, SGI60967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60968, SGI0, SGI0, SGI0, SGI60969, SGI60969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13541[] = {SGI60970, SGI60971, SGI60972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60973, SGI0, SGI0, SGI0, SGI60974, SGI60974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13542[] = {SGI60975, SGI60976, SGI60977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60978, SGI0, SGI0, SGI0, SGI0, SGI60979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13543[] = {SGI60980, SGI60981, SGI60982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60983, SGI0, SGI0, SGI0, SGI60984, SGI60984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13544[] = {SGI60985, SGI60986, SGI60987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60988, SGI0, SGI0, SGI0, SGI60989, SGI60989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13545[] = {SGI60990, SGI60991, SGI60992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60993, SGI0, SGI0, SGI0, SGI60994, SGI60994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13546[] = {SGI60995, SGI60996, SGI60997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI60998, SGI0, SGI0, SGI0, SGI60999, SGI60999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13547[] = {SGI61000, SGI61001, SGI61002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61003, SGI0, SGI0, SGI0, SGI61004, SGI61004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13548[] = {SGI61005, SGI61006, SGI61007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61008, SGI0, SGI0, SGI0, SGI61009, SGI61009, SGI0, SGI0, NULL};
const Segment * const SG13549[] = {SGI61010, SGI61011, SGI61012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61013, SGI0, SGI0, SGI0, SGI61014, SGI61014, SGI0, SGI61015, SGI0, NULL};
const Segment * const SG13550[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61016, SGI0, NULL};
const Segment * const SG13551[] = {SGI61017, SGI61018, SGI61019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61020, SGI0, SGI0, SGI0, SGI61021, SGI61021, SGI0, SGI61022, SGI0, NULL};
const Segment * const SG13552[] = {SGI61023, SGI61024, SGI61025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61026, SGI0, SGI0, SGI0, SGI61027, SGI61027, SGI0, SGI0, NULL};
const Segment * const SG13553[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61028, SGI0, SGI61029, SGI0, NULL};
const Segment * const SG13554[] = {SGI61030, SGI61031, SGI61032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61033, SGI0, SGI0, SGI0, SGI61034, SGI61034, SGI0, SGI0, NULL};
const Segment * const SG13555[] = {SGI61035, SGI61036, SGI61037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61038, SGI0, SGI0, SGI0, SGI61039, SGI61039, SGI0, SGI0, NULL};
const Segment * const SG13556[] = {SGI61040, SGI61041, SGI61042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61043, SGI0, SGI0, SGI0, SGI61044, SGI61044, SGI0, SGI0, NULL};
const Segment * const SG13557[] = {SGI61045, SGI61046, SGI61047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61048, SGI0, SGI0, SGI0, SGI61049, SGI61049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13558[] = {SGI61050, SGI61051, SGI61052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61053, SGI0, SGI0, SGI0, SGI61054, SGI61054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13559[] = {SGI61055, SGI0, SGI61056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13560[] = {SGI61057, SGI61058, SGI61059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61060, SGI0, SGI0, SGI0, SGI61061, SGI61061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13561[] = {SGI61062, SGI61063, SGI61064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61065, SGI0, SGI0, SGI0, SGI61066, SGI61066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13562[] = {SGI61067, SGI61068, SGI61069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61070, SGI0, SGI0, SGI0, SGI61071, SGI61071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13563[] = {SGI61072, SGI61073, SGI61074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61075, SGI0, SGI0, SGI0, SGI61076, SGI61076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13564[] = {SGI61077, SGI61078, SGI61079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61080, SGI0, SGI0, SGI0, SGI61081, SGI61081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13565[] = {SGI61082, SGI0, SGI61083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13566[] = {SGI61084, SGI61085, SGI61086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61087, SGI0, SGI0, SGI0, SGI61088, SGI61088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13567[] = {SGI61089, SGI61090, SGI61091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61092, SGI0, SGI0, SGI0, SGI61093, SGI61093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13568[] = {SGI61094, SGI61095, SGI61096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61097, SGI0, SGI0, SGI0, SGI61098, SGI61098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13569[] = {SGI61099, SGI61100, SGI61101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61102, SGI0, SGI0, SGI0, SGI61103, SGI61103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13570[] = {SGI61104, SGI0, SGI61105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13571[] = {SGI61106, SGI0, SGI61107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13572[] = {SGI61108, SGI61109, SGI61110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61111, SGI61112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13573[] = {SGI61113, SGI61114, SGI61115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61116, SGI0, SGI0, SGI0, SGI61117, SGI61117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13574[] = {SGI61118, SGI61119, SGI61120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61121, SGI0, SGI0, SGI0, SGI61122, SGI61122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13575[] = {SGI61123, SGI61124, SGI61125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61126, SGI0, SGI0, SGI0, SGI61127, SGI61127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13576[] = {SGI61128, SGI0, SGI61129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61130, SGI0, SGI0, SGI0, SGI61131, SGI61131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13577[] = {SGI61132, SGI0, SGI61133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13578[] = {SGI61134, SGI61135, SGI61136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61137, SGI0, SGI0, SGI0, SGI61138, SGI61138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13579[] = {SGI61139, SGI61140, SGI61141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61142, SGI61142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13580[] = {SGI61143, SGI0, SGI61144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13581[] = {SGI0, SGI61145, SGI61146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13582[] = {SGI61147, SGI0, SGI61148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13583[] = {SGI61149, SGI61150, SGI61151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61152, SGI0, SGI0, SGI0, SGI61153, SGI61153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13584[] = {SGI61154, SGI61155, SGI61156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61157, SGI61157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13585[] = {SGI61158, SGI61159, SGI61160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61161, SGI0, SGI0, SGI0, SGI61162, SGI61162, SGI0, SGI0, NULL};
const Segment * const SG13586[] = {SGI61163, SGI61164, SGI61165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61166, SGI61166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13587[] = {SGI61167, SGI61168, SGI61169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61170, SGI0, SGI0, SGI0, SGI61171, SGI61171, SGI0, SGI0, NULL};
const Segment * const SG13588[] = {SGI61172, SGI61173, SGI61174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61175, SGI0, SGI0, SGI0, SGI61176, SGI61176, SGI0, SGI0, NULL};
const Segment * const SG13589[] = {SGI61177, SGI61178, SGI61179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61180, SGI0, SGI0, SGI0, SGI61181, SGI61181, SGI0, SGI0, NULL};
const Segment * const SG13590[] = {SGI61182, SGI61183, SGI61184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61185, SGI0, SGI0, SGI0, SGI61186, SGI0, SGI0, SGI61187, SGI0, NULL};
const Segment * const SG13591[] = {SGI61188, SGI61189, SGI61190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61191, SGI0, SGI0, SGI0, SGI61192, SGI61192, SGI0, SGI0, NULL};
const Segment * const SG13592[] = {SGI61193, SGI61194, SGI61195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61196, SGI0, SGI0, SGI0, SGI61197, SGI61197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13593[] = {SGI61198, SGI61199, SGI61200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61201, SGI0, SGI0, SGI0, SGI61202, SGI61202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13594[] = {SGI61203, SGI61204, SGI61205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61206, SGI0, SGI0, SGI0, SGI61207, SGI61207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13595[] = {SGI61208, SGI61209, SGI61210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61211, SGI0, SGI0, SGI0, SGI61212, SGI61212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13596[] = {SGI61213, SGI61214, SGI61215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61216, SGI0, SGI0, SGI0, SGI61217, SGI61217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13597[] = {SGI61218, SGI61219, SGI61220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61221, SGI0, SGI0, SGI0, SGI61222, SGI61222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13598[] = {SGI61223, SGI0, SGI61224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13599[] = {SGI61225, SGI61226, SGI61227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61228, SGI0, SGI0, SGI0, SGI61229, SGI61229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13600[] = {SGI61230, SGI61231, SGI61232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61233, SGI0, SGI0, SGI0, SGI61234, SGI61234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13601[] = {SGI61235, SGI61236, SGI61237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61238, SGI0, SGI0, SGI0, SGI61239, SGI61239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13602[] = {SGI61240, SGI61241, SGI61242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61243, SGI0, SGI0, SGI0, SGI61244, SGI61244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13603[] = {SGI61245, SGI61246, SGI61247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61248, SGI0, SGI0, SGI0, SGI61249, SGI61249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13604[] = {SGI61250, SGI61251, SGI61252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61253, SGI0, SGI0, SGI0, SGI61254, SGI61254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13605[] = {SGI61255, SGI61256, SGI61257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61258, SGI0, SGI0, SGI0, SGI61259, SGI61259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13606[] = {SGI61260, SGI61261, SGI61262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61263, SGI0, SGI0, SGI0, SGI61264, SGI61264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13607[] = {SGI61265, SGI0, SGI61266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13608[] = {SGI61267, SGI61268, SGI61269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61270, SGI0, SGI0, SGI0, SGI61271, SGI61271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13609[] = {SGI61272, SGI61273, SGI61274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61275, SGI0, SGI0, SGI0, SGI61276, SGI61276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13610[] = {SGI61277, SGI61278, SGI61279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61280, SGI0, SGI0, SGI0, SGI61281, SGI61281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13611[] = {SGI61282, SGI61283, SGI61284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61285, SGI0, SGI0, SGI0, SGI61286, SGI61286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13612[] = {SGI61287, SGI61288, SGI61289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61290, SGI0, SGI0, SGI0, SGI61291, SGI61291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13613[] = {SGI61292, SGI61293, SGI61294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61295, SGI0, SGI0, SGI0, SGI61296, SGI61296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13614[] = {SGI61297, SGI61298, SGI61299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61300, SGI0, SGI0, SGI0, SGI61301, SGI61301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13615[] = {SGI61302, SGI61303, SGI61304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61305, SGI0, SGI0, SGI0, SGI61306, SGI61306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13616[] = {SGI61307, SGI61308, SGI61309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61310, SGI0, SGI0, SGI0, SGI61311, SGI61311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13617[] = {SGI61312, SGI61313, SGI61314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61315, SGI0, SGI0, SGI0, SGI61316, SGI61316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13618[] = {SGI61317, SGI0, SGI61318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13619[] = {SGI61319, SGI61320, SGI61321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61322, SGI0, SGI0, SGI0, SGI61323, SGI61323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13620[] = {SGI61324, SGI61325, SGI61326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61327, SGI0, SGI0, SGI0, SGI61328, SGI61328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13621[] = {SGI61329, SGI0, SGI61330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61331, SGI0, SGI0, SGI0, SGI61332, SGI61332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13622[] = {SGI61333, SGI61334, SGI61335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61336, SGI0, SGI0, SGI0, SGI61337, SGI61337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13623[] = {SGI61338, SGI61339, SGI61340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61341, SGI0, SGI0, SGI0, SGI61342, SGI61342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13624[] = {SGI61343, SGI0, SGI61344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61345, SGI0, SGI0, SGI0, SGI61346, SGI61346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13625[] = {SGI61347, SGI0, SGI61348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13626[] = {SGI61349, SGI61350, SGI61351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61352, SGI0, SGI0, SGI0, SGI61353, SGI61353, SGI0, SGI0, NULL};
const Segment * const SG13627[] = {SGI61354, SGI61355, SGI61356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61357, SGI0, SGI0, SGI0, SGI61358, SGI61358, SGI0, SGI0, NULL};
const Segment * const SG13628[] = {SGI61359, SGI61360, SGI61361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61362, SGI0, SGI0, SGI0, SGI61363, SGI61363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13629[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61364, SGI0, SGI0, SGI0, SGI61365, SGI61366, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13630[] = {SGI61367, SGI61368, SGI61369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61370, SGI0, SGI0, SGI0, SGI61371, SGI61371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13631[] = {SGI61372, SGI61373, SGI61374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61375, SGI0, SGI0, SGI0, SGI61376, SGI61376, SGI0, SGI0, NULL};
const Segment * const SG13632[] = {SGI61377, SGI61378, SGI61379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61380, SGI0, SGI0, SGI0, SGI61381, SGI61381, SGI0, SGI0, NULL};
const Segment * const SG13633[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61382, SGI0, SGI61383, SGI0, NULL};
const Segment * const SG13634[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61384, SGI0, SGI61385, SGI0, NULL};
const Segment * const SG13635[] = {SGI61386, SGI0, SGI61387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61388, SGI0, SGI0, SGI0, SGI61389, SGI61389, SGI0, SGI0, NULL};
const Segment * const SG13636[] = {SGI61390, SGI61391, SGI61392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61393, SGI0, SGI0, SGI0, SGI61394, SGI61394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13637[] = {SGI61395, SGI0, SGI61396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13638[] = {SGI61397, SGI61398, SGI61399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61400, SGI0, SGI0, SGI0, SGI61401, SGI61401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13639[] = {SGI61402, SGI61403, SGI61404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61405, SGI0, SGI0, SGI0, SGI61406, SGI61406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13640[] = {SGI61407, SGI61408, SGI61409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61410, SGI0, SGI0, SGI0, SGI61411, SGI61411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13641[] = {SGI61412, SGI61413, SGI61414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61415, SGI0, SGI0, SGI0, SGI61416, SGI61416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13642[] = {SGI61417, SGI61418, SGI61419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61420, SGI0, SGI0, SGI0, SGI61421, SGI61421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13643[] = {SGI61422, SGI61423, SGI61424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61425, SGI0, SGI0, SGI0, SGI61426, SGI61426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13644[] = {SGI61427, SGI61428, SGI61429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61430, SGI61430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13645[] = {SGI61431, SGI61432, SGI61433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61434, SGI0, SGI0, SGI0, SGI61435, SGI61435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13646[] = {SGI61436, SGI61437, SGI61438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61439, SGI0, SGI0, SGI0, SGI61440, SGI61440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13647[] = {SGI61441, SGI0, SGI61442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13648[] = {SGI61443, SGI61444, SGI61445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61446, SGI0, SGI0, SGI0, SGI61447, SGI61447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13649[] = {SGI61448, SGI0, SGI61449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61450, SGI0, SGI0, SGI0, SGI61451, SGI61451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13650[] = {SGI61452, SGI61453, SGI61454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61455, SGI0, SGI0, SGI0, SGI61456, SGI61456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13651[] = {SGI61457, SGI61458, SGI61459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61460, SGI0, SGI0, SGI0, SGI61461, SGI61461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13652[] = {SGI61462, SGI61463, SGI61464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61465, SGI0, SGI0, SGI0, SGI61466, SGI61466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13653[] = {SGI61467, SGI61468, SGI61469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61470, SGI0, SGI0, SGI0, SGI61471, SGI61471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13654[] = {SGI61472, SGI61473, SGI61474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61475, SGI0, SGI0, SGI0, SGI61476, SGI61476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13655[] = {SGI61477, SGI61478, SGI61479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61480, SGI0, SGI0, SGI0, SGI61481, SGI61481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13656[] = {SGI61482, SGI61483, SGI61484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61485, SGI0, SGI0, SGI0, SGI61486, SGI61486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13657[] = {SGI61487, SGI61488, SGI61489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61490, SGI0, SGI0, SGI0, SGI61491, SGI61491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13658[] = {SGI61492, SGI61493, SGI61494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61495, SGI0, SGI0, SGI0, SGI61496, SGI61496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13659[] = {SGI61497, SGI0, SGI61498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61499, SGI0, SGI0, SGI0, SGI61500, SGI61500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13660[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61501, SGI0, SGI61502, SGI0, NULL};
const Segment * const SG13661[] = {SGI61503, SGI61504, SGI61505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61506, SGI0, SGI0, SGI0, SGI61507, SGI61507, SGI0, SGI0, NULL};
const Segment * const SG13662[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61508, SGI0, SGI61509, SGI0, NULL};
const Segment * const SG13663[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61510, SGI0, SGI61511, SGI0, NULL};
const Segment * const SG13664[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61512, SGI0, SGI61513, SGI0, NULL};
const Segment * const SG13665[] = {SGI61514, SGI0, SGI61515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61516, SGI0, SGI0, SGI0, SGI61517, SGI61517, SGI0, SGI0, NULL};
const Segment * const SG13666[] = {SGI61518, SGI61519, SGI61520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61521, SGI0, SGI0, SGI0, SGI61522, SGI61522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13667[] = {SGI61523, SGI61524, SGI61525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61526, SGI0, SGI0, SGI0, SGI61527, SGI61527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13668[] = {SGI61528, SGI61529, SGI61530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61531, SGI0, SGI0, SGI0, SGI61532, SGI61532, SGI0, SGI0, NULL};
const Segment * const SG13669[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61533, SGI0, SGI61534, SGI0, NULL};
const Segment * const SG13670[] = {SGI61535, SGI61536, SGI61537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61538, SGI0, SGI0, SGI0, SGI61539, SGI61539, SGI0, SGI0, NULL};
const Segment * const SG13671[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61540, SGI0, SGI61541, SGI0, NULL};
const Segment * const SG13672[] = {SGI61542, SGI61543, SGI61544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61545, SGI0, SGI0, SGI0, SGI61546, SGI61546, SGI0, SGI0, NULL};
const Segment * const SG13673[] = {SGI0, SGI61547, SGI61548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61549, SGI0, SGI0, SGI0, SGI61550, SGI61550, SGI0, SGI0, NULL};
const Segment * const SG13674[] = {SGI61551, SGI61552, SGI61553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61554, SGI0, SGI0, SGI0, SGI61555, SGI61555, SGI0, SGI0, NULL};
const Segment * const SG13675[] = {SGI61556, SGI61557, SGI61558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61559, SGI0, SGI0, SGI0, SGI61560, SGI0, SGI0, SGI61561, SGI0, NULL};
const Segment * const SG13676[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61562, SGI0, SGI61563, SGI0, NULL};
const Segment * const SG13677[] = {SGI61569, SGI61570, SGI61571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61572, SGI0, SGI0, SGI0, SGI61573, SGI61573, SGI0, SGI61574, SGI0, NULL};
const Segment * const SG13678[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61575, SGI0, SGI61576, SGI0, NULL};
const Segment * const SG13679[] = {SGI61577, SGI61578, SGI61579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61580, SGI0, SGI0, SGI0, SGI61581, SGI61581, SGI0, SGI61582, SGI0, NULL};
const Segment * const SG13680[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61583, SGI0, SGI61584, SGI0, NULL};
const Segment * const SG13681[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61585, SGI0, SGI61586, SGI0, NULL};
const Segment * const SG13682[] = {SGI61587, SGI61588, SGI61589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61590, SGI0, SGI0, SGI0, SGI61591, SGI61591, SGI0, SGI0, NULL};
const Segment * const SG13683[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61592, SGI0, SGI61593, SGI0, NULL};
const Segment * const SG13684[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61594, SGI0, SGI61595, SGI0, NULL};
const Segment * const SG13685[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61596, SGI0, SGI61597, SGI0, NULL};
const Segment * const SG13686[] = {SGI61598, SGI61599, SGI61600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61601, SGI0, SGI0, SGI0, SGI61602, SGI61602, SGI0, SGI61603, SGI0, NULL};
const Segment * const SG13687[] = {SGI61604, SGI61605, SGI61606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61607, SGI0, SGI0, SGI0, SGI61608, SGI61608, SGI0, SGI61609, SGI0, NULL};
const Segment * const SG13688[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61610, SGI0, SGI61611, SGI0, NULL};
const Segment * const SG13689[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61612, SGI0, SGI61613, SGI0, NULL};
const Segment * const SG13690[] = {SGI61564, SGI61565, SGI61566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61567, SGI0, SGI0, SGI0, SGI61568, SGI61568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13691[] = {SGI61614, SGI61615, SGI61616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61617, SGI0, SGI0, SGI0, SGI61618, SGI61618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13692[] = {SGI61619, SGI0, SGI61620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13693[] = {SGI61621, SGI61622, SGI61623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61624, SGI0, SGI0, SGI0, SGI61625, SGI61625, SGI0, SGI61626, SGI0, NULL};
const Segment * const SG13694[] = {SGI61627, SGI61628, SGI61629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61630, SGI0, SGI0, SGI0, SGI61631, SGI61631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13695[] = {SGI61632, SGI61633, SGI61634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61635, SGI0, SGI0, SGI0, SGI61636, SGI61636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13696[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61637, SGI0, SGI61638, SGI0, NULL};
const Segment * const SG13697[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61639, SGI0, SGI61640, SGI0, NULL};
const Segment * const SG13698[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61641, SGI0, SGI61642, SGI0, NULL};
const Segment * const SG13699[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61643, SGI0, SGI61644, SGI0, NULL};
const Segment * const SG13700[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61645, SGI0, SGI61646, SGI0, NULL};
const Segment * const SG13701[] = {SGI61647, SGI61648, SGI61649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61650, SGI0, SGI0, SGI0, SGI61651, SGI61651, SGI0, SGI61652, SGI0, NULL};
const Segment * const SG13702[] = {SGI61653, SGI61654, SGI61655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61656, SGI0, SGI0, SGI0, SGI61657, SGI61657, SGI0, SGI61658, SGI0, NULL};
const Segment * const SG13703[] = {SGI61659, SGI61660, SGI61661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61662, SGI0, SGI0, SGI0, SGI61663, SGI61663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13704[] = {SGI61664, SGI61665, SGI61666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61667, SGI0, SGI0, SGI0, SGI61668, SGI61668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13705[] = {SGI61669, SGI61670, SGI61671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61672, SGI0, SGI0, SGI0, SGI61673, SGI61673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13706[] = {SGI61674, SGI0, SGI61675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13707[] = {SGI61676, SGI61677, SGI61678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61679, SGI0, SGI0, SGI0, SGI61680, SGI61680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13708[] = {SGI61681, SGI61682, SGI61683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61684, SGI0, SGI0, SGI0, SGI61685, SGI61685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13709[] = {SGI61686, SGI61687, SGI61688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61689, SGI0, SGI0, SGI0, SGI61690, SGI61690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13710[] = {SGI61691, SGI0, SGI61692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61693, SGI0, SGI0, SGI0, SGI61694, SGI61694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13711[] = {SGI61695, SGI61696, SGI61697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61698, SGI61698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13712[] = {SGI61699, SGI0, SGI61700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13713[] = {SGI61701, SGI61702, SGI61703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61704, SGI0, SGI0, SGI0, SGI61705, SGI61705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13714[] = {SGI61706, SGI61707, SGI61708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61709, SGI0, SGI0, SGI0, SGI61710, SGI61710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13715[] = {SGI61711, SGI0, SGI61712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13716[] = {SGI61713, SGI61714, SGI61715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61716, SGI0, SGI0, SGI0, SGI61717, SGI61717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13717[] = {SGI61718, SGI61719, SGI61720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61721, SGI0, SGI0, SGI0, SGI61722, SGI61722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13718[] = {SGI61723, SGI61724, SGI61725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61726, SGI0, SGI0, SGI0, SGI61727, SGI61727, SGI0, SGI61728, SGI0, NULL};
const Segment * const SG13719[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61729, SGI0, SGI61730, SGI0, NULL};
const Segment * const SG13720[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61731, SGI0, SGI61732, SGI0, NULL};
const Segment * const SG13721[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61733, SGI0, SGI61734, SGI0, NULL};
const Segment * const SG13722[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61735, SGI0, SGI61736, SGI0, NULL};
const Segment * const SG13723[] = {SGI61737, SGI61738, SGI61739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61740, SGI0, SGI0, SGI0, SGI61741, SGI61741, SGI0, SGI61742, SGI0, NULL};
const Segment * const SG13724[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61743, SGI0, SGI61744, SGI0, NULL};
const Segment * const SG13725[] = {SGI61745, SGI61746, SGI61747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61748, SGI0, SGI0, SGI0, SGI61749, SGI61749, SGI0, SGI0, NULL};
const Segment * const SG13726[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61750, SGI0, SGI61751, SGI0, NULL};
const Segment * const SG13727[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61752, SGI0, SGI61753, SGI0, NULL};
const Segment * const SG13728[] = {SGI61754, SGI61755, SGI61756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61757, SGI0, SGI0, SGI0, SGI61758, SGI61758, SGI0, SGI61759, SGI0, NULL};
const Segment * const SG13729[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61760, SGI0, SGI61761, SGI0, NULL};
const Segment * const SG13730[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61762, SGI0, SGI61763, SGI0, NULL};
const Segment * const SG13731[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61764, SGI0, SGI61765, SGI0, NULL};
const Segment * const SG13732[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61766, SGI0, SGI61767, SGI0, NULL};
const Segment * const SG13733[] = {SGI61768, SGI61769, SGI61770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61771, SGI0, SGI0, SGI0, SGI0, SGI61772, SGI0, SGI61773, SGI0, NULL};
const Segment * const SG13734[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61774, SGI0, SGI61775, SGI0, NULL};
const Segment * const SG13735[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61776, SGI0, SGI61777, SGI0, NULL};
const Segment * const SG13736[] = {SGI61778, SGI61779, SGI61780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61781, SGI0, SGI0, SGI0, SGI61782, SGI61782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13737[] = {SGI61783, SGI61784, SGI61785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61786, SGI0, SGI0, SGI0, SGI61787, SGI61787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13738[] = {SGI61788, SGI61789, SGI61790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61791, SGI0, SGI0, SGI0, SGI61792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13739[] = {SGI61793, SGI61794, SGI61795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61796, SGI0, SGI0, SGI0, SGI61797, SGI61797, SGI0, SGI61798, SGI0, NULL};
const Segment * const SG13740[] = {SGI61799, SGI61800, SGI61801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61802, SGI0, SGI0, SGI0, SGI61803, SGI61803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13741[] = {SGI61804, SGI61805, SGI61806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61807, SGI0, SGI0, SGI0, SGI61808, SGI61808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13742[] = {SGI61809, SGI61810, SGI61811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61812, SGI0, SGI0, SGI0, SGI61813, SGI61813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13743[] = {SGI61814, SGI61815, SGI61816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61817, SGI0, SGI0, SGI0, SGI61818, SGI61818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13744[] = {SGI61819, SGI61820, SGI61821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61822, SGI0, SGI0, SGI0, SGI61823, SGI61823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13745[] = {SGI61824, SGI61825, SGI61826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61827, SGI0, SGI0, SGI0, SGI61828, SGI61828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13746[] = {SGI61829, SGI61830, SGI61831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13747[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61833, SGI0, SGI61835, SGI0, NULL};
const Segment * const SG13748[] = {SGI61834, SGI0, SGI61836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13749[] = {SGI61837, SGI0, SGI61838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13750[] = {SGI61839, SGI61840, SGI61841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61842, SGI61843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13751[] = {SGI61844, SGI0, SGI61845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13752[] = {SGI61846, SGI0, SGI61847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61848, SGI0, SGI0, SGI0, SGI61849, SGI61849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13753[] = {SGI61850, SGI61851, SGI61852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61853, SGI0, SGI0, SGI0, SGI61854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13754[] = {SGI61855, SGI61856, SGI61857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61858, SGI0, SGI0, SGI0, SGI61859, SGI61859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13755[] = {SGI61860, SGI0, SGI61861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61862, SGI0, SGI0, SGI0, SGI61863, SGI61863, SGI0, SGI0, SGI61864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13756[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61865, SGI0, SGI61866, SGI0, NULL};
const Segment * const SG13757[] = {SGI61867, SGI61868, SGI61869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61870, SGI0, SGI0, SGI0, SGI61871, SGI61871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13758[] = {SGI61872, SGI0, SGI61873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13759[] = {SGI61874, SGI61875, SGI61876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61877, SGI61877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13760[] = {SGI61878, SGI61879, SGI61880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61881, SGI0, SGI0, SGI0, SGI61882, SGI61882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13761[] = {SGI61883, SGI61884, SGI61885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61886, SGI0, SGI0, SGI0, SGI61887, SGI61887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13762[] = {SGI61888, SGI61889, SGI61890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61891, SGI0, SGI0, SGI0, SGI61892, SGI61892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13763[] = {SGI61893, SGI61894, SGI61895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61896, SGI0, SGI0, SGI0, SGI61897, SGI61897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13764[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61898, SGI0, SGI61899, SGI0, NULL};
const Segment * const SG13765[] = {SGI61900, SGI0, SGI61901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61902, SGI0, SGI0, SGI0, SGI61903, SGI61903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13766[] = {SGI0, SGI61904, SGI61905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13767[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61906, SGI0, SGI61909, SGI0, NULL};
const Segment * const SG13768[] = {SGI61907, SGI61908, SGI61910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61911, SGI0, SGI0, SGI0, SGI61912, SGI61912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13769[] = {SGI61913, SGI61914, SGI61915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61916, SGI0, SGI0, SGI0, SGI61917, SGI61917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13770[] = {SGI61918, SGI61919, SGI61920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61921, SGI0, SGI0, SGI0, SGI61922, SGI61922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13771[] = {SGI61923, SGI0, SGI61924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61925, SGI0, SGI0, SGI0, SGI61926, SGI61926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13772[] = {SGI61927, SGI61928, SGI61929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61930, SGI0, SGI0, SGI0, SGI61931, SGI61931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13773[] = {SGI61932, SGI61933, SGI61934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61938, SGI0, SGI0, SGI0, SGI61939, SGI61939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13774[] = {SGI61935, SGI61936, SGI61937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61940, SGI61940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13775[] = {SGI61941, SGI61942, SGI61943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61944, SGI0, SGI0, SGI0, SGI61945, SGI61945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13776[] = {SGI61946, SGI0, SGI61947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61948, SGI0, SGI0, SGI0, SGI61949, SGI61949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13777[] = {SGI61950, SGI61951, SGI61952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61953, SGI61953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13778[] = {SGI61954, SGI61955, SGI61956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61960, SGI0, SGI0, SGI0, SGI61961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13779[] = {SGI61957, SGI61958, SGI61959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61962, SGI0, SGI0, SGI0, SGI61963, SGI61963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13780[] = {SGI61964, SGI61965, SGI61966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61967, SGI0, SGI0, SGI0, SGI61968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13781[] = {SGI61969, SGI61970, SGI61971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61972, SGI61972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13782[] = {SGI61973, SGI61974, SGI61975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61976, SGI0, SGI0, SGI0, SGI61977, SGI61977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13783[] = {SGI61978, SGI61979, SGI61980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61981, SGI0, SGI0, SGI0, SGI61983, SGI61983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13784[] = {SGI61982, SGI0, SGI61984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61985, SGI0, SGI0, SGI0, SGI61986, SGI61986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13785[] = {SGI61987, SGI61988, SGI61989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61990, SGI0, SGI0, SGI0, SGI61991, SGI61991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13786[] = {SGI61992, SGI61993, SGI61994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61995, SGI0, SGI0, SGI0, SGI61996, SGI61996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13787[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI61997, SGI0, SGI62000, SGI0, NULL};
const Segment * const SG13788[] = {SGI61998, SGI61999, SGI62001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62002, SGI0, SGI0, SGI0, SGI62003, SGI62003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13789[] = {SGI62004, SGI62005, SGI62006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62007, SGI0, SGI0, SGI0, SGI62008, SGI62008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13790[] = {SGI62009, SGI62010, SGI62011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62012, SGI0, SGI0, SGI0, SGI62013, SGI62013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13791[] = {SGI62014, SGI62015, SGI62016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62017, SGI0, SGI0, SGI0, SGI62018, SGI62018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13792[] = {SGI62019, SGI62020, SGI62021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62022, SGI0, SGI0, SGI0, SGI62023, SGI62023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13793[] = {SGI62024, SGI62025, SGI62026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62027, SGI0, SGI0, SGI0, SGI62030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13794[] = {SGI62028, SGI62029, SGI62031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62032, SGI0, SGI0, SGI0, SGI62033, SGI62033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13795[] = {SGI62034, SGI62035, SGI62036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62037, SGI0, SGI0, SGI0, SGI62038, SGI62038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13796[] = {SGI62039, SGI62040, SGI62041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62042, SGI0, SGI0, SGI0, SGI62043, SGI62043, SGI0, SGI62044, SGI0, NULL};
const Segment * const SG13797[] = {SGI62045, SGI62046, SGI62047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62048, SGI0, SGI0, SGI0, SGI62049, SGI62049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13798[] = {SGI62050, SGI62051, SGI62052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62053, SGI0, SGI0, SGI0, SGI62054, SGI62054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13799[] = {SGI62055, SGI62056, SGI62057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62058, SGI0, SGI0, SGI0, SGI62059, SGI62059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13800[] = {SGI62060, SGI62061, SGI62062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62063, SGI0, SGI0, SGI0, SGI62064, SGI62064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13801[] = {SGI62065, SGI62066, SGI62067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62068, SGI0, SGI0, SGI0, SGI62069, SGI62069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13802[] = {SGI62070, SGI62071, SGI62072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62073, SGI0, SGI0, SGI0, SGI62074, SGI62074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13803[] = {SGI62075, SGI62076, SGI62077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62078, SGI0, SGI0, SGI0, SGI62079, SGI62079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13804[] = {SGI62080, SGI62081, SGI62082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62083, SGI0, SGI0, SGI0, SGI62084, SGI62084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13805[] = {SGI62085, SGI62086, SGI62087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62088, SGI0, SGI0, SGI0, SGI62089, SGI62089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13806[] = {SGI62090, SGI62091, SGI62092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62093, SGI0, SGI0, SGI0, SGI62094, SGI62094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13807[] = {SGI62095, SGI62096, SGI62097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62098, SGI0, SGI0, SGI0, SGI62099, SGI62099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13808[] = {SGI62100, SGI62101, SGI62102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62103, SGI0, SGI0, SGI0, SGI62104, SGI62104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13809[] = {SGI62105, SGI62106, SGI62107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62108, SGI0, SGI0, SGI0, SGI62109, SGI62109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13810[] = {SGI62110, SGI62111, SGI62112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62113, SGI0, SGI0, SGI0, SGI62114, SGI62114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13811[] = {SGI62115, SGI62116, SGI62117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62118, SGI0, SGI0, SGI0, SGI62119, SGI62119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13812[] = {SGI62120, SGI62121, SGI62122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62123, SGI0, SGI0, SGI0, SGI62124, SGI62124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13813[] = {SGI62125, SGI62126, SGI62127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62128, SGI0, SGI0, SGI0, SGI62129, SGI62129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13814[] = {SGI62130, SGI62131, SGI62132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62133, SGI0, SGI0, SGI0, SGI62134, SGI62134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13815[] = {SGI62135, SGI62136, SGI62137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62138, SGI0, SGI0, SGI0, SGI62139, SGI62139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13816[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62143, SGI0, SGI62145, SGI0, NULL};
const Segment * const SG13817[] = {SGI62140, SGI62141, SGI62142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62144, SGI0, SGI0, SGI0, SGI62146, SGI62146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13818[] = {SGI62147, SGI62148, SGI62149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62150, SGI0, SGI0, SGI0, SGI62151, SGI62151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13819[] = {SGI62152, SGI62153, SGI62154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62155, SGI0, SGI0, SGI0, SGI62156, SGI62156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13820[] = {SGI62157, SGI62158, SGI62159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62160, SGI0, SGI0, SGI0, SGI62161, SGI62161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13821[] = {SGI62162, SGI62163, SGI62164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62165, SGI0, SGI0, SGI0, SGI62166, SGI62166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13822[] = {SGI62167, SGI62168, SGI62169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62170, SGI0, SGI0, SGI0, SGI62171, SGI62171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13823[] = {SGI62172, SGI62173, SGI62174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62175, SGI0, SGI0, SGI0, SGI62176, SGI62176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13824[] = {SGI62177, SGI62178, SGI62179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62180, SGI0, SGI0, SGI0, SGI62181, SGI62181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13825[] = {SGI62182, SGI62183, SGI62184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62185, SGI0, SGI0, SGI0, SGI62186, SGI62186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13826[] = {SGI0, SGI0, SGI62187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62188, SGI0, SGI0, SGI0, SGI62189, SGI62189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13827[] = {SGI62190, SGI62191, SGI62192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62193, SGI0, SGI0, SGI0, SGI62194, SGI62194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13828[] = {SGI62195, SGI62196, SGI62197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62198, SGI0, SGI0, SGI0, SGI62199, SGI62199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13829[] = {SGI62200, SGI62201, SGI62202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62203, SGI0, SGI0, SGI0, SGI62204, SGI62204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13830[] = {SGI62205, SGI62206, SGI62207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62208, SGI0, SGI0, SGI0, SGI62209, SGI62209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13831[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62211, SGI0, SGI62214, SGI0, NULL};
const Segment * const SG13832[] = {SGI62210, SGI62212, SGI62213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62215, SGI0, SGI0, SGI0, SGI62216, SGI62216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13833[] = {SGI62217, SGI62218, SGI62219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62220, SGI0, SGI0, SGI0, SGI62221, SGI62221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13834[] = {SGI62222, SGI62223, SGI62224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62225, SGI0, SGI0, SGI0, SGI62226, SGI62226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13835[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62227, SGI0, SGI62229, SGI0, NULL};
const Segment * const SG13836[] = {SGI62228, SGI62230, SGI62231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62232, SGI0, SGI0, SGI0, SGI62233, SGI62233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13837[] = {SGI62234, SGI62235, SGI62236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62237, SGI0, SGI0, SGI0, SGI62238, SGI62238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13838[] = {SGI62239, SGI62240, SGI62241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62242, SGI0, SGI0, SGI0, SGI62243, SGI62243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13839[] = {SGI62244, SGI62245, SGI62246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62247, SGI0, SGI0, SGI0, SGI62248, SGI62248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13840[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62249, SGI0, SGI62250, SGI0, NULL};
const Segment * const SG13841[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62251, SGI0, SGI62252, SGI0, NULL};
const Segment * const SG13842[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62253, SGI0, SGI62254, SGI0, NULL};
const Segment * const SG13843[] = {SGI62255, SGI62256, SGI62257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62258, SGI0, SGI0, SGI0, SGI62259, SGI62259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13844[] = {SGI62260, SGI62261, SGI62262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62263, SGI0, SGI0, SGI0, SGI62264, SGI62264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13845[] = {SGI62265, SGI62266, SGI62267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62268, SGI0, SGI0, SGI0, SGI62269, SGI62269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13846[] = {SGI62270, SGI62271, SGI62272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62273, SGI0, SGI0, SGI0, SGI62274, SGI62274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13847[] = {SGI62275, SGI62276, SGI62277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62278, SGI0, SGI0, SGI0, SGI62279, SGI62279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13848[] = {SGI62280, SGI62281, SGI62282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62283, SGI0, SGI0, SGI0, SGI62284, SGI62284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13849[] = {SGI62285, SGI62286, SGI62287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62288, SGI0, SGI0, SGI0, SGI62289, SGI62289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13850[] = {SGI62290, SGI62291, SGI62292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62293, SGI0, SGI0, SGI0, SGI62294, SGI62294, SGI0, SGI62295, SGI0, NULL};
const Segment * const SG13851[] = {SGI62296, SGI62297, SGI62298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62299, SGI0, SGI0, SGI0, SGI62300, SGI62300, SGI0, SGI62301, SGI0, NULL};
const Segment * const SG13852[] = {SGI62302, SGI62303, SGI62304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62305, SGI0, SGI0, SGI0, SGI0, SGI62306, SGI0, SGI62307, SGI0, NULL};
const Segment * const SG13853[] = {SGI62308, SGI62309, SGI62310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62311, SGI0, SGI0, SGI0, SGI0, SGI62312, SGI0, SGI62313, SGI0, NULL};
const Segment * const SG13854[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62314, SGI0, SGI62315, SGI0, NULL};
const Segment * const SG13855[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62316, SGI0, SGI62317, SGI0, NULL};
const Segment * const SG13856[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62318, SGI0, SGI62319, SGI0, NULL};
const Segment * const SG13857[] = {SGI62320, SGI62321, SGI62322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62323, SGI0, SGI0, SGI0, SGI62324, SGI62324, SGI0, SGI62325, SGI0, NULL};
const Segment * const SG13858[] = {SGI62326, SGI62328, SGI62329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62332, SGI0, SGI0, SGI0, SGI62334, SGI62334, SGI0, SGI62336, SGI0, NULL};
const Segment * const SG13859[] = {SGI62327, SGI62330, SGI62331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62333, SGI0, SGI0, SGI0, SGI62335, SGI62335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13860[] = {SGI62337, SGI62339, SGI62340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62343, SGI0, SGI0, SGI0, SGI62345, SGI62345, SGI0, SGI62347, SGI0, NULL};
const Segment * const SG13861[] = {SGI62338, SGI62341, SGI62342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62344, SGI0, SGI0, SGI0, SGI62346, SGI62346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13862[] = {SGI62348, SGI62349, SGI62350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62351, SGI0, SGI0, SGI0, SGI62352, SGI62352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13863[] = {SGI62353, SGI62354, SGI62355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62356, SGI0, SGI0, SGI0, SGI62357, SGI62357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13864[] = {SGI62358, SGI62359, SGI62360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62361, SGI0, SGI0, SGI0, SGI62362, SGI62362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13865[] = {SGI62363, SGI62364, SGI62365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62366, SGI0, SGI0, SGI0, SGI62367, SGI62367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13866[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62368, SGI0, SGI62369, SGI0, NULL};
const Segment * const SG13867[] = {SGI62370, SGI62371, SGI62372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62373, SGI0, SGI0, SGI0, SGI62374, SGI62374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13868[] = {SGI62375, SGI62376, SGI62377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62378, SGI0, SGI0, SGI0, SGI62379, SGI62379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13869[] = {SGI62380, SGI62381, SGI62382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62383, SGI0, SGI0, SGI0, SGI62384, SGI62384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13870[] = {SGI62385, SGI62386, SGI62387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62388, SGI0, SGI0, SGI0, SGI62389, SGI62389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13871[] = {SGI62390, SGI62391, SGI62392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62393, SGI0, SGI0, SGI0, SGI62394, SGI62394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13872[] = {SGI62395, SGI62396, SGI62397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62398, SGI0, SGI0, SGI0, SGI62399, SGI62399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13873[] = {SGI62403, SGI62404, SGI62405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62406, SGI0, SGI0, SGI0, SGI62407, SGI62407, SGI0, SGI62408, SGI0, NULL};
const Segment * const SG13874[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62409, SGI0, SGI62410, SGI0, NULL};
const Segment * const SG13875[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62411, SGI0, SGI62412, SGI0, NULL};
const Segment * const SG13876[] = {SGI62413, SGI62414, SGI62415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62416, SGI0, SGI0, SGI0, SGI62417, SGI62417, SGI0, SGI62418, SGI0, NULL};
const Segment * const SG13877[] = {SGI62400, SGI62401, SGI62402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62419, SGI0, SGI0, SGI0, SGI62420, SGI62420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13878[] = {SGI62421, SGI62422, SGI62423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62424, SGI0, SGI0, SGI0, SGI62425, SGI62425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13879[] = {SGI62426, SGI62427, SGI62428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62429, SGI0, SGI0, SGI0, SGI62430, SGI62430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13880[] = {SGI62431, SGI62432, SGI62433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62434, SGI0, SGI0, SGI0, SGI62435, SGI62435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13881[] = {SGI62436, SGI62437, SGI62438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62439, SGI0, SGI0, SGI0, SGI62440, SGI62440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13882[] = {SGI62441, SGI62442, SGI62443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62444, SGI0, SGI0, SGI0, SGI62445, SGI62445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13883[] = {SGI62446, SGI62447, SGI62448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62449, SGI0, SGI0, SGI0, SGI62450, SGI62450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13884[] = {SGI62451, SGI62452, SGI62453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62454, SGI0, SGI0, SGI0, SGI62455, SGI62455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13885[] = {SGI62456, SGI62457, SGI62458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62459, SGI0, SGI0, SGI0, SGI62460, SGI62460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13886[] = {SGI62461, SGI62462, SGI62463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62464, SGI0, SGI0, SGI0, SGI62465, SGI62465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13887[] = {SGI62466, SGI62467, SGI62468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62469, SGI0, SGI0, SGI0, SGI62470, SGI62470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13888[] = {SGI62471, SGI62472, SGI62473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62474, SGI0, SGI0, SGI0, SGI62475, SGI62475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13889[] = {SGI62476, SGI62477, SGI62478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62479, SGI0, SGI0, SGI0, SGI62480, SGI62480, SGI0, SGI62481, SGI0, NULL};
const Segment * const SG13890[] = {SGI62482, SGI62483, SGI62484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62485, SGI0, SGI0, SGI0, SGI62486, SGI62486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13891[] = {SGI62487, SGI62488, SGI62489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62490, SGI0, SGI0, SGI0, SGI62491, SGI62491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13892[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62492, SGI0, SGI62493, SGI0, NULL};
const Segment * const SG13893[] = {SGI62494, SGI62495, SGI62496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62497, SGI0, SGI0, SGI0, SGI62498, SGI62498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13894[] = {SGI62499, SGI62500, SGI62501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62502, SGI0, SGI0, SGI0, SGI62503, SGI62503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13895[] = {SGI62504, SGI62505, SGI62506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62507, SGI0, SGI0, SGI0, SGI62508, SGI62508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13896[] = {SGI62509, SGI62510, SGI62511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62512, SGI0, SGI0, SGI0, SGI62513, SGI62513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13897[] = {SGI62514, SGI62515, SGI62516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62517, SGI0, SGI0, SGI0, SGI62518, SGI62518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13898[] = {SGI62519, SGI62520, SGI62521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62522, SGI0, SGI0, SGI0, SGI62523, SGI62523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13899[] = {SGI62524, SGI62525, SGI62526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62527, SGI0, SGI0, SGI0, SGI62528, SGI62528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13900[] = {SGI62529, SGI62530, SGI62531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62532, SGI0, SGI0, SGI0, SGI62533, SGI62533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13901[] = {SGI62534, SGI62535, SGI62536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62537, SGI0, SGI0, SGI0, SGI62538, SGI62538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13902[] = {SGI62539, SGI62540, SGI62541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62542, SGI0, SGI0, SGI0, SGI62543, SGI62543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13903[] = {SGI62544, SGI62545, SGI62546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62547, SGI0, SGI0, SGI0, SGI62548, SGI62548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13904[] = {SGI62549, SGI62550, SGI62551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62552, SGI0, SGI0, SGI0, SGI62553, SGI62553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13905[] = {SGI62554, SGI62555, SGI62556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62557, SGI0, SGI0, SGI0, SGI62558, SGI62558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13906[] = {SGI62559, SGI62560, SGI62561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62562, SGI0, SGI0, SGI0, SGI62563, SGI62563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13907[] = {SGI62564, SGI0, SGI62565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13908[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62566, SGI0, SGI62567, SGI0, NULL};
const Segment * const SG13909[] = {SGI62568, SGI62569, SGI62570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62571, SGI0, SGI0, SGI0, SGI62572, SGI62572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13910[] = {SGI62573, SGI62574, SGI62575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62576, SGI0, SGI0, SGI0, SGI62577, SGI62577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13911[] = {SGI62578, SGI62579, SGI62580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62581, SGI0, SGI0, SGI0, SGI62582, SGI62582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13912[] = {SGI62583, SGI62584, SGI62585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62586, SGI0, SGI0, SGI0, SGI62587, SGI62587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13913[] = {SGI62588, SGI0, SGI62589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13914[] = {SGI62590, SGI62591, SGI62592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62593, SGI0, SGI0, SGI0, SGI62594, SGI62594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13915[] = {SGI62595, SGI0, SGI62596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13916[] = {SGI62597, SGI62598, SGI62599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62600, SGI0, SGI0, SGI0, SGI62601, SGI62601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13917[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62602, SGI0, SGI62603, SGI0, NULL};
const Segment * const SG13918[] = {SGI62604, SGI62605, SGI62606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62607, SGI0, SGI0, SGI0, SGI62608, SGI62608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13919[] = {SGI62609, SGI62610, SGI62611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62612, SGI0, SGI0, SGI0, SGI62613, SGI62613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13920[] = {SGI62614, SGI62615, SGI62616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62617, SGI0, SGI0, SGI0, SGI62618, SGI62618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13921[] = {SGI62619, SGI62620, SGI62621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62622, SGI0, SGI0, SGI0, SGI62623, SGI62623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13922[] = {SGI62624, SGI62625, SGI62626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62627, SGI0, SGI0, SGI0, SGI62628, SGI62628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13923[] = {SGI62629, SGI62630, SGI62631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62632, SGI0, SGI0, SGI0, SGI62633, SGI62633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13924[] = {SGI62634, SGI62635, SGI62636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62637, SGI0, SGI0, SGI0, SGI62638, SGI62638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13925[] = {SGI62639, SGI62640, SGI62641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62642, SGI0, SGI0, SGI0, SGI62643, SGI62643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13926[] = {SGI62644, SGI62645, SGI62646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62647, SGI0, SGI0, SGI0, SGI62648, SGI62648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13927[] = {SGI0, SGI62649, SGI62650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62651, SGI0, SGI0, SGI0, SGI62652, SGI62652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13928[] = {SGI62653, SGI62654, SGI62655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62656, SGI0, SGI0, SGI0, SGI62657, SGI62657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13929[] = {SGI62658, SGI62659, SGI62660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62661, SGI0, SGI0, SGI0, SGI62662, SGI62662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13930[] = {SGI62663, SGI0, SGI62664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62665, SGI0, SGI0, SGI0, SGI62666, SGI62666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13931[] = {SGI62667, SGI62668, SGI62669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62670, SGI0, SGI0, SGI0, SGI62671, SGI62671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13932[] = {SGI62672, SGI62673, SGI62674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62675, SGI0, SGI0, SGI0, SGI62676, SGI62676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13933[] = {SGI62677, SGI62678, SGI62679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62680, SGI0, SGI0, SGI0, SGI62681, SGI62681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13934[] = {SGI62682, SGI62683, SGI62684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62685, SGI0, SGI0, SGI0, SGI62686, SGI62686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13935[] = {SGI62687, SGI62688, SGI62689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62690, SGI0, SGI0, SGI0, SGI62691, SGI62691, SGI0, SGI62692, SGI0, NULL};
const Segment * const SG13936[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62693, SGI0, SGI62694, SGI0, NULL};
const Segment * const SG13937[] = {SGI62695, SGI62696, SGI62697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62698, SGI0, SGI0, SGI0, SGI62699, SGI62699, SGI0, SGI62700, SGI0, NULL};
const Segment * const SG13938[] = {SGI62701, SGI62702, SGI62703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62704, SGI0, SGI0, SGI0, SGI0, SGI62705, SGI0, SGI62706, SGI0, NULL};
const Segment * const SG13939[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62707, SGI0, SGI62708, SGI0, NULL};
const Segment * const SG13940[] = {SGI62709, SGI62710, SGI62711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62712, SGI0, SGI0, SGI0, SGI62713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13941[] = {SGI62714, SGI62715, SGI62716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62717, SGI0, SGI0, SGI0, SGI62718, SGI62718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13942[] = {SGI62719, SGI62720, SGI62721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62722, SGI0, SGI0, SGI0, SGI62723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13943[] = {SGI62724, SGI62725, SGI62726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62727, SGI0, SGI0, SGI0, SGI62728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13944[] = {SGI62729, SGI62730, SGI62731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62732, SGI0, SGI0, SGI0, SGI62733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13945[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62734, SGI0, SGI62735, SGI0, NULL};
const Segment * const SG13946[] = {SGI62736, SGI62737, SGI62738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62739, SGI0, SGI0, SGI0, SGI62740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13947[] = {SGI62741, SGI62742, SGI62743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62744, SGI0, SGI0, SGI0, SGI62745, SGI62745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13948[] = {SGI62746, SGI62747, SGI62748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62749, SGI0, SGI0, SGI0, SGI62750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13949[] = {SGI62751, SGI62752, SGI62753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62754, SGI0, SGI0, SGI0, SGI62755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13950[] = {SGI62756, SGI62757, SGI62758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62759, SGI0, SGI0, SGI0, SGI62760, SGI62760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13951[] = {SGI62761, SGI62762, SGI62763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62764, SGI0, SGI0, SGI0, SGI62765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13952[] = {SGI62766, SGI62767, SGI62768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62769, SGI0, SGI0, SGI0, SGI62770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13953[] = {SGI62771, SGI62772, SGI62773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62774, SGI0, SGI0, SGI0, SGI62775, SGI62775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13954[] = {SGI62776, SGI62777, SGI62778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62779, SGI0, SGI0, SGI0, SGI62780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13955[] = {SGI62781, SGI62782, SGI62783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62784, SGI0, SGI0, SGI0, SGI62785, SGI62785, SGI0, SGI62786, SGI0, NULL};
const Segment * const SG13956[] = {SGI62787, SGI62788, SGI62789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62790, SGI0, SGI0, SGI0, SGI62791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13957[] = {SGI62792, SGI62793, SGI62794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62795, SGI0, SGI0, SGI0, SGI62796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13958[] = {SGI62797, SGI62798, SGI62799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62800, SGI0, SGI0, SGI0, SGI62801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13959[] = {SGI62802, SGI62803, SGI62804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62805, SGI0, SGI0, SGI0, SGI62806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13960[] = {SGI62807, SGI62808, SGI62809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62810, SGI0, SGI0, SGI0, SGI62811, SGI62811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13961[] = {SGI62812, SGI62813, SGI62814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62815, SGI0, SGI0, SGI0, SGI62816, SGI62816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13962[] = {SGI62817, SGI62818, SGI62819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62820, SGI0, SGI0, SGI0, SGI62821, SGI62821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13963[] = {SGI62822, SGI62823, SGI62824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62825, SGI0, SGI0, SGI0, SGI62826, SGI62826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13964[] = {SGI62827, SGI62828, SGI62829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62830, SGI0, SGI0, SGI0, SGI62831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13965[] = {SGI62832, SGI62833, SGI62834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62835, SGI0, SGI0, SGI0, SGI62836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13966[] = {SGI62837, SGI62838, SGI62839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62840, SGI0, SGI0, SGI0, SGI62841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13967[] = {SGI62842, SGI62843, SGI62844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62845, SGI0, SGI0, SGI0, SGI62846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13968[] = {SGI62847, SGI62848, SGI62849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62850, SGI0, SGI0, SGI0, SGI62851, SGI62851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13969[] = {SGI62852, SGI62853, SGI62854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62855, SGI0, SGI0, SGI0, SGI62856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13970[] = {SGI62857, SGI62858, SGI62859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62860, SGI0, SGI0, SGI0, SGI62861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13971[] = {SGI62862, SGI62863, SGI62864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62865, SGI0, SGI0, SGI0, SGI62866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13972[] = {SGI62867, SGI62868, SGI62869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62870, SGI0, SGI0, SGI0, SGI62871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13973[] = {SGI62872, SGI62873, SGI62874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62875, SGI0, SGI0, SGI0, SGI62876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13974[] = {SGI62877, SGI62878, SGI62879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62880, SGI0, SGI0, SGI0, SGI62881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13975[] = {SGI62882, SGI62883, SGI62884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62885, SGI0, SGI0, SGI0, SGI62886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13976[] = {SGI62887, SGI62888, SGI62889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62890, SGI0, SGI0, SGI0, SGI62891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13977[] = {SGI62892, SGI62893, SGI62894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62895, SGI0, SGI0, SGI0, SGI62896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13978[] = {SGI62897, SGI62898, SGI62899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62900, SGI0, SGI0, SGI0, SGI62901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13979[] = {SGI62902, SGI62903, SGI62904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62905, SGI0, SGI0, SGI0, SGI62906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13980[] = {SGI62907, SGI62908, SGI62909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62910, SGI0, SGI0, SGI0, SGI62911, SGI62911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13981[] = {SGI62912, SGI62913, SGI62914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62915, SGI0, SGI0, SGI0, SGI62916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13982[] = {SGI62917, SGI62918, SGI62919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62920, SGI0, SGI0, SGI0, SGI62921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13983[] = {SGI62922, SGI62923, SGI62924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62925, SGI0, SGI0, SGI0, SGI62926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13984[] = {SGI62927, SGI62928, SGI62929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62930, SGI0, SGI0, SGI0, SGI62931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13985[] = {SGI62932, SGI62933, SGI62934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62935, SGI0, SGI0, SGI0, SGI62936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13986[] = {SGI62937, SGI62938, SGI62939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62940, SGI0, SGI0, SGI0, SGI62941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13987[] = {SGI62942, SGI62943, SGI62944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62948, SGI0, SGI0, SGI0, SGI62950, SGI62950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13988[] = {SGI62945, SGI62946, SGI62947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62949, SGI0, SGI0, SGI0, SGI62951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13989[] = {SGI62952, SGI62953, SGI62955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62958, SGI0, SGI0, SGI0, SGI62960, SGI62960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13990[] = {SGI62954, SGI62956, SGI62957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62959, SGI0, SGI0, SGI0, SGI62961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13991[] = {SGI62962, SGI62964, SGI62966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62968, SGI0, SGI0, SGI0, SGI62970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13992[] = {SGI62963, SGI62965, SGI62967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62969, SGI0, SGI0, SGI0, SGI62971, SGI62971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13993[] = {SGI62972, SGI62973, SGI62974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62978, SGI0, SGI0, SGI0, SGI62979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13994[] = {SGI62975, SGI62976, SGI62977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62980, SGI0, SGI0, SGI0, SGI62981, SGI62981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13995[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62984, SGI0, SGI62986, SGI0, NULL};
const Segment * const SG13996[] = {SGI62982, SGI62983, SGI62985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62988, SGI0, SGI0, SGI0, SGI62991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13997[] = {SGI62987, SGI62989, SGI62990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62992, SGI0, SGI0, SGI0, SGI62993, SGI62993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13998[] = {SGI62994, SGI62995, SGI62996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI62997, SGI0, SGI0, SGI0, SGI62998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG13999[] = {SGI62999, SGI63000, SGI63001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63002, SGI0, SGI0, SGI0, SGI63003, SGI63003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14000[] = {SGI63004, SGI63005, SGI63006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63007, SGI0, SGI0, SGI0, SGI63008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14001[] = {SGI63009, SGI63010, SGI63011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63012, SGI0, SGI0, SGI0, SGI63013, SGI63013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14002[] = {SGI63014, SGI63015, SGI63016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63020, SGI0, SGI0, SGI0, SGI63021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14003[] = {SGI63017, SGI63018, SGI63019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63022, SGI0, SGI0, SGI0, SGI63023, SGI63023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14004[] = {SGI63024, SGI63025, SGI63026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63030, SGI0, SGI0, SGI0, SGI63031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14005[] = {SGI63027, SGI63028, SGI63029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63032, SGI0, SGI0, SGI0, SGI63033, SGI63033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14006[] = {SGI63034, SGI63035, SGI63036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63038, SGI0, SGI0, SGI0, SGI63040, SGI63040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14007[] = {SGI63037, SGI63039, SGI63041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63042, SGI0, SGI0, SGI0, SGI63043, SGI63043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14008[] = {SGI63044, SGI63045, SGI63046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63048, SGI0, SGI0, SGI0, SGI63050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14009[] = {SGI63047, SGI0, SGI63049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14010[] = {SGI63051, SGI63053, SGI63054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63057, SGI0, SGI0, SGI0, SGI63058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14011[] = {SGI63052, SGI63055, SGI63056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63059, SGI63059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14012[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63060, SGI0, SGI63061, SGI0, NULL};
const Segment * const SG14013[] = {SGI63062, SGI63063, SGI63064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63065, SGI0, SGI0, SGI0, SGI63066, SGI63066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14014[] = {SGI63067, SGI63069, SGI63071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63073, SGI0, SGI0, SGI0, SGI63074, SGI63074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14015[] = {SGI63068, SGI63070, SGI63072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63075, SGI63075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14016[] = {SGI63076, SGI0, SGI63078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14017[] = {SGI63077, SGI63079, SGI63080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63081, SGI0, SGI0, SGI0, SGI63082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14018[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63083, SGI0, SGI63084, SGI0, NULL};
const Segment * const SG14019[] = {SGI63085, SGI63086, SGI63087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63089, SGI0, SGI0, SGI0, SGI63092, SGI63092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14020[] = {SGI63088, SGI63090, SGI63091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63093, SGI0, SGI0, SGI0, SGI63094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14021[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63095, SGI0, SGI63096, SGI0, NULL};
const Segment * const SG14022[] = {SGI63098, SGI63100, SGI63102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63103, SGI0, SGI0, SGI0, SGI63105, SGI63105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14023[] = {SGI63097, SGI63099, SGI63101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63104, SGI0, SGI0, SGI0, SGI63106, SGI63106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14024[] = {SGI63108, SGI63110, SGI63112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63115, SGI0, SGI0, SGI0, SGI63118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14025[] = {SGI63107, SGI63109, SGI63111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63116, SGI0, SGI0, SGI0, SGI63119, SGI63119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14026[] = {SGI63113, SGI63114, SGI63117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63120, SGI0, SGI0, SGI0, SGI63121, SGI63121, SGI0, SGI63122, SGI0, NULL};
const Segment * const SG14027[] = {SGI63123, SGI63124, SGI63125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63129, SGI0, SGI0, SGI0, SGI63130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14028[] = {SGI63126, SGI63127, SGI63128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63131, SGI0, SGI0, SGI0, SGI63132, SGI63132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14029[] = {SGI63133, SGI63134, SGI63135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63136, SGI0, SGI0, SGI0, SGI63138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14030[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63141, SGI0, SGI63142, SGI0, NULL};
const Segment * const SG14031[] = {SGI63137, SGI63139, SGI63140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63143, SGI0, SGI0, SGI0, SGI63144, SGI63144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14032[] = {SGI63145, SGI63146, SGI63147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63148, SGI0, SGI0, SGI0, SGI63149, SGI63149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14033[] = {SGI63150, SGI63151, SGI63152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63153, SGI0, SGI0, SGI0, SGI63154, SGI63154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14034[] = {SGI63155, SGI63156, SGI63157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63158, SGI0, SGI0, SGI0, SGI63159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14035[] = {SGI63160, SGI63161, SGI63162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63163, SGI0, SGI0, SGI0, SGI63164, SGI63164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14036[] = {SGI63165, SGI63166, SGI63167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63168, SGI0, SGI0, SGI0, SGI63169, SGI63169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14037[] = {SGI63170, SGI63171, SGI63172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63173, SGI0, SGI0, SGI0, SGI63174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14038[] = {SGI63175, SGI63176, SGI63177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63178, SGI0, SGI0, SGI0, SGI63181, SGI63181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14039[] = {SGI63179, SGI63180, SGI63182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63183, SGI0, SGI0, SGI0, SGI63184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14040[] = {SGI63185, SGI63186, SGI63188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63191, SGI0, SGI0, SGI0, SGI63193, SGI63193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14041[] = {SGI63187, SGI63189, SGI63190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63192, SGI0, SGI0, SGI0, SGI63194, SGI63194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14042[] = {SGI63196, SGI63198, SGI63200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63202, SGI0, SGI0, SGI0, SGI63204, SGI63204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14043[] = {SGI63195, SGI63197, SGI63199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63201, SGI0, SGI0, SGI0, SGI63203, SGI63203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14044[] = {SGI63205, SGI63207, SGI63209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63211, SGI0, SGI0, SGI0, SGI63213, SGI63213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14045[] = {SGI63206, SGI63208, SGI63210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63212, SGI0, SGI0, SGI0, SGI63214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14046[] = {SGI63215, SGI63216, SGI63217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63218, SGI0, SGI0, SGI0, SGI63219, SGI63219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14047[] = {SGI63220, SGI63221, SGI63222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63223, SGI0, SGI0, SGI0, SGI63224, SGI63224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14048[] = {SGI63225, SGI63226, SGI63227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63228, SGI0, SGI0, SGI0, SGI63229, SGI63229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14049[] = {SGI63233, SGI63234, SGI63235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63236, SGI0, SGI0, SGI0, SGI63237, SGI63237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14050[] = {SGI63238, SGI0, SGI63239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63240, SGI0, SGI0, SGI0, SGI63241, SGI63241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14051[] = {SGI63242, SGI63243, SGI63244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63245, SGI0, SGI0, SGI0, SGI63246, SGI63246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14052[] = {SGI63247, SGI0, SGI63248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63249, SGI0, SGI0, SGI0, SGI63250, SGI63250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14053[] = {SGI63251, SGI63252, SGI63253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63254, SGI0, SGI0, SGI0, SGI63255, SGI63255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14054[] = {SGI63230, SGI63231, SGI63232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63256, SGI0, SGI0, SGI0, SGI63257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14055[] = {SGI63258, SGI63259, SGI63260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63261, SGI0, SGI0, SGI0, SGI63262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14056[] = {SGI63263, SGI63264, SGI63265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63266, SGI0, SGI0, SGI0, SGI63267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14057[] = {SGI63268, SGI63269, SGI63270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63271, SGI0, SGI0, SGI0, SGI63272, SGI63272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14058[] = {SGI63273, SGI63274, SGI63275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63276, SGI0, SGI0, SGI0, SGI63277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14059[] = {SGI63278, SGI63279, SGI63280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63281, SGI0, SGI0, SGI0, SGI63282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14060[] = {SGI63283, SGI63284, SGI63285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63286, SGI0, SGI0, SGI0, SGI63287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14061[] = {SGI63288, SGI63289, SGI63290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63291, SGI0, SGI0, SGI0, SGI63292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14062[] = {SGI63293, SGI63294, SGI63295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63296, SGI0, SGI0, SGI0, SGI63297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14063[] = {SGI63298, SGI63299, SGI63300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63301, SGI0, SGI0, SGI0, SGI63302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14064[] = {SGI63303, SGI63304, SGI63305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63306, SGI0, SGI0, SGI0, SGI63307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14065[] = {SGI63308, SGI63309, SGI63310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63311, SGI0, SGI0, SGI0, SGI63312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14066[] = {SGI63313, SGI0, SGI63314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14067[] = {SGI63315, SGI63316, SGI63317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63318, SGI0, SGI0, SGI0, SGI63319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14068[] = {SGI63320, SGI63321, SGI63322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63323, SGI0, SGI0, SGI0, SGI63324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14069[] = {SGI63325, SGI63326, SGI63327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63328, SGI0, SGI0, SGI0, SGI63329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14070[] = {SGI63330, SGI63331, SGI63332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63333, SGI0, SGI0, SGI0, SGI63334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14071[] = {SGI63335, SGI63336, SGI63337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63338, SGI0, SGI0, SGI0, SGI63339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14072[] = {SGI63340, SGI63341, SGI63342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63343, SGI0, SGI0, SGI0, SGI63344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14073[] = {SGI63345, SGI63346, SGI63347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63348, SGI0, SGI0, SGI0, SGI63349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14074[] = {SGI63350, SGI63351, SGI63352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63353, SGI0, SGI0, SGI0, SGI63354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14075[] = {SGI63355, SGI63356, SGI63357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63358, SGI0, SGI0, SGI0, SGI63359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14076[] = {SGI63360, SGI63361, SGI63362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63363, SGI0, SGI0, SGI0, SGI63364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14077[] = {SGI63365, SGI63366, SGI63367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63368, SGI0, SGI0, SGI0, SGI63369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14078[] = {SGI63370, SGI63371, SGI63372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63373, SGI0, SGI0, SGI0, SGI63374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14079[] = {SGI63375, SGI63376, SGI63377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63378, SGI0, SGI0, SGI0, SGI63379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14080[] = {SGI63380, SGI63381, SGI63382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63385, SGI0, SGI0, SGI0, SGI63387, SGI63387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14081[] = {SGI63383, SGI63384, SGI63386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63388, SGI0, SGI0, SGI0, SGI63389, SGI63389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14082[] = {SGI63390, SGI63391, SGI63392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63396, SGI0, SGI0, SGI0, SGI63398, SGI63398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14083[] = {SGI63393, SGI63394, SGI63395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63397, SGI0, SGI0, SGI0, SGI63399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14084[] = {SGI63400, SGI63402, SGI63404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63406, SGI0, SGI0, SGI0, SGI63409, SGI63409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14085[] = {SGI63401, SGI63403, SGI63405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63407, SGI0, SGI0, SGI0, SGI63408, SGI63408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14086[] = {SGI63410, SGI63411, SGI63412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63416, SGI0, SGI0, SGI0, SGI63417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14087[] = {SGI63413, SGI63414, SGI63415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63418, SGI0, SGI0, SGI0, SGI63419, SGI63419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14088[] = {SGI63421, SGI0, SGI63423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14089[] = {SGI63420, SGI63422, SGI63424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63425, SGI0, SGI0, SGI0, SGI63426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14090[] = {SGI63427, SGI63428, SGI63429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63430, SGI0, SGI0, SGI0, SGI63431, SGI63431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14091[] = {SGI63432, SGI63433, SGI63434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63435, SGI0, SGI0, SGI0, SGI63436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14092[] = {SGI63437, SGI63438, SGI63439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63440, SGI0, SGI0, SGI0, SGI63441, SGI63441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14093[] = {SGI63442, SGI63443, SGI63444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63445, SGI0, SGI0, SGI0, SGI63446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14094[] = {SGI63447, SGI63448, SGI63449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63450, SGI0, SGI0, SGI0, SGI63453, SGI63453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14095[] = {SGI63451, SGI63452, SGI63454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63455, SGI0, SGI0, SGI0, SGI63456, SGI63456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14096[] = {SGI63457, SGI63458, SGI63459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63462, SGI0, SGI0, SGI0, SGI63464, SGI63464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14097[] = {SGI63460, SGI63461, SGI63463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63465, SGI0, SGI0, SGI0, SGI63466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14098[] = {SGI63467, SGI63468, SGI63470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63473, SGI0, SGI0, SGI0, SGI63475, SGI63475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14099[] = {SGI63469, SGI63471, SGI63472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63474, SGI0, SGI0, SGI0, SGI63476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14100[] = {SGI63477, SGI63478, SGI63480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63483, SGI0, SGI0, SGI0, SGI63484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14101[] = {SGI63479, SGI63481, SGI63482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63485, SGI0, SGI0, SGI0, SGI63486, SGI63486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14102[] = {SGI63487, SGI63488, SGI63489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63490, SGI0, SGI0, SGI0, SGI63491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14103[] = {SGI63492, SGI63493, SGI63494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63495, SGI0, SGI0, SGI0, SGI63496, SGI63496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14104[] = {SGI63497, SGI63498, SGI63499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63500, SGI0, SGI0, SGI0, SGI63501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14105[] = {SGI63502, SGI63503, SGI63504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63505, SGI0, SGI0, SGI0, SGI63506, SGI63506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14106[] = {SGI63507, SGI63508, SGI63509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63510, SGI0, SGI0, SGI0, SGI63511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14107[] = {SGI63512, SGI0, SGI63513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14108[] = {SGI63514, SGI63515, SGI63516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63517, SGI0, SGI0, SGI0, SGI63519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14109[] = {SGI63518, SGI63520, SGI63521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63522, SGI0, SGI0, SGI0, SGI63523, SGI63523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14110[] = {SGI63524, SGI63525, SGI63526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63527, SGI0, SGI0, SGI0, SGI63528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14111[] = {SGI63529, SGI63530, SGI63531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63532, SGI0, SGI0, SGI0, SGI63533, SGI63533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14112[] = {SGI63534, SGI63535, SGI63536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63537, SGI0, SGI0, SGI0, SGI63538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14113[] = {SGI63539, SGI63540, SGI63541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63542, SGI0, SGI0, SGI0, SGI63543, SGI63543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14114[] = {SGI63544, SGI63545, SGI63546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63549, SGI0, SGI0, SGI0, SGI63551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14115[] = {SGI63547, SGI63548, SGI63550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63552, SGI0, SGI0, SGI0, SGI63553, SGI63553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14116[] = {SGI63554, SGI63555, SGI63556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63558, SGI0, SGI0, SGI0, SGI63561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14117[] = {SGI63557, SGI63559, SGI63560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63562, SGI0, SGI0, SGI0, SGI63563, SGI63563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14118[] = {SGI63564, SGI63565, SGI63566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63567, SGI0, SGI0, SGI0, SGI63568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14119[] = {SGI63569, SGI63570, SGI63571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63572, SGI0, SGI0, SGI0, SGI63573, SGI63573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14120[] = {SGI63574, SGI63575, SGI63576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63577, SGI0, SGI0, SGI0, SGI63578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14121[] = {SGI63579, SGI63580, SGI63581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63582, SGI0, SGI0, SGI0, SGI63583, SGI63583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14122[] = {SGI63584, SGI63585, SGI63586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63587, SGI0, SGI0, SGI0, SGI63588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14123[] = {SGI63589, SGI0, SGI63590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63591, SGI0, SGI0, SGI0, SGI63592, SGI63592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14124[] = {SGI63593, SGI63594, SGI63595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63596, SGI0, SGI0, SGI0, SGI63597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14125[] = {SGI63598, SGI63599, SGI63600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63604, SGI0, SGI0, SGI0, SGI63605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14126[] = {SGI63601, SGI63602, SGI63603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63606, SGI0, SGI0, SGI0, SGI63607, SGI63607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14127[] = {SGI63608, SGI63609, SGI63610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63611, SGI0, SGI0, SGI0, SGI63612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14128[] = {SGI63613, SGI63614, SGI63615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63616, SGI0, SGI0, SGI0, SGI63617, SGI63617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14129[] = {SGI63618, SGI63619, SGI63620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63621, SGI0, SGI0, SGI0, SGI63622, SGI63622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14130[] = {SGI63623, SGI63624, SGI63625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63626, SGI0, SGI0, SGI0, SGI63628, SGI63628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14131[] = {SGI63627, SGI63629, SGI63630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63631, SGI0, SGI0, SGI0, SGI63632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14132[] = {SGI63633, SGI0, SGI63634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14133[] = {SGI63635, SGI63636, SGI63637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63638, SGI0, SGI0, SGI0, SGI63639, SGI63639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14134[] = {SGI63640, SGI63641, SGI63642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63643, SGI0, SGI0, SGI0, SGI63644, SGI63644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14135[] = {SGI63645, SGI63646, SGI63647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63648, SGI0, SGI0, SGI0, SGI63649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14136[] = {SGI63650, SGI63651, SGI63652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63653, SGI0, SGI0, SGI0, SGI63654, SGI63654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14137[] = {SGI63655, SGI63656, SGI63657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63660, SGI0, SGI0, SGI0, SGI63661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14138[] = {SGI63658, SGI0, SGI63659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14139[] = {SGI63662, SGI63663, SGI63664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63665, SGI0, SGI0, SGI0, SGI63666, SGI63666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14140[] = {SGI63667, SGI63668, SGI63669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63670, SGI0, SGI0, SGI0, SGI63671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14141[] = {SGI63672, SGI63673, SGI63674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63675, SGI63675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14142[] = {SGI63676, SGI63677, SGI63678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63679, SGI0, SGI0, SGI0, SGI63680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14143[] = {SGI63681, SGI63682, SGI63683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63686, SGI0, SGI0, SGI0, SGI63688, SGI63688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14144[] = {SGI63684, SGI63685, SGI63687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63689, SGI0, SGI0, SGI0, SGI63690, SGI63690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14145[] = {SGI63691, SGI63692, SGI63693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63694, SGI0, SGI0, SGI0, SGI63695, SGI63695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14146[] = {SGI63696, SGI63697, SGI63698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63699, SGI0, SGI0, SGI0, SGI63700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14147[] = {SGI63701, SGI0, SGI63702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63705, SGI0, SGI0, SGI0, SGI63707, SGI63707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14148[] = {SGI63703, SGI63704, SGI63706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63708, SGI0, SGI0, SGI0, SGI63709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14149[] = {SGI63710, SGI63711, SGI63712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63713, SGI0, SGI0, SGI0, SGI63714, SGI63714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14150[] = {SGI63715, SGI63716, SGI63717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63718, SGI0, SGI0, SGI0, SGI63719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14151[] = {SGI63720, SGI63721, SGI63722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63723, SGI0, SGI0, SGI0, SGI63724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14152[] = {SGI63725, SGI63726, SGI63727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63728, SGI0, SGI0, SGI0, SGI63729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14153[] = {SGI63730, SGI63731, SGI63732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63733, SGI0, SGI0, SGI0, SGI63734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14154[] = {SGI63735, SGI63736, SGI63737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63738, SGI0, SGI0, SGI0, SGI63739, SGI63739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14155[] = {SGI63740, SGI63741, SGI63742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63743, SGI0, SGI0, SGI0, SGI63744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14156[] = {SGI63745, SGI63746, SGI63747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63748, SGI0, SGI0, SGI0, SGI63749, SGI63749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14157[] = {SGI63750, SGI63751, SGI63752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63753, SGI0, SGI0, SGI0, SGI63754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14158[] = {SGI63755, SGI63756, SGI63757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63758, SGI0, SGI0, SGI0, SGI63759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14159[] = {SGI63760, SGI63761, SGI63762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63763, SGI0, SGI0, SGI0, SGI63764, SGI63764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14160[] = {SGI63765, SGI63766, SGI63767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63768, SGI0, SGI0, SGI0, SGI63769, SGI63769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14161[] = {SGI63770, SGI63771, SGI63772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63773, SGI0, SGI0, SGI0, SGI63774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14162[] = {SGI63775, SGI0, SGI63776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14163[] = {SGI63777, SGI63778, SGI63779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63780, SGI0, SGI0, SGI0, SGI63781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14164[] = {SGI63782, SGI63783, SGI63784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63785, SGI0, SGI0, SGI0, SGI63786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14165[] = {SGI63787, SGI63788, SGI63789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63790, SGI0, SGI0, SGI0, SGI63791, SGI63791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14166[] = {SGI63792, SGI63793, SGI63794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63795, SGI0, SGI0, SGI0, SGI63796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14167[] = {SGI63797, SGI63798, SGI63799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63800, SGI0, SGI0, SGI0, SGI63801, SGI63801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14168[] = {SGI63802, SGI63803, SGI63804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63805, SGI0, SGI0, SGI0, SGI63806, SGI63806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14169[] = {SGI63807, SGI63808, SGI63809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63810, SGI0, SGI0, SGI0, SGI63811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14170[] = {SGI63812, SGI63813, SGI63814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63815, SGI63816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14171[] = {SGI63817, SGI63818, SGI63819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63820, SGI0, SGI0, SGI0, SGI63821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14172[] = {SGI0, SGI63822, SGI63823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14173[] = {SGI63824, SGI63825, SGI63826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63827, SGI0, SGI0, SGI0, SGI63828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14174[] = {SGI63829, SGI63830, SGI63831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63832, SGI0, SGI0, SGI0, SGI63833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14175[] = {SGI63834, SGI63835, SGI63836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63837, SGI0, SGI0, SGI0, SGI63838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14176[] = {SGI63839, SGI63840, SGI63841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63842, SGI0, SGI0, SGI0, SGI63843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14177[] = {SGI63844, SGI63845, SGI63846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63849, SGI0, SGI0, SGI0, SGI63851, SGI63851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14178[] = {SGI63847, SGI63848, SGI63850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63852, SGI0, SGI0, SGI0, SGI63853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14179[] = {SGI63854, SGI63855, SGI63856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63857, SGI0, SGI0, SGI0, SGI63858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14180[] = {SGI63859, SGI63860, SGI63861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63862, SGI0, SGI0, SGI0, SGI63863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14181[] = {SGI63864, SGI63865, SGI63866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63867, SGI0, SGI0, SGI0, SGI63868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14182[] = {SGI63869, SGI63870, SGI63871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63872, SGI0, SGI0, SGI0, SGI63873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14183[] = {SGI63874, SGI63875, SGI63876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63877, SGI0, SGI0, SGI0, SGI63878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14184[] = {SGI63879, SGI63880, SGI63881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63882, SGI0, SGI0, SGI0, SGI63883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14185[] = {SGI63884, SGI63885, SGI63886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63887, SGI0, SGI0, SGI0, SGI63888, SGI63888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14186[] = {SGI63889, SGI63890, SGI63891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63892, SGI0, SGI0, SGI0, SGI63893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14187[] = {SGI63894, SGI63895, SGI63896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63897, SGI0, SGI0, SGI0, SGI63898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14188[] = {SGI63899, SGI63900, SGI63901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63902, SGI0, SGI0, SGI0, SGI63903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14189[] = {SGI63904, SGI0, SGI63905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14190[] = {SGI63906, SGI63907, SGI63908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63909, SGI0, SGI0, SGI0, SGI63910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14191[] = {SGI63911, SGI63912, SGI63913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63914, SGI0, SGI0, SGI0, SGI63915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14192[] = {SGI63916, SGI63917, SGI63918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63919, SGI0, SGI0, SGI0, SGI63920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14193[] = {SGI63921, SGI63922, SGI63923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63924, SGI0, SGI0, SGI0, SGI63925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14194[] = {SGI63926, SGI63927, SGI63928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63929, SGI0, SGI0, SGI0, SGI63930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14195[] = {SGI63931, SGI63932, SGI63933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63934, SGI0, SGI0, SGI0, SGI63935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14196[] = {SGI63936, SGI63937, SGI63938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63939, SGI0, SGI0, SGI0, SGI63940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14197[] = {SGI63941, SGI63942, SGI63943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63944, SGI0, SGI0, SGI0, SGI63945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14198[] = {SGI63946, SGI63947, SGI63948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63949, SGI0, SGI0, SGI0, SGI63950, SGI63950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14199[] = {SGI63951, SGI63952, SGI63953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63954, SGI0, SGI0, SGI0, SGI63955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14200[] = {SGI63956, SGI63957, SGI63958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63959, SGI0, SGI0, SGI0, SGI63960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14201[] = {SGI63961, SGI63962, SGI63963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63964, SGI0, SGI0, SGI0, SGI63965, SGI63965, SGI0, SGI0, NULL};
const Segment * const SG14202[] = {SGI63966, SGI63967, SGI63968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63969, SGI0, SGI0, SGI0, SGI63970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14203[] = {SGI63971, SGI63972, SGI63973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63974, SGI0, SGI0, SGI0, SGI63975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14204[] = {SGI63976, SGI63977, SGI63978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63979, SGI0, SGI0, SGI0, SGI63980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14205[] = {SGI63982, SGI63984, SGI63986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63988, SGI0, SGI0, SGI0, SGI63989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14206[] = {SGI63981, SGI63983, SGI63985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63987, SGI0, SGI0, SGI0, SGI63990, SGI63990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14207[] = {SGI63991, SGI63992, SGI63993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63994, SGI0, SGI0, SGI0, SGI63995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14208[] = {SGI63996, SGI63997, SGI63998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI63999, SGI0, SGI0, SGI0, SGI64000, SGI64000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14209[] = {SGI64001, SGI64002, SGI64003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64004, SGI0, SGI0, SGI0, SGI64005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14210[] = {SGI64006, SGI64007, SGI64008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64009, SGI0, SGI0, SGI0, SGI64010, SGI64010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14211[] = {SGI64011, SGI0, SGI64012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14212[] = {SGI64013, SGI64014, SGI64015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64016, SGI0, SGI0, SGI0, SGI64017, SGI64017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14213[] = {SGI64018, SGI64019, SGI64020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64021, SGI0, SGI0, SGI0, SGI64022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14214[] = {SGI64023, SGI64024, SGI64025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64026, SGI0, SGI0, SGI0, SGI64027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14215[] = {SGI64028, SGI64029, SGI64030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64031, SGI0, SGI0, SGI0, SGI64032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14216[] = {SGI64033, SGI64034, SGI64035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64036, SGI0, SGI0, SGI0, SGI64037, SGI64037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14217[] = {SGI64038, SGI64039, SGI64040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64041, SGI0, SGI0, SGI0, SGI64042, SGI64042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14218[] = {SGI64043, SGI64044, SGI64045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64046, SGI64046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14219[] = {SGI64047, SGI64048, SGI64049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64050, SGI0, SGI0, SGI0, SGI64051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14220[] = {SGI0, SGI0, SGI64052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64053, SGI64053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14221[] = {SGI64054, SGI64055, SGI64056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64057, SGI0, SGI0, SGI0, SGI64058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14222[] = {SGI64059, SGI64060, SGI64061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64062, SGI0, SGI0, SGI0, SGI64063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14223[] = {SGI64064, SGI64065, SGI64066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64067, SGI0, SGI0, SGI0, SGI64068, SGI64068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14224[] = {SGI64069, SGI64070, SGI64071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64072, SGI0, SGI0, SGI0, SGI64073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14225[] = {SGI64074, SGI64075, SGI64076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64077, SGI0, SGI0, SGI0, SGI64078, SGI64078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14226[] = {SGI64079, SGI64080, SGI64081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64082, SGI0, SGI0, SGI0, SGI64083, SGI64083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14227[] = {SGI64084, SGI64085, SGI64086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64087, SGI0, SGI0, SGI0, SGI64088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14228[] = {SGI64089, SGI64090, SGI64091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64092, SGI0, SGI0, SGI0, SGI64093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14229[] = {SGI64094, SGI64095, SGI64096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64097, SGI0, SGI0, SGI0, SGI64098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14230[] = {SGI64099, SGI64100, SGI64101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64102, SGI0, SGI0, SGI0, SGI64103, SGI64103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14231[] = {SGI64104, SGI64105, SGI64106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64107, SGI0, SGI0, SGI0, SGI64108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14232[] = {SGI64109, SGI64110, SGI64111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64112, SGI0, SGI0, SGI0, SGI64113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14233[] = {SGI64114, SGI64115, SGI64116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64117, SGI0, SGI0, SGI0, SGI64118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14234[] = {SGI64119, SGI64120, SGI64121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64122, SGI0, SGI0, SGI0, SGI64123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14235[] = {SGI64124, SGI64125, SGI64126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64127, SGI0, SGI0, SGI0, SGI64128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14236[] = {SGI64129, SGI64130, SGI64131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64132, SGI0, SGI0, SGI0, SGI64133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14237[] = {SGI64134, SGI64135, SGI64136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64137, SGI0, SGI0, SGI0, SGI64138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14238[] = {SGI64139, SGI64140, SGI64141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64142, SGI0, SGI0, SGI0, SGI64143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14239[] = {SGI64144, SGI64145, SGI64146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64147, SGI0, SGI0, SGI0, SGI64148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14240[] = {SGI64149, SGI64150, SGI64151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64152, SGI0, SGI0, SGI0, SGI64153, SGI64153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14241[] = {SGI64154, SGI64155, SGI64156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64157, SGI0, SGI0, SGI0, SGI64158, SGI64158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14242[] = {SGI64159, SGI64160, SGI64161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64162, SGI0, SGI0, SGI0, SGI64163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14243[] = {SGI64164, SGI64165, SGI64166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64167, SGI0, SGI0, SGI0, SGI64168, SGI64168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14244[] = {SGI64169, SGI64170, SGI64171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64172, SGI0, SGI0, SGI0, SGI64173, SGI64173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14245[] = {SGI64174, SGI64175, SGI64176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64177, SGI0, SGI0, SGI0, SGI64178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14246[] = {SGI64179, SGI64180, SGI64181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64182, SGI0, SGI0, SGI0, SGI64183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14247[] = {SGI64184, SGI64185, SGI64186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64187, SGI0, SGI0, SGI0, SGI64188, SGI64188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14248[] = {SGI64189, SGI64190, SGI64191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64193, SGI0, SGI0, SGI0, SGI64195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14249[] = {SGI64192, SGI64194, SGI64196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64197, SGI0, SGI0, SGI0, SGI64198, SGI64198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14250[] = {SGI64199, SGI64200, SGI64201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64202, SGI0, SGI0, SGI0, SGI64203, SGI64203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14251[] = {SGI64204, SGI64205, SGI64206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64210, SGI0, SGI0, SGI0, SGI64211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14252[] = {SGI64207, SGI64208, SGI64209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64212, SGI64213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14253[] = {SGI64215, SGI0, SGI64218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14254[] = {SGI64214, SGI64216, SGI64217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64219, SGI0, SGI0, SGI0, SGI64220, SGI64220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14255[] = {SGI64221, SGI64222, SGI64224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64227, SGI0, SGI0, SGI0, SGI64229, SGI64229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14256[] = {SGI64223, SGI64225, SGI64226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64228, SGI0, SGI0, SGI0, SGI64230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14257[] = {SGI64232, SGI64234, SGI64236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64238, SGI0, SGI0, SGI0, SGI64239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14258[] = {SGI64231, SGI64233, SGI64235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64237, SGI0, SGI0, SGI0, SGI64240, SGI64240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14259[] = {SGI64241, SGI64242, SGI64243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64247, SGI0, SGI0, SGI0, SGI64248, SGI64248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14260[] = {SGI64244, SGI64245, SGI64246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64249, SGI0, SGI0, SGI0, SGI64250, SGI64250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14261[] = {SGI64251, SGI64252, SGI64253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64254, SGI0, SGI0, SGI0, SGI64255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14262[] = {SGI64256, SGI64257, SGI64258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64260, SGI0, SGI0, SGI0, SGI64263, SGI64263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14263[] = {SGI64259, SGI64261, SGI64262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64264, SGI0, SGI0, SGI0, SGI64265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14264[] = {SGI64266, SGI64267, SGI64268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64271, SGI64271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14265[] = {SGI64269, SGI64270, SGI64272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64273, SGI0, SGI0, SGI0, SGI64274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14266[] = {SGI64275, SGI0, SGI64276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64279, SGI0, SGI0, SGI0, SGI64281, SGI64281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14267[] = {SGI64277, SGI64278, SGI64280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64282, SGI0, SGI0, SGI0, SGI64283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14268[] = {SGI64284, SGI64285, SGI64286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64287, SGI0, SGI0, SGI0, SGI64288, SGI64288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14269[] = {SGI64289, SGI64290, SGI64291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64292, SGI0, SGI0, SGI0, SGI64294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14270[] = {SGI64293, SGI0, SGI64295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14271[] = {SGI64296, SGI64297, SGI64298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64299, SGI0, SGI0, SGI0, SGI64301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14272[] = {SGI64300, SGI64302, SGI64303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64304, SGI0, SGI0, SGI0, SGI64305, SGI64305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14273[] = {SGI64306, SGI64307, SGI64308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64309, SGI0, SGI0, SGI0, SGI64310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14274[] = {SGI64311, SGI64312, SGI64313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64314, SGI0, SGI0, SGI0, SGI64316, SGI64316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14275[] = {SGI0, SGI0, SGI64315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64317, SGI0, SGI0, SGI0, SGI64318, SGI64318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14276[] = {SGI64319, SGI0, SGI64320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14277[] = {SGI64322, SGI64324, SGI64326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64327, SGI0, SGI0, SGI0, SGI64329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14278[] = {SGI64321, SGI64323, SGI64325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64328, SGI0, SGI0, SGI0, SGI64330, SGI64330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14279[] = {SGI64331, SGI64332, SGI64333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64334, SGI64334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14280[] = {SGI0, SGI64335, SGI64336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14281[] = {SGI64337, SGI64338, SGI64339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64340, SGI0, SGI0, SGI0, SGI64341, SGI64341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14282[] = {SGI64342, SGI64343, SGI64344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64345, SGI0, SGI0, SGI0, SGI64346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14283[] = {SGI64347, SGI64348, SGI64349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64350, SGI0, SGI0, SGI0, SGI64351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14284[] = {SGI64352, SGI64353, SGI64354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64355, SGI0, SGI0, SGI0, SGI64356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14285[] = {SGI64357, SGI64358, SGI64359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64360, SGI0, SGI0, SGI0, SGI64361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14286[] = {SGI64362, SGI64363, SGI64364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64365, SGI0, SGI0, SGI0, SGI64366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14287[] = {SGI64367, SGI64368, SGI64369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64370, SGI0, SGI0, SGI0, SGI64371, SGI64371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14288[] = {SGI64372, SGI64373, SGI64374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64375, SGI0, SGI0, SGI0, SGI64376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14289[] = {SGI64377, SGI64378, SGI64379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64380, SGI0, SGI0, SGI0, SGI64381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14290[] = {SGI64382, SGI64383, SGI64384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64385, SGI0, SGI0, SGI0, SGI64386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14291[] = {SGI64387, SGI64388, SGI64389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64390, SGI0, SGI0, SGI0, SGI64391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14292[] = {SGI64392, SGI64393, SGI64394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64395, SGI0, SGI0, SGI0, SGI64396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14293[] = {SGI64397, SGI64398, SGI64399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64400, SGI0, SGI0, SGI0, SGI64401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14294[] = {SGI64402, SGI64403, SGI64404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64405, SGI0, SGI0, SGI0, SGI64406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14295[] = {SGI64407, SGI64408, SGI64409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64410, SGI0, SGI0, SGI0, SGI64411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14296[] = {SGI64412, SGI64413, SGI64414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64415, SGI0, SGI0, SGI0, SGI64416, SGI64416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14297[] = {SGI64417, SGI64418, SGI64419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64420, SGI0, SGI0, SGI0, SGI64421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14298[] = {SGI64422, SGI64423, SGI64424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64425, SGI0, SGI0, SGI0, SGI64426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14299[] = {SGI64427, SGI64428, SGI64429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64430, SGI0, SGI0, SGI0, SGI64431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14300[] = {SGI64432, SGI64433, SGI64434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64435, SGI0, SGI0, SGI0, SGI64436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14301[] = {SGI64437, SGI64438, SGI64439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64440, SGI0, SGI0, SGI0, SGI64441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14302[] = {SGI64442, SGI64443, SGI64444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64445, SGI0, SGI0, SGI0, SGI64446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14303[] = {SGI64447, SGI64448, SGI64449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64450, SGI0, SGI0, SGI0, SGI64451, SGI64451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14304[] = {SGI64452, SGI64453, SGI64454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64455, SGI0, SGI0, SGI0, SGI64456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14305[] = {SGI64457, SGI64458, SGI64459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64460, SGI0, SGI0, SGI0, SGI64461, SGI64461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14306[] = {SGI64462, SGI64463, SGI64464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64465, SGI0, SGI0, SGI0, SGI64466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14307[] = {SGI64467, SGI64468, SGI64469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64470, SGI0, SGI0, SGI0, SGI64471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14308[] = {SGI64472, SGI64473, SGI64474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64475, SGI0, SGI0, SGI0, SGI64476, SGI64476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14309[] = {SGI64477, SGI64478, SGI64479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64480, SGI0, SGI0, SGI0, SGI64481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14310[] = {SGI64482, SGI64483, SGI64484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64485, SGI0, SGI0, SGI0, SGI64486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14311[] = {SGI64487, SGI64488, SGI64489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64490, SGI0, SGI0, SGI0, SGI64491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14312[] = {SGI64492, SGI64493, SGI64494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64495, SGI0, SGI0, SGI0, SGI64496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14313[] = {SGI64497, SGI64498, SGI64499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64500, SGI0, SGI0, SGI0, SGI64501, SGI64501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14314[] = {SGI64502, SGI64503, SGI64504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64505, SGI0, SGI0, SGI0, SGI64506, SGI64506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14315[] = {SGI64507, SGI64508, SGI64509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64510, SGI0, SGI0, SGI0, SGI64511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14316[] = {SGI64512, SGI64513, SGI64514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64515, SGI0, SGI0, SGI0, SGI64516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14317[] = {SGI64517, SGI64518, SGI64519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64520, SGI0, SGI0, SGI0, SGI64521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14318[] = {SGI64522, SGI64523, SGI64524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64525, SGI0, SGI0, SGI0, SGI64526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14319[] = {SGI64527, SGI64528, SGI64529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64530, SGI0, SGI0, SGI0, SGI64531, SGI64531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14320[] = {SGI64532, SGI64533, SGI64534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64535, SGI0, SGI0, SGI0, SGI64536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14321[] = {SGI64537, SGI64538, SGI64539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64540, SGI0, SGI0, SGI0, SGI64541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14322[] = {SGI64542, SGI64543, SGI64544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64545, SGI0, SGI0, SGI0, SGI64546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14323[] = {SGI64547, SGI64548, SGI64549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64550, SGI0, SGI0, SGI0, SGI64551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14324[] = {SGI64552, SGI64553, SGI64554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64555, SGI0, SGI0, SGI0, SGI64556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14325[] = {SGI64557, SGI64558, SGI64559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64560, SGI0, SGI0, SGI0, SGI64561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14326[] = {SGI64562, SGI64563, SGI64564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64565, SGI0, SGI0, SGI0, SGI64566, SGI64566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14327[] = {SGI64567, SGI64568, SGI64569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64570, SGI0, SGI0, SGI0, SGI64571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14328[] = {SGI64572, SGI64573, SGI64574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64575, SGI0, SGI0, SGI0, SGI64576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14329[] = {SGI64577, SGI64578, SGI64579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64580, SGI0, SGI0, SGI0, SGI64581, SGI64581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14330[] = {SGI64582, SGI64583, SGI64584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64585, SGI0, SGI0, SGI0, SGI64586, SGI64586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14331[] = {SGI64587, SGI64588, SGI64589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64590, SGI0, SGI0, SGI0, SGI64591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14332[] = {SGI64592, SGI64593, SGI64594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64595, SGI0, SGI0, SGI0, SGI64596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14333[] = {SGI64597, SGI64598, SGI64599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64600, SGI0, SGI0, SGI0, SGI64601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14334[] = {SGI64602, SGI64603, SGI64604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64605, SGI0, SGI0, SGI0, SGI64606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14335[] = {SGI64607, SGI64608, SGI64609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64610, SGI0, SGI0, SGI0, SGI64611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14336[] = {SGI64612, SGI64613, SGI64614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64615, SGI0, SGI0, SGI0, SGI64616, SGI64616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14337[] = {SGI64617, SGI64618, SGI64619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64620, SGI0, SGI0, SGI0, SGI64621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14338[] = {SGI64622, SGI64623, SGI64624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64625, SGI0, SGI0, SGI0, SGI64626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14339[] = {SGI64627, SGI64628, SGI64629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64630, SGI0, SGI0, SGI0, SGI64631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14340[] = {SGI64632, SGI64633, SGI64634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64635, SGI0, SGI0, SGI0, SGI64636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14341[] = {SGI64637, SGI64638, SGI64639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64640, SGI0, SGI0, SGI0, SGI64641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14342[] = {SGI64642, SGI64643, SGI64644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64645, SGI0, SGI0, SGI0, SGI64646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14343[] = {SGI64647, SGI64648, SGI64649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64650, SGI0, SGI0, SGI0, SGI64651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14344[] = {SGI64652, SGI64653, SGI64654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64655, SGI0, SGI0, SGI0, SGI64656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14345[] = {SGI64657, SGI64658, SGI64659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64660, SGI0, SGI0, SGI0, SGI64661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14346[] = {SGI64662, SGI64663, SGI64664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64665, SGI0, SGI0, SGI0, SGI64666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14347[] = {SGI64667, SGI64668, SGI64669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64670, SGI0, SGI0, SGI0, SGI64671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14348[] = {SGI64672, SGI64673, SGI64674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64675, SGI0, SGI0, SGI0, SGI64676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14349[] = {SGI64677, SGI64678, SGI64679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64680, SGI0, SGI0, SGI0, SGI64681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14350[] = {SGI64682, SGI64683, SGI64684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64685, SGI0, SGI0, SGI0, SGI64686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14351[] = {SGI64687, SGI64688, SGI64689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64690, SGI0, SGI0, SGI0, SGI64691, SGI64691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14352[] = {SGI64692, SGI64693, SGI64694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64695, SGI0, SGI0, SGI0, SGI64696, SGI64696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14353[] = {SGI64697, SGI64698, SGI64699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64700, SGI0, SGI0, SGI0, SGI64701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14354[] = {SGI64702, SGI64703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64704, SGI0, SGI0, SGI0, SGI64705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14355[] = {SGI64706, SGI64707, SGI64708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64709, SGI0, SGI0, SGI0, SGI64710, SGI64710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14356[] = {SGI64711, SGI64712, SGI64713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64714, SGI0, SGI0, SGI0, SGI64715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14357[] = {SGI64716, SGI64717, SGI64718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64722, SGI0, SGI0, SGI0, SGI64723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14358[] = {SGI64719, SGI64720, SGI64721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64724, SGI0, SGI0, SGI0, SGI64725, SGI64725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14359[] = {SGI64726, SGI64727, SGI64728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64729, SGI0, SGI0, SGI0, SGI64731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14360[] = {SGI64730, SGI64732, SGI64733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64734, SGI0, SGI0, SGI0, SGI64735, SGI64735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14361[] = {SGI64736, SGI64737, SGI64738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64739, SGI0, SGI0, SGI0, SGI64740, SGI64740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14362[] = {SGI64741, SGI64742, SGI64743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64744, SGI0, SGI0, SGI0, SGI64745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14363[] = {SGI64746, SGI64747, SGI64748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64749, SGI0, SGI0, SGI0, SGI64750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14364[] = {SGI64751, SGI64752, SGI64753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64754, SGI0, SGI0, SGI0, SGI64755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14365[] = {SGI64756, SGI64757, SGI64758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64759, SGI0, SGI0, SGI0, SGI64760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14366[] = {SGI64761, SGI64762, SGI64763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64764, SGI0, SGI0, SGI0, SGI64765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14367[] = {SGI64766, SGI64767, SGI64768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64769, SGI0, SGI0, SGI0, SGI64770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14368[] = {SGI64771, SGI64772, SGI64773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64774, SGI0, SGI0, SGI0, SGI64775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14369[] = {SGI64776, SGI64777, SGI64778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64779, SGI0, SGI0, SGI0, SGI64780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14370[] = {SGI64781, SGI64782, SGI64783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64784, SGI0, SGI0, SGI0, SGI64785, SGI64785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14371[] = {SGI64786, SGI64787, SGI64788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64789, SGI0, SGI0, SGI0, SGI64790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14372[] = {SGI64791, SGI64792, SGI64793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64794, SGI0, SGI0, SGI0, SGI64795, SGI64795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14373[] = {SGI64796, SGI64797, SGI64798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64799, SGI0, SGI0, SGI0, SGI64800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14374[] = {SGI64801, SGI64802, SGI64803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64804, SGI0, SGI0, SGI0, SGI64805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14375[] = {SGI64806, SGI64807, SGI64808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64809, SGI0, SGI0, SGI0, SGI64810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14376[] = {SGI64811, SGI64812, SGI64813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64814, SGI0, SGI0, SGI0, SGI64815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14377[] = {SGI64816, SGI64817, SGI64818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64819, SGI0, SGI0, SGI0, SGI64820, SGI64820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14378[] = {SGI64821, SGI64822, SGI64823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64824, SGI0, SGI0, SGI0, SGI64825, SGI64825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14379[] = {SGI64826, SGI64827, SGI64828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64829, SGI0, SGI0, SGI0, SGI64830, SGI64830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14380[] = {SGI64831, SGI64832, SGI64833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64834, SGI0, SGI0, SGI0, SGI64835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14381[] = {SGI64836, SGI64837, SGI64838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64839, SGI0, SGI0, SGI0, SGI64840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14382[] = {SGI64841, SGI64842, SGI64843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64844, SGI0, SGI0, SGI0, SGI64845, SGI64845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14383[] = {SGI64846, SGI64847, SGI64848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64849, SGI0, SGI0, SGI0, SGI64850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14384[] = {SGI64851, SGI64852, SGI64853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64854, SGI0, SGI0, SGI0, SGI64855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14385[] = {SGI64856, SGI64857, SGI64858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64859, SGI0, SGI0, SGI0, SGI64860, SGI64860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14386[] = {SGI64861, SGI64862, SGI64863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64864, SGI0, SGI0, SGI0, SGI64865, SGI64865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14387[] = {SGI64866, SGI64867, SGI64868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64869, SGI0, SGI0, SGI0, SGI64870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14388[] = {SGI64871, SGI64872, SGI64873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64874, SGI0, SGI0, SGI0, SGI64875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14389[] = {SGI64876, SGI64877, SGI64878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64879, SGI0, SGI0, SGI0, SGI64880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14390[] = {SGI64881, SGI64882, SGI64883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64884, SGI0, SGI0, SGI0, SGI64885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14391[] = {SGI64886, SGI64887, SGI64888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64889, SGI0, SGI0, SGI0, SGI64890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14392[] = {SGI64891, SGI64892, SGI64893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64894, SGI64894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14393[] = {SGI64895, SGI64896, SGI64897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64898, SGI0, SGI0, SGI0, SGI64899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14394[] = {SGI64900, SGI64901, SGI64902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64903, SGI0, SGI0, SGI0, SGI64904, SGI64904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14395[] = {SGI64905, SGI64906, SGI64907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64908, SGI0, SGI0, SGI0, SGI64909, SGI64909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14396[] = {SGI64910, SGI64911, SGI64912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64913, SGI0, SGI0, SGI0, SGI64914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14397[] = {SGI64915, SGI64916, SGI64917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64918, SGI0, SGI0, SGI0, SGI64919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14398[] = {SGI64920, SGI64921, SGI64922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64923, SGI0, SGI0, SGI0, SGI64924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14399[] = {SGI64925, SGI64926, SGI64927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64928, SGI0, SGI0, SGI0, SGI64929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14400[] = {SGI64930, SGI64931, SGI64932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64933, SGI0, SGI0, SGI0, SGI64934, SGI64934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14401[] = {SGI64935, SGI64936, SGI64937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64938, SGI0, SGI0, SGI0, SGI64939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14402[] = {SGI64940, SGI64941, SGI64942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64943, SGI0, SGI0, SGI0, SGI64944, SGI64944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14403[] = {SGI64945, SGI64946, SGI64947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64948, SGI0, SGI0, SGI0, SGI64949, SGI64949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14404[] = {SGI64950, SGI64951, SGI64952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64953, SGI0, SGI0, SGI0, SGI64954, SGI64954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14405[] = {SGI64955, SGI64956, SGI64957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64958, SGI0, SGI0, SGI0, SGI64959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14406[] = {SGI64960, SGI64961, SGI64962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64963, SGI0, SGI0, SGI0, SGI64964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14407[] = {SGI64965, SGI64966, SGI64967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64968, SGI0, SGI0, SGI0, SGI64969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14408[] = {SGI64970, SGI64971, SGI64972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64973, SGI0, SGI0, SGI0, SGI64974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14409[] = {SGI64975, SGI64976, SGI64977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64978, SGI0, SGI0, SGI0, SGI64979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14410[] = {SGI64980, SGI64981, SGI64982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64983, SGI0, SGI0, SGI0, SGI64984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14411[] = {SGI64985, SGI64986, SGI64987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64988, SGI0, SGI0, SGI0, SGI64989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14412[] = {SGI64990, SGI64991, SGI64992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64993, SGI0, SGI0, SGI0, SGI64994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14413[] = {SGI64995, SGI64996, SGI64997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI64998, SGI0, SGI0, SGI0, SGI64999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14414[] = {SGI65000, SGI65001, SGI65002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65003, SGI0, SGI0, SGI0, SGI65004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14415[] = {SGI65005, SGI65006, SGI65007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65008, SGI0, SGI0, SGI0, SGI65009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14416[] = {SGI65010, SGI65011, SGI65012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65013, SGI0, SGI0, SGI0, SGI65014, SGI65014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14417[] = {SGI65015, SGI65016, SGI65017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65018, SGI0, SGI0, SGI0, SGI65019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14418[] = {SGI65020, SGI65021, SGI65022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65023, SGI0, SGI0, SGI0, SGI65024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14419[] = {SGI65025, SGI65026, SGI65027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65028, SGI0, SGI0, SGI0, SGI65029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14420[] = {SGI65030, SGI65031, SGI65032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65033, SGI0, SGI0, SGI0, SGI65034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14421[] = {SGI65035, SGI65036, SGI65037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65038, SGI0, SGI0, SGI0, SGI65039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14422[] = {SGI65040, SGI65041, SGI65042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65043, SGI0, SGI0, SGI0, SGI65044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14423[] = {SGI65045, SGI65046, SGI65047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65048, SGI0, SGI0, SGI0, SGI65049, SGI65049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14424[] = {SGI65050, SGI65051, SGI65052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65053, SGI0, SGI0, SGI0, SGI65054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14425[] = {SGI65055, SGI65056, SGI65057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65058, SGI0, SGI0, SGI0, SGI65059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14426[] = {SGI65060, SGI65061, SGI65062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65063, SGI0, SGI0, SGI0, SGI65064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14427[] = {SGI65065, SGI65066, SGI65067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65068, SGI0, SGI0, SGI0, SGI65069, SGI65069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14428[] = {SGI65070, SGI65071, SGI65072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65073, SGI0, SGI0, SGI0, SGI65074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14429[] = {SGI65075, SGI65076, SGI65077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65078, SGI0, SGI0, SGI0, SGI65079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14430[] = {SGI65080, SGI65081, SGI65082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65083, SGI0, SGI0, SGI0, SGI65084, SGI65084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14431[] = {SGI65085, SGI65086, SGI65087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65088, SGI0, SGI0, SGI0, SGI65089, SGI65089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14432[] = {SGI65090, SGI65091, SGI65092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65093, SGI0, SGI0, SGI0, SGI65094, SGI65094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14433[] = {SGI65095, SGI65096, SGI65097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65098, SGI0, SGI0, SGI0, SGI65099, SGI65099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14434[] = {SGI65100, SGI65101, SGI65102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65103, SGI0, SGI0, SGI0, SGI65104, SGI65104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14435[] = {SGI65105, SGI65106, SGI65107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65108, SGI0, SGI0, SGI0, SGI65109, SGI65109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14436[] = {SGI65110, SGI65111, SGI65112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65113, SGI0, SGI0, SGI0, SGI65114, SGI65114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14437[] = {SGI65115, SGI65116, SGI65117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65118, SGI0, SGI0, SGI0, SGI65119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14438[] = {SGI65121, SGI65122, SGI65124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65126, SGI0, SGI0, SGI0, SGI65128, SGI65128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14439[] = {SGI65120, SGI65123, SGI65125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65127, SGI0, SGI0, SGI0, SGI65129, SGI65129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14440[] = {SGI65130, SGI65131, SGI65132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65133, SGI0, SGI0, SGI0, SGI65134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14441[] = {SGI65135, SGI65136, SGI65137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65138, SGI0, SGI0, SGI0, SGI65139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14442[] = {SGI65140, SGI65141, SGI65142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65143, SGI0, SGI0, SGI0, SGI65144, SGI65144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14443[] = {SGI65145, SGI65146, SGI65147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65148, SGI0, SGI0, SGI0, SGI65149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14444[] = {SGI65150, SGI65151, SGI65152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65153, SGI0, SGI0, SGI0, SGI65154, SGI65154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14445[] = {SGI65155, SGI65156, SGI65157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65158, SGI0, SGI0, SGI0, SGI65159, SGI65159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14446[] = {SGI65160, SGI65161, SGI65162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65166, SGI0, SGI0, SGI0, SGI65167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14447[] = {SGI65163, SGI65164, SGI65165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65168, SGI0, SGI0, SGI0, SGI65169, SGI65169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14448[] = {SGI65170, SGI65171, SGI65173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65176, SGI0, SGI0, SGI0, SGI65178, SGI65178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14449[] = {SGI65172, SGI65174, SGI65175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65177, SGI0, SGI0, SGI0, SGI65179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14450[] = {SGI65180, SGI65181, SGI65182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65183, SGI0, SGI0, SGI0, SGI65184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14451[] = {SGI65185, SGI65186, SGI65187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65188, SGI0, SGI0, SGI0, SGI65189, SGI65189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14452[] = {SGI65190, SGI65191, SGI65192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65193, SGI0, SGI0, SGI0, SGI65194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14453[] = {SGI65195, SGI65196, SGI65197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65198, SGI0, SGI0, SGI0, SGI65199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14454[] = {SGI65200, SGI65201, SGI65202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65203, SGI0, SGI0, SGI0, SGI65204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14455[] = {SGI65205, SGI65206, SGI65207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65208, SGI0, SGI0, SGI0, SGI65209, SGI65209, SGI0, SGI0, NULL};
const Segment * const SG14456[] = {SGI65210, SGI65211, SGI65212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65213, SGI0, SGI0, SGI0, SGI65214, SGI65214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14457[] = {SGI65215, SGI65216, SGI65217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65219, SGI0, SGI0, SGI0, SGI65222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14458[] = {SGI65218, SGI65220, SGI65221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65223, SGI0, SGI0, SGI0, SGI65224, SGI65224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14459[] = {SGI65225, SGI65226, SGI65227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65228, SGI0, SGI0, SGI0, SGI65229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14460[] = {SGI65230, SGI65231, SGI65232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65236, SGI0, SGI0, SGI0, SGI65237, SGI65237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14461[] = {SGI65233, SGI65234, SGI65235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65238, SGI0, SGI0, SGI0, SGI65239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14462[] = {SGI65240, SGI65241, SGI65242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65243, SGI0, SGI0, SGI0, SGI65244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14463[] = {SGI65245, SGI65246, SGI65247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65248, SGI0, SGI0, SGI0, SGI65249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14464[] = {SGI65250, SGI65251, SGI65252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65253, SGI0, SGI0, SGI0, SGI65254, SGI65254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14465[] = {SGI65255, SGI65256, SGI65257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65258, SGI0, SGI0, SGI0, SGI65259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14466[] = {SGI65260, SGI65261, SGI65262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65263, SGI0, SGI0, SGI0, SGI65264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14467[] = {SGI65265, SGI65266, SGI65267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65268, SGI0, SGI0, SGI0, SGI65269, SGI65269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14468[] = {SGI65270, SGI65271, SGI65272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65273, SGI0, SGI0, SGI0, SGI65274, SGI65274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14469[] = {SGI65275, SGI65276, SGI65277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65281, SGI0, SGI0, SGI0, SGI65282, SGI65282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14470[] = {SGI65278, SGI65279, SGI65280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65283, SGI0, SGI0, SGI0, SGI65284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14471[] = {SGI65285, SGI65286, SGI65287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65288, SGI0, SGI0, SGI0, SGI65289, SGI65289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14472[] = {SGI65290, SGI65291, SGI65292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65293, SGI0, SGI0, SGI0, SGI65294, SGI65294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14473[] = {SGI65295, SGI65296, SGI65297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65298, SGI0, SGI0, SGI0, SGI65299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14474[] = {SGI65304, SGI65305, SGI65306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65307, SGI0, SGI0, SGI0, SGI65308, SGI65308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14475[] = {SGI65309, SGI65310, SGI65311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65312, SGI0, SGI0, SGI0, SGI65313, SGI65313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14476[] = {SGI65314, SGI65315, SGI65316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65318, SGI0, SGI0, SGI0, SGI65320, SGI65320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14477[] = {SGI65317, SGI0, SGI65319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14478[] = {SGI65321, SGI65322, SGI65323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65324, SGI0, SGI0, SGI0, SGI65325, SGI65325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14479[] = {SGI65326, SGI65327, SGI65328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65329, SGI0, SGI0, SGI0, SGI65330, SGI65330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14480[] = {SGI65331, SGI65332, SGI65333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65334, SGI65334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14481[] = {SGI0, SGI65335, SGI65336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65337, SGI0, SGI0, SGI0, SGI65338, SGI65338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14482[] = {SGI65339, SGI0, SGI65340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65341, SGI0, SGI0, SGI0, SGI65342, SGI65342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14483[] = {SGI65343, SGI65344, SGI65345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65346, SGI65347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14484[] = {SGI65348, SGI65349, SGI65350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65351, SGI0, SGI0, SGI0, SGI65352, SGI65352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14485[] = {SGI65300, SGI65301, SGI65302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65303, SGI0, SGI0, SGI0, SGI65353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14486[] = {SGI65354, SGI65355, SGI65356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65357, SGI0, SGI0, SGI0, SGI65358, SGI65358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14487[] = {SGI65359, SGI0, SGI65360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14488[] = {SGI65361, SGI65362, SGI65363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65364, SGI0, SGI0, SGI0, SGI65365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14489[] = {SGI65366, SGI65367, SGI65368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65369, SGI0, SGI0, SGI0, SGI65370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14490[] = {SGI65371, SGI65372, SGI65373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65374, SGI0, SGI0, SGI0, SGI65375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14491[] = {SGI65376, SGI65377, SGI65378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65379, SGI0, SGI0, SGI0, SGI65380, SGI65380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14492[] = {SGI65381, SGI65382, SGI65383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65384, SGI0, SGI0, SGI0, SGI65385, SGI65385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14493[] = {SGI65386, SGI65387, SGI65388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65389, SGI0, SGI0, SGI0, SGI65390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14494[] = {SGI65391, SGI65392, SGI65393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65394, SGI0, SGI0, SGI0, SGI65395, SGI65395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14495[] = {SGI65396, SGI65397, SGI65398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65399, SGI0, SGI0, SGI0, SGI65400, SGI65400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14496[] = {SGI65401, SGI65402, SGI65403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65404, SGI0, SGI0, SGI0, SGI65405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14497[] = {SGI65406, SGI65407, SGI65408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65409, SGI65409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14498[] = {SGI65410, SGI65411, SGI65412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65413, SGI0, SGI0, SGI0, SGI65414, SGI65414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14499[] = {SGI65415, SGI65416, SGI65417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65418, SGI0, SGI0, SGI0, SGI65419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14500[] = {SGI65420, SGI65421, SGI65422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65423, SGI0, SGI0, SGI0, SGI65424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14501[] = {SGI65425, SGI65426, SGI65427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65428, SGI0, SGI0, SGI0, SGI65429, SGI65429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14502[] = {SGI65430, SGI65431, SGI65432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65433, SGI0, SGI0, SGI0, SGI65434, SGI65434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14503[] = {SGI65435, SGI65436, SGI65437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65438, SGI0, SGI0, SGI0, SGI65439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14504[] = {SGI65440, SGI65441, SGI65442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65443, SGI0, SGI0, SGI0, SGI65444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14505[] = {SGI65445, SGI65446, SGI65447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65448, SGI0, SGI0, SGI0, SGI65449, SGI65449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14506[] = {SGI65450, SGI65451, SGI65452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65453, SGI0, SGI0, SGI0, SGI65454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14507[] = {SGI65455, SGI65456, SGI65457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65458, SGI0, SGI0, SGI0, SGI65459, SGI65459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14508[] = {SGI65460, SGI65461, SGI65462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65463, SGI0, SGI0, SGI0, SGI65464, SGI65464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14509[] = {SGI65465, SGI65466, SGI65467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65468, SGI0, SGI0, SGI0, SGI65469, SGI65469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14510[] = {SGI65470, SGI65471, SGI65472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65473, SGI0, SGI0, SGI0, SGI65474, SGI65474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14511[] = {SGI65475, SGI65476, SGI65477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65478, SGI0, SGI0, SGI0, SGI65479, SGI65479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14512[] = {SGI65480, SGI65481, SGI65482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65483, SGI0, SGI0, SGI0, SGI65484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14513[] = {SGI65485, SGI65486, SGI65487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65488, SGI0, SGI0, SGI0, SGI65489, SGI65489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14514[] = {SGI65490, SGI65491, SGI65492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65493, SGI0, SGI0, SGI0, SGI0, SGI65494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14515[] = {SGI65495, SGI65496, SGI65497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65498, SGI0, SGI0, SGI0, SGI65499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14516[] = {SGI65500, SGI65501, SGI65502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65503, SGI0, SGI0, SGI0, SGI65504, SGI65504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14517[] = {SGI65505, SGI65506, SGI65507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65508, SGI0, SGI0, SGI0, SGI65509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14518[] = {SGI65510, SGI65511, SGI65512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65513, SGI0, SGI0, SGI0, SGI65514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14519[] = {SGI65515, SGI65516, SGI65517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65518, SGI0, SGI0, SGI0, SGI65519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14520[] = {SGI65520, SGI65521, SGI65522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65523, SGI0, SGI0, SGI0, SGI65524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14521[] = {SGI65525, SGI65526, SGI65527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65528, SGI0, SGI0, SGI0, SGI65529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14522[] = {SGI65530, SGI65531, SGI65532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65533, SGI0, SGI0, SGI0, SGI65534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14523[] = {SGI65535, SGI65536, SGI65537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65538, SGI0, SGI0, SGI0, SGI65539, SGI65539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14524[] = {SGI65540, SGI65541, SGI65542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65543, SGI0, SGI0, SGI0, SGI65544, SGI65544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14525[] = {SGI65545, SGI65546, SGI65547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65548, SGI0, SGI0, SGI0, SGI65549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14526[] = {SGI65550, SGI65551, SGI65552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65553, SGI0, SGI0, SGI0, SGI65556, SGI65556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14527[] = {SGI65554, SGI65555, SGI65557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65558, SGI0, SGI0, SGI0, SGI65559, SGI65559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14528[] = {SGI65560, SGI65561, SGI65562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65563, SGI0, SGI0, SGI0, SGI65564, SGI65564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14529[] = {SGI65565, SGI65566, SGI65567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65568, SGI0, SGI0, SGI0, SGI65569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14530[] = {SGI65570, SGI65571, SGI65572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65573, SGI0, SGI0, SGI0, SGI65574, SGI65574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14531[] = {SGI65575, SGI65576, SGI65577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65578, SGI0, SGI0, SGI0, SGI65579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14532[] = {SGI65580, SGI65581, SGI65582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65583, SGI65583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14533[] = {SGI65584, SGI65585, SGI65586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65587, SGI0, SGI0, SGI0, SGI65588, SGI65588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14534[] = {SGI65589, SGI65590, SGI65591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65592, SGI0, SGI0, SGI0, SGI65593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14535[] = {SGI65594, SGI65595, SGI65596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65597, SGI0, SGI0, SGI0, SGI65598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14536[] = {SGI0, SGI0, SGI65599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14537[] = {SGI65601, SGI65602, SGI65603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65604, SGI0, SGI0, SGI0, SGI65605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14538[] = {SGI65606, SGI65607, SGI65608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65609, SGI0, SGI0, SGI0, SGI65610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14539[] = {SGI65611, SGI65612, SGI65613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65617, SGI0, SGI0, SGI0, SGI65618, SGI65618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14540[] = {SGI65614, SGI65615, SGI65616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65619, SGI0, SGI0, SGI0, SGI65620, SGI65620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14541[] = {SGI65621, SGI65622, SGI65623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65624, SGI0, SGI0, SGI0, SGI65625, SGI65625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14542[] = {SGI65626, SGI65627, SGI65628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65629, SGI0, SGI0, SGI0, SGI65630, SGI65630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14543[] = {SGI65631, SGI65632, SGI65633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65634, SGI0, SGI0, SGI0, SGI65635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14544[] = {SGI65636, SGI65637, SGI65638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65639, SGI0, SGI0, SGI0, SGI65640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14545[] = {SGI65641, SGI65642, SGI65643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65644, SGI0, SGI0, SGI0, SGI65645, SGI65645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14546[] = {SGI65646, SGI65647, SGI65648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65649, SGI0, SGI0, SGI0, SGI65650, SGI65650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14547[] = {SGI65651, SGI65652, SGI65653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65654, SGI0, SGI0, SGI0, SGI65655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14548[] = {SGI65656, SGI65657, SGI65658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65659, SGI0, SGI0, SGI0, SGI65660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14549[] = {SGI65661, SGI65662, SGI65663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65664, SGI0, SGI0, SGI0, SGI65665, SGI65665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14550[] = {SGI65666, SGI65667, SGI65668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65669, SGI0, SGI0, SGI0, SGI65670, SGI65670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14551[] = {SGI65671, SGI65672, SGI65673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65674, SGI0, SGI0, SGI0, SGI65675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14552[] = {SGI65676, SGI65677, SGI65678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65679, SGI0, SGI0, SGI0, SGI65680, SGI65680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14553[] = {SGI65683, SGI65685, SGI65686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65688, SGI65688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14554[] = {SGI65681, SGI65682, SGI65684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65687, SGI0, SGI0, SGI0, SGI0, SGI65689, SGI0, SGI0, NULL};
const Segment * const SG14555[] = {SGI65690, SGI65691, SGI65692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65693, SGI65693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14556[] = {SGI65694, SGI65695, SGI65696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65697, SGI0, SGI0, SGI0, SGI65698, SGI65698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14557[] = {SGI65699, SGI65700, SGI65701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65704, SGI0, SGI0, SGI0, SGI65706, SGI65706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14558[] = {SGI65702, SGI65703, SGI65705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65707, SGI0, SGI0, SGI0, SGI65708, SGI65708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14559[] = {SGI65709, SGI65710, SGI65711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65712, SGI0, SGI0, SGI0, SGI65713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14560[] = {SGI65714, SGI65716, SGI65717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65720, SGI0, SGI0, SGI0, SGI65722, SGI65722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14561[] = {SGI65715, SGI65718, SGI65719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65721, SGI0, SGI0, SGI0, SGI65723, SGI65723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14562[] = {SGI65724, SGI65725, SGI65726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65727, SGI0, SGI0, SGI0, SGI65728, SGI65728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14563[] = {SGI65729, SGI65730, SGI65731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65732, SGI0, SGI0, SGI0, SGI65733, SGI65733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14564[] = {SGI65735, SGI65736, SGI65738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65741, SGI0, SGI0, SGI0, SGI65742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14565[] = {SGI65734, SGI65737, SGI65739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65740, SGI0, SGI0, SGI0, SGI65743, SGI65743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14566[] = {SGI65744, SGI65745, SGI65746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65750, SGI0, SGI0, SGI0, SGI65751, SGI65751, SGI0, SGI0, NULL};
const Segment * const SG14567[] = {SGI65747, SGI65748, SGI65749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65752, SGI0, SGI0, SGI0, SGI65753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14568[] = {SGI65754, SGI65755, SGI65756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65757, SGI0, SGI0, SGI0, SGI65758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14569[] = {SGI65759, SGI65760, SGI65761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65762, SGI0, SGI0, SGI0, SGI65763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14570[] = {SGI65764, SGI65765, SGI65766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65767, SGI0, SGI0, SGI0, SGI65768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14571[] = {SGI65769, SGI65770, SGI65771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65772, SGI0, SGI0, SGI0, SGI65773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14572[] = {SGI65774, SGI65775, SGI65776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65777, SGI0, SGI0, SGI0, SGI65778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14573[] = {SGI65779, SGI65780, SGI65781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65782, SGI0, SGI0, SGI0, SGI65783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14574[] = {SGI65784, SGI65785, SGI65786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65787, SGI0, SGI0, SGI0, SGI65788, SGI65788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14575[] = {SGI65789, SGI65790, SGI65791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65792, SGI0, SGI0, SGI0, SGI65793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14576[] = {SGI65794, SGI65795, SGI65796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65797, SGI0, SGI0, SGI0, SGI65798, SGI65798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14577[] = {SGI0, SGI0, SGI65799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65800, SGI0, SGI0, SGI0, SGI65801, SGI65802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14578[] = {SGI65803, SGI65804, SGI65805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65806, SGI0, SGI0, SGI0, SGI65807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14579[] = {SGI65808, SGI65809, SGI65810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65814, SGI65814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14580[] = {SGI65811, SGI65812, SGI65813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65815, SGI0, SGI0, SGI0, SGI65816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14581[] = {SGI65817, SGI65818, SGI65819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65820, SGI0, SGI0, SGI0, SGI65821, SGI65821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14582[] = {SGI65822, SGI65823, SGI65824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65825, SGI0, SGI0, SGI0, SGI65826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14583[] = {SGI0, SGI65827, SGI65828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14584[] = {SGI65829, SGI65830, SGI65831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65832, SGI0, SGI0, SGI0, SGI65833, SGI65833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14585[] = {SGI65834, SGI65835, SGI65836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65837, SGI65837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14586[] = {SGI65838, SGI65839, SGI65840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65841, SGI0, SGI0, SGI0, SGI65842, SGI65842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14587[] = {SGI65843, SGI65844, SGI65845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65846, SGI0, SGI0, SGI0, SGI65847, SGI65847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14588[] = {SGI65848, SGI65849, SGI65850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65851, SGI65851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14589[] = {SGI65852, SGI65853, SGI65854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65855, SGI0, SGI0, SGI0, SGI65856, SGI65856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14590[] = {SGI65857, SGI65858, SGI65859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65860, SGI0, SGI0, SGI0, SGI65861, SGI65861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14591[] = {SGI65862, SGI65863, SGI65864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65865, SGI0, SGI0, SGI0, SGI65866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14592[] = {SGI65867, SGI65868, SGI65869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65870, SGI0, SGI0, SGI0, SGI65871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14593[] = {SGI65872, SGI65873, SGI65874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65875, SGI0, SGI0, SGI0, SGI65876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14594[] = {SGI65877, SGI65878, SGI65879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65880, SGI0, SGI0, SGI0, SGI65883, SGI65883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14595[] = {SGI65881, SGI65882, SGI65884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65885, SGI0, SGI0, SGI0, SGI65886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14596[] = {SGI65887, SGI65888, SGI65889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65890, SGI0, SGI0, SGI0, SGI65891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14597[] = {SGI65892, SGI65893, SGI65894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65895, SGI0, SGI0, SGI0, SGI65896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14598[] = {SGI65897, SGI65898, SGI65899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65900, SGI0, SGI0, SGI0, SGI65901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14599[] = {SGI65902, SGI65903, SGI65904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65907, SGI0, SGI0, SGI0, SGI65909, SGI65909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14600[] = {SGI65905, SGI65906, SGI65908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65910, SGI0, SGI0, SGI0, SGI65911, SGI65911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14601[] = {SGI65912, SGI65913, SGI65914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65915, SGI0, SGI0, SGI0, SGI65916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14602[] = {SGI65917, SGI65918, SGI65919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65920, SGI0, SGI0, SGI0, SGI65921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14603[] = {SGI65922, SGI65923, SGI65924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65925, SGI0, SGI0, SGI0, SGI65926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14604[] = {SGI65927, SGI65929, SGI65931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65933, SGI0, SGI0, SGI0, SGI65934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14605[] = {SGI65928, SGI65930, SGI65932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65935, SGI65935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14606[] = {SGI65936, SGI65937, SGI65938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65939, SGI0, SGI0, SGI0, SGI65940, SGI65940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14607[] = {SGI65941, SGI65942, SGI65943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65944, SGI0, SGI0, SGI0, SGI65945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14608[] = {SGI65946, SGI65947, SGI65948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65949, SGI0, SGI0, SGI0, SGI65950, SGI65950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14609[] = {SGI65951, SGI65952, SGI65953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65954, SGI0, SGI0, SGI0, SGI65955, SGI65955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14610[] = {SGI65956, SGI65957, SGI65958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65959, SGI0, SGI0, SGI0, SGI65960, SGI65960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14611[] = {SGI65961, SGI65962, SGI65963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65967, SGI0, SGI0, SGI0, SGI65968, SGI65968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14612[] = {SGI65964, SGI65965, SGI65966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65969, SGI65969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14613[] = {SGI65970, SGI65971, SGI65972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65973, SGI0, SGI0, SGI0, SGI65974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14614[] = {SGI65975, SGI65976, SGI65977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65978, SGI0, SGI0, SGI0, SGI65979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14615[] = {SGI65980, SGI65981, SGI65982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65983, SGI0, SGI0, SGI0, SGI65984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14616[] = {SGI65985, SGI65986, SGI65987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65988, SGI0, SGI0, SGI0, SGI65989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14617[] = {SGI65990, SGI65991, SGI65992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65993, SGI0, SGI0, SGI0, SGI65994, SGI65994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14618[] = {SGI65995, SGI65996, SGI65997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI65998, SGI0, SGI0, SGI0, SGI65999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14619[] = {SGI66000, SGI66001, SGI66002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66003, SGI0, SGI0, SGI0, SGI66004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14620[] = {SGI66005, SGI66006, SGI66007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66008, SGI0, SGI0, SGI0, SGI66009, SGI66009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14621[] = {SGI66010, SGI66011, SGI66012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66013, SGI0, SGI0, SGI0, SGI66014, SGI66014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14622[] = {SGI66015, SGI66016, SGI66017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66018, SGI66018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14623[] = {SGI66019, SGI66020, SGI66021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66022, SGI0, SGI0, SGI0, SGI66023, SGI66023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14624[] = {SGI66024, SGI66025, SGI66026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66027, SGI0, SGI0, SGI0, SGI66028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14625[] = {SGI66029, SGI66030, SGI66031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66032, SGI0, SGI0, SGI0, SGI66033, SGI66033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14626[] = {SGI66034, SGI66035, SGI66036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66037, SGI0, SGI0, SGI0, SGI66038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14627[] = {SGI66039, SGI66040, SGI66041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66042, SGI0, SGI0, SGI0, SGI66043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14628[] = {SGI66045, SGI66047, SGI66049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66050, SGI0, SGI0, SGI0, SGI66052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14629[] = {SGI66044, SGI66046, SGI66048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66051, SGI0, SGI0, SGI0, SGI66053, SGI66053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14630[] = {SGI0, SGI66054, SGI66055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14631[] = {SGI66056, SGI66057, SGI66058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66059, SGI0, SGI0, SGI0, SGI66060, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14632[] = {SGI66061, SGI66062, SGI66063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66064, SGI0, SGI0, SGI0, SGI66065, SGI66065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14633[] = {SGI66066, SGI66067, SGI66068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66069, SGI0, SGI0, SGI0, SGI66070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14634[] = {SGI66071, SGI66072, SGI66073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66074, SGI0, SGI0, SGI0, SGI66075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14635[] = {SGI66076, SGI66077, SGI66078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66079, SGI0, SGI0, SGI0, SGI66080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14636[] = {SGI66084, SGI66085, SGI66086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66087, SGI0, SGI0, SGI0, SGI66088, SGI66088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14637[] = {SGI66081, SGI66082, SGI66083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66089, SGI0, SGI0, SGI0, SGI66090, SGI66090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14638[] = {SGI66091, SGI66092, SGI66093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66094, SGI0, SGI0, SGI0, SGI66095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14639[] = {SGI66096, SGI66097, SGI66099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66102, SGI0, SGI0, SGI0, SGI66103, SGI66103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14640[] = {SGI66098, SGI66100, SGI66101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66104, SGI0, SGI0, SGI0, SGI66105, SGI66105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14641[] = {SGI66106, SGI66107, SGI66108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66109, SGI0, SGI0, SGI0, SGI66110, SGI66110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14642[] = {SGI66111, SGI66112, SGI66113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66114, SGI0, SGI0, SGI0, SGI66115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14643[] = {SGI66116, SGI66117, SGI66118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66119, SGI0, SGI0, SGI0, SGI66120, SGI66120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14644[] = {SGI66121, SGI66122, SGI66123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66124, SGI0, SGI0, SGI0, SGI66125, SGI66125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14645[] = {SGI66126, SGI66127, SGI66128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66129, SGI0, SGI0, SGI0, SGI66130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14646[] = {SGI66131, SGI66132, SGI66133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66134, SGI0, SGI0, SGI0, SGI66135, SGI66135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14647[] = {SGI66136, SGI66137, SGI66138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66139, SGI0, SGI0, SGI0, SGI66140, SGI66140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14648[] = {SGI66141, SGI66142, SGI66143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66144, SGI0, SGI0, SGI0, SGI66145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14649[] = {SGI66146, SGI66147, SGI66148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66149, SGI0, SGI0, SGI0, SGI66150, SGI66150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14650[] = {SGI66151, SGI66152, SGI66153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66154, SGI0, SGI0, SGI0, SGI66155, SGI66155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14651[] = {SGI66156, SGI66157, SGI66158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66159, SGI0, SGI0, SGI0, SGI66160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14652[] = {SGI66161, SGI66162, SGI66163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66164, SGI0, SGI0, SGI0, SGI66165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14653[] = {SGI66166, SGI66167, SGI66168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66169, SGI0, SGI0, SGI0, SGI66170, SGI66170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14654[] = {SGI66171, SGI66172, SGI66173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66174, SGI0, SGI0, SGI0, SGI66175, SGI66175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14655[] = {SGI66176, SGI66177, SGI66178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66179, SGI0, SGI0, SGI0, SGI66180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14656[] = {SGI66181, SGI66182, SGI66183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66184, SGI0, SGI0, SGI0, SGI66185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14657[] = {SGI66186, SGI66187, SGI66188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66189, SGI0, SGI0, SGI0, SGI66190, SGI66190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14658[] = {SGI66191, SGI66192, SGI66193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66194, SGI0, SGI0, SGI0, SGI66195, SGI66195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14659[] = {SGI66196, SGI66197, SGI66198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66199, SGI0, SGI0, SGI0, SGI66200, SGI66200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14660[] = {SGI66201, SGI66202, SGI66203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66204, SGI0, SGI0, SGI0, SGI66205, SGI66205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14661[] = {SGI66206, SGI66207, SGI66208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66209, SGI0, SGI0, SGI0, SGI0, SGI66210, SGI0, SGI0, NULL};
const Segment * const SG14662[] = {SGI66211, SGI66212, SGI66213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66214, SGI0, SGI0, SGI0, SGI66215, SGI66215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14663[] = {SGI66216, SGI66217, SGI66218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66219, SGI0, SGI0, SGI0, SGI66220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14664[] = {SGI66221, SGI66222, SGI66223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66224, SGI0, SGI0, SGI0, SGI66225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14665[] = {SGI66226, SGI66227, SGI66228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66229, SGI0, SGI0, SGI0, SGI66230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14666[] = {SGI66231, SGI66232, SGI66233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66234, SGI0, SGI0, SGI0, SGI66235, SGI66235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14667[] = {SGI66236, SGI66237, SGI66239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66242, SGI0, SGI0, SGI0, SGI66244, SGI66244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14668[] = {SGI66238, SGI66240, SGI66241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66243, SGI0, SGI0, SGI0, SGI66245, SGI66245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14669[] = {SGI66246, SGI66247, SGI66248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66249, SGI0, SGI0, SGI0, SGI66250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14670[] = {SGI66251, SGI66252, SGI66253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66254, SGI0, SGI0, SGI0, SGI66255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14671[] = {SGI66256, SGI66257, SGI66258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66259, SGI0, SGI0, SGI0, SGI66260, SGI66260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14672[] = {SGI66261, SGI66262, SGI66263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66264, SGI0, SGI0, SGI0, SGI66265, SGI66265, SGI0, SGI0, NULL};
const Segment * const SG14673[] = {SGI66266, SGI66267, SGI66268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66269, SGI0, SGI0, SGI0, SGI66270, SGI66270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14674[] = {SGI66271, SGI66272, SGI66273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66274, SGI0, SGI0, SGI0, SGI66275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14675[] = {SGI66276, SGI66277, SGI66278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66280, SGI0, SGI0, SGI0, SGI66282, SGI66282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14676[] = {SGI66279, SGI66281, SGI66283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66284, SGI0, SGI0, SGI0, SGI66285, SGI66285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14677[] = {SGI66286, SGI66287, SGI66288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66291, SGI0, SGI0, SGI0, SGI66293, SGI66293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14678[] = {SGI66289, SGI66290, SGI66292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66294, SGI0, SGI0, SGI0, SGI66295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14679[] = {SGI0, SGI66296, SGI66297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66298, SGI0, SGI0, SGI0, SGI66299, SGI66299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14680[] = {SGI66300, SGI66301, SGI66302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66303, SGI0, SGI0, SGI0, SGI66304, SGI66304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14681[] = {SGI66305, SGI66306, SGI66307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66308, SGI0, SGI0, SGI0, SGI66309, SGI66309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14682[] = {SGI66310, SGI66311, SGI66312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66313, SGI0, SGI0, SGI0, SGI66314, SGI66314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14683[] = {SGI66315, SGI66316, SGI66317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66321, SGI66322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14684[] = {SGI66318, SGI66319, SGI66320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66323, SGI0, SGI0, SGI0, SGI66324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14685[] = {SGI66325, SGI66326, SGI66327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66328, SGI0, SGI0, SGI0, SGI66329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14686[] = {SGI66330, SGI66331, SGI66332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66333, SGI0, SGI0, SGI0, SGI66334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14687[] = {SGI66336, SGI0, SGI66337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14688[] = {SGI66335, SGI66338, SGI66339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66340, SGI0, SGI0, SGI0, SGI66341, SGI66341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14689[] = {SGI66342, SGI66343, SGI66344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66345, SGI0, SGI0, SGI0, SGI66346, SGI66346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14690[] = {SGI66347, SGI66348, SGI66349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66350, SGI0, SGI0, SGI0, SGI66351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14691[] = {SGI66352, SGI66353, SGI66354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66358, SGI66359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14692[] = {SGI66355, SGI66356, SGI66357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66360, SGI0, SGI0, SGI0, SGI66361, SGI66361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14693[] = {SGI66362, SGI66363, SGI66364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66365, SGI0, SGI0, SGI0, SGI66366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14694[] = {SGI66367, SGI66368, SGI66369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66370, SGI0, SGI0, SGI0, SGI66371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14695[] = {SGI66372, SGI66373, SGI66374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66375, SGI0, SGI0, SGI0, SGI66376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14696[] = {SGI66377, SGI66378, SGI66379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66380, SGI66381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14697[] = {SGI66382, SGI66383, SGI66384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66385, SGI0, SGI0, SGI0, SGI66386, SGI66386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14698[] = {SGI66387, SGI66388, SGI66389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66390, SGI0, SGI0, SGI0, SGI66391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14699[] = {SGI66392, SGI66393, SGI66394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66395, SGI0, SGI0, SGI0, SGI66396, SGI66396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14700[] = {SGI66397, SGI66398, SGI66399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66400, SGI0, SGI0, SGI0, SGI66401, SGI66401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14701[] = {SGI66402, SGI66403, SGI66404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66405, SGI66405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14702[] = {SGI66406, SGI66407, SGI66408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66409, SGI0, SGI0, SGI0, SGI66410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14703[] = {SGI66411, SGI66412, SGI66413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66414, SGI0, SGI0, SGI0, SGI66415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14704[] = {SGI66416, SGI66417, SGI66418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66419, SGI0, SGI0, SGI0, SGI66420, SGI66420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14705[] = {SGI66421, SGI66422, SGI66423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66424, SGI0, SGI0, SGI0, SGI66425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14706[] = {SGI66426, SGI66427, SGI66428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66429, SGI0, SGI0, SGI0, SGI66430, SGI66430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14707[] = {SGI66431, SGI66432, SGI66433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66434, SGI0, SGI0, SGI0, SGI66435, SGI66435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14708[] = {SGI66439, SGI66440, SGI66441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66442, SGI0, SGI0, SGI0, SGI66443, SGI66443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14709[] = {SGI66444, SGI66445, SGI66446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66447, SGI0, SGI0, SGI0, SGI66448, SGI66448, SGI0, SGI0, NULL};
const Segment * const SG14710[] = {SGI66449, SGI66450, SGI66451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66452, SGI0, SGI0, SGI0, SGI66453, SGI66453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14711[] = {SGI66454, SGI66455, SGI66456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66457, SGI0, SGI0, SGI0, SGI0, SGI66458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14712[] = {SGI66459, SGI66460, SGI66461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66462, SGI0, SGI0, SGI0, SGI66463, SGI66463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14713[] = {SGI66464, SGI66465, SGI66466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66467, SGI0, SGI0, SGI0, SGI66468, SGI66468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14714[] = {SGI66469, SGI66470, SGI66471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66472, SGI0, SGI0, SGI0, SGI66473, SGI66473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14715[] = {SGI66474, SGI66475, SGI66476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66477, SGI66478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14716[] = {SGI66479, SGI66480, SGI66481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66482, SGI0, SGI0, SGI0, SGI66483, SGI66483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14717[] = {SGI66484, SGI66485, SGI66486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66487, SGI0, SGI0, SGI0, SGI66488, SGI66488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14718[] = {SGI66489, SGI66490, SGI66491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66492, SGI0, SGI0, SGI0, SGI66493, SGI66493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14719[] = {SGI66494, SGI66495, SGI66496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66497, SGI0, SGI0, SGI0, SGI66498, SGI66498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14720[] = {SGI66499, SGI66500, SGI66501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66502, SGI0, SGI0, SGI0, SGI66503, SGI66503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14721[] = {SGI66504, SGI66505, SGI66506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66507, SGI0, SGI0, SGI0, SGI0, SGI66508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14722[] = {SGI66436, SGI66437, SGI66438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66509, SGI0, SGI0, SGI0, SGI66510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14723[] = {SGI66511, SGI66512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66513, SGI0, SGI0, SGI0, SGI66514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14724[] = {SGI66515, SGI66516, SGI66517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66518, SGI0, SGI0, SGI0, SGI66519, SGI66519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14725[] = {SGI66520, SGI66521, SGI66522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66523, SGI0, SGI0, SGI0, SGI0, SGI66524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14726[] = {SGI66525, SGI66526, SGI66527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66528, SGI0, SGI0, SGI0, SGI66529, SGI66529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14727[] = {SGI66531, SGI66533, SGI66535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66537, SGI0, SGI0, SGI0, SGI66539, SGI66539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14728[] = {SGI66530, SGI66532, SGI66534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66536, SGI0, SGI0, SGI0, SGI66538, SGI66538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14729[] = {SGI66540, SGI66541, SGI66542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66543, SGI0, SGI0, SGI0, SGI66544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14730[] = {SGI66545, SGI66546, SGI66547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66548, SGI0, SGI0, SGI0, SGI66549, SGI66549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14731[] = {SGI66550, SGI66551, SGI66552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66553, SGI0, SGI0, SGI0, SGI66554, SGI66554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14732[] = {SGI66555, SGI66556, SGI66557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66558, SGI0, SGI0, SGI0, SGI66559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14733[] = {SGI66560, SGI66561, SGI66562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66563, SGI0, SGI0, SGI0, SGI66564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14734[] = {SGI66565, SGI66566, SGI66567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66568, SGI0, SGI0, SGI0, SGI66569, SGI66569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14735[] = {SGI66570, SGI66571, SGI66572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66573, SGI0, SGI0, SGI0, SGI66574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14736[] = {SGI66575, SGI66576, SGI66577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66578, SGI0, SGI0, SGI0, SGI66579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14737[] = {SGI66580, SGI66581, SGI66582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66583, SGI0, SGI0, SGI0, SGI66584, SGI66584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14738[] = {SGI66585, SGI66586, SGI66587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66588, SGI0, SGI0, SGI0, SGI66589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14739[] = {SGI66590, SGI66591, SGI66592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66593, SGI0, SGI0, SGI0, SGI66594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14740[] = {SGI66595, SGI66596, SGI66597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66598, SGI0, SGI0, SGI0, SGI66599, SGI66599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14741[] = {SGI66600, SGI66601, SGI66602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66603, SGI0, SGI0, SGI0, SGI66604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14742[] = {SGI66605, SGI66606, SGI66607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66608, SGI0, SGI0, SGI0, SGI66609, SGI66609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14743[] = {SGI66610, SGI66611, SGI66612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66613, SGI0, SGI0, SGI0, SGI66614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14744[] = {SGI66615, SGI66616, SGI66617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66618, SGI0, SGI0, SGI0, SGI66619, SGI66619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14745[] = {SGI66620, SGI0, SGI66621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14746[] = {SGI66622, SGI66623, SGI66624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66625, SGI0, SGI0, SGI0, SGI66626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14747[] = {SGI66627, SGI66628, SGI66629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66630, SGI66630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14748[] = {SGI66631, SGI66632, SGI66633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66634, SGI0, SGI0, SGI0, SGI66635, SGI66635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14749[] = {SGI66636, SGI66637, SGI66638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66639, SGI0, SGI0, SGI0, SGI66640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14750[] = {SGI66641, SGI66642, SGI66643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66644, SGI0, SGI0, SGI0, SGI66645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14751[] = {SGI66646, SGI66647, SGI66648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66649, SGI0, SGI0, SGI0, SGI66650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14752[] = {SGI66651, SGI66652, SGI66653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66654, SGI0, SGI0, SGI0, SGI66655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14753[] = {SGI66656, SGI66657, SGI66658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66659, SGI0, SGI0, SGI0, SGI66660, SGI66660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14754[] = {SGI66661, SGI66662, SGI66663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66664, SGI0, SGI0, SGI0, SGI66665, SGI66665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14755[] = {SGI66666, SGI66667, SGI66668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66669, SGI0, SGI0, SGI0, SGI66670, SGI66670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14756[] = {SGI66671, SGI66672, SGI66673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66674, SGI0, SGI0, SGI0, SGI66675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14757[] = {SGI66676, SGI66677, SGI66678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66679, SGI0, SGI0, SGI0, SGI66680, SGI66680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14758[] = {SGI66681, SGI66682, SGI66683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66684, SGI0, SGI0, SGI0, SGI66685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14759[] = {SGI66686, SGI66687, SGI66688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66689, SGI66689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14760[] = {SGI66690, SGI66691, SGI66692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66693, SGI0, SGI0, SGI0, SGI66694, SGI66694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14761[] = {SGI0, SGI66695, SGI66696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14762[] = {SGI66697, SGI66698, SGI66699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66700, SGI0, SGI0, SGI0, SGI66701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14763[] = {SGI66702, SGI66703, SGI66704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66705, SGI0, SGI0, SGI0, SGI66706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14764[] = {SGI66707, SGI66708, SGI66709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66710, SGI0, SGI0, SGI0, SGI66711, SGI66711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14765[] = {SGI66712, SGI66713, SGI66714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66715, SGI0, SGI0, SGI0, SGI66716, SGI66716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14766[] = {SGI66717, SGI66718, SGI66719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66720, SGI0, SGI0, SGI0, SGI66721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14767[] = {SGI66722, SGI66723, SGI66724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66725, SGI0, SGI0, SGI0, SGI66726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14768[] = {SGI66727, SGI66728, SGI66729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66730, SGI0, SGI0, SGI0, SGI66731, SGI66731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14769[] = {SGI66732, SGI66733, SGI66734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66735, SGI0, SGI0, SGI0, SGI66736, SGI66736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14770[] = {SGI66737, SGI66738, SGI66739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66740, SGI0, SGI0, SGI0, SGI66741, SGI66741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14771[] = {SGI66742, SGI66743, SGI66744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66745, SGI0, SGI0, SGI0, SGI66746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14772[] = {SGI66747, SGI66748, SGI66749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66750, SGI0, SGI0, SGI0, SGI66751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14773[] = {SGI66752, SGI66753, SGI66754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66755, SGI0, SGI0, SGI0, SGI66756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14774[] = {SGI66757, SGI66758, SGI66759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66763, SGI66763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14775[] = {SGI66760, SGI66761, SGI66762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66764, SGI0, SGI0, SGI0, SGI66765, SGI66765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14776[] = {SGI66766, SGI66767, SGI66768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66769, SGI0, SGI0, SGI0, SGI66770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14777[] = {SGI66771, SGI66772, SGI66773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66774, SGI0, SGI0, SGI0, SGI66775, SGI66775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14778[] = {SGI66776, SGI66777, SGI66778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66779, SGI0, SGI0, SGI0, SGI66780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14779[] = {SGI66781, SGI66782, SGI66783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66784, SGI66784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14780[] = {SGI66785, SGI66787, SGI66789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66791, SGI0, SGI0, SGI0, SGI66793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14781[] = {SGI66786, SGI66788, SGI66790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66792, SGI0, SGI0, SGI0, SGI66794, SGI66794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14782[] = {SGI66795, SGI66796, SGI66797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66798, SGI0, SGI0, SGI0, SGI66799, SGI66799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14783[] = {SGI66800, SGI66801, SGI66802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66803, SGI0, SGI0, SGI0, SGI66804, SGI66804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14784[] = {SGI66805, SGI66806, SGI66807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66808, SGI0, SGI0, SGI0, SGI66809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14785[] = {SGI66810, SGI66811, SGI66812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66815, SGI0, SGI0, SGI0, SGI66817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14786[] = {SGI66813, SGI66814, SGI66816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66818, SGI0, SGI0, SGI0, SGI66819, SGI66819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14787[] = {SGI66820, SGI66821, SGI66822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66824, SGI0, SGI0, SGI0, SGI66826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14788[] = {SGI66823, SGI66825, SGI66827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66828, SGI0, SGI0, SGI0, SGI66829, SGI66829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14789[] = {SGI66830, SGI66831, SGI66832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66833, SGI0, SGI0, SGI0, SGI66834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14790[] = {SGI66835, SGI66836, SGI66837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66838, SGI0, SGI0, SGI0, SGI66839, SGI66839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14791[] = {SGI66840, SGI66841, SGI66842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66843, SGI0, SGI0, SGI0, SGI66844, SGI66844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14792[] = {SGI66845, SGI0, SGI66846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14793[] = {SGI66847, SGI66848, SGI66849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66850, SGI0, SGI0, SGI0, SGI66851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14794[] = {SGI66852, SGI66853, SGI66854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66855, SGI0, SGI0, SGI0, SGI66857, SGI66857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14795[] = {SGI66856, SGI66858, SGI66859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66860, SGI0, SGI0, SGI0, SGI66861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14796[] = {SGI66862, SGI66863, SGI66864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66865, SGI0, SGI0, SGI0, SGI66866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14797[] = {SGI66867, SGI66868, SGI66869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66870, SGI0, SGI0, SGI0, SGI66871, SGI66871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14798[] = {SGI66872, SGI66873, SGI66874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66875, SGI0, SGI0, SGI0, SGI66876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14799[] = {SGI66880, SGI66881, SGI66882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66883, SGI0, SGI0, SGI0, SGI66884, SGI66884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14800[] = {SGI66885, SGI0, SGI66886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66887, SGI0, SGI0, SGI0, SGI66888, SGI66888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14801[] = {SGI66889, SGI66890, SGI66891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66892, SGI66892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14802[] = {SGI66893, SGI66894, SGI66895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66896, SGI0, SGI0, SGI0, SGI66897, SGI66897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14803[] = {SGI66898, SGI66899, SGI66900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66901, SGI0, SGI0, SGI0, SGI66902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14804[] = {SGI66877, SGI66878, SGI66879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66903, SGI66903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14805[] = {SGI66904, SGI66905, SGI66906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66907, SGI0, SGI0, SGI0, SGI66908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14806[] = {SGI66909, SGI66910, SGI66911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66912, SGI0, SGI0, SGI0, SGI66913, SGI66913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14807[] = {SGI66914, SGI66915, SGI66916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66917, SGI0, SGI0, SGI0, SGI66918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14808[] = {SGI66919, SGI66920, SGI66921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66922, SGI0, SGI0, SGI0, SGI66923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14809[] = {SGI66924, SGI66925, SGI66926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66927, SGI0, SGI0, SGI0, SGI66928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14810[] = {SGI66929, SGI66930, SGI66931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66932, SGI0, SGI0, SGI0, SGI66933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14811[] = {SGI66934, SGI66935, SGI66936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66937, SGI0, SGI0, SGI0, SGI66938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14812[] = {SGI66939, SGI66940, SGI66941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66942, SGI0, SGI0, SGI0, SGI66943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14813[] = {SGI66944, SGI66945, SGI66946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66947, SGI0, SGI0, SGI0, SGI66948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14814[] = {SGI66949, SGI66950, SGI66951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66952, SGI0, SGI0, SGI0, SGI66953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14815[] = {SGI66954, SGI66955, SGI66956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66957, SGI0, SGI0, SGI0, SGI66958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14816[] = {SGI66959, SGI66960, SGI66961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66962, SGI0, SGI0, SGI0, SGI66963, SGI66963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14817[] = {SGI66964, SGI66965, SGI66966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66967, SGI0, SGI0, SGI0, SGI66968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14818[] = {SGI66969, SGI66970, SGI66971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66972, SGI0, SGI0, SGI0, SGI66973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14819[] = {SGI66974, SGI66975, SGI66976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66977, SGI0, SGI0, SGI0, SGI66978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14820[] = {SGI66979, SGI66980, SGI66981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66982, SGI0, SGI0, SGI0, SGI66983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14821[] = {SGI66984, SGI66985, SGI66986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66987, SGI0, SGI0, SGI0, SGI66988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14822[] = {SGI66989, SGI66990, SGI66991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66992, SGI0, SGI0, SGI0, SGI66993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14823[] = {SGI66994, SGI66995, SGI66996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI66997, SGI0, SGI0, SGI0, SGI66998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14824[] = {SGI66999, SGI67000, SGI67001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67002, SGI0, SGI0, SGI0, SGI67003, SGI67003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14825[] = {SGI67004, SGI67005, SGI67006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67007, SGI0, SGI0, SGI0, SGI67008, SGI67008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14826[] = {SGI67009, SGI67010, SGI67011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67012, SGI0, SGI0, SGI0, SGI67013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14827[] = {SGI67014, SGI67015, SGI67016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67017, SGI0, SGI0, SGI0, SGI67018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14828[] = {SGI67019, SGI67020, SGI67021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67022, SGI0, SGI0, SGI0, SGI67023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14829[] = {SGI67024, SGI67025, SGI67026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67027, SGI0, SGI0, SGI0, SGI67028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14830[] = {SGI67029, SGI67030, SGI67031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67032, SGI67032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14831[] = {SGI0, SGI67033, SGI67034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67035, SGI0, SGI0, SGI0, SGI67036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14832[] = {SGI67037, SGI67038, SGI67039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67040, SGI0, SGI0, SGI0, SGI67041, SGI67041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14833[] = {SGI67042, SGI67043, SGI67044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67045, SGI0, SGI0, SGI0, SGI67046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14834[] = {SGI0, SGI67047, SGI67048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67049, SGI0, SGI0, SGI0, SGI67050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14835[] = {SGI67051, SGI67052, SGI67053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67054, SGI0, SGI0, SGI0, SGI67055, SGI67055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14836[] = {SGI67056, SGI67057, SGI67058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67059, SGI0, SGI0, SGI0, SGI67060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14837[] = {SGI0, SGI67061, SGI67062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67063, SGI0, SGI0, SGI0, SGI67064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14838[] = {SGI67065, SGI67066, SGI67067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67068, SGI0, SGI0, SGI0, SGI67069, SGI67069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14839[] = {SGI67070, SGI67071, SGI67072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67073, SGI0, SGI0, SGI0, SGI67074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14840[] = {SGI67075, SGI67076, SGI67077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67078, SGI0, SGI0, SGI0, SGI67079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14841[] = {SGI67080, SGI67081, SGI67082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67083, SGI0, SGI0, SGI0, SGI67084, SGI67084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14842[] = {SGI67085, SGI0, SGI67086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14843[] = {SGI67087, SGI67088, SGI67089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67090, SGI0, SGI0, SGI0, SGI67091, SGI67091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14844[] = {SGI67092, SGI67093, SGI67094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67095, SGI0, SGI0, SGI0, SGI67096, SGI67096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14845[] = {SGI67097, SGI67098, SGI67099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67100, SGI0, SGI0, SGI0, SGI67101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14846[] = {SGI67102, SGI67103, SGI67104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67105, SGI0, SGI0, SGI0, SGI67106, SGI67106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14847[] = {SGI67107, SGI67108, SGI67109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67110, SGI0, SGI0, SGI0, SGI67111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14848[] = {SGI67112, SGI67113, SGI67114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67115, SGI0, SGI0, SGI0, SGI67116, SGI67116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14849[] = {SGI67117, SGI67118, SGI67119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67120, SGI0, SGI0, SGI0, SGI67121, SGI67121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14850[] = {SGI67122, SGI67123, SGI67124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67125, SGI0, SGI0, SGI0, SGI67126, SGI67126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14851[] = {SGI67127, SGI67128, SGI67129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67130, SGI0, SGI0, SGI0, SGI67131, SGI67131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14852[] = {SGI67132, SGI67133, SGI67134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67135, SGI0, SGI0, SGI0, SGI67136, SGI67136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14853[] = {SGI67137, SGI67138, SGI67139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67140, SGI0, SGI0, SGI0, SGI67141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14854[] = {SGI67142, SGI67143, SGI67144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67145, SGI0, SGI0, SGI0, SGI67146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14855[] = {SGI67147, SGI67148, SGI67149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67150, SGI0, SGI0, SGI0, SGI67151, SGI67151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14856[] = {SGI67152, SGI67153, SGI67154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67155, SGI0, SGI0, SGI0, SGI67156, SGI67156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14857[] = {SGI67157, SGI67158, SGI67159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67160, SGI0, SGI0, SGI0, SGI67161, SGI67161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14858[] = {SGI67162, SGI67163, SGI67164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67165, SGI0, SGI0, SGI0, SGI67166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14859[] = {SGI67167, SGI67168, SGI67169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67170, SGI0, SGI0, SGI0, SGI67171, SGI67171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14860[] = {SGI67172, SGI67173, SGI67174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67175, SGI0, SGI0, SGI0, SGI67176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14861[] = {SGI67177, SGI67178, SGI67179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67180, SGI0, SGI0, SGI0, SGI67181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14862[] = {SGI67182, SGI67183, SGI67184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67185, SGI0, SGI0, SGI0, SGI67186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14863[] = {SGI67187, SGI67188, SGI67189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67190, SGI0, SGI0, SGI0, SGI67191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14864[] = {SGI0, SGI67192, SGI67193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67194, SGI0, SGI0, SGI0, SGI0, SGI67195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14865[] = {SGI67196, SGI67197, SGI67198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67199, SGI0, SGI0, SGI0, SGI67200, SGI67200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14866[] = {SGI67201, SGI67202, SGI67203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67204, SGI0, SGI0, SGI0, SGI67205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14867[] = {SGI67206, SGI67207, SGI67208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67209, SGI0, SGI0, SGI0, SGI67210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14868[] = {SGI67211, SGI67212, SGI67213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67214, SGI0, SGI0, SGI0, SGI0, SGI67215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14869[] = {SGI67216, SGI67217, SGI67218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67219, SGI0, SGI0, SGI0, SGI0, SGI67220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14870[] = {SGI67221, SGI67222, SGI67223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67224, SGI0, SGI0, SGI0, SGI67225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14871[] = {SGI67226, SGI67227, SGI67228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67229, SGI0, SGI0, SGI0, SGI67230, SGI67230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14872[] = {SGI67231, SGI67232, SGI67233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67234, SGI0, SGI0, SGI0, SGI67235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14873[] = {SGI67236, SGI67237, SGI67238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67239, SGI0, SGI0, SGI0, SGI67240, SGI67240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14874[] = {SGI67241, SGI67242, SGI67243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67244, SGI0, SGI0, SGI0, SGI67245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14875[] = {SGI67246, SGI67247, SGI67248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67249, SGI0, SGI0, SGI0, SGI67250, SGI67250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14876[] = {SGI67251, SGI67252, SGI67253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67254, SGI0, SGI0, SGI0, SGI67255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14877[] = {SGI67256, SGI67257, SGI67258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67259, SGI0, SGI0, SGI0, SGI67260, SGI67260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14878[] = {SGI67261, SGI67262, SGI67263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67264, SGI0, SGI0, SGI0, SGI67265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14879[] = {SGI67266, SGI67267, SGI67268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67269, SGI0, SGI0, SGI0, SGI67270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14880[] = {SGI67271, SGI67272, SGI67273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67274, SGI0, SGI0, SGI0, SGI67275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14881[] = {SGI67276, SGI67277, SGI67278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67279, SGI0, SGI0, SGI0, SGI67280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14882[] = {SGI67281, SGI0, SGI67282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14883[] = {SGI67283, SGI67284, SGI67285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67286, SGI0, SGI0, SGI0, SGI67287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14884[] = {SGI67288, SGI67289, SGI67290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67291, SGI0, SGI0, SGI0, SGI67292, SGI67292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14885[] = {SGI67293, SGI67294, SGI67295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67296, SGI0, SGI0, SGI0, SGI67297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14886[] = {SGI67298, SGI67299, SGI67300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67301, SGI0, SGI0, SGI0, SGI67302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14887[] = {SGI67303, SGI67304, SGI67305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67306, SGI0, SGI0, SGI0, SGI67307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14888[] = {SGI67308, SGI67309, SGI67310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67311, SGI0, SGI0, SGI0, SGI67312, SGI67312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14889[] = {SGI67313, SGI67314, SGI67315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67316, SGI0, SGI0, SGI0, SGI67317, SGI67317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14890[] = {SGI67318, SGI67319, SGI67320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67321, SGI0, SGI0, SGI0, SGI67322, SGI67322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14891[] = {SGI67323, SGI0, SGI67324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14892[] = {SGI67325, SGI67326, SGI67327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67328, SGI0, SGI0, SGI0, SGI67331, SGI67331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14893[] = {SGI67329, SGI67330, SGI67332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67333, SGI0, SGI0, SGI0, SGI67334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14894[] = {SGI67335, SGI67336, SGI67337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67338, SGI0, SGI0, SGI0, SGI67339, SGI67339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14895[] = {SGI67340, SGI67341, SGI67342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67343, SGI0, SGI0, SGI0, SGI67344, SGI67344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14896[] = {SGI67345, SGI67346, SGI67347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67348, SGI0, SGI0, SGI0, SGI67349, SGI67349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14897[] = {SGI67350, SGI0, SGI67351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67352, SGI0, SGI0, SGI0, SGI67353, SGI67353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14898[] = {SGI67354, SGI67355, SGI67356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67357, SGI0, SGI0, SGI0, SGI67358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14899[] = {SGI67359, SGI67360, SGI67361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67362, SGI67363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14900[] = {SGI67364, SGI67365, SGI67366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67367, SGI0, SGI0, SGI0, SGI67368, SGI67369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14901[] = {SGI67370, SGI67371, SGI67372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67373, SGI0, SGI0, SGI0, SGI67374, SGI67374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14902[] = {SGI67375, SGI0, SGI67376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14903[] = {SGI67377, SGI0, SGI67378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14904[] = {SGI67379, SGI67380, SGI67381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67382, SGI67382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14905[] = {SGI67383, SGI67384, SGI67385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67386, SGI67386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14906[] = {SGI67387, SGI67388, SGI67389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67390, SGI67390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14907[] = {SGI67391, SGI67392, SGI67393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14908[] = {SGI67394, SGI67395, SGI67396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67397, SGI0, SGI0, SGI0, SGI67398, SGI67398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14909[] = {SGI67399, SGI67400, SGI67401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14910[] = {SGI0, SGI67402, SGI67403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14911[] = {SGI67404, SGI67405, SGI67406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67407, SGI0, SGI0, SGI0, SGI67408, SGI67408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14912[] = {SGI67409, SGI67410, SGI67411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67412, SGI0, SGI0, SGI0, SGI0, SGI67413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14913[] = {SGI67414, SGI67415, SGI67416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14914[] = {SGI67417, SGI67418, SGI67419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14915[] = {SGI67420, SGI67421, SGI67422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67423, SGI0, SGI0, SGI0, SGI67424, SGI67424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14916[] = {SGI67425, SGI67426, SGI67427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67428, SGI0, SGI0, SGI0, SGI67429, SGI67429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14917[] = {SGI67430, SGI67431, SGI67432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14918[] = {SGI67433, SGI0, SGI67434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14919[] = {SGI67435, SGI67436, SGI67437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67438, SGI0, SGI0, SGI0, SGI67439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14920[] = {SGI67440, SGI67441, SGI67442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14921[] = {SGI67443, SGI67444, SGI67445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67446, SGI0, SGI0, SGI0, SGI67447, SGI67447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14922[] = {SGI67448, SGI67449, SGI67450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67451, SGI0, SGI0, SGI0, SGI67452, SGI67452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14923[] = {SGI67453, SGI67454, SGI67455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67456, SGI0, SGI0, SGI0, SGI67457, SGI67457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14924[] = {SGI67458, SGI0, SGI67459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67460, SGI0, SGI0, SGI0, SGI67461, SGI67461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14925[] = {SGI67462, SGI67463, SGI67464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14926[] = {SGI67465, SGI67466, SGI67467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67468, SGI0, SGI0, SGI0, SGI67469, SGI67469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14927[] = {SGI0, SGI67470, SGI67471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14928[] = {SGI67472, SGI67473, SGI67474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67475, SGI0, SGI0, SGI0, SGI67476, SGI67476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14929[] = {SGI67477, SGI0, SGI67478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67479, SGI0, SGI0, SGI0, SGI67480, SGI67480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14930[] = {SGI67481, SGI67482, SGI67483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67484, SGI0, SGI0, SGI0, SGI67485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14931[] = {SGI67486, SGI67487, SGI67488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67489, SGI0, SGI0, SGI0, SGI67490, SGI67490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14932[] = {SGI67491, SGI67492, SGI67493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67494, SGI0, SGI0, SGI0, SGI0, SGI67495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14933[] = {SGI67496, SGI67497, SGI67498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67499, SGI0, SGI0, SGI0, SGI67500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14934[] = {SGI67501, SGI67502, SGI67503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67504, SGI0, SGI0, SGI0, SGI67505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14935[] = {SGI67506, SGI67507, SGI67508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67509, SGI0, SGI0, SGI0, SGI67510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14936[] = {SGI67511, SGI67512, SGI67513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67514, SGI0, SGI0, SGI0, SGI67515, SGI67515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14937[] = {SGI67516, SGI67517, SGI67518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67519, SGI0, SGI0, SGI0, SGI67520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14938[] = {SGI67521, SGI67522, SGI67523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67524, SGI0, SGI0, SGI0, SGI67525, SGI67525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14939[] = {SGI67526, SGI67527, SGI67528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67529, SGI0, SGI0, SGI0, SGI67530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14940[] = {SGI0, SGI0, SGI67531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67532, SGI0, SGI0, SGI0, SGI67533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14941[] = {SGI67534, SGI67535, SGI67536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67537, SGI0, SGI0, SGI0, SGI67538, SGI67538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14942[] = {SGI67539, SGI67540, SGI67541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67542, SGI0, SGI0, SGI0, SGI67543, SGI67543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14943[] = {SGI67544, SGI67545, SGI67546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67547, SGI0, SGI0, SGI0, SGI67548, SGI67548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14944[] = {SGI67549, SGI67550, SGI67551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67552, SGI0, SGI0, SGI0, SGI67553, SGI67553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14945[] = {SGI67554, SGI67555, SGI67556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67557, SGI0, SGI0, SGI0, SGI67558, SGI67558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14946[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67559, SGI0, SGI67560, SGI0, NULL};
const Segment * const SG14947[] = {SGI67561, SGI67562, SGI67563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67564, SGI0, SGI0, SGI0, SGI67565, SGI67565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14948[] = {SGI67566, SGI67567, SGI67568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67569, SGI0, SGI0, SGI0, SGI67570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14949[] = {SGI67571, SGI67572, SGI67573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67574, SGI0, SGI0, SGI0, SGI67575, SGI67575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14950[] = {SGI67576, SGI67577, SGI67578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67579, SGI0, SGI0, SGI0, SGI67580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14951[] = {SGI67581, SGI67582, SGI67583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67584, SGI0, SGI0, SGI0, SGI67585, SGI67585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14952[] = {SGI67586, SGI67587, SGI67588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67589, SGI0, SGI0, SGI0, SGI67590, SGI67590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14953[] = {SGI67591, SGI67592, SGI67593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67594, SGI0, SGI0, SGI0, SGI67595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14954[] = {SGI67596, SGI67597, SGI67598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67599, SGI0, SGI0, SGI0, SGI67600, SGI67600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14955[] = {SGI67601, SGI67602, SGI67603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67604, SGI0, SGI0, SGI0, SGI67605, SGI67605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14956[] = {SGI67606, SGI67607, SGI67608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67609, SGI0, SGI0, SGI0, SGI67610, SGI67610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14957[] = {SGI67611, SGI0, SGI67612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14958[] = {SGI67613, SGI67614, SGI67615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67616, SGI67617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14959[] = {SGI67618, SGI67619, SGI67620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67621, SGI0, SGI0, SGI0, SGI67622, SGI67622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14960[] = {SGI67623, SGI67624, SGI67625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67626, SGI67626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14961[] = {SGI67627, SGI0, SGI67628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14962[] = {SGI67629, SGI0, SGI67630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67631, SGI0, SGI0, SGI0, SGI67632, SGI67632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14963[] = {SGI0, SGI67633, SGI67634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14964[] = {SGI67635, SGI0, SGI67636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14965[] = {SGI67637, SGI67638, SGI67639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67640, SGI0, SGI0, SGI0, SGI67641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14966[] = {SGI67642, SGI67643, SGI67644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67645, SGI0, SGI0, SGI0, SGI67646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14967[] = {SGI67647, SGI67648, SGI67649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67650, SGI0, SGI0, SGI0, SGI67651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14968[] = {SGI67652, SGI67653, SGI67654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67655, SGI0, SGI0, SGI0, SGI67656, SGI67656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14969[] = {SGI67657, SGI67658, SGI67659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67660, SGI0, SGI0, SGI0, SGI67661, SGI67661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14970[] = {SGI67662, SGI67663, SGI67664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67665, SGI0, SGI0, SGI0, SGI67666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14971[] = {SGI67667, SGI67668, SGI67669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67670, SGI0, SGI0, SGI0, SGI67671, SGI67671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14972[] = {SGI67672, SGI67673, SGI67674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67675, SGI0, SGI0, SGI0, SGI67676, SGI67676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14973[] = {SGI67677, SGI67678, SGI67679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67680, SGI0, SGI0, SGI0, SGI67681, SGI67681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14974[] = {SGI67682, SGI67683, SGI67684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67685, SGI0, SGI0, SGI0, SGI67686, SGI67686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14975[] = {SGI67687, SGI67688, SGI67689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67690, SGI0, SGI0, SGI0, SGI67691, SGI67691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14976[] = {SGI67692, SGI67693, SGI67694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67695, SGI0, SGI0, SGI0, SGI67696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14977[] = {SGI67697, SGI67698, SGI67699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67700, SGI0, SGI0, SGI0, SGI67701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14978[] = {SGI67702, SGI0, SGI67703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14979[] = {SGI67704, SGI67705, SGI67706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67707, SGI0, SGI0, SGI0, SGI67708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14980[] = {SGI67709, SGI67710, SGI67711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67712, SGI0, SGI0, SGI0, SGI67713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14981[] = {SGI67714, SGI67715, SGI67716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67717, SGI0, SGI0, SGI0, SGI67718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14982[] = {SGI67719, SGI67720, SGI67721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67722, SGI67722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14983[] = {SGI67723, SGI67724, SGI67725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67726, SGI0, SGI0, SGI0, SGI67727, SGI67727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14984[] = {SGI67728, SGI67729, SGI67730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67731, SGI0, SGI0, SGI0, SGI67732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14985[] = {SGI67733, SGI67734, SGI67735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67736, SGI0, SGI0, SGI0, SGI67737, SGI67737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14986[] = {SGI67738, SGI0, SGI67739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14987[] = {SGI67740, SGI67741, SGI67742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67743, SGI0, SGI0, SGI0, SGI67744, SGI67745, SGI0, SGI0, NULL};
const Segment * const SG14988[] = {SGI67746, SGI67747, SGI67748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67749, SGI0, SGI0, SGI0, SGI67750, SGI67750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14989[] = {SGI67751, SGI67752, SGI67753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67754, SGI0, SGI0, SGI0, SGI67755, SGI67755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14990[] = {SGI67756, SGI67757, SGI67758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67759, SGI0, SGI0, SGI0, SGI67760, SGI67760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14991[] = {SGI67761, SGI67762, SGI67763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67764, SGI0, SGI0, SGI0, SGI67765, SGI67765, SGI0, SGI0, NULL};
const Segment * const SG14992[] = {SGI67766, SGI67767, SGI67768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67769, SGI0, SGI0, SGI0, SGI67770, SGI67770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14993[] = {SGI67771, SGI67772, SGI67773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67774, SGI0, SGI0, SGI0, SGI67775, SGI67775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14994[] = {SGI67776, SGI0, SGI67777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14995[] = {SGI67778, SGI67779, SGI67780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67781, SGI67782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14996[] = {SGI67783, SGI67784, SGI67785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67786, SGI0, SGI0, SGI0, SGI67787, SGI67787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14997[] = {SGI67788, SGI0, SGI67789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67790, SGI0, SGI0, SGI0, SGI67791, SGI67791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14998[] = {SGI67792, SGI0, SGI67793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG14999[] = {SGI67794, SGI67795, SGI67796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67797, SGI0, SGI0, SGI0, SGI67798, SGI67798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15000[] = {SGI67799, SGI67800, SGI67801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67802, SGI0, SGI0, SGI0, SGI67803, SGI67803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15001[] = {SGI67804, SGI0, SGI67805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15002[] = {SGI67806, SGI67807, SGI67808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67809, SGI0, SGI0, SGI0, SGI67810, SGI67810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15003[] = {SGI67811, SGI0, SGI67812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15004[] = {SGI67813, SGI0, SGI67814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15005[] = {SGI67815, SGI0, SGI67816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15006[] = {SGI67817, SGI67818, SGI67819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67820, SGI67821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15007[] = {SGI67822, SGI67823, SGI67824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67825, SGI0, SGI0, SGI0, SGI67826, SGI67826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15008[] = {SGI67827, SGI67828, SGI67829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67830, SGI0, SGI0, SGI0, SGI67831, SGI67831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15009[] = {SGI67832, SGI67833, SGI67834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67835, SGI0, SGI0, SGI0, SGI67836, SGI67836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15010[] = {SGI67837, SGI0, SGI67838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15011[] = {SGI67839, SGI0, SGI67840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67841, SGI0, SGI0, SGI0, SGI67842, SGI67842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15012[] = {SGI67843, SGI67844, SGI67845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67846, SGI67846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15013[] = {SGI67847, SGI67848, SGI67849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67850, SGI67850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15014[] = {SGI67851, SGI67852, SGI67853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67854, SGI0, SGI0, SGI0, SGI67855, SGI67855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15015[] = {SGI0, SGI0, SGI67856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67857, SGI0, SGI0, SGI0, SGI67858, SGI67858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15016[] = {SGI67859, SGI67860, SGI67861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67862, SGI0, SGI0, SGI0, SGI67863, SGI67863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15017[] = {SGI67864, SGI67865, SGI67866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67867, SGI0, SGI0, SGI0, SGI67868, SGI67868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15018[] = {SGI67869, SGI67870, SGI67871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67872, SGI0, SGI0, SGI0, SGI67873, SGI67873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15019[] = {SGI67874, SGI67875, SGI67876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67877, SGI0, SGI0, SGI0, SGI67878, SGI67878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15020[] = {SGI67879, SGI67880, SGI67881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67882, SGI0, SGI0, SGI0, SGI67883, SGI67883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15021[] = {SGI0, SGI0, SGI67884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67885, SGI0, SGI0, SGI0, SGI67886, SGI67886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15022[] = {SGI67887, SGI67888, SGI67889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67890, SGI0, SGI0, SGI0, SGI67891, SGI67891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15023[] = {SGI67892, SGI67893, SGI67894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67895, SGI0, SGI0, SGI0, SGI67896, SGI67896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15024[] = {SGI67897, SGI0, SGI67898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15025[] = {SGI67899, SGI0, SGI67900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15026[] = {SGI67901, SGI67902, SGI67903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67904, SGI0, SGI0, SGI0, SGI67905, SGI67905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15027[] = {SGI67906, SGI67907, SGI67908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67909, SGI0, SGI0, SGI0, SGI67910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15028[] = {SGI67911, SGI67912, SGI67913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15029[] = {SGI67914, SGI67915, SGI67916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67917, SGI0, SGI0, SGI0, SGI67918, SGI67918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15030[] = {SGI67919, SGI67920, SGI67921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67922, SGI67923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15031[] = {SGI67924, SGI67925, SGI67926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67927, SGI0, SGI0, SGI0, SGI67928, SGI67928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15032[] = {SGI67929, SGI67930, SGI67931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67932, SGI0, SGI0, SGI0, SGI67933, SGI67933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15033[] = {SGI67934, SGI67935, SGI67936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67937, SGI0, SGI0, SGI0, SGI67938, SGI67938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15034[] = {SGI67939, SGI67940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67941, SGI0, SGI0, SGI0, SGI67942, SGI67942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15035[] = {SGI67943, SGI0, SGI67944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67945, SGI0, SGI0, SGI0, SGI67946, SGI67946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15036[] = {SGI67947, SGI67948, SGI67949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67950, SGI0, SGI0, SGI0, SGI67951, SGI67951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15037[] = {SGI67952, SGI67953, SGI67954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67955, SGI0, SGI0, SGI0, SGI67956, SGI67957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15038[] = {SGI67958, SGI67959, SGI67960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67961, SGI67961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15039[] = {SGI67962, SGI67963, SGI67964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67965, SGI67965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15040[] = {SGI67966, SGI67967, SGI67968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67969, SGI0, SGI0, SGI0, SGI67970, SGI67970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15041[] = {SGI67971, SGI67972, SGI67973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67974, SGI0, SGI0, SGI0, SGI67975, SGI67975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15042[] = {SGI67976, SGI0, SGI67977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15043[] = {SGI67978, SGI67979, SGI67980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67981, SGI0, SGI0, SGI0, SGI67982, SGI67982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15044[] = {SGI67983, SGI0, SGI67984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15045[] = {SGI67985, SGI0, SGI67986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15046[] = {SGI67987, SGI67988, SGI67989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67990, SGI0, SGI0, SGI0, SGI67991, SGI67991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15047[] = {SGI0, SGI0, SGI67992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15048[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67993, SGI0, NULL};
const Segment * const SG15049[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67994, SGI0, SGI67995, SGI0, NULL};
const Segment * const SG15050[] = {SGI67996, SGI67997, SGI67998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI67999, SGI0, SGI0, SGI0, SGI68000, SGI0, SGI0, SGI68001, SGI0, NULL};
const Segment * const SG15051[] = {SGI68002, SGI68003, SGI68004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68005, SGI0, SGI0, SGI0, SGI68006, SGI68006, SGI0, SGI68007, SGI0, NULL};
const Segment * const SG15052[] = {SGI68008, SGI68009, SGI68010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68011, SGI68011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15053[] = {SGI68012, SGI68013, SGI68014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68015, SGI0, SGI0, SGI0, SGI68016, SGI68016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15054[] = {SGI68017, SGI0, SGI68018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15055[] = {SGI68019, SGI68020, SGI68021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68022, SGI0, SGI0, SGI0, SGI68023, SGI68023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15056[] = {SGI68024, SGI68025, SGI68026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68027, SGI0, SGI0, SGI0, SGI68028, SGI68028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15057[] = {SGI68029, SGI68030, SGI68031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68032, SGI0, SGI0, SGI0, SGI68033, SGI68033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15058[] = {SGI68034, SGI68035, SGI68036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68037, SGI0, SGI0, SGI0, SGI68038, SGI68038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15059[] = {SGI68039, SGI68040, SGI68041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68042, SGI0, SGI0, SGI0, SGI68043, SGI68043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15060[] = {SGI68044, SGI68045, SGI68046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68047, SGI0, SGI0, SGI0, SGI68048, SGI68048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15061[] = {SGI68049, SGI68050, SGI68051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68052, SGI0, SGI0, SGI0, SGI68053, SGI68053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15062[] = {SGI68054, SGI68055, SGI68056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68057, SGI68057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15063[] = {SGI68058, SGI0, SGI68059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15064[] = {SGI68060, SGI0, SGI68061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15065[] = {SGI68062, SGI0, SGI68063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15066[] = {SGI68064, SGI68065, SGI68066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68067, SGI0, SGI0, SGI0, SGI68068, SGI68068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15067[] = {SGI68069, SGI68070, SGI68071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68072, SGI68073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15068[] = {SGI68074, SGI0, SGI68075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68076, SGI0, SGI0, SGI0, SGI68077, SGI68077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15069[] = {SGI68078, SGI68079, SGI68080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68081, SGI0, SGI0, SGI0, SGI68082, SGI68082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15070[] = {SGI68083, SGI0, SGI68084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15071[] = {SGI68085, SGI68086, SGI68087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68088, SGI0, SGI0, SGI0, SGI68089, SGI68089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15072[] = {SGI68090, SGI68091, SGI68092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68093, SGI0, SGI0, SGI0, SGI68094, SGI68094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15073[] = {SGI68095, SGI68096, SGI68097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68098, SGI0, SGI0, SGI0, SGI68099, SGI68099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15074[] = {SGI68100, SGI0, SGI68101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15075[] = {SGI68102, SGI68103, SGI68104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68105, SGI0, SGI0, SGI0, SGI68106, SGI68106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15076[] = {SGI68107, SGI68108, SGI68109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68110, SGI0, SGI0, SGI0, SGI68111, SGI68111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15077[] = {SGI68112, SGI68113, SGI68114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68115, SGI0, SGI0, SGI0, SGI68116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15078[] = {SGI68117, SGI68118, SGI68119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68120, SGI0, SGI0, SGI0, SGI68121, SGI68121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15079[] = {SGI68122, SGI68123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68124, SGI0, SGI0, SGI0, SGI68125, SGI68125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15080[] = {SGI68126, SGI68127, SGI68128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68129, SGI68129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15081[] = {SGI68130, SGI68131, SGI68132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68133, SGI0, SGI0, SGI0, SGI68134, SGI68135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15082[] = {SGI68136, SGI68137, SGI68138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68139, SGI0, SGI0, SGI0, SGI68140, SGI68140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15083[] = {SGI68141, SGI0, SGI68142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68143, SGI0, SGI0, SGI0, SGI68144, SGI68144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15084[] = {SGI68145, SGI68146, SGI68147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68148, SGI0, SGI0, SGI0, SGI68149, SGI68149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15085[] = {SGI68150, SGI68151, SGI68152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68153, SGI0, SGI0, SGI0, SGI68154, SGI68154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15086[] = {SGI68155, SGI68156, SGI68157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68158, SGI0, SGI0, SGI0, SGI68159, SGI68159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15087[] = {SGI68160, SGI68161, SGI68162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68163, SGI0, SGI0, SGI0, SGI68164, SGI68164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15088[] = {SGI68165, SGI0, SGI68166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15089[] = {SGI68167, SGI68168, SGI68169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68170, SGI0, SGI0, SGI0, SGI68171, SGI68171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15090[] = {SGI68172, SGI68173, SGI68174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68175, SGI0, SGI0, SGI0, SGI68176, SGI68176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15091[] = {SGI68177, SGI68178, SGI68179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68180, SGI0, SGI0, SGI0, SGI68181, SGI68181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15092[] = {SGI68182, SGI68183, SGI68184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68185, SGI0, SGI0, SGI0, SGI68186, SGI68186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15093[] = {SGI68187, SGI68188, SGI68189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68190, SGI0, SGI0, SGI0, SGI68191, SGI68191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15094[] = {SGI68192, SGI68193, SGI68194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68195, SGI0, SGI0, SGI0, SGI68196, SGI68196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15095[] = {SGI68197, SGI68198, SGI68199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68200, SGI0, SGI0, SGI0, SGI68201, SGI68201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15096[] = {SGI68202, SGI68203, SGI68204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68205, SGI0, SGI0, SGI0, SGI68206, SGI68206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15097[] = {SGI68207, SGI68208, SGI68209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68210, SGI68210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15098[] = {SGI68211, SGI0, SGI68212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15099[] = {SGI68213, SGI0, SGI68214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68215, SGI0, SGI0, SGI0, SGI68216, SGI68216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15100[] = {SGI68217, SGI0, SGI68218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15101[] = {SGI68219, SGI68220, SGI68221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68222, SGI0, SGI0, SGI0, SGI68223, SGI68223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15102[] = {SGI68224, SGI68225, SGI68226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68227, SGI0, SGI0, SGI0, SGI68228, SGI68228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15103[] = {SGI68224, SGI68225, SGI68226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68227, SGI0, SGI0, SGI0, SGI68228, SGI68228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15104[] = {SGI68229, SGI68230, SGI68231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68232, SGI0, SGI0, SGI0, SGI68233, SGI68233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15105[] = {SGI68234, SGI68235, SGI68236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68237, SGI0, SGI0, SGI0, SGI68238, SGI68238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15106[] = {SGI68239, SGI68240, SGI68241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68242, SGI0, SGI0, SGI0, SGI68243, SGI68243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15107[] = {SGI68244, SGI68245, SGI68246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68247, SGI0, SGI0, SGI0, SGI68248, SGI68248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15108[] = {SGI68249, SGI68250, SGI68251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68252, SGI0, SGI0, SGI0, SGI68253, SGI68253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15109[] = {SGI68254, SGI68255, SGI68256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68257, SGI68257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15110[] = {SGI68258, SGI68259, SGI68260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68261, SGI0, SGI0, SGI0, SGI68262, SGI68262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15111[] = {SGI68263, SGI68264, SGI68265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68266, SGI0, SGI0, SGI0, SGI68267, SGI68267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15112[] = {SGI68268, SGI68269, SGI68270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68271, SGI0, SGI0, SGI0, SGI68272, SGI68272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15113[] = {SGI68273, SGI68274, SGI68275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68276, SGI0, SGI0, SGI0, SGI68277, SGI68277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15114[] = {SGI68278, SGI68279, SGI68280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68281, SGI0, SGI0, SGI0, SGI68282, SGI68282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15115[] = {SGI68283, SGI68284, SGI68285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68286, SGI0, SGI0, SGI0, SGI68287, SGI68287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15116[] = {SGI68288, SGI68289, SGI68290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68291, SGI0, SGI0, SGI0, SGI68292, SGI68292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15117[] = {SGI68293, SGI68294, SGI68295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68296, SGI0, SGI0, SGI0, SGI68297, SGI68297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15118[] = {SGI68298, SGI68299, SGI68300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68301, SGI0, SGI0, SGI0, SGI68302, SGI68302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15119[] = {SGI68303, SGI0, SGI68304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15120[] = {SGI68305, SGI68306, SGI68307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68308, SGI0, SGI0, SGI0, SGI68309, SGI68309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15121[] = {SGI68310, SGI68311, SGI68312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68313, SGI0, SGI0, SGI0, SGI68314, SGI68314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15122[] = {SGI68315, SGI68316, SGI68317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68318, SGI0, SGI0, SGI0, SGI68319, SGI68319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15123[] = {SGI68320, SGI68321, SGI68322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68323, SGI0, SGI0, SGI0, SGI68324, SGI68324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15124[] = {SGI68325, SGI68326, SGI68327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68328, SGI0, SGI0, SGI0, SGI68329, SGI68329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15125[] = {SGI68330, SGI68331, SGI68332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68333, SGI0, SGI0, SGI0, SGI68334, SGI68334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15126[] = {SGI68335, SGI68336, SGI68337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68338, SGI0, SGI0, SGI0, SGI68339, SGI68339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15127[] = {SGI68340, SGI68341, SGI68342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68343, SGI0, SGI0, SGI0, SGI68344, SGI68344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15128[] = {SGI68345, SGI68346, SGI68347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68348, SGI0, SGI0, SGI0, SGI68349, SGI68349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15129[] = {SGI68350, SGI68351, SGI68352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68353, SGI0, SGI0, SGI0, SGI68354, SGI68354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15130[] = {SGI68355, SGI68356, SGI68357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68358, SGI0, SGI0, SGI0, SGI68359, SGI68359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15131[] = {SGI68360, SGI68361, SGI68362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68363, SGI0, SGI0, SGI0, SGI68364, SGI68364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15132[] = {SGI68365, SGI68366, SGI68367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68368, SGI0, SGI0, SGI0, SGI68369, SGI68369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15133[] = {SGI68370, SGI68371, SGI68372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68373, SGI68374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15134[] = {SGI68375, SGI68376, SGI68377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68378, SGI0, SGI0, SGI0, SGI68379, SGI68379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15135[] = {SGI68380, SGI68381, SGI68382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68383, SGI0, SGI0, SGI0, SGI68384, SGI68384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15136[] = {SGI68385, SGI68386, SGI68387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68388, SGI0, SGI0, SGI0, SGI68389, SGI68389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15137[] = {SGI68390, SGI68391, SGI68392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68393, SGI0, SGI0, SGI0, SGI68394, SGI68394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15138[] = {SGI68395, SGI68396, SGI68397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68398, SGI0, SGI0, SGI0, SGI68399, SGI68399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15139[] = {SGI68400, SGI68401, SGI68402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68403, SGI0, SGI0, SGI0, SGI68404, SGI68404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15140[] = {SGI68405, SGI68406, SGI68407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68408, SGI0, SGI0, SGI0, SGI68409, SGI68409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15141[] = {SGI68410, SGI68411, SGI68412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68413, SGI0, SGI0, SGI0, SGI68414, SGI68414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15142[] = {SGI68415, SGI68416, SGI68417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68418, SGI0, SGI0, SGI0, SGI68419, SGI68419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15143[] = {SGI68420, SGI68421, SGI68422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68423, SGI0, SGI0, SGI0, SGI68424, SGI68424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15144[] = {SGI68425, SGI68426, SGI68427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68428, SGI68428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15145[] = {SGI68429, SGI68430, SGI68431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68432, SGI0, SGI0, SGI0, SGI68433, SGI68433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15146[] = {SGI68434, SGI68435, SGI68436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68437, SGI0, SGI0, SGI0, SGI68438, SGI68438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15147[] = {SGI68439, SGI68440, SGI68441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68442, SGI68443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15148[] = {SGI68444, SGI68445, SGI68446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68447, SGI0, SGI0, SGI0, SGI68448, SGI68448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15149[] = {SGI68449, SGI68450, SGI68451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68452, SGI0, SGI0, SGI0, SGI68453, SGI68453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15150[] = {SGI68454, SGI68455, SGI68456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68457, SGI0, SGI0, SGI0, SGI68458, SGI68458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15151[] = {SGI68459, SGI68460, SGI68461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68462, SGI0, SGI0, SGI0, SGI68463, SGI68463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15152[] = {SGI68464, SGI68465, SGI68466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68467, SGI0, SGI0, SGI0, SGI68468, SGI68468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15153[] = {SGI68469, SGI68470, SGI68471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68472, SGI0, SGI0, SGI0, SGI68473, SGI68473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15154[] = {SGI68474, SGI68475, SGI68476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68477, SGI0, SGI0, SGI0, SGI68478, SGI68478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15155[] = {SGI68479, SGI0, SGI68480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15156[] = {SGI68481, SGI0, SGI68482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15157[] = {SGI68483, SGI68484, SGI68485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68486, SGI0, SGI0, SGI0, SGI68487, SGI68488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15158[] = {SGI68489, SGI68490, SGI68491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68492, SGI0, SGI0, SGI0, SGI68493, SGI68493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15159[] = {SGI68494, SGI0, SGI68495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68496, SGI0, SGI0, SGI0, SGI68497, SGI68497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15160[] = {SGI68498, SGI68499, SGI68500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68501, SGI0, SGI0, SGI0, SGI68502, SGI68502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15161[] = {SGI68503, SGI0, SGI68504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15162[] = {SGI68505, SGI68506, SGI68507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68508, SGI0, SGI0, SGI0, SGI68509, SGI68509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15163[] = {SGI68510, SGI0, SGI68511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68512, SGI0, SGI0, SGI0, SGI68513, SGI68513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15164[] = {SGI68514, SGI68515, SGI68516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68517, SGI0, SGI0, SGI0, SGI68518, SGI68518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15165[] = {SGI68519, SGI68520, SGI68521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68522, SGI0, SGI0, SGI0, SGI68523, SGI68523, SGI0, SGI0, NULL};
const Segment * const SG15166[] = {SGI68524, SGI0, SGI68525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15167[] = {SGI68526, SGI68527, SGI68528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68529, SGI0, SGI0, SGI0, SGI68530, SGI68530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15168[] = {SGI68531, SGI68532, SGI68533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68534, SGI0, SGI0, SGI0, SGI68535, SGI68535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15169[] = {SGI68536, SGI68537, SGI68538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68539, SGI0, SGI0, SGI0, SGI68540, SGI68540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15170[] = {SGI68541, SGI68542, SGI68543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68544, SGI0, SGI0, SGI0, SGI68545, SGI68545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15171[] = {SGI68546, SGI68547, SGI68548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68549, SGI0, SGI0, SGI0, SGI68550, SGI68550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15172[] = {SGI68551, SGI68552, SGI68553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68554, SGI0, SGI0, SGI0, SGI68555, SGI68555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15173[] = {SGI0, SGI0, SGI68556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15174[] = {SGI68560, SGI0, SGI68561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15175[] = {SGI68557, SGI68558, SGI68559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68562, SGI0, SGI0, SGI0, SGI68563, SGI68563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15176[] = {SGI68564, SGI68566, SGI68567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68570, SGI0, SGI0, SGI0, SGI68572, SGI68572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15177[] = {SGI68565, SGI68568, SGI68569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68571, SGI0, SGI0, SGI0, SGI68573, SGI68573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15178[] = {SGI68574, SGI68575, SGI68576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68577, SGI0, SGI0, SGI0, SGI68578, SGI68578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15179[] = {SGI68579, SGI68580, SGI68581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68583, SGI68584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15180[] = {SGI68582, SGI68585, SGI68586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68587, SGI0, SGI0, SGI0, SGI68588, SGI68588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15181[] = {SGI68589, SGI68590, SGI68592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68595, SGI0, SGI0, SGI0, SGI68596, SGI68596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15182[] = {SGI68591, SGI68593, SGI68594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68597, SGI68597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15183[] = {SGI68598, SGI68599, SGI68601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68603, SGI0, SGI0, SGI0, SGI68605, SGI68605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15184[] = {SGI68600, SGI0, SGI68602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68604, SGI0, SGI0, SGI0, SGI68606, SGI68606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15185[] = {SGI68607, SGI68608, SGI68609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68610, SGI68611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15186[] = {SGI68612, SGI68613, SGI68614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68615, SGI0, SGI0, SGI0, SGI68616, SGI68616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15187[] = {SGI68617, SGI68618, SGI68619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68620, SGI0, SGI0, SGI0, SGI68621, SGI68621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15188[] = {SGI68622, SGI68623, SGI68624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68625, SGI0, SGI0, SGI0, SGI68626, SGI68626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15189[] = {SGI68627, SGI68628, SGI68629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68630, SGI0, SGI0, SGI0, SGI68631, SGI68631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15190[] = {SGI68632, SGI68633, SGI68634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68635, SGI0, SGI0, SGI0, SGI68636, SGI68636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15191[] = {SGI68637, SGI68638, SGI68639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68640, SGI0, SGI0, SGI0, SGI68641, SGI68641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15192[] = {SGI68642, SGI68643, SGI68644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68645, SGI0, SGI0, SGI0, SGI68646, SGI68646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15193[] = {SGI68647, SGI68648, SGI68649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68650, SGI0, SGI0, SGI0, SGI68651, SGI68651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15194[] = {SGI68652, SGI68653, SGI68654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68655, SGI0, SGI0, SGI0, SGI68656, SGI68656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15195[] = {SGI68657, SGI68659, SGI68661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68663, SGI0, SGI0, SGI0, SGI68665, SGI68665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15196[] = {SGI68658, SGI68660, SGI68662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68664, SGI0, SGI0, SGI0, SGI68666, SGI68666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15197[] = {SGI68668, SGI0, SGI68670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15198[] = {SGI68667, SGI68669, SGI68671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68672, SGI0, SGI0, SGI0, SGI68673, SGI68673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15199[] = {SGI68674, SGI68675, SGI68676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68677, SGI0, SGI0, SGI0, SGI68678, SGI68678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15200[] = {SGI68679, SGI68680, SGI68681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68682, SGI0, SGI0, SGI0, SGI68683, SGI68683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15201[] = {SGI68684, SGI68685, SGI68686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68687, SGI0, SGI0, SGI0, SGI68688, SGI68688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15202[] = {SGI68689, SGI68690, SGI68691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68692, SGI0, SGI0, SGI0, SGI68693, SGI68693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15203[] = {SGI68694, SGI68695, SGI68696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68697, SGI68697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15204[] = {SGI68698, SGI68699, SGI68700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68701, SGI0, SGI0, SGI0, SGI68702, SGI68702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15205[] = {SGI68703, SGI68704, SGI68705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68706, SGI68707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15206[] = {SGI68708, SGI68709, SGI68710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68711, SGI0, SGI0, SGI0, SGI68712, SGI68712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15207[] = {SGI68713, SGI0, SGI68714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68715, SGI0, SGI0, SGI0, SGI68716, SGI68716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15208[] = {SGI68717, SGI68718, SGI68719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68720, SGI0, SGI0, SGI0, SGI68721, SGI68721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15209[] = {SGI68722, SGI0, SGI68723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68724, SGI0, SGI0, SGI0, SGI68725, SGI68725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15210[] = {SGI68726, SGI68727, SGI68728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68729, SGI0, SGI0, SGI0, SGI68730, SGI68730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15211[] = {SGI68731, SGI68732, SGI68733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68734, SGI0, SGI0, SGI0, SGI68735, SGI68735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15212[] = {SGI68736, SGI68737, SGI68738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68739, SGI0, SGI0, SGI0, SGI68740, SGI68740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15213[] = {SGI68741, SGI68742, SGI68743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68744, SGI0, SGI0, SGI0, SGI68745, SGI68745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15214[] = {SGI68746, SGI68747, SGI68748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68749, SGI68749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15215[] = {SGI68750, SGI68751, SGI68752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68753, SGI0, SGI0, SGI0, SGI68754, SGI68754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15216[] = {SGI68755, SGI68756, SGI68757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68758, SGI0, SGI0, SGI0, SGI68759, SGI68759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15217[] = {SGI68760, SGI68761, SGI68762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68763, SGI0, SGI0, SGI0, SGI68764, SGI68764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15218[] = {SGI68765, SGI68766, SGI68767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68768, SGI0, SGI0, SGI0, SGI68769, SGI68769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15219[] = {SGI68770, SGI68771, SGI68772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68773, SGI0, SGI0, SGI0, SGI68774, SGI68774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15220[] = {SGI68775, SGI68776, SGI68777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68778, SGI0, SGI0, SGI0, SGI68779, SGI68779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15221[] = {SGI68780, SGI68781, SGI68782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68783, SGI0, SGI0, SGI0, SGI68784, SGI68784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15222[] = {SGI68785, SGI68786, SGI68787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68788, SGI0, SGI0, SGI0, SGI68789, SGI68789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15223[] = {SGI68790, SGI68791, SGI68792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68793, SGI0, SGI0, SGI0, SGI68794, SGI68794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15224[] = {SGI68795, SGI68796, SGI68797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68798, SGI0, SGI0, SGI0, SGI68799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15225[] = {SGI68800, SGI0, SGI68801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15226[] = {SGI68802, SGI68803, SGI68804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68805, SGI0, SGI0, SGI0, SGI68806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15227[] = {SGI68807, SGI0, SGI68808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68809, SGI0, SGI0, SGI0, SGI68810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15228[] = {SGI68811, SGI68812, SGI68813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68814, SGI0, SGI0, SGI0, SGI68815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15229[] = {SGI68816, SGI68817, SGI68818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15230[] = {SGI68819, SGI68820, SGI68821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68822, SGI68822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15231[] = {SGI68823, SGI68824, SGI68825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15232[] = {SGI68826, SGI68827, SGI68828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68829, SGI0, SGI0, SGI0, SGI68830, SGI68831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15233[] = {SGI0, SGI68832, SGI68833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15234[] = {SGI68834, SGI68835, SGI68836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15235[] = {SGI68837, SGI0, SGI68838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68839, SGI0, SGI0, SGI0, SGI68840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15236[] = {SGI68841, SGI68842, SGI68843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15237[] = {SGI68844, SGI68845, SGI68846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68847, SGI0, SGI0, SGI0, SGI68848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15238[] = {SGI68849, SGI68850, SGI68851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15239[] = {SGI68852, SGI68853, SGI68854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68855, SGI0, SGI0, SGI0, SGI68856, SGI68856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15240[] = {SGI68857, SGI0, SGI68858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15241[] = {SGI68859, SGI68860, SGI68861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68862, SGI0, SGI0, SGI0, SGI68863, SGI68864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15242[] = {SGI68865, SGI68866, SGI68867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15243[] = {SGI68868, SGI68869, SGI68870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15244[] = {SGI68871, SGI68872, SGI68873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68874, SGI0, SGI0, SGI0, SGI68875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15245[] = {SGI68876, SGI68877, SGI68878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15246[] = {SGI68879, SGI68880, SGI68881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68882, SGI0, SGI0, SGI0, SGI0, SGI68883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15247[] = {SGI68884, SGI68885, SGI68886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68887, SGI0, SGI0, SGI0, SGI68888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15248[] = {SGI68889, SGI68890, SGI68891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15249[] = {SGI68892, SGI68893, SGI68894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68895, SGI0, SGI0, SGI0, SGI68896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15250[] = {SGI68897, SGI68898, SGI68899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68900, SGI0, SGI0, SGI0, SGI68901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15251[] = {SGI68902, SGI68903, SGI68904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15252[] = {SGI68905, SGI0, SGI68906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68907, SGI0, SGI0, SGI0, SGI68908, SGI68909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15253[] = {SGI0, SGI68910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15254[] = {SGI68911, SGI68912, SGI68913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15255[] = {SGI68914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15256[] = {SGI0, SGI0, SGI68915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15257[] = {SGI0, SGI68916, SGI68917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15258[] = {SGI68918, SGI68919, SGI68920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68921, SGI0, SGI0, SGI0, SGI68922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15259[] = {SGI68923, SGI68924, SGI68925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68926, SGI68926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15260[] = {SGI68927, SGI68928, SGI68929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15261[] = {SGI68930, SGI68931, SGI68932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68933, SGI0, SGI0, SGI0, SGI68934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15262[] = {SGI68935, SGI68936, SGI68937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68938, SGI0, SGI0, SGI0, SGI68939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15263[] = {SGI68940, SGI68941, SGI68942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68943, SGI0, SGI0, SGI0, SGI68944, SGI68945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15264[] = {SGI68946, SGI68947, SGI68948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15265[] = {SGI68950, SGI68951, SGI68952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68953, SGI0, SGI0, SGI0, SGI68954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15266[] = {SGI68955, SGI68956, SGI68957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68958, SGI0, SGI0, SGI0, SGI68959, SGI68959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15267[] = {SGI68960, SGI68961, SGI68962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68963, SGI0, SGI0, SGI0, SGI68964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15268[] = {SGI68965, SGI68966, SGI68967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68968, SGI68969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15269[] = {SGI68970, SGI68971, SGI68972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68973, SGI0, SGI0, SGI0, SGI68974, SGI68974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15270[] = {SGI68975, SGI68976, SGI68977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68978, SGI0, SGI0, SGI0, SGI68979, SGI68979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15271[] = {SGI68980, SGI0, SGI68981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15272[] = {SGI68982, SGI68983, SGI68984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68985, SGI0, SGI0, SGI0, SGI68986, SGI68986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15273[] = {SGI68987, SGI68988, SGI68989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68990, SGI0, SGI0, SGI0, SGI68991, SGI68991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15274[] = {SGI68992, SGI68993, SGI68994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI68995, SGI0, SGI0, SGI0, SGI68996, SGI68996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15275[] = {SGI68997, SGI68998, SGI68999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69000, SGI0, SGI0, SGI0, SGI69001, SGI69001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15276[] = {SGI69002, SGI69003, SGI69004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69005, SGI0, SGI0, SGI0, SGI69006, SGI69006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15277[] = {SGI69007, SGI69008, SGI69009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69010, SGI0, SGI0, SGI0, SGI69011, SGI69011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15278[] = {SGI69012, SGI69013, SGI69014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69015, SGI69015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15279[] = {SGI69016, SGI69017, SGI69018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69019, SGI0, SGI0, SGI0, SGI69020, SGI69020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15280[] = {SGI69021, SGI0, SGI69022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15281[] = {SGI69023, SGI69024, SGI69025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69026, SGI0, SGI0, SGI0, SGI69027, SGI69027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15282[] = {SGI69028, SGI69029, SGI69030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69031, SGI0, SGI0, SGI0, SGI69032, SGI69032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15283[] = {SGI69033, SGI69034, SGI69035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69036, SGI0, SGI0, SGI0, SGI69037, SGI69037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15284[] = {SGI69038, SGI69039, SGI69040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69041, SGI0, SGI0, SGI0, SGI69042, SGI69042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15285[] = {SGI69043, SGI0, SGI69044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69045, SGI0, SGI0, SGI0, SGI69046, SGI69046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15286[] = {SGI69047, SGI69048, SGI69049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69050, SGI0, SGI0, SGI0, SGI69051, SGI69051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15287[] = {SGI69052, SGI69053, SGI69054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69055, SGI69055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15288[] = {SGI69056, SGI69057, SGI69058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69059, SGI0, SGI0, SGI0, SGI69060, SGI69061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15289[] = {SGI69062, SGI69063, SGI69064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69065, SGI0, SGI0, SGI0, SGI69066, SGI69066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15290[] = {SGI69067, SGI69068, SGI69069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69070, SGI0, SGI0, SGI0, SGI69071, SGI69071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15291[] = {SGI69072, SGI69073, SGI69074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15292[] = {SGI69075, SGI69076, SGI69077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69078, SGI0, SGI0, SGI0, SGI69079, SGI69079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15293[] = {SGI69080, SGI0, SGI69081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69082, SGI0, SGI0, SGI0, SGI69083, SGI69083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15294[] = {SGI69084, SGI69085, SGI69086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69087, SGI0, SGI0, SGI0, SGI69088, SGI69088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15295[] = {SGI69089, SGI69090, SGI69091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69092, SGI0, SGI0, SGI0, SGI69093, SGI69093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15296[] = {SGI69094, SGI69095, SGI69096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69097, SGI0, SGI0, SGI0, SGI69098, SGI69098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15297[] = {SGI69099, SGI0, SGI69100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15298[] = {SGI69101, SGI69102, SGI69103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69104, SGI0, SGI0, SGI0, SGI69105, SGI69105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15299[] = {SGI69106, SGI69107, SGI69108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69109, SGI0, SGI0, SGI0, SGI69110, SGI69110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15300[] = {SGI69111, SGI69112, SGI69113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69114, SGI0, SGI0, SGI0, SGI69115, SGI69115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15301[] = {SGI69116, SGI69117, SGI69118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69119, SGI0, SGI0, SGI0, SGI69120, SGI69120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15302[] = {SGI69121, SGI69122, SGI69123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69124, SGI0, SGI0, SGI0, SGI69125, SGI69125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15303[] = {SGI69126, SGI69127, SGI69128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69129, SGI0, SGI0, SGI0, SGI69130, SGI69130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15304[] = {SGI69131, SGI69132, SGI69133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69134, SGI0, SGI0, SGI0, SGI69135, SGI69135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15305[] = {SGI69136, SGI69137, SGI69138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69139, SGI0, SGI0, SGI0, SGI69140, SGI69140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15306[] = {SGI69141, SGI69142, SGI69143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69144, SGI0, SGI0, SGI0, SGI69145, SGI69145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15307[] = {SGI69146, SGI69147, SGI69148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69149, SGI0, SGI0, SGI0, SGI69150, SGI69150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15308[] = {SGI69151, SGI69152, SGI69153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69154, SGI0, SGI0, SGI0, SGI69155, SGI69155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15309[] = {SGI69156, SGI69157, SGI69158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69159, SGI0, SGI0, SGI0, SGI69160, SGI69160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15310[] = {SGI69161, SGI69162, SGI69163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69164, SGI0, SGI0, SGI0, SGI69165, SGI69165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15311[] = {SGI69166, SGI69167, SGI69168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69169, SGI0, SGI0, SGI0, SGI69170, SGI69170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15312[] = {SGI69171, SGI69172, SGI69173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69174, SGI0, SGI0, SGI0, SGI69175, SGI69175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15313[] = {SGI69176, SGI69177, SGI69178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69179, SGI0, SGI0, SGI0, SGI69180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15314[] = {SGI69181, SGI69182, SGI69183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69184, SGI0, SGI0, SGI0, SGI69185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15315[] = {SGI69186, SGI69187, SGI69188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69189, SGI0, SGI0, SGI0, SGI69190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15316[] = {SGI69191, SGI69192, SGI69193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69194, SGI0, SGI0, SGI0, SGI69195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15317[] = {SGI69196, SGI69197, SGI69198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69199, SGI0, SGI0, SGI0, SGI69200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15318[] = {SGI69201, SGI69202, SGI69203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69204, SGI0, SGI0, SGI0, SGI69205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15319[] = {SGI69206, SGI69207, SGI69208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69209, SGI0, SGI0, SGI0, SGI69210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15320[] = {SGI69211, SGI69212, SGI69213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69214, SGI0, SGI0, SGI0, SGI69215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15321[] = {SGI69216, SGI0, SGI69217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15322[] = {SGI69218, SGI69219, SGI69220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69221, SGI0, SGI0, SGI0, SGI69222, SGI69222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15323[] = {SGI69223, SGI0, SGI69224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15324[] = {SGI69225, SGI0, SGI69226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15325[] = {SGI69227, SGI69228, SGI69229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69230, SGI0, SGI0, SGI0, SGI69231, SGI69232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15326[] = {SGI69233, SGI0, SGI69234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69235, SGI0, SGI0, SGI0, SGI69236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15327[] = {SGI69237, SGI69238, SGI69239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69240, SGI0, SGI0, SGI0, SGI69241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15328[] = {SGI69242, SGI0, SGI69243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69244, SGI0, SGI0, SGI0, SGI69245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15329[] = {SGI69246, SGI0, SGI69247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15330[] = {SGI69248, SGI69249, SGI69250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69251, SGI0, SGI0, SGI0, SGI69252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15331[] = {SGI69253, SGI69254, SGI69255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69256, SGI0, SGI0, SGI0, SGI69257, SGI69257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15332[] = {SGI69258, SGI69259, SGI69260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69261, SGI69262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15333[] = {SGI69263, SGI69264, SGI69265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69266, SGI0, SGI0, SGI0, SGI69267, SGI69268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15334[] = {SGI69269, SGI69270, SGI69271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69272, SGI0, SGI0, SGI0, SGI69273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15335[] = {SGI0, SGI69274, SGI69275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15336[] = {SGI69276, SGI69277, SGI69278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69279, SGI69279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15337[] = {SGI0, SGI69280, SGI69281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15338[] = {SGI69282, SGI69283, SGI69284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15339[] = {SGI69285, SGI69286, SGI69287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15340[] = {SGI69288, SGI69289, SGI69290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69291, SGI0, SGI0, SGI0, SGI69292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15341[] = {SGI69293, SGI69294, SGI69295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69296, SGI0, SGI0, SGI0, SGI69297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15342[] = {SGI69298, SGI69299, SGI69300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69301, SGI0, SGI0, SGI0, SGI69302, SGI69302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15343[] = {SGI69303, SGI69304, SGI69305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15344[] = {SGI69307, SGI0, SGI69308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15345[] = {SGI0, SGI69309, SGI69310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69311, SGI0, SGI0, SGI0, SGI69312, SGI69313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15346[] = {SGI69314, SGI69315, SGI69316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69317, SGI0, SGI0, SGI0, SGI69318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15347[] = {SGI69319, SGI69320, SGI69321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15348[] = {SGI0, SGI69322, SGI69323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15349[] = {SGI69324, SGI69325, SGI69326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69329, SGI0, SGI0, SGI0, SGI69331, SGI69331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15350[] = {SGI69327, SGI69328, SGI69330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69332, SGI0, SGI0, SGI0, SGI69333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15351[] = {SGI69334, SGI69335, SGI69336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15352[] = {SGI69337, SGI69338, SGI69339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15353[] = {SGI69340, SGI69341, SGI69342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15354[] = {SGI0, SGI69343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15355[] = {SGI69344, SGI69345, SGI69346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69347, SGI0, SGI0, SGI0, SGI69348, SGI69348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15356[] = {SGI69349, SGI69350, SGI69351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15357[] = {SGI69352, SGI69353, SGI69354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69355, SGI0, SGI0, SGI0, SGI69356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15358[] = {SGI69357, SGI69358, SGI69359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15359[] = {SGI69360, SGI69361, SGI69362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69363, SGI0, SGI0, SGI0, SGI69364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15360[] = {SGI69365, SGI69366, SGI69367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15361[] = {SGI69368, SGI69369, SGI69370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69371, SGI0, SGI0, SGI0, SGI69372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15362[] = {SGI69373, SGI69374, SGI69375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69376, SGI0, SGI0, SGI0, SGI69377, SGI69378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15363[] = {SGI69379, SGI69380, SGI69381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69382, SGI0, SGI0, SGI0, SGI69383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15364[] = {SGI69384, SGI69385, SGI69386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69387, SGI0, SGI0, SGI0, SGI69388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15365[] = {SGI69389, SGI69390, SGI69391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15366[] = {SGI69392, SGI69393, SGI69394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69395, SGI0, SGI0, SGI0, SGI69396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15367[] = {SGI69397, SGI69398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15368[] = {SGI69399, SGI69400, SGI69401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69402, SGI0, SGI0, SGI0, SGI69403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15369[] = {SGI69404, SGI69405, SGI69406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69407, SGI0, SGI0, SGI0, SGI69408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15370[] = {SGI69409, SGI0, SGI69410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15371[] = {SGI69411, SGI69412, SGI69413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15372[] = {SGI69415, SGI69416, SGI69417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69418, SGI0, SGI0, SGI0, SGI69419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15373[] = {SGI69420, SGI69421, SGI69422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69423, SGI0, SGI0, SGI0, SGI69424, SGI69425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15374[] = {SGI69426, SGI69427, SGI69428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69429, SGI0, SGI0, SGI0, SGI69430, SGI69431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15375[] = {SGI69432, SGI69433, SGI69434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69435, SGI0, SGI0, SGI0, SGI69436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15376[] = {SGI0, SGI69437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15377[] = {SGI69438, SGI69439, SGI69440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69441, SGI0, SGI0, SGI0, SGI69442, SGI69442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15378[] = {SGI69443, SGI0, SGI69444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15379[] = {SGI69445, SGI69446, SGI69447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69448, SGI0, SGI0, SGI0, SGI69449, SGI69449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15380[] = {SGI69450, SGI0, SGI69451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69452, SGI0, SGI0, SGI0, SGI69453, SGI69453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15381[] = {SGI69454, SGI69455, SGI69456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69457, SGI0, SGI0, SGI0, SGI69458, SGI69458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15382[] = {SGI69459, SGI69460, SGI69461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69462, SGI0, SGI0, SGI0, SGI69463, SGI69463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15383[] = {SGI69464, SGI0, SGI69465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15384[] = {SGI69466, SGI69467, SGI69468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69469, SGI0, SGI0, SGI0, SGI69470, SGI69470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15385[] = {SGI69471, SGI69472, SGI69473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69474, SGI0, SGI0, SGI0, SGI69475, SGI69475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15386[] = {SGI69476, SGI69477, SGI69478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69479, SGI69479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15387[] = {SGI69480, SGI69481, SGI69482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69483, SGI69483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15388[] = {SGI69484, SGI69485, SGI69486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69487, SGI69488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15389[] = {SGI69489, SGI69490, SGI69491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69492, SGI0, SGI0, SGI0, SGI69493, SGI69493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15390[] = {SGI69494, SGI69495, SGI69496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69497, SGI0, SGI0, SGI0, SGI69498, SGI69498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15391[] = {SGI69499, SGI0, SGI69500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15392[] = {SGI69501, SGI69502, SGI69503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69504, SGI0, SGI0, SGI0, SGI69505, SGI69505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15393[] = {SGI69506, SGI69507, SGI69508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69509, SGI0, SGI0, SGI0, SGI69510, SGI69510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15394[] = {SGI69511, SGI69512, SGI69513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69514, SGI0, SGI0, SGI0, SGI69515, SGI69515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15395[] = {SGI69516, SGI69517, SGI69518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69519, SGI69519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15396[] = {SGI69520, SGI69521, SGI69522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69523, SGI0, SGI0, SGI0, SGI69524, SGI69524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15397[] = {SGI69525, SGI69526, SGI69527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69528, SGI69529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15398[] = {SGI69530, SGI69531, SGI69532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69533, SGI0, SGI0, SGI0, SGI69534, SGI69534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15399[] = {SGI69535, SGI0, SGI69536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15400[] = {SGI69537, SGI69538, SGI69539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69540, SGI0, SGI0, SGI0, SGI69541, SGI69541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15401[] = {SGI69542, SGI0, SGI69543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69544, SGI0, SGI0, SGI0, SGI69545, SGI69545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15402[] = {SGI69546, SGI69547, SGI69548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69549, SGI0, SGI0, SGI0, SGI69550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15403[] = {SGI69551, SGI69552, SGI69553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69554, SGI69555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15404[] = {SGI69556, SGI69557, SGI69558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69559, SGI0, SGI0, SGI0, SGI69560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15405[] = {SGI69561, SGI0, SGI69562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15406[] = {SGI69563, SGI69564, SGI69565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69566, SGI0, SGI0, SGI0, SGI69567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15407[] = {SGI69568, SGI69569, SGI69570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69571, SGI0, SGI0, SGI0, SGI69572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15408[] = {SGI69573, SGI69574, SGI69575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69576, SGI0, SGI0, SGI0, SGI69577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15409[] = {SGI69578, SGI0, SGI69579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15410[] = {SGI69580, SGI69581, SGI69582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69583, SGI0, SGI0, SGI0, SGI69584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15411[] = {SGI69585, SGI69586, SGI69587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69588, SGI0, SGI0, SGI0, SGI69589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15412[] = {SGI69590, SGI69591, SGI69592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69593, SGI0, SGI0, SGI0, SGI69594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15413[] = {SGI69595, SGI0, SGI69596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69597, SGI0, SGI0, SGI0, SGI69598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15414[] = {SGI69599, SGI69600, SGI69601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69602, SGI0, SGI0, SGI0, SGI69603, SGI69603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15415[] = {SGI69604, SGI69605, SGI69606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69607, SGI0, SGI0, SGI0, SGI69608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15416[] = {SGI69609, SGI69610, SGI69611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69612, SGI0, SGI0, SGI0, SGI69613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15417[] = {SGI69614, SGI69615, SGI69616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69617, SGI0, SGI0, SGI0, SGI69618, SGI69618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15418[] = {SGI69619, SGI69620, SGI69621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15419[] = {SGI69622, SGI69623, SGI69624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69625, SGI0, SGI0, SGI0, SGI69626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15420[] = {SGI69627, SGI0, SGI69628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15421[] = {SGI69629, SGI69630, SGI69631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69632, SGI69633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15422[] = {SGI69634, SGI69635, SGI69636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69637, SGI0, SGI0, SGI0, SGI69638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15423[] = {SGI69639, SGI69640, SGI69641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69642, SGI69642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15424[] = {SGI0, SGI69643, SGI69644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15425[] = {SGI69645, SGI69646, SGI69647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69648, SGI0, SGI0, SGI0, SGI69649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15426[] = {SGI69650, SGI0, SGI69651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15427[] = {SGI69652, SGI0, SGI69653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69654, SGI0, SGI0, SGI0, SGI69655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15428[] = {SGI69656, SGI69657, SGI69658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69659, SGI0, SGI0, SGI0, SGI69660, SGI69660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15429[] = {SGI69661, SGI69662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15430[] = {SGI69663, SGI69664, SGI69665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15431[] = {SGI0, SGI69666, SGI69667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15432[] = {SGI69668, SGI0, SGI69669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69670, SGI0, SGI0, SGI0, SGI69671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15433[] = {SGI69672, SGI0, SGI69673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69674, SGI0, SGI0, SGI0, SGI69675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15434[] = {SGI69676, SGI0, SGI69677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69678, SGI0, SGI0, SGI0, SGI69679, SGI69679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15435[] = {SGI69680, SGI69681, SGI69682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15436[] = {SGI69683, SGI69684, SGI69685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69686, SGI0, SGI0, SGI0, SGI69687, SGI69687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15437[] = {SGI69688, SGI69689, SGI69690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15438[] = {SGI69691, SGI69692, SGI69693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69694, SGI0, SGI0, SGI0, SGI69695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15439[] = {SGI69696, SGI69697, SGI69698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69699, SGI0, SGI0, SGI0, SGI69700, SGI69701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15440[] = {SGI69702, SGI69703, SGI69704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15441[] = {SGI69705, SGI69706, SGI69707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15442[] = {SGI69708, SGI69709, SGI69710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69711, SGI0, SGI0, SGI0, SGI69712, SGI69713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15443[] = {SGI69714, SGI69715, SGI69716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69717, SGI0, SGI0, SGI0, SGI69718, SGI69718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15444[] = {SGI69719, SGI69720, SGI69721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69722, SGI0, SGI0, SGI0, SGI69723, SGI69723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15445[] = {SGI69724, SGI69725, SGI69726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69727, SGI0, SGI0, SGI0, SGI69728, SGI69728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15446[] = {SGI69729, SGI69730, SGI69731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69732, SGI0, SGI0, SGI0, SGI69733, SGI69733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15447[] = {SGI69734, SGI69735, SGI69736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69737, SGI0, SGI0, SGI0, SGI69738, SGI69738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15448[] = {SGI69739, SGI69740, SGI69741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69742, SGI0, SGI0, SGI0, SGI69743, SGI69743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15449[] = {SGI69744, SGI69745, SGI69746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69747, SGI0, SGI0, SGI0, SGI69748, SGI69748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15450[] = {SGI69749, SGI0, SGI69750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15451[] = {SGI69751, SGI69752, SGI69753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69754, SGI0, SGI0, SGI0, SGI69755, SGI69755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15452[] = {SGI69756, SGI69757, SGI69758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69759, SGI69760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15453[] = {SGI69761, SGI69762, SGI69763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69764, SGI0, SGI0, SGI0, SGI69765, SGI69765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15454[] = {SGI69766, SGI69767, SGI69768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69769, SGI0, SGI0, SGI0, SGI69770, SGI69770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15455[] = {SGI69771, SGI69772, SGI69773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69774, SGI0, SGI0, SGI0, SGI69775, SGI69775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15456[] = {SGI69776, SGI69777, SGI69778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69779, SGI0, SGI0, SGI0, SGI69780, SGI69780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15457[] = {SGI69781, SGI69782, SGI69783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69784, SGI0, SGI0, SGI0, SGI69785, SGI69785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15458[] = {SGI69786, SGI69787, SGI69788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69789, SGI0, SGI0, SGI0, SGI69790, SGI69790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15459[] = {SGI69791, SGI69792, SGI69793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69794, SGI0, SGI0, SGI0, SGI69795, SGI69795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15460[] = {SGI69796, SGI69797, SGI69798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69799, SGI69799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15461[] = {SGI69800, SGI69801, SGI69802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69803, SGI69803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15462[] = {SGI69804, SGI69805, SGI69806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69807, SGI0, SGI0, SGI0, SGI69808, SGI69808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15463[] = {SGI69809, SGI69810, SGI69811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69812, SGI69812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15464[] = {SGI69813, SGI69814, SGI69815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69816, SGI0, SGI0, SGI0, SGI69817, SGI69817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15465[] = {SGI69818, SGI69819, SGI69820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69821, SGI69821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15466[] = {SGI69824, SGI69826, SGI69827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69828, SGI69830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15467[] = {SGI69822, SGI69823, SGI69825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69829, SGI0, SGI0, SGI0, SGI69831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15468[] = {SGI69832, SGI69833, SGI69834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69837, SGI69838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15469[] = {SGI69835, SGI0, SGI69836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15470[] = {SGI69839, SGI69840, SGI69841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69842, SGI69842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15471[] = {SGI69843, SGI69844, SGI69845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69846, SGI0, SGI0, SGI0, SGI69847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15472[] = {SGI69848, SGI69849, SGI69850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69851, SGI0, SGI0, SGI0, SGI69852, SGI69852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15473[] = {SGI69853, SGI69854, SGI69855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69856, SGI0, SGI0, SGI0, SGI69857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15474[] = {SGI69858, SGI69859, SGI69860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15475[] = {SGI69861, SGI69862, SGI69863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69864, SGI0, SGI0, SGI0, SGI69865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15476[] = {SGI69866, SGI69867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15477[] = {SGI69868, SGI69869, SGI69870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69871, SGI0, SGI0, SGI0, SGI69872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15478[] = {SGI0, SGI69873, SGI69874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69875, SGI0, SGI0, SGI0, SGI69876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15479[] = {SGI69877, SGI69878, SGI69879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69880, SGI0, SGI0, SGI0, SGI69881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15480[] = {SGI69882, SGI69883, SGI69884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69885, SGI0, SGI0, SGI0, SGI69886, SGI69886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15481[] = {SGI0, SGI0, SGI69887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15482[] = {SGI69888, SGI69889, SGI69890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69891, SGI0, SGI0, SGI0, SGI69892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15483[] = {SGI69893, SGI69894, SGI69895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69896, SGI0, SGI0, SGI0, SGI69897, SGI69898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15484[] = {SGI0, SGI69899, SGI69900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15485[] = {SGI0, SGI69901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15486[] = {SGI69902, SGI69903, SGI69904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15487[] = {SGI0, SGI69905, SGI69906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69907, SGI0, SGI0, SGI0, SGI69908, SGI69909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15488[] = {SGI69910, SGI69911, SGI69912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15489[] = {SGI0, SGI69913, SGI69914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69915, SGI0, SGI0, SGI0, SGI69916, SGI69916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15490[] = {SGI69917, SGI69918, SGI69919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69920, SGI0, SGI0, SGI0, SGI69921, SGI69921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15491[] = {SGI69922, SGI0, SGI69923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15492[] = {SGI69924, SGI69925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69926, SGI0, SGI0, SGI0, SGI69927, SGI69927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15493[] = {SGI69928, SGI69929, SGI69930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69931, SGI0, SGI0, SGI0, SGI69932, SGI69932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15494[] = {SGI69933, SGI69934, SGI69935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69936, SGI0, SGI0, SGI0, SGI69937, SGI69937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15495[] = {SGI69938, SGI69939, SGI69940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69941, SGI0, SGI0, SGI0, SGI69942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15496[] = {SGI69943, SGI69944, SGI69945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69946, SGI0, SGI0, SGI0, SGI69947, SGI69947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15497[] = {SGI69948, SGI69949, SGI69950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69951, SGI0, SGI0, SGI0, SGI69952, SGI69952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15498[] = {SGI69953, SGI0, SGI69954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15499[] = {SGI69955, SGI69956, SGI69957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69958, SGI0, SGI0, SGI0, SGI69959, SGI69959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15500[] = {SGI69960, SGI69961, SGI69962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69963, SGI0, SGI0, SGI0, SGI69964, SGI69964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15501[] = {SGI69965, SGI69966, SGI69967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69968, SGI0, SGI0, SGI0, SGI69969, SGI69969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15502[] = {SGI69970, SGI69971, SGI69972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69973, SGI0, SGI0, SGI0, SGI69974, SGI69974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15503[] = {SGI69975, SGI69976, SGI69977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69978, SGI0, SGI0, SGI0, SGI69979, SGI69979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15504[] = {SGI69980, SGI69981, SGI69982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69983, SGI0, SGI0, SGI0, SGI69984, SGI69984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15505[] = {SGI69985, SGI69986, SGI69987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69988, SGI0, SGI0, SGI0, SGI69989, SGI69989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15506[] = {SGI69990, SGI69991, SGI69992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI69993, SGI0, SGI0, SGI0, SGI69994, SGI69994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15507[] = {SGI69995, SGI0, SGI69996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15508[] = {SGI69997, SGI69998, SGI69999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70000, SGI70001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15509[] = {SGI70002, SGI70003, SGI70004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70005, SGI0, SGI0, SGI0, SGI70006, SGI70006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15510[] = {SGI70007, SGI70008, SGI70009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70010, SGI0, SGI0, SGI0, SGI70011, SGI70011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15511[] = {SGI70012, SGI70013, SGI70014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70015, SGI0, SGI0, SGI0, SGI70016, SGI70016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15512[] = {SGI70017, SGI70018, SGI70019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70020, SGI0, SGI0, SGI0, SGI70021, SGI70021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15513[] = {SGI70022, SGI70023, SGI70024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70025, SGI0, SGI0, SGI0, SGI70026, SGI70026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15514[] = {SGI70027, SGI70028, SGI70029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70030, SGI70031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15515[] = {SGI70032, SGI70033, SGI70034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70035, SGI70035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15516[] = {SGI70036, SGI70037, SGI70038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70039, SGI70040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15517[] = {SGI70041, SGI70042, SGI70043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70044, SGI70044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15518[] = {SGI70045, SGI70046, SGI70047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70048, SGI0, SGI0, SGI0, SGI70049, SGI70049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15519[] = {SGI70050, SGI70051, SGI70052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70053, SGI0, SGI0, SGI0, SGI70054, SGI70054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15520[] = {SGI70055, SGI70056, SGI70057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70058, SGI0, SGI0, SGI0, SGI70059, SGI70059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15521[] = {SGI70060, SGI70061, SGI70062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70063, SGI0, SGI0, SGI0, SGI70064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15522[] = {SGI70065, SGI70066, SGI70067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70068, SGI0, SGI0, SGI0, SGI70069, SGI70069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15523[] = {SGI70070, SGI70071, SGI70072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70073, SGI0, SGI0, SGI0, SGI70074, SGI70074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15524[] = {SGI70075, SGI70076, SGI70077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70078, SGI0, SGI0, SGI0, SGI70079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15525[] = {SGI70080, SGI70081, SGI70082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70083, SGI0, SGI0, SGI0, SGI70084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15526[] = {SGI70085, SGI70086, SGI70087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70088, SGI0, SGI0, SGI0, SGI70089, SGI70089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15527[] = {SGI70090, SGI70091, SGI70092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70093, SGI0, SGI0, SGI0, SGI70094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15528[] = {SGI70095, SGI70096, SGI70097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70098, SGI0, SGI0, SGI0, SGI70099, SGI70099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15529[] = {SGI70100, SGI70101, SGI70102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70103, SGI0, SGI0, SGI0, SGI70104, SGI70104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15530[] = {SGI70105, SGI70106, SGI70107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70108, SGI0, SGI0, SGI0, SGI70109, SGI70109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15531[] = {SGI70110, SGI0, SGI70111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15532[] = {SGI70112, SGI70113, SGI70114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70115, SGI0, SGI0, SGI0, SGI70116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15533[] = {SGI70117, SGI70118, SGI70119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70120, SGI0, SGI0, SGI0, SGI70121, SGI70121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15534[] = {SGI70122, SGI70123, SGI70124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70125, SGI0, SGI0, SGI0, SGI70126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15535[] = {SGI70127, SGI70128, SGI70129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70130, SGI0, SGI0, SGI0, SGI70131, SGI70131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15536[] = {SGI70132, SGI70133, SGI70134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70135, SGI0, SGI0, SGI0, SGI70136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15537[] = {SGI70137, SGI70138, SGI70139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70140, SGI0, SGI0, SGI0, SGI70141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15538[] = {SGI70142, SGI70143, SGI70144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70145, SGI0, SGI0, SGI0, SGI70146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15539[] = {SGI70147, SGI70148, SGI70149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70150, SGI0, SGI0, SGI0, SGI70151, SGI70151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15540[] = {SGI70152, SGI70153, SGI70154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70155, SGI0, SGI0, SGI0, SGI70156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15541[] = {SGI70157, SGI70158, SGI70159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70160, SGI0, SGI0, SGI0, SGI70161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15542[] = {SGI70162, SGI70163, SGI70164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70165, SGI0, SGI0, SGI0, SGI70166, SGI70166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15543[] = {SGI70167, SGI70168, SGI70169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70170, SGI0, SGI0, SGI0, SGI70171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15544[] = {SGI70172, SGI70173, SGI70174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70175, SGI0, SGI0, SGI0, SGI70176, SGI70176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15545[] = {SGI70177, SGI70178, SGI70179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70180, SGI0, SGI0, SGI0, SGI70181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15546[] = {SGI70182, SGI70183, SGI70184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70185, SGI0, SGI0, SGI0, SGI70186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15547[] = {SGI70187, SGI70188, SGI70189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70190, SGI0, SGI0, SGI0, SGI70191, SGI70191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15548[] = {SGI70192, SGI70193, SGI70194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70195, SGI0, SGI0, SGI0, SGI70196, SGI70196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15549[] = {SGI70197, SGI70198, SGI70199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70200, SGI0, SGI0, SGI0, SGI70201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15550[] = {SGI70202, SGI70203, SGI70204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70205, SGI0, SGI0, SGI0, SGI70206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15551[] = {SGI70207, SGI70208, SGI70209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70210, SGI0, SGI0, SGI0, SGI70211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15552[] = {SGI70212, SGI0, SGI70213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15553[] = {SGI70214, SGI70215, SGI70216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70217, SGI0, SGI0, SGI0, SGI70218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15554[] = {SGI70219, SGI70220, SGI70221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70222, SGI0, SGI0, SGI0, SGI70223, SGI70223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15555[] = {SGI70224, SGI70225, SGI70226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70227, SGI0, SGI0, SGI0, SGI70228, SGI70228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15556[] = {SGI70229, SGI70230, SGI70231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70232, SGI0, SGI0, SGI0, SGI70233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15557[] = {SGI70234, SGI70235, SGI70236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70237, SGI0, SGI0, SGI0, SGI70238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15558[] = {SGI70239, SGI70240, SGI70241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70242, SGI0, SGI0, SGI0, SGI70243, SGI70243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15559[] = {SGI70244, SGI70245, SGI70246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70247, SGI0, SGI0, SGI0, SGI70248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15560[] = {SGI70249, SGI70250, SGI70251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70252, SGI0, SGI0, SGI0, SGI70253, SGI70253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15561[] = {SGI70254, SGI70255, SGI70256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70257, SGI0, SGI0, SGI0, SGI70258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15562[] = {SGI70259, SGI70260, SGI70261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70262, SGI0, SGI0, SGI0, SGI70263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15563[] = {SGI70264, SGI70265, SGI70266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70267, SGI0, SGI0, SGI0, SGI70268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15564[] = {SGI70269, SGI70270, SGI70271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70272, SGI0, SGI0, SGI0, SGI70273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15565[] = {SGI70274, SGI70275, SGI70276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70278, SGI0, SGI0, SGI0, SGI70281, SGI70281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15566[] = {SGI70277, SGI70279, SGI70280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70282, SGI0, SGI0, SGI0, SGI70283, SGI70283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15567[] = {SGI70284, SGI70285, SGI70286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70290, SGI0, SGI0, SGI0, SGI70291, SGI70291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15568[] = {SGI70287, SGI70288, SGI70289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70292, SGI0, SGI0, SGI0, SGI70293, SGI70293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15569[] = {SGI70294, SGI0, SGI70295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15570[] = {SGI70296, SGI70297, SGI70298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70299, SGI0, SGI0, SGI0, SGI70300, SGI70300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15571[] = {SGI0, SGI0, SGI70301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70302, SGI0, SGI0, SGI0, SGI70303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15572[] = {SGI70304, SGI0, SGI70305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15573[] = {SGI70306, SGI70307, SGI70308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70309, SGI0, SGI0, SGI0, SGI70310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15574[] = {SGI70311, SGI0, SGI70312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15575[] = {SGI70313, SGI70314, SGI70315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70316, SGI0, SGI0, SGI0, SGI70317, SGI70317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15576[] = {SGI70318, SGI70319, SGI70320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70322, SGI0, SGI0, SGI0, SGI70325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15577[] = {SGI70321, SGI70323, SGI70324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70326, SGI0, SGI0, SGI0, SGI70327, SGI70327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15578[] = {SGI70328, SGI70329, SGI70330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70332, SGI0, SGI0, SGI0, SGI70335, SGI70335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15579[] = {SGI70331, SGI70333, SGI70334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70336, SGI0, SGI0, SGI0, SGI70337, SGI70337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15580[] = {SGI70338, SGI70339, SGI70340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70341, SGI0, SGI0, SGI0, SGI70343, SGI70343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15581[] = {SGI70342, SGI70344, SGI70345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70346, SGI0, SGI0, SGI0, SGI70347, SGI70347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15582[] = {SGI70348, SGI70349, SGI70350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70352, SGI0, SGI0, SGI0, SGI70355, SGI70355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15583[] = {SGI70351, SGI70353, SGI70354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70356, SGI0, SGI0, SGI0, SGI70357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15584[] = {SGI70358, SGI70359, SGI70360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70361, SGI0, SGI0, SGI0, SGI70362, SGI70362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15585[] = {SGI70363, SGI70364, SGI70365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70367, SGI0, SGI0, SGI0, SGI70369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15586[] = {SGI70366, SGI70368, SGI70370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70371, SGI0, SGI0, SGI0, SGI70372, SGI70372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15587[] = {SGI70373, SGI0, SGI70375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15588[] = {SGI70374, SGI70376, SGI70377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70378, SGI0, SGI0, SGI0, SGI70379, SGI70379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15589[] = {SGI70380, SGI70381, SGI70382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70383, SGI0, SGI0, SGI0, SGI70385, SGI70385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15590[] = {SGI70384, SGI70386, SGI70387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70388, SGI0, SGI0, SGI0, SGI70389, SGI70389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15591[] = {SGI70390, SGI70391, SGI70392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70393, SGI0, SGI0, SGI0, SGI70394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15592[] = {SGI70395, SGI70396, SGI70397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70398, SGI0, SGI0, SGI0, SGI70399, SGI70399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15593[] = {SGI70400, SGI70401, SGI70402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70403, SGI0, SGI0, SGI0, SGI70404, SGI70404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15594[] = {SGI70405, SGI70406, SGI70407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70408, SGI0, SGI0, SGI0, SGI70409, SGI70409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15595[] = {SGI70410, SGI70411, SGI70412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70413, SGI0, SGI0, SGI0, SGI70414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15596[] = {SGI70415, SGI70416, SGI70417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70418, SGI0, SGI0, SGI0, SGI70419, SGI70419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15597[] = {SGI70420, SGI70421, SGI70422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70423, SGI0, SGI0, SGI0, SGI70424, SGI70424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15598[] = {SGI70425, SGI70426, SGI70427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70428, SGI0, SGI0, SGI0, SGI70429, SGI70429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15599[] = {SGI70430, SGI70431, SGI70432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70433, SGI0, SGI0, SGI0, SGI70434, SGI70434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15600[] = {SGI70435, SGI70436, SGI70437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70438, SGI70438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15601[] = {SGI70439, SGI0, SGI70440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15602[] = {SGI70441, SGI70442, SGI70443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70444, SGI0, SGI0, SGI0, SGI70445, SGI70445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15603[] = {SGI70446, SGI70447, SGI70448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70449, SGI0, SGI0, SGI0, SGI70450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15604[] = {SGI70451, SGI70452, SGI70453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70454, SGI0, SGI0, SGI0, SGI70455, SGI70455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15605[] = {SGI70456, SGI70457, SGI70458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70459, SGI0, SGI0, SGI0, SGI70460, SGI70460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15606[] = {SGI70461, SGI70462, SGI70463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70464, SGI0, SGI0, SGI0, SGI70465, SGI70465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15607[] = {SGI70466, SGI70467, SGI70468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70469, SGI0, SGI0, SGI0, SGI70470, SGI70470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15608[] = {SGI70471, SGI70472, SGI70473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70474, SGI0, SGI0, SGI0, SGI70475, SGI70475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15609[] = {SGI0, SGI0, SGI70476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70478, SGI0, SGI0, SGI0, SGI70481, SGI70481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15610[] = {SGI70477, SGI70479, SGI70480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70482, SGI0, SGI0, SGI0, SGI70483, SGI70483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15611[] = {SGI70484, SGI0, SGI70485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15612[] = {SGI70486, SGI70487, SGI70488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70489, SGI0, SGI0, SGI0, SGI70490, SGI70490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15613[] = {SGI70491, SGI70492, SGI70493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70494, SGI0, SGI0, SGI0, SGI70495, SGI70495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15614[] = {SGI70496, SGI70497, SGI70498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70499, SGI0, SGI0, SGI0, SGI70500, SGI70500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15615[] = {SGI70501, SGI0, SGI70502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15616[] = {SGI70503, SGI0, SGI70504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70505, SGI70505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15617[] = {SGI70506, SGI70507, SGI70508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70509, SGI0, SGI0, SGI0, SGI70510, SGI70510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15618[] = {SGI70511, SGI70512, SGI70513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70514, SGI0, SGI0, SGI0, SGI70515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15619[] = {SGI70516, SGI70517, SGI70518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70519, SGI70520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15620[] = {SGI70521, SGI70522, SGI70523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70524, SGI0, SGI0, SGI0, SGI70525, SGI70525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15621[] = {SGI70526, SGI70527, SGI70528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70532, SGI0, SGI0, SGI0, SGI70533, SGI70533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15622[] = {SGI70529, SGI70530, SGI70531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70534, SGI0, SGI0, SGI0, SGI70535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15623[] = {SGI70536, SGI70537, SGI70538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70539, SGI0, SGI0, SGI0, SGI70540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15624[] = {SGI70541, SGI0, SGI70542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70543, SGI0, SGI0, SGI0, SGI70544, SGI70544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15625[] = {SGI70545, SGI70546, SGI70547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70548, SGI0, SGI0, SGI0, SGI70549, SGI70549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15626[] = {SGI70550, SGI70551, SGI70552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70553, SGI0, SGI0, SGI0, SGI70554, SGI70554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15627[] = {SGI70555, SGI0, SGI70556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15628[] = {SGI70557, SGI70558, SGI70559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70560, SGI0, SGI0, SGI0, SGI70561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15629[] = {SGI70562, SGI70563, SGI70564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70565, SGI0, SGI0, SGI0, SGI70566, SGI70566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15630[] = {SGI70567, SGI70568, SGI70569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70570, SGI0, SGI0, SGI0, SGI70571, SGI70571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15631[] = {SGI70572, SGI70573, SGI70574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70575, SGI0, SGI0, SGI0, SGI70576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15632[] = {SGI70577, SGI0, SGI70578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15633[] = {SGI70579, SGI70580, SGI70581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70582, SGI0, SGI0, SGI0, SGI70583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15634[] = {SGI70584, SGI70585, SGI70586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70587, SGI0, SGI0, SGI0, SGI70588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15635[] = {SGI70589, SGI70590, SGI70591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70592, SGI0, SGI0, SGI0, SGI70593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15636[] = {SGI70594, SGI70595, SGI70596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70597, SGI0, SGI0, SGI0, SGI70598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15637[] = {SGI70599, SGI70600, SGI70601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70602, SGI0, SGI0, SGI0, SGI70603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15638[] = {SGI70604, SGI70605, SGI70606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70607, SGI0, SGI0, SGI0, SGI70608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15639[] = {SGI70609, SGI70610, SGI70611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70612, SGI0, SGI0, SGI0, SGI70613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15640[] = {SGI70614, SGI70615, SGI70616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70617, SGI0, SGI0, SGI0, SGI70618, SGI70618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15641[] = {SGI70619, SGI70620, SGI70621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70622, SGI0, SGI0, SGI0, SGI70623, SGI70623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15642[] = {SGI70624, SGI70625, SGI70626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70627, SGI0, SGI0, SGI0, SGI70628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15643[] = {SGI70629, SGI70630, SGI70631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70632, SGI0, SGI0, SGI0, SGI70633, SGI70633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15644[] = {SGI70634, SGI70635, SGI70636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70637, SGI0, SGI0, SGI0, SGI70638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15645[] = {SGI70639, SGI70640, SGI70641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70642, SGI0, SGI0, SGI0, SGI70643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15646[] = {SGI70644, SGI70645, SGI70646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70647, SGI0, SGI0, SGI0, SGI70648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15647[] = {SGI70649, SGI70650, SGI70651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70652, SGI0, SGI0, SGI0, SGI70653, SGI70653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15648[] = {SGI70654, SGI70655, SGI70656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70657, SGI0, SGI0, SGI0, SGI70658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15649[] = {SGI70659, SGI70660, SGI70661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70662, SGI0, SGI0, SGI0, SGI70663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15650[] = {SGI70664, SGI70665, SGI70666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70667, SGI0, SGI0, SGI0, SGI70668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15651[] = {SGI70669, SGI70670, SGI70671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70672, SGI0, SGI0, SGI0, SGI70673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15652[] = {SGI70674, SGI70675, SGI70676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70677, SGI0, SGI0, SGI0, SGI70678, SGI70678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15653[] = {SGI70679, SGI70680, SGI70681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70682, SGI0, SGI0, SGI0, SGI70683, SGI70683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15654[] = {SGI70684, SGI70685, SGI70686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70687, SGI0, SGI0, SGI0, SGI70688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15655[] = {SGI70689, SGI70690, SGI70691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70692, SGI0, SGI0, SGI0, SGI70693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15656[] = {SGI70694, SGI70695, SGI70696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70697, SGI0, SGI0, SGI0, SGI70698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15657[] = {SGI70699, SGI70700, SGI70701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70702, SGI0, SGI0, SGI0, SGI70703, SGI70703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15658[] = {SGI70704, SGI70705, SGI70706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70707, SGI0, SGI0, SGI0, SGI70708, SGI70708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15659[] = {SGI70709, SGI70710, SGI70711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70712, SGI0, SGI0, SGI0, SGI70713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15660[] = {SGI70714, SGI70715, SGI70716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70717, SGI0, SGI0, SGI0, SGI70718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15661[] = {SGI70719, SGI70720, SGI70721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70722, SGI0, SGI0, SGI0, SGI70723, SGI70723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15662[] = {SGI70724, SGI70725, SGI70726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70727, SGI0, SGI0, SGI0, SGI70728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15663[] = {SGI70729, SGI70730, SGI70731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70732, SGI0, SGI0, SGI0, SGI70733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15664[] = {SGI70734, SGI70735, SGI70736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70737, SGI0, SGI0, SGI0, SGI70738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15665[] = {SGI70739, SGI70740, SGI70741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70742, SGI0, SGI0, SGI0, SGI70743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15666[] = {SGI70744, SGI70745, SGI70746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70747, SGI70747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15667[] = {SGI70748, SGI70749, SGI70750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70751, SGI0, SGI0, SGI0, SGI70752, SGI70752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15668[] = {SGI70753, SGI70754, SGI70755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70756, SGI0, SGI0, SGI0, SGI70757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15669[] = {SGI70758, SGI70759, SGI70760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70761, SGI0, SGI0, SGI0, SGI70762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15670[] = {SGI70763, SGI70764, SGI70765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70766, SGI0, SGI0, SGI0, SGI70767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15671[] = {SGI70768, SGI70769, SGI70770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70771, SGI0, SGI0, SGI0, SGI70772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15672[] = {SGI70773, SGI70774, SGI70775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70776, SGI0, SGI0, SGI0, SGI70777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15673[] = {SGI70778, SGI70779, SGI70780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70781, SGI0, SGI0, SGI0, SGI70782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15674[] = {SGI70783, SGI70784, SGI70785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70786, SGI0, SGI0, SGI0, SGI70787, SGI70787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15675[] = {SGI70788, SGI70789, SGI70790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70791, SGI0, SGI0, SGI0, SGI70792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15676[] = {SGI0, SGI0, SGI70793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15677[] = {SGI70795, SGI70796, SGI70797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70798, SGI0, SGI0, SGI0, SGI70799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15678[] = {SGI70800, SGI70801, SGI70802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70803, SGI0, SGI0, SGI0, SGI70804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15679[] = {SGI70805, SGI70806, SGI70807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70808, SGI0, SGI0, SGI0, SGI70809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15680[] = {SGI70810, SGI70811, SGI70812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70813, SGI0, SGI0, SGI0, SGI70814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15681[] = {SGI70815, SGI70816, SGI70817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70818, SGI0, SGI0, SGI0, SGI70819, SGI70819, SGI0, SGI70820, SGI0, NULL};
const Segment * const SG15682[] = {SGI70821, SGI70822, SGI70823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70824, SGI0, SGI0, SGI0, SGI70825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15683[] = {SGI0, SGI0, SGI70826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70827, SGI70827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15684[] = {SGI70828, SGI70829, SGI70830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70831, SGI0, SGI0, SGI0, SGI70832, SGI70832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15685[] = {SGI70833, SGI70834, SGI70835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70836, SGI0, SGI0, SGI0, SGI70837, SGI70837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15686[] = {SGI70838, SGI70839, SGI70840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70841, SGI0, SGI0, SGI0, SGI70842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15687[] = {SGI70843, SGI70844, SGI70845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70846, SGI0, SGI0, SGI0, SGI70847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15688[] = {SGI70848, SGI0, SGI70849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70850, SGI0, SGI0, SGI0, SGI70851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15689[] = {SGI70852, SGI70853, SGI70854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70855, SGI0, SGI0, SGI0, SGI70856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15690[] = {SGI70857, SGI70858, SGI70859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70860, SGI0, SGI0, SGI0, SGI70861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15691[] = {SGI70862, SGI70863, SGI70864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70865, SGI0, SGI0, SGI0, SGI70866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15692[] = {SGI70867, SGI70868, SGI70869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70870, SGI0, SGI0, SGI0, SGI70871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15693[] = {SGI70872, SGI0, SGI70873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70874, SGI0, SGI0, SGI0, SGI70875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15694[] = {SGI70876, SGI70877, SGI70878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70879, SGI0, SGI0, SGI0, SGI70880, SGI70880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15695[] = {SGI70881, SGI0, SGI70882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70883, SGI70884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15696[] = {SGI70885, SGI70886, SGI70887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70888, SGI0, SGI0, SGI0, SGI70889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15697[] = {SGI70890, SGI70891, SGI70892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70893, SGI0, SGI0, SGI0, SGI70894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15698[] = {SGI70895, SGI70896, SGI70897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70898, SGI0, SGI0, SGI0, SGI70899, SGI70899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15699[] = {SGI70900, SGI70901, SGI70902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70903, SGI0, SGI0, SGI0, SGI70904, SGI70904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15700[] = {SGI70905, SGI70906, SGI70907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70908, SGI0, SGI0, SGI0, SGI70909, SGI70909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15701[] = {SGI70910, SGI70911, SGI70912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70913, SGI0, SGI0, SGI0, SGI70914, SGI70914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15702[] = {SGI70915, SGI70916, SGI70917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70918, SGI0, SGI0, SGI0, SGI70919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15703[] = {SGI70920, SGI70921, SGI70922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70923, SGI0, SGI0, SGI0, SGI70924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15704[] = {SGI70925, SGI70926, SGI70927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70928, SGI0, SGI0, SGI0, SGI70929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15705[] = {SGI70930, SGI70931, SGI70932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70933, SGI0, SGI0, SGI0, SGI70934, SGI70934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15706[] = {SGI70935, SGI70936, SGI70937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70938, SGI0, SGI0, SGI0, SGI70939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15707[] = {SGI70940, SGI70941, SGI70942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70943, SGI0, SGI0, SGI0, SGI70944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15708[] = {SGI70945, SGI70946, SGI70947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70948, SGI0, SGI0, SGI0, SGI70949, SGI70949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15709[] = {SGI70950, SGI70951, SGI70952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70953, SGI0, SGI0, SGI0, SGI70954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15710[] = {SGI70955, SGI70956, SGI70957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70958, SGI0, SGI0, SGI0, SGI70959, SGI70959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15711[] = {SGI70960, SGI70961, SGI70962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70963, SGI0, SGI0, SGI0, SGI70964, SGI70964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15712[] = {SGI70965, SGI70966, SGI70967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70968, SGI0, SGI0, SGI0, SGI70969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15713[] = {SGI70970, SGI70971, SGI70972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70973, SGI0, SGI0, SGI0, SGI70974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15714[] = {SGI70975, SGI70976, SGI70977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70978, SGI0, SGI0, SGI0, SGI70979, SGI70979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15715[] = {SGI70980, SGI0, SGI70981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70982, SGI0, SGI0, SGI0, SGI70983, SGI70983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15716[] = {SGI70984, SGI70985, SGI70986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70987, SGI0, SGI0, SGI0, SGI70988, SGI70988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15717[] = {SGI70989, SGI0, SGI70990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15718[] = {SGI70991, SGI0, SGI70992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15719[] = {SGI70993, SGI0, SGI70994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15720[] = {SGI70995, SGI70996, SGI70997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI70998, SGI0, SGI0, SGI0, SGI70999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15721[] = {SGI71000, SGI71001, SGI71002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71003, SGI0, SGI0, SGI0, SGI71004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15722[] = {SGI71005, SGI71006, SGI71007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71008, SGI0, SGI0, SGI0, SGI71009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15723[] = {SGI71010, SGI0, SGI71011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71012, SGI0, SGI0, SGI0, SGI71013, SGI71013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15724[] = {SGI71014, SGI0, SGI71015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15725[] = {SGI71016, SGI71017, SGI71018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71019, SGI0, SGI0, SGI0, SGI71020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15726[] = {SGI71021, SGI0, SGI71022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15727[] = {SGI71023, SGI0, SGI71024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15728[] = {SGI71025, SGI71026, SGI71027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71028, SGI0, SGI0, SGI0, SGI71029, SGI71029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15729[] = {SGI71030, SGI71031, SGI71032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71033, SGI0, SGI0, SGI0, SGI71034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15730[] = {SGI71035, SGI71036, SGI71037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71038, SGI0, SGI0, SGI0, SGI71039, SGI71039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15731[] = {SGI71040, SGI71041, SGI71042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71043, SGI0, SGI0, SGI0, SGI71044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15732[] = {SGI0, SGI0, SGI71045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71046, SGI0, SGI0, SGI0, SGI71047, SGI71047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15733[] = {SGI71048, SGI71049, SGI71050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71051, SGI0, SGI0, SGI0, SGI71052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15734[] = {SGI71053, SGI71054, SGI71055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71056, SGI0, SGI0, SGI0, SGI71057, SGI71057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15735[] = {SGI71058, SGI71059, SGI71060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71061, SGI0, SGI0, SGI0, SGI71062, SGI71062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15736[] = {SGI71063, SGI71064, SGI71065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71066, SGI0, SGI0, SGI0, SGI71067, SGI71067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15737[] = {SGI71068, SGI71069, SGI71070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71071, SGI0, SGI0, SGI0, SGI71072, SGI71072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15738[] = {SGI71073, SGI71074, SGI71075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71076, SGI0, SGI0, SGI0, SGI71077, SGI71077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15739[] = {SGI71078, SGI71079, SGI71080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71081, SGI0, SGI0, SGI0, SGI71082, SGI71082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15740[] = {SGI71083, SGI71084, SGI71085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71086, SGI0, SGI0, SGI0, SGI71087, SGI71087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15741[] = {SGI71088, SGI0, SGI71089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15742[] = {SGI71090, SGI71091, SGI71092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71093, SGI0, SGI0, SGI0, SGI71094, SGI71094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15743[] = {SGI71095, SGI71096, SGI71097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71098, SGI0, SGI0, SGI0, SGI71099, SGI71099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15744[] = {SGI71100, SGI71101, SGI71102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71103, SGI0, SGI0, SGI0, SGI71104, SGI71104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15745[] = {SGI71105, SGI71106, SGI71107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71108, SGI0, SGI0, SGI0, SGI71109, SGI71109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15746[] = {SGI71110, SGI71111, SGI71112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71113, SGI0, SGI0, SGI0, SGI71114, SGI71114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15747[] = {SGI71115, SGI71116, SGI71117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71118, SGI0, SGI0, SGI0, SGI71119, SGI71119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15748[] = {SGI71120, SGI71121, SGI71122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71123, SGI0, SGI0, SGI0, SGI71124, SGI71124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15749[] = {SGI71125, SGI0, SGI71126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15750[] = {SGI71127, SGI0, SGI71128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71129, SGI0, SGI0, SGI0, SGI71130, SGI71130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15751[] = {SGI71131, SGI71132, SGI71133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71134, SGI0, SGI0, SGI0, SGI71135, SGI71135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15752[] = {SGI71136, SGI71137, SGI71138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71139, SGI0, SGI0, SGI0, SGI71140, SGI71140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15753[] = {SGI71141, SGI0, SGI71142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15754[] = {SGI71143, SGI71144, SGI71145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71146, SGI0, SGI0, SGI0, SGI71147, SGI71147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15755[] = {SGI71148, SGI0, SGI71149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15756[] = {SGI71150, SGI71151, SGI71152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71153, SGI0, SGI0, SGI0, SGI71154, SGI71154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15757[] = {SGI71155, SGI0, SGI71156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15758[] = {SGI71157, SGI71158, SGI71159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71160, SGI0, SGI0, SGI0, SGI71161, SGI71161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15759[] = {SGI71162, SGI71163, SGI71164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71165, SGI0, SGI0, SGI0, SGI71166, SGI71166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15760[] = {SGI71167, SGI0, SGI71168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71169, SGI0, SGI0, SGI0, SGI71170, SGI71170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15761[] = {SGI71171, SGI71172, SGI71173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71174, SGI0, SGI0, SGI0, SGI71175, SGI71175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15762[] = {SGI71176, SGI0, SGI71177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15763[] = {SGI71178, SGI71179, SGI71180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15764[] = {SGI71181, SGI71182, SGI71183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71184, SGI0, SGI0, SGI0, SGI71185, SGI71185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15765[] = {SGI71186, SGI0, SGI71187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71188, SGI0, SGI0, SGI0, SGI71189, SGI71189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15766[] = {SGI71190, SGI71191, SGI71192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71193, SGI0, SGI0, SGI0, SGI71194, SGI71194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15767[] = {SGI71195, SGI71196, SGI71197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71198, SGI0, SGI0, SGI0, SGI71199, SGI71199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15768[] = {SGI71200, SGI71201, SGI71202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71203, SGI0, SGI0, SGI0, SGI71204, SGI71204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15769[] = {SGI71205, SGI71206, SGI71207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71208, SGI0, SGI0, SGI0, SGI71209, SGI71209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15770[] = {SGI71210, SGI71211, SGI71212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71213, SGI0, SGI0, SGI0, SGI71214, SGI71214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15771[] = {SGI71215, SGI71216, SGI71217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71218, SGI0, SGI0, SGI0, SGI71219, SGI71219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15772[] = {SGI71220, SGI71221, SGI71222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71223, SGI0, SGI0, SGI0, SGI71224, SGI71224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15773[] = {SGI71225, SGI71226, SGI71227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71228, SGI0, SGI0, SGI0, SGI71229, SGI71229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15774[] = {SGI71230, SGI0, SGI71231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15775[] = {SGI71232, SGI71233, SGI71234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71235, SGI0, SGI0, SGI0, SGI71236, SGI71236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15776[] = {SGI71237, SGI71238, SGI71239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71240, SGI0, SGI0, SGI0, SGI71241, SGI71241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15777[] = {SGI71242, SGI0, SGI71243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15778[] = {SGI71244, SGI0, SGI71245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71246, SGI0, SGI0, SGI0, SGI71247, SGI71247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15779[] = {SGI71253, SGI71254, SGI71255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71256, SGI0, SGI0, SGI0, SGI71257, SGI71257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15780[] = {SGI71258, SGI71259, SGI71260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71261, SGI0, SGI0, SGI0, SGI71262, SGI71262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15781[] = {SGI71263, SGI0, SGI71264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15782[] = {SGI71265, SGI71266, SGI71267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71268, SGI0, SGI0, SGI0, SGI71269, SGI71269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15783[] = {SGI71248, SGI71249, SGI71250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71251, SGI0, SGI0, SGI0, SGI71252, SGI71252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15784[] = {SGI71270, SGI71271, SGI71272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71273, SGI71274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15785[] = {SGI71275, SGI0, SGI71276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15786[] = {SGI71277, SGI71278, SGI71279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71280, SGI71280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15787[] = {SGI71281, SGI0, SGI71282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15788[] = {SGI71283, SGI0, SGI71284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71285, SGI0, SGI0, SGI0, SGI71286, SGI71286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15789[] = {SGI71287, SGI71288, SGI71289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71290, SGI0, SGI0, SGI0, SGI71291, SGI71291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15790[] = {SGI71292, SGI0, SGI71293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15791[] = {SGI71294, SGI71295, SGI71296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71297, SGI0, SGI0, SGI0, SGI71298, SGI71298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15792[] = {SGI71299, SGI71300, SGI71301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71302, SGI71302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15793[] = {SGI71303, SGI71304, SGI71305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71306, SGI0, SGI0, SGI0, SGI71307, SGI71307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15794[] = {SGI71308, SGI71309, SGI71310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71311, SGI0, SGI0, SGI0, SGI71312, SGI71312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15795[] = {SGI71313, SGI71314, SGI71315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71316, SGI0, SGI0, SGI0, SGI71317, SGI71317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15796[] = {SGI71318, SGI71319, SGI71320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71321, SGI0, SGI0, SGI0, SGI71322, SGI71322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15797[] = {SGI71323, SGI0, SGI71324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15798[] = {SGI71325, SGI71326, SGI71327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71328, SGI0, SGI0, SGI0, SGI71329, SGI71329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15799[] = {SGI71330, SGI71331, SGI71332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71333, SGI0, SGI0, SGI0, SGI71334, SGI71334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15800[] = {SGI71335, SGI71336, SGI71337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71338, SGI0, SGI0, SGI0, SGI71339, SGI71339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15801[] = {SGI71340, SGI71341, SGI71342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71343, SGI0, SGI0, SGI0, SGI71344, SGI71344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15802[] = {SGI71345, SGI0, SGI71346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15803[] = {SGI71347, SGI71348, SGI71349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71350, SGI71350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15804[] = {SGI71351, SGI0, SGI71352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71353, SGI0, SGI0, SGI0, SGI71354, SGI71354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15805[] = {SGI71355, SGI71356, SGI71357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71358, SGI0, SGI0, SGI0, SGI71359, SGI71359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15806[] = {SGI71360, SGI71361, SGI71362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71363, SGI0, SGI0, SGI0, SGI71364, SGI71364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15807[] = {SGI71365, SGI71366, SGI71367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71368, SGI0, SGI0, SGI0, SGI71369, SGI71369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15808[] = {SGI71370, SGI71371, SGI71372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71373, SGI0, SGI0, SGI0, SGI71374, SGI71374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15809[] = {SGI71375, SGI71376, SGI71377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71378, SGI0, SGI0, SGI0, SGI71379, SGI71379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15810[] = {SGI71380, SGI71381, SGI71382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71383, SGI0, SGI0, SGI0, SGI71384, SGI71384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15811[] = {SGI71385, SGI71386, SGI71387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71388, SGI0, SGI0, SGI0, SGI71389, SGI71389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15812[] = {SGI71390, SGI71391, SGI71392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71393, SGI0, SGI0, SGI0, SGI71394, SGI71394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15813[] = {SGI71395, SGI0, SGI71396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15814[] = {SGI71397, SGI71398, SGI71399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71400, SGI0, SGI0, SGI0, SGI71401, SGI71401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15815[] = {SGI71402, SGI0, SGI71403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15816[] = {SGI71404, SGI71405, SGI71406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71407, SGI0, SGI0, SGI0, SGI71408, SGI71408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15817[] = {SGI71409, SGI71410, SGI71411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71413, SGI0, SGI0, SGI0, SGI71416, SGI71416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15818[] = {SGI71412, SGI71414, SGI71415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71417, SGI0, SGI0, SGI0, SGI71418, SGI71418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15819[] = {SGI71419, SGI0, SGI71420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15820[] = {SGI71421, SGI0, SGI71422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71423, SGI0, SGI0, SGI0, SGI71424, SGI71424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15821[] = {SGI71425, SGI71426, SGI71427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71428, SGI0, SGI0, SGI0, SGI71429, SGI71429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15822[] = {SGI71430, SGI71431, SGI71433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71436, SGI0, SGI0, SGI0, SGI71437, SGI71437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15823[] = {SGI71432, SGI71434, SGI71435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71438, SGI71438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15824[] = {SGI71439, SGI71440, SGI71441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71442, SGI0, SGI0, SGI0, SGI71443, SGI71443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15825[] = {SGI71444, SGI71446, SGI71448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71450, SGI0, SGI0, SGI0, SGI71452, SGI71454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15826[] = {SGI71445, SGI71447, SGI71449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71451, SGI0, SGI0, SGI0, SGI71453, SGI71453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15827[] = {SGI71455, SGI71456, SGI71457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71458, SGI0, SGI0, SGI0, SGI71459, SGI71459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15828[] = {SGI71460, SGI71461, SGI71462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71463, SGI0, SGI0, SGI0, SGI71464, SGI71464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15829[] = {SGI71465, SGI71466, SGI71467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71468, SGI0, SGI0, SGI0, SGI71469, SGI71469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15830[] = {SGI71470, SGI71471, SGI71472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71473, SGI0, SGI0, SGI0, SGI71474, SGI71474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15831[] = {SGI71475, SGI71476, SGI71477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71478, SGI0, SGI0, SGI0, SGI71479, SGI71479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15832[] = {SGI0, SGI71480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71481, SGI0, SGI0, SGI0, SGI71482, SGI71482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15833[] = {SGI71483, SGI71484, SGI71485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71486, SGI0, SGI0, SGI0, SGI71487, SGI71487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15834[] = {SGI71488, SGI71489, SGI71490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71491, SGI71492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15835[] = {SGI71493, SGI71494, SGI71495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71496, SGI0, SGI0, SGI0, SGI71497, SGI71497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15836[] = {SGI71498, SGI71499, SGI71500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71501, SGI0, SGI0, SGI0, SGI71502, SGI71502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15837[] = {SGI71503, SGI0, SGI71504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15838[] = {SGI71505, SGI71506, SGI71507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71508, SGI0, SGI0, SGI0, SGI71509, SGI71509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15839[] = {SGI71510, SGI71511, SGI71512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71513, SGI0, SGI0, SGI0, SGI71514, SGI71514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15840[] = {SGI71515, SGI71516, SGI71517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71521, SGI0, SGI0, SGI0, SGI71522, SGI71522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15841[] = {SGI71518, SGI71519, SGI71520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71523, SGI71523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15842[] = {SGI71524, SGI71525, SGI71526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71527, SGI0, SGI0, SGI0, SGI71528, SGI71528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15843[] = {SGI71529, SGI71530, SGI71531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71532, SGI0, SGI0, SGI0, SGI71533, SGI71533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15844[] = {SGI71534, SGI71535, SGI71536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71537, SGI0, SGI0, SGI0, SGI71538, SGI71538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15845[] = {SGI71539, SGI71540, SGI71541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71542, SGI0, SGI0, SGI0, SGI71544, SGI71544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15846[] = {SGI71543, SGI71545, SGI71546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71547, SGI0, SGI0, SGI0, SGI71548, SGI71548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15847[] = {SGI71549, SGI71550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71551, SGI0, SGI0, SGI0, SGI71552, SGI71552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15848[] = {SGI71553, SGI71554, SGI71555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71556, SGI0, SGI0, SGI0, SGI71557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15849[] = {SGI71558, SGI71559, SGI71560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71561, SGI0, SGI0, SGI0, SGI71562, SGI71562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15850[] = {SGI71564, SGI71566, SGI71568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71571, SGI71571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15851[] = {SGI71563, SGI71565, SGI71567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71569, SGI0, SGI0, SGI0, SGI71570, SGI71570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15852[] = {SGI71572, SGI71573, SGI71574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71575, SGI0, SGI0, SGI0, SGI71576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15853[] = {SGI71577, SGI0, SGI71578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15854[] = {SGI71579, SGI71580, SGI71581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71582, SGI0, SGI0, SGI0, SGI71583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15855[] = {SGI71584, SGI71585, SGI71586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71587, SGI0, SGI0, SGI0, SGI71588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15856[] = {SGI71589, SGI0, SGI71590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15857[] = {SGI71591, SGI71592, SGI71593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71594, SGI0, SGI0, SGI0, SGI71595, SGI71595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15858[] = {SGI71596, SGI71597, SGI71598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71599, SGI0, SGI0, SGI0, SGI71600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15859[] = {SGI71601, SGI0, SGI71602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71603, SGI0, SGI0, SGI0, SGI71604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15860[] = {SGI71605, SGI71606, SGI71607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71608, SGI0, SGI0, SGI0, SGI71609, SGI71610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15861[] = {SGI71611, SGI71612, SGI71613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71614, SGI0, SGI0, SGI0, SGI71615, SGI71616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15862[] = {SGI71617, SGI71618, SGI71619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71620, SGI71620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15863[] = {SGI71621, SGI71622, SGI71623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71624, SGI71625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15864[] = {SGI71626, SGI71627, SGI71628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71629, SGI0, SGI0, SGI0, SGI71630, SGI71630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15865[] = {SGI71631, SGI0, SGI71632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71633, SGI0, SGI0, SGI0, SGI71634, SGI71634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15866[] = {SGI71635, SGI71636, SGI71637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15867[] = {SGI71638, SGI71639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15868[] = {SGI71640, SGI71641, SGI71642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71643, SGI0, SGI0, SGI0, SGI71644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15869[] = {SGI71645, SGI71646, SGI71647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15870[] = {SGI71648, SGI0, SGI71649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15871[] = {SGI71650, SGI71651, SGI71652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15872[] = {SGI71653, SGI71654, SGI71655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71656, SGI0, SGI0, SGI0, SGI71657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15873[] = {SGI0, SGI71658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15874[] = {SGI71659, SGI71660, SGI71661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71662, SGI0, SGI0, SGI0, SGI71663, SGI71663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15875[] = {SGI71664, SGI71665, SGI71666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71667, SGI0, SGI0, SGI0, SGI71668, SGI71668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15876[] = {SGI71669, SGI71670, SGI71671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71672, SGI0, SGI0, SGI0, SGI71673, SGI71673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15877[] = {SGI71674, SGI71675, SGI71676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71677, SGI0, SGI0, SGI0, SGI71678, SGI71678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15878[] = {SGI71679, SGI0, SGI71680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15879[] = {SGI71681, SGI71682, SGI71683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71684, SGI0, SGI0, SGI0, SGI71685, SGI71685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15880[] = {SGI71686, SGI71687, SGI71688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71689, SGI0, SGI0, SGI0, SGI71690, SGI71690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15881[] = {SGI71691, SGI71692, SGI71693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71694, SGI0, SGI0, SGI0, SGI71695, SGI71695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15882[] = {SGI71696, SGI71697, SGI71698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71699, SGI0, SGI0, SGI0, SGI71700, SGI71700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15883[] = {SGI71701, SGI71702, SGI71703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15884[] = {SGI71704, SGI71705, SGI71706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15885[] = {SGI71708, SGI71709, SGI71710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15886[] = {SGI0, SGI71711, SGI71712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15887[] = {SGI71713, SGI71714, SGI71715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71716, SGI0, SGI0, SGI0, SGI71717, SGI71717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15888[] = {SGI71718, SGI0, SGI71719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15889[] = {SGI71720, SGI71721, SGI71722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71723, SGI0, SGI0, SGI0, SGI71724, SGI71724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15890[] = {SGI71725, SGI71726, SGI71727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71728, SGI0, SGI0, SGI0, SGI71729, SGI71729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15891[] = {SGI71730, SGI71731, SGI71732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71733, SGI0, SGI0, SGI0, SGI71734, SGI71734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15892[] = {SGI71735, SGI71736, SGI71737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71738, SGI0, SGI0, SGI0, SGI71739, SGI71739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15893[] = {SGI71740, SGI71741, SGI71742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71743, SGI0, SGI0, SGI0, SGI71744, SGI71744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15894[] = {SGI71745, SGI71746, SGI71747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71748, SGI0, SGI0, SGI0, SGI71749, SGI71749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15895[] = {SGI71750, SGI71751, SGI71752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71753, SGI0, SGI0, SGI0, SGI71754, SGI71754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15896[] = {SGI71755, SGI71756, SGI71757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71758, SGI0, SGI0, SGI0, SGI71759, SGI71759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15897[] = {SGI71760, SGI71761, SGI71762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71763, SGI71763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15898[] = {SGI71764, SGI71765, SGI71766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71767, SGI0, SGI0, SGI0, SGI71768, SGI71768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15899[] = {SGI71769, SGI0, SGI71770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15900[] = {SGI71771, SGI71772, SGI71773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71774, SGI0, SGI0, SGI0, SGI71775, SGI71775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15901[] = {SGI71776, SGI71777, SGI71778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71779, SGI0, SGI0, SGI0, SGI71780, SGI71780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15902[] = {SGI71781, SGI0, SGI71782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15903[] = {SGI71783, SGI71784, SGI71785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71786, SGI0, SGI0, SGI0, SGI71787, SGI71787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15904[] = {SGI71788, SGI71789, SGI71790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71791, SGI0, SGI0, SGI0, SGI71792, SGI71792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15905[] = {SGI71793, SGI71794, SGI71795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71796, SGI0, SGI0, SGI0, SGI71797, SGI71797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15906[] = {SGI71798, SGI71799, SGI71800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71801, SGI0, SGI0, SGI0, SGI71802, SGI71802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15907[] = {SGI71803, SGI71804, SGI71805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71806, SGI0, SGI0, SGI0, SGI71807, SGI71807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15908[] = {SGI71808, SGI71809, SGI71810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71811, SGI0, SGI0, SGI0, SGI71812, SGI71812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15909[] = {SGI71813, SGI71814, SGI71815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71816, SGI0, SGI0, SGI0, SGI71817, SGI71817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15910[] = {SGI71818, SGI71819, SGI71820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71821, SGI0, SGI0, SGI0, SGI71822, SGI71822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15911[] = {SGI71823, SGI71824, SGI71825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71826, SGI0, SGI0, SGI0, SGI71827, SGI71827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15912[] = {SGI71828, SGI71829, SGI71830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71831, SGI0, SGI0, SGI0, SGI71832, SGI71832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15913[] = {SGI71833, SGI71834, SGI71835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71836, SGI0, SGI0, SGI0, SGI71837, SGI71837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15914[] = {SGI71838, SGI71839, SGI71840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71841, SGI0, SGI0, SGI0, SGI71842, SGI71842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15915[] = {SGI71843, SGI71844, SGI71845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71846, SGI71846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15916[] = {SGI71847, SGI71848, SGI71849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71850, SGI0, SGI0, SGI0, SGI71851, SGI71851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15917[] = {SGI71852, SGI71853, SGI71854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71855, SGI0, SGI0, SGI0, SGI71856, SGI71856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15918[] = {SGI71857, SGI71858, SGI71859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71860, SGI0, SGI0, SGI0, SGI71861, SGI71861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15919[] = {SGI71862, SGI71863, SGI71864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71865, SGI0, SGI0, SGI0, SGI71866, SGI71866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15920[] = {SGI71867, SGI71868, SGI71869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71873, SGI0, SGI0, SGI0, SGI71875, SGI71875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15921[] = {SGI71870, SGI71871, SGI71872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71874, SGI0, SGI0, SGI0, SGI71876, SGI71876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15922[] = {SGI71877, SGI71878, SGI71879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71880, SGI0, SGI0, SGI0, SGI71881, SGI71881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15923[] = {SGI71882, SGI71883, SGI71884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71885, SGI0, SGI0, SGI0, SGI71886, SGI71886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15924[] = {SGI0, SGI71888, SGI71890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15925[] = {SGI71887, SGI71889, SGI71891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71892, SGI0, SGI0, SGI0, SGI71893, SGI71893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15926[] = {SGI71894, SGI71895, SGI71896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71897, SGI0, SGI0, SGI0, SGI71898, SGI71898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15927[] = {SGI71899, SGI71900, SGI71901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71902, SGI0, SGI0, SGI0, SGI71903, SGI71903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15928[] = {SGI71904, SGI71905, SGI71906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71907, SGI0, SGI0, SGI0, SGI71908, SGI71908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15929[] = {SGI71909, SGI71910, SGI71911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71912, SGI0, SGI0, SGI0, SGI71913, SGI71913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15930[] = {SGI71914, SGI71915, SGI71916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71917, SGI0, SGI0, SGI0, SGI71918, SGI71918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15931[] = {SGI71919, SGI71920, SGI71921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71922, SGI0, SGI0, SGI0, SGI71923, SGI71923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15932[] = {SGI71924, SGI71925, SGI71926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71927, SGI0, SGI0, SGI0, SGI71928, SGI71928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15933[] = {SGI71929, SGI0, SGI71930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71931, SGI0, SGI0, SGI0, SGI71932, SGI71932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15934[] = {SGI71933, SGI71934, SGI71935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71936, SGI0, SGI0, SGI0, SGI71937, SGI71937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15935[] = {SGI71938, SGI71939, SGI71940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71941, SGI0, SGI0, SGI0, SGI71942, SGI71942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15936[] = {SGI71943, SGI71944, SGI71945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71946, SGI0, SGI0, SGI0, SGI71947, SGI71948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15937[] = {SGI71949, SGI71950, SGI71951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71952, SGI0, SGI0, SGI0, SGI71953, SGI71953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15938[] = {SGI71954, SGI0, SGI71955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15939[] = {SGI71956, SGI71957, SGI71958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71959, SGI0, SGI0, SGI0, SGI71960, SGI71960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15940[] = {SGI71961, SGI71962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71963, SGI0, SGI0, SGI0, SGI71964, SGI71964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15941[] = {SGI71965, SGI71966, SGI71967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71968, SGI0, SGI0, SGI0, SGI71969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15942[] = {SGI71970, SGI71971, SGI71972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71973, SGI0, SGI0, SGI0, SGI71974, SGI71974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15943[] = {SGI71975, SGI71976, SGI71977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71978, SGI0, SGI0, SGI0, SGI71979, SGI71979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15944[] = {SGI71980, SGI71981, SGI71982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71983, SGI0, SGI0, SGI0, SGI71984, SGI71984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15945[] = {SGI71985, SGI71986, SGI71987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71988, SGI0, SGI0, SGI0, SGI71989, SGI71989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15946[] = {SGI71990, SGI0, SGI71991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15947[] = {SGI71992, SGI71993, SGI71994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI71995, SGI0, SGI0, SGI0, SGI71996, SGI71996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15948[] = {SGI71997, SGI71998, SGI71999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72000, SGI0, SGI0, SGI0, SGI72001, SGI72001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15949[] = {SGI72002, SGI72003, SGI72004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72005, SGI0, SGI0, SGI0, SGI72006, SGI72006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15950[] = {SGI72007, SGI0, SGI72008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15951[] = {SGI72009, SGI72010, SGI72011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72012, SGI0, SGI0, SGI0, SGI72013, SGI72013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15952[] = {SGI72014, SGI72015, SGI72016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72017, SGI0, SGI0, SGI0, SGI72018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15953[] = {SGI72019, SGI0, SGI72020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72021, SGI0, SGI0, SGI0, SGI72022, SGI72022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15954[] = {SGI72023, SGI0, SGI72024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15955[] = {SGI72025, SGI72026, SGI72027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72028, SGI0, SGI0, SGI0, SGI72029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15956[] = {SGI72030, SGI0, SGI72031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15957[] = {SGI72032, SGI0, SGI72033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72034, SGI0, SGI0, SGI0, SGI72035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15958[] = {SGI72036, SGI72037, SGI72038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72039, SGI0, SGI0, SGI0, SGI72040, SGI72040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15959[] = {SGI72041, SGI72042, SGI72043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72044, SGI0, SGI0, SGI0, SGI72045, SGI72046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15960[] = {SGI72047, SGI72048, SGI72049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72050, SGI72050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15961[] = {SGI72051, SGI72052, SGI72053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15962[] = {SGI72054, SGI72055, SGI72056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15963[] = {SGI72057, SGI72058, SGI72059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15964[] = {SGI72060, SGI72061, SGI72062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72063, SGI0, SGI0, SGI0, SGI72064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15965[] = {SGI72065, SGI72066, SGI72067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72068, SGI72068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15966[] = {SGI72069, SGI72070, SGI72071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15967[] = {SGI72072, SGI72073, SGI72074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15968[] = {SGI72075, SGI72076, SGI72077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72078, SGI0, SGI0, SGI0, SGI72079, SGI72080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15969[] = {SGI72081, SGI72082, SGI72083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15970[] = {SGI0, SGI72084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15971[] = {SGI72085, SGI72086, SGI72087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15972[] = {SGI72088, SGI72089, SGI72090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72091, SGI0, SGI0, SGI0, SGI72092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15973[] = {SGI72093, SGI72094, SGI72095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72096, SGI0, SGI0, SGI0, SGI72097, SGI72097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15974[] = {SGI72098, SGI72099, SGI72100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72101, SGI0, SGI0, SGI0, SGI72102, SGI72102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15975[] = {SGI72103, SGI0, SGI72104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15976[] = {SGI72105, SGI72106, SGI72107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72108, SGI0, SGI0, SGI0, SGI72109, SGI72109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15977[] = {SGI72110, SGI72111, SGI72112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72113, SGI0, SGI0, SGI0, SGI72114, SGI72114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15978[] = {SGI72115, SGI72116, SGI72117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72118, SGI0, SGI0, SGI0, SGI72119, SGI72119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15979[] = {SGI72120, SGI72121, SGI72122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72123, SGI72124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15980[] = {SGI72125, SGI72126, SGI72127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72128, SGI0, SGI0, SGI0, SGI72129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15981[] = {SGI72130, SGI72131, SGI72132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72133, SGI0, SGI0, SGI0, SGI72134, SGI72134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15982[] = {SGI72135, SGI72136, SGI72137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72138, SGI72138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15983[] = {SGI72139, SGI72140, SGI72141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72142, SGI0, SGI0, SGI0, SGI72143, SGI72143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15984[] = {SGI72144, SGI0, SGI72145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15985[] = {SGI72146, SGI72147, SGI72148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72149, SGI0, SGI0, SGI0, SGI72150, SGI72150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15986[] = {SGI72151, SGI72152, SGI72153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72154, SGI0, SGI0, SGI0, SGI72155, SGI72155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15987[] = {SGI72156, SGI72157, SGI72158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72159, SGI0, SGI0, SGI0, SGI72160, SGI72160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15988[] = {SGI72161, SGI72162, SGI72163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72164, SGI0, SGI0, SGI0, SGI72165, SGI72165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15989[] = {SGI72166, SGI72167, SGI72168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72169, SGI0, SGI0, SGI0, SGI72170, SGI72170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15990[] = {SGI72171, SGI72172, SGI72173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72174, SGI0, SGI0, SGI0, SGI72175, SGI72175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15991[] = {SGI72176, SGI72177, SGI72178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72179, SGI72179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15992[] = {SGI72180, SGI72181, SGI72182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72183, SGI0, SGI0, SGI0, SGI72184, SGI72184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15993[] = {SGI72185, SGI72186, SGI72187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72188, SGI0, SGI0, SGI0, SGI72189, SGI72189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15994[] = {SGI72190, SGI0, SGI72191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15995[] = {SGI72192, SGI72193, SGI72194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72195, SGI0, SGI0, SGI0, SGI72196, SGI72196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15996[] = {SGI72197, SGI72198, SGI72199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72200, SGI0, SGI0, SGI0, SGI72201, SGI72201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15997[] = {SGI72202, SGI72203, SGI72204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72205, SGI0, SGI0, SGI0, SGI72206, SGI72206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15998[] = {SGI72207, SGI72208, SGI72209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72210, SGI0, SGI0, SGI0, SGI72211, SGI72211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG15999[] = {SGI72212, SGI72213, SGI72214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72215, SGI0, SGI0, SGI0, SGI72216, SGI72217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16000[] = {SGI72218, SGI72219, SGI72220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72221, SGI0, SGI0, SGI0, SGI72222, SGI72222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16001[] = {SGI72223, SGI72224, SGI72225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72226, SGI0, SGI0, SGI0, SGI72227, SGI72227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16002[] = {SGI72228, SGI72229, SGI72230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72231, SGI0, SGI0, SGI0, SGI72232, SGI72232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16003[] = {SGI72233, SGI72234, SGI72235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72236, SGI0, SGI0, SGI0, SGI72237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16004[] = {SGI72238, SGI72239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72240, SGI0, SGI0, SGI0, SGI72241, SGI72241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16005[] = {SGI72242, SGI72243, SGI72244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72245, SGI0, SGI0, SGI0, SGI72246, SGI72246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16006[] = {SGI72247, SGI72248, SGI72249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72250, SGI0, SGI0, SGI0, SGI72251, SGI72251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16007[] = {SGI72252, SGI72253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72254, SGI0, SGI0, SGI0, SGI72255, SGI72255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16008[] = {SGI72256, SGI72257, SGI72258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72259, SGI0, SGI0, SGI0, SGI72260, SGI72260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16009[] = {SGI72261, SGI72262, SGI72263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72264, SGI0, SGI0, SGI0, SGI72265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16010[] = {SGI72266, SGI72267, SGI72268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72269, SGI0, SGI0, SGI0, SGI72270, SGI72270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16011[] = {SGI72271, SGI72272, SGI72273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72274, SGI0, SGI0, SGI0, SGI72275, SGI72275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16012[] = {SGI72276, SGI72277, SGI72278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72279, SGI0, SGI0, SGI0, SGI72280, SGI72280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16013[] = {SGI72281, SGI72282, SGI72283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72284, SGI0, SGI0, SGI0, SGI72285, SGI72285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16014[] = {SGI72286, SGI0, SGI72287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16015[] = {SGI72288, SGI72289, SGI72290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72291, SGI0, SGI0, SGI0, SGI72292, SGI72292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16016[] = {SGI72293, SGI72294, SGI72295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72296, SGI0, SGI0, SGI0, SGI72297, SGI72297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16017[] = {SGI72298, SGI72299, SGI72300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72301, SGI0, SGI0, SGI0, SGI72302, SGI72302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16018[] = {SGI72303, SGI72304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72305, SGI0, SGI0, SGI0, SGI72306, SGI72306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16019[] = {SGI72307, SGI72308, SGI72309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72310, SGI0, SGI0, SGI0, SGI72311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16020[] = {SGI72312, SGI72313, SGI72314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72315, SGI0, SGI0, SGI0, SGI72316, SGI72317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16021[] = {SGI72318, SGI0, SGI72319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72320, SGI0, SGI0, SGI0, SGI72321, SGI72321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16022[] = {SGI72322, SGI72323, SGI72324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72325, SGI0, SGI0, SGI0, SGI72326, SGI72326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16023[] = {SGI72327, SGI0, SGI72328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16024[] = {SGI72329, SGI72330, SGI72331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72332, SGI0, SGI0, SGI0, SGI72333, SGI72333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16025[] = {SGI72334, SGI72335, SGI72336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16026[] = {SGI72337, SGI0, SGI72338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16027[] = {SGI72339, SGI72340, SGI72341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72342, SGI72342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16028[] = {SGI72343, SGI72344, SGI72345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72346, SGI0, SGI0, SGI0, SGI72347, SGI72347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16029[] = {SGI72348, SGI72349, SGI72350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72351, SGI0, SGI0, SGI0, SGI72352, SGI72352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16030[] = {SGI72353, SGI72354, SGI72355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72356, SGI0, SGI0, SGI0, SGI72357, SGI72357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16031[] = {SGI72358, SGI72359, SGI72360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72361, SGI0, SGI0, SGI0, SGI72362, SGI72362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16032[] = {SGI72363, SGI72364, SGI72365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72366, SGI0, SGI0, SGI0, SGI72367, SGI72367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16033[] = {SGI72368, SGI72369, SGI72370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72371, SGI0, SGI0, SGI0, SGI72372, SGI72372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16034[] = {SGI72373, SGI72374, SGI72375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72376, SGI0, SGI0, SGI0, SGI72377, SGI72377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16035[] = {SGI72378, SGI72379, SGI72380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72381, SGI0, SGI0, SGI0, SGI72382, SGI72382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16036[] = {SGI72383, SGI72384, SGI72385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72386, SGI72387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16037[] = {SGI72388, SGI72389, SGI72390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72391, SGI0, SGI0, SGI0, SGI72392, SGI72392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16038[] = {SGI72393, SGI72394, SGI72395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72396, SGI0, SGI0, SGI0, SGI72397, SGI72397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16039[] = {SGI72398, SGI72399, SGI72400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72401, SGI0, SGI0, SGI0, SGI72402, SGI72402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16040[] = {SGI72403, SGI72404, SGI72405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72406, SGI0, SGI0, SGI0, SGI72407, SGI72407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16041[] = {SGI72408, SGI72409, SGI72410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72411, SGI0, SGI0, SGI0, SGI72412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16042[] = {SGI72413, SGI72414, SGI72415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72416, SGI0, SGI0, SGI0, SGI72417, SGI72417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16043[] = {SGI72418, SGI72419, SGI72420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72421, SGI0, SGI0, SGI0, SGI72422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16044[] = {SGI72423, SGI72424, SGI72425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72426, SGI72427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16045[] = {SGI72428, SGI72429, SGI72430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72431, SGI0, SGI0, SGI0, SGI72432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16046[] = {SGI72433, SGI72434, SGI72435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72436, SGI72436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16047[] = {SGI72437, SGI0, SGI72438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16048[] = {SGI72439, SGI72440, SGI72441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72442, SGI72443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16049[] = {SGI72444, SGI72445, SGI72446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16050[] = {SGI72447, SGI72448, SGI72449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72452, SGI72452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16051[] = {SGI72450, SGI72451, SGI72453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72454, SGI0, SGI0, SGI0, SGI72455, SGI72455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16052[] = {SGI72456, SGI72457, SGI72458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72460, SGI0, SGI0, SGI0, SGI72463, SGI72463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16053[] = {SGI72459, SGI72461, SGI72462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72464, SGI0, SGI0, SGI0, SGI72465, SGI72466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16054[] = {SGI72467, SGI72468, SGI72469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72471, SGI0, SGI0, SGI0, SGI72473, SGI72473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16055[] = {SGI72470, SGI0, SGI72472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72474, SGI0, SGI0, SGI0, SGI72475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16056[] = {SGI72476, SGI72477, SGI72478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72482, SGI0, SGI0, SGI0, SGI72483, SGI72485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16057[] = {SGI72479, SGI72480, SGI72481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72484, SGI0, SGI0, SGI0, SGI72486, SGI72487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16058[] = {SGI72489, SGI72491, SGI72493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16059[] = {SGI72488, SGI72490, SGI72492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72494, SGI0, SGI0, SGI0, SGI72495, SGI72495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16060[] = {SGI0, SGI72496, SGI72497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16061[] = {SGI72498, SGI0, SGI72499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16062[] = {SGI72503, SGI0, SGI72504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16063[] = {SGI72500, SGI72501, SGI72502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72505, SGI72505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16064[] = {SGI72507, SGI72508, SGI72510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16065[] = {SGI72506, SGI0, SGI72509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72511, SGI0, SGI0, SGI0, SGI72512, SGI72512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16066[] = {SGI72513, SGI72515, SGI72517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72519, SGI0, SGI0, SGI0, SGI72521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16067[] = {SGI72514, SGI72516, SGI72518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72520, SGI0, SGI0, SGI0, SGI72522, SGI72523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16068[] = {SGI72524, SGI0, SGI72526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72529, SGI0, SGI0, SGI0, SGI72530, SGI72530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16069[] = {SGI72525, SGI72527, SGI72528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72531, SGI0, SGI0, SGI0, SGI72532, SGI72532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16070[] = {SGI72533, SGI72534, SGI72535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16071[] = {SGI72536, SGI72537, SGI72538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72539, SGI0, SGI0, SGI0, SGI72540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16072[] = {SGI72541, SGI72542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16073[] = {SGI72543, SGI72544, SGI72545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72546, SGI0, SGI0, SGI0, SGI72547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16074[] = {SGI72548, SGI72549, SGI72550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16075[] = {SGI72551, SGI0, SGI72552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16076[] = {SGI72553, SGI72555, SGI72557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16077[] = {SGI72554, SGI72556, SGI72558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72559, SGI0, SGI0, SGI0, SGI72560, SGI72561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16078[] = {SGI72562, SGI72563, SGI72564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16079[] = {SGI72565, SGI0, SGI72566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16080[] = {SGI72567, SGI72568, SGI72569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16081[] = {SGI72570, SGI72571, SGI72572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72573, SGI0, SGI0, SGI0, SGI72574, SGI72574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16082[] = {SGI72575, SGI72576, SGI72577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72578, SGI0, SGI0, SGI0, SGI72579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16083[] = {SGI0, SGI72580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16084[] = {SGI72581, SGI72582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72583, SGI0, SGI0, SGI0, SGI72584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16085[] = {SGI72585, SGI0, SGI72586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72587, SGI0, SGI0, SGI0, SGI72588, SGI72588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16086[] = {SGI72589, SGI0, SGI72590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72591, SGI0, SGI0, SGI0, SGI72592, SGI72593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16087[] = {SGI72594, SGI72595, SGI72596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72597, SGI0, SGI0, SGI0, SGI72598, SGI72598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16088[] = {SGI72599, SGI72600, SGI72601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72602, SGI0, SGI0, SGI0, SGI72603, SGI72603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16089[] = {SGI72604, SGI72605, SGI72606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72607, SGI0, SGI0, SGI0, SGI72608, SGI72608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16090[] = {SGI72609, SGI72610, SGI72611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72612, SGI0, SGI0, SGI0, SGI72613, SGI72613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16091[] = {SGI72614, SGI72615, SGI72616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72617, SGI0, SGI0, SGI0, SGI72618, SGI72618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16092[] = {SGI72619, SGI72620, SGI72621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72622, SGI0, SGI0, SGI0, SGI72623, SGI72623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16093[] = {SGI72624, SGI0, SGI72625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16094[] = {SGI72626, SGI0, SGI72627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16095[] = {SGI72628, SGI72629, SGI72630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72631, SGI0, SGI0, SGI0, SGI72632, SGI72632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16096[] = {SGI72633, SGI72634, SGI72635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72639, SGI0, SGI0, SGI0, SGI72640, SGI72640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16097[] = {SGI72636, SGI72637, SGI72638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72641, SGI0, SGI0, SGI0, SGI72642, SGI72642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16098[] = {SGI72643, SGI72644, SGI72645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72646, SGI72647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16099[] = {SGI72648, SGI72649, SGI72650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72651, SGI0, SGI0, SGI0, SGI72652, SGI72652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16100[] = {SGI72653, SGI72654, SGI72655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72656, SGI0, SGI0, SGI0, SGI72657, SGI72657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16101[] = {SGI72658, SGI72659, SGI72660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72661, SGI0, SGI0, SGI0, SGI72662, SGI72662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16102[] = {SGI72663, SGI72664, SGI72665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72666, SGI0, SGI0, SGI0, SGI72667, SGI72667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16103[] = {SGI72668, SGI72669, SGI72670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72671, SGI0, SGI0, SGI0, SGI72672, SGI72672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16104[] = {SGI72673, SGI72674, SGI72675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72676, SGI0, SGI0, SGI0, SGI72677, SGI72677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16105[] = {SGI72678, SGI72679, SGI72680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72681, SGI0, SGI0, SGI0, SGI72682, SGI72682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16106[] = {SGI72683, SGI72684, SGI72685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72686, SGI0, SGI0, SGI0, SGI72687, SGI72687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16107[] = {SGI72688, SGI72689, SGI72690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72691, SGI0, SGI0, SGI0, SGI72692, SGI72692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16108[] = {SGI72693, SGI72694, SGI72695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72696, SGI72696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16109[] = {SGI72697, SGI72698, SGI72699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72700, SGI0, SGI0, SGI0, SGI72701, SGI72701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16110[] = {SGI72702, SGI72703, SGI72704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72705, SGI0, SGI0, SGI0, SGI72706, SGI72706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16111[] = {SGI72707, SGI72708, SGI72709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72710, SGI0, SGI0, SGI0, SGI72711, SGI72711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16112[] = {SGI72712, SGI72713, SGI72714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72715, SGI0, SGI0, SGI0, SGI72716, SGI72716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16113[] = {SGI72717, SGI72718, SGI72719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72720, SGI0, SGI0, SGI0, SGI72724, SGI72724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16114[] = {SGI72721, SGI72722, SGI72723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72725, SGI0, SGI0, SGI0, SGI72726, SGI72726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16115[] = {SGI72727, SGI72728, SGI72729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72732, SGI0, SGI0, SGI0, SGI72734, SGI72734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16116[] = {SGI72730, SGI72731, SGI72733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72735, SGI0, SGI0, SGI0, SGI72736, SGI72736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16117[] = {SGI72737, SGI72738, SGI72739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72742, SGI0, SGI0, SGI0, SGI72744, SGI72744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16118[] = {SGI72740, SGI72741, SGI72743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72745, SGI0, SGI0, SGI0, SGI72746, SGI72746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16119[] = {SGI72747, SGI72748, SGI72749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72751, SGI0, SGI0, SGI0, SGI72754, SGI72754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16120[] = {SGI72750, SGI72752, SGI72753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72755, SGI0, SGI0, SGI0, SGI72756, SGI72756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16121[] = {SGI72757, SGI72758, SGI72759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72760, SGI0, SGI0, SGI0, SGI72763, SGI72763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16122[] = {SGI72761, SGI72762, SGI72764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72765, SGI0, SGI0, SGI0, SGI72766, SGI72766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16123[] = {SGI72767, SGI72768, SGI72769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72770, SGI0, SGI0, SGI0, SGI72771, SGI72771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16124[] = {SGI72772, SGI72773, SGI72774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72776, SGI0, SGI0, SGI0, SGI72779, SGI72779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16125[] = {SGI72775, SGI72777, SGI72778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72780, SGI0, SGI0, SGI0, SGI72781, SGI72781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16126[] = {SGI72782, SGI72783, SGI72784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72785, SGI0, SGI0, SGI0, SGI72786, SGI72786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16127[] = {SGI72787, SGI72788, SGI72789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72790, SGI0, SGI0, SGI0, SGI72791, SGI72791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16128[] = {SGI72792, SGI72793, SGI72794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72795, SGI0, SGI0, SGI0, SGI72796, SGI72796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16129[] = {SGI72797, SGI72798, SGI72799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72800, SGI0, SGI0, SGI0, SGI72801, SGI72801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16130[] = {SGI72802, SGI72803, SGI72804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72805, SGI0, SGI0, SGI0, SGI72806, SGI72806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16131[] = {SGI72807, SGI72808, SGI72809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72810, SGI0, SGI0, SGI0, SGI72811, SGI72811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16132[] = {SGI72812, SGI72813, SGI72814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72815, SGI0, SGI0, SGI0, SGI72816, SGI72816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16133[] = {SGI72817, SGI72818, SGI72819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72820, SGI0, SGI0, SGI0, SGI72821, SGI72821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16134[] = {SGI72822, SGI72823, SGI72824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72825, SGI0, SGI0, SGI0, SGI72826, SGI72826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16135[] = {SGI72827, SGI72828, SGI72829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72830, SGI72831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16136[] = {SGI72832, SGI72833, SGI72834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72835, SGI0, SGI0, SGI0, SGI72836, SGI72836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16137[] = {SGI72837, SGI72838, SGI72839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72840, SGI72840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16138[] = {SGI72841, SGI72842, SGI72843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72844, SGI0, SGI0, SGI0, SGI72845, SGI72845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16139[] = {SGI72846, SGI72847, SGI72848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72849, SGI0, SGI0, SGI0, SGI72850, SGI72850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16140[] = {SGI72851, SGI0, SGI72852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16141[] = {SGI72853, SGI72854, SGI72855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72856, SGI0, SGI0, SGI0, SGI72857, SGI72857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16142[] = {SGI72858, SGI72859, SGI72860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72861, SGI72861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16143[] = {SGI72862, SGI72863, SGI72864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72865, SGI0, SGI0, SGI0, SGI72866, SGI72866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16144[] = {SGI72867, SGI72868, SGI72869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72870, SGI0, SGI0, SGI0, SGI72871, SGI72871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16145[] = {SGI72872, SGI72873, SGI72874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72875, SGI0, SGI0, SGI0, SGI72876, SGI72876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16146[] = {SGI72877, SGI72878, SGI72879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72880, SGI0, SGI0, SGI0, SGI72881, SGI72881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16147[] = {SGI72882, SGI72883, SGI72884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72885, SGI72885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16148[] = {SGI72886, SGI72887, SGI72888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72889, SGI0, SGI0, SGI0, SGI72890, SGI72890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16149[] = {SGI72891, SGI72892, SGI72893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72897, SGI72897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16150[] = {SGI72894, SGI72895, SGI72896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72898, SGI0, SGI0, SGI0, SGI72899, SGI72899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16151[] = {SGI72900, SGI0, SGI72901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72902, SGI0, SGI0, SGI0, SGI72903, SGI72903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16152[] = {SGI72904, SGI72905, SGI72906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72908, SGI0, SGI0, SGI0, SGI72910, SGI72910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16153[] = {SGI72907, SGI72909, SGI72911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72912, SGI0, SGI0, SGI0, SGI72913, SGI72913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16154[] = {SGI72914, SGI0, SGI72915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72916, SGI0, SGI0, SGI0, SGI72917, SGI72917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16155[] = {SGI72918, SGI72919, SGI72920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72921, SGI0, SGI0, SGI0, SGI72922, SGI72922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16156[] = {SGI72923, SGI72924, SGI72925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72926, SGI0, SGI0, SGI0, SGI72927, SGI72927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16157[] = {SGI72928, SGI72929, SGI72930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72931, SGI0, SGI0, SGI0, SGI72932, SGI72932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16158[] = {SGI72933, SGI72934, SGI72935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72936, SGI0, SGI0, SGI0, SGI72937, SGI72937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16159[] = {SGI72938, SGI72939, SGI72940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72941, SGI0, SGI0, SGI0, SGI72942, SGI72942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16160[] = {SGI72943, SGI72944, SGI72945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72946, SGI0, SGI0, SGI0, SGI72947, SGI72947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16161[] = {SGI72948, SGI0, SGI72949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16162[] = {SGI72950, SGI72951, SGI72952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72953, SGI0, SGI0, SGI0, SGI72954, SGI72954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16163[] = {SGI72955, SGI72956, SGI72957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72958, SGI0, SGI0, SGI0, SGI72959, SGI72959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16164[] = {SGI72960, SGI72961, SGI72962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72963, SGI0, SGI0, SGI0, SGI72964, SGI72964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16165[] = {SGI72965, SGI72966, SGI72967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72968, SGI0, SGI0, SGI0, SGI72969, SGI72969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16166[] = {SGI72970, SGI72971, SGI72972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72973, SGI0, SGI0, SGI0, SGI72974, SGI72974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16167[] = {SGI72975, SGI72976, SGI72977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72978, SGI0, SGI0, SGI0, SGI72979, SGI72979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16168[] = {SGI72980, SGI72981, SGI72982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72983, SGI0, SGI0, SGI0, SGI72984, SGI72984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16169[] = {SGI72985, SGI72986, SGI72987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72988, SGI0, SGI0, SGI0, SGI72989, SGI72989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16170[] = {SGI72990, SGI72991, SGI72992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72993, SGI0, SGI0, SGI0, SGI72994, SGI72994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16171[] = {SGI72995, SGI0, SGI72996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI72997, SGI0, SGI0, SGI0, SGI72998, SGI72998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16172[] = {SGI72999, SGI73000, SGI73001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73002, SGI0, SGI0, SGI0, SGI73003, SGI73003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16173[] = {SGI73006, SGI73008, SGI73009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73011, SGI0, SGI0, SGI0, SGI73012, SGI73012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16174[] = {SGI73013, SGI73014, SGI73015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73016, SGI0, SGI0, SGI0, SGI73017, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16175[] = {SGI73018, SGI73019, SGI73020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73021, SGI0, SGI0, SGI0, SGI73022, SGI73022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16176[] = {SGI73023, SGI73024, SGI73025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73026, SGI0, SGI0, SGI0, SGI73027, SGI73027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16177[] = {SGI73028, SGI73029, SGI73030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73031, SGI0, SGI0, SGI0, SGI73032, SGI73032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16178[] = {SGI73033, SGI0, SGI73034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16179[] = {SGI73035, SGI73036, SGI73037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73038, SGI0, SGI0, SGI0, SGI73039, SGI73039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16180[] = {SGI73040, SGI73041, SGI73042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73043, SGI0, SGI0, SGI0, SGI73044, SGI73044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16181[] = {SGI73045, SGI73046, SGI73047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73048, SGI0, SGI0, SGI0, SGI73049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16182[] = {SGI73050, SGI73051, SGI73052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73053, SGI73054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16183[] = {SGI73055, SGI0, SGI73056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16184[] = {SGI73004, SGI73005, SGI73007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73010, SGI0, SGI0, SGI0, SGI73058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16185[] = {SGI73057, SGI73059, SGI73060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73061, SGI0, SGI0, SGI0, SGI73062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16186[] = {SGI73066, SGI0, SGI73067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16187[] = {SGI73063, SGI73064, SGI73065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73068, SGI0, SGI0, SGI0, SGI73069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16188[] = {SGI73070, SGI73071, SGI73072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73073, SGI0, SGI0, SGI0, SGI73074, SGI73075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16189[] = {SGI73076, SGI73077, SGI73078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73079, SGI0, SGI0, SGI0, SGI73081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16190[] = {SGI73080, SGI73082, SGI73083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73084, SGI73084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16191[] = {SGI73086, SGI0, SGI73088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16192[] = {SGI73085, SGI73087, SGI73089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73090, SGI0, SGI0, SGI0, SGI73091, SGI73091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16193[] = {SGI73092, SGI0, SGI73093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73094, SGI0, SGI0, SGI0, SGI73095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16194[] = {SGI73096, SGI73097, SGI73098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73099, SGI0, SGI0, SGI0, SGI73100, SGI73101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16195[] = {SGI0, SGI73102, SGI73103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16196[] = {SGI0, SGI73104, SGI73105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16197[] = {SGI73106, SGI73107, SGI73108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73109, SGI0, SGI0, SGI0, SGI73110, SGI73110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16198[] = {SGI73111, SGI73112, SGI73113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73114, SGI0, SGI0, SGI0, SGI73115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16199[] = {SGI73116, SGI73117, SGI73118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73119, SGI0, SGI0, SGI0, SGI73120, SGI73120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16200[] = {SGI73121, SGI0, SGI73122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16201[] = {SGI73123, SGI73124, SGI73125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73126, SGI73126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16202[] = {SGI73127, SGI73128, SGI73129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73130, SGI0, SGI0, SGI0, SGI73131, SGI73132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16203[] = {SGI73133, SGI73134, SGI73135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73136, SGI0, SGI0, SGI0, SGI73137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16204[] = {SGI73138, SGI0, SGI73139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73140, SGI0, SGI0, SGI0, SGI73141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16205[] = {SGI73142, SGI0, SGI73143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73145, SGI0, SGI0, SGI0, SGI73148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16206[] = {SGI73144, SGI73146, SGI73147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73149, SGI0, SGI0, SGI0, SGI73150, SGI73150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16207[] = {SGI73151, SGI73152, SGI73153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73154, SGI0, SGI0, SGI0, SGI73155, SGI73155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16208[] = {SGI0, SGI73156, SGI73157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16209[] = {SGI73159, SGI73161, SGI73163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73166, SGI73166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16210[] = {SGI73160, SGI73162, SGI73164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73165, SGI0, SGI0, SGI0, SGI73167, SGI73167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16211[] = {SGI73168, SGI0, SGI73169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16212[] = {SGI73170, SGI73171, SGI73172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73174, SGI0, SGI0, SGI0, SGI73177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16213[] = {SGI73173, SGI73175, SGI73176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73178, SGI0, SGI0, SGI0, SGI73179, SGI73179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16214[] = {SGI73181, SGI73182, SGI73184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73186, SGI73188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16215[] = {SGI73180, SGI73183, SGI73185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73187, SGI0, SGI0, SGI0, SGI73189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16216[] = {SGI73190, SGI73191, SGI73192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16217[] = {SGI73193, SGI73194, SGI73195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73196, SGI0, SGI0, SGI0, SGI73197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16218[] = {SGI73198, SGI73199, SGI73200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73201, SGI0, SGI0, SGI0, SGI73202, SGI73203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16219[] = {SGI73204, SGI73205, SGI73206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16220[] = {SGI73207, SGI73209, SGI73210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73213, SGI0, SGI0, SGI0, SGI73214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16221[] = {SGI73208, SGI73211, SGI73212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73215, SGI0, SGI0, SGI0, SGI73216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16222[] = {SGI73217, SGI73218, SGI73219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16223[] = {SGI73220, SGI73221, SGI73222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73223, SGI73223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16224[] = {SGI0, SGI73227, SGI73228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16225[] = {SGI73224, SGI73225, SGI73226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73229, SGI0, SGI0, SGI0, SGI73230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16226[] = {SGI73231, SGI73232, SGI73233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73234, SGI0, SGI0, SGI0, SGI73235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16227[] = {SGI73236, SGI73237, SGI73238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73239, SGI0, SGI0, SGI0, SGI73240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16228[] = {SGI73241, SGI73242, SGI73243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73244, SGI0, SGI0, SGI0, SGI73245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16229[] = {SGI73246, SGI73247, SGI73248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16230[] = {SGI0, SGI73249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16231[] = {SGI73250, SGI73251, SGI73252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73253, SGI0, SGI0, SGI0, SGI73254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16232[] = {SGI73255, SGI73256, SGI73257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73258, SGI0, SGI0, SGI0, SGI73259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16233[] = {SGI73260, SGI73261, SGI73262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16234[] = {SGI73263, SGI73264, SGI73265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16235[] = {SGI73266, SGI73267, SGI73268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16236[] = {SGI73269, SGI73270, SGI73271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16237[] = {SGI73273, SGI73274, SGI73275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16238[] = {SGI73276, SGI73277, SGI73278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73279, SGI0, SGI0, SGI0, SGI73280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16239[] = {SGI73281, SGI0, SGI73282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16240[] = {SGI73283, SGI73284, SGI73285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73286, SGI0, SGI0, SGI0, SGI73287, SGI73287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16241[] = {SGI73288, SGI73289, SGI73290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16242[] = {SGI73291, SGI73292, SGI73293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73294, SGI0, SGI0, SGI0, SGI73295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16243[] = {SGI73296, SGI73297, SGI73298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73299, SGI0, SGI0, SGI0, SGI73300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16244[] = {SGI73301, SGI73302, SGI73303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16245[] = {SGI73304, SGI73305, SGI73306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16246[] = {SGI73307, SGI73308, SGI73309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73310, SGI0, SGI0, SGI0, SGI73311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16247[] = {SGI73312, SGI73313, SGI73314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73315, SGI0, SGI0, SGI0, SGI73316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16248[] = {SGI73317, SGI0, SGI73318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73319, SGI0, SGI0, SGI0, SGI73320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16249[] = {SGI73321, SGI0, SGI73322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73323, SGI0, SGI0, SGI0, SGI73324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16250[] = {SGI73325, SGI73326, SGI73327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73328, SGI0, SGI0, SGI0, SGI73329, SGI73329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16251[] = {SGI73330, SGI73331, SGI73332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73333, SGI0, SGI0, SGI0, SGI73334, SGI73334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16252[] = {SGI73335, SGI73336, SGI73337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73338, SGI0, SGI0, SGI0, SGI73339, SGI73339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16253[] = {SGI73340, SGI73341, SGI73342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73343, SGI0, SGI0, SGI0, SGI73344, SGI73344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16254[] = {SGI73345, SGI0, SGI73346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16255[] = {SGI73347, SGI73348, SGI73349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73350, SGI0, SGI0, SGI0, SGI73351, SGI73351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16256[] = {SGI73352, SGI73353, SGI73354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73355, SGI0, SGI0, SGI0, SGI73356, SGI73356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16257[] = {SGI73357, SGI73358, SGI73359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73360, SGI0, SGI0, SGI0, SGI73361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16258[] = {SGI73362, SGI73363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73364, SGI0, SGI0, SGI0, SGI73365, SGI73365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16259[] = {SGI73366, SGI73367, SGI73368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73369, SGI0, SGI0, SGI0, SGI73370, SGI73370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16260[] = {SGI73371, SGI0, SGI73372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16261[] = {SGI73373, SGI73374, SGI73375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73376, SGI0, SGI0, SGI0, SGI73377, SGI73377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16262[] = {SGI73378, SGI73379, SGI73380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73381, SGI0, SGI0, SGI0, SGI73382, SGI73382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16263[] = {SGI73383, SGI73384, SGI73385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73386, SGI0, SGI0, SGI0, SGI73387, SGI73387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16264[] = {SGI73388, SGI73389, SGI73390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73391, SGI0, SGI0, SGI0, SGI73392, SGI73392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16265[] = {SGI73393, SGI0, SGI73394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16266[] = {SGI73395, SGI0, SGI73396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16267[] = {SGI73397, SGI73398, SGI73399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73400, SGI0, SGI0, SGI0, SGI73401, SGI73401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16268[] = {SGI73402, SGI73403, SGI73404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73405, SGI0, SGI0, SGI0, SGI73406, SGI73406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16269[] = {SGI73407, SGI73408, SGI73409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73410, SGI0, SGI0, SGI0, SGI73411, SGI73411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16270[] = {SGI73412, SGI73413, SGI73414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73415, SGI0, SGI0, SGI0, SGI73416, SGI73416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16271[] = {SGI73417, SGI73418, SGI73419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73420, SGI0, SGI0, SGI0, SGI73421, SGI73421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16272[] = {SGI73422, SGI73423, SGI73424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73425, SGI0, SGI0, SGI0, SGI73426, SGI73426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16273[] = {SGI73427, SGI73428, SGI73429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73430, SGI0, SGI0, SGI0, SGI73431, SGI73431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16274[] = {SGI73432, SGI73433, SGI73434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73435, SGI0, SGI0, SGI0, SGI73436, SGI73436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16275[] = {SGI73437, SGI0, SGI73438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16276[] = {SGI73439, SGI73440, SGI73442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73445, SGI0, SGI0, SGI0, SGI73446, SGI73446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16277[] = {SGI73441, SGI73443, SGI73444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73447, SGI73448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16278[] = {SGI73449, SGI0, SGI73450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16279[] = {SGI73451, SGI73452, SGI73453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73454, SGI0, SGI0, SGI0, SGI73455, SGI73455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16280[] = {SGI73456, SGI73457, SGI73458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73459, SGI0, SGI0, SGI0, SGI73460, SGI73460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16281[] = {SGI73461, SGI73462, SGI73463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73464, SGI73464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16282[] = {SGI73465, SGI73466, SGI73467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73468, SGI0, SGI0, SGI0, SGI73469, SGI73469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16283[] = {SGI73470, SGI73471, SGI73472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73473, SGI73473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16284[] = {SGI73474, SGI0, SGI73476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16285[] = {SGI73475, SGI0, SGI73477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73478, SGI0, SGI0, SGI0, SGI73479, SGI73479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16286[] = {SGI73482, SGI0, SGI73484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16287[] = {SGI73480, SGI73481, SGI73483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73485, SGI0, SGI0, SGI0, SGI73486, SGI73486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16288[] = {SGI73487, SGI73488, SGI73489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73493, SGI73494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16289[] = {SGI73490, SGI73491, SGI73492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73495, SGI0, SGI0, SGI0, SGI73496, SGI73496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16290[] = {SGI73497, SGI0, SGI73498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16291[] = {SGI73500, SGI0, SGI73502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73504, SGI0, SGI0, SGI0, SGI73506, SGI73506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16292[] = {SGI73499, SGI73501, SGI73503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73505, SGI0, SGI0, SGI0, SGI73507, SGI73507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16293[] = {SGI73508, SGI73509, SGI73510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73511, SGI0, SGI0, SGI0, SGI73512, SGI73512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16294[] = {SGI73513, SGI73514, SGI73515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16295[] = {SGI73516, SGI0, SGI73517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73518, SGI0, SGI0, SGI0, SGI73519, SGI73519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16296[] = {SGI73520, SGI0, SGI73521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73522, SGI0, SGI0, SGI0, SGI73523, SGI73523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16297[] = {SGI73524, SGI0, SGI73525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16298[] = {SGI73526, SGI73527, SGI73528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73529, SGI0, SGI0, SGI0, SGI73530, SGI73530, SGI0, SGI0, NULL};
const Segment * const SG16299[] = {SGI73531, SGI73532, SGI73533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73534, SGI0, SGI0, SGI0, SGI73535, SGI73535, SGI0, SGI0, NULL};
const Segment * const SG16300[] = {SGI73536, SGI73537, SGI73538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73539, SGI0, SGI0, SGI0, SGI73540, SGI73540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16301[] = {SGI73541, SGI0, SGI73542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73543, SGI0, SGI0, SGI0, SGI73544, SGI73544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16302[] = {SGI73545, SGI0, SGI73546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16303[] = {SGI73547, SGI0, SGI73548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16304[] = {SGI73549, SGI73550, SGI73551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73552, SGI0, SGI0, SGI0, SGI73553, SGI73553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16305[] = {SGI73554, SGI73555, SGI73556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73557, SGI0, SGI0, SGI0, SGI73558, SGI73558, SGI0, SGI73559, SGI0, NULL};
const Segment * const SG16306[] = {SGI73560, SGI73561, SGI73562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73563, SGI0, SGI0, SGI0, SGI73564, SGI73564, SGI0, SGI73565, SGI0, NULL};
const Segment * const SG16307[] = {SGI73566, SGI73567, SGI73568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73569, SGI0, SGI0, SGI0, SGI73570, SGI73570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16308[] = {SGI73571, SGI0, SGI73572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16309[] = {SGI73573, SGI73574, SGI73575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73576, SGI0, SGI0, SGI0, SGI73577, SGI73577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16310[] = {SGI73578, SGI73579, SGI73580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73581, SGI0, SGI0, SGI0, SGI73582, SGI73582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16311[] = {SGI73583, SGI73584, SGI73585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73586, SGI0, SGI0, SGI0, SGI73587, SGI73587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16312[] = {SGI73588, SGI73589, SGI73590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73591, SGI0, SGI0, SGI0, SGI73592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16313[] = {SGI73593, SGI73594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73595, SGI0, SGI0, SGI0, SGI73596, SGI73596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16314[] = {SGI73597, SGI0, SGI73598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73599, SGI0, SGI0, SGI0, SGI73600, SGI73600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16315[] = {SGI73601, SGI73602, SGI73603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73604, SGI0, SGI0, SGI0, SGI73605, SGI73605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16316[] = {SGI73606, SGI0, SGI73607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16317[] = {SGI73608, SGI73609, SGI73610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73611, SGI0, SGI0, SGI0, SGI73612, SGI73612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16318[] = {SGI73613, SGI73614, SGI73615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73616, SGI0, SGI0, SGI0, SGI73617, SGI73617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16319[] = {SGI73618, SGI0, SGI73619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73620, SGI0, SGI0, SGI0, SGI73621, SGI73621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16320[] = {SGI73622, SGI73623, SGI73624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73625, SGI73625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16321[] = {SGI73626, SGI73627, SGI73628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73629, SGI73630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16322[] = {SGI73631, SGI73632, SGI73633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73634, SGI0, SGI0, SGI0, SGI73635, SGI73635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16323[] = {SGI73636, SGI73637, SGI73638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73639, SGI73639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16324[] = {SGI73640, SGI0, SGI73641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16325[] = {SGI73642, SGI73643, SGI73644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73645, SGI0, SGI0, SGI0, SGI73646, SGI73646, SGI0, SGI73647, SGI0, NULL};
const Segment * const SG16326[] = {SGI73648, SGI73649, SGI73650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73651, SGI0, SGI0, SGI0, SGI73652, SGI73652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16327[] = {SGI73653, SGI0, SGI73654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16328[] = {SGI73655, SGI0, SGI73656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73657, SGI0, SGI0, SGI0, SGI73658, SGI73658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16329[] = {SGI73659, SGI73660, SGI73661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73662, SGI0, SGI0, SGI0, SGI73663, SGI73663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16330[] = {SGI73664, SGI0, SGI73665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16331[] = {SGI73666, SGI73667, SGI73668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73669, SGI0, SGI0, SGI0, SGI73670, SGI73670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16332[] = {SGI73671, SGI73672, SGI73673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73674, SGI0, SGI0, SGI0, SGI73675, SGI73675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16333[] = {SGI73676, SGI73677, SGI73678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73679, SGI0, SGI0, SGI0, SGI73680, SGI73680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16334[] = {SGI73681, SGI73682, SGI73683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73684, SGI0, SGI0, SGI0, SGI73685, SGI73685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16335[] = {SGI73686, SGI73687, SGI73688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73689, SGI0, SGI0, SGI0, SGI73690, SGI73690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16336[] = {SGI73691, SGI73692, SGI73693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73694, SGI0, SGI0, SGI0, SGI73695, SGI73695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16337[] = {SGI73696, SGI73697, SGI73698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73699, SGI0, SGI0, SGI0, SGI73700, SGI73700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16338[] = {SGI73701, SGI73702, SGI73703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73704, SGI0, SGI0, SGI0, SGI73705, SGI73705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16339[] = {SGI73706, SGI73707, SGI73708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73709, SGI0, SGI0, SGI0, SGI73710, SGI73710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16340[] = {SGI73711, SGI0, SGI73712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16341[] = {SGI73713, SGI73714, SGI73715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73716, SGI0, SGI0, SGI0, SGI73717, SGI73717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16342[] = {SGI73718, SGI73719, SGI73720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73721, SGI0, SGI0, SGI0, SGI73722, SGI73722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16343[] = {SGI73723, SGI73724, SGI73725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73726, SGI0, SGI0, SGI0, SGI73727, SGI73727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16344[] = {SGI73728, SGI73729, SGI73730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73731, SGI0, SGI0, SGI0, SGI73732, SGI73732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16345[] = {SGI73733, SGI73734, SGI73735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73736, SGI0, SGI0, SGI0, SGI73737, SGI73737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16346[] = {SGI73738, SGI73739, SGI73740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73741, SGI0, SGI0, SGI0, SGI73742, SGI73742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16347[] = {SGI73743, SGI73744, SGI73745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73746, SGI0, SGI0, SGI0, SGI73747, SGI73747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16348[] = {SGI73748, SGI73749, SGI73750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73751, SGI0, SGI0, SGI0, SGI73752, SGI73752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16349[] = {SGI73753, SGI73754, SGI73755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73756, SGI0, SGI0, SGI0, SGI73757, SGI73757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16350[] = {SGI73758, SGI73759, SGI73760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73761, SGI0, SGI0, SGI0, SGI73762, SGI73762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16351[] = {SGI73763, SGI73764, SGI73765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73766, SGI0, SGI0, SGI0, SGI73767, SGI73767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16352[] = {SGI73768, SGI73769, SGI73770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73771, SGI0, SGI0, SGI0, SGI73772, SGI73772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16353[] = {SGI73773, SGI73774, SGI73775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73776, SGI0, SGI0, SGI0, SGI73777, SGI73777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16354[] = {SGI73778, SGI73779, SGI73780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73781, SGI0, SGI0, SGI0, SGI73782, SGI73782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16355[] = {SGI73783, SGI73784, SGI73785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73786, SGI0, SGI0, SGI0, SGI73787, SGI73787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16356[] = {SGI73788, SGI73789, SGI73790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73791, SGI0, SGI0, SGI0, SGI73792, SGI73792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16357[] = {SGI73793, SGI73794, SGI73795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73796, SGI73796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16358[] = {SGI73797, SGI73798, SGI73800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73803, SGI0, SGI0, SGI0, SGI73805, SGI73805, SGI0, SGI73807, SGI0, NULL};
const Segment * const SG16359[] = {SGI73799, SGI73801, SGI73802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73804, SGI0, SGI0, SGI0, SGI73806, SGI73806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16360[] = {SGI73808, SGI0, SGI73809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16361[] = {SGI73810, SGI73811, SGI73812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73813, SGI0, SGI0, SGI0, SGI73814, SGI73814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16362[] = {SGI73815, SGI73816, SGI73817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73818, SGI0, SGI0, SGI0, SGI73819, SGI73819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16363[] = {SGI73820, SGI73821, SGI73822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73823, SGI73823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16364[] = {SGI73824, SGI0, SGI73825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73826, SGI0, SGI0, SGI0, SGI73827, SGI73827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16365[] = {SGI73828, SGI73829, SGI73830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73831, SGI0, SGI0, SGI0, SGI73832, SGI73832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16366[] = {SGI73833, SGI73834, SGI73835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73836, SGI0, SGI0, SGI0, SGI73837, SGI73837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16367[] = {SGI73838, SGI73839, SGI73840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73841, SGI0, SGI0, SGI0, SGI73842, SGI73842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16368[] = {SGI73843, SGI0, SGI73844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16369[] = {SGI73845, SGI73846, SGI73847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73848, SGI0, SGI0, SGI0, SGI73849, SGI73849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16370[] = {SGI73850, SGI0, SGI73851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73852, SGI0, SGI0, SGI0, SGI73853, SGI73853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16371[] = {SGI73854, SGI73855, SGI73856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73857, SGI73857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16372[] = {SGI73858, SGI73859, SGI73860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73861, SGI0, SGI0, SGI0, SGI73862, SGI73862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16373[] = {SGI73858, SGI73859, SGI73860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73861, SGI0, SGI0, SGI0, SGI73862, SGI73862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16374[] = {SGI73863, SGI73864, SGI73865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73866, SGI0, SGI0, SGI0, SGI73867, SGI73867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16375[] = {SGI73868, SGI0, SGI73869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16376[] = {SGI73870, SGI73871, SGI73872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73873, SGI0, SGI0, SGI0, SGI73874, SGI73874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16377[] = {SGI73875, SGI73876, SGI73877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73878, SGI73879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16378[] = {SGI73880, SGI0, SGI73881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16379[] = {SGI73882, SGI73883, SGI73884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73885, SGI73885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16380[] = {SGI73886, SGI73887, SGI73888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73889, SGI0, SGI0, SGI0, SGI73890, SGI73890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16381[] = {SGI73891, SGI73892, SGI73893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73894, SGI0, SGI0, SGI0, SGI73895, SGI73895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16382[] = {SGI73896, SGI0, SGI73897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16383[] = {SGI73898, SGI73899, SGI73900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73901, SGI0, SGI0, SGI0, SGI73902, SGI73902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16384[] = {SGI73903, SGI0, SGI73904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16385[] = {SGI73905, SGI73906, SGI73907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73908, SGI0, SGI0, SGI0, SGI73909, SGI73909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16386[] = {SGI73910, SGI73911, SGI73912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73913, SGI73913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16387[] = {SGI73914, SGI73915, SGI73916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73917, SGI0, SGI0, SGI0, SGI73918, SGI73918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16388[] = {SGI73919, SGI73920, SGI73921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73922, SGI0, SGI0, SGI0, SGI73923, SGI73923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16389[] = {SGI73924, SGI73925, SGI73926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73927, SGI0, SGI0, SGI0, SGI73928, SGI73928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16390[] = {SGI73929, SGI0, SGI73930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16391[] = {SGI73931, SGI73932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73933, SGI0, SGI0, SGI0, SGI73934, SGI73934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16392[] = {SGI73935, SGI73936, SGI73937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73938, SGI0, SGI0, SGI0, SGI73939, SGI73939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16393[] = {SGI73940, SGI73941, SGI73942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73943, SGI0, SGI0, SGI0, SGI73944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16394[] = {SGI73945, SGI73946, SGI73947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73948, SGI0, SGI0, SGI0, SGI73949, SGI73949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16395[] = {SGI73950, SGI73951, SGI73952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73953, SGI0, SGI0, SGI0, SGI73954, SGI73954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16396[] = {SGI73955, SGI73956, SGI73957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73958, SGI0, SGI0, SGI0, SGI73959, SGI73959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16397[] = {SGI73960, SGI73961, SGI73962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73963, SGI0, SGI0, SGI0, SGI73964, SGI73964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16398[] = {SGI73965, SGI73966, SGI73967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73968, SGI0, SGI0, SGI0, SGI73969, SGI73969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16399[] = {SGI73970, SGI73971, SGI73972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73973, SGI0, SGI0, SGI0, SGI73974, SGI73974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16400[] = {SGI0, SGI73975, SGI73976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73977, SGI0, SGI0, SGI0, SGI73978, SGI73978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16401[] = {SGI73979, SGI73980, SGI73981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73982, SGI0, SGI0, SGI0, SGI73983, SGI73983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16402[] = {SGI73984, SGI0, SGI73985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16403[] = {SGI0, SGI0, SGI73986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73987, SGI0, SGI0, SGI0, SGI73988, SGI73988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16404[] = {SGI73989, SGI73990, SGI73991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73992, SGI0, SGI0, SGI0, SGI73993, SGI73993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16405[] = {SGI73994, SGI0, SGI73995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16406[] = {SGI73996, SGI73997, SGI73998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI73999, SGI0, SGI0, SGI0, SGI74000, SGI74000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16407[] = {SGI74001, SGI74002, SGI74003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74004, SGI0, SGI0, SGI0, SGI74005, SGI74005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16408[] = {SGI74006, SGI74007, SGI74008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74009, SGI74009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16409[] = {SGI74010, SGI0, SGI74011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74012, SGI0, SGI0, SGI0, SGI74013, SGI74013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16410[] = {SGI74014, SGI74015, SGI74016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74017, SGI0, SGI0, SGI0, SGI74018, SGI74018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16411[] = {SGI74019, SGI0, SGI74020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16412[] = {SGI74021, SGI0, SGI74022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74023, SGI0, SGI0, SGI0, SGI74024, SGI74024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16413[] = {SGI74025, SGI74026, SGI74027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74028, SGI0, SGI0, SGI0, SGI74029, SGI74029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16414[] = {SGI74030, SGI74031, SGI74032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74033, SGI0, SGI0, SGI0, SGI74034, SGI74034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16415[] = {SGI74035, SGI0, SGI74036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16416[] = {SGI74037, SGI74038, SGI74039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74040, SGI0, SGI0, SGI0, SGI74041, SGI74041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16417[] = {SGI74042, SGI0, SGI74043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16418[] = {SGI74044, SGI74045, SGI74046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74047, SGI0, SGI0, SGI0, SGI74048, SGI74048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16419[] = {SGI74049, SGI74050, SGI74051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74052, SGI0, SGI0, SGI0, SGI74053, SGI74053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16420[] = {SGI74054, SGI74055, SGI74056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74057, SGI0, SGI0, SGI0, SGI74058, SGI74058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16421[] = {SGI74059, SGI74060, SGI74061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74062, SGI74062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16422[] = {SGI74063, SGI74064, SGI74065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74066, SGI0, SGI0, SGI0, SGI74067, SGI74067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16423[] = {SGI74068, SGI0, SGI74069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16424[] = {SGI74070, SGI74071, SGI74072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74073, SGI0, SGI0, SGI0, SGI74074, SGI74074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16425[] = {SGI74075, SGI74076, SGI74077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74078, SGI0, SGI0, SGI0, SGI74079, SGI74079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16426[] = {SGI74080, SGI74081, SGI74082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74083, SGI0, SGI0, SGI0, SGI74084, SGI74084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16427[] = {SGI74085, SGI74086, SGI74087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74088, SGI0, SGI0, SGI0, SGI74089, SGI74089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16428[] = {SGI74090, SGI74091, SGI74092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74093, SGI0, SGI0, SGI0, SGI74094, SGI74094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16429[] = {SGI74095, SGI74096, SGI74097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74098, SGI0, SGI0, SGI0, SGI74099, SGI74099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16430[] = {SGI74100, SGI74101, SGI74102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74103, SGI0, SGI0, SGI0, SGI74104, SGI74104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16431[] = {SGI74105, SGI74106, SGI74107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74108, SGI74109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16432[] = {SGI74110, SGI74111, SGI74112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74113, SGI0, SGI0, SGI0, SGI74114, SGI74114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16433[] = {SGI74115, SGI74116, SGI74117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74118, SGI0, SGI0, SGI0, SGI74119, SGI74119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16434[] = {SGI74120, SGI74121, SGI74122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74123, SGI0, SGI0, SGI0, SGI74124, SGI74124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16435[] = {SGI74125, SGI74126, SGI74127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74128, SGI0, SGI0, SGI0, SGI74129, SGI74129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16436[] = {SGI74130, SGI74131, SGI74132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74133, SGI0, SGI0, SGI0, SGI74134, SGI74134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16437[] = {SGI74135, SGI74136, SGI74137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74138, SGI74138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16438[] = {SGI74139, SGI74140, SGI74141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74142, SGI0, SGI0, SGI0, SGI74143, SGI74143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16439[] = {SGI74144, SGI74145, SGI74146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74147, SGI0, SGI0, SGI0, SGI74148, SGI74148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16440[] = {SGI74149, SGI0, SGI74150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16441[] = {SGI74151, SGI0, SGI74152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74153, SGI0, SGI0, SGI0, SGI74154, SGI74154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16442[] = {SGI74155, SGI74156, SGI74157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74158, SGI0, SGI0, SGI0, SGI74159, SGI74159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16443[] = {SGI74160, SGI74161, SGI74162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74163, SGI0, SGI0, SGI0, SGI74164, SGI74164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16444[] = {SGI74165, SGI74166, SGI74167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74168, SGI0, SGI0, SGI0, SGI74169, SGI74169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16445[] = {SGI74170, SGI74171, SGI74172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74173, SGI74174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16446[] = {SGI74175, SGI0, SGI74176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16447[] = {SGI74177, SGI74178, SGI74179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74180, SGI0, SGI0, SGI0, SGI74181, SGI74181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16448[] = {SGI74182, SGI74183, SGI74184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74185, SGI0, SGI0, SGI0, SGI74186, SGI74186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16449[] = {SGI74187, SGI74188, SGI74189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74190, SGI0, SGI0, SGI0, SGI74191, SGI74191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16450[] = {SGI74192, SGI74193, SGI74194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74195, SGI0, SGI0, SGI0, SGI74196, SGI74196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16451[] = {SGI74197, SGI74198, SGI74199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74200, SGI74200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16452[] = {SGI74201, SGI74202, SGI74203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74204, SGI74204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16453[] = {SGI74205, SGI74206, SGI74207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74208, SGI0, SGI0, SGI0, SGI74209, SGI74209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16454[] = {SGI74210, SGI74211, SGI74212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74213, SGI0, SGI0, SGI0, SGI74214, SGI74214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16455[] = {SGI74215, SGI74216, SGI74217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74218, SGI0, SGI0, SGI0, SGI74219, SGI74219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16456[] = {SGI74220, SGI74221, SGI74222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74223, SGI0, SGI0, SGI0, SGI74224, SGI74224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16457[] = {SGI74225, SGI74226, SGI74227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74228, SGI0, SGI0, SGI0, SGI74229, SGI74229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16458[] = {SGI74230, SGI74231, SGI74232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74233, SGI74233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16459[] = {SGI74234, SGI0, SGI74235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16460[] = {SGI74236, SGI74237, SGI74238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74239, SGI0, SGI0, SGI0, SGI74240, SGI74240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16461[] = {SGI74241, SGI0, SGI74242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16462[] = {SGI74243, SGI0, SGI74244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74245, SGI0, SGI0, SGI0, SGI74246, SGI74246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16463[] = {SGI74247, SGI74248, SGI74249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74250, SGI0, SGI0, SGI0, SGI74251, SGI74251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16464[] = {SGI74252, SGI0, SGI74253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16465[] = {SGI74254, SGI74255, SGI74256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74257, SGI0, SGI0, SGI0, SGI74258, SGI74258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16466[] = {SGI74259, SGI74260, SGI74261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74262, SGI0, SGI0, SGI0, SGI74263, SGI74263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16467[] = {SGI74264, SGI74265, SGI74266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74267, SGI0, SGI0, SGI0, SGI74268, SGI74268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16468[] = {SGI74269, SGI74270, SGI74271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74272, SGI0, SGI0, SGI0, SGI74273, SGI74273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16469[] = {SGI74274, SGI0, SGI74275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16470[] = {SGI74276, SGI74277, SGI74278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74279, SGI0, SGI0, SGI0, SGI74280, SGI74280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16471[] = {SGI0, SGI74281, SGI74282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74283, SGI74283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16472[] = {SGI74284, SGI74285, SGI74286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74287, SGI0, SGI0, SGI0, SGI74288, SGI74288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16473[] = {SGI74289, SGI74290, SGI74291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74292, SGI0, SGI0, SGI0, SGI74293, SGI74293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16474[] = {SGI0, SGI74294, SGI74295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74296, SGI74296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16475[] = {SGI0, SGI74297, SGI74298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74299, SGI74299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16476[] = {SGI74300, SGI74301, SGI74302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74303, SGI0, SGI0, SGI0, SGI74304, SGI74304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16477[] = {SGI74305, SGI74306, SGI74307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74308, SGI0, SGI0, SGI0, SGI74309, SGI74309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16478[] = {SGI74310, SGI0, SGI74311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16479[] = {SGI74314, SGI74316, SGI74317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74319, SGI0, SGI0, SGI0, SGI74321, SGI74321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16480[] = {SGI74312, SGI74313, SGI74315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74318, SGI0, SGI0, SGI0, SGI74320, SGI74320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16481[] = {SGI74322, SGI0, SGI74323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16482[] = {SGI74324, SGI74325, SGI74326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74328, SGI74328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16483[] = {SGI74327, SGI74329, SGI74330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74331, SGI0, SGI0, SGI0, SGI74332, SGI74332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16484[] = {SGI74333, SGI74334, SGI74335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74336, SGI0, SGI0, SGI0, SGI74337, SGI74337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16485[] = {SGI74338, SGI74339, SGI74340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74341, SGI0, SGI0, SGI0, SGI74342, SGI74342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16486[] = {SGI74343, SGI0, SGI74344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16487[] = {SGI74345, SGI74346, SGI74347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74348, SGI0, SGI0, SGI0, SGI74349, SGI74349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16488[] = {SGI74350, SGI74351, SGI74352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74353, SGI0, SGI0, SGI0, SGI74354, SGI74354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16489[] = {SGI74355, SGI74356, SGI74357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74358, SGI74359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16490[] = {SGI74360, SGI74361, SGI74362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74363, SGI74363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16491[] = {SGI74364, SGI74365, SGI74366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74367, SGI0, SGI0, SGI0, SGI74368, SGI74368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16492[] = {SGI74369, SGI74370, SGI74371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74372, SGI74373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16493[] = {SGI74374, SGI74375, SGI74376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74377, SGI0, SGI0, SGI0, SGI74378, SGI74378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16494[] = {SGI74379, SGI74380, SGI74381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74382, SGI74383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16495[] = {SGI74384, SGI74385, SGI74386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74387, SGI0, SGI0, SGI0, SGI74388, SGI74388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16496[] = {SGI74389, SGI74390, SGI74391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74392, SGI0, SGI0, SGI0, SGI74393, SGI74393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16497[] = {SGI74394, SGI74395, SGI74396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74397, SGI74397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16498[] = {SGI74398, SGI74399, SGI74400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74401, SGI0, SGI0, SGI0, SGI74402, SGI74402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16499[] = {SGI74403, SGI74404, SGI74405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74406, SGI74406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16500[] = {SGI74407, SGI0, SGI74408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16501[] = {SGI74409, SGI74410, SGI74411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74412, SGI74413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16502[] = {SGI74414, SGI74415, SGI74416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74417, SGI0, SGI0, SGI0, SGI74418, SGI74418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16503[] = {SGI74419, SGI74420, SGI74421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74422, SGI74422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16504[] = {SGI74423, SGI74424, SGI74425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74426, SGI74426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16505[] = {SGI74427, SGI74428, SGI74429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74430, SGI0, SGI0, SGI0, SGI74431, SGI74431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16506[] = {SGI74432, SGI74433, SGI74434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74435, SGI0, SGI0, SGI0, SGI74436, SGI74436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16507[] = {SGI74437, SGI74438, SGI74439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74440, SGI0, SGI0, SGI0, SGI74441, SGI74441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16508[] = {SGI74442, SGI74443, SGI74444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74445, SGI0, SGI0, SGI0, SGI74446, SGI74446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16509[] = {SGI74447, SGI74448, SGI74449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74450, SGI0, SGI0, SGI0, SGI74451, SGI74451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16510[] = {SGI74452, SGI0, SGI74453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16511[] = {SGI74454, SGI74455, SGI74456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74457, SGI0, SGI0, SGI0, SGI74458, SGI74458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16512[] = {SGI74459, SGI0, SGI74460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16513[] = {SGI74461, SGI0, SGI74462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16514[] = {SGI74463, SGI74464, SGI74465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74466, SGI0, SGI0, SGI0, SGI74467, SGI74467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16515[] = {SGI74468, SGI0, SGI74469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16516[] = {SGI74470, SGI74471, SGI74472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74473, SGI0, SGI0, SGI0, SGI74474, SGI74474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16517[] = {SGI74475, SGI0, SGI74476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74477, SGI0, SGI0, SGI0, SGI74478, SGI74478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16518[] = {SGI74479, SGI74480, SGI74481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74482, SGI0, SGI0, SGI0, SGI74483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16519[] = {SGI74484, SGI74485, SGI74486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74487, SGI0, SGI0, SGI0, SGI74488, SGI74488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16520[] = {SGI74489, SGI74490, SGI74491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74492, SGI0, SGI0, SGI0, SGI74493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16521[] = {SGI74494, SGI0, SGI74495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16522[] = {SGI74496, SGI74497, SGI74498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16523[] = {SGI74499, SGI0, SGI74500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74501, SGI0, SGI0, SGI0, SGI74502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16524[] = {SGI74503, SGI0, SGI74504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16525[] = {SGI74505, SGI74506, SGI74507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74508, SGI0, SGI0, SGI0, SGI74509, SGI74509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16526[] = {SGI74510, SGI0, SGI74511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16527[] = {SGI74512, SGI74513, SGI74514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74515, SGI0, SGI0, SGI0, SGI74516, SGI74516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16528[] = {SGI74517, SGI74518, SGI74519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74520, SGI0, SGI0, SGI0, SGI74521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16529[] = {SGI74522, SGI74523, SGI74524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74525, SGI0, SGI0, SGI0, SGI74526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16530[] = {SGI74527, SGI74528, SGI74529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74530, SGI0, SGI0, SGI0, SGI74531, SGI74531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16531[] = {SGI74532, SGI74533, SGI74534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74535, SGI0, SGI0, SGI0, SGI74536, SGI74536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16532[] = {SGI74537, SGI74538, SGI74539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74540, SGI0, SGI0, SGI0, SGI74541, SGI74541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16533[] = {SGI74542, SGI74543, SGI74544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74548, SGI0, SGI0, SGI0, SGI74549, SGI74549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16534[] = {SGI74545, SGI74546, SGI74547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74550, SGI0, SGI0, SGI0, SGI74551, SGI74551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16535[] = {SGI74555, SGI0, SGI74556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16536[] = {SGI74552, SGI74553, SGI74554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74557, SGI0, SGI0, SGI0, SGI74558, SGI74558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16537[] = {SGI74559, SGI74560, SGI74561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74565, SGI0, SGI0, SGI0, SGI74566, SGI74566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16538[] = {SGI74562, SGI74563, SGI74564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74567, SGI0, SGI0, SGI0, SGI74568, SGI74568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16539[] = {SGI74569, SGI74570, SGI74571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74573, SGI0, SGI0, SGI0, SGI74576, SGI74576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16540[] = {SGI74572, SGI74574, SGI74575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74577, SGI0, SGI0, SGI0, SGI74578, SGI74578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16541[] = {SGI74579, SGI74580, SGI74581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74582, SGI0, SGI0, SGI0, SGI74584, SGI74584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16542[] = {SGI74583, SGI74585, SGI74586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74587, SGI0, SGI0, SGI0, SGI74588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16543[] = {SGI74592, SGI74593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74594, SGI0, SGI0, SGI0, SGI74595, SGI74595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16544[] = {SGI74596, SGI74597, SGI74598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74599, SGI0, SGI0, SGI0, SGI74600, SGI74600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16545[] = {SGI74601, SGI74602, SGI74603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74604, SGI0, SGI0, SGI0, SGI74605, SGI74605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16546[] = {SGI74606, SGI0, SGI74607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16547[] = {SGI74608, SGI74609, SGI74610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74611, SGI0, SGI0, SGI0, SGI74612, SGI74612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16548[] = {SGI0, SGI74613, SGI74614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74615, SGI0, SGI0, SGI0, SGI0, SGI74616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16549[] = {SGI0, SGI74617, SGI74618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74619, SGI0, SGI0, SGI0, SGI0, SGI74620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16550[] = {SGI74621, SGI74622, SGI74623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74624, SGI74624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16551[] = {SGI74625, SGI74626, SGI74627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74628, SGI0, SGI0, SGI0, SGI74629, SGI74629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16552[] = {SGI74630, SGI0, SGI74631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16553[] = {SGI74589, SGI74590, SGI74591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74632, SGI0, SGI0, SGI0, SGI74633, SGI74633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16554[] = {SGI74634, SGI0, SGI74635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16555[] = {SGI74636, SGI0, SGI74637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74638, SGI0, SGI0, SGI0, SGI74639, SGI74639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16556[] = {SGI74640, SGI74641, SGI74642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74643, SGI0, SGI0, SGI0, SGI74644, SGI74644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16557[] = {SGI74645, SGI74646, SGI74647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74648, SGI0, SGI0, SGI0, SGI74649, SGI74649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16558[] = {SGI74650, SGI0, SGI74651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16559[] = {SGI74652, SGI74653, SGI74654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74655, SGI0, SGI0, SGI0, SGI74656, SGI74656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16560[] = {SGI74657, SGI74658, SGI74659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74660, SGI0, SGI0, SGI0, SGI74661, SGI74661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16561[] = {SGI74662, SGI0, SGI74663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16562[] = {SGI74664, SGI74665, SGI74666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74667, SGI0, SGI0, SGI0, SGI74668, SGI74668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16563[] = {SGI74669, SGI74670, SGI74671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74672, SGI0, SGI0, SGI0, SGI74673, SGI74673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16564[] = {SGI74674, SGI0, SGI74675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16565[] = {SGI74676, SGI0, SGI74677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16566[] = {SGI74678, SGI0, SGI74679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16567[] = {SGI74680, SGI74681, SGI74682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74683, SGI0, SGI0, SGI0, SGI74684, SGI74684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16568[] = {SGI74685, SGI74686, SGI74687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74688, SGI0, SGI0, SGI0, SGI74689, SGI74689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16569[] = {SGI74690, SGI74691, SGI74692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74693, SGI0, SGI0, SGI0, SGI74694, SGI74694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16570[] = {SGI74695, SGI74696, SGI74697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74698, SGI0, SGI0, SGI0, SGI74699, SGI74699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16571[] = {SGI74700, SGI74701, SGI74702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74703, SGI0, SGI0, SGI0, SGI74704, SGI74704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16572[] = {SGI74705, SGI74706, SGI74707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74708, SGI0, SGI0, SGI0, SGI74709, SGI74709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16573[] = {SGI74710, SGI0, SGI74711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16574[] = {SGI74712, SGI74713, SGI74714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74715, SGI0, SGI0, SGI0, SGI74716, SGI74716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16575[] = {SGI74717, SGI74718, SGI74719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74720, SGI0, SGI0, SGI0, SGI74721, SGI74721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16576[] = {SGI74722, SGI0, SGI74723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16577[] = {SGI74724, SGI0, SGI74725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74726, SGI0, SGI0, SGI0, SGI74727, SGI74727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16578[] = {SGI74728, SGI74729, SGI74730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74731, SGI0, SGI0, SGI0, SGI74732, SGI74732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16579[] = {SGI74737, SGI74738, SGI74739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74740, SGI0, SGI0, SGI0, SGI74741, SGI74741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16580[] = {SGI74742, SGI0, SGI74743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16581[] = {SGI74744, SGI74745, SGI74746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74747, SGI0, SGI0, SGI0, SGI74748, SGI74748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16582[] = {SGI74733, SGI74734, SGI74735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74736, SGI0, SGI0, SGI0, SGI74749, SGI74749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16583[] = {SGI74750, SGI74751, SGI74752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74753, SGI0, SGI0, SGI0, SGI74754, SGI74754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16584[] = {SGI74755, SGI74756, SGI74757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74758, SGI0, SGI0, SGI0, SGI74759, SGI74759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16585[] = {SGI74760, SGI74761, SGI74762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74763, SGI0, SGI0, SGI0, SGI74764, SGI74764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16586[] = {SGI74765, SGI74766, SGI74767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74768, SGI0, SGI0, SGI0, SGI74769, SGI74769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16587[] = {SGI74770, SGI0, SGI74771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74772, SGI0, SGI0, SGI0, SGI74773, SGI74773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16588[] = {SGI74774, SGI0, SGI74775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16589[] = {SGI74776, SGI74777, SGI74778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74779, SGI74780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16590[] = {SGI74781, SGI0, SGI74782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16591[] = {SGI74783, SGI74784, SGI74785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74786, SGI0, SGI0, SGI0, SGI74787, SGI74787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16592[] = {SGI74788, SGI74789, SGI74790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74791, SGI0, SGI0, SGI0, SGI74792, SGI74792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16593[] = {SGI74793, SGI0, SGI74794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16594[] = {SGI74795, SGI0, SGI74796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16595[] = {SGI74797, SGI74798, SGI74799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74800, SGI0, SGI0, SGI0, SGI74801, SGI74801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16596[] = {SGI74802, SGI74803, SGI74804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74805, SGI74806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16597[] = {SGI74807, SGI0, SGI74808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16598[] = {SGI74809, SGI74810, SGI74811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74812, SGI74812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16599[] = {SGI74813, SGI74814, SGI74815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74816, SGI74816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16600[] = {SGI74817, SGI74818, SGI74819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74820, SGI0, SGI0, SGI0, SGI74821, SGI74821, SGI0, SGI74822, SGI0, NULL};
const Segment * const SG16601[] = {SGI74823, SGI74824, SGI74825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74826, SGI0, SGI0, SGI0, SGI74827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16602[] = {SGI74828, SGI74829, SGI74830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74831, SGI0, SGI0, SGI0, SGI74832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16603[] = {SGI74833, SGI74834, SGI74835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74836, SGI0, SGI0, SGI0, SGI74837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16604[] = {SGI74838, SGI74839, SGI74840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74841, SGI0, SGI0, SGI0, SGI74842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16605[] = {SGI74843, SGI74844, SGI74845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74846, SGI0, SGI0, SGI0, SGI74847, SGI74847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16606[] = {SGI74848, SGI74849, SGI74850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74851, SGI0, SGI0, SGI0, SGI74852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16607[] = {SGI74853, SGI74854, SGI74855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74856, SGI0, SGI0, SGI0, SGI74857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16608[] = {SGI74858, SGI74859, SGI74860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74861, SGI0, SGI0, SGI0, SGI74862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16609[] = {SGI74863, SGI0, SGI74864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16610[] = {SGI74865, SGI74866, SGI74867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74868, SGI0, SGI0, SGI0, SGI74869, SGI74869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16611[] = {SGI74870, SGI0, SGI74871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74872, SGI0, SGI0, SGI0, SGI74873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16612[] = {SGI74874, SGI74875, SGI74876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74877, SGI0, SGI0, SGI0, SGI74878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16613[] = {SGI74879, SGI74880, SGI74881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74882, SGI0, SGI0, SGI0, SGI74883, SGI74883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16614[] = {SGI74884, SGI74885, SGI74886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74887, SGI0, SGI0, SGI0, SGI74888, SGI74888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16615[] = {SGI74889, SGI0, SGI74890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16616[] = {SGI74891, SGI74892, SGI74893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74894, SGI0, SGI0, SGI0, SGI74895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16617[] = {SGI74896, SGI74897, SGI74898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74899, SGI0, SGI0, SGI0, SGI74900, SGI74900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16618[] = {SGI74901, SGI0, SGI74902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16619[] = {SGI74903, SGI74904, SGI74905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74906, SGI0, SGI0, SGI0, SGI74907, SGI74907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16620[] = {SGI74908, SGI74909, SGI74910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74911, SGI74911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16621[] = {SGI74912, SGI0, SGI74913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74914, SGI0, SGI0, SGI0, SGI74915, SGI74915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16622[] = {SGI74916, SGI74917, SGI74918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74919, SGI0, SGI0, SGI0, SGI74920, SGI74920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16623[] = {SGI74921, SGI74922, SGI74923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74924, SGI0, SGI0, SGI0, SGI74925, SGI74925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16624[] = {SGI74926, SGI0, SGI74927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16625[] = {SGI74928, SGI0, SGI74929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16626[] = {SGI74930, SGI74931, SGI74932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74933, SGI0, SGI0, SGI0, SGI74934, SGI74934, SGI0, SGI0, NULL};
const Segment * const SG16627[] = {SGI74935, SGI74936, SGI74937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74938, SGI0, SGI0, SGI0, SGI74939, SGI74939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16628[] = {SGI74940, SGI0, SGI74941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16629[] = {SGI74942, SGI74943, SGI74944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74945, SGI0, SGI0, SGI0, SGI74946, SGI74946, SGI0, SGI0, NULL};
const Segment * const SG16630[] = {SGI74947, SGI74948, SGI74949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74950, SGI0, SGI0, SGI0, SGI74951, SGI74951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16631[] = {SGI74952, SGI0, SGI74953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16632[] = {SGI74954, SGI0, SGI74955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74956, SGI0, SGI0, SGI0, SGI74957, SGI74957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16633[] = {SGI74958, SGI74959, SGI74960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74961, SGI0, SGI0, SGI0, SGI74962, SGI74962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16634[] = {SGI74963, SGI74964, SGI74965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74966, SGI0, SGI0, SGI0, SGI74967, SGI74967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16635[] = {SGI74968, SGI74969, SGI74970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74971, SGI0, SGI0, SGI0, SGI74972, SGI74972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16636[] = {SGI74973, SGI74974, SGI74975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74976, SGI0, SGI0, SGI0, SGI74977, SGI74977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16637[] = {SGI74978, SGI74979, SGI74980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74981, SGI0, SGI0, SGI0, SGI74982, SGI74982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16638[] = {SGI74983, SGI0, SGI74984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16639[] = {SGI74985, SGI74986, SGI74987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74988, SGI0, SGI0, SGI0, SGI74989, SGI74989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16640[] = {SGI74990, SGI74991, SGI74992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74993, SGI0, SGI0, SGI0, SGI74994, SGI74994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16641[] = {SGI74995, SGI74996, SGI74997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI74998, SGI0, SGI0, SGI0, SGI74999, SGI74999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16642[] = {SGI75000, SGI75001, SGI75002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75003, SGI0, SGI0, SGI0, SGI75004, SGI75004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16643[] = {SGI75005, SGI75006, SGI75007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75008, SGI0, SGI0, SGI0, SGI75009, SGI75009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16644[] = {SGI75010, SGI75011, SGI75012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75013, SGI0, SGI0, SGI0, SGI75014, SGI75014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16645[] = {SGI75015, SGI75016, SGI75017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75018, SGI0, SGI0, SGI0, SGI75019, SGI75019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16646[] = {SGI75020, SGI75021, SGI75022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75023, SGI0, SGI0, SGI0, SGI75024, SGI75024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16647[] = {SGI75025, SGI75026, SGI75027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75028, SGI0, SGI0, SGI0, SGI75029, SGI75029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16648[] = {SGI75030, SGI75031, SGI75032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75033, SGI0, SGI0, SGI0, SGI75034, SGI75034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16649[] = {SGI75035, SGI75036, SGI75037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75038, SGI0, SGI0, SGI0, SGI75039, SGI75039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16650[] = {SGI75040, SGI75041, SGI75042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75043, SGI0, SGI0, SGI0, SGI75044, SGI75044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16651[] = {SGI75045, SGI75046, SGI75047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75048, SGI0, SGI0, SGI0, SGI75049, SGI75049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16652[] = {SGI75050, SGI75051, SGI75052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75053, SGI0, SGI0, SGI0, SGI75054, SGI75054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16653[] = {SGI75055, SGI75056, SGI75057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75058, SGI0, SGI0, SGI0, SGI75059, SGI75059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16654[] = {SGI75060, SGI75061, SGI75062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75063, SGI0, SGI0, SGI0, SGI75064, SGI75064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16655[] = {SGI75065, SGI75066, SGI75067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75068, SGI0, SGI0, SGI0, SGI75069, SGI75069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16656[] = {SGI75070, SGI75071, SGI75072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75073, SGI0, SGI0, SGI0, SGI75074, SGI75074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16657[] = {SGI75075, SGI75076, SGI75077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75078, SGI0, SGI0, SGI0, SGI75079, SGI75079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16658[] = {SGI75080, SGI75081, SGI75082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75083, SGI0, SGI0, SGI0, SGI75084, SGI75084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16659[] = {SGI75085, SGI75086, SGI75087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75088, SGI0, SGI0, SGI0, SGI75089, SGI75089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16660[] = {SGI75090, SGI75091, SGI75092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75093, SGI0, SGI0, SGI0, SGI75094, SGI75094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16661[] = {SGI75095, SGI75096, SGI75097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75098, SGI0, SGI0, SGI0, SGI75099, SGI75099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16662[] = {SGI75100, SGI75101, SGI75102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75103, SGI0, SGI0, SGI0, SGI75104, SGI75104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16663[] = {SGI75105, SGI75106, SGI75107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75108, SGI0, SGI0, SGI0, SGI75109, SGI75109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16664[] = {SGI75110, SGI0, SGI75111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75112, SGI0, SGI0, SGI0, SGI75113, SGI75113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16665[] = {SGI75114, SGI75115, SGI75116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75117, SGI0, SGI0, SGI0, SGI75118, SGI75118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16666[] = {SGI75119, SGI75120, SGI75121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75122, SGI0, SGI0, SGI0, SGI75123, SGI75123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16667[] = {SGI75124, SGI0, SGI75125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16668[] = {SGI75126, SGI0, SGI75127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75128, SGI0, SGI0, SGI0, SGI75129, SGI75129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16669[] = {SGI75130, SGI75131, SGI75132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75133, SGI0, SGI0, SGI0, SGI75134, SGI75134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16670[] = {SGI0, SGI75135, SGI75136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75137, SGI75137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16671[] = {SGI75138, SGI75139, SGI75140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75141, SGI0, SGI0, SGI0, SGI75142, SGI75142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16672[] = {SGI0, SGI75143, SGI75144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75145, SGI75145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16673[] = {SGI75146, SGI75147, SGI75148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75149, SGI0, SGI0, SGI0, SGI75150, SGI75150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16674[] = {SGI0, SGI75151, SGI75152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75153, SGI75153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16675[] = {SGI0, SGI75154, SGI75155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75156, SGI75156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16676[] = {SGI75157, SGI75158, SGI75159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75160, SGI0, SGI0, SGI0, SGI75161, SGI75161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16677[] = {SGI0, SGI75162, SGI75163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75164, SGI75164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16678[] = {SGI0, SGI75165, SGI75166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75167, SGI75167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16679[] = {SGI0, SGI75168, SGI75169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75170, SGI75170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16680[] = {SGI75171, SGI75172, SGI75173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75174, SGI0, SGI0, SGI0, SGI75175, SGI75175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16681[] = {SGI75176, SGI75177, SGI75178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75179, SGI0, SGI0, SGI0, SGI75180, SGI75180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16682[] = {SGI0, SGI75181, SGI75182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75183, SGI75183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16683[] = {SGI75184, SGI75185, SGI75186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75187, SGI0, SGI0, SGI0, SGI75188, SGI75188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16684[] = {SGI75189, SGI0, SGI75190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16685[] = {SGI75191, SGI75192, SGI75193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75194, SGI0, SGI0, SGI0, SGI75195, SGI75195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16686[] = {SGI75196, SGI75197, SGI75198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75199, SGI0, SGI0, SGI0, SGI75200, SGI75200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16687[] = {SGI75201, SGI75202, SGI75203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75204, SGI0, SGI0, SGI0, SGI75205, SGI75205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16688[] = {SGI75206, SGI75207, SGI75208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75209, SGI0, SGI0, SGI0, SGI75210, SGI75210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16689[] = {SGI75211, SGI75212, SGI75213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75214, SGI0, SGI0, SGI0, SGI75215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16690[] = {SGI75216, SGI0, SGI75217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16691[] = {SGI75218, SGI75219, SGI75220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75221, SGI0, SGI0, SGI0, SGI75222, SGI75222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16692[] = {SGI75223, SGI75224, SGI75225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75226, SGI0, SGI0, SGI0, SGI75227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16693[] = {SGI75228, SGI75229, SGI75230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75231, SGI0, SGI0, SGI0, SGI75232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16694[] = {SGI75233, SGI0, SGI75234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75235, SGI0, SGI0, SGI0, SGI75236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16695[] = {SGI75237, SGI75238, SGI75239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75240, SGI0, SGI0, SGI0, SGI75241, SGI75242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16696[] = {SGI75243, SGI75244, SGI75245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75246, SGI75246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16697[] = {SGI75247, SGI75248, SGI75249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75250, SGI0, SGI0, SGI0, SGI75251, SGI75251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16698[] = {SGI75252, SGI75253, SGI75254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75255, SGI0, SGI0, SGI0, SGI75256, SGI75256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16699[] = {SGI75257, SGI75258, SGI75259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75260, SGI0, SGI0, SGI0, SGI75261, SGI75261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16700[] = {SGI75262, SGI0, SGI75263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16701[] = {SGI75264, SGI75265, SGI75266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75267, SGI0, SGI0, SGI0, SGI75268, SGI75268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16702[] = {SGI75269, SGI75270, SGI75271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75272, SGI0, SGI0, SGI0, SGI75273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16703[] = {SGI75274, SGI75275, SGI75276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75277, SGI0, SGI0, SGI0, SGI75278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16704[] = {SGI75279, SGI75280, SGI75281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75282, SGI0, SGI0, SGI0, SGI75283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16705[] = {SGI75284, SGI0, SGI75285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16706[] = {SGI75286, SGI75287, SGI75288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75289, SGI0, SGI0, SGI0, SGI75290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16707[] = {SGI75291, SGI75292, SGI75293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75294, SGI0, SGI0, SGI0, SGI75295, SGI75295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16708[] = {SGI75296, SGI0, SGI75297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75298, SGI0, SGI0, SGI0, SGI75299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16709[] = {SGI75300, SGI75301, SGI75302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75303, SGI75303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16710[] = {SGI75304, SGI0, SGI75305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16711[] = {SGI75306, SGI75307, SGI75308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75309, SGI75310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16712[] = {SGI75311, SGI75312, SGI75313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75314, SGI0, SGI0, SGI0, SGI75315, SGI75316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16713[] = {SGI75317, SGI75318, SGI75319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75320, SGI0, SGI0, SGI0, SGI75321, SGI75321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16714[] = {SGI75322, SGI75323, SGI75324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75325, SGI0, SGI0, SGI0, SGI75326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16715[] = {SGI75327, SGI75328, SGI75329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16716[] = {SGI75330, SGI75331, SGI75332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75333, SGI0, SGI0, SGI0, SGI75334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16717[] = {SGI75335, SGI75336, SGI75337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75341, SGI0, SGI0, SGI0, SGI75343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16718[] = {SGI75338, SGI75339, SGI75340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75342, SGI0, SGI0, SGI0, SGI75344, SGI75344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16719[] = {SGI0, SGI75345, SGI75346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16720[] = {SGI75347, SGI75348, SGI75349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16721[] = {SGI75353, SGI0, SGI75354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16722[] = {SGI75350, SGI75351, SGI75352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75355, SGI0, SGI0, SGI0, SGI75356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16723[] = {SGI75357, SGI75358, SGI75359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75360, SGI0, SGI0, SGI0, SGI75361, SGI75362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16724[] = {SGI75363, SGI75364, SGI75365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16725[] = {SGI75366, SGI0, SGI75367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16726[] = {SGI75368, SGI75369, SGI75370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16727[] = {SGI75371, SGI75372, SGI75373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75374, SGI0, SGI0, SGI0, SGI75375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16728[] = {SGI75376, SGI75377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75378, SGI0, SGI0, SGI0, SGI75379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16729[] = {SGI75380, SGI75381, SGI75382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75383, SGI0, SGI0, SGI0, SGI75384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16730[] = {SGI75385, SGI75386, SGI75387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75388, SGI0, SGI0, SGI0, SGI75389, SGI75389, SGI0, SGI0, NULL};
const Segment * const SG16731[] = {SGI75390, SGI75391, SGI75392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75393, SGI0, SGI0, SGI0, SGI75394, SGI75394, SGI0, SGI0, NULL};
const Segment * const SG16732[] = {SGI75395, SGI75396, SGI75397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75398, SGI0, SGI0, SGI0, SGI75399, SGI75399, SGI0, SGI0, NULL};
const Segment * const SG16733[] = {SGI75400, SGI75401, SGI75402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75403, SGI0, SGI0, SGI0, SGI75404, SGI75404, SGI0, SGI0, NULL};
const Segment * const SG16734[] = {SGI75405, SGI75406, SGI75407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75408, SGI0, SGI0, SGI0, SGI75409, SGI75409, SGI0, SGI0, NULL};
const Segment * const SG16735[] = {SGI75410, SGI75411, SGI75412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75413, SGI0, SGI0, SGI0, SGI75414, SGI75414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16736[] = {SGI75415, SGI75416, SGI75417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75418, SGI0, SGI0, SGI0, SGI75419, SGI75419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16737[] = {SGI75420, SGI0, SGI75421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16738[] = {SGI75422, SGI75423, SGI75424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75425, SGI0, SGI0, SGI0, SGI75426, SGI75426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16739[] = {SGI75427, SGI75428, SGI75429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75430, SGI0, SGI0, SGI0, SGI75431, SGI75431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16740[] = {SGI75432, SGI75433, SGI75434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75435, SGI0, SGI0, SGI0, SGI75436, SGI75436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16741[] = {SGI75437, SGI75438, SGI75439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75440, SGI0, SGI0, SGI0, SGI75441, SGI75441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16742[] = {SGI75442, SGI75443, SGI75444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75445, SGI0, SGI0, SGI0, SGI75446, SGI75446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16743[] = {SGI75447, SGI75448, SGI75449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75450, SGI75450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16744[] = {SGI75451, SGI75452, SGI75453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75454, SGI0, SGI0, SGI0, SGI75455, SGI75455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16745[] = {SGI75456, SGI75457, SGI75458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75459, SGI75459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16746[] = {SGI0, SGI75460, SGI75461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16747[] = {SGI75462, SGI75463, SGI75464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75465, SGI0, SGI0, SGI0, SGI75466, SGI75466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16748[] = {SGI75467, SGI0, SGI75468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16749[] = {SGI75472, SGI0, SGI75473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16750[] = {SGI75469, SGI75470, SGI75471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75474, SGI0, SGI0, SGI0, SGI75475, SGI75475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16751[] = {SGI75476, SGI75478, SGI75479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75482, SGI0, SGI0, SGI0, SGI75484, SGI75484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16752[] = {SGI75477, SGI75480, SGI75481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75483, SGI0, SGI0, SGI0, SGI75485, SGI75485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16753[] = {SGI75486, SGI75487, SGI75488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75489, SGI0, SGI0, SGI0, SGI75490, SGI75490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16754[] = {SGI0, SGI75491, SGI75492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75493, SGI0, SGI0, SGI0, SGI0, SGI75494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16755[] = {SGI75495, SGI75496, SGI75497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75498, SGI0, SGI0, SGI0, SGI75499, SGI75499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16756[] = {SGI75500, SGI0, SGI75501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16757[] = {SGI0, SGI75502, SGI75503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75504, SGI0, SGI0, SGI0, SGI0, SGI75505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16758[] = {SGI75506, SGI75507, SGI75508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75509, SGI0, SGI0, SGI0, SGI75510, SGI75510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16759[] = {SGI75511, SGI0, SGI75512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75513, SGI0, SGI0, SGI0, SGI75514, SGI75514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16760[] = {SGI75515, SGI75517, SGI75519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75521, SGI0, SGI0, SGI0, SGI75523, SGI75523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16761[] = {SGI75516, SGI75518, SGI75520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75522, SGI0, SGI0, SGI0, SGI75524, SGI75524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16762[] = {SGI75525, SGI75526, SGI75527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75528, SGI0, SGI0, SGI0, SGI75529, SGI75529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16763[] = {SGI75530, SGI75531, SGI75532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75533, SGI0, SGI0, SGI0, SGI75534, SGI75534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16764[] = {SGI75535, SGI75536, SGI75537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75538, SGI0, SGI0, SGI0, SGI75539, SGI75539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16765[] = {SGI75540, SGI75541, SGI75542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75543, SGI0, SGI0, SGI0, SGI75544, SGI75544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16766[] = {SGI75545, SGI0, SGI75546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16767[] = {SGI75547, SGI75548, SGI75549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75550, SGI0, SGI0, SGI0, SGI75551, SGI75551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16768[] = {SGI75552, SGI0, SGI75553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16769[] = {SGI75554, SGI75555, SGI75556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75557, SGI0, SGI0, SGI0, SGI75558, SGI75558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16770[] = {SGI75559, SGI0, SGI75560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16771[] = {SGI75561, SGI0, SGI75562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75563, SGI0, SGI0, SGI0, SGI75564, SGI75564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16772[] = {SGI75565, SGI0, SGI75566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16773[] = {SGI75567, SGI75568, SGI75569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75570, SGI0, SGI0, SGI0, SGI75571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16774[] = {SGI75572, SGI0, SGI75573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16775[] = {SGI75574, SGI75575, SGI75576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75577, SGI75578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16776[] = {SGI75579, SGI75580, SGI75581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75582, SGI0, SGI0, SGI0, SGI75583, SGI75583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16777[] = {SGI75584, SGI75585, SGI75586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75587, SGI75587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16778[] = {SGI75588, SGI75589, SGI75590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75591, SGI0, SGI0, SGI0, SGI75592, SGI75593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16779[] = {SGI75594, SGI0, SGI75595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75596, SGI0, SGI0, SGI0, SGI75597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16780[] = {SGI0, SGI75598, SGI75599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16781[] = {SGI0, SGI75600, SGI75601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16782[] = {SGI75602, SGI75603, SGI75604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75605, SGI0, SGI0, SGI0, SGI75606, SGI75606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16783[] = {SGI75607, SGI75608, SGI75609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75610, SGI0, SGI0, SGI0, SGI75611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16784[] = {SGI75612, SGI75613, SGI75614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75615, SGI0, SGI0, SGI0, SGI75616, SGI75616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16785[] = {SGI75617, SGI0, SGI75618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16786[] = {SGI75619, SGI75620, SGI75621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75622, SGI0, SGI0, SGI0, SGI75623, SGI75624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16787[] = {SGI75625, SGI75626, SGI75627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16788[] = {SGI75628, SGI75629, SGI75630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16789[] = {SGI75631, SGI75632, SGI75633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16790[] = {SGI75634, SGI75635, SGI75636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75637, SGI0, SGI0, SGI0, SGI75638, SGI75638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16791[] = {SGI75639, SGI75640, SGI75641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75642, SGI0, SGI0, SGI0, SGI75643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16792[] = {SGI0, SGI0, SGI75644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16793[] = {SGI75645, SGI75646, SGI75647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16794[] = {SGI0, SGI75648, SGI75649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75650, SGI0, SGI0, SGI0, SGI75651, SGI75652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16795[] = {SGI75653, SGI75654, SGI75655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75656, SGI0, SGI0, SGI0, SGI75657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16796[] = {SGI75658, SGI75659, SGI75660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75661, SGI0, SGI0, SGI0, SGI75662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16797[] = {SGI0, SGI75663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16798[] = {SGI75664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16799[] = {SGI75666, SGI75667, SGI75668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75669, SGI0, SGI0, SGI0, SGI75670, SGI75670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16800[] = {SGI75671, SGI75672, SGI75673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75674, SGI0, SGI0, SGI0, SGI75675, SGI75675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16801[] = {SGI75676, SGI75677, SGI75678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75679, SGI0, SGI0, SGI0, SGI75680, SGI75680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16802[] = {SGI75681, SGI0, SGI75682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16803[] = {SGI75683, SGI0, SGI75684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16804[] = {SGI75685, SGI75686, SGI75687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75688, SGI0, SGI0, SGI0, SGI75689, SGI75689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16805[] = {SGI75690, SGI0, SGI75691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16806[] = {SGI75692, SGI0, SGI75693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75694, SGI0, SGI0, SGI0, SGI75695, SGI75695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16807[] = {SGI75696, SGI75697, SGI75698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75699, SGI0, SGI0, SGI0, SGI75700, SGI75700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16808[] = {SGI75701, SGI0, SGI75702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16809[] = {SGI75703, SGI75704, SGI75705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75706, SGI0, SGI0, SGI0, SGI75707, SGI75707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16810[] = {SGI75708, SGI75709, SGI75710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75711, SGI0, SGI0, SGI0, SGI75712, SGI75712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16811[] = {SGI75713, SGI75714, SGI75715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75716, SGI0, SGI0, SGI0, SGI75717, SGI75717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16812[] = {SGI75718, SGI0, SGI75719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16813[] = {SGI75720, SGI75721, SGI75722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75723, SGI0, SGI0, SGI0, SGI75724, SGI75724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16814[] = {SGI75725, SGI75726, SGI75727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75728, SGI75730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16815[] = {SGI75729, SGI75731, SGI75732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75733, SGI0, SGI0, SGI0, SGI75734, SGI75734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16816[] = {SGI75735, SGI75736, SGI75737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75738, SGI0, SGI0, SGI0, SGI75739, SGI75739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16817[] = {SGI75740, SGI75741, SGI75742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75743, SGI0, SGI0, SGI0, SGI75744, SGI75744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16818[] = {SGI75745, SGI75746, SGI75747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75748, SGI0, SGI0, SGI0, SGI75749, SGI75749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16819[] = {SGI75750, SGI75751, SGI75752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75753, SGI75753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16820[] = {SGI75754, SGI75755, SGI75756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75757, SGI0, SGI0, SGI0, SGI75758, SGI75758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16821[] = {SGI75759, SGI75760, SGI75761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75762, SGI0, SGI0, SGI0, SGI75763, SGI75763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16822[] = {SGI75764, SGI0, SGI75765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75766, SGI0, SGI0, SGI0, SGI75767, SGI75767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16823[] = {SGI75768, SGI0, SGI75769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16824[] = {SGI75770, SGI75771, SGI75772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75773, SGI0, SGI0, SGI0, SGI75774, SGI75774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16825[] = {SGI75775, SGI75776, SGI75777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75778, SGI0, SGI0, SGI0, SGI75779, SGI75779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16826[] = {SGI75780, SGI75781, SGI75782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75783, SGI75783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16827[] = {SGI0, SGI75784, SGI75785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16828[] = {SGI75786, SGI0, SGI75787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16829[] = {SGI75788, SGI0, SGI75789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75790, SGI0, SGI0, SGI0, SGI75791, SGI75791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16830[] = {SGI75792, SGI75793, SGI75794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75795, SGI0, SGI0, SGI0, SGI75796, SGI75796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16831[] = {SGI75797, SGI75798, SGI75799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75800, SGI0, SGI0, SGI0, SGI75801, SGI75801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16832[] = {SGI75802, SGI0, SGI75803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16833[] = {SGI75804, SGI75805, SGI75806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75807, SGI0, SGI0, SGI0, SGI75808, SGI75808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16834[] = {SGI75809, SGI0, SGI75810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75811, SGI0, SGI0, SGI0, SGI75812, SGI75812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16835[] = {SGI75813, SGI75814, SGI75815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75816, SGI0, SGI0, SGI0, SGI75817, SGI75818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16836[] = {SGI75819, SGI75820, SGI75821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75822, SGI0, SGI0, SGI0, SGI75823, SGI75823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16837[] = {SGI75824, SGI75825, SGI75826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75827, SGI0, SGI0, SGI0, SGI75828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16838[] = {SGI75829, SGI75830, SGI75831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75832, SGI0, SGI0, SGI0, SGI75833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16839[] = {SGI75834, SGI75835, SGI75836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75837, SGI0, SGI0, SGI0, SGI75838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16840[] = {SGI75839, SGI0, SGI75840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16841[] = {SGI75841, SGI75842, SGI75843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75844, SGI0, SGI0, SGI0, SGI75845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16842[] = {SGI75846, SGI0, SGI75847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75848, SGI0, SGI0, SGI0, SGI75849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16843[] = {SGI75850, SGI75851, SGI75852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75853, SGI0, SGI0, SGI0, SGI75854, SGI75854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16844[] = {SGI75855, SGI75856, SGI75857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75858, SGI0, SGI0, SGI0, SGI75859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16845[] = {SGI75860, SGI0, SGI75861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16846[] = {SGI75862, SGI75863, SGI75864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75865, SGI0, SGI0, SGI0, SGI75866, SGI75867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16847[] = {SGI0, SGI75868, SGI75869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16848[] = {SGI75870, SGI75871, SGI75872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75873, SGI0, SGI0, SGI0, SGI75874, SGI75874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16849[] = {SGI75875, SGI75876, SGI75877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75878, SGI75878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16850[] = {SGI75879, SGI75880, SGI75881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75882, SGI0, SGI0, SGI0, SGI75883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16851[] = {SGI75884, SGI0, SGI75885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16852[] = {SGI75886, SGI75887, SGI75888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16853[] = {SGI75889, SGI75890, SGI75891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75892, SGI0, SGI0, SGI0, SGI75893, SGI75894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16854[] = {SGI75895, SGI75896, SGI75897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75898, SGI0, SGI0, SGI0, SGI75899, SGI75899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16855[] = {SGI75900, SGI75901, SGI75902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75903, SGI0, SGI0, SGI0, SGI75904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16856[] = {SGI75905, SGI75906, SGI75907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75908, SGI0, SGI0, SGI0, SGI75909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16857[] = {SGI75910, SGI75911, SGI75912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75913, SGI0, SGI0, SGI0, SGI75914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16858[] = {SGI0, SGI75915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16859[] = {SGI75916, SGI75917, SGI75918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75919, SGI0, SGI0, SGI0, SGI75920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16860[] = {SGI75921, SGI75922, SGI75923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75924, SGI75925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16861[] = {SGI75926, SGI75927, SGI75928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75929, SGI0, SGI0, SGI0, SGI75930, SGI75930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16862[] = {SGI75931, SGI0, SGI75932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16863[] = {SGI75933, SGI75934, SGI75935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75936, SGI0, SGI0, SGI0, SGI75937, SGI75937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16864[] = {SGI75938, SGI75939, SGI75940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75941, SGI0, SGI0, SGI0, SGI75942, SGI75942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16865[] = {SGI75943, SGI75944, SGI75945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75946, SGI0, SGI0, SGI0, SGI75947, SGI75947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16866[] = {SGI75948, SGI75949, SGI75950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75951, SGI0, SGI0, SGI0, SGI75952, SGI75952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16867[] = {SGI75953, SGI75954, SGI75955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75956, SGI0, SGI0, SGI0, SGI75957, SGI75957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16868[] = {SGI75958, SGI75959, SGI75960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75961, SGI0, SGI0, SGI0, SGI75962, SGI75962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16869[] = {SGI75963, SGI75964, SGI75965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75966, SGI0, SGI0, SGI0, SGI75967, SGI75967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16870[] = {SGI75968, SGI75969, SGI75970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75971, SGI0, SGI0, SGI0, SGI75972, SGI75972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16871[] = {SGI75973, SGI75974, SGI75975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75976, SGI0, SGI0, SGI0, SGI75977, SGI75977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16872[] = {SGI75978, SGI75979, SGI75980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75981, SGI75981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16873[] = {SGI75982, SGI75983, SGI75984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75985, SGI75985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16874[] = {SGI75986, SGI0, SGI75987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16875[] = {SGI75988, SGI75989, SGI75990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75991, SGI75992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16876[] = {SGI75993, SGI75994, SGI75995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI75996, SGI0, SGI0, SGI0, SGI75997, SGI75997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16877[] = {SGI75998, SGI75999, SGI76000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76001, SGI0, SGI0, SGI0, SGI76002, SGI76002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16878[] = {SGI76003, SGI76004, SGI76005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76006, SGI0, SGI0, SGI0, SGI76007, SGI76007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16879[] = {SGI76008, SGI76009, SGI76010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76011, SGI0, SGI0, SGI0, SGI76012, SGI76012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16880[] = {SGI76013, SGI0, SGI76015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76018, SGI0, SGI0, SGI0, SGI76019, SGI76019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16881[] = {SGI76014, SGI76016, SGI76017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76020, SGI0, SGI0, SGI0, SGI76021, SGI76021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16882[] = {SGI76022, SGI0, SGI76025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76027, SGI0, SGI0, SGI0, SGI76029, SGI76029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16883[] = {SGI76023, SGI76024, SGI76026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76028, SGI0, SGI0, SGI0, SGI76030, SGI76030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16884[] = {SGI76031, SGI0, SGI76032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16885[] = {SGI0, SGI76033, SGI76034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16886[] = {SGI76035, SGI76036, SGI76037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76038, SGI0, SGI0, SGI0, SGI76039, SGI76039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16887[] = {SGI76040, SGI0, SGI76041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76042, SGI0, SGI0, SGI0, SGI76043, SGI76043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16888[] = {SGI76044, SGI0, SGI76045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76046, SGI0, SGI0, SGI0, SGI76047, SGI76047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16889[] = {SGI76048, SGI76049, SGI76050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76051, SGI0, SGI0, SGI0, SGI76052, SGI76052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16890[] = {SGI76053, SGI76054, SGI76055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76056, SGI0, SGI0, SGI0, SGI76057, SGI76057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16891[] = {SGI76058, SGI76059, SGI76060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76061, SGI0, SGI0, SGI0, SGI76062, SGI76062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16892[] = {SGI76063, SGI0, SGI76064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16893[] = {SGI76065, SGI76066, SGI76067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76068, SGI0, SGI0, SGI0, SGI76069, SGI76069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16894[] = {SGI76070, SGI76071, SGI76072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76073, SGI0, SGI0, SGI0, SGI76074, SGI76074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16895[] = {SGI76075, SGI76076, SGI76077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76078, SGI0, SGI0, SGI0, SGI76079, SGI76079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16896[] = {SGI76080, SGI0, SGI76081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16897[] = {SGI76082, SGI0, SGI76083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16898[] = {SGI76084, SGI76085, SGI76086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76087, SGI0, SGI0, SGI0, SGI76088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16899[] = {SGI76089, SGI76090, SGI76091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76092, SGI0, SGI0, SGI0, SGI76093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16900[] = {SGI76094, SGI0, SGI76095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16901[] = {SGI76096, SGI76097, SGI76098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76099, SGI0, SGI0, SGI0, SGI76100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16902[] = {SGI76101, SGI76102, SGI76103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76104, SGI0, SGI0, SGI0, SGI76105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16903[] = {SGI76106, SGI76107, SGI76108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76109, SGI0, SGI0, SGI0, SGI76110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16904[] = {SGI76111, SGI76112, SGI76113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76114, SGI0, SGI0, SGI0, SGI76115, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16905[] = {SGI76116, SGI76117, SGI76118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76119, SGI0, SGI0, SGI0, SGI76120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16906[] = {SGI76121, SGI76122, SGI76123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76124, SGI0, SGI0, SGI0, SGI76125, SGI76125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16907[] = {SGI76126, SGI76127, SGI76128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76129, SGI0, SGI0, SGI0, SGI76130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16908[] = {SGI76131, SGI76132, SGI76133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76134, SGI0, SGI0, SGI0, SGI76135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16909[] = {SGI76136, SGI76137, SGI76138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76139, SGI0, SGI0, SGI0, SGI76140, SGI76140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16910[] = {SGI76141, SGI76142, SGI76143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76144, SGI0, SGI0, SGI0, SGI76145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16911[] = {SGI76146, SGI0, SGI76147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16912[] = {SGI76148, SGI0, SGI76149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76150, SGI0, SGI0, SGI0, SGI76151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16913[] = {SGI76152, SGI76153, SGI76154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76155, SGI76155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16914[] = {SGI76156, SGI76157, SGI76158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76159, SGI0, SGI0, SGI0, SGI76160, SGI76161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16915[] = {SGI76162, SGI76163, SGI76164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76165, SGI0, SGI0, SGI0, SGI76166, SGI76166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16916[] = {SGI76167, SGI76168, SGI76169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76170, SGI76170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16917[] = {SGI76171, SGI76172, SGI76173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16918[] = {SGI76174, SGI76175, SGI76176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76177, SGI0, SGI0, SGI0, SGI76178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16919[] = {SGI76179, SGI76180, SGI76181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16920[] = {SGI76182, SGI76183, SGI76184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16921[] = {SGI76185, SGI76186, SGI76187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76188, SGI0, SGI0, SGI0, SGI76191, SGI76191, SGI0, SGI76193, SGI0, NULL};
const Segment * const SG16922[] = {SGI76189, SGI76190, SGI76192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76194, SGI0, SGI0, SGI0, SGI76195, SGI76196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16923[] = {SGI76197, SGI76198, SGI76199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76200, SGI0, SGI0, SGI0, SGI76201, SGI76201, SGI0, SGI76202, SGI0, NULL};
const Segment * const SG16924[] = {SGI76203, SGI0, SGI76204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76205, SGI0, NULL};
const Segment * const SG16925[] = {SGI76206, SGI76207, SGI76208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16926[] = {SGI76209, SGI76210, SGI76211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76212, SGI0, SGI0, SGI0, SGI76213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16927[] = {SGI76214, SGI76215, SGI76216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76217, SGI0, SGI0, SGI0, SGI76218, SGI76218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16928[] = {SGI76219, SGI76220, SGI76221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76222, SGI0, SGI0, SGI0, SGI76223, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16929[] = {SGI76224, SGI76225, SGI76226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76227, SGI0, SGI0, SGI0, SGI76228, SGI76228, SGI0, SGI76229, SGI0, NULL};
const Segment * const SG16930[] = {SGI76230, SGI76231, SGI76232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76233, SGI0, SGI0, SGI0, SGI76234, SGI76234, SGI0, SGI76235, SGI0, NULL};
const Segment * const SG16931[] = {SGI76236, SGI76237, SGI76238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76239, SGI0, SGI0, SGI0, SGI76240, SGI76240, SGI0, SGI76241, SGI0, NULL};
const Segment * const SG16932[] = {SGI76242, SGI76243, SGI76244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76245, SGI0, SGI0, SGI0, SGI76246, SGI76246, SGI0, SGI76247, SGI0, NULL};
const Segment * const SG16933[] = {SGI76248, SGI76249, SGI76250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76251, SGI76252, SGI0, SGI76253, SGI0, NULL};
const Segment * const SG16934[] = {SGI76254, SGI76255, SGI76256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76257, SGI0, SGI0, SGI0, SGI76258, SGI76258, SGI0, SGI76259, SGI0, NULL};
const Segment * const SG16935[] = {SGI76260, SGI76261, SGI76262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76263, SGI0, SGI0, SGI0, SGI76264, SGI76264, SGI0, SGI76265, SGI0, NULL};
const Segment * const SG16936[] = {SGI76266, SGI76267, SGI76268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76269, SGI0, SGI0, SGI0, SGI76270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16937[] = {SGI76271, SGI76272, SGI76273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76274, SGI0, SGI0, SGI0, SGI76275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16938[] = {SGI76276, SGI0, SGI76277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16939[] = {SGI76278, SGI76279, SGI76280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76281, SGI0, SGI0, SGI0, SGI76282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16940[] = {SGI76283, SGI0, SGI76284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16941[] = {SGI76285, SGI76286, SGI76287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76288, SGI0, SGI0, SGI0, SGI76289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16942[] = {SGI76290, SGI76291, SGI76292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76293, SGI0, SGI0, SGI0, SGI76294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16943[] = {SGI76295, SGI76296, SGI76297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76301, SGI0, SGI0, SGI0, SGI76302, SGI76302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16944[] = {SGI76298, SGI76299, SGI76300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76303, SGI0, SGI0, SGI0, SGI76304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16945[] = {SGI76305, SGI76306, SGI76308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76311, SGI0, SGI0, SGI0, SGI76313, SGI76313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16946[] = {SGI76307, SGI76309, SGI76310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76312, SGI0, SGI0, SGI0, SGI76314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16947[] = {SGI76315, SGI76316, SGI76317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76321, SGI0, SGI0, SGI0, SGI76323, SGI76323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16948[] = {SGI76318, SGI76319, SGI76320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76322, SGI0, SGI0, SGI0, SGI76324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16949[] = {SGI76325, SGI76326, SGI76327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76328, SGI0, SGI0, SGI0, SGI76329, SGI76329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16950[] = {SGI76331, SGI76333, SGI76335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76337, SGI0, SGI0, SGI0, SGI76339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16951[] = {SGI76330, SGI76332, SGI76334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76336, SGI0, SGI0, SGI0, SGI76338, SGI76338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16952[] = {SGI76340, SGI0, SGI76341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16953[] = {SGI76342, SGI0, SGI76343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16954[] = {SGI76344, SGI76345, SGI76346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76347, SGI0, SGI0, SGI0, SGI76348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16955[] = {SGI76349, SGI76350, SGI76351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76352, SGI0, SGI0, SGI0, SGI76355, SGI76355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16956[] = {SGI76353, SGI76354, SGI76356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76357, SGI0, SGI0, SGI0, SGI76358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16957[] = {SGI76359, SGI76360, SGI76361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76363, SGI0, SGI0, SGI0, SGI76366, SGI76366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16958[] = {SGI76362, SGI76364, SGI76365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76367, SGI0, SGI0, SGI0, SGI76368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16959[] = {SGI76369, SGI76370, SGI76371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76372, SGI0, SGI0, SGI0, SGI76373, SGI76373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16960[] = {SGI76374, SGI76375, SGI76376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76377, SGI0, SGI0, SGI0, SGI76378, SGI76378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16961[] = {SGI76379, SGI0, SGI76380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16962[] = {SGI76381, SGI0, SGI76382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16963[] = {SGI76383, SGI0, SGI76384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76385, SGI0, SGI0, SGI0, SGI76386, SGI76386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16964[] = {SGI76387, SGI76388, SGI76389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76390, SGI0, SGI0, SGI0, SGI76391, SGI76391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16965[] = {SGI76392, SGI76393, SGI76394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76395, SGI0, SGI0, SGI0, SGI76396, SGI76396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16966[] = {SGI76397, SGI76398, SGI76399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76400, SGI76400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16967[] = {SGI76401, SGI76402, SGI76403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76404, SGI0, SGI0, SGI0, SGI76405, SGI76405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16968[] = {SGI76406, SGI76407, SGI76408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76409, SGI0, SGI0, SGI0, SGI76410, SGI76410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16969[] = {SGI76411, SGI76412, SGI76413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76414, SGI0, SGI0, SGI0, SGI76415, SGI76415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16970[] = {SGI76416, SGI76417, SGI76418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76419, SGI0, SGI0, SGI0, SGI76420, SGI76420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16971[] = {SGI76421, SGI76422, SGI76423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76424, SGI0, SGI0, SGI0, SGI76425, SGI76425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16972[] = {SGI76426, SGI76427, SGI76428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76429, SGI0, SGI0, SGI0, SGI76430, SGI76430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16973[] = {SGI76431, SGI76432, SGI76433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76434, SGI0, SGI0, SGI0, SGI76435, SGI76435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16974[] = {SGI76436, SGI0, SGI76437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16975[] = {SGI76438, SGI76439, SGI76440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76441, SGI0, SGI0, SGI0, SGI76442, SGI76442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16976[] = {SGI76443, SGI0, SGI76444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16977[] = {SGI76445, SGI76446, SGI76447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76448, SGI0, SGI0, SGI0, SGI76449, SGI76449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16978[] = {SGI76450, SGI76451, SGI76452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76453, SGI0, SGI0, SGI0, SGI76454, SGI76454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16979[] = {SGI76455, SGI76456, SGI76457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76458, SGI0, SGI0, SGI0, SGI76459, SGI76459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16980[] = {SGI76460, SGI76461, SGI76462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76463, SGI0, SGI0, SGI0, SGI76464, SGI76464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16981[] = {SGI76465, SGI76466, SGI76467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76468, SGI0, SGI0, SGI0, SGI76469, SGI76469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16982[] = {SGI76470, SGI0, SGI76471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16983[] = {SGI76472, SGI76473, SGI76474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76475, SGI0, SGI0, SGI0, SGI76476, SGI76476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16984[] = {SGI76477, SGI76478, SGI76479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76480, SGI76481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16985[] = {SGI76482, SGI76483, SGI76484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76485, SGI0, SGI0, SGI0, SGI76486, SGI76486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16986[] = {SGI76487, SGI76488, SGI76489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76490, SGI0, SGI0, SGI0, SGI76491, SGI76491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16987[] = {SGI76492, SGI76493, SGI76494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76495, SGI76495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16988[] = {SGI76496, SGI76497, SGI76498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76499, SGI0, SGI0, SGI0, SGI76500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16989[] = {SGI76501, SGI76502, SGI76503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76504, SGI0, SGI0, SGI0, SGI76505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16990[] = {SGI76506, SGI0, SGI76507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16991[] = {SGI76508, SGI76509, SGI76510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76511, SGI0, SGI0, SGI0, SGI76512, SGI76512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16992[] = {SGI76513, SGI76514, SGI76515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76516, SGI0, SGI0, SGI0, SGI76517, SGI76517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16993[] = {SGI76518, SGI0, SGI76519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16994[] = {SGI76520, SGI76521, SGI76522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76523, SGI0, SGI0, SGI0, SGI76524, SGI76524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16995[] = {SGI76525, SGI0, SGI76526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16996[] = {SGI76527, SGI76528, SGI76529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76530, SGI0, SGI0, SGI0, SGI76531, SGI76531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16997[] = {SGI76532, SGI76533, SGI76534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76535, SGI0, SGI0, SGI0, SGI76536, SGI76536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16998[] = {SGI76537, SGI76538, SGI76539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76540, SGI0, SGI0, SGI0, SGI76541, SGI76541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG16999[] = {SGI76542, SGI0, SGI76543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17000[] = {SGI76544, SGI76545, SGI76546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76547, SGI0, SGI0, SGI0, SGI76548, SGI76548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17001[] = {SGI76549, SGI76550, SGI76551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76552, SGI0, SGI0, SGI0, SGI76553, SGI76553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17002[] = {SGI76554, SGI76555, SGI76556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76557, SGI0, SGI0, SGI0, SGI76558, SGI76558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17003[] = {SGI76559, SGI0, SGI76560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76561, SGI0, SGI0, SGI0, SGI76562, SGI76562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17004[] = {SGI76563, SGI76564, SGI76565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76566, SGI0, SGI0, SGI0, SGI76567, SGI76567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17005[] = {SGI76568, SGI76569, SGI76570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76571, SGI0, SGI0, SGI0, SGI76572, SGI76572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17006[] = {SGI76573, SGI76574, SGI76575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76576, SGI0, SGI0, SGI0, SGI76577, SGI76577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17007[] = {SGI76578, SGI76579, SGI76580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76581, SGI0, SGI0, SGI0, SGI76582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17008[] = {SGI76583, SGI0, SGI76584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17009[] = {SGI76585, SGI76586, SGI76587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17010[] = {SGI76588, SGI76589, SGI76590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17011[] = {SGI76591, SGI76592, SGI76593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76594, SGI0, SGI0, SGI0, SGI76595, SGI76595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17012[] = {SGI76596, SGI76597, SGI76598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76599, SGI0, SGI0, SGI0, SGI76600, SGI76600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17013[] = {SGI76601, SGI0, SGI76602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17014[] = {SGI76603, SGI76604, SGI76605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17015[] = {SGI76606, SGI0, SGI76607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17016[] = {SGI76608, SGI76609, SGI76610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76611, SGI0, SGI0, SGI0, SGI76612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17017[] = {SGI76613, SGI76614, SGI76615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76616, SGI0, SGI0, SGI0, SGI76617, SGI76617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17018[] = {SGI76618, SGI76619, SGI76620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76621, SGI0, SGI0, SGI0, SGI76622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17019[] = {SGI76623, SGI76624, SGI76625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76626, SGI0, SGI0, SGI0, SGI76627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17020[] = {SGI76628, SGI76629, SGI76630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76631, SGI0, SGI0, SGI0, SGI76632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17021[] = {SGI76633, SGI0, SGI76634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76635, SGI0, SGI0, SGI0, SGI76636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17022[] = {SGI76637, SGI76638, SGI76639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76640, SGI0, SGI0, SGI0, SGI76641, SGI76641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17023[] = {SGI76642, SGI76643, SGI76644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76645, SGI0, SGI0, SGI0, SGI76646, SGI76646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17024[] = {SGI76647, SGI76648, SGI76649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76650, SGI0, SGI0, SGI0, SGI76651, SGI76651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17025[] = {SGI76652, SGI76653, SGI76654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76655, SGI0, SGI0, SGI0, SGI76656, SGI76656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17026[] = {SGI76657, SGI76658, SGI76659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76660, SGI0, SGI0, SGI0, SGI76661, SGI76661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17027[] = {SGI76662, SGI0, SGI76663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17028[] = {SGI76664, SGI76665, SGI76666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76667, SGI0, SGI0, SGI0, SGI76668, SGI76668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17029[] = {SGI76669, SGI76670, SGI76671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76672, SGI0, SGI0, SGI0, SGI76673, SGI76673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17030[] = {SGI76674, SGI76675, SGI76676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76677, SGI0, SGI0, SGI0, SGI76678, SGI76678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17031[] = {SGI76679, SGI76680, SGI76681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76682, SGI0, SGI0, SGI0, SGI76683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17032[] = {SGI76684, SGI76685, SGI76686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76687, SGI0, SGI0, SGI0, SGI76688, SGI76688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17033[] = {SGI76690, SGI0, SGI76692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17034[] = {SGI76689, SGI76691, SGI76693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76694, SGI0, SGI0, SGI0, SGI76695, SGI76695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17035[] = {SGI76699, SGI0, SGI76700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17036[] = {SGI76696, SGI76697, SGI76698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76701, SGI0, SGI0, SGI0, SGI76702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17037[] = {SGI76703, SGI76704, SGI76705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76706, SGI0, SGI0, SGI0, SGI76707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17038[] = {SGI76708, SGI76709, SGI76710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76711, SGI0, SGI0, SGI0, SGI76712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17039[] = {SGI76713, SGI76714, SGI76715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76716, SGI0, SGI0, SGI0, SGI76717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17040[] = {SGI76718, SGI76719, SGI76720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76721, SGI0, SGI0, SGI0, SGI76722, SGI76722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17041[] = {SGI76723, SGI76724, SGI76725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76726, SGI0, SGI0, SGI0, SGI76727, SGI76727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17042[] = {SGI76728, SGI76729, SGI76730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76731, SGI0, SGI0, SGI0, SGI76732, SGI76733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17043[] = {SGI76734, SGI0, SGI76735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76736, SGI0, SGI0, SGI0, SGI76737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17044[] = {SGI76738, SGI76739, SGI76740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17045[] = {SGI76741, SGI76742, SGI76743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76744, SGI0, SGI0, SGI0, SGI76745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17046[] = {SGI76746, SGI76747, SGI76748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17047[] = {SGI76749, SGI76750, SGI76751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17048[] = {SGI76752, SGI76753, SGI76754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17049[] = {SGI76755, SGI76756, SGI76757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76758, SGI0, SGI0, SGI0, SGI76759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17050[] = {SGI76760, SGI76761, SGI76762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76763, SGI0, SGI0, SGI0, SGI76764, SGI76764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17051[] = {SGI76765, SGI76766, SGI76767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76768, SGI0, SGI0, SGI0, SGI76769, SGI76769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17052[] = {SGI76770, SGI76771, SGI76772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76773, SGI76773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17053[] = {SGI76774, SGI76775, SGI76776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76777, SGI0, SGI0, SGI0, SGI76778, SGI76778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17054[] = {SGI76779, SGI76780, SGI76781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76782, SGI0, SGI0, SGI0, SGI76783, SGI76783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17055[] = {SGI76784, SGI76785, SGI76786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76787, SGI0, SGI0, SGI0, SGI76788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17056[] = {SGI76789, SGI76790, SGI76791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76792, SGI0, SGI0, SGI0, SGI76793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17057[] = {SGI76794, SGI0, SGI76795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17058[] = {SGI76796, SGI0, SGI76797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17059[] = {SGI76798, SGI76799, SGI76800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76801, SGI0, SGI0, SGI0, SGI76802, SGI76802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17060[] = {SGI76803, SGI0, SGI76804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17061[] = {SGI76805, SGI76806, SGI76807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76808, SGI0, SGI0, SGI0, SGI76809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17062[] = {SGI76810, SGI76811, SGI76812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76813, SGI0, SGI0, SGI0, SGI76814, SGI76814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17063[] = {SGI76815, SGI76816, SGI76817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76818, SGI0, SGI0, SGI0, SGI76819, SGI76819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17064[] = {SGI76820, SGI76821, SGI76822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76823, SGI0, SGI0, SGI0, SGI76824, SGI76824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17065[] = {SGI76825, SGI76826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76827, SGI0, SGI0, SGI0, SGI76828, SGI76828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17066[] = {SGI76829, SGI76830, SGI76831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76832, SGI0, SGI0, SGI0, SGI76833, SGI76833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17067[] = {SGI76834, SGI76835, SGI76836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76837, SGI0, SGI0, SGI0, SGI76838, SGI76838, SGI0, SGI76839, SGI0, NULL};
const Segment * const SG17068[] = {SGI76840, SGI0, SGI76841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17069[] = {SGI76842, SGI76843, SGI76844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76845, SGI0, SGI0, SGI0, SGI76846, SGI76846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17070[] = {SGI76847, SGI76848, SGI76849, SGI0, SGI0, SGI0, SGI76850, SGI76851, SGI0, SGI76852, SGI0, SGI0, SGI0, SGI76853, SGI76853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17071[] = {SGI76854, SGI0, SGI76855, SGI0, SGI0, SGI0, SGI76856, SGI76857, SGI0, SGI76858, SGI0, SGI0, SGI0, SGI76859, SGI76859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17072[] = {SGI76860, SGI76861, SGI76862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76863, SGI0, SGI0, SGI0, SGI76864, SGI76864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17073[] = {SGI76865, SGI76866, SGI76867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76868, SGI0, SGI0, SGI0, SGI76869, SGI76869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17074[] = {SGI76870, SGI76871, SGI76872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76873, SGI76874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17075[] = {SGI76875, SGI0, SGI76876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17076[] = {SGI76877, SGI0, SGI76878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76879, SGI0, SGI0, SGI0, SGI76880, SGI76880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17077[] = {SGI76881, SGI76882, SGI76883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76884, SGI0, SGI0, SGI0, SGI76885, SGI76885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17078[] = {SGI76886, SGI0, SGI76887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17079[] = {SGI76888, SGI76889, SGI76890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76891, SGI0, SGI0, SGI0, SGI76892, SGI76892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17080[] = {SGI76893, SGI76894, SGI76895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76896, SGI0, SGI0, SGI0, SGI76897, SGI76897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17081[] = {SGI76898, SGI0, SGI76899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76900, SGI0, SGI0, SGI0, SGI76901, SGI76901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17082[] = {SGI76902, SGI0, SGI76903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76904, SGI0, SGI0, SGI0, SGI76905, SGI76905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17083[] = {SGI76906, SGI0, SGI76907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17084[] = {SGI76908, SGI76909, SGI76910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76911, SGI0, SGI0, SGI0, SGI76912, SGI76912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17085[] = {SGI76913, SGI76914, SGI76915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76916, SGI0, SGI0, SGI0, SGI76917, SGI76917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17086[] = {SGI76918, SGI0, SGI76919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17087[] = {SGI76920, SGI76921, SGI76922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76923, SGI0, SGI0, SGI0, SGI76924, SGI76924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17088[] = {SGI76925, SGI76926, SGI76927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76928, SGI0, SGI0, SGI0, SGI76929, SGI76929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17089[] = {SGI76930, SGI76931, SGI76932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76933, SGI76933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17090[] = {SGI76934, SGI76935, SGI76936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76937, SGI0, SGI0, SGI0, SGI76938, SGI76938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17091[] = {SGI76939, SGI0, SGI76940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17092[] = {SGI76941, SGI0, SGI76942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17093[] = {SGI76943, SGI76944, SGI76945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76946, SGI0, SGI0, SGI0, SGI76947, SGI76947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17094[] = {SGI76948, SGI76949, SGI76950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76951, SGI0, SGI0, SGI0, SGI76952, SGI76952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17095[] = {SGI76953, SGI76954, SGI76955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76956, SGI0, SGI0, SGI0, SGI76957, SGI76957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17096[] = {SGI76958, SGI0, SGI76959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17097[] = {SGI76960, SGI76961, SGI76962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76963, SGI0, SGI0, SGI0, SGI76964, SGI76964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17098[] = {SGI76965, SGI76966, SGI76967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76968, SGI0, SGI0, SGI0, SGI76969, SGI0, SGI0, SGI76970, SGI0, NULL};
const Segment * const SG17099[] = {SGI76971, SGI76972, SGI76973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76974, SGI0, SGI0, SGI0, SGI76975, SGI0, SGI0, SGI76976, SGI0, NULL};
const Segment * const SG17100[] = {SGI76977, SGI76978, SGI76979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76980, SGI0, SGI0, SGI0, SGI76981, SGI76981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17101[] = {SGI76982, SGI76983, SGI76984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76985, SGI0, SGI0, SGI0, SGI76986, SGI76986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17102[] = {SGI76987, SGI0, SGI76988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17103[] = {SGI76989, SGI76990, SGI76991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76992, SGI0, SGI0, SGI0, SGI76993, SGI76993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76994, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17104[] = {SGI76995, SGI76996, SGI76997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI76998, SGI0, SGI0, SGI0, SGI76999, SGI76999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17105[] = {SGI77000, SGI77001, SGI77002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77003, SGI0, SGI0, SGI0, SGI77004, SGI77004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17106[] = {SGI77005, SGI77006, SGI77007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77008, SGI0, SGI0, SGI0, SGI77009, SGI77009, SGI77010, SGI0, SGI77011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17107[] = {SGI77012, SGI77013, SGI77014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77015, SGI0, SGI0, SGI0, SGI77016, SGI77016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17108[] = {SGI77017, SGI77018, SGI77019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77020, SGI0, SGI0, SGI0, SGI77021, SGI77021, SGI77022, SGI0, SGI77023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17109[] = {SGI77024, SGI77025, SGI77026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77027, SGI0, SGI0, SGI0, SGI77028, SGI77028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17110[] = {SGI77029, SGI0, SGI77030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17111[] = {SGI77031, SGI77032, SGI77033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77034, SGI0, SGI0, SGI0, SGI77035, SGI77035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17112[] = {SGI77036, SGI0, SGI77037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17113[] = {SGI77038, SGI77039, SGI77040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77041, SGI0, SGI0, SGI0, SGI77042, SGI77042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17114[] = {SGI77043, SGI77044, SGI77045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77046, SGI0, SGI0, SGI0, SGI77047, SGI77047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17115[] = {SGI77048, SGI77049, SGI77050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77051, SGI0, SGI0, SGI0, SGI77052, SGI77052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17116[] = {SGI77053, SGI77054, SGI77055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77056, SGI0, SGI0, SGI0, SGI77057, SGI77057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17117[] = {SGI77058, SGI77059, SGI77060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77061, SGI0, SGI0, SGI0, SGI77062, SGI77062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17118[] = {SGI77063, SGI77064, SGI77065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77066, SGI0, SGI0, SGI0, SGI77067, SGI77067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17119[] = {SGI77068, SGI77069, SGI77070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77071, SGI0, SGI0, SGI0, SGI77072, SGI77072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17120[] = {SGI77073, SGI77074, SGI77075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77076, SGI0, SGI0, SGI0, SGI77077, SGI77077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17121[] = {SGI77078, SGI77079, SGI77080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77081, SGI0, SGI0, SGI0, SGI77082, SGI77082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17122[] = {SGI77083, SGI77084, SGI77085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77086, SGI0, SGI0, SGI0, SGI77087, SGI77087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17123[] = {SGI77088, SGI77089, SGI77090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77091, SGI0, SGI0, SGI0, SGI77092, SGI77092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17124[] = {SGI77093, SGI0, SGI77094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17125[] = {SGI77095, SGI77096, SGI77097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77098, SGI0, SGI0, SGI0, SGI77099, SGI77099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17126[] = {SGI77100, SGI77101, SGI77102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77103, SGI0, SGI0, SGI0, SGI77104, SGI77104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17127[] = {SGI77105, SGI77106, SGI77107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77108, SGI0, SGI0, SGI0, SGI77109, SGI77109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17128[] = {SGI77110, SGI77111, SGI77112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77113, SGI0, SGI0, SGI0, SGI77114, SGI77114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17129[] = {SGI77115, SGI77116, SGI77117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77118, SGI0, SGI0, SGI0, SGI77119, SGI77119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17130[] = {SGI77120, SGI77121, SGI77122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77123, SGI0, SGI0, SGI0, SGI77124, SGI77124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17131[] = {SGI77125, SGI77126, SGI77127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77128, SGI0, SGI0, SGI0, SGI77129, SGI77129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17132[] = {SGI77130, SGI77131, SGI77132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77133, SGI0, SGI0, SGI0, SGI77134, SGI77134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17133[] = {SGI77135, SGI77136, SGI77137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77138, SGI0, SGI0, SGI0, SGI77139, SGI77139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17134[] = {SGI77140, SGI77141, SGI77142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77143, SGI0, SGI0, SGI0, SGI77144, SGI77144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17135[] = {SGI77145, SGI77146, SGI77147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77148, SGI0, SGI0, SGI0, SGI77149, SGI77149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17136[] = {SGI77150, SGI77151, SGI77152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77153, SGI0, SGI0, SGI0, SGI77154, SGI77154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17137[] = {SGI77156, SGI0, SGI77158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17138[] = {SGI77155, SGI0, SGI77157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17139[] = {SGI77159, SGI77160, SGI77161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77162, SGI0, SGI0, SGI0, SGI77163, SGI77163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17140[] = {SGI77164, SGI77165, SGI77166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77167, SGI0, SGI0, SGI0, SGI77168, SGI77168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17141[] = {SGI77169, SGI0, SGI77170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17142[] = {SGI77171, SGI77172, SGI77173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77174, SGI0, SGI0, SGI0, SGI77175, SGI77175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17143[] = {SGI77176, SGI77177, SGI77178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77179, SGI0, SGI0, SGI0, SGI77180, SGI77180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17144[] = {SGI77181, SGI77182, SGI77183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77184, SGI0, SGI0, SGI0, SGI77185, SGI77185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17145[] = {SGI77186, SGI77187, SGI77188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77189, SGI0, SGI0, SGI0, SGI77190, SGI77190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17146[] = {SGI77191, SGI77192, SGI77193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77194, SGI0, SGI0, SGI0, SGI77195, SGI77195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17147[] = {SGI77196, SGI77197, SGI77198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77199, SGI0, SGI0, SGI0, SGI77200, SGI77200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17148[] = {SGI77201, SGI77202, SGI77203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77204, SGI0, SGI0, SGI0, SGI77205, SGI77205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17149[] = {SGI77206, SGI77207, SGI77208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77209, SGI0, SGI0, SGI0, SGI77210, SGI77210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17150[] = {SGI77211, SGI77212, SGI77213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77214, SGI0, SGI0, SGI0, SGI77215, SGI77215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17151[] = {SGI77216, SGI77217, SGI77218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77219, SGI77219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17152[] = {SGI77220, SGI77221, SGI77222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77223, SGI0, SGI0, SGI0, SGI77224, SGI77224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17153[] = {SGI77225, SGI77226, SGI77227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77228, SGI0, SGI0, SGI0, SGI77229, SGI77229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17154[] = {SGI77230, SGI77231, SGI77232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77233, SGI0, SGI0, SGI0, SGI77234, SGI77234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17155[] = {SGI77235, SGI77236, SGI77237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77238, SGI0, SGI0, SGI0, SGI77239, SGI77239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17156[] = {SGI77240, SGI77241, SGI77242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77243, SGI0, SGI0, SGI0, SGI77244, SGI77244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17157[] = {SGI77245, SGI77246, SGI77247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77248, SGI0, SGI0, SGI0, SGI77249, SGI77249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17158[] = {SGI77250, SGI77251, SGI77252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77253, SGI0, SGI0, SGI0, SGI77254, SGI77254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17159[] = {SGI77255, SGI77256, SGI77257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77258, SGI77258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17160[] = {SGI77259, SGI0, SGI77260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17161[] = {SGI77261, SGI77262, SGI77263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77264, SGI77265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17162[] = {SGI77266, SGI77267, SGI77268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77269, SGI0, SGI0, SGI0, SGI77270, SGI77270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17163[] = {SGI77271, SGI77272, SGI77273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77274, SGI77274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17164[] = {SGI77275, SGI77276, SGI77277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77278, SGI0, SGI0, SGI0, SGI77279, SGI77279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17165[] = {SGI77280, SGI77281, SGI77282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77283, SGI0, SGI0, SGI0, SGI77284, SGI77284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17166[] = {SGI77285, SGI0, SGI77286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17167[] = {SGI0, SGI77287, SGI77288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17168[] = {SGI77289, SGI77290, SGI77291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77292, SGI0, SGI0, SGI0, SGI77293, SGI77293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17169[] = {SGI77294, SGI0, SGI77295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17170[] = {SGI77296, SGI77297, SGI77298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77299, SGI0, SGI0, SGI0, SGI77300, SGI77300, SGI0, SGI0, NULL};
const Segment * const SG17171[] = {SGI77301, SGI77302, SGI77303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77304, SGI0, SGI0, SGI0, SGI77305, SGI77305, SGI0, SGI0, NULL};
const Segment * const SG17172[] = {SGI77306, SGI0, SGI77307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17173[] = {SGI77308, SGI77309, SGI77310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17174[] = {SGI77311, SGI77312, SGI77313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77314, SGI0, SGI0, SGI0, SGI77315, SGI77315, SGI0, SGI0, NULL};
const Segment * const SG17175[] = {SGI77316, SGI77317, SGI77318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77319, SGI77320, SGI0, SGI0, NULL};
const Segment * const SG17176[] = {SGI77321, SGI77322, SGI77323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77324, SGI77324, SGI0, SGI0, NULL};
const Segment * const SG17177[] = {SGI77325, SGI77326, SGI77327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77328, SGI0, SGI0, SGI0, SGI77329, SGI77329, SGI0, SGI0, NULL};
const Segment * const SG17178[] = {SGI77330, SGI77331, SGI77332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77333, SGI0, SGI0, SGI0, SGI77334, SGI77334, SGI0, SGI0, NULL};
const Segment * const SG17179[] = {SGI77335, SGI77336, SGI77337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77338, SGI0, SGI0, SGI0, SGI77339, SGI77339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17180[] = {SGI77340, SGI0, SGI77341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17181[] = {SGI77342, SGI77343, SGI77344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77345, SGI0, SGI0, SGI0, SGI77346, SGI77346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17182[] = {SGI77347, SGI0, SGI77348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77349, SGI0, SGI0, SGI0, SGI77350, SGI77350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17183[] = {SGI77351, SGI0, SGI77352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17184[] = {SGI77353, SGI77354, SGI77355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77356, SGI0, SGI0, SGI0, SGI77357, SGI77357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17185[] = {SGI77358, SGI77359, SGI77360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77361, SGI0, SGI0, SGI0, SGI77362, SGI77362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17186[] = {SGI77363, SGI77364, SGI77365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77366, SGI77366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17187[] = {SGI77367, SGI0, SGI77368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17188[] = {SGI77369, SGI77370, SGI77371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77372, SGI0, SGI0, SGI0, SGI77373, SGI77373, SGI0, SGI77374, SGI0, NULL};
const Segment * const SG17189[] = {SGI77375, SGI77376, SGI77377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77378, SGI0, SGI0, SGI0, SGI77379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17190[] = {SGI77381, SGI0, SGI77384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17191[] = {SGI77380, SGI77382, SGI77383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77385, SGI0, SGI0, SGI0, SGI77386, SGI77386, SGI0, SGI77387, SGI0, NULL};
const Segment * const SG17192[] = {SGI77388, SGI77389, SGI77390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77391, SGI0, SGI0, SGI0, SGI77392, SGI77392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17193[] = {SGI77393, SGI0, SGI77394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77395, SGI0, SGI0, SGI0, SGI77396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17194[] = {SGI77397, SGI77398, SGI77399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17195[] = {SGI77400, SGI77401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17196[] = {SGI77402, SGI77403, SGI77404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77405, SGI77405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17197[] = {SGI77406, SGI77407, SGI77408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77409, SGI77409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17198[] = {SGI77410, SGI77411, SGI77412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17199[] = {SGI0, SGI77413, SGI77414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17200[] = {SGI77415, SGI77416, SGI77417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77418, SGI77419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17201[] = {SGI77420, SGI77421, SGI77422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77423, SGI0, SGI0, SGI0, SGI77424, SGI77424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17202[] = {SGI77425, SGI77426, SGI77427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77428, SGI0, SGI0, SGI0, SGI77429, SGI77430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17203[] = {SGI77431, SGI77432, SGI77433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77434, SGI0, SGI0, SGI0, SGI77435, SGI77435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17204[] = {SGI77436, SGI77437, SGI77438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77439, SGI0, SGI0, SGI0, SGI77440, SGI77440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17205[] = {SGI77441, SGI77442, SGI77443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77444, SGI0, SGI0, SGI0, SGI77445, SGI77445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17206[] = {SGI77446, SGI77447, SGI77448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77449, SGI0, SGI0, SGI0, SGI77450, SGI77450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17207[] = {SGI77451, SGI77452, SGI77453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77454, SGI0, SGI0, SGI0, SGI77455, SGI77455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17208[] = {SGI77456, SGI0, SGI77457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17209[] = {SGI77458, SGI77459, SGI77460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77461, SGI0, SGI0, SGI0, SGI77462, SGI77462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17210[] = {SGI77463, SGI77464, SGI77465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77466, SGI0, SGI0, SGI0, SGI77467, SGI77467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17211[] = {SGI77468, SGI77469, SGI77470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77471, SGI0, SGI0, SGI0, SGI77472, SGI77472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17212[] = {SGI77473, SGI0, SGI77474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17213[] = {SGI77475, SGI77476, SGI77477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77478, SGI0, SGI0, SGI0, SGI77479, SGI77479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17214[] = {SGI77480, SGI0, SGI77481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77482, SGI0, SGI0, SGI0, SGI77483, SGI77483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17215[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77484, SGI0, SGI0, SGI77485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17216[] = {SGI77486, SGI77487, SGI77488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77489, SGI0, SGI0, SGI0, SGI77490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17217[] = {SGI77491, SGI0, SGI77492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17218[] = {SGI77493, SGI77494, SGI77495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77496, SGI0, SGI0, SGI0, SGI77497, SGI77497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17219[] = {SGI77498, SGI0, SGI77499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77500, SGI0, SGI0, SGI0, SGI77501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17220[] = {SGI77502, SGI77503, SGI77504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17221[] = {SGI77505, SGI77506, SGI77507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17222[] = {SGI77508, SGI77509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17223[] = {SGI77510, SGI77511, SGI77512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77513, SGI77514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17224[] = {SGI77515, SGI77516, SGI77517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77518, SGI0, SGI0, SGI0, SGI77519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17225[] = {SGI77520, SGI77521, SGI77522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17226[] = {SGI0, SGI77523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17227[] = {SGI77524, SGI77525, SGI77526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77527, SGI0, SGI0, SGI0, SGI77528, SGI77529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17228[] = {SGI77530, SGI77531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17229[] = {SGI77532, SGI77533, SGI77534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77535, SGI0, SGI0, SGI0, SGI77536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17230[] = {SGI77537, SGI77538, SGI77539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77540, SGI77540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17231[] = {SGI77541, SGI77542, SGI77543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77544, SGI0, SGI0, SGI0, SGI77545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17232[] = {SGI77546, SGI77547, SGI77548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77549, SGI0, SGI0, SGI0, SGI77550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17233[] = {SGI77551, SGI0, SGI77552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17234[] = {SGI77553, SGI77554, SGI77555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77556, SGI0, SGI0, SGI0, SGI77557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17235[] = {SGI77558, SGI77559, SGI77560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77561, SGI0, SGI0, SGI0, SGI77562, SGI77562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17236[] = {SGI77563, SGI0, SGI77564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77565, SGI0, SGI0, SGI0, SGI77566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17237[] = {SGI77567, SGI77568, SGI77569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77570, SGI0, SGI0, SGI0, SGI77571, SGI77571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17238[] = {SGI77572, SGI0, SGI77573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17239[] = {SGI77574, SGI77575, SGI77576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77577, SGI0, SGI0, SGI0, SGI77578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17240[] = {SGI77579, SGI0, SGI77580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17241[] = {SGI77581, SGI77582, SGI77583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77584, SGI0, SGI0, SGI0, SGI77585, SGI77585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17242[] = {SGI77586, SGI77587, SGI77588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77589, SGI77589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17243[] = {SGI77590, SGI0, SGI77591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77592, SGI0, SGI0, SGI0, SGI77593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17244[] = {SGI77594, SGI77595, SGI77596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17245[] = {SGI77597, SGI77598, SGI77599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17246[] = {SGI77600, SGI77601, SGI77602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17247[] = {SGI77603, SGI77604, SGI77605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17248[] = {SGI77606, SGI77607, SGI77608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17249[] = {SGI77609, SGI77610, SGI77611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77612, SGI0, SGI0, SGI0, SGI77613, SGI77614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17250[] = {SGI77615, SGI77616, SGI77617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17251[] = {SGI77618, SGI77619, SGI77620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77621, SGI0, SGI0, SGI0, SGI77622, SGI77622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17252[] = {SGI77623, SGI0, SGI77624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17253[] = {SGI77625, SGI77626, SGI77627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17254[] = {SGI0, SGI77628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17255[] = {SGI77629, SGI77630, SGI77631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77632, SGI0, SGI0, SGI0, SGI77633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17256[] = {SGI0, SGI77634, SGI77635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17257[] = {SGI77636, SGI77637, SGI77638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77639, SGI0, SGI0, SGI0, SGI77640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17258[] = {SGI77641, SGI77642, SGI77643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77644, SGI0, SGI0, SGI0, SGI77645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17259[] = {SGI77646, SGI77647, SGI77648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77649, SGI0, SGI0, SGI0, SGI77650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17260[] = {SGI77651, SGI77652, SGI77653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77654, SGI0, SGI0, SGI0, SGI77655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17261[] = {SGI77656, SGI77657, SGI77658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77659, SGI0, SGI0, SGI0, SGI77660, SGI77660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17262[] = {SGI77661, SGI0, SGI77662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17263[] = {SGI77663, SGI77664, SGI77665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77666, SGI0, SGI0, SGI0, SGI77670, SGI77671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17264[] = {SGI77667, SGI77668, SGI77669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77672, SGI0, SGI0, SGI0, SGI77673, SGI77673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17265[] = {SGI77674, SGI77675, SGI77676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77678, SGI0, SGI0, SGI0, SGI77680, SGI77681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17266[] = {SGI77677, SGI0, SGI77679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17267[] = {SGI77682, SGI77683, SGI77684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17268[] = {SGI77685, SGI77686, SGI77687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77688, SGI77688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17269[] = {SGI77689, SGI77690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17270[] = {SGI77691, SGI77692, SGI77693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77694, SGI0, SGI0, SGI0, SGI77695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17271[] = {SGI77696, SGI77697, SGI77698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17272[] = {SGI77699, SGI0, SGI77700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77701, SGI0, SGI0, SGI0, SGI77702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17273[] = {SGI0, SGI0, SGI77703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17274[] = {SGI77704, SGI77705, SGI77706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77707, SGI0, SGI0, SGI0, SGI77708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17275[] = {SGI77709, SGI77710, SGI77711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77712, SGI0, SGI0, SGI0, SGI77713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17276[] = {SGI77714, SGI77715, SGI77716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17277[] = {SGI77717, SGI0, SGI77718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17278[] = {SGI77719, SGI77720, SGI77721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77722, SGI0, SGI0, SGI0, SGI77723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17279[] = {SGI77724, SGI77725, SGI77726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77727, SGI0, SGI0, SGI0, SGI77728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17280[] = {SGI77729, SGI0, SGI77730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17281[] = {SGI77731, SGI0, SGI77732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17282[] = {SGI77733, SGI77734, SGI77735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77736, SGI0, SGI0, SGI0, SGI77737, SGI77737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17283[] = {SGI77738, SGI77739, SGI77740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77741, SGI0, SGI0, SGI0, SGI77742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17284[] = {SGI77743, SGI77744, SGI77745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77746, SGI0, SGI0, SGI0, SGI77747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17285[] = {SGI77748, SGI77749, SGI77750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77751, SGI0, SGI0, SGI0, SGI77752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17286[] = {SGI77753, SGI0, SGI77754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17287[] = {SGI77755, SGI77756, SGI77757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77758, SGI0, SGI0, SGI0, SGI77759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17288[] = {SGI77760, SGI77761, SGI77762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77763, SGI0, SGI0, SGI0, SGI77764, SGI77764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17289[] = {SGI77765, SGI77766, SGI77767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77768, SGI0, SGI0, SGI0, SGI77769, SGI77769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17290[] = {SGI77770, SGI77771, SGI77772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77773, SGI0, SGI0, SGI0, SGI77774, SGI77774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17291[] = {SGI77775, SGI77776, SGI77777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77778, SGI0, SGI0, SGI0, SGI77779, SGI77779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17292[] = {SGI77780, SGI0, SGI77781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17293[] = {SGI77782, SGI0, SGI77783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17294[] = {SGI77784, SGI77785, SGI77786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77787, SGI0, SGI0, SGI0, SGI77788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17295[] = {SGI77789, SGI77790, SGI77791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77792, SGI0, SGI0, SGI0, SGI77793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17296[] = {SGI77794, SGI77795, SGI77796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77797, SGI0, SGI0, SGI0, SGI77798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17297[] = {SGI77799, SGI77800, SGI77801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17298[] = {SGI77803, SGI0, SGI77804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77805, SGI0, SGI0, SGI0, SGI77806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17299[] = {SGI77807, SGI77808, SGI77809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77810, SGI0, SGI0, SGI0, SGI77811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17300[] = {SGI77812, SGI77813, SGI77814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77815, SGI0, SGI0, SGI0, SGI77816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17301[] = {SGI77817, SGI0, SGI77818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17302[] = {SGI77819, SGI77820, SGI77821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77822, SGI0, SGI0, SGI0, SGI77823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17303[] = {SGI77824, SGI77825, SGI77826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77827, SGI0, SGI0, SGI0, SGI77828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17304[] = {SGI77829, SGI77830, SGI77831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77832, SGI0, SGI0, SGI0, SGI77833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17305[] = {SGI77834, SGI0, SGI77835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17306[] = {SGI77836, SGI77837, SGI77838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77839, SGI0, SGI0, SGI0, SGI77840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17307[] = {SGI77841, SGI77842, SGI77843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77844, SGI0, SGI0, SGI0, SGI77845, SGI77845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17308[] = {SGI77846, SGI77847, SGI77848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77849, SGI0, SGI0, SGI0, SGI77850, SGI77850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17309[] = {SGI77851, SGI77852, SGI77853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77854, SGI0, SGI0, SGI0, SGI77855, SGI77855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17310[] = {SGI77856, SGI77857, SGI77858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77859, SGI0, SGI0, SGI0, SGI77860, SGI77860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17311[] = {SGI77861, SGI0, SGI77862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17312[] = {SGI77863, SGI77864, SGI77865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77866, SGI0, SGI0, SGI0, SGI77867, SGI77867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17313[] = {SGI77868, SGI77869, SGI77870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77871, SGI0, SGI0, SGI0, SGI77872, SGI77872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17314[] = {SGI77873, SGI77874, SGI77875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77876, SGI0, SGI0, SGI0, SGI77877, SGI77877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17315[] = {SGI77878, SGI77879, SGI77880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77881, SGI0, SGI0, SGI0, SGI77882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17316[] = {SGI77883, SGI0, SGI77884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17317[] = {SGI77885, SGI77886, SGI77887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77888, SGI0, SGI0, SGI0, SGI77889, SGI77889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17318[] = {SGI77890, SGI77891, SGI77892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77893, SGI0, SGI0, SGI0, SGI77894, SGI77894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17319[] = {SGI77895, SGI77896, SGI77897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77898, SGI0, SGI0, SGI0, SGI77899, SGI77899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17320[] = {SGI77900, SGI77901, SGI77902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77903, SGI0, SGI0, SGI0, SGI77904, SGI77904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17321[] = {SGI77905, SGI77906, SGI77907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77908, SGI0, SGI0, SGI0, SGI77909, SGI77909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17322[] = {SGI77910, SGI77911, SGI77912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77913, SGI0, SGI0, SGI0, SGI77914, SGI77914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17323[] = {SGI77915, SGI77916, SGI77917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77918, SGI0, SGI0, SGI0, SGI77919, SGI77919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17324[] = {SGI77920, SGI77921, SGI77922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77923, SGI0, SGI0, SGI0, SGI77924, SGI77924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17325[] = {SGI77925, SGI0, SGI77926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17326[] = {SGI77927, SGI77928, SGI77929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77930, SGI0, SGI0, SGI0, SGI77931, SGI77931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17327[] = {SGI77932, SGI77933, SGI77934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77935, SGI0, SGI0, SGI0, SGI77936, SGI77936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17328[] = {SGI77937, SGI77938, SGI77939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77940, SGI0, SGI0, SGI0, SGI77941, SGI77941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17329[] = {SGI77942, SGI77943, SGI77944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77945, SGI0, SGI0, SGI0, SGI77946, SGI77946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17330[] = {SGI77947, SGI77948, SGI77949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77950, SGI0, SGI0, SGI0, SGI77951, SGI77951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17331[] = {SGI77952, SGI77953, SGI77954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77955, SGI0, SGI0, SGI0, SGI77956, SGI77956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17332[] = {SGI77957, SGI77958, SGI77959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77960, SGI0, SGI0, SGI0, SGI77961, SGI77961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17333[] = {SGI77962, SGI77963, SGI77964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77965, SGI0, SGI0, SGI0, SGI77966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17334[] = {SGI77967, SGI77968, SGI77969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77970, SGI0, SGI0, SGI0, SGI77971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17335[] = {SGI77972, SGI0, SGI77973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17336[] = {SGI77974, SGI0, SGI77975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17337[] = {SGI77976, SGI77977, SGI77978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77979, SGI0, SGI0, SGI0, SGI77980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17338[] = {SGI77981, SGI77982, SGI77983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77984, SGI0, SGI0, SGI0, SGI77985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17339[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77986, SGI0, SGI0, SGI77987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17340[] = {SGI77988, SGI77989, SGI77990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77991, SGI0, SGI0, SGI0, SGI77992, SGI77992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17341[] = {SGI77993, SGI77994, SGI77995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI77996, SGI0, SGI0, SGI0, SGI77997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17342[] = {SGI77998, SGI77999, SGI78000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78001, SGI0, SGI0, SGI0, SGI78002, SGI78002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17343[] = {SGI78003, SGI78004, SGI78005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78006, SGI0, SGI0, SGI0, SGI78007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17344[] = {SGI78008, SGI0, SGI78009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17345[] = {SGI78010, SGI78011, SGI78012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78014, SGI0, SGI0, SGI0, SGI78017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17346[] = {SGI78013, SGI78015, SGI78016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78018, SGI0, SGI0, SGI0, SGI78019, SGI78019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17347[] = {SGI78020, SGI78021, SGI78022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78026, SGI0, SGI0, SGI0, SGI78027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17348[] = {SGI78023, SGI78024, SGI78025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78028, SGI0, SGI0, SGI0, SGI78029, SGI78029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17349[] = {SGI78030, SGI78031, SGI78032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78036, SGI0, SGI0, SGI0, SGI78037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17350[] = {SGI78033, SGI78034, SGI78035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78038, SGI78038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17351[] = {SGI78039, SGI78040, SGI78042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78045, SGI0, SGI0, SGI0, SGI78047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17352[] = {SGI78041, SGI78043, SGI78044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78046, SGI0, SGI0, SGI0, SGI78048, SGI78048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17353[] = {SGI78049, SGI78050, SGI78051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78055, SGI0, SGI0, SGI0, SGI78056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17354[] = {SGI78052, SGI78053, SGI78054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78057, SGI0, SGI0, SGI0, SGI78058, SGI78058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17355[] = {SGI78059, SGI78060, SGI78061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78062, SGI0, SGI0, SGI0, SGI78063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17356[] = {SGI78064, SGI78065, SGI78066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78067, SGI0, SGI0, SGI0, SGI78068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17357[] = {SGI78069, SGI78070, SGI78071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78072, SGI0, SGI0, SGI0, SGI78073, SGI78073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17358[] = {SGI78074, SGI78075, SGI78076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78077, SGI0, SGI0, SGI0, SGI78078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17359[] = {SGI78079, SGI78080, SGI78081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78082, SGI0, SGI0, SGI0, SGI78083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17360[] = {SGI78084, SGI78085, SGI78086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78087, SGI0, SGI0, SGI0, SGI78088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17361[] = {SGI78089, SGI78090, SGI78091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78092, SGI0, SGI0, SGI0, SGI78093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17362[] = {SGI78094, SGI78095, SGI78096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78097, SGI0, SGI0, SGI0, SGI78098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17363[] = {SGI78099, SGI0, SGI78100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78101, SGI0, SGI0, SGI0, SGI78102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17364[] = {SGI78103, SGI78104, SGI78105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78106, SGI0, SGI0, SGI0, SGI78109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17365[] = {SGI78107, SGI78108, SGI78110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78111, SGI0, SGI0, SGI0, SGI78112, SGI78112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17366[] = {SGI78113, SGI78114, SGI78115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78116, SGI0, SGI0, SGI0, SGI78117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17367[] = {SGI78118, SGI78119, SGI78120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78121, SGI0, SGI0, SGI0, SGI78122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17368[] = {SGI78123, SGI0, SGI78124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17369[] = {SGI78125, SGI78126, SGI78127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78128, SGI0, SGI0, SGI0, SGI78129, SGI78129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17370[] = {SGI78130, SGI78131, SGI78132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78133, SGI0, SGI0, SGI0, SGI78134, SGI78134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17371[] = {SGI78135, SGI78136, SGI78137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78138, SGI0, SGI0, SGI0, SGI78139, SGI78139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17372[] = {SGI78140, SGI78141, SGI78142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78143, SGI0, SGI0, SGI0, SGI78144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17373[] = {SGI78145, SGI78146, SGI78147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78148, SGI0, SGI0, SGI0, SGI78149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17374[] = {SGI78150, SGI78151, SGI78152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78153, SGI0, SGI0, SGI0, SGI78154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17375[] = {SGI78155, SGI78156, SGI78157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78158, SGI0, SGI0, SGI0, SGI78159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17376[] = {SGI78160, SGI78161, SGI78162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78163, SGI0, SGI0, SGI0, SGI78164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17377[] = {SGI78165, SGI78166, SGI78167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78168, SGI0, SGI0, SGI0, SGI78169, SGI78169, SGI0, SGI78170, SGI0, NULL};
const Segment * const SG17378[] = {SGI78171, SGI78172, SGI78173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78174, SGI0, SGI0, SGI0, SGI78175, SGI78175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17379[] = {SGI78176, SGI78177, SGI78178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78179, SGI0, SGI0, SGI0, SGI78180, SGI78180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17380[] = {SGI78181, SGI0, SGI78182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17381[] = {SGI78183, SGI78184, SGI78185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78186, SGI0, SGI0, SGI0, SGI78187, SGI78187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17382[] = {SGI78188, SGI78189, SGI78190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78191, SGI0, SGI0, SGI0, SGI78192, SGI78192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17383[] = {SGI78193, SGI0, SGI78194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17384[] = {SGI78195, SGI78196, SGI78197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78198, SGI0, SGI0, SGI0, SGI78199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17385[] = {SGI78200, SGI0, SGI78201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17386[] = {SGI78202, SGI78203, SGI78204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78205, SGI0, SGI0, SGI0, SGI78206, SGI78206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17387[] = {SGI78207, SGI0, SGI78208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78209, SGI0, SGI0, SGI0, SGI78210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17388[] = {SGI78211, SGI78212, SGI78213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78214, SGI0, SGI0, SGI0, SGI78215, SGI78215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17389[] = {SGI78216, SGI0, SGI78217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17390[] = {SGI78218, SGI78219, SGI78220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78221, SGI0, SGI0, SGI0, SGI78222, SGI78222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17391[] = {SGI78223, SGI0, SGI78224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17392[] = {SGI78225, SGI78226, SGI78227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78228, SGI0, SGI0, SGI0, SGI78229, SGI78229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17393[] = {SGI78230, SGI78231, SGI78232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78233, SGI0, SGI0, SGI0, SGI78234, SGI78234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17394[] = {SGI78235, SGI78236, SGI78237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78238, SGI0, SGI0, SGI0, SGI78239, SGI78239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17395[] = {SGI78240, SGI78241, SGI78242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78243, SGI0, SGI0, SGI0, SGI78244, SGI78244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17396[] = {SGI78245, SGI78246, SGI78247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78248, SGI0, SGI0, SGI0, SGI78249, SGI78249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17397[] = {SGI78250, SGI78251, SGI78252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78253, SGI0, SGI0, SGI0, SGI78254, SGI78254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17398[] = {SGI78255, SGI0, SGI78256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17399[] = {SGI78257, SGI78258, SGI78259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78260, SGI0, SGI0, SGI0, SGI78261, SGI78261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17400[] = {SGI78262, SGI0, SGI78263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17401[] = {SGI78264, SGI78265, SGI78266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78267, SGI0, SGI0, SGI0, SGI78268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17402[] = {SGI78269, SGI78270, SGI78271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78272, SGI0, SGI0, SGI0, SGI78273, SGI78273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17403[] = {SGI78274, SGI0, SGI78275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17404[] = {SGI78276, SGI78277, SGI78278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78279, SGI0, SGI0, SGI0, SGI78280, SGI78280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17405[] = {SGI78281, SGI78282, SGI78283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78284, SGI0, SGI0, SGI0, SGI78285, SGI78285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17406[] = {SGI78286, SGI78287, SGI78288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78289, SGI0, SGI0, SGI0, SGI78290, SGI78290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17407[] = {SGI78291, SGI78292, SGI78293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78294, SGI0, SGI0, SGI0, SGI78295, SGI78295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17408[] = {SGI78296, SGI78297, SGI78298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78299, SGI0, SGI0, SGI0, SGI78300, SGI78300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17409[] = {SGI78301, SGI78302, SGI78303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78304, SGI0, SGI0, SGI0, SGI78305, SGI78305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17410[] = {SGI78306, SGI78307, SGI78308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78309, SGI0, SGI0, SGI0, SGI78310, SGI78310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17411[] = {SGI78311, SGI0, SGI78312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17412[] = {SGI78313, SGI78314, SGI78315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78316, SGI0, SGI0, SGI0, SGI78317, SGI78317, SGI0, SGI78318, SGI0, NULL};
const Segment * const SG17413[] = {SGI78319, SGI78320, SGI78321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78322, SGI0, SGI0, SGI0, SGI78323, SGI78323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17414[] = {SGI78324, SGI78325, SGI78326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78327, SGI0, SGI0, SGI0, SGI78328, SGI78328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17415[] = {SGI78329, SGI0, SGI78330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17416[] = {SGI78331, SGI78332, SGI78333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78334, SGI0, SGI0, SGI0, SGI78335, SGI78335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17417[] = {SGI78336, SGI78337, SGI78338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78339, SGI0, SGI0, SGI0, SGI78340, SGI78340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17418[] = {SGI78341, SGI78342, SGI78343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78344, SGI0, SGI0, SGI0, SGI78345, SGI78345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17419[] = {SGI78346, SGI0, SGI78347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17420[] = {SGI78348, SGI0, SGI78349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17421[] = {SGI78350, SGI78351, SGI78352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78353, SGI0, SGI0, SGI0, SGI78354, SGI78354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17422[] = {SGI78355, SGI78356, SGI78357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78358, SGI0, SGI0, SGI0, SGI78359, SGI78359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17423[] = {SGI78360, SGI78361, SGI78362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78363, SGI0, SGI0, SGI0, SGI78364, SGI78364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17424[] = {SGI78365, SGI78366, SGI78367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78368, SGI0, SGI0, SGI0, SGI78369, SGI78369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17425[] = {SGI78370, SGI78371, SGI78372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78373, SGI0, SGI0, SGI0, SGI78374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17426[] = {SGI78375, SGI78376, SGI78377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78378, SGI0, SGI0, SGI0, SGI78379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17427[] = {SGI78380, SGI78381, SGI78382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78383, SGI0, SGI0, SGI0, SGI78384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17428[] = {SGI78385, SGI0, SGI78386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17429[] = {SGI78387, SGI78388, SGI78389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78390, SGI0, SGI0, SGI0, SGI78391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17430[] = {SGI78392, SGI78393, SGI78394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78395, SGI0, SGI0, SGI0, SGI78396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17431[] = {SGI78397, SGI78398, SGI78399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78400, SGI0, SGI0, SGI0, SGI78401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17432[] = {SGI78402, SGI78403, SGI78404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78405, SGI0, SGI0, SGI0, SGI78406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17433[] = {SGI78407, SGI78408, SGI78409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78410, SGI0, SGI0, SGI0, SGI78411, SGI78411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17434[] = {SGI78412, SGI0, SGI78413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17435[] = {SGI78414, SGI0, SGI78415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17436[] = {SGI78416, SGI78417, SGI78418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78419, SGI78419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17437[] = {SGI78420, SGI78421, SGI78422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78423, SGI0, SGI0, SGI0, SGI78424, SGI78424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17438[] = {SGI78425, SGI78426, SGI78427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78428, SGI0, SGI0, SGI0, SGI78429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17439[] = {SGI78430, SGI78431, SGI78432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78433, SGI0, SGI0, SGI0, SGI78434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17440[] = {SGI78435, SGI0, SGI78436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17441[] = {SGI78437, SGI0, SGI78438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17442[] = {SGI78439, SGI78440, SGI78441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78442, SGI0, SGI0, SGI0, SGI78443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17443[] = {SGI78444, SGI78445, SGI78446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78447, SGI0, SGI0, SGI0, SGI78448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17444[] = {SGI78449, SGI78450, SGI78451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78452, SGI0, SGI0, SGI0, SGI78453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17445[] = {SGI78454, SGI78455, SGI78456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78457, SGI0, SGI0, SGI0, SGI78458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17446[] = {SGI78459, SGI78460, SGI78461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78462, SGI0, SGI0, SGI0, SGI78463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17447[] = {SGI78464, SGI0, SGI78465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78466, SGI0, SGI0, SGI0, SGI78467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17448[] = {SGI78468, SGI78469, SGI78470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78471, SGI0, SGI0, SGI0, SGI78472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17449[] = {SGI78473, SGI78474, SGI78475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78476, SGI0, SGI0, SGI0, SGI78477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17450[] = {SGI78478, SGI78479, SGI78480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78481, SGI0, SGI0, SGI0, SGI78482, SGI78482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17451[] = {SGI78483, SGI78484, SGI78485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78486, SGI0, SGI0, SGI0, SGI78487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17452[] = {SGI78488, SGI78489, SGI78490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78491, SGI0, SGI0, SGI0, SGI78492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17453[] = {SGI78493, SGI78494, SGI78495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78496, SGI0, SGI0, SGI0, SGI78497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17454[] = {SGI78498, SGI78499, SGI78500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78501, SGI0, SGI0, SGI0, SGI78502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17455[] = {SGI78503, SGI78504, SGI78505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78506, SGI0, SGI0, SGI0, SGI78507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17456[] = {SGI78508, SGI78509, SGI78510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78511, SGI0, SGI0, SGI0, SGI78512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17457[] = {SGI78513, SGI78514, SGI78515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78516, SGI0, SGI0, SGI0, SGI78517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17458[] = {SGI78518, SGI78519, SGI78520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78521, SGI0, SGI0, SGI0, SGI78522, SGI78522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17459[] = {SGI78523, SGI78524, SGI78525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78526, SGI0, SGI0, SGI0, SGI78527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17460[] = {SGI78528, SGI0, SGI78529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17461[] = {SGI78530, SGI78531, SGI78532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78533, SGI0, SGI0, SGI0, SGI78534, SGI78534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17462[] = {SGI78535, SGI78536, SGI78537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78538, SGI0, SGI0, SGI0, SGI78539, SGI78539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17463[] = {SGI78540, SGI78541, SGI78542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78543, SGI0, SGI0, SGI0, SGI78544, SGI78544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17464[] = {SGI78545, SGI78546, SGI78547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78548, SGI0, SGI0, SGI0, SGI78549, SGI78549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17465[] = {SGI78550, SGI78551, SGI78552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78553, SGI0, SGI0, SGI0, SGI78554, SGI78554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17466[] = {SGI78555, SGI0, SGI78556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17467[] = {SGI78557, SGI78558, SGI78559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78560, SGI78560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17468[] = {SGI78561, SGI0, SGI78562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17469[] = {SGI78563, SGI0, SGI78564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17470[] = {SGI78565, SGI78566, SGI78567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78568, SGI0, SGI0, SGI0, SGI78569, SGI78569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17471[] = {SGI78570, SGI78571, SGI78572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78573, SGI0, SGI0, SGI0, SGI78574, SGI78574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17472[] = {SGI78575, SGI78576, SGI78577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78578, SGI0, SGI0, SGI0, SGI78579, SGI78579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17473[] = {SGI78580, SGI0, SGI78581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17474[] = {SGI78582, SGI78583, SGI78584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78585, SGI78586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17475[] = {SGI78587, SGI78588, SGI78589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78590, SGI0, SGI0, SGI0, SGI78591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17476[] = {SGI78592, SGI78593, SGI78594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78595, SGI0, SGI0, SGI0, SGI78596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17477[] = {SGI78597, SGI0, SGI78598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17478[] = {SGI78599, SGI78600, SGI78601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78602, SGI0, SGI0, SGI0, SGI78603, SGI78603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17479[] = {SGI78604, SGI78605, SGI78606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78607, SGI0, SGI0, SGI0, SGI78608, SGI78608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17480[] = {SGI78609, SGI78610, SGI78611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78612, SGI0, SGI0, SGI0, SGI78613, SGI78613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17481[] = {SGI78614, SGI78615, SGI78616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78617, SGI0, SGI0, SGI0, SGI78618, SGI78618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17482[] = {SGI78619, SGI0, SGI78620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17483[] = {SGI78621, SGI78622, SGI78623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78624, SGI0, SGI0, SGI0, SGI78625, SGI78625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17484[] = {SGI78626, SGI78627, SGI78628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78629, SGI0, SGI0, SGI0, SGI78630, SGI78630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17485[] = {SGI78631, SGI78632, SGI78633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78634, SGI0, SGI0, SGI0, SGI78635, SGI78635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17486[] = {SGI78636, SGI78637, SGI78638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78639, SGI0, SGI0, SGI0, SGI78640, SGI78640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17487[] = {SGI78641, SGI78642, SGI78643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78644, SGI0, SGI0, SGI0, SGI78645, SGI78645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17488[] = {SGI78646, SGI78647, SGI78648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78649, SGI78649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17489[] = {SGI78650, SGI0, SGI78651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17490[] = {SGI78652, SGI0, SGI78653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78654, SGI0, SGI0, SGI0, SGI78655, SGI78655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17491[] = {SGI78656, SGI78657, SGI78658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78659, SGI0, SGI0, SGI0, SGI78660, SGI78660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17492[] = {SGI78661, SGI0, SGI78662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78663, SGI0, SGI0, SGI0, SGI78664, SGI78664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17493[] = {SGI78665, SGI78666, SGI78667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78668, SGI0, SGI0, SGI0, SGI78669, SGI78669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17494[] = {SGI78670, SGI78671, SGI78672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78673, SGI0, SGI0, SGI0, SGI78674, SGI78674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17495[] = {SGI78675, SGI78676, SGI78677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78678, SGI0, SGI0, SGI0, SGI78679, SGI78679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17496[] = {SGI78680, SGI0, SGI78681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17497[] = {SGI78682, SGI78683, SGI78684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78685, SGI0, SGI0, SGI0, SGI78686, SGI78686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17498[] = {SGI78687, SGI78688, SGI78689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78690, SGI0, SGI0, SGI0, SGI78691, SGI78691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17499[] = {SGI78692, SGI78693, SGI78694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78695, SGI0, SGI0, SGI0, SGI78696, SGI78696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17500[] = {SGI78697, SGI78698, SGI78699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78700, SGI0, SGI0, SGI0, SGI78701, SGI78701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17501[] = {SGI78702, SGI0, SGI78703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78704, SGI0, SGI0, SGI0, SGI78705, SGI78705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17502[] = {SGI78706, SGI78707, SGI78708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78709, SGI0, SGI0, SGI0, SGI78710, SGI78710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17503[] = {SGI78711, SGI78712, SGI78713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78714, SGI0, SGI0, SGI0, SGI78715, SGI78715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17504[] = {SGI78716, SGI78717, SGI78718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78719, SGI0, SGI0, SGI0, SGI78720, SGI78720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17505[] = {SGI78721, SGI78722, SGI78723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78724, SGI0, SGI0, SGI0, SGI78725, SGI78725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17506[] = {SGI78726, SGI78727, SGI78728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78729, SGI0, SGI0, SGI0, SGI78730, SGI78730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17507[] = {SGI78731, SGI78732, SGI78733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78734, SGI0, SGI0, SGI0, SGI78735, SGI78735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17508[] = {SGI78736, SGI0, SGI78737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17509[] = {SGI78738, SGI78739, SGI78740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78741, SGI0, SGI0, SGI0, SGI78742, SGI78742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17510[] = {SGI78743, SGI0, SGI78744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17511[] = {SGI78745, SGI78746, SGI78747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78748, SGI78748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17512[] = {SGI78749, SGI78750, SGI78751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17513[] = {SGI78752, SGI78753, SGI78754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17514[] = {SGI78755, SGI78756, SGI78757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17515[] = {SGI78758, SGI78759, SGI78760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78761, SGI78761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17516[] = {SGI78762, SGI0, SGI78763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78764, SGI0, SGI0, SGI0, SGI78765, SGI78765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17517[] = {SGI78766, SGI78767, SGI78768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78769, SGI0, SGI0, SGI0, SGI78770, SGI78771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17518[] = {SGI78772, SGI78773, SGI78774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78775, SGI0, SGI0, SGI0, SGI78776, SGI78776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17519[] = {SGI78777, SGI78778, SGI78779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78780, SGI0, SGI0, SGI0, SGI78781, SGI78781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17520[] = {SGI78782, SGI0, SGI78783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17521[] = {SGI78784, SGI78785, SGI78786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78787, SGI0, SGI0, SGI0, SGI78788, SGI78788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17522[] = {SGI78789, SGI78790, SGI78791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78792, SGI0, SGI0, SGI0, SGI78793, SGI78793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17523[] = {SGI78794, SGI78795, SGI78796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78797, SGI0, SGI0, SGI0, SGI78798, SGI78798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17524[] = {SGI78799, SGI78800, SGI78801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78802, SGI78803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17525[] = {SGI78804, SGI0, SGI78805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17526[] = {SGI78806, SGI78807, SGI78808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78809, SGI0, SGI0, SGI0, SGI78810, SGI78810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17527[] = {SGI78811, SGI78812, SGI78813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78814, SGI0, SGI0, SGI0, SGI78815, SGI78815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17528[] = {SGI78816, SGI78817, SGI78818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78819, SGI0, SGI0, SGI0, SGI78820, SGI78820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17529[] = {SGI78821, SGI78822, SGI78823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78824, SGI0, SGI0, SGI0, SGI78825, SGI78825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17530[] = {SGI78826, SGI78827, SGI78828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78829, SGI0, SGI0, SGI0, SGI78830, SGI78830, SGI0, SGI78831, SGI0, NULL};
const Segment * const SG17531[] = {SGI78832, SGI78833, SGI78834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78835, SGI0, SGI0, SGI0, SGI78836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17532[] = {SGI78837, SGI78838, SGI78839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78840, SGI0, SGI0, SGI0, SGI78841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17533[] = {SGI78842, SGI78843, SGI78844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17534[] = {SGI78845, SGI78846, SGI78847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78848, SGI0, SGI0, SGI0, SGI78849, SGI78849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17535[] = {SGI78850, SGI0, SGI78852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78855, SGI0, SGI0, SGI0, SGI78857, SGI78857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17536[] = {SGI78851, SGI78853, SGI78854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78856, SGI0, SGI0, SGI0, SGI78858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17537[] = {SGI78859, SGI0, SGI78860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17538[] = {SGI78861, SGI78862, SGI78863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78864, SGI0, SGI0, SGI0, SGI78865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17539[] = {SGI78866, SGI78867, SGI78868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78871, SGI0, SGI0, SGI0, SGI78873, SGI78873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17540[] = {SGI78869, SGI78870, SGI78872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78874, SGI0, SGI0, SGI0, SGI78875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17541[] = {SGI78876, SGI78877, SGI78878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78879, SGI0, SGI0, SGI0, SGI78880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17542[] = {SGI78881, SGI78882, SGI78883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17543[] = {SGI78885, SGI78886, SGI78887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78888, SGI0, SGI0, SGI0, SGI78889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17544[] = {SGI78890, SGI78891, SGI78892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78893, SGI0, SGI0, SGI0, SGI78894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17545[] = {SGI78895, SGI0, SGI78896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17546[] = {SGI78897, SGI78898, SGI78899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78900, SGI0, SGI0, SGI0, SGI78901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17547[] = {SGI78902, SGI78903, SGI78904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78905, SGI0, SGI0, SGI0, SGI78906, SGI78906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17548[] = {SGI78907, SGI78908, SGI78909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78910, SGI0, SGI0, SGI0, SGI78911, SGI78911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17549[] = {SGI78912, SGI78913, SGI78914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78915, SGI0, SGI0, SGI0, SGI78916, SGI78916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17550[] = {SGI78917, SGI78918, SGI78919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78920, SGI0, SGI0, SGI0, SGI78921, SGI78921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17551[] = {SGI78922, SGI78923, SGI78924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78925, SGI0, SGI0, SGI0, SGI78926, SGI78926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17552[] = {SGI78927, SGI78928, SGI78929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78930, SGI0, SGI0, SGI0, SGI78931, SGI78931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17553[] = {SGI78932, SGI78933, SGI78934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78935, SGI0, SGI0, SGI0, SGI78936, SGI78936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17554[] = {SGI78937, SGI78938, SGI78939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78940, SGI0, SGI0, SGI0, SGI78941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17555[] = {SGI78942, SGI78943, SGI78944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78945, SGI0, SGI0, SGI0, SGI78946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17556[] = {SGI78947, SGI78948, SGI78949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78950, SGI0, SGI0, SGI0, SGI78951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17557[] = {SGI78952, SGI0, SGI78953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17558[] = {SGI78954, SGI0, SGI78955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17559[] = {SGI78956, SGI78957, SGI78958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78959, SGI78959, SGI0, SGI0, NULL};
const Segment * const SG17560[] = {SGI78960, SGI0, SGI78961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78962, SGI78963, SGI78963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17561[] = {SGI78964, SGI0, SGI78965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78966, SGI78967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17562[] = {SGI78968, SGI0, SGI78969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78970, SGI78971, SGI78971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17563[] = {SGI78972, SGI0, SGI78973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78974, SGI78975, SGI78975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17564[] = {SGI78976, SGI0, SGI78977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78978, SGI78979, SGI78979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17565[] = {SGI78980, SGI0, SGI78981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78982, SGI78983, SGI78983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17566[] = {SGI78984, SGI0, SGI78985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78986, SGI78987, SGI78987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17567[] = {SGI78988, SGI0, SGI78989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78990, SGI78991, SGI78991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17568[] = {SGI78992, SGI0, SGI78993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78994, SGI78995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17569[] = {SGI78996, SGI0, SGI78997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI78998, SGI78999, SGI78999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17570[] = {SGI79000, SGI79001, SGI79002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79003, SGI0, SGI0, SGI0, SGI79004, SGI79004, SGI79005, SGI0, SGI79006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17571[] = {SGI79007, SGI79008, SGI79009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79010, SGI0, SGI0, SGI0, SGI79011, SGI79011, SGI0, SGI79012, SGI79013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17572[] = {SGI79014, SGI79015, SGI79016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79017, SGI0, SGI0, SGI0, SGI79018, SGI79018, SGI0, SGI0, SGI79019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17573[] = {SGI79020, SGI79021, SGI79022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79023, SGI0, SGI0, SGI0, SGI79024, SGI79024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79025, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17574[] = {SGI79026, SGI79027, SGI79028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79029, SGI0, SGI0, SGI0, SGI79030, SGI79030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17575[] = {SGI79031, SGI79032, SGI79033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79034, SGI0, SGI0, SGI0, SGI79035, SGI79035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17576[] = {SGI79036, SGI79037, SGI79038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79039, SGI0, SGI0, SGI0, SGI79040, SGI79040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17577[] = {SGI79041, SGI79042, SGI79043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79044, SGI0, SGI0, SGI0, SGI79045, SGI79045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17578[] = {SGI79046, SGI79047, SGI79048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79049, SGI0, SGI0, SGI0, SGI79050, SGI79050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17579[] = {SGI79051, SGI0, SGI79052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17580[] = {SGI79053, SGI79054, SGI79055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79056, SGI0, SGI0, SGI0, SGI79057, SGI79057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17581[] = {SGI79058, SGI79059, SGI79060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79061, SGI0, SGI0, SGI0, SGI79062, SGI79062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17582[] = {SGI79063, SGI79064, SGI79065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79066, SGI0, SGI0, SGI0, SGI79067, SGI79067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17583[] = {SGI79068, SGI0, SGI79069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79070, SGI0, SGI0, SGI0, SGI79071, SGI79071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17584[] = {SGI79072, SGI0, SGI79073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17585[] = {SGI79074, SGI79075, SGI79076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79077, SGI0, SGI0, SGI0, SGI79078, SGI79078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17586[] = {SGI79079, SGI79080, SGI79081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79082, SGI0, SGI0, SGI0, SGI79083, SGI79083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17587[] = {SGI79084, SGI79085, SGI79086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79087, SGI0, SGI0, SGI0, SGI79088, SGI79088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17588[] = {SGI79089, SGI0, SGI79090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17589[] = {SGI79091, SGI0, SGI79092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17590[] = {SGI79093, SGI79094, SGI79095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79096, SGI0, SGI0, SGI0, SGI79097, SGI79097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17591[] = {SGI79098, SGI79099, SGI79100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79101, SGI0, SGI0, SGI0, SGI79102, SGI79102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17592[] = {SGI79103, SGI0, SGI79104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17593[] = {SGI79105, SGI79106, SGI79107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79108, SGI0, SGI0, SGI0, SGI79109, SGI79109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17594[] = {SGI79110, SGI79111, SGI79112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79113, SGI0, SGI0, SGI0, SGI79114, SGI79114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17595[] = {SGI79115, SGI79116, SGI79117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79118, SGI0, SGI0, SGI0, SGI79119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17596[] = {SGI79120, SGI79121, SGI79122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79123, SGI0, SGI0, SGI0, SGI79124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17597[] = {SGI79125, SGI79126, SGI79127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79128, SGI0, SGI0, SGI0, SGI79129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17598[] = {SGI79130, SGI0, SGI79131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17599[] = {SGI79132, SGI79133, SGI79134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79135, SGI0, SGI0, SGI0, SGI79136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17600[] = {SGI79137, SGI0, SGI79138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79139, SGI0, SGI0, SGI0, SGI79140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17601[] = {SGI79141, SGI79142, SGI79143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79144, SGI0, SGI0, SGI0, SGI79145, SGI79145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17602[] = {SGI79146, SGI79147, SGI79148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17603[] = {SGI79149, SGI79150, SGI79151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17604[] = {SGI79152, SGI79153, SGI79154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79155, SGI0, SGI0, SGI0, SGI79156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17605[] = {SGI79157, SGI79158, SGI79159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79160, SGI0, SGI0, SGI0, SGI79161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17606[] = {SGI79162, SGI79163, SGI79164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79168, SGI0, SGI0, SGI0, SGI79169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17607[] = {SGI79165, SGI79166, SGI79167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79170, SGI0, SGI0, SGI0, SGI79171, SGI79171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17608[] = {SGI79172, SGI79173, SGI79174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79175, SGI0, SGI0, SGI0, SGI79178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17609[] = {SGI79176, SGI79177, SGI79179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79180, SGI0, SGI0, SGI0, SGI79181, SGI79181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17610[] = {SGI79182, SGI0, SGI79183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17611[] = {SGI79184, SGI79185, SGI79186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79188, SGI0, SGI0, SGI0, SGI79191, SGI79191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17612[] = {SGI79187, SGI79189, SGI79190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79192, SGI0, SGI0, SGI0, SGI79193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17613[] = {SGI79194, SGI0, SGI79195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17614[] = {SGI79196, SGI79197, SGI79198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79199, SGI0, SGI0, SGI0, SGI79200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17615[] = {SGI79201, SGI0, SGI79202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17616[] = {SGI79203, SGI0, SGI79204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17617[] = {SGI79205, SGI79206, SGI79207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79208, SGI0, SGI0, SGI0, SGI79209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17618[] = {SGI79210, SGI79211, SGI79212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79213, SGI0, SGI0, SGI0, SGI79214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17619[] = {SGI79215, SGI79216, SGI79217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79218, SGI0, SGI0, SGI0, SGI79219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17620[] = {SGI79220, SGI79221, SGI79222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79223, SGI0, SGI0, SGI0, SGI79224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17621[] = {SGI79225, SGI79226, SGI79227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79228, SGI0, SGI0, SGI0, SGI79229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17622[] = {SGI79230, SGI79231, SGI79232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79233, SGI0, SGI0, SGI0, SGI79234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17623[] = {SGI79235, SGI79236, SGI79237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79238, SGI0, SGI0, SGI0, SGI79239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17624[] = {SGI79240, SGI79241, SGI79242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79243, SGI0, SGI0, SGI0, SGI79244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17625[] = {SGI79245, SGI79246, SGI79247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79248, SGI0, SGI0, SGI0, SGI79249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17626[] = {SGI79250, SGI79251, SGI79252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79253, SGI0, SGI0, SGI0, SGI79254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17627[] = {SGI79255, SGI79256, SGI79257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79258, SGI79258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17628[] = {SGI79259, SGI79260, SGI79261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79262, SGI0, SGI0, SGI0, SGI79263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17629[] = {SGI79264, SGI79265, SGI79266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79267, SGI0, SGI0, SGI0, SGI79268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17630[] = {SGI79269, SGI79270, SGI79271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79272, SGI0, SGI0, SGI0, SGI79273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17631[] = {SGI79274, SGI79275, SGI79276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79277, SGI0, SGI0, SGI0, SGI79279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17632[] = {SGI79278, SGI79280, SGI79281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79282, SGI0, SGI0, SGI0, SGI79283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17633[] = {SGI79284, SGI79285, SGI79286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79287, SGI0, SGI0, SGI0, SGI79288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17634[] = {SGI79289, SGI0, SGI79290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17635[] = {SGI79291, SGI79292, SGI79293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79294, SGI0, SGI0, SGI0, SGI79295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17636[] = {SGI79297, SGI0, SGI79300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17637[] = {SGI79296, SGI79298, SGI79299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79301, SGI0, SGI0, SGI0, SGI79302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17638[] = {SGI79303, SGI0, SGI79304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17639[] = {SGI79305, SGI79306, SGI79307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79308, SGI0, SGI0, SGI0, SGI79309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17640[] = {SGI79310, SGI79311, SGI79312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79313, SGI0, SGI0, SGI0, SGI79314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17641[] = {SGI79315, SGI79316, SGI79317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79318, SGI0, SGI0, SGI0, SGI79319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17642[] = {SGI79320, SGI79321, SGI79322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79323, SGI0, SGI0, SGI0, SGI79324, SGI79324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17643[] = {SGI79325, SGI79326, SGI79327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79328, SGI0, SGI0, SGI0, SGI79329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17644[] = {SGI79330, SGI79331, SGI79332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79333, SGI0, SGI0, SGI0, SGI79334, SGI79334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17645[] = {SGI79335, SGI79336, SGI79337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79338, SGI0, SGI0, SGI0, SGI79339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17646[] = {SGI79340, SGI79341, SGI79342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79343, SGI0, SGI0, SGI0, SGI79344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17647[] = {SGI79345, SGI79346, SGI79347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79348, SGI0, SGI0, SGI0, SGI79349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17648[] = {SGI79350, SGI79351, SGI79352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79353, SGI0, SGI0, SGI0, SGI79354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17649[] = {SGI79355, SGI79356, SGI79357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79358, SGI0, SGI0, SGI0, SGI79359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17650[] = {SGI79360, SGI0, SGI79361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79362, SGI0, SGI0, SGI0, SGI79363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17651[] = {SGI79364, SGI79365, SGI79366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79367, SGI0, SGI0, SGI0, SGI79368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17652[] = {SGI79369, SGI79370, SGI79371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79372, SGI0, SGI0, SGI0, SGI79373, SGI79373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17653[] = {SGI79374, SGI0, SGI79375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17654[] = {SGI79376, SGI79377, SGI79378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79379, SGI0, SGI0, SGI0, SGI79380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17655[] = {SGI79381, SGI79382, SGI79383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79384, SGI0, SGI0, SGI0, SGI79385, SGI79385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17656[] = {SGI79386, SGI0, SGI79387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17657[] = {SGI79388, SGI79389, SGI79390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79391, SGI0, SGI0, SGI0, SGI79392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17658[] = {SGI79393, SGI79394, SGI79395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79396, SGI0, SGI0, SGI0, SGI79397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17659[] = {SGI79398, SGI79399, SGI79400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79401, SGI0, SGI0, SGI0, SGI79402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17660[] = {SGI79403, SGI79404, SGI79405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79406, SGI0, SGI0, SGI0, SGI79407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17661[] = {SGI79408, SGI79409, SGI79410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79411, SGI0, SGI0, SGI0, SGI79412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17662[] = {SGI79413, SGI79414, SGI79415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79416, SGI0, SGI0, SGI0, SGI79417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17663[] = {SGI79418, SGI79419, SGI79420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79421, SGI0, SGI0, SGI0, SGI79422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17664[] = {SGI79423, SGI79424, SGI79425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79426, SGI0, SGI0, SGI0, SGI79427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17665[] = {SGI79428, SGI79429, SGI79430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79431, SGI0, SGI0, SGI0, SGI79432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17666[] = {SGI79433, SGI0, SGI79434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17667[] = {SGI79435, SGI79436, SGI79437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79438, SGI0, SGI0, SGI0, SGI79439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17668[] = {SGI79440, SGI79441, SGI79442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79443, SGI0, SGI0, SGI0, SGI79444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17669[] = {SGI79445, SGI79446, SGI79447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79448, SGI0, SGI0, SGI0, SGI79449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17670[] = {SGI79450, SGI79451, SGI79452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79453, SGI0, SGI0, SGI0, SGI79454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17671[] = {SGI79455, SGI0, SGI79456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17672[] = {SGI79457, SGI79458, SGI79459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79460, SGI0, SGI0, SGI0, SGI79461, SGI79461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17673[] = {SGI79462, SGI79463, SGI79464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79465, SGI0, SGI0, SGI0, SGI79466, SGI79466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17674[] = {SGI79467, SGI79468, SGI79469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79470, SGI0, SGI0, SGI0, SGI79471, SGI79471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17675[] = {SGI79472, SGI0, SGI79473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17676[] = {SGI79474, SGI79475, SGI79476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79477, SGI0, SGI0, SGI0, SGI79478, SGI79478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17677[] = {SGI79479, SGI79480, SGI79481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79482, SGI0, SGI0, SGI0, SGI79483, SGI79483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17678[] = {SGI79484, SGI79485, SGI79486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79487, SGI0, SGI0, SGI0, SGI79488, SGI79488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17679[] = {SGI79489, SGI0, SGI79490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79491, SGI0, SGI0, SGI0, SGI79492, SGI79492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17680[] = {SGI79493, SGI0, SGI79494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79495, SGI0, SGI0, SGI0, SGI79496, SGI79496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17681[] = {SGI79497, SGI79498, SGI79499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79500, SGI0, SGI0, SGI0, SGI79501, SGI79501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17682[] = {SGI79502, SGI79503, SGI79504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79505, SGI0, SGI0, SGI0, SGI79506, SGI79506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17683[] = {SGI79507, SGI79508, SGI79509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79510, SGI0, SGI0, SGI0, SGI79511, SGI79511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17684[] = {SGI79512, SGI79513, SGI79514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79515, SGI0, SGI0, SGI0, SGI79516, SGI79516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17685[] = {SGI79517, SGI0, SGI79518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17686[] = {SGI79519, SGI0, SGI79520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79521, SGI79522, SGI79522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79523, NULL};
const Segment * const SG17687[] = {SGI79524, SGI0, SGI79525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79526, SGI79527, SGI79527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79528, NULL};
const Segment * const SG17688[] = {SGI79529, SGI0, SGI79530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79531, SGI79532, SGI79532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79533, NULL};
const Segment * const SG17689[] = {SGI79534, SGI79535, SGI79536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79537, SGI0, SGI0, SGI0, SGI79538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17690[] = {SGI79539, SGI79540, SGI79541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79542, SGI0, SGI0, SGI0, SGI79543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17691[] = {SGI79544, SGI0, SGI79545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17692[] = {SGI79546, SGI0, SGI79547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17693[] = {SGI79548, SGI79549, SGI79550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79551, SGI0, SGI0, SGI0, SGI79552, SGI79552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17694[] = {SGI79553, SGI79554, SGI79555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79556, SGI0, SGI0, SGI0, SGI79557, SGI79557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17695[] = {SGI79558, SGI0, SGI79559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17696[] = {SGI79560, SGI79561, SGI79562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79563, SGI0, SGI0, SGI0, SGI79564, SGI79564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17697[] = {SGI79565, SGI79566, SGI79567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79568, SGI0, SGI0, SGI0, SGI79569, SGI79569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17698[] = {SGI79570, SGI0, SGI79571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17699[] = {SGI79572, SGI79573, SGI79574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79575, SGI0, SGI0, SGI0, SGI79576, SGI79576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17700[] = {SGI79577, SGI79578, SGI79579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79580, SGI0, SGI0, SGI0, SGI79581, SGI79581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17701[] = {SGI79582, SGI0, SGI79583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79584, SGI0, SGI0, SGI0, SGI79585, SGI79585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17702[] = {SGI79586, SGI79587, SGI79588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79589, SGI0, SGI0, SGI0, SGI79590, SGI79590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17703[] = {SGI79591, SGI0, SGI79592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17704[] = {SGI79593, SGI79594, SGI79595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79596, SGI0, SGI0, SGI0, SGI79597, SGI79597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17705[] = {SGI79598, SGI79599, SGI79600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79601, SGI0, SGI0, SGI0, SGI79602, SGI79602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17706[] = {SGI79603, SGI79604, SGI79605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79606, SGI0, SGI0, SGI0, SGI79607, SGI79607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17707[] = {SGI79608, SGI0, SGI79609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17708[] = {SGI79610, SGI79611, SGI79612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79613, SGI0, SGI0, SGI0, SGI79614, SGI79614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17709[] = {SGI79615, SGI79616, SGI79617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79618, SGI0, SGI0, SGI0, SGI79619, SGI79619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17710[] = {SGI79620, SGI79621, SGI79622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79623, SGI0, SGI0, SGI0, SGI79624, SGI79624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17711[] = {SGI79625, SGI79626, SGI79627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79628, SGI0, SGI0, SGI0, SGI79629, SGI79629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17712[] = {SGI79630, SGI0, SGI79631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17713[] = {SGI79632, SGI79633, SGI79634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79635, SGI0, SGI0, SGI0, SGI79636, SGI79636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17714[] = {SGI79637, SGI79638, SGI79639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79640, SGI79640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17715[] = {SGI79641, SGI79642, SGI79643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79644, SGI0, SGI0, SGI0, SGI79645, SGI79645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17716[] = {SGI79646, SGI79647, SGI79648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79649, SGI0, SGI0, SGI0, SGI79650, SGI79650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17717[] = {SGI79651, SGI79652, SGI79653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79654, SGI0, SGI0, SGI0, SGI79655, SGI79655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17718[] = {SGI79656, SGI0, SGI79657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79658, SGI0, SGI0, SGI0, SGI79659, SGI79659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17719[] = {SGI79660, SGI0, SGI79661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79662, SGI0, SGI0, SGI0, SGI79663, SGI79663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17720[] = {SGI79664, SGI79665, SGI79666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79667, SGI0, SGI0, SGI0, SGI79668, SGI79668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17721[] = {SGI79669, SGI79670, SGI79671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79672, SGI0, SGI0, SGI0, SGI79673, SGI79673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17722[] = {SGI79674, SGI0, SGI79675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17723[] = {SGI79676, SGI79677, SGI79678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79679, SGI0, SGI0, SGI0, SGI79680, SGI79680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17724[] = {SGI79681, SGI0, SGI79682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17725[] = {SGI79683, SGI0, SGI79684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17726[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79685, SGI0, SGI0, SGI79686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17727[] = {SGI79687, SGI79688, SGI79689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79690, SGI0, SGI0, SGI0, SGI79691, SGI79691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17728[] = {SGI79692, SGI79693, SGI79694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79695, SGI0, SGI0, SGI0, SGI79696, SGI79696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17729[] = {SGI79697, SGI79698, SGI79699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79700, SGI0, SGI0, SGI0, SGI79701, SGI79701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17730[] = {SGI79702, SGI0, SGI79703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17731[] = {SGI79704, SGI0, SGI79705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79706, SGI79707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17732[] = {SGI79708, SGI0, SGI79709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79710, SGI79711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17733[] = {SGI79712, SGI0, SGI79713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79714, SGI79715, SGI79715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17734[] = {SGI79716, SGI0, SGI79717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79718, SGI79719, SGI79719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17735[] = {SGI79720, SGI0, SGI79721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79722, SGI79723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17736[] = {SGI79724, SGI0, SGI79725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79726, SGI79727, SGI79727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17737[] = {SGI79728, SGI0, SGI79729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79730, SGI79731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17738[] = {SGI79732, SGI0, SGI79733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79734, SGI79735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17739[] = {SGI79736, SGI0, SGI79737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79738, SGI79739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17740[] = {SGI79740, SGI0, SGI79741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79742, SGI79743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17741[] = {SGI79744, SGI0, SGI79745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79746, SGI79747, SGI79747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17742[] = {SGI79748, SGI0, SGI79749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79750, SGI79751, SGI79751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17743[] = {SGI79752, SGI0, SGI79753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79754, SGI79755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17744[] = {SGI79756, SGI0, SGI79757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79758, SGI79759, SGI79759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17745[] = {SGI79760, SGI0, SGI79761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79762, SGI79763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17746[] = {SGI79764, SGI0, SGI79765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79766, SGI79767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17747[] = {SGI79768, SGI0, SGI79769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79770, SGI79771, SGI79771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17748[] = {SGI79772, SGI0, SGI79773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79774, SGI79775, SGI79775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17749[] = {SGI79776, SGI0, SGI79777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79778, SGI79779, SGI79779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17750[] = {SGI79780, SGI0, SGI79781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79782, SGI79783, SGI79783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17751[] = {SGI79784, SGI0, SGI79785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79786, SGI79787, SGI79787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17752[] = {SGI79788, SGI0, SGI79789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79790, SGI79791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17753[] = {SGI79792, SGI0, SGI79793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79794, SGI79795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17754[] = {SGI79796, SGI0, SGI79797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79798, SGI79799, SGI79799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17755[] = {SGI79800, SGI0, SGI79801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79802, SGI79803, SGI79803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17756[] = {SGI79804, SGI0, SGI79805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79806, SGI79807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17757[] = {SGI79808, SGI0, SGI79809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79810, SGI79811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17758[] = {SGI79812, SGI0, SGI79813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79814, SGI79815, SGI79815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17759[] = {SGI79816, SGI0, SGI79817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79818, SGI79819, SGI79819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17760[] = {SGI79820, SGI0, SGI79821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79822, SGI79823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17761[] = {SGI79824, SGI0, SGI79825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79826, SGI79827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17762[] = {SGI79828, SGI0, SGI79829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79830, SGI79831, SGI79831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17763[] = {SGI79832, SGI0, SGI79833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79834, SGI79835, SGI79835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17764[] = {SGI79836, SGI0, SGI79837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79838, SGI79839, SGI79839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17765[] = {SGI79840, SGI0, SGI79841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79842, SGI79843, SGI79843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17766[] = {SGI79844, SGI0, SGI79845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79846, SGI79847, SGI79847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17767[] = {SGI79848, SGI0, SGI79849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79850, SGI79851, SGI79851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17768[] = {SGI79852, SGI0, SGI79853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79854, SGI79855, SGI79855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17769[] = {SGI79856, SGI0, SGI79857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79858, SGI79859, SGI79859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17770[] = {SGI79860, SGI0, SGI79861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79862, SGI79863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17771[] = {SGI79864, SGI0, SGI79865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79866, SGI79867, SGI79867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17772[] = {SGI79868, SGI0, SGI79869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79870, SGI79871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17773[] = {SGI79872, SGI0, SGI79873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79874, SGI79875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17774[] = {SGI79876, SGI0, SGI79877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79878, SGI79879, SGI79879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17775[] = {SGI79880, SGI0, SGI79881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79882, SGI79883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17776[] = {SGI0, SGI79884, SGI79885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79886, SGI0, SGI0, SGI0, SGI79887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17777[] = {SGI79888, SGI0, SGI79889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79890, SGI79891, SGI79891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17778[] = {SGI0, SGI79892, SGI79893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79894, SGI0, SGI0, SGI0, SGI79896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17779[] = {SGI79895, SGI0, SGI79897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79898, SGI79899, SGI79899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17780[] = {SGI0, SGI79900, SGI79901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79902, SGI0, SGI0, SGI0, SGI79903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17781[] = {SGI0, SGI79904, SGI79905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79907, SGI0, SGI0, SGI0, SGI79909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17782[] = {SGI79906, SGI0, SGI79908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79910, SGI79911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17783[] = {SGI0, SGI79913, SGI79914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79916, SGI0, SGI0, SGI0, SGI79919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17784[] = {SGI79912, SGI0, SGI79915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79917, SGI79918, SGI79918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17785[] = {SGI0, SGI79920, SGI79921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79922, SGI0, SGI0, SGI0, SGI79924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17786[] = {SGI79923, SGI0, SGI79925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79926, SGI79927, SGI79927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17787[] = {SGI0, SGI79928, SGI79929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79930, SGI0, SGI0, SGI0, SGI79931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17788[] = {SGI79932, SGI0, SGI79933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79935, SGI79937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17789[] = {SGI0, SGI79934, SGI79936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79938, SGI0, SGI0, SGI0, SGI79939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17790[] = {SGI79940, SGI0, SGI79941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79942, SGI79943, SGI79943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17791[] = {SGI0, SGI79944, SGI79945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79946, SGI0, SGI0, SGI0, SGI79947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17792[] = {SGI0, SGI79948, SGI79949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79950, SGI0, SGI0, SGI0, SGI79951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17793[] = {SGI79952, SGI0, SGI79953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79954, SGI79955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17794[] = {SGI0, SGI79956, SGI79957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79958, SGI0, SGI0, SGI0, SGI79959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17795[] = {SGI0, SGI79960, SGI79961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79962, SGI0, SGI0, SGI0, SGI79963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17796[] = {SGI0, SGI79964, SGI79965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79966, SGI0, SGI0, SGI0, SGI79967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17797[] = {SGI0, SGI0, SGI79968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79969, SGI79970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17798[] = {SGI0, SGI79971, SGI79972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79973, SGI0, SGI0, SGI0, SGI79974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17799[] = {SGI79975, SGI0, SGI79976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79977, SGI79978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17800[] = {SGI0, SGI79979, SGI79980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79981, SGI0, SGI0, SGI0, SGI79982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17801[] = {SGI0, SGI79983, SGI79984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79985, SGI0, SGI0, SGI0, SGI79986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17802[] = {SGI0, SGI79987, SGI79988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79989, SGI0, SGI0, SGI0, SGI79990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17803[] = {SGI0, SGI79991, SGI79992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79993, SGI0, SGI0, SGI0, SGI79994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17804[] = {SGI0, SGI79996, SGI79997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI79998, SGI0, SGI0, SGI0, SGI79999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17805[] = {SGI0, SGI80000, SGI80001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80002, SGI0, SGI0, SGI0, SGI80003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17806[] = {SGI0, SGI80004, SGI80005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80006, SGI0, SGI0, SGI0, SGI80007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17807[] = {SGI0, SGI80008, SGI80009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80010, SGI0, SGI0, SGI0, SGI80011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17808[] = {SGI0, SGI80012, SGI80013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80014, SGI0, SGI0, SGI0, SGI80015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17809[] = {SGI0, SGI80016, SGI80017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80018, SGI0, SGI0, SGI0, SGI80019, SGI80019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17810[] = {SGI0, SGI80020, SGI80021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80022, SGI0, SGI0, SGI0, SGI80023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17811[] = {SGI0, SGI80024, SGI80025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80026, SGI0, SGI0, SGI0, SGI80027, SGI80027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17812[] = {SGI0, SGI80028, SGI80029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80030, SGI0, SGI0, SGI0, SGI80031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17813[] = {SGI80032, SGI80033, SGI80034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80035, SGI0, SGI0, SGI0, SGI80036, SGI80036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17814[] = {SGI80037, SGI80038, SGI80039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80040, SGI0, SGI0, SGI0, SGI80041, SGI80041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17815[] = {SGI80042, SGI80043, SGI80044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80045, SGI0, SGI0, SGI0, SGI80046, SGI80046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17816[] = {SGI79995, SGI0, SGI80047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80048, SGI80049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17817[] = {SGI80050, SGI0, SGI80051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80052, SGI80053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17818[] = {SGI80054, SGI80055, SGI80056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80057, SGI0, SGI0, SGI0, SGI80058, SGI80058, SGI80059, SGI0, SGI80060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17819[] = {SGI80062, SGI0, SGI80065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80067, SGI80068, SGI80068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17820[] = {SGI80061, SGI80063, SGI80064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80066, SGI0, SGI0, SGI0, SGI80069, SGI80069, SGI0, SGI0, SGI80070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17821[] = {SGI80071, SGI80072, SGI80073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80074, SGI0, SGI0, SGI0, SGI80075, SGI80075, SGI0, SGI0, SGI80076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17822[] = {SGI80077, SGI0, SGI80079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80082, SGI80084, SGI80084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17823[] = {SGI80078, SGI80080, SGI80081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80083, SGI0, SGI0, SGI0, SGI80085, SGI80085, SGI80086, SGI0, SGI80087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17824[] = {SGI80088, SGI0, SGI80089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80090, SGI80091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17825[] = {SGI80092, SGI0, SGI80093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80094, SGI80095, SGI80095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17826[] = {SGI80096, SGI0, SGI80097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80098, SGI80099, SGI80099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17827[] = {SGI80100, SGI0, SGI80101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80102, SGI80103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17828[] = {SGI80104, SGI0, SGI80105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80106, SGI80107, SGI80107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17829[] = {SGI80108, SGI0, SGI80109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80110, SGI80111, SGI80111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17830[] = {SGI80112, SGI0, SGI80113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80114, SGI80115, SGI80115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17831[] = {SGI80116, SGI0, SGI80117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80118, SGI80119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17832[] = {SGI80120, SGI0, SGI80121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80122, SGI80123, SGI80123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17833[] = {SGI80124, SGI0, SGI80125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80126, SGI80127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17834[] = {SGI80128, SGI0, SGI80129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80130, SGI80131, SGI80131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17835[] = {SGI80132, SGI0, SGI80133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80134, SGI80135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17836[] = {SGI80136, SGI0, SGI80137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80138, SGI80139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17837[] = {SGI80140, SGI0, SGI80141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80142, SGI80143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17838[] = {SGI80144, SGI0, SGI80145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80146, SGI80147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17839[] = {SGI80148, SGI0, SGI80149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80150, SGI80151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17840[] = {SGI80152, SGI0, SGI80153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80154, SGI80155, SGI80155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17841[] = {SGI80156, SGI0, SGI80157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80158, SGI80159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17842[] = {SGI80160, SGI0, SGI80161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80162, SGI80163, SGI80163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17843[] = {SGI80164, SGI0, SGI80165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80166, SGI80167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17844[] = {SGI80168, SGI0, SGI80169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80170, SGI80171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17845[] = {SGI80172, SGI0, SGI80173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80174, SGI80175, SGI80175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17846[] = {SGI80176, SGI0, SGI80177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80178, SGI80179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17847[] = {SGI80180, SGI0, SGI80181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80182, SGI80183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17848[] = {SGI80184, SGI0, SGI80185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80186, SGI80187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17849[] = {SGI80188, SGI0, SGI80189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80190, SGI80191, SGI80191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17850[] = {SGI80192, SGI0, SGI80193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80194, SGI80195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17851[] = {SGI80196, SGI0, SGI80197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80198, SGI80199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17852[] = {SGI80200, SGI0, SGI80201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80202, SGI80203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17853[] = {SGI80204, SGI0, SGI80205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80206, SGI80207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17854[] = {SGI80208, SGI0, SGI80209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80210, SGI80211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17855[] = {SGI80212, SGI0, SGI80213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80214, SGI80215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17856[] = {SGI80216, SGI0, SGI80217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80218, SGI80219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17857[] = {SGI80220, SGI0, SGI80221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80222, SGI80223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17858[] = {SGI80224, SGI80225, SGI80226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80227, SGI0, SGI0, SGI0, SGI80228, SGI80228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17859[] = {SGI80229, SGI0, SGI80230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80231, SGI80232, SGI80232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17860[] = {SGI80233, SGI0, SGI80234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80235, SGI80236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17861[] = {SGI80237, SGI0, SGI80238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80239, SGI80240, SGI80240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17862[] = {SGI80241, SGI80242, SGI80243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80244, SGI0, SGI0, SGI0, SGI80245, SGI80245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17863[] = {SGI80246, SGI80247, SGI80248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80249, SGI80249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17864[] = {SGI80250, SGI80251, SGI80252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80253, SGI0, SGI0, SGI0, SGI80254, SGI80254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17865[] = {SGI80255, SGI80256, SGI80257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80258, SGI0, SGI0, SGI0, SGI80259, SGI80259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17866[] = {SGI80260, SGI80261, SGI80262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80263, SGI0, SGI0, SGI0, SGI80264, SGI80264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17867[] = {SGI80265, SGI80266, SGI80267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80268, SGI0, SGI0, SGI0, SGI80269, SGI80269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17868[] = {SGI80270, SGI80271, SGI80272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80273, SGI0, SGI0, SGI0, SGI80274, SGI80274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17869[] = {SGI80275, SGI80276, SGI80277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80278, SGI0, SGI0, SGI0, SGI80279, SGI80279, SGI0, SGI0, NULL};
const Segment * const SG17870[] = {SGI80280, SGI80281, SGI80282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80283, SGI0, SGI0, SGI0, SGI80284, SGI80284, SGI0, SGI0, NULL};
const Segment * const SG17871[] = {SGI80285, SGI0, SGI80286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17872[] = {SGI80287, SGI80288, SGI80289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80290, SGI0, SGI0, SGI0, SGI80291, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17873[] = {SGI80292, SGI80293, SGI80294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80295, SGI0, SGI0, SGI0, SGI80296, SGI80296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17874[] = {SGI80297, SGI0, SGI80298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17875[] = {SGI80299, SGI80300, SGI80301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80302, SGI80303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17876[] = {SGI80304, SGI80305, SGI80306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80307, SGI0, SGI0, SGI0, SGI80308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17877[] = {SGI80309, SGI0, SGI80310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17878[] = {SGI80311, SGI80312, SGI80313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80314, SGI0, SGI0, SGI0, SGI80315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17879[] = {SGI80316, SGI80317, SGI80318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80319, SGI0, SGI0, SGI0, SGI80320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17880[] = {SGI80321, SGI0, SGI80322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17881[] = {SGI80323, SGI80324, SGI80325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80326, SGI0, SGI0, SGI0, SGI80327, SGI80327, SGI80328, SGI0, SGI80330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17882[] = {SGI80329, SGI80331, SGI80332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80333, SGI0, SGI0, SGI0, SGI80334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17883[] = {SGI80335, SGI80336, SGI80337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80338, SGI0, SGI0, SGI0, SGI80339, SGI80339, SGI0, SGI0, SGI80342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17884[] = {SGI80340, SGI80341, SGI80343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80344, SGI0, SGI0, SGI0, SGI80345, SGI80345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17885[] = {SGI80346, SGI80347, SGI80348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80349, SGI0, SGI0, SGI0, SGI80350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17886[] = {SGI80351, SGI0, SGI80352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17887[] = {SGI80353, SGI80354, SGI80355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80356, SGI0, SGI0, SGI0, SGI80357, SGI80357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17888[] = {SGI80358, SGI80359, SGI80360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80361, SGI0, SGI0, SGI0, SGI80362, SGI80362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17889[] = {SGI80363, SGI0, SGI80364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17890[] = {SGI80365, SGI80366, SGI80367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80368, SGI0, SGI0, SGI0, SGI80369, SGI80369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17891[] = {SGI80370, SGI80371, SGI80372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80373, SGI0, SGI0, SGI0, SGI80374, SGI80374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17892[] = {SGI80375, SGI0, SGI80376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17893[] = {SGI80377, SGI80378, SGI80379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80380, SGI0, SGI0, SGI0, SGI80381, SGI80381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17894[] = {SGI80382, SGI80383, SGI80384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80385, SGI0, SGI0, SGI0, SGI80386, SGI80386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17895[] = {SGI80387, SGI0, SGI80388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17896[] = {SGI80389, SGI80390, SGI80391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80392, SGI0, SGI0, SGI0, SGI80393, SGI80393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17897[] = {SGI80394, SGI80395, SGI80396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80397, SGI0, SGI0, SGI0, SGI80398, SGI80398, SGI80399, SGI0, SGI80400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17898[] = {SGI80401, SGI80402, SGI80403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80404, SGI0, SGI0, SGI0, SGI80405, SGI80405, SGI80406, SGI0, SGI80407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17899[] = {SGI80408, SGI80409, SGI80410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80411, SGI0, SGI0, SGI0, SGI80412, SGI80412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17900[] = {SGI80413, SGI80414, SGI80415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80416, SGI0, SGI0, SGI0, SGI80417, SGI80417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17901[] = {SGI80418, SGI80419, SGI80420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80421, SGI0, SGI0, SGI0, SGI80422, SGI80422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17902[] = {SGI80423, SGI80424, SGI80425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80426, SGI0, SGI0, SGI0, SGI80427, SGI80427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17903[] = {SGI80428, SGI80429, SGI80430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80431, SGI0, SGI0, SGI0, SGI80432, SGI80432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17904[] = {SGI80433, SGI80434, SGI80435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80436, SGI0, SGI0, SGI0, SGI80437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17905[] = {SGI80438, SGI80439, SGI80440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80441, SGI0, SGI0, SGI0, SGI80442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17906[] = {SGI80443, SGI0, SGI80444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17907[] = {SGI80445, SGI80446, SGI80447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80448, SGI0, SGI0, SGI0, SGI80449, SGI80449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17908[] = {SGI80450, SGI80451, SGI80452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80453, SGI0, SGI0, SGI0, SGI80455, SGI80455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17909[] = {SGI80454, SGI80456, SGI80457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80458, SGI0, SGI0, SGI0, SGI80459, SGI80459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17910[] = {SGI80460, SGI80461, SGI80462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80463, SGI0, SGI0, SGI0, SGI80464, SGI80464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17911[] = {SGI80465, SGI80466, SGI80467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80468, SGI0, SGI0, SGI0, SGI80471, SGI80471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17912[] = {SGI80469, SGI80470, SGI80472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80473, SGI0, SGI0, SGI0, SGI80474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17913[] = {SGI80475, SGI80476, SGI80477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80480, SGI0, SGI0, SGI0, SGI80481, SGI80481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17914[] = {SGI80478, SGI0, SGI80479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17915[] = {SGI80482, SGI0, SGI80484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17916[] = {SGI80483, SGI80485, SGI80486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80487, SGI0, SGI0, SGI0, SGI80488, SGI80488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17917[] = {SGI80489, SGI80490, SGI80491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80492, SGI0, SGI0, SGI0, SGI80493, SGI80493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17918[] = {SGI80494, SGI80495, SGI80496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80497, SGI0, SGI0, SGI0, SGI80498, SGI80498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17919[] = {SGI80499, SGI80500, SGI80501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80502, SGI0, SGI0, SGI0, SGI80503, SGI80503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17920[] = {SGI80504, SGI0, SGI80505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17921[] = {SGI80506, SGI80507, SGI80508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80509, SGI0, SGI0, SGI0, SGI80510, SGI80510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17922[] = {SGI80511, SGI80512, SGI80513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80514, SGI0, SGI0, SGI0, SGI80515, SGI80515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17923[] = {SGI80516, SGI80517, SGI80518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80519, SGI0, SGI0, SGI0, SGI80520, SGI80520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17924[] = {SGI80521, SGI80522, SGI80523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80524, SGI0, SGI0, SGI0, SGI80525, SGI80525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17925[] = {SGI80526, SGI80527, SGI80528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80529, SGI0, SGI0, SGI0, SGI80530, SGI80530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17926[] = {SGI80531, SGI80532, SGI80533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80534, SGI0, SGI0, SGI0, SGI80535, SGI80535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17927[] = {SGI80536, SGI80537, SGI80538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80539, SGI0, SGI0, SGI0, SGI80540, SGI80540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17928[] = {SGI80541, SGI80542, SGI80543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80544, SGI0, SGI0, SGI0, SGI80545, SGI80545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17929[] = {SGI80546, SGI80547, SGI80548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80549, SGI0, SGI0, SGI0, SGI80550, SGI80550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17930[] = {SGI80551, SGI80553, SGI80555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80557, SGI0, SGI0, SGI0, SGI80559, SGI80559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17931[] = {SGI80552, SGI80554, SGI80556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80558, SGI0, SGI0, SGI0, SGI80560, SGI80560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17932[] = {SGI80564, SGI0, SGI80565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17933[] = {SGI80561, SGI80562, SGI80563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80566, SGI0, SGI0, SGI0, SGI80567, SGI80567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17934[] = {SGI80568, SGI80569, SGI80570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80571, SGI0, SGI0, SGI0, SGI80572, SGI80572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17935[] = {SGI80573, SGI80574, SGI80575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80576, SGI0, SGI0, SGI0, SGI80577, SGI80577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17936[] = {SGI80578, SGI80579, SGI80580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80581, SGI0, SGI0, SGI0, SGI80582, SGI80582, SGI0, SGI80583, SGI0, NULL};
const Segment * const SG17937[] = {SGI80584, SGI80585, SGI80586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80587, SGI0, SGI0, SGI0, SGI80588, SGI80588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17938[] = {SGI80589, SGI0, SGI80590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17939[] = {SGI80591, SGI80592, SGI80593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80594, SGI0, SGI0, SGI0, SGI80595, SGI80595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17940[] = {SGI80596, SGI80597, SGI80598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80599, SGI0, SGI0, SGI0, SGI80600, SGI80600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17941[] = {SGI80601, SGI80602, SGI80603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80604, SGI0, SGI0, SGI0, SGI80605, SGI80605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17942[] = {SGI80606, SGI80607, SGI80608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80609, SGI0, SGI0, SGI0, SGI80610, SGI80610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17943[] = {SGI80611, SGI80612, SGI80613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80614, SGI0, SGI0, SGI0, SGI80615, SGI80615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17944[] = {SGI80616, SGI80617, SGI80618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80619, SGI80619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17945[] = {SGI80620, SGI80621, SGI80622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80623, SGI0, SGI0, SGI0, SGI80624, SGI80624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17946[] = {SGI80625, SGI0, SGI80626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80627, SGI0, SGI0, SGI0, SGI80628, SGI80628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17947[] = {SGI80629, SGI0, SGI80630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17948[] = {SGI80631, SGI80632, SGI80633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80634, SGI0, SGI0, SGI0, SGI80635, SGI80635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17949[] = {SGI80636, SGI80637, SGI80638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80639, SGI0, SGI0, SGI0, SGI80640, SGI80640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17950[] = {SGI80641, SGI80642, SGI80643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17951[] = {SGI80644, SGI80645, SGI80646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80647, SGI80647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17952[] = {SGI80648, SGI0, SGI80649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17953[] = {SGI80650, SGI80651, SGI80652, SGI0, SGI0, SGI0, SGI80653, SGI80654, SGI0, SGI80655, SGI0, SGI0, SGI0, SGI80656, SGI80656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17954[] = {SGI80657, SGI80658, SGI80659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80660, SGI0, SGI0, SGI0, SGI80661, SGI80661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17955[] = {SGI80662, SGI80663, SGI80664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80665, SGI0, SGI0, SGI0, SGI80666, SGI80666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17956[] = {SGI80667, SGI80668, SGI80669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80670, SGI0, SGI0, SGI0, SGI80671, SGI80671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17957[] = {SGI80672, SGI80673, SGI80674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80675, SGI0, SGI0, SGI0, SGI80676, SGI80676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17958[] = {SGI80677, SGI80678, SGI80679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80680, SGI0, SGI0, SGI0, SGI80681, SGI80681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17959[] = {SGI80682, SGI80683, SGI80684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80685, SGI0, SGI0, SGI0, SGI80686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17960[] = {SGI80687, SGI80688, SGI80689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80690, SGI0, SGI0, SGI0, SGI80691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17961[] = {SGI80692, SGI0, SGI80693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17962[] = {SGI80694, SGI80695, SGI80696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80697, SGI0, SGI0, SGI0, SGI80698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17963[] = {SGI80699, SGI80700, SGI80701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80702, SGI0, SGI0, SGI0, SGI80703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17964[] = {SGI80704, SGI80705, SGI80706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80707, SGI0, SGI0, SGI0, SGI80708, SGI80708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17965[] = {SGI80709, SGI80710, SGI80711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80712, SGI0, SGI0, SGI0, SGI80713, SGI80713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17966[] = {SGI80714, SGI0, SGI80715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17967[] = {SGI80716, SGI80717, SGI80718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80719, SGI0, SGI0, SGI0, SGI80720, SGI80720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17968[] = {SGI80721, SGI80722, SGI80723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80724, SGI0, SGI0, SGI0, SGI80725, SGI80725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17969[] = {SGI80726, SGI80727, SGI80728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80729, SGI0, SGI0, SGI0, SGI80730, SGI80730, SGI80731, SGI0, SGI80732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17970[] = {SGI80733, SGI0, SGI80734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17971[] = {SGI80735, SGI80736, SGI80737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80738, SGI0, SGI0, SGI0, SGI80739, SGI80739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80740, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17972[] = {SGI80741, SGI80742, SGI80743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80744, SGI0, SGI0, SGI0, SGI80746, SGI80746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17973[] = {SGI80745, SGI80747, SGI80748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80749, SGI0, SGI0, SGI0, SGI80750, SGI80750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80751, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17974[] = {SGI80752, SGI0, SGI80753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17975[] = {SGI80754, SGI80755, SGI80756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80757, SGI0, SGI0, SGI0, SGI80758, SGI80758, SGI0, SGI80759, SGI80760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17976[] = {SGI80761, SGI80762, SGI80763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80764, SGI0, SGI0, SGI0, SGI80765, SGI80765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80766, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17977[] = {SGI80767, SGI80768, SGI80769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80770, SGI0, SGI0, SGI0, SGI80771, SGI80771, SGI80772, SGI0, SGI80773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17978[] = {SGI80774, SGI80775, SGI80776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80777, SGI0, SGI0, SGI0, SGI80778, SGI80778, SGI0, SGI0, SGI80779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17979[] = {SGI80780, SGI80781, SGI80782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80783, SGI0, SGI0, SGI0, SGI80784, SGI80784, SGI80785, SGI0, SGI80786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17980[] = {SGI80787, SGI80788, SGI80789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80790, SGI0, SGI0, SGI0, SGI80791, SGI80791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17981[] = {SGI80792, SGI80793, SGI80794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80795, SGI0, SGI0, SGI0, SGI80796, SGI80796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17982[] = {SGI80797, SGI80798, SGI80799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80800, SGI0, SGI0, SGI0, SGI80801, SGI80801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17983[] = {SGI80802, SGI80803, SGI80804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80805, SGI80805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17984[] = {SGI80806, SGI80807, SGI80808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80809, SGI0, SGI0, SGI0, SGI80810, SGI80810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17985[] = {SGI80811, SGI80812, SGI80813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80814, SGI0, SGI0, SGI0, SGI80815, SGI80815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17986[] = {SGI80816, SGI80817, SGI80818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80819, SGI0, SGI0, SGI0, SGI80820, SGI80820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17987[] = {SGI80821, SGI80822, SGI80823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80824, SGI0, SGI0, SGI0, SGI80825, SGI80825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17988[] = {SGI80826, SGI80827, SGI80828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80829, SGI0, SGI0, SGI0, SGI80830, SGI80830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17989[] = {SGI80831, SGI80832, SGI80833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80834, SGI0, SGI0, SGI0, SGI80835, SGI80835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17990[] = {SGI80836, SGI80837, SGI80838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80839, SGI0, SGI0, SGI0, SGI80840, SGI80840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17991[] = {SGI80841, SGI80842, SGI80843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80844, SGI0, SGI0, SGI0, SGI80845, SGI80845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17992[] = {SGI80846, SGI80847, SGI80848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80849, SGI0, SGI0, SGI0, SGI80850, SGI80850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17993[] = {SGI80851, SGI0, SGI80852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17994[] = {SGI80853, SGI80854, SGI80855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80856, SGI0, SGI0, SGI0, SGI80857, SGI80857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17995[] = {SGI80858, SGI80859, SGI80860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80861, SGI0, SGI0, SGI0, SGI80862, SGI80862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17996[] = {SGI80863, SGI80864, SGI80865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80866, SGI0, SGI0, SGI0, SGI80867, SGI80867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17997[] = {SGI80868, SGI80869, SGI80870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80871, SGI0, SGI0, SGI0, SGI80872, SGI80872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17998[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80873, SGI0, SGI0, SGI80874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG17999[] = {SGI80875, SGI80876, SGI80877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80878, SGI0, SGI0, SGI0, SGI80879, SGI80879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18000[] = {SGI80880, SGI80881, SGI80882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80883, SGI80884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18001[] = {SGI80885, SGI80886, SGI80887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80888, SGI80888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18002[] = {SGI80889, SGI80890, SGI80891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80892, SGI0, SGI0, SGI0, SGI80893, SGI80893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18003[] = {SGI80894, SGI0, SGI80895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18004[] = {SGI0, SGI80896, SGI80897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18005[] = {SGI80898, SGI80899, SGI80900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80901, SGI0, SGI0, SGI0, SGI80902, SGI80902, SGI80903, SGI0, SGI80904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18006[] = {SGI80905, SGI80906, SGI80907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80908, SGI0, SGI0, SGI0, SGI80909, SGI80909, SGI80910, SGI0, SGI80911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18007[] = {SGI80912, SGI80913, SGI80914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80915, SGI0, SGI0, SGI0, SGI80916, SGI80916, SGI80917, SGI0, SGI80918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18008[] = {SGI80919, SGI80920, SGI80921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80922, SGI0, SGI0, SGI0, SGI80923, SGI80923, SGI0, SGI80924, SGI0, NULL};
const Segment * const SG18009[] = {SGI80925, SGI80926, SGI80927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80928, SGI0, SGI0, SGI0, SGI80929, SGI80929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18010[] = {SGI80930, SGI80931, SGI80932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80933, SGI0, SGI0, SGI0, SGI80934, SGI80934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18011[] = {SGI80935, SGI80936, SGI80937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80938, SGI0, SGI0, SGI0, SGI80939, SGI80939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18012[] = {SGI80940, SGI80941, SGI80942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80943, SGI0, SGI0, SGI0, SGI80944, SGI80944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18013[] = {SGI80945, SGI80946, SGI80947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80948, SGI0, SGI0, SGI0, SGI80949, SGI80949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18014[] = {SGI80950, SGI80951, SGI80952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80953, SGI0, SGI0, SGI0, SGI80954, SGI80954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18015[] = {SGI80955, SGI0, SGI80956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18016[] = {SGI80957, SGI80958, SGI80959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80960, SGI0, SGI0, SGI0, SGI80961, SGI80961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18017[] = {SGI80962, SGI0, SGI80963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18018[] = {SGI80964, SGI80965, SGI80966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80967, SGI0, SGI0, SGI0, SGI80968, SGI80968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18019[] = {SGI80969, SGI80970, SGI80971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80972, SGI0, SGI0, SGI0, SGI80973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18020[] = {SGI80974, SGI80975, SGI80976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80977, SGI0, SGI0, SGI0, SGI80978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18021[] = {SGI80979, SGI80980, SGI80981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80982, SGI0, SGI0, SGI0, SGI80983, SGI80983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18022[] = {SGI80984, SGI80985, SGI80986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80987, SGI0, SGI0, SGI0, SGI80988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18023[] = {SGI80989, SGI0, SGI80990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18024[] = {SGI80991, SGI80992, SGI80993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80994, SGI0, SGI0, SGI0, SGI80995, SGI80995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18025[] = {SGI80996, SGI80997, SGI80998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI80999, SGI0, SGI0, SGI0, SGI81000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18026[] = {SGI81001, SGI0, SGI81002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81003, SGI0, SGI0, SGI0, SGI81004, SGI81004, SGI0, SGI0, SGI81005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18027[] = {SGI81006, SGI0, SGI81007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81008, SGI0, SGI0, SGI0, SGI81009, SGI81009, SGI0, SGI0, SGI81010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18028[] = {SGI81011, SGI81012, SGI81013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81014, SGI0, SGI0, SGI0, SGI81015, SGI81015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18029[] = {SGI81016, SGI81017, SGI81018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81019, SGI0, SGI0, SGI0, SGI81020, SGI81020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18030[] = {SGI81021, SGI81022, SGI81023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81024, SGI0, SGI0, SGI0, SGI81025, SGI81025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18031[] = {SGI81026, SGI81027, SGI81028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81029, SGI0, SGI0, SGI0, SGI81030, SGI81030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18032[] = {SGI81031, SGI81032, SGI81033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81034, SGI0, SGI0, SGI0, SGI81035, SGI81035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18033[] = {SGI81036, SGI81037, SGI81038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81039, SGI0, SGI0, SGI0, SGI81040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18034[] = {SGI81041, SGI81042, SGI81043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81044, SGI0, SGI0, SGI0, SGI81045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18035[] = {SGI81046, SGI81047, SGI81048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81049, SGI0, SGI0, SGI0, SGI81050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18036[] = {SGI81051, SGI81052, SGI81053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81054, SGI0, SGI0, SGI0, SGI81055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18037[] = {SGI81056, SGI81057, SGI81058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81059, SGI0, SGI0, SGI0, SGI81060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18038[] = {SGI81061, SGI81062, SGI81063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81064, SGI0, SGI0, SGI0, SGI81065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18039[] = {SGI81066, SGI81067, SGI81068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81069, SGI0, SGI0, SGI0, SGI81070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18040[] = {SGI81071, SGI81072, SGI81073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81074, SGI0, SGI0, SGI0, SGI81075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18041[] = {SGI81076, SGI0, SGI81077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18042[] = {SGI81078, SGI0, SGI81079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18043[] = {SGI81080, SGI81081, SGI81082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81083, SGI0, SGI0, SGI0, SGI81084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18044[] = {SGI81085, SGI81086, SGI81087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81088, SGI0, SGI0, SGI0, SGI81089, SGI81089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18045[] = {SGI81090, SGI81091, SGI81092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81093, SGI0, SGI0, SGI0, SGI81094, SGI81094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18046[] = {SGI81095, SGI81096, SGI81097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81098, SGI0, SGI0, SGI0, SGI81099, SGI81099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18047[] = {SGI81100, SGI81101, SGI81102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81103, SGI0, SGI0, SGI0, SGI81104, SGI81104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18048[] = {SGI81105, SGI81106, SGI81107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81108, SGI0, SGI0, SGI0, SGI81109, SGI81109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18049[] = {SGI81110, SGI81111, SGI81112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81114, SGI0, SGI0, SGI0, SGI81116, SGI81116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18050[] = {SGI81113, SGI0, SGI81115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81117, SGI81118, SGI81118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18051[] = {SGI81119, SGI81120, SGI81121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81122, SGI0, SGI0, SGI0, SGI81123, SGI81123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18052[] = {SGI81124, SGI81125, SGI81126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81127, SGI0, SGI0, SGI0, SGI81128, SGI81128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18053[] = {SGI81129, SGI0, SGI81130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18054[] = {SGI81131, SGI0, SGI81132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81133, SGI81134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18055[] = {SGI81135, SGI81136, SGI81137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81138, SGI0, SGI0, SGI0, SGI81139, SGI81139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18056[] = {SGI81140, SGI81141, SGI81142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81143, SGI0, SGI0, SGI0, SGI81144, SGI81144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18057[] = {SGI81145, SGI81146, SGI81147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81148, SGI0, SGI0, SGI0, SGI81149, SGI81149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18058[] = {SGI81150, SGI81151, SGI81152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81153, SGI81153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18059[] = {SGI81154, SGI0, SGI81155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18060[] = {SGI81156, SGI81157, SGI81158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81159, SGI0, SGI0, SGI0, SGI81160, SGI81160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18061[] = {SGI81161, SGI81162, SGI81163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81164, SGI0, SGI0, SGI0, SGI81165, SGI81165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18062[] = {SGI81166, SGI81167, SGI81168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81169, SGI0, SGI0, SGI0, SGI81170, SGI81170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18063[] = {SGI81171, SGI81172, SGI81173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81174, SGI0, SGI0, SGI0, SGI81175, SGI81175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18064[] = {SGI81176, SGI81177, SGI81178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81179, SGI0, SGI0, SGI0, SGI81180, SGI81180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18065[] = {SGI81181, SGI81182, SGI81183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81184, SGI0, SGI0, SGI0, SGI81185, SGI81185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18066[] = {SGI81186, SGI0, SGI81187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18067[] = {SGI81188, SGI81189, SGI81190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81191, SGI81191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18068[] = {SGI81192, SGI81193, SGI81194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81195, SGI0, SGI0, SGI0, SGI81196, SGI81196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18069[] = {SGI81197, SGI0, SGI81198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18070[] = {SGI81199, SGI81200, SGI81201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81202, SGI0, SGI0, SGI0, SGI81203, SGI81203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18071[] = {SGI81204, SGI81205, SGI81206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81207, SGI0, SGI0, SGI0, SGI81208, SGI81208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18072[] = {SGI81209, SGI0, SGI81210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18073[] = {SGI81211, SGI81212, SGI81213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81214, SGI0, SGI0, SGI0, SGI81215, SGI81215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18074[] = {SGI81216, SGI81217, SGI81218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81219, SGI0, SGI0, SGI0, SGI81220, SGI81220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18075[] = {SGI81221, SGI0, SGI81222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18076[] = {SGI81223, SGI0, SGI81224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81225, SGI0, SGI0, SGI0, SGI81226, SGI81226, SGI0, SGI0, SGI81227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18077[] = {SGI81228, SGI0, SGI81229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81230, SGI0, SGI0, SGI0, SGI81231, SGI81231, SGI0, SGI0, SGI81232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18078[] = {SGI81233, SGI0, SGI81234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81235, SGI0, SGI0, SGI0, SGI81236, SGI81236, SGI0, SGI0, SGI81237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18079[] = {SGI81238, SGI0, SGI81239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81240, SGI0, SGI0, SGI0, SGI81241, SGI81241, SGI0, SGI0, SGI81242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18080[] = {SGI81243, SGI81244, SGI81245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81246, SGI0, SGI0, SGI0, SGI81247, SGI81248, SGI0, SGI0, SGI81249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18081[] = {SGI81250, SGI81251, SGI81252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81253, SGI0, SGI0, SGI0, SGI81254, SGI81254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18082[] = {SGI81255, SGI81256, SGI81257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81258, SGI0, SGI0, SGI0, SGI81259, SGI81259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18083[] = {SGI81260, SGI81261, SGI81262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81263, SGI81264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18084[] = {SGI81265, SGI0, SGI81266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18085[] = {SGI81267, SGI81268, SGI81269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81270, SGI81270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18086[] = {SGI81271, SGI0, SGI81272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18087[] = {SGI81273, SGI81274, SGI81275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81276, SGI0, SGI0, SGI0, SGI81277, SGI81277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18088[] = {SGI81278, SGI81279, SGI81280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81281, SGI0, SGI0, SGI0, SGI81282, SGI81282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18089[] = {SGI81283, SGI81284, SGI81285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81286, SGI0, SGI0, SGI0, SGI81287, SGI81287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18090[] = {SGI81288, SGI81289, SGI81290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81291, SGI0, SGI0, SGI0, SGI81292, SGI81292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18091[] = {SGI81293, SGI81294, SGI81295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81296, SGI0, SGI0, SGI0, SGI81297, SGI81297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18092[] = {SGI81298, SGI81299, SGI81300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81301, SGI0, SGI0, SGI0, SGI81302, SGI81302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18093[] = {SGI81303, SGI0, SGI81304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18094[] = {SGI81305, SGI81306, SGI81307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81308, SGI0, SGI0, SGI0, SGI81309, SGI81309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18095[] = {SGI81310, SGI81311, SGI81312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81313, SGI0, SGI0, SGI0, SGI81314, SGI81314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18096[] = {SGI81315, SGI81316, SGI81317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81318, SGI0, SGI0, SGI0, SGI81319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18097[] = {SGI81320, SGI0, SGI81321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18098[] = {SGI81322, SGI81323, SGI81324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81325, SGI0, SGI0, SGI0, SGI81326, SGI81326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18099[] = {SGI81327, SGI0, SGI81328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18100[] = {SGI81329, SGI0, SGI81330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81331, SGI0, SGI0, SGI0, SGI81332, SGI81332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18101[] = {SGI81333, SGI81334, SGI81335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81336, SGI0, SGI0, SGI0, SGI81337, SGI81337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18102[] = {SGI81338, SGI0, SGI81339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18103[] = {SGI81340, SGI81341, SGI81342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81343, SGI81343, SGI0, SGI0, SGI81344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18104[] = {SGI81345, SGI81346, SGI81347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81348, SGI81348, SGI0, SGI0, SGI81349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18105[] = {SGI81350, SGI81351, SGI81352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81353, SGI81353, SGI0, SGI0, SGI81354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18106[] = {SGI81355, SGI81356, SGI81357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81358, SGI0, SGI0, SGI0, SGI81359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18107[] = {SGI81360, SGI81361, SGI81362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81363, SGI0, SGI0, SGI0, SGI81364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18108[] = {SGI81365, SGI81366, SGI81367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81368, SGI0, SGI0, SGI0, SGI81369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18109[] = {SGI81370, SGI81371, SGI81372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81373, SGI0, SGI0, SGI0, SGI81374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18110[] = {SGI81375, SGI81376, SGI81377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81378, SGI0, SGI0, SGI0, SGI81379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18111[] = {SGI81380, SGI81381, SGI81382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81383, SGI0, SGI0, SGI0, SGI81384, SGI81384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18112[] = {SGI81385, SGI81386, SGI81387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81388, SGI0, SGI0, SGI0, SGI81389, SGI81389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18113[] = {SGI81390, SGI0, SGI81391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18114[] = {SGI81392, SGI81393, SGI81394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81395, SGI81396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18115[] = {SGI81397, SGI81398, SGI81399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81400, SGI0, SGI0, SGI0, SGI81401, SGI81401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18116[] = {SGI81402, SGI0, SGI81403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81404, SGI0, SGI0, SGI0, SGI81407, SGI81407, SGI0, SGI0, SGI81409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18117[] = {SGI81405, SGI81406, SGI81408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81410, SGI81410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18118[] = {SGI81411, SGI81412, SGI81413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81414, SGI0, SGI0, SGI0, SGI81415, SGI81415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18119[] = {SGI81416, SGI81417, SGI81418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81419, SGI0, SGI0, SGI0, SGI81420, SGI81420, SGI0, SGI81421, SGI0, NULL};
const Segment * const SG18120[] = {SGI81422, SGI81423, SGI81424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81425, SGI0, SGI0, SGI0, SGI81426, SGI81426, SGI0, SGI81427, SGI0, NULL};
const Segment * const SG18121[] = {SGI81428, SGI81429, SGI81430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81431, SGI0, SGI0, SGI0, SGI81432, SGI81432, SGI0, SGI81433, SGI0, NULL};
const Segment * const SG18122[] = {SGI81434, SGI81435, SGI81436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81437, SGI0, SGI0, SGI0, SGI81438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18123[] = {SGI81439, SGI81440, SGI81441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81442, SGI0, SGI0, SGI0, SGI81443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18124[] = {SGI81444, SGI81445, SGI81446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81447, SGI0, SGI0, SGI0, SGI81448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18125[] = {SGI81449, SGI81450, SGI81451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81452, SGI0, SGI0, SGI0, SGI81453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18126[] = {SGI81454, SGI81455, SGI81456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81457, SGI0, SGI0, SGI0, SGI81458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18127[] = {SGI81459, SGI81460, SGI81461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81462, SGI0, SGI0, SGI0, SGI81463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18128[] = {SGI81464, SGI81465, SGI81466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81467, SGI0, SGI0, SGI0, SGI81468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18129[] = {SGI81469, SGI81470, SGI81471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81472, SGI0, SGI0, SGI0, SGI81473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18130[] = {SGI81474, SGI81475, SGI81476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81477, SGI0, SGI0, SGI0, SGI81478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18131[] = {SGI81479, SGI81480, SGI81481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81482, SGI0, SGI0, SGI0, SGI81483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18132[] = {SGI81484, SGI81485, SGI81486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81487, SGI0, SGI0, SGI0, SGI81488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18133[] = {SGI81489, SGI81490, SGI81491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81492, SGI0, SGI0, SGI0, SGI81493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18134[] = {SGI81494, SGI81495, SGI81496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81497, SGI0, SGI0, SGI0, SGI81498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18135[] = {SGI81499, SGI0, SGI81500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18136[] = {SGI81501, SGI81502, SGI81503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81504, SGI0, SGI0, SGI0, SGI81505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18137[] = {SGI81506, SGI0, SGI81507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18138[] = {SGI81508, SGI81509, SGI81510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81511, SGI0, SGI0, SGI0, SGI81512, SGI81512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18139[] = {SGI81513, SGI0, SGI81514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81515, SGI0, SGI0, SGI0, SGI81516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18140[] = {SGI81517, SGI81518, SGI81519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81520, SGI0, SGI0, SGI0, SGI81521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18141[] = {SGI81522, SGI81523, SGI81524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81525, SGI0, SGI0, SGI0, SGI81526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18142[] = {SGI81527, SGI0, SGI81528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18143[] = {SGI81529, SGI0, SGI81530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18144[] = {SGI81531, SGI81532, SGI81533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81534, SGI0, SGI0, SGI0, SGI81535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18145[] = {SGI81536, SGI81537, SGI81538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81539, SGI0, SGI0, SGI0, SGI81540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18146[] = {SGI81541, SGI0, SGI81542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18147[] = {SGI81543, SGI81544, SGI81545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81546, SGI0, SGI0, SGI0, SGI81547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18148[] = {SGI81548, SGI0, SGI81549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18149[] = {SGI81550, SGI81551, SGI81552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81553, SGI0, SGI0, SGI0, SGI81554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18150[] = {SGI81555, SGI0, SGI81556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18151[] = {SGI81557, SGI81558, SGI81559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81560, SGI0, SGI0, SGI0, SGI81561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18152[] = {SGI81562, SGI81563, SGI81564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81565, SGI0, SGI0, SGI0, SGI81566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18153[] = {SGI81567, SGI81568, SGI81569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81570, SGI0, SGI0, SGI0, SGI81571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18154[] = {SGI81572, SGI81573, SGI81574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81575, SGI0, SGI0, SGI0, SGI81576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18155[] = {SGI81577, SGI81578, SGI81579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81580, SGI0, SGI0, SGI0, SGI81581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18156[] = {SGI81582, SGI81583, SGI81584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81585, SGI0, SGI0, SGI0, SGI81586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18157[] = {SGI81587, SGI81588, SGI81589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81590, SGI0, SGI0, SGI0, SGI81591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18158[] = {SGI81593, SGI81595, SGI81597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81599, SGI0, SGI0, SGI0, SGI81601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18159[] = {SGI81592, SGI81594, SGI81596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81598, SGI0, SGI0, SGI0, SGI81600, SGI81600, SGI81602, SGI0, SGI81603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18160[] = {SGI81604, SGI81605, SGI81606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81607, SGI0, SGI0, SGI0, SGI81608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18161[] = {SGI81609, SGI81610, SGI81611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81612, SGI0, SGI0, SGI0, SGI81613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18162[] = {SGI81614, SGI81615, SGI81616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81617, SGI0, SGI0, SGI0, SGI81618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18163[] = {SGI81619, SGI81620, SGI81621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81622, SGI0, SGI0, SGI0, SGI81623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18164[] = {SGI81624, SGI81625, SGI81626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81627, SGI0, SGI0, SGI0, SGI81628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18165[] = {SGI81629, SGI81630, SGI81631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81632, SGI0, SGI0, SGI0, SGI81633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18166[] = {SGI81634, SGI0, SGI81635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18167[] = {SGI81636, SGI81637, SGI81638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81639, SGI0, SGI0, SGI0, SGI81640, SGI81640, SGI81641, SGI0, SGI81642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18168[] = {SGI81643, SGI81644, SGI81645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81646, SGI0, SGI0, SGI0, SGI81647, SGI81647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81648, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18169[] = {SGI81649, SGI81650, SGI81651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81652, SGI0, SGI0, SGI0, SGI81653, SGI81653, SGI81654, SGI0, SGI81655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18170[] = {SGI81656, SGI81657, SGI81658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81659, SGI0, SGI0, SGI0, SGI81660, SGI81660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81661, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18171[] = {SGI81662, SGI81663, SGI81664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81665, SGI0, SGI0, SGI0, SGI81666, SGI81666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18172[] = {SGI81667, SGI0, SGI81668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18173[] = {SGI81669, SGI81670, SGI81671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81672, SGI0, SGI0, SGI0, SGI81673, SGI81673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18174[] = {SGI81674, SGI0, SGI81675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18175[] = {SGI81676, SGI81677, SGI81678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81679, SGI0, SGI0, SGI0, SGI81680, SGI81680, SGI81681, SGI0, SGI81682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18176[] = {SGI81683, SGI81684, SGI81685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81686, SGI0, SGI0, SGI0, SGI81687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18177[] = {SGI81688, SGI81689, SGI81690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81691, SGI0, SGI0, SGI0, SGI81692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18178[] = {SGI81693, SGI81694, SGI81695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81696, SGI0, SGI0, SGI0, SGI81697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18179[] = {SGI81698, SGI81699, SGI81700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81701, SGI0, SGI0, SGI0, SGI81702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18180[] = {SGI81703, SGI81704, SGI81705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81706, SGI0, SGI0, SGI0, SGI81707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18181[] = {SGI81708, SGI81709, SGI81710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81711, SGI0, SGI0, SGI0, SGI81712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18182[] = {SGI81713, SGI81714, SGI81715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81716, SGI0, SGI0, SGI0, SGI81717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18183[] = {SGI81718, SGI81719, SGI81720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81721, SGI0, SGI0, SGI0, SGI81722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18184[] = {SGI81725, SGI0, SGI81726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81727, SGI81728, SGI81728, SGI0, SGI81729, SGI0, NULL};
const Segment * const SG18185[] = {SGI81730, SGI81731, SGI81732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81733, SGI0, SGI0, SGI0, SGI81734, SGI81734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18186[] = {SGI81735, SGI0, SGI81736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18187[] = {SGI81737, SGI0, SGI81738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81739, SGI81740, SGI81740, SGI0, SGI81741, SGI0, NULL};
const Segment * const SG18188[] = {SGI81742, SGI0, SGI81743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81744, SGI81745, SGI81745, SGI0, SGI81746, SGI0, NULL};
const Segment * const SG18189[] = {SGI81723, SGI0, SGI81724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18190[] = {SGI81747, SGI81748, SGI81749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81750, SGI0, SGI0, SGI0, SGI81751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18191[] = {SGI81752, SGI0, SGI81753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18192[] = {SGI81754, SGI81755, SGI81756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81757, SGI0, SGI0, SGI0, SGI81758, SGI81758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18193[] = {SGI81759, SGI0, SGI81760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81761, SGI0, SGI0, SGI0, SGI81762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18194[] = {SGI81763, SGI81764, SGI81765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81766, SGI0, SGI0, SGI0, SGI81767, SGI81767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18195[] = {SGI81768, SGI81769, SGI81770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81771, SGI0, SGI0, SGI0, SGI81772, SGI81772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18196[] = {SGI81773, SGI0, SGI81774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18197[] = {SGI81775, SGI81776, SGI81777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81778, SGI0, SGI0, SGI0, SGI81779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18198[] = {SGI81780, SGI0, SGI81781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18199[] = {SGI81782, SGI81783, SGI81784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81785, SGI0, SGI0, SGI0, SGI81786, SGI81786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18200[] = {SGI81787, SGI81788, SGI81789, SGI0, SGI0, SGI0, SGI81790, SGI81791, SGI0, SGI81792, SGI0, SGI0, SGI0, SGI81793, SGI81793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18201[] = {SGI81794, SGI81795, SGI81796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81797, SGI0, SGI0, SGI0, SGI81798, SGI81798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18202[] = {SGI81799, SGI81800, SGI81801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81802, SGI0, SGI0, SGI0, SGI81803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18203[] = {SGI81804, SGI81805, SGI81806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81807, SGI0, SGI0, SGI0, SGI81808, SGI81808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18204[] = {SGI81809, SGI81810, SGI81811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81812, SGI0, SGI0, SGI0, SGI81813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18205[] = {SGI81814, SGI0, SGI81815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18206[] = {SGI81816, SGI81817, SGI81818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81819, SGI0, SGI0, SGI0, SGI81820, SGI81820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18207[] = {SGI81821, SGI81822, SGI81823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81827, SGI0, SGI0, SGI0, SGI81828, SGI81828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18208[] = {SGI81824, SGI81825, SGI81826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81829, SGI0, SGI0, SGI0, SGI81830, SGI81830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18209[] = {SGI81831, SGI81832, SGI81833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81834, SGI0, SGI0, SGI0, SGI81835, SGI81835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18210[] = {SGI81836, SGI81837, SGI81838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81839, SGI0, SGI0, SGI0, SGI81840, SGI81840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18211[] = {SGI81841, SGI81842, SGI81843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81844, SGI0, SGI0, SGI0, SGI81845, SGI81845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18212[] = {SGI81846, SGI81847, SGI81848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81849, SGI0, SGI0, SGI0, SGI81850, SGI81850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18213[] = {SGI81851, SGI0, SGI81852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18214[] = {SGI81853, SGI81854, SGI81855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81856, SGI0, SGI0, SGI0, SGI81857, SGI81857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18215[] = {SGI81858, SGI0, SGI81859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81860, SGI0, SGI0, SGI0, SGI81861, SGI81861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18216[] = {SGI81862, SGI0, SGI81863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18217[] = {SGI81864, SGI81865, SGI81866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81867, SGI0, SGI0, SGI0, SGI81868, SGI81868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18218[] = {SGI81869, SGI81870, SGI81871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81872, SGI0, SGI0, SGI0, SGI81873, SGI81873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18219[] = {SGI81874, SGI0, SGI81875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18220[] = {SGI81876, SGI81877, SGI81878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81879, SGI0, SGI0, SGI0, SGI81880, SGI81880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18221[] = {SGI81881, SGI81882, SGI81883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81884, SGI0, SGI0, SGI0, SGI81885, SGI81885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18222[] = {SGI81886, SGI81887, SGI81888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81889, SGI0, SGI0, SGI0, SGI81890, SGI81890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18223[] = {SGI81891, SGI0, SGI81892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18224[] = {SGI81893, SGI81894, SGI81895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81896, SGI0, SGI0, SGI0, SGI81897, SGI81897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18225[] = {SGI81899, SGI81900, SGI81901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81902, SGI0, SGI0, SGI0, SGI81903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18226[] = {SGI81904, SGI81905, SGI81906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81907, SGI0, SGI0, SGI0, SGI81908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18227[] = {SGI81909, SGI0, SGI81910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18228[] = {SGI81898, SGI81911, SGI81912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81913, SGI0, SGI0, SGI0, SGI81914, SGI81914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18229[] = {SGI81915, SGI81916, SGI81917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81918, SGI0, SGI0, SGI0, SGI81919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18230[] = {SGI81920, SGI81921, SGI81922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81923, SGI0, SGI0, SGI0, SGI81924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18231[] = {SGI81925, SGI0, SGI81926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18232[] = {SGI81927, SGI81928, SGI81929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81930, SGI0, SGI0, SGI0, SGI81931, SGI81931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18233[] = {SGI81932, SGI81933, SGI81934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81935, SGI0, SGI0, SGI0, SGI81936, SGI81936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18234[] = {SGI81937, SGI81938, SGI81939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81940, SGI0, SGI0, SGI0, SGI81941, SGI81941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18235[] = {SGI81942, SGI81943, SGI81944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81945, SGI0, SGI0, SGI0, SGI81946, SGI81946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18236[] = {SGI81947, SGI81948, SGI81949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81950, SGI0, SGI0, SGI0, SGI81951, SGI81951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18237[] = {SGI81952, SGI81953, SGI81954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81955, SGI81956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18238[] = {SGI81957, SGI81958, SGI81959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81960, SGI81960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18239[] = {SGI81961, SGI0, SGI81962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18240[] = {SGI81963, SGI81964, SGI81965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81966, SGI0, SGI0, SGI0, SGI81967, SGI81967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18241[] = {SGI81968, SGI81969, SGI81970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81971, SGI0, SGI0, SGI0, SGI81972, SGI81972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18242[] = {SGI81973, SGI81974, SGI81975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81976, SGI0, SGI0, SGI0, SGI81977, SGI81977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18243[] = {SGI81978, SGI81979, SGI81980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81981, SGI0, SGI0, SGI0, SGI81982, SGI81982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18244[] = {SGI81983, SGI81984, SGI81985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81986, SGI0, SGI0, SGI0, SGI81987, SGI81987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18245[] = {SGI81988, SGI0, SGI81989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18246[] = {SGI81990, SGI81991, SGI81992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81993, SGI0, SGI0, SGI0, SGI81994, SGI81994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18247[] = {SGI81995, SGI81996, SGI81997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI81998, SGI0, SGI0, SGI0, SGI81999, SGI81999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18248[] = {SGI82000, SGI82001, SGI82002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82003, SGI0, SGI0, SGI0, SGI82004, SGI82004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18249[] = {SGI82005, SGI0, SGI82006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18250[] = {SGI82007, SGI82008, SGI82009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82010, SGI0, SGI0, SGI0, SGI82011, SGI82011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18251[] = {SGI82012, SGI82013, SGI82014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82015, SGI0, SGI0, SGI0, SGI82016, SGI82016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18252[] = {SGI82017, SGI82018, SGI82019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82020, SGI0, SGI0, SGI0, SGI82021, SGI82021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18253[] = {SGI82022, SGI82023, SGI82024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82025, SGI0, SGI0, SGI0, SGI82026, SGI82026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18254[] = {SGI82027, SGI82028, SGI82029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82030, SGI0, SGI0, SGI0, SGI82031, SGI82031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18255[] = {SGI82032, SGI82033, SGI82034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82035, SGI0, SGI0, SGI0, SGI82036, SGI82036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18256[] = {SGI82037, SGI82038, SGI82039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82040, SGI0, SGI0, SGI0, SGI82041, SGI82041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18257[] = {SGI82042, SGI0, SGI82043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18258[] = {SGI82044, SGI82045, SGI82046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82047, SGI0, SGI0, SGI0, SGI82048, SGI82048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18259[] = {SGI0, SGI82049, SGI82050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82051, SGI0, SGI0, SGI0, SGI82052, SGI82052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18260[] = {SGI0, SGI82053, SGI82054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82055, SGI0, SGI0, SGI0, SGI82056, SGI82056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18261[] = {SGI0, SGI82057, SGI82058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82059, SGI0, SGI0, SGI0, SGI82060, SGI82060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18262[] = {SGI0, SGI82061, SGI82062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82063, SGI0, SGI0, SGI0, SGI82064, SGI82064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18263[] = {SGI0, SGI82065, SGI82066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82067, SGI0, SGI0, SGI0, SGI82068, SGI82068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18264[] = {SGI0, SGI82069, SGI82070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82071, SGI0, SGI0, SGI0, SGI82072, SGI82072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18265[] = {SGI0, SGI82073, SGI82074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82078, SGI0, SGI0, SGI0, SGI82079, SGI82079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18266[] = {SGI82075, SGI82076, SGI82077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82080, SGI0, SGI0, SGI0, SGI82081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18267[] = {SGI82082, SGI0, SGI82083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18268[] = {SGI0, SGI82084, SGI82085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82086, SGI0, SGI0, SGI0, SGI82087, SGI82087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18269[] = {SGI82088, SGI82089, SGI82090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82091, SGI0, SGI0, SGI0, SGI82092, SGI82092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18270[] = {SGI0, SGI82093, SGI82094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82095, SGI0, SGI0, SGI0, SGI82096, SGI82096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18271[] = {SGI82097, SGI82098, SGI82099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82100, SGI82100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18272[] = {SGI0, SGI82101, SGI82102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82103, SGI0, SGI0, SGI0, SGI82104, SGI82104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18273[] = {SGI82105, SGI0, SGI82106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82107, SGI0, SGI0, SGI0, SGI82108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18274[] = {SGI82111, SGI82112, SGI82113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18275[] = {SGI0, SGI82109, SGI82110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82114, SGI0, SGI0, SGI0, SGI82115, SGI82115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18276[] = {SGI0, SGI82116, SGI82117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82120, SGI0, SGI0, SGI0, SGI82122, SGI82122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18277[] = {SGI82118, SGI82119, SGI82121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82123, SGI0, SGI0, SGI0, SGI82124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18278[] = {SGI82127, SGI82128, SGI82129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18279[] = {SGI0, SGI82125, SGI82126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82130, SGI0, SGI0, SGI0, SGI82131, SGI82131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18280[] = {SGI82132, SGI82133, SGI82134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82135, SGI0, SGI0, SGI0, SGI82136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18281[] = {SGI82137, SGI82138, SGI82139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82140, SGI0, SGI0, SGI0, SGI82141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18282[] = {SGI82142, SGI82143, SGI82144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18283[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82145, SGI0, SGI0, SGI82146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18284[] = {SGI82147, SGI82148, SGI82149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82150, SGI0, SGI0, SGI0, SGI82151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18285[] = {SGI82152, SGI82153, SGI82154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82155, SGI0, SGI0, SGI0, SGI82156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18286[] = {SGI82157, SGI0, SGI82158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18287[] = {SGI82159, SGI82160, SGI82161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82162, SGI0, SGI0, SGI0, SGI82163, SGI82164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18288[] = {SGI82165, SGI82166, SGI82167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82168, SGI0, SGI0, SGI0, SGI82169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18289[] = {SGI82170, SGI82171, SGI82172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82173, SGI0, SGI0, SGI0, SGI82174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18290[] = {SGI82175, SGI82176, SGI82177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82178, SGI0, SGI0, SGI0, SGI82179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18291[] = {SGI82180, SGI82181, SGI82182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82183, SGI0, SGI0, SGI0, SGI82184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18292[] = {SGI82185, SGI82186, SGI82187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82188, SGI0, SGI0, SGI0, SGI82189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18293[] = {SGI82190, SGI82191, SGI82192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82193, SGI0, SGI0, SGI0, SGI82194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18294[] = {SGI82195, SGI82196, SGI82197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82198, SGI0, SGI0, SGI0, SGI82199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18295[] = {SGI82200, SGI0, SGI82201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18296[] = {SGI82202, SGI82203, SGI82204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82205, SGI0, SGI0, SGI0, SGI82206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18297[] = {SGI82207, SGI82208, SGI82209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82210, SGI0, SGI0, SGI0, SGI82211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18298[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82212, SGI0, SGI0, SGI82213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18299[] = {SGI82214, SGI82215, SGI82216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82217, SGI0, SGI0, SGI0, SGI82218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18300[] = {SGI82219, SGI82220, SGI82221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82222, SGI0, SGI0, SGI0, SGI82223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18301[] = {SGI82224, SGI82225, SGI82226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82227, SGI0, SGI0, SGI0, SGI82228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18302[] = {SGI82229, SGI0, SGI82230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82231, SGI0, SGI0, SGI0, SGI82232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18303[] = {SGI82233, SGI82234, SGI82235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82236, SGI0, SGI0, SGI0, SGI82237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18304[] = {SGI82238, SGI82239, SGI82240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82241, SGI0, SGI0, SGI0, SGI82242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18305[] = {SGI82243, SGI82244, SGI82245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82246, SGI0, SGI0, SGI0, SGI82247, SGI82248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18306[] = {SGI82249, SGI82250, SGI82251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82252, SGI0, SGI0, SGI0, SGI82253, SGI82253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18307[] = {SGI82254, SGI82255, SGI82256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82257, SGI0, SGI0, SGI0, SGI82258, SGI82258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18308[] = {SGI82259, SGI82260, SGI82261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82262, SGI0, SGI0, SGI0, SGI82263, SGI82263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18309[] = {SGI82264, SGI0, SGI82265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82266, SGI0, SGI0, SGI0, SGI82267, SGI82267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18310[] = {SGI82268, SGI0, SGI82269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18311[] = {SGI82270, SGI82271, SGI82272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82273, SGI0, SGI0, SGI0, SGI82274, SGI82274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18312[] = {SGI82275, SGI82276, SGI82277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82278, SGI0, SGI0, SGI0, SGI82279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18313[] = {SGI82280, SGI82281, SGI82282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82283, SGI0, SGI0, SGI0, SGI82284, SGI82284, SGI82285, SGI0, SGI82286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18314[] = {SGI82287, SGI82288, SGI82289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82290, SGI0, SGI0, SGI0, SGI82291, SGI82291, SGI82292, SGI0, SGI82293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18315[] = {SGI82294, SGI82295, SGI82296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82297, SGI0, SGI0, SGI0, SGI82298, SGI82298, SGI82299, SGI0, SGI82300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18316[] = {SGI0, SGI82301, SGI0, SGI82302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18317[] = {SGI0, SGI82303, SGI0, SGI82304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82305, SGI0, SGI0, SGI0, SGI82306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18318[] = {SGI82307, SGI82308, SGI82309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82310, SGI0, SGI0, SGI0, SGI82311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18319[] = {SGI82312, SGI82313, SGI82314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82315, SGI0, SGI0, SGI0, SGI82316, SGI82316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18320[] = {SGI82317, SGI0, SGI82318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18321[] = {SGI82319, SGI82320, SGI82321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82322, SGI0, SGI0, SGI0, SGI82323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18322[] = {SGI82324, SGI82325, SGI82326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82327, SGI0, SGI0, SGI0, SGI82328, SGI82328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18323[] = {SGI82329, SGI82330, SGI82331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82332, SGI0, SGI0, SGI0, SGI82333, SGI82333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18324[] = {SGI82334, SGI82335, SGI82336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82337, SGI0, SGI0, SGI0, SGI82338, SGI82338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18325[] = {SGI82339, SGI82340, SGI82341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82342, SGI0, SGI0, SGI0, SGI82343, SGI82343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18326[] = {SGI82344, SGI82345, SGI82346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82347, SGI0, SGI0, SGI0, SGI82348, SGI82348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18327[] = {SGI82349, SGI82350, SGI82351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82352, SGI0, SGI0, SGI0, SGI82353, SGI82353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18328[] = {SGI82354, SGI0, SGI82355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18329[] = {SGI82356, SGI82357, SGI82358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82359, SGI0, SGI0, SGI0, SGI82360, SGI82360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18330[] = {SGI82361, SGI0, SGI82362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18331[] = {SGI82363, SGI82364, SGI82365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82366, SGI0, SGI0, SGI0, SGI82367, SGI82367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18332[] = {SGI82368, SGI82369, SGI82370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82371, SGI82371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18333[] = {SGI82372, SGI82373, SGI82374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82375, SGI0, SGI0, SGI0, SGI82376, SGI82376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18334[] = {SGI82377, SGI82378, SGI82379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82380, SGI0, SGI0, SGI0, SGI82381, SGI82381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18335[] = {SGI82382, SGI82383, SGI82384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82385, SGI0, SGI0, SGI0, SGI82386, SGI82386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18336[] = {SGI82387, SGI82388, SGI82389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82390, SGI0, SGI0, SGI0, SGI82391, SGI82391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18337[] = {SGI82392, SGI0, SGI82393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18338[] = {SGI82394, SGI82395, SGI82396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82397, SGI0, SGI0, SGI0, SGI82398, SGI82398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18339[] = {SGI82399, SGI82400, SGI82401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82402, SGI0, SGI0, SGI0, SGI82403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18340[] = {SGI82404, SGI82405, SGI82406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82407, SGI0, SGI0, SGI0, SGI82408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18341[] = {SGI82409, SGI82410, SGI82411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82412, SGI0, SGI0, SGI0, SGI82413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18342[] = {SGI82414, SGI82415, SGI82416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82417, SGI0, SGI0, SGI0, SGI82418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18343[] = {SGI82419, SGI82420, SGI82421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82422, SGI0, SGI0, SGI0, SGI82423, SGI82423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18344[] = {SGI82424, SGI0, SGI82425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18345[] = {SGI82426, SGI82427, SGI82428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82429, SGI82430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18346[] = {SGI82431, SGI82432, SGI82433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82434, SGI82434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18347[] = {SGI82435, SGI82436, SGI82437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82438, SGI0, SGI0, SGI0, SGI82439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18348[] = {SGI82440, SGI82441, SGI82442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82443, SGI0, SGI0, SGI0, SGI82444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18349[] = {SGI82445, SGI82446, SGI82447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82448, SGI0, SGI0, SGI0, SGI82449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18350[] = {SGI82450, SGI82451, SGI82452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82453, SGI0, SGI0, SGI0, SGI82454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18351[] = {SGI82455, SGI0, SGI82456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18352[] = {SGI82457, SGI82458, SGI82459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82460, SGI0, SGI0, SGI0, SGI82461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18353[] = {SGI82462, SGI0, SGI82463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82464, SGI82465, SGI82465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18354[] = {SGI82466, SGI0, SGI82467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82468, SGI82469, SGI82469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18355[] = {SGI82470, SGI0, SGI82471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82472, SGI82473, SGI82473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18356[] = {SGI82474, SGI0, SGI82475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82476, SGI82477, SGI82477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18357[] = {SGI82478, SGI0, SGI82479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82480, SGI82481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18358[] = {SGI82482, SGI0, SGI82483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18359[] = {SGI82489, SGI0, SGI82490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82491, SGI82492, SGI82492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18360[] = {SGI82493, SGI0, SGI82494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82495, SGI82496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18361[] = {SGI82497, SGI0, SGI82498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82499, SGI82500, SGI82500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18362[] = {SGI82501, SGI0, SGI82502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82503, SGI82504, SGI82504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18363[] = {SGI82505, SGI0, SGI82506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82507, SGI82508, SGI82508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18364[] = {SGI82484, SGI82485, SGI82486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82487, SGI0, SGI0, SGI0, SGI82488, SGI82488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18365[] = {SGI82509, SGI0, SGI82510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82511, SGI82512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18366[] = {SGI82513, SGI82514, SGI82515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82516, SGI0, SGI0, SGI0, SGI82517, SGI82517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18367[] = {SGI82518, SGI82519, SGI82520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82521, SGI0, SGI0, SGI0, SGI82522, SGI82522, SGI82523, SGI0, SGI82524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18368[] = {SGI82525, SGI82526, SGI82527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82528, SGI0, SGI0, SGI0, SGI82529, SGI82529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82530, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18369[] = {SGI82531, SGI0, SGI82532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82533, SGI82534, SGI82534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18370[] = {SGI82535, SGI0, SGI82536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82537, SGI82538, SGI82538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18371[] = {SGI82539, SGI0, SGI82540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82541, SGI82542, SGI82542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18372[] = {SGI82543, SGI0, SGI82544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82545, SGI82546, SGI82546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18373[] = {SGI82547, SGI0, SGI82548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82549, SGI82550, SGI82550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18374[] = {SGI82551, SGI0, SGI82552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82553, SGI82554, SGI82554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18375[] = {SGI82555, SGI82556, SGI82557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82558, SGI0, SGI0, SGI0, SGI82559, SGI82559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18376[] = {SGI82560, SGI0, SGI82561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82562, SGI0, SGI0, SGI0, SGI82563, SGI82563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18377[] = {SGI82564, SGI82565, SGI82566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82567, SGI0, SGI0, SGI0, SGI82568, SGI82568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18378[] = {SGI82569, SGI82570, SGI82571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82572, SGI0, SGI0, SGI0, SGI82573, SGI82573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18379[] = {SGI82574, SGI82575, SGI82576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82577, SGI0, SGI0, SGI0, SGI82578, SGI82578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18380[] = {SGI82579, SGI0, SGI82580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82581, SGI82582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18381[] = {SGI82583, SGI0, SGI82584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82585, SGI82586, SGI82586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18382[] = {SGI82587, SGI0, SGI82588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82589, SGI82590, SGI82590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18383[] = {SGI82591, SGI0, SGI82592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82593, SGI82594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18384[] = {SGI82595, SGI0, SGI82596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82597, SGI82598, SGI82598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18385[] = {SGI82599, SGI0, SGI82600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82601, SGI82602, SGI82602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18386[] = {SGI82603, SGI0, SGI82604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82605, SGI82606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18387[] = {SGI82607, SGI0, SGI82608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82609, SGI82610, SGI82610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18388[] = {SGI82611, SGI0, SGI82612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82613, SGI82614, SGI82614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18389[] = {SGI82615, SGI0, SGI82616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82617, SGI82618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18390[] = {SGI82619, SGI0, SGI82620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82621, SGI82622, SGI82622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18391[] = {SGI82623, SGI0, SGI82624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82625, SGI82626, SGI82626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18392[] = {SGI82627, SGI0, SGI82628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82629, SGI82630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18393[] = {SGI82631, SGI0, SGI82632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82633, SGI82634, SGI82634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18394[] = {SGI82635, SGI0, SGI82636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82637, SGI82638, SGI82638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18395[] = {SGI82639, SGI0, SGI82640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82641, SGI82642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18396[] = {SGI82643, SGI0, SGI82644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82645, SGI82646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18397[] = {SGI82647, SGI0, SGI82648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82649, SGI82650, SGI82650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18398[] = {SGI82651, SGI0, SGI82652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82653, SGI82654, SGI82654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18399[] = {SGI82655, SGI0, SGI82656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82657, SGI82658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18400[] = {SGI82659, SGI0, SGI82660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82661, SGI82662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18401[] = {SGI82663, SGI0, SGI82664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82665, SGI82666, SGI82666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18402[] = {SGI82667, SGI82669, SGI82670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82672, SGI0, SGI0, SGI0, SGI82674, SGI82674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18403[] = {SGI82668, SGI0, SGI82671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82673, SGI82675, SGI82675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18404[] = {SGI82677, SGI0, SGI82679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82682, SGI82683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18405[] = {SGI82676, SGI82678, SGI82680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82681, SGI0, SGI0, SGI0, SGI82684, SGI82684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18406[] = {SGI82685, SGI0, SGI82686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82687, SGI82688, SGI82688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18407[] = {SGI82689, SGI0, SGI82690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82691, SGI82692, SGI82692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18408[] = {SGI82693, SGI82694, SGI82696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82698, SGI0, SGI0, SGI0, SGI82700, SGI82700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18409[] = {SGI82695, SGI0, SGI82697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82699, SGI82701, SGI82701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18410[] = {SGI82702, SGI0, SGI82704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18411[] = {SGI82703, SGI0, SGI82705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82706, SGI82707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18412[] = {SGI82708, SGI0, SGI82709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82710, SGI82711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18413[] = {SGI82712, SGI82713, SGI82714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82715, SGI0, SGI0, SGI0, SGI82716, SGI82716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18414[] = {SGI82717, SGI0, SGI82718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82719, SGI82720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18415[] = {SGI82721, SGI0, SGI82722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82723, SGI82724, SGI82724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18416[] = {SGI82725, SGI0, SGI82726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82727, SGI82728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18417[] = {SGI82729, SGI0, SGI82730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82731, SGI82732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18418[] = {SGI82733, SGI0, SGI82734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82735, SGI82736, SGI82736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18419[] = {SGI82737, SGI0, SGI82738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82739, SGI82740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18420[] = {SGI82741, SGI0, SGI82742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82743, SGI82744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18421[] = {SGI82752, SGI0, SGI82753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82754, SGI82755, SGI82755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18422[] = {SGI82756, SGI0, SGI82757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82758, SGI82759, SGI82759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18423[] = {SGI82760, SGI0, SGI82761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82762, SGI82763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18424[] = {SGI0, SGI0, SGI82764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82765, SGI82766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18425[] = {SGI82767, SGI0, SGI82768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82769, SGI82770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18426[] = {SGI82771, SGI0, SGI82772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82773, SGI82774, SGI82774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18427[] = {SGI82745, SGI82746, SGI82747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82748, SGI0, SGI0, SGI0, SGI82749, SGI82749, SGI82750, SGI0, SGI82751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18428[] = {SGI82777, SGI82778, SGI82779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82780, SGI0, SGI0, SGI0, SGI82781, SGI82781, SGI82782, SGI0, SGI82783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18429[] = {SGI82784, SGI82785, SGI82786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82787, SGI0, SGI0, SGI0, SGI82788, SGI82788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82789, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18430[] = {SGI82775, SGI0, SGI82776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82790, SGI82791, SGI82791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18431[] = {SGI82792, SGI0, SGI82793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82794, SGI82795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18432[] = {SGI82796, SGI82797, SGI82798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82799, SGI0, SGI0, SGI0, SGI82800, SGI82800, SGI82801, SGI0, SGI82802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18433[] = {SGI82803, SGI82804, SGI82805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82806, SGI0, SGI0, SGI0, SGI82807, SGI82807, SGI82808, SGI0, SGI82809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18434[] = {SGI82810, SGI82811, SGI82812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82813, SGI0, SGI0, SGI0, SGI82814, SGI82814, SGI82815, SGI0, SGI82816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18435[] = {SGI82817, SGI82818, SGI82819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82820, SGI0, SGI0, SGI0, SGI82821, SGI82821, SGI82822, SGI0, SGI82823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18436[] = {SGI82824, SGI82825, SGI82826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82827, SGI0, SGI0, SGI0, SGI82828, SGI82828, SGI82829, SGI0, SGI82830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18437[] = {SGI82831, SGI82832, SGI82833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82834, SGI0, SGI0, SGI0, SGI82835, SGI82835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18438[] = {SGI82836, SGI82837, SGI82838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82839, SGI0, SGI0, SGI0, SGI82840, SGI82840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18439[] = {SGI82841, SGI0, SGI82842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18440[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82843, SGI0, SGI82844, SGI0, NULL};
const Segment * const SG18441[] = {SGI82845, SGI82846, SGI82847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82848, SGI0, SGI0, SGI0, SGI82849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18442[] = {SGI82850, SGI0, SGI82851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18443[] = {SGI82852, SGI82853, SGI82854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82855, SGI0, SGI0, SGI0, SGI82856, SGI82856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18444[] = {SGI82860, SGI0, SGI82861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82862, SGI82863, SGI82863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18445[] = {SGI82864, SGI0, SGI82865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82866, SGI82867, SGI82867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18446[] = {SGI82857, SGI82858, SGI82859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82868, SGI0, SGI0, SGI0, SGI82869, SGI82869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18447[] = {SGI82870, SGI0, SGI82871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82872, SGI82873, SGI82873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18448[] = {SGI82874, SGI82875, SGI82876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82877, SGI0, SGI0, SGI0, SGI82878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18449[] = {SGI82879, SGI82880, SGI82881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82882, SGI0, SGI0, SGI0, SGI82883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18450[] = {SGI82884, SGI82885, SGI82886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82887, SGI0, SGI0, SGI0, SGI82888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18451[] = {SGI82889, SGI82890, SGI82891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82892, SGI0, SGI0, SGI0, SGI82893, SGI82893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82894, SGI0, SGI0, NULL};
const Segment * const SG18452[] = {SGI82895, SGI82896, SGI82897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82898, SGI0, SGI0, SGI0, SGI82899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18453[] = {SGI82900, SGI82901, SGI82902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82903, SGI0, SGI0, SGI0, SGI82904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18454[] = {SGI82905, SGI82906, SGI82907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82908, SGI0, SGI0, SGI0, SGI82909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18455[] = {SGI82910, SGI0, SGI82911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18456[] = {SGI82912, SGI82913, SGI82914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82915, SGI0, SGI0, SGI0, SGI82916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18457[] = {SGI82917, SGI0, SGI82918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82919, SGI82920, SGI82920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18458[] = {SGI82921, SGI0, SGI82922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82923, SGI82924, SGI82924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18459[] = {SGI82925, SGI0, SGI82926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82927, SGI82928, SGI82928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18460[] = {SGI82929, SGI0, SGI82930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82931, SGI82932, SGI82932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18461[] = {SGI82933, SGI82934, SGI82935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82936, SGI0, SGI0, SGI0, SGI82937, SGI82937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18462[] = {SGI82938, SGI0, SGI82939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18463[] = {SGI82940, SGI0, SGI82941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82942, SGI82943, SGI82943, SGI0, SGI0, NULL};
const Segment * const SG18464[] = {SGI82944, SGI0, SGI82945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82946, SGI82947, SGI82947, SGI0, SGI0, NULL};
const Segment * const SG18465[] = {SGI82948, SGI0, SGI82949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82950, SGI82951, SGI82951, SGI0, SGI0, NULL};
const Segment * const SG18466[] = {SGI82956, SGI82957, SGI82958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82959, SGI0, SGI0, SGI0, SGI82960, SGI82960, SGI0, SGI82961, SGI0, NULL};
const Segment * const SG18467[] = {SGI82962, SGI0, SGI82963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82964, SGI0, NULL};
const Segment * const SG18468[] = {SGI82965, SGI82966, SGI82967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82968, SGI0, SGI0, SGI0, SGI82969, SGI82969, SGI0, SGI82970, SGI0, NULL};
const Segment * const SG18469[] = {SGI82971, SGI0, SGI82972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82973, SGI0, SGI0, SGI0, SGI82974, SGI82974, SGI0, SGI82975, SGI0, NULL};
const Segment * const SG18470[] = {SGI82976, SGI82977, SGI82978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82979, SGI0, SGI0, SGI0, SGI82980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18471[] = {SGI82981, SGI0, SGI82982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18472[] = {SGI82952, SGI82953, SGI82954, SGI0, SGI0, SGI0, SGI82955, SGI82983, SGI0, SGI82984, SGI0, SGI0, SGI0, SGI82985, SGI82985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18473[] = {SGI82986, SGI82987, SGI82988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82989, SGI0, SGI0, SGI0, SGI82990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18474[] = {SGI82991, SGI0, SGI82992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18475[] = {SGI82993, SGI82994, SGI82995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI82996, SGI0, SGI0, SGI0, SGI82997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18476[] = {SGI82998, SGI82999, SGI83000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83001, SGI0, SGI0, SGI0, SGI83002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18477[] = {SGI83003, SGI83004, SGI83005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83006, SGI0, SGI0, SGI0, SGI83007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18478[] = {SGI83008, SGI0, SGI83009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18479[] = {SGI83010, SGI0, SGI83011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18480[] = {SGI83012, SGI83013, SGI83014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83015, SGI0, SGI0, SGI0, SGI83016, SGI83016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18481[] = {SGI83017, SGI83018, SGI83019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83020, SGI0, SGI0, SGI0, SGI83021, SGI83021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18482[] = {SGI83022, SGI83023, SGI83024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83025, SGI0, SGI0, SGI0, SGI83026, SGI83026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18483[] = {SGI83027, SGI83028, SGI83029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83030, SGI0, SGI0, SGI0, SGI83031, SGI83031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18484[] = {SGI83032, SGI83033, SGI83034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83035, SGI0, SGI0, SGI0, SGI83036, SGI83036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18485[] = {SGI83037, SGI83038, SGI83039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83040, SGI0, SGI0, SGI0, SGI83041, SGI83041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18486[] = {SGI83042, SGI83043, SGI83044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83045, SGI0, SGI0, SGI0, SGI83046, SGI83046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18487[] = {SGI83047, SGI0, SGI83048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18488[] = {SGI83049, SGI0, SGI83050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83051, SGI0, SGI0, SGI0, SGI83052, SGI83052, SGI0, SGI0, SGI83053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18489[] = {SGI83054, SGI83055, SGI83056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83057, SGI0, SGI0, SGI0, SGI83058, SGI83059, SGI0, SGI0, SGI83060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18490[] = {SGI83061, SGI83062, SGI83063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83064, SGI0, SGI0, SGI0, SGI83065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18491[] = {SGI83066, SGI0, SGI83067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18492[] = {SGI83068, SGI83069, SGI83070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83071, SGI0, SGI0, SGI0, SGI83072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18493[] = {SGI83073, SGI83074, SGI83075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83076, SGI0, SGI0, SGI0, SGI83077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18494[] = {SGI83078, SGI83079, SGI83080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83081, SGI0, SGI0, SGI0, SGI83082, SGI83082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18495[] = {SGI83083, SGI83084, SGI83085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83086, SGI0, SGI0, SGI0, SGI83087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18496[] = {SGI83088, SGI0, SGI83089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18497[] = {SGI83090, SGI83091, SGI83092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83093, SGI0, SGI0, SGI0, SGI83094, SGI83094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18498[] = {SGI83095, SGI83096, SGI83097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83098, SGI0, SGI0, SGI0, SGI83099, SGI83099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18499[] = {SGI83100, SGI83101, SGI83102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83103, SGI83104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18500[] = {SGI83105, SGI83106, SGI83107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83108, SGI83108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18501[] = {SGI83109, SGI83110, SGI83111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83112, SGI0, SGI0, SGI0, SGI83113, SGI83113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18502[] = {SGI83114, SGI0, SGI83115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18503[] = {SGI83116, SGI0, SGI83117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18504[] = {SGI83118, SGI0, SGI83119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83120, SGI0, SGI0, SGI0, SGI83121, SGI83121, SGI0, SGI0, SGI83122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18505[] = {SGI83123, SGI83124, SGI83125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83126, SGI0, SGI0, SGI0, SGI83127, SGI83127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18506[] = {SGI83128, SGI0, SGI83129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18507[] = {SGI83130, SGI83131, SGI83132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83133, SGI0, SGI0, SGI0, SGI83134, SGI83134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18508[] = {SGI83135, SGI83136, SGI83137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83138, SGI0, SGI0, SGI0, SGI83139, SGI83139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18509[] = {SGI83140, SGI83141, SGI83142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83143, SGI83143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18510[] = {SGI83144, SGI83145, SGI83146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83147, SGI0, SGI0, SGI0, SGI83148, SGI83148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18511[] = {SGI83149, SGI0, SGI83150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18512[] = {SGI83151, SGI83152, SGI83153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83154, SGI0, SGI0, SGI0, SGI83155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18513[] = {SGI83156, SGI0, SGI83157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18514[] = {SGI83158, SGI83159, SGI83160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83161, SGI0, SGI0, SGI0, SGI83162, SGI83162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18515[] = {SGI83163, SGI83164, SGI83165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83166, SGI0, SGI0, SGI0, SGI83167, SGI83167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18516[] = {SGI83168, SGI0, SGI83169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18517[] = {SGI83170, SGI83171, SGI83172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83173, SGI0, SGI0, SGI0, SGI83174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18518[] = {SGI83175, SGI83176, SGI83177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83178, SGI0, SGI0, SGI0, SGI83179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18519[] = {SGI83180, SGI83181, SGI83182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83183, SGI0, SGI0, SGI0, SGI83184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18520[] = {SGI83185, SGI83186, SGI83187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83188, SGI0, SGI0, SGI0, SGI83189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18521[] = {SGI83190, SGI83191, SGI83192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83193, SGI0, SGI0, SGI0, SGI83194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18522[] = {SGI83195, SGI83196, SGI83197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83198, SGI0, SGI0, SGI0, SGI83199, SGI83199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18523[] = {SGI83200, SGI0, SGI83201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18524[] = {SGI83202, SGI83203, SGI83204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83205, SGI0, SGI0, SGI0, SGI83206, SGI83206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18525[] = {SGI83207, SGI83208, SGI83209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83210, SGI0, SGI0, SGI0, SGI83211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18526[] = {SGI83212, SGI0, SGI83213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18527[] = {SGI83214, SGI83215, SGI83216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83217, SGI0, SGI0, SGI0, SGI83218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18528[] = {SGI83219, SGI0, SGI83220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18529[] = {SGI83221, SGI83222, SGI83223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83224, SGI0, SGI0, SGI0, SGI83225, SGI83225, SGI83226, SGI0, SGI83227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18530[] = {SGI83228, SGI83229, SGI83230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83231, SGI0, SGI0, SGI0, SGI83232, SGI83232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83233, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18531[] = {SGI83234, SGI83235, SGI83236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83237, SGI0, SGI0, SGI0, SGI83238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18532[] = {SGI83239, SGI0, SGI83240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18533[] = {SGI83241, SGI83242, SGI83243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83244, SGI0, SGI0, SGI0, SGI83245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18534[] = {SGI83246, SGI83247, SGI83248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83249, SGI0, SGI0, SGI0, SGI83250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18535[] = {SGI83251, SGI83252, SGI83253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83254, SGI0, SGI0, SGI0, SGI83255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18536[] = {SGI83256, SGI83257, SGI83258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83259, SGI0, SGI0, SGI0, SGI83260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18537[] = {SGI83266, SGI83267, SGI83268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83269, SGI0, SGI0, SGI0, SGI83270, SGI83270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18538[] = {SGI83271, SGI0, SGI83272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83273, SGI0, SGI0, SGI0, SGI83274, SGI83274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18539[] = {SGI83275, SGI0, SGI83276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18540[] = {SGI83261, SGI83262, SGI83263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83264, SGI0, SGI0, SGI0, SGI83265, SGI83265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18541[] = {SGI83277, SGI83278, SGI83279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83280, SGI0, SGI0, SGI0, SGI83281, SGI83281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18542[] = {SGI83282, SGI83283, SGI83284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83285, SGI0, SGI0, SGI0, SGI83286, SGI83286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18543[] = {SGI83287, SGI0, SGI83288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18544[] = {SGI83289, SGI83290, SGI83291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83292, SGI0, SGI0, SGI0, SGI83293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18545[] = {SGI83294, SGI83295, SGI83296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83297, SGI0, SGI0, SGI0, SGI83298, SGI83298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18546[] = {SGI83299, SGI0, SGI83300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18547[] = {SGI83301, SGI83302, SGI83303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83304, SGI0, SGI0, SGI0, SGI83305, SGI83305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18548[] = {SGI83306, SGI83307, SGI83308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83309, SGI0, SGI0, SGI0, SGI83310, SGI83310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18549[] = {SGI83311, SGI83312, SGI83313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83314, SGI0, SGI0, SGI0, SGI83315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18550[] = {SGI83316, SGI83317, SGI83318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83319, SGI0, SGI0, SGI0, SGI83320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18551[] = {SGI83321, SGI83322, SGI83323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83324, SGI0, SGI0, SGI0, SGI83325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18552[] = {SGI83326, SGI0, SGI83327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18553[] = {SGI83328, SGI0, SGI83329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83330, SGI0, SGI0, SGI0, SGI83332, SGI83332, SGI0, SGI0, SGI83335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18554[] = {SGI83331, SGI83333, SGI83334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83336, SGI0, SGI0, SGI0, SGI83337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18555[] = {SGI83338, SGI83339, SGI83340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83341, SGI0, SGI0, SGI0, SGI83342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18556[] = {SGI83343, SGI83344, SGI83345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83346, SGI0, SGI0, SGI0, SGI83347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18557[] = {SGI83348, SGI83349, SGI83350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83351, SGI0, SGI0, SGI0, SGI83352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18558[] = {SGI83353, SGI83354, SGI83355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83356, SGI0, SGI0, SGI0, SGI83357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18559[] = {SGI83358, SGI83359, SGI83360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83361, SGI0, SGI0, SGI0, SGI83362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18560[] = {SGI83363, SGI83364, SGI83365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83366, SGI0, SGI0, SGI0, SGI83367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18561[] = {SGI83368, SGI83369, SGI83370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83371, SGI0, SGI0, SGI0, SGI83372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18562[] = {SGI83373, SGI83374, SGI83375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83376, SGI0, SGI0, SGI0, SGI83377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18563[] = {SGI83378, SGI0, SGI83379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18564[] = {SGI83380, SGI83381, SGI83382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83383, SGI0, SGI0, SGI0, SGI83384, SGI83384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18565[] = {SGI83385, SGI83386, SGI83387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83388, SGI83388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18566[] = {SGI83389, SGI0, SGI83390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18567[] = {SGI83391, SGI83392, SGI83393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83394, SGI0, SGI0, SGI0, SGI83395, SGI83395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18568[] = {SGI83396, SGI83397, SGI83398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83399, SGI0, SGI0, SGI0, SGI83400, SGI83400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18569[] = {SGI83401, SGI83402, SGI83403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83404, SGI0, SGI0, SGI0, SGI83405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18570[] = {SGI83406, SGI0, SGI83407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18571[] = {SGI83409, SGI83410, SGI83411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83412, SGI0, SGI0, SGI0, SGI83413, SGI83413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18572[] = {SGI83414, SGI0, SGI83415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18573[] = {SGI83408, SGI83416, SGI83417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83418, SGI0, SGI0, SGI0, SGI83419, SGI83419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18574[] = {SGI83420, SGI0, SGI83421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18575[] = {SGI83422, SGI0, SGI83423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83424, SGI83425, SGI83425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18576[] = {SGI83426, SGI83427, SGI83428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83429, SGI0, SGI0, SGI0, SGI83430, SGI83430, SGI0, SGI83431, SGI0, NULL};
const Segment * const SG18577[] = {SGI83432, SGI0, SGI83433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83434, SGI83435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18578[] = {SGI83436, SGI0, SGI83437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83438, SGI83439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18579[] = {SGI83440, SGI83441, SGI83442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83443, SGI0, SGI0, SGI0, SGI83444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18580[] = {SGI83445, SGI83446, SGI83447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83448, SGI0, SGI0, SGI0, SGI83449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18581[] = {SGI83450, SGI83451, SGI83452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83453, SGI0, SGI0, SGI0, SGI83454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18582[] = {SGI83455, SGI83456, SGI83457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83458, SGI0, SGI0, SGI0, SGI83459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18583[] = {SGI83460, SGI83461, SGI83462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83463, SGI0, SGI0, SGI0, SGI83464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18584[] = {SGI83465, SGI83466, SGI83467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83468, SGI0, SGI0, SGI0, SGI83469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18585[] = {SGI83470, SGI83471, SGI83472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83473, SGI0, SGI0, SGI0, SGI83474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18586[] = {SGI83475, SGI83476, SGI83477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83478, SGI0, SGI0, SGI0, SGI83479, SGI83479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18587[] = {SGI83480, SGI83481, SGI83482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83483, SGI0, SGI0, SGI0, SGI83484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18588[] = {SGI83485, SGI83486, SGI83487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83488, SGI0, SGI0, SGI0, SGI83489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18589[] = {SGI83490, SGI83491, SGI83492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83493, SGI0, SGI0, SGI0, SGI83494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18590[] = {SGI83495, SGI83496, SGI83497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83498, SGI0, SGI0, SGI0, SGI83499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18591[] = {SGI83500, SGI83501, SGI83502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83503, SGI0, SGI0, SGI0, SGI83504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18592[] = {SGI83508, SGI83509, SGI83510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83511, SGI0, SGI0, SGI0, SGI83512, SGI83512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18593[] = {SGI83513, SGI0, SGI83514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18594[] = {SGI83515, SGI83516, SGI83517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83518, SGI0, SGI0, SGI0, SGI83519, SGI83519, SGI83520, SGI0, SGI83521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18595[] = {SGI83522, SGI83523, SGI83524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83525, SGI0, SGI0, SGI0, SGI83526, SGI83526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83527, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18596[] = {SGI83505, SGI83506, SGI83507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83528, SGI0, SGI0, SGI0, SGI83529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18597[] = {SGI83530, SGI83531, SGI83532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83533, SGI0, SGI0, SGI0, SGI83534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18598[] = {SGI83535, SGI83536, SGI83537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83538, SGI0, SGI0, SGI0, SGI83539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18599[] = {SGI83540, SGI83541, SGI83542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83543, SGI0, SGI0, SGI0, SGI83544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18600[] = {SGI83545, SGI83546, SGI83547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83548, SGI0, SGI0, SGI0, SGI83549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18601[] = {SGI83550, SGI83551, SGI83552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83553, SGI0, SGI0, SGI0, SGI83554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18602[] = {SGI83555, SGI83556, SGI83557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83558, SGI0, SGI0, SGI0, SGI83559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18603[] = {SGI83560, SGI83561, SGI83562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83563, SGI0, SGI0, SGI0, SGI83564, SGI83564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18604[] = {SGI83565, SGI83566, SGI83567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83568, SGI0, SGI0, SGI0, SGI83569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18605[] = {SGI83570, SGI83571, SGI83572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83573, SGI0, SGI0, SGI0, SGI83574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18606[] = {SGI83575, SGI83576, SGI83577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83578, SGI0, SGI0, SGI0, SGI83579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18607[] = {SGI83580, SGI83581, SGI83582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83583, SGI0, SGI0, SGI0, SGI83584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18608[] = {SGI83585, SGI83586, SGI83587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83588, SGI0, SGI0, SGI0, SGI83589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18609[] = {SGI83590, SGI83591, SGI83592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83593, SGI0, SGI0, SGI0, SGI83594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18610[] = {SGI83595, SGI83596, SGI83597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83598, SGI0, SGI0, SGI0, SGI83599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18611[] = {SGI83600, SGI83601, SGI83602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83603, SGI0, SGI0, SGI0, SGI83604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18612[] = {SGI83605, SGI83606, SGI83607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83608, SGI0, SGI0, SGI0, SGI83609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18613[] = {SGI83610, SGI83611, SGI83612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83613, SGI0, SGI0, SGI0, SGI83614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18614[] = {SGI83615, SGI83616, SGI83617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83618, SGI0, SGI0, SGI0, SGI83619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18615[] = {SGI83620, SGI83621, SGI83622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83623, SGI0, SGI0, SGI0, SGI83624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18616[] = {SGI83625, SGI83626, SGI83627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83628, SGI0, SGI0, SGI0, SGI83629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18617[] = {SGI83630, SGI83631, SGI83632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83633, SGI0, SGI0, SGI0, SGI83634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18618[] = {SGI83635, SGI0, SGI83636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83637, SGI83638, SGI83638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18619[] = {SGI0, SGI83639, SGI83640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83641, SGI0, SGI0, SGI0, SGI83642, SGI83642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18620[] = {SGI83643, SGI83644, SGI83645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83648, SGI83648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18621[] = {SGI83646, SGI83647, SGI83649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83650, SGI0, SGI0, SGI0, SGI83651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18622[] = {SGI83652, SGI83653, SGI83654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83655, SGI0, SGI0, SGI0, SGI83656, SGI83656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18623[] = {SGI83657, SGI83658, SGI83659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83660, SGI0, SGI0, SGI0, SGI83661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18624[] = {SGI83662, SGI0, SGI83663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18625[] = {SGI83664, SGI83665, SGI83666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83667, SGI0, SGI0, SGI0, SGI83668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18626[] = {SGI83669, SGI83670, SGI83671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83672, SGI0, SGI0, SGI0, SGI83673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18627[] = {SGI83674, SGI83675, SGI83676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83677, SGI0, SGI0, SGI0, SGI83678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18628[] = {SGI83679, SGI0, SGI83680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18629[] = {SGI83681, SGI0, SGI83682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18630[] = {SGI83683, SGI83684, SGI83685, SGI0, SGI0, SGI0, SGI83686, SGI83687, SGI0, SGI83688, SGI0, SGI0, SGI0, SGI83689, SGI83689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18631[] = {SGI83690, SGI83691, SGI83692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83693, SGI0, SGI0, SGI0, SGI83694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18632[] = {SGI83695, SGI0, SGI83696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18633[] = {SGI83697, SGI83698, SGI83699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83700, SGI0, SGI0, SGI0, SGI83701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18634[] = {SGI83702, SGI83703, SGI83704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83705, SGI0, SGI0, SGI0, SGI83706, SGI83706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18635[] = {SGI83707, SGI0, SGI83708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83709, SGI0, SGI0, SGI0, SGI83710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18636[] = {SGI83711, SGI83712, SGI83713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83714, SGI0, SGI0, SGI0, SGI83715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18637[] = {SGI83716, SGI83717, SGI83718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83719, SGI0, SGI0, SGI0, SGI83720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18638[] = {SGI83721, SGI0, SGI83722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18639[] = {SGI83723, SGI83724, SGI83725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83726, SGI0, SGI0, SGI0, SGI83727, SGI83727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18640[] = {SGI83728, SGI0, SGI83729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18641[] = {SGI83730, SGI83731, SGI83732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83733, SGI0, SGI0, SGI0, SGI83734, SGI83734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18642[] = {SGI83735, SGI83736, SGI83737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83738, SGI0, SGI0, SGI0, SGI83739, SGI83739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18643[] = {SGI83740, SGI83741, SGI83742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83743, SGI83744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18644[] = {SGI83745, SGI83746, SGI83747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83748, SGI0, SGI0, SGI0, SGI83749, SGI83749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18645[] = {SGI83750, SGI83751, SGI83752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83753, SGI83753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18646[] = {SGI83754, SGI83755, SGI83756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83757, SGI0, SGI0, SGI0, SGI83758, SGI83758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18647[] = {SGI83759, SGI83760, SGI83761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83762, SGI0, SGI0, SGI0, SGI83763, SGI83763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18648[] = {SGI83764, SGI0, SGI83765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18649[] = {SGI83766, SGI83767, SGI83768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83769, SGI0, SGI0, SGI0, SGI83770, SGI83770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18650[] = {SGI83771, SGI83772, SGI83773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83774, SGI0, SGI0, SGI0, SGI83775, SGI83775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18651[] = {SGI83776, SGI83777, SGI83778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83779, SGI0, SGI0, SGI0, SGI83780, SGI83780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18652[] = {SGI83781, SGI0, SGI83782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83783, SGI83784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18653[] = {SGI83789, SGI83790, SGI83791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83792, SGI0, SGI0, SGI0, SGI83793, SGI83793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18654[] = {SGI83785, SGI0, SGI83786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83787, SGI83788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18655[] = {SGI83794, SGI0, SGI83795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83796, SGI83797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18656[] = {SGI83798, SGI83799, SGI83800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83801, SGI0, SGI0, SGI0, SGI83802, SGI83802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18657[] = {SGI83803, SGI0, SGI83804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18658[] = {SGI83805, SGI83806, SGI83807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83808, SGI0, SGI0, SGI0, SGI83809, SGI83809, SGI0, SGI0, NULL};
const Segment * const SG18659[] = {SGI83810, SGI0, SGI83811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83812, SGI83813, SGI83813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18660[] = {SGI83814, SGI83815, SGI83816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83817, SGI0, SGI0, SGI0, SGI83818, SGI83818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18661[] = {SGI83819, SGI0, SGI83820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18662[] = {SGI83821, SGI83822, SGI83823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83824, SGI0, SGI0, SGI0, SGI83825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18663[] = {SGI83826, SGI0, SGI83827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18664[] = {SGI83828, SGI0, SGI83829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83831, SGI83833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18665[] = {SGI83830, SGI83832, SGI83834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83835, SGI0, SGI0, SGI0, SGI83836, SGI83836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18666[] = {SGI83837, SGI83838, SGI83839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83840, SGI0, SGI0, SGI0, SGI83842, SGI83842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18667[] = {SGI83841, SGI0, SGI83843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83844, SGI83845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18668[] = {SGI83846, SGI83847, SGI83848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83849, SGI0, SGI0, SGI0, SGI83850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18669[] = {SGI83851, SGI83852, SGI83853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83854, SGI0, SGI0, SGI0, SGI83855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18670[] = {SGI83856, SGI0, SGI83857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18671[] = {SGI83858, SGI83859, SGI83860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83861, SGI0, SGI0, SGI0, SGI83862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18672[] = {SGI83863, SGI83864, SGI83865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83866, SGI0, SGI0, SGI0, SGI83867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18673[] = {SGI83868, SGI83869, SGI83870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83871, SGI0, SGI0, SGI0, SGI83872, SGI83872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18674[] = {SGI83873, SGI83874, SGI83875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83876, SGI0, SGI0, SGI0, SGI83877, SGI83877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18675[] = {SGI83878, SGI83879, SGI83880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83881, SGI0, SGI0, SGI0, SGI83882, SGI83882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18676[] = {SGI83883, SGI0, SGI83884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18677[] = {SGI83885, SGI83886, SGI83887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83888, SGI0, SGI0, SGI0, SGI83889, SGI83889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18678[] = {SGI83890, SGI83891, SGI83892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83893, SGI0, SGI0, SGI0, SGI83894, SGI83894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18679[] = {SGI83895, SGI0, SGI83896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18680[] = {SGI83897, SGI83898, SGI83899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83900, SGI0, SGI0, SGI0, SGI83901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18681[] = {SGI83902, SGI0, SGI83903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18682[] = {SGI83904, SGI83905, SGI83906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83907, SGI0, SGI0, SGI0, SGI83908, SGI83908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18683[] = {SGI83909, SGI0, SGI83910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83911, SGI0, SGI0, SGI0, SGI83912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18684[] = {SGI83913, SGI83914, SGI83915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83916, SGI0, SGI0, SGI0, SGI83917, SGI83917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18685[] = {SGI83918, SGI83919, SGI83920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83921, SGI0, SGI0, SGI0, SGI83922, SGI83922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18686[] = {SGI83923, SGI83924, SGI83925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83926, SGI0, SGI0, SGI0, SGI83927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18687[] = {SGI83928, SGI83929, SGI83930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18688[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83931, SGI0, SGI0, SGI83932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18689[] = {SGI83933, SGI0, SGI83934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83935, SGI83936, SGI83936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18690[] = {SGI83937, SGI0, SGI83938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83939, SGI83940, SGI83940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18691[] = {SGI83941, SGI0, SGI83942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83943, SGI83944, SGI83944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18692[] = {SGI83945, SGI0, SGI83946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83947, SGI83948, SGI83948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18693[] = {SGI83949, SGI83950, SGI83951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83952, SGI0, SGI0, SGI0, SGI83953, SGI83953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18694[] = {SGI83954, SGI0, SGI83955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18695[] = {SGI83956, SGI83957, SGI83958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83959, SGI83959, SGI0, SGI0, SGI83960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18696[] = {SGI83961, SGI83962, SGI83963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83964, SGI83964, SGI0, SGI0, SGI83965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18697[] = {SGI83966, SGI83967, SGI83968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83969, SGI83969, SGI0, SGI0, SGI83970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18698[] = {SGI83971, SGI83972, SGI83973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83975, SGI0, SGI0, SGI0, SGI83976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18699[] = {SGI0, SGI0, SGI83974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83977, SGI83978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18700[] = {SGI83979, SGI83980, SGI83981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83982, SGI0, SGI0, SGI0, SGI83983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18701[] = {SGI83984, SGI0, SGI83985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18702[] = {SGI83986, SGI0, SGI83987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18703[] = {SGI83988, SGI83989, SGI83990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83991, SGI83991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18704[] = {SGI83992, SGI83993, SGI83994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI83995, SGI0, SGI0, SGI0, SGI83996, SGI83996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18705[] = {SGI83997, SGI83998, SGI83999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84000, SGI0, SGI0, SGI0, SGI84001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18706[] = {SGI84002, SGI0, SGI84003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84004, SGI0, SGI0, SGI0, SGI84005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18707[] = {SGI84006, SGI84007, SGI84008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18708[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84009, SGI0, SGI0, SGI84010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84011, SGI0, NULL};
const Segment * const SG18709[] = {SGI84012, SGI84013, SGI84014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84015, SGI0, SGI0, SGI0, SGI84016, SGI84016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18710[] = {SGI84017, SGI84018, SGI84019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84020, SGI84020, SGI0, SGI0, SGI84021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18711[] = {SGI84022, SGI84023, SGI84024, SGI0, SGI0, SGI0, SGI84025, SGI84026, SGI0, SGI84027, SGI0, SGI0, SGI0, SGI84028, SGI84028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18712[] = {SGI84029, SGI0, SGI84030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84031, SGI0, SGI0, SGI0, SGI84032, SGI84032, SGI0, SGI0, SGI84033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18713[] = {SGI84034, SGI0, SGI84035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84036, SGI0, SGI0, SGI0, SGI84037, SGI84037, SGI0, SGI0, SGI84038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18714[] = {SGI84039, SGI84040, SGI84041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84042, SGI0, SGI0, SGI0, SGI84043, SGI84044, SGI0, SGI0, SGI84045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18715[] = {SGI84046, SGI84047, SGI84048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84049, SGI0, SGI0, SGI0, SGI84050, SGI84050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18716[] = {SGI84051, SGI84052, SGI84053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84054, SGI0, SGI0, SGI0, SGI84055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18717[] = {SGI84056, SGI84057, SGI84058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84059, SGI0, SGI0, SGI0, SGI84060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18718[] = {SGI84061, SGI84062, SGI84063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84064, SGI0, SGI0, SGI0, SGI84065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18719[] = {SGI84066, SGI84067, SGI84068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84069, SGI0, SGI0, SGI0, SGI84070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18720[] = {SGI84071, SGI84072, SGI84073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84074, SGI0, SGI0, SGI0, SGI84075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18721[] = {SGI84076, SGI0, SGI84077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18722[] = {SGI84078, SGI84079, SGI84080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84083, SGI0, SGI0, SGI0, SGI84085, SGI84085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18723[] = {SGI84081, SGI84082, SGI84084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84086, SGI0, SGI0, SGI0, SGI84087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18724[] = {SGI84088, SGI84089, SGI84090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84094, SGI84094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18725[] = {SGI84091, SGI84092, SGI84093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84095, SGI0, SGI0, SGI0, SGI84096, SGI84096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18726[] = {SGI84097, SGI84098, SGI84099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84102, SGI0, SGI0, SGI0, SGI84104, SGI84104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18727[] = {SGI84100, SGI84101, SGI84103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84105, SGI0, SGI0, SGI0, SGI84106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18728[] = {SGI84107, SGI84108, SGI84109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84110, SGI0, SGI0, SGI0, SGI84113, SGI84113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18729[] = {SGI84111, SGI84112, SGI84114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84115, SGI0, SGI0, SGI0, SGI84116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18730[] = {SGI84117, SGI84118, SGI84119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84120, SGI0, SGI0, SGI0, SGI84121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18731[] = {SGI84122, SGI84123, SGI84124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84125, SGI0, SGI0, SGI0, SGI84126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18732[] = {SGI84127, SGI84128, SGI84129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84130, SGI0, SGI0, SGI0, SGI84131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18733[] = {SGI84132, SGI84133, SGI84134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84135, SGI0, SGI0, SGI0, SGI84136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18734[] = {SGI84137, SGI84138, SGI84139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84140, SGI0, SGI0, SGI0, SGI84141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18735[] = {SGI84142, SGI84143, SGI84144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84145, SGI0, SGI0, SGI0, SGI84146, SGI84146, SGI84147, SGI0, SGI84148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18736[] = {SGI84149, SGI84150, SGI84151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84152, SGI0, SGI0, SGI0, SGI84153, SGI84153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84154, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18737[] = {SGI84155, SGI84156, SGI84157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84158, SGI0, SGI0, SGI0, SGI84159, SGI84159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18738[] = {SGI84160, SGI84161, SGI84162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84163, SGI0, SGI0, SGI0, SGI84164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18739[] = {SGI84165, SGI0, SGI84166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18740[] = {SGI84167, SGI84168, SGI84169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84170, SGI0, SGI0, SGI0, SGI84171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18741[] = {SGI84172, SGI84173, SGI84174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18742[] = {SGI84175, SGI84176, SGI84177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84178, SGI0, SGI0, SGI0, SGI84179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18743[] = {SGI84180, SGI84181, SGI84182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18744[] = {SGI84183, SGI84184, SGI84185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84186, SGI84186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18745[] = {SGI84187, SGI0, SGI84188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84189, SGI84190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18746[] = {SGI84191, SGI0, SGI84192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84193, SGI84194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18747[] = {SGI84195, SGI84196, SGI84197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84198, SGI0, SGI0, SGI0, SGI84199, SGI84199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18748[] = {SGI84200, SGI84201, SGI84202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84203, SGI0, SGI0, SGI0, SGI84204, SGI84204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18749[] = {SGI84205, SGI84206, SGI84207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84208, SGI0, SGI0, SGI0, SGI84209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18750[] = {SGI84210, SGI84211, SGI84212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84213, SGI0, SGI0, SGI0, SGI84214, SGI84214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18751[] = {SGI84215, SGI84216, SGI84217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84218, SGI0, SGI0, SGI0, SGI84219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18752[] = {SGI84220, SGI0, SGI84221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18753[] = {SGI84222, SGI84223, SGI84224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84225, SGI0, SGI0, SGI0, SGI84226, SGI84226, SGI84227, SGI0, SGI84228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18754[] = {SGI84229, SGI84230, SGI84231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84232, SGI0, SGI0, SGI0, SGI84233, SGI84233, SGI84234, SGI0, SGI84235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18755[] = {SGI84236, SGI84237, SGI84238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84239, SGI0, SGI0, SGI0, SGI84240, SGI84240, SGI84241, SGI0, SGI84242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18756[] = {SGI84243, SGI84244, SGI84245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84246, SGI0, SGI0, SGI0, SGI84247, SGI84247, SGI0, SGI84248, SGI0, NULL};
const Segment * const SG18757[] = {SGI84249, SGI0, SGI84250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84251, SGI84252, SGI84252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18758[] = {SGI84254, SGI0, SGI84257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84258, SGI84259, SGI84259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18759[] = {SGI84253, SGI84255, SGI84256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84260, SGI84260, SGI0, SGI0, SGI84261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18760[] = {SGI84262, SGI0, SGI84263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84264, SGI84265, SGI84265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18761[] = {SGI84266, SGI0, SGI84267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84268, SGI84269, SGI84269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18762[] = {SGI84270, SGI0, SGI84271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84272, SGI84273, SGI84273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18763[] = {SGI84274, SGI0, SGI84275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84276, SGI84277, SGI84277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18764[] = {SGI84278, SGI0, SGI84279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84280, SGI84281, SGI84281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18765[] = {SGI84282, SGI0, SGI84283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84284, SGI84285, SGI84285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18766[] = {SGI84290, SGI84291, SGI84292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84293, SGI0, SGI0, SGI0, SGI84294, SGI84294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18767[] = {SGI84295, SGI84296, SGI84297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84298, SGI84299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18768[] = {SGI84300, SGI84301, SGI84302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84303, SGI84303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18769[] = {SGI84304, SGI0, SGI84305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18770[] = {SGI84286, SGI0, SGI84287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84288, SGI84289, SGI84289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18771[] = {SGI84306, SGI0, SGI84307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84308, SGI84309, SGI84309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18772[] = {SGI84310, SGI0, SGI84311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84312, SGI84313, SGI84313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18773[] = {SGI84314, SGI0, SGI84315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84316, SGI84317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18774[] = {SGI84318, SGI0, SGI84319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84320, SGI84321, SGI84321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18775[] = {SGI84322, SGI0, SGI84323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84324, SGI84325, SGI84325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18776[] = {SGI84326, SGI84327, SGI84328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84329, SGI0, SGI0, SGI0, SGI84330, SGI84330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18777[] = {SGI84331, SGI84332, SGI84333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84334, SGI0, SGI0, SGI0, SGI84335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18778[] = {SGI84336, SGI84337, SGI84338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84339, SGI0, SGI0, SGI0, SGI84340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18779[] = {SGI84341, SGI84342, SGI84343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84344, SGI0, SGI0, SGI0, SGI84345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18780[] = {SGI84346, SGI0, SGI84347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18781[] = {SGI84348, SGI84349, SGI84350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84351, SGI0, SGI0, SGI0, SGI84352, SGI84352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18782[] = {SGI84353, SGI84354, SGI84355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84356, SGI0, SGI0, SGI0, SGI84357, SGI84357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18783[] = {SGI84358, SGI84359, SGI84360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84361, SGI84361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18784[] = {SGI84362, SGI84363, SGI84364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84365, SGI0, SGI0, SGI0, SGI84366, SGI84366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18785[] = {SGI84367, SGI84368, SGI84369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84370, SGI0, SGI0, SGI0, SGI84371, SGI84371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18786[] = {SGI84372, SGI84373, SGI84374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84375, SGI0, SGI0, SGI0, SGI84376, SGI84376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18787[] = {SGI84377, SGI84378, SGI84379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84380, SGI0, SGI0, SGI0, SGI84381, SGI84381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18788[] = {SGI84382, SGI84383, SGI84384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84385, SGI0, SGI0, SGI0, SGI84386, SGI84386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18789[] = {SGI84387, SGI0, SGI84388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18790[] = {SGI84389, SGI84390, SGI84391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84392, SGI0, SGI0, SGI0, SGI84393, SGI84393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18791[] = {SGI84394, SGI84395, SGI84396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84397, SGI0, SGI0, SGI0, SGI84398, SGI84398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18792[] = {SGI84399, SGI84400, SGI84401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84402, SGI0, SGI0, SGI0, SGI84403, SGI84403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18793[] = {SGI84404, SGI84405, SGI84406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84407, SGI0, SGI0, SGI0, SGI84408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18794[] = {SGI84409, SGI0, SGI84410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84411, SGI84412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18795[] = {SGI84413, SGI0, SGI84414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84415, SGI84416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18796[] = {SGI84417, SGI0, SGI84418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84419, SGI84420, SGI84420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18797[] = {SGI84421, SGI0, SGI84422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84423, SGI84424, SGI84424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18798[] = {SGI84425, SGI0, SGI84426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84427, SGI84428, SGI84428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18799[] = {SGI84429, SGI0, SGI84430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84431, SGI84432, SGI84432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18800[] = {SGI84433, SGI0, SGI84434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84435, SGI84436, SGI84436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18801[] = {SGI84437, SGI84438, SGI84439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84440, SGI0, SGI0, SGI0, SGI84441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18802[] = {SGI84442, SGI84443, SGI84444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84445, SGI0, SGI0, SGI0, SGI84446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18803[] = {SGI84447, SGI0, SGI84448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18804[] = {SGI84449, SGI0, SGI84450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84451, SGI84452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18805[] = {SGI84453, SGI0, SGI84454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84455, SGI84456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18806[] = {SGI84457, SGI84458, SGI84459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84460, SGI0, SGI0, SGI0, SGI84461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18807[] = {SGI84462, SGI0, SGI84463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18808[] = {SGI84464, SGI84465, SGI84466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84467, SGI0, SGI0, SGI0, SGI84468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18809[] = {SGI84469, SGI0, SGI84470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18810[] = {SGI84471, SGI84472, SGI84473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84474, SGI0, SGI0, SGI0, SGI84475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18811[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84476, SGI0, SGI84477, SGI0, NULL};
const Segment * const SG18812[] = {SGI84478, SGI0, SGI84479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84480, SGI84481, SGI84481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18813[] = {SGI84482, SGI0, SGI84483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84484, SGI84485, SGI84485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18814[] = {SGI84486, SGI0, SGI84487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84488, SGI84489, SGI84489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18815[] = {SGI84490, SGI0, SGI84491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84492, SGI84493, SGI84493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18816[] = {SGI84494, SGI0, SGI84495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84496, SGI84497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18817[] = {SGI84498, SGI84499, SGI84500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84501, SGI0, SGI0, SGI0, SGI84502, SGI84502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18818[] = {SGI84503, SGI84504, SGI84505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84506, SGI84507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18819[] = {SGI84508, SGI84509, SGI84510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84511, SGI84511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18820[] = {SGI0, SGI84512, SGI84513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18821[] = {SGI84514, SGI0, SGI84515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18822[] = {SGI84516, SGI84517, SGI84518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84519, SGI0, SGI0, SGI0, SGI84520, SGI84520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18823[] = {SGI84521, SGI84522, SGI84523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84524, SGI84524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18824[] = {SGI84525, SGI0, SGI84526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18825[] = {SGI0, SGI84527, SGI84528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18826[] = {SGI84529, SGI0, SGI84530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84531, SGI84532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18827[] = {SGI84533, SGI0, SGI84534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84535, SGI84536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18828[] = {SGI84537, SGI0, SGI84538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84539, SGI84540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18829[] = {SGI84541, SGI0, SGI84542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84543, SGI84544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18830[] = {SGI84545, SGI84546, SGI84547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84548, SGI0, SGI0, SGI0, SGI84549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18831[] = {SGI84550, SGI0, SGI84551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18832[] = {SGI84552, SGI84553, SGI84554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84555, SGI0, SGI0, SGI0, SGI84556, SGI84556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18833[] = {SGI84557, SGI0, SGI84558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18834[] = {SGI84559, SGI84560, SGI84561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84562, SGI0, SGI0, SGI0, SGI84563, SGI84563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18835[] = {SGI84564, SGI84565, SGI84566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84567, SGI0, SGI0, SGI0, SGI84568, SGI84568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18836[] = {SGI84569, SGI84570, SGI84571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84572, SGI0, SGI0, SGI0, SGI84573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18837[] = {SGI84574, SGI84575, SGI84576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84577, SGI0, SGI0, SGI0, SGI84578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18838[] = {SGI84579, SGI0, SGI84580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18839[] = {SGI84581, SGI84582, SGI84583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84584, SGI0, SGI0, SGI0, SGI84585, SGI84585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18840[] = {SGI84586, SGI84587, SGI84588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84589, SGI0, SGI0, SGI0, SGI84590, SGI84590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18841[] = {SGI84591, SGI0, SGI84592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18842[] = {SGI84593, SGI84594, SGI84595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84596, SGI0, SGI0, SGI0, SGI84597, SGI84597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18843[] = {SGI84598, SGI84599, SGI84600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84601, SGI0, SGI0, SGI0, SGI84602, SGI84602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18844[] = {SGI84603, SGI84604, SGI84605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84606, SGI0, SGI0, SGI0, SGI84607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18845[] = {SGI84608, SGI84609, SGI84610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84611, SGI0, SGI0, SGI0, SGI84612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18846[] = {SGI84613, SGI84614, SGI84615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84616, SGI0, SGI0, SGI0, SGI84617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18847[] = {SGI84618, SGI0, SGI84619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18848[] = {SGI84620, SGI84621, SGI84622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84623, SGI0, SGI0, SGI0, SGI84624, SGI84624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18849[] = {SGI84625, SGI84626, SGI84627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84628, SGI0, SGI0, SGI0, SGI84629, SGI84629, SGI0, SGI84630, SGI0, NULL};
const Segment * const SG18850[] = {SGI84636, SGI0, SGI84637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84638, SGI84639, SGI84639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18851[] = {SGI84640, SGI0, SGI84641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84642, SGI84643, SGI84643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18852[] = {SGI84647, SGI0, SGI84648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84649, SGI84650, SGI84650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18853[] = {SGI84631, SGI0, SGI84632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84633, SGI0, SGI0, SGI0, SGI84634, SGI84634, SGI0, SGI0, SGI84635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18854[] = {SGI84644, SGI84645, SGI84646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84651, SGI0, SGI0, SGI0, SGI84652, SGI84652, SGI84653, SGI0, SGI84654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18855[] = {SGI84655, SGI84656, SGI84657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84658, SGI0, SGI0, SGI0, SGI84659, SGI84660, SGI0, SGI0, NULL};
const Segment * const SG18856[] = {SGI84664, SGI0, SGI84665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84666, SGI0, SGI0, SGI0, SGI84667, SGI84667, SGI0, SGI0, SGI84668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18857[] = {SGI84669, SGI84670, SGI84671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84672, SGI0, SGI0, SGI0, SGI84673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18858[] = {SGI84674, SGI84675, SGI84676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84677, SGI0, SGI0, SGI0, SGI84678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18859[] = {SGI84679, SGI0, SGI84680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18860[] = {SGI84661, SGI84662, SGI84663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84681, SGI0, SGI0, SGI0, SGI84682, SGI84682, SGI84683, SGI0, SGI84684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18861[] = {SGI84685, SGI84686, SGI84687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84688, SGI0, SGI0, SGI0, SGI84689, SGI84689, SGI0, SGI0, SGI84690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18862[] = {SGI84691, SGI84692, SGI84693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84694, SGI0, SGI0, SGI0, SGI84695, SGI84695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18863[] = {SGI84696, SGI84697, SGI84698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84699, SGI0, SGI0, SGI0, SGI84700, SGI84700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18864[] = {SGI84701, SGI84702, SGI84703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84704, SGI0, SGI0, SGI0, SGI84705, SGI84705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18865[] = {SGI84706, SGI0, SGI84707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18866[] = {SGI84708, SGI0, SGI84709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18867[] = {SGI84710, SGI84711, SGI84712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84713, SGI84713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18868[] = {SGI84714, SGI0, SGI84715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18869[] = {SGI84716, SGI84717, SGI84718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84719, SGI0, SGI0, SGI0, SGI84720, SGI84720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18870[] = {SGI84721, SGI84722, SGI84723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84724, SGI84724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18871[] = {SGI84725, SGI0, SGI84726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84727, SGI84728, SGI84728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18872[] = {SGI84729, SGI0, SGI84730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84731, SGI84732, SGI84732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18873[] = {SGI84733, SGI0, SGI84734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84735, SGI84736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18874[] = {SGI84737, SGI0, SGI84738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84739, SGI84740, SGI84740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18875[] = {SGI84741, SGI84742, SGI84743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84744, SGI0, SGI0, SGI0, SGI84745, SGI84745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18876[] = {SGI84746, SGI84747, SGI84748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84749, SGI84750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18877[] = {SGI84751, SGI84752, SGI84753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84754, SGI0, SGI0, SGI0, SGI84755, SGI84755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18878[] = {SGI84756, SGI0, SGI84757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18879[] = {SGI84758, SGI84759, SGI84760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84761, SGI84761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18880[] = {SGI84762, SGI84763, SGI84764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84765, SGI0, SGI0, SGI0, SGI84766, SGI84766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18881[] = {SGI84767, SGI84768, SGI84769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84770, SGI0, SGI0, SGI0, SGI84771, SGI84771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18882[] = {SGI84772, SGI84773, SGI84774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84775, SGI84775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18883[] = {SGI84776, SGI0, SGI84777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84778, SGI84779, SGI84779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84780, NULL};
const Segment * const SG18884[] = {SGI84781, SGI0, SGI84782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84783, SGI84784, SGI84784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84785, NULL};
const Segment * const SG18885[] = {SGI84786, SGI84787, SGI84788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84789, SGI0, SGI0, SGI0, SGI84790, SGI84790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18886[] = {SGI84791, SGI84792, SGI84793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84794, SGI0, SGI0, SGI0, SGI84795, SGI84795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18887[] = {SGI84796, SGI84797, SGI84798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84799, SGI0, SGI0, SGI0, SGI84800, SGI84800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18888[] = {SGI84801, SGI84802, SGI84803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84804, SGI0, SGI0, SGI0, SGI84805, SGI84805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18889[] = {SGI84806, SGI84807, SGI84808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84809, SGI0, SGI0, SGI0, SGI84810, SGI84810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18890[] = {SGI84811, SGI84812, SGI84813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84814, SGI84814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18891[] = {SGI84815, SGI0, SGI84816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18892[] = {SGI84817, SGI84818, SGI84819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84820, SGI0, SGI0, SGI0, SGI84821, SGI84821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18893[] = {SGI84822, SGI0, SGI84823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18894[] = {SGI84824, SGI84825, SGI84826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84827, SGI0, SGI0, SGI0, SGI84828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18895[] = {SGI84829, SGI0, SGI84830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18896[] = {SGI84831, SGI84832, SGI84833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84834, SGI0, SGI0, SGI0, SGI84835, SGI84835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18897[] = {SGI84836, SGI84837, SGI84838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84839, SGI0, SGI0, SGI0, SGI84840, SGI84840, SGI84841, SGI0, SGI84842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18898[] = {SGI84843, SGI84844, SGI84845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84846, SGI0, SGI0, SGI0, SGI84847, SGI84847, SGI84848, SGI0, SGI84849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18899[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84850, SGI0, SGI0, SGI84851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18900[] = {SGI84852, SGI0, SGI84853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84854, SGI0, SGI0, SGI0, SGI84855, SGI84855, SGI0, SGI0, SGI84856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18901[] = {SGI84857, SGI84858, SGI84859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84860, SGI0, SGI0, SGI0, SGI84861, SGI84862, SGI0, SGI0, SGI84863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18902[] = {SGI84864, SGI0, SGI84865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84866, SGI84867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18903[] = {SGI84868, SGI0, SGI84869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84870, SGI84871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18904[] = {SGI84872, SGI84873, SGI84874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84875, SGI84875, SGI0, SGI0, SGI84876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18905[] = {SGI84877, SGI84878, SGI84879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84880, SGI84880, SGI0, SGI0, SGI84881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18906[] = {SGI84882, SGI84883, SGI84884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84885, SGI0, SGI0, SGI0, SGI84886, SGI84886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18907[] = {SGI84887, SGI84888, SGI84889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84890, SGI0, SGI0, SGI0, SGI84891, SGI84891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18908[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84892, SGI0, SGI0, SGI84893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18909[] = {SGI84894, SGI84895, SGI84896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84897, SGI0, SGI0, SGI0, SGI84898, SGI84898, SGI0, SGI84899, SGI0, NULL};
const Segment * const SG18910[] = {SGI84900, SGI84901, SGI84902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84903, SGI0, SGI0, SGI0, SGI84904, SGI84904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84905, SGI0, NULL};
const Segment * const SG18911[] = {SGI84906, SGI0, SGI84907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84908, SGI0, SGI0, SGI0, SGI84909, SGI84909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84910, SGI0, NULL};
const Segment * const SG18912[] = {SGI84911, SGI84912, SGI84913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84914, SGI0, SGI0, SGI0, SGI84915, SGI84915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18913[] = {SGI84916, SGI84917, SGI84918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84919, SGI84920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18914[] = {SGI84921, SGI0, SGI84922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18915[] = {SGI84923, SGI84924, SGI84925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84926, SGI84926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18916[] = {SGI84927, SGI84928, SGI84929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84930, SGI84930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18917[] = {SGI84931, SGI0, SGI84932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84933, SGI0, SGI0, SGI0, SGI84934, SGI84934, SGI0, SGI0, SGI84935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18918[] = {SGI84936, SGI0, SGI84937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84938, SGI0, SGI0, SGI0, SGI84939, SGI84939, SGI0, SGI0, SGI84940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18919[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84944, SGI0, SGI0, SGI84945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18920[] = {SGI84941, SGI84942, SGI84943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84946, SGI0, SGI0, SGI0, SGI84947, SGI84947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18921[] = {SGI84948, SGI0, SGI84949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18922[] = {SGI84950, SGI84951, SGI84952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84953, SGI0, SGI0, SGI0, SGI84954, SGI84954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18923[] = {SGI84955, SGI84956, SGI84957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84958, SGI84958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18924[] = {SGI84959, SGI0, SGI84960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18925[] = {SGI84961, SGI84962, SGI84963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84964, SGI0, SGI0, SGI0, SGI84965, SGI84965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18926[] = {SGI84966, SGI84967, SGI84968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84969, SGI0, SGI0, SGI0, SGI84970, SGI84970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18927[] = {SGI84971, SGI84972, SGI84973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84974, SGI84975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18928[] = {SGI84976, SGI84977, SGI84978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84979, SGI0, SGI0, SGI0, SGI84980, SGI84980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18929[] = {SGI84981, SGI84982, SGI84983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84984, SGI84984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18930[] = {SGI84985, SGI0, SGI84986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18931[] = {SGI0, SGI84987, SGI84988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18932[] = {SGI84989, SGI84990, SGI84991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84992, SGI0, SGI0, SGI0, SGI84993, SGI84993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18933[] = {SGI84994, SGI84995, SGI84996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI84997, SGI0, SGI0, SGI0, SGI84998, SGI84998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18934[] = {SGI84999, SGI85000, SGI85001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85002, SGI0, SGI0, SGI0, SGI85003, SGI85003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18935[] = {SGI85004, SGI85005, SGI85006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85007, SGI0, SGI0, SGI0, SGI85008, SGI85008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18936[] = {SGI85009, SGI85010, SGI85011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85012, SGI0, SGI0, SGI0, SGI85013, SGI85013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18937[] = {SGI85014, SGI85015, SGI85016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85017, SGI0, SGI0, SGI0, SGI85018, SGI85018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18938[] = {SGI85019, SGI85020, SGI85021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85022, SGI0, SGI0, SGI0, SGI85023, SGI85023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18939[] = {SGI85024, SGI85025, SGI85026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85027, SGI0, SGI0, SGI0, SGI85028, SGI85028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18940[] = {SGI85029, SGI85030, SGI85031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85032, SGI0, SGI0, SGI0, SGI85033, SGI85033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18941[] = {SGI85034, SGI0, SGI85035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18942[] = {SGI85036, SGI85037, SGI85038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85039, SGI85039, SGI0, SGI0, SGI85040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18943[] = {SGI85041, SGI85042, SGI85043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85044, SGI85044, SGI0, SGI0, SGI85045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18944[] = {SGI85046, SGI85047, SGI85048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85049, SGI0, SGI0, SGI0, SGI85050, SGI85050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18945[] = {SGI85051, SGI85052, SGI85053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85054, SGI0, SGI0, SGI0, SGI85055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18946[] = {SGI85056, SGI85057, SGI85058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85059, SGI0, SGI0, SGI0, SGI85060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18947[] = {SGI85061, SGI0, SGI85062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18948[] = {SGI85063, SGI85064, SGI85065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85066, SGI0, SGI0, SGI0, SGI85067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18949[] = {SGI85068, SGI85069, SGI85070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85071, SGI0, SGI0, SGI0, SGI85072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18950[] = {SGI85073, SGI85074, SGI85075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85076, SGI0, SGI0, SGI0, SGI85077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18951[] = {SGI85078, SGI85079, SGI85080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85081, SGI0, SGI0, SGI0, SGI85082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18952[] = {SGI85083, SGI0, SGI85084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85085, SGI0, SGI0, SGI0, SGI85086, SGI85086, SGI0, SGI0, SGI85089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18953[] = {SGI85087, SGI85088, SGI85090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85091, SGI0, SGI0, SGI0, SGI85092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18954[] = {SGI85093, SGI85094, SGI85095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85096, SGI0, SGI0, SGI0, SGI85097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18955[] = {SGI85098, SGI85099, SGI85100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85101, SGI0, SGI0, SGI0, SGI85102, SGI85102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18956[] = {SGI85103, SGI85104, SGI85105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85106, SGI85107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18957[] = {SGI85108, SGI85109, SGI85110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85111, SGI85111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18958[] = {SGI0, SGI85112, SGI85113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18959[] = {SGI85114, SGI85115, SGI85116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85117, SGI0, SGI0, SGI0, SGI85118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18960[] = {SGI85119, SGI0, SGI85120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18961[] = {SGI85121, SGI85122, SGI85123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85124, SGI0, SGI0, SGI0, SGI85125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18962[] = {SGI85126, SGI85127, SGI85128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85129, SGI0, SGI0, SGI0, SGI85130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18963[] = {SGI85131, SGI85132, SGI85133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85134, SGI0, SGI0, SGI0, SGI85135, SGI85135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18964[] = {SGI85136, SGI85137, SGI85138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85139, SGI0, SGI0, SGI0, SGI85140, SGI85140, SGI85141, SGI0, SGI85142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18965[] = {SGI85143, SGI85144, SGI85145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85146, SGI0, SGI0, SGI0, SGI85147, SGI85147, SGI85148, SGI0, SGI85149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18966[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85150, SGI0, SGI0, SGI85151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18967[] = {SGI85152, SGI85153, SGI85154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85155, SGI0, SGI0, SGI0, SGI85156, SGI85156, SGI85157, SGI0, SGI85158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18968[] = {SGI85159, SGI85160, SGI85161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85162, SGI0, SGI0, SGI0, SGI85163, SGI85163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85164, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18969[] = {SGI85165, SGI85166, SGI85167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85168, SGI0, SGI0, SGI0, SGI85169, SGI85169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18970[] = {SGI85170, SGI0, SGI85171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85172, SGI85173, SGI85173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18971[] = {SGI85174, SGI85175, SGI85176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85177, SGI0, SGI0, SGI0, SGI85178, SGI85178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18972[] = {SGI85179, SGI0, SGI85180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18973[] = {SGI0, SGI85181, SGI85182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85183, SGI0, SGI0, SGI0, SGI85184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18974[] = {SGI0, SGI85185, SGI85186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85187, SGI0, SGI0, SGI0, SGI85188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18975[] = {SGI0, SGI85189, SGI85190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85191, SGI0, SGI0, SGI0, SGI85192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18976[] = {SGI0, SGI85193, SGI85194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85195, SGI0, SGI0, SGI0, SGI85196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18977[] = {SGI0, SGI85197, SGI85198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85199, SGI0, SGI0, SGI0, SGI85200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18978[] = {SGI0, SGI85201, SGI85202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85203, SGI0, SGI0, SGI0, SGI85204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18979[] = {SGI0, SGI85205, SGI85206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85207, SGI0, SGI0, SGI0, SGI85208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18980[] = {SGI0, SGI85209, SGI85210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85211, SGI0, SGI0, SGI0, SGI85212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18981[] = {SGI0, SGI85213, SGI85214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85215, SGI0, SGI0, SGI0, SGI85216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18982[] = {SGI0, SGI85217, SGI85218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85219, SGI0, SGI0, SGI0, SGI85220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18983[] = {SGI0, SGI85221, SGI85222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85223, SGI0, SGI0, SGI0, SGI85224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18984[] = {SGI0, SGI85225, SGI85226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85227, SGI0, SGI0, SGI0, SGI85228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18985[] = {SGI0, SGI85229, SGI85230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85231, SGI0, SGI0, SGI0, SGI85232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18986[] = {SGI85233, SGI85234, SGI85235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85236, SGI0, SGI0, SGI0, SGI85237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18987[] = {SGI0, SGI85238, SGI85239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85240, SGI0, SGI0, SGI0, SGI85241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18988[] = {SGI85242, SGI85243, SGI85244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85245, SGI0, SGI0, SGI0, SGI85246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18989[] = {SGI85247, SGI85248, SGI85249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85250, SGI0, SGI0, SGI0, SGI85251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18990[] = {SGI85252, SGI85253, SGI85254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85255, SGI0, SGI0, SGI0, SGI85256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18991[] = {SGI85257, SGI0, SGI85258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18992[] = {SGI85259, SGI0, SGI85260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18993[] = {SGI85261, SGI0, SGI85262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85263, SGI85264, SGI85264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18994[] = {SGI85265, SGI0, SGI85266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85267, SGI85268, SGI85268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18995[] = {SGI85269, SGI85270, SGI85271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85272, SGI0, SGI0, SGI0, SGI85273, SGI85273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18996[] = {SGI85274, SGI85275, SGI85276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85277, SGI0, SGI0, SGI0, SGI85278, SGI85278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18997[] = {SGI85279, SGI0, SGI85280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18998[] = {SGI85281, SGI85282, SGI85283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85284, SGI85285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG18999[] = {SGI85286, SGI85287, SGI85288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85289, SGI0, SGI0, SGI0, SGI85290, SGI85290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19000[] = {SGI85291, SGI0, SGI85292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85293, SGI85294, SGI85294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19001[] = {SGI85295, SGI85296, SGI85297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85298, SGI0, SGI0, SGI0, SGI85299, SGI85299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19002[] = {SGI85300, SGI0, SGI85301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85302, SGI85303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19003[] = {SGI85304, SGI85305, SGI85306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85307, SGI85307, SGI0, SGI0, SGI85308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19004[] = {SGI85309, SGI85310, SGI85311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85312, SGI85312, SGI0, SGI0, SGI85313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19005[] = {SGI85314, SGI0, SGI85315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85316, SGI85317, SGI85317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19006[] = {SGI85318, SGI0, SGI85319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85320, SGI85321, SGI85321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19007[] = {SGI85322, SGI85323, SGI85324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85325, SGI0, SGI0, SGI0, SGI85326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19008[] = {SGI85327, SGI85328, SGI85329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85330, SGI0, SGI0, SGI0, SGI85331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19009[] = {SGI85332, SGI85333, SGI85334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85335, SGI0, SGI0, SGI0, SGI85336, SGI85336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19010[] = {SGI85337, SGI85338, SGI85339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85340, SGI0, SGI0, SGI0, SGI85341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19011[] = {SGI85342, SGI0, SGI85343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19012[] = {SGI85344, SGI0, SGI85345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85346, SGI85347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19013[] = {SGI85348, SGI85349, SGI85350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85351, SGI0, SGI0, SGI0, SGI85352, SGI85352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19014[] = {SGI85353, SGI85354, SGI85355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85356, SGI0, SGI0, SGI0, SGI85357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19015[] = {SGI85358, SGI85359, SGI85360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85361, SGI0, SGI0, SGI0, SGI85362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19016[] = {SGI85363, SGI0, SGI85364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19017[] = {SGI85365, SGI85366, SGI85367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85368, SGI0, SGI0, SGI0, SGI85369, SGI85369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19018[] = {SGI85370, SGI85371, SGI85372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85373, SGI0, SGI0, SGI0, SGI85374, SGI85374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19019[] = {SGI85375, SGI0, SGI85376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19020[] = {SGI85377, SGI85378, SGI85379, SGI0, SGI0, SGI0, SGI85380, SGI85381, SGI0, SGI85382, SGI0, SGI0, SGI0, SGI85383, SGI85383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19021[] = {SGI85384, SGI85385, SGI85386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85387, SGI0, SGI0, SGI0, SGI85388, SGI85388, SGI0, SGI0, NULL};
const Segment * const SG19022[] = {SGI85389, SGI85390, SGI85391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85392, SGI0, SGI0, SGI0, SGI85393, SGI85393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19023[] = {SGI85394, SGI85395, SGI85396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85397, SGI85398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19024[] = {SGI85399, SGI0, SGI85400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19025[] = {SGI85401, SGI85402, SGI85403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85405, SGI0, SGI0, SGI0, SGI85408, SGI85408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19026[] = {SGI85404, SGI85406, SGI85407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85409, SGI0, SGI0, SGI0, SGI85410, SGI85410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19027[] = {SGI85411, SGI85412, SGI85413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85417, SGI85417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19028[] = {SGI85414, SGI85415, SGI85416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85418, SGI0, SGI0, SGI0, SGI85419, SGI85419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19029[] = {SGI85420, SGI85421, SGI85422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85423, SGI0, SGI0, SGI0, SGI85425, SGI85425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19030[] = {SGI85424, SGI0, SGI85426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85427, SGI0, SGI0, SGI0, SGI85428, SGI85428, SGI0, SGI0, SGI85429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19031[] = {SGI85430, SGI0, SGI85431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19032[] = {SGI85432, SGI0, SGI85433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19033[] = {SGI85434, SGI0, SGI85435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19034[] = {SGI85436, SGI0, SGI85437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85438, SGI0, SGI0, SGI0, SGI85440, SGI85440, SGI0, SGI0, SGI85442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19035[] = {SGI85439, SGI0, SGI85441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85443, SGI0, SGI0, SGI0, SGI85444, SGI85444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19036[] = {SGI85445, SGI0, SGI85446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85447, SGI0, SGI0, SGI0, SGI85448, SGI85448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19037[] = {SGI85449, SGI85450, SGI85451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85454, SGI0, SGI0, SGI0, SGI85455, SGI85457, SGI0, SGI0, SGI85458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19038[] = {SGI85452, SGI0, SGI85453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85456, SGI0, SGI0, SGI0, SGI85459, SGI85459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19039[] = {SGI85460, SGI85461, SGI85462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85463, SGI0, SGI0, SGI0, SGI85464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19040[] = {SGI85465, SGI85466, SGI85467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85468, SGI0, SGI0, SGI0, SGI85469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19041[] = {SGI85470, SGI85471, SGI85472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85473, SGI0, SGI0, SGI0, SGI85474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19042[] = {SGI85475, SGI85476, SGI85477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85478, SGI0, SGI0, SGI0, SGI85479, SGI85479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19043[] = {SGI85480, SGI85481, SGI85482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85483, SGI0, SGI0, SGI0, SGI85484, SGI85484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19044[] = {SGI85485, SGI85486, SGI85487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85488, SGI0, SGI0, SGI0, SGI85489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19045[] = {SGI85490, SGI85491, SGI85492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85493, SGI0, SGI0, SGI0, SGI85494, SGI85494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19046[] = {SGI85495, SGI85496, SGI85497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85498, SGI0, SGI0, SGI0, SGI85499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19047[] = {SGI85500, SGI0, SGI85501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19048[] = {SGI85502, SGI85503, SGI85504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85505, SGI0, SGI0, SGI0, SGI85506, SGI85506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19049[] = {SGI85507, SGI0, SGI85508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85509, SGI0, SGI0, SGI0, SGI85510, SGI85510, SGI0, SGI0, SGI85511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19050[] = {SGI85512, SGI0, SGI85513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85514, SGI0, SGI0, SGI0, SGI85515, SGI85515, SGI0, SGI0, SGI85516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19051[] = {SGI85517, SGI85518, SGI85519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85520, SGI0, SGI0, SGI0, SGI85521, SGI85522, SGI0, SGI0, SGI85523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19052[] = {SGI85524, SGI85525, SGI85526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85527, SGI0, SGI0, SGI0, SGI85528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19053[] = {SGI85529, SGI0, SGI85530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19054[] = {SGI85531, SGI85532, SGI85533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85534, SGI0, SGI0, SGI0, SGI85535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19055[] = {SGI85536, SGI0, SGI85537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19056[] = {SGI85538, SGI0, SGI85539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85540, SGI85541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19057[] = {SGI85542, SGI0, SGI85543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85544, SGI85545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19058[] = {SGI85546, SGI0, SGI85547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85548, SGI85549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19059[] = {SGI85550, SGI0, SGI85551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85552, SGI85553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19060[] = {SGI85554, SGI0, SGI85555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85556, SGI85557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19061[] = {SGI85558, SGI85559, SGI85560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85561, SGI0, SGI0, SGI0, SGI85562, SGI85562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19062[] = {SGI85563, SGI85564, SGI85565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85566, SGI0, SGI0, SGI0, SGI85567, SGI85567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19063[] = {SGI85568, SGI85569, SGI85570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85571, SGI0, SGI0, SGI0, SGI85572, SGI85572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19064[] = {SGI85573, SGI85574, SGI85575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85576, SGI85576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19065[] = {SGI85577, SGI85578, SGI85579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85580, SGI0, SGI0, SGI0, SGI85581, SGI85581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19066[] = {SGI85582, SGI85583, SGI85584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85585, SGI0, SGI0, SGI0, SGI85586, SGI85586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19067[] = {SGI85587, SGI0, SGI85588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19068[] = {SGI85589, SGI85590, SGI85591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85592, SGI0, SGI0, SGI0, SGI85593, SGI85593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85594, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19069[] = {SGI85595, SGI85596, SGI85597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85598, SGI0, SGI0, SGI0, SGI85599, SGI85599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85600, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19070[] = {SGI85601, SGI85602, SGI85603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85604, SGI0, SGI0, SGI0, SGI85605, SGI85605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85606, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19071[] = {SGI85607, SGI85608, SGI85609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85610, SGI0, SGI0, SGI0, SGI85611, SGI85611, SGI0, SGI85612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19072[] = {SGI85613, SGI85614, SGI85615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85616, SGI85616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19073[] = {SGI85617, SGI85618, SGI85619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85620, SGI0, SGI0, SGI0, SGI85621, SGI85621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19074[] = {SGI85622, SGI85623, SGI85624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85625, SGI0, SGI0, SGI0, SGI85626, SGI85626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19075[] = {SGI85627, SGI0, SGI85628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19076[] = {SGI85629, SGI85630, SGI85631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85632, SGI0, SGI0, SGI0, SGI85633, SGI85633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19077[] = {SGI85634, SGI85635, SGI85636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85637, SGI0, SGI0, SGI0, SGI85638, SGI85638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19078[] = {SGI85639, SGI85640, SGI85641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85642, SGI0, SGI0, SGI0, SGI85643, SGI85643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19079[] = {SGI85644, SGI85645, SGI85646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85647, SGI0, SGI0, SGI0, SGI85648, SGI85648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19080[] = {SGI85649, SGI85650, SGI85651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85652, SGI0, SGI0, SGI0, SGI85653, SGI85653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19081[] = {SGI85654, SGI85655, SGI85656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85657, SGI0, SGI0, SGI0, SGI85658, SGI85658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19082[] = {SGI85659, SGI0, SGI85660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19083[] = {SGI85661, SGI85662, SGI85663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85664, SGI0, SGI0, SGI0, SGI85665, SGI85665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19084[] = {SGI85666, SGI85667, SGI85668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85669, SGI0, SGI0, SGI0, SGI85670, SGI85670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19085[] = {SGI85671, SGI85672, SGI85673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85674, SGI0, SGI0, SGI0, SGI85675, SGI85675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19086[] = {SGI85676, SGI85677, SGI85678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85679, SGI0, SGI0, SGI0, SGI85680, SGI85680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19087[] = {SGI85681, SGI0, SGI85682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85683, SGI85684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19088[] = {SGI85685, SGI0, SGI85686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85687, SGI85688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19089[] = {SGI85689, SGI85690, SGI85691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85692, SGI0, SGI0, SGI0, SGI85693, SGI85693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19090[] = {SGI85694, SGI85695, SGI85696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85697, SGI0, SGI0, SGI0, SGI85698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19091[] = {SGI85699, SGI85700, SGI85701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85702, SGI0, SGI0, SGI0, SGI85703, SGI85703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19092[] = {SGI85704, SGI0, SGI85705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19093[] = {SGI85706, SGI85707, SGI85708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85709, SGI85709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19094[] = {SGI85710, SGI0, SGI85711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85712, SGI0, SGI0, SGI0, SGI85713, SGI85713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19095[] = {SGI85714, SGI85715, SGI85716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85717, SGI85717, SGI0, SGI0, SGI85718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19096[] = {SGI85719, SGI0, SGI85720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85721, SGI0, SGI0, SGI0, SGI85722, SGI85722, SGI0, SGI0, SGI85723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19097[] = {SGI85724, SGI0, SGI85725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85726, SGI0, SGI0, SGI0, SGI85727, SGI85727, SGI0, SGI0, SGI85728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19098[] = {SGI85729, SGI85730, SGI85731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85732, SGI0, SGI0, SGI0, SGI85733, SGI85733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19099[] = {SGI85734, SGI0, SGI85735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19100[] = {SGI85736, SGI85737, SGI85738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85739, SGI0, SGI0, SGI0, SGI85740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19101[] = {SGI85741, SGI85742, SGI85743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85744, SGI0, SGI0, SGI0, SGI85745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19102[] = {SGI85746, SGI85747, SGI85748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85749, SGI0, SGI0, SGI0, SGI85750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19103[] = {SGI85751, SGI85752, SGI85753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85754, SGI0, SGI0, SGI0, SGI85755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19104[] = {SGI85756, SGI85757, SGI85758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85759, SGI0, SGI0, SGI0, SGI85760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19105[] = {SGI85761, SGI0, SGI85762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19106[] = {SGI85763, SGI85764, SGI85765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85766, SGI0, SGI0, SGI0, SGI85767, SGI85767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19107[] = {SGI85768, SGI85769, SGI85770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85771, SGI0, SGI0, SGI0, SGI85772, SGI85772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19108[] = {SGI85773, SGI0, SGI85774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19109[] = {SGI85775, SGI85776, SGI85777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85778, SGI85778, SGI0, SGI0, SGI85779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19110[] = {SGI85780, SGI0, SGI85781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85782, SGI85783, SGI85783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19111[] = {SGI85784, SGI85785, SGI85786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85787, SGI0, SGI0, SGI0, SGI85788, SGI85788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19112[] = {SGI85789, SGI85790, SGI85791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85792, SGI0, SGI0, SGI0, SGI85793, SGI85793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19113[] = {SGI85794, SGI85795, SGI85796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85797, SGI0, SGI0, SGI0, SGI85798, SGI85798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19114[] = {SGI85799, SGI85800, SGI85801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85802, SGI0, SGI0, SGI0, SGI85803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19115[] = {SGI85804, SGI85805, SGI85806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85807, SGI0, SGI0, SGI0, SGI85808, SGI85808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19116[] = {SGI85809, SGI85810, SGI85811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85812, SGI0, SGI0, SGI0, SGI85813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19117[] = {SGI85814, SGI85815, SGI85816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85817, SGI0, SGI0, SGI0, SGI85819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19118[] = {SGI85818, SGI85820, SGI85821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85822, SGI0, SGI0, SGI0, SGI85823, SGI85823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19119[] = {SGI85824, SGI85825, SGI85826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85827, SGI0, SGI0, SGI0, SGI85828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19120[] = {SGI85829, SGI85830, SGI85831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85832, SGI0, SGI0, SGI0, SGI85833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19121[] = {SGI85834, SGI85835, SGI85836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85837, SGI0, SGI0, SGI0, SGI85838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19122[] = {SGI85842, SGI85843, SGI85844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85846, SGI0, SGI0, SGI0, SGI85849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19123[] = {SGI85839, SGI85840, SGI85841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85845, SGI0, SGI0, SGI0, SGI85847, SGI85847, SGI85848, SGI0, SGI85850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19124[] = {SGI85851, SGI85852, SGI85853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85857, SGI0, SGI0, SGI0, SGI85858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19125[] = {SGI85854, SGI85855, SGI85856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85859, SGI0, SGI0, SGI0, SGI85860, SGI85860, SGI85861, SGI0, SGI85862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19126[] = {SGI85863, SGI85864, SGI85865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85866, SGI0, SGI0, SGI0, SGI85868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19127[] = {SGI85867, SGI85869, SGI85870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85871, SGI0, SGI0, SGI0, SGI85872, SGI85872, SGI85873, SGI0, SGI85874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19128[] = {SGI85875, SGI85876, SGI85877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85878, SGI0, SGI0, SGI0, SGI85879, SGI85879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19129[] = {SGI85880, SGI85881, SGI85882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85883, SGI0, SGI0, SGI0, SGI85884, SGI85884, SGI85885, SGI0, SGI85886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19130[] = {SGI85887, SGI85888, SGI85889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85890, SGI0, SGI0, SGI0, SGI85891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19131[] = {SGI85892, SGI85893, SGI85894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85895, SGI0, SGI0, SGI0, SGI85896, SGI85896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19132[] = {SGI85897, SGI85898, SGI85899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85900, SGI0, SGI0, SGI0, SGI85901, SGI85901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19133[] = {SGI85902, SGI85903, SGI85904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85905, SGI0, SGI0, SGI0, SGI85906, SGI85906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19134[] = {SGI85907, SGI85908, SGI85909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85910, SGI0, SGI0, SGI0, SGI85911, SGI85911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19135[] = {SGI85912, SGI0, SGI85913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85914, SGI0, SGI0, SGI0, SGI85915, SGI85915, SGI0, SGI0, SGI85916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19136[] = {SGI85917, SGI85918, SGI85919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85920, SGI0, SGI0, SGI0, SGI85921, SGI85922, SGI0, SGI0, SGI85923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19137[] = {SGI85924, SGI0, SGI85925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85926, SGI0, SGI0, SGI0, SGI85927, SGI85927, SGI0, SGI0, SGI85928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19138[] = {SGI85929, SGI85930, SGI85931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85932, SGI85933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19139[] = {SGI0, SGI85934, SGI85935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85936, SGI0, SGI0, SGI0, SGI85937, SGI85937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19140[] = {SGI85938, SGI85939, SGI85940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85941, SGI0, SGI0, SGI0, SGI85942, SGI85942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19141[] = {SGI85943, SGI85944, SGI85945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85946, SGI85946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19142[] = {SGI85947, SGI0, SGI85948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85949, SGI85950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19143[] = {SGI85951, SGI0, SGI85952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85953, SGI85954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19144[] = {SGI85955, SGI0, SGI85956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85957, SGI85958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19145[] = {SGI85959, SGI0, SGI85960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85964, SGI85966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19146[] = {SGI85961, SGI85962, SGI85963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85965, SGI0, SGI0, SGI0, SGI85967, SGI85967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19147[] = {SGI85968, SGI0, SGI85969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85970, SGI0, SGI0, SGI0, SGI85971, SGI85971, SGI0, SGI0, SGI85972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19148[] = {SGI85973, SGI85974, SGI85975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85976, SGI0, SGI0, SGI0, SGI85977, SGI85977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19149[] = {SGI85978, SGI85979, SGI85980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85981, SGI0, SGI0, SGI0, SGI85982, SGI85982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19150[] = {SGI85983, SGI85984, SGI85985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85986, SGI0, SGI0, SGI0, SGI85987, SGI85987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19151[] = {SGI85988, SGI85989, SGI85990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85991, SGI0, SGI0, SGI0, SGI85992, SGI85992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19152[] = {SGI85993, SGI85994, SGI85995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI85996, SGI0, SGI0, SGI0, SGI85997, SGI85997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19153[] = {SGI85998, SGI85999, SGI86000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86001, SGI0, SGI0, SGI0, SGI86002, SGI86002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19154[] = {SGI86003, SGI0, SGI86004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19155[] = {SGI86005, SGI86006, SGI86007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86009, SGI0, SGI0, SGI0, SGI86012, SGI86012, SGI0, SGI0, NULL};
const Segment * const SG19156[] = {SGI86008, SGI86010, SGI86011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86013, SGI0, SGI0, SGI0, SGI86014, SGI86014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19157[] = {SGI86015, SGI86016, SGI86017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86018, SGI0, SGI0, SGI0, SGI86019, SGI86019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19158[] = {SGI86020, SGI86021, SGI86022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86023, SGI0, SGI0, SGI0, SGI86024, SGI86024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19159[] = {SGI86025, SGI0, SGI86026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86027, SGI86028, SGI86028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19160[] = {SGI86029, SGI0, SGI86030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86031, SGI86032, SGI86032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19161[] = {SGI86033, SGI86034, SGI86035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86036, SGI0, SGI0, SGI0, SGI86037, SGI86037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19162[] = {SGI86038, SGI86039, SGI86040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86041, SGI0, SGI0, SGI0, SGI86042, SGI86042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19163[] = {SGI86043, SGI0, SGI86044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86045, SGI86046, SGI86046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19164[] = {SGI86047, SGI0, SGI86048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86052, SGI86053, SGI86053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19165[] = {SGI86049, SGI86050, SGI86051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86054, SGI0, SGI0, SGI0, SGI86055, SGI86055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19166[] = {SGI86056, SGI86058, SGI86059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86061, SGI0, SGI0, SGI0, SGI86063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19167[] = {SGI86057, SGI0, SGI86060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86062, SGI0, SGI0, SGI0, SGI86064, SGI86064, SGI0, SGI0, SGI86065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19168[] = {SGI86066, SGI86067, SGI86068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86069, SGI0, SGI0, SGI0, SGI86070, SGI86070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19169[] = {SGI86071, SGI86072, SGI86073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86074, SGI0, SGI0, SGI0, SGI86075, SGI86075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19170[] = {SGI86076, SGI86077, SGI86078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86079, SGI0, SGI0, SGI0, SGI86080, SGI86080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19171[] = {SGI86081, SGI0, SGI86082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19172[] = {SGI86083, SGI86084, SGI86085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86086, SGI0, SGI0, SGI0, SGI86087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19173[] = {SGI86088, SGI86089, SGI86090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86091, SGI0, SGI0, SGI0, SGI86092, SGI86092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19174[] = {SGI86093, SGI0, SGI86094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86095, SGI0, SGI0, SGI0, SGI86096, SGI86096, SGI0, SGI0, SGI86097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19175[] = {SGI86098, SGI0, SGI86099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86100, SGI0, SGI0, SGI0, SGI86101, SGI86101, SGI0, SGI0, SGI86102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19176[] = {SGI86103, SGI86104, SGI86105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86106, SGI0, SGI0, SGI0, SGI86107, SGI86107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19177[] = {SGI86108, SGI86109, SGI86110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86111, SGI0, SGI0, SGI0, SGI86112, SGI86112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19178[] = {SGI86113, SGI86114, SGI86115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86116, SGI0, SGI0, SGI0, SGI86117, SGI86117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19179[] = {SGI86118, SGI86119, SGI86120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86121, SGI0, SGI0, SGI0, SGI86122, SGI86122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19180[] = {SGI86123, SGI86124, SGI86125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86126, SGI0, SGI0, SGI0, SGI86127, SGI86127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19181[] = {SGI86128, SGI0, SGI86129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86130, SGI0, SGI0, SGI0, SGI86131, SGI86131, SGI0, SGI0, SGI86132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19182[] = {SGI86133, SGI86134, SGI86135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86136, SGI0, SGI0, SGI0, SGI86137, SGI86137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19183[] = {SGI86138, SGI86139, SGI86140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86141, SGI0, SGI0, SGI0, SGI86142, SGI86142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19184[] = {SGI86143, SGI86144, SGI86145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86146, SGI0, SGI0, SGI0, SGI86147, SGI86147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19185[] = {SGI86148, SGI0, SGI86149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19186[] = {SGI86150, SGI86151, SGI86152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86153, SGI0, SGI0, SGI0, SGI86154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19187[] = {SGI86155, SGI0, SGI86156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19188[] = {SGI86157, SGI0, SGI86158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86159, SGI86160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19189[] = {SGI86161, SGI0, SGI86162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86163, SGI86164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19190[] = {SGI86165, SGI86166, SGI86167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86168, SGI0, SGI0, SGI0, SGI86169, SGI86169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19191[] = {SGI86170, SGI86171, SGI86172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86173, SGI0, SGI0, SGI0, SGI86174, SGI86174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19192[] = {SGI86175, SGI86176, SGI86177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86178, SGI0, SGI0, SGI0, SGI86179, SGI86179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19193[] = {SGI86180, SGI0, SGI86181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86182, SGI86183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19194[] = {SGI86184, SGI0, SGI86185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86186, SGI86187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19195[] = {SGI86188, SGI86189, SGI86190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86191, SGI0, SGI0, SGI0, SGI86192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19196[] = {SGI86193, SGI0, SGI86194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19197[] = {SGI86195, SGI0, SGI86196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86197, SGI86198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19198[] = {SGI86199, SGI0, SGI86200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86201, SGI86202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19199[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86203, SGI0, SGI0, SGI86204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19200[] = {SGI86205, SGI0, SGI86206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86207, SGI86208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19201[] = {SGI86209, SGI86210, SGI86211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86212, SGI0, SGI0, SGI0, SGI86213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19202[] = {SGI86214, SGI86215, SGI86216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86217, SGI0, SGI0, SGI0, SGI86218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19203[] = {SGI86219, SGI86220, SGI86221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86222, SGI0, SGI0, SGI0, SGI86223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19204[] = {SGI86224, SGI86225, SGI86226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86227, SGI0, SGI0, SGI0, SGI86228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19205[] = {SGI86229, SGI86230, SGI86231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86232, SGI0, SGI0, SGI0, SGI86233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19206[] = {SGI86234, SGI86235, SGI86236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86237, SGI0, SGI0, SGI0, SGI86238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19207[] = {SGI86239, SGI86240, SGI86241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86242, SGI0, SGI0, SGI0, SGI86243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19208[] = {SGI86244, SGI86245, SGI86246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86247, SGI0, SGI0, SGI0, SGI86248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19209[] = {SGI86249, SGI86250, SGI86251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86252, SGI0, SGI0, SGI0, SGI86253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19210[] = {SGI86254, SGI86255, SGI86256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86257, SGI0, SGI0, SGI0, SGI86258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19211[] = {SGI86259, SGI86260, SGI86261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86262, SGI0, SGI0, SGI0, SGI86263, SGI86263, SGI0, SGI86264, SGI0, NULL};
const Segment * const SG19212[] = {SGI0, SGI86265, SGI0, SGI86266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19213[] = {SGI0, SGI86267, SGI0, SGI86268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19214[] = {SGI86269, SGI86270, SGI86271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86272, SGI0, SGI0, SGI0, SGI86273, SGI86273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19215[] = {SGI86274, SGI86275, SGI86276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86277, SGI0, SGI0, SGI0, SGI86278, SGI86278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19216[] = {SGI86279, SGI86280, SGI86281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86282, SGI0, SGI0, SGI0, SGI86283, SGI86283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19217[] = {SGI86284, SGI86285, SGI86286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86287, SGI86287, SGI0, SGI0, SGI86288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19218[] = {SGI86289, SGI86290, SGI86291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86292, SGI86292, SGI0, SGI0, SGI86293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19219[] = {SGI86294, SGI86295, SGI86296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86297, SGI86297, SGI0, SGI0, SGI86298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19220[] = {SGI86299, SGI86300, SGI86301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86302, SGI0, SGI0, SGI0, SGI86303, SGI86303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86304, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19221[] = {SGI86305, SGI86306, SGI86307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86308, SGI0, SGI0, SGI0, SGI86309, SGI86309, SGI86310, SGI0, SGI86311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19222[] = {SGI86312, SGI86313, SGI86314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86315, SGI0, SGI0, SGI0, SGI86316, SGI86316, SGI86317, SGI0, SGI86318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19223[] = {SGI86319, SGI86320, SGI86321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86322, SGI0, SGI0, SGI0, SGI86323, SGI86323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86324, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19224[] = {SGI86325, SGI86326, SGI86327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86328, SGI0, SGI0, SGI0, SGI86329, SGI86329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19225[] = {SGI86330, SGI0, SGI86331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86332, SGI0, SGI0, SGI0, SGI86333, SGI86333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19226[] = {SGI86334, SGI0, SGI86335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86336, SGI86337, SGI86337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19227[] = {SGI86338, SGI0, SGI86339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86340, SGI86341, SGI86341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19228[] = {SGI86342, SGI0, SGI86343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86344, SGI86345, SGI86345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19229[] = {SGI86346, SGI0, SGI86347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86348, SGI86349, SGI86349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19230[] = {SGI0, SGI86350, SGI0, SGI86351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19231[] = {SGI0, SGI86352, SGI0, SGI86353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86354, SGI0, SGI0, SGI0, SGI86355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19232[] = {SGI86356, SGI0, SGI86357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86358, SGI0, SGI0, SGI0, SGI86359, SGI86359, SGI0, SGI0, SGI86360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19233[] = {SGI86361, SGI86362, SGI86363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86364, SGI0, SGI0, SGI0, SGI86365, SGI86366, SGI0, SGI0, SGI86367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19234[] = {SGI86368, SGI0, SGI86369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86370, SGI86371, SGI86371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19235[] = {SGI86372, SGI0, SGI86373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86374, SGI86375, SGI86375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19236[] = {SGI86376, SGI86377, SGI86378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86379, SGI0, SGI0, SGI0, SGI86380, SGI86380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19237[] = {SGI0, SGI86381, SGI86382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19238[] = {SGI86383, SGI86384, SGI86385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86386, SGI86386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19239[] = {SGI86387, SGI86388, SGI86389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86390, SGI0, SGI0, SGI0, SGI86394, SGI86394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19240[] = {SGI86391, SGI86392, SGI86393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86395, SGI0, SGI0, SGI0, SGI86396, SGI86396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19241[] = {SGI86397, SGI0, SGI86398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19242[] = {SGI86399, SGI0, SGI86400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19243[] = {SGI86401, SGI0, SGI86402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86403, SGI0, SGI0, SGI0, SGI86404, SGI86404, SGI0, SGI0, SGI86405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19244[] = {SGI86406, SGI0, SGI86407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86408, SGI86409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19245[] = {SGI86410, SGI0, SGI86411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86412, SGI86413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19246[] = {SGI86414, SGI86415, SGI86416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86417, SGI0, SGI0, SGI0, SGI86418, SGI86418, SGI86421, SGI0, SGI86423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19247[] = {SGI86419, SGI86420, SGI86422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86424, SGI0, SGI0, SGI0, SGI86425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19248[] = {SGI86430, SGI0, SGI86431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19249[] = {SGI86426, SGI86427, SGI86428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86429, SGI0, SGI0, SGI0, SGI86432, SGI86432, SGI86433, SGI0, SGI86434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19250[] = {SGI86435, SGI0, SGI86436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86440, SGI0, SGI0, SGI0, SGI86441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19251[] = {SGI86437, SGI86438, SGI86439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86442, SGI0, SGI0, SGI0, SGI86443, SGI86443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86444, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19252[] = {SGI86445, SGI86446, SGI86447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19253[] = {SGI86449, SGI86450, SGI86451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86452, SGI0, SGI0, SGI0, SGI86453, SGI86453, SGI86454, SGI0, SGI86455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19254[] = {SGI86456, SGI86457, SGI86458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86459, SGI0, SGI0, SGI0, SGI86460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19255[] = {SGI86461, SGI86462, SGI86463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86464, SGI0, SGI0, SGI0, SGI86465, SGI86465, SGI86466, SGI0, SGI86467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19256[] = {SGI86468, SGI86469, SGI86470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86471, SGI0, SGI0, SGI0, SGI86472, SGI86472, SGI86473, SGI0, SGI86474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19257[] = {SGI86475, SGI86476, SGI86477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86478, SGI0, SGI0, SGI0, SGI86479, SGI86479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86480, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19258[] = {SGI86481, SGI86482, SGI86483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86484, SGI0, SGI0, SGI0, SGI86485, SGI86485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86486, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19259[] = {SGI86487, SGI86488, SGI86489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86490, SGI0, SGI0, SGI0, SGI86491, SGI86491, SGI86492, SGI0, SGI86493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19260[] = {SGI86494, SGI86495, SGI86496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86497, SGI0, SGI0, SGI0, SGI86498, SGI86498, SGI86499, SGI0, SGI86500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19261[] = {SGI86501, SGI86502, SGI86503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86504, SGI0, SGI0, SGI0, SGI86505, SGI86505, SGI86506, SGI0, SGI86507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19262[] = {SGI86508, SGI0, SGI86509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86512, SGI86514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19263[] = {SGI86510, SGI86511, SGI86513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86515, SGI0, SGI0, SGI0, SGI86516, SGI86516, SGI86517, SGI0, SGI86518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19264[] = {SGI86519, SGI86520, SGI86521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86524, SGI0, SGI0, SGI0, SGI86525, SGI86525, SGI86526, SGI0, SGI86529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19265[] = {SGI86522, SGI0, SGI86523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86527, SGI86528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19266[] = {SGI86530, SGI86531, SGI86532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86533, SGI0, SGI0, SGI0, SGI86534, SGI86534, SGI86535, SGI0, SGI86536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19267[] = {SGI86537, SGI86538, SGI86539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86540, SGI0, SGI0, SGI0, SGI86541, SGI86541, SGI86542, SGI0, SGI86543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19268[] = {SGI86544, SGI86545, SGI86546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86547, SGI0, SGI0, SGI0, SGI86548, SGI86548, SGI0, SGI0, SGI86549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19269[] = {SGI86550, SGI86551, SGI86552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86553, SGI0, SGI0, SGI0, SGI86554, SGI86554, SGI86555, SGI0, SGI86556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19270[] = {SGI86557, SGI86558, SGI86559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86560, SGI0, SGI0, SGI0, SGI86561, SGI86561, SGI86562, SGI0, SGI86563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19271[] = {SGI86564, SGI86565, SGI86566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86567, SGI0, SGI0, SGI0, SGI86568, SGI86568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86569, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19272[] = {SGI86570, SGI86571, SGI86572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86573, SGI0, SGI0, SGI0, SGI86574, SGI86574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19273[] = {SGI86575, SGI86576, SGI86577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86578, SGI0, SGI0, SGI0, SGI86579, SGI86579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19274[] = {SGI86580, SGI0, SGI86581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19275[] = {SGI86582, SGI86583, SGI86584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86585, SGI86586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19276[] = {SGI86587, SGI86588, SGI86589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86590, SGI86590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19277[] = {SGI86591, SGI0, SGI86592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86593, SGI86594, SGI86594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19278[] = {SGI86595, SGI0, SGI86596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86597, SGI86598, SGI86598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19279[] = {SGI86599, SGI86600, SGI86601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86602, SGI0, SGI0, SGI0, SGI86603, SGI86603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19280[] = {SGI86604, SGI86605, SGI86606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86607, SGI0, SGI0, SGI0, SGI86608, SGI86608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19281[] = {SGI86609, SGI0, SGI86610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19282[] = {SGI86611, SGI0, SGI86612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19283[] = {SGI86613, SGI0, SGI86614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86615, SGI86616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19284[] = {SGI86617, SGI0, SGI86618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86619, SGI86620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19285[] = {SGI86621, SGI86622, SGI86623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86624, SGI0, SGI0, SGI0, SGI86625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19286[] = {SGI86626, SGI86627, SGI86628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86629, SGI0, SGI0, SGI0, SGI86630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19287[] = {SGI86631, SGI0, SGI86632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19288[] = {SGI86633, SGI86634, SGI86635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86636, SGI0, SGI0, SGI0, SGI86637, SGI86637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19289[] = {SGI86638, SGI86639, SGI86640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86641, SGI0, SGI0, SGI0, SGI86642, SGI86642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19290[] = {SGI86643, SGI86644, SGI86645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86646, SGI0, SGI0, SGI0, SGI86647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19291[] = {SGI86648, SGI86649, SGI86650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86651, SGI0, SGI0, SGI0, SGI86652, SGI86652, SGI86653, SGI0, SGI86654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19292[] = {SGI86655, SGI86656, SGI86657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86658, SGI0, SGI0, SGI0, SGI86659, SGI86659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19293[] = {SGI86660, SGI86661, SGI86662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86663, SGI0, SGI0, SGI0, SGI86664, SGI86664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19294[] = {SGI86665, SGI86666, SGI86667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86668, SGI86669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19295[] = {SGI86670, SGI86671, SGI86672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86673, SGI86674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19296[] = {SGI86675, SGI86676, SGI86677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86678, SGI86678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19297[] = {SGI86679, SGI86680, SGI86681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86682, SGI0, SGI0, SGI0, SGI86683, SGI86683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19298[] = {SGI86684, SGI86685, SGI86686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86687, SGI86688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19299[] = {SGI86689, SGI86690, SGI86691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86692, SGI86692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19300[] = {SGI0, SGI86693, SGI86694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19301[] = {SGI86695, SGI86696, SGI86697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86698, SGI0, SGI0, SGI0, SGI86699, SGI86699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19302[] = {SGI86700, SGI0, SGI86701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86702, SGI86703, SGI86703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19303[] = {SGI86704, SGI0, SGI86705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86706, SGI86707, SGI86707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19304[] = {SGI86708, SGI86709, SGI86710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86711, SGI0, SGI0, SGI0, SGI86712, SGI86712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19305[] = {SGI86713, SGI0, SGI86714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86715, SGI86716, SGI86716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19306[] = {SGI86717, SGI0, SGI86718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86719, SGI86720, SGI86720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19307[] = {SGI86721, SGI86722, SGI86723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86724, SGI0, SGI0, SGI0, SGI86725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19308[] = {SGI86726, SGI86727, SGI86728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86729, SGI0, SGI0, SGI0, SGI86730, SGI86730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19309[] = {SGI86731, SGI86732, SGI86733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86734, SGI86734, SGI0, SGI0, SGI86735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19310[] = {SGI86736, SGI86737, SGI86738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86739, SGI0, SGI0, SGI0, SGI86740, SGI86740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19311[] = {SGI86741, SGI0, SGI86742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86743, SGI86744, SGI86744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19312[] = {SGI86745, SGI86746, SGI86747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86748, SGI0, SGI0, SGI0, SGI86749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19313[] = {SGI86750, SGI86751, SGI86752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86753, SGI0, SGI0, SGI0, SGI86754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19314[] = {SGI86755, SGI86756, SGI86757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86758, SGI0, SGI0, SGI0, SGI86759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19315[] = {SGI86760, SGI0, SGI86761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19316[] = {SGI86762, SGI86763, SGI86764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86765, SGI0, SGI0, SGI0, SGI86766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19317[] = {SGI86767, SGI86768, SGI86769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86770, SGI0, SGI0, SGI0, SGI86771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19318[] = {SGI86772, SGI86773, SGI86774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86775, SGI0, SGI0, SGI0, SGI86776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19319[] = {SGI86777, SGI86778, SGI86779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86780, SGI0, SGI0, SGI0, SGI86781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19320[] = {SGI86782, SGI86783, SGI86784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86785, SGI0, SGI0, SGI0, SGI86786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19321[] = {SGI86787, SGI0, SGI86788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19322[] = {SGI0, SGI86789, SGI86790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19323[] = {SGI86791, SGI0, SGI86792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19324[] = {SGI86793, SGI0, SGI86794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86795, SGI86796, SGI86796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19325[] = {SGI86797, SGI0, SGI86798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86799, SGI86800, SGI86800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19326[] = {SGI86801, SGI86802, SGI86803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86804, SGI0, SGI0, SGI0, SGI86805, SGI86805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19327[] = {SGI86806, SGI0, SGI86807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86808, SGI86809, SGI86809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19328[] = {SGI86810, SGI0, SGI86811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86812, SGI86813, SGI86813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19329[] = {SGI86814, SGI0, SGI86815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86816, SGI86817, SGI86817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19330[] = {SGI86818, SGI0, SGI86819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86820, SGI86821, SGI86821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19331[] = {SGI86822, SGI0, SGI86823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86824, SGI0, SGI0, SGI0, SGI86825, SGI86825, SGI0, SGI0, SGI86826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19332[] = {SGI86827, SGI0, SGI86828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86829, SGI0, SGI0, SGI0, SGI86830, SGI86830, SGI0, SGI0, SGI86831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19333[] = {SGI86832, SGI0, SGI86833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86834, SGI86835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19334[] = {SGI86836, SGI0, SGI86837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86838, SGI86839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19335[] = {SGI86840, SGI0, SGI86841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86842, SGI86843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19336[] = {SGI86844, SGI0, SGI86845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86846, SGI86847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19337[] = {SGI86848, SGI0, SGI86849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86850, SGI86851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19338[] = {SGI86852, SGI0, SGI86853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86854, SGI86855, SGI86855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19339[] = {SGI86856, SGI0, SGI86857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86858, SGI0, SGI0, SGI0, SGI86859, SGI86859, SGI0, SGI0, SGI86860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19340[] = {SGI86861, SGI0, SGI86862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86863, SGI0, SGI0, SGI0, SGI86864, SGI86864, SGI0, SGI0, SGI86865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19341[] = {SGI86866, SGI0, SGI86867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86868, SGI86869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19342[] = {SGI86870, SGI86871, SGI86872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86873, SGI0, SGI0, SGI0, SGI86874, SGI86874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19343[] = {SGI0, SGI86875, SGI86876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86877, SGI86877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19344[] = {SGI86878, SGI0, SGI86879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19345[] = {SGI86880, SGI86881, SGI86882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86883, SGI86884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19346[] = {SGI86885, SGI86886, SGI86887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86888, SGI0, SGI0, SGI0, SGI86889, SGI86889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19347[] = {SGI86890, SGI86891, SGI86892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86893, SGI86893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19348[] = {SGI86894, SGI0, SGI86895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19349[] = {SGI0, SGI86896, SGI86897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19350[] = {SGI86898, SGI86899, SGI86900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86901, SGI0, SGI0, SGI0, SGI86902, SGI86902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19351[] = {SGI86903, SGI0, SGI86904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86905, SGI86906, SGI86906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19352[] = {SGI86907, SGI0, SGI86908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86909, SGI86910, SGI86910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19353[] = {SGI86911, SGI0, SGI86912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86913, SGI86914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19354[] = {SGI86915, SGI0, SGI86916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86917, SGI86918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19355[] = {SGI86919, SGI0, SGI86920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86921, SGI86922, SGI86922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19356[] = {SGI86923, SGI0, SGI86924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86925, SGI86926, SGI86926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19357[] = {SGI86927, SGI0, SGI86928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86929, SGI86930, SGI86930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19358[] = {SGI86931, SGI0, SGI86932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86933, SGI86934, SGI86934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19359[] = {SGI86935, SGI0, SGI86936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86937, SGI86938, SGI86938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19360[] = {SGI86939, SGI0, SGI86940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86941, SGI86942, SGI86942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19361[] = {SGI86943, SGI86944, SGI86945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86946, SGI0, SGI0, SGI0, SGI86947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19362[] = {SGI86948, SGI0, SGI86949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19363[] = {SGI86950, SGI86951, SGI86952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86953, SGI0, SGI0, SGI0, SGI86954, SGI86954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19364[] = {SGI0, SGI86955, SGI0, SGI86956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19365[] = {SGI0, SGI86957, SGI0, SGI86958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86959, SGI0, SGI0, SGI0, SGI86960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19366[] = {SGI86961, SGI0, SGI86962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86963, SGI0, SGI0, SGI0, SGI86964, SGI86964, SGI0, SGI0, SGI86965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19367[] = {SGI86966, SGI86967, SGI86968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86969, SGI0, SGI0, SGI0, SGI86970, SGI86970, SGI0, SGI0, NULL};
const Segment * const SG19368[] = {SGI86971, SGI86972, SGI86973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86974, SGI0, SGI0, SGI0, SGI86975, SGI86975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19369[] = {SGI86976, SGI86977, SGI86978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86979, SGI0, SGI0, SGI0, SGI86980, SGI86980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19370[] = {SGI86981, SGI86982, SGI86983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86984, SGI0, SGI0, SGI0, SGI86985, SGI86985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19371[] = {SGI86986, SGI86987, SGI86988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86989, SGI86990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19372[] = {SGI86991, SGI86992, SGI86993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86994, SGI86994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19373[] = {SGI86995, SGI86996, SGI86997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI86998, SGI0, SGI0, SGI0, SGI86999, SGI86999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19374[] = {SGI87000, SGI0, SGI87001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19375[] = {SGI87002, SGI87003, SGI87004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87005, SGI87005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19376[] = {SGI87006, SGI87007, SGI87008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87009, SGI0, SGI0, SGI0, SGI87010, SGI87010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19377[] = {SGI87011, SGI87012, SGI87013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87014, SGI87014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19378[] = {SGI87015, SGI0, SGI87016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19379[] = {SGI87017, SGI87018, SGI87019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87020, SGI0, SGI0, SGI0, SGI87021, SGI87021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19380[] = {SGI87022, SGI87023, SGI87024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87025, SGI0, SGI0, SGI0, SGI87026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19381[] = {SGI87027, SGI0, SGI87028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19382[] = {SGI87029, SGI0, SGI87030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87031, SGI87032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19383[] = {SGI87033, SGI0, SGI87034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87035, SGI87036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19384[] = {SGI87037, SGI0, SGI87039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87042, SGI87044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19385[] = {SGI87038, SGI87040, SGI87041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87043, SGI0, SGI0, SGI0, SGI87045, SGI87045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19386[] = {SGI87046, SGI0, SGI87047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87048, SGI87049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19387[] = {SGI87050, SGI87051, SGI87052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87053, SGI0, SGI0, SGI0, SGI87054, SGI87054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19388[] = {SGI87055, SGI0, SGI87056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87057, SGI87058, SGI87058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19389[] = {SGI87059, SGI0, SGI87060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87061, SGI87062, SGI87062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19390[] = {SGI87063, SGI0, SGI87064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87065, SGI87066, SGI87066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19391[] = {SGI87067, SGI87068, SGI87069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87070, SGI0, SGI0, SGI0, SGI87071, SGI87071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19392[] = {SGI87072, SGI87073, SGI87074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87076, SGI0, SGI0, SGI0, SGI87078, SGI87078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19393[] = {SGI87075, SGI0, SGI87077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87079, SGI87080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19394[] = {SGI87081, SGI87082, SGI87083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87084, SGI0, SGI0, SGI0, SGI87085, SGI87085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19395[] = {SGI87086, SGI0, SGI87087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87088, SGI87089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19396[] = {SGI87090, SGI87091, SGI87092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87093, SGI0, SGI0, SGI0, SGI87094, SGI87094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19397[] = {SGI87095, SGI87096, SGI87097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87098, SGI0, SGI0, SGI0, SGI87099, SGI87099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19398[] = {SGI87100, SGI87101, SGI87102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87103, SGI0, SGI0, SGI0, SGI87104, SGI87104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19399[] = {SGI87105, SGI87106, SGI87107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87108, SGI0, SGI0, SGI0, SGI87109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19400[] = {SGI87110, SGI0, SGI87111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87112, SGI87113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19401[] = {SGI87114, SGI87115, SGI87116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87117, SGI0, SGI0, SGI0, SGI87118, SGI87118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19402[] = {SGI87119, SGI87120, SGI87121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87122, SGI87122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19403[] = {SGI87123, SGI87124, SGI87125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87126, SGI87127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19404[] = {SGI87128, SGI87129, SGI87130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87131, SGI0, SGI0, SGI0, SGI87132, SGI87132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19405[] = {SGI0, SGI87133, SGI87134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19406[] = {SGI87135, SGI0, SGI87136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19407[] = {SGI87137, SGI87138, SGI87139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87140, SGI0, SGI0, SGI0, SGI87141, SGI87141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19408[] = {SGI87142, SGI0, SGI87143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19409[] = {SGI87144, SGI87145, SGI87146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87147, SGI0, SGI0, SGI0, SGI87148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19410[] = {SGI87149, SGI0, SGI87150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19411[] = {SGI87151, SGI87152, SGI87153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87154, SGI0, SGI0, SGI0, SGI87155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19412[] = {SGI87156, SGI0, SGI87157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19413[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87158, SGI0, SGI0, SGI87159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19414[] = {SGI87160, SGI87161, SGI87162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87163, SGI0, SGI0, SGI0, SGI87164, SGI87164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19415[] = {SGI87165, SGI87166, SGI87167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87168, SGI0, SGI0, SGI0, SGI87169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19416[] = {SGI87170, SGI87171, SGI87172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87173, SGI0, SGI0, SGI0, SGI87174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19417[] = {SGI0, SGI87175, SGI87176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87177, SGI0, SGI0, SGI0, SGI87178, SGI87178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19418[] = {SGI87179, SGI87180, SGI87181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87182, SGI87182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19419[] = {SGI87183, SGI87184, SGI87185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87186, SGI0, SGI0, SGI0, SGI87187, SGI87187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19420[] = {SGI87188, SGI87189, SGI87190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87191, SGI0, SGI0, SGI0, SGI87192, SGI87192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19421[] = {SGI87193, SGI87194, SGI87195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87196, SGI87197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19422[] = {SGI87198, SGI87199, SGI87200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87201, SGI87201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19423[] = {SGI87202, SGI0, SGI87203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19424[] = {SGI0, SGI0, SGI87204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87205, SGI87206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19425[] = {SGI87207, SGI87208, SGI87209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87210, SGI0, SGI0, SGI0, SGI87211, SGI87211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19426[] = {SGI87212, SGI87213, SGI87214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87215, SGI0, SGI0, SGI0, SGI87216, SGI87216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19427[] = {SGI87217, SGI87218, SGI87219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87220, SGI87220, SGI0, SGI0, SGI87221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19428[] = {SGI87222, SGI87223, SGI87224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87225, SGI0, SGI0, SGI0, SGI87226, SGI87226, SGI87227, SGI0, SGI87228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19429[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87229, SGI0, SGI87230, SGI0, NULL};
const Segment * const SG19430[] = {SGI87231, SGI87232, SGI87233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87234, SGI0, SGI0, SGI0, SGI87235, SGI87235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19431[] = {SGI87236, SGI0, SGI87237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87238, SGI87239, SGI87239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19432[] = {SGI87240, SGI0, SGI87241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87242, SGI87243, SGI87243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19433[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87244, SGI0, SGI0, SGI87245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19434[] = {SGI87246, SGI87247, SGI87248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87249, SGI0, SGI0, SGI0, SGI87250, SGI87250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19435[] = {SGI87251, SGI87252, SGI87253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87254, SGI0, SGI0, SGI0, SGI87255, SGI87255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19436[] = {SGI87256, SGI87257, SGI87258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87259, SGI0, SGI0, SGI0, SGI87260, SGI87260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19437[] = {SGI87261, SGI87262, SGI87263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87266, SGI0, SGI0, SGI0, SGI87267, SGI87267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19438[] = {SGI87264, SGI0, SGI87265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87268, SGI0, SGI0, SGI0, SGI87269, SGI87269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19439[] = {SGI87272, SGI0, SGI87274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19440[] = {SGI87270, SGI87271, SGI87273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87275, SGI0, SGI0, SGI0, SGI87276, SGI87276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19441[] = {SGI87277, SGI87278, SGI87279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87280, SGI0, SGI0, SGI0, SGI87281, SGI87281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19442[] = {SGI87282, SGI0, SGI87283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87284, SGI0, SGI0, SGI0, SGI87285, SGI87285, SGI0, SGI0, SGI87286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19443[] = {SGI0, SGI0, SGI87287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87288, SGI87289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19444[] = {SGI87290, SGI87291, SGI87292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87293, SGI87293, SGI0, SGI0, SGI87294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19445[] = {SGI87295, SGI87296, SGI87297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87298, SGI0, SGI0, SGI0, SGI87299, SGI87299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19446[] = {SGI87300, SGI87301, SGI87302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87305, SGI0, SGI0, SGI0, SGI87306, SGI87306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19447[] = {SGI87303, SGI0, SGI87304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87307, SGI87308, SGI87308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19448[] = {SGI87309, SGI87310, SGI87311, SGI0, SGI0, SGI0, SGI87312, SGI87313, SGI0, SGI87314, SGI0, SGI0, SGI0, SGI87315, SGI87315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19449[] = {SGI87316, SGI87317, SGI87318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87319, SGI0, SGI0, SGI0, SGI87320, SGI87320, SGI87321, SGI0, SGI87322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19450[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87323, SGI0, SGI0, SGI87324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87325, SGI0, NULL};
const Segment * const SG19451[] = {SGI87326, SGI0, SGI87327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87328, SGI87329, SGI87329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19452[] = {SGI87330, SGI0, SGI87331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87332, SGI87334, SGI87334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19453[] = {SGI87333, SGI87335, SGI87336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87337, SGI0, SGI0, SGI0, SGI87338, SGI87338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19454[] = {SGI87339, SGI87340, SGI87341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87342, SGI87343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19455[] = {SGI87344, SGI87345, SGI87346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87347, SGI87347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19456[] = {SGI87348, SGI0, SGI87349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19457[] = {SGI87350, SGI87351, SGI87352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87353, SGI87353, SGI0, SGI0, SGI87354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19458[] = {SGI87355, SGI87356, SGI87357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87358, SGI0, SGI0, SGI0, SGI87359, SGI87359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19459[] = {SGI87360, SGI87361, SGI87362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87363, SGI0, SGI0, SGI0, SGI87364, SGI87364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19460[] = {SGI87365, SGI87366, SGI87367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87368, SGI0, SGI0, SGI0, SGI87369, SGI87369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19461[] = {SGI87370, SGI87371, SGI87372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87373, SGI0, SGI0, SGI0, SGI87374, SGI87374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19462[] = {SGI87375, SGI87376, SGI87377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87378, SGI0, SGI0, SGI0, SGI87379, SGI87379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19463[] = {SGI87380, SGI87381, SGI87382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87383, SGI87383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19464[] = {SGI87384, SGI87385, SGI87386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87387, SGI87388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19465[] = {SGI87389, SGI0, SGI87390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19466[] = {SGI87391, SGI87392, SGI87393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87394, SGI0, SGI0, SGI0, SGI87395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19467[] = {SGI87396, SGI0, SGI87397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19468[] = {SGI87398, SGI87399, SGI87400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87401, SGI0, SGI0, SGI0, SGI87402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19469[] = {SGI87403, SGI0, SGI87404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87405, SGI0, SGI0, SGI0, SGI87406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19470[] = {SGI87407, SGI0, SGI87408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87409, SGI87410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19471[] = {SGI87411, SGI0, SGI87412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87413, SGI87414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19472[] = {SGI87415, SGI87416, SGI87417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87418, SGI0, SGI0, SGI0, SGI87419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19473[] = {SGI87420, SGI87421, SGI87422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87423, SGI0, SGI0, SGI0, SGI87424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19474[] = {SGI87425, SGI87426, SGI87427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87428, SGI0, SGI0, SGI0, SGI87429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19475[] = {SGI87430, SGI87431, SGI87432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87433, SGI0, SGI0, SGI0, SGI87434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19476[] = {SGI87435, SGI87436, SGI87437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87438, SGI87438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19477[] = {SGI87439, SGI87440, SGI87441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87442, SGI0, SGI0, SGI0, SGI87443, SGI87443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19478[] = {SGI87444, SGI0, SGI87445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19479[] = {SGI87446, SGI87447, SGI87448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87449, SGI0, SGI0, SGI0, SGI87450, SGI87450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19480[] = {SGI87451, SGI0, SGI87452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87453, SGI87454, SGI87454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19481[] = {SGI87455, SGI0, SGI87456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87457, SGI87458, SGI87458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19482[] = {SGI87459, SGI0, SGI87460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87461, SGI87462, SGI87462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19483[] = {SGI87463, SGI0, SGI87464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87465, SGI87466, SGI87466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19484[] = {SGI87467, SGI0, SGI87468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87469, SGI87470, SGI87470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19485[] = {SGI87471, SGI0, SGI87472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87473, SGI87474, SGI87474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19486[] = {SGI87475, SGI0, SGI87476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87477, SGI87478, SGI87478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19487[] = {SGI87479, SGI87480, SGI87481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87482, SGI0, SGI0, SGI0, SGI87483, SGI87483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19488[] = {SGI87484, SGI87485, SGI87486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87487, SGI0, SGI0, SGI0, SGI87488, SGI87488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19489[] = {SGI87489, SGI87490, SGI87491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87492, SGI0, SGI0, SGI0, SGI87493, SGI87493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19490[] = {SGI87494, SGI0, SGI87495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87496, SGI87497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19491[] = {SGI87498, SGI0, SGI87499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87500, SGI87501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19492[] = {SGI87502, SGI87503, SGI87504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87505, SGI0, SGI0, SGI0, SGI87506, SGI87506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19493[] = {SGI87507, SGI0, SGI87508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19494[] = {SGI87509, SGI0, SGI87510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87511, SGI87512, SGI87512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19495[] = {SGI87513, SGI0, SGI87514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87515, SGI87516, SGI87516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19496[] = {SGI87517, SGI87518, SGI87519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87520, SGI0, SGI0, SGI0, SGI87521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19497[] = {SGI87522, SGI0, SGI87523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19498[] = {SGI87524, SGI87525, SGI87526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87527, SGI0, SGI0, SGI0, SGI87528, SGI87528, SGI87529, SGI0, SGI87530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19499[] = {SGI87531, SGI87532, SGI87533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87534, SGI0, SGI0, SGI0, SGI87535, SGI87535, SGI0, SGI0, SGI87536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19500[] = {SGI87537, SGI87538, SGI87539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87540, SGI0, SGI0, SGI0, SGI87541, SGI87541, SGI87542, SGI0, SGI87543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19501[] = {SGI87544, SGI87545, SGI87546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87547, SGI0, SGI0, SGI0, SGI87548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19502[] = {SGI87549, SGI0, SGI87550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19503[] = {SGI87551, SGI87552, SGI87553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87554, SGI0, SGI0, SGI0, SGI87555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19504[] = {SGI87556, SGI0, SGI87557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19505[] = {SGI87558, SGI0, SGI87559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87560, SGI87561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19506[] = {SGI0, SGI87567, SGI0, SGI87568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19507[] = {SGI87569, SGI87570, SGI87571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87572, SGI0, SGI0, SGI0, SGI87573, SGI87573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87574, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19508[] = {SGI87575, SGI87576, SGI87577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87578, SGI0, SGI0, SGI0, SGI87579, SGI87579, SGI0, SGI87580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19509[] = {SGI87581, SGI0, SGI87582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87583, SGI87584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19510[] = {SGI87585, SGI0, SGI87586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87587, SGI87588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19511[] = {SGI87589, SGI0, SGI87590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87591, SGI87592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19512[] = {SGI87593, SGI0, SGI87594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87595, SGI87596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19513[] = {SGI87562, SGI87563, SGI87564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87565, SGI0, SGI0, SGI0, SGI87566, SGI87566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19514[] = {SGI87597, SGI0, SGI87598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87599, SGI0, SGI0, SGI0, SGI87600, SGI87600, SGI0, SGI0, SGI87602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19515[] = {SGI87601, SGI0, SGI87603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19516[] = {SGI87604, SGI87605, SGI87606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87607, SGI0, SGI0, SGI0, SGI87608, SGI87608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19517[] = {SGI87609, SGI87610, SGI87611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87612, SGI0, SGI0, SGI0, SGI87613, SGI87613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19518[] = {SGI87614, SGI0, SGI87615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19519[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87616, SGI0, SGI87617, SGI0, NULL};
const Segment * const SG19520[] = {SGI87618, SGI87619, SGI87620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87621, SGI0, SGI0, SGI0, SGI87622, SGI0, SGI0, SGI87623, SGI0, NULL};
const Segment * const SG19521[] = {SGI87624, SGI87625, SGI87626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87627, SGI0, SGI0, SGI0, SGI87628, SGI87628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19522[] = {SGI87629, SGI0, SGI87630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19523[] = {SGI87631, SGI87632, SGI87633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87634, SGI0, SGI0, SGI0, SGI87635, SGI87635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19524[] = {SGI87636, SGI87637, SGI87638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87639, SGI0, SGI0, SGI0, SGI87640, SGI87640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19525[] = {SGI87641, SGI87642, SGI87643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87646, SGI87648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19526[] = {SGI87644, SGI87645, SGI87647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87649, SGI0, SGI0, SGI0, SGI87650, SGI87650, SGI0, SGI0, NULL};
const Segment * const SG19527[] = {SGI87651, SGI87652, SGI87653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87654, SGI0, SGI0, SGI0, SGI87655, SGI87655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19528[] = {SGI87656, SGI87657, SGI87658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87659, SGI87659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19529[] = {SGI87660, SGI87661, SGI87662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87663, SGI0, SGI0, SGI0, SGI87664, SGI87664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19530[] = {SGI87665, SGI87666, SGI87667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87668, SGI0, SGI0, SGI0, SGI87669, SGI87669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19531[] = {SGI87670, SGI87671, SGI87672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87673, SGI0, SGI0, SGI0, SGI87674, SGI87674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19532[] = {SGI87675, SGI0, SGI87676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87677, SGI0, SGI0, SGI0, SGI87678, SGI87678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19533[] = {SGI87679, SGI0, SGI87680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19534[] = {SGI87681, SGI87682, SGI87683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87684, SGI87685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19535[] = {SGI87686, SGI87687, SGI87688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87689, SGI0, SGI0, SGI0, SGI87690, SGI87690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19536[] = {SGI87691, SGI87692, SGI87693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87694, SGI87694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19537[] = {SGI87695, SGI87696, SGI87697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87698, SGI0, SGI0, SGI0, SGI87699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19538[] = {SGI87700, SGI0, SGI87701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19539[] = {SGI87702, SGI0, SGI87703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87704, SGI87705, SGI87705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19540[] = {SGI87707, SGI87708, SGI87710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87711, SGI0, SGI0, SGI0, SGI87714, SGI87714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19541[] = {SGI87706, SGI0, SGI87709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87712, SGI87713, SGI87713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19542[] = {SGI87715, SGI0, SGI87716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87720, SGI87721, SGI87721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19543[] = {SGI87717, SGI87718, SGI87719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87722, SGI87722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19544[] = {SGI87723, SGI0, SGI87724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19545[] = {SGI87725, SGI87726, SGI87727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87728, SGI0, SGI0, SGI0, SGI87729, SGI87729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19546[] = {SGI87730, SGI0, SGI87731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19547[] = {SGI87732, SGI87733, SGI87734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87735, SGI87736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19548[] = {SGI87737, SGI87738, SGI87739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87740, SGI0, SGI0, SGI0, SGI87741, SGI87741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19549[] = {SGI87742, SGI0, SGI87743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19550[] = {SGI87744, SGI87745, SGI87746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87747, SGI87747, SGI0, SGI0, SGI87748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19551[] = {SGI87749, SGI87750, SGI87751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87752, SGI0, SGI0, SGI0, SGI87753, SGI87753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19552[] = {SGI87754, SGI87755, SGI87756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87757, SGI0, SGI0, SGI0, SGI87758, SGI87758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19553[] = {SGI87759, SGI87760, SGI87761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87762, SGI0, SGI0, SGI0, SGI87763, SGI87763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19554[] = {SGI87764, SGI87765, SGI87766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87767, SGI0, SGI0, SGI0, SGI87768, SGI87768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19555[] = {SGI87769, SGI87770, SGI87771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87772, SGI0, SGI0, SGI0, SGI87773, SGI87774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19556[] = {SGI87775, SGI0, SGI87776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19557[] = {SGI87777, SGI87778, SGI87779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87780, SGI0, SGI0, SGI0, SGI87781, SGI87781, SGI87782, SGI0, SGI87783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19558[] = {SGI87784, SGI87785, SGI87786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87787, SGI0, SGI0, SGI0, SGI87788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19559[] = {SGI87789, SGI87790, SGI87791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87792, SGI0, SGI0, SGI0, SGI87793, SGI87794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19560[] = {SGI87795, SGI0, SGI87796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19561[] = {SGI87797, SGI87798, SGI87799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87800, SGI87800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19562[] = {SGI87801, SGI87802, SGI87803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87804, SGI0, SGI0, SGI0, SGI87805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19563[] = {SGI87806, SGI87807, SGI87808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87809, SGI0, SGI0, SGI0, SGI87810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19564[] = {SGI87811, SGI87812, SGI87813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87814, SGI0, SGI0, SGI0, SGI87815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19565[] = {SGI87816, SGI87817, SGI87818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87819, SGI0, SGI0, SGI0, SGI87820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19566[] = {SGI87821, SGI87822, SGI87823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87824, SGI0, SGI0, SGI0, SGI87825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19567[] = {SGI87826, SGI87827, SGI87828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87829, SGI0, SGI0, SGI0, SGI87830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19568[] = {SGI87831, SGI87832, SGI87833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87834, SGI0, SGI0, SGI0, SGI87835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19569[] = {SGI87836, SGI87837, SGI87838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87839, SGI0, SGI0, SGI0, SGI87840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19570[] = {SGI87841, SGI87842, SGI87843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87844, SGI0, SGI0, SGI0, SGI87845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19571[] = {SGI87846, SGI87847, SGI87848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87849, SGI0, SGI0, SGI0, SGI87850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19572[] = {SGI87851, SGI87852, SGI87853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87854, SGI0, SGI0, SGI0, SGI87855, SGI87855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19573[] = {SGI87856, SGI87857, SGI87858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87859, SGI87859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19574[] = {SGI87860, SGI87861, SGI87862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87863, SGI0, SGI0, SGI0, SGI87864, SGI87864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19575[] = {SGI87865, SGI87866, SGI87867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87868, SGI0, SGI0, SGI0, SGI87869, SGI87869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19576[] = {SGI87870, SGI0, SGI87871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87872, SGI87873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19577[] = {SGI87874, SGI87875, SGI87876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87877, SGI0, SGI0, SGI0, SGI87878, SGI87878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19578[] = {SGI87879, SGI0, SGI87880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19579[] = {SGI87881, SGI87882, SGI87883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87884, SGI0, SGI0, SGI0, SGI87885, SGI87885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87886, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19580[] = {SGI87887, SGI87888, SGI87889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87890, SGI0, SGI0, SGI0, SGI87891, SGI87891, SGI87892, SGI0, SGI87893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19581[] = {SGI87894, SGI87895, SGI87896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87897, SGI0, SGI0, SGI0, SGI87898, SGI87898, SGI0, SGI87899, SGI87900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19582[] = {SGI87901, SGI87902, SGI87903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87904, SGI0, SGI0, SGI0, SGI87905, SGI87905, SGI0, SGI0, SGI87906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19583[] = {SGI87907, SGI87908, SGI87909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87910, SGI0, SGI0, SGI0, SGI87911, SGI87911, SGI0, SGI0, SGI87912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19584[] = {SGI87913, SGI87914, SGI87915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87916, SGI0, SGI0, SGI0, SGI87917, SGI87917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19585[] = {SGI87918, SGI87919, SGI87920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87921, SGI0, SGI0, SGI0, SGI87922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19586[] = {SGI87923, SGI87924, SGI87925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87926, SGI0, SGI0, SGI0, SGI87927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19587[] = {SGI87928, SGI87929, SGI87930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87931, SGI0, SGI0, SGI0, SGI87932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19588[] = {SGI87933, SGI87934, SGI87935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87936, SGI0, SGI0, SGI0, SGI87937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19589[] = {SGI87938, SGI87939, SGI87940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87941, SGI0, SGI0, SGI0, SGI87942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19590[] = {SGI87943, SGI87944, SGI87945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87946, SGI0, SGI0, SGI0, SGI87947, SGI87947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19591[] = {SGI87948, SGI87949, SGI87950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87951, SGI0, SGI0, SGI0, SGI87952, SGI87952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19592[] = {SGI87953, SGI87954, SGI87955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87956, SGI0, SGI0, SGI0, SGI87957, SGI87957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19593[] = {SGI87958, SGI0, SGI87959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87960, SGI87961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19594[] = {SGI87962, SGI87963, SGI87964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87965, SGI0, SGI0, SGI0, SGI87966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19595[] = {SGI87967, SGI0, SGI87968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19596[] = {SGI87969, SGI87970, SGI87971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87972, SGI0, SGI0, SGI0, SGI87973, SGI87973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19597[] = {SGI87974, SGI87975, SGI87976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87977, SGI0, SGI0, SGI0, SGI87978, SGI87978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19598[] = {SGI87979, SGI87980, SGI87981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87982, SGI0, SGI0, SGI0, SGI87983, SGI87983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19599[] = {SGI87984, SGI87985, SGI87986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87987, SGI0, SGI0, SGI0, SGI87988, SGI87988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19600[] = {SGI87989, SGI0, SGI87990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87991, SGI87992, SGI87992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19601[] = {SGI87993, SGI0, SGI87994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87995, SGI87996, SGI87996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19602[] = {SGI87997, SGI0, SGI87998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI87999, SGI88000, SGI88000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19603[] = {SGI88001, SGI88002, SGI88003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88004, SGI88004, SGI0, SGI0, SGI88005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19604[] = {SGI88006, SGI88007, SGI88008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88009, SGI0, SGI0, SGI0, SGI88010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19605[] = {SGI88011, SGI88012, SGI88013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88014, SGI0, SGI0, SGI0, SGI88015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19606[] = {SGI88016, SGI0, SGI88017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88018, SGI0, SGI0, SGI0, SGI88019, SGI88019, SGI0, SGI0, SGI88020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19607[] = {SGI88021, SGI0, SGI88022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88023, SGI0, SGI0, SGI0, SGI88024, SGI88024, SGI0, SGI0, SGI88025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19608[] = {SGI88026, SGI0, SGI88027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88028, SGI88029, SGI88029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19609[] = {SGI88030, SGI0, SGI88031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88032, SGI88033, SGI88033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19610[] = {SGI88034, SGI0, SGI88035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88036, SGI88037, SGI88037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19611[] = {SGI88038, SGI0, SGI88039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88040, SGI88041, SGI88041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19612[] = {SGI88042, SGI0, SGI88043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88044, SGI88045, SGI88045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19613[] = {SGI88046, SGI0, SGI88047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88048, SGI88049, SGI88049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19614[] = {SGI88050, SGI88051, SGI88052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88053, SGI0, SGI0, SGI0, SGI88054, SGI88054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88055, SGI0, NULL};
const Segment * const SG19615[] = {SGI88056, SGI0, SGI88057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88058, SGI88059, SGI88059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19616[] = {SGI88060, SGI0, SGI88061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88062, SGI88063, SGI88063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19617[] = {SGI88064, SGI0, SGI88065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88066, SGI88067, SGI88067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19618[] = {SGI88068, SGI0, SGI88069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88070, SGI88071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19619[] = {SGI88072, SGI88073, SGI88074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88075, SGI0, SGI0, SGI0, SGI88076, SGI88076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19620[] = {SGI88077, SGI88078, SGI88079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88080, SGI0, SGI0, SGI0, SGI88081, SGI88081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19621[] = {SGI88082, SGI0, SGI88083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19622[] = {SGI88084, SGI0, SGI88085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88086, SGI0, SGI0, SGI0, SGI88087, SGI88087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19623[] = {SGI88088, SGI88089, SGI88090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88091, SGI0, SGI0, SGI0, SGI88092, SGI88092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19624[] = {SGI88093, SGI88094, SGI88095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88096, SGI0, SGI0, SGI0, SGI88097, SGI88097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19625[] = {SGI88098, SGI88099, SGI88100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88101, SGI0, SGI0, SGI0, SGI88102, SGI88102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19626[] = {SGI88103, SGI88104, SGI88105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88106, SGI0, SGI0, SGI0, SGI88107, SGI88107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19627[] = {SGI88108, SGI0, SGI88109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88110, SGI88111, SGI88111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19628[] = {SGI88112, SGI0, SGI88113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88114, SGI88115, SGI88115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19629[] = {SGI88116, SGI0, SGI88117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88118, SGI88119, SGI88119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19630[] = {SGI88120, SGI0, SGI88121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88122, SGI88123, SGI88123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19631[] = {SGI88124, SGI0, SGI88125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88126, SGI88127, SGI88127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19632[] = {SGI88128, SGI88129, SGI88130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88131, SGI0, SGI0, SGI0, SGI88132, SGI88132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19633[] = {SGI88133, SGI88134, SGI88135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88136, SGI0, SGI0, SGI0, SGI88137, SGI88137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19634[] = {SGI88138, SGI0, SGI88139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19635[] = {SGI88140, SGI0, SGI88141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88142, SGI88143, SGI88143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19636[] = {SGI88144, SGI88145, SGI88146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88147, SGI0, SGI0, SGI0, SGI88148, SGI88148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19637[] = {SGI88149, SGI88150, SGI88151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88152, SGI0, SGI0, SGI0, SGI88153, SGI88153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19638[] = {SGI88154, SGI88155, SGI88156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88157, SGI0, SGI0, SGI0, SGI88158, SGI88158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19639[] = {SGI88159, SGI88160, SGI88161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88162, SGI0, SGI0, SGI0, SGI88163, SGI88163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19640[] = {SGI0, SGI88164, SGI0, SGI88165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19641[] = {SGI88166, SGI0, SGI88167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88168, SGI88169, SGI88169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19642[] = {SGI88170, SGI88171, SGI88172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88173, SGI0, SGI0, SGI0, SGI88174, SGI88174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19643[] = {SGI88175, SGI88176, SGI88177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88178, SGI88178, SGI0, SGI0, SGI88179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19644[] = {SGI88180, SGI0, SGI88181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88182, SGI88183, SGI88183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19645[] = {SGI88184, SGI88185, SGI88186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88187, SGI0, SGI0, SGI0, SGI88188, SGI88188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19646[] = {SGI88189, SGI0, SGI88190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88191, SGI88192, SGI88192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88193, NULL};
const Segment * const SG19647[] = {SGI88194, SGI88195, SGI88196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88197, SGI0, SGI0, SGI0, SGI88198, SGI88198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19648[] = {SGI88199, SGI88200, SGI88201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88202, SGI0, SGI0, SGI0, SGI88203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19649[] = {SGI88204, SGI0, SGI88205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19650[] = {SGI88206, SGI88207, SGI88208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88209, SGI0, SGI0, SGI0, SGI88210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19651[] = {SGI88211, SGI88212, SGI88213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88214, SGI0, SGI0, SGI0, SGI88215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19652[] = {SGI88216, SGI88217, SGI88218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88219, SGI0, SGI0, SGI0, SGI88220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19653[] = {SGI88221, SGI88222, SGI88223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88224, SGI0, SGI0, SGI0, SGI88225, SGI88225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19654[] = {SGI88226, SGI88227, SGI88228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88229, SGI0, SGI0, SGI0, SGI88230, SGI88230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19655[] = {SGI88231, SGI88232, SGI88233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88234, SGI0, SGI0, SGI0, SGI88235, SGI88235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19656[] = {SGI88236, SGI88237, SGI88238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88239, SGI0, SGI0, SGI0, SGI88240, SGI88240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19657[] = {SGI88241, SGI88242, SGI88243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88244, SGI0, SGI0, SGI0, SGI88245, SGI88245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19658[] = {SGI88246, SGI0, SGI88247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19659[] = {SGI88248, SGI88249, SGI88250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88251, SGI0, SGI0, SGI0, SGI88252, SGI88252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19660[] = {SGI88253, SGI88254, SGI88255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88256, SGI0, SGI0, SGI0, SGI88257, SGI88257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19661[] = {SGI88258, SGI0, SGI88259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88260, SGI88261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19662[] = {SGI88262, SGI88263, SGI88264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88265, SGI0, SGI0, SGI0, SGI88266, SGI88266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19663[] = {SGI0, SGI88267, SGI0, SGI88268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19664[] = {SGI88269, SGI88270, SGI88271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88272, SGI0, SGI0, SGI0, SGI88273, SGI88273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19665[] = {SGI0, SGI88274, SGI0, SGI88275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19666[] = {SGI88276, SGI88277, SGI88278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88279, SGI0, SGI0, SGI0, SGI88280, SGI88280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19667[] = {SGI88281, SGI88282, SGI88283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88284, SGI0, SGI0, SGI0, SGI88285, SGI88285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19668[] = {SGI88286, SGI88287, SGI88288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88289, SGI0, SGI0, SGI0, SGI88290, SGI88290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19669[] = {SGI88291, SGI88292, SGI88293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88294, SGI0, SGI0, SGI0, SGI88295, SGI88295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19670[] = {SGI88296, SGI88297, SGI88298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88299, SGI0, SGI0, SGI0, SGI88300, SGI88300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19671[] = {SGI88301, SGI88302, SGI88303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88304, SGI0, SGI0, SGI0, SGI88305, SGI88305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19672[] = {SGI88306, SGI88307, SGI88308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88309, SGI0, SGI0, SGI0, SGI88310, SGI88310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88311, SGI0, NULL};
const Segment * const SG19673[] = {SGI88312, SGI0, SGI88313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88314, SGI0, SGI0, SGI0, SGI88315, SGI88315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88316, SGI0, NULL};
const Segment * const SG19674[] = {SGI88317, SGI88318, SGI88319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88320, SGI0, SGI0, SGI0, SGI88321, SGI88321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19675[] = {SGI88322, SGI88323, SGI88324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88325, SGI0, SGI0, SGI0, SGI88326, SGI88326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19676[] = {SGI88327, SGI88328, SGI88329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88330, SGI88330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19677[] = {SGI88331, SGI88332, SGI88333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88334, SGI88335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19678[] = {SGI88336, SGI88337, SGI88338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88339, SGI0, SGI0, SGI0, SGI88340, SGI88340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19679[] = {SGI88341, SGI88342, SGI88343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88344, SGI0, SGI0, SGI0, SGI88345, SGI88345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19680[] = {SGI88346, SGI0, SGI88347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19681[] = {SGI88348, SGI88349, SGI88350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88351, SGI88351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19682[] = {SGI88352, SGI88353, SGI88354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88355, SGI0, SGI0, SGI0, SGI88356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19683[] = {SGI88357, SGI0, SGI88358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88359, SGI0, SGI0, SGI0, SGI88360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19684[] = {SGI88361, SGI88362, SGI88363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88364, SGI0, SGI0, SGI0, SGI88365, SGI88365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19685[] = {SGI88366, SGI0, SGI88367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88368, SGI88369, SGI88369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19686[] = {SGI88370, SGI0, SGI88371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88372, SGI88373, SGI88373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19687[] = {SGI88374, SGI0, SGI88375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88376, SGI88377, SGI88377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19688[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88379, SGI0, SGI0, SGI88381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19689[] = {SGI88378, SGI0, SGI88380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88382, SGI88383, SGI88383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19690[] = {SGI88384, SGI0, SGI88385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88386, SGI88387, SGI88387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19691[] = {SGI88388, SGI0, SGI88389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88390, SGI88391, SGI88391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19692[] = {SGI88392, SGI88393, SGI88394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88395, SGI88395, SGI0, SGI0, SGI88396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19693[] = {SGI88397, SGI0, SGI88398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88399, SGI88400, SGI88400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19694[] = {SGI88401, SGI0, SGI88402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88403, SGI88404, SGI88404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19695[] = {SGI88405, SGI88406, SGI88407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88408, SGI0, SGI0, SGI0, SGI88409, SGI88409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19696[] = {SGI88410, SGI88411, SGI88412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88413, SGI88413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19697[] = {SGI88414, SGI0, SGI88415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19698[] = {SGI88416, SGI88417, SGI88418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88419, SGI88420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19699[] = {SGI88421, SGI88422, SGI88423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88424, SGI0, SGI0, SGI0, SGI88425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19700[] = {SGI88426, SGI88427, SGI88429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88432, SGI0, SGI0, SGI0, SGI88433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19701[] = {SGI88428, SGI88430, SGI88431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88434, SGI0, SGI0, SGI0, SGI88435, SGI88435, SGI0, SGI0, SGI0, SGI88436, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19702[] = {SGI88437, SGI88438, SGI88439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88440, SGI0, SGI0, SGI0, SGI88441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19703[] = {SGI88442, SGI88443, SGI88444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88445, SGI0, SGI0, SGI0, SGI88446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19704[] = {SGI88447, SGI0, SGI88448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19705[] = {SGI88449, SGI88450, SGI88451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88452, SGI0, SGI0, SGI0, SGI88453, SGI88453, SGI0, SGI0, SGI0, SGI88454, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19706[] = {SGI88455, SGI88456, SGI88457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88458, SGI0, SGI0, SGI0, SGI88459, SGI88459, SGI0, SGI0, SGI0, SGI88460, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19707[] = {SGI88461, SGI0, SGI88462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88463, SGI0, SGI0, SGI0, SGI88464, SGI88464, SGI0, SGI0, SGI88465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19708[] = {SGI88466, SGI88467, SGI88468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88469, SGI0, SGI0, SGI0, SGI88470, SGI88471, SGI0, SGI0, SGI88472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19709[] = {SGI88473, SGI0, SGI88474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88475, SGI88476, SGI88476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19710[] = {SGI88477, SGI0, SGI88478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88479, SGI88480, SGI88480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19711[] = {SGI88481, SGI0, SGI88482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88483, SGI88484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19712[] = {SGI88485, SGI0, SGI88486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88487, SGI88488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19713[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88489, SGI0, SGI0, SGI88490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19714[] = {SGI88491, SGI88492, SGI88493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88494, SGI0, SGI0, SGI0, SGI88495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19715[] = {SGI88496, SGI0, SGI88497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88498, SGI0, SGI0, SGI0, SGI88499, SGI88499, SGI0, SGI0, SGI88500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19716[] = {SGI88501, SGI88502, SGI88503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88504, SGI0, SGI0, SGI0, SGI88505, SGI88505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19717[] = {SGI88506, SGI88507, SGI88508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88509, SGI0, SGI0, SGI0, SGI88510, SGI88510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19718[] = {SGI88511, SGI88512, SGI88513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88514, SGI0, SGI0, SGI0, SGI88515, SGI88515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19719[] = {SGI88516, SGI0, SGI88517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88518, SGI0, SGI0, SGI0, SGI88519, SGI88519, SGI0, SGI0, SGI88520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19720[] = {SGI88521, SGI0, SGI88522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88523, SGI0, SGI0, SGI0, SGI88524, SGI88524, SGI0, SGI0, SGI88525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19721[] = {SGI88526, SGI88527, SGI88528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88529, SGI0, SGI0, SGI0, SGI88530, SGI88531, SGI0, SGI0, SGI88532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19722[] = {SGI88533, SGI88534, SGI88535, SGI0, SGI0, SGI0, SGI88536, SGI88537, SGI0, SGI88538, SGI0, SGI0, SGI0, SGI88539, SGI88539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19723[] = {SGI88540, SGI88541, SGI88542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88543, SGI0, SGI0, SGI0, SGI88544, SGI88544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19724[] = {SGI88545, SGI88546, SGI88547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88548, SGI0, SGI0, SGI0, SGI88549, SGI88549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19725[] = {SGI88550, SGI0, SGI88551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19726[] = {SGI88552, SGI88553, SGI88554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88555, SGI0, SGI0, SGI0, SGI88556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19727[] = {SGI88557, SGI0, SGI88558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88559, SGI88560, SGI88560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88563, NULL};
const Segment * const SG19728[] = {SGI88561, SGI88562, SGI88564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88565, SGI0, SGI0, SGI0, SGI88566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19729[] = {SGI88567, SGI0, SGI88568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88569, SGI88570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19730[] = {SGI88571, SGI88572, SGI88573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88574, SGI0, SGI0, SGI0, SGI88575, SGI88575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19731[] = {SGI88576, SGI88577, SGI88578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88579, SGI0, SGI0, SGI0, SGI88580, SGI88580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19732[] = {SGI88581, SGI0, SGI88582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19733[] = {SGI88583, SGI88584, SGI88585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88586, SGI0, SGI0, SGI0, SGI88587, SGI88587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19734[] = {SGI88588, SGI88589, SGI88590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88591, SGI0, SGI0, SGI0, SGI88592, SGI88592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19735[] = {SGI88593, SGI88594, SGI88595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88596, SGI0, SGI0, SGI0, SGI88597, SGI88597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19736[] = {SGI88598, SGI88599, SGI88600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88601, SGI0, SGI0, SGI0, SGI88602, SGI88602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19737[] = {SGI88603, SGI88604, SGI88605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88606, SGI88606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19738[] = {SGI88607, SGI0, SGI88608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19739[] = {SGI88609, SGI0, SGI88610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88611, SGI88612, SGI88612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19740[] = {SGI88613, SGI88614, SGI88615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88616, SGI0, SGI0, SGI0, SGI88617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19741[] = {SGI88618, SGI0, SGI88619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88620, SGI88621, SGI88621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19742[] = {SGI88622, SGI0, SGI88623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88624, SGI88625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19743[] = {SGI88626, SGI88627, SGI88628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88629, SGI0, SGI0, SGI0, SGI88630, SGI88630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19744[] = {SGI88631, SGI88632, SGI88633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88634, SGI0, SGI0, SGI0, SGI88635, SGI88635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19745[] = {SGI88636, SGI88637, SGI88638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88639, SGI0, SGI0, SGI0, SGI88640, SGI88640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19746[] = {SGI88641, SGI88642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88643, SGI0, SGI0, SGI0, SGI88644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19747[] = {SGI88645, SGI88646, SGI88647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88648, SGI0, SGI0, SGI0, SGI88649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19748[] = {SGI88650, SGI0, SGI88651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88652, SGI88653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19749[] = {SGI88654, SGI88655, SGI88656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88657, SGI0, SGI0, SGI0, SGI88658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19750[] = {SGI88659, SGI88660, SGI88661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88662, SGI0, SGI0, SGI0, SGI88663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19751[] = {SGI88664, SGI88665, SGI88666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88667, SGI0, SGI0, SGI0, SGI88668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19752[] = {SGI88669, SGI0, SGI88670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88671, SGI88672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19753[] = {SGI88673, SGI0, SGI88674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88675, SGI88676, SGI88676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19754[] = {SGI88677, SGI0, SGI88678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88679, SGI88680, SGI88680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19755[] = {SGI88681, SGI0, SGI88682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88683, SGI88684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19756[] = {SGI88685, SGI88686, SGI88687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88688, SGI0, SGI0, SGI0, SGI88689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19757[] = {SGI88690, SGI88691, SGI88692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88693, SGI0, SGI0, SGI0, SGI88694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19758[] = {SGI88695, SGI88696, SGI88697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88698, SGI0, SGI0, SGI0, SGI88699, SGI88699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19759[] = {SGI88700, SGI88701, SGI88702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88703, SGI0, SGI0, SGI0, SGI88704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19760[] = {SGI88705, SGI88706, SGI88707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88708, SGI0, SGI0, SGI0, SGI88709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19761[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88710, SGI0, SGI0, SGI0, SGI88711, SGI88712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19762[] = {SGI88713, SGI88714, SGI88715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88716, SGI0, SGI0, SGI0, SGI88717, SGI88717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19763[] = {SGI88718, SGI0, SGI88719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19764[] = {SGI88720, SGI0, SGI88721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88722, SGI88723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19765[] = {SGI88724, SGI88725, SGI88726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88727, SGI0, SGI0, SGI0, SGI88728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19766[] = {SGI88729, SGI88730, SGI88731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88732, SGI0, SGI0, SGI0, SGI88733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19767[] = {SGI88734, SGI0, SGI88735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19768[] = {SGI88736, SGI0, SGI88737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88738, SGI0, SGI0, SGI0, SGI88739, SGI88739, SGI0, SGI0, SGI88740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19769[] = {SGI88741, SGI88742, SGI88743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88744, SGI0, SGI0, SGI0, SGI88745, SGI88745, SGI88746, SGI0, SGI88747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19770[] = {SGI88748, SGI88749, SGI88750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88751, SGI0, SGI0, SGI0, SGI88752, SGI88752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88753, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19771[] = {SGI88754, SGI0, SGI88755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88756, SGI88757, SGI88757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19772[] = {SGI88758, SGI0, SGI88759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88760, SGI88761, SGI88761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19773[] = {SGI88762, SGI0, SGI88763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88764, SGI88765, SGI88765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88766, NULL};
const Segment * const SG19774[] = {SGI88767, SGI88768, SGI88769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88770, SGI0, SGI0, SGI0, SGI88771, SGI88771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19775[] = {SGI88772, SGI0, SGI88773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88774, SGI88775, SGI88775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19776[] = {SGI88776, SGI0, SGI88777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88778, SGI88779, SGI88779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19777[] = {SGI88780, SGI88781, SGI88782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88783, SGI88783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88784, SGI0, NULL};
const Segment * const SG19778[] = {SGI88785, SGI0, SGI88786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88787, SGI0, SGI0, SGI0, SGI88788, SGI88788, SGI0, SGI0, SGI88789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19779[] = {SGI88790, SGI88791, SGI88792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88793, SGI0, SGI0, SGI0, SGI88794, SGI88794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19780[] = {SGI88795, SGI88796, SGI88797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88798, SGI0, SGI0, SGI0, SGI88799, SGI88799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19781[] = {SGI88800, SGI88801, SGI88802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88803, SGI0, SGI0, SGI0, SGI88804, SGI88804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19782[] = {SGI88805, SGI88806, SGI88807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88808, SGI0, SGI0, SGI0, SGI88809, SGI88809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19783[] = {SGI88810, SGI0, SGI88811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88812, SGI88813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19784[] = {SGI88814, SGI88815, SGI88816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88817, SGI0, SGI0, SGI0, SGI88818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19785[] = {SGI88819, SGI88820, SGI88821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88822, SGI0, SGI0, SGI0, SGI88823, SGI88823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19786[] = {SGI88824, SGI88825, SGI88826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88827, SGI0, SGI0, SGI0, SGI88828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19787[] = {SGI88829, SGI0, SGI88830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19788[] = {SGI88831, SGI88832, SGI88833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88834, SGI0, SGI0, SGI0, SGI88835, SGI88835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19789[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88836, SGI0, SGI88837, SGI0, NULL};
const Segment * const SG19790[] = {SGI88838, SGI88839, SGI88840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88841, SGI0, SGI0, SGI0, SGI88842, SGI88842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19791[] = {SGI88843, SGI88844, SGI88845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88846, SGI0, SGI0, SGI0, SGI88847, SGI88847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19792[] = {SGI88848, SGI88849, SGI88850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88851, SGI0, SGI0, SGI0, SGI88852, SGI88852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19793[] = {SGI88853, SGI88854, SGI88855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88856, SGI0, SGI0, SGI0, SGI88857, SGI88857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19794[] = {SGI88858, SGI88859, SGI88860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88861, SGI0, SGI0, SGI0, SGI88862, SGI88862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19795[] = {SGI88863, SGI0, SGI88865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88868, SGI88869, SGI88869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19796[] = {SGI88864, SGI88866, SGI88867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88870, SGI0, SGI0, SGI0, SGI88871, SGI88871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19797[] = {SGI88872, SGI0, SGI88873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88877, SGI88878, SGI88878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19798[] = {SGI88874, SGI88875, SGI88876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88879, SGI88880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19799[] = {SGI88881, SGI88882, SGI88883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88884, SGI0, SGI0, SGI0, SGI88885, SGI88885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19800[] = {SGI88886, SGI88887, SGI88888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88889, SGI88889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19801[] = {SGI88890, SGI88891, SGI88892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88893, SGI0, SGI0, SGI0, SGI88894, SGI88894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19802[] = {SGI88895, SGI88896, SGI88897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88898, SGI0, SGI0, SGI0, SGI88899, SGI88899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19803[] = {SGI88900, SGI88901, SGI88902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88903, SGI88903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19804[] = {SGI88904, SGI88905, SGI88906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88907, SGI88908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19805[] = {SGI88909, SGI88910, SGI88911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88912, SGI0, SGI0, SGI0, SGI88913, SGI88913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19806[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88914, SGI0, SGI0, SGI88915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19807[] = {SGI88916, SGI88917, SGI88918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88919, SGI0, SGI0, SGI0, SGI88920, SGI88920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19808[] = {SGI88921, SGI0, SGI88922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88923, SGI0, SGI0, SGI0, SGI88924, SGI88924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19809[] = {SGI88925, SGI88926, SGI88927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88928, SGI0, SGI0, SGI0, SGI88929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19810[] = {SGI88930, SGI0, SGI88931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19811[] = {SGI88932, SGI88933, SGI88934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88935, SGI0, SGI0, SGI0, SGI88936, SGI88936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19812[] = {SGI88937, SGI88938, SGI88939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88943, SGI0, SGI0, SGI0, SGI88944, SGI88944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19813[] = {SGI88940, SGI88941, SGI88942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88945, SGI0, SGI0, SGI0, SGI88946, SGI88946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19814[] = {SGI88947, SGI88948, SGI88949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88950, SGI0, SGI0, SGI0, SGI88951, SGI88951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19815[] = {SGI88952, SGI0, SGI88953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19816[] = {SGI88954, SGI88955, SGI88956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88957, SGI0, SGI0, SGI0, SGI88958, SGI88958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19817[] = {SGI88959, SGI88960, SGI88961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88962, SGI0, SGI0, SGI0, SGI88963, SGI88963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19818[] = {SGI88964, SGI88965, SGI88966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88967, SGI0, SGI0, SGI0, SGI88968, SGI88968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19819[] = {SGI88969, SGI88970, SGI88971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88973, SGI0, SGI0, SGI0, SGI88976, SGI88976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19820[] = {SGI88972, SGI88974, SGI88975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88977, SGI0, SGI0, SGI0, SGI88978, SGI88978, SGI0, SGI0, SGI0, SGI88979, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19821[] = {SGI88980, SGI88981, SGI88982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88983, SGI0, SGI0, SGI0, SGI88984, SGI88984, SGI0, SGI0, SGI0, SGI88985, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19822[] = {SGI88986, SGI88987, SGI88988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88989, SGI0, SGI0, SGI0, SGI88990, SGI88990, SGI0, SGI0, SGI0, SGI88991, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19823[] = {SGI88992, SGI88993, SGI88994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI88995, SGI0, SGI0, SGI0, SGI88996, SGI88996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19824[] = {SGI88997, SGI88998, SGI88999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89000, SGI89000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19825[] = {SGI89001, SGI0, SGI89002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19826[] = {SGI89003, SGI0, SGI89004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89005, SGI89006, SGI89006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19827[] = {SGI89007, SGI89008, SGI89009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89010, SGI0, SGI0, SGI0, SGI89011, SGI89011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89012, SGI0, SGI0, NULL};
const Segment * const SG19828[] = {SGI89013, SGI89014, SGI89015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89016, SGI0, SGI0, SGI0, SGI89017, SGI89017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19829[] = {SGI89018, SGI0, SGI89020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89023, SGI89025, SGI89025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19830[] = {SGI89019, SGI89021, SGI89022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89024, SGI0, SGI0, SGI0, SGI89026, SGI89026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19831[] = {SGI89027, SGI0, SGI89028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89029, SGI89030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19832[] = {SGI89031, SGI0, SGI89032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89033, SGI89034, SGI89034, SGI0, SGI89035, SGI0, NULL};
const Segment * const SG19833[] = {SGI89036, SGI89037, SGI89038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89039, SGI0, SGI0, SGI0, SGI89040, SGI89040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19834[] = {SGI89041, SGI89042, SGI89043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89044, SGI89045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19835[] = {SGI89046, SGI89047, SGI89048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89049, SGI0, SGI0, SGI0, SGI89050, SGI89050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19836[] = {SGI89051, SGI0, SGI89052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89053, SGI89054, SGI89054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19837[] = {SGI89055, SGI0, SGI89056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89057, SGI89058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19838[] = {SGI89059, SGI89060, SGI89061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89062, SGI0, SGI0, SGI0, SGI89063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19839[] = {SGI89064, SGI89065, SGI89066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89067, SGI0, SGI0, SGI0, SGI89068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19840[] = {SGI89069, SGI89070, SGI89071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89072, SGI0, SGI0, SGI0, SGI89073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19841[] = {SGI89074, SGI89075, SGI89076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89077, SGI0, SGI0, SGI0, SGI89078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19842[] = {SGI89079, SGI89080, SGI89081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89082, SGI0, SGI0, SGI0, SGI89083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19843[] = {SGI89084, SGI89085, SGI89086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89087, SGI0, SGI0, SGI0, SGI89088, SGI89088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19844[] = {SGI89089, SGI0, SGI89090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89091, SGI0, SGI0, SGI0, SGI89092, SGI89092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19845[] = {SGI89093, SGI0, SGI89094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19846[] = {SGI89095, SGI0, SGI89096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89097, SGI89098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19847[] = {SGI89099, SGI0, SGI89100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89101, SGI89102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19848[] = {SGI89103, SGI89104, SGI89105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89106, SGI0, SGI0, SGI0, SGI89107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19849[] = {SGI89108, SGI89109, SGI89110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89111, SGI0, SGI0, SGI0, SGI89112, SGI89112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19850[] = {SGI89113, SGI89114, SGI89115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89116, SGI0, SGI0, SGI0, SGI89117, SGI89117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19851[] = {SGI89118, SGI89119, SGI89120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89121, SGI89121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19852[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89122, SGI0, SGI89123, SGI0, NULL};
const Segment * const SG19853[] = {SGI89124, SGI89125, SGI89126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89127, SGI0, SGI0, SGI0, SGI89128, SGI89128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19854[] = {SGI89129, SGI89130, SGI89131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89132, SGI0, SGI0, SGI0, SGI89133, SGI89133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19855[] = {SGI89134, SGI0, SGI89135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89136, SGI89137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19856[] = {SGI89138, SGI0, SGI89139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89140, SGI0, SGI0, SGI0, SGI89141, SGI89141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89142, SGI0, SGI0, NULL};
const Segment * const SG19857[] = {SGI89143, SGI0, SGI89144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89145, SGI89146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19858[] = {SGI89147, SGI0, SGI89148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89149, SGI89150, SGI89150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89151, SGI0, NULL};
const Segment * const SG19859[] = {SGI89152, SGI89153, SGI89154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89156, SGI0, SGI0, SGI0, SGI89158, SGI89158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19860[] = {SGI89155, SGI0, SGI89157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89159, SGI89160, SGI89160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89161, SGI0, NULL};
const Segment * const SG19861[] = {SGI89162, SGI89163, SGI89164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89165, SGI0, SGI0, SGI0, SGI89167, SGI89167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19862[] = {SGI89166, SGI0, SGI89168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89169, SGI89170, SGI89170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89171, SGI0, NULL};
const Segment * const SG19863[] = {SGI89172, SGI0, SGI89173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89174, SGI89175, SGI89175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89176, SGI0, NULL};
const Segment * const SG19864[] = {SGI89177, SGI0, SGI89178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19865[] = {SGI89179, SGI0, SGI89180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89181, SGI89182, SGI89182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89183, SGI0, NULL};
const Segment * const SG19866[] = {SGI89184, SGI89185, SGI89186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89187, SGI0, SGI0, SGI0, SGI89188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19867[] = {SGI89189, SGI0, SGI89190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89191, SGI89192, SGI89192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19868[] = {SGI89193, SGI0, SGI89194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89195, SGI89196, SGI89196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19869[] = {SGI89197, SGI0, SGI89198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89199, SGI89200, SGI89200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19870[] = {SGI89201, SGI0, SGI89202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89203, SGI89204, SGI89204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19871[] = {SGI89205, SGI89206, SGI89207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89208, SGI0, SGI0, SGI0, SGI89209, SGI89209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19872[] = {SGI89210, SGI0, SGI89211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89212, SGI0, SGI0, SGI0, SGI89213, SGI89213, SGI0, SGI0, SGI89214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19873[] = {SGI89215, SGI89216, SGI89217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89218, SGI0, SGI0, SGI0, SGI89219, SGI89219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19874[] = {SGI89220, SGI89221, SGI89222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89223, SGI0, SGI0, SGI0, SGI89224, SGI89224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19875[] = {SGI89225, SGI89226, SGI89227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89228, SGI0, SGI0, SGI0, SGI89229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19876[] = {SGI89230, SGI0, SGI89231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19877[] = {SGI89232, SGI0, SGI89233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89234, SGI0, SGI0, SGI0, SGI89235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19878[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89236, SGI0, SGI89237, SGI0, NULL};
const Segment * const SG19879[] = {SGI89238, SGI0, SGI89239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89240, SGI89241, SGI89241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89244, NULL};
const Segment * const SG19880[] = {SGI89242, SGI0, SGI89243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89245, SGI89246, SGI89246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19881[] = {SGI89247, SGI0, SGI89248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89249, SGI0, SGI0, SGI0, SGI89250, SGI89250, SGI0, SGI0, SGI89251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19882[] = {SGI89252, SGI89253, SGI89254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89255, SGI0, SGI0, SGI0, SGI89256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19883[] = {SGI89257, SGI89258, SGI89259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89260, SGI0, SGI0, SGI0, SGI89261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19884[] = {SGI89262, SGI0, SGI89263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19885[] = {SGI89264, SGI0, SGI89265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89266, SGI0, SGI0, SGI0, SGI89267, SGI89267, SGI0, SGI0, SGI89268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19886[] = {SGI89269, SGI0, SGI89270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89271, SGI89272, SGI89272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19887[] = {SGI89273, SGI89274, SGI89275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89276, SGI89276, SGI0, SGI0, SGI89277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19888[] = {SGI89278, SGI0, SGI89279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89280, SGI89281, SGI89281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19889[] = {SGI89282, SGI89283, SGI89284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89285, SGI89285, SGI0, SGI0, SGI89286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19890[] = {SGI89287, SGI0, SGI89288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89289, SGI89290, SGI89290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19891[] = {SGI89291, SGI89292, SGI89293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89294, SGI0, SGI0, SGI0, SGI89295, SGI89295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19892[] = {SGI89296, SGI89297, SGI89298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89299, SGI0, SGI0, SGI0, SGI89300, SGI89300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19893[] = {SGI89301, SGI89302, SGI89303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89304, SGI0, SGI0, SGI0, SGI89305, SGI89305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19894[] = {SGI89306, SGI89307, SGI89308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89309, SGI0, SGI0, SGI0, SGI89310, SGI89310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19895[] = {SGI89311, SGI0, SGI89312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19896[] = {SGI89313, SGI89314, SGI89315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89316, SGI89317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19897[] = {SGI89318, SGI89319, SGI89320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89321, SGI0, SGI0, SGI0, SGI89322, SGI89322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19898[] = {SGI89323, SGI89324, SGI89325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89326, SGI89326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19899[] = {SGI89327, SGI89328, SGI89329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89330, SGI0, SGI0, SGI0, SGI89332, SGI89332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19900[] = {SGI89331, SGI0, SGI89333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89334, SGI89335, SGI89335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19901[] = {SGI89336, SGI89337, SGI89338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89339, SGI89339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19902[] = {SGI89340, SGI0, SGI89341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89342, SGI89343, SGI89343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19903[] = {SGI89344, SGI89345, SGI89346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89347, SGI0, SGI0, SGI0, SGI89348, SGI89348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19904[] = {SGI89349, SGI89350, SGI89351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89352, SGI0, SGI0, SGI0, SGI89353, SGI89353, SGI89354, SGI0, SGI89355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19905[] = {SGI89356, SGI0, SGI89357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89358, SGI89359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19906[] = {SGI89360, SGI0, SGI89361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89362, SGI89363, SGI89363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19907[] = {SGI89364, SGI0, SGI89365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89366, SGI89367, SGI89367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19908[] = {SGI89368, SGI0, SGI89369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89370, SGI89371, SGI89371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19909[] = {SGI89372, SGI0, SGI89373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89374, SGI89375, SGI89375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19910[] = {SGI89376, SGI89377, SGI89378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89379, SGI0, SGI0, SGI0, SGI89380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19911[] = {SGI89381, SGI0, SGI89382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19912[] = {SGI89383, SGI89384, SGI89385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89386, SGI0, SGI0, SGI0, SGI89387, SGI89387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89388, SGI0, SGI0, NULL};
const Segment * const SG19913[] = {SGI89389, SGI89390, SGI89391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89392, SGI0, SGI0, SGI0, SGI89393, SGI89393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19914[] = {SGI89394, SGI89395, SGI89396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89397, SGI0, SGI0, SGI0, SGI89398, SGI89398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19915[] = {SGI89399, SGI89400, SGI89401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89402, SGI0, SGI0, SGI0, SGI89403, SGI89403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19916[] = {SGI89404, SGI89405, SGI89406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89407, SGI89407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19917[] = {SGI89408, SGI89409, SGI89410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89411, SGI0, SGI0, SGI0, SGI89412, SGI89412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19918[] = {SGI89413, SGI89414, SGI89415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89416, SGI0, SGI0, SGI0, SGI89419, SGI89419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19919[] = {SGI89417, SGI0, SGI89418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19920[] = {SGI89420, SGI89421, SGI89422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89423, SGI0, SGI0, SGI0, SGI89424, SGI89424, SGI89425, SGI0, SGI89426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19921[] = {SGI89427, SGI89428, SGI89429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89430, SGI0, SGI0, SGI0, SGI89431, SGI89431, SGI89432, SGI0, SGI89433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19922[] = {SGI89434, SGI89435, SGI89436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89437, SGI0, SGI0, SGI0, SGI89438, SGI89438, SGI89439, SGI0, SGI89440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19923[] = {SGI89441, SGI89442, SGI89443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89444, SGI0, SGI0, SGI0, SGI89445, SGI89445, SGI89446, SGI0, SGI89447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19924[] = {SGI89448, SGI0, SGI89449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89455, SGI89457, SGI89457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19925[] = {SGI89450, SGI89451, SGI89453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89459, SGI89459, SGI0, SGI0, SGI89461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19926[] = {SGI89452, SGI89454, SGI89456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89458, SGI0, SGI0, SGI0, SGI89460, SGI89460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19927[] = {SGI89462, SGI89463, SGI89464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89465, SGI0, SGI0, SGI0, SGI89466, SGI89466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19928[] = {SGI89467, SGI0, SGI89468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89469, SGI89470, SGI89470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19929[] = {SGI89471, SGI89472, SGI89473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89474, SGI0, SGI0, SGI0, SGI89475, SGI89475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19930[] = {SGI89476, SGI89477, SGI89478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89479, SGI0, SGI0, SGI0, SGI89480, SGI89480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19931[] = {SGI89481, SGI89482, SGI89483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89484, SGI0, SGI0, SGI0, SGI89485, SGI89485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19932[] = {SGI89486, SGI89487, SGI89488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89489, SGI0, SGI0, SGI0, SGI89490, SGI89490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19933[] = {SGI89491, SGI0, SGI89492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89493, SGI0, SGI0, SGI0, SGI89494, SGI89494, SGI0, SGI0, SGI89495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19934[] = {SGI89496, SGI0, SGI89497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89498, SGI89499, SGI89499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19935[] = {SGI89500, SGI0, SGI89501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89502, SGI89503, SGI89503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19936[] = {SGI89504, SGI0, SGI89505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89506, SGI89507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19937[] = {SGI89508, SGI0, SGI89509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89510, SGI0, SGI0, SGI0, SGI89511, SGI89511, SGI0, SGI0, SGI89512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19938[] = {SGI89513, SGI0, SGI89514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89515, SGI89516, SGI89516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19939[] = {SGI89517, SGI89518, SGI89519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89520, SGI0, SGI0, SGI0, SGI89521, SGI89522, SGI0, SGI0, SGI89523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19940[] = {SGI89524, SGI0, SGI89525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89526, SGI89527, SGI89527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19941[] = {SGI89528, SGI0, SGI89529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89530, SGI89531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19942[] = {SGI89532, SGI0, SGI89533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89534, SGI89535, SGI89535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19943[] = {SGI89536, SGI0, SGI89537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89538, SGI89539, SGI89539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19944[] = {SGI89540, SGI0, SGI89541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89542, SGI89543, SGI89543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19945[] = {SGI89544, SGI0, SGI89545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89546, SGI89547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19946[] = {SGI89548, SGI89549, SGI89550, SGI0, SGI0, SGI0, SGI89551, SGI89552, SGI0, SGI89553, SGI0, SGI0, SGI0, SGI89554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19947[] = {SGI89555, SGI0, SGI89556, SGI0, SGI0, SGI0, SGI89557, SGI89558, SGI0, SGI89559, SGI0, SGI0, SGI0, SGI89560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19948[] = {SGI89561, SGI0, SGI89562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89563, SGI89564, SGI89564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19949[] = {SGI89565, SGI0, SGI89566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89567, SGI89568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19950[] = {SGI89569, SGI0, SGI89570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89571, SGI89572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19951[] = {SGI89573, SGI0, SGI89574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89575, SGI89576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19952[] = {SGI89577, SGI0, SGI89578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89579, SGI89580, SGI89580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19953[] = {SGI89581, SGI0, SGI89582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89583, SGI89584, SGI89584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19954[] = {SGI89585, SGI89586, SGI89587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89588, SGI0, SGI0, SGI0, SGI89589, SGI0, SGI0, SGI89590, SGI0, NULL};
const Segment * const SG19955[] = {SGI89591, SGI0, SGI89592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89593, SGI89594, SGI89594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19956[] = {SGI89595, SGI0, SGI89596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89597, SGI89598, SGI89598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19957[] = {SGI89599, SGI0, SGI89600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89601, SGI89602, SGI89602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19958[] = {SGI89603, SGI0, SGI89604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89605, SGI89606, SGI89606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19959[] = {SGI89607, SGI89608, SGI89609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89610, SGI0, SGI0, SGI0, SGI89611, SGI89611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19960[] = {SGI89612, SGI89613, SGI89614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89615, SGI0, SGI0, SGI0, SGI89616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19961[] = {SGI89617, SGI89618, SGI89619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89620, SGI0, SGI0, SGI0, SGI89621, SGI89621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19962[] = {SGI89622, SGI89623, SGI89624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89625, SGI0, SGI0, SGI0, SGI89626, SGI89626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19963[] = {SGI89627, SGI89628, SGI89629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89630, SGI89630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19964[] = {SGI89631, SGI0, SGI89632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89633, SGI89634, SGI89634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19965[] = {SGI89635, SGI89636, SGI89637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89638, SGI0, SGI0, SGI0, SGI89639, SGI89639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19966[] = {SGI89640, SGI89641, SGI89642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89643, SGI0, SGI0, SGI0, SGI89644, SGI89644, SGI89645, SGI0, SGI89646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19967[] = {SGI89647, SGI0, SGI89648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89649, SGI89650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19968[] = {SGI89651, SGI89652, SGI89653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89654, SGI0, SGI0, SGI0, SGI89655, SGI89655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19969[] = {SGI89656, SGI0, SGI89657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89658, SGI89659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19970[] = {SGI89660, SGI0, SGI89661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89662, SGI89663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19971[] = {SGI89664, SGI89665, SGI89666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89667, SGI0, SGI0, SGI0, SGI89668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19972[] = {SGI89669, SGI0, SGI89670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19973[] = {SGI89671, SGI89672, SGI89673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89674, SGI0, SGI0, SGI0, SGI89675, SGI89676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19974[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89677, SGI0, SGI89678, SGI0, NULL};
const Segment * const SG19975[] = {SGI0, SGI89679, SGI0, SGI89680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19976[] = {SGI89681, SGI89682, SGI89683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89684, SGI0, SGI0, SGI0, SGI89685, SGI89685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19977[] = {SGI89686, SGI89687, SGI89688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89689, SGI0, SGI0, SGI0, SGI89690, SGI89690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19978[] = {SGI89691, SGI89692, SGI89693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89694, SGI89694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19979[] = {SGI89695, SGI89696, SGI89697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89698, SGI0, SGI0, SGI0, SGI89699, SGI89699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19980[] = {SGI89700, SGI89701, SGI89702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89703, SGI0, SGI89704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19981[] = {SGI89705, SGI89706, SGI89707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89708, SGI0, SGI0, SGI0, SGI89709, SGI89709, SGI89710, SGI0, SGI89711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19982[] = {SGI89712, SGI0, SGI89713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89714, SGI0, SGI0, SGI0, SGI89715, SGI89715, SGI0, SGI0, SGI89716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19983[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89717, SGI0, SGI0, SGI89718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19984[] = {SGI89719, SGI89720, SGI89721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89722, SGI0, SGI0, SGI0, SGI89723, SGI89723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19985[] = {SGI89724, SGI89725, SGI89726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89727, SGI0, SGI0, SGI0, SGI89728, SGI89728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19986[] = {SGI89729, SGI89730, SGI89731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89732, SGI0, SGI0, SGI0, SGI89733, SGI89733, SGI89734, SGI0, SGI89735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19987[] = {SGI89736, SGI89737, SGI89738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89739, SGI0, SGI0, SGI0, SGI89740, SGI89740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19988[] = {SGI89741, SGI89742, SGI89743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89744, SGI0, SGI0, SGI0, SGI89745, SGI89745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19989[] = {SGI89746, SGI89747, SGI89748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89749, SGI0, SGI0, SGI0, SGI89750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19990[] = {SGI89751, SGI89752, SGI89753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89754, SGI89754, SGI0, SGI0, SGI89755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19991[] = {SGI89756, SGI89757, SGI89758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89759, SGI89759, SGI0, SGI0, SGI89760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19992[] = {SGI89761, SGI89762, SGI89763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89764, SGI89764, SGI0, SGI0, SGI89765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19993[] = {SGI89766, SGI0, SGI89767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89768, SGI0, SGI0, SGI0, SGI89769, SGI89769, SGI0, SGI0, SGI89770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19994[] = {SGI89771, SGI89772, SGI89773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89774, SGI0, SGI0, SGI0, SGI89775, SGI89776, SGI0, SGI0, SGI89777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19995[] = {SGI89778, SGI89779, SGI89780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89781, SGI0, SGI0, SGI0, SGI89782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19996[] = {SGI89783, SGI89784, SGI89785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89786, SGI0, SGI0, SGI0, SGI89787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19997[] = {SGI89788, SGI89789, SGI89790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89791, SGI0, SGI0, SGI0, SGI89792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19998[] = {SGI89793, SGI89794, SGI89795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89796, SGI0, SGI0, SGI0, SGI89797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG19999[] = {SGI89798, SGI89799, SGI89800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89801, SGI0, SGI0, SGI0, SGI89802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20000[] = {SGI89803, SGI0, SGI89806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89812, SGI89813, SGI89813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20001[] = {SGI89804, SGI89805, SGI89807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89811, SGI0, SGI0, SGI0, SGI89814, SGI89814, SGI89816, SGI0, SGI89817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20002[] = {SGI89808, SGI89809, SGI89810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89815, SGI0, SGI0, SGI0, SGI89818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20003[] = {SGI89819, SGI0, SGI89820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89824, SGI89825, SGI89825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20004[] = {SGI89821, SGI89822, SGI89823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89826, SGI0, SGI0, SGI0, SGI89827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20005[] = {SGI89828, SGI0, SGI89829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89832, SGI89834, SGI89834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20006[] = {SGI89830, SGI89831, SGI89833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89835, SGI0, SGI0, SGI0, SGI89836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20007[] = {SGI89837, SGI0, SGI89838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89840, SGI89842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20008[] = {SGI89839, SGI89841, SGI89843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89844, SGI0, SGI0, SGI0, SGI89845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20009[] = {SGI89846, SGI0, SGI89847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89850, SGI89852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20010[] = {SGI89848, SGI89849, SGI89851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89853, SGI0, SGI0, SGI0, SGI89854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20011[] = {SGI89855, SGI89856, SGI89857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89858, SGI0, SGI0, SGI0, SGI89859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20012[] = {SGI89860, SGI89861, SGI89862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89863, SGI0, SGI0, SGI0, SGI89864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20013[] = {SGI89865, SGI89866, SGI89867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89868, SGI0, SGI0, SGI0, SGI89869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20014[] = {SGI89870, SGI89871, SGI89872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89873, SGI0, SGI0, SGI0, SGI89874, SGI89874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20015[] = {SGI89875, SGI89876, SGI89877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89878, SGI0, SGI0, SGI0, SGI89879, SGI89879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20016[] = {SGI89880, SGI0, SGI89881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20017[] = {SGI89882, SGI89883, SGI89884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89885, SGI0, SGI0, SGI0, SGI89886, SGI89886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20018[] = {SGI89887, SGI89888, SGI89889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89890, SGI0, SGI0, SGI0, SGI89891, SGI89891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20019[] = {SGI89892, SGI89893, SGI89894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89895, SGI0, SGI0, SGI0, SGI89896, SGI89896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20020[] = {SGI89897, SGI89898, SGI89899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89900, SGI0, SGI0, SGI0, SGI89901, SGI89901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20021[] = {SGI89902, SGI89903, SGI89904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89905, SGI0, SGI0, SGI0, SGI89906, SGI89906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20022[] = {SGI89907, SGI0, SGI89908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89909, SGI89910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20023[] = {SGI89911, SGI89912, SGI89913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89914, SGI89914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20024[] = {SGI89915, SGI89916, SGI89917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89918, SGI0, SGI0, SGI0, SGI89919, SGI89919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20025[] = {SGI89920, SGI89921, SGI89922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89923, SGI0, SGI0, SGI0, SGI89924, SGI89924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20026[] = {SGI89925, SGI89926, SGI89927, SGI0, SGI0, SGI0, SGI89928, SGI89929, SGI0, SGI89930, SGI0, SGI0, SGI0, SGI89931, SGI89931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20027[] = {SGI89932, SGI89933, SGI89934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89935, SGI0, SGI0, SGI0, SGI89936, SGI89936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20028[] = {SGI89937, SGI0, SGI89938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89939, SGI89940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20029[] = {SGI89941, SGI0, SGI89942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20030[] = {SGI89943, SGI89944, SGI89945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89946, SGI89947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20031[] = {SGI89948, SGI89949, SGI89950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89951, SGI0, SGI0, SGI0, SGI89952, SGI89952, SGI0, SGI0, SGI0, SGI89953, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20032[] = {SGI89954, SGI89955, SGI89956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89957, SGI0, SGI0, SGI0, SGI89958, SGI89958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20033[] = {SGI89959, SGI89960, SGI89961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89962, SGI0, SGI0, SGI0, SGI89963, SGI89963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20034[] = {SGI89964, SGI89965, SGI89966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89967, SGI0, SGI0, SGI0, SGI89968, SGI89968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20035[] = {SGI89969, SGI0, SGI89970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20036[] = {SGI89971, SGI0, SGI89972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20037[] = {SGI89973, SGI0, SGI89974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89975, SGI89976, SGI89976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20038[] = {SGI89977, SGI0, SGI89978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89979, SGI89980, SGI89980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20039[] = {SGI89981, SGI0, SGI89982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89983, SGI89984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20040[] = {SGI89985, SGI89986, SGI89987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89988, SGI0, SGI0, SGI0, SGI89989, SGI89989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20041[] = {SGI89990, SGI0, SGI89991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20042[] = {SGI89992, SGI89993, SGI89994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI89995, SGI0, SGI0, SGI0, SGI89996, SGI89996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20043[] = {SGI89997, SGI89998, SGI89999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90000, SGI0, SGI0, SGI0, SGI90001, SGI90001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20044[] = {SGI90002, SGI90003, SGI90004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90005, SGI0, SGI0, SGI0, SGI90006, SGI90006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20045[] = {SGI90007, SGI90008, SGI90009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90010, SGI0, SGI0, SGI0, SGI90011, SGI90011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20046[] = {SGI90012, SGI90013, SGI90014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90015, SGI90015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20047[] = {SGI90016, SGI0, SGI90017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90018, SGI90019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20048[] = {SGI90020, SGI0, SGI90021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90022, SGI90023, SGI90023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20049[] = {SGI90024, SGI0, SGI90025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90026, SGI90027, SGI90027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20050[] = {SGI90028, SGI0, SGI90029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90030, SGI90031, SGI90031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20051[] = {SGI90032, SGI0, SGI90033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90034, SGI90035, SGI90035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20052[] = {SGI90036, SGI0, SGI90037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90038, SGI90039, SGI90039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20053[] = {SGI90040, SGI0, SGI90041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90042, SGI90043, SGI90043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20054[] = {SGI90044, SGI0, SGI90045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90046, SGI90047, SGI90047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20055[] = {SGI90048, SGI0, SGI90049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90050, SGI0, SGI0, SGI0, SGI90051, SGI90051, SGI0, SGI0, SGI90052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20056[] = {SGI90053, SGI0, SGI90054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90055, SGI0, SGI0, SGI0, SGI90056, SGI90056, SGI0, SGI0, SGI90057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20057[] = {SGI90058, SGI90059, SGI90060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90061, SGI90061, SGI0, SGI0, SGI90062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20058[] = {SGI90063, SGI90064, SGI90065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90066, SGI0, SGI0, SGI0, SGI90067, SGI90067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20059[] = {SGI90068, SGI90069, SGI90070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90071, SGI0, SGI0, SGI0, SGI90072, SGI90072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20060[] = {SGI90073, SGI0, SGI90074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90075, SGI90076, SGI90076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20061[] = {SGI90077, SGI0, SGI90078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90079, SGI90080, SGI90080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20062[] = {SGI90081, SGI0, SGI90082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90083, SGI90084, SGI90084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20063[] = {SGI90085, SGI0, SGI90086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90087, SGI90088, SGI90088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20064[] = {SGI90089, SGI90090, SGI90091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90092, SGI90092, SGI0, SGI0, SGI90093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20065[] = {SGI90094, SGI90095, SGI90096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90097, SGI0, SGI0, SGI0, SGI90098, SGI90098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20066[] = {SGI90099, SGI0, SGI90100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20067[] = {SGI90101, SGI90102, SGI90103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90104, SGI90105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20068[] = {SGI90106, SGI90107, SGI90108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90109, SGI90109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20069[] = {SGI90110, SGI90111, SGI90112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90113, SGI0, SGI0, SGI0, SGI90114, SGI90114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20070[] = {SGI90115, SGI90116, SGI90117, SGI0, SGI0, SGI0, SGI90118, SGI90119, SGI0, SGI90120, SGI0, SGI0, SGI0, SGI90121, SGI90121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20071[] = {SGI90122, SGI0, SGI90123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90124, SGI90125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20072[] = {SGI90126, SGI90127, SGI90128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90129, SGI0, SGI0, SGI0, SGI90130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20073[] = {SGI90131, SGI90132, SGI90133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90134, SGI0, SGI0, SGI0, SGI90135, SGI90135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20074[] = {SGI90136, SGI90137, SGI90138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90139, SGI0, SGI0, SGI0, SGI90140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20075[] = {SGI90141, SGI90142, SGI90143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90144, SGI0, SGI0, SGI0, SGI90145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20076[] = {SGI90146, SGI0, SGI90147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20077[] = {SGI90148, SGI90149, SGI90150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90151, SGI90151, SGI0, SGI0, SGI90152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20078[] = {SGI90153, SGI0, SGI90154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90155, SGI90156, SGI90156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20079[] = {SGI90157, SGI90158, SGI90159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90160, SGI0, SGI0, SGI0, SGI90161, SGI90161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20080[] = {SGI90162, SGI90163, SGI90164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90165, SGI0, SGI0, SGI0, SGI90166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20081[] = {SGI90167, SGI90168, SGI90169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90170, SGI0, SGI0, SGI0, SGI90171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20082[] = {SGI90172, SGI0, SGI90173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20083[] = {SGI90174, SGI0, SGI90175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90176, SGI90177, SGI90177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20084[] = {SGI90178, SGI0, SGI90179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90180, SGI90181, SGI90181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90182, NULL};
const Segment * const SG20085[] = {SGI90183, SGI90184, SGI90185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90186, SGI0, SGI0, SGI0, SGI90187, SGI90187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20086[] = {SGI90188, SGI0, SGI90189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90190, SGI90191, SGI90191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20087[] = {SGI90192, SGI90193, SGI90194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90195, SGI0, SGI0, SGI0, SGI90196, SGI90196, SGI90197, SGI0, SGI90198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20088[] = {SGI90199, SGI90200, SGI90201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90202, SGI0, SGI0, SGI0, SGI90203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20089[] = {SGI0, SGI90204, SGI90205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90206, SGI0, SGI0, SGI0, SGI90207, SGI90207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20090[] = {SGI90208, SGI90209, SGI90210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90211, SGI0, SGI0, SGI0, SGI90212, SGI90212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20091[] = {SGI0, SGI90213, SGI90214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90215, SGI0, SGI0, SGI0, SGI90216, SGI90216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20092[] = {SGI0, SGI90217, SGI90218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90219, SGI0, SGI0, SGI0, SGI90220, SGI90220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20093[] = {SGI0, SGI90221, SGI90222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90223, SGI0, SGI0, SGI0, SGI90224, SGI90224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20094[] = {SGI90225, SGI90226, SGI90227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90228, SGI90229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20095[] = {SGI90230, SGI90231, SGI90232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90233, SGI0, SGI0, SGI0, SGI90234, SGI90234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20096[] = {SGI90235, SGI90236, SGI90237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90238, SGI90239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20097[] = {SGI90240, SGI90241, SGI90242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90243, SGI0, SGI0, SGI0, SGI90244, SGI90244, SGI90245, SGI0, SGI90246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20098[] = {SGI90247, SGI90248, SGI90249, SGI0, SGI0, SGI0, SGI90250, SGI90251, SGI0, SGI90252, SGI0, SGI0, SGI0, SGI90253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20099[] = {SGI90254, SGI0, SGI90255, SGI0, SGI0, SGI0, SGI90256, SGI90257, SGI0, SGI90258, SGI0, SGI0, SGI0, SGI90259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20100[] = {SGI90260, SGI90261, SGI90262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90263, SGI0, SGI0, SGI0, SGI90264, SGI90264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20101[] = {SGI90265, SGI90266, SGI90267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90268, SGI0, SGI0, SGI0, SGI90269, SGI90269, SGI90270, SGI0, SGI90271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20102[] = {SGI90272, SGI90273, SGI90274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90275, SGI0, SGI0, SGI0, SGI90276, SGI90276, SGI90277, SGI0, SGI90278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20103[] = {SGI90279, SGI90280, SGI90281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90282, SGI0, SGI0, SGI0, SGI90283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20104[] = {SGI90284, SGI0, SGI90285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20105[] = {SGI90286, SGI90287, SGI90288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90289, SGI0, SGI0, SGI0, SGI90290, SGI90290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20106[] = {SGI90291, SGI90292, SGI90293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90294, SGI0, SGI0, SGI0, SGI90295, SGI90295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20107[] = {SGI90296, SGI90297, SGI90298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90299, SGI0, SGI0, SGI0, SGI90300, SGI90300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20108[] = {SGI90301, SGI90302, SGI90303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90304, SGI90304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20109[] = {SGI90305, SGI90306, SGI90307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90308, SGI90308, SGI0, SGI0, SGI90309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20110[] = {SGI90310, SGI90312, SGI90313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90316, SGI0, SGI0, SGI0, SGI90318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20111[] = {SGI90311, SGI90314, SGI90315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90317, SGI0, SGI0, SGI0, SGI90319, SGI90319, SGI0, SGI0, SGI0, SGI90320, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20112[] = {SGI90321, SGI90322, SGI90323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90324, SGI0, SGI0, SGI0, SGI90325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20113[] = {SGI90326, SGI90327, SGI90328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90329, SGI0, SGI0, SGI0, SGI90330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20114[] = {SGI90331, SGI90332, SGI90333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90334, SGI0, SGI0, SGI0, SGI90335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20115[] = {SGI90336, SGI90337, SGI90339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90342, SGI0, SGI0, SGI0, SGI90344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20116[] = {SGI90338, SGI90340, SGI90341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90343, SGI0, SGI0, SGI0, SGI90345, SGI90345, SGI0, SGI0, SGI0, SGI90346, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20117[] = {SGI90347, SGI90348, SGI90349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90350, SGI0, SGI0, SGI0, SGI90351, SGI90351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20118[] = {SGI90352, SGI90353, SGI90354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90355, SGI90355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20119[] = {SGI90356, SGI90357, SGI90358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90361, SGI0, SGI0, SGI0, SGI90362, SGI90362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20120[] = {SGI90359, SGI0, SGI90360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90363, SGI90364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20121[] = {SGI90365, SGI0, SGI90366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90367, SGI90368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20122[] = {SGI90369, SGI90370, SGI90371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90372, SGI0, SGI0, SGI0, SGI90373, SGI90373, SGI90374, SGI0, SGI90375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20123[] = {SGI90376, SGI0, SGI90377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90379, SGI90381, SGI90381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20124[] = {SGI90378, SGI90380, SGI90382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90383, SGI0, SGI0, SGI0, SGI90384, SGI90384, SGI90385, SGI0, SGI90386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20125[] = {SGI90387, SGI90388, SGI90389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90390, SGI0, SGI0, SGI0, SGI90391, SGI90391, SGI90392, SGI0, SGI90393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20126[] = {SGI90394, SGI90395, SGI90396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90397, SGI0, SGI0, SGI0, SGI90398, SGI90398, SGI90399, SGI0, SGI90400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20127[] = {SGI90401, SGI0, SGI90402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90406, SGI90407, SGI90407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20128[] = {SGI90403, SGI90404, SGI90405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90408, SGI0, SGI0, SGI0, SGI90409, SGI90409, SGI90410, SGI0, SGI90411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20129[] = {SGI90412, SGI90413, SGI90414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90417, SGI0, SGI0, SGI0, SGI90418, SGI90418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20130[] = {SGI90415, SGI0, SGI90416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90419, SGI90420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20131[] = {SGI90421, SGI0, SGI90422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90423, SGI90424, SGI90424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20132[] = {SGI90425, SGI0, SGI90426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90427, SGI90428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20133[] = {SGI90429, SGI0, SGI90430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90431, SGI90432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20134[] = {SGI90433, SGI0, SGI90434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90435, SGI90436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20135[] = {SGI90437, SGI0, SGI90438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90439, SGI90440, SGI90440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20136[] = {SGI90441, SGI0, SGI90442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90443, SGI90444, SGI90444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20137[] = {SGI90445, SGI0, SGI90446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90447, SGI90448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20138[] = {SGI90449, SGI90450, SGI90451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90452, SGI0, SGI0, SGI0, SGI90453, SGI90453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20139[] = {SGI90454, SGI90455, SGI90456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90457, SGI90457, SGI0, SGI0, SGI90458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20140[] = {SGI90459, SGI90460, SGI90461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90462, SGI90462, SGI0, SGI0, SGI90463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20141[] = {SGI90464, SGI0, SGI90465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90466, SGI90467, SGI90467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20142[] = {SGI90468, SGI0, SGI90469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90470, SGI90471, SGI90471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20143[] = {SGI90472, SGI0, SGI90473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90474, SGI90475, SGI90475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20144[] = {SGI90476, SGI90477, SGI90478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90479, SGI0, SGI0, SGI0, SGI90480, SGI90480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20145[] = {SGI90481, SGI0, SGI90482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90483, SGI90484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20146[] = {SGI90485, SGI0, SGI90486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90487, SGI90488, SGI90488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20147[] = {SGI90489, SGI0, SGI90490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90491, SGI90492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20148[] = {SGI90493, SGI0, SGI90494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90495, SGI90496, SGI90496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20149[] = {SGI90497, SGI90498, SGI90499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90500, SGI0, SGI0, SGI0, SGI90501, SGI90501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20150[] = {SGI90502, SGI0, SGI90503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90504, SGI90505, SGI90505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20151[] = {SGI90506, SGI0, SGI90507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90508, SGI90509, SGI90509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20152[] = {SGI90510, SGI90511, SGI90512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90513, SGI0, SGI0, SGI0, SGI90514, SGI90514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90515, SGI0, NULL};
const Segment * const SG20153[] = {SGI90516, SGI0, SGI90517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90518, SGI90519, SGI90519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90520, SGI0, NULL};
const Segment * const SG20154[] = {SGI90521, SGI0, SGI90522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90523, SGI90524, SGI90524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90525, SGI0, NULL};
const Segment * const SG20155[] = {SGI90526, SGI0, SGI90527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90528, SGI90529, SGI90529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90530, SGI0, NULL};
const Segment * const SG20156[] = {SGI90531, SGI0, SGI90532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90533, SGI90534, SGI90534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90535, SGI0, NULL};
const Segment * const SG20157[] = {SGI90536, SGI0, SGI90537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90538, SGI90539, SGI90539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20158[] = {SGI90540, SGI0, SGI90541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90542, SGI90543, SGI90543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20159[] = {SGI90544, SGI90545, SGI90546, SGI90547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90548, SGI0, SGI0, SGI0, SGI90549, SGI90550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20160[] = {SGI90551, SGI90552, SGI90553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90554, SGI0, SGI0, SGI0, SGI90555, SGI90555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20161[] = {SGI90556, SGI90557, SGI90558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90562, SGI0, SGI0, SGI0, SGI90563, SGI90563, SGI0, SGI0, NULL};
const Segment * const SG20162[] = {SGI90559, SGI90560, SGI90561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90564, SGI0, SGI0, SGI0, SGI90565, SGI90565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20163[] = {SGI90566, SGI90567, SGI90568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90569, SGI90569, SGI0, SGI0, SGI90570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20164[] = {SGI90571, SGI90572, SGI90573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90574, SGI0, SGI0, SGI0, SGI90575, SGI90575, SGI0, SGI0, NULL};
const Segment * const SG20165[] = {SGI90576, SGI90577, SGI90578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90579, SGI0, SGI0, SGI0, SGI90580, SGI90580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20166[] = {SGI90581, SGI90582, SGI90583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90584, SGI0, SGI0, SGI0, SGI90585, SGI90585, SGI0, SGI0, NULL};
const Segment * const SG20167[] = {SGI90586, SGI90587, SGI90588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90589, SGI0, SGI0, SGI0, SGI90590, SGI90590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20168[] = {SGI90591, SGI0, SGI90592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20169[] = {SGI90593, SGI90594, SGI90595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90596, SGI0, SGI0, SGI0, SGI90597, SGI90597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20170[] = {SGI90598, SGI90599, SGI90600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90601, SGI0, SGI0, SGI0, SGI90602, SGI90602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20171[] = {SGI90603, SGI90604, SGI90605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90606, SGI0, SGI0, SGI0, SGI90607, SGI90607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20172[] = {SGI90608, SGI90609, SGI90610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90611, SGI0, SGI0, SGI0, SGI90612, SGI90612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20173[] = {SGI90613, SGI90614, SGI90615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90616, SGI90616, SGI0, SGI0, SGI90617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20174[] = {SGI90618, SGI90619, SGI90620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90621, SGI90621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90622, SGI0, SGI0, NULL};
const Segment * const SG20175[] = {SGI90623, SGI90624, SGI90625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90626, SGI90626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90627, SGI0, SGI0, NULL};
const Segment * const SG20176[] = {SGI90628, SGI90629, SGI90630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90631, SGI90631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90632, SGI0, SGI0, NULL};
const Segment * const SG20177[] = {SGI90633, SGI90634, SGI90635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90636, SGI0, SGI0, SGI0, SGI90637, SGI90637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20178[] = {SGI90638, SGI90639, SGI90640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90641, SGI90641, SGI0, SGI0, SGI90642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20179[] = {SGI90643, SGI90644, SGI90645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90646, SGI0, SGI0, SGI0, SGI90647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20180[] = {SGI90648, SGI90649, SGI90650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90651, SGI0, SGI0, SGI0, SGI90652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20181[] = {SGI90653, SGI90654, SGI90655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90656, SGI0, SGI0, SGI0, SGI90657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20182[] = {SGI90658, SGI90659, SGI90660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90661, SGI0, SGI0, SGI0, SGI90662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20183[] = {SGI90663, SGI90664, SGI90665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90666, SGI0, SGI0, SGI0, SGI90667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20184[] = {SGI90668, SGI90669, SGI90670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90671, SGI0, SGI0, SGI0, SGI90672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20185[] = {SGI90673, SGI90674, SGI90675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90676, SGI0, SGI0, SGI0, SGI90677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20186[] = {SGI90678, SGI90679, SGI90680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90681, SGI0, SGI0, SGI0, SGI90682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20187[] = {SGI90683, SGI90684, SGI90685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90686, SGI0, SGI0, SGI0, SGI90688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20188[] = {SGI90687, SGI90689, SGI90690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90691, SGI0, SGI0, SGI0, SGI90692, SGI90692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20189[] = {SGI90693, SGI90694, SGI90695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90696, SGI0, SGI0, SGI0, SGI90697, SGI90697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20190[] = {SGI90698, SGI0, SGI90699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20191[] = {SGI90700, SGI90701, SGI90702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90703, SGI90703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20192[] = {SGI90704, SGI90705, SGI90706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90707, SGI0, SGI0, SGI0, SGI90708, SGI90708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20193[] = {SGI90709, SGI90710, SGI90711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90712, SGI90712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20194[] = {SGI90713, SGI90714, SGI90715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90716, SGI0, SGI0, SGI0, SGI90717, SGI90717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20195[] = {SGI90718, SGI90719, SGI90720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90721, SGI0, SGI0, SGI0, SGI90722, SGI90722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20196[] = {SGI90723, SGI90724, SGI90725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90726, SGI0, SGI0, SGI0, SGI90727, SGI90727, SGI90728, SGI0, SGI90729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20197[] = {SGI90730, SGI0, SGI90731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90732, SGI0, SGI0, SGI0, SGI90733, SGI90733, SGI0, SGI0, SGI90734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20198[] = {SGI90735, SGI90736, SGI90737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90738, SGI0, SGI0, SGI0, SGI90739, SGI90739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90740, SGI0, NULL};
const Segment * const SG20199[] = {SGI90741, SGI90742, SGI90743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90744, SGI90744, SGI0, SGI0, SGI90745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20200[] = {SGI90746, SGI90747, SGI90748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90749, SGI0, SGI0, SGI0, SGI90750, SGI90750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20201[] = {SGI90751, SGI0, SGI90752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90753, SGI90754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20202[] = {SGI90755, SGI0, SGI90756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90757, SGI90758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20203[] = {SGI90759, SGI0, SGI90760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90761, SGI90762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20204[] = {SGI90763, SGI90764, SGI90765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90766, SGI0, SGI0, SGI0, SGI90767, SGI90767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20205[] = {SGI90768, SGI90769, SGI90770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90771, SGI90771, SGI0, SGI0, SGI90772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20206[] = {SGI90773, SGI0, SGI90774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90775, SGI0, SGI0, SGI0, SGI90779, SGI90779, SGI0, SGI0, SGI90780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20207[] = {SGI90776, SGI90777, SGI90778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90781, SGI0, SGI0, SGI0, SGI90782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20208[] = {SGI90783, SGI0, SGI90785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20209[] = {SGI90784, SGI90786, SGI90787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90788, SGI0, SGI0, SGI0, SGI90789, SGI90790, SGI0, SGI0, SGI90791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20210[] = {SGI90792, SGI90793, SGI90794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90795, SGI0, SGI0, SGI0, SGI90796, SGI90796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20211[] = {SGI90797, SGI0, SGI90798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90799, SGI90800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20212[] = {SGI0, SGI90801, SGI0, SGI90802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20213[] = {SGI90803, SGI0, SGI90804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90805, SGI90806, SGI90806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20214[] = {SGI90807, SGI0, SGI90808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90809, SGI90810, SGI90810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20215[] = {SGI90811, SGI0, SGI90812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90813, SGI90814, SGI90814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20216[] = {SGI90815, SGI0, SGI90816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90817, SGI90818, SGI90818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20217[] = {SGI90819, SGI0, SGI90820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90821, SGI90822, SGI90822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20218[] = {SGI90823, SGI90824, SGI90825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90826, SGI0, SGI0, SGI0, SGI90827, SGI90827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20219[] = {SGI90828, SGI90829, SGI90830, SGI0, SGI0, SGI0, SGI90831, SGI90832, SGI0, SGI90833, SGI0, SGI0, SGI0, SGI90834, SGI90834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20220[] = {SGI90835, SGI0, SGI90836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90837, SGI90838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20221[] = {SGI90839, SGI0, SGI90840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90841, SGI90842, SGI90842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20222[] = {SGI90843, SGI0, SGI90844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90845, SGI90846, SGI90846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20223[] = {SGI90847, SGI0, SGI90848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90849, SGI90850, SGI90850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20224[] = {SGI90851, SGI0, SGI90852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90853, SGI90854, SGI90854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20225[] = {SGI90855, SGI0, SGI90856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90857, SGI90858, SGI90858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20226[] = {SGI90859, SGI0, SGI90860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90861, SGI90862, SGI90862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20227[] = {SGI90863, SGI0, SGI90864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90865, SGI90866, SGI90866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20228[] = {SGI90867, SGI90868, SGI90869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90870, SGI0, SGI0, SGI0, SGI90871, SGI90871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20229[] = {SGI90872, SGI90873, SGI90874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90875, SGI0, SGI0, SGI0, SGI90876, SGI90876, SGI90877, SGI0, SGI90878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20230[] = {SGI90879, SGI90880, SGI90881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90882, SGI0, SGI0, SGI0, SGI90883, SGI90883, SGI90884, SGI0, SGI90885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20231[] = {SGI90886, SGI90887, SGI90888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90889, SGI90889, SGI0, SGI0, SGI90890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20232[] = {SGI90891, SGI90892, SGI90893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90894, SGI90894, SGI0, SGI0, SGI90895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20233[] = {SGI90896, SGI90897, SGI90898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90899, SGI0, SGI0, SGI0, SGI90900, SGI90900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20234[] = {SGI90901, SGI0, SGI90902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90903, SGI0, SGI0, SGI0, SGI90904, SGI90904, SGI0, SGI0, SGI90905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20235[] = {SGI90906, SGI0, SGI90907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90908, SGI90909, SGI90909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90910, NULL};
const Segment * const SG20236[] = {SGI90911, SGI90912, SGI90913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90914, SGI0, SGI0, SGI0, SGI90915, SGI90915, SGI0, SGI0, SGI0, SGI90916, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20237[] = {SGI90917, SGI0, SGI90918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90919, SGI0, SGI0, SGI0, SGI90920, SGI90920, SGI0, SGI0, SGI90921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20238[] = {SGI90922, SGI90923, SGI90924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90925, SGI90925, SGI0, SGI0, SGI90926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20239[] = {SGI90927, SGI0, SGI90928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90929, SGI90930, SGI90930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20240[] = {SGI90931, SGI0, SGI90932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90933, SGI90934, SGI90934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20241[] = {SGI90935, SGI0, SGI90936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90937, SGI90938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20242[] = {SGI90939, SGI0, SGI90940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90941, SGI90942, SGI90942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20243[] = {SGI90943, SGI0, SGI90944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90945, SGI90946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20244[] = {SGI90947, SGI0, SGI90948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90949, SGI90950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20245[] = {SGI90951, SGI0, SGI90952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90956, SGI90958, SGI90958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20246[] = {SGI90953, SGI90954, SGI90955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90957, SGI0, SGI0, SGI0, SGI90959, SGI90959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20247[] = {SGI90960, SGI0, SGI90961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90962, SGI0, SGI0, SGI0, SGI90963, SGI90963, SGI0, SGI0, SGI90964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20248[] = {SGI90965, SGI0, SGI90966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90967, SGI90968, SGI90968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20249[] = {SGI90969, SGI90970, SGI90971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90972, SGI0, SGI0, SGI0, SGI90973, SGI90973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20250[] = {SGI90974, SGI0, SGI90975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20251[] = {SGI90976, SGI90977, SGI90978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90979, SGI90979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20252[] = {SGI90980, SGI0, SGI90981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90982, SGI90983, SGI90983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20253[] = {SGI90984, SGI0, SGI90985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90986, SGI90987, SGI90987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20254[] = {SGI90988, SGI0, SGI90989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90990, SGI90991, SGI90991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20255[] = {SGI90992, SGI0, SGI90993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90994, SGI90995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20256[] = {SGI90996, SGI0, SGI90997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI90998, SGI90999, SGI90999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20257[] = {SGI91000, SGI0, SGI91001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91002, SGI91003, SGI91003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20258[] = {SGI91004, SGI0, SGI91005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91006, SGI91007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20259[] = {SGI91008, SGI0, SGI91009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91010, SGI91011, SGI91011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20260[] = {SGI91012, SGI91013, SGI91014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91015, SGI0, SGI0, SGI0, SGI91016, SGI91016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20261[] = {SGI91017, SGI0, SGI91018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91019, SGI91020, SGI91020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91021, NULL};
const Segment * const SG20262[] = {SGI91022, SGI91023, SGI91024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91025, SGI0, SGI0, SGI0, SGI91026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20263[] = {SGI91027, SGI91028, SGI91029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91030, SGI0, SGI0, SGI0, SGI91031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20264[] = {SGI91032, SGI91033, SGI91034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91035, SGI0, SGI0, SGI0, SGI91036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20265[] = {SGI91037, SGI0, SGI91038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91039, SGI91040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20266[] = {SGI91041, SGI91042, SGI91043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91044, SGI0, SGI0, SGI0, SGI91045, SGI91045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20267[] = {SGI91046, SGI0, SGI91047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20268[] = {SGI91048, SGI91049, SGI91050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91051, SGI0, SGI0, SGI0, SGI91052, SGI91052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20269[] = {SGI91053, SGI91054, SGI91055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91056, SGI0, SGI0, SGI0, SGI91057, SGI91057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20270[] = {SGI91058, SGI0, SGI91059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20271[] = {SGI0, SGI0, SGI91060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91061, SGI91062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20272[] = {SGI91063, SGI0, SGI91064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91065, SGI91066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20273[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91067, SGI0, SGI0, SGI91068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20274[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91069, SGI0, SGI91070, SGI0, NULL};
const Segment * const SG20275[] = {SGI91071, SGI91072, SGI91073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91074, SGI91074, SGI0, SGI0, SGI91075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20276[] = {SGI91076, SGI91077, SGI91078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91079, SGI0, SGI0, SGI0, SGI91080, SGI91080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20277[] = {SGI91081, SGI91082, SGI91083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91084, SGI0, SGI0, SGI0, SGI91085, SGI91085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20278[] = {SGI91086, SGI91087, SGI91088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91089, SGI0, SGI0, SGI0, SGI91090, SGI91090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20279[] = {SGI91091, SGI91092, SGI91093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91094, SGI0, SGI0, SGI0, SGI91095, SGI91095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20280[] = {SGI91096, SGI91097, SGI91098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91099, SGI0, SGI0, SGI0, SGI91100, SGI91100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20281[] = {SGI91101, SGI91102, SGI91103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91104, SGI0, SGI0, SGI0, SGI91105, SGI91105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20282[] = {SGI91106, SGI91107, SGI91108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91109, SGI0, SGI0, SGI0, SGI91110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20283[] = {SGI91111, SGI91112, SGI91113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91114, SGI0, SGI0, SGI0, SGI91115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20284[] = {SGI91116, SGI91117, SGI91118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91119, SGI0, SGI0, SGI0, SGI91120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20285[] = {SGI91121, SGI91122, SGI91123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91124, SGI0, SGI0, SGI0, SGI91125, SGI91125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20286[] = {SGI91126, SGI91127, SGI91128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91129, SGI0, SGI0, SGI0, SGI91130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20287[] = {SGI91131, SGI91132, SGI91133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91134, SGI0, SGI0, SGI0, SGI91135, SGI91135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20288[] = {SGI91136, SGI91137, SGI91138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91139, SGI0, SGI0, SGI0, SGI91140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20289[] = {SGI91141, SGI91142, SGI91143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91144, SGI0, SGI0, SGI0, SGI91145, SGI91145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20290[] = {SGI91146, SGI91147, SGI91149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91151, SGI0, SGI0, SGI0, SGI91152, SGI91152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20291[] = {SGI91148, SGI0, SGI91150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91153, SGI91154, SGI91154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20292[] = {SGI91155, SGI91157, SGI91158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91160, SGI0, SGI0, SGI0, SGI91162, SGI91162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20293[] = {SGI91156, SGI0, SGI91159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91161, SGI91163, SGI91163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20294[] = {SGI91164, SGI91165, SGI91166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91167, SGI0, SGI0, SGI0, SGI91168, SGI91168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20295[] = {SGI91169, SGI0, SGI91170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91171, SGI91172, SGI91172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20296[] = {SGI91173, SGI91174, SGI91175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91176, SGI0, SGI0, SGI0, SGI91177, SGI91177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20297[] = {SGI91178, SGI91179, SGI91180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91181, SGI0, SGI0, SGI0, SGI91182, SGI91182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20298[] = {SGI91183, SGI91184, SGI91185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91186, SGI0, SGI0, SGI0, SGI91187, SGI91187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20299[] = {SGI91188, SGI91189, SGI91190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91191, SGI0, SGI0, SGI0, SGI91192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20300[] = {SGI91193, SGI0, SGI91194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91195, SGI91196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20301[] = {SGI91197, SGI0, SGI91198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91199, SGI91200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20302[] = {SGI91201, SGI91202, SGI91203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91204, SGI0, SGI0, SGI0, SGI91205, SGI91205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20303[] = {SGI91206, SGI0, SGI91207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20304[] = {SGI91208, SGI91209, SGI91210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91211, SGI0, SGI0, SGI0, SGI91212, SGI91212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20305[] = {SGI91213, SGI91214, SGI91215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91216, SGI0, SGI0, SGI0, SGI91217, SGI91217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20306[] = {SGI91218, SGI91219, SGI91220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91221, SGI0, SGI0, SGI0, SGI91222, SGI91222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20307[] = {SGI91223, SGI0, SGI91224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91225, SGI91226, SGI91226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20308[] = {SGI91227, SGI0, SGI91228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91229, SGI91230, SGI91230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20309[] = {SGI91231, SGI91232, SGI91233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91234, SGI91235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20310[] = {SGI91236, SGI91237, SGI91238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91239, SGI0, SGI0, SGI0, SGI91240, SGI91240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20311[] = {SGI91241, SGI91242, SGI91243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91244, SGI91244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20312[] = {SGI0, SGI91245, SGI91246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20313[] = {SGI91247, SGI0, SGI91248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91249, SGI91250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20314[] = {SGI91251, SGI91252, SGI91253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91254, SGI0, SGI0, SGI0, SGI91255, SGI91255, SGI91256, SGI0, SGI91257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20315[] = {SGI91258, SGI91259, SGI91260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91261, SGI0, SGI0, SGI0, SGI91262, SGI91262, SGI91263, SGI0, SGI91264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20316[] = {SGI91265, SGI91266, SGI91267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91268, SGI0, SGI0, SGI0, SGI91269, SGI91269, SGI91270, SGI0, SGI91271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20317[] = {SGI91272, SGI91273, SGI91274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91275, SGI0, SGI0, SGI0, SGI91276, SGI91276, SGI91277, SGI0, SGI91278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20318[] = {SGI91279, SGI91280, SGI91281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91282, SGI0, SGI0, SGI0, SGI91283, SGI91283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20319[] = {SGI91284, SGI0, SGI91285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91286, SGI91287, SGI91287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20320[] = {SGI91288, SGI0, SGI91289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91290, SGI91291, SGI91291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20321[] = {SGI91292, SGI0, SGI91293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91294, SGI91295, SGI91295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20322[] = {SGI91296, SGI0, SGI91297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91298, SGI91299, SGI91299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20323[] = {SGI91300, SGI0, SGI91301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91302, SGI91303, SGI91303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20324[] = {SGI91304, SGI0, SGI91305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91306, SGI91307, SGI91307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20325[] = {SGI91308, SGI0, SGI91309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91310, SGI0, SGI0, SGI0, SGI91311, SGI91311, SGI0, SGI0, SGI91312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20326[] = {SGI91313, SGI91314, SGI91315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91316, SGI0, SGI0, SGI0, SGI91319, SGI91319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20327[] = {SGI91317, SGI91318, SGI91320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91321, SGI0, SGI0, SGI0, SGI91322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20328[] = {SGI91323, SGI91324, SGI91325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91326, SGI0, SGI0, SGI0, SGI91327, SGI91327, SGI0, SGI91328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20329[] = {SGI91329, SGI0, SGI91330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91331, SGI91332, SGI91332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20330[] = {SGI91333, SGI0, SGI91334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91335, SGI91336, SGI91336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20331[] = {SGI91337, SGI0, SGI91338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91339, SGI91340, SGI91340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20332[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91341, SGI0, SGI91344, SGI0, NULL};
const Segment * const SG20333[] = {SGI91342, SGI0, SGI91343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91345, SGI91346, SGI91346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20334[] = {SGI91347, SGI0, SGI91348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91349, SGI91350, SGI91350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20335[] = {SGI91351, SGI0, SGI91352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91353, SGI91354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20336[] = {SGI91355, SGI0, SGI91356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91357, SGI91358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20337[] = {SGI91359, SGI0, SGI91360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91361, SGI91362, SGI91362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20338[] = {SGI91363, SGI0, SGI91364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91365, SGI91366, SGI91366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20339[] = {SGI91367, SGI0, SGI91368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91369, SGI91370, SGI91370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20340[] = {SGI91371, SGI91372, SGI91373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91374, SGI0, SGI0, SGI0, SGI91375, SGI91375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91376, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20341[] = {SGI91377, SGI91378, SGI91379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91380, SGI0, SGI0, SGI0, SGI91381, SGI91381, SGI0, SGI91382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20342[] = {SGI91383, SGI0, SGI91384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91385, SGI0, SGI0, SGI0, SGI91386, SGI91386, SGI0, SGI0, SGI91387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20343[] = {SGI91388, SGI91389, SGI91390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91391, SGI0, SGI0, SGI0, SGI91392, SGI91392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20344[] = {SGI91393, SGI91394, SGI91395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91396, SGI91396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20345[] = {SGI91397, SGI0, SGI91398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20346[] = {SGI91399, SGI91400, SGI91401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91402, SGI0, SGI0, SGI0, SGI91403, SGI91403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91404, SGI0, NULL};
const Segment * const SG20347[] = {SGI91405, SGI91406, SGI91407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91408, SGI91408, SGI0, SGI0, SGI91409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20348[] = {SGI91410, SGI0, SGI91411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91412, SGI91413, SGI91413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20349[] = {SGI91414, SGI91415, SGI91416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91417, SGI0, SGI0, SGI0, SGI91418, SGI91418, SGI0, SGI91419, SGI0, NULL};
const Segment * const SG20350[] = {SGI91420, SGI0, SGI91421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91422, SGI91423, SGI91423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20351[] = {SGI91424, SGI0, SGI91425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91426, SGI91427, SGI91427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20352[] = {SGI91428, SGI0, SGI91429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91430, SGI91431, SGI91431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20353[] = {SGI91432, SGI0, SGI91433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91434, SGI91435, SGI91435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20354[] = {SGI91436, SGI0, SGI91437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91438, SGI91439, SGI91439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20355[] = {SGI91440, SGI0, SGI91441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91442, SGI91443, SGI91443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20356[] = {SGI91444, SGI0, SGI91445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91446, SGI91447, SGI91447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20357[] = {SGI91448, SGI0, SGI91449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91450, SGI91451, SGI91451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20358[] = {SGI91452, SGI91453, SGI91454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91455, SGI0, SGI0, SGI0, SGI91456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20359[] = {SGI91457, SGI91458, SGI91459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91460, SGI0, SGI0, SGI0, SGI91461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20360[] = {SGI91462, SGI0, SGI91463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20361[] = {SGI91464, SGI91465, SGI91466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91467, SGI0, SGI0, SGI0, SGI91468, SGI91468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91469, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20362[] = {SGI91470, SGI91471, SGI91472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91473, SGI0, SGI0, SGI0, SGI91474, SGI91474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20363[] = {SGI91475, SGI91476, SGI91477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91478, SGI0, SGI0, SGI0, SGI0, SGI91479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20364[] = {SGI91480, SGI0, SGI91481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91482, SGI0, SGI0, SGI0, SGI91483, SGI91483, SGI0, SGI0, SGI91484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20365[] = {SGI91485, SGI91486, SGI91487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91488, SGI0, SGI0, SGI0, SGI91489, SGI91489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20366[] = {SGI91490, SGI0, SGI91491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91492, SGI91493, SGI91493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20367[] = {SGI91494, SGI0, SGI91495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91496, SGI91497, SGI91497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20368[] = {SGI91498, SGI91499, SGI91500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91501, SGI0, SGI0, SGI0, SGI91502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20369[] = {SGI91503, SGI0, SGI91504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91505, SGI91506, SGI91506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20370[] = {SGI91507, SGI0, SGI91508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91509, SGI91510, SGI91510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20371[] = {SGI91511, SGI0, SGI91512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91513, SGI91514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20372[] = {SGI91515, SGI0, SGI91516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91517, SGI91518, SGI91518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20373[] = {SGI91519, SGI0, SGI91520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91521, SGI91522, SGI91522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20374[] = {SGI91523, SGI0, SGI91524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91525, SGI91526, SGI91526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20375[] = {SGI91527, SGI0, SGI91528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91529, SGI91530, SGI91530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20376[] = {SGI0, SGI0, SGI91531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91532, SGI0, SGI0, SGI0, SGI91533, SGI91533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20377[] = {SGI91534, SGI91535, SGI91536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91537, SGI0, SGI0, SGI0, SGI91538, SGI91538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20378[] = {SGI91539, SGI91540, SGI91541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91542, SGI91542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20379[] = {SGI0, SGI91543, SGI91544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20380[] = {SGI91545, SGI91546, SGI91547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91548, SGI0, SGI0, SGI0, SGI91549, SGI91549, SGI0, SGI0, SGI0, SGI91550, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20381[] = {SGI91551, SGI91552, SGI91553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91554, SGI0, SGI0, SGI0, SGI91555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20382[] = {SGI91556, SGI91557, SGI91558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91559, SGI0, SGI0, SGI0, SGI91560, SGI91560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91561, SGI0, NULL};
const Segment * const SG20383[] = {SGI91562, SGI91563, SGI91564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91565, SGI0, SGI0, SGI0, SGI91566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20384[] = {SGI91567, SGI91568, SGI91569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91570, SGI0, SGI0, SGI0, SGI91571, SGI91571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20385[] = {SGI91572, SGI0, SGI91573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20386[] = {SGI91574, SGI91575, SGI91576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91577, SGI0, SGI0, SGI0, SGI91578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20387[] = {SGI91579, SGI91580, SGI91581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91582, SGI0, SGI0, SGI0, SGI91583, SGI91583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20388[] = {SGI91584, SGI91585, SGI91586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91587, SGI0, SGI0, SGI0, SGI91588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20389[] = {SGI91589, SGI91590, SGI91591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91592, SGI0, SGI0, SGI0, SGI91593, SGI91593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20390[] = {SGI91594, SGI91595, SGI91596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91597, SGI0, SGI0, SGI0, SGI91598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20391[] = {SGI91599, SGI91600, SGI91601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91602, SGI0, SGI0, SGI0, SGI91603, SGI91603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20392[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91604, SGI0, SGI0, SGI91605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91606, SGI0, NULL};
const Segment * const SG20393[] = {SGI91607, SGI91608, SGI91609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91610, SGI0, SGI0, SGI0, SGI91611, SGI91611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20394[] = {SGI91612, SGI91613, SGI91614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91615, SGI0, SGI0, SGI0, SGI91616, SGI91616, SGI91617, SGI0, SGI91618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20395[] = {SGI0, SGI91619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91620, SGI0, SGI0, SGI0, SGI91621, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20396[] = {SGI91622, SGI91623, SGI91624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91625, SGI0, SGI0, SGI0, SGI91626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20397[] = {SGI91627, SGI91628, SGI91629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91630, SGI0, SGI0, SGI0, SGI91631, SGI91632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20398[] = {SGI91633, SGI91634, SGI91635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91636, SGI0, SGI0, SGI0, SGI91637, SGI91637, SGI0, SGI91638, SGI0, NULL};
const Segment * const SG20399[] = {SGI91639, SGI0, SGI91640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91641, SGI0, SGI0, SGI0, SGI91642, SGI91642, SGI0, SGI0, SGI91643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20400[] = {SGI91644, SGI0, SGI91645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91646, SGI0, SGI0, SGI0, SGI91647, SGI91647, SGI0, SGI0, SGI91648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20401[] = {SGI91649, SGI0, SGI91650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91651, SGI0, SGI0, SGI0, SGI91652, SGI91652, SGI0, SGI0, SGI91653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20402[] = {SGI91654, SGI91655, SGI91656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91657, SGI0, SGI0, SGI0, SGI91658, SGI91658, SGI0, SGI0, NULL};
const Segment * const SG20403[] = {SGI91659, SGI91660, SGI91661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91662, SGI0, SGI0, SGI0, SGI91663, SGI91663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20404[] = {SGI91664, SGI0, SGI91665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91666, SGI91667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20405[] = {SGI91668, SGI0, SGI91669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91670, SGI91671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20406[] = {SGI91672, SGI0, SGI91673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91674, SGI91675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20407[] = {SGI91676, SGI0, SGI91677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91678, SGI91679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20408[] = {SGI91680, SGI0, SGI91681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91682, SGI91683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20409[] = {SGI91684, SGI0, SGI91685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91686, SGI0, SGI0, SGI0, SGI91687, SGI91687, SGI0, SGI0, SGI91688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20410[] = {SGI91689, SGI91690, SGI91691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91692, SGI0, SGI0, SGI0, SGI91693, SGI91693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91694, SGI0, NULL};
const Segment * const SG20411[] = {SGI91695, SGI0, SGI91696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91697, SGI0, SGI0, SGI0, SGI91698, SGI91698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91699, SGI0, NULL};
const Segment * const SG20412[] = {SGI91700, SGI0, SGI91701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91702, SGI91703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20413[] = {SGI91704, SGI91705, SGI91706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91707, SGI0, SGI0, SGI0, SGI91708, SGI91708, SGI0, SGI0, SGI0, SGI91709, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20414[] = {SGI91710, SGI0, SGI91711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91712, SGI91713, SGI91713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20415[] = {SGI91714, SGI91716, SGI91717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91719, SGI0, SGI0, SGI0, SGI91721, SGI91721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20416[] = {SGI91715, SGI0, SGI91718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91720, SGI91722, SGI91722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20417[] = {SGI91723, SGI91724, SGI91725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91726, SGI0, SGI0, SGI0, SGI91727, SGI91727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20418[] = {SGI91728, SGI0, SGI91729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91730, SGI91731, SGI91731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20419[] = {SGI91732, SGI0, SGI91733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91734, SGI91735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20420[] = {SGI91736, SGI0, SGI91737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91738, SGI91739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20421[] = {SGI91740, SGI0, SGI91741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91742, SGI91743, SGI91743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20422[] = {SGI91744, SGI0, SGI91745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91746, SGI91747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20423[] = {SGI91748, SGI0, SGI91749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91750, SGI91751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20424[] = {SGI91752, SGI91753, SGI91754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91755, SGI0, SGI0, SGI0, SGI91756, SGI91756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20425[] = {SGI91757, SGI91758, SGI91759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91760, SGI0, SGI0, SGI0, SGI91761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20426[] = {SGI91762, SGI0, SGI91763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20427[] = {SGI91764, SGI91765, SGI91766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91767, SGI91767, SGI0, SGI0, SGI91768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20428[] = {SGI91769, SGI0, SGI91770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91771, SGI91772, SGI91772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20429[] = {SGI91773, SGI91774, SGI91775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91776, SGI0, SGI0, SGI0, SGI91777, SGI91777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20430[] = {SGI91778, SGI91779, SGI91780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91781, SGI0, SGI0, SGI0, SGI91782, SGI91782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20431[] = {SGI91783, SGI0, SGI91784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20432[] = {SGI91785, SGI91786, SGI91787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91788, SGI0, SGI0, SGI0, SGI91789, SGI91789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20433[] = {SGI91790, SGI91791, SGI91792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91793, SGI0, SGI0, SGI0, SGI91794, SGI91794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20434[] = {SGI91795, SGI0, SGI91796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91797, SGI91798, SGI91798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20435[] = {SGI91799, SGI0, SGI91800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91801, SGI91802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20436[] = {SGI91803, SGI0, SGI91804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91805, SGI91806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20437[] = {SGI91807, SGI91808, SGI91809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91810, SGI0, SGI0, SGI0, SGI91811, SGI91811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20438[] = {SGI91812, SGI91813, SGI91814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91815, SGI0, SGI0, SGI0, SGI91816, SGI91816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20439[] = {SGI91817, SGI91818, SGI91819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91820, SGI91820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20440[] = {SGI91821, SGI0, SGI91822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20441[] = {SGI91823, SGI0, SGI91824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20442[] = {SGI91825, SGI0, SGI91826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91827, SGI0, SGI0, SGI0, SGI91828, SGI91828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20443[] = {SGI91829, SGI91830, SGI91831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91832, SGI0, SGI0, SGI0, SGI91833, SGI91833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20444[] = {SGI91834, SGI91835, SGI91836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91837, SGI91837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20445[] = {SGI91838, SGI0, SGI91839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91840, SGI91841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20446[] = {SGI91842, SGI0, SGI91843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20447[] = {SGI91844, SGI0, SGI91845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91849, SGI91850, SGI91850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20448[] = {SGI91846, SGI91847, SGI91848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91851, SGI0, SGI0, SGI0, SGI91852, SGI91852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20449[] = {SGI91853, SGI91854, SGI91855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91856, SGI0, SGI0, SGI0, SGI91857, SGI91857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20450[] = {SGI91858, SGI91859, SGI91860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91861, SGI0, SGI0, SGI0, SGI91862, SGI91862, SGI0, SGI0, SGI0, SGI91863, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20451[] = {SGI91864, SGI91865, SGI91866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91867, SGI0, SGI0, SGI0, SGI91868, SGI91868, SGI0, SGI0, SGI0, SGI91869, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20452[] = {SGI91870, SGI91871, SGI91872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91873, SGI0, SGI0, SGI0, SGI91874, SGI91874, SGI0, SGI0, SGI0, SGI91875, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20453[] = {SGI91876, SGI91877, SGI91878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91879, SGI91879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91880, SGI0, NULL};
const Segment * const SG20454[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91881, SGI0, SGI91882, SGI0, NULL};
const Segment * const SG20455[] = {SGI91883, SGI91884, SGI91885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91886, SGI0, SGI0, SGI0, SGI91887, SGI91887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20456[] = {SGI91888, SGI91889, SGI91890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91891, SGI0, SGI0, SGI0, SGI91892, SGI91892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20457[] = {SGI91893, SGI0, SGI91894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91895, SGI0, SGI0, SGI0, SGI91896, SGI91896, SGI0, SGI0, SGI91897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20458[] = {SGI91898, SGI91899, SGI91900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91901, SGI0, SGI0, SGI0, SGI91902, SGI91902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20459[] = {SGI91903, SGI91904, SGI91905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91906, SGI0, SGI0, SGI0, SGI91907, SGI91907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20460[] = {SGI91908, SGI0, SGI91909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91910, SGI91911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20461[] = {SGI91912, SGI91913, SGI91914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91915, SGI0, SGI0, SGI0, SGI91916, SGI91916, SGI91917, SGI0, SGI91918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20462[] = {SGI91919, SGI91920, SGI91921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91922, SGI0, SGI0, SGI0, SGI91923, SGI91923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91924, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20463[] = {SGI91925, SGI0, SGI91926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91927, SGI91928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20464[] = {SGI91929, SGI91930, SGI91931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91932, SGI0, SGI0, SGI0, SGI91933, SGI91933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20465[] = {SGI91934, SGI91935, SGI91936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91937, SGI0, SGI0, SGI0, SGI91938, SGI91938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20466[] = {SGI91939, SGI91940, SGI91941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91945, SGI91945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20467[] = {SGI91942, SGI91943, SGI91944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91946, SGI0, SGI0, SGI0, SGI91947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20468[] = {SGI91948, SGI91949, SGI91950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91951, SGI91952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20469[] = {SGI91953, SGI0, SGI91954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20470[] = {SGI0, SGI91955, SGI91956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20471[] = {SGI91957, SGI91958, SGI91959, SGI0, SGI0, SGI0, SGI91960, SGI91961, SGI0, SGI91962, SGI0, SGI0, SGI0, SGI91963, SGI91963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20472[] = {SGI91964, SGI0, SGI91965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91966, SGI91967, SGI91967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91968, NULL};
const Segment * const SG20473[] = {SGI91969, SGI0, SGI91970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91971, SGI91972, SGI91972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20474[] = {SGI91973, SGI0, SGI91974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91975, SGI91976, SGI91976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20475[] = {SGI91977, SGI0, SGI91978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91979, SGI91980, SGI91980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20476[] = {SGI91981, SGI0, SGI91982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91983, SGI91984, SGI91984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20477[] = {SGI91985, SGI0, SGI91986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91987, SGI91988, SGI91988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20478[] = {SGI91989, SGI0, SGI91990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91991, SGI91992, SGI91992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20479[] = {SGI91993, SGI91994, SGI91995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI91996, SGI0, SGI0, SGI0, SGI91997, SGI91997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20480[] = {SGI91998, SGI91999, SGI92000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92001, SGI92002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20481[] = {SGI92003, SGI92004, SGI92005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92006, SGI92006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20482[] = {SGI92007, SGI0, SGI92008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92009, SGI92010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20483[] = {SGI92011, SGI0, SGI92012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92013, SGI0, SGI0, SGI0, SGI92014, SGI92014, SGI0, SGI0, SGI92015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20484[] = {SGI92016, SGI92017, SGI92018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92019, SGI0, SGI0, SGI0, SGI92020, SGI92020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20485[] = {SGI92021, SGI92022, SGI92023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92024, SGI0, SGI0, SGI0, SGI92025, SGI92025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20486[] = {SGI92026, SGI0, SGI92027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92028, SGI92029, SGI92029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92030, NULL};
const Segment * const SG20487[] = {SGI92031, SGI0, SGI92032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92033, SGI92034, SGI92034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20488[] = {SGI92035, SGI0, SGI92036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92037, SGI92038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20489[] = {SGI92039, SGI0, SGI92040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92041, SGI92042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20490[] = {SGI92043, SGI92044, SGI92045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92046, SGI0, SGI0, SGI0, SGI92047, SGI92047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20491[] = {SGI92048, SGI0, SGI92049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92053, SGI92054, SGI92054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20492[] = {SGI92050, SGI92051, SGI92052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92055, SGI0, SGI0, SGI0, SGI92056, SGI92056, SGI92057, SGI0, SGI92058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20493[] = {SGI92060, SGI0, SGI92063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92065, SGI92066, SGI92066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20494[] = {SGI92059, SGI92061, SGI92062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92064, SGI0, SGI0, SGI0, SGI92067, SGI92067, SGI92068, SGI0, SGI92069, SGI0, SGI0, SGI0, SGI0, SGI92070, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20495[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92071, SGI0, SGI0, SGI92072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20496[] = {SGI92073, SGI92074, SGI92075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92076, SGI0, SGI0, SGI0, SGI92077, SGI92077, SGI0, SGI92078, SGI0, NULL};
const Segment * const SG20497[] = {SGI92079, SGI92080, SGI92081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92082, SGI0, SGI0, SGI0, SGI92083, SGI92083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92084, SGI0, NULL};
const Segment * const SG20498[] = {SGI92085, SGI92086, SGI92087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92088, SGI0, SGI0, SGI0, SGI92089, SGI92089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20499[] = {SGI92090, SGI92091, SGI92092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92093, SGI0, SGI0, SGI0, SGI92094, SGI92094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20500[] = {SGI92095, SGI92096, SGI92097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92098, SGI0, SGI0, SGI0, SGI92099, SGI92099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20501[] = {SGI92100, SGI0, SGI92101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92102, SGI0, SGI0, SGI0, SGI92103, SGI92103, SGI0, SGI0, SGI92104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20502[] = {SGI92105, SGI0, SGI92106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92107, SGI92108, SGI92108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20503[] = {SGI92109, SGI0, SGI92110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92111, SGI92112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20504[] = {SGI92113, SGI92114, SGI92115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92116, SGI0, SGI0, SGI0, SGI92117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20505[] = {SGI92118, SGI92119, SGI92120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92121, SGI0, SGI0, SGI0, SGI92122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20506[] = {SGI92123, SGI92124, SGI92125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92126, SGI0, SGI0, SGI0, SGI92127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20507[] = {SGI92128, SGI92129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20508[] = {SGI92131, SGI0, SGI92132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20509[] = {SGI92133, SGI0, SGI92134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92135, SGI92136, SGI92136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20510[] = {SGI92137, SGI92138, SGI92139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92140, SGI0, SGI0, SGI0, SGI92141, SGI92141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20511[] = {SGI92142, SGI92143, SGI92144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92147, SGI0, SGI0, SGI0, SGI92149, SGI92149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20512[] = {SGI92145, SGI92146, SGI92148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92150, SGI92150, SGI0, SGI0, SGI92151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20513[] = {SGI92152, SGI92153, SGI92154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92155, SGI0, SGI0, SGI0, SGI92156, SGI92156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20514[] = {SGI92157, SGI92158, SGI92159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92160, SGI0, SGI0, SGI0, SGI92161, SGI92161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20515[] = {SGI92162, SGI0, SGI92163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92164, SGI92165, SGI92165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20516[] = {SGI92166, SGI0, SGI92167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92168, SGI92169, SGI92169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20517[] = {SGI92170, SGI0, SGI92172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92175, SGI0, SGI0, SGI0, SGI92177, SGI92177, SGI0, SGI0, SGI92179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20518[] = {SGI92171, SGI92173, SGI92174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92176, SGI0, SGI0, SGI0, SGI92178, SGI92178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92180, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20519[] = {SGI92181, SGI92182, SGI92183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92184, SGI0, SGI0, SGI0, SGI92185, SGI92185, SGI0, SGI0, SGI92186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20520[] = {SGI92187, SGI0, SGI92188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92189, SGI0, SGI0, SGI0, SGI92190, SGI92190, SGI0, SGI0, SGI92191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20521[] = {SGI92192, SGI92193, SGI92194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92195, SGI0, SGI0, SGI0, SGI92196, SGI92197, SGI0, SGI0, SGI92198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20522[] = {SGI92199, SGI92200, SGI92201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92202, SGI92202, SGI0, SGI0, SGI92203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20523[] = {SGI92204, SGI92205, SGI92206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92207, SGI92207, SGI0, SGI0, SGI92208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20524[] = {SGI92209, SGI92210, SGI92211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92212, SGI0, SGI0, SGI0, SGI92213, SGI92213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20525[] = {SGI92214, SGI0, SGI92215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92216, SGI92217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20526[] = {SGI92218, SGI92219, SGI92220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92221, SGI0, SGI0, SGI0, SGI92222, SGI92222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92223, SGI0, NULL};
const Segment * const SG20527[] = {SGI92224, SGI0, SGI92225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92226, SGI92227, SGI92227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20528[] = {SGI92228, SGI92229, SGI92230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92231, SGI0, SGI0, SGI0, SGI92232, SGI92232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20529[] = {SGI92233, SGI92234, SGI92235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92236, SGI0, SGI0, SGI0, SGI92237, SGI92237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20530[] = {SGI92238, SGI92239, SGI92240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92241, SGI0, SGI0, SGI0, SGI92242, SGI92242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20531[] = {SGI92243, SGI92244, SGI92245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92246, SGI0, SGI0, SGI0, SGI92247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20532[] = {SGI92248, SGI92249, SGI92250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92251, SGI0, SGI0, SGI0, SGI92252, SGI92252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20533[] = {SGI92253, SGI92254, SGI92255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92256, SGI0, SGI0, SGI0, SGI92257, SGI92257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20534[] = {SGI92258, SGI92259, SGI92260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92261, SGI0, SGI0, SGI0, SGI92263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20535[] = {SGI92262, SGI92264, SGI92265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92266, SGI0, SGI0, SGI0, SGI92267, SGI92267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20536[] = {SGI92268, SGI92269, SGI92270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92271, SGI0, SGI0, SGI0, SGI92272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20537[] = {SGI92273, SGI92274, SGI92275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92276, SGI0, SGI0, SGI0, SGI92277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20538[] = {SGI92278, SGI92279, SGI92280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92281, SGI0, SGI0, SGI0, SGI92282, SGI92282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20539[] = {SGI92283, SGI92284, SGI92285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92286, SGI0, SGI0, SGI0, SGI92287, SGI92287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20540[] = {SGI92288, SGI92289, SGI92290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92291, SGI0, SGI0, SGI0, SGI92292, SGI92292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20541[] = {SGI92293, SGI92294, SGI92295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92296, SGI0, SGI0, SGI0, SGI92297, SGI92297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20542[] = {SGI92298, SGI92299, SGI92300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92301, SGI0, SGI0, SGI0, SGI92302, SGI92302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20543[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92303, SGI0, SGI0, SGI92304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20544[] = {SGI92305, SGI92306, SGI92307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92308, SGI0, SGI0, SGI0, SGI92309, SGI92309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20545[] = {SGI92310, SGI92311, SGI92312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92313, SGI92313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20546[] = {SGI92314, SGI0, SGI92315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92316, SGI92317, SGI92317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20547[] = {SGI92318, SGI0, SGI92319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92320, SGI0, SGI0, SGI0, SGI92321, SGI92321, SGI0, SGI0, SGI92322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20548[] = {SGI92323, SGI92324, SGI92325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92326, SGI0, SGI0, SGI0, SGI92327, SGI92327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20549[] = {SGI92328, SGI92329, SGI92330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92331, SGI92331, SGI0, SGI0, SGI92332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20550[] = {SGI92333, SGI92334, SGI92335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92336, SGI0, SGI0, SGI0, SGI92337, SGI92337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20551[] = {SGI92338, SGI0, SGI92339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20552[] = {SGI92340, SGI92341, SGI92342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92343, SGI0, SGI0, SGI0, SGI92344, SGI92344, SGI0, SGI92345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20553[] = {SGI92346, SGI92347, SGI92348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92349, SGI92349, SGI0, SGI0, SGI92350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20554[] = {SGI92351, SGI92352, SGI92353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92354, SGI92354, SGI0, SGI0, SGI92355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20555[] = {SGI92356, SGI0, SGI92357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92358, SGI0, SGI0, SGI0, SGI92359, SGI92359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92360, SGI0, SGI0, NULL};
const Segment * const SG20556[] = {SGI92361, SGI0, SGI92362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92363, SGI0, SGI0, SGI0, SGI92364, SGI92364, SGI0, SGI0, SGI92365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20557[] = {SGI92366, SGI92367, SGI92368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92370, SGI0, SGI0, SGI0, SGI92373, SGI92374, SGI0, SGI0, SGI92376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20558[] = {SGI92369, SGI92371, SGI92372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92375, SGI0, SGI0, SGI0, SGI92377, SGI92377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20559[] = {SGI92378, SGI92379, SGI92380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92381, SGI92381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20560[] = {SGI92382, SGI92383, SGI92384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92385, SGI0, SGI0, SGI0, SGI92386, SGI92386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20561[] = {SGI92387, SGI0, SGI92388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92389, SGI92390, SGI92390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20562[] = {SGI92391, SGI92392, SGI92393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92394, SGI0, SGI0, SGI0, SGI92395, SGI92395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20563[] = {SGI92396, SGI0, SGI92397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20564[] = {SGI92398, SGI0, SGI92399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92400, SGI92401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20565[] = {SGI92402, SGI0, SGI92403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92404, SGI92405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20566[] = {SGI92406, SGI0, SGI92407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92408, SGI92409, SGI92409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20567[] = {SGI92410, SGI92411, SGI92412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92413, SGI0, SGI0, SGI0, SGI92414, SGI92414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20568[] = {SGI92415, SGI92416, SGI92417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92418, SGI0, SGI0, SGI0, SGI92419, SGI92419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20569[] = {SGI92420, SGI92421, SGI92422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92423, SGI0, SGI0, SGI0, SGI92424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20570[] = {SGI92425, SGI92426, SGI92427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92428, SGI0, SGI0, SGI0, SGI92429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20571[] = {SGI92430, SGI0, SGI92431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20572[] = {SGI92432, SGI92433, SGI92434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92435, SGI0, SGI0, SGI0, SGI92436, SGI92437, SGI0, SGI0, SGI92438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20573[] = {SGI92439, SGI0, SGI92440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92441, SGI0, SGI0, SGI0, SGI92442, SGI92443, SGI0, SGI0, SGI92444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20574[] = {SGI92445, SGI92446, SGI92447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92448, SGI0, SGI0, SGI0, SGI92449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20575[] = {SGI92450, SGI92451, SGI92452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92453, SGI0, SGI0, SGI0, SGI92454, SGI92454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20576[] = {SGI92455, SGI92456, SGI92457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92458, SGI0, SGI0, SGI0, SGI92459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20577[] = {SGI92460, SGI92461, SGI92462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92463, SGI0, SGI0, SGI0, SGI92464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20578[] = {SGI92465, SGI92466, SGI92467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92468, SGI0, SGI0, SGI0, SGI92469, SGI92469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20579[] = {SGI92470, SGI0, SGI92471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20580[] = {SGI92472, SGI0, SGI92473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92474, SGI0, SGI0, SGI0, SGI92475, SGI92475, SGI0, SGI0, SGI92476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20581[] = {SGI92477, SGI0, SGI92478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92479, SGI92480, SGI92480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20582[] = {SGI92481, SGI92482, SGI92483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92484, SGI0, SGI0, SGI0, SGI92485, SGI92485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20583[] = {SGI92486, SGI92487, SGI92488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92489, SGI0, SGI0, SGI0, SGI92490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20584[] = {SGI92491, SGI92492, SGI92493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92494, SGI0, SGI0, SGI0, SGI92495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20585[] = {SGI92496, SGI92497, SGI92498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92499, SGI0, SGI0, SGI0, SGI92500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20586[] = {SGI92501, SGI0, SGI92502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92503, SGI92504, SGI92504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92505, NULL};
const Segment * const SG20587[] = {SGI92506, SGI0, SGI92507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92508, SGI92509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20588[] = {SGI92510, SGI0, SGI92511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92512, SGI92513, SGI92513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20589[] = {SGI92514, SGI92515, SGI92516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92517, SGI0, SGI0, SGI0, SGI92518, SGI92518, SGI92519, SGI0, SGI92520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20590[] = {SGI92521, SGI92522, SGI92523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92524, SGI0, SGI0, SGI0, SGI92525, SGI92525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92526, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20591[] = {SGI92527, SGI92528, SGI92529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92530, SGI0, SGI0, SGI0, SGI92531, SGI92531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20592[] = {SGI92532, SGI0, SGI92533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20593[] = {SGI92534, SGI92535, SGI92536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92537, SGI0, SGI0, SGI0, SGI92538, SGI92538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20594[] = {SGI92539, SGI92540, SGI92541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92542, SGI92543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20595[] = {SGI92544, SGI92545, SGI92546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92547, SGI0, SGI0, SGI0, SGI92548, SGI92548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20596[] = {SGI92549, SGI92550, SGI92551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92552, SGI0, SGI0, SGI0, SGI92553, SGI92553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20597[] = {SGI92554, SGI92555, SGI92556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92557, SGI0, SGI0, SGI0, SGI92558, SGI92558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20598[] = {SGI92559, SGI0, SGI92560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92561, SGI92562, SGI92562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20599[] = {SGI92563, SGI92564, SGI92565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92566, SGI0, SGI0, SGI0, SGI92567, SGI92567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20600[] = {SGI92568, SGI0, SGI92569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92570, SGI92571, SGI92571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20601[] = {SGI92572, SGI0, SGI92573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92574, SGI92575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20602[] = {SGI92576, SGI0, SGI92577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92578, SGI92579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20603[] = {SGI92580, SGI0, SGI92581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92582, SGI92583, SGI92583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20604[] = {SGI92584, SGI0, SGI92585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92586, SGI92587, SGI92587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20605[] = {SGI92588, SGI0, SGI92589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92590, SGI92591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20606[] = {SGI92592, SGI0, SGI92593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92594, SGI0, SGI0, SGI0, SGI92595, SGI92595, SGI0, SGI0, SGI92596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20607[] = {SGI92597, SGI92598, SGI92599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92600, SGI0, SGI0, SGI0, SGI92601, SGI92602, SGI0, SGI0, SGI92603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20608[] = {SGI92604, SGI0, SGI92605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92606, SGI92607, SGI92607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20609[] = {SGI92608, SGI92609, SGI92610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92611, SGI92611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20610[] = {SGI92612, SGI92613, SGI92614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92615, SGI0, SGI0, SGI0, SGI92616, SGI92616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20611[] = {SGI92617, SGI92618, SGI92619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92620, SGI0, SGI0, SGI0, SGI92621, SGI92621, SGI92622, SGI0, SGI92623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20612[] = {SGI92624, SGI92625, SGI92626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92627, SGI0, SGI0, SGI0, SGI92628, SGI92628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20613[] = {SGI92629, SGI92630, SGI92631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92632, SGI0, SGI0, SGI0, SGI92633, SGI92633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20614[] = {SGI92634, SGI92635, SGI92636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92637, SGI0, SGI0, SGI0, SGI92638, SGI92638, SGI92639, SGI0, SGI92640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20615[] = {SGI92641, SGI92642, SGI92643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92644, SGI0, SGI0, SGI0, SGI92645, SGI92645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92646, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20616[] = {SGI92647, SGI92648, SGI92649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92650, SGI0, SGI0, SGI0, SGI92651, SGI92651, SGI92652, SGI0, SGI92653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20617[] = {SGI92654, SGI92655, SGI92656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92657, SGI0, SGI0, SGI0, SGI92658, SGI92658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92659, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20618[] = {SGI92660, SGI92661, SGI92662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92663, SGI0, SGI0, SGI0, SGI92664, SGI92664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20619[] = {SGI92665, SGI0, SGI92666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92667, SGI92668, SGI92668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20620[] = {SGI92669, SGI0, SGI92670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20621[] = {SGI92671, SGI92672, SGI92673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92674, SGI0, SGI0, SGI0, SGI92675, SGI92675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20622[] = {SGI92676, SGI92677, SGI92678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92679, SGI0, SGI0, SGI0, SGI92680, SGI92680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20623[] = {SGI92681, SGI92682, SGI92683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92684, SGI0, SGI0, SGI0, SGI92685, SGI92685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92686, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20624[] = {SGI92688, SGI92689, SGI92691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92693, SGI0, SGI0, SGI0, SGI92695, SGI92695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20625[] = {SGI92687, SGI92690, SGI92692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92694, SGI0, SGI0, SGI0, SGI92696, SGI92696, SGI92697, SGI0, SGI92698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20626[] = {SGI92699, SGI92700, SGI92701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92702, SGI0, SGI0, SGI0, SGI92703, SGI92703, SGI92704, SGI0, SGI92705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20627[] = {SGI92706, SGI92707, SGI92708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92709, SGI92709, SGI0, SGI0, SGI92710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20628[] = {SGI92711, SGI0, SGI92712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92713, SGI92714, SGI92714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92715, NULL};
const Segment * const SG20629[] = {SGI92716, SGI0, SGI92717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92718, SGI92719, SGI92719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92720, NULL};
const Segment * const SG20630[] = {SGI92721, SGI92722, SGI92723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92724, SGI92724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92725, SGI0, NULL};
const Segment * const SG20631[] = {SGI92726, SGI92727, SGI92728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92729, SGI0, SGI0, SGI0, SGI92730, SGI92730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20632[] = {SGI92731, SGI92732, SGI92733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92734, SGI0, SGI0, SGI0, SGI92735, SGI92735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20633[] = {SGI92736, SGI0, SGI92737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92738, SGI92739, SGI92739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20634[] = {SGI92740, SGI0, SGI92741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92742, SGI92743, SGI92743, SGI0, SGI92744, SGI0, NULL};
const Segment * const SG20635[] = {SGI92745, SGI0, SGI92746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92747, SGI0, SGI0, SGI0, SGI92748, SGI92748, SGI0, SGI0, SGI92749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20636[] = {SGI92750, SGI0, SGI92751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92752, SGI92753, SGI92753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20637[] = {SGI92754, SGI0, SGI92755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92756, SGI92757, SGI92757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20638[] = {SGI92758, SGI0, SGI92759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92760, SGI92761, SGI92761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20639[] = {SGI92762, SGI0, SGI92763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92764, SGI92765, SGI92765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20640[] = {SGI92766, SGI92767, SGI92768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92771, SGI0, SGI0, SGI0, SGI92773, SGI92773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20641[] = {SGI92769, SGI92770, SGI92772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92774, SGI0, SGI0, SGI0, SGI92775, SGI92775, SGI92776, SGI0, SGI92777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20642[] = {SGI92778, SGI92779, SGI92780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92781, SGI92781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20643[] = {SGI92782, SGI92783, SGI92784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92785, SGI0, SGI0, SGI0, SGI92786, SGI92786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20644[] = {SGI92787, SGI92788, SGI92789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92790, SGI92790, SGI0, SGI0, SGI92791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20645[] = {SGI0, SGI92792, SGI92793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92794, SGI0, SGI0, SGI0, SGI92795, SGI92795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20646[] = {SGI92796, SGI92797, SGI92798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92799, SGI92799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20647[] = {SGI0, SGI92800, SGI92801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92802, SGI0, SGI0, SGI0, SGI92803, SGI92803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20648[] = {SGI92804, SGI92805, SGI92806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92807, SGI0, SGI0, SGI0, SGI92808, SGI92808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20649[] = {SGI92809, SGI92810, SGI92811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92812, SGI0, SGI0, SGI0, SGI92813, SGI92813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20650[] = {SGI92814, SGI92815, SGI92816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92817, SGI0, SGI0, SGI0, SGI92818, SGI92818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20651[] = {SGI92819, SGI0, SGI92820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92821, SGI92822, SGI92822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20652[] = {SGI92823, SGI0, SGI92824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92825, SGI92826, SGI92826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20653[] = {SGI92827, SGI0, SGI92828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92829, SGI92830, SGI92830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20654[] = {SGI92831, SGI0, SGI92832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92833, SGI92834, SGI92834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20655[] = {SGI92835, SGI0, SGI92836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92837, SGI92838, SGI92838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20656[] = {SGI92839, SGI92840, SGI92841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92844, SGI0, SGI0, SGI0, SGI92845, SGI92845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20657[] = {SGI92842, SGI0, SGI92843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92846, SGI92847, SGI92847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20658[] = {SGI92848, SGI92849, SGI92850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92851, SGI0, SGI0, SGI0, SGI92852, SGI92852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20659[] = {SGI92853, SGI0, SGI92854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92855, SGI92856, SGI92856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20660[] = {SGI92857, SGI0, SGI92858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92859, SGI92860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20661[] = {SGI92861, SGI0, SGI92862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92863, SGI92864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20662[] = {SGI92865, SGI0, SGI92866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92867, SGI92868, SGI92868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20663[] = {SGI92869, SGI92870, SGI92871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92872, SGI0, SGI0, SGI0, SGI92873, SGI92873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20664[] = {SGI92874, SGI0, SGI92875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92876, SGI92877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20665[] = {SGI92878, SGI92879, SGI92880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92881, SGI0, SGI0, SGI0, SGI92882, SGI92882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92883, SGI0, NULL};
const Segment * const SG20666[] = {SGI92884, SGI92885, SGI92886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92887, SGI0, SGI0, SGI0, SGI92888, SGI92888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20667[] = {SGI92889, SGI92890, SGI92891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92892, SGI92892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20668[] = {SGI0, SGI92893, SGI92894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92895, SGI0, SGI0, SGI0, SGI92896, SGI92896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20669[] = {SGI92898, SGI92900, SGI92901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92902, SGI0, SGI0, SGI0, SGI92905, SGI92905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20670[] = {SGI92897, SGI0, SGI92899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92903, SGI92904, SGI92904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20671[] = {SGI92906, SGI0, SGI92907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92908, SGI92909, SGI92909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20672[] = {SGI92910, SGI92911, SGI92912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92913, SGI0, SGI0, SGI0, SGI92914, SGI92914, SGI92915, SGI0, SGI92916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20673[] = {SGI92917, SGI92918, SGI92919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92920, SGI0, SGI0, SGI0, SGI92921, SGI92921, SGI92922, SGI0, SGI92923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20674[] = {SGI92924, SGI92925, SGI92926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92927, SGI0, SGI0, SGI0, SGI92928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20675[] = {SGI92929, SGI92930, SGI92931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92932, SGI0, SGI0, SGI0, SGI92933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20676[] = {SGI92934, SGI0, SGI92935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92936, SGI92937, SGI92937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20677[] = {SGI92938, SGI0, SGI92939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92940, SGI92941, SGI92941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20678[] = {SGI92942, SGI0, SGI92943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92944, SGI92945, SGI92945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20679[] = {SGI0, SGI92946, SGI92947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92948, SGI0, SGI0, SGI0, SGI92949, SGI92949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20680[] = {SGI0, SGI92950, SGI92951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92952, SGI0, SGI0, SGI0, SGI92953, SGI92953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20681[] = {SGI92954, SGI92955, SGI92956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92957, SGI0, SGI0, SGI0, SGI92958, SGI92958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20682[] = {SGI92959, SGI92960, SGI92961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92962, SGI0, SGI0, SGI0, SGI92963, SGI92963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20683[] = {SGI92964, SGI92965, SGI92966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92970, SGI92970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92972, SGI0, SGI0, NULL};
const Segment * const SG20684[] = {SGI92967, SGI92968, SGI92969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92971, SGI0, SGI0, SGI0, SGI92973, SGI92973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20685[] = {SGI92975, SGI92977, SGI92979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92980, SGI0, SGI0, SGI0, SGI92982, SGI92982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20686[] = {SGI92974, SGI92976, SGI92978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92981, SGI92981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92983, SGI0, SGI0, NULL};
const Segment * const SG20687[] = {SGI92984, SGI92985, SGI92986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92987, SGI0, SGI0, SGI0, SGI92989, SGI92989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20688[] = {SGI92988, SGI92990, SGI92991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92992, SGI0, SGI0, SGI0, SGI92993, SGI92993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20689[] = {SGI92994, SGI0, SGI92995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20690[] = {SGI92996, SGI0, SGI92997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI92998, SGI92999, SGI92999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20691[] = {SGI93000, SGI93001, SGI93002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93003, SGI0, SGI0, SGI0, SGI93004, SGI93004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93005, SGI0, NULL};
const Segment * const SG20692[] = {SGI93006, SGI93007, SGI93008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93009, SGI0, SGI0, SGI0, SGI93010, SGI93010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20693[] = {SGI93011, SGI93012, SGI93013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93014, SGI0, SGI0, SGI0, SGI93015, SGI93015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20694[] = {SGI93016, SGI93017, SGI93018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93019, SGI0, SGI0, SGI0, SGI93020, SGI93020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20695[] = {SGI93021, SGI93022, SGI93023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93024, SGI0, SGI0, SGI0, SGI93025, SGI93025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20696[] = {SGI93026, SGI93027, SGI93028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93029, SGI0, SGI0, SGI0, SGI93030, SGI93030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20697[] = {SGI93031, SGI0, SGI93032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93033, SGI93034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93038, SGI0, NULL};
const Segment * const SG20698[] = {SGI93035, SGI93036, SGI93037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93039, SGI0, SGI0, SGI0, SGI93040, SGI93040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20699[] = {SGI93041, SGI0, SGI93042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93043, SGI93044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93045, SGI0, NULL};
const Segment * const SG20700[] = {SGI93046, SGI93047, SGI93048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93049, SGI0, SGI0, SGI0, SGI93050, SGI93050, SGI93051, SGI0, SGI93052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20701[] = {SGI93053, SGI93054, SGI93055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93056, SGI0, SGI0, SGI0, SGI93057, SGI93057, SGI93058, SGI0, SGI93059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20702[] = {SGI93060, SGI0, SGI93061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93062, SGI93063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20703[] = {SGI93064, SGI0, SGI93065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93066, SGI93067, SGI93067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20704[] = {SGI93068, SGI0, SGI93069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93070, SGI93071, SGI93071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20705[] = {SGI93072, SGI0, SGI93073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93074, SGI93075, SGI93075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20706[] = {SGI93076, SGI0, SGI93077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93078, SGI93079, SGI93079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20707[] = {SGI93080, SGI0, SGI93081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93082, SGI93083, SGI93083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20708[] = {SGI93084, SGI0, SGI93085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93086, SGI93087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20709[] = {SGI93088, SGI93089, SGI93090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93091, SGI0, SGI0, SGI0, SGI93092, SGI93092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93093, SGI0, NULL};
const Segment * const SG20710[] = {SGI93094, SGI93095, SGI93096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93097, SGI0, SGI0, SGI0, SGI93098, SGI93098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93099, SGI0, NULL};
const Segment * const SG20711[] = {SGI93100, SGI93101, SGI93102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93103, SGI0, SGI0, SGI0, SGI93104, SGI93104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20712[] = {SGI93105, SGI93106, SGI93107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93108, SGI0, SGI0, SGI0, SGI93109, SGI93109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20713[] = {SGI93110, SGI93111, SGI93112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93113, SGI93113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20714[] = {SGI93114, SGI93115, SGI93116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93117, SGI0, SGI0, SGI0, SGI93118, SGI93118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20715[] = {SGI0, SGI93119, SGI93120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20716[] = {SGI93121, SGI0, SGI93122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93123, SGI93125, SGI93125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20717[] = {SGI93124, SGI93126, SGI93127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93128, SGI0, SGI0, SGI0, SGI93129, SGI93129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20718[] = {SGI93130, SGI0, SGI93131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93132, SGI93133, SGI93133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20719[] = {SGI93134, SGI0, SGI93135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93136, SGI93137, SGI93137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20720[] = {SGI93138, SGI0, SGI93139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93140, SGI93141, SGI93141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20721[] = {SGI93142, SGI0, SGI93143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93144, SGI93145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20722[] = {SGI93146, SGI0, SGI93147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93148, SGI0, SGI0, SGI0, SGI93149, SGI93149, SGI0, SGI0, SGI93150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20723[] = {SGI93151, SGI0, SGI93152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20724[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93153, SGI0, SGI93155, SGI0, NULL};
const Segment * const SG20725[] = {SGI93154, SGI93156, SGI93157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93158, SGI0, SGI0, SGI0, SGI93159, SGI93159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20726[] = {SGI93160, SGI93161, SGI93162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93163, SGI0, SGI0, SGI0, SGI93164, SGI93164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20727[] = {SGI93165, SGI93166, SGI93167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93168, SGI0, SGI0, SGI0, SGI0, SGI93169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20728[] = {SGI93170, SGI0, SGI93171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93172, SGI93173, SGI93173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20729[] = {SGI93174, SGI0, SGI93175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93176, SGI0, SGI0, SGI0, SGI93177, SGI93177, SGI0, SGI0, SGI93178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20730[] = {SGI93179, SGI93180, SGI93181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93182, SGI0, SGI0, SGI0, SGI93183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20731[] = {SGI93184, SGI0, SGI93185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20732[] = {SGI93186, SGI93187, SGI93188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93192, SGI0, SGI0, SGI0, SGI93193, SGI93194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20733[] = {SGI93189, SGI93190, SGI93191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93195, SGI93195, SGI0, SGI0, SGI93196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20734[] = {SGI93197, SGI93198, SGI93199, SGI0, SGI0, SGI0, SGI93200, SGI93201, SGI0, SGI93202, SGI0, SGI0, SGI0, SGI93203, SGI93203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20735[] = {SGI93204, SGI0, SGI93205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93206, SGI93207, SGI93207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20736[] = {SGI93208, SGI93209, SGI93210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93211, SGI0, SGI0, SGI0, SGI93214, SGI93214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20737[] = {SGI93212, SGI0, SGI93213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93215, SGI93216, SGI93216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20738[] = {SGI93217, SGI0, SGI93218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93219, SGI93220, SGI93220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20739[] = {SGI93221, SGI0, SGI93222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93223, SGI0, SGI0, SGI0, SGI93224, SGI93224, SGI0, SGI0, SGI93225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20740[] = {SGI93226, SGI93227, SGI93228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93229, SGI0, SGI0, SGI0, SGI93230, SGI93231, SGI0, SGI0, SGI93232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20741[] = {SGI93233, SGI93234, SGI93235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93236, SGI93236, SGI0, SGI0, SGI93237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20742[] = {SGI93238, SGI0, SGI93239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93240, SGI93241, SGI93241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20743[] = {SGI93242, SGI0, SGI93243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93244, SGI93245, SGI93245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20744[] = {SGI93246, SGI0, SGI93247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93248, SGI93249, SGI93249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20745[] = {SGI93250, SGI0, SGI93251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93252, SGI93253, SGI93253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20746[] = {SGI93254, SGI0, SGI93255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93256, SGI93257, SGI93257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20747[] = {SGI0, SGI93258, SGI0, SGI93259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20748[] = {SGI93260, SGI0, SGI93261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93262, SGI0, SGI0, SGI0, SGI93263, SGI93263, SGI0, SGI0, SGI93264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20749[] = {SGI93265, SGI93266, SGI93267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93268, SGI0, SGI0, SGI0, SGI93269, SGI93269, SGI0, SGI0, SGI0, SGI0, SGI93270, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20750[] = {SGI93271, SGI93272, SGI93273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93274, SGI0, SGI0, SGI0, SGI93275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20751[] = {SGI93276, SGI0, SGI93277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93278, SGI0, SGI0, SGI0, SGI93279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20752[] = {SGI93280, SGI93281, SGI93282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93283, SGI0, SGI0, SGI0, SGI93284, SGI93284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20753[] = {SGI93285, SGI93286, SGI93287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93288, SGI0, SGI0, SGI0, SGI93289, SGI93289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20754[] = {SGI93290, SGI93291, SGI93292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93293, SGI0, SGI0, SGI0, SGI93294, SGI93294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20755[] = {SGI93295, SGI93296, SGI93297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93298, SGI0, SGI0, SGI0, SGI93299, SGI93299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20756[] = {SGI93300, SGI93301, SGI93302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93303, SGI93303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20757[] = {SGI93304, SGI93305, SGI93306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93307, SGI0, SGI0, SGI0, SGI93308, SGI93308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20758[] = {SGI93309, SGI93310, SGI93311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93312, SGI0, SGI0, SGI0, SGI93313, SGI93313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20759[] = {SGI93314, SGI93315, SGI93316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93317, SGI0, SGI0, SGI0, SGI93318, SGI93318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20760[] = {SGI93319, SGI0, SGI93320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20761[] = {SGI93321, SGI0, SGI93322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20762[] = {SGI93323, SGI93324, SGI93325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93326, SGI0, SGI0, SGI0, SGI93327, SGI93327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20763[] = {SGI93328, SGI93329, SGI93330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93331, SGI0, SGI0, SGI0, SGI93332, SGI93332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20764[] = {SGI93333, SGI93334, SGI93335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93336, SGI0, SGI0, SGI0, SGI93337, SGI93337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20765[] = {SGI93338, SGI93339, SGI93340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93341, SGI0, SGI0, SGI0, SGI93342, SGI93342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20766[] = {SGI93343, SGI0, SGI93344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93345, SGI93346, SGI93346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20767[] = {SGI93347, SGI0, SGI93348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93349, SGI93350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20768[] = {SGI93351, SGI93352, SGI93353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93354, SGI93354, SGI0, SGI0, SGI93355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20769[] = {SGI93356, SGI93357, SGI93358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93359, SGI0, SGI0, SGI0, SGI93360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20770[] = {SGI93361, SGI0, SGI93362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93363, SGI93364, SGI93364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20771[] = {SGI93365, SGI0, SGI93366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93367, SGI93368, SGI93368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20772[] = {SGI93369, SGI0, SGI93370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93371, SGI93372, SGI93372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20773[] = {SGI93373, SGI0, SGI93374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93375, SGI93376, SGI93376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20774[] = {SGI93377, SGI0, SGI93378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93379, SGI93380, SGI93380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20775[] = {SGI93381, SGI0, SGI93382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93383, SGI93384, SGI93384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20776[] = {SGI93385, SGI93386, SGI93387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93388, SGI0, SGI0, SGI0, SGI93389, SGI93389, SGI93390, SGI0, SGI93391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20777[] = {SGI93392, SGI93393, SGI93394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93395, SGI0, SGI0, SGI0, SGI93396, SGI93396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20778[] = {SGI93397, SGI93398, SGI93399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93400, SGI0, SGI0, SGI0, SGI93401, SGI93401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20779[] = {SGI93402, SGI93403, SGI93404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93405, SGI0, SGI0, SGI0, SGI93406, SGI93406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20780[] = {SGI93407, SGI93408, SGI93409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93410, SGI0, SGI0, SGI0, SGI93411, SGI93411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20781[] = {SGI93413, SGI93414, SGI93416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93417, SGI0, SGI0, SGI0, SGI93419, SGI93419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20782[] = {SGI93412, SGI0, SGI93415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93418, SGI0, SGI0, SGI0, SGI93420, SGI93420, SGI0, SGI0, SGI93421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20783[] = {SGI93422, SGI0, SGI93423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93427, SGI93428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20784[] = {SGI93424, SGI93425, SGI93426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93429, SGI93430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20785[] = {SGI93431, SGI93432, SGI93433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93434, SGI93434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20786[] = {SGI93435, SGI0, SGI93436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93437, SGI93438, SGI93438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20787[] = {SGI93439, SGI0, SGI93440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93441, SGI93442, SGI93442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20788[] = {SGI93443, SGI0, SGI93444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93445, SGI93446, SGI93446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20789[] = {SGI93447, SGI93448, SGI93449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93450, SGI0, SGI0, SGI0, SGI93451, SGI93451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20790[] = {SGI93452, SGI93453, SGI93454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93455, SGI0, SGI0, SGI0, SGI93456, SGI93457, SGI0, SGI0, SGI93458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20791[] = {SGI93459, SGI0, SGI93460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93461, SGI93462, SGI93462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20792[] = {SGI93463, SGI0, SGI93464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93465, SGI93466, SGI93466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20793[] = {SGI93467, SGI0, SGI93468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93469, SGI93470, SGI93470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20794[] = {SGI93471, SGI93472, SGI93473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93474, SGI0, SGI0, SGI0, SGI93475, SGI93475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20795[] = {SGI93476, SGI93477, SGI93478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93479, SGI0, SGI0, SGI0, SGI93480, SGI93480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20796[] = {SGI93481, SGI93482, SGI93483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93484, SGI0, SGI0, SGI0, SGI93485, SGI93485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20797[] = {SGI0, SGI93486, SGI93487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20798[] = {SGI93488, SGI93489, SGI93490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93491, SGI0, SGI0, SGI0, SGI93492, SGI93492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20799[] = {SGI93493, SGI93494, SGI93495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93496, SGI93496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20800[] = {SGI93497, SGI93498, SGI93499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93500, SGI93501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20801[] = {SGI93502, SGI0, SGI93503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93504, SGI0, SGI0, SGI0, SGI93505, SGI93505, SGI0, SGI0, SGI93506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20802[] = {SGI93507, SGI93508, SGI93509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93510, SGI0, SGI0, SGI0, SGI93511, SGI93512, SGI0, SGI0, SGI93513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20803[] = {SGI93514, SGI0, SGI93515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93516, SGI0, SGI0, SGI0, SGI93517, SGI93517, SGI0, SGI0, SGI93518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20804[] = {SGI93519, SGI93520, SGI93521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93522, SGI0, SGI0, SGI0, SGI93523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20805[] = {SGI93524, SGI93525, SGI93526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93527, SGI0, SGI0, SGI0, SGI93528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20806[] = {SGI93529, SGI93530, SGI93531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93532, SGI0, SGI0, SGI0, SGI93533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20807[] = {SGI93534, SGI0, SGI93535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93536, SGI93537, SGI93537, SGI0, SGI93538, SGI0, NULL};
const Segment * const SG20808[] = {SGI93539, SGI93540, SGI93541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93542, SGI0, SGI0, SGI0, SGI93543, SGI93543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20809[] = {SGI93544, SGI0, SGI93545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93546, SGI93547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20810[] = {SGI93548, SGI93549, SGI93550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93551, SGI0, SGI0, SGI0, SGI93552, SGI93552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20811[] = {SGI93553, SGI0, SGI93554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20812[] = {SGI93555, SGI93556, SGI93557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93558, SGI0, SGI0, SGI0, SGI93559, SGI93559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20813[] = {SGI93560, SGI93561, SGI93562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93563, SGI0, SGI0, SGI0, SGI93564, SGI93564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20814[] = {SGI93565, SGI0, SGI93566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20815[] = {SGI93567, SGI0, SGI93568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93569, SGI93570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20816[] = {SGI0, SGI93571, SGI93572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93573, SGI0, SGI0, SGI0, SGI0, SGI93574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20817[] = {SGI93575, SGI93576, SGI93577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93578, SGI93579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20818[] = {SGI93580, SGI93581, SGI93582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93583, SGI93583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20819[] = {SGI0, SGI93584, SGI93585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93586, SGI0, SGI0, SGI0, SGI0, SGI93587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20820[] = {SGI0, SGI93588, SGI93589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93590, SGI0, SGI0, SGI0, SGI0, SGI93591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20821[] = {SGI0, SGI93592, SGI93593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93594, SGI0, SGI0, SGI0, SGI0, SGI93595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20822[] = {SGI0, SGI93596, SGI93597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93598, SGI0, SGI0, SGI0, SGI0, SGI93599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20823[] = {SGI0, SGI93600, SGI93601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20824[] = {SGI93602, SGI0, SGI93603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93604, SGI0, SGI0, SGI0, SGI93605, SGI93605, SGI0, SGI0, SGI93606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20825[] = {SGI0, SGI93607, SGI93608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93609, SGI0, SGI0, SGI0, SGI0, SGI93610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20826[] = {SGI0, SGI93611, SGI93612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93613, SGI0, SGI0, SGI0, SGI0, SGI93614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20827[] = {SGI93615, SGI0, SGI93616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20828[] = {SGI0, SGI93617, SGI93618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93619, SGI0, SGI0, SGI0, SGI0, SGI93620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20829[] = {SGI93621, SGI93622, SGI93623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93624, SGI0, SGI0, SGI0, SGI93625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20830[] = {SGI93626, SGI93627, SGI93628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93629, SGI0, SGI0, SGI0, SGI93630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20831[] = {SGI93631, SGI93632, SGI93633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93634, SGI0, SGI0, SGI0, SGI93635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20832[] = {SGI93636, SGI93637, SGI93638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93639, SGI0, SGI0, SGI0, SGI93640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20833[] = {SGI93641, SGI93642, SGI93643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93644, SGI0, SGI0, SGI0, SGI93645, SGI93645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20834[] = {SGI93646, SGI93647, SGI93648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93649, SGI0, SGI0, SGI0, SGI93650, SGI93650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20835[] = {SGI93651, SGI93652, SGI93653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93654, SGI0, SGI0, SGI0, SGI93655, SGI93655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20836[] = {SGI93656, SGI93657, SGI93658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93659, SGI0, SGI0, SGI0, SGI93660, SGI93660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20837[] = {SGI93661, SGI93662, SGI93663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93664, SGI0, SGI0, SGI0, SGI93665, SGI93665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20838[] = {SGI93666, SGI93667, SGI93668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93669, SGI0, SGI0, SGI0, SGI93670, SGI93670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20839[] = {SGI93671, SGI93672, SGI93673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93674, SGI0, SGI0, SGI0, SGI93675, SGI93675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20840[] = {SGI93676, SGI93677, SGI93678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93679, SGI0, SGI0, SGI0, SGI93680, SGI93680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20841[] = {SGI93681, SGI93682, SGI93683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93684, SGI0, SGI0, SGI0, SGI93685, SGI93685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20842[] = {SGI93686, SGI93687, SGI93688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93689, SGI0, SGI0, SGI0, SGI93690, SGI93690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20843[] = {SGI93691, SGI93692, SGI93693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93694, SGI0, SGI0, SGI0, SGI93695, SGI93695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20844[] = {SGI93696, SGI93697, SGI93698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93699, SGI0, SGI0, SGI0, SGI93700, SGI93700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93701, SGI0, NULL};
const Segment * const SG20845[] = {SGI93702, SGI93703, SGI93704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93705, SGI0, SGI0, SGI0, SGI93706, SGI93706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93707, SGI0, NULL};
const Segment * const SG20846[] = {SGI93708, SGI93709, SGI93710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93711, SGI0, SGI0, SGI0, SGI93712, SGI93712, SGI0, SGI93713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20847[] = {SGI93714, SGI93715, SGI93716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93717, SGI0, SGI0, SGI0, SGI93718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20848[] = {SGI93719, SGI93720, SGI93721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93722, SGI0, SGI0, SGI0, SGI93723, SGI93723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20849[] = {SGI93724, SGI93725, SGI93726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93727, SGI0, SGI0, SGI0, SGI93728, SGI93728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20850[] = {SGI93729, SGI0, SGI93730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93731, SGI0, SGI0, SGI0, SGI93732, SGI93732, SGI0, SGI0, SGI93733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20851[] = {SGI93734, SGI93735, SGI93736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93737, SGI0, SGI0, SGI0, SGI93738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20852[] = {SGI93739, SGI93740, SGI93741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93742, SGI0, SGI0, SGI0, SGI93743, SGI93743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20853[] = {SGI93744, SGI0, SGI93745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93746, SGI93747, SGI93747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20854[] = {SGI93749, SGI0, SGI93752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93753, SGI93754, SGI93754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20855[] = {SGI93748, SGI93750, SGI93751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93755, SGI93755, SGI0, SGI0, SGI93756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20856[] = {SGI93757, SGI0, SGI93758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93759, SGI93760, SGI93760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20857[] = {SGI93761, SGI0, SGI93762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93763, SGI93764, SGI93764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20858[] = {SGI93765, SGI0, SGI93766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93767, SGI93768, SGI93768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20859[] = {SGI93769, SGI0, SGI93770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93771, SGI93772, SGI93772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20860[] = {SGI93773, SGI93774, SGI93775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93776, SGI0, SGI0, SGI0, SGI93777, SGI93777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20861[] = {SGI93778, SGI93779, SGI93780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93781, SGI0, SGI0, SGI0, SGI93782, SGI93782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93783, SGI0, NULL};
const Segment * const SG20862[] = {SGI93784, SGI93785, SGI93786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93787, SGI0, SGI0, SGI0, SGI93788, SGI93788, SGI0, SGI0, SGI0, SGI93789, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20863[] = {SGI93790, SGI0, SGI93791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93792, SGI93793, SGI93793, SGI0, SGI93794, SGI0, NULL};
const Segment * const SG20864[] = {SGI93795, SGI93796, SGI93797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93798, SGI0, SGI0, SGI0, SGI93799, SGI93799, SGI0, SGI0, SGI0, SGI93800, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20865[] = {SGI93801, SGI93802, SGI93803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93804, SGI0, SGI0, SGI0, SGI93805, SGI93805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20866[] = {SGI93806, SGI0, SGI93807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93808, SGI0, SGI0, SGI0, SGI93809, SGI93809, SGI0, SGI0, SGI93811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20867[] = {SGI93810, SGI93812, SGI93813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93816, SGI0, SGI0, SGI0, SGI93818, SGI93818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20868[] = {SGI93814, SGI93815, SGI93817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93819, SGI0, SGI0, SGI0, SGI93820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20869[] = {SGI93821, SGI0, SGI93822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20870[] = {SGI93823, SGI93824, SGI93825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93826, SGI93827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20871[] = {SGI93828, SGI93829, SGI93830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93831, SGI93831, SGI0, SGI0, SGI93832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20872[] = {SGI93833, SGI93834, SGI93835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93836, SGI93836, SGI0, SGI0, SGI93837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20873[] = {SGI93838, SGI0, SGI93839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93840, SGI93841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20874[] = {SGI93842, SGI0, SGI93843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93844, SGI93845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20875[] = {SGI93846, SGI0, SGI93847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93848, SGI0, SGI0, SGI0, SGI93849, SGI93849, SGI0, SGI0, SGI93850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20876[] = {SGI93851, SGI93852, SGI93853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93854, SGI0, SGI0, SGI0, SGI93855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20877[] = {SGI93856, SGI93857, SGI93858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93859, SGI0, SGI0, SGI0, SGI93860, SGI93860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93861, SGI0, NULL};
const Segment * const SG20878[] = {SGI93862, SGI93863, SGI93864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93865, SGI93865, SGI0, SGI0, SGI93866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20879[] = {SGI93867, SGI93868, SGI93869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93870, SGI93870, SGI0, SGI0, SGI93871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20880[] = {SGI93872, SGI0, SGI93873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93874, SGI93875, SGI93875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20881[] = {SGI93876, SGI0, SGI93877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93878, SGI93879, SGI93879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20882[] = {SGI93880, SGI0, SGI93881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93882, SGI0, SGI0, SGI0, SGI93883, SGI93883, SGI0, SGI0, SGI93884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20883[] = {SGI93885, SGI93886, SGI93887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93888, SGI93889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20884[] = {SGI93890, SGI93892, SGI93894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93896, SGI0, SGI0, SGI0, SGI93898, SGI93898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20885[] = {SGI93891, SGI93893, SGI93895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93897, SGI0, SGI0, SGI0, SGI93899, SGI93899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20886[] = {SGI93900, SGI93901, SGI93903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93907, SGI93907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20887[] = {SGI93902, SGI93904, SGI93905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93906, SGI0, SGI0, SGI0, SGI93908, SGI93908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20888[] = {SGI93909, SGI0, SGI93910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93911, SGI0, SGI0, SGI0, SGI93912, SGI93912, SGI0, SGI0, SGI93913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20889[] = {SGI93914, SGI93915, SGI93916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93917, SGI0, SGI0, SGI0, SGI93918, SGI93918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20890[] = {SGI93919, SGI0, SGI93920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93921, SGI93922, SGI93922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93923, NULL};
const Segment * const SG20891[] = {SGI93924, SGI93925, SGI93926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93927, SGI0, SGI0, SGI0, SGI93928, SGI93928, SGI93929, SGI0, SGI93930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20892[] = {SGI93931, SGI93932, SGI93933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93934, SGI0, SGI0, SGI0, SGI93935, SGI93935, SGI0, SGI0, SGI93936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20893[] = {SGI93937, SGI93938, SGI93939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93940, SGI93940, SGI0, SGI0, SGI93941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20894[] = {SGI93942, SGI93943, SGI93944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93945, SGI93945, SGI0, SGI0, SGI93946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20895[] = {SGI93947, SGI0, SGI93948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93949, SGI93950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20896[] = {SGI93951, SGI0, SGI93952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93953, SGI93954, SGI93954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20897[] = {SGI93955, SGI0, SGI93956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93957, SGI93958, SGI93958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20898[] = {SGI93959, SGI0, SGI93960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93961, SGI93962, SGI93962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20899[] = {SGI93963, SGI93964, SGI93965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93966, SGI0, SGI0, SGI0, SGI93967, SGI93967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20900[] = {SGI93968, SGI0, SGI93969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93970, SGI93971, SGI93971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20901[] = {SGI93972, SGI93973, SGI93974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93975, SGI93975, SGI0, SGI0, SGI93976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20902[] = {SGI93977, SGI93978, SGI93979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93980, SGI0, SGI0, SGI0, SGI93981, SGI93981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20903[] = {SGI93982, SGI93983, SGI93984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93985, SGI0, SGI0, SGI0, SGI93986, SGI93986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20904[] = {SGI93987, SGI93988, SGI93989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93990, SGI93990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20905[] = {SGI93991, SGI0, SGI93992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20906[] = {SGI93993, SGI0, SGI93994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI93998, SGI0, SGI0, SGI0, SGI93999, SGI93999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94002, SGI0, SGI0, NULL};
const Segment * const SG20907[] = {SGI93995, SGI93996, SGI93997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94000, SGI0, SGI0, SGI0, SGI94001, SGI94001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20908[] = {SGI94003, SGI94004, SGI94005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94006, SGI0, SGI0, SGI0, SGI94007, SGI94007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20909[] = {SGI94008, SGI0, SGI94009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94010, SGI94011, SGI94011, SGI0, SGI0, NULL};
const Segment * const SG20910[] = {SGI94012, SGI94013, SGI94014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94015, SGI0, SGI0, SGI0, SGI94016, SGI94016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94017, SGI0, NULL};
const Segment * const SG20911[] = {SGI94018, SGI0, SGI94019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94020, SGI94021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20912[] = {SGI94022, SGI0, SGI94023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94024, SGI94025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20913[] = {SGI94026, SGI0, SGI94027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94028, SGI94029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20914[] = {SGI94030, SGI94031, SGI94032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94033, SGI0, SGI0, SGI0, SGI94034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20915[] = {SGI94035, SGI0, SGI94036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94037, SGI94038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20916[] = {SGI94039, SGI94040, SGI94041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94042, SGI0, SGI0, SGI0, SGI94043, SGI94043, SGI94044, SGI0, SGI94045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20917[] = {SGI94046, SGI94047, SGI94048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94049, SGI0, SGI0, SGI0, SGI94050, SGI94050, SGI94051, SGI0, SGI94052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20918[] = {SGI94053, SGI0, SGI94054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94055, SGI0, SGI0, SGI0, SGI94056, SGI94056, SGI0, SGI0, SGI94057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20919[] = {SGI94058, SGI94059, SGI94060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94061, SGI94061, SGI0, SGI0, SGI94062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20920[] = {SGI94063, SGI94064, SGI94065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94066, SGI94066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20921[] = {SGI94067, SGI94068, SGI94069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94070, SGI94071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20922[] = {SGI0, SGI94072, SGI94073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20923[] = {SGI94074, SGI94075, SGI94076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94077, SGI0, SGI0, SGI0, SGI94078, SGI94078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20924[] = {SGI94079, SGI94080, SGI94081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94082, SGI0, SGI0, SGI0, SGI94083, SGI94083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20925[] = {SGI94084, SGI94085, SGI94086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94087, SGI0, SGI0, SGI0, SGI94088, SGI94088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20926[] = {SGI94089, SGI94090, SGI94091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94092, SGI0, SGI0, SGI0, SGI94093, SGI94093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20927[] = {SGI0, SGI0, SGI94094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94095, SGI0, SGI0, SGI0, SGI94096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20928[] = {SGI94097, SGI94098, SGI94099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94100, SGI0, SGI0, SGI0, SGI94101, SGI94101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20929[] = {SGI94102, SGI0, SGI94103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94104, SGI94105, SGI94105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20930[] = {SGI94106, SGI0, SGI94107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94108, SGI94109, SGI94109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20931[] = {SGI94110, SGI94111, SGI94112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94113, SGI0, SGI0, SGI0, SGI94114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20932[] = {SGI94115, SGI94116, SGI94117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94118, SGI94118, SGI0, SGI0, SGI94119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20933[] = {SGI94120, SGI94121, SGI94122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94123, SGI0, SGI0, SGI0, SGI94124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20934[] = {SGI94125, SGI0, SGI94126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94127, SGI94128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20935[] = {SGI94129, SGI0, SGI94130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94131, SGI94132, SGI94132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20936[] = {SGI94133, SGI0, SGI94134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94135, SGI94136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20937[] = {SGI94137, SGI0, SGI94138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94139, SGI94140, SGI94140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20938[] = {SGI94141, SGI94142, SGI94143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94144, SGI94144, SGI0, SGI0, SGI94145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20939[] = {SGI94146, SGI94147, SGI94148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94149, SGI94149, SGI0, SGI0, SGI94150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20940[] = {SGI94151, SGI0, SGI94152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94153, SGI94154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20941[] = {SGI94155, SGI94156, SGI94157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94158, SGI94158, SGI0, SGI0, SGI94159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20942[] = {SGI94160, SGI0, SGI94161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94162, SGI94163, SGI94163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20943[] = {SGI94164, SGI0, SGI94165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94166, SGI94167, SGI94167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20944[] = {SGI94168, SGI94169, SGI94170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94171, SGI0, SGI0, SGI0, SGI94172, SGI94172, SGI94173, SGI0, SGI94174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20945[] = {SGI94175, SGI0, SGI94176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94177, SGI0, SGI0, SGI0, SGI94178, SGI94178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94179, SGI0, SGI0, NULL};
const Segment * const SG20946[] = {SGI94180, SGI0, SGI94181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94182, SGI0, SGI0, SGI0, SGI94183, SGI94183, SGI0, SGI0, SGI94184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20947[] = {SGI94185, SGI94186, SGI94187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94188, SGI0, SGI0, SGI0, SGI94189, SGI94189, SGI0, SGI0, SGI0, SGI94190, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20948[] = {SGI94191, SGI94192, SGI94193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94194, SGI0, SGI0, SGI0, SGI94195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20949[] = {SGI94196, SGI0, SGI94197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94198, SGI94199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94200, SGI0, NULL};
const Segment * const SG20950[] = {SGI94201, SGI94202, SGI94203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94204, SGI0, SGI0, SGI0, SGI94205, SGI94205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20951[] = {SGI94206, SGI0, SGI94207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20952[] = {SGI94208, SGI94209, SGI94210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94211, SGI0, SGI0, SGI0, SGI94212, SGI94212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20953[] = {SGI94213, SGI94214, SGI94215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94216, SGI0, SGI0, SGI0, SGI94217, SGI94217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20954[] = {SGI94218, SGI0, SGI94219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94220, SGI0, SGI0, SGI0, SGI94221, SGI94221, SGI0, SGI0, SGI94222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20955[] = {SGI94223, SGI0, SGI94224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94225, SGI0, SGI0, SGI0, SGI94226, SGI94226, SGI0, SGI0, SGI94227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20956[] = {SGI94228, SGI94229, SGI94230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94231, SGI0, SGI0, SGI0, SGI94232, SGI94232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94233, SGI0, NULL};
const Segment * const SG20957[] = {SGI94234, SGI94235, SGI94236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94237, SGI0, SGI0, SGI0, SGI94238, SGI94238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20958[] = {SGI94239, SGI94240, SGI94241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94242, SGI0, SGI0, SGI0, SGI94243, SGI94243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20959[] = {SGI94244, SGI94245, SGI94246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94247, SGI0, SGI0, SGI0, SGI94248, SGI94248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20960[] = {SGI94249, SGI94250, SGI94251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94252, SGI0, SGI0, SGI0, SGI94253, SGI94253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20961[] = {SGI94254, SGI94255, SGI94256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94257, SGI94257, SGI0, SGI0, SGI94258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20962[] = {SGI94259, SGI94260, SGI94261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94262, SGI94263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20963[] = {SGI94264, SGI94265, SGI94266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94267, SGI0, SGI0, SGI0, SGI94268, SGI94268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20964[] = {SGI94269, SGI94270, SGI94271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94272, SGI0, SGI0, SGI0, SGI94273, SGI94273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20965[] = {SGI94274, SGI94275, SGI94276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94277, SGI0, SGI0, SGI0, SGI94278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20966[] = {SGI94279, SGI94280, SGI94281, SGI0, SGI0, SGI0, SGI94282, SGI94283, SGI0, SGI94284, SGI0, SGI0, SGI0, SGI94285, SGI94285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20967[] = {SGI94286, SGI0, SGI94287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94288, SGI94289, SGI94289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94290, NULL};
const Segment * const SG20968[] = {SGI94291, SGI0, SGI94292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94293, SGI94294, SGI94294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94295, NULL};
const Segment * const SG20969[] = {SGI94296, SGI94297, SGI94298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94299, SGI0, SGI0, SGI0, SGI94300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20970[] = {SGI94301, SGI94302, SGI94303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94304, SGI0, SGI0, SGI0, SGI94305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20971[] = {SGI94306, SGI94307, SGI94308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94309, SGI0, SGI0, SGI0, SGI94310, SGI94310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20972[] = {SGI94311, SGI94312, SGI94313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94314, SGI0, SGI0, SGI0, SGI94315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20973[] = {SGI94316, SGI94317, SGI94318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94321, SGI0, SGI0, SGI0, SGI94323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20974[] = {SGI94319, SGI94320, SGI94322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94324, SGI0, SGI0, SGI0, SGI94325, SGI94325, SGI0, SGI0, SGI0, SGI94326, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20975[] = {SGI94327, SGI0, SGI94328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94329, SGI94330, SGI94330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94331, NULL};
const Segment * const SG20976[] = {SGI94332, SGI94333, SGI94334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94335, SGI94335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94336, SGI0, NULL};
const Segment * const SG20977[] = {SGI94337, SGI94338, SGI94339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94340, SGI0, SGI0, SGI0, SGI94341, SGI0, SGI0, SGI94342, SGI0, NULL};
const Segment * const SG20978[] = {SGI94343, SGI94344, SGI94345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94346, SGI0, SGI0, SGI0, SGI94347, SGI94347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94348, SGI0, SGI0, NULL};
const Segment * const SG20979[] = {SGI94349, SGI94350, SGI94351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94352, SGI0, SGI0, SGI0, SGI94353, SGI94353, SGI0, SGI0, SGI0, SGI94354, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20980[] = {SGI94355, SGI94356, SGI94357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94358, SGI0, SGI0, SGI0, SGI94359, SGI94359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94360, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20981[] = {SGI94361, SGI94362, SGI94363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94364, SGI0, SGI0, SGI0, SGI94365, SGI94365, SGI0, SGI94366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20982[] = {SGI94367, SGI94368, SGI94369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94370, SGI94370, SGI0, SGI0, SGI94371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20983[] = {SGI94372, SGI94373, SGI94374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94375, SGI0, SGI0, SGI0, SGI94376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20984[] = {SGI94377, SGI0, SGI94378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94379, SGI94380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20985[] = {SGI94381, SGI0, SGI94382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94383, SGI94384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20986[] = {SGI94385, SGI0, SGI94386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94387, SGI94388, SGI94388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20987[] = {SGI94389, SGI0, SGI94390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94391, SGI94392, SGI94392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20988[] = {SGI94393, SGI94394, SGI94395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94396, SGI94396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94397, SGI0, NULL};
const Segment * const SG20989[] = {SGI94398, SGI0, SGI94399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94400, SGI94401, SGI94401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20990[] = {SGI94402, SGI0, SGI94403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94404, SGI94405, SGI94405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20991[] = {SGI94406, SGI0, SGI94407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94408, SGI94409, SGI94409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20992[] = {SGI94410, SGI0, SGI94411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94412, SGI94413, SGI94413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20993[] = {SGI94414, SGI94415, SGI94416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94417, SGI0, SGI0, SGI0, SGI94418, SGI94418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20994[] = {SGI94419, SGI0, SGI94420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20995[] = {SGI94421, SGI0, SGI94422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94423, SGI0, SGI0, SGI0, SGI94424, SGI94424, SGI0, SGI0, SGI94425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20996[] = {SGI94426, SGI94427, SGI94428, SGI0, SGI0, SGI0, SGI94429, SGI94430, SGI0, SGI94431, SGI0, SGI0, SGI0, SGI94432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20997[] = {SGI94433, SGI0, SGI94434, SGI0, SGI0, SGI0, SGI94435, SGI94436, SGI0, SGI94437, SGI0, SGI0, SGI0, SGI94438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20998[] = {SGI94439, SGI0, SGI94440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94441, SGI94442, SGI94442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG20999[] = {SGI94443, SGI94444, SGI94445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94446, SGI0, SGI0, SGI0, SGI94447, SGI94447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94448, SGI0, NULL};
const Segment * const SG21000[] = {SGI94449, SGI94450, SGI94451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94452, SGI0, SGI0, SGI0, SGI94453, SGI94453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21001[] = {SGI94454, SGI94455, SGI94456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94457, SGI94457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94458, SGI0, NULL};
const Segment * const SG21002[] = {SGI94459, SGI0, SGI94460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94461, SGI94462, SGI94462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21003[] = {SGI94463, SGI0, SGI94464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94465, SGI94466, SGI94466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21004[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94467, SGI0, SGI94468, SGI0, NULL};
const Segment * const SG21005[] = {SGI94469, SGI0, SGI94470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94471, SGI94472, SGI94472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21006[] = {SGI94473, SGI94474, SGI94475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94476, SGI0, SGI0, SGI0, SGI94477, SGI94477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94478, SGI0, NULL};
const Segment * const SG21007[] = {SGI94479, SGI0, SGI94480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94481, SGI94482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21008[] = {SGI94483, SGI0, SGI94484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94485, SGI94486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21009[] = {SGI94487, SGI0, SGI94488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94489, SGI94490, SGI94490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21010[] = {SGI94491, SGI94492, SGI94493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94494, SGI0, SGI0, SGI0, SGI94495, SGI94495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21011[] = {SGI94496, SGI0, SGI94497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94498, SGI94499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21012[] = {SGI94500, SGI0, SGI94501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94502, SGI94503, SGI94503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21013[] = {SGI94504, SGI0, SGI94505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94506, SGI94507, SGI94507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21014[] = {SGI94508, SGI0, SGI94509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94510, SGI94511, SGI94511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21015[] = {SGI94512, SGI94513, SGI94514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94515, SGI94515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21016[] = {SGI94516, SGI0, SGI94517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94518, SGI94519, SGI94519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21017[] = {SGI94520, SGI0, SGI94521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94522, SGI94523, SGI94523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21018[] = {SGI94524, SGI0, SGI94525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94526, SGI94527, SGI94527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21019[] = {SGI94528, SGI0, SGI94529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94530, SGI94531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21020[] = {SGI94532, SGI0, SGI94533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94534, SGI94535, SGI94535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21021[] = {SGI94536, SGI0, SGI94537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94538, SGI94539, SGI94539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21022[] = {SGI94540, SGI0, SGI94541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94542, SGI94543, SGI94543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21023[] = {SGI94544, SGI94545, SGI94546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94547, SGI0, SGI0, SGI0, SGI94548, SGI94548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21024[] = {SGI94549, SGI94550, SGI94551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94552, SGI0, SGI0, SGI0, SGI94553, SGI94553, SGI0, SGI0, SGI0, SGI94554, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21025[] = {SGI94555, SGI94556, SGI94557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94558, SGI0, SGI0, SGI0, SGI94559, SGI94559, SGI0, SGI0, SGI0, SGI94560, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21026[] = {SGI94561, SGI94562, SGI94563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94564, SGI0, SGI0, SGI0, SGI94565, SGI0, SGI0, SGI94566, SGI0, NULL};
const Segment * const SG21027[] = {SGI94567, SGI94568, SGI94569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94570, SGI94570, SGI0, SGI0, SGI94571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21028[] = {SGI94572, SGI94573, SGI94574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94575, SGI94575, SGI0, SGI0, SGI94576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21029[] = {SGI94577, SGI94578, SGI94579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94580, SGI0, SGI0, SGI0, SGI94581, SGI94581, SGI94582, SGI0, SGI94583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21030[] = {SGI94584, SGI94585, SGI94586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94587, SGI0, SGI0, SGI0, SGI94588, SGI94588, SGI94589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21031[] = {SGI94590, SGI94591, SGI94592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94593, SGI0, SGI0, SGI0, SGI94594, SGI94594, SGI94595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21032[] = {SGI94596, SGI0, SGI94597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94598, SGI94599, SGI94599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21033[] = {SGI94600, SGI0, SGI94601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94602, SGI94603, SGI94603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21034[] = {SGI94604, SGI0, SGI94605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94606, SGI94607, SGI94607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21035[] = {SGI94608, SGI0, SGI94609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94610, SGI94611, SGI94611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21036[] = {SGI94612, SGI94613, SGI94614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94615, SGI0, SGI0, SGI0, SGI94616, SGI94616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21037[] = {SGI94617, SGI94618, SGI94619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94620, SGI0, SGI0, SGI0, SGI94621, SGI94621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21038[] = {SGI94622, SGI94623, SGI94624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94625, SGI0, SGI0, SGI0, SGI94626, SGI94626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21039[] = {SGI94627, SGI0, SGI94628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94629, SGI94630, SGI94630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21040[] = {SGI94631, SGI94632, SGI94633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94634, SGI0, SGI0, SGI0, SGI94635, SGI94635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21041[] = {SGI94636, SGI94637, SGI94638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94639, SGI0, SGI0, SGI0, SGI94640, SGI94640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21042[] = {SGI94641, SGI94642, SGI94643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94644, SGI94645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21043[] = {SGI94646, SGI0, SGI94647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94648, SGI94649, SGI94649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21044[] = {SGI94650, SGI0, SGI94651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94652, SGI94653, SGI94653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21045[] = {SGI94654, SGI0, SGI94655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94656, SGI94657, SGI94657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21046[] = {SGI94658, SGI0, SGI94659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94660, SGI94661, SGI94661, SGI0, SGI94662, SGI0, NULL};
const Segment * const SG21047[] = {SGI94663, SGI0, SGI94664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94665, SGI94666, SGI94666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21048[] = {SGI94667, SGI0, SGI94668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94669, SGI94670, SGI94670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21049[] = {SGI94671, SGI0, SGI94672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94673, SGI94674, SGI94674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21050[] = {SGI94675, SGI0, SGI94676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94677, SGI94678, SGI94678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21051[] = {SGI94679, SGI0, SGI94680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94681, SGI94682, SGI94682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21052[] = {SGI0, SGI94683, SGI0, SGI94684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21053[] = {SGI0, SGI94685, SGI0, SGI94686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94687, SGI0, SGI0, SGI0, SGI94688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21054[] = {SGI94689, SGI0, SGI94690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94691, SGI94692, SGI94692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21055[] = {SGI94693, SGI0, SGI94694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94695, SGI94696, SGI94696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21056[] = {SGI94697, SGI94698, SGI94699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94700, SGI0, SGI0, SGI0, SGI94701, SGI94701, SGI0, SGI0, SGI0, SGI94702, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21057[] = {SGI94703, SGI94704, SGI94705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94706, SGI0, SGI0, SGI0, SGI94707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21058[] = {SGI94708, SGI94709, SGI94710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94711, SGI94711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94712, SGI0, SGI0, NULL};
const Segment * const SG21059[] = {SGI94713, SGI94714, SGI94715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94716, SGI94716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94717, SGI0, SGI0, NULL};
const Segment * const SG21060[] = {SGI94718, SGI94719, SGI94720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94721, SGI0, SGI0, SGI0, SGI94722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21061[] = {SGI94723, SGI0, SGI94724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21062[] = {SGI94725, SGI94726, SGI94727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94728, SGI0, SGI0, SGI0, SGI94729, SGI94729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21063[] = {SGI94730, SGI0, SGI94731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94732, SGI0, SGI0, SGI0, SGI94733, SGI94733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21064[] = {SGI0, SGI0, SGI94734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94735, SGI94735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21065[] = {SGI94736, SGI94737, SGI94738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94739, SGI94739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21066[] = {SGI94740, SGI94741, SGI94742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94743, SGI0, SGI0, SGI0, SGI94744, SGI94744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21067[] = {SGI94745, SGI0, SGI94746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21068[] = {SGI94747, SGI0, SGI94748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94749, SGI94750, SGI94750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21069[] = {SGI94751, SGI0, SGI94752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94753, SGI94754, SGI94754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21070[] = {SGI94755, SGI94756, SGI94757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94758, SGI0, SGI0, SGI0, SGI94759, SGI94759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21071[] = {SGI94760, SGI94761, SGI94762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94763, SGI0, SGI0, SGI0, SGI94764, SGI94764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94765, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21072[] = {SGI94766, SGI94767, SGI94768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94769, SGI0, SGI0, SGI0, SGI94770, SGI94770, SGI94771, SGI0, SGI94772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21073[] = {SGI94773, SGI0, SGI94774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94775, SGI94776, SGI94776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21074[] = {SGI94777, SGI0, SGI94778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94779, SGI94780, SGI94780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21075[] = {SGI94781, SGI0, SGI94782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94783, SGI94784, SGI94784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21076[] = {SGI94785, SGI94786, SGI94787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94788, SGI0, SGI0, SGI0, SGI94789, SGI94789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21077[] = {SGI94790, SGI0, SGI94791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94792, SGI0, SGI0, SGI0, SGI94793, SGI94793, SGI0, SGI0, SGI94794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21078[] = {SGI94795, SGI0, SGI94796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94797, SGI94798, SGI94798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21079[] = {SGI94799, SGI0, SGI94800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94801, SGI94802, SGI94802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21080[] = {SGI94803, SGI0, SGI94804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94805, SGI94806, SGI94806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21081[] = {SGI94807, SGI0, SGI94808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94809, SGI94810, SGI94810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21082[] = {SGI94811, SGI94812, SGI94813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94814, SGI0, SGI0, SGI0, SGI94815, SGI94815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21083[] = {SGI94816, SGI0, SGI94817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94818, SGI94819, SGI94819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21084[] = {SGI94820, SGI0, SGI94821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94822, SGI94823, SGI94823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21085[] = {SGI94824, SGI0, SGI94825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94826, SGI94827, SGI94827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21086[] = {SGI94828, SGI0, SGI94829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94830, SGI94831, SGI94831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21087[] = {SGI94832, SGI0, SGI94833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94834, SGI94835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21088[] = {SGI94836, SGI0, SGI94837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94838, SGI94839, SGI94839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21089[] = {SGI94840, SGI0, SGI94841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94842, SGI94843, SGI94843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21090[] = {SGI94844, SGI94845, SGI94846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94847, SGI0, SGI0, SGI0, SGI94848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21091[] = {SGI94849, SGI94850, SGI94851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94852, SGI0, SGI0, SGI0, SGI94853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21092[] = {SGI94854, SGI94855, SGI94856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94857, SGI0, SGI0, SGI0, SGI94858, SGI94858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21093[] = {SGI94859, SGI0, SGI94860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94861, SGI94862, SGI94862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21094[] = {SGI94863, SGI0, SGI94864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94865, SGI94866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21095[] = {SGI94867, SGI94868, SGI94869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94870, SGI0, SGI0, SGI0, SGI94871, SGI94871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94872, SGI0, NULL};
const Segment * const SG21096[] = {SGI94873, SGI94874, SGI94875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94876, SGI0, SGI0, SGI0, SGI94877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21097[] = {SGI94878, SGI94879, SGI94880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94881, SGI0, SGI0, SGI0, SGI94882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21098[] = {SGI94883, SGI0, SGI94884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21099[] = {SGI94885, SGI94886, SGI94887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94888, SGI0, SGI0, SGI0, SGI94889, SGI94889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21100[] = {SGI94890, SGI94891, SGI94892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94893, SGI0, SGI0, SGI0, SGI94894, SGI94894, SGI94895, SGI0, SGI94896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21101[] = {SGI94897, SGI94898, SGI94899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94900, SGI0, SGI0, SGI0, SGI94901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21102[] = {SGI94902, SGI94903, SGI94904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94905, SGI0, SGI0, SGI0, SGI94908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21103[] = {SGI0, SGI94906, SGI0, SGI94907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21104[] = {SGI94909, SGI94910, SGI94911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94912, SGI0, SGI0, SGI0, SGI94913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21105[] = {SGI94914, SGI94915, SGI94916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94917, SGI0, SGI0, SGI0, SGI94918, SGI94918, SGI0, SGI0, SGI0, SGI94919, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21106[] = {SGI94920, SGI94921, SGI94922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94923, SGI0, SGI0, SGI0, SGI94924, SGI94924, SGI0, SGI0, SGI0, SGI94925, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21107[] = {SGI94926, SGI94927, SGI94928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94929, SGI0, SGI0, SGI0, SGI94930, SGI94930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21108[] = {SGI94931, SGI94932, SGI94933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94934, SGI0, SGI0, SGI0, SGI94935, SGI94935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21109[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94936, SGI0, SGI94937, SGI0, NULL};
const Segment * const SG21110[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94938, SGI0, SGI94939, SGI0, NULL};
const Segment * const SG21111[] = {SGI94940, SGI94941, SGI94942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94943, SGI0, SGI0, SGI0, SGI94944, SGI94944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21112[] = {SGI94945, SGI94946, SGI94947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94948, SGI94948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21113[] = {SGI94949, SGI0, SGI94950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21114[] = {SGI94951, SGI94952, SGI94953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94954, SGI94954, SGI0, SGI0, SGI94955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21115[] = {SGI94956, SGI94957, SGI94958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94959, SGI0, SGI0, SGI0, SGI94960, SGI94960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21116[] = {SGI94961, SGI94962, SGI94963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94964, SGI0, SGI0, SGI0, SGI94965, SGI94965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21117[] = {SGI94966, SGI0, SGI94967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21118[] = {SGI94968, SGI94969, SGI94970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94971, SGI94971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21119[] = {SGI94972, SGI94973, SGI94974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94975, SGI94975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21120[] = {SGI94976, SGI94977, SGI94978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94979, SGI0, SGI0, SGI0, SGI94980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21121[] = {SGI94981, SGI94982, SGI94983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94984, SGI0, SGI0, SGI0, SGI94985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21122[] = {SGI94986, SGI0, SGI94987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21123[] = {SGI94988, SGI94989, SGI94990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94991, SGI0, SGI0, SGI0, SGI94992, SGI94993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21124[] = {SGI94994, SGI0, SGI94995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI94996, SGI94997, SGI94997, SGI0, SGI0, NULL};
const Segment * const SG21125[] = {SGI94998, SGI0, SGI94999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95000, SGI0, SGI0, SGI0, SGI95001, SGI95001, SGI0, SGI0, SGI95002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21126[] = {SGI95003, SGI0, SGI95004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95005, SGI0, SGI0, SGI0, SGI95006, SGI95006, SGI0, SGI0, SGI95007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21127[] = {SGI95008, SGI95009, SGI95010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95011, SGI0, SGI0, SGI0, SGI95012, SGI95012, SGI0, SGI0, SGI0, SGI95013, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21128[] = {SGI95014, SGI0, SGI95015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95016, SGI0, SGI0, SGI0, SGI95018, SGI95018, SGI0, SGI0, SGI95020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21129[] = {SGI95017, SGI0, SGI95019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95021, SGI95022, SGI95022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21130[] = {SGI95023, SGI0, SGI95024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95025, SGI95026, SGI95026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21131[] = {SGI95027, SGI0, SGI95028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95029, SGI95030, SGI95030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21132[] = {SGI95031, SGI0, SGI95032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95033, SGI95034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21133[] = {SGI95035, SGI0, SGI95036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95037, SGI95038, SGI95038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21134[] = {SGI95039, SGI0, SGI95040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95041, SGI95042, SGI95042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21135[] = {SGI0, SGI95043, SGI0, SGI95044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21136[] = {SGI95045, SGI95046, SGI95047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95048, SGI0, SGI0, SGI0, SGI95049, SGI95049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21137[] = {SGI95050, SGI95051, SGI95052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95053, SGI0, SGI0, SGI0, SGI95054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21138[] = {SGI95055, SGI95056, SGI95057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95058, SGI0, SGI0, SGI0, SGI95059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21139[] = {SGI95060, SGI95061, SGI95062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95063, SGI0, SGI0, SGI0, SGI95064, SGI95064, SGI0, SGI0, SGI0, SGI95065, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21140[] = {SGI95066, SGI95067, SGI95068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95069, SGI0, SGI0, SGI0, SGI95070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21141[] = {SGI95071, SGI0, SGI95072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95073, SGI0, SGI0, SGI0, SGI95074, SGI95074, SGI0, SGI0, SGI95075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21142[] = {SGI95076, SGI95077, SGI95078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95080, SGI0, SGI0, SGI0, SGI95082, SGI95082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21143[] = {SGI95079, SGI0, SGI95081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95083, SGI0, SGI0, SGI0, SGI95084, SGI95084, SGI0, SGI0, SGI95085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21144[] = {SGI95086, SGI95087, SGI95088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95089, SGI0, SGI0, SGI0, SGI95090, SGI95091, SGI0, SGI0, SGI95092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21145[] = {SGI95093, SGI95094, SGI95095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95096, SGI0, SGI0, SGI0, SGI95097, SGI95097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21146[] = {SGI95098, SGI95099, SGI95100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95101, SGI0, SGI0, SGI0, SGI95102, SGI95102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21147[] = {SGI95103, SGI0, SGI95104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95105, SGI95106, SGI95106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21148[] = {SGI95107, SGI95108, SGI95109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95110, SGI95110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95111, SGI0, NULL};
const Segment * const SG21149[] = {SGI95112, SGI0, SGI95113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95114, SGI95115, SGI95115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21150[] = {SGI95116, SGI0, SGI95117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95118, SGI95119, SGI95119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21151[] = {SGI95120, SGI0, SGI95121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95122, SGI95123, SGI95123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21152[] = {SGI95124, SGI0, SGI95125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95126, SGI95127, SGI95127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95128, SGI0, NULL};
const Segment * const SG21153[] = {SGI95129, SGI0, SGI95130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95131, SGI0, SGI0, SGI0, SGI95132, SGI95132, SGI0, SGI0, SGI95133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21154[] = {SGI95134, SGI0, SGI95135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95136, SGI95137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21155[] = {SGI95138, SGI0, SGI95139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95140, SGI0, SGI0, SGI0, SGI95141, SGI95141, SGI0, SGI0, SGI95142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21156[] = {SGI95143, SGI0, SGI95144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95145, SGI95146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21157[] = {SGI95147, SGI95148, SGI95149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95150, SGI0, SGI0, SGI0, SGI95151, SGI95151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95152, SGI0, NULL};
const Segment * const SG21158[] = {SGI95153, SGI0, SGI95154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95155, SGI0, SGI0, SGI0, SGI95156, SGI95156, SGI0, SGI0, SGI95157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21159[] = {SGI95158, SGI0, SGI95159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95160, SGI0, SGI0, SGI0, SGI95161, SGI95161, SGI0, SGI0, SGI95162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21160[] = {SGI95163, SGI0, SGI95164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95165, SGI0, SGI0, SGI0, SGI95166, SGI95166, SGI0, SGI0, SGI95167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21161[] = {SGI95168, SGI95169, SGI95170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95171, SGI0, SGI0, SGI0, SGI95172, SGI95172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21162[] = {SGI95173, SGI0, SGI95174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95175, SGI0, SGI0, SGI0, SGI95176, SGI95176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21163[] = {SGI95177, SGI0, SGI95178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95179, SGI95180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21164[] = {SGI95181, SGI95182, SGI95183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95184, SGI0, SGI0, SGI0, SGI95185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21165[] = {SGI95186, SGI0, SGI95187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95188, SGI0, SGI0, SGI0, SGI95189, SGI95189, SGI0, SGI0, SGI95190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21166[] = {SGI95191, SGI95192, SGI95193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95194, SGI0, SGI0, SGI0, SGI95195, SGI95195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95196, SGI0, NULL};
const Segment * const SG21167[] = {SGI95197, SGI95198, SGI95199, SGI95200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95201, SGI0, SGI0, SGI0, SGI95202, SGI95203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21168[] = {SGI95204, SGI95205, SGI95206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95207, SGI0, SGI0, SGI0, SGI95208, SGI95208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21169[] = {SGI95209, SGI0, SGI95210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95211, SGI95212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21170[] = {SGI95213, SGI0, SGI95214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95215, SGI95216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21171[] = {SGI95217, SGI95218, SGI95219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95220, SGI95220, SGI0, SGI0, SGI95221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21172[] = {SGI95222, SGI95223, SGI95224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95225, SGI95225, SGI0, SGI0, SGI95226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21173[] = {SGI95227, SGI95228, SGI95229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95230, SGI95230, SGI0, SGI0, SGI95231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21174[] = {SGI95232, SGI95233, SGI95234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95235, SGI0, SGI0, SGI0, SGI95236, SGI95236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21175[] = {SGI95237, SGI95238, SGI95239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95240, SGI0, SGI0, SGI0, SGI95241, SGI95241, SGI95242, SGI0, SGI95243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21176[] = {SGI95244, SGI95245, SGI95246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95247, SGI0, SGI0, SGI0, SGI95248, SGI95248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21177[] = {SGI95249, SGI0, SGI95250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95251, SGI95252, SGI95252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21178[] = {SGI95253, SGI95254, SGI95255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95258, SGI0, SGI0, SGI0, SGI95259, SGI95259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21179[] = {SGI95256, SGI0, SGI95257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95260, SGI95261, SGI95261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21180[] = {SGI95262, SGI0, SGI95263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95264, SGI0, SGI0, SGI0, SGI95265, SGI95265, SGI0, SGI0, SGI95266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21181[] = {SGI95267, SGI0, SGI95268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95269, SGI95270, SGI95270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21182[] = {SGI95271, SGI95272, SGI95273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95274, SGI95274, SGI0, SGI0, SGI95275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21183[] = {SGI95276, SGI95277, SGI95278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95282, SGI0, SGI0, SGI0, SGI95283, SGI95283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21184[] = {SGI95279, SGI95280, SGI95281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95284, SGI0, SGI0, SGI0, SGI95285, SGI95285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21185[] = {SGI95286, SGI0, SGI95287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21186[] = {SGI95288, SGI95289, SGI95290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95291, SGI0, SGI0, SGI0, SGI95292, SGI95292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21187[] = {SGI95293, SGI0, SGI95294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95295, SGI95296, SGI95296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21188[] = {SGI95297, SGI0, SGI95298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95299, SGI95300, SGI95300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21189[] = {SGI95301, SGI0, SGI95302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95303, SGI95304, SGI95304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21190[] = {SGI95305, SGI95306, SGI95307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95308, SGI95308, SGI0, SGI0, SGI95309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21191[] = {SGI95310, SGI0, SGI95311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95312, SGI95313, SGI95313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21192[] = {SGI95314, SGI0, SGI95315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95316, SGI95317, SGI95317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21193[] = {SGI95318, SGI0, SGI95319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95320, SGI95321, SGI95321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21194[] = {SGI95322, SGI0, SGI95323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95324, SGI95325, SGI95325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21195[] = {SGI95326, SGI0, SGI95327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95328, SGI0, SGI0, SGI0, SGI95329, SGI95329, SGI0, SGI0, SGI95330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21196[] = {SGI95331, SGI95332, SGI95333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95334, SGI0, SGI0, SGI0, SGI95335, SGI95335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21197[] = {SGI95336, SGI95337, SGI95338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95339, SGI0, SGI0, SGI0, SGI95340, SGI95340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21198[] = {SGI95341, SGI95342, SGI95343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95344, SGI0, SGI0, SGI0, SGI95347, SGI95347, SGI0, SGI0, SGI0, SGI95348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21199[] = {SGI95345, SGI0, SGI95346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95349, SGI0, SGI0, SGI0, SGI95350, SGI95350, SGI0, SGI0, SGI95351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21200[] = {SGI95352, SGI95353, SGI95354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95355, SGI0, SGI0, SGI0, SGI95356, SGI95356, SGI0, SGI0, SGI0, SGI95357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21201[] = {SGI95358, SGI95359, SGI95360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95361, SGI0, SGI0, SGI0, SGI95362, SGI95362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21202[] = {SGI95363, SGI0, SGI95364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21203[] = {SGI95365, SGI0, SGI95366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95367, SGI95368, SGI95368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95369, NULL};
const Segment * const SG21204[] = {SGI95370, SGI0, SGI95371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95372, SGI95373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21205[] = {SGI95374, SGI0, SGI95375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95376, SGI95377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21206[] = {SGI95378, SGI95379, SGI95380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95381, SGI0, SGI0, SGI0, SGI95382, SGI95382, SGI0, SGI0, SGI95383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21207[] = {SGI95384, SGI95385, SGI95386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95387, SGI0, SGI0, SGI0, SGI95388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21208[] = {SGI95389, SGI95390, SGI95391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95392, SGI0, SGI0, SGI0, SGI95393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21209[] = {SGI95394, SGI95395, SGI95396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95397, SGI0, SGI0, SGI0, SGI95398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21210[] = {SGI95399, SGI95400, SGI95401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95402, SGI0, SGI0, SGI0, SGI95403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21211[] = {SGI95404, SGI95405, SGI95406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95407, SGI0, SGI0, SGI0, SGI95408, SGI95408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21212[] = {SGI95409, SGI95410, SGI95411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95412, SGI0, SGI0, SGI0, SGI95413, SGI95413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21213[] = {SGI95414, SGI95415, SGI95416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95417, SGI0, SGI0, SGI0, SGI95418, SGI95418, SGI95419, SGI0, SGI95420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21214[] = {SGI95421, SGI95422, SGI95423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95424, SGI0, SGI0, SGI0, SGI95425, SGI95425, SGI0, SGI0, SGI95426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21215[] = {SGI95427, SGI0, SGI95428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95429, SGI95430, SGI95430, SGI0, SGI0, NULL};
const Segment * const SG21216[] = {SGI95431, SGI95432, SGI95433, SGI0, SGI0, SGI0, SGI95434, SGI95435, SGI0, SGI95436, SGI0, SGI0, SGI0, SGI95437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21217[] = {SGI0, SGI95438, SGI95439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95440, SGI95440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21218[] = {SGI95441, SGI95442, SGI95443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95444, SGI95444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21219[] = {SGI0, SGI95445, SGI95446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95447, SGI95447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21220[] = {SGI0, SGI95448, SGI95449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95450, SGI95450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21221[] = {SGI95451, SGI95452, SGI95453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95454, SGI0, SGI0, SGI0, SGI95455, SGI95455, SGI95456, SGI0, SGI95457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21222[] = {SGI95458, SGI95459, SGI95460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95461, SGI0, SGI0, SGI0, SGI95462, SGI95462, SGI95463, SGI0, SGI95464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21223[] = {SGI95465, SGI0, SGI95466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95469, SGI95470, SGI95470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95472, NULL};
const Segment * const SG21224[] = {SGI95467, SGI0, SGI95468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95471, SGI0, SGI0, SGI0, SGI95473, SGI95473, SGI0, SGI0, SGI95474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21225[] = {SGI95475, SGI0, SGI95476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95477, SGI0, SGI0, SGI0, SGI95478, SGI95478, SGI0, SGI0, SGI95479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21226[] = {SGI95480, SGI0, SGI95481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95482, SGI0, SGI0, SGI0, SGI95483, SGI95483, SGI0, SGI0, SGI95484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21227[] = {SGI95485, SGI95486, SGI95487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95488, SGI0, SGI0, SGI0, SGI95489, SGI95489, SGI95490, SGI0, SGI95491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21228[] = {SGI95492, SGI95493, SGI95494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95495, SGI0, SGI0, SGI0, SGI95496, SGI95496, SGI95497, SGI0, SGI95498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21229[] = {SGI95499, SGI95500, SGI95501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95502, SGI0, SGI0, SGI0, SGI95503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21230[] = {SGI95504, SGI0, SGI95505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21231[] = {SGI0, SGI95506, SGI0, SGI95507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21232[] = {SGI95508, SGI0, SGI95509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95510, SGI95511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21233[] = {SGI95512, SGI95513, SGI95514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95515, SGI0, SGI0, SGI0, SGI95516, SGI95516, SGI95517, SGI0, SGI95518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21234[] = {SGI95519, SGI95520, SGI95521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95522, SGI0, SGI0, SGI0, SGI95523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21235[] = {SGI95524, SGI95525, SGI95526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95527, SGI0, SGI0, SGI0, SGI95528, SGI95528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21236[] = {SGI95529, SGI95530, SGI95531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95532, SGI0, SGI0, SGI0, SGI95533, SGI95533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21237[] = {SGI95534, SGI95535, SGI95536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95537, SGI95537, SGI0, SGI0, SGI95538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21238[] = {SGI95539, SGI95540, SGI95541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95542, SGI0, SGI0, SGI0, SGI95543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21239[] = {SGI95544, SGI0, SGI95545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21240[] = {SGI95546, SGI95547, SGI95548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95549, SGI0, SGI0, SGI0, SGI95550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21241[] = {SGI95551, SGI95552, SGI95553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95554, SGI0, SGI0, SGI0, SGI95555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21242[] = {SGI95556, SGI0, SGI95557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95558, SGI95559, SGI95559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95560, NULL};
const Segment * const SG21243[] = {SGI95561, SGI95562, SGI95563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95564, SGI0, SGI0, SGI0, SGI95565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21244[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95566, SGI0, SGI95567, SGI0, NULL};
const Segment * const SG21245[] = {SGI95568, SGI0, SGI95569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95570, SGI0, SGI0, SGI0, SGI95571, SGI95571, SGI0, SGI0, SGI95572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21246[] = {SGI95573, SGI95574, SGI95575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95576, SGI0, SGI0, SGI0, SGI95577, SGI95577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21247[] = {SGI95578, SGI0, SGI95579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95580, SGI95581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21248[] = {SGI95582, SGI0, SGI95583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95584, SGI95585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21249[] = {SGI95586, SGI95587, SGI95588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95589, SGI0, SGI0, SGI0, SGI95590, SGI95590, SGI0, SGI0, SGI0, SGI95591, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21250[] = {SGI95592, SGI95593, SGI95594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95595, SGI0, SGI0, SGI0, SGI95596, SGI95596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21251[] = {SGI95597, SGI0, SGI95598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95599, SGI95600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21252[] = {SGI95601, SGI0, SGI95602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95603, SGI95604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21253[] = {SGI95605, SGI0, SGI95606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95607, SGI95608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21254[] = {SGI95609, SGI95610, SGI95611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95612, SGI0, SGI0, SGI0, SGI95613, SGI95613, SGI95614, SGI0, SGI95615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21255[] = {SGI95616, SGI95617, SGI95618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95619, SGI0, SGI0, SGI0, SGI95620, SGI95620, SGI0, SGI95621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21256[] = {SGI95622, SGI0, SGI95623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95624, SGI0, SGI0, SGI0, SGI95625, SGI95625, SGI0, SGI0, SGI95626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21257[] = {SGI95627, SGI0, SGI95628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95629, SGI95630, SGI95630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21258[] = {SGI95631, SGI0, SGI95632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95633, SGI0, SGI0, SGI0, SGI95634, SGI95634, SGI0, SGI0, SGI95635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21259[] = {SGI95636, SGI0, SGI95637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95638, SGI95639, SGI95639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21260[] = {SGI95640, SGI0, SGI95641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95642, SGI0, SGI0, SGI0, SGI95643, SGI95643, SGI0, SGI0, SGI95644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21261[] = {SGI95645, SGI0, SGI95646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95647, SGI0, SGI0, SGI0, SGI95648, SGI95648, SGI0, SGI0, SGI95649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21262[] = {SGI95650, SGI95651, SGI95652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95653, SGI0, SGI0, SGI0, SGI95654, SGI95654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21263[] = {SGI95655, SGI95656, SGI95657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95658, SGI0, SGI0, SGI0, SGI95659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21264[] = {SGI95660, SGI95661, SGI95662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95663, SGI0, SGI0, SGI0, SGI95664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21265[] = {SGI95665, SGI0, SGI95666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95667, SGI95668, SGI95668, SGI0, SGI0, NULL};
const Segment * const SG21266[] = {SGI95669, SGI95670, SGI95671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95672, SGI0, SGI0, SGI0, SGI95673, SGI95674, SGI0, SGI0, SGI95675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21267[] = {SGI95676, SGI95677, SGI95678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95679, SGI0, SGI0, SGI0, SGI95680, SGI95680, SGI0, SGI0, SGI0, SGI95681, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21268[] = {SGI95682, SGI95683, SGI95684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95685, SGI0, SGI0, SGI0, SGI95686, SGI95686, SGI0, SGI95687, SGI0, NULL};
const Segment * const SG21269[] = {SGI0, SGI95688, SGI0, SGI95689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95690, SGI0, SGI0, SGI0, SGI95691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21270[] = {SGI0, SGI95692, SGI0, SGI95693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21271[] = {SGI95694, SGI95695, SGI95696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95697, SGI0, SGI0, SGI0, SGI95698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21272[] = {SGI95699, SGI95700, SGI95701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95702, SGI0, SGI0, SGI0, SGI95703, SGI95703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21273[] = {SGI0, SGI0, SGI95704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95705, SGI95706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21274[] = {SGI95707, SGI0, SGI95708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95709, SGI0, SGI0, SGI0, SGI95710, SGI95710, SGI0, SGI0, SGI95711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21275[] = {SGI95712, SGI95713, SGI95714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95715, SGI0, SGI0, SGI0, SGI95716, SGI95716, SGI0, SGI0, SGI0, SGI95717, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21276[] = {SGI95718, SGI95719, SGI95720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95721, SGI0, SGI0, SGI0, SGI95722, SGI95722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21277[] = {SGI95723, SGI95724, SGI95725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95726, SGI0, SGI0, SGI0, SGI95727, SGI95727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21278[] = {SGI95728, SGI95729, SGI95730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95731, SGI95732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21279[] = {SGI95733, SGI95734, SGI95735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95736, SGI95736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21280[] = {SGI95737, SGI95738, SGI95739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95740, SGI0, SGI0, SGI0, SGI95741, SGI95741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21281[] = {SGI95742, SGI0, SGI95743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95744, SGI95745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21282[] = {SGI95746, SGI95747, SGI95748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95749, SGI0, SGI0, SGI0, SGI95750, SGI95750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21283[] = {SGI95751, SGI95752, SGI95753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95755, SGI0, SGI0, SGI0, SGI95758, SGI95758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21284[] = {SGI95754, SGI95756, SGI95757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95759, SGI0, SGI0, SGI0, SGI95760, SGI95760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95761, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21285[] = {SGI95762, SGI95763, SGI95764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95765, SGI0, SGI0, SGI0, SGI95766, SGI95766, SGI0, SGI95767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21286[] = {SGI95768, SGI95769, SGI95770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95771, SGI0, SGI0, SGI0, SGI95772, SGI95772, SGI95773, SGI0, SGI95774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21287[] = {SGI95775, SGI0, SGI95776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95777, SGI95778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21288[] = {SGI95779, SGI95780, SGI95781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95782, SGI0, SGI0, SGI0, SGI95783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21289[] = {SGI95784, SGI0, SGI95785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21290[] = {SGI0, SGI95786, SGI0, SGI95787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21291[] = {SGI95788, SGI0, SGI95789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95790, SGI95791, SGI95791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21292[] = {SGI95792, SGI0, SGI95793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95794, SGI95795, SGI95795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21293[] = {SGI95796, SGI0, SGI95797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95798, SGI95800, SGI95800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21294[] = {SGI95799, SGI95801, SGI95802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95803, SGI95803, SGI0, SGI0, SGI95804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21295[] = {SGI95805, SGI0, SGI95806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95807, SGI95808, SGI95808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21296[] = {SGI95809, SGI95810, SGI95811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95812, SGI0, SGI0, SGI0, SGI95813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21297[] = {SGI95814, SGI95816, SGI95817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95819, SGI0, SGI0, SGI0, SGI95821, SGI95821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21298[] = {SGI95815, SGI0, SGI95818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95820, SGI0, SGI0, SGI0, SGI95822, SGI95822, SGI0, SGI0, SGI95823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21299[] = {SGI95824, SGI0, SGI95825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95826, SGI0, SGI0, SGI0, SGI95827, SGI95827, SGI0, SGI0, SGI95828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21300[] = {SGI95829, SGI95830, SGI95831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95832, SGI0, SGI0, SGI0, SGI95833, SGI95833, SGI0, SGI95834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21301[] = {SGI95835, SGI95836, SGI95837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95838, SGI95838, SGI0, SGI0, SGI95839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21302[] = {SGI95840, SGI95841, SGI95842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95843, SGI95843, SGI0, SGI0, SGI95844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21303[] = {SGI95845, SGI95846, SGI95847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95848, SGI95848, SGI0, SGI0, SGI95849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21304[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95850, SGI0, SGI0, SGI95851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21305[] = {SGI95852, SGI95853, SGI95854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95855, SGI0, SGI0, SGI0, SGI95856, SGI95856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21306[] = {SGI95857, SGI0, SGI95858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95861, SGI95862, SGI95862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95865, NULL};
const Segment * const SG21307[] = {SGI95859, SGI0, SGI95860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95863, SGI95864, SGI95864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21308[] = {SGI95866, SGI0, SGI95867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95868, SGI95869, SGI95869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21309[] = {SGI95870, SGI0, SGI95871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95872, SGI95873, SGI95873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21310[] = {SGI95874, SGI95875, SGI95876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95877, SGI95877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95878, SGI0, NULL};
const Segment * const SG21311[] = {SGI95879, SGI95880, SGI95881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95882, SGI95882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95883, SGI0, NULL};
const Segment * const SG21312[] = {SGI95884, SGI95885, SGI95886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95887, SGI0, SGI0, SGI0, SGI95888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21313[] = {SGI95889, SGI95890, SGI95891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95892, SGI0, SGI0, SGI0, SGI95893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21314[] = {SGI95894, SGI95895, SGI95896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95897, SGI0, SGI0, SGI0, SGI95898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21315[] = {SGI95899, SGI95900, SGI95901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95902, SGI0, SGI0, SGI0, SGI95903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21316[] = {SGI95904, SGI95905, SGI95906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95907, SGI0, SGI0, SGI0, SGI95908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21317[] = {SGI95909, SGI95910, SGI95911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95912, SGI0, SGI0, SGI0, SGI95913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21318[] = {SGI95914, SGI95915, SGI95916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95917, SGI0, SGI0, SGI0, SGI95918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21319[] = {SGI95919, SGI95920, SGI95921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95922, SGI0, SGI0, SGI0, SGI95923, SGI95923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21320[] = {SGI95924, SGI95925, SGI95926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95927, SGI0, SGI0, SGI0, SGI95928, SGI95928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21321[] = {SGI95929, SGI95930, SGI95931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95932, SGI0, SGI0, SGI0, SGI95933, SGI95933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21322[] = {SGI95934, SGI0, SGI95935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95936, SGI95937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21323[] = {SGI95938, SGI95939, SGI95940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95941, SGI0, SGI0, SGI0, SGI95942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21324[] = {SGI95943, SGI95944, SGI95945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95946, SGI0, SGI0, SGI0, SGI95947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21325[] = {SGI95948, SGI95949, SGI95950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95951, SGI0, SGI0, SGI0, SGI95952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21326[] = {SGI95953, SGI95954, SGI95955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95956, SGI0, SGI0, SGI0, SGI95957, SGI95957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21327[] = {SGI95958, SGI95959, SGI95960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95961, SGI0, SGI0, SGI0, SGI95962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21328[] = {SGI95963, SGI95964, SGI95965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95966, SGI0, SGI0, SGI0, SGI95967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21329[] = {SGI95968, SGI95969, SGI95970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95971, SGI0, SGI0, SGI0, SGI95972, SGI95972, SGI95973, SGI0, SGI95974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21330[] = {SGI95975, SGI95976, SGI95977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95978, SGI0, SGI0, SGI0, SGI95979, SGI95979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21331[] = {SGI95980, SGI95981, SGI95982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95983, SGI0, SGI0, SGI0, SGI95984, SGI95984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21332[] = {SGI95985, SGI95986, SGI95987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95988, SGI0, SGI0, SGI0, SGI95989, SGI95989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21333[] = {SGI95990, SGI95991, SGI95992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95993, SGI0, SGI0, SGI0, SGI95994, SGI95994, SGI0, SGI0, SGI0, SGI95995, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21334[] = {SGI95996, SGI0, SGI95997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI95998, SGI95999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21335[] = {SGI96000, SGI96001, SGI96002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96003, SGI0, SGI0, SGI0, SGI96004, SGI96004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21336[] = {SGI96005, SGI96006, SGI96007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96008, SGI0, SGI0, SGI0, SGI96009, SGI96009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21337[] = {SGI96010, SGI96011, SGI96012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96013, SGI0, SGI0, SGI0, SGI96014, SGI96014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21338[] = {SGI96015, SGI96016, SGI96017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96018, SGI0, SGI0, SGI0, SGI96019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21339[] = {SGI96020, SGI96021, SGI96022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96023, SGI0, SGI0, SGI0, SGI96024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21340[] = {SGI96025, SGI96026, SGI96027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96028, SGI0, SGI0, SGI0, SGI96029, SGI96029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21341[] = {SGI96030, SGI0, SGI96031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96032, SGI96033, SGI96033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21342[] = {SGI96034, SGI0, SGI96035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96036, SGI96037, SGI96037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21343[] = {SGI96038, SGI96039, SGI96040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96041, SGI0, SGI0, SGI0, SGI96042, SGI96042, SGI96043, SGI0, SGI96044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21344[] = {SGI96045, SGI96046, SGI96047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96048, SGI0, SGI0, SGI0, SGI96049, SGI96049, SGI0, SGI0, SGI96050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21345[] = {SGI96051, SGI96052, SGI96053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96054, SGI0, SGI0, SGI0, SGI96055, SGI96055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21346[] = {SGI96056, SGI0, SGI96057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21347[] = {SGI96058, SGI96059, SGI96060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96061, SGI0, SGI0, SGI0, SGI96062, SGI96062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21348[] = {SGI96063, SGI96064, SGI96065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96066, SGI0, SGI0, SGI0, SGI96067, SGI96067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21349[] = {SGI96068, SGI96069, SGI96070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96071, SGI0, SGI0, SGI0, SGI96072, SGI96072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21350[] = {SGI96073, SGI96074, SGI96075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96076, SGI96076, SGI0, SGI0, SGI96077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21351[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96078, SGI0, SGI96079, SGI0, NULL};
const Segment * const SG21352[] = {SGI96080, SGI0, SGI96081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96082, SGI96083, SGI96083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21353[] = {SGI96084, SGI0, SGI96085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96086, SGI96087, SGI96087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21354[] = {SGI96088, SGI0, SGI96089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96090, SGI96091, SGI96091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21355[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96092, SGI0, SGI96093, SGI0, NULL};
const Segment * const SG21356[] = {SGI96094, SGI96095, SGI96096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96097, SGI0, SGI0, SGI0, SGI96098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21357[] = {SGI96099, SGI96100, SGI96101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96102, SGI0, SGI0, SGI0, SGI96103, SGI96103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21358[] = {SGI96104, SGI96105, SGI96106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96107, SGI0, SGI0, SGI0, SGI96108, SGI96108, SGI96109, SGI0, SGI96110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21359[] = {SGI96111, SGI96112, SGI96113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96114, SGI0, SGI0, SGI0, SGI96115, SGI96115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21360[] = {SGI96116, SGI96117, SGI96118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96119, SGI0, SGI0, SGI0, SGI96120, SGI96120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21361[] = {SGI96121, SGI96122, SGI96123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96125, SGI0, SGI0, SGI0, SGI96128, SGI96128, SGI96129, SGI0, SGI96130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21362[] = {SGI96124, SGI96126, SGI96127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96131, SGI0, SGI0, SGI0, SGI96132, SGI96132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21363[] = {SGI96133, SGI96134, SGI96135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96136, SGI0, SGI0, SGI0, SGI96137, SGI96137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21364[] = {SGI96138, SGI96139, SGI96140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96141, SGI0, SGI0, SGI0, SGI96142, SGI96142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21365[] = {SGI96143, SGI96144, SGI96145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96146, SGI0, SGI0, SGI0, SGI96147, SGI96147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21366[] = {SGI96148, SGI0, SGI96149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96150, SGI96151, SGI96151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21367[] = {SGI96152, SGI0, SGI96153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96154, SGI96155, SGI96155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21368[] = {SGI96156, SGI0, SGI96157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96158, SGI96159, SGI96159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21369[] = {SGI96160, SGI96161, SGI96162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96163, SGI0, SGI0, SGI0, SGI96164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21370[] = {SGI96165, SGI0, SGI96166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96167, SGI96168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96169, SGI0, NULL};
const Segment * const SG21371[] = {SGI96170, SGI96171, SGI96172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96173, SGI96173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96174, SGI0, SGI0, NULL};
const Segment * const SG21372[] = {SGI96175, SGI96176, SGI96177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96178, SGI0, SGI0, SGI0, SGI96179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21373[] = {SGI96180, SGI96181, SGI96182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96183, SGI0, SGI0, SGI0, SGI96184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21374[] = {SGI96185, SGI0, SGI96186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96187, SGI96188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21375[] = {SGI96189, SGI0, SGI96190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96191, SGI96192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21376[] = {SGI96193, SGI96194, SGI96195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96196, SGI0, SGI0, SGI0, SGI96197, SGI96197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21377[] = {SGI96198, SGI96199, SGI96200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96201, SGI0, SGI0, SGI0, SGI96202, SGI96202, SGI0, SGI0, SGI0, SGI96203, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21378[] = {SGI96204, SGI96205, SGI96206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96207, SGI0, SGI0, SGI0, SGI96208, SGI96208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96209, SGI0, NULL};
const Segment * const SG21379[] = {SGI96210, SGI96211, SGI96212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96213, SGI0, SGI0, SGI0, SGI96214, SGI96214, SGI0, SGI0, NULL};
const Segment * const SG21380[] = {SGI96215, SGI96216, SGI96217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96218, SGI0, SGI0, SGI0, SGI96219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21381[] = {SGI96220, SGI0, SGI96221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21382[] = {SGI96222, SGI0, SGI96223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96224, SGI96225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21383[] = {SGI96226, SGI96227, SGI96228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96229, SGI0, SGI0, SGI0, SGI96230, SGI96230, SGI0, SGI96231, SGI0, NULL};
const Segment * const SG21384[] = {SGI96232, SGI96233, SGI96234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96235, SGI0, SGI0, SGI0, SGI96236, SGI96236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21385[] = {SGI96237, SGI0, SGI96238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96239, SGI96240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21386[] = {SGI96241, SGI96242, SGI96243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96244, SGI0, SGI0, SGI0, SGI96245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21387[] = {SGI96246, SGI96247, SGI96248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96249, SGI0, SGI0, SGI0, SGI96250, SGI96250, SGI0, SGI0, SGI0, SGI96251, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21388[] = {SGI96252, SGI96253, SGI96254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96255, SGI0, SGI0, SGI0, SGI96256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21389[] = {SGI96257, SGI0, SGI96258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96259, SGI96260, SGI96260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21390[] = {SGI96261, SGI0, SGI96262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96263, SGI96264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21391[] = {SGI96265, SGI96266, SGI96267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96268, SGI0, SGI0, SGI0, SGI96269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21392[] = {SGI96270, SGI96271, SGI96272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96273, SGI0, SGI0, SGI0, SGI96274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21393[] = {SGI96275, SGI96276, SGI96277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96278, SGI0, SGI0, SGI0, SGI96279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21394[] = {SGI96280, SGI96281, SGI96282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96283, SGI0, SGI0, SGI0, SGI96284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21395[] = {SGI96285, SGI0, SGI96286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96287, SGI96288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21396[] = {SGI96289, SGI0, SGI96290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96291, SGI96292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21397[] = {SGI96293, SGI0, SGI96294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96295, SGI96296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21398[] = {SGI96297, SGI0, SGI96298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96299, SGI96300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21399[] = {SGI96301, SGI0, SGI96302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96303, SGI96304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21400[] = {SGI96305, SGI96306, SGI96307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96308, SGI0, SGI0, SGI0, SGI96309, SGI96309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21401[] = {SGI96310, SGI0, SGI96311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96312, SGI0, SGI0, SGI0, SGI96313, SGI96313, SGI0, SGI0, SGI96314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21402[] = {SGI96315, SGI96316, SGI96317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96318, SGI0, SGI0, SGI0, SGI96319, SGI96319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21403[] = {SGI96320, SGI96321, SGI96322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96323, SGI0, SGI0, SGI0, SGI96324, SGI96324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96325, SGI0, NULL};
const Segment * const SG21404[] = {SGI96326, SGI0, SGI96327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96328, SGI96329, SGI96329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96330, NULL};
const Segment * const SG21405[] = {SGI96331, SGI96332, SGI96333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96334, SGI96334, SGI0, SGI0, SGI96335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21406[] = {SGI96336, SGI96337, SGI96338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96339, SGI0, SGI0, SGI0, SGI96340, SGI96340, SGI0, SGI0, SGI0, SGI96341, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21407[] = {SGI96342, SGI96343, SGI96344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96345, SGI0, SGI0, SGI0, SGI96346, SGI96346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21408[] = {SGI96347, SGI0, SGI96348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96349, SGI96350, SGI96350, SGI0, SGI0, NULL};
const Segment * const SG21409[] = {SGI96351, SGI96352, SGI96353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96354, SGI0, SGI0, SGI0, SGI96355, SGI96355, SGI96356, SGI0, SGI96357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21410[] = {SGI96358, SGI96359, SGI96360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96361, SGI96361, SGI0, SGI0, SGI96362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21411[] = {SGI96363, SGI0, SGI96364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96365, SGI96366, SGI96366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96367, NULL};
const Segment * const SG21412[] = {SGI96368, SGI0, SGI96369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96370, SGI96371, SGI96371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21413[] = {SGI96372, SGI0, SGI96373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96374, SGI96375, SGI96375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21414[] = {SGI96376, SGI0, SGI96377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96378, SGI96379, SGI96379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21415[] = {SGI96380, SGI0, SGI96381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96382, SGI96383, SGI96383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21416[] = {SGI96384, SGI0, SGI96385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96387, SGI96389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96391, SGI0, NULL};
const Segment * const SG21417[] = {SGI96386, SGI96388, SGI96390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96392, SGI0, SGI0, SGI0, SGI96393, SGI96393, SGI96394, SGI0, SGI96395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21418[] = {SGI96396, SGI96397, SGI96398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96399, SGI0, SGI0, SGI0, SGI96400, SGI96400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96401, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21419[] = {SGI96402, SGI96403, SGI96404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96405, SGI96405, SGI0, SGI0, SGI96406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21420[] = {SGI96407, SGI96408, SGI96409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96410, SGI0, SGI0, SGI0, SGI96411, SGI96411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21421[] = {SGI96414, SGI0, SGI96415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21422[] = {SGI96412, SGI0, SGI96413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96416, SGI0, SGI0, SGI0, SGI96417, SGI96417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96418, SGI0, SGI0, NULL};
const Segment * const SG21423[] = {SGI96419, SGI96420, SGI96421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96422, SGI0, SGI0, SGI0, SGI96423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21424[] = {SGI96424, SGI96425, SGI96426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96427, SGI0, SGI0, SGI0, SGI96428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21425[] = {SGI96429, SGI0, SGI96430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21426[] = {SGI96431, SGI96432, SGI96433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96434, SGI0, SGI0, SGI0, SGI96435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21427[] = {SGI96436, SGI96437, SGI96438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96439, SGI0, SGI0, SGI0, SGI96440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21428[] = {SGI96441, SGI96442, SGI96443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96444, SGI0, SGI0, SGI0, SGI96445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21429[] = {SGI96446, SGI0, SGI96447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96448, SGI0, SGI0, SGI0, SGI96449, SGI96449, SGI0, SGI0, SGI96450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21430[] = {SGI96451, SGI0, SGI96452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96453, SGI96454, SGI96454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21431[] = {SGI96455, SGI0, SGI96456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96457, SGI96458, SGI96458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21432[] = {SGI96459, SGI0, SGI96460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96461, SGI96462, SGI96462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21433[] = {SGI96463, SGI96464, SGI96465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96466, SGI96466, SGI0, SGI0, SGI96467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21434[] = {SGI96468, SGI0, SGI96469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96470, SGI96471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21435[] = {SGI96472, SGI96473, SGI96474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96475, SGI96475, SGI0, SGI0, SGI96476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21436[] = {SGI96477, SGI0, SGI96478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96479, SGI96480, SGI96480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21437[] = {SGI96481, SGI0, SGI96482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96483, SGI96484, SGI96484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96485, SGI0, NULL};
const Segment * const SG21438[] = {SGI96486, SGI96487, SGI96488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96489, SGI0, SGI0, SGI0, SGI96490, SGI96490, SGI0, SGI0, SGI0, SGI96491, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21439[] = {SGI96492, SGI0, SGI96493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96494, SGI96495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96496, SGI0, NULL};
const Segment * const SG21440[] = {SGI96497, SGI0, SGI96498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96499, SGI96500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96501, SGI0, NULL};
const Segment * const SG21441[] = {SGI96502, SGI96503, SGI96504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96505, SGI0, SGI0, SGI0, SGI96506, SGI96506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21442[] = {SGI96507, SGI96508, SGI96509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96510, SGI96510, SGI0, SGI0, SGI96511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21443[] = {SGI96512, SGI0, SGI96513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96514, SGI96515, SGI96515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21444[] = {SGI96516, SGI0, SGI96517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96518, SGI96519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21445[] = {SGI96520, SGI0, SGI96521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96522, SGI96523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21446[] = {SGI96524, SGI0, SGI96525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96526, SGI96527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21447[] = {SGI96528, SGI96529, SGI96530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96531, SGI0, SGI0, SGI0, SGI96532, SGI96532, SGI0, SGI0, SGI0, SGI96533, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21448[] = {SGI96534, SGI0, SGI96535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96536, SGI96537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21449[] = {SGI96538, SGI0, SGI96539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96540, SGI96541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21450[] = {SGI96542, SGI96543, SGI96544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96545, SGI0, SGI0, SGI0, SGI96546, SGI96546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21451[] = {SGI96547, SGI96548, SGI96549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96550, SGI0, SGI0, SGI0, SGI96551, SGI96551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96552, SGI0, NULL};
const Segment * const SG21452[] = {SGI96553, SGI96554, SGI96555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96556, SGI0, SGI0, SGI0, SGI96557, SGI96557, SGI0, SGI0, SGI0, SGI96558, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21453[] = {SGI96559, SGI96560, SGI96561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96562, SGI96562, SGI0, SGI0, SGI96563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21454[] = {SGI96564, SGI96565, SGI96566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96567, SGI0, SGI0, SGI0, SGI96568, SGI96568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96569, SGI0, NULL};
const Segment * const SG21455[] = {SGI96570, SGI96571, SGI96572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96573, SGI0, SGI0, SGI0, SGI96574, SGI96574, SGI96575, SGI0, SGI96576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21456[] = {SGI96577, SGI0, SGI96578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96579, SGI96580, SGI96580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21457[] = {SGI96581, SGI0, SGI96582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96583, SGI96584, SGI96584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21458[] = {SGI96585, SGI96586, SGI96587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96588, SGI0, SGI0, SGI0, SGI96589, SGI96589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21459[] = {SGI96590, SGI96591, SGI96592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96593, SGI0, SGI0, SGI0, SGI96594, SGI96594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96595, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21460[] = {SGI96596, SGI96597, SGI96598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96599, SGI0, SGI0, SGI0, SGI96600, SGI96600, SGI0, SGI96601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21461[] = {SGI96602, SGI96603, SGI96604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96605, SGI96605, SGI0, SGI0, SGI96606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21462[] = {SGI96607, SGI96608, SGI96609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96610, SGI0, SGI0, SGI0, SGI96611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21463[] = {SGI96612, SGI96613, SGI96614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96615, SGI0, SGI0, SGI0, SGI96616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21464[] = {SGI96617, SGI0, SGI96618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96619, SGI96620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21465[] = {SGI96621, SGI96622, SGI96623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96624, SGI0, SGI0, SGI0, SGI96625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21466[] = {SGI96626, SGI96627, SGI96628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96629, SGI0, SGI0, SGI0, SGI96630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21467[] = {SGI96631, SGI0, SGI96632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96633, SGI0, SGI0, SGI0, SGI96634, SGI96634, SGI0, SGI0, SGI96635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21468[] = {SGI96636, SGI0, SGI96637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96638, SGI0, SGI0, SGI0, SGI96639, SGI96639, SGI0, SGI0, SGI96640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21469[] = {SGI0, SGI96641, SGI96642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96643, SGI0, SGI0, SGI0, SGI96644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21470[] = {SGI0, SGI96645, SGI96646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96647, SGI0, SGI0, SGI0, SGI96648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21471[] = {SGI96649, SGI96650, SGI96651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96652, SGI0, SGI0, SGI0, SGI96653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21472[] = {SGI96654, SGI96655, SGI96656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96657, SGI0, SGI0, SGI0, SGI96658, SGI96659, SGI0, SGI0, SGI96660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21473[] = {SGI96661, SGI96662, SGI96663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96664, SGI0, SGI0, SGI0, SGI96665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21474[] = {SGI96666, SGI0, SGI96667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96668, SGI0, SGI0, SGI0, SGI96669, SGI96669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96670, SGI0, SGI0, NULL};
const Segment * const SG21475[] = {SGI96671, SGI96672, SGI96673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96674, SGI96674, SGI0, SGI0, SGI96675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21476[] = {SGI96676, SGI96677, SGI96678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96679, SGI96679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96680, SGI0, NULL};
const Segment * const SG21477[] = {SGI96681, SGI0, SGI96682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96683, SGI96684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21478[] = {SGI96685, SGI96686, SGI96687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96688, SGI0, SGI0, SGI0, SGI96689, SGI96689, SGI96690, SGI0, SGI96691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21479[] = {SGI96692, SGI96693, SGI96694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96695, SGI0, SGI0, SGI0, SGI96696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21480[] = {SGI96697, SGI96698, SGI96699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96700, SGI0, SGI0, SGI0, SGI96701, SGI96701, SGI0, SGI0, SGI0, SGI96702, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21481[] = {SGI96703, SGI96704, SGI96705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96709, SGI0, SGI0, SGI0, SGI96710, SGI96710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96713, SGI0, NULL};
const Segment * const SG21482[] = {SGI96706, SGI96707, SGI96708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96711, SGI0, SGI0, SGI0, SGI96712, SGI96712, SGI0, SGI0, NULL};
const Segment * const SG21483[] = {SGI96714, SGI0, SGI96715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96716, SGI96717, SGI96717, SGI0, SGI96718, SGI0, NULL};
const Segment * const SG21484[] = {SGI96719, SGI0, SGI96720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96721, SGI0, SGI0, SGI0, SGI96722, SGI96722, SGI0, SGI0, SGI96723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21485[] = {SGI96724, SGI96725, SGI96726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96727, SGI0, SGI0, SGI0, SGI96728, SGI96728, SGI0, SGI96729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21486[] = {SGI96730, SGI96731, SGI96732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96733, SGI0, SGI0, SGI0, SGI96734, SGI96734, SGI0, SGI96735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21487[] = {SGI96736, SGI0, SGI96737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96738, SGI96739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21488[] = {SGI96740, SGI96741, SGI96742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96744, SGI0, SGI0, SGI0, SGI96746, SGI96746, SGI0, SGI96748, SGI0, NULL};
const Segment * const SG21489[] = {SGI96743, SGI96745, SGI96747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96749, SGI0, SGI0, SGI0, SGI96750, SGI96750, SGI0, SGI96751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21490[] = {SGI96752, SGI96753, SGI96754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96755, SGI96755, SGI0, SGI0, SGI96756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21491[] = {SGI0, SGI96757, SGI96758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96759, SGI0, SGI0, SGI0, SGI96760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21492[] = {SGI96761, SGI96762, SGI96763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96764, SGI0, SGI0, SGI0, SGI96765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21493[] = {SGI96766, SGI96767, SGI96768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96769, SGI0, SGI0, SGI0, SGI96770, SGI96770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21494[] = {SGI96771, SGI0, SGI96772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21495[] = {SGI96773, SGI0, SGI96774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96775, SGI96776, SGI96776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21496[] = {SGI96777, SGI0, SGI96778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96779, SGI96780, SGI96780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21497[] = {SGI96781, SGI0, SGI96782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96783, SGI96784, SGI96784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21498[] = {SGI96785, SGI0, SGI96786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96787, SGI96788, SGI96788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21499[] = {SGI96789, SGI96790, SGI96791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96792, SGI0, SGI0, SGI0, SGI96793, SGI96793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21500[] = {SGI96794, SGI0, SGI96795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96796, SGI96797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21501[] = {SGI96798, SGI0, SGI96799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96800, SGI96801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21502[] = {SGI96802, SGI0, SGI96803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96804, SGI0, SGI0, SGI0, SGI96805, SGI96805, SGI0, SGI0, SGI96806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21503[] = {SGI96807, SGI96808, SGI96809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96810, SGI0, SGI0, SGI0, SGI96811, SGI96811, SGI0, SGI0, SGI0, SGI96812, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21504[] = {SGI96813, SGI96814, SGI96815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96816, SGI0, SGI0, SGI0, SGI96817, SGI96817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96818, SGI0, NULL};
const Segment * const SG21505[] = {SGI96819, SGI0, SGI96820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96821, SGI96822, SGI96822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21506[] = {SGI96823, SGI0, SGI96824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96825, SGI96826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21507[] = {SGI96827, SGI96828, SGI96829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96830, SGI96830, SGI0, SGI0, SGI96831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21508[] = {SGI96832, SGI96833, SGI96834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96835, SGI0, SGI0, SGI0, SGI96836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21509[] = {SGI96837, SGI0, SGI96838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21510[] = {SGI96839, SGI96840, SGI96841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96842, SGI0, SGI0, SGI0, SGI96843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21511[] = {SGI96844, SGI96845, SGI96846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96847, SGI0, SGI0, SGI0, SGI96848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21512[] = {SGI96849, SGI96850, SGI96851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96852, SGI0, SGI0, SGI0, SGI96853, SGI96853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21513[] = {SGI96854, SGI96855, SGI96856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96857, SGI0, SGI0, SGI0, SGI96858, SGI96858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21514[] = {SGI0, SGI96859, SGI0, SGI96860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21515[] = {SGI96861, SGI0, SGI96862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96863, SGI96864, SGI96864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21516[] = {SGI96865, SGI0, SGI96866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96867, SGI96868, SGI96868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21517[] = {SGI96869, SGI96870, SGI96871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96872, SGI0, SGI0, SGI0, SGI96873, SGI96873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96874, SGI0, NULL};
const Segment * const SG21518[] = {SGI96875, SGI96876, SGI96877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96878, SGI0, SGI0, SGI0, SGI96879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21519[] = {SGI0, SGI96880, SGI96881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96882, SGI0, SGI0, SGI0, SGI96883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21520[] = {SGI96884, SGI96885, SGI96886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96887, SGI0, SGI0, SGI0, SGI96888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21521[] = {SGI96889, SGI0, SGI96890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96891, SGI0, SGI0, SGI0, SGI96892, SGI96892, SGI0, SGI0, SGI96893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21522[] = {SGI96894, SGI96895, SGI96896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96897, SGI96897, SGI0, SGI0, SGI96898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21523[] = {SGI96899, SGI96900, SGI96901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96902, SGI0, SGI0, SGI0, SGI96903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21524[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96904, SGI0, SGI96905, SGI0, NULL};
const Segment * const SG21525[] = {SGI96906, SGI96907, SGI96908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96909, SGI0, SGI0, SGI0, SGI96910, SGI96910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21526[] = {SGI0, SGI96911, SGI96912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96913, SGI0, SGI0, SGI0, SGI96914, SGI96914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21527[] = {SGI96915, SGI96916, SGI96917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96918, SGI96918, SGI0, SGI0, SGI96919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21528[] = {SGI96920, SGI96921, SGI96922, SGI0, SGI0, SGI0, SGI96923, SGI96924, SGI0, SGI96925, SGI0, SGI0, SGI0, SGI96926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21529[] = {SGI96927, SGI0, SGI96928, SGI0, SGI0, SGI0, SGI96929, SGI96930, SGI0, SGI96931, SGI0, SGI0, SGI0, SGI96932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21530[] = {SGI96933, SGI0, SGI96934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96935, SGI96936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21531[] = {SGI96937, SGI96938, SGI96939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96940, SGI0, SGI0, SGI0, SGI96941, SGI96941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21532[] = {SGI96942, SGI0, SGI96943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96944, SGI96945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21533[] = {SGI96946, SGI96947, SGI96948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96949, SGI0, SGI0, SGI0, SGI96950, SGI96950, SGI0, SGI0, SGI0, SGI96951, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21534[] = {SGI96952, SGI96953, SGI96954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96955, SGI0, SGI0, SGI0, SGI96956, SGI96956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21535[] = {SGI96957, SGI96958, SGI96959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96960, SGI0, SGI0, SGI0, SGI96961, SGI96961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96962, SGI0, NULL};
const Segment * const SG21536[] = {SGI96963, SGI0, SGI96964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96965, SGI0, SGI0, SGI0, SGI96966, SGI96966, SGI0, SGI0, SGI96967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21537[] = {SGI96968, SGI96969, SGI96970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96971, SGI0, SGI0, SGI0, SGI96972, SGI96972, SGI0, SGI0, SGI0, SGI96973, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21538[] = {SGI96974, SGI96975, SGI96976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96977, SGI96977, SGI0, SGI0, SGI96978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21539[] = {SGI96979, SGI96980, SGI96981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96982, SGI0, SGI0, SGI0, SGI96983, SGI96983, SGI0, SGI96984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21540[] = {SGI96985, SGI96986, SGI96987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96988, SGI0, SGI0, SGI0, SGI96989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21541[] = {SGI96990, SGI96991, SGI96992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96993, SGI0, SGI0, SGI0, SGI96994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21542[] = {SGI96995, SGI96996, SGI96997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI96998, SGI0, SGI0, SGI0, SGI96999, SGI96999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21543[] = {SGI97000, SGI97001, SGI97002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97003, SGI0, SGI0, SGI0, SGI97004, SGI97004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21544[] = {SGI97005, SGI97006, SGI97007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97008, SGI0, SGI0, SGI0, SGI97009, SGI97009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21545[] = {SGI97010, SGI97011, SGI97012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97013, SGI0, SGI0, SGI0, SGI97014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21546[] = {SGI97015, SGI0, SGI97016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21547[] = {SGI97017, SGI97018, SGI97019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97020, SGI0, SGI0, SGI0, SGI97021, SGI97021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21548[] = {SGI0, SGI97022, SGI97023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97024, SGI0, SGI0, SGI0, SGI97025, SGI97025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21549[] = {SGI97026, SGI97027, SGI97028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97029, SGI0, SGI0, SGI0, SGI97030, SGI97030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21550[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97031, SGI0, SGI97032, SGI0, NULL};
const Segment * const SG21551[] = {SGI97033, SGI97034, SGI97035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97036, SGI0, SGI0, SGI0, SGI97037, SGI97037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21552[] = {SGI97038, SGI0, SGI97039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97040, SGI97041, SGI97041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97042, SGI0, NULL};
const Segment * const SG21553[] = {SGI97043, SGI0, SGI97044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97045, SGI97046, SGI97046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97047, SGI0, NULL};
const Segment * const SG21554[] = {SGI97048, SGI97049, SGI97050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97051, SGI0, SGI0, SGI0, SGI97052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21555[] = {SGI97053, SGI0, SGI97054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21556[] = {SGI97055, SGI97056, SGI97057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97058, SGI0, SGI0, SGI0, SGI97059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21557[] = {SGI97060, SGI97061, SGI97062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97063, SGI0, SGI0, SGI0, SGI97064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21558[] = {SGI97065, SGI97066, SGI97067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97068, SGI0, SGI0, SGI0, SGI97069, SGI97069, SGI0, SGI0, SGI0, SGI97070, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21559[] = {SGI97071, SGI97072, SGI97073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97074, SGI0, SGI0, SGI0, SGI97075, SGI97075, SGI0, SGI97076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21560[] = {SGI97077, SGI97078, SGI97079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97080, SGI0, SGI0, SGI0, SGI97081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21561[] = {SGI97082, SGI97083, SGI97084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97085, SGI0, SGI0, SGI0, SGI97086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21562[] = {SGI97087, SGI97088, SGI97089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97090, SGI0, SGI0, SGI0, SGI97091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21563[] = {SGI97092, SGI0, SGI97093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97094, SGI97096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21564[] = {SGI97095, SGI97097, SGI97098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97099, SGI0, SGI0, SGI0, SGI97100, SGI97100, SGI0, SGI0, SGI0, SGI97101, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21565[] = {SGI97102, SGI0, SGI97103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97104, SGI97106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21566[] = {SGI97105, SGI97107, SGI97108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97109, SGI0, SGI0, SGI0, SGI97110, SGI97110, SGI0, SGI0, SGI0, SGI97111, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21567[] = {SGI97112, SGI0, SGI97113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97114, SGI97115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21568[] = {SGI97116, SGI0, SGI97117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97118, SGI97119, SGI97119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97120, SGI0, NULL};
const Segment * const SG21569[] = {SGI97121, SGI0, SGI97122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97123, SGI97124, SGI97124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97125, SGI0, NULL};
const Segment * const SG21570[] = {SGI97126, SGI97127, SGI97128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97129, SGI0, SGI0, SGI0, SGI97130, SGI97130, SGI0, SGI0, SGI0, SGI97131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21571[] = {SGI97132, SGI0, SGI97133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97134, SGI97135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21572[] = {SGI97136, SGI97137, SGI97138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97139, SGI0, SGI0, SGI0, SGI97140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21573[] = {SGI97141, SGI97142, SGI97143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97144, SGI0, SGI0, SGI0, SGI97145, SGI97145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21574[] = {SGI97146, SGI97147, SGI97148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97149, SGI0, SGI0, SGI0, SGI97150, SGI97150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21575[] = {SGI97151, SGI97152, SGI97153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97154, SGI0, SGI0, SGI0, SGI97155, SGI97156, SGI0, SGI0, SGI97157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21576[] = {SGI97158, SGI0, SGI97159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97160, SGI97161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21577[] = {SGI97162, SGI0, SGI97163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97164, SGI97165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21578[] = {SGI97166, SGI97167, SGI97168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97169, SGI0, SGI0, SGI0, SGI97170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21579[] = {SGI97171, SGI97172, SGI97173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97174, SGI0, SGI0, SGI0, SGI97175, SGI97175, SGI0, SGI0, SGI0, SGI97176, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21580[] = {SGI97177, SGI0, SGI97178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97179, SGI0, SGI0, SGI0, SGI97180, SGI97180, SGI0, SGI0, SGI97181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21581[] = {SGI97182, SGI97183, SGI97184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97185, SGI97185, SGI0, SGI0, SGI97186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21582[] = {SGI97187, SGI97188, SGI97189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97190, SGI0, SGI0, SGI0, SGI97191, SGI97191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21583[] = {SGI97192, SGI0, SGI97193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97194, SGI97195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21584[] = {SGI97196, SGI0, SGI97197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97198, SGI97199, SGI97199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21585[] = {SGI97200, SGI0, SGI97201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97202, SGI97203, SGI97203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21586[] = {SGI97204, SGI0, SGI97205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97206, SGI97207, SGI97207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21587[] = {SGI97208, SGI0, SGI97209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97210, SGI97211, SGI97211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21588[] = {SGI97212, SGI97213, SGI97214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97215, SGI0, SGI0, SGI0, SGI97216, SGI97216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21589[] = {SGI0, SGI97217, SGI0, SGI97218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21590[] = {SGI97219, SGI97220, SGI97221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97222, SGI0, SGI0, SGI0, SGI97223, SGI97223, SGI0, SGI97224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21591[] = {SGI97225, SGI97226, SGI97227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97228, SGI0, SGI0, SGI0, SGI97229, SGI97229, SGI0, SGI97230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21592[] = {SGI97231, SGI0, SGI97232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97233, SGI97234, SGI97234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21593[] = {SGI97235, SGI0, SGI97236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97237, SGI97238, SGI97238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21594[] = {SGI97239, SGI0, SGI97240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97241, SGI97242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21595[] = {SGI97243, SGI97244, SGI97245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97246, SGI0, SGI0, SGI0, SGI97247, SGI97247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97248, SGI0, NULL};
const Segment * const SG21596[] = {SGI97249, SGI97250, SGI97251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97252, SGI97252, SGI0, SGI0, SGI97253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21597[] = {SGI97254, SGI97255, SGI97256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97257, SGI0, SGI0, SGI0, SGI97258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21598[] = {SGI97259, SGI97260, SGI97261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97262, SGI0, SGI0, SGI0, SGI97263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21599[] = {SGI97264, SGI97265, SGI97266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97267, SGI0, SGI0, SGI0, SGI97268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21600[] = {SGI97269, SGI97270, SGI97271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97275, SGI0, SGI0, SGI0, SGI97276, SGI97276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97277, SGI0, SGI0, NULL};
const Segment * const SG21601[] = {SGI97272, SGI97273, SGI97274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97278, SGI0, SGI0, SGI0, SGI0, SGI97279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21602[] = {SGI97280, SGI97281, SGI97282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97283, SGI0, SGI0, SGI0, SGI0, SGI97284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21603[] = {SGI97285, SGI97286, SGI97287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97288, SGI0, SGI0, SGI0, SGI0, SGI97289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21604[] = {SGI97290, SGI97291, SGI97292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97293, SGI0, SGI0, SGI0, SGI97294, SGI97294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21605[] = {SGI97295, SGI97296, SGI97297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97298, SGI0, SGI0, SGI0, SGI97299, SGI97299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21606[] = {SGI97300, SGI0, SGI97301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21607[] = {SGI97302, SGI97303, SGI97304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97305, SGI0, SGI0, SGI0, SGI97306, SGI97306, SGI0, SGI0, SGI0, SGI97307, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21608[] = {SGI0, SGI97308, SGI0, SGI97309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21609[] = {SGI97310, SGI97311, SGI97312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97313, SGI0, SGI0, SGI0, SGI97314, SGI97314, SGI97315, SGI0, SGI97316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21610[] = {SGI97317, SGI97318, SGI97319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97320, SGI97320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97321, SGI0, NULL};
const Segment * const SG21611[] = {SGI97322, SGI97323, SGI97324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97325, SGI97325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97326, SGI0, NULL};
const Segment * const SG21612[] = {SGI97327, SGI97328, SGI97329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97330, SGI0, SGI0, SGI0, SGI97331, SGI97331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21613[] = {SGI97332, SGI97333, SGI97334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97335, SGI0, SGI0, SGI0, SGI97336, SGI97337, SGI0, SGI0, SGI97338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21614[] = {SGI97339, SGI97340, SGI97341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97342, SGI0, SGI0, SGI0, SGI97343, SGI97343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97344, SGI0, SGI0, NULL};
const Segment * const SG21615[] = {SGI97345, SGI97346, SGI97347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97348, SGI0, SGI0, SGI0, SGI97349, SGI97349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97350, SGI0, NULL};
const Segment * const SG21616[] = {SGI97351, SGI97352, SGI97353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97354, SGI0, SGI0, SGI0, SGI97355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21617[] = {SGI97356, SGI97357, SGI97358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97359, SGI0, SGI0, SGI0, SGI97360, SGI97360, SGI0, SGI0, SGI0, SGI97361, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21618[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97362, SGI0, SGI97363, SGI0, NULL};
const Segment * const SG21619[] = {SGI97364, SGI97365, SGI97366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97367, SGI0, SGI0, SGI0, SGI97368, SGI97368, SGI0, SGI97369, SGI0, NULL};
const Segment * const SG21620[] = {SGI97370, SGI97371, SGI97372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97373, SGI0, SGI0, SGI0, SGI97374, SGI97374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97375, SGI0, NULL};
const Segment * const SG21621[] = {SGI97376, SGI0, SGI97377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97378, SGI97379, SGI97379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97380, NULL};
const Segment * const SG21622[] = {SGI97381, SGI97382, SGI97383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97384, SGI0, SGI0, SGI0, SGI97385, SGI97385, SGI0, SGI97386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21623[] = {SGI97387, SGI97388, SGI97389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97390, SGI97390, SGI0, SGI0, SGI97391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21624[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97392, SGI0, SGI97393, SGI0, NULL};
const Segment * const SG21625[] = {SGI97394, SGI0, SGI97395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97396, SGI0, SGI0, SGI0, SGI97397, SGI97397, SGI0, SGI0, SGI97398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21626[] = {SGI97399, SGI97400, SGI97401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97402, SGI0, SGI0, SGI0, SGI97403, SGI97404, SGI0, SGI0, SGI97405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21627[] = {SGI97406, SGI0, SGI97407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97408, SGI0, SGI0, SGI0, SGI97409, SGI97409, SGI0, SGI0, SGI97410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21628[] = {SGI97411, SGI97412, SGI97413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97414, SGI0, SGI0, SGI0, SGI97415, SGI97416, SGI0, SGI0, SGI97417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21629[] = {SGI97418, SGI0, SGI97419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97420, SGI97421, SGI97421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21630[] = {SGI97422, SGI97423, SGI97424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97425, SGI0, SGI0, SGI0, SGI97426, SGI97427, SGI0, SGI0, SGI97428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21631[] = {SGI97429, SGI0, SGI97430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97433, SGI97434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21632[] = {SGI97431, SGI0, SGI97432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97435, SGI0, SGI0, SGI0, SGI97436, SGI97436, SGI0, SGI0, SGI97437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21633[] = {SGI97438, SGI97439, SGI97440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97441, SGI0, SGI0, SGI0, SGI97442, SGI97442, SGI0, SGI97443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21634[] = {SGI97444, SGI97445, SGI97446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97447, SGI0, SGI0, SGI0, SGI97448, SGI97448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21635[] = {SGI97449, SGI0, SGI97450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21636[] = {SGI97451, SGI97452, SGI97453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97454, SGI0, SGI0, SGI0, SGI97455, SGI97455, SGI0, SGI97456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21637[] = {SGI97457, SGI0, SGI97458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97459, SGI97460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97461, SGI0, NULL};
const Segment * const SG21638[] = {SGI97462, SGI0, SGI97463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97464, SGI97465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21639[] = {SGI97466, SGI97467, SGI97468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97469, SGI97469, SGI0, SGI0, SGI97470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21640[] = {SGI97471, SGI97472, SGI97473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97474, SGI0, SGI0, SGI0, SGI97475, SGI97475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21641[] = {SGI97476, SGI0, SGI97477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97478, SGI0, SGI0, SGI0, SGI97479, SGI97479, SGI0, SGI0, SGI97480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21642[] = {SGI97481, SGI97482, SGI97483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97484, SGI0, SGI0, SGI0, SGI97485, SGI97485, SGI0, SGI0, SGI0, SGI97486, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21643[] = {SGI97487, SGI0, SGI97488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97489, SGI0, SGI0, SGI0, SGI97490, SGI97490, SGI0, SGI0, SGI97491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21644[] = {SGI97492, SGI97493, SGI97494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97495, SGI0, SGI0, SGI0, SGI97496, SGI97496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97497, SGI0, NULL};
const Segment * const SG21645[] = {SGI97498, SGI0, SGI97499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97500, SGI97501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21646[] = {SGI97502, SGI97503, SGI97504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97505, SGI0, SGI0, SGI0, SGI97506, SGI97506, SGI0, SGI97507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21647[] = {SGI97508, SGI0, SGI97509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97510, SGI97511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21648[] = {SGI97512, SGI97513, SGI97514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97515, SGI0, SGI0, SGI0, SGI97516, SGI97516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21649[] = {SGI97517, SGI97518, SGI97519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97520, SGI0, SGI0, SGI0, SGI97521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21650[] = {SGI97522, SGI97523, SGI97524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97525, SGI97525, SGI0, SGI0, SGI97526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21651[] = {SGI97527, SGI97528, SGI97529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97530, SGI0, SGI0, SGI0, SGI97531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21652[] = {SGI97532, SGI0, SGI97533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21653[] = {SGI97534, SGI0, SGI97535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97536, SGI97537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21654[] = {SGI97538, SGI97539, SGI97540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97541, SGI0, SGI0, SGI0, SGI97542, SGI97542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21655[] = {SGI97543, SGI97544, SGI97545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97546, SGI0, SGI0, SGI0, SGI97547, SGI97547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21656[] = {SGI97548, SGI97550, SGI97552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97554, SGI0, SGI0, SGI0, SGI97556, SGI97556, SGI0, SGI97557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21657[] = {SGI97549, SGI97551, SGI97553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97555, SGI0, SGI0, SGI0, SGI97558, SGI97558, SGI97559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21658[] = {SGI97560, SGI97561, SGI97562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97563, SGI0, SGI0, SGI0, SGI97564, SGI0, SGI97565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21659[] = {SGI97566, SGI97567, SGI97568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97569, SGI0, SGI0, SGI0, SGI97570, SGI97571, SGI0, SGI0, SGI97572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21660[] = {SGI97573, SGI97574, SGI97575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97576, SGI97577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21661[] = {SGI97578, SGI97579, SGI97580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97581, SGI97581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21662[] = {SGI0, SGI97582, SGI97583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21663[] = {SGI97584, SGI97585, SGI97586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97587, SGI0, SGI0, SGI0, SGI97588, SGI97589, SGI0, SGI0, SGI97590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21664[] = {SGI97591, SGI97592, SGI97593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97594, SGI0, SGI0, SGI0, SGI97595, SGI97596, SGI0, SGI0, SGI97597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21665[] = {SGI97598, SGI97599, SGI97600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97601, SGI97601, SGI0, SGI0, SGI97602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21666[] = {SGI97603, SGI97604, SGI97605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97606, SGI0, SGI0, SGI0, SGI97607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21667[] = {SGI97608, SGI97609, SGI97610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97611, SGI0, SGI0, SGI0, SGI97612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21668[] = {SGI97613, SGI97614, SGI97615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97616, SGI0, SGI0, SGI0, SGI97617, SGI97617, SGI0, SGI97618, SGI0, NULL};
const Segment * const SG21669[] = {SGI97619, SGI0, SGI97620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97621, SGI97622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21670[] = {SGI97623, SGI0, SGI97624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97625, SGI0, SGI0, SGI0, SGI97626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21671[] = {SGI97628, SGI97630, SGI97632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97634, SGI0, SGI0, SGI0, SGI97635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21672[] = {SGI97627, SGI97629, SGI97631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97633, SGI0, SGI0, SGI0, SGI97636, SGI97636, SGI0, SGI0, SGI0, SGI97637, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21673[] = {SGI97638, SGI97639, SGI97640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97641, SGI0, SGI0, SGI0, SGI97642, SGI97642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21674[] = {SGI0, SGI97643, SGI0, SGI97644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21675[] = {SGI97645, SGI97646, SGI97647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97648, SGI0, SGI0, SGI0, SGI97649, SGI97649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97650, SGI0, NULL};
const Segment * const SG21676[] = {SGI97651, SGI97652, SGI97653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97654, SGI0, SGI0, SGI0, SGI97655, SGI97655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97656, SGI0, NULL};
const Segment * const SG21677[] = {SGI97657, SGI0, SGI97658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97659, SGI97660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21678[] = {SGI97661, SGI97662, SGI97663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97664, SGI0, SGI0, SGI0, SGI97665, SGI97665, SGI97666, SGI0, SGI97667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21679[] = {SGI97668, SGI97669, SGI97670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97671, SGI97672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21680[] = {SGI97673, SGI97674, SGI97675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97676, SGI97676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21681[] = {SGI0, SGI97677, SGI97678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21682[] = {SGI97679, SGI97680, SGI97681, SGI97682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97683, SGI0, SGI0, SGI0, SGI97684, SGI97685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21683[] = {SGI97686, SGI97687, SGI97688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97689, SGI97689, SGI0, SGI0, SGI97690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21684[] = {SGI97691, SGI97692, SGI97693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97694, SGI0, SGI0, SGI0, SGI97695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21685[] = {SGI97696, SGI0, SGI97697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97698, SGI97699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97700, SGI0, NULL};
const Segment * const SG21686[] = {SGI97701, SGI97702, SGI97703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97704, SGI0, SGI0, SGI0, SGI97705, SGI97705, SGI0, SGI97706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21687[] = {SGI97707, SGI0, SGI97708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97709, SGI0, SGI0, SGI0, SGI97710, SGI97710, SGI0, SGI0, SGI97711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21688[] = {SGI97712, SGI97713, SGI97714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97715, SGI0, SGI0, SGI0, SGI97716, SGI97717, SGI0, SGI0, SGI97718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21689[] = {SGI97719, SGI97720, SGI97721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97722, SGI0, SGI0, SGI0, SGI97723, SGI97723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21690[] = {SGI97724, SGI0, SGI97725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97726, SGI0, SGI0, SGI0, SGI97727, SGI97727, SGI0, SGI0, SGI97728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21691[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97729, SGI0, SGI0, SGI97730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21692[] = {SGI97731, SGI0, SGI97732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97733, SGI97734, SGI97734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97735, NULL};
const Segment * const SG21693[] = {SGI97736, SGI0, SGI97737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97738, SGI97739, SGI97739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97740, NULL};
const Segment * const SG21694[] = {SGI97741, SGI0, SGI97742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97743, SGI0, SGI0, SGI0, SGI97744, SGI97744, SGI0, SGI0, SGI97745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21695[] = {SGI97746, SGI0, SGI97747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97748, SGI97749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21696[] = {SGI97750, SGI0, SGI97751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97752, SGI97753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21697[] = {SGI97754, SGI97755, SGI97756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97757, SGI0, SGI0, SGI0, SGI97758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21698[] = {SGI97759, SGI0, SGI97760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97761, SGI97762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21699[] = {SGI97763, SGI97764, SGI97765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97766, SGI0, SGI0, SGI0, SGI97767, SGI97767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21700[] = {SGI97768, SGI97769, SGI97770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97771, SGI97771, SGI0, SGI0, SGI97772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21701[] = {SGI97773, SGI97774, SGI97775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97776, SGI0, SGI0, SGI0, SGI97777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21702[] = {SGI97778, SGI0, SGI97779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97780, SGI97781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21703[] = {SGI97782, SGI0, SGI97783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97784, SGI97785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21704[] = {SGI97786, SGI0, SGI97787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97788, SGI97789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21705[] = {SGI97790, SGI97791, SGI97792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97793, SGI0, SGI0, SGI0, SGI97794, SGI97794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21706[] = {SGI97795, SGI97796, SGI97797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97798, SGI97798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21707[] = {SGI97799, SGI97800, SGI97801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97802, SGI0, SGI0, SGI0, SGI97803, SGI97803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21708[] = {SGI97804, SGI97805, SGI97806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97807, SGI0, SGI0, SGI0, SGI97808, SGI97808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21709[] = {SGI97809, SGI97810, SGI97811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97812, SGI0, SGI0, SGI0, SGI97813, SGI97813, SGI97814, SGI0, SGI97815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21710[] = {SGI97816, SGI97817, SGI97818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97819, SGI0, SGI0, SGI0, SGI97820, SGI97820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21711[] = {SGI97821, SGI0, SGI97822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97823, SGI0, SGI0, SGI0, SGI97824, SGI97824, SGI0, SGI0, SGI97825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21712[] = {SGI97826, SGI97827, SGI97828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97829, SGI0, SGI0, SGI0, SGI97830, SGI97830, SGI0, SGI0, SGI0, SGI97831, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21713[] = {SGI97832, SGI97833, SGI97834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97835, SGI0, SGI0, SGI0, SGI97836, SGI97836, SGI0, SGI97837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21714[] = {SGI97838, SGI0, SGI97839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97840, SGI97841, SGI97841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21715[] = {SGI97842, SGI97843, SGI97844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97845, SGI0, SGI0, SGI0, SGI97846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21716[] = {SGI97847, SGI97848, SGI97849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97850, SGI0, SGI0, SGI0, SGI97851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21717[] = {SGI97852, SGI0, SGI97853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97854, SGI97855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21718[] = {SGI97856, SGI0, SGI97857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97858, SGI97859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21719[] = {SGI97860, SGI97861, SGI97862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97863, SGI0, SGI0, SGI0, SGI97864, SGI97865, SGI0, SGI0, SGI97866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21720[] = {SGI97867, SGI97868, SGI97869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97870, SGI0, SGI0, SGI0, SGI97871, SGI97872, SGI0, SGI0, SGI97873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21721[] = {SGI97874, SGI97875, SGI97876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97877, SGI97877, SGI0, SGI0, SGI97878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21722[] = {SGI97879, SGI97880, SGI97881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97882, SGI97882, SGI0, SGI0, SGI97883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21723[] = {SGI97884, SGI97885, SGI97886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97891, SGI97891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97895, SGI0, SGI0, NULL};
const Segment * const SG21724[] = {SGI97887, SGI97888, SGI97889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97890, SGI0, SGI0, SGI0, SGI97892, SGI97893, SGI0, SGI0, SGI97894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21725[] = {SGI97896, SGI97897, SGI97898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97899, SGI0, SGI0, SGI0, SGI97900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21726[] = {SGI97901, SGI0, SGI97902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97906, SGI97907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21727[] = {SGI97903, SGI97904, SGI97905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97908, SGI0, SGI0, SGI0, SGI97909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21728[] = {SGI97910, SGI97911, SGI97912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97913, SGI0, SGI0, SGI0, SGI97914, SGI97914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21729[] = {SGI97915, SGI97916, SGI97917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97918, SGI0, SGI0, SGI0, SGI97919, SGI97919, SGI0, SGI0, SGI0, SGI97920, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21730[] = {SGI97925, SGI97926, SGI97927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97928, SGI0, SGI0, SGI0, SGI97929, SGI97929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97930, SGI0, NULL};
const Segment * const SG21731[] = {SGI97931, SGI0, SGI97932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97933, SGI0, SGI0, SGI0, SGI97934, SGI97934, SGI0, SGI97935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21732[] = {SGI97936, SGI0, SGI97937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97938, SGI97939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21733[] = {SGI97940, SGI97941, SGI97942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97943, SGI0, SGI0, SGI0, SGI97944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21734[] = {SGI97921, SGI97922, SGI97923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97924, SGI0, SGI0, SGI0, SGI97947, SGI97947, SGI0, SGI0, NULL};
const Segment * const SG21735[] = {SGI97948, SGI97949, SGI97950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97951, SGI97951, SGI0, SGI0, SGI97952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21736[] = {SGI97953, SGI97954, SGI97955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97956, SGI0, SGI0, SGI0, SGI97957, SGI97957, SGI0, SGI97958, SGI0, NULL};
const Segment * const SG21737[] = {SGI97959, SGI97960, SGI97961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97962, SGI97962, SGI0, SGI0, SGI97963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21738[] = {SGI97964, SGI97965, SGI97966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97967, SGI0, SGI0, SGI0, SGI97968, SGI97968, SGI0, SGI97969, SGI0, NULL};
const Segment * const SG21739[] = {SGI97970, SGI97971, SGI97972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97973, SGI0, SGI0, SGI0, SGI97974, SGI97974, SGI0, SGI97975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21740[] = {SGI97976, SGI97977, SGI97978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97979, SGI0, SGI0, SGI0, SGI97980, SGI97980, SGI0, SGI97981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21741[] = {SGI97982, SGI97983, SGI97984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97985, SGI0, SGI0, SGI0, SGI97986, SGI97986, SGI0, SGI0, SGI0, SGI97987, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21742[] = {SGI97988, SGI97989, SGI97990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97991, SGI0, SGI0, SGI0, SGI97992, SGI97992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21743[] = {SGI97993, SGI97994, SGI97995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI97996, SGI0, SGI0, SGI0, SGI97997, SGI97997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21744[] = {SGI97998, SGI97999, SGI98000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98001, SGI0, SGI0, SGI0, SGI98002, SGI98002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21745[] = {SGI98003, SGI98004, SGI98005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98006, SGI0, SGI0, SGI0, SGI98007, SGI98007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21746[] = {SGI98008, SGI98009, SGI98010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98011, SGI0, SGI0, SGI0, SGI98012, SGI98012, SGI0, SGI0, SGI0, SGI98013, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21747[] = {SGI98014, SGI0, SGI98015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98016, SGI0, SGI0, SGI0, SGI98017, SGI98017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98018, SGI0, SGI0, NULL};
const Segment * const SG21748[] = {SGI98019, SGI98020, SGI98021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98022, SGI0, SGI0, SGI0, SGI98023, SGI98023, SGI0, SGI0, SGI0, SGI98024, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21749[] = {SGI97945, SGI97946, SGI98025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98026, SGI0, SGI0, SGI0, SGI98027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21750[] = {SGI98028, SGI0, SGI98029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98030, SGI98031, SGI98031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21751[] = {SGI98032, SGI0, SGI98033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98034, SGI98035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21752[] = {SGI98036, SGI0, SGI98037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98038, SGI0, SGI0, SGI0, SGI98039, SGI98039, SGI0, SGI0, SGI98040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21753[] = {SGI98041, SGI0, SGI98042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98043, SGI0, SGI0, SGI0, SGI98044, SGI98044, SGI0, SGI0, SGI98045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21754[] = {SGI98046, SGI98047, SGI98048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98049, SGI0, SGI0, SGI0, SGI98050, SGI98050, SGI0, SGI98051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21755[] = {SGI98052, SGI98053, SGI98054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98055, SGI0, SGI0, SGI0, SGI98056, SGI98056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21756[] = {SGI98057, SGI98058, SGI98059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98060, SGI98060, SGI0, SGI0, SGI98061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21757[] = {SGI98062, SGI98063, SGI98064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98065, SGI0, SGI0, SGI0, SGI98066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21758[] = {SGI98067, SGI98068, SGI98069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98070, SGI0, SGI0, SGI0, SGI98071, SGI98071, SGI0, SGI0, SGI0, SGI98072, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21759[] = {SGI98073, SGI98074, SGI98075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98076, SGI0, SGI0, SGI0, SGI98077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21760[] = {SGI98078, SGI98079, SGI98080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98081, SGI0, SGI0, SGI0, SGI98082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21761[] = {SGI98083, SGI0, SGI98084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98085, SGI0, SGI0, SGI0, SGI98086, SGI98086, SGI0, SGI98087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21762[] = {SGI98088, SGI0, SGI98089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98093, SGI0, SGI0, SGI0, SGI98094, SGI98094, SGI0, SGI98096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21763[] = {SGI98090, SGI98091, SGI98092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98095, SGI0, SGI0, SGI0, SGI98097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21764[] = {SGI98098, SGI98099, SGI98100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98101, SGI0, SGI0, SGI0, SGI98102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21765[] = {SGI98103, SGI98104, SGI98105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98106, SGI0, SGI0, SGI0, SGI98107, SGI98107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98108, SGI0, NULL};
const Segment * const SG21766[] = {SGI98109, SGI98110, SGI98111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98112, SGI0, SGI0, SGI0, SGI98113, SGI98113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98114, SGI0, NULL};
const Segment * const SG21767[] = {SGI98115, SGI0, SGI98116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98117, SGI0, SGI0, SGI0, SGI98118, SGI98118, SGI0, SGI0, SGI98119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21768[] = {SGI98120, SGI0, SGI98121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98122, SGI0, SGI0, SGI0, SGI98123, SGI98123, SGI0, SGI0, SGI98124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21769[] = {SGI98125, SGI0, SGI98126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98127, SGI98128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21770[] = {SGI98129, SGI0, SGI98130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98131, SGI98132, SGI98132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21771[] = {SGI98133, SGI98134, SGI98135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98136, SGI98136, SGI0, SGI0, SGI98137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21772[] = {SGI98138, SGI98139, SGI98140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98141, SGI0, SGI0, SGI0, SGI98142, SGI98142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21773[] = {SGI98143, SGI98144, SGI98145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98146, SGI0, SGI0, SGI0, SGI98147, SGI98147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21774[] = {SGI98148, SGI98149, SGI98150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98151, SGI0, SGI0, SGI0, SGI98152, SGI98152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21775[] = {SGI98153, SGI98154, SGI98155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98156, SGI0, SGI0, SGI0, SGI98157, SGI98157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21776[] = {SGI98158, SGI0, SGI98159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98160, SGI98161, SGI98161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21777[] = {SGI98162, SGI0, SGI98163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98164, SGI98165, SGI98165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21778[] = {SGI98166, SGI0, SGI98167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98168, SGI0, SGI0, SGI0, SGI98169, SGI98169, SGI0, SGI0, SGI98170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21779[] = {SGI98171, SGI98172, SGI98173, SGI0, SGI0, SGI0, SGI98174, SGI98175, SGI0, SGI98176, SGI0, SGI0, SGI0, SGI98177, SGI98177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21780[] = {SGI98178, SGI0, SGI98179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98180, SGI0, SGI0, SGI0, SGI98181, SGI98181, SGI0, SGI0, SGI98182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21781[] = {SGI98183, SGI98184, SGI98185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98186, SGI0, SGI0, SGI0, SGI98187, SGI98187, SGI0, SGI98188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21782[] = {SGI98189, SGI0, SGI98190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98191, SGI98192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21783[] = {SGI98193, SGI0, SGI98194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98195, SGI98196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21784[] = {SGI98197, SGI98198, SGI98199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98200, SGI0, SGI0, SGI0, SGI98201, SGI98201, SGI0, SGI98202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21785[] = {SGI98203, SGI98204, SGI98205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98206, SGI0, SGI0, SGI0, SGI98207, SGI98207, SGI0, SGI0, SGI0, SGI98208, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21786[] = {SGI98209, SGI98210, SGI98211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98212, SGI0, SGI0, SGI0, SGI98213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21787[] = {SGI98214, SGI98215, SGI98216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98217, SGI98217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98218, SGI0, NULL};
const Segment * const SG21788[] = {SGI98219, SGI98220, SGI98221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98222, SGI0, SGI0, SGI0, SGI98223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21789[] = {SGI98224, SGI98225, SGI98226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98227, SGI0, SGI0, SGI0, SGI98228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21790[] = {SGI98229, SGI0, SGI98230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98231, SGI98232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21791[] = {SGI98233, SGI98234, SGI98235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98236, SGI98236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98237, SGI0, SGI0, NULL};
const Segment * const SG21792[] = {SGI98238, SGI98239, SGI98240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98241, SGI98241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98242, SGI0, SGI0, NULL};
const Segment * const SG21793[] = {SGI98243, SGI0, SGI98244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98245, SGI98246, SGI98246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21794[] = {SGI98247, SGI98248, SGI98249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98250, SGI0, SGI0, SGI0, SGI98251, SGI98251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21795[] = {SGI98252, SGI98253, SGI98254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98255, SGI98256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21796[] = {SGI98257, SGI0, SGI98258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98259, SGI98260, SGI98260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21797[] = {SGI98261, SGI0, SGI98262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98263, SGI98264, SGI98264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21798[] = {SGI98265, SGI0, SGI98266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98267, SGI98268, SGI98268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21799[] = {SGI98269, SGI98270, SGI98271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98272, SGI0, SGI0, SGI0, SGI98273, SGI98273, SGI98274, SGI0, SGI98275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21800[] = {SGI98276, SGI98277, SGI98278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98279, SGI0, SGI0, SGI0, SGI98280, SGI98280, SGI98281, SGI0, SGI98282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21801[] = {SGI98283, SGI0, SGI98284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98285, SGI98286, SGI98286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21802[] = {SGI98287, SGI98288, SGI98289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98290, SGI0, SGI0, SGI0, SGI98291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21803[] = {SGI98292, SGI98293, SGI98294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98295, SGI0, SGI0, SGI0, SGI98296, SGI0, SGI0, SGI0, SGI0, SGI98297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21804[] = {SGI98298, SGI98299, SGI98300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98301, SGI0, SGI0, SGI0, SGI98302, SGI0, SGI0, SGI0, SGI0, SGI98303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21805[] = {SGI98304, SGI0, SGI98305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98306, SGI98307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21806[] = {SGI98308, SGI98309, SGI98310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98311, SGI0, SGI0, SGI0, SGI98312, SGI98312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98313, SGI0, NULL};
const Segment * const SG21807[] = {SGI98314, SGI98315, SGI98316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98317, SGI0, SGI0, SGI0, SGI98318, SGI98318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21808[] = {SGI98319, SGI98320, SGI98321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98322, SGI98322, SGI0, SGI0, SGI98323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21809[] = {SGI98324, SGI0, SGI98325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98326, SGI98327, SGI98327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98328, NULL};
const Segment * const SG21810[] = {SGI98329, SGI0, SGI98330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98331, SGI98332, SGI98332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21811[] = {SGI98333, SGI98334, SGI98335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98336, SGI98336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98337, SGI0, SGI0, NULL};
const Segment * const SG21812[] = {SGI98338, SGI98339, SGI98340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98341, SGI0, SGI0, SGI0, SGI98342, SGI98342, SGI98343, SGI0, SGI98344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21813[] = {SGI98345, SGI98346, SGI98347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98348, SGI0, SGI0, SGI0, SGI98349, SGI98349, SGI0, SGI98350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21814[] = {SGI98351, SGI98352, SGI98353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98354, SGI98354, SGI0, SGI0, SGI98355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21815[] = {SGI98356, SGI98357, SGI98358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98359, SGI0, SGI0, SGI0, SGI98360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21816[] = {SGI98361, SGI98362, SGI98363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98364, SGI0, SGI0, SGI0, SGI98365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21817[] = {SGI98366, SGI0, SGI98367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98368, SGI98369, SGI98369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98370, NULL};
const Segment * const SG21818[] = {SGI98371, SGI98372, SGI98373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98374, SGI98374, SGI0, SGI0, SGI98377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21819[] = {SGI98375, SGI98376, SGI98378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98379, SGI0, SGI0, SGI0, SGI98380, SGI98380, SGI0, SGI0, NULL};
const Segment * const SG21820[] = {SGI98381, SGI98382, SGI98383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98384, SGI0, SGI0, SGI0, SGI98385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21821[] = {SGI98386, SGI0, SGI98387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98388, SGI0, SGI0, SGI0, SGI98389, SGI98389, SGI0, SGI0, SGI98390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21822[] = {SGI98391, SGI98392, SGI98393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98394, SGI0, SGI0, SGI0, SGI98395, SGI98395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21823[] = {SGI98396, SGI98397, SGI98398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98399, SGI0, SGI0, SGI0, SGI98400, SGI98400, SGI0, SGI0, SGI0, SGI98401, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21824[] = {SGI98402, SGI98403, SGI98404, SGI0, SGI0, SGI0, SGI98405, SGI98406, SGI0, SGI98407, SGI0, SGI0, SGI0, SGI98408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21825[] = {SGI98409, SGI0, SGI98410, SGI0, SGI0, SGI0, SGI98411, SGI98412, SGI0, SGI98413, SGI0, SGI0, SGI0, SGI98414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21826[] = {SGI98415, SGI0, SGI98416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98417, SGI98418, SGI98418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21827[] = {SGI98419, SGI0, SGI98420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98421, SGI98422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21828[] = {SGI98423, SGI0, SGI98424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98425, SGI98426, SGI98426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21829[] = {SGI98427, SGI0, SGI98428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98429, SGI98430, SGI98430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21830[] = {SGI98431, SGI98432, SGI98433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98434, SGI98434, SGI0, SGI0, SGI98435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21831[] = {SGI98436, SGI98437, SGI98438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98439, SGI0, SGI0, SGI0, SGI98440, SGI0, SGI0, SGI0, SGI0, SGI98441, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21832[] = {SGI98442, SGI0, SGI98443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98444, SGI98445, SGI98445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21833[] = {SGI98446, SGI98447, SGI98448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98449, SGI0, SGI0, SGI0, SGI98450, SGI98450, SGI0, SGI0, SGI0, SGI98451, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21834[] = {SGI98452, SGI98453, SGI98454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98455, SGI98455, SGI0, SGI0, SGI98456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21835[] = {SGI98457, SGI0, SGI98458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98459, SGI98460, SGI98460, SGI0, SGI0, NULL};
const Segment * const SG21836[] = {SGI98461, SGI98462, SGI98463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98464, SGI0, SGI0, SGI0, SGI98465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21837[] = {SGI98466, SGI0, SGI98467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98471, SGI98472, SGI98472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21838[] = {SGI98468, SGI98469, SGI98470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98473, SGI98473, SGI0, SGI0, SGI98474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21839[] = {SGI98475, SGI98476, SGI98477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98478, SGI0, SGI0, SGI0, SGI98479, SGI0, SGI98480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21840[] = {SGI98481, SGI0, SGI98482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98483, SGI98484, SGI98484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21841[] = {SGI98485, SGI0, SGI98486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98487, SGI98488, SGI98488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21842[] = {SGI98489, SGI0, SGI98490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98491, SGI98492, SGI98492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21843[] = {SGI98493, SGI98494, SGI98495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98496, SGI0, SGI0, SGI0, SGI98497, SGI98497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98498, SGI0, NULL};
const Segment * const SG21844[] = {SGI98499, SGI98500, SGI98501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98502, SGI98502, SGI0, SGI0, SGI98503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21845[] = {SGI98504, SGI98505, SGI98506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98507, SGI0, SGI0, SGI0, SGI98508, SGI98508, SGI0, SGI0, SGI0, SGI98509, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21846[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98510, SGI0, SGI98511, SGI0, NULL};
const Segment * const SG21847[] = {SGI98512, SGI0, SGI98513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98514, SGI0, SGI0, SGI0, SGI98515, SGI98515, SGI0, SGI98516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21848[] = {SGI98517, SGI0, SGI98518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98519, SGI0, SGI0, SGI0, SGI98520, SGI98520, SGI0, SGI98521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21849[] = {SGI98522, SGI0, SGI98523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98524, SGI98525, SGI98525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98526, SGI0, NULL};
const Segment * const SG21850[] = {SGI98527, SGI0, SGI98528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98529, SGI98530, SGI98530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98531, SGI0, NULL};
const Segment * const SG21851[] = {SGI0, SGI98532, SGI0, SGI98533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21852[] = {SGI98534, SGI98535, SGI98536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98537, SGI0, SGI0, SGI0, SGI98538, SGI98538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98539, SGI0, SGI0, NULL};
const Segment * const SG21853[] = {SGI98540, SGI98541, SGI98542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98543, SGI0, SGI0, SGI0, SGI98544, SGI98544, SGI0, SGI0, SGI0, SGI98545, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21854[] = {SGI98546, SGI0, SGI98547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98548, SGI0, SGI0, SGI0, SGI98549, SGI98549, SGI0, SGI0, SGI98550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21855[] = {SGI98551, SGI0, SGI98552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98553, SGI98554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21856[] = {SGI98555, SGI98556, SGI98557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98558, SGI0, SGI0, SGI0, SGI98559, SGI98559, SGI0, SGI0, SGI0, SGI98560, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21857[] = {SGI98561, SGI98562, SGI98563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98564, SGI0, SGI0, SGI0, SGI98565, SGI98565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98566, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21858[] = {SGI98567, SGI98568, SGI98569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98573, SGI0, SGI0, SGI0, SGI98574, SGI98574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21859[] = {SGI98570, SGI98571, SGI98572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98575, SGI0, SGI0, SGI0, SGI98576, SGI98576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98577, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21860[] = {SGI98578, SGI0, SGI98579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98580, SGI98581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21861[] = {SGI98582, SGI98583, SGI98584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98585, SGI98585, SGI0, SGI0, SGI98586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21862[] = {SGI98587, SGI98588, SGI98589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98590, SGI0, SGI0, SGI0, SGI98591, SGI98591, SGI0, SGI0, SGI0, SGI98592, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21863[] = {SGI98593, SGI98594, SGI98595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98596, SGI0, SGI0, SGI0, SGI98597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21864[] = {SGI98598, SGI98599, SGI98600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98601, SGI98601, SGI0, SGI0, SGI98602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21865[] = {SGI98603, SGI98604, SGI98605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98606, SGI0, SGI0, SGI0, SGI98607, SGI98607, SGI0, SGI98608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21866[] = {SGI98609, SGI98610, SGI98611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98612, SGI0, SGI0, SGI0, SGI98613, SGI98613, SGI0, SGI98614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21867[] = {SGI98615, SGI98616, SGI98617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98618, SGI0, SGI0, SGI0, SGI98619, SGI98619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21868[] = {SGI98620, SGI0, SGI98621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98622, SGI0, SGI0, SGI0, SGI98623, SGI98623, SGI0, SGI0, SGI98624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21869[] = {SGI98625, SGI0, SGI98626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98627, SGI0, SGI0, SGI0, SGI98628, SGI98628, SGI0, SGI0, SGI98629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21870[] = {SGI98630, SGI98631, SGI98632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98633, SGI0, SGI0, SGI0, SGI98634, SGI98634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98635, SGI0, SGI0, NULL};
const Segment * const SG21871[] = {SGI98636, SGI98637, SGI98638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98639, SGI0, SGI0, SGI0, SGI98640, SGI0, SGI98641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21872[] = {SGI98642, SGI98643, SGI98644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98645, SGI98645, SGI0, SGI0, SGI98646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21873[] = {SGI98647, SGI98648, SGI98649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98650, SGI0, SGI0, SGI0, SGI98651, SGI98651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21874[] = {SGI98652, SGI98653, SGI98654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98655, SGI98656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21875[] = {SGI98657, SGI98658, SGI98659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98660, SGI0, SGI0, SGI0, SGI98661, SGI98661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21876[] = {SGI98662, SGI98663, SGI98664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98665, SGI0, SGI0, SGI0, SGI98666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21877[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98667, SGI0, SGI98668, SGI0, NULL};
const Segment * const SG21878[] = {SGI98669, SGI98670, SGI98671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98672, SGI0, SGI0, SGI0, SGI98673, SGI98673, SGI0, SGI98674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21879[] = {SGI98675, SGI98676, SGI98677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98680, SGI0, SGI0, SGI0, SGI98682, SGI98682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21880[] = {SGI98678, SGI98679, SGI98681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98683, SGI0, SGI0, SGI0, SGI98684, SGI98684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21881[] = {SGI98685, SGI98686, SGI98687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98688, SGI0, SGI0, SGI0, SGI98689, SGI98690, SGI0, SGI0, SGI98691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21882[] = {SGI98692, SGI0, SGI98693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21883[] = {SGI98694, SGI98695, SGI98696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98697, SGI0, SGI0, SGI0, SGI98698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21884[] = {SGI98699, SGI98700, SGI98701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98702, SGI0, SGI0, SGI0, SGI98703, SGI98703, SGI0, SGI98704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21885[] = {SGI98705, SGI98706, SGI98707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98708, SGI0, SGI0, SGI0, SGI98709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21886[] = {SGI98710, SGI98711, SGI98712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98713, SGI0, SGI0, SGI0, SGI98714, SGI98714, SGI0, SGI98715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21887[] = {SGI98716, SGI98717, SGI98718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98719, SGI0, SGI0, SGI0, SGI98720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21888[] = {SGI98721, SGI0, SGI98722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98723, SGI98724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21889[] = {SGI98725, SGI0, SGI98726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98727, SGI98728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21890[] = {SGI98729, SGI0, SGI98730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98731, SGI98732, SGI98732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98733, NULL};
const Segment * const SG21891[] = {SGI98734, SGI0, SGI98735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98736, SGI98737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21892[] = {SGI98738, SGI98739, SGI98740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98741, SGI0, SGI0, SGI0, SGI98742, SGI98742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21893[] = {SGI0, SGI0, SGI98743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98744, SGI98745, SGI0, SGI0, SGI98746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21894[] = {SGI98747, SGI0, SGI98748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98749, SGI98750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21895[] = {SGI98751, SGI0, SGI98752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98753, SGI98754, SGI98754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21896[] = {SGI98755, SGI0, SGI98756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21897[] = {SGI98757, SGI98758, SGI98759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98760, SGI0, SGI0, SGI0, SGI98761, SGI98761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21898[] = {SGI98762, SGI0, SGI98763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98764, SGI98765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21899[] = {SGI98766, SGI0, SGI98767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98768, SGI98769, SGI98769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21900[] = {SGI98770, SGI0, SGI98771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98772, SGI98773, SGI98773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21901[] = {SGI98774, SGI98775, SGI98776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98777, SGI0, SGI0, SGI0, SGI98778, SGI98778, SGI0, SGI0, SGI0, SGI98779, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21902[] = {SGI98780, SGI98781, SGI98782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98783, SGI0, SGI0, SGI0, SGI98784, SGI98784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21903[] = {SGI98785, SGI98786, SGI98787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98788, SGI0, SGI0, SGI0, SGI98789, SGI98789, SGI0, SGI0, SGI0, SGI98790, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21904[] = {SGI98791, SGI0, SGI98792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98793, SGI98794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21905[] = {SGI98795, SGI0, SGI98796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98797, SGI98798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21906[] = {SGI98799, SGI98800, SGI98801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98802, SGI98802, SGI0, SGI0, SGI98803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21907[] = {SGI98804, SGI98805, SGI98806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98807, SGI0, SGI0, SGI0, SGI98808, SGI98808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98809, SGI0, NULL};
const Segment * const SG21908[] = {SGI98810, SGI98811, SGI98812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98813, SGI0, SGI0, SGI0, SGI98814, SGI98814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98815, SGI0, NULL};
const Segment * const SG21909[] = {SGI98816, SGI98817, SGI98818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98819, SGI0, SGI0, SGI0, SGI98820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21910[] = {SGI98821, SGI98822, SGI98823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98824, SGI0, SGI0, SGI0, SGI98825, SGI98825, SGI0, SGI0, SGI0, SGI98826, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21911[] = {SGI98827, SGI98828, SGI98829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98830, SGI0, SGI0, SGI0, SGI98831, SGI0, SGI98832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21912[] = {SGI98833, SGI98834, SGI98835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98836, SGI0, SGI0, SGI0, SGI98837, SGI98837, SGI0, SGI98838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21913[] = {SGI98839, SGI0, SGI98840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98841, SGI98842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21914[] = {SGI98843, SGI98844, SGI98845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98846, SGI0, SGI0, SGI0, SGI98847, SGI98847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21915[] = {SGI98848, SGI0, SGI98849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98850, SGI0, SGI0, SGI0, SGI98851, SGI98851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98852, SGI0, SGI0, NULL};
const Segment * const SG21916[] = {SGI98853, SGI98854, SGI98856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98859, SGI0, SGI0, SGI0, SGI98861, SGI98861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21917[] = {SGI98855, SGI98857, SGI98858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98860, SGI0, SGI0, SGI0, SGI98862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21918[] = {SGI98863, SGI98864, SGI98865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98866, SGI0, SGI0, SGI0, SGI98867, SGI98867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21919[] = {SGI98868, SGI0, SGI98869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98870, SGI98871, SGI98871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21920[] = {SGI98872, SGI0, SGI98873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98874, SGI98875, SGI98875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21921[] = {SGI98876, SGI0, SGI98877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98878, SGI98879, SGI98879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21922[] = {SGI98880, SGI98881, SGI98882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98883, SGI0, SGI0, SGI0, SGI98884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21923[] = {SGI98885, SGI98886, SGI98887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98888, SGI0, SGI0, SGI0, SGI98889, SGI98889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21924[] = {SGI98890, SGI0, SGI98891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98892, SGI98893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21925[] = {SGI0, SGI98894, SGI0, SGI98895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21926[] = {SGI98896, SGI98897, SGI98898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98899, SGI0, SGI0, SGI0, SGI98900, SGI98900, SGI0, SGI0, SGI0, SGI98901, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21927[] = {SGI98902, SGI0, SGI98903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98904, SGI0, SGI0, SGI0, SGI98905, SGI98905, SGI0, SGI0, SGI98906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21928[] = {SGI98907, SGI0, SGI98908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98909, SGI0, SGI0, SGI0, SGI98910, SGI98910, SGI0, SGI0, SGI98911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21929[] = {SGI98912, SGI98913, SGI98914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98915, SGI0, SGI0, SGI0, SGI98916, SGI98916, SGI98917, SGI0, SGI98918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21930[] = {SGI98919, SGI98920, SGI98921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98922, SGI0, SGI0, SGI0, SGI98923, SGI98923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98924, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21931[] = {SGI98925, SGI98926, SGI98927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98928, SGI98928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98929, SGI0, SGI0, NULL};
const Segment * const SG21932[] = {SGI98930, SGI0, SGI98931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98932, SGI98933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21933[] = {SGI98934, SGI98935, SGI98936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98937, SGI0, SGI0, SGI0, SGI98938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21934[] = {SGI98939, SGI98940, SGI98941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98942, SGI0, SGI0, SGI0, SGI98943, SGI98943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21935[] = {SGI98944, SGI98945, SGI98946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98947, SGI0, SGI0, SGI0, SGI98948, SGI98948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21936[] = {SGI98949, SGI98950, SGI98951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98952, SGI0, SGI0, SGI0, SGI98953, SGI98953, SGI0, SGI0, SGI0, SGI98954, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21937[] = {SGI98955, SGI0, SGI98956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21938[] = {SGI98957, SGI98958, SGI98959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98960, SGI0, SGI0, SGI0, SGI98961, SGI98961, SGI0, SGI0, SGI0, SGI98962, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21939[] = {SGI98963, SGI98964, SGI98965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98966, SGI98966, SGI0, SGI0, SGI98967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21940[] = {SGI98968, SGI0, SGI98969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98970, SGI98971, SGI98971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21941[] = {SGI0, SGI98972, SGI98973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98974, SGI0, SGI0, SGI0, SGI98975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21942[] = {SGI0, SGI0, SGI98976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21943[] = {SGI98977, SGI98978, SGI98979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98980, SGI0, SGI0, SGI0, SGI98981, SGI98981, SGI0, SGI98982, SGI0, NULL};
const Segment * const SG21944[] = {SGI98983, SGI0, SGI98984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98985, SGI98986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21945[] = {SGI98987, SGI0, SGI98988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98989, SGI0, SGI0, SGI0, SGI98990, SGI98990, SGI0, SGI0, SGI98991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21946[] = {SGI98992, SGI98993, SGI98994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI98995, SGI0, SGI0, SGI0, SGI98997, SGI98997, SGI99000, SGI0, SGI99001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21947[] = {SGI98996, SGI98998, SGI98999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99002, SGI0, SGI0, SGI0, SGI99003, SGI99003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99004, SGI0, SGI0, NULL};
const Segment * const SG21948[] = {SGI99005, SGI99006, SGI99007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99008, SGI0, SGI0, SGI0, SGI99009, SGI99009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99010, SGI0, SGI0, NULL};
const Segment * const SG21949[] = {SGI99011, SGI0, SGI99013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99016, SGI99019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21950[] = {SGI99012, SGI99014, SGI99015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99017, SGI0, SGI0, SGI0, SGI99022, SGI99022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21951[] = {SGI99018, SGI99020, SGI99021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99023, SGI0, SGI0, SGI0, SGI99024, SGI99024, SGI0, SGI0, SGI0, SGI99025, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21952[] = {SGI99026, SGI0, SGI99027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99028, SGI0, SGI0, SGI0, SGI99029, SGI99029, SGI0, SGI0, SGI99030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21953[] = {SGI99031, SGI99032, SGI99033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99034, SGI0, SGI0, SGI0, SGI99035, SGI99035, SGI0, SGI0, SGI0, SGI99036, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21954[] = {SGI99037, SGI99038, SGI99039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99040, SGI0, SGI0, SGI0, SGI99041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21955[] = {SGI99042, SGI99043, SGI99044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99045, SGI0, SGI0, SGI0, SGI99046, SGI99046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21956[] = {SGI99047, SGI99048, SGI99049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99050, SGI0, SGI0, SGI0, SGI99051, SGI99051, SGI0, SGI99052, SGI0, NULL};
const Segment * const SG21957[] = {SGI99053, SGI99054, SGI99055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99056, SGI0, SGI0, SGI0, SGI99057, SGI99058, SGI0, SGI0, SGI99059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21958[] = {SGI99060, SGI99061, SGI99062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99063, SGI0, SGI0, SGI0, SGI99064, SGI99064, SGI0, SGI0, SGI0, SGI99065, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21959[] = {SGI99066, SGI99067, SGI99068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99069, SGI0, SGI0, SGI0, SGI99070, SGI99070, SGI0, SGI0, SGI0, SGI99071, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21960[] = {SGI99072, SGI99073, SGI99074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99075, SGI0, SGI0, SGI0, SGI99076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21961[] = {SGI99077, SGI99078, SGI99079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99080, SGI0, SGI0, SGI0, SGI99081, SGI99081, SGI0, SGI99082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21962[] = {SGI99083, SGI99084, SGI99085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99086, SGI0, SGI0, SGI0, SGI99087, SGI99087, SGI99088, SGI0, SGI99089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21963[] = {SGI99090, SGI0, SGI99091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99092, SGI99093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21964[] = {SGI99094, SGI0, SGI99095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99096, SGI99097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21965[] = {SGI99098, SGI0, SGI99099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99100, SGI99101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21966[] = {SGI99102, SGI0, SGI99103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99104, SGI99105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21967[] = {SGI99106, SGI99107, SGI99108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99109, SGI0, SGI0, SGI0, SGI99110, SGI99110, SGI0, SGI0, SGI0, SGI99111, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21968[] = {SGI99112, SGI0, SGI99113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99114, SGI0, SGI0, SGI0, SGI99115, SGI99115, SGI0, SGI0, SGI99116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21969[] = {SGI99117, SGI99118, SGI99119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99120, SGI0, SGI0, SGI0, SGI99121, SGI99121, SGI0, SGI0, SGI0, SGI99122, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21970[] = {SGI99123, SGI99124, SGI99125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99126, SGI99126, SGI0, SGI0, SGI0, SGI99127, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21971[] = {SGI99128, SGI99129, SGI99130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99131, SGI0, SGI0, SGI0, SGI99132, SGI0, SGI0, SGI0, SGI0, SGI99133, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21972[] = {SGI99134, SGI0, SGI99135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99136, SGI99137, SGI99137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21973[] = {SGI99138, SGI0, SGI99139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99140, SGI99141, SGI99141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21974[] = {SGI99142, SGI0, SGI99143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99144, SGI99145, SGI99145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21975[] = {SGI99146, SGI99147, SGI99148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99149, SGI0, SGI0, SGI0, SGI99150, SGI99150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21976[] = {SGI99151, SGI99152, SGI99153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99154, SGI0, SGI0, SGI0, SGI99155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21977[] = {SGI99156, SGI99157, SGI99158, SGI99159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99160, SGI0, SGI0, SGI0, SGI99161, SGI99162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21978[] = {SGI99163, SGI99164, SGI99165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99166, SGI0, SGI0, SGI0, SGI99167, SGI99167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21979[] = {SGI99168, SGI99169, SGI99170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99171, SGI0, SGI0, SGI0, SGI99172, SGI99172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21980[] = {SGI99173, SGI99174, SGI99175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99176, SGI99176, SGI0, SGI0, SGI99177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21981[] = {SGI99178, SGI99179, SGI99180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99181, SGI0, SGI0, SGI0, SGI99182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21982[] = {SGI99183, SGI0, SGI99184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99185, SGI0, SGI0, SGI0, SGI99186, SGI99186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99187, SGI0, SGI0, NULL};
const Segment * const SG21983[] = {SGI99188, SGI99189, SGI99190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99191, SGI0, SGI0, SGI0, SGI99192, SGI99193, SGI0, SGI0, SGI99194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21984[] = {SGI99195, SGI0, SGI99196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99197, SGI99198, SGI99198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99199, NULL};
const Segment * const SG21985[] = {SGI99200, SGI99201, SGI99202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99203, SGI0, SGI0, SGI0, SGI99204, SGI99204, SGI0, SGI0, SGI0, SGI99205, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21986[] = {SGI99206, SGI0, SGI99207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99210, SGI99211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21987[] = {SGI99208, SGI0, SGI99209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99212, SGI99213, SGI99213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99214, NULL};
const Segment * const SG21988[] = {SGI99215, SGI0, SGI99216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99217, SGI99218, SGI99218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99219, SGI0, NULL};
const Segment * const SG21989[] = {SGI99220, SGI99221, SGI99222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99223, SGI0, SGI0, SGI0, SGI99224, SGI99224, SGI0, SGI0, SGI0, SGI99225, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21990[] = {SGI99226, SGI0, SGI99227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99228, SGI0, SGI0, SGI0, SGI99229, SGI99229, SGI0, SGI0, SGI99230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21991[] = {SGI99231, SGI99232, SGI99233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99234, SGI99234, SGI0, SGI0, SGI99235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21992[] = {SGI99236, SGI99237, SGI99238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99239, SGI0, SGI0, SGI0, SGI99240, SGI99240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21993[] = {SGI99241, SGI0, SGI99242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99243, SGI99244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21994[] = {SGI99245, SGI99246, SGI99247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99248, SGI99248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21995[] = {SGI99249, SGI99250, SGI99251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99252, SGI0, SGI0, SGI0, SGI99253, SGI99253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21996[] = {SGI99254, SGI99255, SGI99256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99257, SGI99258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21997[] = {SGI99262, SGI0, SGI99263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG21998[] = {SGI99259, SGI99260, SGI99261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99264, SGI0, SGI0, SGI0, SGI99265, SGI99265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99266, SGI0, SGI0, NULL};
const Segment * const SG21999[] = {SGI0, SGI99267, SGI99268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22000[] = {SGI99269, SGI99270, SGI99271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99272, SGI0, SGI0, SGI0, SGI99273, SGI99273, SGI0, SGI0, SGI0, SGI99274, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22001[] = {SGI99275, SGI0, SGI99276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99277, SGI99278, SGI99278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22002[] = {SGI99279, SGI99280, SGI99281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99282, SGI0, SGI0, SGI0, SGI99283, SGI99283, SGI0, SGI0, SGI0, SGI99284, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22003[] = {SGI99285, SGI0, SGI99286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99287, SGI99288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22004[] = {SGI99289, SGI0, SGI99290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99291, SGI99292, SGI99292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22005[] = {SGI99293, SGI0, SGI99294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99295, SGI0, SGI0, SGI0, SGI99296, SGI99296, SGI0, SGI0, SGI99297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22006[] = {SGI99298, SGI99299, SGI99300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99301, SGI0, SGI99302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22007[] = {SGI99303, SGI0, SGI99304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99305, SGI99306, SGI99306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22008[] = {SGI99307, SGI99308, SGI99309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99310, SGI0, SGI0, SGI0, SGI99311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22009[] = {SGI99312, SGI0, SGI99314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22010[] = {SGI99313, SGI99315, SGI99316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99317, SGI0, SGI0, SGI0, SGI99318, SGI99318, SGI0, SGI99319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22011[] = {SGI99320, SGI0, SGI99321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99322, SGI99323, SGI99323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99324, SGI0, NULL};
const Segment * const SG22012[] = {SGI99325, SGI0, SGI99326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99327, SGI0, SGI0, SGI0, SGI99328, SGI99328, SGI0, SGI0, SGI99329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22013[] = {SGI99330, SGI99331, SGI99332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99333, SGI99333, SGI0, SGI0, SGI99334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22014[] = {SGI99335, SGI0, SGI99336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99337, SGI99339, SGI99339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22015[] = {SGI99338, SGI99340, SGI99341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99342, SGI0, SGI0, SGI0, SGI99343, SGI99343, SGI0, SGI0, SGI0, SGI99344, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22016[] = {SGI99345, SGI99346, SGI99347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99348, SGI99348, SGI0, SGI0, SGI99349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22017[] = {SGI99350, SGI99351, SGI99352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99353, SGI0, SGI0, SGI0, SGI99354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22018[] = {SGI0, SGI0, SGI99355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99356, SGI99357, SGI0, SGI0, SGI99358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22019[] = {SGI0, SGI99359, SGI0, SGI99360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22020[] = {SGI0, SGI99361, SGI0, SGI99362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99363, SGI0, SGI0, SGI0, SGI99364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22021[] = {SGI99365, SGI99366, SGI99367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99368, SGI0, SGI0, SGI0, SGI99369, SGI99369, SGI0, SGI0, SGI0, SGI99370, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22022[] = {SGI99371, SGI0, SGI99372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99373, SGI0, SGI0, SGI0, SGI99374, SGI99374, SGI0, SGI0, SGI99375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22023[] = {SGI0, SGI99376, SGI0, SGI99377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22024[] = {SGI99378, SGI0, SGI99379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99380, SGI99381, SGI99381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22025[] = {SGI99382, SGI0, SGI99383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99384, SGI99385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22026[] = {SGI99386, SGI0, SGI99387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99388, SGI99389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99390, SGI0, NULL};
const Segment * const SG22027[] = {SGI99391, SGI0, SGI99392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99393, SGI99394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22028[] = {SGI99395, SGI99396, SGI99397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99398, SGI0, SGI0, SGI0, SGI99399, SGI99399, SGI0, SGI0, SGI0, SGI99400, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22029[] = {SGI99401, SGI0, SGI99402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99403, SGI99404, SGI99404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22030[] = {SGI99405, SGI99406, SGI99407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99408, SGI0, SGI0, SGI0, SGI99409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22031[] = {SGI99410, SGI0, SGI99411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22032[] = {SGI99412, SGI0, SGI99413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99414, SGI99415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22033[] = {SGI99416, SGI99417, SGI99418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99419, SGI99419, SGI0, SGI0, SGI99420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22034[] = {SGI99421, SGI0, SGI99422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99423, SGI99424, SGI99424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99425, NULL};
const Segment * const SG22035[] = {SGI99426, SGI99427, SGI99428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99429, SGI0, SGI0, SGI0, SGI99430, SGI99430, SGI0, SGI99431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22036[] = {SGI99432, SGI0, SGI99433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99434, SGI99435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22037[] = {SGI99436, SGI0, SGI99437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99438, SGI99439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22038[] = {SGI99440, SGI0, SGI99441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99442, SGI99443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22039[] = {SGI99444, SGI99445, SGI99446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99447, SGI0, SGI0, SGI0, SGI99448, SGI99448, SGI99449, SGI0, SGI99450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22040[] = {SGI99451, SGI99452, SGI99453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99454, SGI0, SGI0, SGI0, SGI99455, SGI99455, SGI99456, SGI0, SGI99457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22041[] = {SGI99458, SGI99459, SGI99460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99461, SGI0, SGI0, SGI0, SGI99462, SGI99462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22042[] = {SGI99463, SGI0, SGI99464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99465, SGI99466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22043[] = {SGI99467, SGI99468, SGI99469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99470, SGI0, SGI0, SGI0, SGI99471, SGI99471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22044[] = {SGI99472, SGI99473, SGI99474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99475, SGI99475, SGI0, SGI0, SGI99476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22045[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99477, SGI0, SGI99478, SGI0, NULL};
const Segment * const SG22046[] = {SGI99479, SGI0, SGI99480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99481, SGI0, SGI0, SGI0, SGI99482, SGI99482, SGI0, SGI0, SGI99483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22047[] = {SGI99484, SGI99485, SGI99486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99487, SGI0, SGI0, SGI0, SGI99488, SGI99488, SGI0, SGI0, NULL};
const Segment * const SG22048[] = {SGI99489, SGI99490, SGI99491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99492, SGI0, SGI0, SGI0, SGI99493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22049[] = {SGI99494, SGI99495, SGI99496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99497, SGI0, SGI0, SGI0, SGI99498, SGI99498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99499, SGI0, NULL};
const Segment * const SG22050[] = {SGI99500, SGI99501, SGI99503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99506, SGI0, SGI0, SGI0, SGI99508, SGI99508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99510, SGI0, NULL};
const Segment * const SG22051[] = {SGI99502, SGI99504, SGI99505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99507, SGI0, SGI0, SGI0, SGI99509, SGI99509, SGI99511, SGI0, SGI99512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22052[] = {SGI99513, SGI99514, SGI99515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99516, SGI0, SGI0, SGI0, SGI99517, SGI99517, SGI99518, SGI0, SGI99519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22053[] = {SGI99520, SGI99521, SGI99522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99523, SGI0, SGI0, SGI0, SGI99524, SGI99524, SGI0, SGI0, SGI0, SGI99525, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22054[] = {SGI99526, SGI0, SGI99527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99528, SGI0, SGI0, SGI0, SGI99529, SGI99529, SGI0, SGI0, SGI99530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22055[] = {SGI99531, SGI0, SGI99532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99533, SGI99534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22056[] = {SGI99535, SGI99536, SGI99537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99538, SGI0, SGI99539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22057[] = {SGI99540, SGI99541, SGI99542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99543, SGI0, SGI0, SGI0, SGI99544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22058[] = {SGI99545, SGI0, SGI99546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22059[] = {SGI99547, SGI99548, SGI99549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99550, SGI0, SGI0, SGI0, SGI99552, SGI99552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99554, SGI0, NULL};
const Segment * const SG22060[] = {SGI99551, SGI0, SGI99553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99555, SGI99556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22061[] = {SGI99557, SGI99558, SGI99559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99560, SGI0, SGI0, SGI0, SGI99561, SGI99561, SGI0, SGI0, SGI0, SGI99562, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22062[] = {SGI99563, SGI99564, SGI99565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99566, SGI0, SGI0, SGI0, SGI99567, SGI99567, SGI0, SGI0, SGI0, SGI99568, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22063[] = {SGI99569, SGI99570, SGI99571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99572, SGI99572, SGI0, SGI0, SGI99573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22064[] = {SGI99574, SGI99575, SGI99576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99577, SGI0, SGI0, SGI0, SGI99578, SGI99578, SGI0, SGI0, NULL};
const Segment * const SG22065[] = {SGI99579, SGI99580, SGI99581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99582, SGI0, SGI0, SGI0, SGI99583, SGI99583, SGI0, SGI0, SGI0, SGI99584, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22066[] = {SGI99585, SGI99586, SGI99587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99588, SGI99588, SGI0, SGI0, SGI99589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22067[] = {SGI99590, SGI99591, SGI99592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99593, SGI0, SGI0, SGI0, SGI99594, SGI99594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22068[] = {SGI99595, SGI99596, SGI99597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99598, SGI0, SGI0, SGI0, SGI99599, SGI99599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22069[] = {SGI99600, SGI99601, SGI99602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99603, SGI0, SGI0, SGI0, SGI99604, SGI99604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22070[] = {SGI99605, SGI99606, SGI99607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99608, SGI0, SGI0, SGI0, SGI99609, SGI99609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22071[] = {SGI99610, SGI0, SGI99611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99612, SGI99613, SGI99613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99614, NULL};
const Segment * const SG22072[] = {SGI99615, SGI99616, SGI99617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99618, SGI0, SGI0, SGI0, SGI99619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22073[] = {SGI99620, SGI99621, SGI99622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99623, SGI0, SGI0, SGI0, SGI99624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22074[] = {SGI99625, SGI99626, SGI99627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99628, SGI0, SGI0, SGI0, SGI99629, SGI99629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22075[] = {SGI99630, SGI99631, SGI99632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99633, SGI0, SGI0, SGI0, SGI99634, SGI99634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22076[] = {SGI99635, SGI0, SGI99636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99637, SGI99638, SGI99638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99639, SGI0, NULL};
const Segment * const SG22077[] = {SGI99640, SGI0, SGI99641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99642, SGI99643, SGI99643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99644, SGI0, NULL};
const Segment * const SG22078[] = {SGI99645, SGI0, SGI99646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99647, SGI0, SGI0, SGI0, SGI99648, SGI99648, SGI0, SGI0, SGI99649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22079[] = {SGI99650, SGI99651, SGI99652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99653, SGI0, SGI0, SGI0, SGI99654, SGI99654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22080[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99655, SGI0, SGI99656, SGI0, NULL};
const Segment * const SG22081[] = {SGI99657, SGI0, SGI99658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99659, SGI99660, SGI99660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22082[] = {SGI99661, SGI0, SGI99662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99663, SGI0, SGI0, SGI0, SGI99664, SGI99664, SGI0, SGI0, SGI99665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22083[] = {SGI99666, SGI99667, SGI99668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99669, SGI0, SGI0, SGI0, SGI99670, SGI99670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22084[] = {SGI99671, SGI0, SGI99672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99673, SGI0, SGI0, SGI0, SGI99674, SGI99674, SGI0, SGI0, SGI99675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22085[] = {SGI99676, SGI0, SGI99677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99678, SGI99679, SGI99679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99680, NULL};
const Segment * const SG22086[] = {SGI99681, SGI99682, SGI99683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99684, SGI0, SGI0, SGI0, SGI99685, SGI99685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22087[] = {SGI99686, SGI99687, SGI99688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99689, SGI0, SGI0, SGI0, SGI99690, SGI99690, SGI99691, SGI0, SGI99692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22088[] = {SGI99693, SGI99694, SGI99695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99696, SGI0, SGI0, SGI0, SGI99697, SGI99697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22089[] = {SGI99698, SGI0, SGI99699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99701, SGI99703, SGI99703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99705, NULL};
const Segment * const SG22090[] = {SGI99700, SGI99702, SGI99704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99706, SGI0, SGI0, SGI0, SGI99707, SGI99707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22091[] = {SGI99708, SGI99709, SGI99710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99711, SGI0, SGI0, SGI0, SGI99712, SGI99712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22092[] = {SGI99713, SGI99714, SGI99715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99716, SGI0, SGI0, SGI0, SGI99717, SGI99717, SGI0, SGI99718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22093[] = {SGI99719, SGI0, SGI99720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99721, SGI0, SGI0, SGI0, SGI99722, SGI99722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99723, SGI0, SGI0, NULL};
const Segment * const SG22094[] = {SGI99724, SGI0, SGI99725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99726, SGI0, SGI0, SGI0, SGI99727, SGI99727, SGI0, SGI0, SGI99728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22095[] = {SGI99729, SGI99730, SGI99731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99732, SGI99732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99733, SGI0, NULL};
const Segment * const SG22096[] = {SGI99734, SGI0, SGI99735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99736, SGI99737, SGI99737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99738, SGI0, NULL};
const Segment * const SG22097[] = {SGI99739, SGI0, SGI99740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99741, SGI99742, SGI99742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99743, SGI0, NULL};
const Segment * const SG22098[] = {SGI99744, SGI99745, SGI99746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99747, SGI0, SGI0, SGI0, SGI99748, SGI99749, SGI0, SGI0, SGI99750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22099[] = {SGI99751, SGI99752, SGI99753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99754, SGI99754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99755, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22100[] = {SGI99756, SGI99757, SGI99758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99759, SGI99759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99760, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22101[] = {SGI99761, SGI0, SGI99763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99766, SGI99767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22102[] = {SGI99762, SGI99764, SGI99765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99768, SGI99768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99769, SGI0, SGI0, NULL};
const Segment * const SG22103[] = {SGI99770, SGI0, SGI99771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99772, SGI99773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22104[] = {SGI99774, SGI0, SGI99775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99776, SGI99778, SGI99778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22105[] = {SGI99777, SGI0, SGI99779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99780, SGI0, SGI0, SGI0, SGI99781, SGI99781, SGI0, SGI0, SGI99782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22106[] = {SGI99783, SGI0, SGI99784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99785, SGI99786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22107[] = {SGI99787, SGI99788, SGI99789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99790, SGI0, SGI0, SGI0, SGI99791, SGI99791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99792, SGI0, SGI0, NULL};
const Segment * const SG22108[] = {SGI99793, SGI99794, SGI99795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99796, SGI0, SGI0, SGI0, SGI99797, SGI99797, SGI0, SGI0, SGI0, SGI99798, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22109[] = {SGI99799, SGI99800, SGI99801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99802, SGI0, SGI0, SGI0, SGI99803, SGI99803, SGI0, SGI0, SGI0, SGI99804, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22110[] = {SGI99805, SGI99806, SGI99807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99808, SGI0, SGI0, SGI0, SGI99811, SGI99811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99812, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22111[] = {SGI99809, SGI0, SGI99810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99813, SGI99814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22112[] = {SGI99815, SGI99816, SGI99817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99818, SGI0, SGI0, SGI0, SGI99819, SGI99819, SGI0, SGI99820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22113[] = {SGI99821, SGI99822, SGI99823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99824, SGI0, SGI0, SGI0, SGI99825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22114[] = {SGI99826, SGI99827, SGI99828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99829, SGI0, SGI0, SGI0, SGI99830, SGI99830, SGI0, SGI0, SGI0, SGI99831, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22115[] = {SGI99832, SGI99833, SGI99834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99835, SGI0, SGI0, SGI0, SGI99836, SGI99836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99837, SGI0, NULL};
const Segment * const SG22116[] = {SGI99838, SGI0, SGI99839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99840, SGI99841, SGI99841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22117[] = {SGI99842, SGI0, SGI99843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99844, SGI99845, SGI99845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22118[] = {SGI99846, SGI0, SGI99847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99848, SGI99849, SGI99849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22119[] = {SGI99850, SGI0, SGI99851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99852, SGI0, SGI0, SGI0, SGI99853, SGI99853, SGI0, SGI0, SGI99854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22120[] = {SGI99855, SGI0, SGI99856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99857, SGI0, SGI0, SGI0, SGI99858, SGI99858, SGI0, SGI0, SGI99859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22121[] = {SGI99860, SGI99861, SGI99862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99863, SGI0, SGI0, SGI0, SGI99864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22122[] = {SGI99865, SGI99866, SGI99867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99868, SGI99868, SGI0, SGI0, SGI99869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22123[] = {SGI99870, SGI99871, SGI99872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99873, SGI0, SGI0, SGI0, SGI99874, SGI99874, SGI0, SGI99875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22124[] = {SGI99876, SGI99877, SGI99878, SGI99879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99880, SGI0, SGI0, SGI0, SGI99881, SGI99882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22125[] = {SGI99883, SGI99884, SGI99885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99886, SGI0, SGI0, SGI0, SGI99887, SGI99887, SGI0, SGI99888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22126[] = {SGI99889, SGI99890, SGI99891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99892, SGI0, SGI0, SGI0, SGI99893, SGI99893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22127[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99894, SGI0, SGI99895, SGI0, NULL};
const Segment * const SG22128[] = {SGI99896, SGI0, SGI99897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99898, SGI99899, SGI99899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22129[] = {SGI99900, SGI0, SGI99901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99902, SGI99903, SGI99903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22130[] = {SGI99904, SGI0, SGI99905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99906, SGI99907, SGI99907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22131[] = {SGI99908, SGI0, SGI99909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99910, SGI99911, SGI99911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22132[] = {SGI99912, SGI99913, SGI99914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99915, SGI0, SGI0, SGI0, SGI99916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22133[] = {SGI99917, SGI99918, SGI99919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99920, SGI0, SGI0, SGI0, SGI99921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22134[] = {SGI99922, SGI0, SGI99923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99924, SGI99925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22135[] = {SGI99926, SGI99927, SGI99928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99929, SGI99929, SGI0, SGI0, SGI99930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22136[] = {SGI99931, SGI99932, SGI99933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99934, SGI0, SGI0, SGI0, SGI99935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22137[] = {SGI99936, SGI0, SGI99937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22138[] = {SGI99938, SGI99939, SGI99940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99941, SGI0, SGI0, SGI0, SGI99942, SGI99942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22139[] = {SGI99943, SGI99944, SGI99945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99946, SGI99946, SGI0, SGI0, SGI99947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22140[] = {SGI99948, SGI0, SGI99949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99950, SGI99951, SGI99951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22141[] = {SGI99952, SGI0, SGI99953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99954, SGI99955, SGI99955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22142[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99956, SGI0, SGI99957, SGI0, NULL};
const Segment * const SG22143[] = {SGI99958, SGI99959, SGI99960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99961, SGI0, SGI0, SGI0, SGI99962, SGI99962, SGI99963, SGI0, SGI99964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22144[] = {SGI99965, SGI99967, SGI99968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99970, SGI0, SGI0, SGI0, SGI99972, SGI99972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22145[] = {SGI99966, SGI0, SGI99969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99971, SGI0, SGI0, SGI0, SGI99973, SGI99973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99974, SGI0, SGI0, NULL};
const Segment * const SG22146[] = {SGI99975, SGI99976, SGI99977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99978, SGI99978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22147[] = {SGI99979, SGI99980, SGI99981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99982, SGI0, SGI0, SGI0, SGI99983, SGI99983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22148[] = {SGI99984, SGI99985, SGI99986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99987, SGI0, SGI0, SGI0, SGI99988, SGI99988, SGI0, SGI0, SGI0, SGI99989, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22149[] = {SGI99990, SGI99991, SGI99992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99993, SGI0, SGI0, SGI0, SGI99994, SGI99994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99997, SGI0, SGI0, NULL};
const Segment * const SG22150[] = {SGI99995, SGI99996, SGI99998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI99999, SGI0, SGI0, SGI0, SGI100000, SGI100000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22151[] = {SGI100001, SGI100002, SGI100003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100004, SGI100004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22152[] = {SGI100005, SGI100006, SGI100007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100008, SGI0, SGI0, SGI0, SGI100009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22153[] = {SGI100010, SGI100011, SGI100012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100013, SGI0, SGI0, SGI0, SGI100014, SGI100014, SGI0, SGI0, SGI0, SGI100015, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22154[] = {SGI100016, SGI0, SGI100017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100018, SGI0, SGI0, SGI0, SGI100019, SGI100019, SGI0, SGI0, SGI100020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22155[] = {SGI100021, SGI0, SGI100022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100023, SGI0, SGI0, SGI0, SGI100024, SGI100024, SGI0, SGI0, SGI100025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22156[] = {SGI100026, SGI100027, SGI100028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100029, SGI0, SGI0, SGI0, SGI100030, SGI100030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22157[] = {SGI100031, SGI100032, SGI100033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100034, SGI0, SGI0, SGI0, SGI100035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22158[] = {SGI100036, SGI100037, SGI100038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100039, SGI0, SGI0, SGI0, SGI100040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22159[] = {SGI100041, SGI100042, SGI100043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100044, SGI0, SGI0, SGI0, SGI100045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22160[] = {SGI100046, SGI100047, SGI100048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100049, SGI0, SGI0, SGI0, SGI100050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22161[] = {SGI100051, SGI100052, SGI100053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100054, SGI0, SGI0, SGI0, SGI100055, SGI100055, SGI0, SGI0, SGI0, SGI100056, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22162[] = {SGI100057, SGI0, SGI100058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100059, SGI100060, SGI100060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22163[] = {SGI100061, SGI0, SGI100062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100063, SGI100064, SGI100064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22164[] = {SGI100065, SGI0, SGI100066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100067, SGI100068, SGI100068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22165[] = {SGI100069, SGI100070, SGI100071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100072, SGI0, SGI0, SGI0, SGI100073, SGI100073, SGI0, SGI0, SGI0, SGI100074, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22166[] = {SGI100075, SGI0, SGI100076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100077, SGI100078, SGI100078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100079, SGI0, NULL};
const Segment * const SG22167[] = {SGI100080, SGI100081, SGI100082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100083, SGI0, SGI0, SGI0, SGI100084, SGI100085, SGI0, SGI0, SGI100086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22168[] = {SGI100087, SGI100088, SGI100089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100090, SGI0, SGI0, SGI0, SGI100091, SGI100091, SGI0, SGI0, SGI0, SGI100092, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22169[] = {SGI100093, SGI100094, SGI100095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100096, SGI0, SGI0, SGI0, SGI100097, SGI100097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100098, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22170[] = {SGI100099, SGI100100, SGI100101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100102, SGI0, SGI0, SGI0, SGI100103, SGI100103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100104, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22171[] = {SGI100105, SGI0, SGI100106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100107, SGI100108, SGI100108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22172[] = {SGI100109, SGI100110, SGI100111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100115, SGI0, SGI0, SGI0, SGI100116, SGI100116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22173[] = {SGI100112, SGI100113, SGI100114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100117, SGI0, SGI0, SGI0, SGI100118, SGI0, SGI0, SGI0, SGI0, SGI100119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22174[] = {SGI100120, SGI100121, SGI100122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100123, SGI0, SGI0, SGI0, SGI100124, SGI0, SGI0, SGI0, SGI0, SGI100125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22175[] = {SGI100126, SGI0, SGI100127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100128, SGI100129, SGI100129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22176[] = {SGI100130, SGI100131, SGI100132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100133, SGI0, SGI0, SGI0, SGI100134, SGI100134, SGI0, SGI100135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22177[] = {SGI100136, SGI100137, SGI100138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100139, SGI0, SGI0, SGI0, SGI100140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22178[] = {SGI100141, SGI100142, SGI100143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100144, SGI100144, SGI0, SGI0, SGI100145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22179[] = {SGI100146, SGI0, SGI100147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100148, SGI100149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22180[] = {SGI100150, SGI100151, SGI100152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100153, SGI0, SGI0, SGI0, SGI100154, SGI100154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22181[] = {SGI100155, SGI100156, SGI100157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100158, SGI0, SGI0, SGI0, SGI100159, SGI100159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22182[] = {SGI100160, SGI100161, SGI100162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100163, SGI100163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100164, SGI0, NULL};
const Segment * const SG22183[] = {SGI100165, SGI100166, SGI100167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100168, SGI0, SGI0, SGI0, SGI100169, SGI100169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22184[] = {SGI100170, SGI100171, SGI100172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100173, SGI0, SGI0, SGI0, SGI100174, SGI100174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22185[] = {SGI100175, SGI100176, SGI100177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100178, SGI0, SGI0, SGI0, SGI100179, SGI100180, SGI0, SGI0, SGI100181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22186[] = {SGI100182, SGI0, SGI100183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100184, SGI100185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22187[] = {SGI100186, SGI100187, SGI100188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100189, SGI0, SGI0, SGI0, SGI100190, SGI100190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22188[] = {SGI100191, SGI100192, SGI100193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100194, SGI0, SGI0, SGI0, SGI100195, SGI100195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22189[] = {SGI100196, SGI100197, SGI100198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100199, SGI0, SGI0, SGI0, SGI100200, SGI100200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22190[] = {SGI100201, SGI100202, SGI100203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100204, SGI0, SGI0, SGI0, SGI100205, SGI100205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22191[] = {SGI100206, SGI100207, SGI100208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100209, SGI0, SGI0, SGI0, SGI100210, SGI100210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22192[] = {SGI100211, SGI100212, SGI100213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100214, SGI0, SGI0, SGI0, SGI100215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22193[] = {SGI100216, SGI100217, SGI100218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100219, SGI0, SGI0, SGI0, SGI100220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22194[] = {SGI100221, SGI100222, SGI100223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100224, SGI0, SGI0, SGI0, SGI100225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22195[] = {SGI100226, SGI100227, SGI100228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100229, SGI0, SGI0, SGI0, SGI100230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22196[] = {SGI100231, SGI100232, SGI100233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100234, SGI0, SGI0, SGI0, SGI100235, SGI100235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100236, SGI0, SGI0, NULL};
const Segment * const SG22197[] = {SGI100237, SGI100238, SGI100239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100240, SGI0, SGI0, SGI0, SGI100241, SGI100241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22198[] = {SGI100242, SGI0, SGI100243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100247, SGI0, SGI0, SGI0, SGI100249, SGI100249, SGI0, SGI0, SGI100251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22199[] = {SGI100244, SGI100245, SGI100246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100248, SGI0, SGI0, SGI0, SGI100250, SGI100250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22200[] = {SGI100253, SGI0, SGI100256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100260, SGI100261, SGI100261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100263, SGI0, NULL};
const Segment * const SG22201[] = {SGI100252, SGI100254, SGI100255, SGI0, SGI0, SGI0, SGI100257, SGI100258, SGI0, SGI100259, SGI0, SGI0, SGI0, SGI100262, SGI100262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22202[] = {SGI100264, SGI0, SGI100265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100266, SGI100267, SGI100267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100268, SGI0, NULL};
const Segment * const SG22203[] = {SGI100269, SGI100270, SGI100271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100272, SGI0, SGI0, SGI0, SGI100273, SGI100273, SGI100274, SGI0, SGI100275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22204[] = {SGI100276, SGI100277, SGI100278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100279, SGI0, SGI0, SGI0, SGI100280, SGI100280, SGI100281, SGI0, SGI100282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22205[] = {SGI100283, SGI100284, SGI100285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100286, SGI0, SGI0, SGI0, SGI100287, SGI100287, SGI0, SGI0, SGI100288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22206[] = {SGI100289, SGI100290, SGI100291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100292, SGI0, SGI0, SGI0, SGI100293, SGI100293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100294, SGI0, NULL};
const Segment * const SG22207[] = {SGI100295, SGI0, SGI100296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100297, SGI100298, SGI100298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100299, SGI0, NULL};
const Segment * const SG22208[] = {SGI100300, SGI100301, SGI100302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100303, SGI0, SGI0, SGI0, SGI100304, SGI100304, SGI0, SGI0, SGI0, SGI100305, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22209[] = {SGI100306, SGI100307, SGI100308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100309, SGI0, SGI0, SGI0, SGI100310, SGI100310, SGI0, SGI0, SGI0, SGI100311, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22210[] = {SGI100312, SGI100313, SGI100314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100315, SGI0, SGI0, SGI0, SGI100316, SGI100316, SGI0, SGI0, SGI100317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22211[] = {SGI100318, SGI0, SGI100319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100320, SGI0, SGI0, SGI0, SGI100321, SGI100321, SGI0, SGI0, SGI100322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22212[] = {SGI100323, SGI100324, SGI100325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100326, SGI100326, SGI0, SGI0, SGI100327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22213[] = {SGI100328, SGI100329, SGI100330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100331, SGI0, SGI0, SGI0, SGI100332, SGI100332, SGI0, SGI0, SGI0, SGI100333, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22214[] = {SGI100334, SGI100335, SGI100336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100337, SGI0, SGI0, SGI0, SGI100338, SGI100338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22215[] = {SGI100339, SGI100340, SGI100341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100342, SGI0, SGI0, SGI0, SGI100343, SGI100343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22216[] = {SGI100344, SGI0, SGI100345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100346, SGI100347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22217[] = {SGI100348, SGI0, SGI100349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100350, SGI100351, SGI100351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100352, NULL};
const Segment * const SG22218[] = {SGI100353, SGI100354, SGI100355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100356, SGI0, SGI0, SGI0, SGI100357, SGI100358, SGI0, SGI0, SGI100359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22219[] = {SGI100360, SGI100361, SGI100362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100363, SGI0, SGI0, SGI0, SGI100364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22220[] = {SGI100365, SGI100366, SGI100367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100368, SGI0, SGI0, SGI0, SGI100369, SGI100369, SGI0, SGI100370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22221[] = {SGI100371, SGI100372, SGI100373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100374, SGI100374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100375, SGI0, NULL};
const Segment * const SG22222[] = {SGI100376, SGI100377, SGI100378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100379, SGI100379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100380, SGI0, NULL};
const Segment * const SG22223[] = {SGI100381, SGI100382, SGI100383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100384, SGI0, SGI0, SGI0, SGI100385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22224[] = {SGI100386, SGI0, SGI100387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100388, SGI100389, SGI100389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22225[] = {SGI100390, SGI0, SGI100391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100392, SGI0, SGI0, SGI0, SGI100393, SGI100393, SGI0, SGI0, SGI100394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22226[] = {SGI100395, SGI100396, SGI100397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100398, SGI0, SGI0, SGI0, SGI100399, SGI100399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100400, SGI0, SGI0, NULL};
const Segment * const SG22227[] = {SGI0, SGI100401, SGI0, SGI100402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22228[] = {SGI100403, SGI100404, SGI100405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100406, SGI0, SGI0, SGI0, SGI100407, SGI100407, SGI0, SGI0, SGI0, SGI100408, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22229[] = {SGI100409, SGI100410, SGI100411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100412, SGI0, SGI0, SGI0, SGI100413, SGI100413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100414, SGI0, NULL};
const Segment * const SG22230[] = {SGI100415, SGI100416, SGI100417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100418, SGI0, SGI0, SGI0, SGI100419, SGI100419, SGI0, SGI0, SGI0, SGI100420, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22231[] = {SGI100421, SGI100422, SGI100423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100424, SGI0, SGI0, SGI0, SGI100425, SGI100425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22232[] = {SGI100426, SGI100427, SGI100428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100430, SGI0, SGI0, SGI0, SGI100433, SGI100433, SGI0, SGI0, SGI100434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22233[] = {SGI100429, SGI100431, SGI100432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100435, SGI100435, SGI0, SGI0, SGI100436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22234[] = {SGI100437, SGI0, SGI100438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100439, SGI100440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22235[] = {SGI100441, SGI100442, SGI100443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100444, SGI0, SGI0, SGI0, SGI100446, SGI100446, SGI0, SGI0, SGI0, SGI100448, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22236[] = {SGI100445, SGI0, SGI100447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100449, SGI100450, SGI100450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100451, SGI0, NULL};
const Segment * const SG22237[] = {SGI100452, SGI0, SGI100453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100454, SGI0, SGI0, SGI0, SGI100456, SGI100456, SGI0, SGI0, SGI100458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22238[] = {SGI100455, SGI0, SGI100457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100459, SGI100460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22239[] = {SGI100461, SGI0, SGI100462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100463, SGI100464, SGI100464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100465, NULL};
const Segment * const SG22240[] = {SGI100466, SGI0, SGI100467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100468, SGI0, SGI0, SGI0, SGI100469, SGI100469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100470, SGI0, SGI0, NULL};
const Segment * const SG22241[] = {SGI100471, SGI100472, SGI100473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100474, SGI0, SGI0, SGI0, SGI100475, SGI100475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22242[] = {SGI100476, SGI100477, SGI100478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100479, SGI0, SGI0, SGI0, SGI100480, SGI100480, SGI0, SGI100481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22243[] = {SGI100482, SGI0, SGI100483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100484, SGI100485, SGI100485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100486, NULL};
const Segment * const SG22244[] = {SGI100487, SGI0, SGI100488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100489, SGI100490, SGI100490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22245[] = {SGI100491, SGI0, SGI100492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100493, SGI100494, SGI100494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100497, NULL};
const Segment * const SG22246[] = {SGI100495, SGI100496, SGI100498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100499, SGI0, SGI0, SGI0, SGI100500, SGI100500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22247[] = {SGI100501, SGI100502, SGI100503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100504, SGI0, SGI0, SGI0, SGI100505, SGI100506, SGI0, SGI0, SGI100507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22248[] = {SGI100508, SGI100509, SGI100510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100511, SGI100511, SGI0, SGI0, SGI100512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22249[] = {SGI100513, SGI0, SGI100514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100515, SGI100516, SGI100516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22250[] = {SGI100517, SGI100518, SGI100519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100520, SGI0, SGI0, SGI0, SGI100521, SGI100521, SGI0, SGI0, SGI0, SGI100522, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22251[] = {SGI100523, SGI100524, SGI100525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100526, SGI0, SGI0, SGI0, SGI100527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22252[] = {SGI100528, SGI100529, SGI100530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100531, SGI0, SGI0, SGI0, SGI100532, SGI100532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22253[] = {SGI100533, SGI100534, SGI100535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100536, SGI0, SGI0, SGI0, SGI100537, SGI100537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22254[] = {SGI100538, SGI100539, SGI100540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100541, SGI0, SGI0, SGI0, SGI100542, SGI100542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22255[] = {SGI100543, SGI100544, SGI100545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100546, SGI100546, SGI0, SGI0, SGI100547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22256[] = {SGI100548, SGI0, SGI100549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100550, SGI100551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22257[] = {SGI100552, SGI100553, SGI100554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100555, SGI0, SGI0, SGI0, SGI100556, SGI100556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22258[] = {SGI100557, SGI100558, SGI100559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100560, SGI0, SGI0, SGI0, SGI100561, SGI100561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22259[] = {SGI100562, SGI100563, SGI100564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100565, SGI0, SGI0, SGI0, SGI100566, SGI100566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22260[] = {SGI100567, SGI100568, SGI100569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100570, SGI0, SGI0, SGI0, SGI100571, SGI0, SGI0, SGI0, SGI0, SGI100572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22261[] = {SGI100573, SGI0, SGI100574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100575, SGI0, SGI0, SGI0, SGI100576, SGI100576, SGI0, SGI0, SGI100577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22262[] = {SGI100578, SGI100579, SGI100580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100581, SGI0, SGI0, SGI0, SGI100582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22263[] = {SGI100583, SGI100584, SGI100585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100586, SGI0, SGI0, SGI0, SGI100587, SGI100587, SGI100588, SGI0, SGI100589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22264[] = {SGI100590, SGI0, SGI100591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100592, SGI100593, SGI100593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100594, SGI0, NULL};
const Segment * const SG22265[] = {SGI100595, SGI0, SGI100596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100597, SGI100598, SGI100598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100599, SGI0, NULL};
const Segment * const SG22266[] = {SGI100600, SGI0, SGI100601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100602, SGI100603, SGI100603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100604, SGI0, NULL};
const Segment * const SG22267[] = {SGI100605, SGI0, SGI100606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100607, SGI100608, SGI100608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100609, SGI0, NULL};
const Segment * const SG22268[] = {SGI100610, SGI0, SGI100611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100612, SGI100613, SGI100613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22269[] = {SGI100614, SGI100615, SGI100616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100617, SGI0, SGI0, SGI0, SGI100618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22270[] = {SGI100619, SGI100620, SGI100621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100622, SGI0, SGI0, SGI0, SGI100623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22271[] = {SGI100624, SGI100625, SGI100626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100627, SGI0, SGI0, SGI0, SGI100628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22272[] = {SGI100629, SGI0, SGI100630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100631, SGI100632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22273[] = {SGI100633, SGI100634, SGI100635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100636, SGI0, SGI0, SGI0, SGI100637, SGI100637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22274[] = {SGI100638, SGI100639, SGI100640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100641, SGI0, SGI0, SGI0, SGI100642, SGI100642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22275[] = {SGI100643, SGI100644, SGI100645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100646, SGI0, SGI0, SGI0, SGI100647, SGI100647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22276[] = {SGI100648, SGI100649, SGI100650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100651, SGI0, SGI0, SGI0, SGI100652, SGI100652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100653, SGI0, NULL};
const Segment * const SG22277[] = {SGI100654, SGI100655, SGI100656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100657, SGI0, SGI0, SGI0, SGI100660, SGI100660, SGI0, SGI0, SGI0, SGI100661, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22278[] = {SGI100658, SGI0, SGI100659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100662, SGI100663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22279[] = {SGI100664, SGI0, SGI100665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100666, SGI100667, SGI100667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100668, NULL};
const Segment * const SG22280[] = {SGI100669, SGI0, SGI100670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100671, SGI100672, SGI100672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100673, NULL};
const Segment * const SG22281[] = {SGI100674, SGI100675, SGI100676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100677, SGI0, SGI0, SGI0, SGI100678, SGI100678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22282[] = {SGI100679, SGI100680, SGI100681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100682, SGI0, SGI0, SGI0, SGI100686, SGI100686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22283[] = {SGI100683, SGI100684, SGI100685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100687, SGI100687, SGI0, SGI0, SGI100688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22284[] = {SGI100689, SGI100690, SGI100691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100692, SGI0, SGI0, SGI0, SGI100693, SGI100693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22285[] = {SGI100694, SGI0, SGI100695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100696, SGI100697, SGI100697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100698, SGI0, NULL};
const Segment * const SG22286[] = {SGI100699, SGI100700, SGI100701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100702, SGI100702, SGI0, SGI0, SGI100703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22287[] = {SGI100704, SGI100705, SGI100706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100707, SGI100707, SGI0, SGI0, SGI100708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22288[] = {SGI100709, SGI100710, SGI100711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100712, SGI100712, SGI0, SGI0, SGI100713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22289[] = {SGI100714, SGI0, SGI100715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100716, SGI100717, SGI100717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22290[] = {SGI100718, SGI100719, SGI100720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100721, SGI0, SGI0, SGI0, SGI100722, SGI100722, SGI0, SGI0, SGI100723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22291[] = {SGI100724, SGI100725, SGI100726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100727, SGI0, SGI0, SGI0, SGI100728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22292[] = {SGI100729, SGI100730, SGI100731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100732, SGI0, SGI0, SGI0, SGI100733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22293[] = {SGI100734, SGI100735, SGI100736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100737, SGI0, SGI0, SGI0, SGI100738, SGI100738, SGI100739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22294[] = {SGI100740, SGI100741, SGI100742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100743, SGI0, SGI0, SGI0, SGI100744, SGI100744, SGI100745, SGI0, SGI100746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22295[] = {SGI100747, SGI100748, SGI100749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100750, SGI0, SGI0, SGI0, SGI100751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22296[] = {SGI100752, SGI100753, SGI100754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100755, SGI0, SGI0, SGI0, SGI100756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22297[] = {SGI100757, SGI0, SGI100758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100759, SGI100760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22298[] = {SGI100761, SGI0, SGI100762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100763, SGI100764, SGI100764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22299[] = {SGI100768, SGI100769, SGI100770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100771, SGI0, SGI0, SGI0, SGI100772, SGI100772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22300[] = {SGI100773, SGI0, SGI100774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100775, SGI0, SGI0, SGI0, SGI100776, SGI100776, SGI0, SGI0, SGI100777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22301[] = {SGI100778, SGI100779, SGI100780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100781, SGI0, SGI0, SGI0, SGI100782, SGI100782, SGI0, SGI0, SGI0, SGI100783, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22302[] = {SGI100784, SGI100785, SGI100786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100787, SGI0, SGI0, SGI0, SGI100788, SGI100788, SGI0, SGI0, SGI0, SGI100789, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22303[] = {SGI100790, SGI0, SGI100791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100792, SGI0, SGI0, SGI0, SGI100793, SGI100793, SGI0, SGI0, SGI100794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22304[] = {SGI100795, SGI100796, SGI100797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100798, SGI100798, SGI0, SGI0, SGI100799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22305[] = {SGI100800, SGI100801, SGI100802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100803, SGI0, SGI0, SGI0, SGI100804, SGI100804, SGI0, SGI0, SGI0, SGI100805, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22306[] = {SGI100765, SGI100766, SGI100767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100806, SGI100806, SGI0, SGI0, SGI100807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22307[] = {SGI100808, SGI100809, SGI100810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100811, SGI100811, SGI0, SGI0, SGI100812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22308[] = {SGI100813, SGI0, SGI100814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100815, SGI100816, SGI100816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22309[] = {SGI100817, SGI0, SGI100818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100819, SGI100820, SGI100820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22310[] = {SGI100821, SGI0, SGI100822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100823, SGI100824, SGI100824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100825, SGI0, NULL};
const Segment * const SG22311[] = {SGI100826, SGI0, SGI100827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100828, SGI100829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22312[] = {SGI100830, SGI100831, SGI100832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100833, SGI100833, SGI0, SGI0, SGI100834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22313[] = {SGI100835, SGI0, SGI100836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100837, SGI100838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22314[] = {SGI0, SGI100839, SGI0, SGI100840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22315[] = {SGI100841, SGI100842, SGI100843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100844, SGI0, SGI0, SGI0, SGI100845, SGI100846, SGI0, SGI0, SGI100847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22316[] = {SGI100848, SGI0, SGI100849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100850, SGI100851, SGI100851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22317[] = {SGI100852, SGI100853, SGI100854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100855, SGI0, SGI0, SGI0, SGI100856, SGI100856, SGI0, SGI100857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22318[] = {SGI100858, SGI100859, SGI100860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100861, SGI0, SGI0, SGI0, SGI100862, SGI100862, SGI0, SGI100863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22319[] = {SGI100864, SGI100865, SGI100867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100870, SGI100870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100873, SGI0, NULL};
const Segment * const SG22320[] = {SGI100866, SGI100868, SGI100869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100871, SGI0, SGI0, SGI0, SGI100872, SGI100872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22321[] = {SGI100874, SGI100875, SGI100876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100877, SGI0, SGI0, SGI0, SGI100878, SGI100878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22322[] = {SGI100879, SGI100880, SGI100881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100882, SGI100882, SGI0, SGI0, SGI100883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22323[] = {SGI100884, SGI0, SGI100885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100886, SGI100887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22324[] = {SGI100888, SGI100889, SGI100890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100891, SGI0, SGI0, SGI0, SGI100892, SGI100892, SGI0, SGI0, SGI0, SGI100893, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22325[] = {SGI100894, SGI100895, SGI100896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100897, SGI0, SGI0, SGI0, SGI100898, SGI100898, SGI0, SGI0, SGI0, SGI100899, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22326[] = {SGI100900, SGI0, SGI100901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100902, SGI0, SGI0, SGI0, SGI100903, SGI100903, SGI0, SGI0, SGI100904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22327[] = {SGI100905, SGI100906, SGI100907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100908, SGI0, SGI0, SGI0, SGI100909, SGI100909, SGI0, SGI0, SGI0, SGI100910, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22328[] = {SGI100911, SGI0, SGI100912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100913, SGI0, SGI0, SGI0, SGI100914, SGI100914, SGI0, SGI0, SGI100915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22329[] = {SGI100916, SGI100917, SGI100918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100919, SGI0, SGI0, SGI0, SGI100920, SGI100920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22330[] = {SGI100921, SGI100922, SGI100923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100926, SGI0, SGI0, SGI0, SGI100927, SGI100927, SGI0, SGI100930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22331[] = {SGI100924, SGI0, SGI100925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100928, SGI100929, SGI100929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22332[] = {SGI100931, SGI100932, SGI100933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100934, SGI0, SGI0, SGI0, SGI100935, SGI100935, SGI0, SGI0, SGI0, SGI100936, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22333[] = {SGI100937, SGI0, SGI100938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100939, SGI100940, SGI100940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22334[] = {SGI100941, SGI0, SGI100942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100943, SGI0, SGI0, SGI0, SGI100944, SGI100944, SGI0, SGI0, SGI100945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22335[] = {SGI100946, SGI100947, SGI100948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100949, SGI0, SGI0, SGI0, SGI100950, SGI100950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100951, SGI0, NULL};
const Segment * const SG22336[] = {SGI100952, SGI100953, SGI100954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100955, SGI0, SGI0, SGI0, SGI100956, SGI100957, SGI0, SGI0, SGI100958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22337[] = {SGI100959, SGI100960, SGI100961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100962, SGI0, SGI0, SGI0, SGI100963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22338[] = {SGI100964, SGI0, SGI100965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100966, SGI100967, SGI100967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22339[] = {SGI100968, SGI0, SGI100969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100970, SGI100971, SGI100971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22340[] = {SGI100972, SGI100973, SGI100974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100975, SGI0, SGI0, SGI0, SGI100976, SGI100976, SGI100977, SGI0, SGI100978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22341[] = {SGI100979, SGI100980, SGI100981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100982, SGI0, SGI0, SGI0, SGI100983, SGI100983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22342[] = {SGI100984, SGI100985, SGI100986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100987, SGI0, SGI0, SGI0, SGI100988, SGI100988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22343[] = {SGI100989, SGI0, SGI100990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100994, SGI100995, SGI100995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100996, NULL};
const Segment * const SG22344[] = {SGI100991, SGI100992, SGI100993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI100997, SGI0, SGI0, SGI0, SGI100998, SGI100998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22345[] = {SGI100999, SGI101000, SGI101001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101002, SGI0, SGI0, SGI0, SGI101003, SGI101003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22346[] = {SGI101004, SGI101005, SGI101006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101007, SGI0, SGI0, SGI0, SGI101008, SGI101008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22347[] = {SGI101009, SGI101010, SGI101011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101012, SGI0, SGI0, SGI0, SGI101013, SGI101013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22348[] = {SGI101014, SGI101015, SGI101016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101017, SGI0, SGI0, SGI0, SGI101018, SGI101018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22349[] = {SGI101019, SGI101020, SGI101021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101022, SGI101022, SGI0, SGI0, SGI101023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22350[] = {SGI101024, SGI101025, SGI101026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101027, SGI0, SGI0, SGI0, SGI101028, SGI101028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22351[] = {SGI101029, SGI0, SGI101030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101031, SGI101032, SGI101032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101033, NULL};
const Segment * const SG22352[] = {SGI101034, SGI101035, SGI101036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101037, SGI0, SGI0, SGI0, SGI101038, SGI101038, SGI0, SGI0, SGI0, SGI101039, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22353[] = {SGI101040, SGI101041, SGI101042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101043, SGI101043, SGI0, SGI0, SGI101044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22354[] = {SGI101045, SGI101046, SGI101047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101048, SGI0, SGI0, SGI0, SGI101051, SGI101051, SGI0, SGI101052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22355[] = {SGI101049, SGI0, SGI101050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101053, SGI101054, SGI101054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22356[] = {SGI101055, SGI101056, SGI101057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101058, SGI0, SGI0, SGI0, SGI101059, SGI101059, SGI0, SGI101060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22357[] = {SGI101061, SGI101062, SGI101063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101064, SGI101064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101065, SGI0, NULL};
const Segment * const SG22358[] = {SGI101066, SGI0, SGI101067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101068, SGI101069, SGI101069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22359[] = {SGI101070, SGI101071, SGI101072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101073, SGI0, SGI0, SGI0, SGI101074, SGI101074, SGI0, SGI0, SGI0, SGI101075, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22360[] = {SGI101076, SGI101077, SGI101078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101079, SGI0, SGI0, SGI0, SGI101080, SGI101080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22361[] = {SGI101081, SGI101082, SGI101083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101084, SGI0, SGI0, SGI0, SGI101085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22362[] = {SGI101086, SGI0, SGI101087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101088, SGI101089, SGI101089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22363[] = {SGI101090, SGI0, SGI101091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101092, SGI101093, SGI101093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22364[] = {SGI101094, SGI0, SGI101095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101096, SGI101097, SGI101097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22365[] = {SGI101098, SGI0, SGI101099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101100, SGI101101, SGI101101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22366[] = {SGI101104, SGI101105, SGI101106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101107, SGI0, SGI0, SGI0, SGI101108, SGI101108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101109, SGI0, SGI0, NULL};
const Segment * const SG22367[] = {SGI101110, SGI101111, SGI101112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101113, SGI0, SGI0, SGI0, SGI101114, SGI101114, SGI0, SGI0, SGI0, SGI101115, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22368[] = {SGI101116, SGI101117, SGI101118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101119, SGI0, SGI0, SGI0, SGI101120, SGI101120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22369[] = {SGI101102, SGI0, SGI101103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101121, SGI101122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22370[] = {SGI101123, SGI0, SGI101124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101125, SGI0, SGI0, SGI0, SGI101126, SGI101126, SGI0, SGI0, SGI101127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22371[] = {SGI101128, SGI0, SGI101129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101130, SGI0, SGI0, SGI0, SGI101131, SGI101131, SGI0, SGI0, SGI101132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22372[] = {SGI101133, SGI101134, SGI101135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101136, SGI0, SGI0, SGI0, SGI101137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22373[] = {SGI101138, SGI101139, SGI101140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101141, SGI0, SGI0, SGI0, SGI101142, SGI101142, SGI0, SGI0, SGI0, SGI101143, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22374[] = {SGI101144, SGI101145, SGI101146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101147, SGI0, SGI0, SGI0, SGI101148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22375[] = {SGI101149, SGI101150, SGI101151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101152, SGI0, SGI0, SGI0, SGI101153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22376[] = {SGI101154, SGI101155, SGI101156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101157, SGI0, SGI0, SGI0, SGI101158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22377[] = {SGI101159, SGI0, SGI101160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101161, SGI101162, SGI101162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101163, NULL};
const Segment * const SG22378[] = {SGI101164, SGI101165, SGI101166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101167, SGI0, SGI0, SGI0, SGI101168, SGI101168, SGI0, SGI0, SGI0, SGI101169, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22379[] = {SGI101170, SGI101171, SGI101172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101173, SGI0, SGI0, SGI0, SGI101174, SGI101174, SGI0, SGI0, SGI0, SGI101175, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22380[] = {SGI101176, SGI101177, SGI101178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101179, SGI101179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101180, SGI0, SGI0, NULL};
const Segment * const SG22381[] = {SGI101181, SGI101182, SGI101183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101184, SGI0, SGI0, SGI0, SGI101185, SGI101185, SGI0, SGI0, SGI0, SGI101186, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22382[] = {SGI101187, SGI101188, SGI101189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101190, SGI101190, SGI0, SGI0, SGI101191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22383[] = {SGI101192, SGI101193, SGI101194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101195, SGI0, SGI0, SGI0, SGI101196, SGI101196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22384[] = {SGI101197, SGI101198, SGI101199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101203, SGI101203, SGI0, SGI0, SGI101204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22385[] = {SGI101200, SGI101201, SGI101202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101205, SGI0, SGI0, SGI0, SGI101206, SGI101206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22386[] = {SGI101207, SGI101208, SGI101209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101210, SGI0, SGI0, SGI0, SGI101211, SGI101212, SGI0, SGI0, SGI101213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22387[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101214, SGI0, SGI0, SGI101215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22388[] = {SGI101216, SGI101217, SGI101218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101219, SGI101219, SGI0, SGI0, SGI101220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22389[] = {SGI101221, SGI0, SGI101223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101226, SGI101227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22390[] = {SGI101222, SGI101224, SGI101225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101228, SGI101228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101229, SGI0, NULL};
const Segment * const SG22391[] = {SGI101230, SGI101231, SGI101233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101235, SGI101235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101238, SGI0, NULL};
const Segment * const SG22392[] = {SGI101232, SGI0, SGI101234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101236, SGI101237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22393[] = {SGI101239, SGI101240, SGI101241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101242, SGI101242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101243, SGI0, NULL};
const Segment * const SG22394[] = {SGI101244, SGI101245, SGI101246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101247, SGI0, SGI0, SGI0, SGI101248, SGI101248, SGI0, SGI0, SGI0, SGI101249, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22395[] = {SGI101250, SGI101251, SGI101252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101253, SGI0, SGI0, SGI0, SGI101254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22396[] = {SGI101255, SGI101256, SGI101257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101258, SGI0, SGI0, SGI0, SGI101259, SGI101259, SGI0, SGI0, SGI101260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22397[] = {SGI101261, SGI101262, SGI101263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101264, SGI0, SGI0, SGI0, SGI101265, SGI101265, SGI0, SGI0, SGI0, SGI0, SGI101266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22398[] = {SGI101267, SGI0, SGI101268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101269, SGI101270, SGI101270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101271, NULL};
const Segment * const SG22399[] = {SGI101272, SGI101273, SGI101274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101275, SGI0, SGI0, SGI0, SGI101276, SGI101276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22400[] = {SGI101277, SGI101278, SGI101279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101280, SGI101280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22401[] = {SGI101281, SGI101282, SGI101283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101284, SGI0, SGI0, SGI0, SGI101285, SGI101285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22402[] = {SGI101286, SGI101287, SGI101288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101289, SGI0, SGI0, SGI0, SGI101290, SGI101290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101291, SGI0, SGI0, NULL};
const Segment * const SG22403[] = {SGI0, SGI101292, SGI0, SGI101293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22404[] = {SGI101294, SGI101295, SGI101296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101297, SGI0, SGI0, SGI0, SGI101298, SGI0, SGI0, SGI0, SGI0, SGI101299, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22405[] = {SGI101300, SGI0, SGI101301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101302, SGI0, SGI0, SGI0, SGI101303, SGI101303, SGI0, SGI0, SGI101304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22406[] = {SGI101305, SGI101306, SGI101307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101308, SGI0, SGI0, SGI0, SGI101309, SGI101309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22407[] = {SGI101310, SGI101311, SGI101312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101313, SGI0, SGI0, SGI0, SGI101314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22408[] = {SGI101315, SGI0, SGI101316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101317, SGI101318, SGI101318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22409[] = {SGI101319, SGI101320, SGI101321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101322, SGI0, SGI0, SGI0, SGI101323, SGI101323, SGI0, SGI0, SGI0, SGI101324, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22410[] = {SGI101325, SGI101326, SGI101327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101328, SGI0, SGI0, SGI0, SGI101329, SGI101329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101330, SGI0, NULL};
const Segment * const SG22411[] = {SGI101331, SGI101332, SGI101333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101334, SGI0, SGI0, SGI0, SGI101335, SGI101335, SGI0, SGI0, SGI0, SGI101336, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22412[] = {SGI101337, SGI0, SGI101338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101339, SGI101340, SGI101340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22413[] = {SGI101341, SGI101342, SGI101343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101346, SGI0, SGI0, SGI0, SGI101347, SGI101347, SGI0, SGI0, SGI0, SGI101348, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22414[] = {SGI101344, SGI0, SGI101345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101349, SGI101350, SGI101350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22415[] = {SGI101351, SGI101352, SGI101353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101354, SGI0, SGI0, SGI0, SGI101355, SGI101355, SGI0, SGI0, SGI0, SGI101356, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22416[] = {SGI101357, SGI101358, SGI101359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101360, SGI101360, SGI0, SGI0, SGI101361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22417[] = {SGI101362, SGI0, SGI101363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101364, SGI101365, SGI101365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22418[] = {SGI101366, SGI101367, SGI101368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101369, SGI0, SGI0, SGI0, SGI101370, SGI101370, SGI0, SGI101371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22419[] = {SGI101372, SGI101373, SGI101374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101375, SGI0, SGI0, SGI0, SGI101376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22420[] = {SGI101377, SGI101378, SGI101379, SGI101380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101382, SGI0, SGI0, SGI0, SGI101385, SGI101386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22421[] = {SGI101381, SGI101383, SGI101384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101387, SGI0, SGI0, SGI0, SGI101388, SGI0, SGI0, SGI101389, SGI0, NULL};
const Segment * const SG22422[] = {SGI101390, SGI101391, SGI101392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101393, SGI0, SGI0, SGI0, SGI101394, SGI101394, SGI0, SGI0, SGI0, SGI101395, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22423[] = {SGI101396, SGI101397, SGI101398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101399, SGI0, SGI0, SGI0, SGI101400, SGI101400, SGI0, SGI0, SGI0, SGI101401, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22424[] = {SGI101402, SGI101403, SGI101404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101405, SGI101405, SGI0, SGI0, SGI101406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22425[] = {SGI101407, SGI101408, SGI101409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101410, SGI101410, SGI0, SGI0, SGI101411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22426[] = {SGI101412, SGI101413, SGI101414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101415, SGI0, SGI0, SGI0, SGI101416, SGI101416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22427[] = {SGI101417, SGI0, SGI101418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101419, SGI101420, SGI101420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22428[] = {SGI101421, SGI101422, SGI101423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101424, SGI0, SGI0, SGI0, SGI101425, SGI101425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22429[] = {SGI101426, SGI101427, SGI101428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101429, SGI101429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22430[] = {SGI101430, SGI0, SGI101431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101432, SGI101433, SGI101433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22431[] = {SGI101434, SGI101435, SGI101436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101437, SGI0, SGI0, SGI0, SGI101438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22432[] = {SGI101439, SGI0, SGI101440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101441, SGI101442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22433[] = {SGI101443, SGI101444, SGI101445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101446, SGI0, SGI0, SGI0, SGI101447, SGI101447, SGI0, SGI0, SGI0, SGI0, SGI101448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22434[] = {SGI101449, SGI101450, SGI101451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101452, SGI0, SGI0, SGI0, SGI101453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22435[] = {SGI101454, SGI101455, SGI101456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101457, SGI101457, SGI0, SGI0, SGI101458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22436[] = {SGI101459, SGI101460, SGI101461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101462, SGI0, SGI0, SGI0, SGI101463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22437[] = {SGI101464, SGI101465, SGI101466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101467, SGI101467, SGI0, SGI0, SGI101468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22438[] = {SGI101469, SGI101470, SGI101471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101472, SGI0, SGI0, SGI0, SGI101473, SGI101473, SGI101474, SGI0, SGI101475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22439[] = {SGI101476, SGI101477, SGI101478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101479, SGI0, SGI0, SGI0, SGI101480, SGI101481, SGI0, SGI0, SGI101482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22440[] = {SGI101483, SGI101484, SGI101485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101486, SGI0, SGI0, SGI0, SGI101487, SGI101488, SGI0, SGI0, SGI101489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22441[] = {SGI101490, SGI101491, SGI101492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101493, SGI0, SGI0, SGI0, SGI101494, SGI101494, SGI0, SGI0, SGI0, SGI101495, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22442[] = {SGI101496, SGI101497, SGI101498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101499, SGI101499, SGI0, SGI0, SGI101501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22443[] = {SGI101500, SGI0, SGI101502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101503, SGI101504, SGI101504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101505, SGI0, NULL};
const Segment * const SG22444[] = {SGI101506, SGI101507, SGI101508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101509, SGI101509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22445[] = {SGI101510, SGI0, SGI101511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101512, SGI101513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101514, SGI0, NULL};
const Segment * const SG22446[] = {SGI101515, SGI0, SGI101516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101517, SGI101518, SGI101518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22447[] = {SGI101519, SGI101520, SGI101521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101522, SGI0, SGI0, SGI0, SGI101523, SGI101523, SGI0, SGI101524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22448[] = {SGI101525, SGI0, SGI101526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101527, SGI101528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22449[] = {SGI101529, SGI0, SGI101530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101531, SGI101532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22450[] = {SGI101533, SGI0, SGI101534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101535, SGI101536, SGI101536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22451[] = {SGI101537, SGI0, SGI101538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101539, SGI101540, SGI101540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22452[] = {SGI101541, SGI0, SGI101542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101543, SGI101544, SGI101544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22453[] = {SGI101545, SGI0, SGI101546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101547, SGI101548, SGI101548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22454[] = {SGI101549, SGI101550, SGI101551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101552, SGI0, SGI0, SGI0, SGI101553, SGI101553, SGI0, SGI0, SGI0, SGI101554, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22455[] = {SGI0, SGI101555, SGI0, SGI101556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22456[] = {SGI101557, SGI101558, SGI101559, SGI0, SGI0, SGI0, SGI101560, SGI101561, SGI0, SGI101562, SGI0, SGI0, SGI0, SGI101563, SGI101563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22457[] = {SGI101564, SGI0, SGI101565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101566, SGI0, SGI0, SGI0, SGI101567, SGI101567, SGI0, SGI0, SGI101568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22458[] = {SGI101569, SGI0, SGI101570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101571, SGI0, SGI0, SGI0, SGI101572, SGI101572, SGI0, SGI0, SGI101573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22459[] = {SGI101574, SGI101575, SGI101576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101577, SGI0, SGI0, SGI0, SGI101578, SGI101578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101579, SGI0, NULL};
const Segment * const SG22460[] = {SGI101580, SGI101581, SGI101582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101583, SGI0, SGI0, SGI0, SGI101584, SGI101584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101585, SGI0, SGI0, NULL};
const Segment * const SG22461[] = {SGI101586, SGI0, SGI101587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101588, SGI101589, SGI101589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22462[] = {SGI101590, SGI101591, SGI101592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101596, SGI101596, SGI0, SGI0, SGI101598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22463[] = {SGI101593, SGI101594, SGI101595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101597, SGI0, SGI0, SGI0, SGI101599, SGI101599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101600, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22464[] = {SGI101601, SGI101602, SGI101603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101604, SGI0, SGI0, SGI0, SGI101605, SGI101605, SGI0, SGI0, SGI0, SGI101606, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22465[] = {SGI101607, SGI101608, SGI101609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101610, SGI0, SGI0, SGI0, SGI101611, SGI101611, SGI0, SGI0, SGI0, SGI101612, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22466[] = {SGI101613, SGI0, SGI101614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101615, SGI0, SGI0, SGI0, SGI101616, SGI101616, SGI0, SGI0, SGI101617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22467[] = {SGI101618, SGI0, SGI101619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101620, SGI101621, SGI101621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22468[] = {SGI101622, SGI101623, SGI101624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101625, SGI101625, SGI0, SGI0, SGI101626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22469[] = {SGI101627, SGI101628, SGI101629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101630, SGI101630, SGI0, SGI0, SGI101631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22470[] = {SGI101632, SGI101633, SGI101634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101635, SGI101635, SGI0, SGI0, SGI101636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22471[] = {SGI101637, SGI0, SGI101638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101639, SGI0, SGI0, SGI0, SGI101640, SGI101640, SGI0, SGI0, SGI101641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22472[] = {SGI101642, SGI101643, SGI101644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101645, SGI101645, SGI0, SGI0, SGI101646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22473[] = {SGI101647, SGI101648, SGI101649, SGI101650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101651, SGI0, SGI0, SGI0, SGI101652, SGI101653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22474[] = {SGI101654, SGI0, SGI101655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101656, SGI101657, SGI101657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101658, SGI0, NULL};
const Segment * const SG22475[] = {SGI101659, SGI101660, SGI101661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101662, SGI0, SGI0, SGI0, SGI101663, SGI101663, SGI0, SGI0, SGI0, SGI101664, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22476[] = {SGI101665, SGI0, SGI101666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101667, SGI0, SGI0, SGI0, SGI101668, SGI101668, SGI0, SGI0, SGI101669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22477[] = {SGI101670, SGI101671, SGI101672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101673, SGI0, SGI0, SGI0, SGI101674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22478[] = {SGI101675, SGI0, SGI101676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22479[] = {SGI101677, SGI101678, SGI101679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101680, SGI0, SGI0, SGI0, SGI101681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22480[] = {SGI101682, SGI101683, SGI101684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101685, SGI0, SGI0, SGI0, SGI101686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22481[] = {SGI101687, SGI101688, SGI101689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101690, SGI0, SGI0, SGI0, SGI101691, SGI101692, SGI0, SGI0, SGI101693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22482[] = {SGI101694, SGI101695, SGI101696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101697, SGI0, SGI0, SGI0, SGI101698, SGI101698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22483[] = {SGI101699, SGI101700, SGI101701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101702, SGI101702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22484[] = {SGI101703, SGI101704, SGI101705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101706, SGI0, SGI0, SGI0, SGI101707, SGI101707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22485[] = {SGI0, SGI101708, SGI101709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22486[] = {SGI101710, SGI101711, SGI101712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101713, SGI0, SGI0, SGI0, SGI101714, SGI101714, SGI101715, SGI0, SGI101716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22487[] = {SGI101717, SGI101718, SGI101719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101720, SGI0, SGI0, SGI0, SGI101721, SGI101721, SGI0, SGI101722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22488[] = {SGI101723, SGI0, SGI101724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101725, SGI101726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22489[] = {SGI101727, SGI0, SGI101728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101729, SGI101730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22490[] = {SGI101731, SGI101732, SGI101733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101734, SGI101734, SGI0, SGI0, SGI101735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22491[] = {SGI101736, SGI0, SGI101737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101738, SGI101739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22492[] = {SGI101740, SGI101741, SGI101742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101743, SGI0, SGI0, SGI0, SGI101744, SGI101744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22493[] = {SGI101745, SGI101746, SGI101747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101748, SGI0, SGI0, SGI0, SGI101749, SGI101749, SGI0, SGI0, SGI0, SGI101750, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22494[] = {SGI101751, SGI101752, SGI101753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101754, SGI0, SGI0, SGI0, SGI101755, SGI101755, SGI101756, SGI0, SGI101757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22495[] = {SGI101758, SGI101759, SGI101760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101761, SGI0, SGI0, SGI0, SGI101762, SGI101762, SGI0, SGI0, SGI0, SGI101763, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22496[] = {SGI101764, SGI101765, SGI101766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101767, SGI101767, SGI0, SGI0, SGI101768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22497[] = {SGI101769, SGI101770, SGI101771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101773, SGI0, SGI0, SGI0, SGI101777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22498[] = {SGI101772, SGI101774, SGI101776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101780, SGI0, SGI0, SGI0, SGI101781, SGI101781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101783, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22499[] = {SGI101775, SGI101778, SGI101779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101782, SGI101782, SGI0, SGI0, SGI101784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22500[] = {SGI101785, SGI101786, SGI101787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101788, SGI0, SGI0, SGI0, SGI101789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22501[] = {SGI101790, SGI101791, SGI101792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101793, SGI0, SGI0, SGI0, SGI101794, SGI101795, SGI0, SGI0, SGI101796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22502[] = {SGI101797, SGI101798, SGI101799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101800, SGI0, SGI0, SGI0, SGI101801, SGI0, SGI0, SGI101802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22503[] = {SGI101803, SGI101804, SGI101805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101806, SGI0, SGI0, SGI0, SGI101807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22504[] = {SGI101808, SGI0, SGI101809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22505[] = {SGI101810, SGI101811, SGI101812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101813, SGI0, SGI0, SGI0, SGI101814, SGI101814, SGI0, SGI0, SGI0, SGI101815, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22506[] = {SGI101816, SGI0, SGI101817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101818, SGI101819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22507[] = {SGI101820, SGI0, SGI101821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101823, SGI101825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22508[] = {SGI101822, SGI101824, SGI101826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101827, SGI101827, SGI0, SGI0, SGI101828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22509[] = {SGI101829, SGI101830, SGI101831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101832, SGI101832, SGI0, SGI0, SGI101833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22510[] = {SGI101834, SGI101835, SGI101836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101837, SGI101837, SGI0, SGI0, SGI101838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22511[] = {SGI101839, SGI0, SGI101840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101841, SGI101842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22512[] = {SGI101843, SGI101844, SGI101845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101846, SGI101846, SGI0, SGI0, SGI101847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22513[] = {SGI101848, SGI0, SGI101849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101850, SGI101851, SGI101851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22514[] = {SGI101852, SGI101853, SGI101854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101855, SGI101855, SGI0, SGI0, SGI101856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22515[] = {SGI101857, SGI101858, SGI101859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101860, SGI0, SGI0, SGI0, SGI101861, SGI101861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22516[] = {SGI101862, SGI0, SGI101863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101864, SGI101865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22517[] = {SGI101866, SGI101867, SGI101868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101869, SGI0, SGI0, SGI0, SGI101870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22518[] = {SGI101871, SGI0, SGI101872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101873, SGI101874, SGI101874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22519[] = {SGI101875, SGI101876, SGI101877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101878, SGI101878, SGI0, SGI0, SGI101879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22520[] = {SGI101880, SGI101881, SGI101882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101883, SGI101883, SGI0, SGI0, SGI101884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22521[] = {SGI101885, SGI0, SGI101886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101887, SGI101888, SGI101888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22522[] = {SGI101889, SGI0, SGI101890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101891, SGI101892, SGI101892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22523[] = {SGI101893, SGI0, SGI101894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101895, SGI101896, SGI101896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22524[] = {SGI101897, SGI0, SGI101898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101899, SGI0, SGI0, SGI0, SGI101900, SGI101900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101901, SGI0, SGI0, NULL};
const Segment * const SG22525[] = {SGI101902, SGI101903, SGI101904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101905, SGI0, SGI0, SGI0, SGI101906, SGI101906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101907, SGI0, SGI0, NULL};
const Segment * const SG22526[] = {SGI101908, SGI0, SGI101909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101910, SGI101911, SGI101911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22527[] = {SGI101912, SGI0, SGI101913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101914, SGI101915, SGI101915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22528[] = {SGI101916, SGI0, SGI101917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101918, SGI101919, SGI101919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22529[] = {SGI101920, SGI101921, SGI101922, SGI0, SGI0, SGI0, SGI101923, SGI101924, SGI0, SGI101925, SGI0, SGI0, SGI0, SGI101926, SGI101926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22530[] = {SGI101927, SGI0, SGI101928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101929, SGI101930, SGI101930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22531[] = {SGI101931, SGI0, SGI101932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101933, SGI101934, SGI101934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22532[] = {SGI101935, SGI101936, SGI101937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101938, SGI0, SGI0, SGI0, SGI101939, SGI101939, SGI0, SGI0, SGI0, SGI101940, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22533[] = {SGI101941, SGI0, SGI101942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101943, SGI0, SGI0, SGI0, SGI101944, SGI101944, SGI0, SGI0, SGI101945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22534[] = {SGI101946, SGI0, SGI101947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101948, SGI0, SGI0, SGI0, SGI101949, SGI101949, SGI0, SGI0, SGI101950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22535[] = {SGI101951, SGI101952, SGI101953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101954, SGI101954, SGI0, SGI0, SGI101955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22536[] = {SGI101956, SGI0, SGI101957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101958, SGI101959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101960, SGI0, NULL};
const Segment * const SG22537[] = {SGI0, SGI101961, SGI0, SGI101962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22538[] = {SGI101963, SGI101964, SGI101965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101966, SGI0, SGI0, SGI0, SGI101967, SGI101967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22539[] = {SGI101968, SGI0, SGI101969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101970, SGI0, SGI0, SGI0, SGI101971, SGI101971, SGI0, SGI0, SGI101972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22540[] = {SGI101973, SGI101974, SGI101975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101976, SGI0, SGI0, SGI0, SGI101977, SGI101977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22541[] = {SGI101978, SGI101979, SGI101980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101981, SGI0, SGI0, SGI0, SGI101982, SGI101982, SGI0, SGI0, SGI0, SGI101983, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22542[] = {SGI101984, SGI0, SGI101985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101986, SGI101987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22543[] = {SGI101988, SGI101989, SGI101990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101991, SGI0, SGI0, SGI0, SGI101992, SGI101992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22544[] = {SGI101993, SGI101994, SGI101995, SGI101996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI101997, SGI0, SGI0, SGI0, SGI101998, SGI101999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22545[] = {SGI102000, SGI102001, SGI102002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102004, SGI0, SGI0, SGI0, SGI102007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22546[] = {SGI102003, SGI102005, SGI102006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102008, SGI0, SGI0, SGI0, SGI102009, SGI102009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22547[] = {SGI102010, SGI102011, SGI102012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102013, SGI0, SGI0, SGI0, SGI102014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22548[] = {SGI102015, SGI102016, SGI102017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102018, SGI102018, SGI0, SGI0, SGI102019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22549[] = {SGI102020, SGI102021, SGI102022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102023, SGI0, SGI0, SGI0, SGI102024, SGI102024, SGI102025, SGI0, SGI102026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22550[] = {SGI102027, SGI102028, SGI102029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102030, SGI0, SGI0, SGI0, SGI102031, SGI102031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102032, SGI0, NULL};
const Segment * const SG22551[] = {SGI102033, SGI102034, SGI102035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102036, SGI0, SGI0, SGI0, SGI102037, SGI102037, SGI0, SGI0, SGI0, SGI102038, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22552[] = {SGI102039, SGI102040, SGI102041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102042, SGI0, SGI0, SGI0, SGI102043, SGI102043, SGI0, SGI0, SGI0, SGI0, SGI102044, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22553[] = {SGI102045, SGI102046, SGI102047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102048, SGI102048, SGI0, SGI0, SGI102049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22554[] = {SGI102050, SGI102051, SGI102052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102053, SGI102053, SGI0, SGI0, SGI102054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22555[] = {SGI102055, SGI102056, SGI102057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102058, SGI102058, SGI0, SGI0, SGI102059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22556[] = {SGI102060, SGI0, SGI102061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102062, SGI102063, SGI102063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22557[] = {SGI102064, SGI0, SGI102065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102066, SGI102067, SGI102067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102068, SGI0, NULL};
const Segment * const SG22558[] = {SGI102069, SGI0, SGI102070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102071, SGI102072, SGI102072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22559[] = {SGI102073, SGI0, SGI102074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102075, SGI102076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22560[] = {SGI102077, SGI102078, SGI102079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102080, SGI102080, SGI0, SGI0, SGI102081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22561[] = {SGI102082, SGI102083, SGI102084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102088, SGI0, SGI0, SGI0, SGI102089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22562[] = {SGI102085, SGI102086, SGI102087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102090, SGI0, SGI0, SGI0, SGI102091, SGI102091, SGI0, SGI102092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22563[] = {SGI102093, SGI102094, SGI102095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102096, SGI102096, SGI0, SGI0, SGI102097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22564[] = {SGI102098, SGI102099, SGI102100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102101, SGI0, SGI0, SGI0, SGI102102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22565[] = {SGI102103, SGI102104, SGI102105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102106, SGI0, SGI0, SGI0, SGI102107, SGI102107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22566[] = {SGI102108, SGI102109, SGI102110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102111, SGI0, SGI0, SGI0, SGI102112, SGI102112, SGI0, SGI102113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22567[] = {SGI102114, SGI0, SGI102115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102118, SGI102119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22568[] = {SGI102116, SGI0, SGI102117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102120, SGI0, SGI0, SGI0, SGI102121, SGI102121, SGI0, SGI0, SGI102122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22569[] = {SGI102123, SGI0, SGI102124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102125, SGI102126, SGI102126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22570[] = {SGI102127, SGI102128, SGI102129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102130, SGI0, SGI0, SGI0, SGI102131, SGI102131, SGI0, SGI0, SGI102132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22571[] = {SGI102133, SGI102134, SGI102135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102137, SGI0, SGI0, SGI0, SGI102139, SGI102139, SGI0, SGI102140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22572[] = {SGI102136, SGI0, SGI102138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102141, SGI102142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22573[] = {SGI102143, SGI102144, SGI102145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102146, SGI0, SGI0, SGI0, SGI102147, SGI102147, SGI0, SGI102148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22574[] = {SGI102149, SGI102150, SGI102151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102152, SGI102152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102153, SGI0, NULL};
const Segment * const SG22575[] = {SGI102154, SGI102156, SGI102158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102160, SGI0, SGI0, SGI0, SGI102162, SGI102162, SGI0, SGI102164, SGI0, NULL};
const Segment * const SG22576[] = {SGI102155, SGI102157, SGI102159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102161, SGI0, SGI0, SGI0, SGI102163, SGI102163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22577[] = {SGI102165, SGI102166, SGI102167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102168, SGI102168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22578[] = {SGI102169, SGI0, SGI102170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102171, SGI0, SGI0, SGI0, SGI102172, SGI102172, SGI0, SGI0, SGI102173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22579[] = {SGI102174, SGI0, SGI102175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102177, SGI102179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22580[] = {SGI102176, SGI102178, SGI102180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102181, SGI0, SGI0, SGI0, SGI102182, SGI102183, SGI0, SGI0, SGI102184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22581[] = {SGI102185, SGI0, SGI102186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102187, SGI102188, SGI102188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22582[] = {SGI102189, SGI102190, SGI102191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102192, SGI0, SGI0, SGI0, SGI102193, SGI102193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22583[] = {SGI102194, SGI0, SGI102195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102196, SGI102197, SGI102197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102198, SGI0, NULL};
const Segment * const SG22584[] = {SGI102199, SGI0, SGI102200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102201, SGI102202, SGI102202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102203, SGI0, NULL};
const Segment * const SG22585[] = {SGI0, SGI102204, SGI102205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102206, SGI0, SGI0, SGI0, SGI0, SGI102207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22586[] = {SGI0, SGI102208, SGI102209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102210, SGI0, SGI0, SGI0, SGI0, SGI102211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22587[] = {SGI0, SGI102212, SGI102213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102214, SGI0, SGI0, SGI0, SGI0, SGI102215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22588[] = {SGI102216, SGI102217, SGI102218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102219, SGI102219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22589[] = {SGI102220, SGI102221, SGI102222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102223, SGI102224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22590[] = {SGI0, SGI102225, SGI102226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102227, SGI0, SGI0, SGI0, SGI0, SGI102228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22591[] = {SGI102229, SGI102230, SGI102231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102232, SGI0, SGI0, SGI0, SGI102233, SGI102233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22592[] = {SGI102234, SGI102235, SGI102236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102237, SGI0, SGI0, SGI0, SGI102238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22593[] = {SGI102239, SGI0, SGI102240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102241, SGI0, SGI0, SGI0, SGI102242, SGI102242, SGI0, SGI0, SGI102243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22594[] = {SGI102244, SGI102245, SGI102246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102247, SGI0, SGI0, SGI0, SGI102248, SGI102248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22595[] = {SGI102249, SGI102250, SGI102251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102252, SGI0, SGI0, SGI0, SGI102253, SGI102253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22596[] = {SGI102254, SGI102255, SGI102256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102257, SGI0, SGI0, SGI0, SGI102258, SGI102258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22597[] = {SGI102259, SGI102260, SGI102261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102262, SGI102262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22598[] = {SGI102263, SGI0, SGI102264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102265, SGI0, SGI0, SGI0, SGI102266, SGI102266, SGI0, SGI0, SGI102267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22599[] = {SGI102268, SGI0, SGI102269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102270, SGI0, SGI0, SGI0, SGI102271, SGI102271, SGI0, SGI0, SGI102272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22600[] = {SGI102273, SGI102274, SGI102275, SGI0, SGI0, SGI0, SGI102276, SGI0, SGI0, SGI102277, SGI0, SGI0, SGI0, SGI102278, SGI102278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22601[] = {SGI102279, SGI0, SGI102280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102281, SGI102282, SGI102282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22602[] = {SGI102283, SGI102284, SGI102285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102289, SGI0, SGI0, SGI0, SGI102290, SGI102290, SGI0, SGI0, SGI0, SGI102292, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22603[] = {SGI102286, SGI102287, SGI102288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102291, SGI0, SGI0, SGI0, SGI102293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22604[] = {SGI102294, SGI102295, SGI102296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102297, SGI0, SGI0, SGI0, SGI102298, SGI102298, SGI0, SGI0, SGI0, SGI102299, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22605[] = {SGI102300, SGI102301, SGI102302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102303, SGI102303, SGI0, SGI0, SGI102304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22606[] = {SGI102305, SGI0, SGI102306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102307, SGI102308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22607[] = {SGI102309, SGI102310, SGI102311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102312, SGI0, SGI0, SGI0, SGI102313, SGI102313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22608[] = {SGI102314, SGI102315, SGI102316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102317, SGI0, SGI0, SGI0, SGI102318, SGI102318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102319, SGI0, NULL};
const Segment * const SG22609[] = {SGI102320, SGI0, SGI102321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102322, SGI102323, SGI102323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22610[] = {SGI102324, SGI0, SGI102325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102326, SGI102327, SGI102327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22611[] = {SGI102328, SGI0, SGI102331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102334, SGI102335, SGI102335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22612[] = {SGI102329, SGI102330, SGI102332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102333, SGI0, SGI0, SGI0, SGI102336, SGI102336, SGI0, SGI0, SGI0, SGI102337, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22613[] = {SGI102338, SGI0, SGI102339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102340, SGI102341, SGI102341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22614[] = {SGI102342, SGI102343, SGI102344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102348, SGI0, SGI0, SGI0, SGI102350, SGI0, SGI0, SGI0, SGI0, SGI102352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22615[] = {SGI102345, SGI102346, SGI102347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102349, SGI0, SGI0, SGI0, SGI102351, SGI102351, SGI0, SGI0, SGI0, SGI102353, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22616[] = {SGI102354, SGI0, SGI102355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102356, SGI0, SGI0, SGI0, SGI102357, SGI102357, SGI0, SGI0, SGI102358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22617[] = {SGI102359, SGI0, SGI102360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102361, SGI0, SGI0, SGI0, SGI102362, SGI102362, SGI0, SGI0, SGI102363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22618[] = {SGI102364, SGI0, SGI102365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102366, SGI102367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22619[] = {SGI102368, SGI102369, SGI102370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102371, SGI102371, SGI0, SGI0, SGI102372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22620[] = {SGI102373, SGI0, SGI102374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102375, SGI102376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22621[] = {SGI102377, SGI102378, SGI102379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102380, SGI102380, SGI0, SGI0, SGI102381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22622[] = {SGI102382, SGI102383, SGI102384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102385, SGI0, SGI0, SGI0, SGI102386, SGI102386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22623[] = {SGI102388, SGI102390, SGI102392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102393, SGI0, SGI0, SGI0, SGI102394, SGI102394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22624[] = {SGI102387, SGI102389, SGI102391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102395, SGI102395, SGI0, SGI0, SGI102396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22625[] = {SGI102397, SGI102398, SGI102399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102400, SGI0, SGI0, SGI0, SGI102401, SGI102401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22626[] = {SGI102403, SGI102404, SGI102406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102408, SGI0, SGI0, SGI0, SGI102409, SGI102409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22627[] = {SGI102402, SGI102405, SGI102407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102410, SGI102410, SGI0, SGI0, SGI102411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22628[] = {SGI102412, SGI102413, SGI102414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102415, SGI0, SGI0, SGI0, SGI102416, SGI102416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22629[] = {SGI102417, SGI102418, SGI102419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102420, SGI0, SGI0, SGI0, SGI102421, SGI102421, SGI0, SGI102422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22630[] = {SGI102423, SGI102424, SGI102425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102426, SGI102426, SGI0, SGI0, SGI102427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22631[] = {SGI102428, SGI102429, SGI102430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102431, SGI0, SGI0, SGI0, SGI102432, SGI102432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102433, SGI0, NULL};
const Segment * const SG22632[] = {SGI0, SGI102434, SGI102435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102437, SGI0, SGI0, SGI0, SGI102439, SGI102439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22633[] = {SGI102436, SGI0, SGI102438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102440, SGI102441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102442, SGI0, NULL};
const Segment * const SG22634[] = {SGI102443, SGI102444, SGI102445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102446, SGI102446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22635[] = {SGI102447, SGI102448, SGI102449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102450, SGI0, SGI0, SGI0, SGI102451, SGI102451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22636[] = {SGI102452, SGI102453, SGI102454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102455, SGI0, SGI0, SGI0, SGI102456, SGI102456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22637[] = {SGI102458, SGI102460, SGI102462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102464, SGI102466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22638[] = {SGI102457, SGI102459, SGI102461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102463, SGI0, SGI0, SGI0, SGI102465, SGI102465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102467, SGI0, NULL};
const Segment * const SG22639[] = {SGI102468, SGI102469, SGI102470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102471, SGI102471, SGI0, SGI0, SGI102472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22640[] = {SGI102473, SGI0, SGI102474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102475, SGI0, SGI0, SGI0, SGI102476, SGI102476, SGI0, SGI0, SGI102477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22641[] = {SGI102478, SGI0, SGI102479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102480, SGI102481, SGI102481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102482, SGI0, NULL};
const Segment * const SG22642[] = {SGI102483, SGI102484, SGI102485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102486, SGI0, SGI0, SGI0, SGI102487, SGI102487, SGI0, SGI102488, SGI0, NULL};
const Segment * const SG22643[] = {SGI102489, SGI102490, SGI102491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102492, SGI102492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102493, SGI0, SGI0, NULL};
const Segment * const SG22644[] = {SGI102494, SGI0, SGI102495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102496, SGI102497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22645[] = {SGI102498, SGI0, SGI102499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102500, SGI102501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22646[] = {SGI102502, SGI0, SGI102503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102504, SGI102505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22647[] = {SGI102506, SGI0, SGI102507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102508, SGI102509, SGI102509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22648[] = {SGI102510, SGI102511, SGI102512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102513, SGI0, SGI0, SGI0, SGI102514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22649[] = {SGI102515, SGI0, SGI102516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102517, SGI102518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22650[] = {SGI102519, SGI0, SGI102520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102521, SGI102522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22651[] = {SGI102523, SGI102524, SGI102525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102526, SGI102526, SGI0, SGI0, SGI102527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22652[] = {SGI102528, SGI0, SGI102529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102530, SGI102531, SGI102531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102532, SGI0, NULL};
const Segment * const SG22653[] = {SGI102533, SGI102534, SGI102535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102536, SGI0, SGI0, SGI0, SGI102537, SGI102537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102538, SGI0, NULL};
const Segment * const SG22654[] = {SGI102539, SGI0, SGI102540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102541, SGI102542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22655[] = {SGI102543, SGI102544, SGI102545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102546, SGI0, SGI0, SGI0, SGI102547, SGI102547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102548, SGI0, NULL};
const Segment * const SG22656[] = {SGI102549, SGI102550, SGI102551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102552, SGI102552, SGI0, SGI0, SGI102553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22657[] = {SGI102554, SGI102555, SGI102556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102557, SGI0, SGI0, SGI0, SGI102558, SGI102558, SGI0, SGI0, SGI0, SGI102559, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22658[] = {SGI102560, SGI102561, SGI102562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102563, SGI0, SGI0, SGI0, SGI102564, SGI102564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102565, SGI0, NULL};
const Segment * const SG22659[] = {SGI102566, SGI102567, SGI102568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102569, SGI102569, SGI0, SGI0, SGI102570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22660[] = {SGI102571, SGI102572, SGI102573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102574, SGI0, SGI0, SGI0, SGI102575, SGI102575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102576, SGI0, NULL};
const Segment * const SG22661[] = {SGI102577, SGI0, SGI102578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102579, SGI102580, SGI102580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22662[] = {SGI102581, SGI0, SGI102582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102583, SGI102584, SGI102584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102585, NULL};
const Segment * const SG22663[] = {SGI102586, SGI102587, SGI102588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102589, SGI102589, SGI0, SGI0, SGI102590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22664[] = {SGI102591, SGI102592, SGI102593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102594, SGI0, SGI0, SGI0, SGI102595, SGI102595, SGI102596, SGI0, SGI102597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22665[] = {SGI102598, SGI102599, SGI102600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102601, SGI0, SGI0, SGI0, SGI102602, SGI102602, SGI0, SGI0, SGI0, SGI102603, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22666[] = {SGI102604, SGI102605, SGI102606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102607, SGI0, SGI0, SGI0, SGI102608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22667[] = {SGI102609, SGI102610, SGI102611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102612, SGI0, SGI0, SGI0, SGI102613, SGI102613, SGI0, SGI0, SGI0, SGI102614, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22668[] = {SGI102615, SGI0, SGI102616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102617, SGI102618, SGI102618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22669[] = {SGI102619, SGI102620, SGI102621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102622, SGI102622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102623, SGI0, NULL};
const Segment * const SG22670[] = {SGI102624, SGI102625, SGI102626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102627, SGI102627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102628, SGI0, NULL};
const Segment * const SG22671[] = {SGI102629, SGI102630, SGI102631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102632, SGI0, SGI0, SGI0, SGI102633, SGI102634, SGI0, SGI0, SGI102635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22672[] = {SGI102636, SGI102637, SGI102638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102639, SGI0, SGI0, SGI0, SGI102640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22673[] = {SGI102641, SGI102642, SGI102643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102644, SGI0, SGI0, SGI0, SGI102645, SGI102645, SGI102646, SGI0, SGI102647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22674[] = {SGI102648, SGI102649, SGI102650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102651, SGI0, SGI0, SGI0, SGI102652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22675[] = {SGI102653, SGI0, SGI102654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102655, SGI102656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22676[] = {SGI102657, SGI0, SGI102658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102659, SGI102660, SGI102660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102661, SGI0, NULL};
const Segment * const SG22677[] = {SGI102662, SGI102663, SGI102664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102665, SGI0, SGI0, SGI0, SGI102666, SGI102666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22678[] = {SGI102667, SGI102668, SGI102669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102670, SGI0, SGI0, SGI0, SGI102671, SGI102671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22679[] = {SGI102672, SGI102673, SGI102674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102675, SGI0, SGI0, SGI0, SGI102676, SGI102676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22680[] = {SGI102677, SGI102678, SGI102679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102680, SGI0, SGI0, SGI0, SGI102681, SGI102681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102682, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22681[] = {SGI102683, SGI102684, SGI102685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102686, SGI0, SGI0, SGI0, SGI102687, SGI102687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102688, SGI0, SGI0, NULL};
const Segment * const SG22682[] = {SGI102689, SGI102690, SGI102691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102692, SGI0, SGI0, SGI0, SGI102693, SGI102693, SGI0, SGI0, SGI0, SGI102694, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22683[] = {SGI102695, SGI0, SGI102696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102697, SGI102698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102699, SGI0, NULL};
const Segment * const SG22684[] = {SGI102700, SGI102701, SGI102702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102703, SGI0, SGI0, SGI0, SGI102704, SGI102704, SGI0, SGI0, SGI0, SGI102705, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22685[] = {SGI102706, SGI102707, SGI102708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102709, SGI0, SGI0, SGI0, SGI102710, SGI102710, SGI0, SGI0, SGI0, SGI102711, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22686[] = {SGI102712, SGI102713, SGI102714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102715, SGI0, SGI0, SGI0, SGI102716, SGI102716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102717, SGI0, SGI0, NULL};
const Segment * const SG22687[] = {SGI102718, SGI102719, SGI102720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102721, SGI0, SGI0, SGI0, SGI102722, SGI102722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22688[] = {SGI102723, SGI102724, SGI102725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102726, SGI0, SGI0, SGI0, SGI102727, SGI102727, SGI0, SGI0, SGI0, SGI102728, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22689[] = {SGI102729, SGI0, SGI102730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102731, SGI102732, SGI102732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22690[] = {SGI102733, SGI102734, SGI102735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102736, SGI0, SGI0, SGI0, SGI102737, SGI102737, SGI0, SGI0, SGI0, SGI102738, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22691[] = {SGI102739, SGI102740, SGI102741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102742, SGI0, SGI0, SGI0, SGI102743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22692[] = {SGI102744, SGI102745, SGI102746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102747, SGI0, SGI0, SGI0, SGI102748, SGI102748, SGI0, SGI0, SGI0, SGI102749, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22693[] = {SGI102750, SGI102751, SGI102752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102753, SGI0, SGI0, SGI0, SGI102754, SGI102754, SGI0, SGI0, SGI0, SGI102755, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22694[] = {SGI102756, SGI0, SGI102757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102758, SGI0, SGI0, SGI0, SGI102759, SGI102759, SGI0, SGI0, SGI102760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22695[] = {SGI102761, SGI0, SGI102762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102763, SGI0, SGI0, SGI0, SGI102764, SGI102764, SGI0, SGI0, SGI102766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22696[] = {SGI102765, SGI0, SGI102767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102768, SGI102769, SGI102769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102770, SGI0, NULL};
const Segment * const SG22697[] = {SGI102771, SGI102772, SGI102773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102774, SGI0, SGI0, SGI0, SGI102775, SGI102775, SGI0, SGI0, SGI0, SGI0, SGI102776, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22698[] = {SGI102777, SGI0, SGI102778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102779, SGI0, SGI0, SGI0, SGI102780, SGI102780, SGI0, SGI0, SGI102781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22699[] = {SGI102782, SGI0, SGI102783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102784, SGI102785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22700[] = {SGI102786, SGI102787, SGI102788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102789, SGI0, SGI0, SGI0, SGI102790, SGI0, SGI0, SGI0, SGI0, SGI102791, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22701[] = {SGI102792, SGI102793, SGI102794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102795, SGI0, SGI0, SGI0, SGI102796, SGI102796, SGI0, SGI0, SGI102797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22702[] = {SGI102798, SGI102799, SGI102800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102801, SGI0, SGI0, SGI0, SGI102802, SGI102802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22703[] = {SGI102803, SGI102804, SGI102805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102806, SGI0, SGI0, SGI0, SGI102807, SGI102807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22704[] = {SGI102808, SGI102809, SGI102810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102811, SGI0, SGI0, SGI0, SGI102812, SGI102812, SGI0, SGI0, SGI102813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22705[] = {SGI102814, SGI102816, SGI102818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102821, SGI102821, SGI0, SGI0, SGI102823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22706[] = {SGI102815, SGI102817, SGI102819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102820, SGI0, SGI0, SGI0, SGI102822, SGI102822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22707[] = {SGI102824, SGI102825, SGI102826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102827, SGI0, SGI0, SGI0, SGI102828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22708[] = {SGI102829, SGI102830, SGI102831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102832, SGI0, SGI0, SGI0, SGI102833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22709[] = {SGI102834, SGI102835, SGI102836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102837, SGI102837, SGI0, SGI0, SGI102838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22710[] = {SGI0, SGI102839, SGI0, SGI102840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22711[] = {SGI0, SGI102841, SGI0, SGI102842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102843, SGI0, SGI0, SGI0, SGI102844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22712[] = {SGI102845, SGI102846, SGI102847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102848, SGI0, SGI0, SGI0, SGI102849, SGI102849, SGI0, SGI0, SGI0, SGI102850, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22713[] = {SGI102851, SGI102852, SGI102853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102854, SGI102854, SGI0, SGI0, SGI102855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22714[] = {SGI102856, SGI102857, SGI102858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102859, SGI0, SGI0, SGI0, SGI102860, SGI102860, SGI0, SGI0, SGI0, SGI0, SGI102861, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22715[] = {SGI102862, SGI102863, SGI102864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102865, SGI0, SGI0, SGI0, SGI102866, SGI102866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22716[] = {SGI102867, SGI102868, SGI102869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102870, SGI102870, SGI0, SGI0, SGI102871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22717[] = {SGI102872, SGI102873, SGI102874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102875, SGI0, SGI0, SGI0, SGI102876, SGI102876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102877, SGI0, NULL};
const Segment * const SG22718[] = {SGI102878, SGI0, SGI102879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102880, SGI0, SGI0, SGI0, SGI102881, SGI102881, SGI0, SGI102882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22719[] = {SGI102883, SGI102884, SGI102885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102886, SGI0, SGI0, SGI0, SGI102887, SGI102887, SGI0, SGI0, SGI0, SGI0, SGI102888, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22720[] = {SGI102889, SGI102890, SGI102891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102892, SGI0, SGI0, SGI0, SGI102893, SGI102893, SGI0, SGI0, SGI0, SGI0, SGI102894, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22721[] = {SGI102895, SGI102896, SGI102897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102898, SGI0, SGI0, SGI0, SGI102901, SGI102903, SGI0, SGI0, SGI102904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22722[] = {SGI102899, SGI102900, SGI102902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102905, SGI0, SGI0, SGI0, SGI102906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22723[] = {SGI102907, SGI102908, SGI102909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102910, SGI0, SGI0, SGI0, SGI102911, SGI102912, SGI0, SGI0, SGI102913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22724[] = {SGI102914, SGI0, SGI102915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102916, SGI102917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102918, SGI0, NULL};
const Segment * const SG22725[] = {SGI102919, SGI0, SGI102920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102921, SGI102922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102923, SGI0, NULL};
const Segment * const SG22726[] = {SGI102924, SGI102925, SGI102926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102929, SGI0, SGI0, SGI0, SGI102930, SGI102930, SGI0, SGI0, SGI0, SGI102933, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22727[] = {SGI102927, SGI0, SGI102928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102931, SGI102932, SGI102932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22728[] = {SGI102934, SGI0, SGI102935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102936, SGI102937, SGI102937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22729[] = {SGI102938, SGI102939, SGI102940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102941, SGI102941, SGI0, SGI102942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22730[] = {SGI102943, SGI0, SGI102944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102945, SGI102946, SGI102946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22731[] = {SGI102947, SGI102948, SGI102949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102950, SGI102950, SGI0, SGI0, SGI102951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22732[] = {SGI102952, SGI102953, SGI102954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102955, SGI0, SGI0, SGI0, SGI102956, SGI102956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102957, SGI0, NULL};
const Segment * const SG22733[] = {SGI102958, SGI102959, SGI102960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102961, SGI102961, SGI0, SGI0, SGI102962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22734[] = {SGI102963, SGI102964, SGI102965, SGI0, SGI0, SGI0, SGI102966, SGI102967, SGI0, SGI102968, SGI0, SGI0, SGI0, SGI102969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22735[] = {SGI102970, SGI0, SGI102971, SGI0, SGI0, SGI0, SGI102972, SGI102973, SGI0, SGI102974, SGI0, SGI0, SGI0, SGI102975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22736[] = {SGI102976, SGI102977, SGI102978, SGI102979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102980, SGI0, SGI0, SGI0, SGI102981, SGI102982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22737[] = {SGI102983, SGI102984, SGI102985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102986, SGI102986, SGI0, SGI0, SGI102987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22738[] = {SGI102988, SGI102989, SGI102990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102993, SGI0, SGI0, SGI0, SGI102995, SGI102995, SGI0, SGI102997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22739[] = {SGI102991, SGI102992, SGI102994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI102996, SGI0, SGI0, SGI0, SGI102998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22740[] = {SGI102999, SGI103000, SGI103001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103002, SGI103002, SGI0, SGI0, SGI103003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22741[] = {SGI103004, SGI103005, SGI103006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103007, SGI103007, SGI0, SGI0, SGI103008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22742[] = {SGI103009, SGI0, SGI103010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103011, SGI103012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22743[] = {SGI103013, SGI0, SGI103014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103015, SGI103016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103017, SGI0, NULL};
const Segment * const SG22744[] = {SGI103018, SGI103019, SGI103020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103021, SGI103021, SGI0, SGI0, SGI103022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22745[] = {SGI103023, SGI103024, SGI103025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103026, SGI0, SGI0, SGI0, SGI103027, SGI103028, SGI0, SGI0, SGI103029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22746[] = {SGI103030, SGI103031, SGI103032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103033, SGI0, SGI0, SGI0, SGI103034, SGI103034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22747[] = {SGI103035, SGI103036, SGI103037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103038, SGI103038, SGI0, SGI0, SGI103039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22748[] = {SGI103040, SGI103041, SGI103042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103043, SGI103043, SGI0, SGI0, SGI103044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22749[] = {SGI103045, SGI103046, SGI103047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103048, SGI103048, SGI0, SGI0, SGI103049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22750[] = {SGI103050, SGI103051, SGI103052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103053, SGI0, SGI0, SGI0, SGI103054, SGI103054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22751[] = {SGI103055, SGI103056, SGI103057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103058, SGI0, SGI0, SGI0, SGI103059, SGI0, SGI103060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22752[] = {SGI103061, SGI0, SGI103062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103063, SGI103064, SGI103064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22753[] = {SGI0, SGI103065, SGI0, SGI103066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22754[] = {SGI103067, SGI0, SGI103068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103069, SGI103070, SGI103070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22755[] = {SGI103071, SGI103072, SGI103073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103074, SGI103074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103075, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22756[] = {SGI103076, SGI103077, SGI103078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103079, SGI103079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22757[] = {SGI103080, SGI103081, SGI103082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103083, SGI103084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22758[] = {SGI103085, SGI103086, SGI103087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103088, SGI0, SGI0, SGI0, SGI103089, SGI103089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22759[] = {SGI0, SGI103090, SGI103091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22760[] = {SGI103092, SGI0, SGI103093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103094, SGI103095, SGI103095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22761[] = {SGI103096, SGI103097, SGI103098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103099, SGI0, SGI0, SGI0, SGI103100, SGI103100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22762[] = {SGI103101, SGI103102, SGI103103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103104, SGI0, SGI0, SGI0, SGI103105, SGI103105, SGI0, SGI0, SGI0, SGI103106, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22763[] = {SGI103107, SGI0, SGI103108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103109, SGI103110, SGI103110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22764[] = {SGI103111, SGI103112, SGI103113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103114, SGI0, SGI0, SGI0, SGI103115, SGI103115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103116, SGI0, NULL};
const Segment * const SG22765[] = {SGI103117, SGI103118, SGI103119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103120, SGI103120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22766[] = {SGI103121, SGI103122, SGI103123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103124, SGI0, SGI0, SGI0, SGI103125, SGI103125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22767[] = {SGI103126, SGI0, SGI103127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103128, SGI103129, SGI103129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22768[] = {SGI103130, SGI103131, SGI103132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103133, SGI0, SGI0, SGI0, SGI103134, SGI103134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103135, SGI0, NULL};
const Segment * const SG22769[] = {SGI103136, SGI0, SGI103137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103138, SGI103139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22770[] = {SGI103140, SGI103141, SGI103142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103143, SGI0, SGI0, SGI0, SGI103144, SGI103144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22771[] = {SGI103145, SGI103146, SGI103147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103148, SGI0, SGI0, SGI0, SGI103149, SGI103149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22772[] = {SGI103150, SGI103151, SGI103152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103153, SGI103153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22773[] = {SGI103154, SGI103155, SGI103156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103157, SGI0, SGI0, SGI0, SGI103158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22774[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103159, SGI0, SGI0, SGI103160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22775[] = {SGI103161, SGI103162, SGI103163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103164, SGI0, SGI0, SGI0, SGI103165, SGI103165, SGI0, SGI103166, SGI0, NULL};
const Segment * const SG22776[] = {SGI103167, SGI0, SGI103168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103169, SGI103170, SGI103170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103171, SGI0, NULL};
const Segment * const SG22777[] = {SGI103172, SGI103173, SGI103174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103175, SGI0, SGI0, SGI0, SGI103176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103177, SGI0, SGI0, NULL};
const Segment * const SG22778[] = {SGI103178, SGI103179, SGI103180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103181, SGI0, SGI0, SGI0, SGI103182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103183, SGI0, SGI0, NULL};
const Segment * const SG22779[] = {SGI103184, SGI0, SGI103185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103186, SGI0, SGI0, SGI0, SGI103187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103188, SGI0, SGI0, NULL};
const Segment * const SG22780[] = {SGI103189, SGI103190, SGI103191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103192, SGI0, SGI0, SGI0, SGI103193, SGI103193, SGI0, SGI0, SGI0, SGI103194, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22781[] = {SGI103195, SGI103196, SGI103197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103198, SGI0, SGI0, SGI0, SGI103199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22782[] = {SGI103200, SGI103201, SGI103202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103203, SGI0, SGI0, SGI0, SGI103204, SGI103204, SGI0, SGI0, SGI0, SGI103205, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22783[] = {SGI103206, SGI103207, SGI103208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103209, SGI0, SGI0, SGI0, SGI103210, SGI103210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103211, SGI0, SGI0, NULL};
const Segment * const SG22784[] = {SGI103212, SGI103213, SGI103214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103215, SGI0, SGI0, SGI0, SGI103216, SGI103216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22785[] = {SGI103217, SGI103218, SGI103219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103220, SGI0, SGI0, SGI0, SGI103221, SGI103221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22786[] = {SGI103222, SGI103223, SGI103224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103225, SGI0, SGI0, SGI0, SGI103226, SGI103226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22787[] = {SGI103227, SGI103228, SGI103229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103230, SGI0, SGI0, SGI0, SGI103231, SGI103231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22788[] = {SGI103232, SGI103233, SGI103234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103235, SGI0, SGI0, SGI0, SGI103236, SGI103236, SGI0, SGI0, SGI0, SGI103237, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22789[] = {SGI103238, SGI0, SGI103239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103240, SGI0, SGI0, SGI0, SGI103241, SGI103241, SGI0, SGI0, SGI103242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22790[] = {SGI103243, SGI0, SGI103244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103245, SGI0, SGI0, SGI0, SGI103246, SGI103246, SGI0, SGI0, SGI103247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22791[] = {SGI103248, SGI103249, SGI103250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103251, SGI103251, SGI0, SGI0, SGI103252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22792[] = {SGI103253, SGI103254, SGI103255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103256, SGI0, SGI0, SGI0, SGI103257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22793[] = {SGI103258, SGI103259, SGI103260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103261, SGI0, SGI0, SGI0, SGI103262, SGI103262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22794[] = {SGI103263, SGI103264, SGI103265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103266, SGI103266, SGI0, SGI0, SGI103267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22795[] = {SGI103268, SGI103269, SGI103270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103271, SGI103271, SGI0, SGI0, SGI103272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22796[] = {SGI0, SGI103273, SGI0, SGI103274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22797[] = {SGI103275, SGI0, SGI103276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103279, SGI103281, SGI103281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22798[] = {SGI0, SGI103277, SGI0, SGI103278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103280, SGI0, SGI0, SGI0, SGI103282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22799[] = {SGI103283, SGI103284, SGI103285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103286, SGI0, SGI0, SGI0, SGI103287, SGI103287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22800[] = {SGI103288, SGI0, SGI103289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103290, SGI103291, SGI103291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22801[] = {SGI103292, SGI0, SGI103293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103294, SGI103295, SGI103295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22802[] = {SGI103296, SGI103297, SGI103298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103299, SGI0, SGI0, SGI0, SGI103300, SGI103300, SGI0, SGI0, SGI0, SGI103301, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22803[] = {SGI103302, SGI103303, SGI103304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103305, SGI0, SGI0, SGI0, SGI103306, SGI103306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22804[] = {SGI103307, SGI103308, SGI103309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103310, SGI0, SGI0, SGI0, SGI103311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22805[] = {SGI103312, SGI103313, SGI103314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103315, SGI0, SGI0, SGI0, SGI103316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22806[] = {SGI103317, SGI0, SGI103318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103319, SGI103320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22807[] = {SGI103321, SGI0, SGI103322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103323, SGI0, SGI0, SGI0, SGI103324, SGI103324, SGI0, SGI0, SGI103325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22808[] = {SGI103326, SGI0, SGI103327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103328, SGI0, SGI0, SGI0, SGI103329, SGI103329, SGI0, SGI0, SGI103330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22809[] = {SGI103331, SGI0, SGI103332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103333, SGI0, SGI0, SGI0, SGI103334, SGI103334, SGI0, SGI0, SGI103335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22810[] = {SGI103336, SGI103337, SGI103338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103339, SGI103339, SGI0, SGI0, SGI103340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22811[] = {SGI103341, SGI103342, SGI103343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103344, SGI103344, SGI0, SGI0, SGI103345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22812[] = {SGI103346, SGI0, SGI103347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103348, SGI103349, SGI103349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22813[] = {SGI103350, SGI0, SGI103351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103352, SGI103353, SGI103353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22814[] = {SGI103354, SGI103355, SGI103356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103357, SGI0, SGI0, SGI0, SGI103358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22815[] = {SGI103359, SGI103360, SGI103361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103362, SGI0, SGI0, SGI0, SGI103363, SGI103363, SGI0, SGI0, SGI0, SGI103364, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22816[] = {SGI103365, SGI103366, SGI103367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103368, SGI0, SGI0, SGI0, SGI103369, SGI103369, SGI0, SGI0, SGI0, SGI103370, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22817[] = {SGI0, SGI103371, SGI0, SGI103372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22818[] = {SGI103373, SGI103374, SGI103375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103376, SGI0, SGI0, SGI0, SGI103377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22819[] = {SGI0, SGI103378, SGI103379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103380, SGI0, SGI0, SGI0, SGI103381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22820[] = {SGI103382, SGI103383, SGI103384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103385, SGI0, SGI0, SGI0, SGI103386, SGI103386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22821[] = {SGI103387, SGI103388, SGI103389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103390, SGI0, SGI0, SGI0, SGI103391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22822[] = {SGI103392, SGI0, SGI103393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103394, SGI0, SGI0, SGI0, SGI103395, SGI103395, SGI0, SGI0, SGI103396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22823[] = {SGI103397, SGI103398, SGI103399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103400, SGI0, SGI0, SGI0, SGI103401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22824[] = {SGI103402, SGI103403, SGI103404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103405, SGI0, SGI0, SGI0, SGI103406, SGI103406, SGI0, SGI0, SGI0, SGI0, SGI103407, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22825[] = {SGI103408, SGI103409, SGI103410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103411, SGI0, SGI0, SGI0, SGI103412, SGI103412, SGI0, SGI0, SGI0, SGI103413, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22826[] = {SGI103414, SGI103415, SGI103416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103417, SGI0, SGI0, SGI0, SGI103418, SGI103418, SGI0, SGI0, SGI0, SGI103419, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22827[] = {SGI103420, SGI103421, SGI103422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103423, SGI0, SGI103424, SGI103424, SGI0, SGI103425, SGI0, NULL};
const Segment * const SG22828[] = {SGI103426, SGI103427, SGI103428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103429, SGI0, SGI0, SGI0, SGI103430, SGI0, SGI0, SGI0, SGI0, SGI103431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22829[] = {SGI103432, SGI103433, SGI103434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103435, SGI0, SGI0, SGI0, SGI103436, SGI103436, SGI103437, SGI0, SGI103438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22830[] = {SGI103439, SGI0, SGI103440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103441, SGI103442, SGI103442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22831[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103443, SGI0, SGI103444, SGI0, NULL};
const Segment * const SG22832[] = {SGI103445, SGI0, SGI103446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103447, SGI103448, SGI103448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22833[] = {SGI103449, SGI103450, SGI103451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103452, SGI0, SGI0, SGI0, SGI103453, SGI103453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22834[] = {SGI103454, SGI103455, SGI103456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103457, SGI103457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22835[] = {SGI103458, SGI103459, SGI103460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103461, SGI0, SGI0, SGI0, SGI103462, SGI103462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22836[] = {SGI103463, SGI103464, SGI103465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103466, SGI0, SGI0, SGI0, SGI103467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22837[] = {SGI103468, SGI0, SGI103469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103470, SGI103471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22838[] = {SGI103472, SGI103473, SGI103474, SGI0, SGI0, SGI0, SGI103475, SGI103476, SGI0, SGI103477, SGI0, SGI0, SGI0, SGI103478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22839[] = {SGI103479, SGI103480, SGI103481, SGI0, SGI0, SGI0, SGI103482, SGI103483, SGI0, SGI103484, SGI0, SGI0, SGI0, SGI103485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22840[] = {SGI103486, SGI103487, SGI103488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103489, SGI0, SGI0, SGI0, SGI103490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103491, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22841[] = {SGI103492, SGI0, SGI103493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103494, SGI0, SGI0, SGI0, SGI103495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103496, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22842[] = {SGI103497, SGI0, SGI103498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103499, SGI0, SGI0, SGI0, SGI103500, SGI103500, SGI0, SGI0, SGI103501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22843[] = {SGI103502, SGI103503, SGI103504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103505, SGI0, SGI0, SGI0, SGI103506, SGI103506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22844[] = {SGI103507, SGI0, SGI103508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103509, SGI103510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22845[] = {SGI103511, SGI0, SGI103512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103513, SGI103514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22846[] = {SGI103515, SGI0, SGI103516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103517, SGI103518, SGI103518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22847[] = {SGI103519, SGI103520, SGI103521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103522, SGI0, SGI0, SGI0, SGI103523, SGI103523, SGI0, SGI0, SGI0, SGI103524, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22848[] = {SGI103525, SGI103526, SGI103527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103528, SGI0, SGI0, SGI0, SGI103529, SGI103529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103530, SGI0, SGI0, NULL};
const Segment * const SG22849[] = {SGI103531, SGI103532, SGI103533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103534, SGI0, SGI0, SGI0, SGI103535, SGI103535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103536, SGI0, SGI0, NULL};
const Segment * const SG22850[] = {SGI103537, SGI103538, SGI103539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103540, SGI0, SGI0, SGI0, SGI103541, SGI103541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22851[] = {SGI103542, SGI103543, SGI103544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103545, SGI0, SGI0, SGI0, SGI103546, SGI103546, SGI0, SGI0, SGI0, SGI103547, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22852[] = {SGI103548, SGI0, SGI103549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103550, SGI103551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22853[] = {SGI103552, SGI103553, SGI103554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103555, SGI0, SGI0, SGI0, SGI103556, SGI103556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22854[] = {SGI103557, SGI0, SGI103558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103559, SGI103560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22855[] = {SGI103561, SGI103562, SGI103563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103564, SGI0, SGI0, SGI0, SGI103565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22856[] = {SGI103566, SGI103567, SGI103568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103569, SGI0, SGI0, SGI0, SGI103570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22857[] = {SGI103571, SGI103572, SGI103573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103574, SGI0, SGI0, SGI0, SGI103575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22858[] = {SGI103576, SGI103577, SGI103578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103579, SGI0, SGI0, SGI0, SGI103580, SGI103580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103581, SGI0, SGI0, NULL};
const Segment * const SG22859[] = {SGI103582, SGI103583, SGI103584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103585, SGI0, SGI0, SGI0, SGI103586, SGI103586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22860[] = {SGI103587, SGI103588, SGI103589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103590, SGI0, SGI0, SGI0, SGI103591, SGI103591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22861[] = {SGI103592, SGI103593, SGI103594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103595, SGI103595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22862[] = {SGI103596, SGI0, SGI103597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103598, SGI103599, SGI103599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22863[] = {SGI103600, SGI103601, SGI103602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103603, SGI0, SGI0, SGI0, SGI103604, SGI103604, SGI0, SGI103605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22864[] = {SGI103606, SGI0, SGI103607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103608, SGI103609, SGI103609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22865[] = {SGI103610, SGI103611, SGI103612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103613, SGI0, SGI0, SGI0, SGI103614, SGI103614, SGI0, SGI0, NULL};
const Segment * const SG22866[] = {SGI103615, SGI103616, SGI103617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103618, SGI0, SGI0, SGI0, SGI103619, SGI103619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103620, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22867[] = {SGI103621, SGI103622, SGI103623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103625, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22868[] = {SGI103626, SGI103627, SGI103628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103629, SGI0, SGI103630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22869[] = {SGI103631, SGI0, SGI103632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103633, SGI103634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22870[] = {SGI103635, SGI0, SGI103636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103637, SGI103638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22871[] = {SGI103639, SGI0, SGI103640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103641, SGI0, SGI0, SGI0, SGI103642, SGI103642, SGI0, SGI103643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22872[] = {SGI103644, SGI0, SGI103645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103646, SGI0, SGI0, SGI0, SGI103647, SGI103647, SGI0, SGI103648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22873[] = {SGI103649, SGI0, SGI103650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103651, SGI103652, SGI103652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22874[] = {SGI103653, SGI0, SGI103654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103655, SGI103656, SGI103656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22875[] = {SGI103657, SGI103658, SGI103659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103660, SGI103660, SGI0, SGI0, SGI103661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22876[] = {SGI103662, SGI0, SGI103663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103664, SGI103665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22877[] = {SGI103666, SGI103667, SGI103668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103669, SGI0, SGI0, SGI0, SGI103670, SGI103670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22878[] = {SGI103671, SGI103672, SGI103673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103674, SGI0, SGI0, SGI0, SGI103675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22879[] = {SGI103676, SGI103678, SGI103680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103682, SGI0, SGI0, SGI0, SGI103684, SGI103684, SGI0, SGI103686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22880[] = {SGI103677, SGI103679, SGI103681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103683, SGI0, SGI0, SGI0, SGI103685, SGI103685, SGI0, SGI0, SGI0, SGI103687, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22881[] = {SGI103688, SGI0, SGI103689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103690, SGI103691, SGI103691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22882[] = {SGI103692, SGI0, SGI103693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103694, SGI103695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22883[] = {SGI103696, SGI103697, SGI103698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103699, SGI0, SGI0, SGI0, SGI103700, SGI103700, SGI0, SGI0, SGI0, SGI0, SGI103701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22884[] = {SGI103702, SGI103703, SGI103704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103705, SGI0, SGI0, SGI0, SGI103706, SGI103706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103707, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22885[] = {SGI103708, SGI103709, SGI103710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103711, SGI103711, SGI0, SGI0, SGI103712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22886[] = {SGI103713, SGI0, SGI103714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103715, SGI0, SGI0, SGI0, SGI103716, SGI103716, SGI0, SGI0, SGI103717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22887[] = {SGI103718, SGI103719, SGI103720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103721, SGI0, SGI0, SGI0, SGI103722, SGI103722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22888[] = {SGI103723, SGI0, SGI103724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103725, SGI103726, SGI103726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22889[] = {SGI103728, SGI103730, SGI103732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103734, SGI0, SGI0, SGI0, SGI103736, SGI103736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103738, SGI0, SGI0, NULL};
const Segment * const SG22890[] = {SGI103727, SGI103729, SGI103731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103733, SGI0, SGI0, SGI0, SGI103735, SGI103735, SGI0, SGI0, SGI0, SGI0, SGI103737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22891[] = {SGI103739, SGI0, SGI103740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103741, SGI103742, SGI103742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22892[] = {SGI103743, SGI103744, SGI103745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103746, SGI0, SGI0, SGI0, SGI103747, SGI103747, SGI0, SGI0, SGI0, SGI103748, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22893[] = {SGI103749, SGI103750, SGI103751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103752, SGI0, SGI0, SGI0, SGI103753, SGI103753, SGI0, SGI0, SGI0, SGI103754, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22894[] = {SGI103755, SGI0, SGI103756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103757, SGI103758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22895[] = {SGI103759, SGI0, SGI103760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103761, SGI103762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22896[] = {SGI103763, SGI0, SGI103764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103765, SGI103766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22897[] = {SGI103767, SGI0, SGI103768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103769, SGI103770, SGI103770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103771, NULL};
const Segment * const SG22898[] = {SGI103772, SGI0, SGI103773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103774, SGI103775, SGI103775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22899[] = {SGI103776, SGI103777, SGI103778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103779, SGI103779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103780, SGI0, SGI0, NULL};
const Segment * const SG22900[] = {SGI103781, SGI0, SGI103782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103786, SGI103787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22901[] = {SGI103783, SGI103784, SGI103785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103788, SGI0, SGI0, SGI0, SGI103789, SGI103789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103790, SGI0, NULL};
const Segment * const SG22902[] = {SGI103791, SGI103792, SGI103793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103794, SGI0, SGI0, SGI0, SGI103795, SGI103795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22903[] = {SGI103796, SGI103797, SGI103798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103799, SGI103799, SGI0, SGI0, SGI103800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22904[] = {SGI103801, SGI103802, SGI103803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103804, SGI0, SGI0, SGI0, SGI103805, SGI103805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103806, SGI0, SGI0, NULL};
const Segment * const SG22905[] = {SGI103807, SGI103808, SGI103809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103810, SGI103810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22906[] = {SGI103811, SGI0, SGI103812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22907[] = {SGI103813, SGI103814, SGI103815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103816, SGI103816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22908[] = {SGI103817, SGI103818, SGI103819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103820, SGI0, SGI0, SGI0, SGI103821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22909[] = {SGI103822, SGI103823, SGI103824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103825, SGI0, SGI0, SGI0, SGI103826, SGI103826, SGI0, SGI0, SGI0, SGI103827, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22910[] = {SGI103828, SGI0, SGI103829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103830, SGI103831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22911[] = {SGI103832, SGI103833, SGI103834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103835, SGI103835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22912[] = {SGI0, SGI103836, SGI103837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103838, SGI0, SGI0, SGI0, SGI0, SGI103839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22913[] = {SGI0, SGI103840, SGI103841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103842, SGI0, SGI0, SGI0, SGI0, SGI103843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22914[] = {SGI0, SGI103844, SGI103845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103846, SGI0, SGI0, SGI0, SGI0, SGI103847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22915[] = {SGI0, SGI103848, SGI103849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103850, SGI0, SGI0, SGI0, SGI0, SGI103851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22916[] = {SGI103852, SGI103853, SGI103854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103855, SGI0, SGI0, SGI0, SGI103856, SGI103856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22917[] = {SGI103857, SGI103858, SGI103859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103860, SGI0, SGI0, SGI0, SGI103861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22918[] = {SGI103862, SGI103863, SGI103864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103865, SGI0, SGI0, SGI0, SGI103866, SGI103866, SGI0, SGI0, SGI0, SGI103867, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22919[] = {SGI103868, SGI103869, SGI103870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103871, SGI0, SGI0, SGI0, SGI103872, SGI103872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103873, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22920[] = {SGI103874, SGI103875, SGI103876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103877, SGI0, SGI0, SGI0, SGI103878, SGI103878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22921[] = {SGI103879, SGI103880, SGI103881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103882, SGI0, SGI0, SGI0, SGI103883, SGI103883, SGI0, SGI0, SGI0, SGI103884, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22922[] = {SGI103885, SGI103886, SGI103887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103888, SGI0, SGI0, SGI0, SGI103889, SGI103889, SGI0, SGI0, SGI0, SGI103890, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22923[] = {SGI103891, SGI103892, SGI103893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103894, SGI0, SGI0, SGI0, SGI103895, SGI103895, SGI0, SGI0, SGI0, SGI103896, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22924[] = {SGI103897, SGI103898, SGI103899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103900, SGI0, SGI0, SGI0, SGI103901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22925[] = {SGI103902, SGI103903, SGI103904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103905, SGI0, SGI0, SGI0, SGI103906, SGI103906, SGI0, SGI0, SGI0, SGI103907, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22926[] = {SGI103908, SGI103909, SGI103910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103911, SGI0, SGI0, SGI0, SGI103912, SGI103912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22927[] = {SGI103913, SGI103914, SGI103915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103916, SGI0, SGI0, SGI0, SGI103917, SGI103917, SGI103918, SGI0, SGI103919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22928[] = {SGI103920, SGI0, SGI103921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103922, SGI103923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22929[] = {SGI103924, SGI0, SGI103925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103926, SGI103927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22930[] = {SGI103928, SGI0, SGI103929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103930, SGI0, SGI0, SGI0, SGI103931, SGI103931, SGI0, SGI0, SGI103932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22931[] = {SGI103933, SGI103934, SGI103935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103936, SGI0, SGI0, SGI0, SGI103937, SGI103937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22932[] = {SGI103938, SGI103939, SGI103940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103941, SGI0, SGI0, SGI0, SGI103942, SGI103942, SGI0, SGI0, SGI0, SGI103943, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22933[] = {SGI103944, SGI103945, SGI103946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103947, SGI0, SGI0, SGI0, SGI103948, SGI103948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103949, SGI0, SGI0, NULL};
const Segment * const SG22934[] = {SGI103950, SGI103951, SGI103952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103953, SGI0, SGI0, SGI0, SGI103954, SGI103954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103955, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22935[] = {SGI103956, SGI0, SGI103957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103958, SGI103959, SGI103959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22936[] = {SGI103960, SGI103961, SGI103962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103963, SGI0, SGI0, SGI0, SGI103964, SGI103964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22937[] = {SGI103965, SGI103966, SGI103967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103968, SGI0, SGI0, SGI0, SGI103969, SGI103969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22938[] = {SGI103970, SGI0, SGI103971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103974, SGI103975, SGI103975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103976, NULL};
const Segment * const SG22939[] = {SGI103972, SGI0, SGI103973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103977, SGI103978, SGI103978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22940[] = {SGI103979, SGI0, SGI103980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103982, SGI103984, SGI103984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103985, NULL};
const Segment * const SG22941[] = {SGI103981, SGI0, SGI103983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103986, SGI103987, SGI103987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22942[] = {SGI103988, SGI0, SGI103989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103990, SGI103991, SGI103991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103992, SGI0, NULL};
const Segment * const SG22943[] = {SGI103993, SGI0, SGI103994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI103995, SGI103996, SGI103996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22944[] = {SGI103997, SGI103998, SGI103999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104000, SGI0, SGI0, SGI0, SGI104001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22945[] = {SGI104002, SGI0, SGI104003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104004, SGI104005, SGI104005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22946[] = {SGI104006, SGI0, SGI104007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104008, SGI104009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104010, SGI0, NULL};
const Segment * const SG22947[] = {SGI104011, SGI104012, SGI104013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104014, SGI104014, SGI0, SGI0, SGI104015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22948[] = {SGI104016, SGI104017, SGI104018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104019, SGI104019, SGI0, SGI0, SGI104020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22949[] = {SGI104021, SGI0, SGI104022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104023, SGI104024, SGI104024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22950[] = {SGI104025, SGI104026, SGI104027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104028, SGI0, SGI0, SGI0, SGI104029, SGI104029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22951[] = {SGI104030, SGI104031, SGI104032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104033, SGI0, SGI0, SGI0, SGI104034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22952[] = {SGI104035, SGI0, SGI104036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104037, SGI104038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104039, SGI0, NULL};
const Segment * const SG22953[] = {SGI104040, SGI0, SGI104041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104042, SGI104043, SGI104043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22954[] = {SGI104044, SGI0, SGI104045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104046, SGI104047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22955[] = {SGI104048, SGI104049, SGI104050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104051, SGI0, SGI0, SGI0, SGI104052, SGI104052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104053, SGI0, SGI0, NULL};
const Segment * const SG22956[] = {SGI104054, SGI0, SGI104055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104059, SGI0, SGI0, SGI0, SGI104061, SGI104061, SGI0, SGI0, SGI104063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22957[] = {SGI104056, SGI104057, SGI104058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104060, SGI0, SGI0, SGI0, SGI104062, SGI104062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22958[] = {SGI104064, SGI104066, SGI104067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104070, SGI0, SGI0, SGI0, SGI104072, SGI104072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22959[] = {SGI104065, SGI104068, SGI104069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104071, SGI0, SGI0, SGI0, SGI104073, SGI104073, SGI0, SGI0, SGI0, SGI104074, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22960[] = {SGI104075, SGI104076, SGI104077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104078, SGI0, SGI0, SGI0, SGI104079, SGI104079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104080, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22961[] = {SGI104081, SGI0, SGI104082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104083, SGI104084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22962[] = {SGI104085, SGI0, SGI104086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104087, SGI104088, SGI104088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104089, SGI0, NULL};
const Segment * const SG22963[] = {SGI104090, SGI104091, SGI104092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104093, SGI0, SGI0, SGI0, SGI104094, SGI104094, SGI104095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22964[] = {SGI104096, SGI104097, SGI104098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104099, SGI0, SGI0, SGI0, SGI104100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22965[] = {SGI0, SGI104101, SGI104102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104103, SGI0, SGI0, SGI0, SGI104104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22966[] = {SGI104105, SGI104106, SGI104107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104108, SGI0, SGI0, SGI0, SGI104109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22967[] = {SGI104110, SGI104111, SGI104112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104116, SGI0, SGI0, SGI0, SGI104117, SGI104118, SGI0, SGI0, SGI104120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22968[] = {SGI104113, SGI104114, SGI104115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104119, SGI0, SGI0, SGI0, SGI104121, SGI104121, SGI0, SGI0, SGI0, SGI104122, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22969[] = {SGI104123, SGI0, SGI104124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104125, SGI104126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22970[] = {SGI104127, SGI104128, SGI104129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104130, SGI0, SGI0, SGI0, SGI104131, SGI104131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22971[] = {SGI104132, SGI104133, SGI104134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104135, SGI0, SGI0, SGI0, SGI104136, SGI104136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22972[] = {SGI104137, SGI104138, SGI104139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104140, SGI0, SGI0, SGI0, SGI104141, SGI104141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22973[] = {SGI104142, SGI104143, SGI104144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104145, SGI0, SGI0, SGI0, SGI104146, SGI104146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22974[] = {SGI104147, SGI0, SGI104148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104149, SGI104150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22975[] = {SGI104151, SGI104152, SGI104153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104154, SGI0, SGI0, SGI0, SGI104155, SGI104155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22976[] = {SGI104156, SGI0, SGI104157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22977[] = {SGI104158, SGI0, SGI104159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104160, SGI0, SGI0, SGI0, SGI104161, SGI104161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104165, SGI0, SGI0, NULL};
const Segment * const SG22978[] = {SGI104162, SGI104163, SGI104164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104166, SGI0, SGI0, SGI0, SGI104167, SGI104167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22979[] = {SGI104168, SGI104169, SGI104170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104171, SGI0, SGI0, SGI0, SGI104172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22980[] = {SGI104173, SGI0, SGI104174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104175, SGI104176, SGI104176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22981[] = {SGI104177, SGI104178, SGI104179, SGI104180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104181, SGI0, SGI0, SGI0, SGI104182, SGI104183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22982[] = {SGI104184, SGI104185, SGI104186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104187, SGI0, SGI0, SGI0, SGI104188, SGI104188, SGI0, SGI0, SGI0, SGI104189, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22983[] = {SGI104190, SGI0, SGI104191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104192, SGI104193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22984[] = {SGI104194, SGI104195, SGI104196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104197, SGI0, SGI0, SGI0, SGI104198, SGI104198, SGI0, SGI0, SGI0, SGI104199, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22985[] = {SGI104200, SGI104201, SGI104202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104203, SGI0, SGI0, SGI0, SGI104204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22986[] = {SGI104205, SGI104206, SGI104207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104208, SGI0, SGI0, SGI0, SGI104209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22987[] = {SGI104210, SGI0, SGI104211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104212, SGI104213, SGI104213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22988[] = {SGI104214, SGI104215, SGI104216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104217, SGI0, SGI0, SGI0, SGI104218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22989[] = {SGI104219, SGI104220, SGI104221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104222, SGI104222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104223, SGI0, SGI0, NULL};
const Segment * const SG22990[] = {SGI104224, SGI104225, SGI104226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104227, SGI0, SGI0, SGI0, SGI104228, SGI104228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104229, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22991[] = {SGI104230, SGI0, SGI104231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104232, SGI104233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22992[] = {SGI104234, SGI0, SGI104235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104236, SGI104237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104238, SGI0, NULL};
const Segment * const SG22993[] = {SGI104239, SGI104240, SGI104241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104242, SGI104242, SGI0, SGI0, SGI104243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22994[] = {SGI104244, SGI104245, SGI104246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104247, SGI0, SGI0, SGI0, SGI104248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22995[] = {SGI104249, SGI104250, SGI104251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104252, SGI0, SGI0, SGI0, SGI104253, SGI104253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104254, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22996[] = {SGI104255, SGI104256, SGI104257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104258, SGI0, SGI0, SGI0, SGI104259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22997[] = {SGI104260, SGI104261, SGI104262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104263, SGI104263, SGI0, SGI0, SGI104264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22998[] = {SGI104265, SGI104266, SGI104267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104268, SGI0, SGI104269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG22999[] = {SGI104270, SGI0, SGI104271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104272, SGI104273, SGI104273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104274, SGI0, NULL};
const Segment * const SG23000[] = {SGI104275, SGI0, SGI104276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104277, SGI0, SGI0, SGI0, SGI104278, SGI104278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104279, SGI0, SGI0, NULL};
const Segment * const SG23001[] = {SGI104280, SGI0, SGI104281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104282, SGI104283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23002[] = {SGI104284, SGI104285, SGI104286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104287, SGI0, SGI0, SGI0, SGI104288, SGI104288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104289, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23003[] = {SGI104290, SGI104291, SGI104292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104295, SGI0, SGI0, SGI0, SGI104297, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23004[] = {SGI104293, SGI104294, SGI104296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104298, SGI0, SGI0, SGI0, SGI104299, SGI104299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23005[] = {SGI104300, SGI104301, SGI104302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104303, SGI0, SGI0, SGI0, SGI104304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23006[] = {SGI104305, SGI104306, SGI104307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104308, SGI0, SGI0, SGI0, SGI104309, SGI104309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104310, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23007[] = {SGI104311, SGI104312, SGI104313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104314, SGI0, SGI0, SGI0, SGI104315, SGI104315, SGI0, SGI0, SGI0, SGI104316, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23008[] = {SGI104317, SGI104318, SGI104319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104320, SGI0, SGI0, SGI0, SGI104321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23009[] = {SGI0, SGI104322, SGI0, SGI104323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23010[] = {SGI0, SGI104324, SGI0, SGI104325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104326, SGI0, SGI0, SGI0, SGI104327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23011[] = {SGI104328, SGI104329, SGI104330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104331, SGI0, SGI0, SGI0, SGI104332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23012[] = {SGI104333, SGI0, SGI104334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23013[] = {SGI104335, SGI0, SGI104336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104337, SGI0, SGI0, SGI0, SGI104338, SGI104338, SGI0, SGI0, SGI104339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23014[] = {SGI104340, SGI104341, SGI104342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104343, SGI0, SGI0, SGI0, SGI104344, SGI104344, SGI0, SGI0, SGI0, SGI104345, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23015[] = {SGI104346, SGI0, SGI104347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104348, SGI104349, SGI104349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23016[] = {SGI104350, SGI0, SGI104351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104352, SGI0, SGI0, SGI0, SGI104353, SGI104353, SGI0, SGI0, SGI104354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23017[] = {SGI104355, SGI104356, SGI104357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104358, SGI0, SGI0, SGI0, SGI104359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23018[] = {SGI104360, SGI104361, SGI104362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23019[] = {SGI104363, SGI0, SGI104364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23020[] = {SGI104365, SGI104366, SGI104367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104368, SGI104368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23021[] = {SGI104369, SGI104370, SGI104371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104372, SGI0, SGI0, SGI0, SGI104373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23022[] = {SGI104374, SGI104375, SGI104376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104377, SGI0, SGI0, SGI0, SGI104378, SGI104378, SGI0, SGI0, SGI104379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23023[] = {SGI104380, SGI104381, SGI104382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104383, SGI0, SGI0, SGI0, SGI104384, SGI0, SGI0, SGI0, SGI0, SGI104385, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23024[] = {SGI104386, SGI0, SGI104387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104388, SGI0, SGI0, SGI0, SGI104389, SGI104389, SGI0, SGI0, SGI104390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23025[] = {SGI104391, SGI104392, SGI104393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104394, SGI0, SGI0, SGI0, SGI104395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23026[] = {SGI104396, SGI104397, SGI104398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104399, SGI0, SGI0, SGI0, SGI104400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23027[] = {SGI104401, SGI0, SGI104402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104403, SGI104404, SGI104404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104405, NULL};
const Segment * const SG23028[] = {SGI104406, SGI104407, SGI104408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104409, SGI0, SGI0, SGI0, SGI104410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104411, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23029[] = {SGI104412, SGI104413, SGI104414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104415, SGI0, SGI0, SGI0, SGI104416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104417, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23030[] = {SGI104418, SGI104419, SGI104420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104421, SGI0, SGI0, SGI0, SGI104422, SGI104422, SGI0, SGI0, SGI0, SGI104423, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23031[] = {SGI104424, SGI0, SGI104425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104428, SGI104429, SGI104429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23032[] = {SGI104426, SGI0, SGI104427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104430, SGI0, SGI0, SGI0, SGI104431, SGI104431, SGI0, SGI0, SGI104432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23033[] = {SGI104433, SGI0, SGI104434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104435, SGI104436, SGI104436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23034[] = {SGI104437, SGI0, SGI104438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104439, SGI104440, SGI104440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23035[] = {SGI104441, SGI0, SGI104442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104443, SGI104444, SGI104444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23036[] = {SGI104445, SGI0, SGI104446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104447, SGI104448, SGI104448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23037[] = {SGI104449, SGI0, SGI104450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104451, SGI104452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23038[] = {SGI104453, SGI104454, SGI104455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104456, SGI0, SGI0, SGI0, SGI104457, SGI104457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23039[] = {SGI104458, SGI104459, SGI104460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104461, SGI0, SGI0, SGI0, SGI104462, SGI104462, SGI0, SGI0, NULL};
const Segment * const SG23040[] = {SGI104463, SGI104464, SGI104465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104466, SGI0, SGI0, SGI0, SGI104467, SGI104467, SGI104468, SGI0, SGI104469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23041[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104470, SGI0, SGI104471, SGI0, NULL};
const Segment * const SG23042[] = {SGI104472, SGI104473, SGI104474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104475, SGI0, SGI0, SGI0, SGI104476, SGI104476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23043[] = {SGI104477, SGI0, SGI104478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104479, SGI104480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23044[] = {SGI104481, SGI0, SGI104482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104483, SGI104484, SGI104484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104485, SGI0, NULL};
const Segment * const SG23045[] = {SGI104486, SGI104487, SGI104488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104489, SGI0, SGI0, SGI0, SGI104490, SGI104490, SGI104491, SGI0, SGI104492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23046[] = {SGI104493, SGI104494, SGI104495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104496, SGI0, SGI104497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23047[] = {SGI104498, SGI104499, SGI104500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104501, SGI0, SGI0, SGI0, SGI104502, SGI104502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104503, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23048[] = {SGI104504, SGI104505, SGI104506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104507, SGI0, SGI104508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23049[] = {SGI104509, SGI104510, SGI104511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104512, SGI0, SGI0, SGI0, SGI104513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23050[] = {SGI104514, SGI104515, SGI104516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104517, SGI0, SGI0, SGI0, SGI104518, SGI104518, SGI0, SGI0, SGI0, SGI104519, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23051[] = {SGI104520, SGI0, SGI104521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104522, SGI104523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23052[] = {SGI104524, SGI0, SGI104525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104526, SGI104527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23053[] = {SGI104528, SGI0, SGI104529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104530, SGI0, SGI0, SGI0, SGI104531, SGI104531, SGI0, SGI0, SGI104532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23054[] = {SGI104533, SGI0, SGI104534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104535, SGI0, SGI0, SGI0, SGI104536, SGI104536, SGI0, SGI0, SGI104537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23055[] = {SGI0, SGI104538, SGI104539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104540, SGI104540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23056[] = {SGI104541, SGI104542, SGI104543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104544, SGI0, SGI0, SGI0, SGI104545, SGI104545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23057[] = {SGI104546, SGI104547, SGI104548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104549, SGI104549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104550, SGI0, SGI0, NULL};
const Segment * const SG23058[] = {SGI104551, SGI104552, SGI104553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104554, SGI0, SGI0, SGI0, SGI104555, SGI0, SGI0, SGI0, SGI0, SGI104556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23059[] = {SGI104557, SGI104558, SGI104559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104560, SGI0, SGI0, SGI0, SGI104561, SGI0, SGI104562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23060[] = {SGI104563, SGI104564, SGI104565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104566, SGI0, SGI0, SGI0, SGI104567, SGI0, SGI104568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23061[] = {SGI104569, SGI104570, SGI104571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104572, SGI0, SGI0, SGI0, SGI104573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23062[] = {SGI104574, SGI104575, SGI104576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104577, SGI0, SGI0, SGI0, SGI104578, SGI104578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23063[] = {SGI104579, SGI0, SGI104580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104581, SGI104582, SGI104582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104583, NULL};
const Segment * const SG23064[] = {SGI104584, SGI0, SGI104585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104586, SGI104587, SGI104587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23065[] = {SGI104588, SGI104589, SGI104590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104591, SGI104591, SGI0, SGI0, SGI104592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23066[] = {SGI104593, SGI0, SGI104594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104595, SGI104596, SGI104596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23067[] = {SGI104597, SGI0, SGI104598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104599, SGI104600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23068[] = {SGI104601, SGI0, SGI104602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104603, SGI0, SGI0, SGI0, SGI104604, SGI104604, SGI0, SGI0, SGI104605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23069[] = {SGI104606, SGI104607, SGI104608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104609, SGI0, SGI0, SGI0, SGI104610, SGI104610, SGI0, SGI0, SGI0, SGI104611, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23070[] = {SGI104612, SGI104613, SGI104614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104615, SGI0, SGI0, SGI0, SGI104616, SGI104616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104617, SGI0, SGI0, NULL};
const Segment * const SG23071[] = {SGI104618, SGI104619, SGI104620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104621, SGI0, SGI0, SGI0, SGI104622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23072[] = {SGI104623, SGI104624, SGI104625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104626, SGI0, SGI0, SGI0, SGI104627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23073[] = {SGI104628, SGI104629, SGI104630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104631, SGI0, SGI0, SGI0, SGI104632, SGI104633, SGI0, SGI0, NULL};
const Segment * const SG23074[] = {SGI104634, SGI104635, SGI104636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104637, SGI0, SGI0, SGI0, SGI104638, SGI104638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104639, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23075[] = {SGI104640, SGI104641, SGI104642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104643, SGI104643, SGI0, SGI0, SGI104644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23076[] = {SGI104645, SGI104646, SGI104647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104648, SGI0, SGI0, SGI0, SGI104649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23077[] = {SGI104650, SGI0, SGI104651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104652, SGI0, SGI0, SGI0, SGI104653, SGI104653, SGI0, SGI0, SGI104654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23078[] = {SGI104655, SGI104656, SGI104657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104658, SGI104658, SGI0, SGI0, SGI104659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23079[] = {SGI104660, SGI104661, SGI104662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104663, SGI0, SGI0, SGI0, SGI104664, SGI104664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23080[] = {SGI104665, SGI104666, SGI104667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104668, SGI104668, SGI0, SGI0, SGI104669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23081[] = {SGI104670, SGI104671, SGI104672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104673, SGI104673, SGI0, SGI0, SGI104674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23082[] = {SGI104675, SGI0, SGI104676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104677, SGI104678, SGI104678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104679, NULL};
const Segment * const SG23083[] = {SGI104680, SGI0, SGI104681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104682, SGI104683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23084[] = {SGI104684, SGI104685, SGI104686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104687, SGI0, SGI0, SGI0, SGI104688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23085[] = {SGI104689, SGI104690, SGI104691, SGI0, SGI0, SGI0, SGI104692, SGI104693, SGI0, SGI104694, SGI0, SGI0, SGI0, SGI104695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23086[] = {SGI104696, SGI0, SGI104697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104698, SGI104699, SGI104699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104700, SGI0, NULL};
const Segment * const SG23087[] = {SGI104701, SGI0, SGI104702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104703, SGI104704, SGI104704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104705, SGI0, NULL};
const Segment * const SG23088[] = {SGI104706, SGI104707, SGI104708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104709, SGI0, SGI0, SGI0, SGI104710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23089[] = {SGI104711, SGI0, SGI104712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104713, SGI104714, SGI104714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23090[] = {SGI104715, SGI104716, SGI104717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104718, SGI0, SGI0, SGI0, SGI104719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23091[] = {SGI104720, SGI104721, SGI104722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104723, SGI0, SGI0, SGI0, SGI104724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23092[] = {SGI104725, SGI104726, SGI104727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104728, SGI104728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104729, SGI0, SGI0, NULL};
const Segment * const SG23093[] = {SGI104730, SGI0, SGI104731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104732, SGI104733, SGI104733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23094[] = {SGI104734, SGI0, SGI104735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104736, SGI104737, SGI104737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23095[] = {SGI104738, SGI0, SGI104739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104740, SGI104741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23096[] = {SGI104742, SGI104743, SGI104744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104745, SGI0, SGI104746, SGI104746, SGI0, SGI0, NULL};
const Segment * const SG23097[] = {SGI104747, SGI104748, SGI104749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104750, SGI0, SGI0, SGI0, SGI104751, SGI104751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23098[] = {SGI104752, SGI104753, SGI104754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104755, SGI104756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23099[] = {SGI104757, SGI104758, SGI104759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104760, SGI104760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23100[] = {SGI104761, SGI104762, SGI104763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104764, SGI0, SGI0, SGI0, SGI104765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23101[] = {SGI104766, SGI104767, SGI104768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104771, SGI0, SGI0, SGI0, SGI104772, SGI104772, SGI104773, SGI0, SGI104774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23102[] = {SGI104769, SGI0, SGI104770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104775, SGI104776, SGI104776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104777, NULL};
const Segment * const SG23103[] = {SGI104778, SGI104779, SGI104780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104781, SGI0, SGI0, SGI0, SGI104782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23104[] = {SGI104783, SGI104784, SGI104785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104786, SGI0, SGI0, SGI0, SGI104787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23105[] = {SGI0, SGI104788, SGI0, SGI104789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104790, SGI0, SGI0, SGI0, SGI104791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23106[] = {SGI0, SGI104792, SGI0, SGI104793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23107[] = {SGI104794, SGI0, SGI104795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104796, SGI104797, SGI104797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104798, NULL};
const Segment * const SG23108[] = {SGI104799, SGI0, SGI104800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104801, SGI104802, SGI104802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23109[] = {SGI104803, SGI104804, SGI104805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104806, SGI0, SGI0, SGI0, SGI104807, SGI104807, SGI0, SGI104808, SGI0, NULL};
const Segment * const SG23110[] = {SGI104809, SGI104810, SGI104811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104812, SGI0, SGI0, SGI0, SGI104813, SGI104813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104814, SGI0, NULL};
const Segment * const SG23111[] = {SGI104815, SGI0, SGI104816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104817, SGI104818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104819, SGI0, NULL};
const Segment * const SG23112[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104820, SGI0, SGI104821, SGI0, NULL};
const Segment * const SG23113[] = {SGI104822, SGI0, SGI104823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104824, SGI104825, SGI104825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104826, SGI0, NULL};
const Segment * const SG23114[] = {SGI104827, SGI104828, SGI104829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104830, SGI0, SGI0, SGI0, SGI104831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23115[] = {SGI104832, SGI104833, SGI104834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104835, SGI0, SGI0, SGI0, SGI104836, SGI104836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104837, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23116[] = {SGI104838, SGI104839, SGI104840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104841, SGI0, SGI0, SGI0, SGI104842, SGI104842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23117[] = {SGI104843, SGI0, SGI104844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104845, SGI104846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23118[] = {SGI0, SGI104847, SGI104848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104849, SGI0, SGI0, SGI0, SGI104850, SGI104850, SGI104851, SGI0, SGI104852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23119[] = {SGI104853, SGI104854, SGI104855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104856, SGI0, SGI0, SGI0, SGI104857, SGI104857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104858, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23120[] = {SGI104859, SGI0, SGI104860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104861, SGI104862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104863, SGI0, NULL};
const Segment * const SG23121[] = {SGI104864, SGI0, SGI104865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104866, SGI104867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104868, SGI0, NULL};
const Segment * const SG23122[] = {SGI104869, SGI0, SGI104870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104871, SGI104872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104873, SGI0, NULL};
const Segment * const SG23123[] = {SGI104874, SGI0, SGI104875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104876, SGI104877, SGI104877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23124[] = {SGI104878, SGI104879, SGI104880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104881, SGI0, SGI0, SGI0, SGI104882, SGI104882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23125[] = {SGI104883, SGI104884, SGI104885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104886, SGI0, SGI0, SGI0, SGI104887, SGI104887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104888, SGI0, NULL};
const Segment * const SG23126[] = {SGI104889, SGI104890, SGI104891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104892, SGI0, SGI0, SGI0, SGI104893, SGI104893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104894, SGI0, NULL};
const Segment * const SG23127[] = {SGI104895, SGI0, SGI104896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104897, SGI104898, SGI104898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23128[] = {SGI104899, SGI0, SGI104900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104901, SGI104902, SGI104902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23129[] = {SGI104903, SGI104904, SGI104905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104906, SGI0, SGI0, SGI0, SGI104907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23130[] = {SGI104908, SGI104909, SGI104910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104911, SGI0, SGI0, SGI0, SGI104912, SGI104912, SGI104913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23131[] = {SGI104914, SGI104915, SGI104916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104917, SGI0, SGI0, SGI0, SGI104918, SGI0, SGI104919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23132[] = {SGI104920, SGI0, SGI104921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104922, SGI104923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23133[] = {SGI104924, SGI104925, SGI104926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104927, SGI0, SGI0, SGI0, SGI104928, SGI104928, SGI104929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23134[] = {SGI104930, SGI104931, SGI104932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104933, SGI104933, SGI0, SGI0, SGI104934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23135[] = {SGI104935, SGI104936, SGI104937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104938, SGI0, SGI0, SGI0, SGI104939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23136[] = {SGI104940, SGI0, SGI104941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104942, SGI0, SGI0, SGI0, SGI104943, SGI104943, SGI0, SGI104944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23137[] = {SGI104945, SGI104946, SGI104947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104948, SGI0, SGI0, SGI0, SGI104949, SGI104950, SGI0, SGI104951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23138[] = {SGI104952, SGI104953, SGI104954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104955, SGI0, SGI104957, SGI104957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23139[] = {SGI104956, SGI104958, SGI104959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104960, SGI0, SGI0, SGI0, SGI104961, SGI0, SGI0, SGI0, SGI0, SGI104962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23140[] = {SGI104963, SGI104964, SGI104965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104966, SGI0, SGI0, SGI0, SGI104967, SGI0, SGI0, SGI0, SGI0, SGI104968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23141[] = {SGI104969, SGI0, SGI104970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104971, SGI104972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23142[] = {SGI104973, SGI0, SGI104974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104975, SGI104976, SGI104976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23143[] = {SGI104977, SGI0, SGI104978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104979, SGI104980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23144[] = {SGI104981, SGI104982, SGI104983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104984, SGI0, SGI0, SGI0, SGI104985, SGI104985, SGI0, SGI0, SGI0, SGI0, SGI104986, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23145[] = {SGI104987, SGI104988, SGI104989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104990, SGI0, SGI0, SGI0, SGI104991, SGI104991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23146[] = {SGI104992, SGI0, SGI104993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104994, SGI104995, SGI104995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23147[] = {SGI104996, SGI0, SGI104997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI104998, SGI104999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23148[] = {SGI105000, SGI105001, SGI105002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105003, SGI0, SGI0, SGI0, SGI105004, SGI105004, SGI0, SGI105005, SGI0, NULL};
const Segment * const SG23149[] = {SGI105006, SGI105007, SGI105008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105009, SGI0, SGI0, SGI0, SGI105010, SGI105010, SGI0, SGI105011, SGI0, NULL};
const Segment * const SG23150[] = {SGI105012, SGI0, SGI105013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105014, SGI105016, SGI105016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23151[] = {SGI105015, SGI105017, SGI105018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105019, SGI0, SGI0, SGI0, SGI105020, SGI105020, SGI0, SGI105021, SGI0, NULL};
const Segment * const SG23152[] = {SGI105022, SGI105023, SGI105024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105025, SGI0, SGI0, SGI0, SGI105026, SGI105026, SGI0, SGI105027, SGI0, NULL};
const Segment * const SG23153[] = {SGI105028, SGI105029, SGI105030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105031, SGI0, SGI0, SGI0, SGI105032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23154[] = {SGI105033, SGI0, SGI105034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23155[] = {SGI105035, SGI105036, SGI105037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105038, SGI0, SGI0, SGI0, SGI105039, SGI105039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23156[] = {SGI105040, SGI105041, SGI105042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105043, SGI0, SGI0, SGI0, SGI105044, SGI105044, SGI105045, SGI0, SGI105046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23157[] = {SGI105047, SGI105048, SGI105049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105050, SGI0, SGI0, SGI0, SGI105051, SGI105051, SGI105052, SGI0, SGI105053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23158[] = {SGI105054, SGI105055, SGI105056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105057, SGI0, SGI0, SGI0, SGI105058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23159[] = {SGI105059, SGI105060, SGI105061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105062, SGI0, SGI0, SGI0, SGI105063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23160[] = {SGI105064, SGI0, SGI105065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105066, SGI105067, SGI105067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105068, NULL};
const Segment * const SG23161[] = {SGI105069, SGI105070, SGI105071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105072, SGI0, SGI0, SGI0, SGI105073, SGI105073, SGI0, SGI105074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23162[] = {SGI105075, SGI0, SGI105076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105077, SGI105078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23163[] = {SGI105079, SGI105080, SGI105081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105082, SGI0, SGI0, SGI0, SGI105083, SGI105083, SGI0, SGI0, NULL};
const Segment * const SG23164[] = {SGI105084, SGI0, SGI105085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105086, SGI105087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23165[] = {SGI105088, SGI0, SGI105089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105090, SGI105091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23166[] = {SGI105092, SGI105093, SGI105094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105095, SGI0, SGI0, SGI0, SGI105096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23167[] = {SGI0, SGI105097, SGI0, SGI105098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23168[] = {SGI0, SGI105099, SGI0, SGI105100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23169[] = {SGI105101, SGI105102, SGI105103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105104, SGI0, SGI0, SGI0, SGI105105, SGI105105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105106, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23170[] = {SGI105107, SGI105108, SGI105109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105110, SGI0, SGI0, SGI0, SGI105111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23171[] = {SGI105112, SGI105113, SGI105114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105115, SGI0, SGI0, SGI0, SGI105116, SGI105116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23172[] = {SGI105117, SGI0, SGI105118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105119, SGI105120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105121, SGI0, NULL};
const Segment * const SG23173[] = {SGI105122, SGI105123, SGI105124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105125, SGI0, SGI0, SGI0, SGI105126, SGI105126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23174[] = {SGI105127, SGI0, SGI105128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105129, SGI105130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23175[] = {SGI105131, SGI105132, SGI105133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105134, SGI0, SGI0, SGI0, SGI105135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23176[] = {SGI105136, SGI105137, SGI105138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105139, SGI0, SGI0, SGI0, SGI105140, SGI105140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23177[] = {SGI105141, SGI105142, SGI105143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105144, SGI105144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23178[] = {SGI105145, SGI105146, SGI105147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105148, SGI0, SGI0, SGI0, SGI105149, SGI105149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23179[] = {SGI105150, SGI105151, SGI105152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105153, SGI0, SGI0, SGI0, SGI105154, SGI105154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23180[] = {SGI105155, SGI105156, SGI105157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105158, SGI0, SGI0, SGI0, SGI105159, SGI105159, SGI0, SGI105160, SGI0, NULL};
const Segment * const SG23181[] = {SGI105161, SGI0, SGI105162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105163, SGI105164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23182[] = {SGI105165, SGI105166, SGI105167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105168, SGI0, SGI0, SGI0, SGI105169, SGI105169, SGI105170, SGI0, SGI105171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23183[] = {SGI105172, SGI0, SGI105173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105174, SGI105175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23184[] = {SGI105176, SGI0, SGI105177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105178, SGI105179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23185[] = {SGI105180, SGI105181, SGI105182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105183, SGI105183, SGI0, SGI0, SGI105184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23186[] = {SGI105185, SGI105186, SGI105187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105188, SGI0, SGI0, SGI0, SGI105189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23187[] = {SGI105190, SGI0, SGI105191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105192, SGI105193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23188[] = {SGI105194, SGI105195, SGI105196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105197, SGI0, SGI0, SGI0, SGI105198, SGI105198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23189[] = {SGI105199, SGI0, SGI105200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105201, SGI105202, SGI105202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105203, SGI0, NULL};
const Segment * const SG23190[] = {SGI105204, SGI105205, SGI105206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105207, SGI105207, SGI0, SGI0, SGI105208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23191[] = {SGI105209, SGI105210, SGI105211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105212, SGI0, SGI0, SGI0, SGI105213, SGI105213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23192[] = {SGI105214, SGI0, SGI105215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105216, SGI105217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23193[] = {SGI105218, SGI105219, SGI105220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105221, SGI0, SGI0, SGI0, SGI105222, SGI105222, SGI105223, SGI0, SGI105224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23194[] = {SGI105225, SGI0, SGI105226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105227, SGI105228, SGI105228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23195[] = {SGI105229, SGI0, SGI105230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105231, SGI105232, SGI105232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23196[] = {SGI105233, SGI0, SGI105234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105235, SGI105236, SGI105236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23197[] = {SGI105237, SGI0, SGI105238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105239, SGI105240, SGI105240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23198[] = {SGI105241, SGI0, SGI105242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105243, SGI105244, SGI105244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23199[] = {SGI105245, SGI0, SGI105246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105247, SGI105248, SGI105248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23200[] = {SGI105249, SGI0, SGI105250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105251, SGI105252, SGI105252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23201[] = {SGI105253, SGI0, SGI105254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105255, SGI105256, SGI105256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105257, SGI0, NULL};
const Segment * const SG23202[] = {SGI105258, SGI0, SGI105259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105260, SGI105261, SGI105261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105262, SGI0, NULL};
const Segment * const SG23203[] = {SGI105263, SGI0, SGI105264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105265, SGI105266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105267, SGI0, NULL};
const Segment * const SG23204[] = {SGI105268, SGI0, SGI105269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105270, SGI105271, SGI105271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23205[] = {SGI105272, SGI0, SGI105273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105274, SGI105275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23206[] = {SGI105276, SGI105277, SGI105278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105279, SGI0, SGI0, SGI0, SGI105280, SGI105280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105281, SGI0, NULL};
const Segment * const SG23207[] = {SGI105282, SGI105283, SGI105284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105285, SGI0, SGI0, SGI0, SGI105286, SGI105286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23208[] = {SGI105287, SGI105288, SGI105289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105290, SGI0, SGI0, SGI0, SGI105291, SGI105291, SGI0, SGI105292, SGI0, NULL};
const Segment * const SG23209[] = {SGI105293, SGI105294, SGI105295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105296, SGI0, SGI0, SGI0, SGI105297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23210[] = {SGI105298, SGI105299, SGI105300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105301, SGI105301, SGI0, SGI0, SGI105302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23211[] = {SGI105303, SGI105304, SGI105305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105306, SGI105306, SGI0, SGI0, SGI105307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23212[] = {SGI105308, SGI105309, SGI105310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105311, SGI105311, SGI0, SGI0, SGI105312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23213[] = {SGI105313, SGI105314, SGI105315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105316, SGI105316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23214[] = {SGI105317, SGI0, SGI105318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105319, SGI105320, SGI105320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23215[] = {SGI105321, SGI0, SGI105322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105323, SGI0, SGI0, SGI0, SGI105324, SGI105324, SGI0, SGI0, SGI105325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23216[] = {SGI105326, SGI0, SGI105327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105328, SGI105329, SGI105329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23217[] = {SGI105330, SGI105331, SGI105332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105333, SGI105333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105334, SGI0, SGI0, NULL};
const Segment * const SG23218[] = {SGI105335, SGI105336, SGI105337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105338, SGI105338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105339, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23219[] = {SGI105340, SGI105341, SGI105342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105343, SGI0, SGI0, SGI0, SGI105344, SGI105344, SGI105345, SGI0, SGI105346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23220[] = {SGI105347, SGI0, SGI105348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105349, SGI105350, SGI105350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23221[] = {SGI105351, SGI105352, SGI105353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105354, SGI0, SGI0, SGI0, SGI105355, SGI105355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105356, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23222[] = {SGI105357, SGI105358, SGI105359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105360, SGI0, SGI0, SGI0, SGI105361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23223[] = {SGI105362, SGI0, SGI105363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105364, SGI105365, SGI105365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23224[] = {SGI105366, SGI105367, SGI105368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105369, SGI0, SGI0, SGI0, SGI105370, SGI105370, SGI105371, SGI0, SGI105372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23225[] = {SGI105373, SGI105374, SGI105375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105376, SGI0, SGI0, SGI0, SGI105377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23226[] = {SGI105378, SGI105379, SGI105380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105381, SGI0, SGI0, SGI0, SGI105382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23227[] = {SGI105383, SGI105384, SGI105385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105389, SGI0, SGI0, SGI0, SGI105391, SGI105391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23228[] = {SGI105386, SGI105387, SGI105388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105390, SGI0, SGI0, SGI0, SGI105392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23229[] = {SGI105393, SGI105394, SGI105395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105396, SGI105396, SGI0, SGI0, SGI105397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23230[] = {SGI105398, SGI105399, SGI105400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105401, SGI105401, SGI0, SGI0, SGI105402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23231[] = {SGI105403, SGI0, SGI105404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105405, SGI105406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23232[] = {SGI105407, SGI105408, SGI105409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105410, SGI0, SGI0, SGI0, SGI105411, SGI105411, SGI105412, SGI0, SGI105413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23233[] = {SGI105414, SGI105415, SGI105416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105417, SGI0, SGI0, SGI0, SGI105418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23234[] = {SGI105419, SGI105420, SGI105421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105422, SGI0, SGI0, SGI0, SGI105423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23235[] = {SGI105424, SGI0, SGI105425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105426, SGI105427, SGI105427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105428, SGI0, NULL};
const Segment * const SG23236[] = {SGI105429, SGI105430, SGI105431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105432, SGI105432, SGI0, SGI0, SGI105433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23237[] = {SGI105434, SGI105435, SGI105436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105437, SGI0, SGI0, SGI0, SGI105438, SGI105438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105439, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23238[] = {SGI105440, SGI0, SGI105441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105442, SGI105443, SGI105443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105444, SGI0, NULL};
const Segment * const SG23239[] = {SGI105445, SGI105446, SGI105447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105448, SGI0, SGI0, SGI0, SGI105449, SGI105449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105450, SGI0, NULL};
const Segment * const SG23240[] = {SGI105451, SGI105452, SGI105453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105454, SGI0, SGI0, SGI0, SGI105455, SGI105455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105456, SGI0, NULL};
const Segment * const SG23241[] = {SGI105457, SGI105458, SGI105459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105460, SGI0, SGI0, SGI0, SGI105461, SGI105461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23242[] = {SGI105462, SGI105463, SGI105464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105465, SGI105465, SGI0, SGI105466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23243[] = {SGI105467, SGI0, SGI105468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105469, SGI0, SGI0, SGI0, SGI105470, SGI105470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105471, SGI0, SGI0, NULL};
const Segment * const SG23244[] = {SGI105472, SGI0, SGI105473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105474, SGI0, SGI0, SGI0, SGI105475, SGI105475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105476, SGI0, SGI0, NULL};
const Segment * const SG23245[] = {SGI105477, SGI0, SGI105478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105479, SGI0, SGI0, SGI0, SGI105480, SGI105480, SGI0, SGI0, SGI105481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23246[] = {SGI105482, SGI0, SGI105483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105484, SGI105485, SGI105485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105486, NULL};
const Segment * const SG23247[] = {SGI105487, SGI0, SGI105488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105489, SGI105490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23248[] = {SGI105491, SGI0, SGI105492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105493, SGI105494, SGI105494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105495, SGI0, NULL};
const Segment * const SG23249[] = {SGI105496, SGI105497, SGI105498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105499, SGI0, SGI0, SGI0, SGI105500, SGI105500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105501, SGI0, NULL};
const Segment * const SG23250[] = {SGI105502, SGI105503, SGI105504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105505, SGI0, SGI0, SGI0, SGI105506, SGI105506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23251[] = {SGI105507, SGI0, SGI105508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105509, SGI105510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23252[] = {SGI105511, SGI0, SGI105512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105513, SGI0, SGI0, SGI0, SGI105514, SGI105514, SGI0, SGI105515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23253[] = {SGI105516, SGI0, SGI105517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105518, SGI0, SGI0, SGI0, SGI105519, SGI105519, SGI0, SGI105520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23254[] = {SGI105521, SGI0, SGI105522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105523, SGI0, SGI0, SGI0, SGI105524, SGI105524, SGI0, SGI105525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23255[] = {SGI105526, SGI0, SGI105527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105528, SGI0, SGI0, SGI0, SGI105529, SGI105529, SGI0, SGI105530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23256[] = {SGI105531, SGI0, SGI105532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105533, SGI0, SGI0, SGI0, SGI105534, SGI105534, SGI0, SGI105535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23257[] = {SGI105536, SGI0, SGI105537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105538, SGI105539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23258[] = {SGI105540, SGI0, SGI105541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105542, SGI105543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105544, SGI0, NULL};
const Segment * const SG23259[] = {SGI105545, SGI105546, SGI105547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105548, SGI0, SGI0, SGI0, SGI105549, SGI105549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105550, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23260[] = {SGI105551, SGI105552, SGI105553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105554, SGI0, SGI0, SGI0, SGI105555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23261[] = {SGI105556, SGI0, SGI105557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105558, SGI105559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23262[] = {SGI105560, SGI105561, SGI105562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105563, SGI0, SGI0, SGI0, SGI105564, SGI105564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23263[] = {SGI105565, SGI105566, SGI105567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105568, SGI0, SGI0, SGI0, SGI105569, SGI105569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23264[] = {SGI105570, SGI0, SGI105571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105572, SGI105573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23265[] = {SGI105574, SGI0, SGI105575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105576, SGI105577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105578, SGI0, NULL};
const Segment * const SG23266[] = {SGI105579, SGI0, SGI105580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105581, SGI105582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105583, SGI0, NULL};
const Segment * const SG23267[] = {SGI105584, SGI105585, SGI105586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105587, SGI0, SGI0, SGI0, SGI105588, SGI0, SGI0, SGI105589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23268[] = {SGI105590, SGI105591, SGI105592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105593, SGI0, SGI0, SGI0, SGI105594, SGI105594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23269[] = {SGI105595, SGI0, SGI105596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105597, SGI105598, SGI105598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23270[] = {SGI105599, SGI0, SGI105600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105601, SGI105602, SGI105602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23271[] = {SGI105603, SGI105604, SGI105605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105606, SGI0, SGI0, SGI0, SGI105607, SGI105607, SGI0, SGI0, SGI0, SGI105608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23272[] = {SGI105609, SGI105610, SGI105611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105612, SGI0, SGI0, SGI0, SGI105613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23273[] = {SGI105614, SGI105615, SGI105616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105617, SGI0, SGI0, SGI0, SGI105618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23274[] = {SGI105619, SGI105620, SGI105621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105622, SGI0, SGI0, SGI0, SGI105623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23275[] = {SGI105624, SGI105625, SGI105626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105627, SGI105627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105628, SGI0, SGI0, NULL};
const Segment * const SG23276[] = {SGI105629, SGI105630, SGI105631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105632, SGI105632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105633, SGI0, SGI0, NULL};
const Segment * const SG23277[] = {SGI105634, SGI105635, SGI105636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105637, SGI0, SGI0, SGI0, SGI105638, SGI105638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105639, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23278[] = {SGI105640, SGI105641, SGI105642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105643, SGI0, SGI0, SGI0, SGI105644, SGI105644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105645, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23279[] = {SGI105646, SGI105647, SGI105648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105649, SGI0, SGI0, SGI0, SGI105650, SGI105650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105651, SGI0, SGI0, NULL};
const Segment * const SG23280[] = {SGI105652, SGI105653, SGI105654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105655, SGI0, SGI0, SGI0, SGI105656, SGI105656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105657, SGI0, SGI0, NULL};
const Segment * const SG23281[] = {SGI105658, SGI105659, SGI105660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105661, SGI0, SGI0, SGI0, SGI105662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23282[] = {SGI105663, SGI105664, SGI105665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105666, SGI0, SGI0, SGI0, SGI105667, SGI105668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105669, SGI0, SGI0, NULL};
const Segment * const SG23283[] = {SGI105670, SGI105671, SGI105672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105673, SGI0, SGI0, SGI0, SGI105674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23284[] = {SGI105675, SGI105676, SGI105677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105678, SGI0, SGI0, SGI0, SGI105679, SGI105679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105680, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23285[] = {SGI105681, SGI105682, SGI105683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105684, SGI105684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105685, SGI0, SGI0, NULL};
const Segment * const SG23286[] = {SGI0, SGI105686, SGI0, SGI105687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23287[] = {SGI105688, SGI105689, SGI105690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105691, SGI0, SGI0, SGI0, SGI105692, SGI105692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23288[] = {SGI105693, SGI0, SGI105694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105695, SGI105696, SGI105696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23289[] = {SGI105697, SGI105698, SGI105699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105700, SGI0, SGI0, SGI0, SGI105701, SGI105701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105702, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23290[] = {SGI105703, SGI105704, SGI105705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105706, SGI0, SGI0, SGI0, SGI105707, SGI105707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23291[] = {SGI105708, SGI105709, SGI105710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105711, SGI105711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23292[] = {SGI105712, SGI105713, SGI105714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105715, SGI0, SGI0, SGI0, SGI105716, SGI105716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23293[] = {SGI105717, SGI105718, SGI105719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105720, SGI0, SGI0, SGI0, SGI105721, SGI105721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23294[] = {SGI105722, SGI105723, SGI105724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105725, SGI0, SGI0, SGI0, SGI105726, SGI105726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23295[] = {SGI105727, SGI105728, SGI105729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105730, SGI0, SGI0, SGI0, SGI105731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23296[] = {SGI0, SGI105732, SGI0, SGI105733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23297[] = {SGI0, SGI105734, SGI0, SGI105735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105736, SGI0, SGI0, SGI0, SGI105737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23298[] = {SGI105738, SGI0, SGI105739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23299[] = {SGI105740, SGI105741, SGI105742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105743, SGI0, SGI0, SGI0, SGI105744, SGI105744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105745, SGI0, NULL};
const Segment * const SG23300[] = {SGI105746, SGI0, SGI105747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105748, SGI105749, SGI105749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23301[] = {SGI105750, SGI105751, SGI105752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105753, SGI105753, SGI0, SGI0, SGI105754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23302[] = {SGI105755, SGI0, SGI105756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105757, SGI105758, SGI105758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23303[] = {SGI0, SGI0, SGI105759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105760, SGI0, SGI0, SGI0, SGI105761, SGI105761, SGI0, SGI0, SGI105762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23304[] = {SGI105763, SGI105764, SGI105765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105766, SGI0, SGI0, SGI0, SGI105767, SGI0, SGI0, SGI105768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23305[] = {SGI105769, SGI105770, SGI105771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105772, SGI0, SGI0, SGI0, SGI105773, SGI0, SGI0, SGI105774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23306[] = {SGI105775, SGI0, SGI105776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105777, SGI105778, SGI105778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23307[] = {SGI105779, SGI0, SGI105780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105781, SGI105782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105783, SGI0, NULL};
const Segment * const SG23308[] = {SGI105784, SGI0, SGI105785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105786, SGI105787, SGI105787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23309[] = {SGI105788, SGI105789, SGI105790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105791, SGI0, SGI0, SGI0, SGI105792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23310[] = {SGI105793, SGI105794, SGI105795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105796, SGI0, SGI0, SGI0, SGI105797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23311[] = {SGI105798, SGI0, SGI105799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105800, SGI105801, SGI105801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23312[] = {SGI105802, SGI0, SGI105803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105804, SGI105805, SGI105805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23313[] = {SGI105806, SGI105807, SGI105808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105809, SGI0, SGI0, SGI0, SGI105810, SGI105810, SGI105811, SGI0, SGI105812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23314[] = {SGI105813, SGI0, SGI105814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105815, SGI105816, SGI105816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23315[] = {SGI105817, SGI105818, SGI105819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105820, SGI0, SGI0, SGI0, SGI105821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23316[] = {SGI105822, SGI105823, SGI105824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105825, SGI0, SGI0, SGI0, SGI105826, SGI105826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23317[] = {SGI105827, SGI0, SGI105828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105829, SGI105830, SGI105830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23318[] = {SGI105831, SGI105832, SGI105833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105834, SGI0, SGI0, SGI0, SGI105835, SGI105835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23319[] = {SGI105836, SGI0, SGI105837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105838, SGI105839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23320[] = {SGI105840, SGI0, SGI105841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105842, SGI105843, SGI105843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105844, SGI0, NULL};
const Segment * const SG23321[] = {SGI105845, SGI0, SGI105846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105847, SGI105848, SGI105848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23322[] = {SGI105849, SGI105850, SGI105851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105852, SGI0, SGI0, SGI0, SGI105853, SGI105853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23323[] = {SGI105854, SGI105855, SGI105856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105857, SGI0, SGI0, SGI0, SGI105858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23324[] = {SGI105859, SGI105860, SGI105861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105862, SGI0, SGI0, SGI0, SGI105863, SGI105863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105864, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23325[] = {SGI105865, SGI105866, SGI105867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105868, SGI0, SGI0, SGI0, SGI105869, SGI105869, SGI0, SGI105870, SGI0, NULL};
const Segment * const SG23326[] = {SGI105871, SGI0, SGI105872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105873, SGI0, SGI0, SGI0, SGI105874, SGI105874, SGI0, SGI0, SGI105875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23327[] = {SGI105876, SGI105877, SGI105878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105879, SGI0, SGI0, SGI0, SGI105880, SGI105880, SGI0, SGI105881, SGI0, NULL};
const Segment * const SG23328[] = {SGI105882, SGI105883, SGI105884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105885, SGI0, SGI0, SGI0, SGI105886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23329[] = {SGI105887, SGI105888, SGI105889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105890, SGI0, SGI0, SGI0, SGI105891, SGI0, SGI105892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23330[] = {SGI105893, SGI0, SGI105894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105895, SGI105896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105897, SGI0, NULL};
const Segment * const SG23331[] = {SGI105898, SGI105899, SGI105900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105901, SGI0, SGI0, SGI0, SGI105902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23332[] = {SGI105903, SGI0, SGI105904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105905, SGI0, SGI0, SGI0, SGI105906, SGI105906, SGI0, SGI0, SGI105907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23333[] = {SGI105908, SGI105909, SGI105910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105911, SGI0, SGI0, SGI0, SGI105912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23334[] = {SGI105913, SGI0, SGI105914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105915, SGI0, SGI0, SGI0, SGI105916, SGI105916, SGI0, SGI0, SGI105917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23335[] = {SGI105918, SGI105919, SGI105920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105921, SGI0, SGI0, SGI0, SGI105922, SGI105922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105923, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23336[] = {SGI105924, SGI105925, SGI105926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105927, SGI105927, SGI0, SGI0, SGI105928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23337[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105929, SGI0, SGI105930, SGI0, NULL};
const Segment * const SG23338[] = {SGI105931, SGI0, SGI105932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105933, SGI105934, SGI105934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23339[] = {SGI105935, SGI0, SGI105936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105937, SGI105938, SGI105938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23340[] = {SGI105939, SGI0, SGI105940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105941, SGI105942, SGI105942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23341[] = {SGI105943, SGI105944, SGI105945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105946, SGI105946, SGI0, SGI0, SGI105947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23342[] = {SGI105948, SGI105949, SGI105950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105951, SGI0, SGI0, SGI0, SGI105952, SGI105952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23343[] = {SGI105953, SGI105954, SGI105955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105956, SGI0, SGI0, SGI0, SGI105957, SGI105957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23344[] = {SGI105958, SGI105959, SGI105960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105961, SGI0, SGI0, SGI0, SGI105962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23345[] = {SGI105963, SGI0, SGI105964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105965, SGI105966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23346[] = {SGI105967, SGI105968, SGI105969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105970, SGI0, SGI0, SGI0, SGI105971, SGI105971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23347[] = {SGI105972, SGI0, SGI105973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105974, SGI105975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105976, SGI0, NULL};
const Segment * const SG23348[] = {SGI105977, SGI105978, SGI105979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105980, SGI105980, SGI0, SGI0, SGI105981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23349[] = {SGI105982, SGI105983, SGI105984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105985, SGI0, SGI0, SGI0, SGI105986, SGI105986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23350[] = {SGI105987, SGI0, SGI105988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105989, SGI105990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23351[] = {SGI105991, SGI105992, SGI105993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105994, SGI0, SGI0, SGI0, SGI105995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23352[] = {SGI105996, SGI0, SGI105997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI105998, SGI105999, SGI105999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23353[] = {SGI106000, SGI0, SGI106001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106002, SGI106003, SGI106003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23354[] = {SGI106004, SGI106005, SGI106006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106007, SGI0, SGI0, SGI0, SGI106008, SGI106008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106009, SGI0, NULL};
const Segment * const SG23355[] = {SGI106010, SGI106011, SGI106012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106013, SGI0, SGI0, SGI0, SGI106014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23356[] = {SGI106015, SGI106016, SGI106017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106018, SGI0, SGI0, SGI0, SGI106019, SGI106019, SGI106020, SGI0, SGI106021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23357[] = {SGI106022, SGI106023, SGI106024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106025, SGI0, SGI0, SGI0, SGI106026, SGI106026, SGI106027, SGI0, SGI106028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23358[] = {SGI106029, SGI106030, SGI106031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106032, SGI0, SGI0, SGI0, SGI106033, SGI106033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23359[] = {SGI106034, SGI0, SGI106035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23360[] = {SGI106036, SGI0, SGI106037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106038, SGI106039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23361[] = {SGI106040, SGI106041, SGI106042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106043, SGI0, SGI0, SGI0, SGI106044, SGI106044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23362[] = {SGI106045, SGI106046, SGI106047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106048, SGI106048, SGI0, SGI0, SGI106049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23363[] = {SGI106050, SGI0, SGI106051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106052, SGI106053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23364[] = {SGI106054, SGI0, SGI106055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106056, SGI106057, SGI106057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23365[] = {SGI106058, SGI0, SGI106059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106060, SGI106061, SGI106061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23366[] = {SGI106062, SGI0, SGI106063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106064, SGI106065, SGI106065, SGI0, SGI0, NULL};
const Segment * const SG23367[] = {SGI106066, SGI106067, SGI106068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106069, SGI0, SGI0, SGI0, SGI106070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23368[] = {SGI106071, SGI106072, SGI106073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106074, SGI0, SGI0, SGI0, SGI106075, SGI106075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23369[] = {SGI106076, SGI0, SGI106077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106078, SGI106079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23370[] = {SGI106080, SGI106081, SGI106082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106083, SGI0, SGI0, SGI0, SGI106084, SGI106084, SGI106085, SGI0, SGI106086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23371[] = {SGI106087, SGI106088, SGI106089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106090, SGI0, SGI0, SGI0, SGI106091, SGI106091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106092, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23372[] = {SGI106093, SGI106094, SGI106095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106096, SGI0, SGI0, SGI0, SGI106097, SGI106097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23373[] = {SGI106098, SGI106099, SGI106100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106101, SGI0, SGI0, SGI0, SGI106102, SGI106102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106103, SGI0, NULL};
const Segment * const SG23374[] = {SGI106104, SGI0, SGI106105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106106, SGI106107, SGI106107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106108, SGI0, NULL};
const Segment * const SG23375[] = {SGI106109, SGI0, SGI106110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106111, SGI106112, SGI106112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106113, SGI0, NULL};
const Segment * const SG23376[] = {SGI106114, SGI106115, SGI106116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106117, SGI0, SGI0, SGI0, SGI106118, SGI106118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23377[] = {SGI106119, SGI106120, SGI106121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106122, SGI0, SGI0, SGI0, SGI106123, SGI106123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23378[] = {SGI106124, SGI0, SGI106125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106126, SGI106127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23379[] = {SGI106128, SGI0, SGI106129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106130, SGI106131, SGI106131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23380[] = {SGI106132, SGI106133, SGI106134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106135, SGI0, SGI0, SGI0, SGI106136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23381[] = {SGI106137, SGI0, SGI106138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23382[] = {SGI106139, SGI0, SGI106140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106141, SGI106142, SGI106142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23383[] = {SGI106143, SGI0, SGI106144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106145, SGI106146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23384[] = {SGI106147, SGI0, SGI106148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106149, SGI0, SGI0, SGI0, SGI106150, SGI106150, SGI0, SGI0, SGI106151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23385[] = {SGI106152, SGI0, SGI106153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106154, SGI0, SGI0, SGI0, SGI106155, SGI106155, SGI0, SGI0, SGI106156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23386[] = {SGI106157, SGI0, SGI106158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106159, SGI0, SGI0, SGI0, SGI106160, SGI106160, SGI0, SGI0, SGI106161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23387[] = {SGI106162, SGI106163, SGI106164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106165, SGI106165, SGI0, SGI0, SGI106166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23388[] = {SGI106167, SGI106168, SGI106169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106170, SGI0, SGI0, SGI0, SGI106171, SGI106171, SGI0, SGI106172, SGI0, NULL};
const Segment * const SG23389[] = {SGI106173, SGI106174, SGI106175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106176, SGI0, SGI0, SGI0, SGI106177, SGI106177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23390[] = {SGI0, SGI106178, SGI0, SGI106179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106180, SGI0, SGI0, SGI0, SGI106181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23391[] = {SGI0, SGI106182, SGI0, SGI106183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23392[] = {SGI106184, SGI106185, SGI106186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106187, SGI0, SGI0, SGI0, SGI106188, SGI0, SGI0, SGI106189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23393[] = {SGI106190, SGI0, SGI106191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106192, SGI106193, SGI106193, SGI0, SGI0, NULL};
const Segment * const SG23394[] = {SGI106194, SGI106195, SGI106196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106197, SGI106197, SGI0, SGI0, SGI106198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23395[] = {SGI106199, SGI0, SGI106200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106201, SGI106202, SGI106202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106203, SGI0, NULL};
const Segment * const SG23396[] = {SGI106204, SGI0, SGI106205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106206, SGI106207, SGI106207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23397[] = {SGI106208, SGI106209, SGI106210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106211, SGI0, SGI0, SGI0, SGI106212, SGI106212, SGI106213, SGI0, SGI106214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23398[] = {SGI106215, SGI106216, SGI106217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106218, SGI0, SGI0, SGI0, SGI106219, SGI106219, SGI106220, SGI0, SGI106221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23399[] = {SGI106222, SGI0, SGI106223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106224, SGI106225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23400[] = {SGI106226, SGI0, SGI106227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23401[] = {SGI106228, SGI106229, SGI106230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106231, SGI0, SGI0, SGI0, SGI106232, SGI106232, SGI106233, SGI0, SGI106234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23402[] = {SGI106235, SGI106236, SGI106237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106238, SGI0, SGI0, SGI0, SGI106239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23403[] = {SGI106240, SGI0, SGI106241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106242, SGI106243, SGI106243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23404[] = {SGI106244, SGI106245, SGI106246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106247, SGI0, SGI0, SGI0, SGI106248, SGI106248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106249, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23405[] = {SGI106250, SGI0, SGI106251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106252, SGI106253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23406[] = {SGI106254, SGI0, SGI106255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106256, SGI106257, SGI106257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23407[] = {SGI106258, SGI106259, SGI106260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106261, SGI0, SGI0, SGI0, SGI106262, SGI106262, SGI0, SGI106263, SGI0, NULL};
const Segment * const SG23408[] = {SGI106264, SGI106265, SGI106266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106267, SGI0, SGI0, SGI0, SGI106268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23409[] = {SGI106269, SGI106270, SGI106271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106272, SGI0, SGI0, SGI0, SGI106273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23410[] = {SGI106274, SGI0, SGI106275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106276, SGI106277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23411[] = {SGI106278, SGI106279, SGI106280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106281, SGI0, SGI0, SGI0, SGI106282, SGI106282, SGI106283, SGI0, SGI106284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23412[] = {SGI106285, SGI0, SGI106286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106287, SGI106288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23413[] = {SGI106289, SGI0, SGI106290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106291, SGI106292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23414[] = {SGI106293, SGI106294, SGI106295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106296, SGI0, SGI0, SGI0, SGI106297, SGI106297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23415[] = {SGI106298, SGI0, SGI106299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106300, SGI106301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23416[] = {SGI106302, SGI0, SGI106303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106304, SGI106305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23417[] = {SGI0, SGI0, SGI106306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106307, SGI0, SGI0, SGI0, SGI106308, SGI106308, SGI0, SGI0, SGI106309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23418[] = {SGI106310, SGI106311, SGI106312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106313, SGI0, SGI0, SGI0, SGI106314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23419[] = {SGI106315, SGI106316, SGI106317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106318, SGI0, SGI0, SGI0, SGI106319, SGI106319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23420[] = {SGI106320, SGI0, SGI106321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106322, SGI106323, SGI106323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23421[] = {SGI106324, SGI0, SGI106325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106326, SGI106327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106328, SGI0, NULL};
const Segment * const SG23422[] = {SGI106329, SGI0, SGI106330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106331, SGI106332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106333, SGI0, NULL};
const Segment * const SG23423[] = {SGI106334, SGI0, SGI106335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106336, SGI106337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23424[] = {SGI106338, SGI0, SGI106339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106340, SGI106341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23425[] = {SGI106342, SGI0, SGI106343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106344, SGI106345, SGI106345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23426[] = {SGI106346, SGI0, SGI106347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106348, SGI106349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23427[] = {SGI0, SGI106350, SGI0, SGI106351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23428[] = {SGI0, SGI106352, SGI0, SGI106353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106354, SGI0, SGI0, SGI0, SGI106355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23429[] = {SGI106356, SGI0, SGI106357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106358, SGI106359, SGI106359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23430[] = {SGI106360, SGI106361, SGI106362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106363, SGI0, SGI0, SGI0, SGI106364, SGI0, SGI0, SGI0, SGI0, SGI106365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23431[] = {SGI106366, SGI106367, SGI106368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106369, SGI0, SGI0, SGI0, SGI106370, SGI106370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106371, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23432[] = {SGI106372, SGI0, SGI106373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106374, SGI106375, SGI106375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106376, SGI0, NULL};
const Segment * const SG23433[] = {SGI106377, SGI0, SGI106378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106379, SGI106380, SGI106380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106381, SGI0, NULL};
const Segment * const SG23434[] = {SGI106382, SGI0, SGI106383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106384, SGI106385, SGI106385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23435[] = {SGI106386, SGI106387, SGI106388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106389, SGI0, SGI0, SGI0, SGI106390, SGI106390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23436[] = {SGI106391, SGI106392, SGI106393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106394, SGI0, SGI0, SGI0, SGI106395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23437[] = {SGI106396, SGI106397, SGI106398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106399, SGI0, SGI0, SGI0, SGI106400, SGI106400, SGI106401, SGI0, SGI106402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23438[] = {SGI106403, SGI106404, SGI106405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106406, SGI0, SGI0, SGI0, SGI106407, SGI106407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23439[] = {SGI106408, SGI106409, SGI106410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106411, SGI0, SGI0, SGI0, SGI106412, SGI106412, SGI106413, SGI0, SGI106414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23440[] = {SGI106415, SGI106416, SGI106417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106418, SGI0, SGI0, SGI0, SGI106419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23441[] = {SGI106420, SGI0, SGI106421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106422, SGI106423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23442[] = {SGI106424, SGI106425, SGI106426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106427, SGI0, SGI0, SGI0, SGI106428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23443[] = {SGI106429, SGI106430, SGI106431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106432, SGI106432, SGI0, SGI0, SGI106433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23444[] = {SGI106434, SGI106435, SGI106436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106437, SGI0, SGI0, SGI0, SGI106438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23445[] = {SGI106439, SGI0, SGI106440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106441, SGI106442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23446[] = {SGI106443, SGI106444, SGI106445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106446, SGI0, SGI0, SGI0, SGI106447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23447[] = {SGI106448, SGI106449, SGI106450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106451, SGI106451, SGI0, SGI0, SGI106452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23448[] = {SGI106453, SGI106454, SGI106455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106456, SGI106456, SGI0, SGI0, SGI106457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23449[] = {SGI106458, SGI0, SGI106459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106460, SGI106461, SGI106461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106462, SGI0, NULL};
const Segment * const SG23450[] = {SGI106463, SGI106464, SGI106465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106466, SGI0, SGI0, SGI0, SGI106467, SGI106467, SGI106468, SGI0, SGI106469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23451[] = {SGI106470, SGI106471, SGI106472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106473, SGI0, SGI0, SGI0, SGI106474, SGI106474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23452[] = {SGI106475, SGI0, SGI106476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106477, SGI0, SGI0, SGI0, SGI106478, SGI106478, SGI0, SGI0, SGI106479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23453[] = {SGI0, SGI106480, SGI0, SGI106481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23454[] = {SGI106482, SGI106483, SGI106484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106485, SGI0, SGI0, SGI0, SGI106486, SGI106486, SGI106487, SGI0, SGI106488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23455[] = {SGI106489, SGI106490, SGI106491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106492, SGI0, SGI0, SGI0, SGI106493, SGI106493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106494, SGI0, NULL};
const Segment * const SG23456[] = {SGI106495, SGI106496, SGI106497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106498, SGI0, SGI0, SGI0, SGI106499, SGI106499, SGI0, SGI106500, SGI0, NULL};
const Segment * const SG23457[] = {SGI106501, SGI0, SGI106502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106503, SGI106504, SGI106504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23458[] = {SGI106505, SGI0, SGI106506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23459[] = {SGI106507, SGI0, SGI106508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23460[] = {SGI106509, SGI0, SGI106510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23461[] = {SGI106511, SGI0, SGI106512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23462[] = {SGI106513, SGI0, SGI106514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106515, SGI0, SGI0, SGI0, SGI106516, SGI106516, SGI0, SGI106517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23463[] = {SGI106518, SGI106519, SGI106520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106521, SGI0, SGI0, SGI0, SGI106522, SGI106522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23464[] = {SGI106523, SGI0, SGI106524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106525, SGI0, SGI0, SGI0, SGI106526, SGI106526, SGI0, SGI106527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23465[] = {SGI106528, SGI0, SGI106529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106530, SGI106531, SGI106531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23466[] = {SGI106532, SGI106533, SGI106534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106535, SGI0, SGI0, SGI0, SGI106536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106537, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23467[] = {SGI106538, SGI0, SGI106539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106540, SGI106541, SGI106541, SGI0, SGI0, NULL};
const Segment * const SG23468[] = {SGI106542, SGI0, SGI106543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106544, SGI106545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23469[] = {SGI106546, SGI106547, SGI106548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106549, SGI106549, SGI0, SGI0, SGI106550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23470[] = {SGI106551, SGI106552, SGI106553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106554, SGI106554, SGI0, SGI0, SGI106555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23471[] = {SGI106556, SGI0, SGI106557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106558, SGI106559, SGI106559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23472[] = {SGI106560, SGI106561, SGI106562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106563, SGI0, SGI0, SGI0, SGI106564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23473[] = {SGI106565, SGI106566, SGI106567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106568, SGI106568, SGI0, SGI106569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23474[] = {SGI106570, SGI0, SGI106571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106572, SGI0, SGI0, SGI0, SGI106573, SGI106573, SGI0, SGI106574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23475[] = {SGI106575, SGI0, SGI106576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106577, SGI106578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23476[] = {SGI106579, SGI0, SGI106580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106581, SGI106582, SGI106582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23477[] = {SGI106583, SGI0, SGI106584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106585, SGI0, SGI0, SGI0, SGI106586, SGI106586, SGI0, SGI0, SGI106587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23478[] = {SGI106588, SGI0, SGI106589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106592, SGI106593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23479[] = {SGI106590, SGI106591, SGI106594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106595, SGI0, SGI0, SGI0, SGI106596, SGI106596, SGI106597, SGI0, SGI106598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23480[] = {SGI106599, SGI0, SGI106600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106601, SGI0, SGI0, SGI0, SGI106602, SGI106602, SGI0, SGI0, SGI106603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23481[] = {SGI106604, SGI106605, SGI106606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106607, SGI0, SGI0, SGI0, SGI106608, SGI106608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23482[] = {SGI106609, SGI0, SGI106610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106611, SGI0, SGI0, SGI0, SGI106612, SGI106612, SGI0, SGI0, SGI106613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23483[] = {SGI106614, SGI106615, SGI106616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106617, SGI0, SGI0, SGI0, SGI106618, SGI106618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23484[] = {SGI106619, SGI0, SGI106620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106621, SGI106622, SGI106622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23485[] = {SGI106623, SGI0, SGI106624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106625, SGI106626, SGI106626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23486[] = {SGI106627, SGI0, SGI106628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106629, SGI106630, SGI106630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23487[] = {SGI106631, SGI0, SGI106632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106633, SGI106634, SGI106634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106635, SGI0, NULL};
const Segment * const SG23488[] = {SGI106636, SGI106637, SGI106638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106639, SGI0, SGI0, SGI0, SGI106640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23489[] = {SGI106641, SGI106642, SGI106643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106644, SGI0, SGI0, SGI0, SGI106645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23490[] = {SGI106646, SGI0, SGI106647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106648, SGI106649, SGI106649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23491[] = {SGI106650, SGI106651, SGI106652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106653, SGI0, SGI0, SGI0, SGI106654, SGI106654, SGI0, SGI106655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23492[] = {SGI106656, SGI0, SGI106657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106658, SGI0, SGI0, SGI0, SGI106660, SGI106660, SGI0, SGI106662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23493[] = {SGI106659, SGI106661, SGI106663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106664, SGI0, SGI0, SGI0, SGI106665, SGI106665, SGI0, SGI106666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23494[] = {SGI106667, SGI106668, SGI106669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106670, SGI0, SGI0, SGI0, SGI106671, SGI0, SGI0, SGI0, SGI106672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23495[] = {SGI106673, SGI106674, SGI106675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106676, SGI0, SGI0, SGI0, SGI106677, SGI0, SGI0, SGI0, SGI0, SGI106678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23496[] = {SGI106679, SGI106680, SGI106681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106684, SGI0, SGI0, SGI0, SGI106686, SGI106686, SGI106687, SGI0, SGI106689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23497[] = {SGI106682, SGI0, SGI106683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106685, SGI0, SGI0, SGI0, SGI106688, SGI106688, SGI0, SGI106690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23498[] = {SGI106691, SGI0, SGI106692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106693, SGI106694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23499[] = {SGI106695, SGI106696, SGI106697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106698, SGI0, SGI0, SGI0, SGI106699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106700, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23500[] = {SGI106701, SGI0, SGI106702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106703, SGI106704, SGI106704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23501[] = {SGI106705, SGI0, SGI106706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106707, SGI106708, SGI106708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23502[] = {SGI106709, SGI0, SGI106710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106711, SGI0, SGI0, SGI0, SGI106712, SGI106712, SGI0, SGI0, SGI106713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23503[] = {SGI106714, SGI0, SGI106715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106716, SGI106717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23504[] = {SGI106718, SGI0, SGI106719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23505[] = {SGI106720, SGI0, SGI106721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23506[] = {SGI106722, SGI106723, SGI106724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106726, SGI0, SGI0, SGI0, SGI106728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106729, SGI0, SGI0, NULL};
const Segment * const SG23507[] = {SGI106725, SGI0, SGI106727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106730, SGI0, SGI0, SGI0, SGI106731, SGI106731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106732, SGI0, SGI0, NULL};
const Segment * const SG23508[] = {SGI106733, SGI0, SGI106734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106735, SGI0, SGI0, SGI0, SGI106736, SGI106736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106737, SGI0, SGI0, NULL};
const Segment * const SG23509[] = {SGI106738, SGI106739, SGI106740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106741, SGI0, SGI0, SGI0, SGI106742, SGI106742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23510[] = {SGI106743, SGI106744, SGI106745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106746, SGI0, SGI0, SGI0, SGI106747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23511[] = {SGI106748, SGI0, SGI106749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106750, SGI106751, SGI106751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23512[] = {SGI106752, SGI0, SGI106753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106754, SGI106755, SGI106755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23513[] = {SGI106756, SGI0, SGI106757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106758, SGI106759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23514[] = {SGI106760, SGI0, SGI106761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106762, SGI106763, SGI106763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23515[] = {SGI106764, SGI0, SGI106765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106766, SGI106767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23516[] = {SGI106768, SGI106769, SGI106770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106771, SGI0, SGI0, SGI0, SGI106772, SGI0, SGI0, SGI0, SGI106773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23517[] = {SGI106774, SGI0, SGI106775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106776, SGI106777, SGI106777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23518[] = {SGI106778, SGI106779, SGI106780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106782, SGI0, SGI0, SGI0, SGI106785, SGI106785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106786, SGI0, SGI0, NULL};
const Segment * const SG23519[] = {SGI106781, SGI106783, SGI106784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106787, SGI106787, SGI0, SGI106788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23520[] = {SGI106789, SGI0, SGI106790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106791, SGI106792, SGI106792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23521[] = {SGI106793, SGI0, SGI106794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106795, SGI106796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106797, SGI0, NULL};
const Segment * const SG23522[] = {SGI106798, SGI0, SGI106799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106800, SGI106801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106802, SGI0, NULL};
const Segment * const SG23523[] = {SGI106803, SGI0, SGI106804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106805, SGI106806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23524[] = {SGI106807, SGI0, SGI106808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106809, SGI106810, SGI106810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23525[] = {SGI106811, SGI0, SGI106812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106813, SGI106814, SGI106814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23526[] = {SGI106815, SGI0, SGI106816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106817, SGI106818, SGI106818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23527[] = {SGI106819, SGI106820, SGI106821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106822, SGI0, SGI0, SGI0, SGI106823, SGI106823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23528[] = {SGI106824, SGI106825, SGI106826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106827, SGI0, SGI0, SGI0, SGI106828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23529[] = {SGI106829, SGI0, SGI106830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106831, SGI106832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23530[] = {SGI106833, SGI0, SGI106834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106835, SGI106836, SGI106836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23531[] = {SGI106837, SGI106838, SGI106839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106840, SGI0, SGI0, SGI0, SGI106841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23532[] = {SGI106842, SGI106843, SGI106844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106845, SGI0, SGI0, SGI0, SGI106846, SGI106846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23533[] = {SGI106847, SGI106848, SGI106849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106850, SGI0, SGI0, SGI0, SGI106851, SGI106851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23534[] = {SGI106852, SGI106853, SGI106854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106855, SGI0, SGI0, SGI0, SGI106856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23535[] = {SGI106857, SGI106858, SGI106859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106860, SGI0, SGI0, SGI0, SGI106861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23536[] = {SGI0, SGI0, SGI106862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106863, SGI106864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23537[] = {SGI106865, SGI0, SGI106866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106867, SGI106868, SGI106868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23538[] = {SGI106869, SGI0, SGI106870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106871, SGI0, SGI0, SGI0, SGI106872, SGI106872, SGI0, SGI0, SGI106873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23539[] = {SGI106874, SGI0, SGI106875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106876, SGI0, SGI0, SGI0, SGI106877, SGI106877, SGI0, SGI0, SGI106878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23540[] = {SGI106879, SGI0, SGI106880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106881, SGI0, SGI0, SGI0, SGI106882, SGI106882, SGI0, SGI0, SGI106883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23541[] = {SGI106884, SGI0, SGI106885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106886, SGI106887, SGI106887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23542[] = {SGI106888, SGI106889, SGI106890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106891, SGI0, SGI0, SGI0, SGI106892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106893, SGI0, SGI0, NULL};
const Segment * const SG23543[] = {SGI106894, SGI106895, SGI106896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106897, SGI0, SGI0, SGI0, SGI106898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106899, SGI0, SGI0, NULL};
const Segment * const SG23544[] = {SGI106900, SGI106901, SGI106902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106903, SGI0, SGI0, SGI0, SGI106904, SGI106904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106905, SGI0, SGI0, NULL};
const Segment * const SG23545[] = {SGI106906, SGI0, SGI106907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106908, SGI106909, SGI106909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23546[] = {SGI106910, SGI0, SGI106911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106912, SGI106913, SGI106913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106914, SGI0, NULL};
const Segment * const SG23547[] = {SGI106915, SGI0, SGI106916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106917, SGI106918, SGI106918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23548[] = {SGI106919, SGI106920, SGI106921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106922, SGI0, SGI0, SGI0, SGI106923, SGI106924, SGI0, SGI0, SGI106925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23549[] = {SGI106926, SGI106927, SGI106928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106929, SGI0, SGI0, SGI0, SGI106930, SGI106930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23550[] = {SGI106931, SGI0, SGI106932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106933, SGI0, SGI0, SGI0, SGI106934, SGI106934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106935, SGI0, SGI0, NULL};
const Segment * const SG23551[] = {SGI106936, SGI0, SGI106937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106938, SGI106939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23552[] = {SGI106940, SGI0, SGI106941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106942, SGI0, SGI0, SGI0, SGI106943, SGI106943, SGI0, SGI0, SGI106944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23553[] = {SGI106945, SGI0, SGI106946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106947, SGI106948, SGI106948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23554[] = {SGI106949, SGI106950, SGI106951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106952, SGI106952, SGI0, SGI0, SGI106953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23555[] = {SGI106954, SGI106955, SGI106956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106957, SGI0, SGI0, SGI0, SGI106958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106959, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23556[] = {SGI106960, SGI106961, SGI106962, SGI106963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106964, SGI0, SGI0, SGI0, SGI106965, SGI106966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23557[] = {SGI106967, SGI106968, SGI106969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106970, SGI0, SGI0, SGI0, SGI106971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23558[] = {SGI106973, SGI106975, SGI106976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106978, SGI0, SGI0, SGI0, SGI106980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23559[] = {SGI106972, SGI0, SGI106974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106977, SGI0, SGI0, SGI0, SGI106979, SGI106979, SGI0, SGI106981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23560[] = {SGI106982, SGI0, SGI106983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106984, SGI0, SGI0, SGI0, SGI106985, SGI106985, SGI0, SGI106986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23561[] = {SGI106987, SGI0, SGI106988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106989, SGI0, SGI0, SGI0, SGI106990, SGI106990, SGI0, SGI106991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23562[] = {SGI106992, SGI0, SGI106993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106994, SGI0, SGI0, SGI0, SGI106995, SGI106995, SGI0, SGI0, SGI106996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23563[] = {SGI106997, SGI0, SGI106998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI106999, SGI107000, SGI107000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23564[] = {SGI107001, SGI0, SGI107002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107003, SGI107004, SGI107004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23565[] = {SGI107005, SGI107006, SGI107007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107008, SGI0, SGI0, SGI0, SGI107009, SGI107009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23566[] = {SGI107010, SGI107011, SGI107012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107013, SGI0, SGI0, SGI0, SGI107014, SGI107014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23567[] = {SGI107015, SGI107016, SGI107017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107018, SGI107018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23568[] = {SGI107019, SGI107020, SGI107021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107022, SGI107022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23569[] = {SGI107023, SGI107024, SGI107025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107026, SGI0, SGI0, SGI0, SGI107027, SGI107027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23570[] = {SGI107028, SGI0, SGI107029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23571[] = {SGI107030, SGI107031, SGI107032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107033, SGI0, SGI0, SGI0, SGI107034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23572[] = {SGI107035, SGI107036, SGI107037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107038, SGI0, SGI0, SGI0, SGI107039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23573[] = {SGI107040, SGI107041, SGI107042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107043, SGI0, SGI0, SGI0, SGI107044, SGI107044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23574[] = {SGI107045, SGI0, SGI107046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107047, SGI107048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23575[] = {SGI107049, SGI107050, SGI107051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107052, SGI0, SGI0, SGI0, SGI107053, SGI107053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23576[] = {SGI107054, SGI0, SGI107055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107056, SGI107057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23577[] = {SGI107058, SGI107059, SGI107060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107061, SGI107061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23578[] = {SGI107062, SGI107063, SGI107064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107065, SGI0, SGI0, SGI0, SGI107066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23579[] = {SGI107067, SGI107068, SGI107069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107070, SGI0, SGI0, SGI0, SGI107071, SGI107071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23580[] = {SGI107072, SGI107073, SGI107074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107075, SGI0, SGI0, SGI0, SGI107076, SGI107076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23581[] = {SGI107077, SGI107078, SGI107079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107080, SGI0, SGI0, SGI0, SGI107081, SGI107081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23582[] = {SGI107082, SGI107083, SGI107084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107085, SGI107085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23583[] = {SGI107086, SGI107087, SGI107088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107089, SGI0, SGI0, SGI0, SGI107090, SGI107090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23584[] = {SGI107091, SGI107092, SGI107093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107094, SGI0, SGI0, SGI0, SGI107095, SGI107095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23585[] = {SGI107096, SGI107097, SGI107098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107099, SGI0, SGI0, SGI0, SGI107100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23586[] = {SGI107101, SGI0, SGI107102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23587[] = {SGI107103, SGI107104, SGI107105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107106, SGI0, SGI0, SGI0, SGI107107, SGI107107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23588[] = {SGI107108, SGI107109, SGI107110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107111, SGI0, SGI0, SGI0, SGI107112, SGI107112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23589[] = {SGI107113, SGI107114, SGI107115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107116, SGI0, SGI0, SGI0, SGI107117, SGI107117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23590[] = {SGI107118, SGI0, SGI107119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107120, SGI0, SGI0, SGI0, SGI107121, SGI107121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23591[] = {SGI107122, SGI107123, SGI107124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107125, SGI0, SGI0, SGI0, SGI107126, SGI107126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23592[] = {SGI107127, SGI107128, SGI107129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107130, SGI0, SGI0, SGI0, SGI107131, SGI107131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23593[] = {SGI107132, SGI107133, SGI107134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107135, SGI0, SGI0, SGI0, SGI107136, SGI107136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23594[] = {SGI107137, SGI107138, SGI107139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107143, SGI0, SGI0, SGI0, SGI107144, SGI107144, SGI107146, SGI0, SGI107147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23595[] = {SGI107140, SGI107141, SGI107142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107145, SGI107145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23596[] = {SGI107148, SGI107149, SGI107150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107151, SGI0, SGI0, SGI0, SGI107152, SGI107152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23597[] = {SGI107153, SGI0, SGI107154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107155, SGI0, SGI0, SGI0, SGI107156, SGI107156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23598[] = {SGI107157, SGI107158, SGI107159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107160, SGI107160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23599[] = {SGI107161, SGI0, SGI107162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107163, SGI0, SGI0, SGI0, SGI107164, SGI107164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23600[] = {SGI107165, SGI107166, SGI107167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107168, SGI0, SGI0, SGI0, SGI107169, SGI107169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23601[] = {SGI107170, SGI0, SGI107171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23602[] = {SGI107172, SGI107173, SGI107174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107175, SGI0, SGI0, SGI0, SGI107176, SGI107176, SGI107177, SGI0, SGI107178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23603[] = {SGI107179, SGI107180, SGI107181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107182, SGI0, SGI0, SGI0, SGI107183, SGI107183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23604[] = {SGI107184, SGI107185, SGI107186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107187, SGI0, SGI0, SGI0, SGI107188, SGI107188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23605[] = {SGI107189, SGI107190, SGI107191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107192, SGI0, SGI0, SGI0, SGI107193, SGI107193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23606[] = {SGI107194, SGI0, SGI107195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107196, SGI0, SGI0, SGI0, SGI107198, SGI107198, SGI0, SGI0, SGI107201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23607[] = {SGI107197, SGI107199, SGI107200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107202, SGI107202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23608[] = {SGI107203, SGI0, SGI107204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107205, SGI0, SGI0, SGI0, SGI107206, SGI107206, SGI0, SGI0, SGI107207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23609[] = {SGI107208, SGI107209, SGI107210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107211, SGI0, SGI0, SGI0, SGI107212, SGI107212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23610[] = {SGI107213, SGI107214, SGI107215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107216, SGI0, SGI0, SGI0, SGI107217, SGI107217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23611[] = {SGI107218, SGI107219, SGI107220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107221, SGI0, SGI0, SGI0, SGI107222, SGI107222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23612[] = {SGI107223, SGI0, SGI107224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107228, SGI107229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23613[] = {SGI107225, SGI107226, SGI107227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107230, SGI0, SGI0, SGI0, SGI107231, SGI107231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23614[] = {SGI107232, SGI0, SGI107233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107234, SGI0, SGI0, SGI0, SGI107235, SGI107235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23615[] = {SGI107236, SGI107237, SGI107238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107239, SGI0, SGI0, SGI0, SGI107240, SGI107240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23616[] = {SGI107241, SGI0, SGI107242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107243, SGI107244, SGI107244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107245, SGI0, NULL};
const Segment * const SG23617[] = {SGI107246, SGI0, SGI107247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107248, SGI107249, SGI107249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107250, SGI0, NULL};
const Segment * const SG23618[] = {SGI107252, SGI0, SGI107254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23619[] = {SGI107251, SGI0, SGI107253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107255, SGI107256, SGI107256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23620[] = {SGI107257, SGI0, SGI107258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107259, SGI107260, SGI107260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23621[] = {SGI107262, SGI0, SGI107264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107268, SGI107269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107271, SGI0, NULL};
const Segment * const SG23622[] = {SGI107261, SGI107263, SGI107265, SGI0, SGI0, SGI0, SGI107266, SGI107267, SGI0, SGI107270, SGI0, SGI0, SGI0, SGI107272, SGI107272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23623[] = {SGI107273, SGI107274, SGI107275, SGI0, SGI0, SGI0, SGI107276, SGI107277, SGI0, SGI107278, SGI0, SGI0, SGI0, SGI107279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23624[] = {SGI107280, SGI0, SGI107281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107282, SGI107283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23625[] = {SGI107284, SGI0, SGI107285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107286, SGI0, SGI0, SGI0, SGI107287, SGI107287, SGI0, SGI107288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23626[] = {SGI107289, SGI0, SGI107290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107291, SGI107292, SGI107292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23627[] = {SGI107293, SGI107294, SGI107295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107296, SGI0, SGI0, SGI0, SGI107297, SGI107297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107298, SGI0, NULL};
const Segment * const SG23628[] = {SGI107299, SGI107300, SGI107301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107302, SGI0, SGI0, SGI0, SGI107303, SGI107303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107304, SGI0, NULL};
const Segment * const SG23629[] = {SGI107305, SGI107306, SGI107307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107308, SGI0, SGI0, SGI0, SGI107309, SGI107309, SGI0, SGI0, NULL};
const Segment * const SG23630[] = {SGI107310, SGI107311, SGI107312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107313, SGI0, SGI0, SGI0, SGI107314, SGI107314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23631[] = {SGI107315, SGI0, SGI107316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107317, SGI107318, SGI107318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23632[] = {SGI107319, SGI107320, SGI107321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107322, SGI0, SGI0, SGI0, SGI107323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23633[] = {SGI107324, SGI0, SGI107325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107326, SGI107327, SGI107327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23634[] = {SGI107328, SGI0, SGI107329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23635[] = {SGI107330, SGI107331, SGI107332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107333, SGI0, SGI0, SGI0, SGI107334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23636[] = {SGI107335, SGI0, SGI107336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107337, SGI0, SGI0, SGI0, SGI107338, SGI107338, SGI0, SGI107339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23637[] = {SGI107340, SGI107341, SGI107342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107343, SGI0, SGI0, SGI0, SGI107344, SGI0, SGI0, SGI0, SGI0, SGI107345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23638[] = {SGI107346, SGI107347, SGI107348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107349, SGI0, SGI0, SGI0, SGI107350, SGI107350, SGI107351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23639[] = {SGI107352, SGI107353, SGI107354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107355, SGI0, SGI0, SGI0, SGI107356, SGI0, SGI107357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23640[] = {SGI107358, SGI0, SGI107359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107360, SGI0, SGI0, SGI0, SGI107361, SGI107361, SGI0, SGI107362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23641[] = {SGI107363, SGI0, SGI107364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107365, SGI107366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23642[] = {SGI107367, SGI107368, SGI107369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107370, SGI0, SGI0, SGI0, SGI107371, SGI107371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23643[] = {SGI107372, SGI107373, SGI107374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107375, SGI0, SGI0, SGI0, SGI107376, SGI107376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23644[] = {SGI107377, SGI107378, SGI107379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107380, SGI107380, SGI0, SGI0, SGI107381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23645[] = {SGI107382, SGI107383, SGI107384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107385, SGI0, SGI0, SGI0, SGI107386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23646[] = {SGI107387, SGI0, SGI107388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107389, SGI107390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23647[] = {SGI107391, SGI0, SGI107392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107393, SGI107394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23648[] = {SGI107395, SGI107396, SGI107397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107398, SGI0, SGI0, SGI0, SGI107399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23649[] = {SGI107400, SGI107401, SGI107402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107403, SGI0, SGI0, SGI0, SGI107404, SGI107404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107405, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23650[] = {SGI107406, SGI107407, SGI107408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107409, SGI0, SGI0, SGI0, SGI107410, SGI107410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23651[] = {SGI107411, SGI0, SGI107412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107413, SGI0, SGI0, SGI0, SGI107414, SGI107414, SGI0, SGI107415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23652[] = {SGI107416, SGI107417, SGI107418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107419, SGI0, SGI0, SGI0, SGI107420, SGI107420, SGI107421, SGI0, SGI107422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23653[] = {SGI107423, SGI0, SGI107424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23654[] = {SGI107425, SGI107426, SGI107427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107428, SGI0, SGI0, SGI0, SGI107429, SGI107429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23655[] = {SGI107430, SGI107431, SGI107432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107433, SGI0, SGI0, SGI0, SGI107434, SGI107434, SGI107435, SGI0, SGI107436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23656[] = {SGI107437, SGI107438, SGI107439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107440, SGI0, SGI0, SGI0, SGI107441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23657[] = {SGI107442, SGI107443, SGI107444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107445, SGI0, SGI0, SGI0, SGI107446, SGI107446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23658[] = {SGI107447, SGI107448, SGI107449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107450, SGI0, SGI0, SGI0, SGI107451, SGI0, SGI107452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23659[] = {SGI107453, SGI0, SGI107454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23660[] = {SGI107455, SGI107456, SGI107457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107458, SGI0, SGI0, SGI0, SGI107459, SGI107459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23661[] = {SGI107460, SGI0, SGI107461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107462, SGI0, SGI0, SGI0, SGI107463, SGI107463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107464, SGI0, SGI0, NULL};
const Segment * const SG23662[] = {SGI107465, SGI107466, SGI107467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107468, SGI0, SGI0, SGI0, SGI107469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23663[] = {SGI107470, SGI107471, SGI107472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107473, SGI0, SGI0, SGI0, SGI107474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23664[] = {SGI107475, SGI107476, SGI107477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107478, SGI0, SGI0, SGI0, SGI107479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23665[] = {SGI107480, SGI107481, SGI107482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107483, SGI0, SGI0, SGI0, SGI107484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23666[] = {SGI107485, SGI0, SGI107486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23667[] = {SGI107487, SGI0, SGI107488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23668[] = {SGI107489, SGI107490, SGI107491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107492, SGI0, SGI0, SGI0, SGI107493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23669[] = {SGI107494, SGI107496, SGI107497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107500, SGI0, SGI0, SGI0, SGI107502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23670[] = {SGI107495, SGI107498, SGI107499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107501, SGI0, SGI0, SGI0, SGI107503, SGI107503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107504, SGI0, NULL};
const Segment * const SG23671[] = {SGI107505, SGI107506, SGI107507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107508, SGI0, SGI0, SGI0, SGI107509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23672[] = {SGI107510, SGI107511, SGI107512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107513, SGI0, SGI0, SGI0, SGI107514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23673[] = {SGI107515, SGI107516, SGI107517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107518, SGI0, SGI0, SGI0, SGI107519, SGI107519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23674[] = {SGI107520, SGI107521, SGI107522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107525, SGI0, SGI0, SGI0, SGI107527, SGI107527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23675[] = {SGI107523, SGI107524, SGI107526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107528, SGI0, SGI0, SGI0, SGI107529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23676[] = {SGI107530, SGI0, SGI107531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107532, SGI0, SGI0, SGI0, SGI107533, SGI107533, SGI0, SGI0, SGI107534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23677[] = {SGI107535, SGI107536, SGI107537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107538, SGI0, SGI0, SGI0, SGI107539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23678[] = {SGI107540, SGI0, SGI107541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107542, SGI0, SGI0, SGI0, SGI107543, SGI107543, SGI0, SGI0, SGI107544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23679[] = {SGI107545, SGI107546, SGI107547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107548, SGI0, SGI0, SGI0, SGI107549, SGI107549, SGI107550, SGI0, SGI107551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23680[] = {SGI107552, SGI107553, SGI107554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107555, SGI0, SGI0, SGI0, SGI107556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23681[] = {SGI107557, SGI107558, SGI107559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107560, SGI0, SGI0, SGI0, SGI107561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23682[] = {SGI107562, SGI107563, SGI107564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107565, SGI0, SGI0, SGI0, SGI107566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23683[] = {SGI107567, SGI107568, SGI107569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107570, SGI0, SGI0, SGI0, SGI107571, SGI107571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23684[] = {SGI107572, SGI107573, SGI107574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107575, SGI0, SGI0, SGI0, SGI107576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23685[] = {SGI107577, SGI107578, SGI107579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107580, SGI0, SGI0, SGI0, SGI107581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23686[] = {SGI107582, SGI107583, SGI107584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107585, SGI0, SGI0, SGI0, SGI107586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23687[] = {SGI107587, SGI107588, SGI107589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107590, SGI0, SGI0, SGI0, SGI107591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23688[] = {SGI107592, SGI107593, SGI107594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107595, SGI0, SGI0, SGI0, SGI107596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23689[] = {SGI0, SGI0, SGI107597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107598, SGI107599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23690[] = {SGI107600, SGI107601, SGI107602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107603, SGI0, SGI0, SGI0, SGI107604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23691[] = {SGI107605, SGI107606, SGI107607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107608, SGI0, SGI0, SGI0, SGI107609, SGI107609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23692[] = {SGI107610, SGI107611, SGI107612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107613, SGI0, SGI0, SGI0, SGI107614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23693[] = {SGI107615, SGI0, SGI107616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107617, SGI107618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23694[] = {SGI107619, SGI107620, SGI107621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107622, SGI0, SGI0, SGI0, SGI107623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23695[] = {SGI107624, SGI107625, SGI107626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107627, SGI0, SGI0, SGI0, SGI107628, SGI107628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23696[] = {SGI107629, SGI0, SGI107630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107631, SGI0, SGI0, SGI0, SGI107632, SGI107632, SGI0, SGI107633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23697[] = {SGI107634, SGI107635, SGI107636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107637, SGI0, SGI0, SGI0, SGI107638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23698[] = {SGI107639, SGI0, SGI107640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107641, SGI0, SGI0, SGI0, SGI107642, SGI107642, SGI0, SGI107643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23699[] = {SGI107644, SGI107645, SGI107646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107647, SGI0, SGI0, SGI0, SGI107648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23700[] = {SGI107649, SGI107650, SGI107651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107652, SGI0, SGI0, SGI0, SGI107653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107654, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23701[] = {SGI0, SGI107655, SGI107656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107657, SGI0, SGI0, SGI0, SGI107658, SGI107660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23702[] = {SGI107659, SGI0, SGI107661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107662, SGI0, SGI0, SGI0, SGI107663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107664, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23703[] = {SGI107665, SGI0, SGI107666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107667, SGI0, SGI0, SGI0, SGI107668, SGI107668, SGI0, SGI107669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23704[] = {SGI107670, SGI107671, SGI107672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107673, SGI0, SGI0, SGI0, SGI107674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23705[] = {SGI107675, SGI107676, SGI107677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107678, SGI0, SGI0, SGI0, SGI107679, SGI107680, SGI0, SGI107681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23706[] = {SGI107682, SGI107683, SGI107684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107685, SGI0, SGI0, SGI0, SGI107686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23707[] = {SGI107687, SGI107688, SGI107689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107693, SGI0, SGI0, SGI0, SGI107695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23708[] = {SGI107690, SGI107691, SGI107692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107698, SGI0, SGI0, SGI0, SGI107699, SGI107699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23709[] = {SGI107694, SGI107696, SGI107697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107700, SGI0, SGI0, SGI0, SGI107701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107702, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23710[] = {SGI107703, SGI107704, SGI107705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107706, SGI0, SGI0, SGI0, SGI107707, SGI107707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107708, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23711[] = {SGI107709, SGI107710, SGI107711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107712, SGI0, SGI0, SGI0, SGI107715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107717, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23712[] = {SGI107713, SGI107714, SGI107716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107718, SGI0, SGI0, SGI0, SGI107719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23713[] = {SGI107720, SGI0, SGI107721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107722, SGI107723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23714[] = {SGI107726, SGI107727, SGI107728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107730, SGI0, SGI0, SGI0, SGI107732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23715[] = {SGI107724, SGI0, SGI107725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107729, SGI0, SGI0, SGI0, SGI107731, SGI107731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107733, SGI0, SGI0, NULL};
const Segment * const SG23716[] = {SGI107734, SGI107735, SGI107736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107737, SGI0, SGI0, SGI0, SGI107738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23717[] = {SGI107739, SGI107740, SGI107741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107745, SGI0, SGI0, SGI0, SGI107747, SGI0, SGI0, SGI0, SGI107749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23718[] = {SGI107742, SGI107743, SGI107744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107746, SGI0, SGI0, SGI0, SGI107748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23719[] = {SGI107750, SGI107751, SGI107752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107753, SGI0, SGI0, SGI0, SGI107754, SGI0, SGI0, SGI0, SGI107755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23720[] = {SGI107756, SGI107757, SGI107758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107762, SGI0, SGI0, SGI0, SGI107763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23721[] = {SGI107759, SGI107760, SGI107761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107764, SGI0, SGI0, SGI0, SGI107765, SGI0, SGI0, SGI0, SGI107766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23722[] = {SGI107767, SGI107768, SGI107769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107770, SGI0, SGI0, SGI0, SGI107771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23723[] = {SGI107772, SGI107773, SGI107774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107775, SGI0, SGI0, SGI0, SGI107777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23724[] = {SGI107776, SGI107778, SGI107779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107780, SGI0, SGI0, SGI0, SGI107781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23725[] = {SGI107782, SGI107783, SGI107784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107785, SGI0, SGI0, SGI0, SGI107788, SGI0, SGI0, SGI0, SGI107790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23726[] = {SGI107786, SGI107787, SGI107789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107791, SGI0, SGI0, SGI0, SGI107792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23727[] = {SGI107793, SGI107794, SGI107795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107796, SGI0, SGI0, SGI0, SGI107797, SGI0, SGI0, SGI0, SGI107798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23728[] = {SGI107799, SGI0, SGI107800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107804, SGI107805, SGI107805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23729[] = {SGI107801, SGI107802, SGI107803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107806, SGI0, SGI0, SGI0, SGI107807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107808, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23730[] = {SGI107809, SGI107810, SGI107811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107812, SGI0, SGI0, SGI0, SGI107813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107814, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23731[] = {SGI107815, SGI107816, SGI107817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107818, SGI0, SGI0, SGI0, SGI107819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23732[] = {SGI107820, SGI107821, SGI107822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107824, SGI0, SGI0, SGI0, SGI107827, SGI0, SGI0, SGI0, SGI0, SGI107828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23733[] = {SGI107823, SGI107825, SGI107826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107829, SGI0, SGI0, SGI0, SGI107830, SGI107830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23734[] = {SGI107831, SGI107832, SGI107833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107834, SGI0, SGI0, SGI0, SGI107835, SGI107835, SGI0, SGI0, SGI0, SGI107836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23735[] = {SGI107837, SGI107838, SGI107839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107840, SGI0, SGI0, SGI0, SGI107841, SGI107841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107843, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23736[] = {SGI107842, SGI107844, SGI107845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107846, SGI0, SGI0, SGI0, SGI107847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23737[] = {SGI107848, SGI107849, SGI107850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107851, SGI0, SGI0, SGI0, SGI107852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23738[] = {SGI107853, SGI107854, SGI107855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107856, SGI0, SGI0, SGI0, SGI107857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23739[] = {SGI107858, SGI0, SGI107859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107860, SGI107861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23740[] = {SGI107862, SGI0, SGI107863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107864, SGI0, SGI0, SGI0, SGI107865, SGI107865, SGI0, SGI107866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23741[] = {SGI107867, SGI107868, SGI107869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107870, SGI0, SGI0, SGI0, SGI107871, SGI0, SGI0, SGI107872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23742[] = {SGI107873, SGI107874, SGI107876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107878, SGI0, SGI0, SGI0, SGI107880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23743[] = {SGI107875, SGI0, SGI107877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107879, SGI0, SGI0, SGI0, SGI107881, SGI107881, SGI0, SGI107884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23744[] = {SGI107882, SGI107883, SGI107885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107886, SGI0, SGI0, SGI0, SGI107887, SGI0, SGI0, SGI107888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23745[] = {SGI107889, SGI107890, SGI107891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107892, SGI0, SGI0, SGI0, SGI107893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23746[] = {SGI107895, SGI107897, SGI107899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107900, SGI0, SGI0, SGI0, SGI107901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23747[] = {SGI107894, SGI107896, SGI107898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107902, SGI107902, SGI0, SGI0, SGI107903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23748[] = {SGI107904, SGI107905, SGI107906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107907, SGI0, SGI0, SGI0, SGI107908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23749[] = {SGI107909, SGI107910, SGI107911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107912, SGI0, SGI0, SGI0, SGI107913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23750[] = {SGI107914, SGI107915, SGI107916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107917, SGI107917, SGI0, SGI0, SGI107918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23751[] = {SGI107919, SGI107920, SGI107921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107922, SGI0, SGI0, SGI0, SGI107923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23752[] = {SGI107924, SGI107925, SGI107926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107927, SGI0, SGI0, SGI0, SGI107928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23753[] = {SGI107929, SGI107930, SGI107931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107932, SGI0, SGI0, SGI0, SGI107933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23754[] = {SGI107934, SGI107935, SGI107936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107937, SGI0, SGI0, SGI0, SGI107938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23755[] = {SGI107939, SGI107940, SGI107941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107942, SGI107942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107943, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23756[] = {SGI107944, SGI107945, SGI107946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107947, SGI0, SGI0, SGI0, SGI107948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23757[] = {SGI107949, SGI107950, SGI107951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107952, SGI0, SGI0, SGI0, SGI107953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23758[] = {SGI107954, SGI107955, SGI107956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107957, SGI0, SGI0, SGI0, SGI107958, SGI107958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23759[] = {SGI107959, SGI107960, SGI107961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107962, SGI0, SGI0, SGI0, SGI107963, SGI107963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23760[] = {SGI107964, SGI107965, SGI107966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107967, SGI0, SGI0, SGI0, SGI107968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23761[] = {SGI107969, SGI107970, SGI107971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107972, SGI0, SGI0, SGI0, SGI107973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23762[] = {SGI107974, SGI107975, SGI107976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107977, SGI0, SGI0, SGI0, SGI107978, SGI107978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23763[] = {SGI107979, SGI107980, SGI107981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107982, SGI0, SGI0, SGI0, SGI107983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23764[] = {SGI107984, SGI107985, SGI107986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107987, SGI0, SGI0, SGI0, SGI107988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23765[] = {SGI107989, SGI107990, SGI107991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107992, SGI0, SGI0, SGI0, SGI107993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23766[] = {SGI107994, SGI107995, SGI107996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI107997, SGI0, SGI0, SGI0, SGI107998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23767[] = {SGI107999, SGI108000, SGI108001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108002, SGI0, SGI0, SGI0, SGI108003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23768[] = {SGI108004, SGI108005, SGI108006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108007, SGI0, SGI0, SGI0, SGI108008, SGI108008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23769[] = {SGI108009, SGI108010, SGI108011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108012, SGI0, SGI0, SGI0, SGI108013, SGI108013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23770[] = {SGI108014, SGI108015, SGI108016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108017, SGI0, SGI0, SGI0, SGI108018, SGI108018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23771[] = {SGI108019, SGI0, SGI108020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108021, SGI108022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23772[] = {SGI108023, SGI108024, SGI108025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108026, SGI0, SGI0, SGI0, SGI108027, SGI108027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23773[] = {SGI108028, SGI108029, SGI108030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108031, SGI0, SGI0, SGI0, SGI108032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23774[] = {SGI108033, SGI0, SGI108034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108035, SGI108036, SGI108036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23775[] = {SGI108037, SGI108038, SGI108039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108040, SGI0, SGI0, SGI0, SGI108041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23776[] = {SGI108042, SGI108043, SGI108044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108046, SGI0, SGI0, SGI0, SGI108048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23777[] = {SGI108045, SGI0, SGI108047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108049, SGI108050, SGI108050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23778[] = {SGI108051, SGI108052, SGI108053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108054, SGI0, SGI0, SGI0, SGI108055, SGI108055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23779[] = {SGI108056, SGI108057, SGI108058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108059, SGI0, SGI0, SGI0, SGI108060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23780[] = {SGI108061, SGI108062, SGI108063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108064, SGI0, SGI0, SGI0, SGI108065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23781[] = {SGI108066, SGI108067, SGI108068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108069, SGI0, SGI0, SGI0, SGI108070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23782[] = {SGI108071, SGI108072, SGI108073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108074, SGI0, SGI0, SGI0, SGI108075, SGI108075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23783[] = {SGI108076, SGI108077, SGI108078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108079, SGI0, SGI0, SGI0, SGI108080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23784[] = {SGI108081, SGI108082, SGI108083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108084, SGI0, SGI0, SGI0, SGI108085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23785[] = {SGI108086, SGI108087, SGI108088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108089, SGI0, SGI0, SGI0, SGI108090, SGI108090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23786[] = {SGI108091, SGI0, SGI108092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108094, SGI108096, SGI108096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23787[] = {SGI108093, SGI108095, SGI108097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108098, SGI0, SGI0, SGI0, SGI108099, SGI108099, SGI0, SGI0, SGI0, SGI0, SGI108102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23788[] = {SGI108100, SGI108101, SGI108103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108104, SGI0, SGI0, SGI0, SGI108105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23789[] = {SGI108106, SGI108107, SGI108108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108109, SGI0, SGI0, SGI0, SGI108110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23790[] = {SGI108111, SGI108112, SGI108113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108114, SGI0, SGI0, SGI0, SGI108115, SGI108115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23791[] = {SGI108116, SGI108117, SGI108118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108119, SGI0, SGI0, SGI0, SGI108120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23792[] = {SGI108121, SGI108122, SGI108123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108124, SGI0, SGI0, SGI0, SGI108125, SGI108125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23793[] = {SGI108126, SGI0, SGI108127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108128, SGI108129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23794[] = {SGI108130, SGI108131, SGI108132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108133, SGI0, SGI0, SGI0, SGI108134, SGI108134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23795[] = {SGI108135, SGI0, SGI108136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108137, SGI108138, SGI108138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23796[] = {SGI108139, SGI108140, SGI108141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108142, SGI0, SGI0, SGI0, SGI108143, SGI108143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23797[] = {SGI108144, SGI108145, SGI108146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108147, SGI0, SGI0, SGI0, SGI108148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23798[] = {SGI108149, SGI108150, SGI108151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108152, SGI0, SGI0, SGI0, SGI108153, SGI108153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23799[] = {SGI108154, SGI108155, SGI108156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108157, SGI0, SGI0, SGI0, SGI108158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23800[] = {SGI108159, SGI0, SGI108160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108161, SGI108162, SGI108162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23801[] = {SGI108163, SGI0, SGI108164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108166, SGI108168, SGI108168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23802[] = {SGI108165, SGI108167, SGI108169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108170, SGI0, SGI0, SGI0, SGI108171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23803[] = {SGI108172, SGI108173, SGI108174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108175, SGI0, SGI0, SGI0, SGI108176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23804[] = {SGI108177, SGI108178, SGI108179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108180, SGI0, SGI0, SGI0, SGI108181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23805[] = {SGI0, SGI108182, SGI108183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23806[] = {SGI108184, SGI108185, SGI108186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108187, SGI0, SGI0, SGI0, SGI108188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23807[] = {SGI108189, SGI108190, SGI108191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108192, SGI108192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23808[] = {SGI108193, SGI108194, SGI108195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108196, SGI0, SGI0, SGI0, SGI108197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23809[] = {SGI108198, SGI108199, SGI108200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108201, SGI0, SGI0, SGI0, SGI108202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23810[] = {SGI108203, SGI108204, SGI108205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108206, SGI0, SGI0, SGI0, SGI108207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23811[] = {SGI108208, SGI108209, SGI108210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108211, SGI0, SGI0, SGI0, SGI108212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23812[] = {SGI108213, SGI108214, SGI108215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108216, SGI0, SGI0, SGI0, SGI108217, SGI108217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23813[] = {SGI108218, SGI108219, SGI108220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108221, SGI0, SGI0, SGI0, SGI108222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23814[] = {SGI108223, SGI108224, SGI108225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108226, SGI108226, SGI0, SGI0, SGI108227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23815[] = {SGI108228, SGI108229, SGI108230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108231, SGI0, SGI0, SGI0, SGI108232, SGI108232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23816[] = {SGI108233, SGI108234, SGI108235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108236, SGI0, SGI0, SGI0, SGI108237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23817[] = {SGI108238, SGI0, SGI108239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108240, SGI108241, SGI108241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23818[] = {SGI108242, SGI108243, SGI108244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108245, SGI0, SGI0, SGI0, SGI108246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23819[] = {SGI108247, SGI108248, SGI108249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108250, SGI0, SGI0, SGI0, SGI108251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23820[] = {SGI108252, SGI108253, SGI108254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108255, SGI0, SGI0, SGI0, SGI108256, SGI108256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23821[] = {SGI108257, SGI108258, SGI108259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108260, SGI0, SGI0, SGI0, SGI108261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23822[] = {SGI108262, SGI108263, SGI108264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108265, SGI0, SGI0, SGI0, SGI108266, SGI108266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23823[] = {SGI108267, SGI108268, SGI108269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108270, SGI0, SGI0, SGI0, SGI108271, SGI108271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23824[] = {SGI108272, SGI108273, SGI108274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108275, SGI0, SGI0, SGI0, SGI108276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23825[] = {SGI108277, SGI108278, SGI108279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108280, SGI0, SGI0, SGI0, SGI108281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23826[] = {SGI108282, SGI108283, SGI108284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108285, SGI0, SGI0, SGI0, SGI108286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23827[] = {SGI108287, SGI108288, SGI108289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108290, SGI0, SGI0, SGI0, SGI108291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23828[] = {SGI108292, SGI108293, SGI108294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108295, SGI0, SGI0, SGI0, SGI108296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23829[] = {SGI108297, SGI108298, SGI108299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108300, SGI0, SGI0, SGI0, SGI108301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23830[] = {SGI108302, SGI0, SGI108303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23831[] = {SGI108304, SGI108305, SGI108306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108307, SGI0, SGI0, SGI0, SGI108308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23832[] = {SGI108309, SGI0, SGI108310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108311, SGI0, SGI0, SGI0, SGI108312, SGI108312, SGI0, SGI108313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23833[] = {SGI108314, SGI108315, SGI108316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108317, SGI0, SGI0, SGI0, SGI108318, SGI108318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23834[] = {SGI108319, SGI0, SGI108320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108321, SGI0, SGI0, SGI0, SGI108322, SGI108322, SGI0, SGI108323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23835[] = {SGI108324, SGI108325, SGI108326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108327, SGI0, SGI0, SGI0, SGI108328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23836[] = {SGI108329, SGI108330, SGI108331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108332, SGI108332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108333, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23837[] = {SGI108334, SGI108336, SGI108338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108341, SGI108341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108343, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23838[] = {SGI108335, SGI108337, SGI108339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108340, SGI0, SGI0, SGI0, SGI108342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23839[] = {SGI108344, SGI108345, SGI108347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108350, SGI0, SGI0, SGI0, SGI108352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108354, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23840[] = {SGI108346, SGI108348, SGI108349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108351, SGI0, SGI0, SGI0, SGI108353, SGI108353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23841[] = {SGI108355, SGI108356, SGI108357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108358, SGI0, SGI0, SGI0, SGI108359, SGI108359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23842[] = {SGI108360, SGI108361, SGI108362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108363, SGI0, SGI0, SGI0, SGI108364, SGI108364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23843[] = {SGI108365, SGI108366, SGI108367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108368, SGI0, SGI0, SGI0, SGI108369, SGI108369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23844[] = {SGI108370, SGI108371, SGI108372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108373, SGI0, SGI0, SGI0, SGI108374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23845[] = {SGI108376, SGI108377, SGI108379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108382, SGI0, SGI0, SGI0, SGI108384, SGI108384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23846[] = {SGI108375, SGI108378, SGI108380, SGI108381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108383, SGI0, SGI0, SGI0, SGI108385, SGI108386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23847[] = {SGI108387, SGI108388, SGI108389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108390, SGI0, SGI0, SGI0, SGI108391, SGI108391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23848[] = {SGI108394, SGI108396, SGI108397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108399, SGI0, SGI0, SGI0, SGI108400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23849[] = {SGI108392, SGI108393, SGI108395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108398, SGI0, SGI0, SGI0, SGI108401, SGI0, SGI0, SGI0, SGI108402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23850[] = {SGI108403, SGI108404, SGI108405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108408, SGI0, SGI0, SGI0, SGI108410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23851[] = {SGI108406, SGI108407, SGI108409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108411, SGI0, SGI0, SGI0, SGI108412, SGI0, SGI0, SGI0, SGI108413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23852[] = {SGI108414, SGI108415, SGI108416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108418, SGI0, SGI0, SGI0, SGI108420, SGI108420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23853[] = {SGI108417, SGI0, SGI108419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108421, SGI108422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23854[] = {SGI108423, SGI108424, SGI108425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108426, SGI0, SGI0, SGI0, SGI108427, SGI108427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23855[] = {SGI108428, SGI0, SGI108429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108430, SGI0, SGI0, SGI0, SGI108434, SGI108434, SGI0, SGI0, SGI108435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23856[] = {SGI108431, SGI108432, SGI108433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108436, SGI0, SGI0, SGI0, SGI108437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23857[] = {SGI108438, SGI108439, SGI108440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108441, SGI0, SGI0, SGI0, SGI108442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23858[] = {SGI108443, SGI108444, SGI108445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108446, SGI0, SGI0, SGI0, SGI108447, SGI0, SGI108448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23859[] = {SGI108449, SGI108450, SGI108451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108452, SGI0, SGI0, SGI0, SGI108453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23860[] = {SGI108454, SGI108455, SGI108456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108457, SGI0, SGI0, SGI0, SGI108458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23861[] = {SGI108459, SGI108460, SGI108461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108462, SGI0, SGI0, SGI0, SGI108463, SGI108463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23862[] = {SGI108464, SGI108465, SGI108466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108467, SGI0, SGI0, SGI0, SGI108468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23863[] = {SGI108469, SGI0, SGI108470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108471, SGI108472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23864[] = {SGI108473, SGI108474, SGI108475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108476, SGI0, SGI0, SGI0, SGI108477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23865[] = {SGI108478, SGI108479, SGI108480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108481, SGI0, SGI0, SGI0, SGI108482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23866[] = {SGI108483, SGI108484, SGI108485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108486, SGI0, SGI0, SGI0, SGI108487, SGI108487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23867[] = {SGI108488, SGI108489, SGI108490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108491, SGI0, SGI0, SGI0, SGI108492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23868[] = {SGI108493, SGI108494, SGI108495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108496, SGI0, SGI0, SGI0, SGI108497, SGI108497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23869[] = {SGI108498, SGI0, SGI108499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108502, SGI108504, SGI108504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23870[] = {SGI108500, SGI108501, SGI108503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108505, SGI0, SGI0, SGI0, SGI108506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23871[] = {SGI108507, SGI108508, SGI108509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108510, SGI0, SGI0, SGI0, SGI108511, SGI108511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23872[] = {SGI108512, SGI108513, SGI108514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108515, SGI0, SGI0, SGI0, SGI108516, SGI108516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23873[] = {SGI108517, SGI108518, SGI108519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108520, SGI0, SGI0, SGI0, SGI108521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23874[] = {SGI108522, SGI108523, SGI108524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108525, SGI0, SGI0, SGI0, SGI108526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23875[] = {SGI108527, SGI108528, SGI108529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108530, SGI0, SGI0, SGI0, SGI108531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23876[] = {SGI108532, SGI108533, SGI108534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108535, SGI0, SGI0, SGI0, SGI108536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23877[] = {SGI108537, SGI108538, SGI108539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108540, SGI0, SGI0, SGI0, SGI108541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23878[] = {SGI108542, SGI108543, SGI108544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108545, SGI0, SGI0, SGI0, SGI108546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23879[] = {SGI108547, SGI0, SGI108548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108549, SGI108550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108552, SGI0, NULL};
const Segment * const SG23880[] = {SGI108551, SGI108553, SGI108554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108555, SGI0, SGI0, SGI0, SGI108556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23881[] = {SGI108557, SGI108558, SGI108559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108560, SGI0, SGI0, SGI0, SGI108561, SGI108561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23882[] = {SGI108562, SGI108563, SGI108564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108565, SGI0, SGI0, SGI0, SGI108566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23883[] = {SGI108567, SGI108568, SGI108569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108570, SGI0, SGI0, SGI0, SGI108571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23884[] = {SGI108572, SGI0, SGI108574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108577, SGI108579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23885[] = {SGI108573, SGI108575, SGI108576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108578, SGI0, SGI0, SGI0, SGI108580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23886[] = {SGI108581, SGI108582, SGI108583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108584, SGI0, SGI0, SGI0, SGI108585, SGI108585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23887[] = {SGI108586, SGI108587, SGI108588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108589, SGI0, SGI0, SGI0, SGI108590, SGI108590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23888[] = {SGI108591, SGI108592, SGI108593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108594, SGI0, SGI0, SGI0, SGI108595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23889[] = {SGI108596, SGI108597, SGI108598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108599, SGI0, SGI0, SGI0, SGI108600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23890[] = {SGI108601, SGI108602, SGI108603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108604, SGI0, SGI0, SGI0, SGI108605, SGI108605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23891[] = {SGI108606, SGI108607, SGI108608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108609, SGI0, SGI0, SGI0, SGI108610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23892[] = {SGI108611, SGI108612, SGI108613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108614, SGI0, SGI0, SGI0, SGI108615, SGI108615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23893[] = {SGI108616, SGI108617, SGI108618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108619, SGI0, SGI0, SGI0, SGI108620, SGI108620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23894[] = {SGI108621, SGI108622, SGI108623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108624, SGI0, SGI0, SGI0, SGI108625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23895[] = {SGI108626, SGI108627, SGI108628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108629, SGI0, SGI0, SGI0, SGI108630, SGI108630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23896[] = {SGI108631, SGI108632, SGI108633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108634, SGI0, SGI0, SGI0, SGI108635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23897[] = {SGI108636, SGI108637, SGI108638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108639, SGI0, SGI0, SGI0, SGI108640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23898[] = {SGI108641, SGI108642, SGI108643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108644, SGI0, SGI0, SGI0, SGI108645, SGI108645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23899[] = {SGI108646, SGI108647, SGI108648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108649, SGI0, SGI0, SGI0, SGI108650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23900[] = {SGI108651, SGI108652, SGI108653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108654, SGI0, SGI0, SGI0, SGI108655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23901[] = {SGI108656, SGI108657, SGI108658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108659, SGI0, SGI0, SGI0, SGI108661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23902[] = {SGI108660, SGI108662, SGI108663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108664, SGI0, SGI0, SGI0, SGI108665, SGI108665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23903[] = {SGI108666, SGI108667, SGI108668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108672, SGI0, SGI0, SGI0, SGI108673, SGI108673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23904[] = {SGI108669, SGI108670, SGI108671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108674, SGI0, SGI0, SGI0, SGI108675, SGI108675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23905[] = {SGI108676, SGI108677, SGI108678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108679, SGI0, SGI0, SGI0, SGI108680, SGI108680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23906[] = {SGI108681, SGI108682, SGI108683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108684, SGI0, SGI0, SGI0, SGI108685, SGI108685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23907[] = {SGI108686, SGI108687, SGI108688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108689, SGI0, SGI0, SGI0, SGI108690, SGI108690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23908[] = {SGI108691, SGI108692, SGI108693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108694, SGI0, SGI0, SGI0, SGI108695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23909[] = {SGI108696, SGI0, SGI108697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108698, SGI0, SGI0, SGI0, SGI108699, SGI108699, SGI0, SGI0, SGI108700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23910[] = {SGI108701, SGI108702, SGI108703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108704, SGI0, SGI0, SGI0, SGI108705, SGI108705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23911[] = {SGI108706, SGI108707, SGI108708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108709, SGI0, SGI0, SGI0, SGI108710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23912[] = {SGI108711, SGI0, SGI108712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108716, SGI108717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23913[] = {SGI108713, SGI108714, SGI108715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108718, SGI0, SGI0, SGI0, SGI108719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23914[] = {SGI108720, SGI108721, SGI108722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108723, SGI0, SGI0, SGI0, SGI108724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23915[] = {SGI108725, SGI108726, SGI108727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108728, SGI0, SGI0, SGI0, SGI108729, SGI108729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108730, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23916[] = {SGI108731, SGI108732, SGI108733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108734, SGI0, SGI0, SGI0, SGI108735, SGI108735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23917[] = {SGI108736, SGI108737, SGI108738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108739, SGI0, SGI0, SGI0, SGI108740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23918[] = {SGI108741, SGI108742, SGI108743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108744, SGI0, SGI0, SGI0, SGI108745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23919[] = {SGI108746, SGI108747, SGI108748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108749, SGI0, SGI0, SGI0, SGI108750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23920[] = {SGI108751, SGI108752, SGI108753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108754, SGI0, SGI0, SGI0, SGI108755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23921[] = {SGI108756, SGI108757, SGI108758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108759, SGI0, SGI0, SGI0, SGI108760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23922[] = {SGI108761, SGI108762, SGI108763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108764, SGI0, SGI0, SGI0, SGI108766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23923[] = {SGI108765, SGI0, SGI108767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108768, SGI108769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23924[] = {SGI108770, SGI108771, SGI108772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108773, SGI0, SGI0, SGI0, SGI108774, SGI108774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23925[] = {SGI108775, SGI108776, SGI108777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108778, SGI0, SGI0, SGI0, SGI108779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23926[] = {SGI108780, SGI108781, SGI108782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108783, SGI0, SGI0, SGI0, SGI108784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23927[] = {SGI108785, SGI108786, SGI108787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108788, SGI0, SGI0, SGI0, SGI108789, SGI108789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23928[] = {SGI108790, SGI0, SGI108791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108792, SGI108793, SGI108793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23929[] = {SGI108794, SGI108795, SGI108796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108797, SGI0, SGI0, SGI0, SGI108798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23930[] = {SGI108799, SGI108800, SGI108801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108802, SGI0, SGI0, SGI0, SGI108803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23931[] = {SGI108804, SGI108805, SGI108806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108807, SGI0, SGI0, SGI0, SGI108808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23932[] = {SGI108809, SGI108810, SGI108811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108812, SGI0, SGI0, SGI0, SGI108813, SGI108813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23933[] = {SGI108814, SGI0, SGI108815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108816, SGI108817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108818, SGI0, NULL};
const Segment * const SG23934[] = {SGI108819, SGI108820, SGI108821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108822, SGI0, SGI0, SGI0, SGI108824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23935[] = {SGI108823, SGI0, SGI108825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108826, SGI108827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108828, SGI0, NULL};
const Segment * const SG23936[] = {SGI108829, SGI108830, SGI108831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108832, SGI0, SGI0, SGI0, SGI108833, SGI108833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23937[] = {SGI108834, SGI108835, SGI108836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108837, SGI0, SGI0, SGI0, SGI108838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23938[] = {SGI108839, SGI108840, SGI108841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108842, SGI0, SGI0, SGI0, SGI108843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23939[] = {SGI108844, SGI108845, SGI108846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108847, SGI0, SGI0, SGI0, SGI108848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23940[] = {SGI108849, SGI108850, SGI108851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108852, SGI0, SGI0, SGI0, SGI108853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23941[] = {SGI108854, SGI108855, SGI108856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108857, SGI0, SGI0, SGI0, SGI108858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23942[] = {SGI108859, SGI108860, SGI108861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108862, SGI0, SGI0, SGI0, SGI108863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23943[] = {SGI108864, SGI108865, SGI108866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108867, SGI0, SGI0, SGI0, SGI108868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23944[] = {SGI108869, SGI108870, SGI108871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108872, SGI0, SGI0, SGI0, SGI108873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23945[] = {SGI108874, SGI108875, SGI108876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108877, SGI0, SGI0, SGI0, SGI108878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23946[] = {SGI108879, SGI108880, SGI108881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108882, SGI0, SGI0, SGI0, SGI108883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23947[] = {SGI108884, SGI108885, SGI108886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108887, SGI108887, SGI0, SGI0, SGI108888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23948[] = {SGI108889, SGI108890, SGI108891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108892, SGI0, SGI0, SGI0, SGI108893, SGI108893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23949[] = {SGI108894, SGI108895, SGI108896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108897, SGI0, SGI0, SGI0, SGI108898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23950[] = {SGI108899, SGI108900, SGI108901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108902, SGI0, SGI0, SGI0, SGI108903, SGI108903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23951[] = {SGI108904, SGI108905, SGI108906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108907, SGI0, SGI0, SGI0, SGI108908, SGI108908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23952[] = {SGI108909, SGI108910, SGI108911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108912, SGI0, SGI0, SGI0, SGI108913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23953[] = {SGI108914, SGI108915, SGI108916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108917, SGI0, SGI0, SGI0, SGI108918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23954[] = {SGI108919, SGI108920, SGI108921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108922, SGI0, SGI0, SGI0, SGI108923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23955[] = {SGI108924, SGI108925, SGI108926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108927, SGI0, SGI0, SGI0, SGI108928, SGI108928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23956[] = {SGI108929, SGI108930, SGI108931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108932, SGI0, SGI0, SGI0, SGI108933, SGI108933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23957[] = {SGI108934, SGI108935, SGI108936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108937, SGI0, SGI0, SGI0, SGI108938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23958[] = {SGI108939, SGI108940, SGI108941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108942, SGI0, SGI0, SGI0, SGI108943, SGI108943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23959[] = {SGI108944, SGI108945, SGI108946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108947, SGI0, SGI0, SGI0, SGI108948, SGI108948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23960[] = {SGI108949, SGI108950, SGI108951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108955, SGI0, SGI0, SGI0, SGI108956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23961[] = {SGI108952, SGI108953, SGI108954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108957, SGI0, SGI0, SGI0, SGI108958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23962[] = {SGI108959, SGI108960, SGI108961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108962, SGI0, SGI0, SGI0, SGI108963, SGI108963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23963[] = {SGI108964, SGI0, SGI108965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108966, SGI108967, SGI108967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23964[] = {SGI108968, SGI108969, SGI108970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108971, SGI0, SGI0, SGI0, SGI108972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23965[] = {SGI108973, SGI108974, SGI108975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108976, SGI0, SGI0, SGI0, SGI108977, SGI108977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23966[] = {SGI108978, SGI108979, SGI108980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108981, SGI0, SGI0, SGI0, SGI108982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23967[] = {SGI108983, SGI108984, SGI108985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108986, SGI0, SGI0, SGI0, SGI108987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108988, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23968[] = {SGI108989, SGI108990, SGI108991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108992, SGI0, SGI0, SGI0, SGI108993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23969[] = {SGI108994, SGI108995, SGI108996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108997, SGI0, SGI0, SGI0, SGI108998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI108999, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23970[] = {SGI109000, SGI109001, SGI109002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109003, SGI0, SGI0, SGI0, SGI109004, SGI109004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23971[] = {SGI109005, SGI109006, SGI109007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109008, SGI0, SGI0, SGI0, SGI109009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109010, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23972[] = {SGI109011, SGI109012, SGI109013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109014, SGI0, SGI0, SGI0, SGI109015, SGI109015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23973[] = {SGI109016, SGI109017, SGI109018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109019, SGI0, SGI0, SGI0, SGI109020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109022, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23974[] = {SGI109021, SGI109023, SGI109024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109025, SGI0, SGI0, SGI0, SGI109026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23975[] = {SGI109027, SGI109028, SGI109029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109030, SGI0, SGI0, SGI0, SGI109034, SGI0, SGI0, SGI0, SGI0, SGI109035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23976[] = {SGI109031, SGI109032, SGI109033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109036, SGI0, SGI0, SGI0, SGI109037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23977[] = {SGI109038, SGI109039, SGI109040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109041, SGI0, SGI0, SGI0, SGI109042, SGI109042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23978[] = {SGI109043, SGI109044, SGI109045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109046, SGI0, SGI0, SGI0, SGI109047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23979[] = {SGI109048, SGI109049, SGI109050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109051, SGI0, SGI0, SGI0, SGI109052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23980[] = {SGI109053, SGI109054, SGI109055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109056, SGI0, SGI0, SGI0, SGI109058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109061, SGI0, SGI0, NULL};
const Segment * const SG23981[] = {SGI109057, SGI109059, SGI109060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109062, SGI0, SGI0, SGI0, SGI109063, SGI109063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23982[] = {SGI109064, SGI109065, SGI109066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109067, SGI0, SGI0, SGI0, SGI109068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23983[] = {SGI109069, SGI109070, SGI109071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109072, SGI0, SGI0, SGI0, SGI109073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23984[] = {SGI109074, SGI109075, SGI109076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109077, SGI0, SGI0, SGI0, SGI109078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23985[] = {SGI109079, SGI109080, SGI109081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109082, SGI0, SGI0, SGI0, SGI109083, SGI109083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23986[] = {SGI109084, SGI109085, SGI109086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109087, SGI0, SGI0, SGI0, SGI109088, SGI109088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23987[] = {SGI109089, SGI109090, SGI109091, SGI109092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109093, SGI0, SGI0, SGI0, SGI109094, SGI109096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23988[] = {SGI109095, SGI109097, SGI109098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109099, SGI0, SGI0, SGI0, SGI109100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23989[] = {SGI109101, SGI109102, SGI109103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109104, SGI0, SGI0, SGI0, SGI109105, SGI109105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23990[] = {SGI109106, SGI109107, SGI109108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109109, SGI0, SGI0, SGI0, SGI109110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23991[] = {SGI109111, SGI109112, SGI109113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109114, SGI0, SGI0, SGI0, SGI109115, SGI109115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23992[] = {SGI109116, SGI109117, SGI109118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109119, SGI0, SGI0, SGI0, SGI109120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23993[] = {SGI109121, SGI109122, SGI109123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109124, SGI0, SGI0, SGI0, SGI109125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23994[] = {SGI109126, SGI109127, SGI109128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109129, SGI0, SGI0, SGI0, SGI109130, SGI109130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23995[] = {SGI109131, SGI109132, SGI109133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109134, SGI0, SGI0, SGI0, SGI109135, SGI109135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23996[] = {SGI109136, SGI109137, SGI109138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109139, SGI0, SGI0, SGI0, SGI109140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23997[] = {SGI109141, SGI109142, SGI109143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109144, SGI0, SGI0, SGI0, SGI109145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23998[] = {SGI109146, SGI109147, SGI109148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109149, SGI0, SGI0, SGI0, SGI109150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG23999[] = {SGI109151, SGI109152, SGI109153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109154, SGI0, SGI0, SGI0, SGI109155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24000[] = {SGI109156, SGI109157, SGI109158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109159, SGI0, SGI0, SGI0, SGI109160, SGI109160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24001[] = {SGI109161, SGI109162, SGI109163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109164, SGI0, SGI0, SGI0, SGI109165, SGI109165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24002[] = {SGI109166, SGI109167, SGI109168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109169, SGI0, SGI0, SGI0, SGI109170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24003[] = {SGI109171, SGI109172, SGI109173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109174, SGI0, SGI0, SGI0, SGI109175, SGI109175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24004[] = {SGI109176, SGI109177, SGI109178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109179, SGI0, SGI0, SGI0, SGI109180, SGI109180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24005[] = {SGI109181, SGI109182, SGI109183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109184, SGI0, SGI0, SGI0, SGI109185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24006[] = {SGI109186, SGI109187, SGI109188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109189, SGI0, SGI0, SGI0, SGI109190, SGI109190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24007[] = {SGI109191, SGI109192, SGI109193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109194, SGI0, SGI0, SGI0, SGI109195, SGI109195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24008[] = {SGI109196, SGI109197, SGI109198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109199, SGI0, SGI0, SGI0, SGI109200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24009[] = {SGI109201, SGI109202, SGI109203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109204, SGI0, SGI0, SGI0, SGI109205, SGI109205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24010[] = {SGI109206, SGI109207, SGI109208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109209, SGI0, SGI0, SGI0, SGI109210, SGI109210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24011[] = {SGI109211, SGI0, SGI109212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109213, SGI109214, SGI109214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24012[] = {SGI0, SGI109215, SGI109216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24013[] = {SGI109217, SGI0, SGI109218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109219, SGI109221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24014[] = {SGI109224, SGI0, SGI109225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109226, SGI109227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24015[] = {SGI109228, SGI0, SGI109229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109230, SGI109231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24016[] = {SGI109232, SGI109233, SGI109234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109235, SGI0, SGI0, SGI0, SGI109236, SGI0, SGI109237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24017[] = {SGI109238, SGI109239, SGI109240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109244, SGI0, SGI0, SGI0, SGI109246, SGI109246, SGI0, SGI109247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24018[] = {SGI109241, SGI109242, SGI109243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109245, SGI0, SGI0, SGI0, SGI109248, SGI109248, SGI109249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24019[] = {SGI109250, SGI109251, SGI109252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109253, SGI0, SGI0, SGI0, SGI109254, SGI0, SGI0, SGI109255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24020[] = {SGI109256, SGI0, SGI109257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109258, SGI109259, SGI109259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24021[] = {SGI109260, SGI0, SGI109261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109262, SGI109263, SGI109263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24022[] = {SGI109264, SGI0, SGI109265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109266, SGI109267, SGI109267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24023[] = {SGI109268, SGI109269, SGI109270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109271, SGI0, SGI0, SGI0, SGI109272, SGI109272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24024[] = {SGI109273, SGI0, SGI109274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109275, SGI109276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109277, SGI0, NULL};
const Segment * const SG24025[] = {SGI109220, SGI109222, SGI109223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109278, SGI0, SGI0, SGI0, SGI109281, SGI109281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24026[] = {SGI109279, SGI109280, SGI109282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109283, SGI0, SGI0, SGI0, SGI109284, SGI0, SGI0, SGI0, SGI0, SGI109285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24027[] = {SGI109286, SGI109287, SGI109288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109290, SGI0, SGI0, SGI0, SGI109293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24028[] = {SGI109289, SGI109291, SGI109292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109294, SGI0, SGI0, SGI0, SGI109295, SGI109295, SGI0, SGI0, SGI0, SGI109296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24029[] = {SGI109297, SGI109298, SGI109299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109300, SGI0, SGI0, SGI0, SGI109301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24030[] = {SGI0, SGI109302, SGI109303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24031[] = {SGI109304, SGI109305, SGI109306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109307, SGI0, SGI0, SGI0, SGI109308, SGI109308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24032[] = {SGI109309, SGI109310, SGI109311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109312, SGI0, SGI0, SGI0, SGI109313, SGI109313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24033[] = {SGI109314, SGI109315, SGI109316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109317, SGI0, SGI0, SGI0, SGI109318, SGI109318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24034[] = {SGI109319, SGI109320, SGI109321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109322, SGI0, SGI0, SGI0, SGI109323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24035[] = {SGI109324, SGI109325, SGI109326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109327, SGI0, SGI0, SGI0, SGI109328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24036[] = {SGI109329, SGI109330, SGI109331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109332, SGI0, SGI0, SGI0, SGI109333, SGI109333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24037[] = {SGI109334, SGI109335, SGI109336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109337, SGI0, SGI0, SGI0, SGI109338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24038[] = {SGI109339, SGI109340, SGI109341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109342, SGI0, SGI0, SGI0, SGI109343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24039[] = {SGI109344, SGI109345, SGI109346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109347, SGI0, SGI0, SGI0, SGI109348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24040[] = {SGI109349, SGI109350, SGI109351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109352, SGI0, SGI0, SGI0, SGI109353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24041[] = {SGI109354, SGI109355, SGI109356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109357, SGI0, SGI0, SGI0, SGI109358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24042[] = {SGI109359, SGI109360, SGI109361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109362, SGI0, SGI0, SGI0, SGI109363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24043[] = {SGI109364, SGI109365, SGI109366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109367, SGI0, SGI0, SGI0, SGI109368, SGI109368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24044[] = {SGI109369, SGI109370, SGI109371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109372, SGI0, SGI0, SGI0, SGI109373, SGI109373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24045[] = {SGI109374, SGI109375, SGI109376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109377, SGI0, SGI0, SGI0, SGI109378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24046[] = {SGI109379, SGI109380, SGI109381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109382, SGI0, SGI0, SGI0, SGI109383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24047[] = {SGI109384, SGI109385, SGI109386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109387, SGI0, SGI0, SGI0, SGI109388, SGI109388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24048[] = {SGI109389, SGI109390, SGI109391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109392, SGI0, SGI0, SGI0, SGI109393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24049[] = {SGI109394, SGI109395, SGI109396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109397, SGI0, SGI0, SGI0, SGI109398, SGI109398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24050[] = {SGI109399, SGI109400, SGI109401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109402, SGI0, SGI0, SGI0, SGI109403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24051[] = {SGI109404, SGI109405, SGI109406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109407, SGI0, SGI0, SGI0, SGI109408, SGI109408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24052[] = {SGI109409, SGI109410, SGI109411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109412, SGI0, SGI0, SGI0, SGI109413, SGI109413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24053[] = {SGI109414, SGI109415, SGI109416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109417, SGI0, SGI0, SGI0, SGI109418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24054[] = {SGI109419, SGI109420, SGI109421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109422, SGI0, SGI0, SGI0, SGI109423, SGI109423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24055[] = {SGI109424, SGI109425, SGI109426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109427, SGI0, SGI0, SGI0, SGI109428, SGI109428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24056[] = {SGI109429, SGI109430, SGI109431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109432, SGI0, SGI0, SGI0, SGI109433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24057[] = {SGI109434, SGI109435, SGI109436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109437, SGI0, SGI0, SGI0, SGI109438, SGI109438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24058[] = {SGI109439, SGI109440, SGI109441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109442, SGI0, SGI0, SGI0, SGI109443, SGI109443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24059[] = {SGI109444, SGI109445, SGI109446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109447, SGI0, SGI0, SGI0, SGI109448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24060[] = {SGI109449, SGI109450, SGI109451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109452, SGI0, SGI0, SGI0, SGI109453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24061[] = {SGI109454, SGI109455, SGI109456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109457, SGI0, SGI0, SGI0, SGI109458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24062[] = {SGI109459, SGI109460, SGI109461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109462, SGI0, SGI0, SGI0, SGI109463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24063[] = {SGI109464, SGI109465, SGI109466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109467, SGI109467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24064[] = {SGI109468, SGI109469, SGI109470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109471, SGI0, SGI0, SGI0, SGI109472, SGI109472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24065[] = {SGI109473, SGI109474, SGI109475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109476, SGI0, SGI0, SGI0, SGI109477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24066[] = {SGI109478, SGI109479, SGI109480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109481, SGI0, SGI0, SGI0, SGI109482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24067[] = {SGI109483, SGI0, SGI109484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109485, SGI0, SGI0, SGI0, SGI109486, SGI109486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24068[] = {SGI109487, SGI109488, SGI109489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109490, SGI0, SGI0, SGI0, SGI109491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24069[] = {SGI109492, SGI109493, SGI109494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109495, SGI0, SGI0, SGI0, SGI109496, SGI109496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24070[] = {SGI109497, SGI109498, SGI109499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109500, SGI0, SGI0, SGI0, SGI109501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24071[] = {SGI109502, SGI109503, SGI109504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109505, SGI0, SGI0, SGI0, SGI109506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24072[] = {SGI109507, SGI109508, SGI109509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109510, SGI0, SGI0, SGI0, SGI109511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24073[] = {SGI109512, SGI109513, SGI109514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109515, SGI0, SGI0, SGI0, SGI109516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24074[] = {SGI109517, SGI109518, SGI109519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109520, SGI0, SGI0, SGI0, SGI109521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24075[] = {SGI109522, SGI109523, SGI109524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109525, SGI0, SGI0, SGI0, SGI109526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24076[] = {SGI109527, SGI109528, SGI109529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109530, SGI0, SGI0, SGI0, SGI109531, SGI109531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24077[] = {SGI109532, SGI109533, SGI109534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109535, SGI0, SGI0, SGI0, SGI109536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24078[] = {SGI109537, SGI109538, SGI109539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109540, SGI0, SGI0, SGI0, SGI109541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24079[] = {SGI109542, SGI109543, SGI109544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109545, SGI0, SGI0, SGI0, SGI109546, SGI109546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24080[] = {SGI109547, SGI109548, SGI109549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109550, SGI0, SGI0, SGI0, SGI109551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24081[] = {SGI109552, SGI109553, SGI109554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109555, SGI0, SGI0, SGI0, SGI109556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24082[] = {SGI109557, SGI109558, SGI109559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109560, SGI0, SGI109561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24083[] = {SGI109562, SGI109563, SGI109564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109565, SGI0, SGI0, SGI0, SGI109566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24084[] = {SGI109567, SGI109568, SGI109569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109570, SGI0, SGI0, SGI0, SGI109571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24085[] = {SGI109572, SGI109573, SGI109574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109575, SGI0, SGI0, SGI0, SGI109576, SGI109576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24086[] = {SGI109577, SGI109578, SGI109579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109580, SGI0, SGI0, SGI0, SGI109581, SGI109581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24087[] = {SGI109582, SGI109583, SGI109584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109585, SGI0, SGI0, SGI0, SGI109586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24088[] = {SGI109587, SGI109588, SGI109589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109590, SGI0, SGI0, SGI0, SGI109591, SGI109591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24089[] = {SGI109592, SGI109594, SGI109595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109597, SGI0, SGI0, SGI0, SGI109599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24090[] = {SGI109593, SGI0, SGI109596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109598, SGI109600, SGI109600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24091[] = {SGI109601, SGI109602, SGI109603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109604, SGI0, SGI0, SGI0, SGI109605, SGI109605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24092[] = {SGI109606, SGI109607, SGI109608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109609, SGI0, SGI0, SGI0, SGI109610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24093[] = {SGI109611, SGI109612, SGI109613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109614, SGI0, SGI0, SGI0, SGI109615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24094[] = {SGI109616, SGI109617, SGI109618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109619, SGI0, SGI0, SGI0, SGI109620, SGI109620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24095[] = {SGI109621, SGI109622, SGI109623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109624, SGI0, SGI0, SGI0, SGI109625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24096[] = {SGI109626, SGI0, SGI109627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109628, SGI109629, SGI109629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24097[] = {SGI109630, SGI109631, SGI109632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109633, SGI0, SGI0, SGI0, SGI109634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24098[] = {SGI109635, SGI109636, SGI109637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109638, SGI0, SGI0, SGI0, SGI109640, SGI109640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24099[] = {SGI109639, SGI0, SGI109641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109642, SGI109643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24100[] = {SGI109644, SGI109645, SGI109646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109647, SGI0, SGI0, SGI0, SGI109649, SGI109649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24101[] = {SGI109648, SGI0, SGI109650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109651, SGI0, SGI0, SGI0, SGI109652, SGI109652, SGI0, SGI0, SGI109653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24102[] = {SGI109654, SGI109655, SGI109656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109657, SGI0, SGI0, SGI0, SGI109658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24103[] = {SGI109659, SGI109660, SGI109661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109662, SGI0, SGI0, SGI0, SGI109663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24104[] = {SGI109664, SGI109665, SGI109666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109667, SGI0, SGI0, SGI0, SGI109668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24105[] = {SGI109669, SGI109670, SGI109671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109672, SGI0, SGI0, SGI0, SGI109673, SGI109673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24106[] = {SGI109674, SGI109675, SGI109676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109677, SGI0, SGI0, SGI0, SGI109678, SGI109678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24107[] = {SGI109679, SGI109680, SGI109681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109682, SGI0, SGI0, SGI0, SGI109683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24108[] = {SGI109684, SGI109685, SGI109686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109687, SGI0, SGI0, SGI0, SGI109688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24109[] = {SGI109689, SGI109690, SGI109691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109692, SGI0, SGI0, SGI0, SGI109693, SGI109693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24110[] = {SGI109694, SGI109695, SGI109696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109697, SGI0, SGI0, SGI0, SGI109698, SGI109698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24111[] = {SGI109699, SGI109700, SGI109701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109702, SGI0, SGI0, SGI0, SGI109703, SGI109703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24112[] = {SGI109704, SGI0, SGI109705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109706, SGI109707, SGI109707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24113[] = {SGI109708, SGI109709, SGI109710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109711, SGI0, SGI0, SGI0, SGI109712, SGI109712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24114[] = {SGI109713, SGI109714, SGI109715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109716, SGI0, SGI0, SGI0, SGI109717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24115[] = {SGI109718, SGI109719, SGI109720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109721, SGI0, SGI0, SGI0, SGI109722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24116[] = {SGI0, SGI109723, SGI109724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109725, SGI109725, SGI0, SGI0, SGI109726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24117[] = {SGI109727, SGI109728, SGI109729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109730, SGI0, SGI0, SGI0, SGI109731, SGI109731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24118[] = {SGI109732, SGI0, SGI109733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24119[] = {SGI109734, SGI109735, SGI109736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109737, SGI0, SGI0, SGI0, SGI109738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109739, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24120[] = {SGI109740, SGI109741, SGI109742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109743, SGI0, SGI0, SGI0, SGI109744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109745, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24121[] = {SGI109746, SGI0, SGI109747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109748, SGI109749, SGI109749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24122[] = {SGI109750, SGI109751, SGI109752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109753, SGI109753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109754, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24123[] = {SGI109755, SGI109756, SGI109757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109758, SGI0, SGI0, SGI0, SGI109761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109762, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24124[] = {SGI109759, SGI0, SGI109760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109763, SGI109764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24125[] = {SGI109765, SGI109766, SGI109767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109768, SGI0, SGI0, SGI0, SGI109769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109770, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24126[] = {SGI109771, SGI109772, SGI109773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109775, SGI0, SGI109777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24127[] = {SGI109774, SGI0, SGI109776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109778, SGI0, SGI0, SGI0, SGI109779, SGI109779, SGI0, SGI109780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24128[] = {SGI109781, SGI0, SGI109782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109783, SGI109784, SGI109784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24129[] = {SGI109785, SGI0, SGI109786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109787, SGI0, SGI0, SGI0, SGI109788, SGI109788, SGI0, SGI109789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24130[] = {SGI109790, SGI0, SGI109791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109792, SGI109793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24131[] = {SGI109794, SGI109795, SGI109796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109797, SGI0, SGI0, SGI0, SGI109798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24132[] = {SGI109799, SGI109800, SGI109801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109802, SGI0, SGI0, SGI0, SGI109803, SGI109803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24133[] = {SGI109804, SGI109805, SGI109806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109807, SGI0, SGI0, SGI0, SGI109808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24134[] = {SGI109809, SGI0, SGI109810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109811, SGI109812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24135[] = {SGI109813, SGI109814, SGI109815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109816, SGI0, SGI0, SGI0, SGI109817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24136[] = {SGI109818, SGI109819, SGI109820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109821, SGI0, SGI0, SGI0, SGI109822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24137[] = {SGI109823, SGI109824, SGI109825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109826, SGI0, SGI0, SGI0, SGI109827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24138[] = {SGI109828, SGI109829, SGI109830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109831, SGI0, SGI0, SGI0, SGI109832, SGI109832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24139[] = {SGI109833, SGI109834, SGI109835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109836, SGI0, SGI0, SGI0, SGI109837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24140[] = {SGI109838, SGI109839, SGI109840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109841, SGI0, SGI0, SGI0, SGI109842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24141[] = {SGI109843, SGI109844, SGI109845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109846, SGI0, SGI0, SGI0, SGI109847, SGI109847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24142[] = {SGI109848, SGI109849, SGI109850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109851, SGI0, SGI0, SGI0, SGI109852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24143[] = {SGI109853, SGI109854, SGI109855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109856, SGI0, SGI0, SGI0, SGI109857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24144[] = {SGI109858, SGI109859, SGI109860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109861, SGI109861, SGI0, SGI0, SGI109862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24145[] = {SGI109863, SGI109864, SGI109865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109866, SGI0, SGI0, SGI0, SGI109867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24146[] = {SGI109868, SGI109869, SGI109870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109871, SGI0, SGI0, SGI0, SGI109872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24147[] = {SGI109873, SGI109874, SGI109875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109876, SGI0, SGI0, SGI0, SGI109877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24148[] = {SGI109878, SGI109879, SGI109880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109881, SGI0, SGI0, SGI0, SGI109882, SGI109882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24149[] = {SGI109883, SGI109884, SGI109885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109886, SGI0, SGI0, SGI0, SGI109887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24150[] = {SGI109888, SGI109889, SGI109890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109891, SGI0, SGI0, SGI0, SGI109892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24151[] = {SGI109893, SGI109894, SGI109895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109896, SGI0, SGI0, SGI0, SGI109897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24152[] = {SGI109898, SGI109899, SGI109900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109901, SGI0, SGI0, SGI0, SGI109902, SGI109902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24153[] = {SGI109903, SGI109904, SGI109905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109906, SGI0, SGI0, SGI0, SGI109907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24154[] = {SGI109908, SGI109909, SGI109910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109911, SGI0, SGI0, SGI0, SGI109912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24155[] = {SGI109913, SGI109914, SGI109915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109916, SGI0, SGI0, SGI0, SGI109917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24156[] = {SGI109918, SGI109919, SGI109920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109921, SGI0, SGI0, SGI0, SGI109922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24157[] = {SGI109923, SGI109924, SGI109925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109926, SGI0, SGI0, SGI0, SGI109927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24158[] = {SGI109928, SGI109929, SGI109930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109931, SGI0, SGI0, SGI0, SGI109932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24159[] = {SGI109933, SGI109934, SGI109935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109936, SGI0, SGI0, SGI0, SGI109937, SGI109937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24160[] = {SGI109938, SGI109939, SGI109940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109941, SGI0, SGI0, SGI0, SGI109942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24161[] = {SGI109943, SGI109944, SGI109945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109946, SGI0, SGI0, SGI0, SGI109947, SGI109947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24162[] = {SGI109948, SGI109949, SGI109950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109951, SGI0, SGI0, SGI0, SGI109952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24163[] = {SGI109953, SGI109954, SGI109955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109956, SGI0, SGI0, SGI0, SGI109957, SGI109957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24164[] = {SGI109958, SGI109959, SGI109960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109961, SGI0, SGI0, SGI0, SGI109962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24165[] = {SGI109963, SGI109964, SGI109965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109966, SGI0, SGI0, SGI0, SGI109967, SGI109967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24166[] = {SGI109968, SGI109969, SGI109970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109971, SGI0, SGI0, SGI0, SGI109972, SGI109972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24167[] = {SGI109973, SGI0, SGI109974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109978, SGI109979, SGI109979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24168[] = {SGI109975, SGI109976, SGI109977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109980, SGI0, SGI0, SGI0, SGI109981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24169[] = {SGI109982, SGI109983, SGI109984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109985, SGI0, SGI0, SGI0, SGI109988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24170[] = {SGI109986, SGI109987, SGI109989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109990, SGI0, SGI0, SGI0, SGI109991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24171[] = {SGI109992, SGI109993, SGI109994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI109995, SGI0, SGI0, SGI0, SGI109996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24172[] = {SGI109997, SGI109998, SGI109999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110000, SGI0, SGI0, SGI0, SGI110001, SGI110001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24173[] = {SGI110002, SGI110003, SGI110004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110005, SGI0, SGI0, SGI0, SGI110006, SGI110006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24174[] = {SGI110007, SGI110008, SGI110009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110010, SGI0, SGI0, SGI0, SGI110011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24175[] = {SGI110012, SGI110013, SGI110014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110015, SGI0, SGI0, SGI0, SGI110016, SGI110016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24176[] = {SGI110017, SGI110018, SGI110019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110020, SGI0, SGI0, SGI0, SGI110021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24177[] = {SGI110022, SGI110023, SGI110024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110025, SGI0, SGI0, SGI0, SGI110026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24178[] = {SGI110027, SGI110028, SGI110029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110030, SGI0, SGI0, SGI0, SGI110031, SGI110031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24179[] = {SGI110032, SGI110033, SGI110034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110035, SGI0, SGI0, SGI0, SGI110036, SGI0, SGI0, SGI0, SGI110039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24180[] = {SGI110037, SGI110038, SGI110040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110041, SGI0, SGI0, SGI0, SGI110042, SGI110042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24181[] = {SGI110043, SGI110044, SGI110045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110046, SGI0, SGI0, SGI0, SGI110047, SGI110047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24182[] = {SGI110048, SGI110049, SGI110050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110054, SGI0, SGI0, SGI0, SGI110056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110058, SGI0, SGI0, NULL};
const Segment * const SG24183[] = {SGI110051, SGI110052, SGI110053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110055, SGI0, SGI0, SGI0, SGI110057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24184[] = {SGI110059, SGI110060, SGI110061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110062, SGI0, SGI0, SGI0, SGI110063, SGI110063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110064, SGI0, SGI0, NULL};
const Segment * const SG24185[] = {SGI110065, SGI110066, SGI110067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110068, SGI0, SGI0, SGI0, SGI110069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24186[] = {SGI110070, SGI110071, SGI110072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110073, SGI0, SGI0, SGI0, SGI110074, SGI110074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24187[] = {SGI110075, SGI110076, SGI110077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110078, SGI0, SGI0, SGI0, SGI110080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24188[] = {SGI110079, SGI110081, SGI110082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110083, SGI0, SGI0, SGI0, SGI110084, SGI110084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110085, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24189[] = {SGI110086, SGI110087, SGI110088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110089, SGI0, SGI0, SGI0, SGI110090, SGI110090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24190[] = {SGI110091, SGI110092, SGI110093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110094, SGI0, SGI0, SGI0, SGI110095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110096, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24191[] = {SGI110097, SGI110098, SGI110099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110100, SGI0, SGI0, SGI0, SGI110101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24192[] = {SGI110102, SGI110103, SGI110104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110105, SGI0, SGI0, SGI0, SGI110106, SGI0, SGI0, SGI0, SGI110107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24193[] = {SGI110108, SGI110109, SGI110110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110111, SGI0, SGI0, SGI0, SGI110112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24194[] = {SGI110113, SGI110114, SGI110115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110116, SGI0, SGI0, SGI0, SGI110117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24195[] = {SGI110118, SGI110119, SGI110120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110121, SGI0, SGI0, SGI0, SGI110122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24196[] = {SGI110123, SGI110124, SGI110125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110126, SGI0, SGI0, SGI0, SGI110127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24197[] = {SGI110129, SGI110131, SGI110133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110135, SGI0, SGI0, SGI0, SGI110136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24198[] = {SGI110128, SGI110130, SGI110132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110134, SGI0, SGI0, SGI0, SGI110137, SGI0, SGI0, SGI110138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24199[] = {SGI110139, SGI110140, SGI110141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110142, SGI0, SGI0, SGI0, SGI110146, SGI0, SGI0, SGI110147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24200[] = {SGI110143, SGI110144, SGI110145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110148, SGI0, SGI0, SGI0, SGI110149, SGI110149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24201[] = {SGI110151, SGI0, SGI110153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24202[] = {SGI110150, SGI110152, SGI110154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110155, SGI0, SGI0, SGI0, SGI110156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110157, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24203[] = {SGI0, SGI0, SGI110158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110161, SGI110161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24204[] = {SGI110159, SGI110160, SGI110162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110163, SGI0, SGI0, SGI0, SGI110164, SGI110164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110165, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24205[] = {SGI110166, SGI110167, SGI110168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110169, SGI0, SGI0, SGI0, SGI110170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24206[] = {SGI110171, SGI110172, SGI110173, SGI110174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110175, SGI0, SGI0, SGI0, SGI110176, SGI110177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24207[] = {SGI110178, SGI110179, SGI110180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110181, SGI0, SGI0, SGI0, SGI110182, SGI110182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24208[] = {SGI110183, SGI110184, SGI110185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110186, SGI0, SGI0, SGI0, SGI110187, SGI110187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24209[] = {SGI110188, SGI110189, SGI110190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110191, SGI0, SGI0, SGI0, SGI110192, SGI110192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24210[] = {SGI110193, SGI110194, SGI110195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110196, SGI0, SGI0, SGI0, SGI110197, SGI0, SGI0, SGI0, SGI110198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24211[] = {SGI110199, SGI110200, SGI110201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110202, SGI0, SGI0, SGI0, SGI110203, SGI110203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24212[] = {SGI110204, SGI110205, SGI110206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110207, SGI0, SGI0, SGI0, SGI110208, SGI110208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24213[] = {SGI110209, SGI110210, SGI110211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110212, SGI0, SGI0, SGI0, SGI110216, SGI0, SGI110217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24214[] = {SGI110213, SGI110214, SGI110215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110218, SGI0, SGI0, SGI0, SGI110219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24215[] = {SGI110220, SGI110221, SGI110222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110223, SGI0, SGI0, SGI0, SGI110224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24216[] = {SGI110225, SGI110226, SGI110227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110228, SGI0, SGI0, SGI0, SGI110229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24217[] = {SGI110230, SGI110231, SGI110232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110233, SGI0, SGI0, SGI0, SGI110234, SGI110234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24218[] = {SGI110235, SGI110236, SGI110237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110238, SGI0, SGI0, SGI0, SGI110239, SGI110239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24219[] = {SGI110240, SGI110241, SGI110242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110243, SGI0, SGI0, SGI0, SGI110244, SGI110244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24220[] = {SGI110245, SGI110246, SGI110247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110248, SGI0, SGI0, SGI0, SGI110249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24221[] = {SGI110250, SGI110251, SGI110252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110253, SGI0, SGI0, SGI0, SGI110254, SGI110254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24222[] = {SGI110255, SGI110256, SGI110257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110258, SGI0, SGI0, SGI0, SGI110259, SGI110259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24223[] = {SGI110260, SGI110261, SGI110262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110263, SGI0, SGI0, SGI0, SGI110264, SGI110264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24224[] = {SGI110265, SGI110266, SGI110267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110268, SGI0, SGI0, SGI0, SGI110269, SGI110269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24225[] = {SGI110270, SGI110271, SGI110272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110273, SGI0, SGI0, SGI0, SGI110274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24226[] = {SGI110275, SGI110276, SGI110277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110278, SGI0, SGI0, SGI0, SGI110279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24227[] = {SGI110280, SGI110281, SGI110282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110283, SGI0, SGI0, SGI0, SGI110284, SGI110284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24228[] = {SGI110285, SGI110286, SGI110287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110288, SGI0, SGI0, SGI0, SGI110289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24229[] = {SGI110290, SGI110291, SGI110292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110293, SGI0, SGI0, SGI0, SGI110294, SGI110294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24230[] = {SGI110295, SGI110296, SGI110297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110298, SGI0, SGI0, SGI0, SGI110299, SGI110299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24231[] = {SGI110300, SGI110301, SGI110302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110303, SGI0, SGI0, SGI0, SGI110304, SGI110304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24232[] = {SGI110305, SGI110306, SGI110307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110308, SGI0, SGI0, SGI0, SGI110309, SGI110309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24233[] = {SGI110310, SGI110311, SGI110312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110313, SGI0, SGI0, SGI0, SGI110314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24234[] = {SGI110315, SGI110316, SGI110317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110318, SGI0, SGI0, SGI0, SGI110319, SGI110319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24235[] = {SGI110320, SGI110321, SGI110322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110323, SGI0, SGI0, SGI0, SGI110324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24236[] = {SGI110325, SGI110326, SGI110327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110328, SGI0, SGI0, SGI0, SGI110330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24237[] = {SGI110329, SGI0, SGI110331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110332, SGI110333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24238[] = {SGI110334, SGI110335, SGI110336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110337, SGI0, SGI0, SGI0, SGI110338, SGI110338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24239[] = {SGI110339, SGI110340, SGI110341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110342, SGI0, SGI0, SGI0, SGI110343, SGI110343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24240[] = {SGI110344, SGI110345, SGI110346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110347, SGI0, SGI0, SGI0, SGI110348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24241[] = {SGI110349, SGI110350, SGI110351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110352, SGI0, SGI0, SGI0, SGI110353, SGI110353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24242[] = {SGI110354, SGI110355, SGI110356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110357, SGI0, SGI0, SGI0, SGI110358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24243[] = {SGI110359, SGI110360, SGI110361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110362, SGI0, SGI0, SGI0, SGI110363, SGI110363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24244[] = {SGI110364, SGI110365, SGI110366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110367, SGI0, SGI0, SGI0, SGI110368, SGI110368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24245[] = {SGI110369, SGI110370, SGI110371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110372, SGI0, SGI0, SGI0, SGI110373, SGI110373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24246[] = {SGI110374, SGI0, SGI110375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110376, SGI110377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24247[] = {SGI110378, SGI110379, SGI110380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110381, SGI0, SGI0, SGI0, SGI110382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24248[] = {SGI110383, SGI110384, SGI110385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110386, SGI0, SGI0, SGI0, SGI110387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24249[] = {SGI110388, SGI110389, SGI110390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110391, SGI0, SGI0, SGI0, SGI110392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24250[] = {SGI110393, SGI110394, SGI110395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110396, SGI0, SGI0, SGI0, SGI110397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24251[] = {SGI110398, SGI110399, SGI110400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110401, SGI0, SGI0, SGI0, SGI110403, SGI110403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24252[] = {SGI110402, SGI0, SGI110404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110405, SGI110406, SGI110406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110407, SGI0, NULL};
const Segment * const SG24253[] = {SGI110408, SGI110409, SGI110410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110411, SGI0, SGI0, SGI0, SGI110412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24254[] = {SGI110413, SGI110414, SGI110415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110416, SGI0, SGI0, SGI0, SGI110417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24255[] = {SGI110418, SGI110419, SGI110420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110421, SGI0, SGI0, SGI0, SGI110422, SGI110422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24256[] = {SGI110423, SGI110424, SGI110425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110426, SGI0, SGI0, SGI0, SGI110427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24257[] = {SGI110428, SGI110429, SGI110430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110431, SGI0, SGI0, SGI0, SGI110432, SGI110432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110433, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24258[] = {SGI110434, SGI110435, SGI110436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110437, SGI0, SGI0, SGI0, SGI110438, SGI110438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24259[] = {SGI110439, SGI110440, SGI110441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110442, SGI0, SGI0, SGI0, SGI110443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24260[] = {SGI110444, SGI110445, SGI110446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110447, SGI0, SGI0, SGI0, SGI110448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24261[] = {SGI110449, SGI110450, SGI110451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110452, SGI0, SGI0, SGI0, SGI110453, SGI110453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24262[] = {SGI110454, SGI110455, SGI110456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110457, SGI0, SGI0, SGI0, SGI110458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24263[] = {SGI110459, SGI110460, SGI110461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110462, SGI0, SGI0, SGI0, SGI110463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24264[] = {SGI110464, SGI110465, SGI110466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110467, SGI0, SGI0, SGI0, SGI110468, SGI110468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24265[] = {SGI110469, SGI110470, SGI110471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110472, SGI0, SGI0, SGI0, SGI110473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24266[] = {SGI110474, SGI110475, SGI110476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110477, SGI0, SGI0, SGI0, SGI110478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24267[] = {SGI110479, SGI110480, SGI110481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110482, SGI0, SGI0, SGI0, SGI110483, SGI110483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24268[] = {SGI110484, SGI110485, SGI110486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110487, SGI0, SGI0, SGI0, SGI110488, SGI110488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24269[] = {SGI110489, SGI110490, SGI110491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110492, SGI0, SGI0, SGI0, SGI110493, SGI110493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24270[] = {SGI110494, SGI110495, SGI110496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110497, SGI0, SGI0, SGI0, SGI110498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24271[] = {SGI110499, SGI110500, SGI110501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110502, SGI0, SGI0, SGI0, SGI110503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24272[] = {SGI110504, SGI110505, SGI110506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110507, SGI0, SGI0, SGI0, SGI110508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24273[] = {SGI110509, SGI110510, SGI110511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110512, SGI0, SGI0, SGI0, SGI110513, SGI110513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24274[] = {SGI110514, SGI110515, SGI110516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110517, SGI0, SGI0, SGI0, SGI110518, SGI110518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24275[] = {SGI110519, SGI110520, SGI110521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110522, SGI0, SGI0, SGI0, SGI110523, SGI110523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24276[] = {SGI110524, SGI110525, SGI110526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110527, SGI0, SGI0, SGI0, SGI110528, SGI110528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24277[] = {SGI110529, SGI110530, SGI110531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110532, SGI0, SGI0, SGI0, SGI110533, SGI110533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24278[] = {SGI110534, SGI110535, SGI110536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110537, SGI0, SGI0, SGI0, SGI110538, SGI110538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24279[] = {SGI110539, SGI110540, SGI110541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110542, SGI0, SGI0, SGI0, SGI110543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24280[] = {SGI110544, SGI110545, SGI110546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110547, SGI0, SGI0, SGI0, SGI110548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24281[] = {SGI110549, SGI110550, SGI110551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110552, SGI0, SGI0, SGI0, SGI110553, SGI110553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24282[] = {SGI110554, SGI110555, SGI110556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110557, SGI0, SGI0, SGI0, SGI110558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24283[] = {SGI110559, SGI110560, SGI110561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110562, SGI0, SGI0, SGI0, SGI110563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24284[] = {SGI110564, SGI110565, SGI110566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110567, SGI0, SGI0, SGI0, SGI110568, SGI110568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24285[] = {SGI110569, SGI110570, SGI110571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110572, SGI0, SGI0, SGI0, SGI110573, SGI110573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24286[] = {SGI110574, SGI110575, SGI110576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110577, SGI0, SGI0, SGI0, SGI110578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24287[] = {SGI110579, SGI110580, SGI110581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110582, SGI0, SGI0, SGI0, SGI110583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24288[] = {SGI110584, SGI110585, SGI110586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110587, SGI0, SGI0, SGI0, SGI110588, SGI110588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24289[] = {SGI110589, SGI110590, SGI110591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110592, SGI0, SGI0, SGI0, SGI110593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24290[] = {SGI110594, SGI110595, SGI110596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110597, SGI0, SGI0, SGI0, SGI110598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24291[] = {SGI110599, SGI110600, SGI110601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110602, SGI0, SGI0, SGI0, SGI110603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24292[] = {SGI110604, SGI110605, SGI110606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110607, SGI0, SGI0, SGI0, SGI110608, SGI110608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24293[] = {SGI110609, SGI110610, SGI110611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110612, SGI0, SGI0, SGI0, SGI110613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24294[] = {SGI110614, SGI110615, SGI110616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110617, SGI0, SGI0, SGI0, SGI110618, SGI110618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24295[] = {SGI110619, SGI110620, SGI110621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110622, SGI0, SGI0, SGI0, SGI110623, SGI110623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24296[] = {SGI110624, SGI110625, SGI110626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110627, SGI0, SGI0, SGI0, SGI110628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24297[] = {SGI110629, SGI110630, SGI110631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110632, SGI0, SGI0, SGI0, SGI110633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24298[] = {SGI110634, SGI110635, SGI110636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110637, SGI0, SGI0, SGI0, SGI110638, SGI110638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24299[] = {SGI110639, SGI110640, SGI110641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110642, SGI0, SGI0, SGI0, SGI110643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24300[] = {SGI110644, SGI110645, SGI110646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110647, SGI0, SGI0, SGI0, SGI110648, SGI110648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24301[] = {SGI110649, SGI110650, SGI110651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110652, SGI0, SGI0, SGI0, SGI110653, SGI110653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24302[] = {SGI110654, SGI110655, SGI110656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110657, SGI0, SGI0, SGI0, SGI110658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24303[] = {SGI110659, SGI110660, SGI110661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110662, SGI0, SGI0, SGI0, SGI110663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24304[] = {SGI110664, SGI110665, SGI110666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110667, SGI0, SGI0, SGI0, SGI110668, SGI110668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24305[] = {SGI110669, SGI110670, SGI110671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110672, SGI0, SGI0, SGI0, SGI110673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24306[] = {SGI110674, SGI110675, SGI110676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110677, SGI0, SGI0, SGI0, SGI110678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24307[] = {SGI110679, SGI110680, SGI110681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110682, SGI0, SGI0, SGI0, SGI110683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24308[] = {SGI110684, SGI110685, SGI110686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110687, SGI0, SGI0, SGI0, SGI110688, SGI110688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24309[] = {SGI110689, SGI110690, SGI110691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110692, SGI0, SGI0, SGI0, SGI110693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110694, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24310[] = {SGI110695, SGI110696, SGI110697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110698, SGI0, SGI0, SGI0, SGI110699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24311[] = {SGI110700, SGI110701, SGI110702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110703, SGI0, SGI0, SGI0, SGI110704, SGI110704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24312[] = {SGI110705, SGI110706, SGI110707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110708, SGI0, SGI0, SGI0, SGI110709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24313[] = {SGI110710, SGI110711, SGI110712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110713, SGI0, SGI0, SGI0, SGI110714, SGI110714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24314[] = {SGI110715, SGI110716, SGI110717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110718, SGI0, SGI0, SGI0, SGI110719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24315[] = {SGI110720, SGI110721, SGI110722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110723, SGI0, SGI0, SGI0, SGI110724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24316[] = {SGI0, SGI110725, SGI110726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24317[] = {SGI110727, SGI110728, SGI110729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110730, SGI0, SGI0, SGI0, SGI110731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24318[] = {SGI110732, SGI110733, SGI110734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110735, SGI0, SGI0, SGI0, SGI110736, SGI110736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24319[] = {SGI110737, SGI110738, SGI110739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110740, SGI0, SGI0, SGI0, SGI110741, SGI110741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24320[] = {SGI110742, SGI110743, SGI110744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110745, SGI0, SGI0, SGI0, SGI110746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24321[] = {SGI0, SGI0, SGI110747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110748, SGI0, SGI0, SGI0, SGI110749, SGI110749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24322[] = {SGI110750, SGI110751, SGI110752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110753, SGI0, SGI0, SGI0, SGI110754, SGI110754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24323[] = {SGI110755, SGI110756, SGI110757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110758, SGI0, SGI0, SGI0, SGI110759, SGI110759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24324[] = {SGI110760, SGI110761, SGI110762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110763, SGI0, SGI0, SGI0, SGI110764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24325[] = {SGI110765, SGI110766, SGI110767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110768, SGI0, SGI0, SGI0, SGI110769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24326[] = {SGI110770, SGI110771, SGI110772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110773, SGI0, SGI0, SGI0, SGI110774, SGI110774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24327[] = {SGI110775, SGI110776, SGI110777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110778, SGI0, SGI0, SGI0, SGI110779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24328[] = {SGI110780, SGI110781, SGI110782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110783, SGI0, SGI0, SGI0, SGI110784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24329[] = {SGI110785, SGI110786, SGI110787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110788, SGI0, SGI0, SGI0, SGI110789, SGI110789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24330[] = {SGI110790, SGI110791, SGI110792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110793, SGI0, SGI0, SGI0, SGI110794, SGI110794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24331[] = {SGI110795, SGI110797, SGI110799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110802, SGI0, SGI0, SGI0, SGI110804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110805, SGI0, SGI0, NULL};
const Segment * const SG24332[] = {SGI110796, SGI110798, SGI110800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110801, SGI0, SGI0, SGI0, SGI110803, SGI110803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24333[] = {SGI110806, SGI110807, SGI110808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110809, SGI0, SGI0, SGI0, SGI110810, SGI110810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24334[] = {SGI110811, SGI110812, SGI110813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110814, SGI0, SGI0, SGI0, SGI110815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24335[] = {SGI110816, SGI110817, SGI110818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110819, SGI0, SGI0, SGI0, SGI110820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24336[] = {SGI110821, SGI110822, SGI110823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110824, SGI0, SGI0, SGI0, SGI110825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24337[] = {SGI110827, SGI110828, SGI110829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110831, SGI0, SGI0, SGI0, SGI110834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24338[] = {SGI110830, SGI110832, SGI110833, SGI110835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110836, SGI0, SGI0, SGI0, SGI110837, SGI110838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24339[] = {SGI110839, SGI110840, SGI110841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110842, SGI0, SGI0, SGI0, SGI110843, SGI110843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24340[] = {SGI110844, SGI110845, SGI110846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110847, SGI0, SGI0, SGI0, SGI110848, SGI110848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24341[] = {SGI110849, SGI110850, SGI110851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110852, SGI0, SGI0, SGI0, SGI110853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24342[] = {SGI110854, SGI110855, SGI110856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110857, SGI0, SGI0, SGI0, SGI110858, SGI110858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24343[] = {SGI110859, SGI110860, SGI110861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110862, SGI0, SGI0, SGI0, SGI110864, SGI110864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24344[] = {SGI110863, SGI110865, SGI110866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110867, SGI0, SGI0, SGI0, SGI110868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24345[] = {SGI110869, SGI110870, SGI110871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110872, SGI0, SGI0, SGI0, SGI110873, SGI110873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24346[] = {SGI110874, SGI110875, SGI110876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110877, SGI0, SGI0, SGI0, SGI110878, SGI110878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24347[] = {SGI110879, SGI110880, SGI110881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110882, SGI0, SGI0, SGI0, SGI110883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24348[] = {SGI110884, SGI110885, SGI110886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110887, SGI0, SGI0, SGI0, SGI110888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24349[] = {SGI110889, SGI110890, SGI110891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110892, SGI0, SGI0, SGI0, SGI110893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24350[] = {SGI110894, SGI110895, SGI110896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110897, SGI0, SGI0, SGI0, SGI110898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24351[] = {SGI110899, SGI110900, SGI110901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110902, SGI0, SGI0, SGI0, SGI110903, SGI110903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24352[] = {SGI110904, SGI110905, SGI110906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110907, SGI0, SGI0, SGI0, SGI110908, SGI110908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24353[] = {SGI110909, SGI110910, SGI110911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110912, SGI0, SGI0, SGI0, SGI110913, SGI110913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24354[] = {SGI110914, SGI110915, SGI110916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110917, SGI0, SGI0, SGI0, SGI110918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24355[] = {SGI110919, SGI110920, SGI110921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110923, SGI0, SGI0, SGI0, SGI110925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24356[] = {SGI110922, SGI110924, SGI110926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110927, SGI0, SGI0, SGI0, SGI110928, SGI0, SGI0, SGI0, SGI0, SGI110929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24357[] = {SGI110930, SGI110931, SGI110932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110933, SGI0, SGI0, SGI0, SGI110934, SGI110934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24358[] = {SGI110935, SGI0, SGI110936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110937, SGI0, SGI0, SGI0, SGI110938, SGI110938, SGI0, SGI110939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24359[] = {SGI110940, SGI110941, SGI110942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110943, SGI0, SGI0, SGI0, SGI110944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24360[] = {SGI110945, SGI110946, SGI110947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110948, SGI0, SGI0, SGI0, SGI110949, SGI110949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24361[] = {SGI110950, SGI110951, SGI110952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110953, SGI0, SGI0, SGI0, SGI110954, SGI0, SGI0, SGI0, SGI0, SGI110955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24362[] = {SGI110956, SGI110957, SGI110958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110959, SGI0, SGI0, SGI0, SGI110962, SGI0, SGI0, SGI0, SGI0, SGI110964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24363[] = {SGI110960, SGI110961, SGI110963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110965, SGI0, SGI0, SGI0, SGI110966, SGI110966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24364[] = {SGI110967, SGI110968, SGI110969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110970, SGI0, SGI0, SGI0, SGI110971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24365[] = {SGI110972, SGI110973, SGI110974, SGI110975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110976, SGI0, SGI0, SGI0, SGI110977, SGI110978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24366[] = {SGI110979, SGI110980, SGI110981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110982, SGI0, SGI0, SGI0, SGI110983, SGI110983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24367[] = {SGI110984, SGI110985, SGI110986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110987, SGI0, SGI0, SGI0, SGI110988, SGI110988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24368[] = {SGI110989, SGI110990, SGI110991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI110992, SGI0, SGI0, SGI0, SGI110993, SGI110993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24369[] = {SGI110994, SGI110995, SGI110996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111000, SGI0, SGI0, SGI0, SGI111002, SGI0, SGI0, SGI0, SGI111004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24370[] = {SGI110997, SGI110998, SGI110999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111001, SGI0, SGI0, SGI0, SGI111003, SGI111003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24371[] = {SGI111005, SGI111006, SGI111007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111008, SGI0, SGI0, SGI0, SGI111009, SGI111009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24372[] = {SGI111010, SGI111011, SGI111012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111013, SGI0, SGI0, SGI0, SGI111014, SGI111014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24373[] = {SGI111015, SGI111016, SGI111017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111018, SGI0, SGI0, SGI0, SGI111019, SGI111019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24374[] = {SGI111020, SGI111021, SGI111022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111023, SGI0, SGI0, SGI0, SGI111024, SGI0, SGI0, SGI0, SGI111025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24375[] = {SGI111026, SGI111027, SGI111028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111029, SGI0, SGI0, SGI0, SGI111030, SGI111030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24376[] = {SGI111031, SGI111032, SGI111033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111034, SGI111034, SGI0, SGI111035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24377[] = {SGI111036, SGI111037, SGI111038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111039, SGI0, SGI0, SGI0, SGI111040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24378[] = {SGI111041, SGI111042, SGI111043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111044, SGI0, SGI0, SGI0, SGI111045, SGI111045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24379[] = {SGI111046, SGI111047, SGI111048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111049, SGI0, SGI0, SGI0, SGI111050, SGI111050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24380[] = {SGI111051, SGI111052, SGI111053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111054, SGI0, SGI0, SGI0, SGI111055, SGI111055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24381[] = {SGI111056, SGI111057, SGI111058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111059, SGI0, SGI0, SGI0, SGI111060, SGI0, SGI0, SGI0, SGI111061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24382[] = {SGI111062, SGI111063, SGI111064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111065, SGI0, SGI0, SGI0, SGI111066, SGI111066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24383[] = {SGI111067, SGI111069, SGI111070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111073, SGI0, SGI0, SGI0, SGI111076, SGI111076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111077, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24384[] = {SGI111068, SGI111071, SGI111072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111074, SGI0, SGI0, SGI0, SGI111075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24385[] = {SGI111078, SGI111079, SGI111080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111081, SGI0, SGI0, SGI0, SGI111082, SGI111082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24386[] = {SGI111083, SGI111084, SGI111085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111086, SGI0, SGI0, SGI0, SGI111087, SGI111087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24387[] = {SGI111088, SGI111089, SGI111091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111094, SGI0, SGI0, SGI0, SGI111096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111098, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24388[] = {SGI111090, SGI111092, SGI111093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111095, SGI0, SGI0, SGI0, SGI111097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24389[] = {SGI111099, SGI111101, SGI111103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111105, SGI0, SGI0, SGI0, SGI111108, SGI111108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111109, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24390[] = {SGI111100, SGI111102, SGI111104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111106, SGI0, SGI0, SGI0, SGI111107, SGI111107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24391[] = {SGI111110, SGI111111, SGI111112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111116, SGI0, SGI0, SGI0, SGI111117, SGI111117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24392[] = {SGI111113, SGI111114, SGI111115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111118, SGI0, SGI0, SGI0, SGI111119, SGI0, SGI111120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24393[] = {SGI111121, SGI111122, SGI111123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111124, SGI0, SGI0, SGI0, SGI111125, SGI111125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24394[] = {SGI111126, SGI111127, SGI111128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111129, SGI0, SGI0, SGI0, SGI111130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24395[] = {SGI111131, SGI111133, SGI111134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111135, SGI0, SGI0, SGI0, SGI111136, SGI111136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24396[] = {SGI111137, SGI111138, SGI111139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111140, SGI0, SGI0, SGI0, SGI111141, SGI111141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24397[] = {SGI111142, SGI111143, SGI111144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111145, SGI0, SGI0, SGI0, SGI111146, SGI111146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24398[] = {SGI111147, SGI111148, SGI111149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111150, SGI0, SGI0, SGI0, SGI111151, SGI0, SGI111152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24399[] = {SGI111153, SGI111154, SGI111155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111156, SGI0, SGI0, SGI0, SGI111157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24400[] = {SGI111158, SGI111159, SGI111160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111161, SGI0, SGI0, SGI0, SGI111162, SGI111162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24401[] = {SGI111163, SGI111164, SGI111165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111166, SGI0, SGI0, SGI0, SGI111167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24402[] = {SGI111169, SGI111170, SGI111171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111172, SGI0, SGI0, SGI0, SGI111173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24403[] = {SGI111174, SGI111175, SGI111176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111177, SGI0, SGI0, SGI0, SGI111178, SGI111178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24404[] = {SGI111179, SGI111180, SGI111181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111182, SGI0, SGI0, SGI0, SGI111183, SGI111183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24405[] = {SGI111184, SGI111185, SGI111186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111187, SGI0, SGI0, SGI0, SGI111188, SGI111188, SGI0, SGI111189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24406[] = {SGI111190, SGI111191, SGI111192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111193, SGI0, SGI0, SGI0, SGI111194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24407[] = {SGI111195, SGI111197, SGI111199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111201, SGI0, SGI0, SGI0, SGI111202, SGI111202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24408[] = {SGI111196, SGI111198, SGI111200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111203, SGI111203, SGI0, SGI111204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24409[] = {SGI111205, SGI111206, SGI111207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111208, SGI0, SGI0, SGI0, SGI111209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24410[] = {SGI111210, SGI111211, SGI111212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111213, SGI0, SGI0, SGI0, SGI111214, SGI111214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24411[] = {SGI111215, SGI111216, SGI111217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111218, SGI0, SGI0, SGI0, SGI111219, SGI111219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24412[] = {SGI111220, SGI111221, SGI111222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111223, SGI0, SGI0, SGI0, SGI111224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24413[] = {SGI111225, SGI111226, SGI111227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111228, SGI0, SGI0, SGI0, SGI111229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24414[] = {SGI111230, SGI111231, SGI111232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111233, SGI0, SGI0, SGI0, SGI111234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24415[] = {SGI111235, SGI111236, SGI111237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111238, SGI0, SGI0, SGI0, SGI111239, SGI111239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24416[] = {SGI111240, SGI111241, SGI111242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111243, SGI0, SGI0, SGI0, SGI111244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24417[] = {SGI111245, SGI111246, SGI111247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111248, SGI0, SGI0, SGI0, SGI111249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24418[] = {SGI111250, SGI111251, SGI111252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111253, SGI0, SGI0, SGI0, SGI111254, SGI111254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24419[] = {SGI111255, SGI111256, SGI111257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111258, SGI0, SGI0, SGI0, SGI111259, SGI111259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24420[] = {SGI111260, SGI111261, SGI111262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111263, SGI0, SGI0, SGI0, SGI111264, SGI111264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24421[] = {SGI111265, SGI111266, SGI111267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111268, SGI0, SGI0, SGI0, SGI111269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24422[] = {SGI111270, SGI111271, SGI111272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111273, SGI0, SGI0, SGI0, SGI111274, SGI111274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24423[] = {SGI111275, SGI111276, SGI111277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111278, SGI0, SGI0, SGI0, SGI111279, SGI111279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24424[] = {SGI111280, SGI111281, SGI111282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111283, SGI0, SGI0, SGI0, SGI111284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24425[] = {SGI111285, SGI111286, SGI111287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111288, SGI0, SGI0, SGI0, SGI111289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24426[] = {SGI111290, SGI111291, SGI111292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111293, SGI111293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24427[] = {SGI111294, SGI111295, SGI111296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111297, SGI0, SGI0, SGI0, SGI111298, SGI111298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24428[] = {SGI111299, SGI111300, SGI111301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111302, SGI0, SGI0, SGI0, SGI111303, SGI111303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24429[] = {SGI111304, SGI111305, SGI111306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111307, SGI0, SGI0, SGI0, SGI111308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24430[] = {SGI111309, SGI111310, SGI111311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111312, SGI0, SGI0, SGI0, SGI111313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24431[] = {SGI111314, SGI111315, SGI111316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111317, SGI0, SGI0, SGI0, SGI111318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24432[] = {SGI111319, SGI111320, SGI111321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111322, SGI111322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24433[] = {SGI111323, SGI111324, SGI111325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111326, SGI0, SGI0, SGI0, SGI111327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24434[] = {SGI111328, SGI111329, SGI111330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111331, SGI0, SGI0, SGI0, SGI111332, SGI111332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24435[] = {SGI111333, SGI111334, SGI111335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111336, SGI0, SGI0, SGI0, SGI111337, SGI111337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24436[] = {SGI111338, SGI111339, SGI111340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111341, SGI0, SGI0, SGI0, SGI111342, SGI111342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24437[] = {SGI111343, SGI111344, SGI111345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111346, SGI0, SGI0, SGI0, SGI111347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24438[] = {SGI111348, SGI111349, SGI111350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111351, SGI0, SGI0, SGI0, SGI111352, SGI111352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24439[] = {SGI111353, SGI111354, SGI111355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111356, SGI0, SGI0, SGI0, SGI111357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24440[] = {SGI111358, SGI111359, SGI111360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111361, SGI0, SGI0, SGI0, SGI111362, SGI111362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24441[] = {SGI111363, SGI111364, SGI111365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111366, SGI0, SGI0, SGI0, SGI111367, SGI111367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24442[] = {SGI111368, SGI111369, SGI111370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111371, SGI0, SGI0, SGI0, SGI111372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24443[] = {SGI111373, SGI111374, SGI111375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111376, SGI0, SGI0, SGI0, SGI111377, SGI111377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24444[] = {SGI111378, SGI111379, SGI111380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111381, SGI0, SGI0, SGI0, SGI111382, SGI111382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24445[] = {SGI111383, SGI111384, SGI111385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111386, SGI0, SGI0, SGI0, SGI111387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24446[] = {SGI111388, SGI111389, SGI111390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111391, SGI0, SGI0, SGI0, SGI111392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24447[] = {SGI111393, SGI111394, SGI111395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111396, SGI111396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24448[] = {SGI111397, SGI111398, SGI111399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111400, SGI0, SGI0, SGI0, SGI111401, SGI111401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24449[] = {SGI111402, SGI111403, SGI111404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111405, SGI0, SGI0, SGI0, SGI111406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24450[] = {SGI111407, SGI111408, SGI111409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111410, SGI0, SGI0, SGI0, SGI111411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24451[] = {SGI111412, SGI111413, SGI111414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111415, SGI0, SGI0, SGI0, SGI111416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24452[] = {SGI111417, SGI111418, SGI111419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111420, SGI0, SGI0, SGI0, SGI111421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24453[] = {SGI111422, SGI111423, SGI111424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111425, SGI0, SGI0, SGI0, SGI111426, SGI111426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24454[] = {SGI111427, SGI111428, SGI111429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111430, SGI0, SGI0, SGI0, SGI111431, SGI111431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24455[] = {SGI111432, SGI111433, SGI111434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111435, SGI0, SGI0, SGI0, SGI111436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24456[] = {SGI111437, SGI111438, SGI111439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111440, SGI0, SGI0, SGI0, SGI111441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24457[] = {SGI111442, SGI111443, SGI111444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111445, SGI0, SGI0, SGI0, SGI111446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24458[] = {SGI111447, SGI111448, SGI111449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111450, SGI0, SGI0, SGI0, SGI111451, SGI111451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24459[] = {SGI111452, SGI111453, SGI111454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111455, SGI0, SGI0, SGI0, SGI111456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24460[] = {SGI111457, SGI111458, SGI111459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111460, SGI0, SGI0, SGI0, SGI111461, SGI111461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24461[] = {SGI111462, SGI111463, SGI111464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111465, SGI0, SGI0, SGI0, SGI111466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24462[] = {SGI111467, SGI111468, SGI111469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111470, SGI0, SGI0, SGI0, SGI111471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24463[] = {SGI111472, SGI111473, SGI111474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111475, SGI0, SGI0, SGI0, SGI111476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24464[] = {SGI111477, SGI111478, SGI111479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111480, SGI0, SGI0, SGI0, SGI111481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24465[] = {SGI111482, SGI111483, SGI111484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111485, SGI0, SGI0, SGI0, SGI111486, SGI111486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24466[] = {SGI111487, SGI111488, SGI111489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111490, SGI0, SGI0, SGI0, SGI111491, SGI111491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24467[] = {SGI111492, SGI111493, SGI111494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111495, SGI0, SGI0, SGI0, SGI111496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24468[] = {SGI111497, SGI111498, SGI111499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111500, SGI0, SGI0, SGI0, SGI111501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24469[] = {SGI111502, SGI111503, SGI111504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111505, SGI0, SGI0, SGI0, SGI111506, SGI111506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24470[] = {SGI111507, SGI111508, SGI111509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111510, SGI0, SGI0, SGI0, SGI111511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24471[] = {SGI111512, SGI0, SGI111513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111514, SGI111515, SGI111515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24472[] = {SGI111516, SGI111517, SGI111518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111519, SGI0, SGI0, SGI0, SGI111520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24473[] = {SGI111521, SGI111522, SGI111523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111524, SGI0, SGI0, SGI0, SGI111525, SGI111525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24474[] = {SGI111526, SGI111527, SGI111528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111529, SGI0, SGI0, SGI0, SGI111530, SGI111530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24475[] = {SGI111531, SGI111532, SGI111533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111534, SGI0, SGI0, SGI0, SGI111535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24476[] = {SGI111536, SGI111537, SGI111538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111539, SGI0, SGI0, SGI0, SGI111540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24477[] = {SGI111541, SGI111542, SGI111543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111544, SGI0, SGI0, SGI0, SGI111545, SGI111545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24478[] = {SGI111546, SGI111547, SGI111548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111549, SGI0, SGI0, SGI0, SGI111550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24479[] = {SGI111551, SGI111552, SGI111553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111554, SGI0, SGI0, SGI0, SGI111555, SGI111555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24480[] = {SGI111556, SGI111557, SGI111558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111559, SGI0, SGI0, SGI0, SGI111560, SGI111560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24481[] = {SGI111561, SGI111562, SGI111563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111564, SGI0, SGI0, SGI0, SGI111565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24482[] = {SGI111566, SGI111567, SGI111568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111569, SGI0, SGI0, SGI0, SGI111570, SGI111570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24483[] = {SGI111571, SGI111572, SGI111573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111574, SGI0, SGI0, SGI0, SGI111575, SGI111575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24484[] = {SGI111576, SGI111577, SGI111578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111579, SGI0, SGI0, SGI0, SGI111580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24485[] = {SGI111581, SGI111582, SGI111583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111584, SGI0, SGI0, SGI0, SGI111585, SGI111585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24486[] = {SGI111586, SGI111587, SGI111588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111589, SGI0, SGI0, SGI0, SGI111590, SGI111590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24487[] = {SGI111591, SGI111592, SGI111593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111594, SGI0, SGI0, SGI0, SGI111595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24488[] = {SGI111596, SGI111597, SGI111598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111599, SGI0, SGI0, SGI0, SGI111600, SGI111600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24489[] = {SGI111601, SGI111602, SGI111603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111604, SGI0, SGI0, SGI0, SGI111605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24490[] = {SGI111606, SGI111607, SGI111608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111609, SGI0, SGI0, SGI0, SGI111610, SGI111610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24491[] = {SGI111611, SGI111612, SGI111613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111614, SGI0, SGI0, SGI0, SGI111615, SGI111615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24492[] = {SGI111616, SGI111617, SGI111618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111619, SGI0, SGI0, SGI0, SGI111620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24493[] = {SGI111621, SGI111622, SGI111623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111624, SGI0, SGI0, SGI0, SGI111626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24494[] = {SGI0, SGI111625, SGI111627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111628, SGI0, SGI0, SGI0, SGI111629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24495[] = {SGI111630, SGI0, SGI111631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24496[] = {SGI111632, SGI111633, SGI111634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111635, SGI0, SGI0, SGI0, SGI111636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24497[] = {SGI111637, SGI111638, SGI111639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111640, SGI0, SGI0, SGI0, SGI111641, SGI111641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24498[] = {SGI111642, SGI111643, SGI111644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111645, SGI0, SGI0, SGI0, SGI111646, SGI111646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24499[] = {SGI111647, SGI111648, SGI111649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111650, SGI0, SGI0, SGI0, SGI111651, SGI111651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24500[] = {SGI111652, SGI0, SGI111653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111657, SGI111658, SGI111658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24501[] = {SGI111654, SGI111655, SGI111656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111659, SGI0, SGI0, SGI0, SGI111660, SGI111660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24502[] = {SGI111661, SGI111662, SGI111663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111664, SGI0, SGI0, SGI0, SGI111665, SGI111665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24503[] = {SGI111666, SGI111667, SGI111668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111671, SGI0, SGI0, SGI0, SGI111672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24504[] = {SGI111669, SGI0, SGI111670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111673, SGI111674, SGI111674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24505[] = {SGI111675, SGI111676, SGI111677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111678, SGI0, SGI0, SGI0, SGI111679, SGI111679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24506[] = {SGI111680, SGI111681, SGI111682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111683, SGI0, SGI0, SGI0, SGI111684, SGI111684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24507[] = {SGI111685, SGI111686, SGI111687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111688, SGI0, SGI0, SGI0, SGI111689, SGI111689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24508[] = {SGI111690, SGI111691, SGI111692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111693, SGI0, SGI0, SGI0, SGI111694, SGI111694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24509[] = {SGI111695, SGI111696, SGI111697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111698, SGI0, SGI0, SGI0, SGI111699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24510[] = {SGI111700, SGI111701, SGI111702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111703, SGI0, SGI0, SGI0, SGI111704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24511[] = {SGI111705, SGI111706, SGI111707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111710, SGI0, SGI0, SGI0, SGI111711, SGI111711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24512[] = {SGI111708, SGI0, SGI111709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111712, SGI0, SGI0, SGI0, SGI111713, SGI111713, SGI0, SGI0, SGI111714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24513[] = {SGI111715, SGI111716, SGI111717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111718, SGI0, SGI0, SGI0, SGI111720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24514[] = {SGI0, SGI0, SGI111719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111721, SGI0, SGI0, SGI0, SGI111722, SGI111722, SGI0, SGI0, SGI111723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24515[] = {SGI111724, SGI111725, SGI111726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111727, SGI0, SGI0, SGI0, SGI111728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24516[] = {SGI111729, SGI0, SGI111730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111733, SGI111735, SGI111735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24517[] = {SGI111731, SGI111732, SGI111734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111736, SGI0, SGI0, SGI0, SGI111737, SGI111737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24518[] = {SGI111738, SGI111739, SGI111740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111741, SGI0, SGI0, SGI0, SGI111742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24519[] = {SGI111743, SGI111744, SGI111745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111746, SGI0, SGI0, SGI0, SGI111747, SGI111747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24520[] = {SGI111748, SGI111749, SGI111750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111751, SGI0, SGI0, SGI0, SGI111752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24521[] = {SGI111753, SGI111754, SGI111755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111756, SGI111756, SGI0, SGI0, SGI111757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24522[] = {SGI111758, SGI111759, SGI111760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111761, SGI0, SGI0, SGI0, SGI111762, SGI111762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24523[] = {SGI111763, SGI0, SGI111764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111765, SGI111766, SGI111766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24524[] = {SGI111767, SGI111768, SGI111769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111770, SGI0, SGI0, SGI0, SGI111771, SGI111771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24525[] = {SGI111772, SGI111773, SGI111774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111775, SGI0, SGI0, SGI0, SGI111776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24526[] = {SGI111777, SGI111778, SGI111779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111780, SGI0, SGI0, SGI0, SGI111781, SGI111781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24527[] = {SGI111782, SGI111783, SGI111784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111785, SGI0, SGI0, SGI0, SGI111786, SGI111786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24528[] = {SGI111787, SGI111788, SGI111789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111790, SGI0, SGI0, SGI0, SGI111791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24529[] = {SGI111792, SGI111793, SGI111794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111795, SGI0, SGI0, SGI0, SGI111796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24530[] = {SGI111797, SGI111798, SGI111799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111800, SGI0, SGI0, SGI0, SGI111801, SGI111801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24531[] = {SGI111802, SGI111803, SGI111804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111805, SGI0, SGI0, SGI0, SGI111806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24532[] = {SGI111807, SGI111808, SGI111809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111810, SGI0, SGI0, SGI0, SGI111811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24533[] = {SGI111812, SGI111813, SGI111814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111815, SGI0, SGI0, SGI0, SGI111816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24534[] = {SGI111817, SGI111818, SGI111819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111820, SGI0, SGI0, SGI0, SGI111821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24535[] = {SGI111822, SGI111823, SGI111824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111825, SGI0, SGI0, SGI0, SGI111826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24536[] = {SGI111827, SGI111828, SGI111829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111830, SGI0, SGI0, SGI0, SGI111831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24537[] = {SGI111832, SGI111833, SGI111834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111835, SGI0, SGI0, SGI0, SGI111836, SGI111836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24538[] = {SGI111837, SGI111838, SGI111839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111840, SGI0, SGI0, SGI0, SGI111841, SGI111841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24539[] = {SGI111842, SGI111843, SGI111844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111845, SGI0, SGI0, SGI0, SGI111846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24540[] = {SGI111847, SGI111848, SGI111849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111850, SGI0, SGI0, SGI0, SGI111851, SGI111851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24541[] = {SGI111852, SGI111853, SGI111854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111855, SGI0, SGI0, SGI0, SGI111856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24542[] = {SGI111857, SGI0, SGI111858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24543[] = {SGI111859, SGI111860, SGI111861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111862, SGI0, SGI0, SGI0, SGI111863, SGI111863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24544[] = {SGI111864, SGI111865, SGI111866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111867, SGI0, SGI0, SGI0, SGI111868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24545[] = {SGI111869, SGI111870, SGI111871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111872, SGI0, SGI0, SGI0, SGI111874, SGI111874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24546[] = {SGI111873, SGI0, SGI111875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111876, SGI111877, SGI111877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24547[] = {SGI111878, SGI111879, SGI111880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111881, SGI0, SGI0, SGI0, SGI111882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24548[] = {SGI111883, SGI111884, SGI111885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111886, SGI0, SGI0, SGI0, SGI111887, SGI111887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24549[] = {SGI111888, SGI111889, SGI111890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111891, SGI0, SGI0, SGI0, SGI111892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24550[] = {SGI111893, SGI111894, SGI111895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111896, SGI0, SGI0, SGI0, SGI111897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24551[] = {SGI111898, SGI111899, SGI111900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111901, SGI0, SGI0, SGI0, SGI111902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24552[] = {SGI111903, SGI111904, SGI111905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111906, SGI0, SGI0, SGI0, SGI111907, SGI111907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24553[] = {SGI111908, SGI111909, SGI111910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111911, SGI0, SGI0, SGI0, SGI111912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24554[] = {SGI111913, SGI111914, SGI111915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111916, SGI0, SGI0, SGI0, SGI111917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24555[] = {SGI111918, SGI111919, SGI111920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111921, SGI0, SGI0, SGI0, SGI111922, SGI111922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24556[] = {SGI111923, SGI111924, SGI111925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111926, SGI0, SGI0, SGI0, SGI111927, SGI111927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24557[] = {SGI111928, SGI111929, SGI111930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111931, SGI0, SGI0, SGI0, SGI111932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24558[] = {SGI111933, SGI111934, SGI111935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111936, SGI0, SGI0, SGI0, SGI111937, SGI111937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24559[] = {SGI111938, SGI111939, SGI111940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111941, SGI0, SGI0, SGI0, SGI111942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24560[] = {SGI111943, SGI111944, SGI111945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111949, SGI0, SGI0, SGI0, SGI111950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24561[] = {SGI111946, SGI111947, SGI111948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111951, SGI0, SGI0, SGI0, SGI111952, SGI111952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24562[] = {SGI111953, SGI111954, SGI111955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111956, SGI0, SGI0, SGI0, SGI111957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24563[] = {SGI111958, SGI111959, SGI111960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111961, SGI0, SGI0, SGI0, SGI111962, SGI111962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24564[] = {SGI111963, SGI111964, SGI111965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111966, SGI0, SGI0, SGI0, SGI111967, SGI111967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24565[] = {SGI111968, SGI111969, SGI111970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111971, SGI0, SGI0, SGI0, SGI111972, SGI111972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24566[] = {SGI111973, SGI111974, SGI111975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111978, SGI0, SGI0, SGI0, SGI111980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24567[] = {SGI111976, SGI111977, SGI111979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111981, SGI0, SGI0, SGI0, SGI111982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24568[] = {SGI111983, SGI111984, SGI111985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111986, SGI0, SGI0, SGI0, SGI111987, SGI111987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24569[] = {SGI111988, SGI111989, SGI111990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111991, SGI0, SGI0, SGI0, SGI111993, SGI111993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111996, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24570[] = {SGI111992, SGI111994, SGI111995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI111997, SGI0, SGI0, SGI0, SGI111998, SGI111998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24571[] = {SGI111999, SGI112000, SGI112001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112002, SGI0, SGI0, SGI0, SGI112003, SGI112003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24572[] = {SGI112004, SGI112005, SGI112006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112007, SGI0, SGI0, SGI0, SGI112008, SGI112008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24573[] = {SGI112009, SGI112010, SGI112011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112012, SGI0, SGI0, SGI0, SGI112013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24574[] = {SGI112014, SGI112015, SGI112016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112017, SGI0, SGI0, SGI0, SGI112018, SGI112018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24575[] = {SGI112019, SGI112020, SGI112021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112024, SGI0, SGI0, SGI0, SGI112026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24576[] = {SGI112022, SGI112023, SGI112025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112027, SGI0, SGI0, SGI0, SGI112028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24577[] = {SGI112029, SGI112030, SGI112031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112032, SGI0, SGI0, SGI0, SGI112033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24578[] = {SGI112034, SGI112035, SGI112036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112037, SGI0, SGI0, SGI0, SGI112038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24579[] = {SGI112039, SGI112040, SGI112041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112042, SGI0, SGI0, SGI0, SGI112043, SGI112043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24580[] = {SGI112044, SGI112045, SGI112046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112047, SGI0, SGI0, SGI0, SGI112048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24581[] = {SGI112049, SGI112050, SGI112051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112052, SGI0, SGI0, SGI0, SGI112053, SGI112053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24582[] = {SGI112054, SGI112055, SGI112056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112057, SGI0, SGI0, SGI0, SGI112058, SGI112058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24583[] = {SGI112059, SGI112060, SGI112061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112062, SGI0, SGI0, SGI0, SGI112063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24584[] = {SGI112064, SGI112065, SGI112066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112067, SGI0, SGI0, SGI0, SGI112068, SGI112068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24585[] = {SGI112069, SGI0, SGI112070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112071, SGI0, SGI0, SGI0, SGI112072, SGI112072, SGI0, SGI0, SGI112073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24586[] = {SGI112074, SGI112075, SGI112076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112077, SGI0, SGI0, SGI0, SGI112078, SGI112078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24587[] = {SGI112079, SGI112080, SGI112081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112082, SGI0, SGI0, SGI0, SGI112083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24588[] = {SGI112084, SGI112085, SGI112086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112087, SGI0, SGI0, SGI0, SGI112088, SGI112088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24589[] = {SGI112089, SGI0, SGI112090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112094, SGI112095, SGI112095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24590[] = {SGI112091, SGI112092, SGI112093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112096, SGI0, SGI0, SGI0, SGI112097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24591[] = {SGI112098, SGI112099, SGI112100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112101, SGI0, SGI0, SGI0, SGI112102, SGI112102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24592[] = {SGI112103, SGI112104, SGI112105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112106, SGI0, SGI0, SGI0, SGI112107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24593[] = {SGI112108, SGI112109, SGI112110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112111, SGI0, SGI0, SGI0, SGI112112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24594[] = {SGI112113, SGI112114, SGI112115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112116, SGI0, SGI0, SGI0, SGI112117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24595[] = {SGI112118, SGI112119, SGI112121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112124, SGI0, SGI0, SGI0, SGI112125, SGI112125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24596[] = {SGI112120, SGI112122, SGI112123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112126, SGI0, SGI0, SGI0, SGI112127, SGI0, SGI0, SGI0, SGI0, SGI112128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24597[] = {SGI112129, SGI112130, SGI112131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112132, SGI0, SGI0, SGI0, SGI112133, SGI112133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24598[] = {SGI0, SGI112134, SGI0, SGI112135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24599[] = {SGI112136, SGI112137, SGI112138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112139, SGI0, SGI0, SGI0, SGI112140, SGI112140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24600[] = {SGI112141, SGI112142, SGI112143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112144, SGI0, SGI0, SGI0, SGI112145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112146, SGI0, SGI0, NULL};
const Segment * const SG24601[] = {SGI112147, SGI0, SGI112149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24602[] = {SGI112148, SGI112150, SGI112151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112152, SGI0, SGI0, SGI0, SGI112153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112154, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24603[] = {SGI112155, SGI112156, SGI112157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112158, SGI0, SGI0, SGI0, SGI112160, SGI112160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24604[] = {SGI112159, SGI0, SGI112161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112162, SGI0, SGI0, SGI0, SGI112163, SGI112163, SGI0, SGI112164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24605[] = {SGI112165, SGI112166, SGI112167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112168, SGI0, SGI0, SGI0, SGI112169, SGI112169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24606[] = {SGI112170, SGI112171, SGI112172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112173, SGI0, SGI0, SGI0, SGI112174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112175, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24607[] = {SGI112176, SGI112177, SGI112178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112179, SGI0, SGI0, SGI0, SGI112180, SGI112180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24608[] = {SGI112181, SGI112182, SGI112183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112184, SGI0, SGI0, SGI0, SGI112185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112186, SGI0, SGI0, NULL};
const Segment * const SG24609[] = {SGI112187, SGI0, SGI112189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24610[] = {SGI0, SGI112188, SGI112190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112191, SGI0, SGI0, SGI0, SGI112192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24611[] = {SGI112193, SGI112194, SGI112195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112196, SGI0, SGI0, SGI0, SGI112197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24612[] = {SGI112198, SGI112199, SGI112200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112201, SGI0, SGI0, SGI0, SGI112202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112203, SGI0, SGI0, NULL};
const Segment * const SG24613[] = {SGI112204, SGI112205, SGI112206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112207, SGI0, SGI0, SGI0, SGI112208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24614[] = {SGI112209, SGI112210, SGI112211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112212, SGI0, SGI0, SGI0, SGI112213, SGI112213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24615[] = {SGI112214, SGI112215, SGI112216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112217, SGI0, SGI0, SGI0, SGI112218, SGI112218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24616[] = {SGI112219, SGI0, SGI112220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112224, SGI0, SGI0, SGI0, SGI112225, SGI112225, SGI0, SGI112227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24617[] = {SGI112221, SGI112222, SGI112223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112226, SGI0, SGI0, SGI0, SGI112228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24618[] = {SGI112229, SGI112230, SGI112231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112232, SGI0, SGI0, SGI0, SGI112233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24619[] = {SGI112234, SGI112235, SGI112236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112237, SGI0, SGI0, SGI0, SGI112238, SGI112238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24620[] = {SGI112239, SGI112240, SGI112241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112242, SGI0, SGI0, SGI0, SGI112243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24621[] = {SGI112244, SGI112245, SGI112246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112247, SGI0, SGI0, SGI0, SGI112248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24622[] = {SGI112249, SGI112250, SGI112251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112252, SGI0, SGI0, SGI0, SGI112253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24623[] = {SGI112254, SGI112255, SGI112256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112257, SGI0, SGI0, SGI0, SGI112258, SGI112258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24624[] = {SGI112259, SGI112260, SGI112261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112262, SGI0, SGI0, SGI0, SGI112263, SGI112263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24625[] = {SGI112264, SGI112265, SGI112266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112267, SGI0, SGI0, SGI0, SGI112268, SGI112268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24626[] = {SGI112269, SGI0, SGI112270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112271, SGI0, SGI0, SGI0, SGI112272, SGI112272, SGI0, SGI112273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24627[] = {SGI112274, SGI112275, SGI112276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112277, SGI0, SGI0, SGI0, SGI112278, SGI112278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24628[] = {SGI112279, SGI112280, SGI112281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112282, SGI0, SGI0, SGI0, SGI112283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24629[] = {SGI112284, SGI112285, SGI112286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112287, SGI0, SGI0, SGI0, SGI112288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24630[] = {SGI112289, SGI112290, SGI112291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112292, SGI0, SGI0, SGI0, SGI112293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24631[] = {SGI112295, SGI112296, SGI112297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112298, SGI0, SGI0, SGI0, SGI112299, SGI112299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24632[] = {SGI112300, SGI0, SGI112301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112303, SGI112306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24633[] = {SGI112302, SGI112304, SGI112305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112307, SGI0, SGI0, SGI0, SGI112308, SGI112308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24634[] = {SGI112309, SGI112310, SGI112311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112312, SGI0, SGI0, SGI0, SGI112313, SGI112313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24635[] = {SGI112314, SGI112315, SGI112316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112317, SGI0, SGI0, SGI0, SGI112318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24636[] = {SGI112319, SGI112320, SGI112321, SGI112322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112323, SGI0, SGI0, SGI0, SGI112327, SGI112328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24637[] = {SGI112324, SGI112325, SGI112326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112329, SGI0, SGI0, SGI0, SGI112330, SGI112330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24638[] = {SGI112331, SGI112332, SGI112333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112334, SGI0, SGI0, SGI0, SGI112335, SGI112335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24639[] = {SGI112336, SGI112337, SGI112338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112339, SGI0, SGI0, SGI0, SGI112340, SGI112340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24640[] = {SGI112341, SGI112342, SGI112343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112344, SGI0, SGI0, SGI0, SGI112345, SGI112345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112346, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24641[] = {SGI112347, SGI112348, SGI112349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112350, SGI0, SGI0, SGI0, SGI112351, SGI112351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24642[] = {SGI112352, SGI0, SGI112353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112354, SGI112355, SGI112355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24643[] = {SGI112356, SGI0, SGI112357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112358, SGI112359, SGI112359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24644[] = {SGI112360, SGI112361, SGI112362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112363, SGI0, SGI0, SGI0, SGI112364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24645[] = {SGI112365, SGI112366, SGI112367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112368, SGI0, SGI0, SGI0, SGI112369, SGI0, SGI0, SGI0, SGI0, SGI112370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24646[] = {SGI112371, SGI112372, SGI112373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112375, SGI0, SGI0, SGI0, SGI112378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24647[] = {SGI112374, SGI112376, SGI112377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112379, SGI0, SGI0, SGI0, SGI112380, SGI0, SGI0, SGI0, SGI0, SGI112381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24648[] = {SGI112382, SGI112383, SGI112384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112385, SGI0, SGI0, SGI0, SGI112386, SGI112386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24649[] = {SGI112387, SGI112388, SGI112389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112390, SGI0, SGI0, SGI0, SGI112391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24650[] = {SGI112392, SGI112393, SGI112394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112395, SGI0, SGI0, SGI0, SGI112396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112397, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24651[] = {SGI112398, SGI112399, SGI112400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112401, SGI0, SGI0, SGI0, SGI112402, SGI112402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24652[] = {SGI112403, SGI112404, SGI112405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112406, SGI0, SGI0, SGI0, SGI112407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24653[] = {SGI112408, SGI112409, SGI112410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112411, SGI112411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24654[] = {SGI112412, SGI0, SGI112413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112414, SGI0, SGI0, SGI0, SGI112415, SGI112415, SGI0, SGI112416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24655[] = {SGI112417, SGI112418, SGI112419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112421, SGI0, SGI0, SGI0, SGI112424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24656[] = {SGI112420, SGI112422, SGI112423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112425, SGI0, SGI0, SGI0, SGI112426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24657[] = {SGI0, SGI112430, SGI0, SGI112432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24658[] = {SGI112428, SGI112429, SGI112431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112433, SGI0, SGI0, SGI0, SGI112434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24659[] = {SGI0, SGI112435, SGI0, SGI112437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112439, SGI0, SGI0, SGI0, SGI112441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24660[] = {SGI0, SGI112436, SGI112438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112440, SGI0, SGI0, SGI0, SGI112442, SGI112442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24661[] = {SGI112443, SGI112444, SGI112445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112446, SGI0, SGI0, SGI0, SGI112447, SGI112447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24662[] = {SGI112448, SGI0, SGI112449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112450, SGI0, SGI0, SGI0, SGI112451, SGI112451, SGI0, SGI112452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24663[] = {SGI112453, SGI112454, SGI112455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112456, SGI0, SGI0, SGI0, SGI112457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24664[] = {SGI112458, SGI112459, SGI112460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112461, SGI0, SGI0, SGI0, SGI112462, SGI112462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24665[] = {SGI112463, SGI0, SGI112464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112465, SGI112466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24666[] = {SGI112467, SGI112468, SGI112469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112470, SGI0, SGI0, SGI0, SGI112471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24667[] = {SGI112472, SGI112473, SGI112474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112475, SGI0, SGI0, SGI0, SGI112476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24668[] = {SGI112477, SGI112478, SGI112479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112480, SGI0, SGI0, SGI0, SGI112482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112485, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24669[] = {SGI112481, SGI112483, SGI112484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112486, SGI0, SGI0, SGI0, SGI112487, SGI112487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24670[] = {SGI112488, SGI112489, SGI112490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112491, SGI0, SGI0, SGI0, SGI112493, SGI112493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24671[] = {SGI112492, SGI112494, SGI112495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112496, SGI0, SGI0, SGI0, SGI112497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112498, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24672[] = {SGI112499, SGI112500, SGI112501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112502, SGI0, SGI0, SGI0, SGI112503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24673[] = {SGI112504, SGI112505, SGI112506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112507, SGI0, SGI0, SGI0, SGI112508, SGI112508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24674[] = {SGI112509, SGI112510, SGI112511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112512, SGI0, SGI0, SGI0, SGI112513, SGI112513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24675[] = {SGI112514, SGI0, SGI112515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112516, SGI112517, SGI112517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24676[] = {SGI112518, SGI112519, SGI112520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112521, SGI0, SGI0, SGI0, SGI112522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24677[] = {SGI112524, SGI112526, SGI112527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112529, SGI0, SGI0, SGI0, SGI112531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24678[] = {SGI112523, SGI112525, SGI112528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112530, SGI0, SGI0, SGI0, SGI112532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112533, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24679[] = {SGI112534, SGI112535, SGI112536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112537, SGI0, SGI0, SGI0, SGI112538, SGI0, SGI112540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24680[] = {SGI112539, SGI112541, SGI112542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112543, SGI0, SGI0, SGI0, SGI112545, SGI112545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24681[] = {SGI112544, SGI0, SGI112546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112547, SGI112548, SGI112548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112549, SGI0, NULL};
const Segment * const SG24682[] = {SGI112550, SGI112551, SGI112552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112553, SGI0, SGI0, SGI0, SGI112554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24683[] = {SGI112555, SGI112556, SGI112557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112558, SGI0, SGI0, SGI0, SGI112559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112560, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24684[] = {SGI112561, SGI0, SGI112562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112563, SGI112566, SGI112566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24685[] = {SGI112564, SGI112565, SGI112567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112568, SGI0, SGI0, SGI0, SGI112569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24686[] = {SGI112570, SGI112571, SGI112572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112576, SGI0, SGI0, SGI0, SGI112577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24687[] = {SGI112573, SGI112574, SGI112575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112578, SGI0, SGI0, SGI0, SGI112579, SGI0, SGI0, SGI112580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24688[] = {SGI112581, SGI112582, SGI112583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112588, SGI0, SGI0, SGI0, SGI112591, SGI112591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24689[] = {SGI112584, SGI112586, SGI112589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112592, SGI0, SGI0, SGI0, SGI112594, SGI112594, SGI0, SGI112596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24690[] = {SGI112585, SGI112587, SGI112590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112593, SGI0, SGI0, SGI0, SGI112595, SGI112595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112597, SGI0, SGI0, NULL};
const Segment * const SG24691[] = {SGI112598, SGI112599, SGI112600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112601, SGI0, SGI0, SGI0, SGI112602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24692[] = {SGI112603, SGI112604, SGI112605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112606, SGI0, SGI0, SGI0, SGI112607, SGI112607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112609, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24693[] = {SGI112608, SGI112610, SGI112611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112612, SGI0, SGI0, SGI0, SGI112613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24694[] = {SGI112614, SGI112615, SGI112616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112617, SGI0, SGI0, SGI0, SGI112618, SGI112618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112619, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24695[] = {SGI112620, SGI112621, SGI112622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112623, SGI0, SGI0, SGI0, SGI112624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24696[] = {SGI112625, SGI112626, SGI112627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112628, SGI0, SGI0, SGI0, SGI112629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24697[] = {SGI112630, SGI112631, SGI112632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112633, SGI0, SGI0, SGI0, SGI112634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24698[] = {SGI112635, SGI112636, SGI112637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112638, SGI0, SGI0, SGI0, SGI112639, SGI112639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24699[] = {SGI112640, SGI112641, SGI112642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112646, SGI0, SGI0, SGI0, SGI112647, SGI112647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24700[] = {SGI112643, SGI112644, SGI112645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112648, SGI0, SGI0, SGI0, SGI112649, SGI0, SGI0, SGI0, SGI0, SGI112650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24701[] = {SGI112651, SGI112652, SGI112653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112654, SGI0, SGI0, SGI0, SGI112655, SGI112655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24702[] = {SGI112656, SGI112657, SGI112659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112662, SGI0, SGI0, SGI0, SGI112664, SGI0, SGI112666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24703[] = {SGI112658, SGI112660, SGI112661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112663, SGI0, SGI0, SGI0, SGI112665, SGI112665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24704[] = {SGI112667, SGI112668, SGI112669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112670, SGI0, SGI0, SGI0, SGI112671, SGI112671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24705[] = {SGI112675, SGI112678, SGI112680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112682, SGI0, SGI0, SGI0, SGI112683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24706[] = {SGI112672, SGI112674, SGI112677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112681, SGI0, SGI0, SGI0, SGI112685, SGI0, SGI0, SGI0, SGI112686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24707[] = {SGI112673, SGI112676, SGI112679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112684, SGI112684, SGI0, SGI0, SGI112687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24708[] = {SGI112688, SGI112689, SGI112690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112694, SGI0, SGI0, SGI0, SGI112695, SGI112695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24709[] = {SGI112691, SGI112692, SGI112693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112696, SGI0, SGI0, SGI0, SGI112697, SGI0, SGI112698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24710[] = {SGI112699, SGI112700, SGI112701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112702, SGI0, SGI0, SGI0, SGI112703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24711[] = {SGI112704, SGI112705, SGI112706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112707, SGI0, SGI0, SGI0, SGI112708, SGI112708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24712[] = {SGI112709, SGI112710, SGI112711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112712, SGI112712, SGI0, SGI112714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24713[] = {SGI112713, SGI112715, SGI112716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112717, SGI0, SGI0, SGI0, SGI112718, SGI112718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24714[] = {SGI112719, SGI112721, SGI112722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112725, SGI0, SGI0, SGI0, SGI112726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24715[] = {SGI112720, SGI112723, SGI112724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112727, SGI112727, SGI0, SGI112728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24716[] = {SGI112729, SGI112730, SGI112731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112735, SGI112735, SGI0, SGI0, SGI112736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24717[] = {SGI112732, SGI112733, SGI112734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112737, SGI0, SGI0, SGI0, SGI0, SGI112738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24718[] = {SGI112739, SGI0, SGI112740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112741, SGI0, SGI0, SGI0, SGI112742, SGI112742, SGI0, SGI112743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24719[] = {SGI112744, SGI112745, SGI112746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112747, SGI0, SGI0, SGI0, SGI112748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24720[] = {SGI112749, SGI112750, SGI112751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112755, SGI0, SGI0, SGI0, SGI112756, SGI112756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24721[] = {SGI112752, SGI112753, SGI112754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112757, SGI112757, SGI0, SGI112758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24722[] = {SGI112759, SGI112760, SGI112761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112762, SGI0, SGI0, SGI0, SGI112763, SGI112763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24723[] = {SGI112764, SGI112765, SGI112766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112767, SGI112767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24724[] = {SGI112768, SGI112769, SGI112770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112771, SGI0, SGI0, SGI0, SGI112772, SGI112772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24725[] = {SGI112773, SGI112774, SGI112775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112776, SGI0, SGI0, SGI0, SGI112777, SGI0, SGI0, SGI0, SGI0, SGI112778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24726[] = {SGI112779, SGI112780, SGI112781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112782, SGI0, SGI0, SGI0, SGI112783, SGI112783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24727[] = {SGI112784, SGI112785, SGI112786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112787, SGI112787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24728[] = {SGI112788, SGI112789, SGI112790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112791, SGI0, SGI0, SGI0, SGI112792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24729[] = {SGI112793, SGI112794, SGI112795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112796, SGI0, SGI0, SGI0, SGI112797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24730[] = {SGI112798, SGI112799, SGI112800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112801, SGI0, SGI0, SGI0, SGI112802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24731[] = {SGI112803, SGI112804, SGI112805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112809, SGI0, SGI0, SGI0, SGI112810, SGI112810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112813, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24732[] = {SGI112806, SGI112807, SGI112808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112811, SGI0, SGI0, SGI0, SGI112812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24733[] = {SGI112814, SGI0, SGI112815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24734[] = {SGI112816, SGI112817, SGI112818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112819, SGI0, SGI0, SGI0, SGI112820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24735[] = {SGI112821, SGI112822, SGI112823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112824, SGI0, SGI0, SGI0, SGI112825, SGI112825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24736[] = {SGI112826, SGI112827, SGI112828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112829, SGI0, SGI0, SGI0, SGI112830, SGI112830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24737[] = {SGI112831, SGI112832, SGI112833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112836, SGI0, SGI0, SGI0, SGI112837, SGI112837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24738[] = {SGI112834, SGI0, SGI112835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112838, SGI112839, SGI112839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24739[] = {SGI112840, SGI112841, SGI112842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112843, SGI0, SGI0, SGI0, SGI112844, SGI112844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24740[] = {SGI112845, SGI0, SGI112846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112847, SGI112848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24741[] = {SGI112849, SGI112850, SGI112851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112852, SGI0, SGI0, SGI0, SGI112853, SGI112853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24742[] = {SGI112854, SGI112855, SGI112856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112860, SGI0, SGI0, SGI0, SGI112861, SGI112861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24743[] = {SGI112857, SGI112858, SGI112859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112862, SGI0, SGI0, SGI0, SGI112863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24744[] = {SGI112864, SGI112865, SGI112866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112867, SGI0, SGI0, SGI0, SGI112869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24745[] = {SGI112868, SGI112870, SGI112871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112872, SGI0, SGI0, SGI0, SGI112873, SGI112873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24746[] = {SGI112874, SGI112875, SGI112876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112877, SGI0, SGI0, SGI0, SGI112878, SGI112878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24747[] = {SGI112879, SGI112880, SGI112881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112882, SGI0, SGI0, SGI0, SGI112883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24748[] = {SGI112884, SGI112885, SGI112886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112887, SGI0, SGI0, SGI0, SGI112888, SGI112888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24749[] = {SGI112889, SGI112890, SGI112891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112892, SGI0, SGI0, SGI0, SGI112893, SGI112893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24750[] = {SGI112894, SGI112895, SGI112896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112900, SGI0, SGI0, SGI0, SGI112901, SGI112901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24751[] = {SGI112897, SGI112898, SGI112899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112902, SGI0, SGI0, SGI0, SGI112903, SGI112903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24752[] = {SGI112905, SGI112907, SGI112908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112910, SGI0, SGI0, SGI0, SGI112912, SGI112912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24753[] = {SGI112904, SGI112906, SGI112909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112911, SGI0, SGI0, SGI0, SGI112913, SGI112913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24754[] = {SGI112914, SGI112915, SGI112916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112917, SGI0, SGI0, SGI0, SGI112918, SGI112918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24755[] = {SGI112919, SGI112920, SGI112921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112922, SGI0, SGI0, SGI0, SGI112923, SGI112923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24756[] = {SGI112924, SGI112925, SGI112926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112927, SGI0, SGI0, SGI0, SGI112928, SGI112928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24757[] = {SGI0, SGI112929, SGI112930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112931, SGI0, SGI0, SGI0, SGI112932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24758[] = {SGI112933, SGI112934, SGI112935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112936, SGI0, SGI0, SGI0, SGI112937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24759[] = {SGI112938, SGI112939, SGI112940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112941, SGI0, SGI0, SGI0, SGI112942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24760[] = {SGI112943, SGI112944, SGI112945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112947, SGI0, SGI0, SGI0, SGI112949, SGI112949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24761[] = {SGI112946, SGI0, SGI112948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112950, SGI112951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24762[] = {SGI112952, SGI112953, SGI112954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112955, SGI0, SGI0, SGI0, SGI112956, SGI112956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24763[] = {SGI112957, SGI112958, SGI112959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112960, SGI0, SGI0, SGI0, SGI112961, SGI112961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24764[] = {SGI0, SGI112962, SGI112963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24765[] = {SGI112964, SGI112965, SGI112966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112967, SGI0, SGI0, SGI0, SGI112968, SGI112968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24766[] = {SGI112969, SGI112970, SGI112971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112972, SGI0, SGI0, SGI0, SGI112973, SGI112973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24767[] = {SGI112974, SGI112975, SGI112976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112977, SGI0, SGI0, SGI0, SGI112978, SGI112978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24768[] = {SGI112979, SGI112980, SGI112981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112983, SGI0, SGI0, SGI0, SGI112985, SGI112985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24769[] = {SGI112982, SGI0, SGI112984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112986, SGI0, SGI0, SGI0, SGI112987, SGI112987, SGI0, SGI0, SGI112988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24770[] = {SGI112989, SGI112990, SGI112991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112992, SGI0, SGI0, SGI0, SGI112993, SGI112993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24771[] = {SGI112994, SGI0, SGI112995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI112996, SGI0, SGI0, SGI0, SGI112997, SGI112997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24772[] = {SGI112998, SGI112999, SGI113000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113001, SGI0, SGI0, SGI0, SGI113002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24773[] = {SGI113003, SGI113004, SGI113005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113006, SGI0, SGI0, SGI0, SGI113007, SGI113007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24774[] = {SGI113008, SGI113009, SGI113010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113011, SGI0, SGI0, SGI0, SGI113012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24775[] = {SGI113013, SGI113014, SGI113015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113016, SGI0, SGI0, SGI0, SGI113017, SGI113017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24776[] = {SGI113018, SGI113019, SGI113020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113021, SGI113021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24777[] = {SGI113022, SGI113023, SGI113024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113025, SGI0, SGI0, SGI0, SGI113026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24778[] = {SGI113027, SGI0, SGI113028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113029, SGI113030, SGI113030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24779[] = {SGI113032, SGI0, SGI113035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113037, SGI113039, SGI113039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24780[] = {SGI113031, SGI113033, SGI113034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113036, SGI0, SGI0, SGI0, SGI113038, SGI113038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24781[] = {SGI113040, SGI113041, SGI113042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113043, SGI0, SGI0, SGI0, SGI113044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24782[] = {SGI113045, SGI113046, SGI113047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113048, SGI0, SGI0, SGI0, SGI113049, SGI113049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24783[] = {SGI113050, SGI113051, SGI113052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113053, SGI0, SGI0, SGI0, SGI113054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24784[] = {SGI113055, SGI113056, SGI113057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113058, SGI0, SGI0, SGI0, SGI113059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24785[] = {SGI113060, SGI113061, SGI113062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113063, SGI0, SGI0, SGI0, SGI113064, SGI113064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24786[] = {SGI113065, SGI113066, SGI113067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113068, SGI0, SGI0, SGI0, SGI113069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24787[] = {SGI113070, SGI113071, SGI113072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113073, SGI0, SGI0, SGI0, SGI113074, SGI113074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24788[] = {SGI113075, SGI113076, SGI113077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113078, SGI0, SGI0, SGI0, SGI113079, SGI113079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24789[] = {SGI113080, SGI113081, SGI113082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113084, SGI0, SGI0, SGI0, SGI113087, SGI113087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24790[] = {SGI113083, SGI113085, SGI113086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113088, SGI113088, SGI0, SGI0, SGI113089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24791[] = {SGI113090, SGI113091, SGI113092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113093, SGI0, SGI0, SGI0, SGI113094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24792[] = {SGI113095, SGI113096, SGI113097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113098, SGI0, SGI0, SGI0, SGI113099, SGI113099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24793[] = {SGI113100, SGI113101, SGI113102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113103, SGI0, SGI0, SGI0, SGI113104, SGI113104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24794[] = {SGI113105, SGI113106, SGI113107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113108, SGI0, SGI0, SGI0, SGI113109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24795[] = {SGI113110, SGI113111, SGI113112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113113, SGI0, SGI0, SGI0, SGI113114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24796[] = {SGI113115, SGI113116, SGI113117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113118, SGI0, SGI0, SGI0, SGI113119, SGI113119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24797[] = {SGI113120, SGI113121, SGI113122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113123, SGI0, SGI0, SGI0, SGI113124, SGI113124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24798[] = {SGI113125, SGI113126, SGI113127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113128, SGI0, SGI0, SGI0, SGI113129, SGI113129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24799[] = {SGI113130, SGI113131, SGI113132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113133, SGI0, SGI0, SGI0, SGI113134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24800[] = {SGI113135, SGI113136, SGI113137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113138, SGI0, SGI0, SGI0, SGI113139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24801[] = {SGI113140, SGI113141, SGI113142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113143, SGI0, SGI0, SGI0, SGI113144, SGI113144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24802[] = {SGI113145, SGI113146, SGI113147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113148, SGI0, SGI0, SGI0, SGI113149, SGI113149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24803[] = {SGI113150, SGI113151, SGI113152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113153, SGI0, SGI0, SGI0, SGI113154, SGI113154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24804[] = {SGI113155, SGI113156, SGI113157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113158, SGI0, SGI0, SGI0, SGI113159, SGI113159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24805[] = {SGI113160, SGI113161, SGI113162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113163, SGI0, SGI0, SGI0, SGI113164, SGI113164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24806[] = {SGI113165, SGI113166, SGI113167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113168, SGI0, SGI0, SGI0, SGI113169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24807[] = {SGI113170, SGI113171, SGI113172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113173, SGI0, SGI0, SGI0, SGI113174, SGI113174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24808[] = {SGI113175, SGI113176, SGI113177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113178, SGI0, SGI0, SGI0, SGI113179, SGI113179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24809[] = {SGI113180, SGI113181, SGI113182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113183, SGI0, SGI0, SGI0, SGI113184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24810[] = {SGI113185, SGI113186, SGI113187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113188, SGI0, SGI0, SGI0, SGI113189, SGI113189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24811[] = {SGI113190, SGI113191, SGI113192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113193, SGI0, SGI0, SGI0, SGI113194, SGI113194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24812[] = {SGI113195, SGI113196, SGI113197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113198, SGI0, SGI0, SGI0, SGI113199, SGI113199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24813[] = {SGI113200, SGI113201, SGI113202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113203, SGI0, SGI0, SGI0, SGI113204, SGI113204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24814[] = {SGI113205, SGI113206, SGI113207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113208, SGI0, SGI0, SGI0, SGI113209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24815[] = {SGI113210, SGI113211, SGI113212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113213, SGI0, SGI0, SGI0, SGI113214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24816[] = {SGI113215, SGI113216, SGI113217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113218, SGI0, SGI0, SGI0, SGI113219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24817[] = {SGI113220, SGI113221, SGI113222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113223, SGI0, SGI0, SGI0, SGI113224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24818[] = {SGI113225, SGI113226, SGI113227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113228, SGI0, SGI0, SGI0, SGI113229, SGI113229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24819[] = {SGI113230, SGI113231, SGI113232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113233, SGI0, SGI0, SGI0, SGI113234, SGI113234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24820[] = {SGI113235, SGI0, SGI113236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113237, SGI113238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24821[] = {SGI113239, SGI113240, SGI113241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113242, SGI0, SGI0, SGI0, SGI113243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24822[] = {SGI113244, SGI113245, SGI113246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113247, SGI0, SGI0, SGI0, SGI113248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24823[] = {SGI113249, SGI113250, SGI113251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113252, SGI0, SGI0, SGI0, SGI113253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24824[] = {SGI113254, SGI113255, SGI113256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113257, SGI0, SGI0, SGI0, SGI113258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24825[] = {SGI113259, SGI113260, SGI113261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113262, SGI0, SGI0, SGI0, SGI113263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24826[] = {SGI113264, SGI113265, SGI113266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113267, SGI0, SGI0, SGI0, SGI113268, SGI113268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24827[] = {SGI113269, SGI113270, SGI113271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113272, SGI0, SGI0, SGI0, SGI113273, SGI113273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24828[] = {SGI113274, SGI113275, SGI113276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113277, SGI113277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24829[] = {SGI113278, SGI113279, SGI113280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113281, SGI0, SGI0, SGI0, SGI113282, SGI113282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24830[] = {SGI113283, SGI113284, SGI113285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113286, SGI0, SGI0, SGI0, SGI113287, SGI113287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24831[] = {SGI113288, SGI0, SGI113289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24832[] = {SGI113290, SGI113291, SGI113292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113293, SGI0, SGI0, SGI0, SGI113294, SGI113294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24833[] = {SGI113295, SGI113296, SGI113297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113298, SGI0, SGI0, SGI0, SGI113299, SGI113299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24834[] = {SGI113300, SGI113301, SGI113302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113303, SGI0, SGI0, SGI0, SGI113304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24835[] = {SGI113305, SGI113306, SGI113307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113308, SGI0, SGI0, SGI0, SGI113309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24836[] = {SGI113310, SGI113311, SGI113312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113313, SGI0, SGI0, SGI0, SGI113314, SGI113314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24837[] = {SGI113315, SGI113316, SGI113317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113318, SGI0, SGI0, SGI0, SGI113319, SGI113319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24838[] = {SGI113320, SGI113321, SGI113322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113323, SGI0, SGI0, SGI0, SGI113324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24839[] = {SGI113325, SGI0, SGI113326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24840[] = {SGI113327, SGI113328, SGI113329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113330, SGI0, SGI0, SGI0, SGI113331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24841[] = {SGI113332, SGI113333, SGI113334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113335, SGI0, SGI0, SGI0, SGI113336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24842[] = {SGI113337, SGI113338, SGI113339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113340, SGI0, SGI0, SGI0, SGI113341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24843[] = {SGI113342, SGI113343, SGI113344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113345, SGI0, SGI0, SGI0, SGI113346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24844[] = {SGI113347, SGI113348, SGI113349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113350, SGI0, SGI0, SGI0, SGI113351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24845[] = {SGI113352, SGI113353, SGI113354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113355, SGI0, SGI0, SGI0, SGI113356, SGI113356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24846[] = {SGI113357, SGI113358, SGI113359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113360, SGI0, SGI0, SGI0, SGI113361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24847[] = {SGI0, SGI113362, SGI113363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113364, SGI0, SGI0, SGI0, SGI113365, SGI113365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24848[] = {SGI113366, SGI113367, SGI113368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113369, SGI0, SGI0, SGI0, SGI113370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24849[] = {SGI113371, SGI113372, SGI113373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113374, SGI0, SGI0, SGI0, SGI113375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24850[] = {SGI113376, SGI113377, SGI113378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113379, SGI113379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24851[] = {SGI113380, SGI113381, SGI113382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113383, SGI0, SGI0, SGI0, SGI113384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24852[] = {SGI113385, SGI113386, SGI113387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113388, SGI0, SGI0, SGI0, SGI113389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24853[] = {SGI113390, SGI113391, SGI113392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113393, SGI0, SGI0, SGI0, SGI113394, SGI113394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24854[] = {SGI113395, SGI113396, SGI113397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113398, SGI0, SGI0, SGI0, SGI113399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24855[] = {SGI113400, SGI113401, SGI113402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113403, SGI0, SGI0, SGI0, SGI113404, SGI113404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24856[] = {SGI113405, SGI113406, SGI113407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113408, SGI0, SGI0, SGI0, SGI113409, SGI113409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24857[] = {SGI113410, SGI113411, SGI113412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113413, SGI0, SGI0, SGI0, SGI113414, SGI113414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24858[] = {SGI113415, SGI113416, SGI113417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113418, SGI0, SGI0, SGI0, SGI113419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24859[] = {SGI113420, SGI113421, SGI113422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113423, SGI0, SGI0, SGI0, SGI113424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24860[] = {SGI113425, SGI113426, SGI113427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113428, SGI0, SGI0, SGI0, SGI113429, SGI113429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24861[] = {SGI113430, SGI113431, SGI113432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113433, SGI0, SGI0, SGI0, SGI113434, SGI113434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24862[] = {SGI113435, SGI113436, SGI113437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113438, SGI0, SGI0, SGI0, SGI113439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24863[] = {SGI113440, SGI113441, SGI113442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113443, SGI0, SGI0, SGI0, SGI113444, SGI113444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24864[] = {SGI113445, SGI113446, SGI113447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113448, SGI0, SGI0, SGI0, SGI113449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24865[] = {SGI113450, SGI113451, SGI113452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113453, SGI0, SGI0, SGI0, SGI113454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24866[] = {SGI113455, SGI0, SGI113456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24867[] = {SGI113457, SGI113458, SGI113459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113460, SGI0, SGI0, SGI0, SGI113461, SGI113461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24868[] = {SGI113462, SGI113463, SGI113464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113465, SGI0, SGI0, SGI0, SGI113466, SGI113466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24869[] = {SGI113467, SGI113468, SGI113469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113470, SGI0, SGI0, SGI0, SGI113471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24870[] = {SGI113472, SGI113473, SGI113474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113475, SGI0, SGI0, SGI0, SGI113476, SGI113476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24871[] = {SGI113477, SGI113478, SGI113479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113480, SGI0, SGI0, SGI0, SGI113481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24872[] = {SGI113482, SGI113483, SGI113484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113485, SGI0, SGI0, SGI0, SGI113486, SGI113486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24873[] = {SGI113487, SGI0, SGI113488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24874[] = {SGI113489, SGI113490, SGI113491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113492, SGI0, SGI0, SGI0, SGI113493, SGI113493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24875[] = {SGI113494, SGI113495, SGI113496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113497, SGI0, SGI0, SGI0, SGI113498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24876[] = {SGI113499, SGI0, SGI113500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113502, SGI113504, SGI113504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113506, SGI0, NULL};
const Segment * const SG24877[] = {SGI113501, SGI113503, SGI113505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113507, SGI0, SGI0, SGI0, SGI113508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24878[] = {SGI113509, SGI113510, SGI113511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113512, SGI0, SGI0, SGI0, SGI113513, SGI113513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24879[] = {SGI113514, SGI113515, SGI113516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113517, SGI0, SGI0, SGI0, SGI113518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24880[] = {SGI113519, SGI113520, SGI113521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113522, SGI0, SGI0, SGI0, SGI113523, SGI113523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24881[] = {SGI113524, SGI113525, SGI113526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113527, SGI0, SGI0, SGI0, SGI113528, SGI113528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24882[] = {SGI113529, SGI113530, SGI113531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113532, SGI0, SGI0, SGI0, SGI113533, SGI113533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24883[] = {SGI113534, SGI113535, SGI113536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113537, SGI0, SGI0, SGI0, SGI113538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24884[] = {SGI113539, SGI113540, SGI113541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113542, SGI0, SGI0, SGI0, SGI113543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24885[] = {SGI113544, SGI113545, SGI113546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113547, SGI0, SGI0, SGI0, SGI113548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24886[] = {SGI113549, SGI113550, SGI113551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113552, SGI113552, SGI0, SGI0, SGI113553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24887[] = {SGI113554, SGI0, SGI113555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24888[] = {SGI113556, SGI113557, SGI113558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113559, SGI0, SGI0, SGI0, SGI113560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24889[] = {SGI113561, SGI113562, SGI113563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113564, SGI0, SGI0, SGI0, SGI113565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24890[] = {SGI113566, SGI0, SGI113567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24891[] = {SGI113568, SGI113569, SGI113570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113571, SGI0, SGI0, SGI0, SGI113572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24892[] = {SGI113573, SGI113574, SGI113575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113576, SGI0, SGI0, SGI0, SGI113577, SGI113577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24893[] = {SGI113578, SGI113579, SGI113580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113581, SGI0, SGI0, SGI0, SGI113582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24894[] = {SGI113583, SGI113584, SGI113585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113586, SGI0, SGI0, SGI0, SGI113587, SGI113587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24895[] = {SGI113588, SGI113589, SGI113590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113591, SGI0, SGI0, SGI0, SGI113592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24896[] = {SGI113593, SGI113594, SGI113595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113596, SGI0, SGI0, SGI0, SGI113597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24897[] = {SGI113598, SGI113599, SGI113600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113601, SGI0, SGI0, SGI0, SGI113602, SGI113602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24898[] = {SGI113603, SGI113604, SGI113605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113606, SGI0, SGI0, SGI0, SGI113607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24899[] = {SGI113608, SGI113609, SGI113610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113611, SGI0, SGI0, SGI0, SGI113612, SGI113612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24900[] = {SGI113613, SGI113614, SGI113615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113616, SGI0, SGI0, SGI0, SGI113617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24901[] = {SGI113618, SGI113619, SGI113620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113621, SGI0, SGI0, SGI0, SGI113622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24902[] = {SGI113623, SGI113624, SGI113625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113626, SGI0, SGI0, SGI0, SGI113627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24903[] = {SGI113628, SGI113629, SGI113630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113631, SGI0, SGI0, SGI0, SGI113632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24904[] = {SGI113633, SGI113634, SGI113635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113636, SGI0, SGI0, SGI0, SGI113637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24905[] = {SGI113638, SGI113639, SGI113640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113641, SGI0, SGI0, SGI0, SGI113642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24906[] = {SGI113643, SGI113644, SGI113645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113646, SGI0, SGI0, SGI0, SGI113647, SGI113647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24907[] = {SGI113648, SGI113649, SGI113650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113651, SGI0, SGI0, SGI0, SGI113652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24908[] = {SGI113653, SGI113654, SGI113655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113656, SGI0, SGI0, SGI0, SGI113657, SGI113657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24909[] = {SGI113658, SGI113659, SGI113660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113661, SGI0, SGI0, SGI0, SGI113662, SGI113662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24910[] = {SGI113663, SGI113664, SGI113665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113666, SGI0, SGI0, SGI0, SGI113667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24911[] = {SGI113668, SGI113669, SGI113670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113671, SGI0, SGI0, SGI0, SGI113672, SGI113672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24912[] = {SGI113673, SGI113674, SGI113675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113676, SGI0, SGI0, SGI0, SGI113677, SGI113677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24913[] = {SGI113678, SGI113679, SGI113680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113681, SGI0, SGI0, SGI0, SGI113682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24914[] = {SGI113683, SGI113684, SGI113685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113686, SGI0, SGI0, SGI0, SGI113687, SGI113687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24915[] = {SGI113688, SGI113689, SGI113690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113691, SGI0, SGI0, SGI0, SGI113692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24916[] = {SGI113693, SGI113694, SGI113695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113696, SGI0, SGI0, SGI0, SGI113697, SGI113697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24917[] = {SGI113698, SGI113699, SGI113700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113701, SGI0, SGI0, SGI0, SGI113702, SGI113702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24918[] = {SGI113703, SGI113704, SGI113706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113709, SGI0, SGI0, SGI0, SGI113711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24919[] = {SGI113705, SGI113707, SGI113708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113710, SGI0, SGI0, SGI0, SGI113712, SGI113712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24920[] = {SGI113713, SGI113715, SGI113717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113719, SGI0, SGI0, SGI0, SGI113721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24921[] = {SGI113714, SGI113716, SGI113718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113720, SGI0, SGI0, SGI0, SGI113722, SGI113722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24922[] = {SGI113723, SGI113724, SGI113725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113726, SGI0, SGI0, SGI0, SGI113727, SGI113727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24923[] = {SGI113728, SGI113729, SGI113730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113731, SGI0, SGI0, SGI0, SGI113732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24924[] = {SGI113733, SGI113734, SGI113735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113736, SGI0, SGI0, SGI0, SGI113737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24925[] = {SGI113738, SGI113739, SGI113740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113741, SGI0, SGI0, SGI0, SGI113742, SGI113742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24926[] = {SGI113743, SGI113744, SGI113745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113746, SGI0, SGI0, SGI0, SGI113747, SGI113747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24927[] = {SGI113748, SGI113749, SGI113750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113751, SGI0, SGI0, SGI0, SGI113752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24928[] = {SGI113753, SGI113754, SGI113755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113756, SGI0, SGI0, SGI0, SGI113757, SGI113757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24929[] = {SGI113758, SGI113759, SGI113760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113761, SGI0, SGI0, SGI0, SGI113762, SGI113762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24930[] = {SGI113763, SGI0, SGI113764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113765, SGI113766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24931[] = {SGI0, SGI113767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113768, SGI0, SGI0, SGI0, SGI0, SGI113769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24932[] = {SGI113770, SGI113771, SGI113772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113773, SGI0, SGI0, SGI0, SGI113774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24933[] = {SGI113775, SGI113776, SGI113777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113778, SGI0, SGI0, SGI0, SGI113779, SGI113779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24934[] = {SGI113780, SGI0, SGI113781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113782, SGI113784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24935[] = {SGI113783, SGI113785, SGI113786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113787, SGI0, SGI0, SGI0, SGI113788, SGI113788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24936[] = {SGI113789, SGI113790, SGI113791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113792, SGI0, SGI0, SGI0, SGI113793, SGI113793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24937[] = {SGI113794, SGI113795, SGI113796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113797, SGI0, SGI0, SGI0, SGI113798, SGI113798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24938[] = {SGI113799, SGI113800, SGI113801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113802, SGI0, SGI0, SGI0, SGI113803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24939[] = {SGI113804, SGI113805, SGI113806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113807, SGI0, SGI0, SGI0, SGI113808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24940[] = {SGI113809, SGI113810, SGI113811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113812, SGI0, SGI0, SGI0, SGI113813, SGI113813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24941[] = {SGI113814, SGI113815, SGI113816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113817, SGI0, SGI0, SGI0, SGI113818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24942[] = {SGI113819, SGI113820, SGI113821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113822, SGI0, SGI0, SGI0, SGI113823, SGI113823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24943[] = {SGI113824, SGI113825, SGI113826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113827, SGI0, SGI0, SGI0, SGI113830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24944[] = {SGI113828, SGI113829, SGI113831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113832, SGI0, SGI0, SGI0, SGI113833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24945[] = {SGI113834, SGI113835, SGI113836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113837, SGI0, SGI0, SGI0, SGI113838, SGI113838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24946[] = {SGI113839, SGI113840, SGI113841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113842, SGI0, SGI0, SGI0, SGI113843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24947[] = {SGI113844, SGI113845, SGI113846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113847, SGI0, SGI0, SGI0, SGI113848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24948[] = {SGI113849, SGI0, SGI113850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113851, SGI0, SGI0, SGI0, SGI113852, SGI113852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24949[] = {SGI113853, SGI113854, SGI113855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113856, SGI0, SGI0, SGI0, SGI113857, SGI113857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24950[] = {SGI113858, SGI113859, SGI113860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113861, SGI0, SGI0, SGI0, SGI113862, SGI113862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24951[] = {SGI113863, SGI113864, SGI113865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113866, SGI0, SGI0, SGI0, SGI113867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24952[] = {SGI113868, SGI113869, SGI113870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113871, SGI0, SGI0, SGI0, SGI113872, SGI113872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24953[] = {SGI113873, SGI0, SGI113876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113879, SGI113880, SGI113880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24954[] = {SGI113874, SGI113875, SGI113877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113878, SGI0, SGI0, SGI0, SGI113881, SGI113881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24955[] = {SGI113882, SGI0, SGI113883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113884, SGI113885, SGI113885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24956[] = {SGI113886, SGI113887, SGI113888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113889, SGI0, SGI0, SGI0, SGI113890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24957[] = {SGI113891, SGI0, SGI113892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113893, SGI113894, SGI113894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24958[] = {SGI113895, SGI113896, SGI113897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113898, SGI0, SGI0, SGI0, SGI113899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24959[] = {SGI113900, SGI113901, SGI113902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113903, SGI0, SGI0, SGI0, SGI113904, SGI113904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24960[] = {SGI113905, SGI113906, SGI113907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113908, SGI0, SGI0, SGI0, SGI113909, SGI113909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24961[] = {SGI113910, SGI113911, SGI113912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113913, SGI0, SGI0, SGI0, SGI113914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24962[] = {SGI113915, SGI113916, SGI113917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113918, SGI0, SGI0, SGI0, SGI113919, SGI113919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24963[] = {SGI113920, SGI113921, SGI113922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113923, SGI0, SGI0, SGI0, SGI113924, SGI113924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24964[] = {SGI113925, SGI113926, SGI113927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113930, SGI0, SGI0, SGI0, SGI113931, SGI113931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24965[] = {SGI113928, SGI0, SGI113929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113932, SGI113933, SGI113933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24966[] = {SGI113934, SGI113935, SGI113936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113937, SGI0, SGI0, SGI0, SGI113938, SGI113938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24967[] = {SGI0, SGI113939, SGI113940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113941, SGI0, SGI0, SGI0, SGI113942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24968[] = {SGI113943, SGI113944, SGI113945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113946, SGI0, SGI0, SGI0, SGI113947, SGI113947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24969[] = {SGI113948, SGI113949, SGI113950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113951, SGI0, SGI0, SGI0, SGI113952, SGI113952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24970[] = {SGI113953, SGI113954, SGI113955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113956, SGI0, SGI0, SGI0, SGI113957, SGI113957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24971[] = {SGI113958, SGI113959, SGI113960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113961, SGI0, SGI0, SGI0, SGI113962, SGI113962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24972[] = {SGI113963, SGI113964, SGI113965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113966, SGI0, SGI0, SGI0, SGI113967, SGI113967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24973[] = {SGI113968, SGI113969, SGI113970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113971, SGI0, SGI0, SGI0, SGI113972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24974[] = {SGI113973, SGI0, SGI113974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24975[] = {SGI113975, SGI113976, SGI113977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113978, SGI113978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24976[] = {SGI113979, SGI113980, SGI113981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113982, SGI0, SGI0, SGI0, SGI113983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24977[] = {SGI113984, SGI113985, SGI113986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113987, SGI0, SGI0, SGI0, SGI113988, SGI113988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24978[] = {SGI113989, SGI113990, SGI113991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113992, SGI0, SGI0, SGI0, SGI113993, SGI113993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24979[] = {SGI113994, SGI113995, SGI113996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI113997, SGI0, SGI0, SGI0, SGI113998, SGI113998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24980[] = {SGI113999, SGI114000, SGI114001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114002, SGI0, SGI0, SGI0, SGI114003, SGI114003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24981[] = {SGI114004, SGI114005, SGI114006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114007, SGI114007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24982[] = {SGI114008, SGI114009, SGI114010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114011, SGI0, SGI0, SGI0, SGI114012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24983[] = {SGI114013, SGI114014, SGI114015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114016, SGI0, SGI0, SGI0, SGI114017, SGI114017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24984[] = {SGI114018, SGI114019, SGI114020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114021, SGI0, SGI0, SGI0, SGI114022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24985[] = {SGI0, SGI114023, SGI114024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114025, SGI0, SGI0, SGI0, SGI114026, SGI114026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24986[] = {SGI114027, SGI114028, SGI114029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114030, SGI0, SGI0, SGI0, SGI114031, SGI114031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24987[] = {SGI114032, SGI114033, SGI114034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114035, SGI0, SGI0, SGI0, SGI114036, SGI114036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24988[] = {SGI114037, SGI114038, SGI114039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114040, SGI0, SGI0, SGI0, SGI114041, SGI114041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24989[] = {SGI114042, SGI114043, SGI114044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114045, SGI0, SGI0, SGI0, SGI114046, SGI114046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24990[] = {SGI114047, SGI114048, SGI114049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114050, SGI0, SGI0, SGI0, SGI114051, SGI114051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24991[] = {SGI114052, SGI114053, SGI114054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114055, SGI0, SGI0, SGI0, SGI114056, SGI114056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24992[] = {SGI114057, SGI114058, SGI114059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114060, SGI0, SGI0, SGI0, SGI114061, SGI114061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24993[] = {SGI114062, SGI114063, SGI114064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114065, SGI0, SGI0, SGI0, SGI114066, SGI114066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24994[] = {SGI114067, SGI114068, SGI114069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114070, SGI0, SGI0, SGI0, SGI114071, SGI114071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24995[] = {SGI114072, SGI114073, SGI114074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114075, SGI0, SGI0, SGI0, SGI114076, SGI114076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24996[] = {SGI114077, SGI114078, SGI114079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114080, SGI0, SGI0, SGI0, SGI114081, SGI114081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24997[] = {SGI114082, SGI114083, SGI114084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114085, SGI0, SGI0, SGI0, SGI114086, SGI114086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24998[] = {SGI114087, SGI114088, SGI114089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114090, SGI0, SGI0, SGI0, SGI114091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG24999[] = {SGI0, SGI114092, SGI114093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114094, SGI0, SGI0, SGI0, SGI114095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25000[] = {SGI114096, SGI114097, SGI114098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114099, SGI0, SGI0, SGI0, SGI114100, SGI114100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25001[] = {SGI114101, SGI114102, SGI114103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114104, SGI0, SGI0, SGI0, SGI114105, SGI114105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25002[] = {SGI114106, SGI114107, SGI114108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114109, SGI0, SGI0, SGI0, SGI114111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25003[] = {SGI114110, SGI114112, SGI114113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114114, SGI0, SGI0, SGI0, SGI114115, SGI0, SGI0, SGI0, SGI0, SGI114116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25004[] = {SGI114117, SGI114118, SGI114119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114121, SGI0, SGI0, SGI0, SGI114122, SGI114122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25005[] = {SGI0, SGI0, SGI114120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114123, SGI114124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25006[] = {SGI114125, SGI0, SGI114126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114128, SGI114130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25007[] = {SGI114127, SGI114129, SGI114131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114132, SGI0, SGI0, SGI0, SGI114133, SGI114133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25008[] = {SGI114134, SGI114135, SGI114136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114137, SGI0, SGI0, SGI0, SGI114138, SGI114138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25009[] = {SGI114139, SGI114141, SGI114142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114144, SGI0, SGI0, SGI0, SGI114145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25010[] = {SGI114140, SGI0, SGI114143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114146, SGI114147, SGI114147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25011[] = {SGI114148, SGI114149, SGI114150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114151, SGI0, SGI0, SGI0, SGI114152, SGI114152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25012[] = {SGI114153, SGI114154, SGI114155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114156, SGI0, SGI0, SGI0, SGI114157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25013[] = {SGI114158, SGI114159, SGI114160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114161, SGI0, SGI0, SGI0, SGI114162, SGI114162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25014[] = {SGI114163, SGI114164, SGI114165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114166, SGI0, SGI0, SGI0, SGI114167, SGI114167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25015[] = {SGI114168, SGI114169, SGI114170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114171, SGI0, SGI0, SGI0, SGI114172, SGI114172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25016[] = {SGI114173, SGI0, SGI114174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114175, SGI0, SGI0, SGI0, SGI114176, SGI114176, SGI0, SGI114178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25017[] = {SGI114177, SGI114179, SGI114180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114181, SGI0, SGI0, SGI0, SGI114182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25018[] = {SGI114183, SGI0, SGI114184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114185, SGI0, SGI0, SGI0, SGI114188, SGI114188, SGI0, SGI114190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25019[] = {SGI114186, SGI114187, SGI114189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114191, SGI0, SGI0, SGI0, SGI114192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25020[] = {SGI114193, SGI114194, SGI114195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114196, SGI0, SGI0, SGI0, SGI114197, SGI114197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25021[] = {SGI114198, SGI0, SGI114199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114200, SGI0, SGI0, SGI0, SGI114202, SGI114202, SGI0, SGI114204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25022[] = {SGI114201, SGI114203, SGI114205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114206, SGI0, SGI0, SGI0, SGI0, SGI114207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25023[] = {SGI114208, SGI114209, SGI114210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114213, SGI0, SGI0, SGI0, SGI114214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25024[] = {SGI114211, SGI0, SGI114212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114216, SGI114218, SGI114218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25025[] = {SGI114215, SGI114217, SGI114219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114220, SGI0, SGI0, SGI0, SGI114221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25026[] = {SGI114223, SGI0, SGI114225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114229, SGI114231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25027[] = {SGI114222, SGI114224, SGI114226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114227, SGI0, SGI0, SGI0, SGI114232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114234, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25028[] = {SGI114228, SGI114230, SGI114233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114235, SGI0, SGI0, SGI0, SGI114236, SGI114236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25029[] = {SGI114237, SGI114238, SGI114239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114241, SGI0, SGI0, SGI0, SGI114242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25030[] = {SGI114240, SGI0, SGI114243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114244, SGI0, SGI0, SGI0, SGI114245, SGI114245, SGI0, SGI114246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25031[] = {SGI114247, SGI114248, SGI114249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114250, SGI0, SGI0, SGI0, SGI114251, SGI114251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25032[] = {SGI114252, SGI114253, SGI114254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114255, SGI0, SGI0, SGI0, SGI114256, SGI114256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114257, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25033[] = {SGI0, SGI114258, SGI114259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114260, SGI114260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25034[] = {SGI0, SGI114261, SGI0, SGI114262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25035[] = {SGI114265, SGI0, SGI114267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25036[] = {SGI114263, SGI114264, SGI114266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114268, SGI0, SGI0, SGI0, SGI114269, SGI114269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25037[] = {SGI0, SGI114270, SGI114271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114272, SGI0, SGI0, SGI0, SGI114273, SGI114273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25038[] = {SGI114274, SGI114275, SGI114276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114277, SGI0, SGI0, SGI0, SGI114278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25039[] = {SGI114279, SGI114280, SGI114281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114282, SGI0, SGI0, SGI0, SGI114283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25040[] = {SGI114284, SGI114286, SGI114288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114290, SGI0, SGI0, SGI0, SGI114292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114294, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25041[] = {SGI114285, SGI114287, SGI114289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114291, SGI0, SGI0, SGI0, SGI114293, SGI114293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25042[] = {SGI114295, SGI0, SGI114296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114297, SGI114298, SGI114298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25043[] = {SGI114299, SGI114300, SGI114301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114302, SGI0, SGI0, SGI0, SGI114303, SGI114303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25044[] = {SGI114305, SGI114307, SGI114309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114311, SGI0, SGI0, SGI0, SGI114313, SGI114313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25045[] = {SGI114304, SGI114306, SGI114308, SGI114310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114312, SGI0, SGI0, SGI0, SGI114314, SGI114315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25046[] = {SGI114316, SGI114317, SGI114318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114319, SGI0, SGI0, SGI0, SGI114320, SGI114320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25047[] = {SGI114321, SGI114322, SGI114323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114324, SGI0, SGI0, SGI0, SGI114325, SGI114325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25048[] = {SGI114326, SGI114327, SGI114328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114329, SGI0, SGI0, SGI0, SGI114330, SGI0, SGI0, SGI0, SGI0, SGI114331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25049[] = {SGI114332, SGI114333, SGI114334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114335, SGI0, SGI0, SGI0, SGI114336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25050[] = {SGI114337, SGI114338, SGI114339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114340, SGI0, SGI0, SGI0, SGI114341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114345, SGI0, SGI0, NULL};
const Segment * const SG25051[] = {SGI114342, SGI114343, SGI114344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114346, SGI0, SGI0, SGI0, SGI114347, SGI114347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25052[] = {SGI114348, SGI114349, SGI114350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114351, SGI0, SGI0, SGI0, SGI114353, SGI114353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25053[] = {SGI114352, SGI114354, SGI114355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114356, SGI0, SGI0, SGI0, SGI114357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114358, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25054[] = {SGI114359, SGI114360, SGI114361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114365, SGI114365, SGI0, SGI0, SGI114367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25055[] = {SGI114362, SGI114363, SGI114364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114366, SGI0, SGI0, SGI0, SGI114368, SGI114368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25056[] = {SGI114369, SGI114370, SGI114371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114373, SGI0, SGI0, SGI0, SGI114376, SGI114376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25057[] = {SGI114372, SGI114374, SGI114375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114377, SGI0, SGI0, SGI0, SGI114378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114379, SGI0, SGI0, NULL};
const Segment * const SG25058[] = {SGI114380, SGI114381, SGI114382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114385, SGI114386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25059[] = {SGI114383, SGI0, SGI114384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25060[] = {SGI114387, SGI114388, SGI114389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114390, SGI0, SGI0, SGI0, SGI114392, SGI114392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25061[] = {SGI114391, SGI114393, SGI114394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114395, SGI0, SGI0, SGI0, SGI114396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114397, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25062[] = {SGI114398, SGI114399, SGI114400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114401, SGI0, SGI0, SGI0, SGI114402, SGI114402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25063[] = {SGI114403, SGI114404, SGI114405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114406, SGI114406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25064[] = {SGI114407, SGI114408, SGI114409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114410, SGI0, SGI0, SGI0, SGI114411, SGI114411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25065[] = {SGI114412, SGI114413, SGI114414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114417, SGI0, SGI0, SGI0, SGI114418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25066[] = {SGI114415, SGI0, SGI114416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114419, SGI0, SGI0, SGI0, SGI114420, SGI114420, SGI0, SGI114421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25067[] = {SGI114422, SGI114423, SGI114424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114425, SGI0, SGI0, SGI0, SGI114426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25068[] = {SGI114427, SGI114428, SGI114429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114430, SGI0, SGI0, SGI0, SGI114431, SGI114431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25069[] = {SGI114432, SGI114433, SGI114434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114436, SGI0, SGI0, SGI0, SGI114439, SGI0, SGI0, SGI0, SGI0, SGI114440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25070[] = {SGI114435, SGI114437, SGI114438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114441, SGI0, SGI0, SGI0, SGI0, SGI114442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25071[] = {SGI114443, SGI0, SGI114444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114445, SGI114446, SGI114446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25072[] = {SGI114447, SGI114448, SGI114449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114450, SGI0, SGI0, SGI0, SGI114451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25073[] = {SGI114452, SGI114453, SGI114454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114455, SGI0, SGI0, SGI0, SGI114456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25074[] = {SGI114457, SGI114458, SGI114459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114460, SGI0, SGI0, SGI0, SGI114461, SGI114461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25075[] = {SGI114462, SGI114463, SGI114464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114465, SGI0, SGI0, SGI0, SGI114466, SGI114466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25076[] = {SGI114467, SGI114468, SGI114469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114470, SGI0, SGI0, SGI0, SGI114471, SGI114471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25077[] = {SGI114472, SGI114473, SGI114474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114475, SGI0, SGI0, SGI0, SGI114476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114477, SGI0, SGI0, NULL};
const Segment * const SG25078[] = {SGI114478, SGI114479, SGI114480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114481, SGI0, SGI0, SGI0, SGI114482, SGI114482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25079[] = {SGI114483, SGI114484, SGI114485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114486, SGI0, SGI0, SGI0, SGI114487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25080[] = {SGI114488, SGI114489, SGI114490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114491, SGI0, SGI0, SGI0, SGI114492, SGI114492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25081[] = {SGI114493, SGI114494, SGI114496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114499, SGI0, SGI0, SGI0, SGI114500, SGI114500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25082[] = {SGI114495, SGI114497, SGI114498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114501, SGI0, SGI0, SGI0, SGI114502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114503, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25083[] = {SGI114504, SGI114505, SGI114506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114507, SGI0, SGI0, SGI0, SGI114508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25084[] = {SGI114509, SGI114510, SGI114511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114512, SGI0, SGI0, SGI0, SGI114513, SGI114513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114514, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25085[] = {SGI114515, SGI114516, SGI114517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114518, SGI0, SGI0, SGI0, SGI114519, SGI114519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25086[] = {SGI114520, SGI114521, SGI114522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114523, SGI0, SGI0, SGI0, SGI114524, SGI114524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25087[] = {SGI114525, SGI114526, SGI114527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114528, SGI0, SGI0, SGI0, SGI114529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25088[] = {SGI114530, SGI114531, SGI114532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114533, SGI0, SGI0, SGI0, SGI114534, SGI114534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25089[] = {SGI114535, SGI114536, SGI114537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114538, SGI0, SGI0, SGI0, SGI114539, SGI114539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25090[] = {SGI114540, SGI114541, SGI114542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114543, SGI0, SGI0, SGI0, SGI114544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25091[] = {SGI114545, SGI114546, SGI114547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114548, SGI0, SGI0, SGI0, SGI114549, SGI114549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25092[] = {SGI114550, SGI114551, SGI114552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114553, SGI0, SGI0, SGI0, SGI114554, SGI114554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25093[] = {SGI114555, SGI114556, SGI114557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114558, SGI0, SGI0, SGI0, SGI114559, SGI0, SGI0, SGI0, SGI0, SGI114560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25094[] = {SGI114561, SGI114562, SGI114563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114564, SGI0, SGI0, SGI0, SGI114565, SGI114565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25095[] = {SGI114566, SGI0, SGI114567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114568, SGI114570, SGI114570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114573, SGI0, NULL};
const Segment * const SG25096[] = {SGI114569, SGI114571, SGI114572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114574, SGI0, SGI0, SGI0, SGI114577, SGI114577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25097[] = {SGI114575, SGI114576, SGI114578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114579, SGI0, SGI0, SGI0, SGI114580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114581, SGI0, SGI0, NULL};
const Segment * const SG25098[] = {SGI114582, SGI114583, SGI114584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114585, SGI0, SGI0, SGI0, SGI114587, SGI114587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25099[] = {SGI114586, SGI114588, SGI114589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114590, SGI114590, SGI0, SGI0, SGI114591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25100[] = {SGI114592, SGI114593, SGI114594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114595, SGI0, SGI0, SGI0, SGI114596, SGI114596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25101[] = {SGI114597, SGI114598, SGI114599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114600, SGI0, SGI0, SGI0, SGI114601, SGI114601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25102[] = {SGI114602, SGI114603, SGI114604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114605, SGI0, SGI0, SGI0, SGI114606, SGI114606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25103[] = {SGI114607, SGI114608, SGI114609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114612, SGI0, SGI0, SGI0, SGI114613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25104[] = {SGI114610, SGI0, SGI114611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114614, SGI0, SGI0, SGI0, SGI114615, SGI114615, SGI0, SGI114616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25105[] = {SGI114617, SGI114618, SGI114619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114620, SGI0, SGI0, SGI0, SGI114621, SGI114621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25106[] = {SGI114622, SGI114623, SGI114624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114625, SGI0, SGI0, SGI0, SGI114627, SGI114627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25107[] = {SGI114626, SGI0, SGI114628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114629, SGI0, SGI0, SGI0, SGI114630, SGI114630, SGI0, SGI114631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25108[] = {SGI114632, SGI114633, SGI114634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114635, SGI0, SGI0, SGI0, SGI114636, SGI114636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25109[] = {SGI114637, SGI114638, SGI114639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114640, SGI0, SGI0, SGI0, SGI114642, SGI114642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25110[] = {SGI114641, SGI114643, SGI114644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114645, SGI0, SGI0, SGI0, SGI114646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25111[] = {SGI114647, SGI0, SGI114648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25112[] = {SGI114649, SGI114650, SGI114652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114655, SGI0, SGI0, SGI0, SGI114657, SGI114657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114659, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25113[] = {SGI114651, SGI114653, SGI114654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114656, SGI0, SGI0, SGI0, SGI114658, SGI114658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25114[] = {SGI114660, SGI114661, SGI114662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114663, SGI0, SGI0, SGI0, SGI114664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25115[] = {SGI114665, SGI114666, SGI114667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114668, SGI0, SGI0, SGI0, SGI114669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114670, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25116[] = {SGI114671, SGI114672, SGI114673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114674, SGI0, SGI0, SGI0, SGI114675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25117[] = {SGI114676, SGI114677, SGI114679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114681, SGI0, SGI0, SGI0, SGI114682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25118[] = {SGI114678, SGI0, SGI114680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114683, SGI114684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25119[] = {SGI114685, SGI114686, SGI114687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114688, SGI0, SGI0, SGI0, SGI114689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25120[] = {SGI114690, SGI114691, SGI114692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114693, SGI0, SGI0, SGI0, SGI114694, SGI114694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25121[] = {SGI114695, SGI114696, SGI114697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114698, SGI0, SGI0, SGI0, SGI114699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25122[] = {SGI114700, SGI114701, SGI114702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114703, SGI0, SGI0, SGI0, SGI114704, SGI114704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25123[] = {SGI114705, SGI114706, SGI114707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114708, SGI0, SGI0, SGI0, SGI114709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25124[] = {SGI114710, SGI114711, SGI114712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114713, SGI0, SGI0, SGI0, SGI114714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25125[] = {SGI114715, SGI114716, SGI114717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114720, SGI0, SGI0, SGI0, SGI114722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114724, SGI0, SGI0, NULL};
const Segment * const SG25126[] = {SGI114718, SGI114719, SGI114721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114723, SGI0, SGI0, SGI0, SGI114725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25127[] = {SGI114726, SGI114727, SGI114728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114731, SGI0, SGI0, SGI0, SGI114733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114734, SGI0, SGI0, NULL};
const Segment * const SG25128[] = {SGI114729, SGI114730, SGI114732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114735, SGI0, SGI0, SGI0, SGI114736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25129[] = {SGI114737, SGI114738, SGI114739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114740, SGI0, SGI0, SGI0, SGI114741, SGI114741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25130[] = {SGI114742, SGI114743, SGI114744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114745, SGI0, SGI0, SGI0, SGI114746, SGI114746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25131[] = {SGI114747, SGI114748, SGI114749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114750, SGI0, SGI0, SGI0, SGI114751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25132[] = {SGI114752, SGI114753, SGI114754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114755, SGI0, SGI0, SGI0, SGI114756, SGI114756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25133[] = {SGI114757, SGI114758, SGI114759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114760, SGI0, SGI0, SGI0, SGI114761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25134[] = {SGI114762, SGI114763, SGI114764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114765, SGI0, SGI0, SGI0, SGI114766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114767, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25135[] = {SGI114768, SGI114769, SGI114770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114771, SGI0, SGI0, SGI0, SGI114772, SGI114772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25136[] = {SGI114773, SGI114774, SGI114775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114776, SGI0, SGI0, SGI0, SGI114777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114778, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25137[] = {SGI114779, SGI114780, SGI114781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114782, SGI0, SGI0, SGI0, SGI114783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25138[] = {SGI114784, SGI0, SGI114785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25139[] = {SGI114786, SGI114787, SGI114788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114789, SGI0, SGI0, SGI0, SGI114790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25140[] = {SGI114791, SGI114792, SGI114793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114794, SGI0, SGI0, SGI0, SGI114795, SGI114795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25141[] = {SGI114796, SGI114797, SGI114799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114806, SGI114806, SGI0, SGI0, SGI114808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25142[] = {SGI114798, SGI114800, SGI114801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114804, SGI0, SGI0, SGI0, SGI114807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114810, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25143[] = {SGI114802, SGI114803, SGI114805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114809, SGI0, SGI0, SGI0, SGI114811, SGI114811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25144[] = {SGI0, SGI114812, SGI114813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114814, SGI114814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25145[] = {SGI114815, SGI114816, SGI114817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114818, SGI0, SGI0, SGI0, SGI114819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114820, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25146[] = {SGI114821, SGI114822, SGI114823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114824, SGI0, SGI0, SGI0, SGI114825, SGI114825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25147[] = {SGI114826, SGI114827, SGI114828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114829, SGI0, SGI0, SGI0, SGI114830, SGI114830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25148[] = {SGI114831, SGI114832, SGI114833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114834, SGI0, SGI0, SGI0, SGI114835, SGI114835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25149[] = {SGI114836, SGI114837, SGI114838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114842, SGI0, SGI0, SGI0, SGI114843, SGI114843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25150[] = {SGI114839, SGI114840, SGI114841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114844, SGI0, SGI0, SGI0, SGI114845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114846, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25151[] = {SGI114847, SGI114848, SGI114849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114850, SGI0, SGI0, SGI0, SGI114851, SGI114851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25152[] = {SGI114852, SGI114853, SGI114854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114855, SGI0, SGI0, SGI0, SGI114856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25153[] = {SGI114857, SGI114858, SGI114859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114860, SGI0, SGI0, SGI0, SGI114861, SGI114861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25154[] = {SGI114862, SGI114863, SGI114864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114865, SGI0, SGI0, SGI0, SGI114866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25155[] = {SGI114867, SGI114868, SGI114869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114870, SGI0, SGI0, SGI0, SGI114871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25156[] = {SGI114872, SGI114873, SGI114874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114875, SGI0, SGI0, SGI0, SGI114876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114877, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25157[] = {SGI114878, SGI114879, SGI114880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114881, SGI0, SGI0, SGI0, SGI114882, SGI114882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25158[] = {SGI114883, SGI114884, SGI114885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114886, SGI0, SGI0, SGI0, SGI114887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25159[] = {SGI114888, SGI114889, SGI114890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114891, SGI0, SGI0, SGI0, SGI114892, SGI114892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25160[] = {SGI114893, SGI114894, SGI114895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114896, SGI0, SGI0, SGI0, SGI114897, SGI114897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25161[] = {SGI114898, SGI114899, SGI114900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114901, SGI0, SGI0, SGI0, SGI114902, SGI114902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25162[] = {SGI114903, SGI114904, SGI114905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114906, SGI0, SGI0, SGI0, SGI114907, SGI114907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25163[] = {SGI114908, SGI114909, SGI114910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114911, SGI0, SGI0, SGI0, SGI114912, SGI114912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25164[] = {SGI114913, SGI114914, SGI114915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114916, SGI0, SGI0, SGI0, SGI114917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25165[] = {SGI114918, SGI114919, SGI114920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114921, SGI0, SGI0, SGI0, SGI114922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114923, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25166[] = {SGI114924, SGI114925, SGI114926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114927, SGI0, SGI0, SGI0, SGI114928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25167[] = {SGI114929, SGI114930, SGI114931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114932, SGI0, SGI0, SGI0, SGI114933, SGI114933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25168[] = {SGI114934, SGI114935, SGI114936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114937, SGI0, SGI0, SGI0, SGI114938, SGI114938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114939, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25169[] = {SGI114940, SGI114941, SGI114942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114943, SGI0, SGI0, SGI0, SGI114944, SGI114944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25170[] = {SGI114945, SGI114946, SGI114947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114948, SGI0, SGI0, SGI0, SGI114949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114950, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25171[] = {SGI114951, SGI114952, SGI114953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114954, SGI0, SGI0, SGI0, SGI114955, SGI114955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25172[] = {SGI114956, SGI114957, SGI114958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114959, SGI0, SGI0, SGI0, SGI114960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114963, SGI0, SGI0, NULL};
const Segment * const SG25173[] = {SGI114961, SGI114962, SGI114964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114965, SGI0, SGI0, SGI0, SGI114966, SGI114966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25174[] = {SGI114967, SGI114968, SGI114969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114970, SGI0, SGI0, SGI0, SGI114973, SGI114973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25175[] = {SGI0, SGI114971, SGI0, SGI114972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25176[] = {SGI114974, SGI114975, SGI114976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114977, SGI0, SGI0, SGI0, SGI114978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25177[] = {SGI114979, SGI114980, SGI114981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114982, SGI0, SGI0, SGI0, SGI114983, SGI0, SGI114984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25178[] = {SGI114985, SGI0, SGI114986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114987, SGI0, SGI0, SGI0, SGI114988, SGI114988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25179[] = {SGI114989, SGI114990, SGI114991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114992, SGI0, SGI0, SGI0, SGI114993, SGI114993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25180[] = {SGI0, SGI114995, SGI114997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI114999, SGI0, SGI0, SGI0, SGI115001, SGI115001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25181[] = {SGI114994, SGI114996, SGI114998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115000, SGI0, SGI0, SGI0, SGI115002, SGI115002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115003, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25182[] = {SGI115004, SGI115005, SGI115006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115007, SGI0, SGI0, SGI0, SGI115008, SGI115008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25183[] = {SGI115009, SGI115010, SGI115011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115012, SGI0, SGI0, SGI0, SGI115013, SGI115013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25184[] = {SGI115014, SGI115015, SGI115016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115017, SGI0, SGI0, SGI0, SGI115019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115022, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25185[] = {SGI115018, SGI115020, SGI115021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115023, SGI0, SGI0, SGI0, SGI115024, SGI115024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25186[] = {SGI115025, SGI115026, SGI115027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115028, SGI0, SGI0, SGI0, SGI115029, SGI0, SGI0, SGI0, SGI0, SGI115031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25187[] = {SGI115030, SGI115032, SGI115033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115034, SGI0, SGI0, SGI0, SGI115035, SGI115035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25188[] = {SGI115036, SGI115037, SGI115038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115039, SGI0, SGI0, SGI0, SGI115040, SGI115040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25189[] = {SGI115041, SGI115042, SGI115043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115044, SGI0, SGI0, SGI0, SGI115045, SGI115045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115046, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25190[] = {SGI115047, SGI115048, SGI115049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115050, SGI0, SGI0, SGI0, SGI115051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25191[] = {SGI115052, SGI115053, SGI115054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115055, SGI0, SGI0, SGI0, SGI115056, SGI115057, SGI0, SGI115058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25192[] = {SGI115059, SGI115060, SGI115061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115062, SGI0, SGI0, SGI0, SGI115063, SGI115063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25193[] = {SGI115065, SGI115067, SGI115069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115070, SGI0, SGI0, SGI0, SGI115072, SGI115072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25194[] = {SGI115064, SGI115066, SGI115068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115071, SGI0, SGI0, SGI0, SGI115073, SGI115073, SGI0, SGI115074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25195[] = {SGI115075, SGI115076, SGI115077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115079, SGI0, SGI0, SGI0, SGI115082, SGI115082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25196[] = {SGI115078, SGI115080, SGI115081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115083, SGI115083, SGI0, SGI115084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25197[] = {SGI115085, SGI115086, SGI115087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115088, SGI0, SGI0, SGI0, SGI115089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25198[] = {SGI115090, SGI115091, SGI115092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115093, SGI0, SGI0, SGI0, SGI115094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25199[] = {SGI115095, SGI115096, SGI115097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115098, SGI0, SGI0, SGI0, SGI115099, SGI115099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25200[] = {SGI115100, SGI115101, SGI115103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115106, SGI0, SGI0, SGI0, SGI115109, SGI0, SGI0, SGI0, SGI115110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25201[] = {SGI115102, SGI115104, SGI115105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115107, SGI0, SGI0, SGI0, SGI115108, SGI115108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25202[] = {SGI115111, SGI115112, SGI115113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115114, SGI0, SGI0, SGI0, SGI115115, SGI115115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25203[] = {SGI115116, SGI115117, SGI115118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115119, SGI0, SGI0, SGI0, SGI115120, SGI115120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25204[] = {SGI115121, SGI115122, SGI115123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115124, SGI0, SGI0, SGI0, SGI115125, SGI115125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25205[] = {SGI115126, SGI115127, SGI115128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115129, SGI0, SGI0, SGI0, SGI115130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25206[] = {SGI115131, SGI115133, SGI115135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115137, SGI0, SGI0, SGI0, SGI115139, SGI115139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25207[] = {SGI115132, SGI115134, SGI115136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115138, SGI0, SGI0, SGI0, SGI115140, SGI115140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115141, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25208[] = {SGI115143, SGI115145, SGI115147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115149, SGI0, SGI0, SGI0, SGI115151, SGI115151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115152, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25209[] = {SGI115142, SGI115144, SGI115146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115148, SGI0, SGI0, SGI0, SGI115150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25210[] = {SGI115153, SGI115154, SGI115156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115160, SGI0, SGI0, SGI0, SGI115163, SGI115163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25211[] = {SGI115155, SGI115157, SGI115158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115162, SGI0, SGI0, SGI0, SGI115164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115165, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25212[] = {SGI115159, SGI0, SGI115161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115166, SGI115167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25213[] = {SGI115168, SGI115169, SGI115170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115171, SGI0, SGI0, SGI0, SGI115172, SGI115172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25214[] = {SGI115173, SGI115174, SGI115175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115176, SGI0, SGI0, SGI0, SGI115177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115180, NULL};
const Segment * const SG25215[] = {SGI115178, SGI115179, SGI115181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115182, SGI0, SGI0, SGI0, SGI115183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25216[] = {SGI115184, SGI115185, SGI115186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115187, SGI0, SGI0, SGI0, SGI115188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115191, SGI0, SGI0, NULL};
const Segment * const SG25217[] = {SGI115189, SGI115190, SGI115192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115193, SGI0, SGI0, SGI0, SGI115194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25218[] = {SGI115195, SGI115196, SGI115197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115199, SGI0, SGI0, SGI0, SGI115202, SGI115202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25219[] = {SGI115198, SGI115200, SGI115201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115203, SGI0, SGI0, SGI0, SGI115204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115205, NULL};
const Segment * const SG25220[] = {SGI115206, SGI115207, SGI115208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115209, SGI0, SGI0, SGI0, SGI115210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25221[] = {SGI115211, SGI115212, SGI115213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115214, SGI0, SGI0, SGI0, SGI115215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25222[] = {SGI115216, SGI115217, SGI115218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115219, SGI0, SGI0, SGI0, SGI115220, SGI115220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25223[] = {SGI115221, SGI115222, SGI115223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115224, SGI0, SGI0, SGI0, SGI115225, SGI115225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25224[] = {SGI115226, SGI115227, SGI115228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115229, SGI0, SGI0, SGI0, SGI115230, SGI115230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25225[] = {SGI115231, SGI115232, SGI115233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115234, SGI0, SGI0, SGI0, SGI115235, SGI0, SGI0, SGI115236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25226[] = {SGI115237, SGI115238, SGI115239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115240, SGI0, SGI0, SGI0, SGI115241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115243, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25227[] = {SGI115242, SGI115244, SGI115245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115246, SGI0, SGI0, SGI0, SGI115247, SGI115247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25228[] = {SGI115248, SGI115249, SGI115250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115251, SGI115251, SGI0, SGI115252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25229[] = {SGI115253, SGI115254, SGI115255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115256, SGI0, SGI0, SGI0, SGI115257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25230[] = {SGI115258, SGI115259, SGI115260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115264, SGI0, SGI0, SGI0, SGI115265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25231[] = {SGI115261, SGI115262, SGI115263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115266, SGI115266, SGI0, SGI115267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25232[] = {SGI115268, SGI115269, SGI115270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115272, SGI0, SGI0, SGI0, SGI115275, SGI115275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25233[] = {SGI115271, SGI115273, SGI115274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115276, SGI0, SGI0, SGI0, SGI115277, SGI115277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115278, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25234[] = {SGI115279, SGI115280, SGI115281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115282, SGI0, SGI0, SGI0, SGI115283, SGI115283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25235[] = {SGI115284, SGI115285, SGI115286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115287, SGI0, SGI0, SGI0, SGI115288, SGI115288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115289, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25236[] = {SGI115290, SGI115291, SGI115292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115293, SGI0, SGI0, SGI0, SGI115294, SGI115294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25237[] = {SGI115295, SGI115296, SGI115297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115298, SGI0, SGI0, SGI0, SGI115299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115300, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25238[] = {SGI115301, SGI115302, SGI115303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115304, SGI0, SGI0, SGI0, SGI115305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25239[] = {SGI115306, SGI115307, SGI115308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115309, SGI0, SGI0, SGI0, SGI115310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25240[] = {SGI115311, SGI0, SGI115312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25241[] = {SGI115313, SGI115314, SGI115315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115316, SGI0, SGI0, SGI0, SGI115317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25242[] = {SGI115318, SGI115319, SGI115320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115321, SGI0, SGI0, SGI0, SGI115322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25243[] = {SGI115323, SGI115324, SGI115325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115326, SGI0, SGI0, SGI0, SGI115327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25244[] = {SGI115328, SGI0, SGI115329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25245[] = {SGI115330, SGI115331, SGI115332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115333, SGI0, SGI0, SGI0, SGI115334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25246[] = {SGI115335, SGI115336, SGI115337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115338, SGI0, SGI0, SGI0, SGI115339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25247[] = {SGI115340, SGI0, SGI115341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115342, SGI115343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25248[] = {SGI115344, SGI115345, SGI115346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115348, SGI0, SGI0, SGI0, SGI115351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25249[] = {SGI115347, SGI115349, SGI115350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115352, SGI0, SGI0, SGI0, SGI115353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25250[] = {SGI115354, SGI115355, SGI115356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115357, SGI0, SGI0, SGI0, SGI115358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25251[] = {SGI115359, SGI115360, SGI115361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115362, SGI0, SGI0, SGI0, SGI115363, SGI115363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25252[] = {SGI115364, SGI115365, SGI115366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115367, SGI0, SGI0, SGI0, SGI115368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25253[] = {SGI115369, SGI115370, SGI115371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115372, SGI0, SGI0, SGI0, SGI115373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25254[] = {SGI115374, SGI115375, SGI115376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115377, SGI0, SGI0, SGI0, SGI115378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25255[] = {SGI115379, SGI115380, SGI115381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115382, SGI0, SGI0, SGI0, SGI115383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25256[] = {SGI115384, SGI115385, SGI115386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115387, SGI0, SGI0, SGI0, SGI115388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25257[] = {SGI115389, SGI115390, SGI115391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115392, SGI0, SGI0, SGI0, SGI115393, SGI115393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25258[] = {SGI115394, SGI115395, SGI115397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115400, SGI0, SGI0, SGI0, SGI115401, SGI115401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25259[] = {SGI115396, SGI115398, SGI115399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115402, SGI0, SGI0, SGI0, SGI115403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115404, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25260[] = {SGI115405, SGI115406, SGI115407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115408, SGI0, SGI0, SGI0, SGI115409, SGI115409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25261[] = {SGI115410, SGI115411, SGI115412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115413, SGI0, SGI0, SGI0, SGI115414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25262[] = {SGI115415, SGI115416, SGI115417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115418, SGI0, SGI0, SGI0, SGI115419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25263[] = {SGI115420, SGI115421, SGI115422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115423, SGI0, SGI0, SGI0, SGI115424, SGI115424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25264[] = {SGI115425, SGI115426, SGI115427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115428, SGI0, SGI0, SGI0, SGI115429, SGI115429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25265[] = {SGI115430, SGI115431, SGI115432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115433, SGI0, SGI0, SGI0, SGI115434, SGI115434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25266[] = {SGI115435, SGI115436, SGI115437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115438, SGI0, SGI0, SGI0, SGI115439, SGI115439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25267[] = {SGI115440, SGI115441, SGI115442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115443, SGI0, SGI0, SGI0, SGI115444, SGI115444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25268[] = {SGI115445, SGI0, SGI115446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115448, SGI0, SGI0, SGI0, SGI115453, SGI115453, SGI0, SGI115455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25269[] = {SGI115447, SGI115449, SGI115451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115456, SGI0, SGI0, SGI0, SGI115459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25270[] = {SGI115450, SGI115452, SGI115454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115457, SGI0, SGI0, SGI0, SGI115458, SGI115458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25271[] = {SGI115460, SGI115461, SGI115462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115463, SGI0, SGI0, SGI0, SGI115464, SGI115464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25272[] = {SGI115465, SGI115466, SGI115467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115468, SGI0, SGI0, SGI0, SGI115469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25273[] = {SGI115470, SGI115471, SGI115472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115473, SGI0, SGI0, SGI0, SGI115474, SGI115474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25274[] = {SGI115475, SGI115476, SGI115477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115478, SGI0, SGI0, SGI0, SGI115479, SGI115479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25275[] = {SGI115480, SGI115481, SGI115482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115483, SGI0, SGI0, SGI0, SGI115484, SGI115484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25276[] = {SGI115485, SGI115486, SGI115487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115488, SGI0, SGI0, SGI0, SGI115489, SGI115489, SGI115490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25277[] = {SGI115491, SGI115492, SGI115493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115494, SGI0, SGI0, SGI0, SGI115495, SGI115495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25278[] = {SGI115496, SGI115497, SGI115498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115499, SGI0, SGI0, SGI0, SGI115500, SGI115500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25279[] = {SGI115501, SGI115502, SGI115503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115504, SGI0, SGI0, SGI0, SGI115505, SGI115505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25280[] = {SGI115506, SGI115507, SGI115508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115511, SGI0, SGI0, SGI0, SGI115513, SGI0, SGI0, SGI115514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25281[] = {SGI115509, SGI115510, SGI115512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115515, SGI0, SGI0, SGI0, SGI115516, SGI115516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25282[] = {SGI115517, SGI0, SGI115519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115523, SGI115525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115526, SGI0, NULL};
const Segment * const SG25283[] = {SGI115518, SGI115520, SGI115521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115522, SGI0, SGI0, SGI0, SGI115524, SGI115524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25284[] = {SGI115527, SGI115528, SGI115529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115532, SGI0, SGI0, SGI0, SGI115533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25285[] = {SGI115530, SGI0, SGI115531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115534, SGI0, SGI0, SGI0, SGI115535, SGI115535, SGI0, SGI0, SGI115536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25286[] = {SGI115537, SGI115538, SGI115539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115540, SGI0, SGI0, SGI0, SGI115541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25287[] = {SGI115542, SGI115543, SGI115544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115545, SGI0, SGI0, SGI0, SGI115546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25288[] = {SGI115547, SGI115548, SGI115549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115550, SGI0, SGI0, SGI0, SGI115551, SGI115551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25289[] = {SGI115552, SGI115553, SGI115554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115555, SGI0, SGI0, SGI0, SGI115556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25290[] = {SGI115557, SGI115558, SGI115559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115560, SGI0, SGI0, SGI0, SGI115561, SGI0, SGI0, SGI0, SGI0, SGI115562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25291[] = {SGI115563, SGI115564, SGI115565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115566, SGI0, SGI0, SGI0, SGI115567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25292[] = {SGI115568, SGI0, SGI115569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115573, SGI115575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115577, SGI0, NULL};
const Segment * const SG25293[] = {SGI115570, SGI115571, SGI115572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115574, SGI0, SGI0, SGI0, SGI115576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25294[] = {SGI115578, SGI115579, SGI115580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115581, SGI0, SGI0, SGI0, SGI115582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25295[] = {SGI115583, SGI0, SGI115584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115586, SGI115588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115590, SGI0, NULL};
const Segment * const SG25296[] = {SGI115585, SGI115587, SGI115589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115591, SGI0, SGI0, SGI0, SGI115592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25297[] = {SGI115593, SGI115594, SGI115595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115598, SGI0, SGI0, SGI0, SGI115600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115601, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25298[] = {SGI115596, SGI115597, SGI115599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115602, SGI0, SGI0, SGI0, SGI115603, SGI115603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25299[] = {SGI115604, SGI115605, SGI115606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115607, SGI0, SGI0, SGI0, SGI115608, SGI115608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25300[] = {SGI115609, SGI115610, SGI115611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115612, SGI0, SGI0, SGI0, SGI115613, SGI115613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25301[] = {SGI115614, SGI115615, SGI115616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115617, SGI0, SGI0, SGI0, SGI115618, SGI115618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25302[] = {SGI115619, SGI115620, SGI115621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115622, SGI0, SGI0, SGI0, SGI115623, SGI115623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25303[] = {SGI115624, SGI115625, SGI115626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115627, SGI0, SGI0, SGI0, SGI115628, SGI115628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25304[] = {SGI115629, SGI115630, SGI115631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115632, SGI0, SGI0, SGI0, SGI115633, SGI115633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25305[] = {SGI115634, SGI115635, SGI115636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115637, SGI0, SGI0, SGI0, SGI115638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25306[] = {SGI115639, SGI115640, SGI115641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115642, SGI0, SGI0, SGI0, SGI115643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25307[] = {SGI0, SGI115644, SGI115645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115646, SGI0, SGI0, SGI0, SGI115647, SGI115647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25308[] = {SGI115648, SGI115649, SGI115650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115651, SGI0, SGI0, SGI0, SGI115652, SGI115652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25309[] = {SGI115653, SGI115654, SGI115655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115656, SGI0, SGI0, SGI0, SGI115657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25310[] = {SGI115658, SGI115659, SGI115660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115661, SGI0, SGI0, SGI0, SGI115662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25311[] = {SGI115663, SGI115664, SGI115665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115666, SGI0, SGI0, SGI0, SGI115667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25312[] = {SGI115668, SGI115669, SGI115670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115671, SGI0, SGI0, SGI0, SGI115672, SGI115672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25313[] = {SGI115673, SGI115674, SGI115675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115676, SGI0, SGI0, SGI0, SGI115677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25314[] = {SGI0, SGI115678, SGI115679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115680, SGI0, SGI0, SGI0, SGI115681, SGI115681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25315[] = {SGI115683, SGI115685, SGI115687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115688, SGI0, SGI0, SGI0, SGI115689, SGI115689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25316[] = {SGI115682, SGI115684, SGI115686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115690, SGI115690, SGI0, SGI0, SGI115691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25317[] = {SGI115692, SGI115693, SGI115694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115695, SGI0, SGI0, SGI0, SGI115696, SGI115696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25318[] = {SGI115697, SGI115698, SGI115699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115700, SGI0, SGI0, SGI0, SGI115701, SGI115701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25319[] = {SGI115702, SGI115703, SGI115704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115707, SGI0, SGI0, SGI0, SGI115709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25320[] = {SGI115705, SGI115706, SGI115708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115710, SGI115710, SGI0, SGI0, SGI115711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25321[] = {SGI115712, SGI115713, SGI115714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115715, SGI0, SGI0, SGI0, SGI115716, SGI115716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25322[] = {SGI115717, SGI115718, SGI115719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115720, SGI115720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25323[] = {SGI115721, SGI0, SGI115722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115726, SGI115727, SGI115727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25324[] = {SGI115723, SGI115724, SGI115725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115728, SGI0, SGI0, SGI0, SGI115729, SGI115729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25325[] = {SGI0, SGI115730, SGI115731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115732, SGI0, SGI0, SGI0, SGI115733, SGI115733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25326[] = {SGI115734, SGI115735, SGI115736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115737, SGI0, SGI0, SGI0, SGI115738, SGI115738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25327[] = {SGI115739, SGI115740, SGI115741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115742, SGI0, SGI0, SGI0, SGI115745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25328[] = {SGI115743, SGI115744, SGI115746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115747, SGI115747, SGI0, SGI0, SGI115748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25329[] = {SGI115749, SGI115750, SGI115751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115752, SGI0, SGI0, SGI0, SGI115753, SGI115753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25330[] = {SGI115754, SGI115755, SGI115756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115757, SGI0, SGI0, SGI0, SGI115758, SGI115758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25331[] = {SGI115759, SGI115760, SGI115761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115762, SGI0, SGI0, SGI0, SGI115763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25332[] = {SGI115764, SGI115765, SGI115766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115767, SGI0, SGI0, SGI0, SGI115768, SGI115768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25333[] = {SGI115769, SGI115770, SGI115771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115772, SGI0, SGI0, SGI0, SGI115773, SGI115773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25334[] = {SGI115774, SGI115775, SGI115776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115777, SGI0, SGI0, SGI0, SGI115778, SGI115778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25335[] = {SGI115779, SGI115780, SGI115781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115782, SGI0, SGI0, SGI0, SGI115783, SGI115783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25336[] = {SGI115784, SGI115785, SGI115786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115787, SGI0, SGI0, SGI0, SGI115788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25337[] = {SGI115789, SGI115790, SGI115791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115792, SGI0, SGI0, SGI0, SGI115793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25338[] = {SGI115794, SGI115795, SGI115796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115797, SGI0, SGI0, SGI0, SGI115798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25339[] = {SGI115799, SGI115800, SGI115801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115802, SGI0, SGI0, SGI0, SGI115803, SGI115803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25340[] = {SGI115804, SGI115805, SGI115806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115807, SGI0, SGI0, SGI0, SGI115808, SGI115808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25341[] = {SGI115809, SGI115810, SGI115811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115812, SGI0, SGI0, SGI0, SGI115813, SGI115813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25342[] = {SGI115814, SGI115815, SGI115816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115820, SGI115820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115822, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25343[] = {SGI115817, SGI115818, SGI115819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115821, SGI0, SGI0, SGI0, SGI115823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25344[] = {SGI115824, SGI115825, SGI115826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115829, SGI0, SGI0, SGI0, SGI115831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115833, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25345[] = {SGI115827, SGI115828, SGI115830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115832, SGI0, SGI0, SGI0, SGI115834, SGI115834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25346[] = {SGI115835, SGI115836, SGI115837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115839, SGI0, SGI0, SGI0, SGI115842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25347[] = {SGI115838, SGI115840, SGI115841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115843, SGI0, SGI0, SGI0, SGI115844, SGI115844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115845, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25348[] = {SGI115846, SGI115847, SGI115848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115850, SGI0, SGI0, SGI0, SGI115852, SGI115852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25349[] = {SGI115849, SGI0, SGI115851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115853, SGI115854, SGI115854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25350[] = {SGI115855, SGI115856, SGI115857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115858, SGI0, SGI0, SGI0, SGI115859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25351[] = {SGI115860, SGI115861, SGI115862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115863, SGI0, SGI0, SGI0, SGI115864, SGI115864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25352[] = {SGI115865, SGI115866, SGI115867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115868, SGI0, SGI0, SGI0, SGI115869, SGI115869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25353[] = {SGI115870, SGI115871, SGI115872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115873, SGI0, SGI0, SGI0, SGI115874, SGI115874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25354[] = {SGI115875, SGI115876, SGI115877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115878, SGI0, SGI0, SGI0, SGI115879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25355[] = {SGI115880, SGI115881, SGI115882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115883, SGI0, SGI0, SGI0, SGI115884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25356[] = {SGI115885, SGI115886, SGI115887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115888, SGI0, SGI0, SGI0, SGI115889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25357[] = {SGI0, SGI115890, SGI115891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115892, SGI0, SGI0, SGI0, SGI115893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25358[] = {SGI115894, SGI115895, SGI115896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115897, SGI0, SGI0, SGI0, SGI115898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25359[] = {SGI115899, SGI115900, SGI115901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115902, SGI0, SGI0, SGI0, SGI115903, SGI115903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25360[] = {SGI115904, SGI115905, SGI115906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115907, SGI0, SGI0, SGI0, SGI115908, SGI115908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25361[] = {SGI115909, SGI115910, SGI115911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115912, SGI0, SGI0, SGI0, SGI115913, SGI115913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25362[] = {SGI115914, SGI115915, SGI115916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115917, SGI0, SGI0, SGI0, SGI115918, SGI115918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25363[] = {SGI115919, SGI115920, SGI115921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115922, SGI0, SGI0, SGI0, SGI115923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25364[] = {SGI115924, SGI115925, SGI115926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115927, SGI0, SGI0, SGI0, SGI115928, SGI115928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25365[] = {SGI115929, SGI115930, SGI115931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115932, SGI0, SGI0, SGI0, SGI115933, SGI115933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25366[] = {SGI115934, SGI115935, SGI115936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115937, SGI0, SGI0, SGI0, SGI115938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25367[] = {SGI115939, SGI115940, SGI115941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115942, SGI0, SGI0, SGI0, SGI115943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25368[] = {SGI115944, SGI115945, SGI115946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115947, SGI0, SGI0, SGI0, SGI115948, SGI115948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25369[] = {SGI115949, SGI115950, SGI115951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115952, SGI0, SGI0, SGI0, SGI115953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25370[] = {SGI115954, SGI115955, SGI115956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115957, SGI0, SGI0, SGI0, SGI115958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25371[] = {SGI115959, SGI115960, SGI115961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115962, SGI0, SGI0, SGI0, SGI115963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25372[] = {SGI115964, SGI115965, SGI115966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115967, SGI0, SGI0, SGI0, SGI115968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25373[] = {SGI115969, SGI115970, SGI115971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115972, SGI0, SGI0, SGI0, SGI115973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25374[] = {SGI115974, SGI115975, SGI115976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115977, SGI0, SGI0, SGI0, SGI115978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25375[] = {SGI115979, SGI115980, SGI115981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115982, SGI0, SGI0, SGI0, SGI115983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25376[] = {SGI115984, SGI115985, SGI115986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115987, SGI0, SGI0, SGI0, SGI115988, SGI115988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25377[] = {SGI115989, SGI115990, SGI115991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115992, SGI0, SGI0, SGI0, SGI115993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25378[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25379[] = {SGI115995, SGI115996, SGI115997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI115998, SGI0, SGI0, SGI0, SGI115999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25380[] = {SGI116000, SGI116001, SGI116002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116003, SGI0, SGI0, SGI0, SGI116004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25381[] = {SGI116005, SGI116006, SGI116007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116008, SGI0, SGI0, SGI0, SGI116009, SGI116009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25382[] = {SGI116010, SGI116011, SGI116012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116013, SGI0, SGI0, SGI0, SGI116014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25383[] = {SGI116015, SGI116016, SGI116017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116018, SGI0, SGI0, SGI0, SGI116019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25384[] = {SGI116020, SGI116021, SGI116022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116023, SGI0, SGI0, SGI0, SGI116024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25385[] = {SGI0, SGI116025, SGI116026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116027, SGI0, SGI0, SGI0, SGI116028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25386[] = {SGI116029, SGI116030, SGI116031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116032, SGI0, SGI0, SGI0, SGI116033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25387[] = {SGI116034, SGI116035, SGI116036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116037, SGI0, SGI0, SGI0, SGI116038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25388[] = {SGI116039, SGI116040, SGI116041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116042, SGI0, SGI0, SGI0, SGI116043, SGI116043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25389[] = {SGI116044, SGI116045, SGI116046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116047, SGI0, SGI0, SGI0, SGI116048, SGI116048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25390[] = {SGI116049, SGI116050, SGI116051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116052, SGI0, SGI0, SGI0, SGI116053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25391[] = {SGI116054, SGI116055, SGI116056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116057, SGI0, SGI0, SGI0, SGI116058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25392[] = {SGI116059, SGI116060, SGI116061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116062, SGI0, SGI0, SGI0, SGI116063, SGI116063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25393[] = {SGI116064, SGI116065, SGI116066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116067, SGI0, SGI0, SGI0, SGI116068, SGI116068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25394[] = {SGI116069, SGI116070, SGI116071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116072, SGI0, SGI0, SGI0, SGI116073, SGI116073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25395[] = {SGI116074, SGI116075, SGI116076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116077, SGI0, SGI0, SGI0, SGI116078, SGI116078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25396[] = {SGI116079, SGI116080, SGI116081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116082, SGI0, SGI0, SGI0, SGI116083, SGI116083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25397[] = {SGI116084, SGI116085, SGI116086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116087, SGI0, SGI0, SGI0, SGI116088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25398[] = {SGI116089, SGI116090, SGI116091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116092, SGI0, SGI0, SGI0, SGI116093, SGI116093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25399[] = {SGI116094, SGI116095, SGI116096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116097, SGI0, SGI0, SGI0, SGI116098, SGI116098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25400[] = {SGI116099, SGI116100, SGI116101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116102, SGI0, SGI0, SGI0, SGI116103, SGI116103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25401[] = {SGI116104, SGI116105, SGI116106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116107, SGI0, SGI0, SGI0, SGI116108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25402[] = {SGI116109, SGI116110, SGI116111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116112, SGI0, SGI0, SGI0, SGI116113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25403[] = {SGI116114, SGI116115, SGI116116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116117, SGI0, SGI0, SGI0, SGI116118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25404[] = {SGI116119, SGI116120, SGI116121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116122, SGI0, SGI0, SGI0, SGI116123, SGI116123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25405[] = {SGI116124, SGI116125, SGI116126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116127, SGI0, SGI0, SGI0, SGI116128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25406[] = {SGI116129, SGI116130, SGI116131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116132, SGI0, SGI0, SGI0, SGI116133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25407[] = {SGI116134, SGI0, SGI116135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116136, SGI116137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25408[] = {SGI116138, SGI116139, SGI116140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116141, SGI0, SGI0, SGI0, SGI116142, SGI116142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25409[] = {SGI116143, SGI0, SGI116144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116145, SGI116146, SGI116146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25410[] = {SGI116147, SGI116148, SGI116149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116150, SGI0, SGI0, SGI0, SGI116151, SGI116151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25411[] = {SGI116152, SGI116153, SGI116154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116155, SGI0, SGI0, SGI0, SGI116156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25412[] = {SGI116157, SGI116158, SGI116159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116160, SGI0, SGI0, SGI0, SGI116161, SGI116161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25413[] = {SGI116162, SGI116163, SGI116164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116165, SGI0, SGI0, SGI0, SGI116166, SGI116166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25414[] = {SGI116167, SGI116168, SGI116169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116170, SGI0, SGI0, SGI0, SGI116171, SGI116171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25415[] = {SGI116172, SGI116173, SGI116174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116175, SGI0, SGI0, SGI0, SGI116176, SGI116176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25416[] = {SGI116177, SGI116178, SGI116179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116180, SGI0, SGI0, SGI0, SGI116181, SGI116181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25417[] = {SGI116182, SGI116183, SGI116184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116185, SGI0, SGI0, SGI0, SGI116186, SGI116186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25418[] = {SGI116187, SGI116188, SGI116189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116190, SGI0, SGI0, SGI0, SGI116191, SGI116191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25419[] = {SGI116192, SGI116193, SGI116194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116195, SGI0, SGI0, SGI0, SGI116196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25420[] = {SGI116197, SGI116198, SGI116199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116200, SGI0, SGI0, SGI0, SGI116201, SGI116201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25421[] = {SGI116202, SGI116203, SGI116204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116205, SGI0, SGI0, SGI0, SGI116206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25422[] = {SGI116207, SGI116208, SGI116209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116210, SGI0, SGI0, SGI0, SGI116211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25423[] = {SGI116212, SGI116213, SGI116214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116215, SGI0, SGI0, SGI0, SGI116216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25424[] = {SGI116217, SGI116218, SGI116219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116220, SGI0, SGI0, SGI0, SGI116221, SGI116221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25425[] = {SGI116222, SGI116223, SGI116224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116225, SGI0, SGI0, SGI0, SGI116226, SGI116226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25426[] = {SGI116227, SGI116228, SGI116229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116230, SGI0, SGI0, SGI0, SGI116231, SGI116231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25427[] = {SGI116232, SGI116233, SGI116234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116235, SGI0, SGI0, SGI0, SGI116236, SGI116236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25428[] = {SGI116237, SGI116238, SGI116239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116240, SGI0, SGI0, SGI0, SGI116241, SGI116241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25429[] = {SGI116242, SGI116243, SGI116244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116245, SGI0, SGI0, SGI0, SGI116246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25430[] = {SGI116247, SGI116248, SGI116249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116250, SGI0, SGI0, SGI0, SGI116251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25431[] = {SGI116252, SGI116253, SGI116254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116255, SGI0, SGI0, SGI0, SGI116256, SGI116256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25432[] = {SGI116257, SGI116258, SGI116259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116260, SGI0, SGI0, SGI0, SGI116261, SGI116261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25433[] = {SGI116262, SGI116263, SGI116265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116268, SGI0, SGI0, SGI0, SGI116269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25434[] = {SGI116264, SGI116266, SGI116267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116270, SGI0, SGI0, SGI0, SGI116271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25435[] = {SGI116272, SGI116273, SGI116274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116275, SGI0, SGI0, SGI0, SGI116276, SGI116276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25436[] = {SGI116277, SGI116278, SGI116279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116280, SGI0, SGI0, SGI0, SGI116281, SGI116281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25437[] = {SGI116282, SGI116283, SGI116284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116285, SGI0, SGI0, SGI0, SGI116286, SGI116286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25438[] = {SGI116287, SGI116288, SGI116289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116290, SGI0, SGI0, SGI0, SGI116291, SGI116291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25439[] = {SGI116292, SGI116293, SGI116294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116295, SGI0, SGI0, SGI0, SGI116296, SGI116296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25440[] = {SGI116297, SGI116298, SGI116299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116300, SGI0, SGI0, SGI0, SGI116301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25441[] = {SGI116302, SGI116303, SGI116304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116305, SGI0, SGI0, SGI0, SGI116306, SGI116306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25442[] = {SGI116307, SGI116308, SGI116309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116310, SGI0, SGI0, SGI0, SGI116311, SGI116311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25443[] = {SGI116312, SGI116313, SGI116314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116315, SGI0, SGI0, SGI0, SGI116316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25444[] = {SGI116317, SGI116318, SGI116319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116320, SGI0, SGI0, SGI0, SGI116321, SGI116321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25445[] = {SGI116322, SGI116323, SGI116324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116325, SGI0, SGI0, SGI0, SGI116326, SGI116326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25446[] = {SGI116327, SGI116328, SGI116329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116330, SGI0, SGI0, SGI0, SGI116331, SGI116331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25447[] = {SGI116332, SGI116333, SGI116334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116335, SGI0, SGI0, SGI0, SGI116336, SGI116336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25448[] = {SGI116337, SGI116338, SGI116339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116340, SGI0, SGI0, SGI0, SGI116341, SGI116341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25449[] = {SGI116342, SGI116343, SGI116344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116345, SGI0, SGI0, SGI0, SGI116346, SGI116346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25450[] = {SGI116347, SGI116348, SGI116349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116350, SGI0, SGI0, SGI0, SGI116351, SGI116351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25451[] = {SGI116352, SGI116353, SGI116354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116355, SGI0, SGI0, SGI0, SGI116356, SGI116356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25452[] = {SGI116357, SGI116358, SGI116359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116360, SGI0, SGI0, SGI0, SGI116361, SGI116361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25453[] = {SGI116362, SGI116363, SGI116364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116365, SGI0, SGI0, SGI0, SGI116366, SGI116366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25454[] = {SGI116367, SGI116368, SGI116369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116370, SGI0, SGI0, SGI0, SGI116371, SGI116371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25455[] = {SGI116372, SGI116373, SGI116374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116375, SGI0, SGI0, SGI0, SGI116376, SGI116376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25456[] = {SGI116377, SGI116378, SGI116379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116380, SGI0, SGI0, SGI0, SGI116381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25457[] = {SGI116382, SGI116383, SGI116384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116385, SGI0, SGI0, SGI0, SGI116386, SGI116386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25458[] = {SGI116387, SGI116388, SGI116389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116390, SGI0, SGI0, SGI0, SGI116391, SGI116391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25459[] = {SGI116392, SGI116393, SGI116394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116395, SGI0, SGI0, SGI0, SGI116396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25460[] = {SGI116397, SGI116398, SGI116399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116400, SGI0, SGI0, SGI0, SGI116401, SGI116401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25461[] = {SGI116402, SGI116403, SGI116404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116405, SGI0, SGI0, SGI0, SGI116406, SGI116406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25462[] = {SGI116407, SGI116408, SGI116409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116410, SGI0, SGI0, SGI0, SGI116411, SGI116411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25463[] = {SGI116412, SGI116413, SGI116414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116415, SGI0, SGI0, SGI0, SGI116416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25464[] = {SGI116417, SGI116418, SGI116419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116420, SGI0, SGI0, SGI0, SGI116421, SGI116421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25465[] = {SGI116422, SGI116423, SGI116424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116425, SGI0, SGI0, SGI0, SGI116426, SGI116426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25466[] = {SGI116427, SGI116428, SGI116429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116430, SGI0, SGI0, SGI0, SGI116431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25467[] = {SGI116432, SGI116433, SGI116434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116435, SGI0, SGI0, SGI0, SGI116436, SGI116436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25468[] = {SGI0, SGI116437, SGI116438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116439, SGI0, SGI0, SGI0, SGI116440, SGI116440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25469[] = {SGI116441, SGI116442, SGI116443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116444, SGI0, SGI0, SGI0, SGI116445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25470[] = {SGI116446, SGI116447, SGI116448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116449, SGI0, SGI0, SGI0, SGI116450, SGI116450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25471[] = {SGI116451, SGI116452, SGI116453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116454, SGI0, SGI0, SGI0, SGI116455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25472[] = {SGI116456, SGI116457, SGI116458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116459, SGI0, SGI0, SGI0, SGI116460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25473[] = {SGI116461, SGI116462, SGI116463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116464, SGI0, SGI0, SGI0, SGI116465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25474[] = {SGI0, SGI116466, SGI116467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116468, SGI0, SGI0, SGI0, SGI116469, SGI116469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25475[] = {SGI116470, SGI116471, SGI116472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116473, SGI0, SGI0, SGI0, SGI116474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25476[] = {SGI116475, SGI116476, SGI116477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116478, SGI0, SGI0, SGI0, SGI116479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25477[] = {SGI116480, SGI116481, SGI116482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116483, SGI0, SGI0, SGI0, SGI116484, SGI116484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25478[] = {SGI116485, SGI116486, SGI116487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116488, SGI0, SGI0, SGI0, SGI116489, SGI116489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25479[] = {SGI116490, SGI116491, SGI116492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116493, SGI0, SGI0, SGI0, SGI116494, SGI116494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25480[] = {SGI116495, SGI116496, SGI116497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116498, SGI0, SGI0, SGI0, SGI116499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25481[] = {SGI116501, SGI116503, SGI116504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116507, SGI0, SGI0, SGI0, SGI116508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25482[] = {SGI116500, SGI116502, SGI116505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116506, SGI0, SGI0, SGI0, SGI116509, SGI116509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25483[] = {SGI116510, SGI0, SGI116511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25484[] = {SGI116512, SGI116513, SGI116514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116515, SGI0, SGI0, SGI0, SGI116516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25485[] = {SGI116517, SGI116518, SGI116519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116520, SGI0, SGI0, SGI0, SGI116521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25486[] = {SGI116522, SGI116523, SGI116524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116525, SGI0, SGI0, SGI0, SGI116526, SGI116526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25487[] = {SGI116527, SGI0, SGI116528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116529, SGI116530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25488[] = {SGI116531, SGI116532, SGI116533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116534, SGI0, SGI0, SGI0, SGI116535, SGI116535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25489[] = {SGI116536, SGI116537, SGI116538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116539, SGI0, SGI0, SGI0, SGI116540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25490[] = {SGI116541, SGI0, SGI116542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25491[] = {SGI116543, SGI116544, SGI116545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116546, SGI0, SGI0, SGI0, SGI116547, SGI116547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25492[] = {SGI116548, SGI116549, SGI116550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116551, SGI0, SGI0, SGI0, SGI116552, SGI116552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25493[] = {SGI116553, SGI116554, SGI116555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116556, SGI0, SGI0, SGI0, SGI116557, SGI116557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25494[] = {SGI116558, SGI116559, SGI116560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116561, SGI0, SGI0, SGI0, SGI116562, SGI116562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25495[] = {SGI116566, SGI0, SGI116567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116568, SGI116569, SGI116569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25496[] = {SGI116570, SGI0, SGI116571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116572, SGI116573, SGI116573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25497[] = {SGI116574, SGI0, SGI116575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116576, SGI116577, SGI116577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25498[] = {SGI116563, SGI116564, SGI116565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116578, SGI0, SGI0, SGI0, SGI116579, SGI116579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25499[] = {SGI116580, SGI116581, SGI116582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116583, SGI0, SGI0, SGI0, SGI116584, SGI116584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25500[] = {SGI116585, SGI116586, SGI116587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116588, SGI0, SGI0, SGI0, SGI116589, SGI116589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25501[] = {SGI116590, SGI116591, SGI116592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116593, SGI0, SGI0, SGI0, SGI116594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25502[] = {SGI116595, SGI0, SGI116596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116597, SGI116599, SGI116599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116602, SGI0, NULL};
const Segment * const SG25503[] = {SGI116598, SGI116600, SGI116601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116603, SGI0, SGI0, SGI0, SGI116604, SGI116604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25504[] = {SGI116605, SGI116606, SGI116607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116608, SGI0, SGI0, SGI0, SGI116609, SGI116609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25505[] = {SGI116610, SGI116611, SGI116612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116613, SGI0, SGI0, SGI0, SGI116614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25506[] = {SGI116615, SGI116616, SGI116617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116618, SGI0, SGI0, SGI0, SGI116619, SGI116619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25507[] = {SGI116620, SGI116621, SGI116622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116623, SGI0, SGI0, SGI0, SGI116624, SGI116624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25508[] = {SGI116625, SGI116626, SGI116627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116628, SGI0, SGI0, SGI0, SGI116629, SGI116629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25509[] = {SGI116630, SGI116631, SGI116632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116633, SGI0, SGI0, SGI0, SGI116634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25510[] = {SGI116635, SGI116636, SGI116637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116638, SGI0, SGI0, SGI0, SGI116639, SGI116639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25511[] = {SGI116640, SGI116641, SGI116642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116643, SGI0, SGI0, SGI0, SGI116644, SGI116644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25512[] = {SGI116645, SGI116646, SGI116647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116648, SGI0, SGI0, SGI0, SGI116649, SGI116649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25513[] = {SGI116650, SGI116651, SGI116652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116653, SGI0, SGI0, SGI0, SGI116654, SGI116654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25514[] = {SGI116655, SGI116656, SGI116657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116658, SGI0, SGI0, SGI0, SGI116659, SGI116659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25515[] = {SGI116660, SGI116661, SGI116662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116663, SGI0, SGI0, SGI0, SGI116664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25516[] = {SGI116665, SGI116666, SGI116667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116668, SGI0, SGI0, SGI0, SGI116669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25517[] = {SGI116670, SGI116671, SGI116672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116673, SGI0, SGI0, SGI0, SGI116674, SGI116674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25518[] = {SGI116675, SGI116676, SGI116677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116678, SGI0, SGI0, SGI0, SGI116679, SGI116679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25519[] = {SGI116680, SGI116681, SGI116682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116683, SGI0, SGI0, SGI0, SGI116684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25520[] = {SGI116685, SGI116686, SGI116687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116688, SGI0, SGI0, SGI0, SGI116689, SGI116689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25521[] = {SGI116690, SGI116691, SGI116692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116693, SGI0, SGI0, SGI0, SGI116694, SGI116694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25522[] = {SGI116695, SGI116696, SGI116697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116698, SGI0, SGI0, SGI0, SGI116699, SGI116699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25523[] = {SGI116700, SGI116701, SGI116702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116703, SGI0, SGI0, SGI0, SGI116704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25524[] = {SGI116705, SGI116706, SGI116707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116708, SGI0, SGI0, SGI0, SGI116709, SGI116709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25525[] = {SGI116710, SGI116711, SGI116712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116713, SGI0, SGI0, SGI0, SGI116714, SGI116714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25526[] = {SGI116715, SGI116716, SGI116717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116718, SGI0, SGI0, SGI0, SGI116719, SGI116719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25527[] = {SGI116720, SGI116721, SGI116722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116723, SGI0, SGI0, SGI0, SGI116724, SGI116724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25528[] = {SGI116725, SGI116726, SGI116727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116728, SGI0, SGI0, SGI0, SGI116729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25529[] = {SGI116730, SGI116731, SGI116732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116733, SGI0, SGI0, SGI0, SGI116734, SGI116734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25530[] = {SGI116735, SGI116736, SGI116737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116738, SGI0, SGI0, SGI0, SGI116739, SGI116739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25531[] = {SGI116740, SGI116741, SGI116742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116743, SGI0, SGI0, SGI0, SGI116744, SGI116744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25532[] = {SGI116745, SGI116746, SGI116747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116748, SGI0, SGI0, SGI0, SGI116749, SGI116749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25533[] = {SGI116750, SGI116751, SGI116752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116753, SGI0, SGI0, SGI0, SGI116754, SGI116754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25534[] = {SGI116755, SGI116756, SGI116757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116758, SGI0, SGI0, SGI0, SGI116759, SGI116759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25535[] = {SGI116760, SGI116761, SGI116762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116763, SGI0, SGI0, SGI0, SGI116764, SGI116764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25536[] = {SGI0, SGI116765, SGI116766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116767, SGI0, SGI0, SGI0, SGI116768, SGI116768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25537[] = {SGI116769, SGI116770, SGI116771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116772, SGI0, SGI0, SGI0, SGI116773, SGI116773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25538[] = {SGI116774, SGI116775, SGI116776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116777, SGI0, SGI0, SGI0, SGI116778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25539[] = {SGI116779, SGI116780, SGI116781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116782, SGI0, SGI0, SGI0, SGI116783, SGI116783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25540[] = {SGI116784, SGI116785, SGI116786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116787, SGI0, SGI0, SGI0, SGI116788, SGI116788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25541[] = {SGI116789, SGI116790, SGI116791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116792, SGI0, SGI0, SGI0, SGI116793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25542[] = {SGI116794, SGI116795, SGI116796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116797, SGI0, SGI0, SGI0, SGI116798, SGI116798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25543[] = {SGI116799, SGI116800, SGI116801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116802, SGI0, SGI0, SGI0, SGI116803, SGI116803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25544[] = {SGI116804, SGI0, SGI116805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116806, SGI0, SGI0, SGI0, SGI116807, SGI116807, SGI0, SGI0, SGI116808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25545[] = {SGI116809, SGI116810, SGI116811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116812, SGI0, SGI0, SGI0, SGI116813, SGI116813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25546[] = {SGI116814, SGI116815, SGI116816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116817, SGI0, SGI0, SGI0, SGI116818, SGI116818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25547[] = {SGI116819, SGI116820, SGI116821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116822, SGI0, SGI0, SGI0, SGI116823, SGI116823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25548[] = {SGI116824, SGI116825, SGI116826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116827, SGI0, SGI0, SGI0, SGI116828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25549[] = {SGI116829, SGI116830, SGI116831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116832, SGI0, SGI0, SGI0, SGI116833, SGI116833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25550[] = {SGI116834, SGI116835, SGI116836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116837, SGI0, SGI0, SGI0, SGI116838, SGI116838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25551[] = {SGI116839, SGI116840, SGI116841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116842, SGI0, SGI0, SGI0, SGI116843, SGI116843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25552[] = {SGI116844, SGI116845, SGI116846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116847, SGI0, SGI0, SGI0, SGI116848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25553[] = {SGI116849, SGI116850, SGI116851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116854, SGI0, SGI0, SGI0, SGI116856, SGI116856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25554[] = {SGI116852, SGI116853, SGI116855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116857, SGI0, SGI0, SGI0, SGI116858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116859, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25555[] = {SGI116860, SGI116861, SGI116862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116863, SGI0, SGI0, SGI0, SGI116864, SGI116864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25556[] = {SGI116865, SGI116866, SGI116867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116868, SGI0, SGI0, SGI0, SGI116869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25557[] = {SGI116870, SGI116871, SGI116872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116873, SGI0, SGI0, SGI0, SGI116874, SGI116874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25558[] = {SGI116875, SGI116876, SGI116877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116878, SGI0, SGI0, SGI0, SGI116879, SGI116879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25559[] = {SGI116880, SGI116881, SGI116882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116883, SGI0, SGI0, SGI0, SGI116884, SGI116884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25560[] = {SGI116885, SGI116886, SGI116887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116888, SGI0, SGI0, SGI0, SGI116889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25561[] = {SGI116890, SGI116891, SGI116892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116893, SGI0, SGI0, SGI0, SGI116894, SGI116894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25562[] = {SGI116895, SGI116896, SGI116897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116898, SGI0, SGI0, SGI0, SGI116899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25563[] = {SGI116900, SGI116901, SGI116902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116903, SGI0, SGI0, SGI0, SGI116904, SGI116904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25564[] = {SGI116905, SGI116906, SGI116907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116908, SGI0, SGI0, SGI0, SGI116909, SGI116909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25565[] = {SGI116912, SGI116914, SGI116915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116916, SGI0, SGI0, SGI0, SGI116917, SGI116917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25566[] = {SGI116910, SGI116911, SGI116913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116918, SGI116918, SGI0, SGI0, SGI116919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25567[] = {SGI116920, SGI116921, SGI116922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116923, SGI0, SGI0, SGI0, SGI116924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25568[] = {SGI116925, SGI116926, SGI116927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116928, SGI0, SGI0, SGI0, SGI116929, SGI116929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25569[] = {SGI116930, SGI116931, SGI116932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116933, SGI0, SGI0, SGI0, SGI116934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25570[] = {SGI116935, SGI116936, SGI116937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116938, SGI0, SGI0, SGI0, SGI116939, SGI116939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25571[] = {SGI116940, SGI116941, SGI116942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116943, SGI0, SGI0, SGI0, SGI116944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25572[] = {SGI116945, SGI116946, SGI116947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116948, SGI0, SGI0, SGI0, SGI116949, SGI116949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25573[] = {SGI116950, SGI116951, SGI116952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116953, SGI0, SGI0, SGI0, SGI116954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25574[] = {SGI116955, SGI116956, SGI116957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116958, SGI0, SGI0, SGI0, SGI116959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25575[] = {SGI116960, SGI116961, SGI116962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116963, SGI0, SGI0, SGI0, SGI116964, SGI116964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25576[] = {SGI116965, SGI116966, SGI116967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116968, SGI0, SGI0, SGI0, SGI116969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25577[] = {SGI116970, SGI116971, SGI116972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116973, SGI0, SGI0, SGI0, SGI116974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25578[] = {SGI116975, SGI116976, SGI116977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116978, SGI0, SGI0, SGI0, SGI116979, SGI116979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25579[] = {SGI116980, SGI116981, SGI116982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116986, SGI0, SGI0, SGI0, SGI116987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25580[] = {SGI116983, SGI116984, SGI116985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116988, SGI0, SGI0, SGI0, SGI116989, SGI116989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116990, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25581[] = {SGI116991, SGI116992, SGI116993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116994, SGI0, SGI0, SGI0, SGI116995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25582[] = {SGI116996, SGI0, SGI116997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI116998, SGI116999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25583[] = {SGI117000, SGI117001, SGI117002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117003, SGI0, SGI0, SGI0, SGI117004, SGI117004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25584[] = {SGI117005, SGI117006, SGI117007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117008, SGI0, SGI0, SGI0, SGI117009, SGI117009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25585[] = {SGI117010, SGI117011, SGI117012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117013, SGI0, SGI0, SGI0, SGI117014, SGI117014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25586[] = {SGI117015, SGI117016, SGI117017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117018, SGI0, SGI0, SGI0, SGI117019, SGI117019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25587[] = {SGI117020, SGI117021, SGI117022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117023, SGI0, SGI0, SGI0, SGI117024, SGI117024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25588[] = {SGI117025, SGI117026, SGI117027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117028, SGI0, SGI0, SGI0, SGI117029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25589[] = {SGI117030, SGI117031, SGI117032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117033, SGI0, SGI0, SGI0, SGI117034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25590[] = {SGI117035, SGI117036, SGI117037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117038, SGI0, SGI0, SGI0, SGI117039, SGI117039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25591[] = {SGI117040, SGI0, SGI117041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25592[] = {SGI117042, SGI117043, SGI117044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117045, SGI0, SGI0, SGI0, SGI117046, SGI117046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25593[] = {SGI117047, SGI117048, SGI117049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117050, SGI0, SGI0, SGI0, SGI117051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25594[] = {SGI117052, SGI117053, SGI117054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117055, SGI0, SGI0, SGI0, SGI117056, SGI117056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25595[] = {SGI117057, SGI117058, SGI117059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117060, SGI0, SGI0, SGI0, SGI117061, SGI117061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25596[] = {SGI117062, SGI117063, SGI117064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117065, SGI0, SGI0, SGI0, SGI117066, SGI117066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25597[] = {SGI117067, SGI117068, SGI117069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117070, SGI0, SGI0, SGI0, SGI117071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25598[] = {SGI117072, SGI117073, SGI117074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117075, SGI0, SGI0, SGI0, SGI117076, SGI117076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25599[] = {SGI117077, SGI117078, SGI117079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117080, SGI0, SGI0, SGI0, SGI117081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25600[] = {SGI117082, SGI117083, SGI117084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117085, SGI0, SGI0, SGI0, SGI117086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25601[] = {SGI117087, SGI117088, SGI117089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117090, SGI0, SGI0, SGI0, SGI117091, SGI117091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25602[] = {SGI117092, SGI117093, SGI117094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117095, SGI0, SGI0, SGI0, SGI117096, SGI117096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25603[] = {SGI117097, SGI117098, SGI117099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117100, SGI0, SGI0, SGI0, SGI117101, SGI117101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25604[] = {SGI117102, SGI117103, SGI117104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117105, SGI0, SGI0, SGI0, SGI117106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25605[] = {SGI117107, SGI117108, SGI117109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117110, SGI0, SGI0, SGI0, SGI117111, SGI117111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25606[] = {SGI117112, SGI117113, SGI117114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117115, SGI0, SGI0, SGI0, SGI117116, SGI117116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25607[] = {SGI117117, SGI117118, SGI117119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117120, SGI0, SGI0, SGI0, SGI117121, SGI117121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25608[] = {SGI117122, SGI117123, SGI117124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117125, SGI0, SGI0, SGI0, SGI117126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25609[] = {SGI117127, SGI117128, SGI117129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117130, SGI0, SGI0, SGI0, SGI117131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25610[] = {SGI117132, SGI117133, SGI117134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117135, SGI0, SGI0, SGI0, SGI117136, SGI117136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25611[] = {SGI117137, SGI117138, SGI117139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117140, SGI0, SGI0, SGI0, SGI117141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25612[] = {SGI117142, SGI117143, SGI117144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117145, SGI0, SGI0, SGI0, SGI117146, SGI117146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25613[] = {SGI0, SGI117147, SGI117148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117149, SGI0, SGI0, SGI0, SGI117150, SGI117150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25614[] = {SGI0, SGI117151, SGI117152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117153, SGI0, SGI0, SGI0, SGI117154, SGI117154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25615[] = {SGI117155, SGI117156, SGI117157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117158, SGI0, SGI0, SGI0, SGI117159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25616[] = {SGI117160, SGI0, SGI117161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25617[] = {SGI117162, SGI117163, SGI117164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117165, SGI117165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25618[] = {SGI117166, SGI117167, SGI117168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117169, SGI0, SGI0, SGI0, SGI117170, SGI117170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25619[] = {SGI117171, SGI117172, SGI117173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117174, SGI0, SGI0, SGI0, SGI117175, SGI117175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25620[] = {SGI117176, SGI117177, SGI117178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117179, SGI0, SGI0, SGI0, SGI117180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25621[] = {SGI117181, SGI117182, SGI117183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117184, SGI0, SGI0, SGI0, SGI117185, SGI117185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25622[] = {SGI117186, SGI117187, SGI117188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117189, SGI0, SGI0, SGI0, SGI117190, SGI117190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25623[] = {SGI117191, SGI117192, SGI117193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117194, SGI0, SGI0, SGI0, SGI117195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25624[] = {SGI117196, SGI117197, SGI117198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117199, SGI0, SGI0, SGI0, SGI117200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25625[] = {SGI117201, SGI117202, SGI117203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117204, SGI0, SGI0, SGI0, SGI117205, SGI117205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25626[] = {SGI117206, SGI117207, SGI117208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117209, SGI0, SGI0, SGI0, SGI117210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25627[] = {SGI117211, SGI117212, SGI117213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117214, SGI0, SGI0, SGI0, SGI117215, SGI117215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25628[] = {SGI117216, SGI117217, SGI117218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117219, SGI117219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25629[] = {SGI117220, SGI117221, SGI117222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117223, SGI0, SGI0, SGI0, SGI117224, SGI117224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25630[] = {SGI117225, SGI117226, SGI117227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117228, SGI0, SGI0, SGI0, SGI117229, SGI117229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25631[] = {SGI117230, SGI117231, SGI117232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117233, SGI0, SGI0, SGI0, SGI117234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25632[] = {SGI117235, SGI117236, SGI117237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117240, SGI0, SGI0, SGI0, SGI117241, SGI117241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25633[] = {SGI117238, SGI0, SGI117239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117242, SGI117243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25634[] = {SGI117244, SGI117245, SGI117246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117247, SGI0, SGI0, SGI0, SGI117248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25635[] = {SGI117249, SGI117250, SGI117251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117252, SGI0, SGI0, SGI0, SGI117254, SGI117254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25636[] = {SGI117253, SGI117255, SGI117256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117257, SGI0, SGI0, SGI0, SGI117258, SGI117258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25637[] = {SGI117259, SGI117260, SGI117261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117262, SGI0, SGI0, SGI0, SGI117263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25638[] = {SGI117264, SGI117265, SGI117266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117267, SGI0, SGI0, SGI0, SGI117268, SGI117268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25639[] = {SGI117269, SGI117270, SGI117271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117272, SGI0, SGI0, SGI0, SGI117273, SGI117273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25640[] = {SGI117274, SGI117275, SGI117276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117277, SGI0, SGI0, SGI0, SGI117278, SGI117278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25641[] = {SGI117279, SGI117280, SGI117281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117282, SGI0, SGI0, SGI0, SGI117283, SGI117283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25642[] = {SGI117284, SGI117285, SGI117286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117287, SGI0, SGI0, SGI0, SGI117288, SGI117288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25643[] = {SGI117289, SGI117290, SGI117291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117292, SGI0, SGI0, SGI0, SGI117293, SGI117293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25644[] = {SGI117294, SGI117295, SGI117297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117300, SGI0, SGI0, SGI0, SGI117301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25645[] = {SGI117296, SGI117298, SGI117299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117302, SGI0, SGI117303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25646[] = {SGI117304, SGI117305, SGI117306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117307, SGI0, SGI0, SGI0, SGI117308, SGI117308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25647[] = {SGI117309, SGI117310, SGI117311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117312, SGI0, SGI0, SGI0, SGI117313, SGI0, SGI117314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25648[] = {SGI117315, SGI117316, SGI117317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117318, SGI0, SGI0, SGI0, SGI117319, SGI117319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25649[] = {SGI117320, SGI117321, SGI117322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117323, SGI0, SGI0, SGI0, SGI117324, SGI117324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25650[] = {SGI117325, SGI117326, SGI117327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117328, SGI0, SGI0, SGI0, SGI117329, SGI117329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25651[] = {SGI0, SGI117330, SGI117331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117332, SGI0, SGI0, SGI0, SGI117333, SGI117333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25652[] = {SGI117334, SGI117335, SGI117336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117337, SGI0, SGI0, SGI0, SGI117338, SGI117338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25653[] = {SGI117339, SGI117340, SGI117341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117342, SGI0, SGI0, SGI0, SGI117343, SGI117343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25654[] = {SGI117344, SGI117345, SGI117346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117347, SGI0, SGI0, SGI0, SGI117348, SGI117348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25655[] = {SGI117349, SGI117350, SGI117351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117352, SGI0, SGI0, SGI0, SGI117353, SGI117353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25656[] = {SGI117354, SGI117355, SGI117356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117357, SGI0, SGI0, SGI0, SGI117358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25657[] = {SGI117359, SGI0, SGI117360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117361, SGI117363, SGI117363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25658[] = {SGI117362, SGI0, SGI117364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25659[] = {SGI117365, SGI117366, SGI117367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117368, SGI0, SGI0, SGI0, SGI117369, SGI117369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25660[] = {SGI117370, SGI117371, SGI117372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117373, SGI0, SGI0, SGI0, SGI117374, SGI117374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25661[] = {SGI117375, SGI117376, SGI117377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117378, SGI0, SGI0, SGI0, SGI117379, SGI117379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25662[] = {SGI117380, SGI117381, SGI117382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117383, SGI0, SGI0, SGI0, SGI117384, SGI117384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25663[] = {SGI117385, SGI117386, SGI117387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117388, SGI0, SGI0, SGI0, SGI117389, SGI117389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25664[] = {SGI117390, SGI117391, SGI117392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117393, SGI0, SGI0, SGI0, SGI117394, SGI117394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25665[] = {SGI117395, SGI117396, SGI117397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117398, SGI0, SGI0, SGI0, SGI117399, SGI117399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25666[] = {SGI117400, SGI117401, SGI117402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117403, SGI0, SGI0, SGI0, SGI117404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25667[] = {SGI117405, SGI117406, SGI117407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117408, SGI0, SGI0, SGI0, SGI117409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25668[] = {SGI117410, SGI0, SGI117411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25669[] = {SGI117412, SGI117413, SGI117414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117415, SGI0, SGI0, SGI0, SGI117416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25670[] = {SGI117417, SGI117418, SGI117419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117420, SGI0, SGI0, SGI0, SGI117421, SGI117421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25671[] = {SGI117422, SGI117423, SGI117424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117425, SGI0, SGI0, SGI0, SGI117426, SGI117426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25672[] = {SGI117427, SGI117428, SGI117429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117430, SGI0, SGI0, SGI0, SGI117431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25673[] = {SGI117432, SGI117433, SGI117434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117435, SGI0, SGI0, SGI0, SGI117436, SGI117436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25674[] = {SGI117437, SGI117438, SGI117439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117440, SGI0, SGI0, SGI0, SGI117441, SGI117441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25675[] = {SGI117442, SGI0, SGI117443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117444, SGI0, SGI0, SGI0, SGI117445, SGI117445, SGI0, SGI0, SGI117446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25676[] = {SGI117447, SGI117448, SGI117449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117450, SGI0, SGI0, SGI0, SGI117451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25677[] = {SGI117452, SGI117453, SGI117454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117455, SGI0, SGI0, SGI0, SGI117456, SGI117456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25678[] = {SGI117457, SGI117458, SGI117459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117460, SGI0, SGI0, SGI0, SGI117461, SGI117461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25679[] = {SGI117462, SGI117463, SGI117464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117465, SGI0, SGI0, SGI0, SGI117466, SGI117466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25680[] = {SGI117467, SGI117468, SGI117470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117474, SGI117474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117476, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25681[] = {SGI117469, SGI117471, SGI117472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117473, SGI0, SGI0, SGI0, SGI117475, SGI117475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25682[] = {SGI117477, SGI117478, SGI117479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117480, SGI0, SGI0, SGI0, SGI117481, SGI117481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25683[] = {SGI117482, SGI117483, SGI117484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117485, SGI0, SGI0, SGI0, SGI117486, SGI117486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25684[] = {SGI117487, SGI117488, SGI117489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117490, SGI117490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117491, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25685[] = {SGI117492, SGI117493, SGI117494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117495, SGI0, SGI0, SGI0, SGI117496, SGI117496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25686[] = {SGI117497, SGI117498, SGI117499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117500, SGI0, SGI0, SGI0, SGI117501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25687[] = {SGI117502, SGI117503, SGI117504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117505, SGI0, SGI0, SGI0, SGI117506, SGI117506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25688[] = {SGI117507, SGI117508, SGI117509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117510, SGI0, SGI0, SGI0, SGI117511, SGI117511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25689[] = {SGI117512, SGI117513, SGI117514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117515, SGI0, SGI0, SGI0, SGI117516, SGI117516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25690[] = {SGI117517, SGI0, SGI117518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25691[] = {SGI117519, SGI117520, SGI117521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117522, SGI0, SGI0, SGI0, SGI117523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25692[] = {SGI117524, SGI117525, SGI117526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117527, SGI0, SGI0, SGI0, SGI117528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25693[] = {SGI117529, SGI117530, SGI117531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117532, SGI0, SGI0, SGI0, SGI117533, SGI117533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25694[] = {SGI117534, SGI117535, SGI117536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117537, SGI0, SGI0, SGI0, SGI117538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25695[] = {SGI117539, SGI117540, SGI117541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117542, SGI0, SGI0, SGI0, SGI117543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25696[] = {SGI0, SGI117544, SGI117545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117546, SGI0, SGI0, SGI0, SGI0, SGI117547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25697[] = {SGI117548, SGI117549, SGI117550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117551, SGI0, SGI0, SGI0, SGI117552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25698[] = {SGI117553, SGI117554, SGI117555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117556, SGI0, SGI0, SGI0, SGI117557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25699[] = {SGI117558, SGI117559, SGI117560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117561, SGI0, SGI0, SGI0, SGI117562, SGI117562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25700[] = {SGI117563, SGI0, SGI117564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25701[] = {SGI117565, SGI117566, SGI117567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117568, SGI0, SGI0, SGI0, SGI117569, SGI117569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25702[] = {SGI117570, SGI117571, SGI117572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117573, SGI0, SGI0, SGI0, SGI117574, SGI117574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25703[] = {SGI117575, SGI117576, SGI117577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117578, SGI0, SGI0, SGI0, SGI117579, SGI117579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25704[] = {SGI117580, SGI117581, SGI117582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117583, SGI0, SGI0, SGI0, SGI117584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25705[] = {SGI117585, SGI117586, SGI117587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117588, SGI0, SGI0, SGI0, SGI117589, SGI117589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25706[] = {SGI117590, SGI117591, SGI117592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117593, SGI0, SGI0, SGI0, SGI117594, SGI117594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25707[] = {SGI117595, SGI117596, SGI117597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117598, SGI0, SGI0, SGI0, SGI117599, SGI117599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25708[] = {SGI117600, SGI117601, SGI117602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117603, SGI0, SGI0, SGI0, SGI117604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25709[] = {SGI117605, SGI117606, SGI117607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117608, SGI0, SGI0, SGI0, SGI117609, SGI117609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25710[] = {SGI117610, SGI117611, SGI117612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117613, SGI0, SGI0, SGI0, SGI117614, SGI117614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25711[] = {SGI117615, SGI117616, SGI117617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117618, SGI0, SGI0, SGI0, SGI117619, SGI117619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25712[] = {SGI117620, SGI117621, SGI117622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117623, SGI0, SGI0, SGI0, SGI117624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25713[] = {SGI117625, SGI117626, SGI117627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117628, SGI0, SGI0, SGI0, SGI117629, SGI117629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25714[] = {SGI117630, SGI117631, SGI117632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117633, SGI0, SGI0, SGI0, SGI0, SGI117634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25715[] = {SGI117635, SGI117636, SGI117637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117638, SGI0, SGI0, SGI0, SGI117639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25716[] = {SGI117640, SGI117641, SGI117642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117643, SGI0, SGI0, SGI0, SGI117644, SGI117644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25717[] = {SGI117645, SGI117646, SGI117647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117651, SGI0, SGI0, SGI0, SGI117652, SGI117652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117655, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25718[] = {SGI117648, SGI117649, SGI117650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117653, SGI0, SGI0, SGI0, SGI117654, SGI117654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25719[] = {SGI117656, SGI117657, SGI117658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117659, SGI0, SGI0, SGI0, SGI117660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25720[] = {SGI117661, SGI117662, SGI117663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117664, SGI0, SGI0, SGI0, SGI117665, SGI117665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25721[] = {SGI117666, SGI117667, SGI117668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117669, SGI0, SGI0, SGI0, SGI117670, SGI117670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25722[] = {SGI117671, SGI117672, SGI117673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117674, SGI0, SGI0, SGI0, SGI117675, SGI117675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25723[] = {SGI117676, SGI117677, SGI117678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117679, SGI0, SGI0, SGI0, SGI117680, SGI117680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25724[] = {SGI117681, SGI117682, SGI117683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117684, SGI0, SGI0, SGI0, SGI117685, SGI117685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25725[] = {SGI117686, SGI117687, SGI117688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117689, SGI0, SGI0, SGI0, SGI117690, SGI117690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25726[] = {SGI117691, SGI0, SGI117692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117693, SGI0, SGI0, SGI0, SGI117694, SGI117694, SGI0, SGI0, SGI117695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25727[] = {SGI117696, SGI117697, SGI117698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117699, SGI0, SGI0, SGI0, SGI117700, SGI117700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25728[] = {SGI117701, SGI117702, SGI117703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117704, SGI0, SGI0, SGI0, SGI117706, SGI117706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25729[] = {SGI117705, SGI0, SGI117707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117708, SGI117709, SGI117709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25730[] = {SGI117710, SGI117711, SGI117712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117713, SGI0, SGI0, SGI0, SGI117714, SGI117714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25731[] = {SGI117715, SGI117716, SGI117717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117718, SGI0, SGI0, SGI0, SGI117719, SGI117719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25732[] = {SGI117720, SGI0, SGI117721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25733[] = {SGI117722, SGI117723, SGI117724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117725, SGI0, SGI0, SGI0, SGI117726, SGI117726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25734[] = {SGI117727, SGI117728, SGI117729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117730, SGI0, SGI0, SGI0, SGI117731, SGI117731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25735[] = {SGI117732, SGI0, SGI117733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117735, SGI117737, SGI117737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25736[] = {SGI117734, SGI117736, SGI117738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117739, SGI0, SGI0, SGI0, SGI117740, SGI117740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25737[] = {SGI117741, SGI117742, SGI117743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117744, SGI0, SGI0, SGI0, SGI117745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25738[] = {SGI117746, SGI0, SGI117747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25739[] = {SGI117748, SGI117749, SGI117750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117751, SGI0, SGI0, SGI0, SGI117752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25740[] = {SGI117753, SGI117754, SGI117755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117756, SGI0, SGI0, SGI0, SGI117757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25741[] = {SGI117758, SGI117759, SGI117760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117761, SGI0, SGI0, SGI0, SGI117762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25742[] = {SGI117763, SGI117764, SGI117765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117766, SGI0, SGI0, SGI0, SGI117767, SGI117767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25743[] = {SGI117768, SGI117769, SGI117770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117771, SGI0, SGI0, SGI0, SGI117772, SGI117772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25744[] = {SGI117773, SGI117774, SGI117775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117776, SGI0, SGI0, SGI0, SGI117777, SGI117777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25745[] = {SGI117778, SGI117779, SGI117780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117781, SGI0, SGI0, SGI0, SGI117782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25746[] = {SGI117783, SGI117784, SGI117785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117786, SGI0, SGI0, SGI0, SGI117787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25747[] = {SGI117788, SGI117789, SGI117790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117791, SGI0, SGI0, SGI0, SGI117792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25748[] = {SGI117793, SGI117794, SGI117795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117796, SGI0, SGI0, SGI0, SGI117797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25749[] = {SGI117798, SGI117799, SGI117800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117801, SGI0, SGI0, SGI0, SGI117802, SGI117802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25750[] = {SGI117803, SGI117804, SGI117805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117806, SGI0, SGI0, SGI0, SGI117807, SGI117807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25751[] = {SGI117808, SGI117809, SGI117810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117811, SGI0, SGI0, SGI0, SGI117812, SGI117812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25752[] = {SGI117813, SGI117814, SGI117815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117816, SGI0, SGI0, SGI0, SGI117817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25753[] = {SGI117818, SGI117819, SGI117820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117821, SGI0, SGI0, SGI0, SGI117822, SGI117822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25754[] = {SGI117823, SGI117824, SGI117825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117826, SGI0, SGI0, SGI0, SGI117827, SGI117827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25755[] = {SGI117828, SGI117829, SGI117830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117831, SGI0, SGI0, SGI0, SGI117832, SGI117832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25756[] = {SGI117833, SGI117834, SGI117835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117836, SGI0, SGI0, SGI0, SGI117837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25757[] = {SGI117838, SGI117839, SGI117840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117841, SGI0, SGI0, SGI0, SGI117842, SGI117842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25758[] = {SGI117843, SGI117844, SGI117845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117846, SGI0, SGI0, SGI0, SGI117847, SGI117847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25759[] = {SGI117848, SGI117849, SGI117850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117851, SGI0, SGI0, SGI0, SGI117852, SGI117852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25760[] = {SGI117853, SGI117854, SGI117855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117856, SGI0, SGI0, SGI0, SGI117857, SGI117857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25761[] = {SGI117858, SGI117859, SGI117860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117861, SGI0, SGI0, SGI0, SGI117862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25762[] = {SGI117863, SGI117864, SGI117865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117866, SGI0, SGI0, SGI0, SGI117867, SGI117867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25763[] = {SGI117868, SGI117869, SGI117870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117871, SGI0, SGI0, SGI0, SGI117872, SGI117872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25764[] = {SGI117873, SGI117874, SGI117875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117879, SGI117879, SGI0, SGI0, SGI117880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25765[] = {SGI117876, SGI117877, SGI117878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117881, SGI0, SGI0, SGI0, SGI117882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25766[] = {SGI117883, SGI117884, SGI117885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117886, SGI0, SGI0, SGI0, SGI117887, SGI117887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25767[] = {SGI117888, SGI117889, SGI117890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117891, SGI0, SGI0, SGI0, SGI117892, SGI117892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25768[] = {SGI117893, SGI117894, SGI117895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117896, SGI0, SGI0, SGI0, SGI117897, SGI117897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25769[] = {SGI117898, SGI117899, SGI117900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117901, SGI0, SGI0, SGI0, SGI117902, SGI117902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25770[] = {SGI117903, SGI0, SGI117904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117908, SGI117909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117911, SGI0, NULL};
const Segment * const SG25771[] = {SGI117905, SGI117906, SGI117907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117910, SGI0, SGI0, SGI0, SGI117912, SGI117912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25772[] = {SGI117913, SGI117914, SGI117915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117916, SGI0, SGI0, SGI0, SGI117917, SGI117917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25773[] = {SGI117918, SGI117919, SGI117920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117921, SGI0, SGI0, SGI0, SGI117922, SGI117922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25774[] = {SGI117923, SGI117924, SGI117925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117926, SGI0, SGI0, SGI0, SGI117927, SGI117927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25775[] = {SGI117928, SGI0, SGI117929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25776[] = {SGI117930, SGI117931, SGI117932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117933, SGI0, SGI0, SGI0, SGI117934, SGI117934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25777[] = {SGI117935, SGI117936, SGI117937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117938, SGI0, SGI0, SGI0, SGI117939, SGI117939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25778[] = {SGI117940, SGI117941, SGI117942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117943, SGI0, SGI0, SGI0, SGI117944, SGI117944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25779[] = {SGI117945, SGI117946, SGI117947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117948, SGI0, SGI0, SGI0, SGI117949, SGI117949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25780[] = {SGI117950, SGI117951, SGI117952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117953, SGI0, SGI0, SGI0, SGI117954, SGI117954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25781[] = {SGI117955, SGI117956, SGI117957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117958, SGI0, SGI0, SGI0, SGI117959, SGI117959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25782[] = {SGI117960, SGI117961, SGI117962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117963, SGI0, SGI0, SGI0, SGI117964, SGI117964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25783[] = {SGI117965, SGI0, SGI117966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25784[] = {SGI117967, SGI117968, SGI117969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117970, SGI0, SGI0, SGI0, SGI117971, SGI117971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25785[] = {SGI117972, SGI117973, SGI117974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117975, SGI0, SGI0, SGI0, SGI117976, SGI117976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25786[] = {SGI117977, SGI117978, SGI117979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117980, SGI0, SGI0, SGI0, SGI117981, SGI117981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25787[] = {SGI117982, SGI0, SGI117983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117984, SGI117985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25788[] = {SGI117986, SGI117987, SGI117988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117989, SGI0, SGI0, SGI0, SGI117990, SGI117990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25789[] = {SGI117991, SGI117992, SGI117993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI117994, SGI0, SGI0, SGI0, SGI117995, SGI117995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25790[] = {SGI117996, SGI0, SGI117997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25791[] = {SGI117998, SGI117999, SGI118000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118001, SGI0, SGI0, SGI0, SGI118002, SGI118002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25792[] = {SGI118003, SGI118004, SGI118005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118006, SGI0, SGI0, SGI0, SGI118007, SGI118007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25793[] = {SGI118008, SGI118009, SGI118010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118011, SGI0, SGI0, SGI0, SGI118012, SGI118012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25794[] = {SGI118013, SGI118014, SGI118015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118016, SGI0, SGI0, SGI0, SGI118017, SGI118017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25795[] = {SGI118018, SGI0, SGI118019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118020, SGI0, SGI0, SGI0, SGI118021, SGI118021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25796[] = {SGI118022, SGI118023, SGI118024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118025, SGI118025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25797[] = {SGI118026, SGI0, SGI118027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25798[] = {SGI118028, SGI118029, SGI118030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118031, SGI0, SGI0, SGI0, SGI118032, SGI118032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25799[] = {SGI118033, SGI118034, SGI118035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118036, SGI0, SGI0, SGI0, SGI118037, SGI118037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25800[] = {SGI118038, SGI118039, SGI118040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118041, SGI0, SGI0, SGI0, SGI118042, SGI118042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25801[] = {SGI118043, SGI118044, SGI118045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118046, SGI0, SGI0, SGI0, SGI118047, SGI118047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25802[] = {SGI118048, SGI118049, SGI118050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118051, SGI118052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25803[] = {SGI118053, SGI118054, SGI118055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118056, SGI118056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25804[] = {SGI118057, SGI0, SGI118058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25805[] = {SGI118059, SGI118060, SGI118061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118062, SGI0, SGI0, SGI0, SGI118063, SGI118063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25806[] = {SGI118064, SGI0, SGI118065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25807[] = {SGI118066, SGI118067, SGI118068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118069, SGI0, SGI0, SGI0, SGI118070, SGI118070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25808[] = {SGI118071, SGI118072, SGI118073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118074, SGI0, SGI0, SGI0, SGI118075, SGI118075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25809[] = {SGI118076, SGI118077, SGI118078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118079, SGI0, SGI0, SGI0, SGI118080, SGI118080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25810[] = {SGI118081, SGI0, SGI118082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118083, SGI0, SGI0, SGI0, SGI118084, SGI118084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25811[] = {SGI118085, SGI118086, SGI118087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118088, SGI118088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25812[] = {SGI118089, SGI118090, SGI118091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118092, SGI0, SGI0, SGI0, SGI118093, SGI118093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25813[] = {SGI118094, SGI0, SGI118095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118096, SGI0, SGI0, SGI0, SGI118097, SGI118097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25814[] = {SGI118098, SGI118099, SGI118100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118101, SGI0, SGI0, SGI0, SGI118102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25815[] = {SGI118103, SGI118104, SGI118105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118106, SGI0, SGI0, SGI0, SGI118107, SGI118107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25816[] = {SGI118108, SGI0, SGI118109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118110, SGI0, SGI0, SGI0, SGI118111, SGI118111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25817[] = {SGI118112, SGI118113, SGI118114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118115, SGI118115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25818[] = {SGI118116, SGI118117, SGI118118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118119, SGI118119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25819[] = {SGI118120, SGI118121, SGI118122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118123, SGI0, SGI0, SGI0, SGI118124, SGI118124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25820[] = {SGI118125, SGI118126, SGI118127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118128, SGI0, SGI0, SGI0, SGI118129, SGI118129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25821[] = {SGI118130, SGI118131, SGI118132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118133, SGI118133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25822[] = {SGI118134, SGI0, SGI118135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118136, SGI0, SGI0, SGI0, SGI118137, SGI118137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25823[] = {SGI118138, SGI0, SGI118139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118140, SGI118141, SGI118141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25824[] = {SGI118142, SGI118143, SGI118144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118145, SGI118146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25825[] = {SGI118147, SGI0, SGI118148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118149, SGI0, SGI0, SGI0, SGI118150, SGI118150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25826[] = {SGI0, SGI118151, SGI118152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25827[] = {SGI118153, SGI0, SGI118154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118155, SGI0, SGI0, SGI0, SGI118156, SGI118156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25828[] = {SGI118157, SGI0, SGI118158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118159, SGI118160, SGI118160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25829[] = {SGI118161, SGI118162, SGI118163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118164, SGI0, SGI0, SGI0, SGI118165, SGI118165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25830[] = {SGI118166, SGI118167, SGI118168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118169, SGI118169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25831[] = {SGI0, SGI118170, SGI118171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25832[] = {SGI118172, SGI0, SGI118173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118174, SGI0, SGI0, SGI0, SGI118176, SGI118176, SGI0, SGI0, SGI118178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25833[] = {SGI118175, SGI0, SGI118177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118179, SGI0, SGI0, SGI0, SGI118180, SGI118180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25834[] = {SGI118181, SGI118182, SGI118183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118184, SGI0, SGI0, SGI0, SGI118185, SGI118185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25835[] = {SGI118186, SGI0, SGI118187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118188, SGI118189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25836[] = {SGI118190, SGI0, SGI118191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118192, SGI0, SGI0, SGI0, SGI118193, SGI118193, SGI0, SGI0, SGI118194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25837[] = {SGI118195, SGI118196, SGI118197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118198, SGI0, SGI0, SGI0, SGI118199, SGI118199, SGI0, SGI0, SGI0, SGI118200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25838[] = {SGI118201, SGI118202, SGI118203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118204, SGI118204, SGI0, SGI0, SGI118205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25839[] = {SGI118206, SGI118207, SGI118208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118209, SGI0, SGI0, SGI0, SGI118210, SGI118210, SGI0, SGI0, SGI0, SGI118211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25840[] = {SGI118212, SGI118213, SGI118214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118215, SGI0, SGI0, SGI0, SGI118216, SGI118216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25841[] = {SGI118217, SGI118218, SGI118219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118220, SGI0, SGI0, SGI0, SGI118221, SGI118221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25842[] = {SGI118222, SGI0, SGI118223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25843[] = {SGI118224, SGI118225, SGI118226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118227, SGI0, SGI0, SGI0, SGI118228, SGI118228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25844[] = {SGI118229, SGI0, SGI118230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118231, SGI0, SGI0, SGI0, SGI118232, SGI118232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25845[] = {SGI118233, SGI118234, SGI118235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118236, SGI0, SGI0, SGI0, SGI118237, SGI118237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25846[] = {SGI118238, SGI0, SGI118239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118240, SGI0, SGI0, SGI0, SGI118241, SGI118241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25847[] = {SGI118242, SGI0, SGI118243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118244, SGI0, SGI0, SGI0, SGI118245, SGI118245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25848[] = {SGI118246, SGI0, SGI118247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25849[] = {SGI118248, SGI118249, SGI118250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118251, SGI0, SGI0, SGI0, SGI118252, SGI118252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25850[] = {SGI118253, SGI118254, SGI118255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118256, SGI0, SGI0, SGI0, SGI118257, SGI118257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25851[] = {SGI118258, SGI118259, SGI118260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118261, SGI0, SGI0, SGI0, SGI118262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25852[] = {SGI118263, SGI0, SGI118264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118265, SGI118266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25853[] = {SGI118267, SGI118268, SGI118269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118270, SGI0, SGI0, SGI0, SGI118271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25854[] = {SGI118272, SGI0, SGI118273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118274, SGI118275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25855[] = {SGI118276, SGI118277, SGI118278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118279, SGI0, SGI0, SGI0, SGI118280, SGI118280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118281, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25856[] = {SGI118282, SGI0, SGI118283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118284, SGI118285, SGI118285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25857[] = {SGI118286, SGI0, SGI118287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118288, SGI118289, SGI118289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25858[] = {SGI118290, SGI0, SGI118291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118292, SGI118293, SGI118293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118294, SGI0, NULL};
const Segment * const SG25859[] = {SGI118295, SGI118296, SGI118297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118298, SGI0, SGI0, SGI0, SGI118299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25860[] = {SGI118300, SGI118301, SGI118302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118303, SGI0, SGI0, SGI0, SGI118304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25861[] = {SGI118305, SGI118306, SGI118307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118308, SGI0, SGI0, SGI0, SGI118309, SGI118309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118310, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25862[] = {SGI118311, SGI0, SGI118312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118313, SGI118314, SGI118314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25863[] = {SGI118315, SGI118316, SGI118317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118318, SGI0, SGI0, SGI0, SGI118319, SGI118319, SGI0, SGI118320, SGI0, NULL};
const Segment * const SG25864[] = {SGI118321, SGI118322, SGI118323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118324, SGI0, SGI0, SGI0, SGI118325, SGI118325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118326, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25865[] = {SGI118327, SGI0, SGI118328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118329, SGI0, SGI0, SGI0, SGI118330, SGI118330, SGI0, SGI0, SGI118331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25866[] = {SGI118332, SGI118333, SGI118334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118335, SGI0, SGI0, SGI0, SGI118336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118337, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25867[] = {SGI118338, SGI0, SGI118339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118340, SGI0, SGI0, SGI0, SGI118341, SGI118341, SGI0, SGI0, SGI118342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25868[] = {SGI118343, SGI0, SGI118344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118345, SGI118346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25869[] = {SGI118347, SGI0, SGI118348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118349, SGI118350, SGI118350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25870[] = {SGI118351, SGI0, SGI118352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118353, SGI118354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118355, SGI0, NULL};
const Segment * const SG25871[] = {SGI118356, SGI118357, SGI118358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118359, SGI118359, SGI0, SGI0, SGI118360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25872[] = {SGI118361, SGI118362, SGI118363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118364, SGI118364, SGI0, SGI0, SGI118365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25873[] = {SGI118366, SGI118367, SGI118368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118369, SGI118369, SGI0, SGI0, SGI118370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25874[] = {SGI118371, SGI118372, SGI118373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118374, SGI118374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25875[] = {SGI118375, SGI0, SGI118376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118377, SGI0, SGI0, SGI0, SGI118378, SGI118378, SGI0, SGI0, SGI118379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25876[] = {SGI118380, SGI0, SGI118381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118382, SGI0, SGI0, SGI0, SGI118383, SGI118383, SGI0, SGI0, SGI118384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25877[] = {SGI118385, SGI118386, SGI118387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118388, SGI0, SGI0, SGI0, SGI118389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25878[] = {SGI118390, SGI118391, SGI118392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118393, SGI0, SGI0, SGI0, SGI118394, SGI118394, SGI0, SGI118395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25879[] = {SGI118396, SGI0, SGI118397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118398, SGI118399, SGI118399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25880[] = {SGI118400, SGI0, SGI118401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118402, SGI118403, SGI118403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25881[] = {SGI118404, SGI0, SGI118405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118406, SGI118407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25882[] = {SGI118408, SGI0, SGI118409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118410, SGI118411, SGI118411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118412, SGI0, NULL};
const Segment * const SG25883[] = {SGI118413, SGI118414, SGI118415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118416, SGI0, SGI0, SGI0, SGI118417, SGI118417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118418, SGI0, SGI0, NULL};
const Segment * const SG25884[] = {SGI118419, SGI118420, SGI118421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118422, SGI118422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118423, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25885[] = {SGI118424, SGI0, SGI118425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118426, SGI118427, SGI118427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25886[] = {SGI118428, SGI118429, SGI118430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118431, SGI0, SGI118432, SGI118432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25887[] = {SGI118433, SGI118434, SGI118435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118436, SGI0, SGI0, SGI0, SGI118437, SGI118437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118438, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25888[] = {SGI118439, SGI0, SGI118440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118441, SGI118442, SGI118442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25889[] = {SGI118443, SGI0, SGI118444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118445, SGI118446, SGI118446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25890[] = {SGI118447, SGI118448, SGI118449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118450, SGI0, SGI0, SGI0, SGI118451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25891[] = {SGI118452, SGI118453, SGI118454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118455, SGI0, SGI0, SGI0, SGI118456, SGI118456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118457, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25892[] = {SGI118458, SGI0, SGI118459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118460, SGI118461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25893[] = {SGI118462, SGI0, SGI118463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118464, SGI118465, SGI118465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25894[] = {SGI118466, SGI0, SGI118467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118468, SGI118469, SGI118469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25895[] = {SGI118470, SGI0, SGI118471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118472, SGI118473, SGI118473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25896[] = {SGI118474, SGI118475, SGI118476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118477, SGI0, SGI0, SGI0, SGI118478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25897[] = {SGI118479, SGI118480, SGI118481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118482, SGI0, SGI0, SGI0, SGI118483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25898[] = {SGI118484, SGI0, SGI118485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118486, SGI118487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25899[] = {SGI118488, SGI0, SGI118489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118490, SGI118491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25900[] = {SGI118492, SGI0, SGI118493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118494, SGI118495, SGI118495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25901[] = {SGI118496, SGI0, SGI118497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118498, SGI118499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25902[] = {SGI118500, SGI0, SGI118501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118502, SGI118503, SGI118503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25903[] = {SGI118504, SGI118505, SGI118506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118507, SGI0, SGI0, SGI0, SGI118508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25904[] = {SGI118509, SGI118510, SGI118511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118512, SGI0, SGI0, SGI0, SGI118513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25905[] = {SGI118514, SGI118515, SGI118516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118517, SGI0, SGI0, SGI0, SGI118518, SGI118518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118519, SGI0, SGI0, NULL};
const Segment * const SG25906[] = {SGI118520, SGI118521, SGI118522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118523, SGI118523, SGI0, SGI0, SGI118524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25907[] = {SGI118525, SGI0, SGI118526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118527, SGI118528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25908[] = {SGI118529, SGI118530, SGI118531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118532, SGI0, SGI0, SGI0, SGI118533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118534, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25909[] = {SGI118535, SGI118536, SGI118537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118538, SGI0, SGI0, SGI0, SGI118539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25910[] = {SGI118540, SGI118541, SGI118542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118543, SGI0, SGI0, SGI0, SGI118544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25911[] = {SGI0, SGI118545, SGI118546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118547, SGI0, SGI0, SGI0, SGI118548, SGI118548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25912[] = {SGI118549, SGI118550, SGI118551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118552, SGI0, SGI0, SGI0, SGI118553, SGI118553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25913[] = {SGI118554, SGI0, SGI118555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118556, SGI118557, SGI118557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25914[] = {SGI118558, SGI0, SGI118559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118560, SGI118561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25915[] = {SGI118562, SGI0, SGI118563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118564, SGI118565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25916[] = {SGI118566, SGI0, SGI118567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118568, SGI118569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25917[] = {SGI118570, SGI0, SGI118571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118572, SGI118573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25918[] = {SGI118574, SGI0, SGI118575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118576, SGI118577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118578, SGI0, NULL};
const Segment * const SG25919[] = {SGI118579, SGI0, SGI118580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118581, SGI118582, SGI118582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118583, SGI0, NULL};
const Segment * const SG25920[] = {SGI118584, SGI118585, SGI118586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118587, SGI0, SGI0, SGI0, SGI118588, SGI118588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118589, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25921[] = {SGI118590, SGI118591, SGI118592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118593, SGI0, SGI0, SGI0, SGI118594, SGI118594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25922[] = {SGI118595, SGI118596, SGI118597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118598, SGI0, SGI0, SGI0, SGI118599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25923[] = {SGI118600, SGI0, SGI118601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118602, SGI0, SGI0, SGI0, SGI118603, SGI118603, SGI0, SGI0, SGI118604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25924[] = {SGI118605, SGI118606, SGI118607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118608, SGI0, SGI0, SGI0, SGI118609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25925[] = {SGI118610, SGI0, SGI118611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118612, SGI118613, SGI118613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25926[] = {SGI118614, SGI0, SGI118615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118616, SGI118617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118618, SGI0, NULL};
const Segment * const SG25927[] = {SGI118619, SGI0, SGI118620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118621, SGI118622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25928[] = {SGI118623, SGI0, SGI118624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118625, SGI118626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25929[] = {SGI118627, SGI118628, SGI118629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118630, SGI0, SGI0, SGI0, SGI118631, SGI118631, SGI0, SGI0, SGI118632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25930[] = {SGI118633, SGI118634, SGI118635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118636, SGI118636, SGI0, SGI0, SGI118637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25931[] = {SGI118638, SGI118639, SGI118640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118641, SGI0, SGI0, SGI0, SGI118642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25932[] = {SGI118643, SGI0, SGI118644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118645, SGI118646, SGI118646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25933[] = {SGI118647, SGI0, SGI118648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118649, SGI118650, SGI118650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25934[] = {SGI118651, SGI118652, SGI118653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118654, SGI0, SGI0, SGI0, SGI118655, SGI0, SGI0, SGI118656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25935[] = {SGI118657, SGI0, SGI118658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118659, SGI118660, SGI118660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118661, SGI0, NULL};
const Segment * const SG25936[] = {SGI118662, SGI0, SGI118663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118664, SGI118665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25937[] = {SGI118666, SGI118667, SGI118668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118669, SGI0, SGI0, SGI0, SGI118670, SGI118670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25938[] = {SGI118671, SGI118672, SGI118673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118674, SGI0, SGI0, SGI0, SGI118675, SGI118675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118676, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25939[] = {SGI118677, SGI118678, SGI118679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118680, SGI0, SGI0, SGI0, SGI118681, SGI118681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118682, SGI0, SGI0, NULL};
const Segment * const SG25940[] = {SGI118683, SGI0, SGI118684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118685, SGI118686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25941[] = {SGI118687, SGI0, SGI118688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118689, SGI118690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25942[] = {SGI118691, SGI118692, SGI118693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118694, SGI0, SGI0, SGI0, SGI118695, SGI118695, SGI0, SGI0, SGI0, SGI118696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25943[] = {SGI118697, SGI118698, SGI118699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118700, SGI0, SGI0, SGI0, SGI118701, SGI118701, SGI0, SGI0, NULL};
const Segment * const SG25944[] = {SGI118702, SGI0, SGI118703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118704, SGI118705, SGI118705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25945[] = {SGI118706, SGI0, SGI118707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118708, SGI0, SGI0, SGI0, SGI118709, SGI118709, SGI0, SGI0, SGI118710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25946[] = {SGI118711, SGI0, SGI118712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118713, SGI118714, SGI118714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25947[] = {SGI118715, SGI118716, SGI118717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118718, SGI118718, SGI0, SGI0, SGI118719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25948[] = {SGI118720, SGI0, SGI118721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118722, SGI118723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25949[] = {SGI118724, SGI0, SGI118725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118726, SGI0, SGI0, SGI0, SGI118727, SGI118727, SGI0, SGI0, SGI118728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25950[] = {SGI118729, SGI0, SGI118730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118731, SGI118732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25951[] = {SGI118733, SGI0, SGI118734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118735, SGI118736, SGI118736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25952[] = {SGI118737, SGI0, SGI118738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118739, SGI0, SGI0, SGI0, SGI118740, SGI118740, SGI0, SGI0, SGI118741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25953[] = {SGI118742, SGI118743, SGI118744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118745, SGI0, SGI0, SGI0, SGI118746, SGI118746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118747, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25954[] = {SGI118748, SGI118749, SGI118750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118751, SGI0, SGI0, SGI0, SGI118752, SGI118752, SGI0, SGI118753, SGI0, NULL};
const Segment * const SG25955[] = {SGI118754, SGI0, SGI118755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118756, SGI118757, SGI118757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25956[] = {SGI118758, SGI118759, SGI118760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118761, SGI0, SGI0, SGI0, SGI118762, SGI118762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25957[] = {SGI118763, SGI118764, SGI118765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118766, SGI0, SGI0, SGI0, SGI118767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25958[] = {SGI118768, SGI118769, SGI118770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118771, SGI0, SGI0, SGI0, SGI118772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118773, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25959[] = {SGI118774, SGI0, SGI118775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118776, SGI0, SGI0, SGI0, SGI118777, SGI118777, SGI0, SGI0, SGI118778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25960[] = {SGI118779, SGI118780, SGI118781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118782, SGI118782, SGI0, SGI0, SGI118783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25961[] = {SGI118784, SGI0, SGI118785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118786, SGI118787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25962[] = {SGI118788, SGI0, SGI118789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118790, SGI118791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25963[] = {SGI118792, SGI0, SGI118793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118794, SGI118795, SGI118795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25964[] = {SGI118796, SGI0, SGI118797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118798, SGI118799, SGI118799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118800, SGI0, NULL};
const Segment * const SG25965[] = {SGI118801, SGI0, SGI118802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118803, SGI118804, SGI118804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25966[] = {SGI118805, SGI0, SGI118806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118807, SGI118808, SGI118808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25967[] = {SGI118809, SGI0, SGI118810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118811, SGI118812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25968[] = {SGI118813, SGI0, SGI118814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118815, SGI118816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25969[] = {SGI118817, SGI0, SGI118818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118819, SGI118820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25970[] = {SGI118821, SGI0, SGI118822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118823, SGI118824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25971[] = {SGI118825, SGI0, SGI118826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118827, SGI118828, SGI118828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25972[] = {SGI118829, SGI118830, SGI118831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118832, SGI0, SGI0, SGI0, SGI118833, SGI118833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25973[] = {SGI118834, SGI0, SGI118835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118836, SGI118837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118838, SGI0, NULL};
const Segment * const SG25974[] = {SGI118839, SGI0, SGI118840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118841, SGI118842, SGI118842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118843, SGI0, NULL};
const Segment * const SG25975[] = {SGI118844, SGI118845, SGI118846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118847, SGI0, SGI0, SGI0, SGI118848, SGI118848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25976[] = {SGI118849, SGI0, SGI118850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118851, SGI118852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25977[] = {SGI118853, SGI0, SGI118854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118855, SGI118856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25978[] = {SGI118857, SGI0, SGI118858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118859, SGI118860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25979[] = {SGI118861, SGI118862, SGI118863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118864, SGI0, SGI0, SGI0, SGI118865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25980[] = {SGI118866, SGI0, SGI118867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118868, SGI118869, SGI118869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25981[] = {SGI118870, SGI118871, SGI118872, SGI118873, SGI118874, SGI118874, SGI0, NULL};
const Segment * const SG25982[] = {SGI118875, SGI0, SGI118876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118877, SGI0, SGI0, SGI0, SGI118878, SGI118878, SGI0, SGI0, SGI118879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25983[] = {SGI118880, SGI0, SGI118881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118882, SGI118883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25984[] = {SGI118884, SGI118885, SGI118886, SGI118887, SGI118888, SGI118888, SGI0, NULL};
const Segment * const SG25985[] = {SGI118889, SGI118890, SGI118891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118892, SGI0, SGI0, SGI0, SGI118893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25986[] = {SGI118894, SGI118895, SGI118896, SGI118897, SGI118898, SGI118898, SGI0, NULL};
const Segment * const SG25987[] = {SGI118899, SGI0, SGI118900, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25988[] = {SGI118901, SGI118902, SGI118903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118904, SGI118904, SGI0, SGI0, SGI118905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25989[] = {SGI118906, SGI118907, SGI118908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118909, SGI0, SGI0, SGI0, SGI118910, SGI118910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118911, SGI0, SGI0, NULL};
const Segment * const SG25990[] = {SGI118912, SGI118913, SGI118914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118915, SGI0, SGI0, SGI0, SGI118916, SGI118916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118917, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25991[] = {SGI118918, SGI118919, SGI118920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118921, SGI0, SGI118922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25992[] = {SGI118923, SGI0, SGI118924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118925, SGI118926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25993[] = {SGI118927, SGI118928, SGI118929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118930, SGI0, SGI0, SGI0, SGI118931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118932, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25994[] = {SGI118933, SGI0, SGI118934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118935, SGI118936, SGI118936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25995[] = {SGI118937, SGI0, SGI118938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118939, SGI118940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25996[] = {SGI118941, SGI0, SGI118942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118943, SGI118944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118945, SGI0, NULL};
const Segment * const SG25997[] = {SGI0, SGI0, SGI118946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118947, SGI118948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25998[] = {SGI118949, SGI118950, SGI118951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118952, SGI118952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG25999[] = {SGI118953, SGI118954, SGI118955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118956, SGI0, SGI0, SGI0, SGI118957, SGI118957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26000[] = {SGI118958, SGI118959, SGI118960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118961, SGI0, SGI0, SGI0, SGI118962, SGI118962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26001[] = {SGI118963, SGI0, SGI118964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26002[] = {SGI118965, SGI118966, SGI118967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118968, SGI0, SGI0, SGI0, SGI118969, SGI118969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26003[] = {SGI118970, SGI118971, SGI118972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118973, SGI0, SGI0, SGI0, SGI118974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26004[] = {SGI118975, SGI118976, SGI118977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118978, SGI0, SGI0, SGI0, SGI118979, SGI118979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26005[] = {SGI118980, SGI118981, SGI118982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118983, SGI0, SGI0, SGI0, SGI118984, SGI118984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26006[] = {SGI118985, SGI118986, SGI118987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118988, SGI0, SGI0, SGI0, SGI118989, SGI118989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26007[] = {SGI118990, SGI118991, SGI118992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118993, SGI118993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26008[] = {SGI118994, SGI118995, SGI118996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI118997, SGI0, SGI0, SGI0, SGI118998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26009[] = {SGI118999, SGI119000, SGI119001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119002, SGI0, SGI0, SGI0, SGI119003, SGI119003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26010[] = {SGI119004, SGI0, SGI119005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26011[] = {SGI119006, SGI119007, SGI119008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119009, SGI0, SGI0, SGI0, SGI119010, SGI119010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26012[] = {SGI119011, SGI119012, SGI119013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119014, SGI0, SGI0, SGI0, SGI119015, SGI119015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26013[] = {SGI119016, SGI119017, SGI119018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119019, SGI0, SGI0, SGI0, SGI119020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26014[] = {SGI119021, SGI0, SGI119022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119023, SGI0, SGI0, SGI0, SGI119024, SGI119024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26015[] = {SGI119025, SGI0, SGI119026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26016[] = {SGI119027, SGI119028, SGI119029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119030, SGI0, SGI0, SGI0, SGI119031, SGI119031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26017[] = {SGI119032, SGI119033, SGI119034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119035, SGI119035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26018[] = {SGI119036, SGI119037, SGI119038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119039, SGI0, SGI0, SGI0, SGI119040, SGI119040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26019[] = {SGI119041, SGI0, SGI119042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26020[] = {SGI119043, SGI0, SGI119044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26021[] = {SGI119045, SGI119046, SGI119047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119048, SGI0, SGI0, SGI0, SGI119049, SGI119049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26022[] = {SGI119052, SGI0, SGI119054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26023[] = {SGI119050, SGI119051, SGI119053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119055, SGI119055, SGI0, SGI0, SGI119056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26024[] = {SGI119057, SGI119058, SGI119059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119060, SGI0, SGI0, SGI0, SGI119061, SGI119061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26025[] = {SGI119062, SGI119063, SGI119064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119065, SGI0, SGI0, SGI0, SGI119066, SGI119066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26026[] = {SGI119067, SGI119068, SGI119069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119070, SGI0, SGI0, SGI0, SGI119071, SGI119071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119072, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26027[] = {SGI119073, SGI0, SGI119074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119075, SGI0, SGI0, SGI0, SGI119076, SGI119076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26028[] = {SGI119077, SGI0, SGI119078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119079, SGI119080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119083, SGI0, NULL};
const Segment * const SG26029[] = {SGI119081, SGI0, SGI119082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26030[] = {SGI119084, SGI119085, SGI119086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119087, SGI119087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26031[] = {SGI119088, SGI0, SGI119089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26032[] = {SGI119090, SGI0, SGI119091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119092, SGI119093, SGI119093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26033[] = {SGI119094, SGI119095, SGI119096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119097, SGI0, SGI0, SGI0, SGI119098, SGI119098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26034[] = {SGI119099, SGI119100, SGI119101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119102, SGI0, SGI0, SGI0, SGI119103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26035[] = {SGI119104, SGI119105, SGI119106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119107, SGI0, SGI0, SGI0, SGI119108, SGI119108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119109, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26036[] = {SGI119110, SGI0, SGI119111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119112, SGI119113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26037[] = {SGI119114, SGI119115, SGI119116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119117, SGI0, SGI0, SGI0, SGI119118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26038[] = {SGI119119, SGI119120, SGI119121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119122, SGI0, SGI0, SGI0, SGI119123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119124, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26039[] = {SGI119125, SGI0, SGI119126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119127, SGI119128, SGI119128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26040[] = {SGI119129, SGI119130, SGI119131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119132, SGI0, SGI0, SGI0, SGI119133, SGI119133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119134, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26041[] = {SGI119135, SGI119136, SGI119137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119138, SGI119138, SGI0, SGI0, SGI119139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26042[] = {SGI0, SGI0, SGI119140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119141, SGI119142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26043[] = {SGI119143, SGI119144, SGI119145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119146, SGI0, SGI0, SGI0, SGI119147, SGI119147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26044[] = {SGI119148, SGI0, SGI119149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119150, SGI119151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119152, SGI0, NULL};
const Segment * const SG26045[] = {SGI119153, SGI119154, SGI119155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119156, SGI0, SGI0, SGI0, SGI119157, SGI119157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26046[] = {SGI119158, SGI119159, SGI119160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119161, SGI0, SGI0, SGI0, SGI119162, SGI119162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119163, SGI0, SGI0, NULL};
const Segment * const SG26047[] = {SGI119164, SGI119165, SGI119166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119167, SGI0, SGI0, SGI0, SGI119168, SGI119168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119169, SGI0, SGI0, NULL};
const Segment * const SG26048[] = {SGI119170, SGI119171, SGI119172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119173, SGI119173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119174, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26049[] = {SGI119175, SGI0, SGI119176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119177, SGI119178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26050[] = {SGI119179, SGI119180, SGI119181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119182, SGI0, SGI0, SGI0, SGI119183, SGI119183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26051[] = {SGI119184, SGI119185, SGI119186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119187, SGI0, SGI0, SGI0, SGI119188, SGI119188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26052[] = {SGI119189, SGI119190, SGI119191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119192, SGI0, SGI0, SGI0, SGI119193, SGI119193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26053[] = {SGI119194, SGI119195, SGI119196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119197, SGI0, SGI0, SGI0, SGI119198, SGI119198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26054[] = {SGI119199, SGI119200, SGI119201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119202, SGI0, SGI0, SGI0, SGI119203, SGI119203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26055[] = {SGI119204, SGI0, SGI119205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26056[] = {SGI119206, SGI119207, SGI119208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119209, SGI0, SGI0, SGI0, SGI119210, SGI119210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26057[] = {SGI119211, SGI119212, SGI119213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119214, SGI0, SGI0, SGI0, SGI119215, SGI119215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26058[] = {SGI119216, SGI119217, SGI119218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119219, SGI0, SGI0, SGI0, SGI119220, SGI119220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26059[] = {SGI119221, SGI119222, SGI119223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119224, SGI0, SGI0, SGI0, SGI119225, SGI119225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26060[] = {SGI119226, SGI119227, SGI119228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119229, SGI0, SGI0, SGI0, SGI119230, SGI119230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26061[] = {SGI119231, SGI119232, SGI119233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119234, SGI0, SGI0, SGI0, SGI119235, SGI119235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26062[] = {SGI119236, SGI119237, SGI119238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119239, SGI0, SGI0, SGI0, SGI119240, SGI119240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26063[] = {SGI119241, SGI119242, SGI119243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119247, SGI0, SGI0, SGI0, SGI119248, SGI119248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26064[] = {SGI119244, SGI119245, SGI119246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119249, SGI0, SGI0, SGI0, SGI119250, SGI119250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26065[] = {SGI119251, SGI119252, SGI119253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119254, SGI119254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26066[] = {SGI119255, SGI119256, SGI119257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119258, SGI119258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26067[] = {SGI119259, SGI119260, SGI119261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119262, SGI0, SGI0, SGI0, SGI119263, SGI119263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26068[] = {SGI119264, SGI0, SGI119265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26069[] = {SGI119266, SGI119267, SGI119268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119269, SGI0, SGI0, SGI0, SGI119270, SGI119270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26070[] = {SGI119271, SGI0, SGI119272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26071[] = {SGI119273, SGI0, SGI119274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119275, SGI0, SGI0, SGI0, SGI119276, SGI119276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26072[] = {SGI119277, SGI0, SGI119278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119279, SGI0, SGI0, SGI0, SGI119280, SGI119280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26073[] = {SGI119281, SGI119282, SGI119283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119284, SGI0, SGI0, SGI0, SGI119285, SGI119285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26074[] = {SGI119286, SGI119287, SGI119288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119289, SGI0, SGI0, SGI0, SGI119290, SGI119290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26075[] = {SGI119291, SGI119292, SGI119293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119294, SGI0, SGI0, SGI0, SGI119295, SGI119295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26076[] = {SGI119296, SGI0, SGI119297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26077[] = {SGI119298, SGI119299, SGI119300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119301, SGI0, SGI0, SGI0, SGI119302, SGI119302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26078[] = {SGI119303, SGI0, SGI119304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26079[] = {SGI119305, SGI119306, SGI119307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119308, SGI0, SGI0, SGI0, SGI119309, SGI119309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26080[] = {SGI119310, SGI0, SGI119311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26081[] = {SGI119312, SGI119313, SGI119314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119315, SGI0, SGI0, SGI0, SGI119316, SGI119316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26082[] = {SGI119317, SGI119318, SGI119319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119320, SGI0, SGI0, SGI0, SGI119321, SGI119321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119322, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26083[] = {SGI119323, SGI119324, SGI119325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119327, SGI0, SGI0, SGI0, SGI119330, SGI119330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26084[] = {SGI119326, SGI119328, SGI119329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119331, SGI0, SGI0, SGI0, SGI119332, SGI119332, SGI0, SGI0, NULL};
const Segment * const SG26085[] = {SGI119333, SGI119334, SGI119335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119336, SGI0, SGI0, SGI0, SGI119337, SGI119337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26086[] = {SGI119339, SGI0, SGI119341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119343, SGI0, SGI0, SGI0, SGI119345, SGI119345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26087[] = {SGI119338, SGI119340, SGI119342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119344, SGI0, SGI0, SGI0, SGI119346, SGI119346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119347, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26088[] = {SGI119348, SGI119349, SGI119350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119351, SGI119351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26089[] = {SGI119352, SGI119353, SGI119354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119355, SGI0, SGI0, SGI0, SGI119356, SGI119356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26090[] = {SGI119357, SGI0, SGI119358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119359, SGI0, SGI0, SGI0, SGI119360, SGI119360, SGI0, SGI0, SGI119361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26091[] = {SGI119362, SGI0, SGI119363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119364, SGI119365, SGI119365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26092[] = {SGI119366, SGI119367, SGI119368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119369, SGI0, SGI0, SGI0, SGI119370, SGI119370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26093[] = {SGI119371, SGI119372, SGI119373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119377, SGI119377, SGI0, SGI0, SGI119379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26094[] = {SGI119374, SGI119375, SGI119376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119378, SGI0, SGI0, SGI0, SGI119380, SGI119380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26095[] = {SGI119381, SGI119382, SGI119383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119384, SGI119384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26096[] = {SGI119385, SGI119386, SGI119387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119388, SGI119388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26097[] = {SGI119389, SGI119390, SGI119391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119392, SGI0, SGI0, SGI0, SGI119393, SGI119393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26098[] = {SGI119394, SGI119395, SGI119396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119397, SGI0, SGI0, SGI0, SGI119398, SGI119398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26099[] = {SGI119399, SGI0, SGI119400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119401, SGI0, SGI0, SGI0, SGI119402, SGI119402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26100[] = {SGI119403, SGI119404, SGI119405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119406, SGI0, SGI0, SGI0, SGI119407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26101[] = {SGI119408, SGI0, SGI119409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26102[] = {SGI119410, SGI119411, SGI119412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119413, SGI0, SGI0, SGI0, SGI119414, SGI119414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26103[] = {SGI119415, SGI0, SGI119416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26104[] = {SGI119417, SGI119418, SGI119419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119420, SGI0, SGI0, SGI0, SGI119421, SGI119421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26105[] = {SGI119422, SGI119423, SGI119424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119425, SGI0, SGI0, SGI0, SGI119426, SGI119426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26106[] = {SGI119427, SGI0, SGI119428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26107[] = {SGI119429, SGI119430, SGI119431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119432, SGI0, SGI0, SGI0, SGI119433, SGI119433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26108[] = {SGI119434, SGI119435, SGI119436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119437, SGI0, SGI0, SGI0, SGI119438, SGI119438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26109[] = {SGI119439, SGI119440, SGI119441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119442, SGI0, SGI0, SGI0, SGI119443, SGI119443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26110[] = {SGI119444, SGI119445, SGI119446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119447, SGI0, SGI0, SGI0, SGI119448, SGI119448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26111[] = {SGI119449, SGI0, SGI119450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119451, SGI119452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26112[] = {SGI119453, SGI119454, SGI119455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119456, SGI0, SGI0, SGI0, SGI119457, SGI119457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26113[] = {SGI119458, SGI0, SGI119459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119460, SGI119461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26114[] = {SGI119462, SGI0, SGI119463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119464, SGI119465, SGI119465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26115[] = {SGI119466, SGI0, SGI119467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119468, SGI119469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26116[] = {SGI119470, SGI0, SGI119471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119472, SGI119473, SGI119473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119474, SGI0, NULL};
const Segment * const SG26117[] = {SGI119475, SGI0, SGI119476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119477, SGI119478, SGI119478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119479, SGI0, NULL};
const Segment * const SG26118[] = {SGI119480, SGI0, SGI119481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119482, SGI119483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26119[] = {SGI119484, SGI119485, SGI119486, SGI119487, SGI119488, SGI119488, SGI0, NULL};
const Segment * const SG26120[] = {SGI119489, SGI119490, SGI119491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119492, SGI0, SGI0, SGI0, SGI119493, SGI119493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119494, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26121[] = {SGI119495, SGI119496, SGI119497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119498, SGI0, SGI0, SGI0, SGI119499, SGI119499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119500, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26122[] = {SGI119501, SGI119502, SGI119503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119504, SGI0, SGI0, SGI0, SGI119505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119506, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26123[] = {SGI119507, SGI0, SGI119508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119509, SGI119510, SGI119510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26124[] = {SGI119511, SGI0, SGI119512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119513, SGI119514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26125[] = {SGI119515, SGI119516, SGI119517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119518, SGI0, SGI0, SGI0, SGI119519, SGI119519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119520, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26126[] = {SGI119521, SGI0, SGI119522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119523, SGI0, SGI0, SGI0, SGI119524, SGI119524, SGI0, SGI0, SGI119525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26127[] = {SGI119526, SGI119527, SGI119528, SGI119529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119530, SGI0, SGI0, SGI0, SGI119531, SGI119532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26128[] = {SGI119533, SGI119534, SGI119535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119536, SGI0, SGI0, SGI0, SGI119537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26129[] = {SGI119538, SGI0, SGI119539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119540, SGI119541, SGI119541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26130[] = {SGI119542, SGI119543, SGI119544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119545, SGI119545, SGI0, SGI0, SGI119546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26131[] = {SGI119547, SGI119548, SGI119549, SGI119550, SGI119551, SGI119551, SGI0, NULL};
const Segment * const SG26132[] = {SGI119552, SGI119553, SGI119554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119555, SGI0, SGI0, SGI0, SGI119556, SGI119556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119557, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26133[] = {SGI119558, SGI0, SGI119559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119560, SGI0, SGI0, SGI0, SGI119561, SGI119561, SGI0, SGI0, SGI119562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26134[] = {SGI119563, SGI119564, SGI119565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119566, SGI119566, SGI0, SGI0, SGI119567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26135[] = {SGI119568, SGI119569, SGI119570, SGI119571, SGI119572, SGI119572, SGI0, NULL};
const Segment * const SG26136[] = {SGI119573, SGI119574, SGI119575, SGI119576, SGI119577, SGI119577, SGI0, NULL};
const Segment * const SG26137[] = {SGI119578, SGI0, SGI119579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119580, SGI119581, SGI119581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26138[] = {SGI119582, SGI0, SGI119583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119584, SGI119585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26139[] = {SGI119586, SGI119587, SGI119588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119589, SGI119589, SGI0, SGI0, SGI119590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26140[] = {SGI119591, SGI0, SGI119592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119593, SGI119594, SGI119594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119595, SGI0, NULL};
const Segment * const SG26141[] = {SGI119596, SGI0, SGI119597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119598, SGI119599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26142[] = {SGI119600, SGI0, SGI119601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119602, SGI0, SGI0, SGI0, SGI119603, SGI119603, SGI0, SGI0, SGI119604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26143[] = {SGI119605, SGI0, SGI119606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119607, SGI119608, SGI119608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26144[] = {SGI119609, SGI119610, SGI119611, SGI119612, SGI119613, SGI119613, SGI0, NULL};
const Segment * const SG26145[] = {SGI119614, SGI0, SGI119615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119616, SGI0, SGI0, SGI0, SGI119617, SGI119617, SGI0, SGI0, SGI119618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26146[] = {SGI119619, SGI0, SGI119620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119621, SGI0, SGI0, SGI0, SGI119622, SGI119622, SGI0, SGI0, SGI119623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26147[] = {SGI119624, SGI119625, SGI119626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119627, SGI0, SGI0, SGI0, SGI119628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26148[] = {SGI119629, SGI0, SGI119630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119631, SGI119632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26149[] = {SGI119633, SGI119634, SGI119635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119636, SGI0, SGI0, SGI0, SGI119637, SGI119637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26150[] = {SGI119638, SGI119639, SGI119640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119641, SGI0, SGI0, SGI0, SGI119642, SGI119642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119643, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26151[] = {SGI119644, SGI119645, SGI119646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119647, SGI0, SGI0, SGI0, SGI119648, SGI119648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26152[] = {SGI119649, SGI119650, SGI119651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119652, SGI0, SGI0, SGI0, SGI119653, SGI119653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26153[] = {SGI119654, SGI119656, SGI119658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119660, SGI0, SGI0, SGI0, SGI119662, SGI119662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26154[] = {SGI119655, SGI119657, SGI119659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119661, SGI0, SGI0, SGI0, SGI119663, SGI119663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26155[] = {SGI119664, SGI0, SGI119665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26156[] = {SGI119666, SGI0, SGI119668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26157[] = {SGI119667, SGI119669, SGI119670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119671, SGI0, SGI0, SGI0, SGI119672, SGI119672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26158[] = {SGI119673, SGI119674, SGI119675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119676, SGI0, SGI0, SGI0, SGI119677, SGI119677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26159[] = {SGI119678, SGI119679, SGI119680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119681, SGI0, SGI0, SGI0, SGI119683, SGI119683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26160[] = {SGI119682, SGI119684, SGI119685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119686, SGI0, SGI0, SGI0, SGI119687, SGI119687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26161[] = {SGI119688, SGI119689, SGI119690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119691, SGI119691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26162[] = {SGI119692, SGI119693, SGI119694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119695, SGI119695, SGI0, SGI0, SGI119696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26163[] = {SGI119698, SGI0, SGI119700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26164[] = {SGI119697, SGI119699, SGI119701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119702, SGI0, SGI0, SGI0, SGI119703, SGI119703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26165[] = {SGI119704, SGI119705, SGI119706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119707, SGI0, SGI0, SGI0, SGI119708, SGI119708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26166[] = {SGI119709, SGI119710, SGI119711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119712, SGI0, SGI0, SGI0, SGI119713, SGI119713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26167[] = {SGI119714, SGI119715, SGI119716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119717, SGI0, SGI0, SGI0, SGI119718, SGI119718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26168[] = {SGI119720, SGI119722, SGI119724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119726, SGI119726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26169[] = {SGI119719, SGI119721, SGI119723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119725, SGI0, SGI0, SGI0, SGI119727, SGI119727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26170[] = {SGI119728, SGI119729, SGI119730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119734, SGI0, SGI0, SGI0, SGI119735, SGI119735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26171[] = {SGI119731, SGI119732, SGI119733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119736, SGI0, SGI0, SGI0, SGI119737, SGI119737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26172[] = {SGI119738, SGI119739, SGI119740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119741, SGI0, SGI0, SGI0, SGI119742, SGI119742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26173[] = {SGI119743, SGI119744, SGI119745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119749, SGI0, SGI0, SGI0, SGI119750, SGI119750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26174[] = {SGI119746, SGI119747, SGI119748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119751, SGI0, SGI0, SGI0, SGI119752, SGI119752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26175[] = {SGI119753, SGI119754, SGI119756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119759, SGI0, SGI0, SGI0, SGI119761, SGI119761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26176[] = {SGI119763, SGI119764, SGI119765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119766, SGI119766, SGI0, SGI0, SGI119767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26177[] = {SGI119768, SGI0, SGI119769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119770, SGI0, SGI0, SGI0, SGI119771, SGI119771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26178[] = {SGI119772, SGI119773, SGI119774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119775, SGI0, SGI0, SGI0, SGI119776, SGI119776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26179[] = {SGI119777, SGI0, SGI119778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26180[] = {SGI119779, SGI119780, SGI119781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119782, SGI0, SGI0, SGI0, SGI119783, SGI119783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26181[] = {SGI119784, SGI119785, SGI119786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119787, SGI0, SGI0, SGI0, SGI119788, SGI119788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26182[] = {SGI119789, SGI0, SGI119790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26183[] = {SGI119791, SGI119792, SGI119793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119794, SGI0, SGI0, SGI0, SGI119795, SGI119795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26184[] = {SGI119796, SGI119797, SGI119798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119799, SGI0, SGI0, SGI0, SGI119800, SGI119800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26185[] = {SGI119801, SGI119802, SGI119803, SGI119804, SGI119805, SGI119805, SGI0, NULL};
const Segment * const SG26186[] = {SGI119806, SGI119807, SGI119808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119809, SGI119809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26187[] = {SGI119810, SGI119811, SGI119812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119813, SGI0, SGI0, SGI0, SGI119814, SGI119814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26188[] = {SGI119815, SGI119816, SGI119817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119818, SGI0, SGI0, SGI0, SGI119819, SGI119819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26189[] = {SGI119820, SGI0, SGI119821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119822, SGI0, SGI0, SGI0, SGI119823, SGI119823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26190[] = {SGI119824, SGI119825, SGI119826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119827, SGI0, SGI0, SGI0, SGI119828, SGI119828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26191[] = {SGI119829, SGI119830, SGI119831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119832, SGI119832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26192[] = {SGI119833, SGI119834, SGI119835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119836, SGI0, SGI0, SGI0, SGI119837, SGI119837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26193[] = {SGI119838, SGI119840, SGI119842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119844, SGI0, SGI0, SGI0, SGI119846, SGI119846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26194[] = {SGI119839, SGI119841, SGI119843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119845, SGI0, SGI0, SGI0, SGI119847, SGI119848, SGI0, SGI0, SGI119849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26195[] = {SGI119850, SGI119851, SGI119852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119853, SGI0, SGI0, SGI0, SGI119854, SGI119854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26196[] = {SGI119855, SGI119856, SGI119857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119858, SGI0, SGI0, SGI0, SGI119859, SGI119859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26197[] = {SGI119860, SGI0, SGI119861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26198[] = {SGI119862, SGI119863, SGI119864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119865, SGI0, SGI0, SGI0, SGI119866, SGI119866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26199[] = {SGI119867, SGI119868, SGI119869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119870, SGI0, SGI0, SGI0, SGI119871, SGI119871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26200[] = {SGI119755, SGI119757, SGI119758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119760, SGI0, SGI0, SGI0, SGI119762, SGI119762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26201[] = {SGI119872, SGI119873, SGI119874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119875, SGI0, SGI0, SGI0, SGI119876, SGI119876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26202[] = {SGI119877, SGI119878, SGI119879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119880, SGI0, SGI0, SGI0, SGI119883, SGI119883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26203[] = {SGI119881, SGI119882, SGI119884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119885, SGI0, SGI0, SGI0, SGI119886, SGI119886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26204[] = {SGI119887, SGI119888, SGI119889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119890, SGI0, SGI0, SGI0, SGI119891, SGI119891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26205[] = {SGI119892, SGI119893, SGI119894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119895, SGI0, SGI0, SGI0, SGI119897, SGI119897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26206[] = {SGI119896, SGI0, SGI119898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119899, SGI0, SGI0, SGI0, SGI119900, SGI119900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26207[] = {SGI119901, SGI119902, SGI119903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119904, SGI0, SGI0, SGI0, SGI119905, SGI119905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26208[] = {SGI119906, SGI119907, SGI119908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119909, SGI0, SGI0, SGI0, SGI119911, SGI119911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26209[] = {SGI119910, SGI119912, SGI119913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119914, SGI0, SGI0, SGI0, SGI119915, SGI119915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26210[] = {SGI119916, SGI119917, SGI119918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119919, SGI0, SGI0, SGI0, SGI119920, SGI119920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26211[] = {SGI119921, SGI119922, SGI119923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119924, SGI0, SGI0, SGI0, SGI119925, SGI119925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26212[] = {SGI119926, SGI0, SGI119927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26213[] = {SGI119928, SGI119929, SGI119930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119931, SGI119931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26214[] = {SGI119932, SGI119933, SGI119934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119937, SGI119937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26215[] = {SGI119935, SGI119936, SGI119938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119939, SGI0, SGI0, SGI0, SGI119940, SGI119940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26216[] = {SGI119941, SGI0, SGI119942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119943, SGI0, SGI0, SGI0, SGI119944, SGI119944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26217[] = {SGI119945, SGI119946, SGI119947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119948, SGI0, SGI0, SGI0, SGI119949, SGI119949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26218[] = {SGI119950, SGI119951, SGI119952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119953, SGI0, SGI0, SGI0, SGI119954, SGI119954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26219[] = {SGI119955, SGI119956, SGI119957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119958, SGI0, SGI0, SGI0, SGI119959, SGI119959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26220[] = {SGI119960, SGI0, SGI119961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119962, SGI0, SGI0, SGI0, SGI119963, SGI119963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26221[] = {SGI119964, SGI0, SGI119965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26222[] = {SGI119966, SGI119967, SGI119968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119969, SGI0, SGI0, SGI0, SGI119970, SGI119970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26223[] = {SGI119971, SGI119972, SGI119973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119974, SGI0, SGI0, SGI0, SGI119975, SGI119975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26224[] = {SGI119976, SGI119977, SGI119978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119979, SGI0, SGI0, SGI0, SGI119980, SGI119980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26225[] = {SGI119981, SGI119982, SGI119983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119984, SGI0, SGI0, SGI0, SGI119985, SGI119985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26226[] = {SGI119986, SGI119987, SGI119988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119989, SGI0, SGI0, SGI0, SGI119990, SGI119990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26227[] = {SGI119991, SGI0, SGI119992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26228[] = {SGI119993, SGI119994, SGI119995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI119996, SGI0, SGI0, SGI0, SGI119997, SGI119997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26229[] = {SGI119998, SGI119999, SGI120000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120001, SGI0, SGI0, SGI0, SGI120002, SGI120002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26230[] = {SGI120003, SGI120004, SGI120005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120006, SGI0, SGI0, SGI0, SGI120007, SGI120007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26231[] = {SGI120008, SGI120009, SGI120010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120011, SGI0, SGI0, SGI0, SGI120012, SGI120012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26232[] = {SGI120013, SGI120014, SGI120015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120016, SGI0, SGI0, SGI0, SGI120018, SGI120018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26233[] = {SGI120017, SGI120019, SGI120020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120021, SGI0, SGI0, SGI0, SGI120022, SGI120022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26234[] = {SGI120023, SGI0, SGI120024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26235[] = {SGI120025, SGI120026, SGI120027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120029, SGI0, SGI0, SGI0, SGI120032, SGI120032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26236[] = {SGI120028, SGI120030, SGI120031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120033, SGI0, SGI0, SGI0, SGI120034, SGI120034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26237[] = {SGI120035, SGI120036, SGI120037, SGI120038, SGI120039, SGI120039, SGI0, NULL};
const Segment * const SG26238[] = {SGI120040, SGI120041, SGI120042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120043, SGI120043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26239[] = {SGI120044, SGI0, SGI120045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26240[] = {SGI120046, SGI0, SGI120047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120048, SGI0, SGI0, SGI0, SGI120049, SGI120049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26241[] = {SGI120050, SGI0, SGI120051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120052, SGI0, SGI0, SGI0, SGI120053, SGI120053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26242[] = {SGI120054, SGI120055, SGI120056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120057, SGI0, SGI0, SGI0, SGI120058, SGI120058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26243[] = {SGI120059, SGI0, SGI120060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120061, SGI0, SGI0, SGI0, SGI120062, SGI120062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26244[] = {SGI120063, SGI120064, SGI120065, SGI120066, SGI120067, SGI120067, SGI0, NULL};
const Segment * const SG26245[] = {SGI120068, SGI0, SGI120069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120070, SGI120071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26246[] = {SGI120072, SGI120073, SGI120074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120075, SGI0, SGI0, SGI0, SGI120076, SGI120076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26247[] = {SGI0, SGI120077, SGI0, SGI120078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26248[] = {SGI120079, SGI0, SGI120080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120081, SGI120082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26249[] = {SGI120083, SGI120084, SGI120085, SGI120086, SGI120087, SGI120087, SGI0, NULL};
const Segment * const SG26250[] = {SGI120088, SGI120089, SGI120090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120091, SGI0, SGI0, SGI0, SGI120092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120093, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26251[] = {SGI120094, SGI120095, SGI120096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120097, SGI0, SGI0, SGI0, SGI120098, SGI0, SGI0, SGI0, SGI0, SGI120099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26252[] = {SGI120100, SGI120101, SGI120102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120103, SGI0, SGI0, SGI0, SGI120104, SGI0, SGI0, SGI120105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26253[] = {SGI120106, SGI120107, SGI120108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120109, SGI0, SGI0, SGI0, SGI120110, SGI120110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120111, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26254[] = {SGI120112, SGI0, SGI120113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120114, SGI0, SGI0, SGI0, SGI120115, SGI120115, SGI0, SGI0, SGI120116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26255[] = {SGI120117, SGI120118, SGI120119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120120, SGI120120, SGI0, SGI0, SGI120121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26256[] = {SGI120122, SGI120123, SGI120124, SGI120125, SGI120126, SGI120126, SGI0, NULL};
const Segment * const SG26257[] = {SGI120127, SGI0, SGI120128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120129, SGI120130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26258[] = {SGI120131, SGI120132, SGI120133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120134, SGI0, SGI0, SGI0, SGI120135, SGI120135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26259[] = {SGI120136, SGI0, SGI120137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120138, SGI120139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120140, SGI0, NULL};
const Segment * const SG26260[] = {SGI120141, SGI0, SGI120142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120143, SGI0, SGI0, SGI0, SGI120144, SGI120144, SGI0, SGI0, SGI120145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26261[] = {SGI120146, SGI0, SGI120147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120148, SGI120149, SGI120149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26262[] = {SGI120150, SGI120151, SGI120152, SGI120153, SGI120154, SGI120154, SGI0, NULL};
const Segment * const SG26263[] = {SGI120155, SGI120156, SGI120157, SGI120158, SGI120159, SGI120159, SGI0, NULL};
const Segment * const SG26264[] = {SGI120160, SGI0, SGI120161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120162, SGI0, SGI0, SGI0, SGI120163, SGI120163, SGI0, SGI0, SGI120164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26265[] = {SGI120165, SGI0, SGI120166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120167, SGI120168, SGI120168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26266[] = {SGI120169, SGI0, SGI120170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120171, SGI120172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120173, SGI0, NULL};
const Segment * const SG26267[] = {SGI120174, SGI120175, SGI120176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120177, SGI0, SGI0, SGI0, SGI120178, SGI120178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120179, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26268[] = {SGI120180, SGI120181, SGI120182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120183, SGI0, SGI0, SGI0, SGI120184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26269[] = {SGI120185, SGI120186, SGI120187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120188, SGI0, SGI0, SGI0, SGI120189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26270[] = {SGI120190, SGI0, SGI120191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26271[] = {SGI120192, SGI120193, SGI120194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120195, SGI0, SGI0, SGI0, SGI120196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26272[] = {SGI120197, SGI120198, SGI120199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120200, SGI0, SGI0, SGI0, SGI120201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26273[] = {SGI120202, SGI0, SGI120203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120204, SGI0, SGI0, SGI0, SGI120205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26274[] = {SGI120206, SGI120207, SGI120208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120209, SGI0, SGI0, SGI0, SGI120210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26275[] = {SGI120211, SGI0, SGI120212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120216, SGI120217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26276[] = {SGI120213, SGI120214, SGI120215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120218, SGI0, SGI0, SGI0, SGI120219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26277[] = {SGI120220, SGI120221, SGI120222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120223, SGI0, SGI0, SGI0, SGI120224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26278[] = {SGI120225, SGI120226, SGI120227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120228, SGI0, SGI0, SGI0, SGI120229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26279[] = {SGI120230, SGI120231, SGI120232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120233, SGI0, SGI0, SGI0, SGI120234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26280[] = {SGI120235, SGI120236, SGI120237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120238, SGI120238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26281[] = {SGI120239, SGI120240, SGI120241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120242, SGI0, SGI0, SGI0, SGI120243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26282[] = {SGI120244, SGI120245, SGI120246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120247, SGI0, SGI0, SGI0, SGI120248, SGI120249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26283[] = {SGI120250, SGI120251, SGI120252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120253, SGI0, SGI0, SGI0, SGI120254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26284[] = {SGI120255, SGI0, SGI120256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120257, SGI0, SGI0, SGI0, SGI120258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26285[] = {SGI120259, SGI120260, SGI120261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120262, SGI0, SGI0, SGI0, SGI120263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26286[] = {SGI120264, SGI120265, SGI120266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120267, SGI0, SGI0, SGI0, SGI120268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26287[] = {SGI120269, SGI120270, SGI120271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120272, SGI0, SGI0, SGI0, SGI120273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26288[] = {SGI120274, SGI120275, SGI120276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26289[] = {SGI120277, SGI120278, SGI120279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120280, SGI0, SGI0, SGI0, SGI120281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26290[] = {SGI120282, SGI120283, SGI120284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120285, SGI120285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26291[] = {SGI120286, SGI120287, SGI120288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120289, SGI0, SGI0, SGI0, SGI120290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26292[] = {SGI120291, SGI120292, SGI120293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120294, SGI0, SGI0, SGI0, SGI120295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26293[] = {SGI120296, SGI120297, SGI120298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120299, SGI0, SGI0, SGI0, SGI120300, SGI120300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26294[] = {SGI120301, SGI120302, SGI120303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26295[] = {SGI120304, SGI120305, SGI120306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120307, SGI0, SGI0, SGI0, SGI120308, SGI120308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26296[] = {SGI120309, SGI120310, SGI120311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120312, SGI0, SGI0, SGI0, SGI120313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26297[] = {SGI120314, SGI120315, SGI120316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120317, SGI0, SGI0, SGI0, SGI120318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26298[] = {SGI120319, SGI120320, SGI120321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26299[] = {SGI120322, SGI120323, SGI120324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120325, SGI0, SGI0, SGI0, SGI120326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26300[] = {SGI120327, SGI120328, SGI120329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26301[] = {SGI120330, SGI120331, SGI120332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26302[] = {SGI120333, SGI120334, SGI120335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120336, SGI0, SGI0, SGI0, SGI120337, SGI120337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26303[] = {SGI120338, SGI120339, SGI120340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120341, SGI0, SGI120342, SGI120342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26304[] = {SGI120343, SGI0, SGI120344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120345, SGI0, SGI0, SGI0, SGI120348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26305[] = {SGI120346, SGI120347, SGI120349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120350, SGI0, SGI120351, SGI120351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26306[] = {SGI120352, SGI120353, SGI120354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26307[] = {SGI120355, SGI120356, SGI120357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26308[] = {SGI120358, SGI120359, SGI120360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26309[] = {SGI120361, SGI120362, SGI120363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120364, SGI0, SGI0, SGI0, SGI120365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26310[] = {SGI120366, SGI120367, SGI120368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120369, SGI0, SGI0, SGI0, SGI120370, SGI120370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26311[] = {SGI120371, SGI120372, SGI120373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120374, SGI0, SGI0, SGI0, SGI120375, SGI120376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26312[] = {SGI120377, SGI120378, SGI120379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26313[] = {SGI120381, SGI120382, SGI120383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26314[] = {SGI120385, SGI120386, SGI120388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120390, SGI0, SGI0, SGI0, SGI120391, SGI120391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26315[] = {SGI120387, SGI0, SGI120389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120392, SGI120393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26316[] = {SGI120394, SGI120395, SGI120396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26317[] = {SGI120397, SGI120398, SGI120399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120400, SGI0, SGI0, SGI0, SGI120401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26318[] = {SGI120402, SGI0, SGI120403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26319[] = {SGI120404, SGI0, SGI120405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120409, SGI120410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26320[] = {SGI120406, SGI120407, SGI120408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120411, SGI0, SGI0, SGI0, SGI120412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26321[] = {SGI120413, SGI120414, SGI120415, SGI120416, SGI120417, SGI120417, SGI0, NULL};
const Segment * const SG26322[] = {SGI120418, SGI0, SGI120419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120420, SGI0, SGI0, SGI0, SGI120421, SGI120421, SGI0, SGI0, SGI120422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26323[] = {SGI120423, SGI120424, SGI120425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120426, SGI0, SGI0, SGI0, SGI120427, SGI120427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120428, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26324[] = {SGI120429, SGI0, SGI120430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120431, SGI120432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26325[] = {SGI120433, SGI0, SGI120434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120435, SGI120436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26326[] = {SGI120437, SGI0, SGI120438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120439, SGI120440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26327[] = {SGI120441, SGI0, SGI120442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120443, SGI120444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26328[] = {SGI120445, SGI120446, SGI120447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120448, SGI0, SGI0, SGI0, SGI120449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120450, NULL};
const Segment * const SG26329[] = {SGI120451, SGI120452, SGI120453, SGI120454, SGI120455, SGI120455, SGI0, NULL};
const Segment * const SG26330[] = {SGI120458, SGI120459, SGI120460, SGI120461, SGI120463, SGI120463, SGI0, NULL};
const Segment * const SG26331[] = {SGI120456, SGI0, SGI120457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120462, SGI120464, SGI120464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26332[] = {SGI120465, SGI0, SGI120466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120467, SGI120468, SGI120468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26333[] = {SGI120469, SGI0, SGI120470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26334[] = {SGI120471, SGI120472, SGI120473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120474, SGI0, SGI0, SGI0, SGI120475, SGI120475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26335[] = {SGI120476, SGI120477, SGI120478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120479, SGI0, SGI0, SGI0, SGI120480, SGI120480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26336[] = {SGI120481, SGI120482, SGI120483, SGI120484, SGI120486, SGI120486, SGI0, NULL};
const Segment * const SG26337[] = {SGI120485, SGI120487, SGI120488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120489, SGI0, SGI0, SGI0, SGI120490, SGI120490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26338[] = {SGI120491, SGI120492, SGI120493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120494, SGI0, SGI0, SGI0, SGI120495, SGI120495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26339[] = {SGI120496, SGI120498, SGI120500, SGI120502, SGI120503, SGI0, SGI0, NULL};
const Segment * const SG26340[] = {SGI120497, SGI120499, SGI120501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120504, SGI0, SGI0, SGI0, SGI120505, SGI120505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26341[] = {SGI120506, SGI120507, SGI120508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120509, SGI0, SGI0, SGI0, SGI120510, SGI120510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26342[] = {SGI120511, SGI120512, SGI120513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120514, SGI0, SGI0, SGI0, SGI120515, SGI120515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26343[] = {SGI120516, SGI120517, SGI120518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120519, SGI0, SGI0, SGI0, SGI120520, SGI120520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26344[] = {SGI120521, SGI120522, SGI120523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120524, SGI0, SGI0, SGI0, SGI120525, SGI120525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26345[] = {SGI120526, SGI120527, SGI120528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120529, SGI0, SGI0, SGI0, SGI120530, SGI120530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26346[] = {SGI120531, SGI120532, SGI120533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120534, SGI0, SGI0, SGI0, SGI120535, SGI120535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26347[] = {SGI120536, SGI120537, SGI120538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120539, SGI0, SGI0, SGI0, SGI120540, SGI120540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26348[] = {SGI120541, SGI120542, SGI120543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120544, SGI0, SGI0, SGI0, SGI120545, SGI120545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26349[] = {SGI120546, SGI120547, SGI120548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120549, SGI0, SGI0, SGI0, SGI120550, SGI120550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26350[] = {SGI120551, SGI120552, SGI120553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120554, SGI120554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26351[] = {SGI120555, SGI120556, SGI120557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120558, SGI0, SGI0, SGI0, SGI120559, SGI120559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26352[] = {SGI120560, SGI0, SGI120561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26353[] = {SGI120562, SGI0, SGI120563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26354[] = {SGI120564, SGI0, SGI120565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26355[] = {SGI120566, SGI120567, SGI120568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120569, SGI0, SGI0, SGI0, SGI120570, SGI120570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26356[] = {SGI120571, SGI0, SGI120572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26357[] = {SGI120573, SGI120574, SGI120575, SGI120576, SGI120577, SGI120577, SGI0, NULL};
const Segment * const SG26358[] = {SGI120578, SGI120579, SGI120580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120581, SGI0, SGI0, SGI0, SGI120582, SGI120582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26359[] = {SGI120583, SGI0, SGI120584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26360[] = {SGI120585, SGI0, SGI120586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120587, SGI0, SGI0, SGI0, SGI120588, SGI120588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26361[] = {SGI120589, SGI120590, SGI120591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120592, SGI0, SGI0, SGI0, SGI120593, SGI120593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26362[] = {SGI120594, SGI0, SGI120595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26363[] = {SGI120596, SGI0, SGI120597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26364[] = {SGI120598, SGI120599, SGI120600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120601, SGI0, SGI0, SGI0, SGI120602, SGI120602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26365[] = {SGI120603, SGI120604, SGI120605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120606, SGI0, SGI0, SGI0, SGI120607, SGI120607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26366[] = {SGI120608, SGI120609, SGI120610, SGI120611, SGI120612, SGI120612, SGI0, NULL};
const Segment * const SG26367[] = {SGI120613, SGI0, SGI120614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26368[] = {SGI120615, SGI0, SGI120616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26369[] = {SGI120617, SGI120618, SGI120619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120620, SGI0, SGI0, SGI0, SGI120621, SGI120621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26370[] = {SGI120622, SGI120623, SGI120624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120625, SGI0, SGI0, SGI0, SGI120626, SGI120626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26371[] = {SGI120627, SGI120628, SGI120629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120630, SGI120630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26372[] = {SGI120631, SGI120632, SGI120633, SGI120634, SGI120635, SGI120635, SGI0, NULL};
const Segment * const SG26373[] = {SGI120636, SGI0, SGI120637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26374[] = {SGI120638, SGI120639, SGI120640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120641, SGI0, SGI0, SGI0, SGI120642, SGI120642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26375[] = {SGI120643, SGI120644, SGI120645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120646, SGI0, SGI0, SGI0, SGI120647, SGI120647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26376[] = {SGI120648, SGI0, SGI120649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26377[] = {SGI120650, SGI120651, SGI120652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120653, SGI120653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26378[] = {SGI120654, SGI0, SGI120655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26379[] = {SGI120656, SGI0, SGI120657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26380[] = {SGI120658, SGI120659, SGI120660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120661, SGI0, SGI0, SGI0, SGI120662, SGI120662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26381[] = {SGI120663, SGI120664, SGI120665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120666, SGI0, SGI0, SGI0, SGI120667, SGI120667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26382[] = {SGI0, SGI120668, SGI120669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26383[] = {SGI120670, SGI0, SGI120671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120672, SGI0, SGI0, SGI0, SGI120673, SGI120673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26384[] = {SGI120674, SGI120675, SGI120676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120677, SGI0, SGI0, SGI0, SGI120678, SGI120678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26385[] = {SGI120679, SGI120680, SGI120681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120682, SGI0, SGI0, SGI0, SGI120683, SGI120683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26386[] = {SGI120684, SGI120685, SGI120686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120687, SGI0, SGI0, SGI0, SGI120688, SGI120688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26387[] = {SGI120689, SGI120690, SGI120691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120692, SGI0, SGI0, SGI0, SGI120693, SGI120693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26388[] = {SGI120694, SGI120695, SGI120696, SGI120697, SGI120698, SGI120698, SGI0, NULL};
const Segment * const SG26389[] = {SGI120699, SGI120700, SGI120701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120702, SGI0, SGI0, SGI0, SGI120703, SGI120703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26390[] = {SGI120704, SGI120705, SGI120706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120707, SGI0, SGI0, SGI0, SGI120708, SGI120708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26391[] = {SGI120709, SGI120710, SGI120711, SGI120712, SGI120713, SGI120713, SGI0, NULL};
const Segment * const SG26392[] = {SGI120714, SGI120715, SGI120716, SGI120717, SGI120718, SGI120718, SGI0, NULL};
const Segment * const SG26393[] = {SGI120719, SGI120720, SGI120721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120722, SGI0, SGI0, SGI0, SGI120723, SGI120723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120724, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26394[] = {SGI120725, SGI0, SGI120726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120727, SGI120728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26395[] = {SGI120729, SGI0, SGI120730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120731, SGI120732, SGI120732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26396[] = {SGI120733, SGI120734, SGI120735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120736, SGI0, SGI0, SGI0, SGI120737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26397[] = {SGI120738, SGI0, SGI120739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120740, SGI120741, SGI120741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26398[] = {SGI0, SGI120742, SGI120743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120744, SGI0, SGI0, SGI0, SGI0, SGI120745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120746, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26399[] = {SGI120747, SGI0, SGI120748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120749, SGI0, SGI0, SGI0, SGI120750, SGI120750, SGI0, SGI0, SGI120751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26400[] = {SGI120752, SGI120753, SGI120754, SGI120755, SGI120756, SGI120756, SGI0, NULL};
const Segment * const SG26401[] = {SGI120757, SGI120758, SGI120759, SGI120760, SGI120761, SGI120761, SGI0, NULL};
const Segment * const SG26402[] = {SGI120762, SGI120763, SGI120764, SGI120765, SGI120766, SGI120766, SGI0, NULL};
const Segment * const SG26403[] = {SGI120767, SGI120768, SGI120769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120770, SGI0, SGI0, SGI0, SGI120771, SGI0, SGI0, SGI0, SGI0, SGI120772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26404[] = {SGI120773, SGI120774, SGI120775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120776, SGI0, SGI0, SGI0, SGI120777, SGI120777, SGI0, SGI120778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26405[] = {SGI120779, SGI120780, SGI120781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120782, SGI0, SGI0, SGI0, SGI120783, SGI120783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120784, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26406[] = {SGI120785, SGI0, SGI120786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120787, SGI0, SGI0, SGI0, SGI120788, SGI120788, SGI0, SGI0, SGI120789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26407[] = {SGI120790, SGI0, SGI120791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120792, SGI120793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120794, SGI0, NULL};
const Segment * const SG26408[] = {SGI120795, SGI120796, SGI120797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120798, SGI120798, SGI0, SGI0, SGI120799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26409[] = {SGI120800, SGI120801, SGI120802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120803, SGI0, SGI0, SGI0, SGI120804, SGI0, SGI0, SGI120805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26410[] = {SGI120806, SGI0, SGI120807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120808, SGI120809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120810, SGI0, NULL};
const Segment * const SG26411[] = {SGI120811, SGI0, SGI120812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120813, SGI120814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26412[] = {SGI120815, SGI120816, SGI120817, SGI120818, SGI120819, SGI120819, SGI0, NULL};
const Segment * const SG26413[] = {SGI120820, SGI120821, SGI120822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120823, SGI0, SGI120824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26414[] = {SGI120825, SGI120826, SGI120827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120828, SGI0, SGI120829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26415[] = {SGI120830, SGI120831, SGI120832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120833, SGI0, SGI120834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26416[] = {SGI120835, SGI120836, SGI120837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120838, SGI0, SGI0, SGI0, SGI120839, SGI120839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26417[] = {SGI120840, SGI120841, SGI120842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120843, SGI0, SGI0, SGI0, SGI120844, SGI120844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26418[] = {SGI120845, SGI120846, SGI120847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120848, SGI0, SGI0, SGI0, SGI120849, SGI120849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26419[] = {SGI120850, SGI120851, SGI120852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120853, SGI0, SGI0, SGI0, SGI120854, SGI120854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26420[] = {SGI120855, SGI120856, SGI120857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120858, SGI0, SGI0, SGI0, SGI120859, SGI120859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26421[] = {SGI120860, SGI120861, SGI120862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120863, SGI0, SGI0, SGI0, SGI120864, SGI120864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26422[] = {SGI120865, SGI120866, SGI120867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120868, SGI0, SGI0, SGI0, SGI120869, SGI120869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26423[] = {SGI120870, SGI120871, SGI120872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120873, SGI0, SGI0, SGI0, SGI120874, SGI120874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26424[] = {SGI120875, SGI120877, SGI120878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120880, SGI0, SGI0, SGI0, SGI120882, SGI120882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26425[] = {SGI120876, SGI0, SGI120879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120881, SGI120883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26426[] = {SGI120884, SGI120885, SGI120886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120887, SGI0, SGI0, SGI0, SGI120888, SGI120888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26427[] = {SGI120889, SGI120890, SGI120891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120892, SGI0, SGI0, SGI0, SGI120893, SGI120893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26428[] = {SGI120894, SGI120895, SGI120896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120897, SGI0, SGI0, SGI0, SGI120898, SGI120898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26429[] = {SGI120899, SGI120900, SGI120901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120905, SGI0, SGI0, SGI0, SGI120906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26430[] = {SGI120902, SGI120903, SGI120904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120907, SGI0, SGI0, SGI0, SGI120908, SGI120908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26431[] = {SGI120909, SGI120910, SGI120911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120912, SGI0, SGI0, SGI0, SGI120913, SGI120913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26432[] = {SGI120914, SGI120915, SGI120916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120917, SGI0, SGI0, SGI0, SGI120918, SGI120918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26433[] = {SGI120919, SGI0, SGI120920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26434[] = {SGI120921, SGI0, SGI120922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26435[] = {SGI120923, SGI0, SGI120924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120925, SGI120926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26436[] = {SGI120927, SGI120928, SGI120929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120930, SGI0, SGI0, SGI0, SGI120931, SGI120931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26437[] = {SGI120932, SGI120933, SGI120934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120935, SGI0, SGI0, SGI0, SGI120936, SGI120936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26438[] = {SGI120937, SGI120938, SGI120939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120940, SGI0, SGI0, SGI0, SGI120941, SGI120941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26439[] = {SGI120942, SGI120943, SGI120944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120945, SGI0, SGI0, SGI0, SGI120946, SGI120946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26440[] = {SGI120947, SGI120948, SGI120949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120950, SGI0, SGI0, SGI0, SGI120951, SGI120951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26441[] = {SGI120952, SGI0, SGI120953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26442[] = {SGI120954, SGI120955, SGI120956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120957, SGI120957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26443[] = {SGI120958, SGI0, SGI120959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120960, SGI0, SGI0, SGI0, SGI120961, SGI120961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26444[] = {SGI120962, SGI0, SGI120963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26445[] = {SGI120964, SGI0, SGI120965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120966, SGI120967, SGI120967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26446[] = {SGI120968, SGI0, SGI120969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26447[] = {SGI120970, SGI120971, SGI120972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120973, SGI0, SGI0, SGI0, SGI120974, SGI120974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26448[] = {SGI120975, SGI120976, SGI120977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120978, SGI0, SGI0, SGI0, SGI120979, SGI120979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26449[] = {SGI120980, SGI120981, SGI120982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120983, SGI0, SGI0, SGI0, SGI120984, SGI120984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26450[] = {SGI120985, SGI120986, SGI120987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120988, SGI0, SGI0, SGI0, SGI120989, SGI120989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26451[] = {SGI120990, SGI0, SGI120991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120992, SGI0, SGI0, SGI0, SGI120993, SGI120993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26452[] = {SGI120994, SGI120995, SGI120996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI120997, SGI0, SGI0, SGI0, SGI120998, SGI120998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26453[] = {SGI120999, SGI0, SGI121000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26454[] = {SGI121001, SGI121002, SGI121003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121004, SGI0, SGI0, SGI0, SGI121005, SGI121005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26455[] = {SGI121006, SGI121007, SGI121008, SGI121009, SGI121010, SGI121010, SGI0, NULL};
const Segment * const SG26456[] = {SGI121011, SGI121012, SGI121013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121014, SGI0, SGI0, SGI0, SGI121015, SGI121015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26457[] = {SGI121016, SGI0, SGI121017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121018, SGI0, SGI0, SGI0, SGI121019, SGI121019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26458[] = {SGI121020, SGI0, SGI121021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26459[] = {SGI121022, SGI121023, SGI121024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121025, SGI0, SGI0, SGI0, SGI121026, SGI121026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26460[] = {SGI121027, SGI0, SGI121028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26461[] = {SGI121029, SGI0, SGI121030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26462[] = {SGI121031, SGI121032, SGI121033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121034, SGI0, SGI0, SGI0, SGI121035, SGI121035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26463[] = {SGI121036, SGI121037, SGI121038, SGI121039, SGI121040, SGI121040, SGI0, NULL};
const Segment * const SG26464[] = {SGI121041, SGI121042, SGI121043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121044, SGI0, SGI0, SGI0, SGI121046, SGI121046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26465[] = {SGI121045, SGI0, SGI121047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121048, SGI0, SGI0, SGI0, SGI121049, SGI121049, SGI0, SGI0, SGI121050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26466[] = {SGI121051, SGI121052, SGI121053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121054, SGI0, SGI0, SGI0, SGI121055, SGI121055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26467[] = {SGI121056, SGI121057, SGI121058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121059, SGI0, SGI0, SGI0, SGI121060, SGI121060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26468[] = {SGI121061, SGI121062, SGI121063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121064, SGI0, SGI0, SGI0, SGI121065, SGI121065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26469[] = {SGI121066, SGI0, SGI121067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26470[] = {SGI121068, SGI121069, SGI121070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121071, SGI0, SGI0, SGI0, SGI121072, SGI121072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26471[] = {SGI121076, SGI121077, SGI121079, SGI121080, SGI121082, SGI0, SGI0, NULL};
const Segment * const SG26472[] = {SGI121073, SGI121074, SGI121075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121078, SGI0, SGI0, SGI0, SGI121081, SGI121081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26473[] = {SGI121083, SGI121084, SGI121085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121086, SGI0, SGI0, SGI0, SGI121087, SGI121087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26474[] = {SGI121088, SGI121089, SGI121090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121093, SGI0, SGI0, SGI0, SGI121095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26475[] = {SGI121091, SGI121092, SGI121094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121096, SGI0, SGI0, SGI0, SGI121097, SGI121097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26476[] = {SGI121098, SGI121099, SGI121100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121101, SGI0, SGI0, SGI0, SGI121102, SGI121102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26477[] = {SGI121103, SGI121104, SGI121105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121106, SGI0, SGI0, SGI0, SGI121107, SGI121107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26478[] = {SGI121108, SGI0, SGI121109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26479[] = {SGI121110, SGI121111, SGI121112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121113, SGI0, SGI0, SGI0, SGI121114, SGI121114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26480[] = {SGI121115, SGI0, SGI121116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121117, SGI121118, SGI121118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26481[] = {SGI121119, SGI0, SGI121120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26482[] = {SGI121121, SGI121122, SGI121123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121124, SGI121124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26483[] = {SGI121125, SGI121126, SGI121127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121128, SGI0, SGI0, SGI0, SGI121129, SGI121129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26484[] = {SGI121130, SGI0, SGI121131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121132, SGI121133, SGI121133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26485[] = {SGI121134, SGI0, SGI121135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26486[] = {SGI121136, SGI121137, SGI121138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121139, SGI0, SGI0, SGI0, SGI121140, SGI121140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26487[] = {SGI121141, SGI121142, SGI121143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121144, SGI0, SGI0, SGI0, SGI121145, SGI121145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26488[] = {SGI121146, SGI0, SGI121147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26489[] = {SGI121148, SGI121149, SGI121150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121151, SGI0, SGI0, SGI0, SGI121152, SGI121152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26490[] = {SGI121153, SGI121154, SGI121155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121156, SGI0, SGI0, SGI0, SGI121157, SGI121157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26491[] = {SGI121158, SGI121159, SGI121160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121161, SGI0, SGI0, SGI0, SGI121162, SGI121162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26492[] = {SGI121163, SGI121164, SGI121165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121166, SGI0, SGI0, SGI0, SGI121167, SGI121167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26493[] = {SGI121168, SGI121170, SGI121172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121175, SGI121175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26494[] = {SGI121169, SGI121171, SGI121173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121174, SGI0, SGI0, SGI0, SGI121176, SGI121176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26495[] = {SGI121177, SGI0, SGI121178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26496[] = {SGI121179, SGI121180, SGI121181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121182, SGI0, SGI0, SGI0, SGI121183, SGI121183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26497[] = {SGI121184, SGI121185, SGI121186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121187, SGI0, SGI0, SGI0, SGI121188, SGI121188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26498[] = {SGI121189, SGI121190, SGI121191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121192, SGI121192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26499[] = {SGI121193, SGI121194, SGI121195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121196, SGI0, SGI0, SGI0, SGI121197, SGI121197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26500[] = {SGI121198, SGI121199, SGI121200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121201, SGI0, SGI0, SGI0, SGI121202, SGI121202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26501[] = {SGI121203, SGI121204, SGI121205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121208, SGI121208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26502[] = {SGI121206, SGI121207, SGI121209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121210, SGI0, SGI0, SGI0, SGI121212, SGI121212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26503[] = {SGI121214, SGI121215, SGI121216, SGI121217, SGI121218, SGI121218, SGI0, NULL};
const Segment * const SG26504[] = {SGI121211, SGI0, SGI121213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121219, SGI121220, SGI121220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121221, SGI0, NULL};
const Segment * const SG26505[] = {SGI121222, SGI0, SGI121223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26506[] = {SGI121224, SGI0, SGI121225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26507[] = {SGI121226, SGI0, SGI121227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26508[] = {SGI121228, SGI0, SGI121229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26509[] = {SGI121230, SGI121231, SGI121232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121235, SGI0, SGI0, SGI0, SGI121238, SGI121238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26510[] = {SGI121236, SGI0, SGI121237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26511[] = {SGI121233, SGI0, SGI121234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121239, SGI121240, SGI121240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26512[] = {SGI121241, SGI0, SGI121242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26513[] = {SGI121243, SGI0, SGI121244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121245, SGI0, SGI0, SGI0, SGI121246, SGI121246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26514[] = {SGI121247, SGI121248, SGI121250, SGI121251, SGI121253, SGI121253, SGI0, NULL};
const Segment * const SG26515[] = {SGI121249, SGI121252, SGI121254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121255, SGI0, SGI0, SGI0, SGI121256, SGI121256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26516[] = {SGI121257, SGI0, SGI121258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121259, SGI121260, SGI121260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26517[] = {SGI121261, SGI121262, SGI121263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121264, SGI0, SGI0, SGI0, SGI121265, SGI121265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26518[] = {SGI121266, SGI121267, SGI121268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121269, SGI0, SGI0, SGI0, SGI121270, SGI121270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26519[] = {SGI121271, SGI121272, SGI121273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121276, SGI0, SGI0, SGI0, SGI121280, SGI121280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121281, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26520[] = {SGI121275, SGI0, SGI121279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121282, SGI0, SGI0, SGI0, SGI121284, SGI121284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26521[] = {SGI121274, SGI121277, SGI121278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121283, SGI0, SGI0, SGI0, SGI121285, SGI121285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26522[] = {SGI121286, SGI121287, SGI121288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121289, SGI0, SGI0, SGI0, SGI121290, SGI121290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26523[] = {SGI121291, SGI0, SGI121292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26524[] = {SGI121293, SGI121294, SGI121295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121296, SGI0, SGI0, SGI0, SGI121297, SGI121297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26525[] = {SGI121298, SGI121299, SGI121300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121304, SGI0, SGI0, SGI0, SGI121305, SGI121305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121307, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26526[] = {SGI121301, SGI121302, SGI121303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121306, SGI0, SGI0, SGI0, SGI121308, SGI121308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26527[] = {SGI121309, SGI121310, SGI121311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121312, SGI0, SGI0, SGI0, SGI121313, SGI121313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26528[] = {SGI121314, SGI121315, SGI121316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121317, SGI0, SGI0, SGI0, SGI121318, SGI121318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26529[] = {SGI121319, SGI0, SGI121320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26530[] = {SGI121321, SGI121322, SGI121323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121324, SGI0, SGI0, SGI0, SGI121325, SGI121325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121328, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26531[] = {SGI121326, SGI0, SGI121327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26532[] = {SGI121329, SGI121330, SGI121331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121332, SGI0, SGI0, SGI0, SGI121333, SGI121333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26533[] = {SGI121334, SGI0, SGI121335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26534[] = {SGI121336, SGI0, SGI121337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121338, SGI0, SGI0, SGI0, SGI121339, SGI121339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26535[] = {SGI121340, SGI121341, SGI121342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121343, SGI0, SGI0, SGI0, SGI121344, SGI121344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26536[] = {SGI121345, SGI0, SGI121346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121347, SGI0, SGI0, SGI0, SGI121348, SGI121348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26537[] = {SGI121349, SGI0, SGI121350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121351, SGI0, SGI0, SGI0, SGI121352, SGI121352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26538[] = {SGI121353, SGI121354, SGI121355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121356, SGI0, SGI0, SGI0, SGI121357, SGI121357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26539[] = {SGI121358, SGI0, SGI121359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121363, SGI121364, SGI121364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26540[] = {SGI121360, SGI121361, SGI121362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121365, SGI0, SGI0, SGI0, SGI121366, SGI121366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26541[] = {SGI121367, SGI121368, SGI121369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121370, SGI0, SGI0, SGI0, SGI121371, SGI121371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26542[] = {SGI121372, SGI121373, SGI121374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121375, SGI0, SGI0, SGI0, SGI121376, SGI121376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26543[] = {SGI121377, SGI121378, SGI121379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121380, SGI121380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26544[] = {SGI121386, SGI121387, SGI121388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121389, SGI121389, SGI0, SGI0, SGI121390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26545[] = {SGI121391, SGI0, SGI121392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121393, SGI121394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121395, SGI0, NULL};
const Segment * const SG26546[] = {SGI121381, SGI121382, SGI121383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121384, SGI0, SGI0, SGI0, SGI121385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26547[] = {SGI121396, SGI121397, SGI121398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121399, SGI0, SGI0, SGI0, SGI121400, SGI121400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26548[] = {SGI121401, SGI121402, SGI121403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121404, SGI0, SGI0, SGI0, SGI121405, SGI121405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26549[] = {SGI121406, SGI121407, SGI121408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121409, SGI0, SGI0, SGI0, SGI121410, SGI121410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26550[] = {SGI121411, SGI121412, SGI121413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121414, SGI121414, SGI0, SGI0, SGI121415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26551[] = {SGI121416, SGI121417, SGI121418, SGI121419, SGI121420, SGI121420, SGI0, NULL};
const Segment * const SG26552[] = {SGI121421, SGI121422, SGI121423, SGI121424, SGI121425, SGI121425, SGI0, NULL};
const Segment * const SG26553[] = {SGI121428, SGI121429, SGI121430, SGI121431, SGI121433, SGI121433, SGI0, NULL};
const Segment * const SG26554[] = {SGI121426, SGI0, SGI121427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121432, SGI0, SGI0, SGI0, SGI121434, SGI121434, SGI0, SGI0, SGI121435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26555[] = {SGI121436, SGI121437, SGI121438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121439, SGI0, SGI0, SGI0, SGI121440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26556[] = {SGI121441, SGI121442, SGI121443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121444, SGI0, SGI0, SGI0, SGI121445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26557[] = {SGI121446, SGI121447, SGI121448, SGI121449, SGI121450, SGI121450, SGI0, NULL};
const Segment * const SG26558[] = {SGI121451, SGI0, SGI121452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121453, SGI0, SGI0, SGI0, SGI121454, SGI121454, SGI0, SGI0, SGI121455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26559[] = {SGI121456, SGI121457, SGI121458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121459, SGI0, SGI0, SGI0, SGI121460, SGI121460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26560[] = {SGI121461, SGI121462, SGI121463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121464, SGI0, SGI0, SGI0, SGI121465, SGI121465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26561[] = {SGI121471, SGI0, SGI121472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121473, SGI0, SGI0, SGI0, SGI121474, SGI121474, SGI0, SGI0, SGI121475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26562[] = {SGI121476, SGI121477, SGI121478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121479, SGI0, SGI0, SGI0, SGI121480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121481, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26563[] = {SGI121482, SGI121483, SGI121484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121485, SGI0, SGI0, SGI0, SGI121486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26564[] = {SGI121487, SGI121488, SGI121489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121490, SGI0, SGI0, SGI0, SGI121491, SGI121491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121492, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26565[] = {SGI121493, SGI0, SGI121494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121495, SGI0, SGI0, SGI0, SGI121496, SGI121496, SGI0, SGI0, SGI121497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26566[] = {SGI121498, SGI0, SGI121499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121500, SGI121501, SGI121501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26567[] = {SGI121502, SGI121503, SGI121504, SGI121505, SGI121506, SGI121506, SGI0, NULL};
const Segment * const SG26568[] = {SGI121507, SGI121508, SGI121509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121510, SGI0, SGI0, SGI0, SGI121511, SGI121511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26569[] = {SGI121466, SGI121467, SGI121468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121469, SGI0, SGI0, SGI0, SGI121470, SGI121470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26570[] = {SGI121512, SGI0, SGI121513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121514, SGI121515, SGI121515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121516, SGI0, NULL};
const Segment * const SG26571[] = {SGI121517, SGI0, SGI121518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26572[] = {SGI121519, SGI0, SGI121520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121521, SGI121522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26573[] = {SGI121523, SGI0, SGI121524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121525, SGI121526, SGI121526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26574[] = {SGI121527, SGI121528, SGI121529, SGI121530, SGI121531, SGI121531, SGI0, NULL};
const Segment * const SG26575[] = {SGI121532, SGI121533, SGI121534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121535, SGI0, SGI0, SGI0, SGI121536, SGI121536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121537, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26576[] = {SGI121538, SGI121539, SGI121540, SGI121541, SGI121542, SGI121542, SGI0, NULL};
const Segment * const SG26577[] = {SGI121543, SGI121544, SGI121545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121546, SGI121546, SGI0, SGI0, SGI121547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26578[] = {SGI121548, SGI0, SGI121549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121550, SGI121551, SGI121551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26579[] = {SGI121552, SGI121553, SGI121554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121555, SGI121555, SGI0, SGI0, SGI121556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26580[] = {SGI0, SGI0, SGI121557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121558, SGI121559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26581[] = {SGI121560, SGI0, SGI121561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121562, SGI121563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26582[] = {SGI121564, SGI0, SGI121565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121566, SGI121567, SGI121567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26583[] = {SGI121568, SGI121569, SGI121570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121571, SGI0, SGI121572, SGI121572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26584[] = {SGI121573, SGI121574, SGI121575, SGI121576, SGI121577, SGI121577, SGI0, NULL};
const Segment * const SG26585[] = {SGI121578, SGI121579, SGI121580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121581, SGI0, SGI121582, SGI121582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26586[] = {SGI121583, SGI0, SGI121584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121585, SGI121586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26587[] = {SGI121587, SGI0, SGI121588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121589, SGI0, SGI0, SGI0, SGI121590, SGI121590, SGI0, SGI0, SGI121591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26588[] = {SGI121592, SGI121593, SGI121594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121595, SGI0, SGI121596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26589[] = {SGI121597, SGI121598, SGI121599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121600, SGI0, SGI0, SGI0, SGI121601, SGI0, SGI0, SGI0, SGI0, SGI121602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26590[] = {SGI121603, SGI121604, SGI121605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121606, SGI0, SGI0, SGI0, SGI121607, SGI121607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121608, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26591[] = {SGI121609, SGI121610, SGI121611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121612, SGI0, SGI0, SGI0, SGI121613, SGI121613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121614, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26592[] = {SGI121615, SGI0, SGI121616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121617, SGI121618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26593[] = {SGI121619, SGI0, SGI121620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121621, SGI121622, SGI121622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26594[] = {SGI121623, SGI0, SGI121624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121625, SGI0, SGI0, SGI0, SGI121626, SGI121626, SGI0, SGI0, SGI121627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26595[] = {SGI121628, SGI0, SGI121629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121630, SGI121631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26596[] = {SGI121632, SGI121633, SGI121634, SGI121635, SGI121636, SGI121636, SGI0, NULL};
const Segment * const SG26597[] = {SGI121637, SGI0, SGI121638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121639, SGI121640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26598[] = {SGI121641, SGI121642, SGI121643, SGI121644, SGI121645, SGI121645, SGI0, NULL};
const Segment * const SG26599[] = {SGI121646, SGI0, SGI121647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121648, SGI121649, SGI121649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26600[] = {SGI121650, SGI121651, SGI121652, SGI121653, SGI121654, SGI121654, SGI0, NULL};
const Segment * const SG26601[] = {SGI121655, SGI121656, SGI121657, SGI121658, SGI121659, SGI121659, SGI0, NULL};
const Segment * const SG26602[] = {SGI121660, SGI121661, SGI121662, SGI121663, SGI121664, SGI121664, SGI0, NULL};
const Segment * const SG26603[] = {SGI121665, SGI121666, SGI121667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121668, SGI0, SGI0, SGI0, SGI121669, SGI121669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121670, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26604[] = {SGI121671, SGI0, SGI121672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121673, SGI121674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26605[] = {SGI121675, SGI121676, SGI121677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121678, SGI121678, SGI0, SGI0, SGI121679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26606[] = {SGI121680, SGI121681, SGI121682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121683, SGI121683, SGI0, SGI0, SGI121684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26607[] = {SGI121685, SGI121686, SGI121687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121688, SGI0, SGI0, SGI0, SGI121689, SGI121689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26608[] = {SGI121690, SGI121691, SGI121692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121693, SGI0, SGI0, SGI0, SGI121694, SGI121694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26609[] = {SGI121695, SGI121696, SGI121697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121698, SGI0, SGI0, SGI0, SGI121699, SGI121699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26610[] = {SGI121700, SGI121701, SGI121702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121703, SGI0, SGI0, SGI0, SGI121704, SGI121704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26611[] = {SGI121708, SGI0, SGI121709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26612[] = {SGI121705, SGI121706, SGI121707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121710, SGI0, SGI0, SGI0, SGI121711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26613[] = {SGI121712, SGI121713, SGI121714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121715, SGI0, SGI0, SGI0, SGI121716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26614[] = {SGI121717, SGI121718, SGI121719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121720, SGI0, SGI0, SGI0, SGI121721, SGI121721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26615[] = {SGI121722, SGI121723, SGI121724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121725, SGI0, SGI0, SGI0, SGI121726, SGI121726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26616[] = {SGI121727, SGI121728, SGI121729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121730, SGI0, SGI0, SGI0, SGI121731, SGI121731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26617[] = {SGI121732, SGI121733, SGI121735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121738, SGI0, SGI0, SGI0, SGI121739, SGI121739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26618[] = {SGI121734, SGI121736, SGI121737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121740, SGI0, SGI0, SGI0, SGI121741, SGI121741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26619[] = {SGI121742, SGI121744, SGI121745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121748, SGI0, SGI0, SGI0, SGI121750, SGI121750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26620[] = {SGI121743, SGI121746, SGI121747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121749, SGI0, SGI0, SGI0, SGI121751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26621[] = {SGI121752, SGI121753, SGI121754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121755, SGI0, SGI0, SGI0, SGI121756, SGI121756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26622[] = {SGI121757, SGI121758, SGI121759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121760, SGI121760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26623[] = {SGI121761, SGI0, SGI121762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26624[] = {SGI121763, SGI121764, SGI121765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121766, SGI0, SGI0, SGI0, SGI121767, SGI121767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26625[] = {SGI121768, SGI121769, SGI121770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121771, SGI0, SGI0, SGI0, SGI121772, SGI121772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26626[] = {SGI121774, SGI121776, SGI121778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121779, SGI0, SGI0, SGI0, SGI121781, SGI121781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26627[] = {SGI121773, SGI121775, SGI121777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121780, SGI121780, SGI0, SGI0, SGI121782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26628[] = {SGI121783, SGI121784, SGI121785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121786, SGI0, SGI0, SGI0, SGI121787, SGI121787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26629[] = {SGI121788, SGI121789, SGI121790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121791, SGI0, SGI0, SGI0, SGI121792, SGI121792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26630[] = {SGI121793, SGI0, SGI121794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121795, SGI121796, SGI121796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26631[] = {SGI121797, SGI121798, SGI121799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121800, SGI0, SGI0, SGI0, SGI121801, SGI121801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26632[] = {SGI121802, SGI121803, SGI121804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121805, SGI0, SGI0, SGI0, SGI121806, SGI121806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26633[] = {SGI121807, SGI121808, SGI121809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121810, SGI0, SGI0, SGI0, SGI121811, SGI121811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26634[] = {SGI121812, SGI121813, SGI121814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121815, SGI0, SGI0, SGI0, SGI121816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26635[] = {SGI121817, SGI0, SGI121818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121819, SGI121820, SGI121820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121821, SGI0, NULL};
const Segment * const SG26636[] = {SGI121822, SGI121823, SGI121824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121825, SGI0, SGI0, SGI0, SGI121826, SGI121826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26637[] = {SGI121827, SGI0, SGI121828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121829, SGI121830, SGI121830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26638[] = {SGI121831, SGI121832, SGI121834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121836, SGI0, SGI0, SGI0, SGI121837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26639[] = {SGI121833, SGI0, SGI121835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121838, SGI121839, SGI121839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26640[] = {SGI121840, SGI121841, SGI121842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121843, SGI0, SGI0, SGI0, SGI121844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26641[] = {SGI121845, SGI121846, SGI121847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121848, SGI0, SGI0, SGI0, SGI121849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26642[] = {SGI121850, SGI121851, SGI121852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121853, SGI0, SGI0, SGI0, SGI121854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26643[] = {SGI121855, SGI121856, SGI121857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121858, SGI0, SGI0, SGI0, SGI121859, SGI121859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26644[] = {SGI121860, SGI121861, SGI121862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121863, SGI0, SGI0, SGI0, SGI121864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26645[] = {SGI121865, SGI121866, SGI121867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121868, SGI0, SGI0, SGI0, SGI121869, SGI121869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26646[] = {SGI121870, SGI121871, SGI121872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121873, SGI0, SGI0, SGI0, SGI121874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26647[] = {SGI121875, SGI121876, SGI121877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121878, SGI0, SGI0, SGI0, SGI121879, SGI121879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26648[] = {SGI121880, SGI121881, SGI121882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121883, SGI0, SGI0, SGI0, SGI121884, SGI121884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26649[] = {SGI121885, SGI121886, SGI121887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121888, SGI0, SGI0, SGI0, SGI121889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26650[] = {SGI0, SGI0, SGI121890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121891, SGI0, SGI0, SGI0, SGI121892, SGI121892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26651[] = {SGI121893, SGI121894, SGI121895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121896, SGI0, SGI0, SGI0, SGI121897, SGI121897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26652[] = {SGI121898, SGI121899, SGI121900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121901, SGI0, SGI0, SGI0, SGI121902, SGI121902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26653[] = {SGI121905, SGI0, SGI121907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26654[] = {SGI121903, SGI121904, SGI121906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121908, SGI0, SGI0, SGI0, SGI121909, SGI121909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26655[] = {SGI121910, SGI0, SGI121911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26656[] = {SGI121912, SGI0, SGI121913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26657[] = {SGI121914, SGI0, SGI121915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26658[] = {SGI121916, SGI121917, SGI121918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121919, SGI0, SGI0, SGI0, SGI121920, SGI121920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26659[] = {SGI121921, SGI121922, SGI121923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121927, SGI0, SGI0, SGI0, SGI121928, SGI121928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26660[] = {SGI121924, SGI121925, SGI121926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121929, SGI0, SGI0, SGI0, SGI121930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26661[] = {SGI121931, SGI0, SGI121932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26662[] = {SGI121933, SGI0, SGI121934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121935, SGI121935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26663[] = {SGI121936, SGI121937, SGI121938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121939, SGI0, SGI0, SGI0, SGI121940, SGI121940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26664[] = {SGI121944, SGI0, SGI121945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26665[] = {SGI121941, SGI121942, SGI121943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121946, SGI0, SGI0, SGI0, SGI121947, SGI121947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26666[] = {SGI121948, SGI121949, SGI121950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121951, SGI0, SGI0, SGI0, SGI121952, SGI121952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26667[] = {SGI121953, SGI121954, SGI121955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121956, SGI0, SGI0, SGI0, SGI121957, SGI121957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26668[] = {SGI121958, SGI121959, SGI121960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121961, SGI0, SGI0, SGI0, SGI121962, SGI121962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26669[] = {SGI121963, SGI121964, SGI121965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121968, SGI0, SGI0, SGI0, SGI121970, SGI121970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26670[] = {SGI121966, SGI121967, SGI121969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121971, SGI0, SGI0, SGI0, SGI121972, SGI121972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121973, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26671[] = {SGI121974, SGI121975, SGI121976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121977, SGI0, SGI0, SGI0, SGI121978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26672[] = {SGI121979, SGI121980, SGI121981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121982, SGI0, SGI0, SGI0, SGI121983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26673[] = {SGI121984, SGI121985, SGI121986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121987, SGI0, SGI0, SGI0, SGI121988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26674[] = {SGI121989, SGI121990, SGI121991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121992, SGI0, SGI0, SGI0, SGI121993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26675[] = {SGI121994, SGI121996, SGI121997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI121999, SGI0, SGI0, SGI0, SGI122001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26676[] = {SGI121995, SGI0, SGI121998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122000, SGI122002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26677[] = {SGI122003, SGI122004, SGI122005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122006, SGI0, SGI0, SGI0, SGI122007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26678[] = {SGI122008, SGI122009, SGI122010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122011, SGI0, SGI0, SGI0, SGI122012, SGI122012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26679[] = {SGI122013, SGI122014, SGI122015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122016, SGI0, SGI0, SGI0, SGI122017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26680[] = {SGI122018, SGI122019, SGI122020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122021, SGI0, SGI0, SGI0, SGI122022, SGI122022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26681[] = {SGI122023, SGI122024, SGI122025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122026, SGI0, SGI0, SGI0, SGI122027, SGI122027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26682[] = {SGI122028, SGI122029, SGI122030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122031, SGI0, SGI0, SGI0, SGI122032, SGI122032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26683[] = {SGI122033, SGI122034, SGI122035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122036, SGI0, SGI0, SGI0, SGI122037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26684[] = {SGI122038, SGI122039, SGI122040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122041, SGI0, SGI0, SGI0, SGI122042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26685[] = {SGI122043, SGI0, SGI122044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122047, SGI122049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26686[] = {SGI122045, SGI122046, SGI122048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122050, SGI0, SGI0, SGI0, SGI122051, SGI122051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26687[] = {SGI122052, SGI122053, SGI122054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122055, SGI0, SGI0, SGI0, SGI122056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26688[] = {SGI0, SGI0, SGI122057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122058, SGI0, SGI0, SGI0, SGI122059, SGI122059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26689[] = {SGI122060, SGI122061, SGI122062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122063, SGI0, SGI0, SGI0, SGI122064, SGI122064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26690[] = {SGI122065, SGI122066, SGI122067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122068, SGI0, SGI0, SGI0, SGI122069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26691[] = {SGI122070, SGI0, SGI122071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26692[] = {SGI122072, SGI122073, SGI122074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122075, SGI0, SGI0, SGI0, SGI122076, SGI122076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26693[] = {SGI122077, SGI122078, SGI122079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122080, SGI0, SGI0, SGI0, SGI122081, SGI122081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26694[] = {SGI122082, SGI0, SGI122083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122084, SGI0, SGI0, SGI0, SGI122085, SGI122085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26695[] = {SGI122088, SGI0, SGI122090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26696[] = {SGI122086, SGI122087, SGI122089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122091, SGI0, SGI0, SGI0, SGI122093, SGI122093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26697[] = {SGI122092, SGI122094, SGI122095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122096, SGI0, SGI0, SGI0, SGI122097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26698[] = {SGI122098, SGI122099, SGI122100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122101, SGI0, SGI0, SGI0, SGI122102, SGI122102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26699[] = {SGI122103, SGI122104, SGI122105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122106, SGI0, SGI0, SGI0, SGI122107, SGI122107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26700[] = {SGI122108, SGI122109, SGI122110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122111, SGI0, SGI0, SGI0, SGI122112, SGI122112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26701[] = {SGI122113, SGI122114, SGI122115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122116, SGI0, SGI0, SGI0, SGI122117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26702[] = {SGI122118, SGI122119, SGI122120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122121, SGI0, SGI0, SGI0, SGI122122, SGI122122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26703[] = {SGI122123, SGI122124, SGI122125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122126, SGI0, SGI0, SGI0, SGI122127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26704[] = {SGI122128, SGI0, SGI122129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26705[] = {SGI122130, SGI122131, SGI122132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122133, SGI0, SGI0, SGI0, SGI122134, SGI122134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26706[] = {SGI122135, SGI122136, SGI122137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122138, SGI0, SGI0, SGI0, SGI122139, SGI122139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26707[] = {SGI122140, SGI122141, SGI122142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122143, SGI0, SGI0, SGI0, SGI122144, SGI122144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26708[] = {SGI122145, SGI0, SGI122146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26709[] = {SGI122147, SGI122148, SGI122149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122150, SGI0, SGI0, SGI0, SGI122151, SGI122151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26710[] = {SGI122152, SGI122153, SGI122154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122155, SGI0, SGI0, SGI0, SGI122156, SGI122156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26711[] = {SGI122157, SGI122158, SGI122159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122160, SGI0, SGI0, SGI0, SGI122161, SGI122161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26712[] = {SGI122162, SGI122163, SGI122164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122165, SGI0, SGI0, SGI0, SGI122166, SGI122166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26713[] = {SGI122167, SGI122168, SGI122169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122171, SGI0, SGI0, SGI0, SGI122173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26714[] = {SGI122175, SGI0, SGI122176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26715[] = {SGI122170, SGI122172, SGI122174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122177, SGI0, SGI0, SGI0, SGI122178, SGI122178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122179, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26716[] = {SGI122180, SGI122181, SGI122182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122186, SGI0, SGI0, SGI0, SGI122188, SGI122188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26717[] = {SGI122183, SGI122184, SGI122185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122187, SGI0, SGI0, SGI0, SGI122189, SGI122189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26718[] = {SGI122190, SGI122191, SGI122192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122193, SGI0, SGI0, SGI0, SGI122194, SGI122194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26719[] = {SGI122195, SGI0, SGI122196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122200, SGI0, SGI0, SGI0, SGI122201, SGI122201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26720[] = {SGI122197, SGI122198, SGI122199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122202, SGI0, SGI0, SGI0, SGI122203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26721[] = {SGI122204, SGI122205, SGI122206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122207, SGI0, SGI0, SGI0, SGI122208, SGI122208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26722[] = {SGI122209, SGI122210, SGI122211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122212, SGI0, SGI0, SGI0, SGI122213, SGI122213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26723[] = {SGI122214, SGI122215, SGI122216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122217, SGI0, SGI0, SGI0, SGI122218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26724[] = {SGI122219, SGI122220, SGI122221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122222, SGI0, SGI0, SGI0, SGI122223, SGI122223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26725[] = {SGI122224, SGI0, SGI122225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122229, SGI0, SGI0, SGI0, SGI122231, SGI122231, SGI0, SGI0, SGI122233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26726[] = {SGI122226, SGI122227, SGI122228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122230, SGI0, SGI0, SGI0, SGI122232, SGI122232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26727[] = {SGI122234, SGI122235, SGI122236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122237, SGI0, SGI0, SGI0, SGI122238, SGI122238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26728[] = {SGI122239, SGI122240, SGI122241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122242, SGI0, SGI0, SGI0, SGI122244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26729[] = {SGI122243, SGI0, SGI122245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122246, SGI0, SGI0, SGI0, SGI122247, SGI122247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26730[] = {SGI122248, SGI122249, SGI122250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122252, SGI0, SGI0, SGI0, SGI122254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26731[] = {SGI122251, SGI0, SGI122253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122255, SGI0, SGI0, SGI0, SGI122256, SGI122256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26732[] = {SGI122257, SGI122258, SGI122259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122263, SGI0, SGI0, SGI0, SGI122264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26733[] = {SGI122260, SGI122261, SGI122262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122265, SGI0, SGI0, SGI0, SGI122266, SGI122266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26734[] = {SGI122267, SGI122268, SGI122269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122270, SGI0, SGI0, SGI0, SGI122271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26735[] = {SGI122272, SGI122273, SGI122274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122275, SGI0, SGI0, SGI0, SGI122276, SGI122276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26736[] = {SGI122277, SGI122278, SGI122279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122280, SGI0, SGI0, SGI0, SGI122281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26737[] = {SGI122282, SGI0, SGI122283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122284, SGI0, SGI0, SGI0, SGI122285, SGI122285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26738[] = {SGI122286, SGI122287, SGI122288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122291, SGI0, SGI0, SGI0, SGI122292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26739[] = {SGI122289, SGI0, SGI122290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122293, SGI0, SGI0, SGI0, SGI122294, SGI122294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26740[] = {SGI122295, SGI122296, SGI122297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122298, SGI0, SGI0, SGI0, SGI122299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26741[] = {SGI122300, SGI0, SGI122301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122302, SGI122303, SGI122303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26742[] = {SGI122304, SGI122305, SGI122306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122307, SGI0, SGI0, SGI0, SGI122308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26743[] = {SGI122309, SGI122310, SGI122311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122312, SGI0, SGI0, SGI0, SGI122313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26744[] = {SGI122314, SGI122315, SGI122316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122317, SGI0, SGI0, SGI0, SGI122318, SGI122318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26745[] = {SGI122319, SGI122320, SGI122321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122322, SGI0, SGI0, SGI0, SGI122323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26746[] = {SGI122324, SGI122325, SGI122326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122327, SGI0, SGI0, SGI0, SGI122328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26747[] = {SGI122329, SGI122330, SGI122331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122332, SGI0, SGI0, SGI0, SGI122333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26748[] = {SGI122334, SGI122335, SGI122336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122337, SGI0, SGI0, SGI0, SGI122338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26749[] = {SGI122339, SGI122340, SGI122341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122342, SGI0, SGI0, SGI0, SGI122343, SGI122343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26750[] = {SGI122344, SGI122345, SGI122346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122347, SGI0, SGI0, SGI0, SGI122348, SGI122348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26751[] = {SGI122349, SGI0, SGI122350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122351, SGI0, SGI0, SGI0, SGI122352, SGI122352, SGI0, SGI0, SGI122353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26752[] = {SGI122354, SGI122355, SGI122356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122357, SGI0, SGI0, SGI0, SGI122358, SGI122358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26753[] = {SGI122359, SGI122360, SGI122361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122362, SGI0, SGI0, SGI0, SGI122363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26754[] = {SGI122364, SGI122365, SGI122366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122367, SGI0, SGI0, SGI0, SGI122370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26755[] = {SGI122368, SGI122369, SGI122371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122372, SGI0, SGI0, SGI0, SGI122373, SGI122373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122374, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26756[] = {SGI122375, SGI122376, SGI122377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122378, SGI0, SGI0, SGI0, SGI122379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26757[] = {SGI122380, SGI122381, SGI122382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122383, SGI0, SGI0, SGI0, SGI122384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26758[] = {SGI122385, SGI122386, SGI122387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122388, SGI0, SGI0, SGI0, SGI122389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26759[] = {SGI122390, SGI122391, SGI122392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122393, SGI0, SGI0, SGI0, SGI122394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26760[] = {SGI122395, SGI122396, SGI122397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122398, SGI0, SGI0, SGI0, SGI122399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26761[] = {SGI122400, SGI0, SGI122401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122402, SGI0, SGI0, SGI0, SGI122403, SGI122403, SGI0, SGI0, SGI122404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26762[] = {SGI122405, SGI122406, SGI122407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122408, SGI0, SGI0, SGI0, SGI122409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26763[] = {SGI122410, SGI122411, SGI122412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122413, SGI0, SGI0, SGI0, SGI122414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26764[] = {SGI122415, SGI122416, SGI122417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122418, SGI0, SGI0, SGI0, SGI122419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26765[] = {SGI122420, SGI122421, SGI122422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122423, SGI0, SGI0, SGI0, SGI122424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26766[] = {SGI122425, SGI122426, SGI122427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122428, SGI122428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26767[] = {SGI122429, SGI122430, SGI122431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122432, SGI0, SGI0, SGI0, SGI122433, SGI122433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26768[] = {SGI122434, SGI122435, SGI122436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122437, SGI0, SGI0, SGI0, SGI122438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26769[] = {SGI122439, SGI122440, SGI122441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122442, SGI0, SGI0, SGI0, SGI122443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26770[] = {SGI122444, SGI122445, SGI122446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122447, SGI0, SGI0, SGI0, SGI122448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26771[] = {SGI122449, SGI122450, SGI122451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122452, SGI0, SGI0, SGI0, SGI122453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26772[] = {SGI122454, SGI0, SGI122455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122456, SGI122457, SGI122457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26773[] = {SGI122458, SGI122459, SGI122460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122461, SGI0, SGI0, SGI0, SGI122462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26774[] = {SGI122463, SGI122464, SGI122465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122466, SGI0, SGI0, SGI0, SGI122467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26775[] = {SGI122468, SGI122469, SGI122470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122471, SGI0, SGI0, SGI0, SGI122472, SGI122472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26776[] = {SGI122473, SGI122474, SGI122475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122476, SGI0, SGI0, SGI0, SGI122477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26777[] = {SGI122478, SGI0, SGI122479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122480, SGI122481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26778[] = {SGI122482, SGI122483, SGI122484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122485, SGI0, SGI0, SGI0, SGI122486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26779[] = {SGI122487, SGI122488, SGI122489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122490, SGI0, SGI0, SGI0, SGI122491, SGI122491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26780[] = {SGI122492, SGI0, SGI122493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122494, SGI122495, SGI122495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26781[] = {SGI122496, SGI122497, SGI122498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122499, SGI122499, SGI0, SGI0, SGI122500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26782[] = {SGI122501, SGI0, SGI122502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122503, SGI122504, SGI122504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26783[] = {SGI122505, SGI122506, SGI122507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122508, SGI0, SGI0, SGI0, SGI122509, SGI122509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26784[] = {SGI122510, SGI122511, SGI122512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122513, SGI0, SGI0, SGI0, SGI122514, SGI122514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26785[] = {SGI122515, SGI122516, SGI122517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122518, SGI0, SGI0, SGI0, SGI122519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26786[] = {SGI122520, SGI122521, SGI122522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122523, SGI0, SGI0, SGI0, SGI122524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26787[] = {SGI122525, SGI122526, SGI122527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122528, SGI0, SGI0, SGI0, SGI122529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26788[] = {SGI122530, SGI122531, SGI122532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122533, SGI0, SGI0, SGI0, SGI122534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26789[] = {SGI122535, SGI122536, SGI122537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122538, SGI0, SGI0, SGI0, SGI122539, SGI122539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26790[] = {SGI122540, SGI122541, SGI122542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122543, SGI0, SGI0, SGI0, SGI122544, SGI122544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26791[] = {SGI122545, SGI122546, SGI122547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122548, SGI0, SGI0, SGI0, SGI122549, SGI122549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26792[] = {SGI122550, SGI122551, SGI122552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122553, SGI0, SGI0, SGI0, SGI122554, SGI122554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26793[] = {SGI122555, SGI122556, SGI122557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122558, SGI0, SGI0, SGI0, SGI122559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26794[] = {SGI122560, SGI122561, SGI122562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122563, SGI0, SGI0, SGI0, SGI122564, SGI122564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26795[] = {SGI122565, SGI122566, SGI122567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122568, SGI0, SGI0, SGI0, SGI122569, SGI122569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26796[] = {SGI122570, SGI122571, SGI122572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122573, SGI0, SGI0, SGI0, SGI122574, SGI122574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26797[] = {SGI122575, SGI122576, SGI122577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122578, SGI0, SGI0, SGI0, SGI122579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26798[] = {SGI122580, SGI122581, SGI122582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122583, SGI0, SGI0, SGI0, SGI122584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26799[] = {SGI122585, SGI122586, SGI122587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122588, SGI0, SGI0, SGI0, SGI122589, SGI122589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26800[] = {SGI122590, SGI122591, SGI122592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122593, SGI0, SGI0, SGI0, SGI122594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26801[] = {SGI122595, SGI122596, SGI122597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122598, SGI0, SGI0, SGI0, SGI122599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26802[] = {SGI122600, SGI122601, SGI122602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122603, SGI0, SGI0, SGI0, SGI122604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26803[] = {SGI122605, SGI122606, SGI122607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122608, SGI0, SGI0, SGI0, SGI122609, SGI122609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26804[] = {SGI122610, SGI122611, SGI122612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122613, SGI0, SGI0, SGI0, SGI122614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26805[] = {SGI122615, SGI122616, SGI122617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122618, SGI0, SGI0, SGI0, SGI122619, SGI122619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26806[] = {SGI122620, SGI122621, SGI122622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122623, SGI0, SGI0, SGI0, SGI122624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26807[] = {SGI122625, SGI122626, SGI122627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122628, SGI0, SGI0, SGI0, SGI122629, SGI122629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26808[] = {SGI122630, SGI0, SGI122631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122632, SGI122633, SGI122633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26809[] = {SGI122634, SGI122635, SGI122636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122637, SGI0, SGI0, SGI0, SGI122638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26810[] = {SGI122639, SGI122640, SGI122641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122642, SGI0, SGI0, SGI0, SGI122643, SGI122643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26811[] = {SGI122644, SGI122645, SGI122646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122647, SGI0, SGI0, SGI0, SGI122648, SGI122648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26812[] = {SGI122649, SGI122650, SGI122651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122652, SGI0, SGI0, SGI0, SGI122653, SGI122653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26813[] = {SGI122654, SGI122655, SGI122656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122657, SGI0, SGI0, SGI0, SGI122658, SGI122658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26814[] = {SGI122659, SGI122660, SGI122661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122662, SGI0, SGI0, SGI0, SGI122663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26815[] = {SGI122664, SGI122665, SGI122666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122667, SGI0, SGI0, SGI0, SGI122668, SGI122668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26816[] = {SGI122669, SGI0, SGI122670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26817[] = {SGI122671, SGI122672, SGI122673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122674, SGI0, SGI0, SGI0, SGI122675, SGI122675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26818[] = {SGI122676, SGI122677, SGI122678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122679, SGI122679, SGI0, SGI0, SGI122680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26819[] = {SGI122681, SGI122682, SGI122683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122684, SGI0, SGI0, SGI0, SGI122685, SGI122685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26820[] = {SGI122686, SGI122687, SGI122688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122689, SGI0, SGI0, SGI0, SGI122690, SGI122690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26821[] = {SGI122691, SGI122692, SGI122693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122694, SGI0, SGI0, SGI0, SGI122695, SGI122695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26822[] = {SGI122696, SGI122697, SGI122698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122699, SGI0, SGI0, SGI0, SGI122700, SGI122700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26823[] = {SGI122701, SGI0, SGI122702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122703, SGI122704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26824[] = {SGI122705, SGI122706, SGI122707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122708, SGI0, SGI0, SGI0, SGI122709, SGI122709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26825[] = {SGI122710, SGI122711, SGI122712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122713, SGI0, SGI0, SGI0, SGI122714, SGI122714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26826[] = {SGI122715, SGI122716, SGI122717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122718, SGI0, SGI0, SGI0, SGI122719, SGI122719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26827[] = {SGI122720, SGI122721, SGI122722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122723, SGI0, SGI0, SGI0, SGI122724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26828[] = {SGI122725, SGI122726, SGI122727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122728, SGI0, SGI0, SGI0, SGI122729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26829[] = {SGI122730, SGI0, SGI122731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122732, SGI122734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26830[] = {SGI122733, SGI122735, SGI122736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122737, SGI0, SGI0, SGI0, SGI122738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26831[] = {SGI122739, SGI122740, SGI122741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122742, SGI0, SGI0, SGI0, SGI122743, SGI122743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26832[] = {SGI122744, SGI122745, SGI122746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122747, SGI0, SGI0, SGI0, SGI122748, SGI122748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26833[] = {SGI122749, SGI122750, SGI122751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122752, SGI0, SGI0, SGI0, SGI122753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26834[] = {SGI122754, SGI0, SGI122755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122756, SGI122757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26835[] = {SGI122758, SGI122759, SGI122760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122761, SGI0, SGI0, SGI0, SGI122762, SGI122762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26836[] = {SGI122763, SGI122764, SGI122765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122766, SGI0, SGI0, SGI0, SGI122767, SGI122767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26837[] = {SGI122768, SGI122769, SGI122770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122771, SGI0, SGI0, SGI0, SGI122772, SGI122772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26838[] = {SGI122773, SGI122774, SGI122775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122776, SGI0, SGI0, SGI0, SGI122777, SGI122777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26839[] = {SGI122778, SGI122779, SGI122780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122781, SGI0, SGI0, SGI0, SGI122782, SGI122782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26840[] = {SGI122783, SGI0, SGI122784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26841[] = {SGI122785, SGI122786, SGI122787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122788, SGI0, SGI0, SGI0, SGI122789, SGI122789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26842[] = {SGI122790, SGI122791, SGI122792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122793, SGI0, SGI0, SGI0, SGI122794, SGI122794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26843[] = {SGI122795, SGI122796, SGI122797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122798, SGI0, SGI0, SGI0, SGI122799, SGI122799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26844[] = {SGI122800, SGI122801, SGI122802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122803, SGI0, SGI0, SGI0, SGI122804, SGI122804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26845[] = {SGI122805, SGI122806, SGI122807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122808, SGI0, SGI0, SGI0, SGI122809, SGI122809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26846[] = {SGI122810, SGI122811, SGI122812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122813, SGI0, SGI0, SGI0, SGI122814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26847[] = {SGI122815, SGI122816, SGI122817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122818, SGI0, SGI0, SGI0, SGI122819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26848[] = {SGI122820, SGI122821, SGI122822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122823, SGI122823, SGI0, SGI0, SGI122825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26849[] = {SGI122824, SGI122826, SGI122827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122828, SGI0, SGI0, SGI0, SGI122829, SGI122829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26850[] = {SGI122830, SGI0, SGI122831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26851[] = {SGI122832, SGI122833, SGI122834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122835, SGI0, SGI0, SGI0, SGI122836, SGI122836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26852[] = {SGI122837, SGI122838, SGI122839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122840, SGI0, SGI0, SGI0, SGI122841, SGI122841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26853[] = {SGI122842, SGI122843, SGI122844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122845, SGI0, SGI0, SGI0, SGI122846, SGI122846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122848, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26854[] = {SGI122847, SGI122849, SGI122850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122851, SGI0, SGI0, SGI0, SGI122852, SGI122852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26855[] = {SGI122853, SGI122854, SGI122855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122856, SGI0, SGI0, SGI0, SGI122857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122858, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26856[] = {SGI122859, SGI122860, SGI122861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122862, SGI0, SGI0, SGI0, SGI122863, SGI122863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26857[] = {SGI122864, SGI122865, SGI122866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122867, SGI0, SGI0, SGI0, SGI122868, SGI122868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26858[] = {SGI122869, SGI122870, SGI122871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122872, SGI0, SGI0, SGI0, SGI122873, SGI122873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26859[] = {SGI122874, SGI122875, SGI122876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122879, SGI0, SGI0, SGI0, SGI122880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26860[] = {SGI122877, SGI0, SGI122878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122881, SGI0, SGI0, SGI0, SGI122882, SGI122882, SGI0, SGI0, SGI122883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26861[] = {SGI122884, SGI122885, SGI122886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122887, SGI0, SGI0, SGI0, SGI122888, SGI122888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26862[] = {SGI0, SGI122889, SGI122890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122891, SGI0, SGI0, SGI0, SGI122892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122893, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26863[] = {SGI122894, SGI122895, SGI122896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122897, SGI0, SGI0, SGI0, SGI122898, SGI122898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26864[] = {SGI122899, SGI122900, SGI122901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122902, SGI0, SGI0, SGI0, SGI122903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26865[] = {SGI122904, SGI122905, SGI122906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122907, SGI0, SGI0, SGI0, SGI122908, SGI122908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26866[] = {SGI122909, SGI122910, SGI122911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122912, SGI0, SGI0, SGI0, SGI122913, SGI122913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26867[] = {SGI0, SGI0, SGI122914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26868[] = {SGI122915, SGI0, SGI122916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26869[] = {SGI122917, SGI122918, SGI122919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122920, SGI0, SGI0, SGI0, SGI122921, SGI122921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26870[] = {SGI122922, SGI122923, SGI122924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122925, SGI0, SGI0, SGI0, SGI122926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26871[] = {SGI122927, SGI122928, SGI122929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122930, SGI0, SGI0, SGI0, SGI122931, SGI122931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26872[] = {SGI122933, SGI122934, SGI122936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122937, SGI0, SGI0, SGI0, SGI122939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26873[] = {SGI122932, SGI0, SGI122935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122938, SGI0, SGI0, SGI0, SGI122940, SGI122940, SGI0, SGI0, SGI122941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26874[] = {SGI122942, SGI122943, SGI122944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122945, SGI0, SGI0, SGI0, SGI122946, SGI122946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26875[] = {SGI122947, SGI122948, SGI122949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122950, SGI0, SGI0, SGI0, SGI122951, SGI122951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26876[] = {SGI122952, SGI122953, SGI122954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122955, SGI0, SGI0, SGI0, SGI122956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26877[] = {SGI122957, SGI0, SGI122958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26878[] = {SGI122959, SGI122960, SGI122961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122965, SGI0, SGI0, SGI0, SGI122966, SGI122966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26879[] = {SGI122962, SGI122963, SGI122964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122967, SGI0, SGI0, SGI0, SGI122968, SGI122968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26880[] = {SGI122969, SGI122970, SGI122971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122972, SGI0, SGI0, SGI0, SGI122973, SGI122973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26881[] = {SGI122974, SGI122975, SGI122976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122977, SGI0, SGI0, SGI0, SGI122978, SGI122978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26882[] = {SGI122979, SGI0, SGI122980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26883[] = {SGI122981, SGI122982, SGI122983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122984, SGI0, SGI0, SGI0, SGI122985, SGI122985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26884[] = {SGI122986, SGI0, SGI122988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122992, SGI122993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26885[] = {SGI122987, SGI122989, SGI122990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122991, SGI0, SGI0, SGI0, SGI122994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26886[] = {SGI122995, SGI122996, SGI122997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI122998, SGI0, SGI0, SGI0, SGI122999, SGI122999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26887[] = {SGI123000, SGI0, SGI123001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123002, SGI0, SGI0, SGI0, SGI123003, SGI123003, SGI0, SGI0, SGI123004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26888[] = {SGI123005, SGI123006, SGI123007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123008, SGI0, SGI0, SGI0, SGI123009, SGI123009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26889[] = {SGI123010, SGI123011, SGI123012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123013, SGI0, SGI0, SGI0, SGI123014, SGI123014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26890[] = {SGI123015, SGI0, SGI123016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123020, SGI0, SGI0, SGI0, SGI123022, SGI123022, SGI0, SGI0, SGI123024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26891[] = {SGI123017, SGI123018, SGI123019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123021, SGI0, SGI0, SGI0, SGI123023, SGI123023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26892[] = {SGI123025, SGI123026, SGI123027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123028, SGI0, SGI0, SGI0, SGI123029, SGI123029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26893[] = {SGI123030, SGI123031, SGI123032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123033, SGI0, SGI0, SGI0, SGI123034, SGI123034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26894[] = {SGI123035, SGI123036, SGI123037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123038, SGI0, SGI0, SGI0, SGI123039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26895[] = {SGI123040, SGI123041, SGI123042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123043, SGI0, SGI0, SGI0, SGI123044, SGI123044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26896[] = {SGI123045, SGI0, SGI123046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26897[] = {SGI123047, SGI123048, SGI123049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123050, SGI0, SGI0, SGI0, SGI123051, SGI123051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26898[] = {SGI0, SGI0, SGI123052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123053, SGI123054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26899[] = {SGI123055, SGI123056, SGI123057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123058, SGI0, SGI0, SGI0, SGI123059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26900[] = {SGI123060, SGI123061, SGI123062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123063, SGI0, SGI0, SGI0, SGI123064, SGI123064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26901[] = {SGI123065, SGI123066, SGI123067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123068, SGI0, SGI0, SGI0, SGI123069, SGI123069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26902[] = {SGI123070, SGI0, SGI123071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123072, SGI123073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26903[] = {SGI123074, SGI123075, SGI123076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123077, SGI0, SGI0, SGI0, SGI123078, SGI123078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26904[] = {SGI123079, SGI123080, SGI123081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123082, SGI0, SGI0, SGI0, SGI123083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26905[] = {SGI123084, SGI0, SGI123085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123086, SGI123087, SGI123087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26906[] = {SGI123088, SGI0, SGI123089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123090, SGI123091, SGI123091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26907[] = {SGI123092, SGI123093, SGI123094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123095, SGI0, SGI0, SGI0, SGI123096, SGI123096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26908[] = {SGI123097, SGI123098, SGI123099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123100, SGI0, SGI0, SGI0, SGI123101, SGI123101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123102, SGI0, SGI0, NULL};
const Segment * const SG26909[] = {SGI123103, SGI123104, SGI123105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123106, SGI123106, SGI0, SGI0, SGI123107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26910[] = {SGI123108, SGI123109, SGI123110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123111, SGI123111, SGI0, SGI0, SGI123112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26911[] = {SGI123113, SGI0, SGI123114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123115, SGI123116, SGI123116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26912[] = {SGI123117, SGI123118, SGI123119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123120, SGI0, SGI0, SGI0, SGI123121, SGI123121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26913[] = {SGI123122, SGI123123, SGI123124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123125, SGI0, SGI0, SGI0, SGI123126, SGI123126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26914[] = {SGI0, SGI123127, SGI0, SGI123128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26915[] = {SGI123129, SGI123130, SGI123131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123132, SGI123132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26916[] = {SGI123133, SGI123134, SGI123135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123136, SGI0, SGI0, SGI0, SGI123137, SGI123137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26917[] = {SGI123138, SGI0, SGI123139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26918[] = {SGI123140, SGI123141, SGI123142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123143, SGI0, SGI0, SGI0, SGI123144, SGI123144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26919[] = {SGI123145, SGI123146, SGI123147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123148, SGI123148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26920[] = {SGI123149, SGI123150, SGI123151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123152, SGI0, SGI0, SGI0, SGI123153, SGI123153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26921[] = {SGI123154, SGI123155, SGI123156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123157, SGI0, SGI0, SGI0, SGI123158, SGI123158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26922[] = {SGI123159, SGI123160, SGI123161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123162, SGI123162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26923[] = {SGI123163, SGI123164, SGI123165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123166, SGI0, SGI0, SGI0, SGI123167, SGI123167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26924[] = {SGI123168, SGI123169, SGI123170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123171, SGI123171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26925[] = {SGI123172, SGI123173, SGI123174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123175, SGI123175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26926[] = {SGI123176, SGI123177, SGI123178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123179, SGI0, SGI0, SGI0, SGI123180, SGI123180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26927[] = {SGI123181, SGI0, SGI123182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123183, SGI0, SGI0, SGI0, SGI123184, SGI123184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26928[] = {SGI123185, SGI123186, SGI123187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123188, SGI0, SGI0, SGI0, SGI123189, SGI0, SGI0, SGI0, SGI0, SGI123191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26929[] = {SGI0, SGI123190, SGI123192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26930[] = {SGI123193, SGI0, SGI123194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123195, SGI123196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26931[] = {SGI123197, SGI0, SGI123198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123199, SGI123200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26932[] = {SGI123201, SGI123202, SGI123203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123204, SGI0, SGI0, SGI0, SGI123205, SGI123205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123206, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26933[] = {SGI123207, SGI0, SGI123208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123209, SGI123210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26934[] = {SGI123211, SGI0, SGI123212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123213, SGI0, SGI0, SGI0, SGI123214, SGI123214, SGI0, SGI0, SGI123215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26935[] = {SGI123216, SGI0, SGI123217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123218, SGI0, SGI0, SGI0, SGI123219, SGI123219, SGI0, SGI0, SGI123220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26936[] = {SGI123221, SGI123222, SGI123223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123224, SGI123224, SGI0, SGI0, SGI123225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26937[] = {SGI123226, SGI123227, SGI123228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123229, SGI0, SGI0, SGI0, SGI123230, SGI123230, SGI0, SGI123231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26938[] = {SGI123232, SGI0, SGI123233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123234, SGI123235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26939[] = {SGI123236, SGI123237, SGI123238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123239, SGI0, SGI0, SGI0, SGI123240, SGI123240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26940[] = {SGI123241, SGI123242, SGI123243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123244, SGI0, SGI0, SGI0, SGI123245, SGI123245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123246, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26941[] = {SGI123247, SGI123248, SGI123249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123250, SGI123250, SGI0, SGI0, SGI123251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26942[] = {SGI123252, SGI123253, SGI123254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123255, SGI0, SGI0, SGI0, SGI123256, SGI123256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123257, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26943[] = {SGI123258, SGI123259, SGI123260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123261, SGI123261, SGI0, SGI0, SGI123262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26944[] = {SGI0, SGI123263, SGI123264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123265, SGI123265, SGI0, SGI0, SGI123266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26945[] = {SGI123267, SGI123268, SGI123269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123270, SGI123270, SGI0, SGI0, SGI123271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26946[] = {SGI123272, SGI123273, SGI123274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123275, SGI0, SGI0, SGI0, SGI123276, SGI123276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123277, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26947[] = {SGI123278, SGI123279, SGI123280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123281, SGI0, SGI0, SGI0, SGI123282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123283, SGI0, SGI0, NULL};
const Segment * const SG26948[] = {SGI123284, SGI123285, SGI123286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123287, SGI123287, SGI0, SGI0, SGI123288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26949[] = {SGI123289, SGI0, SGI123290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123291, SGI123292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26950[] = {SGI123293, SGI123294, SGI123295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123296, SGI123296, SGI0, SGI0, SGI123297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26951[] = {SGI123298, SGI123299, SGI123300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123301, SGI123301, SGI0, SGI0, SGI123302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26952[] = {SGI123303, SGI0, SGI123304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123305, SGI123306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26953[] = {SGI123307, SGI0, SGI123308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123309, SGI123310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26954[] = {SGI123311, SGI0, SGI123312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123313, SGI0, SGI0, SGI0, SGI123314, SGI123314, SGI0, SGI0, SGI123315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26955[] = {SGI123316, SGI123317, SGI123318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123319, SGI0, SGI0, SGI0, SGI123320, SGI0, SGI0, SGI0, SGI0, SGI123321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26956[] = {SGI123322, SGI0, SGI123323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123324, SGI123325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26957[] = {SGI123326, SGI0, SGI123327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123328, SGI123329, SGI123329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26958[] = {SGI123330, SGI123331, SGI123332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123333, SGI0, SGI0, SGI0, SGI123334, SGI123334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123335, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26959[] = {SGI123336, SGI0, SGI123337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123338, SGI0, SGI0, SGI0, SGI123339, SGI123339, SGI0, SGI0, SGI123340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26960[] = {SGI123341, SGI0, SGI123342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123343, SGI0, SGI0, SGI0, SGI123344, SGI123344, SGI0, SGI0, SGI123345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26961[] = {SGI123346, SGI0, SGI123347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123348, SGI123349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26962[] = {SGI123350, SGI123351, SGI123352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123353, SGI123353, SGI0, SGI0, SGI123354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26963[] = {SGI123355, SGI0, SGI123356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123357, SGI123358, SGI123358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26964[] = {SGI123359, SGI0, SGI123360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123361, SGI123362, SGI123362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123363, SGI0, NULL};
const Segment * const SG26965[] = {SGI123364, SGI0, SGI123365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123366, SGI123367, SGI123367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26966[] = {SGI123368, SGI0, SGI123369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123370, SGI123371, SGI123371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123372, SGI0, NULL};
const Segment * const SG26967[] = {SGI123373, SGI123374, SGI123375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123376, SGI0, SGI0, SGI0, SGI123377, SGI123377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123378, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26968[] = {SGI123379, SGI123380, SGI123381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123382, SGI123382, SGI0, SGI0, SGI123383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26969[] = {SGI123384, SGI123385, SGI123386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123387, SGI123387, SGI0, SGI0, SGI123388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26970[] = {SGI123389, SGI123390, SGI123391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123392, SGI0, SGI0, SGI0, SGI123393, SGI123393, SGI0, SGI123394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26971[] = {SGI123395, SGI123396, SGI123397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123398, SGI123398, SGI0, SGI0, SGI123399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26972[] = {SGI123400, SGI0, SGI123401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123402, SGI123403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26973[] = {SGI123404, SGI123405, SGI123406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123407, SGI123407, SGI0, SGI0, SGI123408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26974[] = {SGI123409, SGI123410, SGI123411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123412, SGI123412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123413, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26975[] = {SGI123414, SGI0, SGI123415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123416, SGI123417, SGI123417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123418, SGI0, NULL};
const Segment * const SG26976[] = {SGI123419, SGI0, SGI123420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123421, SGI123422, SGI123422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123423, SGI0, NULL};
const Segment * const SG26977[] = {SGI123424, SGI123425, SGI123426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123427, SGI0, SGI0, SGI0, SGI123428, SGI123429, SGI0, SGI0, SGI123430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26978[] = {SGI123431, SGI123432, SGI123433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123434, SGI0, SGI0, SGI0, SGI123435, SGI123436, SGI0, SGI0, SGI123437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26979[] = {SGI123438, SGI123439, SGI123440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123441, SGI123441, SGI0, SGI0, SGI123442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26980[] = {SGI123443, SGI0, SGI123444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123445, SGI123446, SGI123446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26981[] = {SGI0, SGI0, SGI123447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123448, SGI0, SGI0, SGI0, SGI123449, SGI123449, SGI0, SGI0, SGI123450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26982[] = {SGI123451, SGI123452, SGI123453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123454, SGI0, SGI0, SGI0, SGI123455, SGI123455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123456, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26983[] = {SGI123457, SGI123458, SGI123459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123460, SGI0, SGI123461, SGI123461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26984[] = {SGI123462, SGI0, SGI123463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123464, SGI0, SGI0, SGI0, SGI123465, SGI123465, SGI0, SGI0, SGI123466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26985[] = {SGI123467, SGI123468, SGI123469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123470, SGI0, SGI0, SGI0, SGI123471, SGI123472, SGI0, SGI0, SGI123473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26986[] = {SGI123474, SGI0, SGI123475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123476, SGI123477, SGI123477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26987[] = {SGI123478, SGI0, SGI123479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123480, SGI0, SGI0, SGI0, SGI123481, SGI123481, SGI0, SGI0, SGI123482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26988[] = {SGI123483, SGI0, SGI123484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123485, SGI0, SGI0, SGI0, SGI123486, SGI123486, SGI0, SGI0, SGI123487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26989[] = {SGI123488, SGI123489, SGI123490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123491, SGI0, SGI0, SGI0, SGI123492, SGI123492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123493, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26990[] = {SGI123494, SGI123495, SGI123496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123497, SGI0, SGI0, SGI0, SGI123498, SGI123498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26991[] = {SGI123499, SGI0, SGI123500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123501, SGI0, SGI0, SGI0, SGI123502, SGI123502, SGI0, SGI0, SGI123503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26992[] = {SGI123504, SGI0, SGI123505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123506, SGI123507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26993[] = {SGI123508, SGI123509, SGI123510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123511, SGI123511, SGI0, SGI0, SGI123512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26994[] = {SGI123513, SGI123514, SGI123515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123516, SGI0, SGI0, SGI0, SGI123517, SGI123517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123518, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26995[] = {SGI123519, SGI0, SGI123520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123521, SGI123522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26996[] = {SGI123523, SGI0, SGI123524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123525, SGI123526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123527, SGI0, NULL};
const Segment * const SG26997[] = {SGI123528, SGI0, SGI123529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123530, SGI123531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26998[] = {SGI123532, SGI0, SGI123533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123534, SGI123535, SGI123535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG26999[] = {SGI123536, SGI123537, SGI123538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123539, SGI123539, SGI0, SGI0, SGI123540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27000[] = {SGI123541, SGI123542, SGI123543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123544, SGI123544, SGI0, SGI0, SGI123545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27001[] = {SGI123546, SGI0, SGI123547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123548, SGI123549, SGI123549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27002[] = {SGI123550, SGI123551, SGI123552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123553, SGI123553, SGI0, SGI0, SGI123554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27003[] = {SGI123555, SGI123556, SGI123557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123558, SGI123558, SGI0, SGI0, SGI123559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27004[] = {SGI123560, SGI123561, SGI123562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123563, SGI0, SGI0, SGI0, SGI123564, SGI123564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27005[] = {SGI123565, SGI123566, SGI123567, SGI123568, SGI123569, SGI123569, SGI0, NULL};
const Segment * const SG27006[] = {SGI123570, SGI123571, SGI123572, SGI123573, SGI123574, SGI123574, SGI0, NULL};
const Segment * const SG27007[] = {SGI123575, SGI123576, SGI123577, SGI123578, SGI123579, SGI123579, SGI0, NULL};
const Segment * const SG27008[] = {SGI123580, SGI123581, SGI123582, SGI123583, SGI123584, SGI123584, SGI0, NULL};
const Segment * const SG27009[] = {SGI123585, SGI123586, SGI123587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123588, SGI0, SGI0, SGI0, SGI123589, SGI123589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123590, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27010[] = {SGI123591, SGI123592, SGI123593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123594, SGI0, SGI0, SGI0, SGI123595, SGI123595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123596, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27011[] = {SGI123597, SGI0, SGI123598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123599, SGI123600, SGI123600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27012[] = {SGI123601, SGI123602, SGI123603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123604, SGI0, SGI0, SGI0, SGI123605, SGI123605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123606, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27013[] = {SGI123607, SGI0, SGI123608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123609, SGI0, SGI0, SGI0, SGI123610, SGI123610, SGI0, SGI0, SGI123611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27014[] = {SGI123612, SGI123613, SGI123614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123615, SGI0, SGI0, SGI0, SGI123616, SGI123616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27015[] = {SGI123617, SGI0, SGI123618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123619, SGI123620, SGI123620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27016[] = {SGI123621, SGI123622, SGI123623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123625, SGI0, SGI0, SGI0, SGI123628, SGI123628, SGI0, SGI0, NULL};
const Segment * const SG27017[] = {SGI123624, SGI123626, SGI123627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123629, SGI0, SGI0, SGI0, SGI123630, SGI123630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27018[] = {SGI123631, SGI0, SGI123632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123633, SGI0, SGI0, SGI0, SGI123634, SGI123634, SGI0, SGI0, SGI123635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27019[] = {SGI123636, SGI123637, SGI123638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123639, SGI0, SGI0, SGI0, SGI123640, SGI123641, SGI0, SGI0, SGI123642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27020[] = {SGI123643, SGI0, SGI123644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123645, SGI123646, SGI123646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27021[] = {SGI123647, SGI123648, SGI123649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123650, SGI0, SGI0, SGI0, SGI123651, SGI123651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123652, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27022[] = {SGI123653, SGI0, SGI123654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123655, SGI0, SGI0, SGI0, SGI123656, SGI123656, SGI0, SGI0, SGI123657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27023[] = {SGI123658, SGI0, SGI123659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123660, SGI123661, SGI123661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27024[] = {SGI123662, SGI123663, SGI123664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123665, SGI123665, SGI0, SGI0, SGI123666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27025[] = {SGI123667, SGI0, SGI123668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123669, SGI0, SGI0, SGI0, SGI123670, SGI123670, SGI0, SGI0, SGI123671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27026[] = {SGI123672, SGI123673, SGI123674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123675, SGI0, SGI123676, SGI123676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27027[] = {SGI123677, SGI123678, SGI123679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123680, SGI0, SGI0, SGI0, SGI123681, SGI0, SGI0, SGI0, SGI0, SGI123682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27028[] = {SGI123683, SGI123684, SGI123685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123686, SGI0, SGI123687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27029[] = {SGI123688, SGI0, SGI123689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123690, SGI123691, SGI123691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27030[] = {SGI123692, SGI123693, SGI123694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123695, SGI0, SGI0, SGI0, SGI123696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123697, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27031[] = {SGI123698, SGI123699, SGI123700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123701, SGI0, SGI0, SGI0, SGI123702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27032[] = {SGI123703, SGI0, SGI123704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123705, SGI123706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27033[] = {SGI123707, SGI123708, SGI123709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123710, SGI0, SGI0, SGI0, SGI123711, SGI123711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123712, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27034[] = {SGI123713, SGI0, SGI123714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123715, SGI123716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27035[] = {SGI0, SGI123717, SGI0, SGI123718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27036[] = {SGI123719, SGI123720, SGI123721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123722, SGI0, SGI0, SGI0, SGI123723, SGI123723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123724, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27037[] = {SGI123725, SGI123726, SGI123727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123728, SGI0, SGI0, SGI0, SGI123729, SGI123729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27038[] = {SGI123730, SGI0, SGI123731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123732, SGI0, SGI0, SGI0, SGI123733, SGI123733, SGI0, SGI0, SGI123734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27039[] = {SGI123735, SGI0, SGI123736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123737, SGI0, SGI0, SGI0, SGI123738, SGI123738, SGI0, SGI0, SGI123739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27040[] = {SGI123740, SGI123741, SGI123742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123743, SGI0, SGI0, SGI0, SGI123744, SGI123744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123745, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27041[] = {SGI123746, SGI123747, SGI123748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123749, SGI0, SGI0, SGI0, SGI123750, SGI123750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27042[] = {SGI123751, SGI123752, SGI123753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123754, SGI0, SGI0, SGI0, SGI123755, SGI123755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27043[] = {SGI123756, SGI123757, SGI123758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123759, SGI0, SGI0, SGI0, SGI123760, SGI123760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27044[] = {SGI123761, SGI0, SGI123762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27045[] = {SGI123763, SGI123764, SGI123765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123766, SGI0, SGI0, SGI0, SGI123767, SGI123767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27046[] = {SGI123768, SGI123769, SGI123770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123771, SGI0, SGI0, SGI0, SGI123772, SGI123772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27047[] = {SGI123773, SGI123774, SGI123775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123776, SGI0, SGI0, SGI0, SGI123777, SGI123777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27048[] = {SGI123778, SGI123779, SGI123780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123781, SGI0, SGI0, SGI0, SGI123782, SGI123782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27049[] = {SGI123783, SGI0, SGI123784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27050[] = {SGI123785, SGI123786, SGI123787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123788, SGI0, SGI0, SGI0, SGI123789, SGI123789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27051[] = {SGI123790, SGI123791, SGI123792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123793, SGI0, SGI0, SGI0, SGI123794, SGI123794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27052[] = {SGI123795, SGI123796, SGI123797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123798, SGI0, SGI0, SGI0, SGI123799, SGI123799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27053[] = {SGI123800, SGI123801, SGI123802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123803, SGI0, SGI0, SGI0, SGI123804, SGI123804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27054[] = {SGI123805, SGI123806, SGI123807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123808, SGI0, SGI0, SGI0, SGI123809, SGI123809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27055[] = {SGI123810, SGI123811, SGI123812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123813, SGI0, SGI0, SGI0, SGI123814, SGI123814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27056[] = {SGI123815, SGI123816, SGI123817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123818, SGI123818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27057[] = {SGI123819, SGI123820, SGI123821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123822, SGI0, SGI0, SGI0, SGI123823, SGI123823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27058[] = {SGI123824, SGI123825, SGI123826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123827, SGI0, SGI0, SGI0, SGI123828, SGI123828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27059[] = {SGI123829, SGI123830, SGI123831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123832, SGI0, SGI0, SGI0, SGI123833, SGI123833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27060[] = {SGI123834, SGI123835, SGI123836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123837, SGI0, SGI0, SGI0, SGI123838, SGI123838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27061[] = {SGI123839, SGI123840, SGI123841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123842, SGI0, SGI0, SGI0, SGI123843, SGI123843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27062[] = {SGI123844, SGI123845, SGI123846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123847, SGI0, SGI0, SGI0, SGI123848, SGI123848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27063[] = {SGI123849, SGI123850, SGI123851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123852, SGI0, SGI0, SGI0, SGI123853, SGI123853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27064[] = {SGI123854, SGI123855, SGI123856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123857, SGI0, SGI0, SGI0, SGI123858, SGI123858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27065[] = {SGI123859, SGI123860, SGI123861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123862, SGI0, SGI0, SGI0, SGI123863, SGI123863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27066[] = {SGI123864, SGI123865, SGI123866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123867, SGI0, SGI0, SGI0, SGI123868, SGI123868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27067[] = {SGI123869, SGI123870, SGI123871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123872, SGI0, SGI0, SGI0, SGI123873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123874, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27068[] = {SGI123875, SGI0, SGI123876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123877, SGI123878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27069[] = {SGI123879, SGI123880, SGI123881, SGI123882, SGI123883, SGI123883, SGI0, NULL};
const Segment * const SG27070[] = {SGI123884, SGI0, SGI123885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123886, SGI0, SGI0, SGI0, SGI123887, SGI123887, SGI0, SGI0, SGI123888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27071[] = {SGI123889, SGI123890, SGI123891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123892, SGI0, SGI0, SGI0, SGI123893, SGI123893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123894, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27072[] = {SGI123895, SGI0, SGI123896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123897, SGI123898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27073[] = {SGI123899, SGI123900, SGI123901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123902, SGI0, SGI0, SGI0, SGI123903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27074[] = {SGI123904, SGI123905, SGI123906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123907, SGI0, SGI0, SGI0, SGI123908, SGI123908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27075[] = {SGI123909, SGI123910, SGI123911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123912, SGI0, SGI0, SGI0, SGI123913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27076[] = {SGI123914, SGI0, SGI123915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27077[] = {SGI123916, SGI0, SGI123917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123918, SGI0, SGI0, SGI0, SGI123919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27078[] = {SGI123920, SGI123921, SGI123922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123924, SGI0, SGI0, SGI0, SGI123926, SGI123926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27079[] = {SGI123923, SGI0, SGI123925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123927, SGI123928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27080[] = {SGI123929, SGI123930, SGI123931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27081[] = {SGI123932, SGI123933, SGI123934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123937, SGI0, SGI0, SGI0, SGI123939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123940, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27082[] = {SGI123935, SGI123936, SGI123938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123941, SGI123941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27083[] = {SGI123942, SGI0, SGI123943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27084[] = {SGI123944, SGI123945, SGI123946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27085[] = {SGI123947, SGI123948, SGI123949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123950, SGI0, SGI0, SGI0, SGI123951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123952, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27086[] = {SGI123953, SGI123954, SGI123955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123956, SGI0, SGI0, SGI0, SGI123957, SGI123958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27087[] = {SGI123959, SGI123960, SGI123961, SGI123962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123963, SGI0, SGI0, SGI0, SGI123964, SGI123965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27088[] = {SGI123966, SGI123967, SGI123968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123969, SGI123969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27089[] = {SGI123970, SGI0, SGI123971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123972, SGI0, SGI0, SGI0, SGI123973, SGI123973, SGI0, SGI0, SGI123974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27090[] = {SGI123975, SGI123976, SGI123977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27091[] = {SGI123978, SGI123979, SGI123980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27092[] = {SGI123981, SGI123982, SGI123983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123984, SGI0, SGI0, SGI0, SGI123985, SGI123985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27093[] = {SGI123986, SGI123987, SGI123988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123989, SGI0, SGI0, SGI0, SGI123990, SGI123990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27094[] = {SGI123991, SGI123992, SGI123993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123995, SGI0, SGI0, SGI0, SGI123998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27095[] = {SGI123994, SGI123996, SGI123997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI123999, SGI0, SGI0, SGI0, SGI124000, SGI124000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124001, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27096[] = {SGI124002, SGI124003, SGI124004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124005, SGI0, SGI0, SGI0, SGI124006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27097[] = {SGI124007, SGI124008, SGI124009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27098[] = {SGI124011, SGI0, SGI124012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124013, SGI124014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27099[] = {SGI0, SGI124015, SGI124016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27100[] = {SGI124017, SGI124018, SGI124019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27101[] = {SGI124020, SGI124021, SGI124022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27102[] = {SGI124023, SGI124024, SGI124025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124026, SGI124026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27103[] = {SGI124027, SGI124028, SGI124029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124030, SGI0, SGI0, SGI0, SGI124031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27104[] = {SGI124032, SGI124033, SGI124034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124035, SGI0, SGI0, SGI0, SGI124036, SGI124036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27105[] = {SGI124037, SGI124038, SGI124039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124040, SGI0, SGI0, SGI0, SGI124041, SGI124042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27106[] = {SGI0, SGI124043, SGI124044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27107[] = {SGI124045, SGI124046, SGI124047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27108[] = {SGI124049, SGI124050, SGI124051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124052, SGI0, SGI0, SGI0, SGI124053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27109[] = {SGI124054, SGI124055, SGI124056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124057, SGI0, SGI124058, SGI124058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27110[] = {SGI124059, SGI0, SGI124060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124061, SGI0, SGI0, SGI0, SGI124062, SGI124062, SGI0, SGI0, SGI124063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27111[] = {SGI124064, SGI0, SGI124065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124066, SGI0, SGI0, SGI0, SGI124067, SGI124067, SGI0, SGI0, SGI124068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27112[] = {SGI124069, SGI0, SGI124070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124071, SGI0, SGI0, SGI0, SGI124072, SGI124072, SGI0, SGI0, SGI124073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27113[] = {SGI124074, SGI124075, SGI124076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124077, SGI0, SGI0, SGI0, SGI124078, SGI124078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124079, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27114[] = {SGI124080, SGI124081, SGI124082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124083, SGI0, SGI0, SGI0, SGI124084, SGI124084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27115[] = {SGI124085, SGI0, SGI124086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27116[] = {SGI124087, SGI124088, SGI124089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124090, SGI0, SGI0, SGI0, SGI124091, SGI124091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27117[] = {SGI124092, SGI124093, SGI124094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124095, SGI0, SGI0, SGI0, SGI124096, SGI124096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27118[] = {SGI124097, SGI124098, SGI124099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124100, SGI0, SGI0, SGI0, SGI124101, SGI124101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27119[] = {SGI124102, SGI124103, SGI124104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124105, SGI0, SGI0, SGI0, SGI124106, SGI124106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27120[] = {SGI124107, SGI124108, SGI124109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124110, SGI0, SGI0, SGI0, SGI124111, SGI124111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27121[] = {SGI124112, SGI124113, SGI124114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124115, SGI0, SGI0, SGI0, SGI124116, SGI124116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27122[] = {SGI124117, SGI124118, SGI124119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124120, SGI0, SGI0, SGI0, SGI124121, SGI124121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27123[] = {SGI124122, SGI124123, SGI124124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124125, SGI0, SGI0, SGI0, SGI124126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27124[] = {SGI124127, SGI124128, SGI124129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124130, SGI0, SGI0, SGI0, SGI124131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27125[] = {SGI124132, SGI124133, SGI124134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124135, SGI0, SGI0, SGI0, SGI124136, SGI124136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27126[] = {SGI124137, SGI124138, SGI124139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124140, SGI0, SGI0, SGI0, SGI124141, SGI124141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27127[] = {SGI124142, SGI124143, SGI124144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124145, SGI0, SGI0, SGI0, SGI124146, SGI124146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27128[] = {SGI124147, SGI124148, SGI124149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124150, SGI124150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27129[] = {SGI124151, SGI124152, SGI124153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124154, SGI0, SGI0, SGI0, SGI124155, SGI0, SGI0, SGI0, SGI0, SGI124156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27130[] = {SGI124157, SGI124158, SGI124159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124160, SGI0, SGI0, SGI0, SGI124161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124162, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27131[] = {SGI0, SGI124163, SGI0, SGI124164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27132[] = {SGI124165, SGI124166, SGI124167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124168, SGI0, SGI0, SGI0, SGI124169, SGI124169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27133[] = {SGI124170, SGI124171, SGI124172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124173, SGI0, SGI0, SGI0, SGI124174, SGI124174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27134[] = {SGI124175, SGI124176, SGI124177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124178, SGI0, SGI0, SGI0, SGI124179, SGI124179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27135[] = {SGI124180, SGI124181, SGI124182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124183, SGI0, SGI0, SGI0, SGI124184, SGI124184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27136[] = {SGI124185, SGI124186, SGI124187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124188, SGI0, SGI0, SGI0, SGI124189, SGI124189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27137[] = {SGI124190, SGI124191, SGI124192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124193, SGI0, SGI0, SGI0, SGI124194, SGI124194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27138[] = {SGI124195, SGI0, SGI124196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27139[] = {SGI124197, SGI124198, SGI124199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124200, SGI0, SGI0, SGI0, SGI124201, SGI124201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27140[] = {SGI124202, SGI0, SGI124203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27141[] = {SGI124204, SGI124205, SGI124206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124207, SGI0, SGI0, SGI0, SGI124208, SGI124208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27142[] = {SGI124209, SGI0, SGI124210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27143[] = {SGI124211, SGI0, SGI124212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27144[] = {SGI124213, SGI0, SGI124214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27145[] = {SGI124215, SGI124216, SGI124217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124218, SGI124218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27146[] = {SGI124219, SGI0, SGI124220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27147[] = {SGI124221, SGI0, SGI124222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124223, SGI0, SGI0, SGI0, SGI124224, SGI124224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27148[] = {SGI124225, SGI124226, SGI124227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124228, SGI0, SGI0, SGI0, SGI124229, SGI124229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27149[] = {SGI124230, SGI0, SGI124231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124232, SGI0, SGI0, SGI0, SGI124233, SGI124233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27150[] = {SGI124234, SGI0, SGI124235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27151[] = {SGI124236, SGI124237, SGI124238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124239, SGI124239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27152[] = {SGI124240, SGI124241, SGI124242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124243, SGI124243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27153[] = {SGI124244, SGI0, SGI124245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124246, SGI0, SGI0, SGI0, SGI124247, SGI124247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27154[] = {SGI124248, SGI124249, SGI124250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124251, SGI124251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27155[] = {SGI124252, SGI124253, SGI124254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124255, SGI124255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27156[] = {SGI124256, SGI124257, SGI124258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124259, SGI0, SGI0, SGI0, SGI124260, SGI124260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27157[] = {SGI124261, SGI124262, SGI124263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124264, SGI0, SGI0, SGI0, SGI124265, SGI124265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27158[] = {SGI124266, SGI0, SGI124267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27159[] = {SGI124268, SGI124269, SGI124270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124271, SGI124271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27160[] = {SGI124272, SGI124273, SGI124274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124275, SGI124275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27161[] = {SGI124276, SGI124277, SGI124278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124279, SGI0, SGI0, SGI0, SGI124280, SGI124280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27162[] = {SGI124281, SGI124282, SGI124283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124284, SGI0, SGI0, SGI0, SGI124285, SGI124285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27163[] = {SGI124286, SGI124287, SGI124288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124289, SGI0, SGI0, SGI0, SGI124290, SGI124290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27164[] = {SGI124291, SGI124292, SGI124293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124294, SGI0, SGI0, SGI0, SGI124295, SGI124295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27165[] = {SGI124296, SGI124297, SGI124298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124299, SGI0, SGI0, SGI0, SGI124300, SGI124300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27166[] = {SGI124301, SGI124302, SGI124303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124304, SGI0, SGI124305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27167[] = {SGI124306, SGI124307, SGI124308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124309, SGI0, SGI124310, SGI124310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27168[] = {SGI124311, SGI0, SGI124312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124313, SGI124314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124315, SGI0, NULL};
const Segment * const SG27169[] = {SGI124316, SGI124317, SGI124318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124319, SGI0, SGI0, SGI0, SGI124320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27170[] = {SGI124321, SGI0, SGI124322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27171[] = {SGI124323, SGI124324, SGI124325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124326, SGI0, SGI124327, SGI124327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27172[] = {SGI124328, SGI0, SGI124329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124330, SGI0, SGI0, SGI0, SGI124331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27173[] = {SGI124332, SGI124333, SGI124334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124335, SGI0, SGI124336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27174[] = {SGI124337, SGI124338, SGI124339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124340, SGI0, SGI0, SGI0, SGI124341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27175[] = {SGI124342, SGI124343, SGI124344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124345, SGI0, SGI0, SGI0, SGI124346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27176[] = {SGI124347, SGI124348, SGI124349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124350, SGI124350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27177[] = {SGI124351, SGI124352, SGI124353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124354, SGI0, SGI0, SGI0, SGI124355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27178[] = {SGI124356, SGI124357, SGI124358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124359, SGI0, SGI0, SGI0, SGI124360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27179[] = {SGI124361, SGI124362, SGI124363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124364, SGI124365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27180[] = {SGI124366, SGI124367, SGI124368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124369, SGI0, SGI0, SGI0, SGI124370, SGI124370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27181[] = {SGI124371, SGI124372, SGI124373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27182[] = {SGI124374, SGI124375, SGI124376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27183[] = {SGI124377, SGI124378, SGI124379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124380, SGI0, SGI0, SGI0, SGI124381, SGI124382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27184[] = {SGI124383, SGI124384, SGI124385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124386, SGI0, SGI0, SGI0, SGI124387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27185[] = {SGI124388, SGI124389, SGI124390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124391, SGI124391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27186[] = {SGI124392, SGI124393, SGI124394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124395, SGI0, SGI0, SGI0, SGI124396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27187[] = {SGI0, SGI124397, SGI124398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27188[] = {SGI124399, SGI124400, SGI124401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27189[] = {SGI124402, SGI124403, SGI124404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124405, SGI0, SGI0, SGI0, SGI124406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27190[] = {SGI124407, SGI124408, SGI124409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27191[] = {SGI124410, SGI124411, SGI124412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27192[] = {SGI124413, SGI124414, SGI124415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124416, SGI0, SGI0, SGI0, SGI124417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124418, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27193[] = {SGI124419, SGI124420, SGI124421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124422, SGI0, SGI0, SGI0, SGI124423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27194[] = {SGI124424, SGI124425, SGI124426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124427, SGI0, SGI0, SGI0, SGI124428, SGI124428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27195[] = {SGI124429, SGI124430, SGI124431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124432, SGI0, SGI0, SGI0, SGI124433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27196[] = {SGI124434, SGI124435, SGI124436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124437, SGI0, SGI0, SGI0, SGI124438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27197[] = {SGI124439, SGI124440, SGI124441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124444, SGI0, SGI0, SGI0, SGI124446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27198[] = {SGI124442, SGI124443, SGI124445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124448, SGI0, SGI0, SGI0, SGI124449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27199[] = {SGI124450, SGI0, SGI124451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27200[] = {SGI124453, SGI0, SGI124455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124457, SGI124457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27201[] = {SGI124452, SGI0, SGI124454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124456, SGI0, SGI0, SGI0, SGI124458, SGI124458, SGI0, SGI0, SGI124459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27202[] = {SGI124460, SGI0, SGI124461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27203[] = {SGI124462, SGI124463, SGI124464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27204[] = {SGI124465, SGI124466, SGI124467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27205[] = {SGI124468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27206[] = {SGI124469, SGI124470, SGI124471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27207[] = {SGI124472, SGI124473, SGI124474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124475, SGI124475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27208[] = {SGI124476, SGI124477, SGI124478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124479, SGI0, SGI0, SGI0, SGI124480, SGI124480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27209[] = {SGI124481, SGI124482, SGI124483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124484, SGI0, SGI0, SGI0, SGI124485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27210[] = {SGI124486, SGI124487, SGI124488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27211[] = {SGI124489, SGI124490, SGI124491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124492, SGI0, SGI0, SGI0, SGI124493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27212[] = {SGI124494, SGI124495, SGI124496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124497, SGI0, SGI0, SGI0, SGI124498, SGI124499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27213[] = {SGI124500, SGI124501, SGI124502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124503, SGI0, SGI0, SGI0, SGI124504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27214[] = {SGI124505, SGI124506, SGI124507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124508, SGI0, SGI0, SGI0, SGI124509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27215[] = {SGI124510, SGI124511, SGI124512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124513, SGI0, SGI0, SGI0, SGI0, SGI124514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27216[] = {SGI124515, SGI124516, SGI124517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27217[] = {SGI124518, SGI124519, SGI124520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124521, SGI0, SGI0, SGI0, SGI124522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27218[] = {SGI124523, SGI124524, SGI124525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124526, SGI0, SGI0, SGI0, SGI124527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27219[] = {SGI124528, SGI124529, SGI124530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124531, SGI0, SGI0, SGI0, SGI124532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27220[] = {SGI124533, SGI124534, SGI124535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27221[] = {SGI124537, SGI124538, SGI124539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27222[] = {SGI0, SGI124540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27223[] = {SGI124541, SGI124542, SGI124543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124544, SGI0, SGI0, SGI0, SGI124545, SGI124545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27224[] = {SGI124546, SGI124547, SGI124548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27225[] = {SGI124550, SGI0, SGI124551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124552, SGI0, SGI0, SGI0, SGI124553, SGI124553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27226[] = {SGI124554, SGI0, SGI124555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124556, SGI0, SGI0, SGI0, SGI124557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27227[] = {SGI124558, SGI124559, SGI124560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124561, SGI0, SGI0, SGI0, SGI124562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27228[] = {SGI124563, SGI124564, SGI124565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124566, SGI0, SGI0, SGI0, SGI124567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27229[] = {SGI124568, SGI124569, SGI124570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124571, SGI0, SGI0, SGI0, SGI124572, SGI124572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27230[] = {SGI124573, SGI124574, SGI124575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124576, SGI0, SGI0, SGI0, SGI124577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27231[] = {SGI124578, SGI124579, SGI124580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124581, SGI0, SGI0, SGI0, SGI124582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27232[] = {SGI124583, SGI0, SGI124584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124585, SGI0, SGI0, SGI0, SGI124586, SGI124586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27233[] = {SGI124587, SGI124588, SGI124589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27234[] = {SGI124590, SGI124591, SGI124592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124593, SGI0, SGI0, SGI0, SGI124594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27235[] = {SGI124595, SGI124596, SGI124597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124598, SGI0, SGI0, SGI0, SGI124599, SGI124599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27236[] = {SGI124600, SGI124601, SGI124602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124603, SGI0, SGI124604, SGI124604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27237[] = {SGI124605, SGI124606, SGI124607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124608, SGI0, SGI0, SGI0, SGI124609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124610, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27238[] = {SGI124611, SGI0, SGI124612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124613, SGI0, SGI0, SGI0, SGI124614, SGI124614, SGI0, SGI0, SGI124615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27239[] = {SGI124616, SGI124617, SGI124618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124619, SGI0, SGI0, SGI0, SGI124620, SGI124620, SGI0, SGI0, NULL};
const Segment * const SG27240[] = {SGI124621, SGI124622, SGI124623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124624, SGI0, SGI0, SGI0, SGI124625, SGI124625, SGI0, SGI0, NULL};
const Segment * const SG27241[] = {SGI124626, SGI124627, SGI124628, SGI0, SGI0, SGI0, SGI124629, SGI0, SGI0, SGI124630, SGI0, SGI0, SGI0, SGI124631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27242[] = {SGI124632, SGI124633, SGI124634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124636, SGI0, SGI0, SGI0, SGI124638, SGI124638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27243[] = {SGI124635, SGI0, SGI124637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124639, SGI0, SGI0, SGI0, SGI124640, SGI124640, SGI0, SGI0, SGI124641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27244[] = {SGI124642, SGI124643, SGI124644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124645, SGI0, SGI0, SGI0, SGI124646, SGI124646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27245[] = {SGI124647, SGI124648, SGI124649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124653, SGI0, SGI0, SGI0, SGI124654, SGI124654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27246[] = {SGI124650, SGI124651, SGI124652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124655, SGI0, SGI124656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27247[] = {SGI124657, SGI124658, SGI124659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124660, SGI0, SGI0, SGI0, SGI124661, SGI124661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27248[] = {SGI124662, SGI124663, SGI124664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124665, SGI0, SGI0, SGI0, SGI124666, SGI124666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27249[] = {SGI124667, SGI124668, SGI124669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124670, SGI0, SGI0, SGI0, SGI124671, SGI0, SGI0, SGI0, SGI124672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27250[] = {SGI124673, SGI124674, SGI124675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124676, SGI0, SGI0, SGI0, SGI124677, SGI124677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27251[] = {SGI124678, SGI124679, SGI124680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124683, SGI0, SGI0, SGI0, SGI124685, SGI124685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27252[] = {SGI124681, SGI124682, SGI124684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124686, SGI0, SGI0, SGI0, SGI124687, SGI124687, SGI0, SGI0, NULL};
const Segment * const SG27253[] = {SGI124688, SGI124689, SGI124690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124691, SGI0, SGI0, SGI0, SGI124692, SGI124692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27254[] = {SGI124693, SGI124694, SGI124695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124696, SGI0, SGI0, SGI0, SGI124697, SGI124697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27255[] = {SGI124698, SGI124699, SGI124700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124701, SGI0, SGI0, SGI0, SGI124702, SGI124702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27256[] = {SGI124703, SGI124704, SGI124705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124706, SGI0, SGI0, SGI0, SGI124707, SGI124707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27257[] = {SGI124710, SGI0, SGI124712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124714, SGI124716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27258[] = {SGI124708, SGI124709, SGI124711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124713, SGI0, SGI0, SGI0, SGI124715, SGI124715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27259[] = {SGI124717, SGI124718, SGI124719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124720, SGI0, SGI0, SGI0, SGI124721, SGI124721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27260[] = {SGI124722, SGI124723, SGI124724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124725, SGI0, SGI0, SGI0, SGI124726, SGI124726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27261[] = {SGI124727, SGI124728, SGI124729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124730, SGI0, SGI0, SGI0, SGI124731, SGI124731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27262[] = {SGI124732, SGI124733, SGI124734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124735, SGI0, SGI0, SGI0, SGI124736, SGI124736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27263[] = {SGI124737, SGI124738, SGI124739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124740, SGI0, SGI0, SGI0, SGI124741, SGI124741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27264[] = {SGI124742, SGI124743, SGI124744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124745, SGI0, SGI0, SGI0, SGI124746, SGI124746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27265[] = {SGI124747, SGI0, SGI124748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27266[] = {SGI124749, SGI124750, SGI124751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124752, SGI0, SGI0, SGI0, SGI124753, SGI124753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27267[] = {SGI124754, SGI124755, SGI124756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124757, SGI0, SGI0, SGI0, SGI124758, SGI124758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27268[] = {SGI124759, SGI0, SGI124760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27269[] = {SGI124761, SGI124762, SGI124763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124764, SGI0, SGI0, SGI0, SGI124765, SGI124765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27270[] = {SGI124766, SGI124767, SGI124768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124769, SGI0, SGI0, SGI0, SGI124770, SGI124770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27271[] = {SGI124771, SGI124772, SGI124773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124774, SGI0, SGI0, SGI0, SGI124775, SGI124775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27272[] = {SGI124776, SGI124777, SGI124778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124779, SGI0, SGI0, SGI0, SGI124780, SGI124780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27273[] = {SGI124781, SGI124782, SGI124783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124784, SGI0, SGI0, SGI0, SGI124785, SGI124785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27274[] = {SGI124786, SGI0, SGI124787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27275[] = {SGI124788, SGI124789, SGI124790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124791, SGI0, SGI0, SGI0, SGI124792, SGI124792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27276[] = {SGI124793, SGI124794, SGI124795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124796, SGI0, SGI0, SGI0, SGI124797, SGI124797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27277[] = {SGI124798, SGI124799, SGI124800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124801, SGI0, SGI0, SGI0, SGI124802, SGI124802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27278[] = {SGI124803, SGI124804, SGI124805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124806, SGI0, SGI0, SGI0, SGI124807, SGI124807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27279[] = {SGI124808, SGI124809, SGI124810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124811, SGI0, SGI0, SGI0, SGI124812, SGI124812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27280[] = {SGI124813, SGI124814, SGI124815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124816, SGI0, SGI0, SGI0, SGI124817, SGI124817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27281[] = {SGI124818, SGI124819, SGI124820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124821, SGI0, SGI0, SGI0, SGI124822, SGI124822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27282[] = {SGI124823, SGI124824, SGI124825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124826, SGI0, SGI0, SGI0, SGI124827, SGI124827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27283[] = {SGI124828, SGI124829, SGI124830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124831, SGI0, SGI0, SGI0, SGI124832, SGI124832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27284[] = {SGI124833, SGI124834, SGI124835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124836, SGI0, SGI0, SGI0, SGI124837, SGI124837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27285[] = {SGI124838, SGI0, SGI124839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124840, SGI0, SGI0, SGI0, SGI124841, SGI124841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27286[] = {SGI124842, SGI124843, SGI124844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124845, SGI0, SGI0, SGI0, SGI124846, SGI124846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27287[] = {SGI124847, SGI124848, SGI124849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124850, SGI0, SGI0, SGI0, SGI124851, SGI124851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27288[] = {SGI124852, SGI0, SGI124853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124854, SGI0, SGI0, SGI0, SGI124855, SGI124855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27289[] = {SGI124856, SGI124857, SGI124858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124859, SGI0, SGI0, SGI0, SGI124860, SGI124860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27290[] = {SGI124861, SGI0, SGI124862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27291[] = {SGI124863, SGI124864, SGI124865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124866, SGI0, SGI124867, SGI124867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27292[] = {SGI124868, SGI124869, SGI124870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124871, SGI0, SGI0, SGI0, SGI124872, SGI124872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27293[] = {SGI124873, SGI124874, SGI124875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124876, SGI0, SGI124877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27294[] = {SGI124878, SGI0, SGI124879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124880, SGI0, SGI0, SGI0, SGI124881, SGI124881, SGI0, SGI0, SGI124882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27295[] = {SGI124883, SGI124884, SGI124885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124886, SGI0, SGI0, SGI0, SGI124887, SGI124887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27296[] = {SGI124888, SGI124889, SGI124890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124891, SGI0, SGI0, SGI0, SGI124892, SGI124892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27297[] = {SGI124893, SGI124894, SGI124895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124896, SGI0, SGI0, SGI0, SGI124897, SGI124897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27298[] = {SGI124898, SGI124899, SGI124900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124901, SGI0, SGI0, SGI0, SGI124902, SGI124902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27299[] = {SGI124903, SGI124904, SGI124905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124906, SGI0, SGI0, SGI0, SGI124907, SGI124907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27300[] = {SGI124908, SGI124909, SGI124910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124911, SGI0, SGI0, SGI0, SGI124912, SGI124912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27301[] = {SGI124913, SGI124914, SGI124915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124916, SGI0, SGI0, SGI0, SGI124917, SGI124917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27302[] = {SGI124918, SGI124919, SGI124920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124921, SGI0, SGI0, SGI0, SGI124922, SGI124922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27303[] = {SGI124923, SGI124924, SGI124925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124926, SGI0, SGI0, SGI0, SGI124927, SGI124927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27304[] = {SGI124928, SGI124929, SGI124930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124931, SGI0, SGI0, SGI0, SGI124932, SGI124932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27305[] = {SGI124933, SGI0, SGI124934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27306[] = {SGI124935, SGI124936, SGI124937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124938, SGI0, SGI0, SGI0, SGI124939, SGI124939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27307[] = {SGI124940, SGI0, SGI124941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27308[] = {SGI124942, SGI124943, SGI124944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124945, SGI0, SGI0, SGI0, SGI124946, SGI124946, SGI0, SGI0, NULL};
const Segment * const SG27309[] = {SGI124947, SGI124948, SGI124949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124950, SGI0, SGI0, SGI0, SGI124951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124952, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27310[] = {SGI124953, SGI124954, SGI124955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124956, SGI0, SGI124957, SGI124957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27311[] = {SGI124958, SGI124959, SGI124960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124961, SGI0, SGI0, SGI0, SGI124962, SGI124962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27312[] = {SGI0, SGI124963, SGI0, SGI124964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27313[] = {SGI124965, SGI124966, SGI124967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124968, SGI0, SGI0, SGI0, SGI124969, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27314[] = {SGI124970, SGI124971, SGI124972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124973, SGI124973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124974, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27315[] = {SGI124975, SGI124976, SGI124977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124978, SGI0, SGI0, SGI0, SGI0, SGI124979, SGI0, SGI0, NULL};
const Segment * const SG27316[] = {SGI124980, SGI124981, SGI124982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124983, SGI124983, SGI0, SGI0, SGI124984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27317[] = {SGI124985, SGI124986, SGI124987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124988, SGI0, SGI0, SGI0, SGI124989, SGI124989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27318[] = {SGI124990, SGI0, SGI124991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27319[] = {SGI124992, SGI124993, SGI124994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI124995, SGI0, SGI0, SGI0, SGI124996, SGI124996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27320[] = {SGI124997, SGI124998, SGI124999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125000, SGI0, SGI0, SGI0, SGI125001, SGI125001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27321[] = {SGI125002, SGI0, SGI125003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27322[] = {SGI0, SGI125004, SGI125005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125006, SGI0, SGI0, SGI0, SGI125007, SGI125007, SGI0, SGI0, NULL};
const Segment * const SG27323[] = {SGI125008, SGI125009, SGI125010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125011, SGI0, SGI0, SGI0, SGI125012, SGI125012, SGI0, SGI0, NULL};
const Segment * const SG27324[] = {SGI125013, SGI125014, SGI125015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125016, SGI0, SGI0, SGI0, SGI125017, SGI125017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27325[] = {SGI125018, SGI125019, SGI125020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125021, SGI0, SGI0, SGI0, SGI125022, SGI125022, SGI125023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27326[] = {SGI125024, SGI0, SGI125025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125026, SGI0, SGI0, SGI0, SGI125027, SGI125027, SGI0, SGI0, SGI125028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27327[] = {SGI125029, SGI125030, SGI125031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125032, SGI0, SGI0, SGI0, SGI125033, SGI125033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125034, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27328[] = {SGI125035, SGI125036, SGI125037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125038, SGI0, SGI0, SGI0, SGI125039, SGI125039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125040, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27329[] = {SGI125041, SGI125042, SGI125043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125044, SGI0, SGI0, SGI0, SGI125045, SGI125045, SGI0, SGI0, NULL};
const Segment * const SG27330[] = {SGI125046, SGI125047, SGI125048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125049, SGI0, SGI0, SGI0, SGI125050, SGI125050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27331[] = {SGI125051, SGI125052, SGI125053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125054, SGI0, SGI0, SGI0, SGI0, SGI125055, SGI0, SGI0, NULL};
const Segment * const SG27332[] = {SGI125056, SGI125057, SGI125058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125059, SGI0, SGI0, SGI0, SGI125060, SGI125060, SGI0, SGI0, NULL};
const Segment * const SG27333[] = {SGI125061, SGI125062, SGI125063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125064, SGI125064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27334[] = {SGI125065, SGI125066, SGI125067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125068, SGI0, SGI0, SGI0, SGI125069, SGI125069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27335[] = {SGI125070, SGI125071, SGI125072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125073, SGI0, SGI0, SGI0, SGI125074, SGI125074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27336[] = {SGI125075, SGI125076, SGI125077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125078, SGI0, SGI0, SGI0, SGI125079, SGI125079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27337[] = {SGI125080, SGI125081, SGI125082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125083, SGI0, SGI0, SGI0, SGI125084, SGI125084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27338[] = {SGI125085, SGI125086, SGI125087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125088, SGI0, SGI0, SGI0, SGI125089, SGI125089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27339[] = {SGI125090, SGI125091, SGI125092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125093, SGI0, SGI0, SGI0, SGI125094, SGI125094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27340[] = {SGI125095, SGI0, SGI125096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27341[] = {SGI125097, SGI125098, SGI125099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125100, SGI0, SGI0, SGI0, SGI125101, SGI125101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27342[] = {SGI125102, SGI125103, SGI125104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125105, SGI0, SGI0, SGI0, SGI125106, SGI125106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27343[] = {SGI125107, SGI0, SGI125108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27344[] = {SGI125109, SGI125110, SGI125111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125112, SGI0, SGI0, SGI0, SGI125113, SGI125113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27345[] = {SGI125114, SGI0, SGI125115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125116, SGI0, SGI0, SGI0, SGI125117, SGI125117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27346[] = {SGI125118, SGI125119, SGI125120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125121, SGI0, SGI0, SGI0, SGI125122, SGI125122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27347[] = {SGI125123, SGI125124, SGI125125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125126, SGI0, SGI0, SGI0, SGI125127, SGI125127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27348[] = {SGI125128, SGI125129, SGI125130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125131, SGI0, SGI0, SGI0, SGI125132, SGI125132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27349[] = {SGI125133, SGI125134, SGI125135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125136, SGI0, SGI0, SGI0, SGI125137, SGI125137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27350[] = {SGI125138, SGI125139, SGI125140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125141, SGI0, SGI0, SGI0, SGI125142, SGI125142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27351[] = {SGI125143, SGI125144, SGI125145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125146, SGI0, SGI0, SGI0, SGI125147, SGI125147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27352[] = {SGI125148, SGI125149, SGI125150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125151, SGI0, SGI0, SGI0, SGI125152, SGI125152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27353[] = {SGI125153, SGI125154, SGI125155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125156, SGI125156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27354[] = {SGI125157, SGI125158, SGI125159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125160, SGI0, SGI0, SGI0, SGI125161, SGI125161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27355[] = {SGI125162, SGI125163, SGI125164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125165, SGI0, SGI0, SGI0, SGI125166, SGI125166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27356[] = {SGI125167, SGI125168, SGI125169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125170, SGI0, SGI0, SGI0, SGI125171, SGI125171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27357[] = {SGI125172, SGI0, SGI125173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125174, SGI0, SGI0, SGI0, SGI125175, SGI125175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27358[] = {SGI125176, SGI125177, SGI125178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125179, SGI0, SGI0, SGI0, SGI125180, SGI125180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27359[] = {SGI125181, SGI125182, SGI125183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125184, SGI0, SGI0, SGI0, SGI125185, SGI125185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27360[] = {SGI125186, SGI125187, SGI125188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125189, SGI0, SGI0, SGI0, SGI125190, SGI125190, SGI0, SGI0, NULL};
const Segment * const SG27361[] = {SGI125191, SGI125192, SGI125193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125194, SGI0, SGI0, SGI0, SGI125195, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27362[] = {SGI125196, SGI125197, SGI125198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125199, SGI0, SGI125200, SGI125200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27363[] = {SGI125201, SGI125202, SGI125203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125204, SGI0, SGI125205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27364[] = {SGI125206, SGI125207, SGI125208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125209, SGI0, SGI0, SGI0, SGI125210, SGI125210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27365[] = {SGI125211, SGI125212, SGI125213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125214, SGI0, SGI0, SGI0, SGI125215, SGI0, SGI125216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27366[] = {SGI125217, SGI125218, SGI125219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125220, SGI125220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125221, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27367[] = {SGI125222, SGI125223, SGI125224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125225, SGI0, SGI0, SGI0, SGI125226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125227, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27368[] = {SGI125228, SGI0, SGI125229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125230, SGI0, SGI0, SGI0, SGI125231, SGI125231, SGI0, SGI0, SGI125232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27369[] = {SGI125233, SGI125234, SGI125235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125236, SGI0, SGI0, SGI0, SGI125237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125238, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27370[] = {SGI125239, SGI125240, SGI125241, SGI0, SGI0, SGI0, SGI125242, SGI0, SGI0, SGI125243, SGI0, SGI0, SGI0, SGI125244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27371[] = {SGI125248, SGI125249, SGI125250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125251, SGI0, SGI0, SGI0, SGI0, SGI125252, SGI0, SGI0, NULL};
const Segment * const SG27372[] = {SGI125253, SGI125254, SGI125255, SGI125256, SGI125257, SGI125257, SGI0, NULL};
const Segment * const SG27373[] = {SGI125258, SGI125259, SGI125260, SGI125261, SGI125262, SGI125262, SGI0, NULL};
const Segment * const SG27374[] = {SGI125245, SGI125246, SGI125247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125263, SGI0, SGI0, SGI0, SGI125264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27375[] = {SGI125266, SGI0, SGI125267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125268, SGI0, SGI0, SGI0, SGI125269, SGI125269, SGI0, SGI0, SGI125270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27376[] = {SGI125271, SGI125272, SGI125273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125274, SGI125274, SGI0, SGI0, SGI125275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27377[] = {SGI125276, SGI0, SGI125277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125278, SGI0, SGI0, SGI0, SGI125279, SGI125279, SGI0, SGI0, SGI125280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27378[] = {SGI125281, SGI125282, SGI125283, SGI125284, SGI125285, SGI125285, SGI0, NULL};
const Segment * const SG27379[] = {SGI125286, SGI125287, SGI125288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125289, SGI0, SGI0, SGI0, SGI125290, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27380[] = {SGI125291, SGI125292, SGI125293, SGI125294, SGI125295, SGI125295, SGI0, NULL};
const Segment * const SG27381[] = {SGI125296, SGI125297, SGI125298, SGI125299, SGI125300, SGI125300, SGI0, NULL};
const Segment * const SG27382[] = {SGI125301, SGI125302, SGI125303, SGI125304, SGI125305, SGI125305, SGI0, NULL};
const Segment * const SG27383[] = {SGI125306, SGI125307, SGI125308, SGI125309, SGI125310, SGI125310, SGI0, NULL};
const Segment * const SG27384[] = {SGI125311, SGI125312, SGI125313, SGI125314, SGI125315, SGI125315, SGI0, NULL};
const Segment * const SG27385[] = {SGI125316, SGI125317, SGI125318, SGI125319, SGI125320, SGI125320, SGI0, NULL};
const Segment * const SG27386[] = {SGI125321, SGI125322, SGI125323, SGI125324, SGI125325, SGI125325, SGI0, NULL};
const Segment * const SG27387[] = {SGI125326, SGI125327, SGI125328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125329, SGI0, SGI125330, SGI125330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27388[] = {SGI125331, SGI0, SGI125332, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27389[] = {SGI125333, SGI0, SGI125334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125335, SGI0, SGI0, SGI0, SGI125336, SGI125336, SGI0, SGI0, SGI125337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27390[] = {SGI125338, SGI125339, SGI125340, SGI125341, SGI125342, SGI125342, SGI0, NULL};
const Segment * const SG27391[] = {SGI125343, SGI125344, SGI125345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125348, SGI0, SGI0, SGI0, SGI125350, SGI125350, SGI0, SGI0, NULL};
const Segment * const SG27392[] = {SGI125346, SGI125347, SGI125349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125351, SGI0, SGI0, SGI0, SGI125352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125353, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27393[] = {SGI125354, SGI0, SGI125355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125356, SGI0, SGI0, SGI0, SGI125357, SGI125357, SGI0, SGI0, SGI125358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27394[] = {SGI125359, SGI125360, SGI125361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125362, SGI0, SGI0, SGI0, SGI125363, SGI125363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125364, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27395[] = {SGI125365, SGI125366, SGI125367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125368, SGI0, SGI0, SGI0, SGI125369, SGI125369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27396[] = {SGI125370, SGI125371, SGI125372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125373, SGI0, SGI0, SGI0, SGI125374, SGI125374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27397[] = {SGI125375, SGI125376, SGI125377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125378, SGI0, SGI0, SGI0, SGI125379, SGI125379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27398[] = {SGI125380, SGI125381, SGI125382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125383, SGI0, SGI0, SGI0, SGI125384, SGI125384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27399[] = {SGI125385, SGI125386, SGI125387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125388, SGI0, SGI0, SGI0, SGI125389, SGI125389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27400[] = {SGI125390, SGI125391, SGI125392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125393, SGI0, SGI0, SGI0, SGI125394, SGI125394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27401[] = {SGI125395, SGI125396, SGI125397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125398, SGI0, SGI0, SGI0, SGI125399, SGI125399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27402[] = {SGI125400, SGI125401, SGI125402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125403, SGI0, SGI0, SGI0, SGI125404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27403[] = {SGI125405, SGI125406, SGI125407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125408, SGI0, SGI0, SGI0, SGI125409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27404[] = {SGI125410, SGI125411, SGI125412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125413, SGI0, SGI0, SGI0, SGI125414, SGI125414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27405[] = {SGI125415, SGI125416, SGI125417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125418, SGI0, SGI0, SGI0, SGI125419, SGI125419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27406[] = {SGI125420, SGI125421, SGI125422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125423, SGI0, SGI0, SGI0, SGI125424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27407[] = {SGI125425, SGI125426, SGI125427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125428, SGI0, SGI0, SGI0, SGI125429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27408[] = {SGI125430, SGI125431, SGI125432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125433, SGI0, SGI0, SGI0, SGI125434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27409[] = {SGI125435, SGI125436, SGI125437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125438, SGI0, SGI0, SGI0, SGI125439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27410[] = {SGI125440, SGI125441, SGI125442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125443, SGI0, SGI0, SGI0, SGI125444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27411[] = {SGI125445, SGI125446, SGI125447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125448, SGI0, SGI0, SGI0, SGI125449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27412[] = {SGI125450, SGI125451, SGI125452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125453, SGI0, SGI0, SGI0, SGI125454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27413[] = {SGI125455, SGI125456, SGI125457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125458, SGI0, SGI0, SGI0, SGI125459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27414[] = {SGI125460, SGI125461, SGI125462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125463, SGI0, SGI0, SGI0, SGI125464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27415[] = {SGI125465, SGI125466, SGI125467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125468, SGI0, SGI0, SGI0, SGI125469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27416[] = {SGI125470, SGI125471, SGI125472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125473, SGI0, SGI0, SGI0, SGI125474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27417[] = {SGI125475, SGI125476, SGI125477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125478, SGI0, SGI0, SGI0, SGI125479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27418[] = {SGI125480, SGI125481, SGI125482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125483, SGI0, SGI0, SGI0, SGI125484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27419[] = {SGI125485, SGI125486, SGI125487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125488, SGI0, SGI0, SGI0, SGI125489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27420[] = {SGI125490, SGI125491, SGI125492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125493, SGI0, SGI0, SGI0, SGI125494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27421[] = {SGI125495, SGI125496, SGI125497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125498, SGI0, SGI0, SGI0, SGI125499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27422[] = {SGI125500, SGI125501, SGI125502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125503, SGI0, SGI0, SGI0, SGI125504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27423[] = {SGI125505, SGI125506, SGI125507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125508, SGI0, SGI0, SGI0, SGI125509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27424[] = {SGI125510, SGI125511, SGI125512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125513, SGI0, SGI0, SGI0, SGI125514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27425[] = {SGI125515, SGI125516, SGI125517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125518, SGI0, SGI0, SGI0, SGI125519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27426[] = {SGI125520, SGI125521, SGI125522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125523, SGI0, SGI0, SGI0, SGI125524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27427[] = {SGI125525, SGI125526, SGI125527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125528, SGI0, SGI0, SGI0, SGI125529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27428[] = {SGI125530, SGI125531, SGI125532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125533, SGI0, SGI0, SGI0, SGI125534, SGI125534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27429[] = {SGI125535, SGI125536, SGI125537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125538, SGI0, SGI0, SGI0, SGI125539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27430[] = {SGI125540, SGI125541, SGI125542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125543, SGI0, SGI0, SGI0, SGI125544, SGI125544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27431[] = {SGI125545, SGI125546, SGI125547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125548, SGI0, SGI0, SGI0, SGI125549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27432[] = {SGI125550, SGI125551, SGI125552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125553, SGI0, SGI0, SGI0, SGI125554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27433[] = {SGI125555, SGI125556, SGI125557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125558, SGI0, SGI0, SGI0, SGI125559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27434[] = {SGI125560, SGI125561, SGI125562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125563, SGI0, SGI0, SGI0, SGI125564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27435[] = {SGI125565, SGI125566, SGI125567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125568, SGI0, SGI0, SGI0, SGI125569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27436[] = {SGI125570, SGI125571, SGI125572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125573, SGI0, SGI0, SGI0, SGI125574, SGI125574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27437[] = {SGI125575, SGI125576, SGI125577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125578, SGI0, SGI0, SGI0, SGI125579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27438[] = {SGI125580, SGI125581, SGI125582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125583, SGI0, SGI0, SGI0, SGI125585, SGI125585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27439[] = {SGI125584, SGI125586, SGI125587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125588, SGI0, SGI0, SGI0, SGI125589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27440[] = {SGI125590, SGI125591, SGI125592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125593, SGI0, SGI0, SGI0, SGI125596, SGI125596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27441[] = {SGI125594, SGI125595, SGI125597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125598, SGI0, SGI0, SGI0, SGI125599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27442[] = {SGI125600, SGI125601, SGI125602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125606, SGI125606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27443[] = {SGI125603, SGI125604, SGI125605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125607, SGI0, SGI0, SGI0, SGI125608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27444[] = {SGI125609, SGI125610, SGI125612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125615, SGI0, SGI0, SGI0, SGI125617, SGI125617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27445[] = {SGI125611, SGI125613, SGI125614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125616, SGI0, SGI0, SGI0, SGI125618, SGI125618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27446[] = {SGI125619, SGI125620, SGI125621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125622, SGI0, SGI0, SGI0, SGI125623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27447[] = {SGI125624, SGI125625, SGI125626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125627, SGI0, SGI0, SGI0, SGI125628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27448[] = {SGI125630, SGI0, SGI125632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27449[] = {SGI125629, SGI125631, SGI125633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125634, SGI0, SGI0, SGI0, SGI125635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27450[] = {SGI125636, SGI125637, SGI125638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125641, SGI0, SGI0, SGI0, SGI125643, SGI125643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27451[] = {SGI125639, SGI125640, SGI125642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125644, SGI0, SGI0, SGI0, SGI125645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27452[] = {SGI125646, SGI125647, SGI125648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125649, SGI125649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27453[] = {SGI125650, SGI125651, SGI125652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125653, SGI0, SGI0, SGI0, SGI125654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27454[] = {SGI125655, SGI125656, SGI125657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125658, SGI0, SGI0, SGI0, SGI125659, SGI125659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27455[] = {SGI125660, SGI125661, SGI125662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125663, SGI0, SGI0, SGI0, SGI125664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27456[] = {SGI125665, SGI0, SGI125666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27457[] = {SGI125667, SGI125668, SGI125669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125673, SGI0, SGI0, SGI0, SGI125674, SGI125674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27458[] = {SGI125670, SGI125671, SGI125672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125675, SGI0, SGI0, SGI0, SGI125676, SGI125676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27459[] = {SGI125677, SGI125678, SGI125680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125683, SGI0, SGI0, SGI0, SGI125685, SGI125685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27460[] = {SGI125679, SGI125681, SGI125682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125684, SGI0, SGI0, SGI0, SGI125686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27461[] = {SGI125688, SGI125691, SGI125692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125694, SGI0, SGI0, SGI0, SGI125696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27462[] = {SGI125687, SGI125689, SGI125690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125693, SGI0, SGI0, SGI0, SGI125695, SGI125695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27463[] = {SGI125698, SGI125699, SGI125701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125703, SGI0, SGI0, SGI0, SGI125705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27464[] = {SGI125697, SGI0, SGI125700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125702, SGI0, SGI0, SGI0, SGI125704, SGI125704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27465[] = {SGI125706, SGI125707, SGI125708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125709, SGI0, SGI0, SGI0, SGI125710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27466[] = {SGI125711, SGI125712, SGI125713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125714, SGI0, SGI0, SGI0, SGI125715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27467[] = {SGI125716, SGI125717, SGI125718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125719, SGI0, SGI0, SGI0, SGI125720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27468[] = {SGI125721, SGI125722, SGI125723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125724, SGI0, SGI0, SGI0, SGI125725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27469[] = {SGI125726, SGI125727, SGI125728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125729, SGI0, SGI0, SGI0, SGI125730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27470[] = {SGI125731, SGI125732, SGI125733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125734, SGI0, SGI0, SGI0, SGI125735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27471[] = {SGI125736, SGI125737, SGI125738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125739, SGI0, SGI0, SGI0, SGI125740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27472[] = {SGI125741, SGI125742, SGI125743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125744, SGI0, SGI0, SGI0, SGI125745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27473[] = {SGI125746, SGI125747, SGI125748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125749, SGI0, SGI0, SGI0, SGI125750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27474[] = {SGI125751, SGI125752, SGI125753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125754, SGI0, SGI0, SGI0, SGI125755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27475[] = {SGI125756, SGI125757, SGI125758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125759, SGI0, SGI0, SGI0, SGI125760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27476[] = {SGI125761, SGI125762, SGI125763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125764, SGI0, SGI0, SGI0, SGI125765, SGI125765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27477[] = {SGI125766, SGI125767, SGI125768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125769, SGI0, SGI0, SGI0, SGI125770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27478[] = {SGI125771, SGI125772, SGI125773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125774, SGI0, SGI0, SGI0, SGI125775, SGI125775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27479[] = {SGI125776, SGI125777, SGI125778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125779, SGI0, SGI0, SGI0, SGI125780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27480[] = {SGI125781, SGI125782, SGI125783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125784, SGI0, SGI0, SGI0, SGI125785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27481[] = {SGI125786, SGI125787, SGI125788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125789, SGI0, SGI0, SGI0, SGI125790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27482[] = {SGI125791, SGI125792, SGI125793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125794, SGI0, SGI0, SGI0, SGI125795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27483[] = {SGI125796, SGI125797, SGI125798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125799, SGI0, SGI0, SGI0, SGI125800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27484[] = {SGI125801, SGI125802, SGI125803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125804, SGI0, SGI0, SGI0, SGI125805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27485[] = {SGI125806, SGI125807, SGI125808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125809, SGI0, SGI0, SGI0, SGI125810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27486[] = {SGI125811, SGI125812, SGI125813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125814, SGI0, SGI0, SGI0, SGI125815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27487[] = {SGI125816, SGI125817, SGI125818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125819, SGI0, SGI0, SGI0, SGI125820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27488[] = {SGI125821, SGI125822, SGI125823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125824, SGI0, SGI0, SGI0, SGI125825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27489[] = {SGI125826, SGI125827, SGI125828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125830, SGI0, SGI0, SGI0, SGI125832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27490[] = {SGI125829, SGI125831, SGI125833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125834, SGI0, SGI0, SGI0, SGI125835, SGI125835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27491[] = {SGI125836, SGI0, SGI125837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27492[] = {SGI125838, SGI125839, SGI125840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125841, SGI0, SGI0, SGI0, SGI125842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27493[] = {SGI125843, SGI125844, SGI125845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125846, SGI0, SGI0, SGI0, SGI125847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27494[] = {SGI125848, SGI125849, SGI125850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125851, SGI0, SGI0, SGI0, SGI125852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27495[] = {SGI125853, SGI125854, SGI125855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125856, SGI0, SGI0, SGI0, SGI125857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27496[] = {SGI125858, SGI125859, SGI125860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125861, SGI0, SGI0, SGI0, SGI125862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27497[] = {SGI125863, SGI125864, SGI125866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125869, SGI0, SGI125871, SGI125871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27498[] = {SGI125865, SGI125867, SGI125868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125870, SGI0, SGI0, SGI0, SGI125872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27499[] = {SGI125873, SGI125874, SGI125875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125876, SGI0, SGI0, SGI0, SGI125877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27500[] = {SGI125878, SGI125879, SGI125880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125881, SGI0, SGI0, SGI0, SGI125882, SGI125882, SGI125883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27501[] = {SGI125884, SGI125885, SGI125886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125887, SGI0, SGI0, SGI0, SGI125888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27502[] = {SGI125889, SGI125890, SGI125891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125892, SGI0, SGI0, SGI0, SGI125893, SGI125893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27503[] = {SGI125894, SGI125895, SGI125896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125897, SGI0, SGI0, SGI0, SGI125898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27504[] = {SGI125899, SGI125900, SGI125901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125902, SGI0, SGI0, SGI0, SGI125903, SGI125903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27505[] = {SGI125904, SGI125905, SGI125906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125907, SGI0, SGI0, SGI0, SGI125908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27506[] = {SGI125909, SGI125910, SGI125911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125912, SGI0, SGI0, SGI0, SGI125913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27507[] = {SGI125914, SGI125915, SGI125916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125917, SGI0, SGI0, SGI0, SGI125918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27508[] = {SGI125919, SGI125920, SGI125921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125922, SGI0, SGI0, SGI0, SGI125923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27509[] = {SGI125924, SGI125925, SGI125926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125927, SGI0, SGI0, SGI0, SGI125928, SGI125928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27510[] = {SGI125929, SGI125930, SGI125931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125932, SGI0, SGI0, SGI0, SGI125933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27511[] = {SGI125934, SGI125935, SGI125936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125937, SGI0, SGI0, SGI0, SGI125938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27512[] = {SGI125939, SGI125940, SGI125941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125942, SGI0, SGI0, SGI0, SGI125943, SGI125943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27513[] = {SGI125944, SGI125945, SGI125946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125947, SGI0, SGI0, SGI0, SGI125948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27514[] = {SGI125949, SGI125950, SGI125951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125952, SGI0, SGI0, SGI0, SGI125953, SGI125953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27515[] = {SGI125954, SGI125955, SGI125956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125958, SGI0, SGI0, SGI0, SGI125960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27516[] = {SGI0, SGI0, SGI125957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125959, SGI125961, SGI0, SGI0, SGI125962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27517[] = {SGI125963, SGI125964, SGI125965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125966, SGI0, SGI0, SGI0, SGI125967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27518[] = {SGI125968, SGI125969, SGI125970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125971, SGI0, SGI0, SGI0, SGI125972, SGI125972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27519[] = {SGI125973, SGI125974, SGI125975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125976, SGI0, SGI0, SGI0, SGI125977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27520[] = {SGI0, SGI125978, SGI125979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125980, SGI0, SGI0, SGI0, SGI125981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27521[] = {SGI125982, SGI125983, SGI125984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125985, SGI0, SGI0, SGI0, SGI125986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27522[] = {SGI125987, SGI125988, SGI125989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125990, SGI0, SGI0, SGI0, SGI125991, SGI125991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27523[] = {SGI125992, SGI125993, SGI125994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI125995, SGI0, SGI0, SGI0, SGI125996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27524[] = {SGI125997, SGI125998, SGI125999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126000, SGI0, SGI0, SGI0, SGI126001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27525[] = {SGI126002, SGI126003, SGI126004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126005, SGI0, SGI0, SGI0, SGI126006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27526[] = {SGI126007, SGI126008, SGI126009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126010, SGI0, SGI0, SGI0, SGI126011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27527[] = {SGI126012, SGI126013, SGI126014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126015, SGI0, SGI0, SGI0, SGI126016, SGI126016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27528[] = {SGI126017, SGI126018, SGI126019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126020, SGI0, SGI0, SGI0, SGI126021, SGI126021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27529[] = {SGI126022, SGI126023, SGI126024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126025, SGI0, SGI0, SGI0, SGI126026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27530[] = {SGI126027, SGI126028, SGI126029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126030, SGI0, SGI0, SGI0, SGI126031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27531[] = {SGI126032, SGI126033, SGI126034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126035, SGI0, SGI0, SGI0, SGI126036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27532[] = {SGI126037, SGI126038, SGI126039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126040, SGI0, SGI0, SGI0, SGI126041, SGI126041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27533[] = {SGI126042, SGI126043, SGI126044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126045, SGI0, SGI0, SGI0, SGI126046, SGI126046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27534[] = {SGI126047, SGI126048, SGI126049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126050, SGI0, SGI0, SGI0, SGI126051, SGI126051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27535[] = {SGI126052, SGI126053, SGI126054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126058, SGI0, SGI0, SGI0, SGI126059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126062, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27536[] = {SGI126055, SGI126056, SGI126057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126060, SGI0, SGI0, SGI0, SGI126061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27537[] = {SGI126063, SGI126064, SGI126065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126066, SGI0, SGI0, SGI0, SGI126067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27538[] = {SGI126068, SGI126069, SGI126070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126071, SGI0, SGI0, SGI0, SGI126072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126074, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27539[] = {SGI126073, SGI126075, SGI126076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126077, SGI0, SGI0, SGI0, SGI126078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27540[] = {SGI126079, SGI126080, SGI126081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126082, SGI0, SGI0, SGI0, SGI126083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27541[] = {SGI126084, SGI126085, SGI126086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126087, SGI0, SGI0, SGI0, SGI126088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27542[] = {SGI126089, SGI126090, SGI126091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126095, SGI126095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126096, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27543[] = {SGI126092, SGI126093, SGI126094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126097, SGI0, SGI0, SGI0, SGI126098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27544[] = {SGI126099, SGI126100, SGI126101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126102, SGI0, SGI0, SGI0, SGI126103, SGI126103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27545[] = {SGI126104, SGI126105, SGI126106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126107, SGI0, SGI0, SGI0, SGI126108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27546[] = {SGI126109, SGI126110, SGI126111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126112, SGI0, SGI0, SGI0, SGI126113, SGI126113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27547[] = {SGI126114, SGI126115, SGI126116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126117, SGI0, SGI0, SGI0, SGI126118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27548[] = {SGI126119, SGI126120, SGI126121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126122, SGI0, SGI0, SGI0, SGI126123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27549[] = {SGI126124, SGI126125, SGI126126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126127, SGI0, SGI0, SGI0, SGI126128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27550[] = {SGI126129, SGI126130, SGI126131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126135, SGI0, SGI0, SGI0, SGI126136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126138, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27551[] = {SGI126132, SGI126133, SGI126134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126137, SGI0, SGI0, SGI0, SGI126139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27552[] = {SGI126140, SGI126141, SGI126142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126143, SGI0, SGI0, SGI0, SGI126144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27553[] = {SGI126145, SGI126146, SGI126147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126148, SGI0, SGI0, SGI0, SGI126149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27554[] = {SGI126150, SGI126151, SGI126152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126153, SGI0, SGI0, SGI0, SGI126154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27555[] = {SGI126155, SGI126156, SGI126157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126158, SGI0, SGI0, SGI0, SGI126159, SGI126159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27556[] = {SGI126160, SGI126161, SGI126162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126163, SGI0, SGI0, SGI0, SGI126164, SGI126164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27557[] = {SGI126165, SGI126166, SGI126167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126168, SGI0, SGI0, SGI0, SGI126169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27558[] = {SGI126170, SGI126171, SGI126172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126173, SGI0, SGI0, SGI0, SGI126174, SGI126174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27559[] = {SGI126175, SGI126176, SGI126177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126178, SGI0, SGI0, SGI0, SGI126179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27560[] = {SGI126180, SGI126181, SGI126182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126183, SGI0, SGI0, SGI0, SGI126184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27561[] = {SGI126185, SGI126186, SGI126187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126188, SGI0, SGI0, SGI0, SGI126189, SGI126189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27562[] = {SGI126190, SGI126191, SGI126192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126193, SGI0, SGI0, SGI0, SGI126194, SGI126194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27563[] = {SGI126195, SGI126196, SGI126197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126198, SGI0, SGI0, SGI0, SGI126199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27564[] = {SGI126200, SGI126201, SGI126202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126203, SGI0, SGI0, SGI0, SGI126204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27565[] = {SGI126205, SGI126206, SGI126207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126208, SGI0, SGI0, SGI0, SGI126209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27566[] = {SGI126210, SGI126211, SGI126212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126213, SGI0, SGI0, SGI0, SGI126214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27567[] = {SGI126215, SGI126216, SGI126217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126218, SGI0, SGI0, SGI0, SGI126219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27568[] = {SGI126221, SGI126222, SGI126223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126224, SGI0, SGI0, SGI0, SGI126225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27569[] = {SGI126226, SGI126227, SGI126228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126229, SGI0, SGI0, SGI0, SGI126230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27570[] = {SGI126231, SGI126232, SGI126233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126234, SGI0, SGI0, SGI0, SGI126235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27571[] = {SGI126236, SGI126237, SGI126238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126239, SGI0, SGI0, SGI0, SGI126240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27572[] = {SGI126241, SGI126242, SGI126243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126244, SGI0, SGI0, SGI0, SGI126245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27573[] = {SGI126246, SGI126247, SGI126248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126249, SGI0, SGI0, SGI0, SGI126250, SGI126250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27574[] = {SGI126251, SGI126252, SGI126253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126254, SGI0, SGI0, SGI0, SGI126255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27575[] = {SGI126256, SGI126257, SGI126258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126259, SGI0, SGI0, SGI0, SGI126260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27576[] = {SGI126261, SGI126262, SGI126263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126264, SGI0, SGI0, SGI0, SGI126265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27577[] = {SGI126266, SGI126267, SGI126268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126269, SGI0, SGI0, SGI0, SGI126270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27578[] = {SGI126271, SGI126272, SGI126273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126274, SGI0, SGI0, SGI0, SGI126275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27579[] = {SGI126276, SGI126277, SGI126278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126279, SGI0, SGI0, SGI0, SGI126280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27580[] = {SGI126281, SGI126282, SGI126283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126284, SGI0, SGI0, SGI0, SGI126285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27581[] = {SGI126286, SGI126287, SGI126288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126289, SGI0, SGI0, SGI0, SGI126290, SGI126290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27582[] = {SGI126291, SGI126292, SGI126293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126294, SGI0, SGI0, SGI0, SGI126295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27583[] = {SGI126296, SGI126297, SGI126298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126299, SGI0, SGI0, SGI0, SGI126300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27584[] = {SGI126301, SGI126302, SGI126303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126304, SGI0, SGI0, SGI0, SGI126305, SGI126305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27585[] = {SGI126306, SGI126307, SGI126308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126309, SGI0, SGI0, SGI0, SGI126310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27586[] = {SGI126311, SGI126312, SGI126313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126314, SGI0, SGI0, SGI0, SGI126315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27587[] = {SGI126316, SGI126317, SGI126318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126319, SGI0, SGI0, SGI0, SGI126320, SGI126320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27588[] = {SGI126321, SGI126322, SGI126323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126324, SGI0, SGI0, SGI0, SGI126325, SGI126325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27589[] = {SGI126326, SGI126327, SGI126328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126329, SGI0, SGI0, SGI0, SGI126330, SGI126330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27590[] = {SGI126332, SGI126334, SGI126335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126336, SGI0, SGI0, SGI0, SGI126338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27591[] = {SGI126331, SGI0, SGI126333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126337, SGI126339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126340, SGI0, NULL};
const Segment * const SG27592[] = {SGI126341, SGI126342, SGI126343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126344, SGI0, SGI0, SGI0, SGI126345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27593[] = {SGI126346, SGI126347, SGI126348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126349, SGI0, SGI0, SGI0, SGI126350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27594[] = {SGI126351, SGI126352, SGI126353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126354, SGI0, SGI0, SGI0, SGI126355, SGI126355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27595[] = {SGI126356, SGI126357, SGI126358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126359, SGI0, SGI0, SGI0, SGI126360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27596[] = {SGI126361, SGI126362, SGI126363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126364, SGI0, SGI0, SGI0, SGI126365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27597[] = {SGI126366, SGI126367, SGI126368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126369, SGI0, SGI0, SGI0, SGI126370, SGI126370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27598[] = {SGI126371, SGI126372, SGI126373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126374, SGI0, SGI0, SGI0, SGI126375, SGI126375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27599[] = {SGI126376, SGI126377, SGI126378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126379, SGI0, SGI0, SGI0, SGI126380, SGI126380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27600[] = {SGI126381, SGI126382, SGI126383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126384, SGI0, SGI0, SGI0, SGI126385, SGI126385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27601[] = {SGI126386, SGI126387, SGI126388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126389, SGI0, SGI0, SGI0, SGI126390, SGI126390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27602[] = {SGI126391, SGI126392, SGI126393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126394, SGI0, SGI0, SGI0, SGI126395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27603[] = {SGI126396, SGI126397, SGI126398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126399, SGI0, SGI0, SGI0, SGI126400, SGI126400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27604[] = {SGI126401, SGI126402, SGI126403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126404, SGI0, SGI0, SGI0, SGI126405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27605[] = {SGI126406, SGI126407, SGI126408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126409, SGI0, SGI0, SGI0, SGI126410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27606[] = {SGI126411, SGI126412, SGI126413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126414, SGI0, SGI0, SGI0, SGI126415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27607[] = {SGI126416, SGI126417, SGI126418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126419, SGI0, SGI0, SGI0, SGI126420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27608[] = {SGI126421, SGI126422, SGI126423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126424, SGI0, SGI0, SGI0, SGI126425, SGI126425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27609[] = {SGI126426, SGI126427, SGI126428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126429, SGI0, SGI0, SGI0, SGI126430, SGI126430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27610[] = {SGI126431, SGI126432, SGI126433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126434, SGI0, SGI0, SGI0, SGI126435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27611[] = {SGI126436, SGI126437, SGI126438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126439, SGI0, SGI0, SGI0, SGI126440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27612[] = {SGI126441, SGI126442, SGI126443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126444, SGI0, SGI0, SGI0, SGI126445, SGI126445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27613[] = {SGI126446, SGI126447, SGI126448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126449, SGI0, SGI0, SGI0, SGI126450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27614[] = {SGI126451, SGI126452, SGI126453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126454, SGI0, SGI0, SGI0, SGI126455, SGI126455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27615[] = {SGI126456, SGI126457, SGI126458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126459, SGI0, SGI0, SGI0, SGI126460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27616[] = {SGI126461, SGI126462, SGI126463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126464, SGI0, SGI0, SGI0, SGI126465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27617[] = {SGI126466, SGI126467, SGI126468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126469, SGI0, SGI0, SGI0, SGI126470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27618[] = {SGI126471, SGI126472, SGI126473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126474, SGI0, SGI0, SGI0, SGI126475, SGI126475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27619[] = {SGI126476, SGI126477, SGI126478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126479, SGI0, SGI0, SGI0, SGI126480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27620[] = {SGI126481, SGI126482, SGI126483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126484, SGI0, SGI0, SGI0, SGI126485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27621[] = {SGI126486, SGI126487, SGI126488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126489, SGI0, SGI0, SGI0, SGI126490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27622[] = {SGI126491, SGI126492, SGI126493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126494, SGI0, SGI0, SGI0, SGI126495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27623[] = {SGI126496, SGI126497, SGI126498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126499, SGI0, SGI0, SGI0, SGI126500, SGI126500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27624[] = {SGI126501, SGI126502, SGI126503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126504, SGI0, SGI0, SGI0, SGI126505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27625[] = {SGI126506, SGI126507, SGI126508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126509, SGI0, SGI0, SGI0, SGI126510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27626[] = {SGI126511, SGI126512, SGI126513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126514, SGI0, SGI0, SGI0, SGI126515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27627[] = {SGI126516, SGI126517, SGI126518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126519, SGI0, SGI0, SGI0, SGI126520, SGI126520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27628[] = {SGI0, SGI126521, SGI126522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126523, SGI126523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27629[] = {SGI126524, SGI126525, SGI126526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126527, SGI0, SGI0, SGI0, SGI126528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27630[] = {SGI126529, SGI126530, SGI126531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126532, SGI0, SGI0, SGI0, SGI126533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27631[] = {SGI126534, SGI126535, SGI126536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126537, SGI0, SGI0, SGI0, SGI126538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27632[] = {SGI126539, SGI126540, SGI126541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126542, SGI0, SGI0, SGI0, SGI126543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27633[] = {SGI126544, SGI126545, SGI126546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126547, SGI0, SGI0, SGI0, SGI126548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27634[] = {SGI126549, SGI126550, SGI126551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126552, SGI0, SGI0, SGI0, SGI126553, SGI126553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27635[] = {SGI126554, SGI126555, SGI126556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126557, SGI0, SGI0, SGI0, SGI126558, SGI126558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27636[] = {SGI126559, SGI126560, SGI126561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126562, SGI0, SGI0, SGI0, SGI126563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27637[] = {SGI126564, SGI126565, SGI126566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126567, SGI0, SGI0, SGI0, SGI126568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27638[] = {SGI126569, SGI126570, SGI126571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126572, SGI0, SGI0, SGI0, SGI126573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27639[] = {SGI126574, SGI126575, SGI126576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126577, SGI0, SGI0, SGI0, SGI126578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27640[] = {SGI126579, SGI126580, SGI126581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126582, SGI0, SGI0, SGI0, SGI126583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27641[] = {SGI126584, SGI126585, SGI126586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126587, SGI0, SGI0, SGI0, SGI126588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27642[] = {SGI126589, SGI126590, SGI126591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126592, SGI0, SGI0, SGI0, SGI126593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27643[] = {SGI126594, SGI126595, SGI126596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126597, SGI0, SGI0, SGI0, SGI126598, SGI126598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27644[] = {SGI126599, SGI126600, SGI126601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126602, SGI0, SGI0, SGI0, SGI126603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27645[] = {SGI126604, SGI126605, SGI126606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126607, SGI0, SGI0, SGI0, SGI126608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27646[] = {SGI126609, SGI126610, SGI126611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126612, SGI0, SGI0, SGI0, SGI126613, SGI126613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27647[] = {SGI126614, SGI126615, SGI126616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126617, SGI0, SGI0, SGI0, SGI126618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27648[] = {SGI126619, SGI126620, SGI126621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126622, SGI0, SGI0, SGI0, SGI126623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27649[] = {SGI126624, SGI126625, SGI126626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126627, SGI0, SGI0, SGI0, SGI126628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27650[] = {SGI126629, SGI126630, SGI126631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126632, SGI0, SGI0, SGI0, SGI126633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27651[] = {SGI126634, SGI126635, SGI126636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126637, SGI0, SGI0, SGI0, SGI126638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27652[] = {SGI126639, SGI126640, SGI126641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126642, SGI0, SGI0, SGI0, SGI126643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27653[] = {SGI126644, SGI126645, SGI126646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126647, SGI0, SGI0, SGI0, SGI126648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27654[] = {SGI126649, SGI126650, SGI126651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126652, SGI0, SGI0, SGI0, SGI126653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27655[] = {SGI126654, SGI126655, SGI126656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126657, SGI0, SGI0, SGI0, SGI126658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27656[] = {SGI126659, SGI126660, SGI126661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126662, SGI0, SGI0, SGI0, SGI126663, SGI126663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27657[] = {SGI126664, SGI126665, SGI126666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126667, SGI0, SGI0, SGI0, SGI126668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27658[] = {SGI126669, SGI126670, SGI126671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126672, SGI0, SGI0, SGI0, SGI126673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27659[] = {SGI126674, SGI126675, SGI126676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126677, SGI0, SGI0, SGI0, SGI126678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27660[] = {SGI126679, SGI126680, SGI126681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126682, SGI0, SGI0, SGI0, SGI126683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27661[] = {SGI126684, SGI126685, SGI126686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126687, SGI0, SGI0, SGI0, SGI126688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27662[] = {SGI126689, SGI126690, SGI126691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126692, SGI0, SGI0, SGI0, SGI126693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27663[] = {SGI126694, SGI126695, SGI126696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126697, SGI0, SGI0, SGI0, SGI126698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27664[] = {SGI126699, SGI126700, SGI126701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126702, SGI0, SGI0, SGI0, SGI126703, SGI126703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27665[] = {SGI0, SGI126704, SGI126705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126706, SGI0, SGI0, SGI0, SGI126707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27666[] = {SGI126708, SGI126709, SGI126710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126711, SGI0, SGI0, SGI0, SGI126712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27667[] = {SGI126713, SGI126714, SGI126715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126716, SGI0, SGI0, SGI0, SGI126717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27668[] = {SGI126718, SGI126719, SGI126720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126721, SGI0, SGI0, SGI0, SGI126722, SGI126722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27669[] = {SGI126723, SGI126724, SGI126725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126726, SGI0, SGI0, SGI0, SGI126727, SGI126727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27670[] = {SGI126728, SGI126729, SGI126730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126731, SGI0, SGI0, SGI0, SGI126732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27671[] = {SGI126733, SGI126734, SGI126735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126736, SGI0, SGI0, SGI0, SGI126737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27672[] = {SGI126738, SGI126739, SGI126740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126741, SGI0, SGI0, SGI0, SGI126742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27673[] = {SGI126743, SGI126744, SGI126745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126746, SGI0, SGI0, SGI0, SGI126747, SGI126747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27674[] = {SGI126748, SGI126749, SGI126750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126751, SGI0, SGI0, SGI0, SGI126752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27675[] = {SGI126753, SGI126754, SGI126755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126756, SGI0, SGI0, SGI0, SGI126757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27676[] = {SGI126758, SGI126759, SGI126760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126761, SGI0, SGI0, SGI0, SGI126762, SGI126762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27677[] = {SGI126763, SGI126764, SGI126765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126766, SGI0, SGI0, SGI0, SGI126767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27678[] = {SGI126768, SGI126769, SGI126770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126771, SGI0, SGI0, SGI0, SGI126772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27679[] = {SGI126773, SGI126774, SGI126775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126776, SGI0, SGI0, SGI0, SGI126777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27680[] = {SGI126778, SGI126779, SGI126780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126781, SGI0, SGI0, SGI0, SGI126782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27681[] = {SGI126783, SGI126784, SGI126785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126786, SGI0, SGI0, SGI0, SGI126787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27682[] = {SGI126788, SGI126789, SGI126790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126791, SGI0, SGI0, SGI0, SGI126792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27683[] = {SGI126793, SGI126794, SGI126795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126796, SGI0, SGI0, SGI0, SGI126797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27684[] = {SGI126798, SGI126799, SGI126800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126801, SGI0, SGI0, SGI0, SGI126802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27685[] = {SGI0, SGI126803, SGI126804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126805, SGI0, SGI0, SGI0, SGI126806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27686[] = {SGI126807, SGI126808, SGI126809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126810, SGI0, SGI0, SGI0, SGI126811, SGI126811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27687[] = {SGI126812, SGI126813, SGI126814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126815, SGI0, SGI0, SGI0, SGI126816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27688[] = {SGI126817, SGI126818, SGI126819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126820, SGI0, SGI0, SGI0, SGI126821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27689[] = {SGI126822, SGI126823, SGI126824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126825, SGI0, SGI0, SGI0, SGI126826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27690[] = {SGI126827, SGI126828, SGI126829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126830, SGI0, SGI0, SGI0, SGI126831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27691[] = {SGI126832, SGI126833, SGI126834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126835, SGI0, SGI0, SGI0, SGI126836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27692[] = {SGI126837, SGI126838, SGI126839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126840, SGI0, SGI0, SGI0, SGI126841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27693[] = {SGI126842, SGI126843, SGI126844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126845, SGI0, SGI0, SGI0, SGI126846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27694[] = {SGI126847, SGI126848, SGI126849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126850, SGI0, SGI0, SGI0, SGI0, SGI126851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27695[] = {SGI126852, SGI126853, SGI126854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126855, SGI0, SGI0, SGI0, SGI126856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27696[] = {SGI126857, SGI126858, SGI126859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126860, SGI0, SGI0, SGI0, SGI126861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27697[] = {SGI126862, SGI126863, SGI126864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126865, SGI0, SGI0, SGI0, SGI126866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27698[] = {SGI126867, SGI126868, SGI126869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126870, SGI0, SGI0, SGI0, SGI126871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27699[] = {SGI126872, SGI126873, SGI126874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126875, SGI0, SGI0, SGI0, SGI126876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27700[] = {SGI126877, SGI126878, SGI126879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126880, SGI0, SGI0, SGI0, SGI126881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27701[] = {SGI126882, SGI126883, SGI126884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126885, SGI0, SGI0, SGI0, SGI126886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27702[] = {SGI126887, SGI126888, SGI126889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126890, SGI0, SGI0, SGI0, SGI126891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27703[] = {SGI126892, SGI126893, SGI126894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126895, SGI0, SGI0, SGI0, SGI126896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27704[] = {SGI126897, SGI126898, SGI126899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126900, SGI0, SGI0, SGI0, SGI126901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27705[] = {SGI126902, SGI126903, SGI126904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126905, SGI0, SGI0, SGI0, SGI126906, SGI126906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27706[] = {SGI126907, SGI126908, SGI126909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126910, SGI0, SGI0, SGI0, SGI126911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27707[] = {SGI126912, SGI126913, SGI126914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126915, SGI0, SGI0, SGI0, SGI126916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27708[] = {SGI126917, SGI126918, SGI126919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126920, SGI0, SGI0, SGI0, SGI126921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27709[] = {SGI126922, SGI126923, SGI126924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126925, SGI0, SGI0, SGI0, SGI126926, SGI126926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27710[] = {SGI126927, SGI126928, SGI126929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126930, SGI0, SGI0, SGI0, SGI126931, SGI126931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27711[] = {SGI126932, SGI126933, SGI126934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126935, SGI0, SGI0, SGI0, SGI126936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27712[] = {SGI126937, SGI126938, SGI126939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126940, SGI0, SGI0, SGI0, SGI126941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27713[] = {SGI126942, SGI126943, SGI126944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126945, SGI0, SGI0, SGI0, SGI126946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27714[] = {SGI126947, SGI126948, SGI126949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126950, SGI0, SGI0, SGI0, SGI126951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27715[] = {SGI126952, SGI126953, SGI126954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126955, SGI0, SGI0, SGI0, SGI126956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27716[] = {SGI126957, SGI126958, SGI126959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126960, SGI0, SGI0, SGI0, SGI126961, SGI126961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27717[] = {SGI126962, SGI126963, SGI126964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126965, SGI0, SGI0, SGI0, SGI126966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27718[] = {SGI126967, SGI126968, SGI126969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126970, SGI0, SGI0, SGI0, SGI126971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27719[] = {SGI126972, SGI126973, SGI126974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126975, SGI0, SGI0, SGI0, SGI126976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27720[] = {SGI126977, SGI126978, SGI126979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126980, SGI0, SGI0, SGI0, SGI126981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27721[] = {SGI126982, SGI126983, SGI126984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126985, SGI0, SGI0, SGI0, SGI126986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27722[] = {SGI126987, SGI126988, SGI126989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126990, SGI0, SGI0, SGI0, SGI126991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27723[] = {SGI126992, SGI126993, SGI126994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI126995, SGI0, SGI0, SGI0, SGI126996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27724[] = {SGI126997, SGI126998, SGI126999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127000, SGI0, SGI0, SGI0, SGI127001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27725[] = {SGI127002, SGI127003, SGI127004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127005, SGI0, SGI0, SGI0, SGI0, SGI127006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27726[] = {SGI127007, SGI127008, SGI127009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127010, SGI0, SGI0, SGI0, SGI127011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27727[] = {SGI127012, SGI127013, SGI127014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127015, SGI0, SGI0, SGI0, SGI127016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27728[] = {SGI127017, SGI127018, SGI127019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127020, SGI0, SGI0, SGI0, SGI127021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27729[] = {SGI127022, SGI127023, SGI127024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127025, SGI0, SGI0, SGI0, SGI127026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27730[] = {SGI127027, SGI127028, SGI127029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127030, SGI0, SGI0, SGI0, SGI127031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27731[] = {SGI127032, SGI0, SGI127033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27732[] = {SGI127034, SGI127035, SGI127036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127037, SGI0, SGI0, SGI0, SGI127038, SGI127038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27733[] = {SGI127039, SGI127040, SGI127041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127042, SGI0, SGI0, SGI0, SGI127045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27734[] = {SGI127043, SGI127044, SGI127046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127047, SGI0, SGI0, SGI0, SGI127048, SGI127048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27735[] = {SGI127049, SGI127050, SGI127051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127052, SGI0, SGI0, SGI0, SGI127053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27736[] = {SGI127054, SGI127055, SGI127056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127057, SGI0, SGI0, SGI0, SGI127058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27737[] = {SGI127059, SGI127060, SGI127061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127062, SGI0, SGI0, SGI0, SGI127063, SGI127063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27738[] = {SGI127064, SGI127065, SGI127066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127067, SGI0, SGI0, SGI0, SGI127068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27739[] = {SGI127069, SGI127070, SGI127071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127072, SGI0, SGI0, SGI0, SGI127073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27740[] = {SGI127074, SGI127075, SGI127076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127077, SGI0, SGI0, SGI0, SGI127078, SGI127078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27741[] = {SGI127079, SGI127080, SGI127081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127082, SGI0, SGI0, SGI0, SGI127083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27742[] = {SGI127084, SGI127085, SGI127086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127087, SGI0, SGI0, SGI0, SGI127088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27743[] = {SGI127089, SGI127090, SGI127091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127092, SGI0, SGI0, SGI0, SGI127093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27744[] = {SGI127094, SGI127095, SGI127096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127097, SGI0, SGI0, SGI0, SGI127098, SGI127098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27745[] = {SGI127099, SGI127100, SGI127101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127102, SGI0, SGI0, SGI0, SGI127103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27746[] = {SGI127104, SGI127105, SGI127106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127107, SGI0, SGI0, SGI0, SGI127108, SGI127108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27747[] = {SGI127109, SGI127110, SGI127111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127112, SGI0, SGI0, SGI0, SGI127113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27748[] = {SGI127114, SGI127115, SGI127116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127117, SGI0, SGI127118, SGI127118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27749[] = {SGI127119, SGI127120, SGI127121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127122, SGI0, SGI0, SGI0, SGI127123, SGI127123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27750[] = {SGI127124, SGI127125, SGI127126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127127, SGI0, SGI0, SGI0, SGI127128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27751[] = {SGI127129, SGI127130, SGI127131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127132, SGI0, SGI0, SGI0, SGI127133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27752[] = {SGI0, SGI127134, SGI127135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127136, SGI127136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27753[] = {SGI127137, SGI127138, SGI127139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127140, SGI0, SGI0, SGI0, SGI127141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27754[] = {SGI127142, SGI127143, SGI127144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127145, SGI0, SGI0, SGI0, SGI127146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27755[] = {SGI127150, SGI127151, SGI127152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127153, SGI0, SGI0, SGI0, SGI127154, SGI127154, SGI127155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27756[] = {SGI127156, SGI127157, SGI127158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127159, SGI0, SGI127160, SGI127160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27757[] = {SGI127161, SGI127162, SGI127163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127164, SGI0, SGI127165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27758[] = {SGI127166, SGI127167, SGI127168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127169, SGI0, SGI0, SGI0, SGI127170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127171, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27759[] = {SGI127172, SGI0, SGI127173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127174, SGI0, SGI0, SGI0, SGI127175, SGI127175, SGI0, SGI0, SGI127176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27760[] = {SGI127177, SGI0, SGI127178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127179, SGI0, SGI0, SGI0, SGI127180, SGI127180, SGI0, SGI0, SGI127181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27761[] = {SGI127182, SGI127183, SGI127184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127185, SGI0, SGI0, SGI0, SGI127186, SGI127186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127187, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27762[] = {SGI127188, SGI127189, SGI127190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127191, SGI0, SGI0, SGI0, SGI127192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127193, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27763[] = {SGI127147, SGI127148, SGI127149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127194, SGI0, SGI0, SGI0, SGI127195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27764[] = {SGI127196, SGI127197, SGI127198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127199, SGI0, SGI0, SGI0, SGI127200, SGI127200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27765[] = {SGI127201, SGI127202, SGI127203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127204, SGI0, SGI0, SGI0, SGI127205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27766[] = {SGI127206, SGI127207, SGI127208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127209, SGI0, SGI0, SGI0, SGI127210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27767[] = {SGI127211, SGI127212, SGI127213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127214, SGI0, SGI0, SGI0, SGI127215, SGI127215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27768[] = {SGI127216, SGI127217, SGI127218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127219, SGI0, SGI0, SGI0, SGI127220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27769[] = {SGI127221, SGI127222, SGI127223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127224, SGI0, SGI0, SGI0, SGI127225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27770[] = {SGI127226, SGI127227, SGI127228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127229, SGI0, SGI0, SGI0, SGI127230, SGI127230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27771[] = {SGI127231, SGI127232, SGI127233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127234, SGI0, SGI0, SGI0, SGI127235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27772[] = {SGI127236, SGI127237, SGI127238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127239, SGI0, SGI0, SGI0, SGI127240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27773[] = {SGI127241, SGI0, SGI127242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27774[] = {SGI127243, SGI127244, SGI127245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127246, SGI0, SGI0, SGI0, SGI127247, SGI127247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27775[] = {SGI127248, SGI127249, SGI127250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127251, SGI0, SGI0, SGI0, SGI127252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27776[] = {SGI127253, SGI127254, SGI127255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127256, SGI0, SGI0, SGI0, SGI127257, SGI127257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27777[] = {SGI127258, SGI127259, SGI127260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127261, SGI0, SGI0, SGI0, SGI127262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27778[] = {SGI127263, SGI127264, SGI127265, SGI0, SGI0, SGI0, SGI127267, SGI127269, SGI0, SGI127271, SGI0, SGI0, SGI0, SGI127273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27779[] = {SGI127266, SGI127268, SGI127270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127272, SGI0, SGI0, SGI0, SGI127274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27780[] = {SGI127275, SGI127276, SGI127277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127278, SGI0, SGI0, SGI0, SGI127279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27781[] = {SGI127280, SGI127281, SGI127282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127283, SGI0, SGI0, SGI0, SGI127284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27782[] = {SGI127285, SGI127286, SGI127287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127288, SGI0, SGI0, SGI0, SGI127289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27783[] = {SGI127290, SGI127291, SGI127292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127293, SGI0, SGI0, SGI0, SGI127294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27784[] = {SGI127295, SGI127296, SGI127297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127298, SGI0, SGI0, SGI0, SGI127299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27785[] = {SGI127300, SGI127301, SGI127302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127303, SGI0, SGI0, SGI0, SGI127304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27786[] = {SGI127305, SGI127306, SGI127307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127308, SGI0, SGI0, SGI0, SGI127309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27787[] = {SGI127310, SGI127311, SGI127312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127313, SGI0, SGI0, SGI0, SGI127314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27788[] = {SGI127315, SGI127316, SGI127317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127318, SGI0, SGI0, SGI0, SGI127319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27789[] = {SGI127320, SGI127321, SGI127322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127323, SGI0, SGI0, SGI0, SGI127324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27790[] = {SGI127325, SGI127326, SGI127327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127328, SGI0, SGI0, SGI0, SGI127329, SGI127329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27791[] = {SGI127330, SGI127331, SGI127332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127333, SGI0, SGI0, SGI0, SGI127334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27792[] = {SGI127335, SGI127336, SGI127337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127338, SGI0, SGI0, SGI0, SGI127339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27793[] = {SGI127340, SGI127341, SGI127342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127343, SGI0, SGI0, SGI0, SGI127344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27794[] = {SGI127345, SGI127346, SGI127347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127348, SGI0, SGI0, SGI0, SGI127349, SGI127349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127350, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27795[] = {SGI127351, SGI127352, SGI127353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127354, SGI0, SGI0, SGI0, SGI127355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27796[] = {SGI127356, SGI127357, SGI127358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127359, SGI0, SGI0, SGI0, SGI127360, SGI127360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27797[] = {SGI127361, SGI127362, SGI127363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127364, SGI0, SGI0, SGI0, SGI127365, SGI127365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27798[] = {SGI127366, SGI127367, SGI127368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127369, SGI0, SGI0, SGI0, SGI127370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27799[] = {SGI127371, SGI127372, SGI127373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127374, SGI0, SGI0, SGI0, SGI127375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27800[] = {SGI127376, SGI127377, SGI127378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127379, SGI0, SGI0, SGI0, SGI127380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27801[] = {SGI127381, SGI127382, SGI127383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127384, SGI0, SGI0, SGI0, SGI127385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27802[] = {SGI127386, SGI127387, SGI127388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127389, SGI0, SGI0, SGI0, SGI127390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27803[] = {SGI127391, SGI127392, SGI127393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127394, SGI0, SGI0, SGI0, SGI127395, SGI127395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27804[] = {SGI127396, SGI127397, SGI127398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127399, SGI0, SGI0, SGI0, SGI127400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27805[] = {SGI127401, SGI127402, SGI127403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127404, SGI0, SGI0, SGI0, SGI127405, SGI0, SGI0, SGI0, SGI127406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27806[] = {SGI127407, SGI127408, SGI127409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127410, SGI0, SGI0, SGI0, SGI127411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27807[] = {SGI127412, SGI127413, SGI127414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127415, SGI0, SGI0, SGI0, SGI127416, SGI127416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27808[] = {SGI127417, SGI127418, SGI127419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127420, SGI0, SGI0, SGI0, SGI127421, SGI127421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27809[] = {SGI127422, SGI127423, SGI127424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127425, SGI0, SGI0, SGI0, SGI127426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27810[] = {SGI127427, SGI127428, SGI127429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127430, SGI0, SGI0, SGI0, SGI127431, SGI127431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27811[] = {SGI127432, SGI127433, SGI127434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127435, SGI0, SGI0, SGI0, SGI127436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27812[] = {SGI127437, SGI0, SGI127438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127439, SGI0, SGI0, SGI0, SGI127441, SGI127441, SGI0, SGI0, SGI127444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27813[] = {SGI127440, SGI127442, SGI127443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127445, SGI0, SGI0, SGI0, SGI127446, SGI127446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27814[] = {SGI127447, SGI127448, SGI127449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127450, SGI0, SGI0, SGI0, SGI127451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27815[] = {SGI127452, SGI127453, SGI127454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127458, SGI0, SGI0, SGI0, SGI127460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127462, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27816[] = {SGI127455, SGI127456, SGI127457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127459, SGI0, SGI0, SGI0, SGI127461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27817[] = {SGI127463, SGI127464, SGI127465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127466, SGI0, SGI0, SGI0, SGI127467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27818[] = {SGI127468, SGI127469, SGI127470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127471, SGI0, SGI0, SGI0, SGI127472, SGI127472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27819[] = {SGI127473, SGI127474, SGI127475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127476, SGI0, SGI0, SGI0, SGI127478, SGI127478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27820[] = {SGI127477, SGI127479, SGI127480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127481, SGI0, SGI0, SGI0, SGI127482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127483, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27821[] = {SGI127484, SGI127485, SGI127486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127487, SGI0, SGI0, SGI0, SGI127488, SGI127488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27822[] = {SGI127489, SGI127490, SGI127491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127492, SGI0, SGI0, SGI0, SGI127493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27823[] = {SGI127494, SGI127495, SGI127496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127497, SGI0, SGI0, SGI0, SGI127498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27824[] = {SGI127499, SGI127500, SGI127501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127502, SGI0, SGI0, SGI0, SGI127503, SGI127503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27825[] = {SGI127504, SGI127505, SGI127506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127507, SGI0, SGI0, SGI0, SGI127508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27826[] = {SGI127509, SGI127510, SGI127511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127512, SGI0, SGI0, SGI0, SGI127513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27827[] = {SGI127514, SGI127515, SGI127516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127517, SGI0, SGI0, SGI0, SGI127518, SGI127518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27828[] = {SGI127519, SGI127520, SGI127521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127522, SGI0, SGI0, SGI0, SGI127523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27829[] = {SGI127524, SGI127525, SGI127526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127527, SGI0, SGI0, SGI0, SGI127528, SGI127528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27830[] = {SGI127529, SGI127530, SGI127531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127532, SGI0, SGI0, SGI0, SGI127533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27831[] = {SGI127534, SGI127535, SGI127536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127537, SGI0, SGI0, SGI0, SGI127538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27832[] = {SGI127539, SGI127540, SGI127541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127542, SGI0, SGI0, SGI0, SGI127543, SGI127543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27833[] = {SGI127544, SGI127545, SGI127546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127547, SGI0, SGI0, SGI0, SGI127548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27834[] = {SGI127549, SGI127550, SGI127551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127552, SGI0, SGI0, SGI0, SGI127553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27835[] = {SGI127554, SGI127555, SGI127556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127557, SGI0, SGI0, SGI0, SGI127558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27836[] = {SGI127559, SGI127560, SGI127561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127562, SGI0, SGI0, SGI0, SGI127563, SGI127563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27837[] = {SGI127564, SGI127565, SGI127566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127567, SGI0, SGI0, SGI0, SGI127568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27838[] = {SGI127569, SGI127570, SGI127571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127572, SGI0, SGI0, SGI0, SGI127573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27839[] = {SGI127574, SGI127575, SGI127576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127577, SGI0, SGI0, SGI0, SGI127578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27840[] = {SGI127579, SGI127580, SGI127581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127582, SGI0, SGI0, SGI0, SGI127583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27841[] = {SGI127584, SGI127585, SGI127586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127587, SGI0, SGI0, SGI0, SGI127588, SGI127588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27842[] = {SGI127589, SGI127590, SGI127591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127592, SGI0, SGI0, SGI0, SGI127593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27843[] = {SGI127594, SGI127595, SGI127596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127597, SGI0, SGI0, SGI0, SGI127598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27844[] = {SGI127599, SGI127600, SGI127601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127602, SGI0, SGI0, SGI0, SGI127603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27845[] = {SGI127604, SGI0, SGI127605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27846[] = {SGI127606, SGI127607, SGI127608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127609, SGI0, SGI0, SGI0, SGI127610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27847[] = {SGI127611, SGI127612, SGI127613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127614, SGI0, SGI0, SGI0, SGI127615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27848[] = {SGI127616, SGI127617, SGI127618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127619, SGI0, SGI0, SGI0, SGI127620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27849[] = {SGI127621, SGI127622, SGI127623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127624, SGI0, SGI0, SGI0, SGI127625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27850[] = {SGI127626, SGI127627, SGI127628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127629, SGI0, SGI0, SGI0, SGI127630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27851[] = {SGI127631, SGI127632, SGI127633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127634, SGI0, SGI0, SGI0, SGI127635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27852[] = {SGI127636, SGI127637, SGI127638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127639, SGI0, SGI0, SGI0, SGI127640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27853[] = {SGI127641, SGI127642, SGI127643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127644, SGI0, SGI0, SGI0, SGI127645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27854[] = {SGI127646, SGI127647, SGI127648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127649, SGI0, SGI0, SGI0, SGI127650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27855[] = {SGI127651, SGI127652, SGI127653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127654, SGI0, SGI0, SGI0, SGI127655, SGI127655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27856[] = {SGI127656, SGI127657, SGI127658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127659, SGI0, SGI0, SGI0, SGI127660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27857[] = {SGI127661, SGI127662, SGI127663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127664, SGI0, SGI0, SGI0, SGI127665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27858[] = {SGI127666, SGI127667, SGI127668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127669, SGI0, SGI0, SGI0, SGI127670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27859[] = {SGI127671, SGI127672, SGI127673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127674, SGI0, SGI0, SGI0, SGI127675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27860[] = {SGI127676, SGI127677, SGI127678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127679, SGI0, SGI0, SGI0, SGI127680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27861[] = {SGI127681, SGI127682, SGI127683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127684, SGI0, SGI0, SGI0, SGI127685, SGI127685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27862[] = {SGI127686, SGI127687, SGI127688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127689, SGI0, SGI0, SGI0, SGI127690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27863[] = {SGI127691, SGI127692, SGI127693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127694, SGI0, SGI0, SGI0, SGI127695, SGI127695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27864[] = {SGI127696, SGI127697, SGI127698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127699, SGI0, SGI0, SGI0, SGI127700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27865[] = {SGI127701, SGI127702, SGI127703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127704, SGI0, SGI0, SGI0, SGI127705, SGI127705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27866[] = {SGI127706, SGI127707, SGI127708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127709, SGI0, SGI0, SGI0, SGI127710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27867[] = {SGI127711, SGI127712, SGI127713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127714, SGI0, SGI0, SGI0, SGI127715, SGI127715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27868[] = {SGI127716, SGI127717, SGI127718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127719, SGI0, SGI0, SGI0, SGI127720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27869[] = {SGI127721, SGI127722, SGI127723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127724, SGI0, SGI0, SGI0, SGI127725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27870[] = {SGI127726, SGI127727, SGI127728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127729, SGI0, SGI0, SGI0, SGI127730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27871[] = {SGI127731, SGI127732, SGI127733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127734, SGI0, SGI0, SGI0, SGI127735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27872[] = {SGI127736, SGI127737, SGI127738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127739, SGI0, SGI0, SGI0, SGI127740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27873[] = {SGI127741, SGI127742, SGI127743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127744, SGI0, SGI0, SGI0, SGI127745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27874[] = {SGI127746, SGI127747, SGI127748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127749, SGI0, SGI0, SGI0, SGI127750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27875[] = {SGI127751, SGI127752, SGI127753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127754, SGI0, SGI0, SGI0, SGI127755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27876[] = {SGI127756, SGI127757, SGI127758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127759, SGI0, SGI0, SGI0, SGI127760, SGI127760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27877[] = {SGI127761, SGI127762, SGI127763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127764, SGI0, SGI0, SGI0, SGI127765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27878[] = {SGI127766, SGI127767, SGI127768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127769, SGI0, SGI0, SGI0, SGI127770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27879[] = {SGI127771, SGI127772, SGI127773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127774, SGI0, SGI0, SGI0, SGI127775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27880[] = {SGI127776, SGI127777, SGI127778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127779, SGI0, SGI0, SGI0, SGI127780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27881[] = {SGI127781, SGI127782, SGI127783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127784, SGI0, SGI0, SGI0, SGI127785, SGI127785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27882[] = {SGI127786, SGI127787, SGI127788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127789, SGI0, SGI0, SGI0, SGI127790, SGI127790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27883[] = {SGI127791, SGI127792, SGI127793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127794, SGI0, SGI0, SGI0, SGI127795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27884[] = {SGI127796, SGI127797, SGI127798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127799, SGI0, SGI0, SGI0, SGI127800, SGI127800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27885[] = {SGI127801, SGI127802, SGI127803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127804, SGI0, SGI0, SGI0, SGI127805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27886[] = {SGI127806, SGI127807, SGI127808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127809, SGI0, SGI0, SGI0, SGI127810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27887[] = {SGI127811, SGI0, SGI127812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127813, SGI0, SGI0, SGI0, SGI127814, SGI127814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27888[] = {SGI127815, SGI127816, SGI127817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127818, SGI0, SGI0, SGI0, SGI0, SGI127819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27889[] = {SGI127820, SGI127821, SGI127822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127823, SGI0, SGI0, SGI0, SGI127824, SGI127824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27890[] = {SGI127825, SGI127826, SGI127827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127828, SGI0, SGI0, SGI0, SGI127829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27891[] = {SGI127830, SGI127831, SGI127832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127833, SGI0, SGI0, SGI0, SGI127834, SGI127834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27892[] = {SGI127835, SGI127836, SGI127837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127838, SGI0, SGI0, SGI0, SGI127839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27893[] = {SGI127840, SGI127841, SGI127842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127843, SGI0, SGI0, SGI0, SGI127844, SGI127844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27894[] = {SGI127845, SGI127846, SGI127847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127848, SGI0, SGI0, SGI0, SGI127849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27895[] = {SGI127850, SGI127851, SGI127852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127853, SGI0, SGI0, SGI0, SGI127854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27896[] = {SGI127855, SGI127856, SGI127857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127858, SGI0, SGI0, SGI0, SGI127859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27897[] = {SGI127860, SGI127861, SGI127862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127863, SGI0, SGI0, SGI0, SGI127864, SGI127864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27898[] = {SGI127865, SGI127866, SGI127867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127868, SGI0, SGI0, SGI0, SGI127869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27899[] = {SGI127870, SGI127871, SGI127872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127873, SGI0, SGI0, SGI0, SGI127874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27900[] = {SGI127875, SGI127876, SGI127877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127878, SGI0, SGI0, SGI0, SGI127879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27901[] = {SGI127880, SGI127881, SGI127882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127883, SGI0, SGI0, SGI0, SGI127884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27902[] = {SGI0, SGI127885, SGI127886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127887, SGI127887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27903[] = {SGI127888, SGI127889, SGI127890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127891, SGI0, SGI0, SGI0, SGI127892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27904[] = {SGI127893, SGI127894, SGI127895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127896, SGI0, SGI0, SGI0, SGI127897, SGI127897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27905[] = {SGI127898, SGI127899, SGI127900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127901, SGI0, SGI0, SGI0, SGI127902, SGI127902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27906[] = {SGI127903, SGI127904, SGI127905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127906, SGI0, SGI0, SGI0, SGI127907, SGI127907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27907[] = {SGI127908, SGI127909, SGI127910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127911, SGI0, SGI0, SGI0, SGI127912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27908[] = {SGI127913, SGI127914, SGI127915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127916, SGI0, SGI0, SGI0, SGI127917, SGI127917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27909[] = {SGI127918, SGI127919, SGI127920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127921, SGI0, SGI0, SGI0, SGI127922, SGI127922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27910[] = {SGI127923, SGI127924, SGI127925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127926, SGI0, SGI0, SGI0, SGI127927, SGI127927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27911[] = {SGI127928, SGI127929, SGI127930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127931, SGI0, SGI0, SGI0, SGI127932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27912[] = {SGI127933, SGI127934, SGI127935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127936, SGI0, SGI0, SGI0, SGI127937, SGI127937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27913[] = {SGI127938, SGI127939, SGI127940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127941, SGI0, SGI0, SGI0, SGI127942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27914[] = {SGI127943, SGI127944, SGI127945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127946, SGI0, SGI0, SGI0, SGI127947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27915[] = {SGI127948, SGI127949, SGI127950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127951, SGI0, SGI0, SGI0, SGI127952, SGI127952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27916[] = {SGI127953, SGI127954, SGI127955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127956, SGI0, SGI0, SGI0, SGI127957, SGI127957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27917[] = {SGI127958, SGI127959, SGI127960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127961, SGI0, SGI0, SGI0, SGI127962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27918[] = {SGI127963, SGI127964, SGI127965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127966, SGI0, SGI0, SGI0, SGI127967, SGI127967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27919[] = {SGI127968, SGI127969, SGI127970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127971, SGI0, SGI0, SGI0, SGI127972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27920[] = {SGI127973, SGI127974, SGI127975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127976, SGI0, SGI0, SGI0, SGI127977, SGI127977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27921[] = {SGI127978, SGI127979, SGI127980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127981, SGI0, SGI0, SGI0, SGI127982, SGI127982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27922[] = {SGI127983, SGI127984, SGI127985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127986, SGI0, SGI0, SGI0, SGI127987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27923[] = {SGI127988, SGI127989, SGI127990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127991, SGI0, SGI0, SGI0, SGI127992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27924[] = {SGI127993, SGI127994, SGI127995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI127996, SGI0, SGI0, SGI0, SGI127997, SGI127997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27925[] = {SGI127998, SGI127999, SGI128000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128001, SGI0, SGI0, SGI0, SGI0, SGI128002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27926[] = {SGI128003, SGI128004, SGI128005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128006, SGI0, SGI0, SGI0, SGI128007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27927[] = {SGI128008, SGI128009, SGI128010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128011, SGI0, SGI0, SGI0, SGI128012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27928[] = {SGI128013, SGI128014, SGI128015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128016, SGI0, SGI0, SGI0, SGI128017, SGI128017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27929[] = {SGI128018, SGI128019, SGI128020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128021, SGI0, SGI0, SGI0, SGI128022, SGI128022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27930[] = {SGI128023, SGI128024, SGI128025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128026, SGI0, SGI0, SGI0, SGI128027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27931[] = {SGI128028, SGI128029, SGI128030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128031, SGI0, SGI0, SGI0, SGI128032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27932[] = {SGI128033, SGI128034, SGI128035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128036, SGI0, SGI0, SGI0, SGI128037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27933[] = {SGI128038, SGI128039, SGI128040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128041, SGI0, SGI0, SGI0, SGI128042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27934[] = {SGI128043, SGI128044, SGI128045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128046, SGI0, SGI0, SGI0, SGI128047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27935[] = {SGI128048, SGI0, SGI128049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27936[] = {SGI128050, SGI128051, SGI128052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128056, SGI0, SGI0, SGI0, SGI128057, SGI128057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27937[] = {SGI128053, SGI128054, SGI128055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128058, SGI0, SGI0, SGI0, SGI128059, SGI128059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27938[] = {SGI128060, SGI128061, SGI128062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128063, SGI0, SGI0, SGI0, SGI128064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27939[] = {SGI128065, SGI128066, SGI128067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128068, SGI0, SGI0, SGI0, SGI128069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27940[] = {SGI128070, SGI128071, SGI128072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128073, SGI0, SGI0, SGI0, SGI128074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27941[] = {SGI128075, SGI128076, SGI128077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128078, SGI0, SGI0, SGI0, SGI128079, SGI128079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27942[] = {SGI128080, SGI128081, SGI128082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128083, SGI0, SGI0, SGI0, SGI128084, SGI128084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27943[] = {SGI0, SGI0, SGI128085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128086, SGI128087, SGI0, SGI0, SGI128088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27944[] = {SGI128089, SGI128090, SGI128091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128092, SGI0, SGI0, SGI0, SGI128093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27945[] = {SGI128094, SGI128095, SGI128096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128097, SGI0, SGI0, SGI0, SGI128098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27946[] = {SGI128099, SGI128100, SGI128101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128102, SGI0, SGI0, SGI0, SGI128103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27947[] = {SGI128104, SGI128105, SGI128106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128107, SGI0, SGI0, SGI0, SGI128108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27948[] = {SGI128109, SGI128110, SGI128111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128112, SGI0, SGI0, SGI0, SGI128113, SGI128113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27949[] = {SGI128114, SGI128115, SGI128116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128117, SGI0, SGI0, SGI0, SGI128118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27950[] = {SGI128119, SGI128120, SGI128121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128122, SGI0, SGI0, SGI0, SGI128123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27951[] = {SGI128124, SGI128125, SGI128126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128127, SGI0, SGI0, SGI0, SGI128128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27952[] = {SGI128129, SGI128130, SGI128131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128132, SGI0, SGI0, SGI0, SGI128133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27953[] = {SGI128134, SGI128135, SGI128136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128137, SGI0, SGI0, SGI0, SGI128138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27954[] = {SGI128139, SGI128140, SGI128141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128142, SGI0, SGI0, SGI0, SGI128143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27955[] = {SGI128144, SGI128145, SGI128146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128147, SGI0, SGI0, SGI0, SGI128148, SGI128148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27956[] = {SGI128149, SGI128150, SGI128151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128152, SGI0, SGI0, SGI0, SGI128153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27957[] = {SGI128154, SGI128155, SGI128156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128157, SGI0, SGI0, SGI0, SGI128158, SGI128158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27958[] = {SGI128159, SGI128160, SGI128161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128162, SGI0, SGI0, SGI0, SGI128163, SGI128163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27959[] = {SGI128164, SGI128165, SGI128166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128167, SGI0, SGI0, SGI0, SGI128168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27960[] = {SGI128169, SGI128170, SGI128171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128172, SGI0, SGI0, SGI0, SGI128173, SGI128173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27961[] = {SGI128174, SGI128175, SGI128176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128179, SGI0, SGI0, SGI0, SGI128181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27962[] = {SGI128177, SGI128178, SGI128180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128182, SGI0, SGI0, SGI0, SGI128183, SGI128183, SGI128184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27963[] = {SGI128185, SGI128186, SGI128187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128188, SGI0, SGI0, SGI0, SGI128189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27964[] = {SGI128190, SGI128191, SGI128192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128193, SGI0, SGI0, SGI0, SGI128194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27965[] = {SGI128195, SGI128196, SGI128197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128198, SGI0, SGI0, SGI0, SGI128199, SGI128199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27966[] = {SGI128200, SGI128201, SGI128202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128203, SGI0, SGI0, SGI0, SGI128204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27967[] = {SGI128205, SGI128206, SGI128207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128208, SGI0, SGI0, SGI0, SGI128209, SGI128209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27968[] = {SGI128210, SGI128211, SGI128212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128213, SGI0, SGI0, SGI0, SGI128214, SGI128214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27969[] = {SGI128215, SGI128217, SGI128219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128221, SGI0, SGI0, SGI0, SGI128223, SGI128223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27970[] = {SGI128216, SGI128218, SGI128220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128222, SGI0, SGI0, SGI0, SGI128224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128225, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27971[] = {SGI128226, SGI128227, SGI128228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128229, SGI0, SGI0, SGI0, SGI128230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27972[] = {SGI128231, SGI128232, SGI128233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128237, SGI0, SGI0, SGI0, SGI128238, SGI128238, SGI128240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27973[] = {SGI128234, SGI128235, SGI128236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128239, SGI0, SGI0, SGI0, SGI128241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27974[] = {SGI128242, SGI128243, SGI128244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128245, SGI0, SGI0, SGI0, SGI128246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27975[] = {SGI128247, SGI128248, SGI128249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128250, SGI0, SGI0, SGI0, SGI128251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27976[] = {SGI128252, SGI128253, SGI128254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128255, SGI0, SGI0, SGI0, SGI128256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27977[] = {SGI128257, SGI128258, SGI128259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128260, SGI0, SGI0, SGI0, SGI128261, SGI128261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27978[] = {SGI128262, SGI128263, SGI128264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128265, SGI0, SGI0, SGI0, SGI128266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27979[] = {SGI128267, SGI128268, SGI128269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128270, SGI0, SGI0, SGI0, SGI128271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27980[] = {SGI128272, SGI128273, SGI128274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128275, SGI0, SGI0, SGI0, SGI128276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27981[] = {SGI128277, SGI128278, SGI128279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128280, SGI0, SGI0, SGI0, SGI128281, SGI128281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27982[] = {SGI128282, SGI128283, SGI128284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128285, SGI0, SGI0, SGI0, SGI128286, SGI128286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27983[] = {SGI128287, SGI128288, SGI128289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128290, SGI0, SGI0, SGI0, SGI128291, SGI128291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27984[] = {SGI128292, SGI128293, SGI128294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128295, SGI0, SGI0, SGI0, SGI128296, SGI128296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27985[] = {SGI128297, SGI128298, SGI128299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128300, SGI0, SGI0, SGI0, SGI128301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27986[] = {SGI128302, SGI128303, SGI128304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128305, SGI0, SGI0, SGI0, SGI128306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27987[] = {SGI128307, SGI128308, SGI128309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128310, SGI0, SGI0, SGI0, SGI128311, SGI128311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27988[] = {SGI128312, SGI128313, SGI128314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128315, SGI0, SGI0, SGI0, SGI128316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27989[] = {SGI128317, SGI128318, SGI128319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128320, SGI0, SGI0, SGI0, SGI128321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27990[] = {SGI128322, SGI128323, SGI128324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128325, SGI0, SGI0, SGI0, SGI128326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27991[] = {SGI128327, SGI128328, SGI128329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128330, SGI0, SGI0, SGI0, SGI128331, SGI128331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27992[] = {SGI128332, SGI128333, SGI128334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128335, SGI0, SGI0, SGI0, SGI128336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27993[] = {SGI128337, SGI128338, SGI128340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128343, SGI0, SGI0, SGI0, SGI128345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128347, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27994[] = {SGI128339, SGI128341, SGI128342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128344, SGI0, SGI0, SGI0, SGI128346, SGI128346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27995[] = {SGI128348, SGI128349, SGI128350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128351, SGI0, SGI0, SGI0, SGI128352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27996[] = {SGI128353, SGI128354, SGI128355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128356, SGI0, SGI0, SGI0, SGI128357, SGI128357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27997[] = {SGI128358, SGI0, SGI128359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128363, SGI0, SGI0, SGI0, SGI128365, SGI128365, SGI0, SGI0, SGI128367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27998[] = {SGI128360, SGI128361, SGI128362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128364, SGI0, SGI0, SGI0, SGI128366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG27999[] = {SGI128368, SGI128369, SGI128370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128371, SGI0, SGI0, SGI0, SGI128372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28000[] = {SGI128373, SGI128374, SGI128375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128376, SGI0, SGI0, SGI0, SGI128377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28001[] = {SGI128378, SGI128379, SGI128380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128381, SGI0, SGI0, SGI0, SGI128382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28002[] = {SGI128383, SGI128384, SGI128385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128386, SGI0, SGI0, SGI0, SGI128387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28003[] = {SGI128388, SGI128389, SGI128390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128391, SGI0, SGI0, SGI0, SGI128392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28004[] = {SGI128393, SGI128394, SGI128395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128396, SGI0, SGI0, SGI0, SGI128397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28005[] = {SGI128398, SGI128399, SGI128400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128401, SGI0, SGI0, SGI0, SGI128402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28006[] = {SGI128406, SGI128407, SGI128408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128410, SGI0, SGI0, SGI0, SGI128411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28007[] = {SGI128403, SGI128404, SGI128405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128409, SGI0, SGI0, SGI0, SGI128412, SGI0, SGI128413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28008[] = {SGI128414, SGI128415, SGI128416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128417, SGI0, SGI0, SGI0, SGI128418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28009[] = {SGI128419, SGI128420, SGI128421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128422, SGI0, SGI0, SGI0, SGI128423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28010[] = {SGI128424, SGI128425, SGI128426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128427, SGI0, SGI0, SGI0, SGI128428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28011[] = {SGI128429, SGI128430, SGI128431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128432, SGI0, SGI0, SGI0, SGI128433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28012[] = {SGI128435, SGI128437, SGI128439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128441, SGI0, SGI0, SGI0, SGI128443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28013[] = {SGI128434, SGI128436, SGI128438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128440, SGI0, SGI0, SGI0, SGI128442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128444, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28014[] = {SGI128445, SGI128446, SGI128447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128448, SGI0, SGI0, SGI0, SGI128449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28015[] = {SGI128450, SGI128451, SGI128452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128453, SGI0, SGI0, SGI0, SGI128454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28016[] = {SGI128455, SGI128456, SGI128457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128458, SGI0, SGI0, SGI0, SGI128459, SGI128459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28017[] = {SGI128460, SGI128461, SGI128462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128463, SGI0, SGI0, SGI0, SGI128464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28018[] = {SGI128465, SGI128466, SGI128467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128468, SGI0, SGI0, SGI0, SGI128469, SGI128469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28019[] = {SGI128470, SGI128471, SGI128472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128473, SGI0, SGI0, SGI0, SGI128474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28020[] = {SGI128475, SGI128476, SGI128477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128478, SGI0, SGI0, SGI0, SGI128479, SGI128479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28021[] = {SGI128480, SGI128481, SGI128482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128483, SGI0, SGI0, SGI0, SGI128484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28022[] = {SGI128485, SGI128486, SGI128487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128488, SGI0, SGI0, SGI0, SGI128489, SGI128489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28023[] = {SGI128490, SGI128491, SGI128492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128493, SGI0, SGI0, SGI0, SGI0, SGI128494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28024[] = {SGI128495, SGI128496, SGI128497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128498, SGI0, SGI0, SGI0, SGI128499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28025[] = {SGI128500, SGI128501, SGI128502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128503, SGI0, SGI0, SGI0, SGI128504, SGI128504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28026[] = {SGI128505, SGI128506, SGI128507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128508, SGI0, SGI0, SGI0, SGI128509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128510, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28027[] = {SGI128511, SGI128512, SGI128513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128514, SGI0, SGI0, SGI0, SGI128515, SGI128515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28028[] = {SGI128516, SGI128517, SGI128518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128519, SGI0, SGI0, SGI0, SGI128520, SGI128520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28029[] = {SGI128521, SGI128522, SGI128523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128524, SGI0, SGI0, SGI0, SGI128525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28030[] = {SGI128526, SGI128528, SGI128530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128532, SGI0, SGI0, SGI0, SGI128534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28031[] = {SGI128527, SGI128529, SGI128531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128533, SGI0, SGI0, SGI0, SGI128535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128536, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28032[] = {SGI128537, SGI128538, SGI128539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128540, SGI0, SGI0, SGI0, SGI128541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28033[] = {SGI128542, SGI128543, SGI128544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128545, SGI0, SGI0, SGI0, SGI128546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28034[] = {SGI128547, SGI128548, SGI128549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128550, SGI0, SGI0, SGI0, SGI128551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28035[] = {SGI128552, SGI128553, SGI128554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128555, SGI0, SGI0, SGI0, SGI128556, SGI128556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28036[] = {SGI128557, SGI128558, SGI128559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128560, SGI0, SGI0, SGI0, SGI128561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28037[] = {SGI128562, SGI128563, SGI128564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128565, SGI0, SGI0, SGI0, SGI128566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28038[] = {SGI128567, SGI128568, SGI128569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128570, SGI0, SGI0, SGI0, SGI128571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28039[] = {SGI128572, SGI128573, SGI128574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128575, SGI0, SGI0, SGI0, SGI128576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28040[] = {SGI128577, SGI128578, SGI128579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128580, SGI0, SGI0, SGI0, SGI128581, SGI128581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28041[] = {SGI128582, SGI128583, SGI128584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128585, SGI0, SGI0, SGI0, SGI0, SGI128586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28042[] = {SGI128587, SGI128588, SGI128589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128590, SGI0, SGI0, SGI0, SGI128591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28043[] = {SGI0, SGI128592, SGI0, SGI128593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28044[] = {SGI128594, SGI128595, SGI128596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128597, SGI0, SGI0, SGI0, SGI128598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28045[] = {SGI128599, SGI128600, SGI128601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128604, SGI0, SGI0, SGI0, SGI128606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28046[] = {SGI128602, SGI128603, SGI128605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128607, SGI0, SGI0, SGI0, SGI0, SGI128608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28047[] = {SGI128609, SGI128610, SGI128611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128612, SGI0, SGI0, SGI0, SGI128613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28048[] = {SGI128614, SGI128615, SGI128616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128617, SGI0, SGI0, SGI0, SGI0, SGI128618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28049[] = {SGI128619, SGI128620, SGI128621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128622, SGI0, SGI0, SGI0, SGI128623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28050[] = {SGI128624, SGI128625, SGI128626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128627, SGI0, SGI0, SGI0, SGI128628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28051[] = {SGI128629, SGI128630, SGI128631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128632, SGI0, SGI0, SGI0, SGI128633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28052[] = {SGI128634, SGI128635, SGI128636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128637, SGI0, SGI0, SGI0, SGI128638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28053[] = {SGI128639, SGI128640, SGI128641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128642, SGI0, SGI0, SGI0, SGI128643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28054[] = {SGI0, SGI128644, SGI128645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128646, SGI0, SGI0, SGI0, SGI128647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28055[] = {SGI0, SGI128648, SGI128649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128650, SGI0, SGI0, SGI0, SGI128651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28056[] = {SGI128652, SGI128653, SGI128654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128655, SGI0, SGI0, SGI0, SGI128656, SGI128656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28057[] = {SGI128657, SGI128658, SGI128659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128660, SGI0, SGI0, SGI0, SGI128661, SGI128661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28058[] = {SGI128662, SGI128663, SGI128664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128665, SGI0, SGI0, SGI0, SGI128666, SGI128666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28059[] = {SGI128667, SGI128668, SGI128669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128670, SGI0, SGI0, SGI0, SGI128671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28060[] = {SGI128672, SGI128673, SGI128674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128675, SGI0, SGI0, SGI0, SGI128676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28061[] = {SGI128677, SGI128678, SGI128679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128680, SGI0, SGI0, SGI0, SGI128681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28062[] = {SGI128682, SGI128683, SGI128684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128685, SGI0, SGI0, SGI0, SGI128686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28063[] = {SGI128687, SGI128688, SGI128689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128690, SGI0, SGI0, SGI0, SGI128693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28064[] = {SGI128691, SGI128692, SGI128694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128696, SGI0, SGI0, SGI0, SGI128697, SGI128697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28065[] = {SGI128698, SGI128699, SGI128700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128701, SGI0, SGI0, SGI0, SGI128702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28066[] = {SGI128703, SGI128704, SGI128705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128706, SGI0, SGI0, SGI0, SGI128707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28067[] = {SGI128708, SGI0, SGI128709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28068[] = {SGI128710, SGI128711, SGI128712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128713, SGI0, SGI0, SGI0, SGI128714, SGI128714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28069[] = {SGI128715, SGI128716, SGI128717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128718, SGI0, SGI0, SGI0, SGI128719, SGI128719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28070[] = {SGI128720, SGI128721, SGI128722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128723, SGI0, SGI0, SGI0, SGI128724, SGI128724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28071[] = {SGI128725, SGI128726, SGI128727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128728, SGI0, SGI0, SGI0, SGI128729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28072[] = {SGI128730, SGI128731, SGI128732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128733, SGI0, SGI0, SGI0, SGI128734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28073[] = {SGI128735, SGI128736, SGI128737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128738, SGI0, SGI0, SGI0, SGI128739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28074[] = {SGI128740, SGI128741, SGI128742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128743, SGI0, SGI0, SGI0, SGI128744, SGI128744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28075[] = {SGI128745, SGI128746, SGI128747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128748, SGI0, SGI0, SGI0, SGI128749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28076[] = {SGI128750, SGI128751, SGI128752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128753, SGI128753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28077[] = {SGI128754, SGI128755, SGI128756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128757, SGI0, SGI0, SGI0, SGI128760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28078[] = {SGI128758, SGI128759, SGI128761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128762, SGI0, SGI0, SGI0, SGI128763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128764, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28079[] = {SGI128765, SGI128766, SGI128767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128768, SGI0, SGI0, SGI0, SGI128769, SGI128769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28080[] = {SGI128770, SGI128771, SGI128772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128773, SGI0, SGI0, SGI0, SGI128774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28081[] = {SGI128775, SGI128776, SGI128777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128778, SGI0, SGI0, SGI0, SGI128779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28082[] = {SGI128780, SGI128781, SGI128782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128783, SGI0, SGI0, SGI0, SGI128784, SGI128784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28083[] = {SGI128785, SGI128786, SGI128787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128788, SGI0, SGI0, SGI0, SGI128789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28084[] = {SGI128790, SGI128791, SGI128792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128793, SGI0, SGI0, SGI0, SGI128794, SGI128794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28085[] = {SGI128795, SGI0, SGI128796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28086[] = {SGI128797, SGI128798, SGI128799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128800, SGI0, SGI0, SGI0, SGI128801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28087[] = {SGI128802, SGI128803, SGI128804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128805, SGI0, SGI0, SGI0, SGI128806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28088[] = {SGI128807, SGI128808, SGI128809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128810, SGI0, SGI0, SGI0, SGI128811, SGI128811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28089[] = {SGI128812, SGI128813, SGI128814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128815, SGI0, SGI0, SGI0, SGI128816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28090[] = {SGI128817, SGI128818, SGI128819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128820, SGI0, SGI0, SGI0, SGI128821, SGI128821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28091[] = {SGI128822, SGI128823, SGI128824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128825, SGI0, SGI0, SGI0, SGI128826, SGI128826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28092[] = {SGI128827, SGI128828, SGI128829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128830, SGI0, SGI0, SGI0, SGI128831, SGI128831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28093[] = {SGI128832, SGI128833, SGI128834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128835, SGI128835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28094[] = {SGI128836, SGI128837, SGI128838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128839, SGI0, SGI0, SGI0, SGI128840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28095[] = {SGI128841, SGI128842, SGI128843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128844, SGI0, SGI0, SGI0, SGI128845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28096[] = {SGI128846, SGI128847, SGI128848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128849, SGI0, SGI0, SGI0, SGI128850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28097[] = {SGI128851, SGI128852, SGI128853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128854, SGI0, SGI0, SGI0, SGI128855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28098[] = {SGI128856, SGI128857, SGI128858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128859, SGI0, SGI0, SGI0, SGI128860, SGI128860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28099[] = {SGI128861, SGI128862, SGI128863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128864, SGI0, SGI0, SGI0, SGI128865, SGI128865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128866, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28100[] = {SGI128867, SGI128868, SGI128869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128870, SGI0, SGI0, SGI0, SGI128871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28101[] = {SGI128872, SGI128873, SGI128874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128875, SGI0, SGI0, SGI0, SGI128876, SGI128876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28102[] = {SGI128877, SGI128878, SGI128879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128880, SGI0, SGI0, SGI0, SGI128881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28103[] = {SGI128882, SGI128883, SGI128884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128885, SGI0, SGI0, SGI0, SGI128886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28104[] = {SGI128887, SGI128888, SGI128889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128890, SGI0, SGI0, SGI0, SGI128891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28105[] = {SGI128892, SGI128893, SGI128894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128895, SGI0, SGI0, SGI0, SGI128896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28106[] = {SGI128897, SGI128898, SGI128899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128900, SGI0, SGI0, SGI0, SGI128901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28107[] = {SGI128902, SGI128903, SGI128904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128905, SGI0, SGI0, SGI0, SGI128906, SGI128906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28108[] = {SGI128907, SGI128908, SGI128909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128910, SGI0, SGI0, SGI0, SGI128911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28109[] = {SGI128912, SGI128913, SGI128914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128915, SGI0, SGI0, SGI0, SGI128916, SGI128916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28110[] = {SGI128917, SGI128918, SGI128919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128920, SGI0, SGI0, SGI0, SGI128921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28111[] = {SGI128922, SGI0, SGI128923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28112[] = {SGI128924, SGI128925, SGI128926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128927, SGI0, SGI0, SGI0, SGI128928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28113[] = {SGI128929, SGI128930, SGI128931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128932, SGI0, SGI0, SGI0, SGI128933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28114[] = {SGI128934, SGI128935, SGI128936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128937, SGI0, SGI0, SGI0, SGI128938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28115[] = {SGI128939, SGI128940, SGI128941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128942, SGI0, SGI0, SGI0, SGI128943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28116[] = {SGI128944, SGI0, SGI128945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28117[] = {SGI128946, SGI128947, SGI128948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128949, SGI0, SGI0, SGI0, SGI128950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28118[] = {SGI128951, SGI128952, SGI128953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128954, SGI0, SGI0, SGI0, SGI128955, SGI0, SGI0, SGI128956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28119[] = {SGI128957, SGI128958, SGI128959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128960, SGI0, SGI0, SGI0, SGI128961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28120[] = {SGI128962, SGI128963, SGI128964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128965, SGI0, SGI0, SGI0, SGI128966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28121[] = {SGI128967, SGI128968, SGI128969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128970, SGI0, SGI0, SGI0, SGI128971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28122[] = {SGI128972, SGI128973, SGI128974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128975, SGI0, SGI0, SGI0, SGI128976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28123[] = {SGI128977, SGI128978, SGI128979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128980, SGI0, SGI0, SGI0, SGI128981, SGI128981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28124[] = {SGI128982, SGI128983, SGI128984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128985, SGI0, SGI0, SGI0, SGI128986, SGI128986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28125[] = {SGI128987, SGI128988, SGI128989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128990, SGI0, SGI0, SGI0, SGI128991, SGI128991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28126[] = {SGI128992, SGI128993, SGI128994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI128995, SGI0, SGI0, SGI0, SGI128996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28127[] = {SGI128997, SGI128998, SGI128999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129000, SGI0, SGI0, SGI0, SGI129001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28128[] = {SGI129002, SGI129003, SGI129004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129005, SGI0, SGI0, SGI0, SGI129006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28129[] = {SGI129007, SGI129008, SGI129009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129010, SGI0, SGI0, SGI0, SGI129011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28130[] = {SGI129012, SGI129013, SGI129014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129015, SGI0, SGI0, SGI0, SGI129016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28131[] = {SGI129017, SGI129018, SGI129019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129020, SGI0, SGI0, SGI0, SGI129021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28132[] = {SGI129022, SGI129023, SGI129024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129025, SGI0, SGI0, SGI0, SGI129026, SGI129026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28133[] = {SGI129027, SGI129028, SGI129029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129030, SGI0, SGI0, SGI0, SGI129031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129033, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28134[] = {SGI129032, SGI129034, SGI129035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129036, SGI0, SGI0, SGI0, SGI129037, SGI129037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28135[] = {SGI129038, SGI129039, SGI129040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129041, SGI0, SGI0, SGI0, SGI129042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28136[] = {SGI129043, SGI129044, SGI129045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129046, SGI0, SGI0, SGI0, SGI129047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28137[] = {SGI129048, SGI129049, SGI129050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129051, SGI0, SGI0, SGI0, SGI129052, SGI129052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28138[] = {SGI129053, SGI129054, SGI129055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129056, SGI0, SGI0, SGI0, SGI129057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28139[] = {SGI129058, SGI129059, SGI129060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129061, SGI0, SGI0, SGI0, SGI129062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28140[] = {SGI129063, SGI129064, SGI129065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129066, SGI0, SGI0, SGI0, SGI129067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28141[] = {SGI129068, SGI129069, SGI129070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129071, SGI0, SGI0, SGI0, SGI129072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28142[] = {SGI129073, SGI129074, SGI129075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129076, SGI0, SGI0, SGI0, SGI129077, SGI129077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28143[] = {SGI129078, SGI129079, SGI129080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129081, SGI0, SGI0, SGI0, SGI129082, SGI129082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28144[] = {SGI129083, SGI129084, SGI129085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129086, SGI0, SGI0, SGI0, SGI129087, SGI129087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28145[] = {SGI129088, SGI129089, SGI129090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129091, SGI0, SGI0, SGI0, SGI129092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28146[] = {SGI129093, SGI129094, SGI129095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129096, SGI0, SGI0, SGI0, SGI129097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28147[] = {SGI0, SGI129098, SGI129099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129100, SGI0, SGI0, SGI0, SGI129101, SGI129101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28148[] = {SGI129102, SGI129103, SGI129104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129105, SGI0, SGI0, SGI0, SGI129106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28149[] = {SGI129107, SGI129108, SGI129109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129110, SGI0, SGI0, SGI0, SGI129111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28150[] = {SGI129112, SGI129113, SGI129114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129115, SGI0, SGI0, SGI0, SGI129116, SGI129116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28151[] = {SGI129117, SGI129118, SGI129119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129120, SGI0, SGI0, SGI0, SGI129121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28152[] = {SGI129122, SGI129123, SGI129124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129125, SGI0, SGI0, SGI0, SGI129126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28153[] = {SGI129127, SGI129128, SGI129129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129130, SGI0, SGI0, SGI0, SGI129131, SGI129131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28154[] = {SGI129132, SGI129133, SGI129134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129135, SGI0, SGI0, SGI0, SGI129136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28155[] = {SGI129137, SGI129138, SGI129139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129140, SGI0, SGI0, SGI0, SGI129141, SGI129141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28156[] = {SGI129142, SGI129143, SGI129144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129145, SGI0, SGI0, SGI0, SGI129146, SGI129146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28157[] = {SGI129147, SGI129148, SGI129149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129150, SGI0, SGI0, SGI0, SGI129151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28158[] = {SGI129152, SGI129153, SGI129154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129155, SGI0, SGI0, SGI0, SGI129156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28159[] = {SGI129157, SGI129158, SGI129160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129163, SGI0, SGI0, SGI0, SGI129164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28160[] = {SGI129159, SGI129161, SGI129162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129165, SGI0, SGI0, SGI0, SGI129166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129167, NULL};
const Segment * const SG28161[] = {SGI129168, SGI129169, SGI129170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129171, SGI0, SGI0, SGI0, SGI129172, SGI129172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28162[] = {SGI129173, SGI129174, SGI129176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129179, SGI0, SGI0, SGI0, SGI129180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28163[] = {SGI129175, SGI129177, SGI129178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129181, SGI0, SGI0, SGI0, SGI129182, SGI129182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28164[] = {SGI129183, SGI129184, SGI129185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129186, SGI0, SGI0, SGI0, SGI129187, SGI129187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28165[] = {SGI129188, SGI129189, SGI129190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129191, SGI0, SGI0, SGI0, SGI129192, SGI129192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28166[] = {SGI129193, SGI129194, SGI129195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129196, SGI0, SGI0, SGI0, SGI129198, SGI129198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28167[] = {SGI129197, SGI129199, SGI129200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129201, SGI0, SGI0, SGI0, SGI129202, SGI129202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28168[] = {SGI129204, SGI0, SGI129206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28169[] = {SGI129203, SGI129205, SGI129207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129208, SGI0, SGI0, SGI0, SGI129209, SGI129209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28170[] = {SGI129210, SGI129211, SGI129212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129214, SGI0, SGI0, SGI0, SGI129217, SGI129217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28171[] = {SGI129213, SGI129215, SGI129216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129218, SGI0, SGI0, SGI0, SGI129219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28172[] = {SGI129220, SGI129221, SGI129222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129224, SGI0, SGI0, SGI0, SGI129227, SGI129227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28173[] = {SGI129223, SGI129225, SGI129226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129228, SGI0, SGI0, SGI0, SGI129229, SGI129229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28174[] = {SGI129230, SGI129231, SGI129232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129233, SGI0, SGI0, SGI0, SGI129236, SGI129236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28175[] = {SGI129234, SGI129235, SGI129237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129238, SGI0, SGI0, SGI0, SGI129239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28176[] = {SGI129240, SGI129241, SGI129242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129243, SGI0, SGI0, SGI0, SGI129244, SGI129244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28177[] = {SGI129245, SGI129246, SGI129247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129248, SGI0, SGI0, SGI0, SGI129250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28178[] = {SGI129249, SGI129251, SGI129252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129253, SGI0, SGI0, SGI0, SGI129254, SGI129254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28179[] = {SGI129255, SGI129256, SGI129257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129259, SGI0, SGI0, SGI0, SGI129262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28180[] = {SGI129258, SGI129260, SGI129261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129263, SGI0, SGI0, SGI0, SGI129264, SGI129264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28181[] = {SGI129265, SGI129266, SGI129267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129269, SGI0, SGI0, SGI0, SGI129272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28182[] = {SGI129268, SGI129270, SGI129271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129273, SGI0, SGI0, SGI0, SGI129274, SGI129274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28183[] = {SGI129278, SGI0, SGI129279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28184[] = {SGI129275, SGI129276, SGI129277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129280, SGI0, SGI0, SGI0, SGI129281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28185[] = {SGI129282, SGI129283, SGI129284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129285, SGI129285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28186[] = {SGI129286, SGI129287, SGI129288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129289, SGI0, SGI0, SGI0, SGI129290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28187[] = {SGI129291, SGI129292, SGI129293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129294, SGI0, SGI0, SGI0, SGI129295, SGI129295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28188[] = {SGI129296, SGI129297, SGI129298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129299, SGI0, SGI0, SGI0, SGI129300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28189[] = {SGI129301, SGI0, SGI129302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129303, SGI0, SGI0, SGI0, SGI129304, SGI129304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28190[] = {SGI129305, SGI129306, SGI129307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129308, SGI0, SGI0, SGI0, SGI129309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28191[] = {SGI129310, SGI129311, SGI129312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129314, SGI0, SGI0, SGI0, SGI129317, SGI129317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28192[] = {SGI129313, SGI129315, SGI129316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129318, SGI0, SGI0, SGI0, SGI129319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28193[] = {SGI129320, SGI0, SGI129321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28194[] = {SGI129322, SGI129323, SGI129324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129328, SGI0, SGI0, SGI0, SGI129329, SGI129329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28195[] = {SGI129325, SGI129326, SGI129327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129330, SGI129330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28196[] = {SGI129331, SGI129332, SGI129333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129334, SGI0, SGI0, SGI0, SGI129335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28197[] = {SGI129336, SGI129337, SGI129338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129339, SGI0, SGI0, SGI0, SGI129340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28198[] = {SGI129341, SGI129342, SGI129343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129344, SGI0, SGI0, SGI0, SGI129345, SGI129345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28199[] = {SGI129346, SGI129347, SGI129348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129349, SGI0, SGI0, SGI0, SGI129350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28200[] = {SGI129351, SGI129352, SGI129353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129354, SGI0, SGI0, SGI0, SGI129355, SGI129355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28201[] = {SGI129356, SGI129357, SGI129358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129359, SGI0, SGI0, SGI0, SGI129360, SGI129360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28202[] = {SGI129361, SGI129362, SGI129363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129364, SGI0, SGI0, SGI0, SGI129365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28203[] = {SGI129366, SGI129367, SGI129368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129369, SGI0, SGI0, SGI0, SGI129370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28204[] = {SGI129371, SGI129372, SGI129373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129374, SGI0, SGI0, SGI0, SGI129375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28205[] = {SGI129376, SGI129377, SGI129378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129379, SGI0, SGI0, SGI0, SGI129380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28206[] = {SGI129381, SGI129382, SGI129383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129384, SGI0, SGI0, SGI0, SGI129385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28207[] = {SGI129386, SGI129387, SGI129388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129389, SGI0, SGI0, SGI0, SGI129390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28208[] = {SGI129391, SGI129392, SGI129393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129394, SGI0, SGI0, SGI0, SGI129395, SGI129395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28209[] = {SGI129396, SGI129397, SGI129398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129399, SGI0, SGI0, SGI0, SGI129400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28210[] = {SGI129401, SGI129402, SGI129403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129404, SGI0, SGI0, SGI0, SGI129405, SGI129405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28211[] = {SGI129406, SGI0, SGI129407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28212[] = {SGI129408, SGI129409, SGI129410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129411, SGI0, SGI0, SGI0, SGI129412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28213[] = {SGI129413, SGI129414, SGI129415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129416, SGI0, SGI0, SGI0, SGI129417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28214[] = {SGI129418, SGI129419, SGI129420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129421, SGI0, SGI0, SGI0, SGI129422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28215[] = {SGI129423, SGI0, SGI129424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129425, SGI129426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129427, SGI0, NULL};
const Segment * const SG28216[] = {SGI129428, SGI129429, SGI129430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129431, SGI0, SGI0, SGI0, SGI129432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28217[] = {SGI129433, SGI129434, SGI129435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129436, SGI0, SGI0, SGI0, SGI129437, SGI129437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28218[] = {SGI129438, SGI129439, SGI129440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129441, SGI0, SGI0, SGI0, SGI129442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28219[] = {SGI129443, SGI129444, SGI129445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129446, SGI0, SGI0, SGI0, SGI129447, SGI129447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28220[] = {SGI129448, SGI129449, SGI129450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129451, SGI0, SGI0, SGI0, SGI129452, SGI129452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28221[] = {SGI129453, SGI129454, SGI129455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129456, SGI0, SGI0, SGI0, SGI129457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28222[] = {SGI129458, SGI129459, SGI129460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129461, SGI0, SGI0, SGI0, SGI129462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28223[] = {SGI129463, SGI129464, SGI129465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129466, SGI0, SGI0, SGI0, SGI129467, SGI129467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28224[] = {SGI129468, SGI129469, SGI129470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129471, SGI0, SGI0, SGI0, SGI129472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28225[] = {SGI129473, SGI129474, SGI129475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129476, SGI0, SGI0, SGI0, SGI129477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28226[] = {SGI129478, SGI129479, SGI129480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129481, SGI0, SGI0, SGI0, SGI129482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28227[] = {SGI129483, SGI129484, SGI129485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129486, SGI0, SGI0, SGI0, SGI129487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28228[] = {SGI129488, SGI129489, SGI129490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129491, SGI0, SGI0, SGI0, SGI129492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28229[] = {SGI129493, SGI129494, SGI129495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129496, SGI0, SGI0, SGI0, SGI129497, SGI129497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28230[] = {SGI129498, SGI129499, SGI129500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129501, SGI0, SGI0, SGI0, SGI129502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28231[] = {SGI129503, SGI129504, SGI129505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129506, SGI0, SGI0, SGI0, SGI129507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28232[] = {SGI129508, SGI129509, SGI129510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129511, SGI0, SGI0, SGI0, SGI129512, SGI129512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28233[] = {SGI129513, SGI129514, SGI129515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129516, SGI0, SGI0, SGI0, SGI0, SGI129517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28234[] = {SGI129518, SGI129519, SGI129520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129521, SGI0, SGI0, SGI0, SGI129522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28235[] = {SGI129523, SGI129524, SGI129525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129526, SGI0, SGI0, SGI0, SGI129527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28236[] = {SGI129528, SGI129529, SGI129530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129531, SGI0, SGI0, SGI0, SGI129532, SGI129532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28237[] = {SGI129533, SGI129534, SGI129535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129536, SGI0, SGI0, SGI0, SGI129537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28238[] = {SGI129538, SGI129539, SGI129540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129541, SGI0, SGI0, SGI0, SGI129542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28239[] = {SGI129543, SGI129544, SGI129545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129546, SGI0, SGI0, SGI0, SGI129547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28240[] = {SGI129548, SGI129549, SGI129550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129551, SGI0, SGI0, SGI0, SGI129552, SGI129552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28241[] = {SGI129553, SGI129554, SGI129555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129556, SGI0, SGI0, SGI0, SGI129557, SGI129557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28242[] = {SGI129558, SGI129559, SGI129560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129561, SGI0, SGI0, SGI0, SGI129562, SGI129562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28243[] = {SGI129563, SGI129564, SGI129565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129566, SGI0, SGI0, SGI0, SGI129567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28244[] = {SGI129568, SGI129569, SGI129570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129571, SGI0, SGI0, SGI0, SGI129572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28245[] = {SGI129573, SGI129574, SGI129575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129576, SGI0, SGI0, SGI0, SGI129577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28246[] = {SGI129578, SGI129579, SGI129580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129581, SGI0, SGI0, SGI0, SGI129582, SGI129582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28247[] = {SGI129583, SGI129584, SGI129585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129586, SGI0, SGI0, SGI0, SGI129587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28248[] = {SGI129588, SGI129589, SGI129590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129591, SGI0, SGI0, SGI0, SGI129592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28249[] = {SGI129593, SGI129594, SGI129595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129596, SGI0, SGI0, SGI0, SGI129597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28250[] = {SGI129598, SGI129599, SGI129600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129601, SGI0, SGI0, SGI0, SGI129602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28251[] = {SGI129603, SGI129604, SGI129605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129606, SGI0, SGI0, SGI0, SGI129607, SGI129607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28252[] = {SGI129609, SGI129611, SGI129613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129615, SGI0, SGI0, SGI0, SGI129617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28253[] = {SGI129608, SGI129610, SGI129612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129614, SGI0, SGI0, SGI0, SGI129616, SGI129616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28254[] = {SGI129618, SGI129619, SGI129620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129621, SGI0, SGI0, SGI0, SGI129622, SGI129622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28255[] = {SGI129623, SGI129624, SGI129625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129626, SGI0, SGI0, SGI0, SGI129627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28256[] = {SGI129628, SGI129629, SGI129630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129631, SGI0, SGI0, SGI0, SGI129632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28257[] = {SGI129633, SGI129634, SGI129635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129636, SGI0, SGI0, SGI0, SGI129637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28258[] = {SGI129638, SGI129639, SGI129640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129641, SGI0, SGI0, SGI0, SGI129642, SGI129642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28259[] = {SGI129643, SGI129644, SGI129645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129646, SGI0, SGI0, SGI0, SGI129647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28260[] = {SGI129648, SGI129649, SGI129650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129651, SGI0, SGI0, SGI0, SGI129652, SGI129652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28261[] = {SGI129653, SGI129654, SGI129655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129656, SGI0, SGI0, SGI0, SGI129657, SGI129657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28262[] = {SGI129658, SGI129659, SGI129660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129661, SGI0, SGI0, SGI0, SGI129663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28263[] = {SGI129662, SGI129664, SGI129665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129666, SGI0, SGI0, SGI0, SGI129667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28264[] = {SGI129668, SGI129669, SGI129671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129674, SGI0, SGI0, SGI0, SGI129676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28265[] = {SGI129670, SGI129672, SGI129673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129675, SGI0, SGI0, SGI0, SGI129677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28266[] = {SGI129678, SGI129679, SGI129680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129681, SGI0, SGI0, SGI0, SGI129682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28267[] = {SGI129683, SGI129684, SGI129685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129686, SGI0, SGI0, SGI0, SGI129687, SGI129687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28268[] = {SGI129688, SGI129689, SGI129690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129691, SGI0, SGI0, SGI0, SGI129692, SGI129692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28269[] = {SGI129693, SGI129694, SGI129695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129696, SGI0, SGI0, SGI0, SGI129697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28270[] = {SGI129698, SGI129699, SGI129700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129701, SGI0, SGI0, SGI0, SGI129702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28271[] = {SGI129703, SGI129704, SGI129705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129706, SGI0, SGI0, SGI0, SGI129707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28272[] = {SGI129708, SGI129709, SGI129710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129711, SGI0, SGI0, SGI0, SGI129712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28273[] = {SGI129713, SGI129714, SGI129715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129716, SGI0, SGI0, SGI0, SGI129717, SGI129717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28274[] = {SGI129718, SGI129719, SGI129720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129721, SGI0, SGI0, SGI0, SGI129722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28275[] = {SGI129723, SGI129724, SGI129725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129726, SGI0, SGI0, SGI0, SGI129727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28276[] = {SGI129728, SGI129729, SGI129730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129733, SGI0, SGI0, SGI0, SGI129735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28277[] = {SGI129731, SGI129732, SGI129734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129736, SGI0, SGI0, SGI0, SGI129737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28278[] = {SGI129738, SGI129739, SGI129741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129744, SGI0, SGI0, SGI0, SGI129746, SGI129746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28279[] = {SGI129740, SGI129742, SGI129743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129745, SGI0, SGI0, SGI0, SGI129747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28280[] = {SGI129748, SGI129749, SGI129750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129751, SGI0, SGI0, SGI0, SGI129752, SGI129752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28281[] = {SGI129753, SGI129754, SGI129755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129756, SGI0, SGI0, SGI0, SGI129757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28282[] = {SGI129758, SGI129759, SGI129760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129761, SGI0, SGI0, SGI0, SGI129762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28283[] = {SGI129763, SGI129764, SGI129765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129766, SGI0, SGI0, SGI0, SGI129767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28284[] = {SGI129768, SGI129769, SGI129770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129771, SGI0, SGI0, SGI0, SGI129772, SGI129772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28285[] = {SGI129773, SGI129774, SGI129775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129776, SGI0, SGI0, SGI0, SGI129777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28286[] = {SGI129778, SGI129779, SGI129780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129781, SGI0, SGI0, SGI0, SGI129782, SGI129782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28287[] = {SGI129783, SGI129784, SGI129785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129786, SGI0, SGI0, SGI0, SGI129787, SGI129787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28288[] = {SGI129788, SGI129789, SGI129790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129791, SGI0, SGI0, SGI0, SGI129792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28289[] = {SGI129793, SGI129794, SGI129795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129796, SGI0, SGI0, SGI0, SGI129797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28290[] = {SGI129798, SGI129799, SGI129800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129801, SGI0, SGI0, SGI0, SGI129802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28291[] = {SGI129803, SGI129804, SGI129805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129806, SGI0, SGI0, SGI0, SGI129807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28292[] = {SGI129808, SGI129809, SGI129810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129811, SGI0, SGI0, SGI0, SGI129812, SGI129812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28293[] = {SGI129813, SGI129814, SGI129815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129816, SGI0, SGI0, SGI0, SGI129817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28294[] = {SGI129818, SGI129819, SGI129820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129821, SGI0, SGI0, SGI0, SGI129822, SGI129822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28295[] = {SGI129824, SGI129826, SGI129828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129830, SGI0, SGI0, SGI0, SGI129832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28296[] = {SGI129823, SGI129825, SGI129827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129829, SGI0, SGI0, SGI0, SGI129831, SGI129831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28297[] = {SGI129833, SGI129834, SGI129835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129836, SGI0, SGI0, SGI0, SGI129837, SGI129837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28298[] = {SGI129838, SGI129839, SGI129840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129841, SGI0, SGI0, SGI0, SGI129842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28299[] = {SGI129843, SGI129844, SGI129845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129846, SGI0, SGI0, SGI0, SGI129847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28300[] = {SGI129848, SGI129849, SGI129850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129851, SGI0, SGI0, SGI0, SGI129852, SGI129852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28301[] = {SGI129853, SGI129854, SGI129855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129856, SGI0, SGI0, SGI0, SGI129857, SGI129857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28302[] = {SGI129858, SGI129859, SGI129860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129861, SGI0, SGI0, SGI0, SGI129862, SGI129862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28303[] = {SGI129863, SGI129864, SGI129865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129866, SGI0, SGI0, SGI0, SGI129867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28304[] = {SGI129868, SGI129869, SGI129870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129871, SGI0, SGI0, SGI0, SGI129872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28305[] = {SGI129873, SGI0, SGI129874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28306[] = {SGI129875, SGI129876, SGI129877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129878, SGI0, SGI0, SGI0, SGI129879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28307[] = {SGI129880, SGI129881, SGI129882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129883, SGI0, SGI0, SGI0, SGI129884, SGI129884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28308[] = {SGI129885, SGI0, SGI129886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28309[] = {SGI129887, SGI129888, SGI129889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129891, SGI0, SGI0, SGI0, SGI129893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28310[] = {SGI129890, SGI0, SGI129892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28311[] = {SGI129894, SGI0, SGI129895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129896, SGI0, SGI0, SGI0, SGI129897, SGI129897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28312[] = {SGI129898, SGI129899, SGI129900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129901, SGI0, SGI0, SGI0, SGI129902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28313[] = {SGI129903, SGI129904, SGI129905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129908, SGI0, SGI0, SGI0, SGI129909, SGI129909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28314[] = {SGI129906, SGI0, SGI129907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129910, SGI0, SGI0, SGI0, SGI129911, SGI129911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28315[] = {SGI129912, SGI129913, SGI129914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129915, SGI0, SGI0, SGI0, SGI129916, SGI129916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28316[] = {SGI129917, SGI129918, SGI129919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129920, SGI0, SGI0, SGI0, SGI129921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28317[] = {SGI129922, SGI129923, SGI129924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129925, SGI0, SGI0, SGI0, SGI129926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28318[] = {SGI129927, SGI129928, SGI129929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129930, SGI0, SGI0, SGI0, SGI129931, SGI129931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28319[] = {SGI129932, SGI129933, SGI129934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129935, SGI0, SGI0, SGI0, SGI129936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28320[] = {SGI129937, SGI129938, SGI129939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129940, SGI0, SGI0, SGI0, SGI129941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28321[] = {SGI129942, SGI129943, SGI129944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129945, SGI0, SGI0, SGI0, SGI129946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28322[] = {SGI129947, SGI129948, SGI129949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129950, SGI0, SGI0, SGI0, SGI129951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28323[] = {SGI129952, SGI129953, SGI129954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129955, SGI0, SGI0, SGI0, SGI129956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28324[] = {SGI129957, SGI129958, SGI129959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129960, SGI0, SGI0, SGI0, SGI129961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28325[] = {SGI129962, SGI0, SGI129963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129964, SGI0, SGI0, SGI0, SGI129965, SGI129965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28326[] = {SGI129966, SGI129967, SGI129968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129969, SGI0, SGI0, SGI0, SGI129970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28327[] = {SGI129971, SGI129972, SGI129973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129974, SGI0, SGI0, SGI0, SGI129975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28328[] = {SGI129976, SGI129977, SGI129978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129979, SGI0, SGI0, SGI0, SGI129980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28329[] = {SGI129981, SGI129982, SGI129983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129984, SGI0, SGI0, SGI0, SGI129985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28330[] = {SGI129986, SGI129987, SGI129988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129989, SGI0, SGI0, SGI0, SGI129990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28331[] = {SGI129991, SGI129992, SGI129993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129994, SGI0, SGI0, SGI0, SGI129995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28332[] = {SGI129996, SGI129997, SGI129998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI129999, SGI0, SGI0, SGI0, SGI130000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28333[] = {SGI130001, SGI130002, SGI130003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130004, SGI0, SGI0, SGI0, SGI130005, SGI130005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28334[] = {SGI130006, SGI130007, SGI130008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130009, SGI0, SGI0, SGI0, SGI130010, SGI130010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28335[] = {SGI130011, SGI130012, SGI130013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130014, SGI0, SGI0, SGI0, SGI130015, SGI130015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28336[] = {SGI130016, SGI130017, SGI130018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130019, SGI0, SGI0, SGI0, SGI130020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28337[] = {SGI130021, SGI130022, SGI130023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130024, SGI0, SGI0, SGI0, SGI130025, SGI130025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28338[] = {SGI130026, SGI0, SGI130027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28339[] = {SGI130028, SGI130029, SGI130030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130031, SGI0, SGI0, SGI0, SGI130032, SGI130032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28340[] = {SGI130033, SGI130034, SGI130035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130036, SGI0, SGI0, SGI0, SGI130037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28341[] = {SGI130038, SGI130039, SGI130040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130041, SGI0, SGI0, SGI0, SGI130042, SGI130042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28342[] = {SGI130043, SGI130044, SGI130045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130046, SGI0, SGI0, SGI0, SGI130047, SGI130047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28343[] = {SGI130048, SGI130049, SGI130050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130051, SGI0, SGI0, SGI0, SGI130052, SGI130052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28344[] = {SGI130053, SGI130054, SGI130055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130056, SGI0, SGI0, SGI0, SGI130057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28345[] = {SGI130058, SGI130059, SGI130060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130061, SGI0, SGI0, SGI0, SGI130062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28346[] = {SGI130063, SGI130064, SGI130065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130067, SGI0, SGI0, SGI0, SGI130070, SGI130070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28347[] = {SGI130066, SGI130068, SGI130069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130071, SGI0, SGI0, SGI0, SGI130072, SGI130072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28348[] = {SGI130073, SGI130074, SGI130075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130076, SGI0, SGI0, SGI0, SGI130077, SGI130077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28349[] = {SGI130078, SGI130079, SGI130080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130081, SGI0, SGI0, SGI0, SGI130082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28350[] = {SGI130083, SGI130084, SGI130085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130086, SGI0, SGI0, SGI0, SGI130087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28351[] = {SGI130088, SGI130089, SGI130090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130091, SGI0, SGI0, SGI0, SGI130092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28352[] = {SGI130093, SGI130094, SGI130095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130096, SGI0, SGI0, SGI0, SGI130097, SGI130097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28353[] = {SGI130098, SGI0, SGI130099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28354[] = {SGI130100, SGI130101, SGI130102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130103, SGI0, SGI0, SGI0, SGI130104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28355[] = {SGI130105, SGI130106, SGI130107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130108, SGI0, SGI0, SGI0, SGI130109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28356[] = {SGI130110, SGI130111, SGI130112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130113, SGI0, SGI0, SGI0, SGI130114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28357[] = {SGI130115, SGI130116, SGI130117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130118, SGI0, SGI0, SGI0, SGI130119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28358[] = {SGI130120, SGI130121, SGI130122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130123, SGI0, SGI0, SGI0, SGI130124, SGI130124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28359[] = {SGI130125, SGI130126, SGI130127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130128, SGI0, SGI0, SGI0, SGI130129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28360[] = {SGI130130, SGI130131, SGI130132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130133, SGI0, SGI0, SGI0, SGI130134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28361[] = {SGI130135, SGI130136, SGI130137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130138, SGI0, SGI0, SGI0, SGI130140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28362[] = {SGI130139, SGI130141, SGI130142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130143, SGI0, SGI0, SGI0, SGI130144, SGI130144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28363[] = {SGI130145, SGI130146, SGI130147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130148, SGI0, SGI0, SGI0, SGI130149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28364[] = {SGI130150, SGI130151, SGI130153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130156, SGI0, SGI0, SGI0, SGI130158, SGI130158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28365[] = {SGI130152, SGI130154, SGI130155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130157, SGI0, SGI0, SGI0, SGI130159, SGI130159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28366[] = {SGI130160, SGI130161, SGI130162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130163, SGI0, SGI0, SGI0, SGI130164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28367[] = {SGI130165, SGI130166, SGI130167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130169, SGI0, SGI0, SGI0, SGI130172, SGI130172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28368[] = {SGI130168, SGI130170, SGI130171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130173, SGI0, SGI0, SGI0, SGI130174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28369[] = {SGI130175, SGI130176, SGI130177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130178, SGI0, SGI0, SGI0, SGI130179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28370[] = {SGI130180, SGI130181, SGI130182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130183, SGI0, SGI0, SGI0, SGI130184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28371[] = {SGI130185, SGI130186, SGI130187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130190, SGI0, SGI0, SGI0, SGI130192, SGI130192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28372[] = {SGI130188, SGI130189, SGI130191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130193, SGI0, SGI0, SGI0, SGI130194, SGI130194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28373[] = {SGI130195, SGI130196, SGI130197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130198, SGI0, SGI0, SGI0, SGI130199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28374[] = {SGI130200, SGI130201, SGI130202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130203, SGI0, SGI0, SGI0, SGI130204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28375[] = {SGI130205, SGI130206, SGI130207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130208, SGI0, SGI0, SGI0, SGI130209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28376[] = {SGI130210, SGI130211, SGI130212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130213, SGI0, SGI0, SGI0, SGI130214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28377[] = {SGI130215, SGI130216, SGI130217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130218, SGI0, SGI0, SGI0, SGI130219, SGI130219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28378[] = {SGI130220, SGI130221, SGI130222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130223, SGI0, SGI0, SGI0, SGI130224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28379[] = {SGI130225, SGI130226, SGI130227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130228, SGI0, SGI0, SGI0, SGI130229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28380[] = {SGI130230, SGI130231, SGI130232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130233, SGI0, SGI0, SGI0, SGI130234, SGI130234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28381[] = {SGI130235, SGI130236, SGI130237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130238, SGI0, SGI0, SGI0, SGI130239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28382[] = {SGI130240, SGI130241, SGI130242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130243, SGI0, SGI0, SGI0, SGI130244, SGI130244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28383[] = {SGI130245, SGI130246, SGI130247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130248, SGI0, SGI0, SGI0, SGI130249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28384[] = {SGI130250, SGI130251, SGI130252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130253, SGI0, SGI0, SGI0, SGI130254, SGI130254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28385[] = {SGI130255, SGI130256, SGI130257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130258, SGI0, SGI0, SGI0, SGI130259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28386[] = {SGI130260, SGI130261, SGI130262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130263, SGI0, SGI0, SGI0, SGI130264, SGI130264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28387[] = {SGI130265, SGI130266, SGI130267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130268, SGI0, SGI0, SGI0, SGI130269, SGI130269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28388[] = {SGI130270, SGI130271, SGI130272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130273, SGI0, SGI0, SGI0, SGI130274, SGI130274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28389[] = {SGI130275, SGI130276, SGI130277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130278, SGI0, SGI0, SGI0, SGI130279, SGI130279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28390[] = {SGI130280, SGI130281, SGI130282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130283, SGI0, SGI0, SGI0, SGI130284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28391[] = {SGI130285, SGI130286, SGI130287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130288, SGI0, SGI0, SGI0, SGI130289, SGI130289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28392[] = {SGI130290, SGI0, SGI130291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28393[] = {SGI130292, SGI130293, SGI130294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130295, SGI0, SGI0, SGI0, SGI130296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28394[] = {SGI130297, SGI130298, SGI130299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130300, SGI0, SGI0, SGI0, SGI130301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28395[] = {SGI130302, SGI130303, SGI130304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130305, SGI0, SGI0, SGI0, SGI130306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28396[] = {SGI130307, SGI130308, SGI130309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130310, SGI0, SGI0, SGI0, SGI130311, SGI130311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28397[] = {SGI130312, SGI130313, SGI130314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130315, SGI0, SGI0, SGI0, SGI130316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28398[] = {SGI130317, SGI130318, SGI130319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130320, SGI0, SGI0, SGI0, SGI130322, SGI130322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28399[] = {SGI130321, SGI130323, SGI130324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130325, SGI0, SGI0, SGI0, SGI0, SGI130326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28400[] = {SGI130327, SGI130328, SGI130329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130330, SGI0, SGI0, SGI0, SGI130331, SGI130331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28401[] = {SGI130332, SGI130333, SGI130334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130335, SGI0, SGI0, SGI0, SGI130336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28402[] = {SGI130337, SGI130338, SGI130339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130341, SGI0, SGI0, SGI0, SGI130344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28403[] = {SGI130340, SGI130342, SGI130343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130345, SGI0, SGI0, SGI0, SGI130346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28404[] = {SGI130347, SGI130348, SGI130349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130350, SGI0, SGI0, SGI0, SGI130353, SGI130353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28405[] = {SGI130351, SGI130352, SGI130354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130355, SGI0, SGI0, SGI0, SGI130356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28406[] = {SGI130357, SGI130358, SGI130359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130360, SGI0, SGI0, SGI0, SGI130363, SGI130363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28407[] = {SGI130361, SGI130362, SGI130364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130365, SGI0, SGI0, SGI0, SGI130366, SGI130366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28408[] = {SGI130367, SGI130368, SGI130369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130370, SGI0, SGI0, SGI0, SGI130371, SGI130371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28409[] = {SGI130372, SGI130373, SGI130374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130375, SGI0, SGI0, SGI0, SGI130377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28410[] = {SGI130376, SGI130378, SGI130379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130380, SGI0, SGI0, SGI0, SGI130381, SGI130381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28411[] = {SGI130382, SGI130384, SGI130386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130388, SGI0, SGI0, SGI0, SGI130390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28412[] = {SGI130383, SGI130385, SGI130387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130389, SGI0, SGI0, SGI0, SGI130391, SGI130391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28413[] = {SGI130392, SGI130393, SGI130394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130395, SGI0, SGI0, SGI0, SGI130396, SGI130396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28414[] = {SGI130397, SGI130398, SGI130399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130403, SGI0, SGI0, SGI0, SGI130404, SGI130404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28415[] = {SGI130400, SGI130401, SGI130402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130405, SGI0, SGI0, SGI0, SGI130406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28416[] = {SGI130407, SGI130408, SGI130409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130410, SGI0, SGI0, SGI0, SGI130411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28417[] = {SGI0, SGI130413, SGI130415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28418[] = {SGI130412, SGI130414, SGI130416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130417, SGI0, SGI0, SGI0, SGI130418, SGI130418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28419[] = {SGI130419, SGI130421, SGI130423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130425, SGI0, SGI0, SGI0, SGI130427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28420[] = {SGI130420, SGI130422, SGI130424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130426, SGI0, SGI0, SGI0, SGI130428, SGI130428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28421[] = {SGI130429, SGI130430, SGI130431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130432, SGI0, SGI0, SGI0, SGI130433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28422[] = {SGI130434, SGI130435, SGI130436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130437, SGI0, SGI0, SGI0, SGI130438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28423[] = {SGI130439, SGI130440, SGI130441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130442, SGI0, SGI0, SGI0, SGI130443, SGI130443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28424[] = {SGI130444, SGI130445, SGI130446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130447, SGI0, SGI0, SGI0, SGI130448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28425[] = {SGI130449, SGI130450, SGI130451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130452, SGI0, SGI0, SGI0, SGI130453, SGI130453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28426[] = {SGI130454, SGI130455, SGI130456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130457, SGI0, SGI0, SGI0, SGI0, SGI130458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28427[] = {SGI130459, SGI130460, SGI130461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130462, SGI0, SGI0, SGI0, SGI130463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28428[] = {SGI130464, SGI130465, SGI130466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130467, SGI0, SGI0, SGI0, SGI130468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28429[] = {SGI130469, SGI130470, SGI130471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130472, SGI0, SGI0, SGI0, SGI130473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28430[] = {SGI130474, SGI130475, SGI130476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130477, SGI0, SGI0, SGI0, SGI130478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28431[] = {SGI130479, SGI130480, SGI130481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130482, SGI0, SGI0, SGI0, SGI130483, SGI130483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28432[] = {SGI130484, SGI130485, SGI130486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130487, SGI0, SGI0, SGI0, SGI130488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28433[] = {SGI130489, SGI130490, SGI130491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130492, SGI0, SGI0, SGI0, SGI130493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28434[] = {SGI130494, SGI130495, SGI130496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130497, SGI0, SGI0, SGI0, SGI130498, SGI130498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28435[] = {SGI130499, SGI130500, SGI130501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130502, SGI0, SGI0, SGI0, SGI130503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28436[] = {SGI130504, SGI130505, SGI130506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130507, SGI0, SGI0, SGI0, SGI130508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28437[] = {SGI130509, SGI130510, SGI130511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130512, SGI0, SGI0, SGI0, SGI130513, SGI130513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28438[] = {SGI130514, SGI130515, SGI130516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130517, SGI0, SGI0, SGI0, SGI130518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28439[] = {SGI130519, SGI130520, SGI130521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130522, SGI0, SGI0, SGI0, SGI130523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28440[] = {SGI130524, SGI130525, SGI130526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130527, SGI0, SGI0, SGI0, SGI130528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28441[] = {SGI130529, SGI130530, SGI130531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130532, SGI0, SGI0, SGI0, SGI130533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28442[] = {SGI130534, SGI130535, SGI130536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130537, SGI0, SGI0, SGI0, SGI130538, SGI130538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28443[] = {SGI130539, SGI130540, SGI130541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130542, SGI0, SGI0, SGI0, SGI130543, SGI130543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28444[] = {SGI130544, SGI130545, SGI130546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130547, SGI0, SGI0, SGI0, SGI130548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28445[] = {SGI130549, SGI130550, SGI130551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130552, SGI0, SGI0, SGI0, SGI130555, SGI130555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28446[] = {SGI130553, SGI130554, SGI130556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130557, SGI0, SGI0, SGI0, SGI130558, SGI130558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28447[] = {SGI130559, SGI130560, SGI130561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130562, SGI0, SGI0, SGI0, SGI130563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28448[] = {SGI0, SGI130564, SGI130565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130566, SGI0, SGI0, SGI0, SGI130567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28449[] = {SGI130568, SGI130569, SGI130570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130571, SGI0, SGI0, SGI0, SGI130572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28450[] = {SGI130573, SGI130574, SGI130576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130579, SGI0, SGI0, SGI0, SGI130581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28451[] = {SGI130575, SGI130577, SGI130578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130580, SGI0, SGI0, SGI0, SGI130582, SGI130582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28452[] = {SGI130583, SGI130584, SGI130585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130586, SGI0, SGI0, SGI0, SGI130587, SGI130587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28453[] = {SGI130588, SGI130589, SGI130590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130591, SGI0, SGI0, SGI0, SGI130592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28454[] = {SGI130593, SGI130594, SGI130595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130596, SGI0, SGI0, SGI0, SGI130597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28455[] = {SGI130598, SGI130599, SGI130600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130601, SGI0, SGI0, SGI0, SGI130602, SGI130602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28456[] = {SGI130603, SGI130604, SGI130605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130606, SGI0, SGI0, SGI0, SGI130607, SGI130607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28457[] = {SGI130608, SGI130609, SGI130610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130611, SGI0, SGI0, SGI0, SGI130612, SGI130612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28458[] = {SGI130613, SGI130614, SGI130615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130616, SGI0, SGI0, SGI0, SGI130617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28459[] = {SGI130618, SGI130619, SGI130620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130621, SGI0, SGI0, SGI0, SGI130622, SGI130622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28460[] = {SGI130623, SGI130624, SGI130625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130626, SGI0, SGI0, SGI0, SGI130627, SGI130627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28461[] = {SGI130628, SGI130629, SGI130630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130631, SGI0, SGI0, SGI0, SGI130632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28462[] = {SGI130633, SGI130634, SGI130635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130636, SGI0, SGI0, SGI0, SGI130637, SGI130637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28463[] = {SGI130638, SGI130639, SGI130640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130641, SGI0, SGI0, SGI0, SGI130642, SGI130642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28464[] = {SGI130643, SGI130644, SGI130645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130646, SGI0, SGI0, SGI0, SGI130647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28465[] = {SGI130648, SGI130649, SGI130650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130651, SGI0, SGI0, SGI0, SGI130654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28466[] = {SGI130652, SGI130653, SGI130655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130656, SGI0, SGI0, SGI0, SGI130657, SGI130657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28467[] = {SGI130658, SGI130659, SGI130660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130661, SGI0, SGI0, SGI0, SGI130663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28468[] = {SGI130662, SGI130664, SGI130665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130666, SGI0, SGI0, SGI0, SGI130667, SGI130667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28469[] = {SGI130668, SGI130669, SGI130670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130671, SGI0, SGI0, SGI0, SGI130672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28470[] = {SGI130673, SGI130674, SGI130675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130676, SGI0, SGI0, SGI0, SGI130677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28471[] = {SGI130678, SGI130679, SGI130680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130681, SGI0, SGI0, SGI0, SGI130682, SGI130682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28472[] = {SGI130683, SGI130684, SGI130685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130686, SGI0, SGI0, SGI0, SGI130687, SGI130687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28473[] = {SGI130688, SGI130689, SGI130690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130691, SGI0, SGI0, SGI0, SGI130692, SGI130692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28474[] = {SGI130693, SGI130694, SGI130695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130696, SGI0, SGI0, SGI0, SGI130697, SGI130697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28475[] = {SGI130698, SGI130699, SGI130700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130701, SGI0, SGI0, SGI0, SGI130702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28476[] = {SGI130703, SGI130704, SGI130705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130706, SGI0, SGI0, SGI0, SGI130707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28477[] = {SGI130708, SGI130709, SGI130710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130711, SGI0, SGI0, SGI0, SGI130712, SGI130712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28478[] = {SGI130713, SGI130714, SGI130715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130716, SGI0, SGI0, SGI0, SGI130717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28479[] = {SGI130718, SGI130719, SGI130720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130721, SGI0, SGI0, SGI0, SGI130722, SGI130722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28480[] = {SGI130723, SGI130724, SGI130725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130726, SGI0, SGI0, SGI0, SGI130727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28481[] = {SGI130728, SGI130729, SGI130730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130731, SGI0, SGI0, SGI0, SGI130732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28482[] = {SGI130733, SGI130734, SGI130735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130736, SGI0, SGI0, SGI0, SGI130737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28483[] = {SGI130738, SGI130739, SGI130740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130741, SGI0, SGI0, SGI0, SGI130742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28484[] = {SGI130743, SGI130744, SGI130745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130746, SGI0, SGI0, SGI0, SGI0, SGI130747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28485[] = {SGI130748, SGI130749, SGI130750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130751, SGI0, SGI0, SGI0, SGI130752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28486[] = {SGI130753, SGI130754, SGI130755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130756, SGI0, SGI0, SGI0, SGI130757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28487[] = {SGI130758, SGI130759, SGI130760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130761, SGI0, SGI0, SGI0, SGI130762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28488[] = {SGI130763, SGI130764, SGI130765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130766, SGI0, SGI0, SGI0, SGI130767, SGI130767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28489[] = {SGI130768, SGI130769, SGI130770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130771, SGI0, SGI0, SGI0, SGI130772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28490[] = {SGI130773, SGI130774, SGI130775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130776, SGI0, SGI0, SGI0, SGI130777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28491[] = {SGI130778, SGI130779, SGI130780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130781, SGI0, SGI0, SGI0, SGI130782, SGI130782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28492[] = {SGI130783, SGI130784, SGI130785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130786, SGI0, SGI0, SGI0, SGI130787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28493[] = {SGI130788, SGI130789, SGI130790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130791, SGI0, SGI0, SGI0, SGI130792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28494[] = {SGI130793, SGI130794, SGI130795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130796, SGI0, SGI0, SGI0, SGI130797, SGI130797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28495[] = {SGI130798, SGI130799, SGI130800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130801, SGI0, SGI0, SGI0, SGI130802, SGI130802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28496[] = {SGI130803, SGI130804, SGI130805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130806, SGI0, SGI0, SGI0, SGI130807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28497[] = {SGI130808, SGI130809, SGI130810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130811, SGI0, SGI0, SGI0, SGI130812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28498[] = {SGI130813, SGI130814, SGI130815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130816, SGI0, SGI0, SGI0, SGI130819, SGI130819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28499[] = {SGI130817, SGI130818, SGI130820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130821, SGI0, SGI0, SGI0, SGI130822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28500[] = {SGI130823, SGI130824, SGI130825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130826, SGI0, SGI0, SGI0, SGI130827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28501[] = {SGI130828, SGI130829, SGI130830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130834, SGI0, SGI0, SGI0, SGI130835, SGI130835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28502[] = {SGI130831, SGI130832, SGI130833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130836, SGI0, SGI0, SGI0, SGI130837, SGI130837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28503[] = {SGI130838, SGI130839, SGI130840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130841, SGI0, SGI0, SGI0, SGI130842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28504[] = {SGI130843, SGI130844, SGI130845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130846, SGI0, SGI0, SGI0, SGI130847, SGI130847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28505[] = {SGI130848, SGI0, SGI130849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28506[] = {SGI130850, SGI130851, SGI130852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130853, SGI0, SGI0, SGI0, SGI130854, SGI130854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28507[] = {SGI130855, SGI130856, SGI130857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130858, SGI0, SGI0, SGI0, SGI130859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28508[] = {SGI130860, SGI130861, SGI130862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130865, SGI0, SGI0, SGI0, SGI130867, SGI130867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28509[] = {SGI130863, SGI130864, SGI130866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130868, SGI0, SGI0, SGI0, SGI130869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28510[] = {SGI130870, SGI130871, SGI130872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130873, SGI0, SGI0, SGI0, SGI130874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28511[] = {SGI130875, SGI130876, SGI130877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130878, SGI0, SGI0, SGI0, SGI130879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28512[] = {SGI130880, SGI130881, SGI130882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130883, SGI0, SGI0, SGI0, SGI130886, SGI130886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28513[] = {SGI130884, SGI130885, SGI130887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130888, SGI0, SGI0, SGI0, SGI130889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28514[] = {SGI130890, SGI130891, SGI130892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130893, SGI0, SGI0, SGI0, SGI130894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28515[] = {SGI130895, SGI130896, SGI130897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130898, SGI0, SGI0, SGI0, SGI130899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28516[] = {SGI130900, SGI130901, SGI130902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130903, SGI0, SGI0, SGI0, SGI130904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28517[] = {SGI130905, SGI130906, SGI130907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130908, SGI0, SGI0, SGI0, SGI130909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28518[] = {SGI130910, SGI130911, SGI130912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130913, SGI0, SGI0, SGI0, SGI130914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28519[] = {SGI130915, SGI130916, SGI130917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130918, SGI0, SGI0, SGI0, SGI130919, SGI130919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28520[] = {SGI130920, SGI130921, SGI130922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130923, SGI0, SGI0, SGI0, SGI130924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28521[] = {SGI130925, SGI130926, SGI130927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130928, SGI0, SGI0, SGI0, SGI130929, SGI130929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28522[] = {SGI130930, SGI130931, SGI130932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130933, SGI0, SGI0, SGI0, SGI130934, SGI130934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28523[] = {SGI130935, SGI130936, SGI130937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130938, SGI0, SGI0, SGI0, SGI130939, SGI130939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28524[] = {SGI130940, SGI130941, SGI130942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130943, SGI0, SGI0, SGI0, SGI130944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28525[] = {SGI130945, SGI130946, SGI130947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130948, SGI0, SGI0, SGI0, SGI130949, SGI130949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28526[] = {SGI130950, SGI130951, SGI130952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130953, SGI0, SGI0, SGI0, SGI130954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28527[] = {SGI130955, SGI130956, SGI130957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130958, SGI0, SGI0, SGI0, SGI130959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28528[] = {SGI130960, SGI130961, SGI130962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130963, SGI0, SGI0, SGI0, SGI130964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28529[] = {SGI130965, SGI130966, SGI130967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130968, SGI0, SGI0, SGI0, SGI130969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28530[] = {SGI130970, SGI130971, SGI130972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130973, SGI0, SGI0, SGI0, SGI130974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28531[] = {SGI130975, SGI130976, SGI130977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130978, SGI0, SGI0, SGI0, SGI130979, SGI130979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28532[] = {SGI130980, SGI130981, SGI130982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130983, SGI0, SGI0, SGI0, SGI130984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28533[] = {SGI130985, SGI130986, SGI130987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130988, SGI0, SGI0, SGI0, SGI130989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28534[] = {SGI130990, SGI130991, SGI130992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130993, SGI0, SGI0, SGI0, SGI130994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28535[] = {SGI130995, SGI130996, SGI130997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI130998, SGI0, SGI0, SGI0, SGI130999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28536[] = {SGI131000, SGI131001, SGI131002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131003, SGI0, SGI0, SGI0, SGI131004, SGI131004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28537[] = {SGI131005, SGI131006, SGI131007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131008, SGI0, SGI0, SGI0, SGI131009, SGI131009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28538[] = {SGI131010, SGI131011, SGI131012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131013, SGI0, SGI0, SGI0, SGI131014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28539[] = {SGI131015, SGI131016, SGI131017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131018, SGI0, SGI0, SGI0, SGI131019, SGI131019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28540[] = {SGI131020, SGI131021, SGI131022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131023, SGI0, SGI0, SGI0, SGI131024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28541[] = {SGI131025, SGI131026, SGI131027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131028, SGI0, SGI0, SGI0, SGI131029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28542[] = {SGI131030, SGI131031, SGI131032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131033, SGI0, SGI0, SGI0, SGI131034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28543[] = {SGI131035, SGI131036, SGI131037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131038, SGI0, SGI0, SGI0, SGI131039, SGI131039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28544[] = {SGI131040, SGI131041, SGI131042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131043, SGI0, SGI0, SGI0, SGI131044, SGI131044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28545[] = {SGI131045, SGI131046, SGI131047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131048, SGI0, SGI0, SGI0, SGI131049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28546[] = {SGI131050, SGI131051, SGI131052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131053, SGI0, SGI0, SGI0, SGI131054, SGI131054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28547[] = {SGI131055, SGI131056, SGI131057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131058, SGI0, SGI0, SGI0, SGI131059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28548[] = {SGI131060, SGI131061, SGI131062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131063, SGI0, SGI0, SGI0, SGI131064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28549[] = {SGI131065, SGI131066, SGI131067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131068, SGI0, SGI0, SGI0, SGI131069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28550[] = {SGI131070, SGI131071, SGI131072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131073, SGI0, SGI0, SGI0, SGI131074, SGI131074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28551[] = {SGI131075, SGI131076, SGI131077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131078, SGI0, SGI0, SGI0, SGI131079, SGI131079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28552[] = {SGI131080, SGI131081, SGI131082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131083, SGI0, SGI0, SGI0, SGI131084, SGI131084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28553[] = {SGI131085, SGI131086, SGI131087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131088, SGI0, SGI0, SGI0, SGI131089, SGI131089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28554[] = {SGI131090, SGI131091, SGI131093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131096, SGI0, SGI0, SGI0, SGI131098, SGI131098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28555[] = {SGI131092, SGI131094, SGI131095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131097, SGI0, SGI0, SGI0, SGI131099, SGI131099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28556[] = {SGI131100, SGI131101, SGI131102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131103, SGI0, SGI0, SGI0, SGI131104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28557[] = {SGI131105, SGI131106, SGI131107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131108, SGI0, SGI0, SGI0, SGI131109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28558[] = {SGI131110, SGI131111, SGI131112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131113, SGI0, SGI0, SGI0, SGI131114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28559[] = {SGI131115, SGI131116, SGI131117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131118, SGI0, SGI0, SGI0, SGI131119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28560[] = {SGI131120, SGI131121, SGI131122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131123, SGI0, SGI0, SGI0, SGI131124, SGI131124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28561[] = {SGI131125, SGI131126, SGI131127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131128, SGI0, SGI0, SGI0, SGI131129, SGI131129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28562[] = {SGI131130, SGI131131, SGI131132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131133, SGI0, SGI0, SGI0, SGI131134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28563[] = {SGI131135, SGI131136, SGI131137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131138, SGI0, SGI0, SGI0, SGI131139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28564[] = {SGI0, SGI131140, SGI131141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131142, SGI0, SGI0, SGI0, SGI131143, SGI131143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28565[] = {SGI131144, SGI131145, SGI131146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131147, SGI0, SGI0, SGI0, SGI131148, SGI131148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28566[] = {SGI131149, SGI131150, SGI131151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131152, SGI0, SGI0, SGI0, SGI131153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28567[] = {SGI131154, SGI131155, SGI131156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131157, SGI0, SGI0, SGI0, SGI131158, SGI131158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28568[] = {SGI131159, SGI131160, SGI131161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131162, SGI0, SGI0, SGI0, SGI0, SGI131163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28569[] = {SGI131164, SGI131165, SGI131166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131167, SGI0, SGI0, SGI0, SGI131168, SGI131168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28570[] = {SGI131169, SGI131170, SGI131171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131175, SGI0, SGI0, SGI0, SGI131176, SGI131176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28571[] = {SGI131172, SGI131173, SGI131174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131177, SGI0, SGI0, SGI0, SGI131178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28572[] = {SGI131179, SGI131180, SGI131182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131185, SGI0, SGI0, SGI0, SGI131187, SGI131187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28573[] = {SGI131181, SGI131183, SGI131184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131186, SGI0, SGI0, SGI0, SGI131188, SGI131188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28574[] = {SGI131189, SGI131190, SGI131191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131192, SGI0, SGI0, SGI0, SGI131193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28575[] = {SGI131194, SGI131195, SGI131196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131197, SGI0, SGI0, SGI0, SGI131198, SGI131198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28576[] = {SGI131200, SGI131202, SGI131204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131205, SGI0, SGI0, SGI0, SGI131207, SGI131207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28577[] = {SGI131199, SGI131201, SGI131203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131206, SGI0, SGI0, SGI0, SGI131208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28578[] = {SGI131209, SGI131210, SGI131211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131212, SGI0, SGI0, SGI0, SGI131213, SGI131213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28579[] = {SGI131214, SGI131215, SGI131216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131217, SGI0, SGI0, SGI0, SGI131218, SGI131218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28580[] = {SGI131219, SGI131220, SGI131221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131222, SGI0, SGI0, SGI0, SGI131223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28581[] = {SGI131224, SGI131225, SGI131226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131227, SGI0, SGI0, SGI0, SGI131228, SGI131228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28582[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28583[] = {SGI131230, SGI131231, SGI131232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131233, SGI0, SGI0, SGI0, SGI131234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28584[] = {SGI131235, SGI131236, SGI131237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131238, SGI0, SGI0, SGI0, SGI131239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28585[] = {SGI131240, SGI131241, SGI131242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131243, SGI0, SGI0, SGI0, SGI131244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28586[] = {SGI131245, SGI131246, SGI131247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131248, SGI0, SGI0, SGI0, SGI131250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28587[] = {SGI131249, SGI131251, SGI131252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131253, SGI0, SGI0, SGI0, SGI131254, SGI131254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28588[] = {SGI131255, SGI131256, SGI131257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131258, SGI0, SGI0, SGI0, SGI131259, SGI131259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28589[] = {SGI131260, SGI131261, SGI131262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131263, SGI0, SGI0, SGI0, SGI131264, SGI131264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28590[] = {SGI131265, SGI131266, SGI131267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131268, SGI0, SGI0, SGI0, SGI131269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28591[] = {SGI131270, SGI131271, SGI131272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131273, SGI0, SGI0, SGI0, SGI131274, SGI131274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28592[] = {SGI131275, SGI131276, SGI131277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131278, SGI0, SGI0, SGI0, SGI131279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28593[] = {SGI131280, SGI131281, SGI131282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131283, SGI0, SGI0, SGI0, SGI131284, SGI131284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28594[] = {SGI131285, SGI131286, SGI131287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131288, SGI0, SGI0, SGI0, SGI131289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28595[] = {SGI131290, SGI131291, SGI131292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131294, SGI0, SGI0, SGI0, SGI131297, SGI131297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28596[] = {SGI131293, SGI131295, SGI131296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131298, SGI0, SGI0, SGI0, SGI131299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28597[] = {SGI131300, SGI131301, SGI131302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131303, SGI0, SGI0, SGI0, SGI131304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28598[] = {SGI131305, SGI131306, SGI131307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131308, SGI0, SGI0, SGI0, SGI131309, SGI131309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28599[] = {SGI131310, SGI131311, SGI131312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131313, SGI0, SGI0, SGI0, SGI131314, SGI131314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28600[] = {SGI131315, SGI131316, SGI131317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131318, SGI0, SGI0, SGI0, SGI131319, SGI131319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28601[] = {SGI131320, SGI131321, SGI131322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131323, SGI0, SGI0, SGI0, SGI131324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28602[] = {SGI131325, SGI131326, SGI131327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131328, SGI0, SGI0, SGI0, SGI131329, SGI131329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28603[] = {SGI131330, SGI131331, SGI131332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131333, SGI0, SGI0, SGI0, SGI131334, SGI131334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28604[] = {SGI131335, SGI131336, SGI131337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131338, SGI0, SGI0, SGI0, SGI131339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28605[] = {SGI131340, SGI131341, SGI131342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131343, SGI0, SGI0, SGI0, SGI131344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28606[] = {SGI131345, SGI131346, SGI131347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131348, SGI0, SGI0, SGI0, SGI131349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28607[] = {SGI131350, SGI131351, SGI131352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131353, SGI0, SGI0, SGI0, SGI131354, SGI131354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28608[] = {SGI131355, SGI131356, SGI131357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131358, SGI0, SGI0, SGI0, SGI131359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28609[] = {SGI131360, SGI131361, SGI131362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131363, SGI0, SGI0, SGI0, SGI131364, SGI131364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28610[] = {SGI131365, SGI131366, SGI131367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131368, SGI0, SGI0, SGI0, SGI131369, SGI131369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28611[] = {SGI131370, SGI131371, SGI131372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131373, SGI0, SGI0, SGI0, SGI131374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28612[] = {SGI131375, SGI0, SGI131376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28613[] = {SGI131377, SGI131378, SGI131379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131380, SGI0, SGI0, SGI0, SGI131381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28614[] = {SGI131382, SGI131383, SGI131384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131385, SGI0, SGI0, SGI0, SGI131386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28615[] = {SGI131387, SGI131388, SGI131389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131390, SGI0, SGI0, SGI0, SGI131391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28616[] = {SGI131392, SGI131393, SGI131394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131395, SGI0, SGI0, SGI0, SGI131397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28617[] = {SGI131396, SGI131398, SGI131399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131400, SGI0, SGI0, SGI0, SGI131401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28618[] = {SGI131402, SGI131403, SGI131404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131406, SGI0, SGI0, SGI0, SGI131409, SGI131409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28619[] = {SGI131405, SGI131407, SGI131408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131410, SGI0, SGI0, SGI0, SGI131411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28620[] = {SGI131412, SGI131413, SGI131414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131415, SGI0, SGI0, SGI0, SGI131416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28621[] = {SGI131417, SGI131418, SGI131419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131420, SGI0, SGI0, SGI0, SGI131421, SGI131421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28622[] = {SGI131422, SGI131423, SGI131424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131425, SGI0, SGI0, SGI0, SGI131426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28623[] = {SGI131427, SGI131428, SGI131429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131430, SGI0, SGI0, SGI0, SGI131431, SGI131431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28624[] = {SGI131432, SGI131433, SGI131434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131435, SGI0, SGI0, SGI0, SGI131436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28625[] = {SGI131437, SGI0, SGI131438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131439, SGI0, SGI0, SGI0, SGI131440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28626[] = {SGI131441, SGI131442, SGI131443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131444, SGI0, SGI0, SGI0, SGI131445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28627[] = {SGI131446, SGI131447, SGI131448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131451, SGI0, SGI0, SGI0, SGI131453, SGI131453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28628[] = {SGI131449, SGI131450, SGI131452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131454, SGI0, SGI0, SGI0, SGI131455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28629[] = {SGI131456, SGI131457, SGI131458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131459, SGI0, SGI0, SGI0, SGI131460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28630[] = {SGI131462, SGI131464, SGI131466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131468, SGI0, SGI0, SGI0, SGI131470, SGI131470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28631[] = {SGI131461, SGI131463, SGI131465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131467, SGI0, SGI0, SGI0, SGI131469, SGI131469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28632[] = {SGI131472, SGI131474, SGI131476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131478, SGI0, SGI0, SGI0, SGI131480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28633[] = {SGI131471, SGI131473, SGI131475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131477, SGI0, SGI0, SGI0, SGI131479, SGI131479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28634[] = {SGI131481, SGI131482, SGI131483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131484, SGI0, SGI0, SGI0, SGI0, SGI131487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28635[] = {SGI131485, SGI131486, SGI131488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131489, SGI0, SGI0, SGI0, SGI131490, SGI131490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28636[] = {SGI131491, SGI131492, SGI131493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131494, SGI0, SGI0, SGI0, SGI131495, SGI131495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28637[] = {SGI131496, SGI131497, SGI131498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131499, SGI0, SGI0, SGI0, SGI131500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28638[] = {SGI131501, SGI131502, SGI131503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131504, SGI0, SGI0, SGI0, SGI131505, SGI131505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28639[] = {SGI131506, SGI131507, SGI131508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131509, SGI0, SGI0, SGI0, SGI131510, SGI131510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28640[] = {SGI131511, SGI131512, SGI131513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131514, SGI0, SGI0, SGI0, SGI131515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28641[] = {SGI131516, SGI131517, SGI131518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131519, SGI0, SGI0, SGI0, SGI131520, SGI131520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28642[] = {SGI131521, SGI131522, SGI131523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131524, SGI0, SGI0, SGI0, SGI131525, SGI131525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28643[] = {SGI131526, SGI131527, SGI131528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131529, SGI0, SGI0, SGI0, SGI131530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28644[] = {SGI131531, SGI131532, SGI131533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131534, SGI0, SGI0, SGI0, SGI131535, SGI131535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28645[] = {SGI131536, SGI131537, SGI131538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131539, SGI0, SGI0, SGI0, SGI0, SGI131540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28646[] = {SGI131541, SGI131542, SGI131543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131544, SGI0, SGI0, SGI0, SGI131545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28647[] = {SGI131546, SGI131547, SGI131548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131549, SGI0, SGI0, SGI0, SGI131550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28648[] = {SGI131551, SGI131552, SGI131553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131554, SGI0, SGI0, SGI0, SGI131555, SGI131555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28649[] = {SGI131556, SGI131557, SGI131558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131559, SGI0, SGI0, SGI0, SGI0, SGI131560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28650[] = {SGI131561, SGI131562, SGI131563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131564, SGI0, SGI0, SGI0, SGI131565, SGI131565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28651[] = {SGI131566, SGI131567, SGI131568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131569, SGI0, SGI0, SGI0, SGI131570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28652[] = {SGI131571, SGI131572, SGI131573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131574, SGI0, SGI0, SGI0, SGI131575, SGI131575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28653[] = {SGI131576, SGI131577, SGI131578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131579, SGI0, SGI0, SGI0, SGI131580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28654[] = {SGI131581, SGI131582, SGI131583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131584, SGI0, SGI0, SGI0, SGI131585, SGI131585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28655[] = {SGI131586, SGI131587, SGI131588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131589, SGI0, SGI0, SGI0, SGI131590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28656[] = {SGI131591, SGI131592, SGI131593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131594, SGI0, SGI0, SGI0, SGI131595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28657[] = {SGI131596, SGI131597, SGI131598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131599, SGI0, SGI0, SGI0, SGI131600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28658[] = {SGI131601, SGI131602, SGI131603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131604, SGI0, SGI0, SGI0, SGI131606, SGI131606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28659[] = {SGI131605, SGI131607, SGI131608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131609, SGI0, SGI0, SGI0, SGI131610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28660[] = {SGI131611, SGI131612, SGI131613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131616, SGI0, SGI0, SGI0, SGI131618, SGI131618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28661[] = {SGI131614, SGI131615, SGI131617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131619, SGI0, SGI0, SGI0, SGI131620, SGI131620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28662[] = {SGI131621, SGI131622, SGI131623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131624, SGI0, SGI0, SGI0, SGI131625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28663[] = {SGI131626, SGI131627, SGI131628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131629, SGI0, SGI0, SGI0, SGI131630, SGI131630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28664[] = {SGI131631, SGI131632, SGI131633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131634, SGI0, SGI0, SGI0, SGI131635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28665[] = {SGI131636, SGI131637, SGI131638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131639, SGI0, SGI0, SGI0, SGI131640, SGI131640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28666[] = {SGI131641, SGI131642, SGI131643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131644, SGI0, SGI0, SGI0, SGI131645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28667[] = {SGI131646, SGI131647, SGI131648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131649, SGI0, SGI0, SGI0, SGI131650, SGI131650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28668[] = {SGI131651, SGI131652, SGI131653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131654, SGI0, SGI0, SGI0, SGI131656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28669[] = {SGI131655, SGI131657, SGI131658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131659, SGI0, SGI0, SGI0, SGI131660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28670[] = {SGI131661, SGI131662, SGI131663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131664, SGI0, SGI0, SGI0, SGI131665, SGI131665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28671[] = {SGI131666, SGI131667, SGI131668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131669, SGI0, SGI0, SGI0, SGI131670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28672[] = {SGI131671, SGI131672, SGI131673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131674, SGI0, SGI0, SGI0, SGI131675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28673[] = {SGI131676, SGI131677, SGI131678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131679, SGI0, SGI0, SGI0, SGI131680, SGI131680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28674[] = {SGI131681, SGI131682, SGI131683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131684, SGI0, SGI0, SGI0, SGI131685, SGI131685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28675[] = {SGI131686, SGI131687, SGI131688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131689, SGI0, SGI0, SGI0, SGI131690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28676[] = {SGI131691, SGI131692, SGI131693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131694, SGI0, SGI0, SGI0, SGI131695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28677[] = {SGI131696, SGI131697, SGI131698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131699, SGI0, SGI0, SGI0, SGI131700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28678[] = {SGI131701, SGI131702, SGI131703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131704, SGI0, SGI0, SGI0, SGI131705, SGI131705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28679[] = {SGI131706, SGI131707, SGI131708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131709, SGI0, SGI0, SGI0, SGI131710, SGI131710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28680[] = {SGI131711, SGI131712, SGI131713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131714, SGI131714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28681[] = {SGI131715, SGI0, SGI131716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28682[] = {SGI131717, SGI131718, SGI131719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131720, SGI0, SGI0, SGI0, SGI131721, SGI131721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28683[] = {SGI131722, SGI131723, SGI131724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131725, SGI0, SGI0, SGI0, SGI131726, SGI131726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28684[] = {SGI131727, SGI131728, SGI131729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131730, SGI131730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28685[] = {SGI131731, SGI131732, SGI131733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131734, SGI0, SGI0, SGI0, SGI131735, SGI131735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28686[] = {SGI131736, SGI131737, SGI131738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131739, SGI0, SGI0, SGI0, SGI131740, SGI131740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28687[] = {SGI0, SGI131741, SGI131742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28688[] = {SGI131743, SGI131744, SGI131745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131746, SGI0, SGI0, SGI0, SGI131747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28689[] = {SGI131748, SGI0, SGI131749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131750, SGI0, SGI0, SGI0, SGI131751, SGI131751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28690[] = {SGI131752, SGI131753, SGI131754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131755, SGI0, SGI0, SGI0, SGI131756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28691[] = {SGI131757, SGI131758, SGI131759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131760, SGI0, SGI0, SGI0, SGI131761, SGI131761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28692[] = {SGI131762, SGI131763, SGI131764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131765, SGI0, SGI0, SGI0, SGI131766, SGI131766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28693[] = {SGI131767, SGI131768, SGI131769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131770, SGI0, SGI0, SGI0, SGI131771, SGI131771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28694[] = {SGI131772, SGI131773, SGI131774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131775, SGI0, SGI0, SGI0, SGI131776, SGI131776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28695[] = {SGI131777, SGI131778, SGI131779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131780, SGI0, SGI0, SGI0, SGI131781, SGI131781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28696[] = {SGI131782, SGI131783, SGI131784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131785, SGI0, SGI0, SGI0, SGI131786, SGI131786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28697[] = {SGI131787, SGI131788, SGI131789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131790, SGI0, SGI0, SGI0, SGI131791, SGI131791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28698[] = {SGI131792, SGI131793, SGI131794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131795, SGI0, SGI0, SGI0, SGI131796, SGI131796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28699[] = {SGI131797, SGI131798, SGI131799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131800, SGI131800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28700[] = {SGI131801, SGI0, SGI131802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28701[] = {SGI131803, SGI131804, SGI131805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131806, SGI0, SGI0, SGI0, SGI131807, SGI131807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28702[] = {SGI131808, SGI131809, SGI131810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131811, SGI0, SGI0, SGI0, SGI131812, SGI131812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28703[] = {SGI131813, SGI131814, SGI131815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131816, SGI0, SGI0, SGI0, SGI131817, SGI131817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28704[] = {SGI131818, SGI131819, SGI131820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131821, SGI0, SGI0, SGI0, SGI131822, SGI131822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28705[] = {SGI131823, SGI131824, SGI131825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131826, SGI0, SGI0, SGI0, SGI131827, SGI131827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28706[] = {SGI131828, SGI131829, SGI131830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131831, SGI0, SGI0, SGI0, SGI131832, SGI131832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28707[] = {SGI131833, SGI131834, SGI131835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131836, SGI0, SGI0, SGI0, SGI131837, SGI131837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28708[] = {SGI131838, SGI131839, SGI131840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131841, SGI0, SGI0, SGI0, SGI131842, SGI131842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28709[] = {SGI131843, SGI131844, SGI131845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131846, SGI0, SGI0, SGI0, SGI131847, SGI131847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28710[] = {SGI131848, SGI131849, SGI131850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131851, SGI0, SGI0, SGI0, SGI131852, SGI131852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28711[] = {SGI131853, SGI131854, SGI131855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131856, SGI0, SGI0, SGI0, SGI131857, SGI131857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28712[] = {SGI131858, SGI131859, SGI131860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131861, SGI0, SGI0, SGI0, SGI131862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28713[] = {SGI131863, SGI131864, SGI131865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131866, SGI0, SGI0, SGI0, SGI131867, SGI131867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28714[] = {SGI131868, SGI131869, SGI131870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131871, SGI0, SGI0, SGI0, SGI131872, SGI131872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28715[] = {SGI131873, SGI131874, SGI131875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131876, SGI0, SGI0, SGI0, SGI131877, SGI131877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28716[] = {SGI131878, SGI131879, SGI131880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131881, SGI0, SGI0, SGI0, SGI131882, SGI131882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28717[] = {SGI131883, SGI131884, SGI131885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131886, SGI0, SGI0, SGI0, SGI131887, SGI131887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28718[] = {SGI131888, SGI131889, SGI131890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131891, SGI0, SGI0, SGI0, SGI131892, SGI131892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28719[] = {SGI131893, SGI131894, SGI131895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131896, SGI0, SGI0, SGI0, SGI131897, SGI131897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28720[] = {SGI131898, SGI131899, SGI131900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131901, SGI0, SGI0, SGI0, SGI131902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28721[] = {SGI131903, SGI131904, SGI131905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131906, SGI0, SGI0, SGI0, SGI131907, SGI131907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28722[] = {SGI131908, SGI131909, SGI131910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131911, SGI0, SGI0, SGI0, SGI131912, SGI131912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28723[] = {SGI131913, SGI131914, SGI131915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131916, SGI0, SGI0, SGI0, SGI131917, SGI131917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28724[] = {SGI131918, SGI131919, SGI131920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131921, SGI0, SGI0, SGI0, SGI131922, SGI131922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28725[] = {SGI131923, SGI131924, SGI131925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131926, SGI0, SGI0, SGI0, SGI131927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28726[] = {SGI131928, SGI131929, SGI131930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131931, SGI0, SGI0, SGI0, SGI131932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28727[] = {SGI131933, SGI0, SGI131934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28728[] = {SGI131935, SGI131936, SGI131937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131938, SGI0, SGI0, SGI0, SGI131939, SGI131939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28729[] = {SGI131940, SGI131941, SGI131942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131943, SGI0, SGI0, SGI0, SGI131944, SGI131944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28730[] = {SGI131945, SGI131946, SGI131947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131948, SGI0, SGI0, SGI0, SGI131949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28731[] = {SGI131950, SGI131951, SGI131952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131953, SGI0, SGI0, SGI0, SGI131954, SGI131954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28732[] = {SGI131955, SGI131956, SGI131957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131958, SGI0, SGI0, SGI0, SGI131959, SGI131959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28733[] = {SGI131960, SGI131961, SGI131962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131963, SGI0, SGI0, SGI0, SGI131964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28734[] = {SGI131965, SGI131966, SGI131967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131968, SGI0, SGI0, SGI0, SGI131969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28735[] = {SGI131970, SGI131971, SGI131972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131973, SGI0, SGI0, SGI0, SGI131974, SGI131974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28736[] = {SGI131975, SGI131976, SGI131977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131978, SGI0, SGI0, SGI0, SGI131979, SGI131979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28737[] = {SGI131980, SGI131981, SGI131982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131983, SGI0, SGI0, SGI0, SGI131984, SGI131984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28738[] = {SGI131985, SGI0, SGI131986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28739[] = {SGI131987, SGI131988, SGI131989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131990, SGI0, SGI0, SGI0, SGI131991, SGI131991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28740[] = {SGI131992, SGI131993, SGI131994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI131995, SGI0, SGI0, SGI0, SGI131996, SGI131996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28741[] = {SGI131997, SGI131998, SGI131999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132000, SGI0, SGI0, SGI0, SGI132001, SGI132001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28742[] = {SGI132002, SGI132003, SGI132004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132005, SGI0, SGI0, SGI0, SGI132006, SGI132006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28743[] = {SGI132007, SGI132008, SGI132009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132010, SGI0, SGI0, SGI0, SGI132011, SGI132011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28744[] = {SGI132012, SGI132013, SGI132014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132015, SGI0, SGI0, SGI0, SGI0, SGI132016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28745[] = {SGI132017, SGI132018, SGI132019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132020, SGI0, SGI0, SGI0, SGI132021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28746[] = {SGI132022, SGI132023, SGI132024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132025, SGI0, SGI0, SGI0, SGI132026, SGI132026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28747[] = {SGI132027, SGI132028, SGI132029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132030, SGI0, SGI0, SGI0, SGI132031, SGI132031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28748[] = {SGI0, SGI132032, SGI132033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28749[] = {SGI132034, SGI132035, SGI132036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132037, SGI0, SGI0, SGI0, SGI132038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28750[] = {SGI132039, SGI132040, SGI132041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132042, SGI0, SGI0, SGI0, SGI132043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28751[] = {SGI132044, SGI132045, SGI132046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132047, SGI0, SGI0, SGI0, SGI132048, SGI132048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28752[] = {SGI132049, SGI132050, SGI132051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132052, SGI0, SGI0, SGI0, SGI132053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28753[] = {SGI132054, SGI132055, SGI132056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132057, SGI0, SGI0, SGI0, SGI132058, SGI132058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28754[] = {SGI132059, SGI132060, SGI132061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132062, SGI0, SGI0, SGI0, SGI132063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28755[] = {SGI132064, SGI132065, SGI132066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132067, SGI0, SGI0, SGI0, SGI132068, SGI132068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28756[] = {SGI132069, SGI132070, SGI132071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132072, SGI0, SGI0, SGI0, SGI132073, SGI132073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28757[] = {SGI132074, SGI132075, SGI132076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132077, SGI0, SGI0, SGI0, SGI132078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28758[] = {SGI132079, SGI132080, SGI132081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132082, SGI0, SGI0, SGI0, SGI132083, SGI132083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28759[] = {SGI132084, SGI132085, SGI132086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132087, SGI0, SGI0, SGI0, SGI132088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28760[] = {SGI132089, SGI0, SGI132090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28761[] = {SGI132091, SGI132092, SGI132093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132094, SGI0, SGI0, SGI0, SGI132095, SGI132095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28762[] = {SGI132096, SGI132097, SGI132098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132099, SGI0, SGI0, SGI0, SGI132100, SGI132100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28763[] = {SGI132101, SGI0, SGI132102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132103, SGI0, SGI0, SGI0, SGI132104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28764[] = {SGI132105, SGI132106, SGI132107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132108, SGI0, SGI0, SGI0, SGI132109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28765[] = {SGI132110, SGI132111, SGI132112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132113, SGI0, SGI0, SGI0, SGI132114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28766[] = {SGI132115, SGI132116, SGI132117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132118, SGI132118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28767[] = {SGI132119, SGI132120, SGI132121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132122, SGI0, SGI0, SGI0, SGI132123, SGI132123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28768[] = {SGI132124, SGI132125, SGI132126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132127, SGI0, SGI0, SGI0, SGI132128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28769[] = {SGI132129, SGI132130, SGI132131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132132, SGI0, SGI0, SGI0, SGI132133, SGI132133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28770[] = {SGI132134, SGI132135, SGI132136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132137, SGI0, SGI0, SGI0, SGI132138, SGI132138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28771[] = {SGI132139, SGI132140, SGI132141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132142, SGI0, SGI0, SGI0, SGI132143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28772[] = {SGI132144, SGI132145, SGI132146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132147, SGI0, SGI0, SGI0, SGI132148, SGI132148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28773[] = {SGI132152, SGI132153, SGI132154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132155, SGI0, SGI0, SGI0, SGI132156, SGI132156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28774[] = {SGI132157, SGI132158, SGI132159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132160, SGI0, SGI0, SGI0, SGI132161, SGI132161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28775[] = {SGI132162, SGI132163, SGI132164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132165, SGI0, SGI0, SGI0, SGI132166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28776[] = {SGI132167, SGI132168, SGI132169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132170, SGI0, SGI0, SGI0, SGI132171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28777[] = {SGI132172, SGI132173, SGI132174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132175, SGI0, SGI0, SGI0, SGI132176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28778[] = {SGI132177, SGI0, SGI132178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28779[] = {SGI132179, SGI132180, SGI132181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132182, SGI0, SGI0, SGI0, SGI132183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28780[] = {SGI132184, SGI132185, SGI132186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132187, SGI0, SGI0, SGI0, SGI132188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28781[] = {SGI132189, SGI0, SGI132190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28782[] = {SGI132191, SGI0, SGI132192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132193, SGI0, SGI0, SGI0, SGI132194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28783[] = {SGI132195, SGI132196, SGI132197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132198, SGI0, SGI0, SGI0, SGI132199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28784[] = {SGI132200, SGI132201, SGI132202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132203, SGI0, SGI0, SGI0, SGI132204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28785[] = {SGI132205, SGI132206, SGI132207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132208, SGI0, SGI0, SGI0, SGI132209, SGI132210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28786[] = {SGI132149, SGI132150, SGI132151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132213, SGI0, SGI0, SGI0, SGI132214, SGI132214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28787[] = {SGI132211, SGI0, SGI132212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28788[] = {SGI132215, SGI132216, SGI132217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132218, SGI0, SGI0, SGI0, SGI132219, SGI132219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28789[] = {SGI132220, SGI0, SGI132221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132222, SGI0, SGI0, SGI0, SGI132225, SGI132225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28790[] = {SGI132223, SGI0, SGI132224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28791[] = {SGI132226, SGI132227, SGI132228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132229, SGI0, SGI0, SGI0, SGI132230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28792[] = {SGI132232, SGI132234, SGI132236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132237, SGI0, SGI0, SGI0, SGI132239, SGI132239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28793[] = {SGI132231, SGI132233, SGI132235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132238, SGI132238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28794[] = {SGI132240, SGI132241, SGI132242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132246, SGI0, SGI0, SGI0, SGI132247, SGI132247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28795[] = {SGI132243, SGI132244, SGI132245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132248, SGI0, SGI0, SGI0, SGI132249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28796[] = {SGI132250, SGI132251, SGI132252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28797[] = {SGI132255, SGI132256, SGI132257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132258, SGI0, SGI0, SGI0, SGI132259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28798[] = {SGI132260, SGI132261, SGI132262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132263, SGI132263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28799[] = {SGI132264, SGI132265, SGI132266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132267, SGI0, SGI0, SGI0, SGI132268, SGI132268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28800[] = {SGI132269, SGI132270, SGI132271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132272, SGI132272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28801[] = {SGI132273, SGI0, SGI132274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132275, SGI0, SGI0, SGI0, SGI132276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28802[] = {SGI132277, SGI0, SGI132278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132279, SGI132279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28803[] = {SGI132280, SGI132281, SGI132282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28804[] = {SGI132283, SGI132284, SGI132285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132286, SGI0, SGI0, SGI0, SGI132287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28805[] = {SGI132288, SGI132289, SGI132290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132291, SGI0, SGI0, SGI0, SGI132292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28806[] = {SGI132293, SGI132294, SGI132295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28807[] = {SGI132296, SGI132297, SGI132298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28808[] = {SGI0, SGI132299, SGI132300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28809[] = {SGI132301, SGI132302, SGI132303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28810[] = {SGI132304, SGI132305, SGI132306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132307, SGI0, SGI0, SGI0, SGI132308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28811[] = {SGI132309, SGI132310, SGI132311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28812[] = {SGI132312, SGI132313, SGI132314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132315, SGI0, SGI0, SGI0, SGI132316, SGI132316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28813[] = {SGI132317, SGI0, SGI132318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28814[] = {SGI132319, SGI132320, SGI132321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132322, SGI0, SGI0, SGI0, SGI132323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28815[] = {SGI132324, SGI132325, SGI132326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28816[] = {SGI132327, SGI132328, SGI132329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28817[] = {SGI132330, SGI132331, SGI132332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132333, SGI0, SGI0, SGI0, SGI132334, SGI132334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28818[] = {SGI132335, SGI132336, SGI132337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28819[] = {SGI132338, SGI132339, SGI132340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28820[] = {SGI132341, SGI132342, SGI132343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28821[] = {SGI132344, SGI132345, SGI132346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132347, SGI0, SGI0, SGI0, SGI132348, SGI132348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28822[] = {SGI132349, SGI0, SGI132350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132351, SGI0, SGI0, SGI0, SGI132352, SGI132353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28823[] = {SGI132354, SGI132355, SGI132356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132357, SGI0, SGI0, SGI0, SGI132358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28824[] = {SGI132359, SGI132360, SGI132361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28825[] = {SGI132362, SGI0, SGI132363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28826[] = {SGI132364, SGI0, SGI132365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132366, SGI0, SGI0, SGI0, SGI132367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28827[] = {SGI132368, SGI132369, SGI132370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28828[] = {SGI132371, SGI132372, SGI132373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132374, SGI0, SGI0, SGI0, SGI132375, SGI132375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28829[] = {SGI132376, SGI132377, SGI132378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132379, SGI0, SGI0, SGI0, SGI132380, SGI132381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28830[] = {SGI132382, SGI132383, SGI132384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132385, SGI0, SGI0, SGI0, SGI132386, SGI132386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28831[] = {SGI132387, SGI132388, SGI132389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132390, SGI0, SGI0, SGI0, SGI132391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28832[] = {SGI132392, SGI132393, SGI132394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132395, SGI0, SGI0, SGI0, SGI132396, SGI132396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28833[] = {SGI132397, SGI132398, SGI132399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132400, SGI0, SGI0, SGI0, SGI132401, SGI132402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28834[] = {SGI132403, SGI132404, SGI132405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28835[] = {SGI0, SGI0, SGI132406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28836[] = {SGI132407, SGI0, SGI132408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132409, SGI0, SGI0, SGI0, SGI132410, SGI132411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28837[] = {SGI132412, SGI132413, SGI132414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132415, SGI0, SGI0, SGI0, SGI132416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28838[] = {SGI132417, SGI132418, SGI132419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28839[] = {SGI132420, SGI132421, SGI132422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132423, SGI0, SGI0, SGI0, SGI132424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28840[] = {SGI132425, SGI132426, SGI132427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132428, SGI0, SGI0, SGI0, SGI132429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28841[] = {SGI132430, SGI132431, SGI132432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28842[] = {SGI0, SGI132433, SGI132434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28843[] = {SGI132435, SGI132436, SGI132437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132438, SGI0, SGI0, SGI0, SGI132439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28844[] = {SGI132440, SGI132441, SGI132442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132443, SGI0, SGI0, SGI0, SGI132444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28845[] = {SGI0, SGI132253, SGI132254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132445, SGI0, SGI0, SGI0, SGI132446, SGI132446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28846[] = {SGI132447, SGI132448, SGI132449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132450, SGI132451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28847[] = {SGI0, SGI132452, SGI132453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28848[] = {SGI132454, SGI132455, SGI132456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132457, SGI0, SGI0, SGI0, SGI132458, SGI132458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28849[] = {SGI132459, SGI132460, SGI132461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132465, SGI0, SGI0, SGI0, SGI132466, SGI132466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28850[] = {SGI132462, SGI132463, SGI132464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28851[] = {SGI132467, SGI132468, SGI132469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132470, SGI132470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28852[] = {SGI132471, SGI132472, SGI132473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132474, SGI0, SGI0, SGI0, SGI0, SGI132477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28853[] = {SGI132475, SGI132476, SGI132478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28854[] = {SGI132480, SGI132481, SGI132482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132483, SGI0, SGI0, SGI0, SGI132484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28855[] = {SGI0, SGI132485, SGI132486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28856[] = {SGI132487, SGI132488, SGI132489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132490, SGI0, SGI0, SGI0, SGI132491, SGI132491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28857[] = {SGI132492, SGI132493, SGI132494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132496, SGI0, SGI0, SGI0, SGI132499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28858[] = {SGI132495, SGI132497, SGI132498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132500, SGI0, SGI0, SGI0, SGI132501, SGI132501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28859[] = {SGI132502, SGI132503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28860[] = {SGI132504, SGI132505, SGI132506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132507, SGI0, SGI0, SGI0, SGI132508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28861[] = {SGI132509, SGI132510, SGI132511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132512, SGI0, SGI0, SGI0, SGI132513, SGI132513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28862[] = {SGI132514, SGI132515, SGI132516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132517, SGI0, SGI0, SGI0, SGI132518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28863[] = {SGI132519, SGI132520, SGI132521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132525, SGI0, SGI0, SGI0, SGI132526, SGI132528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28864[] = {SGI132522, SGI132523, SGI132524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132527, SGI0, SGI0, SGI0, SGI132529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28865[] = {SGI0, SGI132530, SGI132531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28866[] = {SGI0, SGI132532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28867[] = {SGI132533, SGI132534, SGI132535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132536, SGI0, SGI0, SGI0, SGI132537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28868[] = {SGI132538, SGI0, SGI132539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28869[] = {SGI132540, SGI0, SGI132541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132542, SGI0, SGI0, SGI0, SGI132543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28870[] = {SGI132544, SGI132545, SGI132546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132547, SGI132547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28871[] = {SGI132548, SGI132549, SGI132550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132551, SGI0, SGI0, SGI0, SGI132552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28872[] = {SGI132553, SGI132554, SGI132555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132556, SGI0, SGI0, SGI0, SGI132557, SGI132557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28873[] = {SGI132558, SGI132559, SGI132560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132561, SGI0, SGI0, SGI0, SGI132562, SGI132562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28874[] = {SGI132563, SGI132564, SGI132565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28875[] = {SGI132566, SGI132567, SGI132568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132569, SGI0, SGI0, SGI0, SGI132570, SGI132571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28876[] = {SGI132572, SGI132573, SGI132574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28877[] = {SGI132575, SGI132576, SGI132577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132578, SGI0, SGI0, SGI0, SGI132579, SGI132579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28878[] = {SGI132580, SGI132581, SGI132582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132583, SGI0, SGI0, SGI0, SGI132584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28879[] = {SGI132585, SGI132586, SGI132587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132588, SGI132588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28880[] = {SGI132589, SGI132590, SGI132591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132592, SGI132593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28881[] = {SGI132594, SGI132595, SGI132596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132597, SGI0, SGI0, SGI0, SGI132598, SGI132598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28882[] = {SGI132599, SGI132600, SGI132601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132602, SGI0, SGI0, SGI0, SGI132603, SGI132603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28883[] = {SGI132604, SGI132605, SGI132606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28884[] = {SGI132607, SGI132608, SGI132609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132611, SGI0, SGI0, SGI0, SGI132612, SGI132612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28885[] = {SGI132616, SGI132617, SGI132618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28886[] = {SGI132613, SGI132614, SGI132615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132619, SGI0, SGI0, SGI0, SGI132620, SGI132620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28887[] = {SGI132621, SGI132622, SGI132623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132624, SGI0, SGI0, SGI0, SGI132625, SGI132625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28888[] = {SGI132626, SGI132627, SGI132628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132629, SGI0, SGI0, SGI0, SGI132630, SGI132630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28889[] = {SGI132631, SGI132632, SGI132633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132634, SGI0, SGI0, SGI0, SGI132637, SGI132637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28890[] = {SGI132635, SGI132636, SGI132638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132639, SGI0, SGI0, SGI0, SGI132640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28891[] = {SGI132641, SGI132642, SGI132643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132644, SGI0, SGI0, SGI0, SGI132645, SGI132645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28892[] = {SGI132646, SGI132647, SGI132649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132652, SGI0, SGI0, SGI0, SGI132654, SGI132654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28893[] = {SGI132648, SGI132650, SGI132651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132653, SGI0, SGI0, SGI0, SGI132655, SGI132655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28894[] = {SGI132656, SGI132657, SGI132658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28895[] = {SGI132660, SGI132661, SGI132662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132663, SGI0, SGI0, SGI0, SGI132664, SGI132664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28896[] = {SGI132665, SGI132667, SGI132668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132671, SGI0, SGI0, SGI0, SGI132673, SGI132675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28897[] = {SGI132666, SGI132669, SGI132670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132672, SGI0, SGI0, SGI0, SGI132674, SGI132674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28898[] = {SGI132676, SGI0, SGI132677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28899[] = {SGI132678, SGI132679, SGI132681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132684, SGI0, SGI0, SGI0, SGI132685, SGI132685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28900[] = {SGI132680, SGI132682, SGI132683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28901[] = {SGI132687, SGI132688, SGI132689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132690, SGI0, SGI0, SGI0, SGI132692, SGI132692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28902[] = {SGI132691, SGI132693, SGI132694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132695, SGI0, SGI0, SGI0, SGI132696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28903[] = {SGI132697, SGI132698, SGI132699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132700, SGI0, SGI0, SGI0, SGI132702, SGI132702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28904[] = {SGI0, SGI132701, SGI132703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28905[] = {SGI132704, SGI132705, SGI132706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132707, SGI132707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28906[] = {SGI132708, SGI132709, SGI132710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132711, SGI0, SGI0, SGI0, SGI132712, SGI132712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28907[] = {SGI132713, SGI132714, SGI132715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132716, SGI0, SGI0, SGI0, SGI132717, SGI132717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28908[] = {SGI132718, SGI132719, SGI132720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132721, SGI0, SGI0, SGI0, SGI132722, SGI132722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28909[] = {SGI132723, SGI132724, SGI132725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132726, SGI0, SGI0, SGI0, SGI132727, SGI132727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28910[] = {SGI132728, SGI0, SGI132729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28911[] = {SGI132730, SGI132731, SGI132732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132735, SGI0, SGI0, SGI0, SGI132737, SGI132737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28912[] = {SGI132733, SGI132734, SGI132736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132738, SGI0, SGI0, SGI0, SGI132739, SGI132739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28913[] = {SGI132740, SGI132741, SGI132742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132743, SGI0, SGI0, SGI0, SGI132744, SGI132744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28914[] = {SGI132745, SGI132746, SGI132747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132748, SGI0, SGI0, SGI0, SGI132749, SGI132749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28915[] = {SGI132750, SGI132751, SGI132752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132756, SGI0, SGI0, SGI0, SGI132757, SGI132757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28916[] = {SGI132753, SGI132754, SGI132755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132758, SGI0, SGI0, SGI0, SGI132759, SGI132759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28917[] = {SGI132760, SGI132761, SGI132762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132763, SGI0, SGI0, SGI0, SGI132764, SGI132764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28918[] = {SGI132765, SGI132766, SGI132767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132768, SGI0, SGI0, SGI0, SGI132769, SGI132769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28919[] = {SGI132770, SGI132771, SGI132772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132773, SGI0, SGI0, SGI0, SGI132774, SGI132774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28920[] = {SGI132775, SGI132776, SGI132777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132778, SGI0, SGI0, SGI0, SGI132779, SGI132779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28921[] = {SGI132780, SGI132781, SGI132782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132783, SGI0, SGI0, SGI0, SGI132784, SGI132784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28922[] = {SGI132785, SGI132786, SGI132787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132788, SGI0, SGI0, SGI0, SGI132789, SGI132789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28923[] = {SGI132790, SGI132791, SGI132792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132793, SGI0, SGI0, SGI0, SGI132795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28924[] = {SGI132794, SGI132796, SGI132797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132798, SGI0, SGI0, SGI0, SGI132799, SGI132799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28925[] = {SGI132801, SGI132804, SGI132805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132807, SGI0, SGI0, SGI0, SGI132809, SGI132809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28926[] = {SGI132800, SGI132802, SGI132803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132806, SGI0, SGI0, SGI0, SGI132808, SGI132808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28927[] = {SGI132810, SGI132811, SGI132812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132813, SGI0, SGI0, SGI0, SGI132816, SGI132816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28928[] = {SGI132814, SGI132815, SGI132817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132818, SGI0, SGI0, SGI0, SGI132819, SGI132819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28929[] = {SGI132820, SGI132821, SGI132822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132826, SGI132826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28930[] = {SGI132823, SGI132824, SGI132825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132827, SGI0, SGI0, SGI0, SGI132828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28931[] = {SGI132829, SGI132830, SGI132831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132833, SGI0, SGI0, SGI0, SGI132835, SGI132835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28932[] = {SGI0, SGI132832, SGI132834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132836, SGI132836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28933[] = {SGI132837, SGI132838, SGI132839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132840, SGI0, SGI0, SGI0, SGI132841, SGI132841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28934[] = {SGI132842, SGI132843, SGI132844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132845, SGI0, SGI0, SGI0, SGI132846, SGI132846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28935[] = {SGI132847, SGI132848, SGI132849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132850, SGI0, SGI0, SGI0, SGI132851, SGI132851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28936[] = {SGI132852, SGI132853, SGI132854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132857, SGI0, SGI0, SGI0, SGI132859, SGI132859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28937[] = {SGI132855, SGI132856, SGI132858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132860, SGI0, SGI0, SGI0, SGI132861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28938[] = {SGI132862, SGI132863, SGI132864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132865, SGI0, SGI0, SGI0, SGI0, SGI132866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28939[] = {SGI132867, SGI0, SGI132868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132869, SGI0, SGI0, SGI0, SGI132870, SGI132870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28940[] = {SGI132871, SGI132872, SGI132873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132876, SGI0, SGI0, SGI0, SGI132877, SGI132877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28941[] = {SGI132874, SGI0, SGI132875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132878, SGI0, SGI0, SGI0, SGI132879, SGI132879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28942[] = {SGI132880, SGI132881, SGI132882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132883, SGI0, SGI0, SGI0, SGI132884, SGI132884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28943[] = {SGI132885, SGI132886, SGI132887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132888, SGI0, SGI0, SGI0, SGI132889, SGI132889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28944[] = {SGI132890, SGI132891, SGI132892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132893, SGI0, SGI0, SGI0, SGI132894, SGI132894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28945[] = {SGI132895, SGI132896, SGI132897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132898, SGI0, SGI0, SGI0, SGI132899, SGI132899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28946[] = {SGI132900, SGI132901, SGI132902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132903, SGI0, SGI0, SGI0, SGI132904, SGI132904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28947[] = {SGI132905, SGI132906, SGI132907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132908, SGI0, SGI0, SGI0, SGI132909, SGI132909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28948[] = {SGI132910, SGI0, SGI132911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132912, SGI0, SGI0, SGI0, SGI132913, SGI132913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28949[] = {SGI132914, SGI132915, SGI132916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132917, SGI0, SGI0, SGI0, SGI132918, SGI132918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28950[] = {SGI0, SGI132919, SGI132920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132921, SGI132921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28951[] = {SGI132922, SGI0, SGI132923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28952[] = {SGI132924, SGI132926, SGI132927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132930, SGI0, SGI0, SGI0, SGI132932, SGI132932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28953[] = {SGI132925, SGI132928, SGI132929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132931, SGI0, SGI0, SGI0, SGI132933, SGI132933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28954[] = {SGI132934, SGI132935, SGI132937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132940, SGI0, SGI0, SGI0, SGI132942, SGI132942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28955[] = {SGI132936, SGI132938, SGI132939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132941, SGI0, SGI0, SGI0, SGI132943, SGI132943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28956[] = {SGI132944, SGI132945, SGI132946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132947, SGI0, SGI0, SGI0, SGI132948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28957[] = {SGI132949, SGI132950, SGI132951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132952, SGI0, SGI0, SGI0, SGI132953, SGI132953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28958[] = {SGI132954, SGI132955, SGI132956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132957, SGI0, SGI0, SGI0, SGI132958, SGI132958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28959[] = {SGI132959, SGI132960, SGI132961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132962, SGI0, SGI0, SGI0, SGI132963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28960[] = {SGI132964, SGI132965, SGI132966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132967, SGI0, SGI0, SGI0, SGI132968, SGI132968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28961[] = {SGI132969, SGI132970, SGI132971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132972, SGI0, SGI0, SGI0, SGI132973, SGI132973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28962[] = {SGI132974, SGI132975, SGI132976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132977, SGI0, SGI0, SGI0, SGI132978, SGI132978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28963[] = {SGI132979, SGI132980, SGI132981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132982, SGI0, SGI0, SGI0, SGI132983, SGI132983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28964[] = {SGI132984, SGI132985, SGI132986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132987, SGI0, SGI0, SGI0, SGI132988, SGI132988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28965[] = {SGI132989, SGI132990, SGI132991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132992, SGI0, SGI0, SGI0, SGI132993, SGI132993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28966[] = {SGI132994, SGI132995, SGI132996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI132997, SGI0, SGI0, SGI0, SGI132998, SGI132998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28967[] = {SGI132999, SGI133000, SGI133001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133002, SGI0, SGI0, SGI0, SGI133003, SGI133003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28968[] = {SGI133004, SGI133005, SGI133006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133007, SGI0, SGI0, SGI0, SGI133008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28969[] = {SGI133009, SGI133010, SGI133011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133012, SGI0, SGI0, SGI0, SGI133013, SGI133013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28970[] = {SGI133014, SGI133015, SGI133016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133017, SGI0, SGI0, SGI0, SGI133018, SGI133018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28971[] = {SGI133019, SGI133020, SGI133021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133022, SGI0, SGI0, SGI0, SGI133023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28972[] = {SGI133024, SGI133025, SGI133026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133027, SGI0, SGI0, SGI0, SGI133028, SGI133028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28973[] = {SGI133029, SGI133030, SGI133031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133032, SGI0, SGI0, SGI0, SGI133033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28974[] = {SGI133034, SGI0, SGI133035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133036, SGI0, SGI0, SGI0, SGI133037, SGI133037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28975[] = {SGI133038, SGI133039, SGI133040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133041, SGI0, SGI0, SGI0, SGI133042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28976[] = {SGI133043, SGI133044, SGI133045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133046, SGI0, SGI0, SGI0, SGI133047, SGI133047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28977[] = {SGI133048, SGI133049, SGI133050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133051, SGI0, SGI0, SGI0, SGI133052, SGI133052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28978[] = {SGI133053, SGI133054, SGI133055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133056, SGI0, SGI0, SGI0, SGI133057, SGI133057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28979[] = {SGI133058, SGI133059, SGI133060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133061, SGI0, SGI0, SGI0, SGI133062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28980[] = {SGI133063, SGI133064, SGI133065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133066, SGI0, SGI0, SGI0, SGI133067, SGI133067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28981[] = {SGI133068, SGI133069, SGI133070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133071, SGI0, SGI0, SGI0, SGI133072, SGI133072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28982[] = {SGI133073, SGI133074, SGI133075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133076, SGI0, SGI0, SGI0, SGI133077, SGI133077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28983[] = {SGI133078, SGI133079, SGI133080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133081, SGI0, SGI0, SGI0, SGI133082, SGI133082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28984[] = {SGI133083, SGI133084, SGI133085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133086, SGI0, SGI0, SGI0, SGI133087, SGI133087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28985[] = {SGI133088, SGI133089, SGI133090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133091, SGI0, SGI0, SGI0, SGI133092, SGI133092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28986[] = {SGI133093, SGI133094, SGI133095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133096, SGI0, SGI0, SGI0, SGI133097, SGI133097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28987[] = {SGI133098, SGI133099, SGI133100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133101, SGI0, SGI0, SGI0, SGI133102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28988[] = {SGI133103, SGI133104, SGI133105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133106, SGI0, SGI0, SGI0, SGI133107, SGI133107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28989[] = {SGI133108, SGI133109, SGI133110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133111, SGI0, SGI0, SGI0, SGI133112, SGI133112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28990[] = {SGI133113, SGI133114, SGI133115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133116, SGI0, SGI0, SGI0, SGI133117, SGI133117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28991[] = {SGI133118, SGI133119, SGI133120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133121, SGI0, SGI0, SGI0, SGI133122, SGI133122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28992[] = {SGI133123, SGI0, SGI133124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28993[] = {SGI133125, SGI133126, SGI133127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133128, SGI0, SGI0, SGI0, SGI133129, SGI133129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28994[] = {SGI133130, SGI133131, SGI133132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133133, SGI0, SGI0, SGI0, SGI133134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28995[] = {SGI133135, SGI133136, SGI133137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133138, SGI0, SGI0, SGI0, SGI0, SGI133139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28996[] = {SGI133140, SGI0, SGI133141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133142, SGI0, SGI0, SGI0, SGI133143, SGI133143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28997[] = {SGI133144, SGI133145, SGI133146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133147, SGI0, SGI0, SGI0, SGI133148, SGI133148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28998[] = {SGI133149, SGI133150, SGI133151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133152, SGI0, SGI0, SGI0, SGI133153, SGI133153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG28999[] = {SGI133154, SGI133155, SGI133156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133157, SGI0, SGI0, SGI0, SGI0, SGI133158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29000[] = {SGI133159, SGI133160, SGI133161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133162, SGI0, SGI0, SGI0, SGI133163, SGI133163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29001[] = {SGI133164, SGI133165, SGI133166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133167, SGI0, SGI0, SGI0, SGI133168, SGI133168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29002[] = {SGI133169, SGI133170, SGI133171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133172, SGI0, SGI0, SGI0, SGI133173, SGI133173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29003[] = {SGI133174, SGI133175, SGI133176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133177, SGI0, SGI0, SGI0, SGI133178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29004[] = {SGI133179, SGI133180, SGI133181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133182, SGI0, SGI0, SGI0, SGI133183, SGI133183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29005[] = {SGI133184, SGI133185, SGI133186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133187, SGI0, SGI0, SGI0, SGI133188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29006[] = {SGI133189, SGI133190, SGI133191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133192, SGI0, SGI0, SGI0, SGI133193, SGI133193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29007[] = {SGI133194, SGI133195, SGI133196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133197, SGI0, SGI0, SGI0, SGI133198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29008[] = {SGI133199, SGI0, SGI133200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29009[] = {SGI133201, SGI133202, SGI133203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133206, SGI0, SGI0, SGI0, SGI133207, SGI133207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29010[] = {SGI133204, SGI0, SGI133205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133208, SGI0, SGI0, SGI0, SGI133209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29011[] = {SGI133210, SGI133211, SGI133212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133216, SGI133216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29012[] = {SGI133213, SGI133214, SGI133215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133217, SGI0, SGI0, SGI0, SGI133218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29013[] = {SGI133219, SGI0, SGI133220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29014[] = {SGI133221, SGI133222, SGI133223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133224, SGI0, SGI0, SGI0, SGI133226, SGI133226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29015[] = {SGI133225, SGI133227, SGI133228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133229, SGI0, SGI0, SGI0, SGI133230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29016[] = {SGI133231, SGI133232, SGI133233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133234, SGI0, SGI0, SGI0, SGI133235, SGI133235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29017[] = {SGI133236, SGI133238, SGI133240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29018[] = {SGI133237, SGI133239, SGI133241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133242, SGI0, SGI0, SGI0, SGI133243, SGI133243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29019[] = {SGI133244, SGI133245, SGI133246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133247, SGI0, SGI0, SGI0, SGI133248, SGI133248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29020[] = {SGI133249, SGI133250, SGI133251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133252, SGI133252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29021[] = {SGI133254, SGI133256, SGI133258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133259, SGI0, SGI0, SGI0, SGI133261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29022[] = {SGI133253, SGI133255, SGI133257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133260, SGI133260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29023[] = {SGI133263, SGI133265, SGI133266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133268, SGI0, SGI0, SGI0, SGI133270, SGI133270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29024[] = {SGI133262, SGI133264, SGI133267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133269, SGI0, SGI0, SGI0, SGI133271, SGI133272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29025[] = {SGI133273, SGI133274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29026[] = {SGI133276, SGI133278, SGI133280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29027[] = {SGI133275, SGI133277, SGI133279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133281, SGI0, SGI0, SGI0, SGI133282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29028[] = {SGI133283, SGI0, SGI133285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133288, SGI0, SGI0, SGI0, SGI133290, SGI133290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29029[] = {SGI133284, SGI133286, SGI133287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133289, SGI0, SGI0, SGI0, SGI133291, SGI133291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29030[] = {SGI133295, SGI133296, SGI133297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133300, SGI0, SGI0, SGI0, SGI133301, SGI133301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29031[] = {SGI133292, SGI133293, SGI133294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133298, SGI0, SGI0, SGI0, SGI133299, SGI133299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29032[] = {SGI133302, SGI133303, SGI133304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29033[] = {SGI133305, SGI133306, SGI133307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133308, SGI0, SGI0, SGI0, SGI133309, SGI133309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29034[] = {SGI133310, SGI0, SGI133311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133312, SGI0, SGI0, SGI0, SGI133313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29035[] = {SGI133314, SGI133315, SGI133316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133317, SGI0, SGI0, SGI0, SGI133318, SGI133318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29036[] = {SGI133319, SGI133320, SGI133321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29037[] = {SGI133323, SGI133325, SGI133327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29038[] = {SGI133322, SGI133324, SGI133326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133328, SGI0, SGI0, SGI0, SGI133329, SGI133329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29039[] = {SGI133330, SGI133331, SGI133332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133333, SGI0, SGI0, SGI0, SGI133335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29040[] = {SGI133334, SGI133336, SGI133337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133338, SGI0, SGI0, SGI0, SGI133339, SGI133339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29041[] = {SGI133340, SGI133341, SGI133342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133343, SGI0, SGI0, SGI0, SGI133344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29042[] = {SGI133345, SGI133346, SGI133347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133348, SGI0, SGI0, SGI0, SGI133349, SGI133349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29043[] = {SGI133350, SGI133351, SGI133352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133353, SGI0, SGI0, SGI0, SGI133354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29044[] = {SGI133355, SGI133356, SGI133357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133358, SGI0, SGI0, SGI0, SGI133359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29045[] = {SGI133360, SGI133361, SGI133362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29046[] = {SGI133363, SGI133364, SGI133365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133366, SGI0, SGI0, SGI0, SGI133367, SGI133367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29047[] = {SGI133368, SGI133369, SGI133370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29048[] = {SGI133371, SGI0, SGI133372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29049[] = {SGI0, SGI133373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29050[] = {SGI133374, SGI133375, SGI133376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133377, SGI0, SGI0, SGI0, SGI133378, SGI133378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29051[] = {SGI133379, SGI133380, SGI133381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133382, SGI0, SGI0, SGI0, SGI0, SGI133383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29052[] = {SGI0, SGI133384, SGI133385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29053[] = {SGI133386, SGI133387, SGI133388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133389, SGI0, SGI0, SGI0, SGI133390, SGI133391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29054[] = {SGI133392, SGI133393, SGI133394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133395, SGI0, SGI0, SGI0, SGI133396, SGI133396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29055[] = {SGI133397, SGI133398, SGI133399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133400, SGI0, SGI0, SGI0, SGI133401, SGI133401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29056[] = {SGI133402, SGI133403, SGI133404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133405, SGI0, SGI0, SGI0, SGI133406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29057[] = {SGI133407, SGI133408, SGI133409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133410, SGI0, SGI0, SGI0, SGI133411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29058[] = {SGI133412, SGI133413, SGI133414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29059[] = {SGI133415, SGI133416, SGI133417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29060[] = {SGI133418, SGI133419, SGI133420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133421, SGI0, SGI0, SGI0, SGI133422, SGI133422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29061[] = {SGI133423, SGI133424, SGI133425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29062[] = {SGI133426, SGI133427, SGI133428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29063[] = {SGI133429, SGI133430, SGI133431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133432, SGI0, SGI0, SGI0, SGI133433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29064[] = {SGI133434, SGI133435, SGI133436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133438, SGI0, SGI0, SGI0, SGI133439, SGI133439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29065[] = {SGI0, SGI0, SGI133437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29066[] = {SGI133440, SGI133441, SGI133442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133443, SGI0, SGI0, SGI0, SGI133444, SGI133444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29067[] = {SGI133445, SGI133446, SGI133447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133448, SGI0, SGI0, SGI0, SGI133449, SGI133449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29068[] = {SGI133451, SGI133452, SGI133454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133456, SGI0, SGI0, SGI0, SGI133458, SGI133458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29069[] = {SGI133450, SGI0, SGI133453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133455, SGI0, SGI0, SGI0, SGI133457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29070[] = {SGI133461, SGI133463, SGI133464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29071[] = {SGI133459, SGI133460, SGI133462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133465, SGI0, SGI0, SGI0, SGI133467, SGI133467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29072[] = {SGI133468, SGI133469, SGI133470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133471, SGI0, SGI0, SGI0, SGI133472, SGI133472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29073[] = {SGI133473, SGI133474, SGI133475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133476, SGI0, SGI0, SGI0, SGI133477, SGI133477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29074[] = {SGI133478, SGI133479, SGI133480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133481, SGI0, SGI0, SGI0, SGI133482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29075[] = {SGI133483, SGI133484, SGI133485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133487, SGI0, SGI0, SGI0, SGI133489, SGI133489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29076[] = {SGI0, SGI133486, SGI133488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29077[] = {SGI133490, SGI133491, SGI133492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133494, SGI0, SGI0, SGI0, SGI133496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29078[] = {SGI133493, SGI133495, SGI133497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133498, SGI0, SGI0, SGI0, SGI133499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29079[] = {SGI133500, SGI133501, SGI133502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133503, SGI133503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29080[] = {SGI133504, SGI133505, SGI133506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133507, SGI0, SGI0, SGI0, SGI133508, SGI133508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29081[] = {SGI133509, SGI133510, SGI133511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133512, SGI0, SGI0, SGI0, SGI133513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29082[] = {SGI133514, SGI133515, SGI133516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133518, SGI0, SGI0, SGI0, SGI133521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29083[] = {SGI133517, SGI133519, SGI133520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133522, SGI0, SGI0, SGI0, SGI133523, SGI133523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29084[] = {SGI133524, SGI133525, SGI133526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133527, SGI0, SGI0, SGI0, SGI133528, SGI133529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29085[] = {SGI133530, SGI133531, SGI133532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133533, SGI0, SGI0, SGI0, SGI133534, SGI133534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29086[] = {SGI133535, SGI133536, SGI133537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133538, SGI0, SGI0, SGI0, SGI133539, SGI133539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29087[] = {SGI133540, SGI133541, SGI133542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133543, SGI0, SGI0, SGI0, SGI133544, SGI133544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29088[] = {SGI133545, SGI133546, SGI133547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133548, SGI0, SGI0, SGI0, SGI133549, SGI133549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29089[] = {SGI133550, SGI133551, SGI133552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133553, SGI0, SGI0, SGI0, SGI133554, SGI133554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29090[] = {SGI133555, SGI133556, SGI133557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133558, SGI0, SGI0, SGI0, SGI133559, SGI133559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29091[] = {SGI133560, SGI133561, SGI133562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133563, SGI0, SGI0, SGI0, SGI133564, SGI133564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29092[] = {SGI133565, SGI133566, SGI133567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133568, SGI0, SGI0, SGI0, SGI133569, SGI133569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29093[] = {SGI133570, SGI133571, SGI133572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133573, SGI0, SGI0, SGI0, SGI133574, SGI133574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29094[] = {SGI133575, SGI133576, SGI133577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133578, SGI0, SGI0, SGI0, SGI133579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29095[] = {SGI133580, SGI133581, SGI133582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133583, SGI0, SGI0, SGI0, SGI133584, SGI133584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29096[] = {SGI133585, SGI133586, SGI133587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133588, SGI0, SGI0, SGI0, SGI133589, SGI133589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29097[] = {SGI133590, SGI133591, SGI133592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133593, SGI0, SGI0, SGI0, SGI133594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29098[] = {SGI133595, SGI0, SGI133596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29099[] = {SGI133597, SGI133598, SGI133599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133600, SGI0, SGI0, SGI0, SGI133601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29100[] = {SGI133602, SGI133603, SGI133604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133605, SGI0, SGI0, SGI0, SGI133606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29101[] = {SGI133607, SGI133608, SGI133609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133610, SGI0, SGI0, SGI0, SGI133611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29102[] = {SGI133612, SGI133613, SGI133614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133615, SGI0, SGI0, SGI0, SGI133616, SGI133616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29103[] = {SGI133617, SGI133618, SGI133619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133620, SGI0, SGI0, SGI0, SGI133621, SGI133621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29104[] = {SGI133622, SGI133623, SGI133624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133625, SGI0, SGI0, SGI0, SGI133626, SGI133626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29105[] = {SGI133627, SGI133628, SGI133629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133630, SGI0, SGI0, SGI0, SGI133631, SGI133631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29106[] = {SGI133632, SGI133633, SGI133634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133635, SGI0, SGI0, SGI0, SGI133636, SGI133636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29107[] = {SGI133637, SGI133638, SGI133639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133640, SGI0, SGI0, SGI0, SGI133641, SGI133641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29108[] = {SGI133642, SGI133643, SGI133644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133645, SGI0, SGI0, SGI0, SGI133646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29109[] = {SGI133647, SGI133648, SGI133649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133650, SGI0, SGI0, SGI0, SGI133651, SGI133651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29110[] = {SGI133652, SGI133653, SGI133654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133655, SGI0, SGI0, SGI0, SGI133656, SGI133656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29111[] = {SGI133657, SGI133658, SGI133659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133660, SGI0, SGI0, SGI0, SGI133661, SGI133661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29112[] = {SGI133662, SGI133663, SGI133664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133665, SGI0, SGI0, SGI0, SGI133666, SGI133666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29113[] = {SGI133667, SGI0, SGI133668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29114[] = {SGI133669, SGI0, SGI133670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133671, SGI0, SGI0, SGI0, SGI133672, SGI133672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29115[] = {SGI133673, SGI133674, SGI133675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133676, SGI133676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29116[] = {SGI133677, SGI133678, SGI133679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133680, SGI0, SGI0, SGI0, SGI133681, SGI133681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29117[] = {SGI133682, SGI133683, SGI133684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133685, SGI133685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29118[] = {SGI133686, SGI0, SGI133687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29119[] = {SGI133688, SGI0, SGI133689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29120[] = {SGI133690, SGI133691, SGI133692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133693, SGI0, SGI0, SGI0, SGI133694, SGI133694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29121[] = {SGI133695, SGI133696, SGI133697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133698, SGI133699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29122[] = {SGI133700, SGI133701, SGI133702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133703, SGI0, SGI0, SGI0, SGI133704, SGI133704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29123[] = {SGI133705, SGI133706, SGI133707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133708, SGI0, SGI0, SGI0, SGI133709, SGI133709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29124[] = {SGI133710, SGI133711, SGI133712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133713, SGI0, SGI0, SGI0, SGI133714, SGI133714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29125[] = {SGI133715, SGI133716, SGI133717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133718, SGI0, SGI0, SGI0, SGI133719, SGI133719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29126[] = {SGI133720, SGI133721, SGI133722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133723, SGI0, SGI0, SGI0, SGI133724, SGI133724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29127[] = {SGI133725, SGI133726, SGI133727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133728, SGI0, SGI0, SGI0, SGI133729, SGI133729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29128[] = {SGI133730, SGI133731, SGI133732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133733, SGI0, SGI0, SGI0, SGI133734, SGI133734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29129[] = {SGI133735, SGI133736, SGI133737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133738, SGI0, SGI0, SGI0, SGI133739, SGI133739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29130[] = {SGI133740, SGI133741, SGI133742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133743, SGI0, SGI0, SGI0, SGI133744, SGI133744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29131[] = {SGI133745, SGI133746, SGI133747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133748, SGI0, SGI0, SGI0, SGI133749, SGI133749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29132[] = {SGI133750, SGI133751, SGI133752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133753, SGI0, SGI0, SGI0, SGI133754, SGI133754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29133[] = {SGI133755, SGI133756, SGI133757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133758, SGI0, SGI0, SGI0, SGI133759, SGI133759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29134[] = {SGI133760, SGI133761, SGI133762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133763, SGI0, SGI0, SGI0, SGI133764, SGI133764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29135[] = {SGI133765, SGI133766, SGI133767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133768, SGI0, SGI0, SGI0, SGI133769, SGI133769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29136[] = {SGI133770, SGI133771, SGI133772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133773, SGI0, SGI0, SGI0, SGI133774, SGI133774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29137[] = {SGI133775, SGI133776, SGI133777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133778, SGI0, SGI0, SGI0, SGI133779, SGI133779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29138[] = {SGI133780, SGI133781, SGI133782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133783, SGI0, SGI0, SGI0, SGI133784, SGI133784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29139[] = {SGI133785, SGI133786, SGI133787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133788, SGI133788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29140[] = {SGI133789, SGI133790, SGI133791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133792, SGI0, SGI0, SGI0, SGI133793, SGI133793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29141[] = {SGI133794, SGI133795, SGI133796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133797, SGI0, SGI0, SGI0, SGI133798, SGI133798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29142[] = {SGI133799, SGI133800, SGI133801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133802, SGI0, SGI0, SGI0, SGI133803, SGI133803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29143[] = {SGI133804, SGI133805, SGI133806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133807, SGI0, SGI0, SGI0, SGI133808, SGI133808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29144[] = {SGI133809, SGI133810, SGI133811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133812, SGI0, SGI0, SGI0, SGI133813, SGI133813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29145[] = {SGI133814, SGI133815, SGI133816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133817, SGI0, SGI0, SGI0, SGI133818, SGI133818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29146[] = {SGI133819, SGI133820, SGI133821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133822, SGI0, SGI0, SGI0, SGI133823, SGI133823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29147[] = {SGI133824, SGI133825, SGI133826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133827, SGI0, SGI0, SGI0, SGI133828, SGI133828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29148[] = {SGI133829, SGI133830, SGI133831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133832, SGI0, SGI0, SGI0, SGI133833, SGI133833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29149[] = {SGI133834, SGI133835, SGI133836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133837, SGI0, SGI0, SGI0, SGI133838, SGI133838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29150[] = {SGI133839, SGI133840, SGI133841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133842, SGI0, SGI0, SGI0, SGI133843, SGI133843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29151[] = {SGI133844, SGI133845, SGI133846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133847, SGI0, SGI0, SGI0, SGI133848, SGI133848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29152[] = {SGI133849, SGI0, SGI133850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29153[] = {SGI133851, SGI133852, SGI133853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133854, SGI0, SGI0, SGI0, SGI133855, SGI133855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29154[] = {SGI133856, SGI133857, SGI133858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133859, SGI0, SGI0, SGI0, SGI133860, SGI133860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29155[] = {SGI133861, SGI133862, SGI133863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133864, SGI0, SGI0, SGI0, SGI133865, SGI133865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29156[] = {SGI133866, SGI133867, SGI133868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133869, SGI0, SGI0, SGI0, SGI133870, SGI133870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29157[] = {SGI133871, SGI133872, SGI133873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133874, SGI0, SGI0, SGI0, SGI133875, SGI133875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29158[] = {SGI133876, SGI133877, SGI133878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133879, SGI0, SGI0, SGI0, SGI133880, SGI133880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29159[] = {SGI133881, SGI133882, SGI133883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133884, SGI0, SGI0, SGI0, SGI133885, SGI133885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29160[] = {SGI133886, SGI133887, SGI133888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133889, SGI0, SGI0, SGI0, SGI133890, SGI133890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29161[] = {SGI133891, SGI133892, SGI133893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133894, SGI0, SGI0, SGI0, SGI133895, SGI133895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29162[] = {SGI133896, SGI133897, SGI133898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133899, SGI0, SGI0, SGI0, SGI133900, SGI133900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29163[] = {SGI133901, SGI133902, SGI133903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133904, SGI0, SGI0, SGI0, SGI133905, SGI133905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29164[] = {SGI133906, SGI133907, SGI133908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133909, SGI0, SGI0, SGI0, SGI133910, SGI133910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29165[] = {SGI133911, SGI133912, SGI133913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133914, SGI133914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29166[] = {SGI133915, SGI133916, SGI133917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133918, SGI0, SGI0, SGI0, SGI133919, SGI133919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29167[] = {SGI133920, SGI133921, SGI133922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133923, SGI0, SGI0, SGI0, SGI133924, SGI133924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29168[] = {SGI133925, SGI133926, SGI133927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133928, SGI0, SGI0, SGI0, SGI133929, SGI133929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29169[] = {SGI133930, SGI133931, SGI133932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133933, SGI0, SGI0, SGI0, SGI133934, SGI133934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29170[] = {SGI133935, SGI133936, SGI133937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133938, SGI0, SGI0, SGI0, SGI133939, SGI133939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29171[] = {SGI133940, SGI133941, SGI133942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133943, SGI133943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29172[] = {SGI133944, SGI133945, SGI133946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133947, SGI0, SGI0, SGI0, SGI133948, SGI133948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29173[] = {SGI133949, SGI133950, SGI133951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133952, SGI133952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29174[] = {SGI133953, SGI133954, SGI133955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133956, SGI0, SGI0, SGI0, SGI133957, SGI133957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29175[] = {SGI133958, SGI133959, SGI133960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133961, SGI0, SGI0, SGI0, SGI133962, SGI133962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29176[] = {SGI133963, SGI133964, SGI133965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133966, SGI0, SGI0, SGI0, SGI133967, SGI133967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29177[] = {SGI133968, SGI133969, SGI133970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133971, SGI0, SGI0, SGI0, SGI133972, SGI133972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29178[] = {SGI133973, SGI133974, SGI133975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133976, SGI133976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29179[] = {SGI133977, SGI133978, SGI133979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133980, SGI0, SGI0, SGI0, SGI133981, SGI133981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29180[] = {SGI133982, SGI133983, SGI133984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133985, SGI0, SGI0, SGI0, SGI133986, SGI133986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29181[] = {SGI133987, SGI133988, SGI133989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133990, SGI0, SGI0, SGI0, SGI133991, SGI133991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29182[] = {SGI133992, SGI133993, SGI133994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI133995, SGI0, SGI0, SGI0, SGI133996, SGI133996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29183[] = {SGI133997, SGI133998, SGI133999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134000, SGI0, SGI0, SGI0, SGI134001, SGI134001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29184[] = {SGI134002, SGI134003, SGI134004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134005, SGI0, SGI0, SGI0, SGI134006, SGI134006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29185[] = {SGI134007, SGI134008, SGI134009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134010, SGI0, SGI0, SGI0, SGI134011, SGI134011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29186[] = {SGI134012, SGI134013, SGI134014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134015, SGI0, SGI0, SGI0, SGI134016, SGI134016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29187[] = {SGI134017, SGI134018, SGI134019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134020, SGI0, SGI0, SGI0, SGI134021, SGI134021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29188[] = {SGI134022, SGI134023, SGI134024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134025, SGI0, SGI0, SGI0, SGI134026, SGI134026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29189[] = {SGI134027, SGI134028, SGI134029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134030, SGI134030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29190[] = {SGI134031, SGI134032, SGI134033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134034, SGI0, SGI0, SGI0, SGI134035, SGI134035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29191[] = {SGI134036, SGI0, SGI134037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134038, SGI0, SGI0, SGI0, SGI134039, SGI134039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29192[] = {SGI134040, SGI134041, SGI134042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134043, SGI0, SGI0, SGI0, SGI134044, SGI134044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29193[] = {SGI134045, SGI134046, SGI134047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134048, SGI0, SGI0, SGI0, SGI134049, SGI134049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29194[] = {SGI134050, SGI134051, SGI134052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134053, SGI0, SGI0, SGI0, SGI134054, SGI134054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29195[] = {SGI134055, SGI134056, SGI134057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134058, SGI0, SGI0, SGI0, SGI134059, SGI134059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29196[] = {SGI134060, SGI134061, SGI134062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134063, SGI0, SGI0, SGI0, SGI134064, SGI134064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29197[] = {SGI134065, SGI134066, SGI134067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134068, SGI0, SGI0, SGI0, SGI134069, SGI134069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29198[] = {SGI134070, SGI134071, SGI134072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134076, SGI0, SGI0, SGI0, SGI134077, SGI134077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29199[] = {SGI134073, SGI134074, SGI134075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134078, SGI0, SGI0, SGI0, SGI134079, SGI134079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29200[] = {SGI134080, SGI134081, SGI134082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134083, SGI0, SGI0, SGI0, SGI134084, SGI134084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29201[] = {SGI134085, SGI0, SGI134086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29202[] = {SGI134087, SGI0, SGI134088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134089, SGI0, SGI0, SGI0, SGI134090, SGI134090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29203[] = {SGI134091, SGI134092, SGI134093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134094, SGI0, SGI0, SGI0, SGI134095, SGI134095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29204[] = {SGI134096, SGI0, SGI134097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29205[] = {SGI134098, SGI134099, SGI134100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134101, SGI0, SGI0, SGI0, SGI134102, SGI134102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29206[] = {SGI134105, SGI0, SGI134107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29207[] = {SGI134103, SGI134104, SGI134106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134108, SGI0, SGI0, SGI0, SGI134109, SGI134109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29208[] = {SGI134110, SGI134111, SGI134112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134113, SGI134113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29209[] = {SGI134114, SGI134115, SGI134116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134117, SGI0, SGI0, SGI0, SGI134118, SGI134118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29210[] = {SGI134119, SGI134120, SGI134121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134122, SGI134122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29211[] = {SGI134123, SGI0, SGI134124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29212[] = {SGI134125, SGI134126, SGI134127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134128, SGI0, SGI0, SGI0, SGI134129, SGI134129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29213[] = {SGI134130, SGI134131, SGI134132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134133, SGI134133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29214[] = {SGI134134, SGI0, SGI134135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29215[] = {SGI134136, SGI134137, SGI134138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134139, SGI0, SGI0, SGI0, SGI134140, SGI134140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29216[] = {SGI134141, SGI134142, SGI134143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134144, SGI0, SGI0, SGI0, SGI134145, SGI134145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29217[] = {SGI134146, SGI134147, SGI134148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134149, SGI0, SGI0, SGI0, SGI134150, SGI134150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29218[] = {SGI134151, SGI134152, SGI134153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134154, SGI0, SGI0, SGI0, SGI134155, SGI134155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29219[] = {SGI134156, SGI134157, SGI134158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134159, SGI0, SGI0, SGI0, SGI134160, SGI134160, SGI0, SGI0, NULL};
const Segment * const SG29220[] = {SGI134161, SGI134162, SGI134163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134164, SGI0, SGI0, SGI0, SGI134165, SGI134165, SGI0, SGI0, NULL};
const Segment * const SG29221[] = {SGI134166, SGI134167, SGI134168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134169, SGI0, SGI0, SGI0, SGI134170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29222[] = {SGI134171, SGI0, SGI134172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29223[] = {SGI134173, SGI134174, SGI134175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134176, SGI0, SGI0, SGI0, SGI134177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29224[] = {SGI134178, SGI134179, SGI134180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134181, SGI0, SGI0, SGI0, SGI134182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29225[] = {SGI134183, SGI134184, SGI134185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134186, SGI0, SGI0, SGI0, SGI134187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29226[] = {SGI134188, SGI134189, SGI134190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134191, SGI0, SGI0, SGI0, SGI134192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29227[] = {SGI134193, SGI134194, SGI134195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134196, SGI0, SGI0, SGI0, SGI134197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29228[] = {SGI134198, SGI0, SGI134199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29229[] = {SGI134200, SGI134201, SGI134202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134203, SGI0, SGI0, SGI0, SGI134204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29230[] = {SGI134205, SGI134206, SGI134207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134208, SGI0, SGI0, SGI0, SGI134209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29231[] = {SGI134210, SGI134211, SGI134212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134213, SGI0, SGI0, SGI0, SGI134214, SGI134214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29232[] = {SGI134215, SGI134216, SGI134217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134218, SGI0, SGI0, SGI0, SGI134219, SGI134219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29233[] = {SGI134220, SGI134221, SGI134222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134223, SGI0, SGI0, SGI0, SGI134224, SGI134224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29234[] = {SGI134225, SGI134226, SGI134227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134228, SGI0, SGI0, SGI0, SGI134229, SGI134229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29235[] = {SGI134230, SGI134231, SGI134232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134233, SGI0, SGI0, SGI0, SGI134234, SGI134234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29236[] = {SGI134235, SGI0, SGI134236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29237[] = {SGI134237, SGI0, SGI134238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29238[] = {SGI134239, SGI134240, SGI134241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134242, SGI0, SGI0, SGI0, SGI134243, SGI134243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29239[] = {SGI134244, SGI0, SGI134245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29240[] = {SGI134246, SGI134247, SGI134248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134249, SGI0, SGI0, SGI0, SGI134250, SGI134250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29241[] = {SGI134251, SGI134252, SGI134253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134254, SGI0, SGI0, SGI0, SGI134255, SGI134255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29242[] = {SGI134256, SGI134257, SGI134258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134259, SGI0, SGI0, SGI0, SGI134260, SGI134260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29243[] = {SGI134261, SGI134262, SGI134263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134264, SGI0, SGI0, SGI0, SGI134265, SGI134265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29244[] = {SGI134266, SGI0, SGI134267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134268, SGI0, SGI0, SGI0, SGI134269, SGI134269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29245[] = {SGI134270, SGI134271, SGI134272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134273, SGI0, SGI0, SGI0, SGI134274, SGI134274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29246[] = {SGI134275, SGI134276, SGI134277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134278, SGI0, SGI0, SGI0, SGI134279, SGI134279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29247[] = {SGI134280, SGI134281, SGI134282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134283, SGI0, SGI0, SGI0, SGI134284, SGI134284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29248[] = {SGI134285, SGI134286, SGI134287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134288, SGI0, SGI0, SGI0, SGI134289, SGI134289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29249[] = {SGI134290, SGI0, SGI134291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134292, SGI0, SGI0, SGI0, SGI134293, SGI134293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29250[] = {SGI134294, SGI134295, SGI134296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134297, SGI0, SGI0, SGI0, SGI134298, SGI134298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29251[] = {SGI134299, SGI134300, SGI134301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134302, SGI0, SGI0, SGI0, SGI134303, SGI134303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29252[] = {SGI134304, SGI134305, SGI134306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134307, SGI0, SGI0, SGI0, SGI134308, SGI134308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29253[] = {SGI134309, SGI134310, SGI134311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134312, SGI134312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29254[] = {SGI134313, SGI0, SGI134314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29255[] = {SGI134315, SGI134316, SGI134317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134318, SGI0, SGI0, SGI0, SGI134319, SGI134319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29256[] = {SGI134320, SGI134321, SGI134322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134323, SGI0, SGI0, SGI0, SGI134324, SGI134324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29257[] = {SGI134325, SGI134326, SGI134327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134328, SGI134329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29258[] = {SGI134330, SGI134331, SGI134332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134333, SGI0, SGI0, SGI0, SGI134334, SGI134334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29259[] = {SGI134335, SGI0, SGI134336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134337, SGI0, SGI0, SGI0, SGI134338, SGI134338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29260[] = {SGI0, SGI134339, SGI134340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29261[] = {SGI134341, SGI134342, SGI134343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134344, SGI0, SGI0, SGI0, SGI134345, SGI134345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29262[] = {SGI134346, SGI134347, SGI134348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134349, SGI0, SGI0, SGI0, SGI134350, SGI134350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29263[] = {SGI134351, SGI0, SGI134352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134353, SGI0, SGI0, SGI0, SGI134354, SGI134354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29264[] = {SGI134355, SGI134356, SGI134357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134358, SGI0, SGI0, SGI0, SGI134359, SGI134359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29265[] = {SGI134360, SGI134361, SGI134362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134363, SGI0, SGI0, SGI0, SGI134364, SGI134364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29266[] = {SGI134365, SGI134366, SGI134367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134368, SGI0, SGI0, SGI0, SGI134369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29267[] = {SGI134370, SGI0, SGI134371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29268[] = {SGI134372, SGI0, SGI134373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134374, SGI0, SGI0, SGI0, SGI134375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29269[] = {SGI134376, SGI134377, SGI134378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134379, SGI0, SGI0, SGI0, SGI134380, SGI134380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29270[] = {SGI134381, SGI134382, SGI134383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29271[] = {SGI134384, SGI134385, SGI134386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29272[] = {SGI134387, SGI134388, SGI134389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29273[] = {SGI134390, SGI134391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29274[] = {SGI134392, SGI134393, SGI134394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29275[] = {SGI0, SGI134395, SGI134396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29276[] = {SGI134397, SGI134398, SGI134399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134400, SGI0, SGI0, SGI0, SGI134401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29277[] = {SGI134402, SGI134403, SGI134404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134405, SGI134405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29278[] = {SGI134406, SGI134407, SGI134408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134409, SGI0, SGI0, SGI0, SGI134410, SGI134411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29279[] = {SGI134412, SGI134413, SGI134414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134415, SGI0, SGI0, SGI0, SGI134416, SGI134417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29280[] = {SGI134418, SGI134419, SGI134420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134421, SGI0, SGI0, SGI0, SGI134422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29281[] = {SGI134423, SGI0, SGI134424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29282[] = {SGI134425, SGI134426, SGI134427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134428, SGI0, SGI0, SGI0, SGI134429, SGI134429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29283[] = {SGI134430, SGI134431, SGI134432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134433, SGI0, SGI0, SGI0, SGI134434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29284[] = {SGI134435, SGI134436, SGI134437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29285[] = {SGI134438, SGI134439, SGI134440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134441, SGI0, SGI0, SGI0, SGI134442, SGI134442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29286[] = {SGI134443, SGI134444, SGI134445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134446, SGI0, SGI0, SGI0, SGI134447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29287[] = {SGI134448, SGI134449, SGI134450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29288[] = {SGI0, SGI134451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29289[] = {SGI0, SGI0, SGI134452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29290[] = {SGI134453, SGI134454, SGI134455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134456, SGI134456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29291[] = {SGI134457, SGI134458, SGI134459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134460, SGI0, SGI0, SGI0, SGI134461, SGI134461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29292[] = {SGI134462, SGI134463, SGI134464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134465, SGI0, SGI0, SGI0, SGI134466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29293[] = {SGI134467, SGI0, SGI134468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134469, SGI0, SGI0, SGI0, SGI134470, SGI134470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29294[] = {SGI134471, SGI134472, SGI134473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134474, SGI0, SGI0, SGI0, SGI134475, SGI134475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29295[] = {SGI134476, SGI134477, SGI134478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134479, SGI0, SGI0, SGI0, SGI134480, SGI134480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29296[] = {SGI134481, SGI0, SGI134482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29297[] = {SGI134483, SGI134484, SGI134485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134486, SGI0, SGI0, SGI0, SGI134487, SGI134487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29298[] = {SGI134488, SGI134489, SGI134490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134491, SGI0, SGI0, SGI0, SGI134492, SGI134492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29299[] = {SGI134493, SGI134494, SGI134495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134496, SGI0, SGI0, SGI0, SGI134497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29300[] = {SGI134498, SGI0, SGI134499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29301[] = {SGI134500, SGI134501, SGI134502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134503, SGI0, SGI0, SGI0, SGI134504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29302[] = {SGI134505, SGI134506, SGI134507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134508, SGI0, SGI0, SGI0, SGI134509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29303[] = {SGI134510, SGI134511, SGI134512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134513, SGI0, SGI0, SGI0, SGI134514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29304[] = {SGI134515, SGI134516, SGI134517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134518, SGI0, SGI0, SGI0, SGI134519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29305[] = {SGI134520, SGI0, SGI134521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29306[] = {SGI134522, SGI134523, SGI134524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134525, SGI0, SGI0, SGI0, SGI134526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29307[] = {SGI134527, SGI134528, SGI134529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134530, SGI0, SGI0, SGI0, SGI134531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29308[] = {SGI134532, SGI134533, SGI134534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134535, SGI0, SGI0, SGI0, SGI134536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29309[] = {SGI134537, SGI0, SGI134538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29310[] = {SGI134539, SGI134540, SGI134541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134542, SGI0, SGI0, SGI0, SGI134543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29311[] = {SGI134544, SGI134545, SGI134546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134547, SGI0, SGI0, SGI0, SGI134548, SGI134548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29312[] = {SGI134549, SGI0, SGI134550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134551, SGI0, SGI0, SGI0, SGI134552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29313[] = {SGI134553, SGI134554, SGI134555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134556, SGI0, SGI0, SGI0, SGI134557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29314[] = {SGI134558, SGI134559, SGI134560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134561, SGI0, SGI0, SGI0, SGI134562, SGI134562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29315[] = {SGI134563, SGI134564, SGI134565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134566, SGI0, SGI0, SGI0, SGI134567, SGI134567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29316[] = {SGI134568, SGI134569, SGI134570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134571, SGI0, SGI0, SGI0, SGI134572, SGI134572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29317[] = {SGI134573, SGI134574, SGI134575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134576, SGI0, SGI0, SGI0, SGI134577, SGI134577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29318[] = {SGI134578, SGI134579, SGI134580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134581, SGI0, SGI0, SGI0, SGI134582, SGI134582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29319[] = {SGI134583, SGI134584, SGI134585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134586, SGI0, SGI0, SGI0, SGI134587, SGI134587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29320[] = {SGI134588, SGI134589, SGI134590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134591, SGI0, SGI0, SGI0, SGI134592, SGI134592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29321[] = {SGI134593, SGI134594, SGI134595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134596, SGI0, SGI0, SGI0, SGI134597, SGI134597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29322[] = {SGI134598, SGI134599, SGI134600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134601, SGI0, SGI0, SGI0, SGI134602, SGI134602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29323[] = {SGI134603, SGI134604, SGI134605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134606, SGI0, SGI0, SGI0, SGI134607, SGI134607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29324[] = {SGI134608, SGI134609, SGI134610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134611, SGI0, SGI0, SGI0, SGI134612, SGI134612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29325[] = {SGI134613, SGI0, SGI134614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29326[] = {SGI134615, SGI134616, SGI134617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134618, SGI0, SGI0, SGI0, SGI134619, SGI134619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29327[] = {SGI134620, SGI134621, SGI134622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134623, SGI0, SGI0, SGI0, SGI134624, SGI134624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29328[] = {SGI134625, SGI134626, SGI134627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134628, SGI0, SGI0, SGI0, SGI134629, SGI134629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29329[] = {SGI134630, SGI134631, SGI134632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134633, SGI0, SGI0, SGI0, SGI134634, SGI134634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29330[] = {SGI134635, SGI134636, SGI134637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134638, SGI0, SGI0, SGI0, SGI134639, SGI134639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29331[] = {SGI134640, SGI134641, SGI134642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134643, SGI0, SGI0, SGI0, SGI134644, SGI134644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29332[] = {SGI134645, SGI134646, SGI134647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134648, SGI0, SGI0, SGI0, SGI134649, SGI134649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29333[] = {SGI134650, SGI134651, SGI134652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134653, SGI0, SGI0, SGI0, SGI134654, SGI134654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29334[] = {SGI134655, SGI134656, SGI134657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134658, SGI0, SGI0, SGI0, SGI134659, SGI134659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29335[] = {SGI134660, SGI134661, SGI134662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134663, SGI0, SGI0, SGI0, SGI134664, SGI134664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29336[] = {SGI134665, SGI134666, SGI134667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134668, SGI0, SGI0, SGI0, SGI134669, SGI134669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29337[] = {SGI134670, SGI134671, SGI134672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134673, SGI0, SGI0, SGI0, SGI134674, SGI134674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29338[] = {SGI134675, SGI134676, SGI134677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134678, SGI0, SGI0, SGI0, SGI134679, SGI134679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29339[] = {SGI134680, SGI0, SGI134681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29340[] = {SGI134682, SGI134683, SGI134684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134685, SGI0, SGI0, SGI0, SGI134686, SGI134686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29341[] = {SGI134687, SGI134688, SGI134689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134690, SGI0, SGI0, SGI0, SGI134691, SGI134691, SGI0, SGI0, NULL};
const Segment * const SG29342[] = {SGI134692, SGI134693, SGI134694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134695, SGI134695, SGI0, SGI0, NULL};
const Segment * const SG29343[] = {SGI134696, SGI0, SGI134697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29344[] = {SGI134698, SGI134699, SGI134700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134701, SGI0, SGI0, SGI0, SGI134702, SGI134702, SGI0, SGI0, NULL};
const Segment * const SG29345[] = {SGI134703, SGI134704, SGI134705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134706, SGI0, SGI0, SGI0, SGI134707, SGI134707, SGI0, SGI0, NULL};
const Segment * const SG29346[] = {SGI134708, SGI134709, SGI134710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134711, SGI0, SGI0, SGI0, SGI134712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29347[] = {SGI134713, SGI134714, SGI134715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134716, SGI0, SGI0, SGI0, SGI134717, SGI134717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29348[] = {SGI134718, SGI0, SGI134719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29349[] = {SGI134720, SGI0, SGI134721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134722, SGI0, SGI0, SGI0, SGI134723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29350[] = {SGI134724, SGI134725, SGI134726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29351[] = {SGI134727, SGI134728, SGI134729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29352[] = {SGI134730, SGI134731, SGI134732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134733, SGI0, SGI0, SGI0, SGI134734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29353[] = {SGI134735, SGI134736, SGI134737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134738, SGI0, SGI0, SGI0, SGI134739, SGI134739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29354[] = {SGI134740, SGI134741, SGI134742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29355[] = {SGI134743, SGI134744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29356[] = {SGI134745, SGI134746, SGI134747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134748, SGI134748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29357[] = {SGI134749, SGI134750, SGI134751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134752, SGI0, SGI0, SGI0, SGI134753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29358[] = {SGI134754, SGI134755, SGI134756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134757, SGI134757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29359[] = {SGI134758, SGI134759, SGI134760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29360[] = {SGI134761, SGI134762, SGI134763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29361[] = {SGI0, SGI134764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29362[] = {SGI134765, SGI134766, SGI134767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134768, SGI134768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29363[] = {SGI134769, SGI134770, SGI134771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29364[] = {SGI134772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29365[] = {SGI134773, SGI134774, SGI134775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134776, SGI0, SGI0, SGI0, SGI0, SGI134777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29366[] = {SGI134778, SGI0, SGI134779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29367[] = {SGI134780, SGI134781, SGI134782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134783, SGI0, SGI0, SGI0, SGI134784, SGI134785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29368[] = {SGI134786, SGI134787, SGI134788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134789, SGI0, SGI0, SGI0, SGI134790, SGI134790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29369[] = {SGI134791, SGI134792, SGI134793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29370[] = {SGI0, SGI134794, SGI134795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29371[] = {SGI134796, SGI134797, SGI134798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29372[] = {SGI0, SGI0, SGI134799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29373[] = {SGI134800, SGI134801, SGI134802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29374[] = {SGI134803, SGI134804, SGI134805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134806, SGI0, SGI0, SGI0, SGI134807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29375[] = {SGI134808, SGI134809, SGI134810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134811, SGI0, SGI0, SGI0, SGI134812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29376[] = {SGI0, SGI134813, SGI134814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134815, SGI0, SGI0, SGI0, SGI134816, SGI134817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29377[] = {SGI134818, SGI134819, SGI134820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134821, SGI0, SGI0, SGI0, SGI134822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29378[] = {SGI0, SGI134823, SGI134824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134825, SGI0, SGI0, SGI0, SGI134826, SGI134826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29379[] = {SGI134827, SGI134828, SGI134829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134830, SGI0, SGI0, SGI0, SGI134831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29380[] = {SGI134832, SGI134833, SGI134834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134835, SGI0, SGI0, SGI0, SGI134836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29381[] = {SGI134837, SGI134838, SGI134839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134840, SGI0, SGI0, SGI0, SGI134841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29382[] = {SGI134842, SGI134843, SGI134844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134845, SGI0, SGI0, SGI0, SGI134846, SGI134846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29383[] = {SGI134847, SGI134848, SGI134849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134850, SGI0, SGI0, SGI0, SGI134851, SGI134851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29384[] = {SGI134852, SGI0, SGI134853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29385[] = {SGI134854, SGI134855, SGI134856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134857, SGI0, SGI0, SGI0, SGI134858, SGI134858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29386[] = {SGI134859, SGI0, SGI134860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29387[] = {SGI134861, SGI134862, SGI134863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134864, SGI0, SGI0, SGI0, SGI134865, SGI134865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29388[] = {SGI134866, SGI0, SGI134867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134868, SGI0, SGI0, SGI0, SGI134869, SGI134869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29389[] = {SGI134870, SGI134871, SGI134872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134873, SGI0, SGI0, SGI0, SGI134874, SGI134874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29390[] = {SGI134875, SGI134876, SGI134877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134878, SGI0, SGI0, SGI0, SGI134879, SGI134879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29391[] = {SGI134880, SGI134881, SGI134882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134883, SGI0, SGI0, SGI0, SGI134884, SGI134884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29392[] = {SGI134885, SGI134886, SGI134887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134888, SGI0, SGI0, SGI0, SGI134889, SGI134889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29393[] = {SGI134890, SGI134891, SGI134892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134893, SGI0, SGI0, SGI0, SGI134894, SGI134894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29394[] = {SGI134895, SGI134896, SGI134897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134898, SGI0, SGI0, SGI0, SGI134899, SGI134899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29395[] = {SGI134900, SGI134901, SGI134902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134903, SGI0, SGI0, SGI0, SGI134904, SGI134904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29396[] = {SGI134905, SGI0, SGI134906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29397[] = {SGI134907, SGI0, SGI134908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29398[] = {SGI134909, SGI134910, SGI134911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134912, SGI0, SGI0, SGI0, SGI134913, SGI134913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29399[] = {SGI134914, SGI134915, SGI134916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134917, SGI0, SGI0, SGI0, SGI134918, SGI134918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29400[] = {SGI134919, SGI134920, SGI134921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134922, SGI0, SGI0, SGI0, SGI134923, SGI134923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29401[] = {SGI134924, SGI134925, SGI134926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134927, SGI0, SGI0, SGI0, SGI134928, SGI134928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29402[] = {SGI134929, SGI0, SGI134930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29403[] = {SGI134931, SGI0, SGI134932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29404[] = {SGI134933, SGI134934, SGI134935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134936, SGI0, SGI0, SGI0, SGI134937, SGI134937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29405[] = {SGI134938, SGI0, SGI134939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29406[] = {SGI134940, SGI134941, SGI134942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134943, SGI0, SGI0, SGI0, SGI134944, SGI134944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29407[] = {SGI134945, SGI134946, SGI134947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134948, SGI0, SGI0, SGI0, SGI134949, SGI134949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29408[] = {SGI134950, SGI134951, SGI134952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134953, SGI0, SGI0, SGI0, SGI134954, SGI134954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29409[] = {SGI134955, SGI134956, SGI134957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134958, SGI0, SGI0, SGI0, SGI134959, SGI134959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29410[] = {SGI134960, SGI134961, SGI134962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134963, SGI0, SGI0, SGI0, SGI134964, SGI134964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29411[] = {SGI134965, SGI134966, SGI134967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134968, SGI0, SGI0, SGI0, SGI134969, SGI134969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29412[] = {SGI134970, SGI134971, SGI134972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134973, SGI0, SGI0, SGI0, SGI134974, SGI134974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29413[] = {SGI134975, SGI134976, SGI134977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134978, SGI0, SGI0, SGI0, SGI134979, SGI134979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29414[] = {SGI134980, SGI134981, SGI134982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134983, SGI0, SGI0, SGI0, SGI134984, SGI134984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29415[] = {SGI134985, SGI134986, SGI134987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134988, SGI0, SGI0, SGI0, SGI134989, SGI134989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29416[] = {SGI134990, SGI134991, SGI134992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134993, SGI0, SGI0, SGI0, SGI134994, SGI134994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29417[] = {SGI134995, SGI0, SGI134996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI134997, SGI0, SGI0, SGI0, SGI134998, SGI134998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29418[] = {SGI134999, SGI135000, SGI135001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135002, SGI0, SGI0, SGI0, SGI135003, SGI135003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29419[] = {SGI135004, SGI135005, SGI135006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135007, SGI0, SGI0, SGI0, SGI135008, SGI135008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29420[] = {SGI135009, SGI135010, SGI135011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135012, SGI0, SGI0, SGI0, SGI135013, SGI135013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29421[] = {SGI135014, SGI135015, SGI135016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135017, SGI0, SGI0, SGI0, SGI135018, SGI135018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29422[] = {SGI135019, SGI135020, SGI135021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135022, SGI0, SGI0, SGI0, SGI135023, SGI135023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29423[] = {SGI135024, SGI135025, SGI135026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135027, SGI0, SGI0, SGI0, SGI135028, SGI135028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29424[] = {SGI135029, SGI135030, SGI135031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135032, SGI0, SGI0, SGI0, SGI135033, SGI135033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29425[] = {SGI135034, SGI135035, SGI135036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135037, SGI0, SGI0, SGI0, SGI135038, SGI135038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29426[] = {SGI135039, SGI135040, SGI135041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135042, SGI0, SGI0, SGI0, SGI135043, SGI135043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29427[] = {SGI135049, SGI135050, SGI135051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135052, SGI0, SGI0, SGI0, SGI135053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29428[] = {SGI135054, SGI135055, SGI135056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135057, SGI0, SGI0, SGI0, SGI135058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29429[] = {SGI135059, SGI135060, SGI135061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135062, SGI0, SGI0, SGI0, SGI135063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29430[] = {SGI135064, SGI135065, SGI135066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135067, SGI0, SGI0, SGI0, SGI135068, SGI135068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29431[] = {SGI135044, SGI135045, SGI135046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135047, SGI0, SGI0, SGI0, SGI135048, SGI135048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29432[] = {SGI135069, SGI135070, SGI135071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135072, SGI0, SGI0, SGI0, SGI135073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29433[] = {SGI135074, SGI0, SGI135075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135076, SGI0, SGI0, SGI0, SGI135077, SGI135077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29434[] = {SGI135078, SGI135079, SGI135080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135081, SGI0, SGI0, SGI0, SGI135082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29435[] = {SGI135083, SGI135084, SGI135085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135086, SGI0, SGI0, SGI0, SGI135087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29436[] = {SGI135088, SGI135089, SGI135090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135091, SGI0, SGI0, SGI0, SGI135092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29437[] = {SGI135093, SGI135094, SGI135095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135096, SGI0, SGI0, SGI0, SGI135097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29438[] = {SGI135098, SGI135099, SGI135100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135101, SGI0, SGI0, SGI0, SGI135102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29439[] = {SGI135103, SGI135104, SGI135105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135106, SGI0, SGI0, SGI0, SGI135107, SGI135107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29440[] = {SGI135108, SGI135109, SGI135110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135111, SGI0, SGI0, SGI0, SGI135112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29441[] = {SGI135113, SGI135114, SGI135115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135116, SGI0, SGI0, SGI0, SGI135117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29442[] = {SGI135118, SGI135119, SGI135120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135121, SGI0, SGI0, SGI0, SGI135122, SGI135122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29443[] = {SGI135123, SGI135124, SGI135125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135126, SGI0, SGI0, SGI0, SGI135127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29444[] = {SGI135128, SGI135129, SGI135130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135131, SGI0, SGI0, SGI0, SGI135132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29445[] = {SGI135133, SGI135134, SGI135135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135136, SGI0, SGI0, SGI0, SGI135137, SGI135137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29446[] = {SGI135138, SGI135139, SGI135140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135141, SGI0, SGI0, SGI0, SGI135142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29447[] = {SGI0, SGI0, SGI135143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135144, SGI0, SGI0, SGI0, SGI135145, SGI135145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29448[] = {SGI135146, SGI135147, SGI135148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135149, SGI0, SGI0, SGI0, SGI135150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29449[] = {SGI135151, SGI135152, SGI135153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135154, SGI0, SGI0, SGI0, SGI135155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29450[] = {SGI135156, SGI135157, SGI135158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135159, SGI0, SGI0, SGI0, SGI135160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29451[] = {SGI135161, SGI135162, SGI135163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135164, SGI0, SGI0, SGI0, SGI135165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29452[] = {SGI135166, SGI135167, SGI135168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135169, SGI0, SGI0, SGI0, SGI135170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29453[] = {SGI135171, SGI135172, SGI135173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135174, SGI0, SGI0, SGI0, SGI135175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29454[] = {SGI135176, SGI135177, SGI135178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135179, SGI0, SGI0, SGI0, SGI135180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29455[] = {SGI135181, SGI135182, SGI135183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135184, SGI0, SGI0, SGI0, SGI135185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29456[] = {SGI135186, SGI135187, SGI135188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135189, SGI0, SGI0, SGI0, SGI135190, SGI135190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29457[] = {SGI135191, SGI135192, SGI135193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135194, SGI0, SGI0, SGI0, SGI135195, SGI135195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29458[] = {SGI135196, SGI135197, SGI135198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135199, SGI0, SGI0, SGI0, SGI135200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29459[] = {SGI135201, SGI135202, SGI135203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135204, SGI0, SGI0, SGI0, SGI135205, SGI135205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29460[] = {SGI135206, SGI135207, SGI135208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135209, SGI0, SGI0, SGI0, SGI135210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29461[] = {SGI135211, SGI135212, SGI135213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135214, SGI0, SGI0, SGI0, SGI135215, SGI135215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29462[] = {SGI135216, SGI135217, SGI135218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135219, SGI0, SGI0, SGI0, SGI135220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29463[] = {SGI135221, SGI135222, SGI135223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135224, SGI0, SGI0, SGI0, SGI135225, SGI135225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29464[] = {SGI135226, SGI135227, SGI135228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135229, SGI0, SGI0, SGI0, SGI135230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29465[] = {SGI135231, SGI135232, SGI135233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135234, SGI0, SGI0, SGI0, SGI135235, SGI135235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29466[] = {SGI135236, SGI135237, SGI135238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135239, SGI0, SGI0, SGI0, SGI135240, SGI135240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29467[] = {SGI135241, SGI135242, SGI135243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135244, SGI0, SGI0, SGI0, SGI135245, SGI135245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29468[] = {SGI135246, SGI135247, SGI135248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135249, SGI0, SGI0, SGI0, SGI135250, SGI135250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29469[] = {SGI135251, SGI135252, SGI135253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135254, SGI0, SGI0, SGI0, SGI135255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29470[] = {SGI135256, SGI135257, SGI135258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135259, SGI0, SGI0, SGI0, SGI135260, SGI135260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29471[] = {SGI135261, SGI135262, SGI135263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135264, SGI0, SGI0, SGI0, SGI135265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29472[] = {SGI135266, SGI135267, SGI135268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135269, SGI0, SGI0, SGI0, SGI135270, SGI135270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29473[] = {SGI135271, SGI135272, SGI135273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135274, SGI0, SGI0, SGI0, SGI135275, SGI135275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29474[] = {SGI135276, SGI135277, SGI135278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135279, SGI0, SGI0, SGI0, SGI135280, SGI135280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29475[] = {SGI135281, SGI135282, SGI135283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135284, SGI0, SGI0, SGI0, SGI135285, SGI135285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29476[] = {SGI135286, SGI135287, SGI135288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135289, SGI0, SGI0, SGI0, SGI135290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29477[] = {SGI135291, SGI135292, SGI135293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135294, SGI0, SGI0, SGI0, SGI135295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29478[] = {SGI135296, SGI135297, SGI135298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135299, SGI0, SGI0, SGI0, SGI135300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29479[] = {SGI135301, SGI135302, SGI135303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135304, SGI0, SGI0, SGI0, SGI135305, SGI135305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29480[] = {SGI135306, SGI135307, SGI135308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135309, SGI0, SGI0, SGI0, SGI135310, SGI135310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29481[] = {SGI135311, SGI135312, SGI135313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135314, SGI0, SGI0, SGI0, SGI135315, SGI135315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29482[] = {SGI135316, SGI135317, SGI135318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135319, SGI0, SGI0, SGI0, SGI135320, SGI135320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29483[] = {SGI135321, SGI135322, SGI135323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135324, SGI0, SGI0, SGI0, SGI135325, SGI135325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29484[] = {SGI135326, SGI135327, SGI135328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135329, SGI0, SGI0, SGI0, SGI135330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29485[] = {SGI135331, SGI135332, SGI135333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135334, SGI0, SGI0, SGI0, SGI135335, SGI135335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29486[] = {SGI135336, SGI135337, SGI135338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135339, SGI0, SGI0, SGI0, SGI135340, SGI135340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29487[] = {SGI135341, SGI135342, SGI135343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135344, SGI0, SGI0, SGI0, SGI135345, SGI135345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29488[] = {SGI135346, SGI135347, SGI135348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135349, SGI0, SGI0, SGI0, SGI135350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29489[] = {SGI135351, SGI135352, SGI135353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135354, SGI0, SGI0, SGI0, SGI135355, SGI135355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29490[] = {SGI135356, SGI135357, SGI135358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135359, SGI0, SGI0, SGI0, SGI135360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29491[] = {SGI135361, SGI135362, SGI135363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135364, SGI0, SGI0, SGI0, SGI135365, SGI135365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29492[] = {SGI135366, SGI135367, SGI135368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135369, SGI0, SGI0, SGI0, SGI135370, SGI135370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29493[] = {SGI135371, SGI135372, SGI135373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135374, SGI0, SGI0, SGI0, SGI135375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29494[] = {SGI135376, SGI135377, SGI135378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135379, SGI0, SGI0, SGI0, SGI135380, SGI135380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29495[] = {SGI135381, SGI135382, SGI135383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135384, SGI0, SGI0, SGI0, SGI135385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29496[] = {SGI135386, SGI0, SGI135387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29497[] = {SGI135388, SGI135389, SGI135390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135391, SGI0, SGI0, SGI0, SGI135392, SGI135392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29498[] = {SGI135393, SGI135394, SGI135395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135396, SGI0, SGI0, SGI0, SGI135397, SGI135397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29499[] = {SGI135398, SGI135399, SGI135400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135401, SGI0, SGI0, SGI0, SGI135402, SGI135402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29500[] = {SGI135403, SGI135404, SGI135405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135406, SGI0, SGI0, SGI0, SGI135407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29501[] = {SGI135408, SGI0, SGI135409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29502[] = {SGI135410, SGI135411, SGI135412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135413, SGI0, SGI0, SGI0, SGI135414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29503[] = {SGI135415, SGI135416, SGI135417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135418, SGI0, SGI0, SGI0, SGI135419, SGI135419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29504[] = {SGI135420, SGI135421, SGI135422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135423, SGI0, SGI0, SGI0, SGI135424, SGI135424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29505[] = {SGI135425, SGI135426, SGI135427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135428, SGI0, SGI0, SGI0, SGI135429, SGI135429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29506[] = {SGI135430, SGI135431, SGI135432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135433, SGI0, SGI0, SGI0, SGI135434, SGI135434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29507[] = {SGI135435, SGI135436, SGI135437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135438, SGI0, SGI0, SGI0, SGI135439, SGI135439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29508[] = {SGI135440, SGI135441, SGI135442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135443, SGI0, SGI0, SGI0, SGI135444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29509[] = {SGI135445, SGI135446, SGI135447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135448, SGI0, SGI0, SGI0, SGI135449, SGI135449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29510[] = {SGI135450, SGI135451, SGI135452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135453, SGI0, SGI0, SGI0, SGI135454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29511[] = {SGI135455, SGI135456, SGI135457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135458, SGI0, SGI0, SGI0, SGI135459, SGI135459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29512[] = {SGI135460, SGI135461, SGI135462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135463, SGI0, SGI0, SGI0, SGI135464, SGI135464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29513[] = {SGI135465, SGI135466, SGI135467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135468, SGI0, SGI0, SGI0, SGI135469, SGI135469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29514[] = {SGI135470, SGI0, SGI135471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29515[] = {SGI135472, SGI135473, SGI135474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135475, SGI0, SGI0, SGI0, SGI135476, SGI135476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29516[] = {SGI135477, SGI135478, SGI135479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135480, SGI0, SGI0, SGI0, SGI135481, SGI135481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29517[] = {SGI135482, SGI0, SGI135483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29518[] = {SGI135484, SGI135485, SGI135486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135487, SGI0, SGI0, SGI0, SGI135488, SGI135488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29519[] = {SGI135489, SGI0, SGI135490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29520[] = {SGI135491, SGI135492, SGI135493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135494, SGI0, SGI0, SGI0, SGI135495, SGI135495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29521[] = {SGI135496, SGI135497, SGI135498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135499, SGI0, SGI0, SGI0, SGI135500, SGI135500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29522[] = {SGI135501, SGI135502, SGI135503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135504, SGI0, SGI0, SGI0, SGI135505, SGI135505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29523[] = {SGI135506, SGI135507, SGI135508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135509, SGI0, SGI0, SGI0, SGI135510, SGI135510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29524[] = {SGI135511, SGI0, SGI135512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29525[] = {SGI135513, SGI135514, SGI135515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135516, SGI0, SGI0, SGI0, SGI135517, SGI135517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29526[] = {SGI135518, SGI0, SGI135519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29527[] = {SGI135520, SGI135521, SGI135522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135523, SGI0, SGI0, SGI0, SGI135524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29528[] = {SGI135525, SGI135526, SGI135527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135528, SGI0, SGI0, SGI0, SGI135529, SGI135529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29529[] = {SGI135530, SGI135531, SGI135532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135533, SGI0, SGI0, SGI0, SGI135534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29530[] = {SGI135535, SGI135536, SGI135537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135538, SGI0, SGI0, SGI0, SGI135539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29531[] = {SGI135540, SGI135541, SGI135542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135543, SGI0, SGI0, SGI0, SGI135544, SGI135544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29532[] = {SGI135545, SGI135546, SGI135547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135548, SGI0, SGI0, SGI0, SGI135549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29533[] = {SGI135550, SGI135551, SGI135552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135553, SGI0, SGI0, SGI0, SGI135554, SGI135554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29534[] = {SGI135555, SGI135556, SGI135557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135558, SGI0, SGI0, SGI0, SGI135559, SGI135559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29535[] = {SGI135560, SGI135561, SGI135562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135563, SGI0, SGI0, SGI0, SGI135564, SGI135564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29536[] = {SGI135565, SGI135566, SGI135567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135568, SGI0, SGI0, SGI0, SGI135569, SGI135569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29537[] = {SGI135570, SGI135571, SGI135572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135573, SGI0, SGI0, SGI0, SGI135574, SGI135574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29538[] = {SGI135575, SGI135576, SGI135577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135578, SGI0, SGI0, SGI0, SGI135579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29539[] = {SGI135580, SGI135581, SGI135582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135583, SGI0, SGI0, SGI0, SGI135584, SGI135584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29540[] = {SGI135585, SGI0, SGI135586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29541[] = {SGI135587, SGI0, SGI135588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29542[] = {SGI135589, SGI135590, SGI135591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135592, SGI0, SGI0, SGI0, SGI135593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29543[] = {SGI135594, SGI135595, SGI135596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135597, SGI0, SGI0, SGI0, SGI135598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29544[] = {SGI135599, SGI135600, SGI135601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135602, SGI0, SGI0, SGI0, SGI135603, SGI135603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29545[] = {SGI135604, SGI135605, SGI135606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135607, SGI0, SGI0, SGI0, SGI135608, SGI135608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29546[] = {SGI135609, SGI0, SGI135610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29547[] = {SGI135611, SGI0, SGI135612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29548[] = {SGI135613, SGI0, SGI135614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135615, SGI135616, SGI135616, SGI0, SGI0, NULL};
const Segment * const SG29549[] = {SGI135617, SGI135618, SGI135619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135620, SGI0, SGI0, SGI0, SGI135621, SGI135621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29550[] = {SGI135622, SGI135623, SGI135624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135625, SGI0, SGI0, SGI0, SGI135626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29551[] = {SGI135627, SGI135628, SGI135629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135630, SGI0, SGI0, SGI0, SGI135631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29552[] = {SGI135632, SGI135633, SGI135634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135635, SGI0, SGI0, SGI0, SGI135636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29553[] = {SGI135637, SGI0, SGI135638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29554[] = {SGI135639, SGI135640, SGI135641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135642, SGI0, SGI0, SGI0, SGI135643, SGI135643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29555[] = {SGI135644, SGI0, SGI135645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29556[] = {SGI135646, SGI135647, SGI135648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135649, SGI0, SGI0, SGI0, SGI135650, SGI135650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29557[] = {SGI135651, SGI135652, SGI135653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135654, SGI0, SGI0, SGI0, SGI135655, SGI135655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29558[] = {SGI135656, SGI135657, SGI135658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135659, SGI0, SGI0, SGI0, SGI135660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29559[] = {SGI135661, SGI0, SGI135662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29560[] = {SGI135663, SGI0, SGI135664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29561[] = {SGI135665, SGI135666, SGI135667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135668, SGI0, SGI0, SGI0, SGI135669, SGI135669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29562[] = {SGI135670, SGI135671, SGI135672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135673, SGI0, SGI0, SGI0, SGI135674, SGI135674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29563[] = {SGI135675, SGI135676, SGI135677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135678, SGI0, SGI0, SGI0, SGI135679, SGI135679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29564[] = {SGI135680, SGI0, SGI135681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29565[] = {SGI135682, SGI0, SGI135683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135684, SGI135685, SGI135685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29566[] = {SGI135686, SGI0, SGI135687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135688, SGI135689, SGI135689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29567[] = {SGI135690, SGI0, SGI135691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135692, SGI135693, SGI135693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29568[] = {SGI135694, SGI0, SGI135695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135696, SGI135697, SGI135697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29569[] = {SGI135698, SGI0, SGI135699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135700, SGI135701, SGI135701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29570[] = {SGI135702, SGI0, SGI135703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135704, SGI135705, SGI135705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29571[] = {SGI135706, SGI0, SGI135707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135708, SGI135709, SGI135709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29572[] = {SGI135710, SGI0, SGI135711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135712, SGI135713, SGI135713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29573[] = {SGI135714, SGI0, SGI135715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135716, SGI135717, SGI135717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29574[] = {SGI135718, SGI0, SGI135719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135720, SGI135721, SGI135721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29575[] = {SGI135722, SGI0, SGI135723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135724, SGI135725, SGI135725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29576[] = {SGI135726, SGI0, SGI135727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135728, SGI135729, SGI135729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29577[] = {SGI135730, SGI0, SGI135731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135732, SGI135733, SGI135733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29578[] = {SGI135734, SGI0, SGI135735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135736, SGI135737, SGI135737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29579[] = {SGI135738, SGI0, SGI135739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135740, SGI135741, SGI135741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29580[] = {SGI135742, SGI0, SGI135743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135744, SGI135745, SGI135745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29581[] = {SGI135746, SGI0, SGI135747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135748, SGI135749, SGI135749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29582[] = {SGI135750, SGI135751, SGI135752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135753, SGI0, SGI0, SGI0, SGI135754, SGI135754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29583[] = {SGI135755, SGI135756, SGI135757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135758, SGI0, SGI0, SGI0, SGI135759, SGI135759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29584[] = {SGI135760, SGI0, SGI135761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29585[] = {SGI135762, SGI135763, SGI135764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135765, SGI0, SGI0, SGI0, SGI135766, SGI135766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29586[] = {SGI135767, SGI135768, SGI135769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135770, SGI135770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29587[] = {SGI135771, SGI135772, SGI135773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135774, SGI135774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29588[] = {SGI135775, SGI135776, SGI135777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135778, SGI0, SGI0, SGI0, SGI135779, SGI135779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29589[] = {SGI135780, SGI135781, SGI135782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135783, SGI0, SGI0, SGI0, SGI135784, SGI135784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29590[] = {SGI135785, SGI0, SGI135786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29591[] = {SGI135787, SGI135788, SGI135789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135790, SGI0, SGI0, SGI0, SGI135791, SGI135791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29592[] = {SGI135792, SGI135793, SGI135794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135795, SGI0, SGI0, SGI0, SGI135796, SGI135796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29593[] = {SGI135797, SGI135798, SGI135799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135800, SGI0, SGI0, SGI0, SGI135801, SGI135801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29594[] = {SGI135802, SGI135803, SGI135804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135805, SGI0, SGI0, SGI0, SGI135806, SGI135806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29595[] = {SGI135807, SGI135808, SGI135809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135810, SGI0, SGI0, SGI0, SGI135811, SGI135811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29596[] = {SGI135812, SGI135813, SGI135814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135815, SGI0, SGI0, SGI0, SGI135816, SGI135816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29597[] = {SGI135817, SGI135818, SGI135819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135820, SGI0, SGI0, SGI0, SGI135821, SGI135821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29598[] = {SGI135822, SGI135823, SGI135824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135825, SGI0, SGI0, SGI0, SGI135826, SGI135826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29599[] = {SGI135827, SGI135828, SGI135829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135830, SGI0, SGI0, SGI0, SGI135831, SGI135831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29600[] = {SGI135832, SGI0, SGI135833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29601[] = {SGI135834, SGI135835, SGI135836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135837, SGI0, SGI0, SGI0, SGI135838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29602[] = {SGI135839, SGI135840, SGI135841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135842, SGI0, SGI0, SGI0, SGI135843, SGI135843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29603[] = {SGI135844, SGI135845, SGI135846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135847, SGI0, SGI0, SGI0, SGI135848, SGI135848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29604[] = {SGI135849, SGI135850, SGI135851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135852, SGI0, SGI0, SGI0, SGI135853, SGI135853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29605[] = {SGI135854, SGI0, SGI135855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135856, SGI0, SGI0, SGI0, SGI135857, SGI135857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29606[] = {SGI135858, SGI135859, SGI135860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135861, SGI0, SGI0, SGI0, SGI135862, SGI135862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29607[] = {SGI135863, SGI135864, SGI135865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135866, SGI0, SGI0, SGI0, SGI135867, SGI135867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29608[] = {SGI135868, SGI135869, SGI135870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135871, SGI0, SGI0, SGI0, SGI135872, SGI135872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29609[] = {SGI135873, SGI135874, SGI135875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135876, SGI0, SGI0, SGI0, SGI135877, SGI135877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29610[] = {SGI135878, SGI135879, SGI135880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135881, SGI0, SGI0, SGI0, SGI135882, SGI135882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29611[] = {SGI135883, SGI135884, SGI135885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135886, SGI135886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29612[] = {SGI135887, SGI135888, SGI135889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135890, SGI0, SGI0, SGI0, SGI135891, SGI135891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29613[] = {SGI135892, SGI135893, SGI135894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135895, SGI0, SGI0, SGI0, SGI135896, SGI135896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29614[] = {SGI135897, SGI0, SGI135898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29615[] = {SGI135899, SGI0, SGI135900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135901, SGI0, SGI0, SGI0, SGI135902, SGI135902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29616[] = {SGI135903, SGI135904, SGI135905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135906, SGI0, SGI0, SGI0, SGI135907, SGI135907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29617[] = {SGI135908, SGI135909, SGI135910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135911, SGI0, SGI0, SGI0, SGI135912, SGI135912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29618[] = {SGI135913, SGI135914, SGI135915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135916, SGI0, SGI0, SGI0, SGI135917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29619[] = {SGI135918, SGI0, SGI135919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29620[] = {SGI135920, SGI0, SGI135921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135922, SGI0, SGI0, SGI0, SGI135923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29621[] = {SGI135924, SGI135925, SGI135926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135927, SGI0, SGI0, SGI0, SGI135928, SGI135928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29622[] = {SGI135929, SGI135930, SGI135931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135932, SGI0, SGI0, SGI0, SGI135933, SGI135933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29623[] = {SGI135934, SGI135935, SGI135936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135937, SGI0, SGI0, SGI0, SGI135938, SGI135939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29624[] = {SGI135940, SGI135941, SGI135942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135943, SGI0, SGI0, SGI0, SGI135944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29625[] = {SGI135945, SGI135946, SGI135947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135948, SGI0, SGI0, SGI0, SGI135949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29626[] = {SGI135950, SGI135951, SGI135952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135953, SGI0, SGI0, SGI0, SGI135954, SGI135954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29627[] = {SGI135955, SGI135956, SGI135957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29628[] = {SGI135958, SGI135959, SGI135960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29629[] = {SGI135961, SGI135962, SGI135963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135964, SGI135964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29630[] = {SGI135965, SGI135966, SGI135967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135968, SGI0, SGI0, SGI0, SGI135969, SGI135969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29631[] = {SGI135970, SGI135971, SGI135972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29632[] = {SGI135973, SGI135974, SGI135975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29633[] = {SGI135976, SGI135977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135978, SGI0, SGI0, SGI0, SGI135979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29634[] = {SGI135980, SGI135981, SGI135982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135983, SGI135983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29635[] = {SGI135984, SGI135985, SGI135986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135987, SGI0, SGI0, SGI0, SGI135988, SGI135989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29636[] = {SGI0, SGI135990, SGI135991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29637[] = {SGI135993, SGI135994, SGI135995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI135996, SGI0, SGI0, SGI0, SGI135997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29638[] = {SGI135998, SGI135999, SGI136000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29639[] = {SGI136002, SGI136003, SGI136004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29640[] = {SGI136005, SGI136006, SGI136007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29641[] = {SGI136008, SGI136009, SGI136010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136011, SGI0, SGI0, SGI0, SGI136012, SGI136012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29642[] = {SGI136013, SGI136014, SGI136015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136016, SGI0, SGI0, SGI0, SGI136017, SGI136017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29643[] = {SGI136018, SGI136019, SGI136020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136021, SGI0, SGI0, SGI0, SGI136022, SGI136022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29644[] = {SGI136023, SGI0, SGI136024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29645[] = {SGI136025, SGI136026, SGI136027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136028, SGI0, SGI0, SGI0, SGI136029, SGI136029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29646[] = {SGI136030, SGI136031, SGI136032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136033, SGI0, SGI0, SGI0, SGI136034, SGI136034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29647[] = {SGI136035, SGI136036, SGI136037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136038, SGI0, SGI0, SGI0, SGI136039, SGI136039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29648[] = {SGI136040, SGI136041, SGI136042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136043, SGI0, SGI0, SGI0, SGI136044, SGI136044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29649[] = {SGI136045, SGI136046, SGI136047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136048, SGI0, SGI0, SGI0, SGI136049, SGI136049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29650[] = {SGI136050, SGI136051, SGI136052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136053, SGI0, SGI0, SGI0, SGI136054, SGI136054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29651[] = {SGI136055, SGI136056, SGI136057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136058, SGI0, SGI0, SGI0, SGI136059, SGI136059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29652[] = {SGI136060, SGI136061, SGI136062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136063, SGI136063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29653[] = {SGI136064, SGI136065, SGI136066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136067, SGI0, SGI0, SGI0, SGI136068, SGI136068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29654[] = {SGI136069, SGI136070, SGI136071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136072, SGI136072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29655[] = {SGI136073, SGI136074, SGI136075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136076, SGI0, SGI0, SGI0, SGI136077, SGI136077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29656[] = {SGI136078, SGI136079, SGI136080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136081, SGI136081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29657[] = {SGI136082, SGI136083, SGI136084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136085, SGI0, SGI0, SGI0, SGI136086, SGI136086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29658[] = {SGI136087, SGI136088, SGI136089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136090, SGI0, SGI0, SGI0, SGI136091, SGI136091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29659[] = {SGI136092, SGI136093, SGI136094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136095, SGI0, SGI0, SGI0, SGI136096, SGI136096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29660[] = {SGI136097, SGI136098, SGI136099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136100, SGI136100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29661[] = {SGI136101, SGI136102, SGI136103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136104, SGI0, SGI0, SGI0, SGI136105, SGI136105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29662[] = {SGI136106, SGI0, SGI136107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29663[] = {SGI136108, SGI136109, SGI136110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136111, SGI0, SGI0, SGI0, SGI136112, SGI136112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29664[] = {SGI136113, SGI136114, SGI136115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136116, SGI0, SGI0, SGI0, SGI136117, SGI136117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29665[] = {SGI136118, SGI136119, SGI136120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136121, SGI0, SGI0, SGI0, SGI136122, SGI136122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29666[] = {SGI136123, SGI136124, SGI136125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136126, SGI0, SGI0, SGI0, SGI136127, SGI136127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29667[] = {SGI136128, SGI136129, SGI136130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136131, SGI0, SGI0, SGI0, SGI136132, SGI136132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29668[] = {SGI136133, SGI136134, SGI136135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136136, SGI136136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29669[] = {SGI136137, SGI136138, SGI136139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136143, SGI0, SGI0, SGI0, SGI136144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29670[] = {SGI136140, SGI136141, SGI136142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136145, SGI0, SGI0, SGI0, SGI136146, SGI136146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29671[] = {SGI136147, SGI136148, SGI136149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136150, SGI0, SGI0, SGI0, SGI136151, SGI136151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29672[] = {SGI136152, SGI136153, SGI136154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136155, SGI0, SGI0, SGI0, SGI136156, SGI136156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29673[] = {SGI136157, SGI136158, SGI136159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136160, SGI136160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29674[] = {SGI136161, SGI136162, SGI136163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136164, SGI0, SGI0, SGI0, SGI136165, SGI136165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29675[] = {SGI136166, SGI136167, SGI136168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136169, SGI136169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29676[] = {SGI136170, SGI136171, SGI136172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136173, SGI136174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29677[] = {SGI136175, SGI136176, SGI136177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136178, SGI0, SGI0, SGI0, SGI136179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29678[] = {SGI136180, SGI136181, SGI136182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136183, SGI136183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29679[] = {SGI136184, SGI136185, SGI136186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136187, SGI0, SGI0, SGI0, SGI136188, SGI136188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29680[] = {SGI136189, SGI136190, SGI136191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136192, SGI136192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29681[] = {SGI136193, SGI136194, SGI136195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136196, SGI0, SGI0, SGI0, SGI136197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29682[] = {SGI136198, SGI0, SGI136199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29683[] = {SGI136200, SGI136201, SGI136202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136203, SGI0, SGI0, SGI0, SGI136204, SGI136204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29684[] = {SGI136205, SGI136206, SGI136207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136208, SGI0, SGI0, SGI0, SGI136209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29685[] = {SGI136210, SGI136211, SGI136212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136213, SGI0, SGI0, SGI0, SGI136214, SGI136214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29686[] = {SGI136215, SGI136216, SGI136217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136218, SGI0, SGI0, SGI0, SGI136219, SGI136219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29687[] = {SGI136220, SGI136221, SGI136222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136223, SGI0, SGI0, SGI0, SGI136224, SGI136224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29688[] = {SGI136225, SGI136226, SGI136227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136228, SGI0, SGI0, SGI0, SGI136229, SGI136229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29689[] = {SGI136230, SGI136231, SGI136232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136233, SGI0, SGI0, SGI0, SGI136234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29690[] = {SGI136235, SGI136236, SGI136237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136238, SGI0, SGI0, SGI0, SGI136239, SGI136239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29691[] = {SGI136240, SGI136241, SGI136242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136243, SGI0, SGI0, SGI0, SGI136244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29692[] = {SGI136245, SGI136246, SGI136247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136248, SGI0, SGI0, SGI0, SGI136249, SGI136249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29693[] = {SGI136250, SGI0, SGI136251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29694[] = {SGI136252, SGI136253, SGI136254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136255, SGI0, SGI0, SGI0, SGI136256, SGI136256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29695[] = {SGI136257, SGI136258, SGI136259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136260, SGI0, SGI0, SGI0, SGI136261, SGI136261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29696[] = {SGI136262, SGI136263, SGI136264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136265, SGI0, SGI0, SGI0, SGI136266, SGI136266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29697[] = {SGI136267, SGI136268, SGI136269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136270, SGI0, SGI0, SGI0, SGI136271, SGI136271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29698[] = {SGI0, SGI0, SGI136272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29699[] = {SGI136273, SGI136274, SGI136275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136276, SGI0, SGI0, SGI0, SGI136277, SGI136277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29700[] = {SGI136278, SGI136279, SGI136280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136281, SGI0, SGI0, SGI0, SGI136282, SGI136282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29701[] = {SGI136283, SGI136284, SGI136285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136286, SGI0, SGI0, SGI0, SGI136287, SGI136287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29702[] = {SGI136288, SGI136289, SGI136290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136291, SGI0, SGI0, SGI0, SGI136292, SGI136292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29703[] = {SGI136293, SGI136294, SGI136295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136296, SGI0, SGI0, SGI0, SGI136297, SGI136297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29704[] = {SGI136298, SGI0, SGI136299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29705[] = {SGI136300, SGI136301, SGI136302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136303, SGI0, SGI0, SGI0, SGI136304, SGI136304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29706[] = {SGI136305, SGI136306, SGI136307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136308, SGI0, SGI0, SGI0, SGI136309, SGI136309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29707[] = {SGI136310, SGI136311, SGI136312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136313, SGI0, SGI0, SGI0, SGI136314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29708[] = {SGI136315, SGI0, SGI136316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29709[] = {SGI136317, SGI0, SGI136318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136319, SGI0, SGI0, SGI0, SGI136320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29710[] = {SGI136321, SGI136322, SGI136323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136324, SGI0, SGI0, SGI0, SGI136325, SGI136326, SGI0, SGI0, NULL};
const Segment * const SG29711[] = {SGI136327, SGI0, SGI136328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136329, SGI136330, SGI136330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29712[] = {SGI136331, SGI0, SGI136332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136333, SGI136334, SGI136334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29713[] = {SGI136335, SGI0, SGI136336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136337, SGI136338, SGI136338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29714[] = {SGI136339, SGI0, SGI136340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136341, SGI136342, SGI136342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29715[] = {SGI136343, SGI0, SGI136344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136345, SGI136346, SGI136346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29716[] = {SGI136347, SGI136348, SGI136350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136352, SGI0, SGI0, SGI0, SGI136353, SGI136353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29717[] = {SGI136349, SGI0, SGI136351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136354, SGI136355, SGI136355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29718[] = {SGI136356, SGI0, SGI136357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29719[] = {SGI136358, SGI0, SGI136359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136360, SGI136361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29720[] = {SGI136362, SGI136363, SGI136364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136366, SGI0, SGI0, SGI0, SGI136368, SGI136368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29721[] = {SGI136365, SGI0, SGI136367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136369, SGI136370, SGI136370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29722[] = {SGI136371, SGI136372, SGI136373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136375, SGI0, SGI0, SGI0, SGI136377, SGI136377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29723[] = {SGI136374, SGI0, SGI136376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136378, SGI136379, SGI136379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29724[] = {SGI136380, SGI136381, SGI136382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136385, SGI0, SGI0, SGI0, SGI136386, SGI136386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29725[] = {SGI136383, SGI0, SGI136384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136387, SGI136388, SGI136388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29726[] = {SGI136389, SGI136390, SGI136391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136394, SGI0, SGI0, SGI0, SGI136395, SGI136395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29727[] = {SGI136392, SGI0, SGI136393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136396, SGI136397, SGI136397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29728[] = {SGI136398, SGI136399, SGI136401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136403, SGI136403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29729[] = {SGI136400, SGI0, SGI136402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136404, SGI136405, SGI136405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29730[] = {SGI136406, SGI136407, SGI136408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136409, SGI0, SGI0, SGI0, SGI136410, SGI136410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29731[] = {SGI136411, SGI0, SGI136412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136414, SGI136416, SGI136416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29732[] = {SGI136413, SGI136415, SGI136417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136418, SGI0, SGI0, SGI0, SGI136419, SGI136419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29733[] = {SGI136420, SGI0, SGI136421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136425, SGI136426, SGI136426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29734[] = {SGI136422, SGI136423, SGI136424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136427, SGI0, SGI0, SGI0, SGI136428, SGI136428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29735[] = {SGI136429, SGI0, SGI136430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136434, SGI136435, SGI136435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29736[] = {SGI136431, SGI136432, SGI136433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136436, SGI0, SGI0, SGI0, SGI136437, SGI136437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29737[] = {SGI136438, SGI136439, SGI136440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136441, SGI0, SGI0, SGI0, SGI136442, SGI136442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29738[] = {SGI136443, SGI0, SGI136444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136445, SGI136446, SGI136446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29739[] = {SGI136447, SGI136448, SGI136449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136450, SGI136450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29740[] = {SGI136451, SGI0, SGI136452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136453, SGI136454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29741[] = {SGI136455, SGI136456, SGI136457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136458, SGI0, SGI0, SGI0, SGI136459, SGI136459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29742[] = {SGI136460, SGI0, SGI136461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136462, SGI136463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29743[] = {SGI136464, SGI136465, SGI136466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136467, SGI136467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29744[] = {SGI136468, SGI136469, SGI136470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136471, SGI0, SGI0, SGI0, SGI136472, SGI136472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29745[] = {SGI136473, SGI136474, SGI136475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136476, SGI0, SGI0, SGI0, SGI136477, SGI136477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29746[] = {SGI136478, SGI136479, SGI136480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136481, SGI136481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29747[] = {SGI136482, SGI136483, SGI136484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136485, SGI0, SGI0, SGI0, SGI136486, SGI136486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29748[] = {SGI136487, SGI136488, SGI136489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136490, SGI136490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29749[] = {SGI136491, SGI0, SGI136492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136493, SGI0, SGI0, SGI0, SGI136494, SGI136494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29750[] = {SGI136495, SGI136496, SGI136497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136498, SGI0, SGI0, SGI0, SGI136499, SGI136499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29751[] = {SGI136500, SGI0, SGI136501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29752[] = {SGI136502, SGI136503, SGI136504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136505, SGI0, SGI0, SGI0, SGI136506, SGI136506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29753[] = {SGI136507, SGI136508, SGI136509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136510, SGI0, SGI0, SGI0, SGI136511, SGI136511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29754[] = {SGI136512, SGI136513, SGI136514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136515, SGI0, SGI0, SGI0, SGI136516, SGI136516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29755[] = {SGI136517, SGI0, SGI136518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29756[] = {SGI136519, SGI136520, SGI136521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136522, SGI0, SGI0, SGI0, SGI136523, SGI136523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29757[] = {SGI136524, SGI136525, SGI136526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136527, SGI0, SGI0, SGI0, SGI136528, SGI136528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29758[] = {SGI136529, SGI136530, SGI136531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136532, SGI0, SGI0, SGI0, SGI136533, SGI136533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29759[] = {SGI136534, SGI136535, SGI136536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136537, SGI0, SGI0, SGI0, SGI136538, SGI136538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29760[] = {SGI136539, SGI136540, SGI136541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136542, SGI0, SGI0, SGI0, SGI136543, SGI136543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29761[] = {SGI136544, SGI136545, SGI136546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136547, SGI0, SGI0, SGI0, SGI136548, SGI136548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29762[] = {SGI136549, SGI136550, SGI136551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136552, SGI0, SGI0, SGI0, SGI136553, SGI136553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29763[] = {SGI136554, SGI136555, SGI136556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136557, SGI0, SGI0, SGI0, SGI136558, SGI136558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29764[] = {SGI136559, SGI136560, SGI136561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136562, SGI0, SGI0, SGI0, SGI136563, SGI136563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29765[] = {SGI136564, SGI136565, SGI136566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136567, SGI0, SGI0, SGI0, SGI136568, SGI136568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29766[] = {SGI136569, SGI136570, SGI136571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136572, SGI0, SGI0, SGI0, SGI136573, SGI136573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29767[] = {SGI136574, SGI136575, SGI136576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136577, SGI0, SGI0, SGI0, SGI136578, SGI136578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29768[] = {SGI136579, SGI136580, SGI136581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136582, SGI0, SGI0, SGI0, SGI136583, SGI136583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29769[] = {SGI136584, SGI136585, SGI136586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136587, SGI0, SGI0, SGI0, SGI136588, SGI136588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29770[] = {SGI136589, SGI136590, SGI136591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136592, SGI0, SGI0, SGI0, SGI136593, SGI136593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29771[] = {SGI136594, SGI136595, SGI136596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136597, SGI136597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29772[] = {SGI136598, SGI136599, SGI136600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136601, SGI0, SGI0, SGI0, SGI136602, SGI136602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29773[] = {SGI136603, SGI136604, SGI136605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136606, SGI0, SGI0, SGI0, SGI136607, SGI136607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29774[] = {SGI136608, SGI136609, SGI136610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136611, SGI0, SGI0, SGI0, SGI136612, SGI136612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29775[] = {SGI136613, SGI136614, SGI136615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136616, SGI0, SGI0, SGI0, SGI136617, SGI136617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29776[] = {SGI136618, SGI136619, SGI136620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136621, SGI0, SGI0, SGI0, SGI136622, SGI136622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29777[] = {SGI136623, SGI136624, SGI136625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136626, SGI0, SGI0, SGI0, SGI136627, SGI136627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29778[] = {SGI136628, SGI0, SGI136629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29779[] = {SGI136630, SGI0, SGI136631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29780[] = {SGI136632, SGI136633, SGI136634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136635, SGI0, SGI0, SGI0, SGI136636, SGI136636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29781[] = {SGI136637, SGI0, SGI136638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136639, SGI0, SGI0, SGI0, SGI136640, SGI136640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29782[] = {SGI136641, SGI136642, SGI136643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136644, SGI0, SGI0, SGI0, SGI136645, SGI136645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29783[] = {SGI136646, SGI136647, SGI136648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136649, SGI0, SGI0, SGI0, SGI136650, SGI136650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29784[] = {SGI136651, SGI136652, SGI136653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136654, SGI0, SGI0, SGI0, SGI136655, SGI136655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29785[] = {SGI136656, SGI136657, SGI136658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136659, SGI0, SGI0, SGI0, SGI136660, SGI136660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29786[] = {SGI136661, SGI0, SGI136662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136663, SGI0, SGI0, SGI0, SGI136664, SGI136664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29787[] = {SGI136665, SGI136666, SGI136667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136668, SGI0, SGI0, SGI0, SGI136669, SGI136669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29788[] = {SGI136670, SGI136671, SGI136672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136673, SGI0, SGI0, SGI0, SGI136674, SGI136674, SGI0, SGI0, NULL};
const Segment * const SG29789[] = {SGI136675, SGI0, SGI136676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29790[] = {SGI136677, SGI136678, SGI136679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136680, SGI0, SGI0, SGI0, SGI136681, SGI136681, SGI0, SGI0, NULL};
const Segment * const SG29791[] = {SGI136682, SGI136683, SGI136684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136685, SGI0, SGI0, SGI0, SGI136686, SGI136686, SGI0, SGI0, NULL};
const Segment * const SG29792[] = {SGI136687, SGI136688, SGI136689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136690, SGI0, SGI0, SGI0, SGI136691, SGI136691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29793[] = {SGI136692, SGI0, SGI136693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29794[] = {SGI136694, SGI136695, SGI136696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136697, SGI136697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29795[] = {SGI136698, SGI136699, SGI136700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136701, SGI0, SGI0, SGI0, SGI136702, SGI136702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29796[] = {SGI136703, SGI136704, SGI136705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136706, SGI0, SGI0, SGI0, SGI136707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29797[] = {SGI136708, SGI136709, SGI136710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29798[] = {SGI136711, SGI136712, SGI136713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136714, SGI136714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29799[] = {SGI136715, SGI136716, SGI136717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136718, SGI0, SGI0, SGI0, SGI136719, SGI136720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29800[] = {SGI136721, SGI136722, SGI136723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136724, SGI0, SGI0, SGI0, SGI136725, SGI136725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29801[] = {SGI136726, SGI0, SGI136727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29802[] = {SGI136728, SGI136729, SGI136730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136731, SGI0, SGI0, SGI0, SGI136732, SGI136732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29803[] = {SGI136733, SGI0, SGI136734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29804[] = {SGI136735, SGI136736, SGI136737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136738, SGI0, SGI0, SGI0, SGI136739, SGI136739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29805[] = {SGI136740, SGI136741, SGI136742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136743, SGI0, SGI0, SGI0, SGI136744, SGI136744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29806[] = {SGI136745, SGI136746, SGI136747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136748, SGI0, SGI0, SGI0, SGI136749, SGI136749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29807[] = {SGI136750, SGI136751, SGI136752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136753, SGI0, SGI0, SGI0, SGI136754, SGI136754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29808[] = {SGI136755, SGI136756, SGI136757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136758, SGI0, SGI0, SGI0, SGI136759, SGI136759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29809[] = {SGI136760, SGI136761, SGI136762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136763, SGI0, SGI0, SGI0, SGI136764, SGI136764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29810[] = {SGI136765, SGI136766, SGI136767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136768, SGI0, SGI0, SGI0, SGI136769, SGI136769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29811[] = {SGI136770, SGI136771, SGI136772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136773, SGI0, SGI0, SGI0, SGI136774, SGI136774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29812[] = {SGI136775, SGI136776, SGI136777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136778, SGI0, SGI0, SGI0, SGI136779, SGI136779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29813[] = {SGI136780, SGI136781, SGI136782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136783, SGI0, SGI0, SGI0, SGI136784, SGI136784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29814[] = {SGI136785, SGI136786, SGI136787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136788, SGI0, SGI0, SGI0, SGI136789, SGI136789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29815[] = {SGI136790, SGI136791, SGI136792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136793, SGI0, SGI0, SGI0, SGI136794, SGI136794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29816[] = {SGI136795, SGI136796, SGI136797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136798, SGI0, SGI0, SGI0, SGI136799, SGI136799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29817[] = {SGI136800, SGI136801, SGI136802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136803, SGI0, SGI0, SGI0, SGI136804, SGI136804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29818[] = {SGI136805, SGI136806, SGI136807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136808, SGI0, SGI0, SGI0, SGI136809, SGI136809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29819[] = {SGI136810, SGI136811, SGI136812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136813, SGI0, SGI0, SGI0, SGI136814, SGI136814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29820[] = {SGI136815, SGI0, SGI136816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29821[] = {SGI136817, SGI136818, SGI136819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136820, SGI0, SGI0, SGI0, SGI136821, SGI136821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29822[] = {SGI136822, SGI136823, SGI136824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136825, SGI0, SGI0, SGI0, SGI136826, SGI136826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29823[] = {SGI136827, SGI136828, SGI136829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136830, SGI0, SGI0, SGI0, SGI136831, SGI136831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29824[] = {SGI136832, SGI136833, SGI136834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136835, SGI0, SGI0, SGI0, SGI136836, SGI136836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29825[] = {SGI136837, SGI136838, SGI136839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136840, SGI0, SGI0, SGI0, SGI136841, SGI136841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29826[] = {SGI136842, SGI136843, SGI136844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136845, SGI0, SGI0, SGI0, SGI136846, SGI136846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29827[] = {SGI136847, SGI136848, SGI136849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136850, SGI0, SGI0, SGI0, SGI136851, SGI136851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29828[] = {SGI136852, SGI136853, SGI136854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136855, SGI0, SGI0, SGI0, SGI136856, SGI136856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29829[] = {SGI136857, SGI136858, SGI136859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136860, SGI0, SGI0, SGI0, SGI136861, SGI136861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29830[] = {SGI136862, SGI136863, SGI136864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136865, SGI0, SGI0, SGI0, SGI136866, SGI136866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29831[] = {SGI136867, SGI0, SGI136868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29832[] = {SGI136869, SGI136870, SGI136871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136872, SGI0, SGI0, SGI0, SGI136873, SGI136873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29833[] = {SGI136874, SGI136875, SGI136876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136877, SGI0, SGI0, SGI0, SGI136878, SGI136878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29834[] = {SGI136879, SGI136880, SGI136881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136882, SGI0, SGI0, SGI0, SGI136883, SGI136883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29835[] = {SGI136884, SGI0, SGI136885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29836[] = {SGI136886, SGI136887, SGI136888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136889, SGI0, SGI0, SGI0, SGI136890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29837[] = {SGI136891, SGI136892, SGI136893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136894, SGI0, SGI0, SGI0, SGI136895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29838[] = {SGI136896, SGI136897, SGI136898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136899, SGI0, SGI0, SGI0, SGI136900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29839[] = {SGI136901, SGI136902, SGI136903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136904, SGI0, SGI0, SGI0, SGI136905, SGI136905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29840[] = {SGI136906, SGI136907, SGI136908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136909, SGI0, SGI0, SGI0, SGI136910, SGI136910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29841[] = {SGI136911, SGI136912, SGI136913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136914, SGI0, SGI0, SGI0, SGI136915, SGI136915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29842[] = {SGI136916, SGI136917, SGI136918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136919, SGI136919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29843[] = {SGI136920, SGI0, SGI136921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136922, SGI0, SGI0, SGI0, SGI136923, SGI136923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29844[] = {SGI136924, SGI0, SGI136925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29845[] = {SGI136926, SGI0, SGI136927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29846[] = {SGI136928, SGI136929, SGI136930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136931, SGI0, SGI0, SGI0, SGI136932, SGI136932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29847[] = {SGI136933, SGI136934, SGI136935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136936, SGI0, SGI0, SGI0, SGI136937, SGI136937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29848[] = {SGI136938, SGI0, SGI136939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29849[] = {SGI136940, SGI136941, SGI136942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136943, SGI0, SGI0, SGI0, SGI136944, SGI136944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29850[] = {SGI136945, SGI0, SGI136946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29851[] = {SGI136947, SGI136948, SGI136949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136950, SGI0, SGI0, SGI0, SGI136951, SGI136951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29852[] = {SGI136952, SGI136953, SGI136954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136955, SGI0, SGI0, SGI0, SGI136956, SGI136956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29853[] = {SGI136957, SGI136958, SGI136959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136960, SGI0, SGI0, SGI0, SGI136961, SGI136961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29854[] = {SGI136962, SGI136963, SGI136964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136965, SGI0, SGI0, SGI0, SGI136966, SGI136966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29855[] = {SGI136967, SGI136968, SGI136969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136970, SGI0, SGI0, SGI0, SGI136971, SGI136971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29856[] = {SGI136972, SGI0, SGI136973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29857[] = {SGI136974, SGI136975, SGI136976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136977, SGI0, SGI0, SGI0, SGI136978, SGI136978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29858[] = {SGI136979, SGI136980, SGI136981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136982, SGI0, SGI0, SGI0, SGI136983, SGI136983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29859[] = {SGI136984, SGI136986, SGI136987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136989, SGI0, SGI0, SGI0, SGI136990, SGI136990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29860[] = {SGI136985, SGI0, SGI136988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136991, SGI136992, SGI136992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29861[] = {SGI136993, SGI136995, SGI136996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136998, SGI0, SGI0, SGI0, SGI137000, SGI137000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29862[] = {SGI136994, SGI0, SGI136997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI136999, SGI137001, SGI137001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29863[] = {SGI137002, SGI0, SGI137003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29864[] = {SGI137004, SGI0, SGI137005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137006, SGI137007, SGI137007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29865[] = {SGI137008, SGI0, SGI137009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137010, SGI137011, SGI137011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29866[] = {SGI137012, SGI0, SGI137013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137014, SGI137015, SGI137015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29867[] = {SGI137016, SGI0, SGI137017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137018, SGI137019, SGI137019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29868[] = {SGI137020, SGI0, SGI137021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137022, SGI137023, SGI137023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29869[] = {SGI137024, SGI0, SGI137025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137026, SGI137027, SGI137027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29870[] = {SGI137028, SGI0, SGI137029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137030, SGI137031, SGI137031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29871[] = {SGI137032, SGI0, SGI137033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137034, SGI137035, SGI137035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29872[] = {SGI137036, SGI0, SGI137037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137038, SGI137039, SGI137039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29873[] = {SGI137040, SGI0, SGI137041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137042, SGI137043, SGI137043, SGI0, SGI0, NULL};
const Segment * const SG29874[] = {SGI137044, SGI137045, SGI137046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137047, SGI0, SGI0, SGI0, SGI137048, SGI137048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29875[] = {SGI137049, SGI137050, SGI137051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137052, SGI0, SGI0, SGI0, SGI137053, SGI137053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29876[] = {SGI137054, SGI137055, SGI137056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137057, SGI0, SGI0, SGI0, SGI137058, SGI137058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29877[] = {SGI137059, SGI137060, SGI137061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137062, SGI0, SGI0, SGI0, SGI137063, SGI137063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29878[] = {SGI137064, SGI137065, SGI137066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137067, SGI0, SGI0, SGI0, SGI137068, SGI137068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29879[] = {SGI137069, SGI137070, SGI137071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137072, SGI0, SGI0, SGI0, SGI137073, SGI137073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29880[] = {SGI137074, SGI137075, SGI137076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137077, SGI0, SGI0, SGI0, SGI137078, SGI137078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29881[] = {SGI137079, SGI137080, SGI137081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137082, SGI0, SGI0, SGI0, SGI137083, SGI137083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29882[] = {SGI137084, SGI137085, SGI137086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137087, SGI0, SGI0, SGI0, SGI137088, SGI137088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29883[] = {SGI137089, SGI137090, SGI137091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137092, SGI0, SGI0, SGI0, SGI137093, SGI137093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29884[] = {SGI137094, SGI137095, SGI137096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137097, SGI0, SGI0, SGI0, SGI137098, SGI137098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29885[] = {SGI137099, SGI137100, SGI137101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137102, SGI0, SGI0, SGI0, SGI137103, SGI137103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29886[] = {SGI137104, SGI137105, SGI137106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137107, SGI0, SGI0, SGI0, SGI137108, SGI137108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29887[] = {SGI137109, SGI0, SGI137110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29888[] = {SGI137111, SGI137112, SGI137113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137114, SGI0, SGI0, SGI0, SGI137115, SGI137115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29889[] = {SGI137116, SGI0, SGI137117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29890[] = {SGI137118, SGI137119, SGI137120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137121, SGI0, SGI0, SGI0, SGI137122, SGI137122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29891[] = {SGI137123, SGI137124, SGI137125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137126, SGI0, SGI0, SGI0, SGI137127, SGI137127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29892[] = {SGI137128, SGI0, SGI137129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29893[] = {SGI137130, SGI137131, SGI137132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137133, SGI0, SGI0, SGI0, SGI137134, SGI137134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29894[] = {SGI137135, SGI137136, SGI137137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137138, SGI0, SGI0, SGI0, SGI137139, SGI137139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29895[] = {SGI137140, SGI0, SGI137141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137142, SGI0, SGI0, SGI0, SGI137143, SGI137143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29896[] = {SGI137144, SGI0, SGI137145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137146, SGI0, SGI0, SGI0, SGI137147, SGI137147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29897[] = {SGI137148, SGI137149, SGI137150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137151, SGI0, SGI0, SGI0, SGI137152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29898[] = {SGI137153, SGI137154, SGI137155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137156, SGI0, SGI0, SGI0, SGI137157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29899[] = {SGI137159, SGI137160, SGI137161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137162, SGI0, SGI0, SGI0, SGI137163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29900[] = {SGI137164, SGI137165, SGI137166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137167, SGI0, SGI0, SGI0, SGI137168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29901[] = {SGI137170, SGI137171, SGI137172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137173, SGI0, SGI0, SGI0, SGI137174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29902[] = {SGI137176, SGI0, SGI137177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29903[] = {SGI137178, SGI137179, SGI137180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137181, SGI0, SGI0, SGI0, SGI137182, SGI137182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29904[] = {SGI137184, SGI137185, SGI137186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137187, SGI0, SGI0, SGI0, SGI137188, SGI137188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29905[] = {SGI137190, SGI0, SGI137191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29906[] = {SGI137192, SGI137193, SGI137194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137195, SGI0, SGI0, SGI0, SGI137196, SGI137196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29907[] = {SGI137197, SGI137198, SGI137199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137200, SGI0, SGI0, SGI0, SGI137201, SGI137201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29908[] = {SGI137202, SGI137203, SGI137204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137205, SGI0, SGI0, SGI0, SGI137206, SGI137206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29909[] = {SGI137207, SGI137208, SGI137209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137210, SGI0, SGI0, SGI0, SGI137211, SGI137211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29910[] = {SGI137212, SGI137213, SGI137214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137215, SGI0, SGI0, SGI0, SGI137216, SGI137216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29911[] = {SGI137217, SGI137218, SGI137219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137220, SGI0, SGI0, SGI0, SGI137221, SGI137221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29912[] = {SGI137222, SGI137223, SGI137224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137225, SGI0, SGI0, SGI0, SGI137226, SGI137226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29913[] = {SGI137227, SGI137228, SGI137229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137230, SGI0, SGI0, SGI0, SGI137231, SGI137231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29914[] = {SGI137232, SGI137233, SGI137234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137235, SGI0, SGI0, SGI0, SGI137236, SGI137236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29915[] = {SGI137237, SGI137238, SGI137239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137240, SGI0, SGI0, SGI0, SGI137241, SGI137241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29916[] = {SGI137242, SGI137243, SGI137244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137245, SGI0, SGI0, SGI0, SGI137246, SGI137246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29917[] = {SGI137247, SGI137248, SGI137249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137250, SGI0, SGI0, SGI0, SGI137251, SGI137251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29918[] = {SGI137252, SGI137253, SGI137254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137255, SGI0, SGI0, SGI0, SGI137256, SGI137256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29919[] = {SGI137257, SGI137258, SGI137259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137260, SGI0, SGI0, SGI0, SGI137261, SGI137261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29920[] = {SGI137262, SGI137263, SGI137264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137265, SGI0, SGI0, SGI0, SGI137266, SGI137266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29921[] = {SGI137267, SGI137268, SGI137269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137270, SGI0, SGI0, SGI0, SGI137271, SGI137271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29922[] = {SGI137272, SGI137273, SGI137274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137275, SGI0, SGI0, SGI0, SGI137276, SGI137276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29923[] = {SGI137277, SGI137278, SGI137279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137280, SGI0, SGI0, SGI0, SGI137281, SGI137281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29924[] = {SGI137282, SGI137283, SGI137284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137285, SGI0, SGI0, SGI0, SGI137286, SGI137286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29925[] = {SGI137287, SGI137288, SGI137289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137290, SGI0, SGI0, SGI0, SGI137291, SGI137291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29926[] = {SGI137292, SGI137293, SGI137294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137295, SGI0, SGI0, SGI0, SGI137296, SGI137296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29927[] = {SGI137297, SGI137298, SGI137299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137300, SGI0, SGI0, SGI0, SGI137301, SGI137301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29928[] = {SGI137302, SGI137303, SGI137304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137305, SGI0, SGI0, SGI0, SGI137306, SGI137306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29929[] = {SGI137307, SGI137308, SGI137309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137310, SGI0, SGI0, SGI0, SGI137311, SGI137311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29930[] = {SGI137312, SGI137313, SGI137314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137315, SGI0, SGI0, SGI0, SGI137316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29931[] = {SGI137317, SGI0, SGI137318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29932[] = {SGI137319, SGI0, SGI137320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29933[] = {SGI137321, SGI137322, SGI137323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137324, SGI0, SGI0, SGI0, SGI137325, SGI137325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29934[] = {SGI137326, SGI137327, SGI137328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137329, SGI0, SGI0, SGI0, SGI137330, SGI137330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29935[] = {SGI137331, SGI137332, SGI137333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137334, SGI0, SGI0, SGI0, SGI137335, SGI137335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29936[] = {SGI137336, SGI137337, SGI137338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137342, SGI0, SGI0, SGI0, SGI137343, SGI137343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29937[] = {SGI137339, SGI137340, SGI137341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137344, SGI0, SGI0, SGI0, SGI137345, SGI137345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29938[] = {SGI137349, SGI0, SGI137350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29939[] = {SGI137346, SGI137347, SGI137348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137351, SGI0, SGI0, SGI0, SGI137352, SGI137352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29940[] = {SGI137353, SGI137354, SGI137355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137356, SGI0, SGI0, SGI0, SGI137357, SGI137357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29941[] = {SGI137358, SGI137359, SGI137360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137361, SGI0, SGI0, SGI0, SGI137362, SGI137362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29942[] = {SGI137363, SGI137364, SGI137365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137366, SGI137366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29943[] = {SGI137367, SGI0, SGI137368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29944[] = {SGI137369, SGI137370, SGI137371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137372, SGI0, SGI0, SGI0, SGI137373, SGI137373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29945[] = {SGI137374, SGI137375, SGI137376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137377, SGI0, SGI0, SGI0, SGI137378, SGI137378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29946[] = {SGI137379, SGI137380, SGI137381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137382, SGI0, SGI0, SGI0, SGI137383, SGI137383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29947[] = {SGI137384, SGI137385, SGI137386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137387, SGI0, SGI0, SGI0, SGI137388, SGI137388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29948[] = {SGI137389, SGI137390, SGI137391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137392, SGI0, SGI0, SGI0, SGI137393, SGI137393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29949[] = {SGI137394, SGI137395, SGI137396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137397, SGI0, SGI0, SGI0, SGI137398, SGI137398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29950[] = {SGI137399, SGI137400, SGI137401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137402, SGI0, SGI0, SGI0, SGI137403, SGI137403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29951[] = {SGI137404, SGI137405, SGI137406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137407, SGI0, SGI0, SGI0, SGI137408, SGI137408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29952[] = {SGI137409, SGI137410, SGI137411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137412, SGI0, SGI0, SGI0, SGI137413, SGI137413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29953[] = {SGI137414, SGI137415, SGI137416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137417, SGI0, SGI0, SGI0, SGI137418, SGI137418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29954[] = {SGI137419, SGI0, SGI137420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29955[] = {SGI137421, SGI137422, SGI137423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137424, SGI0, SGI0, SGI0, SGI137425, SGI137425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29956[] = {SGI137426, SGI137427, SGI137428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137429, SGI0, SGI0, SGI0, SGI137430, SGI137430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29957[] = {SGI137431, SGI137432, SGI137433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137434, SGI0, SGI0, SGI0, SGI137435, SGI137435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29958[] = {SGI137436, SGI0, SGI137437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29959[] = {SGI137438, SGI137439, SGI137440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137441, SGI0, SGI0, SGI0, SGI137442, SGI137442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29960[] = {SGI137443, SGI0, SGI137444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29961[] = {SGI137445, SGI137446, SGI137447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137448, SGI0, SGI0, SGI0, SGI137449, SGI137449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29962[] = {SGI137450, SGI0, SGI137451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29963[] = {SGI137452, SGI137453, SGI137454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137455, SGI0, SGI0, SGI0, SGI137456, SGI137456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29964[] = {SGI137457, SGI0, SGI137458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137459, SGI0, SGI0, SGI0, SGI137460, SGI137460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29965[] = {SGI137461, SGI137462, SGI137463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137467, SGI0, SGI0, SGI0, SGI137469, SGI137469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29966[] = {SGI137464, SGI137465, SGI137466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137468, SGI0, SGI0, SGI0, SGI137470, SGI137470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29967[] = {SGI137471, SGI137472, SGI137473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137474, SGI0, SGI0, SGI0, SGI137475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29968[] = {SGI137476, SGI137477, SGI137478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137479, SGI0, SGI0, SGI0, SGI137480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29969[] = {SGI137481, SGI137482, SGI137483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137484, SGI0, SGI0, SGI0, SGI137485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29970[] = {SGI137486, SGI137487, SGI137488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137489, SGI0, SGI0, SGI0, SGI137490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29971[] = {SGI137491, SGI137492, SGI137493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137494, SGI0, SGI0, SGI0, SGI137495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29972[] = {SGI137496, SGI137497, SGI137498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137499, SGI0, SGI0, SGI0, SGI137500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29973[] = {SGI137501, SGI137502, SGI137503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137504, SGI0, SGI0, SGI0, SGI137505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29974[] = {SGI137506, SGI137507, SGI137508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137509, SGI0, SGI0, SGI0, SGI137510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29975[] = {SGI137511, SGI137512, SGI137513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137514, SGI0, SGI0, SGI0, SGI137515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29976[] = {SGI137516, SGI137517, SGI137518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137519, SGI0, SGI0, SGI0, SGI137520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29977[] = {SGI137521, SGI137522, SGI137523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137524, SGI0, SGI0, SGI0, SGI137525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29978[] = {SGI137526, SGI137527, SGI137528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137529, SGI0, SGI0, SGI0, SGI137530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29979[] = {SGI137531, SGI137532, SGI137533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137534, SGI0, SGI0, SGI0, SGI137535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29980[] = {SGI137536, SGI137537, SGI137538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137539, SGI0, SGI0, SGI0, SGI137540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29981[] = {SGI137541, SGI0, SGI137542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29982[] = {SGI137543, SGI137544, SGI137545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137546, SGI0, SGI0, SGI0, SGI137547, SGI137547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29983[] = {SGI137548, SGI137549, SGI137550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137551, SGI0, SGI0, SGI0, SGI137552, SGI137552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29984[] = {SGI137553, SGI137554, SGI137555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137556, SGI0, SGI0, SGI0, SGI137557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29985[] = {SGI137558, SGI0, SGI137559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29986[] = {SGI137560, SGI137561, SGI137562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137563, SGI0, SGI0, SGI0, SGI137564, SGI137564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29987[] = {SGI137565, SGI137566, SGI137567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137568, SGI0, SGI0, SGI0, SGI137569, SGI137569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29988[] = {SGI137570, SGI137571, SGI137572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137573, SGI0, SGI0, SGI0, SGI137574, SGI137574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29989[] = {SGI137575, SGI137576, SGI137577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137578, SGI0, SGI0, SGI0, SGI137579, SGI137579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29990[] = {SGI137580, SGI137581, SGI137582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137583, SGI0, SGI0, SGI0, SGI137584, SGI137584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29991[] = {SGI137585, SGI137586, SGI137587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137588, SGI0, SGI0, SGI0, SGI137589, SGI137589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29992[] = {SGI137590, SGI137591, SGI137592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137593, SGI0, SGI0, SGI0, SGI137594, SGI137594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29993[] = {SGI137595, SGI137596, SGI137597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137598, SGI0, SGI0, SGI0, SGI137599, SGI137599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29994[] = {SGI137600, SGI137601, SGI137602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137603, SGI0, SGI0, SGI0, SGI137604, SGI137604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29995[] = {SGI137605, SGI137606, SGI137607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137608, SGI0, SGI0, SGI0, SGI137609, SGI137609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29996[] = {SGI137610, SGI137611, SGI137612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137613, SGI0, SGI0, SGI0, SGI137614, SGI137614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29997[] = {SGI137615, SGI137616, SGI137617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137618, SGI0, SGI0, SGI0, SGI137619, SGI137619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29998[] = {SGI137620, SGI137621, SGI137622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137623, SGI0, SGI0, SGI0, SGI137624, SGI137624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG29999[] = {SGI137625, SGI137626, SGI137627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137628, SGI0, SGI0, SGI0, SGI137629, SGI137629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30000[] = {SGI137630, SGI137631, SGI137632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137633, SGI0, SGI0, SGI0, SGI137634, SGI137634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30001[] = {SGI137635, SGI137636, SGI137637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137638, SGI0, SGI0, SGI0, SGI137639, SGI137639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30002[] = {SGI137640, SGI137641, SGI137642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137643, SGI0, SGI0, SGI0, SGI137644, SGI137644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30003[] = {SGI137645, SGI0, SGI137646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30004[] = {SGI137647, SGI137648, SGI137649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137650, SGI0, SGI0, SGI0, SGI137651, SGI137651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30005[] = {SGI137652, SGI137653, SGI137654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137655, SGI0, SGI0, SGI0, SGI137656, SGI137656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30006[] = {SGI137657, SGI137658, SGI137659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137660, SGI137660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30007[] = {SGI137661, SGI137662, SGI137663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137664, SGI0, SGI0, SGI0, SGI137665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30008[] = {SGI137666, SGI137667, SGI137668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137669, SGI0, SGI0, SGI0, SGI137670, SGI137670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30009[] = {SGI137671, SGI137672, SGI137673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137674, SGI0, SGI0, SGI0, SGI137675, SGI137675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30010[] = {SGI137676, SGI137677, SGI137678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137679, SGI0, SGI0, SGI0, SGI137680, SGI137680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30011[] = {SGI137681, SGI137682, SGI137683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137684, SGI0, SGI0, SGI0, SGI137685, SGI137685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30012[] = {SGI137686, SGI137687, SGI137688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137689, SGI0, SGI0, SGI0, SGI137690, SGI137690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30013[] = {SGI137691, SGI137692, SGI137693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137694, SGI0, SGI0, SGI0, SGI137695, SGI137695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30014[] = {SGI137696, SGI137697, SGI137698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137699, SGI0, SGI0, SGI0, SGI137700, SGI137700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30015[] = {SGI137701, SGI137702, SGI137703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137704, SGI0, SGI0, SGI0, SGI137705, SGI137705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30016[] = {SGI137706, SGI137707, SGI137708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137709, SGI0, SGI0, SGI0, SGI137710, SGI137710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30017[] = {SGI137711, SGI0, SGI137712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137713, SGI0, SGI0, SGI0, SGI137714, SGI137714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30018[] = {SGI137715, SGI137716, SGI137717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137718, SGI0, SGI0, SGI0, SGI137719, SGI137719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30019[] = {SGI137720, SGI137721, SGI137722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137723, SGI0, SGI0, SGI0, SGI137724, SGI137724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30020[] = {SGI137725, SGI137726, SGI137727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137728, SGI0, SGI0, SGI0, SGI137729, SGI137729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30021[] = {SGI137730, SGI0, SGI137731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30022[] = {SGI137732, SGI137733, SGI137734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137735, SGI0, SGI0, SGI0, SGI137736, SGI137736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30023[] = {SGI137737, SGI0, SGI137738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30024[] = {SGI137739, SGI0, SGI137740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137741, SGI0, SGI0, SGI0, SGI137742, SGI137742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30025[] = {SGI137743, SGI137744, SGI137745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137746, SGI0, SGI0, SGI0, SGI137747, SGI137747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30026[] = {SGI137748, SGI137749, SGI137750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137751, SGI0, SGI0, SGI0, SGI137752, SGI137752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30027[] = {SGI137753, SGI0, SGI137754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137755, SGI137756, SGI137756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30028[] = {SGI137757, SGI0, SGI137758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137759, SGI137760, SGI137760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30029[] = {SGI137761, SGI0, SGI137762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137763, SGI137764, SGI137764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30030[] = {SGI137765, SGI0, SGI137766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137767, SGI137768, SGI137768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30031[] = {SGI137769, SGI0, SGI137770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137771, SGI137772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30032[] = {SGI137773, SGI0, SGI137774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137775, SGI137776, SGI137776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30033[] = {SGI137777, SGI137778, SGI137779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137780, SGI0, SGI0, SGI0, SGI137781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30034[] = {SGI137782, SGI137783, SGI137784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137785, SGI0, SGI0, SGI0, SGI137786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30035[] = {SGI137787, SGI0, SGI137788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137789, SGI0, SGI0, SGI0, SGI137790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30036[] = {SGI137791, SGI0, SGI137792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30037[] = {SGI137793, SGI137794, SGI137795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137796, SGI0, SGI0, SGI0, SGI137797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30038[] = {SGI137798, SGI137799, SGI137800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137801, SGI0, SGI0, SGI0, SGI137802, SGI137802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30039[] = {SGI137803, SGI0, SGI137804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137805, SGI137806, SGI137806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30040[] = {SGI137807, SGI137808, SGI137809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137810, SGI0, SGI0, SGI0, SGI137811, SGI137811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30041[] = {SGI137812, SGI0, SGI137813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137814, SGI137815, SGI137815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30042[] = {SGI137816, SGI137817, SGI137818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30043[] = {SGI137819, SGI0, SGI137820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137822, SGI137823, SGI137823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30044[] = {SGI137821, SGI137824, SGI137825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137826, SGI137826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30045[] = {SGI137827, SGI137828, SGI137829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137830, SGI137830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30046[] = {SGI137831, SGI0, SGI137832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137835, SGI137837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30047[] = {SGI137833, SGI137834, SGI137836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137838, SGI0, SGI0, SGI0, SGI137839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30048[] = {SGI137840, SGI137841, SGI137842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30049[] = {SGI137843, SGI0, SGI137844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137845, SGI137846, SGI137846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30050[] = {SGI137847, SGI137848, SGI137849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137850, SGI0, SGI0, SGI0, SGI137851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30051[] = {SGI137852, SGI137853, SGI137854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30052[] = {SGI137855, SGI0, SGI137856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137857, SGI137858, SGI137858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30053[] = {SGI137859, SGI137860, SGI137861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137862, SGI0, SGI0, SGI0, SGI137863, SGI137863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30054[] = {SGI137864, SGI0, SGI137865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137869, SGI137870, SGI137870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30055[] = {SGI137866, SGI137867, SGI137868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137871, SGI0, SGI0, SGI0, SGI137872, SGI137872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30056[] = {SGI137875, SGI0, SGI137877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137879, SGI137880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30057[] = {SGI137873, SGI137874, SGI137876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137878, SGI0, SGI0, SGI0, SGI137881, SGI137881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30058[] = {SGI137882, SGI137883, SGI137884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137885, SGI0, SGI0, SGI0, SGI137886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30059[] = {SGI137888, SGI0, SGI137890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30060[] = {SGI137887, SGI0, SGI137889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137891, SGI137892, SGI137892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30061[] = {SGI137893, SGI137894, SGI137895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137896, SGI0, SGI0, SGI0, SGI137899, SGI137899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30062[] = {SGI137898, SGI0, SGI137901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137903, SGI137905, SGI137905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30063[] = {SGI137897, SGI0, SGI137900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137902, SGI0, SGI0, SGI0, SGI137904, SGI137904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30064[] = {SGI137906, SGI0, SGI137907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137911, SGI0, SGI0, SGI0, SGI137914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30065[] = {SGI137908, SGI0, SGI137910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137915, SGI137917, SGI137917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30066[] = {SGI137909, SGI137912, SGI137913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137916, SGI0, SGI0, SGI0, SGI137918, SGI137918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30067[] = {SGI137919, SGI0, SGI137920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137923, SGI137925, SGI137925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30068[] = {SGI137921, SGI137922, SGI137924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30069[] = {SGI137927, SGI137928, SGI137929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137930, SGI137930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30070[] = {SGI137931, SGI0, SGI137932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30071[] = {SGI137933, SGI137934, SGI137935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137938, SGI0, SGI0, SGI0, SGI137940, SGI137940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30072[] = {SGI137936, SGI137937, SGI137939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137941, SGI137941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30073[] = {SGI137942, SGI137943, SGI137944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30074[] = {SGI137945, SGI137946, SGI137947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137949, SGI0, SGI0, SGI0, SGI137950, SGI137950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30075[] = {SGI137951, SGI137952, SGI137953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137954, SGI0, SGI0, SGI0, SGI137955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30076[] = {SGI137956, SGI137957, SGI137958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137959, SGI0, SGI0, SGI0, SGI137960, SGI137960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30077[] = {SGI137961, SGI0, SGI137962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137963, SGI0, SGI0, SGI0, SGI137964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30078[] = {SGI137965, SGI0, SGI137966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137967, SGI0, SGI0, SGI0, SGI137968, SGI137968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30079[] = {SGI137969, SGI137970, SGI137971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137972, SGI0, SGI0, SGI0, SGI137973, SGI137973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30080[] = {SGI0, SGI137974, SGI137975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30081[] = {SGI137976, SGI137977, SGI137978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137979, SGI0, SGI0, SGI0, SGI137980, SGI137980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30082[] = {SGI137981, SGI137982, SGI137983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137984, SGI0, SGI0, SGI0, SGI137985, SGI137986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30083[] = {SGI137987, SGI0, SGI137988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137989, SGI137990, SGI137990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30084[] = {SGI137991, SGI0, SGI137992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137993, SGI137994, SGI137994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30085[] = {SGI137995, SGI0, SGI137996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI137997, SGI137998, SGI137998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30086[] = {SGI137999, SGI0, SGI138000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138001, SGI138002, SGI138002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30087[] = {SGI138003, SGI0, SGI138004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138005, SGI138006, SGI138006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30088[] = {SGI138007, SGI0, SGI138008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138009, SGI138010, SGI138010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30089[] = {SGI138011, SGI0, SGI138012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138013, SGI138014, SGI138014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30090[] = {SGI138015, SGI138016, SGI138017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138018, SGI0, SGI0, SGI0, SGI138019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30091[] = {SGI138022, SGI0, SGI138023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30092[] = {SGI138020, SGI0, SGI138021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138024, SGI138025, SGI138025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30093[] = {SGI138026, SGI0, SGI138027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138028, SGI0, SGI0, SGI0, SGI138029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30094[] = {SGI138030, SGI138031, SGI138032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138033, SGI0, SGI0, SGI0, SGI138034, SGI138034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30095[] = {SGI138035, SGI0, SGI138036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138037, SGI138038, SGI138038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30096[] = {SGI138039, SGI0, SGI138040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138041, SGI138042, SGI138042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30097[] = {SGI138043, SGI0, SGI138044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138045, SGI138046, SGI138046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30098[] = {SGI138047, SGI0, SGI138048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138049, SGI138050, SGI138050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30099[] = {SGI138051, SGI0, SGI138052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138053, SGI138054, SGI138054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30100[] = {SGI138055, SGI138056, SGI138057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138058, SGI0, SGI0, SGI0, SGI138059, SGI138059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30101[] = {SGI138060, SGI138061, SGI138062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138063, SGI0, SGI0, SGI0, SGI138064, SGI138064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30102[] = {SGI138065, SGI0, SGI138066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30103[] = {SGI138067, SGI138068, SGI138069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138070, SGI0, SGI0, SGI0, SGI138071, SGI138071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30104[] = {SGI138072, SGI0, SGI138073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138074, SGI138075, SGI138075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30105[] = {SGI138076, SGI0, SGI138077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138078, SGI138079, SGI138079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30106[] = {SGI138080, SGI0, SGI138081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138082, SGI138083, SGI138083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30107[] = {SGI138084, SGI0, SGI138085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138086, SGI138087, SGI138087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30108[] = {SGI138088, SGI0, SGI138089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138090, SGI138091, SGI138091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30109[] = {SGI138092, SGI0, SGI138093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138094, SGI138095, SGI138095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30110[] = {SGI138096, SGI0, SGI138097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138098, SGI138099, SGI138099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30111[] = {SGI138100, SGI0, SGI138101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138102, SGI138103, SGI138103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30112[] = {SGI138104, SGI0, SGI138105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138106, SGI138107, SGI138107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30113[] = {SGI138108, SGI0, SGI138109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138110, SGI138111, SGI138111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30114[] = {SGI138112, SGI0, SGI138113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138114, SGI138115, SGI138115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30115[] = {SGI138116, SGI0, SGI138117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138118, SGI138119, SGI138119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30116[] = {SGI138120, SGI0, SGI138121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138122, SGI138123, SGI138123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30117[] = {SGI138124, SGI0, SGI138125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138126, SGI138127, SGI138127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30118[] = {SGI138128, SGI0, SGI138129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138130, SGI138131, SGI138131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30119[] = {SGI138132, SGI0, SGI138133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138134, SGI138135, SGI138135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30120[] = {SGI138136, SGI0, SGI138137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138138, SGI138139, SGI138139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30121[] = {SGI138140, SGI0, SGI138141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138142, SGI138143, SGI138143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30122[] = {SGI138144, SGI0, SGI138145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138146, SGI138147, SGI138147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30123[] = {SGI138148, SGI0, SGI138149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138150, SGI138151, SGI138151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30124[] = {SGI138152, SGI0, SGI138153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138154, SGI138155, SGI138155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30125[] = {SGI138156, SGI0, SGI138157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138158, SGI138159, SGI138159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30126[] = {SGI138160, SGI138161, SGI138162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138163, SGI0, SGI0, SGI0, SGI138164, SGI138164, SGI0, SGI0, NULL};
const Segment * const SG30127[] = {SGI138165, SGI138166, SGI138167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138168, SGI0, SGI0, SGI0, SGI138169, SGI138169, SGI0, SGI0, NULL};
const Segment * const SG30128[] = {SGI138170, SGI0, SGI138171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30129[] = {SGI138172, SGI138173, SGI138174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138175, SGI0, SGI0, SGI0, SGI138176, SGI138176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30130[] = {SGI138177, SGI138178, SGI138179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138180, SGI0, SGI0, SGI0, SGI138181, SGI138181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30131[] = {SGI138182, SGI138183, SGI138184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138185, SGI138185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30132[] = {SGI138186, SGI138187, SGI138188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138189, SGI0, SGI0, SGI0, SGI138190, SGI138190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30133[] = {SGI138191, SGI138192, SGI138193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138194, SGI0, SGI0, SGI0, SGI138195, SGI138195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30134[] = {SGI138196, SGI0, SGI138197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138198, SGI0, SGI0, SGI0, SGI138199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30135[] = {SGI138200, SGI0, SGI138201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138202, SGI0, SGI0, SGI0, SGI138203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30136[] = {SGI138204, SGI0, SGI138205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138206, SGI0, SGI0, SGI0, SGI138207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30137[] = {SGI138208, SGI138209, SGI138210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138211, SGI0, SGI0, SGI0, SGI138212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30138[] = {SGI138213, SGI0, SGI138214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138215, SGI0, SGI0, SGI0, SGI138216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30139[] = {SGI138217, SGI0, SGI138218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138219, SGI0, SGI0, SGI0, SGI138220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30140[] = {SGI138221, SGI0, SGI138222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138223, SGI0, SGI0, SGI0, SGI138224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30141[] = {SGI138225, SGI138226, SGI138227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138228, SGI0, SGI0, SGI0, SGI138229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30142[] = {SGI138230, SGI0, SGI138231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138232, SGI0, SGI0, SGI0, SGI138233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30143[] = {SGI138234, SGI0, SGI138235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138236, SGI0, SGI0, SGI0, SGI138237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30144[] = {SGI138238, SGI0, SGI138239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138240, SGI0, SGI0, SGI0, SGI138241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30145[] = {SGI138242, SGI0, SGI138243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138244, SGI0, SGI0, SGI0, SGI138245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30146[] = {SGI138246, SGI0, SGI138247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138248, SGI0, SGI0, SGI0, SGI138249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30147[] = {SGI138250, SGI0, SGI138251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138252, SGI0, SGI0, SGI0, SGI138253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30148[] = {SGI138254, SGI0, SGI138255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138256, SGI0, SGI0, SGI0, SGI138257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30149[] = {SGI138258, SGI0, SGI138259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138260, SGI0, SGI0, SGI0, SGI138261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30150[] = {SGI138262, SGI0, SGI138263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30151[] = {SGI138264, SGI0, SGI138265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30152[] = {SGI138266, SGI0, SGI138267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138268, SGI138269, SGI138269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30153[] = {SGI138270, SGI0, SGI138271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138272, SGI138273, SGI138273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30154[] = {SGI138274, SGI0, SGI138275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138276, SGI138277, SGI138277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30155[] = {SGI138278, SGI0, SGI138279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138280, SGI138281, SGI138281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30156[] = {SGI138282, SGI0, SGI138283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138284, SGI138285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30157[] = {SGI138286, SGI0, SGI138287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138288, SGI138289, SGI138289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30158[] = {SGI138290, SGI0, SGI138291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138292, SGI138293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30159[] = {SGI138294, SGI0, SGI138295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138296, SGI138297, SGI138297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30160[] = {SGI138298, SGI0, SGI138299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138300, SGI138301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30161[] = {SGI138302, SGI0, SGI138303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138304, SGI138305, SGI138305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30162[] = {SGI138306, SGI0, SGI138307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138308, SGI138309, SGI138309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30163[] = {SGI138310, SGI0, SGI138311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138312, SGI138313, SGI138313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30164[] = {SGI138314, SGI0, SGI138315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138316, SGI138317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30165[] = {SGI138318, SGI138319, SGI138320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138321, SGI0, SGI0, SGI0, SGI138322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30166[] = {SGI138323, SGI138324, SGI138325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138326, SGI0, SGI0, SGI0, SGI138327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30167[] = {SGI138328, SGI0, SGI138329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138330, SGI138331, SGI138331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30168[] = {SGI138332, SGI138333, SGI138334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138335, SGI0, SGI0, SGI0, SGI138336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30169[] = {SGI138337, SGI0, SGI138338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30170[] = {SGI138339, SGI0, SGI138340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138341, SGI138342, SGI138342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30171[] = {SGI138343, SGI0, SGI138344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138345, SGI138346, SGI138346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30172[] = {SGI138347, SGI0, SGI138348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138349, SGI138350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30173[] = {SGI138351, SGI0, SGI138352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138353, SGI138354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30174[] = {SGI138355, SGI0, SGI138357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138361, SGI138362, SGI138362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30175[] = {SGI138356, SGI138358, SGI138359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138360, SGI0, SGI0, SGI0, SGI138363, SGI138363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30176[] = {SGI138364, SGI0, SGI138365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138366, SGI138367, SGI138367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30177[] = {SGI138368, SGI0, SGI138369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138370, SGI138371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30178[] = {SGI138372, SGI138373, SGI138374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138375, SGI0, SGI0, SGI0, SGI138376, SGI138376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30179[] = {SGI138377, SGI0, SGI138378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30180[] = {SGI138379, SGI0, SGI138380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138381, SGI0, SGI0, SGI0, SGI138382, SGI138382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30181[] = {SGI138383, SGI138384, SGI138385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138386, SGI0, SGI0, SGI0, SGI138387, SGI138387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30182[] = {SGI138388, SGI138389, SGI138390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138391, SGI138391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30183[] = {SGI138392, SGI0, SGI138393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138394, SGI138395, SGI138395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30184[] = {SGI138396, SGI0, SGI138397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138398, SGI138399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30185[] = {SGI138400, SGI138401, SGI138402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138403, SGI0, SGI0, SGI0, SGI138404, SGI138404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30186[] = {SGI138405, SGI138406, SGI138407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138408, SGI0, SGI0, SGI0, SGI138409, SGI138409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30187[] = {SGI138410, SGI0, SGI138411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30188[] = {SGI138412, SGI138413, SGI138414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138415, SGI0, SGI0, SGI0, SGI138416, SGI138416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30189[] = {SGI138417, SGI0, SGI138418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138419, SGI138420, SGI138420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30190[] = {SGI138421, SGI0, SGI138422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138423, SGI138424, SGI138424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30191[] = {SGI138425, SGI138426, SGI138427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138428, SGI0, SGI0, SGI0, SGI138429, SGI138429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30192[] = {SGI138430, SGI0, SGI138431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138432, SGI138433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30193[] = {SGI138434, SGI0, SGI138435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138436, SGI138437, SGI138437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30194[] = {SGI138438, SGI0, SGI138439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138440, SGI138441, SGI138441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30195[] = {SGI138442, SGI0, SGI138443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138444, SGI138445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30196[] = {SGI138446, SGI0, SGI138447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138448, SGI138449, SGI138449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30197[] = {SGI138450, SGI0, SGI138451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138452, SGI138453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30198[] = {SGI138454, SGI0, SGI138455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138456, SGI138457, SGI138457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30199[] = {SGI138458, SGI0, SGI138459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138460, SGI138461, SGI138461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30200[] = {SGI138462, SGI138463, SGI138464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138465, SGI0, SGI0, SGI0, SGI138466, SGI138466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30201[] = {SGI138467, SGI138468, SGI138469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138470, SGI0, SGI0, SGI0, SGI138471, SGI138471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30202[] = {SGI138472, SGI0, SGI138473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30203[] = {SGI138474, SGI138475, SGI138476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138477, SGI0, SGI0, SGI0, SGI138478, SGI138478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30204[] = {SGI138479, SGI0, SGI138480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30205[] = {SGI138481, SGI0, SGI138482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30206[] = {SGI138483, SGI138484, SGI138485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138486, SGI0, SGI0, SGI0, SGI138487, SGI138487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30207[] = {SGI138488, SGI0, SGI138489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30208[] = {SGI138490, SGI138491, SGI138492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138493, SGI0, SGI0, SGI0, SGI138494, SGI138494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30209[] = {SGI138495, SGI138496, SGI138497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138498, SGI0, SGI0, SGI0, SGI138499, SGI138499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30210[] = {SGI138500, SGI0, SGI138501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138502, SGI138503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30211[] = {SGI138504, SGI138505, SGI138506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138507, SGI138507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30212[] = {SGI138508, SGI0, SGI138509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138510, SGI138511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30213[] = {SGI138512, SGI138513, SGI138514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138515, SGI0, SGI0, SGI0, SGI138516, SGI138516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30214[] = {SGI138517, SGI138518, SGI138519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138520, SGI0, SGI0, SGI0, SGI138521, SGI138521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30215[] = {SGI138522, SGI0, SGI138523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138524, SGI138525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30216[] = {SGI138526, SGI138527, SGI138528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138529, SGI0, SGI0, SGI0, SGI138530, SGI138530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30217[] = {SGI138531, SGI0, SGI138532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138533, SGI138534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30218[] = {SGI138535, SGI0, SGI138536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138537, SGI0, SGI0, SGI0, SGI138538, SGI138538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30219[] = {SGI138539, SGI0, SGI138540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138543, SGI138545, SGI138545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30220[] = {SGI138541, SGI138542, SGI138544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138546, SGI0, SGI0, SGI0, SGI138547, SGI138547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30221[] = {SGI138548, SGI0, SGI138549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138550, SGI138551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30222[] = {SGI138552, SGI0, SGI138553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138554, SGI138555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30223[] = {SGI138556, SGI0, SGI138557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138558, SGI138559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30224[] = {SGI138560, SGI0, SGI138561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138562, SGI138563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30225[] = {SGI138564, SGI0, SGI138565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138566, SGI138567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30226[] = {SGI138568, SGI0, SGI138569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138570, SGI138571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30227[] = {SGI138572, SGI0, SGI138573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138574, SGI138575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30228[] = {SGI138576, SGI0, SGI138577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138578, SGI138579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30229[] = {SGI138580, SGI0, SGI138581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138582, SGI138583, SGI138583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30230[] = {SGI138584, SGI0, SGI138585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138586, SGI138587, SGI138587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30231[] = {SGI138588, SGI0, SGI138589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138590, SGI138591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30232[] = {SGI138592, SGI0, SGI138593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138594, SGI138595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30233[] = {SGI138596, SGI0, SGI138597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138598, SGI138599, SGI138599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30234[] = {SGI138600, SGI0, SGI138601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138602, SGI138603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30235[] = {SGI138604, SGI0, SGI138605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138606, SGI138607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30236[] = {SGI138608, SGI0, SGI138609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138610, SGI138611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30237[] = {SGI138612, SGI0, SGI138613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138614, SGI138615, SGI138615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30238[] = {SGI138616, SGI0, SGI138617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138618, SGI138619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30239[] = {SGI138620, SGI0, SGI138621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138622, SGI138623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30240[] = {SGI138624, SGI0, SGI138625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138626, SGI138627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30241[] = {SGI138628, SGI0, SGI138629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138630, SGI138631, SGI138631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30242[] = {SGI138632, SGI0, SGI138633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138634, SGI138635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30243[] = {SGI138636, SGI0, SGI138637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138638, SGI138639, SGI138639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30244[] = {SGI138640, SGI0, SGI138641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138642, SGI138643, SGI138643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30245[] = {SGI138644, SGI0, SGI138645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138646, SGI138647, SGI138647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30246[] = {SGI138648, SGI0, SGI138649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138650, SGI138651, SGI138651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30247[] = {SGI138652, SGI0, SGI138653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138654, SGI138655, SGI138655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30248[] = {SGI138656, SGI0, SGI138657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138658, SGI138659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30249[] = {SGI138660, SGI0, SGI138661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138662, SGI138663, SGI138663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30250[] = {SGI138664, SGI0, SGI138665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138666, SGI138667, SGI138667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30251[] = {SGI138668, SGI0, SGI138669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138670, SGI138671, SGI138671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30252[] = {SGI138672, SGI0, SGI138673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138674, SGI138675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30253[] = {SGI138676, SGI0, SGI138677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138678, SGI138679, SGI138679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30254[] = {SGI138680, SGI0, SGI138681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138682, SGI138683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30255[] = {SGI138684, SGI0, SGI138685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138686, SGI138687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30256[] = {SGI138688, SGI0, SGI138689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138690, SGI138691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30257[] = {SGI138692, SGI0, SGI138693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138694, SGI138695, SGI138695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30258[] = {SGI138696, SGI0, SGI138697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138698, SGI138699, SGI138699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30259[] = {SGI138700, SGI0, SGI138701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138702, SGI138703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30260[] = {SGI138704, SGI0, SGI138705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138706, SGI138707, SGI138707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30261[] = {SGI138708, SGI0, SGI138709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138710, SGI138711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30262[] = {SGI138712, SGI0, SGI138713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138714, SGI138715, SGI138715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30263[] = {SGI138716, SGI0, SGI138717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138718, SGI138719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30264[] = {SGI138720, SGI0, SGI138721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138722, SGI138723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30265[] = {SGI138724, SGI0, SGI138725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138726, SGI138727, SGI138727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30266[] = {SGI138728, SGI0, SGI138729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138730, SGI138731, SGI138731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30267[] = {SGI138732, SGI0, SGI138733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138734, SGI138735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30268[] = {SGI138736, SGI0, SGI138737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138738, SGI138739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30269[] = {SGI138740, SGI0, SGI138741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138742, SGI138743, SGI138743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30270[] = {SGI138744, SGI0, SGI138745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138746, SGI138747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30271[] = {SGI138748, SGI0, SGI138749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138750, SGI138751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30272[] = {SGI138752, SGI0, SGI138753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138754, SGI138755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30273[] = {SGI138756, SGI0, SGI138757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138758, SGI138759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30274[] = {SGI138760, SGI0, SGI138761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138762, SGI138763, SGI138763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30275[] = {SGI138764, SGI0, SGI138765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138766, SGI138767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30276[] = {SGI138768, SGI0, SGI138769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138770, SGI138771, SGI138771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30277[] = {SGI138772, SGI0, SGI138773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138774, SGI138775, SGI138775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30278[] = {SGI138776, SGI0, SGI138777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138778, SGI138779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30279[] = {SGI138780, SGI0, SGI138781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138782, SGI138783, SGI138783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30280[] = {SGI138784, SGI0, SGI138785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138786, SGI138787, SGI138787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30281[] = {SGI138788, SGI0, SGI138789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138790, SGI138791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30282[] = {SGI138792, SGI0, SGI138793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138794, SGI138795, SGI138795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30283[] = {SGI138796, SGI0, SGI138797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138798, SGI138799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30284[] = {SGI138800, SGI0, SGI138801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138802, SGI138803, SGI138803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30285[] = {SGI138804, SGI0, SGI138805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138806, SGI138807, SGI138807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30286[] = {SGI138808, SGI0, SGI138809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138810, SGI138811, SGI138811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30287[] = {SGI138812, SGI0, SGI138813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138814, SGI138815, SGI138815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30288[] = {SGI138816, SGI0, SGI138817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138818, SGI138819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30289[] = {SGI138820, SGI0, SGI138821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138822, SGI138823, SGI138823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30290[] = {SGI138824, SGI0, SGI138825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138826, SGI138827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30291[] = {SGI138828, SGI0, SGI138829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138830, SGI138831, SGI138831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30292[] = {SGI138832, SGI0, SGI138833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138834, SGI138835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30293[] = {SGI138836, SGI0, SGI138837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138838, SGI138839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30294[] = {SGI138840, SGI0, SGI138841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138842, SGI138843, SGI138843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30295[] = {SGI138844, SGI0, SGI138845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138846, SGI138847, SGI138847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30296[] = {SGI138848, SGI0, SGI138849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138850, SGI138851, SGI138851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30297[] = {SGI138852, SGI0, SGI138853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138854, SGI138855, SGI138855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30298[] = {SGI138856, SGI0, SGI138857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138858, SGI138859, SGI138859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30299[] = {SGI138860, SGI0, SGI138861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138862, SGI138863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30300[] = {SGI138864, SGI0, SGI138865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138866, SGI138867, SGI138867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30301[] = {SGI138868, SGI0, SGI138869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138870, SGI138871, SGI138871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30302[] = {SGI138872, SGI0, SGI138873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138874, SGI138875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30303[] = {SGI138876, SGI0, SGI138877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138878, SGI138879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30304[] = {SGI138880, SGI0, SGI138881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138882, SGI138883, SGI138883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30305[] = {SGI138884, SGI0, SGI138885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138886, SGI138887, SGI138887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30306[] = {SGI138888, SGI138889, SGI138890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138893, SGI0, SGI0, SGI0, SGI138894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30307[] = {SGI138891, SGI0, SGI138892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138895, SGI138896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30308[] = {SGI138897, SGI0, SGI138898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138901, SGI138902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30309[] = {SGI138899, SGI0, SGI138900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30310[] = {SGI138903, SGI138905, SGI138906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138908, SGI0, SGI0, SGI0, SGI138910, SGI138910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30311[] = {SGI138904, SGI0, SGI138907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138909, SGI138911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30312[] = {SGI138912, SGI0, SGI138913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138914, SGI138915, SGI138915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30313[] = {SGI138916, SGI0, SGI138917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138918, SGI138919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30314[] = {SGI138920, SGI0, SGI138921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138922, SGI138923, SGI138923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30315[] = {SGI138924, SGI0, SGI138925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138926, SGI138927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30316[] = {SGI138928, SGI0, SGI138929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138930, SGI138931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30317[] = {SGI138932, SGI0, SGI138933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138934, SGI138935, SGI138935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30318[] = {SGI138936, SGI0, SGI138937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138938, SGI138939, SGI138939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30319[] = {SGI138940, SGI138941, SGI138942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138943, SGI0, SGI0, SGI0, SGI138944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30320[] = {SGI138945, SGI0, SGI138946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138947, SGI138948, SGI138948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30321[] = {SGI138949, SGI0, SGI138950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138951, SGI138952, SGI138952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30322[] = {SGI138954, SGI138956, SGI138957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138960, SGI138960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30323[] = {SGI138953, SGI0, SGI138955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138958, SGI138959, SGI138959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30324[] = {SGI138961, SGI0, SGI138962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138965, SGI138967, SGI138967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30325[] = {SGI138963, SGI0, SGI138964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138966, SGI0, SGI0, SGI0, SGI138968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30326[] = {SGI138969, SGI0, SGI138970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138971, SGI138972, SGI138972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30327[] = {SGI138973, SGI0, SGI138974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138975, SGI138976, SGI138976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30328[] = {SGI138977, SGI138978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138979, SGI0, SGI0, SGI0, SGI138980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30329[] = {SGI138981, SGI0, SGI138982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138983, SGI138984, SGI138984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30330[] = {SGI138985, SGI138986, SGI138987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138988, SGI138988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30331[] = {SGI138989, SGI0, SGI138990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138991, SGI138992, SGI138992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30332[] = {SGI138993, SGI138994, SGI138995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI138996, SGI0, SGI0, SGI0, SGI138997, SGI138997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30333[] = {SGI138998, SGI138999, SGI139000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30334[] = {SGI139001, SGI0, SGI139002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139003, SGI139004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30335[] = {SGI139005, SGI139006, SGI139007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139008, SGI0, SGI0, SGI0, SGI139010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30336[] = {SGI139009, SGI0, SGI139011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139012, SGI139013, SGI139013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30337[] = {SGI139014, SGI139015, SGI139016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139017, SGI0, SGI0, SGI0, SGI139018, SGI139019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30338[] = {SGI139020, SGI0, SGI139021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139022, SGI139023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30339[] = {SGI139024, SGI139025, SGI139026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139029, SGI0, SGI0, SGI0, SGI139030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30340[] = {SGI139027, SGI0, SGI139028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139031, SGI139032, SGI139032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30341[] = {SGI139033, SGI139034, SGI139035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139036, SGI0, SGI0, SGI0, SGI139037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30342[] = {SGI139038, SGI0, SGI139039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139040, SGI139041, SGI139041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30343[] = {SGI139042, SGI0, SGI139043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139044, SGI139045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30344[] = {SGI139046, SGI0, SGI139047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139048, SGI139049, SGI139049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30345[] = {SGI139050, SGI0, SGI139051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139052, SGI139053, SGI139053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30346[] = {SGI139054, SGI0, SGI139055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139056, SGI139057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30347[] = {SGI139058, SGI0, SGI139059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139060, SGI139061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30348[] = {SGI0, SGI0, SGI139062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139063, SGI139064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30349[] = {SGI139065, SGI0, SGI139066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139067, SGI139068, SGI139068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30350[] = {SGI139069, SGI0, SGI139070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139071, SGI139072, SGI139072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30351[] = {SGI139073, SGI0, SGI139074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139075, SGI139076, SGI139076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30352[] = {SGI139077, SGI0, SGI139078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139079, SGI139080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30353[] = {SGI139081, SGI0, SGI139082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139083, SGI139084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30354[] = {SGI139085, SGI0, SGI139086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139087, SGI139088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30355[] = {SGI139089, SGI0, SGI139090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139091, SGI139092, SGI139092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30356[] = {SGI139093, SGI0, SGI139094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139095, SGI139096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30357[] = {SGI139097, SGI0, SGI139098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139099, SGI139100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30358[] = {SGI139101, SGI0, SGI139102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139103, SGI139104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30359[] = {SGI139105, SGI0, SGI139106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139107, SGI139108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30360[] = {SGI139109, SGI0, SGI139110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139111, SGI139112, SGI139112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30361[] = {SGI139113, SGI0, SGI139114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139115, SGI139116, SGI139116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30362[] = {SGI139117, SGI0, SGI139118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139121, SGI139123, SGI139123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30363[] = {SGI139119, SGI139120, SGI139122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139124, SGI0, SGI0, SGI0, SGI139125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30364[] = {SGI139126, SGI0, SGI139128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30365[] = {SGI139127, SGI0, SGI139129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139130, SGI139131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30366[] = {SGI139132, SGI139133, SGI139134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139136, SGI0, SGI0, SGI0, SGI139138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30367[] = {SGI139135, SGI0, SGI139137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139139, SGI139140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30368[] = {SGI139141, SGI0, SGI139142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139143, SGI139144, SGI139144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30369[] = {SGI139145, SGI139146, SGI139147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139148, SGI0, SGI0, SGI0, SGI139149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30370[] = {SGI139150, SGI0, SGI139151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139152, SGI139153, SGI139153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30371[] = {SGI139154, SGI139155, SGI139156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139157, SGI0, SGI0, SGI0, SGI139158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30372[] = {SGI139159, SGI0, SGI139160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139161, SGI139162, SGI139162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30373[] = {SGI139163, SGI139164, SGI139165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139166, SGI0, SGI0, SGI0, SGI139167, SGI139167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30374[] = {SGI139168, SGI0, SGI139169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139170, SGI139171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30375[] = {SGI139172, SGI139173, SGI139174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139175, SGI0, SGI0, SGI0, SGI139176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30376[] = {SGI139177, SGI139178, SGI139179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139180, SGI0, SGI0, SGI0, SGI139182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30377[] = {SGI139181, SGI0, SGI139183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139184, SGI139185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30378[] = {SGI139186, SGI0, SGI139187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139189, SGI0, SGI0, SGI0, SGI139191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30379[] = {SGI139188, SGI0, SGI139190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139192, SGI139193, SGI139193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30380[] = {SGI139194, SGI139195, SGI139197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30381[] = {SGI139196, SGI0, SGI139198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139199, SGI139200, SGI139200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30382[] = {SGI139201, SGI139202, SGI139204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30383[] = {SGI139203, SGI0, SGI139205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139206, SGI139207, SGI139207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30384[] = {SGI139208, SGI0, SGI139209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139210, SGI139211, SGI139211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30385[] = {SGI139212, SGI139213, SGI139214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139215, SGI0, SGI0, SGI0, SGI139216, SGI139216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30386[] = {SGI139217, SGI0, SGI139218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139219, SGI139220, SGI139220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30387[] = {SGI139221, SGI139222, SGI139223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139224, SGI0, SGI0, SGI0, SGI139225, SGI139226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30388[] = {SGI139227, SGI0, SGI139228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139229, SGI139230, SGI139230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30389[] = {SGI139231, SGI139232, SGI139233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139234, SGI0, SGI0, SGI0, SGI139236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30390[] = {SGI139235, SGI0, SGI139237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139238, SGI139239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30391[] = {SGI139240, SGI139241, SGI139242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139243, SGI139243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30392[] = {SGI139244, SGI0, SGI139245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139246, SGI139247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30393[] = {SGI139248, SGI139249, SGI139250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139251, SGI0, SGI0, SGI0, SGI139252, SGI139252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30394[] = {SGI139253, SGI139254, SGI139256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139258, SGI139258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30395[] = {SGI139255, SGI0, SGI139257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139259, SGI139260, SGI139260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30396[] = {SGI139261, SGI0, SGI139262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139263, SGI139264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30397[] = {SGI139265, SGI139266, SGI139267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139269, SGI0, SGI0, SGI0, SGI139271, SGI139271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30398[] = {SGI139268, SGI0, SGI139270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139272, SGI139273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30399[] = {SGI139274, SGI0, SGI139275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139276, SGI139277, SGI139277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30400[] = {SGI139278, SGI0, SGI139279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139280, SGI139281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30401[] = {SGI139282, SGI0, SGI139283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139284, SGI139285, SGI139285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30402[] = {SGI139286, SGI0, SGI139287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139288, SGI139289, SGI139289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30403[] = {SGI139290, SGI0, SGI139291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139292, SGI139293, SGI139293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30404[] = {SGI139294, SGI0, SGI139295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139296, SGI139297, SGI139297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30405[] = {SGI139298, SGI0, SGI139299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139300, SGI139301, SGI139301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30406[] = {SGI139302, SGI0, SGI139303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139304, SGI139305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30407[] = {SGI139306, SGI0, SGI139307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139308, SGI139309, SGI139309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30408[] = {SGI139310, SGI0, SGI139311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139312, SGI139313, SGI139313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30409[] = {SGI139314, SGI0, SGI139315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139316, SGI139317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30410[] = {SGI139318, SGI0, SGI139319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139320, SGI139321, SGI139321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30411[] = {SGI139322, SGI0, SGI139323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139324, SGI139325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30412[] = {SGI139326, SGI0, SGI139327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139328, SGI139329, SGI139329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30413[] = {SGI139330, SGI0, SGI139331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139332, SGI139333, SGI139333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30414[] = {SGI139334, SGI0, SGI139335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139336, SGI139337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30415[] = {SGI139338, SGI0, SGI139339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139340, SGI139341, SGI139341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30416[] = {SGI139342, SGI0, SGI139343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139344, SGI139345, SGI139345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30417[] = {SGI139346, SGI0, SGI139347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139348, SGI139349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30418[] = {SGI139350, SGI0, SGI139351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139352, SGI139353, SGI139353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30419[] = {SGI139354, SGI0, SGI139355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139356, SGI139357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30420[] = {SGI139358, SGI0, SGI139359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139360, SGI139361, SGI139361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30421[] = {SGI139362, SGI0, SGI139363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139364, SGI139365, SGI139365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30422[] = {SGI139366, SGI0, SGI139367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139368, SGI139369, SGI139369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30423[] = {SGI139370, SGI0, SGI139371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139372, SGI139373, SGI139373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30424[] = {SGI139374, SGI0, SGI139375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139376, SGI139377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30425[] = {SGI139378, SGI0, SGI139379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139380, SGI139381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30426[] = {SGI139382, SGI0, SGI139383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139384, SGI139385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30427[] = {SGI139386, SGI0, SGI139387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139388, SGI139389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30428[] = {SGI139390, SGI0, SGI139391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139392, SGI139393, SGI139393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30429[] = {SGI139394, SGI0, SGI139395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139396, SGI139397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30430[] = {SGI139398, SGI0, SGI139399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139400, SGI139401, SGI139401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30431[] = {SGI139402, SGI0, SGI139403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139404, SGI139405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30432[] = {SGI139406, SGI0, SGI139407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139408, SGI139409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30433[] = {SGI139410, SGI0, SGI139411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139412, SGI139413, SGI139413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30434[] = {SGI139414, SGI0, SGI139415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139416, SGI139417, SGI139417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30435[] = {SGI139418, SGI0, SGI139419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139420, SGI139421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30436[] = {SGI139422, SGI0, SGI139423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139424, SGI139425, SGI139425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30437[] = {SGI139426, SGI0, SGI139427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139428, SGI139429, SGI139429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30438[] = {SGI139430, SGI0, SGI139431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139432, SGI139433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30439[] = {SGI139434, SGI0, SGI139435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139436, SGI139437, SGI139437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30440[] = {SGI139438, SGI0, SGI139439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139440, SGI139441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30441[] = {SGI139442, SGI0, SGI139443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139444, SGI139445, SGI139445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30442[] = {SGI139446, SGI0, SGI139447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139448, SGI139449, SGI139449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30443[] = {SGI139450, SGI0, SGI139451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139452, SGI139453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30444[] = {SGI139454, SGI0, SGI139455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139456, SGI139457, SGI139457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30445[] = {SGI139458, SGI0, SGI139459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139460, SGI139461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30446[] = {SGI139462, SGI0, SGI139463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139464, SGI139465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30447[] = {SGI139466, SGI0, SGI139467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139468, SGI139469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30448[] = {SGI139470, SGI0, SGI139471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139472, SGI139473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30449[] = {SGI139474, SGI0, SGI139475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139476, SGI139477, SGI139477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30450[] = {SGI139478, SGI0, SGI139479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139480, SGI139481, SGI139481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30451[] = {SGI139482, SGI0, SGI139483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139484, SGI139485, SGI139485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30452[] = {SGI139486, SGI0, SGI139487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139488, SGI139489, SGI139489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30453[] = {SGI139490, SGI0, SGI139491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139492, SGI139493, SGI139493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30454[] = {SGI139494, SGI0, SGI139495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139496, SGI139497, SGI139497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30455[] = {SGI139498, SGI0, SGI139499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139500, SGI139501, SGI139501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30456[] = {SGI139502, SGI0, SGI139503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139504, SGI139505, SGI139505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30457[] = {SGI139506, SGI0, SGI139507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139508, SGI139509, SGI139509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30458[] = {SGI139510, SGI0, SGI139511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139514, SGI139515, SGI139515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30459[] = {SGI139512, SGI0, SGI139513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139516, SGI139517, SGI139517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30460[] = {SGI139518, SGI0, SGI139519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139521, SGI139522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30461[] = {SGI139520, SGI0, SGI139523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139524, SGI139525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30462[] = {SGI139526, SGI0, SGI139528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139530, SGI139532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30463[] = {SGI139527, SGI0, SGI139529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139531, SGI139533, SGI139533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30464[] = {SGI139534, SGI0, SGI139536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139538, SGI139539, SGI139539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30465[] = {SGI139535, SGI0, SGI139537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139540, SGI139541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30466[] = {SGI139542, SGI0, SGI139543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139546, SGI139547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30467[] = {SGI139544, SGI0, SGI139545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139548, SGI139549, SGI139549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30468[] = {SGI139550, SGI0, SGI139551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139554, SGI139555, SGI139555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30469[] = {SGI139552, SGI0, SGI139553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139556, SGI139557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30470[] = {SGI139558, SGI0, SGI139559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139562, SGI139563, SGI139563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30471[] = {SGI139560, SGI0, SGI139561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139564, SGI139565, SGI139565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30472[] = {SGI139566, SGI0, SGI139567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139570, SGI139571, SGI139571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30473[] = {SGI139568, SGI0, SGI139569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139572, SGI139573, SGI139573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30474[] = {SGI139574, SGI0, SGI139575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139578, SGI139579, SGI139579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30475[] = {SGI139576, SGI0, SGI139577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139580, SGI139581, SGI139581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30476[] = {SGI139582, SGI0, SGI139583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139584, SGI139585, SGI139585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30477[] = {SGI139586, SGI0, SGI139587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139588, SGI139589, SGI139589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30478[] = {SGI139590, SGI0, SGI139591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139592, SGI139593, SGI139593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30479[] = {SGI139594, SGI0, SGI139595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139596, SGI139597, SGI139597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30480[] = {SGI139598, SGI0, SGI139599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139600, SGI139601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30481[] = {SGI139602, SGI0, SGI139603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139604, SGI139605, SGI139605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30482[] = {SGI139606, SGI0, SGI139607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139608, SGI139609, SGI139609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30483[] = {SGI139610, SGI0, SGI139611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139614, SGI139615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30484[] = {SGI139612, SGI0, SGI139613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139616, SGI139617, SGI139617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30485[] = {SGI139618, SGI0, SGI139619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139620, SGI139621, SGI139621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30486[] = {SGI139622, SGI0, SGI139623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139624, SGI139625, SGI139625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30487[] = {SGI139626, SGI0, SGI139627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139628, SGI139629, SGI139629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30488[] = {SGI139630, SGI0, SGI139631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139632, SGI139633, SGI139633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30489[] = {SGI139634, SGI0, SGI139635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139636, SGI139637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30490[] = {SGI139638, SGI0, SGI139639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139640, SGI139641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30491[] = {SGI139642, SGI0, SGI139643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139644, SGI139645, SGI139645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30492[] = {SGI139646, SGI0, SGI139647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139648, SGI139649, SGI139649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30493[] = {SGI139650, SGI0, SGI139651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139652, SGI139653, SGI139653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30494[] = {SGI139654, SGI0, SGI139655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139656, SGI139657, SGI139657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30495[] = {SGI139658, SGI0, SGI139659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139660, SGI139661, SGI139661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30496[] = {SGI139662, SGI0, SGI139663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139664, SGI139665, SGI139665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30497[] = {SGI139666, SGI0, SGI139667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139668, SGI139669, SGI139669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30498[] = {SGI139670, SGI0, SGI139671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139672, SGI139673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30499[] = {SGI139674, SGI0, SGI139675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139676, SGI139677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30500[] = {SGI139678, SGI0, SGI139679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139680, SGI139682, SGI139682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30501[] = {SGI139681, SGI139683, SGI139684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139685, SGI0, SGI0, SGI0, SGI139686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30502[] = {SGI139687, SGI0, SGI139688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30503[] = {SGI139689, SGI0, SGI139690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139691, SGI139692, SGI139692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30504[] = {SGI139693, SGI0, SGI139694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139695, SGI0, SGI0, SGI0, SGI139696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30505[] = {SGI139697, SGI0, SGI139698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139699, SGI139700, SGI139700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30506[] = {SGI139701, SGI139702, SGI139703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139705, SGI0, SGI0, SGI0, SGI139707, SGI139707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30507[] = {SGI139704, SGI0, SGI139706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139708, SGI139709, SGI139709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30508[] = {SGI139710, SGI139711, SGI139712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139713, SGI0, SGI0, SGI0, SGI139714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30509[] = {SGI139715, SGI139716, SGI139717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139718, SGI0, SGI0, SGI0, SGI139719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30510[] = {SGI139720, SGI139721, SGI139722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139723, SGI0, SGI0, SGI0, SGI139724, SGI139724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30511[] = {SGI139725, SGI139726, SGI139727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139728, SGI139728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30512[] = {SGI139729, SGI139730, SGI139731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30513[] = {SGI139732, SGI139733, SGI139734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139735, SGI139735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30514[] = {SGI139736, SGI139737, SGI139738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30515[] = {SGI139739, SGI139740, SGI139741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139742, SGI0, SGI0, SGI0, SGI139743, SGI139743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30516[] = {SGI139744, SGI139745, SGI139746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30517[] = {SGI139747, SGI139748, SGI139749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139750, SGI0, SGI0, SGI0, SGI139751, SGI139751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30518[] = {SGI139752, SGI139753, SGI139754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30519[] = {SGI139756, SGI139757, SGI139758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30520[] = {SGI139760, SGI139761, SGI139762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30521[] = {SGI139764, SGI0, SGI139765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139766, SGI139767, SGI139767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30522[] = {SGI139768, SGI139769, SGI139770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139771, SGI0, SGI0, SGI0, SGI139772, SGI139773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30523[] = {SGI139774, SGI0, SGI139775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139776, SGI139777, SGI139777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30524[] = {SGI139778, SGI0, SGI139779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139782, SGI139783, SGI139783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30525[] = {SGI0, SGI139780, SGI139781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30526[] = {SGI139784, SGI0, SGI139785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139786, SGI139787, SGI139787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30527[] = {SGI139788, SGI139789, SGI139790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139791, SGI0, SGI0, SGI0, SGI139792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30528[] = {SGI0, SGI139793, SGI139794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30529[] = {SGI139795, SGI139796, SGI139797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139799, SGI0, SGI0, SGI0, SGI139801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30530[] = {SGI139798, SGI0, SGI139800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139802, SGI139803, SGI139803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30531[] = {SGI139804, SGI0, SGI139805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139806, SGI139807, SGI139807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30532[] = {SGI139808, SGI0, SGI139809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139810, SGI139811, SGI139811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30533[] = {SGI139812, SGI0, SGI139813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139814, SGI139815, SGI139815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30534[] = {SGI139818, SGI0, SGI139820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139821, SGI139822, SGI139822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30535[] = {SGI139816, SGI139817, SGI139819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30536[] = {SGI139824, SGI0, SGI139825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139828, SGI139829, SGI139829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30537[] = {SGI139826, SGI0, SGI139827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139830, SGI139831, SGI139831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30538[] = {SGI139832, SGI139833, SGI139834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30539[] = {SGI139836, SGI0, SGI139837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139839, SGI139840, SGI139840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30540[] = {SGI139838, SGI0, SGI139841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139842, SGI139843, SGI139843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30541[] = {SGI0, SGI139844, SGI139845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30542[] = {SGI139847, SGI0, SGI139848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139850, SGI139852, SGI139852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30543[] = {SGI139849, SGI0, SGI139851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139853, SGI139854, SGI139854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30544[] = {SGI139855, SGI139857, SGI139858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139860, SGI0, SGI0, SGI0, SGI139862, SGI139864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30545[] = {SGI139856, SGI0, SGI139859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139861, SGI139863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30546[] = {SGI139865, SGI0, SGI139866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139867, SGI139870, SGI139870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30547[] = {SGI139868, SGI139869, SGI139872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139874, SGI0, SGI0, SGI0, SGI139875, SGI139875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30548[] = {SGI139871, SGI0, SGI139873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139876, SGI139877, SGI139877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30549[] = {SGI139878, SGI0, SGI139879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139882, SGI139884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30550[] = {SGI139880, SGI0, SGI139881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139887, SGI139888, SGI139888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30551[] = {SGI139883, SGI139885, SGI139886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139889, SGI0, SGI0, SGI0, SGI139890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30552[] = {SGI139891, SGI0, SGI139893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139898, SGI139900, SGI139900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30553[] = {SGI139892, SGI0, SGI139894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139899, SGI139901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30554[] = {SGI139895, SGI139896, SGI139897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139902, SGI0, SGI0, SGI0, SGI139903, SGI139903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30555[] = {SGI139904, SGI0, SGI139905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139908, SGI139909, SGI139909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30556[] = {SGI139906, SGI0, SGI139907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139910, SGI139911, SGI139911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30557[] = {SGI139912, SGI0, SGI139913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30558[] = {SGI139914, SGI0, SGI139915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139919, SGI139921, SGI139921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30559[] = {SGI139916, SGI0, SGI139917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139922, SGI139923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30560[] = {SGI139918, SGI0, SGI139920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139924, SGI0, SGI0, SGI0, SGI139925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30561[] = {SGI139926, SGI0, SGI139927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139931, SGI139932, SGI139932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30562[] = {SGI139928, SGI139929, SGI139930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139933, SGI0, SGI0, SGI0, SGI139934, SGI139934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30563[] = {SGI139935, SGI0, SGI139936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139937, SGI139938, SGI139938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30564[] = {SGI139941, SGI139942, SGI139943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30565[] = {SGI139939, SGI0, SGI139940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139944, SGI139945, SGI139945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30566[] = {SGI139946, SGI0, SGI139947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139948, SGI139949, SGI139949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30567[] = {SGI139950, SGI139951, SGI139953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139955, SGI0, SGI0, SGI0, SGI139956, SGI139956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30568[] = {SGI139952, SGI0, SGI139954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139957, SGI139958, SGI139958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30569[] = {SGI139959, SGI0, SGI139960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139961, SGI139962, SGI139962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30570[] = {SGI139963, SGI139964, SGI139965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139968, SGI139968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30571[] = {SGI139966, SGI0, SGI139967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139970, SGI139972, SGI139972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30572[] = {SGI139969, SGI0, SGI139971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139973, SGI139974, SGI139974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30573[] = {SGI139975, SGI0, SGI139976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139979, SGI139980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30574[] = {SGI139977, SGI0, SGI139978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139983, SGI139985, SGI139985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30575[] = {SGI139981, SGI139982, SGI139984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30576[] = {SGI139986, SGI0, SGI139987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139991, SGI139992, SGI139992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30577[] = {SGI139988, SGI139989, SGI139990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139993, SGI0, SGI0, SGI0, SGI139994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30578[] = {SGI139995, SGI0, SGI139996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI139997, SGI139998, SGI139998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30579[] = {SGI139999, SGI0, SGI140000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140001, SGI140002, SGI140002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30580[] = {SGI140003, SGI140004, SGI140005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140007, SGI0, SGI0, SGI0, SGI140009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30581[] = {SGI140006, SGI0, SGI140008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140010, SGI140011, SGI140011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30582[] = {SGI140012, SGI0, SGI140013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140014, SGI140015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30583[] = {SGI140016, SGI0, SGI140017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140018, SGI140020, SGI140020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30584[] = {SGI140019, SGI140021, SGI140022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140025, SGI0, SGI0, SGI0, SGI140026, SGI140026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30585[] = {SGI140023, SGI0, SGI140024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140027, SGI140028, SGI140028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30586[] = {SGI140029, SGI0, SGI140030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140031, SGI140033, SGI140033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30587[] = {SGI140032, SGI140034, SGI140035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30588[] = {SGI140036, SGI0, SGI140037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140038, SGI140039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30589[] = {SGI140040, SGI0, SGI140041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140044, SGI140045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30590[] = {SGI140042, SGI0, SGI140043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30591[] = {SGI140046, SGI0, SGI140047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140048, SGI140049, SGI140049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30592[] = {SGI140050, SGI140051, SGI140052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140053, SGI0, SGI0, SGI0, SGI140054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30593[] = {SGI140055, SGI0, SGI140056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140057, SGI140058, SGI140058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30594[] = {SGI140059, SGI140060, SGI140061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140062, SGI140062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30595[] = {SGI140063, SGI0, SGI140064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140065, SGI140066, SGI140066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30596[] = {SGI140067, SGI140068, SGI140069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30597[] = {SGI140070, SGI0, SGI140072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140074, SGI140076, SGI140076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30598[] = {SGI140071, SGI0, SGI140073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140075, SGI140077, SGI140077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30599[] = {SGI140078, SGI140079, SGI140080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140081, SGI0, SGI0, SGI0, SGI140082, SGI140083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30600[] = {SGI140084, SGI0, SGI140086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140090, SGI140092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30601[] = {SGI140085, SGI0, SGI140087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140091, SGI140093, SGI140093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30602[] = {SGI140088, SGI140089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30603[] = {SGI140094, SGI0, SGI140095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140098, SGI140099, SGI140099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30604[] = {SGI140096, SGI140097, SGI140100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140101, SGI0, SGI0, SGI0, SGI140102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30605[] = {SGI140103, SGI0, SGI140104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140105, SGI140107, SGI140107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30606[] = {SGI0, SGI0, SGI140106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30607[] = {SGI0, SGI140110, SGI140111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30608[] = {SGI140108, SGI0, SGI140109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140112, SGI140113, SGI140113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30609[] = {SGI140114, SGI140115, SGI140116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140119, SGI0, SGI0, SGI0, SGI0, SGI140120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30610[] = {SGI140117, SGI0, SGI140118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140121, SGI140122, SGI140122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30611[] = {SGI140123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30612[] = {SGI140124, SGI0, SGI140125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140126, SGI140127, SGI140127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30613[] = {SGI0, SGI140128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30614[] = {SGI140129, SGI140130, SGI140131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140132, SGI0, SGI0, SGI0, SGI140133, SGI140133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30615[] = {SGI0, SGI140134, SGI140135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140136, SGI0, SGI0, SGI0, SGI140137, SGI140138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30616[] = {SGI140139, SGI140140, SGI140141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140142, SGI0, SGI0, SGI0, SGI140143, SGI140144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30617[] = {SGI140145, SGI140146, SGI140147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30618[] = {SGI0, SGI140148, SGI140149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140150, SGI0, SGI0, SGI0, SGI140151, SGI140151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30619[] = {SGI140152, SGI0, SGI140153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140154, SGI0, SGI0, SGI0, SGI140155, SGI140155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30620[] = {SGI140156, SGI140157, SGI140158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140159, SGI0, SGI0, SGI0, SGI140160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30621[] = {SGI140161, SGI140162, SGI140163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140164, SGI0, SGI0, SGI0, SGI140165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30622[] = {SGI140166, SGI0, SGI140167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30623[] = {SGI140168, SGI0, SGI140169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140170, SGI0, SGI0, SGI0, SGI140171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30624[] = {SGI140172, SGI140173, SGI140174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140175, SGI0, SGI0, SGI0, SGI140176, SGI140176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30625[] = {SGI140177, SGI140178, SGI140179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140180, SGI0, SGI0, SGI0, SGI140181, SGI140181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30626[] = {SGI140182, SGI140183, SGI140184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140185, SGI0, SGI0, SGI0, SGI140186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30627[] = {SGI140187, SGI140188, SGI140189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30628[] = {SGI140190, SGI140191, SGI140192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140193, SGI0, SGI0, SGI0, SGI140194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30629[] = {SGI140195, SGI140196, SGI140197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30630[] = {SGI0, SGI140198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30631[] = {SGI140199, SGI140200, SGI140201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30632[] = {SGI0, SGI0, SGI140202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30633[] = {SGI140203, SGI140204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30634[] = {SGI0, SGI140205, SGI140206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30635[] = {SGI140207, SGI140208, SGI140209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140210, SGI0, SGI0, SGI0, SGI140211, SGI140211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30636[] = {SGI140212, SGI140213, SGI140214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140215, SGI0, SGI0, SGI0, SGI0, SGI140216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30637[] = {SGI140217, SGI140218, SGI140219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140220, SGI0, SGI0, SGI0, SGI140221, SGI140222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30638[] = {SGI140223, SGI140224, SGI140225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140226, SGI0, SGI0, SGI0, SGI140227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30639[] = {SGI140228, SGI140229, SGI140230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30640[] = {SGI140231, SGI0, SGI140232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30641[] = {SGI0, SGI140233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30642[] = {SGI0, SGI140234, SGI140235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140236, SGI0, SGI0, SGI0, SGI140237, SGI140238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30643[] = {SGI140239, SGI140240, SGI140241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140242, SGI140242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30644[] = {SGI140243, SGI140244, SGI140245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140246, SGI0, SGI0, SGI0, SGI140247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30645[] = {SGI140248, SGI140249, SGI140250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140251, SGI140251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30646[] = {SGI140252, SGI140253, SGI140254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140255, SGI0, SGI0, SGI0, SGI140256, SGI140256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30647[] = {SGI140257, SGI140258, SGI140259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140260, SGI0, SGI0, SGI0, SGI140261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30648[] = {SGI140262, SGI140263, SGI140264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140265, SGI0, SGI0, SGI0, SGI140266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30649[] = {SGI140267, SGI140268, SGI140269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140270, SGI0, SGI0, SGI0, SGI140271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30650[] = {SGI0, SGI140272, SGI140273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140274, SGI0, SGI0, SGI0, SGI140275, SGI140276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30651[] = {SGI140277, SGI140278, SGI140279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140280, SGI0, SGI0, SGI0, SGI140281, SGI140282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30652[] = {SGI140283, SGI140284, SGI140285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140286, SGI0, SGI0, SGI0, SGI140287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30653[] = {SGI140288, SGI0, SGI140289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140290, SGI140291, SGI140291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30654[] = {SGI140292, SGI0, SGI140293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140294, SGI140295, SGI140295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30655[] = {SGI140296, SGI0, SGI140297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140298, SGI140299, SGI140299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30656[] = {SGI140300, SGI0, SGI140301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140302, SGI140303, SGI140303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30657[] = {SGI140304, SGI0, SGI140305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140306, SGI140307, SGI140307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30658[] = {SGI140308, SGI0, SGI140309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140310, SGI140311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30659[] = {SGI140312, SGI0, SGI140313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140314, SGI140315, SGI140315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30660[] = {SGI140316, SGI0, SGI140317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140318, SGI140319, SGI140319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30661[] = {SGI140320, SGI0, SGI140321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140322, SGI140323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30662[] = {SGI140324, SGI0, SGI140325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140326, SGI140327, SGI140327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30663[] = {SGI140328, SGI0, SGI140329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140330, SGI140331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30664[] = {SGI140332, SGI0, SGI140333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140334, SGI140335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30665[] = {SGI140336, SGI0, SGI140337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140338, SGI140339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30666[] = {SGI140340, SGI0, SGI140341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140342, SGI140343, SGI140343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30667[] = {SGI140345, SGI0, SGI140347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140349, SGI140351, SGI140351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30668[] = {SGI140344, SGI0, SGI140346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140348, SGI140350, SGI140350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30669[] = {SGI140352, SGI0, SGI140353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140354, SGI140355, SGI140355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30670[] = {SGI140356, SGI0, SGI140358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140360, SGI140362, SGI140362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30671[] = {SGI140357, SGI0, SGI140359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140361, SGI140363, SGI140363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30672[] = {SGI140364, SGI0, SGI140365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140366, SGI140367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30673[] = {SGI140368, SGI0, SGI140369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140370, SGI140371, SGI140371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30674[] = {SGI140372, SGI0, SGI140373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140374, SGI140375, SGI140375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30675[] = {SGI140376, SGI0, SGI140377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140378, SGI140379, SGI140379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30676[] = {SGI140380, SGI0, SGI140381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140382, SGI140383, SGI140383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30677[] = {SGI140384, SGI0, SGI140385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140386, SGI140387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30678[] = {SGI140388, SGI0, SGI140389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140392, SGI140393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30679[] = {SGI140390, SGI0, SGI140391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140394, SGI140395, SGI140395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30680[] = {SGI140396, SGI0, SGI140397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140398, SGI140399, SGI140399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30681[] = {SGI140401, SGI0, SGI140403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140405, SGI140407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30682[] = {SGI140400, SGI0, SGI140402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140404, SGI140406, SGI140406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30683[] = {SGI140408, SGI0, SGI140409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140410, SGI140411, SGI140411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30684[] = {SGI140412, SGI0, SGI140413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140414, SGI140415, SGI140415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30685[] = {SGI140416, SGI0, SGI140417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140418, SGI140419, SGI140419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30686[] = {SGI140420, SGI0, SGI140421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140422, SGI140423, SGI140423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30687[] = {SGI140424, SGI0, SGI140425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140426, SGI140427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30688[] = {SGI140428, SGI0, SGI140429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140430, SGI140431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30689[] = {SGI140432, SGI0, SGI140433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140434, SGI140435, SGI140435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30690[] = {SGI140436, SGI0, SGI140437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140438, SGI140439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30691[] = {SGI140440, SGI0, SGI140441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140442, SGI140443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30692[] = {SGI140444, SGI0, SGI140445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140446, SGI140447, SGI140447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30693[] = {SGI140448, SGI0, SGI140449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140450, SGI140451, SGI140451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30694[] = {SGI140452, SGI0, SGI140453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140454, SGI140455, SGI140455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30695[] = {SGI140456, SGI0, SGI140457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140458, SGI140459, SGI140459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30696[] = {SGI140460, SGI0, SGI140461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140462, SGI140463, SGI140463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30697[] = {SGI140464, SGI0, SGI140465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140466, SGI140467, SGI140467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30698[] = {SGI140468, SGI0, SGI140469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140470, SGI140471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30699[] = {SGI140472, SGI0, SGI140473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140474, SGI140475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30700[] = {SGI140476, SGI0, SGI140477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140478, SGI140479, SGI140479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30701[] = {SGI140480, SGI0, SGI140481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140482, SGI140483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30702[] = {SGI140484, SGI0, SGI140485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140486, SGI140487, SGI140487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30703[] = {SGI140488, SGI0, SGI140489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140490, SGI140491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30704[] = {SGI140492, SGI0, SGI140493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140494, SGI140495, SGI140495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30705[] = {SGI140496, SGI0, SGI140497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140498, SGI140499, SGI140499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30706[] = {SGI140500, SGI0, SGI140501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140502, SGI140503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30707[] = {SGI140504, SGI0, SGI140505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140506, SGI140507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30708[] = {SGI140508, SGI140509, SGI140510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140511, SGI0, SGI0, SGI0, SGI140512, SGI140512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30709[] = {SGI140513, SGI0, SGI140514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30710[] = {SGI140515, SGI140516, SGI140517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140518, SGI0, SGI0, SGI0, SGI140519, SGI140519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30711[] = {SGI140520, SGI140521, SGI140522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140523, SGI0, SGI0, SGI0, SGI140524, SGI140524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30712[] = {SGI140525, SGI140526, SGI140527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140528, SGI0, SGI0, SGI0, SGI140529, SGI140529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30713[] = {SGI140530, SGI0, SGI140531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30714[] = {SGI140532, SGI0, SGI140533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140534, SGI140535, SGI140535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30715[] = {SGI140536, SGI0, SGI140537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140538, SGI140539, SGI140539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30716[] = {SGI140540, SGI0, SGI140541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140542, SGI140543, SGI140543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30717[] = {SGI140544, SGI0, SGI140545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140546, SGI140547, SGI140547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30718[] = {SGI140548, SGI0, SGI140549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140550, SGI140551, SGI140551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30719[] = {SGI140552, SGI0, SGI140553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140554, SGI140555, SGI140555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30720[] = {SGI140556, SGI0, SGI140557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140558, SGI140559, SGI140559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30721[] = {SGI140560, SGI140561, SGI140562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140563, SGI0, SGI0, SGI0, SGI140564, SGI140564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30722[] = {SGI140565, SGI140566, SGI140567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140568, SGI0, SGI0, SGI0, SGI140569, SGI140569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30723[] = {SGI140570, SGI140571, SGI140572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140573, SGI0, SGI0, SGI0, SGI140574, SGI140574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30724[] = {SGI140575, SGI140576, SGI140577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140578, SGI0, SGI0, SGI0, SGI140579, SGI140579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30725[] = {SGI140580, SGI140581, SGI140582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140583, SGI0, SGI0, SGI0, SGI140584, SGI140584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30726[] = {SGI140585, SGI0, SGI140586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30727[] = {SGI140587, SGI140588, SGI140589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140590, SGI0, SGI0, SGI0, SGI140591, SGI140591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30728[] = {SGI140592, SGI0, SGI140593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30729[] = {SGI140594, SGI140595, SGI140596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140597, SGI0, SGI0, SGI0, SGI140598, SGI140598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30730[] = {SGI140599, SGI0, SGI140600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30731[] = {SGI140601, SGI140602, SGI140603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140604, SGI0, SGI0, SGI0, SGI140605, SGI140605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30732[] = {SGI140606, SGI0, SGI140607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30733[] = {SGI140608, SGI0, SGI140609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140610, SGI0, SGI0, SGI0, SGI140611, SGI140611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30734[] = {SGI140612, SGI140613, SGI140614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140615, SGI0, SGI0, SGI0, SGI140616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30735[] = {SGI140617, SGI140618, SGI140619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140620, SGI0, SGI0, SGI0, SGI140621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30736[] = {SGI140622, SGI0, SGI140623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30737[] = {SGI140624, SGI0, SGI140625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140626, SGI0, SGI0, SGI0, SGI140627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30738[] = {SGI140628, SGI140629, SGI140630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140631, SGI0, SGI0, SGI0, SGI140632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30739[] = {SGI140633, SGI140634, SGI140635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140636, SGI0, SGI0, SGI0, SGI140637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30740[] = {SGI140638, SGI140639, SGI140640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140641, SGI0, SGI0, SGI0, SGI140642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30741[] = {SGI140643, SGI0, SGI140644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30742[] = {SGI140645, SGI140646, SGI140647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140648, SGI0, SGI0, SGI0, SGI140649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30743[] = {SGI140650, SGI140651, SGI140652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140653, SGI0, SGI0, SGI0, SGI140654, SGI140654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30744[] = {SGI140655, SGI140656, SGI140657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140658, SGI0, SGI0, SGI0, SGI140659, SGI140659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30745[] = {SGI140660, SGI140661, SGI140662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140663, SGI0, SGI0, SGI0, SGI140664, SGI140664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30746[] = {SGI140665, SGI140666, SGI140667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140668, SGI0, SGI0, SGI0, SGI140669, SGI140669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30747[] = {SGI140670, SGI140671, SGI140672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140673, SGI0, SGI0, SGI0, SGI140674, SGI140674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30748[] = {SGI140675, SGI140676, SGI140677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140678, SGI0, SGI0, SGI0, SGI140679, SGI140679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30749[] = {SGI140680, SGI140681, SGI140682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140683, SGI0, SGI0, SGI0, SGI140684, SGI140684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30750[] = {SGI140685, SGI140686, SGI140687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140688, SGI0, SGI0, SGI0, SGI140689, SGI140689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30751[] = {SGI140690, SGI140691, SGI140692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140693, SGI0, SGI0, SGI0, SGI140694, SGI140694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30752[] = {SGI140695, SGI140696, SGI140697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140698, SGI0, SGI0, SGI0, SGI140699, SGI140699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30753[] = {SGI140700, SGI140701, SGI140702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140703, SGI0, SGI0, SGI0, SGI140704, SGI140704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30754[] = {SGI140705, SGI140706, SGI140707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140708, SGI0, SGI0, SGI0, SGI140709, SGI140709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30755[] = {SGI140710, SGI140711, SGI140712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140713, SGI0, SGI0, SGI0, SGI140714, SGI140714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30756[] = {SGI140715, SGI140716, SGI140717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140718, SGI0, SGI0, SGI0, SGI140719, SGI140719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30757[] = {SGI140720, SGI140721, SGI140722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140723, SGI0, SGI0, SGI0, SGI140724, SGI140724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30758[] = {SGI140725, SGI140726, SGI140727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140728, SGI0, SGI0, SGI0, SGI140729, SGI140729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30759[] = {SGI140730, SGI140731, SGI140732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140733, SGI0, SGI0, SGI0, SGI140734, SGI140734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30760[] = {SGI140735, SGI140736, SGI140737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140738, SGI0, SGI0, SGI0, SGI140739, SGI140739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30761[] = {SGI140740, SGI140741, SGI140742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140743, SGI0, SGI0, SGI0, SGI140744, SGI140744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30762[] = {SGI140745, SGI0, SGI140746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30763[] = {SGI140747, SGI140748, SGI140749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140750, SGI0, SGI0, SGI0, SGI140751, SGI140751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30764[] = {SGI140752, SGI140753, SGI140754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140755, SGI0, SGI0, SGI0, SGI140756, SGI140756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30765[] = {SGI140757, SGI140758, SGI140759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140760, SGI0, SGI0, SGI0, SGI140761, SGI140761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30766[] = {SGI140762, SGI140763, SGI140764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140765, SGI0, SGI0, SGI0, SGI140766, SGI140766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30767[] = {SGI140767, SGI140768, SGI140769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140770, SGI0, SGI0, SGI0, SGI140771, SGI140771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30768[] = {SGI140772, SGI140773, SGI140774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140775, SGI0, SGI0, SGI0, SGI140776, SGI140776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30769[] = {SGI140777, SGI0, SGI140778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140779, SGI140780, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30770[] = {SGI140781, SGI140782, SGI140783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140784, SGI140785, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30771[] = {SGI140786, SGI0, SGI140787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30772[] = {SGI140788, SGI140789, SGI140790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140791, SGI0, SGI0, SGI0, SGI140792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30773[] = {SGI140793, SGI140794, SGI140795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140796, SGI0, SGI0, SGI0, SGI140797, SGI140797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30774[] = {SGI140798, SGI140799, SGI140800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140801, SGI140801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30775[] = {SGI140802, SGI140803, SGI140804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140805, SGI0, SGI0, SGI0, SGI140806, SGI140806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30776[] = {SGI140807, SGI140808, SGI140809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140810, SGI0, SGI0, SGI0, SGI140811, SGI140812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30777[] = {SGI140813, SGI0, SGI140814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140815, SGI0, SGI0, SGI0, SGI140816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30778[] = {SGI140817, SGI140818, SGI140819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140820, SGI0, SGI0, SGI0, SGI140821, SGI140821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30779[] = {SGI140822, SGI0, SGI140823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30780[] = {SGI140824, SGI140825, SGI140826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140827, SGI0, SGI0, SGI0, SGI140828, SGI140828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30781[] = {SGI140829, SGI140830, SGI140831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140832, SGI0, SGI0, SGI0, SGI140833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30782[] = {SGI140834, SGI0, SGI140835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30783[] = {SGI140836, SGI140837, SGI140838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140839, SGI0, SGI0, SGI0, SGI140840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30784[] = {SGI140841, SGI0, SGI140842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30785[] = {SGI140843, SGI140844, SGI140845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140846, SGI0, SGI0, SGI0, SGI140847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30786[] = {SGI140848, SGI140849, SGI140850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140851, SGI0, SGI0, SGI0, SGI140852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30787[] = {SGI140853, SGI140854, SGI140855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140856, SGI0, SGI0, SGI0, SGI140857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30788[] = {SGI140858, SGI140859, SGI140860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140861, SGI0, SGI0, SGI0, SGI140862, SGI140862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30789[] = {SGI140863, SGI0, SGI140864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30790[] = {SGI140865, SGI140866, SGI140867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30791[] = {SGI140868, SGI140869, SGI140870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140872, SGI0, SGI0, SGI0, SGI140874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30792[] = {SGI140871, SGI0, SGI140873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30793[] = {SGI140875, SGI140876, SGI140877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140878, SGI0, SGI0, SGI0, SGI140879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30794[] = {SGI140880, SGI140881, SGI140882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140883, SGI0, SGI0, SGI0, SGI140884, SGI140884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30795[] = {SGI140885, SGI140886, SGI140887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140888, SGI0, SGI0, SGI0, SGI140889, SGI140889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30796[] = {SGI140890, SGI140891, SGI140892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140893, SGI0, SGI0, SGI0, SGI140894, SGI140894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30797[] = {SGI140895, SGI140896, SGI140897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140898, SGI0, SGI0, SGI0, SGI140899, SGI140899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30798[] = {SGI140900, SGI140901, SGI140902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140903, SGI0, SGI0, SGI0, SGI140904, SGI140904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30799[] = {SGI140905, SGI0, SGI140906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30800[] = {SGI140907, SGI140908, SGI140909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140910, SGI0, SGI0, SGI0, SGI140911, SGI140911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30801[] = {SGI140912, SGI140913, SGI140914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140915, SGI0, SGI0, SGI0, SGI140916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30802[] = {SGI140917, SGI140918, SGI140919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140920, SGI0, SGI0, SGI0, SGI140921, SGI140921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30803[] = {SGI140922, SGI140923, SGI140924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140925, SGI0, SGI0, SGI0, SGI140926, SGI140926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30804[] = {SGI140927, SGI140928, SGI140929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140930, SGI140930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30805[] = {SGI140931, SGI0, SGI140932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30806[] = {SGI140933, SGI0, SGI140934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140935, SGI0, SGI0, SGI0, SGI140936, SGI140936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30807[] = {SGI140937, SGI140938, SGI140939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140940, SGI0, SGI0, SGI0, SGI140941, SGI140941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30808[] = {SGI140942, SGI140943, SGI140944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140945, SGI140945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30809[] = {SGI140946, SGI140947, SGI140948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140949, SGI0, SGI0, SGI0, SGI140950, SGI140950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30810[] = {SGI140951, SGI140952, SGI140953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140954, SGI0, SGI0, SGI0, SGI140955, SGI140955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30811[] = {SGI140956, SGI140957, SGI140958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140959, SGI0, SGI0, SGI0, SGI140960, SGI140960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30812[] = {SGI140961, SGI140962, SGI140963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140964, SGI0, SGI0, SGI0, SGI140965, SGI140965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30813[] = {SGI140966, SGI140967, SGI140968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140969, SGI0, SGI0, SGI0, SGI140970, SGI140970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30814[] = {SGI140971, SGI140972, SGI140973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140974, SGI0, SGI0, SGI0, SGI140975, SGI140975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30815[] = {SGI140976, SGI140977, SGI140978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140979, SGI0, SGI0, SGI0, SGI140980, SGI140980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30816[] = {SGI140981, SGI0, SGI140982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30817[] = {SGI140983, SGI0, SGI140984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30818[] = {SGI140985, SGI140986, SGI140987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140988, SGI140988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30819[] = {SGI140989, SGI0, SGI140990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140991, SGI140992, SGI140992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30820[] = {SGI140993, SGI0, SGI140994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140995, SGI140996, SGI140996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30821[] = {SGI140997, SGI0, SGI140998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI140999, SGI141000, SGI141000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30822[] = {SGI141001, SGI0, SGI141002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141003, SGI141004, SGI141004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30823[] = {SGI141005, SGI0, SGI141006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141007, SGI141008, SGI141008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30824[] = {SGI141009, SGI0, SGI141010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141011, SGI141012, SGI141012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30825[] = {SGI141013, SGI0, SGI141014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141015, SGI141016, SGI141016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30826[] = {SGI141017, SGI0, SGI141018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141019, SGI141020, SGI141020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30827[] = {SGI141021, SGI0, SGI141022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141023, SGI141024, SGI141024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30828[] = {SGI141025, SGI0, SGI141026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141027, SGI141028, SGI141028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30829[] = {SGI141029, SGI0, SGI141030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141031, SGI141032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30830[] = {SGI141033, SGI0, SGI141034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141035, SGI141036, SGI141036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30831[] = {SGI141037, SGI0, SGI141038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141039, SGI141040, SGI141040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30832[] = {SGI141041, SGI0, SGI141042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141043, SGI141044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30833[] = {SGI141045, SGI0, SGI141046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141047, SGI141048, SGI141048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30834[] = {SGI141049, SGI0, SGI141050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141051, SGI141052, SGI141052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30835[] = {SGI141053, SGI0, SGI141054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141055, SGI141056, SGI141056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30836[] = {SGI141061, SGI141062, SGI141063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141064, SGI0, SGI0, SGI0, SGI141065, SGI141065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30837[] = {SGI141066, SGI141067, SGI141068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141069, SGI0, SGI0, SGI0, SGI141070, SGI141070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30838[] = {SGI141071, SGI0, SGI141072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30839[] = {SGI141073, SGI141074, SGI141075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141076, SGI0, SGI0, SGI0, SGI141077, SGI141077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30840[] = {SGI141078, SGI141079, SGI141080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141081, SGI0, SGI0, SGI0, SGI141082, SGI141082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30841[] = {SGI141083, SGI141084, SGI141085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141086, SGI141086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30842[] = {SGI141087, SGI141088, SGI141089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141090, SGI0, SGI0, SGI0, SGI141091, SGI141091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30843[] = {SGI141092, SGI141093, SGI141094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141095, SGI0, SGI0, SGI0, SGI141096, SGI141096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30844[] = {SGI141097, SGI0, SGI141098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141099, SGI141100, SGI141100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30845[] = {SGI141103, SGI0, SGI141104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141105, SGI141106, SGI141106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30846[] = {SGI141107, SGI0, SGI141108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141109, SGI141110, SGI141110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30847[] = {SGI141111, SGI0, SGI141112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141113, SGI141114, SGI141114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30848[] = {SGI141115, SGI0, SGI141116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141117, SGI141118, SGI141118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30849[] = {SGI141119, SGI0, SGI141120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141121, SGI141122, SGI141122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30850[] = {SGI141123, SGI0, SGI141124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141125, SGI141126, SGI141126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30851[] = {SGI141127, SGI0, SGI141128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141129, SGI141130, SGI141130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30852[] = {SGI141131, SGI0, SGI141132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141133, SGI141134, SGI141134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30853[] = {SGI141135, SGI0, SGI141136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141137, SGI141138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30854[] = {SGI141139, SGI0, SGI141140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141141, SGI141142, SGI141142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30855[] = {SGI141143, SGI0, SGI141144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141145, SGI141146, SGI141146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30856[] = {SGI141147, SGI0, SGI141148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141149, SGI141150, SGI141150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30857[] = {SGI141151, SGI0, SGI141152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141153, SGI141154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30858[] = {SGI141155, SGI0, SGI141156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141157, SGI141158, SGI141158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30859[] = {SGI141159, SGI141160, SGI141161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141162, SGI0, SGI0, SGI0, SGI141163, SGI141163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30860[] = {SGI141164, SGI141165, SGI141166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141167, SGI0, SGI0, SGI0, SGI141168, SGI141168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30861[] = {SGI141101, SGI0, SGI141102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141169, SGI141170, SGI141170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30862[] = {SGI141171, SGI0, SGI141172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141173, SGI141174, SGI141174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30863[] = {SGI141175, SGI0, SGI141176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141177, SGI141178, SGI141178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30864[] = {SGI141179, SGI0, SGI141180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141181, SGI141182, SGI141182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30865[] = {SGI141183, SGI0, SGI141184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141185, SGI141186, SGI141186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30866[] = {SGI141187, SGI0, SGI141188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141189, SGI141190, SGI141190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30867[] = {SGI141191, SGI0, SGI141192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141193, SGI141194, SGI141194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30868[] = {SGI141057, SGI0, SGI141058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141059, SGI141060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30869[] = {SGI141195, SGI0, SGI141196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141197, SGI141198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30870[] = {SGI141199, SGI0, SGI141200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141201, SGI141202, SGI141202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30871[] = {SGI141203, SGI0, SGI141204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141205, SGI141206, SGI141206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30872[] = {SGI141207, SGI0, SGI141208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141209, SGI141210, SGI141210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30873[] = {SGI141211, SGI0, SGI141212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141213, SGI141214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30874[] = {SGI141215, SGI0, SGI141216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141217, SGI141218, SGI141218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30875[] = {SGI141219, SGI0, SGI141220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141221, SGI141222, SGI141222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30876[] = {SGI141223, SGI0, SGI141224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141225, SGI141226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30877[] = {SGI141227, SGI0, SGI141228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141229, SGI141230, SGI141230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30878[] = {SGI141231, SGI0, SGI141232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141233, SGI141234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30879[] = {SGI141235, SGI0, SGI141236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141237, SGI141238, SGI141238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30880[] = {SGI141239, SGI0, SGI141240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141241, SGI141242, SGI141242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30881[] = {SGI141243, SGI0, SGI141244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141245, SGI141246, SGI141246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30882[] = {SGI141247, SGI0, SGI141248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141249, SGI141250, SGI141250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30883[] = {SGI141251, SGI0, SGI141252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141253, SGI141254, SGI141254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30884[] = {SGI141255, SGI0, SGI141256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141257, SGI141258, SGI141258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30885[] = {SGI141259, SGI0, SGI141260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141261, SGI141262, SGI141262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30886[] = {SGI141263, SGI0, SGI141264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141265, SGI141266, SGI141266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30887[] = {SGI141267, SGI0, SGI141268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141269, SGI141270, SGI141270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30888[] = {SGI141271, SGI141272, SGI141273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141274, SGI0, SGI0, SGI0, SGI141275, SGI141275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30889[] = {SGI141276, SGI141277, SGI141278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141279, SGI0, SGI0, SGI0, SGI141280, SGI141280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30890[] = {SGI141281, SGI0, SGI141282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30891[] = {SGI141283, SGI141284, SGI141285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141286, SGI0, SGI0, SGI0, SGI141287, SGI141287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30892[] = {SGI141288, SGI141289, SGI141290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141291, SGI0, SGI0, SGI0, SGI141292, SGI141292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30893[] = {SGI141293, SGI141294, SGI141295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141296, SGI0, SGI0, SGI0, SGI141297, SGI141297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30894[] = {SGI141298, SGI141299, SGI141300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141301, SGI0, SGI0, SGI0, SGI141302, SGI141302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30895[] = {SGI141303, SGI0, SGI141304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141305, SGI0, SGI0, SGI0, SGI141306, SGI141306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30896[] = {SGI141307, SGI141308, SGI141309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141310, SGI0, SGI0, SGI0, SGI141311, SGI141311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30897[] = {SGI141312, SGI141313, SGI141314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141315, SGI0, SGI0, SGI0, SGI141316, SGI141316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30898[] = {SGI141317, SGI141318, SGI141319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141320, SGI0, SGI0, SGI0, SGI141321, SGI141321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30899[] = {SGI141322, SGI141323, SGI141324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141325, SGI0, SGI0, SGI0, SGI141326, SGI141326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30900[] = {SGI141327, SGI141328, SGI141329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141330, SGI0, SGI0, SGI0, SGI141331, SGI141331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30901[] = {SGI141332, SGI141333, SGI141334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141335, SGI0, SGI0, SGI0, SGI141336, SGI141336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30902[] = {SGI141337, SGI0, SGI141338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30903[] = {SGI141339, SGI0, SGI141340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141341, SGI0, SGI0, SGI0, SGI141342, SGI141342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30904[] = {SGI141343, SGI141344, SGI141345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141346, SGI0, SGI0, SGI0, SGI141347, SGI141347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30905[] = {SGI141348, SGI141349, SGI141350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141351, SGI0, SGI0, SGI0, SGI141352, SGI141352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30906[] = {SGI141353, SGI141354, SGI141355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141356, SGI0, SGI0, SGI0, SGI141357, SGI141357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30907[] = {SGI141358, SGI141359, SGI141360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141361, SGI0, SGI0, SGI0, SGI141362, SGI141362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30908[] = {SGI141363, SGI141364, SGI141365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141366, SGI0, SGI0, SGI0, SGI141367, SGI141367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30909[] = {SGI141368, SGI141369, SGI141370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141371, SGI0, SGI0, SGI0, SGI141372, SGI141372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30910[] = {SGI141373, SGI0, SGI141374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30911[] = {SGI141375, SGI0, SGI141376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30912[] = {SGI141377, SGI0, SGI141378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141381, SGI141383, SGI141383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30913[] = {SGI141379, SGI0, SGI141380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141382, SGI0, SGI0, SGI0, SGI141384, SGI141384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30914[] = {SGI141385, SGI141386, SGI141387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141388, SGI0, SGI0, SGI0, SGI141389, SGI141389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30915[] = {SGI141390, SGI141391, SGI141392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141393, SGI0, SGI0, SGI0, SGI141394, SGI141394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30916[] = {SGI141395, SGI0, SGI141396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141397, SGI141398, SGI141398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30917[] = {SGI141399, SGI0, SGI141400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141401, SGI141402, SGI141402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30918[] = {SGI141403, SGI0, SGI141404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141405, SGI141406, SGI141406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30919[] = {SGI141407, SGI0, SGI141408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141409, SGI141410, SGI141410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30920[] = {SGI141411, SGI0, SGI141412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141413, SGI141414, SGI141414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30921[] = {SGI141415, SGI0, SGI141416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141417, SGI141418, SGI141418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30922[] = {SGI141419, SGI0, SGI141420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141421, SGI141422, SGI141422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30923[] = {SGI141423, SGI0, SGI141424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141425, SGI141426, SGI141426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30924[] = {SGI141427, SGI0, SGI141428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141429, SGI141430, SGI141430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30925[] = {SGI141431, SGI0, SGI141432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141433, SGI141434, SGI141434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30926[] = {SGI141435, SGI0, SGI141436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141437, SGI141438, SGI141438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30927[] = {SGI141439, SGI0, SGI141440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141441, SGI141442, SGI141442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30928[] = {SGI141443, SGI0, SGI141444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141445, SGI141446, SGI141446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30929[] = {SGI141447, SGI0, SGI141448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141449, SGI141450, SGI141450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30930[] = {SGI141451, SGI0, SGI141452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141453, SGI141454, SGI141454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30931[] = {SGI141455, SGI0, SGI141456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141457, SGI141458, SGI141458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30932[] = {SGI141459, SGI141460, SGI141461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141462, SGI0, SGI0, SGI0, SGI141463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30933[] = {SGI141464, SGI0, SGI141465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30934[] = {SGI141466, SGI141467, SGI141468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141469, SGI0, SGI0, SGI0, SGI141470, SGI141470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30935[] = {SGI141471, SGI0, SGI141472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141473, SGI0, SGI0, SGI0, SGI141474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30936[] = {SGI141475, SGI141476, SGI141477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141478, SGI141478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30937[] = {SGI141479, SGI0, SGI141480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141481, SGI141482, SGI141482, SGI0, SGI0, NULL};
const Segment * const SG30938[] = {SGI141483, SGI0, SGI141484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141485, SGI141486, SGI141486, SGI0, SGI0, NULL};
const Segment * const SG30939[] = {SGI141487, SGI141488, SGI141489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141490, SGI0, SGI0, SGI0, SGI141491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30940[] = {SGI141492, SGI0, SGI141493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141494, SGI141495, SGI141495, SGI0, SGI0, NULL};
const Segment * const SG30941[] = {SGI141496, SGI141497, SGI141498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141499, SGI0, SGI0, SGI0, SGI141500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30942[] = {SGI141501, SGI141502, SGI141503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141504, SGI0, SGI0, SGI0, SGI141505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30943[] = {SGI141506, SGI0, SGI141507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30944[] = {SGI141508, SGI0, SGI141509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141510, SGI141511, SGI141511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30945[] = {SGI141512, SGI0, SGI141513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141514, SGI141515, SGI141515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30946[] = {SGI141516, SGI0, SGI141517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141518, SGI141519, SGI141519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30947[] = {SGI141520, SGI0, SGI141521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141522, SGI141523, SGI141523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30948[] = {SGI141524, SGI0, SGI141525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141526, SGI141527, SGI141527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30949[] = {SGI141528, SGI0, SGI141529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141530, SGI141531, SGI141531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30950[] = {SGI141532, SGI0, SGI141533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141534, SGI141535, SGI141535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30951[] = {SGI141536, SGI0, SGI141537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141538, SGI141539, SGI141539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30952[] = {SGI141540, SGI0, SGI141541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141542, SGI141543, SGI141543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30953[] = {SGI141544, SGI0, SGI141545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141546, SGI141547, SGI141547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30954[] = {SGI141548, SGI0, SGI141549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141550, SGI141551, SGI141551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30955[] = {SGI141552, SGI0, SGI141553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141554, SGI141555, SGI141555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30956[] = {SGI141556, SGI0, SGI141557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141559, SGI141560, SGI141560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30957[] = {SGI141558, SGI0, SGI141561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30958[] = {SGI141562, SGI0, SGI141563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141567, SGI141569, SGI141569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30959[] = {SGI141564, SGI141565, SGI141566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141568, SGI0, SGI0, SGI0, SGI141570, SGI141570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30960[] = {SGI141571, SGI141572, SGI141573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141574, SGI141574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30961[] = {SGI141575, SGI0, SGI141576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141577, SGI0, SGI0, SGI0, SGI141578, SGI141578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30962[] = {SGI141579, SGI0, SGI141580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141581, SGI141582, SGI141582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30963[] = {SGI141583, SGI0, SGI141584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141585, SGI141586, SGI141586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30964[] = {SGI141587, SGI0, SGI141588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141589, SGI141590, SGI141590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30965[] = {SGI141591, SGI0, SGI141592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141593, SGI141594, SGI141594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30966[] = {SGI141595, SGI0, SGI141596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141597, SGI141598, SGI141598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30967[] = {SGI141599, SGI0, SGI141600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141601, SGI141602, SGI141602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30968[] = {SGI141603, SGI0, SGI141604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141605, SGI141606, SGI141606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30969[] = {SGI141607, SGI0, SGI141608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141609, SGI141610, SGI141610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30970[] = {SGI141611, SGI0, SGI141612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141613, SGI141614, SGI141614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30971[] = {SGI141615, SGI0, SGI141616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141617, SGI141618, SGI141618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30972[] = {SGI141619, SGI0, SGI141620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141621, SGI141622, SGI141622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30973[] = {SGI141623, SGI0, SGI141624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141625, SGI141626, SGI141626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30974[] = {SGI141627, SGI0, SGI141628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141629, SGI141630, SGI141630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30975[] = {SGI141631, SGI0, SGI141632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141633, SGI141634, SGI141634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30976[] = {SGI141635, SGI141636, SGI141637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141638, SGI0, SGI0, SGI0, SGI141639, SGI141639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30977[] = {SGI141640, SGI0, SGI141641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30978[] = {SGI141642, SGI141643, SGI141644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141645, SGI0, SGI0, SGI0, SGI141646, SGI141646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30979[] = {SGI141647, SGI141648, SGI141649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141650, SGI0, SGI0, SGI0, SGI141651, SGI141651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30980[] = {SGI141652, SGI141653, SGI141654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141655, SGI0, SGI0, SGI0, SGI141656, SGI141656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30981[] = {SGI141657, SGI141658, SGI141659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141660, SGI0, SGI0, SGI0, SGI141661, SGI141661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30982[] = {SGI141662, SGI141663, SGI141664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141665, SGI0, SGI0, SGI0, SGI141666, SGI141666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30983[] = {SGI141667, SGI141668, SGI141669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141670, SGI0, SGI0, SGI0, SGI141671, SGI141671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30984[] = {SGI141672, SGI141673, SGI141674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141675, SGI0, SGI0, SGI0, SGI141676, SGI141676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30985[] = {SGI141677, SGI141678, SGI141679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141680, SGI0, SGI0, SGI0, SGI141681, SGI141681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30986[] = {SGI141682, SGI141683, SGI141684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141685, SGI0, SGI0, SGI0, SGI141686, SGI141686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30987[] = {SGI141687, SGI141688, SGI141689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141690, SGI0, SGI0, SGI0, SGI141691, SGI141691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30988[] = {SGI141692, SGI0, SGI141693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30989[] = {SGI141694, SGI141695, SGI141696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141697, SGI0, SGI0, SGI0, SGI141698, SGI141698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30990[] = {SGI141699, SGI0, SGI141700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141701, SGI0, SGI0, SGI0, SGI141702, SGI141702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30991[] = {SGI141703, SGI141704, SGI141705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141706, SGI0, SGI0, SGI0, SGI141707, SGI141707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30992[] = {SGI141708, SGI141709, SGI141710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141711, SGI0, SGI0, SGI0, SGI141712, SGI141712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30993[] = {SGI141713, SGI141714, SGI141715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141719, SGI0, SGI0, SGI0, SGI141720, SGI141720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30994[] = {SGI141716, SGI141717, SGI141718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141721, SGI0, SGI0, SGI0, SGI141722, SGI141722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30995[] = {SGI141723, SGI141724, SGI141725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141726, SGI0, SGI0, SGI0, SGI141727, SGI141727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30996[] = {SGI141728, SGI141729, SGI141730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141731, SGI0, SGI0, SGI0, SGI141732, SGI141732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30997[] = {SGI141733, SGI141734, SGI141735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141736, SGI0, SGI0, SGI0, SGI141737, SGI141737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30998[] = {SGI141738, SGI0, SGI141739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG30999[] = {SGI141740, SGI141741, SGI141742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141743, SGI0, SGI0, SGI0, SGI141744, SGI141744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31000[] = {SGI141745, SGI141746, SGI141747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141748, SGI0, SGI0, SGI0, SGI141749, SGI141749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31001[] = {SGI141750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31002[] = {SGI141751, SGI141752, SGI141753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141754, SGI0, SGI0, SGI0, SGI141755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31003[] = {SGI141756, SGI141757, SGI141758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141759, SGI0, SGI0, SGI0, SGI141760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31004[] = {SGI141761, SGI0, SGI141762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31005[] = {SGI141763, SGI141764, SGI141765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141766, SGI0, SGI0, SGI0, SGI141767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31006[] = {SGI141768, SGI141769, SGI141770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141771, SGI0, SGI0, SGI0, SGI141772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31007[] = {SGI141773, SGI0, SGI141774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141775, SGI141776, SGI141776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31008[] = {SGI141777, SGI0, SGI141778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141779, SGI141780, SGI141780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31009[] = {SGI141781, SGI0, SGI141782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141783, SGI141784, SGI141784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31010[] = {SGI141785, SGI0, SGI141786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141787, SGI141788, SGI141788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31011[] = {SGI141789, SGI0, SGI141790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141791, SGI141792, SGI141792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31012[] = {SGI141793, SGI0, SGI141794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141795, SGI141796, SGI141796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31013[] = {SGI141797, SGI0, SGI141798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141799, SGI141800, SGI141800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31014[] = {SGI141801, SGI0, SGI141802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141803, SGI141804, SGI141804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31015[] = {SGI141805, SGI141806, SGI141807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141808, SGI0, SGI0, SGI0, SGI141809, SGI141809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31016[] = {SGI141810, SGI0, SGI141811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141812, SGI141813, SGI141813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31017[] = {SGI141814, SGI0, SGI141815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31018[] = {SGI141816, SGI0, SGI141817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141819, SGI141821, SGI141821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31019[] = {SGI141818, SGI141820, SGI141822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141823, SGI0, SGI0, SGI0, SGI141824, SGI141824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31020[] = {SGI141825, SGI0, SGI141826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141829, SGI141830, SGI141830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31021[] = {SGI141827, SGI0, SGI141828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31022[] = {SGI141831, SGI0, SGI141832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141833, SGI141834, SGI141834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31023[] = {SGI141835, SGI141836, SGI141837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141838, SGI0, SGI0, SGI0, SGI141839, SGI141839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31024[] = {SGI141840, SGI141841, SGI141842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141843, SGI0, SGI0, SGI0, SGI141844, SGI141844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31025[] = {SGI141845, SGI141846, SGI141847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141848, SGI0, SGI0, SGI0, SGI141849, SGI141849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31026[] = {SGI141850, SGI141851, SGI141852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141853, SGI0, SGI0, SGI0, SGI141854, SGI141854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31027[] = {SGI141855, SGI141856, SGI141857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141858, SGI0, SGI0, SGI0, SGI141859, SGI141859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31028[] = {SGI141860, SGI0, SGI141861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31029[] = {SGI141862, SGI141863, SGI141864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141865, SGI0, SGI0, SGI0, SGI141866, SGI141866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31030[] = {SGI141867, SGI141868, SGI141869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141870, SGI0, SGI0, SGI0, SGI141871, SGI141871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31031[] = {SGI141872, SGI141873, SGI141874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141875, SGI0, SGI0, SGI0, SGI141876, SGI141876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31032[] = {SGI141877, SGI0, SGI141878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31033[] = {SGI141879, SGI141880, SGI141881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141882, SGI0, SGI0, SGI0, SGI141883, SGI141883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31034[] = {SGI141884, SGI141885, SGI141886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141887, SGI0, SGI0, SGI0, SGI141888, SGI141888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31035[] = {SGI141889, SGI141890, SGI141891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141892, SGI0, SGI0, SGI0, SGI141893, SGI141893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31036[] = {SGI141894, SGI141895, SGI141896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141897, SGI0, SGI0, SGI0, SGI141898, SGI141898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31037[] = {SGI141899, SGI0, SGI141900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31038[] = {SGI141901, SGI0, SGI141902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141903, SGI0, SGI0, SGI0, SGI141904, SGI141904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31039[] = {SGI141905, SGI0, SGI141906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31040[] = {SGI141907, SGI141908, SGI141909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141910, SGI0, SGI0, SGI0, SGI141911, SGI141911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31041[] = {SGI141912, SGI141913, SGI141914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141915, SGI0, SGI0, SGI0, SGI141916, SGI141916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31042[] = {SGI141917, SGI141918, SGI141919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141920, SGI0, SGI0, SGI0, SGI141921, SGI141921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31043[] = {SGI141922, SGI141923, SGI141924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141925, SGI0, SGI0, SGI0, SGI141926, SGI141926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31044[] = {SGI141927, SGI141928, SGI141929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141930, SGI0, SGI0, SGI0, SGI141931, SGI141931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31045[] = {SGI141932, SGI0, SGI141933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31046[] = {SGI141934, SGI141935, SGI141936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141937, SGI141937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31047[] = {SGI141938, SGI141939, SGI141940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141941, SGI0, SGI0, SGI0, SGI141942, SGI141942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31048[] = {SGI141943, SGI141944, SGI141945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141946, SGI0, SGI0, SGI0, SGI141947, SGI141947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31049[] = {SGI141948, SGI141949, SGI141950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141951, SGI0, SGI0, SGI0, SGI141952, SGI141952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31050[] = {SGI141953, SGI141954, SGI141955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141956, SGI0, SGI0, SGI0, SGI141957, SGI141957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31051[] = {SGI141958, SGI141959, SGI141960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141961, SGI0, SGI0, SGI0, SGI141962, SGI141962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31052[] = {SGI141963, SGI141964, SGI141965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141966, SGI0, SGI0, SGI0, SGI141967, SGI141967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31053[] = {SGI141968, SGI0, SGI141969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31054[] = {SGI141970, SGI141971, SGI141972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141973, SGI0, SGI0, SGI0, SGI141974, SGI141974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31055[] = {SGI141975, SGI141976, SGI141977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141978, SGI0, SGI0, SGI0, SGI141979, SGI141979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31056[] = {SGI141980, SGI141981, SGI141982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141983, SGI0, SGI0, SGI0, SGI141984, SGI141984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31057[] = {SGI141985, SGI0, SGI141986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31058[] = {SGI141987, SGI141988, SGI141989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141990, SGI0, SGI0, SGI0, SGI141991, SGI141991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31059[] = {SGI141992, SGI141993, SGI141994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI141995, SGI0, SGI0, SGI0, SGI141996, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31060[] = {SGI141997, SGI141998, SGI141999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142000, SGI0, SGI0, SGI0, SGI142001, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31061[] = {SGI142002, SGI142003, SGI142004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142005, SGI0, SGI0, SGI0, SGI142006, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31062[] = {SGI142007, SGI142008, SGI142009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142010, SGI0, SGI0, SGI0, SGI142011, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31063[] = {SGI142012, SGI142013, SGI142014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142015, SGI0, SGI0, SGI0, SGI142016, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31064[] = {SGI142017, SGI142018, SGI142019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142020, SGI0, SGI0, SGI0, SGI142021, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31065[] = {SGI142022, SGI142023, SGI142024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142025, SGI0, SGI0, SGI0, SGI142026, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31066[] = {SGI142027, SGI142028, SGI142029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142030, SGI0, SGI0, SGI0, SGI142031, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31067[] = {SGI142032, SGI142033, SGI142034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142035, SGI0, SGI0, SGI0, SGI142036, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31068[] = {SGI142037, SGI142038, SGI142039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142040, SGI0, SGI0, SGI0, SGI142041, SGI142041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31069[] = {SGI142042, SGI142043, SGI142044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142045, SGI142045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31070[] = {SGI142046, SGI142047, SGI142048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142049, SGI0, SGI0, SGI0, SGI142050, SGI142050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31071[] = {SGI142051, SGI0, SGI142052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31072[] = {SGI142053, SGI142054, SGI142055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142056, SGI0, SGI0, SGI0, SGI142057, SGI142057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31073[] = {SGI142058, SGI142059, SGI142060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142061, SGI0, SGI0, SGI0, SGI142062, SGI142062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31074[] = {SGI142063, SGI142064, SGI142065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142066, SGI0, SGI0, SGI0, SGI142067, SGI142067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31075[] = {SGI142068, SGI142069, SGI142070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142071, SGI0, SGI0, SGI0, SGI142072, SGI142072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31076[] = {SGI142073, SGI142074, SGI142075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142076, SGI0, SGI0, SGI0, SGI142077, SGI142077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31077[] = {SGI142078, SGI142079, SGI142080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142081, SGI0, SGI0, SGI0, SGI142082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31078[] = {SGI142083, SGI142084, SGI142085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142086, SGI0, SGI0, SGI0, SGI142087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31079[] = {SGI142088, SGI0, SGI142089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31080[] = {SGI142090, SGI0, SGI142091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31081[] = {SGI142092, SGI142093, SGI142094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142095, SGI0, SGI0, SGI0, SGI142096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31082[] = {SGI142097, SGI142098, SGI142099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142100, SGI0, SGI0, SGI0, SGI142101, SGI142101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31083[] = {SGI142102, SGI142103, SGI142104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142106, SGI0, SGI0, SGI0, SGI142108, SGI142108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31084[] = {SGI142105, SGI0, SGI142107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142109, SGI142110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31085[] = {SGI142111, SGI0, SGI142112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142113, SGI142114, SGI142114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31086[] = {SGI142115, SGI0, SGI142116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31087[] = {SGI142117, SGI142118, SGI142119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142120, SGI142121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31088[] = {SGI142122, SGI0, SGI142123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31089[] = {SGI142124, SGI0, SGI142125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142126, SGI142127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31090[] = {SGI142128, SGI142129, SGI142130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142131, SGI0, SGI0, SGI0, SGI142132, SGI142132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31091[] = {SGI142133, SGI142134, SGI142135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142136, SGI0, SGI0, SGI0, SGI142137, SGI142137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31092[] = {SGI142138, SGI142139, SGI142140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142141, SGI0, SGI0, SGI0, SGI142142, SGI142142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31093[] = {SGI142143, SGI142144, SGI142145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142146, SGI0, SGI0, SGI0, SGI142147, SGI142147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31094[] = {SGI142148, SGI142149, SGI142150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142151, SGI0, SGI0, SGI0, SGI142152, SGI142152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31095[] = {SGI142153, SGI142154, SGI142155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142156, SGI0, SGI0, SGI0, SGI142157, SGI142157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31096[] = {SGI142158, SGI0, SGI142159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31097[] = {SGI142160, SGI142161, SGI142162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142163, SGI0, SGI0, SGI0, SGI142164, SGI142164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31098[] = {SGI142165, SGI142166, SGI142167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142168, SGI0, SGI0, SGI0, SGI142169, SGI142169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31099[] = {SGI142170, SGI142171, SGI142172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142173, SGI0, SGI0, SGI0, SGI142174, SGI142174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31100[] = {SGI142175, SGI142176, SGI142177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142178, SGI0, SGI0, SGI0, SGI142179, SGI142179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31101[] = {SGI142180, SGI142181, SGI142182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142183, SGI0, SGI0, SGI0, SGI142184, SGI142184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31102[] = {SGI142185, SGI142186, SGI142187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142188, SGI0, SGI0, SGI0, SGI142189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31103[] = {SGI142190, SGI142191, SGI142192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142193, SGI0, SGI0, SGI0, SGI142194, SGI142194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31104[] = {SGI142195, SGI142196, SGI142197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142198, SGI142198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31105[] = {SGI142199, SGI142200, SGI142201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142202, SGI0, SGI0, SGI0, SGI142205, SGI142205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31106[] = {SGI142203, SGI142204, SGI142206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142207, SGI0, SGI0, SGI0, SGI142208, SGI142208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31107[] = {SGI142209, SGI142210, SGI142211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142212, SGI0, SGI0, SGI0, SGI142214, SGI142214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31108[] = {SGI142213, SGI0, SGI142215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31109[] = {SGI142216, SGI142217, SGI142218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142219, SGI0, SGI0, SGI0, SGI142220, SGI142220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31110[] = {SGI142221, SGI142222, SGI142223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142224, SGI0, SGI0, SGI0, SGI142225, SGI142225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31111[] = {SGI142226, SGI0, SGI142227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142228, SGI0, SGI0, SGI0, SGI142229, SGI142229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31112[] = {SGI142230, SGI142231, SGI142232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142233, SGI0, SGI0, SGI0, SGI142234, SGI142234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31113[] = {SGI142235, SGI142236, SGI142237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142238, SGI0, SGI0, SGI0, SGI142239, SGI142239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31114[] = {SGI142240, SGI142241, SGI142242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142243, SGI0, SGI0, SGI0, SGI142244, SGI142244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31115[] = {SGI142245, SGI142246, SGI142247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142248, SGI0, SGI0, SGI0, SGI142249, SGI142249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31116[] = {SGI142250, SGI142251, SGI142252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142253, SGI0, SGI0, SGI0, SGI142254, SGI142254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31117[] = {SGI142255, SGI142256, SGI142257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142258, SGI0, SGI0, SGI0, SGI142259, SGI142259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31118[] = {SGI142260, SGI142261, SGI142262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142263, SGI0, SGI0, SGI0, SGI142264, SGI142264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31119[] = {SGI142265, SGI142266, SGI142267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142268, SGI0, SGI0, SGI0, SGI142269, SGI142269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31120[] = {SGI142270, SGI0, SGI142271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142272, SGI142273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31121[] = {SGI142274, SGI0, SGI142275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142276, SGI142277, SGI142277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31122[] = {SGI142278, SGI0, SGI142279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142280, SGI142281, SGI142281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31123[] = {SGI142282, SGI0, SGI142283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142284, SGI142285, SGI142285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31124[] = {SGI142286, SGI0, SGI142287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142288, SGI142289, SGI142289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31125[] = {SGI142290, SGI0, SGI142291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142292, SGI142293, SGI142293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31126[] = {SGI142294, SGI0, SGI142295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142296, SGI142297, SGI142297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31127[] = {SGI142298, SGI0, SGI142299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142300, SGI142301, SGI142301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31128[] = {SGI142302, SGI0, SGI142303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142304, SGI142305, SGI142305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31129[] = {SGI142306, SGI0, SGI142307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142308, SGI142309, SGI142309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31130[] = {SGI142310, SGI0, SGI142311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142312, SGI142313, SGI142313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31131[] = {SGI142314, SGI0, SGI142315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142316, SGI142317, SGI142317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31132[] = {SGI142318, SGI0, SGI142319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142320, SGI142321, SGI142321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31133[] = {SGI142322, SGI0, SGI142323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142324, SGI142325, SGI142325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31134[] = {SGI142326, SGI0, SGI142327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142328, SGI142329, SGI142329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31135[] = {SGI142330, SGI0, SGI142331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142332, SGI142333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31136[] = {SGI142334, SGI0, SGI142335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142336, SGI142337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31137[] = {SGI142338, SGI0, SGI142339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142340, SGI142341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31138[] = {SGI142342, SGI0, SGI142343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142344, SGI142345, SGI142345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31139[] = {SGI142346, SGI0, SGI142347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142348, SGI142349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31140[] = {SGI142350, SGI142351, SGI142352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142353, SGI0, SGI0, SGI0, SGI142355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31141[] = {SGI142354, SGI0, SGI142356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142357, SGI142358, SGI142358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31142[] = {SGI142359, SGI142360, SGI142361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142362, SGI0, SGI0, SGI0, SGI142363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31143[] = {SGI142364, SGI0, SGI142365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142368, SGI142369, SGI142369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31144[] = {SGI142366, SGI0, SGI142367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31145[] = {SGI142370, SGI142371, SGI142372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142373, SGI0, SGI0, SGI0, SGI142374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31146[] = {SGI142375, SGI0, SGI142376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142377, SGI0, SGI0, SGI0, SGI142378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31147[] = {SGI142379, SGI0, SGI142380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142381, SGI142382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31148[] = {SGI142383, SGI0, SGI142384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142385, SGI142386, SGI142386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31149[] = {SGI142387, SGI142388, SGI142389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142390, SGI0, SGI0, SGI0, SGI142391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31150[] = {SGI142392, SGI0, SGI142393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142397, SGI142398, SGI142398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31151[] = {SGI142394, SGI142395, SGI142396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31152[] = {SGI142399, SGI142400, SGI142401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31153[] = {SGI142402, SGI0, SGI142403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142407, SGI142408, SGI142408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31154[] = {SGI142404, SGI142405, SGI142406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31155[] = {SGI142409, SGI0, SGI142410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142414, SGI142415, SGI142415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31156[] = {SGI142411, SGI142412, SGI142413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142416, SGI0, SGI0, SGI0, SGI142417, SGI142417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31157[] = {SGI142418, SGI142419, SGI142420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31158[] = {SGI142421, SGI0, SGI142422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142423, SGI142424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31159[] = {SGI142425, SGI142426, SGI142427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142428, SGI142428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31160[] = {SGI142429, SGI142431, SGI142432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142434, SGI0, SGI0, SGI0, SGI142435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31161[] = {SGI142430, SGI0, SGI142433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142436, SGI142437, SGI142437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31162[] = {SGI0, SGI142438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31163[] = {SGI142439, SGI142440, SGI142441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142443, SGI0, SGI0, SGI0, SGI142445, SGI142445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31164[] = {SGI142442, SGI0, SGI142444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142446, SGI142447, SGI142447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31165[] = {SGI142448, SGI142449, SGI142450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142451, SGI0, SGI0, SGI0, SGI142452, SGI142453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31166[] = {SGI142454, SGI0, SGI142455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142456, SGI142457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31167[] = {SGI142458, SGI142459, SGI142460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142461, SGI142461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31168[] = {SGI142462, SGI0, SGI142463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142464, SGI142465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31169[] = {SGI142466, SGI0, SGI142467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142468, SGI142469, SGI142469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31170[] = {SGI142470, SGI0, SGI142471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142472, SGI142473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31171[] = {SGI142474, SGI0, SGI142475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142476, SGI142477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31172[] = {SGI142478, SGI0, SGI142479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142480, SGI142481, SGI142481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31173[] = {SGI142482, SGI0, SGI142483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142484, SGI142485, SGI142485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31174[] = {SGI142486, SGI142487, SGI142488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142489, SGI0, SGI0, SGI0, SGI142490, SGI142490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31175[] = {SGI142491, SGI0, SGI142492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142493, SGI142494, SGI142494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31176[] = {SGI142495, SGI0, SGI142496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31177[] = {SGI142497, SGI142498, SGI142499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142500, SGI0, SGI0, SGI0, SGI142501, SGI142501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31178[] = {SGI142502, SGI142503, SGI142504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142505, SGI0, SGI0, SGI0, SGI142506, SGI142506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31179[] = {SGI142507, SGI142508, SGI142509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142510, SGI0, SGI0, SGI0, SGI142511, SGI142511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31180[] = {SGI142512, SGI142513, SGI142514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142515, SGI0, SGI0, SGI0, SGI142516, SGI142516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31181[] = {SGI142517, SGI142518, SGI142519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142520, SGI0, SGI0, SGI0, SGI142521, SGI142521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31182[] = {SGI142522, SGI142523, SGI142524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142525, SGI0, SGI0, SGI0, SGI142526, SGI142526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31183[] = {SGI142527, SGI142528, SGI142529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142530, SGI0, SGI0, SGI0, SGI142531, SGI142531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31184[] = {SGI142532, SGI142533, SGI142534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142535, SGI0, SGI0, SGI0, SGI142536, SGI142536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31185[] = {SGI142537, SGI142538, SGI142539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142540, SGI0, SGI0, SGI0, SGI142541, SGI142541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31186[] = {SGI142542, SGI142543, SGI142544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142545, SGI0, SGI0, SGI0, SGI142546, SGI142546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31187[] = {SGI142547, SGI142548, SGI142549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142550, SGI0, SGI0, SGI0, SGI142551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31188[] = {SGI142552, SGI0, SGI142553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31189[] = {SGI142554, SGI0, SGI142555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142556, SGI0, SGI0, SGI0, SGI142557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31190[] = {SGI142558, SGI142559, SGI142560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142561, SGI0, SGI0, SGI0, SGI142562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31191[] = {SGI142563, SGI0, SGI142564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31192[] = {SGI142565, SGI0, SGI142566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142567, SGI0, SGI0, SGI0, SGI142568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31193[] = {SGI142569, SGI142570, SGI142571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142572, SGI0, SGI0, SGI0, SGI142573, SGI142574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31194[] = {SGI142575, SGI142576, SGI142577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142578, SGI142578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31195[] = {SGI142579, SGI142580, SGI142581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31196[] = {SGI142582, SGI142583, SGI142584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31197[] = {SGI0, SGI142585, SGI142586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31198[] = {SGI142587, SGI142588, SGI142589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142590, SGI0, SGI0, SGI0, SGI142591, SGI142591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31199[] = {SGI142592, SGI142593, SGI142594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31200[] = {SGI142595, SGI142596, SGI142597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142598, SGI142598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31201[] = {SGI142599, SGI142600, SGI142601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142602, SGI0, SGI0, SGI0, SGI142603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31202[] = {SGI0, SGI142604, SGI142605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31203[] = {SGI142606, SGI142607, SGI142608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31204[] = {SGI142609, SGI142610, SGI142611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142612, SGI142613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31205[] = {SGI0, SGI142614, SGI142615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142616, SGI0, SGI0, SGI0, SGI142617, SGI142618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31206[] = {SGI142619, SGI0, SGI142620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31207[] = {SGI142621, SGI142622, SGI142623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31208[] = {SGI142624, SGI142625, SGI142626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142627, SGI0, SGI0, SGI0, SGI142628, SGI142629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31209[] = {SGI142630, SGI142631, SGI142632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31210[] = {SGI142633, SGI142634, SGI142635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31211[] = {SGI142636, SGI142637, SGI142638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31212[] = {SGI142639, SGI142640, SGI142641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31213[] = {SGI0, SGI142642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31214[] = {SGI142643, SGI142644, SGI142645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31215[] = {SGI142646, SGI142647, SGI142648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142649, SGI0, SGI0, SGI0, SGI142650, SGI142650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31216[] = {SGI142651, SGI142652, SGI142653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142654, SGI0, SGI0, SGI0, SGI142655, SGI142655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31217[] = {SGI142656, SGI142657, SGI142658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142659, SGI0, SGI0, SGI0, SGI142660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31218[] = {SGI142661, SGI142662, SGI142663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142664, SGI0, SGI0, SGI0, SGI142665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31219[] = {SGI142666, SGI142667, SGI142668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142669, SGI0, SGI0, SGI0, SGI142670, SGI142670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31220[] = {SGI142671, SGI142672, SGI142673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142674, SGI0, SGI0, SGI0, SGI142675, SGI142675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31221[] = {SGI142676, SGI142677, SGI142678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142679, SGI0, SGI0, SGI0, SGI142680, SGI142680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31222[] = {SGI142681, SGI0, SGI142682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31223[] = {SGI142683, SGI142684, SGI142685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142686, SGI0, SGI0, SGI0, SGI142687, SGI142687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31224[] = {SGI142688, SGI142689, SGI142690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142691, SGI0, SGI0, SGI0, SGI142692, SGI142692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31225[] = {SGI142693, SGI142694, SGI142695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142696, SGI0, SGI0, SGI0, SGI142697, SGI142697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31226[] = {SGI142698, SGI142699, SGI142700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142701, SGI0, SGI0, SGI0, SGI142702, SGI142702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31227[] = {SGI142703, SGI0, SGI142704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31228[] = {SGI142705, SGI142706, SGI142707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142708, SGI0, SGI0, SGI0, SGI142709, SGI142709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31229[] = {SGI142710, SGI142711, SGI142712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142713, SGI0, SGI0, SGI0, SGI142714, SGI142714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31230[] = {SGI142715, SGI0, SGI142716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142717, SGI142718, SGI142718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31231[] = {SGI142719, SGI0, SGI142720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142721, SGI142722, SGI142722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31232[] = {SGI142723, SGI0, SGI142724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142725, SGI142726, SGI142726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31233[] = {SGI142727, SGI0, SGI142728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142729, SGI142730, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31234[] = {SGI142731, SGI0, SGI142732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142733, SGI142734, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31235[] = {SGI142735, SGI142736, SGI142737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142738, SGI142739, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31236[] = {SGI142740, SGI142741, SGI142742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142743, SGI0, SGI0, SGI0, SGI142744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31237[] = {SGI142745, SGI0, SGI142746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31238[] = {SGI142747, SGI142748, SGI142749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142750, SGI0, SGI0, SGI0, SGI142751, SGI142751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31239[] = {SGI142752, SGI142753, SGI142754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142755, SGI0, SGI0, SGI0, SGI142756, SGI142756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31240[] = {SGI142757, SGI0, SGI142758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31241[] = {SGI142759, SGI0, SGI142760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142761, SGI0, SGI0, SGI0, SGI142762, SGI142762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31242[] = {SGI142763, SGI142764, SGI142765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31243[] = {SGI142766, SGI142767, SGI142768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31244[] = {SGI142769, SGI142770, SGI142771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142772, SGI0, SGI0, SGI0, SGI142773, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31245[] = {SGI142774, SGI142775, SGI142776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142777, SGI0, SGI0, SGI0, SGI142778, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31246[] = {SGI142779, SGI142780, SGI142781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142782, SGI0, SGI0, SGI0, SGI142783, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31247[] = {SGI142784, SGI142785, SGI142786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142787, SGI0, SGI0, SGI0, SGI142788, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31248[] = {SGI142789, SGI142790, SGI142791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142792, SGI0, SGI0, SGI0, SGI142793, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31249[] = {SGI142794, SGI142795, SGI142796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142797, SGI0, SGI0, SGI0, SGI142798, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31250[] = {SGI142799, SGI0, SGI142800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31251[] = {SGI142801, SGI0, SGI142802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142803, SGI0, SGI0, SGI0, SGI142804, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31252[] = {SGI142805, SGI0, SGI142806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142809, SGI142810, SGI142810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31253[] = {SGI142807, SGI0, SGI142808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142811, SGI142812, SGI142812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31254[] = {SGI142813, SGI0, SGI142814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142817, SGI142818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31255[] = {SGI142815, SGI0, SGI142816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142819, SGI142820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31256[] = {SGI142821, SGI0, SGI142822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142823, SGI142824, SGI142824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31257[] = {SGI142825, SGI0, SGI142826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142828, SGI142829, SGI142829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31258[] = {SGI142827, SGI0, SGI142830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142831, SGI142832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31259[] = {SGI142833, SGI0, SGI142834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142837, SGI142838, SGI142838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31260[] = {SGI142835, SGI0, SGI142836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142839, SGI142840, SGI142840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31261[] = {SGI142841, SGI0, SGI142842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142843, SGI142844, SGI142844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31262[] = {SGI142845, SGI0, SGI142846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142847, SGI142848, SGI142848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31263[] = {SGI142849, SGI0, SGI142850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142851, SGI142852, SGI142852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31264[] = {SGI142853, SGI0, SGI142854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142855, SGI142856, SGI142856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31265[] = {SGI142857, SGI0, SGI142858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142859, SGI142860, SGI142860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31266[] = {SGI142861, SGI0, SGI142862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142863, SGI142864, SGI142864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31267[] = {SGI142865, SGI0, SGI142866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142868, SGI142869, SGI142869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31268[] = {SGI142867, SGI142870, SGI142871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142872, SGI0, SGI0, SGI0, SGI142873, SGI142873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31269[] = {SGI142874, SGI0, SGI142875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31270[] = {SGI142876, SGI0, SGI142877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142878, SGI142879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31271[] = {SGI142880, SGI142881, SGI142882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142883, SGI0, SGI0, SGI0, SGI142884, SGI142884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31272[] = {SGI142885, SGI0, SGI142886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142888, SGI142889, SGI142889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31273[] = {SGI142887, SGI142890, SGI142891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142892, SGI0, SGI0, SGI0, SGI142893, SGI142893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31274[] = {SGI142894, SGI0, SGI142895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142896, SGI142897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31275[] = {SGI142898, SGI142899, SGI142900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142901, SGI0, SGI0, SGI0, SGI142902, SGI142902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31276[] = {SGI142904, SGI0, SGI142905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142906, SGI142908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31277[] = {SGI142907, SGI142909, SGI142910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142911, SGI0, SGI0, SGI0, SGI142912, SGI142912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31278[] = {SGI142914, SGI0, SGI142915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142916, SGI142917, SGI142917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31279[] = {SGI142918, SGI142919, SGI142920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142921, SGI0, SGI0, SGI0, SGI142922, SGI142922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31280[] = {SGI142923, SGI0, SGI142925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142926, SGI142927, SGI142927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31281[] = {SGI142928, SGI142929, SGI142930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142931, SGI0, SGI0, SGI0, SGI142932, SGI142932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31282[] = {SGI142934, SGI0, SGI142935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142936, SGI142937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31283[] = {SGI142938, SGI142939, SGI142940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142941, SGI0, SGI0, SGI0, SGI142942, SGI142942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31284[] = {SGI142944, SGI0, SGI142946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142949, SGI142951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31285[] = {SGI142945, SGI142947, SGI142948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142950, SGI0, SGI0, SGI0, SGI142952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31286[] = {SGI142954, SGI142955, SGI142956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142957, SGI0, SGI0, SGI0, SGI142959, SGI142959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31287[] = {SGI142958, SGI0, SGI142960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142962, SGI142963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31288[] = {SGI142964, SGI142965, SGI142966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142967, SGI0, SGI0, SGI0, SGI142968, SGI142968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31289[] = {SGI142970, SGI0, SGI142971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142972, SGI142973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31290[] = {SGI142974, SGI142975, SGI142976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142977, SGI0, SGI0, SGI0, SGI142978, SGI142978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31291[] = {SGI142980, SGI0, SGI142981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142982, SGI142983, SGI142983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31292[] = {SGI142984, SGI0, SGI142985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142986, SGI142987, SGI142987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31293[] = {SGI142988, SGI0, SGI142989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142990, SGI142991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31294[] = {SGI142992, SGI0, SGI142993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142994, SGI142995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31295[] = {SGI142996, SGI0, SGI142997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI142998, SGI142999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31296[] = {SGI143000, SGI0, SGI143001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143002, SGI143003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31297[] = {SGI143004, SGI0, SGI143005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143006, SGI143007, SGI143007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31298[] = {SGI143008, SGI0, SGI143009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143010, SGI143011, SGI143011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31299[] = {SGI143012, SGI0, SGI143013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143014, SGI143015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31300[] = {SGI143016, SGI0, SGI143017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143018, SGI143019, SGI143019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31301[] = {SGI143020, SGI143021, SGI143022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143023, SGI0, SGI0, SGI0, SGI143024, SGI143024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31302[] = {SGI143025, SGI143026, SGI143027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143028, SGI0, SGI0, SGI0, SGI143029, SGI143029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31303[] = {SGI143030, SGI0, SGI143031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31304[] = {SGI143032, SGI143033, SGI143034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143035, SGI0, SGI0, SGI0, SGI143036, SGI143036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31305[] = {SGI143037, SGI143038, SGI143039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143040, SGI0, SGI0, SGI0, SGI143041, SGI143041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31306[] = {SGI143042, SGI0, SGI143043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143044, SGI0, SGI0, SGI0, SGI143045, SGI143045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31307[] = {SGI143046, SGI143047, SGI143048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143049, SGI0, SGI0, SGI0, SGI143050, SGI143050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31308[] = {SGI143051, SGI143052, SGI143053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143054, SGI0, SGI0, SGI0, SGI143055, SGI143055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31309[] = {SGI143056, SGI0, SGI143057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31310[] = {SGI143058, SGI0, SGI143059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143060, SGI0, SGI0, SGI0, SGI143061, SGI143061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31311[] = {SGI143062, SGI0, SGI143063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143064, SGI143065, SGI143065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31312[] = {SGI143066, SGI0, SGI143067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143068, SGI143069, SGI143069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31313[] = {SGI143070, SGI0, SGI143071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143072, SGI143073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31314[] = {SGI143074, SGI0, SGI143075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143076, SGI143077, SGI143077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31315[] = {SGI143078, SGI0, SGI143079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143080, SGI143081, SGI143081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31316[] = {SGI143082, SGI0, SGI143083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143084, SGI143085, SGI143085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31317[] = {SGI143086, SGI0, SGI143087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143088, SGI143089, SGI143089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31318[] = {SGI143090, SGI0, SGI143091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143092, SGI143093, SGI143093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31319[] = {SGI143094, SGI0, SGI143095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143096, SGI143097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31320[] = {SGI143098, SGI0, SGI143099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143100, SGI143101, SGI143101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31321[] = {SGI143102, SGI0, SGI143103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143104, SGI143105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31322[] = {SGI143106, SGI0, SGI143107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143108, SGI143109, SGI143109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31323[] = {SGI143110, SGI0, SGI143111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143112, SGI143113, SGI143113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31324[] = {SGI143114, SGI0, SGI143115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143116, SGI143117, SGI143117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31325[] = {SGI143118, SGI0, SGI143119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143120, SGI143121, SGI143121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31326[] = {SGI143122, SGI0, SGI143123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143124, SGI143125, SGI143125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31327[] = {SGI143126, SGI0, SGI143127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143128, SGI143129, SGI143129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31328[] = {SGI143130, SGI0, SGI143131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143132, SGI143133, SGI143133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31329[] = {SGI143134, SGI0, SGI143135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143136, SGI143137, SGI143137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31330[] = {SGI143138, SGI0, SGI143139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143140, SGI143141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31331[] = {SGI143142, SGI0, SGI143143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143144, SGI143145, SGI143145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31332[] = {SGI143146, SGI0, SGI143147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143148, SGI143149, SGI143149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31333[] = {SGI143150, SGI0, SGI143151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143152, SGI143153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31334[] = {SGI143154, SGI0, SGI143155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143156, SGI143157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31335[] = {SGI143158, SGI0, SGI143159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143160, SGI143161, SGI143161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31336[] = {SGI143162, SGI0, SGI143163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143164, SGI143165, SGI143165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31337[] = {SGI143166, SGI0, SGI143167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143168, SGI143169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31338[] = {SGI143170, SGI0, SGI143171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143172, SGI143173, SGI143173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31339[] = {SGI143174, SGI0, SGI143175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143176, SGI143177, SGI143177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31340[] = {SGI143178, SGI0, SGI143179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143180, SGI143181, SGI143181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31341[] = {SGI143182, SGI0, SGI143183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143184, SGI143185, SGI143185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31342[] = {SGI143186, SGI0, SGI143187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143188, SGI143189, SGI143189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31343[] = {SGI143190, SGI0, SGI143191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143192, SGI143193, SGI143193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31344[] = {SGI143194, SGI0, SGI143195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143196, SGI143197, SGI143197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31345[] = {SGI0, SGI0, SGI143198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143199, SGI143200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31346[] = {SGI143201, SGI0, SGI143202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143203, SGI143204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31347[] = {SGI143205, SGI0, SGI143206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143207, SGI143208, SGI143208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31348[] = {SGI143209, SGI0, SGI143210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143211, SGI143212, SGI143212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31349[] = {SGI143213, SGI0, SGI143214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143215, SGI143216, SGI143216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31350[] = {SGI143217, SGI0, SGI143218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143219, SGI143220, SGI143220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31351[] = {SGI143221, SGI0, SGI143222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143223, SGI143224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31352[] = {SGI143225, SGI0, SGI143226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143227, SGI143228, SGI143228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31353[] = {SGI143229, SGI0, SGI143230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143231, SGI143232, SGI143232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31354[] = {SGI143233, SGI0, SGI143234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143235, SGI143236, SGI143236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31355[] = {SGI143237, SGI0, SGI143238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143239, SGI143240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31356[] = {SGI143241, SGI0, SGI143242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143243, SGI143244, SGI143244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31357[] = {SGI143245, SGI0, SGI143246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143247, SGI143248, SGI143248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31358[] = {SGI143249, SGI0, SGI143250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143251, SGI143252, SGI143252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31359[] = {SGI143253, SGI0, SGI143254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143255, SGI143256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31360[] = {SGI143257, SGI0, SGI143258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143259, SGI143260, SGI143260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31361[] = {SGI143261, SGI0, SGI143262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143263, SGI143264, SGI143264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31362[] = {SGI143265, SGI0, SGI143266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143267, SGI143268, SGI143268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31363[] = {SGI143269, SGI0, SGI143270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143271, SGI143272, SGI143272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31364[] = {SGI143273, SGI0, SGI143274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143275, SGI143276, SGI143276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31365[] = {SGI143277, SGI0, SGI143278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143279, SGI143280, SGI143280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31366[] = {SGI143281, SGI143282, SGI143283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143284, SGI0, SGI0, SGI0, SGI143285, SGI143285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31367[] = {SGI143286, SGI143287, SGI143288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143289, SGI0, SGI0, SGI0, SGI143290, SGI143290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31368[] = {SGI143291, SGI143292, SGI143293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143294, SGI0, SGI0, SGI0, SGI143295, SGI143295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31369[] = {SGI143296, SGI143297, SGI143298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143299, SGI0, SGI0, SGI0, SGI143300, SGI143300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31370[] = {SGI143301, SGI143302, SGI143303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143304, SGI0, SGI0, SGI0, SGI143305, SGI143305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31371[] = {SGI143306, SGI143307, SGI143308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143309, SGI0, SGI0, SGI0, SGI143310, SGI143310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31372[] = {SGI143311, SGI143312, SGI143313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143314, SGI0, SGI0, SGI0, SGI143315, SGI143315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31373[] = {SGI143316, SGI143317, SGI143318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143319, SGI0, SGI0, SGI0, SGI143320, SGI143320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31374[] = {SGI143321, SGI143322, SGI143323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143324, SGI0, SGI0, SGI0, SGI143325, SGI143325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31375[] = {SGI143326, SGI143327, SGI143328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143329, SGI0, SGI0, SGI0, SGI143330, SGI143330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31376[] = {SGI143331, SGI143332, SGI143333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143334, SGI0, SGI0, SGI0, SGI143335, SGI143335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31377[] = {SGI143336, SGI143337, SGI143338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143339, SGI0, SGI0, SGI0, SGI143340, SGI143340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31378[] = {SGI143341, SGI0, SGI143342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31379[] = {SGI143343, SGI143344, SGI143345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143346, SGI0, SGI0, SGI0, SGI143347, SGI143347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31380[] = {SGI143348, SGI143349, SGI143350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143351, SGI0, SGI0, SGI0, SGI143352, SGI143352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31381[] = {SGI143353, SGI143354, SGI143355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143356, SGI0, SGI0, SGI0, SGI143357, SGI143357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31382[] = {SGI143358, SGI0, SGI143359, SGI0, SGI143360, SGI143360, SGI0, NULL};
const Segment * const SG31383[] = {SGI143361, SGI0, SGI143362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143363, SGI143364, SGI143364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31384[] = {SGI143365, SGI0, SGI143366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143367, SGI143368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31385[] = {SGI143369, SGI0, SGI143370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143371, SGI143372, SGI143372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31386[] = {SGI143373, SGI0, SGI143374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143375, SGI143376, SGI143376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31387[] = {SGI143377, SGI143378, SGI143379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143380, SGI0, SGI0, SGI0, SGI0, SGI143381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31388[] = {SGI143382, SGI143383, SGI143384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143385, SGI0, SGI0, SGI0, SGI143386, SGI143386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31389[] = {SGI143387, SGI0, SGI143388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143389, SGI143390, SGI143390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31390[] = {SGI143391, SGI0, SGI143392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143393, SGI143394, SGI143394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31391[] = {SGI143395, SGI0, SGI143396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143397, SGI143398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31392[] = {SGI143399, SGI0, SGI143400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143401, SGI143402, SGI143402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31393[] = {SGI143403, SGI0, SGI143404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143405, SGI143406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31394[] = {SGI143407, SGI0, SGI143408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143409, SGI143410, SGI143410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31395[] = {SGI143411, SGI0, SGI143412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143413, SGI143414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31396[] = {SGI143415, SGI0, SGI143416, SGI0, SGI143417, SGI143417, SGI0, NULL};
const Segment * const SG31397[] = {SGI143418, SGI0, SGI143419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143420, SGI143421, SGI143421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31398[] = {SGI143422, SGI0, SGI143423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143424, SGI143425, SGI143425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31399[] = {SGI143426, SGI0, SGI143427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143428, SGI143429, SGI143429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31400[] = {SGI143430, SGI0, SGI143431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143432, SGI143433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31401[] = {SGI143434, SGI0, SGI143435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143436, SGI143437, SGI143437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31402[] = {SGI143438, SGI143439, SGI143440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143441, SGI0, SGI0, SGI0, SGI143442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31403[] = {SGI143443, SGI0, SGI143444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31404[] = {SGI143445, SGI143446, SGI143447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143448, SGI0, SGI0, SGI0, SGI143449, SGI143449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31405[] = {SGI143450, SGI0, SGI143451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143452, SGI0, SGI0, SGI0, SGI143453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31406[] = {SGI143454, SGI143455, SGI143456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143457, SGI0, SGI0, SGI0, SGI143458, SGI143458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31407[] = {SGI143459, SGI0, SGI143460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31408[] = {SGI143461, SGI143462, SGI143463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143464, SGI0, SGI0, SGI0, SGI143465, SGI143465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31409[] = {SGI143466, SGI0, SGI143467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31410[] = {SGI143468, SGI0, SGI143469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143470, SGI143471, SGI143471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31411[] = {SGI143472, SGI0, SGI143473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143474, SGI143475, SGI143475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31412[] = {SGI143476, SGI0, SGI143477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143478, SGI143479, SGI143479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31413[] = {SGI143480, SGI0, SGI143481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143482, SGI143483, SGI143483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31414[] = {SGI143484, SGI0, SGI143485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143486, SGI143487, SGI143487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31415[] = {SGI143488, SGI0, SGI143489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143490, SGI143491, SGI143491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31416[] = {SGI143492, SGI0, SGI143493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143494, SGI143495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31417[] = {SGI143496, SGI0, SGI143497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143498, SGI143499, SGI143499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31418[] = {SGI143500, SGI0, SGI143501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143502, SGI143503, SGI143503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31419[] = {SGI143504, SGI0, SGI143505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143506, SGI143507, SGI143507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31420[] = {SGI143508, SGI0, SGI143509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143510, SGI143511, SGI143511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31421[] = {SGI143512, SGI0, SGI143513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143514, SGI143515, SGI143515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31422[] = {SGI143516, SGI0, SGI143517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143518, SGI143519, SGI143519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31423[] = {SGI143520, SGI143521, SGI143522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143523, SGI0, SGI0, SGI0, SGI143524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31424[] = {SGI143525, SGI143526, SGI143527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143528, SGI0, SGI0, SGI0, SGI143529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31425[] = {SGI143530, SGI0, SGI143531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31426[] = {SGI143532, SGI143533, SGI143534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143535, SGI0, SGI0, SGI0, SGI143536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31427[] = {SGI143537, SGI0, SGI143538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143539, SGI0, SGI0, SGI0, SGI143540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31428[] = {SGI143541, SGI143542, SGI143543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143544, SGI0, SGI0, SGI0, SGI143545, SGI143545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31429[] = {SGI143546, SGI0, SGI143547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143548, SGI0, SGI0, SGI0, SGI143549, SGI143549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31430[] = {SGI143550, SGI0, SGI143551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31431[] = {SGI143552, SGI143553, SGI143554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143555, SGI0, SGI0, SGI0, SGI143556, SGI143556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31432[] = {SGI143557, SGI143558, SGI143559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143560, SGI0, SGI0, SGI0, SGI143561, SGI143561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31433[] = {SGI143562, SGI143563, SGI143564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143565, SGI0, SGI0, SGI0, SGI143566, SGI143566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31434[] = {SGI143567, SGI143568, SGI143569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143570, SGI0, SGI0, SGI0, SGI143571, SGI143571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31435[] = {SGI143572, SGI143573, SGI143574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143575, SGI0, SGI0, SGI0, SGI143576, SGI143576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31436[] = {SGI143577, SGI0, SGI143578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31437[] = {SGI143579, SGI0, SGI143580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143581, SGI143582, SGI143582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31438[] = {SGI143583, SGI0, SGI143584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143585, SGI143586, SGI143586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31439[] = {SGI143587, SGI0, SGI143588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143589, SGI143590, SGI143590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31440[] = {SGI143591, SGI0, SGI143592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143593, SGI143594, SGI143594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31441[] = {SGI143595, SGI0, SGI143596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143597, SGI143598, SGI143598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31442[] = {SGI143599, SGI0, SGI143600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143601, SGI143602, SGI143602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31443[] = {SGI143603, SGI0, SGI143604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143605, SGI143606, SGI143606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31444[] = {SGI143607, SGI143608, SGI143609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143610, SGI0, SGI0, SGI0, SGI143611, SGI143611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31445[] = {SGI143612, SGI0, SGI143613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31446[] = {SGI143614, SGI143615, SGI143616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143617, SGI0, SGI0, SGI0, SGI143618, SGI143618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31447[] = {SGI143619, SGI143620, SGI143621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143622, SGI0, SGI0, SGI0, SGI143623, SGI143623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31448[] = {SGI143624, SGI143625, SGI143626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143627, SGI0, SGI0, SGI0, SGI143628, SGI143628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31449[] = {SGI143629, SGI143630, SGI143631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143632, SGI0, SGI0, SGI0, SGI143633, SGI143633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31450[] = {SGI143634, SGI143635, SGI143636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143637, SGI0, SGI0, SGI0, SGI143638, SGI143638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31451[] = {SGI143639, SGI0, SGI143640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31452[] = {SGI143641, SGI143642, SGI143643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143644, SGI0, SGI0, SGI0, SGI143645, SGI143645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31453[] = {SGI143646, SGI143647, SGI143648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143649, SGI143649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31454[] = {SGI143650, SGI0, SGI143651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31455[] = {SGI143652, SGI143653, SGI143654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143655, SGI0, SGI0, SGI0, SGI143656, SGI143656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31456[] = {SGI143657, SGI143658, SGI143659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143660, SGI0, SGI0, SGI0, SGI143661, SGI143661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31457[] = {SGI143662, SGI143663, SGI143664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143665, SGI0, SGI0, SGI0, SGI143666, SGI143666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31458[] = {SGI143667, SGI0, SGI143668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143669, SGI143670, SGI143670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31459[] = {SGI143671, SGI0, SGI143672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143673, SGI143674, SGI143674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31460[] = {SGI143675, SGI0, SGI143676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143677, SGI143678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31461[] = {SGI143679, SGI0, SGI143680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143681, SGI143682, SGI143682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31462[] = {SGI143683, SGI0, SGI143684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143685, SGI143686, SGI143686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31463[] = {SGI143687, SGI0, SGI143688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143689, SGI143690, SGI143690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31464[] = {SGI143691, SGI0, SGI143692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143693, SGI143694, SGI143694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31465[] = {SGI143695, SGI0, SGI143696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143697, SGI143698, SGI143698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31466[] = {SGI143699, SGI0, SGI143700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143701, SGI143702, SGI143702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31467[] = {SGI143703, SGI0, SGI143704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143705, SGI143706, SGI143706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31468[] = {SGI143707, SGI0, SGI143708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143709, SGI143710, SGI143710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31469[] = {SGI143711, SGI143712, SGI143713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143714, SGI0, SGI0, SGI0, SGI143715, SGI143715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31470[] = {SGI143716, SGI0, SGI143717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31471[] = {SGI143718, SGI143719, SGI143720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143721, SGI0, SGI0, SGI0, SGI143722, SGI143722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31472[] = {SGI143723, SGI0, SGI143724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31473[] = {SGI143725, SGI143726, SGI143727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143728, SGI0, SGI0, SGI0, SGI143729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31474[] = {SGI143730, SGI0, SGI143731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31475[] = {SGI143732, SGI143733, SGI143734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143735, SGI0, SGI0, SGI0, SGI143736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31476[] = {SGI143737, SGI0, SGI143738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143739, SGI143740, SGI143740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31477[] = {SGI143741, SGI0, SGI143742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143743, SGI143744, SGI143744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31478[] = {SGI143745, SGI0, SGI143746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143747, SGI143748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31479[] = {SGI143749, SGI0, SGI143750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143751, SGI143752, SGI143752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31480[] = {SGI143753, SGI0, SGI143754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143755, SGI143756, SGI143756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31481[] = {SGI143757, SGI0, SGI143758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143759, SGI143760, SGI143760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31482[] = {SGI143761, SGI0, SGI143762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143763, SGI143764, SGI143764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31483[] = {SGI143765, SGI0, SGI143766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143767, SGI143768, SGI143768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31484[] = {SGI143769, SGI0, SGI143770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143771, SGI143772, SGI143772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31485[] = {SGI143773, SGI143774, SGI143775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143776, SGI0, SGI0, SGI0, SGI143777, SGI143777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31486[] = {SGI143778, SGI0, SGI143779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31487[] = {SGI143780, SGI143781, SGI143782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143783, SGI0, SGI0, SGI0, SGI143784, SGI143784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31488[] = {SGI143785, SGI0, SGI143786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143787, SGI0, SGI0, SGI0, SGI143788, SGI143788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31489[] = {SGI143789, SGI143790, SGI143791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143792, SGI0, SGI0, SGI0, SGI143793, SGI143793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31490[] = {SGI143794, SGI0, SGI143795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31491[] = {SGI143796, SGI143797, SGI143798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143799, SGI0, SGI0, SGI0, SGI143800, SGI143800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31492[] = {SGI143801, SGI143802, SGI143803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143804, SGI143804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31493[] = {SGI143805, SGI143806, SGI143807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143808, SGI0, SGI0, SGI0, SGI143809, SGI143809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31494[] = {SGI143810, SGI143811, SGI143812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143813, SGI0, SGI0, SGI0, SGI143814, SGI143814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31495[] = {SGI143815, SGI143816, SGI143817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143818, SGI0, SGI0, SGI0, SGI143819, SGI143819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31496[] = {SGI143820, SGI143821, SGI143822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143823, SGI0, SGI0, SGI0, SGI143824, SGI143824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31497[] = {SGI143825, SGI143826, SGI143827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143828, SGI0, SGI0, SGI0, SGI143829, SGI143829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31498[] = {SGI143830, SGI0, SGI143831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31499[] = {SGI143832, SGI0, SGI143833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31500[] = {SGI143834, SGI143835, SGI143836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143837, SGI0, SGI0, SGI0, SGI143838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31501[] = {SGI143839, SGI143840, SGI143841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143845, SGI0, SGI0, SGI0, SGI143846, SGI143846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31502[] = {SGI143842, SGI143843, SGI143844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143847, SGI0, SGI0, SGI0, SGI143848, SGI143848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31503[] = {SGI143849, SGI143850, SGI143851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143852, SGI0, SGI0, SGI0, SGI143853, SGI143853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31504[] = {SGI143854, SGI0, SGI143855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143856, SGI0, SGI0, SGI0, SGI143857, SGI143857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31505[] = {SGI143858, SGI143859, SGI143860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143861, SGI143861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31506[] = {SGI143862, SGI143863, SGI143864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143865, SGI0, SGI0, SGI0, SGI143866, SGI143866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31507[] = {SGI143867, SGI143868, SGI143869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143870, SGI0, SGI0, SGI0, SGI143871, SGI143871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31508[] = {SGI143872, SGI0, SGI143873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143874, SGI143875, SGI143875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31509[] = {SGI143876, SGI0, SGI143877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143878, SGI143879, SGI143879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31510[] = {SGI143880, SGI0, SGI143881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143882, SGI143883, SGI143883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31511[] = {SGI143884, SGI0, SGI143885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143886, SGI143887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31512[] = {SGI143888, SGI0, SGI143889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143890, SGI143891, SGI143891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31513[] = {SGI143892, SGI0, SGI143893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143894, SGI143895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31514[] = {SGI143896, SGI0, SGI143897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143898, SGI143899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31515[] = {SGI143900, SGI0, SGI143901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143902, SGI143903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31516[] = {SGI143904, SGI0, SGI143905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143906, SGI143907, SGI143907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31517[] = {SGI143908, SGI0, SGI143909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143910, SGI143911, SGI143911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31518[] = {SGI143912, SGI143913, SGI143914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143915, SGI0, SGI0, SGI0, SGI143916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31519[] = {SGI143917, SGI143918, SGI143919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143920, SGI0, SGI0, SGI0, SGI143921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31520[] = {SGI143922, SGI0, SGI143923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31521[] = {SGI143924, SGI0, SGI143925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31522[] = {SGI143926, SGI0, SGI143927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143928, SGI143929, SGI143929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31523[] = {SGI143930, SGI0, SGI143931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143932, SGI143933, SGI143933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31524[] = {SGI143934, SGI0, SGI143935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143936, SGI143937, SGI143937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31525[] = {SGI143938, SGI0, SGI143939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143940, SGI143941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31526[] = {SGI143942, SGI0, SGI143943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143944, SGI143945, SGI143945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31527[] = {SGI143946, SGI0, SGI143947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143948, SGI143949, SGI143949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31528[] = {SGI143950, SGI0, SGI143951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143952, SGI143953, SGI143953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31529[] = {SGI143954, SGI0, SGI143955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143956, SGI143957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31530[] = {SGI143958, SGI0, SGI143959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143960, SGI143961, SGI143961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31531[] = {SGI143962, SGI0, SGI143963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143964, SGI143965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31532[] = {SGI143966, SGI0, SGI143967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143968, SGI143969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31533[] = {SGI143970, SGI0, SGI143971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143972, SGI143973, SGI143973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31534[] = {SGI143974, SGI0, SGI143975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143976, SGI143977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31535[] = {SGI143978, SGI0, SGI143979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143980, SGI143981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31536[] = {SGI143982, SGI143983, SGI143984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143985, SGI0, SGI0, SGI0, SGI143986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31537[] = {SGI143987, SGI143988, SGI143989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143990, SGI0, SGI0, SGI0, SGI143991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31538[] = {SGI143992, SGI0, SGI143993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31539[] = {SGI143994, SGI143995, SGI143996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI143997, SGI0, SGI0, SGI0, SGI143998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31540[] = {SGI143999, SGI144000, SGI144001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144002, SGI0, SGI0, SGI0, SGI144003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31541[] = {SGI144004, SGI144005, SGI144006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144007, SGI0, SGI0, SGI0, SGI144008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31542[] = {SGI144009, SGI144010, SGI144011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144012, SGI0, SGI0, SGI0, SGI144013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31543[] = {SGI144014, SGI144015, SGI144016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144017, SGI0, SGI0, SGI0, SGI144018, SGI144018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31544[] = {SGI144019, SGI144020, SGI144021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144022, SGI0, SGI0, SGI0, SGI144023, SGI144023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31545[] = {SGI144024, SGI144025, SGI144026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144027, SGI0, SGI0, SGI0, SGI144028, SGI144028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31546[] = {SGI144029, SGI0, SGI144030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31547[] = {SGI144031, SGI144032, SGI144033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144034, SGI0, SGI0, SGI0, SGI144035, SGI144035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31548[] = {SGI144036, SGI144037, SGI144038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144039, SGI144039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31549[] = {SGI144040, SGI144041, SGI144042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144043, SGI144044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31550[] = {SGI144045, SGI144046, SGI144047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144048, SGI0, SGI0, SGI0, SGI144049, SGI144049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31551[] = {SGI144050, SGI144051, SGI144052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144053, SGI0, SGI0, SGI0, SGI144054, SGI144054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31552[] = {SGI144055, SGI144056, SGI144057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144058, SGI0, SGI0, SGI0, SGI144059, SGI144059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31553[] = {SGI144060, SGI144061, SGI144062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144063, SGI0, SGI0, SGI0, SGI144064, SGI144064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31554[] = {SGI144065, SGI144066, SGI144067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144068, SGI0, SGI0, SGI0, SGI144069, SGI144069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31555[] = {SGI144070, SGI0, SGI144071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31556[] = {SGI144072, SGI144073, SGI144074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144075, SGI0, SGI0, SGI0, SGI144076, SGI144076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31557[] = {SGI144077, SGI144078, SGI144079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144080, SGI0, SGI0, SGI0, SGI144081, SGI144081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31558[] = {SGI144082, SGI0, SGI144083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144084, SGI0, SGI0, SGI0, SGI144085, SGI144085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31559[] = {SGI144086, SGI144087, SGI144088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144089, SGI0, SGI0, SGI0, SGI144090, SGI144090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31560[] = {SGI144091, SGI144092, SGI144093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144094, SGI0, SGI0, SGI0, SGI144095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31561[] = {SGI144096, SGI0, SGI144097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31562[] = {SGI144098, SGI144099, SGI144100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144101, SGI0, SGI0, SGI0, SGI144102, SGI144102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31563[] = {SGI144103, SGI144104, SGI144105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144106, SGI0, SGI0, SGI0, SGI144107, SGI144107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31564[] = {SGI144108, SGI0, SGI144109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31565[] = {SGI144110, SGI144111, SGI144112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144113, SGI0, SGI0, SGI0, SGI144114, SGI144114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31566[] = {SGI144115, SGI144116, SGI144117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144118, SGI0, SGI0, SGI0, SGI144119, SGI144119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31567[] = {SGI144120, SGI144121, SGI144122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144123, SGI0, SGI0, SGI0, SGI144124, SGI144124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31568[] = {SGI144125, SGI144126, SGI144127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144128, SGI0, SGI0, SGI0, SGI144129, SGI144129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31569[] = {SGI144130, SGI0, SGI144131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31570[] = {SGI144132, SGI0, SGI144133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144134, SGI144135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31571[] = {SGI144136, SGI0, SGI144137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144138, SGI144139, SGI144139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31572[] = {SGI144140, SGI0, SGI144141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144142, SGI144143, SGI144143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31573[] = {SGI144144, SGI0, SGI144145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144146, SGI144147, SGI144147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31574[] = {SGI144148, SGI0, SGI144149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144150, SGI144151, SGI144151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31575[] = {SGI144152, SGI0, SGI144153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144154, SGI144155, SGI144155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31576[] = {SGI144156, SGI0, SGI144157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144158, SGI144159, SGI144159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31577[] = {SGI144160, SGI0, SGI144161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144162, SGI144163, SGI144163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31578[] = {SGI144164, SGI0, SGI144165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144166, SGI144167, SGI144167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31579[] = {SGI144168, SGI0, SGI144169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144170, SGI144171, SGI144171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31580[] = {SGI144172, SGI0, SGI144173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144174, SGI144175, SGI144175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31581[] = {SGI144176, SGI0, SGI144177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144178, SGI144179, SGI144179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31582[] = {SGI144180, SGI0, SGI144181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144182, SGI144183, SGI144183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31583[] = {SGI144184, SGI144185, SGI144186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144187, SGI0, SGI0, SGI0, SGI144188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31584[] = {SGI144189, SGI0, SGI144190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144191, SGI144192, SGI144192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31585[] = {SGI144193, SGI144194, SGI144195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144196, SGI0, SGI0, SGI144197, SGI144198, SGI144198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31586[] = {SGI144199, SGI144200, SGI144201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144202, SGI0, SGI0, SGI144203, SGI144204, SGI144204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31587[] = {SGI144205, SGI144206, SGI144207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144208, SGI0, SGI0, SGI144209, SGI144210, SGI144210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31588[] = {SGI144211, SGI144212, SGI144213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144214, SGI0, SGI0, SGI144215, SGI144216, SGI144216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31589[] = {SGI144217, SGI144218, SGI144219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144220, SGI0, SGI0, SGI144221, SGI144222, SGI144222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31590[] = {SGI144223, SGI144224, SGI144225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144226, SGI0, SGI0, SGI144227, SGI144228, SGI144228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31591[] = {SGI144229, SGI0, SGI144230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144231, SGI144232, SGI144232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31592[] = {SGI144233, SGI144234, SGI144235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144236, SGI0, SGI0, SGI144237, SGI144238, SGI144238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31593[] = {SGI144239, SGI144240, SGI144241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144242, SGI0, SGI0, SGI144243, SGI144244, SGI144244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31594[] = {SGI144245, SGI144246, SGI144247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144248, SGI0, SGI0, SGI144249, SGI144250, SGI144250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31595[] = {SGI144251, SGI144252, SGI144253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144254, SGI0, SGI0, SGI144255, SGI144256, SGI144256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31596[] = {SGI144257, SGI144258, SGI144259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144260, SGI0, SGI0, SGI144261, SGI144262, SGI144262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31597[] = {SGI144263, SGI144264, SGI144265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144266, SGI0, SGI0, SGI144267, SGI144268, SGI144268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31598[] = {SGI144269, SGI144270, SGI144271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144272, SGI0, SGI0, SGI144273, SGI144274, SGI144274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31599[] = {SGI144275, SGI144276, SGI144277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144278, SGI0, SGI0, SGI144279, SGI144280, SGI144280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31600[] = {SGI144281, SGI144282, SGI144283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144284, SGI0, SGI0, SGI144285, SGI144286, SGI144286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31601[] = {SGI144287, SGI144288, SGI144289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144290, SGI0, SGI0, SGI144291, SGI144292, SGI144292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31602[] = {SGI144293, SGI144294, SGI144295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144296, SGI0, SGI0, SGI144297, SGI144298, SGI144298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31603[] = {SGI144299, SGI144300, SGI144301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144302, SGI0, SGI0, SGI0, SGI144303, SGI144303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31604[] = {SGI144304, SGI144305, SGI144306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144307, SGI0, SGI0, SGI0, SGI144308, SGI144308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31605[] = {SGI144309, SGI144310, SGI144311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144312, SGI0, SGI0, SGI0, SGI144313, SGI144313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31606[] = {SGI144314, SGI144315, SGI144316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144317, SGI0, SGI0, SGI0, SGI144318, SGI144318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31607[] = {SGI144319, SGI144320, SGI144321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144322, SGI0, SGI0, SGI0, SGI144323, SGI144323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31608[] = {SGI144324, SGI144325, SGI144326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144327, SGI0, SGI0, SGI0, SGI144328, SGI144328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31609[] = {SGI144329, SGI144330, SGI144331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144332, SGI0, SGI0, SGI0, SGI144333, SGI144333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31610[] = {SGI144334, SGI0, SGI144335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31611[] = {SGI144336, SGI144337, SGI144338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144339, SGI0, SGI0, SGI0, SGI144340, SGI144340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31612[] = {SGI144341, SGI144342, SGI144343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144344, SGI0, SGI0, SGI0, SGI144345, SGI144345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31613[] = {SGI144346, SGI144347, SGI144348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144349, SGI0, SGI0, SGI0, SGI144350, SGI144350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31614[] = {SGI144351, SGI144352, SGI144353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144354, SGI0, SGI0, SGI0, SGI144355, SGI144355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31615[] = {SGI144356, SGI144357, SGI144358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144359, SGI0, SGI0, SGI0, SGI144360, SGI144360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31616[] = {SGI144361, SGI0, SGI144362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144363, SGI0, SGI0, SGI0, SGI144364, SGI144364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31617[] = {SGI144365, SGI0, SGI144366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144367, SGI144368, SGI144368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31618[] = {SGI144369, SGI0, SGI144370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144371, SGI144372, SGI144372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31619[] = {SGI144373, SGI0, SGI144374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144375, SGI144376, SGI144376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31620[] = {SGI144377, SGI144378, SGI144379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144380, SGI0, SGI0, SGI0, SGI144381, SGI144381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31621[] = {SGI144382, SGI144383, SGI144384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144385, SGI0, SGI0, SGI0, SGI144386, SGI144386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31622[] = {SGI144387, SGI0, SGI144388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31623[] = {SGI144389, SGI144390, SGI144391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144392, SGI0, SGI0, SGI0, SGI144393, SGI144393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31624[] = {SGI144394, SGI0, SGI144395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31625[] = {SGI144396, SGI0, SGI144397, SGI0, SGI144398, SGI144398, SGI0, NULL};
const Segment * const SG31626[] = {SGI144399, SGI0, SGI144401, SGI0, SGI144403, SGI144403, SGI0, NULL};
const Segment * const SG31627[] = {SGI144400, SGI0, SGI144402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31628[] = {SGI144404, SGI0, SGI144405, SGI0, SGI144406, SGI144406, SGI0, NULL};
const Segment * const SG31629[] = {SGI144407, SGI144408, SGI144409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144410, SGI144410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31630[] = {SGI144411, SGI144412, SGI144413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144414, SGI0, SGI0, SGI0, SGI144415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31631[] = {SGI144416, SGI0, SGI144417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144418, SGI144419, SGI144419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31632[] = {SGI144420, SGI144421, SGI144422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144423, SGI144423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31633[] = {SGI144424, SGI144425, SGI144426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144427, SGI0, SGI0, SGI0, SGI144428, SGI144428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31634[] = {SGI144429, SGI144430, SGI144431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144432, SGI0, SGI0, SGI0, SGI144433, SGI144433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31635[] = {SGI144434, SGI144435, SGI144436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144437, SGI0, SGI0, SGI0, SGI144438, SGI144438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31636[] = {SGI144439, SGI0, SGI144440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31637[] = {SGI144441, SGI144442, SGI144443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144444, SGI0, SGI0, SGI0, SGI144445, SGI144445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31638[] = {SGI144446, SGI0, SGI144447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144448, SGI144449, SGI144449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31639[] = {SGI144450, SGI0, SGI144451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144452, SGI144453, SGI144453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31640[] = {SGI144454, SGI144455, SGI144456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144457, SGI0, SGI0, SGI0, SGI144458, SGI144458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31641[] = {SGI144459, SGI144460, SGI144461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144462, SGI0, SGI0, SGI0, SGI144463, SGI144463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31642[] = {SGI144464, SGI144465, SGI144466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144467, SGI0, SGI0, SGI0, SGI144468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31643[] = {SGI144469, SGI0, SGI144470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31644[] = {SGI144471, SGI0, SGI144472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144473, SGI0, SGI0, SGI0, SGI144474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31645[] = {SGI144475, SGI0, SGI144476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31646[] = {SGI144477, SGI144478, SGI144479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144480, SGI0, SGI0, SGI0, SGI144481, SGI144481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31647[] = {SGI144482, SGI144483, SGI144484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144485, SGI0, SGI0, SGI0, SGI144486, SGI144486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31648[] = {SGI144487, SGI0, SGI144488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31649[] = {SGI144489, SGI0, SGI144490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144491, SGI0, SGI0, SGI0, SGI144492, SGI144492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31650[] = {SGI144493, SGI144494, SGI144495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144496, SGI0, SGI0, SGI0, SGI144497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31651[] = {SGI144498, SGI0, SGI144499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31652[] = {SGI144500, SGI144501, SGI144502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144503, SGI0, SGI0, SGI0, SGI144504, SGI144504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31653[] = {SGI144505, SGI0, SGI144506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31654[] = {SGI144507, SGI144508, SGI144509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144510, SGI0, SGI0, SGI0, SGI144511, SGI144511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31655[] = {SGI144512, SGI144513, SGI144514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144515, SGI0, SGI0, SGI0, SGI144516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31656[] = {SGI144517, SGI144518, SGI144519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144520, SGI0, SGI0, SGI0, SGI144521, SGI144521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31657[] = {SGI144524, SGI0, SGI144526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144528, SGI0, SGI0, SGI0, SGI144530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31658[] = {SGI144522, SGI144523, SGI144525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144527, SGI0, SGI0, SGI0, SGI144529, SGI144529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31659[] = {SGI144532, SGI0, SGI144535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31660[] = {SGI144531, SGI144533, SGI144534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144536, SGI0, SGI0, SGI0, SGI144537, SGI144537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31661[] = {SGI144538, SGI144539, SGI144540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144541, SGI144541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31662[] = {SGI144542, SGI144543, SGI144544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144545, SGI0, SGI0, SGI0, SGI144546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31663[] = {SGI144547, SGI144548, SGI144549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144550, SGI0, SGI0, SGI0, SGI144551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31664[] = {SGI144552, SGI144553, SGI144554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144555, SGI0, SGI0, SGI0, SGI144556, SGI144556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31665[] = {SGI144557, SGI144558, SGI144559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144560, SGI0, SGI0, SGI0, SGI144561, SGI144561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31666[] = {SGI144562, SGI0, SGI144563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31667[] = {SGI144564, SGI144565, SGI144566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144567, SGI0, SGI0, SGI0, SGI144568, SGI144568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31668[] = {SGI144569, SGI0, SGI144570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31669[] = {SGI144571, SGI0, SGI144572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144573, SGI0, SGI0, SGI0, SGI144574, SGI144574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31670[] = {SGI144575, SGI144576, SGI144577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144578, SGI0, SGI0, SGI0, SGI144579, SGI144579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31671[] = {SGI144580, SGI144581, SGI144582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144583, SGI0, SGI0, SGI0, SGI144584, SGI144584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31672[] = {SGI144585, SGI0, SGI144586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31673[] = {SGI144587, SGI144588, SGI144589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144590, SGI0, SGI0, SGI0, SGI144591, SGI144591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31674[] = {SGI144592, SGI0, SGI144593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31675[] = {SGI144594, SGI0, SGI144595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144596, SGI144597, SGI144597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31676[] = {SGI144598, SGI0, SGI144599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144600, SGI144601, SGI144601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31677[] = {SGI144602, SGI0, SGI144603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144604, SGI144605, SGI144605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31678[] = {SGI144606, SGI0, SGI144607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144608, SGI144609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31679[] = {SGI144610, SGI0, SGI144611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144612, SGI144613, SGI144613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31680[] = {SGI144614, SGI0, SGI144615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144616, SGI144617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31681[] = {SGI0, SGI0, SGI144618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144619, SGI144620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31682[] = {SGI0, SGI0, SGI144621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144622, SGI144623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31683[] = {SGI144624, SGI0, SGI144625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144626, SGI144627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31684[] = {SGI144628, SGI0, SGI144629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144630, SGI144631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31685[] = {SGI144632, SGI0, SGI144633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144634, SGI144635, SGI144635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31686[] = {SGI144636, SGI0, SGI144637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144638, SGI144639, SGI144639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31687[] = {SGI144640, SGI0, SGI144641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144642, SGI144643, SGI144643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31688[] = {SGI144644, SGI0, SGI144645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144646, SGI144647, SGI144647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31689[] = {SGI144648, SGI0, SGI144649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144650, SGI144651, SGI144651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31690[] = {SGI144652, SGI0, SGI144653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144656, SGI144657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31691[] = {SGI144654, SGI0, SGI144655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144658, SGI144659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31692[] = {SGI144660, SGI0, SGI144661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144664, SGI144665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31693[] = {SGI144662, SGI0, SGI144663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144666, SGI144667, SGI144667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31694[] = {SGI144668, SGI0, SGI144670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144672, SGI144673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31695[] = {SGI144669, SGI0, SGI144671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144674, SGI144675, SGI144675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31696[] = {SGI144676, SGI0, SGI144678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144680, SGI0, SGI0, SGI0, SGI144681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31697[] = {SGI144677, SGI0, SGI144679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144682, SGI144683, SGI144683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31698[] = {SGI144684, SGI144685, SGI144686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144687, SGI0, SGI0, SGI0, SGI144688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31699[] = {SGI144689, SGI144690, SGI144691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144692, SGI0, SGI0, SGI0, SGI144693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31700[] = {SGI144694, SGI0, SGI144695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144698, SGI144700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31701[] = {SGI144696, SGI144697, SGI144699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144701, SGI0, SGI0, SGI0, SGI144702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31702[] = {SGI144703, SGI144704, SGI144705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144708, SGI0, SGI0, SGI0, SGI144709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31703[] = {SGI144706, SGI0, SGI144707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144710, SGI144711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31704[] = {SGI144712, SGI0, SGI144713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31705[] = {SGI144714, SGI0, SGI144715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144716, SGI144717, SGI144717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31706[] = {SGI144718, SGI0, SGI144719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144720, SGI144721, SGI144721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31707[] = {SGI144722, SGI144724, SGI144725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144727, SGI0, SGI0, SGI0, SGI144729, SGI144729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31708[] = {SGI144723, SGI0, SGI144726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144728, SGI144730, SGI144730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31709[] = {SGI144731, SGI0, SGI144732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144733, SGI144734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31710[] = {SGI144735, SGI0, SGI144736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144737, SGI144738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31711[] = {SGI144739, SGI0, SGI144740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144741, SGI144742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31712[] = {SGI144743, SGI0, SGI144744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144745, SGI144746, SGI144746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31713[] = {SGI144747, SGI0, SGI144748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144749, SGI144750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31714[] = {SGI144751, SGI144752, SGI144753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144754, SGI0, SGI0, SGI0, SGI144755, SGI144755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31715[] = {SGI144756, SGI144757, SGI144758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144759, SGI0, SGI0, SGI0, SGI144760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31716[] = {SGI144761, SGI0, SGI144762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31717[] = {SGI144763, SGI144764, SGI144765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144766, SGI0, SGI0, SGI0, SGI144767, SGI144767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31718[] = {SGI144768, SGI144769, SGI144770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144771, SGI0, SGI0, SGI0, SGI144772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31719[] = {SGI144773, SGI0, SGI144774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144775, SGI0, SGI0, SGI0, SGI144776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31720[] = {SGI144777, SGI144778, SGI144779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144780, SGI0, SGI0, SGI0, SGI144781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31721[] = {SGI144782, SGI144783, SGI144784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144785, SGI0, SGI0, SGI0, SGI144786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31722[] = {SGI144787, SGI144788, SGI144789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144790, SGI0, SGI0, SGI0, SGI144791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31723[] = {SGI144792, SGI144793, SGI144794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144795, SGI0, SGI0, SGI0, SGI144796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31724[] = {SGI144797, SGI0, SGI144798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31725[] = {SGI144799, SGI144800, SGI144801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144802, SGI0, SGI0, SGI0, SGI144803, SGI144803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31726[] = {SGI144804, SGI144805, SGI144806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144807, SGI0, SGI0, SGI0, SGI144808, SGI144808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31727[] = {SGI144809, SGI0, SGI144810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31728[] = {SGI144811, SGI144812, SGI144813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144814, SGI0, SGI0, SGI0, SGI144815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31729[] = {SGI144816, SGI144817, SGI144818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144819, SGI0, SGI0, SGI0, SGI144820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31730[] = {SGI144821, SGI0, SGI144822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31731[] = {SGI144823, SGI0, SGI144824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144825, SGI144826, SGI144826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31732[] = {SGI144827, SGI0, SGI144828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144829, SGI144830, SGI144830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31733[] = {SGI144831, SGI0, SGI144832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144833, SGI144834, SGI144834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31734[] = {SGI144835, SGI0, SGI144836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144837, SGI144838, SGI144838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31735[] = {SGI144839, SGI0, SGI144840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144841, SGI144842, SGI144842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31736[] = {SGI144843, SGI144844, SGI144845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144846, SGI0, SGI0, SGI0, SGI144847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31737[] = {SGI144848, SGI0, SGI144849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144850, SGI144851, SGI144851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31738[] = {SGI144852, SGI0, SGI144853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144854, SGI144855, SGI144855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31739[] = {SGI144856, SGI144857, SGI144858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144859, SGI0, SGI0, SGI0, SGI144860, SGI144860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31740[] = {SGI144861, SGI144862, SGI144863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144864, SGI0, SGI0, SGI0, SGI144865, SGI144865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31741[] = {SGI144866, SGI144867, SGI144868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144869, SGI0, SGI0, SGI0, SGI144870, SGI144870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31742[] = {SGI144871, SGI0, SGI144872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31743[] = {SGI144873, SGI0, SGI144874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31744[] = {SGI144875, SGI144876, SGI144877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144878, SGI0, SGI0, SGI0, SGI144879, SGI144879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31745[] = {SGI144880, SGI144881, SGI144882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144883, SGI0, SGI0, SGI0, SGI144884, SGI144884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31746[] = {SGI144885, SGI144886, SGI144887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144888, SGI0, SGI0, SGI0, SGI144889, SGI144889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31747[] = {SGI144890, SGI144891, SGI144892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144893, SGI0, SGI0, SGI0, SGI144894, SGI144894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31748[] = {SGI144895, SGI144896, SGI144897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144898, SGI0, SGI0, SGI0, SGI144899, SGI144899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31749[] = {SGI144902, SGI144903, SGI144904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144906, SGI0, SGI0, SGI0, SGI144907, SGI144907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31750[] = {SGI144900, SGI144901, SGI144905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144908, SGI144908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31751[] = {SGI144909, SGI144910, SGI144911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144912, SGI0, SGI0, SGI0, SGI144913, SGI144913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31752[] = {SGI144914, SGI144915, SGI144916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144917, SGI0, SGI0, SGI0, SGI144918, SGI144918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31753[] = {SGI144919, SGI144920, SGI144921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144922, SGI0, SGI0, SGI0, SGI144923, SGI144923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31754[] = {SGI144924, SGI144925, SGI144926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144927, SGI0, SGI0, SGI0, SGI144928, SGI144928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31755[] = {SGI144929, SGI144930, SGI144931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144932, SGI0, SGI0, SGI0, SGI144933, SGI144933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31756[] = {SGI144934, SGI0, SGI144935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31757[] = {SGI144936, SGI144937, SGI144938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144939, SGI0, SGI0, SGI0, SGI144940, SGI144940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31758[] = {SGI144941, SGI144942, SGI144943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144944, SGI0, SGI0, SGI0, SGI144945, SGI144945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31759[] = {SGI144946, SGI144947, SGI144948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144949, SGI0, SGI0, SGI0, SGI144950, SGI144950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31760[] = {SGI144951, SGI144952, SGI144953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144954, SGI0, SGI0, SGI0, SGI144955, SGI144955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31761[] = {SGI144956, SGI144957, SGI144958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144959, SGI0, SGI0, SGI0, SGI144960, SGI144960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31762[] = {SGI144961, SGI144962, SGI144963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144964, SGI0, SGI0, SGI0, SGI144965, SGI144965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31763[] = {SGI144966, SGI144967, SGI144968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144969, SGI0, SGI0, SGI0, SGI144970, SGI144970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31764[] = {SGI144971, SGI144972, SGI144973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144974, SGI0, SGI0, SGI0, SGI144975, SGI144975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31765[] = {SGI144976, SGI144977, SGI144978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144979, SGI0, SGI0, SGI0, SGI144980, SGI144980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31766[] = {SGI144981, SGI144982, SGI144983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144984, SGI0, SGI0, SGI0, SGI144985, SGI144985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31767[] = {SGI144986, SGI0, SGI144987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31768[] = {SGI144988, SGI144989, SGI144990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144991, SGI0, SGI0, SGI0, SGI144992, SGI144992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31769[] = {SGI144993, SGI144994, SGI144995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI144996, SGI0, SGI0, SGI0, SGI144997, SGI144997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31770[] = {SGI144998, SGI144999, SGI145000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145001, SGI0, SGI0, SGI0, SGI145002, SGI145002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31771[] = {SGI145003, SGI145004, SGI145005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145006, SGI0, SGI0, SGI0, SGI145007, SGI145007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31772[] = {SGI145008, SGI145009, SGI145010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145011, SGI0, SGI0, SGI0, SGI145012, SGI145012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31773[] = {SGI145013, SGI0, SGI145014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31774[] = {SGI145015, SGI145016, SGI145017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145018, SGI0, SGI0, SGI0, SGI145019, SGI145019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31775[] = {SGI145020, SGI145021, SGI145022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145023, SGI0, SGI0, SGI0, SGI145024, SGI145024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31776[] = {SGI145026, SGI145029, SGI145030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145032, SGI0, SGI0, SGI0, SGI145034, SGI145034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31777[] = {SGI145025, SGI145027, SGI145028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145031, SGI0, SGI0, SGI0, SGI145033, SGI145033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31778[] = {SGI145035, SGI145037, SGI145039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145041, SGI0, SGI0, SGI0, SGI145043, SGI145043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31779[] = {SGI145036, SGI145038, SGI145040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145042, SGI0, SGI0, SGI0, SGI145044, SGI145044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31780[] = {SGI145045, SGI145046, SGI145048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145051, SGI0, SGI0, SGI0, SGI145053, SGI145053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31781[] = {SGI145047, SGI145049, SGI145050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145052, SGI0, SGI0, SGI0, SGI145054, SGI145054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31782[] = {SGI145055, SGI0, SGI145056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31783[] = {SGI145057, SGI0, SGI145058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145059, SGI145060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31784[] = {SGI145061, SGI0, SGI145062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145063, SGI145064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31785[] = {SGI145065, SGI0, SGI145066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145067, SGI145068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31786[] = {SGI145069, SGI145070, SGI145071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145072, SGI0, SGI0, SGI0, SGI145073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31787[] = {SGI145074, SGI145075, SGI145076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145077, SGI0, SGI0, SGI0, SGI145078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31788[] = {SGI145079, SGI0, SGI145080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31789[] = {SGI145081, SGI0, SGI145082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145083, SGI145084, SGI145084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31790[] = {SGI145085, SGI0, SGI145086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145087, SGI145088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31791[] = {SGI145089, SGI145090, SGI145091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145092, SGI0, SGI0, SGI0, SGI145093, SGI145093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31792[] = {SGI145094, SGI0, SGI145095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145096, SGI0, SGI0, SGI0, SGI145097, SGI145097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31793[] = {SGI145098, SGI0, SGI145099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31794[] = {SGI145100, SGI0, SGI145101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145102, SGI145103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31795[] = {SGI145104, SGI0, SGI145105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145106, SGI145107, SGI145107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31796[] = {SGI145108, SGI145109, SGI145110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145111, SGI0, SGI0, SGI0, SGI145112, SGI145112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31797[] = {SGI145113, SGI145114, SGI145115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145116, SGI0, SGI0, SGI0, SGI145117, SGI145117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31798[] = {SGI145118, SGI145119, SGI145120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145121, SGI0, SGI0, SGI0, SGI145122, SGI145122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31799[] = {SGI145123, SGI145124, SGI145125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145126, SGI0, SGI0, SGI0, SGI145127, SGI145127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31800[] = {SGI145128, SGI0, SGI145129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31801[] = {SGI145130, SGI145131, SGI145132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145133, SGI0, SGI0, SGI0, SGI145134, SGI145134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31802[] = {SGI145135, SGI145136, SGI145137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145138, SGI0, SGI0, SGI0, SGI145139, SGI145139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31803[] = {SGI145140, SGI145141, SGI145142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145143, SGI0, SGI0, SGI0, SGI145144, SGI145144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31804[] = {SGI145145, SGI145146, SGI145147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145148, SGI0, SGI0, SGI0, SGI145149, SGI145149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31805[] = {SGI145150, SGI145151, SGI145152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145153, SGI0, SGI0, SGI0, SGI145154, SGI145154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31806[] = {SGI145155, SGI145156, SGI145157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145158, SGI0, SGI0, SGI0, SGI145159, SGI145159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31807[] = {SGI145160, SGI145161, SGI145162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145163, SGI0, SGI0, SGI0, SGI145164, SGI145164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31808[] = {SGI145165, SGI145166, SGI145167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145168, SGI0, SGI0, SGI0, SGI145169, SGI145169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31809[] = {SGI145170, SGI145171, SGI145172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145173, SGI0, SGI0, SGI0, SGI145174, SGI145174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31810[] = {SGI145175, SGI145176, SGI145177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145178, SGI0, SGI0, SGI0, SGI145179, SGI145179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31811[] = {SGI145180, SGI145181, SGI145182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145183, SGI0, SGI0, SGI0, SGI145184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31812[] = {SGI145185, SGI0, SGI145186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31813[] = {SGI145187, SGI145188, SGI145189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145190, SGI0, SGI0, SGI0, SGI145191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31814[] = {SGI145192, SGI145193, SGI145194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145195, SGI0, SGI0, SGI0, SGI145196, SGI145196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31815[] = {SGI145197, SGI0, SGI145198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31816[] = {SGI145199, SGI145200, SGI145201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145202, SGI0, SGI0, SGI0, SGI145203, SGI145203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31817[] = {SGI145204, SGI0, SGI145205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145206, SGI145207, SGI145207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31818[] = {SGI145208, SGI0, SGI145209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145210, SGI145211, SGI145211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31819[] = {SGI145212, SGI0, SGI145213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145214, SGI145215, SGI145215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31820[] = {SGI145216, SGI0, SGI145217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145218, SGI145219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31821[] = {SGI145220, SGI145221, SGI145222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145223, SGI0, SGI0, SGI0, SGI145224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31822[] = {SGI145225, SGI0, SGI145226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31823[] = {SGI145227, SGI0, SGI145228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31824[] = {SGI145229, SGI145230, SGI145231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145232, SGI0, SGI0, SGI0, SGI145233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31825[] = {SGI145234, SGI145235, SGI145236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145237, SGI0, SGI0, SGI0, SGI145238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31826[] = {SGI145239, SGI145240, SGI145241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145242, SGI0, SGI0, SGI0, SGI145243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31827[] = {SGI145244, SGI145245, SGI145246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145247, SGI0, SGI0, SGI0, SGI145248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31828[] = {SGI145249, SGI145250, SGI145251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145252, SGI0, SGI0, SGI0, SGI145253, SGI145253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31829[] = {SGI145254, SGI0, SGI145255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31830[] = {SGI145256, SGI145257, SGI145258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145259, SGI0, SGI0, SGI0, SGI145260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31831[] = {SGI145261, SGI0, SGI145262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31832[] = {SGI145263, SGI145264, SGI145265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145266, SGI145266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31833[] = {SGI145267, SGI0, SGI145268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145269, SGI145270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31834[] = {SGI145271, SGI0, SGI145272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145273, SGI145274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31835[] = {SGI145275, SGI145276, SGI145277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145278, SGI0, SGI0, SGI0, SGI145279, SGI145279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31836[] = {SGI145280, SGI0, SGI145281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31837[] = {SGI145282, SGI0, SGI145283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145284, SGI0, SGI0, SGI0, SGI145285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31838[] = {SGI145287, SGI145288, SGI145289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145290, SGI0, SGI0, SGI0, SGI145291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31839[] = {SGI145293, SGI145294, SGI145296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145298, SGI0, SGI0, SGI0, SGI145299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31840[] = {SGI145295, SGI0, SGI145297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145300, SGI145302, SGI145302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31841[] = {SGI145303, SGI145304, SGI145305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145308, SGI0, SGI0, SGI0, SGI145309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31842[] = {SGI145306, SGI0, SGI145307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145310, SGI145312, SGI145312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31843[] = {SGI145313, SGI145314, SGI145315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145316, SGI0, SGI0, SGI0, SGI145317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31844[] = {SGI145319, SGI145320, SGI145321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145322, SGI0, SGI0, SGI0, SGI145323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31845[] = {SGI145325, SGI145326, SGI145327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145328, SGI0, SGI0, SGI0, SGI145329, SGI145329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31846[] = {SGI145330, SGI0, SGI145331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31847[] = {SGI145332, SGI0, SGI145333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145334, SGI145335, SGI145335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31848[] = {SGI145336, SGI0, SGI145337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145338, SGI145339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31849[] = {SGI145340, SGI0, SGI145341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145342, SGI145343, SGI145343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31850[] = {SGI145344, SGI0, SGI145345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145346, SGI145347, SGI145347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31851[] = {SGI145348, SGI0, SGI145349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145350, SGI145351, SGI145351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31852[] = {SGI145352, SGI145353, SGI145354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145355, SGI0, SGI0, SGI0, SGI145356, SGI145356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31853[] = {SGI145357, SGI145358, SGI145359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145360, SGI0, SGI0, SGI0, SGI145361, SGI145361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31854[] = {SGI145362, SGI145363, SGI145364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145365, SGI0, SGI0, SGI0, SGI145366, SGI145366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31855[] = {SGI145367, SGI0, SGI145368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31856[] = {SGI145369, SGI145370, SGI145371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31857[] = {SGI145372, SGI145373, SGI145374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145375, SGI0, SGI0, SGI0, SGI145376, SGI145376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31858[] = {SGI145377, SGI145378, SGI145379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145380, SGI0, SGI0, SGI0, SGI145381, SGI145381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31859[] = {SGI145382, SGI0, SGI145383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31860[] = {SGI145384, SGI145385, SGI145386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145387, SGI0, SGI0, SGI0, SGI145388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31861[] = {SGI145389, SGI0, SGI145390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31862[] = {SGI145391, SGI145392, SGI145393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145394, SGI0, SGI0, SGI0, SGI145395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31863[] = {SGI145396, SGI0, SGI145397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145398, SGI0, SGI0, SGI0, SGI145399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31864[] = {SGI145400, SGI145401, SGI145402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145403, SGI145403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31865[] = {SGI145404, SGI145405, SGI145406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145407, SGI0, SGI0, SGI0, SGI145408, SGI145409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31866[] = {SGI145410, SGI145411, SGI145412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145413, SGI0, SGI0, SGI0, SGI145414, SGI145414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31867[] = {SGI145415, SGI145416, SGI145417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145418, SGI0, SGI0, SGI0, SGI145419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31868[] = {SGI145420, SGI145421, SGI145422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31869[] = {SGI0, SGI145423, SGI145424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31870[] = {SGI145425, SGI0, SGI145426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31871[] = {SGI145427, SGI145428, SGI145429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145430, SGI145430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31872[] = {SGI0, SGI145431, SGI145432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31873[] = {SGI145433, SGI145434, SGI145435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145436, SGI0, SGI0, SGI0, SGI145437, SGI145437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31874[] = {SGI145438, SGI145439, SGI145440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145441, SGI0, SGI0, SGI0, SGI145442, SGI145442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31875[] = {SGI145446, SGI0, SGI145447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145448, SGI145449, SGI145449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31876[] = {SGI145443, SGI145444, SGI145445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31877[] = {SGI145450, SGI145451, SGI145452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31878[] = {SGI145453, SGI145454, SGI145455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145456, SGI0, SGI0, SGI0, SGI145457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31879[] = {SGI145458, SGI145459, SGI145460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31880[] = {SGI145461, SGI145462, SGI145463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31881[] = {SGI145464, SGI145465, SGI145466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145467, SGI0, SGI0, SGI0, SGI145468, SGI145468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31882[] = {SGI145469, SGI145470, SGI145471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145472, SGI0, SGI0, SGI0, SGI145473, SGI145473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31883[] = {SGI145474, SGI0, SGI145475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145476, SGI0, SGI0, SGI0, SGI145477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31884[] = {SGI145478, SGI145479, SGI145480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145481, SGI0, SGI0, SGI0, SGI145482, SGI145482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31885[] = {SGI145483, SGI0, SGI145484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145485, SGI145486, SGI145486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31886[] = {SGI145487, SGI0, SGI145488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145489, SGI145490, SGI145490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31887[] = {SGI145491, SGI0, SGI145492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145493, SGI145494, SGI145494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31888[] = {SGI145495, SGI145496, SGI145497, SGI145498, SGI145499, SGI145499, SGI0, NULL};
const Segment * const SG31889[] = {SGI145500, SGI145501, SGI145502, SGI145503, SGI145504, SGI145504, SGI0, NULL};
const Segment * const SG31890[] = {SGI145505, SGI145506, SGI145507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145508, SGI0, SGI0, SGI0, SGI145509, SGI145509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31891[] = {SGI145510, SGI0, SGI145511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31892[] = {SGI145512, SGI145513, SGI145514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145515, SGI0, SGI0, SGI0, SGI145516, SGI145516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31893[] = {SGI145517, SGI145518, SGI145519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145520, SGI0, SGI0, SGI0, SGI145521, SGI145521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31894[] = {SGI145522, SGI0, SGI145523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31895[] = {SGI145524, SGI0, SGI145525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31896[] = {SGI145526, SGI0, SGI145527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145528, SGI0, SGI0, SGI0, SGI145529, SGI145529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31897[] = {SGI145530, SGI145531, SGI145532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145533, SGI0, SGI0, SGI0, SGI145534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31898[] = {SGI145535, SGI145536, SGI145537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145538, SGI0, SGI0, SGI0, SGI145539, SGI145539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31899[] = {SGI145540, SGI145541, SGI145542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145543, SGI0, SGI0, SGI0, SGI145544, SGI145544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31900[] = {SGI145545, SGI145546, SGI145547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145548, SGI0, SGI0, SGI0, SGI145549, SGI145549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31901[] = {SGI145550, SGI145551, SGI145552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145553, SGI0, SGI0, SGI0, SGI145554, SGI145554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31902[] = {SGI145555, SGI0, SGI145556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31903[] = {SGI145557, SGI145558, SGI145559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145560, SGI0, SGI0, SGI0, SGI145561, SGI145561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31904[] = {SGI145562, SGI145563, SGI145564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145567, SGI0, SGI0, SGI0, SGI145569, SGI145569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31905[] = {SGI145565, SGI145566, SGI145568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145570, SGI0, SGI0, SGI0, SGI145572, SGI145572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31906[] = {SGI145573, SGI145574, SGI145575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145576, SGI0, SGI0, SGI0, SGI145577, SGI145577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31907[] = {SGI145579, SGI145580, SGI145581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145582, SGI0, SGI0, SGI0, SGI145583, SGI145583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31908[] = {SGI145585, SGI145586, SGI145587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145588, SGI0, SGI0, SGI0, SGI145589, SGI145589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31909[] = {SGI145591, SGI0, SGI145592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145593, SGI145594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31910[] = {SGI145595, SGI0, SGI145596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145597, SGI145598, SGI145598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31911[] = {SGI145599, SGI0, SGI145600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145601, SGI145602, SGI145602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31912[] = {SGI145603, SGI0, SGI145604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145605, SGI145606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31913[] = {SGI145607, SGI145608, SGI145609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145610, SGI0, SGI0, SGI0, SGI145611, SGI145611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31914[] = {SGI145612, SGI0, SGI145613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31915[] = {SGI145614, SGI0, SGI145615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145616, SGI0, SGI0, SGI0, SGI145617, SGI145617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31916[] = {SGI145618, SGI145619, SGI145620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145621, SGI0, SGI0, SGI0, SGI145622, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31917[] = {SGI145623, SGI145624, SGI145625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145626, SGI0, SGI0, SGI0, SGI145627, SGI145627, SGI0, SGI0, NULL};
const Segment * const SG31918[] = {SGI145628, SGI145629, SGI145630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145631, SGI0, SGI0, SGI0, SGI145632, SGI145632, SGI0, SGI0, NULL};
const Segment * const SG31919[] = {SGI145633, SGI145634, SGI145635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145636, SGI0, SGI0, SGI0, SGI145637, SGI145637, SGI0, SGI0, NULL};
const Segment * const SG31920[] = {SGI145638, SGI145639, SGI145640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145641, SGI0, SGI0, SGI0, SGI145642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31921[] = {SGI145643, SGI0, SGI145644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31922[] = {SGI145645, SGI145646, SGI145647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145648, SGI0, SGI0, SGI0, SGI145649, SGI145650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31923[] = {SGI145651, SGI145652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31924[] = {SGI145653, SGI145654, SGI145655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31925[] = {SGI145656, SGI145657, SGI145658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31926[] = {SGI145659, SGI145660, SGI145661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145662, SGI145662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31927[] = {SGI145663, SGI145664, SGI145665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31928[] = {SGI0, SGI145666, SGI145667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31929[] = {SGI145668, SGI145669, SGI145670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145671, SGI0, SGI0, SGI0, SGI145672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31930[] = {SGI145673, SGI145674, SGI145675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145676, SGI145676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31931[] = {SGI145677, SGI145678, SGI145679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31932[] = {SGI145680, SGI145681, SGI145682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31933[] = {SGI145683, SGI0, SGI145684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145685, SGI0, SGI0, SGI0, SGI145686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31934[] = {SGI145687, SGI0, SGI145688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145689, SGI145690, SGI145690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31935[] = {SGI145691, SGI0, SGI145692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145693, SGI145694, SGI145694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31936[] = {SGI145695, SGI0, SGI145696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145697, SGI145698, SGI145698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31937[] = {SGI145699, SGI0, SGI145700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145701, SGI145702, SGI145702, SGI0, SGI0, NULL};
const Segment * const SG31938[] = {SGI145703, SGI145704, SGI145705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145706, SGI0, SGI0, SGI0, SGI145707, SGI145707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31939[] = {SGI145708, SGI0, SGI145709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31940[] = {SGI145710, SGI0, SGI145711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145715, SGI145716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31941[] = {SGI145712, SGI145713, SGI145714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145717, SGI0, SGI0, SGI0, SGI145718, SGI145718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31942[] = {SGI145719, SGI0, SGI145720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145724, SGI145725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31943[] = {SGI145721, SGI145722, SGI145723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145726, SGI0, SGI0, SGI0, SGI145727, SGI145727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31944[] = {SGI145728, SGI0, SGI145729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31945[] = {SGI145730, SGI145731, SGI145732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145733, SGI0, SGI0, SGI0, SGI145734, SGI145734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31946[] = {SGI145735, SGI145736, SGI145737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145738, SGI0, SGI0, SGI0, SGI145739, SGI145739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31947[] = {SGI145740, SGI145741, SGI145742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145743, SGI0, SGI0, SGI0, SGI145744, SGI145744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31948[] = {SGI145745, SGI145746, SGI145747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145748, SGI0, SGI0, SGI0, SGI145749, SGI145749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31949[] = {SGI145750, SGI145751, SGI145752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145753, SGI0, SGI0, SGI0, SGI145754, SGI145754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31950[] = {SGI145755, SGI145756, SGI145757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145758, SGI0, SGI0, SGI0, SGI145759, SGI145759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31951[] = {SGI145760, SGI145761, SGI145762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145763, SGI0, SGI0, SGI0, SGI145764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31952[] = {SGI145765, SGI145766, SGI145767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145769, SGI0, SGI0, SGI0, SGI145772, SGI145772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31953[] = {SGI145768, SGI145770, SGI145771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145773, SGI0, SGI0, SGI0, SGI145774, SGI145774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31954[] = {SGI145775, SGI0, SGI145776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31955[] = {SGI145777, SGI145778, SGI145779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145780, SGI0, SGI0, SGI0, SGI145781, SGI145781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31956[] = {SGI145782, SGI145783, SGI145784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145785, SGI0, SGI0, SGI0, SGI145786, SGI145786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31957[] = {SGI145787, SGI145788, SGI145789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145790, SGI0, SGI0, SGI0, SGI145791, SGI145791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31958[] = {SGI145792, SGI145793, SGI145794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145795, SGI0, SGI0, SGI0, SGI145796, SGI145796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31959[] = {SGI145797, SGI145798, SGI145799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145800, SGI0, SGI0, SGI0, SGI145801, SGI145801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31960[] = {SGI145802, SGI0, SGI145803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145806, SGI145808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31961[] = {SGI145804, SGI145805, SGI145807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145809, SGI0, SGI0, SGI0, SGI145810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31962[] = {SGI145811, SGI145812, SGI145813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145816, SGI0, SGI0, SGI0, SGI145817, SGI145817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31963[] = {SGI145814, SGI0, SGI145815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145818, SGI145819, SGI145819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31964[] = {SGI145820, SGI0, SGI145821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31965[] = {SGI145822, SGI0, SGI145823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145824, SGI145825, SGI145825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31966[] = {SGI145826, SGI145827, SGI145828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145829, SGI0, SGI0, SGI0, SGI145830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31967[] = {SGI145831, SGI0, SGI145832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145833, SGI145834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31968[] = {SGI145835, SGI145836, SGI145837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145838, SGI0, SGI0, SGI0, SGI145839, SGI145839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31969[] = {SGI145840, SGI0, SGI145841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31970[] = {SGI145844, SGI0, SGI145845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145846, SGI145847, SGI145847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31971[] = {SGI145848, SGI0, SGI145849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145850, SGI145851, SGI145851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31972[] = {SGI145852, SGI0, SGI145853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145854, SGI145855, SGI145855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31973[] = {SGI145856, SGI0, SGI145857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145858, SGI145859, SGI145859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31974[] = {SGI145860, SGI0, SGI145861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145862, SGI145863, SGI145863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31975[] = {SGI145864, SGI0, SGI145865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145866, SGI145867, SGI145867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31976[] = {SGI145868, SGI0, SGI145869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145870, SGI145871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31977[] = {SGI145842, SGI0, SGI145843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145872, SGI145873, SGI145873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31978[] = {SGI145874, SGI145875, SGI145876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145877, SGI0, SGI0, SGI0, SGI145878, SGI145878, SGI0, SGI0, NULL};
const Segment * const SG31979[] = {SGI145879, SGI145880, SGI145881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145882, SGI0, SGI0, SGI0, SGI145883, SGI145883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31980[] = {SGI145884, SGI145885, SGI145886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145887, SGI0, SGI0, SGI0, SGI145888, SGI145888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31981[] = {SGI145889, SGI145890, SGI145891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145892, SGI0, SGI0, SGI0, SGI145893, SGI145893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31982[] = {SGI145894, SGI0, SGI145895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31983[] = {SGI145896, SGI145897, SGI145898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145899, SGI0, SGI0, SGI0, SGI145900, SGI145900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31984[] = {SGI145901, SGI145902, SGI145903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145904, SGI145904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31985[] = {SGI145905, SGI0, SGI145906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145907, SGI0, SGI0, SGI0, SGI145908, SGI145908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31986[] = {SGI145909, SGI145910, SGI145911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145912, SGI0, SGI0, SGI0, SGI145913, SGI145913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31987[] = {SGI145914, SGI145915, SGI145916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145917, SGI0, SGI0, SGI0, SGI145918, SGI145918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31988[] = {SGI145919, SGI145920, SGI145921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145922, SGI0, SGI0, SGI0, SGI145923, SGI145923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31989[] = {SGI145924, SGI145925, SGI145926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145927, SGI145927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31990[] = {SGI145928, SGI145929, SGI145930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145931, SGI145931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31991[] = {SGI145932, SGI145933, SGI145934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145935, SGI0, SGI0, SGI0, SGI145936, SGI145936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31992[] = {SGI145937, SGI145938, SGI145939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145940, SGI145941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31993[] = {SGI145942, SGI0, SGI145943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31994[] = {SGI145944, SGI145945, SGI145946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145947, SGI0, SGI0, SGI0, SGI145948, SGI145948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31995[] = {SGI145949, SGI145950, SGI145951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145952, SGI0, SGI0, SGI0, SGI145953, SGI145953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31996[] = {SGI145954, SGI145955, SGI145956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145957, SGI0, SGI0, SGI0, SGI145958, SGI145958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31997[] = {SGI145959, SGI145960, SGI145961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145962, SGI0, SGI0, SGI0, SGI145963, SGI145963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31998[] = {SGI145964, SGI145965, SGI145966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145967, SGI0, SGI0, SGI0, SGI145968, SGI145968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG31999[] = {SGI145969, SGI145970, SGI145971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145972, SGI0, SGI0, SGI0, SGI145973, SGI145973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32000[] = {SGI145974, SGI145975, SGI145976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145977, SGI0, SGI0, SGI0, SGI145979, SGI145979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32001[] = {SGI145978, SGI145980, SGI145981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145982, SGI0, SGI0, SGI0, SGI145983, SGI145983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32002[] = {SGI145984, SGI145985, SGI145986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145987, SGI0, SGI0, SGI0, SGI145988, SGI145988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32003[] = {SGI145989, SGI145990, SGI145991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145992, SGI145992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32004[] = {SGI145993, SGI145994, SGI145995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI145996, SGI145996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32005[] = {SGI145997, SGI145998, SGI145999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146000, SGI0, SGI0, SGI0, SGI146001, SGI146001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32006[] = {SGI146002, SGI146003, SGI146004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146005, SGI0, SGI0, SGI0, SGI146006, SGI146006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32007[] = {SGI146007, SGI0, SGI146008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32008[] = {SGI146009, SGI0, SGI146010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146011, SGI146012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32009[] = {SGI146013, SGI0, SGI146014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146015, SGI146016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32010[] = {SGI146017, SGI0, SGI146018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146019, SGI146020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32011[] = {SGI146021, SGI0, SGI146022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146023, SGI146024, SGI146024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32012[] = {SGI146025, SGI0, SGI146026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146027, SGI146028, SGI146028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32013[] = {SGI146029, SGI0, SGI146030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146031, SGI146032, SGI146032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32014[] = {SGI146033, SGI0, SGI146034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146035, SGI146036, SGI146036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32015[] = {SGI146037, SGI0, SGI146038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146039, SGI146040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32016[] = {SGI146041, SGI0, SGI146042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146043, SGI146044, SGI146044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32017[] = {SGI146045, SGI146046, SGI146047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146048, SGI0, SGI0, SGI0, SGI146049, SGI146049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32018[] = {SGI146050, SGI0, SGI146051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32019[] = {SGI146052, SGI0, SGI146053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146054, SGI146055, SGI146055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32020[] = {SGI146056, SGI0, SGI146057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146058, SGI146059, SGI146059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32021[] = {SGI146060, SGI0, SGI146061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146062, SGI146063, SGI146063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32022[] = {SGI146064, SGI0, SGI146065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146066, SGI146067, SGI146067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32023[] = {SGI146068, SGI0, SGI146069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146070, SGI146071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32024[] = {SGI146072, SGI0, SGI146073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146074, SGI146075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32025[] = {SGI146076, SGI146077, SGI146078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146079, SGI0, SGI0, SGI0, SGI146080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32026[] = {SGI146081, SGI0, SGI146082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146083, SGI146084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32027[] = {SGI146085, SGI0, SGI146086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146087, SGI146088, SGI146088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32028[] = {SGI146089, SGI0, SGI146090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146091, SGI146092, SGI146092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32029[] = {SGI146093, SGI0, SGI146094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146095, SGI146096, SGI146096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32030[] = {SGI146097, SGI146098, SGI146099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146100, SGI0, SGI0, SGI0, SGI146101, SGI146101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32031[] = {SGI146102, SGI146103, SGI146104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146105, SGI0, SGI0, SGI0, SGI146106, SGI146106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32032[] = {SGI146107, SGI0, SGI146108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32033[] = {SGI146109, SGI0, SGI146110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146111, SGI146112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32034[] = {SGI146113, SGI146114, SGI146115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146116, SGI0, SGI0, SGI0, SGI146117, SGI146117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32035[] = {SGI146118, SGI0, SGI146119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32036[] = {SGI146120, SGI146121, SGI146122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146123, SGI0, SGI0, SGI0, SGI146124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32037[] = {SGI146125, SGI146126, SGI146127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146128, SGI0, SGI0, SGI0, SGI146129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32038[] = {SGI146130, SGI146131, SGI146132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146133, SGI0, SGI0, SGI0, SGI146134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32039[] = {SGI146135, SGI0, SGI146136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32040[] = {SGI146137, SGI146138, SGI146139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146140, SGI0, SGI0, SGI0, SGI146141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32041[] = {SGI146142, SGI0, SGI146143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146144, SGI146145, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32042[] = {SGI146146, SGI0, SGI146147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146148, SGI146149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32043[] = {SGI146150, SGI0, SGI146151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146152, SGI146153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32044[] = {SGI146154, SGI0, SGI146155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146156, SGI146157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32045[] = {SGI146158, SGI0, SGI146159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146160, SGI146161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32046[] = {SGI146162, SGI0, SGI146163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146164, SGI146165, SGI146165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32047[] = {SGI146166, SGI0, SGI146167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146168, SGI146169, SGI146169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32048[] = {SGI146170, SGI0, SGI146171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146172, SGI146173, SGI146173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32049[] = {SGI146174, SGI146175, SGI146176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146177, SGI0, SGI0, SGI0, SGI146178, SGI146178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32050[] = {SGI146179, SGI146180, SGI146181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146182, SGI0, SGI0, SGI0, SGI146183, SGI146183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32051[] = {SGI146184, SGI146185, SGI146186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146187, SGI0, SGI0, SGI0, SGI146188, SGI146188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32052[] = {SGI146189, SGI0, SGI146190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32053[] = {SGI146191, SGI0, SGI146192, SGI0, SGI146193, SGI146193, SGI0, NULL};
const Segment * const SG32054[] = {SGI146194, SGI0, SGI146195, SGI0, SGI146196, SGI146196, SGI0, NULL};
const Segment * const SG32055[] = {SGI146197, SGI0, SGI146198, SGI0, SGI146199, SGI146199, SGI0, NULL};
const Segment * const SG32056[] = {SGI146200, SGI0, SGI146201, SGI0, SGI146202, SGI146202, SGI0, NULL};
const Segment * const SG32057[] = {SGI146203, SGI0, SGI146204, SGI0, SGI146205, SGI146205, SGI0, NULL};
const Segment * const SG32058[] = {SGI146206, SGI146207, SGI146208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146209, SGI0, SGI0, SGI0, SGI146210, SGI146210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32059[] = {SGI146211, SGI146212, SGI146213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146214, SGI0, SGI0, SGI0, SGI146215, SGI146215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32060[] = {SGI146216, SGI0, SGI146217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32061[] = {SGI146218, SGI146219, SGI146220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146221, SGI0, SGI0, SGI0, SGI146222, SGI146222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32062[] = {SGI146223, SGI146224, SGI146225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146226, SGI0, SGI0, SGI0, SGI146227, SGI146227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32063[] = {SGI146228, SGI146229, SGI146230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146231, SGI0, SGI0, SGI0, SGI146232, SGI146232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32064[] = {SGI146233, SGI146234, SGI146235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146236, SGI0, SGI0, SGI0, SGI146237, SGI146237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32065[] = {SGI146238, SGI146239, SGI146240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146241, SGI0, SGI0, SGI0, SGI146242, SGI146242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32066[] = {SGI146243, SGI146244, SGI146245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146246, SGI0, SGI0, SGI0, SGI146247, SGI146247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32067[] = {SGI146248, SGI146249, SGI146250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146251, SGI0, SGI0, SGI0, SGI146252, SGI146252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32068[] = {SGI146253, SGI146254, SGI146255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146256, SGI0, SGI0, SGI0, SGI146257, SGI146257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32069[] = {SGI146258, SGI146259, SGI146260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146261, SGI0, SGI0, SGI0, SGI146262, SGI146262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32070[] = {SGI146263, SGI146264, SGI146265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146266, SGI146266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32071[] = {SGI0, SGI146267, SGI146268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146269, SGI146269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32072[] = {SGI0, SGI146270, SGI146271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146272, SGI146272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32073[] = {SGI0, SGI146273, SGI146274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146275, SGI146275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32074[] = {SGI0, SGI146276, SGI146277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146278, SGI146278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32075[] = {SGI146279, SGI146280, SGI146281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146282, SGI0, SGI0, SGI0, SGI146283, SGI146283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32076[] = {SGI146284, SGI146285, SGI146286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146287, SGI146287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32077[] = {SGI0, SGI146288, SGI146289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146290, SGI146290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32078[] = {SGI146291, SGI0, SGI146292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32079[] = {SGI146293, SGI146294, SGI146295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146296, SGI146297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32080[] = {SGI146298, SGI146299, SGI146300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146301, SGI0, SGI0, SGI0, SGI146302, SGI146302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32081[] = {SGI0, SGI146303, SGI146304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146305, SGI146305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32082[] = {SGI0, SGI146306, SGI146307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146308, SGI146308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32083[] = {SGI146309, SGI0, SGI146310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32084[] = {SGI146311, SGI0, SGI146312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32085[] = {SGI146313, SGI146314, SGI146315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146316, SGI0, SGI0, SGI0, SGI146317, SGI146317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32086[] = {SGI0, SGI146318, SGI146319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146320, SGI146320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32087[] = {SGI0, SGI146321, SGI146322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146323, SGI146323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32088[] = {SGI0, SGI146324, SGI146325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146326, SGI146326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32089[] = {SGI0, SGI146327, SGI146328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146329, SGI146329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32090[] = {SGI0, SGI146330, SGI146331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146332, SGI146332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32091[] = {SGI146333, SGI146334, SGI146335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146336, SGI0, SGI0, SGI0, SGI146337, SGI146337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32092[] = {SGI146338, SGI146339, SGI146340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146341, SGI0, SGI0, SGI0, SGI146342, SGI146342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32093[] = {SGI146343, SGI0, SGI146344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32094[] = {SGI146345, SGI146346, SGI146347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146348, SGI0, SGI0, SGI0, SGI146349, SGI146349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32095[] = {SGI146350, SGI0, SGI146351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146352, SGI146353, SGI146353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32096[] = {SGI146354, SGI0, SGI146355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146356, SGI146357, SGI146357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32097[] = {SGI146358, SGI146359, SGI146360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146361, SGI0, SGI0, SGI0, SGI146362, SGI146362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32098[] = {SGI146363, SGI146364, SGI146365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146366, SGI0, SGI0, SGI0, SGI146367, SGI146367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32099[] = {SGI146368, SGI0, SGI146369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32100[] = {SGI146370, SGI0, SGI146371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146372, SGI0, SGI0, SGI0, SGI146373, SGI146373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32101[] = {SGI146374, SGI0, SGI146375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146376, SGI146377, SGI146377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32102[] = {SGI146378, SGI0, SGI146379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146380, SGI146381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32103[] = {SGI146382, SGI0, SGI146383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146384, SGI146385, SGI146385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32104[] = {SGI146386, SGI0, SGI146387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146388, SGI146389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32105[] = {SGI146390, SGI0, SGI146391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146392, SGI146393, SGI146393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32106[] = {SGI146394, SGI0, SGI146395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146396, SGI146397, SGI146397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32107[] = {SGI146398, SGI0, SGI146399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146400, SGI146401, SGI146401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32108[] = {SGI146402, SGI0, SGI146403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146404, SGI146405, SGI146405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32109[] = {SGI146406, SGI0, SGI146407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146408, SGI146409, SGI146409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32110[] = {SGI146410, SGI146411, SGI146412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146413, SGI0, SGI0, SGI0, SGI146414, SGI146414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32111[] = {SGI146416, SGI0, SGI146417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32112[] = {SGI146415, SGI0, SGI146418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146419, SGI146420, SGI146420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32113[] = {SGI146421, SGI146422, SGI146423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146426, SGI0, SGI0, SGI0, SGI146427, SGI146427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32114[] = {SGI146424, SGI0, SGI146425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146428, SGI146429, SGI146429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32115[] = {SGI146430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32116[] = {SGI146431, SGI0, SGI146432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146433, SGI146434, SGI146434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32117[] = {SGI146435, SGI146436, SGI146437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146438, SGI0, SGI0, SGI0, SGI146439, SGI146439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32118[] = {SGI146440, SGI0, SGI146441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146442, SGI0, SGI0, SGI0, SGI146443, SGI146443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32119[] = {SGI146444, SGI0, SGI146445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32120[] = {SGI146446, SGI146447, SGI146448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146449, SGI0, SGI0, SGI0, SGI146450, SGI146450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32121[] = {SGI146451, SGI146452, SGI146453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146454, SGI0, SGI0, SGI0, SGI146455, SGI146455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32122[] = {SGI146456, SGI146457, SGI146458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146459, SGI0, SGI0, SGI0, SGI146460, SGI146460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32123[] = {SGI146461, SGI146462, SGI146463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146464, SGI0, SGI0, SGI0, SGI146465, SGI146465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32124[] = {SGI146466, SGI146467, SGI146468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146469, SGI0, SGI0, SGI0, SGI146470, SGI146470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32125[] = {SGI146471, SGI0, SGI146472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32126[] = {SGI146473, SGI0, SGI146474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32127[] = {SGI146475, SGI146476, SGI146477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146478, SGI0, SGI0, SGI0, SGI146479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32128[] = {SGI146480, SGI0, SGI146481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146482, SGI146483, SGI146483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32129[] = {SGI146484, SGI0, SGI146485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146486, SGI146487, SGI146487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32130[] = {SGI146488, SGI0, SGI146489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146490, SGI146491, SGI146491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32131[] = {SGI146492, SGI0, SGI146493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146494, SGI146495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32132[] = {SGI146496, SGI146497, SGI146498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146499, SGI0, SGI0, SGI0, SGI146500, SGI146500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32133[] = {SGI146501, SGI0, SGI146502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146506, SGI146507, SGI146507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32134[] = {SGI146503, SGI146504, SGI146505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146508, SGI0, SGI0, SGI0, SGI146509, SGI146509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32135[] = {SGI146511, SGI0, SGI146512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32136[] = {SGI146510, SGI0, SGI146513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146514, SGI146515, SGI146515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32137[] = {SGI146516, SGI0, SGI146517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146518, SGI146519, SGI146519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32138[] = {SGI146520, SGI146521, SGI146522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146523, SGI0, SGI0, SGI0, SGI146524, SGI146524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32139[] = {SGI146525, SGI146526, SGI146527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146528, SGI0, SGI0, SGI0, SGI146529, SGI146529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32140[] = {SGI146530, SGI0, SGI146531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32141[] = {SGI146532, SGI146533, SGI146534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146535, SGI0, SGI0, SGI0, SGI146536, SGI146536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32142[] = {SGI146537, SGI146538, SGI146539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146540, SGI0, SGI0, SGI0, SGI146541, SGI146541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32143[] = {SGI146542, SGI146543, SGI146544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146545, SGI0, SGI0, SGI0, SGI146546, SGI146546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32144[] = {SGI146547, SGI146548, SGI146549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146551, SGI0, SGI0, SGI0, SGI146553, SGI146553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32145[] = {SGI146550, SGI146552, SGI146554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146555, SGI0, SGI0, SGI0, SGI146558, SGI146558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32146[] = {SGI146556, SGI146557, SGI146559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146560, SGI0, SGI0, SGI0, SGI146562, SGI146562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32147[] = {SGI146561, SGI0, SGI146563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146564, SGI0, SGI0, SGI0, SGI146565, SGI146565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32148[] = {SGI146566, SGI146567, SGI146568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146569, SGI0, SGI0, SGI0, SGI146571, SGI146571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32149[] = {SGI146570, SGI0, SGI146572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32150[] = {SGI146573, SGI0, SGI146574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32151[] = {SGI146575, SGI0, SGI146576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32152[] = {SGI146577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32153[] = {SGI146580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32154[] = {SGI146578, SGI146579, SGI146581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146582, SGI0, SGI0, SGI0, SGI146583, SGI146583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32155[] = {SGI146584, SGI146585, SGI146586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146587, SGI0, SGI0, SGI0, SGI146588, SGI146588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32156[] = {SGI146589, SGI146590, SGI146591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146592, SGI0, SGI0, SGI0, SGI146593, SGI146593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32157[] = {SGI146594, SGI146595, SGI146596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146597, SGI0, SGI0, SGI0, SGI146598, SGI146598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32158[] = {SGI146599, SGI146600, SGI146601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146602, SGI0, SGI0, SGI0, SGI146603, SGI146603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32159[] = {SGI146604, SGI146605, SGI146606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146607, SGI0, SGI0, SGI0, SGI146608, SGI146608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32160[] = {SGI146609, SGI0, SGI146610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32161[] = {SGI146611, SGI146612, SGI146613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146614, SGI0, SGI0, SGI0, SGI146615, SGI146615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32162[] = {SGI146616, SGI146617, SGI146618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146619, SGI0, SGI0, SGI0, SGI146620, SGI146620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32163[] = {SGI146621, SGI146622, SGI146623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146624, SGI0, SGI0, SGI0, SGI146625, SGI146625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32164[] = {SGI146626, SGI146627, SGI146628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146629, SGI0, SGI0, SGI0, SGI146630, SGI146630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32165[] = {SGI146631, SGI146632, SGI146633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146634, SGI0, SGI0, SGI0, SGI146635, SGI146635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32166[] = {SGI146636, SGI146637, SGI146638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146639, SGI0, SGI0, SGI0, SGI146640, SGI146640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32167[] = {SGI146641, SGI146642, SGI146643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146644, SGI0, SGI0, SGI0, SGI146645, SGI146645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32168[] = {SGI146646, SGI146647, SGI146648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146649, SGI0, SGI0, SGI0, SGI146650, SGI146650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32169[] = {SGI146651, SGI146652, SGI146653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146654, SGI0, SGI0, SGI0, SGI146655, SGI146655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32170[] = {SGI146656, SGI146657, SGI146658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146659, SGI0, SGI0, SGI0, SGI146660, SGI146660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32171[] = {SGI146661, SGI146662, SGI146663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146664, SGI0, SGI0, SGI0, SGI146665, SGI146665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32172[] = {SGI146666, SGI146667, SGI146668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146669, SGI0, SGI0, SGI0, SGI146670, SGI146670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32173[] = {SGI146671, SGI146672, SGI146673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146674, SGI0, SGI0, SGI0, SGI146675, SGI146675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32174[] = {SGI146676, SGI146677, SGI146678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146679, SGI0, SGI0, SGI0, SGI146680, SGI146680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32175[] = {SGI146686, SGI0, SGI146687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146688, SGI146689, SGI146689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32176[] = {SGI146690, SGI0, SGI146691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146692, SGI146693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32177[] = {SGI146694, SGI146695, SGI146696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146697, SGI0, SGI0, SGI0, SGI146698, SGI146698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32178[] = {SGI146699, SGI146700, SGI146701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146702, SGI0, SGI0, SGI0, SGI146703, SGI146703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32179[] = {SGI146704, SGI146705, SGI146706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146707, SGI0, SGI0, SGI0, SGI146708, SGI146708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32180[] = {SGI146709, SGI146710, SGI146711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146712, SGI0, SGI0, SGI0, SGI146713, SGI146713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32181[] = {SGI146714, SGI146715, SGI146716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146717, SGI0, SGI0, SGI0, SGI146718, SGI146718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32182[] = {SGI146719, SGI146720, SGI146721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146722, SGI0, SGI0, SGI0, SGI146723, SGI146723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32183[] = {SGI146724, SGI0, SGI146725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32184[] = {SGI146726, SGI0, SGI146727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32185[] = {SGI146728, SGI0, SGI146729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32186[] = {SGI146730, SGI146731, SGI146732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146733, SGI0, SGI0, SGI0, SGI146734, SGI146734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32187[] = {SGI146735, SGI146736, SGI146737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146738, SGI0, SGI0, SGI0, SGI146739, SGI146739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32188[] = {SGI146740, SGI0, SGI146741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32189[] = {SGI146681, SGI146682, SGI146683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146684, SGI0, SGI0, SGI0, SGI146685, SGI146685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32190[] = {SGI146742, SGI146743, SGI146744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146745, SGI0, SGI0, SGI0, SGI146746, SGI146746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32191[] = {SGI146747, SGI146748, SGI146749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146750, SGI0, SGI0, SGI0, SGI146751, SGI146751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32192[] = {SGI146752, SGI146753, SGI146754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146755, SGI0, SGI0, SGI0, SGI146756, SGI146756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32193[] = {SGI146757, SGI146758, SGI146759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146760, SGI0, SGI0, SGI0, SGI146761, SGI146761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32194[] = {SGI146762, SGI0, SGI146763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32195[] = {SGI146764, SGI146765, SGI146766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146767, SGI0, SGI0, SGI0, SGI146768, SGI146768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32196[] = {SGI146769, SGI0, SGI146770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32197[] = {SGI146771, SGI146772, SGI146773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146774, SGI0, SGI0, SGI0, SGI146775, SGI146775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32198[] = {SGI146776, SGI146777, SGI146778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146779, SGI0, SGI0, SGI0, SGI146780, SGI146780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32199[] = {SGI146781, SGI146782, SGI146783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146784, SGI0, SGI0, SGI0, SGI146785, SGI146785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32200[] = {SGI146786, SGI0, SGI146787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146788, SGI0, SGI0, SGI0, SGI146789, SGI146789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32201[] = {SGI146790, SGI146791, SGI146792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146793, SGI0, SGI0, SGI0, SGI146794, SGI146794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32202[] = {SGI146795, SGI146796, SGI146797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146798, SGI0, SGI0, SGI0, SGI146799, SGI146799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32203[] = {SGI146800, SGI146801, SGI146802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146803, SGI0, SGI0, SGI0, SGI146804, SGI146804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32204[] = {SGI146805, SGI146806, SGI146807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146808, SGI0, SGI0, SGI0, SGI146809, SGI146809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32205[] = {SGI146810, SGI146811, SGI146812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146813, SGI0, SGI0, SGI0, SGI146814, SGI146814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32206[] = {SGI146815, SGI146816, SGI146817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146818, SGI0, SGI0, SGI0, SGI146819, SGI146819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32207[] = {SGI146820, SGI146821, SGI146822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146823, SGI0, SGI0, SGI0, SGI146824, SGI146824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32208[] = {SGI146825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32209[] = {SGI146826, SGI146827, SGI146828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146829, SGI0, SGI0, SGI0, SGI146830, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32210[] = {SGI146831, SGI146832, SGI146833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146837, SGI0, SGI0, SGI0, SGI146838, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32211[] = {SGI146834, SGI146835, SGI146836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146839, SGI0, SGI0, SGI0, SGI146840, SGI146840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32212[] = {SGI146841, SGI0, SGI146842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32213[] = {SGI146843, SGI0, SGI146844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146846, SGI146848, SGI146848, SGI0, SGI0, NULL};
const Segment * const SG32214[] = {SGI146845, SGI0, SGI146847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32215[] = {SGI146849, SGI146850, SGI146851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146852, SGI0, SGI0, SGI0, SGI146853, SGI146853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32216[] = {SGI146854, SGI146855, SGI146856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146857, SGI0, SGI0, SGI0, SGI146858, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32217[] = {SGI146859, SGI146860, SGI146861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146862, SGI0, SGI0, SGI0, SGI146863, SGI146863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32218[] = {SGI146864, SGI0, SGI146865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32219[] = {SGI146866, SGI146867, SGI146868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146869, SGI146869, SGI0, SGI0, NULL};
const Segment * const SG32220[] = {SGI146870, SGI146871, SGI146872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146873, SGI0, SGI0, SGI0, SGI146874, SGI146874, SGI0, SGI0, NULL};
const Segment * const SG32221[] = {SGI146875, SGI146876, SGI146877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146878, SGI0, SGI0, SGI0, SGI146879, SGI146879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32222[] = {SGI146880, SGI146881, SGI146882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146883, SGI0, SGI0, SGI0, SGI146884, SGI146884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32223[] = {SGI146885, SGI0, SGI146886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32224[] = {SGI146887, SGI146888, SGI146889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146890, SGI0, SGI0, SGI0, SGI146891, SGI146891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32225[] = {SGI146892, SGI146893, SGI146894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146895, SGI0, SGI0, SGI0, SGI146896, SGI146896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32226[] = {SGI146897, SGI0, SGI146898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146899, SGI0, SGI0, SGI0, SGI146900, SGI146900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32227[] = {SGI146901, SGI146902, SGI146903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146904, SGI0, SGI0, SGI0, SGI146905, SGI146905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32228[] = {SGI146906, SGI146907, SGI146908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146909, SGI0, SGI0, SGI0, SGI146910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32229[] = {SGI146911, SGI0, SGI146912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32230[] = {SGI146913, SGI146914, SGI146915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146916, SGI0, SGI0, SGI0, SGI146917, SGI146917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32231[] = {SGI146918, SGI0, SGI146919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32232[] = {SGI146920, SGI146921, SGI146922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146923, SGI0, SGI0, SGI0, SGI146924, SGI146924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32233[] = {SGI146925, SGI0, SGI146926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32234[] = {SGI146927, SGI0, SGI146928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146929, SGI0, SGI0, SGI0, SGI146930, SGI146930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32235[] = {SGI146931, SGI0, SGI146932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146933, SGI146934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32236[] = {SGI146935, SGI0, SGI146936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146937, SGI146938, SGI146938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32237[] = {SGI146939, SGI0, SGI146940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146941, SGI146942, SGI146942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32238[] = {SGI146943, SGI0, SGI146944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146945, SGI146946, SGI146946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32239[] = {SGI146947, SGI0, SGI146948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146949, SGI146950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32240[] = {SGI146951, SGI146952, SGI146953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146954, SGI0, SGI0, SGI0, SGI146955, SGI146955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32241[] = {SGI146956, SGI146957, SGI146958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146959, SGI0, SGI0, SGI0, SGI146960, SGI146960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32242[] = {SGI146961, SGI146962, SGI146963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146964, SGI0, SGI0, SGI0, SGI146965, SGI146965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32243[] = {SGI146966, SGI0, SGI146967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146968, SGI146969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32244[] = {SGI146970, SGI146971, SGI146972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146973, SGI0, SGI0, SGI0, SGI146974, SGI146974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32245[] = {SGI146975, SGI146976, SGI146977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146978, SGI0, SGI0, SGI0, SGI146979, SGI146979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32246[] = {SGI146980, SGI146981, SGI146982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146983, SGI0, SGI0, SGI0, SGI146984, SGI146984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32247[] = {SGI146985, SGI146986, SGI146987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146988, SGI0, SGI0, SGI0, SGI146989, SGI146989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32248[] = {SGI146990, SGI146991, SGI146992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146993, SGI0, SGI0, SGI0, SGI146994, SGI146994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32249[] = {SGI146995, SGI146996, SGI146997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI146998, SGI0, SGI0, SGI0, SGI146999, SGI146999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32250[] = {SGI147000, SGI147001, SGI147002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147005, SGI0, SGI0, SGI0, SGI147006, SGI147006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32251[] = {SGI147003, SGI0, SGI147004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147007, SGI147008, SGI147008, SGI0, SGI0, NULL};
const Segment * const SG32252[] = {SGI147009, SGI147010, SGI147011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147012, SGI0, SGI0, SGI0, SGI147013, SGI147013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32253[] = {SGI147014, SGI147015, SGI147016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147017, SGI0, SGI0, SGI0, SGI147018, SGI147018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32254[] = {SGI147019, SGI147020, SGI147021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147022, SGI0, SGI0, SGI0, SGI147023, SGI147023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32255[] = {SGI147024, SGI147025, SGI147026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147027, SGI0, SGI0, SGI0, SGI147028, SGI147028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32256[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32257[] = {SGI147030, SGI147031, SGI147032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147033, SGI0, SGI0, SGI0, SGI147034, SGI147034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32258[] = {SGI147035, SGI147036, SGI147037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147038, SGI0, SGI0, SGI0, SGI147039, SGI147039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32259[] = {SGI147040, SGI147041, SGI147042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147043, SGI0, SGI0, SGI0, SGI147044, SGI147044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32260[] = {SGI147045, SGI147046, SGI147047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147048, SGI0, SGI0, SGI0, SGI147049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32261[] = {SGI147050, SGI147051, SGI147052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147053, SGI0, SGI0, SGI0, SGI147054, SGI147054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32262[] = {SGI147055, SGI147056, SGI147057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147058, SGI0, SGI0, SGI0, SGI147059, SGI147059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32263[] = {SGI147060, SGI147061, SGI147062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147063, SGI0, SGI0, SGI0, SGI147064, SGI147064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32264[] = {SGI147065, SGI147066, SGI147067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147068, SGI0, SGI0, SGI0, SGI147069, SGI147069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32265[] = {SGI147070, SGI147071, SGI147072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147073, SGI0, SGI0, SGI0, SGI147074, SGI147074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32266[] = {SGI147075, SGI147076, SGI147077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147078, SGI0, SGI0, SGI0, SGI147079, SGI147079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32267[] = {SGI147080, SGI147081, SGI147082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147083, SGI0, SGI0, SGI0, SGI147084, SGI147084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32268[] = {SGI147085, SGI147086, SGI147087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147088, SGI0, SGI0, SGI0, SGI147089, SGI147089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32269[] = {SGI147090, SGI147091, SGI147092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147093, SGI0, SGI0, SGI0, SGI147094, SGI147094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32270[] = {SGI147095, SGI147096, SGI147097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147098, SGI0, SGI0, SGI0, SGI147099, SGI147099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32271[] = {SGI0, SGI147100, SGI147101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147102, SGI0, SGI0, SGI0, SGI147103, SGI147103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32272[] = {SGI147104, SGI147105, SGI147106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147107, SGI0, SGI0, SGI0, SGI147108, SGI147108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32273[] = {SGI147109, SGI147110, SGI147111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147112, SGI0, SGI0, SGI0, SGI147113, SGI147113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32274[] = {SGI147114, SGI147115, SGI147116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147117, SGI0, SGI0, SGI0, SGI147118, SGI147118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32275[] = {SGI147119, SGI147120, SGI147121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147122, SGI0, SGI0, SGI0, SGI147123, SGI147123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32276[] = {SGI147124, SGI147125, SGI147126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147127, SGI0, SGI0, SGI0, SGI147128, SGI147128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32277[] = {SGI147129, SGI147130, SGI147131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147132, SGI0, SGI0, SGI0, SGI147133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32278[] = {SGI147134, SGI147135, SGI147136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147137, SGI0, SGI0, SGI0, SGI147138, SGI147138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32279[] = {SGI147139, SGI147140, SGI147141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147142, SGI0, SGI0, SGI0, SGI147143, SGI147143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32280[] = {SGI147144, SGI147145, SGI147146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147147, SGI0, SGI0, SGI0, SGI147148, SGI147148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32281[] = {SGI147149, SGI0, SGI147150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147151, SGI147152, SGI147152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32282[] = {SGI147153, SGI0, SGI147155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147158, SGI147160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32283[] = {SGI147154, SGI147156, SGI147157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147159, SGI0, SGI0, SGI0, SGI147161, SGI147161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32284[] = {SGI147162, SGI147163, SGI147164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147165, SGI0, SGI0, SGI0, SGI147166, SGI147166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32285[] = {SGI147167, SGI147168, SGI147169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147170, SGI0, SGI0, SGI0, SGI147171, SGI147171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32286[] = {SGI147172, SGI0, SGI147173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147174, SGI147175, SGI147175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32287[] = {SGI147176, SGI147177, SGI147178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147179, SGI0, SGI0, SGI0, SGI147180, SGI147180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32288[] = {SGI147181, SGI147182, SGI147183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147184, SGI0, SGI0, SGI0, SGI147185, SGI147185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32289[] = {SGI147186, SGI147187, SGI147188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147189, SGI0, SGI0, SGI0, SGI147190, SGI147190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32290[] = {SGI147191, SGI147192, SGI147193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147194, SGI0, SGI0, SGI0, SGI147195, SGI147195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32291[] = {SGI147196, SGI147197, SGI147198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147199, SGI0, SGI0, SGI0, SGI147200, SGI147200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32292[] = {SGI147201, SGI147202, SGI147203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147204, SGI0, SGI0, SGI0, SGI147205, SGI147205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32293[] = {SGI147206, SGI147207, SGI147208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147209, SGI0, SGI0, SGI0, SGI147210, SGI147210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32294[] = {SGI147211, SGI147212, SGI147213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147214, SGI0, SGI0, SGI0, SGI147215, SGI147215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32295[] = {SGI147216, SGI147217, SGI147218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147219, SGI0, SGI0, SGI0, SGI147220, SGI147220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32296[] = {SGI147221, SGI147222, SGI147223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147224, SGI0, SGI0, SGI0, SGI147225, SGI147225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32297[] = {SGI147226, SGI147227, SGI147228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147229, SGI0, SGI0, SGI0, SGI147230, SGI147230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32298[] = {SGI147231, SGI147232, SGI147233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147234, SGI0, SGI0, SGI0, SGI147235, SGI147235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32299[] = {SGI147236, SGI147237, SGI147238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147239, SGI0, SGI0, SGI0, SGI147240, SGI147240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32300[] = {SGI0, SGI147241, SGI147242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147243, SGI0, SGI0, SGI0, SGI147244, SGI147244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32301[] = {SGI147245, SGI147246, SGI147247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147248, SGI0, SGI0, SGI0, SGI147249, SGI147249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32302[] = {SGI147250, SGI147251, SGI147252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147253, SGI0, SGI0, SGI0, SGI147256, SGI147256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32303[] = {SGI147254, SGI147255, SGI147257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147258, SGI0, SGI0, SGI0, SGI147259, SGI147259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32304[] = {SGI147260, SGI147261, SGI147262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147263, SGI0, SGI0, SGI0, SGI147264, SGI147264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32305[] = {SGI147265, SGI147266, SGI147267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147268, SGI0, SGI0, SGI0, SGI147269, SGI147269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32306[] = {SGI0, SGI147270, SGI147271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147272, SGI0, SGI0, SGI0, SGI147273, SGI147273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32307[] = {SGI147274, SGI147275, SGI147276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147277, SGI0, SGI0, SGI0, SGI147279, SGI147279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32308[] = {SGI147278, SGI147280, SGI147281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147282, SGI0, SGI0, SGI0, SGI147283, SGI147283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32309[] = {SGI147284, SGI147285, SGI147286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147289, SGI0, SGI0, SGI0, SGI147291, SGI147291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32310[] = {SGI147287, SGI147288, SGI147290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147292, SGI0, SGI0, SGI0, SGI147293, SGI147293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32311[] = {SGI147295, SGI147297, SGI147299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147301, SGI147301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32312[] = {SGI147294, SGI147296, SGI147298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147300, SGI0, SGI0, SGI0, SGI147302, SGI147302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32313[] = {SGI147303, SGI147304, SGI147305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147308, SGI0, SGI0, SGI0, SGI147309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32314[] = {SGI147306, SGI0, SGI147307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32315[] = {SGI147310, SGI147311, SGI147312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147315, SGI0, SGI0, SGI0, SGI147316, SGI147316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32316[] = {SGI147313, SGI0, SGI147314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32317[] = {SGI147317, SGI147318, SGI147319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147320, SGI0, SGI0, SGI0, SGI147321, SGI147321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32318[] = {SGI147322, SGI147323, SGI147324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147325, SGI0, SGI0, SGI0, SGI147326, SGI147326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32319[] = {SGI147327, SGI147328, SGI147329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147330, SGI0, SGI0, SGI0, SGI147331, SGI147331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32320[] = {SGI0, SGI147332, SGI147333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147334, SGI0, SGI0, SGI0, SGI147335, SGI147335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32321[] = {SGI147336, SGI147337, SGI147338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147339, SGI0, SGI0, SGI0, SGI147340, SGI147340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32322[] = {SGI147341, SGI147342, SGI147343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147344, SGI0, SGI0, SGI0, SGI147345, SGI147345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32323[] = {SGI0, SGI147346, SGI147347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147348, SGI0, SGI0, SGI0, SGI147349, SGI147349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32324[] = {SGI147350, SGI147351, SGI147352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147353, SGI0, SGI0, SGI0, SGI147354, SGI147354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32325[] = {SGI147355, SGI147356, SGI147357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147358, SGI0, SGI0, SGI0, SGI147359, SGI147359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32326[] = {SGI147360, SGI147361, SGI147363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147365, SGI0, SGI0, SGI0, SGI147366, SGI147366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32327[] = {SGI147362, SGI0, SGI147364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147367, SGI147368, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32328[] = {SGI147369, SGI147371, SGI147373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147375, SGI0, SGI0, SGI0, SGI147377, SGI147377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32329[] = {SGI147370, SGI147372, SGI147374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147376, SGI0, SGI0, SGI0, SGI147378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32330[] = {SGI147379, SGI147380, SGI147381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147383, SGI0, SGI0, SGI0, SGI147386, SGI147386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32331[] = {SGI147382, SGI147384, SGI147385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147387, SGI0, SGI0, SGI0, SGI147388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32332[] = {SGI147389, SGI147390, SGI147391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147392, SGI0, SGI0, SGI0, SGI147394, SGI147394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32333[] = {SGI147393, SGI0, SGI147395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32334[] = {SGI147396, SGI147397, SGI147398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147400, SGI0, SGI0, SGI0, SGI147403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32335[] = {SGI147399, SGI147401, SGI147402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147404, SGI0, SGI0, SGI0, SGI147405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32336[] = {SGI147406, SGI147407, SGI147408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147409, SGI0, SGI0, SGI0, SGI147411, SGI147411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32337[] = {SGI147410, SGI147412, SGI147413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32338[] = {SGI147414, SGI147415, SGI147416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147417, SGI0, SGI0, SGI0, SGI147418, SGI147418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32339[] = {SGI147419, SGI147420, SGI147421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32340[] = {SGI147422, SGI147423, SGI147424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147425, SGI0, SGI0, SGI0, SGI147428, SGI147428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32341[] = {SGI0, SGI147426, SGI147427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32342[] = {SGI147429, SGI147430, SGI147431, SGI147432, SGI147433, SGI147433, SGI0, NULL};
const Segment * const SG32343[] = {SGI147434, SGI147435, SGI147436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147440, SGI147440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32344[] = {SGI147437, SGI147438, SGI147439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147441, SGI0, SGI0, SGI0, SGI147442, SGI147442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32345[] = {SGI147443, SGI147444, SGI147445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147449, SGI0, SGI0, SGI0, SGI147450, SGI147450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32346[] = {SGI147446, SGI147447, SGI147448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147451, SGI0, SGI0, SGI0, SGI147452, SGI147452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32347[] = {SGI147453, SGI147454, SGI147455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147459, SGI0, SGI0, SGI0, SGI147460, SGI147460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32348[] = {SGI147456, SGI147457, SGI147458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147461, SGI0, SGI0, SGI0, SGI147462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32349[] = {SGI147463, SGI147464, SGI147465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147466, SGI0, SGI0, SGI0, SGI147470, SGI147470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32350[] = {SGI147467, SGI147468, SGI147469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32351[] = {SGI147471, SGI147472, SGI147473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147475, SGI147475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32352[] = {SGI147474, SGI147476, SGI147477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147478, SGI0, SGI0, SGI0, SGI147479, SGI147479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32353[] = {SGI147480, SGI0, SGI147481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147483, SGI0, SGI0, SGI0, SGI147486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32354[] = {SGI147482, SGI147484, SGI147485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147487, SGI0, SGI0, SGI0, SGI147488, SGI147488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32355[] = {SGI147489, SGI147490, SGI147491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32356[] = {SGI147495, SGI147496, SGI147497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32357[] = {SGI147492, SGI147493, SGI147494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147498, SGI0, SGI0, SGI0, SGI147499, SGI147499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32358[] = {SGI147500, SGI147501, SGI147502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147503, SGI0, SGI0, SGI0, SGI147505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32359[] = {SGI147504, SGI147506, SGI147507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147508, SGI0, SGI0, SGI0, SGI147509, SGI147509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32360[] = {SGI147510, SGI147511, SGI147512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147513, SGI0, SGI0, SGI0, SGI147514, SGI147515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32361[] = {SGI147516, SGI0, SGI147517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32362[] = {SGI147518, SGI147519, SGI147520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147521, SGI0, SGI0, SGI0, SGI147524, SGI147524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32363[] = {SGI147522, SGI147523, SGI147525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147526, SGI0, SGI0, SGI0, SGI147527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32364[] = {SGI147528, SGI147529, SGI147530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147531, SGI0, SGI0, SGI0, SGI147532, SGI147532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32365[] = {SGI147533, SGI147534, SGI147535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147536, SGI0, SGI0, SGI0, SGI147537, SGI147537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32366[] = {SGI147538, SGI147539, SGI147540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147541, SGI0, SGI0, SGI0, SGI147542, SGI147542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32367[] = {SGI147543, SGI147544, SGI147545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147546, SGI0, SGI0, SGI0, SGI147547, SGI147547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32368[] = {SGI147548, SGI147549, SGI147550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147551, SGI0, SGI0, SGI0, SGI147552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32369[] = {SGI147553, SGI147554, SGI147555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147556, SGI0, SGI0, SGI0, SGI147557, SGI147557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32370[] = {SGI147558, SGI147559, SGI147560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147561, SGI0, SGI0, SGI0, SGI147562, SGI147562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32371[] = {SGI147563, SGI147564, SGI147565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147566, SGI0, SGI0, SGI0, SGI147567, SGI147567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32372[] = {SGI147568, SGI147569, SGI147570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147571, SGI0, SGI0, SGI0, SGI147572, SGI147572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32373[] = {SGI147573, SGI147574, SGI147575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147576, SGI0, SGI0, SGI0, SGI147577, SGI147577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32374[] = {SGI147578, SGI147579, SGI147580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147581, SGI0, SGI0, SGI0, SGI147582, SGI147582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32375[] = {SGI147583, SGI147584, SGI147585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147586, SGI0, SGI0, SGI0, SGI147587, SGI147587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32376[] = {SGI147589, SGI147591, SGI147593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147594, SGI0, SGI0, SGI0, SGI147596, SGI147596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32377[] = {SGI147588, SGI147590, SGI147592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147595, SGI0, SGI0, SGI0, SGI147597, SGI147597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32378[] = {SGI147598, SGI147599, SGI147600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147601, SGI0, SGI0, SGI0, SGI147602, SGI147602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32379[] = {SGI147603, SGI147604, SGI147605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147606, SGI0, SGI0, SGI0, SGI147607, SGI147607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32380[] = {SGI147608, SGI147609, SGI147610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147611, SGI0, SGI0, SGI0, SGI147612, SGI147612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32381[] = {SGI147613, SGI147614, SGI147615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147616, SGI0, SGI0, SGI0, SGI147617, SGI147617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32382[] = {SGI147618, SGI147619, SGI147620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147624, SGI0, SGI0, SGI0, SGI147625, SGI147625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32383[] = {SGI147621, SGI147622, SGI147623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147626, SGI0, SGI0, SGI0, SGI147627, SGI147627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32384[] = {SGI147628, SGI147629, SGI147630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147633, SGI0, SGI0, SGI0, SGI147634, SGI147634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32385[] = {SGI147631, SGI0, SGI147632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32386[] = {SGI147635, SGI147636, SGI147637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147638, SGI0, SGI0, SGI0, SGI147639, SGI147639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32387[] = {SGI147640, SGI147641, SGI147642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147643, SGI0, SGI0, SGI0, SGI147644, SGI147644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32388[] = {SGI147646, SGI147648, SGI147649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147650, SGI0, SGI0, SGI0, SGI147653, SGI147653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32389[] = {SGI147645, SGI0, SGI147647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147651, SGI147652, SGI147652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32390[] = {SGI147654, SGI147655, SGI147656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147657, SGI0, SGI0, SGI0, SGI147658, SGI147658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32391[] = {SGI147659, SGI147660, SGI147661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147662, SGI0, SGI0, SGI0, SGI147663, SGI147663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32392[] = {SGI147664, SGI0, SGI147666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147668, SGI0, SGI0, SGI0, SGI147669, SGI147669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32393[] = {SGI147665, SGI0, SGI147667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147671, SGI147673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32394[] = {SGI147670, SGI147672, SGI147674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147675, SGI0, SGI0, SGI0, SGI147676, SGI147676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32395[] = {SGI147677, SGI0, SGI147678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147685, SGI147686, SGI147686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32396[] = {SGI147679, SGI147680, SGI147682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147687, SGI0, SGI0, SGI0, SGI147689, SGI147689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32397[] = {SGI147681, SGI147683, SGI147684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147688, SGI0, SGI0, SGI0, SGI147690, SGI147690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32398[] = {SGI147691, SGI147692, SGI147693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147694, SGI0, SGI0, SGI0, SGI147695, SGI147695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32399[] = {SGI147696, SGI147697, SGI147698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147699, SGI0, SGI0, SGI0, SGI147700, SGI147700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32400[] = {SGI147701, SGI0, SGI147702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147703, SGI147704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32401[] = {SGI147705, SGI147706, SGI147707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147708, SGI0, SGI0, SGI0, SGI147709, SGI147709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32402[] = {SGI147710, SGI147711, SGI147712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147713, SGI0, SGI0, SGI0, SGI147714, SGI147714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32403[] = {SGI147715, SGI147716, SGI147717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147718, SGI0, SGI0, SGI0, SGI147719, SGI147719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32404[] = {SGI0, SGI147720, SGI147721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147722, SGI0, SGI0, SGI0, SGI147723, SGI147723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32405[] = {SGI147724, SGI147725, SGI147726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147727, SGI0, SGI0, SGI0, SGI147728, SGI147728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32406[] = {SGI147729, SGI147730, SGI147731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147732, SGI0, SGI0, SGI0, SGI147733, SGI147733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32407[] = {SGI147734, SGI147735, SGI147736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147737, SGI0, SGI0, SGI0, SGI147738, SGI147738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32408[] = {SGI147739, SGI147740, SGI147741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147742, SGI0, SGI0, SGI0, SGI147743, SGI147743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32409[] = {SGI147744, SGI147745, SGI147746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147747, SGI0, SGI0, SGI0, SGI147748, SGI147748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32410[] = {SGI147749, SGI147750, SGI147751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147752, SGI0, SGI0, SGI0, SGI147753, SGI147753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32411[] = {SGI147754, SGI147755, SGI147756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147757, SGI0, SGI0, SGI0, SGI147758, SGI147758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32412[] = {SGI147759, SGI147760, SGI147761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147762, SGI0, SGI0, SGI0, SGI147763, SGI147763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32413[] = {SGI147764, SGI147765, SGI147766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147767, SGI0, SGI0, SGI0, SGI147768, SGI147768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32414[] = {SGI147769, SGI147770, SGI147771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147772, SGI0, SGI0, SGI0, SGI147773, SGI147773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32415[] = {SGI147774, SGI147775, SGI147776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147777, SGI0, SGI0, SGI0, SGI147778, SGI147778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32416[] = {SGI147779, SGI147780, SGI147781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147782, SGI0, SGI0, SGI0, SGI147783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32417[] = {SGI147784, SGI147785, SGI147786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147787, SGI0, SGI0, SGI0, SGI147788, SGI147788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32418[] = {SGI147789, SGI147790, SGI147791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147792, SGI0, SGI0, SGI0, SGI147793, SGI147793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32419[] = {SGI147794, SGI147795, SGI147796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147797, SGI0, SGI0, SGI0, SGI147798, SGI147798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32420[] = {SGI147799, SGI147800, SGI147801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147805, SGI0, SGI0, SGI0, SGI147806, SGI147806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32421[] = {SGI147802, SGI147803, SGI147804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147807, SGI0, SGI0, SGI0, SGI147808, SGI147808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32422[] = {SGI147809, SGI147810, SGI147811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147812, SGI0, SGI0, SGI0, SGI147813, SGI147813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32423[] = {SGI147814, SGI147815, SGI147816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147820, SGI0, SGI0, SGI0, SGI147821, SGI147821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32424[] = {SGI147817, SGI147818, SGI147819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147822, SGI0, SGI0, SGI0, SGI147823, SGI147823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32425[] = {SGI147824, SGI0, SGI147826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147830, SGI147831, SGI147831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32426[] = {SGI147825, SGI147827, SGI147828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147829, SGI0, SGI0, SGI0, SGI147832, SGI147832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32427[] = {SGI147833, SGI147835, SGI147837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147838, SGI0, SGI0, SGI0, SGI147841, SGI147841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32428[] = {SGI147834, SGI0, SGI147836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147839, SGI147840, SGI147840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32429[] = {SGI147842, SGI147843, SGI147844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147845, SGI0, SGI0, SGI0, SGI147846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32430[] = {SGI147847, SGI0, SGI147848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147849, SGI147850, SGI147850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32431[] = {SGI147851, SGI147852, SGI147853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147854, SGI0, SGI0, SGI0, SGI147855, SGI147855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32432[] = {SGI147856, SGI0, SGI147857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147859, SGI147861, SGI147861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32433[] = {SGI147858, SGI147860, SGI147862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147863, SGI0, SGI0, SGI0, SGI147864, SGI147864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32434[] = {SGI147865, SGI0, SGI147866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147867, SGI147868, SGI147868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32435[] = {SGI147869, SGI0, SGI147870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147874, SGI147875, SGI147875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32436[] = {SGI147871, SGI147872, SGI147873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147876, SGI0, SGI0, SGI0, SGI147877, SGI147877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32437[] = {SGI147878, SGI0, SGI147879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147880, SGI147881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32438[] = {SGI147882, SGI0, SGI147883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147884, SGI147886, SGI147886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32439[] = {SGI147885, SGI147887, SGI147888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147889, SGI0, SGI0, SGI0, SGI147890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32440[] = {SGI147891, SGI0, SGI147892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147893, SGI147894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32441[] = {SGI147895, SGI147896, SGI147897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147898, SGI0, SGI0, SGI0, SGI147899, SGI147899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32442[] = {SGI147900, SGI0, SGI147901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147902, SGI147903, SGI147903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32443[] = {SGI147904, SGI147905, SGI147906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147909, SGI0, SGI0, SGI0, SGI147910, SGI147910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32444[] = {SGI147907, SGI0, SGI147908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147911, SGI147912, SGI147912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32445[] = {SGI147913, SGI0, SGI147914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147916, SGI147917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32446[] = {SGI0, SGI0, SGI147915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147918, SGI147918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32447[] = {SGI147919, SGI147920, SGI147921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147922, SGI0, SGI0, SGI0, SGI147923, SGI147923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32448[] = {SGI147924, SGI147925, SGI147926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147927, SGI0, SGI0, SGI0, SGI147928, SGI147928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32449[] = {SGI147929, SGI147930, SGI147931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147932, SGI0, SGI0, SGI0, SGI147933, SGI147933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32450[] = {SGI147934, SGI147935, SGI147936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147937, SGI0, SGI0, SGI0, SGI147938, SGI147938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32451[] = {SGI147939, SGI147940, SGI147941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147942, SGI0, SGI0, SGI0, SGI147943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32452[] = {SGI147944, SGI147945, SGI147946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147947, SGI0, SGI0, SGI0, SGI147948, SGI147948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32453[] = {SGI147949, SGI0, SGI147950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147951, SGI147952, SGI147952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32454[] = {SGI147953, SGI147954, SGI147956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147958, SGI0, SGI0, SGI0, SGI147959, SGI147959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32455[] = {SGI147955, SGI0, SGI147957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147960, SGI147961, SGI147961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32456[] = {SGI147962, SGI0, SGI147963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147967, SGI147968, SGI147968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32457[] = {SGI147964, SGI147965, SGI147966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147969, SGI0, SGI0, SGI0, SGI147970, SGI147970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32458[] = {SGI147971, SGI147972, SGI147973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147974, SGI0, SGI0, SGI0, SGI147975, SGI147975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32459[] = {SGI147976, SGI147977, SGI147978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147979, SGI0, SGI0, SGI0, SGI147980, SGI147980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32460[] = {SGI147981, SGI147982, SGI147983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147984, SGI0, SGI0, SGI0, SGI147985, SGI147985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32461[] = {SGI147986, SGI147987, SGI147988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147989, SGI0, SGI0, SGI0, SGI147990, SGI147990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32462[] = {SGI147991, SGI147992, SGI147993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147994, SGI0, SGI0, SGI0, SGI147995, SGI147995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32463[] = {SGI147996, SGI147997, SGI147998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI147999, SGI0, SGI0, SGI0, SGI148000, SGI148000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32464[] = {SGI148001, SGI148002, SGI148003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148004, SGI0, SGI0, SGI0, SGI148005, SGI148005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32465[] = {SGI148006, SGI148007, SGI148008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148009, SGI0, SGI0, SGI0, SGI148010, SGI148010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32466[] = {SGI148011, SGI148012, SGI148013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148014, SGI0, SGI0, SGI0, SGI148015, SGI148015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32467[] = {SGI148016, SGI148017, SGI148018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148019, SGI0, SGI0, SGI0, SGI148020, SGI148020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32468[] = {SGI148021, SGI0, SGI148022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148023, SGI148024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32469[] = {SGI148025, SGI0, SGI148026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148027, SGI148028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32470[] = {SGI148029, SGI148030, SGI148031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148032, SGI0, SGI0, SGI0, SGI148033, SGI148033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32471[] = {SGI148034, SGI0, SGI148035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148039, SGI148040, SGI148040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32472[] = {SGI148036, SGI148037, SGI148038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148041, SGI0, SGI0, SGI0, SGI148042, SGI148042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32473[] = {SGI148043, SGI148045, SGI148046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148048, SGI0, SGI0, SGI0, SGI148050, SGI148050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32474[] = {SGI148044, SGI0, SGI148047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148049, SGI148051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32475[] = {SGI148052, SGI148053, SGI148054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148055, SGI0, SGI0, SGI0, SGI148056, SGI148056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32476[] = {SGI148057, SGI148058, SGI148059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148060, SGI0, SGI0, SGI0, SGI148061, SGI148061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32477[] = {SGI148062, SGI148063, SGI148064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148065, SGI0, SGI0, SGI0, SGI148066, SGI148066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32478[] = {SGI148068, SGI148069, SGI148070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148071, SGI0, SGI0, SGI0, SGI148072, SGI148072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32479[] = {SGI148073, SGI148074, SGI148075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148076, SGI0, SGI0, SGI0, SGI148078, SGI148078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32480[] = {SGI148077, SGI148079, SGI148080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148082, SGI0, SGI0, SGI0, SGI148083, SGI148083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32481[] = {SGI148084, SGI148085, SGI148086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148090, SGI0, SGI0, SGI0, SGI148091, SGI148091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32482[] = {SGI148087, SGI148088, SGI148089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148092, SGI0, SGI0, SGI0, SGI148093, SGI148093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32483[] = {SGI148095, SGI148097, SGI148099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148101, SGI0, SGI0, SGI0, SGI148103, SGI148103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32484[] = {SGI148096, SGI148098, SGI148100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148102, SGI0, SGI0, SGI0, SGI148104, SGI148104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32485[] = {SGI148106, SGI148107, SGI148108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148111, SGI0, SGI0, SGI0, SGI148113, SGI148113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32486[] = {SGI148109, SGI148110, SGI148112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148114, SGI0, SGI0, SGI0, SGI148115, SGI148115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32487[] = {SGI148119, SGI148121, SGI148122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148124, SGI0, SGI0, SGI0, SGI148126, SGI148126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32488[] = {SGI148117, SGI148118, SGI148120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148123, SGI0, SGI0, SGI0, SGI148125, SGI148125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32489[] = {SGI148128, SGI148129, SGI148130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148131, SGI0, SGI0, SGI0, SGI148132, SGI148132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32490[] = {SGI148134, SGI148135, SGI148136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148137, SGI0, SGI0, SGI0, SGI148138, SGI148138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32491[] = {SGI148139, SGI148140, SGI148141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148142, SGI0, SGI0, SGI0, SGI148143, SGI148143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32492[] = {SGI148144, SGI0, SGI148145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32493[] = {SGI148146, SGI0, SGI148147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148148, SGI148149, SGI148149, SGI0, SGI0, NULL};
const Segment * const SG32494[] = {SGI148150, SGI148151, SGI148152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148153, SGI0, SGI0, SGI0, SGI148154, SGI148154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32495[] = {SGI148155, SGI148156, SGI148157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148158, SGI0, SGI0, SGI0, SGI148159, SGI148159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32496[] = {SGI148160, SGI148161, SGI148162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148163, SGI0, SGI0, SGI0, SGI148164, SGI148164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32497[] = {SGI148165, SGI148166, SGI148167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148168, SGI0, SGI0, SGI0, SGI148169, SGI148169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32498[] = {SGI148170, SGI148171, SGI148172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148173, SGI0, SGI0, SGI0, SGI148174, SGI148174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32499[] = {SGI148175, SGI148176, SGI148177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148178, SGI0, SGI0, SGI0, SGI148179, SGI148179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32500[] = {SGI148180, SGI0, SGI148181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148184, SGI148186, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32501[] = {SGI148182, SGI148183, SGI148185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148187, SGI0, SGI0, SGI0, SGI148188, SGI148188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32502[] = {SGI148189, SGI148190, SGI148191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148192, SGI0, SGI0, SGI0, SGI148193, SGI148193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32503[] = {SGI148194, SGI0, SGI148195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148196, SGI148197, SGI148197, SGI0, SGI0, NULL};
const Segment * const SG32504[] = {SGI148198, SGI148199, SGI148200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148201, SGI0, SGI0, SGI0, SGI148203, SGI148203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32505[] = {SGI148202, SGI0, SGI148204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148205, SGI148206, SGI148206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32506[] = {SGI148207, SGI148208, SGI148209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148210, SGI0, SGI0, SGI0, SGI148211, SGI148211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32507[] = {SGI148212, SGI148213, SGI148214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148215, SGI0, SGI0, SGI0, SGI148216, SGI148216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32508[] = {SGI148217, SGI148218, SGI148219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148220, SGI0, SGI0, SGI0, SGI148221, SGI148221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32509[] = {SGI148222, SGI148223, SGI148224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148225, SGI0, SGI0, SGI0, SGI148226, SGI148226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32510[] = {SGI148227, SGI148228, SGI148229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148230, SGI0, SGI0, SGI0, SGI148231, SGI148231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32511[] = {SGI148232, SGI148233, SGI148234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148235, SGI0, SGI0, SGI0, SGI148236, SGI148236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32512[] = {SGI148237, SGI0, SGI148238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148239, SGI148240, SGI148240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32513[] = {SGI148241, SGI148242, SGI148243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148244, SGI148244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32514[] = {SGI148245, SGI0, SGI148246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148247, SGI148248, SGI148248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32515[] = {SGI148250, SGI148251, SGI148253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148254, SGI0, SGI0, SGI0, SGI148257, SGI148257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32516[] = {SGI148249, SGI0, SGI148252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148255, SGI148256, SGI148256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32517[] = {SGI148258, SGI148259, SGI148260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148261, SGI0, SGI0, SGI0, SGI148262, SGI148262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32518[] = {SGI148263, SGI148264, SGI148265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148266, SGI0, SGI0, SGI0, SGI148267, SGI148267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32519[] = {SGI148268, SGI148269, SGI148270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148271, SGI0, SGI0, SGI0, SGI148272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32520[] = {SGI148273, SGI148274, SGI148275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148276, SGI0, SGI0, SGI0, SGI148277, SGI148277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32521[] = {SGI148278, SGI0, SGI148279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32522[] = {SGI148280, SGI148282, SGI148284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148286, SGI0, SGI0, SGI0, SGI148288, SGI148288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32523[] = {SGI148281, SGI0, SGI148283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148285, SGI0, SGI0, SGI0, SGI148287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32524[] = {SGI148289, SGI148290, SGI148292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148295, SGI0, SGI0, SGI0, SGI148296, SGI148296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32525[] = {SGI148291, SGI148293, SGI148294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148297, SGI148297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32526[] = {SGI148298, SGI148300, SGI148302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148304, SGI0, SGI0, SGI0, SGI148305, SGI148305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32527[] = {SGI148299, SGI148301, SGI148303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32528[] = {SGI148306, SGI148307, SGI148309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148312, SGI0, SGI0, SGI0, SGI148314, SGI148314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32529[] = {SGI148308, SGI148310, SGI148311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148313, SGI0, SGI0, SGI0, SGI148315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32530[] = {SGI148316, SGI148317, SGI148318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148319, SGI0, SGI0, SGI0, SGI148321, SGI148321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32531[] = {SGI148320, SGI148322, SGI148323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148324, SGI0, SGI0, SGI0, SGI148325, SGI148325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32532[] = {SGI148326, SGI148327, SGI148328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148329, SGI0, SGI0, SGI0, SGI148330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32533[] = {SGI148331, SGI148332, SGI148333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148334, SGI0, SGI0, SGI0, SGI148335, SGI148335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32534[] = {SGI148336, SGI148337, SGI148338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148339, SGI0, SGI0, SGI0, SGI148340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32535[] = {SGI148341, SGI0, SGI148342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148343, SGI148344, SGI148344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32536[] = {SGI148345, SGI0, SGI148346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148347, SGI148348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32537[] = {SGI148349, SGI148350, SGI148351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148352, SGI148353, SGI148353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32538[] = {SGI148354, SGI148355, SGI148356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148357, SGI0, SGI0, SGI0, SGI148358, SGI148358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32539[] = {SGI148359, SGI0, SGI148360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32540[] = {SGI148361, SGI0, SGI148362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148363, SGI0, SGI0, SGI0, SGI148364, SGI148364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32541[] = {SGI148365, SGI148366, SGI148367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148368, SGI148369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32542[] = {SGI148370, SGI148371, SGI148372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148373, SGI0, SGI0, SGI0, SGI148374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32543[] = {SGI148375, SGI148376, SGI148377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148380, SGI0, SGI0, SGI0, SGI148381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32544[] = {SGI148378, SGI0, SGI148379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148382, SGI148383, SGI148383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32545[] = {SGI148384, SGI148385, SGI148386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148387, SGI0, SGI0, SGI0, SGI148388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32546[] = {SGI148389, SGI0, SGI148390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148391, SGI148392, SGI148392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32547[] = {SGI148393, SGI0, SGI148394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148397, SGI148398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32548[] = {SGI148395, SGI0, SGI148396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148399, SGI148400, SGI148400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32549[] = {SGI148401, SGI0, SGI148402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148405, SGI148406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32550[] = {SGI148403, SGI0, SGI148404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148407, SGI148408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32551[] = {SGI148409, SGI0, SGI148410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148411, SGI148412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32552[] = {SGI148413, SGI148414, SGI148415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148416, SGI0, SGI0, SGI0, SGI148417, SGI148417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32553[] = {SGI148418, SGI148419, SGI148420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148421, SGI0, SGI0, SGI0, SGI148422, SGI148422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32554[] = {SGI148423, SGI148424, SGI148425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148426, SGI0, SGI0, SGI0, SGI148427, SGI148427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32555[] = {SGI148428, SGI0, SGI148429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32556[] = {SGI148430, SGI148431, SGI148432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148433, SGI0, SGI0, SGI0, SGI148434, SGI148434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32557[] = {SGI148435, SGI0, SGI148436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32558[] = {SGI148437, SGI148438, SGI148439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148440, SGI0, SGI0, SGI0, SGI148441, SGI148441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32559[] = {SGI148442, SGI0, SGI148443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148444, SGI148445, SGI148445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32560[] = {SGI148446, SGI0, SGI148447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148448, SGI148449, SGI148449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32561[] = {SGI148450, SGI148451, SGI148452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148453, SGI0, SGI0, SGI0, SGI148454, SGI148454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32562[] = {SGI148455, SGI148456, SGI148457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148458, SGI0, SGI0, SGI0, SGI148459, SGI148459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32563[] = {SGI148460, SGI148461, SGI148462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148463, SGI0, SGI0, SGI0, SGI148464, SGI148464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32564[] = {SGI148465, SGI0, SGI148466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32565[] = {SGI148467, SGI148468, SGI148469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148470, SGI0, SGI0, SGI0, SGI148471, SGI148471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32566[] = {SGI148472, SGI0, SGI148473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32567[] = {SGI148474, SGI148475, SGI148476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148477, SGI0, SGI0, SGI0, SGI148478, SGI148478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32568[] = {SGI148479, SGI148480, SGI148481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148482, SGI0, SGI0, SGI0, SGI148483, SGI148483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32569[] = {SGI148484, SGI148485, SGI148486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148487, SGI0, SGI0, SGI0, SGI148488, SGI148488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32570[] = {SGI148489, SGI0, SGI148490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32571[] = {SGI148491, SGI0, SGI148492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148493, SGI0, SGI0, SGI0, SGI148494, SGI148494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32572[] = {SGI148495, SGI148496, SGI148497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148498, SGI0, SGI0, SGI0, SGI148499, SGI148499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32573[] = {SGI148500, SGI0, SGI148501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32574[] = {SGI148502, SGI0, SGI148503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148504, SGI148505, SGI148505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32575[] = {SGI148506, SGI0, SGI148507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148508, SGI148509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32576[] = {SGI148510, SGI0, SGI148511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148512, SGI148513, SGI148513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32577[] = {SGI148514, SGI0, SGI148515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148516, SGI148517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32578[] = {SGI148518, SGI148519, SGI148520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148521, SGI0, SGI0, SGI0, SGI148522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32579[] = {SGI148523, SGI0, SGI148524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32580[] = {SGI148525, SGI148526, SGI148527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148528, SGI0, SGI0, SGI0, SGI148529, SGI148529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32581[] = {SGI148530, SGI0, SGI148531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148532, SGI0, SGI0, SGI0, SGI148533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32582[] = {SGI148534, SGI148535, SGI148536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148537, SGI0, SGI0, SGI0, SGI148538, SGI148538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32583[] = {SGI148539, SGI0, SGI148540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32584[] = {SGI148541, SGI148542, SGI148543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148544, SGI0, SGI0, SGI0, SGI148545, SGI148545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32585[] = {SGI148546, SGI148547, SGI148548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148549, SGI0, SGI0, SGI0, SGI148550, SGI148550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32586[] = {SGI148551, SGI148552, SGI148553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148554, SGI0, SGI0, SGI0, SGI148555, SGI148555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32587[] = {SGI148556, SGI148557, SGI148558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148559, SGI0, SGI0, SGI0, SGI148560, SGI148560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32588[] = {SGI148561, SGI148562, SGI148563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148564, SGI0, SGI0, SGI0, SGI148565, SGI148565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32589[] = {SGI148566, SGI148567, SGI148568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148569, SGI0, SGI0, SGI0, SGI148570, SGI148570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32590[] = {SGI148571, SGI148572, SGI148573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148574, SGI0, SGI0, SGI0, SGI148575, SGI148575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32591[] = {SGI148576, SGI148577, SGI148578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148579, SGI0, SGI0, SGI0, SGI148580, SGI148580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32592[] = {SGI148581, SGI0, SGI148582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32593[] = {SGI148583, SGI148584, SGI148585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148586, SGI0, SGI0, SGI0, SGI148587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32594[] = {SGI148588, SGI148589, SGI148590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148591, SGI0, SGI0, SGI0, SGI148592, SGI148592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32595[] = {SGI148593, SGI0, SGI148594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148595, SGI0, SGI0, SGI0, SGI148596, SGI148596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32596[] = {SGI148597, SGI0, SGI148598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148599, SGI148600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32597[] = {SGI148601, SGI148602, SGI148603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148604, SGI0, SGI0, SGI0, SGI148605, SGI148605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32598[] = {SGI148606, SGI148607, SGI148608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148609, SGI0, SGI0, SGI0, SGI148610, SGI148610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32599[] = {SGI148611, SGI148612, SGI148613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148614, SGI0, SGI0, SGI0, SGI148615, SGI148615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32600[] = {SGI148616, SGI148617, SGI148618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148619, SGI148619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32601[] = {SGI148620, SGI148621, SGI148622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148623, SGI0, SGI0, SGI0, SGI148624, SGI148624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32602[] = {SGI148625, SGI148626, SGI148627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148628, SGI0, SGI0, SGI0, SGI148629, SGI148629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32603[] = {SGI148630, SGI0, SGI148631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32604[] = {SGI148632, SGI0, SGI148633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32605[] = {SGI148634, SGI0, SGI148635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32606[] = {SGI148636, SGI0, SGI148637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148638, SGI148639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32607[] = {SGI148640, SGI0, SGI148641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148642, SGI148643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32608[] = {SGI148644, SGI0, SGI148645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148646, SGI148647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32609[] = {SGI148648, SGI0, SGI148649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148650, SGI148651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32610[] = {SGI148652, SGI0, SGI148653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148654, SGI148655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32611[] = {SGI148656, SGI0, SGI148657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148658, SGI148659, SGI148659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32612[] = {SGI148660, SGI0, SGI148661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148662, SGI148663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32613[] = {SGI148664, SGI0, SGI148665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148666, SGI148667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32614[] = {SGI148668, SGI148669, SGI148670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148671, SGI0, SGI0, SGI0, SGI148672, SGI148672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32615[] = {SGI148673, SGI148674, SGI148675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148676, SGI0, SGI0, SGI0, SGI148677, SGI148677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32616[] = {SGI148678, SGI148679, SGI148680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148681, SGI148681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32617[] = {SGI148682, SGI0, SGI148683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32618[] = {SGI148684, SGI148685, SGI148686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148687, SGI0, SGI0, SGI0, SGI148688, SGI148688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32619[] = {SGI148689, SGI148690, SGI148691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148692, SGI0, SGI0, SGI0, SGI148693, SGI148693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32620[] = {SGI148694, SGI0, SGI148695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32621[] = {SGI148696, SGI148697, SGI148698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148699, SGI0, SGI0, SGI0, SGI148700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32622[] = {SGI148701, SGI148702, SGI148703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148704, SGI0, SGI0, SGI0, SGI148705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32623[] = {SGI148706, SGI0, SGI148707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32624[] = {SGI148708, SGI148709, SGI148710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148711, SGI0, SGI0, SGI0, SGI148712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32625[] = {SGI148713, SGI148714, SGI148715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148716, SGI0, SGI0, SGI0, SGI148717, SGI148717, SGI0, SGI0, NULL};
const Segment * const SG32626[] = {SGI148718, SGI148719, SGI148720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148721, SGI0, SGI0, SGI0, SGI148722, SGI148722, SGI0, SGI0, NULL};
const Segment * const SG32627[] = {SGI148723, SGI148724, SGI148725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148726, SGI0, SGI0, SGI0, SGI148727, SGI148727, SGI0, SGI0, NULL};
const Segment * const SG32628[] = {SGI148728, SGI148729, SGI148730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148731, SGI0, SGI0, SGI0, SGI148732, SGI148732, SGI0, SGI0, NULL};
const Segment * const SG32629[] = {SGI148733, SGI148734, SGI148735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148736, SGI0, SGI0, SGI0, SGI148737, SGI148737, SGI0, SGI0, NULL};
const Segment * const SG32630[] = {SGI148738, SGI148739, SGI148740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148741, SGI0, SGI0, SGI0, SGI148742, SGI148742, SGI0, SGI0, NULL};
const Segment * const SG32631[] = {SGI148743, SGI148744, SGI148745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148747, SGI0, SGI0, SGI0, SGI148749, SGI148749, SGI0, SGI0, NULL};
const Segment * const SG32632[] = {SGI148746, SGI0, SGI148748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148750, SGI148751, SGI148751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32633[] = {SGI148752, SGI148753, SGI148754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32634[] = {SGI148755, SGI148756, SGI148757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148758, SGI0, SGI0, SGI0, SGI148759, SGI148759, SGI0, SGI0, NULL};
const Segment * const SG32635[] = {SGI148760, SGI0, SGI148761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32636[] = {SGI148762, SGI148763, SGI148764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148765, SGI0, SGI0, SGI0, SGI148766, SGI148766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32637[] = {SGI148767, SGI0, SGI148768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148769, SGI148770, SGI148770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32638[] = {SGI148771, SGI0, SGI148772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148773, SGI148774, SGI148774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32639[] = {SGI148775, SGI0, SGI148776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148777, SGI148779, SGI148779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32640[] = {SGI148778, SGI0, SGI148780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148781, SGI148782, SGI148783, SGI0, SGI0, NULL};
const Segment * const SG32641[] = {SGI148784, SGI0, SGI148785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148786, SGI148787, SGI148787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32642[] = {SGI148788, SGI0, SGI148789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148790, SGI148791, SGI148791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32643[] = {SGI148792, SGI0, SGI148793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148794, SGI148795, SGI148795, SGI0, SGI0, NULL};
const Segment * const SG32644[] = {SGI148796, SGI0, SGI148797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148798, SGI148799, SGI148799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32645[] = {SGI148800, SGI0, SGI148801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148802, SGI148803, SGI148803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32646[] = {SGI148804, SGI0, SGI148805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148806, SGI148807, SGI148807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32647[] = {SGI148808, SGI0, SGI148809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148810, SGI148811, SGI148811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32648[] = {SGI148812, SGI0, SGI148813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148814, SGI148815, SGI148815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32649[] = {SGI148816, SGI0, SGI148817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148818, SGI148819, SGI148819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32650[] = {SGI148820, SGI0, SGI148821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148822, SGI148823, SGI148823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32651[] = {SGI148824, SGI0, SGI148825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148826, SGI148827, SGI148827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32652[] = {SGI148828, SGI0, SGI148829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148830, SGI148831, SGI148831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32653[] = {SGI148832, SGI0, SGI148833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148834, SGI148835, SGI148835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32654[] = {SGI148836, SGI0, SGI148837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148838, SGI148839, SGI148839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32655[] = {SGI148840, SGI0, SGI148841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148842, SGI148843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32656[] = {SGI148844, SGI0, SGI148845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148846, SGI148847, SGI148847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32657[] = {SGI148848, SGI0, SGI148849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148850, SGI148851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32658[] = {SGI148852, SGI148853, SGI148854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148855, SGI0, SGI0, SGI0, SGI148856, SGI148856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32659[] = {SGI148857, SGI148858, SGI148859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148860, SGI0, SGI0, SGI0, SGI148861, SGI148861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32660[] = {SGI148862, SGI148863, SGI148864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148865, SGI0, SGI0, SGI0, SGI148866, SGI148866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32661[] = {SGI148867, SGI0, SGI148868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32662[] = {SGI148869, SGI148870, SGI148871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148872, SGI0, SGI0, SGI0, SGI148873, SGI148873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32663[] = {SGI148874, SGI0, SGI148875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32664[] = {SGI148876, SGI148877, SGI148878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148879, SGI0, SGI0, SGI0, SGI148880, SGI148880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32665[] = {SGI148881, SGI0, SGI148882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148883, SGI148884, SGI148884, SGI0, SGI0, NULL};
const Segment * const SG32666[] = {SGI148885, SGI148886, SGI148887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148888, SGI0, SGI0, SGI0, SGI148889, SGI148889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32667[] = {SGI148890, SGI148891, SGI148892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148893, SGI0, SGI0, SGI0, SGI148894, SGI148894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32668[] = {SGI148895, SGI148896, SGI148897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148898, SGI0, SGI0, SGI0, SGI148899, SGI148899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32669[] = {SGI148900, SGI148901, SGI148902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148903, SGI0, SGI0, SGI0, SGI148904, SGI148904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32670[] = {SGI148905, SGI148906, SGI148907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148908, SGI0, SGI0, SGI0, SGI148909, SGI148909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32671[] = {SGI148910, SGI148911, SGI148912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148913, SGI0, SGI0, SGI0, SGI148914, SGI148914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32672[] = {SGI148915, SGI148916, SGI148917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148918, SGI0, SGI0, SGI0, SGI148919, SGI148919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32673[] = {SGI148920, SGI148921, SGI148922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148923, SGI0, SGI0, SGI0, SGI148924, SGI148924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32674[] = {SGI148925, SGI0, SGI148926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148927, SGI148928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32675[] = {SGI148929, SGI148930, SGI148931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148932, SGI0, SGI0, SGI0, SGI148933, SGI148933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32676[] = {SGI148934, SGI148935, SGI148936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148937, SGI0, SGI0, SGI0, SGI148938, SGI148938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32677[] = {SGI148939, SGI0, SGI148940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32678[] = {SGI148941, SGI0, SGI148942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148943, SGI148944, SGI148944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32679[] = {SGI148945, SGI148946, SGI148947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148948, SGI0, SGI0, SGI0, SGI148949, SGI148949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32680[] = {SGI148950, SGI0, SGI148951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32681[] = {SGI148952, SGI148953, SGI148954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148955, SGI0, SGI0, SGI0, SGI148956, SGI148956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32682[] = {SGI148957, SGI148958, SGI148959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148960, SGI0, SGI0, SGI0, SGI148961, SGI148961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32683[] = {SGI148962, SGI148963, SGI148964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148965, SGI0, SGI0, SGI0, SGI148966, SGI148966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32684[] = {SGI148967, SGI0, SGI148968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32685[] = {SGI148969, SGI0, SGI148970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32686[] = {SGI148971, SGI0, SGI148972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148973, SGI148974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32687[] = {SGI148975, SGI0, SGI148976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148977, SGI148978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32688[] = {SGI148979, SGI148980, SGI148981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148982, SGI0, SGI0, SGI0, SGI148983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32689[] = {SGI148984, SGI0, SGI148985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32690[] = {SGI148986, SGI148987, SGI148988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148989, SGI0, SGI0, SGI0, SGI148990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32691[] = {SGI148991, SGI148992, SGI148993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI148994, SGI0, SGI0, SGI0, SGI148995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32692[] = {SGI148996, SGI0, SGI148997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32693[] = {SGI148998, SGI0, SGI148999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32694[] = {SGI149000, SGI149001, SGI149002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149003, SGI0, SGI0, SGI0, SGI149004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32695[] = {SGI149005, SGI149006, SGI149007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149008, SGI0, SGI0, SGI0, SGI149009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32696[] = {SGI149010, SGI0, SGI149011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149012, SGI149013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32697[] = {SGI149014, SGI149015, SGI149016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149017, SGI0, SGI0, SGI0, SGI149018, SGI149018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32698[] = {SGI149019, SGI0, SGI149020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32699[] = {SGI149021, SGI149022, SGI149023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149024, SGI0, SGI0, SGI0, SGI149025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32700[] = {SGI149026, SGI149027, SGI149028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149029, SGI0, SGI0, SGI0, SGI149030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32701[] = {SGI149031, SGI149032, SGI149033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149034, SGI0, SGI0, SGI0, SGI149035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32702[] = {SGI149036, SGI149037, SGI149038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149039, SGI0, SGI0, SGI0, SGI149040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32703[] = {SGI149041, SGI149042, SGI149043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149044, SGI0, SGI0, SGI0, SGI149045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32704[] = {SGI149046, SGI0, SGI149047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32705[] = {SGI149048, SGI0, SGI149049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32706[] = {SGI149050, SGI149051, SGI149052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149053, SGI0, SGI0, SGI0, SGI149054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32707[] = {SGI149055, SGI149056, SGI149057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149058, SGI0, SGI0, SGI0, SGI149059, SGI149059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32708[] = {SGI149060, SGI149061, SGI149062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149063, SGI0, SGI0, SGI0, SGI149064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32709[] = {SGI149065, SGI149066, SGI149067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149068, SGI0, SGI0, SGI0, SGI149069, SGI149069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32710[] = {SGI149070, SGI149071, SGI149072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149073, SGI0, SGI0, SGI0, SGI149074, SGI149074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32711[] = {SGI149075, SGI149076, SGI149077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149078, SGI0, SGI0, SGI0, SGI149079, SGI149079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32712[] = {SGI149080, SGI149081, SGI149082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149083, SGI0, SGI0, SGI0, SGI149084, SGI149084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32713[] = {SGI149085, SGI149086, SGI149087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149088, SGI0, SGI0, SGI0, SGI149089, SGI149089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32714[] = {SGI149090, SGI149091, SGI149092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149093, SGI0, SGI0, SGI0, SGI149094, SGI149094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32715[] = {SGI149095, SGI149096, SGI149097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149098, SGI0, SGI0, SGI0, SGI149099, SGI149099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32716[] = {SGI149100, SGI149101, SGI149102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149103, SGI0, SGI0, SGI0, SGI149104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32717[] = {SGI149105, SGI149106, SGI149107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149108, SGI0, SGI0, SGI0, SGI149109, SGI149109, SGI0, SGI0, NULL};
const Segment * const SG32718[] = {SGI149110, SGI149111, SGI149112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149113, SGI0, SGI0, SGI0, SGI149114, SGI149114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32719[] = {SGI149115, SGI149116, SGI149117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149118, SGI0, SGI0, SGI0, SGI149119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32720[] = {SGI149120, SGI149121, SGI149122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149123, SGI0, SGI0, SGI0, SGI149124, SGI149124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32721[] = {SGI149125, SGI149126, SGI149127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149128, SGI0, SGI0, SGI0, SGI149129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32722[] = {SGI149130, SGI0, SGI149131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32723[] = {SGI149132, SGI149133, SGI149134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149135, SGI0, SGI0, SGI0, SGI149136, SGI149136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32724[] = {SGI149137, SGI0, SGI149138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32725[] = {SGI149139, SGI0, SGI149140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32726[] = {SGI149141, SGI0, SGI149142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149143, SGI149144, SGI149144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32727[] = {SGI149145, SGI0, SGI149146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149147, SGI149148, SGI149148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32728[] = {SGI149149, SGI0, SGI149150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149151, SGI149152, SGI149152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32729[] = {SGI149153, SGI0, SGI149154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149155, SGI149156, SGI149156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32730[] = {SGI149157, SGI149158, SGI149159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149162, SGI0, SGI0, SGI0, SGI149163, SGI149163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32731[] = {SGI149160, SGI0, SGI149161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149164, SGI149165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32732[] = {SGI149166, SGI0, SGI149167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32733[] = {SGI149168, SGI0, SGI149169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149170, SGI149171, SGI149171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32734[] = {SGI149172, SGI149173, SGI149174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149175, SGI0, SGI0, SGI0, SGI149176, SGI149176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32735[] = {SGI149180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32736[] = {SGI149177, SGI149178, SGI149179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149181, SGI0, SGI0, SGI0, SGI149182, SGI149182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32737[] = {SGI149183, SGI0, SGI149185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32738[] = {SGI149184, SGI0, SGI149186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149187, SGI0, SGI0, SGI0, SGI149188, SGI149188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32739[] = {SGI149189, SGI0, SGI149190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149191, SGI0, SGI0, SGI0, SGI149192, SGI149192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32740[] = {SGI149193, SGI149194, SGI149195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149196, SGI0, SGI0, SGI0, SGI149197, SGI149197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32741[] = {SGI149198, SGI149199, SGI149200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149201, SGI0, SGI0, SGI0, SGI149202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32742[] = {SGI149203, SGI149204, SGI149205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149206, SGI0, SGI0, SGI0, SGI149207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32743[] = {SGI149208, SGI0, SGI149209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32744[] = {SGI149210, SGI149211, SGI149212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149213, SGI0, SGI0, SGI0, SGI149214, SGI149214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32745[] = {SGI149215, SGI149216, SGI149217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149218, SGI0, SGI0, SGI0, SGI149219, SGI149219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32746[] = {SGI149220, SGI149221, SGI149222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149223, SGI0, SGI0, SGI0, SGI149224, SGI149224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32747[] = {SGI149225, SGI149226, SGI149227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149228, SGI0, SGI0, SGI0, SGI149229, SGI149229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32748[] = {SGI149230, SGI149231, SGI149232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149233, SGI0, SGI0, SGI0, SGI149234, SGI149234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32749[] = {SGI149235, SGI0, SGI149236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149237, SGI149238, SGI149238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32750[] = {SGI149239, SGI149240, SGI149241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149242, SGI0, SGI0, SGI0, SGI149243, SGI149243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32751[] = {SGI149244, SGI149245, SGI149246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149247, SGI0, SGI0, SGI0, SGI149248, SGI149248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32752[] = {SGI149249, SGI0, SGI149250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32753[] = {SGI149251, SGI0, SGI149252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149253, SGI149254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32754[] = {SGI149255, SGI0, SGI149256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149257, SGI149258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32755[] = {SGI149259, SGI149260, SGI149261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149262, SGI0, SGI0, SGI0, SGI149263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32756[] = {SGI149264, SGI149265, SGI149266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149267, SGI0, SGI0, SGI0, SGI149268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32757[] = {SGI149269, SGI0, SGI149270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32758[] = {SGI149271, SGI149272, SGI149273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149274, SGI0, SGI0, SGI0, SGI149275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32759[] = {SGI149276, SGI149277, SGI149278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149279, SGI0, SGI0, SGI0, SGI149280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32760[] = {SGI149281, SGI0, SGI149282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32761[] = {SGI149283, SGI0, SGI149284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149285, SGI149286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32762[] = {SGI149287, SGI149288, SGI149289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149290, SGI0, SGI0, SGI0, SGI149291, SGI149291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32763[] = {SGI149292, SGI149293, SGI149294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149295, SGI0, SGI0, SGI0, SGI149296, SGI149296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32764[] = {SGI149297, SGI0, SGI149298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32765[] = {SGI149299, SGI0, SGI149300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32766[] = {SGI149301, SGI0, SGI149302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149303, SGI0, SGI0, SGI0, SGI149304, SGI149304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32767[] = {SGI149305, SGI0, SGI149306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149307, SGI0, SGI0, SGI0, SGI149308, SGI149308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32768[] = {SGI149309, SGI149310, SGI149311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149312, SGI0, SGI0, SGI0, SGI149313, SGI149313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32769[] = {SGI149314, SGI149315, SGI149316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149317, SGI0, SGI0, SGI0, SGI149318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32770[] = {SGI149319, SGI149320, SGI149321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149322, SGI0, SGI0, SGI0, SGI149323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32771[] = {SGI149324, SGI149325, SGI149326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149327, SGI0, SGI0, SGI0, SGI149328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32772[] = {SGI149329, SGI0, SGI149330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149331, SGI0, SGI0, SGI0, SGI149332, SGI149332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32773[] = {SGI149333, SGI149334, SGI149335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149336, SGI0, SGI0, SGI0, SGI149337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32774[] = {SGI149338, SGI149339, SGI149340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149341, SGI0, SGI0, SGI0, SGI149342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32775[] = {SGI149343, SGI0, SGI149344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32776[] = {SGI149345, SGI149346, SGI149347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149348, SGI0, SGI0, SGI0, SGI149349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32777[] = {SGI149350, SGI149351, SGI149352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149353, SGI0, SGI0, SGI0, SGI149354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32778[] = {SGI149355, SGI149356, SGI149357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149358, SGI0, SGI0, SGI0, SGI149359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32779[] = {SGI149360, SGI149361, SGI149362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149363, SGI0, SGI0, SGI0, SGI149364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32780[] = {SGI149365, SGI149366, SGI149367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149368, SGI0, SGI0, SGI0, SGI149369, SGI149369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32781[] = {SGI149370, SGI0, SGI149371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149372, SGI149373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32782[] = {SGI149374, SGI149375, SGI149376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149377, SGI0, SGI0, SGI0, SGI149378, SGI149378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32783[] = {SGI149379, SGI149380, SGI149381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149382, SGI0, SGI0, SGI0, SGI149383, SGI149383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32784[] = {SGI149384, SGI149385, SGI149386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149387, SGI0, SGI0, SGI0, SGI149388, SGI149388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32785[] = {SGI149389, SGI149390, SGI149391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149392, SGI0, SGI0, SGI0, SGI149393, SGI149393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32786[] = {SGI149394, SGI0, SGI149395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32787[] = {SGI149396, SGI149397, SGI149398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149399, SGI0, SGI0, SGI0, SGI149400, SGI149400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32788[] = {SGI149401, SGI149402, SGI149403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149404, SGI0, SGI0, SGI0, SGI149405, SGI149405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32789[] = {SGI149406, SGI0, SGI149407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32790[] = {SGI149408, SGI0, SGI149409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149410, SGI149411, SGI149411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32791[] = {SGI149412, SGI0, SGI149413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149414, SGI149415, SGI149415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32792[] = {SGI149416, SGI149417, SGI149418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149419, SGI0, SGI0, SGI0, SGI149420, SGI149420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32793[] = {SGI149421, SGI0, SGI149422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32794[] = {SGI149423, SGI0, SGI149424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149425, SGI0, SGI0, SGI0, SGI149426, SGI149426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32795[] = {SGI149427, SGI149428, SGI149429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149430, SGI0, SGI0, SGI0, SGI149431, SGI149431, SGI0, SGI149432, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32796[] = {SGI149433, SGI149434, SGI149435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149436, SGI0, SGI0, SGI0, SGI149437, SGI149437, SGI0, SGI149438, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32797[] = {SGI149439, SGI149440, SGI149441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149442, SGI0, SGI0, SGI0, SGI149443, SGI149443, SGI0, SGI149444, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32798[] = {SGI149445, SGI149446, SGI149447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149448, SGI0, SGI0, SGI0, SGI149449, SGI149449, SGI0, SGI149450, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32799[] = {SGI149451, SGI149453, SGI149455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149457, SGI0, SGI0, SGI0, SGI149459, SGI149459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32800[] = {SGI149452, SGI149454, SGI149456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149458, SGI0, SGI0, SGI0, SGI149460, SGI149460, SGI0, SGI149461, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32801[] = {SGI149462, SGI149463, SGI149464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149465, SGI0, SGI0, SGI0, SGI149466, SGI149466, SGI0, SGI149467, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32802[] = {SGI149468, SGI0, SGI149469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32803[] = {SGI149470, SGI0, SGI149471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149472, SGI149474, SGI149474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32804[] = {SGI149473, SGI149475, SGI149476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149477, SGI0, SGI0, SGI0, SGI149478, SGI149478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32805[] = {SGI149479, SGI0, SGI149480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149481, SGI149482, SGI149482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32806[] = {SGI149483, SGI0, SGI149484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149485, SGI149486, SGI149486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32807[] = {SGI149487, SGI149488, SGI149489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149490, SGI0, SGI0, SGI0, SGI149491, SGI149491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32808[] = {SGI149492, SGI0, SGI149493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32809[] = {SGI149494, SGI0, SGI149495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149496, SGI0, SGI0, SGI0, SGI149497, SGI149497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32810[] = {SGI149498, SGI0, SGI149499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149500, SGI149501, SGI149501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32811[] = {SGI149502, SGI0, SGI149503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149504, SGI149505, SGI149505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32812[] = {SGI149506, SGI0, SGI149507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149508, SGI149509, SGI149509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32813[] = {SGI149510, SGI0, SGI149511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149512, SGI149513, SGI149513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32814[] = {SGI149514, SGI149515, SGI149516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149517, SGI0, SGI0, SGI0, SGI149518, SGI149518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32815[] = {SGI149519, SGI149520, SGI149521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149522, SGI0, SGI0, SGI0, SGI149523, SGI149523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32816[] = {SGI149524, SGI0, SGI149525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32817[] = {SGI149526, SGI0, SGI149527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32818[] = {SGI149528, SGI0, SGI149529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149530, SGI149531, SGI149531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32819[] = {SGI149532, SGI0, SGI149533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149534, SGI149535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32820[] = {SGI149536, SGI149537, SGI149538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149539, SGI0, SGI0, SGI0, SGI149540, SGI149540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32821[] = {SGI149541, SGI149542, SGI149543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149544, SGI0, SGI0, SGI0, SGI149545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32822[] = {SGI149546, SGI0, SGI149547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32823[] = {SGI149548, SGI149549, SGI149550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149551, SGI0, SGI0, SGI0, SGI149552, SGI149552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32824[] = {SGI149553, SGI149554, SGI149555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149556, SGI0, SGI0, SGI0, SGI149557, SGI149557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32825[] = {SGI149558, SGI0, SGI149559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32826[] = {SGI149560, SGI149561, SGI149562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149563, SGI0, SGI0, SGI0, SGI149564, SGI149564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32827[] = {SGI149565, SGI149566, SGI149567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149568, SGI0, SGI0, SGI0, SGI149569, SGI149569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32828[] = {SGI149570, SGI149571, SGI149572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149573, SGI0, SGI0, SGI0, SGI149574, SGI149574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32829[] = {SGI149575, SGI0, SGI149576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32830[] = {SGI149577, SGI0, SGI149578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149579, SGI0, SGI0, SGI0, SGI149580, SGI149580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32831[] = {SGI149581, SGI149582, SGI149583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149584, SGI0, SGI0, SGI0, SGI149585, SGI149585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32832[] = {SGI149586, SGI149587, SGI149588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149589, SGI0, SGI0, SGI0, SGI149590, SGI149590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32833[] = {SGI149591, SGI149592, SGI149593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149594, SGI0, SGI0, SGI0, SGI149595, SGI149595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32834[] = {SGI149596, SGI149597, SGI149598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149599, SGI0, SGI0, SGI0, SGI149600, SGI149600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32835[] = {SGI149601, SGI149602, SGI149603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149604, SGI0, SGI0, SGI0, SGI149605, SGI149605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32836[] = {SGI149606, SGI0, SGI149607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149608, SGI149609, SGI149609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32837[] = {SGI149610, SGI0, SGI149611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149612, SGI149613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32838[] = {SGI149614, SGI0, SGI149615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149616, SGI149617, SGI149617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32839[] = {SGI149618, SGI149619, SGI149620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149621, SGI0, SGI0, SGI0, SGI149622, SGI149622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32840[] = {SGI149623, SGI0, SGI149624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149625, SGI149626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32841[] = {SGI149627, SGI0, SGI149628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149629, SGI149630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32842[] = {SGI149631, SGI0, SGI149632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149633, SGI149634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32843[] = {SGI149635, SGI0, SGI149636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149637, SGI149638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32844[] = {SGI149639, SGI0, SGI149640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149641, SGI149642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32845[] = {SGI149643, SGI0, SGI149644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149645, SGI149646, SGI149646, SGI0, SGI0, NULL};
const Segment * const SG32846[] = {SGI149647, SGI0, SGI149648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149649, SGI149650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32847[] = {SGI149651, SGI0, SGI149652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149653, SGI149654, SGI149654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32848[] = {SGI149655, SGI149656, SGI149657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149658, SGI0, SGI0, SGI0, SGI149659, SGI149659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32849[] = {SGI149660, SGI0, SGI149661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32850[] = {SGI149662, SGI149663, SGI149664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149665, SGI0, SGI0, SGI0, SGI149666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32851[] = {SGI149667, SGI0, SGI149668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32852[] = {SGI149669, SGI149670, SGI149671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149672, SGI0, SGI0, SGI0, SGI149673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32853[] = {SGI149674, SGI149675, SGI149676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149677, SGI0, SGI0, SGI0, SGI149678, SGI149678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32854[] = {SGI149679, SGI149680, SGI149681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149682, SGI0, SGI0, SGI0, SGI149683, SGI149683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32855[] = {SGI149684, SGI0, SGI149685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32856[] = {SGI149686, SGI149687, SGI149688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149689, SGI0, SGI0, SGI0, SGI149690, SGI149690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32857[] = {SGI149691, SGI149692, SGI149693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149694, SGI0, SGI0, SGI0, SGI149695, SGI149695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32858[] = {SGI149696, SGI149697, SGI149698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149699, SGI0, SGI0, SGI0, SGI149700, SGI149700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32859[] = {SGI149701, SGI149702, SGI149703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149704, SGI0, SGI0, SGI0, SGI149705, SGI149705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32860[] = {SGI149706, SGI149707, SGI149708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149709, SGI0, SGI0, SGI0, SGI149710, SGI149710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32861[] = {SGI149711, SGI149712, SGI149713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149714, SGI0, SGI0, SGI0, SGI149715, SGI149715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32862[] = {SGI149716, SGI149717, SGI149718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149719, SGI0, SGI0, SGI0, SGI149720, SGI149720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32863[] = {SGI149721, SGI149722, SGI149723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149724, SGI0, SGI0, SGI0, SGI149725, SGI149725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32864[] = {SGI149726, SGI149727, SGI149728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149729, SGI0, SGI0, SGI0, SGI149730, SGI149730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32865[] = {SGI149731, SGI149732, SGI149733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149734, SGI0, SGI0, SGI0, SGI149735, SGI149735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32866[] = {SGI149736, SGI149737, SGI149738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149739, SGI0, SGI0, SGI0, SGI149740, SGI149740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32867[] = {SGI149741, SGI0, SGI149742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149743, SGI149744, SGI149744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32868[] = {SGI149745, SGI0, SGI149746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149747, SGI149748, SGI149748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32869[] = {SGI149749, SGI0, SGI149750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149751, SGI149752, SGI149752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32870[] = {SGI149753, SGI0, SGI149754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149755, SGI149756, SGI149756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32871[] = {SGI149757, SGI0, SGI149758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149760, SGI149762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32872[] = {SGI149759, SGI0, SGI149761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149763, SGI149764, SGI149764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32873[] = {SGI149765, SGI0, SGI149766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149767, SGI149768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32874[] = {SGI149769, SGI0, SGI149770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149771, SGI149772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32875[] = {SGI149773, SGI0, SGI149774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149775, SGI149776, SGI149776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32876[] = {SGI149777, SGI0, SGI149778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149779, SGI149780, SGI149780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32877[] = {SGI149781, SGI0, SGI149782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149783, SGI149784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32878[] = {SGI149785, SGI149786, SGI149787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149788, SGI0, SGI0, SGI0, SGI149789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32879[] = {SGI149790, SGI149791, SGI149792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149793, SGI0, SGI0, SGI0, SGI149794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32880[] = {SGI149795, SGI149796, SGI149797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149798, SGI0, SGI0, SGI0, SGI149799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32881[] = {SGI149800, SGI149801, SGI149802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149803, SGI0, SGI0, SGI0, SGI149804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32882[] = {SGI149805, SGI149806, SGI149807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149808, SGI0, SGI0, SGI0, SGI149809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32883[] = {SGI149810, SGI149811, SGI149812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149813, SGI0, SGI0, SGI0, SGI149814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32884[] = {SGI149815, SGI149816, SGI149817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149818, SGI0, SGI0, SGI0, SGI149819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32885[] = {SGI149820, SGI149821, SGI149822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149823, SGI0, SGI0, SGI0, SGI149824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32886[] = {SGI149825, SGI149826, SGI149827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149828, SGI0, SGI0, SGI0, SGI149829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32887[] = {SGI149830, SGI149831, SGI149832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149833, SGI0, SGI0, SGI0, SGI149834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32888[] = {SGI149835, SGI149836, SGI149837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149838, SGI0, SGI0, SGI0, SGI149839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32889[] = {SGI149840, SGI0, SGI149841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32890[] = {SGI149842, SGI0, SGI149843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149844, SGI149845, SGI149845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32891[] = {SGI149846, SGI0, SGI149847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149848, SGI149849, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32892[] = {SGI149850, SGI0, SGI149851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149852, SGI149853, SGI149853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32893[] = {SGI149854, SGI149855, SGI149856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149857, SGI0, SGI0, SGI0, SGI149858, SGI149858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32894[] = {SGI149859, SGI0, SGI149860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32895[] = {SGI149861, SGI0, SGI149862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149863, SGI0, SGI0, SGI0, SGI149864, SGI149864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32896[] = {SGI149865, SGI149866, SGI149867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149868, SGI0, SGI0, SGI0, SGI149869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32897[] = {SGI149870, SGI0, SGI149871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32898[] = {SGI149872, SGI149873, SGI149874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149875, SGI0, SGI0, SGI0, SGI149876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32899[] = {SGI149877, SGI149878, SGI149879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149880, SGI0, SGI0, SGI0, SGI149881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32900[] = {SGI149882, SGI149883, SGI149884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149885, SGI0, SGI0, SGI0, SGI149886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32901[] = {SGI149887, SGI0, SGI149888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32902[] = {SGI149889, SGI0, SGI149890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32903[] = {SGI149891, SGI149892, SGI149893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149894, SGI0, SGI0, SGI0, SGI149895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32904[] = {SGI149896, SGI0, SGI149897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32905[] = {SGI149898, SGI0, SGI149899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149900, SGI0, SGI0, SGI0, SGI149901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32906[] = {SGI149902, SGI0, SGI149903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149904, SGI149905, SGI149905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32907[] = {SGI149906, SGI149907, SGI149908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149909, SGI0, SGI0, SGI0, SGI149910, SGI149910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32908[] = {SGI149911, SGI0, SGI149912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149913, SGI149914, SGI149914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32909[] = {SGI149915, SGI149916, SGI149917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149920, SGI0, SGI0, SGI0, SGI149921, SGI149921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32910[] = {SGI149918, SGI0, SGI149919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149922, SGI149923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32911[] = {SGI149924, SGI149925, SGI149926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149927, SGI0, SGI0, SGI0, SGI149928, SGI149928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32912[] = {SGI149929, SGI149930, SGI149931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149932, SGI0, SGI0, SGI0, SGI149933, SGI149933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32913[] = {SGI149934, SGI149935, SGI149936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149937, SGI0, SGI0, SGI0, SGI149938, SGI149938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32914[] = {SGI149939, SGI149940, SGI149941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149942, SGI0, SGI0, SGI0, SGI149943, SGI149943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32915[] = {SGI149944, SGI0, SGI149945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32916[] = {SGI149946, SGI0, SGI149947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32917[] = {SGI149948, SGI149949, SGI149950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149951, SGI0, SGI0, SGI0, SGI149952, SGI149952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32918[] = {SGI149953, SGI149954, SGI149955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149956, SGI0, SGI0, SGI0, SGI149958, SGI149958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32919[] = {SGI149957, SGI0, SGI149959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149960, SGI149961, SGI149961, SGI0, SGI0, NULL};
const Segment * const SG32920[] = {SGI149962, SGI0, SGI149963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32921[] = {SGI149964, SGI149965, SGI149966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149967, SGI0, SGI0, SGI0, SGI149968, SGI149968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32922[] = {SGI149971, SGI0, SGI149972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32923[] = {SGI149969, SGI0, SGI149970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149973, SGI149974, SGI149974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32924[] = {SGI149975, SGI0, SGI149976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149977, SGI149978, SGI149978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32925[] = {SGI149979, SGI0, SGI149980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149981, SGI149982, SGI149982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32926[] = {SGI149983, SGI0, SGI149984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149985, SGI149986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32927[] = {SGI149987, SGI0, SGI149988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149989, SGI149990, SGI149990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32928[] = {SGI149991, SGI0, SGI149992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149993, SGI149994, SGI149994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32929[] = {SGI149995, SGI0, SGI149996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI149997, SGI149998, SGI149998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32930[] = {SGI149999, SGI0, SGI150000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150001, SGI150002, SGI150002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32931[] = {SGI150003, SGI0, SGI150004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150005, SGI150006, SGI150006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32932[] = {SGI150007, SGI0, SGI150008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150009, SGI150010, SGI150010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32933[] = {SGI150011, SGI0, SGI150012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150013, SGI150014, SGI150014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32934[] = {SGI150015, SGI0, SGI150016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150018, SGI150020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32935[] = {SGI150017, SGI150019, SGI150021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150022, SGI0, SGI0, SGI0, SGI150023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32936[] = {SGI150024, SGI0, SGI150025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150027, SGI150029, SGI150029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32937[] = {SGI150026, SGI150028, SGI150030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150031, SGI0, SGI0, SGI0, SGI150032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32938[] = {SGI150033, SGI0, SGI150034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32939[] = {SGI150036, SGI0, SGI150037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32940[] = {SGI150035, SGI150038, SGI150039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150040, SGI0, SGI0, SGI0, SGI150041, SGI150041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32941[] = {SGI150042, SGI150043, SGI150044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150045, SGI0, SGI0, SGI0, SGI150046, SGI150046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32942[] = {SGI150047, SGI150048, SGI150049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150050, SGI0, SGI0, SGI0, SGI150051, SGI150051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32943[] = {SGI150052, SGI150053, SGI150054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150055, SGI0, SGI0, SGI0, SGI150056, SGI150056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32944[] = {SGI150057, SGI150058, SGI150059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150060, SGI0, SGI0, SGI0, SGI150061, SGI150061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32945[] = {SGI150062, SGI150063, SGI150064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150065, SGI0, SGI0, SGI0, SGI150066, SGI150066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32946[] = {SGI150067, SGI0, SGI150068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32947[] = {SGI150069, SGI150070, SGI150071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150072, SGI0, SGI0, SGI0, SGI150073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32948[] = {SGI150074, SGI150075, SGI150076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150077, SGI0, SGI0, SGI0, SGI150078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32949[] = {SGI150079, SGI150080, SGI150081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150082, SGI0, SGI0, SGI0, SGI150083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32950[] = {SGI150084, SGI0, SGI150085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32951[] = {SGI150086, SGI150087, SGI150088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150089, SGI0, SGI0, SGI0, SGI150090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32952[] = {SGI150091, SGI0, SGI150092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150093, SGI150094, SGI150094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32953[] = {SGI150095, SGI0, SGI150096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150097, SGI150098, SGI150098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32954[] = {SGI150099, SGI0, SGI150100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150101, SGI150102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32955[] = {SGI150103, SGI0, SGI150104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150105, SGI150106, SGI150106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32956[] = {SGI150107, SGI0, SGI150108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150109, SGI150110, SGI150110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32957[] = {SGI150111, SGI0, SGI150112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150113, SGI150114, SGI150114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32958[] = {SGI150115, SGI0, SGI150116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150119, SGI150120, SGI150120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32959[] = {SGI150117, SGI0, SGI150118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150121, SGI150122, SGI150122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32960[] = {SGI150123, SGI0, SGI150124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150125, SGI150126, SGI150126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32961[] = {SGI150127, SGI0, SGI150128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150129, SGI150130, SGI150130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32962[] = {SGI150131, SGI0, SGI150132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150133, SGI150134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32963[] = {SGI150135, SGI0, SGI150136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150137, SGI150138, SGI150138, SGI0, SGI0, NULL};
const Segment * const SG32964[] = {SGI150139, SGI0, SGI150140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150141, SGI150142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32965[] = {SGI150143, SGI0, SGI150144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150145, SGI150146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32966[] = {SGI150147, SGI0, SGI150148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150149, SGI150150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32967[] = {SGI150151, SGI150152, SGI150153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150154, SGI0, SGI0, SGI0, SGI150155, SGI150155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32968[] = {SGI150156, SGI0, SGI150157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32969[] = {SGI150158, SGI0, SGI150159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150160, SGI150161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32970[] = {SGI150162, SGI150163, SGI150164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150165, SGI0, SGI0, SGI0, SGI150166, SGI150166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32971[] = {SGI150167, SGI150168, SGI150169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150170, SGI0, SGI0, SGI0, SGI150171, SGI150171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32972[] = {SGI150172, SGI150173, SGI150174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150175, SGI0, SGI0, SGI0, SGI150176, SGI150176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32973[] = {SGI150177, SGI0, SGI150178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32974[] = {SGI150179, SGI0, SGI150181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32975[] = {SGI150180, SGI0, SGI150182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150183, SGI150184, SGI150184, SGI0, SGI0, NULL};
const Segment * const SG32976[] = {SGI150185, SGI0, SGI150186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150187, SGI150188, SGI150188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32977[] = {SGI150189, SGI0, SGI150190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150191, SGI150192, SGI150192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32978[] = {SGI150193, SGI150194, SGI150195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150196, SGI0, SGI0, SGI0, SGI150197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32979[] = {SGI150198, SGI150199, SGI150200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150201, SGI0, SGI0, SGI0, SGI150202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32980[] = {SGI150203, SGI150204, SGI150205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150206, SGI0, SGI0, SGI0, SGI150207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32981[] = {SGI150208, SGI0, SGI150209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32982[] = {SGI150210, SGI0, SGI150211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150212, SGI150213, SGI150213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32983[] = {SGI150214, SGI0, SGI150215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150216, SGI150217, SGI150217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32984[] = {SGI150218, SGI0, SGI150219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150220, SGI150221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32985[] = {SGI150222, SGI150223, SGI150224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150225, SGI0, SGI0, SGI0, SGI150226, SGI150226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32986[] = {SGI150227, SGI150228, SGI150229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150230, SGI0, SGI0, SGI0, SGI150231, SGI150231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32987[] = {SGI150232, SGI0, SGI150234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150237, SGI150238, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32988[] = {SGI150233, SGI150235, SGI150236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150239, SGI0, SGI0, SGI0, SGI150240, SGI150240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32989[] = {SGI150241, SGI0, SGI150242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32990[] = {SGI150243, SGI150244, SGI150245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150246, SGI0, SGI0, SGI0, SGI150247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32991[] = {SGI150248, SGI0, SGI150249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150250, SGI0, SGI0, SGI0, SGI150251, SGI150251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32992[] = {SGI150252, SGI150253, SGI150254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150255, SGI0, SGI0, SGI0, SGI150256, SGI150256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32993[] = {SGI150257, SGI150258, SGI150259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150260, SGI0, SGI0, SGI0, SGI150261, SGI150261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32994[] = {SGI150262, SGI0, SGI150263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32995[] = {SGI150264, SGI150265, SGI150266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150267, SGI0, SGI0, SGI0, SGI150268, SGI150268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32996[] = {SGI150269, SGI0, SGI150270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32997[] = {SGI150271, SGI0, SGI150272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150273, SGI0, SGI0, SGI0, SGI150274, SGI150274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32998[] = {SGI150275, SGI150276, SGI150277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150278, SGI0, SGI0, SGI0, SGI150279, SGI150279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG32999[] = {SGI150280, SGI150281, SGI150282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150283, SGI0, SGI0, SGI0, SGI150284, SGI150284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33000[] = {SGI150285, SGI150286, SGI150287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150288, SGI0, SGI0, SGI0, SGI150289, SGI150289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33001[] = {SGI150290, SGI0, SGI150291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33002[] = {SGI150292, SGI150293, SGI150294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150295, SGI0, SGI0, SGI0, SGI150296, SGI150296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33003[] = {SGI150297, SGI150298, SGI150299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150300, SGI150301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33004[] = {SGI150302, SGI150303, SGI150304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150305, SGI0, SGI0, SGI0, SGI150306, SGI150306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33005[] = {SGI150308, SGI150310, SGI150312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150314, SGI0, SGI0, SGI0, SGI150316, SGI150316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33006[] = {SGI150307, SGI150309, SGI150311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150313, SGI0, SGI0, SGI0, SGI150315, SGI150315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33007[] = {SGI150317, SGI150318, SGI150319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150320, SGI0, SGI0, SGI0, SGI150321, SGI150321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33008[] = {SGI150322, SGI150323, SGI150324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150325, SGI150325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33009[] = {SGI150326, SGI150327, SGI150328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150329, SGI0, SGI0, SGI0, SGI150330, SGI150330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33010[] = {SGI150331, SGI150332, SGI150333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150334, SGI0, SGI0, SGI0, SGI150335, SGI150335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33011[] = {SGI150336, SGI150337, SGI150338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150339, SGI0, SGI0, SGI0, SGI150340, SGI150340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33012[] = {SGI150341, SGI150343, SGI150344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150347, SGI0, SGI0, SGI0, SGI150349, SGI150349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33013[] = {SGI150342, SGI150345, SGI150346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150348, SGI150348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33014[] = {SGI150350, SGI150351, SGI150352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150353, SGI150354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33015[] = {SGI150355, SGI0, SGI150356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33016[] = {SGI150357, SGI150358, SGI150359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150360, SGI0, SGI0, SGI0, SGI150361, SGI150361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33017[] = {SGI0, SGI0, SGI150362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33018[] = {SGI150363, SGI150364, SGI150365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150366, SGI0, SGI0, SGI0, SGI150367, SGI150367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33019[] = {SGI150368, SGI0, SGI150369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33020[] = {SGI150370, SGI0, SGI150371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150372, SGI150373, SGI150373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33021[] = {SGI150374, SGI0, SGI150375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150376, SGI150377, SGI150377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33022[] = {SGI150378, SGI0, SGI150379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150380, SGI150381, SGI150381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33023[] = {SGI150382, SGI150383, SGI150384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150385, SGI0, SGI0, SGI0, SGI150386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33024[] = {SGI150387, SGI0, SGI150388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33025[] = {SGI150389, SGI0, SGI150390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150391, SGI150392, SGI150392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33026[] = {SGI150393, SGI0, SGI150394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150395, SGI150396, SGI150396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33027[] = {SGI150397, SGI0, SGI150398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150399, SGI150400, SGI150400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33028[] = {SGI150401, SGI0, SGI150402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150403, SGI150404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33029[] = {SGI150405, SGI150406, SGI150407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150408, SGI150409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33030[] = {SGI150410, SGI0, SGI150411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150412, SGI150413, SGI150413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33031[] = {SGI150414, SGI0, SGI150415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150416, SGI150417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33032[] = {SGI150418, SGI0, SGI150419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150420, SGI150421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33033[] = {SGI150422, SGI0, SGI150423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150424, SGI150425, SGI150425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33034[] = {SGI150426, SGI0, SGI150427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150428, SGI150429, SGI150429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33035[] = {SGI150430, SGI0, SGI150431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150432, SGI150433, SGI150433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33036[] = {SGI150434, SGI0, SGI150435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150436, SGI150437, SGI150437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33037[] = {SGI150438, SGI0, SGI150439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150440, SGI150441, SGI150441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33038[] = {SGI150442, SGI0, SGI150443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150444, SGI150445, SGI150445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33039[] = {SGI150446, SGI0, SGI150447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150448, SGI150449, SGI150449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33040[] = {SGI150450, SGI0, SGI150451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150452, SGI150453, SGI150453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33041[] = {SGI150454, SGI0, SGI150455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150456, SGI150457, SGI150457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33042[] = {SGI150458, SGI0, SGI150459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150460, SGI150461, SGI150461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33043[] = {SGI150462, SGI0, SGI150463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150464, SGI150465, SGI150465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33044[] = {SGI150466, SGI0, SGI150467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150468, SGI150469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33045[] = {SGI150470, SGI0, SGI150471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150472, SGI150473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33046[] = {SGI150474, SGI150475, SGI150476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150477, SGI0, SGI0, SGI0, SGI150478, SGI150478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33047[] = {SGI150479, SGI0, SGI150480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33048[] = {SGI150481, SGI0, SGI150482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150483, SGI150484, SGI150484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33049[] = {SGI150485, SGI0, SGI150486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150487, SGI150488, SGI150488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33050[] = {SGI150489, SGI0, SGI150490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150491, SGI150492, SGI150492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33051[] = {SGI150493, SGI0, SGI150494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150495, SGI150496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33052[] = {SGI150497, SGI0, SGI150498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150499, SGI150500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33053[] = {SGI150501, SGI0, SGI150502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150503, SGI150504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33054[] = {SGI150505, SGI150506, SGI150507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150508, SGI0, SGI0, SGI0, SGI150509, SGI150509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33055[] = {SGI150510, SGI0, SGI150511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33056[] = {SGI150512, SGI0, SGI150513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150514, SGI0, SGI0, SGI0, SGI150515, SGI150515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33057[] = {SGI150516, SGI150517, SGI150518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150519, SGI0, SGI0, SGI0, SGI150520, SGI150520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33058[] = {SGI150521, SGI150522, SGI150523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150524, SGI0, SGI0, SGI0, SGI150526, SGI150526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33059[] = {SGI150525, SGI150527, SGI150528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150529, SGI0, SGI0, SGI0, SGI150530, SGI150530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33060[] = {SGI150531, SGI0, SGI150532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150533, SGI0, SGI0, SGI0, SGI150534, SGI150534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33061[] = {SGI150535, SGI0, SGI150536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33062[] = {SGI150537, SGI150538, SGI150539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150540, SGI0, SGI0, SGI0, SGI150541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33063[] = {SGI150542, SGI150543, SGI150544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150545, SGI0, SGI0, SGI0, SGI150548, SGI150548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33064[] = {SGI150546, SGI150547, SGI150549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150550, SGI0, SGI0, SGI0, SGI150551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33065[] = {SGI150553, SGI0, SGI150555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33066[] = {SGI150552, SGI150554, SGI150556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150557, SGI0, SGI0, SGI0, SGI150558, SGI150558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33067[] = {SGI150559, SGI0, SGI150560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33068[] = {SGI150561, SGI150562, SGI150563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150564, SGI0, SGI0, SGI0, SGI150565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33069[] = {SGI150566, SGI150567, SGI150568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150569, SGI0, SGI0, SGI0, SGI150570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33070[] = {SGI150571, SGI150572, SGI150573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150574, SGI150574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33071[] = {SGI150575, SGI150576, SGI150577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150578, SGI0, SGI0, SGI0, SGI150579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33072[] = {SGI150580, SGI150581, SGI150582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150583, SGI150583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33073[] = {SGI150584, SGI150585, SGI150586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150587, SGI0, SGI0, SGI0, SGI150588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33074[] = {SGI0, SGI150589, SGI150590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150591, SGI0, SGI0, SGI0, SGI150592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33075[] = {SGI150593, SGI0, SGI150594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150595, SGI0, SGI0, SGI0, SGI150597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33076[] = {SGI150596, SGI0, SGI150598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150599, SGI150600, SGI150600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33077[] = {SGI150601, SGI0, SGI150602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150603, SGI150604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33078[] = {SGI150605, SGI150606, SGI150607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150608, SGI0, SGI0, SGI0, SGI150609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33079[] = {SGI150610, SGI0, SGI150611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150612, SGI150613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33080[] = {SGI150614, SGI150615, SGI150616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150617, SGI0, SGI0, SGI0, SGI150618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33081[] = {SGI150619, SGI150620, SGI150621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150622, SGI0, SGI0, SGI0, SGI150623, SGI150623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33082[] = {SGI150624, SGI150625, SGI150626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150627, SGI0, SGI0, SGI0, SGI150628, SGI150628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33083[] = {SGI150629, SGI150630, SGI150631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150632, SGI150632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33084[] = {SGI150633, SGI150634, SGI150635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150636, SGI0, SGI0, SGI0, SGI150637, SGI150637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33085[] = {SGI150638, SGI150639, SGI150640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150641, SGI0, SGI0, SGI0, SGI150642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33086[] = {SGI150643, SGI150644, SGI150645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150646, SGI0, SGI0, SGI0, SGI150647, SGI150647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33087[] = {SGI150648, SGI150649, SGI150650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150651, SGI150651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33088[] = {SGI150652, SGI150653, SGI150654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150658, SGI0, SGI0, SGI0, SGI150659, SGI150659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33089[] = {SGI150655, SGI150656, SGI150657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150660, SGI0, SGI0, SGI0, SGI150661, SGI150661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33090[] = {SGI150662, SGI150663, SGI150665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150668, SGI0, SGI0, SGI0, SGI150669, SGI150669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33091[] = {SGI150664, SGI150666, SGI150667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150670, SGI150670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33092[] = {SGI150671, SGI150672, SGI150673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150675, SGI0, SGI0, SGI0, SGI150678, SGI150678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33093[] = {SGI150674, SGI150676, SGI150677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150679, SGI0, SGI0, SGI0, SGI150680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33094[] = {SGI150681, SGI0, SGI150682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33095[] = {SGI150683, SGI150684, SGI150685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150686, SGI0, SGI0, SGI0, SGI150687, SGI150687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33096[] = {SGI150688, SGI150689, SGI150690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150692, SGI0, SGI0, SGI0, SGI150695, SGI150695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33097[] = {SGI150691, SGI150693, SGI150694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150696, SGI0, SGI0, SGI0, SGI150697, SGI150698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33098[] = {SGI150699, SGI150700, SGI150701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150703, SGI0, SGI0, SGI0, SGI150706, SGI150706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33099[] = {SGI150702, SGI150704, SGI150705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33100[] = {SGI150707, SGI150708, SGI150709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150713, SGI0, SGI0, SGI0, SGI150714, SGI150714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33101[] = {SGI150710, SGI150711, SGI150712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33102[] = {SGI150715, SGI150716, SGI150718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33103[] = {SGI150717, SGI150719, SGI150720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150721, SGI0, SGI0, SGI0, SGI150722, SGI150722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33104[] = {SGI150723, SGI150724, SGI150725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33105[] = {SGI150726, SGI150727, SGI150728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150729, SGI0, SGI0, SGI0, SGI150731, SGI150731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33106[] = {SGI150730, SGI150732, SGI150733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33107[] = {SGI150734, SGI0, SGI150735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150736, SGI150737, SGI150737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33108[] = {SGI150738, SGI150739, SGI150741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33109[] = {SGI150740, SGI0, SGI150742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150743, SGI150744, SGI150744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33110[] = {SGI150745, SGI150746, SGI150747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33111[] = {SGI150748, SGI0, SGI150749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150750, SGI150751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33112[] = {SGI150752, SGI150753, SGI150754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33113[] = {SGI150755, SGI150756, SGI150757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33114[] = {SGI150758, SGI0, SGI150759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150760, SGI150761, SGI150761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33115[] = {SGI150762, SGI0, SGI150763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33116[] = {SGI0, SGI0, SGI150764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33117[] = {SGI0, SGI150765, SGI150766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150767, SGI150767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33118[] = {SGI150768, SGI150769, SGI150770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150771, SGI150771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33119[] = {SGI150772, SGI150773, SGI150774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150775, SGI0, SGI0, SGI0, SGI150776, SGI150776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33120[] = {SGI150777, SGI150778, SGI150779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150780, SGI150780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33121[] = {SGI150781, SGI150782, SGI150783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150784, SGI0, SGI0, SGI0, SGI150786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33122[] = {SGI150785, SGI150787, SGI150788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150789, SGI0, SGI0, SGI0, SGI150790, SGI150790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33123[] = {SGI150791, SGI0, SGI150792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33124[] = {SGI150793, SGI150794, SGI150795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150796, SGI150796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33125[] = {SGI150797, SGI150798, SGI150799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150800, SGI0, SGI0, SGI0, SGI150801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33126[] = {SGI150802, SGI150803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33127[] = {SGI150804, SGI150805, SGI150806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150807, SGI0, SGI0, SGI0, SGI150808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33128[] = {SGI150809, SGI0, SGI150810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150811, SGI150812, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33129[] = {SGI150813, SGI150814, SGI150815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150816, SGI0, SGI0, SGI0, SGI150817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33130[] = {SGI150818, SGI150819, SGI150820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150823, SGI0, SGI0, SGI0, SGI150824, SGI150824, SGI0, SGI150825, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33131[] = {SGI150821, SGI0, SGI150822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150826, SGI150827, SGI150827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33132[] = {SGI150828, SGI150829, SGI150831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150833, SGI0, SGI0, SGI0, SGI150835, SGI150835, SGI0, SGI150837, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33133[] = {SGI150830, SGI0, SGI150832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150834, SGI150836, SGI150836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33134[] = {SGI150838, SGI0, SGI150840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150843, SGI150845, SGI150845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33135[] = {SGI150839, SGI150841, SGI150842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150844, SGI0, SGI0, SGI0, SGI150846, SGI150846, SGI0, SGI150847, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33136[] = {SGI150848, SGI0, SGI150849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150850, SGI150851, SGI150851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33137[] = {SGI150852, SGI0, SGI150853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150854, SGI150855, SGI150855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33138[] = {SGI150856, SGI150857, SGI150858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150859, SGI0, SGI0, SGI0, SGI150860, SGI150860, SGI0, SGI150861, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33139[] = {SGI150862, SGI0, SGI150863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150864, SGI150865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33140[] = {SGI150866, SGI0, SGI150867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150868, SGI150869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33141[] = {SGI0, SGI150870, SGI150871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150872, SGI150872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33142[] = {SGI0, SGI150873, SGI150874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150875, SGI150875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33143[] = {SGI150876, SGI150877, SGI150878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150879, SGI150879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33144[] = {SGI150880, SGI150881, SGI150882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150883, SGI0, SGI0, SGI0, SGI150884, SGI150884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33145[] = {SGI0, SGI150885, SGI150886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150887, SGI150887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33146[] = {SGI0, SGI150888, SGI150889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150890, SGI150890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33147[] = {SGI150891, SGI150892, SGI150893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150894, SGI150895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33148[] = {SGI0, SGI150896, SGI150897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150898, SGI150898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33149[] = {SGI0, SGI150899, SGI150900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150901, SGI150901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33150[] = {SGI150902, SGI0, SGI150903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33151[] = {SGI0, SGI150904, SGI150905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33152[] = {SGI0, SGI150906, SGI150907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150908, SGI150908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33153[] = {SGI150909, SGI150910, SGI150911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150913, SGI0, SGI0, SGI0, SGI150916, SGI150916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33154[] = {SGI150912, SGI150914, SGI150915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150917, SGI0, SGI0, SGI0, SGI150918, SGI150918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33155[] = {SGI0, SGI150919, SGI150920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150924, SGI150924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33156[] = {SGI150921, SGI150922, SGI150923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150925, SGI0, SGI0, SGI0, SGI150926, SGI150926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33157[] = {SGI0, SGI150927, SGI150928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150935, SGI150935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33158[] = {SGI150929, SGI150931, SGI150933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150936, SGI0, SGI0, SGI0, SGI150937, SGI150937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33159[] = {SGI150930, SGI150932, SGI150934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150938, SGI150938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33160[] = {SGI150939, SGI0, SGI150941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33161[] = {SGI150940, SGI150942, SGI150943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150944, SGI0, SGI0, SGI0, SGI150945, SGI150945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33162[] = {SGI150946, SGI150947, SGI150948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150952, SGI0, SGI0, SGI0, SGI150953, SGI150953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33163[] = {SGI150949, SGI150950, SGI150951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150954, SGI0, SGI0, SGI0, SGI150955, SGI150955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33164[] = {SGI150956, SGI150957, SGI150958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150959, SGI0, SGI0, SGI0, SGI150960, SGI150960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33165[] = {SGI150961, SGI0, SGI150962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33166[] = {SGI150966, SGI0, SGI150967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33167[] = {SGI150963, SGI150964, SGI150965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150968, SGI0, SGI0, SGI0, SGI150969, SGI150969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33168[] = {SGI150970, SGI150971, SGI150973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150976, SGI0, SGI0, SGI0, SGI150978, SGI150978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33169[] = {SGI150972, SGI150974, SGI150975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150977, SGI0, SGI0, SGI0, SGI150979, SGI150979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33170[] = {SGI150981, SGI0, SGI150983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33171[] = {SGI150980, SGI150982, SGI150984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150985, SGI0, SGI0, SGI0, SGI150986, SGI150986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33172[] = {SGI150987, SGI150988, SGI150989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150992, SGI0, SGI0, SGI0, SGI150994, SGI150994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33173[] = {SGI150990, SGI150991, SGI150993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI150995, SGI0, SGI0, SGI0, SGI150996, SGI150996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33174[] = {SGI150997, SGI0, SGI150998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33175[] = {SGI150999, SGI151000, SGI151001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151002, SGI0, SGI0, SGI0, SGI151003, SGI151003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33176[] = {SGI151004, SGI151005, SGI151006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151007, SGI0, SGI0, SGI0, SGI151008, SGI151008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33177[] = {SGI151009, SGI0, SGI151010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33178[] = {SGI151011, SGI151012, SGI151013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151014, SGI0, SGI0, SGI0, SGI151015, SGI151015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33179[] = {SGI151016, SGI151017, SGI151018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151019, SGI0, SGI0, SGI0, SGI151020, SGI151020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33180[] = {SGI151021, SGI0, SGI151022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33181[] = {SGI151023, SGI151024, SGI151025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151026, SGI0, SGI0, SGI0, SGI151027, SGI151027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33182[] = {SGI151028, SGI151029, SGI151030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151031, SGI0, SGI0, SGI0, SGI151032, SGI151032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33183[] = {SGI151033, SGI0, SGI151034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33184[] = {SGI151035, SGI0, SGI151036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33185[] = {SGI151037, SGI0, SGI151038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151039, SGI151040, SGI151040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33186[] = {SGI151041, SGI0, SGI151042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151043, SGI151044, SGI151044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33187[] = {SGI151045, SGI0, SGI151046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151047, SGI151048, SGI151048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33188[] = {SGI151049, SGI151050, SGI151051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151054, SGI0, SGI0, SGI0, SGI151055, SGI151055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33189[] = {SGI151052, SGI0, SGI151053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151056, SGI151057, SGI151057, SGI0, SGI0, NULL};
const Segment * const SG33190[] = {SGI151058, SGI151059, SGI151060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151061, SGI0, SGI0, SGI0, SGI151062, SGI151062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33191[] = {SGI151063, SGI0, SGI151064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33192[] = {SGI151065, SGI0, SGI151066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151067, SGI151068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33193[] = {SGI151069, SGI0, SGI151070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151071, SGI151072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33194[] = {SGI151073, SGI0, SGI151075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151077, SGI151078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33195[] = {SGI151074, SGI0, SGI151076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151079, SGI151080, SGI151080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33196[] = {SGI151081, SGI0, SGI151082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151083, SGI151084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33197[] = {SGI151085, SGI0, SGI151086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151087, SGI151088, SGI151088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33198[] = {SGI151089, SGI0, SGI151090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151091, SGI151092, SGI151092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33199[] = {SGI151093, SGI0, SGI151094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151095, SGI151096, SGI151096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33200[] = {SGI151097, SGI0, SGI151098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151099, SGI151100, SGI151100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33201[] = {SGI151101, SGI0, SGI151102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151103, SGI151104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33202[] = {SGI151105, SGI0, SGI151106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151107, SGI151108, SGI151108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33203[] = {SGI151109, SGI0, SGI151110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151111, SGI151112, SGI151112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33204[] = {SGI151113, SGI0, SGI151114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151115, SGI151116, SGI151116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33205[] = {SGI151117, SGI151118, SGI151119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151122, SGI0, SGI0, SGI0, SGI151123, SGI151123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33206[] = {SGI151120, SGI0, SGI151121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151124, SGI151125, SGI151125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33207[] = {SGI151126, SGI151127, SGI151128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151130, SGI0, SGI0, SGI0, SGI151132, SGI151132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33208[] = {SGI151129, SGI0, SGI151131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151133, SGI151134, SGI151134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33209[] = {SGI151135, SGI0, SGI151136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33210[] = {SGI151137, SGI0, SGI151138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151139, SGI151140, SGI151140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33211[] = {SGI151141, SGI151142, SGI151143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151144, SGI0, SGI0, SGI0, SGI151145, SGI151145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33212[] = {SGI151146, SGI0, SGI151147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151149, SGI151151, SGI151151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33213[] = {SGI151148, SGI151150, SGI151152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151153, SGI0, SGI0, SGI0, SGI151154, SGI151154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33214[] = {SGI151155, SGI0, SGI151156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151157, SGI151159, SGI151159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33215[] = {SGI151158, SGI0, SGI151160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33216[] = {SGI151161, SGI0, SGI151162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151163, SGI0, SGI0, SGI0, SGI151164, SGI151164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33217[] = {SGI151165, SGI151166, SGI151167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151168, SGI0, SGI0, SGI0, SGI151169, SGI151169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33218[] = {SGI151170, SGI0, SGI151171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33219[] = {SGI151172, SGI0, SGI151173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151174, SGI151175, SGI151175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33220[] = {SGI151176, SGI0, SGI151177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151178, SGI151179, SGI151179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33221[] = {SGI151180, SGI0, SGI151181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151182, SGI151183, SGI151183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33222[] = {SGI151184, SGI0, SGI151185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151186, SGI151187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33223[] = {SGI151188, SGI0, SGI151189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151190, SGI151191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33224[] = {SGI151192, SGI0, SGI151193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151194, SGI151195, SGI151195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33225[] = {SGI151196, SGI0, SGI151197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151198, SGI151199, SGI151199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33226[] = {SGI151200, SGI0, SGI151201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151202, SGI151203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33227[] = {SGI151204, SGI0, SGI151205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151206, SGI151207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33228[] = {SGI151208, SGI0, SGI151209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151210, SGI151211, SGI151211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33229[] = {SGI151212, SGI0, SGI151213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151214, SGI151215, SGI151215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33230[] = {SGI151216, SGI0, SGI151217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151218, SGI151219, SGI151219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33231[] = {SGI151220, SGI0, SGI151221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151222, SGI151223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33232[] = {SGI151224, SGI151225, SGI151226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151227, SGI0, SGI0, SGI0, SGI151228, SGI151228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33233[] = {SGI151229, SGI0, SGI151230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33234[] = {SGI151231, SGI0, SGI151232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151233, SGI0, SGI0, SGI0, SGI151234, SGI151234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33235[] = {SGI151235, SGI151236, SGI151237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151238, SGI0, SGI0, SGI0, SGI151239, SGI151239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33236[] = {SGI151240, SGI151241, SGI151242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151243, SGI0, SGI0, SGI0, SGI151244, SGI151244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33237[] = {SGI151245, SGI0, SGI151246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33238[] = {SGI151247, SGI151248, SGI151249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151250, SGI0, SGI0, SGI0, SGI151251, SGI151251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33239[] = {SGI151252, SGI0, SGI151253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151254, SGI151255, SGI151255, SGI0, SGI0, NULL};
const Segment * const SG33240[] = {SGI151256, SGI151257, SGI151258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151259, SGI0, SGI0, SGI0, SGI151260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33241[] = {SGI151261, SGI0, SGI151262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33242[] = {SGI151263, SGI151264, SGI151265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33243[] = {SGI151266, SGI151267, SGI151268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33244[] = {SGI151269, SGI151270, SGI151271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33245[] = {SGI151272, SGI151273, SGI151274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33246[] = {SGI151275, SGI151276, SGI151277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151278, SGI0, SGI0, SGI0, SGI151279, SGI151279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33247[] = {SGI151280, SGI151281, SGI151282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151283, SGI0, SGI0, SGI0, SGI151284, SGI151284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33248[] = {SGI0, SGI0, SGI151285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33249[] = {SGI151286, SGI151287, SGI151288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151289, SGI0, SGI0, SGI0, SGI151290, SGI151290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33250[] = {SGI151291, SGI151292, SGI151293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151294, SGI0, SGI0, SGI0, SGI151295, SGI151295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33251[] = {SGI151296, SGI151297, SGI151298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151299, SGI0, SGI0, SGI0, SGI151300, SGI151300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33252[] = {SGI151301, SGI151302, SGI151303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151304, SGI0, SGI0, SGI0, SGI151307, SGI151307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33253[] = {SGI151305, SGI151306, SGI151308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151309, SGI0, SGI0, SGI0, SGI151310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33254[] = {SGI151311, SGI151312, SGI151313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151314, SGI0, SGI0, SGI0, SGI151315, SGI151315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33255[] = {SGI151316, SGI151317, SGI151318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151319, SGI0, SGI0, SGI0, SGI151320, SGI151320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33256[] = {SGI151321, SGI151322, SGI151323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151324, SGI0, SGI0, SGI0, SGI151325, SGI151325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33257[] = {SGI151326, SGI151327, SGI151328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151329, SGI0, SGI0, SGI0, SGI151330, SGI151330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33258[] = {SGI151331, SGI151332, SGI151333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151334, SGI0, SGI0, SGI0, SGI151335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33259[] = {SGI151336, SGI151337, SGI151338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151339, SGI0, SGI0, SGI0, SGI151340, SGI151340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33260[] = {SGI151341, SGI151342, SGI151343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151344, SGI0, SGI0, SGI0, SGI151345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33261[] = {SGI151346, SGI151347, SGI151348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151349, SGI0, SGI0, SGI0, SGI151350, SGI151350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33262[] = {SGI151351, SGI151352, SGI151353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151354, SGI151354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33263[] = {SGI151355, SGI151356, SGI151357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151358, SGI0, SGI0, SGI0, SGI151360, SGI151360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33264[] = {SGI151359, SGI151361, SGI151362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151363, SGI0, SGI0, SGI0, SGI151364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33265[] = {SGI151365, SGI151366, SGI151367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151368, SGI0, SGI0, SGI0, SGI151371, SGI151371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33266[] = {SGI151369, SGI0, SGI151370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151372, SGI0, SGI0, SGI0, SGI151373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33267[] = {SGI151374, SGI151375, SGI151376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151377, SGI0, SGI0, SGI0, SGI151378, SGI151378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33268[] = {SGI151379, SGI151380, SGI151381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151382, SGI0, SGI0, SGI0, SGI151383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33269[] = {SGI151384, SGI151385, SGI151386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151387, SGI0, SGI0, SGI0, SGI151388, SGI151388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33270[] = {SGI151389, SGI151390, SGI151391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151392, SGI151392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33271[] = {SGI151393, SGI151394, SGI151395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151396, SGI0, SGI0, SGI0, SGI151397, SGI151397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33272[] = {SGI151398, SGI151399, SGI151400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151404, SGI0, SGI0, SGI0, SGI151405, SGI151405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33273[] = {SGI151401, SGI151402, SGI151403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151406, SGI0, SGI0, SGI0, SGI151407, SGI151407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33274[] = {SGI151408, SGI0, SGI151409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33275[] = {SGI151410, SGI151411, SGI151412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151415, SGI0, SGI0, SGI0, SGI151417, SGI151417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33276[] = {SGI151413, SGI151414, SGI151416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151418, SGI0, SGI0, SGI0, SGI151419, SGI151420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33277[] = {SGI151421, SGI151422, SGI151423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151424, SGI0, SGI0, SGI0, SGI151425, SGI151425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33278[] = {SGI151426, SGI0, SGI151427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33279[] = {SGI151428, SGI151429, SGI151430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151431, SGI0, SGI0, SGI0, SGI151432, SGI151432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33280[] = {SGI151433, SGI151434, SGI151435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151436, SGI0, SGI0, SGI0, SGI151437, SGI151437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33281[] = {SGI151438, SGI151439, SGI151440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151441, SGI0, SGI0, SGI0, SGI151442, SGI151442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33282[] = {SGI151443, SGI0, SGI151444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33283[] = {SGI151445, SGI151446, SGI151447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151448, SGI0, SGI0, SGI0, SGI151449, SGI151449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33284[] = {SGI151450, SGI151451, SGI151452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151453, SGI151453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33285[] = {SGI151454, SGI151455, SGI151456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151458, SGI0, SGI0, SGI0, SGI151461, SGI151461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33286[] = {SGI151457, SGI151459, SGI151460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151462, SGI0, SGI0, SGI0, SGI151463, SGI151463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33287[] = {SGI151464, SGI151465, SGI151466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151467, SGI0, SGI0, SGI0, SGI151468, SGI151468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33288[] = {SGI151469, SGI151470, SGI151471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151472, SGI0, SGI0, SGI0, SGI151473, SGI151473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33289[] = {SGI151474, SGI151475, SGI151476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151477, SGI0, SGI0, SGI0, SGI151478, SGI151478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33290[] = {SGI151479, SGI151480, SGI151481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151482, SGI0, SGI0, SGI0, SGI151483, SGI151483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33291[] = {SGI151484, SGI0, SGI151485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151486, SGI151487, SGI151487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33292[] = {SGI151488, SGI0, SGI151489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151490, SGI151491, SGI151491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33293[] = {SGI151492, SGI0, SGI151493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151494, SGI151495, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33294[] = {SGI151496, SGI0, SGI151497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151498, SGI151499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33295[] = {SGI151500, SGI0, SGI151501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151502, SGI151503, SGI151503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33296[] = {SGI151504, SGI0, SGI151505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151506, SGI151507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33297[] = {SGI151508, SGI151509, SGI151510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151511, SGI0, SGI0, SGI0, SGI151512, SGI151512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33298[] = {SGI151513, SGI151514, SGI151515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151516, SGI0, SGI0, SGI0, SGI151517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33299[] = {SGI151518, SGI0, SGI151519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33300[] = {SGI151520, SGI151521, SGI151522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151523, SGI0, SGI0, SGI0, SGI151524, SGI151524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33301[] = {SGI151525, SGI151526, SGI151527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151528, SGI0, SGI0, SGI0, SGI151529, SGI151529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33302[] = {SGI151530, SGI151531, SGI151532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151533, SGI0, SGI0, SGI0, SGI151534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33303[] = {SGI151535, SGI0, SGI151536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33304[] = {SGI151537, SGI0, SGI151538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33305[] = {SGI151539, SGI151540, SGI151541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151542, SGI0, SGI0, SGI0, SGI151543, SGI151543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33306[] = {SGI151544, SGI151545, SGI151546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151547, SGI0, SGI0, SGI0, SGI151548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33307[] = {SGI151549, SGI151550, SGI151551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151552, SGI0, SGI0, SGI0, SGI151553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33308[] = {SGI151554, SGI0, SGI151555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151556, SGI151557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33309[] = {SGI151558, SGI0, SGI151559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151560, SGI151561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33310[] = {SGI151562, SGI0, SGI151563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151564, SGI151565, SGI151565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33311[] = {SGI151566, SGI0, SGI151567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151568, SGI151569, SGI151569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33312[] = {SGI151570, SGI0, SGI151571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151573, SGI151575, SGI151575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33313[] = {SGI151572, SGI0, SGI151574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151576, SGI151577, SGI151577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33314[] = {SGI151578, SGI0, SGI151579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151580, SGI151581, SGI151581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33315[] = {SGI151582, SGI0, SGI151583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151584, SGI151585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33316[] = {SGI151586, SGI0, SGI151587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151588, SGI151589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33317[] = {SGI151590, SGI0, SGI151591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151592, SGI151593, SGI151593, SGI0, SGI0, NULL};
const Segment * const SG33318[] = {SGI151594, SGI151595, SGI151596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151597, SGI0, SGI0, SGI0, SGI151598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33319[] = {SGI151599, SGI151600, SGI151601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151602, SGI0, SGI0, SGI0, SGI151603, SGI151603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33320[] = {SGI151604, SGI0, SGI151605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33321[] = {SGI151606, SGI151607, SGI151608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151609, SGI0, SGI0, SGI0, SGI151610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33322[] = {SGI151611, SGI151612, SGI151613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151614, SGI0, SGI0, SGI0, SGI151615, SGI151615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33323[] = {SGI151616, SGI151617, SGI151618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151619, SGI0, SGI0, SGI0, SGI151620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33324[] = {SGI151621, SGI151622, SGI151623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151624, SGI0, SGI0, SGI0, SGI151625, SGI151625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33325[] = {SGI151626, SGI0, SGI151627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33326[] = {SGI151628, SGI151629, SGI151630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151631, SGI0, SGI0, SGI0, SGI151632, SGI151632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33327[] = {SGI151633, SGI151634, SGI151635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151636, SGI0, SGI0, SGI0, SGI151637, SGI151637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33328[] = {SGI151638, SGI0, SGI151639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33329[] = {SGI151640, SGI151641, SGI151642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151643, SGI0, SGI0, SGI0, SGI151644, SGI151644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33330[] = {SGI151645, SGI151646, SGI151647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151648, SGI0, SGI0, SGI0, SGI151649, SGI151649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33331[] = {SGI151650, SGI0, SGI151651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33332[] = {SGI151652, SGI151653, SGI151654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151655, SGI0, SGI0, SGI0, SGI151656, SGI151656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33333[] = {SGI151657, SGI151658, SGI151659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151660, SGI0, SGI0, SGI0, SGI151661, SGI151661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33334[] = {SGI151662, SGI151663, SGI151664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151665, SGI0, SGI0, SGI0, SGI151666, SGI151666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33335[] = {SGI151667, SGI0, SGI151668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151669, SGI151670, SGI151670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33336[] = {SGI151671, SGI151672, SGI151673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151674, SGI0, SGI0, SGI0, SGI151675, SGI151675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33337[] = {SGI151676, SGI151677, SGI151678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151679, SGI0, SGI0, SGI0, SGI151680, SGI151680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33338[] = {SGI151681, SGI151682, SGI151683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151684, SGI0, SGI0, SGI0, SGI151685, SGI151685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33339[] = {SGI151686, SGI0, SGI151687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151688, SGI151689, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33340[] = {SGI151690, SGI151691, SGI151692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151693, SGI0, SGI0, SGI0, SGI151694, SGI151694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33341[] = {SGI151695, SGI151696, SGI151697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151698, SGI0, SGI0, SGI0, SGI151699, SGI151699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33342[] = {SGI151700, SGI151701, SGI151702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151703, SGI0, SGI0, SGI0, SGI151704, SGI151704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33343[] = {SGI151705, SGI0, SGI151706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33344[] = {SGI151707, SGI0, SGI151708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33345[] = {SGI151709, SGI0, SGI151710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33346[] = {SGI151711, SGI0, SGI151712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151713, SGI0, SGI0, SGI0, SGI151714, SGI151714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33347[] = {SGI151715, SGI151716, SGI151717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151718, SGI0, SGI0, SGI0, SGI151719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33348[] = {SGI151720, SGI0, SGI151721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151722, SGI0, SGI0, SGI0, SGI151723, SGI151723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33349[] = {SGI151724, SGI0, SGI151725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151726, SGI0, SGI0, SGI0, SGI151727, SGI151727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33350[] = {SGI151728, SGI151729, SGI151730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151731, SGI0, SGI0, SGI0, SGI151732, SGI151732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33351[] = {SGI151733, SGI0, SGI151734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33352[] = {SGI151735, SGI151736, SGI151737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151738, SGI0, SGI0, SGI0, SGI151739, SGI151739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33353[] = {SGI151743, SGI0, SGI151744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33354[] = {SGI151740, SGI151741, SGI151742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151745, SGI0, SGI0, SGI0, SGI151746, SGI151746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33355[] = {SGI151747, SGI0, SGI151748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33356[] = {SGI151749, SGI151750, SGI151751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151752, SGI0, SGI0, SGI0, SGI151753, SGI151753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33357[] = {SGI151754, SGI151755, SGI151756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151757, SGI0, SGI0, SGI0, SGI151758, SGI151758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33358[] = {SGI151759, SGI151760, SGI151761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151762, SGI0, SGI0, SGI0, SGI151763, SGI151763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33359[] = {SGI151764, SGI151765, SGI151766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151767, SGI0, SGI0, SGI0, SGI151768, SGI151768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33360[] = {SGI151769, SGI151770, SGI151771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151772, SGI0, SGI0, SGI0, SGI151773, SGI151773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33361[] = {SGI151774, SGI151775, SGI151776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151777, SGI0, SGI0, SGI0, SGI151778, SGI151778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33362[] = {SGI151779, SGI151780, SGI151781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151782, SGI0, SGI0, SGI0, SGI151783, SGI151783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33363[] = {SGI151784, SGI151785, SGI151786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151787, SGI0, SGI0, SGI0, SGI151788, SGI151788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33364[] = {SGI151789, SGI151790, SGI151791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151792, SGI0, SGI0, SGI0, SGI151793, SGI151793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33365[] = {SGI151794, SGI0, SGI151795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151796, SGI151797, SGI151797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33366[] = {SGI151798, SGI0, SGI151799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151800, SGI151801, SGI151801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33367[] = {SGI151803, SGI0, SGI151805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151808, SGI151809, SGI151809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33368[] = {SGI151802, SGI151804, SGI151806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151807, SGI0, SGI0, SGI0, SGI151810, SGI151810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33369[] = {SGI151811, SGI0, SGI151813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33370[] = {SGI151812, SGI0, SGI151814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151815, SGI151816, SGI151816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33371[] = {SGI151817, SGI0, SGI151818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151819, SGI151821, SGI151821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33372[] = {SGI151820, SGI151822, SGI151823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151824, SGI0, SGI0, SGI0, SGI151825, SGI151825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33373[] = {SGI151826, SGI0, SGI151827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151828, SGI151829, SGI151829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33374[] = {SGI151830, SGI0, SGI151831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33375[] = {SGI151832, SGI0, SGI151833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151837, SGI151838, SGI151838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33376[] = {SGI151834, SGI151835, SGI151836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151839, SGI0, SGI0, SGI0, SGI151840, SGI151840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33377[] = {SGI151841, SGI0, SGI151842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151844, SGI151846, SGI151846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33378[] = {SGI151843, SGI0, SGI151845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151847, SGI151848, SGI151848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33379[] = {SGI151849, SGI0, SGI151850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151853, SGI151854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33380[] = {SGI151851, SGI0, SGI151852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151855, SGI151856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33381[] = {SGI151857, SGI0, SGI151858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151859, SGI151860, SGI151860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33382[] = {SGI151861, SGI0, SGI151862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151863, SGI151864, SGI151864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33383[] = {SGI151865, SGI0, SGI151866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151867, SGI151868, SGI151868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33384[] = {SGI151869, SGI0, SGI151870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151871, SGI151872, SGI151872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33385[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151873, SGI0, SGI0, SGI151874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33386[] = {SGI151875, SGI151876, SGI151877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151878, SGI0, SGI0, SGI0, SGI151879, SGI151879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33387[] = {SGI151880, SGI151881, SGI151882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151883, SGI0, SGI0, SGI0, SGI151884, SGI151884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33388[] = {SGI151885, SGI151886, SGI151887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151888, SGI0, SGI0, SGI0, SGI151889, SGI151889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33389[] = {SGI151890, SGI151891, SGI151892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151893, SGI0, SGI0, SGI0, SGI151894, SGI151894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33390[] = {SGI151895, SGI151896, SGI151897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151898, SGI0, SGI0, SGI0, SGI151899, SGI151899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33391[] = {SGI151900, SGI0, SGI151901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33392[] = {SGI151902, SGI151903, SGI151904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151905, SGI0, SGI0, SGI0, SGI151906, SGI151906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33393[] = {SGI151907, SGI151908, SGI151909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151910, SGI0, SGI0, SGI0, SGI151911, SGI151911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33394[] = {SGI151912, SGI151913, SGI151914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151915, SGI0, SGI0, SGI0, SGI151916, SGI151916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33395[] = {SGI151917, SGI151918, SGI151919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151920, SGI0, SGI0, SGI0, SGI151921, SGI151921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33396[] = {SGI151922, SGI151923, SGI151924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151925, SGI0, SGI0, SGI0, SGI151926, SGI151926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33397[] = {SGI151927, SGI151928, SGI151929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151930, SGI0, SGI0, SGI0, SGI151931, SGI151931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33398[] = {SGI151932, SGI151933, SGI151934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151935, SGI0, SGI0, SGI0, SGI151936, SGI151936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33399[] = {SGI151937, SGI151938, SGI151939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151940, SGI0, SGI0, SGI0, SGI151941, SGI151941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33400[] = {SGI151942, SGI151943, SGI151944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151945, SGI0, SGI0, SGI0, SGI151946, SGI151946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33401[] = {SGI151947, SGI151948, SGI151949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151950, SGI0, SGI0, SGI0, SGI151951, SGI151951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33402[] = {SGI151952, SGI151953, SGI151954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151955, SGI0, SGI0, SGI0, SGI151956, SGI151956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33403[] = {SGI151957, SGI151958, SGI151959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151960, SGI0, SGI0, SGI0, SGI151961, SGI151961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33404[] = {SGI151962, SGI151963, SGI151964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151965, SGI0, SGI0, SGI0, SGI151966, SGI151966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33405[] = {SGI151967, SGI0, SGI151968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33406[] = {SGI151969, SGI151970, SGI151971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151972, SGI0, SGI0, SGI0, SGI151975, SGI151975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33407[] = {SGI151973, SGI151974, SGI151976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151977, SGI0, SGI0, SGI0, SGI151978, SGI151978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33408[] = {SGI151979, SGI151980, SGI151981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151984, SGI0, SGI0, SGI0, SGI151986, SGI151986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33409[] = {SGI151982, SGI151983, SGI151985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151987, SGI0, SGI0, SGI0, SGI151988, SGI151988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33410[] = {SGI151989, SGI0, SGI151990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33411[] = {SGI151991, SGI151992, SGI151993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151994, SGI0, SGI0, SGI0, SGI151995, SGI151995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33412[] = {SGI151996, SGI151997, SGI151998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI151999, SGI0, SGI0, SGI0, SGI152000, SGI152000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33413[] = {SGI152001, SGI0, SGI152002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152003, SGI152004, SGI152004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33414[] = {SGI152005, SGI0, SGI152006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152007, SGI152008, SGI152008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33415[] = {SGI152009, SGI0, SGI152010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152011, SGI152012, SGI152012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33416[] = {SGI152013, SGI0, SGI152014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152015, SGI152016, SGI152016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33417[] = {SGI152017, SGI0, SGI152018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152019, SGI152020, SGI152020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33418[] = {SGI152021, SGI0, SGI152022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152023, SGI152024, SGI152024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33419[] = {SGI152025, SGI0, SGI152026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152027, SGI152028, SGI152028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33420[] = {SGI152029, SGI0, SGI152030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152031, SGI152032, SGI152032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33421[] = {SGI152033, SGI152034, SGI152035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152036, SGI0, SGI0, SGI0, SGI152037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33422[] = {SGI152038, SGI0, SGI152039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33423[] = {SGI152040, SGI0, SGI152041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152042, SGI152043, SGI152043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33424[] = {SGI152044, SGI0, SGI152045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152046, SGI152047, SGI152047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33425[] = {SGI152048, SGI0, SGI152049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152050, SGI152051, SGI152051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33426[] = {SGI152052, SGI0, SGI152053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152054, SGI152055, SGI152055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33427[] = {SGI152056, SGI0, SGI152057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152058, SGI152059, SGI152059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33428[] = {SGI152060, SGI0, SGI152061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152062, SGI152063, SGI152063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33429[] = {SGI152064, SGI0, SGI152065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152066, SGI152067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33430[] = {SGI152068, SGI0, SGI152069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152070, SGI152071, SGI152071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33431[] = {SGI152072, SGI0, SGI152073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152074, SGI152075, SGI152075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33432[] = {SGI152076, SGI0, SGI152077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152078, SGI152079, SGI152079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33433[] = {SGI152080, SGI152081, SGI152082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152083, SGI0, SGI0, SGI0, SGI152084, SGI152084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33434[] = {SGI152085, SGI152086, SGI152087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152088, SGI0, SGI0, SGI0, SGI152089, SGI152089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33435[] = {SGI152090, SGI152091, SGI152092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152096, SGI0, SGI0, SGI0, SGI152097, SGI152097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33436[] = {SGI152093, SGI152094, SGI152095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152098, SGI0, SGI0, SGI0, SGI152099, SGI152099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33437[] = {SGI152100, SGI0, SGI152101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33438[] = {SGI152102, SGI152103, SGI152104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152105, SGI0, SGI0, SGI0, SGI152106, SGI152106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33439[] = {SGI152107, SGI152108, SGI152109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152110, SGI0, SGI0, SGI0, SGI152111, SGI152111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33440[] = {SGI152112, SGI0, SGI152113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33441[] = {SGI152114, SGI152115, SGI152116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152117, SGI0, SGI0, SGI0, SGI152118, SGI152118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33442[] = {SGI152119, SGI152120, SGI152121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152122, SGI0, SGI0, SGI0, SGI152123, SGI152123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33443[] = {SGI152124, SGI152125, SGI152126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152127, SGI0, SGI0, SGI0, SGI152128, SGI152128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33444[] = {SGI152129, SGI0, SGI152130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152131, SGI152132, SGI152132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33445[] = {SGI152133, SGI0, SGI152134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152135, SGI152136, SGI152136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33446[] = {SGI152137, SGI0, SGI152138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152139, SGI152140, SGI152140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33447[] = {SGI152141, SGI0, SGI152142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152143, SGI152144, SGI152144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33448[] = {SGI152145, SGI0, SGI152146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152147, SGI152148, SGI152148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33449[] = {SGI152149, SGI0, SGI152150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152151, SGI152152, SGI152152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33450[] = {SGI152153, SGI0, SGI152154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152155, SGI152156, SGI152156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33451[] = {SGI152157, SGI0, SGI152158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152159, SGI152160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33452[] = {SGI152161, SGI0, SGI152162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152163, SGI152164, SGI152164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33453[] = {SGI152165, SGI0, SGI152166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152167, SGI152168, SGI152168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33454[] = {SGI152169, SGI0, SGI152170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152171, SGI152172, SGI152172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33455[] = {SGI152173, SGI0, SGI152174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152175, SGI152176, SGI152176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33456[] = {SGI152177, SGI0, SGI152178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152179, SGI152180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33457[] = {SGI152181, SGI152182, SGI152183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152184, SGI0, SGI0, SGI0, SGI152185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33458[] = {SGI152186, SGI0, SGI152187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33459[] = {SGI152188, SGI152189, SGI152190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152191, SGI0, SGI0, SGI0, SGI152192, SGI152192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33460[] = {SGI152193, SGI0, SGI152194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152195, SGI0, SGI0, SGI0, SGI152196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33461[] = {SGI152197, SGI152198, SGI152199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152200, SGI0, SGI0, SGI0, SGI152201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33462[] = {SGI152202, SGI152203, SGI152204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152205, SGI0, SGI0, SGI0, SGI152206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33463[] = {SGI152207, SGI152208, SGI152209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152210, SGI0, SGI0, SGI0, SGI152211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33464[] = {SGI152212, SGI0, SGI152213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152214, SGI152215, SGI152215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33465[] = {SGI152216, SGI0, SGI152217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152218, SGI152219, SGI152219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33466[] = {SGI152220, SGI0, SGI152221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152222, SGI152223, SGI152223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33467[] = {SGI152224, SGI0, SGI152225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152226, SGI152227, SGI152227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33468[] = {SGI152228, SGI0, SGI152229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152230, SGI152231, SGI152231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33469[] = {SGI152232, SGI0, SGI152233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152234, SGI152235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33470[] = {SGI152236, SGI0, SGI152237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152238, SGI152239, SGI152239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33471[] = {SGI152240, SGI0, SGI152241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152242, SGI152243, SGI152243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33472[] = {SGI152244, SGI0, SGI152245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152246, SGI152247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33473[] = {SGI152248, SGI152249, SGI152250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152251, SGI0, SGI0, SGI0, SGI152252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33474[] = {SGI152254, SGI152255, SGI152256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152257, SGI0, SGI0, SGI0, SGI152258, SGI152258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33475[] = {SGI152260, SGI152261, SGI152262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152263, SGI0, SGI0, SGI0, SGI152264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33476[] = {SGI152266, SGI152267, SGI152268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152269, SGI0, SGI0, SGI0, SGI152270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33477[] = {SGI152272, SGI152273, SGI152274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152275, SGI0, SGI0, SGI0, SGI152276, SGI152276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33478[] = {SGI152278, SGI152279, SGI152280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152281, SGI0, SGI0, SGI0, SGI152282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33479[] = {SGI152284, SGI152285, SGI152286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152287, SGI0, SGI0, SGI0, SGI152288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33480[] = {SGI152290, SGI152291, SGI152292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152293, SGI0, SGI0, SGI0, SGI152294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33481[] = {SGI152296, SGI152297, SGI152298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152299, SGI0, SGI0, SGI0, SGI152300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33482[] = {SGI152302, SGI152303, SGI152304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152305, SGI0, SGI0, SGI0, SGI152306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33483[] = {SGI152308, SGI152309, SGI152310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152311, SGI0, SGI0, SGI0, SGI152312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33484[] = {SGI152314, SGI152315, SGI152316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152317, SGI0, SGI0, SGI0, SGI152318, SGI152318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33485[] = {SGI152320, SGI152321, SGI152322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152323, SGI0, SGI0, SGI0, SGI152324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33486[] = {SGI152326, SGI0, SGI152327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152328, SGI152329, SGI152329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33487[] = {SGI152330, SGI0, SGI152331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152335, SGI152337, SGI152337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33488[] = {SGI152332, SGI0, SGI152334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152338, SGI152339, SGI152339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33489[] = {SGI152333, SGI0, SGI152336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152340, SGI152341, SGI152341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33490[] = {SGI152342, SGI0, SGI152343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152346, SGI152347, SGI152347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33491[] = {SGI152344, SGI0, SGI152345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152349, SGI152350, SGI152350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33492[] = {SGI152348, SGI0, SGI152351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152352, SGI152353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33493[] = {SGI152354, SGI0, SGI152355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152358, SGI152359, SGI152359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33494[] = {SGI152356, SGI0, SGI152357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152360, SGI152361, SGI152361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33495[] = {SGI152362, SGI0, SGI152363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152364, SGI152365, SGI152365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33496[] = {SGI152366, SGI0, SGI152367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152370, SGI152371, SGI152371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33497[] = {SGI152368, SGI0, SGI152369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152372, SGI152373, SGI152373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33498[] = {SGI152374, SGI0, SGI152375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152376, SGI152377, SGI152377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33499[] = {SGI152378, SGI0, SGI152379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152380, SGI152381, SGI152381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33500[] = {SGI152382, SGI0, SGI152383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152384, SGI152385, SGI152385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33501[] = {SGI152386, SGI0, SGI152387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152388, SGI152389, SGI152389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33502[] = {SGI152390, SGI0, SGI152391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152392, SGI152393, SGI152393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33503[] = {SGI152394, SGI0, SGI152395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152396, SGI152397, SGI152397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33504[] = {SGI152398, SGI152399, SGI152400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152401, SGI0, SGI0, SGI0, SGI152402, SGI152402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33505[] = {SGI152403, SGI152404, SGI152405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152406, SGI0, SGI0, SGI0, SGI152407, SGI152407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33506[] = {SGI152408, SGI152409, SGI152410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152411, SGI152411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33507[] = {SGI152412, SGI0, SGI152413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33508[] = {SGI152414, SGI152415, SGI152416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152417, SGI0, SGI0, SGI0, SGI152418, SGI152418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33509[] = {SGI152419, SGI152420, SGI152421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152422, SGI0, SGI0, SGI0, SGI152423, SGI152423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33510[] = {SGI152424, SGI152425, SGI152426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152427, SGI0, SGI0, SGI0, SGI152428, SGI152428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33511[] = {SGI152429, SGI152430, SGI152431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152432, SGI0, SGI0, SGI0, SGI152433, SGI152433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33512[] = {SGI152434, SGI152435, SGI152436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152437, SGI0, SGI0, SGI0, SGI152438, SGI152438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33513[] = {SGI152439, SGI152440, SGI152441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152442, SGI0, SGI0, SGI0, SGI152443, SGI152443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33514[] = {SGI152444, SGI152445, SGI152446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152447, SGI0, SGI0, SGI0, SGI152448, SGI152448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33515[] = {SGI152449, SGI152450, SGI152451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152452, SGI0, SGI0, SGI0, SGI152453, SGI152453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33516[] = {SGI152454, SGI0, SGI152455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152456, SGI152457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33517[] = {SGI152458, SGI0, SGI152459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152460, SGI152461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33518[] = {SGI152462, SGI0, SGI152463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152464, SGI152465, SGI152465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33519[] = {SGI152466, SGI0, SGI152467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152468, SGI152469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33520[] = {SGI152470, SGI0, SGI152471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152472, SGI152473, SGI152473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33521[] = {SGI152474, SGI0, SGI152475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152476, SGI152477, SGI152477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33522[] = {SGI152478, SGI0, SGI152479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152480, SGI152481, SGI152481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33523[] = {SGI152482, SGI0, SGI152483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152484, SGI152485, SGI152485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33524[] = {SGI152486, SGI152487, SGI152488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152489, SGI0, SGI0, SGI0, SGI152490, SGI152490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33525[] = {SGI152491, SGI0, SGI152492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152493, SGI152494, SGI152494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33526[] = {SGI152495, SGI152496, SGI152497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152498, SGI0, SGI0, SGI0, SGI152499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33527[] = {SGI152500, SGI152501, SGI152502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152503, SGI152503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33528[] = {SGI152504, SGI152505, SGI152506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152507, SGI0, SGI0, SGI0, SGI152508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33529[] = {SGI152509, SGI152510, SGI152511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152512, SGI0, SGI0, SGI0, SGI152513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33530[] = {SGI152514, SGI152515, SGI152516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152517, SGI0, SGI0, SGI0, SGI152518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33531[] = {SGI152519, SGI152520, SGI152521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152522, SGI0, SGI0, SGI0, SGI152523, SGI152523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33532[] = {SGI152524, SGI0, SGI152525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33533[] = {SGI152526, SGI0, SGI152527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152528, SGI152529, SGI152529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33534[] = {SGI152530, SGI0, SGI152531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152532, SGI152533, SGI152533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33535[] = {SGI152534, SGI0, SGI152535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152536, SGI152537, SGI152537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33536[] = {SGI152538, SGI0, SGI152539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152540, SGI152541, SGI152541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33537[] = {SGI152542, SGI0, SGI152543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152544, SGI152545, SGI152545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33538[] = {SGI152546, SGI0, SGI152547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152548, SGI152549, SGI152549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33539[] = {SGI152550, SGI0, SGI152551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152552, SGI152553, SGI152553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33540[] = {SGI152554, SGI0, SGI152555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152556, SGI152557, SGI152557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33541[] = {SGI152558, SGI152559, SGI152560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152561, SGI0, SGI0, SGI0, SGI152562, SGI152562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33542[] = {SGI152564, SGI152565, SGI152566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152567, SGI0, SGI0, SGI0, SGI152568, SGI152568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33543[] = {SGI152570, SGI152571, SGI152572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152573, SGI0, SGI0, SGI0, SGI152574, SGI152574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33544[] = {SGI152576, SGI152577, SGI152578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152579, SGI0, SGI0, SGI0, SGI152580, SGI152580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33545[] = {SGI152582, SGI152583, SGI152584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152585, SGI0, SGI0, SGI0, SGI152586, SGI152586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33546[] = {SGI152588, SGI152589, SGI152590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152591, SGI0, SGI0, SGI0, SGI152592, SGI152592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33547[] = {SGI152594, SGI152595, SGI152596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152597, SGI0, SGI0, SGI0, SGI152598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33548[] = {SGI152599, SGI0, SGI152600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33549[] = {SGI152601, SGI152602, SGI152603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152604, SGI0, SGI0, SGI0, SGI152605, SGI152605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33550[] = {SGI152606, SGI152607, SGI152608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152609, SGI152609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33551[] = {SGI152610, SGI152611, SGI152612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33552[] = {SGI152613, SGI152614, SGI152615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33553[] = {SGI152616, SGI152617, SGI152618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33554[] = {SGI152619, SGI0, SGI152620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152621, SGI0, SGI0, SGI0, SGI152622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33555[] = {SGI152623, SGI152624, SGI152625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152626, SGI152626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33556[] = {SGI152627, SGI152628, SGI152629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152630, SGI0, SGI0, SGI0, SGI152631, SGI152631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33557[] = {SGI152632, SGI152633, SGI152634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152635, SGI0, SGI0, SGI0, SGI152636, SGI152637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33558[] = {SGI152638, SGI0, SGI152639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152640, SGI152641, SGI152641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33559[] = {SGI152642, SGI0, SGI152643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152644, SGI152645, SGI152645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33560[] = {SGI152646, SGI0, SGI152647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152648, SGI152649, SGI152649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33561[] = {SGI152650, SGI0, SGI152651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152652, SGI152653, SGI152653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33562[] = {SGI152654, SGI0, SGI152655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152656, SGI152657, SGI152657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33563[] = {SGI152658, SGI0, SGI152659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152660, SGI152661, SGI152661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33564[] = {SGI152662, SGI0, SGI152663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152664, SGI152665, SGI152665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33565[] = {SGI152666, SGI0, SGI152667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152668, SGI152669, SGI152669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33566[] = {SGI152670, SGI0, SGI152671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152672, SGI152673, SGI152673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33567[] = {SGI152674, SGI0, SGI152675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152676, SGI152677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33568[] = {SGI152678, SGI0, SGI152679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152680, SGI152681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33569[] = {SGI152682, SGI152683, SGI152684, SGI152685, SGI152686, SGI152686, SGI0, NULL};
const Segment * const SG33570[] = {SGI152687, SGI152688, SGI152689, SGI152690, SGI152691, SGI152691, SGI0, NULL};
const Segment * const SG33571[] = {SGI152692, SGI152693, SGI152694, SGI152695, SGI152696, SGI152696, SGI0, NULL};
const Segment * const SG33572[] = {SGI152699, SGI152700, SGI152702, SGI152704, SGI152705, SGI152705, SGI0, NULL};
const Segment * const SG33573[] = {SGI152701, SGI0, SGI152703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33574[] = {SGI152697, SGI0, SGI152698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152706, SGI152707, SGI152707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33575[] = {SGI152708, SGI152709, SGI152710, SGI152711, SGI152712, SGI152712, SGI0, NULL};
const Segment * const SG33576[] = {SGI152713, SGI152714, SGI152715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152716, SGI0, SGI0, SGI0, SGI152717, SGI152717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33577[] = {SGI152720, SGI152721, SGI152723, SGI152725, SGI152726, SGI152726, SGI0, NULL};
const Segment * const SG33578[] = {SGI152718, SGI0, SGI152719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152722, SGI152724, SGI152724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33579[] = {SGI152727, SGI152728, SGI152729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152730, SGI0, SGI0, SGI0, SGI152731, SGI152731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33580[] = {SGI152736, SGI0, SGI152738, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33581[] = {SGI152732, SGI0, SGI152733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152734, SGI152735, SGI152735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33582[] = {SGI152737, SGI152739, SGI152740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152741, SGI0, SGI0, SGI0, SGI152742, SGI152742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33583[] = {SGI152743, SGI0, SGI152744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152745, SGI152746, SGI152746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33584[] = {SGI152747, SGI152748, SGI152749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152750, SGI0, SGI0, SGI0, SGI152751, SGI152751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33585[] = {SGI152752, SGI152753, SGI152754, SGI152755, SGI152756, SGI152756, SGI0, NULL};
const Segment * const SG33586[] = {SGI152757, SGI0, SGI152758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152759, SGI152760, SGI152760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33587[] = {SGI152761, SGI152762, SGI152763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152764, SGI0, SGI0, SGI0, SGI152765, SGI152765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33588[] = {SGI152766, SGI0, SGI152767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152768, SGI152770, SGI152770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33589[] = {SGI152769, SGI0, SGI152771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33590[] = {SGI152774, SGI152776, SGI152778, SGI152780, SGI152782, SGI152782, SGI0, NULL};
const Segment * const SG33591[] = {SGI152772, SGI0, SGI152773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152781, SGI152783, SGI152783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33592[] = {SGI152784, SGI152785, SGI152787, SGI152788, SGI152790, SGI152790, SGI0, NULL};
const Segment * const SG33593[] = {SGI152786, SGI0, SGI152789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152791, SGI152792, SGI152792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33594[] = {SGI152793, SGI152794, SGI152795, SGI152796, SGI152797, SGI152797, SGI0, NULL};
const Segment * const SG33595[] = {SGI152798, SGI0, SGI152799, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33596[] = {SGI152802, SGI152804, SGI152806, SGI152807, SGI152808, SGI152808, SGI0, NULL};
const Segment * const SG33597[] = {SGI152800, SGI0, SGI152801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152803, SGI152805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33598[] = {SGI152809, SGI152810, SGI152811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152812, SGI0, SGI0, SGI0, SGI152813, SGI152813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33599[] = {SGI152814, SGI0, SGI152815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33600[] = {SGI152816, SGI152817, SGI152818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152819, SGI0, SGI0, SGI0, SGI152820, SGI152820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33601[] = {SGI152821, SGI152822, SGI152823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152824, SGI0, SGI0, SGI0, SGI152825, SGI152825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33602[] = {SGI152826, SGI152827, SGI152828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152829, SGI0, SGI0, SGI0, SGI152830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33603[] = {SGI152831, SGI0, SGI152832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33604[] = {SGI152833, SGI152834, SGI152835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152836, SGI0, SGI0, SGI0, SGI152837, SGI152837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33605[] = {SGI152838, SGI152839, SGI152840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152841, SGI0, SGI0, SGI0, SGI152842, SGI152842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33606[] = {SGI152843, SGI0, SGI152844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152845, SGI152846, SGI152846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33607[] = {SGI152847, SGI152848, SGI152849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152850, SGI0, SGI0, SGI0, SGI152851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33608[] = {SGI152852, SGI0, SGI152853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152854, SGI152855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33609[] = {SGI152856, SGI152857, SGI152858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152859, SGI0, SGI0, SGI0, SGI152860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33610[] = {SGI152861, SGI152863, SGI152864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152866, SGI0, SGI0, SGI0, SGI152867, SGI152867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33611[] = {SGI152862, SGI0, SGI152865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152868, SGI152869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33612[] = {SGI152870, SGI152871, SGI152872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152873, SGI0, SGI0, SGI0, SGI152874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33613[] = {SGI152875, SGI0, SGI152876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33614[] = {SGI152877, SGI152878, SGI152879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152880, SGI0, SGI0, SGI0, SGI152881, SGI152881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33615[] = {SGI152882, SGI152884, SGI152885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152887, SGI0, SGI0, SGI0, SGI152888, SGI152888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33616[] = {SGI152883, SGI0, SGI152886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152889, SGI152890, SGI152890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33617[] = {SGI152891, SGI0, SGI152892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152893, SGI152894, SGI152894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33618[] = {SGI152895, SGI0, SGI152896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152897, SGI152898, SGI152898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33619[] = {SGI152899, SGI0, SGI152900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152901, SGI152902, SGI152902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33620[] = {SGI152903, SGI0, SGI152904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152905, SGI152906, SGI152906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33621[] = {SGI152907, SGI0, SGI152908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152909, SGI152910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33622[] = {SGI152911, SGI0, SGI152912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152913, SGI152914, SGI152914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33623[] = {SGI152915, SGI0, SGI152916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152917, SGI152918, SGI152918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33624[] = {SGI152919, SGI0, SGI152920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152921, SGI152922, SGI152922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33625[] = {SGI152923, SGI152924, SGI152925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152926, SGI0, SGI0, SGI0, SGI152927, SGI152927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33626[] = {SGI152928, SGI0, SGI152929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33627[] = {SGI152930, SGI0, SGI152931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152932, SGI0, SGI0, SGI0, SGI152933, SGI152933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33628[] = {SGI152775, SGI152777, SGI152779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152934, SGI0, SGI0, SGI0, SGI152935, SGI152935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33629[] = {SGI152936, SGI152937, SGI152938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152939, SGI0, SGI0, SGI0, SGI152940, SGI152940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33630[] = {SGI152941, SGI0, SGI152942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33631[] = {SGI152943, SGI152944, SGI152945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152946, SGI0, SGI0, SGI0, SGI152947, SGI152947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33632[] = {SGI152948, SGI152949, SGI152950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152951, SGI0, SGI0, SGI0, SGI152952, SGI152952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33633[] = {SGI152953, SGI0, SGI152954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33634[] = {SGI152955, SGI152956, SGI152957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152958, SGI0, SGI0, SGI0, SGI152959, SGI152959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33635[] = {SGI152960, SGI152961, SGI152962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152963, SGI0, SGI0, SGI0, SGI152964, SGI152964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33636[] = {SGI152965, SGI152966, SGI152967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152968, SGI0, SGI0, SGI0, SGI152969, SGI152969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33637[] = {SGI152970, SGI152971, SGI152972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152973, SGI0, SGI0, SGI0, SGI152974, SGI152974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33638[] = {SGI152975, SGI152976, SGI152977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152978, SGI0, SGI0, SGI0, SGI152979, SGI152979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33639[] = {SGI152980, SGI152981, SGI152982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152983, SGI0, SGI0, SGI0, SGI152984, SGI152984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33640[] = {SGI152985, SGI152986, SGI152987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152988, SGI0, SGI0, SGI0, SGI152989, SGI152989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33641[] = {SGI152990, SGI0, SGI152991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152992, SGI0, SGI0, SGI0, SGI152993, SGI152993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33642[] = {SGI152994, SGI152995, SGI152996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI152997, SGI0, SGI0, SGI0, SGI152998, SGI152998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33643[] = {SGI152999, SGI153000, SGI153001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153002, SGI0, SGI0, SGI0, SGI153003, SGI153003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33644[] = {SGI153004, SGI153005, SGI153006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153007, SGI0, SGI0, SGI0, SGI153009, SGI153009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33645[] = {SGI153008, SGI0, SGI153010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153011, SGI153012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33646[] = {SGI153013, SGI0, SGI153014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153015, SGI153016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33647[] = {SGI153017, SGI0, SGI153018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153019, SGI153020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33648[] = {SGI153021, SGI0, SGI153022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153023, SGI153024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33649[] = {SGI153025, SGI0, SGI153026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153027, SGI153028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33650[] = {SGI153029, SGI0, SGI153030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153031, SGI153032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33651[] = {SGI153033, SGI153034, SGI153035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153036, SGI0, SGI0, SGI0, SGI153037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33652[] = {SGI153038, SGI0, SGI153039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153040, SGI153041, SGI153041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33653[] = {SGI153042, SGI0, SGI153043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153044, SGI153045, SGI153045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33654[] = {SGI153046, SGI0, SGI153047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153048, SGI153049, SGI153049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33655[] = {SGI153050, SGI0, SGI153051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153052, SGI153053, SGI153053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33656[] = {SGI153054, SGI0, SGI153055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153056, SGI153057, SGI153057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33657[] = {SGI153058, SGI0, SGI153059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153060, SGI153061, SGI153061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33658[] = {SGI153062, SGI0, SGI153063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153064, SGI153065, SGI153065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33659[] = {SGI153066, SGI0, SGI153067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153068, SGI153069, SGI153069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33660[] = {SGI153070, SGI0, SGI153071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153072, SGI153073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33661[] = {SGI153074, SGI0, SGI153075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153076, SGI153077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33662[] = {SGI153078, SGI153079, SGI153080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153081, SGI0, SGI0, SGI0, SGI153082, SGI153082, SGI0, SGI153083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33663[] = {SGI153084, SGI153085, SGI153086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153087, SGI0, SGI0, SGI0, SGI153088, SGI153088, SGI0, SGI153089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33664[] = {SGI153090, SGI153091, SGI153092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153093, SGI0, SGI0, SGI0, SGI153094, SGI0, SGI0, SGI153095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33665[] = {SGI153096, SGI0, SGI153097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153098, SGI153099, SGI153099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33666[] = {SGI153100, SGI0, SGI153101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153102, SGI153103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33667[] = {SGI153104, SGI0, SGI153105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153106, SGI153107, SGI153107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33668[] = {SGI153108, SGI153109, SGI153110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153111, SGI0, SGI0, SGI0, SGI153112, SGI153112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33669[] = {SGI153114, SGI153115, SGI153116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153117, SGI0, SGI0, SGI0, SGI153118, SGI153118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33670[] = {SGI153120, SGI153121, SGI153122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153123, SGI0, SGI0, SGI0, SGI153124, SGI153124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33671[] = {SGI153126, SGI153127, SGI153128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153129, SGI0, SGI0, SGI0, SGI153130, SGI153130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33672[] = {SGI153131, SGI0, SGI153132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33673[] = {SGI153133, SGI0, SGI153134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153135, SGI0, SGI0, SGI0, SGI153136, SGI153136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33674[] = {SGI153137, SGI0, SGI153138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153139, SGI153140, SGI153140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33675[] = {SGI153141, SGI0, SGI153142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153143, SGI153144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33676[] = {SGI153145, SGI0, SGI153146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153147, SGI153148, SGI153148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33677[] = {SGI153149, SGI0, SGI153151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153154, SGI153156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33678[] = {SGI153150, SGI0, SGI153152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153155, SGI153157, SGI153157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33679[] = {SGI153153, SGI0, SGI153158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153161, SGI153162, SGI153162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33680[] = {SGI153159, SGI0, SGI153160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153163, SGI153164, SGI153164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33681[] = {SGI153165, SGI0, SGI153167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153169, SGI153171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33682[] = {SGI153166, SGI0, SGI153168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153170, SGI153172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33683[] = {SGI153173, SGI0, SGI153174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153175, SGI153176, SGI153176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33684[] = {SGI153177, SGI0, SGI153178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153179, SGI153180, SGI153180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33685[] = {SGI153181, SGI0, SGI153182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153183, SGI153184, SGI153184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33686[] = {SGI153185, SGI0, SGI153186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153187, SGI153188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33687[] = {SGI153189, SGI0, SGI153190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153191, SGI153192, SGI153192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33688[] = {SGI153193, SGI0, SGI153194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153195, SGI153196, SGI153196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33689[] = {SGI153197, SGI153198, SGI153199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153200, SGI0, SGI0, SGI0, SGI153201, SGI153201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33690[] = {SGI153202, SGI153203, SGI153204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153205, SGI0, SGI0, SGI0, SGI153206, SGI153206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33691[] = {SGI153207, SGI153208, SGI153209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153210, SGI0, SGI0, SGI0, SGI153211, SGI153211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33692[] = {SGI153212, SGI153213, SGI153214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153215, SGI0, SGI0, SGI0, SGI153216, SGI153216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33693[] = {SGI153217, SGI0, SGI153218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153219, SGI153220, SGI153220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33694[] = {SGI153221, SGI0, SGI153222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153223, SGI153224, SGI153224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33695[] = {SGI153225, SGI0, SGI153226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153227, SGI153228, SGI153228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33696[] = {SGI153229, SGI0, SGI153230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153231, SGI153232, SGI153232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33697[] = {SGI153233, SGI153234, SGI153235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153236, SGI0, SGI0, SGI0, SGI153237, SGI153237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33698[] = {SGI153238, SGI153239, SGI153240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153241, SGI0, SGI0, SGI0, SGI153242, SGI153242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33699[] = {SGI153243, SGI153244, SGI153245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153246, SGI0, SGI0, SGI0, SGI153247, SGI153247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33700[] = {SGI153248, SGI153249, SGI153250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153251, SGI0, SGI0, SGI0, SGI153252, SGI153252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33701[] = {SGI153253, SGI0, SGI153254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33702[] = {SGI153255, SGI153256, SGI153257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153258, SGI0, SGI0, SGI0, SGI153259, SGI153259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33703[] = {SGI153260, SGI0, SGI153261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33704[] = {SGI153262, SGI153263, SGI153264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153265, SGI0, SGI0, SGI0, SGI153266, SGI153266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33705[] = {SGI153267, SGI0, SGI153268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153269, SGI153270, SGI153270, SGI0, SGI0, NULL};
const Segment * const SG33706[] = {SGI153271, SGI153272, SGI153273, SGI153274, SGI153275, SGI153275, SGI0, NULL};
const Segment * const SG33707[] = {SGI153276, SGI0, SGI153277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153278, SGI153279, SGI153279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33708[] = {SGI153280, SGI153281, SGI153282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153283, SGI0, SGI0, SGI0, SGI153284, SGI153284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33709[] = {SGI153285, SGI153286, SGI153287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153288, SGI0, SGI0, SGI0, SGI153289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33710[] = {SGI153290, SGI153291, SGI153292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153293, SGI0, SGI0, SGI0, SGI153294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33711[] = {SGI153295, SGI153296, SGI153297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153298, SGI0, SGI0, SGI0, SGI153299, SGI153299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33712[] = {SGI153300, SGI0, SGI153301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33713[] = {SGI153302, SGI153303, SGI153304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153305, SGI0, SGI0, SGI0, SGI153306, SGI153306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33714[] = {SGI153307, SGI153308, SGI153309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153310, SGI0, SGI0, SGI0, SGI153311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33715[] = {SGI153312, SGI153313, SGI153314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153315, SGI0, SGI0, SGI0, SGI153316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33716[] = {SGI153317, SGI0, SGI153318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33717[] = {SGI153319, SGI0, SGI153320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33718[] = {SGI153321, SGI153322, SGI153323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153327, SGI0, SGI0, SGI0, SGI153328, SGI153328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33719[] = {SGI153324, SGI153325, SGI153326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153329, SGI0, SGI0, SGI0, SGI153330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33720[] = {SGI153331, SGI153332, SGI153333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153334, SGI0, SGI0, SGI0, SGI153335, SGI153335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33721[] = {SGI153336, SGI153337, SGI153338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153339, SGI0, SGI0, SGI0, SGI153340, SGI153340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33722[] = {SGI153341, SGI153342, SGI153343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153344, SGI0, SGI0, SGI0, SGI153345, SGI153345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33723[] = {SGI153346, SGI153347, SGI153348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153350, SGI0, SGI0, SGI0, SGI153352, SGI153352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33724[] = {SGI153349, SGI0, SGI153351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153353, SGI153354, SGI153354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33725[] = {SGI153355, SGI0, SGI153356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33726[] = {SGI153357, SGI0, SGI153358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153359, SGI153360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33727[] = {SGI153361, SGI0, SGI153362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33728[] = {SGI153363, SGI0, SGI153364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33729[] = {SGI153365, SGI0, SGI153366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33730[] = {SGI153367, SGI153368, SGI153369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153370, SGI0, SGI0, SGI0, SGI153371, SGI153371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33731[] = {SGI153372, SGI153373, SGI153374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153375, SGI0, SGI0, SGI0, SGI153376, SGI153376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33732[] = {SGI153377, SGI153378, SGI153379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153380, SGI0, SGI0, SGI0, SGI153381, SGI153381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33733[] = {SGI153382, SGI153383, SGI153384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153385, SGI0, SGI0, SGI0, SGI153386, SGI153386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33734[] = {SGI153387, SGI153388, SGI153389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153390, SGI0, SGI0, SGI0, SGI153391, SGI153391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33735[] = {SGI153392, SGI0, SGI153393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33736[] = {SGI153394, SGI0, SGI153395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33737[] = {SGI153396, SGI153397, SGI153398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153399, SGI0, SGI0, SGI0, SGI153400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33738[] = {SGI153401, SGI0, SGI153402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33739[] = {SGI153403, SGI153404, SGI153405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153406, SGI0, SGI0, SGI0, SGI153407, SGI153407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33740[] = {SGI153408, SGI153409, SGI153410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153411, SGI0, SGI0, SGI0, SGI153412, SGI153412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33741[] = {SGI153413, SGI0, SGI153414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33742[] = {SGI153415, SGI153416, SGI153417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153418, SGI0, SGI0, SGI0, SGI153419, SGI153419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33743[] = {SGI153420, SGI153421, SGI153422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153423, SGI0, SGI0, SGI0, SGI153424, SGI153424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33744[] = {SGI153425, SGI153426, SGI153427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153428, SGI0, SGI0, SGI0, SGI153429, SGI153429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33745[] = {SGI153430, SGI153431, SGI153432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153433, SGI0, SGI0, SGI0, SGI153436, SGI153436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33746[] = {SGI153434, SGI0, SGI153435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153437, SGI153438, SGI153438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33747[] = {SGI153439, SGI153440, SGI153441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153442, SGI0, SGI0, SGI0, SGI153445, SGI153445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33748[] = {SGI153443, SGI153444, SGI153446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153447, SGI0, SGI0, SGI0, SGI153448, SGI153448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33749[] = {SGI153449, SGI0, SGI153450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33750[] = {SGI153451, SGI0, SGI153452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153453, SGI0, SGI0, SGI0, SGI153454, SGI153454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33751[] = {SGI153455, SGI153456, SGI153457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153458, SGI0, SGI0, SGI0, SGI153459, SGI153459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33752[] = {SGI153460, SGI153461, SGI153462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153463, SGI0, SGI0, SGI0, SGI153464, SGI153464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33753[] = {SGI153465, SGI153466, SGI153467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153468, SGI0, SGI0, SGI0, SGI153469, SGI153469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33754[] = {SGI153470, SGI153471, SGI153472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153474, SGI0, SGI0, SGI0, SGI153477, SGI153477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33755[] = {SGI153473, SGI153475, SGI153476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153478, SGI0, SGI0, SGI0, SGI153479, SGI153479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33756[] = {SGI153483, SGI0, SGI153484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33757[] = {SGI153487, SGI153488, SGI153489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153490, SGI0, SGI0, SGI0, SGI153491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33758[] = {SGI153492, SGI153493, SGI153494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153495, SGI0, SGI0, SGI0, SGI153496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33759[] = {SGI153497, SGI153498, SGI153499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153500, SGI0, SGI0, SGI0, SGI153501, SGI153501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33760[] = {SGI153502, SGI153503, SGI153504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153505, SGI153505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33761[] = {SGI153506, SGI153507, SGI153508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153509, SGI0, SGI0, SGI0, SGI153510, SGI153510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33762[] = {SGI153511, SGI0, SGI153512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33763[] = {SGI0, SGI153513, SGI153514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153515, SGI0, SGI0, SGI0, SGI153516, SGI153516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33764[] = {SGI153517, SGI153518, SGI153519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153520, SGI153521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33765[] = {SGI153522, SGI153523, SGI153524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153525, SGI0, SGI0, SGI0, SGI153526, SGI153526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33766[] = {SGI153527, SGI153528, SGI153529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153530, SGI0, SGI0, SGI0, SGI153531, SGI153531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33767[] = {SGI153532, SGI0, SGI153533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153534, SGI153535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33768[] = {SGI153536, SGI153537, SGI153538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153539, SGI0, SGI0, SGI0, SGI153540, SGI153540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33769[] = {SGI153541, SGI153542, SGI153543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153544, SGI0, SGI0, SGI0, SGI153545, SGI153545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33770[] = {SGI153546, SGI153547, SGI153548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153549, SGI0, SGI0, SGI0, SGI153550, SGI153550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33771[] = {SGI153551, SGI153552, SGI153553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153554, SGI0, SGI0, SGI0, SGI153555, SGI153555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33772[] = {SGI153556, SGI0, SGI153557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33773[] = {SGI153558, SGI153559, SGI153561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153564, SGI0, SGI0, SGI0, SGI153566, SGI153566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33774[] = {SGI153560, SGI153562, SGI153563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153565, SGI0, SGI0, SGI0, SGI153567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33775[] = {SGI153568, SGI153569, SGI153570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153571, SGI0, SGI0, SGI0, SGI153572, SGI153572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33776[] = {SGI153573, SGI0, SGI153574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33777[] = {SGI153575, SGI153576, SGI153577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153578, SGI0, SGI0, SGI0, SGI153579, SGI153579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33778[] = {SGI153580, SGI153581, SGI153582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153583, SGI0, SGI0, SGI0, SGI153584, SGI153584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33779[] = {SGI153585, SGI153586, SGI153587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153588, SGI0, SGI0, SGI0, SGI153589, SGI153589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33780[] = {SGI153590, SGI153591, SGI153592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153593, SGI0, SGI0, SGI0, SGI153594, SGI153594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33781[] = {SGI153595, SGI0, SGI153596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33782[] = {SGI153597, SGI153598, SGI153599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153600, SGI0, SGI0, SGI0, SGI153601, SGI153601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33783[] = {SGI153602, SGI0, SGI153603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33784[] = {SGI153604, SGI153605, SGI153606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153607, SGI0, SGI0, SGI0, SGI153608, SGI153608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33785[] = {SGI153609, SGI0, SGI153610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33786[] = {SGI153611, SGI0, SGI153612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153613, SGI153614, SGI153614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33787[] = {SGI153615, SGI0, SGI153616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153617, SGI153618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33788[] = {SGI153619, SGI153620, SGI153621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153622, SGI0, SGI0, SGI0, SGI153623, SGI153623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33789[] = {SGI153624, SGI153625, SGI153626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153627, SGI0, SGI0, SGI0, SGI153628, SGI153628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33790[] = {SGI153629, SGI0, SGI153630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33791[] = {SGI153631, SGI153632, SGI153633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153634, SGI0, SGI0, SGI0, SGI153635, SGI153635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33792[] = {SGI153636, SGI153637, SGI153638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153639, SGI0, SGI0, SGI0, SGI153640, SGI153640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33793[] = {SGI153641, SGI0, SGI153642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153643, SGI153644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33794[] = {SGI153480, SGI153481, SGI153482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153485, SGI0, SGI0, SGI0, SGI153486, SGI153486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33795[] = {SGI153645, SGI0, SGI153646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33796[] = {SGI153647, SGI153648, SGI153649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153650, SGI0, SGI0, SGI0, SGI153651, SGI153651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33797[] = {SGI153652, SGI153653, SGI153654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153655, SGI0, SGI0, SGI0, SGI153656, SGI153656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33798[] = {SGI153657, SGI0, SGI153658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153662, SGI153663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33799[] = {SGI153659, SGI153660, SGI153661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153664, SGI0, SGI0, SGI0, SGI153665, SGI153665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33800[] = {SGI153666, SGI0, SGI153667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153671, SGI153672, SGI153672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33801[] = {SGI153668, SGI153669, SGI153670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153673, SGI0, SGI0, SGI0, SGI153674, SGI153674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33802[] = {SGI153675, SGI0, SGI153676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153677, SGI153678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33803[] = {SGI153679, SGI153680, SGI153681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153682, SGI0, SGI0, SGI0, SGI153683, SGI153683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33804[] = {SGI153685, SGI0, SGI153686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33805[] = {SGI153684, SGI0, SGI153687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153688, SGI153689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33806[] = {SGI153690, SGI0, SGI153691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153694, SGI0, SGI0, SGI0, SGI153695, SGI153695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33807[] = {SGI153692, SGI0, SGI153693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153696, SGI153697, SGI153697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33808[] = {SGI153698, SGI153699, SGI153700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153703, SGI0, SGI0, SGI0, SGI153704, SGI153704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33809[] = {SGI153701, SGI0, SGI153702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153705, SGI153706, SGI153706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33810[] = {SGI153707, SGI153708, SGI153709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153710, SGI0, SGI0, SGI0, SGI153711, SGI153711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33811[] = {SGI153712, SGI0, SGI153713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153716, SGI153718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33812[] = {SGI153714, SGI153715, SGI153717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153719, SGI0, SGI0, SGI0, SGI153720, SGI153720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33813[] = {SGI153721, SGI0, SGI153722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153723, SGI153724, SGI153724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33814[] = {SGI153725, SGI153726, SGI153727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153728, SGI0, SGI0, SGI0, SGI153729, SGI153729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33815[] = {SGI153730, SGI0, SGI153731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153735, SGI153737, SGI153737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33816[] = {SGI153732, SGI153733, SGI153734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153736, SGI0, SGI0, SGI0, SGI153738, SGI153738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33817[] = {SGI153739, SGI0, SGI153740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153744, SGI153745, SGI153745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33818[] = {SGI153741, SGI153742, SGI153743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153746, SGI0, SGI0, SGI0, SGI153747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33819[] = {SGI153748, SGI153749, SGI153750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153753, SGI0, SGI0, SGI0, SGI153754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33820[] = {SGI153751, SGI0, SGI153752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153755, SGI153756, SGI153756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33821[] = {SGI153757, SGI153758, SGI153759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153760, SGI0, SGI0, SGI0, SGI153761, SGI153761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33822[] = {SGI153762, SGI0, SGI153763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153764, SGI153765, SGI153765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33823[] = {SGI153766, SGI153767, SGI153768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153769, SGI0, SGI0, SGI0, SGI153770, SGI153770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33824[] = {SGI153771, SGI0, SGI153772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153773, SGI153774, SGI153774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33825[] = {SGI153775, SGI153776, SGI153777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153778, SGI0, SGI0, SGI0, SGI153780, SGI153780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33826[] = {SGI153779, SGI0, SGI153781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153782, SGI153783, SGI153783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33827[] = {SGI153784, SGI153785, SGI153786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153787, SGI0, SGI0, SGI0, SGI153788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33828[] = {SGI153789, SGI0, SGI153790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153791, SGI153792, SGI153792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33829[] = {SGI153793, SGI0, SGI153794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153795, SGI153796, SGI153796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33830[] = {SGI153797, SGI0, SGI153798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153799, SGI153800, SGI153800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33831[] = {SGI153801, SGI153802, SGI153803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153804, SGI0, SGI0, SGI0, SGI153805, SGI153805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33832[] = {SGI153807, SGI153808, SGI153810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153811, SGI0, SGI0, SGI0, SGI153814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33833[] = {SGI153806, SGI0, SGI153809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153812, SGI153813, SGI153813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33834[] = {SGI153816, SGI0, SGI153817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33835[] = {SGI153815, SGI0, SGI153818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153819, SGI153820, SGI153820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33836[] = {SGI153821, SGI153822, SGI153823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153824, SGI0, SGI0, SGI0, SGI153825, SGI153825, SGI0, SGI153827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33837[] = {SGI153826, SGI0, SGI153828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153831, SGI0, SGI0, SGI0, SGI153832, SGI153832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33838[] = {SGI153829, SGI0, SGI153830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153833, SGI153834, SGI153834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33839[] = {SGI153835, SGI153836, SGI153837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153839, SGI0, SGI0, SGI0, SGI153842, SGI153842, SGI0, SGI153844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33840[] = {SGI153838, SGI153840, SGI153841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153846, SGI0, SGI0, SGI0, SGI153847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33841[] = {SGI153843, SGI0, SGI153845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153848, SGI153849, SGI153849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33842[] = {SGI153850, SGI153852, SGI153854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153856, SGI0, SGI0, SGI0, SGI153859, SGI153859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33843[] = {SGI153851, SGI153853, SGI153855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153858, SGI0, SGI0, SGI0, SGI153861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33844[] = {SGI153857, SGI0, SGI153860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153862, SGI153863, SGI153863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33845[] = {SGI153864, SGI153865, SGI153866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153868, SGI0, SGI0, SGI0, SGI153871, SGI153871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33846[] = {SGI153867, SGI153869, SGI153870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153874, SGI0, SGI0, SGI0, SGI153875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33847[] = {SGI153872, SGI0, SGI153873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153876, SGI153877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33848[] = {SGI153878, SGI153879, SGI153880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153881, SGI0, SGI0, SGI0, SGI153883, SGI153883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33849[] = {SGI153882, SGI153885, SGI153886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153888, SGI0, SGI0, SGI0, SGI153889, SGI153889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33850[] = {SGI153884, SGI0, SGI153887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153890, SGI153891, SGI153891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33851[] = {SGI153892, SGI153893, SGI153894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153895, SGI0, SGI0, SGI0, SGI153896, SGI153896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33852[] = {SGI153898, SGI0, SGI153900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153903, SGI153905, SGI153905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33853[] = {SGI153897, SGI153899, SGI153901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153902, SGI0, SGI0, SGI0, SGI153904, SGI153904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33854[] = {SGI153906, SGI0, SGI153907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33855[] = {SGI153908, SGI0, SGI153909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153913, SGI153915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33856[] = {SGI153914, SGI0, SGI153916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33857[] = {SGI153910, SGI153911, SGI153912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153917, SGI0, SGI0, SGI0, SGI153918, SGI153918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33858[] = {SGI153920, SGI0, SGI153922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33859[] = {SGI153919, SGI0, SGI153921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153925, SGI153927, SGI153927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33860[] = {SGI153923, SGI153924, SGI153926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153928, SGI0, SGI0, SGI0, SGI153929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33861[] = {SGI153930, SGI153931, SGI153932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153933, SGI0, SGI0, SGI0, SGI153934, SGI153934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33862[] = {SGI153935, SGI153936, SGI153937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153938, SGI0, SGI0, SGI0, SGI153939, SGI153939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33863[] = {SGI153940, SGI153941, SGI153942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153943, SGI0, SGI0, SGI0, SGI153944, SGI153944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33864[] = {SGI153945, SGI153946, SGI153947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153948, SGI0, SGI0, SGI0, SGI153949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33865[] = {SGI153950, SGI153951, SGI153952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153953, SGI0, SGI0, SGI0, SGI153954, SGI153954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33866[] = {SGI153955, SGI153956, SGI153957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153958, SGI0, SGI0, SGI0, SGI153959, SGI153959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33867[] = {SGI153960, SGI153961, SGI153962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153963, SGI0, SGI0, SGI0, SGI153964, SGI153964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33868[] = {SGI153965, SGI0, SGI153966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33869[] = {SGI153968, SGI153970, SGI153972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153974, SGI0, SGI0, SGI0, SGI153976, SGI153976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33870[] = {SGI153967, SGI153969, SGI153971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153973, SGI0, SGI0, SGI0, SGI153975, SGI153975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33871[] = {SGI153977, SGI153979, SGI153981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153983, SGI0, SGI0, SGI0, SGI153985, SGI153985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33872[] = {SGI153978, SGI153980, SGI153982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153984, SGI0, SGI0, SGI0, SGI153986, SGI153986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33873[] = {SGI153987, SGI153988, SGI153989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153990, SGI0, SGI0, SGI0, SGI153991, SGI153991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33874[] = {SGI153992, SGI153993, SGI153994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI153995, SGI0, SGI0, SGI0, SGI153996, SGI153996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33875[] = {SGI153997, SGI0, SGI153998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33876[] = {SGI153999, SGI154000, SGI154001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154002, SGI0, SGI0, SGI0, SGI154003, SGI154003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33877[] = {SGI154004, SGI154005, SGI154006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154007, SGI0, SGI0, SGI0, SGI154008, SGI154008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33878[] = {SGI154009, SGI154010, SGI154011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154013, SGI0, SGI0, SGI0, SGI154015, SGI154015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33879[] = {SGI154012, SGI0, SGI154014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154016, SGI154017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33880[] = {SGI154018, SGI0, SGI154019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154020, SGI154021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33881[] = {SGI154022, SGI0, SGI154023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154024, SGI154025, SGI154025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33882[] = {SGI154026, SGI0, SGI154027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154028, SGI154029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33883[] = {SGI154030, SGI0, SGI154031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154032, SGI154033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33884[] = {SGI154034, SGI154035, SGI154036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154037, SGI0, SGI0, SGI0, SGI154038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33885[] = {SGI154040, SGI0, SGI154042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154045, SGI154047, SGI154047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33886[] = {SGI154039, SGI154041, SGI154043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154044, SGI0, SGI0, SGI0, SGI154046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33887[] = {SGI154048, SGI0, SGI154049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33888[] = {SGI154050, SGI154051, SGI154052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154053, SGI0, SGI0, SGI0, SGI154054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33889[] = {SGI154055, SGI0, SGI154056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33890[] = {SGI154057, SGI154058, SGI154059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154060, SGI0, SGI0, SGI0, SGI154061, SGI154061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33891[] = {SGI154062, SGI0, SGI154063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154064, SGI154065, SGI154065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33892[] = {SGI154066, SGI0, SGI154067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154068, SGI154069, SGI154069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33893[] = {SGI154070, SGI0, SGI154071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154072, SGI154073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33894[] = {SGI154074, SGI0, SGI154075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154078, SGI154079, SGI154079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33895[] = {SGI154076, SGI0, SGI154077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154080, SGI154081, SGI154081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33896[] = {SGI154082, SGI0, SGI154083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154084, SGI154085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33897[] = {SGI154086, SGI0, SGI154087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154088, SGI154089, SGI154089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33898[] = {SGI154090, SGI0, SGI154091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154092, SGI154093, SGI154093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33899[] = {SGI154094, SGI0, SGI154095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154096, SGI154097, SGI154097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33900[] = {SGI154098, SGI0, SGI154099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154100, SGI154101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33901[] = {SGI154102, SGI0, SGI154103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154105, SGI154106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33902[] = {SGI154104, SGI0, SGI154107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154108, SGI154109, SGI154109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33903[] = {SGI154110, SGI0, SGI154111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154112, SGI154114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33904[] = {SGI154113, SGI0, SGI154115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154116, SGI154117, SGI154117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33905[] = {SGI154118, SGI0, SGI154119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154120, SGI154121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33906[] = {SGI154122, SGI0, SGI154124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33907[] = {SGI154123, SGI0, SGI154125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154126, SGI154127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33908[] = {SGI154128, SGI154129, SGI154130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154131, SGI0, SGI0, SGI0, SGI154132, SGI154132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33909[] = {SGI154133, SGI154134, SGI154135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154136, SGI0, SGI0, SGI0, SGI154137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33910[] = {SGI154138, SGI0, SGI154139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154140, SGI154141, SGI154141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33911[] = {SGI154142, SGI0, SGI154143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154144, SGI154145, SGI154145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33912[] = {SGI154146, SGI0, SGI154147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154148, SGI154149, SGI154149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33913[] = {SGI154150, SGI0, SGI154151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154152, SGI154153, SGI154153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33914[] = {SGI154154, SGI154155, SGI154156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154157, SGI0, SGI0, SGI0, SGI154158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33915[] = {SGI154159, SGI0, SGI154160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33916[] = {SGI154161, SGI0, SGI154162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154163, SGI154164, SGI154164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33917[] = {SGI154165, SGI154166, SGI154167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154168, SGI0, SGI0, SGI0, SGI154169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33918[] = {SGI154170, SGI0, SGI154171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33919[] = {SGI154172, SGI0, SGI154173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154174, SGI154175, SGI154175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33920[] = {SGI154176, SGI154177, SGI154178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154179, SGI0, SGI0, SGI0, SGI154180, SGI154180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33921[] = {SGI154181, SGI0, SGI154182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154183, SGI0, SGI0, SGI0, SGI154184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33922[] = {SGI154185, SGI154186, SGI154187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154188, SGI0, SGI0, SGI0, SGI154189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33923[] = {SGI154190, SGI154192, SGI154194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154196, SGI0, SGI0, SGI0, SGI154198, SGI154198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33924[] = {SGI154191, SGI154193, SGI154195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154197, SGI0, SGI0, SGI0, SGI154199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33925[] = {SGI154200, SGI154201, SGI154203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154206, SGI0, SGI0, SGI0, SGI154208, SGI154208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33926[] = {SGI154202, SGI154204, SGI154205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154207, SGI0, SGI0, SGI0, SGI154209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33927[] = {SGI154211, SGI0, SGI154214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33928[] = {SGI154210, SGI154212, SGI154213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154215, SGI0, SGI0, SGI0, SGI154216, SGI154216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33929[] = {SGI154217, SGI0, SGI154219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33930[] = {SGI154218, SGI0, SGI154220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154221, SGI154222, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33931[] = {SGI154223, SGI154224, SGI154225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154226, SGI0, SGI0, SGI0, SGI154227, SGI154227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33932[] = {SGI154228, SGI0, SGI154229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33933[] = {SGI154230, SGI154231, SGI154232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154233, SGI0, SGI0, SGI0, SGI154234, SGI154234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33934[] = {SGI154235, SGI0, SGI154236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154240, SGI154241, SGI154241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33935[] = {SGI154237, SGI154238, SGI154239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154242, SGI0, SGI0, SGI0, SGI154243, SGI154243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33936[] = {SGI154244, SGI0, SGI154245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154249, SGI154250, SGI154250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33937[] = {SGI154246, SGI154247, SGI154248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154251, SGI0, SGI0, SGI0, SGI154252, SGI154252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33938[] = {SGI154253, SGI154254, SGI154255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154256, SGI0, SGI0, SGI0, SGI154257, SGI154257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33939[] = {SGI154258, SGI154259, SGI154260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154261, SGI0, SGI0, SGI0, SGI154262, SGI154262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33940[] = {SGI154263, SGI0, SGI154264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154265, SGI154266, SGI154266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33941[] = {SGI154267, SGI154268, SGI154269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154270, SGI0, SGI0, SGI0, SGI154271, SGI154271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33942[] = {SGI154272, SGI0, SGI154273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33943[] = {SGI154274, SGI154275, SGI154276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154277, SGI0, SGI0, SGI0, SGI154278, SGI154278, SGI0, SGI154279, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33944[] = {SGI154280, SGI154281, SGI154282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154283, SGI0, SGI0, SGI0, SGI154284, SGI154284, SGI0, SGI154285, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33945[] = {SGI154286, SGI154287, SGI154288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154289, SGI0, SGI0, SGI0, SGI154290, SGI154290, SGI0, SGI154291, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33946[] = {SGI154292, SGI154293, SGI154294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154295, SGI0, SGI0, SGI0, SGI154296, SGI154296, SGI0, SGI154297, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33947[] = {SGI154298, SGI0, SGI154299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154300, SGI154301, SGI154301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33948[] = {SGI154302, SGI0, SGI154303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154304, SGI154305, SGI154305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33949[] = {SGI154306, SGI0, SGI154307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154308, SGI154309, SGI154309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33950[] = {SGI154310, SGI0, SGI154311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154312, SGI154313, SGI154313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33951[] = {SGI154314, SGI0, SGI154315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154318, SGI154319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33952[] = {SGI154316, SGI0, SGI154317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154320, SGI154321, SGI154321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33953[] = {SGI154322, SGI0, SGI154323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154324, SGI154325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33954[] = {SGI154326, SGI154327, SGI154328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154329, SGI0, SGI0, SGI0, SGI154330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33955[] = {SGI154331, SGI154332, SGI154333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154334, SGI0, SGI0, SGI0, SGI154335, SGI154335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33956[] = {SGI154336, SGI0, SGI154337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33957[] = {SGI154338, SGI0, SGI154339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154340, SGI154341, SGI154341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33958[] = {SGI154342, SGI0, SGI154343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154344, SGI154345, SGI154345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33959[] = {SGI154346, SGI0, SGI154347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154348, SGI154349, SGI154349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33960[] = {SGI154350, SGI0, SGI154351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154352, SGI154353, SGI154353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33961[] = {SGI154354, SGI0, SGI154355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154356, SGI154357, SGI154357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33962[] = {SGI154358, SGI0, SGI154359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154360, SGI154361, SGI154361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33963[] = {SGI154362, SGI0, SGI154363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154364, SGI154365, SGI154365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33964[] = {SGI154366, SGI0, SGI154367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154368, SGI154369, SGI154369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33965[] = {SGI154370, SGI154371, SGI154372, SGI0, SGI0, SGI0, SGI154373, SGI154374, SGI0, SGI154375, SGI0, SGI0, SGI0, SGI154376, SGI154376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33966[] = {SGI154377, SGI154378, SGI154379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154380, SGI0, SGI0, SGI0, SGI154381, SGI154381, SGI0, SGI154382, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33967[] = {SGI154383, SGI154384, SGI154385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154386, SGI0, SGI0, SGI0, SGI154389, SGI154389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33968[] = {SGI154387, SGI154388, SGI154390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154391, SGI0, SGI0, SGI0, SGI154392, SGI154392, SGI0, SGI154393, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33969[] = {SGI154394, SGI0, SGI154395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33970[] = {SGI154396, SGI0, SGI154397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154398, SGI0, SGI0, SGI0, SGI154399, SGI154399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33971[] = {SGI154400, SGI154401, SGI154402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154403, SGI0, SGI0, SGI0, SGI154404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33972[] = {SGI154405, SGI154406, SGI154407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154408, SGI0, SGI0, SGI0, SGI154409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33973[] = {SGI154410, SGI154411, SGI154412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154413, SGI0, SGI0, SGI0, SGI154414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33974[] = {SGI154415, SGI154416, SGI154417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154418, SGI0, SGI0, SGI0, SGI154419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33975[] = {SGI154420, SGI0, SGI154421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33976[] = {SGI154422, SGI154423, SGI154424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154425, SGI0, SGI0, SGI0, SGI154426, SGI154426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33977[] = {SGI154427, SGI154428, SGI154429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154430, SGI0, SGI0, SGI0, SGI154431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33978[] = {SGI154432, SGI0, SGI154433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154434, SGI154436, SGI154436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33979[] = {SGI154435, SGI154437, SGI154438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154439, SGI0, SGI0, SGI0, SGI154440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33980[] = {SGI154441, SGI0, SGI154442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154443, SGI154444, SGI154444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33981[] = {SGI154445, SGI0, SGI154446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154447, SGI154448, SGI154448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33982[] = {SGI154449, SGI0, SGI154450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154451, SGI154452, SGI154452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33983[] = {SGI154453, SGI0, SGI154454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154455, SGI154456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33984[] = {SGI154457, SGI0, SGI154458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154459, SGI154460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33985[] = {SGI154461, SGI0, SGI154462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154463, SGI154464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33986[] = {SGI154465, SGI0, SGI154466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154467, SGI154468, SGI154468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33987[] = {SGI154469, SGI0, SGI154470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154471, SGI154472, SGI154472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33988[] = {SGI154473, SGI154474, SGI154475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154476, SGI0, SGI0, SGI0, SGI154477, SGI154477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33989[] = {SGI154478, SGI154479, SGI154480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154481, SGI0, SGI0, SGI0, SGI154482, SGI154482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33990[] = {SGI154483, SGI0, SGI154484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33991[] = {SGI154485, SGI154486, SGI154487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154488, SGI0, SGI0, SGI0, SGI154489, SGI154489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33992[] = {SGI154490, SGI154491, SGI154492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154493, SGI0, SGI0, SGI0, SGI154494, SGI154494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33993[] = {SGI154495, SGI154496, SGI154497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154498, SGI0, SGI0, SGI0, SGI154499, SGI154499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33994[] = {SGI154500, SGI0, SGI154501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33995[] = {SGI154502, SGI0, SGI154503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154504, SGI154505, SGI154505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33996[] = {SGI154506, SGI0, SGI154507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154508, SGI154509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33997[] = {SGI154510, SGI154511, SGI154512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154513, SGI0, SGI0, SGI0, SGI154514, SGI154514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33998[] = {SGI154516, SGI154517, SGI154518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154519, SGI0, SGI0, SGI0, SGI154520, SGI154520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG33999[] = {SGI154521, SGI154523, SGI154524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154525, SGI0, SGI0, SGI0, SGI154526, SGI154526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34000[] = {SGI154527, SGI154528, SGI154529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154530, SGI0, SGI0, SGI0, SGI154531, SGI154531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34001[] = {SGI154532, SGI154533, SGI154535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154536, SGI0, SGI0, SGI0, SGI154537, SGI154537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34002[] = {SGI154541, SGI0, SGI154543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34003[] = {SGI154538, SGI154539, SGI154540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154542, SGI0, SGI0, SGI0, SGI154544, SGI154544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34004[] = {SGI154546, SGI154547, SGI154549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154552, SGI0, SGI0, SGI0, SGI154553, SGI154553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34005[] = {SGI154548, SGI154550, SGI154551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154554, SGI0, SGI0, SGI0, SGI154555, SGI154555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34006[] = {SGI154557, SGI0, SGI154558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34007[] = {SGI154559, SGI154560, SGI154561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154562, SGI0, SGI0, SGI0, SGI154563, SGI154563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34008[] = {SGI154564, SGI0, SGI154566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34009[] = {SGI154567, SGI154568, SGI154569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154570, SGI0, SGI0, SGI0, SGI154571, SGI154571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34010[] = {SGI154573, SGI154574, SGI154575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154576, SGI0, SGI0, SGI0, SGI154577, SGI154577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34011[] = {SGI154579, SGI154580, SGI154581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154582, SGI0, SGI0, SGI0, SGI154583, SGI154583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34012[] = {SGI154585, SGI0, SGI154586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154587, SGI154588, SGI154588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34013[] = {SGI154589, SGI0, SGI154590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154591, SGI154592, SGI154592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34014[] = {SGI154593, SGI0, SGI154594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154595, SGI154596, SGI154596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34015[] = {SGI154597, SGI0, SGI154598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154602, SGI154603, SGI154603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34016[] = {SGI154599, SGI154600, SGI154601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154604, SGI0, SGI0, SGI0, SGI154605, SGI154605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34017[] = {SGI154608, SGI0, SGI154609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34018[] = {SGI154606, SGI0, SGI154607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154610, SGI154611, SGI154611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34019[] = {SGI154612, SGI0, SGI154613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154614, SGI154615, SGI154615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34020[] = {SGI154616, SGI0, SGI154617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154618, SGI154619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34021[] = {SGI154620, SGI0, SGI154621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154622, SGI154623, SGI154623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34022[] = {SGI154624, SGI0, SGI154625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154626, SGI154627, SGI154627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34023[] = {SGI154628, SGI0, SGI154629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154630, SGI154631, SGI154631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34024[] = {SGI154632, SGI0, SGI154633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154634, SGI154635, SGI154635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34025[] = {SGI154636, SGI0, SGI154637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154638, SGI154639, SGI154639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34026[] = {SGI154640, SGI0, SGI154641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154642, SGI154643, SGI154643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34027[] = {SGI154644, SGI154645, SGI154646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154647, SGI0, SGI0, SGI0, SGI154648, SGI154648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34028[] = {SGI154649, SGI0, SGI154650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34029[] = {SGI154651, SGI154652, SGI154653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154654, SGI0, SGI0, SGI0, SGI154655, SGI154655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34030[] = {SGI154656, SGI0, SGI154657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154658, SGI154659, SGI154659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34031[] = {SGI154660, SGI154661, SGI154662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154663, SGI0, SGI0, SGI0, SGI154664, SGI154664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34032[] = {SGI154665, SGI154666, SGI154667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154668, SGI0, SGI0, SGI0, SGI154669, SGI154669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34033[] = {SGI154670, SGI0, SGI154671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34034[] = {SGI154672, SGI0, SGI154673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154674, SGI154675, SGI154675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34035[] = {SGI154676, SGI154677, SGI154678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154679, SGI0, SGI0, SGI0, SGI154680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34036[] = {SGI154681, SGI154682, SGI154683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154684, SGI0, SGI0, SGI0, SGI154685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34037[] = {SGI154686, SGI154687, SGI154688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154689, SGI0, SGI0, SGI0, SGI154690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34038[] = {SGI154691, SGI154692, SGI154693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154694, SGI0, SGI0, SGI0, SGI154695, SGI154695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34039[] = {SGI154696, SGI0, SGI154697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34040[] = {SGI154698, SGI154699, SGI154700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154701, SGI0, SGI0, SGI0, SGI154702, SGI154702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34041[] = {SGI154703, SGI0, SGI154704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34042[] = {SGI154705, SGI0, SGI154706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154707, SGI154708, SGI154708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34043[] = {SGI154709, SGI0, SGI154710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154711, SGI154712, SGI154712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34044[] = {SGI154713, SGI0, SGI154714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154715, SGI154716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34045[] = {SGI154717, SGI0, SGI154718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154719, SGI154720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34046[] = {SGI154721, SGI0, SGI154722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154723, SGI154724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34047[] = {SGI154725, SGI154726, SGI154727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154728, SGI0, SGI0, SGI154729, SGI154730, SGI154730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34048[] = {SGI154731, SGI0, SGI154732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154733, SGI154734, SGI154734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34049[] = {SGI154735, SGI0, SGI154736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154737, SGI154738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34050[] = {SGI154739, SGI0, SGI154740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154741, SGI154742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34051[] = {SGI154743, SGI0, SGI154744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154745, SGI154746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34052[] = {SGI154747, SGI0, SGI154748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154749, SGI154750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34053[] = {SGI154751, SGI0, SGI154752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154753, SGI154754, SGI154754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34054[] = {SGI154755, SGI0, SGI154756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154757, SGI154758, SGI154758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34055[] = {SGI154759, SGI0, SGI154760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154761, SGI154762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34056[] = {SGI154763, SGI0, SGI154764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154765, SGI154766, SGI154766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34057[] = {SGI154767, SGI0, SGI154768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154769, SGI154770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34058[] = {SGI154771, SGI0, SGI154772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154773, SGI154774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34059[] = {SGI154775, SGI0, SGI154776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154777, SGI154778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34060[] = {SGI154779, SGI0, SGI154780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154781, SGI154782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34061[] = {SGI154783, SGI0, SGI154784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154785, SGI154786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34062[] = {SGI154787, SGI0, SGI154788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154789, SGI154790, SGI154790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34063[] = {SGI154791, SGI154792, SGI154793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154794, SGI0, SGI0, SGI0, SGI154795, SGI154795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34064[] = {SGI154796, SGI154797, SGI154798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154799, SGI0, SGI0, SGI0, SGI154800, SGI154800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34065[] = {SGI154801, SGI0, SGI154802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34066[] = {SGI154803, SGI0, SGI154804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34067[] = {SGI154805, SGI154806, SGI154807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154808, SGI0, SGI0, SGI0, SGI154809, SGI154809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34068[] = {SGI154810, SGI0, SGI154811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154812, SGI0, SGI0, SGI0, SGI154813, SGI154813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34069[] = {SGI154814, SGI0, SGI154815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154816, SGI154817, SGI154817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34070[] = {SGI154818, SGI0, SGI154819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154820, SGI154821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34071[] = {SGI154822, SGI154823, SGI154824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154825, SGI0, SGI0, SGI0, SGI154826, SGI154826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34072[] = {SGI154827, SGI154828, SGI154829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154830, SGI0, SGI0, SGI0, SGI154831, SGI154831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34073[] = {SGI154832, SGI154833, SGI154834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154835, SGI0, SGI0, SGI0, SGI154836, SGI154836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34074[] = {SGI154837, SGI0, SGI154838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34075[] = {SGI154839, SGI0, SGI154840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34076[] = {SGI154841, SGI154842, SGI154843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154844, SGI0, SGI0, SGI0, SGI154845, SGI154845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34077[] = {SGI154846, SGI0, SGI154847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34078[] = {SGI154848, SGI154849, SGI154850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154851, SGI0, SGI0, SGI0, SGI154852, SGI154852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34079[] = {SGI154853, SGI154854, SGI154855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154856, SGI0, SGI0, SGI0, SGI154857, SGI154857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34080[] = {SGI154858, SGI154859, SGI154860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154861, SGI0, SGI0, SGI0, SGI154862, SGI154862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34081[] = {SGI154863, SGI154864, SGI154865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154866, SGI0, SGI0, SGI0, SGI154867, SGI154867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34082[] = {SGI154868, SGI0, SGI154869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154870, SGI154871, SGI154871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34083[] = {SGI154872, SGI0, SGI154873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154876, SGI154877, SGI154877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34084[] = {SGI154874, SGI0, SGI154875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154878, SGI154879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34085[] = {SGI154880, SGI0, SGI154881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154882, SGI154883, SGI154883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34086[] = {SGI154884, SGI0, SGI154885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154886, SGI154887, SGI154887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34087[] = {SGI154888, SGI0, SGI154889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34088[] = {SGI154890, SGI154891, SGI154892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154893, SGI0, SGI0, SGI0, SGI154894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34089[] = {SGI154895, SGI154896, SGI154897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154898, SGI0, SGI0, SGI0, SGI154899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34090[] = {SGI154900, SGI0, SGI154901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154902, SGI154903, SGI154903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34091[] = {SGI154904, SGI0, SGI154905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154906, SGI154907, SGI154907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34092[] = {SGI154908, SGI154909, SGI154910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154911, SGI0, SGI0, SGI0, SGI154912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34093[] = {SGI154913, SGI154914, SGI154915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154916, SGI0, SGI0, SGI0, SGI154917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34094[] = {SGI154918, SGI154919, SGI154920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154921, SGI0, SGI0, SGI0, SGI154922, SGI154922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34095[] = {SGI154923, SGI0, SGI154924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34096[] = {SGI154925, SGI0, SGI154926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154927, SGI154928, SGI154928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34097[] = {SGI154929, SGI154930, SGI154931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154932, SGI0, SGI0, SGI0, SGI154933, SGI154933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34098[] = {SGI154934, SGI154935, SGI154936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154937, SGI0, SGI0, SGI0, SGI154938, SGI154938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34099[] = {SGI154939, SGI154940, SGI154941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154942, SGI0, SGI0, SGI0, SGI154943, SGI154943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34100[] = {SGI154944, SGI0, SGI154945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34101[] = {SGI154946, SGI154947, SGI154948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154949, SGI0, SGI0, SGI0, SGI154950, SGI154950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34102[] = {SGI154951, SGI154952, SGI154953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154954, SGI0, SGI0, SGI0, SGI154955, SGI154955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34103[] = {SGI154956, SGI154957, SGI154958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154959, SGI0, SGI0, SGI0, SGI154960, SGI154960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34104[] = {SGI154961, SGI154962, SGI154963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154965, SGI0, SGI0, SGI0, SGI154967, SGI154967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34105[] = {SGI154964, SGI0, SGI154966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154968, SGI154969, SGI154969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34106[] = {SGI154970, SGI154971, SGI154972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154973, SGI0, SGI0, SGI0, SGI154974, SGI154974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34107[] = {SGI154975, SGI0, SGI154976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154977, SGI154978, SGI154978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34108[] = {SGI154979, SGI0, SGI154980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154981, SGI154982, SGI154982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34109[] = {SGI154983, SGI0, SGI154984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154985, SGI154986, SGI154986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34110[] = {SGI154987, SGI0, SGI154988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154989, SGI154990, SGI154990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34111[] = {SGI0, SGI0, SGI154991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154992, SGI0, SGI0, SGI0, SGI154993, SGI154993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34112[] = {SGI154994, SGI154995, SGI154996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI154997, SGI0, SGI0, SGI0, SGI154998, SGI154998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34113[] = {SGI154999, SGI0, SGI155000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34114[] = {SGI155001, SGI155002, SGI155003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155004, SGI155004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34115[] = {SGI155005, SGI155006, SGI155007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155008, SGI0, SGI0, SGI0, SGI155009, SGI155009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34116[] = {SGI155010, SGI0, SGI155011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155012, SGI155013, SGI155013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34117[] = {SGI155014, SGI0, SGI155015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155016, SGI155017, SGI155017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34118[] = {SGI155018, SGI0, SGI155019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155020, SGI155021, SGI155021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34119[] = {SGI155022, SGI0, SGI155023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155024, SGI155025, SGI155025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34120[] = {SGI155026, SGI0, SGI155027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155028, SGI155029, SGI155029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34121[] = {SGI155030, SGI0, SGI155031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155032, SGI155033, SGI155033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34122[] = {SGI155034, SGI0, SGI155035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155036, SGI155037, SGI155037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34123[] = {SGI155038, SGI0, SGI155039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155040, SGI155041, SGI155041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34124[] = {SGI155042, SGI0, SGI155043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155044, SGI155045, SGI155045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34125[] = {SGI155046, SGI0, SGI155047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155048, SGI155049, SGI155049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34126[] = {SGI155050, SGI0, SGI155051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155052, SGI155053, SGI155053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34127[] = {SGI155054, SGI0, SGI155055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155056, SGI155057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34128[] = {SGI155058, SGI0, SGI155059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155063, SGI155064, SGI155064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34129[] = {SGI155060, SGI155061, SGI155062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155065, SGI0, SGI0, SGI0, SGI155066, SGI155066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34130[] = {SGI155067, SGI155068, SGI155069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155070, SGI0, SGI0, SGI0, SGI155072, SGI155072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34131[] = {SGI155071, SGI0, SGI155073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155074, SGI155075, SGI155075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34132[] = {SGI155076, SGI0, SGI155077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34133[] = {SGI155078, SGI155079, SGI155080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155081, SGI0, SGI0, SGI0, SGI155082, SGI155082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34134[] = {SGI155083, SGI0, SGI155084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155085, SGI155086, SGI155086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34135[] = {SGI155087, SGI0, SGI155088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155089, SGI155090, SGI155090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34136[] = {SGI155091, SGI0, SGI155092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155093, SGI155094, SGI155094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34137[] = {SGI155095, SGI0, SGI155096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155097, SGI155098, SGI155098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34138[] = {SGI155099, SGI0, SGI155100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155101, SGI155102, SGI155102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34139[] = {SGI155103, SGI0, SGI155104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155108, SGI155110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34140[] = {SGI155105, SGI155106, SGI155107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155109, SGI0, SGI0, SGI0, SGI155111, SGI155111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34141[] = {SGI155112, SGI0, SGI155113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34142[] = {SGI155114, SGI155115, SGI155116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155117, SGI0, SGI0, SGI0, SGI155118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34143[] = {SGI155119, SGI0, SGI155120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155121, SGI155122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34144[] = {SGI155123, SGI0, SGI155124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155125, SGI155126, SGI155126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34145[] = {SGI155127, SGI0, SGI155128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155131, SGI155132, SGI155132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34146[] = {SGI155129, SGI0, SGI155130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155133, SGI155134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34147[] = {SGI155135, SGI0, SGI155136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155139, SGI155140, SGI155140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34148[] = {SGI155137, SGI0, SGI155138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155141, SGI155142, SGI155142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34149[] = {SGI155143, SGI0, SGI155144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155145, SGI155146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34150[] = {SGI155147, SGI0, SGI155148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155149, SGI155150, SGI155150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34151[] = {SGI155151, SGI0, SGI155152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155153, SGI155154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34152[] = {SGI155155, SGI0, SGI155156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155157, SGI155158, SGI155158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34153[] = {SGI155159, SGI155160, SGI155161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155162, SGI0, SGI0, SGI0, SGI155163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34154[] = {SGI155164, SGI0, SGI155166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34155[] = {SGI155165, SGI0, SGI155167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155168, SGI155169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34156[] = {SGI155170, SGI155171, SGI155172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155174, SGI0, SGI0, SGI0, SGI155176, SGI155177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34157[] = {SGI155173, SGI0, SGI155175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155178, SGI155179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34158[] = {SGI155180, SGI0, SGI155181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155182, SGI155183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34159[] = {SGI155184, SGI0, SGI155185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155186, SGI155187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34160[] = {SGI155188, SGI155189, SGI155190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155191, SGI0, SGI0, SGI0, SGI155192, SGI155192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34161[] = {SGI0, SGI0, SGI155193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155194, SGI155195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34162[] = {SGI155196, SGI155197, SGI155198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155199, SGI0, SGI0, SGI0, SGI155200, SGI155200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34163[] = {SGI155201, SGI0, SGI155202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155203, SGI155204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34164[] = {SGI155205, SGI155206, SGI155207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155208, SGI0, SGI0, SGI0, SGI155209, SGI155209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34165[] = {SGI155210, SGI0, SGI155211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155212, SGI155213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34166[] = {SGI155214, SGI155215, SGI155216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155217, SGI0, SGI0, SGI0, SGI155218, SGI155218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34167[] = {SGI155219, SGI0, SGI155220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155221, SGI155222, SGI155222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34168[] = {SGI155223, SGI155224, SGI155225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155226, SGI0, SGI0, SGI0, SGI155227, SGI155227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34169[] = {SGI155228, SGI0, SGI155229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155231, SGI155232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34170[] = {SGI155230, SGI155233, SGI155234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155235, SGI0, SGI0, SGI0, SGI155236, SGI155236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34171[] = {SGI155237, SGI0, SGI155238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155239, SGI155241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34172[] = {SGI155240, SGI155242, SGI155243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155244, SGI0, SGI0, SGI0, SGI155245, SGI155245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34173[] = {SGI155246, SGI0, SGI155247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155249, SGI155251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34174[] = {SGI155248, SGI155250, SGI155252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155253, SGI0, SGI0, SGI0, SGI155254, SGI155254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34175[] = {SGI155255, SGI155256, SGI155257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155259, SGI0, SGI0, SGI0, SGI155261, SGI155261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34176[] = {SGI155258, SGI0, SGI155260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155262, SGI155263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34177[] = {SGI155264, SGI155265, SGI155266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155267, SGI0, SGI0, SGI0, SGI155269, SGI155269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34178[] = {SGI155268, SGI0, SGI155270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155271, SGI155272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34179[] = {SGI155273, SGI155274, SGI155275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155277, SGI0, SGI0, SGI0, SGI155279, SGI155279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34180[] = {SGI155276, SGI0, SGI155278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155280, SGI155281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34181[] = {SGI155282, SGI155283, SGI155284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155288, SGI0, SGI0, SGI0, SGI155290, SGI155290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34182[] = {SGI155285, SGI0, SGI155286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155291, SGI155292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34183[] = {SGI155287, SGI0, SGI155289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155293, SGI155294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34184[] = {SGI155295, SGI0, SGI155297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155302, SGI155304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34185[] = {SGI155296, SGI155298, SGI155299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155303, SGI0, SGI0, SGI0, SGI155305, SGI155305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34186[] = {SGI155300, SGI0, SGI155301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155306, SGI155307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34187[] = {SGI155308, SGI0, SGI155309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155313, SGI155314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34188[] = {SGI155310, SGI155311, SGI155312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155316, SGI0, SGI0, SGI0, SGI155318, SGI155318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34189[] = {SGI155315, SGI0, SGI155317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155319, SGI155320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34190[] = {SGI155321, SGI155322, SGI155323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155325, SGI0, SGI0, SGI0, SGI155327, SGI155327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34191[] = {SGI155324, SGI0, SGI155326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155328, SGI155329, SGI155329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34192[] = {SGI155330, SGI155331, SGI155332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155334, SGI0, SGI0, SGI0, SGI155336, SGI155336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34193[] = {SGI155333, SGI0, SGI155335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155337, SGI155338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34194[] = {SGI155339, SGI155340, SGI155341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155344, SGI0, SGI0, SGI0, SGI155345, SGI155345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34195[] = {SGI155342, SGI0, SGI155343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155346, SGI155347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34196[] = {SGI155348, SGI155349, SGI155350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155353, SGI0, SGI0, SGI0, SGI155354, SGI155354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34197[] = {SGI155351, SGI0, SGI155352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155355, SGI155356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34198[] = {SGI155357, SGI155358, SGI155359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155362, SGI0, SGI0, SGI0, SGI155363, SGI155363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34199[] = {SGI155360, SGI0, SGI155361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155364, SGI155365, SGI155365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34200[] = {SGI155366, SGI155367, SGI155368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155369, SGI0, SGI0, SGI0, SGI155370, SGI155370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34201[] = {SGI155371, SGI0, SGI155372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155373, SGI155374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34202[] = {SGI155375, SGI155376, SGI155377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155378, SGI0, SGI0, SGI0, SGI155379, SGI155379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34203[] = {SGI155380, SGI0, SGI155381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155382, SGI155383, SGI155383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34204[] = {SGI155384, SGI155385, SGI155386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155387, SGI0, SGI0, SGI0, SGI155388, SGI155388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34205[] = {SGI155389, SGI0, SGI155390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155391, SGI155393, SGI155393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34206[] = {SGI155392, SGI155394, SGI155395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155396, SGI0, SGI0, SGI0, SGI155397, SGI155397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34207[] = {SGI155398, SGI0, SGI155399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155401, SGI155403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34208[] = {SGI155400, SGI155402, SGI155404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155405, SGI0, SGI0, SGI0, SGI155406, SGI155406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34209[] = {SGI155407, SGI155408, SGI155409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155410, SGI0, SGI0, SGI0, SGI155411, SGI155411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34210[] = {SGI155412, SGI0, SGI155413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155414, SGI155416, SGI155416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34211[] = {SGI155415, SGI155417, SGI155418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155419, SGI0, SGI0, SGI0, SGI155420, SGI155420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34212[] = {SGI155421, SGI0, SGI155422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155423, SGI155424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34213[] = {SGI155425, SGI0, SGI155426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155427, SGI155428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34214[] = {SGI155429, SGI0, SGI155430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155431, SGI155432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34215[] = {SGI155433, SGI0, SGI155434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155435, SGI155436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34216[] = {SGI155437, SGI0, SGI155438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155439, SGI155440, SGI155440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34217[] = {SGI155441, SGI0, SGI155442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155443, SGI155444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34218[] = {SGI155445, SGI0, SGI155446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155447, SGI155448, SGI155448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34219[] = {SGI155449, SGI0, SGI155450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155451, SGI155452, SGI155452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34220[] = {SGI155453, SGI0, SGI155454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155455, SGI155456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34221[] = {SGI155457, SGI0, SGI155458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155459, SGI155460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34222[] = {SGI155461, SGI0, SGI155462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155463, SGI155464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34223[] = {SGI155465, SGI0, SGI155466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155467, SGI155468, SGI155468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34224[] = {SGI155469, SGI0, SGI155470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155471, SGI155472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34225[] = {SGI155473, SGI0, SGI155474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155475, SGI155476, SGI155476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34226[] = {SGI155477, SGI0, SGI155478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155479, SGI155480, SGI155480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34227[] = {SGI155481, SGI0, SGI155482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155483, SGI155484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34228[] = {SGI155485, SGI0, SGI155486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155487, SGI155488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34229[] = {SGI155489, SGI0, SGI155490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155491, SGI155492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34230[] = {SGI155493, SGI0, SGI155494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155495, SGI155496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34231[] = {SGI155497, SGI0, SGI155498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155499, SGI155500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34232[] = {SGI155501, SGI0, SGI155502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155503, SGI155504, SGI155504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34233[] = {SGI155505, SGI0, SGI155506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155507, SGI155508, SGI155508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34234[] = {SGI155509, SGI0, SGI155510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155511, SGI155512, SGI155512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34235[] = {SGI155513, SGI0, SGI155514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155515, SGI155516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34236[] = {SGI155518, SGI0, SGI155520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155523, SGI155524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34237[] = {SGI155517, SGI155519, SGI155521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155522, SGI0, SGI0, SGI0, SGI155525, SGI155525, SGI0, SGI155526, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34238[] = {SGI155527, SGI0, SGI155528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155532, SGI155533, SGI155533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34239[] = {SGI155529, SGI155530, SGI155531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155534, SGI0, SGI0, SGI0, SGI155535, SGI155535, SGI0, SGI155536, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34240[] = {SGI155537, SGI0, SGI155538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155539, SGI155540, SGI155540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34241[] = {SGI155541, SGI0, SGI155542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155543, SGI155544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34242[] = {SGI155545, SGI0, SGI155546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155547, SGI155548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34243[] = {SGI155549, SGI0, SGI155550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155551, SGI155552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34244[] = {SGI155553, SGI155554, SGI155555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155556, SGI0, SGI0, SGI0, SGI155557, SGI155557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34245[] = {SGI155558, SGI0, SGI155559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34246[] = {SGI155560, SGI155562, SGI155563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155566, SGI0, SGI0, SGI0, SGI155568, SGI155568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34247[] = {SGI155561, SGI155564, SGI155565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155567, SGI0, SGI0, SGI0, SGI155569, SGI155569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34248[] = {SGI155570, SGI155571, SGI155572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155573, SGI0, SGI0, SGI0, SGI155574, SGI155574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34249[] = {SGI155575, SGI155576, SGI155577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155578, SGI0, SGI0, SGI0, SGI155579, SGI155579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34250[] = {SGI155580, SGI155581, SGI155582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155583, SGI0, SGI0, SGI0, SGI155584, SGI155584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34251[] = {SGI155585, SGI155586, SGI155587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155588, SGI0, SGI0, SGI0, SGI155589, SGI155589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34252[] = {SGI155590, SGI155591, SGI155592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155593, SGI0, SGI0, SGI0, SGI155594, SGI155594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34253[] = {SGI155595, SGI155596, SGI155597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155598, SGI0, SGI0, SGI0, SGI155599, SGI155599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34254[] = {SGI155600, SGI0, SGI155601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34255[] = {SGI155602, SGI155603, SGI155604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155605, SGI0, SGI0, SGI0, SGI155606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34256[] = {SGI155607, SGI0, SGI155608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34257[] = {SGI155609, SGI155610, SGI155611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155612, SGI0, SGI0, SGI0, SGI155613, SGI155613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34258[] = {SGI155614, SGI155615, SGI155616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155617, SGI0, SGI0, SGI0, SGI155618, SGI155618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34259[] = {SGI155619, SGI155620, SGI155621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155622, SGI0, SGI0, SGI0, SGI155623, SGI155623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34260[] = {SGI155624, SGI0, SGI155625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34261[] = {SGI155626, SGI0, SGI155627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155628, SGI155629, SGI155629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34262[] = {SGI155630, SGI0, SGI155631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155632, SGI155633, SGI155633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34263[] = {SGI155634, SGI0, SGI155635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155636, SGI155637, SGI155637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34264[] = {SGI155638, SGI0, SGI155639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155640, SGI155641, SGI155641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34265[] = {SGI155642, SGI0, SGI155643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155644, SGI155645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34266[] = {SGI155646, SGI155647, SGI155648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155649, SGI0, SGI0, SGI0, SGI155650, SGI155650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34267[] = {SGI155651, SGI155652, SGI155653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155654, SGI0, SGI0, SGI0, SGI155655, SGI155655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34268[] = {SGI155656, SGI155657, SGI155658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155659, SGI0, SGI0, SGI0, SGI155660, SGI155660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34269[] = {SGI155661, SGI0, SGI155662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34270[] = {SGI155663, SGI0, SGI155664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34271[] = {SGI155665, SGI155666, SGI155667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155668, SGI0, SGI0, SGI0, SGI155669, SGI155669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34272[] = {SGI155670, SGI155671, SGI155672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155673, SGI0, SGI0, SGI0, SGI155674, SGI155674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34273[] = {SGI155675, SGI155676, SGI155677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155678, SGI0, SGI0, SGI0, SGI155679, SGI155679, SGI0, SGI155680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34274[] = {SGI155681, SGI155682, SGI155683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155684, SGI0, SGI0, SGI0, SGI155685, SGI155685, SGI0, SGI155686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34275[] = {SGI155687, SGI155688, SGI155689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155690, SGI0, SGI0, SGI0, SGI155691, SGI155691, SGI0, SGI155692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34276[] = {SGI155693, SGI155694, SGI155695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155696, SGI0, SGI0, SGI0, SGI155697, SGI155697, SGI0, SGI155698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34277[] = {SGI155699, SGI155700, SGI155701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155702, SGI0, SGI0, SGI0, SGI155703, SGI155703, SGI0, SGI155704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34278[] = {SGI155705, SGI155706, SGI155707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155708, SGI155708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34279[] = {SGI155709, SGI155710, SGI155711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155712, SGI0, SGI0, SGI0, SGI155713, SGI155713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34280[] = {SGI155714, SGI0, SGI155715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34281[] = {SGI155716, SGI0, SGI155717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155718, SGI155719, SGI155719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34282[] = {SGI155720, SGI0, SGI155721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155722, SGI155723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34283[] = {SGI155724, SGI0, SGI155725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155726, SGI155727, SGI155727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34284[] = {SGI155728, SGI0, SGI155729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155730, SGI155731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34285[] = {SGI155732, SGI0, SGI155733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155734, SGI155735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34286[] = {SGI155736, SGI0, SGI155737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155738, SGI155739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34287[] = {SGI155740, SGI0, SGI155741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155742, SGI155743, SGI155743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34288[] = {SGI155744, SGI0, SGI155745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155746, SGI155747, SGI155747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34289[] = {SGI155748, SGI0, SGI155749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155750, SGI155751, SGI155751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34290[] = {SGI155752, SGI0, SGI155753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155754, SGI155755, SGI155755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34291[] = {SGI155756, SGI0, SGI155757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155758, SGI155759, SGI155759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34292[] = {SGI155760, SGI0, SGI155761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155762, SGI155763, SGI155763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34293[] = {SGI155764, SGI155765, SGI155766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155767, SGI0, SGI0, SGI0, SGI155768, SGI155768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34294[] = {SGI155769, SGI155770, SGI155771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155772, SGI0, SGI0, SGI0, SGI155773, SGI155773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34295[] = {SGI155774, SGI155775, SGI155776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155777, SGI155777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34296[] = {SGI155778, SGI0, SGI155779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34297[] = {SGI155780, SGI155781, SGI155782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155783, SGI155783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34298[] = {SGI155784, SGI0, SGI155785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155786, SGI155787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34299[] = {SGI155788, SGI0, SGI155789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155791, SGI155793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34300[] = {SGI155790, SGI0, SGI155792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155794, SGI155795, SGI155795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34301[] = {SGI155796, SGI0, SGI155797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155798, SGI155799, SGI155799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34302[] = {SGI155800, SGI0, SGI155801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155802, SGI155803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34303[] = {SGI155804, SGI0, SGI155805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155806, SGI155807, SGI155807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34304[] = {SGI155808, SGI0, SGI155809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155810, SGI155811, SGI155811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34305[] = {SGI155812, SGI0, SGI155813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155814, SGI155815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34306[] = {SGI155816, SGI0, SGI155817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155818, SGI155819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34307[] = {SGI155820, SGI155821, SGI155822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155823, SGI0, SGI0, SGI0, SGI155824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34308[] = {SGI155825, SGI155826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155827, SGI0, SGI0, SGI0, SGI155828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34309[] = {SGI155829, SGI155830, SGI155831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155832, SGI0, SGI0, SGI0, SGI155833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34310[] = {SGI155834, SGI0, SGI155835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155836, SGI155837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34311[] = {SGI155838, SGI0, SGI155839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155840, SGI155841, SGI155841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34312[] = {SGI155842, SGI0, SGI155843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155844, SGI155845, SGI155845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34313[] = {SGI155846, SGI0, SGI155847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155848, SGI155849, SGI155849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34314[] = {SGI155850, SGI0, SGI155851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155852, SGI155853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34315[] = {SGI155854, SGI0, SGI155855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155859, SGI155860, SGI155860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34316[] = {SGI155856, SGI155857, SGI155858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155861, SGI0, SGI0, SGI0, SGI155862, SGI155862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34317[] = {SGI155863, SGI0, SGI155864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34318[] = {SGI155865, SGI155866, SGI155867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155868, SGI0, SGI0, SGI0, SGI155869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34319[] = {SGI155870, SGI0, SGI155871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155872, SGI155873, SGI155873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34320[] = {SGI155874, SGI0, SGI155875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155876, SGI155877, SGI155877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34321[] = {SGI155878, SGI0, SGI155879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155880, SGI155881, SGI155881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34322[] = {SGI155882, SGI0, SGI155883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155884, SGI155885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34323[] = {SGI155886, SGI155887, SGI155888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155889, SGI0, SGI0, SGI0, SGI155890, SGI155890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34324[] = {SGI155891, SGI0, SGI155892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155893, SGI155894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34325[] = {SGI155895, SGI0, SGI155896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155897, SGI155898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34326[] = {SGI155899, SGI0, SGI155900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155901, SGI155902, SGI155902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34327[] = {SGI155903, SGI0, SGI155904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155905, SGI155906, SGI155906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34328[] = {SGI155907, SGI0, SGI155908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34329[] = {SGI155909, SGI0, SGI155910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155911, SGI155912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34330[] = {SGI155913, SGI155914, SGI155915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155916, SGI0, SGI0, SGI0, SGI155917, SGI155917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34331[] = {SGI155918, SGI0, SGI155919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155920, SGI155922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34332[] = {SGI155921, SGI155923, SGI155924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155925, SGI0, SGI0, SGI0, SGI155926, SGI155926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34333[] = {SGI155927, SGI0, SGI155928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155930, SGI155932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34334[] = {SGI155929, SGI155931, SGI155933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155934, SGI0, SGI0, SGI0, SGI155935, SGI155935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34335[] = {SGI155936, SGI155937, SGI155938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155939, SGI0, SGI0, SGI0, SGI155940, SGI155940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34336[] = {SGI155941, SGI155942, SGI155943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155944, SGI0, SGI0, SGI0, SGI155945, SGI155945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34337[] = {SGI155946, SGI0, SGI155947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34338[] = {SGI155948, SGI155949, SGI155950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155951, SGI0, SGI0, SGI0, SGI155952, SGI155952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34339[] = {SGI155953, SGI0, SGI155954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155955, SGI0, SGI0, SGI0, SGI155956, SGI155956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34340[] = {SGI155957, SGI155958, SGI155959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155960, SGI0, SGI0, SGI0, SGI155961, SGI155961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34341[] = {SGI155962, SGI155963, SGI155964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155965, SGI0, SGI0, SGI0, SGI155966, SGI155966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34342[] = {SGI155967, SGI155968, SGI155969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155970, SGI0, SGI0, SGI0, SGI155971, SGI155971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34343[] = {SGI155972, SGI155973, SGI155974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155975, SGI0, SGI0, SGI0, SGI155976, SGI155976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34344[] = {SGI155977, SGI155978, SGI155979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155980, SGI0, SGI0, SGI0, SGI155981, SGI155981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34345[] = {SGI155982, SGI155983, SGI155984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155985, SGI0, SGI0, SGI0, SGI155986, SGI155986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34346[] = {SGI155987, SGI155988, SGI155989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155990, SGI0, SGI0, SGI0, SGI155991, SGI155991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34347[] = {SGI155992, SGI155993, SGI155994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155995, SGI0, SGI0, SGI0, SGI155996, SGI155996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34348[] = {SGI155997, SGI0, SGI155998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34349[] = {SGI155999, SGI0, SGI156000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156001, SGI156002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34350[] = {SGI156003, SGI0, SGI156004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156005, SGI156006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34351[] = {SGI156007, SGI156008, SGI156010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156013, SGI0, SGI0, SGI0, SGI156015, SGI156015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34352[] = {SGI156009, SGI156011, SGI156012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156014, SGI0, SGI0, SGI0, SGI156016, SGI156016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34353[] = {SGI156017, SGI0, SGI156018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34354[] = {SGI156019, SGI0, SGI156020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34355[] = {SGI156021, SGI0, SGI156022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156023, SGI156024, SGI156024, SGI0, SGI0, NULL};
const Segment * const SG34356[] = {SGI156025, SGI156026, SGI156027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156028, SGI0, SGI0, SGI0, SGI156029, SGI156029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34357[] = {SGI156030, SGI156031, SGI156032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156033, SGI0, SGI0, SGI0, SGI156034, SGI156034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34358[] = {SGI156035, SGI0, SGI156036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34359[] = {SGI156037, SGI156038, SGI156039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156040, SGI0, SGI0, SGI0, SGI156041, SGI156041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34360[] = {SGI156042, SGI156043, SGI156044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156045, SGI0, SGI0, SGI0, SGI156046, SGI156046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34361[] = {SGI156047, SGI156048, SGI156049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156050, SGI0, SGI0, SGI0, SGI156051, SGI156051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34362[] = {SGI156052, SGI156053, SGI156054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156055, SGI0, SGI0, SGI0, SGI156056, SGI156056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34363[] = {SGI156057, SGI0, SGI156058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156059, SGI156060, SGI156060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34364[] = {SGI156061, SGI0, SGI156062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156063, SGI156064, SGI156064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34365[] = {SGI156065, SGI0, SGI156066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156067, SGI156068, SGI156068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34366[] = {SGI156069, SGI0, SGI156070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156071, SGI156072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34367[] = {SGI156073, SGI156074, SGI156075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156076, SGI0, SGI0, SGI0, SGI156077, SGI156077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34368[] = {SGI156079, SGI0, SGI156081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156084, SGI156086, SGI156086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34369[] = {SGI156078, SGI156080, SGI156082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156083, SGI0, SGI0, SGI0, SGI156085, SGI156085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34370[] = {SGI156087, SGI0, SGI156088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156091, SGI156092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34371[] = {SGI156089, SGI0, SGI156090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156093, SGI0, SGI0, SGI0, SGI156094, SGI156094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34372[] = {SGI156095, SGI0, SGI156096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34373[] = {SGI156097, SGI0, SGI156098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156099, SGI156100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34374[] = {SGI156101, SGI0, SGI156102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156103, SGI156104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34375[] = {SGI156105, SGI0, SGI156106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156107, SGI156108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34376[] = {SGI156109, SGI0, SGI156110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156111, SGI156112, SGI156112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34377[] = {SGI156113, SGI0, SGI156114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156115, SGI156116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34378[] = {SGI156117, SGI156118, SGI156119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156120, SGI0, SGI0, SGI0, SGI156121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34379[] = {SGI156122, SGI0, SGI156123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34380[] = {SGI156124, SGI0, SGI156125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156126, SGI156127, SGI156127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34381[] = {SGI156128, SGI0, SGI156129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156130, SGI156131, SGI156131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34382[] = {SGI156132, SGI0, SGI156133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156134, SGI156135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34383[] = {SGI156136, SGI0, SGI156137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156138, SGI156139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34384[] = {SGI156140, SGI0, SGI156141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156142, SGI156143, SGI156143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34385[] = {SGI156144, SGI0, SGI156145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156146, SGI156147, SGI156147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34386[] = {SGI156148, SGI0, SGI156149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156150, SGI156151, SGI156151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34387[] = {SGI156152, SGI0, SGI156153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156154, SGI156155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34388[] = {SGI156156, SGI0, SGI156157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156159, SGI156161, SGI156161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34389[] = {SGI156158, SGI0, SGI156160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156162, SGI156163, SGI156163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34390[] = {SGI156164, SGI0, SGI156165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156168, SGI156169, SGI156169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34391[] = {SGI156166, SGI0, SGI156167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156170, SGI156171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34392[] = {SGI156172, SGI0, SGI156173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156174, SGI156175, SGI156175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34393[] = {SGI156176, SGI0, SGI156177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156178, SGI156179, SGI156179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34394[] = {SGI156180, SGI0, SGI156181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156182, SGI156183, SGI156183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34395[] = {SGI156184, SGI0, SGI156185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156186, SGI156187, SGI156187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34396[] = {SGI156188, SGI0, SGI156189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156190, SGI156191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34397[] = {SGI156192, SGI0, SGI156193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156194, SGI156195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34398[] = {SGI156196, SGI0, SGI156197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156198, SGI156199, SGI156199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34399[] = {SGI156200, SGI0, SGI156201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156202, SGI156203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34400[] = {SGI0, SGI0, SGI156204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156205, SGI156206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34401[] = {SGI156207, SGI0, SGI156208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156209, SGI156210, SGI156210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34402[] = {SGI156211, SGI0, SGI156212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156213, SGI156214, SGI156214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34403[] = {SGI156215, SGI0, SGI156216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156217, SGI156218, SGI156218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34404[] = {SGI156219, SGI0, SGI156220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156221, SGI156222, SGI156222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34405[] = {SGI156223, SGI0, SGI156224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156225, SGI156226, SGI156226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34406[] = {SGI156227, SGI0, SGI156228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156229, SGI156230, SGI156230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34407[] = {SGI156231, SGI0, SGI156232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156233, SGI156234, SGI156234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34408[] = {SGI156235, SGI0, SGI156236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156237, SGI156238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34409[] = {SGI156243, SGI0, SGI156244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156245, SGI156246, SGI156246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34410[] = {SGI156247, SGI0, SGI156248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156249, SGI156250, SGI156250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34411[] = {SGI156251, SGI0, SGI156252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156253, SGI156254, SGI156254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34412[] = {SGI156255, SGI156256, SGI156257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156258, SGI0, SGI0, SGI0, SGI156259, SGI156259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34413[] = {SGI156260, SGI156261, SGI156262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156263, SGI0, SGI0, SGI0, SGI156264, SGI156264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34414[] = {SGI156265, SGI156266, SGI156267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156268, SGI0, SGI0, SGI0, SGI156269, SGI156269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34415[] = {SGI156270, SGI156271, SGI156272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156273, SGI0, SGI0, SGI0, SGI156274, SGI156274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34416[] = {SGI156275, SGI156276, SGI156277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156278, SGI0, SGI0, SGI0, SGI156279, SGI156279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34417[] = {SGI156280, SGI156281, SGI156282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156283, SGI0, SGI0, SGI0, SGI156284, SGI156284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34418[] = {SGI156285, SGI156286, SGI156287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156288, SGI0, SGI0, SGI0, SGI156289, SGI156289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34419[] = {SGI156290, SGI156291, SGI156292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156293, SGI0, SGI0, SGI0, SGI156294, SGI156294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34420[] = {SGI156295, SGI0, SGI156296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34421[] = {SGI156297, SGI156298, SGI156299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156300, SGI0, SGI0, SGI0, SGI156301, SGI156301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34422[] = {SGI156302, SGI156303, SGI156304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156305, SGI0, SGI0, SGI0, SGI156306, SGI156306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34423[] = {SGI156307, SGI156308, SGI156309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156310, SGI0, SGI0, SGI0, SGI156311, SGI156311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34424[] = {SGI156312, SGI156313, SGI156314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156315, SGI0, SGI0, SGI0, SGI156316, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34425[] = {SGI156317, SGI156318, SGI156319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156320, SGI0, SGI0, SGI0, SGI156321, SGI156321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34426[] = {SGI156322, SGI156323, SGI156324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156326, SGI0, SGI0, SGI0, SGI156328, SGI156328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34427[] = {SGI156330, SGI156331, SGI156332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156335, SGI0, SGI0, SGI0, SGI156337, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34428[] = {SGI156325, SGI156327, SGI156329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156333, SGI0, SGI0, SGI0, SGI156334, SGI156334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34429[] = {SGI156338, SGI0, SGI156339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34430[] = {SGI156340, SGI156341, SGI156342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156343, SGI0, SGI0, SGI0, SGI156345, SGI156345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34431[] = {SGI156344, SGI156346, SGI156347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156349, SGI0, SGI0, SGI0, SGI156350, SGI156350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34432[] = {SGI156351, SGI0, SGI156352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34433[] = {SGI156353, SGI156354, SGI156355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156356, SGI0, SGI0, SGI0, SGI156359, SGI156359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34434[] = {SGI156357, SGI156358, SGI156360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156362, SGI0, SGI0, SGI0, SGI156363, SGI156363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34435[] = {SGI156364, SGI156365, SGI156366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156369, SGI0, SGI0, SGI0, SGI156371, SGI156371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34436[] = {SGI156367, SGI156368, SGI156370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156372, SGI0, SGI0, SGI0, SGI156374, SGI156374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34437[] = {SGI156373, SGI156375, SGI156376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156378, SGI0, SGI0, SGI0, SGI156379, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34438[] = {SGI156380, SGI156381, SGI156382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156383, SGI0, SGI0, SGI0, SGI156384, SGI156384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34439[] = {SGI156385, SGI0, SGI156386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34440[] = {SGI156387, SGI156388, SGI156389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156390, SGI0, SGI0, SGI0, SGI156391, SGI156391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34441[] = {SGI156392, SGI156393, SGI156394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156395, SGI0, SGI0, SGI0, SGI156396, SGI156396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34442[] = {SGI156397, SGI156398, SGI156399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156400, SGI0, SGI0, SGI0, SGI156401, SGI156401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34443[] = {SGI156402, SGI156403, SGI156404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156405, SGI0, SGI0, SGI0, SGI156406, SGI156406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34444[] = {SGI156407, SGI156408, SGI156409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156410, SGI0, SGI0, SGI0, SGI156411, SGI156411, SGI0, SGI156412, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34445[] = {SGI156413, SGI156414, SGI156415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156416, SGI0, SGI0, SGI0, SGI156417, SGI156417, SGI0, SGI156418, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34446[] = {SGI156419, SGI156420, SGI156421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156422, SGI0, SGI0, SGI0, SGI156423, SGI156423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34447[] = {SGI156424, SGI156425, SGI156426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156427, SGI0, SGI0, SGI0, SGI156428, SGI156428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34448[] = {SGI156429, SGI156430, SGI156431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156432, SGI0, SGI0, SGI0, SGI156433, SGI156433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34449[] = {SGI156434, SGI156435, SGI156436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156437, SGI0, SGI0, SGI0, SGI156438, SGI156438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34450[] = {SGI156439, SGI156440, SGI156441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156442, SGI0, SGI0, SGI0, SGI156443, SGI156443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34451[] = {SGI156239, SGI0, SGI156240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156241, SGI156242, SGI156242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34452[] = {SGI156444, SGI156445, SGI156446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156447, SGI0, SGI0, SGI0, SGI156448, SGI156448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34453[] = {SGI156449, SGI0, SGI156450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156451, SGI156452, SGI156452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34454[] = {SGI156453, SGI0, SGI156454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156455, SGI156456, SGI156456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34455[] = {SGI156457, SGI0, SGI156458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156459, SGI156460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34456[] = {SGI156461, SGI0, SGI156462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156463, SGI156464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34457[] = {SGI156465, SGI0, SGI156466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156467, SGI156468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34458[] = {SGI156469, SGI156470, SGI156471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156472, SGI0, SGI0, SGI0, SGI156473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34459[] = {SGI156474, SGI0, SGI156475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34460[] = {SGI156476, SGI0, SGI156477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156478, SGI156479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34461[] = {SGI156480, SGI156481, SGI156482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156484, SGI0, SGI0, SGI0, SGI156486, SGI156486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34462[] = {SGI156483, SGI0, SGI156485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156487, SGI156488, SGI156488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34463[] = {SGI156489, SGI0, SGI156490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34464[] = {SGI156491, SGI0, SGI156492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156493, SGI156494, SGI156494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34465[] = {SGI156495, SGI156496, SGI156497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156498, SGI0, SGI0, SGI0, SGI156499, SGI156499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34466[] = {SGI156500, SGI156501, SGI156502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156503, SGI0, SGI0, SGI0, SGI156504, SGI156504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34467[] = {SGI156505, SGI156506, SGI156507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156508, SGI0, SGI0, SGI0, SGI156509, SGI156509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34468[] = {SGI156510, SGI156511, SGI156512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156513, SGI0, SGI0, SGI0, SGI156514, SGI156514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34469[] = {SGI156515, SGI156516, SGI156517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156518, SGI0, SGI0, SGI0, SGI156519, SGI156519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34470[] = {SGI156520, SGI156521, SGI156522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156523, SGI0, SGI0, SGI0, SGI156524, SGI156524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34471[] = {SGI156525, SGI156526, SGI156527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156528, SGI0, SGI0, SGI0, SGI156529, SGI156529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34472[] = {SGI156530, SGI156531, SGI156532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156533, SGI0, SGI0, SGI0, SGI156534, SGI156534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34473[] = {SGI156535, SGI156536, SGI156537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156538, SGI0, SGI0, SGI0, SGI156539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34474[] = {SGI156540, SGI0, SGI156541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156542, SGI0, SGI0, SGI0, SGI156543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34475[] = {SGI156544, SGI0, SGI156545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34476[] = {SGI156546, SGI156547, SGI156548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156549, SGI0, SGI0, SGI0, SGI156550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34477[] = {SGI156551, SGI0, SGI156552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156553, SGI156554, SGI156554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34478[] = {SGI156555, SGI156556, SGI156557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156558, SGI0, SGI0, SGI0, SGI156559, SGI156559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34479[] = {SGI156560, SGI156561, SGI156562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156563, SGI0, SGI0, SGI0, SGI156564, SGI156564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34480[] = {SGI156565, SGI0, SGI156566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34481[] = {SGI156567, SGI156568, SGI156569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156570, SGI0, SGI0, SGI0, SGI156571, SGI156571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34482[] = {SGI156572, SGI0, SGI156573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34483[] = {SGI156574, SGI156575, SGI156576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156577, SGI0, SGI0, SGI0, SGI156578, SGI156578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34484[] = {SGI156579, SGI0, SGI156580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34485[] = {SGI156581, SGI0, SGI156582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156583, SGI156584, SGI156584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34486[] = {SGI156585, SGI156586, SGI156587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156588, SGI0, SGI0, SGI0, SGI156589, SGI156589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34487[] = {SGI156590, SGI156591, SGI156592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156593, SGI0, SGI0, SGI0, SGI156595, SGI156595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34488[] = {SGI156594, SGI0, SGI156596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156597, SGI156598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34489[] = {SGI156599, SGI156600, SGI156601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156603, SGI0, SGI0, SGI0, SGI156605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34490[] = {SGI156602, SGI0, SGI156604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156606, SGI156607, SGI156607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34491[] = {SGI156608, SGI156609, SGI156610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156611, SGI0, SGI0, SGI0, SGI156612, SGI156612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34492[] = {SGI156613, SGI0, SGI156614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156615, SGI156616, SGI156616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34493[] = {SGI156617, SGI156618, SGI156619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156620, SGI0, SGI0, SGI0, SGI156621, SGI156621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34494[] = {SGI156622, SGI156623, SGI156624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156625, SGI0, SGI0, SGI0, SGI156626, SGI156626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34495[] = {SGI156627, SGI0, SGI156628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34496[] = {SGI156629, SGI0, SGI156630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34497[] = {SGI156631, SGI156632, SGI156633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156634, SGI0, SGI0, SGI0, SGI156635, SGI156635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34498[] = {SGI156636, SGI156637, SGI156638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156639, SGI0, SGI0, SGI0, SGI156640, SGI156640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34499[] = {SGI156641, SGI156642, SGI156643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156644, SGI0, SGI0, SGI0, SGI156645, SGI156645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34500[] = {SGI156646, SGI156647, SGI156648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156649, SGI0, SGI0, SGI0, SGI156650, SGI156650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34501[] = {SGI156651, SGI156652, SGI156653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156654, SGI0, SGI0, SGI0, SGI156655, SGI156655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34502[] = {SGI156656, SGI156657, SGI156658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156659, SGI0, SGI0, SGI0, SGI156660, SGI156660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34503[] = {SGI156661, SGI156662, SGI156663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156664, SGI0, SGI0, SGI0, SGI156665, SGI156665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34504[] = {SGI156666, SGI156667, SGI156668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156669, SGI0, SGI0, SGI0, SGI156670, SGI156670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34505[] = {SGI156671, SGI156672, SGI156673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156674, SGI0, SGI0, SGI0, SGI156675, SGI156675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34506[] = {SGI156676, SGI156677, SGI156678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156679, SGI0, SGI0, SGI0, SGI156680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34507[] = {SGI156681, SGI0, SGI156682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34508[] = {SGI156683, SGI0, SGI156684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156685, SGI156686, SGI156686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34509[] = {SGI156687, SGI0, SGI156688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156689, SGI156690, SGI156690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34510[] = {SGI156691, SGI0, SGI156692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156693, SGI156694, SGI156694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34511[] = {SGI156695, SGI0, SGI156696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156697, SGI156698, SGI156698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34512[] = {SGI156699, SGI156700, SGI156701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156702, SGI0, SGI0, SGI0, SGI156703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34513[] = {SGI156704, SGI156705, SGI156706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156707, SGI0, SGI0, SGI0, SGI156708, SGI156708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34514[] = {SGI156709, SGI156710, SGI156711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156712, SGI0, SGI0, SGI0, SGI156713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34515[] = {SGI156714, SGI156715, SGI156716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156717, SGI0, SGI0, SGI0, SGI156718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34516[] = {SGI156719, SGI0, SGI156720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156721, SGI0, SGI0, SGI0, SGI156722, SGI156722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34517[] = {SGI156723, SGI0, SGI156724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156725, SGI156726, SGI156726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34518[] = {SGI156727, SGI156728, SGI156729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156730, SGI0, SGI0, SGI0, SGI156731, SGI156731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34519[] = {SGI156733, SGI156734, SGI156735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156736, SGI0, SGI0, SGI0, SGI156737, SGI156737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34520[] = {SGI156738, SGI156739, SGI156740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156741, SGI0, SGI0, SGI0, SGI156742, SGI156742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34521[] = {SGI156744, SGI156745, SGI156746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156747, SGI0, SGI0, SGI0, SGI156748, SGI156748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34522[] = {SGI156751, SGI156752, SGI156754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156756, SGI0, SGI0, SGI0, SGI156758, SGI156758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34523[] = {SGI156750, SGI156753, SGI156755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156757, SGI0, SGI0, SGI0, SGI156759, SGI156759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34524[] = {SGI156761, SGI156762, SGI156763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156764, SGI0, SGI0, SGI0, SGI156765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34525[] = {SGI156767, SGI0, SGI156768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156769, SGI156770, SGI156770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34526[] = {SGI156771, SGI0, SGI156772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156773, SGI156774, SGI156774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34527[] = {SGI156775, SGI0, SGI156776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156777, SGI156778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34528[] = {SGI156779, SGI0, SGI156780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156781, SGI156782, SGI156782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34529[] = {SGI156783, SGI156784, SGI156785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156786, SGI0, SGI0, SGI0, SGI156787, SGI156787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34530[] = {SGI156788, SGI0, SGI156789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156790, SGI156791, SGI156791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156794, SGI0, SGI0, NULL};
const Segment * const SG34531[] = {SGI156792, SGI0, SGI156793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34532[] = {SGI156795, SGI156796, SGI156797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156799, SGI0, SGI0, SGI0, SGI156801, SGI156801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34533[] = {SGI156798, SGI0, SGI156800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156802, SGI156803, SGI156803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156804, SGI0, SGI0, NULL};
const Segment * const SG34534[] = {SGI156805, SGI156806, SGI156807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156808, SGI0, SGI0, SGI0, SGI156809, SGI156809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34535[] = {SGI156810, SGI0, SGI156811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156812, SGI156813, SGI156813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156814, SGI0, SGI0, NULL};
const Segment * const SG34536[] = {SGI156815, SGI0, SGI156816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156817, SGI156818, SGI156818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156819, SGI0, SGI0, NULL};
const Segment * const SG34537[] = {SGI156820, SGI0, SGI156821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156822, SGI156823, SGI156823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34538[] = {SGI156824, SGI0, SGI156826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156828, SGI156830, SGI156830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34539[] = {SGI156825, SGI0, SGI156827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156829, SGI156831, SGI156831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34540[] = {SGI156832, SGI0, SGI156833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156834, SGI156835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34541[] = {SGI156836, SGI0, SGI156837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156838, SGI156839, SGI156839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34542[] = {SGI156840, SGI0, SGI156841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156842, SGI156843, SGI156843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34543[] = {SGI156844, SGI156845, SGI156846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156847, SGI0, SGI0, SGI0, SGI156848, SGI156848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34544[] = {SGI156849, SGI156850, SGI156851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156852, SGI0, SGI0, SGI0, SGI156853, SGI156853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34545[] = {SGI156854, SGI156855, SGI156856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156857, SGI0, SGI0, SGI0, SGI156858, SGI156858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34546[] = {SGI156859, SGI156860, SGI156861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156862, SGI0, SGI0, SGI0, SGI156863, SGI156863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34547[] = {SGI156864, SGI156865, SGI156866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156867, SGI0, SGI0, SGI0, SGI156868, SGI156868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34548[] = {SGI156869, SGI156870, SGI156871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156872, SGI0, SGI0, SGI0, SGI156873, SGI156873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34549[] = {SGI156874, SGI156875, SGI156876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156877, SGI0, SGI0, SGI0, SGI156878, SGI156878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34550[] = {SGI156879, SGI156880, SGI156881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156882, SGI0, SGI0, SGI0, SGI156883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34551[] = {SGI156884, SGI156885, SGI156886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156887, SGI0, SGI0, SGI0, SGI156888, SGI156888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34552[] = {SGI156889, SGI156890, SGI156891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156892, SGI0, SGI0, SGI0, SGI156893, SGI156893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34553[] = {SGI156894, SGI156895, SGI156896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156897, SGI0, SGI0, SGI0, SGI156898, SGI156898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34554[] = {SGI156899, SGI156900, SGI156901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156902, SGI0, SGI0, SGI0, SGI156903, SGI156903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34555[] = {SGI0, SGI156904, SGI156905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156906, SGI0, SGI0, SGI0, SGI156907, SGI156907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34556[] = {SGI156912, SGI0, SGI156913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34557[] = {SGI156908, SGI156909, SGI156910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156915, SGI0, SGI0, SGI0, SGI156916, SGI156916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34558[] = {SGI156911, SGI0, SGI156914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156917, SGI156918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34559[] = {SGI156919, SGI0, SGI156920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34560[] = {SGI156921, SGI0, SGI156922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34561[] = {SGI156923, SGI156924, SGI156925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156926, SGI0, SGI0, SGI0, SGI156929, SGI156929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34562[] = {SGI156927, SGI156928, SGI156930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156931, SGI0, SGI0, SGI0, SGI156932, SGI156932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34563[] = {SGI156933, SGI156934, SGI156935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156938, SGI0, SGI0, SGI0, SGI156940, SGI156940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34564[] = {SGI156936, SGI156937, SGI156939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156941, SGI0, SGI0, SGI0, SGI156942, SGI156942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34565[] = {SGI156943, SGI156944, SGI156945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156946, SGI0, SGI0, SGI0, SGI156947, SGI156947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34566[] = {SGI156948, SGI0, SGI156950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156953, SGI0, SGI0, SGI0, SGI156955, SGI156955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34567[] = {SGI156949, SGI156951, SGI156952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156954, SGI0, SGI0, SGI0, SGI156956, SGI156956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34568[] = {SGI156957, SGI0, SGI156958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34569[] = {SGI156959, SGI156960, SGI156961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156962, SGI0, SGI0, SGI0, SGI156963, SGI156963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34570[] = {SGI156964, SGI156965, SGI156966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156967, SGI0, SGI0, SGI0, SGI156968, SGI156968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34571[] = {SGI156969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34572[] = {SGI156970, SGI156971, SGI156972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156973, SGI0, SGI0, SGI0, SGI156974, SGI156974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34573[] = {SGI156975, SGI0, SGI156976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156977, SGI156978, SGI156978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34574[] = {SGI156979, SGI0, SGI156980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156981, SGI156982, SGI156982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34575[] = {SGI156983, SGI0, SGI156984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156985, SGI156986, SGI156986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34576[] = {SGI156987, SGI156988, SGI156989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156990, SGI0, SGI0, SGI0, SGI156991, SGI156991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34577[] = {SGI156992, SGI0, SGI156993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34578[] = {SGI0, SGI0, SGI0, SGI156999, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34579[] = {SGI156994, SGI156995, SGI156996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI156997, SGI0, SGI0, SGI0, SGI156998, SGI156998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34580[] = {SGI157000, SGI157001, SGI157002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157003, SGI0, SGI0, SGI0, SGI157006, SGI157006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34581[] = {SGI157004, SGI0, SGI157005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34582[] = {SGI157007, SGI157008, SGI157009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157012, SGI0, SGI0, SGI0, SGI157013, SGI157013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34583[] = {SGI157010, SGI0, SGI157011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157014, SGI157015, SGI157015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34584[] = {SGI157016, SGI157018, SGI157019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157021, SGI0, SGI0, SGI0, SGI157023, SGI157023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34585[] = {SGI157017, SGI0, SGI157020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157022, SGI157024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34586[] = {SGI157025, SGI157026, SGI157027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157030, SGI0, SGI0, SGI0, SGI157031, SGI157031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34587[] = {SGI157028, SGI0, SGI157029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157032, SGI157033, SGI157033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34588[] = {SGI157034, SGI157035, SGI157036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157037, SGI0, SGI0, SGI0, SGI157038, SGI157038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34589[] = {SGI157039, SGI0, SGI157040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157041, SGI157042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34590[] = {SGI157043, SGI0, SGI157044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34591[] = {SGI157045, SGI0, SGI157046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157049, SGI157051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34592[] = {SGI157047, SGI157048, SGI157050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157052, SGI0, SGI0, SGI0, SGI157053, SGI157053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34593[] = {SGI157054, SGI157055, SGI157056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157057, SGI0, SGI0, SGI0, SGI157058, SGI157058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34594[] = {SGI157059, SGI0, SGI157060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34595[] = {SGI157061, SGI0, SGI157062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157063, SGI157064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34596[] = {SGI157065, SGI157066, SGI157067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157068, SGI0, SGI0, SGI0, SGI157069, SGI157069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34597[] = {SGI157070, SGI157071, SGI157072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157073, SGI0, SGI0, SGI0, SGI157074, SGI157074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34598[] = {SGI157075, SGI157076, SGI157077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157078, SGI0, SGI0, SGI0, SGI157079, SGI157079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34599[] = {SGI157080, SGI0, SGI157081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34600[] = {SGI157082, SGI157083, SGI157084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157085, SGI0, SGI0, SGI0, SGI157086, SGI157086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34601[] = {SGI157087, SGI0, SGI157088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34602[] = {SGI157089, SGI157090, SGI157091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157094, SGI0, SGI0, SGI0, SGI157095, SGI157095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34603[] = {SGI157092, SGI0, SGI157093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157096, SGI157097, SGI157097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34604[] = {SGI157098, SGI0, SGI157099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157100, SGI157101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34605[] = {SGI157102, SGI0, SGI157103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157104, SGI157105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34606[] = {SGI157106, SGI0, SGI157107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157108, SGI157109, SGI157109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157110, SGI0, SGI0, NULL};
const Segment * const SG34607[] = {SGI157111, SGI157112, SGI157113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157114, SGI0, SGI0, SGI0, SGI157115, SGI157115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34608[] = {SGI157116, SGI157117, SGI157118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157119, SGI0, SGI0, SGI0, SGI157120, SGI157120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34609[] = {SGI157121, SGI0, SGI157122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157123, SGI157124, SGI157124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157125, SGI0, SGI0, NULL};
const Segment * const SG34610[] = {SGI0, SGI157126, SGI157127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157128, SGI0, SGI0, SGI0, SGI157129, SGI157129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34611[] = {SGI157130, SGI157131, SGI157132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157133, SGI0, SGI0, SGI0, SGI157134, SGI157134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34612[] = {SGI157135, SGI0, SGI157136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34613[] = {SGI157137, SGI0, SGI157138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157139, SGI157140, SGI157140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34614[] = {SGI157141, SGI0, SGI157142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157143, SGI157144, SGI157144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34615[] = {SGI157145, SGI0, SGI157146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157149, SGI157150, SGI157150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34616[] = {SGI157147, SGI0, SGI157148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157151, SGI157152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34617[] = {SGI157153, SGI0, SGI157154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157155, SGI157156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34618[] = {SGI157157, SGI0, SGI157158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157159, SGI157160, SGI157160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34619[] = {SGI157161, SGI0, SGI157162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34620[] = {SGI157164, SGI157166, SGI157168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157170, SGI0, SGI0, SGI0, SGI157172, SGI157172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34621[] = {SGI157163, SGI157165, SGI157167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157169, SGI0, SGI0, SGI0, SGI157171, SGI157171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34622[] = {SGI157174, SGI157175, SGI157176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157180, SGI0, SGI0, SGI0, SGI157181, SGI157181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34623[] = {SGI157177, SGI157178, SGI157179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157182, SGI0, SGI0, SGI0, SGI157183, SGI157183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34624[] = {SGI157185, SGI157186, SGI157187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157191, SGI0, SGI0, SGI0, SGI157192, SGI157192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34625[] = {SGI157188, SGI157189, SGI157190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157193, SGI0, SGI0, SGI0, SGI157194, SGI157194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34626[] = {SGI157196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34627[] = {SGI157197, SGI157198, SGI157199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157200, SGI0, SGI0, SGI0, SGI157201, SGI157201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34628[] = {SGI157203, SGI0, SGI157204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157205, SGI157206, SGI157206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34629[] = {SGI157207, SGI0, SGI157208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157209, SGI157210, SGI157210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34630[] = {SGI157211, SGI0, SGI157212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157213, SGI157214, SGI157214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34631[] = {SGI157215, SGI0, SGI157216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157218, SGI157219, SGI157219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34632[] = {SGI157217, SGI0, SGI157220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157221, SGI157222, SGI157222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34633[] = {SGI157223, SGI0, SGI157224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157225, SGI157226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34634[] = {SGI157227, SGI157228, SGI157229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157230, SGI0, SGI0, SGI0, SGI157231, SGI157231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34635[] = {SGI157232, SGI157233, SGI157234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157235, SGI157235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34636[] = {SGI157236, SGI0, SGI157237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157238, SGI157239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34637[] = {SGI157240, SGI0, SGI157241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34638[] = {SGI157242, SGI157243, SGI157244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157245, SGI0, SGI0, SGI0, SGI157246, SGI157246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34639[] = {SGI157247, SGI157248, SGI157249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157250, SGI0, SGI0, SGI0, SGI157251, SGI157251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34640[] = {SGI157252, SGI0, SGI157253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34641[] = {SGI157254, SGI0, SGI157255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157256, SGI157257, SGI157257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34642[] = {SGI157258, SGI0, SGI157259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157260, SGI157261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34643[] = {SGI157262, SGI0, SGI157263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157264, SGI157265, SGI157265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34644[] = {SGI157266, SGI0, SGI157267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157268, SGI157269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34645[] = {SGI157270, SGI0, SGI157271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157272, SGI157273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34646[] = {SGI157274, SGI157275, SGI157276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157277, SGI0, SGI0, SGI0, SGI157278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34647[] = {SGI157279, SGI157280, SGI157281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157282, SGI0, SGI0, SGI0, SGI157283, SGI157283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34648[] = {SGI157284, SGI0, SGI157285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34649[] = {SGI157286, SGI0, SGI157287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157288, SGI157289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34650[] = {SGI157290, SGI0, SGI157291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157292, SGI157293, SGI157293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34651[] = {SGI157294, SGI0, SGI157295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157296, SGI157297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34652[] = {SGI157298, SGI0, SGI157299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157300, SGI157301, SGI157301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157302, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34653[] = {SGI157303, SGI0, SGI157304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157305, SGI157306, SGI157306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157307, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34654[] = {SGI157308, SGI0, SGI157309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157310, SGI157311, SGI157311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157312, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34655[] = {SGI157313, SGI0, SGI157314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157315, SGI157316, SGI157316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34656[] = {SGI157317, SGI0, SGI157318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157319, SGI157320, SGI157320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34657[] = {SGI157321, SGI0, SGI157322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157323, SGI157324, SGI157324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34658[] = {SGI157325, SGI157326, SGI157327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157328, SGI0, SGI0, SGI0, SGI157329, SGI157329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34659[] = {SGI157330, SGI157331, SGI157332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157333, SGI0, SGI0, SGI0, SGI157334, SGI157334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34660[] = {SGI157335, SGI157336, SGI157337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157338, SGI0, SGI0, SGI0, SGI157339, SGI157339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34661[] = {SGI157340, SGI0, SGI157341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34662[] = {SGI157342, SGI0, SGI157343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157344, SGI0, SGI0, SGI0, SGI157345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34663[] = {SGI157346, SGI157347, SGI157348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157349, SGI0, SGI0, SGI0, SGI157350, SGI157350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34664[] = {SGI157351, SGI0, SGI157352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157353, SGI0, SGI0, SGI0, SGI157354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34665[] = {SGI157355, SGI0, SGI157356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157357, SGI157358, SGI157358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34666[] = {SGI157359, SGI0, SGI157360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157361, SGI157362, SGI157362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34667[] = {SGI157363, SGI0, SGI157364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157365, SGI157366, SGI157366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34668[] = {SGI157367, SGI157368, SGI157369, SGI157370, SGI157371, SGI157371, SGI0, NULL};
const Segment * const SG34669[] = {SGI157372, SGI0, SGI157373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157374, SGI157375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34670[] = {SGI157376, SGI0, SGI157377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157378, SGI157379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34671[] = {SGI157380, SGI0, SGI157381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157382, SGI157383, SGI157383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34672[] = {SGI157384, SGI0, SGI157385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157386, SGI157387, SGI157387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34673[] = {SGI157388, SGI0, SGI157389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157390, SGI157391, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34674[] = {SGI157392, SGI0, SGI157393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157394, SGI157395, SGI157395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34675[] = {SGI157396, SGI157397, SGI157398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157399, SGI0, SGI0, SGI0, SGI157400, SGI157400, SGI0, SGI157401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34676[] = {SGI157402, SGI157403, SGI157404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157405, SGI0, SGI0, SGI0, SGI157406, SGI157406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157407, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34677[] = {SGI157408, SGI157409, SGI157410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157411, SGI0, SGI0, SGI0, SGI157412, SGI157412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157413, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34678[] = {SGI157414, SGI157415, SGI157416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157420, SGI0, SGI0, SGI0, SGI157421, SGI157421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157423, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34679[] = {SGI157417, SGI157418, SGI157419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157422, SGI0, SGI0, SGI0, SGI157424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34680[] = {SGI157428, SGI0, SGI157429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34681[] = {SGI157425, SGI157426, SGI157427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157430, SGI0, SGI0, SGI0, SGI157431, SGI157431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157432, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34682[] = {SGI157433, SGI157434, SGI157435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157436, SGI0, SGI0, SGI0, SGI157437, SGI157437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157438, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34683[] = {SGI157439, SGI157440, SGI157441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157442, SGI0, SGI0, SGI0, SGI157443, SGI157443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157444, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34684[] = {SGI157445, SGI157446, SGI157447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157448, SGI0, SGI0, SGI0, SGI157449, SGI157449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157450, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34685[] = {SGI157451, SGI157452, SGI157453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157454, SGI0, SGI0, SGI0, SGI157455, SGI157455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157456, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34686[] = {SGI157457, SGI157458, SGI157459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157461, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34687[] = {SGI157462, SGI157463, SGI157464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157465, SGI0, SGI0, SGI0, SGI157466, SGI157466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157467, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34688[] = {SGI157468, SGI157469, SGI157470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157472, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34689[] = {SGI157473, SGI157474, SGI157475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157476, SGI0, SGI0, SGI0, SGI157477, SGI157477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157478, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34690[] = {SGI157479, SGI157480, SGI157481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157482, SGI0, SGI0, SGI0, SGI157483, SGI157483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34691[] = {SGI157484, SGI157485, SGI157486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157487, SGI0, SGI0, SGI0, SGI157488, SGI157488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34692[] = {SGI157489, SGI0, SGI157490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157491, SGI157492, SGI157492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34693[] = {SGI157493, SGI0, SGI157494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34694[] = {SGI157495, SGI0, SGI157496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157497, SGI0, SGI0, SGI0, SGI157498, SGI157498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34695[] = {SGI157499, SGI157500, SGI157501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157502, SGI0, SGI0, SGI0, SGI157503, SGI157503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34696[] = {SGI157504, SGI0, SGI157505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157506, SGI157507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34697[] = {SGI157508, SGI157509, SGI157510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157511, SGI0, SGI0, SGI0, SGI157512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34698[] = {SGI157513, SGI157514, SGI157515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157516, SGI0, SGI0, SGI0, SGI157517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34699[] = {SGI157518, SGI157519, SGI157520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157521, SGI0, SGI0, SGI0, SGI157522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34700[] = {SGI157523, SGI157524, SGI157525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157526, SGI0, SGI0, SGI0, SGI157527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34701[] = {SGI157528, SGI157529, SGI157530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157531, SGI0, SGI0, SGI0, SGI157532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34702[] = {SGI157533, SGI157534, SGI157535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157536, SGI0, SGI0, SGI0, SGI157537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34703[] = {SGI157540, SGI157541, SGI157542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157543, SGI0, SGI0, SGI0, SGI157544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34704[] = {SGI157545, SGI0, SGI157546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157547, SGI157548, SGI157548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34705[] = {SGI157549, SGI0, SGI157550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157551, SGI157552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34706[] = {SGI157553, SGI0, SGI157554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157555, SGI157556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34707[] = {SGI157557, SGI0, SGI157558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157559, SGI157560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34708[] = {SGI157561, SGI0, SGI157562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157563, SGI157564, SGI157564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34709[] = {SGI157565, SGI0, SGI157566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157567, SGI157568, SGI157568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34710[] = {SGI157569, SGI0, SGI157570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157571, SGI157572, SGI157572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34711[] = {SGI157573, SGI0, SGI157574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157575, SGI157576, SGI157576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34712[] = {SGI157577, SGI0, SGI157578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157579, SGI157580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34713[] = {SGI157581, SGI0, SGI157582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157583, SGI157584, SGI157584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34714[] = {SGI157585, SGI0, SGI157586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157587, SGI157588, SGI157588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34715[] = {SGI157589, SGI0, SGI157591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157593, SGI157594, SGI157594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34716[] = {SGI157590, SGI0, SGI157592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157595, SGI157596, SGI157596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34717[] = {SGI157597, SGI0, SGI157598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157599, SGI157600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34718[] = {SGI157538, SGI0, SGI157539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157601, SGI157602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34719[] = {SGI157603, SGI0, SGI157604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157605, SGI157606, SGI157606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34720[] = {SGI157607, SGI0, SGI157608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157609, SGI157610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34721[] = {SGI157611, SGI0, SGI157612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157613, SGI157614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34722[] = {SGI157615, SGI0, SGI157616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157617, SGI157618, SGI157618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34723[] = {SGI157619, SGI0, SGI157620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157621, SGI157622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34724[] = {SGI157623, SGI157624, SGI157625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157626, SGI0, SGI0, SGI0, SGI157627, SGI157627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34725[] = {SGI157628, SGI0, SGI157629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34726[] = {SGI157630, SGI157631, SGI157632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157633, SGI0, SGI0, SGI0, SGI157634, SGI157634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34727[] = {SGI157635, SGI157636, SGI157637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157638, SGI0, SGI0, SGI0, SGI157639, SGI157639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34728[] = {SGI157640, SGI157641, SGI157642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157643, SGI0, SGI0, SGI0, SGI157644, SGI157644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34729[] = {SGI157645, SGI157646, SGI157647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157648, SGI0, SGI0, SGI0, SGI157649, SGI157649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34730[] = {SGI157650, SGI157651, SGI157652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157653, SGI0, SGI0, SGI0, SGI157654, SGI157654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34731[] = {SGI157655, SGI0, SGI157656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157659, SGI157661, SGI157661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34732[] = {SGI157657, SGI157658, SGI157660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157662, SGI0, SGI0, SGI0, SGI157663, SGI157663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34733[] = {SGI157664, SGI0, SGI157665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157667, SGI157669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34734[] = {SGI157666, SGI157668, SGI157670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157671, SGI0, SGI0, SGI0, SGI157672, SGI157672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34735[] = {SGI157673, SGI0, SGI157674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34736[] = {SGI157675, SGI0, SGI157676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157678, SGI157679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34737[] = {SGI157677, SGI0, SGI157680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157681, SGI157682, SGI157682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34738[] = {SGI157683, SGI0, SGI157684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157685, SGI157686, SGI157686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34739[] = {SGI157687, SGI0, SGI157688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157689, SGI157690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34740[] = {SGI157691, SGI0, SGI157692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157693, SGI157694, SGI157694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34741[] = {SGI157695, SGI0, SGI157696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157697, SGI157698, SGI157698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34742[] = {SGI157699, SGI0, SGI157700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157701, SGI157702, SGI157702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34743[] = {SGI157704, SGI157705, SGI157707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157708, SGI0, SGI0, SGI0, SGI157711, SGI157711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34744[] = {SGI157703, SGI0, SGI157706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157709, SGI157710, SGI157710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34745[] = {SGI157712, SGI0, SGI157714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34746[] = {SGI157713, SGI0, SGI157715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157716, SGI157717, SGI157717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34747[] = {SGI157718, SGI157719, SGI157720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157723, SGI0, SGI0, SGI0, SGI157724, SGI157724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34748[] = {SGI157721, SGI0, SGI157722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157725, SGI157726, SGI157726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34749[] = {SGI157727, SGI157728, SGI157729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157732, SGI0, SGI0, SGI0, SGI157733, SGI157733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34750[] = {SGI157730, SGI0, SGI157731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157734, SGI157735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34751[] = {SGI157736, SGI0, SGI157737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34752[] = {SGI157738, SGI0, SGI157739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157740, SGI157741, SGI157741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34753[] = {SGI157742, SGI157743, SGI157744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157745, SGI0, SGI0, SGI0, SGI157746, SGI157746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34754[] = {SGI157747, SGI0, SGI157748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157749, SGI157750, SGI157750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34755[] = {SGI157751, SGI157752, SGI157753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157754, SGI0, SGI0, SGI0, SGI157755, SGI157755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34756[] = {SGI157756, SGI0, SGI157757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157758, SGI157759, SGI157759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34757[] = {SGI157760, SGI157761, SGI157762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157763, SGI0, SGI0, SGI0, SGI157764, SGI157764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34758[] = {SGI157765, SGI0, SGI157766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157767, SGI157768, SGI157768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34759[] = {SGI157769, SGI157770, SGI157771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157772, SGI0, SGI0, SGI0, SGI157773, SGI157773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34760[] = {SGI157774, SGI0, SGI157775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157776, SGI157777, SGI157777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34761[] = {SGI157778, SGI157779, SGI157780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157781, SGI0, SGI0, SGI0, SGI157782, SGI157782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34762[] = {SGI157783, SGI0, SGI157784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157785, SGI157786, SGI157786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34763[] = {SGI157787, SGI0, SGI157788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34764[] = {SGI157789, SGI0, SGI157790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157791, SGI157792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34765[] = {SGI157793, SGI0, SGI157794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157795, SGI0, SGI0, SGI0, SGI157796, SGI157796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34766[] = {SGI157797, SGI0, SGI157798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157799, SGI157800, SGI157800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34767[] = {SGI157801, SGI157802, SGI157803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157804, SGI0, SGI0, SGI0, SGI157805, SGI157805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34768[] = {SGI157806, SGI0, SGI157807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157808, SGI157809, SGI157809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34769[] = {SGI157810, SGI0, SGI157811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157812, SGI157813, SGI157813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34770[] = {SGI157814, SGI0, SGI157815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157816, SGI157817, SGI157817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34771[] = {SGI157818, SGI0, SGI157819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157820, SGI157821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34772[] = {SGI157822, SGI0, SGI157823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157824, SGI157825, SGI157825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34773[] = {SGI157826, SGI0, SGI157827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157828, SGI157829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34774[] = {SGI157830, SGI0, SGI157831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157832, SGI157833, SGI157833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34775[] = {SGI0, SGI0, SGI157834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157835, SGI157836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34776[] = {SGI157837, SGI0, SGI157838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157839, SGI157840, SGI157840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34777[] = {SGI157841, SGI0, SGI157842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157843, SGI157844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34778[] = {SGI157845, SGI0, SGI157846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157847, SGI157848, SGI157848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34779[] = {SGI157849, SGI0, SGI157850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157851, SGI157852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34780[] = {SGI157853, SGI0, SGI157854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157855, SGI157856, SGI157856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34781[] = {SGI157857, SGI0, SGI157858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157859, SGI157860, SGI157860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34782[] = {SGI157861, SGI0, SGI157862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157863, SGI157864, SGI157864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34783[] = {SGI157865, SGI0, SGI157866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157867, SGI157868, SGI157868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34784[] = {SGI157869, SGI0, SGI157870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157871, SGI157872, SGI157872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34785[] = {SGI157873, SGI0, SGI157874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157875, SGI157876, SGI157876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34786[] = {SGI157877, SGI0, SGI157878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157879, SGI157880, SGI157880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34787[] = {SGI157881, SGI157882, SGI157883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157884, SGI0, SGI0, SGI0, SGI157885, SGI157885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34788[] = {SGI157886, SGI0, SGI157887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157888, SGI157889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34789[] = {SGI157890, SGI157891, SGI157893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157895, SGI0, SGI0, SGI0, SGI157896, SGI157896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34790[] = {SGI157892, SGI0, SGI157894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157897, SGI157898, SGI157898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34791[] = {SGI157899, SGI0, SGI157900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157901, SGI157902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34792[] = {SGI157903, SGI0, SGI157904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157905, SGI157906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34793[] = {SGI157907, SGI0, SGI157908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157909, SGI157910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34794[] = {SGI157911, SGI0, SGI157912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157913, SGI157914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34795[] = {SGI157915, SGI0, SGI157916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157917, SGI157918, SGI157918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34796[] = {SGI157919, SGI0, SGI157920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157921, SGI157922, SGI157922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34797[] = {SGI157923, SGI0, SGI157924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157925, SGI157926, SGI157926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34798[] = {SGI157927, SGI0, SGI157928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157929, SGI157930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34799[] = {SGI157931, SGI0, SGI157932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157934, SGI157936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34800[] = {SGI157933, SGI0, SGI157935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157937, SGI157938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34801[] = {SGI157939, SGI0, SGI157940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157941, SGI157942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34802[] = {SGI157943, SGI157944, SGI157945, SGI157946, SGI157947, SGI157947, SGI0, NULL};
const Segment * const SG34803[] = {SGI157948, SGI0, SGI157949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157950, SGI157951, SGI157951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34804[] = {SGI157952, SGI0, SGI157953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157957, SGI157958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34805[] = {SGI157954, SGI0, SGI157955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157960, SGI157961, SGI157961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34806[] = {SGI157956, SGI0, SGI157959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157962, SGI157963, SGI157963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34807[] = {SGI157964, SGI0, SGI157965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157967, SGI157969, SGI157969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34808[] = {SGI157966, SGI0, SGI157968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157970, SGI157971, SGI157971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34809[] = {SGI157972, SGI0, SGI157973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157976, SGI157977, SGI157977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34810[] = {SGI157974, SGI0, SGI157975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157978, SGI157979, SGI157979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34811[] = {SGI157980, SGI0, SGI157981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157982, SGI157983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34812[] = {SGI157984, SGI0, SGI157985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157986, SGI157987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34813[] = {SGI157988, SGI0, SGI157989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157990, SGI157991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34814[] = {SGI157992, SGI0, SGI157993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157994, SGI157995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34815[] = {SGI157996, SGI0, SGI157997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI157998, SGI157999, SGI157999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34816[] = {SGI158000, SGI0, SGI158001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158002, SGI158003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34817[] = {SGI158004, SGI0, SGI158005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158006, SGI158007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34818[] = {SGI158008, SGI0, SGI158009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158010, SGI158011, SGI158011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34819[] = {SGI158012, SGI0, SGI158013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158014, SGI158015, SGI158015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34820[] = {SGI158016, SGI0, SGI158017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158018, SGI158019, SGI158019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34821[] = {SGI158020, SGI158021, SGI158022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158023, SGI0, SGI0, SGI0, SGI158024, SGI158024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34822[] = {SGI158025, SGI158026, SGI158027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158028, SGI0, SGI0, SGI0, SGI158029, SGI158029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34823[] = {SGI158030, SGI0, SGI158031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34824[] = {SGI158032, SGI158033, SGI158034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158035, SGI0, SGI0, SGI0, SGI158036, SGI158036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34825[] = {SGI158037, SGI158038, SGI158039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158040, SGI0, SGI0, SGI0, SGI158041, SGI158041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34826[] = {SGI158042, SGI158043, SGI158044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158045, SGI0, SGI0, SGI0, SGI158046, SGI158046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34827[] = {SGI158047, SGI158048, SGI158049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158050, SGI0, SGI0, SGI0, SGI158051, SGI158051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34828[] = {SGI158052, SGI158053, SGI158054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158055, SGI0, SGI0, SGI0, SGI158056, SGI158056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34829[] = {SGI158057, SGI0, SGI158058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34830[] = {SGI158059, SGI0, SGI158060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158061, SGI158062, SGI158062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34831[] = {SGI158063, SGI0, SGI158064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158065, SGI158066, SGI158066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34832[] = {SGI158067, SGI0, SGI158068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158069, SGI158070, SGI158070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34833[] = {SGI158071, SGI0, SGI158072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158073, SGI158074, SGI158074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34834[] = {SGI158075, SGI0, SGI158076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158079, SGI158080, SGI158080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158082, SGI0, SGI0, NULL};
const Segment * const SG34835[] = {SGI158077, SGI0, SGI158078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158081, SGI158083, SGI158083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34836[] = {SGI158084, SGI0, SGI158085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158086, SGI158087, SGI158087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158088, SGI0, SGI0, NULL};
const Segment * const SG34837[] = {SGI158089, SGI0, SGI158090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158091, SGI158092, SGI158092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158093, SGI0, SGI0, NULL};
const Segment * const SG34838[] = {SGI158094, SGI0, SGI158095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158096, SGI158097, SGI158097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158098, SGI0, SGI0, NULL};
const Segment * const SG34839[] = {SGI158099, SGI0, SGI158100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158101, SGI158102, SGI158102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158103, SGI0, SGI0, NULL};
const Segment * const SG34840[] = {SGI158104, SGI0, SGI158105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158106, SGI158107, SGI158107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34841[] = {SGI158108, SGI0, SGI158109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158111, SGI158112, SGI158112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34842[] = {SGI158110, SGI0, SGI158113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158114, SGI158115, SGI158115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34843[] = {SGI158116, SGI158117, SGI158118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158119, SGI0, SGI0, SGI0, SGI158120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34844[] = {SGI158121, SGI158122, SGI158123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158124, SGI0, SGI0, SGI0, SGI158125, SGI158125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34845[] = {SGI158126, SGI0, SGI158127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34846[] = {SGI158128, SGI158129, SGI158130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158131, SGI0, SGI0, SGI0, SGI158132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34847[] = {SGI158133, SGI0, SGI158134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158136, SGI158137, SGI158137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34848[] = {SGI158135, SGI0, SGI158138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158139, SGI158140, SGI158140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34849[] = {SGI158141, SGI0, SGI158142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158143, SGI158144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34850[] = {SGI158145, SGI0, SGI158146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158147, SGI158148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34851[] = {SGI158149, SGI0, SGI158150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158151, SGI158152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34852[] = {SGI158153, SGI0, SGI158154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158155, SGI158156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34853[] = {SGI158157, SGI0, SGI158158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158162, SGI158163, SGI158163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34854[] = {SGI158159, SGI158160, SGI158161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158164, SGI158164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34855[] = {SGI158165, SGI0, SGI158166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158170, SGI158171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34856[] = {SGI158167, SGI158168, SGI158169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158172, SGI0, SGI0, SGI0, SGI158173, SGI158173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34857[] = {SGI158174, SGI158175, SGI158176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158178, SGI158178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34858[] = {SGI158177, SGI0, SGI158179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158180, SGI158181, SGI158181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34859[] = {SGI158182, SGI158183, SGI158184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158185, SGI0, SGI0, SGI0, SGI158186, SGI158186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34860[] = {SGI158187, SGI0, SGI158188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158189, SGI158190, SGI158190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34861[] = {SGI0, SGI158191, SGI158192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34862[] = {SGI0, SGI158193, SGI158194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34863[] = {SGI158195, SGI158196, SGI158197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158198, SGI158199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34864[] = {SGI158200, SGI0, SGI158201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34865[] = {SGI0, SGI0, SGI158202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34866[] = {SGI158203, SGI158204, SGI158205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158206, SGI0, SGI0, SGI0, SGI158207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34867[] = {SGI158208, SGI0, SGI158209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34868[] = {SGI158210, SGI158211, SGI158212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158213, SGI0, SGI0, SGI0, SGI158214, SGI158214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34869[] = {SGI158216, SGI0, SGI158218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158224, SGI158225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34870[] = {SGI158215, SGI158217, SGI158219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158223, SGI0, SGI0, SGI0, SGI158227, SGI158227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34871[] = {SGI158220, SGI158221, SGI158222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158226, SGI0, SGI0, SGI0, SGI158228, SGI158228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34872[] = {SGI158229, SGI158230, SGI158231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158233, SGI0, SGI0, SGI0, SGI158235, SGI158235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34873[] = {SGI158232, SGI0, SGI158234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158236, SGI158237, SGI158237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34874[] = {SGI158238, SGI158239, SGI158240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158241, SGI0, SGI0, SGI0, SGI158242, SGI158242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34875[] = {SGI158243, SGI0, SGI158244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34876[] = {SGI158245, SGI0, SGI158246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158247, SGI158248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34877[] = {SGI158249, SGI0, SGI158250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158251, SGI158252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34878[] = {SGI158253, SGI158254, SGI158255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158256, SGI0, SGI0, SGI0, SGI158257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34879[] = {SGI158258, SGI0, SGI158259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34880[] = {SGI158260, SGI158261, SGI158262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158263, SGI0, SGI0, SGI0, SGI158264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34881[] = {SGI158265, SGI158266, SGI158267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158268, SGI0, SGI0, SGI0, SGI158269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34882[] = {SGI158270, SGI158272, SGI158273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158275, SGI0, SGI0, SGI0, SGI158277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34883[] = {SGI158271, SGI0, SGI158274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158276, SGI158278, SGI158278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34884[] = {SGI158279, SGI0, SGI158280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158281, SGI158282, SGI158282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34885[] = {SGI158283, SGI0, SGI158284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158285, SGI158286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34886[] = {SGI158287, SGI0, SGI158288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158289, SGI158290, SGI158290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34887[] = {SGI158291, SGI0, SGI158292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158293, SGI158294, SGI158294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34888[] = {SGI158295, SGI0, SGI158296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158297, SGI158298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34889[] = {SGI158299, SGI0, SGI158300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158301, SGI158302, SGI158302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34890[] = {SGI158303, SGI0, SGI158304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158305, SGI158306, SGI158306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34891[] = {SGI158307, SGI0, SGI158308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158309, SGI158310, SGI158310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34892[] = {SGI158311, SGI158312, SGI158313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158316, SGI0, SGI0, SGI0, SGI158317, SGI158317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34893[] = {SGI158314, SGI0, SGI158315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158318, SGI158319, SGI158319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34894[] = {SGI158320, SGI158321, SGI158322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158324, SGI0, SGI0, SGI0, SGI158326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34895[] = {SGI158323, SGI0, SGI158325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158327, SGI158328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34896[] = {SGI158329, SGI0, SGI158330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158331, SGI158332, SGI158332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158333, SGI0, SGI0, NULL};
const Segment * const SG34897[] = {SGI158334, SGI0, SGI158335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158336, SGI158337, SGI158337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158339, SGI0, SGI0, NULL};
const Segment * const SG34898[] = {SGI158338, SGI158340, SGI158341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158342, SGI0, SGI0, SGI0, SGI158343, SGI158343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34899[] = {SGI158344, SGI0, SGI158345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158346, SGI158347, SGI158347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158350, SGI0, SGI0, NULL};
const Segment * const SG34900[] = {SGI158348, SGI158349, SGI158351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158352, SGI0, SGI0, SGI0, SGI158353, SGI158353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34901[] = {SGI158354, SGI0, SGI158355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158356, SGI158357, SGI158357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158358, SGI0, SGI0, NULL};
const Segment * const SG34902[] = {SGI158359, SGI0, SGI158360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158361, SGI158362, SGI158362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158363, SGI0, SGI0, NULL};
const Segment * const SG34903[] = {SGI158364, SGI158365, SGI158366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158367, SGI0, SGI0, SGI0, SGI158368, SGI158368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34904[] = {SGI158369, SGI158370, SGI158371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158372, SGI0, SGI0, SGI0, SGI158373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34905[] = {SGI158374, SGI0, SGI158375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158376, SGI158377, SGI158377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34906[] = {SGI158378, SGI0, SGI158379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34907[] = {SGI158380, SGI0, SGI158381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158382, SGI158383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34908[] = {SGI158384, SGI158385, SGI158386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158387, SGI0, SGI0, SGI0, SGI158388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34909[] = {SGI158389, SGI158390, SGI158391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158392, SGI0, SGI0, SGI0, SGI158393, SGI158393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34910[] = {SGI158394, SGI158395, SGI158396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158398, SGI0, SGI0, SGI0, SGI158401, SGI158401, SGI0, SGI0, NULL};
const Segment * const SG34911[] = {SGI158397, SGI158399, SGI158400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158402, SGI0, SGI0, SGI0, SGI158403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34912[] = {SGI158404, SGI158405, SGI158406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158407, SGI0, SGI0, SGI0, SGI158409, SGI158409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34913[] = {SGI158408, SGI158410, SGI158411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158412, SGI0, SGI0, SGI0, SGI158413, SGI158413, SGI0, SGI0, NULL};
const Segment * const SG34914[] = {SGI158414, SGI158415, SGI158416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158417, SGI0, SGI0, SGI0, SGI158418, SGI158418, SGI0, SGI0, NULL};
const Segment * const SG34915[] = {SGI158419, SGI158420, SGI158421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158422, SGI0, SGI0, SGI0, SGI158423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34916[] = {SGI158424, SGI158425, SGI158426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158427, SGI0, SGI0, SGI0, SGI158428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34917[] = {SGI0, SGI0, SGI158429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158430, SGI158430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34918[] = {SGI158431, SGI158432, SGI158433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158434, SGI0, SGI0, SGI0, SGI158435, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34919[] = {SGI158436, SGI158437, SGI158438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158439, SGI0, SGI0, SGI0, SGI158440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34920[] = {SGI158441, SGI158442, SGI158443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158444, SGI0, SGI0, SGI0, SGI158445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34921[] = {SGI158446, SGI0, SGI158447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158448, SGI158449, SGI158449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34922[] = {SGI158450, SGI158451, SGI158452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158453, SGI0, SGI0, SGI0, SGI158454, SGI158454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34923[] = {SGI158455, SGI0, SGI158456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158457, SGI158458, SGI158458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34924[] = {SGI158459, SGI158460, SGI158461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158462, SGI0, SGI0, SGI0, SGI158463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34925[] = {SGI158464, SGI0, SGI158465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158466, SGI158467, SGI158467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34926[] = {SGI158468, SGI0, SGI158469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158470, SGI158471, SGI158471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34927[] = {SGI158472, SGI158473, SGI158474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158475, SGI0, SGI0, SGI0, SGI158476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34928[] = {SGI158477, SGI158478, SGI158479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158480, SGI0, SGI0, SGI0, SGI158481, SGI158481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34929[] = {SGI158482, SGI0, SGI158483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158484, SGI158485, SGI158485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34930[] = {SGI158486, SGI158487, SGI158488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158489, SGI0, SGI0, SGI0, SGI158490, SGI158490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34931[] = {SGI158491, SGI0, SGI158492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158493, SGI158494, SGI158494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34932[] = {SGI158495, SGI158496, SGI158497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158498, SGI0, SGI0, SGI0, SGI158499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34933[] = {SGI158500, SGI0, SGI158501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158502, SGI158503, SGI158503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34934[] = {SGI158504, SGI158505, SGI158506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158507, SGI0, SGI0, SGI0, SGI158508, SGI158508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34935[] = {SGI158509, SGI158510, SGI158511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158513, SGI0, SGI0, SGI0, SGI158516, SGI158516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34936[] = {SGI158512, SGI158514, SGI158515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158517, SGI0, SGI0, SGI0, SGI158518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34937[] = {SGI158519, SGI0, SGI158520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34938[] = {SGI158521, SGI158522, SGI158523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158524, SGI0, SGI0, SGI0, SGI158525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34939[] = {SGI158526, SGI158527, SGI158528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158529, SGI158529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34940[] = {SGI158530, SGI158531, SGI158532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158533, SGI0, SGI0, SGI0, SGI158534, SGI158534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34941[] = {SGI158535, SGI0, SGI158537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158541, SGI158542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34942[] = {SGI158536, SGI158538, SGI158539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158540, SGI0, SGI0, SGI0, SGI158543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34943[] = {SGI158544, SGI158545, SGI158546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158547, SGI0, SGI0, SGI0, SGI158549, SGI158549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34944[] = {SGI158548, SGI0, SGI158550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158551, SGI158552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34945[] = {SGI158553, SGI158554, SGI158555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158556, SGI0, SGI0, SGI0, SGI158557, SGI158557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34946[] = {SGI158558, SGI158559, SGI158561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158563, SGI0, SGI0, SGI0, SGI158564, SGI158564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34947[] = {SGI158560, SGI0, SGI158562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158565, SGI158566, SGI158566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34948[] = {SGI0, SGI158567, SGI158568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158569, SGI0, SGI0, SGI0, SGI158570, SGI158570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34949[] = {SGI158571, SGI0, SGI158572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158573, SGI158575, SGI158575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34950[] = {SGI158574, SGI158576, SGI158577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158578, SGI0, SGI0, SGI0, SGI158579, SGI158579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34951[] = {SGI158580, SGI0, SGI158581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158582, SGI158584, SGI158584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34952[] = {SGI158583, SGI158585, SGI158586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158587, SGI0, SGI0, SGI0, SGI158588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34953[] = {SGI0, SGI158589, SGI158590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158591, SGI0, SGI0, SGI0, SGI158592, SGI158592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34954[] = {SGI158593, SGI158594, SGI158595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158596, SGI0, SGI0, SGI0, SGI158597, SGI158597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34955[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34956[] = {SGI158599, SGI158600, SGI158601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158602, SGI0, SGI0, SGI0, SGI158603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34957[] = {SGI158604, SGI0, SGI158605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158606, SGI158607, SGI158607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34958[] = {SGI158608, SGI158609, SGI158610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158611, SGI0, SGI0, SGI0, SGI158612, SGI158612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34959[] = {SGI158613, SGI0, SGI158614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158615, SGI158616, SGI158616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34960[] = {SGI158617, SGI158618, SGI158619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158620, SGI0, SGI0, SGI0, SGI158621, SGI158621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34961[] = {SGI158622, SGI158623, SGI158624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158625, SGI0, SGI0, SGI0, SGI158626, SGI158626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34962[] = {SGI158627, SGI158628, SGI158629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158630, SGI0, SGI0, SGI0, SGI158631, SGI158631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34963[] = {SGI158632, SGI158633, SGI158634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158635, SGI0, SGI0, SGI0, SGI158636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34964[] = {SGI158637, SGI0, SGI158638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34965[] = {SGI158639, SGI158640, SGI158641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158642, SGI0, SGI0, SGI0, SGI158643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34966[] = {SGI158645, SGI158647, SGI158648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158649, SGI0, SGI0, SGI0, SGI158652, SGI158652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34967[] = {SGI158644, SGI0, SGI158646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158650, SGI158651, SGI158651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34968[] = {SGI158653, SGI158654, SGI158655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158656, SGI0, SGI0, SGI0, SGI158657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34969[] = {SGI158658, SGI158659, SGI158660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158661, SGI0, SGI0, SGI0, SGI158662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34970[] = {SGI158663, SGI158664, SGI158665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158666, SGI0, SGI0, SGI0, SGI158667, SGI158667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34971[] = {SGI158668, SGI158669, SGI158670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158671, SGI0, SGI0, SGI0, SGI158672, SGI158672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34972[] = {SGI158673, SGI158674, SGI158675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158676, SGI0, SGI0, SGI0, SGI158677, SGI158677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34973[] = {SGI158678, SGI158679, SGI158680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158681, SGI0, SGI0, SGI0, SGI158682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34974[] = {SGI158683, SGI158684, SGI158685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158686, SGI0, SGI0, SGI0, SGI158687, SGI158687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34975[] = {SGI158688, SGI158689, SGI158690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158691, SGI0, SGI0, SGI0, SGI158692, SGI158692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34976[] = {SGI158693, SGI158694, SGI158695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158696, SGI0, SGI0, SGI0, SGI158697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34977[] = {SGI158698, SGI158699, SGI158700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158701, SGI0, SGI0, SGI0, SGI158702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34978[] = {SGI158703, SGI158704, SGI158705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158706, SGI0, SGI0, SGI0, SGI158707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34979[] = {SGI158708, SGI158709, SGI158710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158713, SGI0, SGI0, SGI0, SGI158714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34980[] = {SGI158711, SGI0, SGI158712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34981[] = {SGI158715, SGI158716, SGI158717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158718, SGI0, SGI0, SGI0, SGI158719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34982[] = {SGI158720, SGI158721, SGI158722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158723, SGI0, SGI0, SGI0, SGI158724, SGI158724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34983[] = {SGI158725, SGI158726, SGI158727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158728, SGI0, SGI0, SGI0, SGI158729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34984[] = {SGI158730, SGI158731, SGI158732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158733, SGI0, SGI0, SGI0, SGI158734, SGI158734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34985[] = {SGI158736, SGI158737, SGI158738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158739, SGI0, SGI0, SGI0, SGI158740, SGI158740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34986[] = {SGI158741, SGI158742, SGI158743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158744, SGI0, SGI0, SGI0, SGI158745, SGI158745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34987[] = {SGI158747, SGI158748, SGI158749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158753, SGI0, SGI0, SGI0, SGI158754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34988[] = {SGI158750, SGI158751, SGI158752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158755, SGI0, SGI0, SGI0, SGI158756, SGI158756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34989[] = {SGI158758, SGI158759, SGI158760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158764, SGI0, SGI0, SGI0, SGI158765, SGI158765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34990[] = {SGI158761, SGI158762, SGI158763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158766, SGI0, SGI0, SGI0, SGI158768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34991[] = {SGI158770, SGI158772, SGI158774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158775, SGI0, SGI0, SGI0, SGI158777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34992[] = {SGI158769, SGI158771, SGI158773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158776, SGI0, SGI0, SGI0, SGI158778, SGI158778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34993[] = {SGI158780, SGI158781, SGI158782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158785, SGI0, SGI0, SGI0, SGI158787, SGI158787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34994[] = {SGI158783, SGI158784, SGI158786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158788, SGI0, SGI0, SGI0, SGI158789, SGI158789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34995[] = {SGI158792, SGI158794, SGI158796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158797, SGI0, SGI0, SGI0, SGI158799, SGI158799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34996[] = {SGI158791, SGI158793, SGI158795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158798, SGI0, SGI0, SGI0, SGI158800, SGI158800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34997[] = {SGI158802, SGI158803, SGI158805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158808, SGI0, SGI0, SGI0, SGI158809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34998[] = {SGI158804, SGI158806, SGI158807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158810, SGI158810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG34999[] = {SGI158811, SGI158812, SGI158813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158814, SGI0, SGI0, SGI0, SGI158816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35000[] = {SGI158815, SGI0, SGI158817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158820, SGI158822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35001[] = {SGI158818, SGI158819, SGI158821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158823, SGI0, SGI0, SGI0, SGI158824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35002[] = {SGI158825, SGI0, SGI158826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35003[] = {SGI158827, SGI158828, SGI158829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158830, SGI0, SGI0, SGI0, SGI158831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35004[] = {SGI158832, SGI158833, SGI158834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158835, SGI0, SGI0, SGI0, SGI158836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35005[] = {SGI158837, SGI158838, SGI158839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158840, SGI0, SGI0, SGI0, SGI158841, SGI158841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35006[] = {SGI158842, SGI158843, SGI158844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158845, SGI0, SGI0, SGI0, SGI158846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35007[] = {SGI158847, SGI158848, SGI158849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158850, SGI0, SGI0, SGI0, SGI158852, SGI158852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35008[] = {SGI158851, SGI158853, SGI158854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158855, SGI0, SGI0, SGI0, SGI158856, SGI158856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35009[] = {SGI158857, SGI158858, SGI158859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158862, SGI0, SGI0, SGI0, SGI158864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35010[] = {SGI158860, SGI158861, SGI158863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158865, SGI0, SGI0, SGI0, SGI158867, SGI158867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35011[] = {SGI158866, SGI158868, SGI158869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158870, SGI0, SGI0, SGI0, SGI158871, SGI158871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35012[] = {SGI158872, SGI158873, SGI158874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158875, SGI0, SGI0, SGI0, SGI158877, SGI158877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35013[] = {SGI158876, SGI158879, SGI158881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158883, SGI0, SGI0, SGI0, SGI158885, SGI158885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35014[] = {SGI158878, SGI158880, SGI158882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158884, SGI0, SGI0, SGI0, SGI158886, SGI158886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35015[] = {SGI158887, SGI0, SGI158888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35016[] = {SGI158889, SGI0, SGI158890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35017[] = {SGI158891, SGI158892, SGI158893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158894, SGI0, SGI0, SGI0, SGI158895, SGI158895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35018[] = {SGI158896, SGI0, SGI158897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35019[] = {SGI158898, SGI158899, SGI158900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158901, SGI0, SGI0, SGI0, SGI158902, SGI158902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35020[] = {SGI158903, SGI158904, SGI158905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158906, SGI0, SGI0, SGI0, SGI158907, SGI158907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35021[] = {SGI158908, SGI158909, SGI158910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158911, SGI0, SGI0, SGI0, SGI158912, SGI158912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35022[] = {SGI158913, SGI0, SGI158914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35023[] = {SGI158915, SGI158916, SGI158917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158918, SGI0, SGI0, SGI0, SGI158920, SGI158920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35024[] = {SGI0, SGI158919, SGI158921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158922, SGI0, SGI0, SGI0, SGI158923, SGI158923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35025[] = {SGI158924, SGI158925, SGI158926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158927, SGI0, SGI0, SGI0, SGI158928, SGI158928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35026[] = {SGI158929, SGI158930, SGI158931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158932, SGI0, SGI0, SGI0, SGI158933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35027[] = {SGI158934, SGI158935, SGI158936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158937, SGI0, SGI0, SGI0, SGI158938, SGI158938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35028[] = {SGI158939, SGI158940, SGI158941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158942, SGI0, SGI0, SGI0, SGI158943, SGI158943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35029[] = {SGI158944, SGI158945, SGI158946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158947, SGI0, SGI0, SGI0, SGI158948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35030[] = {SGI158949, SGI158950, SGI158951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158952, SGI0, SGI0, SGI0, SGI158953, SGI158953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35031[] = {SGI158954, SGI0, SGI158955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158956, SGI158957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35032[] = {SGI158958, SGI158959, SGI158960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158961, SGI0, SGI0, SGI0, SGI158962, SGI158962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35033[] = {SGI158963, SGI158964, SGI158965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158966, SGI0, SGI0, SGI0, SGI158967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35034[] = {SGI158968, SGI158969, SGI158970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158971, SGI0, SGI0, SGI0, SGI158972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35035[] = {SGI158973, SGI158974, SGI158975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158976, SGI0, SGI0, SGI0, SGI158977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35036[] = {SGI158978, SGI158979, SGI158980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158982, SGI0, SGI0, SGI0, SGI158984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35037[] = {SGI158981, SGI0, SGI158983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158985, SGI158986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35038[] = {SGI158987, SGI158988, SGI158989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158990, SGI0, SGI0, SGI0, SGI158992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35039[] = {SGI158991, SGI0, SGI158993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158994, SGI158995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35040[] = {SGI158996, SGI158997, SGI158998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI158999, SGI0, SGI0, SGI0, SGI159000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35041[] = {SGI159001, SGI159002, SGI159003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159006, SGI0, SGI0, SGI0, SGI159007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35042[] = {SGI159004, SGI0, SGI159005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159008, SGI159009, SGI159009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35043[] = {SGI159010, SGI159011, SGI159012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159015, SGI0, SGI0, SGI0, SGI159016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35044[] = {SGI159013, SGI0, SGI159014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159017, SGI159018, SGI159018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35045[] = {SGI159019, SGI159020, SGI159021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159022, SGI0, SGI0, SGI0, SGI159023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35046[] = {SGI159024, SGI159025, SGI159026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159027, SGI0, SGI0, SGI0, SGI159028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35047[] = {SGI159029, SGI159030, SGI159031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159032, SGI0, SGI0, SGI0, SGI159033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35048[] = {SGI159034, SGI159035, SGI159036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159037, SGI0, SGI0, SGI0, SGI159038, SGI159038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35049[] = {SGI159039, SGI159040, SGI159041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159042, SGI0, SGI0, SGI0, SGI159043, SGI159043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35050[] = {SGI159044, SGI159045, SGI159046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159047, SGI0, SGI0, SGI0, SGI159048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35051[] = {SGI159049, SGI159050, SGI159051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159052, SGI0, SGI0, SGI0, SGI159053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35052[] = {SGI159054, SGI159055, SGI159056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159057, SGI0, SGI0, SGI0, SGI159058, SGI159058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35053[] = {SGI159059, SGI159060, SGI159061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159062, SGI0, SGI0, SGI0, SGI159063, SGI159063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35054[] = {SGI159064, SGI159065, SGI159066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159067, SGI0, SGI0, SGI0, SGI159068, SGI159068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35055[] = {SGI159069, SGI159070, SGI159071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159072, SGI0, SGI0, SGI0, SGI159073, SGI159073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35056[] = {SGI159074, SGI159075, SGI159076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159077, SGI0, SGI0, SGI0, SGI159078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35057[] = {SGI159079, SGI159080, SGI159081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159082, SGI0, SGI0, SGI0, SGI159083, SGI159083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35058[] = {SGI159085, SGI0, SGI159087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35059[] = {SGI159084, SGI159086, SGI159088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159089, SGI0, SGI0, SGI0, SGI159090, SGI159090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35060[] = {SGI159091, SGI159092, SGI159093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159094, SGI0, SGI0, SGI0, SGI159095, SGI159095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35061[] = {SGI159096, SGI159097, SGI159098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159099, SGI0, SGI0, SGI0, SGI159100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35062[] = {SGI159101, SGI159102, SGI159103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159104, SGI0, SGI0, SGI0, SGI159105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35063[] = {SGI159106, SGI159107, SGI159108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159109, SGI0, SGI0, SGI0, SGI159110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35064[] = {SGI159111, SGI0, SGI159112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35065[] = {SGI159113, SGI159114, SGI159115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159116, SGI0, SGI0, SGI0, SGI159117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35066[] = {SGI159118, SGI159119, SGI159120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159121, SGI0, SGI0, SGI0, SGI159122, SGI159122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35067[] = {SGI159123, SGI159124, SGI159125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159126, SGI0, SGI0, SGI0, SGI159127, SGI159127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35068[] = {SGI159128, SGI0, SGI159129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159133, SGI159134, SGI159134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35069[] = {SGI159130, SGI159131, SGI159132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159135, SGI0, SGI0, SGI0, SGI159138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35070[] = {SGI159136, SGI0, SGI159137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35071[] = {SGI159139, SGI159140, SGI159141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159142, SGI0, SGI0, SGI0, SGI159143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35072[] = {SGI159144, SGI159145, SGI159146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159147, SGI0, SGI0, SGI0, SGI159148, SGI159148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35073[] = {SGI159149, SGI159150, SGI159151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159152, SGI0, SGI0, SGI0, SGI159153, SGI159153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35074[] = {SGI0, SGI159154, SGI159155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159156, SGI0, SGI0, SGI0, SGI159157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35075[] = {SGI159158, SGI159159, SGI159160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159161, SGI0, SGI0, SGI0, SGI159162, SGI159162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35076[] = {SGI159163, SGI159164, SGI159165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159166, SGI0, SGI0, SGI0, SGI159168, SGI159168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35077[] = {SGI159167, SGI159169, SGI159170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159171, SGI0, SGI0, SGI0, SGI159172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35078[] = {SGI159174, SGI159175, SGI159176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159180, SGI0, SGI0, SGI0, SGI159181, SGI159181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35079[] = {SGI159177, SGI159178, SGI159179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159182, SGI0, SGI0, SGI0, SGI159183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35080[] = {SGI159184, SGI159185, SGI159186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159189, SGI0, SGI0, SGI0, SGI159191, SGI159191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35081[] = {SGI159187, SGI159188, SGI159190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159192, SGI0, SGI0, SGI0, SGI159193, SGI159193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35082[] = {SGI159194, SGI0, SGI159195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35083[] = {SGI159196, SGI159197, SGI159198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159199, SGI0, SGI0, SGI0, SGI159200, SGI159200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35084[] = {SGI159201, SGI159202, SGI159203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159204, SGI0, SGI0, SGI0, SGI159205, SGI159205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35085[] = {SGI159206, SGI159207, SGI159208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159209, SGI0, SGI0, SGI0, SGI159210, SGI159210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35086[] = {SGI159211, SGI159212, SGI159213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159214, SGI0, SGI0, SGI0, SGI159215, SGI159215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35087[] = {SGI159216, SGI159217, SGI159218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159219, SGI0, SGI0, SGI0, SGI159220, SGI159220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35088[] = {SGI159221, SGI159222, SGI159223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159224, SGI0, SGI0, SGI0, SGI159225, SGI159225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35089[] = {SGI159226, SGI159227, SGI159228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159229, SGI0, SGI0, SGI0, SGI159230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35090[] = {SGI159231, SGI159232, SGI159233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159234, SGI0, SGI0, SGI0, SGI159235, SGI159235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35091[] = {SGI159236, SGI159237, SGI159238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159239, SGI0, SGI0, SGI0, SGI159240, SGI159240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35092[] = {SGI159241, SGI159242, SGI159243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159244, SGI0, SGI0, SGI0, SGI159245, SGI159245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35093[] = {SGI159246, SGI159247, SGI159248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159249, SGI0, SGI0, SGI0, SGI159250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35094[] = {SGI159251, SGI159252, SGI159253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159254, SGI0, SGI0, SGI0, SGI159255, SGI159255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35095[] = {SGI159256, SGI159257, SGI159258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159259, SGI0, SGI0, SGI0, SGI159260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35096[] = {SGI159261, SGI159262, SGI159263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159264, SGI0, SGI0, SGI0, SGI159265, SGI159265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35097[] = {SGI159266, SGI159267, SGI159268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159269, SGI0, SGI0, SGI0, SGI159270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35098[] = {SGI159271, SGI159272, SGI159273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159275, SGI0, SGI0, SGI0, SGI159277, SGI159277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35099[] = {SGI159274, SGI0, SGI159276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159281, SGI159282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35100[] = {SGI159278, SGI159279, SGI159280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159283, SGI0, SGI0, SGI0, SGI159284, SGI159284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35101[] = {SGI159285, SGI0, SGI159286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35102[] = {SGI159287, SGI159288, SGI159289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159290, SGI0, SGI0, SGI0, SGI159291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35103[] = {SGI159292, SGI0, SGI159293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159296, SGI159297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35104[] = {SGI159294, SGI0, SGI159295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159298, SGI159299, SGI159299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35105[] = {SGI159300, SGI159301, SGI159302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159303, SGI0, SGI0, SGI0, SGI159304, SGI159304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35106[] = {SGI159306, SGI0, SGI159308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159311, SGI159312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35107[] = {SGI159305, SGI159307, SGI159309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159310, SGI0, SGI0, SGI0, SGI159313, SGI159313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35108[] = {SGI159314, SGI159315, SGI159316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159317, SGI0, SGI0, SGI0, SGI159318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35109[] = {SGI159319, SGI0, SGI159321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159325, SGI159326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35110[] = {SGI159320, SGI159322, SGI159323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159324, SGI0, SGI0, SGI0, SGI159327, SGI159327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35111[] = {SGI159328, SGI0, SGI159329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159333, SGI159334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35112[] = {SGI159330, SGI159331, SGI159332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159335, SGI0, SGI0, SGI0, SGI159336, SGI159336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35113[] = {SGI159337, SGI0, SGI159338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159339, SGI159340, SGI159340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35114[] = {SGI159341, SGI159342, SGI159343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159344, SGI0, SGI0, SGI0, SGI159345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35115[] = {SGI159346, SGI159347, SGI159348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159349, SGI0, SGI0, SGI0, SGI159350, SGI159350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35116[] = {SGI159351, SGI159352, SGI159353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159354, SGI0, SGI0, SGI0, SGI159355, SGI159355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35117[] = {SGI0, SGI159356, SGI159357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159358, SGI0, SGI0, SGI0, SGI159359, SGI159359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35118[] = {SGI159360, SGI159361, SGI159362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159363, SGI0, SGI0, SGI0, SGI159364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35119[] = {SGI159365, SGI159366, SGI159367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159368, SGI0, SGI0, SGI0, SGI159369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35120[] = {SGI159370, SGI159371, SGI159372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159373, SGI0, SGI0, SGI0, SGI159374, SGI159374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35121[] = {SGI159375, SGI159376, SGI159377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159378, SGI0, SGI0, SGI0, SGI159379, SGI159379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35122[] = {SGI159380, SGI159381, SGI159382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159383, SGI0, SGI0, SGI0, SGI159384, SGI159384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35123[] = {SGI159385, SGI159386, SGI159387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159388, SGI0, SGI0, SGI0, SGI159389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35124[] = {SGI159390, SGI159392, SGI159394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159396, SGI0, SGI0, SGI0, SGI159398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35125[] = {SGI159391, SGI159393, SGI159395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159397, SGI0, SGI0, SGI0, SGI159399, SGI159399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35126[] = {SGI159400, SGI159401, SGI159402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159403, SGI0, SGI0, SGI0, SGI159404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35127[] = {SGI159405, SGI0, SGI159406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159407, SGI159408, SGI159408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35128[] = {SGI159409, SGI0, SGI159410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159411, SGI159412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35129[] = {SGI159413, SGI0, SGI159414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159415, SGI159416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35130[] = {SGI159417, SGI0, SGI159418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159419, SGI159420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35131[] = {SGI159421, SGI0, SGI159422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159423, SGI159424, SGI159424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35132[] = {SGI159425, SGI159426, SGI159427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159428, SGI0, SGI0, SGI0, SGI159429, SGI159429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35133[] = {SGI159430, SGI159431, SGI159432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159433, SGI0, SGI0, SGI0, SGI159434, SGI159434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35134[] = {SGI159435, SGI159436, SGI159437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159438, SGI0, SGI0, SGI0, SGI159439, SGI159439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35135[] = {SGI159440, SGI159441, SGI159442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159443, SGI0, SGI0, SGI0, SGI159444, SGI159444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35136[] = {SGI159445, SGI159446, SGI159447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159448, SGI0, SGI0, SGI0, SGI159449, SGI159449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35137[] = {SGI159450, SGI0, SGI159451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35138[] = {SGI159452, SGI0, SGI159453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159454, SGI159455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35139[] = {SGI159456, SGI159457, SGI159458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159459, SGI0, SGI0, SGI0, SGI159460, SGI159460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35140[] = {SGI159461, SGI0, SGI159462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159463, SGI159464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35141[] = {SGI159465, SGI159466, SGI159467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159468, SGI0, SGI0, SGI0, SGI159469, SGI159469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35142[] = {SGI159470, SGI0, SGI159471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35143[] = {SGI159472, SGI159473, SGI159474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159475, SGI0, SGI0, SGI0, SGI159476, SGI159476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35144[] = {SGI159477, SGI0, SGI159478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159479, SGI159480, SGI159480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35145[] = {SGI159481, SGI0, SGI159482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159483, SGI0, SGI0, SGI0, SGI159484, SGI159484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35146[] = {SGI159485, SGI0, SGI159487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159489, SGI159490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35147[] = {SGI159486, SGI0, SGI159488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159491, SGI159492, SGI159492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35148[] = {SGI159493, SGI0, SGI159494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159495, SGI159496, SGI159496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35149[] = {SGI159497, SGI0, SGI159498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159501, SGI159502, SGI159502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35150[] = {SGI159499, SGI0, SGI159500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159503, SGI159504, SGI159504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35151[] = {SGI159505, SGI0, SGI159506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159509, SGI159510, SGI159510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35152[] = {SGI159507, SGI0, SGI159508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159511, SGI159512, SGI159512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35153[] = {SGI159513, SGI0, SGI159514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159516, SGI159517, SGI159517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35154[] = {SGI159515, SGI0, SGI159518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159519, SGI159520, SGI159520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35155[] = {SGI159521, SGI0, SGI159522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159523, SGI159524, SGI159524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35156[] = {SGI159525, SGI0, SGI159526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159527, SGI159528, SGI159528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35157[] = {SGI159529, SGI0, SGI159530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159531, SGI159532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35158[] = {SGI159533, SGI0, SGI159534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159535, SGI159536, SGI159536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35159[] = {SGI159537, SGI0, SGI159538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159539, SGI159540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35160[] = {SGI159541, SGI0, SGI159542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159543, SGI159544, SGI159544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35161[] = {SGI159545, SGI0, SGI159546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159547, SGI159548, SGI159548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35162[] = {SGI159549, SGI159550, SGI159551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159552, SGI0, SGI0, SGI0, SGI159553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35163[] = {SGI159554, SGI159555, SGI159556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159557, SGI0, SGI0, SGI0, SGI159558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35164[] = {SGI159559, SGI0, SGI159560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35165[] = {SGI159561, SGI0, SGI159562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159563, SGI159564, SGI159564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35166[] = {SGI159565, SGI0, SGI159566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159567, SGI159568, SGI159568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35167[] = {SGI159569, SGI0, SGI159570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159571, SGI159572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35168[] = {SGI159573, SGI0, SGI159574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159575, SGI159576, SGI159576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35169[] = {SGI159577, SGI0, SGI159578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159580, SGI159582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35170[] = {SGI159579, SGI0, SGI159581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159583, SGI159584, SGI159584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35171[] = {SGI159585, SGI0, SGI159586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159587, SGI159588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35172[] = {SGI159589, SGI159590, SGI159591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159592, SGI0, SGI0, SGI0, SGI159593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35173[] = {SGI159594, SGI0, SGI159595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35174[] = {SGI159596, SGI0, SGI159597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159598, SGI159599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35175[] = {SGI159600, SGI0, SGI159601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159602, SGI159603, SGI159603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35176[] = {SGI159604, SGI0, SGI159605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159606, SGI159608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35177[] = {SGI159607, SGI0, SGI159609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159610, SGI159611, SGI159611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35178[] = {SGI159612, SGI159613, SGI159614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159615, SGI0, SGI0, SGI0, SGI159616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35179[] = {SGI159617, SGI159618, SGI159619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159620, SGI0, SGI0, SGI0, SGI159621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35180[] = {SGI159622, SGI159623, SGI159624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159625, SGI0, SGI0, SGI0, SGI159626, SGI159626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35181[] = {SGI159627, SGI159628, SGI159629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159630, SGI0, SGI0, SGI0, SGI159631, SGI159631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35182[] = {SGI159632, SGI159633, SGI159634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159635, SGI0, SGI0, SGI0, SGI159636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35183[] = {SGI159637, SGI0, SGI159638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159639, SGI159641, SGI159641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35184[] = {SGI159640, SGI159642, SGI159643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159644, SGI0, SGI0, SGI0, SGI159645, SGI159645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35185[] = {SGI159646, SGI0, SGI159647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159648, SGI159650, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35186[] = {SGI159649, SGI0, SGI159651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159652, SGI159654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35187[] = {SGI159653, SGI159655, SGI159656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159657, SGI0, SGI0, SGI0, SGI159658, SGI159658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35188[] = {SGI159659, SGI159660, SGI159661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159662, SGI0, SGI0, SGI0, SGI159663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35189[] = {SGI159664, SGI0, SGI159665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159666, SGI159667, SGI159667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35190[] = {SGI159668, SGI0, SGI159669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159670, SGI159671, SGI159671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35191[] = {SGI159672, SGI159673, SGI159674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159675, SGI0, SGI0, SGI0, SGI159676, SGI159676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35192[] = {SGI159677, SGI0, SGI159678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159679, SGI159680, SGI159680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35193[] = {SGI159681, SGI0, SGI159682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35194[] = {SGI159683, SGI0, SGI159684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159685, SGI159686, SGI159686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35195[] = {SGI159687, SGI159688, SGI159689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159693, SGI0, SGI0, SGI0, SGI159694, SGI159694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35196[] = {SGI159690, SGI159691, SGI159692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159695, SGI0, SGI0, SGI0, SGI159696, SGI159696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35197[] = {SGI159697, SGI159698, SGI159700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159703, SGI0, SGI0, SGI0, SGI159704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35198[] = {SGI159699, SGI159701, SGI159702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159705, SGI159705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35199[] = {SGI0, SGI159708, SGI159710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35200[] = {SGI159706, SGI159707, SGI159709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159711, SGI0, SGI0, SGI0, SGI159712, SGI159712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35201[] = {SGI159713, SGI159714, SGI159715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159719, SGI159719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35202[] = {SGI159716, SGI159717, SGI159718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159720, SGI0, SGI0, SGI0, SGI159721, SGI159721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35203[] = {SGI159722, SGI159723, SGI159724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159727, SGI0, SGI0, SGI0, SGI159729, SGI159729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35204[] = {SGI159725, SGI159726, SGI159728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159730, SGI0, SGI0, SGI0, SGI159731, SGI159731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35205[] = {SGI159732, SGI159733, SGI159734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159735, SGI0, SGI0, SGI0, SGI159736, SGI159736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35206[] = {SGI159737, SGI0, SGI159738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159740, SGI159741, SGI159741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35207[] = {SGI159739, SGI159742, SGI159743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159744, SGI0, SGI0, SGI0, SGI159745, SGI159745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35208[] = {SGI159746, SGI0, SGI159747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159748, SGI159750, SGI159750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35209[] = {SGI159749, SGI159751, SGI159752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159753, SGI0, SGI0, SGI0, SGI159754, SGI159754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35210[] = {SGI159757, SGI0, SGI159758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35211[] = {SGI159755, SGI0, SGI159756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159759, SGI159760, SGI159760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35212[] = {SGI159761, SGI159762, SGI159763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159766, SGI159766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35213[] = {SGI159764, SGI0, SGI159765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159767, SGI159768, SGI159768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35214[] = {SGI159769, SGI159770, SGI159771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159772, SGI159772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35215[] = {SGI0, SGI0, SGI159773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35216[] = {SGI159774, SGI159775, SGI159776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159779, SGI0, SGI0, SGI0, SGI159780, SGI159780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35217[] = {SGI159777, SGI0, SGI159778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159781, SGI159782, SGI159782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35218[] = {SGI159783, SGI159784, SGI159785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159786, SGI0, SGI0, SGI0, SGI159787, SGI159787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35219[] = {SGI159790, SGI0, SGI159791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35220[] = {SGI159788, SGI0, SGI159789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159792, SGI159793, SGI159793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35221[] = {SGI159794, SGI159795, SGI159796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159797, SGI0, SGI0, SGI0, SGI159798, SGI159798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35222[] = {SGI159799, SGI159800, SGI159801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159802, SGI0, SGI0, SGI0, SGI159803, SGI159803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35223[] = {SGI159804, SGI0, SGI159805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35224[] = {SGI159806, SGI159807, SGI159808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159809, SGI0, SGI0, SGI0, SGI159810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35225[] = {SGI159811, SGI159813, SGI159815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159816, SGI0, SGI0, SGI0, SGI159818, SGI159818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35226[] = {SGI159812, SGI0, SGI159814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159817, SGI159819, SGI159819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35227[] = {SGI159820, SGI0, SGI159821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159822, SGI159823, SGI159823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35228[] = {SGI159824, SGI0, SGI159825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159826, SGI159827, SGI159827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35229[] = {SGI159828, SGI0, SGI159829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159830, SGI159831, SGI159831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35230[] = {SGI159832, SGI0, SGI159833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159834, SGI159835, SGI159835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35231[] = {SGI159836, SGI0, SGI159837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159838, SGI159839, SGI159839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35232[] = {SGI159840, SGI159841, SGI159842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159843, SGI0, SGI0, SGI0, SGI159844, SGI159844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35233[] = {SGI159845, SGI159846, SGI159847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159848, SGI0, SGI0, SGI0, SGI159849, SGI159849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35234[] = {SGI159850, SGI0, SGI159851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159852, SGI159853, SGI159853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35235[] = {SGI159854, SGI0, SGI159855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159856, SGI159857, SGI159857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35236[] = {SGI159858, SGI0, SGI159859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159860, SGI159862, SGI159862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35237[] = {SGI159861, SGI0, SGI159863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159864, SGI159865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35238[] = {SGI159866, SGI0, SGI159867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159868, SGI159869, SGI159869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35239[] = {SGI159870, SGI0, SGI159871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159872, SGI159873, SGI159873, SGI0, SGI0, NULL};
const Segment * const SG35240[] = {SGI159874, SGI159875, SGI159876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159877, SGI0, SGI0, SGI0, SGI159878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35241[] = {SGI159879, SGI0, SGI159880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35242[] = {SGI159881, SGI159882, SGI159883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159884, SGI0, SGI0, SGI0, SGI159885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35243[] = {SGI159886, SGI159887, SGI159888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159889, SGI0, SGI0, SGI0, SGI159890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35244[] = {SGI159891, SGI159892, SGI159893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159894, SGI0, SGI0, SGI0, SGI159895, SGI159895, SGI0, SGI159897, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35245[] = {SGI159896, SGI0, SGI159898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159902, SGI159903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35246[] = {SGI159899, SGI159900, SGI159901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159904, SGI0, SGI0, SGI0, SGI159905, SGI159905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35247[] = {SGI159907, SGI159908, SGI159909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159910, SGI0, SGI0, SGI0, SGI159911, SGI159911, SGI0, SGI159912, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35248[] = {SGI159913, SGI159914, SGI159915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159916, SGI0, SGI0, SGI0, SGI159917, SGI159917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35249[] = {SGI159919, SGI159920, SGI159921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159922, SGI0, SGI0, SGI0, SGI159923, SGI159923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35250[] = {SGI159925, SGI0, SGI159926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159927, SGI159928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35251[] = {SGI159929, SGI0, SGI159930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159931, SGI159932, SGI159932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35252[] = {SGI159933, SGI0, SGI159934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159935, SGI159936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35253[] = {SGI159937, SGI0, SGI159938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159939, SGI159940, SGI159940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35254[] = {SGI159941, SGI0, SGI159942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159943, SGI159944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35255[] = {SGI159945, SGI0, SGI159946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159947, SGI159948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35256[] = {SGI159949, SGI0, SGI159950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159951, SGI159952, SGI159952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35257[] = {SGI159953, SGI0, SGI159954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159955, SGI159956, SGI159956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35258[] = {SGI159957, SGI0, SGI159958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159959, SGI159960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35259[] = {SGI159961, SGI159962, SGI159963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159964, SGI0, SGI0, SGI0, SGI159965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35260[] = {SGI159966, SGI159967, SGI159968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159969, SGI0, SGI0, SGI0, SGI159970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35261[] = {SGI159971, SGI159972, SGI159973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159974, SGI0, SGI0, SGI0, SGI159975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35262[] = {SGI159976, SGI159977, SGI159978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159979, SGI0, SGI0, SGI0, SGI159980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35263[] = {SGI159981, SGI159982, SGI159983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159984, SGI0, SGI0, SGI0, SGI159985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35264[] = {SGI159986, SGI159987, SGI159988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159989, SGI0, SGI0, SGI0, SGI159990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35265[] = {SGI159991, SGI159992, SGI159993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159994, SGI0, SGI0, SGI0, SGI159995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35266[] = {SGI159996, SGI159997, SGI159998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI159999, SGI0, SGI0, SGI0, SGI160000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35267[] = {SGI160001, SGI160002, SGI160003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160006, SGI0, SGI0, SGI0, SGI160008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35268[] = {SGI160004, SGI160005, SGI160007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160009, SGI0, SGI0, SGI0, SGI160010, SGI160010, SGI0, SGI0, NULL};
const Segment * const SG35269[] = {SGI160011, SGI160012, SGI160013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160014, SGI0, SGI0, SGI0, SGI160015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35270[] = {SGI160016, SGI0, SGI160017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160018, SGI160019, SGI160019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35271[] = {SGI160020, SGI0, SGI160021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160022, SGI160023, SGI160023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35272[] = {SGI160024, SGI0, SGI160025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160026, SGI160027, SGI160027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35273[] = {SGI160028, SGI160029, SGI160030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160031, SGI0, SGI0, SGI0, SGI160032, SGI160032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35274[] = {SGI160033, SGI0, SGI160034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35275[] = {SGI160035, SGI0, SGI160036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160037, SGI0, SGI0, SGI0, SGI160038, SGI160038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35276[] = {SGI160039, SGI0, SGI160040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160041, SGI160042, SGI160042, SGI0, SGI0, NULL};
const Segment * const SG35277[] = {SGI160043, SGI160044, SGI160045, SGI0, SGI0, SGI0, SGI160046, SGI160047, SGI0, SGI160048, SGI0, SGI0, SGI0, SGI160049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35278[] = {SGI160050, SGI0, SGI160051, SGI0, SGI0, SGI0, SGI160052, SGI160053, SGI0, SGI160054, SGI0, SGI0, SGI0, SGI160055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35279[] = {SGI160056, SGI160057, SGI160058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160059, SGI0, SGI0, SGI0, SGI160060, SGI160060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35280[] = {SGI160061, SGI160062, SGI160063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160064, SGI0, SGI0, SGI0, SGI160065, SGI160065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35281[] = {SGI160066, SGI0, SGI160067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35282[] = {SGI160068, SGI0, SGI160069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160070, SGI160071, SGI160071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35283[] = {SGI160072, SGI0, SGI160073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160074, SGI160075, SGI160075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35284[] = {SGI160076, SGI0, SGI160077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160078, SGI160079, SGI160079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35285[] = {SGI160080, SGI0, SGI160081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160082, SGI160083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35286[] = {SGI160084, SGI0, SGI160085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160086, SGI160087, SGI160087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35287[] = {SGI160088, SGI0, SGI160089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160090, SGI160091, SGI160091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35288[] = {SGI160092, SGI0, SGI160093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160094, SGI160095, SGI160095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35289[] = {SGI160096, SGI0, SGI160097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160098, SGI160099, SGI160099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35290[] = {SGI160100, SGI160101, SGI160102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160103, SGI0, SGI0, SGI0, SGI160104, SGI160104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35291[] = {SGI160105, SGI0, SGI160106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35292[] = {SGI160107, SGI0, SGI160108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160109, SGI160110, SGI160110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35293[] = {SGI160112, SGI160114, SGI160115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160116, SGI0, SGI0, SGI0, SGI160119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35294[] = {SGI160111, SGI0, SGI160113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160117, SGI160118, SGI160118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35295[] = {SGI160120, SGI160121, SGI160122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160123, SGI0, SGI0, SGI0, SGI160124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35296[] = {SGI160125, SGI0, SGI160126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35297[] = {SGI160127, SGI160128, SGI160129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160130, SGI0, SGI0, SGI0, SGI160131, SGI160131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35298[] = {SGI160132, SGI0, SGI160133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160134, SGI0, SGI0, SGI0, SGI160135, SGI160135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35299[] = {SGI160136, SGI0, SGI160137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35300[] = {SGI160138, SGI0, SGI160139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160140, SGI160141, SGI160141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35301[] = {SGI160142, SGI0, SGI160143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160146, SGI160147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35302[] = {SGI160144, SGI0, SGI160145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160148, SGI160149, SGI160149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35303[] = {SGI160150, SGI0, SGI160151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160152, SGI160153, SGI160153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35304[] = {SGI160154, SGI0, SGI160155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160156, SGI160157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35305[] = {SGI160158, SGI0, SGI160159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160160, SGI160161, SGI160161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35306[] = {SGI160162, SGI0, SGI160163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160164, SGI160165, SGI160165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35307[] = {SGI160166, SGI0, SGI160167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160168, SGI160169, SGI160169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35308[] = {SGI160170, SGI0, SGI160171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160172, SGI160173, SGI160173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35309[] = {SGI160174, SGI0, SGI160175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160176, SGI160177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35310[] = {SGI160178, SGI0, SGI160179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160180, SGI160181, SGI160181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35311[] = {SGI160182, SGI0, SGI160183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160184, SGI160185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35312[] = {SGI160186, SGI0, SGI160187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160188, SGI160189, SGI160189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35313[] = {SGI160190, SGI0, SGI160191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160194, SGI160195, SGI160195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35314[] = {SGI160192, SGI0, SGI160193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160196, SGI160197, SGI160197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35315[] = {SGI160198, SGI0, SGI160199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160200, SGI160201, SGI160201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35316[] = {SGI160202, SGI0, SGI160203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160204, SGI160205, SGI160205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35317[] = {SGI160206, SGI0, SGI160207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160208, SGI160209, SGI160209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35318[] = {SGI160210, SGI0, SGI160211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160212, SGI160213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35319[] = {SGI160214, SGI0, SGI160215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160216, SGI160217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35320[] = {SGI160218, SGI0, SGI160219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160220, SGI160221, SGI160221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35321[] = {SGI160222, SGI0, SGI160223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160226, SGI160227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35322[] = {SGI160224, SGI0, SGI160225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160228, SGI160229, SGI160229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35323[] = {SGI160230, SGI160231, SGI160232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160233, SGI0, SGI0, SGI0, SGI160234, SGI160234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35324[] = {SGI160235, SGI0, SGI160236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35325[] = {SGI160237, SGI0, SGI160238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160239, SGI160240, SGI160240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35326[] = {SGI160241, SGI160242, SGI160243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160244, SGI0, SGI0, SGI0, SGI160245, SGI160245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35327[] = {SGI160246, SGI0, SGI160247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35328[] = {SGI160248, SGI160249, SGI160250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160251, SGI0, SGI0, SGI0, SGI160252, SGI160252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35329[] = {SGI160253, SGI160254, SGI160255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160256, SGI0, SGI0, SGI0, SGI160257, SGI160257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35330[] = {SGI160260, SGI0, SGI160261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35331[] = {SGI160258, SGI0, SGI160259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160262, SGI160263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35332[] = {SGI160264, SGI0, SGI160265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35333[] = {SGI160266, SGI0, SGI160267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160268, SGI160269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35334[] = {SGI160270, SGI0, SGI160271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160272, SGI160273, SGI160273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35335[] = {SGI160274, SGI0, SGI160275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160276, SGI160277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35336[] = {SGI160278, SGI160279, SGI160280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160281, SGI0, SGI0, SGI0, SGI160283, SGI160283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35337[] = {SGI160282, SGI0, SGI160284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160285, SGI160286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35338[] = {SGI160287, SGI0, SGI160288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35339[] = {SGI160289, SGI0, SGI160290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160291, SGI160292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35340[] = {SGI160293, SGI0, SGI160294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160296, SGI160298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35341[] = {SGI160295, SGI0, SGI160297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160299, SGI160300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35342[] = {SGI160301, SGI0, SGI160302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160304, SGI160306, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35343[] = {SGI160303, SGI0, SGI160305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160307, SGI160308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35344[] = {SGI160309, SGI0, SGI160310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160311, SGI160312, SGI160312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35345[] = {SGI160313, SGI0, SGI160314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160315, SGI160316, SGI160316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35346[] = {SGI160317, SGI0, SGI160318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160319, SGI160320, SGI160320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35347[] = {SGI160321, SGI0, SGI160322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160323, SGI160324, SGI160324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35348[] = {SGI160325, SGI0, SGI160326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160327, SGI160328, SGI160328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35349[] = {SGI160329, SGI0, SGI160330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160331, SGI160332, SGI160332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35350[] = {SGI0, SGI160333, SGI160334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160335, SGI160335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35351[] = {SGI160336, SGI160337, SGI160338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160339, SGI160339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35352[] = {SGI0, SGI160340, SGI160341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160342, SGI160342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35353[] = {SGI160343, SGI160344, SGI160345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160346, SGI0, SGI0, SGI0, SGI160347, SGI160347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35354[] = {SGI160348, SGI160349, SGI160350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160353, SGI0, SGI0, SGI0, SGI160354, SGI160354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35355[] = {SGI0, SGI160351, SGI160352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160355, SGI160355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35356[] = {SGI160356, SGI160357, SGI160358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160359, SGI0, SGI0, SGI0, SGI160362, SGI160362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35357[] = {SGI0, SGI160360, SGI160361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160363, SGI160363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35358[] = {SGI160364, SGI0, SGI160365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35359[] = {SGI0, SGI160366, SGI160367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35360[] = {SGI160368, SGI160369, SGI160370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160371, SGI0, SGI0, SGI0, SGI160372, SGI160372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35361[] = {SGI0, SGI160373, SGI160374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160375, SGI160375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35362[] = {SGI160376, SGI160377, SGI160378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160379, SGI0, SGI0, SGI0, SGI160380, SGI160380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35363[] = {SGI0, SGI160381, SGI160382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160383, SGI160383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35364[] = {SGI160384, SGI0, SGI160385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35365[] = {SGI0, SGI160386, SGI160387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160388, SGI160388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35366[] = {SGI0, SGI160390, SGI160391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160394, SGI160394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35367[] = {SGI160389, SGI0, SGI160392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160393, SGI160395, SGI160395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35368[] = {SGI160396, SGI160397, SGI160398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160400, SGI0, SGI0, SGI0, SGI160402, SGI160402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35369[] = {SGI160399, SGI0, SGI160401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160403, SGI160404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35370[] = {SGI160405, SGI0, SGI160406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160409, SGI160410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35371[] = {SGI160407, SGI0, SGI160408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35372[] = {SGI160411, SGI0, SGI160412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160413, SGI160414, SGI160414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35373[] = {SGI0, SGI160415, SGI160416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160417, SGI160417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35374[] = {SGI160418, SGI160419, SGI160420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160421, SGI160421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35375[] = {SGI160422, SGI0, SGI160423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160425, SGI160426, SGI160426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35376[] = {SGI160424, SGI0, SGI160427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160429, SGI160430, SGI160430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35377[] = {SGI0, SGI160428, SGI160431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160432, SGI160432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35378[] = {SGI160433, SGI0, SGI160434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160435, SGI160436, SGI160436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35379[] = {SGI0, SGI160437, SGI160438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160439, SGI160439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35380[] = {SGI160440, SGI0, SGI160441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160442, SGI160443, SGI160443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35381[] = {SGI0, SGI160444, SGI160445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160446, SGI160446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35382[] = {SGI160447, SGI0, SGI160448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160449, SGI160450, SGI160450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35383[] = {SGI160451, SGI0, SGI160452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160453, SGI160454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35384[] = {SGI160455, SGI0, SGI160456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160457, SGI160458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35385[] = {SGI160459, SGI0, SGI160460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160461, SGI160462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35386[] = {SGI160463, SGI160464, SGI160465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160466, SGI0, SGI0, SGI0, SGI160467, SGI160467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35387[] = {SGI160468, SGI0, SGI160469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160470, SGI160471, SGI160471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35388[] = {SGI160472, SGI0, SGI160473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35389[] = {SGI160474, SGI0, SGI160475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160476, SGI160477, SGI160477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35390[] = {SGI160478, SGI160479, SGI160480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160481, SGI0, SGI0, SGI0, SGI160482, SGI160482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35391[] = {SGI160484, SGI160485, SGI160486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160487, SGI0, SGI0, SGI0, SGI160488, SGI160488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35392[] = {SGI160490, SGI160491, SGI160492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160493, SGI0, SGI0, SGI0, SGI160494, SGI160494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35393[] = {SGI160496, SGI0, SGI160497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160498, SGI160499, SGI160499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35394[] = {SGI160500, SGI0, SGI160501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160502, SGI160503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35395[] = {SGI160504, SGI160505, SGI160506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160507, SGI0, SGI0, SGI0, SGI160508, SGI160508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35396[] = {SGI160509, SGI0, SGI160510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160511, SGI160512, SGI160512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35397[] = {SGI160513, SGI0, SGI160514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160515, SGI160516, SGI160516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35398[] = {SGI160517, SGI0, SGI160518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160519, SGI160520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35399[] = {SGI160521, SGI0, SGI160522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160523, SGI160524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35400[] = {SGI160525, SGI0, SGI160526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160527, SGI160528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35401[] = {SGI160529, SGI0, SGI160530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160531, SGI160532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35402[] = {SGI160533, SGI0, SGI160534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160535, SGI160536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35403[] = {SGI160537, SGI0, SGI160538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160539, SGI160540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35404[] = {SGI160541, SGI0, SGI160542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160543, SGI160544, SGI160544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35405[] = {SGI160545, SGI0, SGI160546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160547, SGI160548, SGI160548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35406[] = {SGI160549, SGI0, SGI160550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160551, SGI160552, SGI160552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35407[] = {SGI160553, SGI0, SGI160554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160555, SGI160556, SGI160556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35408[] = {SGI160557, SGI0, SGI160558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160559, SGI160560, SGI160560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35409[] = {SGI160561, SGI0, SGI160562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160563, SGI160564, SGI160564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35410[] = {SGI160565, SGI160566, SGI160567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160568, SGI0, SGI0, SGI0, SGI160569, SGI160569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35411[] = {SGI160570, SGI0, SGI160571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160572, SGI160573, SGI160573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35412[] = {SGI160574, SGI0, SGI160575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160576, SGI160577, SGI160577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35413[] = {SGI160578, SGI160579, SGI160580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160581, SGI0, SGI0, SGI0, SGI160582, SGI160582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35414[] = {SGI160583, SGI0, SGI160584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160585, SGI160587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35415[] = {SGI160586, SGI0, SGI160588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160589, SGI160591, SGI160591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160594, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35416[] = {SGI160590, SGI160592, SGI160593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160595, SGI0, SGI0, SGI0, SGI160596, SGI160596, SGI0, SGI160597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35417[] = {SGI160598, SGI0, SGI160599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160602, SGI160603, SGI160603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35418[] = {SGI160600, SGI0, SGI160601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160604, SGI160606, SGI160606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160609, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35419[] = {SGI160605, SGI160607, SGI160608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160610, SGI0, SGI0, SGI0, SGI160611, SGI160611, SGI0, SGI160612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35420[] = {SGI160613, SGI160614, SGI160615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160616, SGI0, SGI0, SGI0, SGI160617, SGI160617, SGI0, SGI160618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35421[] = {SGI160619, SGI160620, SGI160621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160622, SGI0, SGI0, SGI0, SGI160623, SGI160623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35422[] = {SGI160624, SGI0, SGI160625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160626, SGI160627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35423[] = {SGI160628, SGI0, SGI160629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35424[] = {SGI160630, SGI160631, SGI160632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160633, SGI160633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35425[] = {SGI160634, SGI0, SGI160635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160636, SGI160637, SGI160637, SGI0, SGI0, NULL};
const Segment * const SG35426[] = {SGI160638, SGI0, SGI160639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160640, SGI160641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35427[] = {SGI160642, SGI0, SGI160643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160644, SGI160645, SGI160645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35428[] = {SGI160646, SGI0, SGI160647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160648, SGI160649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35429[] = {SGI160650, SGI0, SGI160651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160652, SGI160653, SGI160653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35430[] = {SGI160654, SGI0, SGI160655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160656, SGI160657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35431[] = {SGI0, SGI0, SGI160658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160659, SGI160660, SGI160660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35432[] = {SGI160661, SGI0, SGI160662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160663, SGI160664, SGI160664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35433[] = {SGI160665, SGI0, SGI160666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160667, SGI160668, SGI160668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35434[] = {SGI160669, SGI0, SGI160670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160673, SGI160674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35435[] = {SGI160671, SGI0, SGI160672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160675, SGI160676, SGI160676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35436[] = {SGI160677, SGI0, SGI160678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160679, SGI160680, SGI160680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35437[] = {SGI160681, SGI0, SGI160682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160683, SGI160684, SGI160684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35438[] = {SGI160685, SGI160686, SGI160687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160688, SGI0, SGI0, SGI0, SGI160689, SGI160689, SGI0, SGI160690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35439[] = {SGI160691, SGI160692, SGI160693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160694, SGI0, SGI0, SGI0, SGI160696, SGI160696, SGI0, SGI160698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35440[] = {SGI160695, SGI0, SGI160697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160699, SGI160700, SGI160700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160701, SGI0, SGI0, NULL};
const Segment * const SG35441[] = {SGI160702, SGI160703, SGI160704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160705, SGI0, SGI0, SGI0, SGI160706, SGI160706, SGI0, SGI160707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35442[] = {SGI160708, SGI160709, SGI160710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160711, SGI0, SGI0, SGI0, SGI160712, SGI160712, SGI0, SGI160713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35443[] = {SGI160714, SGI0, SGI160715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160717, SGI160719, SGI160719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160721, SGI0, SGI0, NULL};
const Segment * const SG35444[] = {SGI160716, SGI160718, SGI160720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160722, SGI0, SGI0, SGI0, SGI160723, SGI160723, SGI0, SGI160724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35445[] = {SGI160725, SGI160726, SGI160727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160728, SGI0, SGI0, SGI0, SGI160729, SGI160729, SGI0, SGI160730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35446[] = {SGI160731, SGI160732, SGI160733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160734, SGI0, SGI0, SGI0, SGI160735, SGI160735, SGI0, SGI160736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35447[] = {SGI160737, SGI160738, SGI160739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160740, SGI0, SGI0, SGI0, SGI160741, SGI160741, SGI0, SGI160742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35448[] = {SGI160743, SGI0, SGI160744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160748, SGI160749, SGI160749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35449[] = {SGI160745, SGI160746, SGI160747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160750, SGI0, SGI0, SGI0, SGI160752, SGI160752, SGI0, SGI160754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35450[] = {SGI160751, SGI0, SGI160753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160755, SGI160756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35451[] = {SGI160757, SGI0, SGI160758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160761, SGI160762, SGI160762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35452[] = {SGI160759, SGI160760, SGI160763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160764, SGI0, SGI0, SGI0, SGI160765, SGI160765, SGI0, SGI160766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35453[] = {SGI160767, SGI0, SGI160768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160769, SGI160770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35454[] = {SGI160771, SGI0, SGI160772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160773, SGI160774, SGI160774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35455[] = {SGI160775, SGI0, SGI160776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160777, SGI160778, SGI160778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35456[] = {SGI160779, SGI0, SGI160780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160781, SGI160782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35457[] = {SGI160783, SGI160784, SGI160785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160786, SGI0, SGI0, SGI0, SGI160787, SGI160787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35458[] = {SGI160788, SGI0, SGI160789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35459[] = {SGI160790, SGI0, SGI160791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160792, SGI160794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35460[] = {SGI160793, SGI160795, SGI160796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160797, SGI0, SGI0, SGI0, SGI160798, SGI160798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35461[] = {SGI160799, SGI160800, SGI160801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160802, SGI0, SGI0, SGI0, SGI160803, SGI160803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35462[] = {SGI160804, SGI0, SGI160805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160806, SGI160807, SGI160807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35463[] = {SGI160808, SGI160809, SGI160810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160811, SGI0, SGI0, SGI0, SGI160812, SGI160812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35464[] = {SGI160813, SGI160814, SGI160815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160816, SGI0, SGI0, SGI0, SGI160817, SGI0, SGI0, SGI160818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35465[] = {SGI160819, SGI160820, SGI160821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160822, SGI0, SGI0, SGI0, SGI160823, SGI160823, SGI0, SGI160824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35466[] = {SGI160825, SGI160826, SGI160827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160828, SGI0, SGI0, SGI0, SGI160829, SGI0, SGI0, SGI160830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35467[] = {SGI160831, SGI160832, SGI160833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160834, SGI0, SGI0, SGI0, SGI160835, SGI0, SGI0, SGI160836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35468[] = {SGI160837, SGI0, SGI160838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160839, SGI160840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35469[] = {SGI160841, SGI0, SGI160842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160843, SGI160844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35470[] = {SGI160845, SGI160846, SGI160847, SGI160848, SGI160849, SGI160849, SGI0, NULL};
const Segment * const SG35471[] = {SGI160850, SGI160851, SGI160852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160853, SGI0, SGI0, SGI0, SGI160854, SGI160854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35472[] = {SGI160855, SGI160856, SGI160857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160858, SGI0, SGI0, SGI0, SGI160859, SGI160859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35473[] = {SGI160860, SGI0, SGI160861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35474[] = {SGI160862, SGI160863, SGI160864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160865, SGI0, SGI0, SGI0, SGI160866, SGI160866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35475[] = {SGI160867, SGI160868, SGI160869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160870, SGI0, SGI0, SGI0, SGI160871, SGI160871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35476[] = {SGI160872, SGI160873, SGI160874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160875, SGI0, SGI0, SGI0, SGI160876, SGI160876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35477[] = {SGI160877, SGI0, SGI160878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35478[] = {SGI160879, SGI160880, SGI160881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160882, SGI0, SGI0, SGI0, SGI160883, SGI160883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35479[] = {SGI160884, SGI160885, SGI160886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160887, SGI0, SGI0, SGI0, SGI160888, SGI160888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35480[] = {SGI160889, SGI160890, SGI160891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160892, SGI0, SGI0, SGI0, SGI160893, SGI160893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35481[] = {SGI160894, SGI160895, SGI160896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160897, SGI0, SGI0, SGI0, SGI160898, SGI160898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35482[] = {SGI160899, SGI160900, SGI160901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160902, SGI0, SGI0, SGI0, SGI160903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35483[] = {SGI160904, SGI160905, SGI160906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160907, SGI160907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35484[] = {SGI160908, SGI0, SGI160909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160910, SGI160911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35485[] = {SGI160912, SGI160913, SGI160914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160915, SGI0, SGI0, SGI0, SGI160916, SGI160916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35486[] = {SGI160917, SGI0, SGI160918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35487[] = {SGI160919, SGI160920, SGI160921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160922, SGI0, SGI0, SGI0, SGI160923, SGI160923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35488[] = {SGI160924, SGI0, SGI160925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160926, SGI0, SGI0, SGI0, SGI160927, SGI160927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35489[] = {SGI160928, SGI0, SGI160929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35490[] = {SGI160930, SGI0, SGI160931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160932, SGI160933, SGI160933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35491[] = {SGI160934, SGI0, SGI160935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160936, SGI160937, SGI160937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35492[] = {SGI160938, SGI0, SGI160939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160940, SGI160941, SGI160941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35493[] = {SGI160942, SGI0, SGI160943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160944, SGI160945, SGI160945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35494[] = {SGI160946, SGI0, SGI160947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160948, SGI160949, SGI160949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35495[] = {SGI160950, SGI0, SGI160951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160952, SGI160953, SGI160953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35496[] = {SGI160954, SGI0, SGI160955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160956, SGI160957, SGI160957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35497[] = {SGI160958, SGI0, SGI160959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160960, SGI160961, SGI160961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35498[] = {SGI160962, SGI0, SGI160963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160964, SGI160965, SGI160965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35499[] = {SGI160966, SGI0, SGI160967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160968, SGI160969, SGI160969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35500[] = {SGI160970, SGI0, SGI160971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160972, SGI160973, SGI160973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35501[] = {SGI160974, SGI0, SGI160975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160976, SGI160977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35502[] = {SGI160978, SGI0, SGI160979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160980, SGI160981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35503[] = {SGI160982, SGI0, SGI160983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160984, SGI160985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35504[] = {SGI160986, SGI0, SGI160987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160988, SGI160989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35505[] = {SGI160990, SGI0, SGI160991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160992, SGI160993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35506[] = {SGI160994, SGI0, SGI160995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI160996, SGI160997, SGI160997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35507[] = {SGI160998, SGI0, SGI160999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161000, SGI161001, SGI161001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35508[] = {SGI161002, SGI0, SGI161003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161004, SGI161005, SGI161005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35509[] = {SGI161006, SGI0, SGI161007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161008, SGI161009, SGI161009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35510[] = {SGI161010, SGI0, SGI161011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161012, SGI161013, SGI161013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35511[] = {SGI161014, SGI0, SGI161015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161016, SGI161017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35512[] = {SGI161018, SGI0, SGI161019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161020, SGI161021, SGI161021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35513[] = {SGI161022, SGI0, SGI161023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161024, SGI161025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35514[] = {SGI161026, SGI0, SGI161027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161028, SGI161029, SGI161029, SGI0, SGI0, NULL};
const Segment * const SG35515[] = {SGI161030, SGI0, SGI161031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161032, SGI161033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35516[] = {SGI161034, SGI0, SGI161035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161036, SGI161037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35517[] = {SGI161038, SGI0, SGI161039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161040, SGI161041, SGI161041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35518[] = {SGI161042, SGI0, SGI161043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161044, SGI161045, SGI161045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35519[] = {SGI161046, SGI0, SGI161047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161048, SGI161049, SGI161049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35520[] = {SGI161050, SGI0, SGI161051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161052, SGI161053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35521[] = {SGI161054, SGI0, SGI161055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161056, SGI161057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35522[] = {SGI161058, SGI0, SGI161059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161060, SGI161061, SGI161061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35523[] = {SGI161062, SGI0, SGI161063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161064, SGI161065, SGI161065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35524[] = {SGI161066, SGI0, SGI161067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161068, SGI161069, SGI161069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35525[] = {SGI161070, SGI0, SGI161071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161072, SGI161073, SGI161073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35526[] = {SGI161074, SGI0, SGI161075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161076, SGI161077, SGI161077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35527[] = {SGI161078, SGI0, SGI161079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161080, SGI161081, SGI161081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35528[] = {SGI161082, SGI0, SGI161083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161084, SGI161085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35529[] = {SGI161086, SGI0, SGI161087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161088, SGI161089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35530[] = {SGI161090, SGI0, SGI161091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161092, SGI161093, SGI161093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35531[] = {SGI161094, SGI161095, SGI161096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161097, SGI0, SGI0, SGI0, SGI161098, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35532[] = {SGI161099, SGI0, SGI161100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35533[] = {SGI161101, SGI0, SGI161102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161103, SGI161104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35534[] = {SGI161105, SGI0, SGI161106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161107, SGI161108, SGI161108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35535[] = {SGI161109, SGI0, SGI161110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161111, SGI161112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35536[] = {SGI161113, SGI161114, SGI161115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161116, SGI0, SGI0, SGI161117, SGI161118, SGI161118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35537[] = {SGI161119, SGI0, SGI161120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161121, SGI161122, SGI161122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35538[] = {SGI161123, SGI0, SGI161124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161125, SGI161126, SGI161126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35539[] = {SGI161127, SGI161128, SGI161129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161130, SGI0, SGI0, SGI0, SGI161131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35540[] = {SGI161132, SGI0, SGI161133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35541[] = {SGI161134, SGI0, SGI161135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161136, SGI161137, SGI161137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35542[] = {SGI161138, SGI161139, SGI161140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161141, SGI0, SGI0, SGI0, SGI161142, SGI161142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35543[] = {SGI161143, SGI0, SGI161144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161145, SGI161146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35544[] = {SGI161147, SGI161148, SGI161149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161150, SGI0, SGI0, SGI0, SGI161151, SGI161151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35545[] = {SGI161152, SGI0, SGI161153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35546[] = {SGI161154, SGI161155, SGI161156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161157, SGI0, SGI0, SGI0, SGI161158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35547[] = {SGI161159, SGI161160, SGI161161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161162, SGI0, SGI0, SGI0, SGI161163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35548[] = {SGI161164, SGI0, SGI161165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161166, SGI161167, SGI161167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35549[] = {SGI161168, SGI0, SGI161169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161170, SGI161171, SGI161171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35550[] = {SGI161172, SGI0, SGI161173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161174, SGI161175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35551[] = {SGI161176, SGI0, SGI161177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161178, SGI161179, SGI161179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35552[] = {SGI161180, SGI0, SGI161181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161182, SGI161183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35553[] = {SGI161184, SGI0, SGI161185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161186, SGI161187, SGI161187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35554[] = {SGI161188, SGI0, SGI161189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161190, SGI161191, SGI161191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35555[] = {SGI161192, SGI161193, SGI161194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161195, SGI0, SGI0, SGI0, SGI161196, SGI161196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35556[] = {SGI161198, SGI161199, SGI161200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161201, SGI0, SGI0, SGI0, SGI161203, SGI161203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35557[] = {SGI161202, SGI0, SGI161204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161206, SGI161207, SGI161207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35558[] = {SGI161208, SGI161209, SGI161210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161211, SGI0, SGI0, SGI0, SGI161213, SGI161213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35559[] = {SGI161212, SGI0, SGI161214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161216, SGI161217, SGI161217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35560[] = {SGI161218, SGI0, SGI161219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161220, SGI161221, SGI161221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35561[] = {SGI161222, SGI161223, SGI161224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161225, SGI0, SGI0, SGI0, SGI161226, SGI161226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35562[] = {SGI161227, SGI161228, SGI161229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161230, SGI0, SGI0, SGI0, SGI161231, SGI161231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35563[] = {SGI161232, SGI161233, SGI161234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161235, SGI0, SGI0, SGI0, SGI161236, SGI161236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35564[] = {SGI161237, SGI161238, SGI161239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161240, SGI0, SGI0, SGI0, SGI161241, SGI161241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35565[] = {SGI161244, SGI0, SGI161246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35566[] = {SGI161242, SGI161243, SGI161245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161247, SGI0, SGI0, SGI0, SGI161248, SGI161248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35567[] = {SGI161249, SGI161250, SGI161251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161254, SGI0, SGI0, SGI0, SGI161255, SGI161255, SGI161258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35568[] = {SGI161252, SGI0, SGI161253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161256, SGI161257, SGI161257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35569[] = {SGI161259, SGI161260, SGI161261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161262, SGI0, SGI0, SGI0, SGI161263, SGI161263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35570[] = {SGI161264, SGI161265, SGI161266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161267, SGI0, SGI0, SGI0, SGI161269, SGI161269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35571[] = {SGI161268, SGI161270, SGI161271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161272, SGI0, SGI0, SGI0, SGI161275, SGI161275, SGI161276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35572[] = {SGI161273, SGI0, SGI161274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161277, SGI161278, SGI161278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35573[] = {SGI161279, SGI161280, SGI161281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161282, SGI0, SGI0, SGI0, SGI161283, SGI161283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35574[] = {SGI161284, SGI0, SGI161285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161287, SGI161289, SGI161289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35575[] = {SGI161286, SGI161288, SGI161290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161291, SGI0, SGI0, SGI0, SGI161292, SGI161292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35576[] = {SGI161293, SGI161294, SGI161295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161296, SGI0, SGI0, SGI0, SGI161297, SGI161297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35577[] = {SGI161298, SGI161299, SGI161300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161301, SGI0, SGI0, SGI0, SGI161302, SGI161302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35578[] = {SGI161303, SGI161304, SGI161305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161306, SGI0, SGI0, SGI0, SGI161307, SGI161307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35579[] = {SGI161308, SGI161309, SGI161310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161311, SGI0, SGI0, SGI0, SGI161312, SGI161312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35580[] = {SGI161313, SGI0, SGI161314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161315, SGI161316, SGI161316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35581[] = {SGI161317, SGI0, SGI161318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161321, SGI161323, SGI161323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35582[] = {SGI161319, SGI161320, SGI161322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161324, SGI0, SGI0, SGI0, SGI161325, SGI161325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35583[] = {SGI161326, SGI161327, SGI161328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161329, SGI0, SGI0, SGI0, SGI161330, SGI161330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35584[] = {SGI161331, SGI161332, SGI161333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161334, SGI0, SGI0, SGI0, SGI161335, SGI161335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35585[] = {SGI161336, SGI0, SGI161337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35586[] = {SGI161338, SGI161340, SGI161341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161343, SGI0, SGI0, SGI0, SGI161345, SGI161345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35587[] = {SGI161339, SGI0, SGI161342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161344, SGI161346, SGI161346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35588[] = {SGI161347, SGI0, SGI161348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161349, SGI161350, SGI161350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35589[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35590[] = {SGI161352, SGI161353, SGI161354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161355, SGI0, SGI0, SGI0, SGI161356, SGI161356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35591[] = {SGI161357, SGI161358, SGI161359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161360, SGI0, SGI0, SGI0, SGI161361, SGI161361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35592[] = {SGI161362, SGI161363, SGI161364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161365, SGI0, SGI0, SGI0, SGI161366, SGI161366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35593[] = {SGI161367, SGI161368, SGI161369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161370, SGI0, SGI0, SGI0, SGI161371, SGI161371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35594[] = {SGI161372, SGI0, SGI161373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161374, SGI161375, SGI161375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35595[] = {SGI161376, SGI0, SGI161378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161381, SGI161383, SGI161383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35596[] = {SGI161377, SGI161379, SGI161380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161382, SGI0, SGI0, SGI0, SGI161384, SGI161384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35597[] = {SGI161385, SGI0, SGI161386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161387, SGI161388, SGI161388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35598[] = {SGI161389, SGI0, SGI161390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161391, SGI161392, SGI161392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35599[] = {SGI161393, SGI0, SGI161394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161395, SGI161396, SGI161396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35600[] = {SGI161397, SGI0, SGI161398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161399, SGI161400, SGI161400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35601[] = {SGI161401, SGI0, SGI161402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161403, SGI161404, SGI161404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35602[] = {SGI161405, SGI0, SGI161406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161407, SGI161408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35603[] = {SGI161409, SGI0, SGI161410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161411, SGI161412, SGI161412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35604[] = {SGI161413, SGI0, SGI161414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161415, SGI161416, SGI161416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35605[] = {SGI161417, SGI0, SGI161418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161419, SGI161420, SGI161420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35606[] = {SGI161421, SGI161422, SGI161423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161424, SGI0, SGI0, SGI0, SGI161425, SGI161425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35607[] = {SGI161426, SGI0, SGI161427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161428, SGI161429, SGI161429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35608[] = {SGI161430, SGI0, SGI161431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161432, SGI161433, SGI161433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35609[] = {SGI161434, SGI0, SGI161435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161436, SGI161437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35610[] = {SGI161438, SGI0, SGI161439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161440, SGI161441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35611[] = {SGI161442, SGI0, SGI161443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161444, SGI161445, SGI161445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35612[] = {SGI161446, SGI0, SGI161447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161448, SGI161449, SGI161449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35613[] = {SGI161450, SGI0, SGI161451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161452, SGI161453, SGI161453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35614[] = {SGI161454, SGI0, SGI161455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161456, SGI161457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35615[] = {SGI161458, SGI161459, SGI161460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161461, SGI0, SGI0, SGI0, SGI161462, SGI161462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35616[] = {SGI161463, SGI161464, SGI161465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161466, SGI0, SGI0, SGI0, SGI161467, SGI161467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35617[] = {SGI161468, SGI161469, SGI161470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161473, SGI0, SGI0, SGI0, SGI161475, SGI161475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35618[] = {SGI161471, SGI161472, SGI161474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161476, SGI0, SGI0, SGI0, SGI161477, SGI161477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35619[] = {SGI161478, SGI0, SGI161479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35620[] = {SGI161480, SGI161481, SGI161482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161483, SGI0, SGI0, SGI0, SGI161484, SGI161484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35621[] = {SGI161485, SGI0, SGI161486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35622[] = {SGI161487, SGI161488, SGI161489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161490, SGI0, SGI0, SGI0, SGI161491, SGI161491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35623[] = {SGI161492, SGI0, SGI161493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161496, SGI161497, SGI161497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35624[] = {SGI161494, SGI0, SGI161495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161498, SGI161499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35625[] = {SGI161500, SGI0, SGI161501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161504, SGI161505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35626[] = {SGI161502, SGI0, SGI161503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161506, SGI161507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35627[] = {SGI161508, SGI0, SGI161509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161510, SGI161511, SGI161511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35628[] = {SGI161512, SGI0, SGI161513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161514, SGI161515, SGI161515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35629[] = {SGI161516, SGI0, SGI161517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161518, SGI161519, SGI161519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35630[] = {SGI161520, SGI0, SGI161521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161522, SGI161523, SGI161523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35631[] = {SGI161524, SGI0, SGI161525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161526, SGI161527, SGI161527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35632[] = {SGI161528, SGI0, SGI161529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161530, SGI161531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35633[] = {SGI161532, SGI0, SGI161533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161534, SGI161535, SGI161535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35634[] = {SGI161536, SGI0, SGI161537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161538, SGI161539, SGI161539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35635[] = {SGI161540, SGI0, SGI161541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161542, SGI161543, SGI161543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35636[] = {SGI161544, SGI0, SGI161545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161546, SGI161547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35637[] = {SGI161548, SGI0, SGI161549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161550, SGI161551, SGI161551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35638[] = {SGI161552, SGI0, SGI161553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161554, SGI161555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35639[] = {SGI161556, SGI0, SGI161557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161558, SGI161559, SGI161559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35640[] = {SGI161560, SGI0, SGI161561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161562, SGI161563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35641[] = {SGI161564, SGI0, SGI161566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35642[] = {SGI161565, SGI0, SGI161567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161568, SGI161569, SGI161569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35643[] = {SGI161570, SGI161571, SGI161572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35644[] = {SGI161573, SGI0, SGI161574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161575, SGI161576, SGI161576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35645[] = {SGI161577, SGI161578, SGI161579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161582, SGI0, SGI0, SGI0, SGI161583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35646[] = {SGI161580, SGI0, SGI161581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161584, SGI161585, SGI161585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35647[] = {SGI161586, SGI161587, SGI161588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161592, SGI0, SGI0, SGI0, SGI161593, SGI161595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35648[] = {SGI161589, SGI161590, SGI161591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161594, SGI0, SGI0, SGI0, SGI161596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35649[] = {SGI161597, SGI0, SGI161598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161599, SGI161600, SGI161600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35650[] = {SGI161601, SGI161602, SGI161603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161604, SGI0, SGI0, SGI0, SGI161605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35651[] = {SGI161606, SGI0, SGI161607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161608, SGI161609, SGI161609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35652[] = {SGI161610, SGI161611, SGI161612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161613, SGI161613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35653[] = {SGI161614, SGI0, SGI161615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161616, SGI161617, SGI161617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35654[] = {SGI161618, SGI161619, SGI161620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161622, SGI0, SGI0, SGI0, SGI161624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35655[] = {SGI161621, SGI0, SGI161623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161625, SGI161626, SGI161626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35656[] = {SGI161627, SGI0, SGI161628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161629, SGI161631, SGI161631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35657[] = {SGI161630, SGI161632, SGI161633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35658[] = {SGI161634, SGI161635, SGI161636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161637, SGI0, SGI0, SGI0, SGI161638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35659[] = {SGI161639, SGI161640, SGI161641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161642, SGI161642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35660[] = {SGI161643, SGI0, SGI161644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35661[] = {SGI161645, SGI0, SGI161646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161647, SGI161648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35662[] = {SGI161649, SGI0, SGI161650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161651, SGI161652, SGI161652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35663[] = {SGI161653, SGI0, SGI161654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161655, SGI161656, SGI161656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35664[] = {SGI161657, SGI0, SGI161658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161659, SGI161660, SGI161660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35665[] = {SGI161661, SGI0, SGI161662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161663, SGI161664, SGI161664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35666[] = {SGI161665, SGI0, SGI161666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161667, SGI161668, SGI161668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35667[] = {SGI161669, SGI0, SGI161670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161671, SGI161672, SGI161672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35668[] = {SGI161673, SGI0, SGI161674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161675, SGI161676, SGI161676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35669[] = {SGI161677, SGI0, SGI161678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161679, SGI161680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35670[] = {SGI161681, SGI0, SGI161682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161683, SGI161684, SGI161684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35671[] = {SGI161685, SGI0, SGI161686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161687, SGI161688, SGI161688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35672[] = {SGI161689, SGI0, SGI161690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161691, SGI161692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35673[] = {SGI161693, SGI0, SGI161694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161695, SGI161696, SGI161696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35674[] = {SGI161697, SGI0, SGI161698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161699, SGI161700, SGI161700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35675[] = {SGI161705, SGI0, SGI161706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161707, SGI161708, SGI161708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35676[] = {SGI161709, SGI0, SGI161710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161711, SGI161712, SGI161712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35677[] = {SGI161713, SGI161714, SGI161715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161716, SGI0, SGI0, SGI0, SGI161717, SGI161717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35678[] = {SGI161718, SGI0, SGI161719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35679[] = {SGI161720, SGI161721, SGI161722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161723, SGI0, SGI0, SGI0, SGI161724, SGI161724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35680[] = {SGI161725, SGI0, SGI161726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161727, SGI161728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35681[] = {SGI161729, SGI0, SGI161730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161731, SGI161732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35682[] = {SGI161733, SGI0, SGI161734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161735, SGI161736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35683[] = {SGI161737, SGI0, SGI161738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161739, SGI161740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35684[] = {SGI161701, SGI0, SGI161702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161703, SGI161704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35685[] = {SGI161741, SGI0, SGI161742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161743, SGI161744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35686[] = {SGI161745, SGI0, SGI161746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161747, SGI161748, SGI161748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35687[] = {SGI161749, SGI0, SGI161750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161751, SGI161752, SGI161752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35688[] = {SGI161753, SGI0, SGI161754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161755, SGI161756, SGI161756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35689[] = {SGI161757, SGI0, SGI161758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161762, SGI161763, SGI161763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161766, SGI0, SGI0, NULL};
const Segment * const SG35690[] = {SGI161759, SGI161760, SGI161761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161764, SGI0, SGI0, SGI0, SGI161765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35691[] = {SGI161767, SGI161768, SGI161769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161770, SGI0, SGI0, SGI0, SGI161771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35692[] = {SGI161772, SGI161773, SGI161774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161775, SGI0, SGI0, SGI0, SGI161776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35693[] = {SGI161777, SGI0, SGI161778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161779, SGI161780, SGI161780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161781, SGI0, SGI0, NULL};
const Segment * const SG35694[] = {SGI161782, SGI0, SGI161783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161784, SGI161785, SGI161785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161786, SGI0, SGI0, NULL};
const Segment * const SG35695[] = {SGI161787, SGI0, SGI161789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161791, SGI161792, SGI161792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35696[] = {SGI161788, SGI0, SGI161790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161793, SGI161794, SGI161794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161795, SGI0, SGI0, NULL};
const Segment * const SG35697[] = {SGI161796, SGI0, SGI161797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161798, SGI161800, SGI161800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35698[] = {SGI161799, SGI0, SGI161801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161802, SGI161803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35699[] = {SGI161804, SGI0, SGI161805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161807, SGI161809, SGI161809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161810, SGI0, SGI0, NULL};
const Segment * const SG35700[] = {SGI161806, SGI0, SGI161808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161811, SGI161813, SGI161813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35701[] = {SGI161812, SGI0, SGI161814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161815, SGI161816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35702[] = {SGI161817, SGI0, SGI161818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161819, SGI161820, SGI161820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35703[] = {SGI161821, SGI0, SGI161822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161823, SGI161824, SGI161824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161825, SGI0, SGI0, NULL};
const Segment * const SG35704[] = {SGI161826, SGI0, SGI161827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161828, SGI161829, SGI161829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35705[] = {SGI161830, SGI0, SGI161831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161832, SGI161833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35706[] = {SGI0, SGI161834, SGI161835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161836, SGI161836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35707[] = {SGI0, SGI161837, SGI161838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161839, SGI161839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35708[] = {SGI161840, SGI161841, SGI161842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161843, SGI0, SGI0, SGI0, SGI161844, SGI161844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35709[] = {SGI161845, SGI0, SGI161846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161848, SGI161849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35710[] = {SGI0, SGI161847, SGI161850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161851, SGI161851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35711[] = {SGI161852, SGI161853, SGI161854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161857, SGI0, SGI0, SGI0, SGI161858, SGI161858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35712[] = {SGI161855, SGI0, SGI161856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161859, SGI161860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35713[] = {SGI161861, SGI161862, SGI161863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161864, SGI161864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35714[] = {SGI161865, SGI0, SGI161866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35715[] = {SGI0, SGI161867, SGI161868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161869, SGI161869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35716[] = {SGI0, SGI161870, SGI161871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161872, SGI161872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35717[] = {SGI0, SGI161873, SGI161874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35718[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161875, SGI0, SGI0, SGI161876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35719[] = {SGI161877, SGI0, SGI161878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161879, SGI161880, SGI161880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35720[] = {SGI161881, SGI0, SGI161882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161883, SGI161884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35721[] = {SGI161885, SGI0, SGI161886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161887, SGI161888, SGI161888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35722[] = {SGI161889, SGI0, SGI161890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161891, SGI161892, SGI161892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35723[] = {SGI161893, SGI0, SGI161894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161895, SGI161896, SGI161896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35724[] = {SGI161897, SGI0, SGI161898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161899, SGI161900, SGI161900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35725[] = {SGI161901, SGI0, SGI161902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161903, SGI161904, SGI161904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35726[] = {SGI161905, SGI0, SGI161906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161907, SGI161908, SGI161908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35727[] = {SGI161909, SGI0, SGI161910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161913, SGI161914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35728[] = {SGI161911, SGI0, SGI161912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161915, SGI161916, SGI161916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35729[] = {SGI161917, SGI0, SGI161918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161919, SGI161920, SGI161920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35730[] = {SGI161921, SGI0, SGI161922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161926, SGI161927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35731[] = {SGI161923, SGI161924, SGI161925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161928, SGI0, SGI0, SGI0, SGI161929, SGI161929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35732[] = {SGI161931, SGI161932, SGI161933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161934, SGI0, SGI0, SGI0, SGI161935, SGI161935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35733[] = {SGI161937, SGI0, SGI161938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161939, SGI161940, SGI161940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35734[] = {SGI161941, SGI161942, SGI161943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161944, SGI0, SGI0, SGI0, SGI161945, SGI161945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35735[] = {SGI161947, SGI161948, SGI161949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161950, SGI0, SGI0, SGI0, SGI161951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35736[] = {SGI161952, SGI161953, SGI161954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161955, SGI0, SGI0, SGI0, SGI161956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35737[] = {SGI161957, SGI161958, SGI161959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161960, SGI0, SGI0, SGI0, SGI161961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35738[] = {SGI161962, SGI0, SGI161963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161964, SGI161965, SGI161965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35739[] = {SGI161966, SGI0, SGI161967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161968, SGI161969, SGI161969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35740[] = {SGI161970, SGI0, SGI161971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161972, SGI161973, SGI161973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35741[] = {SGI161974, SGI0, SGI161975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161976, SGI161977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35742[] = {SGI161978, SGI0, SGI161979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161980, SGI161981, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35743[] = {SGI161982, SGI0, SGI161983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161984, SGI161985, SGI161985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35744[] = {SGI161986, SGI161987, SGI161988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161989, SGI0, SGI0, SGI0, SGI161990, SGI161990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35745[] = {SGI161991, SGI0, SGI161992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35746[] = {SGI161993, SGI161994, SGI161995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI161996, SGI0, SGI0, SGI0, SGI161997, SGI161997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35747[] = {SGI161998, SGI161999, SGI162000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162003, SGI0, SGI0, SGI0, SGI162005, SGI162005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35748[] = {SGI162001, SGI0, SGI162002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162004, SGI0, SGI0, SGI0, SGI162006, SGI162006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35749[] = {SGI162007, SGI162008, SGI162009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162010, SGI0, SGI0, SGI0, SGI162011, SGI162011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35750[] = {SGI162012, SGI0, SGI162013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35751[] = {SGI162014, SGI162015, SGI162016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162017, SGI0, SGI0, SGI0, SGI162018, SGI162018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35752[] = {SGI162019, SGI162020, SGI162021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162022, SGI0, SGI0, SGI0, SGI162023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35753[] = {SGI162025, SGI162026, SGI162027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162028, SGI0, SGI0, SGI0, SGI162029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35754[] = {SGI162031, SGI162032, SGI162033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162034, SGI0, SGI0, SGI0, SGI162035, SGI162035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35755[] = {SGI162037, SGI162038, SGI162039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162040, SGI0, SGI0, SGI0, SGI162041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35756[] = {SGI162043, SGI0, SGI162044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162045, SGI162046, SGI162046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35757[] = {SGI162047, SGI162048, SGI162049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162050, SGI0, SGI0, SGI0, SGI162051, SGI162051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35758[] = {SGI162052, SGI0, SGI162053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35759[] = {SGI162054, SGI162055, SGI162056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162057, SGI0, SGI0, SGI0, SGI162058, SGI162058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35760[] = {SGI162062, SGI0, SGI162064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35761[] = {SGI162059, SGI162060, SGI162061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162063, SGI0, SGI0, SGI0, SGI162065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35762[] = {SGI162066, SGI162067, SGI162068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162069, SGI0, SGI0, SGI0, SGI162070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35763[] = {SGI162071, SGI0, SGI162072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162073, SGI162074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35764[] = {SGI162075, SGI162076, SGI162077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162079, SGI0, SGI0, SGI0, SGI162081, SGI162081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35765[] = {SGI162078, SGI0, SGI162080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162082, SGI162083, SGI162083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35766[] = {SGI162084, SGI162085, SGI162086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162087, SGI0, SGI0, SGI0, SGI162088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35767[] = {SGI162089, SGI0, SGI162090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162095, SGI162096, SGI162096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35768[] = {SGI162092, SGI0, SGI162094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162098, SGI162100, SGI162100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35769[] = {SGI162091, SGI0, SGI162093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162097, SGI162099, SGI162099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35770[] = {SGI162101, SGI0, SGI162103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162105, SGI162107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35771[] = {SGI162102, SGI0, SGI162104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162106, SGI162108, SGI162108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35772[] = {SGI162109, SGI0, SGI162110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162111, SGI162112, SGI162112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35773[] = {SGI162113, SGI0, SGI162114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162115, SGI162116, SGI162116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35774[] = {SGI162117, SGI0, SGI162118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162119, SGI162120, SGI162120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35775[] = {SGI162121, SGI0, SGI162122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162123, SGI162124, SGI162124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35776[] = {SGI162125, SGI0, SGI162126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162127, SGI162128, SGI162128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35777[] = {SGI162129, SGI0, SGI162130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162131, SGI162132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35778[] = {SGI162133, SGI0, SGI162134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162135, SGI162136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35779[] = {SGI162137, SGI0, SGI162138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162139, SGI162140, SGI162140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35780[] = {SGI162141, SGI0, SGI162143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162145, SGI162146, SGI162146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35781[] = {SGI162142, SGI0, SGI162144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162147, SGI162148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35782[] = {SGI162149, SGI0, SGI162150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162153, SGI162154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35783[] = {SGI162151, SGI0, SGI162152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162155, SGI162156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35784[] = {SGI162157, SGI0, SGI162158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162161, SGI162162, SGI162162, SGI0, SGI0, NULL};
const Segment * const SG35785[] = {SGI162159, SGI0, SGI162160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162163, SGI162164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35786[] = {SGI162165, SGI0, SGI162166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162167, SGI162168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35787[] = {SGI162169, SGI0, SGI162170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162171, SGI162172, SGI162172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35788[] = {SGI162173, SGI0, SGI162174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162175, SGI162176, SGI162176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35789[] = {SGI162177, SGI162178, SGI162179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162180, SGI0, SGI0, SGI0, SGI162181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35790[] = {SGI162182, SGI162183, SGI162184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162185, SGI0, SGI0, SGI0, SGI162186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35791[] = {SGI162187, SGI162188, SGI162189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162190, SGI0, SGI0, SGI0, SGI162191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35792[] = {SGI162192, SGI0, SGI162193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35793[] = {SGI162194, SGI0, SGI162195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162196, SGI162197, SGI162197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35794[] = {SGI162198, SGI0, SGI162199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162200, SGI162201, SGI162201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162202, SGI0, SGI0, NULL};
const Segment * const SG35795[] = {SGI162203, SGI0, SGI162204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162205, SGI162206, SGI162206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162207, SGI0, SGI0, NULL};
const Segment * const SG35796[] = {SGI162208, SGI0, SGI162209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162210, SGI162211, SGI162211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35797[] = {SGI162212, SGI0, SGI162213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162214, SGI162215, SGI162215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162216, SGI0, SGI0, NULL};
const Segment * const SG35798[] = {SGI162217, SGI0, SGI162218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162219, SGI162220, SGI162220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162221, SGI0, SGI0, NULL};
const Segment * const SG35799[] = {SGI162222, SGI0, SGI162223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162224, SGI162225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35800[] = {SGI162226, SGI0, SGI162227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162228, SGI162229, SGI162229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162230, SGI0, SGI0, NULL};
const Segment * const SG35801[] = {SGI162231, SGI0, SGI162232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162233, SGI162234, SGI162234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162235, SGI0, SGI0, NULL};
const Segment * const SG35802[] = {SGI162236, SGI0, SGI162237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162238, SGI162239, SGI162239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162240, SGI0, SGI0, NULL};
const Segment * const SG35803[] = {SGI162241, SGI162242, SGI162243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162244, SGI0, SGI0, SGI0, SGI162245, SGI162245, SGI162246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35804[] = {SGI162247, SGI162248, SGI162249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162250, SGI0, SGI0, SGI0, SGI162251, SGI162251, SGI162252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35805[] = {SGI162253, SGI0, SGI162254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162258, SGI162259, SGI162259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35806[] = {SGI162255, SGI162256, SGI162257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162260, SGI0, SGI0, SGI0, SGI162261, SGI162261, SGI162262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35807[] = {SGI162263, SGI0, SGI162264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162265, SGI162266, SGI162266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35808[] = {SGI162267, SGI0, SGI162268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162269, SGI162270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35809[] = {SGI162271, SGI0, SGI162272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162273, SGI162274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35810[] = {SGI162275, SGI0, SGI162276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162277, SGI162278, SGI162278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35811[] = {SGI162279, SGI0, SGI162280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162283, SGI162284, SGI162284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35812[] = {SGI162281, SGI0, SGI162282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162285, SGI162286, SGI162286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35813[] = {SGI162287, SGI0, SGI162288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162289, SGI162290, SGI162290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35814[] = {SGI162291, SGI0, SGI162292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162293, SGI162294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35815[] = {SGI162295, SGI0, SGI162296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162297, SGI162298, SGI162298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35816[] = {SGI162299, SGI0, SGI162300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162301, SGI162302, SGI162302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35817[] = {SGI162303, SGI0, SGI162304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162305, SGI162306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35818[] = {SGI162307, SGI0, SGI162308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162309, SGI162310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35819[] = {SGI162311, SGI162312, SGI162313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162314, SGI0, SGI0, SGI0, SGI162315, SGI162315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35820[] = {SGI162316, SGI0, SGI162317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35821[] = {SGI162318, SGI162319, SGI162320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162321, SGI0, SGI0, SGI0, SGI162322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35822[] = {SGI162323, SGI162324, SGI162325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162326, SGI0, SGI0, SGI0, SGI162327, SGI162327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35823[] = {SGI162328, SGI0, SGI162329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35824[] = {SGI162330, SGI0, SGI162331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162332, SGI162333, SGI162333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35825[] = {SGI162334, SGI0, SGI162335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162336, SGI162337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35826[] = {SGI162338, SGI0, SGI162339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162340, SGI162341, SGI162341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35827[] = {SGI162342, SGI162343, SGI162344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162345, SGI0, SGI0, SGI0, SGI162346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35828[] = {SGI162347, SGI162348, SGI162349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162350, SGI0, SGI0, SGI0, SGI162351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35829[] = {SGI162352, SGI162353, SGI162354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162356, SGI0, SGI0, SGI0, SGI162359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35830[] = {SGI162355, SGI162357, SGI162358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162360, SGI0, SGI0, SGI0, SGI162361, SGI162361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35831[] = {SGI162362, SGI0, SGI162363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35832[] = {SGI162364, SGI162365, SGI162366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162367, SGI0, SGI0, SGI0, SGI162368, SGI162368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35833[] = {SGI162369, SGI0, SGI162370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35834[] = {SGI162371, SGI162372, SGI162373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162374, SGI0, SGI0, SGI0, SGI162375, SGI162375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35835[] = {SGI162376, SGI162377, SGI162378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162379, SGI0, SGI0, SGI0, SGI162380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35836[] = {SGI162381, SGI0, SGI162383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35837[] = {SGI162382, SGI0, SGI162384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162385, SGI162386, SGI162386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35838[] = {SGI162387, SGI162388, SGI162390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162392, SGI0, SGI0, SGI0, SGI162393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35839[] = {SGI162389, SGI0, SGI162391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162394, SGI162395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35840[] = {SGI162396, SGI0, SGI162398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162401, SGI162403, SGI162403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35841[] = {SGI162397, SGI162399, SGI162400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162402, SGI0, SGI0, SGI0, SGI162404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35842[] = {SGI162405, SGI162406, SGI162407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162408, SGI0, SGI0, SGI0, SGI162409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35843[] = {SGI162410, SGI162411, SGI162412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162415, SGI0, SGI0, SGI0, SGI162416, SGI162416, SGI0, SGI0, NULL};
const Segment * const SG35844[] = {SGI162413, SGI0, SGI162414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162417, SGI162418, SGI162418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35845[] = {SGI162419, SGI0, SGI162420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162421, SGI162422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35846[] = {SGI162423, SGI0, SGI162424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162425, SGI162426, SGI162426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35847[] = {SGI162427, SGI0, SGI162428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162429, SGI162430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35848[] = {SGI162431, SGI0, SGI162432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162433, SGI162434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35849[] = {SGI162435, SGI162436, SGI162437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162438, SGI0, SGI0, SGI0, SGI162439, SGI162439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35850[] = {SGI162440, SGI0, SGI162441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162444, SGI162446, SGI162446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35851[] = {SGI162442, SGI162443, SGI162445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162447, SGI0, SGI0, SGI0, SGI162448, SGI162448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35852[] = {SGI162449, SGI0, SGI162450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162451, SGI162452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35853[] = {SGI162453, SGI162454, SGI162455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162456, SGI0, SGI0, SGI0, SGI162457, SGI162457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35854[] = {SGI162458, SGI162459, SGI162460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162461, SGI0, SGI0, SGI0, SGI162462, SGI162462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35855[] = {SGI162463, SGI162464, SGI162465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162466, SGI0, SGI0, SGI0, SGI162467, SGI162467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35856[] = {SGI162468, SGI162469, SGI162470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162471, SGI0, SGI0, SGI0, SGI162472, SGI162472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35857[] = {SGI162473, SGI0, SGI162474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162476, SGI162478, SGI162478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35858[] = {SGI162475, SGI162477, SGI162479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162480, SGI0, SGI0, SGI0, SGI162481, SGI162481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35859[] = {SGI162482, SGI162483, SGI162484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162485, SGI0, SGI0, SGI0, SGI162488, SGI162488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35860[] = {SGI162486, SGI0, SGI162487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162489, SGI162490, SGI162490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35861[] = {SGI162491, SGI0, SGI162492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162493, SGI162494, SGI162494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35862[] = {SGI162495, SGI0, SGI162496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162499, SGI162500, SGI162500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35863[] = {SGI162497, SGI0, SGI162498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162501, SGI162502, SGI162502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35864[] = {SGI162503, SGI0, SGI162504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35865[] = {SGI162505, SGI0, SGI162506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35866[] = {SGI162507, SGI0, SGI162508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162509, SGI162510, SGI162510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35867[] = {SGI162511, SGI0, SGI162513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162515, SGI162517, SGI162517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35868[] = {SGI162512, SGI0, SGI162514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162516, SGI162518, SGI162518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35869[] = {SGI162519, SGI0, SGI162520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162521, SGI162522, SGI162522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35870[] = {SGI162523, SGI0, SGI162524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162525, SGI162526, SGI162526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35871[] = {SGI162527, SGI0, SGI162528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162529, SGI162530, SGI162530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35872[] = {SGI162531, SGI0, SGI162532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162533, SGI162534, SGI162534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35873[] = {SGI162535, SGI0, SGI162536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162537, SGI162538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35874[] = {SGI162539, SGI0, SGI162540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162541, SGI162542, SGI162542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35875[] = {SGI162543, SGI0, SGI162544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162545, SGI162546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35876[] = {SGI162547, SGI0, SGI162548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162549, SGI162550, SGI162550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35877[] = {SGI162551, SGI162552, SGI162553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162554, SGI0, SGI0, SGI0, SGI162555, SGI162555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35878[] = {SGI162556, SGI0, SGI162557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162558, SGI162559, SGI162559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35879[] = {SGI162560, SGI162561, SGI162562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162563, SGI0, SGI0, SGI0, SGI162564, SGI162564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35880[] = {SGI162565, SGI0, SGI162566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35881[] = {SGI162567, SGI0, SGI162568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162569, SGI162570, SGI162570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35882[] = {SGI162571, SGI0, SGI162572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162573, SGI162574, SGI162574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35883[] = {SGI162575, SGI0, SGI162576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162577, SGI162578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35884[] = {SGI162579, SGI0, SGI162580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162581, SGI162582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35885[] = {SGI162583, SGI0, SGI162584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162585, SGI162586, SGI162586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35886[] = {SGI162587, SGI0, SGI162588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162589, SGI162590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35887[] = {SGI162591, SGI162592, SGI162593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162594, SGI0, SGI0, SGI0, SGI162595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35888[] = {SGI162596, SGI0, SGI162597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162598, SGI162599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35889[] = {SGI162600, SGI162601, SGI162602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162603, SGI0, SGI0, SGI0, SGI162604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35890[] = {SGI162605, SGI0, SGI162606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162607, SGI162608, SGI162608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35891[] = {SGI162609, SGI0, SGI162610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162611, SGI162612, SGI162612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35892[] = {SGI162613, SGI0, SGI162614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162615, SGI162616, SGI162616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35893[] = {SGI162617, SGI0, SGI162618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162619, SGI162620, SGI162620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35894[] = {SGI162621, SGI0, SGI162622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162623, SGI162624, SGI162624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35895[] = {SGI162625, SGI0, SGI162626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162627, SGI162628, SGI162628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35896[] = {SGI162629, SGI162630, SGI162631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162632, SGI162632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35897[] = {SGI162633, SGI162634, SGI162635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162636, SGI0, SGI0, SGI0, SGI162637, SGI162637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35898[] = {SGI162638, SGI0, SGI162639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35899[] = {SGI0, SGI162640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162641, SGI0, SGI0, SGI0, SGI0, SGI162642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35900[] = {SGI162643, SGI162644, SGI162645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162646, SGI0, SGI0, SGI0, SGI162647, SGI162647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35901[] = {SGI162648, SGI162649, SGI162650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162651, SGI0, SGI0, SGI0, SGI162652, SGI162652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35902[] = {SGI162653, SGI0, SGI162654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35903[] = {SGI162655, SGI162656, SGI162657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162658, SGI0, SGI0, SGI0, SGI162659, SGI162659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35904[] = {SGI0, SGI162660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162661, SGI0, SGI0, SGI0, SGI162662, SGI162662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35905[] = {SGI162663, SGI162664, SGI162665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162668, SGI162668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35906[] = {SGI162666, SGI0, SGI162667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162669, SGI162670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35907[] = {SGI0, SGI162671, SGI162672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162673, SGI0, SGI0, SGI0, SGI0, SGI162674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35908[] = {SGI162675, SGI0, SGI162676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162677, SGI162678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35909[] = {SGI162679, SGI162680, SGI162681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162682, SGI0, SGI0, SGI0, SGI162683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35910[] = {SGI162684, SGI162685, SGI162686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162687, SGI0, SGI0, SGI0, SGI162688, SGI162688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35911[] = {SGI162689, SGI0, SGI162690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35912[] = {SGI162691, SGI162692, SGI162693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162694, SGI0, SGI0, SGI0, SGI162695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35913[] = {SGI162696, SGI162697, SGI162698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35914[] = {SGI162699, SGI0, SGI162700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35915[] = {SGI162701, SGI0, SGI162702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162703, SGI162704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35916[] = {SGI162705, SGI162706, SGI162707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162708, SGI162708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35917[] = {SGI162709, SGI162710, SGI162711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162712, SGI0, SGI0, SGI0, SGI162713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35918[] = {SGI162714, SGI162715, SGI162716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162717, SGI162717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35919[] = {SGI0, SGI162718, SGI162719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162720, SGI0, SGI0, SGI0, SGI162721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35920[] = {SGI162722, SGI162723, SGI162724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162725, SGI162725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35921[] = {SGI162726, SGI162727, SGI162728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35922[] = {SGI162729, SGI162730, SGI162731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162732, SGI162732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35923[] = {SGI162733, SGI0, SGI162734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162735, SGI162736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35924[] = {SGI162737, SGI162738, SGI162739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162740, SGI162740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35925[] = {SGI162741, SGI162742, SGI162743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35926[] = {SGI162744, SGI162745, SGI162746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35927[] = {SGI162747, SGI162748, SGI162749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162750, SGI0, SGI0, SGI0, SGI162751, SGI162752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35928[] = {SGI162753, SGI162754, SGI162755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162756, SGI0, SGI0, SGI0, SGI162757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35929[] = {SGI162758, SGI162759, SGI162760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162761, SGI0, SGI0, SGI0, SGI162762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35930[] = {SGI162763, SGI162764, SGI162765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162766, SGI0, SGI0, SGI0, SGI162767, SGI162767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35931[] = {SGI0, SGI162768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35932[] = {SGI162769, SGI162770, SGI162771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162773, SGI0, SGI0, SGI0, SGI162775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35933[] = {SGI162772, SGI0, SGI162774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162776, SGI162777, SGI162777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35934[] = {SGI162778, SGI162779, SGI162780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162781, SGI0, SGI0, SGI0, SGI162782, SGI162782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35935[] = {SGI162783, SGI162784, SGI162786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162788, SGI0, SGI0, SGI0, SGI162789, SGI162789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35936[] = {SGI162785, SGI0, SGI162787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162790, SGI162791, SGI162791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35937[] = {SGI162792, SGI0, SGI162793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35938[] = {SGI162794, SGI162795, SGI162796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162797, SGI0, SGI0, SGI0, SGI162798, SGI162798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35939[] = {SGI162800, SGI162801, SGI162802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162803, SGI0, SGI0, SGI0, SGI162804, SGI162804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35940[] = {SGI162806, SGI162807, SGI162808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162809, SGI0, SGI0, SGI0, SGI162810, SGI162810, SGI0, SGI162811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35941[] = {SGI162812, SGI162813, SGI162814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162815, SGI0, SGI0, SGI0, SGI162816, SGI162816, SGI0, SGI162817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35942[] = {SGI162818, SGI162819, SGI162820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162821, SGI0, SGI0, SGI0, SGI162822, SGI162822, SGI0, SGI162823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35943[] = {SGI162824, SGI162825, SGI162826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162827, SGI0, SGI0, SGI0, SGI162828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35944[] = {SGI162830, SGI0, SGI162832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35945[] = {SGI162829, SGI162831, SGI162833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162834, SGI0, SGI0, SGI0, SGI162835, SGI162835, SGI0, SGI162836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35946[] = {SGI162837, SGI162838, SGI162839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162840, SGI0, SGI0, SGI0, SGI162841, SGI162841, SGI0, SGI162842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35947[] = {SGI162843, SGI162844, SGI162845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162846, SGI0, SGI0, SGI0, SGI162847, SGI162847, SGI0, SGI162848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35948[] = {SGI162849, SGI162850, SGI162851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162852, SGI0, SGI0, SGI0, SGI162854, SGI162854, SGI0, SGI162856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35949[] = {SGI162853, SGI0, SGI162855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162857, SGI162858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35950[] = {SGI162859, SGI162860, SGI162861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162862, SGI0, SGI0, SGI0, SGI162863, SGI162863, SGI0, SGI162864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35951[] = {SGI162865, SGI0, SGI162866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162867, SGI162868, SGI162868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35952[] = {SGI162869, SGI0, SGI162871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162873, SGI162876, SGI162876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35953[] = {SGI162870, SGI0, SGI162872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162874, SGI162875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35954[] = {SGI162877, SGI0, SGI162878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162879, SGI162880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35955[] = {SGI162881, SGI162882, SGI162883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162884, SGI0, SGI0, SGI0, SGI162885, SGI162885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35956[] = {SGI162887, SGI162888, SGI162889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162890, SGI0, SGI0, SGI0, SGI162891, SGI162891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35957[] = {SGI162893, SGI162894, SGI162895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162896, SGI0, SGI0, SGI0, SGI162897, SGI162897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35958[] = {SGI162899, SGI162900, SGI162901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162902, SGI0, SGI0, SGI0, SGI162903, SGI162903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35959[] = {SGI162905, SGI162906, SGI162907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162908, SGI0, SGI0, SGI0, SGI162909, SGI162909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35960[] = {SGI162911, SGI0, SGI162912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162914, SGI162916, SGI162916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35961[] = {SGI162913, SGI162915, SGI162917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162918, SGI0, SGI0, SGI0, SGI162919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35962[] = {SGI162921, SGI162922, SGI162923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162924, SGI0, SGI0, SGI0, SGI162926, SGI162926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35963[] = {SGI162925, SGI162927, SGI162928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162929, SGI0, SGI0, SGI0, SGI162930, SGI162930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35964[] = {SGI162931, SGI162932, SGI162933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162934, SGI0, SGI0, SGI0, SGI162935, SGI162935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35965[] = {SGI162936, SGI0, SGI162937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162939, SGI162941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35966[] = {SGI162943, SGI0, SGI162945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35967[] = {SGI162938, SGI162940, SGI162942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162944, SGI0, SGI0, SGI0, SGI162946, SGI162946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35968[] = {SGI162947, SGI0, SGI162948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162949, SGI162951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35969[] = {SGI162950, SGI0, SGI162952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162953, SGI0, SGI0, SGI0, SGI162954, SGI162954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35970[] = {SGI162955, SGI162956, SGI162957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162958, SGI0, SGI0, SGI0, SGI162959, SGI162959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35971[] = {SGI162960, SGI162961, SGI162962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162963, SGI0, SGI0, SGI0, SGI162964, SGI162964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35972[] = {SGI162965, SGI162966, SGI162967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162968, SGI0, SGI0, SGI0, SGI162969, SGI162969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35973[] = {SGI162970, SGI162971, SGI162973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162976, SGI0, SGI0, SGI0, SGI162978, SGI162978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35974[] = {SGI162972, SGI162974, SGI162975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162977, SGI0, SGI0, SGI0, SGI162979, SGI162979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35975[] = {SGI162980, SGI0, SGI162981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35976[] = {SGI162982, SGI162983, SGI162984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162985, SGI0, SGI0, SGI0, SGI162986, SGI162986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35977[] = {SGI162987, SGI162988, SGI162989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162990, SGI0, SGI0, SGI0, SGI162991, SGI162991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35978[] = {SGI162992, SGI162993, SGI162994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI162995, SGI0, SGI0, SGI0, SGI162996, SGI162996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35979[] = {SGI162997, SGI162998, SGI162999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163000, SGI0, SGI0, SGI0, SGI163001, SGI163001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35980[] = {SGI163002, SGI163003, SGI163004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163005, SGI0, SGI0, SGI0, SGI163006, SGI163006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35981[] = {SGI163007, SGI163008, SGI163009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163010, SGI0, SGI0, SGI0, SGI163011, SGI163011, SGI0, SGI163012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35982[] = {SGI163013, SGI163014, SGI163015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163016, SGI0, SGI0, SGI0, SGI163017, SGI163017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35983[] = {SGI163018, SGI163019, SGI163020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163023, SGI0, SGI0, SGI0, SGI163025, SGI163025, SGI0, SGI163026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35984[] = {SGI163021, SGI163022, SGI163024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163027, SGI0, SGI0, SGI0, SGI163028, SGI163028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35985[] = {SGI163029, SGI0, SGI163030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35986[] = {SGI163031, SGI163032, SGI163033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163036, SGI0, SGI0, SGI0, SGI163038, SGI163038, SGI0, SGI163041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35987[] = {SGI163034, SGI163035, SGI163037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163042, SGI0, SGI0, SGI0, SGI163044, SGI163044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35988[] = {SGI163039, SGI163040, SGI163043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163045, SGI0, SGI0, SGI0, SGI163046, SGI163046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35989[] = {SGI163047, SGI0, SGI163048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35990[] = {SGI163049, SGI0, SGI163050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163051, SGI163052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35991[] = {SGI163053, SGI163054, SGI163055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163056, SGI0, SGI0, SGI0, SGI163057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35992[] = {SGI163058, SGI0, SGI163059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163060, SGI163061, SGI163061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35993[] = {SGI163062, SGI0, SGI163063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163064, SGI163065, SGI163065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35994[] = {SGI163066, SGI0, SGI163067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163068, SGI163069, SGI163069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35995[] = {SGI163070, SGI0, SGI163071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163072, SGI163073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35996[] = {SGI163074, SGI0, SGI163075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163076, SGI163077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35997[] = {SGI163078, SGI0, SGI163079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163080, SGI163081, SGI163081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35998[] = {SGI163082, SGI0, SGI163083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163084, SGI163085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG35999[] = {SGI163086, SGI0, SGI163087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163088, SGI163089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36000[] = {SGI163090, SGI0, SGI163091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163092, SGI163093, SGI163093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36001[] = {SGI163094, SGI0, SGI163095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163096, SGI163097, SGI163097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36002[] = {SGI163098, SGI0, SGI163099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163100, SGI163101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36003[] = {SGI163102, SGI163103, SGI163104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163105, SGI0, SGI0, SGI0, SGI163106, SGI163106, SGI163107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36004[] = {SGI163108, SGI163109, SGI163110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163111, SGI0, SGI0, SGI0, SGI163112, SGI163112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36005[] = {SGI163115, SGI0, SGI163117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163119, SGI163120, SGI163120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36006[] = {SGI163113, SGI163114, SGI163116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163118, SGI0, SGI0, SGI0, SGI163122, SGI163122, SGI163125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36007[] = {SGI163121, SGI163123, SGI163124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163126, SGI0, SGI0, SGI0, SGI163127, SGI163127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36008[] = {SGI163128, SGI0, SGI163129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163130, SGI163131, SGI163131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36009[] = {SGI163132, SGI163133, SGI163134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163135, SGI0, SGI0, SGI0, SGI163138, SGI163138, SGI163139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36010[] = {SGI163136, SGI0, SGI163137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163140, SGI163141, SGI163141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36011[] = {SGI163142, SGI163143, SGI163144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163145, SGI0, SGI0, SGI0, SGI163146, SGI163146, SGI163147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36012[] = {SGI163148, SGI163149, SGI163150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163153, SGI0, SGI0, SGI0, SGI163154, SGI163154, SGI163157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36013[] = {SGI163151, SGI0, SGI163152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163155, SGI163156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36014[] = {SGI163158, SGI0, SGI163159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163162, SGI163163, SGI163163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36015[] = {SGI163160, SGI0, SGI163161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163164, SGI163165, SGI163165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36016[] = {SGI163166, SGI0, SGI163167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163168, SGI163169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36017[] = {SGI163170, SGI0, SGI163171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163172, SGI163173, SGI163173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36018[] = {SGI163174, SGI0, SGI163175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163176, SGI163177, SGI163177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36019[] = {SGI163178, SGI0, SGI163179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163180, SGI163181, SGI163181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36020[] = {SGI163182, SGI0, SGI163183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163184, SGI163185, SGI163185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36021[] = {SGI163186, SGI0, SGI163187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163188, SGI163189, SGI163189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36022[] = {SGI163190, SGI0, SGI163191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163192, SGI163193, SGI163193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36023[] = {SGI163194, SGI0, SGI163195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163196, SGI163197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36024[] = {SGI163198, SGI0, SGI163199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163200, SGI163201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36025[] = {SGI163202, SGI0, SGI163203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163204, SGI163205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36026[] = {SGI163206, SGI0, SGI163207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163208, SGI163209, SGI163209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36027[] = {SGI163210, SGI0, SGI163211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163212, SGI163213, SGI163213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36028[] = {SGI163214, SGI0, SGI163215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163216, SGI163217, SGI163217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36029[] = {SGI163218, SGI163219, SGI163220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163223, SGI0, SGI0, SGI0, SGI163224, SGI163224, SGI0, SGI0, NULL};
const Segment * const SG36030[] = {SGI163221, SGI0, SGI163222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163225, SGI163226, SGI163226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36031[] = {SGI163227, SGI0, SGI163228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163229, SGI163230, SGI163230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36032[] = {SGI163231, SGI0, SGI163232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36033[] = {SGI163233, SGI163234, SGI163235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163236, SGI0, SGI0, SGI0, SGI163237, SGI163237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36034[] = {SGI163238, SGI0, SGI163239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36035[] = {SGI163240, SGI0, SGI163241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163242, SGI163243, SGI163243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36036[] = {SGI163244, SGI0, SGI163245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163246, SGI163247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36037[] = {SGI163248, SGI0, SGI163249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163250, SGI163251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36038[] = {SGI163252, SGI0, SGI163253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163254, SGI163255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36039[] = {SGI163256, SGI0, SGI163257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163258, SGI163259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36040[] = {SGI163260, SGI0, SGI163261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163262, SGI163263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36041[] = {SGI163264, SGI0, SGI163265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163266, SGI163267, SGI163267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36042[] = {SGI163268, SGI0, SGI163269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163270, SGI163271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36043[] = {SGI163272, SGI0, SGI163273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163274, SGI163275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36044[] = {SGI163276, SGI163277, SGI163278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163279, SGI0, SGI0, SGI0, SGI163280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36045[] = {SGI163281, SGI0, SGI163282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163285, SGI163286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36046[] = {SGI163283, SGI0, SGI163284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163287, SGI163288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36047[] = {SGI163289, SGI0, SGI163290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163291, SGI163292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36048[] = {SGI163293, SGI0, SGI163294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163295, SGI163296, SGI163296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36049[] = {SGI163297, SGI0, SGI163298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163299, SGI163300, SGI163300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36050[] = {SGI163301, SGI0, SGI163302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163303, SGI163304, SGI163304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36051[] = {SGI163305, SGI0, SGI163306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163307, SGI163308, SGI163308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36052[] = {SGI163309, SGI163310, SGI163311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163312, SGI0, SGI0, SGI0, SGI163313, SGI163313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36053[] = {SGI163314, SGI163315, SGI163316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163317, SGI0, SGI0, SGI0, SGI163318, SGI163318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36054[] = {SGI163319, SGI0, SGI163320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163321, SGI163322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36055[] = {SGI163323, SGI0, SGI163324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163325, SGI163326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36056[] = {SGI163327, SGI0, SGI163328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163329, SGI163330, SGI163330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36057[] = {SGI163331, SGI0, SGI163332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163333, SGI163334, SGI163334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36058[] = {SGI163335, SGI0, SGI163336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163337, SGI163338, SGI163338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36059[] = {SGI163339, SGI0, SGI163340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163341, SGI163342, SGI163342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36060[] = {SGI163343, SGI0, SGI163344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163345, SGI163346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36061[] = {SGI163347, SGI0, SGI163348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163349, SGI163350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36062[] = {SGI163351, SGI0, SGI163352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163353, SGI163354, SGI163354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36063[] = {SGI163355, SGI0, SGI163356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163357, SGI163358, SGI163358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36064[] = {SGI163359, SGI0, SGI163360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163361, SGI163362, SGI163362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36065[] = {SGI163363, SGI0, SGI163364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163365, SGI163366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36066[] = {SGI163367, SGI0, SGI163368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163369, SGI163370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36067[] = {SGI163371, SGI0, SGI163372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163373, SGI163374, SGI163374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36068[] = {SGI163375, SGI0, SGI163376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163377, SGI163378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36069[] = {SGI163379, SGI163380, SGI163381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163382, SGI0, SGI0, SGI0, SGI163383, SGI163383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36070[] = {SGI163384, SGI0, SGI163385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36071[] = {SGI163386, SGI163387, SGI163388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163391, SGI0, SGI0, SGI0, SGI163392, SGI163392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36072[] = {SGI163389, SGI0, SGI163390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163393, SGI163394, SGI163394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36073[] = {SGI163395, SGI163396, SGI163397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163399, SGI0, SGI0, SGI0, SGI163401, SGI163401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36074[] = {SGI163398, SGI0, SGI163400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163402, SGI163403, SGI163403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36075[] = {SGI163404, SGI163405, SGI163406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163407, SGI0, SGI0, SGI0, SGI163409, SGI163409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36076[] = {SGI163408, SGI0, SGI163410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163411, SGI163412, SGI163412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36077[] = {SGI163413, SGI0, SGI163414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163415, SGI163416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36078[] = {SGI163417, SGI0, SGI163418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163420, SGI0, SGI0, SGI0, SGI163422, SGI163422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36079[] = {SGI163419, SGI0, SGI163421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163423, SGI163424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36080[] = {SGI163425, SGI163426, SGI163427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163428, SGI0, SGI0, SGI0, SGI163429, SGI163429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36081[] = {SGI163430, SGI163431, SGI163432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163433, SGI0, SGI0, SGI0, SGI163434, SGI163434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36082[] = {SGI163435, SGI163436, SGI163437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163438, SGI0, SGI0, SGI0, SGI163439, SGI163439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36083[] = {SGI163440, SGI163441, SGI163442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163443, SGI0, SGI0, SGI0, SGI163444, SGI163444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36084[] = {SGI163445, SGI0, SGI163446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163447, SGI163448, SGI163448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36085[] = {SGI163449, SGI163450, SGI163451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163452, SGI0, SGI0, SGI0, SGI163453, SGI163453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36086[] = {SGI163454, SGI163455, SGI163456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163457, SGI0, SGI0, SGI0, SGI163458, SGI163458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36087[] = {SGI163459, SGI163460, SGI163461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163462, SGI0, SGI0, SGI0, SGI163463, SGI163463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36088[] = {SGI163464, SGI163465, SGI163466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163467, SGI0, SGI0, SGI0, SGI163468, SGI163468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36089[] = {SGI163471, SGI0, SGI163472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36090[] = {SGI163469, SGI0, SGI163470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163473, SGI163474, SGI163474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36091[] = {SGI163475, SGI163476, SGI163477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36092[] = {SGI163478, SGI0, SGI163479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163480, SGI163481, SGI163481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36093[] = {SGI163482, SGI0, SGI163483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163484, SGI163485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36094[] = {SGI163486, SGI163487, SGI163488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163489, SGI0, SGI0, SGI0, SGI163490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36095[] = {SGI163491, SGI0, SGI163492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163493, SGI163494, SGI163494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36096[] = {SGI163495, SGI163496, SGI163497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163498, SGI163498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36097[] = {SGI163500, SGI0, SGI163502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163506, SGI163509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36098[] = {SGI163499, SGI0, SGI163501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163504, SGI163507, SGI163507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36099[] = {SGI163503, SGI163505, SGI163508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163510, SGI0, SGI0, SGI0, SGI163511, SGI163511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36100[] = {SGI163512, SGI163513, SGI163514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163515, SGI0, SGI0, SGI0, SGI163516, SGI163516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36101[] = {SGI163517, SGI0, SGI163518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36102[] = {SGI163519, SGI0, SGI163520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163521, SGI163522, SGI163522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36103[] = {SGI163523, SGI163524, SGI163525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163526, SGI0, SGI0, SGI0, SGI163527, SGI163527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36104[] = {SGI163528, SGI0, SGI163529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163533, SGI163534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36105[] = {SGI163530, SGI163531, SGI163532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163535, SGI0, SGI0, SGI0, SGI163536, SGI163536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36106[] = {SGI163537, SGI0, SGI163538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163542, SGI163543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36107[] = {SGI163539, SGI163540, SGI163541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163544, SGI0, SGI0, SGI0, SGI163545, SGI163545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36108[] = {SGI163546, SGI163547, SGI163548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163549, SGI0, SGI0, SGI0, SGI163550, SGI163550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36109[] = {SGI163551, SGI163552, SGI163553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163554, SGI0, SGI0, SGI0, SGI163555, SGI163555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36110[] = {SGI163556, SGI163557, SGI163558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163559, SGI0, SGI0, SGI0, SGI163560, SGI163560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36111[] = {SGI163561, SGI163562, SGI163563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163564, SGI0, SGI0, SGI0, SGI163565, SGI163565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36112[] = {SGI163566, SGI0, SGI163567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36113[] = {SGI163568, SGI0, SGI163569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163570, SGI163571, SGI163571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36114[] = {SGI163572, SGI0, SGI163573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163574, SGI163575, SGI163575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36115[] = {SGI163576, SGI163577, SGI163578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163579, SGI0, SGI0, SGI0, SGI163580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36116[] = {SGI163581, SGI0, SGI163582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36117[] = {SGI163583, SGI0, SGI163584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163585, SGI163586, SGI163586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36118[] = {SGI163587, SGI0, SGI163588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163589, SGI163590, SGI163590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36119[] = {SGI163591, SGI0, SGI163592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163593, SGI163594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36120[] = {SGI163595, SGI163596, SGI163597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163598, SGI0, SGI0, SGI0, SGI163599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36121[] = {SGI163600, SGI163601, SGI163602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163603, SGI0, SGI0, SGI0, SGI163604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36122[] = {SGI163605, SGI163606, SGI163607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163608, SGI0, SGI0, SGI0, SGI163609, SGI163609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36123[] = {SGI163610, SGI0, SGI163611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163612, SGI163613, SGI163613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36124[] = {SGI163614, SGI0, SGI163615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163616, SGI163617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36125[] = {SGI163618, SGI0, SGI163619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163620, SGI163621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36126[] = {SGI163622, SGI0, SGI163623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163624, SGI163625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36127[] = {SGI163626, SGI163627, SGI163628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163629, SGI0, SGI0, SGI0, SGI163630, SGI163630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36128[] = {SGI163631, SGI163632, SGI163633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163634, SGI0, SGI0, SGI0, SGI163635, SGI163635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36129[] = {SGI163636, SGI0, SGI163637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36130[] = {SGI163638, SGI163639, SGI163640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163641, SGI0, SGI0, SGI0, SGI163642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36131[] = {SGI163643, SGI163644, SGI163645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163646, SGI0, SGI0, SGI0, SGI163647, SGI163647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36132[] = {SGI163649, SGI163650, SGI163651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163652, SGI0, SGI0, SGI0, SGI163653, SGI163653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36133[] = {SGI163655, SGI163656, SGI163657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163658, SGI0, SGI0, SGI0, SGI163659, SGI163659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36134[] = {SGI163661, SGI0, SGI163662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163663, SGI163664, SGI163664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36135[] = {SGI163665, SGI0, SGI163666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163667, SGI163668, SGI163668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36136[] = {SGI163669, SGI0, SGI163670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163671, SGI163672, SGI163672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36137[] = {SGI163673, SGI0, SGI163674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163677, SGI163678, SGI163678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36138[] = {SGI163675, SGI0, SGI163676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163679, SGI163680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36139[] = {SGI163681, SGI0, SGI163682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163683, SGI163684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36140[] = {SGI163685, SGI0, SGI163686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163687, SGI163688, SGI163688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36141[] = {SGI163689, SGI0, SGI163690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163691, SGI163692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36142[] = {SGI163693, SGI0, SGI163694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163695, SGI163696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36143[] = {SGI163697, SGI0, SGI163698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163699, SGI163700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36144[] = {SGI163701, SGI0, SGI163702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163703, SGI163704, SGI163704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36145[] = {SGI163705, SGI0, SGI163706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163707, SGI163708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36146[] = {SGI163709, SGI0, SGI163710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163712, SGI163714, SGI163714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36147[] = {SGI163711, SGI0, SGI163713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163715, SGI163716, SGI163716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36148[] = {SGI163717, SGI0, SGI163718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163719, SGI163720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36149[] = {SGI163721, SGI0, SGI163722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163723, SGI163724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36150[] = {SGI163725, SGI0, SGI163726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163728, SGI163730, SGI163730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36151[] = {SGI163727, SGI0, SGI163729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163731, SGI163732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36152[] = {SGI163733, SGI163734, SGI163735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163736, SGI0, SGI0, SGI0, SGI163737, SGI163737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36153[] = {SGI163738, SGI0, SGI163739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163740, SGI163741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36154[] = {SGI163742, SGI0, SGI163743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36155[] = {SGI163744, SGI0, SGI163745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163746, SGI163747, SGI163747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36156[] = {SGI163748, SGI0, SGI163750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163752, SGI163753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36157[] = {SGI163749, SGI0, SGI163751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163754, SGI163755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36158[] = {SGI163756, SGI0, SGI163757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163758, SGI163759, SGI163759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36159[] = {SGI163760, SGI0, SGI163761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163762, SGI163763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36160[] = {SGI163764, SGI0, SGI163765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163766, SGI163767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36161[] = {SGI163768, SGI0, SGI163769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163770, SGI163771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36162[] = {SGI163772, SGI0, SGI163773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163774, SGI163775, SGI163775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163776, SGI0, SGI0, NULL};
const Segment * const SG36163[] = {SGI163777, SGI0, SGI163778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163781, SGI163782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36164[] = {SGI163779, SGI0, SGI163780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163783, SGI163784, SGI163784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163785, SGI0, SGI0, NULL};
const Segment * const SG36165[] = {SGI163786, SGI0, SGI163787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163788, SGI163789, SGI163789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163790, SGI0, SGI0, NULL};
const Segment * const SG36166[] = {SGI163791, SGI0, SGI163792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163795, SGI163796, SGI163796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163798, SGI0, SGI0, NULL};
const Segment * const SG36167[] = {SGI163793, SGI0, SGI163794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163797, SGI163799, SGI163799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36168[] = {SGI163800, SGI0, SGI163801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163802, SGI163803, SGI163803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163804, SGI0, SGI0, NULL};
const Segment * const SG36169[] = {SGI163805, SGI0, SGI163806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163807, SGI163808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36170[] = {SGI163809, SGI0, SGI163810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163813, SGI163814, SGI163814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163816, SGI0, SGI0, NULL};
const Segment * const SG36171[] = {SGI163811, SGI0, SGI163812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163815, SGI163817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36172[] = {SGI163818, SGI0, SGI163819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163822, SGI163823, SGI163823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163826, SGI0, SGI0, NULL};
const Segment * const SG36173[] = {SGI163820, SGI0, SGI163821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163824, SGI163825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36174[] = {SGI163827, SGI0, SGI163828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163831, SGI163832, SGI163832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163835, SGI0, SGI0, NULL};
const Segment * const SG36175[] = {SGI163829, SGI0, SGI163830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163833, SGI163834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36176[] = {SGI163836, SGI0, SGI163837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163838, SGI163839, SGI163839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163840, SGI0, SGI0, NULL};
const Segment * const SG36177[] = {SGI163841, SGI0, SGI163842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163843, SGI163844, SGI163844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36178[] = {SGI163845, SGI0, SGI163846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163847, SGI163848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36179[] = {SGI163849, SGI0, SGI163850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163851, SGI163852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36180[] = {SGI163853, SGI0, SGI163854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163855, SGI163856, SGI163856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36181[] = {SGI163857, SGI0, SGI163858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163859, SGI163860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36182[] = {SGI163861, SGI0, SGI163862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163863, SGI163864, SGI163864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36183[] = {SGI163865, SGI0, SGI163866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163867, SGI163868, SGI163868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36184[] = {SGI163869, SGI0, SGI163870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163871, SGI163872, SGI163872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36185[] = {SGI163873, SGI0, SGI163874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163875, SGI163876, SGI163876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36186[] = {SGI163877, SGI0, SGI163878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163879, SGI163880, SGI163880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36187[] = {SGI163881, SGI0, SGI163882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163885, SGI163886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36188[] = {SGI163883, SGI0, SGI163884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163887, SGI163888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36189[] = {SGI163889, SGI0, SGI163890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163891, SGI163892, SGI163892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163893, SGI0, SGI0, NULL};
const Segment * const SG36190[] = {SGI163894, SGI0, SGI163895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163896, SGI163897, SGI163897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163898, SGI0, SGI0, NULL};
const Segment * const SG36191[] = {SGI163899, SGI0, SGI163900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163901, SGI163902, SGI163902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163903, SGI0, SGI0, NULL};
const Segment * const SG36192[] = {SGI163904, SGI0, SGI163905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163906, SGI163907, SGI163907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163908, SGI0, SGI0, NULL};
const Segment * const SG36193[] = {SGI163909, SGI0, SGI163910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163911, SGI163912, SGI163912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163913, SGI0, SGI0, NULL};
const Segment * const SG36194[] = {SGI163914, SGI163915, SGI163916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163917, SGI0, SGI0, SGI0, SGI163918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36195[] = {SGI163919, SGI0, SGI163920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163921, SGI163922, SGI163922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163923, SGI0, SGI0, NULL};
const Segment * const SG36196[] = {SGI163924, SGI0, SGI163925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36197[] = {SGI163926, SGI0, SGI163927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163928, SGI163929, SGI163929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163930, SGI0, SGI0, NULL};
const Segment * const SG36198[] = {SGI163931, SGI0, SGI163932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163933, SGI163934, SGI163934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163935, SGI0, SGI0, NULL};
const Segment * const SG36199[] = {SGI163936, SGI0, SGI163937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163938, SGI163939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36200[] = {SGI163940, SGI0, SGI163941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163942, SGI163943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36201[] = {SGI163944, SGI163945, SGI163946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163947, SGI0, SGI0, SGI0, SGI163948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36202[] = {SGI163949, SGI163950, SGI163951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163952, SGI0, SGI0, SGI0, SGI163953, SGI163953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36203[] = {SGI163954, SGI163955, SGI163956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163957, SGI0, SGI0, SGI0, SGI163958, SGI163958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36204[] = {SGI163959, SGI163960, SGI163961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163962, SGI0, SGI0, SGI0, SGI163963, SGI163963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36205[] = {SGI163964, SGI0, SGI163965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36206[] = {SGI163966, SGI163967, SGI163968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163969, SGI0, SGI0, SGI0, SGI163970, SGI163970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36207[] = {SGI163971, SGI163972, SGI163973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163974, SGI0, SGI0, SGI0, SGI163976, SGI163976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36208[] = {SGI163975, SGI0, SGI163977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36209[] = {SGI163980, SGI0, SGI163982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36210[] = {SGI163978, SGI163979, SGI163981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163983, SGI0, SGI0, SGI0, SGI163984, SGI163984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36211[] = {SGI163985, SGI163986, SGI163987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163988, SGI0, SGI0, SGI0, SGI163991, SGI163991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36212[] = {SGI163989, SGI163990, SGI163992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163993, SGI0, SGI0, SGI0, SGI163994, SGI163994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36213[] = {SGI163995, SGI163996, SGI163997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI163998, SGI0, SGI0, SGI0, SGI163999, SGI163999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36214[] = {SGI164000, SGI164001, SGI164002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164003, SGI0, SGI0, SGI0, SGI164004, SGI164004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36215[] = {SGI164005, SGI164006, SGI164007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164008, SGI0, SGI0, SGI0, SGI164009, SGI164009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36216[] = {SGI164011, SGI164013, SGI164014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164015, SGI0, SGI0, SGI0, SGI164017, SGI164017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36217[] = {SGI164010, SGI0, SGI164012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164016, SGI164018, SGI164018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164019, SGI0, SGI0, NULL};
const Segment * const SG36218[] = {SGI164020, SGI164021, SGI164022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164023, SGI0, SGI0, SGI0, SGI164024, SGI164024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36219[] = {SGI164025, SGI164026, SGI164027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164028, SGI0, SGI0, SGI0, SGI164029, SGI164029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36220[] = {SGI164030, SGI0, SGI164031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164032, SGI164033, SGI164033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164034, SGI0, SGI0, NULL};
const Segment * const SG36221[] = {SGI164035, SGI0, SGI164036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164037, SGI164038, SGI164038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164039, SGI0, SGI0, NULL};
const Segment * const SG36222[] = {SGI164040, SGI164041, SGI164042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164043, SGI0, SGI0, SGI0, SGI164044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36223[] = {SGI164045, SGI164046, SGI164047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164048, SGI0, SGI0, SGI0, SGI164049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36224[] = {SGI164050, SGI164051, SGI164052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164053, SGI0, SGI0, SGI0, SGI164054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36225[] = {SGI164055, SGI164056, SGI164057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164058, SGI0, SGI0, SGI0, SGI164059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36226[] = {SGI164060, SGI164061, SGI164062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164063, SGI0, SGI0, SGI0, SGI164064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36227[] = {SGI164065, SGI164066, SGI164067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164068, SGI0, SGI0, SGI0, SGI164069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36228[] = {SGI164070, SGI164071, SGI164072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164073, SGI0, SGI0, SGI0, SGI164074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36229[] = {SGI164075, SGI164076, SGI164077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164078, SGI0, SGI0, SGI0, SGI164079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36230[] = {SGI164080, SGI0, SGI164081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164082, SGI164083, SGI164083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36231[] = {SGI164084, SGI0, SGI164085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164086, SGI164087, SGI164087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36232[] = {SGI164088, SGI0, SGI164089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164090, SGI164091, SGI164091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36233[] = {SGI164092, SGI164093, SGI164094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164095, SGI0, SGI0, SGI0, SGI164096, SGI164096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36234[] = {SGI164097, SGI164098, SGI164099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164100, SGI0, SGI0, SGI0, SGI164101, SGI164101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36235[] = {SGI164102, SGI164103, SGI164104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164105, SGI0, SGI0, SGI0, SGI164106, SGI164106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36236[] = {SGI164107, SGI164108, SGI164109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164110, SGI0, SGI0, SGI0, SGI164111, SGI164111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36237[] = {SGI164112, SGI164113, SGI164114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164115, SGI0, SGI0, SGI0, SGI164116, SGI164116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36238[] = {SGI164117, SGI0, SGI164118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164119, SGI164120, SGI164120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36239[] = {SGI164121, SGI164122, SGI164123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164124, SGI0, SGI0, SGI0, SGI164125, SGI164125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36240[] = {SGI164126, SGI0, SGI164127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164128, SGI164129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36241[] = {SGI164130, SGI0, SGI164131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164132, SGI164133, SGI164133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36242[] = {SGI164134, SGI0, SGI164135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164136, SGI164137, SGI164137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36243[] = {SGI164138, SGI164139, SGI164140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164141, SGI0, SGI0, SGI0, SGI164142, SGI164142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36244[] = {SGI164143, SGI0, SGI164144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36245[] = {SGI164145, SGI0, SGI164146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164147, SGI0, SGI0, SGI0, SGI164148, SGI164148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36246[] = {SGI164149, SGI164150, SGI164151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164152, SGI0, SGI0, SGI0, SGI164153, SGI164153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36247[] = {SGI164154, SGI0, SGI164155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36248[] = {SGI164156, SGI0, SGI164157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164158, SGI164159, SGI164159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36249[] = {SGI164160, SGI0, SGI164161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164162, SGI164163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36250[] = {SGI164164, SGI164165, SGI164166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164167, SGI0, SGI0, SGI0, SGI164168, SGI164168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36251[] = {SGI164169, SGI0, SGI164170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36252[] = {SGI164171, SGI0, SGI164172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164173, SGI164174, SGI164174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36253[] = {SGI164175, SGI0, SGI164176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164177, SGI164178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36254[] = {SGI164179, SGI0, SGI164180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164181, SGI164182, SGI164182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36255[] = {SGI164183, SGI164184, SGI164185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164186, SGI0, SGI0, SGI0, SGI164187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36256[] = {SGI164188, SGI0, SGI164189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164190, SGI164191, SGI164191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36257[] = {SGI164192, SGI0, SGI164193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164194, SGI164195, SGI164195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36258[] = {SGI164196, SGI0, SGI164197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164198, SGI164199, SGI164199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36259[] = {SGI164200, SGI0, SGI164201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164202, SGI164203, SGI164203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36260[] = {SGI164204, SGI0, SGI164205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164206, SGI164207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36261[] = {SGI164208, SGI0, SGI164209, SGI0, SGI164210, SGI164210, SGI0, NULL};
const Segment * const SG36262[] = {SGI164211, SGI0, SGI164212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164213, SGI164214, SGI164214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36263[] = {SGI164215, SGI0, SGI164216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164217, SGI164218, SGI164218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36264[] = {SGI164219, SGI0, SGI164220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164221, SGI164222, SGI164222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36265[] = {SGI164223, SGI0, SGI164224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164225, SGI164226, SGI164226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36266[] = {SGI164227, SGI0, SGI164228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164229, SGI164230, SGI164230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36267[] = {SGI164231, SGI0, SGI164232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164235, SGI164236, SGI164236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36268[] = {SGI164233, SGI0, SGI164234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164237, SGI164238, SGI164238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36269[] = {SGI164239, SGI0, SGI164240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164241, SGI164242, SGI164242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36270[] = {SGI164243, SGI164244, SGI164245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164246, SGI0, SGI0, SGI0, SGI164247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36271[] = {SGI164248, SGI164249, SGI164250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164251, SGI0, SGI0, SGI0, SGI164252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36272[] = {SGI164253, SGI0, SGI164254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36273[] = {SGI164255, SGI0, SGI164256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164257, SGI164258, SGI164258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36274[] = {SGI164259, SGI0, SGI164260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36275[] = {SGI164261, SGI0, SGI164262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36276[] = {SGI164263, SGI164264, SGI164265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164266, SGI0, SGI0, SGI0, SGI164267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36277[] = {SGI164268, SGI0, SGI164269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164270, SGI164271, SGI164271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36278[] = {SGI164272, SGI0, SGI164273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164274, SGI164275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36279[] = {SGI164276, SGI0, SGI164277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164278, SGI164279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36280[] = {SGI164280, SGI0, SGI164281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164282, SGI164283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36281[] = {SGI164284, SGI0, SGI164285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164286, SGI164287, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36282[] = {SGI164288, SGI0, SGI164290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164292, SGI164294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36283[] = {SGI164289, SGI0, SGI164291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164293, SGI164295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36284[] = {SGI164296, SGI164297, SGI164298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164301, SGI164302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36285[] = {SGI164299, SGI0, SGI164300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164303, SGI164304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36286[] = {SGI164305, SGI0, SGI164306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164307, SGI164308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36287[] = {SGI164309, SGI164310, SGI164311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164312, SGI0, SGI0, SGI0, SGI164313, SGI164313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36288[] = {SGI164315, SGI164316, SGI164317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164318, SGI0, SGI0, SGI0, SGI164319, SGI164319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36289[] = {SGI164323, SGI0, SGI164325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164327, SGI164329, SGI164329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36290[] = {SGI164330, SGI0, SGI164331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164332, SGI164333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36291[] = {SGI164334, SGI0, SGI164335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164336, SGI164337, SGI164337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36292[] = {SGI164338, SGI0, SGI164339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164340, SGI164341, SGI164341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36293[] = {SGI164342, SGI0, SGI164343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164344, SGI164345, SGI164345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36294[] = {SGI164346, SGI0, SGI164347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164348, SGI164349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36295[] = {SGI164350, SGI164351, SGI164352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164353, SGI0, SGI0, SGI0, SGI164354, SGI164354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36296[] = {SGI164355, SGI0, SGI164356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36297[] = {SGI164357, SGI164358, SGI164359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164360, SGI0, SGI0, SGI0, SGI164361, SGI0, SGI0, SGI164362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36298[] = {SGI164363, SGI164364, SGI164365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164366, SGI0, SGI0, SGI0, SGI164367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36299[] = {SGI164368, SGI164369, SGI164370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164371, SGI0, SGI0, SGI0, SGI164372, SGI164372, SGI0, SGI164373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36300[] = {SGI164374, SGI0, SGI164375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164376, SGI164377, SGI164377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36301[] = {SGI164378, SGI0, SGI164379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164382, SGI164383, SGI164383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36302[] = {SGI164380, SGI0, SGI164381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164384, SGI164385, SGI164385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36303[] = {SGI164386, SGI0, SGI164387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164388, SGI164389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36304[] = {SGI164390, SGI0, SGI164391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164392, SGI164393, SGI164393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36305[] = {SGI164394, SGI0, SGI164395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164396, SGI164397, SGI164397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36306[] = {SGI164398, SGI0, SGI164399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164400, SGI164401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36307[] = {SGI164402, SGI164403, SGI164404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164405, SGI0, SGI0, SGI0, SGI164406, SGI164406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36308[] = {SGI164407, SGI164408, SGI164409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164410, SGI0, SGI0, SGI0, SGI164411, SGI164411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36309[] = {SGI164412, SGI0, SGI164413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36310[] = {SGI164414, SGI0, SGI164415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164416, SGI164417, SGI164417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36311[] = {SGI164321, SGI164322, SGI164324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164326, SGI0, SGI0, SGI0, SGI164328, SGI164328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36312[] = {SGI164418, SGI0, SGI164419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164420, SGI164421, SGI164421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36313[] = {SGI164423, SGI0, SGI164424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164425, SGI164426, SGI164426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36314[] = {SGI164427, SGI0, SGI164428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164429, SGI164430, SGI164430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36315[] = {SGI164431, SGI0, SGI164432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164433, SGI164434, SGI164434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36316[] = {SGI164435, SGI0, SGI164436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164437, SGI164438, SGI164438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36317[] = {SGI164439, SGI164440, SGI164441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164442, SGI0, SGI0, SGI0, SGI164443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36318[] = {SGI164445, SGI0, SGI164447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164450, SGI164452, SGI164452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36319[] = {SGI164446, SGI164448, SGI164449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164451, SGI0, SGI0, SGI0, SGI164453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36320[] = {SGI164455, SGI164456, SGI164457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164458, SGI0, SGI0, SGI0, SGI164459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36321[] = {SGI164461, SGI164462, SGI164463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164467, SGI0, SGI0, SGI0, SGI164468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36322[] = {SGI164464, SGI164465, SGI164466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164469, SGI0, SGI0, SGI0, SGI164471, SGI0, SGI0, SGI164472, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36323[] = {SGI164475, SGI164477, SGI164478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164480, SGI0, SGI0, SGI0, SGI164482, SGI0, SGI0, SGI164484, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36324[] = {SGI164473, SGI164474, SGI164476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164479, SGI0, SGI0, SGI0, SGI164481, SGI164481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36325[] = {SGI164485, SGI164486, SGI164487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164488, SGI0, SGI0, SGI0, SGI164489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36326[] = {SGI164491, SGI164492, SGI164493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164494, SGI0, SGI0, SGI0, SGI164495, SGI164495, SGI0, SGI164496, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36327[] = {SGI164497, SGI164498, SGI164499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164500, SGI0, SGI0, SGI0, SGI164501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36328[] = {SGI164503, SGI164504, SGI164505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164506, SGI0, SGI0, SGI0, SGI164507, SGI164507, SGI0, SGI164508, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36329[] = {SGI164509, SGI164510, SGI164511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164512, SGI0, SGI0, SGI0, SGI164513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36330[] = {SGI164515, SGI164516, SGI164517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164518, SGI0, SGI0, SGI0, SGI164519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36331[] = {SGI164520, SGI164521, SGI164522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164523, SGI0, SGI0, SGI0, SGI164524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36332[] = {SGI164525, SGI164526, SGI164527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164528, SGI0, SGI0, SGI0, SGI164529, SGI164529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36333[] = {SGI164530, SGI0, SGI164531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36334[] = {SGI164532, SGI164533, SGI164534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164535, SGI0, SGI0, SGI0, SGI164536, SGI164536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36335[] = {SGI164537, SGI0, SGI164538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164539, SGI0, SGI0, SGI0, SGI164540, SGI164540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36336[] = {SGI164541, SGI164542, SGI164543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164544, SGI0, SGI0, SGI0, SGI164545, SGI164545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36337[] = {SGI164546, SGI164547, SGI164548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164549, SGI0, SGI0, SGI0, SGI164550, SGI164550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36338[] = {SGI164551, SGI0, SGI164552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36339[] = {SGI164553, SGI164554, SGI164555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164558, SGI0, SGI0, SGI0, SGI164559, SGI164559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36340[] = {SGI164556, SGI0, SGI164557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164560, SGI164561, SGI164561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36341[] = {SGI164562, SGI164563, SGI164564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164566, SGI0, SGI0, SGI0, SGI164568, SGI164568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36342[] = {SGI164565, SGI0, SGI164567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164569, SGI164570, SGI164570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36343[] = {SGI164571, SGI164572, SGI164573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164574, SGI0, SGI0, SGI0, SGI164575, SGI164575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36344[] = {SGI164576, SGI0, SGI164577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36345[] = {SGI164578, SGI164579, SGI164580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164581, SGI0, SGI0, SGI0, SGI164582, SGI164582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36346[] = {SGI164583, SGI0, SGI164584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36347[] = {SGI164585, SGI164586, SGI164587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164588, SGI0, SGI0, SGI0, SGI164589, SGI164589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36348[] = {SGI164590, SGI164591, SGI164593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36349[] = {SGI164592, SGI164594, SGI164595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164596, SGI0, SGI0, SGI0, SGI164597, SGI164597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36350[] = {SGI164598, SGI164599, SGI164600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164601, SGI0, SGI0, SGI0, SGI164602, SGI164602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36351[] = {SGI164603, SGI0, SGI164604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164605, SGI164606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36352[] = {SGI164607, SGI0, SGI164608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164609, SGI164610, SGI164610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36353[] = {SGI164611, SGI0, SGI164612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164613, SGI164614, SGI164614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36354[] = {SGI164615, SGI0, SGI164616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164617, SGI164618, SGI164618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36355[] = {SGI164619, SGI0, SGI164620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164621, SGI164622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36356[] = {SGI164623, SGI0, SGI164624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164625, SGI164626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36357[] = {SGI164627, SGI0, SGI164628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164629, SGI164630, SGI164630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36358[] = {SGI164631, SGI0, SGI164632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164633, SGI164634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36359[] = {SGI164635, SGI0, SGI164636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164637, SGI164638, SGI164638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36360[] = {SGI164639, SGI0, SGI164640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164641, SGI164642, SGI164642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36361[] = {SGI164643, SGI0, SGI164644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164645, SGI164646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36362[] = {SGI164647, SGI0, SGI164648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164649, SGI164650, SGI164650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36363[] = {SGI164651, SGI0, SGI164652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164653, SGI164654, SGI164654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36364[] = {SGI164655, SGI164656, SGI164657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164658, SGI0, SGI0, SGI0, SGI164659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36365[] = {SGI164662, SGI0, SGI164663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36366[] = {SGI164660, SGI0, SGI164661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164664, SGI164665, SGI164665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36367[] = {SGI164666, SGI0, SGI164667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164668, SGI164669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36368[] = {SGI164670, SGI0, SGI164671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164672, SGI164673, SGI164673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36369[] = {SGI164674, SGI0, SGI164675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164676, SGI164677, SGI164677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36370[] = {SGI164678, SGI0, SGI164679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164680, SGI164681, SGI164681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36371[] = {SGI164682, SGI164683, SGI164684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164685, SGI0, SGI0, SGI0, SGI164686, SGI164686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36372[] = {SGI164687, SGI0, SGI164688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36373[] = {SGI164689, SGI164690, SGI164691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164692, SGI0, SGI0, SGI0, SGI164693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36374[] = {SGI164694, SGI164695, SGI164696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164697, SGI0, SGI0, SGI0, SGI164698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36375[] = {SGI164699, SGI164700, SGI164701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164702, SGI0, SGI0, SGI0, SGI164703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36376[] = {SGI164704, SGI164705, SGI164706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164707, SGI0, SGI0, SGI0, SGI164708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36377[] = {SGI164709, SGI164710, SGI164711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164712, SGI0, SGI0, SGI0, SGI164713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36378[] = {SGI164714, SGI0, SGI164715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164716, SGI164717, SGI164717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36379[] = {SGI164718, SGI0, SGI164719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164720, SGI164721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36380[] = {SGI164722, SGI0, SGI164723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164724, SGI164725, SGI164725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36381[] = {SGI164726, SGI0, SGI164727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164728, SGI164729, SGI164729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36382[] = {SGI164730, SGI0, SGI164731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164732, SGI164733, SGI164733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36383[] = {SGI164734, SGI0, SGI164735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164736, SGI164737, SGI164737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36384[] = {SGI164738, SGI0, SGI164739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36385[] = {SGI164740, SGI164741, SGI164742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164743, SGI0, SGI0, SGI0, SGI164744, SGI164744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36386[] = {SGI164745, SGI164746, SGI164747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164751, SGI0, SGI0, SGI0, SGI164752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36387[] = {SGI164748, SGI164749, SGI164750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164753, SGI0, SGI0, SGI0, SGI164754, SGI164754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36388[] = {SGI164755, SGI0, SGI164756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36389[] = {SGI164757, SGI164758, SGI164759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164760, SGI0, SGI0, SGI0, SGI164761, SGI164761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36390[] = {SGI164762, SGI164763, SGI164764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164765, SGI0, SGI0, SGI0, SGI164766, SGI164766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36391[] = {SGI164767, SGI164768, SGI164769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164770, SGI0, SGI0, SGI0, SGI164771, SGI164771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36392[] = {SGI164772, SGI164773, SGI164774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164775, SGI0, SGI0, SGI0, SGI164777, SGI164777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36393[] = {SGI164776, SGI0, SGI164778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164779, SGI164780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36394[] = {SGI164781, SGI0, SGI164782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36395[] = {SGI164783, SGI164784, SGI164785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164786, SGI0, SGI0, SGI0, SGI164787, SGI164787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36396[] = {SGI164788, SGI164789, SGI164790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164791, SGI0, SGI0, SGI0, SGI164792, SGI164792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36397[] = {SGI164793, SGI164794, SGI164795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164796, SGI0, SGI0, SGI0, SGI164797, SGI164797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36398[] = {SGI164798, SGI0, SGI164799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164801, SGI164802, SGI164802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36399[] = {SGI164800, SGI0, SGI164803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164804, SGI164806, SGI164806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36400[] = {SGI164805, SGI164807, SGI164808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164809, SGI0, SGI0, SGI0, SGI164810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36401[] = {SGI164811, SGI0, SGI164812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164814, SGI164816, SGI164816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36402[] = {SGI164813, SGI0, SGI164815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164817, SGI164818, SGI164818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36403[] = {SGI164819, SGI164820, SGI164821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164822, SGI0, SGI0, SGI0, SGI164823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36404[] = {SGI164826, SGI0, SGI164827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36405[] = {SGI164824, SGI0, SGI164825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164828, SGI164829, SGI164829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36406[] = {SGI164830, SGI0, SGI164831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164832, SGI164833, SGI164833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36407[] = {SGI164834, SGI164835, SGI164836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164837, SGI0, SGI0, SGI0, SGI164839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36408[] = {SGI164838, SGI0, SGI164840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164843, SGI164844, SGI164844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36409[] = {SGI164841, SGI0, SGI164842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164845, SGI164846, SGI164846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36410[] = {SGI164847, SGI0, SGI164848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164850, SGI164852, SGI164852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36411[] = {SGI164849, SGI0, SGI164851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164853, SGI164854, SGI164854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36412[] = {SGI164855, SGI0, SGI164856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164859, SGI164860, SGI164860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36413[] = {SGI164857, SGI0, SGI164858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164861, SGI164862, SGI164862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36414[] = {SGI164863, SGI164864, SGI164865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164866, SGI0, SGI0, SGI0, SGI164867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36415[] = {SGI164868, SGI0, SGI164869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164870, SGI164871, SGI164871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36416[] = {SGI164872, SGI0, SGI164873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164874, SGI164875, SGI164875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36417[] = {SGI164876, SGI0, SGI164877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164881, SGI164882, SGI164882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36418[] = {SGI164878, SGI164879, SGI164880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164883, SGI0, SGI0, SGI0, SGI164884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36419[] = {SGI164885, SGI0, SGI164886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164887, SGI164889, SGI164889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36420[] = {SGI164888, SGI164890, SGI164891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164892, SGI0, SGI0, SGI0, SGI164893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36421[] = {SGI164894, SGI0, SGI164895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164896, SGI164897, SGI164897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36422[] = {SGI164898, SGI0, SGI164899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164900, SGI164901, SGI164901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36423[] = {SGI164902, SGI0, SGI164903, SGI0, SGI164904, SGI164904, SGI0, NULL};
const Segment * const SG36424[] = {SGI164905, SGI0, SGI164906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164909, SGI164910, SGI164910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36425[] = {SGI164907, SGI0, SGI164908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164911, SGI164912, SGI164912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36426[] = {SGI164915, SGI164916, SGI164917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164918, SGI0, SGI0, SGI0, SGI164919, SGI164919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36427[] = {SGI164920, SGI0, SGI164921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36428[] = {SGI164922, SGI0, SGI164923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164924, SGI164925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36429[] = {SGI164926, SGI164927, SGI164928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164929, SGI0, SGI0, SGI0, SGI164930, SGI164930, SGI164931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36430[] = {SGI164932, SGI164933, SGI164934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164935, SGI0, SGI0, SGI0, SGI164936, SGI164936, SGI164937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36431[] = {SGI0, SGI164938, SGI164939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164940, SGI0, SGI0, SGI0, SGI164941, SGI164941, SGI0, SGI0, NULL};
const Segment * const SG36432[] = {SGI0, SGI164942, SGI164943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164944, SGI0, SGI0, SGI0, SGI164945, SGI164945, SGI0, SGI0, NULL};
const Segment * const SG36433[] = {SGI164946, SGI0, SGI164947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164948, SGI164949, SGI164949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36434[] = {SGI164950, SGI0, SGI164951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164952, SGI164954, SGI164954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36435[] = {SGI164953, SGI0, SGI164955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164956, SGI164957, SGI164957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36436[] = {SGI164958, SGI0, SGI164959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164961, SGI164962, SGI164962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36437[] = {SGI164960, SGI0, SGI164963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164965, SGI164967, SGI164967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36438[] = {SGI164964, SGI0, SGI164966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164968, SGI164969, SGI164969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36439[] = {SGI164970, SGI0, SGI164971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164972, SGI164973, SGI164973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36440[] = {SGI164974, SGI0, SGI164975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164976, SGI164977, SGI164977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36441[] = {SGI164978, SGI0, SGI164979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164980, SGI164981, SGI164981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36442[] = {SGI164982, SGI0, SGI164983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164984, SGI164985, SGI164985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36443[] = {SGI164986, SGI0, SGI164987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164988, SGI164989, SGI164989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36444[] = {SGI164990, SGI0, SGI164991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164992, SGI164993, SGI164993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36445[] = {SGI164994, SGI0, SGI164995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI164996, SGI164997, SGI164997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36446[] = {SGI164998, SGI0, SGI164999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165000, SGI165001, SGI165001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36447[] = {SGI165002, SGI0, SGI165003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165004, SGI165005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36448[] = {SGI165006, SGI0, SGI165007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165008, SGI165009, SGI165009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36449[] = {SGI165010, SGI0, SGI165011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165012, SGI165013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36450[] = {SGI165014, SGI0, SGI165015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165016, SGI165017, SGI165017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36451[] = {SGI165018, SGI0, SGI165019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165020, SGI165021, SGI165021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36452[] = {SGI165022, SGI0, SGI165023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165024, SGI165025, SGI165025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36453[] = {SGI165029, SGI0, SGI165030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165032, SGI165034, SGI165034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36454[] = {SGI165026, SGI165027, SGI165028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165031, SGI0, SGI0, SGI0, SGI165033, SGI165033, SGI165035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36455[] = {SGI165036, SGI0, SGI165037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165041, SGI165042, SGI165042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36456[] = {SGI165038, SGI165039, SGI165040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165043, SGI0, SGI0, SGI0, SGI165044, SGI165044, SGI165045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36457[] = {SGI165046, SGI0, SGI165047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165048, SGI165049, SGI165049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36458[] = {SGI165050, SGI165051, SGI165052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165053, SGI0, SGI0, SGI0, SGI165054, SGI165054, SGI165055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36459[] = {SGI165056, SGI0, SGI165057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165058, SGI165059, SGI165059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36460[] = {SGI165060, SGI165061, SGI165062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165064, SGI0, SGI0, SGI0, SGI165066, SGI165066, SGI165067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36461[] = {SGI165063, SGI0, SGI165065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165068, SGI165069, SGI165069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36462[] = {SGI165070, SGI0, SGI165071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165072, SGI165073, SGI165073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36463[] = {SGI165074, SGI0, SGI165075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165076, SGI165077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36464[] = {SGI165078, SGI0, SGI165079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165080, SGI165081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36465[] = {SGI165082, SGI0, SGI165083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165084, SGI165085, SGI165085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36466[] = {SGI165086, SGI0, SGI165087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165088, SGI165089, SGI165089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36467[] = {SGI165090, SGI0, SGI165091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165092, SGI165093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36468[] = {SGI165094, SGI0, SGI165095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165096, SGI165097, SGI165097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36469[] = {SGI165098, SGI0, SGI165099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165100, SGI165101, SGI165101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36470[] = {SGI165102, SGI0, SGI165103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165104, SGI165105, SGI165105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36471[] = {SGI165106, SGI0, SGI165107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165108, SGI165109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36472[] = {SGI165110, SGI165111, SGI165112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165113, SGI0, SGI0, SGI0, SGI165114, SGI165114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36473[] = {SGI164913, SGI0, SGI164914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165115, SGI165116, SGI165116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36474[] = {SGI165117, SGI0, SGI165118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165119, SGI165120, SGI165120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36475[] = {SGI165121, SGI0, SGI165122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165123, SGI165124, SGI165124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36476[] = {SGI165125, SGI0, SGI165126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165127, SGI165128, SGI165128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36477[] = {SGI165129, SGI165130, SGI165131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165132, SGI0, SGI0, SGI0, SGI165133, SGI165133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36478[] = {SGI165134, SGI0, SGI165135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36479[] = {SGI165136, SGI165137, SGI165138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165139, SGI0, SGI0, SGI0, SGI165140, SGI165140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36480[] = {SGI165141, SGI0, SGI165142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165143, SGI165144, SGI165144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36481[] = {SGI165145, SGI165146, SGI165147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165148, SGI0, SGI0, SGI0, SGI165149, SGI165149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36482[] = {SGI165150, SGI165151, SGI165152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165153, SGI0, SGI0, SGI0, SGI165154, SGI165154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36483[] = {SGI165155, SGI165156, SGI165157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165158, SGI0, SGI0, SGI0, SGI165159, SGI165159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36484[] = {SGI165160, SGI165161, SGI165162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165163, SGI0, SGI0, SGI0, SGI165164, SGI165164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36485[] = {SGI165165, SGI0, SGI165166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165167, SGI165168, SGI165168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36486[] = {SGI165169, SGI0, SGI165170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165171, SGI165172, SGI165172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36487[] = {SGI165173, SGI0, SGI165174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165175, SGI165176, SGI165176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36488[] = {SGI165177, SGI0, SGI165178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165179, SGI165180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36489[] = {SGI165181, SGI0, SGI165182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165183, SGI165184, SGI165184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36490[] = {SGI165185, SGI0, SGI165186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165187, SGI165188, SGI165188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36491[] = {SGI165189, SGI0, SGI165190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165191, SGI165192, SGI165192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36492[] = {SGI165193, SGI0, SGI165194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165195, SGI165196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36493[] = {SGI165197, SGI0, SGI165198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165199, SGI165200, SGI165200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36494[] = {SGI165201, SGI0, SGI165202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165203, SGI165204, SGI165204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36495[] = {SGI165205, SGI0, SGI165206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165207, SGI165208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36496[] = {SGI165209, SGI0, SGI165210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165211, SGI165212, SGI165212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36497[] = {SGI165213, SGI0, SGI165214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165217, SGI165218, SGI165218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36498[] = {SGI165215, SGI0, SGI165216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165219, SGI165220, SGI165220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36499[] = {SGI165221, SGI0, SGI165222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165223, SGI165224, SGI165224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36500[] = {SGI165225, SGI0, SGI165226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165227, SGI165228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36501[] = {SGI165229, SGI0, SGI165230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165231, SGI165232, SGI165232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36502[] = {SGI165233, SGI0, SGI165234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165237, SGI165239, SGI165239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36503[] = {SGI165235, SGI0, SGI165236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165238, SGI165240, SGI165240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36504[] = {SGI165241, SGI0, SGI165243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165245, SGI165246, SGI165246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36505[] = {SGI165242, SGI0, SGI165244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165247, SGI165248, SGI165248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36506[] = {SGI165249, SGI0, SGI165250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165251, SGI165252, SGI165252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36507[] = {SGI165253, SGI0, SGI165254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165255, SGI165257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36508[] = {SGI165256, SGI165258, SGI165259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165260, SGI0, SGI0, SGI0, SGI165261, SGI165261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36509[] = {SGI165263, SGI0, SGI165264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165265, SGI165266, SGI165266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36510[] = {SGI165267, SGI165268, SGI165269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165270, SGI0, SGI0, SGI0, SGI165271, SGI165271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36511[] = {SGI165273, SGI0, SGI165274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165275, SGI165276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36512[] = {SGI165277, SGI165278, SGI165279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165280, SGI0, SGI0, SGI0, SGI165281, SGI165281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36513[] = {SGI165282, SGI0, SGI165284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165285, SGI165286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36514[] = {SGI165287, SGI0, SGI165288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165289, SGI165291, SGI165291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36515[] = {SGI165290, SGI0, SGI165292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165293, SGI165294, SGI165294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36516[] = {SGI165295, SGI0, SGI165296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165297, SGI165298, SGI165298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36517[] = {SGI165299, SGI0, SGI165300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165301, SGI165302, SGI165302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36518[] = {SGI165303, SGI0, SGI165304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165305, SGI165306, SGI165306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36519[] = {SGI165307, SGI0, SGI165308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165309, SGI165310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36520[] = {SGI165311, SGI0, SGI165312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165313, SGI165314, SGI165314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36521[] = {SGI165315, SGI0, SGI165316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165317, SGI165318, SGI165318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36522[] = {SGI165319, SGI0, SGI165320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165321, SGI165322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36523[] = {SGI165323, SGI0, SGI165324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165325, SGI165326, SGI165326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36524[] = {SGI165327, SGI0, SGI165328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165329, SGI165330, SGI165330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36525[] = {SGI165331, SGI0, SGI165332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165333, SGI165334, SGI165334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36526[] = {SGI165335, SGI165336, SGI165337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165341, SGI0, SGI0, SGI0, SGI165343, SGI165343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36527[] = {SGI165338, SGI165339, SGI165340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165342, SGI0, SGI0, SGI0, SGI165344, SGI165344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36528[] = {SGI165347, SGI165349, SGI165351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165352, SGI0, SGI0, SGI0, SGI165354, SGI165354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36529[] = {SGI165346, SGI165348, SGI165350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165353, SGI0, SGI0, SGI0, SGI165355, SGI165355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36530[] = {SGI165357, SGI165358, SGI165359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165362, SGI0, SGI0, SGI0, SGI165364, SGI165364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36531[] = {SGI165360, SGI165361, SGI165363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165365, SGI0, SGI0, SGI0, SGI165366, SGI165366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36532[] = {SGI165368, SGI165369, SGI165370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165371, SGI0, SGI0, SGI0, SGI165372, SGI165372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36533[] = {SGI165373, SGI165374, SGI165376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165379, SGI0, SGI0, SGI0, SGI165381, SGI165381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36534[] = {SGI165375, SGI165377, SGI165378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165380, SGI0, SGI0, SGI0, SGI165382, SGI165382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36535[] = {SGI165383, SGI0, SGI165385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36536[] = {SGI165384, SGI0, SGI165386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36537[] = {SGI165387, SGI0, SGI165389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165391, SGI0, SGI0, SGI0, SGI165392, SGI165392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36538[] = {SGI165388, SGI0, SGI165390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165393, SGI165394, SGI165394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36539[] = {SGI165395, SGI0, SGI165396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165397, SGI165398, SGI165398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36540[] = {SGI165399, SGI0, SGI165400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165401, SGI165402, SGI165402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36541[] = {SGI165403, SGI0, SGI165404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165405, SGI165406, SGI165406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36542[] = {SGI165407, SGI0, SGI165408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165409, SGI165410, SGI165410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36543[] = {SGI165411, SGI0, SGI165412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165413, SGI165414, SGI165414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36544[] = {SGI165415, SGI0, SGI165416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165417, SGI165418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36545[] = {SGI165419, SGI0, SGI165420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165421, SGI165422, SGI165422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36546[] = {SGI165423, SGI165424, SGI165425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165426, SGI0, SGI0, SGI0, SGI165427, SGI165427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36547[] = {SGI165428, SGI165429, SGI165430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165431, SGI0, SGI0, SGI0, SGI165432, SGI165432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36548[] = {SGI165433, SGI165434, SGI165435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165436, SGI0, SGI0, SGI0, SGI165437, SGI165437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36549[] = {SGI165438, SGI165439, SGI165440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165441, SGI0, SGI0, SGI0, SGI165442, SGI165442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36550[] = {SGI165443, SGI165444, SGI165445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165446, SGI0, SGI0, SGI0, SGI165447, SGI165447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36551[] = {SGI165448, SGI165449, SGI165450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165452, SGI0, SGI0, SGI0, SGI165455, SGI165455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36552[] = {SGI165451, SGI165453, SGI165454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165456, SGI0, SGI0, SGI0, SGI165457, SGI165457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36553[] = {SGI165461, SGI0, SGI165462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36554[] = {SGI165458, SGI165459, SGI165460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165463, SGI0, SGI0, SGI0, SGI165464, SGI165464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36555[] = {SGI165465, SGI165467, SGI165469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36556[] = {SGI165466, SGI165468, SGI165470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165471, SGI0, SGI0, SGI0, SGI165472, SGI165472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36557[] = {SGI165473, SGI165474, SGI165476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36558[] = {SGI165475, SGI165477, SGI165478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165479, SGI0, SGI0, SGI0, SGI165480, SGI165480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36559[] = {SGI165481, SGI165482, SGI165483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36560[] = {SGI165484, SGI165485, SGI165486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165487, SGI0, SGI0, SGI0, SGI165488, SGI165488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36561[] = {SGI165489, SGI165490, SGI165491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36562[] = {SGI165492, SGI0, SGI165493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165494, SGI165495, SGI165495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36563[] = {SGI165496, SGI165497, SGI165498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36564[] = {SGI165499, SGI165500, SGI165501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36565[] = {SGI165502, SGI165503, SGI165504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36566[] = {SGI165505, SGI165506, SGI165507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165508, SGI165508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36567[] = {SGI165509, SGI165510, SGI165511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165512, SGI0, SGI0, SGI0, SGI165513, SGI165513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36568[] = {SGI165514, SGI165515, SGI165516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165517, SGI165517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36569[] = {SGI165518, SGI165519, SGI165520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165521, SGI0, SGI0, SGI0, SGI165523, SGI165523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36570[] = {SGI165522, SGI0, SGI165524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165525, SGI165526, SGI165526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36571[] = {SGI165527, SGI165528, SGI165530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165532, SGI165532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36572[] = {SGI165529, SGI0, SGI165531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165533, SGI165534, SGI165534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36573[] = {SGI165535, SGI165536, SGI165538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36574[] = {SGI165537, SGI0, SGI165539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165540, SGI165541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36575[] = {SGI165542, SGI0, SGI165543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165544, SGI165545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36576[] = {SGI165546, SGI165547, SGI165548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165549, SGI0, SGI0, SGI0, SGI165550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36577[] = {SGI165551, SGI165552, SGI165553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165554, SGI0, SGI0, SGI0, SGI165555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36578[] = {SGI165556, SGI0, SGI165557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36579[] = {SGI165558, SGI0, SGI165559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165560, SGI165561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36580[] = {SGI165562, SGI0, SGI165563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165564, SGI165565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36581[] = {SGI165566, SGI0, SGI165567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165568, SGI165569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36582[] = {SGI165570, SGI0, SGI165571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165573, SGI165575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36583[] = {SGI165572, SGI0, SGI165574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165576, SGI165577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36584[] = {SGI165578, SGI0, SGI165579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165580, SGI165581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36585[] = {SGI165582, SGI0, SGI165583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165584, SGI165585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36586[] = {SGI165586, SGI165587, SGI165588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165589, SGI0, SGI0, SGI0, SGI165590, SGI165590, SGI165591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36587[] = {SGI165592, SGI165593, SGI165594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165595, SGI0, SGI0, SGI0, SGI165596, SGI165596, SGI165597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36588[] = {SGI165598, SGI165599, SGI165600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165601, SGI0, SGI0, SGI0, SGI165602, SGI165602, SGI165603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36589[] = {SGI165604, SGI0, SGI165605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165606, SGI165607, SGI165607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36590[] = {SGI165608, SGI0, SGI165609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165610, SGI165611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36591[] = {SGI165612, SGI165613, SGI165614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165615, SGI0, SGI0, SGI0, SGI165616, SGI165616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36592[] = {SGI165617, SGI0, SGI165618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36593[] = {SGI165619, SGI0, SGI165620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165621, SGI165622, SGI165622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36594[] = {SGI165623, SGI165624, SGI165625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165626, SGI0, SGI0, SGI0, SGI165627, SGI165627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36595[] = {SGI165628, SGI165629, SGI165630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165631, SGI0, SGI0, SGI0, SGI165632, SGI165632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36596[] = {SGI165633, SGI0, SGI165634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36597[] = {SGI165635, SGI165636, SGI165637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165638, SGI0, SGI0, SGI0, SGI165639, SGI165639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36598[] = {SGI165641, SGI165642, SGI165643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165644, SGI0, SGI0, SGI0, SGI165645, SGI165645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36599[] = {SGI165647, SGI165648, SGI165649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165650, SGI0, SGI0, SGI0, SGI165651, SGI165651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36600[] = {SGI165653, SGI165654, SGI165655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165656, SGI0, SGI0, SGI0, SGI165657, SGI165657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36601[] = {SGI165659, SGI0, SGI165660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165661, SGI165663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36602[] = {SGI165662, SGI165664, SGI165665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165666, SGI0, SGI0, SGI0, SGI165667, SGI165667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36603[] = {SGI165669, SGI0, SGI165670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165671, SGI165672, SGI165672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36604[] = {SGI165673, SGI0, SGI165674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165675, SGI165676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36605[] = {SGI165677, SGI0, SGI165678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165679, SGI165680, SGI165680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36606[] = {SGI165681, SGI0, SGI165682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165683, SGI165684, SGI165684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36607[] = {SGI165685, SGI0, SGI165686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165687, SGI165688, SGI165688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36608[] = {SGI165689, SGI0, SGI165690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165691, SGI165692, SGI165692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36609[] = {SGI165693, SGI0, SGI165694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165695, SGI165696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36610[] = {SGI165697, SGI0, SGI165698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165699, SGI165700, SGI165700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36611[] = {SGI165701, SGI0, SGI165702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165703, SGI165704, SGI165704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36612[] = {SGI165705, SGI0, SGI165706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165707, SGI165708, SGI165708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36613[] = {SGI165709, SGI0, SGI165710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165711, SGI165712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36614[] = {SGI165713, SGI0, SGI165714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165715, SGI165716, SGI165716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36615[] = {SGI165717, SGI165718, SGI165719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165720, SGI0, SGI0, SGI0, SGI165721, SGI165721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36616[] = {SGI165722, SGI0, SGI165723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165724, SGI0, SGI0, SGI0, SGI165725, SGI165725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36617[] = {SGI165726, SGI0, SGI165727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36618[] = {SGI165728, SGI0, SGI165729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165730, SGI165731, SGI165731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36619[] = {SGI165732, SGI0, SGI165733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165734, SGI165735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36620[] = {SGI165736, SGI165737, SGI165738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165739, SGI0, SGI0, SGI0, SGI165740, SGI165740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36621[] = {SGI165741, SGI165742, SGI165743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165744, SGI165744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36622[] = {SGI165745, SGI0, SGI165746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36623[] = {SGI165747, SGI165748, SGI165749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165750, SGI0, SGI0, SGI0, SGI165751, SGI165751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36624[] = {SGI165752, SGI165753, SGI165754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165755, SGI165755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36625[] = {SGI165756, SGI0, SGI165757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36626[] = {SGI165758, SGI165759, SGI165760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165761, SGI0, SGI0, SGI0, SGI165762, SGI165762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36627[] = {SGI165763, SGI165764, SGI165765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165766, SGI0, SGI0, SGI0, SGI165767, SGI165767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36628[] = {SGI165768, SGI165769, SGI165770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165771, SGI0, SGI0, SGI0, SGI165772, SGI165772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36629[] = {SGI165773, SGI0, SGI165774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36630[] = {SGI165775, SGI165776, SGI165777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165778, SGI0, SGI0, SGI0, SGI165779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36631[] = {SGI165780, SGI165781, SGI165782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165783, SGI0, SGI0, SGI0, SGI165784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36632[] = {SGI165785, SGI0, SGI165786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36633[] = {SGI165787, SGI165788, SGI165789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165790, SGI0, SGI0, SGI0, SGI165791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36634[] = {SGI165792, SGI0, SGI165793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165794, SGI165795, SGI165795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36635[] = {SGI165796, SGI165797, SGI165798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165799, SGI0, SGI0, SGI0, SGI165800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36636[] = {SGI165801, SGI165802, SGI165803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165804, SGI0, SGI0, SGI0, SGI165805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36637[] = {SGI165806, SGI0, SGI165808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165811, SGI165813, SGI165813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36638[] = {SGI165807, SGI165809, SGI165810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165812, SGI0, SGI0, SGI0, SGI165814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36639[] = {SGI165815, SGI165816, SGI165817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165818, SGI0, SGI0, SGI0, SGI165819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36640[] = {SGI165820, SGI165821, SGI165822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165823, SGI0, SGI0, SGI0, SGI165824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36641[] = {SGI165825, SGI0, SGI165826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165827, SGI165828, SGI165828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36642[] = {SGI165829, SGI165830, SGI165831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165832, SGI0, SGI0, SGI0, SGI165833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36643[] = {SGI165834, SGI0, SGI165835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165836, SGI165837, SGI165837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36644[] = {SGI165838, SGI165839, SGI165840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165841, SGI0, SGI0, SGI0, SGI165842, SGI165842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36645[] = {SGI165843, SGI165844, SGI165845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165846, SGI0, SGI0, SGI0, SGI165847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36646[] = {SGI165848, SGI165849, SGI165850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165851, SGI0, SGI0, SGI0, SGI165852, SGI165852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36647[] = {SGI165853, SGI165854, SGI165855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165856, SGI0, SGI0, SGI0, SGI165857, SGI165857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36648[] = {SGI165858, SGI0, SGI165859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165860, SGI165861, SGI165861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36649[] = {SGI165862, SGI165863, SGI165864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165865, SGI0, SGI0, SGI0, SGI165866, SGI165866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36650[] = {SGI165867, SGI0, SGI165868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165869, SGI165870, SGI165870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36651[] = {SGI165871, SGI165872, SGI165873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165874, SGI0, SGI0, SGI0, SGI165875, SGI165875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36652[] = {SGI165876, SGI0, SGI165877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165878, SGI165879, SGI165879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36653[] = {SGI165880, SGI165881, SGI165882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165883, SGI0, SGI0, SGI0, SGI0, SGI165884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36654[] = {SGI165885, SGI165886, SGI165887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165888, SGI165888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36655[] = {SGI165889, SGI165890, SGI165891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165892, SGI0, SGI0, SGI0, SGI165893, SGI165893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36656[] = {SGI165894, SGI0, SGI165895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36657[] = {SGI165896, SGI0, SGI165897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165898, SGI165899, SGI165899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36658[] = {SGI165900, SGI0, SGI165901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165902, SGI165903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36659[] = {SGI165904, SGI0, SGI165905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165906, SGI165907, SGI165907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36660[] = {SGI165908, SGI0, SGI165909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165910, SGI165911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36661[] = {SGI165912, SGI0, SGI165913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165914, SGI165915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36662[] = {SGI165916, SGI0, SGI165917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165918, SGI165919, SGI165919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36663[] = {SGI165920, SGI0, SGI165921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165922, SGI165923, SGI165923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36664[] = {SGI165924, SGI0, SGI165925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165926, SGI165927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36665[] = {SGI165928, SGI0, SGI165929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165930, SGI165931, SGI165931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36666[] = {SGI165932, SGI0, SGI165933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165934, SGI165935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36667[] = {SGI165936, SGI165937, SGI165938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165939, SGI0, SGI0, SGI0, SGI165940, SGI165940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36668[] = {SGI165942, SGI165943, SGI165944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165945, SGI0, SGI0, SGI0, SGI165946, SGI165946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36669[] = {SGI165948, SGI165949, SGI165950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165951, SGI0, SGI0, SGI0, SGI165952, SGI165952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36670[] = {SGI165954, SGI165955, SGI165956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165957, SGI0, SGI0, SGI0, SGI165958, SGI165958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36671[] = {SGI165960, SGI165961, SGI165962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165963, SGI0, SGI0, SGI0, SGI165964, SGI165964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36672[] = {SGI165966, SGI0, SGI165967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165968, SGI165969, SGI165969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36673[] = {SGI165970, SGI0, SGI165971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165972, SGI165973, SGI165973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36674[] = {SGI165974, SGI165975, SGI165976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165977, SGI0, SGI0, SGI0, SGI165978, SGI165978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36675[] = {SGI165979, SGI0, SGI165980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165981, SGI0, SGI0, SGI0, SGI165982, SGI165982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36676[] = {SGI165983, SGI165984, SGI165985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165986, SGI0, SGI0, SGI0, SGI165987, SGI165987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36677[] = {SGI165988, SGI0, SGI165989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36678[] = {SGI165990, SGI165991, SGI165992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165993, SGI0, SGI0, SGI0, SGI165994, SGI165994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36679[] = {SGI165995, SGI165996, SGI165997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI165998, SGI0, SGI0, SGI0, SGI165999, SGI165999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36680[] = {SGI166000, SGI0, SGI166001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36681[] = {SGI166002, SGI166003, SGI166004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166005, SGI0, SGI0, SGI0, SGI166006, SGI166006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36682[] = {SGI166007, SGI166008, SGI166009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166010, SGI0, SGI0, SGI0, SGI166011, SGI166011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36683[] = {SGI166012, SGI0, SGI166013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166014, SGI166015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36684[] = {SGI166016, SGI0, SGI166017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166018, SGI166019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36685[] = {SGI166020, SGI0, SGI166021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166022, SGI166023, SGI166023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36686[] = {SGI166024, SGI166025, SGI166026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166027, SGI0, SGI0, SGI0, SGI166028, SGI166028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36687[] = {SGI166029, SGI0, SGI166030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166031, SGI166032, SGI166032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36688[] = {SGI166033, SGI0, SGI166034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166035, SGI166036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36689[] = {SGI166037, SGI166038, SGI166039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166040, SGI0, SGI0, SGI0, SGI166041, SGI166041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36690[] = {SGI166046, SGI166047, SGI166048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166049, SGI0, SGI0, SGI0, SGI166050, SGI166050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36691[] = {SGI166051, SGI0, SGI166052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36692[] = {SGI166053, SGI166054, SGI166055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166056, SGI0, SGI0, SGI0, SGI166057, SGI166057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36693[] = {SGI166058, SGI166059, SGI166060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166061, SGI0, SGI0, SGI0, SGI166062, SGI166062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36694[] = {SGI166063, SGI166064, SGI166065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166066, SGI0, SGI0, SGI0, SGI166067, SGI166067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36695[] = {SGI166068, SGI0, SGI166069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166070, SGI0, SGI0, SGI0, SGI166071, SGI166071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36696[] = {SGI166072, SGI166073, SGI166074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166075, SGI0, SGI0, SGI0, SGI166076, SGI166076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36697[] = {SGI166077, SGI166078, SGI166079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166080, SGI0, SGI0, SGI0, SGI166081, SGI166081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36698[] = {SGI166082, SGI0, SGI166083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166084, SGI166085, SGI166085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36699[] = {SGI166086, SGI0, SGI166087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166088, SGI166089, SGI166089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36700[] = {SGI166090, SGI0, SGI166091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166092, SGI166093, SGI166093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36701[] = {SGI166094, SGI0, SGI166095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166096, SGI166097, SGI166097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36702[] = {SGI166098, SGI0, SGI166099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166103, SGI166104, SGI166104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36703[] = {SGI166100, SGI166101, SGI166102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166105, SGI0, SGI0, SGI0, SGI166106, SGI166106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36704[] = {SGI166108, SGI166109, SGI166110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166111, SGI0, SGI0, SGI0, SGI166112, SGI166112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36705[] = {SGI166114, SGI166115, SGI166116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166117, SGI0, SGI0, SGI0, SGI166118, SGI166118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36706[] = {SGI166120, SGI0, SGI166121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166122, SGI166123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36707[] = {SGI166124, SGI0, SGI166125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166126, SGI166127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36708[] = {SGI166128, SGI166129, SGI166130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166131, SGI0, SGI0, SGI0, SGI166132, SGI166132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36709[] = {SGI166134, SGI166135, SGI166136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166137, SGI0, SGI0, SGI0, SGI166138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36710[] = {SGI166140, SGI166141, SGI166142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166143, SGI0, SGI0, SGI0, SGI166144, SGI166144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36711[] = {SGI166146, SGI0, SGI166147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166148, SGI166149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36712[] = {SGI166150, SGI166151, SGI166152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166153, SGI0, SGI0, SGI0, SGI166154, SGI166154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36713[] = {SGI166156, SGI0, SGI166157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166158, SGI166159, SGI166159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36714[] = {SGI166160, SGI0, SGI166161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166162, SGI166163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36715[] = {SGI166164, SGI0, SGI166165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166166, SGI166167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36716[] = {SGI166042, SGI0, SGI166043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166044, SGI166045, SGI166045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36717[] = {SGI166168, SGI0, SGI166169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166170, SGI166171, SGI166171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36718[] = {SGI166172, SGI0, SGI166173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166174, SGI166175, SGI166175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36719[] = {SGI166176, SGI0, SGI166177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166178, SGI166179, SGI166179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36720[] = {SGI166180, SGI0, SGI166181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166182, SGI166183, SGI166183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36721[] = {SGI166184, SGI0, SGI166185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166186, SGI166187, SGI166187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36722[] = {SGI166188, SGI0, SGI166189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166192, SGI166193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36723[] = {SGI166190, SGI0, SGI166191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166194, SGI166195, SGI166195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36724[] = {SGI166196, SGI0, SGI166197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166198, SGI166199, SGI166199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36725[] = {SGI166200, SGI0, SGI166201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166202, SGI166203, SGI166203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36726[] = {SGI166204, SGI0, SGI166205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166206, SGI166207, SGI166207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36727[] = {SGI166208, SGI166209, SGI166210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166211, SGI0, SGI0, SGI0, SGI166212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36728[] = {SGI166213, SGI0, SGI166214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36729[] = {SGI166215, SGI166216, SGI166217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166218, SGI0, SGI0, SGI0, SGI166219, SGI166219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36730[] = {SGI166220, SGI166221, SGI166222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166223, SGI0, SGI0, SGI0, SGI166224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36731[] = {SGI166226, SGI166227, SGI166228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166229, SGI0, SGI0, SGI0, SGI166230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36732[] = {SGI166232, SGI166233, SGI166234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166235, SGI0, SGI0, SGI0, SGI166236, SGI166236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36733[] = {SGI166238, SGI166239, SGI166240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166241, SGI0, SGI0, SGI0, SGI166242, SGI166242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36734[] = {SGI166244, SGI166245, SGI166246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166247, SGI0, SGI0, SGI0, SGI166248, SGI166248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36735[] = {SGI166250, SGI166251, SGI166252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166253, SGI0, SGI0, SGI0, SGI166254, SGI166254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36736[] = {SGI166256, SGI166257, SGI166258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166259, SGI0, SGI0, SGI0, SGI166260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36737[] = {SGI166261, SGI0, SGI166262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36738[] = {SGI166263, SGI0, SGI166264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166265, SGI166266, SGI166266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36739[] = {SGI166267, SGI0, SGI166268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166269, SGI166270, SGI166270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36740[] = {SGI166271, SGI166272, SGI166273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166274, SGI0, SGI0, SGI0, SGI166275, SGI166275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36741[] = {SGI166276, SGI0, SGI166277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166281, SGI166282, SGI166282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36742[] = {SGI166278, SGI166279, SGI166280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166283, SGI0, SGI0, SGI0, SGI166284, SGI166284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36743[] = {SGI166285, SGI166286, SGI166288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166290, SGI0, SGI0, SGI0, SGI166292, SGI166292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36744[] = {SGI166287, SGI0, SGI166289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166291, SGI166293, SGI166293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36745[] = {SGI166294, SGI0, SGI166295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166296, SGI166297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36746[] = {SGI166298, SGI166299, SGI166300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166301, SGI0, SGI0, SGI0, SGI166302, SGI166302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36747[] = {SGI166303, SGI0, SGI166304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36748[] = {SGI166305, SGI166306, SGI166307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166308, SGI0, SGI0, SGI0, SGI166309, SGI166309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36749[] = {SGI166310, SGI166311, SGI166312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166313, SGI0, SGI0, SGI0, SGI166314, SGI166314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36750[] = {SGI166315, SGI166316, SGI166317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166318, SGI0, SGI0, SGI0, SGI166319, SGI166319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36751[] = {SGI166320, SGI166321, SGI166322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166323, SGI0, SGI0, SGI0, SGI166324, SGI166324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36752[] = {SGI166325, SGI166326, SGI166327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166328, SGI0, SGI0, SGI0, SGI166329, SGI166329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36753[] = {SGI166330, SGI0, SGI166331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166332, SGI166333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36754[] = {SGI166334, SGI0, SGI166335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166336, SGI166337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36755[] = {SGI166338, SGI0, SGI166339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166340, SGI166341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36756[] = {SGI166342, SGI0, SGI166343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166344, SGI166345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36757[] = {SGI166346, SGI0, SGI166347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166348, SGI166349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36758[] = {SGI166350, SGI0, SGI166351, SGI0, SGI166352, SGI166352, SGI0, NULL};
const Segment * const SG36759[] = {SGI166353, SGI166354, SGI166355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166356, SGI0, SGI0, SGI0, SGI166357, SGI166357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36760[] = {SGI166358, SGI0, SGI166359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36761[] = {SGI166360, SGI166361, SGI166362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166363, SGI0, SGI0, SGI0, SGI166364, SGI166364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36762[] = {SGI166365, SGI166366, SGI166367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166368, SGI0, SGI0, SGI166369, SGI166370, SGI166370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36763[] = {SGI166371, SGI0, SGI166372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166373, SGI166374, SGI166374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36764[] = {SGI166375, SGI166376, SGI166377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166378, SGI0, SGI0, SGI166379, SGI166380, SGI166380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36765[] = {SGI166381, SGI166382, SGI166383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166384, SGI0, SGI0, SGI166385, SGI166386, SGI166386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36766[] = {SGI166387, SGI0, SGI166388, SGI0, SGI166389, SGI166389, SGI0, NULL};
const Segment * const SG36767[] = {SGI166390, SGI166391, SGI166392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166393, SGI0, SGI0, SGI166394, SGI166395, SGI166395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36768[] = {SGI166396, SGI0, SGI166397, SGI0, SGI166398, SGI166398, SGI0, NULL};
const Segment * const SG36769[] = {SGI166399, SGI0, SGI166400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166401, SGI166402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36770[] = {SGI166403, SGI0, SGI166404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166405, SGI166406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36771[] = {SGI166407, SGI166408, SGI166409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166410, SGI0, SGI0, SGI0, SGI166411, SGI166411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36772[] = {SGI166412, SGI166413, SGI166414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166415, SGI0, SGI0, SGI0, SGI166416, SGI166416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36773[] = {SGI166417, SGI166418, SGI166419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166420, SGI0, SGI0, SGI0, SGI166421, SGI166421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36774[] = {SGI166422, SGI166423, SGI166424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166425, SGI0, SGI0, SGI0, SGI166426, SGI166426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36775[] = {SGI166427, SGI0, SGI166428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36776[] = {SGI166429, SGI166430, SGI166431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166432, SGI0, SGI0, SGI0, SGI166434, SGI166434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36777[] = {SGI166433, SGI166435, SGI166436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166437, SGI166437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36778[] = {SGI166438, SGI166439, SGI166440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166441, SGI0, SGI0, SGI0, SGI166442, SGI166442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36779[] = {SGI166443, SGI166444, SGI166445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166446, SGI0, SGI0, SGI0, SGI166447, SGI166447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36780[] = {SGI166448, SGI166449, SGI166450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166451, SGI0, SGI0, SGI0, SGI166452, SGI166452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36781[] = {SGI166453, SGI166454, SGI166455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166458, SGI0, SGI0, SGI0, SGI166459, SGI166459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36782[] = {SGI166456, SGI0, SGI166457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166460, SGI166461, SGI166461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36783[] = {SGI166462, SGI0, SGI166465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166468, SGI166469, SGI166469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36784[] = {SGI166463, SGI166464, SGI166466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166467, SGI0, SGI0, SGI0, SGI166470, SGI166470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36785[] = {SGI166471, SGI0, SGI166472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166473, SGI166474, SGI166474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36786[] = {SGI166475, SGI0, SGI166476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166477, SGI166478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36787[] = {SGI166479, SGI0, SGI166480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166481, SGI166482, SGI166482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36788[] = {SGI166483, SGI0, SGI166484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166485, SGI166486, SGI166486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36789[] = {SGI166487, SGI0, SGI166488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166489, SGI166490, SGI166490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36790[] = {SGI166491, SGI0, SGI166492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166493, SGI166494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36791[] = {SGI166495, SGI166496, SGI166497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166498, SGI0, SGI0, SGI0, SGI166499, SGI166499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36792[] = {SGI166500, SGI0, SGI166502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166505, SGI166507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36793[] = {SGI166501, SGI166503, SGI166504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166506, SGI0, SGI0, SGI0, SGI166508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36794[] = {SGI166509, SGI0, SGI166510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166514, SGI166515, SGI166515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36795[] = {SGI166511, SGI166512, SGI166513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166516, SGI0, SGI0, SGI0, SGI166517, SGI166517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36796[] = {SGI166518, SGI0, SGI166519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166520, SGI166521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36797[] = {SGI166522, SGI166523, SGI166524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166525, SGI0, SGI0, SGI0, SGI166526, SGI166526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36798[] = {SGI166527, SGI0, SGI166528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166532, SGI166534, SGI166534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36799[] = {SGI166529, SGI166530, SGI166531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166533, SGI0, SGI0, SGI0, SGI166535, SGI166535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36800[] = {SGI166537, SGI0, SGI166539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36801[] = {SGI166536, SGI0, SGI166538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166540, SGI166541, SGI166541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36802[] = {SGI166542, SGI0, SGI166543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166546, SGI166547, SGI166547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36803[] = {SGI166544, SGI0, SGI166545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166548, SGI166549, SGI166549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36804[] = {SGI166550, SGI0, SGI166551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166554, SGI166555, SGI166555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36805[] = {SGI166552, SGI0, SGI166553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166556, SGI166557, SGI166557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36806[] = {SGI166558, SGI0, SGI166559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166562, SGI166563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36807[] = {SGI166560, SGI0, SGI166561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166564, SGI166565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36808[] = {SGI166566, SGI0, SGI166567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166568, SGI166569, SGI166569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36809[] = {SGI166570, SGI0, SGI166571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166572, SGI166573, SGI166573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36810[] = {SGI166574, SGI166575, SGI166576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166577, SGI0, SGI0, SGI0, SGI166578, SGI166578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36811[] = {SGI166579, SGI0, SGI166580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166581, SGI166583, SGI166583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36812[] = {SGI166582, SGI166585, SGI166586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166588, SGI0, SGI0, SGI0, SGI166590, SGI166590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36813[] = {SGI166584, SGI0, SGI166587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166589, SGI166591, SGI166591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36814[] = {SGI166592, SGI166593, SGI166594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166595, SGI0, SGI0, SGI0, SGI166596, SGI166596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36815[] = {SGI166597, SGI0, SGI166598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36816[] = {SGI166599, SGI0, SGI166600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166606, SGI166607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36817[] = {SGI166601, SGI166602, SGI166604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166608, SGI0, SGI0, SGI0, SGI166609, SGI166609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36818[] = {SGI166603, SGI0, SGI166605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166610, SGI166611, SGI166611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36819[] = {SGI166612, SGI166613, SGI166614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166615, SGI0, SGI0, SGI0, SGI166617, SGI166617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36820[] = {SGI166616, SGI0, SGI166618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166621, SGI166622, SGI166622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36821[] = {SGI166619, SGI0, SGI166620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166623, SGI166624, SGI166624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36822[] = {SGI166625, SGI166626, SGI166627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166628, SGI0, SGI0, SGI0, SGI166629, SGI166629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36823[] = {SGI166630, SGI0, SGI166631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166632, SGI166633, SGI166633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36824[] = {SGI166634, SGI166636, SGI166637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166639, SGI0, SGI0, SGI0, SGI166642, SGI166642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36825[] = {SGI166635, SGI0, SGI166638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166640, SGI166641, SGI166641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36826[] = {SGI166643, SGI0, SGI166644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166645, SGI166646, SGI166646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36827[] = {SGI166647, SGI0, SGI166648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166649, SGI166650, SGI166650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36828[] = {SGI166651, SGI0, SGI166652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166653, SGI166654, SGI166654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36829[] = {SGI166655, SGI0, SGI166656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166657, SGI166658, SGI166658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36830[] = {SGI166659, SGI0, SGI166660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166661, SGI166662, SGI166662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36831[] = {SGI166663, SGI0, SGI166664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166665, SGI166666, SGI166666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36832[] = {SGI166667, SGI0, SGI166668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166669, SGI166670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36833[] = {SGI166671, SGI0, SGI166672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166673, SGI166674, SGI166674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36834[] = {SGI166675, SGI0, SGI166676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166677, SGI166678, SGI166678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36835[] = {SGI166679, SGI0, SGI166680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166681, SGI166682, SGI166682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36836[] = {SGI166683, SGI0, SGI166684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166685, SGI166686, SGI166686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36837[] = {SGI166687, SGI0, SGI166688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166689, SGI166690, SGI166690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36838[] = {SGI166691, SGI0, SGI166692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166693, SGI166695, SGI166695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36839[] = {SGI166696, SGI166698, SGI166700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166701, SGI0, SGI0, SGI0, SGI166703, SGI166703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36840[] = {SGI166694, SGI166697, SGI166699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166702, SGI0, SGI0, SGI0, SGI166704, SGI166704, SGI0, SGI166705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36841[] = {SGI166706, SGI166708, SGI166710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166714, SGI0, SGI0, SGI0, SGI166716, SGI166716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36842[] = {SGI166707, SGI0, SGI166711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166715, SGI166718, SGI166718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36843[] = {SGI166709, SGI166712, SGI166713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166717, SGI0, SGI0, SGI0, SGI166719, SGI166719, SGI0, SGI166720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36844[] = {SGI166721, SGI166722, SGI166723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166724, SGI0, SGI0, SGI0, SGI166727, SGI166727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36845[] = {SGI166725, SGI0, SGI166726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36846[] = {SGI166728, SGI0, SGI166729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166731, SGI166732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36847[] = {SGI166733, SGI166735, SGI166737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166741, SGI0, SGI0, SGI0, SGI166743, SGI166743, SGI0, SGI166746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36848[] = {SGI166734, SGI166738, SGI166740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166745, SGI0, SGI0, SGI0, SGI166747, SGI166747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36849[] = {SGI166742, SGI0, SGI166744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166748, SGI166750, SGI166750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36850[] = {SGI166751, SGI166752, SGI166753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166754, SGI0, SGI0, SGI0, SGI166755, SGI166755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36851[] = {SGI166756, SGI166757, SGI166758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166759, SGI0, SGI0, SGI0, SGI166760, SGI166760, SGI0, SGI166761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36852[] = {SGI166763, SGI0, SGI166765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166768, SGI166770, SGI166770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36853[] = {SGI166762, SGI166764, SGI166766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166767, SGI0, SGI0, SGI0, SGI166769, SGI166769, SGI0, SGI166771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36854[] = {SGI166772, SGI0, SGI166773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166775, SGI166777, SGI166777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36855[] = {SGI166774, SGI166776, SGI166778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166779, SGI0, SGI0, SGI0, SGI166780, SGI166780, SGI0, SGI166783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36856[] = {SGI166781, SGI0, SGI166782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166784, SGI166785, SGI166785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36857[] = {SGI166786, SGI166787, SGI166788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166790, SGI0, SGI0, SGI0, SGI166792, SGI166792, SGI0, SGI166793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36858[] = {SGI166789, SGI0, SGI166791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166794, SGI166795, SGI166795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36859[] = {SGI166796, SGI166797, SGI166799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166801, SGI0, SGI0, SGI0, SGI166802, SGI166802, SGI0, SGI166805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36860[] = {SGI166798, SGI0, SGI166800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166803, SGI166804, SGI166804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36861[] = {SGI166806, SGI166807, SGI166808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166809, SGI0, SGI0, SGI0, SGI166810, SGI166810, SGI0, SGI166811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36862[] = {SGI166812, SGI0, SGI166813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166817, SGI166818, SGI166818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36863[] = {SGI166814, SGI166815, SGI166816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166819, SGI0, SGI0, SGI0, SGI166820, SGI166820, SGI0, SGI166821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36864[] = {SGI166822, SGI0, SGI166823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166824, SGI166825, SGI166825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36865[] = {SGI166826, SGI166827, SGI166828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166829, SGI0, SGI0, SGI0, SGI166830, SGI166830, SGI0, SGI166831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36866[] = {SGI166832, SGI0, SGI166833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166836, SGI166837, SGI166837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36867[] = {SGI166834, SGI0, SGI166835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166838, SGI166839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36868[] = {SGI166840, SGI166841, SGI166842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166844, SGI0, SGI0, SGI0, SGI166846, SGI166846, SGI0, SGI166847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36869[] = {SGI166843, SGI0, SGI166845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166848, SGI166849, SGI166849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36870[] = {SGI166850, SGI166851, SGI166852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166853, SGI0, SGI0, SGI0, SGI166854, SGI166854, SGI0, SGI166855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36871[] = {SGI166856, SGI0, SGI166857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166858, SGI166859, SGI166859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36872[] = {SGI166860, SGI0, SGI166861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166862, SGI166863, SGI166863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36873[] = {SGI166864, SGI0, SGI166865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166866, SGI166867, SGI166867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36874[] = {SGI166868, SGI0, SGI166869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166870, SGI166871, SGI166871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36875[] = {SGI166872, SGI0, SGI166873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166874, SGI166875, SGI166875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36876[] = {SGI166876, SGI0, SGI166877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166878, SGI166879, SGI166879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36877[] = {SGI166880, SGI0, SGI166881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166882, SGI166883, SGI166883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36878[] = {SGI166884, SGI0, SGI166885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166886, SGI166887, SGI166887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36879[] = {SGI166888, SGI0, SGI166889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166893, SGI166894, SGI166894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36880[] = {SGI166890, SGI166891, SGI166892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166895, SGI0, SGI0, SGI0, SGI166896, SGI166896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36881[] = {SGI166736, SGI166739, SGI166897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166898, SGI0, SGI0, SGI0, SGI166899, SGI166899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36882[] = {SGI166900, SGI0, SGI166902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36883[] = {SGI166901, SGI0, SGI166903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166904, SGI166905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36884[] = {SGI166906, SGI166907, SGI166908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166911, SGI0, SGI0, SGI0, SGI166912, SGI166912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36885[] = {SGI166909, SGI0, SGI166910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166913, SGI166914, SGI166914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36886[] = {SGI166915, SGI0, SGI166916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36887[] = {SGI166917, SGI0, SGI166918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166919, SGI166920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36888[] = {SGI166921, SGI0, SGI166922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166923, SGI166924, SGI166924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36889[] = {SGI0, SGI0, SGI0, SGI166931, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36890[] = {SGI166925, SGI0, SGI166926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166929, SGI166930, SGI166930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36891[] = {SGI166927, SGI0, SGI166928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166932, SGI166933, SGI166933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36892[] = {SGI166934, SGI0, SGI166935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166936, SGI166937, SGI166937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36893[] = {SGI166938, SGI0, SGI166939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166940, SGI166941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36894[] = {SGI166942, SGI0, SGI166943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166944, SGI166945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36895[] = {SGI166946, SGI0, SGI166947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166948, SGI166949, SGI166949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36896[] = {SGI166950, SGI0, SGI166951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166952, SGI166953, SGI166953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36897[] = {SGI166954, SGI0, SGI166955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166956, SGI166957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36898[] = {SGI166958, SGI166959, SGI166960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166961, SGI0, SGI0, SGI0, SGI166962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36899[] = {SGI166963, SGI0, SGI166964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166965, SGI166966, SGI166966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36900[] = {SGI166967, SGI0, SGI166968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166969, SGI166970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36901[] = {SGI0, SGI0, SGI166971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166972, SGI0, SGI0, SGI0, SGI166973, SGI166973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36902[] = {SGI166974, SGI166975, SGI166976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166977, SGI0, SGI0, SGI0, SGI166978, SGI166978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36903[] = {SGI166979, SGI0, SGI166980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36904[] = {SGI166981, SGI166982, SGI166983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166984, SGI0, SGI0, SGI0, SGI166986, SGI166986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36905[] = {SGI166985, SGI166987, SGI166988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166989, SGI0, SGI0, SGI0, SGI166990, SGI166990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36906[] = {SGI166991, SGI0, SGI166992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36907[] = {SGI166993, SGI0, SGI166994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI166995, SGI166996, SGI166996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36908[] = {SGI166997, SGI166998, SGI166999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167000, SGI0, SGI0, SGI0, SGI167001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36909[] = {SGI167002, SGI0, SGI167003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36910[] = {SGI167004, SGI167005, SGI167006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167007, SGI0, SGI0, SGI0, SGI167008, SGI167008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36911[] = {SGI167009, SGI0, SGI167010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36912[] = {SGI167011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36913[] = {SGI167012, SGI167013, SGI167014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167015, SGI0, SGI0, SGI0, SGI167016, SGI167016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36914[] = {SGI167017, SGI0, SGI167018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36915[] = {SGI167019, SGI167020, SGI167021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167022, SGI167022, SGI0, SGI0, SGI0, SGI167023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36916[] = {SGI167024, SGI167025, SGI167026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167027, SGI167027, SGI0, SGI0, SGI0, SGI167028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36917[] = {SGI167029, SGI0, SGI167030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167031, SGI167032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36918[] = {SGI167033, SGI0, SGI167034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167035, SGI167036, SGI167036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36919[] = {SGI167037, SGI0, SGI167038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167039, SGI167040, SGI167040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36920[] = {SGI167041, SGI167043, SGI167044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167046, SGI0, SGI0, SGI0, SGI167048, SGI167048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36921[] = {SGI167042, SGI0, SGI167045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167047, SGI167049, SGI167049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36922[] = {SGI167050, SGI167051, SGI167052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167053, SGI0, SGI0, SGI0, SGI167054, SGI167054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36923[] = {SGI167055, SGI0, SGI167056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167057, SGI167058, SGI167058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36924[] = {SGI167059, SGI167060, SGI167061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167062, SGI0, SGI0, SGI0, SGI167064, SGI167064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36925[] = {SGI167063, SGI0, SGI167066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167068, SGI167069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36926[] = {SGI167065, SGI0, SGI167067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167070, SGI167071, SGI167071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36927[] = {SGI167072, SGI167073, SGI167074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167075, SGI0, SGI0, SGI0, SGI167076, SGI167076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36928[] = {SGI167077, SGI167078, SGI167079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167082, SGI0, SGI0, SGI0, SGI167083, SGI167083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36929[] = {SGI167080, SGI0, SGI167081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167084, SGI167085, SGI167085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36930[] = {SGI167087, SGI0, SGI167089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167092, SGI167093, SGI167093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36931[] = {SGI167086, SGI167088, SGI167090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167091, SGI0, SGI0, SGI0, SGI167094, SGI167094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36932[] = {SGI167095, SGI167096, SGI167097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167098, SGI0, SGI0, SGI0, SGI167099, SGI167099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36933[] = {SGI167102, SGI0, SGI167103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36934[] = {SGI167100, SGI0, SGI167101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167104, SGI167105, SGI167105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36935[] = {SGI167106, SGI0, SGI167107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167108, SGI167109, SGI167109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36936[] = {SGI167110, SGI0, SGI167111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167112, SGI167113, SGI167113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36937[] = {SGI167114, SGI0, SGI167115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167116, SGI167117, SGI167117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36938[] = {SGI167118, SGI0, SGI167119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167120, SGI167121, SGI167121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36939[] = {SGI167122, SGI0, SGI167123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167124, SGI167125, SGI167125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36940[] = {SGI167126, SGI0, SGI167127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167128, SGI167129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36941[] = {SGI167130, SGI0, SGI167131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167132, SGI167133, SGI167133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36942[] = {SGI167134, SGI0, SGI167135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167138, SGI167139, SGI167139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36943[] = {SGI167136, SGI0, SGI167137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167140, SGI167141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36944[] = {SGI167142, SGI0, SGI167143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167144, SGI167145, SGI167145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36945[] = {SGI167146, SGI0, SGI167147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167150, SGI167151, SGI167151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36946[] = {SGI167148, SGI0, SGI167149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167152, SGI167153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36947[] = {SGI167154, SGI0, SGI167155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167156, SGI167157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36948[] = {SGI167158, SGI0, SGI167159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167160, SGI167161, SGI167161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36949[] = {SGI167162, SGI0, SGI167163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167164, SGI167165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36950[] = {SGI167166, SGI0, SGI167167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167168, SGI167169, SGI167169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36951[] = {SGI167170, SGI0, SGI167171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167172, SGI167173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36952[] = {SGI167174, SGI0, SGI167175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167176, SGI167177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36953[] = {SGI167178, SGI167179, SGI167180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167181, SGI0, SGI0, SGI0, SGI167183, SGI167183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36954[] = {SGI167182, SGI0, SGI167184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167187, SGI167188, SGI167188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36955[] = {SGI167185, SGI0, SGI167186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167189, SGI167190, SGI167190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36956[] = {SGI167191, SGI167192, SGI167193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167194, SGI0, SGI0, SGI0, SGI167195, SGI167195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36957[] = {SGI167196, SGI0, SGI167197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167198, SGI167199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36958[] = {SGI167200, SGI167201, SGI167202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167204, SGI0, SGI0, SGI0, SGI167206, SGI167206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36959[] = {SGI167203, SGI0, SGI167205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167207, SGI167208, SGI167208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36960[] = {SGI167209, SGI167210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167213, SGI0, SGI0, SGI0, SGI167214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36961[] = {SGI167211, SGI0, SGI167212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167215, SGI167216, SGI167216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36962[] = {SGI167217, SGI167218, SGI167219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167220, SGI0, SGI0, SGI0, SGI167221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36963[] = {SGI167222, SGI0, SGI167223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167225, SGI167227, SGI167227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36964[] = {SGI167224, SGI167226, SGI167228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167229, SGI0, SGI0, SGI0, SGI167230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36965[] = {SGI167231, SGI0, SGI167232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167233, SGI167234, SGI167234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36966[] = {SGI167235, SGI0, SGI167236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167237, SGI167238, SGI167238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36967[] = {SGI167239, SGI0, SGI167240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167241, SGI167242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36968[] = {SGI167243, SGI0, SGI167244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167245, SGI167246, SGI167246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36969[] = {SGI167247, SGI0, SGI167248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167249, SGI167250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36970[] = {SGI167251, SGI0, SGI167252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167253, SGI167254, SGI167254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36971[] = {SGI167255, SGI0, SGI167256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167257, SGI167258, SGI167258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36972[] = {SGI167259, SGI167260, SGI167261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167262, SGI0, SGI0, SGI0, SGI167263, SGI167263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36973[] = {SGI167264, SGI0, SGI167265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167266, SGI167267, SGI167267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36974[] = {SGI167268, SGI0, SGI167269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36975[] = {SGI167270, SGI0, SGI167271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167272, SGI167273, SGI167273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36976[] = {SGI167274, SGI0, SGI167276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167279, SGI167281, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36977[] = {SGI167275, SGI0, SGI167277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167278, SGI167280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36978[] = {SGI167282, SGI0, SGI167283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167284, SGI167285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36979[] = {SGI167286, SGI0, SGI167287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167288, SGI167289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36980[] = {SGI167290, SGI167291, SGI167292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167293, SGI0, SGI0, SGI0, SGI167294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36981[] = {SGI167295, SGI0, SGI167296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36982[] = {SGI167297, SGI167298, SGI167299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167300, SGI0, SGI0, SGI0, SGI167301, SGI167301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36983[] = {SGI167302, SGI0, SGI167303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36984[] = {SGI167304, SGI0, SGI167305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167306, SGI167307, SGI167307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167308, SGI0, SGI0, NULL};
const Segment * const SG36985[] = {SGI167309, SGI0, SGI167310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167311, SGI167312, SGI167312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167313, SGI0, SGI0, NULL};
const Segment * const SG36986[] = {SGI167314, SGI167315, SGI167316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167319, SGI0, SGI0, SGI0, SGI167320, SGI167320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36987[] = {SGI167317, SGI0, SGI167318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167321, SGI167322, SGI167322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167323, SGI0, SGI0, NULL};
const Segment * const SG36988[] = {SGI167324, SGI167325, SGI167326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167327, SGI167327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36989[] = {SGI167328, SGI0, SGI167329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167330, SGI167331, SGI167331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167333, SGI0, SGI0, NULL};
const Segment * const SG36990[] = {SGI167332, SGI0, SGI167334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36991[] = {SGI167335, SGI167336, SGI167337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167340, SGI0, SGI0, SGI0, SGI167341, SGI167341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36992[] = {SGI167338, SGI0, SGI167339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167342, SGI167343, SGI167343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167344, SGI0, SGI0, NULL};
const Segment * const SG36993[] = {SGI167345, SGI167346, SGI167347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167349, SGI167351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36994[] = {SGI167348, SGI0, SGI167350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167352, SGI167353, SGI167353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167354, SGI0, SGI0, NULL};
const Segment * const SG36995[] = {SGI167355, SGI0, SGI167356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167357, SGI167358, SGI167358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167359, SGI0, SGI0, NULL};
const Segment * const SG36996[] = {SGI167360, SGI167361, SGI167362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167363, SGI0, SGI0, SGI0, SGI167364, SGI167364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36997[] = {SGI167365, SGI167366, SGI167367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167368, SGI0, SGI0, SGI0, SGI167369, SGI167369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36998[] = {SGI167370, SGI167371, SGI167372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167373, SGI0, SGI0, SGI0, SGI167374, SGI167374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG36999[] = {SGI167375, SGI167376, SGI167377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167378, SGI0, SGI0, SGI0, SGI167379, SGI167379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37000[] = {SGI167380, SGI167381, SGI167382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167383, SGI0, SGI0, SGI0, SGI167384, SGI167384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37001[] = {SGI167385, SGI0, SGI167386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37002[] = {SGI167387, SGI0, SGI167388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167389, SGI167390, SGI167390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37003[] = {SGI167391, SGI0, SGI167392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167393, SGI167394, SGI167394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37004[] = {SGI167395, SGI0, SGI167396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167399, SGI167400, SGI167400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37005[] = {SGI167397, SGI0, SGI167398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167401, SGI167402, SGI167402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37006[] = {SGI167403, SGI0, SGI167404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167408, SGI167409, SGI167409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37007[] = {SGI167405, SGI0, SGI167406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167411, SGI167412, SGI167412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37008[] = {SGI167407, SGI0, SGI167410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167413, SGI167414, SGI167414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37009[] = {SGI167415, SGI0, SGI167416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167417, SGI167418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37010[] = {SGI167419, SGI0, SGI167420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167421, SGI167422, SGI167422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37011[] = {SGI167423, SGI0, SGI167424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167425, SGI167426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37012[] = {SGI167427, SGI0, SGI167428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167429, SGI167430, SGI167430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37013[] = {SGI167431, SGI0, SGI167432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167433, SGI167434, SGI167434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37014[] = {SGI167435, SGI0, SGI167436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167437, SGI167438, SGI167438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37015[] = {SGI167439, SGI167440, SGI167441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167442, SGI0, SGI0, SGI0, SGI167443, SGI167443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37016[] = {SGI167444, SGI167445, SGI167446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167447, SGI0, SGI0, SGI0, SGI167448, SGI167448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37017[] = {SGI167449, SGI0, SGI167450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37018[] = {SGI167451, SGI0, SGI167452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167456, SGI167457, SGI167457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37019[] = {SGI167453, SGI167454, SGI167455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167458, SGI0, SGI0, SGI0, SGI167459, SGI167459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37020[] = {SGI167463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37021[] = {SGI167460, SGI167461, SGI167462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167464, SGI0, SGI0, SGI0, SGI167465, SGI167465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37022[] = {SGI167466, SGI0, SGI167467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167471, SGI167473, SGI167473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37023[] = {SGI167468, SGI167469, SGI167470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167472, SGI0, SGI0, SGI0, SGI167474, SGI167474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37024[] = {SGI167475, SGI167476, SGI167477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167478, SGI0, SGI0, SGI0, SGI167479, SGI167479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37025[] = {SGI167480, SGI167481, SGI167482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167485, SGI0, SGI0, SGI0, SGI167486, SGI167486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37026[] = {SGI167483, SGI0, SGI167484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167487, SGI167488, SGI167488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37027[] = {SGI167489, SGI167490, SGI167491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167492, SGI0, SGI0, SGI0, SGI167493, SGI167493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37028[] = {SGI167494, SGI167495, SGI167496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167497, SGI0, SGI0, SGI0, SGI167498, SGI167498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37029[] = {SGI167499, SGI0, SGI167500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167501, SGI167502, SGI167502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37030[] = {SGI167503, SGI0, SGI167504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167505, SGI167506, SGI167506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37031[] = {SGI167507, SGI0, SGI167508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167509, SGI167510, SGI167510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37032[] = {SGI167511, SGI0, SGI167512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167513, SGI167514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37033[] = {SGI167515, SGI0, SGI167516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167517, SGI167518, SGI167518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37034[] = {SGI167519, SGI0, SGI167520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167521, SGI167522, SGI167522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37035[] = {SGI167523, SGI0, SGI167524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167525, SGI167526, SGI167526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37036[] = {SGI167527, SGI167528, SGI167529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167530, SGI0, SGI0, SGI0, SGI167531, SGI167531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37037[] = {SGI167532, SGI167533, SGI167534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167535, SGI0, SGI0, SGI0, SGI167536, SGI167536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37038[] = {SGI167537, SGI0, SGI167538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37039[] = {SGI167539, SGI167540, SGI167541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167543, SGI0, SGI0, SGI0, SGI167545, SGI167545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37040[] = {SGI167542, SGI0, SGI167544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167546, SGI167547, SGI167547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37041[] = {SGI167548, SGI167549, SGI167550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167551, SGI0, SGI0, SGI0, SGI167552, SGI167552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37042[] = {SGI167553, SGI167554, SGI167555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167556, SGI0, SGI0, SGI0, SGI167557, SGI167557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37043[] = {SGI167558, SGI167559, SGI167560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167561, SGI0, SGI0, SGI0, SGI167562, SGI167562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37044[] = {SGI167563, SGI167564, SGI167565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167566, SGI0, SGI0, SGI0, SGI167567, SGI167567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37045[] = {SGI167568, SGI0, SGI167569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167570, SGI0, SGI0, SGI0, SGI167571, SGI167571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37046[] = {SGI167572, SGI167573, SGI167574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167575, SGI0, SGI0, SGI0, SGI167576, SGI167576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37047[] = {SGI167577, SGI167578, SGI167579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167580, SGI0, SGI0, SGI0, SGI167581, SGI167581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37048[] = {SGI167582, SGI167583, SGI167584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167585, SGI0, SGI0, SGI0, SGI167586, SGI167586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37049[] = {SGI167587, SGI167588, SGI167589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167590, SGI0, SGI0, SGI0, SGI167591, SGI167591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37050[] = {SGI167592, SGI167593, SGI167594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167595, SGI0, SGI0, SGI0, SGI167596, SGI167596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37051[] = {SGI167597, SGI167598, SGI167599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167600, SGI0, SGI0, SGI0, SGI167601, SGI167601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37052[] = {SGI167602, SGI0, SGI167603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167604, SGI167605, SGI167605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37053[] = {SGI167606, SGI167607, SGI167608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167609, SGI0, SGI0, SGI0, SGI167610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37054[] = {SGI167611, SGI167612, SGI167613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167614, SGI0, SGI0, SGI0, SGI167615, SGI167615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37055[] = {SGI167616, SGI167617, SGI167618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167619, SGI0, SGI0, SGI0, SGI167620, SGI167620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37056[] = {SGI167621, SGI167622, SGI167623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167624, SGI0, SGI0, SGI0, SGI167625, SGI167625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37057[] = {SGI167626, SGI167627, SGI167628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167630, SGI0, SGI0, SGI0, SGI167632, SGI167632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37058[] = {SGI167629, SGI167631, SGI167633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167634, SGI0, SGI0, SGI0, SGI167635, SGI167635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37059[] = {SGI167636, SGI0, SGI167637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37060[] = {SGI167638, SGI0, SGI167639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37061[] = {SGI167640, SGI0, SGI167641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167642, SGI0, SGI0, SGI0, SGI167643, SGI167643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37062[] = {SGI167644, SGI0, SGI167645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167646, SGI167647, SGI167647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37063[] = {SGI167648, SGI0, SGI167649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167650, SGI167651, SGI167651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37064[] = {SGI167652, SGI0, SGI167653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167654, SGI167655, SGI167655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37065[] = {SGI167656, SGI0, SGI167657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167658, SGI167659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37066[] = {SGI167660, SGI0, SGI167661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167662, SGI167663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37067[] = {SGI167664, SGI0, SGI167665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167666, SGI167667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37068[] = {SGI167668, SGI167669, SGI167670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167671, SGI0, SGI0, SGI0, SGI167672, SGI167672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37069[] = {SGI167673, SGI167674, SGI167675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167676, SGI0, SGI0, SGI0, SGI167677, SGI167677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37070[] = {SGI167678, SGI167679, SGI167680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167681, SGI0, SGI0, SGI0, SGI167682, SGI167682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37071[] = {SGI167683, SGI0, SGI167684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167685, SGI167686, SGI167686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37072[] = {SGI167687, SGI0, SGI167688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167689, SGI167690, SGI167690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37073[] = {SGI167691, SGI0, SGI167693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167695, SGI167697, SGI167697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37074[] = {SGI167692, SGI0, SGI167694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167696, SGI167698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37075[] = {SGI167699, SGI0, SGI167700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167701, SGI167702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37076[] = {SGI167703, SGI0, SGI167704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167707, SGI167708, SGI167708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37077[] = {SGI167705, SGI0, SGI167706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167709, SGI167710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37078[] = {SGI167711, SGI0, SGI167712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37079[] = {SGI167713, SGI0, SGI167714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167715, SGI167716, SGI167716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37080[] = {SGI167717, SGI0, SGI167718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167719, SGI167720, SGI167720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37081[] = {SGI167721, SGI0, SGI167722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167723, SGI167724, SGI167724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37082[] = {SGI167725, SGI167726, SGI167727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167729, SGI0, SGI0, SGI0, SGI167731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37083[] = {SGI167728, SGI0, SGI167730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167732, SGI167733, SGI167733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37084[] = {SGI167734, SGI0, SGI167735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167736, SGI167737, SGI167737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37085[] = {SGI167738, SGI0, SGI167739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167740, SGI167741, SGI167741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37086[] = {SGI167742, SGI0, SGI167743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167744, SGI167745, SGI167745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37087[] = {SGI167746, SGI0, SGI167747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167750, SGI167751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37088[] = {SGI167748, SGI0, SGI167749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167752, SGI167753, SGI167753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37089[] = {SGI167754, SGI0, SGI167756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167758, SGI167759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37090[] = {SGI167755, SGI0, SGI167757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167760, SGI167761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37091[] = {SGI167762, SGI0, SGI167763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167764, SGI167765, SGI167765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37092[] = {SGI167766, SGI0, SGI167767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167768, SGI167769, SGI167769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37093[] = {SGI167770, SGI0, SGI167772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167774, SGI167775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37094[] = {SGI167771, SGI0, SGI167773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167776, SGI167777, SGI167777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37095[] = {SGI167778, SGI0, SGI167779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167780, SGI167781, SGI167781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37096[] = {SGI167782, SGI0, SGI167783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167784, SGI167785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37097[] = {SGI167786, SGI0, SGI167787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167788, SGI167789, SGI167789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37098[] = {SGI167790, SGI0, SGI167791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167792, SGI167793, SGI167793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37099[] = {SGI167794, SGI0, SGI167795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167796, SGI167797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37100[] = {SGI167798, SGI0, SGI167799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167800, SGI167801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37101[] = {SGI167802, SGI0, SGI167803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167804, SGI167805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37102[] = {SGI167806, SGI0, SGI167807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167808, SGI167809, SGI167809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37103[] = {SGI167810, SGI0, SGI167811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167812, SGI167813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37104[] = {SGI167814, SGI0, SGI167815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167816, SGI167817, SGI167817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37105[] = {SGI167818, SGI0, SGI167819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167820, SGI167821, SGI167821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37106[] = {SGI167822, SGI0, SGI167823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167824, SGI167825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37107[] = {SGI167826, SGI167827, SGI167828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167829, SGI0, SGI0, SGI0, SGI167830, SGI167830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37108[] = {SGI167832, SGI167833, SGI167834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167835, SGI0, SGI0, SGI0, SGI167836, SGI167836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37109[] = {SGI167837, SGI0, SGI167838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37110[] = {SGI167839, SGI0, SGI167840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167841, SGI0, SGI0, SGI0, SGI167842, SGI167842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37111[] = {SGI167843, SGI167844, SGI167845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167846, SGI0, SGI0, SGI0, SGI167847, SGI167847, SGI0, SGI167848, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37112[] = {SGI167849, SGI167850, SGI167851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167852, SGI0, SGI0, SGI0, SGI167853, SGI167853, SGI0, SGI167854, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37113[] = {SGI167855, SGI167856, SGI167857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167858, SGI0, SGI0, SGI0, SGI167859, SGI167859, SGI0, SGI167860, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37114[] = {SGI167861, SGI167862, SGI167863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167864, SGI0, SGI0, SGI167865, SGI167866, SGI167866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37115[] = {SGI167867, SGI167868, SGI167869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167870, SGI0, SGI0, SGI167871, SGI167872, SGI167872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37116[] = {SGI167873, SGI0, SGI167874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167875, SGI167876, SGI167876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37117[] = {SGI167877, SGI167878, SGI167879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167880, SGI0, SGI0, SGI167882, SGI167883, SGI167883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37118[] = {SGI167881, SGI0, SGI167884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167885, SGI167886, SGI167886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37119[] = {SGI167887, SGI0, SGI167888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167889, SGI167890, SGI167890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37120[] = {SGI167891, SGI0, SGI167892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167893, SGI167894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37121[] = {SGI167895, SGI167896, SGI167897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167898, SGI0, SGI0, SGI0, SGI167899, SGI167899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37122[] = {SGI167900, SGI167901, SGI167902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167903, SGI0, SGI0, SGI0, SGI167904, SGI167904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37123[] = {SGI167905, SGI167906, SGI167907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167908, SGI0, SGI0, SGI0, SGI167909, SGI167909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37124[] = {SGI167910, SGI167911, SGI167912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167913, SGI0, SGI0, SGI0, SGI167914, SGI167914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37125[] = {SGI167915, SGI0, SGI167916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37126[] = {SGI167917, SGI167918, SGI167919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167920, SGI0, SGI0, SGI167921, SGI167922, SGI167922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37127[] = {SGI167923, SGI167924, SGI167925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167926, SGI0, SGI0, SGI167927, SGI167928, SGI167928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37128[] = {SGI167929, SGI0, SGI167930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167931, SGI167932, SGI167932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37129[] = {SGI167933, SGI0, SGI167934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167935, SGI167936, SGI167936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37130[] = {SGI167937, SGI0, SGI167938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167940, SGI167941, SGI167941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37131[] = {SGI167939, SGI0, SGI167942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167943, SGI167944, SGI167944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37132[] = {SGI167945, SGI0, SGI167946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167947, SGI167948, SGI167948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37133[] = {SGI167949, SGI167950, SGI167951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167952, SGI0, SGI0, SGI0, SGI167953, SGI167953, SGI0, SGI167954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37134[] = {SGI167955, SGI167956, SGI167957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167958, SGI0, SGI0, SGI0, SGI167959, SGI167959, SGI0, SGI167960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37135[] = {SGI167961, SGI0, SGI167962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167963, SGI167964, SGI167964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37136[] = {SGI0, SGI0, SGI167965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167966, SGI167966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37137[] = {SGI167967, SGI167968, SGI167969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167970, SGI167970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37138[] = {SGI167971, SGI167972, SGI167973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167974, SGI0, SGI0, SGI0, SGI167975, SGI167975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37139[] = {SGI0, SGI167976, SGI167977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167978, SGI167978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37140[] = {SGI167979, SGI167980, SGI167981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167985, SGI167985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37141[] = {SGI167982, SGI167983, SGI167984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167986, SGI0, SGI0, SGI0, SGI167987, SGI167987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37142[] = {SGI167989, SGI167990, SGI167991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167992, SGI0, SGI0, SGI167994, SGI167996, SGI167996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37143[] = {SGI0, SGI167993, SGI167995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI167997, SGI167997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37144[] = {SGI168001, SGI0, SGI168002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37145[] = {SGI167998, SGI167999, SGI168000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168003, SGI0, SGI0, SGI168004, SGI168005, SGI168005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37146[] = {SGI0, SGI168006, SGI168008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168011, SGI168011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37147[] = {SGI168007, SGI168009, SGI168010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168012, SGI0, SGI0, SGI168013, SGI168014, SGI168014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37148[] = {SGI168015, SGI168016, SGI168017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168019, SGI0, SGI0, SGI0, SGI168022, SGI168022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37149[] = {SGI168018, SGI168020, SGI168021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168023, SGI0, SGI0, SGI168024, SGI168025, SGI168025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37150[] = {SGI0, SGI168026, SGI168027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168031, SGI168031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37151[] = {SGI168028, SGI168029, SGI168030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168032, SGI0, SGI0, SGI168033, SGI168034, SGI168034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37152[] = {SGI0, SGI168035, SGI168036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168039, SGI168039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37153[] = {SGI168037, SGI168038, SGI168040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168041, SGI0, SGI0, SGI168042, SGI168043, SGI168043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37154[] = {SGI168044, SGI0, SGI168045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37155[] = {SGI168046, SGI168047, SGI168048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168049, SGI0, SGI0, SGI168050, SGI168051, SGI168051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37156[] = {SGI168052, SGI0, SGI168053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168054, SGI168055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37157[] = {SGI168056, SGI168057, SGI168058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168059, SGI0, SGI0, SGI168060, SGI168061, SGI168061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37158[] = {SGI168062, SGI168063, SGI168064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168065, SGI0, SGI0, SGI168066, SGI168067, SGI168067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37159[] = {SGI168068, SGI168069, SGI168070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168071, SGI0, SGI0, SGI0, SGI168072, SGI168072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37160[] = {SGI168073, SGI0, SGI168074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37161[] = {SGI168075, SGI0, SGI168076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168077, SGI168078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37162[] = {SGI168079, SGI0, SGI168080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168081, SGI168082, SGI168082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37163[] = {SGI168083, SGI0, SGI168084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168085, SGI168086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37164[] = {SGI168087, SGI0, SGI168088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168089, SGI168090, SGI168090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37165[] = {SGI168091, SGI168092, SGI168093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168094, SGI0, SGI0, SGI0, SGI168095, SGI168095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37166[] = {SGI168096, SGI0, SGI168099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168102, SGI168103, SGI168103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37167[] = {SGI168097, SGI168098, SGI168100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168101, SGI0, SGI0, SGI0, SGI168104, SGI168104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37168[] = {SGI168105, SGI168106, SGI168107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168108, SGI0, SGI0, SGI0, SGI168109, SGI168109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37169[] = {SGI168110, SGI168111, SGI168112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168113, SGI0, SGI0, SGI0, SGI168114, SGI168114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37170[] = {SGI168115, SGI168116, SGI168117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168118, SGI0, SGI0, SGI0, SGI168119, SGI168119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37171[] = {SGI168120, SGI0, SGI168121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168122, SGI168123, SGI168123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37172[] = {SGI168124, SGI0, SGI168125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168126, SGI168127, SGI168127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37173[] = {SGI168129, SGI168131, SGI168133, SGI168134, SGI168135, SGI168135, SGI0, NULL};
const Segment * const SG37174[] = {SGI168128, SGI168130, SGI168132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168136, SGI0, SGI0, SGI0, SGI168137, SGI0, SGI0, SGI168138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37175[] = {SGI168139, SGI168140, SGI168141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168142, SGI0, SGI0, SGI0, SGI168143, SGI0, SGI0, SGI168144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37176[] = {SGI168145, SGI168146, SGI168147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168148, SGI0, SGI0, SGI0, SGI168149, SGI168149, SGI0, SGI168150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37177[] = {SGI168151, SGI168152, SGI168153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168154, SGI0, SGI0, SGI0, SGI168155, SGI0, SGI0, SGI168156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37178[] = {SGI168157, SGI0, SGI168158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168159, SGI0, SGI0, SGI0, SGI168160, SGI0, SGI0, SGI168161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37179[] = {SGI168162, SGI168163, SGI168164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168165, SGI0, SGI0, SGI0, SGI168166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37180[] = {SGI168167, SGI0, SGI168168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37181[] = {SGI168169, SGI168170, SGI168171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168172, SGI0, SGI0, SGI0, SGI168173, SGI168173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37182[] = {SGI168174, SGI0, SGI168175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37183[] = {SGI168176, SGI168177, SGI168178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168179, SGI0, SGI0, SGI0, SGI168180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37184[] = {SGI168181, SGI168182, SGI168183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168184, SGI0, SGI0, SGI0, SGI168185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37185[] = {SGI168186, SGI0, SGI168187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37186[] = {SGI168188, SGI0, SGI168189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168190, SGI168191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37187[] = {SGI168192, SGI0, SGI168193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168194, SGI168195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37188[] = {SGI168196, SGI168197, SGI168198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168199, SGI0, SGI0, SGI0, SGI168200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37189[] = {SGI168201, SGI168202, SGI168203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168204, SGI0, SGI0, SGI0, SGI168205, SGI168205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37190[] = {SGI168207, SGI168208, SGI168210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168212, SGI0, SGI0, SGI0, SGI168213, SGI168213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37191[] = {SGI168209, SGI0, SGI168211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168214, SGI168215, SGI168215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37192[] = {SGI168217, SGI0, SGI168220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168223, SGI168224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37193[] = {SGI168218, SGI168219, SGI168221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168222, SGI0, SGI0, SGI0, SGI168225, SGI168225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37194[] = {SGI168227, SGI168228, SGI168229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168230, SGI0, SGI0, SGI0, SGI168231, SGI168231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37195[] = {SGI168233, SGI0, SGI168234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168236, SGI168238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37196[] = {SGI168235, SGI168237, SGI168239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168240, SGI0, SGI0, SGI0, SGI168241, SGI168241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37197[] = {SGI168243, SGI168244, SGI168245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168246, SGI0, SGI0, SGI0, SGI168247, SGI168247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37198[] = {SGI168249, SGI168250, SGI168251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168252, SGI0, SGI0, SGI0, SGI168253, SGI168253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37199[] = {SGI168255, SGI168256, SGI168257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168258, SGI0, SGI0, SGI0, SGI168259, SGI168259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37200[] = {SGI168261, SGI168262, SGI168263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168264, SGI0, SGI0, SGI0, SGI168265, SGI168265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37201[] = {SGI168267, SGI168268, SGI168269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168270, SGI0, SGI0, SGI0, SGI168271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37202[] = {SGI168273, SGI168274, SGI168275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168276, SGI0, SGI0, SGI0, SGI168277, SGI168277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37203[] = {SGI168279, SGI168280, SGI168281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168282, SGI0, SGI0, SGI0, SGI168283, SGI168283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37204[] = {SGI168287, SGI168288, SGI168289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168290, SGI0, SGI0, SGI0, SGI168291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37205[] = {SGI168293, SGI0, SGI168294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168295, SGI168296, SGI168296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37206[] = {SGI168297, SGI0, SGI168298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37207[] = {SGI168299, SGI168300, SGI168301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168302, SGI0, SGI0, SGI0, SGI168303, SGI168303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37208[] = {SGI168304, SGI168305, SGI168306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168310, SGI0, SGI0, SGI0, SGI168311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37209[] = {SGI168307, SGI168308, SGI168309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168312, SGI0, SGI0, SGI0, SGI168313, SGI168313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37210[] = {SGI168315, SGI168316, SGI168319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168321, SGI0, SGI0, SGI0, SGI168323, SGI168323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37211[] = {SGI168314, SGI168317, SGI168318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168320, SGI0, SGI0, SGI0, SGI168322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37212[] = {SGI168324, SGI168325, SGI168326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168327, SGI0, SGI0, SGI0, SGI168328, SGI168328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37213[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168329, SGI0, SGI0, SGI168330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37214[] = {SGI168331, SGI168332, SGI168333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168334, SGI0, SGI0, SGI0, SGI168335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37215[] = {SGI168336, SGI0, SGI168337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37216[] = {SGI168338, SGI0, SGI168341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168344, SGI168345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37217[] = {SGI168339, SGI168340, SGI168342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168343, SGI0, SGI0, SGI0, SGI168346, SGI168346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37218[] = {SGI168347, SGI168348, SGI168349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168350, SGI0, SGI0, SGI0, SGI168351, SGI168351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37219[] = {SGI168352, SGI168353, SGI168354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168355, SGI0, SGI0, SGI0, SGI168356, SGI168356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37220[] = {SGI168357, SGI0, SGI168358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168359, SGI168360, SGI168360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37221[] = {SGI168361, SGI0, SGI168362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37222[] = {SGI168363, SGI168364, SGI168366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168368, SGI0, SGI0, SGI0, SGI168369, SGI168369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37223[] = {SGI168365, SGI0, SGI168367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168370, SGI168371, SGI168371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37224[] = {SGI168372, SGI168373, SGI168374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168375, SGI0, SGI0, SGI0, SGI168376, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37225[] = {SGI168377, SGI0, SGI168380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168383, SGI168384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37226[] = {SGI168378, SGI168379, SGI168381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168382, SGI0, SGI0, SGI0, SGI168385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37227[] = {SGI168387, SGI168388, SGI168390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168391, SGI0, SGI0, SGI0, SGI168393, SGI168393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37228[] = {SGI168386, SGI0, SGI168389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168392, SGI168394, SGI168394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37229[] = {SGI168395, SGI0, SGI168396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168397, SGI168398, SGI168398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37230[] = {SGI168399, SGI0, SGI168400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168401, SGI168402, SGI168402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37231[] = {SGI168403, SGI0, SGI168404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168405, SGI168406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37232[] = {SGI168407, SGI0, SGI168408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168409, SGI168410, SGI168410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37233[] = {SGI168411, SGI0, SGI168412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168413, SGI168414, SGI168414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37234[] = {SGI168415, SGI0, SGI168416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168417, SGI168418, SGI168418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37235[] = {SGI168419, SGI0, SGI168420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168421, SGI168422, SGI168422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37236[] = {SGI168423, SGI0, SGI168424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168425, SGI168426, SGI168426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37237[] = {SGI168427, SGI0, SGI168428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168429, SGI168430, SGI168430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37238[] = {SGI168431, SGI0, SGI168432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168433, SGI168434, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37239[] = {SGI168435, SGI168436, SGI168437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168438, SGI0, SGI0, SGI0, SGI168439, SGI168439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37240[] = {SGI168440, SGI168441, SGI168442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168443, SGI0, SGI0, SGI0, SGI168444, SGI168444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37241[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37242[] = {SGI168446, SGI168447, SGI168448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168449, SGI0, SGI0, SGI0, SGI168450, SGI168450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37243[] = {SGI168451, SGI0, SGI168452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168453, SGI168454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37244[] = {SGI168455, SGI168456, SGI168457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168458, SGI0, SGI0, SGI0, SGI168459, SGI168459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37245[] = {SGI168460, SGI168461, SGI168462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168463, SGI0, SGI0, SGI0, SGI168464, SGI168464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37246[] = {SGI168465, SGI0, SGI168466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37247[] = {SGI168285, SGI168286, SGI168467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168468, SGI0, SGI0, SGI0, SGI168469, SGI168469, SGI0, SGI168472, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37248[] = {SGI168470, SGI168471, SGI168473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168474, SGI0, SGI0, SGI0, SGI168475, SGI168475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37249[] = {SGI168476, SGI168477, SGI168478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168479, SGI0, SGI0, SGI0, SGI168480, SGI168480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37250[] = {SGI168481, SGI0, SGI168482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37251[] = {SGI168483, SGI0, SGI168484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168485, SGI168486, SGI168486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37252[] = {SGI168487, SGI0, SGI168488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168489, SGI168490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37253[] = {SGI168491, SGI0, SGI168492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168495, SGI168496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37254[] = {SGI168493, SGI0, SGI168494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168497, SGI168498, SGI168498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37255[] = {SGI168499, SGI0, SGI168500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168501, SGI168502, SGI168502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37256[] = {SGI168503, SGI0, SGI168504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168505, SGI168506, SGI168506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37257[] = {SGI168507, SGI0, SGI168508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168509, SGI168510, SGI168510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37258[] = {SGI168511, SGI168512, SGI168513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168514, SGI0, SGI0, SGI0, SGI168515, SGI168515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37259[] = {SGI168517, SGI168518, SGI168519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168520, SGI0, SGI0, SGI0, SGI168521, SGI168521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37260[] = {SGI168523, SGI168524, SGI168525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168526, SGI0, SGI0, SGI0, SGI168527, SGI168527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37261[] = {SGI168529, SGI0, SGI168530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168531, SGI168532, SGI168532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37262[] = {SGI168533, SGI0, SGI168534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168535, SGI168536, SGI168536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37263[] = {SGI168537, SGI0, SGI168538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168539, SGI168540, SGI168540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37264[] = {SGI168541, SGI168542, SGI168543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168544, SGI0, SGI0, SGI0, SGI168545, SGI168545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37265[] = {SGI168546, SGI0, SGI168547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37266[] = {SGI168548, SGI168549, SGI168550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168551, SGI0, SGI0, SGI0, SGI168552, SGI168552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37267[] = {SGI168553, SGI0, SGI168554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168555, SGI168556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37268[] = {SGI168557, SGI168558, SGI168559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168560, SGI0, SGI0, SGI0, SGI168561, SGI168561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37269[] = {SGI168563, SGI168564, SGI168565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168566, SGI0, SGI0, SGI0, SGI168567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37270[] = {SGI168568, SGI0, SGI168569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37271[] = {SGI168570, SGI0, SGI168571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168572, SGI168573, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37272[] = {SGI168574, SGI0, SGI168575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168576, SGI168577, SGI168577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37273[] = {SGI168578, SGI0, SGI168579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168580, SGI168581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37274[] = {SGI168582, SGI168583, SGI168584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168585, SGI168585, SGI0, SGI0, SGI0, SGI168586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37275[] = {SGI168587, SGI168588, SGI168589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168590, SGI168590, SGI0, SGI0, SGI0, SGI168591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37276[] = {SGI168592, SGI168593, SGI168594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168595, SGI168595, SGI0, SGI0, SGI0, SGI168596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37277[] = {SGI168597, SGI168598, SGI168599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168600, SGI168600, SGI0, SGI0, SGI0, SGI168601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37278[] = {SGI168602, SGI168603, SGI168604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168605, SGI168605, SGI0, SGI0, SGI0, SGI168606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37279[] = {SGI168607, SGI168608, SGI168609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168610, SGI168610, SGI0, SGI0, SGI0, SGI168611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37280[] = {SGI168612, SGI0, SGI168613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168614, SGI168615, SGI168615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37281[] = {SGI168616, SGI0, SGI168617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168618, SGI168619, SGI168619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37282[] = {SGI168620, SGI0, SGI168621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168622, SGI168623, SGI168623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37283[] = {SGI168624, SGI0, SGI168625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168626, SGI168627, SGI168627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37284[] = {SGI168628, SGI0, SGI168629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168630, SGI168631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37285[] = {SGI168632, SGI168633, SGI168634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168635, SGI0, SGI0, SGI0, SGI168636, SGI168636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37286[] = {SGI168637, SGI0, SGI168638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168642, SGI168643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37287[] = {SGI168639, SGI168640, SGI168641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168644, SGI0, SGI0, SGI0, SGI168645, SGI168645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37288[] = {SGI168646, SGI0, SGI168647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168648, SGI168649, SGI168649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37289[] = {SGI168650, SGI0, SGI168651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168652, SGI168653, SGI168653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37290[] = {SGI168654, SGI0, SGI168655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168656, SGI168657, SGI168657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37291[] = {SGI168658, SGI168659, SGI168660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168661, SGI0, SGI0, SGI0, SGI168662, SGI168662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37292[] = {SGI168664, SGI168665, SGI168666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168667, SGI0, SGI0, SGI0, SGI168668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37293[] = {SGI168669, SGI168670, SGI168671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168672, SGI0, SGI0, SGI0, SGI168673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37294[] = {SGI168674, SGI0, SGI168675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37295[] = {SGI168676, SGI168677, SGI168678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168679, SGI0, SGI0, SGI0, SGI168680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37296[] = {SGI168681, SGI168682, SGI168683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168684, SGI0, SGI0, SGI0, SGI168685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37297[] = {SGI168686, SGI168687, SGI168688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168689, SGI0, SGI0, SGI0, SGI168690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37298[] = {SGI168691, SGI168692, SGI168693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168694, SGI0, SGI0, SGI0, SGI168695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37299[] = {SGI168696, SGI168697, SGI168698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168699, SGI0, SGI0, SGI0, SGI168700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37300[] = {SGI168701, SGI168702, SGI168703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168704, SGI0, SGI0, SGI0, SGI168705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37301[] = {SGI168706, SGI168707, SGI168708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168712, SGI0, SGI0, SGI0, SGI168713, SGI168713, SGI0, SGI168714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37302[] = {SGI168709, SGI168710, SGI168711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168715, SGI168715, SGI0, SGI0, SGI0, SGI168716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37303[] = {SGI168717, SGI168718, SGI168719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168720, SGI0, SGI0, SGI0, SGI168721, SGI168721, SGI0, SGI168722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37304[] = {SGI168723, SGI168724, SGI168725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168726, SGI168726, SGI0, SGI0, SGI0, SGI168727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37305[] = {SGI168728, SGI0, SGI168729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168730, SGI168731, SGI168731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37306[] = {SGI168732, SGI0, SGI168733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168734, SGI168735, SGI168735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37307[] = {SGI168736, SGI0, SGI168737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168739, SGI168740, SGI168740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37308[] = {SGI168738, SGI0, SGI168741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168742, SGI168743, SGI168743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37309[] = {SGI168744, SGI0, SGI168745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168746, SGI168747, SGI168747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37310[] = {SGI168748, SGI0, SGI168749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168750, SGI168751, SGI168751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37311[] = {SGI168752, SGI0, SGI168753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168754, SGI168755, SGI168755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37312[] = {SGI168756, SGI0, SGI168757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168758, SGI168759, SGI168759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37313[] = {SGI168760, SGI0, SGI168762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168764, SGI168765, SGI168765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37314[] = {SGI168761, SGI0, SGI168763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168766, SGI168767, SGI168767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37315[] = {SGI168768, SGI0, SGI168769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168770, SGI168771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37316[] = {SGI168772, SGI0, SGI168773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168774, SGI168775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37317[] = {SGI168776, SGI0, SGI168777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168778, SGI168779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37318[] = {SGI168780, SGI0, SGI168781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168782, SGI168783, SGI168783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37319[] = {SGI168784, SGI0, SGI168785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168786, SGI168787, SGI168787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37320[] = {SGI168788, SGI0, SGI168789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168790, SGI168791, SGI168791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37321[] = {SGI168792, SGI0, SGI168793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168794, SGI168795, SGI168795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37322[] = {SGI168796, SGI0, SGI168797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168798, SGI168799, SGI168799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37323[] = {SGI168800, SGI0, SGI168801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168802, SGI168803, SGI168803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37324[] = {SGI168804, SGI0, SGI168805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168806, SGI168807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37325[] = {SGI168808, SGI0, SGI168809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168813, SGI168814, SGI168814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37326[] = {SGI168810, SGI168811, SGI168812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168815, SGI0, SGI0, SGI0, SGI168817, SGI168817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37327[] = {SGI168816, SGI0, SGI168818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168819, SGI168820, SGI168820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37328[] = {SGI168821, SGI168822, SGI168823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168824, SGI0, SGI0, SGI0, SGI168825, SGI168825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37329[] = {SGI168826, SGI0, SGI168827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37330[] = {SGI168828, SGI168829, SGI168830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168831, SGI0, SGI0, SGI0, SGI168832, SGI168832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37331[] = {SGI168833, SGI168834, SGI168835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168837, SGI0, SGI0, SGI0, SGI168840, SGI168840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37332[] = {SGI168836, SGI168838, SGI168839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37333[] = {SGI168841, SGI0, SGI168842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37334[] = {SGI168843, SGI168844, SGI168845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168846, SGI0, SGI0, SGI0, SGI168847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37335[] = {SGI168848, SGI168849, SGI168850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168851, SGI0, SGI0, SGI0, SGI168852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37336[] = {SGI168853, SGI0, SGI168854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37337[] = {SGI168855, SGI168856, SGI168857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168858, SGI0, SGI0, SGI0, SGI168859, SGI168859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37338[] = {SGI168860, SGI0, SGI168861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168863, SGI168865, SGI168865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37339[] = {SGI168866, SGI0, SGI168867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37340[] = {SGI168862, SGI0, SGI168864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168868, SGI168869, SGI168869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37341[] = {SGI168870, SGI0, SGI168871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168873, SGI168875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37342[] = {SGI168872, SGI0, SGI168874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168876, SGI168877, SGI168877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37343[] = {SGI168878, SGI0, SGI168879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168880, SGI168881, SGI168881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37344[] = {SGI168882, SGI0, SGI168883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168884, SGI168885, SGI168885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37345[] = {SGI168886, SGI0, SGI168887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168888, SGI168889, SGI168889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37346[] = {SGI168890, SGI0, SGI168891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168892, SGI168893, SGI168893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37347[] = {SGI168894, SGI0, SGI168895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168896, SGI168897, SGI168897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37348[] = {SGI168898, SGI0, SGI168899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168900, SGI168901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37349[] = {SGI168902, SGI0, SGI168903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168904, SGI168905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37350[] = {SGI168906, SGI0, SGI168907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168909, SGI168911, SGI168911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37351[] = {SGI168908, SGI168910, SGI168912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168913, SGI0, SGI0, SGI0, SGI168914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37352[] = {SGI168917, SGI0, SGI168918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37353[] = {SGI168915, SGI0, SGI168916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168919, SGI168920, SGI168920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37354[] = {SGI168921, SGI168922, SGI168923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168924, SGI0, SGI0, SGI0, SGI168925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37355[] = {SGI168926, SGI168927, SGI168928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168929, SGI0, SGI0, SGI0, SGI168930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37356[] = {SGI168931, SGI0, SGI168932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168933, SGI168934, SGI168934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37357[] = {SGI168935, SGI0, SGI168936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168937, SGI168938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37358[] = {SGI168939, SGI0, SGI168940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168941, SGI168942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37359[] = {SGI168943, SGI168944, SGI168945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168946, SGI0, SGI0, SGI0, SGI168947, SGI168947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37360[] = {SGI168948, SGI168949, SGI168950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168951, SGI0, SGI0, SGI0, SGI168952, SGI168952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37361[] = {SGI168956, SGI0, SGI168957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37362[] = {SGI168953, SGI168954, SGI168955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168958, SGI0, SGI0, SGI0, SGI168959, SGI168959, SGI168960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37363[] = {SGI168961, SGI168962, SGI168963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168965, SGI0, SGI0, SGI0, SGI168968, SGI168968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37364[] = {SGI168964, SGI168966, SGI168967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168969, SGI0, SGI0, SGI0, SGI168970, SGI168970, SGI168971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37365[] = {SGI168972, SGI0, SGI168973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168974, SGI168975, SGI168975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37366[] = {SGI168976, SGI0, SGI168977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168978, SGI168979, SGI168979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37367[] = {SGI168980, SGI0, SGI168981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168982, SGI168983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37368[] = {SGI168984, SGI0, SGI168986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168988, SGI168989, SGI168989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37369[] = {SGI168985, SGI0, SGI168987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168990, SGI168991, SGI168991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37370[] = {SGI168992, SGI168993, SGI168994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI168995, SGI0, SGI0, SGI0, SGI168996, SGI168996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37371[] = {SGI168997, SGI168998, SGI168999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169000, SGI0, SGI0, SGI0, SGI169001, SGI169001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37372[] = {SGI169002, SGI169003, SGI169004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169005, SGI0, SGI0, SGI0, SGI169006, SGI169006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37373[] = {SGI169007, SGI0, SGI169008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37374[] = {SGI169009, SGI0, SGI169010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37375[] = {SGI169011, SGI169012, SGI169013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169014, SGI0, SGI0, SGI0, SGI169015, SGI169015, SGI169016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37376[] = {SGI169017, SGI169018, SGI169019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169020, SGI0, SGI0, SGI0, SGI169021, SGI169021, SGI169022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37377[] = {SGI169023, SGI169024, SGI169025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169026, SGI0, SGI0, SGI0, SGI169027, SGI169027, SGI169028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37378[] = {SGI169029, SGI0, SGI169030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169031, SGI169032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37379[] = {SGI169033, SGI169034, SGI169035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169039, SGI0, SGI0, SGI0, SGI169040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37380[] = {SGI169036, SGI169037, SGI169038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169041, SGI169041, SGI0, SGI0, SGI0, SGI169042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37381[] = {SGI169043, SGI169044, SGI169045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169049, SGI0, SGI0, SGI0, SGI169050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37382[] = {SGI169046, SGI169047, SGI169048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169051, SGI169051, SGI0, SGI0, SGI0, SGI169052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37383[] = {SGI169053, SGI0, SGI169054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37384[] = {SGI169055, SGI169056, SGI169057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169058, SGI0, SGI0, SGI0, SGI169059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37385[] = {SGI169060, SGI0, SGI169061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37386[] = {SGI169062, SGI0, SGI169064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169066, SGI169067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37387[] = {SGI169063, SGI0, SGI169065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169068, SGI169069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37388[] = {SGI169070, SGI0, SGI169072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169074, SGI169075, SGI169075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37389[] = {SGI169071, SGI0, SGI169073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169076, SGI169077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37390[] = {SGI169078, SGI169079, SGI169080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169081, SGI0, SGI0, SGI0, SGI169082, SGI169082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37391[] = {SGI169083, SGI169084, SGI169085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169086, SGI0, SGI0, SGI0, SGI169087, SGI169087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37392[] = {SGI169088, SGI169089, SGI169090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169091, SGI0, SGI0, SGI0, SGI169092, SGI169092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37393[] = {SGI169093, SGI169094, SGI169095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169096, SGI0, SGI0, SGI0, SGI169097, SGI169097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37394[] = {SGI169098, SGI0, SGI169099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37395[] = {SGI169100, SGI169101, SGI169102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169103, SGI0, SGI0, SGI0, SGI169104, SGI169104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37396[] = {SGI169105, SGI169106, SGI169107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169108, SGI0, SGI0, SGI0, SGI169109, SGI169109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37397[] = {SGI169110, SGI0, SGI169111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169112, SGI169113, SGI169113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37398[] = {SGI169114, SGI0, SGI169115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169116, SGI169117, SGI169117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37399[] = {SGI169118, SGI0, SGI169119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169120, SGI169121, SGI169121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37400[] = {SGI169122, SGI169123, SGI169124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169125, SGI0, SGI0, SGI0, SGI169126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37401[] = {SGI169127, SGI0, SGI169128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37402[] = {SGI169129, SGI169131, SGI169133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169135, SGI0, SGI0, SGI0, SGI169137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37403[] = {SGI169130, SGI169132, SGI169134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169136, SGI0, SGI0, SGI0, SGI169138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37404[] = {SGI169139, SGI169140, SGI169141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169142, SGI0, SGI0, SGI0, SGI169143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37405[] = {SGI169144, SGI0, SGI169145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169148, SGI169149, SGI169149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37406[] = {SGI169146, SGI0, SGI169147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169150, SGI0, SGI0, SGI0, SGI169151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37407[] = {SGI169152, SGI0, SGI169153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169156, SGI169157, SGI169157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37408[] = {SGI169154, SGI0, SGI169155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169158, SGI169159, SGI169159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37409[] = {SGI169160, SGI0, SGI169161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169164, SGI169165, SGI169165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37410[] = {SGI169162, SGI0, SGI169163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169166, SGI169167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37411[] = {SGI169168, SGI0, SGI169169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169170, SGI169171, SGI169171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37412[] = {SGI169172, SGI0, SGI169173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169174, SGI169175, SGI169175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37413[] = {SGI169176, SGI0, SGI169177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169178, SGI169179, SGI169179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37414[] = {SGI169180, SGI0, SGI169181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169182, SGI169183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37415[] = {SGI169184, SGI0, SGI169185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169186, SGI169187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37416[] = {SGI169188, SGI169189, SGI169190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169191, SGI0, SGI0, SGI0, SGI169192, SGI169192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37417[] = {SGI169193, SGI169194, SGI169196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169199, SGI0, SGI0, SGI0, SGI169200, SGI169200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37418[] = {SGI169195, SGI169197, SGI169198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169201, SGI0, SGI0, SGI0, SGI169202, SGI169202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37419[] = {SGI169203, SGI0, SGI169204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169205, SGI169206, SGI169206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37420[] = {SGI169207, SGI169208, SGI169209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169211, SGI0, SGI0, SGI0, SGI169213, SGI169213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37421[] = {SGI0, SGI0, SGI169210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169212, SGI0, SGI0, SGI0, SGI169214, SGI169214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37422[] = {SGI169215, SGI0, SGI169216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37423[] = {SGI169217, SGI0, SGI169218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37424[] = {SGI169219, SGI0, SGI169220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37425[] = {SGI169221, SGI0, SGI169222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169224, SGI169226, SGI169226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37426[] = {SGI169223, SGI169225, SGI169227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169228, SGI0, SGI0, SGI0, SGI169229, SGI169229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37427[] = {SGI169230, SGI0, SGI169231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169233, SGI169235, SGI169235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37428[] = {SGI169232, SGI169234, SGI169236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169237, SGI0, SGI0, SGI0, SGI169238, SGI169238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37429[] = {SGI169239, SGI169240, SGI169241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169242, SGI0, SGI0, SGI0, SGI169243, SGI169243, SGI0, SGI169244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37430[] = {SGI169245, SGI169246, SGI169247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169248, SGI0, SGI0, SGI0, SGI169249, SGI169249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37431[] = {SGI169250, SGI0, SGI169251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169255, SGI169256, SGI169256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37432[] = {SGI169252, SGI169253, SGI169254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169257, SGI0, SGI0, SGI0, SGI169258, SGI169258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37433[] = {SGI169259, SGI169260, SGI169261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169262, SGI0, SGI0, SGI0, SGI169263, SGI169263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37434[] = {SGI169264, SGI169265, SGI169266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169267, SGI0, SGI0, SGI0, SGI169268, SGI169268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37435[] = {SGI169269, SGI169270, SGI169271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169272, SGI0, SGI0, SGI0, SGI169273, SGI169273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37436[] = {SGI169274, SGI169275, SGI169276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169277, SGI0, SGI0, SGI0, SGI169278, SGI169278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37437[] = {SGI169279, SGI169280, SGI169281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169282, SGI0, SGI0, SGI0, SGI169284, SGI169284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37438[] = {SGI169283, SGI0, SGI169285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169286, SGI169287, SGI169287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37439[] = {SGI169288, SGI0, SGI169289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169290, SGI169291, SGI169291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37440[] = {SGI169292, SGI169293, SGI169294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169295, SGI0, SGI0, SGI0, SGI169296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37441[] = {SGI169297, SGI169298, SGI169299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169300, SGI0, SGI0, SGI0, SGI169301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37442[] = {SGI169302, SGI0, SGI169303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37443[] = {SGI169304, SGI169305, SGI169306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169307, SGI0, SGI0, SGI0, SGI169308, SGI169308, SGI0, SGI169309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37444[] = {SGI169310, SGI0, SGI169311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169314, SGI169316, SGI169316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37445[] = {SGI169312, SGI0, SGI169313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169318, SGI169319, SGI169319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37446[] = {SGI169315, SGI0, SGI169317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169320, SGI169321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37447[] = {SGI169322, SGI0, SGI169323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169324, SGI169325, SGI169325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37448[] = {SGI169326, SGI0, SGI169327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169328, SGI169329, SGI169329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37449[] = {SGI169330, SGI169331, SGI169332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169333, SGI0, SGI0, SGI0, SGI169334, SGI169334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37450[] = {SGI169335, SGI0, SGI169336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37451[] = {SGI169337, SGI169338, SGI169339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169340, SGI0, SGI0, SGI0, SGI169341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37452[] = {SGI169344, SGI0, SGI169345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37453[] = {SGI169342, SGI0, SGI169343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169346, SGI169347, SGI169347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37454[] = {SGI169348, SGI169349, SGI169350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169351, SGI0, SGI0, SGI0, SGI169352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37455[] = {SGI169353, SGI0, SGI169354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169355, SGI169356, SGI169356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37456[] = {SGI169357, SGI0, SGI169358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169359, SGI169360, SGI169360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37457[] = {SGI169361, SGI169362, SGI169363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169364, SGI0, SGI0, SGI0, SGI169365, SGI169365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37458[] = {SGI169367, SGI169368, SGI169369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169370, SGI0, SGI0, SGI0, SGI169371, SGI169371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37459[] = {SGI169373, SGI169374, SGI169376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169378, SGI0, SGI0, SGI0, SGI169379, SGI169379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37460[] = {SGI169375, SGI0, SGI169377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169380, SGI169381, SGI169381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37461[] = {SGI169384, SGI0, SGI169387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169389, SGI169391, SGI169391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37462[] = {SGI169383, SGI169385, SGI169386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169388, SGI0, SGI0, SGI0, SGI169390, SGI169390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37463[] = {SGI169393, SGI0, SGI169394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169395, SGI169396, SGI169396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37464[] = {SGI169397, SGI169398, SGI169399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169400, SGI0, SGI0, SGI0, SGI169401, SGI169401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37465[] = {SGI169403, SGI169404, SGI169405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169406, SGI0, SGI0, SGI0, SGI169407, SGI169407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37466[] = {SGI169409, SGI169410, SGI169411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169412, SGI0, SGI0, SGI0, SGI169413, SGI169413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37467[] = {SGI169415, SGI169416, SGI169417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169418, SGI0, SGI0, SGI0, SGI169419, SGI169419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37468[] = {SGI169421, SGI169422, SGI169423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169424, SGI0, SGI0, SGI0, SGI169425, SGI169425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37469[] = {SGI169426, SGI0, SGI169427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37470[] = {SGI169428, SGI169429, SGI169430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169431, SGI0, SGI0, SGI169432, SGI169433, SGI169433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37471[] = {SGI169434, SGI169435, SGI169436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169437, SGI0, SGI0, SGI169438, SGI169439, SGI169439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37472[] = {SGI169440, SGI169441, SGI169442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169443, SGI0, SGI0, SGI169444, SGI169445, SGI169445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37473[] = {SGI169446, SGI169447, SGI169448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169451, SGI0, SGI0, SGI0, SGI169452, SGI169452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37474[] = {SGI169449, SGI0, SGI169450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169453, SGI169454, SGI169454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37475[] = {SGI169455, SGI0, SGI169456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37476[] = {SGI169457, SGI169458, SGI169459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169460, SGI0, SGI0, SGI0, SGI169461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37477[] = {SGI169462, SGI0, SGI169463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169464, SGI169465, SGI169465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37478[] = {SGI169466, SGI0, SGI169467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169468, SGI169469, SGI169469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37479[] = {SGI169470, SGI0, SGI169471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169472, SGI169473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37480[] = {SGI169474, SGI0, SGI169475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169476, SGI169477, SGI169477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37481[] = {SGI169478, SGI0, SGI169479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169480, SGI169481, SGI169481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37482[] = {SGI169482, SGI169483, SGI169484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169488, SGI0, SGI0, SGI0, SGI169490, SGI169490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37483[] = {SGI169485, SGI169486, SGI169487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169489, SGI0, SGI0, SGI0, SGI169491, SGI169491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37484[] = {SGI169492, SGI0, SGI169494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37485[] = {SGI169493, SGI169495, SGI169496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169497, SGI0, SGI0, SGI0, SGI169498, SGI169498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37486[] = {SGI169499, SGI169500, SGI169501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169502, SGI0, SGI0, SGI0, SGI169503, SGI169503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37487[] = {SGI169504, SGI169505, SGI169506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169507, SGI0, SGI0, SGI0, SGI169508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37488[] = {SGI169509, SGI169510, SGI169511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169512, SGI0, SGI0, SGI0, SGI169513, SGI169513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37489[] = {SGI169514, SGI169515, SGI169516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169517, SGI0, SGI0, SGI0, SGI169518, SGI169518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37490[] = {SGI169520, SGI0, SGI169521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169522, SGI169523, SGI169523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37491[] = {SGI169524, SGI0, SGI169525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169526, SGI169527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37492[] = {SGI169528, SGI0, SGI169529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169530, SGI169531, SGI169531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37493[] = {SGI169532, SGI0, SGI169533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169534, SGI169535, SGI169535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37494[] = {SGI169536, SGI0, SGI169537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169538, SGI169539, SGI169539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37495[] = {SGI169540, SGI169541, SGI169542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169543, SGI0, SGI0, SGI0, SGI169544, SGI169544, SGI0, SGI169545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37496[] = {SGI169546, SGI169547, SGI169548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169549, SGI0, SGI0, SGI0, SGI169550, SGI169550, SGI0, SGI169551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37497[] = {SGI169552, SGI0, SGI169553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169554, SGI169555, SGI169555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37498[] = {SGI169556, SGI0, SGI169557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169558, SGI169559, SGI169559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37499[] = {SGI169560, SGI0, SGI169561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169562, SGI169563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37500[] = {SGI169564, SGI0, SGI169565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169566, SGI169567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37501[] = {SGI169568, SGI0, SGI169569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169570, SGI169571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37502[] = {SGI169572, SGI169573, SGI169574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169575, SGI169575, SGI0, SGI0, SGI169576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37503[] = {SGI169577, SGI169578, SGI169579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169580, SGI169580, SGI0, SGI0, SGI169581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37504[] = {SGI169582, SGI0, SGI169583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169584, SGI169585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37505[] = {SGI169586, SGI0, SGI169587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169588, SGI169589, SGI169589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37506[] = {SGI169590, SGI169591, SGI169592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169593, SGI0, SGI0, SGI0, SGI169594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37507[] = {SGI169595, SGI169596, SGI169597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169598, SGI0, SGI0, SGI0, SGI169599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37508[] = {SGI169600, SGI169601, SGI169602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169603, SGI0, SGI0, SGI0, SGI169604, SGI169604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37509[] = {SGI169605, SGI169606, SGI169607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169608, SGI0, SGI0, SGI0, SGI169609, SGI169609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37510[] = {SGI169610, SGI169611, SGI169612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169613, SGI169613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37511[] = {SGI169614, SGI169615, SGI169616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169617, SGI0, SGI0, SGI0, SGI169618, SGI169618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37512[] = {SGI169619, SGI0, SGI169620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37513[] = {SGI169621, SGI0, SGI169622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37514[] = {SGI169623, SGI169624, SGI169625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169626, SGI0, SGI0, SGI0, SGI169627, SGI169627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37515[] = {SGI169628, SGI169629, SGI169630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169631, SGI0, SGI0, SGI0, SGI169632, SGI169632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37516[] = {SGI169634, SGI169635, SGI169636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169637, SGI0, SGI0, SGI0, SGI169638, SGI169638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37517[] = {SGI169640, SGI169641, SGI169642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169643, SGI0, SGI0, SGI0, SGI169644, SGI169644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37518[] = {SGI169646, SGI0, SGI169647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169650, SGI169651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37519[] = {SGI169648, SGI0, SGI169649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169652, SGI169653, SGI169653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37520[] = {SGI169654, SGI0, SGI169655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169656, SGI169657, SGI169657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37521[] = {SGI169658, SGI0, SGI169659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169660, SGI169661, SGI169661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37522[] = {SGI169662, SGI169663, SGI169664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169665, SGI0, SGI0, SGI0, SGI169666, SGI169666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37523[] = {SGI169668, SGI0, SGI169671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169673, SGI169675, SGI169675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37524[] = {SGI169667, SGI169669, SGI169670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169672, SGI0, SGI0, SGI0, SGI169674, SGI169674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37525[] = {SGI169676, SGI0, SGI169677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169678, SGI0, SGI0, SGI0, SGI169679, SGI169679, SGI0, SGI0, SGI0, SGI169680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37526[] = {SGI169681, SGI0, SGI169682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169683, SGI169684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37527[] = {SGI169685, SGI169686, SGI169687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169688, SGI0, SGI0, SGI0, SGI169689, SGI169690, SGI0, SGI0, SGI0, SGI169691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37528[] = {SGI169692, SGI0, SGI169693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169694, SGI169695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37529[] = {SGI169696, SGI0, SGI169697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169698, SGI169699, SGI169699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37530[] = {SGI169700, SGI0, SGI169701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169702, SGI169703, SGI169703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37531[] = {SGI169704, SGI0, SGI169705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169706, SGI169707, SGI169707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37532[] = {SGI169708, SGI0, SGI169709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169710, SGI169711, SGI169711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37533[] = {SGI169712, SGI0, SGI169713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169714, SGI169716, SGI169716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37534[] = {SGI169715, SGI169717, SGI169718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169719, SGI0, SGI0, SGI0, SGI169720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37535[] = {SGI169723, SGI0, SGI169724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37536[] = {SGI169721, SGI0, SGI169722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169725, SGI169726, SGI169726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37537[] = {SGI169727, SGI0, SGI169728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169729, SGI169730, SGI169730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37538[] = {SGI169731, SGI0, SGI169732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169733, SGI169734, SGI169734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37539[] = {SGI169735, SGI0, SGI169736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169737, SGI169738, SGI169738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37540[] = {SGI169739, SGI0, SGI169740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169741, SGI169742, SGI169742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37541[] = {SGI169743, SGI0, SGI169744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169745, SGI169746, SGI169746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37542[] = {SGI169747, SGI0, SGI169748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169749, SGI169750, SGI169750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37543[] = {SGI169751, SGI0, SGI169752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169753, SGI169754, SGI169754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37544[] = {SGI169755, SGI0, SGI169756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169757, SGI169758, SGI169758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37545[] = {SGI169759, SGI0, SGI169760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169761, SGI169762, SGI169762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37546[] = {SGI169763, SGI0, SGI169764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169765, SGI169766, SGI169766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37547[] = {SGI169767, SGI169768, SGI169769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169770, SGI0, SGI0, SGI0, SGI169771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37548[] = {SGI169772, SGI169773, SGI169774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169775, SGI0, SGI0, SGI0, SGI169776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37549[] = {SGI169777, SGI0, SGI169778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37550[] = {SGI169779, SGI169780, SGI169781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169782, SGI0, SGI0, SGI0, SGI169783, SGI169783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37551[] = {SGI169784, SGI169785, SGI169786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169787, SGI0, SGI0, SGI0, SGI169788, SGI169788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37552[] = {SGI169789, SGI169790, SGI169791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169792, SGI0, SGI0, SGI0, SGI169793, SGI169793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37553[] = {SGI169794, SGI0, SGI169795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37554[] = {SGI169796, SGI0, SGI169797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169798, SGI169799, SGI169799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37555[] = {SGI169800, SGI0, SGI169801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169802, SGI169803, SGI169803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37556[] = {SGI169804, SGI169805, SGI169806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169807, SGI0, SGI0, SGI0, SGI169808, SGI169808, SGI169809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37557[] = {SGI169810, SGI169811, SGI169812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169813, SGI0, SGI0, SGI0, SGI169814, SGI169814, SGI169815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37558[] = {SGI169816, SGI169817, SGI169818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169819, SGI0, SGI0, SGI0, SGI169820, SGI169820, SGI169821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37559[] = {SGI169822, SGI169823, SGI169824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169825, SGI0, SGI0, SGI0, SGI169826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37560[] = {SGI169827, SGI169828, SGI169829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169830, SGI0, SGI0, SGI0, SGI169831, SGI169831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37561[] = {SGI169832, SGI169833, SGI169834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169835, SGI0, SGI0, SGI0, SGI0, SGI169836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37562[] = {SGI169837, SGI0, SGI169838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37563[] = {SGI169839, SGI169840, SGI169841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169842, SGI0, SGI0, SGI0, SGI0, SGI169843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37564[] = {SGI169844, SGI169845, SGI169846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169847, SGI0, SGI0, SGI0, SGI0, SGI169848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37565[] = {SGI169849, SGI169850, SGI169851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169852, SGI0, SGI0, SGI0, SGI0, SGI169853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37566[] = {SGI169854, SGI169855, SGI169856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169857, SGI0, SGI0, SGI0, SGI0, SGI169858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37567[] = {SGI169859, SGI169860, SGI169861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169862, SGI0, SGI0, SGI0, SGI0, SGI169863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37568[] = {SGI169864, SGI169865, SGI169866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169867, SGI0, SGI0, SGI0, SGI0, SGI169868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37569[] = {SGI169870, SGI169872, SGI169873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169875, SGI0, SGI0, SGI0, SGI169877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37570[] = {SGI169869, SGI169871, SGI169874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169876, SGI0, SGI0, SGI0, SGI0, SGI169878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37571[] = {SGI169879, SGI0, SGI169880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37572[] = {SGI169881, SGI169882, SGI169883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169884, SGI0, SGI0, SGI0, SGI169885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37573[] = {SGI169886, SGI0, SGI169887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37574[] = {SGI169888, SGI0, SGI169889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169890, SGI169891, SGI169891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37575[] = {SGI169892, SGI0, SGI169893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169894, SGI169895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37576[] = {SGI169896, SGI169897, SGI169898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169899, SGI0, SGI0, SGI0, SGI169900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37577[] = {SGI169901, SGI169902, SGI169903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169904, SGI0, SGI0, SGI0, SGI169905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37578[] = {SGI169906, SGI0, SGI169907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169908, SGI169909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37579[] = {SGI169910, SGI0, SGI169911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169912, SGI169913, SGI169913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37580[] = {SGI169914, SGI0, SGI169915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169918, SGI169919, SGI169919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37581[] = {SGI169916, SGI0, SGI169917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169920, SGI169921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37582[] = {SGI169922, SGI0, SGI169923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169924, SGI169925, SGI169925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37583[] = {SGI169927, SGI169929, SGI169931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169932, SGI0, SGI0, SGI0, SGI169934, SGI169934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37584[] = {SGI169926, SGI169928, SGI169930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169933, SGI169933, SGI0, SGI0, SGI169935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37585[] = {SGI169936, SGI0, SGI169937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37586[] = {SGI169938, SGI169939, SGI169940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169941, SGI0, SGI0, SGI0, SGI169942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37587[] = {SGI169943, SGI169944, SGI169945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169946, SGI0, SGI0, SGI0, SGI169948, SGI169948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37588[] = {SGI169947, SGI169949, SGI169950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169951, SGI169951, SGI0, SGI0, SGI169952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37589[] = {SGI169953, SGI169954, SGI169955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169956, SGI0, SGI0, SGI0, SGI169957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37590[] = {SGI169958, SGI0, SGI169960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169962, SGI0, SGI0, SGI0, SGI169963, SGI169963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37591[] = {SGI169959, SGI0, SGI169961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169964, SGI169965, SGI169965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37592[] = {SGI169966, SGI169968, SGI169969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169971, SGI0, SGI0, SGI0, SGI169973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37593[] = {SGI169967, SGI0, SGI169970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169972, SGI169974, SGI169974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37594[] = {SGI169976, SGI169977, SGI169979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169980, SGI0, SGI0, SGI0, SGI169983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37595[] = {SGI169975, SGI0, SGI169978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169981, SGI169982, SGI169982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37596[] = {SGI169984, SGI0, SGI169985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169986, SGI169987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37597[] = {SGI169988, SGI169989, SGI169990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169991, SGI0, SGI0, SGI0, SGI169992, SGI169992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37598[] = {SGI169993, SGI0, SGI169994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37599[] = {SGI169995, SGI169996, SGI169997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI169998, SGI169998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37600[] = {SGI169999, SGI0, SGI170000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170001, SGI170002, SGI170002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37601[] = {SGI170003, SGI170004, SGI170005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170006, SGI0, SGI0, SGI0, SGI170007, SGI170007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37602[] = {SGI170008, SGI170009, SGI170010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170011, SGI0, SGI0, SGI0, SGI170012, SGI170012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37603[] = {SGI170013, SGI0, SGI170014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170015, SGI170016, SGI170016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170017, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37604[] = {SGI170018, SGI0, SGI170019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170020, SGI170021, SGI170021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37605[] = {SGI170022, SGI0, SGI170023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170024, SGI170025, SGI170025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37606[] = {SGI170026, SGI170027, SGI170028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170029, SGI0, SGI0, SGI0, SGI170030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37607[] = {SGI170031, SGI170032, SGI170033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170034, SGI0, SGI0, SGI0, SGI170035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37608[] = {SGI170036, SGI170037, SGI170038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170039, SGI0, SGI0, SGI0, SGI170040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37609[] = {SGI170041, SGI170042, SGI170043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170044, SGI0, SGI0, SGI0, SGI170045, SGI170045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37610[] = {SGI170046, SGI0, SGI170047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170048, SGI170049, SGI170049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37611[] = {SGI170050, SGI170051, SGI170052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170053, SGI0, SGI0, SGI0, SGI170054, SGI170054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37612[] = {SGI170055, SGI170056, SGI170057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170058, SGI0, SGI0, SGI0, SGI170059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37613[] = {SGI170060, SGI170061, SGI170062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170063, SGI0, SGI0, SGI0, SGI170064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37614[] = {SGI170065, SGI170066, SGI170067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170068, SGI0, SGI0, SGI0, SGI170069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37615[] = {SGI170070, SGI170071, SGI170072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170073, SGI0, SGI0, SGI0, SGI170074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37616[] = {SGI170075, SGI0, SGI170076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170077, SGI170078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37617[] = {SGI170079, SGI0, SGI170080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170081, SGI170082, SGI170082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37618[] = {SGI170083, SGI0, SGI170084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170085, SGI170086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37619[] = {SGI170087, SGI0, SGI170088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170089, SGI170090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37620[] = {SGI170091, SGI170092, SGI170093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170094, SGI0, SGI0, SGI0, SGI170095, SGI170095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37621[] = {SGI170096, SGI170097, SGI170098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170099, SGI170099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37622[] = {SGI170100, SGI0, SGI170101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170102, SGI170103, SGI170103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37623[] = {SGI170104, SGI0, SGI170105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37624[] = {SGI0, SGI0, SGI170106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170107, SGI0, SGI0, SGI0, SGI170108, SGI170108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37625[] = {SGI170109, SGI0, SGI170110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170111, SGI170112, SGI170112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37626[] = {SGI170113, SGI0, SGI170114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170115, SGI170116, SGI170116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37627[] = {SGI170117, SGI0, SGI170118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170119, SGI170120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37628[] = {SGI170121, SGI170122, SGI170123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170124, SGI0, SGI0, SGI0, SGI170126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37629[] = {SGI170125, SGI0, SGI170127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170128, SGI170129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37630[] = {SGI170130, SGI0, SGI170131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170133, SGI170134, SGI170134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37631[] = {SGI170132, SGI0, SGI170135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170136, SGI170137, SGI170137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37632[] = {SGI170138, SGI0, SGI170139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170140, SGI170141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37633[] = {SGI170142, SGI0, SGI170143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170144, SGI170145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37634[] = {SGI170146, SGI0, SGI170147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170148, SGI170149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37635[] = {SGI170150, SGI0, SGI170151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170152, SGI170153, SGI170153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37636[] = {SGI170154, SGI0, SGI170155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170156, SGI170157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37637[] = {SGI170158, SGI0, SGI170159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170160, SGI170161, SGI170161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37638[] = {SGI170162, SGI0, SGI170163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170164, SGI170165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37639[] = {SGI170166, SGI0, SGI170167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170168, SGI170169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37640[] = {SGI170170, SGI0, SGI170171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170172, SGI170173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37641[] = {SGI170174, SGI0, SGI170175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170176, SGI170177, SGI170177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37642[] = {SGI170178, SGI0, SGI170179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170180, SGI170181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37643[] = {SGI170182, SGI0, SGI170183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170184, SGI170185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37644[] = {SGI170186, SGI0, SGI170187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170188, SGI170189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37645[] = {SGI170190, SGI0, SGI170191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170192, SGI170193, SGI170193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37646[] = {SGI170194, SGI0, SGI170195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170196, SGI170197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37647[] = {SGI170198, SGI170199, SGI170200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170201, SGI0, SGI0, SGI0, SGI170202, SGI170202, SGI0, SGI170203, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37648[] = {SGI170204, SGI170205, SGI170206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170207, SGI0, SGI0, SGI0, SGI170209, SGI170209, SGI0, SGI170211, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37649[] = {SGI170208, SGI0, SGI170210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170212, SGI170213, SGI170213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37650[] = {SGI170214, SGI0, SGI170215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170216, SGI170217, SGI170217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37651[] = {SGI170218, SGI0, SGI170219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170220, SGI170221, SGI170221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37652[] = {SGI170222, SGI0, SGI170223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170224, SGI170225, SGI170225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37653[] = {SGI170226, SGI0, SGI170227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170228, SGI170229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37654[] = {SGI170230, SGI170231, SGI170232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170233, SGI0, SGI0, SGI0, SGI170234, SGI170234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37655[] = {SGI170236, SGI170237, SGI170238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170239, SGI0, SGI0, SGI0, SGI170240, SGI170240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37656[] = {SGI170242, SGI0, SGI170243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170244, SGI170245, SGI170245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37657[] = {SGI170246, SGI170247, SGI170248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170249, SGI0, SGI0, SGI0, SGI170250, SGI170250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37658[] = {SGI170252, SGI170253, SGI170254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170255, SGI0, SGI0, SGI0, SGI170256, SGI170256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37659[] = {SGI170258, SGI170259, SGI170260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170261, SGI0, SGI0, SGI0, SGI170262, SGI170262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37660[] = {SGI170264, SGI0, SGI170265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170266, SGI170267, SGI170267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37661[] = {SGI170268, SGI0, SGI170269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170270, SGI170271, SGI170271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37662[] = {SGI170272, SGI0, SGI170273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170274, SGI170275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37663[] = {SGI170276, SGI0, SGI170277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170278, SGI170279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37664[] = {SGI170280, SGI0, SGI170281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170282, SGI170283, SGI170283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37665[] = {SGI170284, SGI0, SGI170285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170286, SGI170287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37666[] = {SGI170288, SGI0, SGI170289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170290, SGI170291, SGI170291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37667[] = {SGI170292, SGI0, SGI170293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170294, SGI170295, SGI170295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37668[] = {SGI170296, SGI0, SGI170297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170298, SGI170299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37669[] = {SGI170300, SGI0, SGI170301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170302, SGI170304, SGI170304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37670[] = {SGI170303, SGI0, SGI170305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170306, SGI170307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37671[] = {SGI170308, SGI0, SGI170309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170310, SGI170311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37672[] = {SGI170312, SGI0, SGI170313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170314, SGI170315, SGI170315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37673[] = {SGI170316, SGI0, SGI170317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170318, SGI170319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37674[] = {SGI170320, SGI0, SGI170321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170322, SGI170323, SGI170323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37675[] = {SGI170324, SGI0, SGI170325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170326, SGI170327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37676[] = {SGI170328, SGI0, SGI170329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170333, SGI170334, SGI170334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37677[] = {SGI170330, SGI170331, SGI170332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170335, SGI0, SGI0, SGI0, SGI170336, SGI170336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37678[] = {SGI170338, SGI0, SGI170339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170343, SGI170344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37679[] = {SGI170340, SGI170341, SGI170342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170345, SGI0, SGI0, SGI0, SGI170346, SGI170346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37680[] = {SGI170348, SGI170349, SGI170350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170351, SGI0, SGI0, SGI0, SGI170352, SGI170352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37681[] = {SGI170353, SGI0, SGI170355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170356, SGI170357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37682[] = {SGI170358, SGI170359, SGI170360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170361, SGI0, SGI0, SGI0, SGI170363, SGI170363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37683[] = {SGI170362, SGI0, SGI170364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170366, SGI170367, SGI170367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37684[] = {SGI170368, SGI0, SGI170369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170372, SGI170373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37685[] = {SGI170370, SGI0, SGI170371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170374, SGI170375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37686[] = {SGI170376, SGI0, SGI170377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170378, SGI170379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37687[] = {SGI170380, SGI0, SGI170381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170382, SGI170383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37688[] = {SGI170384, SGI0, SGI170385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170388, SGI170389, SGI170389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37689[] = {SGI170386, SGI0, SGI170387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170390, SGI170391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37690[] = {SGI170392, SGI0, SGI170393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170396, SGI170397, SGI170397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37691[] = {SGI170394, SGI0, SGI170395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170398, SGI170399, SGI170399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37692[] = {SGI170400, SGI0, SGI170401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170402, SGI170403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37693[] = {SGI170404, SGI0, SGI170405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170406, SGI170407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37694[] = {SGI170408, SGI0, SGI170409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170410, SGI170411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37695[] = {SGI170412, SGI0, SGI170413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170414, SGI170415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37696[] = {SGI170416, SGI0, SGI170417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170421, SGI170422, SGI170422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37697[] = {SGI170418, SGI170419, SGI170420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170423, SGI0, SGI0, SGI0, SGI170424, SGI170424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37698[] = {SGI170425, SGI0, SGI170426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170430, SGI170431, SGI170431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37699[] = {SGI170427, SGI170428, SGI170429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170432, SGI0, SGI0, SGI0, SGI170433, SGI170433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37700[] = {SGI170437, SGI0, SGI170439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170441, SGI170443, SGI170443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37701[] = {SGI170440, SGI0, SGI170442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170444, SGI170445, SGI170445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37702[] = {SGI170446, SGI0, SGI170447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170450, SGI170451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37703[] = {SGI170448, SGI0, SGI170449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170452, SGI170453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37704[] = {SGI170454, SGI0, SGI170455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170456, SGI170457, SGI170457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37705[] = {SGI170458, SGI170459, SGI170460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170461, SGI0, SGI0, SGI0, SGI170462, SGI0, SGI0, SGI170463, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37706[] = {SGI170464, SGI170465, SGI170466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170467, SGI0, SGI0, SGI0, SGI170468, SGI0, SGI0, SGI170469, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37707[] = {SGI170470, SGI0, SGI170471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170472, SGI170473, SGI170473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37708[] = {SGI170474, SGI0, SGI170475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170479, SGI170480, SGI170480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37709[] = {SGI170476, SGI170477, SGI170478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170481, SGI0, SGI0, SGI0, SGI170482, SGI170482, SGI0, SGI170483, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37710[] = {SGI170484, SGI170485, SGI170486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170487, SGI0, SGI0, SGI0, SGI170488, SGI170488, SGI0, SGI170489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37711[] = {SGI170490, SGI170491, SGI170492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170493, SGI0, SGI0, SGI0, SGI170494, SGI170494, SGI0, SGI170495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37712[] = {SGI0, SGI170496, SGI170497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170498, SGI0, SGI0, SGI0, SGI170499, SGI170499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37713[] = {SGI170500, SGI170501, SGI170502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170503, SGI0, SGI0, SGI0, SGI170504, SGI170504, SGI0, SGI170505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37714[] = {SGI0, SGI170506, SGI170507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170508, SGI0, SGI0, SGI0, SGI170509, SGI170509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37715[] = {SGI170510, SGI170511, SGI170512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170513, SGI0, SGI0, SGI0, SGI170514, SGI170514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37716[] = {SGI170515, SGI0, SGI170516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37717[] = {SGI170517, SGI0, SGI170518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170519, SGI170520, SGI170520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37718[] = {SGI170521, SGI0, SGI170522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170523, SGI170524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37719[] = {SGI170525, SGI0, SGI170526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170527, SGI170528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37720[] = {SGI170529, SGI0, SGI170530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170531, SGI170532, SGI170532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37721[] = {SGI170533, SGI0, SGI170534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170535, SGI170536, SGI170536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37722[] = {SGI170537, SGI170538, SGI170539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170540, SGI0, SGI0, SGI170541, SGI170542, SGI170542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37723[] = {SGI170543, SGI0, SGI170544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170545, SGI170546, SGI170546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37724[] = {SGI170547, SGI0, SGI170548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170549, SGI170550, SGI170550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37725[] = {SGI170551, SGI0, SGI170552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170553, SGI170554, SGI170554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37726[] = {SGI170555, SGI170556, SGI170557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170558, SGI0, SGI0, SGI0, SGI170559, SGI170559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37727[] = {SGI170560, SGI170561, SGI170562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170563, SGI0, SGI0, SGI0, SGI170564, SGI170564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37728[] = {SGI170565, SGI0, SGI170566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37729[] = {SGI170567, SGI170568, SGI170569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170570, SGI0, SGI0, SGI0, SGI170571, SGI170571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37730[] = {SGI170572, SGI0, SGI170573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170574, SGI170575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37731[] = {SGI170576, SGI0, SGI170577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170578, SGI170579, SGI170579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37732[] = {SGI170580, SGI0, SGI170581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170583, SGI170585, SGI170585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37733[] = {SGI170582, SGI0, SGI170584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170586, SGI170587, SGI170587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170588, SGI0, SGI0, NULL};
const Segment * const SG37734[] = {SGI170589, SGI0, SGI170590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170591, SGI170592, SGI170592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170593, SGI0, SGI0, NULL};
const Segment * const SG37735[] = {SGI170434, SGI0, SGI170435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170436, SGI170438, SGI170438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37736[] = {SGI170596, SGI0, SGI170597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170598, SGI170599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37737[] = {SGI170600, SGI0, SGI170601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170602, SGI170603, SGI170603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37738[] = {SGI170604, SGI0, SGI170605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170606, SGI170607, SGI170607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37739[] = {SGI170608, SGI0, SGI170609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170610, SGI170611, SGI170611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37740[] = {SGI170612, SGI170613, SGI170614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170615, SGI0, SGI0, SGI0, SGI170616, SGI170616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37741[] = {SGI170617, SGI170618, SGI170619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170620, SGI0, SGI0, SGI0, SGI170621, SGI170621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37742[] = {SGI170622, SGI0, SGI170623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37743[] = {SGI170624, SGI0, SGI170625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170626, SGI170627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37744[] = {SGI170628, SGI170629, SGI170630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170631, SGI0, SGI0, SGI0, SGI170632, SGI170632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37745[] = {SGI170594, SGI0, SGI170595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170635, SGI170637, SGI170637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170639, SGI0, SGI0, NULL};
const Segment * const SG37746[] = {SGI170633, SGI170634, SGI170636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170638, SGI0, SGI0, SGI0, SGI170640, SGI170640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37747[] = {SGI170643, SGI0, SGI170644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37748[] = {SGI170641, SGI0, SGI170642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170645, SGI170646, SGI170646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170647, SGI0, SGI0, NULL};
const Segment * const SG37749[] = {SGI170648, SGI0, SGI170649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170650, SGI170651, SGI170651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37750[] = {SGI170652, SGI0, SGI170653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170654, SGI170655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37751[] = {SGI170656, SGI170657, SGI170658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170659, SGI0, SGI0, SGI0, SGI170660, SGI170660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37752[] = {SGI170662, SGI170663, SGI170664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170665, SGI0, SGI0, SGI0, SGI170666, SGI170666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37753[] = {SGI170668, SGI170669, SGI170670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170671, SGI0, SGI0, SGI0, SGI170672, SGI170672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37754[] = {SGI170674, SGI170675, SGI170676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170677, SGI0, SGI0, SGI0, SGI170678, SGI170678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37755[] = {SGI170679, SGI170680, SGI170681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170682, SGI170682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37756[] = {SGI170685, SGI170686, SGI170687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170688, SGI0, SGI0, SGI0, SGI170689, SGI170689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37757[] = {SGI170690, SGI0, SGI170691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37758[] = {SGI170692, SGI0, SGI170693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170694, SGI170695, SGI170695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37759[] = {SGI170696, SGI0, SGI170697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170698, SGI170699, SGI170699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37760[] = {SGI170700, SGI0, SGI170701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170702, SGI170703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37761[] = {SGI170704, SGI0, SGI170705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170706, SGI170707, SGI170707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37762[] = {SGI170708, SGI170709, SGI170710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170711, SGI0, SGI0, SGI0, SGI170712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37763[] = {SGI170713, SGI0, SGI170714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170715, SGI170716, SGI170716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37764[] = {SGI170717, SGI0, SGI170718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170719, SGI170720, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37765[] = {SGI170721, SGI170722, SGI170723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170724, SGI0, SGI0, SGI0, SGI170725, SGI170725, SGI0, SGI170726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37766[] = {SGI170727, SGI0, SGI170728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170729, SGI170730, SGI170730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37767[] = {SGI170731, SGI170732, SGI170733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170734, SGI0, SGI0, SGI0, SGI170735, SGI170735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37768[] = {SGI170736, SGI0, SGI170737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37769[] = {SGI170738, SGI0, SGI170739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170740, SGI170741, SGI170741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37770[] = {SGI170742, SGI0, SGI170743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170744, SGI170745, SGI170745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37771[] = {SGI170683, SGI0, SGI170684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37772[] = {SGI170746, SGI0, SGI170747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170748, SGI170749, SGI170749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37773[] = {SGI170750, SGI0, SGI170751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170752, SGI170753, SGI170753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37774[] = {SGI170754, SGI0, SGI170755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170756, SGI170757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37775[] = {SGI170758, SGI0, SGI170759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170760, SGI170761, SGI170761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37776[] = {SGI170762, SGI170763, SGI170764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170765, SGI0, SGI0, SGI0, SGI170766, SGI170766, SGI0, SGI170767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37777[] = {SGI170768, SGI170769, SGI170770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170771, SGI0, SGI0, SGI0, SGI170772, SGI170772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37778[] = {SGI170773, SGI170774, SGI170775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170776, SGI0, SGI0, SGI0, SGI170777, SGI170777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37779[] = {SGI170778, SGI170779, SGI170780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170781, SGI0, SGI0, SGI0, SGI170782, SGI170782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37780[] = {SGI170783, SGI170784, SGI170785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170786, SGI0, SGI0, SGI0, SGI170787, SGI170787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37781[] = {SGI170788, SGI170789, SGI170790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170791, SGI0, SGI0, SGI0, SGI170792, SGI170792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37782[] = {SGI170793, SGI170794, SGI170795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170796, SGI0, SGI0, SGI0, SGI170797, SGI170797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37783[] = {SGI170798, SGI170799, SGI170800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170801, SGI0, SGI0, SGI0, SGI170802, SGI170802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37784[] = {SGI170803, SGI0, SGI170804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37785[] = {SGI170805, SGI170806, SGI170807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170808, SGI0, SGI0, SGI0, SGI170809, SGI170809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37786[] = {SGI170810, SGI0, SGI170811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170812, SGI170813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37787[] = {SGI170814, SGI0, SGI170815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170816, SGI170817, SGI170817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37788[] = {SGI170818, SGI0, SGI170819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170820, SGI170821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37789[] = {SGI170822, SGI0, SGI170823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170824, SGI170825, SGI170825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37790[] = {SGI170826, SGI0, SGI170827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170830, SGI170831, SGI170831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37791[] = {SGI170828, SGI0, SGI170829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170832, SGI170833, SGI170833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37792[] = {SGI170834, SGI0, SGI170835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170838, SGI170839, SGI170839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37793[] = {SGI170836, SGI0, SGI170837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170840, SGI170841, SGI170841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37794[] = {SGI170842, SGI0, SGI170843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170844, SGI170845, SGI170845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37795[] = {SGI170846, SGI0, SGI170847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170848, SGI170849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37796[] = {SGI170850, SGI0, SGI170851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170852, SGI170853, SGI170853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37797[] = {SGI170854, SGI0, SGI170855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170856, SGI170857, SGI170857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37798[] = {SGI170858, SGI0, SGI170859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170860, SGI170861, SGI170861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37799[] = {SGI170862, SGI0, SGI170863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170866, SGI170867, SGI170867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37800[] = {SGI170864, SGI0, SGI170865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170868, SGI170869, SGI170869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37801[] = {SGI170870, SGI0, SGI170871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170872, SGI170873, SGI170873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37802[] = {SGI170874, SGI170875, SGI170876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170877, SGI0, SGI0, SGI0, SGI170878, SGI170878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37803[] = {SGI170879, SGI0, SGI170880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37804[] = {SGI170881, SGI0, SGI170882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170883, SGI170884, SGI170884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37805[] = {SGI170885, SGI170886, SGI170887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170888, SGI0, SGI0, SGI0, SGI170889, SGI170889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37806[] = {SGI170890, SGI0, SGI170891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170892, SGI170893, SGI170893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37807[] = {SGI170894, SGI0, SGI170895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170896, SGI170897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37808[] = {SGI170898, SGI0, SGI170899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170900, SGI170901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37809[] = {SGI170902, SGI0, SGI170903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170904, SGI170905, SGI170905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37810[] = {SGI170906, SGI0, SGI170907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170908, SGI170909, SGI170909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37811[] = {SGI170910, SGI0, SGI170911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170912, SGI170913, SGI170913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37812[] = {SGI170914, SGI0, SGI170915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170916, SGI170917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37813[] = {SGI170918, SGI0, SGI170919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170920, SGI170921, SGI170921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37814[] = {SGI170922, SGI0, SGI170923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170924, SGI170925, SGI170925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37815[] = {SGI170926, SGI0, SGI170927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170928, SGI170929, SGI170929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37816[] = {SGI170930, SGI170931, SGI170932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170933, SGI0, SGI0, SGI0, SGI170934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37817[] = {SGI170935, SGI0, SGI170936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37818[] = {SGI170937, SGI170938, SGI170939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170940, SGI0, SGI0, SGI0, SGI170941, SGI170941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37819[] = {SGI170942, SGI170943, SGI170944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170945, SGI0, SGI0, SGI0, SGI170946, SGI170946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37820[] = {SGI170947, SGI170948, SGI170949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170950, SGI0, SGI0, SGI0, SGI170951, SGI170951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37821[] = {SGI170952, SGI0, SGI170953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37822[] = {SGI170954, SGI0, SGI170955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170956, SGI170957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37823[] = {SGI170958, SGI0, SGI170959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170960, SGI170961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37824[] = {SGI170962, SGI0, SGI170963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170964, SGI170965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37825[] = {SGI170966, SGI0, SGI170967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170968, SGI170969, SGI170969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37826[] = {SGI170970, SGI170971, SGI170972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170973, SGI0, SGI0, SGI0, SGI170974, SGI170974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37827[] = {SGI170976, SGI170977, SGI170978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170979, SGI0, SGI0, SGI0, SGI170980, SGI170980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37828[] = {SGI170982, SGI170983, SGI170984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170985, SGI0, SGI0, SGI0, SGI170986, SGI170986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37829[] = {SGI170988, SGI170989, SGI170990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170991, SGI0, SGI0, SGI0, SGI170992, SGI170992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37830[] = {SGI170994, SGI170995, SGI170996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170997, SGI0, SGI0, SGI0, SGI170998, SGI170998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI170999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37831[] = {SGI171000, SGI171001, SGI171002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171003, SGI0, SGI0, SGI0, SGI171004, SGI171004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37832[] = {SGI171006, SGI171007, SGI171008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171009, SGI0, SGI0, SGI0, SGI171010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37833[] = {SGI171012, SGI171013, SGI171014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171015, SGI0, SGI0, SGI0, SGI171016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37834[] = {SGI171017, SGI0, SGI171018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37835[] = {SGI171019, SGI0, SGI171020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171021, SGI171022, SGI171022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37836[] = {SGI171023, SGI0, SGI171024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171025, SGI171026, SGI171026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37837[] = {SGI171027, SGI0, SGI171028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171029, SGI171030, SGI171030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37838[] = {SGI171031, SGI171032, SGI171033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171034, SGI0, SGI0, SGI0, SGI171035, SGI171035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37839[] = {SGI171037, SGI171038, SGI171039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171040, SGI0, SGI0, SGI0, SGI171041, SGI171041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37840[] = {SGI171043, SGI0, SGI171044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171045, SGI171046, SGI171046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37841[] = {SGI171047, SGI0, SGI171048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171049, SGI171050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37842[] = {SGI171051, SGI0, SGI171052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171053, SGI171054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37843[] = {SGI171055, SGI0, SGI171056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171057, SGI171058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37844[] = {SGI171059, SGI0, SGI171060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171061, SGI171062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37845[] = {SGI171063, SGI0, SGI171064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171065, SGI171066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37846[] = {SGI171067, SGI171068, SGI171069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171070, SGI0, SGI0, SGI0, SGI171071, SGI171071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37847[] = {SGI171073, SGI171074, SGI171075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171076, SGI0, SGI0, SGI0, SGI171077, SGI171077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37848[] = {SGI171079, SGI171080, SGI171081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171082, SGI0, SGI0, SGI0, SGI171083, SGI171083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37849[] = {SGI171085, SGI0, SGI171086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171087, SGI171088, SGI171088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37850[] = {SGI171089, SGI0, SGI171090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171091, SGI171092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37851[] = {SGI171093, SGI0, SGI171094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171095, SGI171096, SGI171096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37852[] = {SGI171097, SGI0, SGI171098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171099, SGI171100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37853[] = {SGI171101, SGI0, SGI171102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171103, SGI171104, SGI171104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37854[] = {SGI171105, SGI0, SGI171106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171107, SGI171108, SGI171108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37855[] = {SGI171109, SGI0, SGI171110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171111, SGI171112, SGI171112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37856[] = {SGI171113, SGI0, SGI171114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171115, SGI171116, SGI171116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171117, SGI0, SGI0, NULL};
const Segment * const SG37857[] = {SGI171118, SGI0, SGI171119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171120, SGI171121, SGI171121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171122, SGI0, SGI0, NULL};
const Segment * const SG37858[] = {SGI171123, SGI0, SGI171124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171125, SGI171127, SGI171127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37859[] = {SGI171126, SGI0, SGI171128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171129, SGI171130, SGI171130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171131, SGI0, SGI0, NULL};
const Segment * const SG37860[] = {SGI171132, SGI0, SGI171133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171134, SGI171135, SGI171135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37861[] = {SGI171136, SGI0, SGI171137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171138, SGI171139, SGI171139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171140, SGI0, SGI0, NULL};
const Segment * const SG37862[] = {SGI171141, SGI0, SGI171142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171143, SGI171144, SGI171144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37863[] = {SGI171145, SGI0, SGI171146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171147, SGI171148, SGI171148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171149, SGI0, SGI0, NULL};
const Segment * const SG37864[] = {SGI171150, SGI0, SGI171151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171152, SGI171153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37865[] = {SGI171154, SGI171155, SGI171156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171157, SGI0, SGI0, SGI0, SGI171158, SGI171158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37866[] = {SGI171159, SGI171160, SGI171161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171162, SGI0, SGI0, SGI0, SGI171163, SGI171163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37867[] = {SGI171164, SGI171165, SGI171166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171167, SGI0, SGI0, SGI0, SGI171168, SGI171168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37868[] = {SGI171169, SGI171170, SGI171171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171172, SGI0, SGI0, SGI0, SGI171173, SGI171173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37869[] = {SGI171174, SGI0, SGI171175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37870[] = {SGI171176, SGI171177, SGI171178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171179, SGI0, SGI0, SGI0, SGI171180, SGI171180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37871[] = {SGI171181, SGI0, SGI171182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171183, SGI171185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37872[] = {SGI171184, SGI171186, SGI171187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171188, SGI0, SGI0, SGI0, SGI171189, SGI171189, SGI0, SGI171190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37873[] = {SGI171191, SGI0, SGI171192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171193, SGI171194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37874[] = {SGI171195, SGI171196, SGI171197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171198, SGI0, SGI0, SGI0, SGI171199, SGI171199, SGI0, SGI171200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37875[] = {SGI171201, SGI171202, SGI171203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171204, SGI0, SGI0, SGI0, SGI171205, SGI171205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37876[] = {SGI171206, SGI171207, SGI171208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171209, SGI0, SGI0, SGI0, SGI171210, SGI171210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37877[] = {SGI171211, SGI171212, SGI171213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171214, SGI0, SGI0, SGI0, SGI171215, SGI171215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37878[] = {SGI171216, SGI0, SGI171217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37879[] = {SGI171218, SGI171219, SGI171220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171221, SGI0, SGI0, SGI0, SGI171222, SGI171222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37880[] = {SGI171223, SGI171224, SGI171225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171226, SGI0, SGI0, SGI0, SGI171227, SGI171227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37881[] = {SGI171228, SGI171229, SGI171230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171231, SGI0, SGI0, SGI0, SGI171232, SGI171232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37882[] = {SGI171233, SGI171234, SGI171235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171236, SGI0, SGI0, SGI0, SGI171237, SGI171237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37883[] = {SGI171238, SGI171239, SGI171240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171243, SGI0, SGI0, SGI0, SGI171245, SGI171245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37884[] = {SGI171241, SGI171242, SGI171244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171246, SGI0, SGI0, SGI0, SGI171247, SGI171247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37885[] = {SGI171248, SGI171249, SGI171250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171251, SGI0, SGI0, SGI0, SGI171254, SGI171254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37886[] = {SGI171252, SGI171253, SGI171255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171256, SGI0, SGI0, SGI0, SGI171257, SGI171257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37887[] = {SGI171258, SGI0, SGI171259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37888[] = {SGI171260, SGI0, SGI171261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37889[] = {SGI171262, SGI171263, SGI171264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171265, SGI0, SGI0, SGI0, SGI171266, SGI171266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37890[] = {SGI171267, SGI171268, SGI171269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171270, SGI0, SGI0, SGI0, SGI171271, SGI171271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37891[] = {SGI171272, SGI171273, SGI171274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171275, SGI0, SGI0, SGI0, SGI171276, SGI171276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37892[] = {SGI171277, SGI171278, SGI171279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171280, SGI0, SGI0, SGI0, SGI171281, SGI171281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37893[] = {SGI171282, SGI171283, SGI171284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171285, SGI0, SGI0, SGI0, SGI171286, SGI171286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37894[] = {SGI171287, SGI0, SGI171288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37895[] = {SGI171289, SGI171290, SGI171291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171292, SGI0, SGI0, SGI0, SGI171293, SGI171293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37896[] = {SGI171294, SGI171295, SGI171296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171297, SGI171297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37897[] = {SGI171301, SGI0, SGI171302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171303, SGI171304, SGI171304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37898[] = {SGI171305, SGI0, SGI171306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171307, SGI171308, SGI171308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37899[] = {SGI171309, SGI0, SGI171310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171311, SGI171312, SGI171312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37900[] = {SGI171313, SGI0, SGI171314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171315, SGI171316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37901[] = {SGI171317, SGI0, SGI171318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171319, SGI171320, SGI171320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37902[] = {SGI171323, SGI0, SGI171325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171327, SGI171329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37903[] = {SGI171321, SGI171322, SGI171324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171326, SGI0, SGI0, SGI171328, SGI171330, SGI171330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37904[] = {SGI171331, SGI171332, SGI171333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171334, SGI0, SGI0, SGI171335, SGI171337, SGI171337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37905[] = {SGI171336, SGI0, SGI171338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171339, SGI171340, SGI171340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37906[] = {SGI171341, SGI171342, SGI171343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171344, SGI0, SGI0, SGI171345, SGI171347, SGI171347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37907[] = {SGI171346, SGI0, SGI171348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171349, SGI171350, SGI171350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37908[] = {SGI171351, SGI171352, SGI171353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171354, SGI0, SGI0, SGI171356, SGI171357, SGI171357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37909[] = {SGI171355, SGI0, SGI171358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171359, SGI171360, SGI171360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37910[] = {SGI0, SGI171361, SGI171362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171364, SGI0, SGI0, SGI0, SGI171366, SGI171366, SGI0, SGI0, NULL};
const Segment * const SG37911[] = {SGI171363, SGI0, SGI171365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171367, SGI171368, SGI171368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37912[] = {SGI171369, SGI0, SGI171370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171373, SGI171374, SGI171374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37913[] = {SGI171371, SGI0, SGI171372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171375, SGI171376, SGI171376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37914[] = {SGI171298, SGI171299, SGI171300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171377, SGI0, SGI0, SGI0, SGI171378, SGI171378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37915[] = {SGI171379, SGI0, SGI171381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171383, SGI171385, SGI171385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37916[] = {SGI171380, SGI0, SGI171382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171384, SGI171386, SGI171386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37917[] = {SGI171387, SGI171388, SGI171389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171390, SGI0, SGI0, SGI0, SGI171391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37918[] = {SGI171392, SGI0, SGI171393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171394, SGI171395, SGI171395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37919[] = {SGI171396, SGI0, SGI171397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171398, SGI171399, SGI171399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37920[] = {SGI171400, SGI0, SGI171401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171402, SGI171403, SGI171403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37921[] = {SGI171404, SGI0, SGI171405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171406, SGI171407, SGI171407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37922[] = {SGI171408, SGI171409, SGI171410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171411, SGI0, SGI0, SGI0, SGI171412, SGI171412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37923[] = {SGI171413, SGI171414, SGI171415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171416, SGI171416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37924[] = {SGI171417, SGI0, SGI171418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37925[] = {SGI171419, SGI0, SGI171420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171424, SGI171425, SGI171425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37926[] = {SGI171421, SGI171422, SGI171423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171426, SGI0, SGI0, SGI0, SGI171427, SGI171427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37927[] = {SGI171428, SGI171429, SGI171430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171431, SGI0, SGI0, SGI0, SGI171432, SGI171432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37928[] = {SGI171434, SGI171435, SGI171436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171437, SGI0, SGI0, SGI0, SGI171438, SGI171438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37929[] = {SGI171440, SGI171441, SGI171442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171443, SGI0, SGI0, SGI0, SGI171444, SGI171444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37930[] = {SGI171446, SGI171447, SGI171448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171449, SGI0, SGI0, SGI0, SGI171450, SGI171450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37931[] = {SGI171451, SGI0, SGI171452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37932[] = {SGI171453, SGI0, SGI171454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171455, SGI171456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37933[] = {SGI171457, SGI0, SGI171458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171459, SGI171460, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37934[] = {SGI171461, SGI0, SGI171462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171463, SGI171464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37935[] = {SGI171465, SGI0, SGI171466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171467, SGI171468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37936[] = {SGI171469, SGI171470, SGI171471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171473, SGI0, SGI0, SGI0, SGI171475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37937[] = {SGI171472, SGI0, SGI171474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171476, SGI171477, SGI171477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37938[] = {SGI171478, SGI0, SGI171479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37939[] = {SGI171480, SGI0, SGI171481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171482, SGI171483, SGI171483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37940[] = {SGI171484, SGI0, SGI171485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171488, SGI171489, SGI171489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37941[] = {SGI171486, SGI0, SGI171487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171490, SGI171491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37942[] = {SGI171492, SGI0, SGI171493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171494, SGI171495, SGI171495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37943[] = {SGI171496, SGI0, SGI171497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171500, SGI171501, SGI171501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37944[] = {SGI171498, SGI0, SGI171499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171502, SGI171503, SGI171503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37945[] = {SGI171504, SGI0, SGI171505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171508, SGI171509, SGI171509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37946[] = {SGI171506, SGI0, SGI171507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171510, SGI171511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37947[] = {SGI171512, SGI171513, SGI171514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171515, SGI0, SGI0, SGI0, SGI171516, SGI171516, SGI0, SGI171517, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37948[] = {SGI171518, SGI171519, SGI171520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171521, SGI0, SGI0, SGI0, SGI171522, SGI171522, SGI0, SGI171523, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37949[] = {SGI171524, SGI0, SGI171525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171526, SGI171527, SGI171527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37950[] = {SGI171528, SGI0, SGI171529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171530, SGI171531, SGI171531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37951[] = {SGI171532, SGI0, SGI171533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171534, SGI171535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37952[] = {SGI171536, SGI0, SGI171537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171538, SGI171540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37953[] = {SGI171539, SGI0, SGI171541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171542, SGI171543, SGI171543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37954[] = {SGI171544, SGI0, SGI171545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171546, SGI171548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37955[] = {SGI171547, SGI171549, SGI171550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171551, SGI171551, SGI0, SGI0, SGI171552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37956[] = {SGI171553, SGI0, SGI171554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171555, SGI171556, SGI171556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37957[] = {SGI171557, SGI0, SGI171558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171559, SGI171560, SGI171560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37958[] = {SGI171561, SGI0, SGI171562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171563, SGI171564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37959[] = {SGI171565, SGI0, SGI171566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171567, SGI171568, SGI171568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37960[] = {SGI171569, SGI0, SGI171570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171571, SGI171572, SGI171572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37961[] = {SGI171573, SGI171574, SGI171575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171576, SGI0, SGI0, SGI0, SGI171577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37962[] = {SGI171578, SGI0, SGI171579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37963[] = {SGI171580, SGI0, SGI171581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171582, SGI171583, SGI171583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37964[] = {SGI171584, SGI171585, SGI171586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171587, SGI0, SGI0, SGI0, SGI171588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37965[] = {SGI171589, SGI0, SGI171591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37966[] = {SGI171590, SGI0, SGI171592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171593, SGI171594, SGI171594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37967[] = {SGI171595, SGI0, SGI171596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171597, SGI171598, SGI171598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37968[] = {SGI171599, SGI171600, SGI171601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171602, SGI0, SGI0, SGI0, SGI171603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37969[] = {SGI171604, SGI0, SGI171605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171606, SGI171607, SGI171607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37970[] = {SGI171608, SGI0, SGI171609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171610, SGI171611, SGI171611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37971[] = {SGI171612, SGI0, SGI171613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171614, SGI171615, SGI171615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37972[] = {SGI171616, SGI0, SGI171617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171618, SGI171619, SGI171619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37973[] = {SGI171620, SGI171621, SGI171622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171623, SGI171623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37974[] = {SGI171624, SGI171625, SGI171626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171627, SGI0, SGI0, SGI0, SGI171628, SGI171628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37975[] = {SGI171629, SGI0, SGI171630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37976[] = {SGI171631, SGI171632, SGI171633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171634, SGI0, SGI0, SGI0, SGI171635, SGI171635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37977[] = {SGI0, SGI0, SGI171636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171637, SGI0, SGI0, SGI0, SGI171638, SGI171638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37978[] = {SGI171639, SGI171640, SGI171641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171642, SGI0, SGI0, SGI0, SGI171643, SGI171643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37979[] = {SGI171644, SGI0, SGI171645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171646, SGI171647, SGI171647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37980[] = {SGI171648, SGI171649, SGI171650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171651, SGI0, SGI0, SGI0, SGI171652, SGI171652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37981[] = {SGI171653, SGI171654, SGI171655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171656, SGI0, SGI0, SGI0, SGI171657, SGI171657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37982[] = {SGI171658, SGI0, SGI171659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171660, SGI171661, SGI171661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37983[] = {SGI171662, SGI0, SGI171663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171664, SGI171665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37984[] = {SGI171666, SGI171667, SGI171668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171669, SGI0, SGI0, SGI0, SGI171670, SGI171670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37985[] = {SGI171671, SGI171672, SGI171673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171674, SGI0, SGI0, SGI0, SGI171675, SGI171675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37986[] = {SGI171676, SGI0, SGI171677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37987[] = {SGI171678, SGI0, SGI171679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171680, SGI171681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37988[] = {SGI171682, SGI171683, SGI171684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171685, SGI0, SGI0, SGI0, SGI171686, SGI171686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37989[] = {SGI171687, SGI0, SGI171688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37990[] = {SGI171689, SGI171690, SGI171691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171692, SGI0, SGI0, SGI0, SGI171693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37991[] = {SGI171694, SGI0, SGI171695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171696, SGI171697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37992[] = {SGI171698, SGI0, SGI171699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171700, SGI171701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37993[] = {SGI171702, SGI0, SGI171703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171704, SGI171705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37994[] = {SGI171706, SGI0, SGI171707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171710, SGI171712, SGI171712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37995[] = {SGI171708, SGI171709, SGI171711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171713, SGI0, SGI0, SGI0, SGI171714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37996[] = {SGI171715, SGI0, SGI171716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171717, SGI171718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37997[] = {SGI171719, SGI0, SGI171720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171721, SGI171722, SGI171722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37998[] = {SGI171723, SGI171724, SGI171725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171726, SGI0, SGI0, SGI0, SGI171727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG37999[] = {SGI171728, SGI0, SGI171729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171730, SGI171731, SGI171731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38000[] = {SGI171732, SGI171733, SGI171734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171735, SGI0, SGI0, SGI0, SGI171736, SGI171736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38001[] = {SGI171737, SGI0, SGI171738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38002[] = {SGI171739, SGI171740, SGI171741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171742, SGI0, SGI0, SGI0, SGI171743, SGI171743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38003[] = {SGI171744, SGI171745, SGI171746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171747, SGI0, SGI0, SGI0, SGI171748, SGI171748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38004[] = {SGI171749, SGI0, SGI171750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171751, SGI171752, SGI171752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38005[] = {SGI171753, SGI171754, SGI171755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171756, SGI171756, SGI0, SGI0, SGI171757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38006[] = {SGI171758, SGI0, SGI171759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171760, SGI171761, SGI171761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38007[] = {SGI171762, SGI0, SGI171763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171764, SGI171765, SGI171765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38008[] = {SGI171766, SGI0, SGI171767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171768, SGI171769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38009[] = {SGI171770, SGI171771, SGI171772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171773, SGI171773, SGI0, SGI0, SGI0, SGI171774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38010[] = {SGI171775, SGI171776, SGI171777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171778, SGI171778, SGI0, SGI0, SGI0, SGI171779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38011[] = {SGI171780, SGI171781, SGI171782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171783, SGI171783, SGI0, SGI0, SGI0, SGI171784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38012[] = {SGI171785, SGI171786, SGI171787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171788, SGI171788, SGI0, SGI0, SGI0, SGI171789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38013[] = {SGI171790, SGI171791, SGI171792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171793, SGI171793, SGI0, SGI0, SGI0, SGI171794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38014[] = {SGI171795, SGI171796, SGI171797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171798, SGI171798, SGI0, SGI0, SGI0, SGI171799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38015[] = {SGI171800, SGI171801, SGI171802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171803, SGI0, SGI0, SGI0, SGI171804, SGI171804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38016[] = {SGI171805, SGI171806, SGI171807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171808, SGI0, SGI0, SGI0, SGI171809, SGI171809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38017[] = {SGI171810, SGI0, SGI171811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38018[] = {SGI171812, SGI171813, SGI171814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171815, SGI0, SGI0, SGI0, SGI171816, SGI171816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38019[] = {SGI171817, SGI171818, SGI171819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171820, SGI0, SGI0, SGI0, SGI171821, SGI171821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38020[] = {SGI171822, SGI0, SGI171823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171824, SGI171825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38021[] = {SGI171826, SGI0, SGI171827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171828, SGI171829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38022[] = {SGI171830, SGI0, SGI171831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171832, SGI171833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38023[] = {SGI171834, SGI0, SGI171835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171836, SGI171837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38024[] = {SGI171838, SGI0, SGI171839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171840, SGI171841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38025[] = {SGI171842, SGI0, SGI171843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171844, SGI171845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38026[] = {SGI171846, SGI0, SGI171847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171848, SGI171849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38027[] = {SGI171850, SGI171851, SGI171852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171853, SGI0, SGI0, SGI0, SGI171854, SGI171854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38028[] = {SGI171855, SGI0, SGI171856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171857, SGI171858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38029[] = {SGI171859, SGI0, SGI171860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171861, SGI171862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38030[] = {SGI171863, SGI0, SGI171864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171865, SGI171866, SGI171866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38031[] = {SGI171867, SGI0, SGI171868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171869, SGI171870, SGI171870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38032[] = {SGI171871, SGI0, SGI171872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171873, SGI171874, SGI171874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38033[] = {SGI171875, SGI0, SGI171876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171877, SGI171878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38034[] = {SGI171879, SGI0, SGI171880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171881, SGI171882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38035[] = {SGI171883, SGI171884, SGI171885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171886, SGI0, SGI0, SGI0, SGI171887, SGI171887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38036[] = {SGI171888, SGI171889, SGI171890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171892, SGI0, SGI0, SGI0, SGI171894, SGI171894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38037[] = {SGI171891, SGI0, SGI171893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171895, SGI171896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38038[] = {SGI171897, SGI171898, SGI171899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171900, SGI0, SGI0, SGI0, SGI171901, SGI171901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38039[] = {SGI171902, SGI0, SGI171903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171904, SGI171905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38040[] = {SGI171906, SGI171907, SGI171908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171909, SGI0, SGI0, SGI0, SGI171910, SGI171910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38041[] = {SGI171911, SGI0, SGI171912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171913, SGI171914, SGI171914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171915, SGI0, SGI0, NULL};
const Segment * const SG38042[] = {SGI171916, SGI0, SGI171917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171918, SGI171919, SGI171919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38043[] = {SGI171920, SGI171921, SGI171922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171923, SGI0, SGI0, SGI0, SGI171924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38044[] = {SGI171925, SGI171926, SGI171927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171928, SGI0, SGI0, SGI0, SGI171929, SGI171929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38045[] = {SGI171932, SGI0, SGI171934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38046[] = {SGI171930, SGI171931, SGI171933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171935, SGI0, SGI0, SGI0, SGI171936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38047[] = {SGI171938, SGI0, SGI171940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38048[] = {SGI171937, SGI171939, SGI171941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171942, SGI0, SGI0, SGI0, SGI171943, SGI171943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38049[] = {SGI171944, SGI171945, SGI171946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171947, SGI0, SGI0, SGI0, SGI171948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38050[] = {SGI171949, SGI171950, SGI171951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171952, SGI0, SGI0, SGI0, SGI171953, SGI171953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38051[] = {SGI171954, SGI171955, SGI171956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171957, SGI0, SGI0, SGI0, SGI171958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38052[] = {SGI171959, SGI171960, SGI171961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171962, SGI0, SGI0, SGI0, SGI171963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38053[] = {SGI171964, SGI0, SGI171965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38054[] = {SGI171966, SGI0, SGI171967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171968, SGI171969, SGI171969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171970, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38055[] = {SGI171971, SGI0, SGI171972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171973, SGI171974, SGI171974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171975, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38056[] = {SGI171976, SGI171977, SGI171978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171979, SGI0, SGI0, SGI0, SGI171980, SGI171980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38057[] = {SGI171981, SGI171982, SGI171983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171984, SGI0, SGI0, SGI0, SGI171985, SGI171985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38058[] = {SGI171986, SGI171987, SGI171988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171989, SGI0, SGI0, SGI0, SGI171990, SGI171990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38059[] = {SGI171991, SGI171992, SGI171993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171994, SGI0, SGI0, SGI0, SGI171995, SGI171995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38060[] = {SGI171996, SGI171997, SGI171998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI171999, SGI0, SGI0, SGI0, SGI172000, SGI172000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38061[] = {SGI172001, SGI172002, SGI172003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172004, SGI0, SGI0, SGI0, SGI172005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38062[] = {SGI172006, SGI172007, SGI172008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172009, SGI0, SGI0, SGI0, SGI172010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38063[] = {SGI172011, SGI0, SGI172012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38064[] = {SGI172013, SGI172014, SGI172015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172016, SGI0, SGI0, SGI0, SGI172017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38065[] = {SGI172018, SGI0, SGI172019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38066[] = {SGI172020, SGI0, SGI172021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172022, SGI172023, SGI172023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38067[] = {SGI172024, SGI0, SGI172025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172026, SGI172027, SGI172027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38068[] = {SGI172028, SGI0, SGI172029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172030, SGI172031, SGI172031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38069[] = {SGI172032, SGI0, SGI172033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172034, SGI172035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38070[] = {SGI172036, SGI0, SGI172037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172038, SGI172039, SGI172039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38071[] = {SGI172040, SGI0, SGI172041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172042, SGI172043, SGI172043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38072[] = {SGI172044, SGI0, SGI172045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172048, SGI172049, SGI172049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38073[] = {SGI172046, SGI0, SGI172047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172050, SGI172051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38074[] = {SGI172052, SGI0, SGI172053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172054, SGI172055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38075[] = {SGI172056, SGI0, SGI172057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172058, SGI172059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38076[] = {SGI172060, SGI0, SGI172061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172062, SGI0, SGI0, SGI0, SGI172063, SGI172063, SGI0, SGI0, SGI0, SGI172064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38077[] = {SGI172065, SGI0, SGI172066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172067, SGI0, SGI0, SGI0, SGI172068, SGI172068, SGI0, SGI0, SGI0, SGI172069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38078[] = {SGI172070, SGI172071, SGI172072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172073, SGI0, SGI0, SGI0, SGI172074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38079[] = {SGI172075, SGI0, SGI172076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38080[] = {SGI172077, SGI0, SGI172078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172079, SGI172080, SGI172080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38081[] = {SGI172081, SGI172082, SGI172083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172084, SGI0, SGI0, SGI0, SGI172085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38082[] = {SGI172086, SGI172087, SGI172088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172089, SGI0, SGI0, SGI0, SGI172090, SGI172090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38083[] = {SGI172091, SGI0, SGI172092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172093, SGI172094, SGI172094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38084[] = {SGI172095, SGI172096, SGI172097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172098, SGI0, SGI0, SGI0, SGI172099, SGI172099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38085[] = {SGI172100, SGI0, SGI172101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172102, SGI172103, SGI172103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38086[] = {SGI172104, SGI172105, SGI172106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172107, SGI0, SGI0, SGI0, SGI172109, SGI172109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38087[] = {SGI172108, SGI0, SGI172110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172111, SGI172112, SGI172112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38088[] = {SGI172113, SGI172114, SGI172115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172117, SGI0, SGI0, SGI0, SGI172119, SGI172119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38089[] = {SGI172116, SGI0, SGI172118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172120, SGI172121, SGI172121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38090[] = {SGI172122, SGI0, SGI172123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38091[] = {SGI172124, SGI0, SGI172125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172126, SGI172127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38092[] = {SGI172128, SGI172129, SGI172130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172131, SGI0, SGI0, SGI0, SGI172132, SGI172132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38093[] = {SGI172133, SGI172134, SGI172135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172136, SGI0, SGI0, SGI0, SGI172137, SGI172137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38094[] = {SGI172138, SGI0, SGI172139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38095[] = {SGI172140, SGI0, SGI172141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38096[] = {SGI172142, SGI172143, SGI172144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172145, SGI0, SGI0, SGI0, SGI172146, SGI172146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38097[] = {SGI172147, SGI172148, SGI172149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172150, SGI0, SGI0, SGI0, SGI172151, SGI172151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38098[] = {SGI172152, SGI172153, SGI172154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172155, SGI0, SGI0, SGI0, SGI172156, SGI172156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38099[] = {SGI172157, SGI172158, SGI172159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172160, SGI0, SGI0, SGI0, SGI172161, SGI172161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38100[] = {SGI172162, SGI172163, SGI172164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172165, SGI0, SGI0, SGI0, SGI172166, SGI172166, SGI0, SGI172167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38101[] = {SGI172168, SGI172169, SGI172170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172171, SGI0, SGI0, SGI0, SGI172172, SGI172172, SGI0, SGI172173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38102[] = {SGI172174, SGI172175, SGI172176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172177, SGI0, SGI0, SGI0, SGI172178, SGI172178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38103[] = {SGI172179, SGI172180, SGI172181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172182, SGI0, SGI0, SGI0, SGI172183, SGI172183, SGI0, SGI172184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38104[] = {SGI172185, SGI0, SGI172186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172187, SGI172188, SGI172188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38105[] = {SGI172189, SGI0, SGI172190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172191, SGI172192, SGI172192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38106[] = {SGI172193, SGI0, SGI172194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172195, SGI172196, SGI172196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38107[] = {SGI172197, SGI0, SGI172198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172199, SGI172200, SGI172200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38108[] = {SGI172201, SGI0, SGI172202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172203, SGI172204, SGI172204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38109[] = {SGI172205, SGI0, SGI172206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172207, SGI172208, SGI172208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38110[] = {SGI172209, SGI0, SGI172210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172211, SGI172212, SGI172212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38111[] = {SGI172213, SGI0, SGI172214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172215, SGI172216, SGI172216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38112[] = {SGI172217, SGI172218, SGI172219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172220, SGI0, SGI0, SGI0, SGI172221, SGI172221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38113[] = {SGI172222, SGI172223, SGI172224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172225, SGI0, SGI0, SGI0, SGI172226, SGI172226, SGI0, SGI172227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38114[] = {SGI172228, SGI0, SGI172229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38115[] = {SGI172230, SGI172231, SGI172232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172235, SGI0, SGI0, SGI0, SGI172237, SGI172237, SGI0, SGI172238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38116[] = {SGI172233, SGI172234, SGI172236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172239, SGI0, SGI0, SGI0, SGI172240, SGI172240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38117[] = {SGI172241, SGI172242, SGI172243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172244, SGI0, SGI0, SGI0, SGI172246, SGI172246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38118[] = {SGI172245, SGI172247, SGI172248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172249, SGI0, SGI0, SGI0, SGI172250, SGI172250, SGI0, SGI172251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38119[] = {SGI172252, SGI0, SGI172253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172254, SGI0, SGI0, SGI0, SGI172255, SGI172255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38120[] = {SGI172256, SGI0, SGI172257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38121[] = {SGI172258, SGI0, SGI172259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172260, SGI172261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38122[] = {SGI172262, SGI172263, SGI172264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172265, SGI0, SGI0, SGI0, SGI172268, SGI172268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38123[] = {SGI172266, SGI0, SGI172267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172269, SGI172270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38124[] = {SGI172271, SGI172272, SGI172273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172274, SGI0, SGI0, SGI0, SGI172275, SGI172275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38125[] = {SGI172276, SGI172277, SGI172278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172279, SGI0, SGI0, SGI0, SGI172280, SGI0, SGI172281, SGI0, NULL};
const Segment * const SG38126[] = {SGI172282, SGI172283, SGI172284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172285, SGI0, SGI0, SGI0, SGI172286, SGI172286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38127[] = {SGI172287, SGI172288, SGI172289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172290, SGI0, SGI0, SGI0, SGI172291, SGI172291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38128[] = {SGI172292, SGI172293, SGI172294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172295, SGI0, SGI0, SGI0, SGI172296, SGI172296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38129[] = {SGI172297, SGI0, SGI172298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38130[] = {SGI172299, SGI0, SGI172300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172301, SGI172302, SGI172302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38131[] = {SGI172303, SGI0, SGI172304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172305, SGI172306, SGI172306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38132[] = {SGI172307, SGI0, SGI172308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172309, SGI172310, SGI172310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38133[] = {SGI172311, SGI0, SGI172312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172313, SGI172314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38134[] = {SGI172315, SGI0, SGI172316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172317, SGI172318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38135[] = {SGI172319, SGI0, SGI172320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172321, SGI172322, SGI172322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38136[] = {SGI172323, SGI0, SGI172324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172325, SGI172326, SGI172326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38137[] = {SGI172327, SGI0, SGI172328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172329, SGI172330, SGI172330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38138[] = {SGI172331, SGI172332, SGI172333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172334, SGI0, SGI0, SGI0, SGI172335, SGI172335, SGI0, SGI0, SGI0, SGI172336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38139[] = {SGI172337, SGI172338, SGI172339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172340, SGI0, SGI0, SGI0, SGI172341, SGI172341, SGI0, SGI0, SGI0, SGI172342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38140[] = {SGI172343, SGI172344, SGI172345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172346, SGI0, SGI0, SGI0, SGI172347, SGI172347, SGI0, SGI0, SGI0, SGI172348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38141[] = {SGI172349, SGI172350, SGI172351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172352, SGI0, SGI0, SGI0, SGI172353, SGI172353, SGI0, SGI0, SGI0, SGI172354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38142[] = {SGI172355, SGI172356, SGI172357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172358, SGI0, SGI0, SGI0, SGI172359, SGI172359, SGI0, SGI0, SGI0, SGI172360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38143[] = {SGI172361, SGI0, SGI172362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172363, SGI172364, SGI172364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38144[] = {SGI172365, SGI172366, SGI172367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172368, SGI0, SGI0, SGI0, SGI172369, SGI172369, SGI0, SGI172370, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38145[] = {SGI172371, SGI172372, SGI172373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172374, SGI0, SGI0, SGI0, SGI172375, SGI172375, SGI0, SGI172376, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38146[] = {SGI172377, SGI172378, SGI172379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172380, SGI0, SGI0, SGI0, SGI172381, SGI172381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38147[] = {SGI172382, SGI172383, SGI172384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172385, SGI0, SGI0, SGI0, SGI172386, SGI172386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38148[] = {SGI172387, SGI172388, SGI172389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172390, SGI0, SGI0, SGI0, SGI172391, SGI172391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38149[] = {SGI172392, SGI0, SGI172393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172394, SGI172395, SGI172395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38150[] = {SGI172396, SGI0, SGI172397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172398, SGI172399, SGI172399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38151[] = {SGI172400, SGI172401, SGI172403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172405, SGI0, SGI0, SGI0, SGI172406, SGI172406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38152[] = {SGI172402, SGI0, SGI172404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172407, SGI172408, SGI172408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38153[] = {SGI172409, SGI172410, SGI172411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172412, SGI0, SGI0, SGI0, SGI172413, SGI172413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38154[] = {SGI172414, SGI172415, SGI172416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172420, SGI0, SGI0, SGI0, SGI172421, SGI172421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38155[] = {SGI172417, SGI172418, SGI172419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172422, SGI0, SGI0, SGI0, SGI172423, SGI172423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38156[] = {SGI172426, SGI172428, SGI172429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172431, SGI0, SGI0, SGI0, SGI172433, SGI172433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38157[] = {SGI172424, SGI172425, SGI172427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172430, SGI0, SGI0, SGI0, SGI172432, SGI172432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38158[] = {SGI172434, SGI172435, SGI172436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172437, SGI0, SGI0, SGI0, SGI172439, SGI172439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38159[] = {SGI172438, SGI172440, SGI172441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172442, SGI0, SGI0, SGI0, SGI172443, SGI172443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38160[] = {SGI172444, SGI172445, SGI172446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172447, SGI0, SGI0, SGI0, SGI172448, SGI172448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38161[] = {SGI172449, SGI0, SGI172450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172451, SGI172452, SGI172452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38162[] = {SGI172453, SGI0, SGI172454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172455, SGI172456, SGI172456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38163[] = {SGI172457, SGI0, SGI172458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172459, SGI172460, SGI172460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38164[] = {SGI172461, SGI0, SGI172462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172463, SGI172464, SGI172464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38165[] = {SGI172465, SGI0, SGI172466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172467, SGI172468, SGI172468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38166[] = {SGI172469, SGI0, SGI172470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172471, SGI172472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38167[] = {SGI0, SGI0, SGI172473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172474, SGI0, SGI0, SGI0, SGI172475, SGI172475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38168[] = {SGI172476, SGI172477, SGI172478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172479, SGI0, SGI0, SGI0, SGI172480, SGI172480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38169[] = {SGI172481, SGI0, SGI172482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38170[] = {SGI172483, SGI0, SGI172484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172485, SGI172486, SGI172486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38171[] = {SGI172487, SGI0, SGI172488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172492, SGI172493, SGI172493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38172[] = {SGI172489, SGI172490, SGI172491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172494, SGI0, SGI0, SGI0, SGI172495, SGI172495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38173[] = {SGI172496, SGI0, SGI172498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172500, SGI172501, SGI172501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38174[] = {SGI172497, SGI0, SGI172499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172502, SGI172503, SGI172503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38175[] = {SGI172504, SGI0, SGI172505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172506, SGI172507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38176[] = {SGI172508, SGI0, SGI172509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172510, SGI172511, SGI172511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38177[] = {SGI172512, SGI0, SGI172513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172514, SGI172515, SGI172515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38178[] = {SGI172516, SGI0, SGI172517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172518, SGI172519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38179[] = {SGI172520, SGI0, SGI172521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172522, SGI172523, SGI172523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38180[] = {SGI172524, SGI172525, SGI172526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172527, SGI0, SGI0, SGI0, SGI172528, SGI172528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38181[] = {SGI172529, SGI172530, SGI172531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172532, SGI0, SGI0, SGI0, SGI172533, SGI172533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38182[] = {SGI172534, SGI0, SGI172535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38183[] = {SGI172536, SGI172537, SGI172538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172539, SGI0, SGI0, SGI0, SGI172540, SGI172540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38184[] = {SGI172541, SGI172542, SGI172543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172544, SGI0, SGI0, SGI0, SGI172545, SGI172545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38185[] = {SGI172546, SGI0, SGI172547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172551, SGI172552, SGI172552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38186[] = {SGI172548, SGI172549, SGI172550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172553, SGI0, SGI0, SGI0, SGI172554, SGI172554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38187[] = {SGI172555, SGI0, SGI172556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172557, SGI172558, SGI172558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38188[] = {SGI172559, SGI172560, SGI172561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172562, SGI0, SGI0, SGI0, SGI172563, SGI172563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38189[] = {SGI172564, SGI0, SGI172565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172566, SGI172567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38190[] = {SGI172568, SGI172569, SGI172570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172571, SGI0, SGI0, SGI0, SGI172572, SGI172572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38191[] = {SGI172573, SGI172574, SGI172575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172576, SGI0, SGI0, SGI0, SGI172577, SGI172577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38192[] = {SGI172578, SGI172579, SGI172580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172581, SGI0, SGI0, SGI0, SGI172582, SGI172582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38193[] = {SGI172583, SGI0, SGI172584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172585, SGI172586, SGI172586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38194[] = {SGI172587, SGI0, SGI172588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172589, SGI172590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38195[] = {SGI172591, SGI0, SGI172592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172593, SGI172594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38196[] = {SGI172595, SGI0, SGI172596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172598, SGI172599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38197[] = {SGI172597, SGI0, SGI172600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172601, SGI172602, SGI172602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38198[] = {SGI172603, SGI0, SGI172604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172605, SGI172606, SGI172606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38199[] = {SGI172607, SGI0, SGI172608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172609, SGI172610, SGI172610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38200[] = {SGI172611, SGI0, SGI172612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172613, SGI172614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38201[] = {SGI172615, SGI0, SGI172616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172617, SGI172618, SGI172618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38202[] = {SGI172619, SGI0, SGI172620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172621, SGI172622, SGI172622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38203[] = {SGI172623, SGI172624, SGI172625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172626, SGI0, SGI0, SGI0, SGI172627, SGI172627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38204[] = {SGI172628, SGI0, SGI172629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172630, SGI172631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38205[] = {SGI172632, SGI172633, SGI172634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172635, SGI0, SGI0, SGI0, SGI172636, SGI172636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38206[] = {SGI172637, SGI0, SGI172638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38207[] = {SGI172639, SGI0, SGI172640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172641, SGI172642, SGI172642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38208[] = {SGI172643, SGI0, SGI172644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172645, SGI172646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38209[] = {SGI172647, SGI172648, SGI172649, SGI172650, SGI172651, SGI172651, SGI0, NULL};
const Segment * const SG38210[] = {SGI172652, SGI0, SGI172653, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38211[] = {SGI172654, SGI172655, SGI172656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172657, SGI0, SGI0, SGI0, SGI172658, SGI172658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38212[] = {SGI172659, SGI0, SGI172660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172661, SGI172662, SGI172662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38213[] = {SGI172663, SGI0, SGI172664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172665, SGI172666, SGI172666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38214[] = {SGI172667, SGI0, SGI172668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172669, SGI172670, SGI172670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38215[] = {SGI172671, SGI172672, SGI172673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172674, SGI0, SGI0, SGI0, SGI172675, SGI172675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38216[] = {SGI172676, SGI172677, SGI172678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172679, SGI0, SGI0, SGI0, SGI172680, SGI172680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38217[] = {SGI172681, SGI0, SGI172682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38218[] = {SGI172683, SGI172684, SGI172685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172686, SGI0, SGI0, SGI0, SGI172687, SGI172687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38219[] = {SGI172688, SGI0, SGI172689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172690, SGI172691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38220[] = {SGI172692, SGI0, SGI172694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172696, SGI172697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38221[] = {SGI172693, SGI0, SGI172695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172698, SGI172699, SGI172699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38222[] = {SGI172700, SGI0, SGI172701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172702, SGI172703, SGI172703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38223[] = {SGI172704, SGI0, SGI172705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172706, SGI172707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38224[] = {SGI172708, SGI0, SGI172709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172710, SGI172711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38225[] = {SGI172712, SGI0, SGI172713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172714, SGI172715, SGI172715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38226[] = {SGI172716, SGI0, SGI172717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172718, SGI172719, SGI172719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38227[] = {SGI172720, SGI0, SGI172721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172722, SGI172723, SGI172723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38228[] = {SGI172724, SGI172725, SGI172726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172727, SGI0, SGI0, SGI0, SGI172728, SGI172728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38229[] = {SGI0, SGI0, SGI172729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172730, SGI172730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38230[] = {SGI172731, SGI0, SGI172732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172733, SGI172734, SGI172734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38231[] = {SGI172735, SGI172736, SGI172737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172738, SGI172738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38232[] = {SGI172739, SGI172740, SGI172741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172742, SGI0, SGI0, SGI0, SGI172743, SGI172743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38233[] = {SGI172744, SGI0, SGI172745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38234[] = {SGI172746, SGI0, SGI172747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172750, SGI172751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38235[] = {SGI172748, SGI0, SGI172749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172752, SGI172753, SGI172753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38236[] = {SGI172754, SGI0, SGI172755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172756, SGI172757, SGI172757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38237[] = {SGI172758, SGI172759, SGI172760, SGI0, SGI0, SGI0, SGI172761, SGI172762, SGI0, SGI172763, SGI0, SGI0, SGI0, SGI172764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38238[] = {SGI172765, SGI172766, SGI172767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172768, SGI0, SGI0, SGI0, SGI172769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38239[] = {SGI172773, SGI0, SGI172775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38240[] = {SGI172770, SGI0, SGI172771, SGI0, SGI0, SGI0, SGI172772, SGI172774, SGI0, SGI172776, SGI0, SGI0, SGI0, SGI172777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38241[] = {SGI172778, SGI0, SGI172779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172780, SGI172781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38242[] = {SGI172782, SGI172783, SGI172784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172785, SGI0, SGI0, SGI0, SGI172786, SGI172786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38243[] = {SGI172789, SGI0, SGI172790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38244[] = {SGI172787, SGI0, SGI172788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172791, SGI172792, SGI172792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38245[] = {SGI172793, SGI0, SGI172794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172795, SGI172796, SGI172796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38246[] = {SGI172797, SGI0, SGI172798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172799, SGI172800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38247[] = {SGI172801, SGI172802, SGI172803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172804, SGI0, SGI0, SGI0, SGI172805, SGI172805, SGI0, SGI172806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38248[] = {SGI172807, SGI172808, SGI172809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172810, SGI0, SGI0, SGI0, SGI172811, SGI172811, SGI0, SGI172812, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38249[] = {SGI172813, SGI0, SGI172814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172815, SGI172816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38250[] = {SGI172817, SGI0, SGI172818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172819, SGI172820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38251[] = {SGI172821, SGI0, SGI172822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172823, SGI172824, SGI172824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38252[] = {SGI172825, SGI172826, SGI172827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172828, SGI0, SGI0, SGI0, SGI172830, SGI172830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38253[] = {SGI172829, SGI0, SGI172831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172832, SGI172833, SGI172833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38254[] = {SGI172834, SGI0, SGI172835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38255[] = {SGI172836, SGI0, SGI172837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172838, SGI172839, SGI172839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38256[] = {SGI172840, SGI0, SGI172842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172844, SGI172846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38257[] = {SGI172841, SGI0, SGI172843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172845, SGI172847, SGI172847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38258[] = {SGI172848, SGI0, SGI172849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172852, SGI172853, SGI172853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38259[] = {SGI172850, SGI0, SGI172851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172854, SGI172855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38260[] = {SGI172856, SGI0, SGI172857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172858, SGI172859, SGI172859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38261[] = {SGI172860, SGI0, SGI172861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172862, SGI172863, SGI172863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38262[] = {SGI172864, SGI0, SGI172865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172866, SGI172867, SGI172867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38263[] = {SGI172868, SGI0, SGI172869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172870, SGI0, SGI0, SGI0, SGI172871, SGI172871, SGI0, SGI0, SGI0, SGI172872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38264[] = {SGI172873, SGI0, SGI172874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172875, SGI0, SGI0, SGI0, SGI172876, SGI172876, SGI0, SGI0, SGI0, SGI172877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38265[] = {SGI172878, SGI172879, SGI172880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172881, SGI0, SGI0, SGI0, SGI172882, SGI172882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38266[] = {SGI172883, SGI172884, SGI172885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172886, SGI0, SGI0, SGI0, SGI172887, SGI172887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38267[] = {SGI172888, SGI0, SGI172889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38268[] = {SGI172890, SGI0, SGI172891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38269[] = {SGI172892, SGI0, SGI172893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172894, SGI172895, SGI172895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38270[] = {SGI172896, SGI0, SGI172897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172898, SGI172899, SGI172899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38271[] = {SGI172900, SGI0, SGI172901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172902, SGI172903, SGI172903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38272[] = {SGI172904, SGI0, SGI172905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172906, SGI172907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38273[] = {SGI172908, SGI0, SGI172909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172910, SGI172911, SGI172911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38274[] = {SGI172912, SGI0, SGI172913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172914, SGI172915, SGI172915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38275[] = {SGI172916, SGI0, SGI172917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172918, SGI172919, SGI172919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38276[] = {SGI172920, SGI0, SGI172921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172922, SGI172923, SGI172923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38277[] = {SGI172924, SGI0, SGI172925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172926, SGI172927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38278[] = {SGI172928, SGI0, SGI172929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172930, SGI172931, SGI172931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38279[] = {SGI172932, SGI0, SGI172933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172934, SGI172935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38280[] = {SGI172936, SGI0, SGI172937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172938, SGI172939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38281[] = {SGI172940, SGI0, SGI172941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172942, SGI172943, SGI172943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38282[] = {SGI172944, SGI0, SGI172945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172946, SGI172947, SGI172947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38283[] = {SGI172948, SGI172949, SGI172950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172951, SGI0, SGI0, SGI0, SGI172952, SGI172952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38284[] = {SGI172953, SGI0, SGI172954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38285[] = {SGI172955, SGI0, SGI172956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172957, SGI0, SGI0, SGI0, SGI172958, SGI172958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38286[] = {SGI172959, SGI172960, SGI172961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172962, SGI0, SGI0, SGI0, SGI172963, SGI172963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38287[] = {SGI172964, SGI172965, SGI172966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172968, SGI0, SGI0, SGI0, SGI172970, SGI172970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38288[] = {SGI172967, SGI0, SGI172969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172971, SGI172972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38289[] = {SGI172973, SGI172974, SGI172975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172976, SGI0, SGI0, SGI0, SGI172977, SGI172977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38290[] = {SGI172978, SGI0, SGI172979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172980, SGI172981, SGI172981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38291[] = {SGI172982, SGI0, SGI172983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172984, SGI172985, SGI172985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38292[] = {SGI172986, SGI172987, SGI172988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172992, SGI0, SGI0, SGI0, SGI172993, SGI172993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38293[] = {SGI172989, SGI172990, SGI172991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI172994, SGI0, SGI0, SGI0, SGI172995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38294[] = {SGI172996, SGI0, SGI172997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38295[] = {SGI172998, SGI0, SGI173000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38296[] = {SGI172999, SGI0, SGI173001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173002, SGI173003, SGI173003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38297[] = {SGI173004, SGI0, SGI173005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173006, SGI173007, SGI173007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38298[] = {SGI173008, SGI0, SGI173009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173010, SGI173011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38299[] = {SGI173012, SGI173013, SGI173014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173015, SGI0, SGI0, SGI0, SGI173016, SGI173016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38300[] = {SGI173017, SGI173018, SGI173019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173020, SGI0, SGI0, SGI0, SGI173021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38301[] = {SGI173022, SGI0, SGI173023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173026, SGI173027, SGI173027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38302[] = {SGI173024, SGI0, SGI173025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173028, SGI173029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38303[] = {SGI173030, SGI0, SGI173031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173032, SGI173033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38304[] = {SGI173034, SGI0, SGI173035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173036, SGI173037, SGI173037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38305[] = {SGI173038, SGI0, SGI173039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173040, SGI173041, SGI173041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38306[] = {SGI173042, SGI0, SGI173043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173044, SGI173045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38307[] = {SGI173046, SGI0, SGI173047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173048, SGI173049, SGI173049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38308[] = {SGI173050, SGI0, SGI173051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173052, SGI173054, SGI173054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38309[] = {SGI173053, SGI173055, SGI173056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173057, SGI0, SGI0, SGI0, SGI173058, SGI173058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38310[] = {SGI173059, SGI173060, SGI173061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173062, SGI0, SGI0, SGI0, SGI173063, SGI173063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38311[] = {SGI173064, SGI173066, SGI173068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173070, SGI0, SGI0, SGI0, SGI173072, SGI173072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38312[] = {SGI173065, SGI173067, SGI173069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173071, SGI0, SGI0, SGI0, SGI173073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38313[] = {SGI173074, SGI0, SGI173075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38314[] = {SGI173076, SGI173077, SGI173078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173079, SGI0, SGI0, SGI0, SGI173080, SGI173080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38315[] = {SGI173081, SGI173082, SGI173083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173084, SGI0, SGI0, SGI0, SGI173087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38316[] = {SGI173085, SGI173086, SGI173088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173089, SGI0, SGI0, SGI0, SGI173090, SGI173090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38317[] = {SGI173091, SGI173092, SGI173093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173094, SGI0, SGI0, SGI0, SGI173095, SGI173095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38318[] = {SGI173096, SGI0, SGI173097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173098, SGI173099, SGI173099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38319[] = {SGI173100, SGI173101, SGI173102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173103, SGI0, SGI0, SGI0, SGI173104, SGI173104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38320[] = {SGI173106, SGI0, SGI173107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173109, SGI173111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38321[] = {SGI173108, SGI173110, SGI173112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173113, SGI0, SGI0, SGI0, SGI173114, SGI173114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38322[] = {SGI173116, SGI0, SGI173117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173118, SGI173120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38323[] = {SGI173119, SGI0, SGI173121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173122, SGI173123, SGI173123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38324[] = {SGI173124, SGI173125, SGI173126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173127, SGI0, SGI0, SGI0, SGI173128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38325[] = {SGI173129, SGI0, SGI173130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173131, SGI173132, SGI173132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38326[] = {SGI173133, SGI0, SGI173134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173135, SGI173136, SGI173136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38327[] = {SGI173137, SGI0, SGI173138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173139, SGI173140, SGI173140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38328[] = {SGI173141, SGI0, SGI173142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173143, SGI173144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38329[] = {SGI173145, SGI0, SGI173146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173147, SGI173148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38330[] = {SGI173149, SGI173150, SGI173151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173152, SGI0, SGI0, SGI0, SGI173153, SGI173153, SGI0, SGI173154, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38331[] = {SGI173155, SGI173156, SGI173157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173158, SGI0, SGI0, SGI0, SGI173159, SGI173159, SGI0, SGI173160, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38332[] = {SGI173161, SGI173162, SGI173163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173164, SGI0, SGI0, SGI0, SGI173165, SGI173165, SGI0, SGI173166, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38333[] = {SGI173167, SGI0, SGI173168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173169, SGI173170, SGI173170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38334[] = {SGI173171, SGI173172, SGI173173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173174, SGI0, SGI0, SGI0, SGI173175, SGI173175, SGI0, SGI173177, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38335[] = {SGI173176, SGI0, SGI173178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173179, SGI173180, SGI173180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38336[] = {SGI173181, SGI173182, SGI173183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173184, SGI0, SGI0, SGI173185, SGI173186, SGI173186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38337[] = {SGI173187, SGI0, SGI173188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173189, SGI173190, SGI173190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38338[] = {SGI173191, SGI0, SGI173192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173193, SGI173194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38339[] = {SGI173195, SGI0, SGI173196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173197, SGI173198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38340[] = {SGI173199, SGI0, SGI173200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173201, SGI173202, SGI173202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38341[] = {SGI173203, SGI0, SGI173204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173205, SGI173206, SGI173206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38342[] = {SGI173207, SGI173208, SGI173209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173210, SGI0, SGI0, SGI0, SGI173211, SGI173211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38343[] = {SGI173212, SGI173213, SGI173214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173215, SGI0, SGI0, SGI0, SGI173216, SGI173216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38344[] = {SGI173217, SGI0, SGI173218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173219, SGI173220, SGI173220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38345[] = {SGI173221, SGI0, SGI173222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173223, SGI173224, SGI173224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38346[] = {SGI173225, SGI0, SGI173226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173227, SGI173228, SGI173228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38347[] = {SGI173229, SGI173230, SGI173231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173234, SGI0, SGI0, SGI0, SGI173235, SGI173235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38348[] = {SGI173232, SGI0, SGI173233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173236, SGI173237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38349[] = {SGI173238, SGI173239, SGI173240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173241, SGI0, SGI0, SGI0, SGI173242, SGI173242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38350[] = {SGI173243, SGI0, SGI173244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38351[] = {SGI173245, SGI173246, SGI173247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173248, SGI0, SGI0, SGI0, SGI173249, SGI173249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38352[] = {SGI173250, SGI173251, SGI173252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173253, SGI0, SGI0, SGI0, SGI173254, SGI173254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38353[] = {SGI173255, SGI173256, SGI173257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173258, SGI0, SGI0, SGI0, SGI173260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38354[] = {SGI173259, SGI173261, SGI173262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173263, SGI0, SGI0, SGI0, SGI173264, SGI173264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38355[] = {SGI173265, SGI0, SGI173266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38356[] = {SGI173267, SGI173268, SGI173269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173270, SGI0, SGI0, SGI0, SGI173271, SGI173271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38357[] = {SGI173272, SGI173273, SGI173274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173275, SGI0, SGI0, SGI0, SGI173277, SGI173277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38358[] = {SGI173276, SGI0, SGI173278, SGI0, SGI173279, SGI173279, SGI0, NULL};
const Segment * const SG38359[] = {SGI173280, SGI0, SGI173281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173282, SGI173283, SGI173283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38360[] = {SGI173284, SGI0, SGI173285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173286, SGI173287, SGI173287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38361[] = {SGI173288, SGI173289, SGI173290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173294, SGI0, SGI0, SGI0, SGI173295, SGI173295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38362[] = {SGI173291, SGI173292, SGI173293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173296, SGI0, SGI0, SGI0, SGI173297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38363[] = {SGI173298, SGI0, SGI173299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38364[] = {SGI173300, SGI0, SGI173301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173302, SGI173303, SGI173303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38365[] = {SGI173304, SGI173305, SGI173306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173307, SGI0, SGI0, SGI0, SGI173308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38366[] = {SGI173309, SGI173310, SGI173311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173312, SGI0, SGI0, SGI0, SGI173313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38367[] = {SGI173314, SGI0, SGI173315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38368[] = {SGI173316, SGI173317, SGI173318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173319, SGI0, SGI0, SGI0, SGI173320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38369[] = {SGI173321, SGI0, SGI173322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173323, SGI173324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38370[] = {SGI173325, SGI0, SGI173326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173327, SGI173328, SGI173328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38371[] = {SGI173329, SGI0, SGI173330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173331, SGI173332, SGI173332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38372[] = {SGI173333, SGI173334, SGI173335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173336, SGI0, SGI0, SGI0, SGI173337, SGI173337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38373[] = {SGI173338, SGI0, SGI173339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173340, SGI173341, SGI173341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38374[] = {SGI173342, SGI0, SGI173343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38375[] = {SGI173344, SGI173345, SGI173346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173347, SGI0, SGI0, SGI0, SGI173348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38376[] = {SGI173349, SGI173350, SGI173351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173352, SGI0, SGI0, SGI0, SGI173353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38377[] = {SGI173354, SGI0, SGI173355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173356, SGI173357, SGI173357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38378[] = {SGI173358, SGI0, SGI173359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173360, SGI173361, SGI173361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38379[] = {SGI173362, SGI0, SGI173363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173364, SGI173365, SGI173365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38380[] = {SGI173366, SGI0, SGI173367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173368, SGI173369, SGI173369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38381[] = {SGI173370, SGI0, SGI173371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173372, SGI173373, SGI173373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38382[] = {SGI173374, SGI173375, SGI173376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173377, SGI0, SGI0, SGI0, SGI173378, SGI173378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38383[] = {SGI173379, SGI0, SGI173380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38384[] = {SGI173381, SGI0, SGI173382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173383, SGI173384, SGI173384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38385[] = {SGI173385, SGI173386, SGI173387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173388, SGI0, SGI0, SGI0, SGI173389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38386[] = {SGI173390, SGI173391, SGI173392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173393, SGI0, SGI0, SGI0, SGI173394, SGI173394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38387[] = {SGI173395, SGI0, SGI173396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38388[] = {SGI173397, SGI0, SGI173398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173399, SGI173400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38389[] = {SGI173401, SGI0, SGI173402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173403, SGI173404, SGI173404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38390[] = {SGI173405, SGI173406, SGI173407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173408, SGI0, SGI0, SGI0, SGI173409, SGI173409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38391[] = {SGI173410, SGI173411, SGI173412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173413, SGI0, SGI0, SGI0, SGI173414, SGI173414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38392[] = {SGI173415, SGI173416, SGI173417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173418, SGI0, SGI0, SGI0, SGI173419, SGI173419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38393[] = {SGI173420, SGI173421, SGI173422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173423, SGI0, SGI0, SGI0, SGI173424, SGI173424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38394[] = {SGI173425, SGI173426, SGI173427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173428, SGI0, SGI0, SGI0, SGI173429, SGI173429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38395[] = {SGI173430, SGI0, SGI173431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38396[] = {SGI173432, SGI173433, SGI173434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173435, SGI0, SGI0, SGI0, SGI173437, SGI173437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38397[] = {SGI173436, SGI0, SGI173438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173439, SGI173440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38398[] = {SGI173441, SGI173442, SGI173443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173446, SGI0, SGI0, SGI0, SGI173447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38399[] = {SGI173444, SGI0, SGI173445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173448, SGI173449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38400[] = {SGI173450, SGI173451, SGI173452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173453, SGI0, SGI0, SGI0, SGI173454, SGI173454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38401[] = {SGI173455, SGI173456, SGI173457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173458, SGI0, SGI0, SGI0, SGI173459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38402[] = {SGI173460, SGI0, SGI173461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38403[] = {SGI173462, SGI0, SGI173463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38404[] = {SGI173464, SGI173465, SGI173466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173467, SGI0, SGI0, SGI0, SGI173468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38405[] = {SGI173469, SGI173470, SGI173471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173472, SGI0, SGI0, SGI0, SGI173473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38406[] = {SGI173474, SGI0, SGI173475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173476, SGI173477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38407[] = {SGI173478, SGI0, SGI173479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173480, SGI173481, SGI173481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38408[] = {SGI0, SGI0, SGI173482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173483, SGI173484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38409[] = {SGI173485, SGI173486, SGI173487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173488, SGI0, SGI0, SGI0, SGI173489, SGI173489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38410[] = {SGI173490, SGI0, SGI173491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38411[] = {SGI173492, SGI173493, SGI173494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173498, SGI0, SGI0, SGI0, SGI173499, SGI0, SGI0, SGI173501, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38412[] = {SGI173495, SGI173496, SGI173497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173500, SGI0, SGI0, SGI0, SGI173502, SGI173502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38413[] = {SGI173503, SGI173504, SGI173505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173506, SGI0, SGI0, SGI0, SGI173507, SGI0, SGI0, SGI173508, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38414[] = {SGI173509, SGI173510, SGI173511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173512, SGI0, SGI0, SGI0, SGI173513, SGI173513, SGI0, SGI173514, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38415[] = {SGI173515, SGI0, SGI173516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173517, SGI0, SGI0, SGI0, SGI173518, SGI0, SGI0, SGI173519, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38416[] = {SGI173520, SGI0, SGI173521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173522, SGI173523, SGI173523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38417[] = {SGI173524, SGI0, SGI173525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173526, SGI173527, SGI173527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38418[] = {SGI173528, SGI0, SGI173529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173530, SGI173532, SGI173532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38419[] = {SGI173531, SGI0, SGI173533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173534, SGI173535, SGI173535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38420[] = {SGI173536, SGI0, SGI173537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173538, SGI173539, SGI173539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38421[] = {SGI173540, SGI0, SGI173541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173542, SGI173543, SGI173543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38422[] = {SGI173544, SGI0, SGI173545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173546, SGI173547, SGI173547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38423[] = {SGI173548, SGI0, SGI173549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173550, SGI173551, SGI173551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38424[] = {SGI173552, SGI173553, SGI173554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173555, SGI0, SGI0, SGI0, SGI173556, SGI173556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38425[] = {SGI173557, SGI0, SGI173558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38426[] = {SGI173559, SGI0, SGI173560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173561, SGI173562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38427[] = {SGI173563, SGI0, SGI173564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173565, SGI173566, SGI173566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38428[] = {SGI173567, SGI173568, SGI173569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173570, SGI0, SGI0, SGI0, SGI173571, SGI173571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38429[] = {SGI173572, SGI173573, SGI173574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173575, SGI0, SGI0, SGI0, SGI173576, SGI173576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38430[] = {SGI173577, SGI0, SGI173578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38431[] = {SGI173579, SGI173580, SGI173581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173582, SGI0, SGI0, SGI0, SGI173583, SGI173583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38432[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173587, SGI0, SGI0, SGI173589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38433[] = {SGI173584, SGI173585, SGI173586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173588, SGI0, SGI0, SGI0, SGI173590, SGI173590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38434[] = {SGI173591, SGI0, SGI173592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38435[] = {SGI173593, SGI173594, SGI173595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173596, SGI0, SGI0, SGI0, SGI173597, SGI173597, SGI0, SGI173598, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38436[] = {SGI173600, SGI173601, SGI173603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173604, SGI0, SGI0, SGI0, SGI173607, SGI173607, SGI0, SGI173608, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38437[] = {SGI173599, SGI0, SGI173602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173605, SGI173606, SGI173606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38438[] = {SGI173609, SGI0, SGI173610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173611, SGI173612, SGI173612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38439[] = {SGI173613, SGI0, SGI173614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173615, SGI173616, SGI173616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38440[] = {SGI173617, SGI0, SGI173618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173619, SGI173620, SGI173620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38441[] = {SGI173621, SGI0, SGI173622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173623, SGI173624, SGI173624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38442[] = {SGI173625, SGI0, SGI173626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173627, SGI173628, SGI173628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38443[] = {SGI173629, SGI173630, SGI173631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173632, SGI0, SGI0, SGI0, SGI173633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38444[] = {SGI173634, SGI0, SGI173635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38445[] = {SGI173636, SGI0, SGI173637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173638, SGI173639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38446[] = {SGI173640, SGI0, SGI173641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173642, SGI173643, SGI173643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38447[] = {SGI173644, SGI0, SGI173645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173646, SGI173647, SGI173647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38448[] = {SGI173648, SGI0, SGI173649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173650, SGI173651, SGI173651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38449[] = {SGI173652, SGI0, SGI173653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173654, SGI173655, SGI173655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38450[] = {SGI173656, SGI0, SGI173657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173658, SGI173659, SGI173659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38451[] = {SGI173660, SGI0, SGI173661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173662, SGI173663, SGI173663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38452[] = {SGI173664, SGI173665, SGI173666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173667, SGI0, SGI0, SGI0, SGI173668, SGI173668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38453[] = {SGI173669, SGI173670, SGI173671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173672, SGI0, SGI0, SGI0, SGI173673, SGI173673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38454[] = {SGI173674, SGI173675, SGI173676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173677, SGI0, SGI0, SGI0, SGI173678, SGI173678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38455[] = {SGI173679, SGI173680, SGI173681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173682, SGI0, SGI0, SGI0, SGI173683, SGI173683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38456[] = {SGI173684, SGI0, SGI173685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38457[] = {SGI173686, SGI173687, SGI173689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173691, SGI0, SGI0, SGI0, SGI173693, SGI0, SGI0, SGI173695, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38458[] = {SGI173688, SGI0, SGI173690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173692, SGI173694, SGI173694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38459[] = {SGI173696, SGI173697, SGI173698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173699, SGI0, SGI0, SGI0, SGI173701, SGI173701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38460[] = {SGI173700, SGI173702, SGI173703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173704, SGI0, SGI0, SGI0, SGI173705, SGI0, SGI0, SGI173706, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38461[] = {SGI173707, SGI173708, SGI173709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173710, SGI0, SGI0, SGI0, SGI173711, SGI173711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38462[] = {SGI173712, SGI0, SGI173713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173717, SGI173718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38463[] = {SGI173714, SGI173715, SGI173716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173719, SGI0, SGI0, SGI0, SGI173720, SGI173720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38464[] = {SGI173721, SGI173722, SGI173723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173724, SGI0, SGI0, SGI0, SGI173725, SGI173725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38465[] = {SGI173727, SGI0, SGI173729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38466[] = {SGI173726, SGI173728, SGI173730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173731, SGI0, SGI0, SGI0, SGI173732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38467[] = {SGI173735, SGI173737, SGI173738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173740, SGI0, SGI0, SGI0, SGI173742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38468[] = {SGI173733, SGI173734, SGI173736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173739, SGI0, SGI0, SGI0, SGI173741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38469[] = {SGI173743, SGI173744, SGI173745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173746, SGI0, SGI0, SGI0, SGI173747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38470[] = {SGI173748, SGI173749, SGI173750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173751, SGI0, SGI0, SGI0, SGI173752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38471[] = {SGI173753, SGI173754, SGI173755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173759, SGI0, SGI0, SGI0, SGI173760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38472[] = {SGI173756, SGI173757, SGI173758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173761, SGI0, SGI0, SGI0, SGI173762, SGI173762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38473[] = {SGI173763, SGI173764, SGI173765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173766, SGI0, SGI0, SGI0, SGI173767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38474[] = {SGI173768, SGI0, SGI173769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38475[] = {SGI173770, SGI173771, SGI173772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173773, SGI0, SGI0, SGI0, SGI173774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38476[] = {SGI173775, SGI173776, SGI173777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173778, SGI0, SGI0, SGI0, SGI173779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38477[] = {SGI173780, SGI0, SGI173781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173782, SGI173783, SGI173783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38478[] = {SGI173784, SGI0, SGI173785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173786, SGI173787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38479[] = {SGI173788, SGI0, SGI173789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173790, SGI173791, SGI173791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38480[] = {SGI173792, SGI0, SGI173793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173794, SGI173795, SGI173795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38481[] = {SGI173796, SGI0, SGI173797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173798, SGI173799, SGI173799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38482[] = {SGI173800, SGI0, SGI173801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173802, SGI173803, SGI173803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38483[] = {SGI173804, SGI173805, SGI173806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173807, SGI0, SGI0, SGI0, SGI173808, SGI173808, SGI0, SGI173809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38484[] = {SGI173810, SGI0, SGI173811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173812, SGI173813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38485[] = {SGI173814, SGI173815, SGI173816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173817, SGI0, SGI0, SGI0, SGI173818, SGI173818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38486[] = {SGI173820, SGI173822, SGI173824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173826, SGI0, SGI0, SGI0, SGI173828, SGI173828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38487[] = {SGI173821, SGI173823, SGI173825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173827, SGI0, SGI0, SGI0, SGI173829, SGI173829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38488[] = {SGI173831, SGI0, SGI173832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38489[] = {SGI173833, SGI0, SGI173834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173835, SGI0, SGI0, SGI0, SGI173836, SGI173836, SGI0, SGI0, SGI0, SGI173837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38490[] = {SGI173838, SGI173839, SGI173840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173841, SGI0, SGI0, SGI0, SGI173842, SGI173842, SGI0, SGI0, SGI173843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38491[] = {SGI173844, SGI173845, SGI173846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173847, SGI0, SGI0, SGI0, SGI173848, SGI173848, SGI0, SGI0, SGI173849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38492[] = {SGI173850, SGI173851, SGI173852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173853, SGI0, SGI0, SGI0, SGI173854, SGI173854, SGI0, SGI0, SGI173855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38493[] = {SGI173856, SGI173857, SGI173858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173859, SGI0, SGI0, SGI0, SGI173860, SGI173860, SGI0, SGI0, SGI173861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38494[] = {SGI173862, SGI173863, SGI173864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173866, SGI0, SGI0, SGI0, SGI173868, SGI173868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173869, SGI0, NULL};
const Segment * const SG38495[] = {SGI173865, SGI0, SGI173867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173870, SGI173871, SGI173871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38496[] = {SGI173872, SGI173873, SGI173874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173875, SGI0, SGI0, SGI0, SGI173876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38497[] = {SGI173877, SGI173878, SGI173879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173880, SGI0, SGI0, SGI0, SGI173881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38498[] = {SGI173882, SGI0, SGI173883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38499[] = {SGI173884, SGI0, SGI173885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173886, SGI173887, SGI173887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38500[] = {SGI173888, SGI0, SGI173889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173890, SGI173891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38501[] = {SGI173892, SGI0, SGI173893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173894, SGI173895, SGI173895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38502[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173896, SGI0, SGI0, SGI173897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38503[] = {SGI173898, SGI0, SGI173899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173900, SGI173901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38504[] = {SGI173902, SGI0, SGI173903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173904, SGI173905, SGI173905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38505[] = {SGI173906, SGI0, SGI173907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173908, SGI173909, SGI173909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38506[] = {SGI173910, SGI0, SGI173911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173912, SGI173913, SGI173913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38507[] = {SGI173914, SGI173915, SGI173916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173917, SGI0, SGI0, SGI0, SGI173918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38508[] = {SGI173919, SGI173920, SGI173921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173922, SGI0, SGI0, SGI0, SGI173923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38509[] = {SGI173924, SGI173925, SGI173926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173927, SGI0, SGI0, SGI0, SGI173928, SGI173928, SGI0, SGI173929, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38510[] = {SGI173930, SGI173931, SGI173932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173933, SGI0, SGI0, SGI0, SGI173934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38511[] = {SGI0, SGI0, SGI173935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173936, SGI173936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38512[] = {SGI173937, SGI173938, SGI173939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173940, SGI0, SGI0, SGI0, SGI173941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38513[] = {SGI173942, SGI173943, SGI173944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173945, SGI173945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38514[] = {SGI173946, SGI173947, SGI173948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173949, SGI0, SGI0, SGI0, SGI173950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38515[] = {SGI173951, SGI173952, SGI173953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173954, SGI0, SGI0, SGI0, SGI173956, SGI173956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38516[] = {SGI173955, SGI0, SGI173957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38517[] = {SGI173959, SGI0, SGI173961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38518[] = {SGI173958, SGI0, SGI173960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38519[] = {SGI173962, SGI173963, SGI173964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173965, SGI173965, SGI0, SGI0, SGI0, SGI0, SGI173966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38520[] = {SGI173967, SGI173968, SGI173969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173970, SGI173970, SGI0, SGI0, SGI0, SGI0, SGI173971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38521[] = {SGI173972, SGI173973, SGI173974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173975, SGI173975, SGI0, SGI0, SGI0, SGI0, SGI173976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38522[] = {SGI173977, SGI0, SGI173978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173979, SGI173980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38523[] = {SGI173981, SGI0, SGI173982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173983, SGI173984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38524[] = {SGI173985, SGI173986, SGI173987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173988, SGI0, SGI0, SGI0, SGI173989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38525[] = {SGI173990, SGI0, SGI173991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173992, SGI173993, SGI173993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173994, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38526[] = {SGI173995, SGI0, SGI173996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173997, SGI173998, SGI173998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI173999, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38527[] = {SGI174000, SGI0, SGI174001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174004, SGI174005, SGI174005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174008, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38528[] = {SGI174002, SGI0, SGI174003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174006, SGI174007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38529[] = {SGI174009, SGI174010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174011, SGI174012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38530[] = {SGI174013, SGI0, SGI174014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174015, SGI174016, SGI174016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38531[] = {SGI174017, SGI174018, SGI174019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174020, SGI0, SGI0, SGI0, SGI174021, SGI174021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38532[] = {SGI174022, SGI174023, SGI174024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174025, SGI0, SGI0, SGI0, SGI174026, SGI174026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38533[] = {SGI174027, SGI174028, SGI174029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174030, SGI0, SGI0, SGI0, SGI174031, SGI174031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38534[] = {SGI174032, SGI174033, SGI174034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174035, SGI0, SGI0, SGI0, SGI174036, SGI174036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38535[] = {SGI174037, SGI0, SGI174038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174039, SGI174040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38536[] = {SGI174041, SGI0, SGI174042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174043, SGI174044, SGI174044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38537[] = {SGI174045, SGI0, SGI174046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174047, SGI174048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38538[] = {SGI174049, SGI0, SGI174050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174051, SGI174052, SGI174052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38539[] = {SGI174053, SGI0, SGI174054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174055, SGI174056, SGI174056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38540[] = {SGI174057, SGI174058, SGI174059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174060, SGI0, SGI0, SGI0, SGI174061, SGI174061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38541[] = {SGI174062, SGI174063, SGI174064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174065, SGI0, SGI0, SGI0, SGI174066, SGI174066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38542[] = {SGI174067, SGI174068, SGI174069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174070, SGI0, SGI0, SGI0, SGI174072, SGI174072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38543[] = {SGI174071, SGI0, SGI174073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174074, SGI174075, SGI174075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38544[] = {SGI174076, SGI0, SGI174077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38545[] = {SGI174078, SGI174079, SGI174080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174081, SGI0, SGI0, SGI0, SGI174082, SGI174082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38546[] = {SGI174083, SGI0, SGI174084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38547[] = {SGI174085, SGI0, SGI174086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174087, SGI174088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38548[] = {SGI174089, SGI174090, SGI174091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174092, SGI0, SGI0, SGI0, SGI174093, SGI174093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38549[] = {SGI174094, SGI174095, SGI174096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174097, SGI0, SGI0, SGI0, SGI174098, SGI174098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38550[] = {SGI174099, SGI174100, SGI174101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174102, SGI0, SGI0, SGI0, SGI174103, SGI174103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38551[] = {SGI174104, SGI174105, SGI174106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174107, SGI0, SGI0, SGI0, SGI174108, SGI174108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38552[] = {SGI174109, SGI174110, SGI174111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174112, SGI0, SGI0, SGI0, SGI174113, SGI174113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38553[] = {SGI174114, SGI0, SGI174115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174116, SGI174117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38554[] = {SGI174118, SGI174119, SGI174120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174121, SGI0, SGI0, SGI0, SGI174122, SGI174122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38555[] = {SGI174123, SGI174124, SGI174126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174129, SGI0, SGI0, SGI0, SGI174130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38556[] = {SGI174125, SGI174127, SGI174128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174131, SGI0, SGI0, SGI0, SGI174132, SGI174132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38557[] = {SGI174133, SGI0, SGI174134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38558[] = {SGI174135, SGI174136, SGI174137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174138, SGI0, SGI0, SGI0, SGI174139, SGI174139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38559[] = {SGI174140, SGI174141, SGI174142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174143, SGI174143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38560[] = {SGI174144, SGI0, SGI174145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38561[] = {SGI174146, SGI0, SGI174147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38562[] = {SGI174148, SGI0, SGI174149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38563[] = {SGI174150, SGI0, SGI174151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174152, SGI174153, SGI174153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38564[] = {SGI174154, SGI174155, SGI174156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174157, SGI0, SGI0, SGI0, SGI174158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38565[] = {SGI174159, SGI174160, SGI174161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174162, SGI0, SGI0, SGI0, SGI174163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38566[] = {SGI174164, SGI0, SGI174165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174166, SGI174167, SGI174167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38567[] = {SGI174168, SGI174169, SGI174170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174171, SGI0, SGI0, SGI0, SGI174172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38568[] = {SGI174173, SGI0, SGI174174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174175, SGI174176, SGI174176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38569[] = {SGI174177, SGI0, SGI174178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174179, SGI174180, SGI174180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38570[] = {SGI174181, SGI174182, SGI174183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174184, SGI0, SGI0, SGI0, SGI174185, SGI174185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38571[] = {SGI174186, SGI174187, SGI174188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174189, SGI0, SGI0, SGI0, SGI174190, SGI174190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38572[] = {SGI174191, SGI0, SGI174192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38573[] = {SGI174193, SGI0, SGI174194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174195, SGI174196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38574[] = {SGI174197, SGI0, SGI174198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174199, SGI174200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38575[] = {SGI174201, SGI0, SGI174202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174203, SGI174204, SGI174204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38576[] = {SGI174205, SGI174206, SGI174207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174208, SGI0, SGI0, SGI0, SGI174209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38577[] = {SGI174210, SGI0, SGI174211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174212, SGI174213, SGI174213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38578[] = {SGI174214, SGI0, SGI174215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174216, SGI174217, SGI174217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38579[] = {SGI174218, SGI0, SGI174219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174220, SGI174221, SGI174221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38580[] = {SGI174222, SGI0, SGI174223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174226, SGI174227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38581[] = {SGI174224, SGI0, SGI174225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174228, SGI174229, SGI174229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38582[] = {SGI174230, SGI0, SGI174231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174232, SGI174233, SGI174233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38583[] = {SGI174234, SGI0, SGI174235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174236, SGI174237, SGI174237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38584[] = {SGI174238, SGI174239, SGI174240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174241, SGI0, SGI0, SGI0, SGI174242, SGI174242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38585[] = {SGI174243, SGI174244, SGI174245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174246, SGI0, SGI0, SGI0, SGI174247, SGI174247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38586[] = {SGI174248, SGI174249, SGI174250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174251, SGI0, SGI0, SGI0, SGI174252, SGI174252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38587[] = {SGI174253, SGI0, SGI174254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38588[] = {SGI174255, SGI174256, SGI174257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174258, SGI0, SGI0, SGI0, SGI174259, SGI174259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38589[] = {SGI174260, SGI0, SGI174261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38590[] = {SGI174262, SGI174263, SGI174264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174265, SGI0, SGI0, SGI0, SGI174266, SGI174266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38591[] = {SGI174267, SGI174268, SGI174269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174270, SGI0, SGI0, SGI0, SGI174273, SGI174273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38592[] = {SGI174271, SGI174272, SGI174274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174275, SGI0, SGI0, SGI0, SGI174276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38593[] = {SGI174277, SGI174278, SGI174279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174280, SGI0, SGI0, SGI0, SGI174281, SGI174281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38594[] = {SGI174282, SGI174283, SGI174284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174285, SGI0, SGI0, SGI0, SGI174286, SGI174286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38595[] = {SGI174287, SGI174288, SGI174289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174290, SGI0, SGI0, SGI0, SGI174291, SGI174291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38596[] = {SGI174292, SGI174293, SGI174294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174295, SGI0, SGI0, SGI0, SGI174296, SGI174296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38597[] = {SGI174297, SGI0, SGI174298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38598[] = {SGI174299, SGI0, SGI174300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174301, SGI174302, SGI174302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38599[] = {SGI174303, SGI0, SGI174304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174305, SGI174306, SGI174306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38600[] = {SGI174307, SGI174308, SGI174309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174310, SGI0, SGI0, SGI0, SGI174311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38601[] = {SGI174312, SGI174313, SGI174314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174315, SGI0, SGI0, SGI0, SGI174316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38602[] = {SGI174317, SGI0, SGI174318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38603[] = {SGI174319, SGI174320, SGI174321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174322, SGI0, SGI0, SGI0, SGI174323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38604[] = {SGI174324, SGI0, SGI174325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38605[] = {SGI174326, SGI0, SGI174327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174328, SGI174329, SGI174329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38606[] = {SGI174330, SGI0, SGI174331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174332, SGI174333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38607[] = {SGI174334, SGI0, SGI174335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174336, SGI174337, SGI174337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38608[] = {SGI174338, SGI174339, SGI174340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174341, SGI0, SGI0, SGI0, SGI174342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38609[] = {SGI174344, SGI174345, SGI174346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174347, SGI0, SGI0, SGI0, SGI174348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38610[] = {SGI174349, SGI174350, SGI174352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174353, SGI0, SGI0, SGI0, SGI174354, SGI174354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38611[] = {SGI174355, SGI174356, SGI174357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174358, SGI0, SGI0, SGI0, SGI174359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38612[] = {SGI174361, SGI0, SGI174362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174363, SGI174364, SGI174364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38613[] = {SGI0, SGI174365, SGI174366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174367, SGI0, SGI0, SGI0, SGI174368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38614[] = {SGI174369, SGI0, SGI174370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174371, SGI174372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38615[] = {SGI174373, SGI0, SGI174374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174375, SGI174376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38616[] = {SGI174377, SGI174378, SGI174379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174380, SGI174380, SGI0, SGI0, SGI0, SGI0, SGI174381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38617[] = {SGI174382, SGI174383, SGI174384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174385, SGI174385, SGI0, SGI0, SGI0, SGI0, SGI174386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38618[] = {SGI174387, SGI174388, SGI174389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174390, SGI174390, SGI0, SGI0, SGI0, SGI0, SGI174391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38619[] = {SGI174392, SGI174393, SGI174394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174395, SGI174395, SGI0, SGI0, SGI0, SGI0, SGI174396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38620[] = {SGI174397, SGI0, SGI174398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174399, SGI174400, SGI174400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38621[] = {SGI174401, SGI174402, SGI174403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174405, SGI0, SGI0, SGI0, SGI174407, SGI174407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38622[] = {SGI174404, SGI0, SGI174406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174409, SGI174410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38623[] = {SGI174411, SGI174412, SGI174413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174417, SGI0, SGI0, SGI0, SGI174419, SGI174419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38624[] = {SGI174414, SGI174415, SGI174416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174418, SGI0, SGI0, SGI0, SGI174420, SGI174420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38625[] = {SGI174422, SGI0, SGI174423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38626[] = {SGI174424, SGI174425, SGI174426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174427, SGI0, SGI0, SGI0, SGI174428, SGI174428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38627[] = {SGI174430, SGI0, SGI174431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174432, SGI174433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38628[] = {SGI174434, SGI0, SGI174435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174436, SGI174437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38629[] = {SGI174438, SGI0, SGI174439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174440, SGI174441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38630[] = {SGI174442, SGI0, SGI174443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174444, SGI174445, SGI174445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38631[] = {SGI174446, SGI0, SGI174447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174448, SGI174449, SGI174449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38632[] = {SGI174450, SGI0, SGI174451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174452, SGI174453, SGI174453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38633[] = {SGI174454, SGI0, SGI174455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174456, SGI174457, SGI174457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38634[] = {SGI174458, SGI0, SGI174459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174460, SGI174461, SGI174461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38635[] = {SGI174462, SGI174463, SGI174464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174465, SGI0, SGI0, SGI0, SGI174467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38636[] = {SGI174466, SGI174468, SGI174469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174471, SGI0, SGI0, SGI0, SGI174472, SGI174472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38637[] = {SGI174473, SGI0, SGI174474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38638[] = {SGI174475, SGI174476, SGI174477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174478, SGI0, SGI0, SGI0, SGI174479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38639[] = {SGI174481, SGI0, SGI174482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174483, SGI174484, SGI174484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38640[] = {SGI174485, SGI174486, SGI174487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174488, SGI0, SGI0, SGI0, SGI174489, SGI174489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38641[] = {SGI174490, SGI0, SGI174491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38642[] = {SGI174492, SGI174493, SGI174494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174495, SGI0, SGI0, SGI0, SGI174496, SGI174496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38643[] = {SGI174497, SGI174498, SGI174499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174500, SGI0, SGI0, SGI0, SGI174501, SGI174501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38644[] = {SGI174502, SGI174503, SGI174504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174505, SGI0, SGI0, SGI0, SGI174506, SGI174506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38645[] = {SGI174507, SGI0, SGI174508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38646[] = {SGI174509, SGI0, SGI174510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38647[] = {SGI174511, SGI174512, SGI174513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174514, SGI0, SGI0, SGI0, SGI174515, SGI174515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38648[] = {SGI174516, SGI174517, SGI174518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174519, SGI0, SGI0, SGI0, SGI174520, SGI174520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38649[] = {SGI174523, SGI0, SGI174525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38650[] = {SGI174521, SGI174522, SGI174524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174526, SGI0, SGI0, SGI0, SGI174527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38651[] = {SGI174528, SGI174529, SGI174530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174531, SGI0, SGI0, SGI0, SGI174534, SGI174534, SGI0, SGI174538, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38652[] = {SGI174532, SGI174533, SGI174536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174539, SGI0, SGI0, SGI0, SGI174540, SGI174540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38653[] = {SGI174535, SGI0, SGI174537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174541, SGI174542, SGI174542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38654[] = {SGI174543, SGI174544, SGI174545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174546, SGI0, SGI0, SGI0, SGI174547, SGI174547, SGI0, SGI174548, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38655[] = {SGI174549, SGI0, SGI174550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174551, SGI174552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38656[] = {SGI174553, SGI174554, SGI174555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174556, SGI0, SGI0, SGI0, SGI174557, SGI174557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38657[] = {SGI174558, SGI0, SGI174559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174560, SGI174561, SGI174561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38658[] = {SGI174562, SGI174563, SGI174564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174565, SGI0, SGI0, SGI0, SGI174566, SGI174566, SGI0, SGI174567, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38659[] = {SGI174568, SGI174569, SGI174570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174571, SGI0, SGI0, SGI0, SGI174572, SGI174572, SGI0, SGI174573, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38660[] = {SGI174574, SGI174575, SGI174576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174577, SGI0, SGI0, SGI0, SGI174578, SGI174578, SGI0, SGI174579, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38661[] = {SGI174580, SGI174581, SGI174582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174583, SGI0, SGI0, SGI0, SGI174584, SGI174584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38662[] = {SGI174585, SGI174586, SGI174587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174588, SGI0, SGI0, SGI0, SGI174589, SGI174589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38663[] = {SGI174590, SGI0, SGI174591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38664[] = {SGI174592, SGI0, SGI174593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174594, SGI174595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38665[] = {SGI174596, SGI0, SGI174597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174598, SGI174599, SGI174599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38666[] = {SGI174600, SGI0, SGI174601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174602, SGI174603, SGI174603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38667[] = {SGI174604, SGI0, SGI174605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174606, SGI174607, SGI174607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174608, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38668[] = {SGI174609, SGI0, SGI174610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174612, SGI174613, SGI174613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38669[] = {SGI174611, SGI0, SGI174614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174615, SGI174616, SGI174616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174617, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38670[] = {SGI174618, SGI0, SGI174619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174620, SGI174621, SGI174621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174622, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38671[] = {SGI174623, SGI0, SGI174624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174625, SGI174626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38672[] = {SGI174627, SGI174628, SGI174630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174632, SGI0, SGI0, SGI0, SGI174633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38673[] = {SGI174629, SGI0, SGI174631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174634, SGI174635, SGI174635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38674[] = {SGI174636, SGI174637, SGI174638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174639, SGI0, SGI0, SGI0, SGI174640, SGI174640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38675[] = {SGI174641, SGI0, SGI174642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38676[] = {SGI174643, SGI174644, SGI174645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174646, SGI0, SGI0, SGI0, SGI174647, SGI174647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38677[] = {SGI174648, SGI174649, SGI174650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174651, SGI0, SGI0, SGI0, SGI174653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38678[] = {SGI174652, SGI0, SGI174654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174655, SGI174656, SGI174656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38679[] = {SGI174657, SGI0, SGI174658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174660, SGI174661, SGI174661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38680[] = {SGI174659, SGI0, SGI174662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174663, SGI174664, SGI174664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38681[] = {SGI174665, SGI0, SGI174666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174667, SGI174668, SGI174668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38682[] = {SGI174669, SGI0, SGI174670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174671, SGI174672, SGI174672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38683[] = {SGI174673, SGI0, SGI174674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174675, SGI174676, SGI174676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38684[] = {SGI174677, SGI0, SGI174678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174679, SGI174680, SGI174680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38685[] = {SGI174681, SGI0, SGI174682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174684, SGI174686, SGI174686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38686[] = {SGI174683, SGI0, SGI174685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174687, SGI174688, SGI174688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38687[] = {SGI174689, SGI0, SGI174690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174691, SGI174692, SGI174692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38688[] = {SGI174693, SGI0, SGI174694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174695, SGI174696, SGI174696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38689[] = {SGI174697, SGI0, SGI174698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174699, SGI174700, SGI174700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38690[] = {SGI174701, SGI0, SGI174702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174703, SGI174704, SGI174704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38691[] = {SGI174705, SGI0, SGI174706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174707, SGI174708, SGI174708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38692[] = {SGI174709, SGI0, SGI174710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174711, SGI174712, SGI174712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38693[] = {SGI174713, SGI174714, SGI174715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174716, SGI0, SGI0, SGI0, SGI174717, SGI174717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38694[] = {SGI174718, SGI0, SGI174719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38695[] = {SGI174720, SGI174721, SGI174722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174726, SGI0, SGI0, SGI0, SGI174727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38696[] = {SGI174723, SGI174724, SGI174725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174728, SGI0, SGI0, SGI0, SGI174729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38697[] = {SGI174731, SGI174732, SGI174733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174734, SGI0, SGI0, SGI0, SGI174735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38698[] = {SGI174736, SGI174738, SGI174739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174740, SGI0, SGI0, SGI0, SGI174741, SGI174741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38699[] = {SGI174742, SGI174743, SGI174744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174745, SGI0, SGI0, SGI0, SGI174747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38700[] = {SGI174746, SGI0, SGI174748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38701[] = {SGI174750, SGI174751, SGI174752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174753, SGI0, SGI0, SGI0, SGI174754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38702[] = {SGI174756, SGI174757, SGI174758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174759, SGI0, SGI0, SGI0, SGI174760, SGI174760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38703[] = {SGI174761, SGI174762, SGI174763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174764, SGI0, SGI0, SGI0, SGI174765, SGI174765, SGI0, SGI174766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38704[] = {SGI174767, SGI0, SGI174768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174769, SGI174770, SGI174770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38705[] = {SGI174771, SGI174772, SGI174773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174774, SGI0, SGI0, SGI0, SGI174775, SGI174775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38706[] = {SGI174776, SGI0, SGI174777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174778, SGI174779, SGI174779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38707[] = {SGI174780, SGI0, SGI174781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174782, SGI174783, SGI174783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38708[] = {SGI174784, SGI0, SGI174785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174786, SGI174787, SGI174787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38709[] = {SGI174788, SGI174789, SGI174790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174791, SGI0, SGI0, SGI0, SGI174792, SGI174792, SGI0, SGI174793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38710[] = {SGI174794, SGI174795, SGI174796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174797, SGI0, SGI0, SGI0, SGI174798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38711[] = {SGI174799, SGI174800, SGI174801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174802, SGI0, SGI0, SGI0, SGI174803, SGI174803, SGI0, SGI174804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38712[] = {SGI174805, SGI0, SGI174806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38713[] = {SGI174807, SGI0, SGI174808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174809, SGI174810, SGI174810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38714[] = {SGI174811, SGI0, SGI174812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174813, SGI174814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38715[] = {SGI174815, SGI0, SGI174816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174817, SGI174818, SGI174818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38716[] = {SGI174819, SGI0, SGI174820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174821, SGI174822, SGI174822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38717[] = {SGI174823, SGI0, SGI174824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174825, SGI174826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38718[] = {SGI174827, SGI0, SGI174828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174829, SGI174830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38719[] = {SGI174831, SGI174832, SGI174833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174834, SGI0, SGI0, SGI0, SGI174835, SGI174835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38720[] = {SGI174836, SGI0, SGI174837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38721[] = {SGI174838, SGI0, SGI174839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174840, SGI174841, SGI174841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38722[] = {SGI174842, SGI0, SGI174843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174844, SGI174845, SGI174845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38723[] = {SGI174846, SGI0, SGI174847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174848, SGI174849, SGI174849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38724[] = {SGI174850, SGI0, SGI174851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174852, SGI174853, SGI174853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38725[] = {SGI174854, SGI0, SGI174855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174856, SGI174857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38726[] = {SGI174858, SGI0, SGI174859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174860, SGI174861, SGI174861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38727[] = {SGI174862, SGI0, SGI174863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174866, SGI174867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38728[] = {SGI174864, SGI0, SGI174865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174868, SGI174869, SGI174869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38729[] = {SGI174870, SGI0, SGI174872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174874, SGI174876, SGI174876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38730[] = {SGI174871, SGI0, SGI174873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174875, SGI174877, SGI174877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38731[] = {SGI174878, SGI0, SGI174879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174881, SGI174883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38732[] = {SGI174880, SGI174882, SGI174884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174885, SGI0, SGI0, SGI0, SGI174886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38733[] = {SGI174887, SGI0, SGI174888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174890, SGI174891, SGI174891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38734[] = {SGI174889, SGI0, SGI174892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174893, SGI174894, SGI174894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38735[] = {SGI174895, SGI0, SGI174896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174897, SGI174898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38736[] = {SGI174899, SGI0, SGI174900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174901, SGI174902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38737[] = {SGI174903, SGI0, SGI174904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174905, SGI174906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38738[] = {SGI174907, SGI0, SGI174908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174909, SGI174910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38739[] = {SGI174911, SGI0, SGI174913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174915, SGI174916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38740[] = {SGI174912, SGI0, SGI174914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174917, SGI174918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38741[] = {SGI174919, SGI0, SGI174920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174923, SGI174924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38742[] = {SGI174921, SGI0, SGI174922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174925, SGI174926, SGI174926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38743[] = {SGI174927, SGI0, SGI174928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174929, SGI174930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38744[] = {SGI174931, SGI0, SGI174932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174935, SGI174936, SGI174936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38745[] = {SGI174933, SGI0, SGI174934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174937, SGI174938, SGI174938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38746[] = {SGI174939, SGI0, SGI174940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174941, SGI174943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38747[] = {SGI174942, SGI0, SGI174944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174945, SGI174946, SGI174946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38748[] = {SGI174947, SGI0, SGI174948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174949, SGI174950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38749[] = {SGI174951, SGI0, SGI174952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174955, SGI174956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38750[] = {SGI174953, SGI0, SGI174954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174957, SGI174958, SGI174958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38751[] = {SGI174959, SGI0, SGI174960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174962, SGI174964, SGI174964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38752[] = {SGI174961, SGI0, SGI174963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174965, SGI174966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38753[] = {SGI174967, SGI0, SGI174968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174969, SGI174970, SGI174970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38754[] = {SGI174971, SGI0, SGI174972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174973, SGI174974, SGI174974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38755[] = {SGI174975, SGI0, SGI174976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174979, SGI174980, SGI174980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38756[] = {SGI174977, SGI0, SGI174978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174981, SGI174982, SGI174982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38757[] = {SGI174983, SGI0, SGI174984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174985, SGI0, SGI0, SGI0, SGI174986, SGI174986, SGI0, SGI0, SGI0, SGI174988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38758[] = {SGI174987, SGI0, SGI174989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174990, SGI174991, SGI174991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38759[] = {SGI174992, SGI0, SGI174993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174996, SGI174997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38760[] = {SGI174994, SGI0, SGI174995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI174998, SGI0, SGI0, SGI0, SGI174999, SGI174999, SGI0, SGI0, SGI0, SGI175000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38761[] = {SGI175001, SGI0, SGI175002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175003, SGI175005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38762[] = {SGI175004, SGI175006, SGI175007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175008, SGI0, SGI0, SGI0, SGI175009, SGI175010, SGI0, SGI0, SGI0, SGI175011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38763[] = {SGI175012, SGI0, SGI175013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175014, SGI175015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38764[] = {SGI175016, SGI175017, SGI175018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175019, SGI0, SGI0, SGI0, SGI175020, SGI175021, SGI0, SGI0, SGI0, SGI175022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38765[] = {SGI175023, SGI0, SGI175024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175025, SGI175026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38766[] = {SGI175027, SGI0, SGI175028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175029, SGI175030, SGI175030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38767[] = {SGI175031, SGI0, SGI175032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175033, SGI175034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38768[] = {SGI175035, SGI0, SGI175036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175037, SGI175038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38769[] = {SGI175039, SGI0, SGI175040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175041, SGI175042, SGI175042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38770[] = {SGI175043, SGI0, SGI175044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175045, SGI175046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38771[] = {SGI175047, SGI0, SGI175048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175049, SGI175050, SGI175050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38772[] = {SGI175051, SGI175052, SGI175053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175054, SGI0, SGI0, SGI0, SGI175055, SGI175055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38773[] = {SGI175056, SGI0, SGI175057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38774[] = {SGI175058, SGI175059, SGI175060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175061, SGI0, SGI0, SGI0, SGI175062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38775[] = {SGI175063, SGI0, SGI175064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175065, SGI175066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38776[] = {SGI175067, SGI175068, SGI175069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175070, SGI0, SGI0, SGI0, SGI175071, SGI175071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38777[] = {SGI175072, SGI0, SGI175073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175074, SGI175075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38778[] = {SGI175076, SGI175077, SGI175078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175079, SGI0, SGI0, SGI0, SGI175080, SGI175080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38779[] = {SGI175081, SGI175082, SGI175083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175085, SGI0, SGI0, SGI0, SGI175088, SGI175088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38780[] = {SGI175084, SGI175086, SGI175087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175089, SGI0, SGI0, SGI0, SGI175090, SGI175090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38781[] = {SGI175091, SGI0, SGI175092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38782[] = {SGI175093, SGI175094, SGI175095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175096, SGI0, SGI0, SGI0, SGI175097, SGI175097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38783[] = {SGI175098, SGI0, SGI175099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38784[] = {SGI175100, SGI175101, SGI175102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175103, SGI0, SGI0, SGI0, SGI175104, SGI175104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38785[] = {SGI175105, SGI175106, SGI175107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175108, SGI0, SGI0, SGI0, SGI175109, SGI175109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38786[] = {SGI175110, SGI0, SGI175111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38787[] = {SGI175112, SGI0, SGI175113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38788[] = {SGI175114, SGI175115, SGI175116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175117, SGI0, SGI0, SGI0, SGI175118, SGI175118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38789[] = {SGI155886, SGI155887, SGI155888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI155889, SGI0, SGI0, SGI0, SGI155890, SGI155890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38790[] = {SGI175119, SGI175120, SGI175121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175122, SGI0, SGI0, SGI0, SGI175123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38791[] = {SGI175124, SGI175125, SGI175126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175127, SGI0, SGI0, SGI0, SGI175128, SGI175128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38792[] = {SGI175129, SGI175130, SGI175131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175132, SGI0, SGI0, SGI0, SGI175133, SGI175133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38793[] = {SGI175134, SGI0, SGI175135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38794[] = {SGI175136, SGI0, SGI175137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175138, SGI175139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38795[] = {SGI175140, SGI0, SGI175141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175145, SGI175146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38796[] = {SGI175147, SGI0, SGI175148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175149, SGI175150, SGI175150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38797[] = {SGI175151, SGI0, SGI175152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175154, SGI175155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38798[] = {SGI175153, SGI0, SGI175156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175157, SGI175158, SGI175158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38799[] = {SGI175160, SGI0, SGI175162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175164, SGI175166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38800[] = {SGI175159, SGI0, SGI175161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175163, SGI175165, SGI175165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38801[] = {SGI175167, SGI175168, SGI175169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175170, SGI0, SGI0, SGI0, SGI175171, SGI175171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38802[] = {SGI175172, SGI0, SGI175173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175174, SGI175175, SGI175175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38803[] = {SGI175176, SGI175177, SGI175178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175179, SGI0, SGI0, SGI0, SGI175180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38804[] = {SGI175181, SGI0, SGI175182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38805[] = {SGI175183, SGI175184, SGI175185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175186, SGI0, SGI0, SGI0, SGI175187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38806[] = {SGI175188, SGI0, SGI175189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175190, SGI175191, SGI175191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38807[] = {SGI175192, SGI0, SGI175193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175194, SGI175195, SGI175195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38808[] = {SGI175196, SGI0, SGI175197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175198, SGI175199, SGI175199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38809[] = {SGI175200, SGI0, SGI175201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175202, SGI175203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38810[] = {SGI175204, SGI0, SGI175205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175206, SGI175207, SGI175207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38811[] = {SGI175208, SGI175209, SGI175210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175211, SGI0, SGI0, SGI0, SGI175212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38812[] = {SGI175214, SGI175215, SGI175216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175217, SGI0, SGI0, SGI0, SGI175218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38813[] = {SGI175220, SGI175221, SGI175222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175223, SGI0, SGI0, SGI0, SGI175224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38814[] = {SGI175226, SGI175227, SGI175228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175229, SGI0, SGI0, SGI0, SGI175230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38815[] = {SGI175232, SGI175233, SGI175234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175235, SGI0, SGI0, SGI0, SGI175236, SGI175236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38816[] = {SGI175238, SGI0, SGI175239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175240, SGI175241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38817[] = {SGI175242, SGI0, SGI175243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38818[] = {SGI175244, SGI175245, SGI175246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175247, SGI0, SGI0, SGI0, SGI175248, SGI175248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38819[] = {SGI175249, SGI0, SGI175250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175251, SGI0, SGI0, SGI0, SGI175252, SGI175252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38820[] = {SGI175253, SGI175254, SGI175255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175257, SGI0, SGI0, SGI0, SGI175260, SGI175260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38821[] = {SGI175256, SGI175258, SGI175259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175261, SGI0, SGI0, SGI0, SGI175262, SGI175262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38822[] = {SGI175263, SGI175264, SGI175265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175266, SGI0, SGI0, SGI0, SGI175267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38823[] = {SGI175268, SGI175269, SGI175270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175274, SGI0, SGI0, SGI0, SGI175275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38824[] = {SGI175271, SGI175272, SGI175273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175276, SGI0, SGI0, SGI0, SGI175277, SGI175277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38825[] = {SGI175142, SGI175143, SGI175144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175278, SGI0, SGI0, SGI0, SGI175279, SGI175279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38826[] = {SGI175280, SGI0, SGI175281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38827[] = {SGI175282, SGI175283, SGI175284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175285, SGI175285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38828[] = {SGI175286, SGI0, SGI175287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175288, SGI175289, SGI175289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38829[] = {SGI175290, SGI0, SGI175291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175292, SGI175293, SGI175293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38830[] = {SGI175294, SGI0, SGI175295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175296, SGI175297, SGI175297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38831[] = {SGI175298, SGI175299, SGI175300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175301, SGI0, SGI0, SGI0, SGI175302, SGI175302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38832[] = {SGI175303, SGI0, SGI175304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38833[] = {SGI175305, SGI175306, SGI175307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175308, SGI0, SGI0, SGI0, SGI175309, SGI175309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38834[] = {SGI175310, SGI0, SGI175311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175312, SGI175313, SGI175313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38835[] = {SGI175314, SGI0, SGI175315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175316, SGI175317, SGI175317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38836[] = {SGI175318, SGI0, SGI175319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175322, SGI175323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38837[] = {SGI175320, SGI0, SGI175321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175324, SGI175325, SGI175325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38838[] = {SGI175326, SGI0, SGI175327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175328, SGI175329, SGI175329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38839[] = {SGI175330, SGI175331, SGI175332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175333, SGI0, SGI0, SGI0, SGI175334, SGI175334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38840[] = {SGI175336, SGI175337, SGI175338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175339, SGI0, SGI0, SGI0, SGI175340, SGI175340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38841[] = {SGI175342, SGI175343, SGI175344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175345, SGI0, SGI0, SGI0, SGI175346, SGI175346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38842[] = {SGI175347, SGI175348, SGI175349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175350, SGI0, SGI0, SGI0, SGI175351, SGI175351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38843[] = {SGI175352, SGI175353, SGI175354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175355, SGI0, SGI0, SGI0, SGI175356, SGI175356, SGI0, SGI175357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38844[] = {SGI175358, SGI175359, SGI175360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175361, SGI0, SGI0, SGI0, SGI175362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38845[] = {SGI175363, SGI175364, SGI175365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175366, SGI0, SGI0, SGI0, SGI175367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38846[] = {SGI175368, SGI175369, SGI175370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175371, SGI0, SGI0, SGI0, SGI175372, SGI175372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38847[] = {SGI175373, SGI175374, SGI175375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175379, SGI175379, SGI0, SGI0, SGI175381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38848[] = {SGI175376, SGI175377, SGI175378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175380, SGI0, SGI0, SGI0, SGI175382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38849[] = {SGI175383, SGI175384, SGI175385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175386, SGI175386, SGI0, SGI0, SGI175387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38850[] = {SGI175388, SGI175389, SGI175390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175391, SGI175391, SGI0, SGI0, SGI175392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38851[] = {SGI175393, SGI0, SGI175394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175395, SGI175396, SGI175396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38852[] = {SGI175397, SGI0, SGI175398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175399, SGI175400, SGI175400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38853[] = {SGI175401, SGI175402, SGI175403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175404, SGI0, SGI0, SGI0, SGI175405, SGI175405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38854[] = {SGI175406, SGI0, SGI175407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175408, SGI175409, SGI175409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38855[] = {SGI175410, SGI0, SGI175411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175412, SGI175413, SGI175413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38856[] = {SGI175414, SGI0, SGI175415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175416, SGI175417, SGI175417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38857[] = {SGI175418, SGI0, SGI175419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175420, SGI175421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38858[] = {SGI175422, SGI175423, SGI175424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175425, SGI175425, SGI0, SGI0, SGI0, SGI175426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38859[] = {SGI175427, SGI175428, SGI175429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175430, SGI175430, SGI0, SGI0, SGI0, SGI175431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38860[] = {SGI175432, SGI0, SGI175433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175434, SGI175435, SGI175435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38861[] = {SGI175436, SGI175437, SGI175438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175440, SGI175440, SGI0, SGI0, SGI0, SGI175442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38862[] = {SGI175439, SGI0, SGI175441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175443, SGI175444, SGI175444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38863[] = {SGI175445, SGI175446, SGI175447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175448, SGI175448, SGI0, SGI0, SGI0, SGI175449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38864[] = {SGI175450, SGI0, SGI175451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175452, SGI175453, SGI175453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38865[] = {SGI175454, SGI175455, SGI175456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175457, SGI0, SGI0, SGI0, SGI175458, SGI175458, SGI0, SGI175459, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38866[] = {SGI175460, SGI0, SGI175461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175462, SGI175463, SGI175463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38867[] = {SGI175464, SGI0, SGI175465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175466, SGI175467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38868[] = {SGI175468, SGI0, SGI175469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175470, SGI175471, SGI175471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38869[] = {SGI175472, SGI175473, SGI175474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175475, SGI0, SGI0, SGI0, SGI175476, SGI175476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38870[] = {SGI175477, SGI175478, SGI175479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175480, SGI0, SGI0, SGI0, SGI175481, SGI175481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38871[] = {SGI175482, SGI0, SGI175483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38872[] = {SGI175484, SGI0, SGI175485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38873[] = {SGI175486, SGI0, SGI175487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175488, SGI175489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38874[] = {SGI175490, SGI0, SGI175491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175492, SGI175493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38875[] = {SGI175494, SGI0, SGI175495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175496, SGI175497, SGI175497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38876[] = {SGI175498, SGI0, SGI175499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175500, SGI175501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38877[] = {SGI175502, SGI0, SGI175504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175506, SGI175507, SGI175507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38878[] = {SGI175503, SGI0, SGI175505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175508, SGI175509, SGI175509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38879[] = {SGI175510, SGI175511, SGI175512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175513, SGI0, SGI0, SGI0, SGI175514, SGI175514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38880[] = {SGI175515, SGI0, SGI175516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175518, SGI175520, SGI175520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38881[] = {SGI175517, SGI0, SGI175519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175521, SGI175522, SGI175522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38882[] = {SGI175523, SGI0, SGI175524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38883[] = {SGI175525, SGI0, SGI175526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175527, SGI175528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38884[] = {SGI175530, SGI0, SGI175532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175534, SGI175536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38885[] = {SGI175529, SGI0, SGI175531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175533, SGI175535, SGI175535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38886[] = {SGI175537, SGI0, SGI175538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175539, SGI175540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38887[] = {SGI175541, SGI0, SGI175542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175543, SGI175544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38888[] = {SGI175545, SGI0, SGI175546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175547, SGI175548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38889[] = {SGI175549, SGI0, SGI175550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175553, SGI175555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38890[] = {SGI175551, SGI175552, SGI175554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175556, SGI0, SGI0, SGI0, SGI175557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38891[] = {SGI175558, SGI175559, SGI175560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175561, SGI0, SGI0, SGI0, SGI175562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38892[] = {SGI175563, SGI175564, SGI175565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175566, SGI0, SGI0, SGI0, SGI175567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38893[] = {SGI175568, SGI175569, SGI175571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175573, SGI0, SGI0, SGI0, SGI175574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38894[] = {SGI175570, SGI0, SGI175572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175575, SGI175576, SGI175576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38895[] = {SGI175577, SGI0, SGI175578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175579, SGI175581, SGI175581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38896[] = {SGI175580, SGI175582, SGI175583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175584, SGI0, SGI0, SGI0, SGI175585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38897[] = {SGI175586, SGI0, SGI175587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175588, SGI175589, SGI175589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38898[] = {SGI175590, SGI175591, SGI175592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175595, SGI0, SGI0, SGI0, SGI175596, SGI175596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38899[] = {SGI175593, SGI0, SGI175594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175597, SGI175598, SGI175598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38900[] = {SGI175599, SGI0, SGI175600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175601, SGI175602, SGI175602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38901[] = {SGI175603, SGI0, SGI175604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175605, SGI175606, SGI175606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38902[] = {SGI175607, SGI175608, SGI175609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175610, SGI0, SGI0, SGI0, SGI175611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38903[] = {SGI175612, SGI175613, SGI175614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175615, SGI0, SGI0, SGI0, SGI175616, SGI175616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38904[] = {SGI175617, SGI0, SGI175618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175619, SGI175620, SGI175620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38905[] = {SGI175621, SGI0, SGI175622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175623, SGI175624, SGI175624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38906[] = {SGI175625, SGI0, SGI175626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175627, SGI175628, SGI175628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38907[] = {SGI175629, SGI0, SGI175630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175631, SGI175632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38908[] = {SGI175633, SGI0, SGI175634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175635, SGI175636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38909[] = {SGI175637, SGI0, SGI175638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175639, SGI175640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38910[] = {SGI175641, SGI0, SGI175642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175643, SGI175644, SGI175644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38911[] = {SGI175645, SGI0, SGI175646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175647, SGI175648, SGI175648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38912[] = {SGI175649, SGI0, SGI175650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175651, SGI175652, SGI175652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38913[] = {SGI175653, SGI0, SGI175654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175656, SGI175657, SGI175657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38914[] = {SGI175655, SGI0, SGI175658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175659, SGI175660, SGI175660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38915[] = {SGI175661, SGI0, SGI175662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175663, SGI175664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38916[] = {SGI175665, SGI0, SGI175666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175667, SGI175668, SGI175668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38917[] = {SGI175669, SGI175670, SGI175671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175672, SGI0, SGI0, SGI0, SGI175673, SGI175673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38918[] = {SGI175674, SGI175675, SGI175676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175677, SGI0, SGI0, SGI0, SGI175678, SGI175678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38919[] = {SGI175679, SGI0, SGI175680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175681, SGI175682, SGI175682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38920[] = {SGI175683, SGI0, SGI175684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175685, SGI175686, SGI175686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38921[] = {SGI175687, SGI0, SGI175688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175689, SGI175690, SGI175690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175691, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38922[] = {SGI175692, SGI175693, SGI175694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175695, SGI0, SGI0, SGI0, SGI175696, SGI175696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38923[] = {SGI175697, SGI175698, SGI175699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175700, SGI0, SGI0, SGI0, SGI175703, SGI175703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38924[] = {SGI175701, SGI175702, SGI175704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175705, SGI0, SGI0, SGI0, SGI175706, SGI175706, SGI0, SGI175707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38925[] = {SGI175708, SGI175709, SGI175710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175711, SGI0, SGI0, SGI0, SGI175712, SGI175712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38926[] = {SGI175713, SGI175714, SGI175715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175716, SGI0, SGI0, SGI0, SGI175717, SGI175717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38927[] = {SGI175718, SGI0, SGI175719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38928[] = {SGI175720, SGI175721, SGI175722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175725, SGI0, SGI0, SGI0, SGI175727, SGI175727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38929[] = {SGI175723, SGI175724, SGI175726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175728, SGI0, SGI0, SGI0, SGI175729, SGI175729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38930[] = {SGI175731, SGI0, SGI175732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38931[] = {SGI175733, SGI175734, SGI175735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175736, SGI0, SGI0, SGI0, SGI175737, SGI175737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38932[] = {SGI175739, SGI0, SGI175740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175741, SGI0, SGI0, SGI0, SGI175742, SGI175742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38933[] = {SGI175743, SGI175744, SGI175745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175746, SGI0, SGI0, SGI0, SGI175747, SGI175747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38934[] = {SGI175749, SGI0, SGI175750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175751, SGI175752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175753, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38935[] = {SGI175754, SGI0, SGI175755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175756, SGI175757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175758, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38936[] = {SGI175759, SGI0, SGI175760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175761, SGI175762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175763, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38937[] = {SGI175764, SGI175765, SGI175766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175767, SGI0, SGI0, SGI0, SGI175768, SGI175768, SGI0, SGI175769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38938[] = {SGI175770, SGI0, SGI175771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175772, SGI175773, SGI175773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38939[] = {SGI175774, SGI0, SGI175775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175776, SGI175777, SGI175777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38940[] = {SGI175778, SGI175779, SGI175780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175781, SGI0, SGI0, SGI0, SGI175782, SGI175782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38941[] = {SGI175783, SGI175784, SGI175785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175786, SGI0, SGI0, SGI0, SGI175787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38942[] = {SGI175788, SGI175789, SGI175790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175791, SGI0, SGI0, SGI0, SGI175792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38943[] = {SGI175793, SGI0, SGI175794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175795, SGI175796, SGI175796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38944[] = {SGI175797, SGI0, SGI175798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175799, SGI175800, SGI175800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38945[] = {SGI175801, SGI175802, SGI175804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175806, SGI0, SGI0, SGI0, SGI175807, SGI175807, SGI0, SGI0, NULL};
const Segment * const SG38946[] = {SGI175803, SGI0, SGI175805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175808, SGI175809, SGI175809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38947[] = {SGI175810, SGI175811, SGI175812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175813, SGI0, SGI0, SGI0, SGI175814, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38948[] = {SGI175815, SGI0, SGI175816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175820, SGI175821, SGI175821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38949[] = {SGI175817, SGI175818, SGI175819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175822, SGI0, SGI0, SGI0, SGI175823, SGI175823, SGI0, SGI0, NULL};
const Segment * const SG38950[] = {SGI175824, SGI175825, SGI175826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175827, SGI0, SGI0, SGI0, SGI175828, SGI175828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38951[] = {SGI175829, SGI0, SGI175830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175831, SGI175832, SGI175832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38952[] = {SGI175834, SGI0, SGI175836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175837, SGI175839, SGI175839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38953[] = {SGI175833, SGI0, SGI175835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175838, SGI175840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38954[] = {SGI175841, SGI0, SGI175842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175843, SGI175844, SGI175844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38955[] = {SGI175845, SGI0, SGI175846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175850, SGI175851, SGI175851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38956[] = {SGI175847, SGI175848, SGI175849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175852, SGI0, SGI0, SGI0, SGI175853, SGI175853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38957[] = {SGI175854, SGI175855, SGI175856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175857, SGI175858, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38958[] = {SGI175859, SGI0, SGI175860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175861, SGI175862, SGI175862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38959[] = {SGI175863, SGI175864, SGI175865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175866, SGI0, SGI0, SGI0, SGI175867, SGI175867, SGI0, SGI175868, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38960[] = {SGI175869, SGI175870, SGI175871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175872, SGI0, SGI0, SGI0, SGI175873, SGI175873, SGI0, SGI175874, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38961[] = {SGI175875, SGI0, SGI175876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175877, SGI175878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38962[] = {SGI175879, SGI175880, SGI175881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175882, SGI0, SGI0, SGI0, SGI175883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38963[] = {SGI175884, SGI0, SGI175885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38964[] = {SGI175886, SGI0, SGI175887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175888, SGI175889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38965[] = {SGI175890, SGI0, SGI175891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175892, SGI175893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38966[] = {SGI175894, SGI0, SGI175895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175896, SGI175897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38967[] = {SGI175898, SGI0, SGI175899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175900, SGI175901, SGI175901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38968[] = {SGI175902, SGI0, SGI175903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175904, SGI175905, SGI175905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38969[] = {SGI175906, SGI0, SGI175907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175908, SGI175909, SGI175909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38970[] = {SGI175910, SGI0, SGI175911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175912, SGI175913, SGI175913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38971[] = {SGI175914, SGI0, SGI175915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175916, SGI175917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38972[] = {SGI175918, SGI175919, SGI175920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175921, SGI0, SGI0, SGI0, SGI175922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38973[] = {SGI175923, SGI175924, SGI175925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175926, SGI0, SGI0, SGI0, SGI175927, SGI175927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38974[] = {SGI175928, SGI175929, SGI175930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175931, SGI0, SGI0, SGI0, SGI175932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38975[] = {SGI175933, SGI0, SGI175934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175935, SGI175937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38976[] = {SGI175936, SGI175938, SGI175939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175940, SGI0, SGI0, SGI0, SGI175941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38977[] = {SGI175942, SGI0, SGI175943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175944, SGI175945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38978[] = {SGI175946, SGI175947, SGI175948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175949, SGI0, SGI0, SGI0, SGI175950, SGI175950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38979[] = {SGI175952, SGI0, SGI175953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175958, SGI175959, SGI175959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38980[] = {SGI175954, SGI175955, SGI175956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175961, SGI0, SGI0, SGI0, SGI175962, SGI175962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38981[] = {SGI175957, SGI0, SGI175960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175963, SGI175965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38982[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175966, SGI0, SGI0, SGI175967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38983[] = {SGI175968, SGI175969, SGI175970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175971, SGI0, SGI0, SGI0, SGI175972, SGI175972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38984[] = {SGI175974, SGI0, SGI175975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175976, SGI175977, SGI175977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38985[] = {SGI175978, SGI175979, SGI175980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175981, SGI0, SGI0, SGI0, SGI175982, SGI175982, SGI175983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38986[] = {SGI175984, SGI0, SGI175985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175986, SGI175987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38987[] = {SGI175988, SGI175989, SGI175991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175994, SGI0, SGI0, SGI0, SGI175995, SGI175995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38988[] = {SGI175990, SGI175992, SGI175993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI175996, SGI0, SGI0, SGI0, SGI175997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38989[] = {SGI175998, SGI0, SGI175999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38990[] = {SGI176000, SGI176001, SGI176002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176003, SGI0, SGI0, SGI0, SGI176004, SGI176004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38991[] = {SGI176005, SGI176006, SGI176007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176008, SGI0, SGI0, SGI0, SGI176009, SGI176009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38992[] = {SGI176010, SGI176011, SGI176012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176013, SGI0, SGI0, SGI0, SGI176014, SGI176014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38993[] = {SGI176015, SGI0, SGI176016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176017, SGI0, SGI0, SGI0, SGI176018, SGI176018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38994[] = {SGI176019, SGI176020, SGI176021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176022, SGI176022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38995[] = {SGI176023, SGI176024, SGI176025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176026, SGI0, SGI0, SGI0, SGI176027, SGI176027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38996[] = {SGI176028, SGI176029, SGI176030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176031, SGI0, SGI0, SGI0, SGI176032, SGI176032, SGI0, SGI176033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38997[] = {SGI176034, SGI176035, SGI176036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176037, SGI0, SGI0, SGI0, SGI176038, SGI0, SGI0, SGI176039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38998[] = {SGI176040, SGI176041, SGI176042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176043, SGI0, SGI0, SGI0, SGI176044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG38999[] = {SGI176045, SGI176046, SGI176047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176048, SGI0, SGI0, SGI0, SGI176049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39000[] = {SGI176050, SGI176051, SGI176052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176053, SGI0, SGI0, SGI0, SGI176054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39001[] = {SGI176055, SGI176057, SGI176059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176060, SGI0, SGI0, SGI0, SGI176062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39002[] = {SGI176056, SGI0, SGI176058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176061, SGI176063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39003[] = {SGI176064, SGI176065, SGI176066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176067, SGI0, SGI0, SGI0, SGI176068, SGI176068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39004[] = {SGI176069, SGI0, SGI176070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39005[] = {SGI176071, SGI176072, SGI176073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176074, SGI0, SGI0, SGI0, SGI176075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39006[] = {SGI176076, SGI0, SGI176077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176078, SGI176079, SGI176079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39007[] = {SGI176080, SGI0, SGI176081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176082, SGI176083, SGI176083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39008[] = {SGI176084, SGI0, SGI176085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176086, SGI176087, SGI176087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39009[] = {SGI176088, SGI176089, SGI176090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176091, SGI0, SGI0, SGI0, SGI176092, SGI176092, SGI0, SGI176093, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39010[] = {SGI176094, SGI0, SGI176095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176096, SGI176097, SGI176097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39011[] = {SGI176098, SGI176099, SGI176100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176101, SGI0, SGI0, SGI0, SGI176102, SGI176102, SGI0, SGI176103, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39012[] = {SGI176104, SGI176105, SGI176106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176107, SGI0, SGI0, SGI0, SGI176108, SGI176108, SGI0, SGI176109, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39013[] = {SGI176110, SGI176111, SGI176112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176113, SGI0, SGI0, SGI0, SGI176115, SGI176115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39014[] = {SGI176114, SGI176116, SGI176117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176118, SGI0, SGI0, SGI0, SGI176119, SGI176119, SGI0, SGI176120, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39015[] = {SGI176121, SGI0, SGI176122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39016[] = {SGI176123, SGI0, SGI176124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176125, SGI176126, SGI176127, SGI0, SGI0, NULL};
const Segment * const SG39017[] = {SGI176128, SGI0, SGI176129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176130, SGI176131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39018[] = {SGI176132, SGI0, SGI176133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176134, SGI176135, SGI176135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39019[] = {SGI176136, SGI176137, SGI176138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176139, SGI0, SGI0, SGI0, SGI176140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39020[] = {SGI176141, SGI0, SGI176142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39021[] = {SGI176143, SGI176144, SGI176145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176146, SGI176146, SGI0, SGI0, SGI0, SGI0, SGI176147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39022[] = {SGI176148, SGI176149, SGI176150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176151, SGI176151, SGI0, SGI0, SGI0, SGI0, SGI176152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39023[] = {SGI176153, SGI176154, SGI176155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176156, SGI176156, SGI0, SGI0, SGI0, SGI0, SGI176157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39024[] = {SGI176158, SGI0, SGI176159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176160, SGI176161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39025[] = {SGI176162, SGI176163, SGI176164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176165, SGI0, SGI0, SGI0, SGI176166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39026[] = {SGI176167, SGI0, SGI176168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176169, SGI176170, SGI176170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39027[] = {SGI176171, SGI0, SGI176172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176173, SGI176174, SGI176174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39028[] = {SGI176175, SGI0, SGI176176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176177, SGI176178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39029[] = {SGI176179, SGI176180, SGI176181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176182, SGI0, SGI0, SGI0, SGI176183, SGI176183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39030[] = {SGI176185, SGI176186, SGI176187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176188, SGI0, SGI0, SGI0, SGI176189, SGI176189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39031[] = {SGI176191, SGI176192, SGI176193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176194, SGI0, SGI0, SGI0, SGI176195, SGI176195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39032[] = {SGI176197, SGI176198, SGI176199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176200, SGI0, SGI0, SGI0, SGI176201, SGI176201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39033[] = {SGI176203, SGI176204, SGI176205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176206, SGI0, SGI0, SGI0, SGI176207, SGI176207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39034[] = {SGI176209, SGI176210, SGI176211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176212, SGI0, SGI0, SGI0, SGI176213, SGI176213, SGI0, SGI176214, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39035[] = {SGI176215, SGI176216, SGI176217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176218, SGI0, SGI0, SGI0, SGI176219, SGI176219, SGI0, SGI176220, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39036[] = {SGI176221, SGI176222, SGI176223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176224, SGI0, SGI0, SGI0, SGI176225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39037[] = {SGI176226, SGI0, SGI176227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39038[] = {SGI176228, SGI0, SGI176229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176230, SGI176231, SGI176231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39039[] = {SGI176232, SGI0, SGI176233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176235, SGI176236, SGI176236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39040[] = {SGI176234, SGI0, SGI176237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176238, SGI176239, SGI176239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39041[] = {SGI176240, SGI0, SGI176241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176242, SGI176243, SGI176243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39042[] = {SGI176244, SGI0, SGI176245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176246, SGI176247, SGI176247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39043[] = {SGI176248, SGI0, SGI176249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176250, SGI176251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39044[] = {SGI176252, SGI0, SGI176253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176254, SGI176255, SGI176255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39045[] = {SGI176256, SGI0, SGI176257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176258, SGI176259, SGI176259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39046[] = {SGI176260, SGI0, SGI176261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176262, SGI176263, SGI176263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39047[] = {SGI176264, SGI0, SGI176265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176266, SGI176267, SGI176267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39048[] = {SGI176268, SGI176269, SGI176270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176271, SGI0, SGI0, SGI0, SGI176272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39049[] = {SGI176273, SGI0, SGI176274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176275, SGI176276, SGI176276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39050[] = {SGI176277, SGI176278, SGI176279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176280, SGI0, SGI0, SGI0, SGI176281, SGI176281, SGI0, SGI0, SGI176282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39051[] = {SGI176283, SGI0, SGI176284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176286, SGI176289, SGI176289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39052[] = {SGI176285, SGI176287, SGI176288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176290, SGI0, SGI0, SGI0, SGI176291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39053[] = {SGI176293, SGI0, SGI176294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39054[] = {SGI176292, SGI0, SGI176295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176296, SGI176297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39055[] = {SGI176298, SGI0, SGI176299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39056[] = {SGI176300, SGI176301, SGI176302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176303, SGI0, SGI0, SGI0, SGI176304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39057[] = {SGI176305, SGI0, SGI176306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176307, SGI176308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39058[] = {SGI176309, SGI0, SGI176310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176311, SGI176312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39059[] = {SGI176313, SGI0, SGI176314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176315, SGI176316, SGI176316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39060[] = {SGI176317, SGI0, SGI176318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176319, SGI176320, SGI176320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39061[] = {SGI176321, SGI176322, SGI176323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176324, SGI176324, SGI0, SGI0, SGI0, SGI0, SGI176325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39062[] = {SGI176326, SGI0, SGI176327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176328, SGI176329, SGI176329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39063[] = {SGI176330, SGI176331, SGI176332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176333, SGI176333, SGI0, SGI0, SGI0, SGI0, SGI176334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39064[] = {SGI176335, SGI176336, SGI176337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176338, SGI176338, SGI0, SGI0, SGI0, SGI0, SGI176339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39065[] = {SGI176340, SGI176341, SGI176342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176343, SGI176343, SGI0, SGI0, SGI0, SGI0, SGI176344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39066[] = {SGI176345, SGI176346, SGI176347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176348, SGI176348, SGI0, SGI0, SGI0, SGI0, SGI176349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39067[] = {SGI176350, SGI0, SGI176351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176352, SGI176353, SGI176353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39068[] = {SGI176354, SGI0, SGI176355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176356, SGI176357, SGI176357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39069[] = {SGI176358, SGI0, SGI176359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176360, SGI176361, SGI176361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39070[] = {SGI176362, SGI0, SGI176363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176364, SGI176365, SGI176365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39071[] = {SGI176366, SGI176367, SGI176368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176369, SGI0, SGI0, SGI0, SGI176370, SGI176370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39072[] = {SGI176371, SGI0, SGI176372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176373, SGI176374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39073[] = {SGI176375, SGI0, SGI176376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176377, SGI176378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39074[] = {SGI176379, SGI176380, SGI176381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176382, SGI0, SGI0, SGI0, SGI176383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39075[] = {SGI176384, SGI176385, SGI176386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176387, SGI0, SGI176388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39076[] = {SGI176389, SGI0, SGI176390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176391, SGI176392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39077[] = {SGI176393, SGI0, SGI176394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176395, SGI176396, SGI176396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39078[] = {SGI176397, SGI176398, SGI176399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176400, SGI0, SGI0, SGI0, SGI176401, SGI176401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39079[] = {SGI176402, SGI0, SGI176403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176404, SGI176405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39080[] = {SGI176406, SGI176407, SGI176408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176409, SGI0, SGI0, SGI0, SGI176410, SGI176410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39081[] = {SGI176411, SGI0, SGI176412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176413, SGI176414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39082[] = {SGI176415, SGI176416, SGI176417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176418, SGI0, SGI0, SGI0, SGI176419, SGI176419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39083[] = {SGI176420, SGI176421, SGI176423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176425, SGI0, SGI0, SGI0, SGI176426, SGI176426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39084[] = {SGI176422, SGI0, SGI176424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176427, SGI176428, SGI176428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176429, SGI0, SGI0, NULL};
const Segment * const SG39085[] = {SGI176430, SGI0, SGI176431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176432, SGI176433, SGI176433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176436, SGI0, SGI0, NULL};
const Segment * const SG39086[] = {SGI176434, SGI0, SGI176435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176437, SGI176438, SGI176438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39087[] = {SGI176439, SGI0, SGI176440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176441, SGI176443, SGI176443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176445, SGI0, SGI0, NULL};
const Segment * const SG39088[] = {SGI176442, SGI0, SGI176444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176446, SGI176447, SGI176447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39089[] = {SGI176448, SGI0, SGI176450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176452, SGI176453, SGI176453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39090[] = {SGI176449, SGI0, SGI176451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176454, SGI176455, SGI176455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176456, SGI0, SGI0, NULL};
const Segment * const SG39091[] = {SGI176457, SGI0, SGI176458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176459, SGI176460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39092[] = {SGI176461, SGI176462, SGI176463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176464, SGI0, SGI0, SGI0, SGI176465, SGI176465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39093[] = {SGI176466, SGI176467, SGI176468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176469, SGI0, SGI0, SGI0, SGI176470, SGI176470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39094[] = {SGI176471, SGI176472, SGI176473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176474, SGI0, SGI0, SGI0, SGI176475, SGI176475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39095[] = {SGI176476, SGI176477, SGI176478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176479, SGI0, SGI0, SGI0, SGI176480, SGI176480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39096[] = {SGI176481, SGI176482, SGI176483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176484, SGI0, SGI0, SGI0, SGI176485, SGI176485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39097[] = {SGI176486, SGI0, SGI176487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176488, SGI176489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39098[] = {SGI176490, SGI0, SGI176491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39099[] = {SGI176492, SGI176493, SGI176494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176495, SGI0, SGI0, SGI0, SGI176496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39100[] = {SGI176497, SGI176498, SGI176499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176500, SGI0, SGI0, SGI0, SGI176501, SGI176501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39101[] = {SGI176502, SGI176503, SGI176504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176505, SGI0, SGI0, SGI0, SGI176506, SGI176506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39102[] = {SGI176507, SGI176508, SGI176509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176510, SGI0, SGI0, SGI0, SGI176511, SGI176511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39103[] = {SGI176512, SGI176513, SGI176514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176515, SGI0, SGI0, SGI0, SGI176516, SGI176516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39104[] = {SGI176518, SGI176520, SGI176521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176523, SGI0, SGI0, SGI0, SGI176525, SGI176525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39105[] = {SGI176517, SGI0, SGI176519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176522, SGI176524, SGI176524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39106[] = {SGI176526, SGI0, SGI176527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39107[] = {SGI176528, SGI176529, SGI176530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176531, SGI0, SGI0, SGI0, SGI176532, SGI176532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39108[] = {SGI176533, SGI176534, SGI176535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176536, SGI0, SGI0, SGI0, SGI176537, SGI176537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39109[] = {SGI176538, SGI176539, SGI176540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176541, SGI0, SGI0, SGI0, SGI176542, SGI176542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39110[] = {SGI176543, SGI0, SGI176544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39111[] = {SGI176545, SGI176546, SGI176547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176548, SGI0, SGI0, SGI0, SGI176549, SGI176549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39112[] = {SGI176550, SGI176551, SGI176552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176553, SGI0, SGI0, SGI0, SGI176554, SGI176554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39113[] = {SGI176555, SGI176556, SGI176557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176558, SGI0, SGI0, SGI0, SGI176559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39114[] = {SGI176560, SGI176561, SGI176562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176566, SGI0, SGI0, SGI176568, SGI176569, SGI176569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39115[] = {SGI176563, SGI176564, SGI176565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176567, SGI0, SGI0, SGI0, SGI176570, SGI176570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39116[] = {SGI176571, SGI176572, SGI176573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176574, SGI176574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39117[] = {SGI176575, SGI0, SGI176576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176577, SGI176578, SGI176578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39118[] = {SGI176579, SGI176580, SGI176581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176582, SGI0, SGI0, SGI0, SGI176583, SGI176583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39119[] = {SGI176584, SGI0, SGI176585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176586, SGI176587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39120[] = {SGI176588, SGI0, SGI176589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39121[] = {SGI176590, SGI0, SGI176591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176592, SGI176593, SGI176593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39122[] = {SGI176594, SGI0, SGI176595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176596, SGI176597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39123[] = {SGI176598, SGI176599, SGI176600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176601, SGI0, SGI0, SGI0, SGI176602, SGI176602, SGI0, SGI176603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39124[] = {SGI176604, SGI176605, SGI176606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176607, SGI0, SGI0, SGI0, SGI176608, SGI176608, SGI0, SGI176609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39125[] = {SGI176610, SGI0, SGI176611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176612, SGI176613, SGI176613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39126[] = {SGI176614, SGI0, SGI176615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176616, SGI176617, SGI176617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39127[] = {SGI176618, SGI0, SGI176619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176620, SGI176621, SGI176621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39128[] = {SGI176622, SGI176623, SGI176624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176628, SGI0, SGI0, SGI0, SGI176629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39129[] = {SGI176625, SGI176626, SGI176627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176630, SGI0, SGI0, SGI0, SGI176631, SGI176631, SGI0, SGI176632, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39130[] = {SGI176633, SGI0, SGI176634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176637, SGI176639, SGI176639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39131[] = {SGI176635, SGI176636, SGI176638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176640, SGI0, SGI0, SGI0, SGI176641, SGI176641, SGI0, SGI176642, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39132[] = {SGI176643, SGI0, SGI176644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176645, SGI176646, SGI176646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39133[] = {SGI176647, SGI176648, SGI176649, SGI0, SGI0, SGI0, SGI176650, SGI176651, SGI0, SGI176653, SGI0, SGI0, SGI0, SGI176655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39134[] = {SGI176652, SGI0, SGI176654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176656, SGI176657, SGI176657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39135[] = {SGI176659, SGI0, SGI176661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176664, SGI176665, SGI176665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39136[] = {SGI176658, SGI176660, SGI176662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176663, SGI0, SGI0, SGI0, SGI176666, SGI176666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176667, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39137[] = {SGI176668, SGI0, SGI176669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176673, SGI176674, SGI176674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39138[] = {SGI176670, SGI176671, SGI176672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176675, SGI0, SGI0, SGI0, SGI176676, SGI176676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176677, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39139[] = {SGI176678, SGI176679, SGI176681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176686, SGI0, SGI0, SGI0, SGI176687, SGI176687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176691, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39140[] = {SGI176683, SGI0, SGI176685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176689, SGI176690, SGI176690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39141[] = {SGI176680, SGI176682, SGI176684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176688, SGI0, SGI0, SGI0, SGI176692, SGI0, SGI176693, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39142[] = {SGI176694, SGI176695, SGI176696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176697, SGI0, SGI0, SGI0, SGI176698, SGI176698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176699, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39143[] = {SGI176700, SGI0, SGI176701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176705, SGI176706, SGI176706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39144[] = {SGI176702, SGI176703, SGI176704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176707, SGI0, SGI0, SGI0, SGI176708, SGI176708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176709, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39145[] = {SGI176710, SGI176711, SGI176712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176713, SGI0, SGI0, SGI0, SGI176714, SGI176714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176715, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39146[] = {SGI176716, SGI176717, SGI176718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176719, SGI0, SGI0, SGI0, SGI176720, SGI176720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176721, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39147[] = {SGI176722, SGI176723, SGI176724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176725, SGI0, SGI0, SGI0, SGI176726, SGI176726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176727, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39148[] = {SGI176728, SGI176729, SGI176730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176731, SGI0, SGI0, SGI0, SGI176732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176733, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39149[] = {SGI176734, SGI0, SGI176735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176736, SGI176737, SGI176737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39150[] = {SGI176738, SGI0, SGI176739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176740, SGI176741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39151[] = {SGI176742, SGI176743, SGI176744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176745, SGI0, SGI0, SGI0, SGI176746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39152[] = {SGI176747, SGI0, SGI176748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176749, SGI176750, SGI176750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39153[] = {SGI176751, SGI0, SGI176752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176753, SGI176754, SGI176754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39154[] = {SGI176755, SGI0, SGI176756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176758, SGI176760, SGI176760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39155[] = {SGI176757, SGI0, SGI176759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176761, SGI176762, SGI176762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39156[] = {SGI176763, SGI0, SGI176764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176765, SGI176766, SGI176766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39157[] = {SGI176767, SGI0, SGI176768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176769, SGI176770, SGI176770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39158[] = {SGI176771, SGI0, SGI176772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176773, SGI176774, SGI176774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39159[] = {SGI176775, SGI0, SGI176776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176777, SGI176778, SGI176778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39160[] = {SGI176779, SGI176780, SGI176781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176782, SGI0, SGI0, SGI0, SGI176783, SGI176783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39161[] = {SGI176784, SGI0, SGI176785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176786, SGI176787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39162[] = {SGI176788, SGI0, SGI176789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176790, SGI176791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39163[] = {SGI176792, SGI0, SGI176793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176794, SGI176795, SGI176795, SGI0, SGI0, NULL};
const Segment * const SG39164[] = {SGI176796, SGI0, SGI176797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176798, SGI176799, SGI176799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39165[] = {SGI176800, SGI0, SGI176801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176802, SGI176803, SGI176803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39166[] = {SGI176804, SGI176805, SGI176806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176807, SGI0, SGI0, SGI0, SGI176808, SGI176808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39167[] = {SGI176809, SGI176810, SGI176811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176812, SGI0, SGI0, SGI0, SGI176813, SGI176813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39168[] = {SGI176814, SGI176815, SGI176816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176817, SGI0, SGI0, SGI0, SGI176818, SGI176818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39169[] = {SGI176819, SGI0, SGI176820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176821, SGI176822, SGI176822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39170[] = {SGI176823, SGI0, SGI176824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176825, SGI176826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39171[] = {SGI176827, SGI176828, SGI176829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176830, SGI0, SGI0, SGI0, SGI176831, SGI176831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39172[] = {SGI176832, SGI176833, SGI176834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176835, SGI0, SGI0, SGI0, SGI176836, SGI176836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39173[] = {SGI176837, SGI176838, SGI176839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176840, SGI0, SGI0, SGI0, SGI176841, SGI176841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39174[] = {SGI176842, SGI176843, SGI176844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176845, SGI0, SGI0, SGI0, SGI176846, SGI176846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39175[] = {SGI176847, SGI176848, SGI176849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176850, SGI0, SGI0, SGI0, SGI176851, SGI176851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39176[] = {SGI176852, SGI0, SGI176853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176856, SGI176857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39177[] = {SGI176854, SGI0, SGI176855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176858, SGI176859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39178[] = {SGI176860, SGI0, SGI176861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176862, SGI176863, SGI176863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39179[] = {SGI176864, SGI0, SGI176865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176867, SGI176869, SGI176869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39180[] = {SGI176866, SGI0, SGI176868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176871, SGI176872, SGI176872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39181[] = {SGI176870, SGI0, SGI176873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176874, SGI176875, SGI176875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39182[] = {SGI176876, SGI176877, SGI176878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176879, SGI0, SGI0, SGI0, SGI176880, SGI176880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39183[] = {SGI176882, SGI176883, SGI176884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176885, SGI0, SGI0, SGI0, SGI176886, SGI176886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39184[] = {SGI176888, SGI176889, SGI176890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176891, SGI0, SGI0, SGI0, SGI176892, SGI176892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39185[] = {SGI176894, SGI176895, SGI176896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176897, SGI0, SGI0, SGI0, SGI176898, SGI176898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39186[] = {SGI176899, SGI0, SGI176900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39187[] = {SGI176901, SGI0, SGI176902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176903, SGI176904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39188[] = {SGI176905, SGI0, SGI176906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176907, SGI176908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39189[] = {SGI176909, SGI176910, SGI176911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176912, SGI0, SGI0, SGI0, SGI176913, SGI176913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39190[] = {SGI176915, SGI176916, SGI176917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176918, SGI0, SGI0, SGI0, SGI176919, SGI176919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39191[] = {SGI176921, SGI176922, SGI176923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176924, SGI0, SGI0, SGI0, SGI176926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39192[] = {SGI176925, SGI176927, SGI176928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176929, SGI0, SGI0, SGI0, SGI176930, SGI176930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39193[] = {SGI176932, SGI0, SGI176933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39194[] = {SGI176934, SGI176935, SGI176936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176937, SGI0, SGI0, SGI0, SGI176938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39195[] = {SGI176939, SGI176940, SGI176941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176942, SGI0, SGI0, SGI0, SGI176943, SGI176943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39196[] = {SGI176945, SGI176946, SGI176947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176948, SGI0, SGI0, SGI0, SGI176949, SGI176949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39197[] = {SGI176951, SGI176952, SGI176953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176954, SGI0, SGI0, SGI0, SGI176955, SGI176955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39198[] = {SGI176957, SGI176958, SGI176959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176960, SGI0, SGI0, SGI0, SGI176961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39199[] = {SGI176962, SGI176963, SGI176964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176965, SGI0, SGI0, SGI0, SGI176966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39200[] = {SGI176967, SGI176968, SGI176969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176970, SGI0, SGI0, SGI0, SGI176971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39201[] = {SGI176972, SGI0, SGI176973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176974, SGI176975, SGI176975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39202[] = {SGI176976, SGI176977, SGI176978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176979, SGI0, SGI0, SGI0, SGI176980, SGI176980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39203[] = {SGI176981, SGI0, SGI176982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39204[] = {SGI176983, SGI176984, SGI176985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176986, SGI0, SGI0, SGI0, SGI176987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39205[] = {SGI176988, SGI0, SGI176989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176990, SGI176991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39206[] = {SGI176992, SGI176993, SGI176994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176995, SGI0, SGI0, SGI0, SGI176996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39207[] = {SGI176997, SGI0, SGI176998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI176999, SGI177000, SGI177000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39208[] = {SGI177001, SGI177002, SGI177003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177004, SGI177004, SGI0, SGI0, SGI0, SGI177005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39209[] = {SGI177006, SGI0, SGI177007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177008, SGI177009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39210[] = {SGI177010, SGI0, SGI177011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177012, SGI177013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39211[] = {SGI177014, SGI0, SGI177015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177016, SGI177017, SGI177017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39212[] = {SGI177018, SGI177019, SGI177020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177021, SGI0, SGI0, SGI0, SGI177022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39213[] = {SGI177023, SGI0, SGI177024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177025, SGI177026, SGI177026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39214[] = {SGI177027, SGI177028, SGI177029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177030, SGI0, SGI0, SGI0, SGI177031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39215[] = {SGI177032, SGI177033, SGI177034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177035, SGI0, SGI0, SGI0, SGI177036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39216[] = {SGI177037, SGI177038, SGI177039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177040, SGI0, SGI0, SGI0, SGI177041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39217[] = {SGI177042, SGI177043, SGI177044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39218[] = {SGI177045, SGI177046, SGI177047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39219[] = {SGI177048, SGI177049, SGI177050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39220[] = {SGI177051, SGI0, SGI177052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39221[] = {SGI177053, SGI177054, SGI177055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39222[] = {SGI0, SGI177058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39223[] = {SGI177056, SGI0, SGI177057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177059, SGI177060, SGI177060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39224[] = {SGI177061, SGI177062, SGI177063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177064, SGI0, SGI0, SGI0, SGI177065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39225[] = {SGI177066, SGI177067, SGI177068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39226[] = {SGI177069, SGI177070, SGI177071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39227[] = {SGI177072, SGI177073, SGI177074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177075, SGI177075, SGI0, SGI0, SGI0, SGI177076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39228[] = {SGI177077, SGI177078, SGI177079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177080, SGI177080, SGI0, SGI0, SGI0, SGI177081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39229[] = {SGI177082, SGI177083, SGI177084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177085, SGI177085, SGI0, SGI0, SGI0, SGI177086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39230[] = {SGI177087, SGI177088, SGI177089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177090, SGI177090, SGI0, SGI0, SGI0, SGI177091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39231[] = {SGI177092, SGI177093, SGI177094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177095, SGI177095, SGI0, SGI0, SGI0, SGI177096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39232[] = {SGI177097, SGI177098, SGI177099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177100, SGI177100, SGI0, SGI0, SGI0, SGI177101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39233[] = {SGI177102, SGI0, SGI177103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177104, SGI177105, SGI177105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39234[] = {SGI177106, SGI0, SGI177107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177108, SGI177109, SGI177109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39235[] = {SGI177110, SGI0, SGI177111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177112, SGI177113, SGI177113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39236[] = {SGI177114, SGI0, SGI177115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177116, SGI177117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39237[] = {SGI177119, SGI0, SGI177121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177124, SGI177125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39238[] = {SGI177118, SGI177120, SGI177122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177123, SGI0, SGI0, SGI0, SGI177126, SGI177126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39239[] = {SGI177127, SGI0, SGI177128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177132, SGI177133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39240[] = {SGI177129, SGI177130, SGI177131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177134, SGI0, SGI0, SGI0, SGI177135, SGI177135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39241[] = {SGI177136, SGI0, SGI177137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177138, SGI177139, SGI177139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39242[] = {SGI177140, SGI0, SGI177141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177142, SGI177143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39243[] = {SGI177144, SGI0, SGI177145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177146, SGI177147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39244[] = {SGI177148, SGI177149, SGI177150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177151, SGI0, SGI0, SGI0, SGI177152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39245[] = {SGI177153, SGI177154, SGI177155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177156, SGI0, SGI0, SGI0, SGI177157, SGI0, SGI0, SGI177158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39246[] = {SGI177159, SGI177160, SGI177161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177162, SGI0, SGI0, SGI0, SGI177163, SGI177163, SGI0, SGI177164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39247[] = {SGI177165, SGI177166, SGI177167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177168, SGI0, SGI0, SGI0, SGI177169, SGI0, SGI0, SGI177170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39248[] = {SGI177171, SGI0, SGI177172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177173, SGI177174, SGI177174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39249[] = {SGI177175, SGI177176, SGI177177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177178, SGI0, SGI0, SGI0, SGI177180, SGI177180, SGI0, SGI177183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39250[] = {SGI177179, SGI177181, SGI177182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177184, SGI0, SGI0, SGI0, SGI177185, SGI177185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39251[] = {SGI177186, SGI177187, SGI177188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177189, SGI0, SGI0, SGI0, SGI177190, SGI0, SGI0, SGI177191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39252[] = {SGI177192, SGI177193, SGI177194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177195, SGI0, SGI0, SGI0, SGI177196, SGI177196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39253[] = {SGI177197, SGI0, SGI177198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177199, SGI177200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39254[] = {SGI177201, SGI0, SGI177202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177203, SGI177204, SGI177204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39255[] = {SGI177205, SGI177206, SGI177207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177208, SGI0, SGI0, SGI0, SGI177209, SGI177209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39256[] = {SGI177210, SGI0, SGI177211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177212, SGI177213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39257[] = {SGI177214, SGI177215, SGI177216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177217, SGI0, SGI0, SGI0, SGI177218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39258[] = {SGI177219, SGI177220, SGI177221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177222, SGI0, SGI0, SGI0, SGI177223, SGI177223, SGI0, SGI177224, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39259[] = {SGI177225, SGI0, SGI177226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177230, SGI177231, SGI177231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39260[] = {SGI177227, SGI177228, SGI177229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177232, SGI0, SGI0, SGI0, SGI177233, SGI0, SGI0, SGI177234, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39261[] = {SGI177235, SGI177236, SGI177237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177238, SGI0, SGI0, SGI0, SGI177239, SGI0, SGI0, SGI177240, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39262[] = {SGI177241, SGI177242, SGI177243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177244, SGI0, SGI0, SGI0, SGI177245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39263[] = {SGI177246, SGI0, SGI177247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39264[] = {SGI177248, SGI177249, SGI177250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177251, SGI0, SGI0, SGI0, SGI177252, SGI177252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39265[] = {SGI177253, SGI177254, SGI177255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177256, SGI0, SGI0, SGI0, SGI177257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39266[] = {SGI177258, SGI177259, SGI177260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177261, SGI0, SGI0, SGI0, SGI177262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39267[] = {SGI177263, SGI177264, SGI177265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177266, SGI0, SGI0, SGI0, SGI177267, SGI177267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39268[] = {SGI177268, SGI0, SGI177269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177270, SGI177271, SGI177271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39269[] = {SGI177272, SGI0, SGI177273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177274, SGI177275, SGI177275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39270[] = {SGI177276, SGI0, SGI177277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177278, SGI177279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39271[] = {SGI177280, SGI177281, SGI177282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177283, SGI0, SGI0, SGI0, SGI177284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39272[] = {SGI177285, SGI0, SGI177286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39273[] = {SGI177287, SGI0, SGI177288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177289, SGI177290, SGI177290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39274[] = {SGI177291, SGI0, SGI177292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177293, SGI177294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39275[] = {SGI177295, SGI0, SGI177296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177297, SGI177298, SGI177298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39276[] = {SGI177299, SGI0, SGI177301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177303, SGI177304, SGI177304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39277[] = {SGI177300, SGI0, SGI177302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177305, SGI177306, SGI177306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39278[] = {SGI177307, SGI0, SGI177308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177309, SGI177310, SGI177310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39279[] = {SGI177311, SGI177312, SGI177313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177314, SGI0, SGI0, SGI0, SGI177315, SGI177315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39280[] = {SGI177317, SGI177318, SGI177319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177320, SGI0, SGI0, SGI0, SGI177321, SGI177321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39281[] = {SGI177322, SGI177323, SGI177324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177325, SGI0, SGI0, SGI0, SGI177326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39282[] = {SGI177327, SGI0, SGI177328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39283[] = {SGI177329, SGI177330, SGI177331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177332, SGI177332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39284[] = {SGI177333, SGI177334, SGI177335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177336, SGI0, SGI0, SGI0, SGI177337, SGI177337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39285[] = {SGI177338, SGI177339, SGI177340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177341, SGI0, SGI0, SGI0, SGI177342, SGI177342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39286[] = {SGI177343, SGI177344, SGI177345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177346, SGI0, SGI0, SGI0, SGI177347, SGI177347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39287[] = {SGI177348, SGI0, SGI177349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39288[] = {SGI177350, SGI177351, SGI177352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177353, SGI0, SGI0, SGI0, SGI177354, SGI177354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39289[] = {SGI177355, SGI177356, SGI177357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177358, SGI0, SGI0, SGI0, SGI177359, SGI177359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39290[] = {SGI177360, SGI0, SGI177361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177362, SGI177363, SGI177363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39291[] = {SGI177364, SGI0, SGI177365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177366, SGI177367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39292[] = {SGI177368, SGI0, SGI177369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177370, SGI177371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39293[] = {SGI177372, SGI177373, SGI177374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177375, SGI0, SGI0, SGI0, SGI177376, SGI177376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39294[] = {SGI177377, SGI0, SGI177378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39295[] = {SGI177379, SGI177380, SGI177381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177382, SGI0, SGI0, SGI0, SGI177383, SGI177383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39296[] = {SGI177384, SGI177385, SGI177386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177387, SGI0, SGI0, SGI0, SGI177388, SGI177388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39297[] = {SGI177389, SGI177390, SGI177391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177392, SGI0, SGI0, SGI0, SGI177393, SGI177393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39298[] = {SGI177394, SGI177395, SGI177396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177397, SGI0, SGI0, SGI0, SGI177398, SGI177398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39299[] = {SGI177399, SGI177400, SGI177401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177402, SGI0, SGI0, SGI0, SGI177403, SGI177403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39300[] = {SGI177404, SGI177405, SGI177406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177407, SGI0, SGI0, SGI0, SGI177408, SGI177408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39301[] = {SGI177409, SGI0, SGI177410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177411, SGI177412, SGI177412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39302[] = {SGI177413, SGI0, SGI177414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177417, SGI177419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39303[] = {SGI177415, SGI177416, SGI177418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177420, SGI0, SGI0, SGI0, SGI177421, SGI177421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39304[] = {SGI177422, SGI0, SGI177423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177424, SGI177425, SGI177425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39305[] = {SGI177426, SGI0, SGI177427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177428, SGI177429, SGI177429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39306[] = {SGI177430, SGI0, SGI177431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177432, SGI177433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39307[] = {SGI177434, SGI0, SGI177435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177436, SGI177437, SGI177437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39308[] = {SGI177438, SGI177439, SGI177440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177441, SGI0, SGI0, SGI0, SGI177443, SGI177443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39309[] = {SGI177442, SGI0, SGI177444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177445, SGI177446, SGI177446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39310[] = {SGI177447, SGI177448, SGI177449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177450, SGI177450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39311[] = {SGI177451, SGI177452, SGI177453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177454, SGI0, SGI0, SGI0, SGI177455, SGI177455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39312[] = {SGI177456, SGI177457, SGI177458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177459, SGI0, SGI0, SGI0, SGI177460, SGI177460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39313[] = {SGI177461, SGI177462, SGI177463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177464, SGI0, SGI0, SGI0, SGI177465, SGI177465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39314[] = {SGI177466, SGI0, SGI177467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39315[] = {SGI177468, SGI177469, SGI177470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177474, SGI0, SGI0, SGI0, SGI177475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39316[] = {SGI177471, SGI177472, SGI177473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177476, SGI0, SGI0, SGI0, SGI177477, SGI177477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39317[] = {SGI177478, SGI177479, SGI177480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177481, SGI0, SGI0, SGI0, SGI177482, SGI177482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39318[] = {SGI177483, SGI177484, SGI177485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177486, SGI0, SGI0, SGI0, SGI177487, SGI177487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39319[] = {SGI177488, SGI0, SGI177489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39320[] = {SGI177490, SGI0, SGI177491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39321[] = {SGI177492, SGI0, SGI177493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177494, SGI177495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39322[] = {SGI177496, SGI0, SGI177497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177498, SGI177499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39323[] = {SGI177500, SGI177501, SGI177502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177503, SGI0, SGI0, SGI0, SGI177504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39324[] = {SGI177505, SGI0, SGI177506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177507, SGI177508, SGI177508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39325[] = {SGI177509, SGI0, SGI177510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177511, SGI177512, SGI177512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39326[] = {SGI177513, SGI0, SGI177514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177515, SGI177516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39327[] = {SGI177517, SGI0, SGI177518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177519, SGI177520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39328[] = {SGI177521, SGI0, SGI177522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177523, SGI177524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39329[] = {SGI177525, SGI0, SGI177526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177527, SGI177528, SGI177528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39330[] = {SGI177529, SGI0, SGI177530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177531, SGI177532, SGI177532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39331[] = {SGI177533, SGI0, SGI177534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177535, SGI177536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39332[] = {SGI177537, SGI177538, SGI177539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177540, SGI0, SGI0, SGI0, SGI177541, SGI177541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39333[] = {SGI177543, SGI177544, SGI177545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177546, SGI0, SGI0, SGI0, SGI177547, SGI177547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39334[] = {SGI177549, SGI0, SGI177550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177553, SGI177554, SGI177554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39335[] = {SGI177551, SGI0, SGI177552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177555, SGI177556, SGI177556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39336[] = {SGI177557, SGI0, SGI177558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177559, SGI177560, SGI177560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39337[] = {SGI177561, SGI0, SGI177562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177563, SGI177564, SGI177564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39338[] = {SGI177565, SGI0, SGI177566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177567, SGI177568, SGI177568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39339[] = {SGI177569, SGI0, SGI177570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177571, SGI177572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39340[] = {SGI177573, SGI0, SGI177574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177575, SGI177576, SGI177576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39341[] = {SGI177577, SGI0, SGI177578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177579, SGI177580, SGI177580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39342[] = {SGI177582, SGI177583, SGI177585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177586, SGI0, SGI0, SGI0, SGI177588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39343[] = {SGI177581, SGI0, SGI177584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177587, SGI177589, SGI177589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39344[] = {SGI177590, SGI0, SGI177591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177592, SGI177593, SGI177593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39345[] = {SGI177594, SGI177596, SGI177598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177599, SGI0, SGI0, SGI0, SGI177601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39346[] = {SGI177595, SGI0, SGI177597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177600, SGI177602, SGI177602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39347[] = {SGI177603, SGI0, SGI177604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177605, SGI177606, SGI177606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39348[] = {SGI177607, SGI177608, SGI177609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177610, SGI0, SGI0, SGI0, SGI177611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39349[] = {SGI177612, SGI0, SGI177613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177614, SGI177615, SGI177615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39350[] = {SGI177616, SGI177617, SGI177618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177619, SGI0, SGI0, SGI0, SGI177620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39351[] = {SGI177621, SGI0, SGI177622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177623, SGI177624, SGI177624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39352[] = {SGI177625, SGI177626, SGI177627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177628, SGI0, SGI0, SGI0, SGI177629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39353[] = {SGI177630, SGI177631, SGI177632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177633, SGI0, SGI0, SGI0, SGI177634, SGI177634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39354[] = {SGI177635, SGI177636, SGI177637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177638, SGI0, SGI0, SGI0, SGI177639, SGI177639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39355[] = {SGI177640, SGI0, SGI177641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177642, SGI177643, SGI177643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39356[] = {SGI177644, SGI0, SGI177645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177646, SGI177647, SGI177647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39357[] = {SGI177648, SGI0, SGI177649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177650, SGI177651, SGI177651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39358[] = {SGI177652, SGI0, SGI177653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177654, SGI177655, SGI177655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39359[] = {SGI177656, SGI177657, SGI177658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177659, SGI0, SGI0, SGI0, SGI177660, SGI177660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39360[] = {SGI177661, SGI177662, SGI177663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177664, SGI0, SGI0, SGI0, SGI177665, SGI177665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39361[] = {SGI177666, SGI0, SGI177667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39362[] = {SGI177668, SGI177669, SGI177670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177671, SGI0, SGI0, SGI0, SGI177672, SGI177672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39363[] = {SGI177673, SGI177674, SGI177675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177676, SGI0, SGI0, SGI0, SGI177677, SGI177677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39364[] = {SGI177679, SGI0, SGI177682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177684, SGI177685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39365[] = {SGI177678, SGI177680, SGI177681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177683, SGI0, SGI0, SGI0, SGI177686, SGI177686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39366[] = {SGI177687, SGI177688, SGI177689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177690, SGI0, SGI0, SGI0, SGI177691, SGI177691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39367[] = {SGI177692, SGI177693, SGI177694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177697, SGI0, SGI0, SGI0, SGI177698, SGI177698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39368[] = {SGI177695, SGI0, SGI177696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177699, SGI177700, SGI177700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177701, SGI0, SGI0, NULL};
const Segment * const SG39369[] = {SGI177702, SGI0, SGI177703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177704, SGI177705, SGI177705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39370[] = {SGI177706, SGI0, SGI177707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177708, SGI177709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39371[] = {SGI177710, SGI0, SGI177711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177712, SGI177713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39372[] = {SGI177714, SGI0, SGI177715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177716, SGI177717, SGI177717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39373[] = {SGI177718, SGI177719, SGI177720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177721, SGI0, SGI0, SGI0, SGI177722, SGI177722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39374[] = {SGI177723, SGI177724, SGI177725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177726, SGI0, SGI0, SGI0, SGI177727, SGI177727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39375[] = {SGI177728, SGI0, SGI177729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39376[] = {SGI177730, SGI0, SGI177731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177732, SGI177733, SGI177733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39377[] = {SGI177734, SGI0, SGI177735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177736, SGI177737, SGI177737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39378[] = {SGI177738, SGI0, SGI177739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177740, SGI177741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39379[] = {SGI177742, SGI0, SGI177743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177744, SGI177745, SGI177745, SGI0, SGI0, NULL};
const Segment * const SG39380[] = {SGI177747, SGI0, SGI177749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177752, SGI177753, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39381[] = {SGI177746, SGI177748, SGI177750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177751, SGI0, SGI0, SGI177754, SGI177755, SGI177755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39382[] = {SGI177756, SGI177757, SGI177758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177760, SGI177760, SGI0, SGI0, SGI177763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39383[] = {SGI177759, SGI177761, SGI177762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177764, SGI0, SGI0, SGI177765, SGI177766, SGI177766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39384[] = {SGI177767, SGI177768, SGI177769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177770, SGI0, SGI0, SGI177771, SGI177772, SGI177772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39385[] = {SGI177773, SGI0, SGI177774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177775, SGI177776, SGI177776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39386[] = {SGI177777, SGI0, SGI177778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177779, SGI177780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39387[] = {SGI177781, SGI0, SGI177782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177783, SGI177784, SGI177784, SGI0, SGI0, NULL};
const Segment * const SG39388[] = {SGI177785, SGI177786, SGI177787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177788, SGI0, SGI0, SGI0, SGI177789, SGI177789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39389[] = {SGI177791, SGI0, SGI177792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177793, SGI177794, SGI177794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39390[] = {SGI177795, SGI0, SGI177796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177797, SGI177798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39391[] = {SGI177799, SGI0, SGI177800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177801, SGI177802, SGI177802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39392[] = {SGI177803, SGI0, SGI177804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177805, SGI177806, SGI177806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39393[] = {SGI177807, SGI0, SGI177808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177809, SGI177810, SGI177810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39394[] = {SGI177811, SGI0, SGI177812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177813, SGI177814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39395[] = {SGI177815, SGI0, SGI177816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177819, SGI177820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39396[] = {SGI177817, SGI0, SGI177818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177821, SGI177822, SGI177822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39397[] = {SGI177823, SGI0, SGI177824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177825, SGI177826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39398[] = {SGI177827, SGI0, SGI177828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177829, SGI177830, SGI177830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39399[] = {SGI177831, SGI0, SGI177832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177833, SGI177834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39400[] = {SGI177835, SGI0, SGI177836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177837, SGI177838, SGI177838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39401[] = {SGI177839, SGI0, SGI177840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177841, SGI177842, SGI177842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39402[] = {SGI177843, SGI0, SGI177844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177845, SGI177846, SGI177846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39403[] = {SGI177847, SGI0, SGI177848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177849, SGI177850, SGI177850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177851, SGI0, SGI0, NULL};
const Segment * const SG39404[] = {SGI177852, SGI0, SGI177853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177854, SGI177855, SGI177855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39405[] = {SGI177856, SGI0, SGI177857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177858, SGI177859, SGI177859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39406[] = {SGI177860, SGI177861, SGI177862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177863, SGI0, SGI0, SGI0, SGI177864, SGI177864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39407[] = {SGI177865, SGI177866, SGI177867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177868, SGI0, SGI0, SGI0, SGI177869, SGI177869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39408[] = {SGI177870, SGI177871, SGI177872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177873, SGI0, SGI0, SGI0, SGI177874, SGI177874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39409[] = {SGI177875, SGI0, SGI177876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177880, SGI177881, SGI177881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39410[] = {SGI177877, SGI177878, SGI177879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177882, SGI0, SGI0, SGI0, SGI177883, SGI177883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39411[] = {SGI177884, SGI177885, SGI177886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177887, SGI0, SGI0, SGI0, SGI177888, SGI177888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39412[] = {SGI177889, SGI177890, SGI177891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177892, SGI0, SGI0, SGI0, SGI177893, SGI177893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39413[] = {SGI177894, SGI177895, SGI177896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177897, SGI0, SGI0, SGI0, SGI177898, SGI177898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39414[] = {SGI177899, SGI0, SGI177900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177901, SGI177902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39415[] = {SGI177903, SGI0, SGI177904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177905, SGI0, SGI0, SGI0, SGI177906, SGI177906, SGI0, SGI0, SGI0, SGI177907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39416[] = {SGI177908, SGI177909, SGI177910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177911, SGI0, SGI0, SGI0, SGI177912, SGI177913, SGI0, SGI0, SGI0, SGI177914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39417[] = {SGI177915, SGI0, SGI177916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177917, SGI177918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177919, SGI0, SGI0, NULL};
const Segment * const SG39418[] = {SGI177920, SGI0, SGI177921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177922, SGI177923, SGI177923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39419[] = {SGI177924, SGI0, SGI177925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177926, SGI177927, SGI177927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39420[] = {SGI177928, SGI0, SGI177929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177930, SGI177931, SGI177931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39421[] = {SGI177932, SGI0, SGI177933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177934, SGI177935, SGI177935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39422[] = {SGI177936, SGI0, SGI177937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177938, SGI177939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39423[] = {SGI177940, SGI0, SGI177941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177942, SGI177943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39424[] = {SGI177944, SGI0, SGI177945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177946, SGI177947, SGI177947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39425[] = {SGI177948, SGI177949, SGI177950, SGI0, SGI0, SGI0, SGI177951, SGI177952, SGI0, SGI177953, SGI0, SGI0, SGI0, SGI177954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39426[] = {SGI177955, SGI177956, SGI177957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177958, SGI177958, SGI0, SGI0, SGI0, SGI177959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39427[] = {SGI177960, SGI0, SGI177961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177962, SGI177963, SGI177963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39428[] = {SGI177964, SGI0, SGI177965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177966, SGI177967, SGI177967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39429[] = {SGI177968, SGI0, SGI177969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177970, SGI177971, SGI177971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39430[] = {SGI177972, SGI0, SGI177973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177974, SGI177975, SGI177975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39431[] = {SGI177976, SGI0, SGI177977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177978, SGI177979, SGI177979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39432[] = {SGI177980, SGI0, SGI177981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177982, SGI177983, SGI177983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39433[] = {SGI177984, SGI0, SGI177985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177988, SGI177989, SGI177989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39434[] = {SGI177986, SGI0, SGI177987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177990, SGI177991, SGI177991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39435[] = {SGI177992, SGI0, SGI177993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177994, SGI177995, SGI177995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39436[] = {SGI177996, SGI0, SGI177997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI177998, SGI177999, SGI177999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39437[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178000, SGI0, SGI0, SGI178001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39438[] = {SGI178002, SGI0, SGI178003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178004, SGI178005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39439[] = {SGI178006, SGI178007, SGI178008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178009, SGI0, SGI0, SGI0, SGI178010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39440[] = {SGI178011, SGI0, SGI178012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178013, SGI178014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39441[] = {SGI178015, SGI0, SGI178016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178017, SGI178018, SGI178018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39442[] = {SGI178019, SGI0, SGI178020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178021, SGI178022, SGI178022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39443[] = {SGI178023, SGI0, SGI178025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178027, SGI178028, SGI178028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39444[] = {SGI178024, SGI0, SGI178026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178029, SGI178030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39445[] = {SGI178031, SGI178032, SGI178033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178034, SGI0, SGI0, SGI0, SGI178035, SGI178035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39446[] = {SGI178037, SGI178038, SGI178039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178040, SGI0, SGI0, SGI0, SGI178041, SGI178041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39447[] = {SGI178042, SGI178043, SGI178044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178045, SGI0, SGI0, SGI0, SGI178047, SGI178047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39448[] = {SGI178046, SGI0, SGI178048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39449[] = {SGI178050, SGI0, SGI178051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178052, SGI178053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39450[] = {SGI178054, SGI0, SGI178055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178056, SGI178057, SGI178057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39451[] = {SGI178058, SGI0, SGI178059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178060, SGI178061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39452[] = {SGI178062, SGI0, SGI178063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178064, SGI178065, SGI178065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178066, SGI0, SGI0, NULL};
const Segment * const SG39453[] = {SGI178067, SGI0, SGI178068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178069, SGI178070, SGI178070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178071, SGI0, SGI0, NULL};
const Segment * const SG39454[] = {SGI178072, SGI0, SGI178073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178074, SGI178075, SGI178075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39455[] = {SGI178076, SGI178077, SGI178078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178079, SGI0, SGI0, SGI0, SGI178080, SGI178080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39456[] = {SGI178081, SGI178082, SGI178083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178084, SGI178084, SGI0, SGI0, SGI178085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39457[] = {SGI178086, SGI0, SGI178087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39458[] = {SGI178088, SGI178090, SGI178092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178094, SGI0, SGI0, SGI0, SGI178096, SGI178096, SGI0, SGI178097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39459[] = {SGI178089, SGI178091, SGI178093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178095, SGI178095, SGI0, SGI0, SGI178098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39460[] = {SGI178099, SGI178100, SGI178101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178102, SGI178102, SGI0, SGI0, SGI178103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39461[] = {SGI178104, SGI178105, SGI178106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178107, SGI178107, SGI0, SGI0, SGI178108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39462[] = {SGI178109, SGI178110, SGI178111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178112, SGI178112, SGI0, SGI0, SGI178113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39463[] = {SGI178114, SGI0, SGI178115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178116, SGI178117, SGI178117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39464[] = {SGI178118, SGI0, SGI178119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178120, SGI178121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39465[] = {SGI178122, SGI0, SGI178123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178124, SGI178125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39466[] = {SGI178126, SGI0, SGI178127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178128, SGI178129, SGI178129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39467[] = {SGI178130, SGI0, SGI178131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178132, SGI178133, SGI178133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39468[] = {SGI178134, SGI0, SGI178135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178136, SGI178137, SGI178137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39469[] = {SGI178138, SGI0, SGI178139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178140, SGI178141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39470[] = {SGI178142, SGI0, SGI178143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178144, SGI178145, SGI178145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39471[] = {SGI0, SGI178146, SGI178147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178148, SGI0, SGI0, SGI0, SGI178149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39472[] = {SGI178150, SGI178151, SGI178152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178153, SGI0, SGI0, SGI0, SGI178154, SGI178154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39473[] = {SGI178155, SGI0, SGI178156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39474[] = {SGI178157, SGI0, SGI178158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178159, SGI178160, SGI178160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39475[] = {SGI178161, SGI0, SGI178162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178163, SGI178164, SGI178164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39476[] = {SGI178165, SGI178166, SGI178167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178168, SGI0, SGI0, SGI0, SGI178169, SGI178169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39477[] = {SGI178171, SGI178172, SGI178173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178174, SGI0, SGI0, SGI0, SGI178175, SGI178175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39478[] = {SGI178177, SGI0, SGI178178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178179, SGI178181, SGI178181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39479[] = {SGI178180, SGI178182, SGI178183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178186, SGI0, SGI0, SGI0, SGI178188, SGI178188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39480[] = {SGI178184, SGI178185, SGI178187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178189, SGI0, SGI0, SGI0, SGI178190, SGI178190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39481[] = {SGI178192, SGI178193, SGI178194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178195, SGI0, SGI0, SGI0, SGI178196, SGI178196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39482[] = {SGI178197, SGI0, SGI178198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178200, SGI178202, SGI178202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39483[] = {SGI178199, SGI178201, SGI178203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178204, SGI0, SGI0, SGI0, SGI178205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39484[] = {SGI178206, SGI0, SGI178207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178211, SGI178212, SGI178212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39485[] = {SGI178208, SGI178209, SGI178210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178213, SGI0, SGI0, SGI0, SGI178214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39486[] = {SGI178215, SGI0, SGI178216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178217, SGI178218, SGI178218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39487[] = {SGI178219, SGI0, SGI178220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178221, SGI178222, SGI178222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39488[] = {SGI178223, SGI178224, SGI178225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178226, SGI0, SGI0, SGI0, SGI178229, SGI178229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39489[] = {SGI178227, SGI178228, SGI178230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178231, SGI0, SGI0, SGI0, SGI178232, SGI178232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39490[] = {SGI178233, SGI178234, SGI178235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178236, SGI0, SGI0, SGI0, SGI178239, SGI178239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39491[] = {SGI178237, SGI178238, SGI178240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178241, SGI0, SGI0, SGI0, SGI178242, SGI178242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39492[] = {SGI178243, SGI0, SGI178244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39493[] = {SGI178245, SGI0, SGI178246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39494[] = {SGI178247, SGI0, SGI178248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178249, SGI178250, SGI178250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39495[] = {SGI178251, SGI0, SGI178252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178253, SGI178254, SGI178254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39496[] = {SGI178255, SGI178256, SGI178257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178258, SGI0, SGI0, SGI0, SGI178259, SGI178259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39497[] = {SGI178260, SGI0, SGI178261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39498[] = {SGI178262, SGI0, SGI178263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178264, SGI178265, SGI178265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39499[] = {SGI178266, SGI0, SGI178267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178268, SGI178269, SGI178269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39500[] = {SGI178270, SGI0, SGI178271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178272, SGI178273, SGI178273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39501[] = {SGI178274, SGI0, SGI178275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178276, SGI178277, SGI178277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39502[] = {SGI178279, SGI0, SGI178280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178282, SGI178284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39503[] = {SGI178278, SGI0, SGI178281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178283, SGI178285, SGI178285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39504[] = {SGI0, SGI0, SGI178287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178289, SGI178292, SGI178292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39505[] = {SGI178286, SGI0, SGI178288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178290, SGI178291, SGI178291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39506[] = {SGI178293, SGI0, SGI178294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178295, SGI178296, SGI178296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39507[] = {SGI178297, SGI0, SGI178298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178299, SGI178300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39508[] = {SGI178301, SGI0, SGI178302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178303, SGI178304, SGI178304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39509[] = {SGI178305, SGI178306, SGI178307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178308, SGI0, SGI0, SGI0, SGI178309, SGI178309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39510[] = {SGI178310, SGI178311, SGI178312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178313, SGI0, SGI0, SGI0, SGI178314, SGI178314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39511[] = {SGI178315, SGI0, SGI178316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178317, SGI178318, SGI178318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39512[] = {SGI178319, SGI0, SGI178320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178321, SGI178322, SGI178322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39513[] = {SGI178323, SGI178324, SGI178325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178326, SGI0, SGI0, SGI0, SGI178327, SGI178327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39514[] = {SGI178328, SGI0, SGI178329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39515[] = {SGI178330, SGI178331, SGI178332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178333, SGI0, SGI0, SGI0, SGI178334, SGI178334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39516[] = {SGI178335, SGI0, SGI178336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39517[] = {SGI178337, SGI0, SGI178338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178339, SGI178340, SGI178340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39518[] = {SGI178341, SGI0, SGI178342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178343, SGI178344, SGI178344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39519[] = {SGI178345, SGI178346, SGI178347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178348, SGI0, SGI0, SGI0, SGI178349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39520[] = {SGI178350, SGI0, SGI178351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178352, SGI178353, SGI178353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39521[] = {SGI178354, SGI0, SGI178355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39522[] = {SGI178356, SGI178357, SGI178358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178359, SGI0, SGI0, SGI0, SGI178360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39523[] = {SGI178361, SGI0, SGI178362, SGI0, SGI178363, SGI178363, SGI0, NULL};
const Segment * const SG39524[] = {SGI178364, SGI0, SGI178365, SGI0, SGI178366, SGI178366, SGI0, NULL};
const Segment * const SG39525[] = {SGI178367, SGI178368, SGI178369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178370, SGI0, SGI0, SGI0, SGI178371, SGI178371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39526[] = {SGI178372, SGI0, SGI178373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39527[] = {SGI178374, SGI178375, SGI178376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178377, SGI0, SGI0, SGI0, SGI178378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39528[] = {SGI178379, SGI178380, SGI178381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178382, SGI0, SGI0, SGI0, SGI178383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39529[] = {SGI178384, SGI0, SGI178385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39530[] = {SGI178386, SGI178387, SGI178388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178389, SGI0, SGI0, SGI0, SGI178390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39531[] = {SGI178391, SGI178392, SGI178393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178394, SGI0, SGI0, SGI0, SGI178395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39532[] = {SGI178396, SGI0, SGI178397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39533[] = {SGI178398, SGI0, SGI178399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178400, SGI178401, SGI178401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39534[] = {SGI178402, SGI0, SGI178403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178404, SGI178405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39535[] = {SGI178406, SGI178407, SGI178408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178409, SGI0, SGI0, SGI0, SGI178410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39536[] = {SGI178411, SGI0, SGI178412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39537[] = {SGI178413, SGI0, SGI178414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178415, SGI178416, SGI178416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39538[] = {SGI178417, SGI0, SGI178418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178419, SGI178421, SGI178421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39539[] = {SGI178420, SGI0, SGI178422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178423, SGI178424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39540[] = {SGI178425, SGI0, SGI178426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178427, SGI178428, SGI178428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39541[] = {SGI178429, SGI0, SGI178430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178431, SGI178432, SGI178432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39542[] = {SGI178433, SGI0, SGI178434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178435, SGI178436, SGI178436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39543[] = {SGI178437, SGI0, SGI178438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178439, SGI178440, SGI178440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39544[] = {SGI178441, SGI178442, SGI178443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178444, SGI0, SGI0, SGI0, SGI178445, SGI178445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39545[] = {SGI178446, SGI178447, SGI178448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178449, SGI0, SGI0, SGI0, SGI178450, SGI178450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39546[] = {SGI178451, SGI0, SGI178452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39547[] = {SGI178453, SGI178454, SGI178455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178456, SGI0, SGI0, SGI0, SGI178457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39548[] = {SGI178458, SGI0, SGI178459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178462, SGI178463, SGI178463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39549[] = {SGI178460, SGI0, SGI178461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39550[] = {SGI178464, SGI0, SGI178465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178466, SGI178467, SGI178467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39551[] = {SGI178468, SGI178469, SGI178470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178473, SGI0, SGI0, SGI0, SGI178474, SGI178474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39552[] = {SGI178471, SGI0, SGI178472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178475, SGI178476, SGI178476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39553[] = {SGI178479, SGI0, SGI178480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39554[] = {SGI178477, SGI0, SGI178478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178481, SGI178482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39555[] = {SGI178485, SGI178487, SGI178488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178490, SGI0, SGI0, SGI0, SGI178492, SGI178492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39556[] = {SGI178483, SGI178484, SGI178486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178489, SGI0, SGI0, SGI0, SGI178491, SGI178491, SGI0, SGI178493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39557[] = {SGI178494, SGI0, SGI178495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178496, SGI178497, SGI178497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39558[] = {SGI178498, SGI178499, SGI178500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178501, SGI0, SGI0, SGI0, SGI178502, SGI178502, SGI0, SGI178503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39559[] = {SGI178504, SGI0, SGI178505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178506, SGI178507, SGI178507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39560[] = {SGI178509, SGI0, SGI178511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178514, SGI178515, SGI178515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39561[] = {SGI178508, SGI178510, SGI178512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178513, SGI0, SGI0, SGI0, SGI178516, SGI178516, SGI0, SGI178517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39562[] = {SGI178518, SGI178519, SGI178520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178521, SGI0, SGI0, SGI0, SGI178522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39563[] = {SGI178523, SGI0, SGI178524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39564[] = {SGI178525, SGI0, SGI178526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178527, SGI178528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39565[] = {SGI178529, SGI178530, SGI178531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178532, SGI0, SGI0, SGI0, SGI178533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39566[] = {SGI178534, SGI178535, SGI178536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178537, SGI0, SGI0, SGI0, SGI178538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39567[] = {SGI178539, SGI0, SGI178540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178541, SGI178542, SGI178542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39568[] = {SGI178543, SGI0, SGI178544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178545, SGI178546, SGI178546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39569[] = {SGI178547, SGI0, SGI178548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178549, SGI178550, SGI178550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39570[] = {SGI178551, SGI0, SGI178552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178553, SGI178554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39571[] = {SGI178555, SGI0, SGI178556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178557, SGI178558, SGI178558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39572[] = {SGI178559, SGI0, SGI178560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178561, SGI178562, SGI178562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39573[] = {SGI178563, SGI0, SGI178564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178565, SGI178566, SGI178566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39574[] = {SGI178567, SGI178568, SGI178569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178570, SGI0, SGI0, SGI0, SGI178571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39575[] = {SGI178572, SGI178573, SGI178574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178575, SGI0, SGI0, SGI0, SGI178576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39576[] = {SGI178577, SGI178578, SGI178579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178580, SGI0, SGI0, SGI0, SGI178581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39577[] = {SGI178582, SGI178583, SGI178584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178585, SGI0, SGI0, SGI0, SGI178586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39578[] = {SGI178587, SGI178588, SGI178589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178590, SGI0, SGI0, SGI0, SGI178591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39579[] = {SGI178592, SGI0, SGI178593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178594, SGI178595, SGI178595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39580[] = {SGI178596, SGI0, SGI178597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178598, SGI178599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39581[] = {SGI178600, SGI0, SGI178601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178602, SGI178603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39582[] = {SGI0, SGI178604, SGI178605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178606, SGI0, SGI0, SGI0, SGI178607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39583[] = {SGI178608, SGI0, SGI178609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178610, SGI178611, SGI178611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39584[] = {SGI178612, SGI0, SGI178613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178614, SGI178615, SGI178615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178616, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39585[] = {SGI178617, SGI0, SGI178618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178619, SGI178620, SGI178620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39586[] = {SGI178621, SGI0, SGI178622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178623, SGI178624, SGI178624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39587[] = {SGI178625, SGI0, SGI178626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178627, SGI178628, SGI178628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39588[] = {SGI0, SGI0, SGI178629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178633, SGI178634, SGI178634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39589[] = {SGI178630, SGI178631, SGI178632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178635, SGI0, SGI0, SGI0, SGI178636, SGI0, SGI0, SGI178637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39590[] = {SGI178638, SGI0, SGI178639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178640, SGI178641, SGI178641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39591[] = {SGI178642, SGI178643, SGI178644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178645, SGI0, SGI0, SGI0, SGI178646, SGI0, SGI0, SGI178647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39592[] = {SGI178648, SGI178649, SGI178650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178651, SGI0, SGI0, SGI0, SGI178652, SGI178652, SGI0, SGI178653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39593[] = {SGI178654, SGI0, SGI178655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178656, SGI0, SGI0, SGI0, SGI178657, SGI178657, SGI0, SGI0, SGI178658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39594[] = {SGI178659, SGI0, SGI178660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178661, SGI0, SGI0, SGI0, SGI178662, SGI178662, SGI0, SGI0, SGI178663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39595[] = {SGI178664, SGI0, SGI178665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178669, SGI178670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39596[] = {SGI178666, SGI178667, SGI178668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178673, SGI0, SGI0, SGI0, SGI178674, SGI178674, SGI0, SGI0, NULL};
const Segment * const SG39597[] = {SGI178671, SGI0, SGI178672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178675, SGI0, SGI0, SGI0, SGI178676, SGI178676, SGI0, SGI0, SGI178677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39598[] = {SGI178678, SGI0, SGI178679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178680, SGI178681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39599[] = {SGI178682, SGI0, SGI178683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178684, SGI178685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39600[] = {SGI178686, SGI0, SGI178687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178688, SGI178689, SGI178689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39601[] = {SGI178690, SGI0, SGI178691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178692, SGI178693, SGI178693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39602[] = {SGI178694, SGI0, SGI178695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178696, SGI178697, SGI178697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39603[] = {SGI178698, SGI0, SGI178699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178700, SGI178701, SGI178701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39604[] = {SGI178702, SGI0, SGI178703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178704, SGI178705, SGI178705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39605[] = {SGI178706, SGI0, SGI178707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178708, SGI178709, SGI178709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39606[] = {SGI178710, SGI178711, SGI178712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178713, SGI0, SGI0, SGI0, SGI178714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39607[] = {SGI178715, SGI0, SGI178716, SGI0, SGI178717, SGI178717, SGI0, NULL};
const Segment * const SG39608[] = {SGI178718, SGI0, SGI178719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178720, SGI178721, SGI178721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39609[] = {SGI178722, SGI0, SGI178723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178724, SGI178725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39610[] = {SGI178726, SGI0, SGI178727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178728, SGI178729, SGI178729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39611[] = {SGI178730, SGI0, SGI178731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178732, SGI178733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39612[] = {SGI178734, SGI0, SGI178735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178736, SGI178737, SGI178737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39613[] = {SGI178738, SGI178739, SGI178740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178741, SGI0, SGI0, SGI0, SGI178744, SGI178744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39614[] = {SGI178742, SGI0, SGI178743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178746, SGI178747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39615[] = {SGI178748, SGI178749, SGI178750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178751, SGI0, SGI0, SGI0, SGI178752, SGI178752, SGI0, SGI0, SGI0, SGI178753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39616[] = {SGI178754, SGI178755, SGI178756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178757, SGI0, SGI0, SGI0, SGI178758, SGI178758, SGI0, SGI0, SGI0, SGI178759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39617[] = {SGI178760, SGI178761, SGI178762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178763, SGI0, SGI0, SGI0, SGI178764, SGI178764, SGI0, SGI0, SGI0, SGI178765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39618[] = {SGI178766, SGI178767, SGI178768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178769, SGI0, SGI0, SGI0, SGI178770, SGI178770, SGI0, SGI0, SGI0, SGI178771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39619[] = {SGI178772, SGI178773, SGI178774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178775, SGI0, SGI0, SGI0, SGI178776, SGI178776, SGI0, SGI0, SGI0, SGI178777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39620[] = {SGI178778, SGI178779, SGI178780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178781, SGI0, SGI0, SGI0, SGI178782, SGI178782, SGI0, SGI0, SGI0, SGI178783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39621[] = {SGI178784, SGI0, SGI178785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178786, SGI178787, SGI178787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39622[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178788, SGI0, SGI0, SGI178789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39623[] = {SGI178790, SGI0, SGI178791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178792, SGI178793, SGI178793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39624[] = {SGI0, SGI178794, SGI178795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178796, SGI0, SGI0, SGI0, SGI0, SGI178797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39625[] = {SGI178798, SGI178799, SGI178800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178801, SGI178801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39626[] = {SGI178802, SGI178803, SGI178804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178805, SGI0, SGI0, SGI0, SGI178807, SGI178807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39627[] = {SGI0, SGI178806, SGI178808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178809, SGI0, SGI0, SGI0, SGI0, SGI178810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39628[] = {SGI178811, SGI0, SGI178812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178816, SGI178817, SGI178817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39629[] = {SGI178813, SGI178814, SGI178815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178820, SGI0, SGI0, SGI0, SGI178821, SGI178821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39630[] = {SGI0, SGI178818, SGI178819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178822, SGI0, SGI0, SGI0, SGI0, SGI178823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39631[] = {SGI178824, SGI0, SGI178825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178826, SGI178827, SGI178827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39632[] = {SGI0, SGI178828, SGI178829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178830, SGI0, SGI0, SGI0, SGI0, SGI178831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39633[] = {SGI178832, SGI178833, SGI178834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178835, SGI0, SGI0, SGI0, SGI178836, SGI178836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39634[] = {SGI0, SGI178837, SGI178838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178839, SGI0, SGI0, SGI0, SGI0, SGI178840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39635[] = {SGI0, SGI178841, SGI178842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178843, SGI0, SGI0, SGI0, SGI0, SGI178844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39636[] = {SGI0, SGI178845, SGI178846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178847, SGI0, SGI0, SGI0, SGI0, SGI178848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39637[] = {SGI178849, SGI0, SGI178850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39638[] = {SGI178851, SGI0, SGI178853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178856, SGI178858, SGI178858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39639[] = {SGI178852, SGI0, SGI178854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178855, SGI178857, SGI178857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39640[] = {SGI178859, SGI0, SGI178860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178861, SGI178862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39641[] = {SGI178863, SGI178865, SGI178866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178868, SGI0, SGI0, SGI0, SGI178869, SGI0, SGI0, SGI178872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39642[] = {SGI178864, SGI0, SGI178867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178870, SGI178871, SGI178871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39643[] = {SGI178873, SGI0, SGI178874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178875, SGI178876, SGI178876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39644[] = {SGI178877, SGI178878, SGI178879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178881, SGI0, SGI0, SGI0, SGI178883, SGI178883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178886, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39645[] = {SGI178880, SGI0, SGI178882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178887, SGI178888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39646[] = {SGI178884, SGI0, SGI178885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178889, SGI178890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178891, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39647[] = {SGI178892, SGI178893, SGI178894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178895, SGI0, SGI0, SGI0, SGI178896, SGI178896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178897, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39648[] = {SGI178898, SGI0, SGI178899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178900, SGI178901, SGI178901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39649[] = {SGI178902, SGI178903, SGI178904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178905, SGI0, SGI0, SGI0, SGI178906, SGI178906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178907, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39650[] = {SGI178908, SGI178909, SGI178910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178912, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39651[] = {SGI178913, SGI0, SGI178914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178915, SGI178916, SGI178916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39652[] = {SGI178917, SGI0, SGI178918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178919, SGI178920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39653[] = {SGI178921, SGI0, SGI178923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178925, SGI178926, SGI178926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39654[] = {SGI178922, SGI0, SGI178924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178927, SGI178928, SGI178928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39655[] = {SGI178929, SGI0, SGI178930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178931, SGI178932, SGI178932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39656[] = {SGI178933, SGI0, SGI178934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178935, SGI178936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39657[] = {SGI178937, SGI178938, SGI178939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178942, SGI0, SGI0, SGI0, SGI178943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39658[] = {SGI178940, SGI0, SGI178941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178944, SGI178945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39659[] = {SGI178946, SGI0, SGI178947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178948, SGI178949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39660[] = {SGI178950, SGI178951, SGI178952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178955, SGI0, SGI0, SGI0, SGI178956, SGI178956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39661[] = {SGI178953, SGI0, SGI178954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178957, SGI178958, SGI178958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39662[] = {SGI178959, SGI0, SGI178960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178961, SGI178962, SGI178962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39663[] = {SGI178963, SGI0, SGI178964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178965, SGI178966, SGI178966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39664[] = {SGI178967, SGI0, SGI178968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178969, SGI178970, SGI178970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39665[] = {SGI178971, SGI0, SGI178972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178973, SGI178974, SGI178974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39666[] = {SGI178975, SGI0, SGI178976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178977, SGI178978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39667[] = {SGI178979, SGI0, SGI178980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178981, SGI178982, SGI178982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39668[] = {SGI178983, SGI178984, SGI178985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178986, SGI0, SGI0, SGI0, SGI178987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39669[] = {SGI178988, SGI178989, SGI178990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178991, SGI0, SGI0, SGI0, SGI178992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39670[] = {SGI178993, SGI0, SGI178994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178995, SGI178996, SGI178996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39671[] = {SGI178997, SGI0, SGI178998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI178999, SGI179000, SGI179000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39672[] = {SGI179001, SGI0, SGI179002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179003, SGI179004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39673[] = {SGI179005, SGI179006, SGI179007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179010, SGI0, SGI0, SGI0, SGI179011, SGI179011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39674[] = {SGI179008, SGI0, SGI179009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179012, SGI179013, SGI179013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39675[] = {SGI179014, SGI0, SGI179015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179016, SGI179017, SGI179017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39676[] = {SGI179018, SGI0, SGI179019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179020, SGI179021, SGI179021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39677[] = {SGI179022, SGI0, SGI179023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179024, SGI179025, SGI179025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39678[] = {SGI179026, SGI0, SGI179027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179028, SGI179029, SGI179029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39679[] = {SGI179030, SGI0, SGI179031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179032, SGI179033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39680[] = {SGI179034, SGI0, SGI179035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179036, SGI179037, SGI179037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39681[] = {SGI179038, SGI0, SGI179039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179040, SGI179041, SGI179041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39682[] = {SGI179042, SGI0, SGI179043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179044, SGI179045, SGI179045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39683[] = {SGI179046, SGI0, SGI179047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179048, SGI179049, SGI179049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39684[] = {SGI179050, SGI0, SGI179051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179052, SGI179053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39685[] = {SGI179054, SGI0, SGI179055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179056, SGI179057, SGI179057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39686[] = {SGI179058, SGI0, SGI179059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179060, SGI179061, SGI179061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39687[] = {SGI179062, SGI0, SGI179063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179064, SGI179065, SGI179065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39688[] = {SGI179066, SGI0, SGI179067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179068, SGI179069, SGI179069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39689[] = {SGI179070, SGI0, SGI179071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179072, SGI179073, SGI179073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39690[] = {SGI179074, SGI0, SGI179075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179077, SGI179078, SGI179078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39691[] = {SGI179076, SGI179079, SGI179080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179081, SGI0, SGI0, SGI0, SGI179082, SGI179082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39692[] = {SGI179084, SGI0, SGI179085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179086, SGI179087, SGI179087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39693[] = {SGI179088, SGI179089, SGI179090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179092, SGI0, SGI0, SGI0, SGI179094, SGI179094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39694[] = {SGI179091, SGI0, SGI179093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179098, SGI179099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39695[] = {SGI179095, SGI0, SGI179097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179100, SGI179101, SGI179101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39696[] = {SGI179102, SGI179103, SGI179104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179105, SGI0, SGI0, SGI0, SGI179106, SGI179106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39697[] = {SGI179109, SGI0, SGI179112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179115, SGI179116, SGI179116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39698[] = {SGI179108, SGI179110, SGI179111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179113, SGI0, SGI0, SGI0, SGI179114, SGI179114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39699[] = {SGI179118, SGI0, SGI179119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179120, SGI179121, SGI179121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39700[] = {SGI179122, SGI0, SGI179123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179126, SGI179127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39701[] = {SGI179124, SGI0, SGI179125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179128, SGI179129, SGI179129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179130, SGI0, SGI0, NULL};
const Segment * const SG39702[] = {SGI179131, SGI0, SGI179132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179133, SGI179134, SGI179134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39703[] = {SGI179135, SGI0, SGI179136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179137, SGI179138, SGI179138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39704[] = {SGI179139, SGI0, SGI179140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179142, SGI179143, SGI179143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39705[] = {SGI179141, SGI0, SGI179144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179145, SGI179146, SGI179146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39706[] = {SGI179147, SGI0, SGI179148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179150, SGI179151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39707[] = {SGI179149, SGI0, SGI179152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179153, SGI179154, SGI179154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39708[] = {SGI179155, SGI0, SGI179156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179158, SGI179160, SGI179160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39709[] = {SGI179157, SGI0, SGI179159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179161, SGI179162, SGI179162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39710[] = {SGI179163, SGI179164, SGI179165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179166, SGI0, SGI0, SGI0, SGI179167, SGI179167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39711[] = {SGI179168, SGI0, SGI179169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39712[] = {SGI179170, SGI0, SGI179172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179174, SGI179176, SGI179176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39713[] = {SGI179171, SGI0, SGI179173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179175, SGI179177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39714[] = {SGI179178, SGI0, SGI179179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179182, SGI179183, SGI179183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39715[] = {SGI179180, SGI0, SGI179181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179185, SGI179187, SGI179187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39716[] = {SGI179184, SGI0, SGI179186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179188, SGI179189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39717[] = {SGI179190, SGI0, SGI179191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179194, SGI179195, SGI179195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39718[] = {SGI179192, SGI0, SGI179193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179196, SGI179197, SGI179197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39719[] = {SGI179198, SGI0, SGI179199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179202, SGI179203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39720[] = {SGI179200, SGI0, SGI179201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179204, SGI179205, SGI179205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39721[] = {SGI179206, SGI0, SGI179207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179208, SGI179209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39722[] = {SGI179210, SGI0, SGI179211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179212, SGI179213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39723[] = {SGI179214, SGI0, SGI179215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179216, SGI179217, SGI179217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39724[] = {SGI179218, SGI179219, SGI179220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179222, SGI0, SGI0, SGI0, SGI179224, SGI179224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39725[] = {SGI179221, SGI0, SGI179223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179225, SGI179226, SGI179226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39726[] = {SGI179227, SGI0, SGI179229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179231, SGI179232, SGI179232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39727[] = {SGI179228, SGI0, SGI179230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179233, SGI179234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39728[] = {SGI179235, SGI0, SGI179236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179237, SGI179238, SGI179238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39729[] = {SGI179239, SGI179240, SGI179241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179242, SGI179242, SGI0, SGI0, SGI0, SGI179243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39730[] = {SGI179246, SGI0, SGI179248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179250, SGI179251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39731[] = {SGI179244, SGI179245, SGI179247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179249, SGI179249, SGI0, SGI0, SGI0, SGI179252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39732[] = {SGI179253, SGI0, SGI179254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179255, SGI179256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39733[] = {SGI179257, SGI179258, SGI179259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179260, SGI0, SGI0, SGI0, SGI179261, SGI179261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39734[] = {SGI179262, SGI0, SGI179263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179264, SGI179265, SGI179265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39735[] = {SGI179266, SGI179267, SGI179268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179269, SGI0, SGI0, SGI0, SGI179270, SGI179270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39736[] = {SGI179271, SGI0, SGI179272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39737[] = {SGI179273, SGI0, SGI179274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179275, SGI179276, SGI179276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39738[] = {SGI179277, SGI179278, SGI179280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179283, SGI0, SGI0, SGI0, SGI179285, SGI0, SGI0, SGI179287, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39739[] = {SGI179279, SGI179281, SGI179282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179284, SGI0, SGI0, SGI0, SGI179286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39740[] = {SGI179288, SGI0, SGI179289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179293, SGI179294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39741[] = {SGI179290, SGI179291, SGI179292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179295, SGI0, SGI0, SGI0, SGI179296, SGI0, SGI0, SGI179297, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39742[] = {SGI179298, SGI179299, SGI179300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179302, SGI0, SGI0, SGI0, SGI179304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39743[] = {SGI179301, SGI0, SGI179303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179305, SGI179306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39744[] = {SGI179307, SGI179308, SGI179309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179313, SGI0, SGI0, SGI0, SGI179314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39745[] = {SGI179310, SGI179311, SGI179312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179315, SGI0, SGI0, SGI0, SGI179316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39746[] = {SGI179317, SGI0, SGI179318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179319, SGI179321, SGI179321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39747[] = {SGI179320, SGI179322, SGI179323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179325, SGI0, SGI0, SGI0, SGI179328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39748[] = {SGI179324, SGI179326, SGI179327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179329, SGI0, SGI0, SGI0, SGI179330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39749[] = {SGI179331, SGI179332, SGI179333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179334, SGI0, SGI0, SGI0, SGI179336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39750[] = {SGI179335, SGI0, SGI179337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179338, SGI179339, SGI179339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39751[] = {SGI179340, SGI0, SGI179341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179342, SGI179343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39752[] = {SGI179344, SGI0, SGI179345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179346, SGI179347, SGI179347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39753[] = {SGI179348, SGI0, SGI179349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179350, SGI179351, SGI179351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39754[] = {SGI179352, SGI0, SGI179353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179354, SGI179355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39755[] = {SGI179356, SGI0, SGI179357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179358, SGI179359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39756[] = {SGI179360, SGI0, SGI179361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179362, SGI179363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39757[] = {SGI179364, SGI0, SGI179365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179366, SGI179367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39758[] = {SGI179368, SGI0, SGI179369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179370, SGI179371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39759[] = {SGI179372, SGI0, SGI179373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179374, SGI179375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39760[] = {SGI179376, SGI0, SGI179377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179378, SGI179379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39761[] = {SGI179380, SGI0, SGI179381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179382, SGI179383, SGI179383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39762[] = {SGI179384, SGI0, SGI179385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179386, SGI179387, SGI179387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39763[] = {SGI179388, SGI0, SGI179389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179390, SGI179391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39764[] = {SGI179392, SGI0, SGI179393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179394, SGI179395, SGI179395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39765[] = {SGI179396, SGI0, SGI179397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179398, SGI179399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39766[] = {SGI179400, SGI0, SGI179401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179402, SGI0, SGI0, SGI0, SGI179403, SGI179403, SGI0, SGI0, SGI0, SGI179404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39767[] = {SGI179405, SGI179406, SGI179407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179408, SGI0, SGI0, SGI0, SGI179409, SGI179409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39768[] = {SGI179410, SGI179411, SGI179412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179413, SGI0, SGI0, SGI0, SGI179414, SGI179414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39769[] = {SGI179415, SGI0, SGI179416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39770[] = {SGI179417, SGI179418, SGI179419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179420, SGI0, SGI0, SGI0, SGI179421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39771[] = {SGI179422, SGI0, SGI179423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179424, SGI179425, SGI179425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39772[] = {SGI179426, SGI0, SGI179427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179428, SGI179429, SGI179429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39773[] = {SGI179430, SGI0, SGI179431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179432, SGI179433, SGI179433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39774[] = {SGI179434, SGI0, SGI179435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179436, SGI179437, SGI179437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39775[] = {SGI179438, SGI0, SGI179439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179440, SGI179441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39776[] = {SGI179442, SGI0, SGI179443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179444, SGI179445, SGI179445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39777[] = {SGI179446, SGI0, SGI179447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179448, SGI179449, SGI179449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39778[] = {SGI179450, SGI179451, SGI179452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179453, SGI0, SGI0, SGI0, SGI179454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39779[] = {SGI179455, SGI0, SGI179456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179457, SGI179458, SGI179458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39780[] = {SGI179459, SGI0, SGI179460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39781[] = {SGI179461, SGI179462, SGI179463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179464, SGI0, SGI0, SGI0, SGI179465, SGI179465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39782[] = {SGI179467, SGI179468, SGI179469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179470, SGI0, SGI0, SGI0, SGI179471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39783[] = {SGI179472, SGI179473, SGI179474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179475, SGI0, SGI0, SGI0, SGI179476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39784[] = {SGI179477, SGI179478, SGI179479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179480, SGI0, SGI0, SGI0, SGI179481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39785[] = {SGI179482, SGI179483, SGI179484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179485, SGI0, SGI0, SGI0, SGI179486, SGI179486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39786[] = {SGI179487, SGI179488, SGI179489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179490, SGI0, SGI0, SGI0, SGI179491, SGI179491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39787[] = {SGI179492, SGI179493, SGI179494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179495, SGI0, SGI0, SGI0, SGI179496, SGI179496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39788[] = {SGI179497, SGI0, SGI179498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179499, SGI179500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39789[] = {SGI179501, SGI179502, SGI179503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179504, SGI0, SGI0, SGI0, SGI179505, SGI179505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39790[] = {SGI179506, SGI179507, SGI179508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179509, SGI0, SGI0, SGI0, SGI179510, SGI179510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39791[] = {SGI179511, SGI0, SGI179512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179513, SGI179514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39792[] = {SGI179515, SGI179516, SGI179517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179518, SGI0, SGI0, SGI0, SGI179519, SGI179519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39793[] = {SGI179521, SGI0, SGI179522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179523, SGI179524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39794[] = {SGI179525, SGI0, SGI179526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179527, SGI179528, SGI179528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39795[] = {SGI179529, SGI0, SGI179530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179531, SGI179532, SGI179532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39796[] = {SGI179533, SGI0, SGI179534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179535, SGI179536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39797[] = {SGI179537, SGI179538, SGI179539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179540, SGI0, SGI0, SGI0, SGI179541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39798[] = {SGI179542, SGI179543, SGI179544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179545, SGI0, SGI0, SGI0, SGI179546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39799[] = {SGI179547, SGI179548, SGI179549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179550, SGI179550, SGI0, SGI0, SGI179551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39800[] = {SGI179552, SGI179553, SGI179554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179555, SGI179555, SGI0, SGI0, SGI179556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39801[] = {SGI179557, SGI179558, SGI179559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179560, SGI179560, SGI0, SGI0, SGI179561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39802[] = {SGI179562, SGI179563, SGI179564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179565, SGI179565, SGI0, SGI0, SGI179566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39803[] = {SGI179567, SGI179568, SGI179569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179570, SGI179570, SGI0, SGI0, SGI179571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39804[] = {SGI179572, SGI0, SGI179573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179574, SGI179575, SGI179575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39805[] = {SGI179576, SGI179577, SGI179578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179579, SGI0, SGI0, SGI0, SGI179580, SGI179580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39806[] = {SGI179581, SGI0, SGI179582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39807[] = {SGI179583, SGI0, SGI179584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179585, SGI179586, SGI179586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39808[] = {SGI179587, SGI0, SGI179588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179591, SGI179592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39809[] = {SGI179589, SGI0, SGI179590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179593, SGI179594, SGI179594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39810[] = {SGI179595, SGI0, SGI179596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179597, SGI179598, SGI179598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39811[] = {SGI179599, SGI0, SGI179600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179601, SGI179602, SGI179602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39812[] = {SGI179603, SGI0, SGI179604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179605, SGI179606, SGI179606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39813[] = {SGI179607, SGI0, SGI179608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179609, SGI179610, SGI179610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39814[] = {SGI179611, SGI179612, SGI179613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179614, SGI179614, SGI0, SGI0, SGI0, SGI179615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39815[] = {SGI179616, SGI179617, SGI179618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179619, SGI0, SGI0, SGI0, SGI179620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39816[] = {SGI179621, SGI0, SGI179622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179623, SGI179624, SGI179624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39817[] = {SGI179625, SGI0, SGI179626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179627, SGI179628, SGI179628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39818[] = {SGI179629, SGI179630, SGI179631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179632, SGI0, SGI0, SGI0, SGI179633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39819[] = {SGI179634, SGI0, SGI179635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39820[] = {SGI179636, SGI0, SGI179637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179638, SGI179639, SGI179639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39821[] = {SGI179640, SGI0, SGI179641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179642, SGI179643, SGI179643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39822[] = {SGI179644, SGI179645, SGI179646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179647, SGI0, SGI0, SGI0, SGI179648, SGI0, SGI0, SGI179649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39823[] = {SGI179650, SGI0, SGI179651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179652, SGI179653, SGI179653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179654, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39824[] = {SGI179655, SGI0, SGI179657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39825[] = {SGI179656, SGI0, SGI179658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179659, SGI179660, SGI179660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179661, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39826[] = {SGI179662, SGI179663, SGI179664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179665, SGI0, SGI0, SGI0, SGI179666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39827[] = {SGI179667, SGI0, SGI179668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179669, SGI179670, SGI179670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39828[] = {SGI179671, SGI0, SGI179672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179673, SGI179674, SGI179674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39829[] = {SGI179675, SGI0, SGI179676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179677, SGI179679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39830[] = {SGI179678, SGI179680, SGI179681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179682, SGI0, SGI0, SGI0, SGI179683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39831[] = {SGI179684, SGI0, SGI179685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179686, SGI179687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39832[] = {SGI179688, SGI179689, SGI179690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179691, SGI0, SGI0, SGI0, SGI179693, SGI179693, SGI0, SGI0, SGI0, SGI0, SGI179695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39833[] = {SGI179692, SGI0, SGI179694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179696, SGI179697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39834[] = {SGI179698, SGI179699, SGI179700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179701, SGI0, SGI0, SGI0, SGI179702, SGI179702, SGI0, SGI0, SGI0, SGI0, SGI179703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39835[] = {SGI179704, SGI0, SGI179705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179706, SGI179707, SGI179707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39836[] = {SGI179708, SGI0, SGI179709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179710, SGI179711, SGI179711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39837[] = {SGI179712, SGI179713, SGI179714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179715, SGI0, SGI0, SGI0, SGI179716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39838[] = {SGI179717, SGI179718, SGI179719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179720, SGI0, SGI0, SGI0, SGI179721, SGI179721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39839[] = {SGI179722, SGI0, SGI179723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39840[] = {SGI0, SGI0, SGI179724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179725, SGI179725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39841[] = {SGI179726, SGI179727, SGI179729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179731, SGI0, SGI0, SGI0, SGI179732, SGI179732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39842[] = {SGI179728, SGI0, SGI179730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179733, SGI179734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39843[] = {SGI179735, SGI179736, SGI179737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179738, SGI179738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39844[] = {SGI179739, SGI0, SGI179740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39845[] = {SGI179741, SGI0, SGI179742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179743, SGI179744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39846[] = {SGI179745, SGI0, SGI179746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179747, SGI179748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39847[] = {SGI179749, SGI0, SGI179750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179751, SGI179752, SGI179752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39848[] = {SGI179753, SGI0, SGI179754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179755, SGI179756, SGI179756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39849[] = {SGI179757, SGI0, SGI179758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179759, SGI179760, SGI179760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39850[] = {SGI179761, SGI179762, SGI179763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179766, SGI0, SGI0, SGI0, SGI179767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39851[] = {SGI179764, SGI0, SGI179765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179768, SGI179769, SGI179769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39852[] = {SGI179770, SGI0, SGI179771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179772, SGI179773, SGI179773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39853[] = {SGI179774, SGI0, SGI179775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179776, SGI179777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39854[] = {SGI179778, SGI0, SGI179779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179783, SGI179784, SGI179784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39855[] = {SGI179780, SGI179781, SGI179782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179785, SGI0, SGI0, SGI0, SGI179786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39856[] = {SGI179787, SGI0, SGI179788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179789, SGI179790, SGI179790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39857[] = {SGI179791, SGI179792, SGI179793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179794, SGI0, SGI0, SGI0, SGI179795, SGI179795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39858[] = {SGI179796, SGI0, SGI179797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179798, SGI179799, SGI179799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39859[] = {SGI179800, SGI0, SGI179801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179802, SGI179803, SGI179803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39860[] = {SGI179804, SGI0, SGI179805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179806, SGI179807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39861[] = {SGI179808, SGI0, SGI179809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179810, SGI179811, SGI179811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39862[] = {SGI179812, SGI179813, SGI179814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179815, SGI0, SGI0, SGI0, SGI179816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39863[] = {SGI179817, SGI0, SGI179818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179819, SGI179820, SGI179820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39864[] = {SGI179821, SGI0, SGI179822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179823, SGI179824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39865[] = {SGI179825, SGI0, SGI179826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179827, SGI179828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39866[] = {SGI179829, SGI0, SGI179830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179831, SGI179832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39867[] = {SGI179833, SGI0, SGI179834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179835, SGI179836, SGI179836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39868[] = {SGI179837, SGI179838, SGI179839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179840, SGI0, SGI0, SGI0, SGI179841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39869[] = {SGI179842, SGI0, SGI179843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39870[] = {SGI179844, SGI0, SGI179845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179846, SGI179847, SGI179847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39871[] = {SGI179848, SGI179849, SGI179850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179851, SGI0, SGI0, SGI0, SGI179853, SGI179853, SGI0, SGI179855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39872[] = {SGI179852, SGI0, SGI179854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179856, SGI179857, SGI179857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39873[] = {SGI179858, SGI179859, SGI179860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179861, SGI0, SGI0, SGI0, SGI179862, SGI179862, SGI0, SGI179863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39874[] = {SGI179864, SGI179865, SGI179866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179867, SGI0, SGI0, SGI0, SGI179868, SGI179868, SGI0, SGI179869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39875[] = {SGI179870, SGI179871, SGI179872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179873, SGI0, SGI0, SGI0, SGI179874, SGI179874, SGI0, SGI179875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39876[] = {SGI179876, SGI179877, SGI179878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179879, SGI0, SGI0, SGI0, SGI179880, SGI179880, SGI0, SGI179881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39877[] = {SGI179882, SGI179883, SGI179884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179885, SGI0, SGI0, SGI0, SGI179886, SGI179886, SGI0, SGI179887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39878[] = {SGI179888, SGI179889, SGI179890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179891, SGI0, SGI0, SGI0, SGI179892, SGI179892, SGI0, SGI179893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39879[] = {SGI179894, SGI179895, SGI179896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179897, SGI0, SGI0, SGI0, SGI179898, SGI179898, SGI0, SGI179899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39880[] = {SGI179900, SGI179901, SGI179902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179903, SGI0, SGI0, SGI0, SGI179904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39881[] = {SGI179906, SGI179908, SGI179910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179912, SGI0, SGI0, SGI0, SGI179914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39882[] = {SGI179905, SGI179907, SGI179909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179911, SGI0, SGI0, SGI0, SGI179913, SGI179913, SGI0, SGI179915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39883[] = {SGI179916, SGI179917, SGI179918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179919, SGI0, SGI0, SGI0, SGI179920, SGI179920, SGI0, SGI179921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39884[] = {SGI179922, SGI179923, SGI179924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179925, SGI0, SGI0, SGI0, SGI179926, SGI179926, SGI0, SGI179927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39885[] = {SGI179928, SGI0, SGI179929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179930, SGI179931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39886[] = {SGI179932, SGI179933, SGI179934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179935, SGI0, SGI0, SGI0, SGI179936, SGI179936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39887[] = {SGI179937, SGI0, SGI179938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179939, SGI179940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39888[] = {SGI179941, SGI0, SGI179942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39889[] = {SGI179943, SGI0, SGI179944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179945, SGI179946, SGI179946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39890[] = {SGI179947, SGI179948, SGI179949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179950, SGI0, SGI0, SGI0, SGI179951, SGI179951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39891[] = {SGI179952, SGI0, SGI179953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179954, SGI179955, SGI179955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39892[] = {SGI179956, SGI179957, SGI179958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179959, SGI179959, SGI0, SGI0, SGI0, SGI179960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39893[] = {SGI179961, SGI179962, SGI179963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179964, SGI179964, SGI0, SGI0, SGI0, SGI179967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39894[] = {SGI179965, SGI179966, SGI179968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179969, SGI0, SGI0, SGI0, SGI179970, SGI179970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39895[] = {SGI179974, SGI0, SGI179975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39896[] = {SGI179971, SGI179972, SGI179973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179976, SGI179976, SGI0, SGI0, SGI0, SGI179977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39897[] = {SGI179978, SGI179979, SGI179980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179981, SGI0, SGI0, SGI0, SGI179982, SGI179982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39898[] = {SGI179983, SGI179984, SGI179985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179986, SGI0, SGI0, SGI0, SGI179987, SGI179987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39899[] = {SGI179988, SGI179989, SGI179990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179991, SGI0, SGI0, SGI0, SGI179992, SGI179992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39900[] = {SGI179993, SGI179994, SGI179995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI179999, SGI0, SGI0, SGI0, SGI180000, SGI180000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39901[] = {SGI179996, SGI179997, SGI179998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180001, SGI0, SGI0, SGI0, SGI180002, SGI180002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180003, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39902[] = {SGI180004, SGI0, SGI180005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39903[] = {SGI180006, SGI180007, SGI180008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180009, SGI0, SGI0, SGI0, SGI180010, SGI180010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39904[] = {SGI180011, SGI0, SGI180012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180013, SGI180014, SGI180014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39905[] = {SGI180015, SGI180016, SGI180017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180018, SGI0, SGI0, SGI0, SGI180019, SGI0, SGI0, SGI180020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39906[] = {SGI180021, SGI0, SGI180022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180023, SGI180024, SGI180024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39907[] = {SGI180025, SGI0, SGI180026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180027, SGI180028, SGI180028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39908[] = {SGI180029, SGI180030, SGI180031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180032, SGI0, SGI0, SGI0, SGI180033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39909[] = {SGI180034, SGI180035, SGI180036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180037, SGI0, SGI0, SGI0, SGI180038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39910[] = {SGI180039, SGI180040, SGI180041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180042, SGI0, SGI0, SGI0, SGI180043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39911[] = {SGI180044, SGI180045, SGI180046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180047, SGI0, SGI0, SGI0, SGI180048, SGI180048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39912[] = {SGI180049, SGI180050, SGI180051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180052, SGI0, SGI0, SGI0, SGI180053, SGI180053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39913[] = {SGI180054, SGI180056, SGI180057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180058, SGI0, SGI0, SGI0, SGI180059, SGI180059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39914[] = {SGI180060, SGI180061, SGI180062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180063, SGI0, SGI0, SGI0, SGI180064, SGI180064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39915[] = {SGI180065, SGI180066, SGI180068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180069, SGI0, SGI0, SGI0, SGI180070, SGI180070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39916[] = {SGI180071, SGI180072, SGI180073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180077, SGI0, SGI0, SGI0, SGI180078, SGI180078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39917[] = {SGI180074, SGI180075, SGI180076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180079, SGI0, SGI0, SGI0, SGI180080, SGI180080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39918[] = {SGI180082, SGI180083, SGI180084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180085, SGI0, SGI0, SGI0, SGI180086, SGI180086, SGI0, SGI0, SGI0, SGI0, SGI180087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39919[] = {SGI180088, SGI180089, SGI180090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180091, SGI0, SGI0, SGI0, SGI180092, SGI180092, SGI0, SGI0, SGI0, SGI0, SGI180093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39920[] = {SGI180094, SGI180095, SGI180096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180097, SGI0, SGI0, SGI0, SGI180098, SGI180098, SGI0, SGI0, SGI0, SGI0, SGI180099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39921[] = {SGI180100, SGI0, SGI180101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180102, SGI180103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39922[] = {SGI180104, SGI180105, SGI180106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180107, SGI0, SGI0, SGI0, SGI180108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39923[] = {SGI180109, SGI0, SGI180110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180111, SGI180112, SGI180112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39924[] = {SGI180114, SGI180116, SGI180118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180120, SGI0, SGI0, SGI0, SGI180122, SGI180122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39925[] = {SGI180113, SGI180115, SGI180117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180119, SGI0, SGI0, SGI0, SGI180121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39926[] = {SGI180124, SGI0, SGI180125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39927[] = {SGI180126, SGI180127, SGI180128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180129, SGI0, SGI0, SGI0, SGI180130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39928[] = {SGI180132, SGI180133, SGI180134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180135, SGI0, SGI0, SGI0, SGI180136, SGI180136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39929[] = {SGI180137, SGI180138, SGI180139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180140, SGI0, SGI0, SGI0, SGI180141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39930[] = {SGI180143, SGI0, SGI180146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180149, SGI180150, SGI180150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39931[] = {SGI180144, SGI180145, SGI180147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180148, SGI0, SGI0, SGI0, SGI180151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39932[] = {SGI180152, SGI0, SGI180154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39933[] = {SGI180153, SGI0, SGI180155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180156, SGI180157, SGI180157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39934[] = {SGI180158, SGI0, SGI180159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180160, SGI180161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39935[] = {SGI180162, SGI0, SGI180163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180164, SGI180165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39936[] = {SGI180166, SGI0, SGI180167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180168, SGI180169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39937[] = {SGI180170, SGI0, SGI180171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180175, SGI180176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39938[] = {SGI180172, SGI180173, SGI180174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180177, SGI0, SGI0, SGI0, SGI180178, SGI180178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39939[] = {SGI180179, SGI0, SGI180180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180181, SGI180182, SGI180182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39940[] = {SGI180183, SGI0, SGI180184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180185, SGI180186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39941[] = {SGI180187, SGI0, SGI180188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180189, SGI180190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39942[] = {SGI180191, SGI0, SGI180192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180193, SGI180194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39943[] = {SGI180195, SGI0, SGI180196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180197, SGI180198, SGI180198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39944[] = {SGI180199, SGI0, SGI180200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180201, SGI180202, SGI180202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39945[] = {SGI180203, SGI180204, SGI180205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180206, SGI0, SGI0, SGI0, SGI180207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39946[] = {SGI180209, SGI0, SGI180210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180214, SGI180215, SGI180215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39947[] = {SGI180211, SGI180212, SGI180213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180216, SGI0, SGI0, SGI0, SGI180217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39948[] = {SGI180219, SGI180220, SGI180221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180222, SGI0, SGI0, SGI0, SGI180223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39949[] = {SGI180225, SGI180226, SGI180227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180228, SGI0, SGI0, SGI0, SGI180229, SGI180229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39950[] = {SGI180231, SGI180232, SGI180233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180234, SGI0, SGI0, SGI0, SGI180235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39951[] = {SGI180237, SGI180238, SGI180239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180240, SGI0, SGI0, SGI0, SGI180241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39952[] = {SGI180242, SGI180243, SGI180244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180245, SGI0, SGI0, SGI0, SGI180246, SGI180246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39953[] = {SGI180247, SGI0, SGI180248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180249, SGI180250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39954[] = {SGI180251, SGI180252, SGI180253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180254, SGI0, SGI0, SGI0, SGI180255, SGI180255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39955[] = {SGI180256, SGI0, SGI180257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39956[] = {SGI180258, SGI180259, SGI180260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180263, SGI180263, SGI0, SGI0, SGI0, SGI0, SGI180265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39957[] = {SGI180261, SGI0, SGI180262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180264, SGI180266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39958[] = {SGI180267, SGI0, SGI180269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180272, SGI180273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39959[] = {SGI180268, SGI180270, SGI180271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180274, SGI180274, SGI0, SGI0, SGI0, SGI0, SGI180275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39960[] = {SGI180276, SGI180277, SGI180278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180279, SGI180279, SGI0, SGI0, SGI0, SGI0, SGI180280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39961[] = {SGI180281, SGI180282, SGI180283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180284, SGI180284, SGI0, SGI0, SGI0, SGI0, SGI180285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39962[] = {SGI180286, SGI0, SGI180287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180288, SGI180289, SGI180289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39963[] = {SGI180290, SGI0, SGI180291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180292, SGI180293, SGI180293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39964[] = {SGI180294, SGI0, SGI180295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180296, SGI180297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39965[] = {SGI180298, SGI0, SGI180299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180300, SGI180301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39966[] = {SGI180302, SGI180303, SGI180304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180305, SGI0, SGI0, SGI0, SGI180306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39967[] = {SGI180307, SGI180308, SGI180309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180310, SGI0, SGI0, SGI0, SGI180311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39968[] = {SGI180312, SGI180313, SGI180314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180315, SGI0, SGI0, SGI0, SGI180316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39969[] = {SGI180317, SGI0, SGI180318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180319, SGI180320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39970[] = {SGI180321, SGI0, SGI180322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180323, SGI180324, SGI180324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39971[] = {SGI180325, SGI0, SGI180326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180327, SGI180328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39972[] = {SGI180329, SGI0, SGI180330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180331, SGI180332, SGI180332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39973[] = {SGI180333, SGI0, SGI180334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180335, SGI180336, SGI180336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39974[] = {SGI180337, SGI0, SGI180338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180339, SGI180340, SGI180340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39975[] = {SGI180341, SGI180342, SGI180343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180344, SGI0, SGI0, SGI0, SGI180345, SGI180345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39976[] = {SGI180346, SGI0, SGI180347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180348, SGI180349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39977[] = {SGI180350, SGI0, SGI180351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180352, SGI180353, SGI180353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39978[] = {SGI180354, SGI0, SGI180355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180356, SGI180357, SGI180357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39979[] = {SGI180358, SGI180359, SGI180360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180361, SGI0, SGI0, SGI0, SGI180362, SGI180362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39980[] = {SGI180363, SGI0, SGI180364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180365, SGI180366, SGI180366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39981[] = {SGI180367, SGI0, SGI180368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180369, SGI180370, SGI180370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39982[] = {SGI180371, SGI180372, SGI180373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180374, SGI0, SGI0, SGI0, SGI180375, SGI180375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39983[] = {SGI180376, SGI0, SGI180377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39984[] = {SGI180378, SGI0, SGI180379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180380, SGI180381, SGI180381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39985[] = {SGI180382, SGI180383, SGI180384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180385, SGI0, SGI0, SGI0, SGI180386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39986[] = {SGI180387, SGI0, SGI180388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180389, SGI180390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39987[] = {SGI180391, SGI0, SGI180392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180393, SGI180394, SGI180394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39988[] = {SGI180395, SGI0, SGI180396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180397, SGI0, SGI0, SGI0, SGI180398, SGI180398, SGI0, SGI0, SGI0, SGI180399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39989[] = {SGI180400, SGI0, SGI180401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180402, SGI0, SGI0, SGI0, SGI180403, SGI180403, SGI0, SGI0, SGI0, SGI180404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39990[] = {SGI180405, SGI0, SGI180406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180407, SGI180408, SGI180408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39991[] = {SGI180409, SGI0, SGI180410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180411, SGI180412, SGI180412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39992[] = {SGI180413, SGI0, SGI180414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180415, SGI180416, SGI180416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39993[] = {SGI180417, SGI0, SGI180418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180419, SGI180420, SGI180420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39994[] = {SGI180421, SGI0, SGI180422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180425, SGI180426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39995[] = {SGI180423, SGI0, SGI180424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180427, SGI180428, SGI180428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39996[] = {SGI180429, SGI0, SGI180430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180433, SGI180434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39997[] = {SGI180431, SGI0, SGI180432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180435, SGI180436, SGI180436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39998[] = {SGI0, SGI0, SGI180438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180440, SGI180442, SGI180442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG39999[] = {SGI180437, SGI0, SGI180439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180441, SGI180443, SGI180443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40000[] = {SGI180444, SGI0, SGI180445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180446, SGI180447, SGI180447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40001[] = {SGI180448, SGI0, SGI180449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180450, SGI180451, SGI180451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40002[] = {SGI180452, SGI0, SGI180453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180454, SGI180455, SGI180455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40003[] = {SGI180456, SGI180457, SGI180458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180461, SGI0, SGI0, SGI0, SGI180462, SGI180462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40004[] = {SGI180459, SGI0, SGI180460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180463, SGI180464, SGI180464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40005[] = {SGI180465, SGI180466, SGI180467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180468, SGI0, SGI0, SGI0, SGI180469, SGI180469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40006[] = {SGI180470, SGI180471, SGI180472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180473, SGI0, SGI0, SGI0, SGI180474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40007[] = {SGI180475, SGI0, SGI180476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180477, SGI180478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40008[] = {SGI180479, SGI0, SGI180480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180481, SGI180482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40009[] = {SGI180483, SGI180484, SGI180485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180486, SGI0, SGI0, SGI0, SGI180487, SGI180487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40010[] = {SGI180488, SGI180489, SGI180490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180491, SGI0, SGI0, SGI0, SGI180492, SGI180492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40011[] = {SGI180493, SGI0, SGI180494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180495, SGI180496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40012[] = {SGI180497, SGI180498, SGI180499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180500, SGI0, SGI0, SGI0, SGI180501, SGI180501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40013[] = {SGI180502, SGI180503, SGI180504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180505, SGI0, SGI0, SGI0, SGI180506, SGI180506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40014[] = {SGI0, SGI180507, SGI180509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180511, SGI0, SGI0, SGI0, SGI180512, SGI180512, SGI0, SGI0, NULL};
const Segment * const SG40015[] = {SGI180508, SGI0, SGI180510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180513, SGI180514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40016[] = {SGI180515, SGI0, SGI180516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180517, SGI180518, SGI180518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40017[] = {SGI180519, SGI0, SGI180520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180521, SGI180522, SGI180522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180523, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40018[] = {SGI180524, SGI0, SGI180525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180526, SGI180527, SGI180527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180528, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40019[] = {SGI180529, SGI0, SGI180530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180531, SGI180532, SGI180532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40020[] = {SGI180533, SGI0, SGI180534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180535, SGI180536, SGI180536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40021[] = {SGI180537, SGI0, SGI180538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180539, SGI180540, SGI180540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40022[] = {SGI180541, SGI180542, SGI180543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180544, SGI0, SGI0, SGI0, SGI180545, SGI180545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40023[] = {SGI180546, SGI180547, SGI180548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180549, SGI0, SGI0, SGI0, SGI180550, SGI180550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40024[] = {SGI180551, SGI0, SGI180552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180553, SGI180554, SGI180554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40025[] = {SGI180555, SGI0, SGI180556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180557, SGI180558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40026[] = {SGI180559, SGI0, SGI180560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180561, SGI180562, SGI180562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40027[] = {SGI180563, SGI180564, SGI180565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180566, SGI0, SGI0, SGI0, SGI180567, SGI180567, SGI0, SGI180568, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40028[] = {SGI180569, SGI0, SGI180570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180571, SGI180572, SGI180572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40029[] = {SGI180573, SGI180574, SGI180575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180576, SGI0, SGI0, SGI0, SGI180577, SGI180577, SGI0, SGI180578, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40030[] = {SGI180579, SGI0, SGI180580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180581, SGI180582, SGI180582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40031[] = {SGI180583, SGI0, SGI180584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180585, SGI180586, SGI180586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40032[] = {SGI180587, SGI0, SGI180588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180589, SGI180590, SGI180590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40033[] = {SGI180591, SGI180592, SGI180593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180594, SGI0, SGI0, SGI0, SGI180595, SGI180595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40034[] = {SGI180597, SGI180598, SGI180599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180600, SGI0, SGI0, SGI0, SGI180601, SGI180601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40035[] = {SGI180602, SGI180603, SGI180604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180605, SGI0, SGI0, SGI0, SGI180606, SGI180606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40036[] = {SGI180607, SGI0, SGI180608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40037[] = {SGI180609, SGI180610, SGI180611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180612, SGI0, SGI0, SGI0, SGI180613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40038[] = {SGI180614, SGI0, SGI180615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40039[] = {SGI180616, SGI180617, SGI180618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180619, SGI0, SGI0, SGI0, SGI180620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40040[] = {SGI180621, SGI0, SGI180622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180623, SGI180624, SGI180624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40041[] = {SGI180625, SGI0, SGI180626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180627, SGI180628, SGI180628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40042[] = {SGI180629, SGI0, SGI180630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180631, SGI180632, SGI180632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40043[] = {SGI180633, SGI0, SGI180634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180635, SGI180636, SGI180636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40044[] = {SGI180637, SGI0, SGI180638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180639, SGI180640, SGI180640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40045[] = {SGI180641, SGI180642, SGI180643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180644, SGI0, SGI0, SGI180645, SGI180646, SGI180646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40046[] = {SGI180647, SGI180648, SGI180649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180650, SGI180650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40047[] = {SGI180651, SGI180652, SGI180653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180654, SGI0, SGI0, SGI0, SGI180656, SGI180656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40048[] = {SGI180655, SGI180657, SGI180658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180659, SGI0, SGI0, SGI0, SGI180660, SGI180660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40049[] = {SGI180661, SGI180662, SGI180663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180667, SGI0, SGI0, SGI0, SGI180668, SGI180668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40050[] = {SGI180664, SGI180665, SGI180666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180669, SGI0, SGI0, SGI0, SGI180672, SGI180672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40051[] = {SGI180670, SGI180671, SGI180673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180674, SGI0, SGI0, SGI0, SGI180675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40052[] = {SGI180676, SGI180677, SGI180678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180679, SGI0, SGI0, SGI0, SGI180680, SGI180680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40053[] = {SGI180681, SGI180682, SGI180683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180684, SGI0, SGI0, SGI0, SGI180685, SGI180685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40054[] = {SGI180686, SGI180687, SGI180688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180689, SGI0, SGI0, SGI0, SGI180690, SGI180690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40055[] = {SGI180691, SGI180692, SGI180693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180694, SGI0, SGI0, SGI0, SGI180695, SGI180695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40056[] = {SGI180696, SGI180697, SGI180698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180699, SGI0, SGI0, SGI0, SGI180700, SGI180700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40057[] = {SGI180701, SGI180702, SGI180703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180704, SGI0, SGI0, SGI0, SGI180705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40058[] = {SGI180706, SGI180707, SGI180708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180709, SGI0, SGI0, SGI0, SGI180710, SGI180710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40059[] = {SGI180711, SGI180712, SGI180713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180714, SGI0, SGI0, SGI0, SGI180715, SGI180715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40060[] = {SGI180716, SGI180717, SGI180718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180719, SGI0, SGI0, SGI0, SGI180720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40061[] = {SGI180721, SGI180722, SGI180723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180724, SGI0, SGI0, SGI0, SGI180725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40062[] = {SGI180726, SGI180727, SGI180728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180729, SGI0, SGI0, SGI0, SGI180730, SGI180730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40063[] = {SGI180734, SGI0, SGI180735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40064[] = {SGI180731, SGI180732, SGI180733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180736, SGI0, SGI0, SGI0, SGI180737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40065[] = {SGI180739, SGI180740, SGI180741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180742, SGI0, SGI0, SGI0, SGI180743, SGI180743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40066[] = {SGI180745, SGI180746, SGI180747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180748, SGI0, SGI0, SGI0, SGI180749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40067[] = {SGI180751, SGI180752, SGI180753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180754, SGI0, SGI0, SGI0, SGI180755, SGI180755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40068[] = {SGI180757, SGI180758, SGI180760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180763, SGI0, SGI0, SGI0, SGI180764, SGI180764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40069[] = {SGI180759, SGI180761, SGI180762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180765, SGI0, SGI0, SGI0, SGI180766, SGI180766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40070[] = {SGI180768, SGI0, SGI180769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40071[] = {SGI180770, SGI0, SGI180771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180772, SGI180773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40072[] = {SGI180774, SGI0, SGI180775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180776, SGI180777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40073[] = {SGI180778, SGI180779, SGI180780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180781, SGI0, SGI0, SGI0, SGI180782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40074[] = {SGI180783, SGI180784, SGI180785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180786, SGI0, SGI0, SGI0, SGI180787, SGI180787, SGI0, SGI180788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40075[] = {SGI180789, SGI180790, SGI180791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180794, SGI0, SGI0, SGI0, SGI180796, SGI180796, SGI0, SGI180797, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40076[] = {SGI180792, SGI180793, SGI180795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180798, SGI0, SGI0, SGI0, SGI180799, SGI180799, SGI0, SGI180800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40077[] = {SGI180801, SGI180802, SGI180803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180804, SGI0, SGI0, SGI0, SGI180805, SGI180805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40078[] = {SGI180806, SGI0, SGI180807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180809, SGI180812, SGI180812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40079[] = {SGI180811, SGI0, SGI180814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40080[] = {SGI180808, SGI180810, SGI180813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180815, SGI0, SGI0, SGI0, SGI180816, SGI180816, SGI0, SGI180817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40081[] = {SGI180818, SGI0, SGI180819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180820, SGI180821, SGI180821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40082[] = {SGI180822, SGI0, SGI180823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180824, SGI180825, SGI180825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40083[] = {SGI180826, SGI0, SGI180827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180828, SGI180829, SGI180829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40084[] = {SGI180830, SGI180831, SGI180832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180833, SGI0, SGI0, SGI0, SGI180834, SGI180834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40085[] = {SGI180835, SGI180836, SGI180837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180838, SGI0, SGI0, SGI0, SGI180839, SGI180839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40086[] = {SGI180840, SGI0, SGI180841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180842, SGI0, SGI0, SGI0, SGI180843, SGI180843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40087[] = {SGI180844, SGI0, SGI180845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180846, SGI180847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40088[] = {SGI180848, SGI0, SGI180849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40089[] = {SGI180850, SGI180851, SGI180852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180853, SGI0, SGI0, SGI0, SGI180854, SGI180854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40090[] = {SGI180855, SGI180856, SGI180857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180858, SGI180858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40091[] = {SGI180859, SGI180860, SGI180861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180862, SGI0, SGI0, SGI0, SGI180863, SGI180863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40092[] = {SGI180864, SGI0, SGI180865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40093[] = {SGI180866, SGI0, SGI180867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40094[] = {SGI180868, SGI0, SGI180869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180873, SGI180874, SGI180874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40095[] = {SGI180870, SGI180871, SGI180872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180875, SGI0, SGI0, SGI0, SGI180876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40096[] = {SGI180877, SGI0, SGI180878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180882, SGI180883, SGI180883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40097[] = {SGI180879, SGI180880, SGI180881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180884, SGI0, SGI0, SGI0, SGI180885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40098[] = {SGI180886, SGI180887, SGI180888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180889, SGI0, SGI0, SGI0, SGI180890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40099[] = {SGI180891, SGI0, SGI180892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180893, SGI180894, SGI180894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40100[] = {SGI180895, SGI0, SGI180896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180897, SGI180898, SGI180898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40101[] = {SGI180899, SGI0, SGI180900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180901, SGI180902, SGI180902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40102[] = {SGI180903, SGI0, SGI180904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180905, SGI180906, SGI180906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40103[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180907, SGI0, SGI0, SGI180908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40104[] = {SGI180909, SGI0, SGI180910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180911, SGI180912, SGI180912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40105[] = {SGI180913, SGI0, SGI180914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180915, SGI180916, SGI180916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40106[] = {SGI180917, SGI0, SGI180918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180919, SGI180920, SGI180920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40107[] = {SGI180921, SGI0, SGI180922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180923, SGI180924, SGI180924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40108[] = {SGI180925, SGI0, SGI180926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180927, SGI180928, SGI180928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40109[] = {SGI180929, SGI0, SGI180930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180931, SGI180932, SGI180932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40110[] = {SGI180933, SGI0, SGI180934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180935, SGI180936, SGI180936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40111[] = {SGI180937, SGI0, SGI180938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180939, SGI180940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40112[] = {SGI180941, SGI0, SGI180942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180943, SGI180944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40113[] = {SGI180945, SGI0, SGI180946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180947, SGI180948, SGI180948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40114[] = {SGI180949, SGI0, SGI180950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180951, SGI180952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40115[] = {SGI180953, SGI0, SGI180954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180955, SGI180956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40116[] = {SGI180957, SGI180958, SGI180959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180960, SGI0, SGI0, SGI0, SGI180961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40117[] = {SGI180962, SGI180963, SGI180964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180965, SGI0, SGI0, SGI0, SGI180966, SGI0, SGI180967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40118[] = {SGI180968, SGI180969, SGI180970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180971, SGI0, SGI0, SGI0, SGI180972, SGI0, SGI180973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40119[] = {SGI180974, SGI0, SGI180975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180976, SGI180977, SGI180977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40120[] = {SGI180978, SGI0, SGI180979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180980, SGI180981, SGI180981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40121[] = {SGI180982, SGI0, SGI180983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180984, SGI180985, SGI180985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40122[] = {SGI180986, SGI0, SGI180987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180988, SGI180989, SGI180989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40123[] = {SGI180990, SGI0, SGI180992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180994, SGI180996, SGI180996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40124[] = {SGI180991, SGI0, SGI180993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI180995, SGI180997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40125[] = {SGI180998, SGI0, SGI180999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181000, SGI181001, SGI181001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40126[] = {SGI181002, SGI181003, SGI181004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181005, SGI0, SGI0, SGI0, SGI181006, SGI181006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40127[] = {SGI181009, SGI181011, SGI181013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181015, SGI0, SGI0, SGI0, SGI181016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40128[] = {SGI181008, SGI181010, SGI181012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181014, SGI0, SGI0, SGI0, SGI181017, SGI181017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40129[] = {SGI181019, SGI181020, SGI181021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181022, SGI0, SGI0, SGI0, SGI181023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40130[] = {SGI181024, SGI181025, SGI181026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181027, SGI0, SGI0, SGI0, SGI181028, SGI181028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40131[] = {SGI181029, SGI0, SGI181030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181031, SGI181032, SGI181032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40132[] = {SGI181033, SGI181034, SGI181035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181036, SGI0, SGI0, SGI0, SGI181037, SGI181037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40133[] = {SGI181041, SGI0, SGI181042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40134[] = {SGI181045, SGI181046, SGI181047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181048, SGI0, SGI0, SGI0, SGI181049, SGI181049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40135[] = {SGI181050, SGI0, SGI181051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40136[] = {SGI181052, SGI181053, SGI181054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181055, SGI0, SGI0, SGI0, SGI181056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40137[] = {SGI181057, SGI0, SGI181058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40138[] = {SGI181059, SGI181060, SGI181061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181062, SGI0, SGI0, SGI0, SGI181063, SGI181063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40139[] = {SGI181064, SGI0, SGI181065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40140[] = {SGI181066, SGI0, SGI181067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181068, SGI181069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40141[] = {SGI181070, SGI0, SGI181071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181072, SGI181073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40142[] = {SGI181074, SGI181075, SGI181076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181077, SGI0, SGI0, SGI0, SGI181078, SGI181078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40143[] = {SGI181079, SGI0, SGI181080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40144[] = {SGI181081, SGI181082, SGI181083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181084, SGI0, SGI0, SGI0, SGI181085, SGI181085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40145[] = {SGI181086, SGI181087, SGI181088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181089, SGI0, SGI0, SGI0, SGI181090, SGI181090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40146[] = {SGI181091, SGI181092, SGI181093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181094, SGI0, SGI0, SGI0, SGI181095, SGI181095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40147[] = {SGI181096, SGI0, SGI181097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40148[] = {SGI181098, SGI0, SGI181099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181100, SGI181101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40149[] = {SGI181102, SGI181103, SGI181104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181105, SGI0, SGI0, SGI0, SGI181106, SGI181106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40150[] = {SGI181108, SGI181109, SGI181110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181111, SGI0, SGI0, SGI0, SGI181112, SGI181112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40151[] = {SGI181114, SGI181115, SGI181116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181117, SGI0, SGI0, SGI0, SGI181118, SGI181118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40152[] = {SGI181120, SGI181121, SGI181122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181123, SGI0, SGI0, SGI0, SGI181124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40153[] = {SGI181125, SGI181126, SGI181127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181128, SGI0, SGI0, SGI0, SGI181129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40154[] = {SGI181130, SGI181131, SGI181132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181133, SGI0, SGI0, SGI0, SGI181134, SGI181134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40155[] = {SGI181135, SGI0, SGI181136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181137, SGI181138, SGI181138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40156[] = {SGI181139, SGI181140, SGI181141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181142, SGI0, SGI0, SGI0, SGI181143, SGI181143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40157[] = {SGI181144, SGI0, SGI181145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40158[] = {SGI181146, SGI0, SGI181147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181148, SGI181149, SGI181149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40159[] = {SGI181150, SGI0, SGI181151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181152, SGI181153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40160[] = {SGI181154, SGI0, SGI181155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181156, SGI181157, SGI181157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40161[] = {SGI181158, SGI0, SGI181159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181163, SGI181164, SGI181164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40162[] = {SGI181160, SGI181161, SGI181162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181165, SGI0, SGI0, SGI0, SGI181166, SGI181166, SGI0, SGI181167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40163[] = {SGI181168, SGI181169, SGI181170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181171, SGI0, SGI0, SGI0, SGI181174, SGI181174, SGI0, SGI181175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40164[] = {SGI181172, SGI0, SGI181173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181176, SGI181177, SGI181177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40165[] = {SGI181178, SGI0, SGI181179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181182, SGI181183, SGI181183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40166[] = {SGI181180, SGI0, SGI181181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181184, SGI181185, SGI181185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40167[] = {SGI181186, SGI0, SGI181187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181190, SGI181191, SGI181191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40168[] = {SGI181188, SGI0, SGI181189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181192, SGI181193, SGI181193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40169[] = {SGI181194, SGI0, SGI181195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181196, SGI181197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40170[] = {SGI181198, SGI181199, SGI181200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181201, SGI0, SGI0, SGI0, SGI181202, SGI181202, SGI181203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40171[] = {SGI181204, SGI181205, SGI181206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181207, SGI0, SGI0, SGI0, SGI181208, SGI181208, SGI181209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40172[] = {SGI181210, SGI181211, SGI181212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181213, SGI0, SGI0, SGI0, SGI181214, SGI181214, SGI181215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40173[] = {SGI181216, SGI181217, SGI181218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181219, SGI0, SGI0, SGI0, SGI181220, SGI181220, SGI181221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40174[] = {SGI181222, SGI181223, SGI181224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181225, SGI0, SGI0, SGI0, SGI181226, SGI181226, SGI181227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40175[] = {SGI181228, SGI181229, SGI181230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181231, SGI0, SGI0, SGI0, SGI181232, SGI181232, SGI181233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40176[] = {SGI181234, SGI181235, SGI181236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181237, SGI0, SGI0, SGI0, SGI181238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40177[] = {SGI181239, SGI181240, SGI181241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181242, SGI0, SGI0, SGI0, SGI181243, SGI181243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40178[] = {SGI181244, SGI181245, SGI181246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181247, SGI0, SGI0, SGI0, SGI181248, SGI181248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40179[] = {SGI181249, SGI0, SGI181250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40180[] = {SGI181251, SGI0, SGI181252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181253, SGI181254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40181[] = {SGI181255, SGI181256, SGI181257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181258, SGI0, SGI0, SGI181259, SGI181260, SGI181260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40182[] = {SGI181261, SGI0, SGI181262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181263, SGI181264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40183[] = {SGI181265, SGI181266, SGI181267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181268, SGI0, SGI0, SGI181269, SGI181270, SGI181270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40184[] = {SGI181271, SGI0, SGI181272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181273, SGI181274, SGI181274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181275, SGI0, SGI0, NULL};
const Segment * const SG40185[] = {SGI181276, SGI0, SGI181277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181278, SGI181279, SGI181279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40186[] = {SGI181280, SGI0, SGI181281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181282, SGI181283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40187[] = {SGI181284, SGI181285, SGI181286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181287, SGI0, SGI0, SGI0, SGI181288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40188[] = {SGI181290, SGI181291, SGI181292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181293, SGI0, SGI0, SGI0, SGI181294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40189[] = {SGI181296, SGI0, SGI181297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181298, SGI181299, SGI181299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40190[] = {SGI181300, SGI0, SGI181301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181302, SGI181303, SGI181303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40191[] = {SGI181304, SGI181305, SGI181306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181307, SGI0, SGI0, SGI0, SGI181308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40192[] = {SGI181309, SGI181310, SGI181311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181312, SGI0, SGI0, SGI0, SGI181313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40193[] = {SGI181314, SGI0, SGI181315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40194[] = {SGI181316, SGI181317, SGI181318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181319, SGI0, SGI0, SGI0, SGI181320, SGI181320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40195[] = {SGI181321, SGI0, SGI181322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40196[] = {SGI181323, SGI0, SGI181324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181325, SGI181326, SGI181326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40197[] = {SGI181327, SGI0, SGI181328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181329, SGI181330, SGI181330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40198[] = {SGI181331, SGI0, SGI181332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181333, SGI181334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181335, SGI0, SGI0, NULL};
const Segment * const SG40199[] = {SGI181336, SGI0, SGI181337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181338, SGI181339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40200[] = {SGI181340, SGI0, SGI181341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181344, SGI181345, SGI181345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40201[] = {SGI181342, SGI0, SGI181343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181346, SGI181347, SGI181347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181348, SGI0, SGI0, NULL};
const Segment * const SG40202[] = {SGI181038, SGI181039, SGI181040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181043, SGI0, SGI0, SGI0, SGI181044, SGI181044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40203[] = {SGI181349, SGI0, SGI181350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181351, SGI181352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181353, SGI0, NULL};
const Segment * const SG40204[] = {SGI181354, SGI181355, SGI181356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181357, SGI0, SGI0, SGI0, SGI181358, SGI181358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40205[] = {SGI181359, SGI0, SGI181360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181361, SGI181362, SGI181362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181363, SGI0, SGI0, NULL};
const Segment * const SG40206[] = {SGI181364, SGI0, SGI181365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181366, SGI181367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40207[] = {SGI181368, SGI181369, SGI181370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181371, SGI0, SGI0, SGI0, SGI181372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40208[] = {SGI181373, SGI0, SGI181374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40209[] = {SGI181375, SGI0, SGI181376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181377, SGI181378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40210[] = {SGI181379, SGI181380, SGI181381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181382, SGI0, SGI0, SGI0, SGI181383, SGI181383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40211[] = {SGI181384, SGI0, SGI181385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40212[] = {SGI181386, SGI0, SGI181387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181389, SGI181391, SGI181391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181393, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40213[] = {SGI181388, SGI181390, SGI181392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181394, SGI0, SGI0, SGI0, SGI181395, SGI181395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40214[] = {SGI181396, SGI0, SGI181397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181398, SGI181399, SGI181399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181400, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40215[] = {SGI181401, SGI0, SGI181402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181403, SGI181404, SGI181404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40216[] = {SGI181405, SGI0, SGI181406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181407, SGI181408, SGI181408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40217[] = {SGI181409, SGI0, SGI181410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181411, SGI181412, SGI181412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40218[] = {SGI181413, SGI0, SGI181414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181415, SGI181416, SGI181416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40219[] = {SGI181417, SGI181418, SGI181419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181420, SGI0, SGI0, SGI0, SGI181421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40220[] = {SGI181422, SGI181423, SGI181424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181426, SGI0, SGI0, SGI0, SGI181428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40221[] = {SGI181425, SGI0, SGI181427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181429, SGI181430, SGI181430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40222[] = {SGI181431, SGI181433, SGI181434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181436, SGI0, SGI0, SGI0, SGI181438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40223[] = {SGI181432, SGI0, SGI181435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181437, SGI181439, SGI181439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40224[] = {SGI181440, SGI181441, SGI181442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181443, SGI0, SGI0, SGI0, SGI181444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40225[] = {SGI181445, SGI0, SGI181446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181447, SGI181448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40226[] = {SGI181449, SGI0, SGI181450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181451, SGI181452, SGI181452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40227[] = {SGI181453, SGI0, SGI181454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181457, SGI181458, SGI181458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40228[] = {SGI181455, SGI0, SGI181456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181459, SGI181460, SGI181460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40229[] = {SGI181461, SGI0, SGI181462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181463, SGI181464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40230[] = {SGI181465, SGI0, SGI181466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181467, SGI181468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40231[] = {SGI181469, SGI181470, SGI181471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181472, SGI0, SGI0, SGI0, SGI181473, SGI181473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40232[] = {SGI181474, SGI181475, SGI181476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181477, SGI0, SGI0, SGI0, SGI181478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40233[] = {SGI181479, SGI0, SGI181480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40234[] = {SGI181481, SGI181482, SGI181483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181484, SGI0, SGI0, SGI0, SGI181485, SGI181485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40235[] = {SGI181486, SGI181487, SGI181488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181489, SGI0, SGI0, SGI0, SGI181490, SGI181490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40236[] = {SGI181491, SGI181492, SGI181493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181494, SGI0, SGI0, SGI0, SGI181495, SGI181495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40237[] = {SGI181496, SGI181497, SGI181498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181501, SGI0, SGI0, SGI0, SGI181503, SGI181503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40238[] = {SGI181499, SGI181500, SGI181502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181504, SGI0, SGI0, SGI0, SGI181505, SGI181505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40239[] = {SGI181506, SGI181507, SGI181509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181512, SGI0, SGI0, SGI0, SGI181514, SGI181514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40240[] = {SGI181508, SGI181510, SGI181511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181513, SGI0, SGI0, SGI0, SGI181515, SGI181515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40241[] = {SGI181516, SGI0, SGI181517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181518, SGI181519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40242[] = {SGI181520, SGI181521, SGI181522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181523, SGI0, SGI0, SGI0, SGI181524, SGI181524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40243[] = {SGI181525, SGI0, SGI181526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181527, SGI181528, SGI181528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40244[] = {SGI181529, SGI0, SGI181531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181534, SGI181536, SGI181536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40245[] = {SGI181530, SGI181532, SGI181533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181535, SGI0, SGI0, SGI0, SGI181537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40246[] = {SGI181538, SGI0, SGI181539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181540, SGI181541, SGI181541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40247[] = {SGI181542, SGI181543, SGI181544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181545, SGI0, SGI0, SGI0, SGI181546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40248[] = {SGI181547, SGI181548, SGI181549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181550, SGI0, SGI0, SGI0, SGI181551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40249[] = {SGI181552, SGI0, SGI181553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181554, SGI181555, SGI181555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40250[] = {SGI181556, SGI0, SGI181557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40251[] = {SGI181558, SGI181559, SGI181560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40252[] = {SGI181561, SGI0, SGI181562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181563, SGI181564, SGI181564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40253[] = {SGI181565, SGI0, SGI181566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181567, SGI0, SGI0, SGI0, SGI181568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40254[] = {SGI181569, SGI181570, SGI181571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181574, SGI0, SGI0, SGI0, SGI181576, SGI181576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40255[] = {SGI181572, SGI181573, SGI181575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181577, SGI0, SGI0, SGI0, SGI181578, SGI181578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40256[] = {SGI181579, SGI181580, SGI181581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181582, SGI0, SGI0, SGI0, SGI181583, SGI181583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40257[] = {SGI181585, SGI0, SGI181587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181590, SGI181591, SGI181591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40258[] = {SGI181584, SGI181586, SGI181588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181589, SGI0, SGI0, SGI0, SGI181592, SGI181592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40259[] = {SGI181593, SGI181594, SGI181595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181596, SGI0, SGI0, SGI0, SGI181597, SGI181597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40260[] = {SGI181598, SGI181599, SGI181600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181601, SGI0, SGI0, SGI0, SGI181602, SGI181602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40261[] = {SGI181603, SGI181604, SGI181605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181606, SGI0, SGI0, SGI0, SGI181607, SGI181607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40262[] = {SGI181608, SGI181609, SGI181610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181611, SGI0, SGI0, SGI0, SGI181612, SGI181612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40263[] = {SGI181613, SGI0, SGI181614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181615, SGI181616, SGI181616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181617, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40264[] = {SGI181618, SGI0, SGI181619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181620, SGI181621, SGI181621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181622, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40265[] = {SGI181623, SGI181624, SGI181625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181626, SGI0, SGI0, SGI0, SGI181627, SGI181627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40266[] = {SGI181629, SGI181630, SGI181631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181632, SGI0, SGI0, SGI0, SGI181633, SGI181633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40267[] = {SGI181635, SGI181636, SGI181637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181638, SGI0, SGI0, SGI0, SGI181639, SGI181639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40268[] = {SGI181641, SGI181642, SGI181643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181644, SGI0, SGI0, SGI0, SGI181645, SGI181645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40269[] = {SGI181647, SGI181648, SGI181649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181650, SGI0, SGI0, SGI0, SGI181651, SGI181651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40270[] = {SGI181653, SGI181654, SGI181655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181656, SGI0, SGI0, SGI0, SGI181657, SGI181657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40271[] = {SGI181658, SGI181659, SGI181660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181661, SGI0, SGI0, SGI0, SGI181662, SGI181662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40272[] = {SGI181663, SGI181664, SGI181665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181666, SGI0, SGI0, SGI0, SGI181667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40273[] = {SGI181668, SGI181669, SGI181670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181671, SGI0, SGI0, SGI0, SGI181672, SGI181672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40274[] = {SGI181673, SGI181674, SGI181675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181676, SGI0, SGI0, SGI0, SGI181677, SGI181677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40275[] = {SGI181678, SGI0, SGI181679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181680, SGI181681, SGI181681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40276[] = {SGI181682, SGI0, SGI181683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181684, SGI181685, SGI181685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40277[] = {SGI181686, SGI181687, SGI181688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181689, SGI0, SGI0, SGI0, SGI181690, SGI181690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40278[] = {SGI181692, SGI181693, SGI181694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181695, SGI0, SGI0, SGI0, SGI181696, SGI181696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40279[] = {SGI181698, SGI181699, SGI181700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181701, SGI0, SGI0, SGI0, SGI181702, SGI181702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40280[] = {SGI181704, SGI0, SGI181705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181706, SGI181707, SGI181707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40281[] = {SGI181708, SGI0, SGI181709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181710, SGI181711, SGI181711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40282[] = {SGI181712, SGI0, SGI181713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181714, SGI181715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40283[] = {SGI181716, SGI0, SGI181717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181718, SGI181719, SGI181719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40284[] = {SGI181720, SGI0, SGI181721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181722, SGI181723, SGI181723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40285[] = {SGI181724, SGI0, SGI181725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181726, SGI181727, SGI181727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40286[] = {SGI181728, SGI0, SGI181729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181730, SGI181731, SGI181731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40287[] = {SGI181732, SGI181733, SGI181734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181735, SGI0, SGI0, SGI0, SGI181736, SGI181736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40288[] = {SGI181737, SGI0, SGI181738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181739, SGI0, SGI0, SGI0, SGI181740, SGI181740, SGI0, SGI0, SGI0, SGI181741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40289[] = {SGI181742, SGI0, SGI181743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181744, SGI181745, SGI181745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40290[] = {SGI181746, SGI0, SGI181747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181748, SGI181749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40291[] = {SGI181750, SGI0, SGI181751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181752, SGI181753, SGI181753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40292[] = {SGI181754, SGI0, SGI181755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40293[] = {SGI181756, SGI181757, SGI181758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181759, SGI0, SGI0, SGI0, SGI181760, SGI181760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40294[] = {SGI181761, SGI181763, SGI181765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181767, SGI0, SGI0, SGI0, SGI181769, SGI181769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40295[] = {SGI181762, SGI181764, SGI181766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181768, SGI0, SGI0, SGI0, SGI181770, SGI181770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40296[] = {SGI181772, SGI181774, SGI181775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181778, SGI0, SGI0, SGI0, SGI181780, SGI181780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40297[] = {SGI181773, SGI181776, SGI181777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181779, SGI0, SGI0, SGI0, SGI181781, SGI181781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40298[] = {SGI181783, SGI181784, SGI181785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181789, SGI0, SGI0, SGI0, SGI181790, SGI181790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40299[] = {SGI181786, SGI181787, SGI181788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181791, SGI0, SGI0, SGI0, SGI181792, SGI181792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40300[] = {SGI181794, SGI0, SGI181797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40301[] = {SGI181793, SGI181796, SGI181798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181800, SGI0, SGI0, SGI0, SGI181802, SGI181802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40302[] = {SGI181795, SGI0, SGI181799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181801, SGI181803, SGI181803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40303[] = {SGI181804, SGI0, SGI181805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181806, SGI181807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40304[] = {SGI181808, SGI0, SGI181809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181810, SGI181811, SGI181811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40305[] = {SGI181812, SGI181813, SGI181814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181815, SGI0, SGI0, SGI0, SGI181816, SGI0, SGI181817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40306[] = {SGI181818, SGI181819, SGI181820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181821, SGI0, SGI0, SGI0, SGI181822, SGI0, SGI181823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40307[] = {SGI181824, SGI0, SGI181825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181826, SGI181827, SGI181827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40308[] = {SGI181828, SGI181829, SGI181830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181832, SGI0, SGI0, SGI0, SGI181834, SGI0, SGI181835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40309[] = {SGI181831, SGI0, SGI181833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181836, SGI181837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40310[] = {SGI181838, SGI0, SGI181839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181840, SGI181841, SGI181841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40311[] = {SGI181842, SGI181843, SGI181844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181845, SGI181845, SGI0, SGI0, SGI0, SGI181847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40312[] = {SGI181846, SGI0, SGI181848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181849, SGI181850, SGI181850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40313[] = {SGI181851, SGI181852, SGI181853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181854, SGI181854, SGI0, SGI0, SGI0, SGI181855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40314[] = {SGI181856, SGI181857, SGI181858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181859, SGI0, SGI0, SGI0, SGI181860, SGI181860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40315[] = {SGI181861, SGI181862, SGI181863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181864, SGI0, SGI0, SGI0, SGI181865, SGI181865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40316[] = {SGI181866, SGI0, SGI181867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40317[] = {SGI181868, SGI181869, SGI181870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181871, SGI0, SGI0, SGI0, SGI181872, SGI181872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40318[] = {SGI181873, SGI0, SGI181874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181876, SGI181878, SGI181878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40319[] = {SGI181875, SGI0, SGI181877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181879, SGI181880, SGI181880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40320[] = {SGI181881, SGI0, SGI181882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181883, SGI181884, SGI181884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40321[] = {SGI181885, SGI0, SGI181886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181887, SGI181888, SGI181888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40322[] = {SGI181889, SGI0, SGI181890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181891, SGI181892, SGI181892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40323[] = {SGI181893, SGI0, SGI181894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181895, SGI181897, SGI181897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40324[] = {SGI181896, SGI0, SGI181898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181899, SGI181900, SGI181900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40325[] = {SGI181901, SGI0, SGI181902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181903, SGI181904, SGI181904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40326[] = {SGI181905, SGI0, SGI181906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181907, SGI181908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181909, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40327[] = {SGI181910, SGI0, SGI181911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181912, SGI181913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40328[] = {SGI181914, SGI0, SGI181915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181916, SGI181917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181919, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40329[] = {SGI181918, SGI0, SGI181920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181921, SGI181922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40330[] = {SGI181923, SGI0, SGI181924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181925, SGI181926, SGI181926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40331[] = {SGI181927, SGI0, SGI181928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181931, SGI181932, SGI181932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40332[] = {SGI181929, SGI0, SGI181930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181933, SGI181934, SGI181934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181935, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40333[] = {SGI181936, SGI0, SGI181937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181938, SGI181939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40334[] = {SGI181940, SGI0, SGI181941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181942, SGI181943, SGI181943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181944, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40335[] = {SGI181945, SGI0, SGI181946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181947, SGI181948, SGI181948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40336[] = {SGI181949, SGI0, SGI181951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181953, SGI181954, SGI181954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40337[] = {SGI181950, SGI0, SGI181952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181955, SGI181956, SGI181956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40338[] = {SGI181957, SGI0, SGI181958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181959, SGI181960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40339[] = {SGI181961, SGI181962, SGI181963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181964, SGI0, SGI0, SGI0, SGI181965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40340[] = {SGI181967, SGI181968, SGI181969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181970, SGI0, SGI0, SGI0, SGI181971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40341[] = {SGI0, SGI181973, SGI181974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40342[] = {SGI181975, SGI181976, SGI181977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181978, SGI0, SGI0, SGI0, SGI181979, SGI181979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40343[] = {SGI181980, SGI0, SGI181981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40344[] = {SGI181982, SGI0, SGI181983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181984, SGI181986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40345[] = {SGI181985, SGI181987, SGI181988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181989, SGI181989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40346[] = {SGI181990, SGI0, SGI181991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181995, SGI181996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40347[] = {SGI181992, SGI181993, SGI181994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI181997, SGI0, SGI0, SGI0, SGI181998, SGI181998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40348[] = {SGI181999, SGI0, SGI182000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40349[] = {SGI182001, SGI0, SGI182002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182003, SGI182004, SGI182004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40350[] = {SGI182005, SGI0, SGI182006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182007, SGI182008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40351[] = {SGI182009, SGI182010, SGI182011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182012, SGI0, SGI0, SGI0, SGI182013, SGI182013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40352[] = {SGI182014, SGI0, SGI182015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182016, SGI182017, SGI182017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40353[] = {SGI182018, SGI0, SGI182019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182020, SGI182021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40354[] = {SGI182022, SGI182023, SGI182024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182025, SGI0, SGI0, SGI0, SGI182026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40355[] = {SGI182027, SGI0, SGI182028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182029, SGI182030, SGI182030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40356[] = {SGI182031, SGI0, SGI182032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182033, SGI182034, SGI182034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40357[] = {SGI182035, SGI0, SGI182036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182037, SGI182038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40358[] = {SGI182039, SGI0, SGI182042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182045, SGI182046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40359[] = {SGI182040, SGI182041, SGI182043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182044, SGI0, SGI0, SGI0, SGI182047, SGI182047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40360[] = {SGI182048, SGI0, SGI182049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40361[] = {SGI182050, SGI182051, SGI182052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182053, SGI0, SGI0, SGI0, SGI182054, SGI182054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40362[] = {SGI182055, SGI0, SGI182056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182057, SGI182058, SGI182058, SGI0, SGI0, NULL};
const Segment * const SG40363[] = {SGI182059, SGI0, SGI182060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182061, SGI182062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40364[] = {SGI182063, SGI0, SGI182064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182065, SGI182066, SGI182066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40365[] = {SGI182067, SGI182068, SGI182069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182071, SGI0, SGI0, SGI0, SGI182073, SGI182073, SGI0, SGI182074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40366[] = {SGI182070, SGI0, SGI182072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182075, SGI182076, SGI182076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40367[] = {SGI182077, SGI182078, SGI182079, SGI0, SGI0, SGI0, SGI182080, SGI182081, SGI0, SGI182082, SGI0, SGI0, SGI0, SGI182083, SGI182083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40368[] = {SGI182084, SGI182085, SGI182086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182087, SGI0, SGI0, SGI0, SGI182088, SGI182088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182090, SGI0, NULL};
const Segment * const SG40369[] = {SGI182091, SGI182092, SGI182093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182094, SGI0, SGI0, SGI0, SGI182095, SGI182095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182097, SGI0, NULL};
const Segment * const SG40370[] = {SGI182098, SGI182099, SGI182100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182101, SGI0, SGI0, SGI0, SGI182102, SGI182102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182104, SGI0, NULL};
const Segment * const SG40371[] = {SGI182105, SGI0, SGI182106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182107, SGI182108, SGI182108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40372[] = {SGI182109, SGI182110, SGI182111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182112, SGI0, SGI0, SGI0, SGI182113, SGI182113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40373[] = {SGI182114, SGI0, SGI182115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40374[] = {SGI182116, SGI182117, SGI182118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182119, SGI0, SGI0, SGI0, SGI182120, SGI182120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40375[] = {SGI182121, SGI0, SGI182122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40376[] = {SGI182123, SGI0, SGI182124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182125, SGI182126, SGI182126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40377[] = {SGI182127, SGI182128, SGI182129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182130, SGI0, SGI0, SGI0, SGI182131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40378[] = {SGI182132, SGI0, SGI182133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182134, SGI182135, SGI182135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40379[] = {SGI182136, SGI0, SGI182137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182138, SGI182139, SGI182139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40380[] = {SGI182140, SGI0, SGI182141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182142, SGI182143, SGI182143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182144, SGI0, SGI0, NULL};
const Segment * const SG40381[] = {SGI182145, SGI182146, SGI182147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182149, SGI0, SGI0, SGI0, SGI182151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40382[] = {SGI182148, SGI0, SGI182150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182153, SGI182154, SGI182154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182155, SGI0, SGI0, NULL};
const Segment * const SG40383[] = {SGI182156, SGI182157, SGI182158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182159, SGI0, SGI0, SGI0, SGI182160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40384[] = {SGI182162, SGI182163, SGI182164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182165, SGI0, SGI0, SGI0, SGI182166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40385[] = {SGI182168, SGI182169, SGI182170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182171, SGI0, SGI0, SGI0, SGI182172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40386[] = {SGI182173, SGI0, SGI182174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182175, SGI182176, SGI182176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40387[] = {SGI182177, SGI0, SGI182178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182179, SGI182180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40388[] = {SGI182181, SGI0, SGI182182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182183, SGI182184, SGI182184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40389[] = {SGI182185, SGI0, SGI182186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182187, SGI182188, SGI182188, SGI0, SGI0, NULL};
const Segment * const SG40390[] = {SGI182189, SGI0, SGI182190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182191, SGI182192, SGI182192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40391[] = {SGI182193, SGI0, SGI182194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182195, SGI182196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40392[] = {SGI182197, SGI182198, SGI182199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182200, SGI0, SGI0, SGI0, SGI182201, SGI182201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40393[] = {SGI182203, SGI182204, SGI182205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182206, SGI0, SGI0, SGI0, SGI182207, SGI182207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40394[] = {SGI182209, SGI182210, SGI182211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182212, SGI0, SGI0, SGI0, SGI182213, SGI182213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40395[] = {SGI182215, SGI0, SGI182216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182217, SGI182218, SGI182218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40396[] = {SGI182219, SGI182220, SGI182221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182222, SGI0, SGI0, SGI0, SGI182223, SGI182223, SGI0, SGI182224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40397[] = {SGI182225, SGI182226, SGI182227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182228, SGI0, SGI0, SGI0, SGI182229, SGI182229, SGI0, SGI182230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40398[] = {SGI182231, SGI182232, SGI182233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182237, SGI0, SGI0, SGI0, SGI182238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40399[] = {SGI182234, SGI182235, SGI182236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182239, SGI0, SGI0, SGI0, SGI182240, SGI182240, SGI0, SGI182241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40400[] = {SGI182242, SGI182243, SGI182244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182245, SGI0, SGI0, SGI0, SGI182246, SGI182246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40401[] = {SGI182247, SGI0, SGI182248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182249, SGI182250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40402[] = {SGI182251, SGI0, SGI182252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182253, SGI182254, SGI182254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40403[] = {SGI182255, SGI0, SGI182256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182257, SGI182258, SGI182258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40404[] = {SGI182259, SGI182260, SGI182261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182263, SGI182263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40405[] = {SGI182262, SGI0, SGI182264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182265, SGI182266, SGI182266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40406[] = {SGI182267, SGI182268, SGI182269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182270, SGI0, SGI0, SGI0, SGI182271, SGI182271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40407[] = {SGI182272, SGI0, SGI182273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182274, SGI182275, SGI182275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40408[] = {SGI182276, SGI0, SGI182277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40409[] = {SGI182278, SGI0, SGI182279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182280, SGI182281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40410[] = {SGI182282, SGI182283, SGI182284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182285, SGI0, SGI0, SGI0, SGI182286, SGI182286, SGI0, SGI182287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40411[] = {SGI182288, SGI182289, SGI182290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182291, SGI0, SGI0, SGI0, SGI182292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40412[] = {SGI182293, SGI0, SGI182294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40413[] = {SGI182295, SGI0, SGI182297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182300, SGI182302, SGI182302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40414[] = {SGI182296, SGI182298, SGI182299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182301, SGI0, SGI0, SGI0, SGI182303, SGI182303, SGI0, SGI182304, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40415[] = {SGI182305, SGI0, SGI182306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182307, SGI182308, SGI182308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40416[] = {SGI182309, SGI182310, SGI182311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182312, SGI0, SGI0, SGI0, SGI182313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40417[] = {SGI182314, SGI0, SGI182315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182316, SGI182317, SGI182317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40418[] = {SGI182318, SGI182319, SGI182320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182321, SGI0, SGI0, SGI0, SGI182322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40419[] = {SGI182324, SGI182325, SGI182327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182328, SGI0, SGI0, SGI0, SGI182332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40420[] = {SGI182323, SGI0, SGI182326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182330, SGI0, SGI0, SGI0, SGI182333, SGI182333, SGI0, SGI0, NULL};
const Segment * const SG40421[] = {SGI182329, SGI0, SGI182331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182334, SGI182335, SGI182335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40422[] = {SGI182336, SGI182337, SGI182338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182339, SGI0, SGI0, SGI0, SGI182340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40423[] = {SGI182341, SGI182342, SGI182343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182344, SGI0, SGI0, SGI0, SGI182345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40424[] = {SGI182346, SGI0, SGI182347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40425[] = {SGI182348, SGI182349, SGI182350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182351, SGI0, SGI0, SGI0, SGI182352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40426[] = {SGI182354, SGI182356, SGI182358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182361, SGI0, SGI0, SGI0, SGI182365, SGI182365, SGI0, SGI0, NULL};
const Segment * const SG40427[] = {SGI182353, SGI182355, SGI182357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182359, SGI0, SGI0, SGI0, SGI182362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40428[] = {SGI182360, SGI182363, SGI182364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182366, SGI0, SGI0, SGI0, SGI182367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40429[] = {SGI182368, SGI182369, SGI182370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182371, SGI0, SGI0, SGI0, SGI182372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40430[] = {SGI182373, SGI0, SGI182374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182378, SGI182379, SGI182379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40431[] = {SGI182375, SGI182376, SGI182377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182380, SGI0, SGI0, SGI0, SGI182381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40432[] = {SGI182382, SGI0, SGI182383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182387, SGI182388, SGI182388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40433[] = {SGI182384, SGI182385, SGI182386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182389, SGI0, SGI0, SGI0, SGI182390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40434[] = {SGI182391, SGI0, SGI182392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182393, SGI182394, SGI182394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40435[] = {SGI182395, SGI0, SGI182396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182397, SGI182398, SGI182398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40436[] = {SGI182399, SGI0, SGI182400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182401, SGI182402, SGI182402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40437[] = {SGI182403, SGI182404, SGI182405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182406, SGI0, SGI0, SGI0, SGI182407, SGI0, SGI182408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40438[] = {SGI182409, SGI182410, SGI182411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182412, SGI0, SGI0, SGI0, SGI182413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40439[] = {SGI182414, SGI0, SGI182415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182416, SGI182417, SGI182417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40440[] = {SGI182418, SGI182419, SGI182420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182422, SGI182422, SGI0, SGI0, SGI182425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40441[] = {SGI182421, SGI182423, SGI182424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182426, SGI0, SGI0, SGI0, SGI182427, SGI182427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40442[] = {SGI182431, SGI0, SGI182432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40443[] = {SGI182428, SGI182429, SGI182430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182433, SGI182433, SGI0, SGI0, SGI182434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40444[] = {SGI182435, SGI182436, SGI182438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182441, SGI0, SGI0, SGI0, SGI182442, SGI182442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40445[] = {SGI182443, SGI182444, SGI182445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182446, SGI0, SGI0, SGI0, SGI182447, SGI182447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40446[] = {SGI182448, SGI182449, SGI182450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182451, SGI0, SGI0, SGI0, SGI182452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40447[] = {SGI182453, SGI182454, SGI182455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182456, SGI0, SGI0, SGI0, SGI182457, SGI182457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40448[] = {SGI182458, SGI0, SGI182459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40449[] = {SGI182460, SGI0, SGI182461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182462, SGI182463, SGI182463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40450[] = {SGI182464, SGI0, SGI182465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182466, SGI182467, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40451[] = {SGI182468, SGI0, SGI182469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182470, SGI182471, SGI182471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40452[] = {SGI182472, SGI0, SGI182473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182474, SGI182475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40453[] = {SGI182476, SGI0, SGI182477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182478, SGI182479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40454[] = {SGI182480, SGI0, SGI182481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182482, SGI182483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40455[] = {SGI182484, SGI182485, SGI182486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182487, SGI0, SGI0, SGI0, SGI182488, SGI0, SGI0, SGI182489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40456[] = {SGI182490, SGI0, SGI182491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182492, SGI0, SGI0, SGI0, SGI182493, SGI0, SGI0, SGI182494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40457[] = {SGI182495, SGI0, SGI182497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182500, SGI182502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40458[] = {SGI182496, SGI182498, SGI182499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182501, SGI0, SGI0, SGI0, SGI182503, SGI0, SGI0, SGI182504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40459[] = {SGI182505, SGI0, SGI182506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182507, SGI182508, SGI182508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40460[] = {SGI182509, SGI0, SGI182510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182513, SGI182515, SGI182515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40461[] = {SGI182511, SGI182512, SGI182514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182516, SGI0, SGI0, SGI0, SGI182517, SGI182517, SGI0, SGI182518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40462[] = {SGI182519, SGI0, SGI182520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182523, SGI182525, SGI182525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40463[] = {SGI182521, SGI182522, SGI182524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182526, SGI0, SGI0, SGI0, SGI182527, SGI0, SGI0, SGI182528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40464[] = {SGI0, SGI182529, SGI182530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182531, SGI0, SGI0, SGI0, SGI182532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40465[] = {SGI182533, SGI0, SGI182534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182535, SGI182536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40466[] = {SGI182537, SGI0, SGI182538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182539, SGI182540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40467[] = {SGI182541, SGI182542, SGI182543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182544, SGI0, SGI0, SGI0, SGI182545, SGI182545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40468[] = {SGI0, SGI182546, SGI182547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182548, SGI0, SGI0, SGI0, SGI182549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40469[] = {SGI182550, SGI0, SGI182551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182552, SGI182553, SGI182553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40470[] = {SGI182554, SGI0, SGI182555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182556, SGI182557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40471[] = {SGI182558, SGI0, SGI182559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182560, SGI182561, SGI182561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40472[] = {SGI182562, SGI0, SGI182563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182564, SGI182565, SGI182565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40473[] = {SGI182566, SGI182567, SGI182568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182569, SGI0, SGI0, SGI0, SGI182570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40474[] = {SGI182571, SGI182572, SGI182573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182574, SGI0, SGI0, SGI0, SGI182575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40475[] = {SGI182437, SGI182439, SGI182440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182576, SGI182576, SGI0, SGI0, SGI182577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40476[] = {SGI182578, SGI182579, SGI182580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182581, SGI182581, SGI0, SGI0, SGI182582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40477[] = {SGI182583, SGI182585, SGI182587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182590, SGI182590, SGI0, SGI0, SGI182592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40478[] = {SGI182584, SGI182586, SGI182588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182589, SGI0, SGI0, SGI0, SGI182591, SGI182591, SGI0, SGI182593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40479[] = {SGI182594, SGI182595, SGI182596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182597, SGI0, SGI0, SGI0, SGI182598, SGI182598, SGI0, SGI182599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40480[] = {SGI182600, SGI182601, SGI182602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182605, SGI0, SGI0, SGI0, SGI182606, SGI182606, SGI0, SGI182607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40481[] = {SGI182603, SGI0, SGI182604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182608, SGI182609, SGI182609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40482[] = {SGI182610, SGI182611, SGI182612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182613, SGI0, SGI0, SGI0, SGI182614, SGI182614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40483[] = {SGI182616, SGI182617, SGI182618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182619, SGI0, SGI0, SGI0, SGI182620, SGI182620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40484[] = {SGI182622, SGI182623, SGI182624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182625, SGI0, SGI0, SGI0, SGI182626, SGI182626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40485[] = {SGI182628, SGI0, SGI182629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182630, SGI182631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40486[] = {SGI182632, SGI0, SGI182633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182634, SGI182635, SGI182635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40487[] = {SGI182636, SGI0, SGI182637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182638, SGI182639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40488[] = {SGI182640, SGI182641, SGI182642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182643, SGI0, SGI0, SGI0, SGI182644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40489[] = {SGI182645, SGI0, SGI182646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40490[] = {SGI182647, SGI0, SGI182648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182649, SGI182650, SGI182650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40491[] = {SGI182651, SGI182652, SGI182653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182654, SGI0, SGI0, SGI0, SGI182655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40492[] = {SGI182656, SGI182657, SGI182658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182659, SGI0, SGI0, SGI0, SGI182660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40493[] = {SGI182661, SGI0, SGI182662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182663, SGI182664, SGI182664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40494[] = {SGI182665, SGI0, SGI182666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182667, SGI182668, SGI182668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40495[] = {SGI182669, SGI0, SGI182670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182671, SGI182672, SGI182672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40496[] = {SGI182673, SGI182674, SGI182675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182676, SGI0, SGI0, SGI0, SGI182677, SGI182677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40497[] = {SGI182678, SGI0, SGI182679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182680, SGI182681, SGI182681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40498[] = {SGI182682, SGI0, SGI182683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182684, SGI182685, SGI182685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40499[] = {SGI182686, SGI182687, SGI182688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182689, SGI0, SGI0, SGI0, SGI182690, SGI182690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40500[] = {SGI182691, SGI0, SGI182692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40501[] = {SGI182693, SGI0, SGI182694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182695, SGI182696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40502[] = {SGI182697, SGI182698, SGI182699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182700, SGI0, SGI0, SGI0, SGI182701, SGI182701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40503[] = {SGI182702, SGI0, SGI182703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182704, SGI182705, SGI182705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40504[] = {SGI182706, SGI0, SGI182707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182708, SGI182709, SGI182709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40505[] = {SGI182710, SGI0, SGI182711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182714, SGI182715, SGI182715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40506[] = {SGI182712, SGI0, SGI182713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182716, SGI182717, SGI182717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40507[] = {SGI182718, SGI0, SGI182719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182720, SGI182721, SGI182721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40508[] = {SGI182722, SGI182723, SGI182724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182725, SGI0, SGI0, SGI0, SGI182726, SGI182726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40509[] = {SGI182729, SGI0, SGI182731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182734, SGI182735, SGI182735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40510[] = {SGI182728, SGI182730, SGI182732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182733, SGI0, SGI0, SGI0, SGI182736, SGI182736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40511[] = {SGI182738, SGI0, SGI182739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182740, SGI182742, SGI182742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40512[] = {SGI182741, SGI182743, SGI182744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182745, SGI0, SGI0, SGI0, SGI182746, SGI182746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40513[] = {SGI182747, SGI0, SGI182748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182750, SGI182751, SGI182751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40514[] = {SGI182752, SGI0, SGI182753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182754, SGI182755, SGI182755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40515[] = {SGI182756, SGI0, SGI182757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182758, SGI182759, SGI182759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40516[] = {SGI182760, SGI0, SGI182761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182762, SGI182763, SGI182763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40517[] = {SGI182764, SGI182765, SGI182766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182767, SGI182767, SGI0, SGI0, SGI182768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40518[] = {SGI182769, SGI182770, SGI182771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182772, SGI0, SGI0, SGI0, SGI182773, SGI182773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40519[] = {SGI182774, SGI182775, SGI182776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182777, SGI0, SGI0, SGI0, SGI182778, SGI182778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40520[] = {SGI182779, SGI182780, SGI182781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182782, SGI0, SGI0, SGI0, SGI182783, SGI182783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40521[] = {SGI182784, SGI182785, SGI182786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182787, SGI0, SGI0, SGI0, SGI182788, SGI182788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40522[] = {SGI182789, SGI182790, SGI182791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182792, SGI0, SGI0, SGI0, SGI182793, SGI182793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40523[] = {SGI182794, SGI182795, SGI182796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182797, SGI0, SGI0, SGI0, SGI182798, SGI182798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40524[] = {SGI182799, SGI182800, SGI182801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182802, SGI0, SGI0, SGI0, SGI182803, SGI182803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40525[] = {SGI182804, SGI0, SGI182805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40526[] = {SGI182806, SGI182807, SGI182808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182809, SGI0, SGI0, SGI0, SGI182810, SGI182810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40527[] = {SGI182812, SGI182813, SGI182814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182815, SGI0, SGI0, SGI0, SGI182816, SGI182816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40528[] = {SGI182818, SGI182819, SGI182820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182821, SGI0, SGI0, SGI0, SGI182822, SGI182822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40529[] = {SGI182824, SGI182825, SGI182826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182827, SGI0, SGI0, SGI0, SGI182828, SGI182828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40530[] = {SGI182829, SGI0, SGI182830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40531[] = {SGI182831, SGI182832, SGI182833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182834, SGI0, SGI0, SGI0, SGI182835, SGI182835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40532[] = {SGI182836, SGI182837, SGI182838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182839, SGI0, SGI0, SGI0, SGI182840, SGI182840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40533[] = {SGI182841, SGI182842, SGI182843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182844, SGI182844, SGI0, SGI0, SGI182845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40534[] = {SGI182846, SGI182847, SGI182848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182849, SGI0, SGI0, SGI0, SGI182850, SGI182850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40535[] = {SGI182851, SGI0, SGI182852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182853, SGI182854, SGI182854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40536[] = {SGI182855, SGI0, SGI182856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182857, SGI182858, SGI182858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40537[] = {SGI182859, SGI0, SGI182860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182861, SGI182862, SGI182862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40538[] = {SGI182863, SGI0, SGI182864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182865, SGI182866, SGI182866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40539[] = {SGI182867, SGI182868, SGI182869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182870, SGI0, SGI0, SGI0, SGI182871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40540[] = {SGI182872, SGI0, SGI182873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40541[] = {SGI182874, SGI182875, SGI182876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182877, SGI0, SGI0, SGI0, SGI182878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40542[] = {SGI182879, SGI182880, SGI182881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182882, SGI182882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40543[] = {SGI182883, SGI182884, SGI182885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182888, SGI0, SGI0, SGI0, SGI182889, SGI182889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40544[] = {SGI182886, SGI0, SGI182887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182890, SGI0, SGI0, SGI0, SGI182891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40545[] = {SGI182892, SGI0, SGI182893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40546[] = {SGI182894, SGI182895, SGI182896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40547[] = {SGI182897, SGI182898, SGI182899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40548[] = {SGI182900, SGI182901, SGI182902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40549[] = {SGI182903, SGI182904, SGI182905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182906, SGI0, SGI0, SGI0, SGI182907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40550[] = {SGI182908, SGI182909, SGI182910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40551[] = {SGI182911, SGI182912, SGI182913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182914, SGI0, SGI0, SGI0, SGI182915, SGI182915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40552[] = {SGI182916, SGI0, SGI182917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40553[] = {SGI182918, SGI0, SGI182919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182920, SGI182921, SGI182921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40554[] = {SGI182922, SGI0, SGI182923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182924, SGI182925, SGI182925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40555[] = {SGI182926, SGI182927, SGI182928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182929, SGI0, SGI0, SGI0, SGI182930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40556[] = {SGI182931, SGI0, SGI182932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182933, SGI182934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40557[] = {SGI182935, SGI0, SGI182936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40558[] = {SGI182937, SGI0, SGI182938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182939, SGI182940, SGI182940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40559[] = {SGI182941, SGI182942, SGI182943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182944, SGI0, SGI0, SGI0, SGI182945, SGI182945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40560[] = {SGI182946, SGI0, SGI182947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182948, SGI182949, SGI182949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40561[] = {SGI182950, SGI0, SGI182951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182952, SGI182953, SGI182953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40562[] = {SGI182954, SGI182955, SGI182956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182957, SGI0, SGI0, SGI0, SGI182958, SGI182958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40563[] = {SGI182959, SGI182960, SGI182961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182962, SGI0, SGI0, SGI0, SGI182963, SGI182963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40564[] = {SGI182964, SGI182965, SGI182966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182967, SGI0, SGI0, SGI0, SGI182968, SGI182968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40565[] = {SGI182969, SGI0, SGI182970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182971, SGI182972, SGI182972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40566[] = {SGI182973, SGI0, SGI182974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182975, SGI0, SGI0, SGI0, SGI182976, SGI182976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40567[] = {SGI182977, SGI0, SGI182978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182979, SGI182980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40568[] = {SGI182981, SGI0, SGI182982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182983, SGI182984, SGI182984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40569[] = {SGI182985, SGI0, SGI182986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182987, SGI182988, SGI182988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40570[] = {SGI182989, SGI0, SGI182990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182991, SGI182992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40571[] = {SGI182993, SGI0, SGI182994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182995, SGI182996, SGI182996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40572[] = {SGI182997, SGI0, SGI182998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI182999, SGI183000, SGI183000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40573[] = {SGI183001, SGI183002, SGI183003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183004, SGI0, SGI0, SGI0, SGI183005, SGI183005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40574[] = {SGI183006, SGI183007, SGI183008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183009, SGI0, SGI0, SGI0, SGI183010, SGI183010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40575[] = {SGI183012, SGI183013, SGI183014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183015, SGI0, SGI0, SGI0, SGI183016, SGI183016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40576[] = {SGI183018, SGI0, SGI183019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183020, SGI183021, SGI183021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40577[] = {SGI183022, SGI0, SGI183023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183024, SGI183025, SGI183025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40578[] = {SGI183026, SGI0, SGI183027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183030, SGI183031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40579[] = {SGI183028, SGI0, SGI183029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183032, SGI183033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40580[] = {SGI183034, SGI0, SGI183035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183038, SGI183039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40581[] = {SGI183036, SGI0, SGI183037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183040, SGI183041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40582[] = {SGI183042, SGI0, SGI183043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183044, SGI183045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40583[] = {SGI183046, SGI183047, SGI183048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183049, SGI0, SGI0, SGI0, SGI183050, SGI183050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40584[] = {SGI183051, SGI0, SGI183052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40585[] = {SGI183053, SGI0, SGI183054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183055, SGI183056, SGI183056, SGI0, SGI0, SGI0, SGI0, SGI183057, SGI0, NULL};
const Segment * const SG40586[] = {SGI183058, SGI0, SGI183059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183060, SGI183061, SGI183061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40587[] = {SGI183062, SGI0, SGI183063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183064, SGI183065, SGI183065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40588[] = {SGI183066, SGI0, SGI183067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183068, SGI183069, SGI183069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40589[] = {SGI183070, SGI183071, SGI183072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183073, SGI0, SGI0, SGI0, SGI183074, SGI183074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40590[] = {SGI183075, SGI0, SGI183076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183078, SGI183079, SGI183079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183081, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40591[] = {SGI183077, SGI0, SGI183080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183082, SGI183083, SGI183083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40592[] = {SGI183084, SGI183085, SGI183086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183087, SGI0, SGI0, SGI0, SGI183088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40593[] = {SGI183091, SGI0, SGI183092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40594[] = {SGI183089, SGI0, SGI183090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183093, SGI183094, SGI183094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183097, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40595[] = {SGI183095, SGI0, SGI183096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183098, SGI183099, SGI183099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40596[] = {SGI183100, SGI0, SGI183101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183102, SGI183103, SGI183103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40597[] = {SGI183104, SGI0, SGI183105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183106, SGI183108, SGI183108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40598[] = {SGI183107, SGI0, SGI183109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183110, SGI183111, SGI183111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40599[] = {SGI183112, SGI183113, SGI183114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183115, SGI0, SGI0, SGI0, SGI183116, SGI183116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40600[] = {SGI183117, SGI183118, SGI183119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183120, SGI0, SGI0, SGI0, SGI183121, SGI183121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40601[] = {SGI183122, SGI0, SGI183123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40602[] = {SGI183124, SGI183125, SGI183126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183127, SGI0, SGI0, SGI0, SGI183128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40603[] = {SGI183129, SGI183130, SGI183131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183132, SGI0, SGI0, SGI0, SGI183133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40604[] = {SGI183134, SGI0, SGI183135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183138, SGI183139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40605[] = {SGI183136, SGI0, SGI183137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183140, SGI183141, SGI183141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40606[] = {SGI183142, SGI0, SGI183143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183144, SGI183145, SGI183145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40607[] = {SGI183146, SGI0, SGI183147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183148, SGI183149, SGI183149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40608[] = {SGI183150, SGI183151, SGI183152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183153, SGI0, SGI0, SGI0, SGI183154, SGI0, SGI0, SGI183155, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40609[] = {SGI183156, SGI183157, SGI183158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183159, SGI0, SGI0, SGI0, SGI183160, SGI0, SGI0, SGI183161, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40610[] = {SGI183162, SGI0, SGI183163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183164, SGI183165, SGI183165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40611[] = {SGI183166, SGI0, SGI183167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183168, SGI183169, SGI183169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40612[] = {SGI183170, SGI0, SGI183171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183172, SGI0, SGI0, SGI0, SGI183173, SGI183173, SGI0, SGI0, SGI0, SGI183174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40613[] = {SGI183175, SGI183176, SGI183177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183178, SGI0, SGI0, SGI0, SGI183179, SGI183179, SGI183180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40614[] = {SGI183181, SGI183182, SGI183183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183184, SGI0, SGI0, SGI0, SGI183185, SGI183185, SGI183187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40615[] = {SGI183186, SGI183188, SGI183189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183190, SGI0, SGI0, SGI0, SGI183191, SGI183191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40616[] = {SGI183192, SGI183193, SGI183194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183195, SGI0, SGI0, SGI0, SGI183196, SGI183196, SGI183197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40617[] = {SGI183198, SGI183199, SGI183200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183203, SGI0, SGI0, SGI0, SGI183204, SGI183204, SGI183206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40618[] = {SGI183201, SGI0, SGI183202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183205, SGI183208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40619[] = {SGI183207, SGI0, SGI183209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183210, SGI183211, SGI183211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40620[] = {SGI183212, SGI183213, SGI183214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183217, SGI0, SGI0, SGI0, SGI183218, SGI183218, SGI183222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40621[] = {SGI183215, SGI0, SGI183216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183220, SGI183223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40622[] = {SGI183219, SGI0, SGI183221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183224, SGI183225, SGI183225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40623[] = {SGI183226, SGI0, SGI183227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183228, SGI183229, SGI183229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40624[] = {SGI183230, SGI0, SGI183231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183232, SGI183234, SGI183234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40625[] = {SGI183233, SGI183235, SGI183236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183237, SGI0, SGI0, SGI0, SGI183238, SGI183238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40626[] = {SGI183239, SGI0, SGI183240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183241, SGI183242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40627[] = {SGI183243, SGI0, SGI183244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183245, SGI183246, SGI183246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40628[] = {SGI183247, SGI183248, SGI183249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183250, SGI0, SGI0, SGI0, SGI183251, SGI183251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40629[] = {SGI183252, SGI0, SGI183253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183254, SGI183255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40630[] = {SGI183256, SGI0, SGI183257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183261, SGI183262, SGI183262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40631[] = {SGI183258, SGI183259, SGI183260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183263, SGI0, SGI0, SGI0, SGI183264, SGI183264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40632[] = {SGI183265, SGI183266, SGI183267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183268, SGI0, SGI0, SGI0, SGI183269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40633[] = {SGI183270, SGI183271, SGI183272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183273, SGI0, SGI0, SGI0, SGI183274, SGI183274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40634[] = {SGI183275, SGI0, SGI183276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40635[] = {SGI183277, SGI0, SGI183278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183279, SGI183280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40636[] = {SGI183281, SGI183282, SGI183283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183284, SGI0, SGI0, SGI0, SGI183286, SGI183286, SGI0, SGI183288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40637[] = {SGI183285, SGI0, SGI183287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183289, SGI183290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40638[] = {SGI183291, SGI183292, SGI183293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183294, SGI0, SGI0, SGI0, SGI183296, SGI183296, SGI0, SGI183298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40639[] = {SGI183295, SGI0, SGI183297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183299, SGI183300, SGI183300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183301, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40640[] = {SGI183302, SGI0, SGI183303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183304, SGI183305, SGI183305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40641[] = {SGI183306, SGI0, SGI183307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183309, SGI183311, SGI183311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40642[] = {SGI183308, SGI0, SGI183310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183312, SGI183313, SGI183313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40643[] = {SGI183314, SGI0, SGI183315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183316, SGI183317, SGI183317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40644[] = {SGI183318, SGI0, SGI183319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183322, SGI183323, SGI183323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40645[] = {SGI183320, SGI0, SGI183321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183324, SGI183325, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40646[] = {SGI183326, SGI0, SGI183327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183328, SGI183329, SGI183329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40647[] = {SGI183330, SGI0, SGI183331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183332, SGI183333, SGI183333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40648[] = {SGI183335, SGI0, SGI183338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183340, SGI183342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40649[] = {SGI183334, SGI183336, SGI183337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183339, SGI0, SGI0, SGI0, SGI183341, SGI183341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40650[] = {SGI183343, SGI0, SGI183344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183345, SGI183346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40651[] = {SGI183350, SGI0, SGI183351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40652[] = {SGI183347, SGI183348, SGI183349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183353, SGI0, SGI0, SGI0, SGI183355, SGI183355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40653[] = {SGI183352, SGI0, SGI183354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183356, SGI183357, SGI183357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40654[] = {SGI183358, SGI183359, SGI183360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183361, SGI0, SGI0, SGI0, SGI183362, SGI183362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40655[] = {SGI183363, SGI183364, SGI183365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183366, SGI0, SGI0, SGI0, SGI183367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40656[] = {SGI183368, SGI183369, SGI183370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183371, SGI0, SGI0, SGI0, SGI183372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183373, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40657[] = {SGI183374, SGI0, SGI183375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183376, SGI183377, SGI183377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40658[] = {SGI183378, SGI183379, SGI183380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183381, SGI0, SGI0, SGI0, SGI183382, SGI183382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183383, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40659[] = {SGI183384, SGI0, SGI183385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183386, SGI183387, SGI183387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40660[] = {SGI183388, SGI183389, SGI183390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183391, SGI0, SGI0, SGI0, SGI183392, SGI183392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183395, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40661[] = {SGI183393, SGI0, SGI183394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183396, SGI183397, SGI183397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40662[] = {SGI183398, SGI183399, SGI183400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183402, SGI0, SGI0, SGI0, SGI183404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183405, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40663[] = {SGI183401, SGI0, SGI183403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183406, SGI183407, SGI183407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40664[] = {SGI183408, SGI183409, SGI183410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183411, SGI0, SGI0, SGI0, SGI183413, SGI183413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40665[] = {SGI183412, SGI0, SGI183414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183419, SGI183420, SGI183420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40666[] = {SGI183415, SGI183417, SGI183418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183421, SGI0, SGI0, SGI0, SGI183422, SGI183422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40667[] = {SGI183423, SGI183424, SGI183425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183426, SGI0, SGI0, SGI0, SGI183428, SGI183428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40668[] = {SGI183427, SGI0, SGI183430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183435, SGI183438, SGI183438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40669[] = {SGI183429, SGI183431, SGI183432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183436, SGI0, SGI0, SGI0, SGI183439, SGI183439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40670[] = {SGI183434, SGI0, SGI183437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183440, SGI183441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40671[] = {SGI183442, SGI0, SGI183443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40672[] = {SGI183444, SGI0, SGI183445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183446, SGI183447, SGI183447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40673[] = {SGI183448, SGI183449, SGI183450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183451, SGI0, SGI0, SGI0, SGI183452, SGI183452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40674[] = {SGI183453, SGI183454, SGI183455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183456, SGI0, SGI0, SGI0, SGI183457, SGI183457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40675[] = {SGI183458, SGI0, SGI183459, SGI0, SGI183460, SGI183460, SGI0, NULL};
const Segment * const SG40676[] = {SGI183461, SGI183462, SGI183463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183464, SGI0, SGI0, SGI0, SGI183465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40677[] = {SGI183466, SGI0, SGI183467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40678[] = {SGI183468, SGI183469, SGI183470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40679[] = {SGI183471, SGI0, SGI183472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183473, SGI183474, SGI183474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40680[] = {SGI183475, SGI183476, SGI183477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183478, SGI0, SGI0, SGI0, SGI183479, SGI183479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40681[] = {SGI183480, SGI0, SGI183481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40682[] = {SGI0, SGI0, SGI183482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183483, SGI183483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40683[] = {SGI183484, SGI183485, SGI183486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183487, SGI0, SGI0, SGI0, SGI183488, SGI183488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40684[] = {SGI183489, SGI183490, SGI183491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183492, SGI0, SGI0, SGI0, SGI183493, SGI183493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40685[] = {SGI183494, SGI0, SGI183495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183498, SGI183499, SGI183499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40686[] = {SGI183496, SGI0, SGI183497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40687[] = {SGI183500, SGI0, SGI183501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40688[] = {SGI183502, SGI0, SGI183503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183504, SGI183505, SGI183505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40689[] = {SGI183506, SGI183507, SGI183508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183509, SGI183509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40690[] = {SGI183510, SGI0, SGI183511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183512, SGI183513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40691[] = {SGI183514, SGI183515, SGI183516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183517, SGI183517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40692[] = {SGI183518, SGI0, SGI183519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183520, SGI183521, SGI183521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40693[] = {SGI183522, SGI183523, SGI183524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183525, SGI0, SGI0, SGI0, SGI183526, SGI183526, SGI0, SGI183527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40694[] = {SGI183528, SGI183529, SGI183530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183531, SGI0, SGI0, SGI0, SGI183532, SGI183532, SGI0, SGI183533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40695[] = {SGI183534, SGI183535, SGI183536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183537, SGI0, SGI0, SGI0, SGI183538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40696[] = {SGI183539, SGI183540, SGI183541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183542, SGI0, SGI0, SGI0, SGI183543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40697[] = {SGI183544, SGI183545, SGI183546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183547, SGI0, SGI0, SGI0, SGI183548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183549, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40698[] = {SGI183550, SGI0, SGI183551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183552, SGI183553, SGI183553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40699[] = {SGI183554, SGI0, SGI183555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183556, SGI183557, SGI183557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40700[] = {SGI183558, SGI0, SGI183559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183560, SGI183561, SGI183561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40701[] = {SGI183562, SGI183563, SGI183564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183565, SGI0, SGI0, SGI0, SGI183566, SGI183566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40702[] = {SGI183567, SGI183568, SGI183569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183570, SGI0, SGI0, SGI0, SGI183571, SGI183571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40703[] = {SGI183572, SGI183573, SGI183574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183575, SGI0, SGI0, SGI0, SGI183576, SGI183576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40704[] = {SGI183577, SGI183578, SGI183579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183583, SGI0, SGI0, SGI0, SGI183584, SGI183584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40705[] = {SGI183580, SGI183581, SGI183582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183585, SGI0, SGI0, SGI0, SGI183586, SGI183586, SGI0, SGI0, SGI0, SGI183587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40706[] = {SGI183588, SGI183589, SGI183590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183591, SGI0, SGI0, SGI0, SGI183592, SGI183592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40707[] = {SGI183593, SGI183594, SGI183595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183596, SGI0, SGI0, SGI0, SGI183597, SGI183597, SGI0, SGI0, SGI0, SGI183598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40708[] = {SGI183599, SGI183600, SGI183601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183602, SGI0, SGI0, SGI0, SGI183603, SGI183603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40709[] = {SGI183604, SGI183605, SGI183606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183607, SGI0, SGI0, SGI0, SGI183608, SGI183608, SGI0, SGI0, SGI0, SGI183609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40710[] = {SGI183610, SGI183611, SGI183612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183613, SGI0, SGI0, SGI0, SGI183614, SGI183614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40711[] = {SGI183615, SGI183616, SGI183617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183618, SGI0, SGI0, SGI0, SGI183619, SGI183619, SGI0, SGI0, SGI0, SGI183620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40712[] = {SGI183621, SGI183622, SGI183623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183624, SGI0, SGI0, SGI0, SGI183625, SGI183625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40713[] = {SGI183626, SGI183627, SGI183628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183629, SGI0, SGI0, SGI0, SGI183630, SGI183630, SGI0, SGI0, SGI0, SGI183631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40714[] = {SGI183632, SGI183633, SGI183634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183635, SGI0, SGI0, SGI0, SGI183636, SGI183636, SGI0, SGI0, SGI0, SGI183637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40715[] = {SGI183638, SGI0, SGI183639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183640, SGI183641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40716[] = {SGI183642, SGI0, SGI183643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183644, SGI183645, SGI183645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40717[] = {SGI183646, SGI0, SGI183647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183648, SGI183649, SGI183649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40718[] = {SGI183650, SGI0, SGI183651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183652, SGI183653, SGI183653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40719[] = {SGI183654, SGI0, SGI183655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183656, SGI183657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40720[] = {SGI183658, SGI0, SGI183659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183660, SGI183661, SGI183661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40721[] = {SGI183662, SGI183663, SGI183664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183665, SGI0, SGI0, SGI0, SGI183666, SGI183666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40722[] = {SGI183667, SGI183668, SGI183669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183670, SGI0, SGI0, SGI0, SGI183671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40723[] = {SGI183672, SGI183673, SGI183674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183675, SGI0, SGI0, SGI0, SGI183678, SGI183678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40724[] = {SGI183676, SGI183677, SGI183679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183680, SGI0, SGI0, SGI0, SGI183681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40725[] = {SGI183682, SGI183683, SGI183684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183685, SGI0, SGI0, SGI0, SGI183686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40726[] = {SGI183687, SGI183688, SGI183689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183690, SGI0, SGI0, SGI0, SGI183691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40727[] = {SGI183692, SGI0, SGI183693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40728[] = {SGI183694, SGI0, SGI183695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183696, SGI183697, SGI183697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40729[] = {SGI183698, SGI0, SGI183699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183700, SGI183701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40730[] = {SGI183702, SGI183703, SGI183704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183705, SGI0, SGI0, SGI0, SGI183706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40731[] = {SGI183707, SGI183708, SGI183709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183710, SGI0, SGI0, SGI0, SGI183711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40732[] = {SGI183713, SGI183714, SGI183716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183717, SGI0, SGI0, SGI0, SGI183719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40733[] = {SGI183712, SGI0, SGI183715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183718, SGI183720, SGI183720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40734[] = {SGI183721, SGI183722, SGI183723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183724, SGI0, SGI0, SGI0, SGI183725, SGI183725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40735[] = {SGI183726, SGI0, SGI183727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183728, SGI183729, SGI183729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40736[] = {SGI183730, SGI0, SGI183731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183732, SGI183733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40737[] = {SGI183734, SGI0, SGI183735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183737, SGI183739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40738[] = {SGI183736, SGI0, SGI183738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183740, SGI183741, SGI183741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40739[] = {SGI183742, SGI0, SGI183743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183746, SGI183747, SGI183747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40740[] = {SGI183744, SGI0, SGI183745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183748, SGI183749, SGI183749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40741[] = {SGI183750, SGI0, SGI183751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183752, SGI183753, SGI183753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40742[] = {SGI183754, SGI0, SGI183755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183756, SGI183757, SGI183757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40743[] = {SGI183758, SGI0, SGI183759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183760, SGI183761, SGI183761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40744[] = {SGI183762, SGI0, SGI183763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183764, SGI183765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40745[] = {SGI183766, SGI0, SGI183767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183768, SGI183769, SGI183769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183770, SGI0, SGI0, NULL};
const Segment * const SG40746[] = {SGI183771, SGI0, SGI183772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183773, SGI183774, SGI183774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183775, SGI0, SGI0, NULL};
const Segment * const SG40747[] = {SGI183776, SGI0, SGI183777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183778, SGI183779, SGI183779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183780, SGI0, SGI0, NULL};
const Segment * const SG40748[] = {SGI183781, SGI0, SGI183782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183783, SGI183784, SGI183784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183785, SGI0, SGI0, NULL};
const Segment * const SG40749[] = {SGI183786, SGI0, SGI183787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183788, SGI183789, SGI183789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183790, SGI0, SGI0, NULL};
const Segment * const SG40750[] = {SGI183791, SGI0, SGI183792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183793, SGI183794, SGI183794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183796, SGI0, SGI0, NULL};
const Segment * const SG40751[] = {SGI183795, SGI0, SGI183797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183798, SGI183799, SGI183799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40752[] = {SGI183800, SGI183801, SGI183802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183803, SGI0, SGI0, SGI0, SGI183804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40753[] = {SGI183805, SGI0, SGI183806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183807, SGI183808, SGI183808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40754[] = {SGI183809, SGI0, SGI183810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183811, SGI183812, SGI183812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40755[] = {SGI183813, SGI0, SGI183814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183815, SGI183816, SGI183816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40756[] = {SGI183817, SGI0, SGI183818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183819, SGI183820, SGI183820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40757[] = {SGI183821, SGI0, SGI183822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183823, SGI183824, SGI183824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40758[] = {SGI183825, SGI0, SGI183826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183827, SGI183828, SGI183828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40759[] = {SGI183829, SGI0, SGI183830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183831, SGI183832, SGI183832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40760[] = {SGI183833, SGI0, SGI183834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183835, SGI183836, SGI183836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40761[] = {SGI183837, SGI0, SGI183838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183839, SGI183840, SGI183840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40762[] = {SGI183841, SGI0, SGI183842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183843, SGI183844, SGI183844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40763[] = {SGI183845, SGI0, SGI183846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183847, SGI183848, SGI183848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40764[] = {SGI183849, SGI0, SGI183850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183851, SGI183852, SGI183852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40765[] = {SGI183853, SGI0, SGI183854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183855, SGI183856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40766[] = {SGI183857, SGI0, SGI183858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183859, SGI183860, SGI183860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40767[] = {SGI183861, SGI0, SGI183862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183865, SGI183867, SGI183867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40768[] = {SGI183863, SGI183864, SGI183866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183868, SGI183868, SGI0, SGI0, SGI0, SGI0, SGI183869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40769[] = {SGI183870, SGI0, SGI183871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183872, SGI183873, SGI183873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40770[] = {SGI183874, SGI183875, SGI183876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183877, SGI183877, SGI0, SGI0, SGI0, SGI0, SGI183878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40771[] = {SGI183879, SGI0, SGI183880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183881, SGI183882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40772[] = {SGI183883, SGI183884, SGI183885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183886, SGI183886, SGI0, SGI0, SGI0, SGI0, SGI183887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40773[] = {SGI183888, SGI0, SGI183889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183893, SGI183894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40774[] = {SGI183890, SGI183891, SGI183892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183895, SGI183895, SGI0, SGI0, SGI0, SGI0, SGI183896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40775[] = {SGI183897, SGI183898, SGI183899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183900, SGI183900, SGI0, SGI0, SGI0, SGI0, SGI183901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40776[] = {SGI183902, SGI183903, SGI183904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183905, SGI0, SGI0, SGI0, SGI183906, SGI183906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40777[] = {SGI183907, SGI183908, SGI183909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183910, SGI0, SGI0, SGI0, SGI183911, SGI183911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40778[] = {SGI183912, SGI183913, SGI183914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183915, SGI0, SGI0, SGI0, SGI183916, SGI183916, SGI0, SGI183917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40779[] = {SGI183918, SGI183919, SGI183920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183921, SGI0, SGI0, SGI0, SGI183922, SGI183922, SGI0, SGI183923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40780[] = {SGI183924, SGI0, SGI183925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183926, SGI183927, SGI183927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40781[] = {SGI183928, SGI0, SGI183929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183930, SGI183931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40782[] = {SGI183932, SGI0, SGI183933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183934, SGI183935, SGI183935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40783[] = {SGI183936, SGI0, SGI183937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183938, SGI183939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40784[] = {SGI183940, SGI0, SGI183941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183942, SGI183943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40785[] = {SGI183944, SGI0, SGI183945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183946, SGI183947, SGI183947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40786[] = {SGI183948, SGI0, SGI183949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183950, SGI183951, SGI183951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40787[] = {SGI183952, SGI0, SGI183953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183954, SGI183955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40788[] = {SGI183956, SGI183957, SGI183958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183959, SGI0, SGI0, SGI0, SGI183960, SGI183960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40789[] = {SGI183961, SGI183962, SGI183963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183964, SGI0, SGI0, SGI0, SGI183965, SGI183965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40790[] = {SGI183966, SGI183967, SGI183968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183970, SGI0, SGI0, SGI0, SGI183972, SGI183972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40791[] = {SGI183969, SGI0, SGI183971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183973, SGI183974, SGI183974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40792[] = {SGI183975, SGI183976, SGI183977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183980, SGI0, SGI0, SGI0, SGI183981, SGI183981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40793[] = {SGI183978, SGI0, SGI183979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183982, SGI183983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40794[] = {SGI183984, SGI183985, SGI183986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183987, SGI0, SGI0, SGI0, SGI183988, SGI183988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40795[] = {SGI183989, SGI183990, SGI183991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183992, SGI183992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40796[] = {SGI183993, SGI183994, SGI183995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI183996, SGI0, SGI0, SGI0, SGI183997, SGI183997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40797[] = {SGI183998, SGI0, SGI183999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184000, SGI184001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40798[] = {SGI184002, SGI184003, SGI184004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184005, SGI0, SGI0, SGI0, SGI184007, SGI184007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40799[] = {SGI184006, SGI0, SGI184008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184009, SGI184010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40800[] = {SGI184011, SGI184012, SGI184013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184014, SGI184014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40801[] = {SGI184015, SGI0, SGI184016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184017, SGI184018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40802[] = {SGI184019, SGI0, SGI184020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184021, SGI184022, SGI184022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40803[] = {SGI184023, SGI184024, SGI184025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184026, SGI0, SGI0, SGI0, SGI184027, SGI184027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40804[] = {SGI184028, SGI0, SGI184029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184030, SGI184031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40805[] = {SGI184032, SGI184033, SGI184034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184035, SGI0, SGI0, SGI184036, SGI184037, SGI184037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40806[] = {SGI184038, SGI0, SGI184039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184040, SGI184041, SGI184041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40807[] = {SGI184042, SGI0, SGI184043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184044, SGI184045, SGI184045, SGI0, SGI0, NULL};
const Segment * const SG40808[] = {SGI184046, SGI184047, SGI184048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184049, SGI0, SGI0, SGI0, SGI184050, SGI184050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40809[] = {SGI184051, SGI0, SGI184052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40810[] = {SGI184053, SGI184054, SGI184055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184057, SGI0, SGI0, SGI0, SGI184060, SGI184060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40811[] = {SGI184056, SGI184058, SGI184059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184061, SGI0, SGI0, SGI0, SGI184062, SGI184062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40812[] = {SGI184063, SGI184064, SGI184065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184066, SGI0, SGI0, SGI0, SGI184067, SGI184067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40813[] = {SGI184068, SGI184069, SGI184070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184071, SGI0, SGI0, SGI0, SGI184072, SGI184072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40814[] = {SGI184073, SGI0, SGI184074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40815[] = {SGI184075, SGI184076, SGI184077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184078, SGI0, SGI0, SGI0, SGI184079, SGI184079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40816[] = {SGI184080, SGI0, SGI184081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184082, SGI184083, SGI184083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40817[] = {SGI184084, SGI0, SGI184085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184087, SGI184088, SGI184088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40818[] = {SGI184086, SGI0, SGI184089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184090, SGI184091, SGI184091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184092, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40819[] = {SGI184093, SGI0, SGI184094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184095, SGI184096, SGI184096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184097, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40820[] = {SGI184098, SGI0, SGI184099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184100, SGI184101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184102, SGI0, SGI0, NULL};
const Segment * const SG40821[] = {SGI184103, SGI0, SGI184104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184105, SGI184106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184107, SGI0, SGI0, NULL};
const Segment * const SG40822[] = {SGI184108, SGI184109, SGI184110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184111, SGI0, SGI0, SGI0, SGI184112, SGI184112, SGI0, SGI0, NULL};
const Segment * const SG40823[] = {SGI184113, SGI0, SGI184114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184115, SGI184116, SGI184116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184117, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40824[] = {SGI184118, SGI0, SGI184119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184120, SGI184121, SGI184121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40825[] = {SGI184122, SGI0, SGI184123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184124, SGI184125, SGI184125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40826[] = {SGI0, SGI0, SGI184126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184127, SGI184128, SGI184128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40827[] = {SGI184130, SGI0, SGI184131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184133, SGI184134, SGI184134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40828[] = {SGI184129, SGI0, SGI184132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184135, SGI184136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40829[] = {SGI184137, SGI0, SGI184138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184139, SGI184140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40830[] = {SGI184141, SGI0, SGI184142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184143, SGI184144, SGI184144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40831[] = {SGI184145, SGI0, SGI184146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184147, SGI184148, SGI184148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40832[] = {SGI184149, SGI0, SGI184150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184151, SGI184152, SGI184152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40833[] = {SGI184153, SGI0, SGI184154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184155, SGI184156, SGI184156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40834[] = {SGI184157, SGI0, SGI184159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184161, SGI184162, SGI184162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40835[] = {SGI184158, SGI0, SGI184160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184163, SGI184164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40836[] = {SGI184165, SGI0, SGI184166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184167, SGI184168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40837[] = {SGI184169, SGI0, SGI184170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184171, SGI184172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40838[] = {SGI184173, SGI0, SGI184174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184175, SGI184176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40839[] = {SGI184177, SGI0, SGI184178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184179, SGI184180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40840[] = {SGI184181, SGI0, SGI184182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184183, SGI184184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40841[] = {SGI184185, SGI0, SGI184186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184187, SGI184188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40842[] = {SGI184189, SGI184190, SGI184191, SGI184192, SGI184193, SGI184193, SGI0, NULL};
const Segment * const SG40843[] = {SGI184194, SGI0, SGI184195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184196, SGI184197, SGI184197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40844[] = {SGI184198, SGI0, SGI184199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184200, SGI184201, SGI184201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40845[] = {SGI184202, SGI0, SGI184203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184204, SGI184205, SGI184205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40846[] = {SGI184206, SGI0, SGI184207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184208, SGI184209, SGI184209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40847[] = {SGI184210, SGI0, SGI184211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184212, SGI184213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40848[] = {SGI184214, SGI0, SGI184215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184217, SGI184219, SGI184219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40849[] = {SGI184216, SGI0, SGI184218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184220, SGI184221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40850[] = {SGI184222, SGI0, SGI184223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184226, SGI184227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40851[] = {SGI184224, SGI0, SGI184225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184228, SGI184229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40852[] = {SGI184230, SGI184231, SGI184232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184235, SGI0, SGI0, SGI0, SGI184237, SGI184237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40853[] = {SGI184233, SGI0, SGI184234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184239, SGI184240, SGI184240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40854[] = {SGI184236, SGI0, SGI184238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184241, SGI184242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40855[] = {SGI184243, SGI0, SGI184244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40856[] = {SGI184245, SGI0, SGI184246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184247, SGI184248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40857[] = {SGI184249, SGI0, SGI184250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184254, SGI184255, SGI184255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40858[] = {SGI184251, SGI184252, SGI184253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184256, SGI0, SGI0, SGI0, SGI184257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40859[] = {SGI184258, SGI0, SGI184259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184260, SGI184261, SGI184261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40860[] = {SGI184262, SGI0, SGI184263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184264, SGI184265, SGI184265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40861[] = {SGI184266, SGI184267, SGI184268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184269, SGI0, SGI0, SGI0, SGI184270, SGI184270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40862[] = {SGI184271, SGI0, SGI184272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184273, SGI184274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40863[] = {SGI184275, SGI184276, SGI184277, SGI184278, SGI184279, SGI184279, SGI0, NULL};
const Segment * const SG40864[] = {SGI184280, SGI0, SGI184281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184282, SGI184283, SGI184283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40865[] = {SGI184284, SGI184285, SGI184286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184287, SGI0, SGI0, SGI0, SGI184288, SGI184288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40866[] = {SGI184289, SGI0, SGI184290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184291, SGI184293, SGI184293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40867[] = {SGI184292, SGI0, SGI184294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184295, SGI184296, SGI184296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40868[] = {SGI184297, SGI0, SGI184298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184299, SGI184300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40869[] = {SGI184301, SGI184302, SGI184303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184304, SGI0, SGI0, SGI0, SGI184305, SGI184305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40870[] = {SGI184306, SGI0, SGI184307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184308, SGI184309, SGI184309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40871[] = {SGI184310, SGI0, SGI184311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184312, SGI184313, SGI184313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40872[] = {SGI184314, SGI0, SGI184315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184316, SGI184317, SGI184317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40873[] = {SGI184318, SGI0, SGI184319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184320, SGI184321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40874[] = {SGI184322, SGI0, SGI184323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184324, SGI184325, SGI184325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40875[] = {SGI184326, SGI184327, SGI184328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184329, SGI0, SGI0, SGI0, SGI184331, SGI184331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40876[] = {SGI184330, SGI0, SGI184332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184333, SGI184334, SGI184334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40877[] = {SGI184335, SGI0, SGI184336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184337, SGI184338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40878[] = {SGI184339, SGI0, SGI184340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184341, SGI184342, SGI184342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40879[] = {SGI184343, SGI0, SGI184344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184345, SGI184346, SGI184346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40880[] = {SGI184347, SGI0, SGI184348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184349, SGI184350, SGI184350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40881[] = {SGI184351, SGI0, SGI184352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184353, SGI184354, SGI184354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40882[] = {SGI184355, SGI184356, SGI184357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184358, SGI184358, SGI0, SGI0, SGI0, SGI184359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40883[] = {SGI184360, SGI184361, SGI184362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184363, SGI0, SGI0, SGI0, SGI184364, SGI184364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40884[] = {SGI184365, SGI0, SGI184366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184367, SGI184368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40885[] = {SGI184369, SGI0, SGI184370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184371, SGI184372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40886[] = {SGI0, SGI0, SGI184373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184374, SGI184375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40887[] = {SGI184376, SGI0, SGI184377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184378, SGI184379, SGI184379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40888[] = {SGI184380, SGI0, SGI184381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184382, SGI184383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40889[] = {SGI184384, SGI184385, SGI184386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184387, SGI0, SGI0, SGI0, SGI184388, SGI0, SGI0, SGI184389, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40890[] = {SGI184390, SGI184391, SGI184392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184393, SGI0, SGI0, SGI0, SGI184394, SGI0, SGI0, SGI184395, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40891[] = {SGI184396, SGI184397, SGI184398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184399, SGI0, SGI0, SGI0, SGI184401, SGI184401, SGI0, SGI184403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40892[] = {SGI184400, SGI184402, SGI184404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184405, SGI0, SGI0, SGI0, SGI184406, SGI184406, SGI0, SGI184407, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40893[] = {SGI184408, SGI184409, SGI184410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184411, SGI0, SGI0, SGI0, SGI184412, SGI0, SGI0, SGI184413, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40894[] = {SGI184414, SGI184415, SGI184416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184417, SGI0, SGI0, SGI0, SGI184418, SGI184418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40895[] = {SGI184419, SGI184420, SGI184421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184422, SGI0, SGI0, SGI0, SGI184423, SGI184423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40896[] = {SGI184424, SGI0, SGI184425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40897[] = {SGI184426, SGI0, SGI184427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184428, SGI184429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40898[] = {SGI184430, SGI0, SGI184431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184432, SGI184433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40899[] = {SGI184434, SGI184435, SGI184436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184437, SGI0, SGI0, SGI0, SGI184438, SGI184438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40900[] = {SGI184439, SGI184440, SGI184441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184442, SGI0, SGI0, SGI0, SGI184443, SGI184443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40901[] = {SGI184444, SGI184445, SGI184446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184447, SGI0, SGI0, SGI0, SGI184448, SGI184448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40902[] = {SGI184449, SGI184450, SGI184451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184452, SGI0, SGI0, SGI0, SGI184453, SGI184453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40903[] = {SGI184454, SGI0, SGI184455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40904[] = {SGI184456, SGI184457, SGI184458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184459, SGI0, SGI0, SGI0, SGI184460, SGI184460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40905[] = {SGI184461, SGI184462, SGI184463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184464, SGI0, SGI0, SGI0, SGI184465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40906[] = {SGI184466, SGI0, SGI184467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40907[] = {SGI184468, SGI184469, SGI184470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184471, SGI0, SGI0, SGI0, SGI184472, SGI184472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40908[] = {SGI184473, SGI184474, SGI184475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184476, SGI0, SGI0, SGI0, SGI184477, SGI184477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40909[] = {SGI184478, SGI0, SGI184479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40910[] = {SGI184480, SGI184481, SGI184482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184483, SGI0, SGI0, SGI0, SGI184484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40911[] = {SGI184485, SGI0, SGI184486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40912[] = {SGI184487, SGI0, SGI184488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184489, SGI184490, SGI184490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40913[] = {SGI184491, SGI184492, SGI184493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184494, SGI0, SGI0, SGI0, SGI184495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40914[] = {SGI184496, SGI0, SGI184497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184499, SGI184500, SGI184500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40915[] = {SGI184501, SGI184502, SGI184503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184504, SGI0, SGI0, SGI0, SGI184505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40916[] = {SGI184507, SGI184508, SGI184509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184510, SGI0, SGI0, SGI0, SGI184511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40917[] = {SGI184513, SGI184514, SGI184515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184516, SGI0, SGI0, SGI0, SGI184517, SGI184517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40918[] = {SGI184518, SGI0, SGI184519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40919[] = {SGI184520, SGI184521, SGI184522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184523, SGI0, SGI0, SGI0, SGI184524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40920[] = {SGI184525, SGI184526, SGI184527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184528, SGI0, SGI0, SGI0, SGI184529, SGI184529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40921[] = {SGI184530, SGI184531, SGI184532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184533, SGI0, SGI0, SGI0, SGI184535, SGI184535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40922[] = {SGI184534, SGI0, SGI184536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184537, SGI184538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40923[] = {SGI184539, SGI184540, SGI184541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184542, SGI0, SGI0, SGI0, SGI184543, SGI184543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40924[] = {SGI184544, SGI184545, SGI184546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184547, SGI0, SGI0, SGI0, SGI184548, SGI184548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40925[] = {SGI184549, SGI0, SGI184550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40926[] = {SGI184551, SGI184552, SGI184553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184554, SGI0, SGI0, SGI0, SGI184555, SGI184555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40927[] = {SGI184556, SGI0, SGI184557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184558, SGI184559, SGI184559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40928[] = {SGI184560, SGI0, SGI184561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184562, SGI184563, SGI184563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40929[] = {SGI184564, SGI184565, SGI184566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184567, SGI184567, SGI0, SGI0, SGI0, SGI184569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40930[] = {SGI184568, SGI184570, SGI184571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184572, SGI0, SGI0, SGI0, SGI184573, SGI184573, SGI0, SGI184574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40931[] = {SGI184575, SGI184576, SGI184577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184578, SGI0, SGI0, SGI0, SGI184579, SGI184579, SGI0, SGI184580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40932[] = {SGI184581, SGI184582, SGI184583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184584, SGI0, SGI0, SGI0, SGI184585, SGI184585, SGI0, SGI184586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40933[] = {SGI184587, SGI184588, SGI184589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184590, SGI0, SGI0, SGI0, SGI184593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40934[] = {SGI184591, SGI184592, SGI184594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184595, SGI0, SGI0, SGI0, SGI184596, SGI184596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40935[] = {SGI184597, SGI184598, SGI184599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184600, SGI0, SGI0, SGI0, SGI184601, SGI184601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40936[] = {SGI184602, SGI184603, SGI184604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184605, SGI0, SGI0, SGI0, SGI184606, SGI184606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40937[] = {SGI184607, SGI184608, SGI184609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184610, SGI0, SGI0, SGI0, SGI184611, SGI184611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40938[] = {SGI184612, SGI184613, SGI184614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184615, SGI0, SGI0, SGI0, SGI184616, SGI184616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40939[] = {SGI184617, SGI184618, SGI184619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184620, SGI0, SGI0, SGI0, SGI184621, SGI184621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40940[] = {SGI184622, SGI184623, SGI184624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184625, SGI0, SGI0, SGI0, SGI184626, SGI184626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40941[] = {SGI184629, SGI0, SGI184630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40942[] = {SGI184627, SGI0, SGI184628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184631, SGI0, SGI0, SGI0, SGI184632, SGI184632, SGI0, SGI0, SGI0, SGI184633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40943[] = {SGI184634, SGI184635, SGI184636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184637, SGI0, SGI0, SGI0, SGI184638, SGI184638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40944[] = {SGI184639, SGI184640, SGI184641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184642, SGI0, SGI0, SGI0, SGI184643, SGI184643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40945[] = {SGI184647, SGI0, SGI184648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40946[] = {SGI184644, SGI184645, SGI184646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184649, SGI0, SGI0, SGI0, SGI184650, SGI184650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40947[] = {SGI184651, SGI0, SGI184652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40948[] = {SGI184653, SGI184654, SGI184655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184656, SGI0, SGI0, SGI0, SGI184657, SGI184657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40949[] = {SGI184658, SGI184659, SGI184660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184662, SGI0, SGI0, SGI0, SGI184664, SGI184664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40950[] = {SGI184661, SGI0, SGI184663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40951[] = {SGI184665, SGI184666, SGI184667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184668, SGI0, SGI0, SGI0, SGI184669, SGI184669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40952[] = {SGI184670, SGI184671, SGI184672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184673, SGI0, SGI0, SGI0, SGI184674, SGI184674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40953[] = {SGI184675, SGI0, SGI184676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40954[] = {SGI184677, SGI184678, SGI184679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184680, SGI0, SGI0, SGI0, SGI184681, SGI184681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40955[] = {SGI184682, SGI184683, SGI184684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184685, SGI0, SGI0, SGI0, SGI184686, SGI184686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40956[] = {SGI184687, SGI0, SGI184688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184689, SGI184690, SGI184690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40957[] = {SGI184691, SGI0, SGI184692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184693, SGI184694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40958[] = {SGI184695, SGI184696, SGI184697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184698, SGI0, SGI0, SGI0, SGI184699, SGI184699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40959[] = {SGI184700, SGI184701, SGI184702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184705, SGI0, SGI0, SGI0, SGI184707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40960[] = {SGI184703, SGI184704, SGI184706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184708, SGI0, SGI0, SGI0, SGI184709, SGI184709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40961[] = {SGI184710, SGI184711, SGI184712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184713, SGI0, SGI0, SGI0, SGI184714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40962[] = {SGI184715, SGI0, SGI184716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40963[] = {SGI184717, SGI184718, SGI184719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184720, SGI0, SGI0, SGI0, SGI184721, SGI184721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40964[] = {SGI184722, SGI184723, SGI184724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184725, SGI0, SGI0, SGI0, SGI184726, SGI184726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40965[] = {SGI184727, SGI184728, SGI184729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184730, SGI0, SGI0, SGI0, SGI184731, SGI184731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40966[] = {SGI184732, SGI184733, SGI184734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184735, SGI0, SGI0, SGI0, SGI184736, SGI184736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40967[] = {SGI184737, SGI0, SGI184738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184739, SGI184740, SGI184740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40968[] = {SGI184741, SGI0, SGI184742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40969[] = {SGI184743, SGI0, SGI184744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184747, SGI184748, SGI184748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40970[] = {SGI184745, SGI0, SGI184746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184749, SGI184750, SGI184750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40971[] = {SGI184751, SGI0, SGI184752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184753, SGI184755, SGI184755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40972[] = {SGI184754, SGI0, SGI184756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184757, SGI184758, SGI184758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40973[] = {SGI184759, SGI0, SGI184760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184762, SGI184763, SGI184763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40974[] = {SGI184761, SGI0, SGI184764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184765, SGI184766, SGI184766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40975[] = {SGI184767, SGI0, SGI184768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184769, SGI184771, SGI184771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40976[] = {SGI184770, SGI0, SGI184772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184773, SGI184774, SGI184774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40977[] = {SGI184775, SGI0, SGI184776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184777, SGI184778, SGI184778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40978[] = {SGI184779, SGI0, SGI184780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184781, SGI184782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40979[] = {SGI184783, SGI0, SGI184784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184785, SGI184786, SGI184786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40980[] = {SGI184787, SGI0, SGI184788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184789, SGI184790, SGI184790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40981[] = {SGI184791, SGI0, SGI184792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184793, SGI184794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40982[] = {SGI184795, SGI0, SGI184796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184797, SGI184798, SGI184798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40983[] = {SGI184799, SGI0, SGI184800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184801, SGI184802, SGI184802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40984[] = {SGI184803, SGI0, SGI184804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184805, SGI184806, SGI184806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40985[] = {SGI184807, SGI184808, SGI184809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184810, SGI0, SGI0, SGI0, SGI184811, SGI184811, SGI0, SGI0, SGI184812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40986[] = {SGI184813, SGI0, SGI184815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184818, SGI184820, SGI184820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40987[] = {SGI184814, SGI184816, SGI184817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184819, SGI0, SGI0, SGI0, SGI184821, SGI184821, SGI0, SGI0, SGI184822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40988[] = {SGI184823, SGI0, SGI184824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184825, SGI184826, SGI184826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40989[] = {SGI184827, SGI184828, SGI184829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184830, SGI0, SGI0, SGI0, SGI184831, SGI184831, SGI0, SGI0, SGI184832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40990[] = {SGI184833, SGI0, SGI184834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184835, SGI0, SGI0, SGI0, SGI184837, SGI184837, SGI0, SGI0, SGI0, SGI184839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40991[] = {SGI184836, SGI0, SGI184838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184840, SGI184841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40992[] = {SGI184842, SGI0, SGI184843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184844, SGI184845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40993[] = {SGI184846, SGI0, SGI184847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184848, SGI184849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40994[] = {SGI184850, SGI0, SGI184851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184852, SGI184853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40995[] = {SGI184854, SGI184855, SGI184856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184857, SGI0, SGI0, SGI0, SGI184858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40996[] = {SGI184860, SGI184861, SGI184862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184863, SGI0, SGI0, SGI0, SGI184864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40997[] = {SGI184866, SGI184867, SGI184868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184869, SGI0, SGI0, SGI0, SGI184870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40998[] = {SGI184872, SGI0, SGI184873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG40999[] = {SGI184874, SGI184875, SGI184876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184877, SGI0, SGI0, SGI0, SGI184878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41000[] = {SGI184879, SGI184880, SGI184881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184882, SGI0, SGI0, SGI0, SGI184883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41001[] = {SGI184884, SGI184885, SGI184886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184887, SGI0, SGI0, SGI0, SGI184888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41002[] = {SGI184889, SGI184890, SGI184891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184892, SGI0, SGI0, SGI0, SGI184893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41003[] = {SGI184894, SGI0, SGI184895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184897, SGI184899, SGI184899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41004[] = {SGI184896, SGI184898, SGI184900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184901, SGI0, SGI0, SGI0, SGI184902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41005[] = {SGI184903, SGI184904, SGI184905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184906, SGI0, SGI0, SGI0, SGI184907, SGI184907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41006[] = {SGI184908, SGI0, SGI184909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41007[] = {SGI184910, SGI0, SGI184912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184915, SGI0, SGI0, SGI0, SGI184917, SGI184917, SGI0, SGI0, SGI0, SGI0, SGI184919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41008[] = {SGI184911, SGI184913, SGI184914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184916, SGI0, SGI0, SGI0, SGI184918, SGI184918, SGI0, SGI184920, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41009[] = {SGI184921, SGI184922, SGI184923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184927, SGI0, SGI0, SGI0, SGI184928, SGI184928, SGI0, SGI184930, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41010[] = {SGI184924, SGI184925, SGI184926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184929, SGI0, SGI0, SGI0, SGI184931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41011[] = {SGI184932, SGI0, SGI184933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184934, SGI0, SGI0, SGI0, SGI184935, SGI184935, SGI0, SGI0, SGI0, SGI0, SGI184936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41012[] = {SGI184937, SGI0, SGI184938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184939, SGI0, SGI0, SGI0, SGI184940, SGI184940, SGI0, SGI0, SGI0, SGI0, SGI184941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41013[] = {SGI184942, SGI184943, SGI184944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184945, SGI0, SGI0, SGI0, SGI184946, SGI184947, SGI0, SGI0, SGI0, SGI0, SGI184948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41014[] = {SGI184949, SGI0, SGI184950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184951, SGI184952, SGI184952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41015[] = {SGI184953, SGI0, SGI184954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184955, SGI184956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41016[] = {SGI184957, SGI0, SGI184958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184959, SGI184960, SGI184960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41017[] = {SGI184961, SGI184962, SGI184963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184964, SGI0, SGI0, SGI0, SGI184965, SGI184965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41018[] = {SGI184966, SGI184967, SGI184968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184969, SGI0, SGI0, SGI0, SGI184970, SGI184970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41019[] = {SGI184971, SGI184972, SGI184973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184974, SGI0, SGI0, SGI0, SGI184975, SGI184975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41020[] = {SGI184976, SGI0, SGI184977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184978, SGI184979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41021[] = {SGI184980, SGI184981, SGI184982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184983, SGI0, SGI0, SGI0, SGI184984, SGI184984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41022[] = {SGI184985, SGI0, SGI184986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41023[] = {SGI184987, SGI184988, SGI184989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184990, SGI0, SGI0, SGI0, SGI184993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41024[] = {SGI184991, SGI184992, SGI184994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI184995, SGI0, SGI0, SGI0, SGI184996, SGI184996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41025[] = {SGI184997, SGI184998, SGI184999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185000, SGI0, SGI0, SGI0, SGI185001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41026[] = {SGI185002, SGI0, SGI185003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185004, SGI185005, SGI185005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41027[] = {SGI185006, SGI185007, SGI185008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185009, SGI0, SGI0, SGI0, SGI185010, SGI185010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41028[] = {SGI185011, SGI0, SGI185012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185013, SGI185014, SGI185014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41029[] = {SGI185015, SGI185016, SGI185017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185018, SGI0, SGI0, SGI0, SGI185019, SGI185019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41030[] = {SGI185020, SGI0, SGI185021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185022, SGI185023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41031[] = {SGI185024, SGI185025, SGI185026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185028, SGI0, SGI0, SGI0, SGI185031, SGI185031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41032[] = {SGI185027, SGI185029, SGI185030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185032, SGI0, SGI0, SGI0, SGI185033, SGI185033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41033[] = {SGI185034, SGI0, SGI185035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185036, SGI185037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41034[] = {SGI185038, SGI185039, SGI185040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185041, SGI0, SGI0, SGI0, SGI185042, SGI185042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41035[] = {SGI185043, SGI0, SGI185044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185045, SGI185046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41036[] = {SGI185047, SGI0, SGI185048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185049, SGI185050, SGI185050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41037[] = {SGI185051, SGI0, SGI185052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185053, SGI185054, SGI185054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41038[] = {SGI185055, SGI0, SGI185056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185057, SGI185058, SGI185058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41039[] = {SGI185059, SGI0, SGI185060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185061, SGI185062, SGI185062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41040[] = {SGI185063, SGI185064, SGI185065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185066, SGI0, SGI0, SGI0, SGI185067, SGI185067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41041[] = {SGI185069, SGI0, SGI185070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185071, SGI185073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41042[] = {SGI185072, SGI185074, SGI185075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185076, SGI0, SGI0, SGI0, SGI185077, SGI185077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41043[] = {SGI185079, SGI0, SGI185080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185082, SGI185083, SGI185083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41044[] = {SGI185081, SGI185084, SGI185085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185086, SGI0, SGI0, SGI0, SGI185087, SGI185087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41045[] = {SGI185089, SGI0, SGI185090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185091, SGI185092, SGI185092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41046[] = {SGI185093, SGI0, SGI185094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185095, SGI185096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41047[] = {SGI185097, SGI0, SGI185098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185099, SGI185100, SGI185100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41048[] = {SGI185101, SGI185102, SGI185103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185104, SGI0, SGI0, SGI0, SGI185105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41049[] = {SGI185106, SGI0, SGI185107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185110, SGI185112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41050[] = {SGI185108, SGI185109, SGI185111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185113, SGI0, SGI0, SGI0, SGI185116, SGI185116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41051[] = {SGI185114, SGI0, SGI185117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41052[] = {SGI185115, SGI185118, SGI185119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185120, SGI0, SGI0, SGI0, SGI185121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41053[] = {SGI185122, SGI0, SGI185123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41054[] = {SGI185124, SGI0, SGI185125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41055[] = {SGI185126, SGI185127, SGI185128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185129, SGI0, SGI0, SGI0, SGI185130, SGI185130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41056[] = {SGI185131, SGI185132, SGI185133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185134, SGI185134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41057[] = {SGI185137, SGI0, SGI185139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41058[] = {SGI185135, SGI185136, SGI185138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185140, SGI0, SGI0, SGI0, SGI185141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41059[] = {SGI185142, SGI185143, SGI185144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185145, SGI0, SGI0, SGI0, SGI185148, SGI185148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41060[] = {SGI185146, SGI185147, SGI185149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185150, SGI0, SGI0, SGI0, SGI185151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41061[] = {SGI185152, SGI0, SGI185153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41062[] = {SGI185154, SGI185155, SGI185156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185157, SGI0, SGI0, SGI0, SGI185158, SGI185158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41063[] = {SGI185159, SGI0, SGI185160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41064[] = {SGI185161, SGI0, SGI185162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185163, SGI185164, SGI185164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41065[] = {SGI185165, SGI0, SGI185166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185167, SGI185168, SGI185168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41066[] = {SGI185169, SGI0, SGI185170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185171, SGI185172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41067[] = {SGI185173, SGI0, SGI185174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185175, SGI185176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41068[] = {SGI185177, SGI0, SGI185178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185179, SGI185180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41069[] = {SGI185181, SGI0, SGI185182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185183, SGI185184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41070[] = {SGI185185, SGI185186, SGI185187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185188, SGI0, SGI0, SGI0, SGI185189, SGI185189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41071[] = {SGI185190, SGI185191, SGI185192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185193, SGI0, SGI0, SGI0, SGI185194, SGI185194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41072[] = {SGI185195, SGI185196, SGI185197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185198, SGI0, SGI0, SGI0, SGI185201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41073[] = {SGI185199, SGI0, SGI185200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41074[] = {SGI185203, SGI185205, SGI185206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185209, SGI185209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41075[] = {SGI185202, SGI0, SGI185204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185207, SGI185208, SGI185208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41076[] = {SGI185210, SGI185211, SGI185213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185215, SGI0, SGI0, SGI0, SGI185216, SGI185216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41077[] = {SGI185212, SGI0, SGI185214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185217, SGI185218, SGI185218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41078[] = {SGI185219, SGI185220, SGI185221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185225, SGI0, SGI0, SGI0, SGI185226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41079[] = {SGI185222, SGI185223, SGI185224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185227, SGI0, SGI0, SGI185228, SGI185229, SGI185229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41080[] = {SGI185230, SGI0, SGI185231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185232, SGI185233, SGI185233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185234, SGI0, SGI0, NULL};
const Segment * const SG41081[] = {SGI185235, SGI0, SGI185236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185237, SGI185239, SGI185239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185241, SGI0, SGI0, NULL};
const Segment * const SG41082[] = {SGI185238, SGI0, SGI185240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185242, SGI185243, SGI185243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41083[] = {SGI185244, SGI0, SGI185245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185246, SGI185248, SGI185248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185250, SGI0, SGI0, NULL};
const Segment * const SG41084[] = {SGI185247, SGI0, SGI185249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185251, SGI185252, SGI185252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41085[] = {SGI185253, SGI0, SGI185254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185255, SGI185256, SGI185256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41086[] = {SGI185257, SGI185258, SGI185259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185260, SGI0, SGI0, SGI0, SGI185261, SGI185261, SGI0, SGI0, NULL};
const Segment * const SG41087[] = {SGI185262, SGI0, SGI185263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185264, SGI185265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41088[] = {SGI185266, SGI185267, SGI185268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185269, SGI0, SGI0, SGI0, SGI185270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41089[] = {SGI185271, SGI185272, SGI185273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185274, SGI0, SGI0, SGI0, SGI185275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41090[] = {SGI185276, SGI185277, SGI185278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185279, SGI0, SGI0, SGI0, SGI185280, SGI185280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185281, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41091[] = {SGI185282, SGI0, SGI185283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185284, SGI185285, SGI185285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41092[] = {SGI185286, SGI185287, SGI185288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185289, SGI0, SGI0, SGI0, SGI185290, SGI185290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41093[] = {SGI185292, SGI185293, SGI185294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185295, SGI0, SGI0, SGI0, SGI185296, SGI185296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41094[] = {SGI185297, SGI0, SGI185298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185299, SGI0, SGI0, SGI0, SGI185300, SGI185300, SGI0, SGI0, SGI0, SGI185301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41095[] = {SGI185302, SGI185303, SGI185304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185305, SGI0, SGI0, SGI0, SGI185306, SGI185307, SGI0, SGI0, SGI0, SGI185308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41096[] = {SGI185309, SGI185310, SGI185311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185312, SGI0, SGI0, SGI0, SGI185313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41097[] = {SGI185314, SGI185315, SGI185316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185317, SGI0, SGI0, SGI0, SGI185318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41098[] = {SGI185319, SGI0, SGI185320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185324, SGI185325, SGI185325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41099[] = {SGI185321, SGI185322, SGI185323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185326, SGI0, SGI0, SGI0, SGI185327, SGI185327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41100[] = {SGI185329, SGI0, SGI185331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41101[] = {SGI185328, SGI0, SGI185330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185332, SGI185333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41102[] = {SGI185334, SGI0, SGI185335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185336, SGI185337, SGI185337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41103[] = {SGI185339, SGI0, SGI185341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185343, SGI185344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41104[] = {SGI185338, SGI0, SGI185340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185342, SGI185345, SGI185345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41105[] = {SGI185346, SGI0, SGI185347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185348, SGI185349, SGI185349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41106[] = {SGI185351, SGI0, SGI185353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185356, SGI185357, SGI185357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41107[] = {SGI185350, SGI185352, SGI185354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185355, SGI0, SGI0, SGI0, SGI185358, SGI185358, SGI0, SGI185359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41108[] = {SGI185360, SGI185361, SGI185362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185363, SGI0, SGI0, SGI0, SGI185364, SGI185364, SGI0, SGI185365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41109[] = {SGI185366, SGI0, SGI185367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185368, SGI185369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41110[] = {SGI185370, SGI185371, SGI185372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185373, SGI0, SGI0, SGI0, SGI185374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41111[] = {SGI185375, SGI0, SGI185376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185378, SGI185379, SGI185379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41112[] = {SGI185377, SGI0, SGI185380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41113[] = {SGI185381, SGI0, SGI185382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185383, SGI185384, SGI185384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185385, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41114[] = {SGI185386, SGI185387, SGI185388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185389, SGI0, SGI0, SGI0, SGI185390, SGI185390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185391, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41115[] = {SGI185392, SGI0, SGI185393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185394, SGI185395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41116[] = {SGI185396, SGI0, SGI185397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185398, SGI185399, SGI185399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41117[] = {SGI185400, SGI0, SGI185401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185402, SGI185403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41118[] = {SGI185404, SGI0, SGI185405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185406, SGI185408, SGI185408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41119[] = {SGI185407, SGI0, SGI185409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185410, SGI185411, SGI185411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41120[] = {SGI185412, SGI0, SGI185413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185414, SGI185415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41121[] = {SGI185416, SGI0, SGI185417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185418, SGI185419, SGI185419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41122[] = {SGI185420, SGI0, SGI185421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185422, SGI185423, SGI185423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41123[] = {SGI185424, SGI0, SGI185425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185426, SGI185427, SGI185427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41124[] = {SGI185428, SGI0, SGI185429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185430, SGI185431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41125[] = {SGI185432, SGI0, SGI185433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185434, SGI185435, SGI185435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41126[] = {SGI185436, SGI0, SGI185437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185438, SGI185439, SGI185439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41127[] = {SGI185440, SGI0, SGI185441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185442, SGI185443, SGI185443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41128[] = {SGI185444, SGI185445, SGI185446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185447, SGI0, SGI0, SGI0, SGI185448, SGI185448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41129[] = {SGI185449, SGI185450, SGI185451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185452, SGI0, SGI0, SGI0, SGI185453, SGI185453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41130[] = {SGI185454, SGI185455, SGI185456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185457, SGI0, SGI0, SGI0, SGI185458, SGI185458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41131[] = {SGI185459, SGI0, SGI185460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185461, SGI185462, SGI185462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41132[] = {SGI185463, SGI0, SGI185464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185465, SGI185466, SGI185466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41133[] = {SGI185467, SGI0, SGI185468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185469, SGI185470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41134[] = {SGI185471, SGI0, SGI185472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185473, SGI185474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41135[] = {SGI185475, SGI0, SGI185476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185477, SGI185478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41136[] = {SGI185479, SGI185480, SGI185481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185482, SGI0, SGI0, SGI0, SGI185483, SGI185483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41137[] = {SGI185485, SGI185486, SGI185487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185488, SGI0, SGI0, SGI0, SGI185489, SGI185489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41138[] = {SGI185491, SGI0, SGI185492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185493, SGI185494, SGI185494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41139[] = {SGI185495, SGI185496, SGI185497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185498, SGI185498, SGI0, SGI0, SGI185499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41140[] = {SGI185500, SGI185501, SGI185502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185503, SGI185503, SGI0, SGI0, SGI185504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41141[] = {SGI185505, SGI185506, SGI185507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185508, SGI0, SGI0, SGI0, SGI185509, SGI185509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41142[] = {SGI185512, SGI0, SGI185513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41143[] = {SGI185510, SGI0, SGI185511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185515, SGI185516, SGI185516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41144[] = {SGI185514, SGI0, SGI185517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185518, SGI185519, SGI185519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41145[] = {SGI185520, SGI0, SGI185521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185524, SGI0, SGI0, SGI0, SGI185525, SGI185525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41146[] = {SGI185522, SGI0, SGI185523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185526, SGI185527, SGI185527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41147[] = {SGI185528, SGI185529, SGI185530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185531, SGI0, SGI0, SGI0, SGI185532, SGI185532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41148[] = {SGI185533, SGI185534, SGI185535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185536, SGI0, SGI0, SGI0, SGI185537, SGI185537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41149[] = {SGI185538, SGI0, SGI185539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185540, SGI185541, SGI185541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41150[] = {SGI185542, SGI0, SGI185543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185544, SGI185545, SGI185545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41151[] = {SGI185546, SGI0, SGI185547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185548, SGI185549, SGI185549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41152[] = {SGI185550, SGI0, SGI185551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185552, SGI185553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41153[] = {SGI185554, SGI185555, SGI185556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185557, SGI0, SGI0, SGI0, SGI185558, SGI185558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41154[] = {SGI185559, SGI0, SGI185560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185561, SGI185562, SGI185562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41155[] = {SGI185563, SGI185564, SGI185565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185566, SGI0, SGI0, SGI0, SGI185567, SGI185567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41156[] = {SGI185568, SGI0, SGI185569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41157[] = {SGI185570, SGI185571, SGI185572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185573, SGI0, SGI0, SGI0, SGI185574, SGI185574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41158[] = {SGI185575, SGI0, SGI185576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185577, SGI185578, SGI185578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41159[] = {SGI185579, SGI0, SGI185580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185581, SGI185583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41160[] = {SGI185582, SGI0, SGI185584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185585, SGI185586, SGI185586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185587, SGI0, SGI0, NULL};
const Segment * const SG41161[] = {SGI185588, SGI185589, SGI185590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185592, SGI0, SGI0, SGI0, SGI185595, SGI185595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41162[] = {SGI185591, SGI185593, SGI185594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185596, SGI0, SGI0, SGI0, SGI185597, SGI185597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41163[] = {SGI185598, SGI0, SGI185599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41164[] = {SGI185600, SGI0, SGI185601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41165[] = {SGI185602, SGI185603, SGI185604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185606, SGI0, SGI0, SGI0, SGI185608, SGI185608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41166[] = {SGI185605, SGI0, SGI185607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185609, SGI185610, SGI185610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41167[] = {SGI185611, SGI0, SGI185612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41168[] = {SGI185613, SGI185614, SGI185615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185616, SGI185616, SGI0, SGI0, SGI0, SGI185617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41169[] = {SGI185618, SGI185619, SGI185620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185621, SGI0, SGI0, SGI0, SGI185622, SGI185622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41170[] = {SGI185623, SGI0, SGI185624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41171[] = {SGI185625, SGI185626, SGI185627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185628, SGI0, SGI0, SGI0, SGI185629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41172[] = {SGI185630, SGI0, SGI185631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185632, SGI185633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41173[] = {SGI185634, SGI185635, SGI185636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185637, SGI0, SGI0, SGI0, SGI185638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41174[] = {SGI185639, SGI185640, SGI185641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185642, SGI0, SGI0, SGI0, SGI185643, SGI185643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41175[] = {SGI185644, SGI185645, SGI185646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185647, SGI0, SGI0, SGI0, SGI185648, SGI185648, SGI0, SGI185649, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41176[] = {SGI185650, SGI185651, SGI185652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185653, SGI0, SGI0, SGI0, SGI185654, SGI185654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41177[] = {SGI185655, SGI185656, SGI185657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185658, SGI0, SGI0, SGI0, SGI185659, SGI185659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41178[] = {SGI185660, SGI185661, SGI185662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185663, SGI0, SGI0, SGI0, SGI185664, SGI185664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41179[] = {SGI185665, SGI185666, SGI185667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185668, SGI0, SGI0, SGI0, SGI185669, SGI185669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41180[] = {SGI185670, SGI185671, SGI185672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185673, SGI0, SGI0, SGI0, SGI185674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41181[] = {SGI185675, SGI0, SGI185676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185677, SGI185678, SGI185678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41182[] = {SGI185679, SGI0, SGI185680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185681, SGI185682, SGI185682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41183[] = {SGI185683, SGI0, SGI185684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185685, SGI185686, SGI185686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41184[] = {SGI185687, SGI0, SGI185688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185689, SGI185690, SGI185690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41185[] = {SGI185691, SGI0, SGI185692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185693, SGI185694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41186[] = {SGI185695, SGI0, SGI185696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185697, SGI185698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41187[] = {SGI185700, SGI0, SGI185702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185704, SGI185705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41188[] = {SGI185699, SGI0, SGI185701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185703, SGI185706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41189[] = {SGI185707, SGI0, SGI185708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185709, SGI185710, SGI185710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41190[] = {SGI185711, SGI185712, SGI185713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185714, SGI0, SGI0, SGI0, SGI185715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41191[] = {SGI185716, SGI185717, SGI185719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185722, SGI0, SGI0, SGI0, SGI185724, SGI0, SGI0, SGI185726, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41192[] = {SGI185718, SGI185720, SGI185721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185723, SGI0, SGI0, SGI0, SGI185725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41193[] = {SGI185727, SGI185728, SGI185729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185730, SGI0, SGI0, SGI0, SGI185731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41194[] = {SGI185732, SGI185733, SGI185734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185735, SGI0, SGI0, SGI0, SGI185736, SGI0, SGI0, SGI185737, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41195[] = {SGI185738, SGI185739, SGI185740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185741, SGI0, SGI0, SGI0, SGI185742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41196[] = {SGI185743, SGI185744, SGI185745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185746, SGI0, SGI0, SGI0, SGI185749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41197[] = {SGI185747, SGI185748, SGI185750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185751, SGI0, SGI0, SGI0, SGI185752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41198[] = {SGI185753, SGI0, SGI185754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185755, SGI185756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41199[] = {SGI185757, SGI185758, SGI185759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185761, SGI0, SGI0, SGI0, SGI185763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41200[] = {SGI185760, SGI0, SGI185762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185764, SGI185765, SGI185765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41201[] = {SGI185766, SGI0, SGI185767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185770, SGI185771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41202[] = {SGI185768, SGI0, SGI185769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185772, SGI185773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41203[] = {SGI185774, SGI0, SGI185775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185776, SGI185777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41204[] = {SGI185778, SGI0, SGI185779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185780, SGI0, SGI0, SGI0, SGI185781, SGI185781, SGI0, SGI0, SGI0, SGI185782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41205[] = {SGI185783, SGI0, SGI185784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185785, SGI185786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41206[] = {SGI185787, SGI0, SGI185788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185789, SGI0, SGI0, SGI0, SGI185790, SGI185790, SGI0, SGI0, SGI0, SGI185792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41207[] = {SGI185791, SGI0, SGI185793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185794, SGI185795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41208[] = {SGI185796, SGI185797, SGI185798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185799, SGI0, SGI0, SGI0, SGI185801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41209[] = {SGI185800, SGI0, SGI185802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185803, SGI185804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41210[] = {SGI185805, SGI185806, SGI185807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185808, SGI0, SGI0, SGI0, SGI185809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41211[] = {SGI185810, SGI185811, SGI185812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185813, SGI0, SGI0, SGI0, SGI185814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185815, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41212[] = {SGI185816, SGI185817, SGI185818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185819, SGI0, SGI0, SGI0, SGI185820, SGI185820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185821, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41213[] = {SGI185822, SGI185823, SGI185824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185825, SGI0, SGI0, SGI0, SGI185826, SGI185826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185827, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41214[] = {SGI185828, SGI185829, SGI185830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185831, SGI0, SGI0, SGI0, SGI185832, SGI185832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185833, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41215[] = {SGI185834, SGI0, SGI185835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185839, SGI185840, SGI185840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41216[] = {SGI185836, SGI185837, SGI185838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185841, SGI0, SGI0, SGI0, SGI185842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185843, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41217[] = {SGI185844, SGI0, SGI185845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185846, SGI185847, SGI185847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41218[] = {SGI185848, SGI0, SGI185849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185850, SGI185851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185852, SGI0, SGI0, NULL};
const Segment * const SG41219[] = {SGI185853, SGI0, SGI185854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185855, SGI185856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185857, SGI0, SGI0, NULL};
const Segment * const SG41220[] = {SGI185858, SGI185859, SGI185860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185861, SGI0, SGI0, SGI0, SGI185862, SGI185862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41221[] = {SGI185863, SGI0, SGI185864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185865, SGI185866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185867, SGI0, SGI0, NULL};
const Segment * const SG41222[] = {SGI185868, SGI185869, SGI185870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185871, SGI0, SGI0, SGI0, SGI185872, SGI185872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41223[] = {SGI185873, SGI185874, SGI185875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185876, SGI0, SGI0, SGI0, SGI185877, SGI185877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41224[] = {SGI185878, SGI185879, SGI185880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185881, SGI0, SGI0, SGI0, SGI185882, SGI185882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41225[] = {SGI185883, SGI0, SGI185884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185885, SGI185886, SGI185886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41226[] = {SGI185887, SGI185888, SGI185889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185892, SGI0, SGI0, SGI0, SGI185893, SGI185893, SGI0, SGI185895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41227[] = {SGI185890, SGI0, SGI185891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185894, SGI185896, SGI185896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41228[] = {SGI185897, SGI185898, SGI185899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185903, SGI0, SGI0, SGI0, SGI185904, SGI185904, SGI0, SGI185906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41229[] = {SGI185900, SGI185901, SGI185902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185905, SGI0, SGI0, SGI0, SGI185907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41230[] = {SGI185908, SGI185909, SGI185910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185914, SGI0, SGI0, SGI0, SGI185915, SGI185915, SGI0, SGI185917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41231[] = {SGI185911, SGI185912, SGI185913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185916, SGI0, SGI0, SGI0, SGI185918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41232[] = {SGI185919, SGI185920, SGI185921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185922, SGI0, SGI0, SGI0, SGI185923, SGI185923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41233[] = {SGI185925, SGI185926, SGI185927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185928, SGI0, SGI0, SGI0, SGI185929, SGI185929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41234[] = {SGI185930, SGI185931, SGI185932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185933, SGI0, SGI0, SGI0, SGI185934, SGI185934, SGI0, SGI185935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41235[] = {SGI185936, SGI0, SGI185937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185938, SGI185939, SGI185939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41236[] = {SGI185940, SGI185941, SGI185942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185943, SGI0, SGI0, SGI185944, SGI185945, SGI185945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41237[] = {SGI185946, SGI0, SGI185947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185948, SGI185949, SGI185949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41238[] = {SGI185950, SGI185951, SGI185952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185953, SGI0, SGI0, SGI0, SGI185954, SGI185954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41239[] = {SGI185955, SGI0, SGI185956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185957, SGI185958, SGI185958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41240[] = {SGI185959, SGI0, SGI185960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185961, SGI185962, SGI185962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41241[] = {SGI185963, SGI185964, SGI185965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185966, SGI0, SGI0, SGI0, SGI185967, SGI185967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41242[] = {SGI185968, SGI0, SGI185969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185970, SGI185971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41243[] = {SGI185972, SGI185973, SGI185974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185975, SGI0, SGI0, SGI0, SGI185976, SGI185976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41244[] = {SGI185977, SGI185978, SGI185979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185980, SGI0, SGI0, SGI0, SGI185981, SGI185981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41245[] = {SGI185982, SGI185983, SGI185984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185985, SGI0, SGI0, SGI0, SGI185986, SGI185986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41246[] = {SGI185987, SGI0, SGI185988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41247[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185989, SGI0, SGI0, SGI185990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41248[] = {SGI185991, SGI0, SGI185992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI185993, SGI185994, SGI185994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41249[] = {SGI185995, SGI0, SGI185996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186000, SGI186001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41250[] = {SGI185997, SGI185998, SGI185999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186002, SGI0, SGI0, SGI0, SGI186003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41251[] = {SGI186004, SGI186005, SGI186006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186007, SGI0, SGI0, SGI0, SGI186008, SGI186008, SGI0, SGI186009, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41252[] = {SGI186010, SGI186011, SGI186012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186013, SGI0, SGI0, SGI0, SGI186014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41253[] = {SGI186015, SGI186016, SGI186017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186018, SGI0, SGI0, SGI0, SGI186019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41254[] = {SGI186020, SGI186021, SGI186022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186023, SGI0, SGI0, SGI0, SGI186024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41255[] = {SGI186025, SGI0, SGI186026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186027, SGI186028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41256[] = {SGI186029, SGI0, SGI186030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186031, SGI186032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41257[] = {SGI186033, SGI0, SGI186034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186037, SGI186038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41258[] = {SGI186035, SGI0, SGI186036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186039, SGI186040, SGI186040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41259[] = {SGI186041, SGI186042, SGI186043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186044, SGI186044, SGI0, SGI0, SGI0, SGI186045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41260[] = {SGI186046, SGI186047, SGI186048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186049, SGI0, SGI0, SGI0, SGI186050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41261[] = {SGI186051, SGI0, SGI186052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41262[] = {SGI186053, SGI186054, SGI186055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186056, SGI0, SGI0, SGI0, SGI186057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41263[] = {SGI186058, SGI0, SGI186059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186060, SGI0, SGI0, SGI0, SGI186061, SGI186061, SGI0, SGI0, SGI0, SGI186062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41264[] = {SGI186063, SGI0, SGI186064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186065, SGI186066, SGI186066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41265[] = {SGI186067, SGI186068, SGI186069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186070, SGI0, SGI0, SGI0, SGI186071, SGI186071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41266[] = {SGI186072, SGI0, SGI186073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41267[] = {SGI186074, SGI186075, SGI186076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186079, SGI0, SGI0, SGI0, SGI186081, SGI186081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41268[] = {SGI186077, SGI186078, SGI186080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186082, SGI0, SGI0, SGI0, SGI186083, SGI186083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41269[] = {SGI186084, SGI0, SGI186085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41270[] = {SGI186086, SGI186087, SGI186088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186089, SGI0, SGI0, SGI0, SGI186090, SGI186090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41271[] = {SGI186091, SGI186092, SGI186093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186094, SGI0, SGI0, SGI0, SGI186095, SGI186095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41272[] = {SGI186096, SGI186097, SGI186098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186099, SGI0, SGI0, SGI0, SGI186100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41273[] = {SGI186101, SGI0, SGI186102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41274[] = {SGI186103, SGI0, SGI186104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41275[] = {SGI186105, SGI186106, SGI186107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186108, SGI0, SGI0, SGI0, SGI186109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41276[] = {SGI186110, SGI0, SGI186111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186112, SGI186113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41277[] = {SGI186114, SGI0, SGI186115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186116, SGI186117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41278[] = {SGI186118, SGI186119, SGI186120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186121, SGI0, SGI0, SGI0, SGI186122, SGI186122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41279[] = {SGI186123, SGI186124, SGI186125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186127, SGI0, SGI0, SGI0, SGI186130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41280[] = {SGI186126, SGI186128, SGI186129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186131, SGI0, SGI0, SGI0, SGI186132, SGI186132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41281[] = {SGI186133, SGI0, SGI186135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41282[] = {SGI186134, SGI186136, SGI186137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186138, SGI0, SGI0, SGI0, SGI186141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41283[] = {SGI186139, SGI0, SGI186140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186142, SGI186143, SGI186143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41284[] = {SGI186144, SGI186145, SGI186146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186147, SGI0, SGI0, SGI0, SGI186148, SGI186148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41285[] = {SGI186149, SGI0, SGI186150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186151, SGI0, SGI0, SGI0, SGI186152, SGI186152, SGI0, SGI0, SGI0, SGI186153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41286[] = {SGI186154, SGI0, SGI186155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186156, SGI186157, SGI186157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41287[] = {SGI0, SGI0, SGI186158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186159, SGI186160, SGI186160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41288[] = {SGI186161, SGI0, SGI186162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186163, SGI186164, SGI186164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41289[] = {SGI186165, SGI186166, SGI186167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186168, SGI186168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186169, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41290[] = {SGI186170, SGI186171, SGI186172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186173, SGI0, SGI0, SGI0, SGI186174, SGI186174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41291[] = {SGI186175, SGI186176, SGI186177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186179, SGI0, SGI0, SGI0, SGI186182, SGI186182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41292[] = {SGI186178, SGI186180, SGI186181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186183, SGI0, SGI0, SGI0, SGI186184, SGI186184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41293[] = {SGI186188, SGI0, SGI186189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41294[] = {SGI186185, SGI186186, SGI186187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186190, SGI0, SGI0, SGI0, SGI186191, SGI186191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41295[] = {SGI186192, SGI0, SGI186193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186194, SGI186195, SGI186195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41296[] = {SGI186196, SGI186197, SGI186198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186199, SGI0, SGI0, SGI0, SGI186200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41297[] = {SGI186201, SGI0, SGI186202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186203, SGI186204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41298[] = {SGI186205, SGI0, SGI186206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41299[] = {SGI186207, SGI186208, SGI186209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186210, SGI0, SGI0, SGI0, SGI186211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41300[] = {SGI186212, SGI186213, SGI186214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186215, SGI0, SGI0, SGI0, SGI186216, SGI186216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41301[] = {SGI186217, SGI0, SGI186218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186219, SGI186220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41302[] = {SGI186221, SGI0, SGI186222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186223, SGI186224, SGI186224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41303[] = {SGI186225, SGI186226, SGI186227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186228, SGI0, SGI0, SGI0, SGI186229, SGI186229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186230, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41304[] = {SGI186231, SGI186232, SGI186233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186234, SGI0, SGI0, SGI0, SGI186235, SGI186235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186236, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41305[] = {SGI186237, SGI186238, SGI186239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186240, SGI0, SGI0, SGI0, SGI186241, SGI186241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186242, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41306[] = {SGI186243, SGI186244, SGI186245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186246, SGI0, SGI0, SGI0, SGI186247, SGI186247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41307[] = {SGI186248, SGI0, SGI186249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41308[] = {SGI186250, SGI0, SGI186251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186252, SGI186253, SGI186253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41309[] = {SGI186254, SGI0, SGI186255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186256, SGI186257, SGI186257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41310[] = {SGI186258, SGI0, SGI186259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186260, SGI186261, SGI186261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41311[] = {SGI186262, SGI0, SGI186263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186264, SGI186265, SGI186265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41312[] = {SGI186266, SGI186267, SGI186268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186269, SGI0, SGI0, SGI0, SGI186270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41313[] = {SGI186271, SGI186272, SGI186273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186274, SGI0, SGI0, SGI0, SGI186275, SGI186275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41314[] = {SGI186276, SGI186277, SGI186278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186279, SGI0, SGI0, SGI0, SGI186280, SGI186280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41315[] = {SGI186281, SGI186282, SGI186283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186284, SGI0, SGI0, SGI0, SGI186285, SGI186285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41316[] = {SGI186286, SGI0, SGI186287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186289, SGI186290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41317[] = {SGI186288, SGI186291, SGI186292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186293, SGI0, SGI0, SGI0, SGI186294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41318[] = {SGI186295, SGI0, SGI186296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186297, SGI186298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41319[] = {SGI186299, SGI186300, SGI186301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186302, SGI0, SGI0, SGI0, SGI186303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41320[] = {SGI186304, SGI186305, SGI186306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186307, SGI0, SGI0, SGI0, SGI186308, SGI186308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41321[] = {SGI186309, SGI0, SGI186310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186311, SGI186312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41322[] = {SGI186313, SGI0, SGI186314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186315, SGI186316, SGI186316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41323[] = {SGI186317, SGI0, SGI186318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186319, SGI186320, SGI186320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41324[] = {SGI186321, SGI0, SGI186322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186323, SGI186324, SGI186324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41325[] = {SGI186325, SGI186326, SGI186327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186328, SGI0, SGI0, SGI0, SGI186329, SGI186329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41326[] = {SGI186330, SGI186331, SGI186332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186336, SGI0, SGI0, SGI0, SGI186337, SGI186337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41327[] = {SGI186333, SGI186334, SGI186335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186338, SGI0, SGI0, SGI0, SGI186339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41328[] = {SGI186340, SGI0, SGI186341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186342, SGI186343, SGI186343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41329[] = {SGI186344, SGI0, SGI186345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186346, SGI186347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41330[] = {SGI186348, SGI0, SGI186349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186350, SGI186351, SGI186351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41331[] = {SGI0, SGI0, SGI186352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186353, SGI186354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41332[] = {SGI186355, SGI0, SGI186356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186357, SGI186358, SGI186358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41333[] = {SGI186359, SGI0, SGI186360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186361, SGI186362, SGI186362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41334[] = {SGI186363, SGI0, SGI186364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186365, SGI186366, SGI186366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41335[] = {SGI186367, SGI0, SGI186368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186369, SGI186370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41336[] = {SGI186371, SGI0, SGI186372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186374, SGI186376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41337[] = {SGI186373, SGI186375, SGI186377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186378, SGI0, SGI0, SGI0, SGI186379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41338[] = {SGI186380, SGI186381, SGI186382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186384, SGI0, SGI0, SGI0, SGI186386, SGI186386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41339[] = {SGI186383, SGI186385, SGI186387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186389, SGI0, SGI0, SGI0, SGI186390, SGI186390, SGI186391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41340[] = {SGI186392, SGI186393, SGI186394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186398, SGI0, SGI0, SGI0, SGI186399, SGI186399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41341[] = {SGI186395, SGI186396, SGI186397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186400, SGI0, SGI0, SGI0, SGI186402, SGI186402, SGI186403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41342[] = {SGI186404, SGI186405, SGI186406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186407, SGI0, SGI0, SGI0, SGI186408, SGI186408, SGI186409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41343[] = {SGI186410, SGI186411, SGI186412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186413, SGI0, SGI0, SGI0, SGI186414, SGI186414, SGI186415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41344[] = {SGI186416, SGI186417, SGI186418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186419, SGI0, SGI0, SGI0, SGI186420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41345[] = {SGI186421, SGI0, SGI186422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186423, SGI186424, SGI186424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41346[] = {SGI186425, SGI0, SGI186426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186427, SGI186428, SGI186428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41347[] = {SGI186429, SGI186430, SGI186431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186432, SGI0, SGI0, SGI0, SGI186433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41348[] = {SGI186434, SGI0, SGI186435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186436, SGI0, SGI0, SGI0, SGI186437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41349[] = {SGI186438, SGI186439, SGI186440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186441, SGI0, SGI0, SGI0, SGI186442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41350[] = {SGI186443, SGI186444, SGI186445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186446, SGI0, SGI0, SGI0, SGI186447, SGI186447, SGI0, SGI0, SGI186448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41351[] = {SGI186449, SGI186450, SGI186452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186455, SGI0, SGI0, SGI0, SGI186457, SGI186457, SGI0, SGI0, SGI186459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41352[] = {SGI186451, SGI186453, SGI186454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186456, SGI0, SGI0, SGI0, SGI186458, SGI0, SGI0, SGI186460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41353[] = {SGI186461, SGI0, SGI186462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186463, SGI186464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41354[] = {SGI186465, SGI0, SGI186466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186467, SGI0, SGI0, SGI0, SGI186468, SGI0, SGI0, SGI186469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41355[] = {SGI186470, SGI186471, SGI186472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186473, SGI0, SGI0, SGI0, SGI186474, SGI0, SGI0, SGI186475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41356[] = {SGI186476, SGI0, SGI186477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186478, SGI186479, SGI186479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41357[] = {SGI186480, SGI0, SGI186481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186482, SGI186483, SGI186483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41358[] = {SGI186484, SGI0, SGI186485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186486, SGI186487, SGI186487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41359[] = {SGI186488, SGI0, SGI186489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186490, SGI186491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186492, SGI0, SGI0, NULL};
const Segment * const SG41360[] = {SGI186493, SGI0, SGI186494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186495, SGI186496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186497, SGI0, SGI0, NULL};
const Segment * const SG41361[] = {SGI186498, SGI0, SGI186499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186500, SGI186501, SGI186501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41362[] = {SGI186502, SGI186503, SGI186504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186505, SGI0, SGI0, SGI0, SGI186506, SGI186506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41363[] = {SGI186507, SGI0, SGI186508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41364[] = {SGI186509, SGI186510, SGI186511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186512, SGI0, SGI0, SGI0, SGI186513, SGI186513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41365[] = {SGI186514, SGI186515, SGI186516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186517, SGI0, SGI0, SGI0, SGI186518, SGI186518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41366[] = {SGI186519, SGI0, SGI186520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186523, SGI186524, SGI186524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41367[] = {SGI186521, SGI0, SGI186522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186525, SGI186526, SGI186526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41368[] = {SGI186527, SGI0, SGI186528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186529, SGI186530, SGI186530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41369[] = {SGI186531, SGI0, SGI186532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186533, SGI186534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186535, SGI0, SGI0, NULL};
const Segment * const SG41370[] = {SGI186536, SGI186537, SGI186538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186539, SGI0, SGI0, SGI0, SGI186540, SGI186540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41371[] = {SGI186541, SGI0, SGI186542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41372[] = {SGI186543, SGI186544, SGI186545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186546, SGI0, SGI0, SGI0, SGI186547, SGI186547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41373[] = {SGI186549, SGI0, SGI186550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186551, SGI186552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41374[] = {SGI0, SGI186553, SGI186554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186555, SGI0, SGI0, SGI0, SGI186556, SGI186556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41375[] = {SGI186557, SGI186558, SGI186559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186560, SGI0, SGI0, SGI0, SGI186561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41376[] = {SGI186562, SGI186563, SGI186564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186565, SGI0, SGI0, SGI0, SGI186566, SGI186566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41377[] = {SGI186567, SGI0, SGI186568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186569, SGI186570, SGI186570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41378[] = {SGI186571, SGI186572, SGI186573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186574, SGI0, SGI0, SGI0, SGI186575, SGI186575, SGI0, SGI186576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41379[] = {SGI186577, SGI186578, SGI186579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186580, SGI0, SGI0, SGI0, SGI186581, SGI186581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186582, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41380[] = {SGI186583, SGI186584, SGI186585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186586, SGI0, SGI0, SGI0, SGI186587, SGI186587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186588, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41381[] = {SGI186589, SGI186590, SGI186591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186592, SGI0, SGI0, SGI0, SGI186593, SGI186593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186594, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41382[] = {SGI186595, SGI186596, SGI186597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186598, SGI0, SGI0, SGI0, SGI186599, SGI186599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186600, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41383[] = {SGI186601, SGI186602, SGI186603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186604, SGI0, SGI0, SGI0, SGI186605, SGI186605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41384[] = {SGI186606, SGI0, SGI186607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186608, SGI186609, SGI186609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41385[] = {SGI186610, SGI186611, SGI186612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186613, SGI0, SGI0, SGI0, SGI186614, SGI186614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186615, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41386[] = {SGI186616, SGI186617, SGI186618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186619, SGI0, SGI0, SGI0, SGI186620, SGI186620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186621, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41387[] = {SGI186622, SGI186623, SGI186624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186625, SGI0, SGI0, SGI0, SGI186626, SGI186626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186627, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41388[] = {SGI186628, SGI186629, SGI186630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186631, SGI0, SGI0, SGI0, SGI186632, SGI186632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186633, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41389[] = {SGI186634, SGI186635, SGI186636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186637, SGI0, SGI0, SGI0, SGI186638, SGI186638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41390[] = {SGI186639, SGI186640, SGI186641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186642, SGI186643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41391[] = {SGI186644, SGI186645, SGI186646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186647, SGI186647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41392[] = {SGI186648, SGI186649, SGI186650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186651, SGI0, SGI0, SGI0, SGI186652, SGI186652, SGI0, SGI186653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41393[] = {SGI186654, SGI186655, SGI186656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186657, SGI0, SGI0, SGI0, SGI186658, SGI186658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41394[] = {SGI186659, SGI0, SGI186661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41395[] = {SGI186660, SGI186662, SGI186663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186664, SGI0, SGI0, SGI0, SGI186665, SGI186665, SGI0, SGI186666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41396[] = {SGI186667, SGI186668, SGI186669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186670, SGI0, SGI0, SGI0, SGI186671, SGI186671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41397[] = {SGI186672, SGI0, SGI186673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186674, SGI186675, SGI186675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41398[] = {SGI186676, SGI0, SGI186677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186678, SGI186679, SGI186679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41399[] = {SGI186680, SGI0, SGI186681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186682, SGI186683, SGI186683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41400[] = {SGI186684, SGI0, SGI186685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186686, SGI186687, SGI186687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41401[] = {SGI186688, SGI186689, SGI186690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186691, SGI0, SGI0, SGI0, SGI186692, SGI186692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186694, SGI0, NULL};
const Segment * const SG41402[] = {SGI186695, SGI186696, SGI186697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186698, SGI0, SGI0, SGI0, SGI186699, SGI186699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186701, SGI0, NULL};
const Segment * const SG41403[] = {SGI186702, SGI186703, SGI186704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186705, SGI0, SGI0, SGI0, SGI186706, SGI186706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41404[] = {SGI186707, SGI186708, SGI186709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186710, SGI0, SGI0, SGI0, SGI186711, SGI186711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41405[] = {SGI186712, SGI186713, SGI186714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186715, SGI0, SGI0, SGI0, SGI186716, SGI186716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41406[] = {SGI186717, SGI186718, SGI186719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186720, SGI0, SGI0, SGI0, SGI186721, SGI186721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41407[] = {SGI186722, SGI0, SGI186723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186724, SGI186725, SGI186725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41408[] = {SGI186726, SGI0, SGI186727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186728, SGI186729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41409[] = {SGI186730, SGI186731, SGI186732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186733, SGI0, SGI0, SGI0, SGI186734, SGI186734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41410[] = {SGI186736, SGI0, SGI186737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186741, SGI186742, SGI186742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41411[] = {SGI186738, SGI186739, SGI186740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186743, SGI0, SGI0, SGI0, SGI186744, SGI186744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41412[] = {SGI186746, SGI186747, SGI186748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186750, SGI0, SGI0, SGI0, SGI186753, SGI186753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41413[] = {SGI186749, SGI186751, SGI186752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186754, SGI0, SGI0, SGI0, SGI186755, SGI186755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41414[] = {SGI186756, SGI186757, SGI186758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186759, SGI0, SGI0, SGI0, SGI186760, SGI186760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41415[] = {SGI186761, SGI186762, SGI186763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186764, SGI0, SGI0, SGI0, SGI186765, SGI186765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41416[] = {SGI186766, SGI186767, SGI186768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186769, SGI0, SGI0, SGI0, SGI186770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41417[] = {SGI186771, SGI186772, SGI186773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186774, SGI0, SGI0, SGI0, SGI186775, SGI186775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41418[] = {SGI186776, SGI0, SGI186777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186778, SGI186779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41419[] = {SGI186780, SGI0, SGI186781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41420[] = {SGI186782, SGI186783, SGI186784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186788, SGI186788, SGI0, SGI0, SGI0, SGI186790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41421[] = {SGI186785, SGI186786, SGI186787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186789, SGI0, SGI0, SGI0, SGI186791, SGI186791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41422[] = {SGI186792, SGI186793, SGI186794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186795, SGI0, SGI0, SGI0, SGI186796, SGI186796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41423[] = {SGI186797, SGI0, SGI186798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186799, SGI186800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41424[] = {SGI186801, SGI0, SGI186802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186803, SGI186804, SGI186804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41425[] = {SGI186805, SGI186806, SGI186807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186808, SGI0, SGI0, SGI0, SGI186812, SGI186812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41426[] = {SGI186809, SGI186810, SGI186811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186813, SGI0, SGI0, SGI0, SGI186814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41427[] = {SGI186815, SGI0, SGI186816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41428[] = {SGI186817, SGI186818, SGI186819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186820, SGI0, SGI0, SGI0, SGI186821, SGI186821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41429[] = {SGI186822, SGI186823, SGI186824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186825, SGI0, SGI0, SGI0, SGI186826, SGI186826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41430[] = {SGI186827, SGI0, SGI186828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41431[] = {SGI186829, SGI0, SGI186830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41432[] = {SGI186831, SGI186832, SGI186833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186834, SGI0, SGI0, SGI0, SGI186835, SGI0, SGI0, SGI186836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41433[] = {SGI186837, SGI0, SGI186838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186839, SGI0, SGI0, SGI0, SGI186840, SGI0, SGI0, SGI186841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41434[] = {SGI186842, SGI186843, SGI186844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186845, SGI0, SGI0, SGI0, SGI186846, SGI0, SGI0, SGI186847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41435[] = {SGI186848, SGI0, SGI186849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186850, SGI186851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41436[] = {SGI186852, SGI186853, SGI186854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186855, SGI0, SGI0, SGI0, SGI186856, SGI186856, SGI0, SGI0, SGI0, SGI186857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41437[] = {SGI186858, SGI186859, SGI186860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186861, SGI186861, SGI0, SGI0, SGI0, SGI186862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41438[] = {SGI186863, SGI0, SGI186864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186865, SGI186866, SGI186866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41439[] = {SGI186867, SGI186868, SGI186869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186870, SGI0, SGI0, SGI0, SGI186871, SGI186871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41440[] = {SGI186872, SGI0, SGI186873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186874, SGI186875, SGI186875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41441[] = {SGI186876, SGI0, SGI186877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186878, SGI186879, SGI186879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41442[] = {SGI186880, SGI0, SGI186881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186882, SGI186883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41443[] = {SGI186884, SGI186885, SGI186886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186887, SGI0, SGI0, SGI0, SGI186888, SGI186888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41444[] = {SGI186890, SGI0, SGI186891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186892, SGI186893, SGI186893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41445[] = {SGI186894, SGI0, SGI186895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186896, SGI186897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41446[] = {SGI186898, SGI0, SGI186899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186900, SGI186901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41447[] = {SGI186902, SGI0, SGI186903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186904, SGI186905, SGI186905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41448[] = {SGI186906, SGI186907, SGI186908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186909, SGI0, SGI0, SGI0, SGI186910, SGI186910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41449[] = {SGI186911, SGI186912, SGI186913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186914, SGI0, SGI0, SGI0, SGI186915, SGI186915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41450[] = {SGI186916, SGI0, SGI186917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186918, SGI186919, SGI186919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41451[] = {SGI186920, SGI0, SGI186921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186922, SGI186923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41452[] = {SGI186924, SGI0, SGI186925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186926, SGI186927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41453[] = {SGI186928, SGI186929, SGI186930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186931, SGI0, SGI0, SGI0, SGI186932, SGI186932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41454[] = {SGI186934, SGI186935, SGI186936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186937, SGI0, SGI0, SGI0, SGI186938, SGI186938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41455[] = {SGI186940, SGI186941, SGI186942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186943, SGI0, SGI0, SGI0, SGI186944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41456[] = {SGI186946, SGI186947, SGI186948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186949, SGI0, SGI0, SGI0, SGI186950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41457[] = {SGI186952, SGI186953, SGI186954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186955, SGI0, SGI0, SGI0, SGI186956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41458[] = {SGI186957, SGI0, SGI186958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186959, SGI186960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41459[] = {SGI186961, SGI0, SGI186962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186963, SGI186964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41460[] = {SGI186965, SGI0, SGI186966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186967, SGI186968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41461[] = {SGI186969, SGI186970, SGI186971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186972, SGI0, SGI0, SGI0, SGI186973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41462[] = {SGI186974, SGI0, SGI186975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186978, SGI186979, SGI186979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41463[] = {SGI186976, SGI0, SGI186977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186980, SGI186981, SGI186981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41464[] = {SGI186982, SGI0, SGI186984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186986, SGI186987, SGI186987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41465[] = {SGI186983, SGI0, SGI186985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186988, SGI186989, SGI186989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41466[] = {SGI186990, SGI0, SGI186991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186992, SGI186993, SGI186993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41467[] = {SGI186994, SGI0, SGI186995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI186998, SGI186999, SGI186999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41468[] = {SGI186996, SGI0, SGI186997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187000, SGI187001, SGI187001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41469[] = {SGI187002, SGI0, SGI187003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187004, SGI187005, SGI187005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41470[] = {SGI187006, SGI0, SGI187007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187008, SGI187009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41471[] = {SGI187010, SGI0, SGI187011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187012, SGI187013, SGI187013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41472[] = {SGI187014, SGI0, SGI187015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187016, SGI187017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41473[] = {SGI187018, SGI0, SGI187019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187020, SGI187021, SGI187021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41474[] = {SGI187022, SGI0, SGI187023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187024, SGI187025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41475[] = {SGI187026, SGI0, SGI187027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187028, SGI187029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41476[] = {SGI187030, SGI187031, SGI187032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187033, SGI0, SGI0, SGI0, SGI187034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41477[] = {SGI187035, SGI187036, SGI187037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187038, SGI187038, SGI0, SGI0, SGI0, SGI187039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41478[] = {SGI187040, SGI187041, SGI187042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187043, SGI187043, SGI0, SGI0, SGI0, SGI187044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41479[] = {SGI187045, SGI187046, SGI187047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187048, SGI0, SGI0, SGI0, SGI187049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41480[] = {SGI187050, SGI187051, SGI187052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187053, SGI0, SGI0, SGI0, SGI187054, SGI0, SGI187055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41481[] = {SGI187056, SGI187057, SGI187058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187059, SGI0, SGI0, SGI0, SGI187060, SGI187060, SGI0, SGI187061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41482[] = {SGI187062, SGI0, SGI187063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187066, SGI187068, SGI187068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41483[] = {SGI187064, SGI187065, SGI187067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187069, SGI0, SGI0, SGI0, SGI187070, SGI187070, SGI0, SGI187071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41484[] = {SGI187072, SGI0, SGI187073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187074, SGI187075, SGI187075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41485[] = {SGI187076, SGI0, SGI187077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187078, SGI187079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41486[] = {SGI187080, SGI187081, SGI187082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187083, SGI0, SGI0, SGI0, SGI187084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41487[] = {SGI187085, SGI187086, SGI187087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187088, SGI0, SGI0, SGI0, SGI187089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41488[] = {SGI187090, SGI187091, SGI187092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187093, SGI0, SGI0, SGI0, SGI187094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41489[] = {SGI187095, SGI187096, SGI187097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187098, SGI0, SGI0, SGI0, SGI187099, SGI187099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41490[] = {SGI187100, SGI187101, SGI187102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187103, SGI0, SGI0, SGI0, SGI187104, SGI187104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41491[] = {SGI187105, SGI187106, SGI187107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187108, SGI0, SGI0, SGI0, SGI187109, SGI187109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41492[] = {SGI187110, SGI187111, SGI187112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187113, SGI0, SGI0, SGI0, SGI187114, SGI187114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41493[] = {SGI187115, SGI187116, SGI187117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187118, SGI0, SGI0, SGI0, SGI187119, SGI187119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41494[] = {SGI187120, SGI187121, SGI187122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187123, SGI0, SGI0, SGI0, SGI187124, SGI187124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41495[] = {SGI187125, SGI0, SGI187126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187127, SGI187128, SGI187128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41496[] = {SGI187129, SGI187130, SGI187131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187132, SGI0, SGI0, SGI0, SGI187133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41497[] = {SGI187134, SGI0, SGI187135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187136, SGI187137, SGI187137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41498[] = {SGI187138, SGI0, SGI187139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187140, SGI187141, SGI187141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41499[] = {SGI187142, SGI187143, SGI187144, SGI187145, SGI187146, SGI187146, SGI0, NULL};
const Segment * const SG41500[] = {SGI187147, SGI0, SGI187148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187149, SGI187150, SGI187150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41501[] = {SGI187151, SGI0, SGI187152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187153, SGI187154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41502[] = {SGI187155, SGI0, SGI187156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187157, SGI187158, SGI187158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41503[] = {SGI187159, SGI0, SGI187160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187161, SGI0, SGI0, SGI0, SGI187162, SGI187162, SGI0, SGI0, SGI187163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41504[] = {SGI187164, SGI187165, SGI187166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187167, SGI0, SGI0, SGI0, SGI187168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41505[] = {SGI187169, SGI187170, SGI187171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187172, SGI0, SGI0, SGI0, SGI187173, SGI187173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41506[] = {SGI187174, SGI187175, SGI187177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187180, SGI0, SGI0, SGI0, SGI187183, SGI187183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41507[] = {SGI187176, SGI187178, SGI187179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187182, SGI0, SGI0, SGI0, SGI187185, SGI187185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41508[] = {SGI187181, SGI0, SGI187184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187186, SGI187187, SGI187187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41509[] = {SGI187188, SGI0, SGI187189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41510[] = {SGI187190, SGI187191, SGI187192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187193, SGI0, SGI0, SGI0, SGI187194, SGI187194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41511[] = {SGI187195, SGI187196, SGI187197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187198, SGI0, SGI0, SGI0, SGI187199, SGI187199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41512[] = {SGI187200, SGI187201, SGI187202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187203, SGI0, SGI0, SGI0, SGI187204, SGI187204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41513[] = {SGI187205, SGI0, SGI187206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41514[] = {SGI187207, SGI187208, SGI187209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187210, SGI0, SGI0, SGI0, SGI187211, SGI187211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41515[] = {SGI187212, SGI187213, SGI187214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187215, SGI0, SGI0, SGI0, SGI187216, SGI187216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41516[] = {SGI187217, SGI187218, SGI187219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187220, SGI0, SGI0, SGI0, SGI187221, SGI187221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41517[] = {SGI187222, SGI187223, SGI187224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187225, SGI187225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41518[] = {SGI187226, SGI0, SGI187227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41519[] = {SGI187228, SGI187229, SGI187230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187231, SGI0, SGI0, SGI0, SGI0, SGI187232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41520[] = {SGI187233, SGI187234, SGI187235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187236, SGI0, SGI0, SGI0, SGI0, SGI187237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41521[] = {SGI187238, SGI187239, SGI187240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187241, SGI0, SGI0, SGI0, SGI0, SGI187242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41522[] = {SGI187243, SGI187244, SGI187245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187246, SGI0, SGI0, SGI0, SGI187247, SGI187247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187248, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41523[] = {SGI187249, SGI187250, SGI187251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187252, SGI0, SGI0, SGI0, SGI187253, SGI187253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187254, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41524[] = {SGI187255, SGI187256, SGI187257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187258, SGI0, SGI0, SGI0, SGI187259, SGI187259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41525[] = {SGI187260, SGI0, SGI187261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187262, SGI187263, SGI187263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41526[] = {SGI187265, SGI187267, SGI187268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187270, SGI0, SGI0, SGI0, SGI187272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41527[] = {SGI187264, SGI187266, SGI187269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187271, SGI0, SGI0, SGI187273, SGI187274, SGI187274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41528[] = {SGI187275, SGI187276, SGI187277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187278, SGI0, SGI0, SGI0, SGI187279, SGI187279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41529[] = {SGI187280, SGI187281, SGI187282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187283, SGI0, SGI0, SGI0, SGI187284, SGI187284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41530[] = {SGI187285, SGI0, SGI187286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187287, SGI187288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41531[] = {SGI187289, SGI0, SGI187290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187291, SGI187292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41532[] = {SGI187293, SGI0, SGI187294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187295, SGI187296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41533[] = {SGI187297, SGI0, SGI187298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187299, SGI187300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41534[] = {SGI187301, SGI187302, SGI187303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187304, SGI0, SGI0, SGI0, SGI187305, SGI187305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41535[] = {SGI187306, SGI0, SGI187307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187309, SGI187310, SGI187310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41536[] = {SGI187311, SGI0, SGI187312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187313, SGI187314, SGI187314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41537[] = {SGI187315, SGI0, SGI187316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187317, SGI187318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41538[] = {SGI187319, SGI0, SGI187320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187321, SGI187322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41539[] = {SGI187323, SGI187324, SGI187325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187326, SGI0, SGI0, SGI0, SGI187327, SGI187327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41540[] = {SGI187328, SGI0, SGI187329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187330, SGI187331, SGI187331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41541[] = {SGI187332, SGI0, SGI187333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187334, SGI187335, SGI187335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41542[] = {SGI187336, SGI0, SGI187337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187338, SGI187339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41543[] = {SGI187340, SGI0, SGI187342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187344, SGI187345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41544[] = {SGI187341, SGI0, SGI187343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187346, SGI187347, SGI187347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41545[] = {SGI187348, SGI0, SGI187349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187350, SGI187351, SGI187351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41546[] = {SGI187352, SGI0, SGI187353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187354, SGI0, SGI0, SGI0, SGI187355, SGI187355, SGI0, SGI0, SGI0, SGI187356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41547[] = {SGI187357, SGI187358, SGI187359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187360, SGI0, SGI0, SGI0, SGI187361, SGI187361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41548[] = {SGI187362, SGI187363, SGI187364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187365, SGI0, SGI0, SGI0, SGI187366, SGI187366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41549[] = {SGI187367, SGI0, SGI187368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187369, SGI187370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41550[] = {SGI187371, SGI0, SGI187372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187373, SGI187374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41551[] = {SGI187375, SGI0, SGI187376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187377, SGI187378, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41552[] = {SGI187379, SGI0, SGI187380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187381, SGI187382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41553[] = {SGI187383, SGI187384, SGI187385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187389, SGI0, SGI0, SGI0, SGI187390, SGI187390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41554[] = {SGI187386, SGI187387, SGI187388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187391, SGI0, SGI0, SGI0, SGI187392, SGI187392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41555[] = {SGI187393, SGI187394, SGI187395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187399, SGI0, SGI0, SGI0, SGI187400, SGI187400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41556[] = {SGI187396, SGI187397, SGI187398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187401, SGI0, SGI0, SGI0, SGI187402, SGI187402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41557[] = {SGI187403, SGI0, SGI187405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41558[] = {SGI187404, SGI0, SGI187406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187407, SGI187408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41559[] = {SGI187409, SGI187410, SGI187411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187412, SGI0, SGI0, SGI0, SGI187413, SGI187413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41560[] = {SGI187414, SGI0, SGI187415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41561[] = {SGI187416, SGI187417, SGI187418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187419, SGI0, SGI0, SGI0, SGI187420, SGI187420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41562[] = {SGI187421, SGI187422, SGI187423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187424, SGI0, SGI0, SGI0, SGI187425, SGI187425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41563[] = {SGI187426, SGI187427, SGI187428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187429, SGI187429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41564[] = {SGI187430, SGI0, SGI187431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187432, SGI187433, SGI187433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41565[] = {SGI187434, SGI0, SGI187435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41566[] = {SGI187436, SGI0, SGI187437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187438, SGI187439, SGI187439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41567[] = {SGI187440, SGI0, SGI187441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187442, SGI187443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41568[] = {SGI187444, SGI0, SGI187445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187446, SGI187447, SGI187447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41569[] = {SGI187448, SGI0, SGI187449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187450, SGI187451, SGI187451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41570[] = {SGI187452, SGI187453, SGI187454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187455, SGI0, SGI0, SGI0, SGI187456, SGI187456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41571[] = {SGI187457, SGI187458, SGI187459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187460, SGI0, SGI0, SGI0, SGI187461, SGI187461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41572[] = {SGI187462, SGI0, SGI187463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187464, SGI187465, SGI187465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41573[] = {SGI187466, SGI0, SGI187467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41574[] = {SGI187468, SGI0, SGI187469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187470, SGI187471, SGI187471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41575[] = {SGI187472, SGI0, SGI187473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187474, SGI187475, SGI187475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41576[] = {SGI187476, SGI0, SGI187477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187478, SGI187479, SGI187479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41577[] = {SGI187480, SGI187481, SGI187482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187483, SGI0, SGI0, SGI0, SGI187484, SGI187484, SGI0, SGI187485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41578[] = {SGI187486, SGI187487, SGI187488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187489, SGI0, SGI0, SGI0, SGI187490, SGI187490, SGI0, SGI187491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41579[] = {SGI187492, SGI0, SGI187493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187494, SGI187495, SGI187495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41580[] = {SGI187496, SGI187497, SGI187498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187499, SGI0, SGI0, SGI0, SGI187500, SGI187500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41581[] = {SGI187501, SGI0, SGI187502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187503, SGI187504, SGI187504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41582[] = {SGI187505, SGI187506, SGI187507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187508, SGI0, SGI0, SGI0, SGI187509, SGI187509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41583[] = {SGI187510, SGI0, SGI187511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41584[] = {SGI187512, SGI187513, SGI187514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187515, SGI0, SGI0, SGI0, SGI187516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41585[] = {SGI187517, SGI0, SGI187518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41586[] = {SGI187519, SGI187520, SGI187521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187522, SGI0, SGI0, SGI0, SGI187523, SGI187523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41587[] = {SGI187524, SGI187525, SGI187526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187527, SGI0, SGI0, SGI0, SGI187529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41588[] = {SGI187528, SGI187530, SGI187531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187532, SGI0, SGI0, SGI0, SGI187533, SGI187533, SGI0, SGI187536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41589[] = {SGI187534, SGI187535, SGI187537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187538, SGI0, SGI0, SGI0, SGI187539, SGI187539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41590[] = {SGI187542, SGI187544, SGI187545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187547, SGI0, SGI0, SGI0, SGI187549, SGI187549, SGI0, SGI187550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41591[] = {SGI187540, SGI187541, SGI187543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187546, SGI0, SGI0, SGI0, SGI187548, SGI187548, SGI187551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41592[] = {SGI187552, SGI187553, SGI187554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187555, SGI0, SGI0, SGI0, SGI187556, SGI187556, SGI0, SGI187557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41593[] = {SGI187558, SGI187559, SGI187560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187561, SGI0, SGI0, SGI0, SGI187563, SGI0, SGI0, SGI187565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41594[] = {SGI187562, SGI187564, SGI187566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187567, SGI0, SGI0, SGI0, SGI187568, SGI187568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41595[] = {SGI187570, SGI187571, SGI187572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187573, SGI0, SGI0, SGI0, SGI187575, SGI187575, SGI0, SGI187577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41596[] = {SGI187574, SGI187576, SGI187578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187579, SGI0, SGI0, SGI0, SGI187580, SGI187580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41597[] = {SGI187582, SGI0, SGI187583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187584, SGI187585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41598[] = {SGI187586, SGI187587, SGI187588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187589, SGI0, SGI0, SGI0, SGI187590, SGI187590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41599[] = {SGI187591, SGI0, SGI187592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187596, SGI187597, SGI187597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41600[] = {SGI187593, SGI187594, SGI187595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187598, SGI0, SGI0, SGI0, SGI187599, SGI187599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41601[] = {SGI187600, SGI0, SGI187601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41602[] = {SGI187603, SGI0, SGI187604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187606, SGI187608, SGI187608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41603[] = {SGI187605, SGI187607, SGI187609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187610, SGI0, SGI0, SGI0, SGI187611, SGI187611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41604[] = {SGI187612, SGI0, SGI187613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187614, SGI187615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41605[] = {SGI187616, SGI0, SGI187617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187618, SGI187619, SGI187619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41606[] = {SGI187620, SGI187621, SGI187622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187623, SGI0, SGI0, SGI0, SGI187624, SGI187624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41607[] = {SGI187625, SGI0, SGI187626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41608[] = {SGI187627, SGI0, SGI187628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187629, SGI187630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41609[] = {SGI187631, SGI187632, SGI187633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187634, SGI0, SGI0, SGI0, SGI187635, SGI187635, SGI187636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41610[] = {SGI187637, SGI187638, SGI187639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187640, SGI0, SGI0, SGI0, SGI187641, SGI187641, SGI187642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41611[] = {SGI187643, SGI187644, SGI187645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187646, SGI0, SGI0, SGI0, SGI187647, SGI187647, SGI187648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41612[] = {SGI187649, SGI187650, SGI187651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187652, SGI0, SGI0, SGI0, SGI187653, SGI187653, SGI187654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41613[] = {SGI187655, SGI0, SGI187656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187657, SGI187658, SGI187658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41614[] = {SGI187659, SGI187660, SGI187661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187662, SGI0, SGI0, SGI0, SGI187663, SGI187663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41615[] = {SGI187664, SGI187665, SGI187666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187667, SGI0, SGI0, SGI0, SGI187668, SGI187668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41616[] = {SGI187669, SGI0, SGI187670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187671, SGI187672, SGI187672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41617[] = {SGI187673, SGI0, SGI187674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41618[] = {SGI187675, SGI187676, SGI187677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187678, SGI0, SGI0, SGI0, SGI187679, SGI187679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41619[] = {SGI187680, SGI187681, SGI187682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187683, SGI0, SGI0, SGI0, SGI187684, SGI187684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41620[] = {SGI187685, SGI0, SGI187686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187687, SGI187688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41621[] = {SGI187689, SGI0, SGI187690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187691, SGI187692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41622[] = {SGI187693, SGI0, SGI187695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187697, SGI187699, SGI187699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41623[] = {SGI187694, SGI0, SGI187696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187698, SGI187701, SGI187701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41624[] = {SGI187700, SGI0, SGI187702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187703, SGI187704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41625[] = {SGI187705, SGI0, SGI187706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187710, SGI187712, SGI187712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41626[] = {SGI187707, SGI187708, SGI187709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187713, SGI0, SGI0, SGI0, SGI187715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41627[] = {SGI187711, SGI0, SGI187714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187716, SGI187717, SGI187717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41628[] = {SGI187718, SGI0, SGI187719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187723, SGI187724, SGI187724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41629[] = {SGI187720, SGI187721, SGI187722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187725, SGI0, SGI0, SGI0, SGI187726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41630[] = {SGI187727, SGI0, SGI187728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187731, SGI187733, SGI187733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41631[] = {SGI187729, SGI187730, SGI187732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187734, SGI0, SGI0, SGI0, SGI187735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41632[] = {SGI187736, SGI0, SGI187737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187738, SGI187740, SGI187740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41633[] = {SGI187739, SGI0, SGI187741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187742, SGI187744, SGI187744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41634[] = {SGI187743, SGI187745, SGI187746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187747, SGI0, SGI0, SGI0, SGI187748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41635[] = {SGI187749, SGI0, SGI187750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187751, SGI187752, SGI187752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41636[] = {SGI187753, SGI187754, SGI187755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187756, SGI0, SGI0, SGI0, SGI187757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41637[] = {SGI187760, SGI0, SGI187761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41638[] = {SGI187758, SGI0, SGI187759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187762, SGI187763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41639[] = {SGI187764, SGI187766, SGI187767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187769, SGI0, SGI0, SGI0, SGI187771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41640[] = {SGI187765, SGI0, SGI187768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187770, SGI187772, SGI187772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41641[] = {SGI187773, SGI0, SGI187774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187775, SGI187776, SGI187776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41642[] = {SGI187777, SGI0, SGI187778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187781, SGI187782, SGI187782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41643[] = {SGI187779, SGI0, SGI187780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187783, SGI187784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41644[] = {SGI187785, SGI0, SGI187786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187787, SGI187788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41645[] = {SGI187789, SGI187790, SGI187791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187792, SGI0, SGI0, SGI0, SGI187793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41646[] = {SGI187794, SGI0, SGI187795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187796, SGI187797, SGI187797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41647[] = {SGI187798, SGI0, SGI187799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187800, SGI187801, SGI187801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41648[] = {SGI187802, SGI0, SGI187803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187804, SGI187805, SGI187805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41649[] = {SGI187806, SGI0, SGI187807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187808, SGI187809, SGI187809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41650[] = {SGI187810, SGI187811, SGI187812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187813, SGI0, SGI0, SGI0, SGI187814, SGI187814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41651[] = {SGI187815, SGI187816, SGI187817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187818, SGI0, SGI0, SGI0, SGI187819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41652[] = {SGI187820, SGI187821, SGI187822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187823, SGI0, SGI0, SGI0, SGI187824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41653[] = {SGI187825, SGI187826, SGI187827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187828, SGI0, SGI0, SGI0, SGI187829, SGI0, SGI0, SGI187830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41654[] = {SGI187831, SGI187832, SGI187833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187836, SGI0, SGI0, SGI0, SGI187837, SGI187837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41655[] = {SGI187834, SGI0, SGI187835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187838, SGI187839, SGI187839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41656[] = {SGI187840, SGI187841, SGI187842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187843, SGI0, SGI0, SGI0, SGI187844, SGI187844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41657[] = {SGI187845, SGI0, SGI187846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41658[] = {SGI187847, SGI0, SGI187848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187849, SGI187850, SGI187850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41659[] = {SGI187853, SGI0, SGI187854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187857, SGI187858, SGI187858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41660[] = {SGI187851, SGI0, SGI187852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187855, SGI187856, SGI187856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187859, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41661[] = {SGI187860, SGI0, SGI187861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187862, SGI187863, SGI187863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41662[] = {SGI187864, SGI0, SGI187865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187866, SGI187867, SGI187867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41663[] = {SGI187868, SGI187869, SGI187870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187871, SGI0, SGI0, SGI0, SGI187872, SGI187872, SGI0, SGI0, SGI187873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41664[] = {SGI187874, SGI187875, SGI187876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187877, SGI0, SGI0, SGI0, SGI187878, SGI187878, SGI0, SGI0, SGI187879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41665[] = {SGI187880, SGI187881, SGI187882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187883, SGI0, SGI0, SGI0, SGI187884, SGI187884, SGI0, SGI0, SGI187885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41666[] = {SGI187886, SGI187887, SGI187888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187889, SGI0, SGI0, SGI0, SGI187890, SGI187890, SGI0, SGI0, SGI187891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41667[] = {SGI187892, SGI187893, SGI187894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187895, SGI0, SGI0, SGI0, SGI187896, SGI187896, SGI0, SGI0, SGI187897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41668[] = {SGI187898, SGI0, SGI187899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187900, SGI187901, SGI187901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41669[] = {SGI187902, SGI0, SGI187903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187907, SGI187908, SGI187908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41670[] = {SGI187904, SGI187905, SGI187906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187909, SGI0, SGI0, SGI0, SGI187910, SGI187910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187911, SGI0, NULL};
const Segment * const SG41671[] = {SGI187912, SGI0, SGI187913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187914, SGI187915, SGI187915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41672[] = {SGI187916, SGI0, SGI187917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187918, SGI187919, SGI187919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41673[] = {SGI187920, SGI0, SGI187921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187922, SGI187923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41674[] = {SGI187924, SGI187925, SGI187926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187927, SGI0, SGI0, SGI0, SGI187928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187929, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41675[] = {SGI187930, SGI187931, SGI187932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187933, SGI0, SGI0, SGI0, SGI187934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187935, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41676[] = {SGI187936, SGI187937, SGI187938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187939, SGI0, SGI0, SGI0, SGI187940, SGI187940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41677[] = {SGI187941, SGI0, SGI187942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187943, SGI187944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41678[] = {SGI187945, SGI0, SGI187946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187947, SGI187948, SGI187948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41679[] = {SGI187949, SGI187950, SGI187951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187952, SGI0, SGI0, SGI0, SGI187953, SGI187953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41680[] = {SGI187954, SGI187955, SGI187956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187957, SGI0, SGI0, SGI0, SGI187958, SGI187958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41681[] = {SGI187959, SGI187960, SGI187961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187962, SGI0, SGI0, SGI0, SGI187963, SGI187963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41682[] = {SGI187964, SGI187965, SGI187966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187967, SGI0, SGI0, SGI0, SGI187968, SGI187968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41683[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187969, SGI0, SGI0, SGI187970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41684[] = {SGI187971, SGI0, SGI187972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187973, SGI187974, SGI187974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41685[] = {SGI187975, SGI0, SGI187976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187977, SGI187978, SGI187978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41686[] = {SGI187979, SGI0, SGI187980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187981, SGI187982, SGI187982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41687[] = {SGI187983, SGI187984, SGI187985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187986, SGI0, SGI0, SGI0, SGI187987, SGI187987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41688[] = {SGI187988, SGI187989, SGI187990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187991, SGI0, SGI0, SGI0, SGI187992, SGI187992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41689[] = {SGI187993, SGI0, SGI187994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI187995, SGI187996, SGI187996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41690[] = {SGI187997, SGI187998, SGI187999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188000, SGI188000, SGI0, SGI0, SGI0, SGI188001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41691[] = {SGI188002, SGI0, SGI188003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188004, SGI188005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41692[] = {SGI188006, SGI0, SGI188007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188008, SGI188009, SGI188009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41693[] = {SGI188010, SGI0, SGI188011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188012, SGI188013, SGI188013, SGI0, SGI0, SGI0, SGI0, SGI188014, SGI0, NULL};
const Segment * const SG41694[] = {SGI188015, SGI0, SGI188016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188018, SGI188020, SGI188020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41695[] = {SGI188017, SGI0, SGI188019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188021, SGI188022, SGI188022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41696[] = {SGI188023, SGI0, SGI188024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188025, SGI188026, SGI188026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41697[] = {SGI188027, SGI0, SGI188028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188029, SGI188030, SGI188030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41698[] = {SGI188031, SGI0, SGI188032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188033, SGI188034, SGI188034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41699[] = {SGI188035, SGI188036, SGI188037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188038, SGI0, SGI0, SGI0, SGI188039, SGI188039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41700[] = {SGI188040, SGI0, SGI188041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41701[] = {SGI188042, SGI188043, SGI188044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188045, SGI188045, SGI0, SGI0, SGI188046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41702[] = {SGI188047, SGI0, SGI188048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188049, SGI188050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41703[] = {SGI188051, SGI0, SGI188052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188053, SGI188054, SGI188054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41704[] = {SGI188055, SGI188056, SGI188057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188058, SGI0, SGI0, SGI0, SGI188060, SGI188060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41705[] = {SGI188059, SGI0, SGI188061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188062, SGI188063, SGI188063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41706[] = {SGI188064, SGI0, SGI188065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188066, SGI188067, SGI188067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41707[] = {SGI188068, SGI0, SGI188069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188072, SGI188073, SGI188073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41708[] = {SGI188070, SGI0, SGI188071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188074, SGI188075, SGI188075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41709[] = {SGI188076, SGI0, SGI188077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188078, SGI188079, SGI188079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41710[] = {SGI188080, SGI0, SGI188081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188082, SGI188083, SGI188083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41711[] = {SGI188084, SGI0, SGI188085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188086, SGI188087, SGI188087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41712[] = {SGI188088, SGI188089, SGI188090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188091, SGI0, SGI0, SGI0, SGI188092, SGI188092, SGI0, SGI188093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41713[] = {SGI188094, SGI0, SGI188095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188096, SGI188097, SGI188097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41714[] = {SGI188098, SGI0, SGI188099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188100, SGI188101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41715[] = {SGI188102, SGI0, SGI188103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188104, SGI188105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41716[] = {SGI188106, SGI0, SGI188107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188108, SGI188109, SGI188109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41717[] = {SGI188110, SGI0, SGI188111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188112, SGI188113, SGI188113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41718[] = {SGI188114, SGI188115, SGI188116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188117, SGI0, SGI0, SGI0, SGI188118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41719[] = {SGI188119, SGI0, SGI188120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41720[] = {SGI188121, SGI188122, SGI188123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188124, SGI0, SGI0, SGI0, SGI188125, SGI188125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41721[] = {SGI188126, SGI0, SGI188127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41722[] = {SGI188128, SGI0, SGI188129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188130, SGI188131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41723[] = {SGI188132, SGI0, SGI188133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188134, SGI188135, SGI188135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41724[] = {SGI188136, SGI0, SGI188137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188138, SGI188139, SGI188139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41725[] = {SGI188140, SGI188141, SGI188142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188146, SGI0, SGI0, SGI0, SGI188147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41726[] = {SGI188143, SGI188144, SGI188145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188148, SGI0, SGI0, SGI0, SGI188149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41727[] = {SGI188150, SGI0, SGI188152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188153, SGI188154, SGI188154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41728[] = {SGI188155, SGI188156, SGI188157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188161, SGI0, SGI0, SGI0, SGI188163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41729[] = {SGI188158, SGI188159, SGI188160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188166, SGI0, SGI0, SGI0, SGI188169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41730[] = {SGI188162, SGI188164, SGI188167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188171, SGI0, SGI0, SGI0, SGI188173, SGI188173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41731[] = {SGI188165, SGI0, SGI188168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188172, SGI188174, SGI188174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41732[] = {SGI188179, SGI0, SGI188181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41733[] = {SGI188175, SGI188176, SGI188177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188182, SGI0, SGI0, SGI0, SGI188184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41734[] = {SGI188178, SGI188180, SGI188183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188185, SGI0, SGI0, SGI0, SGI188186, SGI188186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41735[] = {SGI188187, SGI188188, SGI188189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188190, SGI0, SGI0, SGI0, SGI188191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41736[] = {SGI188192, SGI188193, SGI188194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188195, SGI0, SGI0, SGI0, SGI188196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41737[] = {SGI188197, SGI188198, SGI188199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188200, SGI0, SGI0, SGI0, SGI188201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41738[] = {SGI188202, SGI0, SGI188203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41739[] = {SGI188204, SGI188205, SGI188206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188207, SGI0, SGI0, SGI0, SGI188208, SGI188208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188209, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41740[] = {SGI188210, SGI188211, SGI188212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188213, SGI0, SGI0, SGI0, SGI188214, SGI188214, SGI0, SGI188216, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41741[] = {SGI188215, SGI188217, SGI188218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188219, SGI0, SGI0, SGI0, SGI188220, SGI188220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188221, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41742[] = {SGI188222, SGI0, SGI188223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188224, SGI188225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41743[] = {SGI188226, SGI0, SGI188227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188228, SGI188229, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41744[] = {SGI188230, SGI0, SGI188232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188234, SGI188235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41745[] = {SGI188231, SGI0, SGI188233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188236, SGI188237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41746[] = {SGI188238, SGI0, SGI188239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188240, SGI188241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41747[] = {SGI188242, SGI0, SGI188243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188244, SGI188245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41748[] = {SGI188246, SGI0, SGI188247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188248, SGI188249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41749[] = {SGI188250, SGI0, SGI188251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188252, SGI188253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41750[] = {SGI188254, SGI0, SGI188255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188256, SGI188257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41751[] = {SGI188258, SGI0, SGI188259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188260, SGI188262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41752[] = {SGI188261, SGI188263, SGI188264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188265, SGI0, SGI0, SGI0, SGI188266, SGI188266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41753[] = {SGI188267, SGI188268, SGI188269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188270, SGI0, SGI0, SGI0, SGI188271, SGI188271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41754[] = {SGI188272, SGI0, SGI188273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188274, SGI188275, SGI188275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41755[] = {SGI188276, SGI0, SGI188277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188278, SGI188279, SGI188279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41756[] = {SGI188280, SGI0, SGI188281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188285, SGI188286, SGI188286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41757[] = {SGI188282, SGI188283, SGI188284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188287, SGI0, SGI0, SGI0, SGI188288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41758[] = {SGI188289, SGI0, SGI188290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188291, SGI188292, SGI188292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41759[] = {SGI188293, SGI0, SGI188294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188295, SGI188296, SGI188296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41760[] = {SGI188297, SGI0, SGI188298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188299, SGI188300, SGI188300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41761[] = {SGI188301, SGI188302, SGI188303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188304, SGI0, SGI0, SGI0, SGI188305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41762[] = {SGI188306, SGI0, SGI188307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188308, SGI188309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41763[] = {SGI188310, SGI0, SGI188311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41764[] = {SGI188312, SGI188313, SGI188314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188317, SGI0, SGI0, SGI0, SGI188318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41765[] = {SGI188315, SGI0, SGI188316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188319, SGI188320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41766[] = {SGI188321, SGI188322, SGI188323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188324, SGI0, SGI0, SGI0, SGI188325, SGI188325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41767[] = {SGI188326, SGI0, SGI188327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188328, SGI188329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41768[] = {SGI188330, SGI0, SGI188331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188332, SGI188333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41769[] = {SGI188334, SGI188335, SGI188336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188337, SGI0, SGI0, SGI0, SGI188338, SGI188338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41770[] = {SGI188339, SGI0, SGI188340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41771[] = {SGI188341, SGI188342, SGI188343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188344, SGI0, SGI0, SGI0, SGI188345, SGI188345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41772[] = {SGI188346, SGI188347, SGI188348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188349, SGI0, SGI0, SGI0, SGI188350, SGI188350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41773[] = {SGI188351, SGI188352, SGI188353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188354, SGI0, SGI0, SGI0, SGI188355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41774[] = {SGI188356, SGI0, SGI188357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41775[] = {SGI188358, SGI188359, SGI188360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188361, SGI0, SGI0, SGI0, SGI188362, SGI188362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41776[] = {SGI188363, SGI188364, SGI188365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188366, SGI0, SGI0, SGI0, SGI188367, SGI188367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41777[] = {SGI188368, SGI188369, SGI188370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188371, SGI188371, SGI0, SGI0, SGI0, SGI188372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41778[] = {SGI188373, SGI188374, SGI188375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188376, SGI0, SGI0, SGI0, SGI188377, SGI188377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41779[] = {SGI188379, SGI188380, SGI188381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188382, SGI0, SGI0, SGI0, SGI188383, SGI188383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41780[] = {SGI188385, SGI188386, SGI188387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188388, SGI0, SGI0, SGI0, SGI188389, SGI188389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41781[] = {SGI188391, SGI188392, SGI188393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188394, SGI0, SGI0, SGI0, SGI188395, SGI188395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41782[] = {SGI188396, SGI188397, SGI188398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188402, SGI0, SGI0, SGI0, SGI188403, SGI188403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41783[] = {SGI188399, SGI188400, SGI188401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188404, SGI0, SGI0, SGI0, SGI188405, SGI188405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41784[] = {SGI188409, SGI0, SGI188411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41785[] = {SGI188407, SGI188408, SGI188410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188412, SGI0, SGI0, SGI0, SGI188413, SGI188413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41786[] = {SGI188414, SGI0, SGI188415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188419, SGI188420, SGI188420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41787[] = {SGI188416, SGI188417, SGI188418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188421, SGI0, SGI0, SGI0, SGI188422, SGI188422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41788[] = {SGI188423, SGI0, SGI188424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188425, SGI188426, SGI188426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41789[] = {SGI188427, SGI0, SGI188428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188429, SGI188430, SGI188430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41790[] = {SGI188431, SGI188432, SGI188433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188434, SGI0, SGI0, SGI0, SGI188436, SGI188436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41791[] = {SGI188435, SGI188437, SGI188438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188439, SGI0, SGI0, SGI0, SGI188440, SGI188440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41792[] = {SGI188441, SGI0, SGI188442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188443, SGI188444, SGI188444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41793[] = {SGI188445, SGI0, SGI188446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188447, SGI188448, SGI188448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41794[] = {SGI188449, SGI0, SGI188450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188451, SGI188452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41795[] = {SGI188453, SGI188454, SGI188455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188456, SGI188456, SGI0, SGI0, SGI0, SGI0, SGI188457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41796[] = {SGI188458, SGI188459, SGI188460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188461, SGI188461, SGI0, SGI0, SGI0, SGI0, SGI188462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41797[] = {SGI188463, SGI0, SGI188464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188465, SGI188466, SGI188466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41798[] = {SGI188467, SGI0, SGI188468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188469, SGI188470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41799[] = {SGI188471, SGI0, SGI188473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188475, SGI188476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41800[] = {SGI188472, SGI0, SGI188474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188477, SGI188478, SGI188478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188479, SGI0, SGI0, NULL};
const Segment * const SG41801[] = {SGI188480, SGI0, SGI188481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188482, SGI188483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41802[] = {SGI188484, SGI0, SGI188485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188486, SGI188487, SGI188487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41803[] = {SGI188488, SGI0, SGI188489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188490, SGI188491, SGI188491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41804[] = {SGI188492, SGI0, SGI188493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188494, SGI188495, SGI188495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41805[] = {SGI188496, SGI188497, SGI188498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188499, SGI188499, SGI0, SGI0, SGI0, SGI188500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41806[] = {SGI188501, SGI0, SGI188502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188503, SGI188504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41807[] = {SGI188505, SGI0, SGI188506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188507, SGI188508, SGI188508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41808[] = {SGI188509, SGI0, SGI188510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188511, SGI188512, SGI188512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41809[] = {SGI188513, SGI0, SGI188514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188515, SGI188516, SGI188516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41810[] = {SGI188517, SGI0, SGI188518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188519, SGI188520, SGI188520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41811[] = {SGI188521, SGI188522, SGI188523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188524, SGI0, SGI0, SGI0, SGI188525, SGI0, SGI0, SGI188526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41812[] = {SGI188527, SGI0, SGI188528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188529, SGI0, SGI0, SGI0, SGI188530, SGI0, SGI0, SGI188531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41813[] = {SGI188532, SGI188533, SGI188534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188535, SGI0, SGI0, SGI0, SGI188536, SGI0, SGI188537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41814[] = {SGI188538, SGI188539, SGI188540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188541, SGI0, SGI0, SGI0, SGI188542, SGI0, SGI188543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41815[] = {SGI188545, SGI188547, SGI188549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188550, SGI0, SGI0, SGI0, SGI188552, SGI188552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41816[] = {SGI188544, SGI188546, SGI188548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188551, SGI0, SGI0, SGI0, SGI188553, SGI0, SGI188554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41817[] = {SGI188555, SGI0, SGI188556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41818[] = {SGI188557, SGI0, SGI188558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188562, SGI188563, SGI188563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41819[] = {SGI188559, SGI188560, SGI188561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188564, SGI0, SGI0, SGI0, SGI188565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41820[] = {SGI188566, SGI188567, SGI188568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188569, SGI0, SGI0, SGI0, SGI188570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41821[] = {SGI188571, SGI0, SGI188572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188573, SGI188574, SGI188574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41822[] = {SGI188575, SGI0, SGI188576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188577, SGI188578, SGI188578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41823[] = {SGI188579, SGI0, SGI188580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188581, SGI188582, SGI188582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41824[] = {SGI188583, SGI0, SGI188584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188585, SGI188586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41825[] = {SGI188587, SGI0, SGI188588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188589, SGI188590, SGI188590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41826[] = {SGI188591, SGI0, SGI188592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188593, SGI188594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41827[] = {SGI188595, SGI0, SGI188596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188597, SGI188598, SGI188598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41828[] = {SGI188599, SGI188600, SGI188601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188602, SGI0, SGI0, SGI0, SGI188603, SGI188603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41829[] = {SGI188604, SGI0, SGI188605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188606, SGI188607, SGI188607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41830[] = {SGI188608, SGI0, SGI188609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41831[] = {SGI188610, SGI188611, SGI188612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188613, SGI0, SGI0, SGI0, SGI188614, SGI188614, SGI0, SGI188615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41832[] = {SGI188616, SGI188617, SGI188618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188619, SGI0, SGI0, SGI0, SGI188620, SGI188620, SGI0, SGI188621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41833[] = {SGI188622, SGI0, SGI188623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188624, SGI188625, SGI188625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41834[] = {SGI188626, SGI0, SGI188627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188628, SGI188629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188630, SGI0, SGI0, NULL};
const Segment * const SG41835[] = {SGI188631, SGI0, SGI188632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188633, SGI188634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188635, SGI0, SGI0, NULL};
const Segment * const SG41836[] = {SGI188636, SGI188637, SGI188638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188639, SGI0, SGI0, SGI0, SGI188640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41837[] = {SGI188641, SGI188642, SGI188643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188644, SGI0, SGI0, SGI0, SGI188645, SGI188645, SGI0, SGI0, SGI0, SGI188646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41838[] = {SGI188647, SGI0, SGI188648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188649, SGI188650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41839[] = {SGI188651, SGI188652, SGI188653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188654, SGI0, SGI0, SGI0, SGI188655, SGI188655, SGI0, SGI0, SGI0, SGI188656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41840[] = {SGI188657, SGI188658, SGI188659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188660, SGI0, SGI0, SGI0, SGI188661, SGI188661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41841[] = {SGI188662, SGI188663, SGI188664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188665, SGI0, SGI0, SGI0, SGI188666, SGI188666, SGI0, SGI0, SGI0, SGI188667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41842[] = {SGI188668, SGI188669, SGI188670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188671, SGI0, SGI0, SGI0, SGI188672, SGI188672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41843[] = {SGI188673, SGI0, SGI188674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188675, SGI0, SGI0, SGI0, SGI188676, SGI188676, SGI0, SGI0, SGI0, SGI188677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41844[] = {SGI188678, SGI0, SGI188679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188680, SGI188681, SGI188681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188682, SGI0, SGI0, NULL};
const Segment * const SG41845[] = {SGI188683, SGI0, SGI188684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188685, SGI188686, SGI188686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188687, SGI0, SGI0, NULL};
const Segment * const SG41846[] = {SGI188688, SGI0, SGI188689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188691, SGI188693, SGI188693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188694, SGI0, SGI0, NULL};
const Segment * const SG41847[] = {SGI188690, SGI0, SGI188692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188695, SGI188696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41848[] = {SGI188697, SGI0, SGI188698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188701, SGI188702, SGI188702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188705, SGI0, SGI0, NULL};
const Segment * const SG41849[] = {SGI188699, SGI0, SGI188700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188703, SGI188704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41850[] = {SGI188706, SGI0, SGI188707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188708, SGI188709, SGI188709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188710, SGI0, SGI0, NULL};
const Segment * const SG41851[] = {SGI188711, SGI0, SGI188712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188713, SGI188714, SGI188714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188715, SGI0, SGI0, NULL};
const Segment * const SG41852[] = {SGI188716, SGI0, SGI188717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188718, SGI188719, SGI188719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41853[] = {SGI188720, SGI0, SGI188721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188722, SGI188723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41854[] = {SGI188724, SGI0, SGI188725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188726, SGI188727, SGI188727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41855[] = {SGI188728, SGI0, SGI188729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188730, SGI188731, SGI188731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41856[] = {SGI188732, SGI0, SGI188733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188734, SGI188735, SGI188735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41857[] = {SGI188736, SGI188737, SGI188738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188739, SGI0, SGI0, SGI0, SGI188740, SGI188740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41858[] = {SGI188741, SGI188742, SGI188743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188744, SGI0, SGI0, SGI0, SGI188745, SGI188745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41859[] = {SGI188746, SGI0, SGI188747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41860[] = {SGI188748, SGI0, SGI188749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188750, SGI188751, SGI188751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41861[] = {SGI188752, SGI0, SGI188753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188754, SGI188755, SGI188755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41862[] = {SGI188756, SGI0, SGI188757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188758, SGI188759, SGI188759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41863[] = {SGI188760, SGI0, SGI188761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188762, SGI188763, SGI188763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41864[] = {SGI188764, SGI0, SGI188765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188766, SGI188767, SGI188767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41865[] = {SGI188768, SGI0, SGI188769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188770, SGI188771, SGI188771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41866[] = {SGI188772, SGI188773, SGI188774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188775, SGI0, SGI0, SGI0, SGI188776, SGI188776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41867[] = {SGI188777, SGI0, SGI188778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41868[] = {SGI188779, SGI188780, SGI188781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188782, SGI0, SGI0, SGI0, SGI188783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41869[] = {SGI188784, SGI0, SGI188785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188786, SGI188787, SGI188787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41870[] = {SGI188788, SGI188789, SGI188790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188791, SGI0, SGI0, SGI0, SGI188792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41871[] = {SGI188793, SGI0, SGI188794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41872[] = {SGI188795, SGI0, SGI188796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188797, SGI188798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41873[] = {SGI188799, SGI0, SGI188800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188801, SGI188802, SGI188802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41874[] = {SGI188803, SGI0, SGI188804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188805, SGI188806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41875[] = {SGI188807, SGI0, SGI188808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188809, SGI188810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41876[] = {SGI188811, SGI0, SGI188812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188813, SGI188814, SGI188814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41877[] = {SGI188815, SGI188816, SGI188817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188818, SGI0, SGI0, SGI0, SGI188819, SGI188819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41878[] = {SGI188820, SGI188821, SGI188822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188823, SGI0, SGI0, SGI0, SGI188824, SGI188824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41879[] = {SGI188825, SGI188826, SGI188827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188828, SGI0, SGI0, SGI0, SGI188829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41880[] = {SGI188830, SGI0, SGI188831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188832, SGI188833, SGI188833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41881[] = {SGI188834, SGI0, SGI188835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188836, SGI188837, SGI188837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41882[] = {SGI188838, SGI188839, SGI188840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188841, SGI0, SGI0, SGI0, SGI188842, SGI188842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41883[] = {SGI188843, SGI188844, SGI188845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188846, SGI0, SGI0, SGI0, SGI188847, SGI188847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41884[] = {SGI188848, SGI0, SGI188849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41885[] = {SGI188850, SGI188851, SGI188852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188854, SGI0, SGI0, SGI0, SGI188856, SGI188856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41886[] = {SGI188853, SGI0, SGI188855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188857, SGI188858, SGI188858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188859, SGI0, SGI0, NULL};
const Segment * const SG41887[] = {SGI188860, SGI0, SGI188861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41888[] = {SGI188862, SGI0, SGI188863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188864, SGI188865, SGI188865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41889[] = {SGI188866, SGI0, SGI188867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188868, SGI188869, SGI188869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41890[] = {SGI188870, SGI0, SGI188871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188872, SGI188873, SGI188873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41891[] = {SGI188874, SGI0, SGI188875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188879, SGI188880, SGI188880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41892[] = {SGI188876, SGI188877, SGI188878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188881, SGI0, SGI0, SGI0, SGI188882, SGI188882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41893[] = {SGI188883, SGI188884, SGI188885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188886, SGI188886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41894[] = {SGI188887, SGI0, SGI188888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41895[] = {SGI188889, SGI188890, SGI188891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188892, SGI0, SGI0, SGI0, SGI188893, SGI188893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41896[] = {SGI188894, SGI188895, SGI188897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188900, SGI0, SGI0, SGI0, SGI188901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41897[] = {SGI188896, SGI188898, SGI188899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188902, SGI188902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41898[] = {SGI188903, SGI0, SGI188904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41899[] = {SGI188905, SGI188906, SGI188907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188908, SGI0, SGI0, SGI0, SGI188909, SGI188909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41900[] = {SGI188910, SGI188911, SGI188912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188913, SGI0, SGI0, SGI0, SGI188914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41901[] = {SGI188915, SGI0, SGI188916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41902[] = {SGI188917, SGI0, SGI188918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188919, SGI188920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41903[] = {SGI188921, SGI188922, SGI188923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188924, SGI188924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41904[] = {SGI0, SGI188925, SGI188926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41905[] = {SGI188927, SGI0, SGI188928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41906[] = {SGI188929, SGI188930, SGI188931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188932, SGI0, SGI0, SGI0, SGI188933, SGI188933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41907[] = {SGI188934, SGI188935, SGI188936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188937, SGI0, SGI0, SGI0, SGI188938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41908[] = {SGI188939, SGI188940, SGI188941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188942, SGI0, SGI0, SGI0, SGI188943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41909[] = {SGI188944, SGI0, SGI188945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188946, SGI188947, SGI188947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41910[] = {SGI188948, SGI0, SGI188949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188951, SGI188953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41911[] = {SGI188950, SGI0, SGI188952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188954, SGI188955, SGI188955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41912[] = {SGI188956, SGI0, SGI188957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188958, SGI188959, SGI188959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41913[] = {SGI188960, SGI0, SGI188961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188962, SGI188963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41914[] = {SGI188964, SGI0, SGI188965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188966, SGI188967, SGI188967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41915[] = {SGI188968, SGI0, SGI188969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188970, SGI188971, SGI188971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41916[] = {SGI188972, SGI0, SGI188973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188974, SGI188975, SGI188975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41917[] = {SGI188976, SGI0, SGI188977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188978, SGI188979, SGI188979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41918[] = {SGI188980, SGI0, SGI188981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188982, SGI188983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41919[] = {SGI188984, SGI188985, SGI188986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188989, SGI188989, SGI0, SGI0, SGI188992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41920[] = {SGI188987, SGI0, SGI188988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188990, SGI188991, SGI188991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41921[] = {SGI188994, SGI0, SGI188996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188997, SGI188999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41922[] = {SGI188993, SGI0, SGI188995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188998, SGI189000, SGI189000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41923[] = {SGI189001, SGI0, SGI189002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189003, SGI189004, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41924[] = {SGI189005, SGI0, SGI189006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189007, SGI189008, SGI189008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41925[] = {SGI189009, SGI189010, SGI189011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189012, SGI0, SGI0, SGI189013, SGI189014, SGI189014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41926[] = {SGI189015, SGI189016, SGI189017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189018, SGI0, SGI0, SGI0, SGI189020, SGI189020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41927[] = {SGI189019, SGI189021, SGI189022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189023, SGI0, SGI0, SGI189024, SGI189025, SGI189025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41928[] = {SGI189027, SGI189029, SGI189031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189033, SGI189033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41929[] = {SGI189026, SGI189028, SGI189030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189032, SGI0, SGI0, SGI189034, SGI189035, SGI189035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41930[] = {SGI189036, SGI189037, SGI189038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189042, SGI0, SGI0, SGI0, SGI189043, SGI189043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41931[] = {SGI189039, SGI189040, SGI189041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189044, SGI0, SGI0, SGI189045, SGI189046, SGI189046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41932[] = {SGI189047, SGI189048, SGI189049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189050, SGI0, SGI0, SGI189051, SGI189052, SGI189052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41933[] = {SGI189053, SGI189054, SGI189055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189056, SGI0, SGI0, SGI189057, SGI189058, SGI189058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41934[] = {SGI189059, SGI0, SGI189060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189061, SGI189062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41935[] = {SGI189063, SGI0, SGI189064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189065, SGI189066, SGI189066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41936[] = {SGI189067, SGI189068, SGI189069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189070, SGI0, SGI0, SGI0, SGI189071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41937[] = {SGI189072, SGI0, SGI189073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189074, SGI189075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41938[] = {SGI189076, SGI189077, SGI189078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189079, SGI0, SGI0, SGI0, SGI189080, SGI189080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41939[] = {SGI189082, SGI189083, SGI189084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189085, SGI189085, SGI0, SGI0, SGI0, SGI0, SGI189086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41940[] = {SGI189087, SGI189088, SGI189089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189090, SGI189090, SGI0, SGI0, SGI0, SGI0, SGI189093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41941[] = {SGI189091, SGI189092, SGI189094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189095, SGI0, SGI0, SGI0, SGI189096, SGI189096, SGI0, SGI189097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41942[] = {SGI189098, SGI189099, SGI189100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189104, SGI189104, SGI0, SGI0, SGI0, SGI0, SGI189106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41943[] = {SGI189101, SGI189102, SGI189103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189105, SGI0, SGI0, SGI0, SGI189107, SGI189107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41944[] = {SGI189108, SGI189109, SGI189110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189111, SGI0, SGI0, SGI0, SGI189112, SGI189112, SGI0, SGI189113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41945[] = {SGI189114, SGI189115, SGI189116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189117, SGI0, SGI0, SGI0, SGI189119, SGI189119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41946[] = {SGI189118, SGI189120, SGI189121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189123, SGI0, SGI0, SGI0, SGI189124, SGI189124, SGI0, SGI189125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41947[] = {SGI189126, SGI189127, SGI189128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189129, SGI0, SGI0, SGI0, SGI189130, SGI189130, SGI0, SGI189131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41948[] = {SGI189132, SGI189133, SGI189134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189135, SGI0, SGI0, SGI0, SGI189136, SGI189136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41949[] = {SGI189137, SGI189138, SGI189139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189140, SGI0, SGI0, SGI0, SGI189141, SGI189141, SGI0, SGI189144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41950[] = {SGI189142, SGI0, SGI189143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41951[] = {SGI189145, SGI0, SGI189146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189147, SGI189148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41952[] = {SGI189149, SGI0, SGI189150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189153, SGI189154, SGI189154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41953[] = {SGI189151, SGI0, SGI189152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189155, SGI189156, SGI189156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41954[] = {SGI189157, SGI0, SGI189158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189159, SGI189160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41955[] = {SGI189161, SGI189162, SGI189163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189166, SGI0, SGI0, SGI0, SGI189167, SGI189167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41956[] = {SGI189164, SGI0, SGI189165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189168, SGI189169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41957[] = {SGI189170, SGI0, SGI189172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189174, SGI189175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41958[] = {SGI189171, SGI0, SGI189173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189176, SGI189177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41959[] = {SGI189178, SGI0, SGI189179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189180, SGI189181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41960[] = {SGI189182, SGI0, SGI189183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189184, SGI189185, SGI189185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41961[] = {SGI189186, SGI0, SGI189187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189188, SGI189189, SGI189189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41962[] = {SGI189190, SGI189191, SGI189192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189193, SGI0, SGI0, SGI0, SGI189194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41963[] = {SGI189195, SGI0, SGI189196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189197, SGI189198, SGI189198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41964[] = {SGI189199, SGI0, SGI189200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189201, SGI189202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41965[] = {SGI189203, SGI189204, SGI189205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189206, SGI0, SGI0, SGI0, SGI189207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41966[] = {SGI189208, SGI189209, SGI189210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189211, SGI0, SGI0, SGI0, SGI189212, SGI189212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41967[] = {SGI189213, SGI189214, SGI189215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189216, SGI0, SGI0, SGI0, SGI189217, SGI189217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41968[] = {SGI189218, SGI0, SGI189219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189220, SGI189221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41969[] = {SGI189222, SGI0, SGI189223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189224, SGI189225, SGI189225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41970[] = {SGI189226, SGI0, SGI189227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189231, SGI189232, SGI189232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41971[] = {SGI189228, SGI189229, SGI189230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189233, SGI0, SGI0, SGI189234, SGI189235, SGI189235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41972[] = {SGI189236, SGI189237, SGI189238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189239, SGI0, SGI0, SGI189240, SGI189241, SGI189241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41973[] = {SGI189242, SGI189243, SGI189244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189245, SGI0, SGI0, SGI189246, SGI189247, SGI189247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41974[] = {SGI189248, SGI0, SGI189249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189250, SGI189251, SGI189251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41975[] = {SGI189252, SGI189253, SGI189254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189255, SGI189255, SGI0, SGI0, SGI0, SGI189256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41976[] = {SGI189257, SGI189258, SGI189259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189260, SGI189260, SGI0, SGI0, SGI0, SGI189261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41977[] = {SGI189262, SGI189263, SGI189264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189265, SGI189265, SGI0, SGI0, SGI0, SGI189266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41978[] = {SGI189267, SGI189268, SGI189269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189270, SGI189270, SGI0, SGI0, SGI0, SGI189271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41979[] = {SGI189272, SGI0, SGI189273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189275, SGI189277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41980[] = {SGI189274, SGI189276, SGI189278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189279, SGI189279, SGI0, SGI0, SGI0, SGI189280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41981[] = {SGI189281, SGI189282, SGI189283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189284, SGI189284, SGI0, SGI0, SGI0, SGI189285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41982[] = {SGI189286, SGI0, SGI189287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189288, SGI189289, SGI189289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41983[] = {SGI189290, SGI0, SGI189291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189292, SGI189293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41984[] = {SGI189294, SGI0, SGI189295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189296, SGI189297, SGI189297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41985[] = {SGI189298, SGI0, SGI189299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189300, SGI189301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41986[] = {SGI189302, SGI189303, SGI189304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189305, SGI0, SGI0, SGI0, SGI189306, SGI189306, SGI0, SGI189307, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41987[] = {SGI189308, SGI189309, SGI189310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189311, SGI0, SGI0, SGI0, SGI189312, SGI189312, SGI0, SGI189313, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41988[] = {SGI189314, SGI189315, SGI189316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189317, SGI0, SGI0, SGI0, SGI189318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41989[] = {SGI189319, SGI0, SGI189320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41990[] = {SGI189321, SGI189322, SGI189323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189324, SGI0, SGI0, SGI0, SGI189325, SGI189325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41991[] = {SGI189326, SGI189327, SGI189328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189329, SGI0, SGI0, SGI0, SGI189330, SGI189330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41992[] = {SGI189331, SGI189332, SGI189333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189334, SGI189334, SGI0, SGI0, SGI0, SGI0, SGI189335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41993[] = {SGI189336, SGI0, SGI189337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189338, SGI189339, SGI189339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41994[] = {SGI189340, SGI189341, SGI189342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189343, SGI0, SGI0, SGI0, SGI189344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41995[] = {SGI189345, SGI0, SGI189346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189347, SGI189348, SGI189348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41996[] = {SGI189349, SGI0, SGI189350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41997[] = {SGI189351, SGI189352, SGI189353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189354, SGI0, SGI0, SGI0, SGI189355, SGI189355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41998[] = {SGI189356, SGI0, SGI189357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189358, SGI189359, SGI189359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG41999[] = {SGI189360, SGI189361, SGI189362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189363, SGI0, SGI0, SGI0, SGI189364, SGI189364, SGI0, SGI189365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42000[] = {SGI189366, SGI189367, SGI189368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189369, SGI0, SGI0, SGI0, SGI189370, SGI189370, SGI0, SGI189371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42001[] = {SGI189372, SGI0, SGI189373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189374, SGI189375, SGI189375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42002[] = {SGI189378, SGI189380, SGI189381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189384, SGI0, SGI0, SGI0, SGI189386, SGI189386, SGI0, SGI189387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42003[] = {SGI189376, SGI189377, SGI189379, SGI0, SGI0, SGI0, SGI189382, SGI189383, SGI0, SGI189385, SGI0, SGI0, SGI0, SGI189388, SGI189388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42004[] = {SGI189389, SGI189390, SGI189391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189392, SGI0, SGI0, SGI0, SGI189393, SGI189393, SGI0, SGI189394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42005[] = {SGI189395, SGI0, SGI189396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189397, SGI189398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42006[] = {SGI189399, SGI0, SGI189400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189401, SGI189403, SGI189403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42007[] = {SGI189402, SGI0, SGI189404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189405, SGI189406, SGI189406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42008[] = {SGI189407, SGI189408, SGI189409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189410, SGI0, SGI0, SGI0, SGI189411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42009[] = {SGI189412, SGI0, SGI189413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189414, SGI189415, SGI189415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42010[] = {SGI189416, SGI0, SGI189417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189418, SGI189419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42011[] = {SGI189420, SGI0, SGI189421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189422, SGI189424, SGI189424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42012[] = {SGI189423, SGI189425, SGI189426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189427, SGI0, SGI0, SGI0, SGI189428, SGI189428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42013[] = {SGI189429, SGI0, SGI189430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189431, SGI189432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42014[] = {SGI189433, SGI189434, SGI189435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189436, SGI0, SGI0, SGI0, SGI189437, SGI189437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42015[] = {SGI189438, SGI189439, SGI189440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189441, SGI0, SGI0, SGI0, SGI189442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42016[] = {SGI189443, SGI189444, SGI189445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189446, SGI189446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42017[] = {SGI189447, SGI0, SGI189448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42018[] = {SGI189449, SGI189451, SGI189453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189455, SGI0, SGI0, SGI0, SGI189456, SGI189456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42019[] = {SGI0, SGI189450, SGI189452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189454, SGI0, SGI0, SGI0, SGI189457, SGI189457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42020[] = {SGI189458, SGI0, SGI189459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42021[] = {SGI189460, SGI189461, SGI189462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189463, SGI0, SGI0, SGI0, SGI189464, SGI189464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42022[] = {SGI189465, SGI189466, SGI189467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189468, SGI0, SGI0, SGI0, SGI189469, SGI189469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42023[] = {SGI189470, SGI189471, SGI189472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189473, SGI0, SGI0, SGI0, SGI189474, SGI189474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42024[] = {SGI189475, SGI0, SGI189476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189477, SGI189478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42025[] = {SGI189479, SGI189480, SGI189481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189482, SGI0, SGI0, SGI0, SGI189483, SGI189483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42026[] = {SGI189484, SGI189485, SGI189486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189487, SGI189487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42027[] = {SGI189488, SGI0, SGI189489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189490, SGI189491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42028[] = {SGI189492, SGI189493, SGI189494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189495, SGI189496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42029[] = {SGI189497, SGI0, SGI189498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189499, SGI189500, SGI189500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189501, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42030[] = {SGI189502, SGI0, SGI189503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189504, SGI189505, SGI189505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189506, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42031[] = {SGI189507, SGI0, SGI189508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189509, SGI189510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42032[] = {SGI189511, SGI0, SGI189512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189513, SGI189514, SGI189514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42033[] = {SGI189515, SGI189516, SGI189517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189518, SGI0, SGI0, SGI0, SGI189519, SGI189519, SGI0, SGI189520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42034[] = {SGI189521, SGI0, SGI189522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189523, SGI189524, SGI189524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42035[] = {SGI189525, SGI0, SGI189526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189527, SGI189528, SGI189528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42036[] = {SGI189529, SGI189530, SGI189531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189532, SGI0, SGI0, SGI0, SGI189533, SGI189533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42037[] = {SGI189534, SGI189535, SGI189536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189537, SGI0, SGI0, SGI0, SGI189538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42038[] = {SGI189539, SGI0, SGI189540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42039[] = {SGI189541, SGI189542, SGI189543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189544, SGI0, SGI0, SGI0, SGI189545, SGI189545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42040[] = {SGI189546, SGI189548, SGI189550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189552, SGI0, SGI0, SGI0, SGI189554, SGI189554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42041[] = {SGI189547, SGI189549, SGI189551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189553, SGI0, SGI0, SGI0, SGI189555, SGI189555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42042[] = {SGI189557, SGI189558, SGI189559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189560, SGI189560, SGI0, SGI0, SGI0, SGI189561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42043[] = {SGI189562, SGI189563, SGI189564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189565, SGI189565, SGI0, SGI0, SGI0, SGI189566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42044[] = {SGI189567, SGI0, SGI189568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189569, SGI189570, SGI189570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42045[] = {SGI189571, SGI0, SGI189572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189573, SGI189574, SGI189574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42046[] = {SGI189575, SGI0, SGI189576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189577, SGI189578, SGI189578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42047[] = {SGI189579, SGI0, SGI189580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189581, SGI189582, SGI189582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42048[] = {SGI189583, SGI189584, SGI189585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189586, SGI0, SGI0, SGI189587, SGI189588, SGI189588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42049[] = {SGI189589, SGI0, SGI189590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42050[] = {SGI189591, SGI189592, SGI189593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189594, SGI0, SGI0, SGI0, SGI189595, SGI189595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42051[] = {SGI189596, SGI189597, SGI189598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189599, SGI0, SGI0, SGI0, SGI189600, SGI189600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42052[] = {SGI189601, SGI0, SGI189602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189603, SGI189604, SGI189604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42053[] = {SGI189605, SGI189606, SGI189607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189608, SGI189608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42054[] = {SGI189609, SGI0, SGI189610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189611, SGI189612, SGI189612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42055[] = {SGI189614, SGI189615, SGI189617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189619, SGI0, SGI0, SGI0, SGI189621, SGI189621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42056[] = {SGI189613, SGI189616, SGI189618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189620, SGI0, SGI0, SGI0, SGI189622, SGI189622, SGI0, SGI189623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42057[] = {SGI189624, SGI0, SGI189625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189626, SGI189627, SGI189627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42058[] = {SGI189628, SGI189629, SGI189630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189631, SGI0, SGI0, SGI0, SGI189632, SGI189632, SGI0, SGI189633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42059[] = {SGI189634, SGI0, SGI189635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189636, SGI189637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42060[] = {SGI189638, SGI0, SGI189639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189640, SGI189641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42061[] = {SGI189642, SGI0, SGI189643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189644, SGI189645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42062[] = {SGI189646, SGI0, SGI189647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189648, SGI189649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42063[] = {SGI189650, SGI0, SGI189651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189652, SGI189653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42064[] = {SGI189654, SGI189655, SGI189656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189657, SGI189657, SGI0, SGI0, SGI0, SGI189658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42065[] = {SGI189659, SGI0, SGI189660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189661, SGI189662, SGI189662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42066[] = {SGI189663, SGI0, SGI189664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189665, SGI189666, SGI189666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189667, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42067[] = {SGI189668, SGI189669, SGI189670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189671, SGI0, SGI0, SGI0, SGI0, SGI189672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42068[] = {SGI189673, SGI0, SGI189674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189678, SGI189679, SGI189679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42069[] = {SGI189675, SGI189676, SGI189677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189680, SGI189680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42070[] = {SGI189681, SGI0, SGI189683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189686, SGI189688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42071[] = {SGI189682, SGI189684, SGI189685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189687, SGI0, SGI0, SGI0, SGI189689, SGI189689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42072[] = {SGI189690, SGI0, SGI189691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189692, SGI189693, SGI189693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42073[] = {SGI189694, SGI0, SGI189695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189696, SGI189697, SGI189697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42074[] = {SGI189698, SGI0, SGI189699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189700, SGI189701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42075[] = {SGI189702, SGI0, SGI189703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189704, SGI189705, SGI189705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42076[] = {SGI189706, SGI0, SGI189707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189708, SGI189709, SGI189709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42077[] = {SGI189710, SGI189711, SGI189712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189713, SGI0, SGI0, SGI0, SGI189714, SGI189714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42078[] = {SGI189715, SGI0, SGI189716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189717, SGI189718, SGI189718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42079[] = {SGI189719, SGI189720, SGI189721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189722, SGI0, SGI0, SGI0, SGI189724, SGI189724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42080[] = {SGI189723, SGI0, SGI189725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189726, SGI189727, SGI189727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42081[] = {SGI189728, SGI0, SGI189729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189730, SGI189731, SGI189731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42082[] = {SGI189732, SGI0, SGI189733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189734, SGI189735, SGI189735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42083[] = {SGI189736, SGI0, SGI189737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189738, SGI189739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42084[] = {SGI189740, SGI0, SGI189741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189742, SGI189743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42085[] = {SGI189744, SGI0, SGI189745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189746, SGI189747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42086[] = {SGI189748, SGI0, SGI189749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189750, SGI189751, SGI189751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42087[] = {SGI189752, SGI189753, SGI189754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189755, SGI0, SGI0, SGI0, SGI189756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42088[] = {SGI189757, SGI0, SGI189758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189759, SGI189760, SGI189760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42089[] = {SGI189761, SGI0, SGI189762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189763, SGI189764, SGI189764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42090[] = {SGI189765, SGI189766, SGI189767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189768, SGI0, SGI0, SGI0, SGI189769, SGI189769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42091[] = {SGI189770, SGI0, SGI189771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189772, SGI189773, SGI189773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42092[] = {SGI189774, SGI189775, SGI189776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189777, SGI0, SGI0, SGI0, SGI189778, SGI189778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42093[] = {SGI189779, SGI189780, SGI189781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189782, SGI0, SGI0, SGI0, SGI189783, SGI189783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42094[] = {SGI189784, SGI189785, SGI189786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189787, SGI0, SGI0, SGI0, SGI189788, SGI189788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42095[] = {SGI189789, SGI0, SGI189790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42096[] = {SGI189791, SGI189792, SGI189793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189794, SGI0, SGI0, SGI0, SGI189795, SGI189795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42097[] = {SGI189796, SGI0, SGI189797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189798, SGI189799, SGI189799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42098[] = {SGI189800, SGI0, SGI189801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189802, SGI189803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42099[] = {SGI189804, SGI0, SGI189805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189806, SGI189807, SGI189807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42100[] = {SGI189808, SGI189809, SGI189810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189811, SGI0, SGI0, SGI0, SGI189812, SGI0, SGI0, SGI189813, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42101[] = {SGI189814, SGI189815, SGI189816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189817, SGI0, SGI0, SGI0, SGI189818, SGI0, SGI0, SGI189819, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42102[] = {SGI189820, SGI189821, SGI189822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189823, SGI0, SGI0, SGI0, SGI189824, SGI189824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42103[] = {SGI189825, SGI0, SGI189826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189827, SGI189828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42104[] = {SGI189829, SGI0, SGI189830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189831, SGI189832, SGI189832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42105[] = {SGI189833, SGI0, SGI189834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189837, SGI189838, SGI189838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42106[] = {SGI189835, SGI0, SGI189836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189839, SGI189840, SGI189840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189841, SGI0, SGI0, NULL};
const Segment * const SG42107[] = {SGI189842, SGI0, SGI189843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189844, SGI189845, SGI189845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42108[] = {SGI189846, SGI0, SGI189847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189848, SGI189849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42109[] = {SGI189850, SGI189851, SGI189852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189853, SGI0, SGI0, SGI0, SGI189854, SGI189854, SGI0, SGI189855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42110[] = {SGI189856, SGI0, SGI189857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189861, SGI189862, SGI189862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42111[] = {SGI189858, SGI189859, SGI189860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189863, SGI0, SGI0, SGI0, SGI189864, SGI189864, SGI0, SGI189865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42112[] = {SGI189866, SGI0, SGI189867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189868, SGI189869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42113[] = {SGI189870, SGI0, SGI189871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189873, SGI189874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42114[] = {SGI189872, SGI0, SGI189875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189876, SGI189877, SGI189877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42115[] = {SGI189878, SGI0, SGI189879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189880, SGI0, SGI0, SGI0, SGI189881, SGI189881, SGI0, SGI0, SGI0, SGI189882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42116[] = {SGI189883, SGI0, SGI189884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189885, SGI189886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42117[] = {SGI189887, SGI189888, SGI189889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189890, SGI0, SGI0, SGI0, SGI189891, SGI0, SGI189893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42118[] = {SGI189892, SGI0, SGI189894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189895, SGI189896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42119[] = {SGI189897, SGI0, SGI189898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189899, SGI189900, SGI189900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42120[] = {SGI189901, SGI0, SGI189902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189903, SGI189904, SGI189904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42121[] = {SGI189905, SGI189906, SGI189907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189908, SGI0, SGI0, SGI0, SGI189909, SGI189909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42122[] = {SGI189910, SGI189911, SGI189912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189913, SGI189913, SGI0, SGI0, SGI0, SGI0, SGI189914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42123[] = {SGI189915, SGI189916, SGI189917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189918, SGI0, SGI0, SGI0, SGI189919, SGI189919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42124[] = {SGI189920, SGI189921, SGI189922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189923, SGI189923, SGI0, SGI0, SGI0, SGI0, SGI189924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42125[] = {SGI189925, SGI189926, SGI189927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189928, SGI189928, SGI0, SGI0, SGI0, SGI0, SGI189929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42126[] = {SGI189930, SGI0, SGI189931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189932, SGI189933, SGI189933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42127[] = {SGI189934, SGI0, SGI189935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189936, SGI189937, SGI189937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42128[] = {SGI189938, SGI0, SGI189939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189940, SGI189941, SGI189941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42129[] = {SGI189942, SGI189943, SGI189944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189945, SGI189945, SGI0, SGI0, SGI0, SGI0, SGI189946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42130[] = {SGI189947, SGI189948, SGI189949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189950, SGI189950, SGI0, SGI0, SGI0, SGI0, SGI189951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42131[] = {SGI189952, SGI189953, SGI189954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189955, SGI189955, SGI0, SGI0, SGI0, SGI0, SGI189956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42132[] = {SGI189957, SGI189958, SGI189959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189960, SGI189960, SGI0, SGI0, SGI0, SGI0, SGI189961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42133[] = {SGI189962, SGI0, SGI189963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189964, SGI189965, SGI189965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42134[] = {SGI189966, SGI189967, SGI189968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189969, SGI0, SGI0, SGI0, SGI189970, SGI189970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42135[] = {SGI189971, SGI189972, SGI189973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189974, SGI0, SGI0, SGI0, SGI189975, SGI189975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42136[] = {SGI189976, SGI0, SGI189977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189978, SGI189979, SGI189979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42137[] = {SGI189980, SGI0, SGI189981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189982, SGI189983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42138[] = {SGI189984, SGI0, SGI189985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189986, SGI189987, SGI189987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42139[] = {SGI189988, SGI0, SGI189989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189990, SGI189991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42140[] = {SGI189992, SGI0, SGI189993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189994, SGI189995, SGI189995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42141[] = {SGI189996, SGI189997, SGI189998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI189999, SGI0, SGI0, SGI0, SGI190000, SGI190000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42142[] = {SGI190001, SGI0, SGI190003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190004, SGI190005, SGI190005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42143[] = {SGI190006, SGI190007, SGI190008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190009, SGI0, SGI0, SGI0, SGI190010, SGI190010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42144[] = {SGI190012, SGI190013, SGI190014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190016, SGI0, SGI0, SGI0, SGI190018, SGI190018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42145[] = {SGI190015, SGI0, SGI190017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190020, SGI190021, SGI190021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190022, SGI0, SGI0, NULL};
const Segment * const SG42146[] = {SGI190023, SGI190024, SGI190025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190026, SGI0, SGI0, SGI0, SGI190027, SGI190027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42147[] = {SGI190028, SGI0, SGI190029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42148[] = {SGI190030, SGI0, SGI190031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190032, SGI190033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42149[] = {SGI190034, SGI190035, SGI190036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190037, SGI0, SGI0, SGI0, SGI190038, SGI190038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42150[] = {SGI190039, SGI190040, SGI190041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190042, SGI190042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42151[] = {SGI190043, SGI190044, SGI190045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190046, SGI0, SGI0, SGI0, SGI190047, SGI190047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42152[] = {SGI190048, SGI0, SGI190049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42153[] = {SGI190050, SGI0, SGI190051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42154[] = {SGI190052, SGI0, SGI190053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190054, SGI190055, SGI190055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42155[] = {SGI190056, SGI0, SGI190057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190058, SGI190059, SGI190059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42156[] = {SGI190060, SGI190061, SGI190062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190063, SGI0, SGI0, SGI0, SGI190064, SGI190064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42157[] = {SGI190065, SGI190066, SGI190067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190068, SGI0, SGI0, SGI0, SGI190069, SGI190069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42158[] = {SGI190070, SGI0, SGI190071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190072, SGI190073, SGI190073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42159[] = {SGI190074, SGI0, SGI190075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190076, SGI190077, SGI190077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42160[] = {SGI190078, SGI0, SGI190079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190080, SGI190081, SGI190081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42161[] = {SGI190082, SGI190083, SGI190084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190085, SGI0, SGI0, SGI0, SGI190086, SGI190086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42162[] = {SGI190087, SGI0, SGI190088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42163[] = {SGI190091, SGI190092, SGI190093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190095, SGI0, SGI0, SGI0, SGI190097, SGI190097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42164[] = {SGI190089, SGI0, SGI190090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190094, SGI0, SGI0, SGI0, SGI190096, SGI190096, SGI0, SGI0, SGI190098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42165[] = {SGI190099, SGI190100, SGI190101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190102, SGI0, SGI0, SGI0, SGI190103, SGI190103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42166[] = {SGI190104, SGI0, SGI190105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190106, SGI0, SGI0, SGI0, SGI190108, SGI190108, SGI0, SGI0, SGI190110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42167[] = {SGI190107, SGI0, SGI190109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190111, SGI190112, SGI190112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42168[] = {SGI190113, SGI0, SGI190114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190115, SGI190116, SGI190116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42169[] = {SGI190117, SGI0, SGI190118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190119, SGI190120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42170[] = {SGI190121, SGI190122, SGI190123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190124, SGI0, SGI0, SGI0, SGI190125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42171[] = {SGI190126, SGI190127, SGI190128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190129, SGI0, SGI0, SGI0, SGI190130, SGI190130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42172[] = {SGI190131, SGI0, SGI190132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190133, SGI190134, SGI190134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42173[] = {SGI190135, SGI0, SGI190136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190137, SGI190138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42174[] = {SGI190139, SGI0, SGI190140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190141, SGI0, SGI0, SGI0, SGI190142, SGI190142, SGI0, SGI0, SGI0, SGI190143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42175[] = {SGI190144, SGI190145, SGI190146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190147, SGI0, SGI0, SGI0, SGI190148, SGI190148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42176[] = {SGI190149, SGI0, SGI190150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190151, SGI190152, SGI190152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190153, SGI0, SGI0, NULL};
const Segment * const SG42177[] = {SGI190154, SGI0, SGI190155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190156, SGI190157, SGI190157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190158, SGI0, SGI0, NULL};
const Segment * const SG42178[] = {SGI190159, SGI0, SGI190160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190161, SGI190162, SGI190162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190163, SGI0, SGI0, NULL};
const Segment * const SG42179[] = {SGI190164, SGI0, SGI190165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190166, SGI190167, SGI190167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42180[] = {SGI190168, SGI190169, SGI190170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190171, SGI0, SGI0, SGI0, SGI190172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42181[] = {SGI190173, SGI190174, SGI190175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190176, SGI0, SGI0, SGI0, SGI190177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42182[] = {SGI190178, SGI0, SGI190179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190180, SGI190181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42183[] = {SGI190182, SGI0, SGI190183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190184, SGI190185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42184[] = {SGI190186, SGI0, SGI190187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190188, SGI190189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42185[] = {SGI0, SGI190190, SGI190191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190192, SGI0, SGI0, SGI0, SGI190193, SGI190193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42186[] = {SGI0, SGI190194, SGI190195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190198, SGI0, SGI0, SGI0, SGI190200, SGI190200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42187[] = {SGI190196, SGI190197, SGI190199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190201, SGI0, SGI0, SGI0, SGI190202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42188[] = {SGI0, SGI190203, SGI190204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190208, SGI0, SGI0, SGI0, SGI190209, SGI190209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42189[] = {SGI190205, SGI190206, SGI190207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190210, SGI0, SGI0, SGI0, SGI190211, SGI190211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42190[] = {SGI190213, SGI0, SGI190215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42191[] = {SGI0, SGI190212, SGI190214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190216, SGI0, SGI0, SGI0, SGI190217, SGI190217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42192[] = {SGI0, SGI190218, SGI190219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190220, SGI0, SGI0, SGI0, SGI190221, SGI190221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42193[] = {SGI190222, SGI190223, SGI190224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190225, SGI0, SGI0, SGI0, SGI190226, SGI190226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42194[] = {SGI190227, SGI190228, SGI190229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190230, SGI0, SGI0, SGI0, SGI190231, SGI0, SGI190232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42195[] = {SGI190233, SGI190234, SGI190235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190236, SGI0, SGI0, SGI0, SGI190237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42196[] = {SGI190238, SGI190239, SGI190240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190241, SGI0, SGI0, SGI0, SGI190242, SGI190242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42197[] = {SGI190243, SGI190244, SGI190245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190246, SGI0, SGI0, SGI0, SGI190247, SGI190247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42198[] = {SGI190248, SGI190249, SGI190250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190251, SGI0, SGI0, SGI0, SGI190252, SGI190252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42199[] = {SGI190253, SGI0, SGI190254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190255, SGI190256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42200[] = {SGI190257, SGI190258, SGI190259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190260, SGI0, SGI0, SGI0, SGI190261, SGI190261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42201[] = {SGI190262, SGI0, SGI190263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190264, SGI190265, SGI190265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42202[] = {SGI190266, SGI190267, SGI190268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190269, SGI0, SGI0, SGI0, SGI190270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42203[] = {SGI190273, SGI0, SGI190274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42204[] = {SGI190271, SGI0, SGI190272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190275, SGI190276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42205[] = {SGI190277, SGI0, SGI190278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190279, SGI190280, SGI190280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42206[] = {SGI190281, SGI0, SGI190282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190285, SGI190287, SGI190287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42207[] = {SGI190283, SGI190284, SGI190286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190288, SGI0, SGI0, SGI0, SGI190289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42208[] = {SGI190290, SGI0, SGI190291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190292, SGI190293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42209[] = {SGI190294, SGI190295, SGI190296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190297, SGI0, SGI0, SGI0, SGI190298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42210[] = {SGI190299, SGI0, SGI190300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190301, SGI190302, SGI190302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42211[] = {SGI190303, SGI0, SGI190304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190305, SGI190306, SGI190306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42212[] = {SGI190307, SGI0, SGI190308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190309, SGI190310, SGI190310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42213[] = {SGI190311, SGI0, SGI190313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190315, SGI190317, SGI190317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42214[] = {SGI190312, SGI0, SGI190314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190316, SGI190318, SGI190318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42215[] = {SGI190319, SGI0, SGI190321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190324, SGI190325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42216[] = {SGI190320, SGI190322, SGI190323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190326, SGI190326, SGI0, SGI0, SGI0, SGI190327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42217[] = {SGI190328, SGI190329, SGI190330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190331, SGI190331, SGI0, SGI0, SGI0, SGI190332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42218[] = {SGI190333, SGI190334, SGI190335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190336, SGI0, SGI0, SGI0, SGI190337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42219[] = {SGI190339, SGI190340, SGI190341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190342, SGI0, SGI0, SGI0, SGI190343, SGI190343, SGI190344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42220[] = {SGI190345, SGI190346, SGI190347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190349, SGI0, SGI0, SGI0, SGI190352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42221[] = {SGI190348, SGI190350, SGI190351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190354, SGI0, SGI0, SGI0, SGI190355, SGI190355, SGI190356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42222[] = {SGI190357, SGI190358, SGI190359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190360, SGI0, SGI0, SGI0, SGI190361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42223[] = {SGI190362, SGI190363, SGI190364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190365, SGI0, SGI0, SGI0, SGI190366, SGI190366, SGI190367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42224[] = {SGI190368, SGI190369, SGI190370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190371, SGI0, SGI0, SGI0, SGI190372, SGI190372, SGI190373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42225[] = {SGI190374, SGI190375, SGI190376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190377, SGI0, SGI0, SGI0, SGI190378, SGI190378, SGI190379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42226[] = {SGI190380, SGI190381, SGI190382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190383, SGI0, SGI0, SGI0, SGI190384, SGI190384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42227[] = {SGI190385, SGI0, SGI190386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42228[] = {SGI190387, SGI190388, SGI190389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190390, SGI190390, SGI0, SGI0, SGI0, SGI0, SGI190391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42229[] = {SGI190392, SGI0, SGI190393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190394, SGI190395, SGI190395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42230[] = {SGI190396, SGI0, SGI190397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190398, SGI190399, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42231[] = {SGI190400, SGI190401, SGI190402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190403, SGI0, SGI0, SGI0, SGI190404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42232[] = {SGI190405, SGI190406, SGI190407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190408, SGI0, SGI0, SGI0, SGI190409, SGI190409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42233[] = {SGI190411, SGI0, SGI190413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190416, SGI190417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42234[] = {SGI190410, SGI190412, SGI190414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190415, SGI0, SGI0, SGI0, SGI190418, SGI190418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42235[] = {SGI190419, SGI0, SGI190420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190421, SGI190422, SGI190422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42236[] = {SGI190423, SGI190424, SGI190425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190426, SGI0, SGI0, SGI0, SGI190427, SGI190427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42237[] = {SGI190428, SGI0, SGI190429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190430, SGI190431, SGI190431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42238[] = {SGI190432, SGI0, SGI190433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190434, SGI190435, SGI190435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42239[] = {SGI190436, SGI0, SGI190437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190438, SGI190439, SGI190439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42240[] = {SGI190440, SGI0, SGI190441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190442, SGI190443, SGI190443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42241[] = {SGI190444, SGI0, SGI190445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190446, SGI190447, SGI190447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42242[] = {SGI190448, SGI0, SGI190449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190450, SGI190451, SGI190451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42243[] = {SGI190452, SGI0, SGI190453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190454, SGI190455, SGI190455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42244[] = {SGI190456, SGI190457, SGI190458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190459, SGI0, SGI0, SGI0, SGI190460, SGI190460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42245[] = {SGI190461, SGI0, SGI190462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42246[] = {SGI190463, SGI0, SGI190464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190465, SGI190466, SGI190466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42247[] = {SGI190467, SGI0, SGI190468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190469, SGI190470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42248[] = {SGI190471, SGI0, SGI190472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42249[] = {SGI190473, SGI0, SGI190474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190475, SGI190476, SGI190476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42250[] = {SGI190477, SGI190478, SGI190479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190480, SGI0, SGI0, SGI0, SGI190481, SGI190481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42251[] = {SGI190482, SGI190483, SGI190484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190485, SGI0, SGI0, SGI0, SGI190486, SGI190486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42252[] = {SGI190487, SGI190488, SGI190489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190490, SGI0, SGI0, SGI0, SGI190491, SGI190491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42253[] = {SGI190492, SGI0, SGI190493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190494, SGI190495, SGI190495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42254[] = {SGI190496, SGI0, SGI190497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190498, SGI190499, SGI190499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42255[] = {SGI190500, SGI0, SGI190501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190502, SGI190503, SGI190503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42256[] = {SGI0, SGI0, SGI190504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190505, SGI190506, SGI190506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42257[] = {SGI190507, SGI0, SGI190508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190509, SGI190510, SGI190510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42258[] = {SGI190511, SGI0, SGI190512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190513, SGI190514, SGI190514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42259[] = {SGI190515, SGI0, SGI190516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190517, SGI190518, SGI190518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42260[] = {SGI190519, SGI0, SGI190520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190523, SGI190525, SGI190525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42261[] = {SGI190521, SGI190522, SGI190524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190526, SGI0, SGI0, SGI0, SGI190527, SGI190527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42262[] = {SGI190529, SGI190530, SGI190531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190534, SGI0, SGI0, SGI0, SGI190535, SGI190535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42263[] = {SGI190532, SGI0, SGI190533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190536, SGI190538, SGI190538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42264[] = {SGI190539, SGI0, SGI190540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190541, SGI190542, SGI190542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42265[] = {SGI190543, SGI190544, SGI190545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190548, SGI0, SGI0, SGI0, SGI190549, SGI190549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42266[] = {SGI190546, SGI0, SGI190547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190551, SGI190552, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42267[] = {SGI190553, SGI0, SGI190554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190558, SGI190559, SGI190559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42268[] = {SGI190555, SGI190556, SGI190557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190560, SGI0, SGI0, SGI0, SGI190561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42269[] = {SGI190562, SGI0, SGI190563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190564, SGI190565, SGI190565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42270[] = {SGI190566, SGI0, SGI190567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190568, SGI190569, SGI190569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42271[] = {SGI190570, SGI0, SGI190571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190572, SGI190573, SGI190573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42272[] = {SGI190574, SGI0, SGI190575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190576, SGI190577, SGI190577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42273[] = {SGI190578, SGI0, SGI190579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190581, SGI190582, SGI190582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42274[] = {SGI190580, SGI0, SGI190583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190584, SGI190585, SGI190585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42275[] = {SGI190586, SGI0, SGI190587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190588, SGI190589, SGI190589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42276[] = {SGI190590, SGI0, SGI190591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190592, SGI190593, SGI190593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42277[] = {SGI190594, SGI0, SGI190595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190596, SGI190597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42278[] = {SGI190598, SGI0, SGI190599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190600, SGI190601, SGI190601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42279[] = {SGI190602, SGI0, SGI190603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190604, SGI190605, SGI190605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42280[] = {SGI190606, SGI0, SGI190607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190608, SGI190610, SGI190610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42281[] = {SGI190609, SGI0, SGI190611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190612, SGI190613, SGI190613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42282[] = {SGI190614, SGI0, SGI190615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190616, SGI190617, SGI190617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42283[] = {SGI190618, SGI0, SGI190619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190620, SGI190621, SGI190621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42284[] = {SGI190622, SGI0, SGI190623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190624, SGI0, SGI0, SGI0, SGI190625, SGI190625, SGI0, SGI0, SGI0, SGI190627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42285[] = {SGI190626, SGI0, SGI190628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190629, SGI190630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190631, SGI0, NULL};
const Segment * const SG42286[] = {SGI190632, SGI190633, SGI190634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190635, SGI0, SGI0, SGI0, SGI190636, SGI190637, SGI0, SGI0, SGI0, SGI190638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42287[] = {SGI190639, SGI0, SGI190640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190641, SGI190642, SGI190642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42288[] = {SGI190643, SGI0, SGI190644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190645, SGI190646, SGI190646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42289[] = {SGI190647, SGI0, SGI190648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190649, SGI190650, SGI190650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42290[] = {SGI190651, SGI0, SGI190652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190653, SGI190654, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42291[] = {SGI190655, SGI0, SGI190656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190657, SGI190658, SGI190658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42292[] = {SGI190659, SGI0, SGI190660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190661, SGI190662, SGI190662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42293[] = {SGI190663, SGI0, SGI190664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190665, SGI190666, SGI190666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42294[] = {SGI190667, SGI0, SGI190668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190669, SGI190670, SGI190670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42295[] = {SGI190671, SGI0, SGI190672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190673, SGI190674, SGI190674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42296[] = {SGI190675, SGI0, SGI190676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190677, SGI190678, SGI190678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42297[] = {SGI190679, SGI190680, SGI190681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190682, SGI0, SGI0, SGI0, SGI190683, SGI190683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42298[] = {SGI190684, SGI190685, SGI190686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190687, SGI0, SGI0, SGI0, SGI190688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42299[] = {SGI190689, SGI0, SGI190690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42300[] = {SGI190691, SGI0, SGI190692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190693, SGI190694, SGI190694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42301[] = {SGI190695, SGI190696, SGI190697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190698, SGI0, SGI0, SGI0, SGI190699, SGI190699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190700, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42302[] = {SGI190701, SGI190702, SGI190703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190704, SGI0, SGI0, SGI0, SGI190705, SGI0, SGI190706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42303[] = {SGI190707, SGI190708, SGI190709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190710, SGI0, SGI0, SGI0, SGI190711, SGI0, SGI190712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42304[] = {SGI190713, SGI190714, SGI190715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190719, SGI0, SGI0, SGI0, SGI190720, SGI0, SGI0, SGI190721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42305[] = {SGI190716, SGI190717, SGI190718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190722, SGI0, SGI0, SGI0, SGI190723, SGI0, SGI190724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42306[] = {SGI190725, SGI190726, SGI190727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190728, SGI0, SGI0, SGI0, SGI190729, SGI0, SGI190730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42307[] = {SGI190731, SGI190732, SGI190733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190734, SGI0, SGI0, SGI0, SGI190735, SGI0, SGI0, SGI190736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42308[] = {SGI190737, SGI190738, SGI190739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190740, SGI0, SGI0, SGI0, SGI190741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42309[] = {SGI190742, SGI0, SGI190743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190744, SGI190745, SGI190745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190746, SGI0, SGI0, NULL};
const Segment * const SG42310[] = {SGI190747, SGI0, SGI190748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190749, SGI190750, SGI190750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190751, SGI0, SGI0, NULL};
const Segment * const SG42311[] = {SGI190752, SGI0, SGI190753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190754, SGI190755, SGI190755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190756, SGI0, SGI0, NULL};
const Segment * const SG42312[] = {SGI190757, SGI0, SGI190758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190759, SGI190760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42313[] = {SGI190761, SGI190762, SGI190763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190764, SGI0, SGI0, SGI0, SGI190765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42314[] = {SGI190766, SGI0, SGI190767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190768, SGI0, SGI0, SGI0, SGI190769, SGI190769, SGI0, SGI0, SGI0, SGI190770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42315[] = {SGI190771, SGI190772, SGI190773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190774, SGI0, SGI0, SGI0, SGI190775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42316[] = {SGI190776, SGI0, SGI190777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190778, SGI0, SGI0, SGI0, SGI190779, SGI190779, SGI0, SGI0, SGI0, SGI190780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42317[] = {SGI190781, SGI190782, SGI190783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190784, SGI0, SGI0, SGI0, SGI190785, SGI190786, SGI0, SGI0, SGI0, SGI190787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42318[] = {SGI190788, SGI0, SGI190789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190790, SGI190791, SGI190791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42319[] = {SGI190792, SGI0, SGI190793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190794, SGI190795, SGI190795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42320[] = {SGI190796, SGI190797, SGI190798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190799, SGI190799, SGI0, SGI0, SGI0, SGI190800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42321[] = {SGI190801, SGI190802, SGI190803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190804, SGI190804, SGI0, SGI0, SGI0, SGI190805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42322[] = {SGI190806, SGI190807, SGI190808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190809, SGI190809, SGI0, SGI0, SGI0, SGI190810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42323[] = {SGI190811, SGI190813, SGI190814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190816, SGI0, SGI0, SGI0, SGI190819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42324[] = {SGI190812, SGI0, SGI190815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190817, SGI190818, SGI190818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42325[] = {SGI190820, SGI190821, SGI190822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190823, SGI0, SGI0, SGI0, SGI190824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42326[] = {SGI190825, SGI0, SGI190826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190827, SGI190828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42327[] = {SGI190829, SGI0, SGI190830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190831, SGI190832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42328[] = {SGI190833, SGI0, SGI190834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190835, SGI190836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190837, SGI0, SGI0, NULL};
const Segment * const SG42329[] = {SGI190838, SGI190839, SGI190840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190841, SGI0, SGI0, SGI0, SGI190842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42330[] = {SGI190843, SGI190844, SGI190845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190846, SGI0, SGI0, SGI0, SGI190847, SGI190847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42331[] = {SGI190848, SGI0, SGI190849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42332[] = {SGI190850, SGI0, SGI190851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42333[] = {SGI190852, SGI190853, SGI190854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190855, SGI0, SGI0, SGI0, SGI190856, SGI190856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42334[] = {SGI190858, SGI0, SGI190859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190860, SGI190861, SGI190861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42335[] = {SGI190862, SGI190863, SGI190864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190865, SGI0, SGI0, SGI0, SGI190866, SGI190866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42336[] = {SGI190868, SGI0, SGI190869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190870, SGI190871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42337[] = {SGI190872, SGI0, SGI190873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190874, SGI190875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42338[] = {SGI190876, SGI190877, SGI190878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190879, SGI0, SGI0, SGI0, SGI190880, SGI190880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42339[] = {SGI190881, SGI0, SGI190882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190883, SGI190884, SGI190884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42340[] = {SGI190885, SGI190886, SGI190887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190888, SGI0, SGI0, SGI0, SGI190889, SGI190889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42341[] = {SGI190890, SGI190891, SGI190892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190893, SGI0, SGI0, SGI0, SGI190894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42342[] = {SGI190895, SGI190896, SGI190897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190898, SGI0, SGI0, SGI0, SGI190899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42343[] = {SGI190900, SGI0, SGI190901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190902, SGI190903, SGI190903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190904, SGI0, SGI0, NULL};
const Segment * const SG42344[] = {SGI190905, SGI0, SGI190907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190909, SGI190911, SGI190911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190913, SGI0, SGI0, NULL};
const Segment * const SG42345[] = {SGI190906, SGI0, SGI190908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190910, SGI190912, SGI190912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42346[] = {SGI190914, SGI0, SGI190915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190916, SGI190917, SGI190917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42347[] = {SGI190918, SGI190919, SGI190920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190921, SGI0, SGI0, SGI0, SGI190922, SGI190922, SGI0, SGI190923, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42348[] = {SGI190924, SGI0, SGI190925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190926, SGI190927, SGI190927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42349[] = {SGI190928, SGI190929, SGI190930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190931, SGI0, SGI0, SGI0, SGI190932, SGI190932, SGI0, SGI190933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42350[] = {SGI190934, SGI190935, SGI190936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190937, SGI0, SGI0, SGI0, SGI190938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42351[] = {SGI190939, SGI190940, SGI190941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190942, SGI0, SGI0, SGI0, SGI190943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42352[] = {SGI190944, SGI190945, SGI190946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190947, SGI0, SGI0, SGI0, SGI190948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42353[] = {SGI190949, SGI0, SGI190950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190951, SGI190952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42354[] = {SGI190953, SGI190954, SGI190955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190956, SGI0, SGI0, SGI0, SGI190957, SGI190957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42355[] = {SGI190958, SGI190959, SGI190960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190961, SGI190961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42356[] = {SGI190962, SGI190963, SGI190964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190965, SGI0, SGI0, SGI0, SGI190966, SGI190966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42357[] = {SGI190967, SGI190968, SGI190969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190972, SGI0, SGI0, SGI0, SGI190973, SGI190973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42358[] = {SGI190970, SGI0, SGI190971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190974, SGI190975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42359[] = {SGI190976, SGI190977, SGI190978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190979, SGI0, SGI0, SGI0, SGI190980, SGI190980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42360[] = {SGI190981, SGI190982, SGI190983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190984, SGI190984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42361[] = {SGI190985, SGI190986, SGI190987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190988, SGI0, SGI0, SGI0, SGI190989, SGI190989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42362[] = {SGI190990, SGI190991, SGI190992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190993, SGI0, SGI0, SGI0, SGI190994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42363[] = {SGI190995, SGI190996, SGI190997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI190998, SGI0, SGI0, SGI0, SGI190999, SGI190999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42364[] = {SGI191000, SGI191001, SGI191002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191003, SGI0, SGI0, SGI0, SGI191004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42365[] = {SGI191005, SGI0, SGI191006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191007, SGI191008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42366[] = {SGI191009, SGI0, SGI191010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191011, SGI191012, SGI191012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42367[] = {SGI191013, SGI0, SGI191014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191015, SGI191016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42368[] = {SGI191017, SGI191018, SGI191019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191020, SGI0, SGI0, SGI0, SGI191021, SGI191021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42369[] = {SGI191022, SGI0, SGI191023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42370[] = {SGI191024, SGI191025, SGI191026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191027, SGI0, SGI0, SGI0, SGI191028, SGI191028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42371[] = {SGI191029, SGI191030, SGI191031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191032, SGI0, SGI0, SGI0, SGI191033, SGI191033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42372[] = {SGI191034, SGI191035, SGI191036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191037, SGI0, SGI0, SGI0, SGI191038, SGI191038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42373[] = {SGI191039, SGI0, SGI191040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191041, SGI191042, SGI191042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42374[] = {SGI191043, SGI191044, SGI191045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191046, SGI0, SGI0, SGI0, SGI191047, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42375[] = {SGI191048, SGI191049, SGI191050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191053, SGI0, SGI0, SGI0, SGI191054, SGI191054, SGI0, SGI0, NULL};
const Segment * const SG42376[] = {SGI191051, SGI0, SGI191052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191055, SGI191056, SGI191056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42377[] = {SGI191057, SGI191058, SGI191059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191060, SGI0, SGI0, SGI0, SGI191061, SGI191061, SGI0, SGI0, NULL};
const Segment * const SG42378[] = {SGI191062, SGI191063, SGI191064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191065, SGI0, SGI0, SGI0, SGI191066, SGI191066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42379[] = {SGI191067, SGI0, SGI191068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42380[] = {SGI191069, SGI191070, SGI191071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191072, SGI0, SGI0, SGI0, SGI191073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42381[] = {SGI191074, SGI191075, SGI191076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191077, SGI0, SGI0, SGI0, SGI191078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42382[] = {SGI191079, SGI0, SGI191080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191081, SGI191082, SGI191082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42383[] = {SGI191083, SGI0, SGI191084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191085, SGI191086, SGI191086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42384[] = {SGI191087, SGI0, SGI191088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191089, SGI191090, SGI191090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42385[] = {SGI191091, SGI0, SGI191092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191093, SGI191094, SGI191094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42386[] = {SGI191095, SGI0, SGI191096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191097, SGI191098, SGI191098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42387[] = {SGI191099, SGI0, SGI191100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191101, SGI191102, SGI191102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42388[] = {SGI191103, SGI191104, SGI191105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191106, SGI0, SGI0, SGI0, SGI191107, SGI191107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42389[] = {SGI191108, SGI0, SGI191109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191110, SGI191111, SGI191111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42390[] = {SGI191112, SGI191113, SGI191114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191115, SGI0, SGI0, SGI0, SGI191116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42391[] = {SGI191117, SGI0, SGI191118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191119, SGI191120, SGI191120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42392[] = {SGI191121, SGI191122, SGI191123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191124, SGI0, SGI0, SGI0, SGI191125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42393[] = {SGI191126, SGI0, SGI191127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191128, SGI191129, SGI191129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42394[] = {SGI191130, SGI191131, SGI191132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191133, SGI0, SGI0, SGI0, SGI191134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42395[] = {SGI191135, SGI0, SGI191136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191137, SGI191138, SGI191138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42396[] = {SGI191139, SGI0, SGI191140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191141, SGI191142, SGI191142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42397[] = {SGI191143, SGI0, SGI191144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191145, SGI191146, SGI191146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42398[] = {SGI191147, SGI0, SGI191148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191149, SGI191150, SGI191150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42399[] = {SGI191151, SGI191152, SGI191153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191154, SGI0, SGI0, SGI0, SGI191155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42400[] = {SGI191156, SGI191157, SGI191158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191159, SGI0, SGI0, SGI0, SGI191160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42401[] = {SGI191161, SGI191162, SGI191163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191164, SGI0, SGI0, SGI0, SGI191165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42402[] = {SGI191166, SGI191167, SGI191168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191169, SGI0, SGI0, SGI0, SGI191170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42403[] = {SGI191171, SGI191172, SGI191174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191177, SGI0, SGI0, SGI0, SGI191179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42404[] = {SGI191173, SGI191175, SGI191176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191178, SGI0, SGI0, SGI0, SGI191180, SGI191180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42405[] = {SGI191181, SGI191182, SGI191183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191184, SGI191184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42406[] = {SGI191185, SGI0, SGI191186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42407[] = {SGI191187, SGI0, SGI191188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191189, SGI191191, SGI191191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42408[] = {SGI191190, SGI0, SGI191192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191193, SGI191194, SGI191194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42409[] = {SGI191195, SGI0, SGI191196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191198, SGI191199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42410[] = {SGI191197, SGI0, SGI191200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191203, SGI191204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42411[] = {SGI191201, SGI0, SGI191202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191205, SGI191206, SGI191206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42412[] = {SGI191207, SGI0, SGI191208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191210, SGI191211, SGI191211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42413[] = {SGI191209, SGI0, SGI191212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191213, SGI191214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42414[] = {SGI191215, SGI0, SGI191216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191217, SGI191219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42415[] = {SGI191218, SGI0, SGI191220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191221, SGI191222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42416[] = {SGI191223, SGI0, SGI191224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191225, SGI191226, SGI191226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42417[] = {SGI191227, SGI0, SGI191228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191229, SGI191230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42418[] = {SGI191231, SGI191232, SGI191233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191234, SGI0, SGI0, SGI0, SGI191235, SGI191235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42419[] = {SGI191237, SGI191238, SGI191239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191240, SGI0, SGI0, SGI0, SGI191241, SGI191241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42420[] = {SGI191243, SGI191244, SGI191245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191246, SGI0, SGI0, SGI0, SGI191247, SGI191247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42421[] = {SGI191249, SGI0, SGI191250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191251, SGI191252, SGI191252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42422[] = {SGI191253, SGI0, SGI191254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191255, SGI191256, SGI191256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42423[] = {SGI191257, SGI0, SGI191258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191259, SGI191260, SGI191260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42424[] = {SGI191261, SGI0, SGI191262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191263, SGI191264, SGI191264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42425[] = {SGI191265, SGI0, SGI191267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191270, SGI191272, SGI191272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42426[] = {SGI191266, SGI191268, SGI191269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191271, SGI0, SGI0, SGI0, SGI191273, SGI191273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42427[] = {SGI191274, SGI0, SGI191275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191276, SGI191277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42428[] = {SGI191278, SGI191279, SGI191280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191281, SGI0, SGI0, SGI0, SGI191282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42429[] = {SGI191283, SGI191284, SGI191286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191289, SGI0, SGI0, SGI0, SGI191291, SGI191291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42430[] = {SGI191285, SGI191287, SGI191288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191290, SGI0, SGI0, SGI0, SGI191292, SGI191292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42431[] = {SGI191293, SGI0, SGI191294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42432[] = {SGI191295, SGI191296, SGI191297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191299, SGI0, SGI0, SGI0, SGI191302, SGI191302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42433[] = {SGI191298, SGI191300, SGI191301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191303, SGI0, SGI0, SGI0, SGI191304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42434[] = {SGI191305, SGI191306, SGI191307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191308, SGI0, SGI0, SGI0, SGI191309, SGI191309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42435[] = {SGI191310, SGI0, SGI191311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191312, SGI191313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42436[] = {SGI191314, SGI0, SGI191315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191316, SGI191317, SGI191317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42437[] = {SGI191318, SGI191319, SGI191320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191321, SGI0, SGI0, SGI0, SGI191322, SGI191322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42438[] = {SGI191323, SGI0, SGI191324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191325, SGI191326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42439[] = {SGI191327, SGI191328, SGI191329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191330, SGI191330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42440[] = {SGI191331, SGI0, SGI191332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191333, SGI191334, SGI191334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42441[] = {SGI191335, SGI191336, SGI191337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191338, SGI0, SGI0, SGI0, SGI191340, SGI191340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42442[] = {SGI191339, SGI0, SGI191341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191342, SGI191343, SGI191343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42443[] = {SGI0, SGI191344, SGI191345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42444[] = {SGI191347, SGI0, SGI191349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42445[] = {SGI191346, SGI0, SGI191348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191352, SGI191354, SGI191354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42446[] = {SGI191350, SGI191351, SGI191353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191355, SGI0, SGI0, SGI0, SGI191356, SGI191356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42447[] = {SGI191358, SGI191360, SGI191362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191367, SGI191367, SGI0, SGI0, SGI0, SGI191369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42448[] = {SGI191359, SGI191361, SGI191363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191366, SGI0, SGI0, SGI0, SGI191368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42449[] = {SGI191364, SGI0, SGI191365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191370, SGI191372, SGI191372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42450[] = {SGI191376, SGI0, SGI191380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191382, SGI191384, SGI191384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42451[] = {SGI191373, SGI191375, SGI191378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191383, SGI191383, SGI0, SGI0, SGI0, SGI191386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42452[] = {SGI191374, SGI191377, SGI191379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191381, SGI0, SGI0, SGI0, SGI191385, SGI191385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42453[] = {SGI191388, SGI191389, SGI191390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191391, SGI0, SGI0, SGI0, SGI191392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42454[] = {SGI191396, SGI0, SGI191398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191401, SGI191402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42455[] = {SGI191394, SGI191395, SGI191397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191399, SGI0, SGI0, SGI0, SGI191400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42456[] = {SGI191404, SGI0, SGI191405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191406, SGI0, SGI0, SGI0, SGI191407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42457[] = {SGI191409, SGI0, SGI191410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191411, SGI191412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42458[] = {SGI191413, SGI191414, SGI191415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191416, SGI0, SGI0, SGI0, SGI191417, SGI191417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42459[] = {SGI191418, SGI191419, SGI191420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191421, SGI0, SGI0, SGI0, SGI191422, SGI191422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42460[] = {SGI191423, SGI0, SGI191424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42461[] = {SGI191425, SGI191426, SGI191427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191428, SGI0, SGI0, SGI0, SGI191429, SGI191429, SGI0, SGI191430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42462[] = {SGI191431, SGI0, SGI191432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191433, SGI191434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191435, SGI0, SGI0, NULL};
const Segment * const SG42463[] = {SGI191436, SGI0, SGI191437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191438, SGI191439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42464[] = {SGI191440, SGI0, SGI191441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191442, SGI191443, SGI191443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42465[] = {SGI191444, SGI0, SGI191445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191446, SGI191447, SGI191447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42466[] = {SGI191449, SGI191451, SGI191453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191455, SGI0, SGI0, SGI0, SGI191456, SGI191456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42467[] = {SGI191448, SGI191450, SGI191452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191454, SGI0, SGI0, SGI0, SGI191457, SGI0, SGI0, SGI191458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42468[] = {SGI191459, SGI0, SGI191460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42469[] = {SGI191461, SGI191462, SGI191463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191464, SGI191464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42470[] = {SGI191465, SGI0, SGI191466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191467, SGI191468, SGI191468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42471[] = {SGI191469, SGI0, SGI191470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191471, SGI191472, SGI191472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42472[] = {SGI191473, SGI0, SGI191474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191475, SGI191476, SGI191476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42473[] = {SGI191477, SGI0, SGI191478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191479, SGI191480, SGI191480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42474[] = {SGI191481, SGI0, SGI191482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191483, SGI191484, SGI191484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42475[] = {SGI191485, SGI0, SGI191486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191487, SGI191488, SGI191488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42476[] = {SGI191490, SGI0, SGI191493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191495, SGI191497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42477[] = {SGI191489, SGI191491, SGI191492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191494, SGI0, SGI0, SGI0, SGI191496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42478[] = {SGI191498, SGI0, SGI191499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191500, SGI191501, SGI191501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42479[] = {SGI191502, SGI191503, SGI191504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191505, SGI0, SGI0, SGI0, SGI191506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42480[] = {SGI191507, SGI0, SGI191508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191512, SGI191513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42481[] = {SGI191509, SGI191510, SGI191511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42482[] = {SGI191516, SGI0, SGI191517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42483[] = {SGI191514, SGI0, SGI191515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191518, SGI191519, SGI191519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42484[] = {SGI191520, SGI191521, SGI191522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191523, SGI191523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42485[] = {SGI191524, SGI191525, SGI191526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42486[] = {SGI191527, SGI0, SGI191528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191529, SGI191530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42487[] = {SGI191531, SGI0, SGI191532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191533, SGI191534, SGI191534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42488[] = {SGI191535, SGI191536, SGI191537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191538, SGI0, SGI0, SGI0, SGI191539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42489[] = {SGI191540, SGI0, SGI191541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191542, SGI191543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42490[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191544, SGI0, SGI0, SGI191545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42491[] = {SGI191546, SGI191547, SGI191548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191549, SGI0, SGI0, SGI0, SGI191550, SGI191550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42492[] = {SGI191551, SGI191552, SGI191553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191554, SGI191554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42493[] = {SGI191555, SGI0, SGI191556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42494[] = {SGI191557, SGI191558, SGI191559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191560, SGI0, SGI0, SGI0, SGI191561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42495[] = {SGI191562, SGI0, SGI191563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42496[] = {SGI191564, SGI0, SGI191565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191566, SGI191567, SGI191567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42497[] = {SGI191568, SGI191569, SGI191570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191571, SGI0, SGI0, SGI0, SGI191572, SGI191572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42498[] = {SGI191573, SGI191574, SGI191575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191576, SGI0, SGI0, SGI0, SGI191577, SGI191577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42499[] = {SGI191578, SGI0, SGI191579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42500[] = {SGI191580, SGI0, SGI191581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191582, SGI0, SGI0, SGI0, SGI191583, SGI191583, SGI0, SGI0, SGI191584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42501[] = {SGI191585, SGI0, SGI191586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191587, SGI191588, SGI191588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42502[] = {SGI191589, SGI0, SGI191590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191591, SGI191592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42503[] = {SGI191593, SGI191594, SGI191595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191596, SGI0, SGI0, SGI0, SGI191597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42504[] = {SGI191598, SGI191599, SGI191600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191601, SGI0, SGI0, SGI0, SGI191602, SGI191602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42505[] = {SGI191603, SGI0, SGI191604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42506[] = {SGI191605, SGI0, SGI191606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191607, SGI191608, SGI191608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191609, SGI0, SGI0, NULL};
const Segment * const SG42507[] = {SGI191610, SGI191611, SGI191612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191613, SGI0, SGI0, SGI0, SGI191614, SGI191614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42508[] = {SGI191615, SGI0, SGI191616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42509[] = {SGI191617, SGI0, SGI191618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191619, SGI191620, SGI191620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42510[] = {SGI191621, SGI191622, SGI191623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191624, SGI0, SGI0, SGI0, SGI191625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42511[] = {SGI191626, SGI191627, SGI191628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191629, SGI0, SGI0, SGI0, SGI191630, SGI191630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42512[] = {SGI191631, SGI191632, SGI191633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191634, SGI0, SGI0, SGI0, SGI191635, SGI191635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42513[] = {SGI191636, SGI191637, SGI191638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191639, SGI0, SGI0, SGI0, SGI191640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42514[] = {SGI191641, SGI191642, SGI191643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191644, SGI0, SGI0, SGI0, SGI191645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42515[] = {SGI191646, SGI191647, SGI191648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191649, SGI0, SGI0, SGI0, SGI191650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42516[] = {SGI191652, SGI0, SGI191654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191657, SGI191659, SGI191659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42517[] = {SGI191653, SGI191655, SGI191656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191658, SGI0, SGI0, SGI0, SGI191660, SGI191660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42518[] = {SGI191662, SGI0, SGI191663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42519[] = {SGI191661, SGI191664, SGI191665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191666, SGI0, SGI0, SGI0, SGI191667, SGI191667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42520[] = {SGI191669, SGI191670, SGI191671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191672, SGI0, SGI0, SGI0, SGI191673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42521[] = {SGI191675, SGI191676, SGI191677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191678, SGI0, SGI0, SGI0, SGI191679, SGI191679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42522[] = {SGI191681, SGI191682, SGI191683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191684, SGI0, SGI0, SGI0, SGI191685, SGI191685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42523[] = {SGI191686, SGI191687, SGI191688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191689, SGI0, SGI0, SGI0, SGI191690, SGI191690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42524[] = {SGI191691, SGI191692, SGI191693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191694, SGI0, SGI0, SGI0, SGI191695, SGI191695, SGI0, SGI191696, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42525[] = {SGI191697, SGI0, SGI191698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191699, SGI191700, SGI191700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42526[] = {SGI191701, SGI191702, SGI191704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191708, SGI0, SGI0, SGI0, SGI191709, SGI191709, SGI0, SGI191712, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42527[] = {SGI191703, SGI0, SGI191705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191710, SGI191711, SGI191711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42528[] = {SGI191706, SGI0, SGI191707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191713, SGI191714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42529[] = {SGI191716, SGI0, SGI191719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191721, SGI191723, SGI191723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42530[] = {SGI191715, SGI191717, SGI191718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191720, SGI0, SGI0, SGI0, SGI191722, SGI191722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42531[] = {SGI191724, SGI0, SGI191725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191726, SGI191727, SGI191727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42532[] = {SGI191728, SGI0, SGI191729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191730, SGI191731, SGI191731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42533[] = {SGI191732, SGI0, SGI191733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191734, SGI191735, SGI191735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42534[] = {SGI191736, SGI0, SGI191737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191738, SGI191739, SGI191739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42535[] = {SGI191740, SGI0, SGI191741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191742, SGI191743, SGI191743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42536[] = {SGI191744, SGI0, SGI191745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191746, SGI191747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42537[] = {SGI191748, SGI191749, SGI191750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191751, SGI0, SGI0, SGI0, SGI191752, SGI191752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42538[] = {SGI191753, SGI191754, SGI191755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191756, SGI0, SGI0, SGI0, SGI191757, SGI191757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42539[] = {SGI191758, SGI191759, SGI191760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191761, SGI0, SGI0, SGI0, SGI191762, SGI191762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42540[] = {SGI191763, SGI0, SGI191764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42541[] = {SGI191765, SGI0, SGI191766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191767, SGI191768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42542[] = {SGI191769, SGI0, SGI191770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191771, SGI191772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42543[] = {SGI191773, SGI0, SGI191774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191778, SGI191780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42544[] = {SGI191775, SGI191776, SGI191777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191779, SGI0, SGI0, SGI0, SGI191781, SGI191781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42545[] = {SGI191782, SGI0, SGI191783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42546[] = {SGI191784, SGI0, SGI191785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191786, SGI191787, SGI191787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42547[] = {SGI191788, SGI0, SGI191789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191790, SGI191791, SGI191791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42548[] = {SGI191792, SGI0, SGI191793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191794, SGI191795, SGI191795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42549[] = {SGI191796, SGI0, SGI191797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191798, SGI191799, SGI191799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42550[] = {SGI191800, SGI191802, SGI191804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191805, SGI0, SGI0, SGI0, SGI191807, SGI191807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42551[] = {SGI191801, SGI0, SGI191803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191806, SGI191808, SGI191808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42552[] = {SGI191809, SGI0, SGI191810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42553[] = {SGI191811, SGI0, SGI191812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191813, SGI191814, SGI191814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42554[] = {SGI191815, SGI0, SGI191816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191817, SGI191818, SGI191818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42555[] = {SGI191819, SGI0, SGI191820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191822, SGI191824, SGI191824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42556[] = {SGI191821, SGI0, SGI191823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191825, SGI191826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42557[] = {SGI191827, SGI0, SGI191828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191829, SGI191830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42558[] = {SGI191831, SGI191832, SGI191833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191834, SGI0, SGI0, SGI0, SGI191835, SGI191835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42559[] = {SGI191837, SGI191838, SGI191839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191840, SGI0, SGI0, SGI0, SGI191841, SGI191841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42560[] = {SGI191843, SGI0, SGI191844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191848, SGI191849, SGI191849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42561[] = {SGI191845, SGI191846, SGI191847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191850, SGI0, SGI0, SGI0, SGI191851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42562[] = {SGI191852, SGI0, SGI191853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191854, SGI191855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42563[] = {SGI191856, SGI0, SGI191857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191858, SGI191859, SGI191859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42564[] = {SGI191860, SGI0, SGI191861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191862, SGI191863, SGI191863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42565[] = {SGI191864, SGI0, SGI191865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191866, SGI191867, SGI191867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42566[] = {SGI191868, SGI0, SGI191869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191871, SGI191873, SGI191873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42567[] = {SGI191870, SGI0, SGI191872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191874, SGI191875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42568[] = {SGI191876, SGI0, SGI191877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191878, SGI191879, SGI191879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42569[] = {SGI191880, SGI191881, SGI191882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191883, SGI191883, SGI0, SGI0, SGI0, SGI191884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42570[] = {SGI191885, SGI191886, SGI191887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191888, SGI191888, SGI0, SGI0, SGI0, SGI191889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42571[] = {SGI191890, SGI191891, SGI191892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191893, SGI0, SGI0, SGI0, SGI191894, SGI191894, SGI0, SGI191895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42572[] = {SGI191896, SGI0, SGI191897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191898, SGI0, SGI0, SGI0, SGI191899, SGI191899, SGI0, SGI0, SGI0, SGI191900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42573[] = {SGI191901, SGI191902, SGI191903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191904, SGI0, SGI0, SGI0, SGI191905, SGI191905, SGI0, SGI191906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42574[] = {SGI191907, SGI0, SGI191908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191909, SGI191910, SGI191910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42575[] = {SGI191911, SGI0, SGI191912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191913, SGI191914, SGI191914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42576[] = {SGI191915, SGI0, SGI191916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191917, SGI191918, SGI191918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42577[] = {SGI191919, SGI0, SGI191920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191921, SGI191922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42578[] = {SGI191923, SGI0, SGI191924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191925, SGI191926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42579[] = {SGI191927, SGI191928, SGI191929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191930, SGI0, SGI0, SGI0, SGI191931, SGI191931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42580[] = {SGI191932, SGI191933, SGI191934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191935, SGI0, SGI0, SGI0, SGI191936, SGI191936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42581[] = {SGI191937, SGI191938, SGI191939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191940, SGI0, SGI0, SGI0, SGI191941, SGI191941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42582[] = {SGI191942, SGI0, SGI191943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191944, SGI191945, SGI191945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42583[] = {SGI191946, SGI0, SGI191947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191948, SGI191949, SGI191949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42584[] = {SGI191950, SGI0, SGI191951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191952, SGI191953, SGI191953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42585[] = {SGI191954, SGI0, SGI191955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191956, SGI191957, SGI191957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42586[] = {SGI191958, SGI0, SGI191959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191961, SGI191962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42587[] = {SGI191960, SGI0, SGI191963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191964, SGI191965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42588[] = {SGI191966, SGI0, SGI191967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191968, SGI191969, SGI191969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42589[] = {SGI191970, SGI0, SGI191971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191972, SGI191973, SGI191973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42590[] = {SGI191974, SGI0, SGI191975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191976, SGI191977, SGI191977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42591[] = {SGI191978, SGI0, SGI191979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191980, SGI191981, SGI191981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42592[] = {SGI191982, SGI0, SGI191983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191984, SGI191986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42593[] = {SGI191985, SGI0, SGI191987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191988, SGI191989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42594[] = {SGI191990, SGI0, SGI191991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191994, SGI191995, SGI191995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42595[] = {SGI191992, SGI0, SGI191993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI191996, SGI191997, SGI191997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42596[] = {SGI191998, SGI191999, SGI192000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192001, SGI0, SGI0, SGI0, SGI192002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42597[] = {SGI192003, SGI192004, SGI192005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192006, SGI0, SGI0, SGI0, SGI192007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42598[] = {SGI192008, SGI0, SGI192009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192010, SGI192011, SGI192011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192012, SGI0, SGI0, NULL};
const Segment * const SG42599[] = {SGI192013, SGI0, SGI192014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192015, SGI192016, SGI192016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192017, SGI0, SGI0, NULL};
const Segment * const SG42600[] = {SGI192018, SGI0, SGI192019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192020, SGI192021, SGI192021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192022, SGI0, SGI0, NULL};
const Segment * const SG42601[] = {SGI192023, SGI0, SGI192024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192025, SGI192026, SGI192026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42602[] = {SGI192027, SGI192028, SGI192029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192030, SGI0, SGI0, SGI0, SGI192031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42603[] = {SGI192032, SGI0, SGI192033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192034, SGI192035, SGI192035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42604[] = {SGI192036, SGI192037, SGI192038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192039, SGI0, SGI0, SGI0, SGI192040, SGI192040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42605[] = {SGI192042, SGI0, SGI192044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42606[] = {SGI192041, SGI192043, SGI192045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192046, SGI0, SGI0, SGI0, SGI192047, SGI192047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42607[] = {SGI192048, SGI0, SGI192049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192050, SGI192051, SGI192051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42608[] = {SGI192052, SGI0, SGI192053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192054, SGI192056, SGI192056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42609[] = {SGI192055, SGI192057, SGI192058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192059, SGI0, SGI0, SGI0, SGI192060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42610[] = {SGI192062, SGI0, SGI192063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192064, SGI192065, SGI192065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42611[] = {SGI192066, SGI0, SGI192067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192068, SGI192069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42612[] = {SGI192070, SGI0, SGI192071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192074, SGI192076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42613[] = {SGI192072, SGI192073, SGI192075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192078, SGI0, SGI0, SGI0, SGI192080, SGI192080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192081, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42614[] = {SGI192077, SGI0, SGI192079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192082, SGI192083, SGI192083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42615[] = {SGI192084, SGI0, SGI192085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192086, SGI192087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42616[] = {SGI192088, SGI0, SGI192090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192092, SGI192093, SGI192093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42617[] = {SGI192089, SGI0, SGI192091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192094, SGI192095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42618[] = {SGI192096, SGI0, SGI192097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192099, SGI192101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42619[] = {SGI192098, SGI192100, SGI192102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192103, SGI0, SGI0, SGI0, SGI192104, SGI192104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42620[] = {SGI192105, SGI0, SGI192106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42621[] = {SGI192107, SGI0, SGI192108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192109, SGI192110, SGI192110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42622[] = {SGI192111, SGI192112, SGI192113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192114, SGI0, SGI0, SGI0, SGI192115, SGI192115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42623[] = {SGI192116, SGI192117, SGI192118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192119, SGI0, SGI0, SGI0, SGI192120, SGI192120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42624[] = {SGI192121, SGI0, SGI192122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42625[] = {SGI192123, SGI192124, SGI192125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192126, SGI0, SGI0, SGI0, SGI192127, SGI192127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42626[] = {SGI192128, SGI0, SGI192129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192130, SGI192131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42627[] = {SGI192132, SGI0, SGI192133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192134, SGI192135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42628[] = {SGI192136, SGI192137, SGI192138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192139, SGI0, SGI0, SGI0, SGI192140, SGI192140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42629[] = {SGI192141, SGI192142, SGI192143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192144, SGI0, SGI0, SGI0, SGI192145, SGI192145, SGI0, SGI192146, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42630[] = {SGI192147, SGI192148, SGI192149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192150, SGI0, SGI0, SGI0, SGI192151, SGI192151, SGI0, SGI192152, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42631[] = {SGI192153, SGI0, SGI192154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192155, SGI192156, SGI192156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42632[] = {SGI192157, SGI0, SGI192159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192161, SGI192162, SGI192162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42633[] = {SGI192158, SGI0, SGI192160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192163, SGI192164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42634[] = {SGI192165, SGI0, SGI192166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192167, SGI192168, SGI192168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42635[] = {SGI192169, SGI0, SGI192170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192171, SGI192172, SGI192172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42636[] = {SGI192173, SGI0, SGI192174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192175, SGI192176, SGI192176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192177, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42637[] = {SGI192178, SGI0, SGI192179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192180, SGI192181, SGI192181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42638[] = {SGI192182, SGI192183, SGI192184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192185, SGI0, SGI0, SGI0, SGI192186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42639[] = {SGI192187, SGI0, SGI192188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42640[] = {SGI192189, SGI0, SGI192190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192191, SGI192192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42641[] = {SGI192193, SGI192194, SGI192195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192196, SGI0, SGI0, SGI0, SGI192197, SGI192197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42642[] = {SGI192198, SGI0, SGI192199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192203, SGI192205, SGI192205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42643[] = {SGI192200, SGI192201, SGI192202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192204, SGI0, SGI0, SGI0, SGI192206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42644[] = {SGI192207, SGI0, SGI192208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192209, SGI192210, SGI192210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42645[] = {SGI192211, SGI0, SGI192212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192213, SGI192214, SGI192214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42646[] = {SGI192215, SGI192217, SGI192218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192220, SGI0, SGI0, SGI0, SGI192222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42647[] = {SGI192216, SGI0, SGI192219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192221, SGI192223, SGI192223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42648[] = {SGI192224, SGI0, SGI192225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42649[] = {SGI192226, SGI0, SGI192227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192228, SGI192229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42650[] = {SGI192230, SGI192231, SGI192232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192233, SGI0, SGI0, SGI0, SGI192234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42651[] = {SGI192235, SGI192236, SGI192237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192238, SGI0, SGI0, SGI0, SGI192239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42652[] = {SGI192240, SGI192241, SGI192242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192243, SGI0, SGI0, SGI0, SGI192244, SGI192244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42653[] = {SGI192245, SGI192246, SGI192247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192248, SGI0, SGI0, SGI0, SGI192249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42654[] = {SGI192250, SGI0, SGI192251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192252, SGI192253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42655[] = {SGI192254, SGI192255, SGI192256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192257, SGI0, SGI0, SGI0, SGI192258, SGI192258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42656[] = {SGI192259, SGI192260, SGI192261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192262, SGI192262, SGI0, SGI0, SGI0, SGI192263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42657[] = {SGI192264, SGI192265, SGI192266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192267, SGI0, SGI0, SGI0, SGI192268, SGI192268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42658[] = {SGI192269, SGI0, SGI192270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192271, SGI192272, SGI192272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42659[] = {SGI192273, SGI0, SGI192274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192275, SGI192276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42660[] = {SGI192277, SGI0, SGI192278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192279, SGI192280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42661[] = {SGI192281, SGI0, SGI192282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192283, SGI192284, SGI192284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42662[] = {SGI192285, SGI0, SGI192286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192287, SGI192288, SGI192288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42663[] = {SGI192289, SGI192290, SGI192291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192292, SGI0, SGI0, SGI0, SGI192293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42664[] = {SGI192294, SGI0, SGI192295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192296, SGI192297, SGI192297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42665[] = {SGI192298, SGI192299, SGI192300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192301, SGI0, SGI0, SGI0, SGI192302, SGI192302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42666[] = {SGI192303, SGI0, SGI192304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192308, SGI192310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42667[] = {SGI192305, SGI192306, SGI192307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192309, SGI0, SGI0, SGI0, SGI192311, SGI192311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42668[] = {SGI192312, SGI0, SGI192313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192314, SGI192315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42669[] = {SGI192316, SGI0, SGI192317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192318, SGI192319, SGI192319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42670[] = {SGI192320, SGI192321, SGI192322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192323, SGI0, SGI0, SGI192324, SGI192325, SGI192325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42671[] = {SGI192326, SGI0, SGI192327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192328, SGI192329, SGI192329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42672[] = {SGI192330, SGI0, SGI192331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192332, SGI192333, SGI192333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42673[] = {SGI192335, SGI0, SGI192337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192340, SGI192341, SGI192341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42674[] = {SGI192334, SGI192336, SGI192338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192339, SGI0, SGI0, SGI0, SGI192342, SGI192342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192343, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42675[] = {SGI192344, SGI0, SGI192345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192346, SGI192347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42676[] = {SGI192348, SGI192349, SGI192350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192351, SGI0, SGI0, SGI0, SGI192352, SGI192352, SGI0, SGI192354, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42677[] = {SGI192353, SGI0, SGI192355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192356, SGI192357, SGI192357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42678[] = {SGI192358, SGI0, SGI192359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192365, SGI192366, SGI192366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42679[] = {SGI192360, SGI192361, SGI192363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192367, SGI0, SGI0, SGI0, SGI192368, SGI192368, SGI0, SGI192371, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42680[] = {SGI192362, SGI0, SGI192364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192369, SGI192370, SGI192370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42681[] = {SGI192372, SGI0, SGI192373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192376, SGI192377, SGI192377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42682[] = {SGI192374, SGI0, SGI192375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192378, SGI192379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42683[] = {SGI192380, SGI0, SGI192381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192382, SGI192383, SGI192383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42684[] = {SGI192384, SGI192385, SGI192386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192387, SGI0, SGI0, SGI0, SGI192388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42685[] = {SGI192389, SGI192390, SGI192392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192393, SGI192393, SGI0, SGI0, SGI0, SGI192394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42686[] = {SGI192395, SGI0, SGI192396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192397, SGI192398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42687[] = {SGI192399, SGI192400, SGI192401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192402, SGI192402, SGI0, SGI0, SGI0, SGI0, SGI192403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42688[] = {SGI192404, SGI192405, SGI192406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192407, SGI192407, SGI0, SGI0, SGI0, SGI0, SGI192408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42689[] = {SGI192409, SGI192410, SGI192411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192412, SGI192412, SGI0, SGI0, SGI0, SGI0, SGI192413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42690[] = {SGI192414, SGI0, SGI192415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192416, SGI192417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42691[] = {SGI192418, SGI0, SGI192419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192420, SGI192421, SGI192421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42692[] = {SGI192422, SGI0, SGI192423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192424, SGI192425, SGI192425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42693[] = {SGI192426, SGI192427, SGI192428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192431, SGI0, SGI0, SGI0, SGI192432, SGI192432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42694[] = {SGI192429, SGI0, SGI192430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192433, SGI192434, SGI192434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42695[] = {SGI192435, SGI0, SGI192436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192437, SGI192438, SGI192438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42696[] = {SGI192439, SGI0, SGI192440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192444, SGI192445, SGI192445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42697[] = {SGI192441, SGI192442, SGI192443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192446, SGI0, SGI0, SGI0, SGI192447, SGI192447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42698[] = {SGI192448, SGI0, SGI192449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192453, SGI192454, SGI192454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42699[] = {SGI192450, SGI192451, SGI192452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192455, SGI0, SGI0, SGI0, SGI192456, SGI192456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42700[] = {SGI192457, SGI0, SGI192458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192459, SGI192461, SGI192461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42701[] = {SGI192460, SGI192462, SGI192463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192464, SGI0, SGI0, SGI0, SGI192465, SGI192465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42702[] = {SGI192466, SGI0, SGI192467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192468, SGI192469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42703[] = {SGI192470, SGI0, SGI192471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42704[] = {SGI192472, SGI192473, SGI192474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192475, SGI0, SGI0, SGI0, SGI192476, SGI192476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42705[] = {SGI192477, SGI0, SGI192478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192479, SGI192480, SGI192480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42706[] = {SGI192481, SGI0, SGI192482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192483, SGI192484, SGI192484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42707[] = {SGI192485, SGI0, SGI192486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192487, SGI192488, SGI192488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42708[] = {SGI192489, SGI0, SGI192490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192491, SGI192492, SGI192492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42709[] = {SGI192493, SGI0, SGI192494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192495, SGI192496, SGI192496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42710[] = {SGI192497, SGI0, SGI192499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42711[] = {SGI192498, SGI0, SGI192500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192501, SGI192502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42712[] = {SGI192503, SGI192504, SGI192505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192506, SGI0, SGI0, SGI0, SGI192507, SGI192507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42713[] = {SGI192508, SGI192509, SGI192510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192511, SGI0, SGI0, SGI0, SGI192512, SGI192512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42714[] = {SGI192513, SGI192514, SGI192515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192516, SGI0, SGI0, SGI0, SGI192517, SGI192517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42715[] = {SGI192518, SGI192519, SGI192520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192521, SGI0, SGI0, SGI0, SGI192522, SGI192522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42716[] = {SGI0, SGI192523, SGI192524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192525, SGI0, SGI0, SGI0, SGI192526, SGI192526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42717[] = {SGI0, SGI192527, SGI192528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192529, SGI0, SGI0, SGI0, SGI192530, SGI192530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42718[] = {SGI192531, SGI0, SGI192532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192535, SGI192537, SGI192537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42719[] = {SGI192533, SGI192534, SGI192536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192538, SGI0, SGI0, SGI0, SGI192539, SGI192539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42720[] = {SGI192540, SGI192541, SGI192542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192543, SGI0, SGI0, SGI0, SGI192544, SGI192544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42721[] = {SGI192545, SGI0, SGI192546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42722[] = {SGI192547, SGI192548, SGI192549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192550, SGI0, SGI0, SGI0, SGI192551, SGI192551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42723[] = {SGI192552, SGI192553, SGI192554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192555, SGI0, SGI0, SGI0, SGI192556, SGI192556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42724[] = {SGI192557, SGI192558, SGI192559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192560, SGI0, SGI0, SGI0, SGI192561, SGI192561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42725[] = {SGI192562, SGI0, SGI192563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192564, SGI192565, SGI192565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42726[] = {SGI192566, SGI0, SGI192567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192568, SGI192569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42727[] = {SGI192570, SGI192571, SGI192572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192573, SGI192573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192574, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42728[] = {SGI192575, SGI0, SGI192576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192577, SGI192578, SGI192578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42729[] = {SGI192579, SGI0, SGI192580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192581, SGI192582, SGI192582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42730[] = {SGI192583, SGI0, SGI192584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192585, SGI192586, SGI192586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42731[] = {SGI192587, SGI0, SGI192588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192589, SGI192590, SGI192590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42732[] = {SGI192591, SGI0, SGI192592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192593, SGI192594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42733[] = {SGI192595, SGI0, SGI192596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192597, SGI192598, SGI192598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42734[] = {SGI192599, SGI192600, SGI192601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192602, SGI0, SGI0, SGI0, SGI192603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42735[] = {SGI192604, SGI0, SGI192605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192606, SGI0, SGI0, SGI0, SGI192607, SGI192607, SGI0, SGI0, SGI0, SGI0, SGI192608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42736[] = {SGI192609, SGI192610, SGI192611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192612, SGI0, SGI0, SGI0, SGI192613, SGI192614, SGI0, SGI0, SGI0, SGI0, SGI192615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42737[] = {SGI192616, SGI192617, SGI192618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192619, SGI0, SGI0, SGI0, SGI192620, SGI192620, SGI0, SGI0, NULL};
const Segment * const SG42738[] = {SGI192621, SGI0, SGI192622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192623, SGI192624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192625, SGI0, SGI0, NULL};
const Segment * const SG42739[] = {SGI192626, SGI0, SGI192627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192628, SGI192629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192630, SGI0, SGI0, NULL};
const Segment * const SG42740[] = {SGI192631, SGI192632, SGI192633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192634, SGI0, SGI0, SGI0, SGI192635, SGI192635, SGI0, SGI0, NULL};
const Segment * const SG42741[] = {SGI192636, SGI0, SGI192637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192638, SGI192639, SGI192639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42742[] = {SGI192640, SGI192641, SGI192642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192643, SGI0, SGI0, SGI0, SGI192644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42743[] = {SGI192645, SGI0, SGI192646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192648, SGI192649, SGI192649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42744[] = {SGI192647, SGI0, SGI192650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192651, SGI192652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42745[] = {SGI192653, SGI0, SGI192654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192655, SGI192656, SGI192656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42746[] = {SGI192657, SGI0, SGI192658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192659, SGI192660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42747[] = {SGI192661, SGI0, SGI192662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192663, SGI192664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42748[] = {SGI192665, SGI192666, SGI192667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192668, SGI0, SGI0, SGI0, SGI192670, SGI192670, SGI0, SGI192672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42749[] = {SGI192669, SGI0, SGI192671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192673, SGI192674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42750[] = {SGI192675, SGI0, SGI192676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192677, SGI192678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42751[] = {SGI192679, SGI192680, SGI192681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192682, SGI0, SGI0, SGI0, SGI192683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42752[] = {SGI192684, SGI0, SGI192685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192686, SGI192687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42753[] = {SGI192688, SGI0, SGI192689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192690, SGI192691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42754[] = {SGI192692, SGI192693, SGI192694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192695, SGI0, SGI0, SGI0, SGI192696, SGI192696, SGI0, SGI192697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42755[] = {SGI192698, SGI0, SGI192699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192700, SGI192701, SGI192701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42756[] = {SGI192702, SGI0, SGI192703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192704, SGI192705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42757[] = {SGI192706, SGI0, SGI192707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192708, SGI192709, SGI192709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42758[] = {SGI192710, SGI0, SGI192711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192712, SGI192713, SGI192713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42759[] = {SGI192714, SGI0, SGI192715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192716, SGI192717, SGI192717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42760[] = {SGI192718, SGI0, SGI192719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192720, SGI192721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42761[] = {SGI192722, SGI0, SGI192724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192726, SGI192727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42762[] = {SGI192723, SGI0, SGI192725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192728, SGI192729, SGI192729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42763[] = {SGI192730, SGI0, SGI192731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192734, SGI192735, SGI192735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42764[] = {SGI192732, SGI0, SGI192733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192736, SGI192737, SGI192737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42765[] = {SGI192738, SGI0, SGI192739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192740, SGI192741, SGI192741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42766[] = {SGI192742, SGI0, SGI192743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192744, SGI192745, SGI192745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42767[] = {SGI192746, SGI0, SGI192747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192748, SGI192749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42768[] = {SGI192750, SGI0, SGI192751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192752, SGI192753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42769[] = {SGI192754, SGI0, SGI192755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192756, SGI192757, SGI192757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42770[] = {SGI192758, SGI0, SGI192759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192760, SGI192761, SGI192761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42771[] = {SGI192762, SGI0, SGI192763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192765, SGI192766, SGI192766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42772[] = {SGI192764, SGI0, SGI192767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192768, SGI192769, SGI192769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42773[] = {SGI192770, SGI0, SGI192771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192773, SGI192775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42774[] = {SGI192772, SGI0, SGI192774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192776, SGI192777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42775[] = {SGI192778, SGI0, SGI192779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192780, SGI192781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42776[] = {SGI192782, SGI192783, SGI192784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192785, SGI0, SGI0, SGI0, SGI192786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42777[] = {SGI192787, SGI0, SGI192788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42778[] = {SGI192789, SGI192790, SGI192791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192792, SGI0, SGI0, SGI0, SGI192793, SGI192793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42779[] = {SGI192794, SGI192795, SGI192796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192797, SGI0, SGI0, SGI0, SGI192798, SGI192798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42780[] = {SGI192799, SGI192800, SGI192801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192803, SGI0, SGI0, SGI0, SGI192805, SGI192805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42781[] = {SGI192802, SGI0, SGI192804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192806, SGI192807, SGI192807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42782[] = {SGI192808, SGI192809, SGI192810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192811, SGI0, SGI0, SGI0, SGI192812, SGI192812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42783[] = {SGI192813, SGI192814, SGI192815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192816, SGI0, SGI0, SGI0, SGI192817, SGI192817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42784[] = {SGI192818, SGI0, SGI192821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192824, SGI192825, SGI192825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42785[] = {SGI192819, SGI192820, SGI192822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192823, SGI0, SGI0, SGI0, SGI192826, SGI0, SGI0, SGI192827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42786[] = {SGI192828, SGI192829, SGI192830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192833, SGI0, SGI0, SGI0, SGI192834, SGI0, SGI0, SGI192836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42787[] = {SGI192831, SGI0, SGI192832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192835, SGI192837, SGI192837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42788[] = {SGI192838, SGI192839, SGI192840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192841, SGI0, SGI0, SGI0, SGI192842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42789[] = {SGI192843, SGI192844, SGI192845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192846, SGI0, SGI0, SGI0, SGI192847, SGI192847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42790[] = {SGI192848, SGI0, SGI192849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42791[] = {SGI192850, SGI0, SGI192851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42792[] = {SGI192852, SGI192853, SGI192854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192855, SGI0, SGI0, SGI0, SGI192856, SGI192856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42793[] = {SGI192857, SGI0, SGI192858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192859, SGI192860, SGI192860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42794[] = {SGI192861, SGI0, SGI192863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192865, SGI192866, SGI192866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42795[] = {SGI192862, SGI0, SGI192864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192867, SGI192868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42796[] = {SGI192869, SGI0, SGI192870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192871, SGI192872, SGI192872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42797[] = {SGI192873, SGI0, SGI192874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192875, SGI192876, SGI192876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42798[] = {SGI192877, SGI0, SGI192878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192879, SGI192880, SGI192880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42799[] = {SGI192881, SGI0, SGI192882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192883, SGI192884, SGI192884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42800[] = {SGI192885, SGI0, SGI192886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192887, SGI192888, SGI192888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42801[] = {SGI192889, SGI0, SGI192890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192891, SGI192892, SGI192892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42802[] = {SGI192893, SGI0, SGI192894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192895, SGI192896, SGI192896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42803[] = {SGI192897, SGI192898, SGI192899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192900, SGI0, SGI0, SGI0, SGI192901, SGI192901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42804[] = {SGI192902, SGI0, SGI192903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192904, SGI192905, SGI192905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42805[] = {SGI192906, SGI192907, SGI192908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192909, SGI0, SGI0, SGI0, SGI192910, SGI192910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42806[] = {SGI192911, SGI192912, SGI192913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192914, SGI192914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42807[] = {SGI192915, SGI192916, SGI192917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192918, SGI0, SGI0, SGI0, SGI192919, SGI192919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42808[] = {SGI192920, SGI0, SGI192921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192925, SGI192926, SGI192926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42809[] = {SGI192922, SGI192923, SGI192924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192927, SGI192927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42810[] = {SGI192928, SGI0, SGI192929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42811[] = {SGI192930, SGI192931, SGI192932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192936, SGI0, SGI0, SGI0, SGI192939, SGI192939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42812[] = {SGI192933, SGI192934, SGI192935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192938, SGI0, SGI0, SGI0, SGI192942, SGI192942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42813[] = {SGI192937, SGI192940, SGI192941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192943, SGI0, SGI0, SGI0, SGI192944, SGI192944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42814[] = {SGI192945, SGI0, SGI192946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42815[] = {SGI192947, SGI192948, SGI192949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192950, SGI0, SGI0, SGI0, SGI192951, SGI192951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42816[] = {SGI192952, SGI192953, SGI192954, SGI192955, SGI192956, SGI192956, SGI0, NULL};
const Segment * const SG42817[] = {SGI192957, SGI0, SGI192958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192959, SGI192960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42818[] = {SGI192961, SGI192962, SGI192963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192964, SGI0, SGI0, SGI0, SGI192965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42819[] = {SGI192966, SGI192967, SGI192968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192969, SGI0, SGI0, SGI0, SGI192970, SGI192970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42820[] = {SGI192971, SGI0, SGI192972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192973, SGI192974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42821[] = {SGI192975, SGI0, SGI192976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192977, SGI192978, SGI192978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192979, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42822[] = {SGI192980, SGI0, SGI192981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192982, SGI192983, SGI192983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42823[] = {SGI192984, SGI0, SGI192985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192986, SGI192987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42824[] = {SGI192988, SGI0, SGI192989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192990, SGI192991, SGI192991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42825[] = {SGI192992, SGI0, SGI192993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192994, SGI192995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42826[] = {SGI192996, SGI192997, SGI192998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI192999, SGI0, SGI0, SGI0, SGI193000, SGI193000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42827[] = {SGI193001, SGI193002, SGI193003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193007, SGI0, SGI0, SGI0, SGI193008, SGI193008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42828[] = {SGI193004, SGI193005, SGI193006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193009, SGI0, SGI0, SGI0, SGI193010, SGI193010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42829[] = {SGI193011, SGI193012, SGI193013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193014, SGI0, SGI0, SGI0, SGI193015, SGI193015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42830[] = {SGI193016, SGI0, SGI193018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193025, SGI193026, SGI193026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42831[] = {SGI193017, SGI193019, SGI193020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193024, SGI0, SGI0, SGI0, SGI193027, SGI193027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42832[] = {SGI193021, SGI193022, SGI193023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193028, SGI0, SGI0, SGI0, SGI193029, SGI193029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42833[] = {SGI193031, SGI193032, SGI193033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193034, SGI0, SGI0, SGI0, SGI193035, SGI193035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42834[] = {SGI193036, SGI0, SGI193037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42835[] = {SGI193039, SGI0, SGI193042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193043, SGI0, SGI0, SGI0, SGI193045, SGI193045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42836[] = {SGI193038, SGI193040, SGI193041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193044, SGI0, SGI0, SGI0, SGI193046, SGI193046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193047, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42837[] = {SGI193048, SGI193049, SGI193050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193051, SGI0, SGI0, SGI0, SGI193052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42838[] = {SGI193053, SGI193054, SGI193055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42839[] = {SGI193056, SGI193057, SGI193058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193059, SGI0, SGI0, SGI0, SGI193060, SGI193060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42840[] = {SGI193061, SGI193062, SGI193063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193064, SGI193064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42841[] = {SGI193065, SGI0, SGI193066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42842[] = {SGI193067, SGI0, SGI193069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193073, SGI193074, SGI193074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42843[] = {SGI193068, SGI193070, SGI193071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193072, SGI0, SGI0, SGI0, SGI193075, SGI193075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42844[] = {SGI193076, SGI0, SGI193077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193078, SGI193079, SGI193079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42845[] = {SGI193080, SGI0, SGI193081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193082, SGI193083, SGI193083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42846[] = {SGI193084, SGI0, SGI193085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193086, SGI193087, SGI193087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42847[] = {SGI193088, SGI193089, SGI193090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193091, SGI193091, SGI0, SGI0, SGI0, SGI193092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42848[] = {SGI193093, SGI0, SGI193094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193095, SGI193096, SGI193096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42849[] = {SGI193097, SGI0, SGI193098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193099, SGI193100, SGI193100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42850[] = {SGI193101, SGI0, SGI193102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193103, SGI193104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42851[] = {SGI193105, SGI193106, SGI193107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193108, SGI0, SGI0, SGI0, SGI193109, SGI193109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42852[] = {SGI193110, SGI0, SGI193111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193112, SGI193113, SGI193113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42853[] = {SGI193114, SGI193115, SGI193116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193117, SGI0, SGI0, SGI0, SGI193118, SGI193118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42854[] = {SGI193120, SGI193121, SGI193122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193126, SGI0, SGI0, SGI0, SGI193127, SGI193127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42855[] = {SGI193123, SGI193124, SGI193125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193128, SGI0, SGI0, SGI0, SGI193129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42856[] = {SGI193131, SGI193132, SGI193133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193134, SGI0, SGI0, SGI0, SGI193135, SGI193135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42857[] = {SGI193137, SGI0, SGI193140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42858[] = {SGI193136, SGI0, SGI193139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193144, SGI193145, SGI193145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42859[] = {SGI193138, SGI193141, SGI193142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193143, SGI0, SGI0, SGI0, SGI193146, SGI193146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42860[] = {SGI193147, SGI193148, SGI193150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193151, SGI0, SGI0, SGI0, SGI193152, SGI193152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42861[] = {SGI193153, SGI193154, SGI193155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193157, SGI0, SGI0, SGI0, SGI193158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42862[] = {SGI0, SGI0, SGI193156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193160, SGI193161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42863[] = {SGI193162, SGI0, SGI193163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193166, SGI193168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42864[] = {SGI193164, SGI193165, SGI193167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193169, SGI0, SGI0, SGI0, SGI193170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42865[] = {SGI193171, SGI193172, SGI193173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193174, SGI0, SGI0, SGI0, SGI193175, SGI193175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42866[] = {SGI193176, SGI193177, SGI193178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193179, SGI0, SGI0, SGI0, SGI193180, SGI193180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42867[] = {SGI0, SGI193181, SGI193182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193183, SGI0, SGI0, SGI0, SGI193184, SGI193184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42868[] = {SGI193185, SGI193186, SGI193187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193188, SGI0, SGI0, SGI0, SGI193189, SGI193189, SGI0, SGI193190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42869[] = {SGI193191, SGI193192, SGI193193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193194, SGI0, SGI0, SGI0, SGI193195, SGI193195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42870[] = {SGI193196, SGI0, SGI193197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42871[] = {SGI193198, SGI193199, SGI193200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193201, SGI0, SGI0, SGI0, SGI193202, SGI193202, SGI0, SGI193203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42872[] = {SGI193204, SGI193205, SGI193206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193207, SGI0, SGI0, SGI0, SGI193208, SGI0, SGI0, SGI193209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42873[] = {SGI193210, SGI193211, SGI193212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193213, SGI0, SGI0, SGI0, SGI193214, SGI193214, SGI0, SGI193215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42874[] = {SGI193216, SGI193217, SGI193218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193220, SGI0, SGI0, SGI0, SGI193223, SGI193223, SGI0, SGI193226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42875[] = {SGI193219, SGI0, SGI193221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193227, SGI193228, SGI193228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42876[] = {SGI193222, SGI193224, SGI193225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193229, SGI193229, SGI0, SGI0, SGI0, SGI193230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42877[] = {SGI193231, SGI193232, SGI193233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193234, SGI0, SGI0, SGI0, SGI193235, SGI193235, SGI0, SGI193236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42878[] = {SGI193237, SGI193238, SGI193239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193240, SGI0, SGI0, SGI0, SGI193241, SGI193241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193242, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42879[] = {SGI193243, SGI193244, SGI193245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193246, SGI0, SGI0, SGI0, SGI193247, SGI193247, SGI0, SGI193248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42880[] = {SGI193249, SGI193250, SGI193251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193254, SGI0, SGI0, SGI0, SGI193255, SGI193255, SGI0, SGI193257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42881[] = {SGI193252, SGI0, SGI193253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193256, SGI193258, SGI193258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42882[] = {SGI193259, SGI193260, SGI193261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193262, SGI0, SGI0, SGI0, SGI193263, SGI193263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42883[] = {SGI193264, SGI0, SGI193265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42884[] = {SGI193266, SGI0, SGI193267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42885[] = {SGI193268, SGI193269, SGI193270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193271, SGI0, SGI0, SGI0, SGI193272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42886[] = {SGI193273, SGI193274, SGI193275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193276, SGI0, SGI0, SGI0, SGI193277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42887[] = {SGI193278, SGI193279, SGI193280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193281, SGI0, SGI0, SGI0, SGI193282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42888[] = {SGI193283, SGI0, SGI193284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193285, SGI193286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42889[] = {SGI193287, SGI0, SGI193288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193289, SGI193290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193291, SGI0, SGI0, NULL};
const Segment * const SG42890[] = {SGI193292, SGI0, SGI193293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193294, SGI193295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193296, SGI0, SGI0, NULL};
const Segment * const SG42891[] = {SGI193297, SGI193298, SGI193300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193302, SGI0, SGI0, SGI0, SGI193303, SGI193303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193307, SGI0, NULL};
const Segment * const SG42892[] = {SGI193299, SGI0, SGI193301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193304, SGI193305, SGI193305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42893[] = {SGI193309, SGI0, SGI193312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193314, SGI193316, SGI193316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42894[] = {SGI193308, SGI193310, SGI193311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193313, SGI0, SGI0, SGI0, SGI193315, SGI193315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193318, SGI0, NULL};
const Segment * const SG42895[] = {SGI193319, SGI193320, SGI193321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193322, SGI0, SGI0, SGI0, SGI193323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42896[] = {SGI193324, SGI0, SGI193325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193326, SGI193327, SGI193327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42897[] = {SGI193328, SGI0, SGI193329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193330, SGI193331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42898[] = {SGI193332, SGI0, SGI193333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193334, SGI193335, SGI193335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42899[] = {SGI193336, SGI0, SGI193337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193338, SGI193339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42900[] = {SGI193340, SGI193341, SGI193342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193343, SGI0, SGI0, SGI0, SGI193344, SGI193344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42901[] = {SGI193345, SGI0, SGI193346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42902[] = {SGI193347, SGI193348, SGI193349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193350, SGI0, SGI0, SGI0, SGI193351, SGI193351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42903[] = {SGI193352, SGI193353, SGI193354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193355, SGI0, SGI0, SGI0, SGI193356, SGI193356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42904[] = {SGI193357, SGI193358, SGI193359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193360, SGI0, SGI0, SGI0, SGI193361, SGI193361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42905[] = {SGI193362, SGI0, SGI193363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193364, SGI193365, SGI193365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42906[] = {SGI193366, SGI0, SGI193367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193368, SGI193369, SGI193369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42907[] = {SGI193370, SGI0, SGI193371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193372, SGI193373, SGI193373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42908[] = {SGI193374, SGI0, SGI193375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193376, SGI193377, SGI193377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42909[] = {SGI193378, SGI0, SGI193379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193380, SGI193381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42910[] = {SGI193382, SGI0, SGI193383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193385, SGI193387, SGI193387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42911[] = {SGI193384, SGI193386, SGI193388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193389, SGI0, SGI0, SGI0, SGI193390, SGI193390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42912[] = {SGI193391, SGI0, SGI193392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193393, SGI193394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193395, SGI0, SGI0, NULL};
const Segment * const SG42913[] = {SGI193396, SGI0, SGI193397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193401, SGI193402, SGI193402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42914[] = {SGI193398, SGI193399, SGI193400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193403, SGI0, SGI0, SGI0, SGI193404, SGI193404, SGI0, SGI193405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42915[] = {SGI193406, SGI0, SGI193407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193408, SGI193409, SGI193409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193410, SGI0, SGI0, NULL};
const Segment * const SG42916[] = {SGI193411, SGI0, SGI193412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193413, SGI193414, SGI193414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193415, SGI0, SGI0, NULL};
const Segment * const SG42917[] = {SGI193416, SGI193417, SGI193418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193419, SGI0, SGI0, SGI0, SGI193420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42918[] = {SGI193421, SGI0, SGI193422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193423, SGI193424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42919[] = {SGI193425, SGI0, SGI193426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193427, SGI193428, SGI193428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42920[] = {SGI193429, SGI0, SGI193430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193431, SGI193432, SGI193432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42921[] = {SGI193433, SGI193434, SGI193435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193436, SGI0, SGI0, SGI0, SGI193437, SGI193437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42922[] = {SGI193438, SGI0, SGI193439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193440, SGI193441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42923[] = {SGI193442, SGI0, SGI193443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193444, SGI193445, SGI193445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42924[] = {SGI193446, SGI0, SGI193447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193448, SGI193449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42925[] = {SGI193450, SGI0, SGI193451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193452, SGI193453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42926[] = {SGI193454, SGI193455, SGI193456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193457, SGI0, SGI0, SGI0, SGI193458, SGI193458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42927[] = {SGI193459, SGI193460, SGI193461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193462, SGI0, SGI0, SGI0, SGI193463, SGI193463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42928[] = {SGI193464, SGI0, SGI193465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193466, SGI193467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42929[] = {SGI193468, SGI0, SGI193469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193472, SGI193473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42930[] = {SGI193470, SGI0, SGI193471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193474, SGI193475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42931[] = {SGI193476, SGI0, SGI193477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193478, SGI193479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42932[] = {SGI193480, SGI193481, SGI193482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193483, SGI193483, SGI0, SGI0, SGI0, SGI193484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42933[] = {SGI193485, SGI193486, SGI193487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193488, SGI0, SGI0, SGI0, SGI193489, SGI193489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42934[] = {SGI193490, SGI0, SGI193491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42935[] = {SGI193492, SGI193493, SGI193494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193495, SGI0, SGI0, SGI0, SGI193496, SGI193496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42936[] = {SGI193497, SGI193498, SGI193499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193500, SGI0, SGI0, SGI0, SGI193501, SGI193501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42937[] = {SGI193502, SGI193503, SGI193504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193505, SGI0, SGI0, SGI0, SGI193506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42938[] = {SGI193507, SGI193508, SGI193509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193510, SGI0, SGI0, SGI0, SGI193511, SGI193511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42939[] = {SGI193512, SGI0, SGI193513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193514, SGI193515, SGI193515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42940[] = {SGI193516, SGI0, SGI193517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193518, SGI193519, SGI193519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42941[] = {SGI193520, SGI193521, SGI193523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193525, SGI0, SGI0, SGI0, SGI193526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42942[] = {SGI193522, SGI0, SGI193524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193527, SGI193528, SGI193528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42943[] = {SGI193529, SGI0, SGI193530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193531, SGI193532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42944[] = {SGI193533, SGI0, SGI193534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193535, SGI193536, SGI193536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193537, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42945[] = {SGI193538, SGI0, SGI193539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193540, SGI193541, SGI193541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42946[] = {SGI193542, SGI193543, SGI193544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193545, SGI0, SGI0, SGI0, SGI193546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42947[] = {SGI193547, SGI193548, SGI193549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193550, SGI0, SGI0, SGI0, SGI193551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42948[] = {SGI193552, SGI193553, SGI193554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193555, SGI0, SGI0, SGI0, SGI193556, SGI193556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42949[] = {SGI193557, SGI0, SGI193558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42950[] = {SGI193559, SGI0, SGI193560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193561, SGI193562, SGI193562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42951[] = {SGI193563, SGI0, SGI193564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193565, SGI193566, SGI193566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42952[] = {SGI193567, SGI0, SGI193568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193569, SGI193570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42953[] = {SGI193571, SGI0, SGI193572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193573, SGI193574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42954[] = {SGI193575, SGI193576, SGI193577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193578, SGI0, SGI0, SGI0, SGI193579, SGI0, SGI193580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42955[] = {SGI193581, SGI193582, SGI193583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193584, SGI0, SGI0, SGI0, SGI193585, SGI193585, SGI193586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42956[] = {SGI193587, SGI0, SGI193588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193589, SGI193590, SGI193590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42957[] = {SGI193591, SGI0, SGI193592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193593, SGI193594, SGI193594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42958[] = {SGI193595, SGI193596, SGI193597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193598, SGI0, SGI0, SGI0, SGI193599, SGI193599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42959[] = {SGI193600, SGI0, SGI193601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42960[] = {SGI193602, SGI0, SGI193603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193604, SGI193605, SGI193605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42961[] = {SGI193606, SGI193607, SGI193608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193609, SGI0, SGI0, SGI0, SGI193610, SGI193610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42962[] = {SGI193611, SGI193612, SGI193613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193614, SGI0, SGI0, SGI0, SGI193615, SGI193615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42963[] = {SGI193616, SGI0, SGI193617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193618, SGI193619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42964[] = {SGI193620, SGI193621, SGI193622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193625, SGI0, SGI0, SGI0, SGI193627, SGI193627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42965[] = {SGI193623, SGI193624, SGI193626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193628, SGI0, SGI0, SGI0, SGI193629, SGI0, SGI193630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42966[] = {SGI193631, SGI193632, SGI193633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193634, SGI0, SGI0, SGI0, SGI193635, SGI193635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42967[] = {SGI193636, SGI0, SGI193637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193638, SGI193639, SGI193639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42968[] = {SGI193640, SGI0, SGI193641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42969[] = {SGI193642, SGI193643, SGI193645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193647, SGI0, SGI0, SGI0, SGI193648, SGI193648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42970[] = {SGI193644, SGI0, SGI193646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193649, SGI193650, SGI193650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42971[] = {SGI193651, SGI193652, SGI193653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193656, SGI0, SGI0, SGI0, SGI193657, SGI193657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42972[] = {SGI193654, SGI0, SGI193655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193658, SGI193659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42973[] = {SGI193660, SGI193661, SGI193662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193663, SGI0, SGI0, SGI0, SGI193665, SGI193665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42974[] = {SGI193664, SGI0, SGI193666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193667, SGI193668, SGI193668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42975[] = {SGI193669, SGI0, SGI193670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193671, SGI193672, SGI193672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42976[] = {SGI193673, SGI0, SGI193674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193675, SGI193676, SGI193676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42977[] = {SGI193677, SGI0, SGI193678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193679, SGI193680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42978[] = {SGI193681, SGI193682, SGI193683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193684, SGI0, SGI0, SGI0, SGI193685, SGI193685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42979[] = {SGI193687, SGI193688, SGI193689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193692, SGI0, SGI0, SGI0, SGI193695, SGI193695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42980[] = {SGI193691, SGI193693, SGI193696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193699, SGI0, SGI0, SGI0, SGI193701, SGI193701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42981[] = {SGI193690, SGI193694, SGI193697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193700, SGI0, SGI0, SGI0, SGI193702, SGI193702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42982[] = {SGI193703, SGI193704, SGI193705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193706, SGI0, SGI0, SGI0, SGI193707, SGI193707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42983[] = {SGI193708, SGI0, SGI193709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193710, SGI193711, SGI193711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42984[] = {SGI193712, SGI193713, SGI193714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193715, SGI0, SGI0, SGI0, SGI193716, SGI193716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42985[] = {SGI193717, SGI193718, SGI193719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193720, SGI0, SGI0, SGI0, SGI193721, SGI193721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42986[] = {SGI193722, SGI193723, SGI193724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193726, SGI0, SGI0, SGI0, SGI193728, SGI193728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42987[] = {SGI193725, SGI0, SGI193727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42988[] = {SGI193729, SGI0, SGI193730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193731, SGI193732, SGI193732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42989[] = {SGI193733, SGI193734, SGI193735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193736, SGI0, SGI0, SGI0, SGI193737, SGI193737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42990[] = {SGI193738, SGI0, SGI193739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193740, SGI193741, SGI193741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42991[] = {SGI193742, SGI0, SGI193743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42992[] = {SGI193744, SGI0, SGI193745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193746, SGI193747, SGI193747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42993[] = {SGI193748, SGI0, SGI193749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193750, SGI193751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42994[] = {SGI193752, SGI0, SGI193753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193754, SGI193755, SGI193755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42995[] = {SGI193756, SGI0, SGI193757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193758, SGI193759, SGI193759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42996[] = {SGI193760, SGI193761, SGI193762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193763, SGI0, SGI0, SGI0, SGI193764, SGI193764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42997[] = {SGI193766, SGI0, SGI193767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42998[] = {SGI193765, SGI0, SGI193768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193770, SGI193771, SGI193771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG42999[] = {SGI0, SGI0, SGI193769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193772, SGI193773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43000[] = {SGI193774, SGI193775, SGI193776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193777, SGI0, SGI0, SGI0, SGI193778, SGI193778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43001[] = {SGI193779, SGI193780, SGI193781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193782, SGI0, SGI0, SGI0, SGI193783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43002[] = {SGI193784, SGI193785, SGI193786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193787, SGI0, SGI0, SGI0, SGI193788, SGI193788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43003[] = {SGI193791, SGI0, SGI193792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43004[] = {SGI193789, SGI0, SGI193790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193793, SGI193794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43005[] = {SGI193795, SGI0, SGI193796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193797, SGI193798, SGI193798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43006[] = {SGI193799, SGI0, SGI193800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193803, SGI193804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43007[] = {SGI193801, SGI0, SGI193802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193805, SGI193806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43008[] = {SGI193807, SGI193808, SGI193809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193810, SGI0, SGI0, SGI0, SGI193811, SGI193811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43009[] = {SGI193812, SGI193813, SGI193814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193815, SGI0, SGI0, SGI0, SGI193818, SGI193818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43010[] = {SGI193816, SGI193817, SGI193819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193820, SGI0, SGI0, SGI0, SGI193821, SGI193821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43011[] = {SGI193826, SGI0, SGI193828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43012[] = {SGI193822, SGI193823, SGI193824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193830, SGI0, SGI0, SGI0, SGI193831, SGI193831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43013[] = {SGI193825, SGI193827, SGI193829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193832, SGI193832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193833, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43014[] = {SGI193834, SGI193835, SGI193836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193837, SGI0, SGI0, SGI0, SGI193840, SGI193840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43015[] = {SGI193838, SGI193839, SGI193841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193842, SGI193842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193843, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43016[] = {SGI193844, SGI0, SGI193845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193847, SGI193849, SGI193849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43017[] = {SGI193846, SGI193848, SGI193850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193851, SGI193851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193852, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43018[] = {SGI193853, SGI0, SGI193854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193856, SGI193858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43019[] = {SGI193855, SGI193857, SGI193859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193863, SGI193863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193866, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43020[] = {SGI193860, SGI193861, SGI193862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193864, SGI0, SGI0, SGI0, SGI193865, SGI193865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43021[] = {SGI193867, SGI193868, SGI193869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193873, SGI193873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193876, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43022[] = {SGI193870, SGI193871, SGI193872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193874, SGI0, SGI0, SGI0, SGI193875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43023[] = {SGI193877, SGI193878, SGI193879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193880, SGI0, SGI0, SGI0, SGI193881, SGI193881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43024[] = {SGI193882, SGI0, SGI193883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43025[] = {SGI193884, SGI193885, SGI193886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193887, SGI0, SGI0, SGI0, SGI193888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43026[] = {SGI193889, SGI193890, SGI193891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193892, SGI0, SGI0, SGI0, SGI193893, SGI193893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43027[] = {SGI193894, SGI193895, SGI193896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193897, SGI0, SGI0, SGI0, SGI193898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43028[] = {SGI193899, SGI193900, SGI193901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193902, SGI0, SGI0, SGI193903, SGI193904, SGI193904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43029[] = {SGI193905, SGI193906, SGI193907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193908, SGI0, SGI0, SGI0, SGI193909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43030[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193910, SGI0, SGI0, SGI193911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43031[] = {SGI193912, SGI193913, SGI193914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193915, SGI0, SGI0, SGI0, SGI193916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43032[] = {SGI193917, SGI0, SGI193918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193919, SGI193920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43033[] = {SGI193921, SGI193922, SGI193923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193924, SGI0, SGI0, SGI0, SGI193925, SGI193925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43034[] = {SGI193926, SGI193927, SGI193928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193929, SGI193929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43035[] = {SGI193930, SGI0, SGI193931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43036[] = {SGI193932, SGI0, SGI193933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193934, SGI193935, SGI193935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43037[] = {SGI193936, SGI0, SGI193937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193938, SGI193939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43038[] = {SGI193940, SGI0, SGI193941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193942, SGI193943, SGI193943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43039[] = {SGI193944, SGI0, SGI193945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193946, SGI193947, SGI193947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43040[] = {SGI193948, SGI0, SGI193949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193950, SGI193951, SGI193951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43041[] = {SGI193952, SGI193953, SGI193954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193955, SGI0, SGI0, SGI0, SGI193956, SGI193956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43042[] = {SGI193957, SGI0, SGI193958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193959, SGI193960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43043[] = {SGI193961, SGI193963, SGI193964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193966, SGI0, SGI0, SGI0, SGI193968, SGI193968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43044[] = {SGI193962, SGI0, SGI193965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193967, SGI193969, SGI193969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193970, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43045[] = {SGI193971, SGI0, SGI193972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193973, SGI193974, SGI193974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193975, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43046[] = {SGI193976, SGI0, SGI193977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193978, SGI193979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43047[] = {SGI193980, SGI0, SGI193981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193982, SGI193983, SGI193983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43048[] = {SGI193984, SGI0, SGI193985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193986, SGI193987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43049[] = {SGI193988, SGI193989, SGI193990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193991, SGI0, SGI0, SGI0, SGI193992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43050[] = {SGI193993, SGI0, SGI193994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI193995, SGI193996, SGI193996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43051[] = {SGI193997, SGI0, SGI193998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194001, SGI194002, SGI194002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43052[] = {SGI193999, SGI0, SGI194000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194003, SGI194004, SGI194004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43053[] = {SGI194005, SGI194006, SGI194007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194008, SGI0, SGI0, SGI0, SGI194009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43054[] = {SGI194010, SGI0, SGI194011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194012, SGI194013, SGI194013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43055[] = {SGI194014, SGI194015, SGI194016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194017, SGI0, SGI0, SGI0, SGI194018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43056[] = {SGI194019, SGI194020, SGI194021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194023, SGI194023, SGI0, SGI0, SGI0, SGI194026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43057[] = {SGI194022, SGI194024, SGI194025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194027, SGI0, SGI0, SGI0, SGI194028, SGI194028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43058[] = {SGI194029, SGI194030, SGI194031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194032, SGI0, SGI0, SGI0, SGI194033, SGI194033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43059[] = {SGI194035, SGI194036, SGI194037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194038, SGI0, SGI0, SGI0, SGI194039, SGI0, SGI0, SGI194040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43060[] = {SGI194041, SGI0, SGI194042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194043, SGI0, SGI0, SGI0, SGI194044, SGI0, SGI0, SGI194045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43061[] = {SGI194046, SGI194047, SGI194048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194049, SGI0, SGI0, SGI0, SGI194050, SGI194050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43062[] = {SGI194051, SGI0, SGI194052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194053, SGI194054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43063[] = {SGI194055, SGI194056, SGI194057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194058, SGI0, SGI0, SGI0, SGI194059, SGI194059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43064[] = {SGI194060, SGI0, SGI194061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194062, SGI194063, SGI194063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43065[] = {SGI194064, SGI194065, SGI194066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194067, SGI0, SGI0, SGI0, SGI194068, SGI194068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43066[] = {SGI194069, SGI194070, SGI194071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194072, SGI0, SGI0, SGI0, SGI194073, SGI194073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43067[] = {SGI194074, SGI0, SGI194075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194076, SGI194077, SGI194077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43068[] = {SGI194078, SGI194079, SGI194080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194081, SGI0, SGI0, SGI0, SGI194082, SGI194082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43069[] = {SGI194083, SGI0, SGI194084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43070[] = {SGI194085, SGI0, SGI194086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194087, SGI194088, SGI194088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43071[] = {SGI194089, SGI194091, SGI194092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194094, SGI0, SGI0, SGI0, SGI194095, SGI194095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43072[] = {SGI194096, SGI194097, SGI194098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194099, SGI194099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43073[] = {SGI194100, SGI0, SGI194101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43074[] = {SGI0, SGI194102, SGI194103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43075[] = {SGI194104, SGI0, SGI194105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194106, SGI194107, SGI194107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43076[] = {SGI194108, SGI0, SGI194109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194110, SGI194111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43077[] = {SGI194112, SGI0, SGI194113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194114, SGI194115, SGI194115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43078[] = {SGI194116, SGI0, SGI194117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194118, SGI194119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43079[] = {SGI194120, SGI0, SGI194121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194122, SGI194123, SGI194123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43080[] = {SGI194124, SGI194125, SGI194126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194127, SGI0, SGI0, SGI0, SGI194128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43081[] = {SGI194129, SGI194130, SGI194131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194132, SGI0, SGI0, SGI0, SGI194134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43082[] = {SGI194133, SGI0, SGI194135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194136, SGI194137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43083[] = {SGI194138, SGI0, SGI194139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194142, SGI194143, SGI194143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43084[] = {SGI194140, SGI0, SGI194141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194144, SGI194145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43085[] = {SGI194146, SGI0, SGI194147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194148, SGI194149, SGI194149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43086[] = {SGI194150, SGI0, SGI194151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194152, SGI194153, SGI194153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43087[] = {SGI194154, SGI194155, SGI194157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194160, SGI0, SGI0, SGI0, SGI194161, SGI194161, SGI0, SGI194163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43088[] = {SGI194156, SGI194158, SGI194159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194162, SGI194162, SGI0, SGI0, SGI0, SGI194164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43089[] = {SGI194165, SGI194166, SGI194167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194168, SGI0, SGI0, SGI0, SGI194169, SGI194169, SGI0, SGI194170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43090[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194174, SGI0, SGI0, SGI194174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43091[] = {SGI194171, SGI194172, SGI194173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194175, SGI194175, SGI0, SGI0, SGI0, SGI194176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43092[] = {SGI194177, SGI194178, SGI194179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194180, SGI0, SGI0, SGI0, SGI194181, SGI194181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43093[] = {SGI194182, SGI194183, SGI194184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194185, SGI0, SGI0, SGI0, SGI194186, SGI194186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43094[] = {SGI194187, SGI194188, SGI194189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194190, SGI0, SGI0, SGI0, SGI194191, SGI194191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43095[] = {SGI194192, SGI194193, SGI194194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194195, SGI0, SGI0, SGI0, SGI194196, SGI194196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43096[] = {SGI194197, SGI194198, SGI194199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194200, SGI0, SGI0, SGI0, SGI194201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43097[] = {SGI194202, SGI194203, SGI194204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194205, SGI0, SGI0, SGI0, SGI194206, SGI194206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43098[] = {SGI194207, SGI194208, SGI194209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194210, SGI0, SGI0, SGI0, SGI194211, SGI194211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43099[] = {SGI194213, SGI194214, SGI194215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194216, SGI0, SGI0, SGI0, SGI194217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43100[] = {SGI194218, SGI194219, SGI194220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194221, SGI0, SGI0, SGI0, SGI194222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43101[] = {SGI194223, SGI194224, SGI194225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194226, SGI0, SGI0, SGI0, SGI194227, SGI194227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43102[] = {SGI194228, SGI0, SGI194229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194230, SGI194231, SGI194231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43103[] = {SGI194232, SGI0, SGI194233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194234, SGI194235, SGI194235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43104[] = {SGI194236, SGI0, SGI194237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194238, SGI194239, SGI194239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43105[] = {SGI194240, SGI194241, SGI194242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194243, SGI0, SGI0, SGI0, SGI194244, SGI194244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43106[] = {SGI194245, SGI194246, SGI194247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194248, SGI0, SGI0, SGI0, SGI194249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43107[] = {SGI194250, SGI194251, SGI194252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194253, SGI0, SGI0, SGI0, SGI194256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43108[] = {SGI194254, SGI194255, SGI194257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194258, SGI0, SGI0, SGI0, SGI194259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43109[] = {SGI194260, SGI194261, SGI194262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194263, SGI0, SGI0, SGI0, SGI194264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43110[] = {SGI194265, SGI194266, SGI194267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194268, SGI0, SGI0, SGI0, SGI194269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43111[] = {SGI194270, SGI0, SGI194271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43112[] = {SGI194090, SGI0, SGI194093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194272, SGI194273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43113[] = {SGI194274, SGI194275, SGI194276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194277, SGI0, SGI0, SGI194278, SGI194279, SGI194279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43114[] = {SGI194280, SGI194281, SGI194282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194283, SGI0, SGI0, SGI0, SGI194284, SGI194284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43115[] = {SGI194285, SGI0, SGI194286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194289, SGI194290, SGI194290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43116[] = {SGI194287, SGI0, SGI194288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194291, SGI0, SGI0, SGI0, SGI194292, SGI194292, SGI0, SGI0, SGI0, SGI194293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43117[] = {SGI194294, SGI0, SGI194295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194298, SGI0, SGI0, SGI0, SGI194299, SGI194299, SGI0, SGI0, SGI0, SGI194301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43118[] = {SGI194296, SGI0, SGI194297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194300, SGI194302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43119[] = {SGI194305, SGI0, SGI194307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194309, SGI194311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43120[] = {SGI194303, SGI194304, SGI194306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194308, SGI0, SGI0, SGI0, SGI194310, SGI194312, SGI0, SGI0, SGI0, SGI194313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43121[] = {SGI194314, SGI0, SGI194316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194319, SGI194321, SGI194321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43122[] = {SGI194315, SGI194317, SGI194318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194320, SGI0, SGI0, SGI0, SGI194322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43123[] = {SGI194323, SGI194324, SGI194325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194326, SGI0, SGI0, SGI0, SGI194327, SGI194327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43124[] = {SGI194328, SGI0, SGI194329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43125[] = {SGI194330, SGI0, SGI194331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194332, SGI194333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43126[] = {SGI194334, SGI194335, SGI194336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194337, SGI0, SGI0, SGI0, SGI194338, SGI194338, SGI0, SGI194339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43127[] = {SGI194340, SGI0, SGI194341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194345, SGI194346, SGI194346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43128[] = {SGI194342, SGI194343, SGI194344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194347, SGI0, SGI0, SGI0, SGI194348, SGI194348, SGI0, SGI194349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43129[] = {SGI194350, SGI194351, SGI194352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194353, SGI0, SGI0, SGI0, SGI194354, SGI194354, SGI0, SGI194355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43130[] = {SGI194357, SGI0, SGI194358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194360, SGI194363, SGI194363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43131[] = {SGI194356, SGI0, SGI194359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194361, SGI194362, SGI194362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194364, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43132[] = {SGI194366, SGI194367, SGI194369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194370, SGI0, SGI0, SGI0, SGI194372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43133[] = {SGI194365, SGI0, SGI194368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194371, SGI194373, SGI194373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43134[] = {SGI194374, SGI0, SGI194375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194376, SGI194377, SGI194377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43135[] = {SGI194378, SGI0, SGI194380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194382, SGI194383, SGI194383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43136[] = {SGI194379, SGI0, SGI194381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194384, SGI194385, SGI194385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43137[] = {SGI194386, SGI0, SGI194387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194388, SGI194389, SGI194389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43138[] = {SGI194390, SGI0, SGI194391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194392, SGI194393, SGI194393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43139[] = {SGI194394, SGI0, SGI194395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194396, SGI194397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43140[] = {SGI194398, SGI0, SGI194399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194400, SGI194401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43141[] = {SGI194402, SGI0, SGI194403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194404, SGI194405, SGI194405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43142[] = {SGI194406, SGI0, SGI194407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194408, SGI194409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43143[] = {SGI194410, SGI0, SGI194411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194414, SGI194415, SGI194415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43144[] = {SGI194412, SGI0, SGI194413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194416, SGI194417, SGI194417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43145[] = {SGI194418, SGI0, SGI194419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194422, SGI194423, SGI194423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43146[] = {SGI194420, SGI0, SGI194421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194424, SGI194425, SGI194425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43147[] = {SGI194426, SGI0, SGI194429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194432, SGI194433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43148[] = {SGI194427, SGI194428, SGI194430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194431, SGI0, SGI0, SGI0, SGI194434, SGI194434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43149[] = {SGI194436, SGI0, SGI194437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194438, SGI194439, SGI194439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43150[] = {SGI194440, SGI0, SGI194441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194442, SGI194443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43151[] = {SGI194444, SGI194445, SGI194446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194447, SGI194447, SGI0, SGI0, SGI0, SGI194448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43152[] = {SGI194449, SGI194450, SGI194451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194452, SGI194452, SGI0, SGI0, SGI0, SGI194453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43153[] = {SGI194454, SGI0, SGI194455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194456, SGI194457, SGI194457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43154[] = {SGI194458, SGI194460, SGI194461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194463, SGI0, SGI0, SGI0, SGI194467, SGI194467, SGI0, SGI194470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43155[] = {SGI194459, SGI0, SGI194462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194465, SGI194468, SGI194468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43156[] = {SGI194464, SGI194466, SGI194469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194471, SGI0, SGI0, SGI0, SGI194472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43157[] = {SGI194473, SGI0, SGI194474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194475, SGI194476, SGI194476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43158[] = {SGI194477, SGI194478, SGI194479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194480, SGI0, SGI0, SGI0, SGI194481, SGI194481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43159[] = {SGI194482, SGI194483, SGI194484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194485, SGI0, SGI0, SGI0, SGI194486, SGI194486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43160[] = {SGI194487, SGI0, SGI194488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194489, SGI194490, SGI194490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43161[] = {SGI194491, SGI0, SGI194492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194494, SGI194495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43162[] = {SGI194493, SGI0, SGI194496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194497, SGI194498, SGI194498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43163[] = {SGI194499, SGI0, SGI194500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194501, SGI194502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43164[] = {SGI194504, SGI0, SGI194506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194508, SGI194509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43165[] = {SGI194503, SGI0, SGI194505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194507, SGI0, SGI0, SGI0, SGI194510, SGI194510, SGI0, SGI0, SGI0, SGI0, SGI194511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43166[] = {SGI194512, SGI194513, SGI194514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194515, SGI0, SGI0, SGI0, SGI194516, SGI194517, SGI0, SGI0, SGI0, SGI0, SGI194518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43167[] = {SGI194519, SGI0, SGI194520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194521, SGI0, SGI0, SGI0, SGI194522, SGI194522, SGI0, SGI0, SGI0, SGI0, SGI194523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43168[] = {SGI194524, SGI0, SGI194525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194526, SGI194527, SGI194527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43169[] = {SGI194528, SGI0, SGI194529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194530, SGI194531, SGI194531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43170[] = {SGI194532, SGI0, SGI194533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194534, SGI194535, SGI194535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43171[] = {SGI194536, SGI0, SGI194537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194538, SGI194539, SGI194539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43172[] = {SGI194540, SGI0, SGI194542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194544, SGI194545, SGI194545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43173[] = {SGI194541, SGI0, SGI194543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194546, SGI194547, SGI194547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43174[] = {SGI194548, SGI0, SGI194549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194550, SGI194551, SGI194551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43175[] = {SGI194552, SGI0, SGI194553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194554, SGI194555, SGI194555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43176[] = {SGI194556, SGI0, SGI194557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194558, SGI194559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43177[] = {SGI194560, SGI194561, SGI194562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194563, SGI0, SGI0, SGI0, SGI194564, SGI194564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43178[] = {SGI194565, SGI0, SGI194566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43179[] = {SGI194567, SGI194568, SGI194569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194570, SGI0, SGI0, SGI0, SGI194571, SGI194571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43180[] = {SGI194572, SGI194573, SGI194574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194575, SGI0, SGI0, SGI0, SGI194576, SGI194576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43181[] = {SGI194577, SGI0, SGI194578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194579, SGI194580, SGI194580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194581, SGI0, SGI0, NULL};
const Segment * const SG43182[] = {SGI194582, SGI194583, SGI194584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194585, SGI0, SGI0, SGI0, SGI194586, SGI194586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43183[] = {SGI194587, SGI194588, SGI194589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194590, SGI0, SGI0, SGI0, SGI194591, SGI194591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43184[] = {SGI194592, SGI0, SGI194593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194594, SGI194595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43185[] = {SGI194596, SGI0, SGI194597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194598, SGI194599, SGI194599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43186[] = {SGI194600, SGI194601, SGI194602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194603, SGI0, SGI0, SGI0, SGI194604, SGI194604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43187[] = {SGI194605, SGI0, SGI194606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43188[] = {SGI194607, SGI0, SGI194608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194609, SGI194610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43189[] = {SGI194611, SGI194612, SGI194613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194614, SGI0, SGI0, SGI0, SGI194615, SGI194615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43190[] = {SGI194616, SGI194617, SGI194618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194619, SGI0, SGI0, SGI0, SGI194620, SGI194620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43191[] = {SGI194621, SGI0, SGI194622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43192[] = {SGI194623, SGI0, SGI194624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194625, SGI194626, SGI194626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43193[] = {SGI194627, SGI0, SGI194628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194629, SGI194630, SGI194630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43194[] = {SGI194631, SGI194632, SGI194633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194634, SGI0, SGI0, SGI0, SGI194635, SGI194635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43195[] = {SGI194636, SGI0, SGI194637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194638, SGI194639, SGI194639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43196[] = {SGI194640, SGI0, SGI194641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194642, SGI194643, SGI194643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43197[] = {SGI194644, SGI0, SGI194645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194648, SGI194649, SGI194649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43198[] = {SGI194646, SGI0, SGI194647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194650, SGI194651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43199[] = {SGI194652, SGI194653, SGI194655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194658, SGI0, SGI0, SGI0, SGI194660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43200[] = {SGI194654, SGI0, SGI194656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194661, SGI194662, SGI194662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43201[] = {SGI194657, SGI0, SGI194659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194663, SGI194664, SGI194664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43202[] = {SGI194665, SGI194666, SGI194667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194668, SGI0, SGI0, SGI0, SGI194670, SGI194670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43203[] = {SGI194669, SGI0, SGI194671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194672, SGI194673, SGI194673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43204[] = {SGI194674, SGI0, SGI194675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194676, SGI194677, SGI194677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43205[] = {SGI194678, SGI194679, SGI194681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194683, SGI0, SGI0, SGI0, SGI194684, SGI194684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43206[] = {SGI194680, SGI0, SGI194682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194685, SGI194686, SGI194686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43207[] = {SGI194687, SGI0, SGI194688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194692, SGI194693, SGI194693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43208[] = {SGI194690, SGI0, SGI194694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194697, SGI194698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43209[] = {SGI194689, SGI0, SGI194691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194695, SGI194696, SGI194696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43210[] = {SGI194699, SGI0, SGI194700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194701, SGI194702, SGI194702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43211[] = {SGI194703, SGI194704, SGI194705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194706, SGI0, SGI0, SGI0, SGI194707, SGI194707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194708, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43212[] = {SGI194710, SGI0, SGI194713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194715, SGI194717, SGI194717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43213[] = {SGI194709, SGI194711, SGI194712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194714, SGI0, SGI0, SGI0, SGI194716, SGI194716, SGI0, SGI194718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43214[] = {SGI194719, SGI194720, SGI194721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194722, SGI0, SGI0, SGI0, SGI194723, SGI194723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194727, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43215[] = {SGI194724, SGI0, SGI194725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194730, SGI194731, SGI194731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43216[] = {SGI194726, SGI194728, SGI194729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194732, SGI0, SGI0, SGI0, SGI194733, SGI194733, SGI0, SGI194734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43217[] = {SGI194735, SGI0, SGI194736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194737, SGI194738, SGI194738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43218[] = {SGI194739, SGI0, SGI194740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194741, SGI194742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43219[] = {SGI194743, SGI0, SGI194744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194745, SGI194746, SGI194746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43220[] = {SGI194747, SGI0, SGI194748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194751, SGI194752, SGI194752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43221[] = {SGI194749, SGI0, SGI194750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194753, SGI194754, SGI194754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43222[] = {SGI194755, SGI0, SGI194756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194757, SGI194759, SGI194759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43223[] = {SGI194758, SGI194760, SGI194761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194762, SGI0, SGI0, SGI0, SGI194763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43224[] = {SGI194764, SGI0, SGI194765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194766, SGI194767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43225[] = {SGI194768, SGI194769, SGI194770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194771, SGI0, SGI0, SGI0, SGI194772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43226[] = {SGI194773, SGI194774, SGI194775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194779, SGI0, SGI0, SGI0, SGI194780, SGI194780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43227[] = {SGI194776, SGI194777, SGI194778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194781, SGI0, SGI0, SGI0, SGI194782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43228[] = {SGI194783, SGI194784, SGI194785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194786, SGI0, SGI0, SGI0, SGI194787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43229[] = {SGI194788, SGI194789, SGI194790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194791, SGI0, SGI0, SGI0, SGI194792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43230[] = {SGI194793, SGI0, SGI194794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194795, SGI194796, SGI194796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43231[] = {SGI194797, SGI0, SGI194798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194799, SGI194800, SGI194800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43232[] = {SGI194801, SGI0, SGI194802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194803, SGI194804, SGI194804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43233[] = {SGI194805, SGI0, SGI194806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194807, SGI194808, SGI194808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43234[] = {SGI194809, SGI0, SGI194810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194811, SGI194812, SGI194812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43235[] = {SGI194813, SGI0, SGI194814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194815, SGI194816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43236[] = {SGI194817, SGI0, SGI194818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194819, SGI194820, SGI194820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43237[] = {SGI194821, SGI194822, SGI194823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194824, SGI0, SGI0, SGI0, SGI194825, SGI194825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43238[] = {SGI194826, SGI0, SGI194827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43239[] = {SGI194828, SGI0, SGI194829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194830, SGI194831, SGI194831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43240[] = {SGI194832, SGI194833, SGI194834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194835, SGI0, SGI0, SGI0, SGI194836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43241[] = {SGI194837, SGI0, SGI194838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43242[] = {SGI194839, SGI194840, SGI194841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194842, SGI0, SGI0, SGI0, SGI194843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43243[] = {SGI194844, SGI194845, SGI194846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194849, SGI0, SGI0, SGI0, SGI194850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43244[] = {SGI194847, SGI0, SGI194848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43245[] = {SGI194851, SGI194852, SGI194854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194857, SGI0, SGI0, SGI0, SGI194859, SGI194859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43246[] = {SGI194853, SGI194855, SGI194856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194858, SGI0, SGI0, SGI0, SGI194860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43247[] = {SGI194861, SGI194862, SGI194864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194867, SGI0, SGI0, SGI0, SGI194869, SGI194869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43248[] = {SGI194863, SGI194865, SGI194866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194868, SGI0, SGI0, SGI0, SGI194870, SGI194870, SGI0, SGI194871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43249[] = {SGI194873, SGI0, SGI194875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194879, SGI194881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43250[] = {SGI194872, SGI0, SGI194874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194880, SGI194883, SGI194883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43251[] = {SGI194876, SGI194877, SGI194878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194882, SGI0, SGI0, SGI0, SGI194884, SGI194884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43252[] = {SGI194885, SGI0, SGI194886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194887, SGI194888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43253[] = {SGI194889, SGI0, SGI194891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194893, SGI194896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43254[] = {SGI194890, SGI0, SGI194892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194894, SGI194895, SGI194895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43255[] = {SGI194897, SGI0, SGI194898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194899, SGI194900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194901, SGI0, SGI0, NULL};
const Segment * const SG43256[] = {SGI194902, SGI194903, SGI194904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194905, SGI0, SGI0, SGI0, SGI194906, SGI194906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43257[] = {SGI194907, SGI0, SGI194908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194909, SGI194910, SGI194910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43258[] = {SGI194911, SGI0, SGI194912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194914, SGI194916, SGI194916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194917, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43259[] = {SGI194913, SGI0, SGI194915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194918, SGI194919, SGI194919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43260[] = {SGI194920, SGI0, SGI194921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194923, SGI194924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43261[] = {SGI194922, SGI0, SGI194925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194926, SGI194928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43262[] = {SGI194927, SGI0, SGI194929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194930, SGI194931, SGI194931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43263[] = {SGI194932, SGI0, SGI194933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194934, SGI194935, SGI194935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43264[] = {SGI194936, SGI0, SGI194937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194938, SGI194940, SGI194940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43265[] = {SGI194939, SGI0, SGI194941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194942, SGI194943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43266[] = {SGI194944, SGI0, SGI194945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194946, SGI194947, SGI194947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43267[] = {SGI194948, SGI194949, SGI194950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194951, SGI0, SGI0, SGI0, SGI194952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43268[] = {SGI194953, SGI194954, SGI194955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194956, SGI0, SGI0, SGI0, SGI194957, SGI194957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43269[] = {SGI194958, SGI0, SGI194959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43270[] = {SGI194960, SGI0, SGI194961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194962, SGI194963, SGI194963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43271[] = {SGI194964, SGI194965, SGI194966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194967, SGI0, SGI0, SGI0, SGI194968, SGI194968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194969, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43272[] = {SGI194970, SGI0, SGI194971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194972, SGI194973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43273[] = {SGI194974, SGI194975, SGI194976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194977, SGI0, SGI0, SGI0, SGI194978, SGI194978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43274[] = {SGI194979, SGI194980, SGI194981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194982, SGI0, SGI0, SGI0, SGI194983, SGI194983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43275[] = {SGI194984, SGI0, SGI194985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194986, SGI194987, SGI194987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43276[] = {SGI194988, SGI0, SGI194989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194990, SGI194991, SGI194991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43277[] = {SGI194992, SGI0, SGI194993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194994, SGI194995, SGI194995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43278[] = {SGI194996, SGI194997, SGI194998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI194999, SGI0, SGI0, SGI0, SGI195000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43279[] = {SGI195002, SGI0, SGI195004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195008, SGI195009, SGI195009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43280[] = {SGI195003, SGI195005, SGI195006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195007, SGI0, SGI0, SGI0, SGI195010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43281[] = {SGI195011, SGI0, SGI195012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195013, SGI195014, SGI195014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43282[] = {SGI195015, SGI0, SGI195016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195017, SGI195018, SGI195018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43283[] = {SGI195019, SGI0, SGI195020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195021, SGI195022, SGI195022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43284[] = {SGI195023, SGI0, SGI195024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195025, SGI195026, SGI195026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43285[] = {SGI195027, SGI0, SGI195028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195029, SGI195030, SGI195030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43286[] = {SGI195031, SGI195032, SGI195033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195034, SGI0, SGI0, SGI0, SGI195035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43287[] = {SGI195036, SGI0, SGI195037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195038, SGI195039, SGI195039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43288[] = {SGI195040, SGI0, SGI195041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195042, SGI195043, SGI195043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43289[] = {SGI195044, SGI195045, SGI195046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195047, SGI0, SGI0, SGI0, SGI195048, SGI195048, SGI0, SGI195049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43290[] = {SGI195050, SGI0, SGI195051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195052, SGI195053, SGI195053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43291[] = {SGI195054, SGI0, SGI195055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195056, SGI195057, SGI195057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43292[] = {SGI195058, SGI0, SGI195059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195060, SGI195061, SGI195061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43293[] = {SGI195062, SGI0, SGI195063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195066, SGI195067, SGI195067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43294[] = {SGI195064, SGI0, SGI195065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195068, SGI195069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195070, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43295[] = {SGI195071, SGI0, SGI195072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195073, SGI195074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195075, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43296[] = {SGI195076, SGI0, SGI195077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195078, SGI195079, SGI195079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43297[] = {SGI195080, SGI195081, SGI195082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195083, SGI0, SGI0, SGI0, SGI195084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195086, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43298[] = {SGI195085, SGI195087, SGI195088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195089, SGI0, SGI0, SGI0, SGI195090, SGI195090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43299[] = {SGI195091, SGI0, SGI195092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43300[] = {SGI195093, SGI0, SGI195094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195095, SGI0, SGI0, SGI0, SGI195096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195097, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43301[] = {SGI195098, SGI0, SGI195099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195100, SGI195101, SGI195101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43302[] = {SGI195102, SGI195103, SGI195104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195105, SGI0, SGI0, SGI0, SGI195106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43303[] = {SGI195107, SGI195108, SGI195109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195110, SGI0, SGI0, SGI0, SGI195111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43304[] = {SGI195112, SGI195113, SGI195114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195115, SGI0, SGI0, SGI0, SGI195116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43305[] = {SGI195117, SGI0, SGI195118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195119, SGI195120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43306[] = {SGI195121, SGI195122, SGI195123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195124, SGI0, SGI0, SGI0, SGI195125, SGI195125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43307[] = {SGI195126, SGI0, SGI195127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195128, SGI195129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43308[] = {SGI195130, SGI0, SGI195131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195132, SGI195133, SGI195133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43309[] = {SGI195134, SGI0, SGI195135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195136, SGI195137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43310[] = {SGI195138, SGI195139, SGI195140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195141, SGI195141, SGI0, SGI0, SGI195142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43311[] = {SGI195143, SGI195144, SGI195145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195146, SGI0, SGI0, SGI0, SGI195148, SGI195148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43312[] = {SGI195147, SGI195149, SGI195150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195151, SGI0, SGI0, SGI0, SGI195152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43313[] = {SGI195153, SGI0, SGI195154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43314[] = {SGI195155, SGI195156, SGI195157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195158, SGI0, SGI0, SGI0, SGI195159, SGI195159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43315[] = {SGI195160, SGI0, SGI195161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195162, SGI195163, SGI195163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195164, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43316[] = {SGI195165, SGI195166, SGI195167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195168, SGI0, SGI0, SGI0, SGI195169, SGI195169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43317[] = {SGI195170, SGI195171, SGI195172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195173, SGI0, SGI0, SGI0, SGI195174, SGI195174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43318[] = {SGI195175, SGI0, SGI195176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195177, SGI195178, SGI195178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43319[] = {SGI195179, SGI0, SGI195180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195181, SGI195182, SGI195182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43320[] = {SGI195183, SGI0, SGI195184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195185, SGI195186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43321[] = {SGI195187, SGI195188, SGI195189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195190, SGI0, SGI0, SGI0, SGI195191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43322[] = {SGI195192, SGI195193, SGI195194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195195, SGI0, SGI0, SGI0, SGI195196, SGI195196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43323[] = {SGI195198, SGI0, SGI195199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195200, SGI195201, SGI195201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43324[] = {SGI195202, SGI0, SGI195203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195204, SGI195205, SGI195205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43325[] = {SGI195206, SGI0, SGI195207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195208, SGI195209, SGI195209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43326[] = {SGI195210, SGI195211, SGI195212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195213, SGI195213, SGI0, SGI0, SGI0, SGI195214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43327[] = {SGI195215, SGI0, SGI195216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195217, SGI195218, SGI195218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43328[] = {SGI195219, SGI195220, SGI195221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195222, SGI195222, SGI0, SGI0, SGI195223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43329[] = {SGI195224, SGI195225, SGI195226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195227, SGI195227, SGI0, SGI0, SGI0, SGI195228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43330[] = {SGI195229, SGI195230, SGI195231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195232, SGI195232, SGI0, SGI0, SGI195233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43331[] = {SGI195234, SGI195235, SGI195236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195237, SGI195237, SGI0, SGI0, SGI195238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43332[] = {SGI195239, SGI195240, SGI195241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195242, SGI195242, SGI0, SGI0, SGI195243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43333[] = {SGI195244, SGI195245, SGI195246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195247, SGI195247, SGI0, SGI0, SGI195248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43334[] = {SGI195249, SGI0, SGI195250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195251, SGI195252, SGI195252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43335[] = {SGI195253, SGI195254, SGI195255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195256, SGI0, SGI0, SGI0, SGI195257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43336[] = {SGI195258, SGI0, SGI195259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195260, SGI195261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43337[] = {SGI195262, SGI0, SGI195263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195264, SGI195265, SGI195265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43338[] = {SGI195266, SGI0, SGI195267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195268, SGI195269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43339[] = {SGI195270, SGI195271, SGI195272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195273, SGI0, SGI0, SGI0, SGI195274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43340[] = {SGI195275, SGI195276, SGI195277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195278, SGI0, SGI0, SGI0, SGI195279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43341[] = {SGI195280, SGI195281, SGI195282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195283, SGI0, SGI0, SGI0, SGI195284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43342[] = {SGI195285, SGI195286, SGI195287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195288, SGI0, SGI0, SGI0, SGI195289, SGI195289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43343[] = {SGI195290, SGI0, SGI195291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195292, SGI195293, SGI195293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43344[] = {SGI195294, SGI0, SGI195295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195296, SGI195297, SGI195297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43345[] = {SGI195298, SGI195299, SGI195300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195301, SGI195301, SGI0, SGI0, SGI0, SGI195302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43346[] = {SGI195303, SGI195304, SGI195305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195306, SGI0, SGI0, SGI0, SGI195307, SGI195307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43347[] = {SGI195308, SGI0, SGI195309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195310, SGI195311, SGI195311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195312, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43348[] = {SGI195313, SGI0, SGI195314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43349[] = {SGI195315, SGI0, SGI195316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195317, SGI195318, SGI195318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43350[] = {SGI195319, SGI0, SGI195320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195321, SGI195322, SGI195322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43351[] = {SGI195323, SGI0, SGI195324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195325, SGI195326, SGI195326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43352[] = {SGI195327, SGI195328, SGI195329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195330, SGI0, SGI0, SGI0, SGI195331, SGI195331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43353[] = {SGI195332, SGI0, SGI195333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43354[] = {SGI195334, SGI195335, SGI195336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195337, SGI195337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43355[] = {SGI195338, SGI0, SGI195339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195340, SGI195341, SGI195341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43356[] = {SGI195342, SGI0, SGI195343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195344, SGI195345, SGI195345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43357[] = {SGI195346, SGI0, SGI195347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195348, SGI195349, SGI195349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43358[] = {SGI195350, SGI0, SGI195351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195352, SGI195353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43359[] = {SGI195354, SGI0, SGI195355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195356, SGI195357, SGI195357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43360[] = {SGI195358, SGI0, SGI195359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195360, SGI195361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43361[] = {SGI195362, SGI195363, SGI195364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195365, SGI0, SGI0, SGI0, SGI195367, SGI195367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43362[] = {SGI195366, SGI0, SGI195368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195369, SGI195370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43363[] = {SGI195371, SGI0, SGI195372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195373, SGI195374, SGI195374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43364[] = {SGI195375, SGI195376, SGI195377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195378, SGI195378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195379, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43365[] = {SGI195380, SGI195381, SGI195382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195383, SGI0, SGI0, SGI0, SGI195384, SGI195384, SGI0, SGI195385, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43366[] = {SGI195386, SGI195387, SGI195388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195389, SGI0, SGI0, SGI0, SGI195391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43367[] = {SGI195390, SGI195392, SGI195393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195394, SGI0, SGI0, SGI0, SGI195395, SGI0, SGI0, SGI0, SGI195396, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43368[] = {SGI195397, SGI0, SGI195398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195399, SGI195400, SGI195400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43369[] = {SGI195401, SGI0, SGI195402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195403, SGI195404, SGI195404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43370[] = {SGI195405, SGI0, SGI195406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195407, SGI195408, SGI195408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43371[] = {SGI195409, SGI0, SGI195410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195411, SGI195412, SGI195412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43372[] = {SGI195413, SGI0, SGI195414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195415, SGI195416, SGI195416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43373[] = {SGI195417, SGI0, SGI195418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195419, SGI195420, SGI195420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43374[] = {SGI195421, SGI195422, SGI195423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195424, SGI0, SGI0, SGI0, SGI195425, SGI195425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43375[] = {SGI195426, SGI0, SGI195427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43376[] = {SGI195428, SGI195429, SGI195430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195431, SGI0, SGI0, SGI0, SGI195432, SGI195432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43377[] = {SGI195433, SGI0, SGI195434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195435, SGI195436, SGI195436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43378[] = {SGI195438, SGI0, SGI195440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195442, SGI195444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43379[] = {SGI195437, SGI0, SGI195439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195441, SGI195443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43380[] = {SGI195445, SGI195446, SGI195447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195449, SGI0, SGI0, SGI0, SGI195451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43381[] = {SGI195448, SGI0, SGI195450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195453, SGI195455, SGI195455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43382[] = {SGI0, SGI0, SGI195452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195458, SGI195459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43383[] = {SGI195454, SGI195456, SGI195457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195460, SGI0, SGI0, SGI0, SGI195461, SGI195461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195462, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43384[] = {SGI195463, SGI195464, SGI195465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195468, SGI0, SGI0, SGI0, SGI195469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43385[] = {SGI195466, SGI0, SGI195467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195472, SGI195473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43386[] = {SGI195470, SGI0, SGI195471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195477, SGI195478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43387[] = {SGI195474, SGI195475, SGI195476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195479, SGI0, SGI0, SGI0, SGI195480, SGI195480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195482, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43388[] = {SGI195481, SGI0, SGI195483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43389[] = {SGI195484, SGI0, SGI195485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195487, SGI195489, SGI195489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43390[] = {SGI195486, SGI195488, SGI195490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195493, SGI0, SGI0, SGI195495, SGI195496, SGI195496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43391[] = {SGI195491, SGI195492, SGI195494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195497, SGI0, SGI0, SGI0, SGI195498, SGI195498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43392[] = {SGI195499, SGI195500, SGI195501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195504, SGI0, SGI0, SGI195506, SGI195507, SGI195507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43393[] = {SGI195502, SGI195503, SGI195505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195508, SGI0, SGI0, SGI0, SGI195509, SGI195509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43394[] = {SGI195510, SGI195511, SGI195512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195515, SGI0, SGI0, SGI195516, SGI195517, SGI195517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43395[] = {SGI195513, SGI0, SGI195514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195518, SGI195519, SGI195519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43396[] = {SGI195521, SGI0, SGI195524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195527, SGI195530, SGI195530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43397[] = {SGI195520, SGI195522, SGI195523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195525, SGI0, SGI0, SGI195528, SGI195531, SGI195531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43398[] = {SGI195526, SGI0, SGI195529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195532, SGI195533, SGI195533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43399[] = {SGI195534, SGI195535, SGI195536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195537, SGI0, SGI0, SGI195538, SGI195539, SGI195539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43400[] = {SGI195540, SGI0, SGI195541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195542, SGI195543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43401[] = {SGI195544, SGI195545, SGI195546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195547, SGI0, SGI0, SGI195548, SGI195549, SGI195549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43402[] = {SGI195550, SGI195551, SGI195553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195556, SGI0, SGI0, SGI0, SGI195557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43403[] = {SGI195552, SGI195554, SGI195555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195558, SGI0, SGI0, SGI195559, SGI195560, SGI195560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43404[] = {SGI195561, SGI195562, SGI195563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195567, SGI0, SGI0, SGI0, SGI195568, SGI195568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43405[] = {SGI195564, SGI195565, SGI195566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195569, SGI0, SGI0, SGI195570, SGI195571, SGI195571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43406[] = {SGI195572, SGI0, SGI195573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195574, SGI195575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43407[] = {SGI195576, SGI195577, SGI195578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195579, SGI0, SGI195580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43408[] = {SGI195581, SGI0, SGI195582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195586, SGI195587, SGI195587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43409[] = {SGI195583, SGI195584, SGI195585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195588, SGI0, SGI0, SGI195589, SGI195590, SGI195590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43410[] = {SGI195591, SGI195592, SGI195593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195594, SGI0, SGI0, SGI195595, SGI195596, SGI195596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43411[] = {SGI195597, SGI0, SGI195598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195599, SGI195600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43412[] = {SGI195601, SGI195602, SGI195603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195604, SGI0, SGI0, SGI195605, SGI195606, SGI195606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43413[] = {SGI195607, SGI195608, SGI195609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195610, SGI0, SGI0, SGI195611, SGI195612, SGI195612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43414[] = {SGI195613, SGI195614, SGI195615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195616, SGI0, SGI0, SGI195617, SGI195618, SGI195618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43415[] = {SGI195619, SGI0, SGI195620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195621, SGI195622, SGI195622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43416[] = {SGI195623, SGI0, SGI195624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195625, SGI195626, SGI195626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43417[] = {SGI195627, SGI0, SGI195628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195629, SGI195630, SGI195630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43418[] = {SGI195631, SGI0, SGI195632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195633, SGI195634, SGI195634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43419[] = {SGI195635, SGI195636, SGI195637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195638, SGI0, SGI0, SGI0, SGI195639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43420[] = {SGI195641, SGI0, SGI195643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195645, SGI195647, SGI195647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43421[] = {SGI195640, SGI0, SGI195642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195644, SGI195646, SGI195646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43422[] = {SGI195648, SGI195649, SGI195650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195651, SGI0, SGI0, SGI0, SGI195652, SGI195652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43423[] = {SGI195653, SGI0, SGI195654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195655, SGI195656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43424[] = {SGI195657, SGI195658, SGI195659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195660, SGI0, SGI0, SGI0, SGI195661, SGI195661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43425[] = {SGI195663, SGI195664, SGI195665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195666, SGI0, SGI0, SGI0, SGI195667, SGI195667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195668, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43426[] = {SGI195669, SGI195670, SGI195671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195672, SGI0, SGI0, SGI0, SGI195673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43427[] = {SGI195674, SGI195675, SGI195676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195677, SGI0, SGI0, SGI0, SGI195678, SGI195678, SGI0, SGI195679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43428[] = {SGI195680, SGI195681, SGI195682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195683, SGI0, SGI0, SGI0, SGI195684, SGI195684, SGI0, SGI195685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43429[] = {SGI195686, SGI0, SGI195687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195688, SGI195689, SGI195689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43430[] = {SGI195690, SGI195691, SGI195692, SGI195693, SGI195694, SGI195694, SGI0, NULL};
const Segment * const SG43431[] = {SGI195695, SGI195696, SGI195697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195698, SGI0, SGI0, SGI0, SGI195699, SGI195699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43432[] = {SGI195700, SGI195701, SGI195702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195703, SGI0, SGI0, SGI0, SGI195704, SGI195704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43433[] = {SGI195705, SGI195706, SGI195707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195708, SGI0, SGI0, SGI0, SGI195709, SGI195709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43434[] = {SGI195710, SGI195711, SGI195712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195713, SGI195713, SGI0, SGI0, SGI0, SGI0, SGI195714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43435[] = {SGI195715, SGI195716, SGI195717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195718, SGI0, SGI0, SGI0, SGI195719, SGI195719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43436[] = {SGI195720, SGI195721, SGI195722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195723, SGI195723, SGI0, SGI0, SGI0, SGI0, SGI195724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43437[] = {SGI195725, SGI195726, SGI195727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195728, SGI0, SGI0, SGI0, SGI195729, SGI195729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195730, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43438[] = {SGI195731, SGI195732, SGI195733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195734, SGI0, SGI0, SGI0, SGI195735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43439[] = {SGI195736, SGI195737, SGI195738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195739, SGI0, SGI0, SGI0, SGI195740, SGI195740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43440[] = {SGI195741, SGI0, SGI195742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195743, SGI195744, SGI195744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43441[] = {SGI195745, SGI0, SGI195746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195747, SGI195748, SGI195748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43442[] = {SGI195749, SGI0, SGI195750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195751, SGI195752, SGI195752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43443[] = {SGI195753, SGI195754, SGI195755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195756, SGI0, SGI0, SGI0, SGI195757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43444[] = {SGI195758, SGI0, SGI195759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195760, SGI195761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43445[] = {SGI195762, SGI0, SGI195763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195764, SGI195765, SGI195765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43446[] = {SGI195766, SGI195767, SGI195768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195769, SGI0, SGI0, SGI0, SGI195770, SGI195770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43447[] = {SGI195771, SGI195772, SGI195773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195774, SGI0, SGI0, SGI0, SGI195775, SGI195775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195776, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43448[] = {SGI195777, SGI195778, SGI195779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195780, SGI0, SGI0, SGI0, SGI195781, SGI195781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195782, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43449[] = {SGI195783, SGI195784, SGI195785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195786, SGI0, SGI0, SGI0, SGI195787, SGI195787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195788, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43450[] = {SGI195789, SGI0, SGI195790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195791, SGI195792, SGI195792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43451[] = {SGI195793, SGI0, SGI195794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195795, SGI195796, SGI195796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43452[] = {SGI195797, SGI0, SGI195798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195800, SGI195802, SGI195802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43453[] = {SGI195799, SGI195801, SGI195803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195804, SGI0, SGI0, SGI0, SGI195805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43454[] = {SGI195807, SGI0, SGI195808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195809, SGI195810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43455[] = {SGI195811, SGI195812, SGI195813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195814, SGI0, SGI0, SGI0, SGI195815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43456[] = {SGI195817, SGI0, SGI195818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195819, SGI195820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43457[] = {SGI195822, SGI195823, SGI195825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195826, SGI0, SGI0, SGI0, SGI195828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43458[] = {SGI195821, SGI0, SGI195824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195827, SGI195829, SGI195829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43459[] = {SGI195830, SGI0, SGI195831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195832, SGI195833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43460[] = {SGI195834, SGI0, SGI195835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195836, SGI195837, SGI195837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43461[] = {SGI195838, SGI0, SGI195839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195840, SGI195841, SGI195841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43462[] = {SGI195842, SGI0, SGI195843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195844, SGI195845, SGI195845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43463[] = {SGI195846, SGI195847, SGI195848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195849, SGI0, SGI0, SGI0, SGI195850, SGI195850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195851, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43464[] = {SGI195852, SGI0, SGI195853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195854, SGI195855, SGI195855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43465[] = {SGI195856, SGI0, SGI195857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195858, SGI195859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43466[] = {SGI195860, SGI0, SGI195861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195862, SGI195863, SGI195863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43467[] = {SGI195864, SGI0, SGI195865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195866, SGI195867, SGI195867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43468[] = {SGI195868, SGI0, SGI195869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195871, SGI195872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43469[] = {SGI195870, SGI0, SGI195873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195874, SGI195875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195876, SGI0, SGI0, NULL};
const Segment * const SG43470[] = {SGI195877, SGI0, SGI195878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195879, SGI195880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195881, SGI0, SGI0, NULL};
const Segment * const SG43471[] = {SGI195882, SGI195883, SGI195884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195885, SGI0, SGI0, SGI0, SGI195886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43472[] = {SGI195887, SGI195888, SGI195889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195890, SGI0, SGI0, SGI0, SGI195891, SGI195891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195892, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43473[] = {SGI195893, SGI195894, SGI195895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195896, SGI0, SGI0, SGI0, SGI195897, SGI195897, SGI0, SGI0, SGI195899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43474[] = {SGI195898, SGI195900, SGI195901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195902, SGI0, SGI0, SGI195903, SGI195904, SGI195904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43475[] = {SGI195905, SGI0, SGI195906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195907, SGI195908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43476[] = {SGI195909, SGI0, SGI195910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195911, SGI195912, SGI195912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43477[] = {SGI195913, SGI195914, SGI195915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195916, SGI195916, SGI0, SGI0, SGI0, SGI195917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43478[] = {SGI195918, SGI0, SGI195919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195920, SGI195921, SGI195921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43479[] = {SGI195922, SGI0, SGI195923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195924, SGI195925, SGI195925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43480[] = {SGI195926, SGI0, SGI195927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195928, SGI195929, SGI195929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43481[] = {SGI195930, SGI0, SGI195931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195932, SGI195933, SGI195933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43482[] = {SGI195934, SGI0, SGI195935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195936, SGI195937, SGI195937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43483[] = {SGI195938, SGI0, SGI195939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195940, SGI195941, SGI195941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43484[] = {SGI195942, SGI0, SGI195943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195944, SGI195945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43485[] = {SGI195946, SGI0, SGI195947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195948, SGI195949, SGI195949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43486[] = {SGI195950, SGI0, SGI195951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195952, SGI195953, SGI195953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43487[] = {SGI195954, SGI0, SGI195955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195956, SGI195957, SGI195957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43488[] = {SGI195958, SGI0, SGI195959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195960, SGI195961, SGI195961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43489[] = {SGI195962, SGI195963, SGI195964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195965, SGI0, SGI0, SGI0, SGI195966, SGI195966, SGI0, SGI195967, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43490[] = {SGI195968, SGI195969, SGI195970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195976, SGI0, SGI0, SGI0, SGI195979, SGI195979, SGI0, SGI0, SGI195981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43491[] = {SGI195971, SGI195972, SGI195973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195978, SGI0, SGI0, SGI0, SGI195980, SGI195980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43492[] = {SGI195974, SGI195975, SGI195977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195982, SGI0, SGI0, SGI0, SGI195983, SGI195983, SGI0, SGI195984, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43493[] = {SGI195985, SGI195986, SGI195987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195988, SGI0, SGI0, SGI0, SGI195989, SGI195989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43494[] = {SGI195990, SGI0, SGI195991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195992, SGI195993, SGI195993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43495[] = {SGI195994, SGI195995, SGI195996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI195997, SGI0, SGI0, SGI0, SGI195998, SGI195998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43496[] = {SGI195999, SGI196000, SGI196001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196002, SGI0, SGI0, SGI0, SGI196003, SGI196003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43497[] = {SGI196004, SGI0, SGI196005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43498[] = {SGI196006, SGI0, SGI196007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196008, SGI196009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43499[] = {SGI196010, SGI0, SGI196011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196013, SGI196015, SGI196015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43500[] = {SGI196012, SGI0, SGI196014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196016, SGI196017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43501[] = {SGI196018, SGI0, SGI196019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196021, SGI196022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43502[] = {SGI196020, SGI0, SGI196023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196024, SGI196025, SGI196025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43503[] = {SGI196026, SGI196027, SGI196028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196029, SGI0, SGI0, SGI0, SGI196030, SGI196030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43504[] = {SGI196031, SGI0, SGI196032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196033, SGI196034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43505[] = {SGI196035, SGI0, SGI196036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196037, SGI196038, SGI196038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43506[] = {SGI196039, SGI196040, SGI196041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196042, SGI0, SGI0, SGI0, SGI196043, SGI196043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43507[] = {SGI196044, SGI196045, SGI196046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196047, SGI0, SGI0, SGI0, SGI196048, SGI0, SGI0, SGI196049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43508[] = {SGI196050, SGI0, SGI196051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196052, SGI196053, SGI196053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43509[] = {SGI196054, SGI196055, SGI196056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196057, SGI0, SGI0, SGI0, SGI196058, SGI196058, SGI0, SGI196059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43510[] = {SGI196060, SGI196061, SGI196062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196063, SGI0, SGI0, SGI0, SGI196064, SGI196064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43511[] = {SGI196065, SGI196066, SGI196067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196068, SGI0, SGI0, SGI0, SGI196069, SGI0, SGI0, SGI196070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43512[] = {SGI196071, SGI0, SGI196072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196073, SGI196074, SGI196074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43513[] = {SGI196075, SGI0, SGI196076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196077, SGI196078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43514[] = {SGI196079, SGI0, SGI196080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196081, SGI196082, SGI196082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196083, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43515[] = {SGI196084, SGI0, SGI196085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196086, SGI196087, SGI196087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43516[] = {SGI196088, SGI0, SGI196089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196090, SGI196091, SGI196091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43517[] = {SGI196092, SGI0, SGI196093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196094, SGI196095, SGI196095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43518[] = {SGI196096, SGI0, SGI196097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196098, SGI196099, SGI196099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196100, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43519[] = {SGI196101, SGI0, SGI196102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196103, SGI196104, SGI196104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43520[] = {SGI196105, SGI0, SGI196106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196107, SGI196108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43521[] = {SGI196109, SGI0, SGI196110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196111, SGI196112, SGI196112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43522[] = {SGI196113, SGI0, SGI196114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196115, SGI196116, SGI196116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43523[] = {SGI196117, SGI196118, SGI196119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196120, SGI0, SGI0, SGI0, SGI196121, SGI0, SGI196122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43524[] = {SGI196123, SGI196124, SGI196125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196126, SGI0, SGI0, SGI0, SGI196127, SGI196127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43525[] = {SGI196128, SGI196129, SGI196130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196131, SGI0, SGI0, SGI0, SGI196132, SGI196132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43526[] = {SGI196133, SGI196134, SGI196135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196136, SGI0, SGI0, SGI0, SGI196137, SGI196137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196138, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43527[] = {SGI196139, SGI0, SGI196141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196143, SGI196144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43528[] = {SGI196140, SGI0, SGI196142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196145, SGI196146, SGI196146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43529[] = {SGI196147, SGI0, SGI196148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196149, SGI196150, SGI196150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43530[] = {SGI196151, SGI0, SGI196152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196153, SGI196154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43531[] = {SGI196155, SGI196156, SGI196157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196158, SGI0, SGI0, SGI0, SGI196159, SGI196159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43532[] = {SGI196160, SGI0, SGI196161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196162, SGI196163, SGI196163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43533[] = {SGI196164, SGI0, SGI196165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196166, SGI196167, SGI196167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43534[] = {SGI196168, SGI196169, SGI196170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196171, SGI0, SGI0, SGI0, SGI196172, SGI196172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43535[] = {SGI196173, SGI0, SGI196174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196175, SGI196176, SGI196176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43536[] = {SGI196177, SGI196178, SGI196179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196180, SGI0, SGI0, SGI0, SGI196181, SGI196181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43537[] = {SGI196182, SGI0, SGI196183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43538[] = {SGI196184, SGI196185, SGI196186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196187, SGI0, SGI0, SGI0, SGI196188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43539[] = {SGI196189, SGI196190, SGI196191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196192, SGI0, SGI0, SGI0, SGI196193, SGI196193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43540[] = {SGI196194, SGI0, SGI196195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43541[] = {SGI196196, SGI196197, SGI196198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196199, SGI0, SGI0, SGI0, SGI196200, SGI196200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43542[] = {SGI196201, SGI196202, SGI196203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196204, SGI0, SGI0, SGI0, SGI196207, SGI196207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43543[] = {SGI196205, SGI196206, SGI196208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196209, SGI0, SGI0, SGI0, SGI196210, SGI196210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43544[] = {SGI196211, SGI0, SGI196212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196213, SGI0, SGI0, SGI0, SGI196216, SGI196216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43545[] = {SGI196214, SGI196215, SGI196217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196218, SGI0, SGI0, SGI0, SGI196219, SGI196219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43546[] = {SGI196220, SGI196221, SGI196222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196223, SGI0, SGI0, SGI0, SGI196224, SGI196224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43547[] = {SGI196225, SGI0, SGI196226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43548[] = {SGI196227, SGI196228, SGI196229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196230, SGI0, SGI0, SGI0, SGI196232, SGI196232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43549[] = {SGI196231, SGI0, SGI196233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196235, SGI196236, SGI196236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43550[] = {SGI196237, SGI196238, SGI196239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196242, SGI0, SGI0, SGI0, SGI196243, SGI196243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43551[] = {SGI196240, SGI0, SGI196241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196245, SGI196246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43552[] = {SGI196247, SGI196248, SGI196249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196250, SGI0, SGI0, SGI0, SGI196251, SGI196251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43553[] = {SGI196253, SGI196254, SGI196255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196256, SGI196256, SGI0, SGI0, SGI0, SGI196257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43554[] = {SGI196258, SGI196259, SGI196260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196263, SGI0, SGI0, SGI0, SGI196264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43555[] = {SGI196261, SGI0, SGI196262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196265, SGI196266, SGI196266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43556[] = {SGI196267, SGI0, SGI196268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196269, SGI196270, SGI196270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43557[] = {SGI196271, SGI0, SGI196272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196275, SGI196276, SGI196276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43558[] = {SGI196273, SGI0, SGI196274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196277, SGI196278, SGI196278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196279, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43559[] = {SGI196280, SGI0, SGI196281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196282, SGI196283, SGI196283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43560[] = {SGI196284, SGI0, SGI196285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196286, SGI196287, SGI196287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196288, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43561[] = {SGI196289, SGI0, SGI196290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196291, SGI196292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43562[] = {SGI196293, SGI0, SGI196294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196295, SGI196296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43563[] = {SGI196298, SGI0, SGI196300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196301, SGI196303, SGI196303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43564[] = {SGI196297, SGI0, SGI196299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196302, SGI196304, SGI196304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43565[] = {SGI196305, SGI196306, SGI196307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196308, SGI0, SGI0, SGI0, SGI196309, SGI196309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43566[] = {SGI196310, SGI196311, SGI196312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196313, SGI196313, SGI0, SGI0, SGI0, SGI0, SGI196314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43567[] = {SGI196315, SGI196316, SGI196317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196318, SGI196318, SGI0, SGI0, SGI0, SGI0, SGI196319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43568[] = {SGI196320, SGI0, SGI196321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196322, SGI196323, SGI196323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43569[] = {SGI196324, SGI0, SGI196325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196328, SGI196329, SGI196329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43570[] = {SGI196326, SGI0, SGI196327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196330, SGI196331, SGI196331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43571[] = {SGI196332, SGI0, SGI196333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196334, SGI196335, SGI196335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43572[] = {SGI196336, SGI0, SGI196337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196338, SGI196339, SGI196339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43573[] = {SGI196340, SGI0, SGI196341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196342, SGI196343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43574[] = {SGI196344, SGI0, SGI196345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196346, SGI196347, SGI196347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43575[] = {SGI196348, SGI196349, SGI196350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196351, SGI0, SGI0, SGI0, SGI196352, SGI196352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43576[] = {SGI196354, SGI196355, SGI196356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196357, SGI0, SGI0, SGI0, SGI196358, SGI196358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43577[] = {SGI196360, SGI0, SGI196361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196362, SGI196364, SGI196364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43578[] = {SGI196363, SGI196365, SGI196366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196369, SGI0, SGI0, SGI0, SGI196371, SGI196371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43579[] = {SGI196367, SGI196368, SGI196370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196373, SGI0, SGI0, SGI0, SGI196374, SGI196374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43580[] = {SGI196375, SGI196376, SGI196377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196378, SGI0, SGI0, SGI0, SGI196379, SGI196379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43581[] = {SGI196380, SGI0, SGI196381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196382, SGI196383, SGI196383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43582[] = {SGI196384, SGI0, SGI196385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196386, SGI196387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43583[] = {SGI196388, SGI0, SGI196389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196390, SGI196391, SGI196391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43584[] = {SGI196392, SGI0, SGI196393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196394, SGI196395, SGI196395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43585[] = {SGI196396, SGI0, SGI196397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196398, SGI196399, SGI196399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43586[] = {SGI196400, SGI0, SGI196401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196402, SGI196403, SGI196403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43587[] = {SGI196404, SGI0, SGI196405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196406, SGI196407, SGI196407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43588[] = {SGI196408, SGI196409, SGI196410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196411, SGI0, SGI0, SGI0, SGI196412, SGI196412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43589[] = {SGI196413, SGI0, SGI196414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196415, SGI196416, SGI196416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43590[] = {SGI196417, SGI0, SGI196418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196419, SGI196420, SGI196420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43591[] = {SGI196421, SGI0, SGI196422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196423, SGI196424, SGI196424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43592[] = {SGI196425, SGI196426, SGI196427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196431, SGI0, SGI0, SGI0, SGI196432, SGI196432, SGI0, SGI0, SGI0, SGI196434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43593[] = {SGI196428, SGI196429, SGI196430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196433, SGI0, SGI0, SGI0, SGI196435, SGI0, SGI0, SGI196436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43594[] = {SGI196437, SGI196438, SGI196439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196440, SGI0, SGI0, SGI0, SGI196441, SGI0, SGI0, SGI196442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43595[] = {SGI196443, SGI196444, SGI196445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196448, SGI196448, SGI0, SGI0, SGI0, SGI196449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43596[] = {SGI196446, SGI0, SGI196447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196450, SGI196451, SGI196451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43597[] = {SGI196452, SGI0, SGI196453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196454, SGI196455, SGI196455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43598[] = {SGI196456, SGI196457, SGI196458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196459, SGI0, SGI0, SGI0, SGI196460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43599[] = {SGI196461, SGI196462, SGI196463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196464, SGI0, SGI0, SGI0, SGI196465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43600[] = {SGI196466, SGI196467, SGI196468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196469, SGI0, SGI0, SGI0, SGI196470, SGI196470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43601[] = {SGI196471, SGI0, SGI196472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196473, SGI196474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43602[] = {SGI196475, SGI196476, SGI196477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196478, SGI0, SGI0, SGI0, SGI196479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43603[] = {SGI196480, SGI196481, SGI196482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196483, SGI196483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196484, NULL};
const Segment * const SG43604[] = {SGI196485, SGI196487, SGI196489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196492, SGI196492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196494, NULL};
const Segment * const SG43605[] = {SGI196486, SGI196488, SGI196490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196491, SGI0, SGI0, SGI0, SGI196493, SGI196493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43606[] = {SGI196495, SGI196496, SGI196498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196501, SGI196501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196503, NULL};
const Segment * const SG43607[] = {SGI196497, SGI196499, SGI196500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196502, SGI196502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43608[] = {SGI196504, SGI0, SGI196505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43609[] = {SGI196506, SGI0, SGI196507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196509, SGI196510, SGI196510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43610[] = {SGI196508, SGI0, SGI196511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196512, SGI196513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43611[] = {SGI196514, SGI0, SGI196515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196516, SGI196517, SGI196517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43612[] = {SGI196518, SGI0, SGI196519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196520, SGI196521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43613[] = {SGI196522, SGI196523, SGI196524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196525, SGI0, SGI0, SGI0, SGI196526, SGI196526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43614[] = {SGI196527, SGI196528, SGI196529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196530, SGI0, SGI0, SGI0, SGI196531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43615[] = {SGI196532, SGI0, SGI196533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196534, SGI196535, SGI196535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43616[] = {SGI196536, SGI196538, SGI196539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196541, SGI0, SGI0, SGI0, SGI196542, SGI196542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43617[] = {SGI196543, SGI0, SGI196544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196545, SGI196546, SGI196546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43618[] = {SGI196547, SGI196548, SGI196549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196550, SGI0, SGI0, SGI0, SGI196551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43619[] = {SGI196552, SGI196553, SGI196554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196555, SGI0, SGI0, SGI0, SGI196556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43620[] = {SGI196557, SGI196558, SGI196559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196560, SGI0, SGI0, SGI0, SGI196561, SGI196561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43621[] = {SGI196562, SGI0, SGI196563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196564, SGI196565, SGI196565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43622[] = {SGI196566, SGI0, SGI196567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196568, SGI196569, SGI196569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43623[] = {SGI196570, SGI0, SGI196571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196572, SGI196573, SGI196573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43624[] = {SGI196574, SGI196575, SGI196576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196577, SGI0, SGI0, SGI0, SGI196578, SGI196578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43625[] = {SGI196579, SGI0, SGI196580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196584, SGI196585, SGI196585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43626[] = {SGI196581, SGI196582, SGI196583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196586, SGI0, SGI0, SGI0, SGI196587, SGI196587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43627[] = {SGI196588, SGI0, SGI196589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43628[] = {SGI196590, SGI0, SGI196591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196592, SGI196593, SGI196593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43629[] = {SGI196594, SGI196595, SGI196596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196597, SGI196597, SGI0, SGI0, SGI0, SGI0, SGI196598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43630[] = {SGI196599, SGI0, SGI196600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196601, SGI196602, SGI196602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43631[] = {SGI196603, SGI0, SGI196604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196607, SGI196608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43632[] = {SGI196605, SGI0, SGI196606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196609, SGI196610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43633[] = {SGI196611, SGI196612, SGI196613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196614, SGI0, SGI0, SGI0, SGI196615, SGI196615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196616, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43634[] = {SGI196617, SGI196618, SGI196619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196620, SGI196620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43635[] = {SGI196621, SGI196622, SGI196623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196624, SGI0, SGI0, SGI0, SGI196625, SGI196625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43636[] = {SGI196626, SGI0, SGI196627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43637[] = {SGI196628, SGI196629, SGI196630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196631, SGI0, SGI0, SGI0, SGI196632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43638[] = {SGI196633, SGI0, SGI196634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43639[] = {SGI196636, SGI196638, SGI196640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196641, SGI0, SGI0, SGI0, SGI196643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43640[] = {SGI196635, SGI196637, SGI196639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196642, SGI0, SGI0, SGI0, SGI196644, SGI196644, SGI0, SGI196645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43641[] = {SGI196646, SGI196647, SGI196648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196649, SGI0, SGI0, SGI0, SGI196651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43642[] = {SGI196650, SGI196652, SGI196653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196654, SGI0, SGI0, SGI0, SGI196655, SGI196655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43643[] = {SGI196656, SGI0, SGI196657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196658, SGI196659, SGI196659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43644[] = {SGI196660, SGI196661, SGI196662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196663, SGI0, SGI0, SGI0, SGI196664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43645[] = {SGI196665, SGI0, SGI196666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196667, SGI196668, SGI196668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43646[] = {SGI196669, SGI196670, SGI196671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196672, SGI0, SGI0, SGI0, SGI196673, SGI196673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43647[] = {SGI196674, SGI0, SGI196675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196676, SGI196677, SGI196677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43648[] = {SGI196678, SGI0, SGI196679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196680, SGI196681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43649[] = {SGI196682, SGI0, SGI196683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196684, SGI196685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43650[] = {SGI196686, SGI196687, SGI196688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196689, SGI0, SGI0, SGI0, SGI196690, SGI196690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43651[] = {SGI196691, SGI196692, SGI196693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196694, SGI0, SGI0, SGI0, SGI196695, SGI196695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43652[] = {SGI196696, SGI0, SGI196697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43653[] = {SGI196698, SGI196699, SGI196700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196701, SGI196701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43654[] = {SGI196702, SGI0, SGI196703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196704, SGI196705, SGI196705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43655[] = {SGI196706, SGI196707, SGI196708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196709, SGI0, SGI0, SGI0, SGI196710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43656[] = {SGI196711, SGI196712, SGI196713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196714, SGI0, SGI0, SGI0, SGI196715, SGI196715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43657[] = {SGI196537, SGI0, SGI196540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196716, SGI196717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43658[] = {SGI196718, SGI0, SGI196719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196720, SGI196721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43659[] = {SGI196722, SGI0, SGI196723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196724, SGI196725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43660[] = {SGI196726, SGI196727, SGI196728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196729, SGI0, SGI0, SGI0, SGI196730, SGI196730, SGI0, SGI196731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43661[] = {SGI196732, SGI0, SGI196733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196734, SGI196735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43662[] = {SGI196736, SGI0, SGI196738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196740, SGI196742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43663[] = {SGI196737, SGI0, SGI196739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196741, SGI196743, SGI196743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43664[] = {SGI196744, SGI0, SGI196745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196746, SGI196747, SGI196747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43665[] = {SGI196748, SGI196749, SGI196750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196751, SGI0, SGI0, SGI0, SGI196752, SGI196752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43666[] = {SGI196753, SGI0, SGI196754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196755, SGI196756, SGI196756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43667[] = {SGI196757, SGI196758, SGI196759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196760, SGI0, SGI0, SGI0, SGI196761, SGI196761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43668[] = {SGI196762, SGI196763, SGI196764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196765, SGI0, SGI0, SGI0, SGI196766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43669[] = {SGI196767, SGI0, SGI196769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43670[] = {SGI196768, SGI196770, SGI196771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196772, SGI196772, SGI0, SGI0, SGI0, SGI196773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43671[] = {SGI196774, SGI0, SGI196775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196776, SGI196777, SGI196777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43672[] = {SGI196778, SGI0, SGI196779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196780, SGI196781, SGI196781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43673[] = {SGI196782, SGI0, SGI196783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196784, SGI196785, SGI196785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43674[] = {SGI196786, SGI0, SGI196787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196788, SGI196789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43675[] = {SGI196790, SGI0, SGI196791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196792, SGI196793, SGI196793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43676[] = {SGI196794, SGI0, SGI196795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196796, SGI196797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43677[] = {SGI196798, SGI0, SGI196799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196800, SGI196801, SGI196801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43678[] = {SGI196802, SGI0, SGI196803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196804, SGI196805, SGI196805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43679[] = {SGI196806, SGI0, SGI196807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196808, SGI196809, SGI196809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43680[] = {SGI196810, SGI0, SGI196811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196812, SGI196813, SGI196813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43681[] = {SGI196814, SGI196815, SGI196816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196817, SGI0, SGI0, SGI0, SGI196818, SGI196818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43682[] = {SGI196819, SGI0, SGI196820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43683[] = {SGI196821, SGI196822, SGI196823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196824, SGI0, SGI0, SGI0, SGI196825, SGI196825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43684[] = {SGI196826, SGI196827, SGI196828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196829, SGI0, SGI0, SGI0, SGI196830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43685[] = {SGI196831, SGI196832, SGI196833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196834, SGI0, SGI0, SGI0, SGI196835, SGI196835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43686[] = {SGI196836, SGI196837, SGI196838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196839, SGI0, SGI0, SGI0, SGI196840, SGI196840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43687[] = {SGI196841, SGI0, SGI196842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196843, SGI196844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43688[] = {SGI196845, SGI0, SGI196846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196847, SGI196848, SGI196848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43689[] = {SGI196849, SGI196850, SGI196851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196852, SGI0, SGI0, SGI0, SGI196853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43690[] = {SGI196854, SGI0, SGI196855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196856, SGI196857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43691[] = {SGI196858, SGI196859, SGI196860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196861, SGI0, SGI0, SGI0, SGI196862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43692[] = {SGI196863, SGI0, SGI196864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43693[] = {SGI196865, SGI0, SGI196866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196867, SGI196868, SGI196868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43694[] = {SGI196869, SGI0, SGI196870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196871, SGI196872, SGI196872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196873, SGI0, SGI0, NULL};
const Segment * const SG43695[] = {SGI196874, SGI0, SGI196875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196876, SGI196877, SGI196877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43696[] = {SGI196878, SGI196879, SGI196880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196881, SGI196881, SGI0, SGI0, SGI0, SGI196882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43697[] = {SGI196883, SGI196884, SGI196885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196886, SGI196886, SGI0, SGI0, SGI0, SGI196887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43698[] = {SGI196888, SGI0, SGI196889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196890, SGI196891, SGI196891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43699[] = {SGI196892, SGI196893, SGI196894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196895, SGI0, SGI0, SGI0, SGI196896, SGI196896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43700[] = {SGI196897, SGI0, SGI196898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43701[] = {SGI196899, SGI196900, SGI196901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196902, SGI196902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43702[] = {SGI196903, SGI196904, SGI196905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196906, SGI0, SGI0, SGI0, SGI196907, SGI196907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43703[] = {SGI196908, SGI196909, SGI196910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196911, SGI196911, SGI0, SGI0, SGI196912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43704[] = {SGI196913, SGI0, SGI196914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196918, SGI196919, SGI196919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43705[] = {SGI196915, SGI196916, SGI196917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196920, SGI0, SGI0, SGI0, SGI196921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43706[] = {SGI196922, SGI0, SGI196923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196924, SGI196925, SGI196925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43707[] = {SGI196926, SGI196927, SGI196928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196929, SGI0, SGI0, SGI0, SGI196930, SGI196930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43708[] = {SGI196931, SGI0, SGI196932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196933, SGI196934, SGI196934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43709[] = {SGI196935, SGI0, SGI196936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43710[] = {SGI196937, SGI0, SGI196938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196939, SGI196940, SGI196940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43711[] = {SGI196941, SGI0, SGI196942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196943, SGI196944, SGI196944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43712[] = {SGI196945, SGI0, SGI196946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196947, SGI196948, SGI196948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43713[] = {SGI196949, SGI0, SGI196950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196951, SGI196952, SGI196952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43714[] = {SGI196953, SGI0, SGI196955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196957, SGI196958, SGI196958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43715[] = {SGI196954, SGI0, SGI196956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196959, SGI196960, SGI196960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43716[] = {SGI196961, SGI0, SGI196962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196963, SGI196964, SGI196964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43717[] = {SGI196965, SGI0, SGI196966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196967, SGI196968, SGI196968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43718[] = {SGI196969, SGI196970, SGI196971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196972, SGI0, SGI0, SGI0, SGI196973, SGI196973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43719[] = {SGI196974, SGI196975, SGI196976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196977, SGI0, SGI0, SGI0, SGI196978, SGI196978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43720[] = {SGI196979, SGI0, SGI196980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43721[] = {SGI196981, SGI196982, SGI196983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196984, SGI0, SGI0, SGI0, SGI196985, SGI196985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43722[] = {SGI196986, SGI0, SGI196987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196988, SGI196989, SGI196989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196990, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43723[] = {SGI196991, SGI0, SGI196992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196993, SGI196994, SGI196994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43724[] = {SGI196995, SGI0, SGI196996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI196997, SGI196998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43725[] = {SGI196999, SGI0, SGI197000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197001, SGI197002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43726[] = {SGI197003, SGI0, SGI197004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197005, SGI197006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43727[] = {SGI197007, SGI0, SGI197009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197011, SGI197012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43728[] = {SGI197008, SGI0, SGI197010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197013, SGI197014, SGI197014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43729[] = {SGI197015, SGI0, SGI197016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197019, SGI197020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43730[] = {SGI197017, SGI0, SGI197018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197021, SGI197022, SGI197022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43731[] = {SGI197027, SGI197028, SGI197029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197030, SGI0, SGI0, SGI0, SGI197031, SGI197031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43732[] = {SGI197032, SGI197033, SGI197034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197035, SGI197035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43733[] = {SGI197036, SGI0, SGI197037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43734[] = {SGI0, SGI197038, SGI197039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43735[] = {SGI197040, SGI197041, SGI197042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197043, SGI0, SGI0, SGI0, SGI197044, SGI197044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43736[] = {SGI197045, SGI0, SGI197046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43737[] = {SGI197047, SGI0, SGI197048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197049, SGI197050, SGI197050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43738[] = {SGI197051, SGI0, SGI197052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197053, SGI197054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43739[] = {SGI197055, SGI0, SGI197056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197057, SGI197058, SGI197058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43740[] = {SGI197059, SGI0, SGI197060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197061, SGI197062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43741[] = {SGI197063, SGI0, SGI197064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197065, SGI197066, SGI197066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43742[] = {SGI197067, SGI0, SGI197068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197069, SGI197070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43743[] = {SGI197071, SGI197072, SGI197073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197074, SGI0, SGI0, SGI0, SGI197075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43744[] = {SGI197076, SGI0, SGI197077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197078, SGI197079, SGI197079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43745[] = {SGI197080, SGI197081, SGI197082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197083, SGI0, SGI0, SGI0, SGI197084, SGI197084, SGI0, SGI197087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43746[] = {SGI197085, SGI197086, SGI197088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197089, SGI0, SGI0, SGI0, SGI197090, SGI197090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43747[] = {SGI197091, SGI197092, SGI197093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197094, SGI0, SGI0, SGI0, SGI197095, SGI0, SGI0, SGI197096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43748[] = {SGI197097, SGI197098, SGI197099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197100, SGI0, SGI0, SGI0, SGI197101, SGI197101, SGI0, SGI197102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43749[] = {SGI197103, SGI197104, SGI197105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197106, SGI0, SGI0, SGI0, SGI197107, SGI197107, SGI0, SGI197108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43750[] = {SGI197109, SGI197110, SGI197111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197112, SGI0, SGI0, SGI0, SGI197113, SGI197113, SGI0, SGI197114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43751[] = {SGI197115, SGI197116, SGI197117, SGI197118, SGI197119, SGI197119, SGI0, NULL};
const Segment * const SG43752[] = {SGI197120, SGI0, SGI197121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197122, SGI197123, SGI197123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43753[] = {SGI197124, SGI0, SGI197125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197126, SGI197127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43754[] = {SGI197128, SGI0, SGI197129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197130, SGI197131, SGI197131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43755[] = {SGI197132, SGI0, SGI197133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197134, SGI197135, SGI197135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43756[] = {SGI197023, SGI0, SGI197024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197025, SGI197026, SGI197026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43757[] = {SGI0, SGI197136, SGI197137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197138, SGI0, SGI0, SGI0, SGI197139, SGI197139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43758[] = {SGI197140, SGI197141, SGI197142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197143, SGI0, SGI0, SGI0, SGI197144, SGI197144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43759[] = {SGI197145, SGI0, SGI197146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43760[] = {SGI197147, SGI197148, SGI197149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197150, SGI0, SGI0, SGI0, SGI197151, SGI197151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43761[] = {SGI197152, SGI0, SGI197153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197154, SGI197155, SGI197155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43762[] = {SGI197156, SGI197157, SGI197158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197159, SGI0, SGI0, SGI0, SGI197160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43763[] = {SGI197161, SGI0, SGI197162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197163, SGI197164, SGI197164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43764[] = {SGI0, SGI0, SGI197165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197170, SGI197171, SGI197171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43765[] = {SGI197167, SGI0, SGI197169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43766[] = {SGI197166, SGI0, SGI197168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197172, SGI197173, SGI197173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43767[] = {SGI197174, SGI197175, SGI197176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197177, SGI0, SGI0, SGI0, SGI197178, SGI197178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43768[] = {SGI197179, SGI0, SGI197180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197181, SGI197182, SGI197182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43769[] = {SGI197183, SGI0, SGI197184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197187, SGI0, SGI0, SGI0, SGI197188, SGI197188, SGI0, SGI0, SGI0, SGI0, SGI197190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43770[] = {SGI197185, SGI0, SGI197186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197189, SGI197191, SGI197191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43771[] = {SGI197192, SGI0, SGI197193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197194, SGI197195, SGI197195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43772[] = {SGI197196, SGI0, SGI197197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197198, SGI197199, SGI197199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43773[] = {SGI197200, SGI0, SGI197201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197202, SGI197203, SGI197203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43774[] = {SGI197204, SGI197205, SGI197206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197207, SGI0, SGI0, SGI0, SGI197208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43775[] = {SGI197210, SGI197211, SGI197212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197213, SGI0, SGI0, SGI0, SGI197214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43776[] = {SGI197216, SGI0, SGI197217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197219, SGI197221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43777[] = {SGI197218, SGI197220, SGI197222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197223, SGI0, SGI0, SGI0, SGI197224, SGI197224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43778[] = {SGI197225, SGI197227, SGI197228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197231, SGI0, SGI0, SGI0, SGI197233, SGI197233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43779[] = {SGI197226, SGI197229, SGI197230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197232, SGI0, SGI0, SGI0, SGI197234, SGI197234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43780[] = {SGI197235, SGI0, SGI197236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43781[] = {SGI197237, SGI0, SGI197238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197239, SGI197240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43782[] = {SGI197241, SGI0, SGI197242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197243, SGI197244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43783[] = {SGI197245, SGI197246, SGI197247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197248, SGI0, SGI0, SGI0, SGI197249, SGI197249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43784[] = {SGI197250, SGI0, SGI197251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197252, SGI197254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43785[] = {SGI197253, SGI0, SGI197255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197256, SGI197257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43786[] = {SGI197258, SGI0, SGI197259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197260, SGI197261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43787[] = {SGI197262, SGI0, SGI197263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197264, SGI197265, SGI197265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43788[] = {SGI197266, SGI197267, SGI197268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197269, SGI0, SGI0, SGI0, SGI197270, SGI197270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43789[] = {SGI197271, SGI197272, SGI197273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197274, SGI197274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43790[] = {SGI197275, SGI0, SGI197276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43791[] = {SGI197277, SGI197278, SGI197279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197280, SGI0, SGI0, SGI0, SGI197281, SGI197281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43792[] = {SGI197282, SGI0, SGI197283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197284, SGI197285, SGI197285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197286, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43793[] = {SGI197287, SGI0, SGI197288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197289, SGI197290, SGI197290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43794[] = {SGI197291, SGI0, SGI197292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197293, SGI197294, SGI197294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43795[] = {SGI197295, SGI0, SGI197296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197297, SGI197298, SGI197298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43796[] = {SGI197299, SGI0, SGI197300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197301, SGI197302, SGI197302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43797[] = {SGI197303, SGI0, SGI197304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197305, SGI197306, SGI197306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43798[] = {SGI197307, SGI0, SGI197308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197309, SGI197310, SGI197310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43799[] = {SGI197311, SGI0, SGI197312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197313, SGI197314, SGI197314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197315, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43800[] = {SGI197316, SGI0, SGI197317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197318, SGI197319, SGI197319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197320, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43801[] = {SGI197321, SGI197322, SGI197323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197324, SGI0, SGI0, SGI0, SGI197325, SGI0, SGI197326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43802[] = {SGI197327, SGI0, SGI197328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197329, SGI197330, SGI197330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43803[] = {SGI197331, SGI0, SGI197332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197333, SGI197334, SGI197334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43804[] = {SGI197335, SGI0, SGI197336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197337, SGI197338, SGI197338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43805[] = {SGI197339, SGI197340, SGI197341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197342, SGI0, SGI0, SGI0, SGI197343, SGI197343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197344, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43806[] = {SGI197345, SGI0, SGI197346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197347, SGI197348, SGI197348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43807[] = {SGI197349, SGI197350, SGI197351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197352, SGI0, SGI0, SGI0, SGI197353, SGI197353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43808[] = {SGI197354, SGI197355, SGI197356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197357, SGI0, SGI0, SGI0, SGI197358, SGI197358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43809[] = {SGI197359, SGI197360, SGI197361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197362, SGI197362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43810[] = {SGI197363, SGI197364, SGI197365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197366, SGI197366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43811[] = {SGI197367, SGI0, SGI197368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197369, SGI197370, SGI197370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43812[] = {SGI197371, SGI0, SGI197373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197375, SGI197376, SGI197376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43813[] = {SGI197372, SGI0, SGI197374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197377, SGI197378, SGI197378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43814[] = {SGI197379, SGI0, SGI197380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197381, SGI197382, SGI197382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43815[] = {SGI197383, SGI0, SGI197384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197385, SGI197386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43816[] = {SGI197387, SGI197388, SGI197389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197390, SGI0, SGI0, SGI0, SGI197391, SGI197391, SGI0, SGI197392, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43817[] = {SGI197393, SGI197394, SGI197395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197396, SGI0, SGI0, SGI0, SGI197397, SGI197397, SGI0, SGI197398, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43818[] = {SGI197399, SGI0, SGI197400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197401, SGI0, SGI0, SGI0, SGI197402, SGI197402, SGI0, SGI0, SGI0, SGI197403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43819[] = {SGI197404, SGI0, SGI197405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197408, SGI197409, SGI197409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43820[] = {SGI197406, SGI0, SGI197407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197410, SGI0, SGI0, SGI0, SGI197411, SGI197411, SGI0, SGI0, SGI0, SGI197412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43821[] = {SGI197413, SGI0, SGI197414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197415, SGI0, SGI0, SGI0, SGI197416, SGI197416, SGI0, SGI0, SGI0, SGI197417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43822[] = {SGI197418, SGI0, SGI197419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197422, SGI197423, SGI197423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43823[] = {SGI197420, SGI0, SGI197421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197424, SGI197425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43824[] = {SGI197426, SGI197427, SGI197428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197429, SGI0, SGI0, SGI0, SGI197430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43825[] = {SGI197431, SGI197432, SGI197433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197434, SGI0, SGI0, SGI0, SGI197435, SGI0, SGI0, SGI197436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43826[] = {SGI197437, SGI0, SGI197438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197439, SGI197440, SGI197440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43827[] = {SGI197441, SGI0, SGI197442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197443, SGI197444, SGI197444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43828[] = {SGI197445, SGI0, SGI197446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197447, SGI197448, SGI197448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197449, SGI0, SGI0, NULL};
const Segment * const SG43829[] = {SGI197450, SGI0, SGI197451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197452, SGI197453, SGI197453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43830[] = {SGI197454, SGI197455, SGI197456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197457, SGI0, SGI0, SGI197459, SGI197460, SGI197460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43831[] = {SGI197458, SGI0, SGI197461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197462, SGI197463, SGI197463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43832[] = {SGI197464, SGI197465, SGI197466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197467, SGI0, SGI0, SGI0, SGI197468, SGI197468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43833[] = {SGI197469, SGI0, SGI197470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197471, SGI197472, SGI197472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43834[] = {SGI197473, SGI0, SGI197474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197475, SGI197476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43835[] = {SGI197477, SGI197478, SGI197479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197480, SGI197480, SGI0, SGI0, SGI0, SGI197481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43836[] = {SGI197482, SGI0, SGI197483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197484, SGI197485, SGI197485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43837[] = {SGI197486, SGI0, SGI197487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197488, SGI197489, SGI197489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43838[] = {SGI197490, SGI0, SGI197491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197492, SGI197493, SGI197493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43839[] = {SGI197494, SGI0, SGI197495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197496, SGI197497, SGI197497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43840[] = {SGI197498, SGI197499, SGI197500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197501, SGI0, SGI0, SGI0, SGI197502, SGI197502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43841[] = {SGI197503, SGI0, SGI197504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197505, SGI197506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43842[] = {SGI197507, SGI0, SGI197508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197509, SGI197510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43843[] = {SGI197511, SGI197512, SGI197513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197514, SGI0, SGI0, SGI0, SGI197515, SGI197515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43844[] = {SGI197517, SGI0, SGI197518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197522, SGI197524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43845[] = {SGI197519, SGI197520, SGI197521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197523, SGI0, SGI0, SGI0, SGI197525, SGI197525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197529, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43846[] = {SGI197526, SGI197527, SGI197528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197530, SGI0, SGI0, SGI0, SGI197531, SGI0, SGI0, SGI197532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43847[] = {SGI197533, SGI197534, SGI197535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197536, SGI0, SGI0, SGI0, SGI197537, SGI197537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43848[] = {SGI197538, SGI197539, SGI197540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197541, SGI0, SGI0, SGI0, SGI197542, SGI197542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43849[] = {SGI197543, SGI197544, SGI197545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197546, SGI0, SGI0, SGI0, SGI197547, SGI197547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43850[] = {SGI197548, SGI0, SGI197549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197550, SGI197551, SGI197551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43851[] = {SGI197552, SGI0, SGI197553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197554, SGI197555, SGI197555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43852[] = {SGI197556, SGI197557, SGI197558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197559, SGI0, SGI0, SGI0, SGI197560, SGI197560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43853[] = {SGI197561, SGI197562, SGI197563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197564, SGI0, SGI0, SGI0, SGI197565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43854[] = {SGI197566, SGI0, SGI197567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197568, SGI197569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43855[] = {SGI197570, SGI197571, SGI197572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197573, SGI197573, SGI0, SGI0, SGI0, SGI0, SGI197574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43856[] = {SGI197575, SGI197576, SGI197577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197578, SGI0, SGI0, SGI0, SGI197579, SGI197579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43857[] = {SGI197580, SGI197581, SGI197582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197583, SGI0, SGI0, SGI0, SGI197584, SGI197584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43858[] = {SGI197585, SGI0, SGI197586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43859[] = {SGI197587, SGI197588, SGI197589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197590, SGI0, SGI0, SGI0, SGI197591, SGI197591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43860[] = {SGI197593, SGI197594, SGI197595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197596, SGI0, SGI0, SGI0, SGI197597, SGI197597, SGI0, SGI197598, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43861[] = {SGI197599, SGI0, SGI197600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197601, SGI197602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43862[] = {SGI197603, SGI0, SGI197604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197605, SGI197607, SGI197607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43863[] = {SGI197606, SGI197608, SGI197609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197610, SGI0, SGI0, SGI0, SGI197611, SGI197611, SGI0, SGI197612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43864[] = {SGI197613, SGI0, SGI197614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197615, SGI197616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43865[] = {SGI197617, SGI0, SGI197618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197619, SGI197620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43866[] = {SGI197621, SGI197622, SGI197623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197624, SGI0, SGI0, SGI0, SGI197625, SGI197625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43867[] = {SGI197627, SGI197628, SGI197630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197631, SGI0, SGI0, SGI0, SGI197633, SGI197633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43868[] = {SGI197626, SGI0, SGI197629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197632, SGI0, SGI0, SGI0, SGI197634, SGI197634, SGI0, SGI0, SGI0, SGI197635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43869[] = {SGI197636, SGI0, SGI197637, SGI0, SGI197638, SGI197638, SGI0, NULL};
const Segment * const SG43870[] = {SGI197639, SGI0, SGI197640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43871[] = {SGI197641, SGI0, SGI197642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197643, SGI197644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43872[] = {SGI197645, SGI0, SGI197646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197647, SGI197648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43873[] = {SGI197649, SGI197650, SGI197651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197652, SGI0, SGI0, SGI0, SGI197653, SGI0, SGI197654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43874[] = {SGI197655, SGI0, SGI197656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197659, SGI197660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43875[] = {SGI197657, SGI0, SGI197658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197661, SGI197662, SGI197662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43876[] = {SGI197663, SGI0, SGI197664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197665, SGI197666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43877[] = {SGI197667, SGI0, SGI197668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197669, SGI197670, SGI197670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43878[] = {SGI197671, SGI0, SGI197672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197673, SGI197674, SGI197674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43879[] = {SGI197675, SGI197676, SGI197677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197678, SGI0, SGI0, SGI0, SGI197679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43880[] = {SGI197680, SGI0, SGI197681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197682, SGI197683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43881[] = {SGI197684, SGI197685, SGI197686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197687, SGI0, SGI0, SGI0, SGI197688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43882[] = {SGI197689, SGI197690, SGI197691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197692, SGI0, SGI0, SGI0, SGI197693, SGI197693, SGI0, SGI197694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43883[] = {SGI197695, SGI197696, SGI197697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197698, SGI0, SGI0, SGI0, SGI197699, SGI0, SGI0, SGI197700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43884[] = {SGI197701, SGI197702, SGI197703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197706, SGI0, SGI0, SGI0, SGI197707, SGI0, SGI0, SGI197708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43885[] = {SGI197704, SGI0, SGI197705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197709, SGI197710, SGI197710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197711, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43886[] = {SGI197712, SGI197713, SGI197714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197715, SGI0, SGI0, SGI0, SGI197716, SGI197716, SGI0, SGI197717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43887[] = {SGI197718, SGI0, SGI197719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197720, SGI197721, SGI197721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197722, SGI0, SGI0, NULL};
const Segment * const SG43888[] = {SGI197723, SGI197724, SGI197725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197726, SGI0, SGI0, SGI0, SGI197727, SGI197727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43889[] = {SGI197729, SGI197730, SGI197731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197732, SGI0, SGI0, SGI0, SGI197733, SGI197733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43890[] = {SGI197734, SGI197735, SGI197736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197739, SGI197739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43891[] = {SGI197737, SGI0, SGI197738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197740, SGI197741, SGI197741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43892[] = {SGI197742, SGI197743, SGI197744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197745, SGI0, SGI0, SGI0, SGI197746, SGI197746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43893[] = {SGI197747, SGI0, SGI197748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197749, SGI197750, SGI197750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43894[] = {SGI197751, SGI0, SGI197752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197753, SGI197754, SGI197754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43895[] = {SGI197755, SGI197756, SGI197757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197758, SGI0, SGI0, SGI0, SGI197759, SGI197759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43896[] = {SGI197760, SGI197761, SGI197762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197763, SGI0, SGI0, SGI0, SGI197764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43897[] = {SGI197765, SGI197766, SGI197767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197771, SGI0, SGI0, SGI0, SGI197772, SGI0, SGI0, SGI197774, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43898[] = {SGI197768, SGI197769, SGI197770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197773, SGI0, SGI0, SGI0, SGI197775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43899[] = {SGI197776, SGI197777, SGI197778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197779, SGI0, SGI0, SGI0, SGI197780, SGI0, SGI0, SGI197781, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43900[] = {SGI197782, SGI0, SGI197783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197784, SGI197785, SGI197785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43901[] = {SGI197786, SGI0, SGI197787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197788, SGI197789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43902[] = {SGI197790, SGI0, SGI197791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197792, SGI197793, SGI197793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43903[] = {SGI197794, SGI0, SGI197795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197796, SGI197797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197798, SGI0, SGI0, NULL};
const Segment * const SG43904[] = {SGI197799, SGI0, SGI197800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197801, SGI197802, SGI197802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43905[] = {SGI197803, SGI0, SGI197804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197805, SGI197806, SGI197806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43906[] = {SGI197807, SGI0, SGI197808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197809, SGI197810, SGI197810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43907[] = {SGI197811, SGI197812, SGI197813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197814, SGI0, SGI0, SGI0, SGI197815, SGI197815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43908[] = {SGI197816, SGI197817, SGI197818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197819, SGI0, SGI0, SGI0, SGI197820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43909[] = {SGI197822, SGI197823, SGI197825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197826, SGI0, SGI0, SGI0, SGI197829, SGI197829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43910[] = {SGI197821, SGI0, SGI197824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197827, SGI197828, SGI197828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43911[] = {SGI197830, SGI197832, SGI197833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197835, SGI0, SGI0, SGI0, SGI197837, SGI197837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43912[] = {SGI197831, SGI0, SGI197834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197836, SGI197838, SGI197838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43913[] = {SGI197839, SGI0, SGI197840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197841, SGI197842, SGI197842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43914[] = {SGI197843, SGI0, SGI197844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197845, SGI197846, SGI197846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43915[] = {SGI197847, SGI197848, SGI197849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197850, SGI0, SGI0, SGI0, SGI197851, SGI197851, SGI0, SGI197852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43916[] = {SGI197853, SGI197854, SGI197855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197856, SGI0, SGI0, SGI0, SGI197857, SGI197857, SGI0, SGI197858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43917[] = {SGI197859, SGI197860, SGI197861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197862, SGI0, SGI0, SGI0, SGI197863, SGI197863, SGI0, SGI197864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43918[] = {SGI197865, SGI197866, SGI197867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197868, SGI0, SGI0, SGI0, SGI197869, SGI197869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43919[] = {SGI197870, SGI0, SGI197871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43920[] = {SGI197872, SGI0, SGI197873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197874, SGI197875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43921[] = {SGI197876, SGI197877, SGI197878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197879, SGI0, SGI0, SGI0, SGI197880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43922[] = {SGI197881, SGI0, SGI197882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43923[] = {SGI197883, SGI0, SGI197884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197885, SGI197886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43924[] = {SGI197887, SGI0, SGI197888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197889, SGI197890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43925[] = {SGI197891, SGI197892, SGI197893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197894, SGI0, SGI0, SGI0, SGI197895, SGI197895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43926[] = {SGI197896, SGI0, SGI197897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197898, SGI197899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43927[] = {SGI197900, SGI197901, SGI197902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197903, SGI197903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43928[] = {SGI197904, SGI197905, SGI197906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197907, SGI0, SGI0, SGI0, SGI197908, SGI197908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43929[] = {SGI197909, SGI0, SGI197910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43930[] = {SGI197911, SGI197912, SGI197913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197914, SGI0, SGI0, SGI0, SGI197915, SGI197915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43931[] = {SGI197916, SGI197917, SGI197918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197919, SGI0, SGI0, SGI0, SGI197920, SGI197920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43932[] = {SGI197921, SGI0, SGI197922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197923, SGI197924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43933[] = {SGI197925, SGI197926, SGI197927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197928, SGI197928, SGI0, SGI0, SGI197929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43934[] = {SGI197930, SGI0, SGI197931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197932, SGI197933, SGI197933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43935[] = {SGI197934, SGI0, SGI197935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197936, SGI197937, SGI197937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43936[] = {SGI197938, SGI0, SGI197939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197940, SGI197941, SGI197941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43937[] = {SGI197942, SGI0, SGI197943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197944, SGI197945, SGI197945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43938[] = {SGI197946, SGI0, SGI197947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197948, SGI197949, SGI197949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43939[] = {SGI197950, SGI197951, SGI197952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197953, SGI197953, SGI0, SGI0, SGI0, SGI197954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43940[] = {SGI197955, SGI0, SGI197956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197957, SGI197958, SGI197958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43941[] = {SGI197959, SGI197960, SGI197961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197962, SGI0, SGI0, SGI0, SGI197963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43942[] = {SGI197964, SGI0, SGI197965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197966, SGI197967, SGI197967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43943[] = {SGI197968, SGI0, SGI197969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197970, SGI197971, SGI197971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43944[] = {SGI197972, SGI0, SGI197973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197974, SGI197975, SGI197975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43945[] = {SGI197976, SGI197977, SGI197978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197979, SGI0, SGI0, SGI0, SGI197980, SGI197980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197981, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43946[] = {SGI197982, SGI197983, SGI197984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197987, SGI0, SGI0, SGI0, SGI197988, SGI197988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197991, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43947[] = {SGI197985, SGI0, SGI197986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197989, SGI197990, SGI197990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43948[] = {SGI197992, SGI197993, SGI197994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197995, SGI0, SGI0, SGI0, SGI197996, SGI197996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI197997, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43949[] = {SGI197998, SGI197999, SGI198000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198001, SGI0, SGI0, SGI0, SGI198002, SGI198002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198003, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43950[] = {SGI198004, SGI198005, SGI198006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198007, SGI0, SGI0, SGI0, SGI198008, SGI198008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43951[] = {SGI198009, SGI0, SGI198010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43952[] = {SGI198011, SGI198012, SGI198013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198014, SGI0, SGI0, SGI0, SGI198015, SGI198015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43953[] = {SGI198016, SGI198017, SGI198018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198019, SGI0, SGI0, SGI0, SGI198020, SGI198020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43954[] = {SGI198021, SGI0, SGI198022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198023, SGI198024, SGI198024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43955[] = {SGI198025, SGI198026, SGI198027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198028, SGI198028, SGI0, SGI0, SGI0, SGI198029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43956[] = {SGI198030, SGI198031, SGI198032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198033, SGI0, SGI0, SGI0, SGI198034, SGI0, SGI0, SGI198036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43957[] = {SGI198035, SGI198037, SGI198038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198042, SGI0, SGI0, SGI0, SGI198043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43958[] = {SGI198039, SGI198040, SGI198041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198044, SGI0, SGI198045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43959[] = {SGI198046, SGI198047, SGI198048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198049, SGI0, SGI0, SGI0, SGI198050, SGI198050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198051, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43960[] = {SGI198052, SGI198053, SGI198054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198055, SGI0, SGI0, SGI0, SGI198056, SGI198056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198057, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43961[] = {SGI198058, SGI198059, SGI198060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198061, SGI0, SGI0, SGI0, SGI198062, SGI198062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198063, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43962[] = {SGI198064, SGI198065, SGI198066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198067, SGI0, SGI0, SGI0, SGI198068, SGI198068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198069, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43963[] = {SGI198070, SGI198071, SGI198072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198073, SGI0, SGI0, SGI0, SGI198074, SGI198074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198075, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43964[] = {SGI198076, SGI0, SGI198077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198078, SGI198079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43965[] = {SGI198080, SGI0, SGI198081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198082, SGI198083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43966[] = {SGI198084, SGI198085, SGI198086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198087, SGI0, SGI0, SGI0, SGI198088, SGI198088, SGI0, SGI198089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43967[] = {SGI198090, SGI198091, SGI198092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198093, SGI0, SGI0, SGI0, SGI198095, SGI198095, SGI0, SGI198097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43968[] = {SGI198094, SGI0, SGI198096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198098, SGI198099, SGI198099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43969[] = {SGI198100, SGI0, SGI198101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198102, SGI198103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43970[] = {SGI198104, SGI0, SGI198105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198106, SGI198107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43971[] = {SGI198108, SGI0, SGI198109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198110, SGI198112, SGI198112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43972[] = {SGI198111, SGI198113, SGI198114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198115, SGI0, SGI0, SGI0, SGI198116, SGI198116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43973[] = {SGI198117, SGI0, SGI198118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198119, SGI198120, SGI198120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43974[] = {SGI198121, SGI0, SGI198122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198123, SGI198124, SGI198124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43975[] = {SGI0, SGI198125, SGI0, SGI198126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43976[] = {SGI198127, SGI0, SGI198128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198129, SGI198130, SGI198130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43977[] = {SGI0, SGI198131, SGI0, SGI198132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198133, SGI0, SGI0, SGI0, SGI198134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43978[] = {SGI198135, SGI198136, SGI198137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198138, SGI0, SGI0, SGI0, SGI198139, SGI198139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43979[] = {SGI198140, SGI198141, SGI198142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198143, SGI0, SGI0, SGI0, SGI198144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43980[] = {SGI198145, SGI0, SGI198146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198147, SGI198148, SGI198148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43981[] = {SGI198149, SGI0, SGI198150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198151, SGI198152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43982[] = {SGI198153, SGI0, SGI198154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198155, SGI198156, SGI198156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43983[] = {SGI198157, SGI0, SGI198158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198159, SGI198160, SGI198160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198161, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43984[] = {SGI198162, SGI0, SGI198163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198164, SGI198165, SGI198165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198166, SGI0, SGI0, NULL};
const Segment * const SG43985[] = {SGI198167, SGI0, SGI198168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198169, SGI198170, SGI198170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198171, SGI0, SGI0, NULL};
const Segment * const SG43986[] = {SGI198172, SGI0, SGI198173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198174, SGI198175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43987[] = {SGI198176, SGI0, SGI198177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198178, SGI198179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43988[] = {SGI198180, SGI198181, SGI198182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198183, SGI0, SGI0, SGI0, SGI198184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43989[] = {SGI198186, SGI198187, SGI198188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198189, SGI0, SGI0, SGI0, SGI198190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43990[] = {SGI198192, SGI198193, SGI198194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198195, SGI0, SGI0, SGI0, SGI198196, SGI198196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43991[] = {SGI198197, SGI0, SGI198198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43992[] = {SGI198199, SGI198200, SGI198201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198202, SGI0, SGI0, SGI0, SGI198203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43993[] = {SGI198204, SGI198205, SGI198206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198207, SGI0, SGI0, SGI0, SGI198210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43994[] = {SGI198208, SGI198209, SGI198211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198212, SGI0, SGI0, SGI0, SGI198213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43995[] = {SGI198214, SGI0, SGI198215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198216, SGI198217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43996[] = {SGI198218, SGI0, SGI198220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198222, SGI198223, SGI198223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43997[] = {SGI198219, SGI0, SGI198221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198224, SGI198225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43998[] = {SGI198226, SGI198227, SGI198228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198229, SGI0, SGI0, SGI0, SGI198230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG43999[] = {SGI198231, SGI0, SGI198232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198233, SGI198234, SGI198234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44000[] = {SGI198235, SGI198236, SGI198237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198238, SGI0, SGI0, SGI0, SGI198239, SGI198239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198240, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44001[] = {SGI198241, SGI198242, SGI198243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198244, SGI0, SGI0, SGI0, SGI198245, SGI198245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44002[] = {SGI198246, SGI0, SGI198247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198248, SGI198249, SGI198249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44003[] = {SGI198250, SGI0, SGI198251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198252, SGI198253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44004[] = {SGI198254, SGI0, SGI198255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198256, SGI198257, SGI198257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44005[] = {SGI198258, SGI198259, SGI198260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198261, SGI0, SGI0, SGI0, SGI198262, SGI198262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198263, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44006[] = {SGI198264, SGI198265, SGI198266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198267, SGI0, SGI0, SGI0, SGI198268, SGI198268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44007[] = {SGI198269, SGI0, SGI198270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44008[] = {SGI198271, SGI198272, SGI198273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198274, SGI0, SGI0, SGI0, SGI198275, SGI198275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44009[] = {SGI198277, SGI0, SGI198278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198279, SGI198280, SGI198280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44010[] = {SGI198281, SGI0, SGI198282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198283, SGI198284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44011[] = {SGI198285, SGI0, SGI198286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198287, SGI0, SGI0, SGI0, SGI198288, SGI198288, SGI0, SGI0, SGI0, SGI198289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44012[] = {SGI198290, SGI198291, SGI198292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198293, SGI0, SGI0, SGI0, SGI198294, SGI198294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44013[] = {SGI198295, SGI198296, SGI198297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198298, SGI0, SGI0, SGI0, SGI198299, SGI198299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44014[] = {SGI198300, SGI0, SGI198301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198302, SGI198303, SGI198303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44015[] = {SGI198304, SGI0, SGI198305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198306, SGI198307, SGI198307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44016[] = {SGI198308, SGI0, SGI198309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198310, SGI198311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44017[] = {SGI198312, SGI0, SGI198313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198314, SGI198315, SGI198315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44018[] = {SGI198316, SGI198317, SGI198318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198319, SGI0, SGI0, SGI0, SGI198320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44019[] = {SGI198321, SGI198322, SGI198323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198324, SGI0, SGI0, SGI0, SGI198325, SGI0, SGI0, SGI198326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44020[] = {SGI198327, SGI198328, SGI198329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198330, SGI0, SGI0, SGI0, SGI198331, SGI198331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44021[] = {SGI198332, SGI198333, SGI198334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198335, SGI0, SGI0, SGI0, SGI198336, SGI198336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44022[] = {SGI198337, SGI0, SGI198338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44023[] = {SGI198339, SGI198340, SGI198341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198342, SGI0, SGI0, SGI0, SGI198343, SGI198343, SGI0, SGI0, SGI198344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44024[] = {SGI198345, SGI0, SGI198346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198347, SGI198348, SGI198348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44025[] = {SGI198349, SGI198350, SGI198351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198352, SGI0, SGI0, SGI0, SGI198353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44026[] = {SGI198354, SGI198355, SGI198356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198357, SGI0, SGI0, SGI0, SGI198358, SGI198358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44027[] = {SGI198359, SGI198360, SGI198361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198362, SGI0, SGI0, SGI0, SGI198363, SGI198363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44028[] = {SGI198364, SGI0, SGI198365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198366, SGI198367, SGI198367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44029[] = {SGI198368, SGI198369, SGI198370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198371, SGI198371, SGI0, SGI0, SGI198372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44030[] = {SGI198373, SGI198374, SGI198375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198376, SGI198376, SGI0, SGI0, SGI0, SGI198377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44031[] = {SGI198378, SGI0, SGI198379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198380, SGI198381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198382, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44032[] = {SGI198383, SGI198384, SGI198385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198386, SGI0, SGI0, SGI0, SGI198387, SGI198387, SGI0, SGI198388, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44033[] = {SGI198389, SGI198390, SGI198391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198392, SGI198392, SGI0, SGI0, SGI0, SGI198393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44034[] = {SGI198394, SGI0, SGI198395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198396, SGI198397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44035[] = {SGI198398, SGI0, SGI198399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198400, SGI198401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44036[] = {SGI198402, SGI0, SGI198403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198404, SGI198406, SGI198406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44037[] = {SGI198405, SGI198407, SGI198408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198409, SGI0, SGI0, SGI0, SGI198410, SGI198410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44038[] = {SGI198411, SGI198412, SGI198413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198414, SGI0, SGI0, SGI0, SGI198415, SGI198415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44039[] = {SGI198416, SGI198417, SGI198418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198419, SGI0, SGI0, SGI0, SGI198420, SGI198420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44040[] = {SGI198421, SGI0, SGI198422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198423, SGI198424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44041[] = {SGI198425, SGI0, SGI198426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44042[] = {SGI198427, SGI198428, SGI198429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198430, SGI0, SGI0, SGI0, SGI198431, SGI198431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44043[] = {SGI198432, SGI198433, SGI198435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198438, SGI0, SGI0, SGI0, SGI198439, SGI198439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44044[] = {SGI198434, SGI198436, SGI198437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198440, SGI0, SGI0, SGI0, SGI198441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44045[] = {SGI198442, SGI198443, SGI198444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198445, SGI0, SGI0, SGI0, SGI198446, SGI198446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44046[] = {SGI198447, SGI0, SGI198448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198449, SGI198450, SGI198450, SGI0, SGI0, NULL};
const Segment * const SG44047[] = {SGI198451, SGI198452, SGI198453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198454, SGI0, SGI0, SGI0, SGI198455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44048[] = {SGI198456, SGI0, SGI198457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198458, SGI198459, SGI198459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44049[] = {SGI198460, SGI198461, SGI198462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198463, SGI0, SGI0, SGI0, SGI198464, SGI198464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44050[] = {SGI198465, SGI0, SGI198466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198467, SGI198468, SGI198468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44051[] = {SGI198469, SGI0, SGI198470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198471, SGI198472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44052[] = {SGI198473, SGI0, SGI198474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198475, SGI198476, SGI198476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44053[] = {SGI198477, SGI0, SGI198478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198479, SGI198480, SGI198480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44054[] = {SGI198481, SGI0, SGI198482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198483, SGI198484, SGI198484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44055[] = {SGI198485, SGI0, SGI198486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198487, SGI198488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44056[] = {SGI198489, SGI0, SGI198490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198491, SGI198492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44057[] = {SGI198493, SGI0, SGI198494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198495, SGI198496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198497, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44058[] = {SGI198498, SGI0, SGI198499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198500, SGI198501, SGI198501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44059[] = {SGI198502, SGI0, SGI198503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198504, SGI198505, SGI198505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44060[] = {SGI198506, SGI0, SGI198507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198508, SGI198509, SGI198509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44061[] = {SGI198510, SGI0, SGI198511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198512, SGI198513, SGI198513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44062[] = {SGI198514, SGI198515, SGI198516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198517, SGI0, SGI0, SGI0, SGI198518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44063[] = {SGI198519, SGI0, SGI198520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198521, SGI198522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44064[] = {SGI198523, SGI0, SGI198524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198525, SGI198526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44065[] = {SGI198527, SGI0, SGI198528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198529, SGI198530, SGI198530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44066[] = {SGI198531, SGI0, SGI198532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198533, SGI198534, SGI198534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44067[] = {SGI198535, SGI0, SGI198536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198537, SGI198538, SGI198538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44068[] = {SGI198539, SGI198540, SGI198541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198542, SGI0, SGI0, SGI0, SGI198543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44069[] = {SGI198544, SGI0, SGI198545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198546, SGI198547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44070[] = {SGI198548, SGI0, SGI198549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198550, SGI198551, SGI198551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44071[] = {SGI198552, SGI0, SGI198553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198554, SGI0, SGI0, SGI0, SGI198555, SGI198555, SGI0, SGI0, SGI0, SGI198556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44072[] = {SGI198557, SGI0, SGI198558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198559, SGI198560, SGI198560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44073[] = {SGI198561, SGI0, SGI198562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198563, SGI198564, SGI198564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44074[] = {SGI198565, SGI0, SGI198566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198567, SGI198568, SGI198568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44075[] = {SGI198569, SGI0, SGI198570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198571, SGI198572, SGI198572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44076[] = {SGI198573, SGI198574, SGI198575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198576, SGI0, SGI0, SGI0, SGI198577, SGI198577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44077[] = {SGI198578, SGI0, SGI198579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198580, SGI198581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44078[] = {SGI198582, SGI0, SGI198583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198584, SGI198585, SGI198585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44079[] = {SGI198586, SGI0, SGI198587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198588, SGI198589, SGI198589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44080[] = {SGI198590, SGI0, SGI198591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198592, SGI198593, SGI198593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44081[] = {SGI198594, SGI0, SGI198595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198596, SGI198597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44082[] = {SGI198598, SGI198599, SGI198600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198601, SGI0, SGI0, SGI0, SGI198602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44083[] = {SGI198603, SGI0, SGI198604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44084[] = {SGI198605, SGI198606, SGI198607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198611, SGI0, SGI0, SGI0, SGI198612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44085[] = {SGI198608, SGI198609, SGI198610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198613, SGI198613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44086[] = {SGI198614, SGI198616, SGI198617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198620, SGI0, SGI0, SGI0, SGI198622, SGI198622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44087[] = {SGI198615, SGI198618, SGI198619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198621, SGI0, SGI0, SGI0, SGI198623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44088[] = {SGI198624, SGI198625, SGI198626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198627, SGI0, SGI0, SGI0, SGI198628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44089[] = {SGI198629, SGI198630, SGI198631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198632, SGI198632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44090[] = {SGI198633, SGI0, SGI198634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44091[] = {SGI0, SGI198635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44092[] = {SGI198636, SGI0, SGI198637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198641, SGI198642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44093[] = {SGI198638, SGI198639, SGI198640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198643, SGI0, SGI0, SGI0, SGI198644, SGI198644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44094[] = {SGI198645, SGI0, SGI198646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44095[] = {SGI198647, SGI0, SGI198648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198649, SGI198650, SGI198650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44096[] = {SGI198651, SGI198652, SGI198653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198654, SGI0, SGI0, SGI0, SGI198655, SGI198655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44097[] = {SGI198656, SGI198657, SGI198658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198659, SGI0, SGI0, SGI0, SGI198660, SGI198660, SGI0, SGI198661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44098[] = {SGI198662, SGI0, SGI198663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44099[] = {SGI198664, SGI198665, SGI198666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198667, SGI0, SGI0, SGI0, SGI198668, SGI198668, SGI0, SGI198669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44100[] = {SGI198670, SGI0, SGI198671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198672, SGI198673, SGI198673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44101[] = {SGI198674, SGI0, SGI198675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198676, SGI198677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44102[] = {SGI198678, SGI198679, SGI198680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198681, SGI0, SGI0, SGI0, SGI198682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44103[] = {SGI198683, SGI0, SGI198684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198688, SGI198689, SGI198689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44104[] = {SGI198685, SGI198686, SGI198687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198690, SGI0, SGI0, SGI0, SGI198691, SGI198691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44105[] = {SGI198692, SGI198693, SGI198694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198695, SGI198695, SGI0, SGI0, SGI198696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44106[] = {SGI198697, SGI198698, SGI198699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198700, SGI0, SGI0, SGI0, SGI198701, SGI0, SGI0, SGI198702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44107[] = {SGI198703, SGI198704, SGI198705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198706, SGI0, SGI0, SGI0, SGI198707, SGI198707, SGI0, SGI198708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44108[] = {SGI198709, SGI198710, SGI198711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198712, SGI0, SGI0, SGI0, SGI198713, SGI0, SGI0, SGI198714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44109[] = {SGI198715, SGI198716, SGI198717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198718, SGI0, SGI0, SGI0, SGI198719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44110[] = {SGI198720, SGI0, SGI198721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198722, SGI0, SGI0, SGI0, SGI198723, SGI198723, SGI0, SGI0, SGI0, SGI0, SGI198724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44111[] = {SGI198725, SGI198726, SGI198727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198728, SGI0, SGI0, SGI0, SGI198729, SGI198730, SGI0, SGI0, SGI0, SGI0, SGI198731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44112[] = {SGI198732, SGI0, SGI198733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198734, SGI198735, SGI198735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44113[] = {SGI198736, SGI198737, SGI198738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198739, SGI0, SGI0, SGI0, SGI198740, SGI198740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44114[] = {SGI198743, SGI0, SGI198744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44115[] = {SGI198741, SGI0, SGI198742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198745, SGI198746, SGI198746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44116[] = {SGI198747, SGI198748, SGI198749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198750, SGI0, SGI0, SGI0, SGI198751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44117[] = {SGI198753, SGI198754, SGI198755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198756, SGI0, SGI0, SGI0, SGI198757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44118[] = {SGI198759, SGI0, SGI198760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198761, SGI198762, SGI198762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44119[] = {SGI198763, SGI198764, SGI198765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198766, SGI198766, SGI0, SGI0, SGI198767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44120[] = {SGI198768, SGI198769, SGI198770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198771, SGI0, SGI0, SGI0, SGI198772, SGI198772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44121[] = {SGI198773, SGI0, SGI198774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44122[] = {SGI198775, SGI198776, SGI198777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198778, SGI0, SGI0, SGI0, SGI198779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44123[] = {SGI198780, SGI198781, SGI198782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198783, SGI0, SGI0, SGI0, SGI198784, SGI198784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44124[] = {SGI198785, SGI0, SGI198786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44125[] = {SGI198787, SGI198788, SGI198789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198790, SGI0, SGI0, SGI0, SGI198791, SGI198791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44126[] = {SGI198792, SGI198793, SGI198794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198798, SGI0, SGI0, SGI0, SGI198799, SGI198799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198801, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44127[] = {SGI198795, SGI198796, SGI198797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198800, SGI0, SGI0, SGI0, SGI198802, SGI198802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44128[] = {SGI198803, SGI0, SGI198804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198805, SGI198806, SGI198806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44129[] = {SGI198807, SGI0, SGI198808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198809, SGI198810, SGI198810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198811, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44130[] = {SGI198812, SGI0, SGI198813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198814, SGI198815, SGI198815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198816, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44131[] = {SGI198817, SGI198818, SGI198819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198820, SGI0, SGI0, SGI0, SGI198821, SGI0, SGI0, SGI198822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44132[] = {SGI198823, SGI0, SGI198824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198825, SGI198826, SGI198826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44133[] = {SGI198827, SGI0, SGI198828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198829, SGI198830, SGI198830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44134[] = {SGI198831, SGI0, SGI198832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198833, SGI198834, SGI198834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44135[] = {SGI198835, SGI0, SGI198836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198837, SGI198838, SGI198838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44136[] = {SGI198839, SGI0, SGI198840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198841, SGI198842, SGI198842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44137[] = {SGI0, SGI198843, SGI198844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198845, SGI0, SGI0, SGI0, SGI198846, SGI198846, SGI0, SGI0, NULL};
const Segment * const SG44138[] = {SGI198847, SGI198848, SGI198849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198850, SGI0, SGI198851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44139[] = {SGI198852, SGI0, SGI198853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198854, SGI198855, SGI198855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44140[] = {SGI198856, SGI198857, SGI198858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198862, SGI0, SGI0, SGI0, SGI198863, SGI198863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44141[] = {SGI198859, SGI198860, SGI198861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198864, SGI0, SGI198865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44142[] = {SGI198866, SGI0, SGI198867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44143[] = {SGI198868, SGI198869, SGI198870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198871, SGI0, SGI198872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44144[] = {SGI198873, SGI198874, SGI198875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198876, SGI0, SGI0, SGI0, SGI198877, SGI198877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44145[] = {SGI198878, SGI198879, SGI198880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198881, SGI0, SGI198882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44146[] = {SGI198883, SGI0, SGI198884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198885, SGI198886, SGI198886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198887, SGI0, SGI0, NULL};
const Segment * const SG44147[] = {SGI198889, SGI0, SGI198892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198894, SGI198895, SGI198895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44148[] = {SGI198888, SGI198890, SGI198891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198893, SGI0, SGI0, SGI0, SGI198896, SGI198896, SGI0, SGI198897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44149[] = {SGI198898, SGI0, SGI198899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198900, SGI198901, SGI198901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44150[] = {SGI198902, SGI0, SGI198903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198904, SGI198905, SGI198905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44151[] = {SGI198906, SGI0, SGI198907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198908, SGI198909, SGI198909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44152[] = {SGI198910, SGI198911, SGI198912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198913, SGI0, SGI0, SGI0, SGI198914, SGI198914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44153[] = {SGI198915, SGI198916, SGI198917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198918, SGI0, SGI0, SGI0, SGI198919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44154[] = {SGI0, SGI198920, SGI198921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198922, SGI0, SGI0, SGI0, SGI198923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44155[] = {SGI198924, SGI0, SGI198925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198926, SGI198927, SGI198927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44156[] = {SGI198928, SGI0, SGI198929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198930, SGI198931, SGI198931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44157[] = {SGI198932, SGI198933, SGI198934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198935, SGI0, SGI0, SGI0, SGI198936, SGI198936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44158[] = {SGI198937, SGI198938, SGI198939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198940, SGI0, SGI0, SGI0, SGI198941, SGI198941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44159[] = {SGI198942, SGI198943, SGI198944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198945, SGI198945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44160[] = {SGI198946, SGI198947, SGI198948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198949, SGI0, SGI0, SGI0, SGI198950, SGI198950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44161[] = {SGI198951, SGI0, SGI198952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198953, SGI198954, SGI198954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44162[] = {SGI198955, SGI0, SGI198956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198957, SGI198958, SGI198958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44163[] = {SGI198959, SGI0, SGI198960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198961, SGI198962, SGI198962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44164[] = {SGI198963, SGI0, SGI198964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198965, SGI198966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44165[] = {SGI198967, SGI0, SGI198968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198969, SGI198970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44166[] = {SGI198971, SGI198972, SGI198973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198974, SGI0, SGI0, SGI0, SGI198975, SGI198975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44167[] = {SGI198976, SGI0, SGI198977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44168[] = {SGI198978, SGI198979, SGI198980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198981, SGI0, SGI0, SGI0, SGI198982, SGI198982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44169[] = {SGI198983, SGI198984, SGI198985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198986, SGI0, SGI0, SGI0, SGI198987, SGI198987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44170[] = {SGI198988, SGI198989, SGI198990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198991, SGI0, SGI0, SGI0, SGI198992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44171[] = {SGI198993, SGI0, SGI198994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198995, SGI198996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44172[] = {SGI198997, SGI0, SGI198998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI198999, SGI199000, SGI199000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44173[] = {SGI199001, SGI199002, SGI199003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199010, SGI0, SGI0, SGI0, SGI199012, SGI0, SGI199015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44174[] = {SGI199004, SGI199006, SGI199008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199011, SGI0, SGI0, SGI0, SGI199014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44175[] = {SGI199005, SGI199007, SGI199009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199013, SGI0, SGI0, SGI0, SGI199016, SGI199016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44176[] = {SGI199017, SGI0, SGI199018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199019, SGI199020, SGI199020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44177[] = {SGI199021, SGI199022, SGI199023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199024, SGI199024, SGI0, SGI0, SGI199025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44178[] = {SGI199026, SGI0, SGI199027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199028, SGI199029, SGI199029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44179[] = {SGI199030, SGI0, SGI199031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199032, SGI199033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44180[] = {SGI199034, SGI0, SGI199035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199036, SGI199037, SGI199037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44181[] = {SGI199038, SGI0, SGI199039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199040, SGI199041, SGI199041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44182[] = {SGI199042, SGI0, SGI199043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199044, SGI199045, SGI199045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44183[] = {SGI199046, SGI199047, SGI199048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199049, SGI0, SGI0, SGI0, SGI199050, SGI199050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44184[] = {SGI199051, SGI199052, SGI199053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199054, SGI199054, SGI0, SGI0, SGI0, SGI199055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44185[] = {SGI199056, SGI0, SGI199057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199058, SGI199059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44186[] = {SGI199060, SGI199061, SGI199062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199063, SGI199063, SGI0, SGI0, SGI199064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44187[] = {SGI199065, SGI0, SGI199066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199067, SGI199068, SGI199068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44188[] = {SGI199069, SGI0, SGI199070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199071, SGI199072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44189[] = {SGI199073, SGI0, SGI199074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199075, SGI199076, SGI199076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44190[] = {SGI199077, SGI0, SGI199078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199079, SGI199080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44191[] = {SGI199081, SGI0, SGI199082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199083, SGI199084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44192[] = {SGI199085, SGI0, SGI199086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199087, SGI199088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44193[] = {SGI199089, SGI0, SGI199090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199091, SGI199092, SGI199092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44194[] = {SGI199093, SGI0, SGI199094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199095, SGI199096, SGI199096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44195[] = {SGI199097, SGI0, SGI199098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199101, SGI199102, SGI199102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44196[] = {SGI199099, SGI0, SGI199100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199103, SGI199104, SGI199104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44197[] = {SGI199105, SGI0, SGI199106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199107, SGI199108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44198[] = {SGI199109, SGI0, SGI199110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199111, SGI199112, SGI199112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44199[] = {SGI199113, SGI0, SGI199114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199115, SGI199116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44200[] = {SGI199117, SGI0, SGI199118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199119, SGI199120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44201[] = {SGI199121, SGI199122, SGI199123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199124, SGI0, SGI0, SGI0, SGI199125, SGI199125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44202[] = {SGI199126, SGI199127, SGI199128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199131, SGI0, SGI0, SGI0, SGI199132, SGI199132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44203[] = {SGI199133, SGI199134, SGI199135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199136, SGI0, SGI0, SGI0, SGI199137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44204[] = {SGI199138, SGI0, SGI199139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199140, SGI199141, SGI199141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44205[] = {SGI199142, SGI0, SGI199143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199144, SGI199145, SGI199145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44206[] = {SGI199146, SGI0, SGI199147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199148, SGI199149, SGI199149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44207[] = {SGI199150, SGI199151, SGI199152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199153, SGI0, SGI0, SGI0, SGI199154, SGI0, SGI0, SGI199155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44208[] = {SGI199156, SGI0, SGI199157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199158, SGI199159, SGI199159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44209[] = {SGI199160, SGI0, SGI199161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199162, SGI199163, SGI199163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44210[] = {SGI199164, SGI199165, SGI199166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199167, SGI0, SGI0, SGI0, SGI199168, SGI199168, SGI0, SGI199169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44211[] = {SGI199170, SGI0, SGI199171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199172, SGI199173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44212[] = {SGI199174, SGI0, SGI199175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199176, SGI199177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44213[] = {SGI199178, SGI0, SGI199179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199181, SGI199182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44214[] = {SGI199180, SGI0, SGI199183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199184, SGI199185, SGI199185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44215[] = {SGI199186, SGI0, SGI199187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199188, SGI199190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44216[] = {SGI199189, SGI0, SGI199191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199192, SGI199193, SGI199193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44217[] = {SGI199194, SGI0, SGI199195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199196, SGI199197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44218[] = {SGI199198, SGI0, SGI199200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199203, SGI199205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44219[] = {SGI199199, SGI199201, SGI199202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199204, SGI0, SGI0, SGI0, SGI199206, SGI199206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44220[] = {SGI199207, SGI199208, SGI199209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199210, SGI199210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44221[] = {SGI199211, SGI199212, SGI199213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199214, SGI0, SGI0, SGI0, SGI199215, SGI199215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44222[] = {SGI199216, SGI0, SGI199217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44223[] = {SGI199218, SGI0, SGI199219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199220, SGI199221, SGI199221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44224[] = {SGI199222, SGI0, SGI199223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199224, SGI199225, SGI199225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44225[] = {SGI199226, SGI0, SGI199227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199228, SGI199229, SGI199229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44226[] = {SGI199230, SGI199231, SGI199232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199233, SGI199233, SGI0, SGI0, SGI199234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44227[] = {SGI199235, SGI0, SGI199236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199237, SGI199238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44228[] = {SGI199239, SGI0, SGI199240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199241, SGI199242, SGI199242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44229[] = {SGI199243, SGI0, SGI199244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199245, SGI199246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44230[] = {SGI199247, SGI0, SGI199248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199249, SGI199250, SGI199250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44231[] = {SGI199129, SGI0, SGI199130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199251, SGI199252, SGI199252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44232[] = {SGI199253, SGI0, SGI199254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199255, SGI199256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44233[] = {SGI199257, SGI0, SGI199258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199259, SGI199260, SGI199260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44234[] = {SGI199261, SGI0, SGI199262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199263, SGI199264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199265, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44235[] = {SGI199266, SGI199267, SGI199268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199269, SGI0, SGI0, SGI0, SGI199270, SGI199270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44236[] = {SGI199272, SGI199273, SGI199274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199275, SGI0, SGI0, SGI0, SGI199276, SGI199276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44237[] = {SGI199278, SGI199279, SGI199280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199281, SGI0, SGI0, SGI0, SGI199282, SGI199282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44238[] = {SGI199284, SGI199285, SGI199286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199287, SGI0, SGI0, SGI0, SGI199288, SGI199288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44239[] = {SGI199290, SGI0, SGI199291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199292, SGI199293, SGI199293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44240[] = {SGI199294, SGI199295, SGI199296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199297, SGI199297, SGI0, SGI0, SGI0, SGI199298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44241[] = {SGI199299, SGI0, SGI199300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199301, SGI199302, SGI199302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44242[] = {SGI199303, SGI0, SGI199304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199305, SGI199306, SGI199306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44243[] = {SGI199307, SGI199308, SGI199309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199310, SGI0, SGI0, SGI0, SGI199311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44244[] = {SGI199312, SGI199313, SGI199314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199315, SGI0, SGI0, SGI0, SGI199316, SGI199316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44245[] = {SGI199317, SGI0, SGI199318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199319, SGI199320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44246[] = {SGI199321, SGI0, SGI199322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199323, SGI199324, SGI199324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44247[] = {SGI199325, SGI0, SGI199326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199327, SGI199328, SGI199328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44248[] = {SGI199329, SGI0, SGI199330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199331, SGI199332, SGI199332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44249[] = {SGI199333, SGI199334, SGI199335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199336, SGI0, SGI0, SGI0, SGI199337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44250[] = {SGI199338, SGI0, SGI199339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44251[] = {SGI199340, SGI199341, SGI199342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199343, SGI199343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44252[] = {SGI199344, SGI0, SGI199345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199346, SGI199347, SGI199347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44253[] = {SGI199348, SGI199349, SGI199350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44254[] = {SGI199351, SGI199352, SGI199353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199354, SGI199354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44255[] = {SGI199355, SGI199356, SGI199357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199358, SGI0, SGI0, SGI0, SGI199359, SGI199359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44256[] = {SGI199360, SGI0, SGI199361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44257[] = {SGI199362, SGI199363, SGI199364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44258[] = {SGI0, SGI199365, SGI199366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44259[] = {SGI0, SGI199367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44260[] = {SGI199368, SGI199369, SGI199370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199371, SGI199371, SGI0, SGI0, SGI0, SGI199372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44261[] = {SGI199373, SGI199374, SGI199375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199377, SGI0, SGI0, SGI0, SGI199380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44262[] = {SGI199376, SGI199378, SGI199379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199381, SGI199381, SGI0, SGI0, SGI0, SGI199382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44263[] = {SGI199383, SGI0, SGI199384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199385, SGI199386, SGI199386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44264[] = {SGI199387, SGI0, SGI199388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199389, SGI199390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44265[] = {SGI199391, SGI0, SGI199392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199393, SGI199394, SGI199394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44266[] = {SGI199395, SGI0, SGI199396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199397, SGI199398, SGI199398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44267[] = {SGI199399, SGI199400, SGI199401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199402, SGI0, SGI0, SGI0, SGI199403, SGI199403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44268[] = {SGI199404, SGI0, SGI199405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44269[] = {SGI199406, SGI199407, SGI199408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199409, SGI0, SGI0, SGI0, SGI199410, SGI199410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44270[] = {SGI199411, SGI199412, SGI199413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199415, SGI199415, SGI0, SGI0, SGI0, SGI199418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44271[] = {SGI199414, SGI199416, SGI199417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199419, SGI199419, SGI0, SGI0, SGI0, SGI0, SGI199420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44272[] = {SGI199421, SGI0, SGI199422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199423, SGI199425, SGI199425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44273[] = {SGI199424, SGI199426, SGI199427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199428, SGI0, SGI0, SGI0, SGI199429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44274[] = {SGI199430, SGI0, SGI199431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199432, SGI199433, SGI199433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44275[] = {SGI199434, SGI0, SGI199435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199438, SGI199439, SGI199439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44276[] = {SGI199436, SGI0, SGI199437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199440, SGI199441, SGI199441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199442, SGI0, SGI0, NULL};
const Segment * const SG44277[] = {SGI199443, SGI0, SGI199445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199447, SGI199449, SGI199449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44278[] = {SGI199444, SGI0, SGI199446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199448, SGI199450, SGI199450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44279[] = {SGI199451, SGI0, SGI199452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199453, SGI199454, SGI199454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44280[] = {SGI199455, SGI0, SGI199456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199457, SGI199458, SGI199458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44281[] = {SGI199459, SGI0, SGI199460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199461, SGI199462, SGI199462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44282[] = {SGI199463, SGI0, SGI199464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199465, SGI199466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44283[] = {SGI199467, SGI0, SGI199468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199469, SGI0, SGI0, SGI0, SGI199470, SGI199470, SGI0, SGI0, SGI0, SGI199471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44284[] = {SGI199472, SGI0, SGI199473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44285[] = {SGI199474, SGI199475, SGI199476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199477, SGI0, SGI0, SGI0, SGI199478, SGI199478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44286[] = {SGI199479, SGI0, SGI199480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199481, SGI199482, SGI199482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44287[] = {SGI199483, SGI0, SGI199484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199485, SGI199486, SGI199486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44288[] = {SGI199487, SGI0, SGI199488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199492, SGI199493, SGI199493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44289[] = {SGI199489, SGI199490, SGI199491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199494, SGI0, SGI0, SGI0, SGI199495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44290[] = {SGI199496, SGI0, SGI199497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199501, SGI199502, SGI199502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44291[] = {SGI199498, SGI199499, SGI199500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199503, SGI0, SGI0, SGI0, SGI199504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44292[] = {SGI199505, SGI199506, SGI199507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199508, SGI199508, SGI0, SGI0, SGI0, SGI199509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44293[] = {SGI199510, SGI199511, SGI199512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199513, SGI199513, SGI0, SGI0, SGI0, SGI199514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44294[] = {SGI199515, SGI199516, SGI199517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199518, SGI199518, SGI0, SGI0, SGI0, SGI199519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44295[] = {SGI199520, SGI0, SGI199521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199522, SGI199523, SGI199523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199524, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44296[] = {SGI199525, SGI0, SGI199526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199527, SGI199528, SGI199528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44297[] = {SGI199529, SGI0, SGI199530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199531, SGI199532, SGI199532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44298[] = {SGI199533, SGI0, SGI199534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199535, SGI199536, SGI199536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44299[] = {SGI199537, SGI0, SGI199538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199539, SGI199540, SGI199540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44300[] = {SGI199541, SGI199542, SGI199543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199544, SGI0, SGI0, SGI0, SGI199545, SGI199545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44301[] = {SGI199546, SGI0, SGI199547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44302[] = {SGI199548, SGI199549, SGI199550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199551, SGI0, SGI0, SGI0, SGI199552, SGI0, SGI0, SGI199553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44303[] = {SGI199554, SGI0, SGI199555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199556, SGI199557, SGI199557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44304[] = {SGI199558, SGI199559, SGI199560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199561, SGI0, SGI0, SGI0, SGI199562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44305[] = {SGI0, SGI199563, SGI0, SGI199564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44306[] = {SGI199565, SGI199566, SGI199567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199568, SGI0, SGI0, SGI0, SGI199569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44307[] = {SGI199570, SGI199571, SGI199572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199573, SGI0, SGI0, SGI0, SGI199574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44308[] = {SGI199575, SGI0, SGI199576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199577, SGI199578, SGI199578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44309[] = {SGI199579, SGI0, SGI199580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199581, SGI199582, SGI199582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44310[] = {SGI199583, SGI0, SGI199584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199585, SGI199586, SGI199586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44311[] = {SGI199587, SGI0, SGI199588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199589, SGI199590, SGI199590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44312[] = {SGI199591, SGI199592, SGI199593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199595, SGI0, SGI0, SGI0, SGI199598, SGI199598, SGI0, SGI199599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44313[] = {SGI199594, SGI199596, SGI199597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199600, SGI0, SGI0, SGI0, SGI199601, SGI199601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44314[] = {SGI199602, SGI0, SGI199603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199606, SGI199608, SGI199608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44315[] = {SGI199604, SGI199605, SGI199607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199612, SGI0, SGI0, SGI0, SGI199613, SGI199613, SGI0, SGI199615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44316[] = {SGI199609, SGI199610, SGI199611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199614, SGI0, SGI0, SGI0, SGI199616, SGI199616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44317[] = {SGI199617, SGI0, SGI199618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44318[] = {SGI199619, SGI199620, SGI199621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199622, SGI0, SGI0, SGI0, SGI199623, SGI199623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44319[] = {SGI199624, SGI0, SGI199625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199626, SGI199627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44320[] = {SGI199628, SGI0, SGI199629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199630, SGI199631, SGI199631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44321[] = {SGI199632, SGI199633, SGI199634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199635, SGI0, SGI0, SGI0, SGI199636, SGI199636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44322[] = {SGI199637, SGI0, SGI199638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199641, SGI199642, SGI199642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44323[] = {SGI199639, SGI0, SGI199640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199643, SGI199644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44324[] = {SGI199645, SGI0, SGI199646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199647, SGI199648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44325[] = {SGI199649, SGI199650, SGI199651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199652, SGI0, SGI0, SGI0, SGI199653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44326[] = {SGI199656, SGI0, SGI199657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44327[] = {SGI199654, SGI0, SGI199655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199658, SGI199659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44328[] = {SGI199660, SGI199661, SGI199662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199664, SGI199664, SGI0, SGI0, SGI0, SGI199666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44329[] = {SGI199663, SGI0, SGI199665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199667, SGI199668, SGI199668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44330[] = {SGI199669, SGI199670, SGI199671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199672, SGI0, SGI0, SGI0, SGI199673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44331[] = {SGI199674, SGI199675, SGI199676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199677, SGI199677, SGI0, SGI0, SGI199678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44332[] = {SGI199679, SGI0, SGI199680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199681, SGI199682, SGI199682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44333[] = {SGI199683, SGI0, SGI199684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199685, SGI199686, SGI199686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44334[] = {SGI199687, SGI0, SGI199688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199689, SGI199690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44335[] = {SGI199691, SGI199692, SGI199693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199694, SGI0, SGI0, SGI0, SGI199695, SGI199695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44336[] = {SGI199697, SGI199698, SGI199699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199700, SGI0, SGI0, SGI0, SGI199701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44337[] = {SGI199703, SGI199704, SGI199705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199706, SGI0, SGI0, SGI0, SGI199707, SGI199707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44338[] = {SGI199708, SGI199709, SGI199710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199711, SGI0, SGI0, SGI0, SGI199712, SGI199712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44339[] = {SGI199713, SGI0, SGI199714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44340[] = {SGI199715, SGI199716, SGI199717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199718, SGI199718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199719, NULL};
const Segment * const SG44341[] = {SGI199720, SGI199721, SGI199722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199723, SGI199723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199724, NULL};
const Segment * const SG44342[] = {SGI199725, SGI199726, SGI199727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199728, SGI199728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199729, NULL};
const Segment * const SG44343[] = {SGI199730, SGI199731, SGI199732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199733, SGI199733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199734, NULL};
const Segment * const SG44344[] = {SGI199735, SGI0, SGI199736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199737, SGI199738, SGI199738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44345[] = {SGI199739, SGI199741, SGI199743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199744, SGI0, SGI0, SGI0, SGI199746, SGI199746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44346[] = {SGI199740, SGI0, SGI199742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199745, SGI199747, SGI199747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44347[] = {SGI199748, SGI0, SGI199749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44348[] = {SGI199750, SGI199751, SGI199752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199753, SGI0, SGI0, SGI0, SGI199755, SGI199755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44349[] = {SGI199754, SGI0, SGI199756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199757, SGI199758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44350[] = {SGI199759, SGI199760, SGI199761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199762, SGI0, SGI0, SGI0, SGI199763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44351[] = {SGI199764, SGI0, SGI199765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199766, SGI199767, SGI199767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44352[] = {SGI199768, SGI199769, SGI199770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199771, SGI0, SGI0, SGI0, SGI199772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44353[] = {SGI199773, SGI0, SGI199774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199775, SGI199776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44354[] = {SGI199777, SGI0, SGI199778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199779, SGI199780, SGI199780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44355[] = {SGI0, SGI199781, SGI199782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199783, SGI0, SGI0, SGI0, SGI199784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44356[] = {SGI199785, SGI199786, SGI199787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199788, SGI0, SGI0, SGI0, SGI199789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44357[] = {SGI0, SGI199790, SGI199791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199792, SGI0, SGI0, SGI0, SGI199793, SGI199794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44358[] = {SGI199795, SGI0, SGI199796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44359[] = {SGI0, SGI0, SGI199797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44360[] = {SGI199798, SGI199799, SGI199800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199802, SGI0, SGI0, SGI0, SGI199804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44361[] = {SGI199801, SGI0, SGI199803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199805, SGI0, SGI0, SGI0, SGI199806, SGI199806, SGI0, SGI0, SGI199807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44362[] = {SGI199808, SGI0, SGI199809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199810, SGI199811, SGI199811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44363[] = {SGI199812, SGI0, SGI199813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199814, SGI199815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44364[] = {SGI199816, SGI0, SGI199817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199818, SGI199819, SGI199819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44365[] = {SGI199820, SGI0, SGI199821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199822, SGI199823, SGI199823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44366[] = {SGI199824, SGI0, SGI199825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199826, SGI199827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44367[] = {SGI199828, SGI199829, SGI199830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199831, SGI0, SGI0, SGI0, SGI199832, SGI199832, SGI0, SGI199833, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44368[] = {SGI199834, SGI0, SGI199835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199836, SGI199837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44369[] = {SGI199838, SGI0, SGI199839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199840, SGI199841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44370[] = {SGI199842, SGI199843, SGI199844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199845, SGI0, SGI0, SGI0, SGI199846, SGI199846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44371[] = {SGI199847, SGI0, SGI199848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44372[] = {SGI199849, SGI0, SGI199850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199851, SGI199852, SGI199852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44373[] = {SGI199853, SGI0, SGI199854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199855, SGI199856, SGI199856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44374[] = {SGI199857, SGI0, SGI199858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199859, SGI199860, SGI199860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44375[] = {SGI199861, SGI0, SGI199862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199863, SGI199864, SGI199864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44376[] = {SGI199865, SGI0, SGI199866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199867, SGI199869, SGI199869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44377[] = {SGI199868, SGI0, SGI199870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199871, SGI199872, SGI199872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44378[] = {SGI199873, SGI199874, SGI199876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199878, SGI0, SGI0, SGI0, SGI199880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44379[] = {SGI199875, SGI0, SGI199877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199881, SGI199882, SGI199882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44380[] = {SGI199879, SGI0, SGI199883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199885, SGI199886, SGI199886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44381[] = {SGI199887, SGI199888, SGI199889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199890, SGI0, SGI0, SGI0, SGI199891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44382[] = {SGI199893, SGI0, SGI199894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199895, SGI199896, SGI199896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199897, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44383[] = {SGI199898, SGI0, SGI199899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199900, SGI199901, SGI199901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44384[] = {SGI199902, SGI0, SGI199903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199904, SGI199905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44385[] = {SGI0, SGI199906, SGI199907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199908, SGI0, SGI0, SGI0, SGI199909, SGI199909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44386[] = {SGI0, SGI199910, SGI199911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199912, SGI0, SGI0, SGI0, SGI199913, SGI199913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44387[] = {SGI199914, SGI199915, SGI199916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199917, SGI0, SGI0, SGI0, SGI199918, SGI199918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44388[] = {SGI199919, SGI199920, SGI199921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199922, SGI0, SGI0, SGI0, SGI199923, SGI199923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44389[] = {SGI199924, SGI0, SGI199925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44390[] = {SGI199926, SGI199927, SGI199928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199929, SGI0, SGI0, SGI0, SGI199930, SGI199930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44391[] = {SGI199931, SGI199932, SGI199933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199934, SGI0, SGI0, SGI0, SGI199935, SGI199935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44392[] = {SGI199936, SGI0, SGI199937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199938, SGI199939, SGI199939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44393[] = {SGI199940, SGI199941, SGI199942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199943, SGI0, SGI0, SGI0, SGI199944, SGI199944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44394[] = {SGI199945, SGI0, SGI199946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199947, SGI199948, SGI199948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44395[] = {SGI199949, SGI0, SGI199950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199951, SGI199952, SGI199952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44396[] = {SGI199953, SGI0, SGI199954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199955, SGI199956, SGI199956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44397[] = {SGI199957, SGI0, SGI199958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44398[] = {SGI199959, SGI199960, SGI199961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199962, SGI0, SGI0, SGI0, SGI199963, SGI199963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44399[] = {SGI199964, SGI199965, SGI199966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199967, SGI0, SGI0, SGI0, SGI199968, SGI199968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44400[] = {SGI199969, SGI199970, SGI199971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199972, SGI0, SGI0, SGI0, SGI199973, SGI199973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44401[] = {SGI199974, SGI199975, SGI199976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199977, SGI0, SGI0, SGI0, SGI199978, SGI199978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44402[] = {SGI199979, SGI0, SGI199980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199981, SGI199982, SGI199982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44403[] = {SGI199983, SGI199984, SGI199985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199986, SGI199986, SGI0, SGI0, SGI199987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44404[] = {SGI199988, SGI199989, SGI199990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199991, SGI0, SGI0, SGI0, SGI199992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44405[] = {SGI199993, SGI0, SGI199994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44406[] = {SGI199995, SGI199996, SGI199997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI199998, SGI0, SGI0, SGI0, SGI199999, SGI199999, SGI0, SGI200000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44407[] = {SGI200001, SGI200002, SGI200003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200004, SGI0, SGI0, SGI0, SGI200005, SGI200005, SGI0, SGI200006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44408[] = {SGI200007, SGI200008, SGI200009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200010, SGI0, SGI0, SGI0, SGI200012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44409[] = {SGI200011, SGI200013, SGI200014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200015, SGI0, SGI0, SGI0, SGI200016, SGI200016, SGI0, SGI200017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44410[] = {SGI200018, SGI200019, SGI200020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200021, SGI0, SGI0, SGI0, SGI200022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44411[] = {SGI200023, SGI200024, SGI200025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200026, SGI0, SGI0, SGI0, SGI200027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44412[] = {SGI200028, SGI200029, SGI200030, SGI0, SGI0, SGI0, SGI200031, SGI200032, SGI0, SGI200033, SGI0, SGI0, SGI0, SGI200034, SGI200034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44413[] = {SGI200035, SGI200036, SGI200037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200038, SGI0, SGI0, SGI0, SGI200039, SGI200039, SGI0, SGI200040, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44414[] = {SGI200041, SGI0, SGI200042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200043, SGI200044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44415[] = {SGI200045, SGI200046, SGI200047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200048, SGI0, SGI0, SGI0, SGI200049, SGI0, SGI200050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44416[] = {SGI200051, SGI200052, SGI200053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200054, SGI0, SGI0, SGI0, SGI200055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44417[] = {SGI200056, SGI0, SGI200057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200058, SGI200059, SGI200059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44418[] = {SGI200060, SGI0, SGI200061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200062, SGI200063, SGI200063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44419[] = {SGI200064, SGI200065, SGI200066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200067, SGI0, SGI0, SGI0, SGI200070, SGI200070, SGI0, SGI200071, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44420[] = {SGI200068, SGI0, SGI200069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200072, SGI200073, SGI200073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44421[] = {SGI200074, SGI0, SGI200075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200076, SGI200077, SGI200077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44422[] = {SGI200078, SGI0, SGI200079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200080, SGI200081, SGI200081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44423[] = {SGI200082, SGI0, SGI200083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200084, SGI200085, SGI200085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44424[] = {SGI200086, SGI200087, SGI200088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200089, SGI0, SGI0, SGI0, SGI200090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44425[] = {SGI200091, SGI0, SGI200092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200093, SGI200094, SGI200094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44426[] = {SGI200095, SGI200096, SGI200097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200098, SGI0, SGI0, SGI0, SGI200099, SGI200099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200100, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44427[] = {SGI200101, SGI200102, SGI200103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200104, SGI0, SGI0, SGI0, SGI200105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44428[] = {SGI200106, SGI200107, SGI200108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200109, SGI0, SGI0, SGI0, SGI200110, SGI200110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200111, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44429[] = {SGI200112, SGI0, SGI200113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200114, SGI0, SGI0, SGI0, SGI200115, SGI200115, SGI0, SGI0, SGI0, SGI200116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44430[] = {SGI200117, SGI0, SGI200118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200122, SGI200123, SGI200123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44431[] = {SGI200119, SGI200120, SGI200121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200124, SGI0, SGI0, SGI0, SGI200125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44432[] = {SGI200126, SGI0, SGI200127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44433[] = {SGI200128, SGI200129, SGI200130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200131, SGI0, SGI0, SGI0, SGI200132, SGI200132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44434[] = {SGI200133, SGI200134, SGI200135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200136, SGI0, SGI0, SGI0, SGI200137, SGI200137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44435[] = {SGI200138, SGI0, SGI200139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200140, SGI200141, SGI200141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44436[] = {SGI200142, SGI0, SGI200143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200144, SGI200145, SGI200145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44437[] = {SGI200146, SGI0, SGI200147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200148, SGI0, SGI0, SGI0, SGI200149, SGI200149, SGI0, SGI0, SGI0, SGI200150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44438[] = {SGI200151, SGI0, SGI200152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200153, SGI0, SGI0, SGI0, SGI200154, SGI200154, SGI0, SGI0, SGI0, SGI200155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44439[] = {SGI200156, SGI200157, SGI200158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200159, SGI0, SGI0, SGI0, SGI200160, SGI200161, SGI0, SGI0, SGI0, SGI200162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44440[] = {SGI200163, SGI0, SGI200164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200165, SGI200166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44441[] = {SGI200167, SGI200168, SGI200169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200171, SGI0, SGI0, SGI0, SGI200174, SGI200174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44442[] = {SGI200170, SGI200172, SGI200173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200175, SGI0, SGI0, SGI0, SGI200176, SGI200176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44443[] = {SGI200177, SGI0, SGI200178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200179, SGI200180, SGI200180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44444[] = {SGI200181, SGI200183, SGI200185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200188, SGI0, SGI0, SGI0, SGI200190, SGI200190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44445[] = {SGI200182, SGI200184, SGI200186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200187, SGI0, SGI0, SGI0, SGI200189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44446[] = {SGI200191, SGI0, SGI200192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200193, SGI200194, SGI200194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44447[] = {SGI200195, SGI200197, SGI200199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200200, SGI0, SGI0, SGI0, SGI200203, SGI200203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44448[] = {SGI200196, SGI0, SGI200198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200201, SGI200202, SGI200202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44449[] = {SGI200204, SGI0, SGI200205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44450[] = {SGI200206, SGI200207, SGI200208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200209, SGI0, SGI0, SGI0, SGI200210, SGI200210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44451[] = {SGI200211, SGI200212, SGI200213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200214, SGI0, SGI0, SGI0, SGI200215, SGI200215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44452[] = {SGI200216, SGI200217, SGI200218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200219, SGI0, SGI0, SGI0, SGI200222, SGI200222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44453[] = {SGI200220, SGI200221, SGI200223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200224, SGI0, SGI0, SGI0, SGI200225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44454[] = {SGI200229, SGI0, SGI200230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44455[] = {SGI200226, SGI200227, SGI200228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200231, SGI0, SGI0, SGI0, SGI200232, SGI200232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200233, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44456[] = {SGI200234, SGI200235, SGI200236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200237, SGI0, SGI0, SGI0, SGI200238, SGI200238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200239, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44457[] = {SGI200240, SGI200241, SGI200242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200243, SGI0, SGI0, SGI0, SGI200244, SGI200244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200245, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44458[] = {SGI200246, SGI0, SGI200247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200248, SGI200249, SGI200249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44459[] = {SGI200250, SGI200251, SGI200252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200253, SGI200253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200254, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44460[] = {SGI200255, SGI0, SGI200256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200257, SGI200258, SGI200258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44461[] = {SGI200259, SGI0, SGI200260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200267, SGI200268, SGI200268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44462[] = {SGI200262, SGI200264, SGI200266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200270, SGI0, SGI0, SGI0, SGI200272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44463[] = {SGI200261, SGI200263, SGI200265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200269, SGI0, SGI0, SGI0, SGI200271, SGI200271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44464[] = {SGI200274, SGI200275, SGI200276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200277, SGI0, SGI0, SGI0, SGI200278, SGI200278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44465[] = {SGI200280, SGI200281, SGI200282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200283, SGI0, SGI0, SGI0, SGI200284, SGI0, SGI0, SGI200287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44466[] = {SGI200285, SGI200286, SGI200288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200289, SGI0, SGI0, SGI0, SGI200290, SGI200290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44467[] = {SGI200291, SGI0, SGI200292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200293, SGI200294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44468[] = {SGI200295, SGI0, SGI200296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200297, SGI200298, SGI200298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44469[] = {SGI200299, SGI200300, SGI200301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200302, SGI0, SGI0, SGI0, SGI200303, SGI200303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44470[] = {SGI200304, SGI200305, SGI200306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200307, SGI0, SGI0, SGI0, SGI200308, SGI200308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44471[] = {SGI200309, SGI200310, SGI200311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200314, SGI0, SGI0, SGI0, SGI200315, SGI200315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44472[] = {SGI200312, SGI0, SGI200313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200316, SGI200317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200318, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44473[] = {SGI200319, SGI200320, SGI200321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200322, SGI200323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44474[] = {SGI200324, SGI0, SGI200325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200326, SGI200327, SGI200327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44475[] = {SGI200328, SGI0, SGI200329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200330, SGI200331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200333, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44476[] = {SGI200332, SGI0, SGI200334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200335, SGI200336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44477[] = {SGI200337, SGI200338, SGI200339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200340, SGI0, SGI0, SGI0, SGI200341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44478[] = {SGI200342, SGI200343, SGI200344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200345, SGI0, SGI0, SGI0, SGI200346, SGI200346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44479[] = {SGI200347, SGI0, SGI200348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200349, SGI200350, SGI200350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200351, SGI0, SGI0, NULL};
const Segment * const SG44480[] = {SGI200352, SGI200353, SGI200354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200355, SGI0, SGI0, SGI0, SGI200356, SGI200356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44481[] = {SGI200357, SGI0, SGI200358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200359, SGI200360, SGI200360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44482[] = {SGI200361, SGI0, SGI200362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200363, SGI200364, SGI200364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44483[] = {SGI200365, SGI0, SGI200366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200367, SGI200368, SGI200368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44484[] = {SGI200369, SGI200370, SGI200371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200372, SGI0, SGI0, SGI0, SGI200373, SGI200373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44485[] = {SGI200374, SGI0, SGI200375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200376, SGI200377, SGI200377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44486[] = {SGI200378, SGI200379, SGI200380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200381, SGI0, SGI0, SGI0, SGI200382, SGI200382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44487[] = {SGI200383, SGI0, SGI200384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44488[] = {SGI200385, SGI0, SGI200386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200387, SGI200388, SGI200388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44489[] = {SGI200389, SGI0, SGI200390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200391, SGI200392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44490[] = {SGI200393, SGI0, SGI200394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200395, SGI200396, SGI200396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44491[] = {SGI200397, SGI0, SGI200398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200399, SGI200401, SGI200401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44492[] = {SGI200400, SGI0, SGI200402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200403, SGI200404, SGI200404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44493[] = {SGI200405, SGI0, SGI200406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200407, SGI200408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44494[] = {SGI200409, SGI0, SGI200410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200411, SGI200412, SGI200412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44495[] = {SGI200413, SGI0, SGI200414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200415, SGI200416, SGI200416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44496[] = {SGI200417, SGI0, SGI200418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200421, SGI200423, SGI200423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44497[] = {SGI200419, SGI200420, SGI200422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200424, SGI200424, SGI0, SGI0, SGI0, SGI0, SGI200425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44498[] = {SGI200426, SGI0, SGI200427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200428, SGI200429, SGI200429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44499[] = {SGI200430, SGI0, SGI200431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200432, SGI200433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44500[] = {SGI200434, SGI200435, SGI200436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200437, SGI0, SGI0, SGI0, SGI200438, SGI200438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44501[] = {SGI200439, SGI200440, SGI200441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200442, SGI0, SGI0, SGI0, SGI200443, SGI200443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44502[] = {SGI200444, SGI0, SGI200445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200446, SGI200447, SGI200447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200448, SGI0, SGI0, NULL};
const Segment * const SG44503[] = {SGI200449, SGI200450, SGI200451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200452, SGI0, SGI0, SGI0, SGI200453, SGI200453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44504[] = {SGI200454, SGI0, SGI200455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200456, SGI200457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44505[] = {SGI200458, SGI0, SGI200459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200460, SGI200461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44506[] = {SGI200462, SGI0, SGI200463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200464, SGI0, SGI0, SGI0, SGI200465, SGI200465, SGI0, SGI0, SGI0, SGI200466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44507[] = {SGI200467, SGI0, SGI200468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200469, SGI200470, SGI200470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44508[] = {SGI200471, SGI0, SGI200472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200473, SGI200474, SGI200474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44509[] = {SGI200475, SGI0, SGI200476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200477, SGI200478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44510[] = {SGI200479, SGI200480, SGI200481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200482, SGI200482, SGI0, SGI0, SGI0, SGI200483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44511[] = {SGI200484, SGI0, SGI200485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200486, SGI200487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44512[] = {SGI200488, SGI0, SGI200489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200490, SGI200491, SGI200491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44513[] = {SGI200492, SGI0, SGI200493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200494, SGI200495, SGI200495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44514[] = {SGI200496, SGI200497, SGI200498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200499, SGI0, SGI0, SGI0, SGI200500, SGI200500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44515[] = {SGI200502, SGI0, SGI200503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200504, SGI200505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44516[] = {SGI200507, SGI0, SGI200509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200510, SGI200512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44517[] = {SGI200506, SGI0, SGI200508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200511, SGI200513, SGI200513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44518[] = {SGI200514, SGI0, SGI200515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200516, SGI200517, SGI200517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44519[] = {SGI200518, SGI0, SGI200519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200520, SGI200521, SGI200521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44520[] = {SGI200522, SGI200523, SGI200524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200525, SGI0, SGI0, SGI0, SGI200526, SGI200526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44521[] = {SGI200527, SGI0, SGI200528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200529, SGI200530, SGI200530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44522[] = {SGI200531, SGI0, SGI200532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200533, SGI200534, SGI200534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44523[] = {SGI200535, SGI0, SGI200536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200537, SGI200538, SGI200538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44524[] = {SGI200539, SGI0, SGI200540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200541, SGI200542, SGI200542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44525[] = {SGI200543, SGI0, SGI200544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200545, SGI200546, SGI200546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44526[] = {SGI200547, SGI200548, SGI200549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200550, SGI0, SGI0, SGI0, SGI200551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44527[] = {SGI200552, SGI0, SGI200553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200554, SGI200555, SGI200555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44528[] = {SGI200556, SGI200557, SGI200558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200559, SGI0, SGI0, SGI0, SGI200560, SGI200560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44529[] = {SGI200561, SGI200562, SGI200563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200564, SGI0, SGI0, SGI0, SGI200565, SGI200565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44530[] = {SGI200566, SGI0, SGI200567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200568, SGI200569, SGI200569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44531[] = {SGI200570, SGI0, SGI200571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200572, SGI200573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44532[] = {SGI200574, SGI0, SGI200575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200576, SGI200577, SGI200577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44533[] = {SGI200578, SGI200579, SGI200580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200581, SGI200581, SGI0, SGI0, SGI0, SGI0, SGI200582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44534[] = {SGI200583, SGI200584, SGI200585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200586, SGI0, SGI0, SGI0, SGI200587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44535[] = {SGI200588, SGI0, SGI200589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200590, SGI200591, SGI200591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44536[] = {SGI200592, SGI200593, SGI200594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200595, SGI200595, SGI0, SGI0, SGI0, SGI0, SGI200596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44537[] = {SGI200597, SGI200598, SGI200599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200600, SGI0, SGI0, SGI0, SGI200601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44538[] = {SGI200602, SGI0, SGI200603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200604, SGI200605, SGI200605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44539[] = {SGI200606, SGI0, SGI200607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200608, SGI200609, SGI200609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44540[] = {SGI200610, SGI200611, SGI200612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200613, SGI0, SGI0, SGI0, SGI200614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44541[] = {SGI200615, SGI200616, SGI200617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200618, SGI200618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44542[] = {SGI200619, SGI200620, SGI200621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200622, SGI0, SGI0, SGI0, SGI200624, SGI200624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44543[] = {SGI200623, SGI200625, SGI200626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200627, SGI200627, SGI0, SGI0, SGI0, SGI0, SGI200628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44544[] = {SGI200629, SGI0, SGI200630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200631, SGI200632, SGI200632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44545[] = {SGI200633, SGI200634, SGI200635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200636, SGI200636, SGI0, SGI0, SGI0, SGI0, SGI200637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44546[] = {SGI200638, SGI0, SGI200639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200640, SGI200641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44547[] = {SGI200642, SGI0, SGI200643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200644, SGI200645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44548[] = {SGI200648, SGI0, SGI200650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200652, SGI200653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44549[] = {SGI200646, SGI200647, SGI200649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200651, SGI200651, SGI0, SGI0, SGI0, SGI200654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44550[] = {SGI200655, SGI200656, SGI200657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200658, SGI0, SGI0, SGI0, SGI200659, SGI200659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200660, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44551[] = {SGI200661, SGI200662, SGI200663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200664, SGI0, SGI0, SGI0, SGI200665, SGI200665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200666, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44552[] = {SGI200667, SGI200668, SGI200669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200670, SGI0, SGI0, SGI0, SGI200671, SGI200671, SGI0, SGI200672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44553[] = {SGI200673, SGI200674, SGI200675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200676, SGI0, SGI0, SGI0, SGI200677, SGI200677, SGI0, SGI200679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44554[] = {SGI200678, SGI0, SGI200680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200681, SGI200682, SGI200682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200683, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44555[] = {SGI200684, SGI200685, SGI200686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200687, SGI0, SGI0, SGI0, SGI200688, SGI200688, SGI200689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44556[] = {SGI200690, SGI200691, SGI200692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200693, SGI0, SGI0, SGI0, SGI200694, SGI0, SGI200695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44557[] = {SGI200696, SGI0, SGI200697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200698, SGI200699, SGI200699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44558[] = {SGI200700, SGI0, SGI200701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200702, SGI200703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44559[] = {SGI200704, SGI0, SGI200705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200706, SGI200707, SGI200707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44560[] = {SGI200708, SGI0, SGI200709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200710, SGI200711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44561[] = {SGI200712, SGI200713, SGI200714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200715, SGI0, SGI0, SGI200716, SGI200717, SGI200717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44562[] = {SGI200718, SGI200719, SGI200720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200721, SGI0, SGI200722, SGI200722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44563[] = {SGI200723, SGI200724, SGI200725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200726, SGI0, SGI0, SGI0, SGI200727, SGI200727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44564[] = {SGI200728, SGI0, SGI200729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200730, SGI200731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44565[] = {SGI200732, SGI0, SGI200733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200734, SGI200735, SGI200735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44566[] = {SGI200736, SGI0, SGI200737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200738, SGI200739, SGI200739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44567[] = {SGI200740, SGI0, SGI200741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200742, SGI200743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44568[] = {SGI200744, SGI200745, SGI200746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200747, SGI0, SGI0, SGI0, SGI200748, SGI200748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44569[] = {SGI200749, SGI200750, SGI200751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200752, SGI0, SGI0, SGI0, SGI200753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44570[] = {SGI200754, SGI0, SGI200755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200756, SGI200757, SGI200757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44571[] = {SGI200758, SGI0, SGI200759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200760, SGI200761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44572[] = {SGI200762, SGI200763, SGI200764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200765, SGI200765, SGI0, SGI0, SGI0, SGI200766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44573[] = {SGI200767, SGI200768, SGI200769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200770, SGI0, SGI0, SGI0, SGI200771, SGI200771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44574[] = {SGI200772, SGI200773, SGI200774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200775, SGI0, SGI0, SGI0, SGI200776, SGI200776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44575[] = {SGI200777, SGI200778, SGI200779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200783, SGI200783, SGI0, SGI0, SGI0, SGI200785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44576[] = {SGI200780, SGI200781, SGI200782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200784, SGI0, SGI0, SGI0, SGI200786, SGI200786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44577[] = {SGI200787, SGI0, SGI200789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44578[] = {SGI200788, SGI200790, SGI200791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200792, SGI200792, SGI0, SGI0, SGI0, SGI200793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44579[] = {SGI200794, SGI200795, SGI200796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200797, SGI200797, SGI0, SGI0, SGI0, SGI200798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44580[] = {SGI200799, SGI200800, SGI200801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200802, SGI0, SGI0, SGI0, SGI200803, SGI200803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44581[] = {SGI200804, SGI0, SGI200805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200806, SGI200807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44582[] = {SGI200808, SGI0, SGI200809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200810, SGI200811, SGI200811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44583[] = {SGI200812, SGI200813, SGI200814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200815, SGI0, SGI0, SGI0, SGI200816, SGI200816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44584[] = {SGI200817, SGI200818, SGI200819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200821, SGI0, SGI0, SGI0, SGI200823, SGI200823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44585[] = {SGI200820, SGI0, SGI200822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200824, SGI200825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44586[] = {SGI200826, SGI0, SGI200827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200828, SGI200829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44587[] = {SGI200830, SGI200831, SGI200832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200833, SGI0, SGI0, SGI0, SGI200834, SGI200834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44588[] = {SGI200835, SGI0, SGI200836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200837, SGI200838, SGI200838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44589[] = {SGI200839, SGI200840, SGI200841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200842, SGI0, SGI0, SGI0, SGI200843, SGI200843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44590[] = {SGI200844, SGI200845, SGI200846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200850, SGI0, SGI0, SGI0, SGI200851, SGI200851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44591[] = {SGI200847, SGI200848, SGI200849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200852, SGI0, SGI0, SGI0, SGI200853, SGI200853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44592[] = {SGI200854, SGI200855, SGI200856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200857, SGI0, SGI0, SGI0, SGI200858, SGI200858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44593[] = {SGI200859, SGI0, SGI200860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200861, SGI200862, SGI200862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44594[] = {SGI200863, SGI0, SGI200864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200865, SGI200866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44595[] = {SGI200867, SGI200868, SGI200869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200870, SGI0, SGI0, SGI0, SGI200871, SGI200871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44596[] = {SGI200872, SGI200873, SGI200874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200875, SGI0, SGI0, SGI0, SGI200876, SGI200876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44597[] = {SGI200877, SGI200878, SGI200879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200880, SGI0, SGI0, SGI0, SGI200881, SGI200881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44598[] = {SGI200882, SGI0, SGI200883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200887, SGI200888, SGI200888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44599[] = {SGI200884, SGI200885, SGI200886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200889, SGI0, SGI0, SGI0, SGI200890, SGI200890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44600[] = {SGI200891, SGI0, SGI200892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200893, SGI200894, SGI200894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44601[] = {SGI200895, SGI0, SGI200896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200897, SGI200898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44602[] = {SGI200899, SGI0, SGI200900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200901, SGI200902, SGI200902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44603[] = {SGI200903, SGI0, SGI200904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200905, SGI200906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44604[] = {SGI200907, SGI0, SGI200908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200909, SGI200910, SGI200910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44605[] = {SGI200911, SGI0, SGI200912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200913, SGI200914, SGI200914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44606[] = {SGI200915, SGI0, SGI200916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200917, SGI200918, SGI200918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44607[] = {SGI200919, SGI0, SGI200920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200921, SGI200922, SGI200922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44608[] = {SGI200923, SGI0, SGI200924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200925, SGI200926, SGI200926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44609[] = {SGI200927, SGI0, SGI200928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200929, SGI200930, SGI200930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44610[] = {SGI200931, SGI0, SGI200932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200933, SGI200934, SGI200934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44611[] = {SGI200935, SGI200936, SGI200937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200938, SGI200938, SGI0, SGI0, SGI0, SGI0, SGI200939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44612[] = {SGI200940, SGI0, SGI200941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200942, SGI200943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44613[] = {SGI200944, SGI200945, SGI200946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200947, SGI0, SGI0, SGI0, SGI200948, SGI200948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44614[] = {SGI200949, SGI0, SGI200950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44615[] = {SGI200951, SGI200952, SGI200953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200954, SGI0, SGI0, SGI0, SGI200955, SGI200955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44616[] = {SGI200956, SGI200957, SGI200958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200959, SGI0, SGI0, SGI0, SGI200960, SGI200960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44617[] = {SGI200961, SGI0, SGI200962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200965, SGI0, SGI0, SGI0, SGI200967, SGI200967, SGI0, SGI0, SGI0, SGI200969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44618[] = {SGI200963, SGI200964, SGI200966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200968, SGI0, SGI0, SGI0, SGI200970, SGI200970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44619[] = {SGI0, SGI0, SGI200971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200974, SGI200975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44620[] = {SGI200972, SGI0, SGI200973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200976, SGI200977, SGI200977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44621[] = {SGI200978, SGI0, SGI200979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200980, SGI200981, SGI200981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44622[] = {SGI200982, SGI0, SGI200983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200984, SGI200985, SGI200985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44623[] = {SGI200986, SGI200987, SGI200988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200989, SGI0, SGI0, SGI0, SGI200990, SGI200990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44624[] = {SGI200992, SGI0, SGI200994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44625[] = {SGI200991, SGI0, SGI200993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200995, SGI200996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44626[] = {SGI200997, SGI0, SGI200998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI200999, SGI201000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44627[] = {SGI201001, SGI0, SGI201002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201003, SGI201004, SGI201004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44628[] = {SGI201005, SGI0, SGI201006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201007, SGI201008, SGI201008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201009, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44629[] = {SGI201010, SGI0, SGI201011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201012, SGI201013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44630[] = {SGI201014, SGI0, SGI201015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201016, SGI201018, SGI201018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201021, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44631[] = {SGI201017, SGI201019, SGI201020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201022, SGI0, SGI0, SGI0, SGI201023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44632[] = {SGI201024, SGI0, SGI201025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201026, SGI201027, SGI201027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201028, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44633[] = {SGI201029, SGI201030, SGI201031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201032, SGI0, SGI0, SGI0, SGI201033, SGI201033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44634[] = {SGI201035, SGI201036, SGI201037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201038, SGI0, SGI0, SGI0, SGI201039, SGI201039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44635[] = {SGI201040, SGI0, SGI201041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44636[] = {SGI201042, SGI0, SGI201043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201044, SGI201045, SGI201045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44637[] = {SGI201046, SGI201047, SGI201048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201049, SGI0, SGI0, SGI0, SGI201050, SGI201050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44638[] = {SGI201052, SGI201053, SGI201054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201055, SGI0, SGI0, SGI0, SGI201056, SGI201056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44639[] = {SGI201058, SGI201059, SGI201060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201061, SGI0, SGI0, SGI0, SGI201062, SGI201062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44640[] = {SGI201064, SGI0, SGI201065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201066, SGI201067, SGI201067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44641[] = {SGI201068, SGI201069, SGI201070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201071, SGI0, SGI0, SGI0, SGI201072, SGI201072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44642[] = {SGI201074, SGI0, SGI201075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201076, SGI201077, SGI201077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44643[] = {SGI201078, SGI201079, SGI201080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201081, SGI0, SGI0, SGI0, SGI201082, SGI201082, SGI0, SGI201083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44644[] = {SGI201084, SGI0, SGI201085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201086, SGI201087, SGI201087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44645[] = {SGI201088, SGI0, SGI201089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201090, SGI201091, SGI201091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44646[] = {SGI201092, SGI201093, SGI201094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201095, SGI0, SGI0, SGI0, SGI201096, SGI201096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44647[] = {SGI201097, SGI201098, SGI201099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201100, SGI0, SGI0, SGI0, SGI201101, SGI201101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201104, SGI0, SGI0, NULL};
const Segment * const SG44648[] = {SGI201102, SGI0, SGI201103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201105, SGI201106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44649[] = {SGI201107, SGI201108, SGI201109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201110, SGI0, SGI0, SGI0, SGI201111, SGI201111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201112, SGI0, SGI0, NULL};
const Segment * const SG44650[] = {SGI201113, SGI201114, SGI201115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201116, SGI0, SGI0, SGI0, SGI201117, SGI201117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201118, SGI0, SGI0, NULL};
const Segment * const SG44651[] = {SGI201119, SGI201120, SGI201121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201122, SGI0, SGI0, SGI0, SGI201123, SGI201123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201124, SGI0, SGI0, NULL};
const Segment * const SG44652[] = {SGI201125, SGI201126, SGI201127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201128, SGI0, SGI0, SGI0, SGI201129, SGI201129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201130, SGI0, SGI0, NULL};
const Segment * const SG44653[] = {SGI201131, SGI0, SGI201132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201133, SGI201134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44654[] = {SGI201135, SGI201136, SGI201137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201138, SGI0, SGI0, SGI0, SGI201139, SGI201139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201140, SGI0, SGI0, NULL};
const Segment * const SG44655[] = {SGI201141, SGI0, SGI201142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201143, SGI201144, SGI201144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44656[] = {SGI201145, SGI0, SGI201146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201147, SGI201148, SGI201148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44657[] = {SGI201149, SGI0, SGI201150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201153, SGI0, SGI0, SGI0, SGI201154, SGI201154, SGI0, SGI0, SGI0, SGI201156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44658[] = {SGI201151, SGI0, SGI201152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201155, SGI201157, SGI201157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44659[] = {SGI201158, SGI0, SGI201159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201160, SGI201161, SGI201161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44660[] = {SGI201162, SGI0, SGI201163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201164, SGI201165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44661[] = {SGI201166, SGI0, SGI201167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201168, SGI201169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44662[] = {SGI201170, SGI0, SGI201171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201174, SGI201175, SGI201175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44663[] = {SGI201172, SGI0, SGI201173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201176, SGI201177, SGI201177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44664[] = {SGI201178, SGI0, SGI201179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201180, SGI201181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44665[] = {SGI201182, SGI0, SGI201183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201184, SGI201185, SGI201185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44666[] = {SGI201186, SGI201187, SGI201188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201189, SGI0, SGI0, SGI0, SGI201190, SGI201190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44667[] = {SGI201191, SGI201192, SGI201193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201194, SGI201194, SGI0, SGI0, SGI0, SGI201195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44668[] = {SGI201196, SGI0, SGI201197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201198, SGI201199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44669[] = {SGI201200, SGI0, SGI201201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201202, SGI201203, SGI201203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44670[] = {SGI201204, SGI0, SGI201205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201206, SGI201207, SGI201207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44671[] = {SGI201208, SGI201209, SGI201210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201211, SGI0, SGI0, SGI0, SGI201212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44672[] = {SGI201213, SGI0, SGI201214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201215, SGI201216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44673[] = {SGI201217, SGI201218, SGI201219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201220, SGI201220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44674[] = {SGI201221, SGI201222, SGI201223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201224, SGI0, SGI0, SGI0, SGI201225, SGI201225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44675[] = {SGI201226, SGI201227, SGI201228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201229, SGI0, SGI0, SGI0, SGI0, SGI201230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44676[] = {SGI201231, SGI201232, SGI201233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201236, SGI0, SGI0, SGI0, SGI201238, SGI201238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44677[] = {SGI201234, SGI201235, SGI201237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201239, SGI201240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44678[] = {SGI201241, SGI0, SGI201242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44679[] = {SGI201243, SGI201244, SGI201245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201246, SGI0, SGI0, SGI0, SGI201248, SGI201248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44680[] = {SGI201247, SGI201249, SGI201250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201252, SGI0, SGI0, SGI0, SGI201253, SGI201253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44681[] = {SGI201254, SGI0, SGI201255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201256, SGI201257, SGI201257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44682[] = {SGI201258, SGI0, SGI201259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201260, SGI201261, SGI201261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44683[] = {SGI201262, SGI201263, SGI201264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201267, SGI0, SGI0, SGI0, SGI201268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44684[] = {SGI201265, SGI0, SGI201266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201269, SGI201270, SGI201270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44685[] = {SGI201271, SGI0, SGI201272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201273, SGI201275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44686[] = {SGI201274, SGI0, SGI201276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44687[] = {SGI201277, SGI0, SGI201278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201279, SGI201280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44688[] = {SGI201281, SGI0, SGI201282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201283, SGI201284, SGI201284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44689[] = {SGI201285, SGI0, SGI201286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201287, SGI201288, SGI201288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201290, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44690[] = {SGI201289, SGI0, SGI201291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201292, SGI201293, SGI201293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44691[] = {SGI201294, SGI0, SGI201295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201297, SGI201299, SGI201299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44692[] = {SGI201296, SGI0, SGI201298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201300, SGI201301, SGI201301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44693[] = {SGI201302, SGI201303, SGI201304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201305, SGI201305, SGI0, SGI0, SGI0, SGI201306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44694[] = {SGI201307, SGI0, SGI201308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201309, SGI201310, SGI201310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44695[] = {SGI201311, SGI0, SGI201312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201313, SGI201314, SGI201314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44696[] = {SGI201315, SGI0, SGI201316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201317, SGI201318, SGI201318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44697[] = {SGI201319, SGI201320, SGI201321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201322, SGI0, SGI0, SGI0, SGI201323, SGI201323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44698[] = {SGI201324, SGI0, SGI201325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201326, SGI201327, SGI201327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44699[] = {SGI201328, SGI201329, SGI201330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201331, SGI0, SGI0, SGI0, SGI201332, SGI201332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44700[] = {SGI0, SGI201333, SGI0, SGI201334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44701[] = {SGI201335, SGI201336, SGI201337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201338, SGI0, SGI0, SGI0, SGI201339, SGI201339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44702[] = {SGI201340, SGI0, SGI201341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201342, SGI201343, SGI201343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44703[] = {SGI201344, SGI0, SGI201345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201346, SGI201347, SGI201347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44704[] = {SGI201348, SGI0, SGI201349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201351, SGI201352, SGI201352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44705[] = {SGI201350, SGI0, SGI201353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201354, SGI0, SGI0, SGI0, SGI201355, SGI201355, SGI0, SGI0, SGI0, SGI201356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44706[] = {SGI201357, SGI0, SGI201358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201359, SGI201360, SGI201360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44707[] = {SGI201361, SGI201362, SGI201363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201364, SGI0, SGI0, SGI0, SGI201365, SGI201365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44708[] = {SGI201366, SGI0, SGI201367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44709[] = {SGI201368, SGI201369, SGI201370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201371, SGI0, SGI0, SGI0, SGI201372, SGI201372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44710[] = {SGI201373, SGI0, SGI201374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201377, SGI201378, SGI201378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44711[] = {SGI201375, SGI0, SGI201376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201379, SGI201380, SGI201380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44712[] = {SGI201381, SGI201382, SGI201383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201384, SGI0, SGI0, SGI0, SGI201385, SGI201385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201386, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44713[] = {SGI201387, SGI201388, SGI201389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201390, SGI0, SGI0, SGI0, SGI201391, SGI201391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201392, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44714[] = {SGI201393, SGI0, SGI201394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201395, SGI0, SGI0, SGI0, SGI201396, SGI201396, SGI0, SGI0, SGI201398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44715[] = {SGI201397, SGI201399, SGI201400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201401, SGI201401, SGI0, SGI0, SGI0, SGI201402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44716[] = {SGI201403, SGI0, SGI201404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201405, SGI0, SGI0, SGI0, SGI201406, SGI201406, SGI0, SGI0, SGI201407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44717[] = {SGI201408, SGI0, SGI201409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201410, SGI0, SGI0, SGI0, SGI201411, SGI201411, SGI0, SGI0, SGI201414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44718[] = {SGI0, SGI201412, SGI201413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201415, SGI0, SGI0, SGI0, SGI0, SGI201416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44719[] = {SGI0, SGI201417, SGI201418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201419, SGI0, SGI0, SGI0, SGI201420, SGI201420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44720[] = {SGI201421, SGI201422, SGI201423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201424, SGI0, SGI0, SGI0, SGI201425, SGI201425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44721[] = {SGI201426, SGI0, SGI201427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201428, SGI201429, SGI201429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44722[] = {SGI201430, SGI0, SGI201431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201432, SGI201433, SGI201433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44723[] = {SGI201434, SGI201435, SGI201436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201437, SGI0, SGI0, SGI0, SGI201438, SGI201438, SGI0, SGI201439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44724[] = {SGI201440, SGI201441, SGI201442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201443, SGI0, SGI0, SGI0, SGI201444, SGI201444, SGI0, SGI201445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44725[] = {SGI201446, SGI201447, SGI201448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201451, SGI0, SGI0, SGI0, SGI201452, SGI201452, SGI0, SGI201453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44726[] = {SGI201449, SGI0, SGI201450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201454, SGI201455, SGI201455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44727[] = {SGI201458, SGI0, SGI201460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201462, SGI201464, SGI201464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44728[] = {SGI201456, SGI201457, SGI201459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201461, SGI0, SGI0, SGI0, SGI201463, SGI201463, SGI0, SGI201465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44729[] = {SGI201466, SGI201467, SGI201468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201469, SGI0, SGI0, SGI0, SGI201470, SGI201470, SGI0, SGI201471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44730[] = {SGI201472, SGI0, SGI201473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201474, SGI201475, SGI201475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44731[] = {SGI201476, SGI0, SGI201477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201478, SGI201479, SGI201479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44732[] = {SGI201480, SGI0, SGI201481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201482, SGI201483, SGI201483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44733[] = {SGI201484, SGI201485, SGI201486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201487, SGI0, SGI0, SGI0, SGI201488, SGI201488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44734[] = {SGI201490, SGI201491, SGI201492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201493, SGI0, SGI0, SGI0, SGI201494, SGI201494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44735[] = {SGI201496, SGI0, SGI201497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201498, SGI201499, SGI201499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44736[] = {SGI201500, SGI201501, SGI201502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201503, SGI0, SGI0, SGI0, SGI201504, SGI201504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44737[] = {SGI201505, SGI201506, SGI201507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201508, SGI0, SGI0, SGI0, SGI201509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44738[] = {SGI201510, SGI201511, SGI201512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201513, SGI0, SGI0, SGI0, SGI201514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44739[] = {SGI201515, SGI0, SGI201516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201517, SGI201518, SGI201518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44740[] = {SGI201519, SGI201520, SGI201521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201522, SGI0, SGI0, SGI0, SGI201523, SGI201523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44741[] = {SGI201524, SGI0, SGI201525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201526, SGI201527, SGI201527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44742[] = {SGI201528, SGI0, SGI201529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201530, SGI201531, SGI201531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44743[] = {SGI201532, SGI0, SGI201533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201534, SGI201535, SGI201535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44744[] = {SGI201536, SGI0, SGI201537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201538, SGI201539, SGI201539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44745[] = {SGI201540, SGI201541, SGI201542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201543, SGI0, SGI0, SGI0, SGI201544, SGI201544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44746[] = {SGI201545, SGI0, SGI201546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201547, SGI201548, SGI201548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44747[] = {SGI201549, SGI0, SGI201550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201551, SGI201552, SGI201552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44748[] = {SGI201553, SGI0, SGI201554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201557, SGI201558, SGI201558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44749[] = {SGI201555, SGI0, SGI201556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201559, SGI201560, SGI201560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44750[] = {SGI201561, SGI0, SGI201562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201563, SGI201564, SGI201564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44751[] = {SGI201565, SGI0, SGI201566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201569, SGI201570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44752[] = {SGI201567, SGI0, SGI201568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201571, SGI201572, SGI201572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44753[] = {SGI201573, SGI0, SGI201574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201575, SGI201576, SGI201576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44754[] = {SGI201577, SGI201578, SGI201579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201582, SGI0, SGI0, SGI0, SGI201583, SGI201583, SGI0, SGI201584, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44755[] = {SGI201580, SGI0, SGI201581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201585, SGI201586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44756[] = {SGI201587, SGI0, SGI201588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201589, SGI201590, SGI201590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44757[] = {SGI201591, SGI0, SGI201592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201593, SGI201594, SGI201594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44758[] = {SGI201595, SGI201596, SGI201597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201598, SGI0, SGI0, SGI0, SGI201599, SGI201599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44759[] = {SGI201600, SGI201601, SGI201602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201603, SGI0, SGI0, SGI0, SGI201604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44760[] = {SGI201605, SGI201606, SGI201607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201608, SGI0, SGI0, SGI0, SGI201609, SGI201609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44761[] = {SGI201610, SGI201611, SGI201612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201613, SGI0, SGI0, SGI0, SGI201614, SGI201614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44762[] = {SGI201615, SGI0, SGI201616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44763[] = {SGI201617, SGI201618, SGI201619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201620, SGI0, SGI0, SGI0, SGI201621, SGI201621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201622, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44764[] = {SGI201623, SGI201624, SGI201625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201626, SGI0, SGI0, SGI0, SGI201627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44765[] = {SGI201628, SGI201629, SGI201630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201631, SGI0, SGI0, SGI0, SGI201632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44766[] = {SGI201633, SGI201634, SGI201635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201636, SGI0, SGI0, SGI0, SGI201637, SGI201637, SGI0, SGI0, SGI201638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44767[] = {SGI201639, SGI0, SGI201640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201643, SGI201644, SGI201644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44768[] = {SGI201641, SGI0, SGI201642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201645, SGI201646, SGI201646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44769[] = {SGI201647, SGI0, SGI201648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201649, SGI201650, SGI201650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44770[] = {SGI201651, SGI0, SGI201652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201653, SGI0, SGI0, SGI0, SGI201654, SGI201654, SGI0, SGI0, SGI0, SGI0, SGI201655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44771[] = {SGI201656, SGI201657, SGI201658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201661, SGI0, SGI0, SGI0, SGI201662, SGI201662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44772[] = {SGI201659, SGI0, SGI201660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201663, SGI201664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44773[] = {SGI201665, SGI0, SGI201666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201667, SGI0, SGI0, SGI0, SGI201668, SGI201668, SGI0, SGI0, SGI0, SGI0, SGI201671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44774[] = {SGI201669, SGI201670, SGI201672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201673, SGI0, SGI0, SGI0, SGI201674, SGI201674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44775[] = {SGI201675, SGI201676, SGI201677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201678, SGI0, SGI0, SGI0, SGI201679, SGI201680, SGI0, SGI0, SGI0, SGI0, SGI201681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44776[] = {SGI201682, SGI0, SGI201683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201684, SGI201685, SGI201685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44777[] = {SGI201686, SGI0, SGI201687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201688, SGI201689, SGI201689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44778[] = {SGI201690, SGI201691, SGI201692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201693, SGI201693, SGI0, SGI0, SGI0, SGI201694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44779[] = {SGI201695, SGI0, SGI201696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201697, SGI201698, SGI201698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44780[] = {SGI201699, SGI201700, SGI201701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201702, SGI0, SGI0, SGI0, SGI201705, SGI201705, SGI0, SGI201709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44781[] = {SGI201703, SGI201704, SGI201707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201710, SGI0, SGI0, SGI0, SGI201711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44782[] = {SGI201706, SGI0, SGI201708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201712, SGI201713, SGI201713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44783[] = {SGI201715, SGI201716, SGI201717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201718, SGI0, SGI0, SGI0, SGI201719, SGI201719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44784[] = {SGI201721, SGI201722, SGI201723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201724, SGI0, SGI0, SGI0, SGI201725, SGI201725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44785[] = {SGI201727, SGI201728, SGI201729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201730, SGI0, SGI0, SGI0, SGI201731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44786[] = {SGI201733, SGI201734, SGI201735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201736, SGI201736, SGI0, SGI0, SGI0, SGI0, SGI201737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44787[] = {SGI201738, SGI0, SGI201739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201740, SGI201741, SGI201741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44788[] = {SGI201742, SGI0, SGI201743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201744, SGI201745, SGI201745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44789[] = {SGI201746, SGI0, SGI201747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201748, SGI201749, SGI201749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44790[] = {SGI201750, SGI0, SGI201751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201752, SGI201753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44791[] = {SGI201755, SGI201757, SGI201758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201761, SGI0, SGI0, SGI0, SGI201763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44792[] = {SGI201754, SGI0, SGI201756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201762, SGI0, SGI0, SGI0, SGI201764, SGI201764, SGI0, SGI0, SGI201766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44793[] = {SGI201759, SGI0, SGI201760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201765, SGI201767, SGI201767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44794[] = {SGI201768, SGI201770, SGI201771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201773, SGI0, SGI0, SGI0, SGI201775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44795[] = {SGI201769, SGI0, SGI201772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201774, SGI201776, SGI201776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44796[] = {SGI201777, SGI0, SGI201778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201779, SGI201780, SGI201780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44797[] = {SGI201781, SGI201782, SGI201783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201784, SGI0, SGI0, SGI0, SGI201785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44798[] = {SGI201786, SGI0, SGI201787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201788, SGI201789, SGI201789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44799[] = {SGI201790, SGI0, SGI201791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201792, SGI0, SGI0, SGI0, SGI201793, SGI201793, SGI0, SGI0, SGI0, SGI201794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44800[] = {SGI201795, SGI201796, SGI201797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201798, SGI0, SGI0, SGI0, SGI201799, SGI201800, SGI0, SGI0, SGI0, SGI201801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44801[] = {SGI201802, SGI201803, SGI201804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201805, SGI201805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44802[] = {SGI201806, SGI201807, SGI201808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201809, SGI0, SGI0, SGI0, SGI201810, SGI201810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201811, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44803[] = {SGI201812, SGI0, SGI201813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201814, SGI201815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44804[] = {SGI201816, SGI201817, SGI201818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201819, SGI0, SGI0, SGI0, SGI201820, SGI201820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44805[] = {SGI201821, SGI0, SGI201822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201823, SGI201824, SGI201824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44806[] = {SGI201825, SGI0, SGI201826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201830, SGI201832, SGI201832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201835, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44807[] = {SGI201827, SGI201828, SGI201829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201831, SGI0, SGI0, SGI0, SGI201833, SGI201833, SGI0, SGI201834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44808[] = {SGI201836, SGI201837, SGI201838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201839, SGI201839, SGI0, SGI0, SGI0, SGI201840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44809[] = {SGI201841, SGI0, SGI201842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201843, SGI201844, SGI201844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44810[] = {SGI201845, SGI0, SGI201846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201847, SGI201848, SGI201848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44811[] = {SGI201849, SGI0, SGI201850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201851, SGI201852, SGI201852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44812[] = {SGI201853, SGI0, SGI201854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201855, SGI201856, SGI201856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44813[] = {SGI201857, SGI0, SGI201858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201859, SGI201860, SGI201860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44814[] = {SGI201861, SGI0, SGI201862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201863, SGI201864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44815[] = {SGI201865, SGI201866, SGI201867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201868, SGI0, SGI0, SGI0, SGI201869, SGI201869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44816[] = {SGI201870, SGI201871, SGI201872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201874, SGI0, SGI0, SGI0, SGI201876, SGI201876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44817[] = {SGI201873, SGI0, SGI201875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201877, SGI201878, SGI201878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44818[] = {SGI201879, SGI0, SGI201880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201881, SGI201882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44819[] = {SGI201883, SGI201884, SGI201885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201886, SGI0, SGI0, SGI0, SGI201887, SGI201887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44820[] = {SGI201888, SGI201889, SGI201890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201891, SGI0, SGI0, SGI0, SGI201892, SGI201892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44821[] = {SGI201893, SGI201894, SGI201895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201896, SGI0, SGI0, SGI0, SGI201897, SGI201897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44822[] = {SGI201899, SGI201900, SGI201901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201902, SGI0, SGI0, SGI0, SGI201903, SGI201903, SGI0, SGI201904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44823[] = {SGI201905, SGI201906, SGI201907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201908, SGI0, SGI0, SGI0, SGI201909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44824[] = {SGI201910, SGI201911, SGI201912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201913, SGI0, SGI0, SGI0, SGI201914, SGI201914, SGI0, SGI201915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44825[] = {SGI201917, SGI0, SGI201919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44826[] = {SGI201916, SGI0, SGI201918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201920, SGI201921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44827[] = {SGI201922, SGI0, SGI201923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201924, SGI201925, SGI201925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44828[] = {SGI201926, SGI201927, SGI201928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201929, SGI0, SGI0, SGI0, SGI201930, SGI201930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44829[] = {SGI201931, SGI201933, SGI201934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201937, SGI0, SGI0, SGI0, SGI201940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44830[] = {SGI201932, SGI201935, SGI201936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201938, SGI0, SGI0, SGI0, SGI201942, SGI201942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44831[] = {SGI201939, SGI201941, SGI201943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201944, SGI0, SGI0, SGI0, SGI201945, SGI201945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44832[] = {SGI201946, SGI0, SGI201947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44833[] = {SGI201948, SGI201949, SGI201950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201956, SGI0, SGI0, SGI0, SGI201958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44834[] = {SGI201951, SGI201952, SGI201953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201959, SGI0, SGI0, SGI0, SGI201960, SGI201960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44835[] = {SGI201954, SGI201955, SGI201957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201961, SGI0, SGI0, SGI0, SGI201962, SGI201962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44836[] = {SGI201963, SGI0, SGI201964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44837[] = {SGI201965, SGI0, SGI201966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44838[] = {SGI201967, SGI0, SGI201968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201969, SGI201970, SGI201970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44839[] = {SGI201971, SGI0, SGI201972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201973, SGI201974, SGI201974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44840[] = {SGI201975, SGI0, SGI201976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201977, SGI201978, SGI201978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44841[] = {SGI201979, SGI0, SGI201980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201981, SGI201982, SGI201982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44842[] = {SGI201983, SGI0, SGI201984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201985, SGI201986, SGI201986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44843[] = {SGI201987, SGI0, SGI201988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201989, SGI201990, SGI201990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44844[] = {SGI201991, SGI0, SGI201992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201993, SGI201994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44845[] = {SGI201995, SGI0, SGI201996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI201997, SGI201998, SGI201998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44846[] = {SGI201999, SGI0, SGI202000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202001, SGI202002, SGI202002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44847[] = {SGI202003, SGI0, SGI202004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202005, SGI202006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44848[] = {SGI202007, SGI0, SGI202008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202009, SGI202010, SGI202010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44849[] = {SGI202012, SGI0, SGI202014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202017, SGI202018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44850[] = {SGI202011, SGI202013, SGI202015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202016, SGI0, SGI0, SGI0, SGI202019, SGI202019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44851[] = {SGI202020, SGI0, SGI202022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202025, SGI202027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44852[] = {SGI202021, SGI202023, SGI202024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202026, SGI0, SGI0, SGI0, SGI202028, SGI202028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44853[] = {SGI202029, SGI0, SGI202030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44854[] = {SGI202031, SGI0, SGI202032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202033, SGI202034, SGI202034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44855[] = {SGI202035, SGI202036, SGI202037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202038, SGI0, SGI0, SGI0, SGI202039, SGI0, SGI0, SGI202040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44856[] = {SGI202041, SGI0, SGI202042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202043, SGI202044, SGI202044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44857[] = {SGI202045, SGI0, SGI202046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202047, SGI202048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44858[] = {SGI202049, SGI202050, SGI202051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202053, SGI0, SGI0, SGI0, SGI202055, SGI202055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202056, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44859[] = {SGI202052, SGI0, SGI202054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202057, SGI202058, SGI202058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44860[] = {SGI202059, SGI202060, SGI202061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202062, SGI0, SGI0, SGI0, SGI202063, SGI202063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202064, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44861[] = {SGI202065, SGI202066, SGI202067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202068, SGI0, SGI0, SGI0, SGI202069, SGI202069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202070, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44862[] = {SGI202071, SGI202072, SGI202073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202074, SGI0, SGI0, SGI0, SGI202075, SGI202075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202076, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44863[] = {SGI202077, SGI202078, SGI202079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202080, SGI0, SGI0, SGI0, SGI202081, SGI202081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202082, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44864[] = {SGI202083, SGI0, SGI202084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202085, SGI202086, SGI202086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44865[] = {SGI202087, SGI0, SGI202088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202089, SGI202090, SGI202090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202091, SGI0, SGI0, NULL};
const Segment * const SG44866[] = {SGI202092, SGI0, SGI202093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202094, SGI202095, SGI202095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44867[] = {SGI202096, SGI0, SGI202097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202098, SGI202099, SGI202099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44868[] = {SGI202100, SGI0, SGI202101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202102, SGI202103, SGI202103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44869[] = {SGI202104, SGI202105, SGI202106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202107, SGI0, SGI0, SGI0, SGI202108, SGI202108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44870[] = {SGI202109, SGI202110, SGI202111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202112, SGI0, SGI0, SGI0, SGI202113, SGI202113, SGI0, SGI202114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44871[] = {SGI202115, SGI202116, SGI202117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202118, SGI0, SGI0, SGI0, SGI202119, SGI202119, SGI0, SGI202120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44872[] = {SGI202121, SGI0, SGI202122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202123, SGI202124, SGI202124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44873[] = {SGI202125, SGI0, SGI202126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202127, SGI202128, SGI202128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44874[] = {SGI202129, SGI0, SGI202131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202133, SGI202134, SGI202134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44875[] = {SGI202130, SGI0, SGI202132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202135, SGI202136, SGI202136, SGI0, SGI0, SGI0, SGI0, SGI202137, SGI0, NULL};
const Segment * const SG44876[] = {SGI202138, SGI0, SGI202139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202140, SGI202141, SGI202141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44877[] = {SGI202142, SGI202143, SGI202144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202145, SGI0, SGI0, SGI0, SGI202146, SGI202146, SGI202147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44878[] = {SGI202148, SGI0, SGI202149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202150, SGI202151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44879[] = {SGI202152, SGI0, SGI202153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202154, SGI202155, SGI202155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44880[] = {SGI202156, SGI0, SGI202157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202158, SGI202159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44881[] = {SGI202160, SGI202161, SGI202162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202163, SGI0, SGI0, SGI0, SGI202164, SGI0, SGI202165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44882[] = {SGI202166, SGI202167, SGI202168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202169, SGI0, SGI0, SGI0, SGI202170, SGI202170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44883[] = {SGI202171, SGI202172, SGI202173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202175, SGI0, SGI0, SGI0, SGI202177, SGI202177, SGI0, SGI202178, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44884[] = {SGI202174, SGI0, SGI202176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202179, SGI202180, SGI202180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44885[] = {SGI202181, SGI0, SGI202182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202183, SGI202184, SGI202184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44886[] = {SGI202185, SGI202186, SGI202187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202188, SGI0, SGI0, SGI0, SGI202189, SGI202189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44887[] = {SGI202190, SGI202191, SGI202192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202196, SGI202196, SGI0, SGI0, SGI0, SGI202198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44888[] = {SGI202193, SGI202194, SGI202195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202197, SGI0, SGI0, SGI0, SGI202199, SGI202199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202200, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44889[] = {SGI202201, SGI202202, SGI202204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202208, SGI202208, SGI0, SGI0, SGI0, SGI202210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44890[] = {SGI202203, SGI202205, SGI202206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202207, SGI0, SGI0, SGI0, SGI202209, SGI202209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202211, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44891[] = {SGI202212, SGI0, SGI202213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202214, SGI202215, SGI202215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44892[] = {SGI202216, SGI0, SGI202218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202220, SGI202221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44893[] = {SGI202224, SGI0, SGI202225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202226, SGI202227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44894[] = {SGI202228, SGI202229, SGI202230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202231, SGI0, SGI0, SGI0, SGI202232, SGI202232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44895[] = {SGI202233, SGI202234, SGI202235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202236, SGI0, SGI0, SGI0, SGI202237, SGI202237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44896[] = {SGI202238, SGI0, SGI202239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44897[] = {SGI202240, SGI0, SGI202241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202244, SGI202246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44898[] = {SGI202242, SGI202243, SGI202245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202247, SGI0, SGI0, SGI0, SGI202248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44899[] = {SGI202249, SGI0, SGI202250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202251, SGI0, SGI0, SGI0, SGI202252, SGI202252, SGI0, SGI0, SGI0, SGI202253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44900[] = {SGI202254, SGI202255, SGI202256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202257, SGI0, SGI0, SGI0, SGI202258, SGI202258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44901[] = {SGI202259, SGI0, SGI202260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44902[] = {SGI202261, SGI202262, SGI202263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202265, SGI0, SGI0, SGI0, SGI202268, SGI202268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202269, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44903[] = {SGI202264, SGI202266, SGI202267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202270, SGI202270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44904[] = {SGI202271, SGI0, SGI202272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202273, SGI202274, SGI202274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44905[] = {SGI202275, SGI202276, SGI202277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202278, SGI0, SGI0, SGI0, SGI202279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44906[] = {SGI202280, SGI0, SGI202281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202282, SGI202283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44907[] = {SGI202284, SGI202285, SGI202286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202287, SGI0, SGI0, SGI0, SGI202288, SGI202288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202289, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44908[] = {SGI202290, SGI202291, SGI202292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202293, SGI202293, SGI0, SGI0, SGI0, SGI202294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44909[] = {SGI202295, SGI202296, SGI202297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202302, SGI0, SGI0, SGI0, SGI202304, SGI0, SGI0, SGI202305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44910[] = {SGI202299, SGI202300, SGI202303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202306, SGI0, SGI0, SGI0, SGI202309, SGI202309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44911[] = {SGI202298, SGI0, SGI202301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202307, SGI202308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44912[] = {SGI202310, SGI202311, SGI202313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202316, SGI0, SGI0, SGI0, SGI202318, SGI0, SGI0, SGI202320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44913[] = {SGI202312, SGI202314, SGI202315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202317, SGI0, SGI0, SGI0, SGI202319, SGI202319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44914[] = {SGI202322, SGI202324, SGI202326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202328, SGI0, SGI0, SGI0, SGI202330, SGI202330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44915[] = {SGI202321, SGI202323, SGI202325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202327, SGI0, SGI0, SGI0, SGI202329, SGI202329, SGI0, SGI202331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44916[] = {SGI202332, SGI202333, SGI202334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202335, SGI202335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202336, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44917[] = {SGI202337, SGI202338, SGI202339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202340, SGI0, SGI0, SGI0, SGI202341, SGI0, SGI0, SGI202342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44918[] = {SGI202343, SGI0, SGI202344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202345, SGI202346, SGI202346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44919[] = {SGI202347, SGI0, SGI202348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202349, SGI202350, SGI202350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44920[] = {SGI202351, SGI0, SGI202352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202353, SGI202354, SGI202354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44921[] = {SGI0, SGI202355, SGI0, SGI202356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44922[] = {SGI202357, SGI0, SGI202358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44923[] = {SGI202359, SGI202360, SGI202361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202362, SGI0, SGI0, SGI0, SGI202363, SGI202364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44924[] = {SGI202365, SGI202366, SGI202367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202368, SGI0, SGI0, SGI0, SGI202369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44925[] = {SGI202217, SGI0, SGI202219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202222, SGI202223, SGI202223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44926[] = {SGI202370, SGI202371, SGI202372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202373, SGI0, SGI0, SGI0, SGI202374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44927[] = {SGI202375, SGI0, SGI202376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202377, SGI202378, SGI202378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44928[] = {SGI202379, SGI0, SGI202380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202381, SGI202382, SGI202382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44929[] = {SGI202383, SGI0, SGI202384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202385, SGI202386, SGI202386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44930[] = {SGI202388, SGI202390, SGI202391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202392, SGI0, SGI0, SGI0, SGI202394, SGI202394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202396, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44931[] = {SGI202387, SGI0, SGI202389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202393, SGI202395, SGI202395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202397, SGI0, SGI0, NULL};
const Segment * const SG44932[] = {SGI202398, SGI0, SGI202399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202403, SGI202405, SGI202405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202407, SGI0, SGI0, NULL};
const Segment * const SG44933[] = {SGI202400, SGI202401, SGI202402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202404, SGI0, SGI0, SGI0, SGI202406, SGI202406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202408, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44934[] = {SGI202409, SGI0, SGI202411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202415, SGI202416, SGI202416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202418, SGI0, SGI0, NULL};
const Segment * const SG44935[] = {SGI202410, SGI202412, SGI202413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202414, SGI0, SGI0, SGI0, SGI202417, SGI202417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44936[] = {SGI202420, SGI0, SGI202422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202425, SGI202427, SGI202427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44937[] = {SGI202419, SGI202421, SGI202423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202424, SGI0, SGI0, SGI0, SGI202426, SGI202426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44938[] = {SGI202428, SGI0, SGI202429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202430, SGI202431, SGI202431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44939[] = {SGI202432, SGI202433, SGI202434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202435, SGI0, SGI0, SGI0, SGI202436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44940[] = {SGI202437, SGI0, SGI202438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202439, SGI202440, SGI202440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44941[] = {SGI202441, SGI0, SGI202442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202443, SGI202444, SGI202444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44942[] = {SGI202445, SGI0, SGI202446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202447, SGI202448, SGI202448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44943[] = {SGI202449, SGI0, SGI202450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202451, SGI202452, SGI202452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44944[] = {SGI202453, SGI0, SGI202454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202455, SGI202456, SGI202456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44945[] = {SGI202457, SGI202458, SGI202459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202460, SGI202460, SGI0, SGI0, SGI0, SGI202461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44946[] = {SGI202462, SGI202463, SGI202464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202467, SGI202467, SGI0, SGI0, SGI0, SGI202468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44947[] = {SGI202465, SGI0, SGI202466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202469, SGI202470, SGI202470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44948[] = {SGI202471, SGI0, SGI202472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202473, SGI202474, SGI202474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44949[] = {SGI202475, SGI0, SGI202476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202477, SGI202478, SGI202478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44950[] = {SGI202479, SGI0, SGI202480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202481, SGI202482, SGI202482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44951[] = {SGI202483, SGI0, SGI202484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202485, SGI202486, SGI202486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44952[] = {SGI202487, SGI202488, SGI202489, SGI202490, SGI202491, SGI0, SGI0, NULL};
const Segment * const SG44953[] = {SGI202492, SGI202493, SGI202494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202495, SGI0, SGI0, SGI0, SGI202496, SGI202496, SGI0, SGI0, SGI0, SGI202497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44954[] = {SGI202498, SGI0, SGI202499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202500, SGI202501, SGI202501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44955[] = {SGI202502, SGI0, SGI202503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202504, SGI202505, SGI202505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44956[] = {SGI202506, SGI0, SGI202507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202508, SGI202509, SGI202509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44957[] = {SGI202510, SGI0, SGI202511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202512, SGI202513, SGI202513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44958[] = {SGI202514, SGI0, SGI202516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202518, SGI202520, SGI202520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44959[] = {SGI202515, SGI0, SGI202517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202522, SGI202524, SGI202524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44960[] = {SGI202519, SGI202521, SGI202523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202525, SGI0, SGI0, SGI0, SGI202526, SGI202526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202527, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44961[] = {SGI202528, SGI0, SGI202529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202530, SGI202531, SGI202531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44962[] = {SGI202532, SGI0, SGI202533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202534, SGI202535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44963[] = {SGI202536, SGI0, SGI202537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202538, SGI202539, SGI202539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44964[] = {SGI202540, SGI202541, SGI202542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202543, SGI0, SGI0, SGI0, SGI202544, SGI202544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44965[] = {SGI202545, SGI0, SGI202546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202547, SGI202548, SGI202548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44966[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202551, SGI0, SGI0, SGI202553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44967[] = {SGI202549, SGI0, SGI202550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202552, SGI202554, SGI202554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44968[] = {SGI202555, SGI0, SGI202556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202557, SGI202558, SGI202558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44969[] = {SGI202559, SGI0, SGI202560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202561, SGI202562, SGI202562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44970[] = {SGI202563, SGI202564, SGI202565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202566, SGI0, SGI0, SGI0, SGI202567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44971[] = {SGI202568, SGI202569, SGI202570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202571, SGI0, SGI0, SGI0, SGI202572, SGI202572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202574, SGI0, NULL};
const Segment * const SG44972[] = {SGI202575, SGI202576, SGI202577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202578, SGI0, SGI0, SGI0, SGI202579, SGI202579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202581, SGI0, NULL};
const Segment * const SG44973[] = {SGI202582, SGI0, SGI202583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202584, SGI202585, SGI202585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44974[] = {SGI202586, SGI0, SGI202587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202588, SGI202589, SGI202589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44975[] = {SGI202590, SGI0, SGI202591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202592, SGI202593, SGI202593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44976[] = {SGI202594, SGI0, SGI202595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202596, SGI202597, SGI202597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202598, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44977[] = {SGI202599, SGI0, SGI202600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202601, SGI202602, SGI202602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44978[] = {SGI202603, SGI0, SGI202604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202605, SGI202606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44979[] = {SGI202607, SGI0, SGI202608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202609, SGI202610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44980[] = {SGI202611, SGI0, SGI202612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202613, SGI202614, SGI202614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44981[] = {SGI0, SGI202615, SGI202616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44982[] = {SGI202617, SGI202618, SGI202619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202620, SGI0, SGI0, SGI0, SGI202621, SGI202621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44983[] = {SGI202622, SGI202623, SGI202624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202625, SGI202625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44984[] = {SGI202626, SGI0, SGI202627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44985[] = {SGI202628, SGI202629, SGI202630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202632, SGI0, SGI0, SGI0, SGI202634, SGI202634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44986[] = {SGI202631, SGI0, SGI202633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202635, SGI202636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44987[] = {SGI202637, SGI0, SGI202638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44988[] = {SGI202639, SGI0, SGI202640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202641, SGI202642, SGI202642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44989[] = {SGI202643, SGI202644, SGI202645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202646, SGI0, SGI0, SGI0, SGI202647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44990[] = {SGI202648, SGI202649, SGI202651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202653, SGI0, SGI0, SGI0, SGI202654, SGI202654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44991[] = {SGI202650, SGI0, SGI202652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202655, SGI202656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44992[] = {SGI202657, SGI0, SGI202658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202659, SGI202660, SGI202660, SGI0, SGI0, SGI0, SGI0, SGI202661, SGI0, NULL};
const Segment * const SG44993[] = {SGI202662, SGI0, SGI202663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202664, SGI202666, SGI202666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44994[] = {SGI202665, SGI202667, SGI202668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202669, SGI0, SGI0, SGI0, SGI202670, SGI202670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202671, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44995[] = {SGI202672, SGI202673, SGI202674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202675, SGI0, SGI0, SGI0, SGI202676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44996[] = {SGI202677, SGI202678, SGI202679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202680, SGI0, SGI0, SGI0, SGI202681, SGI202681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44997[] = {SGI202682, SGI202683, SGI202684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202685, SGI0, SGI0, SGI0, SGI202686, SGI202686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44998[] = {SGI202687, SGI0, SGI202688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202689, SGI202690, SGI202690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG44999[] = {SGI202691, SGI0, SGI202692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202693, SGI202694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45000[] = {SGI202695, SGI0, SGI202696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202697, SGI202698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45001[] = {SGI202699, SGI0, SGI202700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202701, SGI202702, SGI202702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45002[] = {SGI202703, SGI0, SGI202704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202705, SGI202706, SGI202706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45003[] = {SGI202707, SGI0, SGI202710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202713, SGI202714, SGI202714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45004[] = {SGI202708, SGI202709, SGI202711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202712, SGI0, SGI0, SGI0, SGI202715, SGI202715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45005[] = {SGI202717, SGI0, SGI202719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45006[] = {SGI202716, SGI0, SGI202718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202720, SGI202721, SGI202721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45007[] = {SGI202722, SGI0, SGI202723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202724, SGI202725, SGI202725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45008[] = {SGI202726, SGI0, SGI202727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202728, SGI202730, SGI202730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45009[] = {SGI202729, SGI202731, SGI202732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202733, SGI202733, SGI0, SGI0, SGI202734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45010[] = {SGI202735, SGI0, SGI202736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202737, SGI202738, SGI202738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45011[] = {SGI202739, SGI0, SGI202740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202741, SGI202742, SGI202742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45012[] = {SGI202743, SGI0, SGI202744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202745, SGI202746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45013[] = {SGI202747, SGI0, SGI202748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202752, SGI202753, SGI202753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45014[] = {SGI202749, SGI202750, SGI202751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202754, SGI0, SGI0, SGI0, SGI202755, SGI202755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45015[] = {SGI202756, SGI0, SGI202757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45016[] = {SGI202758, SGI0, SGI202760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202762, SGI202764, SGI202764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45017[] = {SGI202759, SGI0, SGI202761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202763, SGI202765, SGI202765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45018[] = {SGI202766, SGI202767, SGI202768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202769, SGI0, SGI0, SGI0, SGI202770, SGI202770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45019[] = {SGI202771, SGI202772, SGI202773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202774, SGI0, SGI0, SGI0, SGI202776, SGI202776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45020[] = {SGI202775, SGI202777, SGI202778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202779, SGI0, SGI0, SGI0, SGI202780, SGI202780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202781, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45021[] = {SGI202782, SGI0, SGI202783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45022[] = {SGI202784, SGI202785, SGI202786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202787, SGI0, SGI0, SGI0, SGI202788, SGI202788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45023[] = {SGI202789, SGI0, SGI202790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45024[] = {SGI202791, SGI0, SGI202792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202793, SGI202794, SGI202794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45025[] = {SGI202795, SGI0, SGI202796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202797, SGI202798, SGI202798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45026[] = {SGI202799, SGI0, SGI202800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202801, SGI202802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45027[] = {SGI202803, SGI0, SGI202804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202808, SGI202809, SGI202809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202812, SGI0, SGI0, NULL};
const Segment * const SG45028[] = {SGI202805, SGI0, SGI202806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202811, SGI202813, SGI202813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45029[] = {SGI202807, SGI0, SGI202810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202814, SGI202815, SGI202815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45030[] = {SGI202816, SGI0, SGI202817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202818, SGI202819, SGI202819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45031[] = {SGI202820, SGI0, SGI202821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202822, SGI202823, SGI202823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45032[] = {SGI202824, SGI0, SGI202825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202826, SGI202827, SGI202827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45033[] = {SGI202828, SGI202829, SGI202830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202831, SGI0, SGI0, SGI0, SGI202832, SGI202832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202833, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45034[] = {SGI202834, SGI202835, SGI202836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202837, SGI0, SGI0, SGI0, SGI202838, SGI202838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202839, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45035[] = {SGI202840, SGI202841, SGI202842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202844, SGI0, SGI0, SGI0, SGI202846, SGI0, SGI0, SGI202847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45036[] = {SGI202843, SGI0, SGI202845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202848, SGI202849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45037[] = {SGI202850, SGI202851, SGI202852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202853, SGI0, SGI0, SGI0, SGI202854, SGI0, SGI0, SGI202855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45038[] = {SGI202856, SGI0, SGI202857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202858, SGI202859, SGI202859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45039[] = {SGI202860, SGI0, SGI202861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202864, SGI202865, SGI202865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45040[] = {SGI202862, SGI0, SGI202863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202866, SGI202867, SGI202867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45041[] = {SGI202868, SGI0, SGI202869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202870, SGI202871, SGI202871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45042[] = {SGI202872, SGI0, SGI202873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202874, SGI202875, SGI202875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45043[] = {SGI202876, SGI0, SGI202877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202878, SGI202879, SGI202879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45044[] = {SGI202880, SGI0, SGI202881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202882, SGI202883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45045[] = {SGI202884, SGI0, SGI202885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202886, SGI202887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45046[] = {SGI202888, SGI0, SGI202889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202890, SGI202891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45047[] = {SGI202892, SGI0, SGI202893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202894, SGI202895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45048[] = {SGI202896, SGI0, SGI202897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202898, SGI202899, SGI202899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45049[] = {SGI202900, SGI0, SGI202901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202902, SGI202903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45050[] = {SGI202904, SGI0, SGI202905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202906, SGI202907, SGI202907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202908, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45051[] = {SGI202909, SGI202910, SGI202911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202912, SGI0, SGI0, SGI0, SGI202913, SGI0, SGI0, SGI202914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45052[] = {SGI202915, SGI202916, SGI202917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202918, SGI0, SGI0, SGI0, SGI202919, SGI202919, SGI0, SGI202920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45053[] = {SGI202921, SGI202922, SGI202923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202924, SGI0, SGI0, SGI0, SGI202925, SGI202925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45054[] = {SGI202926, SGI202927, SGI202928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202929, SGI0, SGI0, SGI0, SGI202930, SGI202930, SGI0, SGI202931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45055[] = {SGI202932, SGI202933, SGI202934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202935, SGI0, SGI0, SGI0, SGI202936, SGI202936, SGI0, SGI202937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45056[] = {SGI202938, SGI202939, SGI202940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202941, SGI0, SGI0, SGI0, SGI202942, SGI202942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202943, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45057[] = {SGI202944, SGI202945, SGI202946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202950, SGI0, SGI0, SGI0, SGI202951, SGI202951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202953, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45058[] = {SGI202947, SGI202948, SGI202949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202952, SGI0, SGI0, SGI0, SGI202954, SGI202954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45059[] = {SGI202955, SGI202956, SGI202957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202958, SGI0, SGI0, SGI0, SGI202959, SGI202959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202960, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45060[] = {SGI202961, SGI0, SGI202962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202963, SGI202964, SGI202964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45061[] = {SGI202965, SGI0, SGI202966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202967, SGI202968, SGI202968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45062[] = {SGI202969, SGI202970, SGI202971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202972, SGI0, SGI0, SGI0, SGI202973, SGI202973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45063[] = {SGI202974, SGI202975, SGI202976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202977, SGI0, SGI0, SGI0, SGI202978, SGI202978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45064[] = {SGI202979, SGI202980, SGI202981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202982, SGI0, SGI0, SGI0, SGI202983, SGI202983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45065[] = {SGI202984, SGI0, SGI202985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202986, SGI202987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45066[] = {SGI202988, SGI0, SGI202989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202993, SGI202994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45067[] = {SGI202990, SGI202991, SGI202992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202995, SGI0, SGI0, SGI0, SGI202996, SGI202996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI202997, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45068[] = {SGI202998, SGI202999, SGI203000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203001, SGI0, SGI0, SGI0, SGI203002, SGI203002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45069[] = {SGI203003, SGI203004, SGI203006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203011, SGI0, SGI0, SGI0, SGI203013, SGI203013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203017, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45070[] = {SGI203005, SGI203007, SGI203009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203012, SGI0, SGI0, SGI0, SGI203014, SGI203014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45071[] = {SGI203008, SGI0, SGI203010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203015, SGI203016, SGI203016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45072[] = {SGI203018, SGI203019, SGI203020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203021, SGI0, SGI0, SGI0, SGI203022, SGI203022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45073[] = {SGI203023, SGI203024, SGI203025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203026, SGI0, SGI0, SGI0, SGI203027, SGI203027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203028, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45074[] = {SGI203029, SGI203030, SGI203031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203032, SGI203032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45075[] = {SGI203033, SGI0, SGI203035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203037, SGI203038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45076[] = {SGI203034, SGI0, SGI203036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203039, SGI203040, SGI203040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45077[] = {SGI203041, SGI0, SGI203042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203043, SGI203044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45078[] = {SGI203045, SGI0, SGI203046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203047, SGI203048, SGI203048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45079[] = {SGI203049, SGI203050, SGI203051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203052, SGI0, SGI0, SGI0, SGI203053, SGI203053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45080[] = {SGI203054, SGI203055, SGI203056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203057, SGI0, SGI0, SGI0, SGI203058, SGI203058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203059, SGI0, SGI0, NULL};
const Segment * const SG45081[] = {SGI203060, SGI203061, SGI203062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203063, SGI0, SGI0, SGI0, SGI203064, SGI203064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203065, SGI0, SGI0, NULL};
const Segment * const SG45082[] = {SGI203066, SGI0, SGI203067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203068, SGI203069, SGI203069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45083[] = {SGI203070, SGI0, SGI203071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203072, SGI203073, SGI203073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45084[] = {SGI203074, SGI0, SGI203075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203076, SGI203077, SGI203077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203078, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45085[] = {SGI203079, SGI0, SGI203080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45086[] = {SGI203081, SGI203082, SGI203083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203084, SGI0, SGI0, SGI0, SGI203085, SGI203085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45087[] = {SGI203086, SGI203087, SGI203088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203089, SGI0, SGI0, SGI0, SGI203090, SGI203090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45088[] = {SGI203091, SGI203092, SGI203093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203094, SGI0, SGI0, SGI0, SGI203095, SGI203095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45089[] = {SGI203097, SGI203098, SGI203099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203100, SGI0, SGI0, SGI0, SGI203101, SGI203101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45090[] = {SGI203103, SGI203104, SGI203105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203106, SGI0, SGI0, SGI0, SGI203107, SGI203107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203109, SGI0, NULL};
const Segment * const SG45091[] = {SGI203110, SGI203111, SGI203112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203113, SGI0, SGI0, SGI0, SGI203114, SGI203114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203116, SGI0, NULL};
const Segment * const SG45092[] = {SGI203117, SGI0, SGI203118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203119, SGI203120, SGI203120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203121, SGI0, SGI0, NULL};
const Segment * const SG45093[] = {SGI203122, SGI0, SGI203123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203124, SGI203125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45094[] = {SGI203126, SGI0, SGI203127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203128, SGI0, SGI0, SGI0, SGI203129, SGI203129, SGI0, SGI0, SGI0, SGI203130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45095[] = {SGI203131, SGI0, SGI203132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203133, SGI203134, SGI203134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45096[] = {SGI203135, SGI0, SGI203136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203138, SGI203140, SGI203140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45097[] = {SGI203137, SGI203139, SGI203141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203142, SGI0, SGI0, SGI0, SGI203143, SGI203143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45098[] = {SGI203144, SGI0, SGI203145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203146, SGI203147, SGI203147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203148, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45099[] = {SGI203149, SGI0, SGI203150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203151, SGI203152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45100[] = {SGI203153, SGI0, SGI203154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203155, SGI203156, SGI203156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45101[] = {SGI203157, SGI203158, SGI203159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203160, SGI0, SGI0, SGI0, SGI203161, SGI203161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45102[] = {SGI203162, SGI0, SGI203163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45103[] = {SGI203164, SGI0, SGI203165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203166, SGI203167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203168, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45104[] = {SGI203169, SGI0, SGI203170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203171, SGI203172, SGI203172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203173, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45105[] = {SGI203174, SGI0, SGI203175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203176, SGI203177, SGI203177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45106[] = {SGI203178, SGI0, SGI203179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203180, SGI203181, SGI203181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45107[] = {SGI203182, SGI0, SGI203183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203184, SGI203185, SGI203185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45108[] = {SGI203186, SGI203187, SGI203188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203189, SGI0, SGI0, SGI0, SGI203190, SGI203190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203191, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45109[] = {SGI203192, SGI203193, SGI203194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203195, SGI0, SGI0, SGI0, SGI203196, SGI203196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203197, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45110[] = {SGI203198, SGI0, SGI203199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203200, SGI203201, SGI203201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45111[] = {SGI203202, SGI0, SGI203203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203204, SGI203205, SGI203205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45112[] = {SGI203206, SGI0, SGI203207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203208, SGI203209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45113[] = {SGI203210, SGI0, SGI203211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203212, SGI203213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45114[] = {SGI203214, SGI0, SGI203215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203216, SGI203217, SGI203217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45115[] = {SGI203218, SGI0, SGI203219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203220, SGI203221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45116[] = {SGI203222, SGI203223, SGI203224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203225, SGI0, SGI0, SGI0, SGI203226, SGI203226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45117[] = {SGI203227, SGI203228, SGI203229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203230, SGI0, SGI0, SGI0, SGI203231, SGI203231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45118[] = {SGI203232, SGI0, SGI203233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203234, SGI203235, SGI203235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45119[] = {SGI203236, SGI0, SGI203237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45120[] = {SGI203238, SGI203239, SGI203240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203241, SGI0, SGI0, SGI0, SGI203242, SGI203242, SGI0, SGI0, SGI0, SGI203243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45121[] = {SGI203244, SGI203245, SGI203246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203247, SGI0, SGI0, SGI0, SGI203248, SGI203248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45122[] = {SGI203249, SGI203250, SGI203251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203252, SGI0, SGI0, SGI0, SGI203253, SGI203253, SGI0, SGI0, SGI0, SGI203254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45123[] = {SGI203255, SGI203256, SGI203257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203258, SGI0, SGI0, SGI0, SGI203259, SGI203259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45124[] = {SGI203261, SGI0, SGI203262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203263, SGI203264, SGI203264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45125[] = {SGI203265, SGI203266, SGI203267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203270, SGI0, SGI0, SGI0, SGI203271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45126[] = {SGI203268, SGI0, SGI203269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203272, SGI203273, SGI203273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45127[] = {SGI203274, SGI0, SGI203275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203276, SGI203277, SGI203277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45128[] = {SGI203278, SGI203279, SGI203281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203283, SGI0, SGI0, SGI0, SGI203284, SGI203284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45129[] = {SGI203280, SGI0, SGI203282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203285, SGI203286, SGI203286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45130[] = {SGI203287, SGI0, SGI203288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203289, SGI203290, SGI203290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203291, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45131[] = {SGI203292, SGI0, SGI203293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203294, SGI203295, SGI203295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45132[] = {SGI203296, SGI0, SGI203297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203298, SGI203299, SGI203299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45133[] = {SGI203300, SGI203301, SGI203302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203303, SGI0, SGI0, SGI0, SGI203304, SGI203304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45134[] = {SGI203305, SGI0, SGI203306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203307, SGI203308, SGI203308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45135[] = {SGI203309, SGI203310, SGI203311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203314, SGI0, SGI0, SGI0, SGI203316, SGI0, SGI0, SGI203317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45136[] = {SGI203312, SGI203313, SGI203315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203318, SGI0, SGI0, SGI0, SGI203319, SGI203319, SGI0, SGI0, NULL};
const Segment * const SG45137[] = {SGI203321, SGI203323, SGI203325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203326, SGI0, SGI0, SGI0, SGI203328, SGI203328, SGI0, SGI0, NULL};
const Segment * const SG45138[] = {SGI203320, SGI203322, SGI203324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203327, SGI0, SGI0, SGI0, SGI203329, SGI0, SGI0, SGI203330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45139[] = {SGI203331, SGI0, SGI203332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203333, SGI203334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45140[] = {SGI203335, SGI0, SGI203336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203342, SGI203344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45141[] = {SGI203337, SGI203339, SGI203341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203345, SGI0, SGI0, SGI0, SGI203347, SGI203347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45142[] = {SGI203338, SGI203340, SGI203343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203346, SGI0, SGI0, SGI0, SGI203348, SGI203348, SGI0, SGI203350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45143[] = {SGI203351, SGI203352, SGI203353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203354, SGI0, SGI0, SGI0, SGI203355, SGI203355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45144[] = {SGI203356, SGI0, SGI203357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203358, SGI203359, SGI203359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45145[] = {SGI203360, SGI203361, SGI203362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203363, SGI0, SGI0, SGI0, SGI203364, SGI203364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45146[] = {SGI203365, SGI0, SGI203366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203367, SGI203368, SGI203368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203369, SGI0, SGI0, NULL};
const Segment * const SG45147[] = {SGI203370, SGI0, SGI203371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203372, SGI203373, SGI203373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203374, SGI0, SGI0, NULL};
const Segment * const SG45148[] = {SGI203375, SGI0, SGI203376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203377, SGI203378, SGI203378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203381, SGI0, SGI0, NULL};
const Segment * const SG45149[] = {SGI203379, SGI0, SGI203380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203382, SGI203383, SGI203383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45150[] = {SGI203384, SGI0, SGI203385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203386, SGI0, SGI0, SGI0, SGI203387, SGI203387, SGI0, SGI0, SGI0, SGI203388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45151[] = {SGI203389, SGI203390, SGI203391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203392, SGI0, SGI0, SGI0, SGI203393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45152[] = {SGI203394, SGI203395, SGI203396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203397, SGI0, SGI0, SGI0, SGI203398, SGI203398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45153[] = {SGI203399, SGI203400, SGI203401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203402, SGI0, SGI0, SGI0, SGI203403, SGI203403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45154[] = {SGI203404, SGI203405, SGI203406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203407, SGI0, SGI0, SGI0, SGI203408, SGI203408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45155[] = {SGI203409, SGI203410, SGI203411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203412, SGI0, SGI0, SGI0, SGI203413, SGI203413, SGI0, SGI203414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45156[] = {SGI203415, SGI203416, SGI203417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203418, SGI0, SGI0, SGI0, SGI203419, SGI0, SGI0, SGI203420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45157[] = {SGI203421, SGI203422, SGI203423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203424, SGI0, SGI0, SGI0, SGI203425, SGI0, SGI0, SGI203426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45158[] = {SGI203427, SGI0, SGI203428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203429, SGI203430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45159[] = {SGI203431, SGI0, SGI203432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203433, SGI203434, SGI203434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203435, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45160[] = {SGI203436, SGI0, SGI203437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203438, SGI203439, SGI203439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45161[] = {SGI203440, SGI0, SGI203441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203442, SGI203443, SGI203443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45162[] = {SGI203444, SGI203445, SGI203446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203447, SGI0, SGI0, SGI0, SGI203448, SGI203448, SGI0, SGI203449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45163[] = {SGI203450, SGI0, SGI203451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203452, SGI203453, SGI203453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45164[] = {SGI203454, SGI0, SGI203456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203458, SGI203460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45165[] = {SGI203455, SGI0, SGI203457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203459, SGI203461, SGI203461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45166[] = {SGI203462, SGI0, SGI203463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203464, SGI203465, SGI203465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45167[] = {SGI203466, SGI203467, SGI203468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203469, SGI0, SGI0, SGI0, SGI203470, SGI203471, SGI0, SGI0, SGI0, SGI203472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45168[] = {SGI203473, SGI0, SGI203474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203478, SGI203479, SGI203479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45169[] = {SGI203475, SGI203476, SGI203477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203480, SGI0, SGI0, SGI0, SGI203481, SGI203481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45170[] = {SGI203483, SGI203484, SGI203485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203487, SGI0, SGI0, SGI0, SGI203489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45171[] = {SGI203486, SGI0, SGI203488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203491, SGI203492, SGI203492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45172[] = {SGI203493, SGI203494, SGI203495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203498, SGI0, SGI0, SGI0, SGI203499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45173[] = {SGI203496, SGI0, SGI203497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203500, SGI203502, SGI203502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45174[] = {SGI203503, SGI203504, SGI203505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203510, SGI0, SGI0, SGI0, SGI203512, SGI203512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45175[] = {SGI203506, SGI203507, SGI203508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203513, SGI0, SGI0, SGI0, SGI203514, SGI203514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45176[] = {SGI203509, SGI0, SGI203511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203515, SGI203516, SGI203516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45177[] = {SGI203518, SGI0, SGI203519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45178[] = {SGI203520, SGI0, SGI203522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203525, SGI203526, SGI203526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45179[] = {SGI203521, SGI203523, SGI203524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203527, SGI0, SGI0, SGI0, SGI203528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45180[] = {SGI203530, SGI0, SGI203531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203535, SGI203536, SGI203536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45181[] = {SGI203532, SGI203533, SGI203534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203537, SGI0, SGI0, SGI0, SGI203538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45182[] = {SGI203539, SGI203540, SGI203541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203542, SGI0, SGI0, SGI0, SGI203543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45183[] = {SGI203544, SGI203545, SGI203546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203547, SGI0, SGI0, SGI0, SGI203548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45184[] = {SGI203549, SGI0, SGI203550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203551, SGI203552, SGI203552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45185[] = {SGI203553, SGI0, SGI203554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203555, SGI203556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45186[] = {SGI203557, SGI203558, SGI203559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203560, SGI0, SGI0, SGI0, SGI203561, SGI203561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203562, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45187[] = {SGI203563, SGI203564, SGI203565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203566, SGI0, SGI0, SGI0, SGI203567, SGI203567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203568, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45188[] = {SGI203569, SGI0, SGI203570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203572, SGI0, SGI0, SGI0, SGI203574, SGI203574, SGI0, SGI0, SGI0, SGI203575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45189[] = {SGI203571, SGI0, SGI203573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203576, SGI203577, SGI203577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203578, SGI0, SGI0, NULL};
const Segment * const SG45190[] = {SGI203579, SGI0, SGI203580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203581, SGI0, SGI0, SGI0, SGI203582, SGI203582, SGI0, SGI0, SGI0, SGI203583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45191[] = {SGI203584, SGI0, SGI203585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203586, SGI203587, SGI203587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45192[] = {SGI203588, SGI0, SGI203589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203590, SGI203591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45193[] = {SGI203592, SGI0, SGI203593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203597, SGI203598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45194[] = {SGI203594, SGI203595, SGI203596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203599, SGI0, SGI0, SGI0, SGI203600, SGI203600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45195[] = {SGI203601, SGI203602, SGI203603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203604, SGI0, SGI0, SGI0, SGI0, SGI203605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45196[] = {SGI203606, SGI203607, SGI203608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203609, SGI0, SGI0, SGI0, SGI203610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45197[] = {SGI203611, SGI0, SGI203612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45198[] = {SGI203613, SGI0, SGI203614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203617, SGI0, SGI0, SGI0, SGI203618, SGI203618, SGI0, SGI0, SGI0, SGI203620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45199[] = {SGI203615, SGI0, SGI203616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203619, SGI203621, SGI203621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203622, SGI0, SGI0, NULL};
const Segment * const SG45200[] = {SGI203623, SGI0, SGI203624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203625, SGI203626, SGI203626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45201[] = {SGI203627, SGI0, SGI203628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203629, SGI203631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45202[] = {SGI203630, SGI0, SGI203632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203634, SGI203635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45203[] = {SGI203633, SGI0, SGI203636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203637, SGI203638, SGI203638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45204[] = {SGI203639, SGI0, SGI203640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203641, SGI203642, SGI203642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45205[] = {SGI203643, SGI0, SGI203644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203645, SGI203646, SGI203646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203647, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45206[] = {SGI203648, SGI0, SGI203649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203650, SGI203651, SGI203651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45207[] = {SGI203652, SGI203653, SGI203654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203655, SGI0, SGI0, SGI0, SGI203656, SGI203656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45208[] = {SGI203657, SGI203658, SGI203659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203660, SGI0, SGI0, SGI0, SGI203661, SGI203661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45209[] = {SGI203662, SGI0, SGI203663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45210[] = {SGI203664, SGI0, SGI203665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203666, SGI203667, SGI203667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45211[] = {SGI203668, SGI203669, SGI203670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203671, SGI203671, SGI0, SGI0, SGI0, SGI203672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45212[] = {SGI203673, SGI203674, SGI203675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203676, SGI203676, SGI0, SGI0, SGI0, SGI203678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45213[] = {SGI203677, SGI0, SGI203679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203680, SGI203681, SGI203681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45214[] = {SGI203682, SGI0, SGI203683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203684, SGI203685, SGI203685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45215[] = {SGI203686, SGI0, SGI203687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203688, SGI203689, SGI203689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45216[] = {SGI203690, SGI203691, SGI203692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203693, SGI0, SGI0, SGI0, SGI203694, SGI203694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45217[] = {SGI203695, SGI203696, SGI203697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203698, SGI0, SGI0, SGI0, SGI203699, SGI203699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45218[] = {SGI203700, SGI0, SGI203701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45219[] = {SGI203702, SGI203703, SGI203704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203705, SGI0, SGI0, SGI0, SGI203706, SGI203706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45220[] = {SGI203707, SGI203708, SGI203709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203712, SGI0, SGI0, SGI0, SGI203714, SGI203714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45221[] = {SGI203710, SGI203711, SGI203713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203715, SGI0, SGI0, SGI0, SGI203716, SGI203716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45222[] = {SGI203717, SGI203718, SGI203719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203720, SGI0, SGI0, SGI0, SGI203721, SGI203721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45223[] = {SGI203722, SGI0, SGI203723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45224[] = {SGI203724, SGI203725, SGI203727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203730, SGI0, SGI0, SGI0, SGI203732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45225[] = {SGI203726, SGI203728, SGI203729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203731, SGI0, SGI0, SGI0, SGI203733, SGI203733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45226[] = {SGI203734, SGI203735, SGI203736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203737, SGI203737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45227[] = {SGI203738, SGI0, SGI203739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203740, SGI203741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45228[] = {SGI203742, SGI203743, SGI203744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203748, SGI0, SGI0, SGI0, SGI203749, SGI203749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45229[] = {SGI203745, SGI203746, SGI203747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203751, SGI203751, SGI0, SGI0, SGI203753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45230[] = {SGI203750, SGI0, SGI203752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203754, SGI203755, SGI203755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45231[] = {SGI203756, SGI0, SGI203757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45232[] = {SGI203758, SGI203759, SGI203760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203761, SGI0, SGI0, SGI0, SGI203762, SGI203762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45233[] = {SGI203763, SGI0, SGI203764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203766, SGI203767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45234[] = {SGI203765, SGI0, SGI203768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203769, SGI203770, SGI203770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45235[] = {SGI203771, SGI0, SGI203772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203773, SGI203774, SGI203774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45236[] = {SGI0, SGI203775, SGI0, SGI203776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45237[] = {SGI203777, SGI0, SGI203778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203779, SGI203781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45238[] = {SGI203780, SGI0, SGI203782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203783, SGI203784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45239[] = {SGI203785, SGI203786, SGI203787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203788, SGI0, SGI0, SGI0, SGI203789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45240[] = {SGI203790, SGI0, SGI203791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203792, SGI203793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45241[] = {SGI203794, SGI203795, SGI203796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203797, SGI0, SGI0, SGI0, SGI203798, SGI203798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45242[] = {SGI203799, SGI203800, SGI203801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203802, SGI0, SGI0, SGI0, SGI203803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45243[] = {SGI203804, SGI0, SGI203805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203806, SGI203807, SGI203807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45244[] = {SGI203808, SGI0, SGI203809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203810, SGI203811, SGI203811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45245[] = {SGI203812, SGI0, SGI203813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203814, SGI203815, SGI203815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45246[] = {SGI203816, SGI0, SGI203817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203818, SGI203819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45247[] = {SGI203820, SGI203821, SGI203822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203823, SGI0, SGI0, SGI0, SGI203824, SGI203824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45248[] = {SGI203825, SGI0, SGI203826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203827, SGI203828, SGI203828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45249[] = {SGI203829, SGI203830, SGI203831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203832, SGI0, SGI0, SGI0, SGI203833, SGI203833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45250[] = {SGI203834, SGI203835, SGI203836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203837, SGI0, SGI0, SGI0, SGI203838, SGI203838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45251[] = {SGI203839, SGI0, SGI203840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203841, SGI203842, SGI203842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45252[] = {SGI203843, SGI203844, SGI203845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203846, SGI203846, SGI0, SGI0, SGI0, SGI203847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45253[] = {SGI203848, SGI203849, SGI203850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203853, SGI203853, SGI0, SGI0, SGI0, SGI203854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45254[] = {SGI203851, SGI0, SGI203852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203855, SGI203856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45255[] = {SGI203857, SGI0, SGI203858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203859, SGI203860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45256[] = {SGI203861, SGI0, SGI203862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203863, SGI203864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45257[] = {SGI203865, SGI0, SGI203866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203867, SGI203868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45258[] = {SGI203869, SGI0, SGI203870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203873, SGI203874, SGI203874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45259[] = {SGI203871, SGI0, SGI203872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203875, SGI203876, SGI203876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45260[] = {SGI203877, SGI0, SGI203878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203879, SGI203880, SGI203880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45261[] = {SGI203881, SGI0, SGI203882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203883, SGI203884, SGI203884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45262[] = {SGI203885, SGI0, SGI203886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203887, SGI203888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45263[] = {SGI203889, SGI0, SGI203890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203891, SGI203892, SGI203892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45264[] = {SGI203893, SGI0, SGI203894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203895, SGI203896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45265[] = {SGI203897, SGI0, SGI203898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203899, SGI203900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45266[] = {SGI203901, SGI203902, SGI203903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203904, SGI203904, SGI0, SGI0, SGI203905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45267[] = {SGI203906, SGI203907, SGI203908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203909, SGI0, SGI0, SGI0, SGI203910, SGI203910, SGI0, SGI0, NULL};
const Segment * const SG45268[] = {SGI203911, SGI203912, SGI203913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203914, SGI0, SGI0, SGI0, SGI203915, SGI203915, SGI0, SGI203916, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45269[] = {SGI203917, SGI0, SGI203918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203919, SGI203920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45270[] = {SGI203921, SGI0, SGI203922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203923, SGI203924, SGI203924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45271[] = {SGI203925, SGI0, SGI203926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203927, SGI203929, SGI203929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45272[] = {SGI203928, SGI0, SGI203930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203931, SGI203932, SGI203932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45273[] = {SGI203933, SGI0, SGI203934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203935, SGI203936, SGI203936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45274[] = {SGI203937, SGI0, SGI203938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203939, SGI203940, SGI203940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45275[] = {SGI203941, SGI0, SGI203942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203944, SGI203946, SGI203946, SGI0, SGI0, NULL};
const Segment * const SG45276[] = {SGI203943, SGI203945, SGI203947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203948, SGI0, SGI0, SGI0, SGI203949, SGI203949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45277[] = {SGI203950, SGI0, SGI203951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203952, SGI203953, SGI203953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45278[] = {SGI203954, SGI0, SGI203955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203956, SGI203957, SGI203957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45279[] = {SGI203958, SGI0, SGI203959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203960, SGI203961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45280[] = {SGI203962, SGI0, SGI203963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203964, SGI203965, SGI203965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45281[] = {SGI203966, SGI203967, SGI203968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203972, SGI203972, SGI0, SGI0, SGI0, SGI0, SGI203973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45282[] = {SGI203969, SGI203970, SGI203971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203974, SGI0, SGI0, SGI0, SGI203975, SGI203975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45283[] = {SGI203978, SGI0, SGI203980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45284[] = {SGI203976, SGI203977, SGI203979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203981, SGI203981, SGI0, SGI0, SGI0, SGI0, SGI203982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45285[] = {SGI203983, SGI0, SGI203984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203985, SGI203986, SGI203986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45286[] = {SGI203987, SGI203988, SGI203989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203990, SGI0, SGI0, SGI0, SGI203991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45287[] = {SGI203992, SGI203993, SGI203994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI203995, SGI0, SGI0, SGI0, SGI203996, SGI203996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45288[] = {SGI203997, SGI203998, SGI203999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204000, SGI204000, SGI0, SGI0, SGI204001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45289[] = {SGI204002, SGI0, SGI204003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204004, SGI204005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204006, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45290[] = {SGI204007, SGI204008, SGI204009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204010, SGI204010, SGI0, SGI0, SGI204011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45291[] = {SGI204012, SGI0, SGI204013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204014, SGI204015, SGI204015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45292[] = {SGI204016, SGI0, SGI204017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204018, SGI204019, SGI204019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204022, SGI0, SGI0, NULL};
const Segment * const SG45293[] = {SGI204020, SGI204021, SGI204023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204024, SGI0, SGI0, SGI0, SGI204025, SGI204025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204026, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45294[] = {SGI204027, SGI0, SGI204028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204029, SGI204030, SGI204030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45295[] = {SGI204031, SGI0, SGI204032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204033, SGI204034, SGI204034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45296[] = {SGI204035, SGI0, SGI204036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204039, SGI204041, SGI204041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45297[] = {SGI204037, SGI204038, SGI204040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204042, SGI0, SGI0, SGI0, SGI204043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45298[] = {SGI204044, SGI0, SGI204045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204046, SGI204047, SGI204047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45299[] = {SGI204048, SGI0, SGI204050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204052, SGI204053, SGI204053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45300[] = {SGI204049, SGI0, SGI204051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204054, SGI204055, SGI204055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45301[] = {SGI204056, SGI204057, SGI204058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204059, SGI0, SGI0, SGI0, SGI204060, SGI204060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204061, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45302[] = {SGI204062, SGI204063, SGI204064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204065, SGI0, SGI0, SGI0, SGI204066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204067, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45303[] = {SGI204068, SGI204069, SGI204070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204071, SGI0, SGI0, SGI0, SGI204072, SGI204072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45304[] = {SGI204074, SGI0, SGI204075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204076, SGI0, SGI0, SGI0, SGI204077, SGI204077, SGI0, SGI0, SGI204078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45305[] = {SGI204079, SGI0, SGI204080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204081, SGI204082, SGI204082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45306[] = {SGI204083, SGI0, SGI204084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204085, SGI204086, SGI204086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45307[] = {SGI204087, SGI204088, SGI204089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204090, SGI0, SGI0, SGI0, SGI204091, SGI204091, SGI0, SGI204094, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45308[] = {SGI204092, SGI0, SGI204093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204095, SGI204096, SGI204096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45309[] = {SGI204097, SGI0, SGI204098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204099, SGI204100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45310[] = {SGI204101, SGI204102, SGI204103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204104, SGI0, SGI0, SGI0, SGI204105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45311[] = {SGI204106, SGI204107, SGI204108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204109, SGI204109, SGI0, SGI0, SGI204110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45312[] = {SGI204111, SGI0, SGI204112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204113, SGI204114, SGI204114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45313[] = {SGI204115, SGI0, SGI204116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204117, SGI204118, SGI204118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45314[] = {SGI204119, SGI0, SGI204120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204121, SGI204122, SGI204122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45315[] = {SGI204123, SGI204124, SGI204125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204126, SGI0, SGI0, SGI0, SGI204127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45316[] = {SGI204128, SGI204129, SGI204130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204131, SGI0, SGI0, SGI0, SGI204132, SGI204132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45317[] = {SGI204133, SGI0, SGI204134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45318[] = {SGI204135, SGI0, SGI204136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204137, SGI204138, SGI204138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45319[] = {SGI204139, SGI0, SGI204140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204141, SGI204142, SGI204142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45320[] = {SGI204143, SGI0, SGI204144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204145, SGI204146, SGI204146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45321[] = {SGI204147, SGI204148, SGI204149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204152, SGI0, SGI0, SGI0, SGI204153, SGI204153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45322[] = {SGI204150, SGI0, SGI204151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204154, SGI204155, SGI204155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45323[] = {SGI204156, SGI0, SGI204157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204158, SGI204159, SGI204159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45324[] = {SGI204160, SGI204162, SGI204163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204166, SGI204166, SGI0, SGI0, SGI204168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45325[] = {SGI204161, SGI0, SGI204164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204165, SGI204167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45326[] = {SGI204169, SGI0, SGI204170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204171, SGI204172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45327[] = {SGI204173, SGI204174, SGI204175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204179, SGI0, SGI0, SGI0, SGI204180, SGI204180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45328[] = {SGI204176, SGI204177, SGI204178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204181, SGI0, SGI0, SGI0, SGI204183, SGI204183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45329[] = {SGI204184, SGI204185, SGI204186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204187, SGI0, SGI0, SGI0, SGI204188, SGI204188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204189, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45330[] = {SGI204190, SGI0, SGI204191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204192, SGI204193, SGI204193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45331[] = {SGI204194, SGI0, SGI204195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204196, SGI204197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45332[] = {SGI204198, SGI0, SGI204199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204200, SGI204201, SGI204201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45333[] = {SGI204202, SGI204203, SGI204204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204205, SGI0, SGI0, SGI0, SGI204206, SGI204206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45334[] = {SGI204207, SGI0, SGI204208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204212, SGI204213, SGI204213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45335[] = {SGI204209, SGI204210, SGI204211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204214, SGI0, SGI0, SGI0, SGI204215, SGI204215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45336[] = {SGI204216, SGI0, SGI204217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204218, SGI204219, SGI204219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45337[] = {SGI204220, SGI0, SGI204221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204222, SGI204223, SGI204223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45338[] = {SGI204224, SGI0, SGI204225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204226, SGI204227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45339[] = {SGI204228, SGI204229, SGI204230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204231, SGI0, SGI0, SGI0, SGI204232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45340[] = {SGI204233, SGI0, SGI204234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204235, SGI204236, SGI204236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45341[] = {SGI204237, SGI0, SGI204238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204239, SGI204240, SGI204240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45342[] = {SGI204241, SGI0, SGI204242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204243, SGI204244, SGI204244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45343[] = {SGI204245, SGI0, SGI204246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204247, SGI204248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45344[] = {SGI0, SGI0, SGI204249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204253, SGI204254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45345[] = {SGI204250, SGI204251, SGI204252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204256, SGI0, SGI0, SGI0, SGI204259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45346[] = {SGI204255, SGI204257, SGI204258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204260, SGI0, SGI0, SGI0, SGI204261, SGI0, SGI0, SGI204262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45347[] = {SGI204263, SGI204264, SGI204265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204266, SGI0, SGI0, SGI0, SGI204267, SGI204267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45348[] = {SGI204268, SGI204269, SGI204270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204271, SGI0, SGI0, SGI0, SGI204272, SGI204272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45349[] = {SGI204273, SGI0, SGI204274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204275, SGI204276, SGI204276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45350[] = {SGI204277, SGI204278, SGI204279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204280, SGI0, SGI0, SGI0, SGI204281, SGI204281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45351[] = {SGI204282, SGI204283, SGI204285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204291, SGI0, SGI0, SGI0, SGI204293, SGI204293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45352[] = {SGI204284, SGI204287, SGI204288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204292, SGI0, SGI0, SGI0, SGI204295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45353[] = {SGI204286, SGI204289, SGI204290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204294, SGI0, SGI0, SGI0, SGI204296, SGI204296, SGI0, SGI204297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45354[] = {SGI204298, SGI0, SGI204299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45355[] = {SGI204300, SGI204301, SGI204302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204303, SGI0, SGI0, SGI0, SGI204304, SGI204304, SGI0, SGI204305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45356[] = {SGI204306, SGI204307, SGI204308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204309, SGI0, SGI0, SGI0, SGI204310, SGI204310, SGI0, SGI204311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45357[] = {SGI204312, SGI204313, SGI204314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204316, SGI0, SGI0, SGI0, SGI204318, SGI204318, SGI0, SGI204319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45358[] = {SGI204315, SGI0, SGI204317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45359[] = {SGI204320, SGI204321, SGI204322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204323, SGI0, SGI0, SGI0, SGI204324, SGI204324, SGI0, SGI204325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45360[] = {SGI204326, SGI204327, SGI204328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204329, SGI0, SGI0, SGI0, SGI204330, SGI204330, SGI0, SGI204331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45361[] = {SGI204332, SGI204333, SGI204334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204335, SGI0, SGI0, SGI0, SGI204336, SGI204336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45362[] = {SGI204337, SGI0, SGI204338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204339, SGI204340, SGI204340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204341, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45363[] = {SGI204342, SGI0, SGI204343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204344, SGI204345, SGI204345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45364[] = {SGI204346, SGI0, SGI204347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204348, SGI204349, SGI204349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45365[] = {SGI204350, SGI0, SGI204351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204352, SGI204353, SGI204353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45366[] = {SGI204354, SGI0, SGI204355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204357, SGI204359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45367[] = {SGI204356, SGI204358, SGI204360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204361, SGI0, SGI0, SGI0, SGI204363, SGI204363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45368[] = {SGI204362, SGI0, SGI204364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204365, SGI204366, SGI204366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45369[] = {SGI204367, SGI0, SGI204368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45370[] = {SGI204369, SGI204370, SGI204371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204372, SGI0, SGI0, SGI204373, SGI0, SGI204374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45371[] = {SGI204375, SGI204376, SGI204377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204378, SGI0, SGI0, SGI204379, SGI204380, SGI204380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45372[] = {SGI204381, SGI0, SGI204382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204383, SGI204384, SGI204384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45373[] = {SGI204385, SGI0, SGI204386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204387, SGI204388, SGI204388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45374[] = {SGI204389, SGI0, SGI204390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204391, SGI204392, SGI204392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45375[] = {SGI204393, SGI0, SGI204394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204395, SGI204396, SGI204396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45376[] = {SGI204397, SGI204398, SGI204399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204402, SGI204402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204405, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45377[] = {SGI204400, SGI0, SGI204401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204403, SGI204404, SGI204404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45378[] = {SGI204406, SGI204407, SGI204408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204409, SGI0, SGI0, SGI0, SGI204410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45379[] = {SGI204411, SGI0, SGI204412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204413, SGI204414, SGI204414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45380[] = {SGI204415, SGI0, SGI204416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204417, SGI204418, SGI204418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204419, SGI0, SGI0, NULL};
const Segment * const SG45381[] = {SGI204420, SGI0, SGI204421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204422, SGI204423, SGI204423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45382[] = {SGI204424, SGI0, SGI204425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204426, SGI204427, SGI204427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45383[] = {SGI204428, SGI204429, SGI204430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204431, SGI0, SGI0, SGI0, SGI204432, SGI204432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45384[] = {SGI204433, SGI204434, SGI204435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204436, SGI0, SGI0, SGI0, SGI204437, SGI204437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45385[] = {SGI204438, SGI0, SGI204439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204443, SGI204444, SGI204444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45386[] = {SGI204440, SGI204441, SGI204442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204445, SGI0, SGI0, SGI0, SGI204446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45387[] = {SGI204447, SGI204448, SGI204449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204450, SGI0, SGI0, SGI0, SGI204451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45388[] = {SGI204452, SGI0, SGI204453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45389[] = {SGI204454, SGI204455, SGI204456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204457, SGI0, SGI0, SGI0, SGI204458, SGI0, SGI0, SGI0, SGI204459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45390[] = {SGI204460, SGI0, SGI204461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204462, SGI204463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45391[] = {SGI204464, SGI204465, SGI204466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204469, SGI0, SGI0, SGI0, SGI204470, SGI204470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45392[] = {SGI204467, SGI0, SGI204468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204471, SGI204472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45393[] = {SGI204473, SGI204474, SGI204475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204476, SGI0, SGI0, SGI0, SGI204477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45394[] = {SGI204478, SGI204479, SGI204480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204483, SGI204483, SGI0, SGI0, SGI0, SGI204484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45395[] = {SGI204481, SGI0, SGI204482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204485, SGI204486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45396[] = {SGI204487, SGI204488, SGI204489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204490, SGI0, SGI0, SGI0, SGI204491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45397[] = {SGI204492, SGI204493, SGI204494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204495, SGI204495, SGI0, SGI0, SGI0, SGI204498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45398[] = {SGI204496, SGI0, SGI204497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45399[] = {SGI204499, SGI0, SGI204501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204506, SGI204507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45400[] = {SGI204500, SGI204502, SGI204503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204508, SGI204508, SGI0, SGI0, SGI0, SGI204510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45401[] = {SGI204504, SGI0, SGI204505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204509, SGI204511, SGI204511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45402[] = {SGI204512, SGI204513, SGI204514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204517, SGI0, SGI0, SGI0, SGI204519, SGI204519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45403[] = {SGI204515, SGI204516, SGI204518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204520, SGI0, SGI0, SGI0, SGI204522, SGI204522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45404[] = {SGI204521, SGI204523, SGI204524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204525, SGI0, SGI0, SGI0, SGI204527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45405[] = {SGI204526, SGI0, SGI204528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204529, SGI204530, SGI204530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45406[] = {SGI204531, SGI0, SGI204532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45407[] = {SGI204533, SGI204534, SGI204535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204538, SGI0, SGI0, SGI0, SGI204539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45408[] = {SGI204536, SGI0, SGI204537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204540, SGI204541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45409[] = {SGI204542, SGI204543, SGI204544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204546, SGI0, SGI0, SGI0, SGI204548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45410[] = {SGI204545, SGI0, SGI204547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204549, SGI204550, SGI204550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204551, SGI0, SGI0, NULL};
const Segment * const SG45411[] = {SGI204552, SGI204553, SGI204554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204555, SGI204555, SGI0, SGI0, SGI0, SGI204556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45412[] = {SGI204557, SGI204558, SGI204559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204560, SGI0, SGI0, SGI0, SGI204561, SGI204561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45413[] = {SGI204562, SGI0, SGI204563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45414[] = {SGI204564, SGI0, SGI204565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204566, SGI204567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45415[] = {SGI204568, SGI204569, SGI204570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204571, SGI0, SGI0, SGI0, SGI204572, SGI204572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45416[] = {SGI204573, SGI0, SGI204574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204575, SGI204576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45417[] = {SGI204577, SGI204578, SGI204579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204580, SGI0, SGI0, SGI0, SGI204581, SGI204581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45418[] = {SGI204582, SGI204583, SGI204584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204585, SGI0, SGI0, SGI0, SGI204586, SGI204586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45419[] = {SGI204587, SGI0, SGI204588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204589, SGI204590, SGI204590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45420[] = {SGI204591, SGI0, SGI204592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204593, SGI204594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45421[] = {SGI204595, SGI0, SGI204596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204599, SGI204600, SGI204600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45422[] = {SGI204597, SGI0, SGI204598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204601, SGI204602, SGI204602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45423[] = {SGI204603, SGI0, SGI204604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204605, SGI204606, SGI204606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45424[] = {SGI204607, SGI0, SGI204608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204609, SGI204610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45425[] = {SGI204611, SGI0, SGI204612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204613, SGI204614, SGI204614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45426[] = {SGI204615, SGI204616, SGI204617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204620, SGI0, SGI0, SGI0, SGI204621, SGI204621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45427[] = {SGI204618, SGI0, SGI204619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204622, SGI204623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45428[] = {SGI204624, SGI0, SGI204625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204626, SGI204627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45429[] = {SGI204628, SGI204629, SGI204630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204631, SGI0, SGI0, SGI0, SGI204632, SGI204632, SGI0, SGI204633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45430[] = {SGI204634, SGI204635, SGI204636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204637, SGI0, SGI0, SGI0, SGI204638, SGI0, SGI0, SGI0, SGI204640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45431[] = {SGI204639, SGI204641, SGI204642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204643, SGI0, SGI0, SGI0, SGI204644, SGI204644, SGI0, SGI204645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45432[] = {SGI204646, SGI0, SGI204647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204648, SGI204649, SGI204649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45433[] = {SGI204650, SGI0, SGI204651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204652, SGI204653, SGI204653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45434[] = {SGI204654, SGI0, SGI204655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204658, SGI204659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45435[] = {SGI204656, SGI0, SGI204657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204660, SGI204661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45436[] = {SGI204662, SGI204663, SGI204664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204665, SGI0, SGI0, SGI0, SGI204666, SGI204666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45437[] = {SGI204667, SGI0, SGI204668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204669, SGI204670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45438[] = {SGI204671, SGI204672, SGI204673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204674, SGI204674, SGI0, SGI0, SGI0, SGI204675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45439[] = {SGI204676, SGI0, SGI204677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204678, SGI204679, SGI204679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45440[] = {SGI204680, SGI0, SGI204681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204685, SGI204686, SGI204686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204688, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45441[] = {SGI204682, SGI204683, SGI204684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204687, SGI0, SGI0, SGI0, SGI204689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45442[] = {SGI204690, SGI204691, SGI204692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204693, SGI0, SGI0, SGI0, SGI204694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45443[] = {SGI204695, SGI204696, SGI204697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204700, SGI0, SGI0, SGI0, SGI204701, SGI204701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45444[] = {SGI204698, SGI0, SGI204699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204702, SGI204703, SGI204703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45445[] = {SGI204704, SGI204705, SGI204706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204707, SGI0, SGI0, SGI0, SGI204708, SGI204708, SGI0, SGI204709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45446[] = {SGI204710, SGI204711, SGI204712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204717, SGI0, SGI0, SGI0, SGI204718, SGI0, SGI0, SGI204719, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45447[] = {SGI204714, SGI0, SGI204715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204720, SGI204722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45448[] = {SGI204713, SGI0, SGI204716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204721, SGI204723, SGI204723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45449[] = {SGI204724, SGI0, SGI204726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204730, SGI204731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45450[] = {SGI204725, SGI0, SGI204728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204732, SGI204733, SGI204733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45451[] = {SGI204727, SGI0, SGI204729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204734, SGI204735, SGI204735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45452[] = {SGI204736, SGI0, SGI204737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204738, SGI204739, SGI204739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45453[] = {SGI204740, SGI0, SGI204741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204742, SGI204743, SGI204743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45454[] = {SGI204744, SGI204745, SGI204746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204747, SGI0, SGI0, SGI0, SGI204748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45455[] = {SGI204749, SGI0, SGI204750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204751, SGI204752, SGI204752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45456[] = {SGI204753, SGI0, SGI204754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204755, SGI204756, SGI204756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45457[] = {SGI204757, SGI0, SGI204758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204759, SGI204760, SGI204760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45458[] = {SGI204761, SGI0, SGI204762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204763, SGI204764, SGI204764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45459[] = {SGI204765, SGI0, SGI204766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204767, SGI204768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45460[] = {SGI204769, SGI204770, SGI204771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204772, SGI0, SGI0, SGI0, SGI204773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45461[] = {SGI204774, SGI0, SGI204775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204776, SGI204777, SGI204777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45462[] = {SGI204778, SGI0, SGI204779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204780, SGI204781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45463[] = {SGI204782, SGI204783, SGI204784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204785, SGI0, SGI0, SGI0, SGI204786, SGI204786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45464[] = {SGI204787, SGI0, SGI204788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204789, SGI204790, SGI204790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45465[] = {SGI204791, SGI204792, SGI204793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204794, SGI0, SGI0, SGI0, SGI204795, SGI204795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45466[] = {SGI204796, SGI0, SGI204797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204798, SGI204799, SGI204799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45467[] = {SGI204800, SGI0, SGI204801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204802, SGI0, SGI0, SGI0, SGI204803, SGI204803, SGI0, SGI0, SGI0, SGI204804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45468[] = {SGI204805, SGI0, SGI204806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204807, SGI204808, SGI204808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45469[] = {SGI204809, SGI0, SGI204810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204811, SGI204812, SGI204812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45470[] = {SGI204813, SGI204814, SGI204815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204816, SGI0, SGI0, SGI0, SGI204817, SGI204817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45471[] = {SGI204818, SGI204819, SGI204820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204821, SGI0, SGI0, SGI0, SGI204822, SGI204822, SGI0, SGI0, SGI204823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45472[] = {SGI204824, SGI204825, SGI204826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204827, SGI0, SGI0, SGI0, SGI204828, SGI204828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204829, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45473[] = {SGI204830, SGI204831, SGI204832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204833, SGI0, SGI0, SGI0, SGI204834, SGI204834, SGI0, SGI204835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45474[] = {SGI204836, SGI0, SGI204837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204838, SGI204839, SGI204839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204840, SGI0, SGI0, NULL};
const Segment * const SG45475[] = {SGI204841, SGI204842, SGI204843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204844, SGI204844, SGI0, SGI0, SGI0, SGI0, SGI204845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45476[] = {SGI204846, SGI204847, SGI204848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204849, SGI0, SGI0, SGI0, SGI204850, SGI204850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45477[] = {SGI204851, SGI204852, SGI204853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204854, SGI0, SGI0, SGI0, SGI204855, SGI204855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45478[] = {SGI204858, SGI0, SGI204859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45479[] = {SGI204856, SGI0, SGI204857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204860, SGI204861, SGI204861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45480[] = {SGI204862, SGI0, SGI204863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204864, SGI204865, SGI204865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45481[] = {SGI204866, SGI204867, SGI204868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204869, SGI0, SGI0, SGI0, SGI204870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45482[] = {SGI204871, SGI204872, SGI204873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204874, SGI0, SGI0, SGI0, SGI204875, SGI204875, SGI0, SGI0, SGI0, SGI204876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45483[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204877, SGI0, SGI0, SGI204878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45484[] = {SGI204879, SGI204880, SGI204881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204882, SGI0, SGI0, SGI0, SGI204883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45485[] = {SGI204884, SGI204885, SGI204886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204887, SGI0, SGI0, SGI0, SGI204888, SGI204888, SGI0, SGI204889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45486[] = {SGI204890, SGI204891, SGI204892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204893, SGI0, SGI0, SGI0, SGI204894, SGI204894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45487[] = {SGI204895, SGI204896, SGI204897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204898, SGI0, SGI0, SGI0, SGI204899, SGI204899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45488[] = {SGI204900, SGI0, SGI204901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204902, SGI204903, SGI204903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45489[] = {SGI204904, SGI0, SGI204905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204906, SGI204907, SGI204907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45490[] = {SGI204908, SGI204909, SGI204910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204911, SGI0, SGI0, SGI0, SGI204912, SGI204912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45491[] = {SGI0, SGI0, SGI204913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204914, SGI204915, SGI204915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45492[] = {SGI204916, SGI0, SGI204917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204918, SGI204919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45493[] = {SGI204920, SGI0, SGI204921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204922, SGI204923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45494[] = {SGI204924, SGI0, SGI204925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204926, SGI204927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45495[] = {SGI204928, SGI0, SGI204929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204930, SGI204931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45496[] = {SGI204932, SGI204934, SGI204935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204937, SGI0, SGI0, SGI0, SGI204939, SGI204939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45497[] = {SGI204933, SGI0, SGI204936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204938, SGI204940, SGI204940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45498[] = {SGI204941, SGI0, SGI204942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45499[] = {SGI204943, SGI204944, SGI204945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204946, SGI0, SGI0, SGI0, SGI204947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45500[] = {SGI204948, SGI0, SGI204949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204950, SGI204951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45501[] = {SGI204952, SGI0, SGI204953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204955, SGI204956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45502[] = {SGI204954, SGI0, SGI204957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204958, SGI204959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45503[] = {SGI204960, SGI0, SGI204961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204962, SGI204963, SGI204963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45504[] = {SGI204964, SGI0, SGI204965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204966, SGI204967, SGI204967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45505[] = {SGI204968, SGI0, SGI204969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204970, SGI204971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45506[] = {SGI204972, SGI0, SGI204973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204974, SGI204975, SGI204975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45507[] = {SGI204976, SGI0, SGI204977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204978, SGI204979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45508[] = {SGI204980, SGI204981, SGI204982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204983, SGI0, SGI0, SGI0, SGI204984, SGI204984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45509[] = {SGI204985, SGI0, SGI204986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204987, SGI204988, SGI204988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45510[] = {SGI204989, SGI0, SGI204991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45511[] = {SGI204990, SGI204992, SGI204993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204994, SGI0, SGI0, SGI0, SGI204995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45512[] = {SGI204996, SGI0, SGI204997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI204998, SGI204999, SGI204999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45513[] = {SGI205000, SGI0, SGI205001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205002, SGI205003, SGI205003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45514[] = {SGI205004, SGI0, SGI205005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205006, SGI205007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45515[] = {SGI205008, SGI0, SGI205009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205010, SGI205011, SGI205011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45516[] = {SGI205013, SGI0, SGI205015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205017, SGI205019, SGI205019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45517[] = {SGI205012, SGI0, SGI205014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205016, SGI205018, SGI205018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45518[] = {SGI205020, SGI0, SGI205021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205022, SGI205023, SGI205023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45519[] = {SGI205024, SGI0, SGI205025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205026, SGI205027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45520[] = {SGI205028, SGI0, SGI205029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205030, SGI205031, SGI205031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45521[] = {SGI205032, SGI0, SGI205033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205034, SGI205035, SGI205035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45522[] = {SGI205036, SGI205037, SGI205038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205039, SGI0, SGI0, SGI0, SGI205040, SGI0, SGI0, SGI205041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45523[] = {SGI205042, SGI0, SGI205043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205044, SGI205045, SGI205045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45524[] = {SGI205046, SGI205047, SGI205048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205049, SGI0, SGI0, SGI0, SGI205050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45525[] = {SGI205051, SGI0, SGI205052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205053, SGI205054, SGI205054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45526[] = {SGI205055, SGI0, SGI205057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205059, SGI205061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45527[] = {SGI205056, SGI0, SGI205058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205060, SGI205062, SGI205062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45528[] = {SGI205063, SGI205064, SGI205065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205066, SGI0, SGI0, SGI0, SGI205067, SGI205067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45529[] = {SGI205068, SGI0, SGI205069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45530[] = {SGI205070, SGI205071, SGI205072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205073, SGI0, SGI0, SGI0, SGI205074, SGI205074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45531[] = {SGI205075, SGI0, SGI205076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205077, SGI205078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45532[] = {SGI205079, SGI0, SGI205080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205081, SGI205082, SGI205082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45533[] = {SGI205083, SGI0, SGI205084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205085, SGI205086, SGI205086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45534[] = {SGI205087, SGI205088, SGI205089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205090, SGI0, SGI0, SGI0, SGI205091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45535[] = {SGI205092, SGI0, SGI205093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205094, SGI205095, SGI205095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45536[] = {SGI205096, SGI205097, SGI205098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205099, SGI0, SGI0, SGI0, SGI205100, SGI205100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205102, SGI0, NULL};
const Segment * const SG45537[] = {SGI205103, SGI0, SGI205104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205105, SGI0, SGI0, SGI0, SGI205106, SGI205106, SGI0, SGI0, SGI0, SGI205107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45538[] = {SGI205108, SGI0, SGI205109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205110, SGI205111, SGI205111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45539[] = {SGI0, SGI205112, SGI0, SGI205113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45540[] = {SGI205114, SGI205115, SGI205116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205117, SGI0, SGI0, SGI0, SGI205118, SGI205118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45541[] = {SGI205119, SGI205120, SGI205121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205122, SGI0, SGI0, SGI0, SGI205123, SGI205123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45542[] = {SGI205124, SGI205125, SGI205126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205127, SGI0, SGI0, SGI0, SGI205128, SGI205128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45543[] = {SGI205129, SGI0, SGI205130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205131, SGI205132, SGI205132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45544[] = {SGI205133, SGI0, SGI205134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205135, SGI205136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45545[] = {SGI205137, SGI205138, SGI205139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205140, SGI0, SGI0, SGI0, SGI205141, SGI205141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45546[] = {SGI205142, SGI0, SGI205143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205144, SGI0, SGI0, SGI0, SGI205145, SGI205145, SGI0, SGI0, SGI0, SGI205146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45547[] = {SGI205147, SGI0, SGI205148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205149, SGI205150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205151, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45548[] = {SGI205152, SGI0, SGI205154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205156, SGI205157, SGI205157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45549[] = {SGI205153, SGI0, SGI205155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205158, SGI205159, SGI205159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205160, SGI0, SGI0, NULL};
const Segment * const SG45550[] = {SGI205161, SGI0, SGI205162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205163, SGI205164, SGI205164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45551[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205168, SGI0, SGI0, SGI205169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45552[] = {SGI205165, SGI205166, SGI205167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205170, SGI205170, SGI0, SGI0, SGI0, SGI205171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45553[] = {SGI205172, SGI205173, SGI205174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205175, SGI0, SGI0, SGI205176, SGI205177, SGI205177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45554[] = {SGI205178, SGI0, SGI205179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205180, SGI205181, SGI205181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45555[] = {SGI205182, SGI0, SGI205183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205184, SGI205185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45556[] = {SGI205186, SGI0, SGI205187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205188, SGI205189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45557[] = {SGI205190, SGI0, SGI205191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205192, SGI205193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45558[] = {SGI205195, SGI0, SGI205198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205199, SGI205201, SGI205201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45559[] = {SGI205194, SGI205196, SGI205197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205200, SGI205200, SGI0, SGI0, SGI0, SGI205202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45560[] = {SGI205203, SGI0, SGI205204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205205, SGI205206, SGI205206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45561[] = {SGI205207, SGI0, SGI205208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205209, SGI205210, SGI205210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45562[] = {SGI205211, SGI0, SGI205212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205213, SGI205214, SGI205214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45563[] = {SGI205215, SGI0, SGI205216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205217, SGI205218, SGI205218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45564[] = {SGI205219, SGI0, SGI205220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205222, SGI205224, SGI205224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45565[] = {SGI205221, SGI0, SGI205223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205225, SGI205226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205227, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45566[] = {SGI205228, SGI0, SGI205229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205230, SGI205231, SGI205231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45567[] = {SGI205232, SGI0, SGI205233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205234, SGI205235, SGI205235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45568[] = {SGI205236, SGI0, SGI205237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205238, SGI205239, SGI205239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45569[] = {SGI205240, SGI205241, SGI205242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205243, SGI0, SGI0, SGI0, SGI205244, SGI205244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45570[] = {SGI205245, SGI205246, SGI205247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205248, SGI0, SGI0, SGI0, SGI205249, SGI205249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45571[] = {SGI205250, SGI0, SGI205251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205252, SGI205253, SGI205253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45572[] = {SGI205254, SGI0, SGI205255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205256, SGI205257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45573[] = {SGI205258, SGI0, SGI205259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205260, SGI0, SGI0, SGI0, SGI205261, SGI205261, SGI0, SGI0, SGI0, SGI205262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45574[] = {SGI205263, SGI0, SGI205264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205265, SGI205266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45575[] = {SGI205267, SGI205268, SGI205269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205270, SGI0, SGI0, SGI0, SGI205271, SGI205271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205272, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45576[] = {SGI205273, SGI205274, SGI205275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205276, SGI0, SGI0, SGI0, SGI205277, SGI205277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45577[] = {SGI205278, SGI205279, SGI205280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205281, SGI0, SGI0, SGI0, SGI205282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45578[] = {SGI205283, SGI0, SGI205284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205285, SGI205286, SGI205286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45579[] = {SGI205287, SGI205288, SGI205289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205293, SGI0, SGI0, SGI0, SGI205294, SGI205294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205296, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45580[] = {SGI205290, SGI205291, SGI205292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205295, SGI0, SGI0, SGI0, SGI205297, SGI0, SGI205298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45581[] = {SGI205299, SGI205300, SGI205301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205302, SGI0, SGI0, SGI0, SGI205303, SGI205303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205304, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45582[] = {SGI205305, SGI0, SGI205306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205307, SGI205308, SGI205308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45583[] = {SGI205309, SGI0, SGI205310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205311, SGI205312, SGI205312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45584[] = {SGI205313, SGI0, SGI205314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205315, SGI205316, SGI205316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45585[] = {SGI205317, SGI0, SGI205318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205319, SGI205320, SGI205320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45586[] = {SGI205321, SGI205322, SGI205323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205324, SGI0, SGI0, SGI0, SGI205325, SGI205325, SGI0, SGI205326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45587[] = {SGI205327, SGI0, SGI205328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205329, SGI205330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45588[] = {SGI205331, SGI0, SGI205332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205333, SGI205334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45589[] = {SGI205335, SGI0, SGI205336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205337, SGI205338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45590[] = {SGI205339, SGI205340, SGI205341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205342, SGI0, SGI0, SGI0, SGI205343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45591[] = {SGI205344, SGI205345, SGI205346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205347, SGI205347, SGI0, SGI0, SGI0, SGI0, SGI205348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45592[] = {SGI205349, SGI205350, SGI205351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205352, SGI205352, SGI0, SGI0, SGI0, SGI0, SGI205353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45593[] = {SGI205354, SGI0, SGI205355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205356, SGI205357, SGI205357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45594[] = {SGI205358, SGI0, SGI205359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205360, SGI205361, SGI205361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45595[] = {SGI205362, SGI0, SGI205363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205364, SGI205365, SGI205365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45596[] = {SGI205366, SGI0, SGI205367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205368, SGI205369, SGI205369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45597[] = {SGI205370, SGI0, SGI205371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205372, SGI205373, SGI205373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205374, SGI0, SGI0, NULL};
const Segment * const SG45598[] = {SGI205375, SGI205376, SGI205377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205378, SGI0, SGI0, SGI0, SGI205379, SGI205379, SGI0, SGI205380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45599[] = {SGI205381, SGI205382, SGI205383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205384, SGI0, SGI0, SGI0, SGI205385, SGI205385, SGI0, SGI205386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45600[] = {SGI205387, SGI205388, SGI205389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205390, SGI0, SGI0, SGI0, SGI205391, SGI205391, SGI0, SGI205392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45601[] = {SGI205394, SGI205396, SGI205398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205400, SGI0, SGI0, SGI0, SGI205402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45602[] = {SGI205393, SGI205395, SGI205397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205399, SGI0, SGI0, SGI0, SGI205401, SGI205401, SGI0, SGI205403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45603[] = {SGI205404, SGI205405, SGI205406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205407, SGI0, SGI0, SGI0, SGI205408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45604[] = {SGI205409, SGI0, SGI205410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45605[] = {SGI205411, SGI205412, SGI205413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205416, SGI0, SGI0, SGI0, SGI205417, SGI205417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205420, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45606[] = {SGI205414, SGI0, SGI205415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205418, SGI205419, SGI205419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45607[] = {SGI205421, SGI205422, SGI205423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205426, SGI0, SGI0, SGI0, SGI205427, SGI205427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205430, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45608[] = {SGI205424, SGI0, SGI205425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205428, SGI205429, SGI205429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45609[] = {SGI205434, SGI0, SGI205435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205437, SGI205438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45610[] = {SGI205431, SGI205432, SGI205433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205436, SGI205436, SGI0, SGI0, SGI205439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45611[] = {SGI205440, SGI205441, SGI205442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205443, SGI0, SGI0, SGI0, SGI205444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45612[] = {SGI205445, SGI205446, SGI205447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205448, SGI205448, SGI0, SGI0, SGI205450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45613[] = {SGI205449, SGI205451, SGI205453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205456, SGI0, SGI0, SGI0, SGI205458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45614[] = {SGI205452, SGI205454, SGI205455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205457, SGI0, SGI0, SGI0, SGI205459, SGI205459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45615[] = {SGI205460, SGI0, SGI205461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205462, SGI205463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45616[] = {SGI205464, SGI205465, SGI205466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205467, SGI0, SGI0, SGI0, SGI205468, SGI205468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45617[] = {SGI205469, SGI0, SGI205470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205471, SGI205472, SGI205472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45618[] = {SGI205473, SGI205474, SGI205475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205476, SGI0, SGI0, SGI0, SGI205477, SGI205477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45619[] = {SGI205478, SGI0, SGI205479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205480, SGI205481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45620[] = {SGI205482, SGI0, SGI205483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205484, SGI205485, SGI205485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205486, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45621[] = {SGI205488, SGI205490, SGI205491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205492, SGI0, SGI0, SGI0, SGI205495, SGI205495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45622[] = {SGI205487, SGI0, SGI205489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205493, SGI205494, SGI205494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45623[] = {SGI205496, SGI0, SGI205497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45624[] = {SGI205498, SGI205499, SGI205500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205501, SGI205501, SGI0, SGI0, SGI0, SGI0, SGI205502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45625[] = {SGI205503, SGI0, SGI205504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205507, SGI205508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45626[] = {SGI205505, SGI0, SGI205506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205509, SGI205510, SGI205510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45627[] = {SGI205511, SGI0, SGI205512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205513, SGI205514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45628[] = {SGI205515, SGI0, SGI205516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205517, SGI205518, SGI205518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205519, SGI0, SGI0, NULL};
const Segment * const SG45629[] = {SGI205520, SGI0, SGI205521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205522, SGI205523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45630[] = {SGI205524, SGI0, SGI205525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205529, SGI205530, SGI205530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45631[] = {SGI205526, SGI205527, SGI205528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205531, SGI0, SGI0, SGI0, SGI205532, SGI205532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45632[] = {SGI205533, SGI205534, SGI205535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205536, SGI0, SGI0, SGI0, SGI205537, SGI205537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45633[] = {SGI205538, SGI205539, SGI205540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205541, SGI0, SGI0, SGI0, SGI205542, SGI205542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45634[] = {SGI205543, SGI205544, SGI205545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205546, SGI0, SGI0, SGI0, SGI205547, SGI205547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45635[] = {SGI205548, SGI205549, SGI205550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205553, SGI0, SGI0, SGI0, SGI205555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45636[] = {SGI205551, SGI205552, SGI205554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205556, SGI0, SGI0, SGI0, SGI205557, SGI205557, SGI0, SGI205558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45637[] = {SGI205559, SGI0, SGI205560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205561, SGI0, SGI0, SGI0, SGI205562, SGI205562, SGI0, SGI0, SGI0, SGI205563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45638[] = {SGI205564, SGI0, SGI205565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205566, SGI205567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45639[] = {SGI205568, SGI0, SGI205569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205570, SGI205571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45640[] = {SGI205572, SGI0, SGI205573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205577, SGI205578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45641[] = {SGI205574, SGI205575, SGI205576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205579, SGI0, SGI0, SGI0, SGI205580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45642[] = {SGI205581, SGI0, SGI205582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205583, SGI205584, SGI205584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45643[] = {SGI205585, SGI205586, SGI205587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205588, SGI0, SGI0, SGI0, SGI205590, SGI205590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45644[] = {SGI205589, SGI205591, SGI205592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205594, SGI0, SGI0, SGI0, SGI205595, SGI205595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205597, SGI0, NULL};
const Segment * const SG45645[] = {SGI205598, SGI0, SGI205599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205600, SGI205601, SGI205601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45646[] = {SGI205602, SGI205603, SGI205604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205605, SGI0, SGI0, SGI0, SGI205606, SGI205606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205608, SGI0, NULL};
const Segment * const SG45647[] = {SGI205609, SGI0, SGI205610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205611, SGI205612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45648[] = {SGI205613, SGI0, SGI205614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205617, SGI205618, SGI205618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45649[] = {SGI205615, SGI0, SGI205616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205619, SGI205620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205621, SGI0, SGI0, NULL};
const Segment * const SG45650[] = {SGI205622, SGI0, SGI205623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205627, SGI205629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205631, SGI0, SGI0, NULL};
const Segment * const SG45651[] = {SGI205624, SGI205625, SGI205626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205628, SGI0, SGI0, SGI0, SGI205630, SGI205630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205632, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45652[] = {SGI205633, SGI205634, SGI205635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205636, SGI0, SGI0, SGI0, SGI205637, SGI205637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205638, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45653[] = {SGI205639, SGI0, SGI205640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205641, SGI205642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45654[] = {SGI205643, SGI0, SGI205644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205645, SGI205646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45655[] = {SGI205647, SGI0, SGI205648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205649, SGI205650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45656[] = {SGI205651, SGI0, SGI205652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205653, SGI205654, SGI205654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45657[] = {SGI205655, SGI0, SGI205656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205657, SGI205658, SGI205658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45658[] = {SGI205659, SGI0, SGI205660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205661, SGI205662, SGI205662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45659[] = {SGI205663, SGI0, SGI205664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205665, SGI205666, SGI205666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45660[] = {SGI205667, SGI0, SGI205668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205669, SGI205670, SGI205670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45661[] = {SGI205671, SGI0, SGI205672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205673, SGI205674, SGI205674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45662[] = {SGI205675, SGI0, SGI205676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205677, SGI205678, SGI205678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45663[] = {SGI205679, SGI205680, SGI205681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205685, SGI0, SGI0, SGI0, SGI205686, SGI205686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45664[] = {SGI205682, SGI205683, SGI205684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205687, SGI205687, SGI0, SGI0, SGI0, SGI205688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45665[] = {SGI205689, SGI0, SGI205690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205691, SGI205692, SGI205692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45666[] = {SGI205693, SGI0, SGI205694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205695, SGI205696, SGI205696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45667[] = {SGI205697, SGI205698, SGI205699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205700, SGI0, SGI0, SGI0, SGI205701, SGI205701, SGI0, SGI205702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45668[] = {SGI205703, SGI205704, SGI205706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205709, SGI0, SGI0, SGI0, SGI205710, SGI205710, SGI0, SGI205712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45669[] = {SGI205705, SGI205707, SGI205708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205711, SGI205711, SGI0, SGI0, SGI205713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45670[] = {SGI205714, SGI0, SGI205715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205719, SGI205720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45671[] = {SGI205716, SGI205717, SGI205718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205721, SGI0, SGI0, SGI0, SGI205722, SGI205722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45672[] = {SGI205723, SGI0, SGI205724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205725, SGI205726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45673[] = {SGI205727, SGI0, SGI205728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205729, SGI205730, SGI205730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45674[] = {SGI205731, SGI205732, SGI205733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205734, SGI0, SGI0, SGI0, SGI205735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45675[] = {SGI205736, SGI0, SGI205737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205738, SGI205739, SGI205739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45676[] = {SGI205740, SGI0, SGI205741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205742, SGI205743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45677[] = {SGI205744, SGI205745, SGI205746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205747, SGI0, SGI0, SGI0, SGI205748, SGI205748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205749, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45678[] = {SGI205750, SGI0, SGI205751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205752, SGI205753, SGI205753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45679[] = {SGI205754, SGI0, SGI205755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205756, SGI205757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45680[] = {SGI205758, SGI0, SGI205759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205760, SGI205761, SGI205761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45681[] = {SGI205762, SGI0, SGI205763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205764, SGI205765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45682[] = {SGI205766, SGI205767, SGI205768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205769, SGI205769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45683[] = {SGI205770, SGI0, SGI205771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205772, SGI205773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45684[] = {SGI205774, SGI205775, SGI205776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205777, SGI0, SGI0, SGI0, SGI205778, SGI205778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45685[] = {SGI205779, SGI0, SGI205780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205781, SGI205782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45686[] = {SGI205783, SGI205784, SGI205785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205786, SGI0, SGI0, SGI0, SGI205787, SGI205787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45687[] = {SGI205788, SGI0, SGI205789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205790, SGI205791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45688[] = {SGI205792, SGI0, SGI205793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205794, SGI205795, SGI205795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45689[] = {SGI205796, SGI0, SGI205797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205798, SGI205799, SGI205799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45690[] = {SGI205800, SGI0, SGI205802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205805, SGI205807, SGI205807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45691[] = {SGI205801, SGI205803, SGI205804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205806, SGI0, SGI0, SGI0, SGI205808, SGI205808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45692[] = {SGI205809, SGI0, SGI205810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205811, SGI205812, SGI205812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45693[] = {SGI205813, SGI0, SGI205814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205815, SGI205816, SGI205816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45694[] = {SGI205817, SGI0, SGI205818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205819, SGI205820, SGI205820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45695[] = {SGI205821, SGI205822, SGI205823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205824, SGI0, SGI0, SGI0, SGI205825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45696[] = {SGI205826, SGI0, SGI205827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205828, SGI205829, SGI205829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45697[] = {SGI205830, SGI0, SGI205831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205832, SGI205833, SGI205833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45698[] = {SGI205834, SGI0, SGI205835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205836, SGI205837, SGI205837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45699[] = {SGI205838, SGI0, SGI205839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205840, SGI0, SGI0, SGI0, SGI205841, SGI205841, SGI0, SGI0, SGI0, SGI205842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45700[] = {SGI205843, SGI205844, SGI205845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205846, SGI0, SGI0, SGI0, SGI205847, SGI205847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45701[] = {SGI205848, SGI205849, SGI205850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205851, SGI0, SGI0, SGI0, SGI205852, SGI205852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45702[] = {SGI205853, SGI205854, SGI205855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205856, SGI0, SGI0, SGI0, SGI205857, SGI205857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45703[] = {SGI205858, SGI0, SGI205859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205861, SGI205862, SGI205862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45704[] = {SGI205860, SGI0, SGI205863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205867, SGI205868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45705[] = {SGI205864, SGI205865, SGI205866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205869, SGI0, SGI0, SGI0, SGI205870, SGI205870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45706[] = {SGI205871, SGI0, SGI205872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45707[] = {SGI205873, SGI205874, SGI205875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205878, SGI0, SGI0, SGI0, SGI205879, SGI205879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45708[] = {SGI205876, SGI0, SGI205877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205880, SGI205881, SGI205881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45709[] = {SGI205882, SGI205883, SGI205884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205886, SGI0, SGI0, SGI0, SGI205888, SGI205888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45710[] = {SGI205885, SGI0, SGI205887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205889, SGI205890, SGI205890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45711[] = {SGI205891, SGI0, SGI205892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205893, SGI205894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45712[] = {SGI205895, SGI205896, SGI205897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205898, SGI205898, SGI0, SGI0, SGI0, SGI205899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45713[] = {SGI205900, SGI0, SGI205901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205902, SGI205903, SGI205903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45714[] = {SGI205904, SGI205905, SGI205906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205909, SGI0, SGI0, SGI0, SGI205910, SGI205910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45715[] = {SGI205907, SGI0, SGI205908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205912, SGI205913, SGI205913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45716[] = {SGI205914, SGI0, SGI205915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205916, SGI205917, SGI205917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45717[] = {SGI205918, SGI205919, SGI205920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205921, SGI0, SGI0, SGI0, SGI205922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45718[] = {SGI205923, SGI205924, SGI205925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205926, SGI0, SGI0, SGI0, SGI205927, SGI205927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205928, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45719[] = {SGI205929, SGI0, SGI205930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205931, SGI205932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45720[] = {SGI205933, SGI0, SGI205934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205935, SGI205936, SGI205936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45721[] = {SGI205937, SGI0, SGI205938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205939, SGI205940, SGI205940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45722[] = {SGI205941, SGI205942, SGI205943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205944, SGI0, SGI0, SGI0, SGI205945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45723[] = {SGI205946, SGI205947, SGI205948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205949, SGI0, SGI0, SGI0, SGI205950, SGI205950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45724[] = {SGI205951, SGI205952, SGI205953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205954, SGI0, SGI0, SGI0, SGI205955, SGI205955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45725[] = {SGI205956, SGI205957, SGI205958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205960, SGI0, SGI0, SGI0, SGI205963, SGI205963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45726[] = {SGI205959, SGI205961, SGI205962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205964, SGI0, SGI0, SGI0, SGI205965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45727[] = {SGI205966, SGI205967, SGI205968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205969, SGI0, SGI0, SGI0, SGI205970, SGI205970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45728[] = {SGI205971, SGI205972, SGI205973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205974, SGI0, SGI0, SGI0, SGI205975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45729[] = {SGI205976, SGI0, SGI205977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205978, SGI205979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45730[] = {SGI205980, SGI205981, SGI205982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205983, SGI205983, SGI0, SGI0, SGI205984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45731[] = {SGI205985, SGI205986, SGI205987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205988, SGI205988, SGI0, SGI0, SGI0, SGI0, SGI205989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45732[] = {SGI205990, SGI0, SGI205991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205992, SGI0, SGI0, SGI0, SGI205993, SGI205993, SGI0, SGI0, SGI0, SGI205994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45733[] = {SGI205995, SGI0, SGI205996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI205997, SGI205998, SGI205998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45734[] = {SGI205999, SGI0, SGI206000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206001, SGI206002, SGI206002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45735[] = {SGI206003, SGI206004, SGI206005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206006, SGI0, SGI0, SGI0, SGI206007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45736[] = {SGI206008, SGI206009, SGI206010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206011, SGI206011, SGI0, SGI0, SGI206012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45737[] = {SGI206013, SGI206014, SGI206015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206016, SGI0, SGI0, SGI0, SGI206017, SGI206017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45738[] = {SGI206018, SGI0, SGI206019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206020, SGI206021, SGI206021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45739[] = {SGI206022, SGI0, SGI206024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206027, SGI206028, SGI206028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45740[] = {SGI206023, SGI206025, SGI206026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206029, SGI0, SGI0, SGI0, SGI206030, SGI206030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206032, SGI0, NULL};
const Segment * const SG45741[] = {SGI206033, SGI0, SGI206034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206038, SGI206039, SGI206039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45742[] = {SGI206035, SGI206036, SGI206037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206040, SGI0, SGI0, SGI0, SGI206042, SGI206042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206046, SGI0, NULL};
const Segment * const SG45743[] = {SGI206041, SGI206043, SGI206044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206047, SGI0, SGI0, SGI0, SGI206048, SGI206048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45744[] = {SGI206049, SGI206050, SGI206051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206052, SGI0, SGI0, SGI0, SGI206053, SGI206053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206055, SGI0, NULL};
const Segment * const SG45745[] = {SGI206056, SGI206057, SGI206058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206059, SGI206059, SGI0, SGI0, SGI0, SGI0, SGI206060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45746[] = {SGI206061, SGI0, SGI206062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206063, SGI206064, SGI206064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45747[] = {SGI206065, SGI206066, SGI206067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206068, SGI0, SGI0, SGI0, SGI206069, SGI206069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45748[] = {SGI206071, SGI0, SGI206072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206073, SGI206074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45749[] = {SGI206075, SGI206076, SGI206077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206080, SGI0, SGI0, SGI0, SGI206081, SGI0, SGI0, SGI206082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45750[] = {SGI206078, SGI0, SGI206079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206083, SGI206084, SGI206084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45751[] = {SGI206089, SGI0, SGI206090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206091, SGI206092, SGI206092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45752[] = {SGI206093, SGI0, SGI206094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206095, SGI206096, SGI206096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45753[] = {SGI206097, SGI206098, SGI206099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206100, SGI0, SGI0, SGI0, SGI206101, SGI206101, SGI0, SGI0, SGI206102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45754[] = {SGI206103, SGI206104, SGI206105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206106, SGI206106, SGI0, SGI0, SGI0, SGI0, SGI206107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45755[] = {SGI206108, SGI206109, SGI206110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206111, SGI206111, SGI0, SGI0, SGI0, SGI0, SGI206112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45756[] = {SGI206113, SGI206114, SGI206115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206116, SGI206116, SGI0, SGI0, SGI0, SGI0, SGI206117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45757[] = {SGI206118, SGI0, SGI206119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206120, SGI206121, SGI206121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45758[] = {SGI206122, SGI0, SGI206123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206124, SGI206125, SGI206125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45759[] = {SGI206126, SGI0, SGI206127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206128, SGI206129, SGI206129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45760[] = {SGI206130, SGI206131, SGI206132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206133, SGI0, SGI0, SGI0, SGI206134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45761[] = {SGI206135, SGI0, SGI206136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206137, SGI206138, SGI206138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45762[] = {SGI206139, SGI206140, SGI206141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206142, SGI0, SGI0, SGI0, SGI206143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45763[] = {SGI206085, SGI0, SGI206086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206087, SGI206088, SGI206088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45764[] = {SGI206144, SGI206145, SGI206146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206147, SGI0, SGI0, SGI0, SGI206148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45765[] = {SGI206149, SGI206150, SGI206151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206152, SGI206152, SGI0, SGI0, SGI0, SGI206153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45766[] = {SGI206154, SGI206155, SGI206156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206159, SGI0, SGI0, SGI0, SGI206160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45767[] = {SGI206157, SGI0, SGI206158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206161, SGI206162, SGI206162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45768[] = {SGI206163, SGI206164, SGI206165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206166, SGI0, SGI0, SGI0, SGI206167, SGI206167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45769[] = {SGI206168, SGI0, SGI206169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206170, SGI206171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45770[] = {SGI206172, SGI0, SGI206173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206174, SGI206175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45771[] = {SGI206176, SGI206177, SGI206178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206179, SGI0, SGI0, SGI0, SGI206180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45772[] = {SGI206181, SGI206182, SGI206183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206184, SGI0, SGI0, SGI0, SGI206185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45773[] = {SGI206186, SGI206187, SGI206188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206189, SGI0, SGI0, SGI0, SGI206190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45774[] = {SGI206191, SGI0, SGI206192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206193, SGI206194, SGI206194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206195, SGI0, SGI0, NULL};
const Segment * const SG45775[] = {SGI206196, SGI206197, SGI206198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206199, SGI0, SGI0, SGI0, SGI206200, SGI206200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45776[] = {SGI206201, SGI206202, SGI206203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206204, SGI0, SGI0, SGI0, SGI206205, SGI206205, SGI0, SGI206206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45777[] = {SGI206207, SGI0, SGI206208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206209, SGI206210, SGI206210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45778[] = {SGI206211, SGI206212, SGI206213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206214, SGI0, SGI0, SGI0, SGI206215, SGI206215, SGI0, SGI206216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45779[] = {SGI206217, SGI0, SGI206218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206219, SGI206220, SGI206220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45780[] = {SGI206221, SGI0, SGI206222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206223, SGI206224, SGI206224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45781[] = {SGI206225, SGI0, SGI206226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206227, SGI206228, SGI206228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45782[] = {SGI206233, SGI206234, SGI206235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206236, SGI0, SGI0, SGI0, SGI206237, SGI206237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45783[] = {SGI206229, SGI0, SGI206230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206231, SGI0, SGI0, SGI0, SGI206232, SGI206232, SGI0, SGI0, SGI0, SGI206241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45784[] = {SGI206238, SGI206239, SGI206240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206243, SGI0, SGI0, SGI0, SGI206245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45785[] = {SGI206242, SGI0, SGI206244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206246, SGI206247, SGI206247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45786[] = {SGI206248, SGI206249, SGI206250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206253, SGI0, SGI0, SGI0, SGI206255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45787[] = {SGI206251, SGI206252, SGI206254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206256, SGI0, SGI0, SGI0, SGI206257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45788[] = {SGI206258, SGI206259, SGI206260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206261, SGI0, SGI0, SGI0, SGI206262, SGI206262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45789[] = {SGI206264, SGI0, SGI206265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206266, SGI206267, SGI206267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45790[] = {SGI206268, SGI206269, SGI206270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206271, SGI0, SGI0, SGI0, SGI206272, SGI206272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45791[] = {SGI206273, SGI0, SGI206274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45792[] = {SGI206275, SGI206276, SGI206277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206278, SGI0, SGI0, SGI0, SGI206279, SGI206279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45793[] = {SGI206280, SGI0, SGI206281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206282, SGI206283, SGI206283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45794[] = {SGI206284, SGI206285, SGI206286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206287, SGI0, SGI0, SGI0, SGI206288, SGI206288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45795[] = {SGI206289, SGI206290, SGI206291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206292, SGI0, SGI0, SGI0, SGI206293, SGI206293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45796[] = {SGI206294, SGI206295, SGI206296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206297, SGI0, SGI0, SGI0, SGI206298, SGI206298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45797[] = {SGI206299, SGI206300, SGI206301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206302, SGI0, SGI0, SGI0, SGI206303, SGI206303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45798[] = {SGI206304, SGI0, SGI206305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206306, SGI206307, SGI206307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45799[] = {SGI206308, SGI206309, SGI206310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206314, SGI0, SGI0, SGI0, SGI206315, SGI206315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45800[] = {SGI206311, SGI206312, SGI206313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206316, SGI0, SGI0, SGI0, SGI206317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45801[] = {SGI206318, SGI206319, SGI206320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206321, SGI0, SGI0, SGI0, SGI206322, SGI206322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45802[] = {SGI206323, SGI0, SGI206324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206325, SGI206326, SGI206326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45803[] = {SGI206327, SGI206328, SGI206329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206330, SGI0, SGI0, SGI0, SGI206331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45804[] = {SGI206332, SGI0, SGI206333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206334, SGI206335, SGI206335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45805[] = {SGI206336, SGI206337, SGI206338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206339, SGI206339, SGI0, SGI0, SGI0, SGI206340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45806[] = {SGI206341, SGI0, SGI206342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206343, SGI206344, SGI206344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45807[] = {SGI206345, SGI0, SGI206346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206347, SGI206348, SGI206348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45808[] = {SGI206349, SGI206350, SGI206351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206352, SGI206352, SGI0, SGI0, SGI0, SGI0, SGI206353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45809[] = {SGI206354, SGI206355, SGI206356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206357, SGI206357, SGI0, SGI0, SGI0, SGI206358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45810[] = {SGI206359, SGI0, SGI206360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206361, SGI206362, SGI206362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206363, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45811[] = {SGI206364, SGI0, SGI206365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206368, SGI206369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206372, SGI0, SGI0, NULL};
const Segment * const SG45812[] = {SGI206366, SGI0, SGI206367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206370, SGI206371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206373, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45813[] = {SGI206374, SGI0, SGI206375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206376, SGI206377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45814[] = {SGI206378, SGI206379, SGI206380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206381, SGI0, SGI0, SGI0, SGI206382, SGI206382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206383, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45815[] = {SGI206384, SGI206385, SGI206386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206387, SGI0, SGI0, SGI0, SGI206388, SGI206388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206389, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45816[] = {SGI206390, SGI0, SGI206391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206392, SGI206393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45817[] = {SGI206394, SGI0, SGI206395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206396, SGI206397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45818[] = {SGI206398, SGI206399, SGI206400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206401, SGI0, SGI0, SGI0, SGI206402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45819[] = {SGI206403, SGI0, SGI206404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206405, SGI206406, SGI206406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45820[] = {SGI206407, SGI0, SGI206408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206411, SGI206412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45821[] = {SGI0, SGI206409, SGI206410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206413, SGI0, SGI0, SGI0, SGI206414, SGI206414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45822[] = {SGI206415, SGI206416, SGI206417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206418, SGI206418, SGI0, SGI0, SGI0, SGI0, SGI206419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45823[] = {SGI206420, SGI206421, SGI206422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206423, SGI0, SGI0, SGI0, SGI206424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45824[] = {SGI206425, SGI0, SGI206426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206427, SGI206428, SGI206428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45825[] = {SGI206429, SGI0, SGI206430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206431, SGI206432, SGI206432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45826[] = {SGI206433, SGI206434, SGI206435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206436, SGI0, SGI0, SGI0, SGI206437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45827[] = {SGI206438, SGI0, SGI206439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206440, SGI0, SGI0, SGI0, SGI206441, SGI206441, SGI0, SGI0, SGI0, SGI206442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45828[] = {SGI206443, SGI0, SGI206444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206445, SGI206446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45829[] = {SGI206447, SGI206448, SGI206449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206450, SGI0, SGI0, SGI0, SGI206451, SGI206451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45830[] = {SGI206452, SGI206453, SGI206454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206456, SGI0, SGI0, SGI0, SGI206457, SGI206457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45831[] = {SGI206458, SGI206459, SGI206460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206461, SGI0, SGI0, SGI0, SGI206462, SGI206462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45832[] = {SGI206464, SGI206465, SGI206466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206467, SGI0, SGI0, SGI0, SGI206468, SGI206468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45833[] = {SGI206470, SGI206471, SGI206472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206476, SGI206476, SGI0, SGI0, SGI0, SGI206477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45834[] = {SGI206473, SGI206474, SGI206475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206478, SGI0, SGI0, SGI0, SGI206479, SGI206479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45835[] = {SGI206480, SGI0, SGI206481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206482, SGI206483, SGI206483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45836[] = {SGI206484, SGI0, SGI206485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206486, SGI206487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45837[] = {SGI206488, SGI0, SGI206489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206490, SGI206491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45838[] = {SGI206492, SGI0, SGI206493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206494, SGI206495, SGI206495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45839[] = {SGI206496, SGI0, SGI206497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206500, SGI206501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45840[] = {SGI206498, SGI0, SGI206499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206502, SGI206503, SGI206503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45841[] = {SGI206504, SGI0, SGI206505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206506, SGI206507, SGI206507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45842[] = {SGI206508, SGI0, SGI206509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206510, SGI206511, SGI206511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45843[] = {SGI206512, SGI0, SGI206513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206514, SGI206515, SGI206515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45844[] = {SGI206516, SGI0, SGI206517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206518, SGI206519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45845[] = {SGI206520, SGI0, SGI206521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206522, SGI206523, SGI206523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45846[] = {SGI206524, SGI206525, SGI206526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206527, SGI206527, SGI0, SGI0, SGI0, SGI206528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45847[] = {SGI206529, SGI0, SGI206530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206531, SGI206532, SGI206532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45848[] = {SGI206533, SGI0, SGI206534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206535, SGI206536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45849[] = {SGI206537, SGI0, SGI206538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206539, SGI206540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45850[] = {SGI206541, SGI206542, SGI206543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206546, SGI0, SGI0, SGI206548, SGI206549, SGI206549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45851[] = {SGI206544, SGI206545, SGI206547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206550, SGI206550, SGI0, SGI0, SGI206551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45852[] = {SGI206552, SGI206553, SGI206554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206555, SGI0, SGI0, SGI206556, SGI206557, SGI206557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45853[] = {SGI206558, SGI0, SGI206559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206560, SGI206561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45854[] = {SGI206562, SGI206563, SGI206564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206565, SGI0, SGI0, SGI0, SGI206566, SGI206566, SGI0, SGI0, SGI0, SGI0, SGI206567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45855[] = {SGI206568, SGI206569, SGI206570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206571, SGI0, SGI0, SGI0, SGI0, SGI206572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45856[] = {SGI206573, SGI0, SGI206574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206575, SGI206577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45857[] = {SGI206576, SGI206578, SGI206579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206580, SGI0, SGI0, SGI0, SGI206581, SGI206581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45858[] = {SGI206582, SGI206583, SGI206584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206585, SGI206585, SGI0, SGI0, SGI0, SGI0, SGI206586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45859[] = {SGI206587, SGI206588, SGI206589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206590, SGI0, SGI0, SGI0, SGI206591, SGI206591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206592, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45860[] = {SGI206593, SGI206594, SGI206595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206596, SGI0, SGI0, SGI0, SGI206597, SGI206597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206598, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45861[] = {SGI206599, SGI0, SGI206600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206603, SGI206605, SGI206605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45862[] = {SGI206601, SGI206602, SGI206604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206606, SGI0, SGI0, SGI0, SGI206607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45863[] = {SGI206608, SGI206609, SGI206610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206611, SGI0, SGI0, SGI0, SGI206613, SGI206613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206615, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45864[] = {SGI206612, SGI0, SGI206614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206616, SGI206617, SGI206617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45865[] = {SGI206618, SGI0, SGI206619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206620, SGI206621, SGI206621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45866[] = {SGI206622, SGI0, SGI206623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206624, SGI206625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45867[] = {SGI206626, SGI206627, SGI206628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206629, SGI0, SGI0, SGI0, SGI206630, SGI206630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206631, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45868[] = {SGI206632, SGI206633, SGI206634, SGI0, SGI0, SGI0, SGI206635, SGI206637, SGI0, SGI206639, SGI0, SGI0, SGI0, SGI206640, SGI206640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45869[] = {SGI206636, SGI0, SGI206638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206641, SGI206642, SGI206642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45870[] = {SGI206643, SGI0, SGI206644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206645, SGI206646, SGI206646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45871[] = {SGI206647, SGI0, SGI206648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206649, SGI206650, SGI206650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45872[] = {SGI206651, SGI0, SGI206652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206653, SGI206654, SGI206654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45873[] = {SGI206655, SGI0, SGI206656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206657, SGI206658, SGI206658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45874[] = {SGI206659, SGI0, SGI206660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206667, SGI206668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45875[] = {SGI206661, SGI206663, SGI206665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206669, SGI0, SGI0, SGI0, SGI206671, SGI206671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45876[] = {SGI206662, SGI206664, SGI206666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206670, SGI0, SGI0, SGI0, SGI206672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45877[] = {SGI206673, SGI0, SGI206674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206675, SGI0, SGI0, SGI0, SGI206676, SGI206676, SGI0, SGI0, SGI0, SGI206677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45878[] = {SGI206678, SGI0, SGI206679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206680, SGI0, SGI0, SGI0, SGI206681, SGI206681, SGI0, SGI0, SGI0, SGI206682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45879[] = {SGI206683, SGI0, SGI206684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206685, SGI206686, SGI206686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45880[] = {SGI206687, SGI206688, SGI206689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206690, SGI0, SGI0, SGI0, SGI206691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45881[] = {SGI206692, SGI206693, SGI206694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206695, SGI0, SGI0, SGI0, SGI206696, SGI0, SGI206697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45882[] = {SGI206698, SGI206699, SGI206700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206703, SGI0, SGI0, SGI0, SGI206704, SGI0, SGI206706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45883[] = {SGI206701, SGI0, SGI206702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206705, SGI206707, SGI206707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45884[] = {SGI206708, SGI0, SGI206709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206711, SGI206713, SGI206713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45885[] = {SGI206710, SGI0, SGI206712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206714, SGI206715, SGI206715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45886[] = {SGI206716, SGI0, SGI206717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206718, SGI206719, SGI206719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45887[] = {SGI206720, SGI206721, SGI206722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206723, SGI0, SGI0, SGI0, SGI206724, SGI206724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45888[] = {SGI206726, SGI0, SGI206727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206728, SGI206729, SGI206729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45889[] = {SGI206730, SGI206731, SGI206732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206733, SGI0, SGI0, SGI0, SGI206734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45890[] = {SGI206735, SGI0, SGI206736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206737, SGI206738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45891[] = {SGI206739, SGI0, SGI206740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206741, SGI206742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45892[] = {SGI206743, SGI0, SGI206744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206745, SGI206746, SGI206746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45893[] = {SGI206747, SGI206749, SGI206750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206752, SGI0, SGI0, SGI0, SGI206754, SGI206754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45894[] = {SGI206748, SGI0, SGI206751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206753, SGI206755, SGI206755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45895[] = {SGI206757, SGI0, SGI206758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45896[] = {SGI206756, SGI0, SGI206759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206760, SGI206761, SGI206761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45897[] = {SGI206762, SGI206763, SGI206764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206765, SGI0, SGI0, SGI0, SGI206766, SGI206766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45898[] = {SGI206767, SGI206768, SGI206769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206770, SGI206770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45899[] = {SGI206771, SGI0, SGI206772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206773, SGI206774, SGI206774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45900[] = {SGI206775, SGI0, SGI206776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206777, SGI206778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45901[] = {SGI206779, SGI0, SGI206780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206781, SGI206782, SGI206782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45902[] = {SGI206783, SGI0, SGI206784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206785, SGI206786, SGI206786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45903[] = {SGI206787, SGI0, SGI206788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206789, SGI206790, SGI206790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45904[] = {SGI206791, SGI0, SGI206792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206793, SGI206794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45905[] = {SGI206795, SGI206796, SGI206797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206798, SGI0, SGI0, SGI0, SGI206799, SGI206799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45906[] = {SGI206801, SGI206802, SGI206803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206804, SGI0, SGI0, SGI0, SGI206805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45907[] = {SGI206806, SGI0, SGI206807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45908[] = {SGI206808, SGI0, SGI206809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206810, SGI206811, SGI206811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45909[] = {SGI206812, SGI0, SGI206813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206814, SGI206815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45910[] = {SGI206816, SGI0, SGI206818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206821, SGI206823, SGI206823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45911[] = {SGI206817, SGI206819, SGI206820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206822, SGI0, SGI0, SGI0, SGI206825, SGI206825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45912[] = {SGI206824, SGI0, SGI206826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206828, SGI206829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45913[] = {SGI206830, SGI0, SGI206831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206835, SGI206836, SGI206836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45914[] = {SGI206832, SGI206833, SGI206834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206838, SGI0, SGI0, SGI0, SGI206840, SGI206840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45915[] = {SGI206837, SGI0, SGI206839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206842, SGI206843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45916[] = {SGI206844, SGI206845, SGI206846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206847, SGI0, SGI0, SGI0, SGI206848, SGI206848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45917[] = {SGI206850, SGI206851, SGI206852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206853, SGI0, SGI0, SGI0, SGI206854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45918[] = {SGI206855, SGI0, SGI206856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206857, SGI206858, SGI206858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45919[] = {SGI206859, SGI206860, SGI206861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206862, SGI0, SGI0, SGI0, SGI206863, SGI206863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45920[] = {SGI206864, SGI206865, SGI206866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206867, SGI0, SGI0, SGI0, SGI206868, SGI206868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45921[] = {SGI206869, SGI206870, SGI206871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206872, SGI0, SGI0, SGI0, SGI206873, SGI206873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45922[] = {SGI206874, SGI0, SGI206875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45923[] = {SGI206876, SGI206877, SGI206878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206881, SGI0, SGI0, SGI0, SGI206882, SGI206882, SGI0, SGI206883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45924[] = {SGI206879, SGI0, SGI206880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206884, SGI206885, SGI206885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45925[] = {SGI206886, SGI206887, SGI206888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206891, SGI0, SGI0, SGI0, SGI206892, SGI206892, SGI0, SGI206894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45926[] = {SGI206889, SGI0, SGI206890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206893, SGI0, SGI0, SGI0, SGI206895, SGI206895, SGI0, SGI0, SGI0, SGI206896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45927[] = {SGI206897, SGI0, SGI206898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206899, SGI206900, SGI206900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45928[] = {SGI206901, SGI0, SGI206902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206903, SGI206904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45929[] = {SGI206905, SGI0, SGI206906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206907, SGI206908, SGI206908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45930[] = {SGI206909, SGI206910, SGI206911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206914, SGI206914, SGI0, SGI0, SGI0, SGI0, SGI206917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45931[] = {SGI206912, SGI0, SGI206913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206915, SGI206916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45932[] = {SGI206918, SGI0, SGI206919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206924, SGI206925, SGI206925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45933[] = {SGI206920, SGI0, SGI206922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206926, SGI206927, SGI206927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45934[] = {SGI206921, SGI0, SGI206923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206928, SGI206929, SGI206929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206930, SGI0, SGI0, NULL};
const Segment * const SG45935[] = {SGI206931, SGI0, SGI206932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206933, SGI206934, SGI206934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45936[] = {SGI206935, SGI0, SGI206936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206937, SGI206938, SGI206938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45937[] = {SGI206939, SGI206940, SGI206941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206942, SGI206942, SGI0, SGI0, SGI0, SGI206943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45938[] = {SGI206944, SGI0, SGI206945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206946, SGI206947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45939[] = {SGI206948, SGI206949, SGI206950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206951, SGI0, SGI0, SGI0, SGI206952, SGI206952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45940[] = {SGI206953, SGI0, SGI206954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206955, SGI206956, SGI206956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45941[] = {SGI206957, SGI0, SGI206958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206959, SGI206960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45942[] = {SGI206961, SGI0, SGI206962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206966, SGI0, SGI0, SGI0, SGI206967, SGI206967, SGI0, SGI0, SGI0, SGI206969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45943[] = {SGI206963, SGI206964, SGI206965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206968, SGI0, SGI0, SGI0, SGI206970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45944[] = {SGI206971, SGI0, SGI206972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206976, SGI206977, SGI206977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45945[] = {SGI206973, SGI206974, SGI206975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206978, SGI0, SGI0, SGI0, SGI206979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45946[] = {SGI206980, SGI206981, SGI206982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206983, SGI0, SGI0, SGI0, SGI206984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45947[] = {SGI206985, SGI0, SGI206986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206987, SGI206988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45948[] = {SGI206989, SGI206990, SGI206991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI206992, SGI0, SGI0, SGI0, SGI206993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45949[] = {SGI206994, SGI206995, SGI206996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207000, SGI0, SGI0, SGI0, SGI207001, SGI207001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45950[] = {SGI206997, SGI206998, SGI206999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207002, SGI207002, SGI0, SGI0, SGI0, SGI207003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45951[] = {SGI207004, SGI0, SGI207005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207008, SGI0, SGI0, SGI0, SGI207011, SGI207011, SGI0, SGI0, SGI0, SGI0, SGI207014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45952[] = {SGI207006, SGI0, SGI207007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207013, SGI207015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45953[] = {SGI207009, SGI207010, SGI207012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207016, SGI0, SGI0, SGI0, SGI207017, SGI207017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45954[] = {SGI207018, SGI0, SGI207019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45955[] = {SGI207020, SGI207021, SGI207022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45956[] = {SGI207023, SGI207024, SGI207025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207026, SGI0, SGI0, SGI0, SGI207027, SGI207027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45957[] = {SGI207028, SGI207029, SGI207030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207031, SGI0, SGI0, SGI0, SGI207032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45958[] = {SGI207033, SGI0, SGI207034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207035, SGI207036, SGI207036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45959[] = {SGI207037, SGI0, SGI207038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207039, SGI207040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45960[] = {SGI207041, SGI207042, SGI207043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207044, SGI0, SGI0, SGI0, SGI207045, SGI207045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45961[] = {SGI207046, SGI207047, SGI207048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207052, SGI207052, SGI0, SGI0, SGI207053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45962[] = {SGI207049, SGI207050, SGI207051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207054, SGI0, SGI0, SGI0, SGI207055, SGI207055, SGI207056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45963[] = {SGI207059, SGI0, SGI207061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207062, SGI207064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45964[] = {SGI207057, SGI207058, SGI207060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207063, SGI207063, SGI0, SGI0, SGI207065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45965[] = {SGI207066, SGI0, SGI207067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207068, SGI207069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45966[] = {SGI207070, SGI0, SGI207071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207072, SGI207073, SGI207073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45967[] = {SGI207074, SGI0, SGI207075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207076, SGI207077, SGI207077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45968[] = {SGI207078, SGI0, SGI207079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207080, SGI207081, SGI207081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45969[] = {SGI207082, SGI0, SGI207083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207084, SGI207085, SGI207085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207086, SGI0, SGI0, NULL};
const Segment * const SG45970[] = {SGI207087, SGI0, SGI207088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207089, SGI207090, SGI207090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45971[] = {SGI207091, SGI0, SGI207092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207093, SGI207094, SGI207094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45972[] = {SGI207095, SGI0, SGI207096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207097, SGI207098, SGI207098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45973[] = {SGI207099, SGI207100, SGI207101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207102, SGI207102, SGI0, SGI0, SGI207103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45974[] = {SGI207104, SGI207105, SGI207106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207107, SGI207107, SGI0, SGI0, SGI207108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45975[] = {SGI207109, SGI0, SGI207110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207111, SGI207112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45976[] = {SGI207113, SGI0, SGI207114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207115, SGI207116, SGI207116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45977[] = {SGI207117, SGI0, SGI207118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207119, SGI207120, SGI207120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45978[] = {SGI207121, SGI207122, SGI207123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207124, SGI0, SGI0, SGI0, SGI207125, SGI207125, SGI0, SGI207126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45979[] = {SGI207127, SGI0, SGI207128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207129, SGI207130, SGI207130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45980[] = {SGI207131, SGI207132, SGI207133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207134, SGI0, SGI0, SGI0, SGI207135, SGI0, SGI0, SGI207136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45981[] = {SGI207137, SGI0, SGI207138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207139, SGI207140, SGI207140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207141, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45982[] = {SGI207142, SGI0, SGI207143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207144, SGI207145, SGI207145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45983[] = {SGI207146, SGI207147, SGI207148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207149, SGI0, SGI0, SGI0, SGI207150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45984[] = {SGI207151, SGI0, SGI207152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207153, SGI207154, SGI207154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45985[] = {SGI207155, SGI0, SGI207156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207157, SGI207158, SGI207158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45986[] = {SGI207159, SGI0, SGI207160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207164, SGI207165, SGI207165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45987[] = {SGI207161, SGI207162, SGI207163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207166, SGI0, SGI0, SGI0, SGI207167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45988[] = {SGI207168, SGI0, SGI207169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207170, SGI207171, SGI207171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207172, SGI0, SGI0, NULL};
const Segment * const SG45989[] = {SGI207173, SGI0, SGI207174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207175, SGI207176, SGI207176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45990[] = {SGI207177, SGI0, SGI207178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207179, SGI207180, SGI207180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45991[] = {SGI207181, SGI207182, SGI207183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207184, SGI0, SGI0, SGI0, SGI207185, SGI207185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45992[] = {SGI207186, SGI207187, SGI207188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207192, SGI0, SGI0, SGI0, SGI207193, SGI207193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45993[] = {SGI207189, SGI207190, SGI207191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207194, SGI0, SGI0, SGI0, SGI207195, SGI207195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45994[] = {SGI207196, SGI0, SGI207197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207200, SGI207202, SGI207202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG45995[] = {SGI207198, SGI207199, SGI207201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207203, SGI0, SGI0, SGI0, SGI207204, SGI207204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207205, SGI0, SGI0, NULL};
const Segment * const SG45996[] = {SGI207206, SGI207207, SGI207208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207209, SGI0, SGI0, SGI0, SGI207210, SGI207210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207211, SGI0, SGI0, NULL};
const Segment * const SG45997[] = {SGI207212, SGI207213, SGI207214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207215, SGI0, SGI0, SGI0, SGI207216, SGI207216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207217, SGI0, SGI0, NULL};
const Segment * const SG45998[] = {SGI207218, SGI207219, SGI207220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207221, SGI0, SGI0, SGI0, SGI207222, SGI207222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207223, SGI0, SGI0, NULL};
const Segment * const SG45999[] = {SGI207224, SGI0, SGI207225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207226, SGI207227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46000[] = {SGI207228, SGI0, SGI207229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207230, SGI207231, SGI207231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46001[] = {SGI207232, SGI0, SGI207233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207234, SGI207235, SGI207235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46002[] = {SGI207236, SGI207237, SGI207238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207239, SGI0, SGI0, SGI0, SGI207240, SGI207240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46003[] = {SGI207241, SGI0, SGI207242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207243, SGI207244, SGI207244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46004[] = {SGI207245, SGI0, SGI207246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207247, SGI207248, SGI207248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46005[] = {SGI207249, SGI0, SGI207250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207251, SGI207252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46006[] = {SGI207253, SGI0, SGI207254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207255, SGI207256, SGI207256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207257, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46007[] = {SGI207258, SGI0, SGI207259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207260, SGI207261, SGI207261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207262, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46008[] = {SGI207263, SGI207264, SGI207265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207266, SGI0, SGI0, SGI0, SGI207267, SGI207267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46009[] = {SGI207268, SGI207269, SGI207270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207271, SGI207271, SGI0, SGI0, SGI0, SGI207272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46010[] = {SGI207273, SGI0, SGI207274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207275, SGI207276, SGI207276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46011[] = {SGI207277, SGI0, SGI207278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207279, SGI0, SGI0, SGI0, SGI207280, SGI207280, SGI0, SGI0, SGI0, SGI207281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46012[] = {SGI207282, SGI207283, SGI207284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207285, SGI0, SGI0, SGI0, SGI207286, SGI207287, SGI0, SGI0, SGI0, SGI207288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46013[] = {SGI207289, SGI0, SGI207290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207291, SGI207292, SGI207292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46014[] = {SGI207293, SGI0, SGI207294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207295, SGI207296, SGI207296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46015[] = {SGI207297, SGI0, SGI207298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207299, SGI207300, SGI207300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46016[] = {SGI207301, SGI0, SGI207302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207303, SGI207304, SGI207304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46017[] = {SGI207305, SGI0, SGI207306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207307, SGI207308, SGI207308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46018[] = {SGI207309, SGI207310, SGI207311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207313, SGI0, SGI0, SGI0, SGI207317, SGI207317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46019[] = {SGI207312, SGI0, SGI207315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207319, SGI207320, SGI207320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46020[] = {SGI207314, SGI207316, SGI207318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207321, SGI0, SGI0, SGI0, SGI207322, SGI207322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46021[] = {SGI207324, SGI207325, SGI207326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207327, SGI0, SGI0, SGI0, SGI207328, SGI207328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46022[] = {SGI207329, SGI207330, SGI207331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207332, SGI0, SGI0, SGI0, SGI207333, SGI0, SGI207334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46023[] = {SGI207335, SGI207336, SGI207337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207338, SGI0, SGI0, SGI0, SGI207339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46024[] = {SGI207340, SGI0, SGI207341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207342, SGI207343, SGI207343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46025[] = {SGI207344, SGI207345, SGI207346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207347, SGI0, SGI0, SGI0, SGI207348, SGI207348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46026[] = {SGI207349, SGI207350, SGI207351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207354, SGI207354, SGI0, SGI0, SGI207356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46027[] = {SGI207358, SGI0, SGI207360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46028[] = {SGI207352, SGI207353, SGI207355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207359, SGI0, SGI0, SGI0, SGI207362, SGI207362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207363, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46029[] = {SGI207357, SGI0, SGI207361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207364, SGI207365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46030[] = {SGI207366, SGI0, SGI207367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207368, SGI207369, SGI207369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46031[] = {SGI207370, SGI207371, SGI207372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207373, SGI0, SGI0, SGI0, SGI207374, SGI207374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207375, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46032[] = {SGI207376, SGI0, SGI207377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207378, SGI207379, SGI207379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46033[] = {SGI207380, SGI0, SGI207381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207382, SGI207383, SGI207383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46034[] = {SGI207384, SGI207385, SGI207386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207387, SGI207387, SGI0, SGI0, SGI0, SGI0, SGI207388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46035[] = {SGI207389, SGI207390, SGI207391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207392, SGI0, SGI0, SGI0, SGI207393, SGI207393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46036[] = {SGI0, SGI207394, SGI0, SGI207395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46037[] = {SGI207396, SGI207397, SGI207398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207399, SGI0, SGI0, SGI0, SGI207400, SGI0, SGI0, SGI0, SGI207401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46038[] = {SGI207402, SGI0, SGI207403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207404, SGI207405, SGI207405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46039[] = {SGI207406, SGI0, SGI207407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207408, SGI207409, SGI207409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46040[] = {SGI207410, SGI0, SGI207411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207412, SGI207413, SGI207413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46041[] = {SGI207414, SGI0, SGI207415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207416, SGI207417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46042[] = {SGI207418, SGI0, SGI207419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207420, SGI207421, SGI207421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46043[] = {SGI207422, SGI207423, SGI207424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207425, SGI0, SGI0, SGI0, SGI207426, SGI207427, SGI0, SGI0, SGI0, SGI207428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46044[] = {SGI207429, SGI0, SGI207430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207431, SGI0, SGI0, SGI0, SGI207432, SGI207432, SGI0, SGI0, SGI0, SGI207433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46045[] = {SGI207434, SGI0, SGI207435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207436, SGI207437, SGI207437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46046[] = {SGI207438, SGI207439, SGI207440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207441, SGI0, SGI0, SGI0, SGI207442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46047[] = {SGI207444, SGI207445, SGI207446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207447, SGI0, SGI0, SGI0, SGI207448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46048[] = {SGI207450, SGI207451, SGI207452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207455, SGI0, SGI0, SGI0, SGI207456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46049[] = {SGI207453, SGI0, SGI207454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207457, SGI207459, SGI207459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46050[] = {SGI207460, SGI207461, SGI207462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207463, SGI0, SGI0, SGI0, SGI207464, SGI207464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207465, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46051[] = {SGI207466, SGI207467, SGI207468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207469, SGI0, SGI0, SGI0, SGI207470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46052[] = {SGI207471, SGI0, SGI207472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46053[] = {SGI207473, SGI207474, SGI207475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207476, SGI0, SGI0, SGI0, SGI207477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46054[] = {SGI207478, SGI0, SGI207479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207480, SGI207481, SGI207481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46055[] = {SGI207482, SGI207483, SGI207485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207487, SGI0, SGI0, SGI0, SGI207488, SGI207488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46056[] = {SGI207484, SGI0, SGI207486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207489, SGI207490, SGI207490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46057[] = {SGI207491, SGI207492, SGI207493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207494, SGI0, SGI0, SGI0, SGI207495, SGI207495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46058[] = {SGI207496, SGI0, SGI207497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46059[] = {SGI207498, SGI207499, SGI207500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207501, SGI0, SGI0, SGI0, SGI207502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46060[] = {SGI207503, SGI207504, SGI207505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207506, SGI0, SGI0, SGI0, SGI207507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46061[] = {SGI207508, SGI0, SGI207509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207510, SGI207511, SGI207511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46062[] = {SGI207512, SGI207513, SGI207514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207515, SGI0, SGI0, SGI0, SGI207516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46063[] = {SGI207517, SGI0, SGI207518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207519, SGI207520, SGI207520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46064[] = {SGI207521, SGI207522, SGI207523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207524, SGI0, SGI0, SGI0, SGI207525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46065[] = {SGI207526, SGI0, SGI207527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207528, SGI207529, SGI207529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46066[] = {SGI207530, SGI0, SGI207531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207532, SGI207533, SGI207533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46067[] = {SGI207534, SGI207535, SGI207536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207537, SGI0, SGI0, SGI0, SGI207538, SGI207538, SGI0, SGI0, SGI207539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46068[] = {SGI207540, SGI0, SGI207541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207542, SGI207543, SGI207543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46069[] = {SGI207544, SGI0, SGI207545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207546, SGI207547, SGI207547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46070[] = {SGI207548, SGI0, SGI207549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207550, SGI0, SGI0, SGI0, SGI207551, SGI207551, SGI0, SGI0, SGI0, SGI207552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46071[] = {SGI207553, SGI0, SGI207554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207555, SGI207556, SGI207556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46072[] = {SGI207557, SGI0, SGI207558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207559, SGI207560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46073[] = {SGI207561, SGI207562, SGI207563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207564, SGI0, SGI0, SGI0, SGI207565, SGI207565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207566, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46074[] = {SGI207567, SGI0, SGI207568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207570, SGI207571, SGI207571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46075[] = {SGI207569, SGI207572, SGI207573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207574, SGI0, SGI0, SGI0, SGI207575, SGI207575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207576, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46076[] = {SGI207577, SGI0, SGI207578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207579, SGI207580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46077[] = {SGI207581, SGI207582, SGI207583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207584, SGI207584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207585, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46078[] = {SGI207586, SGI0, SGI207587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207588, SGI207589, SGI207589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46079[] = {SGI207590, SGI0, SGI207591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207592, SGI207593, SGI207593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46080[] = {SGI207594, SGI0, SGI207595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207596, SGI207597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46081[] = {SGI207598, SGI207599, SGI207600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207601, SGI0, SGI0, SGI0, SGI207602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207603, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46082[] = {SGI207604, SGI0, SGI207605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207606, SGI207607, SGI207607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46083[] = {SGI207608, SGI0, SGI207609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207610, SGI207611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46084[] = {SGI207612, SGI0, SGI207613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207614, SGI207615, SGI207615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46085[] = {SGI207616, SGI207617, SGI207618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207619, SGI207619, SGI0, SGI0, SGI0, SGI207620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46086[] = {SGI207621, SGI207622, SGI207623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207624, SGI0, SGI0, SGI0, SGI207625, SGI207625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207626, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46087[] = {SGI207627, SGI207628, SGI207629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207630, SGI0, SGI0, SGI0, SGI207631, SGI207631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207632, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46088[] = {SGI207633, SGI0, SGI207634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207635, SGI207636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46089[] = {SGI207637, SGI0, SGI207638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207639, SGI207640, SGI207640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46090[] = {SGI207641, SGI0, SGI207642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207643, SGI207644, SGI207644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46091[] = {SGI207645, SGI207646, SGI207647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207648, SGI0, SGI0, SGI0, SGI207649, SGI207649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46092[] = {SGI207650, SGI207651, SGI207652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207653, SGI207653, SGI0, SGI0, SGI0, SGI207654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46093[] = {SGI207655, SGI207657, SGI207658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207661, SGI0, SGI0, SGI0, SGI207662, SGI207662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46094[] = {SGI207656, SGI207659, SGI207660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207663, SGI207663, SGI0, SGI0, SGI0, SGI207664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46095[] = {SGI207665, SGI0, SGI207666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207667, SGI207668, SGI207668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46096[] = {SGI207669, SGI0, SGI207670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207671, SGI207672, SGI207672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46097[] = {SGI207673, SGI0, SGI207676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207679, SGI207680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46098[] = {SGI207674, SGI207675, SGI207677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207678, SGI0, SGI0, SGI0, SGI207681, SGI207681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46099[] = {SGI207682, SGI207683, SGI207684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207685, SGI0, SGI0, SGI0, SGI207686, SGI207686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46100[] = {SGI207687, SGI0, SGI207688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46101[] = {SGI207689, SGI207690, SGI207691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207692, SGI0, SGI0, SGI0, SGI207693, SGI207693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46102[] = {SGI207694, SGI207695, SGI207696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207697, SGI0, SGI0, SGI0, SGI207698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46103[] = {SGI207699, SGI0, SGI207700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207701, SGI207702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46104[] = {SGI207703, SGI207704, SGI207705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207706, SGI207706, SGI0, SGI0, SGI0, SGI0, SGI207707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46105[] = {SGI207708, SGI207709, SGI207710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207711, SGI207711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207712, NULL};
const Segment * const SG46106[] = {SGI207713, SGI0, SGI207714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207715, SGI207716, SGI207716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46107[] = {SGI207717, SGI207718, SGI207719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207720, SGI207720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207721, NULL};
const Segment * const SG46108[] = {SGI207722, SGI0, SGI207723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207724, SGI207725, SGI207725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46109[] = {SGI207726, SGI207727, SGI207728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207729, SGI207729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207730, NULL};
const Segment * const SG46110[] = {SGI207731, SGI207732, SGI207733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207734, SGI0, SGI0, SGI0, SGI207735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46111[] = {SGI207736, SGI207737, SGI207738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207739, SGI207739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207740, NULL};
const Segment * const SG46112[] = {SGI207741, SGI207742, SGI207743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207744, SGI0, SGI0, SGI0, SGI207745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207746, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46113[] = {SGI207747, SGI0, SGI207748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207749, SGI207750, SGI207750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46114[] = {SGI207751, SGI0, SGI207752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207753, SGI207754, SGI207754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46115[] = {SGI207755, SGI207756, SGI207757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207758, SGI0, SGI0, SGI0, SGI207759, SGI207759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46116[] = {SGI207760, SGI207761, SGI207762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207763, SGI0, SGI0, SGI0, SGI207764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46117[] = {SGI207765, SGI0, SGI207766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207767, SGI207768, SGI207768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46118[] = {SGI207769, SGI0, SGI207770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207771, SGI207772, SGI207772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46119[] = {SGI207773, SGI207774, SGI207775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207776, SGI0, SGI0, SGI0, SGI207777, SGI207777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46120[] = {SGI207779, SGI0, SGI207780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207781, SGI207782, SGI207782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46121[] = {SGI207783, SGI0, SGI207784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207785, SGI207786, SGI207786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46122[] = {SGI207787, SGI207788, SGI207789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207790, SGI0, SGI0, SGI0, SGI207791, SGI207791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46123[] = {SGI207792, SGI207793, SGI207794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207795, SGI0, SGI0, SGI0, SGI207796, SGI207796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207797, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46124[] = {SGI207798, SGI207799, SGI207801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207803, SGI0, SGI0, SGI0, SGI207805, SGI207805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46125[] = {SGI207800, SGI0, SGI207802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207804, SGI207806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46126[] = {SGI207807, SGI0, SGI207808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207809, SGI0, SGI0, SGI0, SGI207810, SGI207810, SGI0, SGI0, SGI0, SGI207811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46127[] = {SGI207812, SGI207813, SGI207814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207815, SGI0, SGI0, SGI0, SGI207816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46128[] = {SGI207817, SGI207818, SGI207819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207820, SGI0, SGI0, SGI0, SGI207821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46129[] = {SGI207822, SGI207823, SGI207824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207825, SGI0, SGI0, SGI0, SGI207826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46130[] = {SGI207827, SGI207828, SGI207829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207832, SGI207832, SGI0, SGI0, SGI0, SGI207833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46131[] = {SGI207830, SGI0, SGI207831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207834, SGI207835, SGI207835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46132[] = {SGI207836, SGI207837, SGI207838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207841, SGI207841, SGI0, SGI0, SGI0, SGI207843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46133[] = {SGI207839, SGI207840, SGI207842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207844, SGI0, SGI0, SGI0, SGI207846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46134[] = {SGI207845, SGI0, SGI207847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207848, SGI207849, SGI207849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46135[] = {SGI207851, SGI0, SGI207852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46136[] = {SGI207850, SGI0, SGI207853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207856, SGI207857, SGI207857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207859, SGI0, SGI0, NULL};
const Segment * const SG46137[] = {SGI207854, SGI0, SGI207855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207860, SGI207862, SGI207862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46138[] = {SGI207858, SGI0, SGI207861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207863, SGI207864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46139[] = {SGI207865, SGI0, SGI207866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207867, SGI207868, SGI207868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207869, SGI0, SGI0, NULL};
const Segment * const SG46140[] = {SGI207870, SGI0, SGI207871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207873, SGI207874, SGI207874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207876, SGI0, SGI0, NULL};
const Segment * const SG46141[] = {SGI207872, SGI0, SGI207875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207877, SGI207878, SGI207878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46142[] = {SGI207879, SGI0, SGI207880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207881, SGI207882, SGI207882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46143[] = {SGI207883, SGI0, SGI207884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207885, SGI207886, SGI207886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46144[] = {SGI207887, SGI207888, SGI207889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207890, SGI0, SGI0, SGI0, SGI207891, SGI207891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46145[] = {SGI207892, SGI0, SGI207893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207894, SGI207895, SGI207895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46146[] = {SGI207896, SGI0, SGI207897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207898, SGI207899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46147[] = {SGI207902, SGI0, SGI207903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207904, SGI207905, SGI207905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46148[] = {SGI207906, SGI0, SGI207907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207908, SGI207909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46149[] = {SGI207910, SGI0, SGI207911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207912, SGI207913, SGI207913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46150[] = {SGI207914, SGI0, SGI207915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207916, SGI207917, SGI207917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46151[] = {SGI207918, SGI0, SGI207919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207920, SGI207921, SGI207921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46152[] = {SGI207922, SGI207923, SGI207924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207925, SGI0, SGI0, SGI0, SGI207926, SGI207926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46153[] = {SGI207927, SGI207928, SGI207929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207930, SGI0, SGI0, SGI0, SGI207932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46154[] = {SGI207931, SGI207933, SGI207934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207935, SGI0, SGI0, SGI0, SGI207936, SGI207936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46155[] = {SGI207937, SGI207938, SGI207939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207940, SGI0, SGI0, SGI0, SGI207941, SGI207941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46156[] = {SGI207942, SGI207943, SGI207944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207945, SGI0, SGI0, SGI0, SGI207946, SGI207946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46157[] = {SGI207947, SGI0, SGI207948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207949, SGI207950, SGI207950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46158[] = {SGI207951, SGI207952, SGI207953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207954, SGI0, SGI0, SGI0, SGI207955, SGI207955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46159[] = {SGI207956, SGI0, SGI207957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46160[] = {SGI207958, SGI207959, SGI207960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207961, SGI0, SGI0, SGI0, SGI207962, SGI207962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46161[] = {SGI207964, SGI0, SGI207966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207968, SGI207970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46162[] = {SGI207963, SGI0, SGI207965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207967, SGI207969, SGI207969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46163[] = {SGI207971, SGI0, SGI207972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207973, SGI207974, SGI207974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46164[] = {SGI207975, SGI0, SGI207976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207977, SGI207978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46165[] = {SGI207979, SGI207980, SGI207981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207982, SGI0, SGI0, SGI0, SGI207983, SGI207983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46166[] = {SGI207985, SGI207986, SGI207987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207988, SGI0, SGI0, SGI0, SGI207989, SGI207989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46167[] = {SGI207990, SGI0, SGI207991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207992, SGI207993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46168[] = {SGI207994, SGI0, SGI207995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI207996, SGI207997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46169[] = {SGI207998, SGI0, SGI207999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208000, SGI208001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46170[] = {SGI207900, SGI0, SGI207901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208004, SGI208005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46171[] = {SGI208002, SGI0, SGI208003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208006, SGI208007, SGI208007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46172[] = {SGI208008, SGI0, SGI208009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208010, SGI208011, SGI208011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46173[] = {SGI208012, SGI0, SGI208013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208014, SGI208015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46174[] = {SGI208016, SGI0, SGI208017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208018, SGI208019, SGI208019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46175[] = {SGI208020, SGI208021, SGI208022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208023, SGI0, SGI0, SGI0, SGI208024, SGI208024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46176[] = {SGI208025, SGI0, SGI208026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208027, SGI208028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46177[] = {SGI208029, SGI0, SGI208030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208031, SGI208032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46178[] = {SGI208033, SGI0, SGI208034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208035, SGI208036, SGI208036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208037, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46179[] = {SGI208038, SGI0, SGI208039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208042, SGI208043, SGI208043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46180[] = {SGI208040, SGI0, SGI208041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208044, SGI208045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46181[] = {SGI208046, SGI0, SGI208047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208048, SGI208049, SGI208049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46182[] = {SGI208050, SGI0, SGI208051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208052, SGI208053, SGI208053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46183[] = {SGI208054, SGI0, SGI208055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208056, SGI0, SGI0, SGI0, SGI208057, SGI208057, SGI0, SGI0, SGI208058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46184[] = {SGI208059, SGI0, SGI208060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208061, SGI208062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46185[] = {SGI208063, SGI208064, SGI208065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208066, SGI0, SGI0, SGI0, SGI208067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208068, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46186[] = {SGI208069, SGI0, SGI208070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208071, SGI0, SGI0, SGI0, SGI208072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208073, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46187[] = {SGI208074, SGI0, SGI208075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208076, SGI208077, SGI208077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46188[] = {SGI208078, SGI208079, SGI208080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208081, SGI0, SGI0, SGI0, SGI208082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208083, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46189[] = {SGI208084, SGI0, SGI208085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208086, SGI208087, SGI208087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46190[] = {SGI208088, SGI208089, SGI208090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208091, SGI0, SGI0, SGI0, SGI208092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46191[] = {SGI208093, SGI208094, SGI208095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208096, SGI208096, SGI0, SGI0, SGI208097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46192[] = {SGI208098, SGI208099, SGI208100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208101, SGI0, SGI0, SGI0, SGI208102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46193[] = {SGI208103, SGI208104, SGI208105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208106, SGI0, SGI0, SGI0, SGI208107, SGI208107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46194[] = {SGI208108, SGI0, SGI208109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46195[] = {SGI208110, SGI208111, SGI208112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208113, SGI0, SGI0, SGI0, SGI208114, SGI208114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46196[] = {SGI208115, SGI0, SGI208117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208119, SGI208120, SGI208120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46197[] = {SGI208116, SGI0, SGI208118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208121, SGI208122, SGI208122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46198[] = {SGI208123, SGI0, SGI208124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208125, SGI208126, SGI208126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46199[] = {SGI208127, SGI208128, SGI208129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208130, SGI0, SGI0, SGI0, SGI208131, SGI208131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208133, SGI0, NULL};
const Segment * const SG46200[] = {SGI208134, SGI208135, SGI208136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208137, SGI0, SGI0, SGI0, SGI208138, SGI208138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46201[] = {SGI208140, SGI0, SGI208141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208144, SGI208145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46202[] = {SGI208142, SGI0, SGI208143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208146, SGI208147, SGI208147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46203[] = {SGI208148, SGI0, SGI208149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208150, SGI208151, SGI208151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46204[] = {SGI208152, SGI0, SGI208153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208154, SGI208155, SGI208155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46205[] = {SGI208156, SGI0, SGI208157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208158, SGI208159, SGI208159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46206[] = {SGI208160, SGI208161, SGI208162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208163, SGI0, SGI0, SGI0, SGI208164, SGI208164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46207[] = {SGI208166, SGI208167, SGI208168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208169, SGI0, SGI0, SGI0, SGI208170, SGI208170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208171, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46208[] = {SGI208172, SGI208173, SGI208174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208175, SGI0, SGI0, SGI0, SGI208176, SGI208176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208177, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46209[] = {SGI208178, SGI0, SGI208179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208180, SGI208181, SGI208181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46210[] = {SGI208182, SGI208183, SGI208184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208187, SGI0, SGI0, SGI0, SGI208188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46211[] = {SGI208185, SGI0, SGI208186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208189, SGI208190, SGI208190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46212[] = {SGI208191, SGI0, SGI208192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46213[] = {SGI208193, SGI0, SGI208195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208199, SGI208200, SGI208200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46214[] = {SGI208194, SGI208196, SGI208197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208198, SGI0, SGI0, SGI0, SGI208201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46215[] = {SGI208202, SGI0, SGI208203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208206, SGI208208, SGI208208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46216[] = {SGI208204, SGI208205, SGI208207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208209, SGI208209, SGI0, SGI0, SGI208210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46217[] = {SGI208211, SGI208212, SGI208213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208214, SGI0, SGI0, SGI0, SGI208215, SGI208215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46218[] = {SGI208216, SGI0, SGI208217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208220, SGI208221, SGI208221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46219[] = {SGI208218, SGI0, SGI208219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208222, SGI208223, SGI208223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46220[] = {SGI208224, SGI0, SGI208225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208226, SGI208227, SGI208227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46221[] = {SGI208228, SGI0, SGI208229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208230, SGI208231, SGI208231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46222[] = {SGI208232, SGI208233, SGI208234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208235, SGI0, SGI0, SGI0, SGI208236, SGI208236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46223[] = {SGI208237, SGI208238, SGI208239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208240, SGI0, SGI0, SGI0, SGI208241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46224[] = {SGI208242, SGI0, SGI208243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208244, SGI208245, SGI208245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46225[] = {SGI208246, SGI208247, SGI208248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208249, SGI0, SGI0, SGI0, SGI208250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46226[] = {SGI208251, SGI0, SGI208252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46227[] = {SGI208253, SGI0, SGI208254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208255, SGI208256, SGI208256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46228[] = {SGI208257, SGI208258, SGI208259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208260, SGI0, SGI0, SGI0, SGI208261, SGI208261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208262, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46229[] = {SGI208263, SGI208264, SGI208265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208266, SGI208266, SGI0, SGI0, SGI0, SGI208267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46230[] = {SGI208268, SGI208269, SGI208270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208271, SGI208271, SGI0, SGI0, SGI0, SGI208272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46231[] = {SGI208273, SGI0, SGI208274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208275, SGI208276, SGI208276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46232[] = {SGI208277, SGI0, SGI208278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46233[] = {SGI208279, SGI208280, SGI208281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208282, SGI0, SGI0, SGI0, SGI208283, SGI208283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46234[] = {SGI208284, SGI0, SGI208285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208286, SGI208287, SGI208287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46235[] = {SGI208288, SGI0, SGI208289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208290, SGI208291, SGI208291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46236[] = {SGI208292, SGI0, SGI208293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208294, SGI208295, SGI208295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46237[] = {SGI208296, SGI0, SGI208297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208298, SGI208299, SGI208299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46238[] = {SGI208300, SGI0, SGI208301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208302, SGI208303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46239[] = {SGI208304, SGI208305, SGI208306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208307, SGI0, SGI0, SGI0, SGI208308, SGI208308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46240[] = {SGI208309, SGI208310, SGI208311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208312, SGI0, SGI0, SGI0, SGI208313, SGI208313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46241[] = {SGI208314, SGI208315, SGI208316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208317, SGI0, SGI0, SGI0, SGI208318, SGI208318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46242[] = {SGI208319, SGI0, SGI208320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208321, SGI208322, SGI208322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46243[] = {SGI208323, SGI0, SGI208324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208325, SGI208326, SGI208326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46244[] = {SGI208327, SGI0, SGI208328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208329, SGI208330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46245[] = {SGI208331, SGI208332, SGI208333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208334, SGI0, SGI0, SGI0, SGI208335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46246[] = {SGI208336, SGI0, SGI208337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208340, SGI208341, SGI208341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46247[] = {SGI208338, SGI0, SGI208339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208342, SGI208343, SGI208343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46248[] = {SGI208344, SGI0, SGI208345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208346, SGI208347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46249[] = {SGI208348, SGI0, SGI208349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208350, SGI208351, SGI208351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46250[] = {SGI208352, SGI0, SGI208353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208354, SGI208355, SGI208355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46251[] = {SGI208356, SGI0, SGI208357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208358, SGI208359, SGI208359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46252[] = {SGI208360, SGI208361, SGI208362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208363, SGI0, SGI0, SGI0, SGI208364, SGI208364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208365, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46253[] = {SGI208366, SGI208367, SGI208368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208369, SGI0, SGI0, SGI0, SGI208370, SGI208370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46254[] = {SGI208371, SGI0, SGI208372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208373, SGI208374, SGI208374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46255[] = {SGI208375, SGI0, SGI208376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208377, SGI208378, SGI208378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46256[] = {SGI208379, SGI208380, SGI208381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208382, SGI208382, SGI0, SGI0, SGI208384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46257[] = {SGI208383, SGI208385, SGI208386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208387, SGI208387, SGI0, SGI0, SGI0, SGI208388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46258[] = {SGI208389, SGI0, SGI208390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208391, SGI208392, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46259[] = {SGI208393, SGI0, SGI208394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208395, SGI208396, SGI208396, SGI0, SGI0, NULL};
const Segment * const SG46260[] = {SGI208397, SGI0, SGI208398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208399, SGI208400, SGI208400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46261[] = {SGI208401, SGI0, SGI208402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208403, SGI208404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46262[] = {SGI208406, SGI208408, SGI208410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208412, SGI0, SGI0, SGI0, SGI208413, SGI208413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46263[] = {SGI208405, SGI208407, SGI208409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208411, SGI0, SGI0, SGI0, SGI208414, SGI208414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46264[] = {SGI208416, SGI0, SGI208417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46265[] = {SGI208418, SGI208419, SGI208420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208421, SGI0, SGI0, SGI0, SGI208422, SGI208422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46266[] = {SGI208424, SGI0, SGI208425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208426, SGI208427, SGI208427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208428, SGI0, SGI0, NULL};
const Segment * const SG46267[] = {SGI208429, SGI0, SGI208430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208431, SGI208432, SGI208432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46268[] = {SGI208433, SGI0, SGI208434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208435, SGI208436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46269[] = {SGI208437, SGI0, SGI208438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208439, SGI208440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46270[] = {SGI208441, SGI208442, SGI208443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208444, SGI0, SGI0, SGI0, SGI208445, SGI208445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46271[] = {SGI208446, SGI208447, SGI208448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208449, SGI0, SGI0, SGI0, SGI208450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46272[] = {SGI208452, SGI0, SGI208455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208456, SGI208458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46273[] = {SGI208451, SGI208453, SGI208454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208457, SGI208457, SGI0, SGI0, SGI0, SGI208459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46274[] = {SGI208460, SGI0, SGI208461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208462, SGI208463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46275[] = {SGI208464, SGI208465, SGI208466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208468, SGI0, SGI0, SGI0, SGI208470, SGI0, SGI208471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46276[] = {SGI208467, SGI0, SGI208469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208472, SGI208473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46277[] = {SGI208474, SGI0, SGI208475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208476, SGI208477, SGI208477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46278[] = {SGI208478, SGI0, SGI208479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208481, SGI208482, SGI208482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46279[] = {SGI208480, SGI0, SGI208483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208484, SGI208485, SGI208485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46280[] = {SGI208486, SGI0, SGI208487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208488, SGI208489, SGI208489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46281[] = {SGI0, SGI0, SGI208490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208491, SGI208492, SGI208492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46282[] = {SGI208493, SGI0, SGI208494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208495, SGI208496, SGI208496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46283[] = {SGI208497, SGI208498, SGI208499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208500, SGI0, SGI0, SGI0, SGI208501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46284[] = {SGI208502, SGI208504, SGI208505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208508, SGI0, SGI0, SGI0, SGI208511, SGI208511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46285[] = {SGI208503, SGI0, SGI208506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208510, SGI208512, SGI208512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46286[] = {SGI208507, SGI0, SGI208509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208513, SGI208514, SGI208514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46287[] = {SGI208515, SGI0, SGI208516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208519, SGI208520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46288[] = {SGI208517, SGI0, SGI208518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208521, SGI208522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46289[] = {SGI208523, SGI0, SGI208524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208525, SGI208526, SGI208526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46290[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46291[] = {SGI208528, SGI0, SGI208529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208530, SGI208531, SGI208531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46292[] = {SGI208532, SGI208533, SGI208534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208535, SGI0, SGI0, SGI0, SGI208536, SGI208536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46293[] = {SGI208537, SGI0, SGI208538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46294[] = {SGI208539, SGI208540, SGI208541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208542, SGI0, SGI0, SGI0, SGI208543, SGI208543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46295[] = {SGI208544, SGI208545, SGI208546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208547, SGI0, SGI0, SGI0, SGI208548, SGI208548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46296[] = {SGI208549, SGI0, SGI208550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208551, SGI208552, SGI208552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46297[] = {SGI208553, SGI0, SGI208554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208555, SGI0, SGI0, SGI0, SGI208556, SGI208556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46298[] = {SGI208557, SGI0, SGI208558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208559, SGI208560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46299[] = {SGI208561, SGI0, SGI208562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208563, SGI208564, SGI208564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46300[] = {SGI208565, SGI0, SGI208566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208569, SGI208571, SGI208571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46301[] = {SGI208567, SGI0, SGI208570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208573, SGI208575, SGI208575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46302[] = {SGI208568, SGI0, SGI208572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208574, SGI208576, SGI208576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46303[] = {SGI208577, SGI0, SGI208578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208579, SGI208580, SGI208580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46304[] = {SGI208581, SGI208582, SGI208583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208584, SGI0, SGI0, SGI0, SGI208585, SGI208585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46305[] = {SGI208586, SGI0, SGI208587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46306[] = {SGI208588, SGI0, SGI208589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208590, SGI208591, SGI208591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46307[] = {SGI208592, SGI208593, SGI208594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208595, SGI0, SGI0, SGI0, SGI208596, SGI208596, SGI0, SGI208598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46308[] = {SGI208597, SGI0, SGI208599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208600, SGI208601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46309[] = {SGI208602, SGI208603, SGI208604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208605, SGI0, SGI0, SGI0, SGI208607, SGI208607, SGI0, SGI208610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46310[] = {SGI208606, SGI208608, SGI208609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208611, SGI0, SGI0, SGI0, SGI208612, SGI208612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46311[] = {SGI208613, SGI0, SGI208614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208616, SGI208618, SGI208618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46312[] = {SGI208615, SGI208617, SGI208619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208620, SGI0, SGI0, SGI0, SGI208621, SGI208621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46313[] = {SGI208622, SGI0, SGI208623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208626, SGI208628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46314[] = {SGI208624, SGI208625, SGI208627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208629, SGI0, SGI0, SGI0, SGI208630, SGI208630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46315[] = {SGI208631, SGI208632, SGI208633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208634, SGI0, SGI0, SGI0, SGI208635, SGI208635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46316[] = {SGI208636, SGI208637, SGI208638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208639, SGI0, SGI0, SGI0, SGI208640, SGI208640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46317[] = {SGI208641, SGI208642, SGI208643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208644, SGI0, SGI0, SGI0, SGI208645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46318[] = {SGI208646, SGI0, SGI208647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46319[] = {SGI208648, SGI208649, SGI208650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208651, SGI0, SGI0, SGI0, SGI208652, SGI208652, SGI0, SGI208653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46320[] = {SGI208654, SGI0, SGI208655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208656, SGI208657, SGI208657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46321[] = {SGI208658, SGI0, SGI208659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208660, SGI208661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46322[] = {SGI208662, SGI0, SGI208663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208664, SGI208665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46323[] = {SGI208666, SGI0, SGI208667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208668, SGI208669, SGI208669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46324[] = {SGI208670, SGI208671, SGI208672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208673, SGI0, SGI0, SGI0, SGI208674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46325[] = {SGI208675, SGI208676, SGI208677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208678, SGI0, SGI0, SGI0, SGI208679, SGI208679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208680, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46326[] = {SGI208681, SGI0, SGI208682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208683, SGI208684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46327[] = {SGI208685, SGI0, SGI208686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208687, SGI208688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46328[] = {SGI208689, SGI0, SGI208690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208691, SGI208692, SGI208692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46329[] = {SGI208693, SGI0, SGI208694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208695, SGI208696, SGI208696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208697, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46330[] = {SGI208698, SGI208699, SGI208700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208701, SGI0, SGI0, SGI0, SGI208702, SGI208702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46331[] = {SGI208703, SGI0, SGI208704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46332[] = {SGI208705, SGI208706, SGI208707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208708, SGI0, SGI0, SGI0, SGI208709, SGI208709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46333[] = {SGI208710, SGI208711, SGI208712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208713, SGI0, SGI0, SGI0, SGI208714, SGI208714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46334[] = {SGI208715, SGI208716, SGI208717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208718, SGI0, SGI0, SGI0, SGI208721, SGI208721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46335[] = {SGI208719, SGI0, SGI208720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46336[] = {SGI208722, SGI0, SGI208723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46337[] = {SGI208724, SGI208725, SGI208726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208727, SGI0, SGI0, SGI0, SGI208728, SGI208728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46338[] = {SGI208729, SGI208730, SGI208731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208732, SGI0, SGI0, SGI0, SGI208733, SGI208733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46339[] = {SGI208734, SGI208735, SGI208736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208737, SGI0, SGI0, SGI0, SGI208738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46340[] = {SGI208739, SGI208740, SGI208741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208742, SGI0, SGI0, SGI0, SGI208743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46341[] = {SGI208744, SGI0, SGI208745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208746, SGI208747, SGI208747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46342[] = {SGI208748, SGI0, SGI208749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208750, SGI208751, SGI208751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46343[] = {SGI208752, SGI208753, SGI208754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208755, SGI0, SGI0, SGI0, SGI208756, SGI208756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46344[] = {SGI208758, SGI208759, SGI208760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208761, SGI0, SGI0, SGI0, SGI208762, SGI208762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46345[] = {SGI208764, SGI208765, SGI208766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208767, SGI0, SGI0, SGI0, SGI208768, SGI208768, SGI0, SGI208769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46346[] = {SGI208770, SGI0, SGI208771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208772, SGI208773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46347[] = {SGI208774, SGI208775, SGI208776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208777, SGI0, SGI0, SGI0, SGI208778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46348[] = {SGI208779, SGI0, SGI208780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208781, SGI0, SGI0, SGI0, SGI208782, SGI208782, SGI0, SGI0, SGI0, SGI208783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46349[] = {SGI208784, SGI208785, SGI208786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208787, SGI0, SGI0, SGI0, SGI208788, SGI208788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46350[] = {SGI208789, SGI208790, SGI208791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208792, SGI0, SGI0, SGI0, SGI208793, SGI208793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46351[] = {SGI208794, SGI0, SGI208795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208796, SGI208797, SGI208797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46352[] = {SGI208798, SGI0, SGI208799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208802, SGI208803, SGI208803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46353[] = {SGI208800, SGI0, SGI208801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208804, SGI208805, SGI208805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46354[] = {SGI208806, SGI0, SGI208807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208808, SGI208809, SGI208809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46355[] = {SGI208810, SGI0, SGI208811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208812, SGI208813, SGI208813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46356[] = {SGI208814, SGI208815, SGI208816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208817, SGI0, SGI0, SGI0, SGI208818, SGI208818, SGI0, SGI0, SGI208819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46357[] = {SGI208820, SGI0, SGI208821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208822, SGI208823, SGI208823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46358[] = {SGI208824, SGI0, SGI208825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208826, SGI0, SGI0, SGI0, SGI208827, SGI208827, SGI0, SGI0, SGI0, SGI208828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46359[] = {SGI208829, SGI0, SGI208830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208831, SGI0, SGI0, SGI0, SGI208832, SGI208832, SGI0, SGI0, SGI0, SGI208833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46360[] = {SGI208834, SGI208835, SGI208836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208837, SGI0, SGI0, SGI0, SGI208838, SGI208838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46361[] = {SGI208839, SGI0, SGI208840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208841, SGI208842, SGI208842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46362[] = {SGI208843, SGI0, SGI208844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208845, SGI208846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46363[] = {SGI208847, SGI208848, SGI208849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208850, SGI0, SGI0, SGI0, SGI208851, SGI208851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46364[] = {SGI208853, SGI208854, SGI208855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208856, SGI0, SGI0, SGI0, SGI208857, SGI208857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46365[] = {SGI208858, SGI0, SGI208859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208861, SGI208862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46366[] = {SGI208860, SGI0, SGI208863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208865, SGI208866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46367[] = {SGI208864, SGI0, SGI208867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208868, SGI208869, SGI208869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46368[] = {SGI208870, SGI0, SGI208871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208872, SGI0, SGI0, SGI0, SGI208873, SGI208873, SGI0, SGI0, SGI208874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46369[] = {SGI208875, SGI208876, SGI208877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208878, SGI0, SGI0, SGI0, SGI208879, SGI208880, SGI0, SGI0, SGI208881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46370[] = {SGI208882, SGI208883, SGI208884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208885, SGI0, SGI0, SGI0, SGI208886, SGI208886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46371[] = {SGI208887, SGI0, SGI208888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208889, SGI208890, SGI208890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46372[] = {SGI208891, SGI208892, SGI208893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208894, SGI0, SGI0, SGI0, SGI208895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46373[] = {SGI208896, SGI0, SGI208897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208898, SGI208899, SGI208899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46374[] = {SGI208900, SGI208901, SGI208902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208903, SGI208903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208904, NULL};
const Segment * const SG46375[] = {SGI208905, SGI0, SGI208906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208907, SGI208908, SGI208908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46376[] = {SGI208909, SGI0, SGI208910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208911, SGI208912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46377[] = {SGI208913, SGI0, SGI208914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208915, SGI208916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46378[] = {SGI208917, SGI0, SGI208918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208919, SGI208920, SGI208920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46379[] = {SGI208921, SGI0, SGI208922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208923, SGI208924, SGI208924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46380[] = {SGI208925, SGI0, SGI208926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208927, SGI0, SGI0, SGI0, SGI208928, SGI208928, SGI0, SGI0, SGI0, SGI0, SGI208929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46381[] = {SGI208930, SGI208931, SGI208932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208933, SGI0, SGI0, SGI0, SGI208935, SGI208935, SGI0, SGI208937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46382[] = {SGI208934, SGI0, SGI208936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208939, SGI208941, SGI208941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46383[] = {SGI208938, SGI0, SGI208940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208942, SGI0, SGI0, SGI0, SGI208943, SGI208943, SGI0, SGI0, SGI0, SGI0, SGI208944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46384[] = {SGI208945, SGI0, SGI208946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208947, SGI208948, SGI208948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46385[] = {SGI208949, SGI0, SGI208950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208951, SGI208952, SGI208952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46386[] = {SGI208953, SGI0, SGI208954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208955, SGI208956, SGI208956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46387[] = {SGI208957, SGI208958, SGI208959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208960, SGI208960, SGI0, SGI0, SGI0, SGI208961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46388[] = {SGI208962, SGI208963, SGI208964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208965, SGI208965, SGI0, SGI0, SGI0, SGI208966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46389[] = {SGI208967, SGI208968, SGI208969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208970, SGI0, SGI0, SGI0, SGI208971, SGI208971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46390[] = {SGI208972, SGI208973, SGI208974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208975, SGI208975, SGI0, SGI0, SGI0, SGI208976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46391[] = {SGI208977, SGI208978, SGI208979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208980, SGI208980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46392[] = {SGI208981, SGI0, SGI208982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208985, SGI208986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46393[] = {SGI0, SGI208983, SGI208984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208987, SGI208987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46394[] = {SGI208989, SGI0, SGI208991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46395[] = {SGI208988, SGI0, SGI208990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208992, SGI208993, SGI208993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46396[] = {SGI0, SGI208994, SGI208995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208998, SGI208998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46397[] = {SGI208996, SGI0, SGI208997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI208999, SGI209000, SGI209000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46398[] = {SGI209001, SGI0, SGI209002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209005, SGI209006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46399[] = {SGI0, SGI209003, SGI209004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209007, SGI209007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46400[] = {SGI209008, SGI0, SGI209009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209010, SGI209011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46401[] = {SGI209012, SGI209013, SGI209014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209015, SGI0, SGI0, SGI0, SGI209016, SGI209016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46402[] = {SGI209017, SGI0, SGI209018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209019, SGI209020, SGI209020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46403[] = {SGI209021, SGI209022, SGI209023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209024, SGI0, SGI0, SGI0, SGI209025, SGI209025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46404[] = {SGI209026, SGI0, SGI209027, SGI0, SGI209028, SGI209028, SGI0, NULL};
const Segment * const SG46405[] = {SGI209029, SGI209030, SGI209031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209032, SGI209032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209033, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46406[] = {SGI209034, SGI209035, SGI209036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209037, SGI209037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209038, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46407[] = {SGI209039, SGI0, SGI209040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209041, SGI209042, SGI209042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209043, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46408[] = {SGI209044, SGI209045, SGI209046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209047, SGI209047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209048, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46409[] = {SGI209049, SGI209050, SGI209051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209052, SGI209052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209053, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46410[] = {SGI209054, SGI209055, SGI209056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209057, SGI0, SGI0, SGI0, SGI209058, SGI209058, SGI0, SGI0, SGI0, SGI0, SGI209059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46411[] = {SGI209060, SGI209061, SGI209062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209063, SGI0, SGI0, SGI0, SGI209064, SGI209064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209065, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46412[] = {SGI209066, SGI0, SGI209067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209068, SGI209069, SGI209069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209070, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46413[] = {SGI209071, SGI209072, SGI209073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209074, SGI0, SGI0, SGI0, SGI209075, SGI209075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209076, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46414[] = {SGI209077, SGI0, SGI209078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209082, SGI209083, SGI209083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46415[] = {SGI209079, SGI209080, SGI209081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209084, SGI0, SGI0, SGI0, SGI209085, SGI209085, SGI0, SGI209086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46416[] = {SGI209087, SGI209088, SGI209089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209092, SGI0, SGI0, SGI0, SGI209093, SGI209093, SGI0, SGI209095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46417[] = {SGI209090, SGI0, SGI209091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209094, SGI0, SGI0, SGI0, SGI209096, SGI209096, SGI0, SGI0, SGI209097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46418[] = {SGI209098, SGI209099, SGI209100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209101, SGI0, SGI0, SGI0, SGI209102, SGI209102, SGI0, SGI209103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46419[] = {SGI209104, SGI209105, SGI209106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209107, SGI0, SGI0, SGI0, SGI209108, SGI209108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46420[] = {SGI209109, SGI209110, SGI209111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209112, SGI0, SGI0, SGI0, SGI209113, SGI0, SGI0, SGI0, SGI209114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46421[] = {SGI209115, SGI0, SGI209116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209117, SGI209118, SGI209118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46422[] = {SGI209119, SGI209120, SGI209121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209123, SGI0, SGI0, SGI0, SGI209125, SGI0, SGI0, SGI0, SGI0, SGI209126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46423[] = {SGI209122, SGI0, SGI209124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209129, SGI209130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46424[] = {SGI209127, SGI0, SGI209128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209131, SGI209132, SGI209132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46425[] = {SGI209133, SGI0, SGI209134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209135, SGI209136, SGI209136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46426[] = {SGI209137, SGI209138, SGI209139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209140, SGI0, SGI0, SGI0, SGI209141, SGI209141, SGI209142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46427[] = {SGI209143, SGI0, SGI209144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209145, SGI209146, SGI209146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46428[] = {SGI209147, SGI0, SGI209148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209152, SGI209153, SGI209153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46429[] = {SGI209149, SGI209150, SGI209151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209154, SGI0, SGI0, SGI0, SGI209155, SGI209155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46430[] = {SGI209156, SGI0, SGI209157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209158, SGI209159, SGI209159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46431[] = {SGI209160, SGI209161, SGI209162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209163, SGI0, SGI0, SGI0, SGI209164, SGI209164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46432[] = {SGI209166, SGI209167, SGI209168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209169, SGI0, SGI0, SGI0, SGI209170, SGI209170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46433[] = {SGI209172, SGI209173, SGI209174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209175, SGI0, SGI0, SGI0, SGI209176, SGI209176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46434[] = {SGI209178, SGI0, SGI209179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209180, SGI0, SGI0, SGI0, SGI209182, SGI209182, SGI0, SGI0, SGI0, SGI209184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46435[] = {SGI209181, SGI0, SGI209183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209185, SGI209186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46436[] = {SGI209187, SGI0, SGI209188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209189, SGI209190, SGI209190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46437[] = {SGI209191, SGI209192, SGI209193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209194, SGI0, SGI0, SGI0, SGI209195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209196, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46438[] = {SGI209197, SGI0, SGI209199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209202, SGI209203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46439[] = {SGI209198, SGI0, SGI209200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209201, SGI0, SGI0, SGI0, SGI209204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209205, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46440[] = {SGI209206, SGI209207, SGI209208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209209, SGI0, SGI0, SGI0, SGI209210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209211, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46441[] = {SGI209212, SGI0, SGI209213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209214, SGI209215, SGI209215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46442[] = {SGI209216, SGI0, SGI209217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209218, SGI209219, SGI209219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46443[] = {SGI209220, SGI0, SGI209221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209222, SGI209223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46444[] = {SGI209224, SGI209225, SGI209226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209227, SGI0, SGI0, SGI0, SGI209228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46445[] = {SGI209229, SGI0, SGI209230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209231, SGI209232, SGI209232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46446[] = {SGI209233, SGI209234, SGI209235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209237, SGI0, SGI0, SGI0, SGI209238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46447[] = {SGI0, SGI0, SGI209236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209239, SGI209240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46448[] = {SGI209241, SGI0, SGI209242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209243, SGI209244, SGI209244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46449[] = {SGI209245, SGI209246, SGI209247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209248, SGI0, SGI0, SGI0, SGI209249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46450[] = {SGI209250, SGI0, SGI209251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209252, SGI209253, SGI209253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46451[] = {SGI209254, SGI209255, SGI209256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209257, SGI0, SGI0, SGI0, SGI209258, SGI0, SGI0, SGI209259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46452[] = {SGI209260, SGI0, SGI209261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209262, SGI209263, SGI209263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46453[] = {SGI209264, SGI209265, SGI209266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209267, SGI0, SGI0, SGI0, SGI209268, SGI0, SGI0, SGI209270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46454[] = {SGI209269, SGI0, SGI209271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209272, SGI209273, SGI209273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46455[] = {SGI209274, SGI0, SGI209275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46456[] = {SGI209276, SGI0, SGI209277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209280, SGI209282, SGI209282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46457[] = {SGI209278, SGI209279, SGI209281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209284, SGI0, SGI0, SGI0, SGI209287, SGI209287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46458[] = {SGI209283, SGI209285, SGI209286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209288, SGI0, SGI0, SGI0, SGI209290, SGI209290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46459[] = {SGI209289, SGI209291, SGI209292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209293, SGI0, SGI0, SGI0, SGI209294, SGI209294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46460[] = {SGI209297, SGI209299, SGI209300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209302, SGI0, SGI0, SGI0, SGI209304, SGI209304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46461[] = {SGI209295, SGI209296, SGI209298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209301, SGI0, SGI0, SGI0, SGI209303, SGI209303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209305, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46462[] = {SGI209308, SGI209310, SGI209311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209312, SGI0, SGI0, SGI0, SGI209314, SGI209314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46463[] = {SGI209306, SGI209307, SGI209309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209313, SGI209313, SGI0, SGI0, SGI0, SGI209315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46464[] = {SGI209316, SGI0, SGI209317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209318, SGI209319, SGI209319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46465[] = {SGI209320, SGI0, SGI209321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209322, SGI209323, SGI209323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46466[] = {SGI209324, SGI0, SGI209325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209326, SGI209327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46467[] = {SGI209328, SGI209329, SGI209330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209331, SGI0, SGI0, SGI0, SGI209332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209333, SGI0, SGI0, NULL};
const Segment * const SG46468[] = {SGI209334, SGI209335, SGI209336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209337, SGI0, SGI0, SGI0, SGI209338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46469[] = {SGI209339, SGI209340, SGI209341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209342, SGI0, SGI0, SGI0, SGI209343, SGI0, SGI0, SGI0, SGI209344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46470[] = {SGI209345, SGI0, SGI209346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209347, SGI209348, SGI209348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46471[] = {SGI209349, SGI209350, SGI209351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209352, SGI209352, SGI0, SGI0, SGI0, SGI209353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46472[] = {SGI209354, SGI209355, SGI209356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209357, SGI0, SGI0, SGI0, SGI209358, SGI209358, SGI0, SGI0, SGI0, SGI209359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46473[] = {SGI209360, SGI0, SGI209361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209362, SGI209363, SGI209363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46474[] = {SGI209364, SGI0, SGI209365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209366, SGI209367, SGI209367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46475[] = {SGI209368, SGI209369, SGI209370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209371, SGI209371, SGI0, SGI0, SGI0, SGI209372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46476[] = {SGI209373, SGI0, SGI209374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209375, SGI209376, SGI209376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46477[] = {SGI209377, SGI209378, SGI209379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209381, SGI0, SGI0, SGI0, SGI209383, SGI0, SGI0, SGI209384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46478[] = {SGI209380, SGI0, SGI209382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209385, SGI209386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46479[] = {SGI209387, SGI0, SGI209388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209389, SGI209390, SGI209390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46480[] = {SGI209391, SGI0, SGI209392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209395, SGI0, SGI0, SGI0, SGI209396, SGI0, SGI0, SGI209397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46481[] = {SGI209393, SGI0, SGI209394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209398, SGI209399, SGI209399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46482[] = {SGI209400, SGI0, SGI209401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209402, SGI209403, SGI209403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46483[] = {SGI209404, SGI0, SGI209405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209406, SGI209407, SGI209407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46484[] = {SGI209408, SGI0, SGI209409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209412, SGI209414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46485[] = {SGI209410, SGI209411, SGI209413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209415, SGI0, SGI0, SGI0, SGI209416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46486[] = {SGI209417, SGI209418, SGI209419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209420, SGI0, SGI0, SGI0, SGI209422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46487[] = {SGI209424, SGI209425, SGI209426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209427, SGI0, SGI0, SGI0, SGI209428, SGI209428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46488[] = {SGI209429, SGI0, SGI209430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209431, SGI209432, SGI209432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46489[] = {SGI209433, SGI0, SGI209434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209435, SGI209436, SGI209436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46490[] = {SGI209437, SGI0, SGI209438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209439, SGI209440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46491[] = {SGI209441, SGI0, SGI209442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209443, SGI209444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46492[] = {SGI209445, SGI209446, SGI209447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209448, SGI0, SGI0, SGI0, SGI209449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46493[] = {SGI209450, SGI0, SGI209451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209452, SGI209453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46494[] = {SGI209454, SGI209455, SGI209456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209457, SGI0, SGI0, SGI0, SGI209458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46495[] = {SGI209459, SGI209460, SGI209461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209462, SGI0, SGI0, SGI0, SGI209463, SGI209463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46496[] = {SGI209464, SGI209465, SGI209466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209467, SGI0, SGI0, SGI0, SGI209468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46497[] = {SGI209469, SGI0, SGI209470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209471, SGI0, SGI0, SGI0, SGI209472, SGI209472, SGI0, SGI0, SGI0, SGI0, SGI209473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46498[] = {SGI209474, SGI0, SGI209475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209476, SGI209477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46499[] = {SGI209478, SGI0, SGI209479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209480, SGI209481, SGI209481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46500[] = {SGI209482, SGI0, SGI209483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209486, SGI209487, SGI209487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46501[] = {SGI209484, SGI0, SGI209485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209488, SGI209489, SGI209489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46502[] = {SGI209490, SGI0, SGI209491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209492, SGI209493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46503[] = {SGI209494, SGI0, SGI209495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209496, SGI209497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46504[] = {SGI209498, SGI0, SGI209499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209500, SGI209501, SGI209501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46505[] = {SGI209502, SGI0, SGI209503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209504, SGI209505, SGI209505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46506[] = {SGI209506, SGI0, SGI209507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209508, SGI209509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46507[] = {SGI209510, SGI0, SGI209511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209512, SGI209514, SGI209514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46508[] = {SGI209513, SGI0, SGI209515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209516, SGI0, SGI0, SGI0, SGI209517, SGI209517, SGI0, SGI0, SGI0, SGI209518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46509[] = {SGI209519, SGI0, SGI209520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209521, SGI209522, SGI209522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46510[] = {SGI209523, SGI209524, SGI209525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209526, SGI0, SGI0, SGI0, SGI209527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46511[] = {SGI209528, SGI0, SGI209529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209530, SGI209531, SGI209531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46512[] = {SGI209532, SGI209533, SGI209534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209535, SGI0, SGI0, SGI0, SGI209536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46513[] = {SGI209537, SGI0, SGI209538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209541, SGI209542, SGI209542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46514[] = {SGI209539, SGI0, SGI209540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46515[] = {SGI209543, SGI209544, SGI209545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209546, SGI209546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46516[] = {SGI209547, SGI209548, SGI209549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209550, SGI0, SGI0, SGI0, SGI209551, SGI209551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209552, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46517[] = {SGI209553, SGI209554, SGI209555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209556, SGI0, SGI0, SGI0, SGI209557, SGI209557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209558, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46518[] = {SGI209559, SGI209560, SGI209561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209562, SGI0, SGI0, SGI0, SGI209563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46519[] = {SGI209564, SGI209565, SGI209566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209569, SGI0, SGI0, SGI0, SGI209570, SGI209570, SGI209571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46520[] = {SGI209567, SGI0, SGI209568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209572, SGI209573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46521[] = {SGI209574, SGI0, SGI209575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209576, SGI209577, SGI209577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46522[] = {SGI209578, SGI0, SGI209579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209580, SGI209581, SGI209581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46523[] = {SGI209582, SGI0, SGI209583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209584, SGI209585, SGI209585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46524[] = {SGI209586, SGI0, SGI209587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209591, SGI209592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46525[] = {SGI209588, SGI209589, SGI209590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209593, SGI0, SGI0, SGI0, SGI209594, SGI209594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46526[] = {SGI209595, SGI0, SGI209596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209597, SGI209598, SGI209598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209599, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46527[] = {SGI209600, SGI0, SGI209601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209602, SGI209603, SGI209603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209604, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46528[] = {SGI209605, SGI0, SGI209606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209607, SGI209608, SGI209608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46529[] = {SGI209609, SGI0, SGI209610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209611, SGI209612, SGI209612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46530[] = {SGI209421, SGI0, SGI209423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209613, SGI209614, SGI209614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46531[] = {SGI209615, SGI209616, SGI209617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209618, SGI209618, SGI0, SGI0, SGI0, SGI209619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46532[] = {SGI209620, SGI209621, SGI209622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209623, SGI0, SGI0, SGI0, SGI209625, SGI209625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46533[] = {SGI209624, SGI0, SGI209626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209627, SGI209628, SGI209628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46534[] = {SGI209629, SGI0, SGI209630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209631, SGI209632, SGI209632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46535[] = {SGI209633, SGI0, SGI209634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209635, SGI209636, SGI209636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46536[] = {SGI209637, SGI0, SGI209638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209639, SGI209640, SGI209640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46537[] = {SGI209641, SGI209642, SGI209643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209644, SGI0, SGI0, SGI0, SGI209645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46538[] = {SGI209646, SGI0, SGI209647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209648, SGI209649, SGI209649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46539[] = {SGI209650, SGI209651, SGI209652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209653, SGI209653, SGI0, SGI0, SGI209654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46540[] = {SGI209655, SGI0, SGI209656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209657, SGI209658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46541[] = {SGI209659, SGI209660, SGI209661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209662, SGI0, SGI0, SGI0, SGI209663, SGI209663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46542[] = {SGI209664, SGI209665, SGI209666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209667, SGI0, SGI0, SGI0, SGI209668, SGI209668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46543[] = {SGI209669, SGI209670, SGI209671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209672, SGI0, SGI0, SGI0, SGI209673, SGI209673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46544[] = {SGI209674, SGI209675, SGI209676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209677, SGI0, SGI0, SGI0, SGI209678, SGI209678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46545[] = {SGI209680, SGI209681, SGI209682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209683, SGI0, SGI0, SGI0, SGI209684, SGI209684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46546[] = {SGI209686, SGI209687, SGI209688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209689, SGI0, SGI0, SGI0, SGI209690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46547[] = {SGI209691, SGI209692, SGI209693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209694, SGI0, SGI0, SGI0, SGI209695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46548[] = {SGI209696, SGI0, SGI209697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209698, SGI209699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46549[] = {SGI209700, SGI0, SGI209701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209702, SGI209703, SGI209703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46550[] = {SGI209704, SGI0, SGI209705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209706, SGI209707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46551[] = {SGI209708, SGI0, SGI209709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209710, SGI209711, SGI209711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46552[] = {SGI209712, SGI0, SGI209713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209714, SGI209715, SGI209715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46553[] = {SGI209716, SGI0, SGI209717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209718, SGI209719, SGI209719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46554[] = {SGI209720, SGI0, SGI209721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209722, SGI0, SGI0, SGI0, SGI209723, SGI209723, SGI0, SGI0, SGI0, SGI209724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46555[] = {SGI209725, SGI0, SGI209726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209727, SGI209728, SGI209728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46556[] = {SGI209729, SGI209730, SGI209731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209732, SGI0, SGI0, SGI0, SGI209733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46557[] = {SGI209734, SGI209735, SGI209736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209737, SGI0, SGI0, SGI0, SGI209738, SGI209738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46558[] = {SGI209739, SGI0, SGI209740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209741, SGI209742, SGI209742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46559[] = {SGI209743, SGI0, SGI209744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209745, SGI209746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46560[] = {SGI209747, SGI0, SGI209748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209749, SGI209750, SGI209750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46561[] = {SGI209751, SGI209752, SGI209753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209754, SGI0, SGI0, SGI0, SGI209755, SGI209755, SGI0, SGI209756, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46562[] = {SGI209757, SGI209758, SGI209759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209760, SGI0, SGI0, SGI0, SGI209761, SGI209761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46563[] = {SGI209762, SGI209763, SGI209764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209765, SGI0, SGI0, SGI0, SGI209766, SGI209766, SGI209767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46564[] = {SGI209768, SGI209769, SGI209770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209771, SGI0, SGI0, SGI0, SGI209772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46565[] = {SGI209773, SGI0, SGI209774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209775, SGI209776, SGI209776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46566[] = {SGI209777, SGI209778, SGI209779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209780, SGI0, SGI0, SGI0, SGI209781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46567[] = {SGI209782, SGI0, SGI209783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209784, SGI209785, SGI209785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46568[] = {SGI209786, SGI0, SGI209787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209790, SGI209791, SGI209791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46569[] = {SGI209788, SGI0, SGI209789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209792, SGI209793, SGI209793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209794, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46570[] = {SGI209795, SGI0, SGI209796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209797, SGI209798, SGI209798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46571[] = {SGI209799, SGI209800, SGI209801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209802, SGI209802, SGI0, SGI0, SGI0, SGI209803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46572[] = {SGI209804, SGI0, SGI209805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209806, SGI209807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46573[] = {SGI209808, SGI0, SGI209809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209810, SGI209811, SGI209811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46574[] = {SGI209812, SGI0, SGI209813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209814, SGI209815, SGI209815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46575[] = {SGI209816, SGI0, SGI209817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209818, SGI209819, SGI209819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46576[] = {SGI209820, SGI0, SGI209821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209822, SGI209823, SGI209823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46577[] = {SGI209824, SGI209825, SGI209826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209827, SGI0, SGI0, SGI0, SGI209828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46578[] = {SGI209830, SGI209831, SGI209832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209833, SGI0, SGI0, SGI0, SGI209834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46579[] = {SGI209836, SGI0, SGI209837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209838, SGI209839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46580[] = {SGI209840, SGI0, SGI209841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209842, SGI209843, SGI209843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46581[] = {SGI209844, SGI0, SGI209845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209846, SGI209847, SGI209847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46582[] = {SGI209848, SGI0, SGI209849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209850, SGI209851, SGI209851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46583[] = {SGI209852, SGI0, SGI209853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209854, SGI209855, SGI209855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46584[] = {SGI209856, SGI209857, SGI209859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209863, SGI0, SGI0, SGI0, SGI209866, SGI209866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46585[] = {SGI209858, SGI209860, SGI209861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209865, SGI0, SGI0, SGI0, SGI209867, SGI209867, SGI0, SGI209868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46586[] = {SGI209862, SGI0, SGI209864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209869, SGI209870, SGI209870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46587[] = {SGI209871, SGI209872, SGI209873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209877, SGI209877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46588[] = {SGI209874, SGI209875, SGI209876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209878, SGI0, SGI0, SGI0, SGI209879, SGI209879, SGI0, SGI209880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46589[] = {SGI209881, SGI209882, SGI209883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209887, SGI0, SGI0, SGI0, SGI209888, SGI209888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209891, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46590[] = {SGI209884, SGI209885, SGI209886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209889, SGI0, SGI0, SGI0, SGI209890, SGI209890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46591[] = {SGI209892, SGI209893, SGI209894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209895, SGI0, SGI0, SGI0, SGI209896, SGI209896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209897, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46592[] = {SGI209898, SGI0, SGI209899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209900, SGI209901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46593[] = {SGI209902, SGI0, SGI209903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209905, SGI209907, SGI209907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46594[] = {SGI209904, SGI0, SGI209906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209908, SGI209909, SGI209909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46595[] = {SGI209910, SGI0, SGI209911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209912, SGI209913, SGI209913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46596[] = {SGI209914, SGI209915, SGI209916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209917, SGI0, SGI0, SGI0, SGI209918, SGI209918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46597[] = {SGI209919, SGI0, SGI209920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209923, SGI209924, SGI209924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46598[] = {SGI209921, SGI0, SGI209922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209925, SGI209926, SGI209926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46599[] = {SGI209927, SGI0, SGI209929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209932, SGI209934, SGI209934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46600[] = {SGI209928, SGI209930, SGI209931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209933, SGI0, SGI0, SGI0, SGI209935, SGI209935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46601[] = {SGI209937, SGI209938, SGI209939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209940, SGI0, SGI0, SGI0, SGI209941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46602[] = {SGI209943, SGI209944, SGI209945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209946, SGI0, SGI0, SGI0, SGI209947, SGI209947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209950, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46603[] = {SGI209948, SGI0, SGI209949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209951, SGI209952, SGI209952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46604[] = {SGI209953, SGI209954, SGI209955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209956, SGI0, SGI0, SGI0, SGI209957, SGI209957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46605[] = {SGI209958, SGI0, SGI209959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209960, SGI209961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46606[] = {SGI209962, SGI209963, SGI209964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209965, SGI0, SGI0, SGI0, SGI209966, SGI209966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46607[] = {SGI209967, SGI0, SGI209968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209971, SGI209972, SGI209972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46608[] = {SGI209969, SGI0, SGI209970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209973, SGI209974, SGI209974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46609[] = {SGI209975, SGI0, SGI209976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209977, SGI209978, SGI209978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46610[] = {SGI209979, SGI0, SGI209980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209984, SGI209985, SGI209985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46611[] = {SGI209981, SGI209982, SGI209983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209986, SGI0, SGI0, SGI0, SGI209987, SGI209987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46612[] = {SGI209988, SGI0, SGI209989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209990, SGI209991, SGI209991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209992, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46613[] = {SGI209993, SGI0, SGI209994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI209995, SGI209996, SGI209996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46614[] = {SGI209997, SGI0, SGI209998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210001, SGI210002, SGI210002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210005, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46615[] = {SGI209999, SGI0, SGI210000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210003, SGI210004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46616[] = {SGI210006, SGI0, SGI210007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210011, SGI210012, SGI210012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46617[] = {SGI210008, SGI210009, SGI210010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210013, SGI0, SGI0, SGI0, SGI210014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46618[] = {SGI210015, SGI0, SGI210016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210017, SGI210018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46619[] = {SGI210019, SGI0, SGI210020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46620[] = {SGI210021, SGI0, SGI210022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210023, SGI210024, SGI210024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210025, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46621[] = {SGI210026, SGI0, SGI210027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210028, SGI210029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46622[] = {SGI210030, SGI0, SGI210031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210032, SGI210033, SGI210033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46623[] = {SGI210034, SGI0, SGI210035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210036, SGI210037, SGI210037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46624[] = {SGI210038, SGI0, SGI210039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210040, SGI210041, SGI210041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46625[] = {SGI210042, SGI0, SGI210043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210044, SGI210045, SGI210045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46626[] = {SGI210046, SGI0, SGI210047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210048, SGI210049, SGI210049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46627[] = {SGI210050, SGI0, SGI210051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210052, SGI210053, SGI210053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46628[] = {SGI210054, SGI0, SGI210055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210056, SGI210057, SGI210057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46629[] = {SGI210058, SGI0, SGI210059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210060, SGI210061, SGI210061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46630[] = {SGI210062, SGI210063, SGI210064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210065, SGI0, SGI210066, SGI210066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46631[] = {SGI210067, SGI210068, SGI210069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210070, SGI0, SGI210071, SGI210071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46632[] = {SGI210072, SGI210073, SGI210074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210075, SGI0, SGI210076, SGI210076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46633[] = {SGI210077, SGI0, SGI210078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210081, SGI210083, SGI210083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46634[] = {SGI210079, SGI210080, SGI210082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210084, SGI0, SGI210085, SGI210085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46635[] = {SGI210086, SGI210087, SGI210088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210089, SGI0, SGI210090, SGI210090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46636[] = {SGI210091, SGI0, SGI210092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210093, SGI210094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46637[] = {SGI210095, SGI210096, SGI210097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210098, SGI0, SGI0, SGI0, SGI210099, SGI210099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46638[] = {SGI210100, SGI0, SGI210101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210102, SGI210103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46639[] = {SGI210104, SGI0, SGI210105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210106, SGI210107, SGI210107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46640[] = {SGI210108, SGI0, SGI210109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210110, SGI210111, SGI210111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46641[] = {SGI210112, SGI0, SGI210113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210114, SGI210115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46642[] = {SGI210116, SGI0, SGI210117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210118, SGI210119, SGI210119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46643[] = {SGI210120, SGI210121, SGI210122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210124, SGI0, SGI0, SGI0, SGI210127, SGI210127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46644[] = {SGI210123, SGI210125, SGI210126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210128, SGI0, SGI0, SGI0, SGI210129, SGI210129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46645[] = {SGI210131, SGI210132, SGI210133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210134, SGI0, SGI0, SGI0, SGI210135, SGI210135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46646[] = {SGI210137, SGI0, SGI210138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210139, SGI210140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46647[] = {SGI210141, SGI0, SGI210142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210143, SGI210144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46648[] = {SGI210145, SGI210146, SGI210147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210148, SGI0, SGI0, SGI0, SGI210149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46649[] = {SGI210150, SGI210151, SGI210152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210153, SGI0, SGI0, SGI0, SGI210154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46650[] = {SGI210156, SGI0, SGI210158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210160, SGI210161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46651[] = {SGI210155, SGI0, SGI210157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210159, SGI0, SGI0, SGI0, SGI210162, SGI210162, SGI0, SGI0, SGI0, SGI210163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46652[] = {SGI210164, SGI0, SGI210165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210166, SGI210167, SGI210167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46653[] = {SGI210168, SGI210169, SGI210170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210171, SGI210171, SGI0, SGI0, SGI0, SGI0, SGI210172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46654[] = {SGI210173, SGI0, SGI210174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46655[] = {SGI210175, SGI0, SGI210176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210177, SGI210178, SGI210178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46656[] = {SGI210179, SGI210180, SGI210182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210184, SGI0, SGI0, SGI0, SGI210185, SGI210185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46657[] = {SGI210181, SGI0, SGI210183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210186, SGI210187, SGI210187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46658[] = {SGI210188, SGI0, SGI210189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210190, SGI210191, SGI210191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46659[] = {SGI210192, SGI210193, SGI210194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210195, SGI0, SGI0, SGI0, SGI210196, SGI210196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46660[] = {SGI210197, SGI210198, SGI210199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210200, SGI0, SGI0, SGI0, SGI210201, SGI210201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46661[] = {SGI210202, SGI210203, SGI210204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210205, SGI0, SGI0, SGI0, SGI210206, SGI210206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46662[] = {SGI210207, SGI210208, SGI210209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210210, SGI0, SGI0, SGI0, SGI210211, SGI210211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46663[] = {SGI210212, SGI0, SGI210213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210214, SGI210215, SGI210215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46664[] = {SGI210216, SGI210217, SGI210218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210219, SGI210219, SGI0, SGI0, SGI0, SGI0, SGI210220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46665[] = {SGI210221, SGI210222, SGI210223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210224, SGI210224, SGI0, SGI0, SGI0, SGI0, SGI210225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46666[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210226, SGI0, SGI0, SGI210227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46667[] = {SGI210228, SGI0, SGI210229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210230, SGI210231, SGI210231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46668[] = {SGI210232, SGI0, SGI210233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210234, SGI210235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46669[] = {SGI210236, SGI0, SGI210237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210238, SGI210239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46670[] = {SGI210240, SGI210241, SGI210242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210243, SGI0, SGI0, SGI0, SGI210244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46671[] = {SGI210245, SGI210246, SGI210247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210248, SGI0, SGI0, SGI0, SGI210249, SGI210249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46672[] = {SGI210250, SGI0, SGI210251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210252, SGI210253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46673[] = {SGI210254, SGI210255, SGI210256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210257, SGI0, SGI0, SGI0, SGI210258, SGI210258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46674[] = {SGI210259, SGI0, SGI210260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210261, SGI210262, SGI210262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46675[] = {SGI210263, SGI0, SGI210264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210265, SGI210266, SGI210266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46676[] = {SGI210267, SGI210268, SGI210269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210270, SGI210270, SGI0, SGI0, SGI0, SGI210271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46677[] = {SGI210272, SGI210273, SGI210274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210275, SGI210275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210276, NULL};
const Segment * const SG46678[] = {SGI210277, SGI0, SGI210278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210279, SGI210280, SGI210280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46679[] = {SGI210281, SGI210282, SGI210283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210286, SGI0, SGI0, SGI0, SGI210290, SGI210290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46680[] = {SGI210284, SGI210287, SGI210289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210292, SGI0, SGI0, SGI0, SGI210294, SGI0, SGI0, SGI210296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46681[] = {SGI210285, SGI210288, SGI210291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210293, SGI0, SGI0, SGI0, SGI210295, SGI210295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46682[] = {SGI210297, SGI210298, SGI210299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210300, SGI0, SGI0, SGI0, SGI210301, SGI0, SGI0, SGI210302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46683[] = {SGI210303, SGI0, SGI210304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210305, SGI210306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210307, SGI0, SGI0, NULL};
const Segment * const SG46684[] = {SGI210308, SGI0, SGI210309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210310, SGI210311, SGI210311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210314, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46685[] = {SGI210312, SGI0, SGI210313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210315, SGI210316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210317, SGI0, SGI0, NULL};
const Segment * const SG46686[] = {SGI210318, SGI0, SGI210319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210320, SGI210321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46687[] = {SGI210322, SGI0, SGI210323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46688[] = {SGI210324, SGI210325, SGI210326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210327, SGI0, SGI0, SGI0, SGI210328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46689[] = {SGI210329, SGI210330, SGI210331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210332, SGI0, SGI0, SGI0, SGI210333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46690[] = {SGI210334, SGI0, SGI210335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210336, SGI210337, SGI210337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46691[] = {SGI210338, SGI0, SGI210340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210342, SGI210343, SGI210343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46692[] = {SGI210339, SGI0, SGI210341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210344, SGI210345, SGI210345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46693[] = {SGI210346, SGI0, SGI210347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210348, SGI210349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46694[] = {SGI210350, SGI0, SGI210351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210352, SGI210353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46695[] = {SGI210354, SGI210355, SGI210356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210357, SGI0, SGI0, SGI0, SGI210359, SGI210359, SGI0, SGI0, SGI0, SGI0, SGI210361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46696[] = {SGI210358, SGI0, SGI210360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210362, SGI210363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46697[] = {SGI210364, SGI210366, SGI210367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210370, SGI0, SGI0, SGI0, SGI210372, SGI210372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46698[] = {SGI210365, SGI210368, SGI210369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210371, SGI0, SGI0, SGI0, SGI210373, SGI210373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46699[] = {SGI210374, SGI0, SGI210375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210376, SGI210377, SGI210377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46700[] = {SGI210378, SGI210379, SGI210380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210381, SGI0, SGI0, SGI0, SGI210382, SGI210382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46701[] = {SGI210383, SGI210384, SGI210385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210386, SGI0, SGI0, SGI0, SGI210387, SGI210387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46702[] = {SGI210388, SGI210389, SGI210390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210391, SGI0, SGI0, SGI0, SGI210392, SGI210392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210394, SGI0, NULL};
const Segment * const SG46703[] = {SGI210395, SGI0, SGI210396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210397, SGI210398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46704[] = {SGI210399, SGI210400, SGI210401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210402, SGI0, SGI0, SGI0, SGI210403, SGI210403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210404, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46705[] = {SGI210405, SGI0, SGI210406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210407, SGI210408, SGI210408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46706[] = {SGI210409, SGI210410, SGI210411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210412, SGI0, SGI0, SGI0, SGI210413, SGI210413, SGI0, SGI210414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46707[] = {SGI210415, SGI210416, SGI210417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210418, SGI0, SGI0, SGI0, SGI210419, SGI210419, SGI0, SGI210420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46708[] = {SGI210421, SGI210422, SGI210423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210424, SGI0, SGI0, SGI0, SGI210425, SGI210425, SGI0, SGI210426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46709[] = {SGI210427, SGI0, SGI210428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210429, SGI0, SGI0, SGI0, SGI210430, SGI210430, SGI0, SGI0, SGI0, SGI210431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46710[] = {SGI210432, SGI210433, SGI210434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210435, SGI0, SGI0, SGI0, SGI210436, SGI210436, SGI0, SGI210437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46711[] = {SGI210438, SGI0, SGI210439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210440, SGI210441, SGI210441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46712[] = {SGI210442, SGI0, SGI210443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210444, SGI210445, SGI210445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46713[] = {SGI210446, SGI0, SGI210447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210448, SGI210449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46714[] = {SGI210450, SGI0, SGI210451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210452, SGI210453, SGI210453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46715[] = {SGI0, SGI0, SGI210454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210455, SGI210456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46716[] = {SGI210457, SGI0, SGI210458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210459, SGI210460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46717[] = {SGI210461, SGI210462, SGI210463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210464, SGI0, SGI0, SGI0, SGI210465, SGI210465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46718[] = {SGI210466, SGI210467, SGI210468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210469, SGI0, SGI0, SGI0, SGI210470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46719[] = {SGI210471, SGI0, SGI210472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210473, SGI210474, SGI210474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46720[] = {SGI210475, SGI0, SGI210476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210477, SGI210478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46721[] = {SGI210480, SGI0, SGI210482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46722[] = {SGI210479, SGI0, SGI210481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210483, SGI210484, SGI210484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46723[] = {SGI210485, SGI0, SGI210486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210487, SGI210488, SGI210488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46724[] = {SGI210489, SGI0, SGI210490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210491, SGI210492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46725[] = {SGI210493, SGI0, SGI210494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210495, SGI210496, SGI210496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46726[] = {SGI210497, SGI0, SGI210498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210499, SGI210500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46727[] = {SGI210501, SGI210502, SGI210503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210504, SGI0, SGI0, SGI0, SGI210505, SGI210505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210506, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46728[] = {SGI210507, SGI0, SGI210508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210509, SGI210510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46729[] = {SGI210511, SGI210512, SGI210513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210514, SGI0, SGI0, SGI0, SGI210515, SGI210515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46730[] = {SGI210516, SGI0, SGI210517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210518, SGI210519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46731[] = {SGI210520, SGI0, SGI210521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210522, SGI210523, SGI210523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46732[] = {SGI210525, SGI0, SGI210527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210529, SGI210531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46733[] = {SGI210524, SGI0, SGI210526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210528, SGI210530, SGI210530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46734[] = {SGI210532, SGI0, SGI210534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210536, SGI210537, SGI210537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46735[] = {SGI210533, SGI0, SGI210535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210538, SGI210539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46736[] = {SGI210540, SGI0, SGI210541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210542, SGI210543, SGI210543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46737[] = {SGI210544, SGI0, SGI210545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210546, SGI210547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46738[] = {SGI210548, SGI0, SGI210549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210550, SGI210551, SGI210551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46739[] = {SGI210552, SGI0, SGI210553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210554, SGI210555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46740[] = {SGI210556, SGI0, SGI210557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210558, SGI210559, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46741[] = {SGI210560, SGI0, SGI210561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210562, SGI210563, SGI210563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46742[] = {SGI210564, SGI0, SGI210565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210566, SGI210567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46743[] = {SGI210568, SGI210569, SGI210570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210572, SGI0, SGI0, SGI0, SGI210574, SGI210574, SGI0, SGI210575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46744[] = {SGI210571, SGI0, SGI210573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210576, SGI210577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46745[] = {SGI210578, SGI210579, SGI210580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210583, SGI0, SGI0, SGI0, SGI210584, SGI210584, SGI0, SGI210585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46746[] = {SGI210581, SGI0, SGI210582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210586, SGI210587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46747[] = {SGI210588, SGI210589, SGI210590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210594, SGI0, SGI0, SGI0, SGI210595, SGI210595, SGI0, SGI210596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46748[] = {SGI210591, SGI210592, SGI210593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210597, SGI210597, SGI0, SGI0, SGI0, SGI0, SGI210598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46749[] = {SGI210599, SGI210600, SGI210601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210602, SGI0, SGI0, SGI0, SGI210603, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46750[] = {SGI210604, SGI210605, SGI210606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210607, SGI0, SGI0, SGI0, SGI210608, SGI210608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46751[] = {SGI210609, SGI210610, SGI210611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210612, SGI0, SGI0, SGI0, SGI210613, SGI210613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46752[] = {SGI210614, SGI210615, SGI210616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210619, SGI0, SGI0, SGI0, SGI210620, SGI210620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46753[] = {SGI210617, SGI0, SGI210618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210621, SGI210622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46754[] = {SGI210623, SGI210624, SGI210625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210626, SGI0, SGI0, SGI0, SGI210627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46755[] = {SGI210628, SGI210629, SGI210630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210631, SGI0, SGI0, SGI0, SGI210632, SGI210632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46756[] = {SGI210633, SGI210634, SGI210635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210636, SGI0, SGI0, SGI0, SGI210637, SGI210637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210638, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46757[] = {SGI210639, SGI0, SGI210640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210641, SGI210642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46758[] = {SGI210643, SGI0, SGI210644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210645, SGI210646, SGI210646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46759[] = {SGI210647, SGI0, SGI210648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210651, SGI210652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46760[] = {SGI210649, SGI0, SGI210650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210653, SGI210654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46761[] = {SGI210655, SGI0, SGI210656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210658, SGI210660, SGI210660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46762[] = {SGI210657, SGI210659, SGI210661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210662, SGI0, SGI0, SGI0, SGI210663, SGI210663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46763[] = {SGI210664, SGI0, SGI210665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210666, SGI210667, SGI210667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46764[] = {SGI210668, SGI210669, SGI210670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210671, SGI0, SGI0, SGI0, SGI210672, SGI210672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210673, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46765[] = {SGI210674, SGI0, SGI210675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210676, SGI210677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46766[] = {SGI210679, SGI0, SGI210681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210683, SGI210684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46767[] = {SGI210678, SGI210680, SGI210682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210685, SGI210685, SGI0, SGI0, SGI0, SGI0, SGI210686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46768[] = {SGI210687, SGI0, SGI210688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210689, SGI210690, SGI210690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46769[] = {SGI210691, SGI0, SGI210692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210693, SGI210694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46770[] = {SGI210695, SGI0, SGI210696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210697, SGI210698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46771[] = {SGI210699, SGI0, SGI210700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210701, SGI210702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46772[] = {SGI210703, SGI0, SGI210704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210705, SGI210706, SGI210706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46773[] = {SGI210707, SGI0, SGI210708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210709, SGI210710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46774[] = {SGI210711, SGI0, SGI210712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210713, SGI210714, SGI210714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46775[] = {SGI210715, SGI210716, SGI210717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210718, SGI210718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210719, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46776[] = {SGI210720, SGI0, SGI210721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210722, SGI210723, SGI210723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46777[] = {SGI210725, SGI0, SGI210727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210730, SGI210731, SGI210731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46778[] = {SGI210724, SGI210726, SGI210728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210729, SGI0, SGI0, SGI0, SGI210732, SGI210732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46779[] = {SGI210733, SGI0, SGI210734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210738, SGI210739, SGI210739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210742, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46780[] = {SGI210735, SGI210736, SGI210737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210741, SGI0, SGI0, SGI0, SGI210744, SGI210744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46781[] = {SGI210740, SGI0, SGI210743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210745, SGI210746, SGI210746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46782[] = {SGI210747, SGI0, SGI210748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210749, SGI210750, SGI210750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46783[] = {SGI210751, SGI0, SGI210752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210753, SGI210754, SGI210754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46784[] = {SGI210755, SGI0, SGI210756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210757, SGI210758, SGI210758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46785[] = {SGI210759, SGI0, SGI210760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210761, SGI210762, SGI210762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46786[] = {SGI210763, SGI0, SGI210764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210765, SGI210766, SGI210766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46787[] = {SGI210767, SGI0, SGI210768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210769, SGI210770, SGI210770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46788[] = {SGI210771, SGI210772, SGI210773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210774, SGI0, SGI0, SGI0, SGI210775, SGI0, SGI0, SGI210776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46789[] = {SGI210777, SGI210779, SGI210780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210782, SGI0, SGI0, SGI0, SGI210784, SGI210784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46790[] = {SGI210778, SGI0, SGI210781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210783, SGI210785, SGI210785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46791[] = {SGI210786, SGI210787, SGI210788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210789, SGI0, SGI0, SGI0, SGI210790, SGI210790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46792[] = {SGI210793, SGI0, SGI210794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210795, SGI210797, SGI210797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46793[] = {SGI210791, SGI0, SGI210792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210796, SGI210798, SGI210798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46794[] = {SGI210799, SGI210800, SGI210801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210803, SGI0, SGI0, SGI0, SGI210805, SGI210805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46795[] = {SGI210802, SGI0, SGI210804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210806, SGI210807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46796[] = {SGI210808, SGI0, SGI210809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46797[] = {SGI210810, SGI210811, SGI210812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210813, SGI0, SGI0, SGI0, SGI210814, SGI210814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46798[] = {SGI210815, SGI210816, SGI210817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210818, SGI0, SGI0, SGI0, SGI210819, SGI210819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46799[] = {SGI210820, SGI0, SGI210821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46800[] = {SGI210822, SGI210823, SGI210824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210825, SGI0, SGI0, SGI0, SGI210826, SGI210826, SGI210827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46801[] = {SGI210828, SGI210829, SGI210830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210831, SGI0, SGI0, SGI0, SGI210832, SGI210832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46802[] = {SGI210833, SGI210834, SGI210835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210837, SGI0, SGI0, SGI0, SGI210839, SGI210839, SGI0, SGI210840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46803[] = {SGI210836, SGI0, SGI210838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210841, SGI210842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46804[] = {SGI210843, SGI0, SGI210844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210845, SGI210846, SGI210846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46805[] = {SGI210847, SGI210848, SGI210849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210851, SGI210851, SGI0, SGI0, SGI210853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46806[] = {SGI210850, SGI0, SGI210852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210854, SGI210855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46807[] = {SGI210856, SGI210857, SGI210858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210859, SGI0, SGI0, SGI0, SGI210860, SGI210860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46808[] = {SGI210861, SGI0, SGI210862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210863, SGI210864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46809[] = {SGI210865, SGI0, SGI210866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210867, SGI210868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46810[] = {SGI210869, SGI210870, SGI210871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210872, SGI0, SGI0, SGI0, SGI210873, SGI210873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210874, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46811[] = {SGI210875, SGI0, SGI210876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210877, SGI210878, SGI210878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210879, SGI0, SGI0, NULL};
const Segment * const SG46812[] = {SGI210880, SGI0, SGI210881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210882, SGI210883, SGI210883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46813[] = {SGI210884, SGI0, SGI210885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210886, SGI210888, SGI210888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210890, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46814[] = {SGI210887, SGI0, SGI210889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210891, SGI210892, SGI210892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46815[] = {SGI210893, SGI0, SGI210894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210895, SGI210896, SGI210896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46816[] = {SGI210897, SGI210898, SGI210899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210900, SGI0, SGI0, SGI0, SGI210901, SGI210901, SGI0, SGI210903, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46817[] = {SGI210902, SGI0, SGI210904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210905, SGI210908, SGI210908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46818[] = {SGI210906, SGI0, SGI210907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210909, SGI210910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46819[] = {SGI210911, SGI0, SGI210913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210915, SGI210916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46820[] = {SGI210912, SGI0, SGI210914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210917, SGI210918, SGI210918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46821[] = {SGI210919, SGI210920, SGI210921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210922, SGI0, SGI0, SGI0, SGI210923, SGI210923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46822[] = {SGI210924, SGI210925, SGI210926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210927, SGI210927, SGI0, SGI0, SGI0, SGI210928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46823[] = {SGI210929, SGI210930, SGI210931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210932, SGI0, SGI0, SGI0, SGI210933, SGI210933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46824[] = {SGI210934, SGI0, SGI210935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210936, SGI210937, SGI210937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46825[] = {SGI210942, SGI0, SGI210944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46826[] = {SGI210938, SGI210939, SGI210940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210941, SGI0, SGI0, SGI0, SGI210943, SGI210943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46827[] = {SGI210946, SGI210947, SGI210948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210949, SGI0, SGI0, SGI0, SGI210950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46828[] = {SGI210951, SGI210952, SGI210953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210954, SGI0, SGI0, SGI0, SGI210955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46829[] = {SGI210956, SGI210957, SGI210958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210959, SGI0, SGI0, SGI0, SGI210960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46830[] = {SGI210961, SGI0, SGI210962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210963, SGI210964, SGI210964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46831[] = {SGI210965, SGI0, SGI210966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210967, SGI210968, SGI210968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210969, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46832[] = {SGI210970, SGI0, SGI210971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210972, SGI210973, SGI210973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46833[] = {SGI210974, SGI210975, SGI210976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210977, SGI0, SGI0, SGI0, SGI210978, SGI210978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46834[] = {SGI210979, SGI0, SGI210980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210981, SGI210982, SGI210982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46835[] = {SGI210983, SGI0, SGI210984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210985, SGI210986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46836[] = {SGI210987, SGI0, SGI210988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210989, SGI210991, SGI210991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46837[] = {SGI210990, SGI0, SGI210992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210993, SGI210994, SGI210994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46838[] = {SGI210995, SGI0, SGI210996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI210997, SGI210998, SGI210998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46839[] = {SGI210999, SGI0, SGI211000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211002, SGI211004, SGI211004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46840[] = {SGI211001, SGI0, SGI211003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211005, SGI211006, SGI211006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46841[] = {SGI211007, SGI0, SGI211008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211009, SGI211010, SGI211010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46842[] = {SGI211011, SGI0, SGI211012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211013, SGI211014, SGI211014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46843[] = {SGI211015, SGI211016, SGI211017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211018, SGI0, SGI0, SGI0, SGI211019, SGI211019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46844[] = {SGI211020, SGI211021, SGI211022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211023, SGI0, SGI0, SGI0, SGI211024, SGI211024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46845[] = {SGI188321, SGI188322, SGI188323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI188324, SGI0, SGI0, SGI0, SGI188325, SGI188325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46846[] = {SGI211025, SGI0, SGI211026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211027, SGI211028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46847[] = {SGI211029, SGI0, SGI211030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211031, SGI211032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46848[] = {SGI211033, SGI0, SGI211034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211035, SGI211036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46849[] = {SGI211037, SGI211038, SGI211039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211040, SGI0, SGI0, SGI0, SGI211041, SGI211041, SGI0, SGI211042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46850[] = {SGI211043, SGI0, SGI211044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211045, SGI211046, SGI211046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46851[] = {SGI211047, SGI0, SGI211048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211049, SGI211050, SGI211050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46852[] = {SGI211051, SGI211052, SGI211053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211054, SGI0, SGI0, SGI0, SGI211055, SGI211055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46853[] = {SGI211056, SGI0, SGI211057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211058, SGI211059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46854[] = {SGI211060, SGI211061, SGI211062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211063, SGI211063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46855[] = {SGI211064, SGI0, SGI211065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46856[] = {SGI211066, SGI211067, SGI211068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211069, SGI0, SGI0, SGI0, SGI211070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46857[] = {SGI211071, SGI211072, SGI211073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211076, SGI0, SGI0, SGI0, SGI211077, SGI211077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211080, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46858[] = {SGI211074, SGI0, SGI211075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211078, SGI211079, SGI211079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46859[] = {SGI211081, SGI0, SGI211082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211083, SGI211084, SGI211084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46860[] = {SGI211085, SGI0, SGI211086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211088, SGI211090, SGI211090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46861[] = {SGI211087, SGI0, SGI211089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211091, SGI211092, SGI211092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46862[] = {SGI211093, SGI211094, SGI211095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211096, SGI0, SGI0, SGI0, SGI211097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46863[] = {SGI211098, SGI0, SGI211099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211100, SGI0, SGI0, SGI0, SGI211101, SGI211101, SGI0, SGI0, SGI0, SGI211102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46864[] = {SGI211103, SGI0, SGI211104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211105, SGI211106, SGI211106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211107, SGI0, SGI0, NULL};
const Segment * const SG46865[] = {SGI211108, SGI0, SGI211109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211110, SGI211111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46866[] = {SGI211112, SGI0, SGI211113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211114, SGI211115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46867[] = {SGI211116, SGI0, SGI211117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211118, SGI211119, SGI211119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211120, SGI0, SGI0, NULL};
const Segment * const SG46868[] = {SGI211121, SGI0, SGI211122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211123, SGI0, SGI0, SGI0, SGI211124, SGI211124, SGI0, SGI0, SGI0, SGI211125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46869[] = {SGI211126, SGI211127, SGI211128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211129, SGI0, SGI0, SGI0, SGI211130, SGI211130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46870[] = {SGI211131, SGI0, SGI211132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211133, SGI211134, SGI211134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46871[] = {SGI211135, SGI0, SGI211136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211137, SGI211138, SGI211138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46872[] = {SGI211139, SGI0, SGI211140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211141, SGI211142, SGI211142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46873[] = {SGI211143, SGI211144, SGI211145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211146, SGI211146, SGI0, SGI0, SGI0, SGI211147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46874[] = {SGI211148, SGI211149, SGI211150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211151, SGI211151, SGI0, SGI0, SGI0, SGI211152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46875[] = {SGI211153, SGI211154, SGI211155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211156, SGI211156, SGI0, SGI0, SGI0, SGI211157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46876[] = {SGI211158, SGI211159, SGI211160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211161, SGI0, SGI0, SGI0, SGI211162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46877[] = {SGI211163, SGI0, SGI211164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211165, SGI211166, SGI211166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46878[] = {SGI211167, SGI211168, SGI211169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211170, SGI0, SGI0, SGI0, SGI211171, SGI211171, SGI0, SGI211172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46879[] = {SGI211173, SGI211174, SGI211175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211176, SGI0, SGI0, SGI0, SGI211177, SGI211177, SGI0, SGI211178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46880[] = {SGI211179, SGI211180, SGI211181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211182, SGI0, SGI0, SGI0, SGI211183, SGI211183, SGI0, SGI211184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46881[] = {SGI211185, SGI0, SGI211186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211187, SGI211188, SGI211188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46882[] = {SGI211190, SGI0, SGI211192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211195, SGI211196, SGI211196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46883[] = {SGI211189, SGI0, SGI211191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211193, SGI211194, SGI211194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211197, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46884[] = {SGI211198, SGI0, SGI211199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211200, SGI211201, SGI211201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46885[] = {SGI211202, SGI0, SGI211203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211204, SGI211205, SGI211205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46886[] = {SGI211206, SGI0, SGI211207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211208, SGI211210, SGI211210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46887[] = {SGI211209, SGI211211, SGI211212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211213, SGI0, SGI0, SGI0, SGI211214, SGI211214, SGI0, SGI0, NULL};
const Segment * const SG46888[] = {SGI211215, SGI0, SGI211216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211217, SGI211218, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46889[] = {SGI211219, SGI0, SGI211220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211221, SGI211222, SGI211222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46890[] = {SGI211223, SGI0, SGI211224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211225, SGI211226, SGI211226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46891[] = {SGI211227, SGI0, SGI211228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211229, SGI211230, SGI211230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46892[] = {SGI211231, SGI0, SGI211232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211233, SGI211234, SGI211234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46893[] = {SGI211235, SGI0, SGI211236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211237, SGI211238, SGI211238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46894[] = {SGI211239, SGI0, SGI211240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211241, SGI211242, SGI211242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46895[] = {SGI211243, SGI0, SGI211244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211245, SGI211246, SGI211246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46896[] = {SGI211247, SGI211248, SGI211249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211250, SGI211250, SGI0, SGI0, SGI0, SGI0, SGI211251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46897[] = {SGI211252, SGI0, SGI211253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211254, SGI211255, SGI211255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46898[] = {SGI211256, SGI0, SGI211257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211258, SGI211259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46899[] = {SGI211260, SGI0, SGI211261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211262, SGI211263, SGI211263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46900[] = {SGI211264, SGI211265, SGI211266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211267, SGI0, SGI0, SGI0, SGI211268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46901[] = {SGI211269, SGI211270, SGI211271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211272, SGI0, SGI0, SGI0, SGI211273, SGI211273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211274, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46902[] = {SGI211275, SGI211276, SGI211278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211280, SGI0, SGI0, SGI0, SGI211281, SGI211281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211284, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46903[] = {SGI211277, SGI0, SGI211279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211282, SGI211283, SGI211283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46904[] = {SGI211285, SGI0, SGI211286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211287, SGI211288, SGI211288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46905[] = {SGI211289, SGI211290, SGI211291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211292, SGI0, SGI0, SGI0, SGI211293, SGI211293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46906[] = {SGI211294, SGI211295, SGI211296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211298, SGI0, SGI0, SGI0, SGI211300, SGI211300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46907[] = {SGI211297, SGI0, SGI211299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211301, SGI0, SGI0, SGI0, SGI211302, SGI211302, SGI0, SGI0, SGI0, SGI211304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46908[] = {SGI211303, SGI211305, SGI211306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211307, SGI0, SGI0, SGI0, SGI211308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46909[] = {SGI211311, SGI211312, SGI211313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46910[] = {SGI211315, SGI0, SGI211316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211321, SGI211322, SGI211322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46911[] = {SGI211317, SGI0, SGI211319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211323, SGI211325, SGI211325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46912[] = {SGI211318, SGI0, SGI211320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211324, SGI211326, SGI211326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46913[] = {SGI211327, SGI0, SGI211328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46914[] = {SGI211329, SGI211330, SGI211331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46915[] = {SGI211332, SGI211333, SGI211334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211335, SGI211335, SGI0, SGI0, SGI0, SGI211336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46916[] = {SGI211337, SGI211338, SGI211339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211340, SGI211340, SGI0, SGI0, SGI0, SGI211341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46917[] = {SGI211342, SGI0, SGI211343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211344, SGI211345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46918[] = {SGI211346, SGI211347, SGI211348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211349, SGI0, SGI0, SGI0, SGI211350, SGI211350, SGI0, SGI211351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46919[] = {SGI211352, SGI211353, SGI211354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211355, SGI0, SGI0, SGI0, SGI211356, SGI211356, SGI0, SGI211357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46920[] = {SGI211358, SGI0, SGI211359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211363, SGI211365, SGI211365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46921[] = {SGI211360, SGI211361, SGI211362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211364, SGI0, SGI0, SGI0, SGI211366, SGI211366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46922[] = {SGI211367, SGI0, SGI211368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211369, SGI211370, SGI211370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46923[] = {SGI211371, SGI0, SGI211372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211373, SGI211374, SGI211374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46924[] = {SGI211375, SGI0, SGI211376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211377, SGI211378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46925[] = {SGI211379, SGI211380, SGI211381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211382, SGI0, SGI0, SGI0, SGI211383, SGI211383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46926[] = {SGI211384, SGI211385, SGI211386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211387, SGI211387, SGI0, SGI0, SGI0, SGI211388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46927[] = {SGI211389, SGI0, SGI211390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211391, SGI211392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46928[] = {SGI211393, SGI0, SGI211394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211395, SGI211396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46929[] = {SGI211397, SGI211398, SGI211399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211400, SGI0, SGI0, SGI0, SGI211401, SGI211401, SGI0, SGI211402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46930[] = {SGI211403, SGI211404, SGI211405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211406, SGI0, SGI0, SGI0, SGI211407, SGI211407, SGI0, SGI211408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46931[] = {SGI211409, SGI0, SGI211410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211411, SGI211412, SGI211412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211413, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46932[] = {SGI211415, SGI0, SGI211417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211419, SGI211421, SGI211421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46933[] = {SGI211414, SGI0, SGI211416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211418, SGI211420, SGI211420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46934[] = {SGI211422, SGI0, SGI211423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211424, SGI211425, SGI211425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211426, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46935[] = {SGI211427, SGI0, SGI211428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211429, SGI211430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46936[] = {SGI211431, SGI0, SGI211432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211433, SGI211434, SGI211434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46937[] = {SGI211435, SGI211436, SGI211437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211438, SGI211438, SGI0, SGI0, SGI0, SGI211439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46938[] = {SGI211440, SGI0, SGI211441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211442, SGI211443, SGI211443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211444, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46939[] = {SGI211445, SGI0, SGI211446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211447, SGI211448, SGI211448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211449, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46940[] = {SGI211450, SGI0, SGI211451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211452, SGI211453, SGI211453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46941[] = {SGI211454, SGI0, SGI211455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211456, SGI211457, SGI211457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46942[] = {SGI211458, SGI0, SGI211459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211460, SGI211461, SGI211461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46943[] = {SGI211462, SGI211463, SGI211464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211465, SGI0, SGI0, SGI0, SGI211466, SGI211466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46944[] = {SGI211468, SGI211469, SGI211470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211471, SGI0, SGI0, SGI0, SGI211472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46945[] = {SGI211473, SGI0, SGI211474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211475, SGI211476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46946[] = {SGI211477, SGI0, SGI211478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211479, SGI211480, SGI211480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46947[] = {SGI211481, SGI0, SGI211482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211483, SGI211484, SGI211484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46948[] = {SGI211485, SGI0, SGI211486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211487, SGI211488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46949[] = {SGI211489, SGI0, SGI211490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211491, SGI211492, SGI211492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46950[] = {SGI211493, SGI0, SGI211494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211495, SGI211496, SGI211496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46951[] = {SGI211497, SGI211498, SGI211499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211500, SGI0, SGI0, SGI0, SGI211501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46952[] = {SGI211502, SGI211503, SGI211504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211505, SGI0, SGI0, SGI0, SGI211506, SGI211506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46953[] = {SGI211507, SGI0, SGI211508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211512, SGI211513, SGI211513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46954[] = {SGI211509, SGI211510, SGI211511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211514, SGI0, SGI0, SGI0, SGI211515, SGI211515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46955[] = {SGI211516, SGI211517, SGI211518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211519, SGI0, SGI0, SGI0, SGI211520, SGI211520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46956[] = {SGI211521, SGI211522, SGI211523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211524, SGI0, SGI0, SGI0, SGI211525, SGI211525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46957[] = {SGI211527, SGI0, SGI211529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46958[] = {SGI211526, SGI0, SGI211528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211530, SGI211531, SGI211531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46959[] = {SGI211532, SGI0, SGI211533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211534, SGI211535, SGI211535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211536, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46960[] = {SGI211537, SGI0, SGI211538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211540, SGI211542, SGI211542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211543, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46961[] = {SGI211539, SGI0, SGI211541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211545, SGI211546, SGI211546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46962[] = {SGI211544, SGI0, SGI211547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211548, SGI211549, SGI211549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46963[] = {SGI211550, SGI0, SGI211551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211552, SGI211553, SGI211553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46964[] = {SGI0, SGI0, SGI211554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211555, SGI211556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46965[] = {SGI211557, SGI211558, SGI211559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211560, SGI0, SGI0, SGI0, SGI211561, SGI211561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46966[] = {SGI211562, SGI0, SGI211563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211564, SGI211565, SGI211565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46967[] = {SGI211566, SGI0, SGI211567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211568, SGI211569, SGI211569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211570, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46968[] = {SGI211571, SGI0, SGI211572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211573, SGI211574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46969[] = {SGI211575, SGI211576, SGI211577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211578, SGI0, SGI0, SGI0, SGI211579, SGI211579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46970[] = {SGI211580, SGI0, SGI211581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211582, SGI211583, SGI211583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211584, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46971[] = {SGI211585, SGI211587, SGI211589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211590, SGI0, SGI0, SGI0, SGI211593, SGI211593, SGI0, SGI211597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46972[] = {SGI211586, SGI0, SGI211588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211592, SGI211595, SGI211595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211598, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46973[] = {SGI211591, SGI211594, SGI211596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211599, SGI0, SGI0, SGI0, SGI211600, SGI211600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46974[] = {SGI211602, SGI211604, SGI211605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211607, SGI0, SGI0, SGI0, SGI211608, SGI211608, SGI0, SGI211613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46975[] = {SGI211603, SGI0, SGI211606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211609, SGI211611, SGI211611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211615, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46976[] = {SGI211610, SGI211612, SGI211614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211616, SGI0, SGI0, SGI0, SGI211617, SGI211617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46977[] = {SGI211619, SGI0, SGI211620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211621, SGI211622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46978[] = {SGI211623, SGI0, SGI211624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211625, SGI211626, SGI211626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46979[] = {SGI211627, SGI0, SGI211628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211629, SGI211630, SGI211630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46980[] = {SGI211631, SGI211632, SGI211633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211634, SGI211634, SGI0, SGI0, SGI0, SGI211635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46981[] = {SGI211636, SGI0, SGI211637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211640, SGI211641, SGI211641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46982[] = {SGI211638, SGI0, SGI211639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211642, SGI211643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46983[] = {SGI211644, SGI0, SGI211645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211646, SGI211647, SGI211647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46984[] = {SGI211648, SGI0, SGI211649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211650, SGI211651, SGI211651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211654, SGI0, SGI0, NULL};
const Segment * const SG46985[] = {SGI211652, SGI0, SGI211653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211655, SGI211656, SGI211656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46986[] = {SGI211657, SGI211658, SGI211659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211660, SGI211660, SGI0, SGI0, SGI211661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46987[] = {SGI211309, SGI0, SGI211310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211314, SGI211662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46988[] = {SGI211663, SGI211664, SGI211665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211666, SGI0, SGI0, SGI0, SGI211667, SGI211667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46989[] = {SGI211669, SGI0, SGI211670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211671, SGI211672, SGI211672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46990[] = {SGI211673, SGI211674, SGI211675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211676, SGI0, SGI0, SGI0, SGI211677, SGI211677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46991[] = {SGI211679, SGI0, SGI211680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211681, SGI0, SGI0, SGI0, SGI211682, SGI211682, SGI0, SGI0, SGI0, SGI211683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46992[] = {SGI211684, SGI0, SGI211685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211686, SGI211687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46993[] = {SGI211688, SGI0, SGI211689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211690, SGI211691, SGI211691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46994[] = {SGI211692, SGI0, SGI211693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211694, SGI0, SGI0, SGI0, SGI211695, SGI211695, SGI0, SGI0, SGI0, SGI211696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46995[] = {SGI211697, SGI211698, SGI211699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211700, SGI0, SGI0, SGI0, SGI211701, SGI211701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46996[] = {SGI211702, SGI0, SGI211703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211704, SGI211705, SGI211705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46997[] = {SGI211706, SGI211707, SGI211708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211709, SGI0, SGI0, SGI0, SGI211710, SGI0, SGI0, SGI211711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46998[] = {SGI211712, SGI0, SGI211713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211714, SGI211715, SGI211715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG46999[] = {SGI211716, SGI211717, SGI211718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211719, SGI0, SGI0, SGI0, SGI211720, SGI0, SGI0, SGI211721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47000[] = {SGI211722, SGI211723, SGI211724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211725, SGI0, SGI0, SGI0, SGI211726, SGI0, SGI211727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47001[] = {SGI211728, SGI0, SGI211729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211730, SGI211732, SGI211732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47002[] = {SGI211731, SGI211733, SGI211734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211735, SGI0, SGI0, SGI0, SGI211736, SGI0, SGI211737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47003[] = {SGI211738, SGI0, SGI211739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211740, SGI211741, SGI211741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47004[] = {SGI211742, SGI211743, SGI211744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211745, SGI0, SGI0, SGI0, SGI211746, SGI211746, SGI0, SGI0, SGI0, SGI211747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47005[] = {SGI211748, SGI0, SGI211749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211750, SGI211751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47006[] = {SGI211752, SGI211753, SGI211754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211755, SGI0, SGI0, SGI0, SGI211756, SGI211756, SGI0, SGI0, SGI0, SGI211757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47007[] = {SGI211758, SGI211760, SGI211761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211763, SGI0, SGI0, SGI0, SGI211765, SGI211765, SGI0, SGI0, SGI0, SGI211767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47008[] = {SGI211759, SGI0, SGI211762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211764, SGI211766, SGI211766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47009[] = {SGI211768, SGI0, SGI211769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211770, SGI211771, SGI211771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47010[] = {SGI211772, SGI0, SGI211773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211774, SGI211775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47011[] = {SGI211776, SGI211777, SGI211778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211779, SGI211779, SGI0, SGI0, SGI0, SGI211780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47012[] = {SGI211781, SGI0, SGI211782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211783, SGI211784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47013[] = {SGI211785, SGI0, SGI211786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211787, SGI211788, SGI211788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47014[] = {SGI211789, SGI0, SGI211790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211791, SGI211792, SGI211792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47015[] = {SGI211793, SGI0, SGI211794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211795, SGI211796, SGI211796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47016[] = {SGI211797, SGI0, SGI211798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211799, SGI211800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47017[] = {SGI211801, SGI0, SGI211802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211803, SGI211805, SGI211805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47018[] = {SGI211804, SGI0, SGI211806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211807, SGI211808, SGI211808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211809, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47019[] = {SGI0, SGI0, SGI211810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211811, SGI211812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47020[] = {SGI211813, SGI0, SGI211814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211815, SGI211816, SGI211816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47021[] = {SGI211817, SGI0, SGI211818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211819, SGI211820, SGI211820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47022[] = {SGI211821, SGI0, SGI211822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211823, SGI211824, SGI211824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47023[] = {SGI211825, SGI0, SGI211826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211827, SGI211828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47024[] = {SGI211829, SGI0, SGI211830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211831, SGI211832, SGI211832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47025[] = {SGI211833, SGI0, SGI211834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211835, SGI211836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47026[] = {SGI211837, SGI211838, SGI211839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211840, SGI0, SGI0, SGI211841, SGI211842, SGI211842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47027[] = {SGI211843, SGI211844, SGI211845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211846, SGI0, SGI0, SGI0, SGI211847, SGI211847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47028[] = {SGI211848, SGI211849, SGI211850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211851, SGI0, SGI0, SGI0, SGI211852, SGI211852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47029[] = {SGI211853, SGI0, SGI211854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211855, SGI211856, SGI211856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47030[] = {SGI211857, SGI0, SGI211858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211859, SGI211860, SGI211860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47031[] = {SGI211861, SGI0, SGI211862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211863, SGI211864, SGI211864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47032[] = {SGI211865, SGI211866, SGI211867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211868, SGI0, SGI0, SGI0, SGI211869, SGI211869, SGI0, SGI0, SGI211870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47033[] = {SGI211871, SGI0, SGI211872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211873, SGI211874, SGI211874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47034[] = {SGI211875, SGI0, SGI211876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211877, SGI211878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47035[] = {SGI211879, SGI0, SGI211880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211881, SGI211882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47036[] = {SGI211883, SGI211884, SGI211885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211886, SGI0, SGI0, SGI0, SGI211887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47037[] = {SGI211888, SGI211889, SGI211890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211891, SGI0, SGI0, SGI0, SGI211892, SGI211892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47038[] = {SGI211893, SGI211894, SGI211895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211896, SGI0, SGI0, SGI0, SGI211897, SGI211897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47039[] = {SGI211900, SGI211902, SGI211903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211905, SGI0, SGI0, SGI0, SGI211907, SGI211907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47040[] = {SGI211898, SGI211899, SGI211901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211904, SGI0, SGI0, SGI0, SGI211906, SGI211906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47041[] = {SGI211908, SGI211909, SGI211910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211911, SGI0, SGI0, SGI0, SGI211912, SGI211912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47042[] = {SGI211913, SGI211914, SGI211915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211916, SGI0, SGI0, SGI0, SGI211917, SGI211917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211918, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47043[] = {SGI211919, SGI211920, SGI211921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211922, SGI0, SGI0, SGI0, SGI211923, SGI211923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211924, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47044[] = {SGI211925, SGI0, SGI211926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211927, SGI211928, SGI211928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47045[] = {SGI211929, SGI0, SGI211930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211931, SGI211932, SGI211932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47046[] = {SGI211933, SGI211934, SGI211935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211936, SGI211936, SGI0, SGI0, SGI0, SGI211937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47047[] = {SGI211938, SGI211939, SGI211940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211941, SGI211941, SGI0, SGI0, SGI0, SGI211942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47048[] = {SGI211943, SGI0, SGI211944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211945, SGI211946, SGI211946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47049[] = {SGI211947, SGI0, SGI211948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211949, SGI211950, SGI211950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47050[] = {SGI211951, SGI211952, SGI211953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211954, SGI211954, SGI0, SGI0, SGI0, SGI211955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47051[] = {SGI211956, SGI0, SGI211957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211958, SGI211959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47052[] = {SGI211960, SGI0, SGI211961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211962, SGI211963, SGI211963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47053[] = {SGI211964, SGI211965, SGI211966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211969, SGI211969, SGI0, SGI0, SGI0, SGI211970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47054[] = {SGI211967, SGI0, SGI211968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211971, SGI211972, SGI211972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47055[] = {SGI211973, SGI0, SGI211974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211975, SGI211976, SGI211976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47056[] = {SGI211977, SGI0, SGI211978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211979, SGI211980, SGI211980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47057[] = {SGI211981, SGI0, SGI211982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211983, SGI211984, SGI211984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47058[] = {SGI211985, SGI211986, SGI211987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211988, SGI0, SGI0, SGI0, SGI211989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47059[] = {SGI211990, SGI0, SGI211991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211992, SGI211993, SGI211993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47060[] = {SGI211994, SGI211995, SGI211996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI211997, SGI0, SGI0, SGI0, SGI211998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47061[] = {SGI211999, SGI0, SGI212000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212003, SGI0, SGI0, SGI0, SGI212004, SGI212004, SGI0, SGI0, SGI212005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47062[] = {SGI212001, SGI0, SGI212002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212006, SGI212007, SGI212007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47063[] = {SGI212008, SGI0, SGI212009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212011, SGI212013, SGI212013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47064[] = {SGI212010, SGI0, SGI212012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212014, SGI212015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47065[] = {SGI212016, SGI0, SGI212017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212018, SGI212019, SGI212019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47066[] = {SGI212020, SGI0, SGI212021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212022, SGI212023, SGI212023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47067[] = {SGI212026, SGI0, SGI212028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212031, SGI212032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47068[] = {SGI212033, SGI212034, SGI212035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212036, SGI0, SGI0, SGI0, SGI212037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47069[] = {SGI212038, SGI0, SGI212039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212040, SGI0, SGI0, SGI0, SGI212041, SGI212041, SGI0, SGI0, SGI0, SGI212042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47070[] = {SGI212043, SGI212044, SGI212045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212046, SGI0, SGI0, SGI0, SGI212047, SGI212047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47071[] = {SGI212049, SGI212050, SGI212051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212054, SGI0, SGI0, SGI0, SGI212055, SGI212055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47072[] = {SGI212052, SGI0, SGI212053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212056, SGI212057, SGI212057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47073[] = {SGI212058, SGI212059, SGI212060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212061, SGI0, SGI0, SGI0, SGI212063, SGI212063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47074[] = {SGI212062, SGI0, SGI212064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212065, SGI0, SGI0, SGI0, SGI212066, SGI212066, SGI0, SGI0, SGI0, SGI212067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47075[] = {SGI212068, SGI212069, SGI212070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212072, SGI0, SGI0, SGI0, SGI212074, SGI0, SGI0, SGI212075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47076[] = {SGI212071, SGI0, SGI212073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212079, SGI212080, SGI212080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47077[] = {SGI212076, SGI212077, SGI212078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212081, SGI0, SGI0, SGI0, SGI212082, SGI212082, SGI0, SGI212083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47078[] = {SGI212084, SGI0, SGI212085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212086, SGI0, SGI0, SGI0, SGI212087, SGI212087, SGI0, SGI0, SGI0, SGI212088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47079[] = {SGI212089, SGI0, SGI212090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212091, SGI0, SGI0, SGI0, SGI212092, SGI212092, SGI0, SGI0, SGI0, SGI212093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47080[] = {SGI212094, SGI0, SGI212095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212098, SGI212100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47081[] = {SGI212096, SGI212097, SGI212099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212101, SGI0, SGI0, SGI0, SGI212102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47082[] = {SGI212103, SGI0, SGI212104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212105, SGI212106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47083[] = {SGI212107, SGI212108, SGI212109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212110, SGI0, SGI0, SGI0, SGI212111, SGI212111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47084[] = {SGI212112, SGI0, SGI212113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212114, SGI212115, SGI212115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212118, SGI0, SGI0, NULL};
const Segment * const SG47085[] = {SGI212116, SGI212117, SGI212119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212120, SGI212120, SGI0, SGI0, SGI212121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47086[] = {SGI212122, SGI0, SGI212123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212124, SGI212125, SGI212125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212126, SGI0, SGI0, NULL};
const Segment * const SG47087[] = {SGI212127, SGI0, SGI212128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212131, SGI212132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47088[] = {SGI212129, SGI0, SGI212130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212133, SGI212134, SGI212134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47089[] = {SGI212135, SGI212136, SGI212137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212138, SGI0, SGI0, SGI0, SGI212139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47090[] = {SGI212140, SGI0, SGI212141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212142, SGI212143, SGI212143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47091[] = {SGI212144, SGI0, SGI212145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212148, SGI212150, SGI212150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47092[] = {SGI212146, SGI212147, SGI212149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212151, SGI0, SGI0, SGI0, SGI212152, SGI212152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212153, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47093[] = {SGI212154, SGI0, SGI212155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212156, SGI212157, SGI212157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47094[] = {SGI212158, SGI0, SGI212159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212160, SGI212161, SGI212161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47095[] = {SGI212162, SGI0, SGI212163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212164, SGI212165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47096[] = {SGI212166, SGI0, SGI212167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212170, SGI212172, SGI212172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47097[] = {SGI212168, SGI212169, SGI212171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212173, SGI212173, SGI0, SGI0, SGI0, SGI212174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47098[] = {SGI212175, SGI0, SGI212176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212177, SGI212178, SGI212178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47099[] = {SGI212179, SGI0, SGI212180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212181, SGI212182, SGI212182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47100[] = {SGI212183, SGI0, SGI212184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212185, SGI212186, SGI212186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47101[] = {SGI212187, SGI212188, SGI212189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212190, SGI212190, SGI0, SGI0, SGI0, SGI212191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47102[] = {SGI212192, SGI0, SGI212193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212194, SGI212195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47103[] = {SGI212196, SGI212197, SGI212198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212199, SGI212199, SGI0, SGI0, SGI0, SGI212200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47104[] = {SGI212024, SGI212025, SGI212027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212029, SGI0, SGI0, SGI0, SGI212030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47105[] = {SGI212201, SGI0, SGI212202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47106[] = {SGI212203, SGI212204, SGI212205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212206, SGI212206, SGI0, SGI0, SGI0, SGI212207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47107[] = {SGI212208, SGI212209, SGI212210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212211, SGI0, SGI0, SGI0, SGI212212, SGI212212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47108[] = {SGI212213, SGI212214, SGI212215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212216, SGI212216, SGI0, SGI0, SGI0, SGI212217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47109[] = {SGI212218, SGI212219, SGI212220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212221, SGI0, SGI0, SGI0, SGI212222, SGI212222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212227, SGI0, NULL};
const Segment * const SG47110[] = {SGI212223, SGI212226, SGI212228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212230, SGI0, SGI0, SGI0, SGI212231, SGI212231, SGI0, SGI0, SGI212234, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47111[] = {SGI212225, SGI0, SGI212229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212232, SGI212233, SGI212233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212235, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47112[] = {SGI212236, SGI212237, SGI212238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212239, SGI0, SGI0, SGI0, SGI212240, SGI212240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212244, SGI0, NULL};
const Segment * const SG47113[] = {SGI212241, SGI0, SGI212243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212245, SGI212246, SGI212246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212247, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47114[] = {SGI212248, SGI0, SGI212249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212250, SGI212251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212252, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47115[] = {SGI212253, SGI0, SGI212254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212255, SGI212256, SGI212256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47116[] = {SGI212257, SGI0, SGI212258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212259, SGI212260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47117[] = {SGI212261, SGI212262, SGI212263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212264, SGI212264, SGI0, SGI0, SGI0, SGI212265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47118[] = {SGI212266, SGI212267, SGI212268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212269, SGI0, SGI0, SGI0, SGI212270, SGI212270, SGI0, SGI212271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47119[] = {SGI212272, SGI212273, SGI212274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212275, SGI0, SGI0, SGI0, SGI212276, SGI212276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212277, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47120[] = {SGI212278, SGI0, SGI212279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212280, SGI212281, SGI212281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47121[] = {SGI212282, SGI0, SGI212283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212287, SGI212288, SGI212288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47122[] = {SGI212284, SGI212285, SGI212286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212289, SGI0, SGI0, SGI0, SGI212290, SGI212290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47123[] = {SGI212291, SGI0, SGI212292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212293, SGI212294, SGI212294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47124[] = {SGI212295, SGI0, SGI212296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212297, SGI212298, SGI212298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47125[] = {SGI212299, SGI0, SGI212300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212301, SGI212302, SGI212302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47126[] = {SGI212303, SGI0, SGI212304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212305, SGI212306, SGI212306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212307, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47127[] = {SGI212308, SGI212309, SGI212310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212311, SGI0, SGI0, SGI0, SGI212312, SGI212312, SGI0, SGI212313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47128[] = {SGI212314, SGI212315, SGI212316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212317, SGI0, SGI0, SGI0, SGI212318, SGI212318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47129[] = {SGI212319, SGI0, SGI212320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212321, SGI212322, SGI212322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47130[] = {SGI212323, SGI212324, SGI212325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212326, SGI0, SGI0, SGI0, SGI212327, SGI212327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47131[] = {SGI212328, SGI212329, SGI212330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212331, SGI0, SGI0, SGI0, SGI212332, SGI212332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47132[] = {SGI212333, SGI212334, SGI212335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212336, SGI0, SGI0, SGI0, SGI212337, SGI212337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47133[] = {SGI212338, SGI212339, SGI212340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212341, SGI0, SGI0, SGI0, SGI212342, SGI212342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47134[] = {SGI212343, SGI0, SGI212344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212345, SGI212346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47135[] = {SGI212347, SGI0, SGI212348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212349, SGI0, SGI0, SGI0, SGI212350, SGI212350, SGI0, SGI0, SGI0, SGI212351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47136[] = {SGI212352, SGI0, SGI212353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212354, SGI212355, SGI212355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47137[] = {SGI212356, SGI0, SGI212357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212358, SGI212359, SGI212359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47138[] = {SGI212360, SGI0, SGI212361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212362, SGI212363, SGI212363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47139[] = {SGI212364, SGI212365, SGI212366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212367, SGI0, SGI0, SGI0, SGI212369, SGI212369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47140[] = {SGI212368, SGI0, SGI212370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212371, SGI212372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47141[] = {SGI212373, SGI0, SGI212374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47142[] = {SGI212375, SGI0, SGI212376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212380, SGI212381, SGI212381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47143[] = {SGI212377, SGI212378, SGI212379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212382, SGI0, SGI0, SGI0, SGI212383, SGI212383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47144[] = {SGI212384, SGI0, SGI212385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212386, SGI212387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47145[] = {SGI212388, SGI212389, SGI212390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212391, SGI0, SGI0, SGI0, SGI212393, SGI212393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47146[] = {SGI212392, SGI0, SGI212394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212395, SGI212396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47147[] = {SGI212397, SGI0, SGI212398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212399, SGI212400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47148[] = {SGI212403, SGI0, SGI212404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47149[] = {SGI212401, SGI0, SGI212402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212405, SGI212406, SGI212406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47150[] = {SGI212407, SGI0, SGI212409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212411, SGI212412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47151[] = {SGI212408, SGI0, SGI212410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212413, SGI212414, SGI212414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47152[] = {SGI212415, SGI0, SGI212416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212419, SGI212420, SGI212420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47153[] = {SGI212417, SGI0, SGI212418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212421, SGI212423, SGI212423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47154[] = {SGI212422, SGI0, SGI212424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212425, SGI212426, SGI212426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47155[] = {SGI212427, SGI0, SGI212428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212429, SGI212430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47156[] = {SGI212431, SGI212432, SGI212433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212434, SGI0, SGI0, SGI0, SGI212435, SGI212435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47157[] = {SGI212436, SGI212437, SGI212438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212439, SGI212439, SGI0, SGI0, SGI0, SGI212440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47158[] = {SGI212441, SGI0, SGI212442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212443, SGI212444, SGI212444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47159[] = {SGI212445, SGI0, SGI212446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212449, SGI212450, SGI212450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212452, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47160[] = {SGI212447, SGI212448, SGI212451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212453, SGI212453, SGI0, SGI0, SGI0, SGI0, SGI212454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47161[] = {SGI212455, SGI0, SGI212456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212457, SGI212458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47162[] = {SGI212459, SGI212460, SGI212461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212462, SGI0, SGI0, SGI0, SGI212463, SGI212463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47163[] = {SGI212464, SGI0, SGI212465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212466, SGI212467, SGI212467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212468, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47164[] = {SGI212469, SGI0, SGI212470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212471, SGI212472, SGI212472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47165[] = {SGI212473, SGI0, SGI212474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212475, SGI212476, SGI212476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212477, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47166[] = {SGI212478, SGI0, SGI212479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212480, SGI212481, SGI212481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212482, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47167[] = {SGI212483, SGI212484, SGI212485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212486, SGI212486, SGI0, SGI0, SGI0, SGI212487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47168[] = {SGI212488, SGI212489, SGI212490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212491, SGI212491, SGI0, SGI0, SGI0, SGI212492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47169[] = {SGI212493, SGI212494, SGI212495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212496, SGI0, SGI0, SGI0, SGI212497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47170[] = {SGI0, SGI0, SGI212498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212499, SGI212500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47171[] = {SGI212501, SGI0, SGI212502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212503, SGI212504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47172[] = {SGI212505, SGI0, SGI212506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212507, SGI212508, SGI212508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47173[] = {SGI212509, SGI212510, SGI212511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212512, SGI0, SGI0, SGI0, SGI212513, SGI212513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47174[] = {SGI212514, SGI0, SGI212515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47175[] = {SGI212516, SGI212517, SGI212518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212519, SGI0, SGI0, SGI0, SGI212520, SGI212520, SGI0, SGI212521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47176[] = {SGI212522, SGI0, SGI212523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212524, SGI212525, SGI212525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47177[] = {SGI212526, SGI212527, SGI212528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212529, SGI0, SGI0, SGI0, SGI212530, SGI212530, SGI0, SGI212531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47178[] = {SGI212532, SGI212533, SGI212534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212535, SGI0, SGI0, SGI0, SGI212536, SGI212536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47179[] = {SGI212537, SGI0, SGI212538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47180[] = {SGI212539, SGI212540, SGI212541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212542, SGI0, SGI0, SGI0, SGI212543, SGI212543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47181[] = {SGI212546, SGI0, SGI212547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47182[] = {SGI212544, SGI0, SGI212545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212548, SGI212549, SGI212549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47183[] = {SGI212550, SGI0, SGI212551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212552, SGI212553, SGI212553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47184[] = {SGI212554, SGI212555, SGI212556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212557, SGI0, SGI0, SGI0, SGI212558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47185[] = {SGI212559, SGI0, SGI212560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212561, SGI212562, SGI212562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47186[] = {SGI212563, SGI0, SGI212564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212565, SGI212566, SGI212566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47187[] = {SGI212567, SGI212568, SGI212569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212570, SGI0, SGI0, SGI0, SGI212571, SGI212571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47188[] = {SGI212572, SGI0, SGI212573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212574, SGI0, SGI0, SGI0, SGI212575, SGI212575, SGI0, SGI0, SGI0, SGI212576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47189[] = {SGI212577, SGI0, SGI212578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212581, SGI212583, SGI212583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47190[] = {SGI212579, SGI212580, SGI212582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212584, SGI0, SGI0, SGI0, SGI212585, SGI212585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47191[] = {SGI212586, SGI0, SGI212587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212592, SGI212594, SGI212594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47192[] = {SGI212588, SGI212589, SGI212591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212596, SGI0, SGI0, SGI0, SGI212597, SGI212597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47193[] = {SGI212590, SGI212593, SGI212595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212598, SGI0, SGI0, SGI0, SGI212599, SGI212599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47194[] = {SGI212600, SGI212601, SGI212602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212603, SGI0, SGI0, SGI0, SGI212604, SGI212604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47195[] = {SGI212605, SGI0, SGI212606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47196[] = {SGI212607, SGI212608, SGI212609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212610, SGI212610, SGI0, SGI0, SGI0, SGI212611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47197[] = {SGI212612, SGI212613, SGI212614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212615, SGI0, SGI0, SGI0, SGI212616, SGI212616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212617, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47198[] = {SGI212618, SGI0, SGI212619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212620, SGI212621, SGI212621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47199[] = {SGI212622, SGI0, SGI212623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212624, SGI212625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47200[] = {SGI212626, SGI0, SGI212627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212628, SGI212629, SGI212629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47201[] = {SGI212630, SGI212631, SGI212632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212633, SGI0, SGI0, SGI0, SGI212634, SGI212634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47202[] = {SGI212635, SGI0, SGI212636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212637, SGI212638, SGI212638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47203[] = {SGI212639, SGI0, SGI212640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212641, SGI212642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47204[] = {SGI212643, SGI212644, SGI212645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212646, SGI0, SGI0, SGI0, SGI212647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47205[] = {SGI212648, SGI0, SGI212650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212652, SGI212653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47206[] = {SGI212649, SGI0, SGI212651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212654, SGI212655, SGI212655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47207[] = {SGI212656, SGI0, SGI212657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212658, SGI212659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47208[] = {SGI212660, SGI0, SGI212661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212662, SGI212663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47209[] = {SGI212664, SGI0, SGI212665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212666, SGI212667, SGI212667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47210[] = {SGI212668, SGI0, SGI212669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212670, SGI212671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47211[] = {SGI212672, SGI212673, SGI212674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212675, SGI0, SGI0, SGI0, SGI212676, SGI212676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212677, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47212[] = {SGI212678, SGI212679, SGI212680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212683, SGI0, SGI0, SGI0, SGI212684, SGI212684, SGI0, SGI212686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47213[] = {SGI0, SGI212681, SGI212682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212685, SGI0, SGI0, SGI0, SGI212687, SGI212687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47214[] = {SGI212688, SGI212689, SGI212690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212693, SGI0, SGI0, SGI0, SGI212695, SGI212695, SGI0, SGI212696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47215[] = {SGI212691, SGI212692, SGI212694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212697, SGI0, SGI212698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47216[] = {SGI212699, SGI212700, SGI212701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212702, SGI0, SGI0, SGI0, SGI212703, SGI212703, SGI0, SGI212704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47217[] = {SGI212705, SGI0, SGI212706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212707, SGI212708, SGI212708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47218[] = {SGI212709, SGI0, SGI212710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212711, SGI212712, SGI212712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47219[] = {SGI212713, SGI212714, SGI212715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212716, SGI0, SGI0, SGI0, SGI212717, SGI212717, SGI212718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47220[] = {SGI212719, SGI212720, SGI212721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212722, SGI212722, SGI0, SGI0, SGI0, SGI212723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47221[] = {SGI212724, SGI0, SGI212725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212726, SGI212727, SGI212727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47222[] = {SGI212728, SGI0, SGI212729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212730, SGI212731, SGI212731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47223[] = {SGI212732, SGI212733, SGI212734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212735, SGI0, SGI0, SGI0, SGI212736, SGI212736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47224[] = {SGI212737, SGI0, SGI212738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212739, SGI212740, SGI212740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47225[] = {SGI212741, SGI0, SGI212742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212745, SGI212746, SGI212746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47226[] = {SGI212743, SGI0, SGI212744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212747, SGI212748, SGI212748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47227[] = {SGI212749, SGI212750, SGI212751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212752, SGI0, SGI0, SGI0, SGI212753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47228[] = {SGI212754, SGI0, SGI212755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47229[] = {SGI212756, SGI212757, SGI212758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212759, SGI0, SGI0, SGI0, SGI212760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47230[] = {SGI212761, SGI0, SGI212762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212763, SGI212764, SGI212764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47231[] = {SGI212770, SGI0, SGI212771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212772, SGI212773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47232[] = {SGI212774, SGI0, SGI212775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212776, SGI212777, SGI212777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47233[] = {SGI212778, SGI0, SGI212779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212780, SGI212781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47234[] = {SGI212782, SGI0, SGI212783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212784, SGI212785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47235[] = {SGI212786, SGI212787, SGI212788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212790, SGI0, SGI0, SGI0, SGI212793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47236[] = {SGI212789, SGI212791, SGI212792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212794, SGI212794, SGI0, SGI0, SGI0, SGI0, SGI212795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47237[] = {SGI212796, SGI0, SGI212797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212798, SGI212799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47238[] = {SGI212800, SGI0, SGI212801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212802, SGI212803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47239[] = {SGI212804, SGI0, SGI212805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212806, SGI212807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47240[] = {SGI212808, SGI212809, SGI212810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212811, SGI0, SGI0, SGI0, SGI212812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47241[] = {SGI212813, SGI0, SGI212814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212815, SGI212816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47242[] = {SGI212817, SGI212818, SGI212819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212820, SGI0, SGI0, SGI0, SGI212821, SGI212821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47243[] = {SGI212822, SGI212823, SGI212824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212825, SGI0, SGI0, SGI0, SGI212826, SGI212826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47244[] = {SGI212828, SGI212829, SGI212830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212831, SGI212831, SGI0, SGI0, SGI0, SGI0, SGI212832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47245[] = {SGI212833, SGI212834, SGI212835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212836, SGI0, SGI0, SGI0, SGI212837, SGI212837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47246[] = {SGI212838, SGI0, SGI212839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212840, SGI212841, SGI212841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47247[] = {SGI212842, SGI0, SGI212843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212844, SGI212845, SGI212845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212846, SGI0, SGI0, NULL};
const Segment * const SG47248[] = {SGI212847, SGI0, SGI212848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47249[] = {SGI212849, SGI212850, SGI212851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212854, SGI212854, SGI0, SGI0, SGI0, SGI0, SGI212856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47250[] = {SGI212852, SGI212853, SGI212855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212857, SGI0, SGI0, SGI0, SGI212858, SGI212858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47251[] = {SGI212859, SGI0, SGI212860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212861, SGI212862, SGI212862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47252[] = {SGI212863, SGI0, SGI212864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212865, SGI212866, SGI212866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47253[] = {SGI212867, SGI212868, SGI212869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212870, SGI0, SGI0, SGI0, SGI212871, SGI0, SGI0, SGI212872, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47254[] = {SGI212873, SGI0, SGI212874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212875, SGI212876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47255[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212877, SGI0, SGI0, SGI212878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47256[] = {SGI212879, SGI212880, SGI212881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212882, SGI0, SGI0, SGI0, SGI212883, SGI0, SGI0, SGI212884, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47257[] = {SGI212885, SGI0, SGI212886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212889, SGI212890, SGI212890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47258[] = {SGI212887, SGI0, SGI212888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212891, SGI212892, SGI212892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47259[] = {SGI212893, SGI212894, SGI212895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212896, SGI0, SGI0, SGI0, SGI212897, SGI0, SGI0, SGI212898, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47260[] = {SGI212765, SGI212766, SGI212767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212768, SGI0, SGI0, SGI0, SGI212769, SGI212769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212899, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47261[] = {SGI212900, SGI0, SGI212901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212902, SGI212903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47262[] = {SGI212904, SGI0, SGI212905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212906, SGI212907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47263[] = {SGI212908, SGI0, SGI212909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212910, SGI212911, SGI212911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47264[] = {SGI212912, SGI0, SGI212913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212914, SGI212915, SGI212915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47265[] = {SGI212916, SGI0, SGI212917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212918, SGI212919, SGI212919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47266[] = {SGI212920, SGI0, SGI212921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212922, SGI212923, SGI212923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212924, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47267[] = {SGI212925, SGI0, SGI212926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212930, SGI212931, SGI212931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212934, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47268[] = {SGI212927, SGI212928, SGI212929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212932, SGI0, SGI0, SGI0, SGI212933, SGI212933, SGI0, SGI212935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47269[] = {SGI212936, SGI212937, SGI212938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212939, SGI0, SGI0, SGI0, SGI212940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47270[] = {SGI212941, SGI0, SGI212942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212943, SGI212944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47271[] = {SGI212945, SGI212946, SGI212947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212948, SGI212948, SGI0, SGI0, SGI0, SGI212949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47272[] = {SGI212950, SGI0, SGI212952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212954, SGI0, SGI0, SGI0, SGI212955, SGI212955, SGI0, SGI0, SGI0, SGI0, SGI212958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47273[] = {SGI212951, SGI0, SGI212953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212956, SGI212957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47274[] = {SGI212959, SGI0, SGI212960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212961, SGI212962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47275[] = {SGI212963, SGI212964, SGI212965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212967, SGI0, SGI0, SGI0, SGI212969, SGI212969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47276[] = {SGI212966, SGI0, SGI212968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212972, SGI212974, SGI212974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212975, SGI0, SGI0, NULL};
const Segment * const SG47277[] = {SGI212971, SGI0, SGI212973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212976, SGI212977, SGI212977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47278[] = {SGI212978, SGI212979, SGI212980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212981, SGI0, SGI0, SGI0, SGI212982, SGI212982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212983, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47279[] = {SGI212984, SGI0, SGI212985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212986, SGI212987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47280[] = {SGI212988, SGI0, SGI212989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212990, SGI212991, SGI212991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47281[] = {SGI212992, SGI212993, SGI212994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212995, SGI0, SGI0, SGI0, SGI212996, SGI212996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI212997, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47282[] = {SGI212998, SGI212999, SGI213000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213001, SGI0, SGI0, SGI0, SGI213002, SGI213002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213003, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47283[] = {SGI213004, SGI0, SGI213005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213006, SGI213007, SGI213007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47284[] = {SGI213008, SGI0, SGI213009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213010, SGI213011, SGI213011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47285[] = {SGI213012, SGI0, SGI213013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213014, SGI213015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47286[] = {SGI213016, SGI0, SGI213017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213018, SGI213019, SGI213019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47287[] = {SGI213020, SGI0, SGI213021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213022, SGI213023, SGI213023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47288[] = {SGI213024, SGI0, SGI213025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213026, SGI213027, SGI213027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47289[] = {SGI213028, SGI213029, SGI213030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213031, SGI0, SGI0, SGI0, SGI213032, SGI213032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47290[] = {SGI213033, SGI213034, SGI213035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213036, SGI213036, SGI0, SGI0, SGI0, SGI0, SGI213037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47291[] = {SGI213038, SGI0, SGI213039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213040, SGI213041, SGI213041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47292[] = {SGI213042, SGI0, SGI213043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213044, SGI213045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47293[] = {SGI213046, SGI0, SGI213047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213048, SGI213049, SGI213049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47294[] = {SGI213050, SGI0, SGI213051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213052, SGI213053, SGI213053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47295[] = {SGI213054, SGI0, SGI213055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213056, SGI213057, SGI213057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47296[] = {SGI213058, SGI0, SGI213059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213060, SGI0, SGI0, SGI0, SGI213061, SGI213061, SGI0, SGI0, NULL};
const Segment * const SG47297[] = {SGI213062, SGI213063, SGI213064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213065, SGI0, SGI0, SGI0, SGI213066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47298[] = {SGI213067, SGI0, SGI213068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213069, SGI213070, SGI213070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47299[] = {SGI213073, SGI0, SGI213074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213075, SGI213076, SGI213076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47300[] = {SGI213077, SGI0, SGI213078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213079, SGI213080, SGI213080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47301[] = {SGI213081, SGI213082, SGI213083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213084, SGI0, SGI0, SGI0, SGI213085, SGI213085, SGI0, SGI0, SGI0, SGI213086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47302[] = {SGI213087, SGI0, SGI213088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213089, SGI213090, SGI213090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47303[] = {SGI213091, SGI0, SGI213092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213093, SGI213094, SGI213094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47304[] = {SGI213095, SGI0, SGI213096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213097, SGI213098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47305[] = {SGI213099, SGI0, SGI213100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213101, SGI213102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47306[] = {SGI213103, SGI213104, SGI213105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213106, SGI0, SGI0, SGI0, SGI213107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47307[] = {SGI213108, SGI213109, SGI213110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213114, SGI0, SGI0, SGI0, SGI213115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47308[] = {SGI213111, SGI213112, SGI213113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213116, SGI0, SGI0, SGI0, SGI213117, SGI213117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47309[] = {SGI213118, SGI213119, SGI213120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213121, SGI0, SGI0, SGI0, SGI213122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47310[] = {SGI213123, SGI213124, SGI213125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213126, SGI0, SGI0, SGI0, SGI213127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47311[] = {SGI213128, SGI213129, SGI213130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213131, SGI0, SGI0, SGI0, SGI213132, SGI213132, SGI0, SGI0, SGI0, SGI0, SGI213133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47312[] = {SGI213134, SGI0, SGI213135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213136, SGI213137, SGI213137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47313[] = {SGI213138, SGI0, SGI213139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213140, SGI213141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47314[] = {SGI213142, SGI0, SGI213143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213144, SGI213145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47315[] = {SGI213146, SGI0, SGI213147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213148, SGI213149, SGI213149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47316[] = {SGI213150, SGI0, SGI213151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213152, SGI213153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47317[] = {SGI213154, SGI0, SGI213155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213156, SGI213157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47318[] = {SGI213158, SGI213159, SGI213160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213161, SGI0, SGI0, SGI0, SGI213162, SGI213162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47319[] = {SGI213163, SGI0, SGI213164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213165, SGI213166, SGI213166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47320[] = {SGI213167, SGI0, SGI213168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213169, SGI213170, SGI213170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47321[] = {SGI213171, SGI0, SGI213172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213173, SGI213174, SGI213174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47322[] = {SGI213175, SGI0, SGI213176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213177, SGI213178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47323[] = {SGI213179, SGI0, SGI213180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213181, SGI213182, SGI213182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47324[] = {SGI213183, SGI0, SGI213184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213185, SGI213186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47325[] = {SGI213071, SGI0, SGI213072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213189, SGI213190, SGI213190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47326[] = {SGI213187, SGI0, SGI213188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213191, SGI213192, SGI213192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47327[] = {SGI213193, SGI213194, SGI213195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213196, SGI0, SGI0, SGI0, SGI213197, SGI213197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213198, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47328[] = {SGI213199, SGI0, SGI213200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213201, SGI213202, SGI213202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47329[] = {SGI213203, SGI0, SGI213204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213205, SGI213206, SGI213206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47330[] = {SGI213207, SGI0, SGI213208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213209, SGI213210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47331[] = {SGI213211, SGI0, SGI213212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213215, SGI213216, SGI213216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47332[] = {SGI213213, SGI0, SGI213214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213217, SGI213218, SGI213218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47333[] = {SGI213219, SGI213220, SGI213221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213222, SGI0, SGI0, SGI0, SGI213223, SGI213223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213224, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47334[] = {SGI213225, SGI213226, SGI213227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213228, SGI213228, SGI0, SGI0, SGI0, SGI213229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47335[] = {SGI213230, SGI213231, SGI213232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213233, SGI213233, SGI0, SGI0, SGI0, SGI213234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47336[] = {SGI213235, SGI0, SGI213236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213237, SGI213238, SGI213238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47337[] = {SGI213239, SGI213240, SGI213241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213242, SGI0, SGI0, SGI0, SGI213243, SGI213243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47338[] = {SGI213244, SGI213245, SGI213246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213247, SGI0, SGI0, SGI0, SGI213248, SGI213248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47339[] = {SGI213249, SGI213250, SGI213251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213252, SGI0, SGI0, SGI0, SGI213253, SGI213253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47340[] = {SGI213254, SGI213255, SGI213256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213257, SGI213257, SGI0, SGI0, SGI0, SGI213258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47341[] = {SGI213259, SGI0, SGI213260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213261, SGI213262, SGI213262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47342[] = {SGI213263, SGI0, SGI213264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213266, SGI213268, SGI213268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47343[] = {SGI213265, SGI0, SGI213267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213269, SGI213270, SGI213270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213271, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47344[] = {SGI213272, SGI0, SGI213273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213274, SGI213275, SGI213275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213276, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47345[] = {SGI213277, SGI0, SGI213278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213279, SGI213280, SGI213280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213281, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47346[] = {SGI213282, SGI213283, SGI213284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213285, SGI213285, SGI0, SGI0, SGI0, SGI213286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47347[] = {SGI213287, SGI0, SGI213288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213289, SGI213290, SGI213290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47348[] = {SGI213292, SGI0, SGI213294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213296, SGI213298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47349[] = {SGI213291, SGI0, SGI213293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213295, SGI213297, SGI213297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47350[] = {SGI213299, SGI0, SGI213300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213301, SGI213302, SGI213302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47351[] = {SGI213303, SGI0, SGI213304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213305, SGI213306, SGI213306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47352[] = {SGI213307, SGI213308, SGI213309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213310, SGI0, SGI0, SGI0, SGI213311, SGI213311, SGI213312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47353[] = {SGI213313, SGI0, SGI213314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213315, SGI213316, SGI213316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47354[] = {SGI213317, SGI0, SGI213318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213319, SGI213320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47355[] = {SGI213321, SGI213322, SGI213323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213324, SGI0, SGI0, SGI0, SGI213325, SGI213325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47356[] = {SGI213326, SGI0, SGI213328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213330, SGI213332, SGI213332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47357[] = {SGI213327, SGI0, SGI213329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213331, SGI213333, SGI213333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47358[] = {SGI213334, SGI213335, SGI213336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213337, SGI0, SGI0, SGI0, SGI213338, SGI213338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213339, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47359[] = {SGI213340, SGI213341, SGI213342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213343, SGI0, SGI0, SGI0, SGI213344, SGI213345, SGI0, SGI0, SGI0, SGI0, SGI213346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47360[] = {SGI213347, SGI0, SGI213348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213349, SGI0, SGI0, SGI0, SGI213350, SGI213350, SGI0, SGI0, SGI0, SGI0, SGI213351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47361[] = {SGI213352, SGI213353, SGI213354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213355, SGI0, SGI0, SGI0, SGI213356, SGI213356, SGI0, SGI0, SGI213357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47362[] = {SGI213358, SGI0, SGI213359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213360, SGI213361, SGI213361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47363[] = {SGI213362, SGI213363, SGI213364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213365, SGI0, SGI0, SGI0, SGI213366, SGI213366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47364[] = {SGI213367, SGI213368, SGI213369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213370, SGI0, SGI0, SGI0, SGI213371, SGI213371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47365[] = {SGI213372, SGI213373, SGI213374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213375, SGI213375, SGI0, SGI0, SGI0, SGI213376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47366[] = {SGI213377, SGI0, SGI213378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213379, SGI213380, SGI213380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47367[] = {SGI213381, SGI0, SGI213382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213383, SGI213384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47368[] = {SGI213385, SGI0, SGI213386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213389, SGI213390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47369[] = {SGI213387, SGI0, SGI213388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213391, SGI213392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47370[] = {SGI213393, SGI213394, SGI213395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213396, SGI0, SGI0, SGI0, SGI213397, SGI213397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47371[] = {SGI213398, SGI213399, SGI213400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213401, SGI0, SGI0, SGI0, SGI213402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47372[] = {SGI213403, SGI0, SGI213404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213405, SGI0, SGI0, SGI0, SGI213406, SGI213406, SGI0, SGI0, SGI0, SGI213407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47373[] = {SGI213408, SGI0, SGI213409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213410, SGI213411, SGI213411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47374[] = {SGI213412, SGI0, SGI213413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213416, SGI213417, SGI213417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47375[] = {SGI213414, SGI0, SGI213415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213418, SGI213419, SGI213419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213420, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47376[] = {SGI213421, SGI0, SGI213422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213423, SGI213424, SGI213424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47377[] = {SGI213425, SGI0, SGI213426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213427, SGI213428, SGI213428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47378[] = {SGI213429, SGI0, SGI213430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213433, SGI213435, SGI213435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47379[] = {SGI213431, SGI213432, SGI213434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213436, SGI0, SGI0, SGI0, SGI213437, SGI213437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47380[] = {SGI213438, SGI0, SGI213439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213440, SGI213441, SGI213441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47381[] = {SGI213442, SGI213443, SGI213444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213445, SGI213445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213446, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47382[] = {SGI213447, SGI0, SGI213448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213449, SGI213450, SGI213450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47383[] = {SGI213451, SGI0, SGI213452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213453, SGI213454, SGI213454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47384[] = {SGI213455, SGI0, SGI213456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213459, SGI213460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47385[] = {SGI213457, SGI0, SGI213458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213461, SGI213462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47386[] = {SGI213463, SGI0, SGI213464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213465, SGI213466, SGI213466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47387[] = {SGI213467, SGI0, SGI213468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213469, SGI213470, SGI213470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47388[] = {SGI213471, SGI0, SGI213472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213473, SGI213474, SGI213474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47389[] = {SGI213475, SGI213476, SGI213477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213478, SGI0, SGI0, SGI0, SGI213479, SGI213479, SGI0, SGI0, SGI213480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47390[] = {SGI213481, SGI213482, SGI213483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213484, SGI0, SGI0, SGI0, SGI213485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47391[] = {SGI213486, SGI0, SGI213487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213488, SGI213489, SGI213489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47392[] = {SGI213490, SGI213491, SGI213492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213493, SGI0, SGI0, SGI0, SGI213494, SGI213494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47393[] = {SGI213495, SGI213496, SGI213497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213498, SGI0, SGI0, SGI0, SGI213499, SGI213499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47394[] = {SGI213500, SGI0, SGI213501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213502, SGI213503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47395[] = {SGI213504, SGI213505, SGI213506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213507, SGI0, SGI0, SGI0, SGI213508, SGI213508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213509, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47396[] = {SGI213510, SGI0, SGI213511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213512, SGI213513, SGI213513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213514, SGI0, SGI0, NULL};
const Segment * const SG47397[] = {SGI213515, SGI0, SGI213516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213517, SGI213518, SGI213518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47398[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213519, SGI0, SGI0, SGI213520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47399[] = {SGI213521, SGI0, SGI213522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213523, SGI0, SGI0, SGI0, SGI213524, SGI213524, SGI0, SGI0, SGI0, SGI213525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47400[] = {SGI213526, SGI0, SGI213527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213528, SGI213529, SGI213529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47401[] = {SGI213530, SGI0, SGI213531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213532, SGI213533, SGI213533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47402[] = {SGI213534, SGI0, SGI213535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213536, SGI213537, SGI213537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213538, SGI0, SGI0, NULL};
const Segment * const SG47403[] = {SGI213539, SGI0, SGI213540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213541, SGI213542, SGI213542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213543, SGI0, SGI0, NULL};
const Segment * const SG47404[] = {SGI213544, SGI0, SGI213545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213546, SGI213547, SGI213547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47405[] = {SGI213548, SGI0, SGI213549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213550, SGI213551, SGI213551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47406[] = {SGI213552, SGI0, SGI213553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213554, SGI213555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47407[] = {SGI213556, SGI0, SGI213557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213558, SGI213559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47408[] = {SGI213560, SGI0, SGI213561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213562, SGI213563, SGI213563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47409[] = {SGI213564, SGI213565, SGI213566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213567, SGI0, SGI0, SGI0, SGI213568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47410[] = {SGI213569, SGI213570, SGI213571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213572, SGI0, SGI0, SGI0, SGI213573, SGI213573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47411[] = {SGI213574, SGI0, SGI213575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213576, SGI0, SGI0, SGI0, SGI213577, SGI213577, SGI0, SGI0, SGI0, SGI213578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47412[] = {SGI213579, SGI0, SGI213580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213581, SGI213582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47413[] = {SGI213583, SGI0, SGI213584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213585, SGI213586, SGI213586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47414[] = {SGI213587, SGI0, SGI213588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213589, SGI213590, SGI213590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47415[] = {SGI213591, SGI0, SGI213592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213596, SGI213597, SGI213597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47416[] = {SGI213593, SGI213594, SGI213595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213598, SGI0, SGI0, SGI0, SGI213599, SGI0, SGI0, SGI0, SGI213600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47417[] = {SGI213601, SGI213602, SGI213603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213604, SGI0, SGI0, SGI0, SGI213605, SGI213605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213606, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47418[] = {SGI213607, SGI213608, SGI213609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213610, SGI0, SGI0, SGI0, SGI213611, SGI213611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47419[] = {SGI213612, SGI213613, SGI213614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213615, SGI0, SGI0, SGI0, SGI213616, SGI213616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47420[] = {SGI213617, SGI213618, SGI213619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213620, SGI0, SGI0, SGI0, SGI213621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47421[] = {SGI213622, SGI213623, SGI213624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213625, SGI0, SGI0, SGI0, SGI213626, SGI213626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47422[] = {SGI213627, SGI0, SGI213628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213629, SGI213630, SGI213630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47423[] = {SGI213631, SGI213632, SGI213633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213634, SGI0, SGI0, SGI0, SGI213635, SGI213635, SGI0, SGI213636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47424[] = {SGI213637, SGI213638, SGI213639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213640, SGI0, SGI0, SGI0, SGI213641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47425[] = {SGI213642, SGI0, SGI213643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213646, SGI213647, SGI213647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47426[] = {SGI213644, SGI0, SGI213645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213648, SGI213649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47427[] = {SGI213650, SGI0, SGI213651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213652, SGI213653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47428[] = {SGI213654, SGI213655, SGI213656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213657, SGI0, SGI0, SGI0, SGI213658, SGI213658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47429[] = {SGI213659, SGI0, SGI213660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213661, SGI213662, SGI213662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47430[] = {SGI213663, SGI213664, SGI213665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213666, SGI213666, SGI0, SGI0, SGI0, SGI213667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47431[] = {SGI213668, SGI213669, SGI213670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213671, SGI0, SGI0, SGI0, SGI213672, SGI213672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47432[] = {SGI213673, SGI213674, SGI213675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213676, SGI0, SGI0, SGI0, SGI213677, SGI213677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47433[] = {SGI213678, SGI0, SGI213679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213680, SGI213681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47434[] = {SGI213682, SGI213683, SGI213684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213687, SGI0, SGI0, SGI0, SGI213691, SGI213691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47435[] = {SGI213685, SGI213686, SGI213689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213692, SGI0, SGI0, SGI0, SGI213694, SGI213694, SGI0, SGI213696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47436[] = {SGI213688, SGI0, SGI213690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213693, SGI213695, SGI213695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47437[] = {SGI213697, SGI0, SGI213698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213699, SGI213700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47438[] = {SGI213701, SGI0, SGI213702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213703, SGI213704, SGI213704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47439[] = {SGI213705, SGI213706, SGI213707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213708, SGI213708, SGI0, SGI0, SGI0, SGI213709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47440[] = {SGI0, SGI213710, SGI213711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213712, SGI0, SGI0, SGI0, SGI213713, SGI213713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47441[] = {SGI213714, SGI0, SGI213715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213716, SGI213717, SGI213717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47442[] = {SGI213718, SGI0, SGI213719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213720, SGI213721, SGI213721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213722, SGI0, SGI0, NULL};
const Segment * const SG47443[] = {SGI213723, SGI0, SGI213724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213725, SGI213726, SGI213726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47444[] = {SGI213727, SGI213728, SGI213729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213730, SGI213730, SGI0, SGI0, SGI0, SGI213731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47445[] = {SGI213732, SGI0, SGI213733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213734, SGI213735, SGI213735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47446[] = {SGI213736, SGI0, SGI213737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213738, SGI213739, SGI213739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47447[] = {SGI213740, SGI213741, SGI213742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213743, SGI213743, SGI0, SGI0, SGI0, SGI0, SGI213744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47448[] = {SGI213745, SGI0, SGI213746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213747, SGI213748, SGI213748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47449[] = {SGI213749, SGI0, SGI213750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213751, SGI213752, SGI213752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47450[] = {SGI213753, SGI213754, SGI213755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213756, SGI0, SGI0, SGI0, SGI213757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47451[] = {SGI213758, SGI213759, SGI213760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213761, SGI0, SGI0, SGI0, SGI213762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47452[] = {SGI213763, SGI0, SGI213764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213765, SGI213766, SGI213766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47453[] = {SGI213767, SGI213768, SGI213769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213770, SGI0, SGI0, SGI0, SGI213771, SGI213771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47454[] = {SGI213772, SGI0, SGI213773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213774, SGI213775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47455[] = {SGI213776, SGI213777, SGI213778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213781, SGI0, SGI0, SGI0, SGI213782, SGI213782, SGI0, SGI213784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47456[] = {SGI213779, SGI0, SGI213780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213783, SGI213785, SGI213785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47457[] = {SGI213786, SGI0, SGI213787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213788, SGI213789, SGI213789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47458[] = {SGI213790, SGI0, SGI213791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213792, SGI213793, SGI213793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47459[] = {SGI213794, SGI0, SGI213795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213796, SGI213797, SGI213797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47460[] = {SGI213798, SGI0, SGI213799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213800, SGI213801, SGI213801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47461[] = {SGI213802, SGI213803, SGI213804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213805, SGI0, SGI0, SGI0, SGI213806, SGI213806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47462[] = {SGI213807, SGI213808, SGI213809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213810, SGI0, SGI0, SGI0, SGI213811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47463[] = {SGI213812, SGI213813, SGI213814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213815, SGI0, SGI0, SGI0, SGI213816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47464[] = {SGI213817, SGI0, SGI213818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213819, SGI213820, SGI213820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47465[] = {SGI213821, SGI213822, SGI213823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213824, SGI0, SGI0, SGI0, SGI213825, SGI213825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47466[] = {SGI213827, SGI213828, SGI213829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213830, SGI0, SGI0, SGI0, SGI213831, SGI213831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47467[] = {SGI213833, SGI0, SGI213834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213835, SGI213836, SGI213836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47468[] = {SGI213837, SGI0, SGI213838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213839, SGI213840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47469[] = {SGI213841, SGI0, SGI213842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213843, SGI213844, SGI213844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47470[] = {SGI213845, SGI0, SGI213846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213847, SGI213848, SGI213848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47471[] = {SGI213849, SGI213850, SGI213851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213852, SGI0, SGI0, SGI0, SGI213853, SGI213853, SGI0, SGI0, SGI213854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47472[] = {SGI213855, SGI0, SGI213856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213857, SGI213858, SGI213858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47473[] = {SGI213859, SGI0, SGI213860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213861, SGI213862, SGI213862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47474[] = {SGI213863, SGI213864, SGI213865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213866, SGI0, SGI0, SGI0, SGI213867, SGI213867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213868, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47475[] = {SGI213869, SGI0, SGI213870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213873, SGI213874, SGI213874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47476[] = {SGI213871, SGI0, SGI213872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213875, SGI213876, SGI213876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47477[] = {SGI213877, SGI0, SGI213878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213879, SGI213880, SGI213880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47478[] = {SGI213881, SGI0, SGI213882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213883, SGI213884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47479[] = {SGI213885, SGI0, SGI213886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213887, SGI213888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47480[] = {SGI213889, SGI0, SGI213890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213891, SGI213892, SGI213892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47481[] = {SGI213893, SGI0, SGI213894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213895, SGI213896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47482[] = {SGI213897, SGI0, SGI213898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213899, SGI213900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47483[] = {SGI213901, SGI0, SGI213902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213903, SGI213904, SGI213904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213905, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47484[] = {SGI213906, SGI0, SGI213907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213908, SGI213909, SGI213909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47485[] = {SGI213910, SGI213911, SGI213912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213913, SGI0, SGI0, SGI0, SGI213914, SGI213914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47486[] = {SGI213915, SGI0, SGI213916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213917, SGI213918, SGI213918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47487[] = {SGI213919, SGI213920, SGI213921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213922, SGI0, SGI0, SGI0, SGI213923, SGI0, SGI0, SGI213924, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47488[] = {SGI213925, SGI0, SGI213926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213927, SGI213928, SGI213928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47489[] = {SGI213929, SGI0, SGI213930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213931, SGI213932, SGI213932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47490[] = {SGI213933, SGI213934, SGI213935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213938, SGI213938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213939, NULL};
const Segment * const SG47491[] = {SGI213936, SGI0, SGI213937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213940, SGI213941, SGI213941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47492[] = {SGI213942, SGI0, SGI213943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213944, SGI213945, SGI213945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47493[] = {SGI213946, SGI213947, SGI213948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213949, SGI0, SGI0, SGI0, SGI213950, SGI213950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47494[] = {SGI213951, SGI213952, SGI213953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213954, SGI213954, SGI0, SGI0, SGI0, SGI213955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47495[] = {SGI213956, SGI213957, SGI213958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213959, SGI0, SGI0, SGI0, SGI213960, SGI213960, SGI0, SGI213961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47496[] = {SGI213962, SGI213963, SGI213964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213965, SGI0, SGI0, SGI0, SGI213966, SGI213966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213967, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47497[] = {SGI213968, SGI0, SGI213969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213970, SGI213971, SGI213971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47498[] = {SGI213972, SGI0, SGI213973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213974, SGI213975, SGI213975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47499[] = {SGI213976, SGI0, SGI213977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213978, SGI213979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47500[] = {SGI213980, SGI0, SGI213981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213982, SGI213983, SGI213983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47501[] = {SGI213984, SGI0, SGI213985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213986, SGI213987, SGI213987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47502[] = {SGI213988, SGI0, SGI213989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213990, SGI213991, SGI213991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47503[] = {SGI213992, SGI0, SGI213993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213994, SGI213995, SGI213995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47504[] = {SGI213996, SGI0, SGI213997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI213998, SGI213999, SGI213999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47505[] = {SGI214000, SGI214001, SGI214002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214003, SGI0, SGI0, SGI0, SGI214004, SGI214004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47506[] = {SGI214005, SGI214006, SGI214007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214008, SGI0, SGI0, SGI0, SGI214009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47507[] = {SGI214010, SGI214011, SGI214012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214013, SGI0, SGI0, SGI0, SGI214014, SGI214014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214015, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47508[] = {SGI214016, SGI214017, SGI214018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214019, SGI0, SGI0, SGI0, SGI214020, SGI214020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47509[] = {SGI214021, SGI0, SGI214022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214023, SGI214024, SGI214024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47510[] = {SGI214025, SGI0, SGI214026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214030, SGI214031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47511[] = {SGI214027, SGI214028, SGI214029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214032, SGI0, SGI0, SGI0, SGI214033, SGI214033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214034, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47512[] = {SGI0, SGI0, SGI214035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214036, SGI214037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47513[] = {SGI214038, SGI214039, SGI214040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214041, SGI0, SGI0, SGI0, SGI214042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47514[] = {SGI214043, SGI214044, SGI214045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214046, SGI0, SGI0, SGI0, SGI214047, SGI214047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47515[] = {SGI214048, SGI0, SGI214050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214052, SGI214053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47516[] = {SGI214049, SGI0, SGI214051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214054, SGI214055, SGI214055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47517[] = {SGI214056, SGI0, SGI214058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214061, SGI214063, SGI214063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47518[] = {SGI214057, SGI214059, SGI214060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214062, SGI0, SGI0, SGI0, SGI214064, SGI214064, SGI0, SGI214065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47519[] = {SGI214066, SGI214067, SGI214068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214069, SGI0, SGI0, SGI0, SGI214070, SGI214070, SGI0, SGI214071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47520[] = {SGI214072, SGI0, SGI214073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214074, SGI214075, SGI214075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47521[] = {SGI214076, SGI0, SGI214077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214078, SGI214079, SGI214079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47522[] = {SGI214080, SGI214081, SGI214082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214083, SGI0, SGI0, SGI0, SGI214084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47523[] = {SGI214085, SGI214086, SGI214087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214088, SGI0, SGI0, SGI0, SGI214089, SGI214089, SGI0, SGI214090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47524[] = {SGI214091, SGI0, SGI214093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214096, SGI214098, SGI214098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47525[] = {SGI214092, SGI214094, SGI214095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214097, SGI0, SGI0, SGI0, SGI214100, SGI214100, SGI0, SGI214102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47526[] = {SGI214099, SGI0, SGI214101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214103, SGI214104, SGI214104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214105, SGI0, SGI0, NULL};
const Segment * const SG47527[] = {SGI214106, SGI0, SGI214107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214110, SGI214111, SGI214111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47528[] = {SGI214108, SGI0, SGI214109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214112, SGI214113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47529[] = {SGI214114, SGI0, SGI214115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214116, SGI214117, SGI214117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47530[] = {SGI214118, SGI0, SGI214119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214120, SGI214121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47531[] = {SGI214122, SGI0, SGI214123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214124, SGI214125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47532[] = {SGI214126, SGI0, SGI214127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214128, SGI214129, SGI214129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47533[] = {SGI214130, SGI214131, SGI214132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214133, SGI214133, SGI0, SGI0, SGI0, SGI214134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47534[] = {SGI214135, SGI0, SGI214136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214137, SGI214138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214139, SGI0, SGI0, NULL};
const Segment * const SG47535[] = {SGI214140, SGI214141, SGI214142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214143, SGI0, SGI0, SGI0, SGI214144, SGI214144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47536[] = {SGI214145, SGI214146, SGI214148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214150, SGI214150, SGI0, SGI0, SGI0, SGI214153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47537[] = {SGI214147, SGI0, SGI214149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214151, SGI214152, SGI214152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47538[] = {SGI214155, SGI0, SGI214157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214159, SGI214160, SGI214160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47539[] = {SGI214154, SGI214156, SGI214158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214161, SGI214161, SGI0, SGI0, SGI0, SGI214162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47540[] = {SGI214163, SGI0, SGI214164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214165, SGI214166, SGI214166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47541[] = {SGI214167, SGI214168, SGI214169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214170, SGI214170, SGI0, SGI0, SGI0, SGI214171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47542[] = {SGI214172, SGI214173, SGI214174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214175, SGI214175, SGI0, SGI0, SGI0, SGI214176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47543[] = {SGI214177, SGI0, SGI214178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214179, SGI214180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47544[] = {SGI214181, SGI0, SGI214182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214183, SGI214184, SGI214184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47545[] = {SGI214185, SGI0, SGI214186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214187, SGI214188, SGI214188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47546[] = {SGI214189, SGI0, SGI214190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214191, SGI214192, SGI214192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47547[] = {SGI214193, SGI0, SGI214194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214195, SGI214196, SGI214196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47548[] = {SGI214197, SGI214198, SGI214199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214200, SGI0, SGI0, SGI0, SGI214201, SGI214201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47549[] = {SGI214202, SGI214203, SGI214204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214205, SGI214205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214206, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47550[] = {SGI214207, SGI214208, SGI214209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214210, SGI0, SGI0, SGI0, SGI214211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214212, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47551[] = {SGI214213, SGI0, SGI214214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214215, SGI214216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214217, SGI0, SGI0, NULL};
const Segment * const SG47552[] = {SGI214220, SGI0, SGI214221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214224, SGI214225, SGI214225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47553[] = {SGI214218, SGI0, SGI214219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214222, SGI214223, SGI214223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214226, SGI0, SGI0, NULL};
const Segment * const SG47554[] = {SGI214227, SGI0, SGI214228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214230, SGI214232, SGI214232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47555[] = {SGI214229, SGI0, SGI214231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214233, SGI214234, SGI214234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47556[] = {SGI214235, SGI0, SGI214236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214237, SGI214238, SGI214238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47557[] = {SGI214239, SGI0, SGI214240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214241, SGI214242, SGI214242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47558[] = {SGI214243, SGI0, SGI214244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214247, SGI214249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47559[] = {SGI214245, SGI0, SGI214246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214248, SGI214250, SGI214250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47560[] = {SGI214251, SGI0, SGI214252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214253, SGI0, SGI0, SGI0, SGI214254, SGI214254, SGI0, SGI0, SGI0, SGI214255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47561[] = {SGI214256, SGI0, SGI214257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214258, SGI214259, SGI214259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47562[] = {SGI214260, SGI0, SGI214261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214262, SGI214263, SGI214263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47563[] = {SGI214264, SGI0, SGI214265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214266, SGI214267, SGI214267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47564[] = {SGI214268, SGI0, SGI214269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214270, SGI214271, SGI214271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47565[] = {SGI214272, SGI0, SGI214273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214274, SGI214275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47566[] = {SGI214276, SGI0, SGI214277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214278, SGI214279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47567[] = {SGI214280, SGI214281, SGI214282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214283, SGI0, SGI0, SGI0, SGI214284, SGI214284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47568[] = {SGI214285, SGI214286, SGI214287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214288, SGI0, SGI0, SGI0, SGI214290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47569[] = {SGI214289, SGI214291, SGI214292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214293, SGI0, SGI0, SGI0, SGI214294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47570[] = {SGI214295, SGI0, SGI214296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47571[] = {SGI214297, SGI214298, SGI214299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214303, SGI0, SGI0, SGI0, SGI214304, SGI214304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47572[] = {SGI214300, SGI214301, SGI214302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214305, SGI0, SGI0, SGI0, SGI214306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47573[] = {SGI214308, SGI214309, SGI214310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214312, SGI0, SGI0, SGI0, SGI214314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47574[] = {SGI214311, SGI0, SGI214313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214316, SGI214317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47575[] = {SGI214318, SGI0, SGI214319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214320, SGI214321, SGI214321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47576[] = {SGI214322, SGI214323, SGI214324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214325, SGI0, SGI0, SGI0, SGI214326, SGI214326, SGI0, SGI214327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47577[] = {SGI214328, SGI214329, SGI214330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214331, SGI0, SGI0, SGI0, SGI214332, SGI214332, SGI0, SGI214333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47578[] = {SGI214334, SGI214335, SGI214336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214337, SGI0, SGI0, SGI0, SGI214339, SGI0, SGI0, SGI214342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47579[] = {SGI214338, SGI214340, SGI214341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214343, SGI214343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214344, NULL};
const Segment * const SG47580[] = {SGI214345, SGI0, SGI214346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214347, SGI214348, SGI214348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47581[] = {SGI214349, SGI214350, SGI214351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214352, SGI0, SGI0, SGI0, SGI214353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47582[] = {SGI214354, SGI0, SGI214355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214356, SGI214357, SGI214357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47583[] = {SGI214358, SGI0, SGI214359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214360, SGI214361, SGI214361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47584[] = {SGI214362, SGI214363, SGI214364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214365, SGI0, SGI0, SGI0, SGI214366, SGI214366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47585[] = {SGI214367, SGI0, SGI214368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214369, SGI214370, SGI214370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47586[] = {SGI214371, SGI0, SGI214372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214373, SGI214374, SGI214374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214375, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47587[] = {SGI214376, SGI0, SGI214377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214378, SGI214379, SGI214379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47588[] = {SGI214380, SGI0, SGI214381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214382, SGI214383, SGI214383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47589[] = {SGI214384, SGI214385, SGI214386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214387, SGI0, SGI0, SGI0, SGI214388, SGI214388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214389, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47590[] = {SGI214390, SGI214391, SGI214392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214393, SGI214393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214394, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47591[] = {SGI214395, SGI214396, SGI214397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214400, SGI0, SGI0, SGI0, SGI214401, SGI214401, SGI0, SGI0, SGI214403, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47592[] = {SGI214398, SGI0, SGI214399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214402, SGI214404, SGI214404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214405, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47593[] = {SGI214406, SGI0, SGI214407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214411, SGI214412, SGI214412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47594[] = {SGI214408, SGI214409, SGI214410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214413, SGI0, SGI0, SGI0, SGI214414, SGI214414, SGI0, SGI214415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47595[] = {SGI214416, SGI214417, SGI214418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214419, SGI0, SGI0, SGI0, SGI214420, SGI214420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47596[] = {SGI214421, SGI0, SGI214422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214423, SGI214424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47597[] = {SGI214425, SGI0, SGI214426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214427, SGI214428, SGI214428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47598[] = {SGI214429, SGI0, SGI214430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214431, SGI214432, SGI214432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47599[] = {SGI214433, SGI214434, SGI214435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214436, SGI214436, SGI0, SGI0, SGI0, SGI214437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47600[] = {SGI214438, SGI0, SGI214439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214440, SGI214441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47601[] = {SGI214442, SGI214443, SGI214444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214445, SGI214445, SGI0, SGI0, SGI0, SGI214446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47602[] = {SGI214447, SGI0, SGI214448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214451, SGI214452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47603[] = {SGI214449, SGI0, SGI214450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214453, SGI214454, SGI214454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47604[] = {SGI214455, SGI214456, SGI214458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214460, SGI214460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214463, NULL};
const Segment * const SG47605[] = {SGI214457, SGI0, SGI214459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214461, SGI214462, SGI214462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47606[] = {SGI214464, SGI214465, SGI214467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214469, SGI214469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214472, NULL};
const Segment * const SG47607[] = {SGI214466, SGI0, SGI214468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214470, SGI214471, SGI214471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47608[] = {SGI214473, SGI0, SGI214474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214475, SGI214476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47609[] = {SGI214477, SGI0, SGI214478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214479, SGI214480, SGI214480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47610[] = {SGI214481, SGI214482, SGI214483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214484, SGI0, SGI0, SGI0, SGI214485, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47611[] = {SGI214486, SGI0, SGI214487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214488, SGI214489, SGI214489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47612[] = {SGI214490, SGI0, SGI214491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214492, SGI214493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47613[] = {SGI214494, SGI0, SGI214495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214498, SGI214499, SGI214499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47614[] = {SGI214496, SGI0, SGI214497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214500, SGI214501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47615[] = {SGI214502, SGI214503, SGI214504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214505, SGI0, SGI0, SGI0, SGI214506, SGI214506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214507, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47616[] = {SGI214508, SGI214509, SGI214510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214511, SGI0, SGI0, SGI0, SGI214512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47617[] = {SGI214513, SGI0, SGI214514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214515, SGI214516, SGI214516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47618[] = {SGI214517, SGI0, SGI214518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214519, SGI214520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214521, SGI0, SGI0, NULL};
const Segment * const SG47619[] = {SGI214522, SGI214523, SGI214524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214525, SGI0, SGI0, SGI0, SGI214526, SGI214526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214527, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47620[] = {SGI214528, SGI214529, SGI214530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214531, SGI0, SGI0, SGI0, SGI214532, SGI214532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214533, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47621[] = {SGI214534, SGI0, SGI214535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214536, SGI214537, SGI214537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47622[] = {SGI214538, SGI0, SGI214539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214540, SGI214541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47623[] = {SGI214542, SGI0, SGI214543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214544, SGI214545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47624[] = {SGI214546, SGI0, SGI214547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214548, SGI214549, SGI214549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47625[] = {SGI214550, SGI214551, SGI214552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214553, SGI0, SGI0, SGI0, SGI214554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47626[] = {SGI214556, SGI0, SGI214557, SGI0, SGI214559, SGI214559, SGI0, NULL};
const Segment * const SG47627[] = {SGI214558, SGI214560, SGI214561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214562, SGI0, SGI0, SGI0, SGI214563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47628[] = {SGI214564, SGI0, SGI214565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214566, SGI214567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47629[] = {SGI214568, SGI0, SGI214569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214570, SGI214572, SGI214572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47630[] = {SGI214571, SGI0, SGI214573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214574, SGI214575, SGI214575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47631[] = {SGI214576, SGI0, SGI214577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214578, SGI214579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47632[] = {SGI214580, SGI0, SGI214581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214582, SGI214583, SGI214583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47633[] = {SGI0, SGI214586, SGI0, SGI214587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47634[] = {SGI214584, SGI0, SGI214585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214588, SGI214589, SGI214589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47635[] = {SGI214590, SGI0, SGI214591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214592, SGI214593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47636[] = {SGI214594, SGI214595, SGI214597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214599, SGI0, SGI0, SGI0, SGI214600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214603, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47637[] = {SGI214596, SGI0, SGI214598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214601, SGI214602, SGI214602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47638[] = {SGI214604, SGI0, SGI214605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214606, SGI214607, SGI214607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47639[] = {SGI214608, SGI0, SGI214609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214610, SGI214611, SGI214611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47640[] = {SGI214612, SGI214613, SGI214615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214617, SGI0, SGI0, SGI0, SGI214618, SGI214618, SGI0, SGI214621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47641[] = {SGI214614, SGI0, SGI214616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214619, SGI214620, SGI214620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47642[] = {SGI214622, SGI0, SGI214623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214624, SGI214625, SGI214625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47643[] = {SGI214626, SGI0, SGI214627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214630, SGI0, SGI0, SGI0, SGI214631, SGI214631, SGI0, SGI0, SGI0, SGI0, SGI214634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47644[] = {SGI214628, SGI0, SGI214629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214632, SGI214633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47645[] = {SGI214635, SGI214636, SGI214637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214638, SGI0, SGI0, SGI0, SGI214639, SGI214639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47646[] = {SGI214640, SGI0, SGI214641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214642, SGI214643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47647[] = {SGI214644, SGI0, SGI214645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214646, SGI214647, SGI214647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47648[] = {SGI214648, SGI214649, SGI214650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214651, SGI0, SGI0, SGI0, SGI214652, SGI214652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47649[] = {SGI214653, SGI0, SGI214654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214657, SGI214659, SGI214659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47650[] = {SGI214655, SGI214656, SGI214658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214660, SGI214660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47651[] = {SGI214661, SGI0, SGI214662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214666, SGI214667, SGI214667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47652[] = {SGI214663, SGI214664, SGI214665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214668, SGI0, SGI0, SGI0, SGI214669, SGI214669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47653[] = {SGI214670, SGI214671, SGI214672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214673, SGI214673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214674, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47654[] = {SGI214675, SGI0, SGI214676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214677, SGI214678, SGI214678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47655[] = {SGI214679, SGI0, SGI214680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214681, SGI214682, SGI214682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47656[] = {SGI214683, SGI0, SGI214684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214685, SGI214686, SGI214686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47657[] = {SGI214687, SGI0, SGI214688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214689, SGI214690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47658[] = {SGI214691, SGI0, SGI214692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214693, SGI214694, SGI214694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214695, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47659[] = {SGI214696, SGI214697, SGI214698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214699, SGI214699, SGI0, SGI0, SGI0, SGI0, SGI214700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47660[] = {SGI214701, SGI0, SGI214702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214703, SGI214704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47661[] = {SGI214705, SGI214706, SGI214707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214711, SGI0, SGI0, SGI0, SGI214712, SGI214712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47662[] = {SGI214708, SGI214709, SGI214710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214713, SGI214713, SGI0, SGI0, SGI0, SGI0, SGI214714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47663[] = {SGI214715, SGI214716, SGI214717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214718, SGI0, SGI0, SGI0, SGI214719, SGI214719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47664[] = {SGI214720, SGI214721, SGI214722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214723, SGI214723, SGI0, SGI0, SGI0, SGI214724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47665[] = {SGI214725, SGI214726, SGI214727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214728, SGI214728, SGI0, SGI0, SGI0, SGI214729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47666[] = {SGI214730, SGI214731, SGI214732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214733, SGI0, SGI0, SGI0, SGI214734, SGI214734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47667[] = {SGI214735, SGI0, SGI214736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214737, SGI214738, SGI214738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47668[] = {SGI214739, SGI0, SGI214740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214741, SGI214742, SGI214742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47669[] = {SGI214743, SGI0, SGI214744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214745, SGI214746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47670[] = {SGI214747, SGI214748, SGI214749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214750, SGI214750, SGI0, SGI0, SGI0, SGI214751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47671[] = {SGI214752, SGI214753, SGI214754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214755, SGI214755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214756, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47672[] = {SGI214757, SGI0, SGI214759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214762, SGI214764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47673[] = {SGI214758, SGI0, SGI214760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214761, SGI214763, SGI214763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47674[] = {SGI214765, SGI214766, SGI214767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214768, SGI214768, SGI0, SGI0, SGI0, SGI214769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47675[] = {SGI214770, SGI214771, SGI214772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214773, SGI214773, SGI0, SGI0, SGI0, SGI214774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47676[] = {SGI214775, SGI214776, SGI214777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214778, SGI214778, SGI0, SGI0, SGI0, SGI214779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47677[] = {SGI214780, SGI0, SGI214781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214782, SGI214783, SGI214783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47678[] = {SGI214784, SGI214785, SGI214786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214787, SGI0, SGI0, SGI0, SGI214788, SGI214788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47679[] = {SGI214789, SGI0, SGI214790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47680[] = {SGI214791, SGI214792, SGI214793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214794, SGI0, SGI0, SGI0, SGI214795, SGI214795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47681[] = {SGI214796, SGI214797, SGI214798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214799, SGI0, SGI0, SGI0, SGI214800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47682[] = {SGI214801, SGI0, SGI214802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214803, SGI214804, SGI214804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47683[] = {SGI214805, SGI214806, SGI214807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214808, SGI0, SGI0, SGI0, SGI214809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47684[] = {SGI214810, SGI214811, SGI214812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214813, SGI0, SGI0, SGI0, SGI214814, SGI214814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214815, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47685[] = {SGI214816, SGI0, SGI214817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214818, SGI0, SGI0, SGI0, SGI214819, SGI214819, SGI0, SGI0, SGI0, SGI214820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47686[] = {SGI214821, SGI0, SGI214822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214823, SGI0, SGI0, SGI0, SGI214824, SGI214824, SGI0, SGI0, SGI0, SGI214825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47687[] = {SGI214826, SGI0, SGI214827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214828, SGI0, SGI0, SGI0, SGI214829, SGI214829, SGI0, SGI0, SGI0, SGI214830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47688[] = {SGI214831, SGI0, SGI214832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214833, SGI214834, SGI214834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47689[] = {SGI214835, SGI0, SGI214836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214837, SGI214838, SGI214838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47690[] = {SGI214839, SGI0, SGI214840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214841, SGI214842, SGI214842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47691[] = {SGI214843, SGI0, SGI214844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214845, SGI214847, SGI214847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47692[] = {SGI214846, SGI214848, SGI214849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214850, SGI0, SGI0, SGI0, SGI214851, SGI214851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214852, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47693[] = {SGI214853, SGI214854, SGI214855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214856, SGI0, SGI0, SGI0, SGI214857, SGI214857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47694[] = {SGI214858, SGI0, SGI214859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214860, SGI214861, SGI214861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47695[] = {SGI214862, SGI214863, SGI214864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214865, SGI214865, SGI0, SGI0, SGI0, SGI0, SGI214867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47696[] = {SGI214866, SGI0, SGI214868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214869, SGI214870, SGI214870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47697[] = {SGI214871, SGI214872, SGI214873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214874, SGI214874, SGI0, SGI0, SGI0, SGI214875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47698[] = {SGI214876, SGI0, SGI214877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214878, SGI214879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47699[] = {SGI214880, SGI0, SGI214881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214882, SGI214883, SGI214883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47700[] = {SGI214884, SGI214885, SGI214886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214887, SGI0, SGI0, SGI0, SGI214888, SGI214888, SGI214889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47701[] = {SGI214890, SGI214891, SGI214892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214893, SGI214893, SGI0, SGI0, SGI0, SGI214894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47702[] = {SGI214895, SGI214896, SGI214897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214901, SGI214901, SGI0, SGI0, SGI0, SGI214903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47703[] = {SGI214898, SGI214899, SGI214900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214902, SGI0, SGI0, SGI0, SGI214904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47704[] = {SGI214905, SGI214906, SGI214907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214908, SGI0, SGI0, SGI0, SGI214909, SGI214909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47705[] = {SGI214910, SGI214911, SGI214912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214915, SGI214915, SGI0, SGI0, SGI0, SGI214916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47706[] = {SGI214913, SGI0, SGI214914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214917, SGI214918, SGI214918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47707[] = {SGI214919, SGI214920, SGI214921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214922, SGI0, SGI0, SGI0, SGI214923, SGI214923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47708[] = {SGI214924, SGI214925, SGI214926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214927, SGI214927, SGI0, SGI0, SGI0, SGI214928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47709[] = {SGI214929, SGI214930, SGI214931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214932, SGI214932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214933, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47710[] = {SGI214934, SGI214935, SGI214936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214937, SGI214937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214938, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47711[] = {SGI214939, SGI0, SGI214940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214941, SGI214942, SGI214942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47712[] = {SGI214944, SGI0, SGI214946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214949, SGI214951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47713[] = {SGI214943, SGI0, SGI214947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214950, SGI214952, SGI214952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47714[] = {SGI214945, SGI0, SGI214948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214953, SGI214954, SGI214954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214955, SGI0, SGI0, NULL};
const Segment * const SG47715[] = {SGI214956, SGI0, SGI214957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214958, SGI214959, SGI214959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214960, SGI0, SGI0, NULL};
const Segment * const SG47716[] = {SGI214961, SGI214963, SGI214964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214966, SGI214966, SGI0, SGI0, SGI0, SGI214969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47717[] = {SGI214962, SGI0, SGI214965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214967, SGI214968, SGI214968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214970, SGI0, SGI0, NULL};
const Segment * const SG47718[] = {SGI214971, SGI214972, SGI214973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214974, SGI214974, SGI0, SGI0, SGI214975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47719[] = {SGI214976, SGI0, SGI214977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214978, SGI214979, SGI214979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47720[] = {SGI214980, SGI0, SGI214981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214982, SGI214983, SGI214983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214984, SGI0, SGI0, NULL};
const Segment * const SG47721[] = {SGI214985, SGI0, SGI214986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214987, SGI214988, SGI214988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47722[] = {SGI214989, SGI0, SGI214990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214991, SGI214992, SGI214992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214996, SGI0, SGI0, NULL};
const Segment * const SG47723[] = {SGI214993, SGI214994, SGI214995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI214997, SGI214997, SGI0, SGI0, SGI0, SGI0, SGI214998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47724[] = {SGI214999, SGI0, SGI215000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215001, SGI215002, SGI215002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47725[] = {SGI215003, SGI0, SGI215004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215007, SGI215008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47726[] = {SGI215005, SGI0, SGI215006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215009, SGI215010, SGI215010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215011, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47727[] = {SGI215012, SGI0, SGI215013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215014, SGI215015, SGI215015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47728[] = {SGI215016, SGI0, SGI215017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215018, SGI215019, SGI215019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47729[] = {SGI215020, SGI0, SGI215021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215022, SGI215023, SGI215023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47730[] = {SGI215024, SGI0, SGI215025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215026, SGI215027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47731[] = {SGI215028, SGI215029, SGI215030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215031, SGI0, SGI0, SGI0, SGI215032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47732[] = {SGI215033, SGI0, SGI215034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215035, SGI215036, SGI215036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215037, SGI0, SGI0, NULL};
const Segment * const SG47733[] = {SGI215038, SGI0, SGI215039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215040, SGI215041, SGI215041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47734[] = {SGI215042, SGI0, SGI215043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215044, SGI215045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47735[] = {SGI215046, SGI0, SGI215047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215048, SGI215049, SGI215049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47736[] = {SGI215050, SGI215051, SGI215052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215053, SGI0, SGI0, SGI0, SGI215054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47737[] = {SGI215055, SGI0, SGI215056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215057, SGI215058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47738[] = {SGI215059, SGI215060, SGI215061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215062, SGI0, SGI0, SGI0, SGI215063, SGI215063, SGI0, SGI215064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47739[] = {SGI215065, SGI215066, SGI215067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215068, SGI0, SGI0, SGI0, SGI215069, SGI215069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47740[] = {SGI215070, SGI0, SGI215071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215072, SGI215073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47741[] = {SGI215074, SGI0, SGI215075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215076, SGI215077, SGI215077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47742[] = {SGI215078, SGI215079, SGI215080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215081, SGI0, SGI0, SGI0, SGI215082, SGI215082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47743[] = {SGI215083, SGI0, SGI215084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215085, SGI215086, SGI215086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47744[] = {SGI215087, SGI0, SGI215088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215089, SGI215090, SGI215090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47745[] = {SGI215091, SGI0, SGI215092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215093, SGI215094, SGI215094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47746[] = {SGI215095, SGI0, SGI215096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215097, SGI0, SGI0, SGI0, SGI215098, SGI215098, SGI0, SGI0, SGI215099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47747[] = {SGI215100, SGI215101, SGI215102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215103, SGI0, SGI0, SGI0, SGI215104, SGI215104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215105, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47748[] = {SGI215106, SGI215107, SGI215108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215109, SGI0, SGI0, SGI0, SGI215110, SGI215110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47749[] = {SGI215111, SGI215112, SGI215113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215114, SGI0, SGI0, SGI0, SGI215115, SGI215115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215116, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47750[] = {SGI215117, SGI0, SGI215118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215119, SGI215120, SGI215120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47751[] = {SGI215121, SGI0, SGI215123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215125, SGI215126, SGI215126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47752[] = {SGI215122, SGI0, SGI215124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215127, SGI215128, SGI215128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47753[] = {SGI215129, SGI215130, SGI215131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215132, SGI0, SGI0, SGI0, SGI215133, SGI215133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47754[] = {SGI215134, SGI0, SGI215135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215136, SGI215137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47755[] = {SGI215138, SGI0, SGI215139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215140, SGI215141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47756[] = {SGI215142, SGI0, SGI215143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215144, SGI215145, SGI215145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47757[] = {SGI215146, SGI0, SGI215147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215148, SGI215149, SGI215149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47758[] = {SGI215150, SGI0, SGI215151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215152, SGI215153, SGI215153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47759[] = {SGI215154, SGI0, SGI215155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215156, SGI215157, SGI215157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47760[] = {SGI215158, SGI0, SGI215159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215160, SGI215161, SGI215161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47761[] = {SGI215162, SGI0, SGI215163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215164, SGI215165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47762[] = {SGI215166, SGI215167, SGI215168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215169, SGI0, SGI0, SGI0, SGI215170, SGI215170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47763[] = {SGI215171, SGI0, SGI215172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215173, SGI0, SGI0, SGI0, SGI215174, SGI215174, SGI0, SGI0, SGI0, SGI215175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47764[] = {SGI215176, SGI215177, SGI215178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215179, SGI215179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215180, NULL};
const Segment * const SG47765[] = {SGI215181, SGI215182, SGI215183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215184, SGI0, SGI0, SGI0, SGI215185, SGI215185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47766[] = {SGI215186, SGI215187, SGI215188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215189, SGI0, SGI0, SGI0, SGI215190, SGI215190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47767[] = {SGI215191, SGI0, SGI215192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215193, SGI215194, SGI215194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47768[] = {SGI215195, SGI215196, SGI215197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215198, SGI0, SGI0, SGI0, SGI215200, SGI215200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47769[] = {SGI215199, SGI0, SGI215201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215202, SGI215203, SGI215203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47770[] = {SGI215204, SGI215205, SGI215206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215207, SGI0, SGI0, SGI0, SGI215208, SGI215208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47771[] = {SGI215209, SGI0, SGI215210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215211, SGI215212, SGI215212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47772[] = {SGI215213, SGI215214, SGI215215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215216, SGI215216, SGI0, SGI0, SGI0, SGI0, SGI215217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47773[] = {SGI215218, SGI0, SGI215219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215220, SGI215221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47774[] = {SGI215222, SGI0, SGI215223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215224, SGI215225, SGI215225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215226, SGI0, SGI0, NULL};
const Segment * const SG47775[] = {SGI215227, SGI0, SGI215228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215229, SGI215230, SGI215230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47776[] = {SGI215231, SGI0, SGI215232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215235, SGI215236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47777[] = {SGI215233, SGI0, SGI215234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215237, SGI215238, SGI215238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47778[] = {SGI215239, SGI0, SGI215240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215243, SGI215244, SGI215244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47779[] = {SGI215241, SGI0, SGI215242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215245, SGI215246, SGI215246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47780[] = {SGI215247, SGI0, SGI215248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215249, SGI215250, SGI215250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47781[] = {SGI215251, SGI215252, SGI215253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215254, SGI0, SGI0, SGI0, SGI215255, SGI215255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47782[] = {SGI215256, SGI0, SGI215257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215258, SGI215260, SGI215260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47783[] = {SGI215259, SGI0, SGI215261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47784[] = {SGI215262, SGI215263, SGI215264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215265, SGI0, SGI0, SGI0, SGI215266, SGI215266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47785[] = {SGI215267, SGI215268, SGI215269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215270, SGI0, SGI0, SGI0, SGI215271, SGI215271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47786[] = {SGI215272, SGI0, SGI215273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215274, SGI215275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47787[] = {SGI215276, SGI0, SGI215277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215278, SGI215279, SGI215279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47788[] = {SGI215280, SGI0, SGI215281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215282, SGI215284, SGI215284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47789[] = {SGI215283, SGI0, SGI215285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215286, SGI215287, SGI215287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47790[] = {SGI215288, SGI0, SGI215289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215290, SGI215291, SGI215291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47791[] = {SGI215292, SGI0, SGI215293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215297, SGI215298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47792[] = {SGI215294, SGI215295, SGI215296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215299, SGI0, SGI0, SGI0, SGI215300, SGI215300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215301, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47793[] = {SGI215302, SGI0, SGI215303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215304, SGI215305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47794[] = {SGI215306, SGI215307, SGI215308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215309, SGI0, SGI0, SGI0, SGI215310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47795[] = {SGI215311, SGI215312, SGI215313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215314, SGI0, SGI0, SGI0, SGI215315, SGI215315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47796[] = {SGI215316, SGI215317, SGI215318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215319, SGI215319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215320, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47797[] = {SGI215321, SGI0, SGI215322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215323, SGI215324, SGI215324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47798[] = {SGI215325, SGI0, SGI215326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215327, SGI215328, SGI215328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47799[] = {SGI215329, SGI0, SGI215330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215331, SGI215332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47800[] = {SGI215333, SGI215334, SGI215335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215336, SGI215336, SGI0, SGI0, SGI0, SGI215337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47801[] = {SGI215338, SGI0, SGI215339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215341, SGI215343, SGI215343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47802[] = {SGI215340, SGI215342, SGI215344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215345, SGI215345, SGI0, SGI0, SGI0, SGI215346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47803[] = {SGI215347, SGI0, SGI215348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215349, SGI215350, SGI215350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47804[] = {SGI215351, SGI215352, SGI215353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215354, SGI215354, SGI0, SGI0, SGI0, SGI215355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47805[] = {SGI215356, SGI0, SGI215358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215360, SGI215362, SGI215362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47806[] = {SGI215357, SGI0, SGI215359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215361, SGI215363, SGI215363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47807[] = {SGI215364, SGI215365, SGI215367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215369, SGI215369, SGI0, SGI0, SGI0, SGI215372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47808[] = {SGI215366, SGI0, SGI215368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215370, SGI215371, SGI215371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47809[] = {SGI215373, SGI0, SGI215374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215375, SGI215376, SGI215376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47810[] = {SGI215377, SGI0, SGI215378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215379, SGI215380, SGI215380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47811[] = {SGI215381, SGI0, SGI215382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215383, SGI215384, SGI215384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47812[] = {SGI215385, SGI215386, SGI215387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215388, SGI215388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215389, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47813[] = {SGI215390, SGI215391, SGI215392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215393, SGI0, SGI0, SGI0, SGI215394, SGI215394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215396, SGI0, NULL};
const Segment * const SG47814[] = {SGI215397, SGI215398, SGI215399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215400, SGI0, SGI0, SGI0, SGI215401, SGI215401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47815[] = {SGI215402, SGI215403, SGI215404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215405, SGI0, SGI0, SGI0, SGI215406, SGI215406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215411, SGI0, NULL};
const Segment * const SG47816[] = {SGI215407, SGI215408, SGI215410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215412, SGI0, SGI0, SGI0, SGI215413, SGI215413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47817[] = {SGI215414, SGI0, SGI215415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215416, SGI215417, SGI215417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47818[] = {SGI215418, SGI0, SGI215419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215420, SGI215421, SGI215421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47819[] = {SGI215422, SGI0, SGI215423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215425, SGI215427, SGI215427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47820[] = {SGI215424, SGI215426, SGI215428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215429, SGI0, SGI0, SGI0, SGI215430, SGI215430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47821[] = {SGI215431, SGI0, SGI215432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215433, SGI215435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47822[] = {SGI215434, SGI0, SGI215436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215439, SGI215440, SGI215440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47823[] = {SGI215437, SGI0, SGI215438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215441, SGI215442, SGI215442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47824[] = {SGI215443, SGI0, SGI215444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215445, SGI215446, SGI215446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47825[] = {SGI215447, SGI0, SGI215448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215449, SGI215450, SGI215450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47826[] = {SGI215451, SGI0, SGI215452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215453, SGI215454, SGI215454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47827[] = {SGI215455, SGI0, SGI215456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215457, SGI215458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47828[] = {SGI215459, SGI0, SGI215460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215461, SGI215462, SGI215462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47829[] = {SGI215463, SGI215464, SGI215465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215466, SGI0, SGI0, SGI0, SGI215467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47830[] = {SGI215468, SGI215469, SGI215470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215471, SGI0, SGI0, SGI0, SGI215472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47831[] = {SGI215473, SGI0, SGI215474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215475, SGI215476, SGI215476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47832[] = {SGI215477, SGI0, SGI215478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215479, SGI215480, SGI215480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47833[] = {SGI215481, SGI215482, SGI215483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215484, SGI215484, SGI0, SGI0, SGI215485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47834[] = {SGI215486, SGI0, SGI215487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215488, SGI0, SGI0, SGI0, SGI215489, SGI215489, SGI0, SGI0, SGI0, SGI215490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47835[] = {SGI215491, SGI215492, SGI215493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215494, SGI0, SGI0, SGI0, SGI215495, SGI215495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47836[] = {SGI215496, SGI215497, SGI215498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215499, SGI0, SGI0, SGI0, SGI215500, SGI215500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47837[] = {SGI215501, SGI0, SGI215502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215503, SGI215504, SGI215504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47838[] = {SGI215505, SGI215506, SGI215507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215508, SGI0, SGI0, SGI0, SGI215509, SGI215509, SGI0, SGI215510, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47839[] = {SGI215511, SGI0, SGI215512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215513, SGI215514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47840[] = {SGI215515, SGI0, SGI215516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215517, SGI215518, SGI215518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47841[] = {SGI215519, SGI0, SGI215520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215521, SGI215522, SGI215522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47842[] = {SGI215523, SGI215525, SGI215526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215528, SGI0, SGI0, SGI0, SGI215530, SGI215530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47843[] = {SGI215524, SGI0, SGI215527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215529, SGI215531, SGI215531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47844[] = {SGI215532, SGI0, SGI215533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215534, SGI215535, SGI215535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47845[] = {SGI215536, SGI0, SGI215537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215538, SGI215539, SGI215539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47846[] = {SGI215540, SGI215541, SGI215542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215543, SGI0, SGI0, SGI0, SGI215544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47847[] = {SGI215545, SGI0, SGI215546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215547, SGI215548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47848[] = {SGI215549, SGI0, SGI215550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215551, SGI215552, SGI215552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47849[] = {SGI215553, SGI0, SGI215554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215555, SGI215556, SGI215556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215557, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47850[] = {SGI215558, SGI215559, SGI215560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215564, SGI0, SGI0, SGI0, SGI215565, SGI215565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215567, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47851[] = {SGI215561, SGI215562, SGI215563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215566, SGI0, SGI0, SGI0, SGI215568, SGI215568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47852[] = {SGI215569, SGI0, SGI215570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47853[] = {SGI215571, SGI215572, SGI215573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215577, SGI0, SGI0, SGI0, SGI215578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47854[] = {SGI215574, SGI215575, SGI215576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215579, SGI0, SGI0, SGI0, SGI215581, SGI215581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47855[] = {SGI215580, SGI215582, SGI215583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215585, SGI215585, SGI0, SGI0, SGI0, SGI215587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47856[] = {SGI215584, SGI0, SGI215586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47857[] = {SGI215588, SGI0, SGI215590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215594, SGI215596, SGI215596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47858[] = {SGI215589, SGI215591, SGI215592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215597, SGI215597, SGI0, SGI0, SGI0, SGI215598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47859[] = {SGI215593, SGI0, SGI215595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215599, SGI215600, SGI215600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47860[] = {SGI215601, SGI215602, SGI215603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215606, SGI215606, SGI0, SGI0, SGI0, SGI215607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47861[] = {SGI215604, SGI0, SGI215605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215608, SGI215609, SGI215609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47862[] = {SGI215610, SGI0, SGI215611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215612, SGI215613, SGI215613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47863[] = {SGI215615, SGI0, SGI215617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215618, SGI215620, SGI215620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47864[] = {SGI215614, SGI0, SGI215616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215619, SGI215621, SGI215621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47865[] = {SGI215622, SGI215623, SGI215624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215625, SGI215625, SGI0, SGI0, SGI0, SGI215626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47866[] = {SGI215627, SGI0, SGI215628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215629, SGI215630, SGI215630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47867[] = {SGI215631, SGI215632, SGI215634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215637, SGI215637, SGI0, SGI0, SGI0, SGI0, SGI215640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47868[] = {SGI215633, SGI215635, SGI215636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215639, SGI215639, SGI0, SGI0, SGI0, SGI215642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47869[] = {SGI215638, SGI0, SGI215641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215643, SGI215644, SGI215644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47870[] = {SGI215645, SGI0, SGI215646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215647, SGI215648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47871[] = {SGI215649, SGI0, SGI215650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215654, SGI215655, SGI215655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47872[] = {SGI215651, SGI215652, SGI215653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215656, SGI215656, SGI0, SGI0, SGI0, SGI215657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47873[] = {SGI215658, SGI0, SGI215659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215661, SGI215663, SGI215663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47874[] = {SGI215660, SGI0, SGI215662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215664, SGI0, SGI0, SGI0, SGI215665, SGI215665, SGI0, SGI0, SGI0, SGI0, SGI215666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47875[] = {SGI215667, SGI0, SGI215668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215669, SGI215670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47876[] = {SGI215671, SGI215672, SGI215673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215674, SGI0, SGI0, SGI0, SGI215675, SGI215676, SGI0, SGI0, SGI0, SGI0, SGI215677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47877[] = {SGI215678, SGI0, SGI215679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215680, SGI215681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47878[] = {SGI215682, SGI0, SGI215683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215684, SGI215685, SGI215685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47879[] = {SGI215686, SGI0, SGI215687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215688, SGI215689, SGI215689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47880[] = {SGI215690, SGI215691, SGI215692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215693, SGI0, SGI0, SGI0, SGI215694, SGI215694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215695, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47881[] = {SGI215696, SGI0, SGI215697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215698, SGI215699, SGI215699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47882[] = {SGI215700, SGI215701, SGI215702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215703, SGI0, SGI0, SGI0, SGI215704, SGI215704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47883[] = {SGI215705, SGI0, SGI215706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47884[] = {SGI215707, SGI0, SGI215708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215709, SGI215710, SGI215710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47885[] = {SGI0, SGI0, SGI215711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215715, SGI215716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47886[] = {SGI215712, SGI215713, SGI215714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215717, SGI0, SGI0, SGI0, SGI215718, SGI215718, SGI0, SGI215719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47887[] = {SGI215720, SGI215721, SGI215722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215724, SGI0, SGI0, SGI0, SGI215726, SGI215726, SGI0, SGI215727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47888[] = {SGI215723, SGI0, SGI215725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215728, SGI215729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47889[] = {SGI215730, SGI0, SGI215731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215732, SGI215733, SGI215733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47890[] = {SGI215734, SGI0, SGI215735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215736, SGI215737, SGI215737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47891[] = {SGI215738, SGI0, SGI215739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215740, SGI215741, SGI215741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47892[] = {SGI215742, SGI0, SGI215743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215744, SGI215745, SGI215745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47893[] = {SGI215746, SGI0, SGI215747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215750, SGI215752, SGI215752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47894[] = {SGI215748, SGI215749, SGI215751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215753, SGI0, SGI0, SGI0, SGI215754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47895[] = {SGI215755, SGI0, SGI215756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215757, SGI215758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47896[] = {SGI215761, SGI0, SGI215763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215765, SGI215766, SGI215766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47897[] = {SGI215759, SGI215760, SGI215762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215764, SGI0, SGI0, SGI0, SGI215767, SGI0, SGI215768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47898[] = {SGI215769, SGI215770, SGI215771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215772, SGI0, SGI0, SGI0, SGI215773, SGI215773, SGI215774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47899[] = {SGI215775, SGI215776, SGI215777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215778, SGI0, SGI0, SGI0, SGI215779, SGI0, SGI0, SGI0, SGI215780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47900[] = {SGI215781, SGI215782, SGI215783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215784, SGI0, SGI0, SGI0, SGI215785, SGI0, SGI0, SGI0, SGI215786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47901[] = {SGI215787, SGI215788, SGI215789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215790, SGI0, SGI0, SGI0, SGI215791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215792, SGI0, SGI0, NULL};
const Segment * const SG47902[] = {SGI215793, SGI215794, SGI215795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215796, SGI215796, SGI0, SGI0, SGI0, SGI0, SGI215797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47903[] = {SGI215798, SGI0, SGI215799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215800, SGI215801, SGI215801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47904[] = {SGI215802, SGI0, SGI215803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215804, SGI215805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47905[] = {SGI215806, SGI0, SGI215807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215808, SGI215809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47906[] = {SGI215810, SGI0, SGI215811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215812, SGI215813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47907[] = {SGI215814, SGI215815, SGI215816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215817, SGI215817, SGI0, SGI0, SGI0, SGI215818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47908[] = {SGI215819, SGI215820, SGI215821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215822, SGI215822, SGI0, SGI0, SGI0, SGI215823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47909[] = {SGI215824, SGI0, SGI215825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215826, SGI215827, SGI215827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47910[] = {SGI215828, SGI0, SGI215829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215830, SGI215831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47911[] = {SGI215832, SGI215833, SGI215834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215835, SGI0, SGI0, SGI0, SGI215836, SGI215836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47912[] = {SGI215838, SGI0, SGI215839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215840, SGI215841, SGI215841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215842, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47913[] = {SGI215843, SGI0, SGI215844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215845, SGI215846, SGI215846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47914[] = {SGI215847, SGI215848, SGI215849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215851, SGI0, SGI0, SGI0, SGI215853, SGI215853, SGI0, SGI215854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47915[] = {SGI215850, SGI0, SGI215852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215855, SGI215856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47916[] = {SGI215857, SGI215858, SGI215859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215860, SGI0, SGI0, SGI0, SGI215861, SGI215861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47917[] = {SGI215862, SGI215863, SGI215864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215865, SGI0, SGI0, SGI0, SGI215866, SGI215866, SGI0, SGI215867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47918[] = {SGI215868, SGI0, SGI215869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215870, SGI215871, SGI215871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47919[] = {SGI215872, SGI215873, SGI215874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215875, SGI0, SGI0, SGI0, SGI215876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47920[] = {SGI215877, SGI215878, SGI215879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215880, SGI0, SGI0, SGI0, SGI215881, SGI215881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47921[] = {SGI215882, SGI215883, SGI215884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215888, SGI0, SGI0, SGI0, SGI215889, SGI215889, SGI0, SGI0, SGI0, SGI215891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47922[] = {SGI215885, SGI215886, SGI215887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215890, SGI0, SGI0, SGI0, SGI215892, SGI215892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47923[] = {SGI215893, SGI215894, SGI215895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215897, SGI0, SGI0, SGI0, SGI215899, SGI215899, SGI0, SGI0, SGI0, SGI215900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47924[] = {SGI215896, SGI0, SGI215898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215901, SGI215903, SGI215903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47925[] = {SGI215902, SGI215904, SGI215905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215906, SGI0, SGI0, SGI0, SGI215907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47926[] = {SGI215908, SGI215909, SGI215910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215911, SGI0, SGI0, SGI0, SGI215912, SGI215912, SGI0, SGI0, SGI0, SGI215913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47927[] = {SGI215914, SGI215915, SGI215916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215917, SGI215917, SGI0, SGI0, SGI215918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47928[] = {SGI215919, SGI0, SGI215920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215921, SGI215922, SGI215922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47929[] = {SGI215923, SGI0, SGI215924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215928, SGI215929, SGI215929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47930[] = {SGI215925, SGI215926, SGI215927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215930, SGI215930, SGI0, SGI0, SGI0, SGI215931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47931[] = {SGI215932, SGI0, SGI215933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215937, SGI215938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47932[] = {SGI215934, SGI215935, SGI215936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215939, SGI215939, SGI0, SGI0, SGI0, SGI215940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47933[] = {SGI215941, SGI0, SGI215942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215943, SGI215944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47934[] = {SGI215945, SGI0, SGI215946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215947, SGI215948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47935[] = {SGI215949, SGI215950, SGI215951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215952, SGI215952, SGI0, SGI0, SGI0, SGI215953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47936[] = {SGI215954, SGI0, SGI215955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215956, SGI215957, SGI215957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47937[] = {SGI215958, SGI0, SGI215959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215960, SGI215961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47938[] = {SGI215962, SGI215963, SGI215964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215965, SGI0, SGI0, SGI0, SGI215966, SGI215966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47939[] = {SGI215967, SGI215968, SGI215969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215970, SGI0, SGI0, SGI0, SGI215971, SGI215971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47940[] = {SGI215972, SGI215973, SGI215974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215975, SGI0, SGI0, SGI0, SGI215976, SGI215976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47941[] = {SGI215977, SGI0, SGI215978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47942[] = {SGI215979, SGI215980, SGI215981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215982, SGI0, SGI0, SGI0, SGI215983, SGI215983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47943[] = {SGI215984, SGI215985, SGI215986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215987, SGI0, SGI0, SGI0, SGI215988, SGI215988, SGI0, SGI215989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47944[] = {SGI215990, SGI215991, SGI215993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215996, SGI0, SGI0, SGI0, SGI215997, SGI215997, SGI0, SGI215999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47945[] = {SGI215992, SGI215994, SGI215995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI215998, SGI0, SGI0, SGI0, SGI216000, SGI216000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47946[] = {SGI216002, SGI216003, SGI216004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216005, SGI0, SGI0, SGI0, SGI216006, SGI216006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47947[] = {SGI216008, SGI0, SGI216009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216010, SGI216011, SGI216011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47948[] = {SGI216012, SGI216013, SGI216014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216015, SGI0, SGI0, SGI0, SGI216016, SGI216016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47949[] = {SGI216018, SGI0, SGI216019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216021, SGI216022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47950[] = {SGI216020, SGI0, SGI216023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216024, SGI216025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47951[] = {SGI216026, SGI0, SGI216027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216028, SGI216029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47952[] = {SGI216030, SGI0, SGI216031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216032, SGI216033, SGI216033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47953[] = {SGI216034, SGI0, SGI216035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216036, SGI216037, SGI216037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47954[] = {SGI216038, SGI216039, SGI216040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216041, SGI0, SGI0, SGI0, SGI216042, SGI216042, SGI0, SGI216043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47955[] = {SGI216044, SGI216045, SGI216046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216047, SGI0, SGI0, SGI0, SGI216049, SGI216049, SGI0, SGI216051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47956[] = {SGI216048, SGI0, SGI216050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216052, SGI216053, SGI216053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47957[] = {SGI216054, SGI0, SGI216055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216056, SGI216057, SGI216057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47958[] = {SGI216058, SGI216059, SGI216060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216061, SGI0, SGI0, SGI0, SGI216062, SGI216062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216063, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47959[] = {SGI216064, SGI216065, SGI216066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216067, SGI0, SGI0, SGI0, SGI216068, SGI216068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47960[] = {SGI216070, SGI216071, SGI216072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216073, SGI0, SGI0, SGI0, SGI216074, SGI216074, SGI0, SGI0, SGI0, SGI216075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47961[] = {SGI216076, SGI216077, SGI216078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216080, SGI0, SGI0, SGI0, SGI216082, SGI216082, SGI0, SGI0, SGI0, SGI216083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47962[] = {SGI216079, SGI0, SGI216081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216084, SGI216085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47963[] = {SGI216086, SGI0, SGI216087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216089, SGI216091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47964[] = {SGI216088, SGI216090, SGI216092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216093, SGI0, SGI0, SGI0, SGI216094, SGI216094, SGI0, SGI0, SGI0, SGI216095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47965[] = {SGI216096, SGI0, SGI216097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216100, SGI0, SGI0, SGI0, SGI216103, SGI216103, SGI0, SGI0, SGI0, SGI216105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47966[] = {SGI216098, SGI216099, SGI216101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216106, SGI216106, SGI0, SGI0, SGI0, SGI0, SGI216108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47967[] = {SGI216102, SGI0, SGI216104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216107, SGI216109, SGI216109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216110, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47968[] = {SGI216111, SGI0, SGI216112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216113, SGI216115, SGI216115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47969[] = {SGI216114, SGI216116, SGI216117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216118, SGI0, SGI0, SGI0, SGI216119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47970[] = {SGI216120, SGI0, SGI216121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216122, SGI0, SGI0, SGI0, SGI216123, SGI216123, SGI0, SGI0, SGI0, SGI216124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47971[] = {SGI216125, SGI216126, SGI216127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216128, SGI0, SGI0, SGI0, SGI216129, SGI216129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216130, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47972[] = {SGI216131, SGI0, SGI216132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216136, SGI216137, SGI216137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47973[] = {SGI216133, SGI216134, SGI216135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216138, SGI0, SGI0, SGI0, SGI216139, SGI216139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216140, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47974[] = {SGI216141, SGI0, SGI216142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216145, SGI216146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47975[] = {SGI216143, SGI0, SGI216144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216147, SGI216148, SGI216148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47976[] = {SGI216149, SGI216150, SGI216151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216152, SGI0, SGI0, SGI0, SGI216153, SGI216153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47977[] = {SGI216154, SGI0, SGI216155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216156, SGI216157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216158, SGI0, SGI0, NULL};
const Segment * const SG47978[] = {SGI216159, SGI0, SGI216160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216161, SGI216162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47979[] = {SGI216163, SGI0, SGI216164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216165, SGI216166, SGI216166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47980[] = {SGI216167, SGI0, SGI216168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216169, SGI216170, SGI216170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47981[] = {SGI216171, SGI216172, SGI216173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216174, SGI0, SGI0, SGI0, SGI216175, SGI216175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47982[] = {SGI216176, SGI216177, SGI216178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216179, SGI0, SGI0, SGI0, SGI216180, SGI216180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47983[] = {SGI216181, SGI0, SGI216182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216183, SGI216184, SGI216184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47984[] = {SGI216185, SGI0, SGI216186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216187, SGI216188, SGI216188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216189, SGI0, SGI0, NULL};
const Segment * const SG47985[] = {SGI216190, SGI0, SGI216191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216192, SGI216193, SGI216193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47986[] = {SGI216194, SGI0, SGI216195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216196, SGI216197, SGI216197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216198, SGI0, SGI0, NULL};
const Segment * const SG47987[] = {SGI216199, SGI0, SGI216200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216201, SGI216202, SGI216202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47988[] = {SGI216203, SGI216204, SGI216205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216206, SGI0, SGI0, SGI0, SGI216207, SGI216207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47989[] = {SGI216208, SGI0, SGI216209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216210, SGI216211, SGI216211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47990[] = {SGI216212, SGI0, SGI216213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216214, SGI216215, SGI216215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216216, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47991[] = {SGI216217, SGI0, SGI216218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216219, SGI216220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47992[] = {SGI216221, SGI0, SGI216222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216223, SGI216224, SGI216224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47993[] = {SGI216225, SGI216226, SGI216227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216228, SGI0, SGI0, SGI0, SGI216229, SGI216229, SGI0, SGI0, SGI0, SGI0, SGI216230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47994[] = {SGI216231, SGI0, SGI216232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216233, SGI216234, SGI216234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47995[] = {SGI216235, SGI216236, SGI216237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216238, SGI216238, SGI0, SGI0, SGI0, SGI216239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47996[] = {SGI216240, SGI0, SGI216241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216242, SGI216243, SGI216243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216244, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47997[] = {SGI216245, SGI0, SGI216246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216247, SGI216248, SGI216248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47998[] = {SGI216249, SGI0, SGI216250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216251, SGI216252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG47999[] = {SGI216253, SGI0, SGI216254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216255, SGI216256, SGI216256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48000[] = {SGI216257, SGI0, SGI216258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216259, SGI216260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48001[] = {SGI216261, SGI0, SGI216262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216263, SGI216264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48002[] = {SGI216265, SGI0, SGI216266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216267, SGI216268, SGI216268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48003[] = {SGI216269, SGI0, SGI216270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216271, SGI216272, SGI216272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48004[] = {SGI216273, SGI0, SGI216274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216275, SGI216276, SGI216276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48005[] = {SGI216277, SGI216278, SGI216279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216280, SGI0, SGI0, SGI0, SGI216281, SGI216281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48006[] = {SGI216282, SGI0, SGI216283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216284, SGI216285, SGI216285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48007[] = {SGI216286, SGI0, SGI216287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216288, SGI216289, SGI216289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48008[] = {SGI216290, SGI0, SGI216291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216292, SGI216293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216294, SGI0, SGI0, NULL};
const Segment * const SG48009[] = {SGI216295, SGI0, SGI216296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216297, SGI216298, SGI216298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48010[] = {SGI216299, SGI0, SGI216300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216301, SGI216302, SGI216302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48011[] = {SGI216303, SGI216304, SGI216305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216306, SGI0, SGI0, SGI0, SGI216307, SGI216307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48012[] = {SGI216309, SGI216310, SGI216311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216312, SGI0, SGI0, SGI0, SGI216313, SGI216313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48013[] = {SGI216315, SGI216316, SGI216317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216318, SGI0, SGI0, SGI0, SGI216319, SGI216319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48014[] = {SGI216321, SGI0, SGI216322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216323, SGI216324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48015[] = {SGI216325, SGI216326, SGI216327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216328, SGI0, SGI0, SGI0, SGI216329, SGI216329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216330, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48016[] = {SGI216331, SGI216332, SGI216333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216334, SGI0, SGI0, SGI0, SGI216337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48017[] = {SGI216335, SGI216336, SGI216338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216339, SGI0, SGI0, SGI0, SGI216340, SGI216340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216342, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48018[] = {SGI216341, SGI216343, SGI216344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216345, SGI0, SGI0, SGI0, SGI216346, SGI216346, SGI0, SGI216347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48019[] = {SGI216348, SGI216349, SGI216350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216351, SGI0, SGI0, SGI0, SGI216352, SGI216352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216355, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48020[] = {SGI216353, SGI0, SGI216354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216356, SGI216357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48021[] = {SGI216358, SGI0, SGI216359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216360, SGI216361, SGI216361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48022[] = {SGI216362, SGI0, SGI216363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216364, SGI216365, SGI216365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48023[] = {SGI216366, SGI0, SGI216367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216368, SGI216369, SGI216369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48024[] = {SGI216370, SGI0, SGI216372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216376, SGI216377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48025[] = {SGI216371, SGI0, SGI216374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216378, SGI216379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48026[] = {SGI216373, SGI0, SGI216375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216380, SGI216381, SGI216381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48027[] = {SGI216382, SGI0, SGI216383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216384, SGI216385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48028[] = {SGI216386, SGI216387, SGI216388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216389, SGI0, SGI0, SGI0, SGI216390, SGI216390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216391, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48029[] = {SGI216392, SGI0, SGI216393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216394, SGI216395, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48030[] = {SGI216396, SGI0, SGI216397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216398, SGI216399, SGI216399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48031[] = {SGI216401, SGI216403, SGI216405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216406, SGI0, SGI0, SGI0, SGI216408, SGI216408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48032[] = {SGI216400, SGI216402, SGI216404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216407, SGI216407, SGI0, SGI0, SGI0, SGI0, SGI216409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48033[] = {SGI216410, SGI0, SGI216411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216412, SGI216413, SGI216413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48034[] = {SGI216414, SGI0, SGI216415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216416, SGI216417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48035[] = {SGI216418, SGI216419, SGI216420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216421, SGI0, SGI0, SGI0, SGI216424, SGI216424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48036[] = {SGI216422, SGI216423, SGI216425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216426, SGI216426, SGI0, SGI0, SGI0, SGI216427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48037[] = {SGI216428, SGI0, SGI216429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216430, SGI216431, SGI216431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48038[] = {SGI216432, SGI0, SGI216433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216434, SGI216435, SGI216435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48039[] = {SGI216436, SGI216437, SGI216438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216439, SGI0, SGI0, SGI0, SGI216440, SGI216440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216441, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48040[] = {SGI216442, SGI216443, SGI216444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216445, SGI0, SGI0, SGI0, SGI216446, SGI216446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48041[] = {SGI216447, SGI216448, SGI216449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216450, SGI216450, SGI0, SGI0, SGI0, SGI216451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48042[] = {SGI216452, SGI216453, SGI216454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216455, SGI0, SGI0, SGI0, SGI216456, SGI216456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48043[] = {SGI216457, SGI0, SGI216458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216459, SGI216460, SGI216460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48044[] = {SGI216461, SGI0, SGI216462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216463, SGI216464, SGI216464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48045[] = {SGI216465, SGI216466, SGI216467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216468, SGI0, SGI0, SGI0, SGI216469, SGI216469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48046[] = {SGI216470, SGI0, SGI216471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216472, SGI216473, SGI216473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48047[] = {SGI216474, SGI0, SGI216475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216476, SGI216477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48048[] = {SGI216478, SGI0, SGI216479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216480, SGI216481, SGI216481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48049[] = {SGI216482, SGI0, SGI216483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216484, SGI216485, SGI216485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48050[] = {SGI216486, SGI216487, SGI216488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216489, SGI216489, SGI0, SGI0, SGI0, SGI216490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48051[] = {SGI216491, SGI216492, SGI216493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216494, SGI0, SGI0, SGI0, SGI216497, SGI0, SGI0, SGI216499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48052[] = {SGI216495, SGI216496, SGI216498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216500, SGI0, SGI0, SGI0, SGI216501, SGI216501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216502, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48053[] = {SGI216503, SGI0, SGI216504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216505, SGI0, SGI0, SGI0, SGI216506, SGI216506, SGI0, SGI0, SGI216507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48054[] = {SGI216508, SGI0, SGI216509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216510, SGI216511, SGI216511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48055[] = {SGI216512, SGI216513, SGI216514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216515, SGI0, SGI0, SGI0, SGI216516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48056[] = {SGI216517, SGI0, SGI216518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216519, SGI216520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48057[] = {SGI216521, SGI216522, SGI216523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216524, SGI0, SGI0, SGI0, SGI216525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48058[] = {SGI216526, SGI216527, SGI216528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216529, SGI0, SGI0, SGI0, SGI216530, SGI216530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48059[] = {SGI216531, SGI0, SGI216532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216533, SGI216534, SGI216534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48060[] = {SGI216535, SGI0, SGI216536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216537, SGI0, SGI0, SGI0, SGI216538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48061[] = {SGI216540, SGI216541, SGI216542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216543, SGI0, SGI0, SGI0, SGI216544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48062[] = {SGI216546, SGI0, SGI216547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216548, SGI216549, SGI216549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48063[] = {SGI216550, SGI216551, SGI216552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216553, SGI0, SGI0, SGI0, SGI216554, SGI0, SGI0, SGI216555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48064[] = {SGI216556, SGI0, SGI216557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216558, SGI216559, SGI216559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48065[] = {SGI216560, SGI0, SGI216561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216562, SGI0, SGI0, SGI0, SGI216563, SGI0, SGI0, SGI216564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48066[] = {SGI216565, SGI216566, SGI216567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216568, SGI0, SGI0, SGI0, SGI216569, SGI216570, SGI0, SGI0, SGI0, SGI216571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48067[] = {SGI216572, SGI216573, SGI216574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216575, SGI216575, SGI0, SGI0, SGI0, SGI216576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48068[] = {SGI216577, SGI0, SGI216578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216579, SGI216580, SGI216580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48069[] = {SGI216581, SGI216582, SGI216583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216584, SGI0, SGI0, SGI0, SGI216585, SGI216585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48070[] = {SGI216586, SGI0, SGI216587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216588, SGI216589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48071[] = {SGI216590, SGI216591, SGI216592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216593, SGI216593, SGI0, SGI0, SGI0, SGI216594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48072[] = {SGI216595, SGI0, SGI216596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216597, SGI216598, SGI216598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48073[] = {SGI216599, SGI0, SGI216600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216601, SGI216602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48074[] = {SGI216603, SGI216604, SGI216605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216606, SGI0, SGI0, SGI0, SGI216607, SGI216607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48075[] = {SGI216608, SGI216609, SGI216610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216611, SGI0, SGI0, SGI0, SGI216612, SGI216612, SGI216613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48076[] = {SGI216614, SGI0, SGI216615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216616, SGI216617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48077[] = {SGI216618, SGI0, SGI216619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216620, SGI216621, SGI216621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48078[] = {SGI216622, SGI216623, SGI216624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216625, SGI0, SGI216626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48079[] = {SGI216627, SGI216628, SGI216629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216630, SGI0, SGI216631, SGI216631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48080[] = {SGI216632, SGI0, SGI216633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216634, SGI216635, SGI216635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48081[] = {SGI216636, SGI0, SGI216637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216638, SGI216639, SGI216639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48082[] = {SGI216640, SGI0, SGI216641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216642, SGI0, SGI0, SGI0, SGI216643, SGI216643, SGI0, SGI0, SGI0, SGI216644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48083[] = {SGI216645, SGI0, SGI216646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216649, SGI0, SGI0, SGI0, SGI216650, SGI216650, SGI0, SGI0, SGI0, SGI216653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48084[] = {SGI216647, SGI0, SGI216648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216651, SGI216652, SGI216652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48085[] = {SGI216654, SGI0, SGI216656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216658, SGI216659, SGI216659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48086[] = {SGI216655, SGI0, SGI216657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216660, SGI216661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48087[] = {SGI216662, SGI216663, SGI216664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216665, SGI0, SGI0, SGI0, SGI216667, SGI216667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48088[] = {SGI216666, SGI216668, SGI216669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216670, SGI0, SGI0, SGI0, SGI216671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48089[] = {SGI216672, SGI0, SGI216673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216674, SGI216675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48090[] = {SGI216676, SGI0, SGI216677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216678, SGI216679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48091[] = {SGI216680, SGI0, SGI216681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216682, SGI216683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48092[] = {SGI216684, SGI0, SGI216685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216686, SGI216687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216688, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48093[] = {SGI216689, SGI0, SGI216690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216691, SGI216692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216693, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48094[] = {SGI216694, SGI0, SGI216695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216696, SGI216697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48095[] = {SGI216698, SGI216699, SGI216700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216701, SGI0, SGI0, SGI0, SGI216702, SGI216702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48096[] = {SGI216703, SGI216704, SGI216705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216706, SGI216706, SGI0, SGI0, SGI0, SGI216707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48097[] = {SGI216708, SGI0, SGI216709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216710, SGI216711, SGI216711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48098[] = {SGI216712, SGI0, SGI216713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216714, SGI216715, SGI216715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48099[] = {SGI216716, SGI0, SGI216717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216718, SGI216719, SGI216719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48100[] = {SGI216720, SGI216721, SGI216722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216723, SGI0, SGI0, SGI0, SGI216724, SGI216724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216726, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48101[] = {SGI216725, SGI216727, SGI216728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216729, SGI216729, SGI0, SGI0, SGI216730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48102[] = {SGI216731, SGI216732, SGI216733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216734, SGI0, SGI0, SGI0, SGI216735, SGI216735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48103[] = {SGI216736, SGI0, SGI216737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216738, SGI216739, SGI216739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48104[] = {SGI216740, SGI216741, SGI216742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216743, SGI0, SGI0, SGI0, SGI216744, SGI216744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48105[] = {SGI216745, SGI0, SGI216746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216750, SGI216751, SGI216751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48106[] = {SGI216747, SGI216748, SGI216749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216752, SGI0, SGI0, SGI0, SGI216753, SGI216753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48107[] = {SGI216754, SGI0, SGI216755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216756, SGI216757, SGI216757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48108[] = {SGI216758, SGI0, SGI216759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216760, SGI216761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48109[] = {SGI216762, SGI216763, SGI216764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216765, SGI216765, SGI0, SGI0, SGI0, SGI216766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48110[] = {SGI216767, SGI216768, SGI216769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216770, SGI0, SGI0, SGI0, SGI216771, SGI0, SGI0, SGI216772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48111[] = {SGI216773, SGI216774, SGI216775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216776, SGI0, SGI0, SGI0, SGI216777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48112[] = {SGI216778, SGI216779, SGI216780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216781, SGI216781, SGI0, SGI0, SGI0, SGI216783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48113[] = {SGI216782, SGI0, SGI216784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216785, SGI216786, SGI216786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48114[] = {SGI216787, SGI0, SGI216788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216789, SGI216790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48115[] = {SGI216791, SGI216792, SGI216793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216794, SGI216794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216795, NULL};
const Segment * const SG48116[] = {SGI216796, SGI216797, SGI216798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216799, SGI0, SGI0, SGI0, SGI216800, SGI216800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48117[] = {SGI216801, SGI216802, SGI216803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216804, SGI216804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48118[] = {SGI216805, SGI0, SGI216806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48119[] = {SGI216807, SGI0, SGI216808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216809, SGI216810, SGI216810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48120[] = {SGI216811, SGI0, SGI216812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216813, SGI216814, SGI216814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48121[] = {SGI216816, SGI0, SGI216818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216819, SGI216821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48122[] = {SGI216815, SGI0, SGI216817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216820, SGI216822, SGI216822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216823, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48123[] = {SGI216824, SGI0, SGI216825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216830, SGI216831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48124[] = {SGI216827, SGI0, SGI216829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216832, SGI216834, SGI216834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48125[] = {SGI216826, SGI0, SGI216828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216833, SGI216835, SGI216835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216836, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48126[] = {SGI216837, SGI216838, SGI216839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216840, SGI0, SGI0, SGI0, SGI216841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48127[] = {SGI216842, SGI216843, SGI216844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216845, SGI0, SGI0, SGI0, SGI216846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48128[] = {SGI216847, SGI0, SGI216848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216849, SGI216850, SGI216850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216851, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48129[] = {SGI216852, SGI216853, SGI216854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216855, SGI0, SGI0, SGI0, SGI216856, SGI216856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216857, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48130[] = {SGI216858, SGI216859, SGI216860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216863, SGI0, SGI0, SGI0, SGI216864, SGI216864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48131[] = {SGI216861, SGI0, SGI216862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216865, SGI216866, SGI216866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48132[] = {SGI216867, SGI216868, SGI216869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216870, SGI216870, SGI0, SGI0, SGI0, SGI216871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48133[] = {SGI216872, SGI216873, SGI216874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216875, SGI0, SGI0, SGI0, SGI216876, SGI216876, SGI0, SGI216877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48134[] = {SGI216878, SGI216879, SGI216880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216881, SGI0, SGI0, SGI0, SGI216882, SGI216882, SGI0, SGI216883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48135[] = {SGI216884, SGI216885, SGI216886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216888, SGI0, SGI0, SGI0, SGI216890, SGI216890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216891, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48136[] = {SGI216887, SGI0, SGI216889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216892, SGI216893, SGI216893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48137[] = {SGI216894, SGI0, SGI216895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216896, SGI216898, SGI216898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48138[] = {SGI216897, SGI0, SGI216899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216900, SGI216901, SGI216901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48139[] = {SGI216902, SGI216903, SGI216904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216905, SGI0, SGI0, SGI0, SGI216906, SGI216906, SGI0, SGI216907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48140[] = {SGI216908, SGI216909, SGI216910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216911, SGI0, SGI0, SGI0, SGI216912, SGI216912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48141[] = {SGI216913, SGI0, SGI216914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48142[] = {SGI216915, SGI0, SGI216916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216917, SGI216918, SGI216918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48143[] = {SGI216919, SGI0, SGI216920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216921, SGI216922, SGI216922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48144[] = {SGI216923, SGI0, SGI216924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216925, SGI216926, SGI216926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48145[] = {SGI216927, SGI216928, SGI216929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216930, SGI0, SGI0, SGI0, SGI216931, SGI216931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48146[] = {SGI216932, SGI216933, SGI216934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216936, SGI0, SGI0, SGI0, SGI216938, SGI216938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48147[] = {SGI216935, SGI0, SGI216937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216940, SGI216941, SGI216941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48148[] = {SGI216942, SGI0, SGI216943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216944, SGI216945, SGI216945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48149[] = {SGI216946, SGI216947, SGI216948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216949, SGI0, SGI0, SGI0, SGI216950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48150[] = {SGI216951, SGI0, SGI216952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216953, SGI216954, SGI216954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48151[] = {SGI216955, SGI0, SGI216956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216957, SGI216958, SGI216958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48152[] = {SGI216959, SGI0, SGI216960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216961, SGI216962, SGI216962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48153[] = {SGI216963, SGI216964, SGI216965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216966, SGI0, SGI0, SGI0, SGI216967, SGI216967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48154[] = {SGI216968, SGI216969, SGI216970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216971, SGI0, SGI0, SGI0, SGI216972, SGI216972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48155[] = {SGI216973, SGI0, SGI216974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216975, SGI216976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48156[] = {SGI216977, SGI0, SGI216978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216979, SGI216980, SGI216980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48157[] = {SGI216981, SGI0, SGI216982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216983, SGI216984, SGI216984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48158[] = {SGI216985, SGI0, SGI216986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216987, SGI216988, SGI216988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216989, SGI0, SGI0, NULL};
const Segment * const SG48159[] = {SGI216990, SGI216991, SGI216992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216993, SGI216993, SGI0, SGI0, SGI0, SGI216994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48160[] = {SGI216995, SGI0, SGI216996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI216997, SGI216998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48161[] = {SGI216999, SGI0, SGI217000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217001, SGI0, SGI0, SGI0, SGI217002, SGI217002, SGI0, SGI0, SGI0, SGI217003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48162[] = {SGI217004, SGI0, SGI217005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217006, SGI217007, SGI217007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48163[] = {SGI217008, SGI0, SGI217009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217010, SGI217011, SGI217011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48164[] = {SGI217012, SGI217013, SGI217014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217015, SGI217015, SGI0, SGI0, SGI0, SGI217016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48165[] = {SGI217017, SGI0, SGI217018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217019, SGI217020, SGI217020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48166[] = {SGI217021, SGI0, SGI217022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217023, SGI217024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48167[] = {SGI217025, SGI0, SGI217026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217027, SGI217028, SGI217028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48168[] = {SGI217029, SGI217030, SGI217031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217032, SGI0, SGI0, SGI0, SGI217033, SGI217033, SGI0, SGI0, SGI217034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48169[] = {SGI217035, SGI217036, SGI217037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217038, SGI0, SGI0, SGI0, SGI217039, SGI217039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48170[] = {SGI217041, SGI217042, SGI217043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217044, SGI0, SGI0, SGI0, SGI217045, SGI217045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48171[] = {SGI217047, SGI217048, SGI217049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217050, SGI0, SGI0, SGI0, SGI217051, SGI217051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48172[] = {SGI217052, SGI217053, SGI217054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217055, SGI217055, SGI0, SGI0, SGI0, SGI217056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48173[] = {SGI217057, SGI217058, SGI217059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217060, SGI0, SGI0, SGI0, SGI217061, SGI217061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48174[] = {SGI217062, SGI0, SGI217063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217064, SGI217065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48175[] = {SGI217066, SGI0, SGI217067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48176[] = {SGI217068, SGI0, SGI217069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217070, SGI217071, SGI217071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48177[] = {SGI217072, SGI217073, SGI217074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217075, SGI0, SGI0, SGI0, SGI217076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48178[] = {SGI217077, SGI0, SGI217078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217079, SGI217080, SGI217080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48179[] = {SGI217081, SGI217082, SGI217083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217084, SGI0, SGI0, SGI0, SGI217085, SGI217085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48180[] = {SGI217086, SGI217087, SGI217088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217089, SGI0, SGI0, SGI0, SGI217090, SGI217090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48181[] = {SGI217091, SGI217092, SGI217093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217094, SGI0, SGI0, SGI0, SGI217095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48182[] = {SGI217096, SGI0, SGI217097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217098, SGI217099, SGI217099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48183[] = {SGI217100, SGI217101, SGI217102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217103, SGI0, SGI0, SGI0, SGI217104, SGI217104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48184[] = {SGI217105, SGI0, SGI217106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217107, SGI217108, SGI217108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48185[] = {SGI217109, SGI0, SGI217110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217111, SGI217112, SGI217112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48186[] = {SGI217113, SGI0, SGI217114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217115, SGI217116, SGI217116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48187[] = {SGI217117, SGI217118, SGI217119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217120, SGI217120, SGI0, SGI0, SGI0, SGI217121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48188[] = {SGI217122, SGI217123, SGI217124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217125, SGI0, SGI0, SGI0, SGI217126, SGI217126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48189[] = {SGI217127, SGI217128, SGI217129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217130, SGI217130, SGI0, SGI0, SGI0, SGI217131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48190[] = {SGI217132, SGI217133, SGI217134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217135, SGI217135, SGI0, SGI0, SGI0, SGI217138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48191[] = {SGI217136, SGI0, SGI217137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217139, SGI217140, SGI217140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48192[] = {SGI217141, SGI0, SGI217142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217143, SGI217144, SGI217144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48193[] = {SGI217145, SGI0, SGI217146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217147, SGI217148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48194[] = {SGI217149, SGI0, SGI217150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217154, SGI217155, SGI217155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48195[] = {SGI217151, SGI217152, SGI217153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217156, SGI0, SGI0, SGI0, SGI217157, SGI217157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48196[] = {SGI217158, SGI217159, SGI217160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217161, SGI217161, SGI0, SGI0, SGI0, SGI217162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48197[] = {SGI217163, SGI217164, SGI217165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217166, SGI0, SGI0, SGI0, SGI217167, SGI217167, SGI0, SGI217168, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48198[] = {SGI217169, SGI0, SGI217170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217171, SGI217172, SGI217172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217173, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48199[] = {SGI217174, SGI0, SGI217175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217176, SGI217177, SGI217177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48200[] = {SGI217178, SGI217179, SGI217180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217181, SGI217181, SGI0, SGI0, SGI0, SGI217182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48201[] = {SGI217183, SGI217184, SGI217185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217186, SGI217186, SGI0, SGI0, SGI0, SGI217187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48202[] = {SGI217188, SGI217189, SGI217190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217191, SGI217191, SGI0, SGI0, SGI0, SGI217192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48203[] = {SGI217193, SGI0, SGI217194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217195, SGI217196, SGI217196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48204[] = {SGI217197, SGI0, SGI217198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217199, SGI217200, SGI217200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217201, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48205[] = {SGI217202, SGI0, SGI217203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217204, SGI217205, SGI217205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48206[] = {SGI217206, SGI217207, SGI217208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217209, SGI0, SGI0, SGI0, SGI217210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217211, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48207[] = {SGI217212, SGI0, SGI217213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217214, SGI217215, SGI217215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48208[] = {SGI217216, SGI0, SGI217217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217218, SGI217219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48209[] = {SGI217220, SGI217221, SGI217222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217225, SGI0, SGI0, SGI0, SGI217226, SGI217226, SGI0, SGI217227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48210[] = {SGI217223, SGI0, SGI217224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217228, SGI217229, SGI217229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48211[] = {SGI217230, SGI217231, SGI217232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217233, SGI0, SGI0, SGI0, SGI217234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48212[] = {SGI217235, SGI0, SGI217236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217237, SGI217238, SGI217238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48213[] = {SGI217239, SGI217240, SGI217241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217242, SGI0, SGI0, SGI0, SGI217243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48214[] = {SGI217244, SGI217245, SGI217246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217247, SGI0, SGI0, SGI0, SGI217248, SGI217248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48215[] = {SGI217249, SGI217250, SGI217251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217252, SGI217252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48216[] = {SGI217253, SGI0, SGI217254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217255, SGI0, SGI0, SGI0, SGI217256, SGI217256, SGI0, SGI0, SGI0, SGI217257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48217[] = {SGI217258, SGI217259, SGI217260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217261, SGI0, SGI0, SGI0, SGI217262, SGI217262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48218[] = {SGI217263, SGI0, SGI217264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217266, SGI217268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48219[] = {SGI217265, SGI0, SGI217267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217269, SGI217270, SGI217270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48220[] = {SGI217271, SGI0, SGI217272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217273, SGI217274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48221[] = {SGI217275, SGI0, SGI217276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217277, SGI217278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48222[] = {SGI217279, SGI217280, SGI217281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217282, SGI0, SGI0, SGI0, SGI217283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48223[] = {SGI217284, SGI0, SGI217285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217286, SGI217287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48224[] = {SGI217288, SGI0, SGI217289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217290, SGI217291, SGI217291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48225[] = {SGI217292, SGI0, SGI217293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217294, SGI217295, SGI217295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48226[] = {SGI217296, SGI0, SGI217297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217300, SGI217301, SGI217301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48227[] = {SGI217298, SGI0, SGI217299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217302, SGI217303, SGI217303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48228[] = {SGI217304, SGI217305, SGI217306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217307, SGI0, SGI0, SGI0, SGI217308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48229[] = {SGI217309, SGI0, SGI217310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217311, SGI217312, SGI217312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48230[] = {SGI217313, SGI217314, SGI217315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217316, SGI0, SGI0, SGI0, SGI217317, SGI217317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217318, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48231[] = {SGI217319, SGI217320, SGI217321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217322, SGI0, SGI0, SGI0, SGI217323, SGI217323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217324, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48232[] = {SGI217325, SGI0, SGI217326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217327, SGI217328, SGI217328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48233[] = {SGI217329, SGI0, SGI217330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217331, SGI217332, SGI217332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48234[] = {SGI217333, SGI0, SGI217334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217337, SGI217338, SGI217338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48235[] = {SGI217335, SGI0, SGI217336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217339, SGI217340, SGI217340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48236[] = {SGI217341, SGI0, SGI217342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217344, SGI217346, SGI217346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48237[] = {SGI217343, SGI0, SGI217345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217347, SGI217348, SGI217348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48238[] = {SGI217349, SGI217350, SGI217351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217352, SGI0, SGI0, SGI0, SGI217355, SGI217355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48239[] = {SGI217353, SGI217354, SGI217356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217357, SGI0, SGI0, SGI0, SGI217358, SGI217358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48240[] = {SGI217359, SGI217360, SGI217361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217362, SGI217362, SGI0, SGI0, SGI0, SGI217363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48241[] = {SGI217364, SGI217365, SGI217366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217368, SGI217368, SGI0, SGI0, SGI0, SGI0, SGI217370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48242[] = {SGI217367, SGI0, SGI217369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217372, SGI217374, SGI217374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217375, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48243[] = {SGI217371, SGI0, SGI217373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217376, SGI217377, SGI217377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48244[] = {SGI217378, SGI217379, SGI217380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217381, SGI217381, SGI0, SGI0, SGI0, SGI0, SGI217384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48245[] = {SGI217382, SGI0, SGI217383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217385, SGI217386, SGI217386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48246[] = {SGI217387, SGI0, SGI217388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217389, SGI217390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48247[] = {SGI217391, SGI0, SGI217392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217393, SGI217394, SGI217394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48248[] = {SGI217395, SGI0, SGI217396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217397, SGI217398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48249[] = {SGI217399, SGI0, SGI217400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217401, SGI217402, SGI217402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48250[] = {SGI217403, SGI0, SGI217404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217405, SGI217406, SGI217406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48251[] = {SGI217407, SGI0, SGI217408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217409, SGI0, SGI0, SGI0, SGI217410, SGI217410, SGI0, SGI0, SGI0, SGI0, SGI217411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48252[] = {SGI217412, SGI0, SGI217413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217414, SGI217415, SGI217415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48253[] = {SGI217416, SGI0, SGI217417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217418, SGI217419, SGI217419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217420, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48254[] = {SGI217421, SGI217422, SGI217423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217426, SGI0, SGI0, SGI0, SGI217427, SGI217427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217429, NULL};
const Segment * const SG48255[] = {SGI217424, SGI0, SGI217425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217428, SGI217430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48256[] = {SGI217431, SGI0, SGI217432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217433, SGI217434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48257[] = {SGI217435, SGI217436, SGI217437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217438, SGI0, SGI0, SGI0, SGI217439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217440, NULL};
const Segment * const SG48258[] = {SGI217441, SGI0, SGI217442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217443, SGI0, SGI0, SGI0, SGI217444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217445, NULL};
const Segment * const SG48259[] = {SGI217446, SGI217447, SGI217448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217449, SGI0, SGI0, SGI0, SGI217450, SGI217450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48260[] = {SGI217451, SGI217452, SGI217453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217454, SGI0, SGI0, SGI0, SGI217455, SGI217455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48261[] = {SGI217456, SGI217457, SGI217458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217461, SGI217461, SGI0, SGI0, SGI0, SGI0, SGI217463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48262[] = {SGI217459, SGI0, SGI217460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217465, SGI217466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48263[] = {SGI217464, SGI0, SGI217468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48264[] = {SGI217462, SGI217467, SGI217469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217470, SGI0, SGI0, SGI0, SGI217471, SGI217471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48265[] = {SGI217473, SGI217474, SGI217475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217476, SGI0, SGI0, SGI0, SGI217477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48266[] = {SGI217478, SGI0, SGI217479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217480, SGI217481, SGI217481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48267[] = {SGI217482, SGI0, SGI217483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217484, SGI217485, SGI217485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48268[] = {SGI217486, SGI0, SGI217487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217488, SGI217489, SGI217489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48269[] = {SGI217490, SGI0, SGI217491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217493, SGI217494, SGI217494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48270[] = {SGI217492, SGI0, SGI217495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217496, SGI217497, SGI217497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48271[] = {SGI217498, SGI0, SGI217499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217502, SGI217503, SGI217503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48272[] = {SGI217500, SGI0, SGI217501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217504, SGI217505, SGI217505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48273[] = {SGI217506, SGI217507, SGI217508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217509, SGI0, SGI0, SGI0, SGI217510, SGI0, SGI217511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48274[] = {SGI217512, SGI0, SGI217513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217514, SGI217515, SGI217515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48275[] = {SGI217516, SGI217517, SGI217518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217519, SGI217519, SGI0, SGI0, SGI217520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48276[] = {SGI217521, SGI0, SGI217522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217523, SGI217524, SGI217524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48277[] = {SGI217525, SGI217526, SGI217527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217528, SGI0, SGI0, SGI0, SGI217529, SGI217529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48278[] = {SGI217530, SGI217531, SGI217532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217533, SGI0, SGI0, SGI0, SGI217535, SGI217535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48279[] = {SGI217534, SGI217536, SGI217537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217538, SGI0, SGI0, SGI0, SGI217539, SGI217539, SGI0, SGI217540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48280[] = {SGI217541, SGI217542, SGI217543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217544, SGI0, SGI0, SGI0, SGI217545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48281[] = {SGI217547, SGI0, SGI217549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217552, SGI217553, SGI217553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48282[] = {SGI217546, SGI217548, SGI217550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217551, SGI0, SGI0, SGI0, SGI217554, SGI217554, SGI0, SGI217555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48283[] = {SGI217556, SGI217557, SGI217558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217559, SGI0, SGI0, SGI0, SGI217560, SGI217560, SGI0, SGI217561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48284[] = {SGI217562, SGI0, SGI217563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217564, SGI217565, SGI217565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48285[] = {SGI217566, SGI217567, SGI217568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217569, SGI0, SGI0, SGI0, SGI217570, SGI217570, SGI0, SGI217571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48286[] = {SGI217572, SGI0, SGI217573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217574, SGI217575, SGI217575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48287[] = {SGI217576, SGI217577, SGI217578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217579, SGI0, SGI0, SGI0, SGI217580, SGI217580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217581, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48288[] = {SGI217582, SGI0, SGI217583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217584, SGI217585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48289[] = {SGI217586, SGI0, SGI217587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217588, SGI217589, SGI217589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48290[] = {SGI217590, SGI217591, SGI217592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217593, SGI0, SGI0, SGI0, SGI217594, SGI0, SGI0, SGI0, SGI217595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48291[] = {SGI217596, SGI217597, SGI217598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217599, SGI0, SGI0, SGI0, SGI217600, SGI0, SGI0, SGI0, SGI217601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48292[] = {SGI217602, SGI217603, SGI217604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217605, SGI0, SGI0, SGI0, SGI217606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217607, SGI0, SGI0, NULL};
const Segment * const SG48293[] = {SGI217608, SGI0, SGI217609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217610, SGI0, SGI0, SGI0, SGI217611, SGI217611, SGI0, SGI0, SGI0, SGI217614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48294[] = {SGI217612, SGI0, SGI217613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217615, SGI217616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48295[] = {SGI217617, SGI0, SGI217618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217619, SGI217620, SGI217620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48296[] = {SGI217621, SGI217622, SGI217623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217624, SGI0, SGI0, SGI0, SGI217625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48297[] = {SGI217626, SGI217627, SGI217628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217629, SGI0, SGI0, SGI0, SGI217630, SGI217630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48298[] = {SGI217631, SGI0, SGI217632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217633, SGI217634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48299[] = {SGI217635, SGI0, SGI217636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217637, SGI217638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48300[] = {SGI217639, SGI217640, SGI217641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217642, SGI0, SGI0, SGI0, SGI217643, SGI217643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217644, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48301[] = {SGI217645, SGI0, SGI217646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217647, SGI217648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48302[] = {SGI217649, SGI217650, SGI217651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217652, SGI0, SGI0, SGI0, SGI217653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48303[] = {SGI217654, SGI217655, SGI217656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217657, SGI0, SGI0, SGI0, SGI217658, SGI217658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48304[] = {SGI217659, SGI0, SGI217660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217661, SGI217662, SGI217662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48305[] = {SGI217663, SGI0, SGI217664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217665, SGI217666, SGI217666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48306[] = {SGI217667, SGI0, SGI217668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217671, SGI217672, SGI217672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48307[] = {SGI217669, SGI0, SGI217670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217673, SGI217674, SGI217674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48308[] = {SGI217675, SGI0, SGI217676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217677, SGI217679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48309[] = {SGI217678, SGI0, SGI217680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217681, SGI217682, SGI217682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48310[] = {SGI217683, SGI0, SGI217684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217686, SGI217687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48311[] = {SGI217685, SGI0, SGI217688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217689, SGI217690, SGI217690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48312[] = {SGI217691, SGI0, SGI217692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217693, SGI217694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48313[] = {SGI217695, SGI0, SGI217696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217699, SGI217700, SGI217700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48314[] = {SGI217697, SGI0, SGI217698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217702, SGI217704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48315[] = {SGI217701, SGI217703, SGI217705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217706, SGI217706, SGI0, SGI0, SGI0, SGI217707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48316[] = {SGI217708, SGI0, SGI217709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217710, SGI217711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48317[] = {SGI217712, SGI217713, SGI217714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217715, SGI0, SGI0, SGI0, SGI217716, SGI217716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48318[] = {SGI217717, SGI0, SGI217718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217719, SGI217720, SGI217720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48319[] = {SGI217721, SGI0, SGI217722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217723, SGI217724, SGI217724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48320[] = {SGI217725, SGI0, SGI217726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217727, SGI217728, SGI217728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48321[] = {SGI217729, SGI0, SGI217730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217731, SGI217732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48322[] = {SGI217733, SGI0, SGI217734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217735, SGI217736, SGI217736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48323[] = {SGI217737, SGI0, SGI217738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217739, SGI217740, SGI217740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217741, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48324[] = {SGI217742, SGI0, SGI217743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217746, SGI217747, SGI217747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48325[] = {SGI217744, SGI0, SGI217745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217748, SGI217749, SGI217749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217750, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48326[] = {SGI217751, SGI0, SGI217753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217758, SGI217760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48327[] = {SGI217752, SGI217754, SGI217755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217759, SGI0, SGI0, SGI0, SGI217761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48328[] = {SGI217756, SGI0, SGI217757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217762, SGI217763, SGI217763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48329[] = {SGI217764, SGI0, SGI217765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217768, SGI217769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48330[] = {SGI217766, SGI0, SGI217767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217770, SGI217771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48331[] = {SGI217772, SGI0, SGI217773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217774, SGI217775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48332[] = {SGI217776, SGI0, SGI217777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217778, SGI217779, SGI217779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48333[] = {SGI217780, SGI0, SGI217781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217782, SGI217783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48334[] = {SGI217784, SGI0, SGI217785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217786, SGI217788, SGI217788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48335[] = {SGI217787, SGI0, SGI217789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217790, SGI217791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48336[] = {SGI217792, SGI0, SGI217793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217794, SGI217795, SGI217795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48337[] = {SGI217796, SGI0, SGI217797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48338[] = {SGI217798, SGI0, SGI217799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217800, SGI217801, SGI217801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48339[] = {SGI217802, SGI217803, SGI217804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217805, SGI0, SGI0, SGI0, SGI217806, SGI217806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48340[] = {SGI217807, SGI0, SGI217808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217809, SGI217810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48341[] = {SGI217811, SGI0, SGI217812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217813, SGI217814, SGI217814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48342[] = {SGI217815, SGI0, SGI217816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217817, SGI217818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48343[] = {SGI217819, SGI0, SGI217820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217821, SGI217822, SGI217822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48344[] = {SGI217823, SGI217824, SGI217825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217826, SGI0, SGI0, SGI0, SGI217827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48345[] = {SGI217828, SGI0, SGI217829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217832, SGI217834, SGI217834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48346[] = {SGI217830, SGI217831, SGI217833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217835, SGI0, SGI0, SGI0, SGI217836, SGI217836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48347[] = {SGI217837, SGI217838, SGI217839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217840, SGI0, SGI0, SGI0, SGI217841, SGI217841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48348[] = {SGI217842, SGI0, SGI217843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217844, SGI217845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217846, SGI0, SGI0, NULL};
const Segment * const SG48349[] = {SGI217847, SGI0, SGI217848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217849, SGI217850, SGI217850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48350[] = {SGI217851, SGI0, SGI217852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217853, SGI217854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48351[] = {SGI217855, SGI0, SGI217856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217857, SGI217858, SGI217858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48352[] = {SGI217859, SGI0, SGI217860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217861, SGI217862, SGI217862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48353[] = {SGI217863, SGI0, SGI217864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217865, SGI217866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48354[] = {SGI217870, SGI0, SGI217871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217872, SGI217873, SGI217873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48355[] = {SGI217874, SGI0, SGI217875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217878, SGI217880, SGI217880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48356[] = {SGI217876, SGI217877, SGI217879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217881, SGI0, SGI0, SGI0, SGI217882, SGI217882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48357[] = {SGI217883, SGI0, SGI217884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217885, SGI217886, SGI217886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48358[] = {SGI217887, SGI0, SGI217888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217889, SGI217890, SGI217890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48359[] = {SGI217891, SGI0, SGI217892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217893, SGI217894, SGI217894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48360[] = {SGI217895, SGI0, SGI217896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217897, SGI217898, SGI217898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48361[] = {SGI217899, SGI217900, SGI217901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217902, SGI0, SGI0, SGI0, SGI217903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48362[] = {SGI217904, SGI0, SGI217905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217906, SGI217907, SGI217907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48363[] = {SGI217908, SGI0, SGI217909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217911, SGI217913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48364[] = {SGI217910, SGI0, SGI217912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217914, SGI217915, SGI217915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48365[] = {SGI217916, SGI0, SGI217917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217920, SGI217921, SGI217921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48366[] = {SGI217918, SGI0, SGI217919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217922, SGI217923, SGI217923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48367[] = {SGI217924, SGI217925, SGI217927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217929, SGI0, SGI0, SGI0, SGI217930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48368[] = {SGI217926, SGI0, SGI217928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217931, SGI217932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48369[] = {SGI217933, SGI0, SGI217934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217935, SGI217936, SGI217936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48370[] = {SGI217937, SGI217938, SGI217939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217940, SGI0, SGI0, SGI0, SGI217941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48371[] = {SGI217942, SGI217943, SGI217944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217947, SGI0, SGI0, SGI0, SGI217948, SGI217948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48372[] = {SGI217945, SGI0, SGI217946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217949, SGI217950, SGI217950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48373[] = {SGI217952, SGI0, SGI217954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217956, SGI217958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48374[] = {SGI217951, SGI0, SGI217953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217955, SGI0, SGI0, SGI0, SGI217957, SGI217957, SGI0, SGI0, SGI0, SGI217959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48375[] = {SGI217960, SGI0, SGI217961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217962, SGI217963, SGI217963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48376[] = {SGI217964, SGI0, SGI217965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217966, SGI217967, SGI217967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48377[] = {SGI217968, SGI217969, SGI217970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217971, SGI0, SGI0, SGI0, SGI217972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48378[] = {SGI217973, SGI0, SGI217975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48379[] = {SGI217974, SGI217976, SGI217977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217978, SGI217978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48380[] = {SGI217979, SGI217980, SGI217981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217983, SGI0, SGI0, SGI0, SGI217986, SGI217986, SGI0, SGI0, SGI0, SGI217987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48381[] = {SGI217982, SGI217984, SGI217985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217988, SGI0, SGI0, SGI0, SGI217989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48382[] = {SGI217990, SGI0, SGI217991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48383[] = {SGI217992, SGI217993, SGI217994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI217995, SGI0, SGI0, SGI0, SGI217997, SGI217997, SGI0, SGI217999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48384[] = {SGI0, SGI217996, SGI217998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48385[] = {SGI218000, SGI0, SGI218001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218005, SGI218006, SGI218006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48386[] = {SGI218002, SGI218003, SGI218004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218007, SGI0, SGI0, SGI0, SGI218009, SGI218011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48387[] = {SGI218008, SGI0, SGI218010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218012, SGI218013, SGI218013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48388[] = {SGI218014, SGI0, SGI218015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218016, SGI218017, SGI218017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48389[] = {SGI218018, SGI0, SGI218019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218020, SGI218021, SGI218021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48390[] = {SGI218022, SGI218023, SGI218024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218025, SGI0, SGI0, SGI0, SGI218026, SGI218026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48391[] = {SGI218027, SGI218028, SGI218029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218030, SGI0, SGI0, SGI0, SGI218031, SGI218031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218032, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48392[] = {SGI218033, SGI218034, SGI218035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218036, SGI218036, SGI0, SGI0, SGI0, SGI218037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48393[] = {SGI218038, SGI0, SGI218039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218040, SGI218041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48394[] = {SGI218042, SGI0, SGI218043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218044, SGI218045, SGI218045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48395[] = {SGI218046, SGI218047, SGI218048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218049, SGI0, SGI0, SGI0, SGI218050, SGI218050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48396[] = {SGI218051, SGI218052, SGI218053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218054, SGI0, SGI0, SGI0, SGI218055, SGI218055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48397[] = {SGI218056, SGI0, SGI218057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48398[] = {SGI218058, SGI0, SGI218059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218060, SGI218061, SGI218061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48399[] = {SGI218062, SGI0, SGI218063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218064, SGI218065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48400[] = {SGI218066, SGI0, SGI218067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218068, SGI218069, SGI218069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48401[] = {SGI218070, SGI218071, SGI218072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218073, SGI0, SGI0, SGI0, SGI218074, SGI218074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48402[] = {SGI218075, SGI218076, SGI218077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218078, SGI0, SGI0, SGI0, SGI218079, SGI218079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48403[] = {SGI218080, SGI218081, SGI218082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218083, SGI0, SGI0, SGI0, SGI218084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48404[] = {SGI218085, SGI0, SGI218086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218087, SGI218088, SGI218088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218089, SGI0, SGI0, NULL};
const Segment * const SG48405[] = {SGI218090, SGI218091, SGI218092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218093, SGI218093, SGI0, SGI0, SGI0, SGI218094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48406[] = {SGI218095, SGI218096, SGI218097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218098, SGI218098, SGI0, SGI0, SGI0, SGI218099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48407[] = {SGI218100, SGI0, SGI218101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218102, SGI0, SGI0, SGI0, SGI218103, SGI218103, SGI0, SGI0, SGI0, SGI218104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48408[] = {SGI218105, SGI218106, SGI218107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218108, SGI0, SGI0, SGI0, SGI218109, SGI218109, SGI0, SGI218110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48409[] = {SGI218111, SGI0, SGI218112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218113, SGI218114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48410[] = {SGI218115, SGI0, SGI218116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218117, SGI218118, SGI218118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218119, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48411[] = {SGI218120, SGI0, SGI218121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218122, SGI218123, SGI218123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218124, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48412[] = {SGI218125, SGI0, SGI218126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218127, SGI218128, SGI218128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48413[] = {SGI218129, SGI0, SGI218130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218131, SGI218132, SGI218132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48414[] = {SGI218133, SGI0, SGI218134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218135, SGI218136, SGI218136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48415[] = {SGI218137, SGI0, SGI218138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218139, SGI218140, SGI218140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48416[] = {SGI218141, SGI218142, SGI218143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218144, SGI0, SGI0, SGI0, SGI218145, SGI218145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48417[] = {SGI218146, SGI218147, SGI218148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218149, SGI0, SGI0, SGI0, SGI218150, SGI218150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218151, SGI0, SGI0, NULL};
const Segment * const SG48418[] = {SGI218152, SGI0, SGI218153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218154, SGI218155, SGI218155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48419[] = {SGI218156, SGI218157, SGI218158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218159, SGI0, SGI0, SGI0, SGI218160, SGI218160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218161, SGI0, SGI0, NULL};
const Segment * const SG48420[] = {SGI218162, SGI0, SGI218163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218164, SGI218165, SGI218165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218166, SGI0, SGI0, NULL};
const Segment * const SG48421[] = {SGI218167, SGI218168, SGI218169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218170, SGI218170, SGI0, SGI0, SGI0, SGI0, SGI218171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48422[] = {SGI218172, SGI218173, SGI218174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218175, SGI0, SGI0, SGI0, SGI218176, SGI218176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218177, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48423[] = {SGI218178, SGI218179, SGI218180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218181, SGI0, SGI0, SGI0, SGI218182, SGI218182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218183, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48424[] = {SGI218184, SGI0, SGI218185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218186, SGI218187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48425[] = {SGI218188, SGI0, SGI218189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218190, SGI218191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48426[] = {SGI218192, SGI0, SGI218193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218194, SGI218195, SGI218195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48427[] = {SGI218196, SGI0, SGI218197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218198, SGI218199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48428[] = {SGI218200, SGI0, SGI218201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218204, SGI218205, SGI218205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48429[] = {SGI218202, SGI0, SGI218203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218206, SGI218207, SGI218207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48430[] = {SGI218208, SGI0, SGI218209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218210, SGI218211, SGI218211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48431[] = {SGI218212, SGI218213, SGI218215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218218, SGI0, SGI0, SGI0, SGI218219, SGI218219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48432[] = {SGI218214, SGI218216, SGI218217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218220, SGI0, SGI0, SGI0, SGI218221, SGI218221, SGI218222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48433[] = {SGI218223, SGI218224, SGI218225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218226, SGI0, SGI0, SGI0, SGI218227, SGI218227, SGI0, SGI0, SGI0, SGI218228, SGI0, SGI0, NULL};
const Segment * const SG48434[] = {SGI217867, SGI217868, SGI217869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218229, SGI0, SGI0, SGI0, SGI218230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48435[] = {SGI218231, SGI218232, SGI218233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218234, SGI0, SGI0, SGI0, SGI218235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48436[] = {SGI218236, SGI218237, SGI218238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218239, SGI0, SGI0, SGI0, SGI218240, SGI218240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48437[] = {SGI218241, SGI0, SGI218242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218243, SGI218244, SGI218244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48438[] = {SGI218245, SGI0, SGI218246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218248, SGI218249, SGI218249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48439[] = {SGI218247, SGI0, SGI218250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218251, SGI218252, SGI218252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48440[] = {SGI218253, SGI0, SGI218254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218255, SGI218256, SGI218256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48441[] = {SGI218257, SGI0, SGI218258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218259, SGI218260, SGI218260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48442[] = {SGI218261, SGI0, SGI218262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218263, SGI218264, SGI218264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48443[] = {SGI218265, SGI0, SGI218266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218267, SGI218268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48444[] = {SGI218269, SGI0, SGI218270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218271, SGI218272, SGI218272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48445[] = {SGI218276, SGI0, SGI218277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218280, SGI218281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48446[] = {SGI218273, SGI218274, SGI218275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218278, SGI0, SGI0, SGI0, SGI218279, SGI218279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48447[] = {SGI218283, SGI0, SGI218284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218285, SGI218286, SGI218286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48448[] = {SGI218287, SGI0, SGI218288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218289, SGI218290, SGI218290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48449[] = {SGI218291, SGI218292, SGI218293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218294, SGI0, SGI0, SGI0, SGI218295, SGI218295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218296, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48450[] = {SGI218297, SGI0, SGI218298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218299, SGI218300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48451[] = {SGI218301, SGI218302, SGI218303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218304, SGI0, SGI0, SGI0, SGI218305, SGI218305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218306, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48452[] = {SGI218307, SGI0, SGI218308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218309, SGI218310, SGI218310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48453[] = {SGI218311, SGI0, SGI218312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218313, SGI218314, SGI218314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48454[] = {SGI218315, SGI0, SGI218316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218317, SGI218318, SGI218318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48455[] = {SGI218319, SGI0, SGI218320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218321, SGI218322, SGI218322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48456[] = {SGI218323, SGI0, SGI218324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218325, SGI218326, SGI218326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48457[] = {SGI218327, SGI0, SGI218328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218329, SGI218330, SGI218330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48458[] = {SGI218331, SGI0, SGI218332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218333, SGI218334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48459[] = {SGI218335, SGI218336, SGI218337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218338, SGI0, SGI0, SGI0, SGI218339, SGI218339, SGI0, SGI218341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48460[] = {SGI218340, SGI0, SGI218342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218343, SGI218344, SGI218344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48461[] = {SGI218345, SGI0, SGI218346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218347, SGI218348, SGI218348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48462[] = {SGI218349, SGI0, SGI218350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218351, SGI218352, SGI218352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48463[] = {SGI218353, SGI218354, SGI218355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218356, SGI0, SGI0, SGI0, SGI218357, SGI218357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48464[] = {SGI218358, SGI0, SGI218359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218360, SGI218361, SGI218361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48465[] = {SGI218362, SGI218363, SGI218364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218365, SGI0, SGI0, SGI0, SGI218366, SGI218366, SGI0, SGI218367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48466[] = {SGI218368, SGI218369, SGI218370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218372, SGI0, SGI0, SGI0, SGI218374, SGI218374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48467[] = {SGI218371, SGI0, SGI218373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218378, SGI218379, SGI218379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48468[] = {SGI218375, SGI218376, SGI218377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218380, SGI0, SGI0, SGI0, SGI218381, SGI218381, SGI0, SGI218382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48469[] = {SGI218383, SGI218384, SGI218385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218388, SGI0, SGI0, SGI0, SGI218390, SGI218390, SGI0, SGI218391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48470[] = {SGI218386, SGI218387, SGI218389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218392, SGI218392, SGI0, SGI0, SGI0, SGI218393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48471[] = {SGI218394, SGI218395, SGI218396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218397, SGI0, SGI0, SGI0, SGI218398, SGI218398, SGI0, SGI0, SGI0, SGI0, SGI218399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48472[] = {SGI218400, SGI218401, SGI218402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218403, SGI0, SGI0, SGI0, SGI218404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48473[] = {SGI218405, SGI0, SGI218406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218407, SGI218408, SGI218408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218409, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48474[] = {SGI218410, SGI0, SGI218411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218412, SGI218413, SGI218413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218414, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48475[] = {SGI218415, SGI218416, SGI218417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218418, SGI0, SGI0, SGI0, SGI218421, SGI218421, SGI0, SGI218423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48476[] = {SGI218419, SGI218420, SGI218422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218424, SGI218424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218425, NULL};
const Segment * const SG48477[] = {SGI218426, SGI218428, SGI218429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218431, SGI0, SGI0, SGI0, SGI218433, SGI218433, SGI0, SGI218435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48478[] = {SGI218427, SGI0, SGI218430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218432, SGI218434, SGI218434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48479[] = {SGI218436, SGI218437, SGI218438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218439, SGI218439, SGI0, SGI0, SGI0, SGI218441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48480[] = {SGI218440, SGI0, SGI218442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218443, SGI218444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48481[] = {SGI218445, SGI218446, SGI218447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218448, SGI218448, SGI0, SGI0, SGI0, SGI0, SGI218449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48482[] = {SGI218450, SGI218451, SGI218452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218453, SGI218453, SGI0, SGI0, SGI0, SGI0, SGI218454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48483[] = {SGI218455, SGI218456, SGI218457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218458, SGI218458, SGI0, SGI0, SGI0, SGI0, SGI218459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48484[] = {SGI218460, SGI218461, SGI218462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218465, SGI0, SGI0, SGI0, SGI218466, SGI218466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48485[] = {SGI218467, SGI218469, SGI218471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218473, SGI0, SGI0, SGI0, SGI218474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48486[] = {SGI218475, SGI0, SGI218476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218477, SGI218478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48487[] = {SGI218479, SGI218480, SGI218481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218482, SGI218482, SGI0, SGI0, SGI0, SGI218483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48488[] = {SGI218484, SGI218485, SGI218486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218487, SGI0, SGI0, SGI0, SGI218488, SGI218488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218489, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48489[] = {SGI218490, SGI218491, SGI218492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218493, SGI0, SGI0, SGI0, SGI218494, SGI218494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218495, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48490[] = {SGI218496, SGI0, SGI218497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218498, SGI218499, SGI218499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48491[] = {SGI218500, SGI218501, SGI218502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218503, SGI218503, SGI0, SGI0, SGI0, SGI218504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48492[] = {SGI218505, SGI218506, SGI218507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218508, SGI218508, SGI0, SGI0, SGI0, SGI218509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48493[] = {SGI218510, SGI218511, SGI218512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218513, SGI0, SGI0, SGI0, SGI218514, SGI218514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48494[] = {SGI218515, SGI218516, SGI218517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218518, SGI0, SGI0, SGI0, SGI218519, SGI218519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48495[] = {SGI218520, SGI0, SGI218521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48496[] = {SGI218522, SGI218523, SGI218524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218525, SGI0, SGI0, SGI0, SGI218526, SGI218526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48497[] = {SGI218527, SGI218528, SGI218529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218530, SGI0, SGI0, SGI0, SGI218531, SGI218531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48498[] = {SGI218532, SGI0, SGI218533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218534, SGI218535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48499[] = {SGI218536, SGI218537, SGI218538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218539, SGI0, SGI0, SGI0, SGI218540, SGI218540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48500[] = {SGI218541, SGI0, SGI218542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218543, SGI218544, SGI218544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48501[] = {SGI218545, SGI0, SGI218546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218547, SGI218548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48502[] = {SGI218549, SGI0, SGI218550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218551, SGI0, SGI0, SGI0, SGI218552, SGI218552, SGI0, SGI0, SGI218553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48503[] = {SGI218554, SGI218555, SGI218556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218557, SGI0, SGI0, SGI0, SGI218558, SGI218558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218559, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48504[] = {SGI218560, SGI0, SGI218561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218562, SGI218563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48505[] = {SGI218564, SGI0, SGI218565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218566, SGI218567, SGI218567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48506[] = {SGI218568, SGI0, SGI218569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218570, SGI218571, SGI218571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48507[] = {SGI218572, SGI0, SGI218573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218574, SGI218575, SGI218575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48508[] = {SGI218576, SGI0, SGI218577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218578, SGI218579, SGI218579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48509[] = {SGI218580, SGI218581, SGI218582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218583, SGI0, SGI0, SGI0, SGI218584, SGI218584, SGI0, SGI0, SGI218585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48510[] = {SGI218586, SGI218587, SGI218588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218589, SGI218589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48511[] = {SGI218590, SGI0, SGI218591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48512[] = {SGI218592, SGI218593, SGI218594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218595, SGI0, SGI0, SGI0, SGI218596, SGI218596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48513[] = {SGI218597, SGI218598, SGI218599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218600, SGI0, SGI218601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48514[] = {SGI218602, SGI218603, SGI218604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218605, SGI0, SGI0, SGI0, SGI218606, SGI218606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48515[] = {SGI218607, SGI218608, SGI218609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218610, SGI0, SGI218611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48516[] = {SGI218612, SGI0, SGI218613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218614, SGI218615, SGI218615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48517[] = {SGI218616, SGI218617, SGI218618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218620, SGI0, SGI0, SGI0, SGI218622, SGI0, SGI218623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48518[] = {SGI218619, SGI0, SGI218621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218624, SGI218625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48519[] = {SGI218626, SGI218627, SGI218628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218629, SGI0, SGI0, SGI0, SGI218630, SGI218630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218631, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48520[] = {SGI218632, SGI218633, SGI218634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218635, SGI0, SGI0, SGI0, SGI218636, SGI218636, SGI0, SGI218637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48521[] = {SGI218638, SGI0, SGI218639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218640, SGI218641, SGI218641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48522[] = {SGI218642, SGI218643, SGI218644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218645, SGI218645, SGI0, SGI0, SGI0, SGI218646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48523[] = {SGI218647, SGI218648, SGI218649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218650, SGI0, SGI0, SGI0, SGI218651, SGI218651, SGI0, SGI0, NULL};
const Segment * const SG48524[] = {SGI218652, SGI0, SGI218653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218654, SGI218655, SGI218655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48525[] = {SGI218656, SGI0, SGI218657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218658, SGI218659, SGI218659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48526[] = {SGI218660, SGI0, SGI218661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218662, SGI218663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48527[] = {SGI218664, SGI0, SGI218665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218666, SGI218667, SGI218667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48528[] = {SGI218668, SGI0, SGI218669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218670, SGI218671, SGI218671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48529[] = {SGI218672, SGI0, SGI218673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218674, SGI218675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48530[] = {SGI218676, SGI0, SGI218677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218678, SGI218679, SGI218679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48531[] = {SGI218680, SGI0, SGI218681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218682, SGI218683, SGI218683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48532[] = {SGI218684, SGI0, SGI218685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218686, SGI218687, SGI218687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48533[] = {SGI218688, SGI218689, SGI218690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218691, SGI0, SGI0, SGI0, SGI218692, SGI218692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48534[] = {SGI218693, SGI0, SGI218694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218695, SGI218696, SGI218696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48535[] = {SGI218697, SGI0, SGI218698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218699, SGI218700, SGI218700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48536[] = {SGI218701, SGI218702, SGI218703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218706, SGI0, SGI0, SGI0, SGI218707, SGI218707, SGI0, SGI218708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48537[] = {SGI218704, SGI0, SGI218705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218709, SGI218710, SGI218710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218711, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48538[] = {SGI218712, SGI218713, SGI218714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218715, SGI0, SGI0, SGI0, SGI218716, SGI0, SGI0, SGI218717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48539[] = {SGI218718, SGI218719, SGI218720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218721, SGI0, SGI0, SGI0, SGI218722, SGI218722, SGI0, SGI218723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48540[] = {SGI218724, SGI0, SGI218725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218726, SGI0, SGI0, SGI0, SGI218727, SGI218727, SGI0, SGI0, SGI0, SGI218728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48541[] = {SGI218729, SGI0, SGI218730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218733, SGI218734, SGI218734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48542[] = {SGI218731, SGI0, SGI218732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218735, SGI218736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48543[] = {SGI218737, SGI0, SGI218738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218742, SGI218743, SGI218743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48544[] = {SGI218739, SGI218740, SGI218741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218744, SGI0, SGI0, SGI0, SGI218745, SGI218745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48545[] = {SGI218746, SGI218747, SGI218748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218749, SGI0, SGI0, SGI0, SGI218750, SGI218750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48546[] = {SGI218751, SGI0, SGI218752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48547[] = {SGI218753, SGI218754, SGI218755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218756, SGI218756, SGI0, SGI0, SGI0, SGI218757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48548[] = {SGI218758, SGI218759, SGI218760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218761, SGI0, SGI0, SGI0, SGI218762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48549[] = {SGI218763, SGI218764, SGI218765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218766, SGI0, SGI0, SGI0, SGI218767, SGI218767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218768, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48550[] = {SGI218769, SGI218770, SGI218771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218772, SGI0, SGI0, SGI0, SGI218773, SGI218773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218774, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48551[] = {SGI218775, SGI0, SGI218776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218777, SGI218778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218779, SGI0, SGI0, NULL};
const Segment * const SG48552[] = {SGI218780, SGI0, SGI218781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218782, SGI218783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218784, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48553[] = {SGI218785, SGI0, SGI218786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218787, SGI218788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218789, SGI0, SGI0, NULL};
const Segment * const SG48554[] = {SGI218790, SGI0, SGI218791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218792, SGI218793, SGI218793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218794, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48555[] = {SGI218795, SGI0, SGI218796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218797, SGI218798, SGI218798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48556[] = {SGI218799, SGI0, SGI218800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218802, SGI218804, SGI218804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48557[] = {SGI218801, SGI218803, SGI218805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218809, SGI218809, SGI0, SGI0, SGI218810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48558[] = {SGI218806, SGI218807, SGI218808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218811, SGI218811, SGI0, SGI0, SGI0, SGI218812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48559[] = {SGI218813, SGI0, SGI218814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218815, SGI218816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48560[] = {SGI218817, SGI0, SGI218818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218819, SGI218820, SGI218820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48561[] = {SGI218821, SGI0, SGI218822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218823, SGI218824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48562[] = {SGI218825, SGI0, SGI218826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218827, SGI218828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48563[] = {SGI218829, SGI0, SGI218830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218831, SGI218832, SGI218832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48564[] = {SGI218833, SGI218834, SGI218835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218836, SGI218836, SGI0, SGI0, SGI0, SGI218838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48565[] = {SGI218837, SGI0, SGI218839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218840, SGI218841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48566[] = {SGI218842, SGI0, SGI218843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218844, SGI218845, SGI218845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48567[] = {SGI218846, SGI0, SGI218847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218848, SGI218849, SGI218849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48568[] = {SGI218850, SGI0, SGI218851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218852, SGI218853, SGI218853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48569[] = {SGI218854, SGI0, SGI218855, SGI0, SGI218856, SGI218856, SGI0, NULL};
const Segment * const SG48570[] = {SGI218857, SGI218858, SGI218859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218860, SGI0, SGI0, SGI0, SGI218861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48571[] = {SGI218863, SGI0, SGI218864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218865, SGI218866, SGI218866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48572[] = {SGI218867, SGI218868, SGI218869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218870, SGI0, SGI0, SGI0, SGI218871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48573[] = {SGI218872, SGI0, SGI218873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218874, SGI218875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48574[] = {SGI218876, SGI0, SGI218877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218878, SGI0, SGI0, SGI0, SGI218879, SGI218879, SGI0, SGI0, SGI0, SGI218880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48575[] = {SGI218881, SGI0, SGI218882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218883, SGI218884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48576[] = {SGI218885, SGI0, SGI218886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218887, SGI218888, SGI218888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48577[] = {SGI218889, SGI218890, SGI218891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218892, SGI0, SGI0, SGI0, SGI218893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48578[] = {SGI0, SGI218894, SGI0, SGI218895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48579[] = {SGI218896, SGI218897, SGI218898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218899, SGI0, SGI0, SGI0, SGI218900, SGI218900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48580[] = {SGI218901, SGI218902, SGI218903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218904, SGI0, SGI0, SGI0, SGI218905, SGI218905, SGI0, SGI0, SGI218906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48581[] = {SGI218907, SGI218908, SGI218909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218910, SGI218910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48582[] = {SGI218911, SGI0, SGI218912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218913, SGI218914, SGI218914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218915, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48583[] = {SGI218916, SGI0, SGI218917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218918, SGI0, SGI0, SGI0, SGI218919, SGI218919, SGI0, SGI0, SGI0, SGI218920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48584[] = {SGI218921, SGI0, SGI218922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218923, SGI218924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48585[] = {SGI218925, SGI0, SGI218926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218927, SGI218928, SGI218928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48586[] = {SGI218929, SGI218930, SGI218931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218932, SGI0, SGI0, SGI0, SGI218933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48587[] = {SGI218934, SGI0, SGI218935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218937, SGI218938, SGI218938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48588[] = {SGI218936, SGI0, SGI218939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218940, SGI218941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48589[] = {SGI218463, SGI0, SGI218464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218468, SGI218470, SGI218470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48590[] = {SGI218942, SGI0, SGI218943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218944, SGI218945, SGI218945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48591[] = {SGI218946, SGI0, SGI218947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218948, SGI218949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48592[] = {SGI218950, SGI0, SGI218951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218952, SGI218953, SGI218953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48593[] = {SGI218954, SGI0, SGI218955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218956, SGI218957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48594[] = {SGI218958, SGI0, SGI218959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218960, SGI218962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48595[] = {SGI218961, SGI0, SGI218963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218964, SGI218965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48596[] = {SGI218966, SGI0, SGI218967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218968, SGI218969, SGI218969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48597[] = {SGI218970, SGI218971, SGI218972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218973, SGI218973, SGI0, SGI0, SGI0, SGI0, SGI218974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48598[] = {SGI218975, SGI218976, SGI218977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218978, SGI218978, SGI0, SGI0, SGI0, SGI0, SGI218979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48599[] = {SGI218980, SGI218981, SGI218982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218983, SGI0, SGI0, SGI0, SGI218984, SGI218984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48600[] = {SGI218985, SGI0, SGI218986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218987, SGI218988, SGI218988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48601[] = {SGI218989, SGI0, SGI218990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218991, SGI218992, SGI218992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48602[] = {SGI218993, SGI218994, SGI218995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218996, SGI218996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI218997, NULL};
const Segment * const SG48603[] = {SGI218998, SGI218999, SGI219000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219001, SGI219001, SGI0, SGI0, SGI219002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48604[] = {SGI219003, SGI219004, SGI219005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219006, SGI0, SGI0, SGI0, SGI219007, SGI219007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48605[] = {SGI219008, SGI0, SGI219009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219010, SGI219011, SGI219011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48606[] = {SGI219012, SGI0, SGI219013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219014, SGI219015, SGI219015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48607[] = {SGI219016, SGI219017, SGI219018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219019, SGI219019, SGI0, SGI0, SGI219020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48608[] = {SGI219021, SGI219022, SGI219023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219024, SGI219024, SGI0, SGI0, SGI0, SGI0, SGI219025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48609[] = {SGI219026, SGI219027, SGI219028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219029, SGI219029, SGI0, SGI0, SGI0, SGI0, SGI219030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48610[] = {SGI219031, SGI0, SGI219032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219033, SGI219034, SGI219034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48611[] = {SGI219035, SGI219036, SGI219037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219038, SGI0, SGI0, SGI0, SGI219039, SGI219039, SGI0, SGI0, SGI0, SGI0, SGI219040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48612[] = {SGI219041, SGI0, SGI219042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219043, SGI219044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48613[] = {SGI219045, SGI0, SGI219046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219047, SGI219048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48614[] = {SGI219049, SGI219050, SGI219051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219052, SGI0, SGI0, SGI0, SGI219053, SGI219053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48615[] = {SGI219054, SGI0, SGI219055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219056, SGI219057, SGI219057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48616[] = {SGI219058, SGI0, SGI219059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219060, SGI219061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48617[] = {SGI219062, SGI0, SGI219063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219066, SGI219067, SGI219067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48618[] = {SGI219064, SGI0, SGI219065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219068, SGI219069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48619[] = {SGI219070, SGI0, SGI219071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219072, SGI219073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48620[] = {SGI219074, SGI0, SGI219075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219076, SGI219077, SGI219077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48621[] = {SGI219078, SGI219079, SGI219080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219083, SGI0, SGI0, SGI0, SGI219084, SGI219084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219086, NULL};
const Segment * const SG48622[] = {SGI219081, SGI0, SGI219082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219085, SGI219087, SGI219087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48623[] = {SGI219088, SGI219089, SGI219090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219091, SGI0, SGI0, SGI0, SGI219092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48624[] = {SGI219093, SGI219094, SGI219095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219099, SGI219099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219102, NULL};
const Segment * const SG48625[] = {SGI219096, SGI219097, SGI219098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219100, SGI0, SGI0, SGI0, SGI219101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48626[] = {SGI219103, SGI0, SGI219104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219106, SGI219108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48627[] = {SGI219105, SGI219107, SGI219109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219110, SGI0, SGI0, SGI0, SGI219111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48628[] = {SGI219112, SGI0, SGI219113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219114, SGI219116, SGI219116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48629[] = {SGI219115, SGI0, SGI219117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219118, SGI0, SGI0, SGI0, SGI219119, SGI219119, SGI0, SGI0, SGI0, SGI219120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48630[] = {SGI219121, SGI0, SGI219122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219123, SGI219124, SGI219124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48631[] = {SGI219125, SGI0, SGI219126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219127, SGI219128, SGI219128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48632[] = {SGI219129, SGI219130, SGI219131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219132, SGI0, SGI0, SGI0, SGI219133, SGI219133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219134, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48633[] = {SGI219135, SGI219136, SGI219137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219138, SGI0, SGI0, SGI0, SGI219139, SGI219139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48634[] = {SGI219140, SGI219141, SGI219142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219143, SGI219143, SGI0, SGI0, SGI0, SGI219144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48635[] = {SGI219145, SGI219146, SGI219147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219148, SGI0, SGI0, SGI0, SGI219149, SGI219149, SGI0, SGI0, SGI0, SGI219150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48636[] = {SGI219151, SGI219152, SGI219153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219154, SGI219154, SGI0, SGI0, SGI0, SGI219155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48637[] = {SGI219156, SGI219157, SGI219158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219161, SGI219161, SGI0, SGI0, SGI0, SGI219164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48638[] = {SGI219159, SGI0, SGI219160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219165, SGI219166, SGI219166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219169, SGI0, SGI0, NULL};
const Segment * const SG48639[] = {SGI219162, SGI0, SGI219163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219167, SGI0, SGI0, SGI0, SGI219168, SGI219168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219170, NULL};
const Segment * const SG48640[] = {SGI219171, SGI219173, SGI219175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219177, SGI0, SGI0, SGI0, SGI219179, SGI219179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48641[] = {SGI219172, SGI219174, SGI219176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219178, SGI219178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219180, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48642[] = {SGI219181, SGI219182, SGI219183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219184, SGI0, SGI0, SGI0, SGI219185, SGI219185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48643[] = {SGI219186, SGI0, SGI219187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219188, SGI219189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48644[] = {SGI219190, SGI0, SGI219192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219194, SGI219196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48645[] = {SGI219191, SGI0, SGI219193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219195, SGI0, SGI0, SGI0, SGI219197, SGI219197, SGI0, SGI0, SGI0, SGI219198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48646[] = {SGI219199, SGI0, SGI219200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219201, SGI219202, SGI219202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219203, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48647[] = {SGI219204, SGI0, SGI219205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219206, SGI219207, SGI219207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219208, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48648[] = {SGI219209, SGI0, SGI219210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219211, SGI219212, SGI219212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48649[] = {SGI219213, SGI219214, SGI219215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219218, SGI0, SGI0, SGI0, SGI219221, SGI219221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48650[] = {SGI219217, SGI0, SGI219220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219223, SGI219225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48651[] = {SGI219216, SGI0, SGI219219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219222, SGI219224, SGI219224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48652[] = {SGI219226, SGI0, SGI219228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219230, SGI219232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48653[] = {SGI219227, SGI0, SGI219229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219231, SGI219233, SGI219233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48654[] = {SGI219234, SGI0, SGI219235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219239, SGI219240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48655[] = {SGI219236, SGI219237, SGI219238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219241, SGI0, SGI0, SGI0, SGI219242, SGI219242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48656[] = {SGI219243, SGI0, SGI219244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219245, SGI219246, SGI219246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48657[] = {SGI219247, SGI219248, SGI219249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219250, SGI219250, SGI0, SGI0, SGI0, SGI219251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48658[] = {SGI219252, SGI0, SGI219253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219254, SGI219255, SGI219255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219256, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48659[] = {SGI219257, SGI0, SGI219258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219259, SGI219260, SGI219260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48660[] = {SGI219261, SGI0, SGI219262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219263, SGI219264, SGI219264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48661[] = {SGI219265, SGI219266, SGI219267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219268, SGI0, SGI0, SGI0, SGI219269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48662[] = {SGI219270, SGI219271, SGI219272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219273, SGI0, SGI0, SGI0, SGI219274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219275, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48663[] = {SGI219276, SGI219277, SGI219278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219279, SGI0, SGI0, SGI0, SGI219280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48664[] = {SGI219281, SGI0, SGI219282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219283, SGI0, SGI0, SGI0, SGI219284, SGI219284, SGI0, SGI0, SGI0, SGI0, SGI219285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48665[] = {SGI219286, SGI0, SGI219287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219290, SGI219291, SGI219291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48666[] = {SGI219288, SGI0, SGI219289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219292, SGI219293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48667[] = {SGI219294, SGI0, SGI219295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219296, SGI219297, SGI219297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48668[] = {SGI219298, SGI0, SGI219299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219300, SGI219301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48669[] = {SGI219302, SGI219303, SGI219304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219305, SGI0, SGI0, SGI0, SGI219307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48670[] = {SGI219306, SGI219308, SGI219309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219310, SGI0, SGI0, SGI0, SGI219311, SGI219311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48671[] = {SGI219312, SGI0, SGI219313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219314, SGI219315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48672[] = {SGI219316, SGI0, SGI219317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219320, SGI219322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48673[] = {SGI219318, SGI219319, SGI219321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219323, SGI0, SGI0, SGI0, SGI219324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48674[] = {SGI219325, SGI219326, SGI219327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219328, SGI219328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219331, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48675[] = {SGI219329, SGI0, SGI219330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219332, SGI219333, SGI219333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48676[] = {SGI219334, SGI0, SGI219335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219336, SGI219337, SGI219337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48677[] = {SGI219338, SGI0, SGI219339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219340, SGI219341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48678[] = {SGI219342, SGI0, SGI219343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219344, SGI219345, SGI219345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48679[] = {SGI219346, SGI0, SGI219347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219348, SGI0, SGI0, SGI0, SGI219349, SGI219349, SGI0, SGI0, SGI0, SGI219350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48680[] = {SGI219351, SGI219352, SGI219353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219354, SGI0, SGI0, SGI0, SGI219355, SGI219355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48681[] = {SGI219357, SGI0, SGI219358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219359, SGI219360, SGI219360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48682[] = {SGI219361, SGI0, SGI219362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219363, SGI219364, SGI219364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48683[] = {SGI219365, SGI0, SGI219366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219367, SGI219368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48684[] = {SGI219369, SGI219370, SGI219371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219372, SGI0, SGI0, SGI0, SGI219373, SGI219373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219374, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48685[] = {SGI219375, SGI0, SGI219376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219377, SGI219378, SGI219378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48686[] = {SGI219379, SGI219380, SGI219381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219382, SGI0, SGI0, SGI0, SGI219383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48687[] = {SGI219384, SGI0, SGI219385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219386, SGI219387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48688[] = {SGI219388, SGI219389, SGI219390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219391, SGI0, SGI0, SGI0, SGI219392, SGI0, SGI0, SGI0, SGI219393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48689[] = {SGI219394, SGI0, SGI219395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219396, SGI219397, SGI219397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48690[] = {SGI219398, SGI219399, SGI219400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219401, SGI0, SGI0, SGI0, SGI219402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219403, SGI0, NULL};
const Segment * const SG48691[] = {SGI219404, SGI0, SGI219405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219407, SGI219409, SGI219409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48692[] = {SGI219406, SGI0, SGI219408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219410, SGI219411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48693[] = {SGI219412, SGI0, SGI219413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219414, SGI219415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48694[] = {SGI219416, SGI0, SGI219417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219418, SGI219419, SGI219419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48695[] = {SGI219420, SGI0, SGI219421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219422, SGI219423, SGI219423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48696[] = {SGI219424, SGI0, SGI219425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219426, SGI219427, SGI219427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48697[] = {SGI219428, SGI0, SGI219429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219430, SGI219431, SGI219431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48698[] = {SGI219432, SGI0, SGI219433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219434, SGI219435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48699[] = {SGI219436, SGI0, SGI219437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219438, SGI219439, SGI219439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48700[] = {SGI219440, SGI0, SGI219441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219442, SGI219443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48701[] = {SGI219444, SGI0, SGI219445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219446, SGI219447, SGI219447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48702[] = {SGI219448, SGI219449, SGI219450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219451, SGI219451, SGI0, SGI0, SGI0, SGI0, SGI219452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48703[] = {SGI219453, SGI0, SGI219454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219455, SGI219456, SGI219456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48704[] = {SGI219457, SGI0, SGI219458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219459, SGI0, SGI0, SGI0, SGI219460, SGI219460, SGI0, SGI0, SGI0, SGI219461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48705[] = {SGI219462, SGI219463, SGI219464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219465, SGI219465, SGI0, SGI0, SGI0, SGI219466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48706[] = {SGI219467, SGI0, SGI219468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219469, SGI219470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48707[] = {SGI219471, SGI0, SGI219472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219473, SGI219474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48708[] = {SGI219475, SGI219476, SGI219477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219478, SGI0, SGI0, SGI0, SGI219479, SGI219479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48709[] = {SGI219481, SGI0, SGI219483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48710[] = {SGI219480, SGI219482, SGI219484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219485, SGI219485, SGI0, SGI0, SGI0, SGI219486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48711[] = {SGI219487, SGI219488, SGI219489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219491, SGI0, SGI0, SGI0, SGI219493, SGI219493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219494, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48712[] = {SGI219490, SGI0, SGI219492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219495, SGI219496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48713[] = {SGI219497, SGI0, SGI219498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219499, SGI219500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48714[] = {SGI219501, SGI0, SGI219502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219503, SGI219504, SGI219504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48715[] = {SGI219505, SGI0, SGI219506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219507, SGI219508, SGI219508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48716[] = {SGI219509, SGI0, SGI219510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219511, SGI219512, SGI219512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48717[] = {SGI219513, SGI219514, SGI219515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219516, SGI0, SGI0, SGI0, SGI219517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48718[] = {SGI219518, SGI0, SGI219519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48719[] = {SGI219520, SGI219521, SGI219522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219523, SGI0, SGI0, SGI0, SGI219525, SGI219525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48720[] = {SGI219524, SGI0, SGI219526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219527, SGI219528, SGI219528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48721[] = {SGI219529, SGI219530, SGI219531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219532, SGI219532, SGI0, SGI0, SGI0, SGI219533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48722[] = {SGI219534, SGI0, SGI219535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219536, SGI219537, SGI219537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48723[] = {SGI219538, SGI219539, SGI219540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219541, SGI219541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219542, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48724[] = {SGI219543, SGI0, SGI219544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219545, SGI219546, SGI219546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48725[] = {SGI219547, SGI0, SGI219548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219549, SGI219550, SGI219550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219551, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48726[] = {SGI219552, SGI0, SGI219553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219554, SGI219555, SGI219555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48727[] = {SGI219556, SGI0, SGI219557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219558, SGI219559, SGI219559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48728[] = {SGI219560, SGI0, SGI219561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219562, SGI219563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48729[] = {SGI219564, SGI0, SGI219565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219566, SGI219567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48730[] = {SGI219568, SGI0, SGI219569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219570, SGI219571, SGI219571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48731[] = {SGI219572, SGI0, SGI219573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219574, SGI219575, SGI219575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48732[] = {SGI219576, SGI0, SGI219577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219578, SGI219579, SGI219579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48733[] = {SGI219580, SGI219581, SGI219582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219583, SGI219583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219584, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48734[] = {SGI219585, SGI219586, SGI219587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219588, SGI219588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48735[] = {SGI219589, SGI219590, SGI219591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219592, SGI0, SGI0, SGI0, SGI219593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48736[] = {SGI219594, SGI0, SGI219595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48737[] = {SGI219596, SGI0, SGI219597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219598, SGI219599, SGI219599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48738[] = {SGI219600, SGI0, SGI219601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219602, SGI219603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48739[] = {SGI219604, SGI219605, SGI219606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219607, SGI0, SGI0, SGI0, SGI219608, SGI219608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48740[] = {SGI219609, SGI219610, SGI219611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219612, SGI219612, SGI0, SGI0, SGI0, SGI0, SGI219613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48741[] = {SGI219614, SGI0, SGI219616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219619, SGI219621, SGI219621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48742[] = {SGI219615, SGI219617, SGI219618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219620, SGI0, SGI0, SGI0, SGI219622, SGI219622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48743[] = {SGI219624, SGI0, SGI219625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219628, SGI219630, SGI219630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48744[] = {SGI219626, SGI219627, SGI219629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219631, SGI0, SGI0, SGI0, SGI219632, SGI219632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48745[] = {SGI219634, SGI0, SGI219635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219636, SGI219637, SGI219637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48746[] = {SGI219638, SGI0, SGI219639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219640, SGI219641, SGI219641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48747[] = {SGI219642, SGI219643, SGI219644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219645, SGI0, SGI0, SGI0, SGI219646, SGI219646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48748[] = {SGI219647, SGI219648, SGI219649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219650, SGI0, SGI0, SGI0, SGI219651, SGI219651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219652, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48749[] = {SGI219653, SGI219654, SGI219655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219656, SGI0, SGI0, SGI0, SGI219657, SGI219657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48750[] = {SGI219658, SGI219659, SGI219660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219661, SGI219661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48751[] = {SGI219662, SGI0, SGI219663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219664, SGI219665, SGI219665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48752[] = {SGI219666, SGI0, SGI219667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219668, SGI219669, SGI219669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219670, SGI0, SGI0, NULL};
const Segment * const SG48753[] = {SGI219671, SGI0, SGI219672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219673, SGI219674, SGI219674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48754[] = {SGI219675, SGI0, SGI219676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219677, SGI219678, SGI219678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48755[] = {SGI219679, SGI219680, SGI219681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219682, SGI0, SGI0, SGI0, SGI219683, SGI219683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48756[] = {SGI219685, SGI0, SGI219686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219687, SGI219688, SGI219688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48757[] = {SGI219689, SGI219690, SGI219691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219692, SGI0, SGI0, SGI0, SGI219693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48758[] = {SGI219694, SGI0, SGI219695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219696, SGI219697, SGI219697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48759[] = {SGI219698, SGI0, SGI219699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219700, SGI219701, SGI219701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48760[] = {SGI219702, SGI219703, SGI219704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219705, SGI0, SGI0, SGI0, SGI219706, SGI219706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48761[] = {SGI219707, SGI0, SGI219708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219709, SGI0, SGI0, SGI0, SGI219710, SGI219710, SGI0, SGI0, SGI0, SGI219711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48762[] = {SGI219712, SGI219713, SGI219714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219715, SGI0, SGI0, SGI0, SGI219716, SGI219716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48763[] = {SGI219717, SGI0, SGI219718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219719, SGI0, SGI0, SGI0, SGI219720, SGI219720, SGI0, SGI0, SGI0, SGI219723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48764[] = {SGI219721, SGI0, SGI219722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48765[] = {SGI219724, SGI219725, SGI219726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219727, SGI0, SGI0, SGI0, SGI219728, SGI219728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48766[] = {SGI219729, SGI0, SGI219730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219731, SGI219733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48767[] = {SGI219732, SGI219734, SGI219735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219736, SGI0, SGI0, SGI0, SGI219737, SGI219737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48768[] = {SGI219738, SGI219739, SGI219740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219741, SGI0, SGI0, SGI0, SGI219742, SGI219742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48769[] = {SGI219743, SGI219744, SGI219745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219747, SGI0, SGI0, SGI0, SGI219749, SGI219749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219750, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48770[] = {SGI219746, SGI0, SGI219748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219751, SGI219752, SGI219752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48771[] = {SGI219753, SGI0, SGI219754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219758, SGI219759, SGI219759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48772[] = {SGI219755, SGI219756, SGI219757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219760, SGI0, SGI0, SGI0, SGI219761, SGI219761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48773[] = {SGI219762, SGI0, SGI219763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219764, SGI219765, SGI219765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48774[] = {SGI219766, SGI0, SGI219767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219770, SGI219771, SGI219771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48775[] = {SGI219768, SGI0, SGI219769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219772, SGI219773, SGI219773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48776[] = {SGI219774, SGI219775, SGI219776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219779, SGI0, SGI0, SGI0, SGI219780, SGI219780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48777[] = {SGI219777, SGI0, SGI219778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219781, SGI219782, SGI219782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48778[] = {SGI219783, SGI219784, SGI219785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219786, SGI219786, SGI0, SGI0, SGI219787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48779[] = {SGI219788, SGI0, SGI219789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219790, SGI219791, SGI219791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219792, SGI0, SGI0, NULL};
const Segment * const SG48780[] = {SGI219793, SGI0, SGI219794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219795, SGI219796, SGI219796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48781[] = {SGI219797, SGI0, SGI219798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219799, SGI219800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48782[] = {SGI219801, SGI0, SGI219802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219803, SGI219804, SGI219804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48783[] = {SGI219805, SGI0, SGI219806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219807, SGI219808, SGI219808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48784[] = {SGI219809, SGI219810, SGI219811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219812, SGI0, SGI0, SGI0, SGI219813, SGI219813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48785[] = {SGI219815, SGI219816, SGI219817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219818, SGI0, SGI0, SGI0, SGI219819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48786[] = {SGI219820, SGI0, SGI219821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219822, SGI219823, SGI219823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48787[] = {SGI219824, SGI219825, SGI219826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219827, SGI0, SGI0, SGI0, SGI219828, SGI219828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219829, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48788[] = {SGI219830, SGI0, SGI219831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219832, SGI219833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48789[] = {SGI219834, SGI219835, SGI219836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219837, SGI219837, SGI0, SGI0, SGI0, SGI219838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48790[] = {SGI219839, SGI0, SGI219840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219841, SGI219842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48791[] = {SGI219843, SGI219844, SGI219845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219846, SGI0, SGI0, SGI0, SGI219847, SGI0, SGI0, SGI219848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48792[] = {SGI219849, SGI0, SGI219850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219851, SGI219852, SGI219852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48793[] = {SGI219853, SGI0, SGI219854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219857, SGI219858, SGI219858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48794[] = {SGI219855, SGI0, SGI219856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219859, SGI219860, SGI219860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48795[] = {SGI219861, SGI0, SGI219862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219863, SGI219864, SGI219864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48796[] = {SGI219865, SGI219866, SGI219867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219871, SGI219871, SGI0, SGI0, SGI0, SGI219874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48797[] = {SGI219868, SGI219869, SGI219870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219872, SGI0, SGI0, SGI0, SGI219873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48798[] = {SGI219875, SGI219876, SGI219877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219878, SGI0, SGI0, SGI0, SGI219879, SGI219879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48799[] = {SGI219880, SGI0, SGI219881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219882, SGI219883, SGI219883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48800[] = {SGI219884, SGI0, SGI219885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219886, SGI219887, SGI219887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48801[] = {SGI219888, SGI0, SGI219889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219890, SGI219891, SGI219891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48802[] = {SGI219892, SGI0, SGI219893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219894, SGI219895, SGI219895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219897, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48803[] = {SGI219896, SGI0, SGI219898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219899, SGI219900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48804[] = {SGI219901, SGI0, SGI219902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219903, SGI219904, SGI219904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48805[] = {SGI219905, SGI0, SGI219906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219907, SGI219908, SGI219908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219909, SGI0, SGI0, NULL};
const Segment * const SG48806[] = {SGI219910, SGI0, SGI219911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219912, SGI219913, SGI219913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48807[] = {SGI219914, SGI0, SGI219915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219916, SGI219917, SGI219917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48808[] = {SGI219918, SGI0, SGI219919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219920, SGI219921, SGI219921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48809[] = {SGI219922, SGI219923, SGI219924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219925, SGI219925, SGI0, SGI0, SGI219926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48810[] = {SGI219927, SGI0, SGI219928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219929, SGI219930, SGI219930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48811[] = {SGI219931, SGI219932, SGI219933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219934, SGI0, SGI0, SGI0, SGI219935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48812[] = {SGI219936, SGI219937, SGI219938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219939, SGI0, SGI0, SGI0, SGI219940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48813[] = {SGI219942, SGI0, SGI219943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219944, SGI219945, SGI219945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48814[] = {SGI219946, SGI219947, SGI219948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219949, SGI0, SGI219950, SGI219950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48815[] = {SGI219951, SGI0, SGI219952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219953, SGI219954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48816[] = {SGI219955, SGI219956, SGI219957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219959, SGI0, SGI0, SGI0, SGI219961, SGI219961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48817[] = {SGI219958, SGI0, SGI219960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219962, SGI219963, SGI219963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48818[] = {SGI219964, SGI219965, SGI219966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219967, SGI0, SGI0, SGI0, SGI219969, SGI219969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48819[] = {SGI219968, SGI0, SGI219970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219971, SGI219972, SGI219972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48820[] = {SGI219973, SGI0, SGI219974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48821[] = {SGI219975, SGI0, SGI219976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219977, SGI219979, SGI219979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48822[] = {SGI219978, SGI0, SGI219980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219981, SGI219982, SGI219982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48823[] = {SGI219983, SGI219984, SGI219985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219988, SGI0, SGI0, SGI0, SGI219990, SGI219990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48824[] = {SGI219986, SGI219987, SGI219989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219991, SGI0, SGI0, SGI0, SGI219992, SGI219992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219994, SGI0, NULL};
const Segment * const SG48825[] = {SGI219995, SGI219996, SGI219997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI219998, SGI0, SGI0, SGI0, SGI219999, SGI219999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220001, SGI0, NULL};
const Segment * const SG48826[] = {SGI220002, SGI0, SGI220003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220004, SGI220005, SGI220005, SGI0, SGI0, SGI0, SGI0, SGI220006, SGI0, NULL};
const Segment * const SG48827[] = {SGI220007, SGI220008, SGI220009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220010, SGI0, SGI0, SGI0, SGI220013, SGI220013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220016, SGI0, NULL};
const Segment * const SG48828[] = {SGI220011, SGI220012, SGI220014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220017, SGI0, SGI0, SGI0, SGI220018, SGI220018, SGI0, SGI0, SGI220019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48829[] = {SGI220020, SGI0, SGI220021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220022, SGI0, SGI0, SGI0, SGI220023, SGI220023, SGI0, SGI0, SGI0, SGI220024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48830[] = {SGI220025, SGI220026, SGI220027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220028, SGI0, SGI0, SGI0, SGI220029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220030, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48831[] = {SGI220031, SGI0, SGI220032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220033, SGI0, SGI0, SGI0, SGI220034, SGI220034, SGI0, SGI0, SGI0, SGI220035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48832[] = {SGI220036, SGI0, SGI220037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220038, SGI220039, SGI220039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48833[] = {SGI220040, SGI0, SGI220041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220042, SGI220043, SGI220043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48834[] = {SGI220044, SGI0, SGI220045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220046, SGI220047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48835[] = {SGI220048, SGI0, SGI220049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220050, SGI220051, SGI220051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48836[] = {SGI220052, SGI220053, SGI220054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220055, SGI0, SGI0, SGI0, SGI220056, SGI220056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220057, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48837[] = {SGI220058, SGI0, SGI220059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48838[] = {SGI220060, SGI0, SGI220061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220062, SGI220063, SGI220063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48839[] = {SGI220064, SGI220065, SGI220066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220067, SGI0, SGI0, SGI0, SGI220068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48840[] = {SGI220069, SGI0, SGI220070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220071, SGI220072, SGI220072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48841[] = {SGI220073, SGI220074, SGI220075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48842[] = {SGI220076, SGI0, SGI220077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220078, SGI220079, SGI220079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48843[] = {SGI220080, SGI220081, SGI220082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220086, SGI0, SGI0, SGI0, SGI220087, SGI220087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220089, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48844[] = {SGI220083, SGI220084, SGI220085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220088, SGI220088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48845[] = {SGI220090, SGI0, SGI220091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220092, SGI220093, SGI220093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48846[] = {SGI220094, SGI0, SGI220095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220096, SGI220097, SGI220097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48847[] = {SGI220098, SGI0, SGI220099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220100, SGI220101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48848[] = {SGI220103, SGI0, SGI220105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220107, SGI220108, SGI220108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48849[] = {SGI220102, SGI220104, SGI220106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220109, SGI220109, SGI0, SGI0, SGI0, SGI0, SGI220111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48850[] = {SGI220110, SGI220112, SGI220113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220114, SGI0, SGI0, SGI0, SGI220115, SGI220115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48851[] = {SGI220116, SGI0, SGI220118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220120, SGI220121, SGI220121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48852[] = {SGI220117, SGI0, SGI220119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220122, SGI220123, SGI220123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48853[] = {SGI220124, SGI220125, SGI220126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220127, SGI0, SGI0, SGI0, SGI220128, SGI220128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48854[] = {SGI220129, SGI0, SGI220130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220131, SGI220132, SGI220132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48855[] = {SGI220133, SGI0, SGI220134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220138, SGI0, SGI0, SGI0, SGI220139, SGI220139, SGI0, SGI0, SGI220141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48856[] = {SGI220135, SGI220136, SGI220137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220140, SGI220140, SGI0, SGI0, SGI0, SGI220142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48857[] = {SGI220143, SGI0, SGI220144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220145, SGI220146, SGI220146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220147, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48858[] = {SGI220148, SGI0, SGI220149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220150, SGI220151, SGI220151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48859[] = {SGI220152, SGI220153, SGI220154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220155, SGI220155, SGI0, SGI0, SGI220156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48860[] = {SGI220157, SGI0, SGI220158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220159, SGI220160, SGI220160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48861[] = {SGI220161, SGI220162, SGI220163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220164, SGI220164, SGI0, SGI0, SGI0, SGI220165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48862[] = {SGI220166, SGI0, SGI220167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220170, SGI220171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48863[] = {SGI220168, SGI0, SGI220169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220172, SGI220173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48864[] = {SGI220174, SGI0, SGI220175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220176, SGI220177, SGI220177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48865[] = {SGI220178, SGI0, SGI220179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220180, SGI0, SGI0, SGI0, SGI220181, SGI220181, SGI0, SGI0, SGI0, SGI0, SGI220182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48866[] = {SGI220183, SGI0, SGI220184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220185, SGI220186, SGI220186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48867[] = {SGI220187, SGI220188, SGI220189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220190, SGI0, SGI0, SGI0, SGI220191, SGI220191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220192, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48868[] = {SGI220193, SGI220194, SGI220195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220196, SGI0, SGI0, SGI0, SGI220197, SGI220197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220198, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48869[] = {SGI220199, SGI220200, SGI220201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220202, SGI220202, SGI0, SGI0, SGI0, SGI220203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48870[] = {SGI220204, SGI0, SGI220205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220206, SGI220207, SGI220207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220208, SGI0, SGI0, NULL};
const Segment * const SG48871[] = {SGI220209, SGI0, SGI220210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220211, SGI220212, SGI220212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220213, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48872[] = {SGI220214, SGI0, SGI220216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220218, SGI220220, SGI220220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220222, SGI0, SGI0, NULL};
const Segment * const SG48873[] = {SGI220215, SGI0, SGI220217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220219, SGI220221, SGI220221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48874[] = {SGI220223, SGI0, SGI220224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220225, SGI220226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48875[] = {SGI220227, SGI220228, SGI220229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220230, SGI220230, SGI0, SGI0, SGI0, SGI220231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48876[] = {SGI220232, SGI0, SGI220233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220234, SGI220235, SGI220235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220236, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48877[] = {SGI220237, SGI0, SGI220238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220239, SGI220240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48878[] = {SGI220241, SGI0, SGI220242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220243, SGI220244, SGI220244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48879[] = {SGI220245, SGI0, SGI220246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220247, SGI220248, SGI220248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220249, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48880[] = {SGI220250, SGI0, SGI220251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220252, SGI220253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48881[] = {SGI220254, SGI0, SGI220255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220256, SGI220257, SGI220257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48882[] = {SGI220258, SGI220259, SGI220260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220261, SGI0, SGI0, SGI0, SGI220262, SGI220262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48883[] = {SGI220263, SGI0, SGI220264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220265, SGI220267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48884[] = {SGI220266, SGI0, SGI220268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220269, SGI220270, SGI220270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48885[] = {SGI220271, SGI0, SGI220272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220273, SGI220274, SGI220274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48886[] = {SGI220275, SGI220276, SGI220277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220278, SGI0, SGI0, SGI0, SGI220279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48887[] = {SGI220280, SGI0, SGI220281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220284, SGI220285, SGI220285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48888[] = {SGI220282, SGI0, SGI220283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220286, SGI220287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48889[] = {SGI220288, SGI0, SGI220290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48890[] = {SGI220289, SGI0, SGI220291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220292, SGI220293, SGI220293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48891[] = {SGI220294, SGI0, SGI220295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220296, SGI0, SGI0, SGI0, SGI220297, SGI220297, SGI0, SGI0, SGI0, SGI220298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48892[] = {SGI220299, SGI0, SGI220300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220304, SGI220305, SGI220305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48893[] = {SGI220301, SGI220302, SGI220303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220306, SGI0, SGI0, SGI0, SGI220307, SGI220307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48894[] = {SGI220308, SGI220309, SGI220310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220311, SGI220311, SGI0, SGI0, SGI220312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48895[] = {SGI220313, SGI0, SGI220314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220315, SGI220316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48896[] = {SGI220317, SGI0, SGI220318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220319, SGI220320, SGI220320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48897[] = {SGI220321, SGI0, SGI220322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220325, SGI220326, SGI220326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48898[] = {SGI220323, SGI0, SGI220324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220327, SGI220328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48899[] = {SGI220329, SGI220330, SGI220331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220332, SGI220332, SGI0, SGI0, SGI220333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48900[] = {SGI220334, SGI220335, SGI220336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220337, SGI0, SGI0, SGI0, SGI220338, SGI220338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220339, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48901[] = {SGI220340, SGI0, SGI220341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220342, SGI220343, SGI220343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48902[] = {SGI220344, SGI0, SGI220345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220346, SGI220347, SGI220347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48903[] = {SGI220348, SGI0, SGI220349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220350, SGI220351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48904[] = {SGI220352, SGI220353, SGI220354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220357, SGI0, SGI0, SGI0, SGI220358, SGI220358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220360, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48905[] = {SGI220355, SGI0, SGI220356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220359, SGI220361, SGI220361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48906[] = {SGI220362, SGI220363, SGI220364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220365, SGI0, SGI0, SGI0, SGI220366, SGI220366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48907[] = {SGI220368, SGI220369, SGI220370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220371, SGI0, SGI0, SGI0, SGI220373, SGI220373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48908[] = {SGI220372, SGI0, SGI220374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220376, SGI220377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48909[] = {SGI220378, SGI220379, SGI220380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220381, SGI0, SGI0, SGI0, SGI220382, SGI220382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48910[] = {SGI220383, SGI0, SGI220384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220385, SGI220386, SGI220386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48911[] = {SGI220387, SGI0, SGI220388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220390, SGI220392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48912[] = {SGI220389, SGI0, SGI220391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220393, SGI220394, SGI220394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48913[] = {SGI220395, SGI0, SGI220396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220399, SGI220400, SGI220400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220401, SGI0, SGI0, NULL};
const Segment * const SG48914[] = {SGI220397, SGI0, SGI220398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220402, SGI220403, SGI220403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48915[] = {SGI220404, SGI0, SGI220405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220408, SGI220409, SGI220409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48916[] = {SGI220406, SGI0, SGI220407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220412, SGI220413, SGI220413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220414, SGI0, SGI0, NULL};
const Segment * const SG48917[] = {SGI220410, SGI0, SGI220411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220415, SGI220416, SGI220416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48918[] = {SGI220417, SGI0, SGI220418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220419, SGI220420, SGI220420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48919[] = {SGI220421, SGI0, SGI220422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220423, SGI0, SGI0, SGI0, SGI220424, SGI220424, SGI0, SGI0, SGI0, SGI220425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48920[] = {SGI220426, SGI0, SGI220427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220428, SGI220429, SGI220429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48921[] = {SGI220430, SGI0, SGI220431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220435, SGI220436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48922[] = {SGI220432, SGI220433, SGI220434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220437, SGI0, SGI0, SGI0, SGI220438, SGI220438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48923[] = {SGI220439, SGI0, SGI220440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48924[] = {SGI220441, SGI0, SGI220442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220443, SGI220444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48925[] = {SGI220445, SGI0, SGI220446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220447, SGI220448, SGI220448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48926[] = {SGI220449, SGI220450, SGI220451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220452, SGI0, SGI0, SGI0, SGI220453, SGI220453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220454, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48927[] = {SGI220455, SGI0, SGI220456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220457, SGI220458, SGI220458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48928[] = {SGI220459, SGI0, SGI220460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220461, SGI220462, SGI220462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48929[] = {SGI220463, SGI0, SGI220464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220465, SGI220466, SGI220466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48930[] = {SGI220467, SGI0, SGI220468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220469, SGI220470, SGI220470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48931[] = {SGI220471, SGI0, SGI220472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220473, SGI220474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48932[] = {SGI220475, SGI0, SGI220476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220477, SGI220478, SGI220478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48933[] = {SGI220479, SGI0, SGI220480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220481, SGI220482, SGI220482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48934[] = {SGI220483, SGI0, SGI220484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220485, SGI220486, SGI220486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48935[] = {SGI220487, SGI0, SGI220488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220489, SGI220490, SGI220490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48936[] = {SGI220491, SGI220492, SGI220493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220494, SGI0, SGI0, SGI0, SGI220495, SGI220495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48937[] = {SGI220497, SGI220498, SGI220499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220500, SGI0, SGI0, SGI0, SGI220501, SGI220501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48938[] = {SGI220502, SGI220503, SGI220504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220505, SGI220505, SGI0, SGI0, SGI0, SGI220506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48939[] = {SGI220507, SGI0, SGI220508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220509, SGI220510, SGI220510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48940[] = {SGI220511, SGI0, SGI220512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220516, SGI220517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48941[] = {SGI220513, SGI220514, SGI220515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220518, SGI0, SGI0, SGI0, SGI220519, SGI220519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48942[] = {SGI220521, SGI220522, SGI220523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220526, SGI220526, SGI0, SGI0, SGI0, SGI220528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48943[] = {SGI220524, SGI220525, SGI220527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220529, SGI0, SGI0, SGI0, SGI220530, SGI220530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48944[] = {SGI220531, SGI220532, SGI220533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220534, SGI220534, SGI0, SGI0, SGI0, SGI220535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48945[] = {SGI220536, SGI0, SGI220537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220538, SGI220539, SGI220539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48946[] = {SGI220540, SGI0, SGI220541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220542, SGI220543, SGI220543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48947[] = {SGI220544, SGI0, SGI220545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220546, SGI220547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48948[] = {SGI220548, SGI220549, SGI220550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220551, SGI220551, SGI0, SGI0, SGI0, SGI0, SGI220552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48949[] = {SGI220553, SGI220554, SGI220555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220556, SGI220556, SGI0, SGI0, SGI0, SGI0, SGI220557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48950[] = {SGI220558, SGI220559, SGI220560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220561, SGI220561, SGI0, SGI0, SGI0, SGI0, SGI220562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48951[] = {SGI220563, SGI220564, SGI220565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220566, SGI0, SGI0, SGI0, SGI220567, SGI220567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48952[] = {SGI220568, SGI220569, SGI220570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220571, SGI0, SGI0, SGI0, SGI220572, SGI220572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48953[] = {SGI220573, SGI220574, SGI220575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220576, SGI0, SGI220577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48954[] = {SGI220578, SGI0, SGI220579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220580, SGI220581, SGI220581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48955[] = {SGI220582, SGI220583, SGI220584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220585, SGI0, SGI0, SGI0, SGI220586, SGI220586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48956[] = {SGI220588, SGI0, SGI220589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220590, SGI220591, SGI220591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48957[] = {SGI220592, SGI220593, SGI220594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220595, SGI0, SGI0, SGI0, SGI220596, SGI220596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48958[] = {SGI220598, SGI0, SGI220599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220602, SGI220604, SGI220604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48959[] = {SGI220600, SGI220601, SGI220603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220605, SGI0, SGI0, SGI0, SGI220606, SGI220606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48960[] = {SGI220610, SGI0, SGI220612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220615, SGI220616, SGI220616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48961[] = {SGI220608, SGI220609, SGI220611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220613, SGI0, SGI0, SGI0, SGI220614, SGI220614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48962[] = {SGI220618, SGI0, SGI220619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220623, SGI220624, SGI220624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48963[] = {SGI220620, SGI220621, SGI220622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220625, SGI0, SGI0, SGI0, SGI220626, SGI220626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48964[] = {SGI220628, SGI0, SGI220629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220630, SGI220631, SGI220631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48965[] = {SGI0, SGI220632, SGI220633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220634, SGI0, SGI0, SGI0, SGI0, SGI220635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48966[] = {SGI220636, SGI220637, SGI220638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220639, SGI0, SGI0, SGI0, SGI220640, SGI220640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48967[] = {SGI220641, SGI220642, SGI220643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220644, SGI0, SGI0, SGI0, SGI220645, SGI220645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220646, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48968[] = {SGI220648, SGI220649, SGI220650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220651, SGI0, SGI0, SGI0, SGI220652, SGI220652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48969[] = {SGI220654, SGI0, SGI220655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220656, SGI220657, SGI220657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48970[] = {SGI220658, SGI0, SGI220659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220660, SGI220661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48971[] = {SGI220662, SGI220663, SGI220664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220665, SGI220665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220666, NULL};
const Segment * const SG48972[] = {SGI220667, SGI0, SGI220668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220669, SGI220670, SGI220670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48973[] = {SGI220671, SGI0, SGI220672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220673, SGI220674, SGI220674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48974[] = {SGI220675, SGI0, SGI220676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220677, SGI0, SGI0, SGI0, SGI220678, SGI220678, SGI0, SGI0, SGI0, SGI220679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48975[] = {SGI220680, SGI220681, SGI220683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220685, SGI0, SGI0, SGI0, SGI220686, SGI220686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48976[] = {SGI220682, SGI0, SGI220684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220687, SGI220688, SGI220688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48977[] = {SGI220689, SGI0, SGI220690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220691, SGI220692, SGI220692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48978[] = {SGI220695, SGI0, SGI220696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220699, SGI220700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48979[] = {SGI220693, SGI0, SGI220694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220697, SGI220698, SGI220698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48980[] = {SGI220701, SGI0, SGI220702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220703, SGI220704, SGI220704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48981[] = {SGI220705, SGI0, SGI220706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220707, SGI220708, SGI220708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48982[] = {SGI220709, SGI0, SGI220710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220711, SGI0, SGI0, SGI0, SGI220712, SGI220712, SGI0, SGI0, SGI0, SGI220713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48983[] = {SGI220714, SGI220715, SGI220716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220717, SGI0, SGI0, SGI0, SGI220718, SGI220718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48984[] = {SGI220719, SGI0, SGI220720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48985[] = {SGI220721, SGI0, SGI220722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220723, SGI220724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48986[] = {SGI0, SGI220647, SGI220725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220726, SGI0, SGI0, SGI0, SGI0, SGI220727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48987[] = {SGI220728, SGI220729, SGI220730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220731, SGI220731, SGI0, SGI0, SGI220732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48988[] = {SGI220733, SGI220734, SGI220735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220736, SGI220736, SGI0, SGI0, SGI220737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48989[] = {SGI220738, SGI0, SGI220739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220740, SGI0, SGI0, SGI0, SGI220741, SGI220741, SGI0, SGI0, SGI0, SGI220742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48990[] = {SGI220743, SGI0, SGI220744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220745, SGI220746, SGI220746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48991[] = {SGI220747, SGI220748, SGI220749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220750, SGI220750, SGI0, SGI0, SGI0, SGI0, SGI220751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48992[] = {SGI220752, SGI0, SGI220753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220754, SGI220755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48993[] = {SGI220756, SGI220757, SGI220758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220759, SGI220759, SGI0, SGI0, SGI0, SGI0, SGI220760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48994[] = {SGI220761, SGI0, SGI220762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220763, SGI220764, SGI220764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48995[] = {SGI220765, SGI220766, SGI220767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220768, SGI0, SGI0, SGI0, SGI220769, SGI220769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220770, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48996[] = {SGI220771, SGI0, SGI220772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220773, SGI220774, SGI220774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48997[] = {SGI220775, SGI220776, SGI220777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220778, SGI0, SGI0, SGI0, SGI220779, SGI220779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48998[] = {SGI220780, SGI0, SGI220781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG48999[] = {SGI220782, SGI220783, SGI220784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220785, SGI0, SGI0, SGI0, SGI220786, SGI220786, SGI0, SGI0, SGI220787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49000[] = {SGI220788, SGI0, SGI220789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220790, SGI220791, SGI220791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49001[] = {SGI220792, SGI0, SGI220793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220794, SGI220795, SGI220795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49002[] = {SGI220796, SGI0, SGI220797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220798, SGI220799, SGI220799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49003[] = {SGI220800, SGI0, SGI220801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220802, SGI220803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49004[] = {SGI220804, SGI220805, SGI220806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220807, SGI220807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220809, NULL};
const Segment * const SG49005[] = {SGI220808, SGI220810, SGI220811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220812, SGI0, SGI0, SGI0, SGI220813, SGI220813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220815, SGI0, NULL};
const Segment * const SG49006[] = {SGI220816, SGI0, SGI220817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220818, SGI220820, SGI220820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49007[] = {SGI220819, SGI220821, SGI220822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220823, SGI220823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220824, NULL};
const Segment * const SG49008[] = {SGI220825, SGI0, SGI220826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220827, SGI220828, SGI220828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49009[] = {SGI220829, SGI0, SGI220830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220831, SGI220832, SGI220832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49010[] = {SGI220833, SGI220834, SGI220835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220836, SGI0, SGI0, SGI0, SGI220837, SGI220837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49011[] = {SGI220839, SGI0, SGI220840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220841, SGI220842, SGI220842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220845, SGI0, SGI0, NULL};
const Segment * const SG49012[] = {SGI220843, SGI220844, SGI220846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220847, SGI220847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220848, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49013[] = {SGI220849, SGI0, SGI220850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220851, SGI220852, SGI220852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49014[] = {SGI220853, SGI220854, SGI220855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220856, SGI220856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220857, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49015[] = {SGI220858, SGI0, SGI220859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220860, SGI220861, SGI220861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49016[] = {SGI220862, SGI0, SGI220863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220864, SGI220865, SGI220865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49017[] = {SGI220866, SGI0, SGI220867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220868, SGI220869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49018[] = {SGI220870, SGI220871, SGI220872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220873, SGI220873, SGI0, SGI0, SGI0, SGI220874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49019[] = {SGI220876, SGI0, SGI220878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220880, SGI220882, SGI220882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49020[] = {SGI220875, SGI0, SGI220877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220879, SGI220881, SGI220881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220883, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49021[] = {SGI220884, SGI0, SGI220885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220886, SGI220887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49022[] = {SGI220888, SGI220889, SGI220890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220891, SGI0, SGI0, SGI0, SGI220892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49023[] = {SGI220893, SGI220894, SGI220895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49024[] = {SGI220896, SGI0, SGI220897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49025[] = {SGI220898, SGI0, SGI220899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220900, SGI220901, SGI220901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220902, SGI0, SGI0, NULL};
const Segment * const SG49026[] = {SGI220903, SGI0, SGI220904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220905, SGI220906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220907, SGI0, SGI0, NULL};
const Segment * const SG49027[] = {SGI220908, SGI0, SGI220909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220910, SGI220911, SGI220911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220912, SGI0, SGI0, NULL};
const Segment * const SG49028[] = {SGI220913, SGI220914, SGI220915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220916, SGI0, SGI0, SGI0, SGI220917, SGI220917, SGI0, SGI220918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49029[] = {SGI220919, SGI0, SGI220920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220921, SGI220922, SGI220922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49030[] = {SGI220923, SGI0, SGI220924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220925, SGI220926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49031[] = {SGI220927, SGI220928, SGI220929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220930, SGI0, SGI0, SGI0, SGI220931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220932, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49032[] = {SGI220933, SGI0, SGI220934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220937, SGI220938, SGI220938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49033[] = {SGI220935, SGI0, SGI220936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220939, SGI0, SGI0, SGI0, SGI220940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220941, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49034[] = {SGI220942, SGI220943, SGI220944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220945, SGI0, SGI0, SGI0, SGI220946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49035[] = {SGI220947, SGI220948, SGI220949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220950, SGI0, SGI0, SGI0, SGI220951, SGI220951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49036[] = {SGI220952, SGI220953, SGI220954, SGI0, SGI220955, SGI220955, SGI0, NULL};
const Segment * const SG49037[] = {SGI220956, SGI0, SGI220958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220960, SGI220962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49038[] = {SGI220957, SGI0, SGI220959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220961, SGI220963, SGI220963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49039[] = {SGI220964, SGI220965, SGI220966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220969, SGI0, SGI0, SGI0, SGI220970, SGI220970, SGI0, SGI220972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49040[] = {SGI220967, SGI0, SGI220968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220971, SGI220973, SGI220973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49041[] = {SGI220974, SGI0, SGI220975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220976, SGI220977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49042[] = {SGI220978, SGI0, SGI220979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220980, SGI220981, SGI220981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49043[] = {SGI220982, SGI0, SGI220983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220984, SGI220985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49044[] = {SGI220986, SGI0, SGI220987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220988, SGI220989, SGI220989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49045[] = {SGI220990, SGI220991, SGI220992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220993, SGI0, SGI0, SGI0, SGI220994, SGI0, SGI0, SGI220995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49046[] = {SGI220996, SGI0, SGI220997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI220998, SGI220999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49047[] = {SGI221000, SGI0, SGI221001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221002, SGI221003, SGI221003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49048[] = {SGI221004, SGI0, SGI221005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221006, SGI221007, SGI221007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49049[] = {SGI221008, SGI0, SGI221009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221010, SGI221011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49050[] = {SGI221012, SGI0, SGI221013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221014, SGI221015, SGI221015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49051[] = {SGI221016, SGI221017, SGI221018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221019, SGI0, SGI0, SGI0, SGI221020, SGI221020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221021, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49052[] = {SGI221022, SGI221023, SGI221024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221025, SGI0, SGI0, SGI0, SGI221026, SGI221026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221027, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49053[] = {SGI221029, SGI0, SGI221032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221034, SGI221036, SGI221036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49054[] = {SGI221028, SGI221030, SGI221031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221033, SGI0, SGI0, SGI0, SGI221035, SGI221035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49055[] = {SGI221037, SGI0, SGI221038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221039, SGI221040, SGI221040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49056[] = {SGI221041, SGI0, SGI221042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221043, SGI221044, SGI221044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49057[] = {SGI221045, SGI0, SGI221046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221047, SGI221048, SGI221048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49058[] = {SGI221049, SGI0, SGI221050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221051, SGI221052, SGI221052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49059[] = {SGI221053, SGI221054, SGI221055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221056, SGI0, SGI0, SGI0, SGI221057, SGI221057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49060[] = {SGI221058, SGI0, SGI221059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221060, SGI221061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49061[] = {SGI221062, SGI221063, SGI221064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221065, SGI0, SGI0, SGI0, SGI221066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49062[] = {SGI221071, SGI0, SGI221072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221073, SGI221074, SGI221074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49063[] = {SGI221075, SGI0, SGI221076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221077, SGI221078, SGI221078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49064[] = {SGI0, SGI221079, SGI221080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221081, SGI0, SGI0, SGI0, SGI221082, SGI221082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49065[] = {SGI221083, SGI221084, SGI221085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221086, SGI221086, SGI0, SGI0, SGI0, SGI0, SGI221088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49066[] = {SGI221087, SGI0, SGI221089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221090, SGI221092, SGI221092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221095, SGI0, SGI0, NULL};
const Segment * const SG49067[] = {SGI221091, SGI221093, SGI221094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49068[] = {SGI221096, SGI221097, SGI221098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221099, SGI0, SGI0, SGI0, SGI221100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49069[] = {SGI221101, SGI0, SGI221102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49070[] = {SGI221103, SGI0, SGI221104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221105, SGI221106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49071[] = {SGI221107, SGI0, SGI221108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221109, SGI221110, SGI221110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49072[] = {SGI221111, SGI0, SGI221112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221113, SGI221114, SGI221114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49073[] = {SGI221115, SGI221116, SGI221117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221118, SGI0, SGI0, SGI0, SGI221119, SGI221119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221121, SGI0, NULL};
const Segment * const SG49074[] = {SGI221122, SGI0, SGI221123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221124, SGI221125, SGI221125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49075[] = {SGI221126, SGI0, SGI221127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221128, SGI221129, SGI221129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49076[] = {SGI221130, SGI221131, SGI221132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221133, SGI0, SGI0, SGI0, SGI221134, SGI221134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221135, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49077[] = {SGI221136, SGI221137, SGI221138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221139, SGI0, SGI0, SGI0, SGI221140, SGI221140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49078[] = {SGI221141, SGI0, SGI221142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221143, SGI221144, SGI221144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49079[] = {SGI221145, SGI0, SGI221146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221147, SGI221148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49080[] = {SGI221149, SGI0, SGI221150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221151, SGI221152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49081[] = {SGI221153, SGI0, SGI221154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221155, SGI221156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49082[] = {SGI221157, SGI0, SGI221158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221159, SGI221160, SGI221160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49083[] = {SGI221161, SGI221162, SGI221163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221164, SGI0, SGI0, SGI0, SGI221165, SGI221165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49084[] = {SGI221166, SGI0, SGI221167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221168, SGI221169, SGI221169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49085[] = {SGI221170, SGI0, SGI221171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221172, SGI221173, SGI221173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49086[] = {SGI221174, SGI0, SGI221175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221176, SGI221177, SGI221177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49087[] = {SGI221178, SGI221179, SGI221180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221181, SGI221181, SGI0, SGI0, SGI0, SGI0, SGI221182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49088[] = {SGI221067, SGI0, SGI221068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221069, SGI221070, SGI221070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49089[] = {SGI221183, SGI0, SGI221184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221185, SGI221186, SGI221186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49090[] = {SGI221187, SGI0, SGI221188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221189, SGI221190, SGI221190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49091[] = {SGI221191, SGI0, SGI221192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221193, SGI221194, SGI221194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49092[] = {SGI221195, SGI221196, SGI221197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221198, SGI221198, SGI0, SGI0, SGI0, SGI221199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49093[] = {SGI221200, SGI0, SGI221201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221205, SGI221206, SGI221206, SGI0, SGI0, NULL};
const Segment * const SG49094[] = {SGI221202, SGI221203, SGI221204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221207, SGI0, SGI0, SGI0, SGI221208, SGI221208, SGI0, SGI221209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49095[] = {SGI221210, SGI0, SGI221211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221212, SGI221213, SGI221213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49096[] = {SGI221214, SGI221215, SGI221216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221217, SGI221217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221218, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49097[] = {SGI221219, SGI0, SGI221220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221221, SGI221222, SGI221222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49098[] = {SGI221223, SGI0, SGI221224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221225, SGI221226, SGI221226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49099[] = {SGI221227, SGI221228, SGI221229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221231, SGI221231, SGI0, SGI0, SGI0, SGI221234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49100[] = {SGI221230, SGI221232, SGI221233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221235, SGI0, SGI0, SGI0, SGI221236, SGI221236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49101[] = {SGI221237, SGI0, SGI221238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221239, SGI221240, SGI221240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221241, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49102[] = {SGI221242, SGI0, SGI221243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221244, SGI221245, SGI221245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49103[] = {SGI221246, SGI0, SGI221247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221248, SGI221249, SGI221249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49104[] = {SGI221250, SGI0, SGI221251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221252, SGI221253, SGI221253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49105[] = {SGI221254, SGI0, SGI221255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221258, SGI221259, SGI221259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49106[] = {SGI221256, SGI0, SGI221257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221261, SGI221263, SGI221263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49107[] = {SGI221260, SGI221262, SGI221264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221265, SGI0, SGI0, SGI0, SGI221266, SGI221266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221267, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49108[] = {SGI221268, SGI0, SGI221269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221271, SGI221272, SGI221272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49109[] = {SGI221270, SGI221273, SGI221274, SGI221275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221276, SGI0, SGI0, SGI0, SGI221277, SGI221278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49110[] = {SGI221279, SGI0, SGI221280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221281, SGI221282, SGI221282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49111[] = {SGI221283, SGI0, SGI221284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221285, SGI221286, SGI221286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49112[] = {SGI221287, SGI221288, SGI221289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221290, SGI0, SGI0, SGI0, SGI221291, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49113[] = {SGI221292, SGI0, SGI221293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221294, SGI221295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49114[] = {SGI221296, SGI0, SGI221297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221298, SGI221299, SGI221299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49115[] = {SGI221300, SGI0, SGI221301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221302, SGI221303, SGI221303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49116[] = {SGI221304, SGI0, SGI221305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221306, SGI221307, SGI221307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49117[] = {SGI221308, SGI0, SGI221309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49118[] = {SGI221310, SGI0, SGI221311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221312, SGI221313, SGI221313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49119[] = {SGI221314, SGI0, SGI221315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221316, SGI221317, SGI221317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49120[] = {SGI221318, SGI221319, SGI221320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221321, SGI0, SGI0, SGI0, SGI221322, SGI221322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49121[] = {SGI221323, SGI221324, SGI221325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221326, SGI0, SGI0, SGI0, SGI221327, SGI221327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49122[] = {SGI221328, SGI0, SGI221329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221330, SGI221331, SGI221331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49123[] = {SGI221332, SGI0, SGI221333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221334, SGI221335, SGI221335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49124[] = {SGI221336, SGI0, SGI221337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221338, SGI221339, SGI221339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49125[] = {SGI221340, SGI221341, SGI221342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221343, SGI0, SGI0, SGI0, SGI221344, SGI221344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221345, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49126[] = {SGI221346, SGI0, SGI221347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221348, SGI0, SGI0, SGI0, SGI221349, SGI221349, SGI0, SGI0, SGI0, SGI221350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49127[] = {SGI0, SGI221351, SGI0, SGI221352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49128[] = {SGI221353, SGI0, SGI221354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221355, SGI221356, SGI221356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49129[] = {SGI221357, SGI0, SGI221358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221359, SGI221360, SGI221360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49130[] = {SGI221361, SGI221362, SGI221363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221364, SGI0, SGI0, SGI0, SGI221365, SGI221365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49131[] = {SGI221366, SGI0, SGI221367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221368, SGI221369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49132[] = {SGI221370, SGI0, SGI221372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49133[] = {SGI221371, SGI221373, SGI221374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221375, SGI0, SGI0, SGI0, SGI221376, SGI221376, SGI0, SGI221377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49134[] = {SGI221379, SGI221381, SGI221382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221386, SGI0, SGI0, SGI0, SGI221388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49135[] = {SGI221383, SGI0, SGI221385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221390, SGI221391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49136[] = {SGI221378, SGI221380, SGI221384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221387, SGI0, SGI0, SGI0, SGI221389, SGI221389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221392, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49137[] = {SGI221393, SGI0, SGI221394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221395, SGI221396, SGI221396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49138[] = {SGI221397, SGI0, SGI221398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221399, SGI221400, SGI221400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49139[] = {SGI221401, SGI0, SGI221402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221403, SGI0, SGI0, SGI0, SGI221404, SGI221404, SGI0, SGI0, SGI0, SGI221405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49140[] = {SGI221406, SGI221407, SGI221408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221409, SGI0, SGI0, SGI0, SGI221410, SGI221410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221411, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49141[] = {SGI221414, SGI221416, SGI221417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221419, SGI0, SGI0, SGI0, SGI221420, SGI221420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221423, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49142[] = {SGI221412, SGI221413, SGI221415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221418, SGI0, SGI0, SGI0, SGI221421, SGI0, SGI221422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49143[] = {SGI221424, SGI221425, SGI221426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221427, SGI0, SGI0, SGI0, SGI221428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49144[] = {SGI221429, SGI0, SGI221430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221431, SGI221432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49145[] = {SGI221433, SGI0, SGI221434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221435, SGI221436, SGI221436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49146[] = {SGI221437, SGI0, SGI221438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221439, SGI221440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49147[] = {SGI221441, SGI0, SGI221442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221443, SGI221444, SGI221444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49148[] = {SGI221445, SGI0, SGI221446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221447, SGI221448, SGI221448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49149[] = {SGI221449, SGI0, SGI221450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221451, SGI221452, SGI221452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49150[] = {SGI221453, SGI0, SGI221454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221455, SGI221456, SGI221456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49151[] = {SGI221457, SGI221458, SGI221459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221460, SGI0, SGI0, SGI0, SGI221461, SGI221461, SGI0, SGI221462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49152[] = {SGI221463, SGI221464, SGI221465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221469, SGI0, SGI0, SGI0, SGI221470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49153[] = {SGI221466, SGI221467, SGI221468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221471, SGI0, SGI0, SGI0, SGI221472, SGI221472, SGI0, SGI221473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49154[] = {SGI221474, SGI0, SGI221475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221476, SGI221477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49155[] = {SGI221478, SGI221479, SGI221480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221482, SGI0, SGI0, SGI0, SGI221485, SGI221485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49156[] = {SGI221481, SGI221483, SGI221484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221486, SGI0, SGI0, SGI0, SGI221487, SGI221487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221488, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49157[] = {SGI221489, SGI0, SGI221490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221493, SGI221494, SGI221494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49158[] = {SGI221491, SGI0, SGI221492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221495, SGI221496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49159[] = {SGI221497, SGI0, SGI221498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221499, SGI221500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49160[] = {SGI221501, SGI0, SGI221502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221503, SGI221504, SGI221504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49161[] = {SGI221505, SGI0, SGI221506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221507, SGI221508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49162[] = {SGI221509, SGI0, SGI221510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221511, SGI221512, SGI221512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49163[] = {SGI221513, SGI221514, SGI221515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221516, SGI0, SGI0, SGI0, SGI221517, SGI221517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49164[] = {SGI221518, SGI221519, SGI221520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221521, SGI0, SGI0, SGI0, SGI221522, SGI221522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49165[] = {SGI221523, SGI0, SGI221524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221525, SGI221526, SGI221526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49166[] = {SGI221527, SGI0, SGI221528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221529, SGI221530, SGI221530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49167[] = {SGI221531, SGI0, SGI221532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221533, SGI221534, SGI221534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49168[] = {SGI221535, SGI0, SGI221536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221539, SGI221540, SGI221540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49169[] = {SGI221537, SGI0, SGI221538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221541, SGI221542, SGI221542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49170[] = {SGI221543, SGI0, SGI221544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221545, SGI221546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49171[] = {SGI221547, SGI221548, SGI221549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221550, SGI0, SGI0, SGI0, SGI221551, SGI221551, SGI0, SGI0, SGI221552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49172[] = {SGI221553, SGI0, SGI221554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221555, SGI221556, SGI221556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49173[] = {SGI221557, SGI0, SGI221558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221559, SGI221560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49174[] = {SGI221561, SGI0, SGI221562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221563, SGI221564, SGI221564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49175[] = {SGI221565, SGI0, SGI221566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221567, SGI221568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49176[] = {SGI221569, SGI221570, SGI221571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221572, SGI0, SGI0, SGI0, SGI221573, SGI221573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221574, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49177[] = {SGI221575, SGI0, SGI221576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221577, SGI221578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49178[] = {SGI221579, SGI0, SGI221580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221581, SGI221582, SGI221582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49179[] = {SGI221583, SGI0, SGI221584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221585, SGI221586, SGI221586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49180[] = {SGI221587, SGI0, SGI221588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221589, SGI221590, SGI221590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49181[] = {SGI221591, SGI221592, SGI221593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221594, SGI0, SGI0, SGI0, SGI221595, SGI221595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49182[] = {SGI221596, SGI221597, SGI221598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221599, SGI0, SGI0, SGI0, SGI221600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49183[] = {SGI221601, SGI0, SGI221602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49184[] = {SGI221603, SGI0, SGI221604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221605, SGI221606, SGI221606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49185[] = {SGI221607, SGI0, SGI221609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221611, SGI221613, SGI221613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49186[] = {SGI221608, SGI0, SGI221610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221612, SGI221614, SGI221614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49187[] = {SGI221615, SGI221617, SGI221619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221621, SGI0, SGI0, SGI0, SGI221623, SGI221623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49188[] = {SGI221616, SGI221618, SGI221620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221622, SGI0, SGI0, SGI0, SGI221624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49189[] = {SGI221625, SGI221626, SGI221627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221628, SGI0, SGI0, SGI0, SGI221629, SGI221629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49190[] = {SGI221630, SGI221631, SGI221632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221633, SGI221633, SGI0, SGI0, SGI221634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49191[] = {SGI221635, SGI221636, SGI221637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221638, SGI0, SGI0, SGI0, SGI221639, SGI221639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49192[] = {SGI221640, SGI221641, SGI221642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221644, SGI0, SGI0, SGI0, SGI221646, SGI221646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49193[] = {SGI221643, SGI0, SGI221645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221647, SGI221648, SGI221648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221649, SGI0, SGI0, NULL};
const Segment * const SG49194[] = {SGI221650, SGI0, SGI221651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221652, SGI221653, SGI221653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49195[] = {SGI221654, SGI0, SGI221655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221656, SGI221657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49196[] = {SGI221658, SGI0, SGI221659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221660, SGI221661, SGI221661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221662, SGI0, SGI0, NULL};
const Segment * const SG49197[] = {SGI221663, SGI0, SGI221665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221668, SGI221669, SGI221669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49198[] = {SGI221664, SGI221666, SGI221667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221670, SGI221670, SGI0, SGI0, SGI221671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49199[] = {SGI221672, SGI0, SGI221673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221674, SGI221675, SGI221675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49200[] = {SGI221676, SGI0, SGI221677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221678, SGI221679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49201[] = {SGI221680, SGI221681, SGI221682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221683, SGI221683, SGI0, SGI0, SGI221684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49202[] = {SGI221685, SGI0, SGI221686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221687, SGI221688, SGI221688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49203[] = {SGI221689, SGI0, SGI221690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221691, SGI221692, SGI221692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49204[] = {SGI221693, SGI0, SGI221694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221695, SGI221696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49205[] = {SGI221697, SGI0, SGI221698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221699, SGI221700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49206[] = {SGI221701, SGI0, SGI221702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221703, SGI221704, SGI221704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221705, SGI0, SGI0, NULL};
const Segment * const SG49207[] = {SGI221706, SGI221707, SGI221708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221709, SGI0, SGI0, SGI0, SGI221710, SGI221710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221711, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49208[] = {SGI221712, SGI221713, SGI221714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221715, SGI221715, SGI0, SGI0, SGI0, SGI221716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49209[] = {SGI221717, SGI221718, SGI221719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221720, SGI221720, SGI0, SGI0, SGI0, SGI221721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49210[] = {SGI221722, SGI221723, SGI221724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221725, SGI221725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221726, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49211[] = {SGI221727, SGI221728, SGI221729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221730, SGI0, SGI0, SGI0, SGI221731, SGI221731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49212[] = {SGI221732, SGI221733, SGI221734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221735, SGI221735, SGI0, SGI0, SGI0, SGI221736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49213[] = {SGI221737, SGI221738, SGI221739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221740, SGI0, SGI0, SGI0, SGI221741, SGI221741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49214[] = {SGI221742, SGI0, SGI221743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221744, SGI221745, SGI221745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49215[] = {SGI221746, SGI0, SGI221748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221750, SGI221751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49216[] = {SGI221747, SGI0, SGI221749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221752, SGI221753, SGI221753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49217[] = {SGI221754, SGI0, SGI221755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221756, SGI221758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49218[] = {SGI221757, SGI0, SGI221759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221760, SGI221761, SGI221761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49219[] = {SGI221762, SGI0, SGI221763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221764, SGI221765, SGI221765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49220[] = {SGI221766, SGI0, SGI221767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221768, SGI221769, SGI221769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49221[] = {SGI221770, SGI0, SGI221771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221772, SGI0, SGI0, SGI0, SGI221773, SGI221773, SGI0, SGI0, SGI0, SGI0, SGI221774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49222[] = {SGI221775, SGI221776, SGI221778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221781, SGI0, SGI0, SGI0, SGI221783, SGI221784, SGI0, SGI0, SGI0, SGI0, SGI221786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49223[] = {SGI221777, SGI221779, SGI221780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221782, SGI0, SGI0, SGI0, SGI221785, SGI221785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221787, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49224[] = {SGI221788, SGI221789, SGI221790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221791, SGI0, SGI0, SGI0, SGI221792, SGI221792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221793, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49225[] = {SGI221794, SGI221795, SGI221796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221797, SGI221797, SGI0, SGI0, SGI0, SGI221798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49226[] = {SGI221799, SGI0, SGI221800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221801, SGI221802, SGI221802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221803, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49227[] = {SGI221804, SGI0, SGI221805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221806, SGI221807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49228[] = {SGI221808, SGI0, SGI221809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221810, SGI221811, SGI221811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49229[] = {SGI221812, SGI0, SGI221813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221814, SGI221815, SGI221815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49230[] = {SGI221816, SGI0, SGI221817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221818, SGI221819, SGI221819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49231[] = {SGI221820, SGI0, SGI221821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221822, SGI221823, SGI221823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49232[] = {SGI221824, SGI0, SGI221825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221826, SGI221827, SGI221827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49233[] = {SGI221828, SGI221829, SGI221830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221831, SGI221831, SGI0, SGI0, SGI0, SGI221832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49234[] = {SGI221833, SGI221834, SGI221835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221836, SGI221836, SGI0, SGI0, SGI221837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49235[] = {SGI221838, SGI221839, SGI221840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221841, SGI0, SGI0, SGI0, SGI221842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49236[] = {SGI221843, SGI0, SGI221844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221845, SGI0, SGI0, SGI0, SGI221846, SGI221846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221847, NULL};
const Segment * const SG49237[] = {SGI221848, SGI221849, SGI221850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221851, SGI0, SGI0, SGI0, SGI221852, SGI221852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49238[] = {SGI221853, SGI221854, SGI221855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221856, SGI221856, SGI0, SGI0, SGI221857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49239[] = {SGI221858, SGI0, SGI221859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221860, SGI221861, SGI221861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49240[] = {SGI221862, SGI0, SGI221863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221864, SGI221865, SGI221865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49241[] = {SGI221866, SGI0, SGI221867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221868, SGI221869, SGI221869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49242[] = {SGI221870, SGI0, SGI221871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221872, SGI221873, SGI221873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49243[] = {SGI221874, SGI0, SGI221875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221876, SGI221877, SGI221877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49244[] = {SGI221878, SGI0, SGI221879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221880, SGI221881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49245[] = {SGI221882, SGI0, SGI221883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221884, SGI221885, SGI221885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49246[] = {SGI221886, SGI0, SGI221887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221891, SGI221892, SGI221892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49247[] = {SGI221888, SGI221889, SGI221890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221893, SGI0, SGI0, SGI0, SGI221894, SGI221894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49248[] = {SGI221896, SGI0, SGI221897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221898, SGI0, SGI0, SGI0, SGI221899, SGI221899, SGI0, SGI0, SGI0, SGI221900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49249[] = {SGI221901, SGI0, SGI221902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221903, SGI221904, SGI221904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49250[] = {SGI221905, SGI0, SGI221906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221909, SGI221911, SGI221911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49251[] = {SGI221907, SGI221908, SGI221910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221912, SGI221912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221913, NULL};
const Segment * const SG49252[] = {SGI221914, SGI0, SGI221915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221917, SGI221918, SGI221918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49253[] = {SGI221916, SGI0, SGI221919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221920, SGI221921, SGI221921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49254[] = {SGI0, SGI221922, SGI0, SGI221923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49255[] = {SGI221924, SGI0, SGI221925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221926, SGI221927, SGI221927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49256[] = {SGI221928, SGI0, SGI221929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221930, SGI221931, SGI221931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49257[] = {SGI221932, SGI221933, SGI221934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221935, SGI0, SGI0, SGI0, SGI221936, SGI221936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221937, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49258[] = {SGI221938, SGI221939, SGI221940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221941, SGI221941, SGI0, SGI0, SGI0, SGI221942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49259[] = {SGI221943, SGI0, SGI221944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221945, SGI221946, SGI221946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49260[] = {SGI221947, SGI221948, SGI221949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221950, SGI0, SGI0, SGI0, SGI221951, SGI221951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49261[] = {SGI221952, SGI221953, SGI221954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221955, SGI0, SGI0, SGI0, SGI221956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49262[] = {SGI221957, SGI0, SGI221958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221959, SGI221960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49263[] = {SGI221961, SGI0, SGI221962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221963, SGI221964, SGI221964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49264[] = {SGI221965, SGI0, SGI221966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221967, SGI221968, SGI221968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49265[] = {SGI221969, SGI0, SGI221970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221971, SGI221972, SGI221972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49266[] = {SGI221973, SGI0, SGI221974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221975, SGI221976, SGI221976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49267[] = {SGI221977, SGI0, SGI221978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221979, SGI221980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49268[] = {SGI221981, SGI0, SGI221982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221983, SGI221984, SGI221984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49269[] = {SGI221985, SGI221986, SGI221987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221988, SGI0, SGI0, SGI0, SGI221989, SGI221989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221990, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49270[] = {SGI221991, SGI0, SGI221992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221993, SGI221994, SGI221994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49271[] = {SGI221995, SGI221996, SGI221997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI221999, SGI221999, SGI0, SGI0, SGI222001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49272[] = {SGI221998, SGI0, SGI222000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222002, SGI222003, SGI222003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49273[] = {SGI222004, SGI222005, SGI222006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222007, SGI0, SGI0, SGI0, SGI222008, SGI222008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222009, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49274[] = {SGI222010, SGI0, SGI222011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222013, SGI222015, SGI222015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49275[] = {SGI222012, SGI0, SGI222014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222016, SGI222017, SGI222017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49276[] = {SGI222018, SGI0, SGI222019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222020, SGI0, SGI0, SGI0, SGI222021, SGI222021, SGI0, SGI0, SGI0, SGI222022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49277[] = {SGI222023, SGI0, SGI222024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222025, SGI222026, SGI222026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49278[] = {SGI222027, SGI0, SGI222028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222029, SGI222030, SGI222030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49279[] = {SGI222031, SGI222032, SGI222033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222035, SGI0, SGI0, SGI0, SGI222037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49280[] = {SGI222034, SGI0, SGI222036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222038, SGI222039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49281[] = {SGI222040, SGI0, SGI222041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222042, SGI222043, SGI222043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49282[] = {SGI222044, SGI0, SGI222045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222046, SGI222047, SGI222047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49283[] = {SGI222048, SGI0, SGI222049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222050, SGI222051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49284[] = {SGI222052, SGI0, SGI222053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222054, SGI222055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49285[] = {SGI222056, SGI0, SGI222057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222058, SGI222059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49286[] = {SGI222060, SGI0, SGI222061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222062, SGI222063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49287[] = {SGI222064, SGI0, SGI222065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222066, SGI222067, SGI222067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49288[] = {SGI222068, SGI0, SGI222069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222070, SGI222071, SGI222071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222072, SGI0, SGI0, NULL};
const Segment * const SG49289[] = {SGI222073, SGI0, SGI222074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222075, SGI0, SGI0, SGI0, SGI222076, SGI222076, SGI0, SGI0, SGI0, SGI222077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49290[] = {SGI222078, SGI0, SGI222079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222080, SGI222081, SGI222081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49291[] = {SGI222082, SGI222083, SGI222084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222085, SGI0, SGI0, SGI0, SGI222086, SGI222086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49292[] = {SGI222087, SGI222088, SGI222089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222090, SGI0, SGI0, SGI0, SGI222091, SGI222091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49293[] = {SGI222093, SGI0, SGI222094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222095, SGI222096, SGI222096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49294[] = {SGI222097, SGI0, SGI222098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222099, SGI222100, SGI222100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49295[] = {SGI222101, SGI0, SGI222102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222103, SGI222104, SGI222104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49296[] = {SGI222105, SGI0, SGI222106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222107, SGI222108, SGI222108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49297[] = {SGI222109, SGI222110, SGI222111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222112, SGI222112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222113, NULL};
const Segment * const SG49298[] = {SGI222114, SGI0, SGI222115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222116, SGI222117, SGI222117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49299[] = {SGI222118, SGI0, SGI222119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222120, SGI222121, SGI222121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222122, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49300[] = {SGI222123, SGI222124, SGI222125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222126, SGI0, SGI0, SGI0, SGI222127, SGI222127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49301[] = {SGI222128, SGI222129, SGI222130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222131, SGI0, SGI0, SGI222132, SGI222133, SGI222133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49302[] = {SGI222134, SGI0, SGI222135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222136, SGI222137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49303[] = {SGI222138, SGI222139, SGI222140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222141, SGI222141, SGI0, SGI0, SGI0, SGI222142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49304[] = {SGI222143, SGI222144, SGI222145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222146, SGI222146, SGI0, SGI0, SGI0, SGI222147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49305[] = {SGI222148, SGI222149, SGI222150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222151, SGI222151, SGI0, SGI0, SGI0, SGI222152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49306[] = {SGI222153, SGI0, SGI222154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222156, SGI222158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49307[] = {SGI222155, SGI0, SGI222157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222159, SGI222160, SGI222160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49308[] = {SGI222161, SGI222162, SGI222163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222164, SGI0, SGI0, SGI0, SGI222165, SGI222165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49309[] = {SGI222166, SGI0, SGI222167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222168, SGI222169, SGI222169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49310[] = {SGI222170, SGI222171, SGI222172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222173, SGI0, SGI0, SGI0, SGI222174, SGI222174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49311[] = {SGI222175, SGI0, SGI222177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222179, SGI222180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49312[] = {SGI222176, SGI0, SGI222178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222181, SGI222182, SGI222182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49313[] = {SGI222183, SGI0, SGI222184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222185, SGI222186, SGI222186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49314[] = {SGI222187, SGI0, SGI222188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222189, SGI222190, SGI222190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49315[] = {SGI222191, SGI222192, SGI222193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222194, SGI0, SGI0, SGI0, SGI222195, SGI222195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49316[] = {SGI222196, SGI222197, SGI222198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222199, SGI0, SGI0, SGI0, SGI222200, SGI222200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49317[] = {SGI222201, SGI0, SGI222202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222203, SGI222204, SGI222204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49318[] = {SGI222205, SGI0, SGI222206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222207, SGI222208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49319[] = {SGI222209, SGI222210, SGI222211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222212, SGI0, SGI0, SGI0, SGI222213, SGI222213, SGI0, SGI222214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49320[] = {SGI222215, SGI222216, SGI222217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222218, SGI0, SGI0, SGI0, SGI222219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49321[] = {SGI222220, SGI222221, SGI222222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222223, SGI0, SGI0, SGI0, SGI222224, SGI222224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49322[] = {SGI222225, SGI0, SGI222226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49323[] = {SGI222227, SGI222228, SGI222229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222230, SGI0, SGI0, SGI0, SGI222231, SGI222231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49324[] = {SGI222232, SGI0, SGI222233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222234, SGI222235, SGI222235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49325[] = {SGI222236, SGI0, SGI222237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222238, SGI222239, SGI222239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49326[] = {SGI222240, SGI222241, SGI222242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222243, SGI0, SGI0, SGI0, SGI222244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49327[] = {SGI222246, SGI0, SGI222247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222248, SGI222249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222250, SGI0, SGI0, NULL};
const Segment * const SG49328[] = {SGI222251, SGI0, SGI222252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222253, SGI222254, SGI222254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49329[] = {SGI222255, SGI222256, SGI222257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222258, SGI222258, SGI0, SGI0, SGI222259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49330[] = {SGI222260, SGI0, SGI222261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222262, SGI222263, SGI222263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49331[] = {SGI222264, SGI0, SGI222265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222266, SGI222267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49332[] = {SGI222268, SGI0, SGI222269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222270, SGI0, SGI0, SGI0, SGI222271, SGI222271, SGI0, SGI0, SGI0, SGI222272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49333[] = {SGI222273, SGI0, SGI222274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222275, SGI222276, SGI222276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49334[] = {SGI222277, SGI222278, SGI222279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222280, SGI0, SGI0, SGI0, SGI222281, SGI222281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49335[] = {SGI222282, SGI0, SGI222283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222284, SGI222285, SGI222285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49336[] = {SGI222286, SGI0, SGI222287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222288, SGI222289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222290, SGI0, NULL};
const Segment * const SG49337[] = {SGI222291, SGI0, SGI222292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222293, SGI222294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49338[] = {SGI222295, SGI222296, SGI222297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222298, SGI0, SGI0, SGI0, SGI222299, SGI222299, SGI0, SGI0, NULL};
const Segment * const SG49339[] = {SGI222300, SGI222301, SGI222302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222304, SGI0, SGI0, SGI0, SGI222307, SGI222307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222308, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49340[] = {SGI222303, SGI222305, SGI222306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222309, SGI222309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222310, NULL};
const Segment * const SG49341[] = {SGI222311, SGI222312, SGI222313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222314, SGI222314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222315, NULL};
const Segment * const SG49342[] = {SGI222316, SGI0, SGI222317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222319, SGI222321, SGI222321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49343[] = {SGI222318, SGI0, SGI222320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222322, SGI222323, SGI222323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49344[] = {SGI222324, SGI0, SGI222325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222326, SGI222327, SGI222327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49345[] = {SGI222328, SGI0, SGI222329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222330, SGI222331, SGI222331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49346[] = {SGI222332, SGI0, SGI222333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222334, SGI222335, SGI222335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49347[] = {SGI222336, SGI0, SGI222337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222338, SGI222340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49348[] = {SGI222339, SGI222341, SGI222342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222343, SGI0, SGI0, SGI0, SGI222344, SGI222344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49349[] = {SGI222346, SGI0, SGI222347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222350, SGI222351, SGI222351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49350[] = {SGI222348, SGI0, SGI222349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222352, SGI222353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49351[] = {SGI222354, SGI0, SGI222355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222356, SGI222357, SGI222357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49352[] = {SGI222358, SGI0, SGI222359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222360, SGI222361, SGI222361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49353[] = {SGI222362, SGI0, SGI222363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222364, SGI222365, SGI222365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49354[] = {SGI222366, SGI0, SGI222367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222368, SGI222369, SGI222369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49355[] = {SGI222370, SGI0, SGI222371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222372, SGI222373, SGI222373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49356[] = {SGI222374, SGI0, SGI222375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222376, SGI222377, SGI222377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49357[] = {SGI222378, SGI0, SGI222379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222380, SGI222381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49358[] = {SGI222382, SGI0, SGI222383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222384, SGI0, SGI0, SGI0, SGI222385, SGI222385, SGI0, SGI0, SGI0, SGI222386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49359[] = {SGI222387, SGI0, SGI222388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222389, SGI222390, SGI222390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49360[] = {SGI222391, SGI222392, SGI222393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222394, SGI0, SGI0, SGI0, SGI222395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49361[] = {SGI222396, SGI222397, SGI222398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222399, SGI0, SGI0, SGI0, SGI222400, SGI222400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49362[] = {SGI222401, SGI0, SGI222402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222403, SGI222404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49363[] = {SGI222405, SGI0, SGI222406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222407, SGI222408, SGI222408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49364[] = {SGI222409, SGI222410, SGI222411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222412, SGI0, SGI0, SGI0, SGI222413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49365[] = {SGI222414, SGI0, SGI222415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222416, SGI222417, SGI222417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49366[] = {SGI222418, SGI0, SGI222419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222420, SGI222421, SGI222421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49367[] = {SGI222422, SGI0, SGI222423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222424, SGI222425, SGI222425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49368[] = {SGI222426, SGI0, SGI222427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222428, SGI222429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49369[] = {SGI222430, SGI222431, SGI222432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222433, SGI0, SGI0, SGI0, SGI222435, SGI0, SGI222438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49370[] = {SGI222434, SGI222436, SGI222437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222439, SGI0, SGI0, SGI0, SGI222440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49371[] = {SGI222441, SGI0, SGI222442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222443, SGI0, SGI0, SGI0, SGI222444, SGI222444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222445, NULL};
const Segment * const SG49372[] = {SGI222446, SGI0, SGI222447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222448, SGI222449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222450, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49373[] = {SGI222451, SGI0, SGI222452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222453, SGI222454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49374[] = {SGI222455, SGI0, SGI222456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222457, SGI222458, SGI222458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49375[] = {SGI222459, SGI0, SGI222460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222461, SGI222462, SGI222462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49376[] = {SGI222463, SGI0, SGI222464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222465, SGI222466, SGI222466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49377[] = {SGI222467, SGI222468, SGI222469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222470, SGI0, SGI0, SGI0, SGI222471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49378[] = {SGI222472, SGI0, SGI222473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222474, SGI222475, SGI222475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49379[] = {SGI222476, SGI0, SGI222477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222478, SGI222479, SGI222479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49380[] = {SGI222480, SGI0, SGI222481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222482, SGI222483, SGI222483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222484, SGI0, SGI0, NULL};
const Segment * const SG49381[] = {SGI222485, SGI222487, SGI222488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222490, SGI0, SGI0, SGI0, SGI222491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49382[] = {SGI222486, SGI0, SGI222489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222492, SGI222493, SGI222493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222494, SGI0, SGI0, NULL};
const Segment * const SG49383[] = {SGI222495, SGI222496, SGI222497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222498, SGI222498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222499, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49384[] = {SGI222500, SGI222501, SGI222502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222503, SGI222503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222504, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49385[] = {SGI222505, SGI0, SGI222506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222509, SGI222510, SGI222510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49386[] = {SGI222507, SGI0, SGI222508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222511, SGI222512, SGI222512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222513, SGI0, SGI0, NULL};
const Segment * const SG49387[] = {SGI222514, SGI0, SGI222515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222516, SGI222517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49388[] = {SGI222518, SGI0, SGI222519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222520, SGI222521, SGI222521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49389[] = {SGI222522, SGI0, SGI222523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222524, SGI222525, SGI222525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49390[] = {SGI222526, SGI0, SGI222527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222528, SGI222529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222530, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49391[] = {SGI222531, SGI0, SGI222532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222533, SGI222534, SGI222534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49392[] = {SGI222537, SGI0, SGI222538, SGI0, SGI222541, SGI222541, SGI0, NULL};
const Segment * const SG49393[] = {SGI222535, SGI0, SGI222536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222539, SGI222540, SGI222540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49394[] = {SGI222542, SGI0, SGI222543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222544, SGI222545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49395[] = {SGI222546, SGI0, SGI222547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222548, SGI222549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49396[] = {SGI222550, SGI0, SGI222551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222552, SGI222553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49397[] = {SGI222554, SGI0, SGI222555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222559, SGI222560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49398[] = {SGI222556, SGI222557, SGI222558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222561, SGI0, SGI0, SGI0, SGI222562, SGI222562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49399[] = {SGI222563, SGI222564, SGI222565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222566, SGI0, SGI0, SGI0, SGI222567, SGI0, SGI222568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49400[] = {SGI222569, SGI0, SGI222570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222571, SGI222572, SGI222572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49401[] = {SGI222573, SGI222574, SGI222575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222576, SGI222576, SGI0, SGI0, SGI222577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49402[] = {SGI222578, SGI0, SGI222580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222582, SGI222583, SGI222583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49403[] = {SGI222579, SGI0, SGI222581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222584, SGI222585, SGI222585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49404[] = {SGI222586, SGI0, SGI222587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222588, SGI222589, SGI222589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49405[] = {SGI222590, SGI0, SGI222591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222592, SGI222593, SGI222593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49406[] = {SGI222594, SGI0, SGI222595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222596, SGI222597, SGI222597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49407[] = {SGI222598, SGI0, SGI222599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222600, SGI222601, SGI222601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49408[] = {SGI222602, SGI222603, SGI222604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222605, SGI0, SGI0, SGI0, SGI222606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49409[] = {SGI222608, SGI222609, SGI222610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222611, SGI0, SGI0, SGI0, SGI222612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49410[] = {SGI222614, SGI0, SGI222615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222616, SGI222617, SGI222617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49411[] = {SGI222618, SGI0, SGI222619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222620, SGI222621, SGI222621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49412[] = {SGI222622, SGI0, SGI222623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222624, SGI222625, SGI222625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49413[] = {SGI222626, SGI222627, SGI222628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222630, SGI0, SGI0, SGI0, SGI222632, SGI222632, SGI0, SGI222634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49414[] = {SGI222629, SGI0, SGI222631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222636, SGI222637, SGI222637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49415[] = {SGI222633, SGI0, SGI222635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222638, SGI222639, SGI222639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49416[] = {SGI222640, SGI222641, SGI222642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222643, SGI0, SGI0, SGI0, SGI222644, SGI222644, SGI0, SGI222645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49417[] = {SGI222646, SGI0, SGI222648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222650, SGI222652, SGI222652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49418[] = {SGI222647, SGI0, SGI222649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222651, SGI222653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49419[] = {SGI222654, SGI0, SGI222655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222656, SGI222657, SGI222657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49420[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222658, SGI0, SGI0, SGI222659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49421[] = {SGI222660, SGI222661, SGI222662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222663, SGI0, SGI0, SGI0, SGI222664, SGI222664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49422[] = {SGI222665, SGI0, SGI222666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222667, SGI222668, SGI222668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49423[] = {SGI222669, SGI222670, SGI222671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222672, SGI0, SGI0, SGI0, SGI222673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49424[] = {SGI222674, SGI0, SGI222675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222676, SGI222677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49425[] = {SGI222678, SGI222679, SGI222680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222681, SGI0, SGI0, SGI0, SGI222682, SGI222682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222683, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49426[] = {SGI222684, SGI0, SGI222685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222686, SGI222687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49427[] = {SGI222688, SGI0, SGI222689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222690, SGI222691, SGI222691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49428[] = {SGI222692, SGI222693, SGI222694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222695, SGI0, SGI0, SGI0, SGI222696, SGI222696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49429[] = {SGI222698, SGI0, SGI222699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222700, SGI222702, SGI222702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49430[] = {SGI222701, SGI0, SGI222703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222704, SGI222705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222706, SGI0, SGI0, NULL};
const Segment * const SG49431[] = {SGI222707, SGI0, SGI222708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222709, SGI0, SGI0, SGI0, SGI222710, SGI222710, SGI0, SGI0, SGI0, SGI222711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49432[] = {SGI222712, SGI0, SGI222713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222714, SGI0, SGI0, SGI0, SGI222715, SGI222715, SGI0, SGI0, SGI0, SGI222716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49433[] = {SGI222717, SGI0, SGI222718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222721, SGI222722, SGI222722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49434[] = {SGI222719, SGI0, SGI222720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222723, SGI222724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49435[] = {SGI222725, SGI0, SGI222726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222727, SGI222728, SGI222728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49436[] = {SGI222729, SGI0, SGI222730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222731, SGI222732, SGI222732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49437[] = {SGI222733, SGI0, SGI222734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222735, SGI222736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49438[] = {SGI222737, SGI0, SGI222738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222739, SGI222740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49439[] = {SGI222741, SGI0, SGI222742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222743, SGI222744, SGI222744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49440[] = {SGI222745, SGI222746, SGI222747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222748, SGI0, SGI0, SGI0, SGI222749, SGI222749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49441[] = {SGI222750, SGI222751, SGI222752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222753, SGI0, SGI0, SGI0, SGI222754, SGI222754, SGI0, SGI0, SGI222755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49442[] = {SGI222756, SGI0, SGI222757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222758, SGI222759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49443[] = {SGI222760, SGI0, SGI222761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222762, SGI222763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49444[] = {SGI222764, SGI0, SGI222765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222766, SGI222767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49445[] = {SGI222768, SGI222769, SGI222770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222771, SGI0, SGI0, SGI0, SGI222772, SGI222772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49446[] = {SGI222773, SGI0, SGI222774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222775, SGI222776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49447[] = {SGI222777, SGI222778, SGI222779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222780, SGI0, SGI0, SGI0, SGI222782, SGI222782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49448[] = {SGI222781, SGI0, SGI222783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222784, SGI222785, SGI222785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49449[] = {SGI222786, SGI0, SGI222787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222788, SGI222789, SGI222789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49450[] = {SGI222790, SGI222791, SGI222792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222793, SGI0, SGI0, SGI0, SGI222794, SGI222794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222795, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49451[] = {SGI222796, SGI0, SGI222797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222798, SGI0, SGI0, SGI0, SGI222800, SGI222800, SGI0, SGI0, SGI222803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49452[] = {SGI222799, SGI222801, SGI222802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222804, SGI0, SGI0, SGI0, SGI222805, SGI222805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222806, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49453[] = {SGI222807, SGI0, SGI222808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222809, SGI222810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49454[] = {SGI222811, SGI0, SGI222812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222813, SGI0, SGI0, SGI0, SGI222814, SGI222814, SGI0, SGI0, SGI0, SGI222815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49455[] = {SGI222816, SGI222817, SGI222818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222819, SGI0, SGI0, SGI0, SGI222820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49456[] = {SGI222821, SGI0, SGI222822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222823, SGI222825, SGI222825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49457[] = {SGI222824, SGI0, SGI222826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222827, SGI222828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49458[] = {SGI222829, SGI222830, SGI222831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222833, SGI0, SGI0, SGI0, SGI222835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49459[] = {SGI222832, SGI0, SGI222834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222837, SGI222838, SGI222838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222839, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49460[] = {SGI222840, SGI222841, SGI222842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222843, SGI0, SGI0, SGI0, SGI222844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49461[] = {SGI222846, SGI0, SGI222847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222850, SGI222851, SGI222851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49462[] = {SGI222848, SGI0, SGI222849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222852, SGI222853, SGI222853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49463[] = {SGI222854, SGI0, SGI222855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222856, SGI222857, SGI222857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49464[] = {SGI222858, SGI222859, SGI222860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222861, SGI0, SGI0, SGI0, SGI222862, SGI222862, SGI0, SGI222863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49465[] = {SGI222864, SGI222865, SGI222866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222867, SGI0, SGI0, SGI0, SGI222868, SGI222868, SGI0, SGI222869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49466[] = {SGI222870, SGI222871, SGI222872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222873, SGI0, SGI0, SGI0, SGI222874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49467[] = {SGI222875, SGI0, SGI222876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222877, SGI222878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49468[] = {SGI222879, SGI222880, SGI222881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222882, SGI0, SGI0, SGI0, SGI222883, SGI222883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49469[] = {SGI222885, SGI222886, SGI222887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222888, SGI0, SGI0, SGI0, SGI222890, SGI222890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49470[] = {SGI222889, SGI222891, SGI222892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222893, SGI0, SGI0, SGI0, SGI222895, SGI222895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49471[] = {SGI222894, SGI222896, SGI222897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222898, SGI222898, SGI0, SGI0, SGI0, SGI222899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49472[] = {SGI222900, SGI0, SGI222901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222902, SGI222903, SGI222903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49473[] = {SGI222904, SGI0, SGI222905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222908, SGI222909, SGI222909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49474[] = {SGI222906, SGI0, SGI222907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222910, SGI222911, SGI222911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222912, SGI0, SGI0, NULL};
const Segment * const SG49475[] = {SGI222913, SGI222914, SGI222915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222919, SGI0, SGI0, SGI0, SGI222920, SGI222920, SGI0, SGI222922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49476[] = {SGI222916, SGI222917, SGI222918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222921, SGI222921, SGI0, SGI0, SGI0, SGI222923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49477[] = {SGI222924, SGI0, SGI222925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222926, SGI222927, SGI222927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49478[] = {SGI222928, SGI222929, SGI222930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222931, SGI222931, SGI0, SGI0, SGI0, SGI222932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49479[] = {SGI222933, SGI0, SGI222934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222935, SGI222936, SGI222936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49480[] = {SGI222937, SGI0, SGI222938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222940, SGI222941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49481[] = {SGI222945, SGI0, SGI222948, SGI0, SGI222950, SGI222950, SGI0, NULL};
const Segment * const SG49482[] = {SGI222939, SGI222942, SGI222943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222946, SGI222946, SGI0, SGI0, SGI0, SGI222949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49483[] = {SGI222944, SGI0, SGI222947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222951, SGI222952, SGI222952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49484[] = {SGI222953, SGI0, SGI222954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222955, SGI222956, SGI222956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49485[] = {SGI222957, SGI0, SGI222958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222959, SGI222960, SGI222960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49486[] = {SGI222961, SGI0, SGI222962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222963, SGI222964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49487[] = {SGI222965, SGI0, SGI222966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222967, SGI222968, SGI222968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49488[] = {SGI222969, SGI0, SGI222970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222971, SGI222972, SGI222972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49489[] = {SGI222973, SGI0, SGI222974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222975, SGI222976, SGI222976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49490[] = {SGI222977, SGI222978, SGI222979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222980, SGI0, SGI0, SGI0, SGI222981, SGI222981, SGI0, SGI222982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49491[] = {SGI222983, SGI222984, SGI222985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222986, SGI0, SGI0, SGI0, SGI222987, SGI222987, SGI0, SGI222988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49492[] = {SGI222989, SGI222990, SGI222991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222992, SGI0, SGI0, SGI0, SGI222993, SGI222993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49493[] = {SGI222995, SGI222996, SGI222997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI222998, SGI0, SGI0, SGI0, SGI222999, SGI222999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49494[] = {SGI223001, SGI223002, SGI223003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223004, SGI0, SGI0, SGI0, SGI223005, SGI223005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49495[] = {SGI223006, SGI223007, SGI223008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223009, SGI0, SGI0, SGI0, SGI223010, SGI223010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49496[] = {SGI223011, SGI223012, SGI223013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223014, SGI0, SGI0, SGI0, SGI223015, SGI223015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49497[] = {SGI223016, SGI0, SGI223017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223018, SGI223019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49498[] = {SGI223020, SGI0, SGI223021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223022, SGI223023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49499[] = {SGI223024, SGI0, SGI223025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223026, SGI223027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49500[] = {SGI223028, SGI0, SGI223029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223030, SGI223031, SGI223031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49501[] = {SGI223032, SGI0, SGI223033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223034, SGI223035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49502[] = {SGI223036, SGI0, SGI223037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223038, SGI223039, SGI223039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49503[] = {SGI223040, SGI223041, SGI223042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223043, SGI0, SGI0, SGI0, SGI223044, SGI223044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49504[] = {SGI223045, SGI223046, SGI223047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223048, SGI0, SGI0, SGI0, SGI223049, SGI223049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49505[] = {SGI223050, SGI0, SGI223051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223052, SGI223053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49506[] = {SGI223054, SGI223055, SGI223056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223057, SGI0, SGI0, SGI0, SGI223058, SGI223058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49507[] = {SGI223059, SGI223060, SGI223061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223062, SGI0, SGI0, SGI0, SGI223064, SGI223064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223066, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49508[] = {SGI223063, SGI0, SGI223065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49509[] = {SGI223067, SGI223068, SGI223069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223070, SGI0, SGI0, SGI0, SGI223071, SGI223071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49510[] = {SGI223072, SGI0, SGI223073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223074, SGI223075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49511[] = {SGI223076, SGI0, SGI223077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223078, SGI223079, SGI223079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49512[] = {SGI223080, SGI0, SGI223081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223084, SGI223085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49513[] = {SGI223082, SGI0, SGI223083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223086, SGI223087, SGI223087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49514[] = {SGI223088, SGI0, SGI223089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223090, SGI223091, SGI223091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49515[] = {SGI223092, SGI223093, SGI223094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223095, SGI0, SGI0, SGI0, SGI223096, SGI223096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49516[] = {SGI223098, SGI223099, SGI223100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223101, SGI0, SGI0, SGI0, SGI223102, SGI223102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49517[] = {SGI223104, SGI223105, SGI223106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223107, SGI0, SGI0, SGI0, SGI223108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49518[] = {SGI223109, SGI0, SGI223110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223111, SGI223112, SGI223112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49519[] = {SGI223113, SGI223114, SGI223115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223116, SGI0, SGI0, SGI0, SGI223117, SGI223117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223118, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49520[] = {SGI223119, SGI0, SGI223120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223121, SGI0, SGI0, SGI0, SGI223122, SGI223122, SGI0, SGI0, SGI0, SGI223123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49521[] = {SGI223124, SGI0, SGI223125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223126, SGI223127, SGI223127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49522[] = {SGI223128, SGI0, SGI223129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223130, SGI0, SGI0, SGI0, SGI223131, SGI223131, SGI0, SGI0, SGI0, SGI223132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49523[] = {SGI223133, SGI223134, SGI223135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223136, SGI0, SGI0, SGI0, SGI223137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49524[] = {SGI223138, SGI0, SGI223139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223140, SGI223141, SGI223141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49525[] = {SGI223142, SGI0, SGI223143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223144, SGI0, SGI0, SGI0, SGI223146, SGI223146, SGI0, SGI0, SGI0, SGI223148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49526[] = {SGI223145, SGI0, SGI223147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223149, SGI223150, SGI223150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49527[] = {SGI223151, SGI223152, SGI223153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223154, SGI223154, SGI0, SGI0, SGI0, SGI223155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49528[] = {SGI223156, SGI0, SGI223157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223158, SGI223159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49529[] = {SGI223160, SGI0, SGI223161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223162, SGI223164, SGI223164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49530[] = {SGI223163, SGI223165, SGI223166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223167, SGI0, SGI0, SGI0, SGI223168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49531[] = {SGI223169, SGI0, SGI223170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223171, SGI223172, SGI223172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49532[] = {SGI223173, SGI0, SGI223174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223175, SGI223176, SGI223176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49533[] = {SGI223177, SGI0, SGI223178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223179, SGI223180, SGI223180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49534[] = {SGI223181, SGI0, SGI223183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223188, SGI223190, SGI223190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49535[] = {SGI223182, SGI223184, SGI223185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223187, SGI0, SGI0, SGI0, SGI223191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49536[] = {SGI223186, SGI0, SGI223189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223192, SGI223193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49537[] = {SGI223195, SGI0, SGI223198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223201, SGI223202, SGI223202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49538[] = {SGI223194, SGI0, SGI223196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223200, SGI0, SGI0, SGI0, SGI223203, SGI223203, SGI0, SGI0, SGI223206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49539[] = {SGI223197, SGI0, SGI223199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223204, SGI223205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49540[] = {SGI223207, SGI223208, SGI223209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223210, SGI0, SGI0, SGI0, SGI223211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49541[] = {SGI223212, SGI0, SGI223213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223217, SGI223218, SGI223218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223220, SGI0, SGI0, NULL};
const Segment * const SG49542[] = {SGI223214, SGI223215, SGI223216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223219, SGI0, SGI0, SGI0, SGI223221, SGI223221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223222, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49543[] = {SGI223223, SGI0, SGI223224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223228, SGI223229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223232, SGI0, SGI0, NULL};
const Segment * const SG49544[] = {SGI223225, SGI223226, SGI223227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223230, SGI0, SGI0, SGI0, SGI223231, SGI223231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49545[] = {SGI223233, SGI0, SGI223234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223235, SGI223236, SGI223236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49546[] = {SGI223237, SGI223238, SGI223239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223240, SGI0, SGI0, SGI0, SGI223241, SGI223241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49547[] = {SGI223243, SGI0, SGI223244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223248, SGI223249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49548[] = {SGI223245, SGI223246, SGI223247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223250, SGI0, SGI0, SGI0, SGI223252, SGI223252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49549[] = {SGI223251, SGI0, SGI223253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223254, SGI223255, SGI223255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223256, SGI0, SGI0, NULL};
const Segment * const SG49550[] = {SGI223257, SGI0, SGI223258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223259, SGI223260, SGI223260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49551[] = {SGI223261, SGI0, SGI223262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223265, SGI223266, SGI223266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49552[] = {SGI223263, SGI0, SGI223264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223267, SGI223268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49553[] = {SGI223269, SGI0, SGI223270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223271, SGI223272, SGI223272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49554[] = {SGI223273, SGI0, SGI223274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223275, SGI223276, SGI223276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223277, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49555[] = {SGI223278, SGI223279, SGI223280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223281, SGI223281, SGI0, SGI0, SGI223282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49556[] = {SGI223283, SGI223284, SGI223285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223286, SGI223286, SGI0, SGI0, SGI0, SGI0, SGI223287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49557[] = {SGI223288, SGI223289, SGI223290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223291, SGI0, SGI0, SGI0, SGI223292, SGI223292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49558[] = {SGI223293, SGI0, SGI223294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223295, SGI223296, SGI223296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49559[] = {SGI223297, SGI0, SGI223298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223299, SGI223300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49560[] = {SGI223301, SGI0, SGI223302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223303, SGI223304, SGI223304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49561[] = {SGI223305, SGI0, SGI223306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223307, SGI223308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49562[] = {SGI223309, SGI223310, SGI223311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223316, SGI223316, SGI0, SGI0, SGI223318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49563[] = {SGI223312, SGI223313, SGI223314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223315, SGI0, SGI0, SGI0, SGI223317, SGI223317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49564[] = {SGI223319, SGI0, SGI223320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223321, SGI223322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49565[] = {SGI223323, SGI0, SGI223324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223325, SGI223326, SGI223326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49566[] = {SGI223327, SGI0, SGI223328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223329, SGI223330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49567[] = {SGI223331, SGI223332, SGI223333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223334, SGI223334, SGI0, SGI0, SGI0, SGI223335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49568[] = {SGI223336, SGI223337, SGI223338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223339, SGI223339, SGI0, SGI0, SGI0, SGI223340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49569[] = {SGI223341, SGI223342, SGI223343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223344, SGI0, SGI0, SGI0, SGI223345, SGI223345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223346, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49570[] = {SGI223347, SGI0, SGI223348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223349, SGI0, SGI0, SGI0, SGI223350, SGI223350, SGI0, SGI0, SGI0, SGI223351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49571[] = {SGI223352, SGI223353, SGI223354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223355, SGI223355, SGI0, SGI0, SGI0, SGI223356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49572[] = {SGI223357, SGI223358, SGI223359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223360, SGI223360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223361, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49573[] = {SGI223362, SGI223363, SGI223364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223365, SGI0, SGI0, SGI0, SGI223366, SGI223366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49574[] = {SGI223367, SGI0, SGI223368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223369, SGI223370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49575[] = {SGI223371, SGI0, SGI223373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223376, SGI223378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49576[] = {SGI223372, SGI0, SGI223374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223375, SGI223377, SGI223377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49577[] = {SGI223379, SGI223380, SGI223381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223382, SGI0, SGI0, SGI0, SGI223383, SGI223383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49578[] = {SGI223384, SGI223385, SGI223386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223389, SGI223389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223391, NULL};
const Segment * const SG49579[] = {SGI223387, SGI223388, SGI223390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223392, SGI0, SGI0, SGI0, SGI223393, SGI223393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49580[] = {SGI223394, SGI223395, SGI223396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223398, SGI0, SGI0, SGI0, SGI223401, SGI223401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49581[] = {SGI223397, SGI223399, SGI223400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223402, SGI0, SGI0, SGI0, SGI223404, SGI223404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49582[] = {SGI223403, SGI223405, SGI223406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223407, SGI0, SGI0, SGI0, SGI223408, SGI223408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49583[] = {SGI223409, SGI223410, SGI223411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223412, SGI0, SGI0, SGI0, SGI223413, SGI223413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49584[] = {SGI223414, SGI223415, SGI223416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223417, SGI0, SGI0, SGI0, SGI223418, SGI223418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49585[] = {SGI223419, SGI223420, SGI223421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223422, SGI0, SGI0, SGI0, SGI223423, SGI223423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49586[] = {SGI223424, SGI223425, SGI223426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223427, SGI0, SGI0, SGI0, SGI223428, SGI223428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223429, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49587[] = {SGI223430, SGI223431, SGI223432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223433, SGI0, SGI0, SGI0, SGI223434, SGI223434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49588[] = {SGI223435, SGI223436, SGI223437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223438, SGI0, SGI0, SGI0, SGI223439, SGI223439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49589[] = {SGI223440, SGI223441, SGI223442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223443, SGI0, SGI0, SGI0, SGI223444, SGI223444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49590[] = {SGI223445, SGI0, SGI223446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223447, SGI223448, SGI223448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223450, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49591[] = {SGI223449, SGI0, SGI223451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49592[] = {SGI223452, SGI0, SGI223453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223454, SGI223455, SGI223455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49593[] = {SGI223456, SGI0, SGI223457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223458, SGI223459, SGI223459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49594[] = {SGI223460, SGI223461, SGI223462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223463, SGI0, SGI0, SGI0, SGI223464, SGI223464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223465, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49595[] = {SGI223466, SGI223467, SGI223468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223469, SGI0, SGI0, SGI0, SGI223470, SGI223470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223471, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49596[] = {SGI223472, SGI0, SGI223473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223474, SGI223475, SGI223475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49597[] = {SGI223476, SGI0, SGI223477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223478, SGI223479, SGI223479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49598[] = {SGI223480, SGI0, SGI223481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223482, SGI223483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49599[] = {SGI223484, SGI0, SGI223485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223486, SGI223487, SGI223487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49600[] = {SGI223488, SGI0, SGI223489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223490, SGI223491, SGI223491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49601[] = {SGI223492, SGI0, SGI223493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223494, SGI223495, SGI223495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49602[] = {SGI223496, SGI0, SGI223497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223498, SGI223499, SGI223499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49603[] = {SGI223500, SGI0, SGI223502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49604[] = {SGI223501, SGI223503, SGI223504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223505, SGI0, SGI0, SGI0, SGI223506, SGI0, SGI0, SGI223507, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49605[] = {SGI223508, SGI0, SGI223509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223510, SGI223511, SGI223511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49606[] = {SGI223512, SGI0, SGI223513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223514, SGI223515, SGI223515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49607[] = {SGI223516, SGI0, SGI223517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223521, SGI223522, SGI223522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49608[] = {SGI223518, SGI223519, SGI223520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223523, SGI0, SGI0, SGI0, SGI223524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49609[] = {SGI223525, SGI0, SGI223526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223527, SGI223529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49610[] = {SGI223528, SGI0, SGI223530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223531, SGI0, SGI0, SGI0, SGI223532, SGI223532, SGI0, SGI0, SGI0, SGI223533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49611[] = {SGI223534, SGI0, SGI223535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223536, SGI223537, SGI223537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49612[] = {SGI223538, SGI0, SGI223539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223540, SGI223541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49613[] = {SGI223542, SGI0, SGI223543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223544, SGI223545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49614[] = {SGI223546, SGI0, SGI223547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223548, SGI223549, SGI223549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49615[] = {SGI223550, SGI0, SGI223551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223552, SGI223553, SGI223553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49616[] = {SGI223554, SGI0, SGI223555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223556, SGI223557, SGI223557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49617[] = {SGI223558, SGI0, SGI223559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223560, SGI223561, SGI223561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49618[] = {SGI223562, SGI0, SGI223563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223564, SGI223565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223566, SGI0, SGI0, NULL};
const Segment * const SG49619[] = {SGI223567, SGI0, SGI223568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223569, SGI223570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49620[] = {SGI223571, SGI0, SGI223572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223573, SGI223574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223575, SGI0, SGI0, NULL};
const Segment * const SG49621[] = {SGI223576, SGI0, SGI223577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223578, SGI223579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49622[] = {SGI223580, SGI0, SGI223581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223582, SGI223583, SGI223583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49623[] = {SGI223584, SGI0, SGI223585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223586, SGI223587, SGI223587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49624[] = {SGI223588, SGI0, SGI223589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223590, SGI223591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49625[] = {SGI223592, SGI0, SGI223593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223594, SGI223595, SGI223595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49626[] = {SGI223596, SGI0, SGI223597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223598, SGI223599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49627[] = {SGI223600, SGI223601, SGI223602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223603, SGI0, SGI0, SGI0, SGI223604, SGI223604, SGI0, SGI223605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49628[] = {SGI223606, SGI223607, SGI223608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223609, SGI0, SGI0, SGI0, SGI223610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49629[] = {SGI223611, SGI0, SGI223612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223613, SGI223614, SGI223614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49630[] = {SGI223615, SGI0, SGI223616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223617, SGI223619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49631[] = {SGI223618, SGI0, SGI223620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223621, SGI223622, SGI223622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49632[] = {SGI223623, SGI0, SGI223624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223625, SGI223626, SGI223626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49633[] = {SGI223627, SGI0, SGI223628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223629, SGI223630, SGI223630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49634[] = {SGI223631, SGI0, SGI223632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223633, SGI223634, SGI223634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49635[] = {SGI223635, SGI0, SGI223636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223638, SGI223639, SGI223639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49636[] = {SGI223637, SGI0, SGI223640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223641, SGI223642, SGI223642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49637[] = {SGI223643, SGI0, SGI223644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223645, SGI223646, SGI223646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49638[] = {SGI223647, SGI0, SGI223648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223649, SGI223650, SGI223650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49639[] = {SGI223651, SGI0, SGI223652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223653, SGI223654, SGI223654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49640[] = {SGI223655, SGI0, SGI223656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223659, SGI223660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49641[] = {SGI223657, SGI0, SGI223658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223661, SGI223662, SGI223662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49642[] = {SGI223663, SGI0, SGI223664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223667, SGI223669, SGI223669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49643[] = {SGI223665, SGI223666, SGI223668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223670, SGI0, SGI0, SGI0, SGI223671, SGI223671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49644[] = {SGI223672, SGI223673, SGI223674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223675, SGI0, SGI0, SGI0, SGI223676, SGI223676, SGI223677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49645[] = {SGI223678, SGI0, SGI223679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223680, SGI223681, SGI223681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223682, SGI0, SGI0, NULL};
const Segment * const SG49646[] = {SGI223683, SGI223684, SGI223685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223686, SGI0, SGI0, SGI0, SGI223687, SGI223687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49647[] = {SGI223689, SGI0, SGI223690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49648[] = {SGI223688, SGI0, SGI223691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223692, SGI223693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49649[] = {SGI223694, SGI0, SGI223695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223699, SGI223700, SGI223700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49650[] = {SGI223696, SGI223697, SGI223698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223701, SGI0, SGI0, SGI0, SGI223702, SGI223702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49651[] = {SGI223703, SGI0, SGI223705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223709, SGI223710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49652[] = {SGI223704, SGI223706, SGI223707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223708, SGI0, SGI0, SGI0, SGI223711, SGI223711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49653[] = {SGI223712, SGI0, SGI223713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223714, SGI223715, SGI223715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49654[] = {SGI223716, SGI0, SGI223717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223718, SGI223719, SGI223719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49655[] = {SGI223720, SGI223721, SGI223722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223723, SGI0, SGI0, SGI0, SGI223724, SGI223724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49656[] = {SGI223726, SGI223727, SGI223728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223729, SGI0, SGI0, SGI0, SGI223730, SGI223730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49657[] = {SGI223731, SGI223732, SGI223733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223734, SGI0, SGI0, SGI0, SGI223735, SGI223735, SGI0, SGI223736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49658[] = {SGI223737, SGI0, SGI223738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223742, SGI223743, SGI223743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49659[] = {SGI223739, SGI223740, SGI223741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223744, SGI0, SGI0, SGI0, SGI223745, SGI223745, SGI0, SGI223747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49660[] = {SGI223746, SGI223748, SGI223749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223750, SGI0, SGI0, SGI0, SGI223751, SGI223751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49661[] = {SGI223752, SGI0, SGI223753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223754, SGI223755, SGI223755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49662[] = {SGI223756, SGI223757, SGI223758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223759, SGI0, SGI0, SGI0, SGI223760, SGI223760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49663[] = {SGI223761, SGI0, SGI223762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223763, SGI223764, SGI223764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49664[] = {SGI223765, SGI0, SGI223766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223767, SGI223768, SGI223768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49665[] = {SGI223769, SGI0, SGI223770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223771, SGI223772, SGI223772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223773, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49666[] = {SGI223774, SGI0, SGI223775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223776, SGI223777, SGI223777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49667[] = {SGI223778, SGI0, SGI223779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223780, SGI223781, SGI223781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49668[] = {SGI223782, SGI0, SGI223783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223784, SGI223785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49669[] = {SGI223786, SGI0, SGI223787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223788, SGI223789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49670[] = {SGI223790, SGI223791, SGI223792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223793, SGI0, SGI0, SGI0, SGI223794, SGI223794, SGI0, SGI223795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49671[] = {SGI223796, SGI0, SGI223797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223798, SGI223799, SGI223799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49672[] = {SGI223800, SGI223801, SGI223802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223803, SGI0, SGI0, SGI0, SGI223804, SGI223804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223805, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49673[] = {SGI223806, SGI223807, SGI223808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223809, SGI0, SGI0, SGI0, SGI223810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49674[] = {SGI223811, SGI0, SGI223812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223813, SGI223814, SGI223814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223815, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49675[] = {SGI223816, SGI223817, SGI223818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223819, SGI0, SGI0, SGI0, SGI223820, SGI223820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49676[] = {SGI223821, SGI0, SGI223822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223823, SGI223824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49677[] = {SGI223826, SGI0, SGI223829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223830, SGI223832, SGI223832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49678[] = {SGI223825, SGI223827, SGI223828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223831, SGI223831, SGI0, SGI0, SGI0, SGI223833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49679[] = {SGI223834, SGI0, SGI223835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223836, SGI223837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49680[] = {SGI223838, SGI223839, SGI223840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223841, SGI0, SGI0, SGI0, SGI223842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49681[] = {SGI223843, SGI0, SGI223844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223845, SGI223846, SGI223846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49682[] = {SGI223847, SGI0, SGI223848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223849, SGI223850, SGI223850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49683[] = {SGI223851, SGI0, SGI223852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223853, SGI223854, SGI223854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49684[] = {SGI223855, SGI223856, SGI223857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223858, SGI0, SGI0, SGI0, SGI223859, SGI223859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223860, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49685[] = {SGI223861, SGI0, SGI223862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223863, SGI223864, SGI223864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223865, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49686[] = {SGI223866, SGI0, SGI223867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223868, SGI223869, SGI223869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49687[] = {SGI223870, SGI0, SGI223871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223872, SGI223873, SGI223873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49688[] = {SGI223874, SGI223875, SGI223876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223878, SGI0, SGI0, SGI0, SGI223880, SGI223880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223881, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49689[] = {SGI223877, SGI0, SGI223879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223882, SGI223883, SGI223883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49690[] = {SGI223884, SGI223885, SGI223886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223887, SGI0, SGI0, SGI0, SGI223888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49691[] = {SGI223890, SGI223891, SGI223892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223893, SGI0, SGI0, SGI0, SGI223894, SGI223894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223895, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49692[] = {SGI223896, SGI223897, SGI223898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223899, SGI0, SGI0, SGI0, SGI223900, SGI223900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223901, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49693[] = {SGI223902, SGI223903, SGI223904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223905, SGI223905, SGI0, SGI0, SGI223906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49694[] = {SGI223907, SGI223908, SGI223909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223910, SGI0, SGI0, SGI0, SGI223911, SGI223911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49695[] = {SGI223912, SGI0, SGI223913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223914, SGI223915, SGI223915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49696[] = {SGI223916, SGI0, SGI223917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223918, SGI223919, SGI223919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49697[] = {SGI223920, SGI0, SGI223921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223922, SGI223923, SGI223923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49698[] = {SGI223924, SGI0, SGI223925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223926, SGI223927, SGI223927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49699[] = {SGI223928, SGI223929, SGI223930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223931, SGI0, SGI0, SGI0, SGI223932, SGI223932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49700[] = {SGI223934, SGI0, SGI223935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223936, SGI223937, SGI223937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49701[] = {SGI223938, SGI0, SGI223939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223940, SGI223941, SGI223941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49702[] = {SGI223942, SGI0, SGI223943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223944, SGI223945, SGI223945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49703[] = {SGI223946, SGI223947, SGI223948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223949, SGI0, SGI0, SGI0, SGI223950, SGI223950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223951, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49704[] = {SGI0, SGI223952, SGI223953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223954, SGI0, SGI0, SGI0, SGI223955, SGI223955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223956, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49705[] = {SGI223957, SGI0, SGI223958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223959, SGI223960, SGI223960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49706[] = {SGI223961, SGI0, SGI223962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223965, SGI223966, SGI223966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49707[] = {SGI223963, SGI0, SGI223964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223967, SGI223968, SGI223968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49708[] = {SGI223969, SGI0, SGI223970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223971, SGI223972, SGI223972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49709[] = {SGI223973, SGI0, SGI223974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223978, SGI223980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49710[] = {SGI223976, SGI0, SGI223981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223983, SGI223984, SGI223984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49711[] = {SGI223975, SGI223977, SGI223979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223982, SGI0, SGI0, SGI0, SGI223985, SGI223985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223986, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49712[] = {SGI223987, SGI223988, SGI223989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223990, SGI0, SGI0, SGI0, SGI223991, SGI223991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223992, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49713[] = {SGI223993, SGI0, SGI223994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI223995, SGI223996, SGI223996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49714[] = {SGI223997, SGI0, SGI223998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224001, SGI224002, SGI224002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49715[] = {SGI223999, SGI0, SGI224000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224003, SGI224004, SGI224004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49716[] = {SGI224005, SGI0, SGI224006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224009, SGI224010, SGI224010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49717[] = {SGI224007, SGI0, SGI224008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224011, SGI224012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49718[] = {SGI224013, SGI224014, SGI224015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224016, SGI0, SGI0, SGI0, SGI224017, SGI0, SGI0, SGI224018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49719[] = {SGI224019, SGI224020, SGI224021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224022, SGI0, SGI0, SGI0, SGI224023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49720[] = {SGI224024, SGI0, SGI224025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224026, SGI0, SGI0, SGI0, SGI224027, SGI224027, SGI0, SGI0, SGI0, SGI224028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49721[] = {SGI224029, SGI0, SGI224030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224031, SGI0, SGI0, SGI0, SGI224032, SGI224032, SGI0, SGI0, SGI0, SGI224033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49722[] = {SGI224034, SGI0, SGI224035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224036, SGI224037, SGI224037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49723[] = {SGI224038, SGI0, SGI224039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224040, SGI224041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49724[] = {SGI224042, SGI224043, SGI224044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224045, SGI0, SGI0, SGI0, SGI224046, SGI224046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224047, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49725[] = {SGI224048, SGI0, SGI224049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224050, SGI224051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49726[] = {SGI224052, SGI224053, SGI224054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224055, SGI0, SGI0, SGI0, SGI224056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49727[] = {SGI224057, SGI224058, SGI224059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224060, SGI0, SGI0, SGI0, SGI224061, SGI224061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224062, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49728[] = {SGI224063, SGI0, SGI224064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224067, SGI224068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49729[] = {SGI224065, SGI0, SGI224066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224069, SGI224070, SGI224070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49730[] = {SGI224071, SGI0, SGI224072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224073, SGI224074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49731[] = {SGI224075, SGI0, SGI224076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224077, SGI224078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49732[] = {SGI224079, SGI0, SGI224080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224081, SGI224082, SGI224082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49733[] = {SGI224083, SGI0, SGI224084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224085, SGI224086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49734[] = {SGI224087, SGI224088, SGI224089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224090, SGI0, SGI0, SGI0, SGI224091, SGI224091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49735[] = {SGI224092, SGI224093, SGI224094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224095, SGI0, SGI0, SGI0, SGI224096, SGI224096, SGI0, SGI224097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49736[] = {SGI224098, SGI0, SGI224099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224100, SGI224101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49737[] = {SGI224102, SGI224103, SGI224104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224105, SGI0, SGI0, SGI0, SGI224106, SGI224106, SGI0, SGI224108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49738[] = {SGI224107, SGI0, SGI224109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224110, SGI224111, SGI224111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49739[] = {SGI224112, SGI0, SGI224113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224114, SGI224115, SGI224115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49740[] = {SGI224116, SGI224117, SGI224118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224119, SGI0, SGI0, SGI0, SGI224121, SGI224121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224123, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49741[] = {SGI224120, SGI0, SGI224122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224124, SGI224125, SGI224125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49742[] = {SGI224126, SGI224127, SGI224128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224129, SGI224129, SGI0, SGI0, SGI0, SGI224130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49743[] = {SGI224131, SGI224132, SGI224133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224134, SGI224134, SGI0, SGI0, SGI0, SGI0, SGI224136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49744[] = {SGI224135, SGI0, SGI224137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224138, SGI224139, SGI224139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224140, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49745[] = {SGI224141, SGI224142, SGI224143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224144, SGI224144, SGI0, SGI0, SGI0, SGI0, SGI224145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49746[] = {SGI224146, SGI224147, SGI224148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224149, SGI224149, SGI0, SGI0, SGI0, SGI0, SGI224150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49747[] = {SGI224151, SGI0, SGI224152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224153, SGI224154, SGI224154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224155, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49748[] = {SGI224156, SGI224157, SGI224158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224159, SGI0, SGI0, SGI0, SGI224160, SGI0, SGI224161, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49749[] = {SGI224162, SGI0, SGI224163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224164, SGI224165, SGI224165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49750[] = {SGI224166, SGI224167, SGI224168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224169, SGI0, SGI0, SGI0, SGI224170, SGI0, SGI0, SGI0, SGI0, SGI224172, SGI0, SGI0, NULL};
const Segment * const SG49751[] = {SGI224171, SGI0, SGI224173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224174, SGI224175, SGI224175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49752[] = {SGI224176, SGI0, SGI224177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224178, SGI224179, SGI224179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49753[] = {SGI224180, SGI0, SGI224181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224182, SGI224183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49754[] = {SGI224184, SGI0, SGI224185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224186, SGI224187, SGI224187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49755[] = {SGI224188, SGI0, SGI224189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224190, SGI224191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49756[] = {SGI224192, SGI224193, SGI224194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224195, SGI224195, SGI0, SGI0, SGI0, SGI224196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49757[] = {SGI224197, SGI224198, SGI224199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224200, SGI0, SGI0, SGI0, SGI224201, SGI224201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49758[] = {SGI224202, SGI224203, SGI224204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224205, SGI224205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224206, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49759[] = {SGI224207, SGI0, SGI224208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224209, SGI224210, SGI224210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49760[] = {SGI224211, SGI0, SGI224212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224213, SGI224214, SGI224214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49761[] = {SGI224215, SGI0, SGI224216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224217, SGI224218, SGI224218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49762[] = {SGI0, SGI0, SGI224219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224220, SGI224221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49763[] = {SGI224222, SGI0, SGI224223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224224, SGI224225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49764[] = {SGI224226, SGI0, SGI224227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224228, SGI224229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49765[] = {SGI224230, SGI0, SGI224231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224232, SGI224233, SGI224233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49766[] = {SGI224234, SGI0, SGI224235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224236, SGI224237, SGI224237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49767[] = {SGI224238, SGI224239, SGI224240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224241, SGI224241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224242, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49768[] = {SGI224243, SGI0, SGI224244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224245, SGI224246, SGI224246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49769[] = {SGI224247, SGI0, SGI224248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224249, SGI224250, SGI224250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49770[] = {SGI224251, SGI0, SGI224252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224253, SGI224254, SGI224254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49771[] = {SGI224255, SGI224256, SGI224257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224259, SGI0, SGI0, SGI0, SGI224261, SGI224262, SGI0, SGI0, SGI0, SGI224263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49772[] = {SGI224258, SGI0, SGI224260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224264, SGI224265, SGI224265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49773[] = {SGI224266, SGI224267, SGI224268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224269, SGI0, SGI0, SGI0, SGI224270, SGI224271, SGI0, SGI0, SGI0, SGI224272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49774[] = {SGI224273, SGI224274, SGI224275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224276, SGI224276, SGI0, SGI0, SGI0, SGI224277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49775[] = {SGI224278, SGI0, SGI224279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224280, SGI224281, SGI224281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49776[] = {SGI224282, SGI0, SGI224283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224284, SGI224285, SGI224285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49777[] = {SGI224286, SGI0, SGI224287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224288, SGI224289, SGI224289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49778[] = {SGI224290, SGI224291, SGI224292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224293, SGI0, SGI0, SGI0, SGI224294, SGI0, SGI0, SGI224295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49779[] = {SGI224296, SGI0, SGI224297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224298, SGI224299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49780[] = {SGI224300, SGI0, SGI224301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224302, SGI224303, SGI224303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49781[] = {SGI224304, SGI0, SGI224305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224306, SGI0, SGI0, SGI0, SGI224307, SGI224307, SGI0, SGI0, SGI0, SGI224308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49782[] = {SGI224309, SGI0, SGI224310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224314, SGI224315, SGI224315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49783[] = {SGI224311, SGI224312, SGI224313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224316, SGI0, SGI0, SGI0, SGI224317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49784[] = {SGI224318, SGI0, SGI224319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224320, SGI224321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49785[] = {SGI224322, SGI0, SGI224323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224324, SGI224325, SGI224325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49786[] = {SGI224326, SGI0, SGI224327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224328, SGI224329, SGI224329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49787[] = {SGI224330, SGI224331, SGI224332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224333, SGI0, SGI0, SGI0, SGI224334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224335, SGI0, NULL};
const Segment * const SG49788[] = {SGI224336, SGI224337, SGI224338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224339, SGI0, SGI0, SGI0, SGI224340, SGI0, SGI0, SGI0, SGI224341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49789[] = {SGI224342, SGI0, SGI224343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224344, SGI224345, SGI224345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49790[] = {SGI224346, SGI0, SGI224347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224350, SGI224351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49791[] = {SGI224348, SGI0, SGI224349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224352, SGI224353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49792[] = {SGI224354, SGI224355, SGI224356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224357, SGI0, SGI0, SGI0, SGI224358, SGI224358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49793[] = {SGI224360, SGI224362, SGI224363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224364, SGI0, SGI0, SGI0, SGI224367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49794[] = {SGI224359, SGI0, SGI224361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224365, SGI224366, SGI224366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49795[] = {SGI224368, SGI0, SGI224369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49796[] = {SGI224370, SGI0, SGI224371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224372, SGI224373, SGI224373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49797[] = {SGI224374, SGI0, SGI224375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224376, SGI224378, SGI224378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49798[] = {SGI224377, SGI0, SGI224379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224380, SGI224381, SGI224381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49799[] = {SGI224382, SGI0, SGI224383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224384, SGI224385, SGI224385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49800[] = {SGI224386, SGI224387, SGI224388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224389, SGI0, SGI0, SGI0, SGI224390, SGI224390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224391, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49801[] = {SGI224392, SGI0, SGI224393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224394, SGI224395, SGI224395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49802[] = {SGI224396, SGI224397, SGI224398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224399, SGI0, SGI0, SGI0, SGI224400, SGI224400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224401, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49803[] = {SGI224402, SGI0, SGI224403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224404, SGI0, SGI0, SGI0, SGI224406, SGI224406, SGI0, SGI0, SGI0, SGI224408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49804[] = {SGI224405, SGI0, SGI224407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224409, SGI224410, SGI224410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224411, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49805[] = {SGI224412, SGI0, SGI224413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224414, SGI224415, SGI224415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49806[] = {SGI224416, SGI224417, SGI224418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224419, SGI0, SGI0, SGI0, SGI224420, SGI0, SGI0, SGI0, SGI224421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49807[] = {SGI224422, SGI224423, SGI224424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224425, SGI0, SGI0, SGI0, SGI224426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224427, SGI0, NULL};
const Segment * const SG49808[] = {SGI224428, SGI0, SGI224429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224432, SGI0, SGI0, SGI0, SGI224433, SGI224433, SGI0, SGI0, SGI0, SGI224436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49809[] = {SGI224430, SGI0, SGI224431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224434, SGI224435, SGI224435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49810[] = {SGI224437, SGI224438, SGI224439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224440, SGI0, SGI0, SGI0, SGI224441, SGI224441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49811[] = {SGI224442, SGI0, SGI224443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224444, SGI224445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49812[] = {SGI224446, SGI224447, SGI224448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224449, SGI0, SGI0, SGI0, SGI224450, SGI0, SGI224451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49813[] = {SGI224452, SGI0, SGI224453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224454, SGI224455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49814[] = {SGI224456, SGI0, SGI224457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224458, SGI224459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49815[] = {SGI224460, SGI0, SGI224461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224462, SGI0, SGI0, SGI0, SGI224463, SGI224463, SGI0, SGI0, SGI0, SGI224464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49816[] = {SGI224465, SGI0, SGI224466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224467, SGI224468, SGI224468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49817[] = {SGI224469, SGI0, SGI224470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224471, SGI224472, SGI224472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49818[] = {SGI224473, SGI0, SGI224474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224475, SGI224476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49819[] = {SGI224477, SGI0, SGI224478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224480, SGI224481, SGI224481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49820[] = {SGI224479, SGI0, SGI224482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224483, SGI224484, SGI224484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49821[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224485, SGI0, SGI0, SGI224486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49822[] = {SGI224487, SGI224488, SGI224489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224490, SGI0, SGI0, SGI0, SGI224491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49823[] = {SGI224493, SGI224495, SGI224496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224498, SGI0, SGI0, SGI0, SGI224500, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49824[] = {SGI224492, SGI0, SGI224494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224497, SGI224499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49825[] = {SGI224501, SGI224502, SGI224503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224504, SGI0, SGI0, SGI0, SGI224505, SGI224505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224506, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49826[] = {SGI224507, SGI224508, SGI224509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224510, SGI0, SGI0, SGI0, SGI224511, SGI224511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224512, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49827[] = {SGI224513, SGI0, SGI224514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224515, SGI224516, SGI224516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49828[] = {SGI224517, SGI0, SGI224518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224519, SGI224520, SGI224520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49829[] = {SGI224521, SGI0, SGI224522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224524, SGI224526, SGI224526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49830[] = {SGI224523, SGI0, SGI224525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224527, SGI224528, SGI224528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49831[] = {SGI224529, SGI224530, SGI224531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224532, SGI0, SGI0, SGI0, SGI224533, SGI224533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224534, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49832[] = {SGI224535, SGI224536, SGI224537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224538, SGI0, SGI0, SGI0, SGI224539, SGI224539, SGI0, SGI0, SGI0, SGI0, SGI224540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49833[] = {SGI224541, SGI0, SGI224542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224546, SGI224547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49834[] = {SGI224543, SGI224544, SGI224545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224548, SGI0, SGI0, SGI0, SGI224549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49835[] = {SGI224550, SGI0, SGI224551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224555, SGI224556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49836[] = {SGI224552, SGI224553, SGI224554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224557, SGI0, SGI0, SGI0, SGI224558, SGI224558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224560, SGI0, NULL};
const Segment * const SG49837[] = {SGI224561, SGI224562, SGI224563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224564, SGI0, SGI0, SGI0, SGI224565, SGI224565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49838[] = {SGI224566, SGI0, SGI224567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224568, SGI224569, SGI224569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224570, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49839[] = {SGI224571, SGI0, SGI224572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224573, SGI224574, SGI224574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49840[] = {SGI224575, SGI224576, SGI224577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224578, SGI0, SGI0, SGI0, SGI224579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49841[] = {SGI224580, SGI0, SGI224581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224582, SGI224583, SGI224583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49842[] = {SGI224584, SGI0, SGI224585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224586, SGI224587, SGI224587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224588, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49843[] = {SGI224589, SGI224590, SGI224591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224592, SGI0, SGI0, SGI0, SGI224594, SGI224594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224596, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49844[] = {SGI224593, SGI0, SGI224595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224597, SGI224598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49845[] = {SGI224599, SGI224600, SGI224601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224602, SGI0, SGI0, SGI0, SGI224603, SGI224603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224604, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49846[] = {SGI224605, SGI224606, SGI224607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224608, SGI0, SGI0, SGI0, SGI224609, SGI224609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49847[] = {SGI224610, SGI0, SGI224611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224612, SGI224613, SGI224613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49848[] = {SGI224614, SGI224615, SGI224616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224617, SGI0, SGI0, SGI0, SGI224618, SGI224618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49849[] = {SGI224620, SGI224621, SGI224622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224623, SGI0, SGI0, SGI0, SGI224624, SGI224624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49850[] = {SGI224626, SGI0, SGI224627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224628, SGI224629, SGI224629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49851[] = {SGI224630, SGI224631, SGI224632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224633, SGI0, SGI0, SGI0, SGI224634, SGI224634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49852[] = {SGI224636, SGI0, SGI224638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224640, SGI224641, SGI224641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49853[] = {SGI224637, SGI0, SGI224639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224642, SGI224643, SGI224643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49854[] = {SGI224644, SGI224645, SGI224646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224647, SGI224647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224648, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49855[] = {SGI224649, SGI224650, SGI224651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224652, SGI224652, SGI0, SGI0, SGI0, SGI224653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49856[] = {SGI224654, SGI224655, SGI224656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224657, SGI224657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224658, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49857[] = {SGI224659, SGI224660, SGI224661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224662, SGI224662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224663, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49858[] = {SGI224664, SGI0, SGI224665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224666, SGI224667, SGI224667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49859[] = {SGI224668, SGI0, SGI224669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224670, SGI224671, SGI224671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49860[] = {SGI224674, SGI224675, SGI224676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224677, SGI0, SGI0, SGI0, SGI224678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49861[] = {SGI224679, SGI0, SGI224680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224681, SGI224682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49862[] = {SGI224683, SGI0, SGI224684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224685, SGI224686, SGI224686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49863[] = {SGI224687, SGI0, SGI224688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224689, SGI224690, SGI224690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49864[] = {SGI224691, SGI224692, SGI224693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224694, SGI224694, SGI0, SGI0, SGI224695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49865[] = {SGI224696, SGI224697, SGI224698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224699, SGI0, SGI0, SGI0, SGI224700, SGI224700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49866[] = {SGI224701, SGI0, SGI224702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224703, SGI224704, SGI224704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224705, SGI0, SGI0, NULL};
const Segment * const SG49867[] = {SGI224706, SGI0, SGI224707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224708, SGI224709, SGI224709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49868[] = {SGI224710, SGI224711, SGI224712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224713, SGI0, SGI0, SGI0, SGI224714, SGI224714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224715, NULL};
const Segment * const SG49869[] = {SGI224716, SGI224717, SGI224718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224722, SGI224722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224723, NULL};
const Segment * const SG49870[] = {SGI224719, SGI224720, SGI224721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224724, SGI224724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224725, NULL};
const Segment * const SG49871[] = {SGI224726, SGI0, SGI224727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224728, SGI224729, SGI224729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49872[] = {SGI224730, SGI0, SGI224731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224732, SGI224733, SGI224733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49873[] = {SGI224734, SGI224735, SGI224736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224737, SGI0, SGI0, SGI0, SGI224738, SGI224738, SGI0, SGI224739, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49874[] = {SGI224740, SGI224741, SGI224742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224743, SGI0, SGI0, SGI0, SGI224744, SGI224744, SGI0, SGI224746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49875[] = {SGI224745, SGI224747, SGI224748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224749, SGI0, SGI0, SGI0, SGI224750, SGI224750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49876[] = {SGI224751, SGI224752, SGI224753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224754, SGI0, SGI0, SGI0, SGI224755, SGI224755, SGI0, SGI224756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49877[] = {SGI224757, SGI224758, SGI224759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224760, SGI0, SGI0, SGI0, SGI224761, SGI224761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224762, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49878[] = {SGI224763, SGI0, SGI224764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224765, SGI224766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49879[] = {SGI224767, SGI0, SGI224768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224769, SGI224770, SGI224770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49880[] = {SGI224771, SGI0, SGI224773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224775, SGI224777, SGI224777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49881[] = {SGI224772, SGI0, SGI224774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224776, SGI224778, SGI224778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49882[] = {SGI224779, SGI0, SGI224780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224781, SGI224782, SGI224782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49883[] = {SGI224783, SGI0, SGI224784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224785, SGI224786, SGI224786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49884[] = {SGI224787, SGI0, SGI224788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224789, SGI224790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49885[] = {SGI224791, SGI224792, SGI224793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224794, SGI224794, SGI0, SGI0, SGI0, SGI0, SGI224795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49886[] = {SGI224796, SGI224797, SGI224798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224799, SGI224799, SGI0, SGI0, SGI0, SGI0, SGI224800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49887[] = {SGI224801, SGI224802, SGI224803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224806, SGI224806, SGI0, SGI0, SGI0, SGI0, SGI224808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49888[] = {SGI224804, SGI0, SGI224805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224807, SGI224809, SGI224809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49889[] = {SGI224672, SGI0, SGI224673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224810, SGI224811, SGI224811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49890[] = {SGI224812, SGI0, SGI224813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224814, SGI224815, SGI224815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49891[] = {SGI224816, SGI224817, SGI224818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224819, SGI0, SGI0, SGI0, SGI224820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49892[] = {SGI224821, SGI0, SGI224822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224823, SGI224824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49893[] = {SGI224825, SGI224826, SGI224827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224828, SGI224828, SGI0, SGI0, SGI224829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49894[] = {SGI224830, SGI0, SGI224831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224832, SGI224833, SGI224833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49895[] = {SGI224834, SGI224835, SGI224836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224837, SGI224837, SGI0, SGI0, SGI224838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49896[] = {SGI224839, SGI0, SGI224840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224842, SGI224843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224845, SGI0, SGI0, NULL};
const Segment * const SG49897[] = {SGI224841, SGI0, SGI224844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224846, SGI224847, SGI224847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49898[] = {SGI224848, SGI0, SGI224849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224850, SGI224851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49899[] = {SGI224852, SGI0, SGI224853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224854, SGI224855, SGI224855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49900[] = {SGI224856, SGI224857, SGI224858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224859, SGI0, SGI0, SGI0, SGI224860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49901[] = {SGI224861, SGI224862, SGI224863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224864, SGI0, SGI0, SGI0, SGI224865, SGI0, SGI0, SGI0, SGI224866, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49902[] = {SGI224867, SGI224868, SGI224869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224870, SGI224870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224871, NULL};
const Segment * const SG49903[] = {SGI224872, SGI0, SGI224873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224874, SGI224875, SGI224875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49904[] = {SGI224876, SGI0, SGI224877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224878, SGI224879, SGI224879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49905[] = {SGI224880, SGI224881, SGI224882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224883, SGI0, SGI0, SGI0, SGI224884, SGI224884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49906[] = {SGI224885, SGI0, SGI224886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224887, SGI224888, SGI224888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49907[] = {SGI224889, SGI0, SGI224890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224891, SGI224892, SGI224892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49908[] = {SGI224893, SGI0, SGI224894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224895, SGI224896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49909[] = {SGI224897, SGI0, SGI224898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224899, SGI224900, SGI224900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49910[] = {SGI224901, SGI0, SGI224902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224903, SGI224904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49911[] = {SGI224905, SGI0, SGI224906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224907, SGI224908, SGI224908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49912[] = {SGI224909, SGI0, SGI224910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224911, SGI224912, SGI224912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49913[] = {SGI224913, SGI224914, SGI224915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224916, SGI0, SGI0, SGI0, SGI224917, SGI224917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224918, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49914[] = {SGI224919, SGI0, SGI224920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224921, SGI224922, SGI224922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49915[] = {SGI224923, SGI224924, SGI224925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224928, SGI224928, SGI0, SGI0, SGI0, SGI224929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49916[] = {SGI224926, SGI0, SGI224927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224930, SGI224931, SGI224931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49917[] = {SGI224932, SGI0, SGI224933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224934, SGI224935, SGI224935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49918[] = {SGI224936, SGI0, SGI224937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224938, SGI224939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49919[] = {SGI224940, SGI224941, SGI224942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224943, SGI0, SGI0, SGI0, SGI224944, SGI224944, SGI0, SGI0, SGI224945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49920[] = {SGI224946, SGI0, SGI224947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224948, SGI224949, SGI224949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49921[] = {SGI224950, SGI224951, SGI224952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224953, SGI0, SGI0, SGI0, SGI224954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49922[] = {SGI224955, SGI0, SGI224956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224957, SGI224958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49923[] = {SGI224959, SGI224960, SGI224961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224962, SGI0, SGI0, SGI0, SGI224963, SGI224963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49924[] = {SGI224964, SGI224965, SGI224966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224967, SGI224967, SGI0, SGI0, SGI224968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49925[] = {SGI224969, SGI224970, SGI224971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224972, SGI0, SGI0, SGI0, SGI224973, SGI224973, SGI0, SGI224974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49926[] = {SGI224975, SGI224976, SGI224977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224978, SGI0, SGI0, SGI0, SGI224980, SGI224980, SGI0, SGI224983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49927[] = {SGI224979, SGI224981, SGI224982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224984, SGI0, SGI0, SGI0, SGI224985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49928[] = {SGI224986, SGI0, SGI224987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224988, SGI224989, SGI224989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49929[] = {SGI224990, SGI224991, SGI224992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224993, SGI0, SGI0, SGI0, SGI224994, SGI224994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49930[] = {SGI224995, SGI224996, SGI224997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI224998, SGI0, SGI0, SGI0, SGI224999, SGI224999, SGI0, SGI225000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49931[] = {SGI225001, SGI225002, SGI225003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225006, SGI0, SGI0, SGI0, SGI225007, SGI225007, SGI0, SGI225008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49932[] = {SGI225004, SGI0, SGI225005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225009, SGI225010, SGI225010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49933[] = {SGI225011, SGI225012, SGI225013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225014, SGI0, SGI0, SGI0, SGI225015, SGI225015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225017, SGI0, NULL};
const Segment * const SG49934[] = {SGI225018, SGI225019, SGI225020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225021, SGI0, SGI0, SGI0, SGI225022, SGI225022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225024, SGI0, NULL};
const Segment * const SG49935[] = {SGI225025, SGI225026, SGI225027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225028, SGI0, SGI0, SGI0, SGI225029, SGI225029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225030, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49936[] = {SGI225031, SGI225032, SGI225033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225034, SGI0, SGI0, SGI0, SGI225035, SGI225035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225036, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49937[] = {SGI225037, SGI0, SGI225038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225039, SGI225040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49938[] = {SGI225041, SGI0, SGI225042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225043, SGI225044, SGI225044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49939[] = {SGI225046, SGI225048, SGI225049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225051, SGI0, SGI0, SGI0, SGI225053, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49940[] = {SGI225045, SGI0, SGI225047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225050, SGI225052, SGI225052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49941[] = {SGI225054, SGI0, SGI225055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225056, SGI225057, SGI225057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49942[] = {SGI225058, SGI225059, SGI225060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225061, SGI225061, SGI0, SGI0, SGI0, SGI225062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49943[] = {SGI225063, SGI0, SGI225064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225067, SGI225068, SGI225068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49944[] = {SGI225065, SGI0, SGI225066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225069, SGI225070, SGI225070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49945[] = {SGI225071, SGI0, SGI225072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225073, SGI225074, SGI225074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49946[] = {SGI225075, SGI0, SGI225076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225077, SGI225078, SGI225078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49947[] = {SGI225079, SGI0, SGI225081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225083, SGI225084, SGI225084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49948[] = {SGI225080, SGI0, SGI225082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225085, SGI225086, SGI225086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49949[] = {SGI225087, SGI0, SGI225088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225089, SGI225090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49950[] = {SGI225091, SGI0, SGI225092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225093, SGI225094, SGI225094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49951[] = {SGI225095, SGI0, SGI225096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225097, SGI225098, SGI225098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49952[] = {SGI225099, SGI0, SGI225100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225101, SGI225102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49953[] = {SGI225103, SGI0, SGI225104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225105, SGI225106, SGI225106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49954[] = {SGI225107, SGI0, SGI225108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225109, SGI225110, SGI225110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49955[] = {SGI225111, SGI0, SGI225112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225113, SGI225114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49956[] = {SGI225115, SGI225116, SGI225117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225118, SGI0, SGI0, SGI0, SGI225119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49957[] = {SGI225120, SGI0, SGI225121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225122, SGI225123, SGI225123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49958[] = {SGI225124, SGI0, SGI225125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225126, SGI225127, SGI225127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49959[] = {SGI225128, SGI0, SGI225129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225130, SGI225131, SGI225131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225133, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49960[] = {SGI225132, SGI225134, SGI225135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225136, SGI0, SGI0, SGI0, SGI225137, SGI225137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225138, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49961[] = {SGI225139, SGI0, SGI225140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225141, SGI225142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49962[] = {SGI225143, SGI0, SGI225144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225145, SGI225146, SGI225146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49963[] = {SGI225147, SGI225148, SGI225149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225150, SGI0, SGI0, SGI0, SGI225151, SGI225151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49964[] = {SGI225152, SGI0, SGI225153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225154, SGI225155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49965[] = {SGI0, SGI225156, SGI225157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225158, SGI0, SGI0, SGI0, SGI225159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49966[] = {SGI225160, SGI0, SGI225161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225162, SGI225163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49967[] = {SGI225164, SGI0, SGI225165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225166, SGI225167, SGI225167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49968[] = {SGI225168, SGI225169, SGI225170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225171, SGI0, SGI0, SGI0, SGI225172, SGI225172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225173, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49969[] = {SGI225174, SGI0, SGI225175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225176, SGI225177, SGI225177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49970[] = {SGI225178, SGI0, SGI225179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225180, SGI225181, SGI225181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49971[] = {SGI225182, SGI0, SGI225183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225184, SGI225185, SGI225185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49972[] = {SGI225186, SGI225187, SGI225188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225189, SGI0, SGI0, SGI0, SGI225191, SGI225191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225194, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49973[] = {SGI225190, SGI225192, SGI225193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225195, SGI225195, SGI0, SGI0, SGI225196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49974[] = {SGI225197, SGI0, SGI225198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225199, SGI225200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49975[] = {SGI225201, SGI0, SGI225202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225203, SGI225204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49976[] = {SGI225205, SGI0, SGI225206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225207, SGI225208, SGI225208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49977[] = {SGI225209, SGI225210, SGI225211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225214, SGI0, SGI0, SGI0, SGI225217, SGI225217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49978[] = {SGI225212, SGI225213, SGI225215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225219, SGI0, SGI0, SGI0, SGI225220, SGI225220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49979[] = {SGI225216, SGI0, SGI225218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225221, SGI225222, SGI225222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49980[] = {SGI225223, SGI0, SGI225224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225225, SGI225226, SGI225226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49981[] = {SGI225227, SGI225228, SGI225229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225230, SGI0, SGI0, SGI0, SGI225231, SGI225231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49982[] = {SGI225232, SGI225233, SGI225234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225235, SGI0, SGI0, SGI0, SGI225236, SGI225236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49983[] = {SGI225237, SGI0, SGI225239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225240, SGI225241, SGI225241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225242, SGI0, SGI0, NULL};
const Segment * const SG49984[] = {SGI225243, SGI0, SGI225244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225248, SGI225250, SGI225250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49985[] = {SGI225245, SGI225246, SGI225247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225249, SGI0, SGI0, SGI0, SGI225251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49986[] = {SGI225252, SGI0, SGI225253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225254, SGI225255, SGI225255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49987[] = {SGI225256, SGI225257, SGI225258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225259, SGI0, SGI0, SGI0, SGI225260, SGI225260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49988[] = {SGI225262, SGI225263, SGI225264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225265, SGI0, SGI0, SGI0, SGI225266, SGI225266, SGI0, SGI225268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49989[] = {SGI225267, SGI225269, SGI225270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225271, SGI0, SGI0, SGI0, SGI225272, SGI225272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49990[] = {SGI225273, SGI225274, SGI225275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225276, SGI0, SGI0, SGI0, SGI225277, SGI225277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49991[] = {SGI225278, SGI0, SGI225279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225280, SGI225281, SGI225281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49992[] = {SGI225282, SGI225283, SGI225284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225285, SGI0, SGI0, SGI0, SGI225286, SGI225286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49993[] = {SGI225287, SGI0, SGI225288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225289, SGI225290, SGI225290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49994[] = {SGI225291, SGI225292, SGI225293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225296, SGI0, SGI0, SGI0, SGI225297, SGI225297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49995[] = {SGI225294, SGI0, SGI225295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225298, SGI225299, SGI225299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49996[] = {SGI225300, SGI0, SGI225301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225302, SGI225303, SGI225303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225304, SGI0, SGI0, NULL};
const Segment * const SG49997[] = {SGI225305, SGI0, SGI225306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225307, SGI225308, SGI225308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49998[] = {SGI225309, SGI225310, SGI225311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225312, SGI225312, SGI0, SGI0, SGI0, SGI0, SGI225313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG49999[] = {SGI225314, SGI0, SGI225315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225319, SGI225320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50000[] = {SGI225316, SGI225317, SGI225318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225321, SGI225321, SGI0, SGI0, SGI0, SGI0, SGI225322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50001[] = {SGI225323, SGI0, SGI225324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225325, SGI225326, SGI225326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50002[] = {SGI225327, SGI0, SGI225328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225329, SGI0, SGI0, SGI0, SGI225330, SGI225330, SGI0, SGI0, SGI0, SGI0, SGI225331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50003[] = {SGI225332, SGI0, SGI225333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225334, SGI225335, SGI225335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50004[] = {SGI225336, SGI225337, SGI225338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225339, SGI0, SGI0, SGI0, SGI225340, SGI225340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225341, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50005[] = {SGI225342, SGI0, SGI225343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225344, SGI0, SGI0, SGI0, SGI225345, SGI225345, SGI0, SGI0, SGI0, SGI225346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50006[] = {SGI225347, SGI0, SGI225348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225349, SGI0, SGI0, SGI0, SGI225350, SGI225350, SGI0, SGI0, SGI0, SGI225351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50007[] = {SGI225352, SGI225353, SGI225354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225355, SGI0, SGI0, SGI0, SGI225356, SGI225356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50008[] = {SGI225357, SGI225358, SGI225359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225360, SGI0, SGI0, SGI0, SGI225361, SGI225361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50009[] = {SGI225362, SGI225363, SGI225364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225365, SGI0, SGI0, SGI0, SGI225366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225367, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50010[] = {SGI225368, SGI225369, SGI225370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225371, SGI0, SGI0, SGI0, SGI225372, SGI225372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225373, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50011[] = {SGI225375, SGI0, SGI225377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225378, SGI225380, SGI225380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50012[] = {SGI225374, SGI0, SGI225376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225379, SGI225381, SGI225381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50013[] = {SGI225382, SGI225383, SGI225384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225385, SGI0, SGI0, SGI0, SGI225386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225387, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50014[] = {SGI225388, SGI225389, SGI225390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225391, SGI0, SGI0, SGI0, SGI225392, SGI225392, SGI0, SGI0, SGI0, SGI0, SGI225393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50015[] = {SGI225394, SGI225395, SGI225396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225397, SGI0, SGI0, SGI0, SGI225398, SGI225398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225399, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50016[] = {SGI225400, SGI0, SGI225401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225402, SGI225403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50017[] = {SGI225404, SGI0, SGI225405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225406, SGI225407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50018[] = {SGI225408, SGI225409, SGI225410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225411, SGI0, SGI0, SGI0, SGI225412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50019[] = {SGI225413, SGI0, SGI225414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225415, SGI225416, SGI225416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50020[] = {SGI225417, SGI0, SGI225418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225419, SGI225420, SGI225420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50021[] = {SGI225421, SGI0, SGI225422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225423, SGI225424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50022[] = {SGI225425, SGI0, SGI225427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225430, SGI225431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50023[] = {SGI225426, SGI225428, SGI225429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225432, SGI0, SGI0, SGI0, SGI225433, SGI225433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225434, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50024[] = {SGI225435, SGI0, SGI225436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225437, SGI0, SGI0, SGI0, SGI225438, SGI225438, SGI0, SGI0, SGI0, SGI225439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50025[] = {SGI225441, SGI0, SGI225443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225446, SGI225447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50026[] = {SGI225440, SGI225442, SGI225444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225445, SGI0, SGI0, SGI0, SGI225448, SGI225448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225449, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50027[] = {SGI225450, SGI225451, SGI225452, SGI225453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225454, SGI0, SGI0, SGI0, SGI225455, SGI225456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50028[] = {SGI225457, SGI225458, SGI225459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225460, SGI0, SGI0, SGI0, SGI225461, SGI225461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50029[] = {SGI225463, SGI225464, SGI225465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225466, SGI225466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50030[] = {SGI0, SGI225467, SGI225468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225469, SGI0, SGI0, SGI0, SGI225470, SGI225470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50031[] = {SGI225471, SGI0, SGI225472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225476, SGI225477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50032[] = {SGI225473, SGI225474, SGI225475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225478, SGI0, SGI0, SGI0, SGI225479, SGI225479, SGI0, SGI225480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50033[] = {SGI225481, SGI225482, SGI225483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225484, SGI0, SGI0, SGI0, SGI225485, SGI225485, SGI0, SGI225486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50034[] = {SGI225487, SGI225488, SGI225489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225490, SGI0, SGI0, SGI0, SGI225491, SGI225491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50035[] = {SGI225492, SGI225493, SGI225494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225495, SGI0, SGI0, SGI0, SGI225496, SGI225496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225497, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50036[] = {SGI225498, SGI0, SGI225499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225503, SGI225504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225506, SGI0, SGI0, NULL};
const Segment * const SG50037[] = {SGI225500, SGI225501, SGI225502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225505, SGI0, SGI0, SGI0, SGI225507, SGI225507, SGI0, SGI0, SGI0, SGI225508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50038[] = {SGI225509, SGI0, SGI225510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225512, SGI225513, SGI225513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50039[] = {SGI225511, SGI0, SGI225514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225515, SGI225516, SGI225516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50040[] = {SGI225517, SGI225518, SGI225519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225520, SGI0, SGI0, SGI0, SGI225521, SGI225521, SGI0, SGI225523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50041[] = {SGI225522, SGI0, SGI225524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225525, SGI225526, SGI225526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50042[] = {SGI225527, SGI0, SGI225528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225529, SGI225530, SGI225530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50043[] = {SGI225531, SGI225532, SGI225533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225534, SGI0, SGI0, SGI0, SGI225535, SGI225535, SGI0, SGI0, SGI225536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50044[] = {SGI225537, SGI0, SGI225538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225539, SGI225540, SGI225540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50045[] = {SGI225541, SGI0, SGI225542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225544, SGI225545, SGI225545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50046[] = {SGI225543, SGI0, SGI225546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225547, SGI225548, SGI225548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50047[] = {SGI225549, SGI225550, SGI225551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225552, SGI225552, SGI0, SGI0, SGI0, SGI225553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50048[] = {SGI225554, SGI225555, SGI225556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225559, SGI0, SGI0, SGI0, SGI225560, SGI225560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225563, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50049[] = {SGI225557, SGI0, SGI225558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225561, SGI225562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50050[] = {SGI225564, SGI0, SGI225565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225566, SGI225567, SGI225567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50051[] = {SGI225568, SGI0, SGI225569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225570, SGI225571, SGI225571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225572, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50052[] = {SGI225573, SGI225574, SGI225575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225576, SGI225576, SGI0, SGI0, SGI225577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50053[] = {SGI225578, SGI0, SGI225579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225582, SGI225583, SGI225583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50054[] = {SGI225580, SGI0, SGI225581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225584, SGI225585, SGI225585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50055[] = {SGI225586, SGI0, SGI225587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225588, SGI225589, SGI225589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50056[] = {SGI225590, SGI0, SGI225591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225592, SGI225593, SGI225593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50057[] = {SGI225594, SGI0, SGI225595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225596, SGI225597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50058[] = {SGI225598, SGI0, SGI225599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225600, SGI225601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50059[] = {SGI225602, SGI0, SGI225603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225604, SGI225605, SGI225605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50060[] = {SGI225606, SGI0, SGI225607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225608, SGI225609, SGI225609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50061[] = {SGI225610, SGI0, SGI225611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225612, SGI225613, SGI225613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50062[] = {SGI225614, SGI225615, SGI225616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225617, SGI0, SGI0, SGI0, SGI225618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50063[] = {SGI225619, SGI0, SGI225621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225624, SGI225626, SGI225626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50064[] = {SGI225620, SGI225622, SGI225623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225625, SGI0, SGI0, SGI0, SGI225627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50065[] = {SGI225628, SGI0, SGI225629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225630, SGI225631, SGI225631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50066[] = {SGI225632, SGI225633, SGI225634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225635, SGI0, SGI0, SGI0, SGI225636, SGI225636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225637, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50067[] = {SGI225638, SGI0, SGI225639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225640, SGI225641, SGI225641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225642, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50068[] = {SGI225643, SGI0, SGI225644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225645, SGI225646, SGI225646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50069[] = {SGI225647, SGI0, SGI225648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225649, SGI225650, SGI225650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50070[] = {SGI225651, SGI225652, SGI225653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225654, SGI0, SGI0, SGI0, SGI225655, SGI225657, SGI0, SGI0, SGI0, SGI225659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50071[] = {SGI225656, SGI0, SGI225658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50072[] = {SGI225660, SGI225661, SGI225662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225663, SGI0, SGI0, SGI0, SGI225664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50073[] = {SGI225665, SGI0, SGI225666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225667, SGI225668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50074[] = {SGI225669, SGI0, SGI225670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225671, SGI225672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50075[] = {SGI225673, SGI0, SGI225674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225675, SGI225676, SGI225676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50076[] = {SGI225677, SGI225678, SGI225679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225680, SGI0, SGI0, SGI0, SGI225681, SGI225681, SGI225682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50077[] = {SGI225683, SGI0, SGI225684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225685, SGI225686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50078[] = {SGI225687, SGI0, SGI225688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225689, SGI225690, SGI225690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50079[] = {SGI225691, SGI0, SGI225692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225693, SGI225694, SGI225694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50080[] = {SGI225695, SGI0, SGI225696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225697, SGI225698, SGI225698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50081[] = {SGI225699, SGI0, SGI225700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225701, SGI225702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50082[] = {SGI225703, SGI0, SGI225704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225705, SGI225706, SGI225706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225707, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50083[] = {SGI225708, SGI0, SGI225709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225710, SGI225711, SGI225711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50084[] = {SGI225712, SGI0, SGI225713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225715, SGI225717, SGI225717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50085[] = {SGI225714, SGI0, SGI225716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225718, SGI225719, SGI225719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50086[] = {SGI225720, SGI0, SGI225721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225722, SGI225723, SGI225723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50087[] = {SGI225724, SGI225725, SGI225726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225727, SGI225727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225728, NULL};
const Segment * const SG50088[] = {SGI225729, SGI225730, SGI225731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225732, SGI0, SGI0, SGI0, SGI225733, SGI225733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50089[] = {SGI225735, SGI0, SGI225738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225739, SGI225740, SGI225740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50090[] = {SGI225734, SGI225736, SGI225737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225741, SGI225741, SGI0, SGI0, SGI0, SGI225742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50091[] = {SGI225743, SGI0, SGI225744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225745, SGI225746, SGI225746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50092[] = {SGI225747, SGI0, SGI225748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225749, SGI225750, SGI225750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50093[] = {SGI225751, SGI225752, SGI225753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225754, SGI0, SGI0, SGI0, SGI225755, SGI225755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225756, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50094[] = {SGI225757, SGI225758, SGI225759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225760, SGI0, SGI0, SGI0, SGI225761, SGI225761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225762, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50095[] = {SGI225763, SGI0, SGI225764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225765, SGI225767, SGI225767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50096[] = {SGI225766, SGI0, SGI225768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50097[] = {SGI225769, SGI0, SGI225770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225771, SGI225772, SGI225772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50098[] = {SGI225773, SGI225774, SGI225775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225776, SGI0, SGI0, SGI0, SGI225777, SGI225777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50099[] = {SGI225778, SGI225779, SGI225780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225781, SGI0, SGI0, SGI0, SGI225782, SGI225782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50100[] = {SGI225784, SGI0, SGI225785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225786, SGI225787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50101[] = {SGI225788, SGI225789, SGI225790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225791, SGI225791, SGI0, SGI0, SGI0, SGI225792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50102[] = {SGI225793, SGI225794, SGI225795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225796, SGI225796, SGI0, SGI0, SGI0, SGI225797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50103[] = {SGI225798, SGI0, SGI225799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225800, SGI225801, SGI225801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50104[] = {SGI225802, SGI0, SGI225803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225804, SGI225805, SGI225805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50105[] = {SGI225806, SGI0, SGI225807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225809, SGI225810, SGI225810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50106[] = {SGI225808, SGI0, SGI225811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225812, SGI225813, SGI225813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50107[] = {SGI225814, SGI0, SGI225815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225819, SGI225820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50108[] = {SGI225816, SGI225817, SGI225818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225821, SGI0, SGI0, SGI0, SGI225822, SGI225822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225823, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50109[] = {SGI225824, SGI0, SGI225825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225826, SGI225827, SGI225827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50110[] = {SGI225828, SGI0, SGI225829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225830, SGI225831, SGI225831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50111[] = {SGI225832, SGI225833, SGI225834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225835, SGI0, SGI0, SGI0, SGI225836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50112[] = {SGI225837, SGI225839, SGI225840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225842, SGI0, SGI0, SGI0, SGI225843, SGI225843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50113[] = {SGI225838, SGI0, SGI225841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225844, SGI225845, SGI225845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50114[] = {SGI225846, SGI0, SGI225847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225848, SGI225849, SGI225849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50115[] = {SGI225850, SGI0, SGI225851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50116[] = {SGI225852, SGI0, SGI225853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225854, SGI225855, SGI225855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50117[] = {SGI225856, SGI0, SGI225857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225858, SGI225859, SGI225859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50118[] = {SGI225860, SGI225861, SGI225862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225863, SGI0, SGI0, SGI0, SGI225864, SGI225864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225865, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50119[] = {SGI225866, SGI0, SGI225868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225870, SGI225872, SGI225872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50120[] = {SGI225867, SGI0, SGI225869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225871, SGI225873, SGI225873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50121[] = {SGI225874, SGI225875, SGI225876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225877, SGI0, SGI0, SGI0, SGI225878, SGI225878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50122[] = {SGI225880, SGI0, SGI225881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225882, SGI225883, SGI225883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50123[] = {SGI225884, SGI0, SGI225885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225886, SGI225887, SGI225887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50124[] = {SGI225888, SGI0, SGI225889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225890, SGI225891, SGI225891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50125[] = {SGI225892, SGI0, SGI225894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225896, SGI225898, SGI225898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50126[] = {SGI225893, SGI0, SGI225895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225897, SGI225899, SGI225899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50127[] = {SGI225900, SGI225901, SGI225902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225903, SGI0, SGI0, SGI0, SGI225904, SGI225904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50128[] = {SGI225905, SGI225906, SGI225907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225908, SGI225908, SGI0, SGI0, SGI0, SGI225909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50129[] = {SGI225910, SGI0, SGI225911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225912, SGI225913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50130[] = {SGI225914, SGI0, SGI225915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225916, SGI225917, SGI225917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50131[] = {SGI225918, SGI225919, SGI225920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225921, SGI225921, SGI0, SGI0, SGI225922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50132[] = {SGI225923, SGI225924, SGI225925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225926, SGI225926, SGI0, SGI0, SGI0, SGI0, SGI225927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50133[] = {SGI225928, SGI225929, SGI225930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225931, SGI225931, SGI0, SGI0, SGI0, SGI0, SGI225932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50134[] = {SGI225933, SGI0, SGI225934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225935, SGI225936, SGI225936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225937, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50135[] = {SGI225938, SGI0, SGI225939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225940, SGI225941, SGI225941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225942, SGI0, SGI0, NULL};
const Segment * const SG50136[] = {SGI225943, SGI0, SGI225944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225945, SGI225946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50137[] = {SGI225947, SGI0, SGI225948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225949, SGI225950, SGI225950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50138[] = {SGI225951, SGI225952, SGI225953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225954, SGI0, SGI0, SGI0, SGI225955, SGI225955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225956, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50139[] = {SGI225957, SGI0, SGI225958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225959, SGI225960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50140[] = {SGI225961, SGI0, SGI225962, SGI0, SGI225963, SGI225963, SGI0, NULL};
const Segment * const SG50141[] = {SGI225964, SGI225965, SGI225966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225967, SGI0, SGI0, SGI0, SGI225968, SGI0, SGI0, SGI225969, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50142[] = {SGI225970, SGI0, SGI225971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225972, SGI225973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50143[] = {SGI225974, SGI0, SGI225975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225976, SGI0, SGI0, SGI0, SGI225977, SGI225977, SGI0, SGI0, SGI0, SGI225978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50144[] = {SGI225979, SGI0, SGI225980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225981, SGI225982, SGI225982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50145[] = {SGI225983, SGI225984, SGI225985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225986, SGI0, SGI0, SGI0, SGI225987, SGI225987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50146[] = {SGI225988, SGI225989, SGI225991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225993, SGI0, SGI0, SGI0, SGI225994, SGI225994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225997, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50147[] = {SGI225990, SGI0, SGI225992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI225995, SGI225996, SGI225996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50148[] = {SGI225998, SGI225999, SGI226000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226001, SGI0, SGI0, SGI0, SGI226002, SGI226002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50149[] = {SGI226003, SGI0, SGI226004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226005, SGI226006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50150[] = {SGI226007, SGI0, SGI226008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226009, SGI226010, SGI226010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50151[] = {SGI226011, SGI226012, SGI226013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226014, SGI0, SGI0, SGI0, SGI226015, SGI226015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50152[] = {SGI226016, SGI226017, SGI226018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226019, SGI0, SGI0, SGI0, SGI226020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226021, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50153[] = {SGI226022, SGI0, SGI226023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226024, SGI226025, SGI226025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50154[] = {SGI226026, SGI0, SGI226027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226028, SGI226029, SGI226029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50155[] = {SGI226030, SGI226031, SGI226032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226033, SGI0, SGI0, SGI0, SGI226035, SGI226035, SGI0, SGI0, SGI0, SGI226038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50156[] = {SGI226034, SGI226036, SGI226037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226040, SGI0, SGI0, SGI0, SGI226042, SGI226042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50157[] = {SGI226039, SGI0, SGI226041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226044, SGI226045, SGI226045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50158[] = {SGI226046, SGI0, SGI226047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226050, SGI226051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50159[] = {SGI226048, SGI0, SGI226049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226052, SGI226053, SGI226053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50160[] = {SGI226054, SGI0, SGI226055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226056, SGI226057, SGI226057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50161[] = {SGI226058, SGI0, SGI226059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226060, SGI226061, SGI226061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50162[] = {SGI226062, SGI0, SGI226063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226064, SGI226065, SGI226065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50163[] = {SGI226066, SGI226067, SGI226068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226069, SGI0, SGI0, SGI0, SGI226070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226072, SGI0, NULL};
const Segment * const SG50164[] = {SGI226073, SGI0, SGI226074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226075, SGI226076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50165[] = {SGI226077, SGI226078, SGI226079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226080, SGI0, SGI0, SGI0, SGI226081, SGI226081, SGI0, SGI0, NULL};
const Segment * const SG50166[] = {SGI226082, SGI226083, SGI226084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226085, SGI226085, SGI0, SGI0, SGI0, SGI226086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50167[] = {SGI226087, SGI0, SGI226088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226089, SGI226090, SGI226090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50168[] = {SGI226091, SGI0, SGI226092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226096, SGI226097, SGI226097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50169[] = {SGI226093, SGI226094, SGI226095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226098, SGI0, SGI0, SGI0, SGI226099, SGI226099, SGI0, SGI226100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50170[] = {SGI226101, SGI0, SGI226102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226103, SGI226104, SGI226104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50171[] = {SGI226105, SGI0, SGI226106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226107, SGI226108, SGI226108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50172[] = {SGI226109, SGI0, SGI226110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226114, SGI226115, SGI226115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50173[] = {SGI226111, SGI226112, SGI226113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226116, SGI0, SGI0, SGI0, SGI226117, SGI226117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50174[] = {SGI226119, SGI0, SGI226120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226121, SGI226122, SGI226122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50175[] = {SGI226123, SGI0, SGI226124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226125, SGI226126, SGI226126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50176[] = {SGI226127, SGI0, SGI226128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226129, SGI226130, SGI226130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50177[] = {SGI226131, SGI0, SGI226132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226133, SGI226134, SGI226134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50178[] = {SGI226135, SGI0, SGI226136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226137, SGI226138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50179[] = {SGI226139, SGI226140, SGI226141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226142, SGI0, SGI0, SGI0, SGI226143, SGI226143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50180[] = {SGI226145, SGI0, SGI226148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226151, SGI226152, SGI226152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50181[] = {SGI226146, SGI226147, SGI226149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226150, SGI0, SGI0, SGI0, SGI226153, SGI226153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226154, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50182[] = {SGI226155, SGI0, SGI226156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226157, SGI226158, SGI226158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50183[] = {SGI226159, SGI226160, SGI226161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226164, SGI0, SGI0, SGI0, SGI226165, SGI226165, SGI0, SGI0, SGI226167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50184[] = {SGI226162, SGI0, SGI226163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226166, SGI226168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50185[] = {SGI226169, SGI0, SGI226170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226171, SGI226172, SGI226172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50186[] = {SGI226173, SGI226174, SGI226175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226176, SGI0, SGI0, SGI0, SGI226179, SGI226179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226182, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50187[] = {SGI226178, SGI0, SGI226181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226183, SGI226185, SGI226185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50188[] = {SGI226177, SGI0, SGI226180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226184, SGI226186, SGI226186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50189[] = {SGI226187, SGI0, SGI226188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226189, SGI226190, SGI226190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50190[] = {SGI226191, SGI0, SGI226192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226193, SGI226194, SGI226194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226195, SGI0, SGI0, NULL};
const Segment * const SG50191[] = {SGI226196, SGI0, SGI226197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226198, SGI226199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50192[] = {SGI226200, SGI226201, SGI226202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226203, SGI0, SGI0, SGI0, SGI226204, SGI226204, SGI0, SGI226205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50193[] = {SGI226206, SGI0, SGI226207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226208, SGI226209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50194[] = {SGI226210, SGI226211, SGI226212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226213, SGI0, SGI0, SGI0, SGI226214, SGI226214, SGI0, SGI226215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50195[] = {SGI226216, SGI226217, SGI226218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226219, SGI226219, SGI0, SGI0, SGI0, SGI0, SGI226220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50196[] = {SGI226221, SGI226222, SGI226223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226224, SGI0, SGI0, SGI0, SGI226225, SGI226225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50197[] = {SGI226226, SGI0, SGI226227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226228, SGI226229, SGI226229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50198[] = {SGI226230, SGI0, SGI226231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226234, SGI226235, SGI226235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226236, SGI0, SGI0, NULL};
const Segment * const SG50199[] = {SGI226232, SGI0, SGI226233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226237, SGI226238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50200[] = {SGI226239, SGI226240, SGI226241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226242, SGI0, SGI0, SGI0, SGI226243, SGI226243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50201[] = {SGI226244, SGI226245, SGI226246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226249, SGI0, SGI0, SGI0, SGI226250, SGI226250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226252, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50202[] = {SGI226247, SGI0, SGI226248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226251, SGI226253, SGI226253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50203[] = {SGI226254, SGI0, SGI226255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226256, SGI226257, SGI226257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50204[] = {SGI226258, SGI0, SGI226259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226260, SGI226261, SGI226261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50205[] = {SGI226262, SGI226263, SGI226264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226265, SGI226265, SGI0, SGI0, SGI0, SGI0, SGI226266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50206[] = {SGI226267, SGI0, SGI226268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226269, SGI226270, SGI226270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50207[] = {SGI226271, SGI0, SGI226272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226273, SGI226274, SGI226274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226275, SGI0, SGI0, NULL};
const Segment * const SG50208[] = {SGI226276, SGI0, SGI226277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226278, SGI226280, SGI226280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226282, SGI0, SGI0, NULL};
const Segment * const SG50209[] = {SGI226279, SGI0, SGI226281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226283, SGI226284, SGI226284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50210[] = {SGI226285, SGI226286, SGI226287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226288, SGI0, SGI0, SGI0, SGI226289, SGI226289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226290, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50211[] = {SGI226291, SGI0, SGI226292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226293, SGI226294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50212[] = {SGI226295, SGI226296, SGI226297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226298, SGI226298, SGI0, SGI0, SGI0, SGI226299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50213[] = {SGI226300, SGI0, SGI226301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226302, SGI226303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50214[] = {SGI226304, SGI226305, SGI226306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226307, SGI0, SGI0, SGI0, SGI226308, SGI226308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50215[] = {SGI226309, SGI0, SGI226310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226311, SGI226312, SGI226312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50216[] = {SGI226313, SGI226314, SGI226315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226316, SGI0, SGI0, SGI0, SGI226317, SGI226317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50217[] = {SGI226318, SGI0, SGI226319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226320, SGI226321, SGI226321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50218[] = {SGI226322, SGI0, SGI226323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226324, SGI226325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50219[] = {SGI226326, SGI0, SGI226327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226328, SGI226329, SGI226329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50220[] = {SGI226330, SGI0, SGI226331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226334, SGI226335, SGI226335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50221[] = {SGI226332, SGI0, SGI226333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226336, SGI226337, SGI226337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50222[] = {SGI226338, SGI0, SGI226339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226340, SGI226341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50223[] = {SGI226342, SGI226343, SGI226344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226345, SGI0, SGI0, SGI0, SGI226346, SGI226346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226347, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50224[] = {SGI226348, SGI0, SGI226349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226350, SGI226351, SGI226351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50225[] = {SGI226352, SGI226353, SGI226354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226355, SGI0, SGI0, SGI0, SGI226356, SGI226356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226359, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50226[] = {SGI226357, SGI226358, SGI226360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226361, SGI0, SGI0, SGI0, SGI226362, SGI226362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50227[] = {SGI226364, SGI0, SGI226365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226366, SGI226367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50228[] = {SGI226368, SGI0, SGI226369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226370, SGI226371, SGI226371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50229[] = {SGI226372, SGI0, SGI226373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226376, SGI226377, SGI226377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50230[] = {SGI226374, SGI0, SGI226375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226378, SGI226379, SGI226379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50231[] = {SGI226380, SGI226381, SGI226382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226385, SGI226385, SGI0, SGI0, SGI0, SGI226386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50232[] = {SGI226383, SGI0, SGI226384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226387, SGI226388, SGI226388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50233[] = {SGI226389, SGI226390, SGI226391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226392, SGI226392, SGI0, SGI0, SGI0, SGI226393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50234[] = {SGI0, SGI226394, SGI226395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226396, SGI0, SGI0, SGI0, SGI226397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50235[] = {SGI226398, SGI0, SGI226399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226400, SGI226401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50236[] = {SGI226402, SGI0, SGI226403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226406, SGI226407, SGI226407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50237[] = {SGI226404, SGI0, SGI226405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226408, SGI226409, SGI226409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50238[] = {SGI226410, SGI0, SGI226411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226412, SGI226413, SGI226413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50239[] = {SGI226414, SGI226415, SGI226416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226417, SGI0, SGI0, SGI0, SGI226418, SGI226418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50240[] = {SGI226419, SGI226420, SGI226421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226422, SGI0, SGI0, SGI0, SGI226423, SGI226423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50241[] = {SGI226424, SGI0, SGI226425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226426, SGI226427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50242[] = {SGI226428, SGI0, SGI226429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226430, SGI226431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50243[] = {SGI226432, SGI0, SGI226433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226434, SGI226435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50244[] = {SGI226436, SGI226437, SGI226438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226439, SGI0, SGI0, SGI0, SGI226440, SGI226440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226441, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50245[] = {SGI226442, SGI226443, SGI226444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226445, SGI0, SGI0, SGI0, SGI226446, SGI226446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50246[] = {SGI226447, SGI0, SGI226448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226449, SGI226450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50247[] = {SGI226451, SGI226452, SGI226453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226454, SGI0, SGI0, SGI0, SGI226455, SGI226455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50248[] = {SGI226457, SGI0, SGI226458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226462, SGI226463, SGI226463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50249[] = {SGI226459, SGI226460, SGI226461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226464, SGI0, SGI0, SGI0, SGI226465, SGI226465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50250[] = {SGI226467, SGI0, SGI226468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226469, SGI226470, SGI226470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50251[] = {SGI226471, SGI0, SGI226472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226473, SGI226474, SGI226474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50252[] = {SGI226475, SGI0, SGI226476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226477, SGI226478, SGI226478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50253[] = {SGI226479, SGI0, SGI226480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226483, SGI226485, SGI226485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50254[] = {SGI226481, SGI226482, SGI226484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226486, SGI0, SGI0, SGI0, SGI226487, SGI226487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226488, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50255[] = {SGI226489, SGI0, SGI226490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226491, SGI226492, SGI226492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50256[] = {SGI226493, SGI226494, SGI226495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226496, SGI0, SGI0, SGI0, SGI226497, SGI226497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50257[] = {SGI226498, SGI226499, SGI226500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226501, SGI0, SGI0, SGI0, SGI226502, SGI226502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50258[] = {SGI226503, SGI226504, SGI226505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226506, SGI226506, SGI0, SGI0, SGI0, SGI0, SGI226507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50259[] = {SGI226508, SGI0, SGI226510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226512, SGI226513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50260[] = {SGI226509, SGI0, SGI226511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226514, SGI226515, SGI226515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50261[] = {SGI226516, SGI0, SGI226517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226518, SGI226519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50262[] = {SGI226520, SGI0, SGI226521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226522, SGI226523, SGI226523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50263[] = {SGI226524, SGI0, SGI226525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226528, SGI226529, SGI226529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50264[] = {SGI226526, SGI0, SGI226527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226530, SGI226531, SGI226531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50265[] = {SGI226532, SGI226533, SGI226534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226535, SGI0, SGI0, SGI0, SGI226536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226537, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50266[] = {SGI226538, SGI226539, SGI226540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226541, SGI0, SGI0, SGI0, SGI226542, SGI0, SGI0, SGI226543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50267[] = {SGI226544, SGI226545, SGI226546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226547, SGI0, SGI0, SGI0, SGI226548, SGI226548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50268[] = {SGI226550, SGI226551, SGI226552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226553, SGI0, SGI0, SGI0, SGI226554, SGI0, SGI0, SGI0, SGI226555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50269[] = {SGI226556, SGI0, SGI226557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226558, SGI226559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50270[] = {SGI226560, SGI226561, SGI226562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226563, SGI0, SGI0, SGI0, SGI226564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50271[] = {SGI226565, SGI0, SGI226566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226567, SGI226568, SGI226568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50272[] = {SGI226569, SGI0, SGI226570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226571, SGI226572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50273[] = {SGI226573, SGI0, SGI226574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226575, SGI0, SGI0, SGI0, SGI226577, SGI226577, SGI0, SGI0, SGI0, SGI226580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50274[] = {SGI226576, SGI226578, SGI226579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226581, SGI0, SGI0, SGI0, SGI226582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50275[] = {SGI226583, SGI0, SGI226584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226585, SGI226586, SGI226586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50276[] = {SGI226587, SGI0, SGI226588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226589, SGI0, SGI0, SGI0, SGI226590, SGI226590, SGI0, SGI0, SGI0, SGI226591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50277[] = {SGI226592, SGI0, SGI226593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226594, SGI226595, SGI226595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50278[] = {SGI226596, SGI0, SGI226597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226598, SGI226599, SGI226599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50279[] = {SGI226600, SGI226601, SGI226602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226603, SGI0, SGI0, SGI0, SGI226604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50280[] = {SGI226605, SGI0, SGI226606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226607, SGI226608, SGI226608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50281[] = {SGI226609, SGI0, SGI226610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226611, SGI226612, SGI226612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50282[] = {SGI226613, SGI0, SGI226614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226615, SGI226616, SGI226616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50283[] = {SGI226617, SGI0, SGI226618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226619, SGI0, SGI0, SGI0, SGI226620, SGI226620, SGI0, SGI0, SGI0, SGI226624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50284[] = {SGI226621, SGI226622, SGI226623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226625, SGI0, SGI0, SGI0, SGI226626, SGI226626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50285[] = {SGI226627, SGI0, SGI226628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226629, SGI226630, SGI226630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50286[] = {SGI226631, SGI226632, SGI226633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226634, SGI0, SGI0, SGI0, SGI226635, SGI226635, SGI0, SGI0, SGI226637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50287[] = {SGI226636, SGI0, SGI226638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226639, SGI0, SGI0, SGI0, SGI226640, SGI226640, SGI0, SGI0, SGI0, SGI226641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50288[] = {SGI226642, SGI226643, SGI226644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226645, SGI226645, SGI0, SGI0, SGI0, SGI226646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50289[] = {SGI226647, SGI226648, SGI226649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226650, SGI0, SGI0, SGI0, SGI226651, SGI226651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50290[] = {SGI226652, SGI226653, SGI226654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226655, SGI0, SGI0, SGI0, SGI226656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50291[] = {SGI226657, SGI0, SGI226658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226659, SGI226660, SGI226660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50292[] = {SGI226661, SGI0, SGI226662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226663, SGI226664, SGI226664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50293[] = {SGI226665, SGI226666, SGI226667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226668, SGI0, SGI0, SGI0, SGI226669, SGI226669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226671, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50294[] = {SGI226670, SGI0, SGI226672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226673, SGI226674, SGI226674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50295[] = {SGI226675, SGI0, SGI226676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226680, SGI226681, SGI226681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50296[] = {SGI226677, SGI226678, SGI226679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226682, SGI0, SGI0, SGI0, SGI226683, SGI226683, SGI0, SGI0, SGI226684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50297[] = {SGI226685, SGI226686, SGI226687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226688, SGI0, SGI0, SGI0, SGI226689, SGI226689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50298[] = {SGI226690, SGI0, SGI226691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226692, SGI226693, SGI226693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50299[] = {SGI226694, SGI226695, SGI226696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226697, SGI0, SGI0, SGI0, SGI226698, SGI226698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50300[] = {SGI226699, SGI0, SGI226700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226701, SGI0, SGI0, SGI0, SGI226703, SGI226703, SGI0, SGI0, SGI0, SGI226705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50301[] = {SGI226702, SGI0, SGI226704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226706, SGI226707, SGI226707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50302[] = {SGI226708, SGI226709, SGI226710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226711, SGI0, SGI0, SGI0, SGI226712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50303[] = {SGI226713, SGI0, SGI226714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226715, SGI226716, SGI226716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50304[] = {SGI226717, SGI226718, SGI226719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226720, SGI0, SGI0, SGI0, SGI226721, SGI226721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226722, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50305[] = {SGI226723, SGI0, SGI226724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226725, SGI226726, SGI226726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50306[] = {SGI226727, SGI226728, SGI226729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226730, SGI226730, SGI0, SGI0, SGI0, SGI0, SGI226731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50307[] = {SGI226732, SGI226733, SGI226734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226735, SGI0, SGI0, SGI0, SGI226736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50308[] = {SGI226737, SGI0, SGI226738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226739, SGI226740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50309[] = {SGI226741, SGI0, SGI226742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226743, SGI226744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50310[] = {SGI226745, SGI0, SGI226746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226747, SGI226748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50311[] = {SGI226749, SGI0, SGI226750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226751, SGI226752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50312[] = {SGI0, SGI0, SGI226753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226754, SGI226755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50313[] = {SGI226756, SGI226757, SGI226758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226759, SGI0, SGI0, SGI0, SGI226760, SGI226760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226761, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50314[] = {SGI226762, SGI0, SGI226763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226764, SGI226765, SGI226765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50315[] = {SGI226766, SGI0, SGI226767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226768, SGI226769, SGI226769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50316[] = {SGI226770, SGI0, SGI226771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226773, SGI226775, SGI226775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50317[] = {SGI226772, SGI0, SGI226774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226776, SGI226777, SGI226777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50318[] = {SGI226778, SGI0, SGI226779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226780, SGI226781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50319[] = {SGI226782, SGI0, SGI226783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226784, SGI226785, SGI226785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50320[] = {SGI226786, SGI0, SGI226787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226788, SGI226789, SGI226789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50321[] = {SGI226790, SGI0, SGI226791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226793, SGI226794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50322[] = {SGI226792, SGI0, SGI226795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226796, SGI226797, SGI226797, SGI0, SGI0, SGI0, SGI0, SGI226798, SGI0, NULL};
const Segment * const SG50323[] = {SGI226799, SGI0, SGI226800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226801, SGI226802, SGI226802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50324[] = {SGI226803, SGI226804, SGI226805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226806, SGI0, SGI0, SGI0, SGI226807, SGI226807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226808, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50325[] = {SGI226809, SGI226810, SGI226811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226812, SGI0, SGI0, SGI0, SGI226813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50326[] = {SGI226814, SGI0, SGI226815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226816, SGI226817, SGI226817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50327[] = {SGI226818, SGI0, SGI226819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226820, SGI226821, SGI226821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226822, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50328[] = {SGI226823, SGI0, SGI226825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226828, SGI226829, SGI226829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50329[] = {SGI226824, SGI226826, SGI226827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226830, SGI0, SGI0, SGI0, SGI226831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50330[] = {SGI226832, SGI0, SGI226833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226834, SGI226835, SGI226835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50331[] = {SGI226836, SGI0, SGI226837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226838, SGI226839, SGI226839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50332[] = {SGI226840, SGI0, SGI226841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226842, SGI226843, SGI226843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50333[] = {SGI226844, SGI0, SGI226845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226846, SGI226847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50334[] = {SGI226848, SGI0, SGI226849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226850, SGI226851, SGI226851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50335[] = {SGI226852, SGI0, SGI226853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226854, SGI226855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50336[] = {SGI226856, SGI226857, SGI226858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226859, SGI0, SGI0, SGI0, SGI226860, SGI0, SGI0, SGI0, SGI226861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50337[] = {SGI226864, SGI226866, SGI226867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226868, SGI0, SGI0, SGI0, SGI226870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226872, SGI0, NULL};
const Segment * const SG50338[] = {SGI226862, SGI226863, SGI226865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226869, SGI226869, SGI0, SGI0, SGI0, SGI226871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50339[] = {SGI226873, SGI0, SGI226874, SGI0, SGI226875, SGI226875, SGI0, NULL};
const Segment * const SG50340[] = {SGI226876, SGI0, SGI226877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226878, SGI226879, SGI226879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50341[] = {SGI226880, SGI0, SGI226881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226883, SGI226885, SGI226885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226886, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50342[] = {SGI226882, SGI0, SGI226884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226887, SGI226888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50343[] = {SGI226889, SGI0, SGI226890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226891, SGI226892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50344[] = {SGI226893, SGI226894, SGI226895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226896, SGI0, SGI0, SGI0, SGI226897, SGI0, SGI0, SGI226898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50345[] = {SGI226899, SGI0, SGI226900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226901, SGI226902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50346[] = {SGI226903, SGI0, SGI226904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226905, SGI226906, SGI226906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50347[] = {SGI226907, SGI226908, SGI226909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226910, SGI0, SGI0, SGI0, SGI226911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50348[] = {SGI226912, SGI0, SGI226913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226914, SGI226915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50349[] = {SGI226916, SGI226917, SGI226918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226919, SGI226919, SGI0, SGI0, SGI0, SGI0, SGI226920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50350[] = {SGI226921, SGI226922, SGI226923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226924, SGI0, SGI0, SGI0, SGI226925, SGI226925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50351[] = {SGI226927, SGI226928, SGI226929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226930, SGI226930, SGI0, SGI0, SGI0, SGI0, SGI226931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50352[] = {SGI226932, SGI0, SGI226933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226934, SGI226935, SGI226935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50353[] = {SGI226936, SGI226937, SGI226938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226939, SGI0, SGI0, SGI0, SGI226940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50354[] = {SGI226941, SGI226942, SGI226943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226944, SGI0, SGI0, SGI0, SGI226945, SGI226945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50355[] = {SGI226946, SGI0, SGI226947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226948, SGI226949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50356[] = {SGI226950, SGI0, SGI226951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226952, SGI226953, SGI226953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50357[] = {SGI226954, SGI0, SGI226955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226956, SGI226957, SGI226957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50358[] = {SGI226958, SGI226959, SGI226960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226961, SGI0, SGI0, SGI0, SGI226962, SGI226963, SGI0, SGI0, SGI0, SGI0, SGI226964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50359[] = {SGI226965, SGI0, SGI226966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226967, SGI226968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50360[] = {SGI226969, SGI226970, SGI226971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226972, SGI226972, SGI0, SGI0, SGI0, SGI226973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50361[] = {SGI226974, SGI226975, SGI226976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226977, SGI226977, SGI0, SGI0, SGI0, SGI226978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50362[] = {SGI226979, SGI226980, SGI226981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226984, SGI226984, SGI0, SGI0, SGI0, SGI226986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50363[] = {SGI226982, SGI0, SGI226983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226985, SGI226987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50364[] = {SGI226988, SGI0, SGI226989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226990, SGI226991, SGI226991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226992, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50365[] = {SGI226993, SGI226994, SGI226995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI226996, SGI0, SGI0, SGI0, SGI226997, SGI226997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50366[] = {SGI226998, SGI0, SGI226999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227000, SGI227001, SGI227001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50367[] = {SGI227002, SGI227003, SGI227004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227005, SGI0, SGI0, SGI0, SGI227006, SGI227006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50368[] = {SGI227007, SGI0, SGI227008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227009, SGI227010, SGI227010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50369[] = {SGI227011, SGI0, SGI227012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227013, SGI227014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50370[] = {SGI227015, SGI227016, SGI227017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227018, SGI227018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227019, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50371[] = {SGI227020, SGI0, SGI227021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227022, SGI227023, SGI227023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50372[] = {SGI227024, SGI0, SGI227026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227028, SGI227029, SGI227029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50373[] = {SGI227032, SGI227033, SGI227034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227035, SGI0, SGI0, SGI0, SGI227036, SGI227036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227037, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50374[] = {SGI227038, SGI0, SGI227039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227040, SGI227041, SGI227041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227042, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50375[] = {SGI227043, SGI227044, SGI227045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227046, SGI0, SGI0, SGI0, SGI227047, SGI227047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50376[] = {SGI227048, SGI227049, SGI227050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227051, SGI227051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227052, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50377[] = {SGI227053, SGI227054, SGI227055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227056, SGI227056, SGI0, SGI0, SGI227057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50378[] = {SGI227058, SGI0, SGI227059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227060, SGI227061, SGI227061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50379[] = {SGI227062, SGI0, SGI227063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227064, SGI227065, SGI227065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50380[] = {SGI227066, SGI0, SGI227067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227068, SGI227069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50381[] = {SGI227070, SGI227071, SGI227072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227073, SGI227073, SGI0, SGI0, SGI0, SGI227074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50382[] = {SGI227075, SGI0, SGI227076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227077, SGI227078, SGI227078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50383[] = {SGI227079, SGI0, SGI227080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227081, SGI227082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50384[] = {SGI227083, SGI0, SGI227084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227085, SGI227086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50385[] = {SGI227087, SGI0, SGI227088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227089, SGI227090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50386[] = {SGI227091, SGI0, SGI227092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227093, SGI227094, SGI227094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50387[] = {SGI227095, SGI0, SGI227096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227100, SGI227101, SGI227101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50388[] = {SGI227097, SGI227098, SGI227099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227102, SGI0, SGI0, SGI0, SGI227103, SGI0, SGI0, SGI227104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50389[] = {SGI227105, SGI227106, SGI227107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227108, SGI0, SGI0, SGI0, SGI227109, SGI227109, SGI0, SGI227110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50390[] = {SGI227111, SGI227112, SGI227113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227114, SGI0, SGI0, SGI0, SGI227115, SGI227115, SGI0, SGI227116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50391[] = {SGI227117, SGI0, SGI227118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227119, SGI227120, SGI227120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50392[] = {SGI227121, SGI227122, SGI227123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227124, SGI0, SGI0, SGI0, SGI227125, SGI0, SGI0, SGI227126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50393[] = {SGI227127, SGI227128, SGI227129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227130, SGI0, SGI0, SGI0, SGI227131, SGI0, SGI0, SGI227132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50394[] = {SGI227133, SGI227134, SGI227135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227138, SGI227138, SGI0, SGI0, SGI0, SGI0, SGI227139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50395[] = {SGI227136, SGI0, SGI227137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227140, SGI227141, SGI227141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50396[] = {SGI227142, SGI227143, SGI227144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227145, SGI0, SGI0, SGI0, SGI227146, SGI227146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227147, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50397[] = {SGI227148, SGI227149, SGI227150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227151, SGI0, SGI0, SGI0, SGI227152, SGI227152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227153, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50398[] = {SGI227154, SGI0, SGI227155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227156, SGI227157, SGI227157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50399[] = {SGI227158, SGI0, SGI227159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227160, SGI227161, SGI227161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50400[] = {SGI227162, SGI227163, SGI227164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227165, SGI0, SGI0, SGI0, SGI227166, SGI227166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50401[] = {SGI227167, SGI0, SGI227168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227169, SGI227170, SGI227170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50402[] = {SGI227171, SGI0, SGI227172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227173, SGI227174, SGI227174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227175, SGI0, SGI0, NULL};
const Segment * const SG50403[] = {SGI227176, SGI0, SGI227177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227178, SGI227180, SGI227180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227182, SGI0, SGI0, NULL};
const Segment * const SG50404[] = {SGI227179, SGI0, SGI227181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227183, SGI227185, SGI227185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50405[] = {SGI227184, SGI227186, SGI227187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227188, SGI0, SGI0, SGI0, SGI227189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50406[] = {SGI227191, SGI0, SGI227192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227193, SGI227194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50407[] = {SGI227195, SGI0, SGI227196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227197, SGI227198, SGI227198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50408[] = {SGI227199, SGI227200, SGI227201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227202, SGI0, SGI0, SGI0, SGI227203, SGI227203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227204, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50409[] = {SGI227205, SGI0, SGI227206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227207, SGI227208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50410[] = {SGI227209, SGI0, SGI227210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227211, SGI227212, SGI227212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50411[] = {SGI227213, SGI0, SGI227214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227215, SGI227216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50412[] = {SGI227217, SGI0, SGI227218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227220, SGI0, SGI0, SGI0, SGI227223, SGI227223, SGI0, SGI0, SGI0, SGI227224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50413[] = {SGI227219, SGI227221, SGI227222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227225, SGI0, SGI0, SGI0, SGI227226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50414[] = {SGI227227, SGI0, SGI227228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227229, SGI227230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50415[] = {SGI227231, SGI0, SGI227232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227233, SGI227234, SGI227234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50416[] = {SGI227235, SGI227236, SGI227237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227238, SGI0, SGI0, SGI0, SGI227239, SGI227239, SGI0, SGI0, SGI227240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50417[] = {SGI227241, SGI0, SGI227242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227243, SGI227244, SGI227244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50418[] = {SGI227245, SGI227246, SGI227247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227248, SGI0, SGI0, SGI0, SGI227249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50419[] = {SGI227251, SGI227252, SGI227253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227254, SGI0, SGI0, SGI0, SGI227255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50420[] = {SGI227257, SGI227258, SGI227259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227262, SGI0, SGI0, SGI0, SGI227264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227266, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50421[] = {SGI227260, SGI227261, SGI227263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227265, SGI0, SGI0, SGI0, SGI227267, SGI227267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50422[] = {SGI227268, SGI0, SGI227269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227270, SGI227271, SGI227271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50423[] = {SGI227272, SGI227273, SGI227274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227275, SGI227275, SGI0, SGI0, SGI227276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50424[] = {SGI227277, SGI227278, SGI227279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227280, SGI0, SGI0, SGI0, SGI227281, SGI227281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227282, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50425[] = {SGI227283, SGI0, SGI227284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227285, SGI227286, SGI227286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50426[] = {SGI227287, SGI0, SGI227288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227294, SGI0, SGI0, SGI0, SGI227295, SGI227295, SGI0, SGI0, SGI0, SGI227298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50427[] = {SGI227290, SGI0, SGI227293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227297, SGI227299, SGI227299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50428[] = {SGI227289, SGI227291, SGI227292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227296, SGI0, SGI0, SGI0, SGI227300, SGI227300, SGI0, SGI227301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50429[] = {SGI227302, SGI0, SGI227303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227304, SGI0, SGI0, SGI0, SGI227305, SGI227305, SGI0, SGI0, SGI0, SGI227306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50430[] = {SGI227307, SGI0, SGI227308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227309, SGI227310, SGI227310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50431[] = {SGI227311, SGI0, SGI227312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227313, SGI227314, SGI227314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50432[] = {SGI227315, SGI0, SGI227316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227317, SGI227318, SGI227318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50433[] = {SGI227319, SGI227320, SGI227321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227323, SGI227323, SGI0, SGI0, SGI0, SGI0, SGI227325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50434[] = {SGI227322, SGI0, SGI227324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227326, SGI227327, SGI227327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50435[] = {SGI227328, SGI0, SGI227329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227330, SGI227331, SGI227331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50436[] = {SGI227332, SGI227333, SGI227334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227335, SGI0, SGI0, SGI0, SGI227336, SGI227336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227337, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50437[] = {SGI227338, SGI227339, SGI227340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227341, SGI0, SGI0, SGI0, SGI227342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50438[] = {SGI227343, SGI227344, SGI227345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227346, SGI0, SGI0, SGI0, SGI227347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50439[] = {SGI227348, SGI227349, SGI227350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227351, SGI0, SGI0, SGI0, SGI227352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50440[] = {SGI227353, SGI227354, SGI227355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227356, SGI0, SGI0, SGI0, SGI227357, SGI227357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50441[] = {SGI227358, SGI227359, SGI227360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227363, SGI0, SGI0, SGI0, SGI227364, SGI227364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50442[] = {SGI227361, SGI0, SGI227362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227365, SGI227366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50443[] = {SGI227367, SGI227368, SGI227369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227370, SGI0, SGI0, SGI0, SGI227371, SGI227371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227372, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50444[] = {SGI227373, SGI0, SGI227374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227375, SGI227376, SGI227376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50445[] = {SGI227377, SGI0, SGI227378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227379, SGI227380, SGI227380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50446[] = {SGI227381, SGI227382, SGI227383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227384, SGI0, SGI0, SGI0, SGI227385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50447[] = {SGI227386, SGI0, SGI227387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227388, SGI227389, SGI227389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50448[] = {SGI227390, SGI0, SGI227391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227394, SGI227395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50449[] = {SGI227392, SGI0, SGI227393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227396, SGI227397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50450[] = {SGI227398, SGI0, SGI227399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227400, SGI227401, SGI227401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50451[] = {SGI227402, SGI0, SGI227403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227404, SGI227405, SGI227405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50452[] = {SGI227407, SGI0, SGI227409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227410, SGI227412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50453[] = {SGI227406, SGI0, SGI227408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227411, SGI227413, SGI227413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50454[] = {SGI227414, SGI227415, SGI227416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227417, SGI227417, SGI0, SGI0, SGI0, SGI0, SGI227419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50455[] = {SGI227418, SGI0, SGI227420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227421, SGI227422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50456[] = {SGI227423, SGI0, SGI227425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227427, SGI227428, SGI227428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50457[] = {SGI227424, SGI0, SGI227426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227429, SGI227430, SGI227430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50458[] = {SGI227431, SGI0, SGI227432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227433, SGI227434, SGI227434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50459[] = {SGI227435, SGI227436, SGI227437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227438, SGI0, SGI0, SGI0, SGI227439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50460[] = {SGI227440, SGI0, SGI227441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227442, SGI227443, SGI227443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50461[] = {SGI227444, SGI0, SGI227445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227446, SGI227447, SGI227447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50462[] = {SGI227448, SGI227449, SGI227450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227451, SGI0, SGI0, SGI0, SGI227452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50463[] = {SGI227453, SGI0, SGI227454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227455, SGI227456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50464[] = {SGI227457, SGI0, SGI227458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227459, SGI227460, SGI227460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50465[] = {SGI227461, SGI0, SGI227462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227463, SGI227464, SGI227464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50466[] = {SGI227465, SGI227466, SGI227467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227468, SGI0, SGI0, SGI0, SGI227469, SGI227469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50467[] = {SGI227470, SGI227471, SGI227472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227473, SGI0, SGI0, SGI0, SGI227474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227475, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50468[] = {SGI227476, SGI0, SGI227477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227478, SGI227479, SGI227479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50469[] = {SGI227480, SGI227481, SGI227482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227483, SGI0, SGI0, SGI0, SGI227484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50470[] = {SGI227485, SGI227486, SGI227487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227488, SGI0, SGI0, SGI0, SGI227489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50471[] = {SGI227490, SGI0, SGI227491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227492, SGI227493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50472[] = {SGI227494, SGI0, SGI227495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227496, SGI227497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50473[] = {SGI227498, SGI0, SGI227499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227500, SGI227501, SGI227501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50474[] = {SGI227502, SGI0, SGI227503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227504, SGI227505, SGI227505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50475[] = {SGI227506, SGI227507, SGI227508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227509, SGI0, SGI0, SGI0, SGI227511, SGI227511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50476[] = {SGI227510, SGI0, SGI227512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227513, SGI227514, SGI227514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50477[] = {SGI227515, SGI0, SGI227516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227517, SGI227518, SGI227518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50478[] = {SGI227519, SGI0, SGI227520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227523, SGI227524, SGI227524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50479[] = {SGI227521, SGI0, SGI227522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227525, SGI227526, SGI227526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227527, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50480[] = {SGI227528, SGI227529, SGI227530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227531, SGI0, SGI0, SGI0, SGI227532, SGI227532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50481[] = {SGI227533, SGI0, SGI227534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227535, SGI227536, SGI227536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50482[] = {SGI227537, SGI227538, SGI227539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227541, SGI0, SGI0, SGI0, SGI227543, SGI227543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50483[] = {SGI227540, SGI0, SGI227542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227544, SGI227545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227546, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50484[] = {SGI227547, SGI0, SGI227548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227549, SGI227550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50485[] = {SGI227551, SGI0, SGI227552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227553, SGI227554, SGI227554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50486[] = {SGI227555, SGI0, SGI227556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227557, SGI227558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50487[] = {SGI227559, SGI227560, SGI227561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227562, SGI0, SGI0, SGI0, SGI227563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50488[] = {SGI227564, SGI0, SGI227565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227566, SGI227567, SGI227567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50489[] = {SGI227568, SGI0, SGI227569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227570, SGI227571, SGI227571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50490[] = {SGI227572, SGI227574, SGI227576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227578, SGI0, SGI0, SGI0, SGI227580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50491[] = {SGI227573, SGI227575, SGI227577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227579, SGI0, SGI0, SGI0, SGI227581, SGI227581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50492[] = {SGI227583, SGI0, SGI227584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227585, SGI227586, SGI227586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50493[] = {SGI227587, SGI0, SGI227588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227589, SGI227590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50494[] = {SGI227591, SGI227592, SGI227593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227594, SGI0, SGI0, SGI0, SGI227595, SGI227595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227596, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50495[] = {SGI227597, SGI227598, SGI227599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227600, SGI227600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227601, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50496[] = {SGI227602, SGI227603, SGI227604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227605, SGI0, SGI0, SGI0, SGI227606, SGI227606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227607, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50497[] = {SGI227608, SGI227609, SGI227610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227611, SGI227611, SGI0, SGI0, SGI227612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50498[] = {SGI227613, SGI227614, SGI227615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227616, SGI227616, SGI0, SGI0, SGI0, SGI227617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50499[] = {SGI227618, SGI227619, SGI227620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227621, SGI227621, SGI0, SGI0, SGI0, SGI227622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50500[] = {SGI227623, SGI0, SGI227624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227625, SGI227626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50501[] = {SGI227627, SGI0, SGI227628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227629, SGI227630, SGI227630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50502[] = {SGI227631, SGI0, SGI227632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227635, SGI227636, SGI227636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50503[] = {SGI227633, SGI0, SGI227634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227637, SGI227638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50504[] = {SGI227639, SGI0, SGI227640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227641, SGI227642, SGI227642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50505[] = {SGI227643, SGI0, SGI227644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227645, SGI227646, SGI227646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50506[] = {SGI227647, SGI0, SGI227648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227649, SGI227651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50507[] = {SGI227650, SGI0, SGI227652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227653, SGI227654, SGI227654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50508[] = {SGI227655, SGI0, SGI227656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227657, SGI227658, SGI227658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50509[] = {SGI227659, SGI227660, SGI227661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227662, SGI0, SGI0, SGI0, SGI227663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50510[] = {SGI227664, SGI0, SGI227665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50511[] = {SGI227666, SGI0, SGI227667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227668, SGI227669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50512[] = {SGI227670, SGI227671, SGI227672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227675, SGI227675, SGI0, SGI0, SGI227676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50513[] = {SGI227673, SGI0, SGI227674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227677, SGI227678, SGI227678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50514[] = {SGI227679, SGI0, SGI227680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227681, SGI227682, SGI227682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50515[] = {SGI227683, SGI0, SGI227684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227685, SGI227686, SGI227686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227687, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50516[] = {SGI227688, SGI0, SGI227689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227690, SGI227691, SGI227691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50517[] = {SGI227692, SGI0, SGI227693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227694, SGI227695, SGI227695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50518[] = {SGI227696, SGI227697, SGI227698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227699, SGI0, SGI0, SGI0, SGI227700, SGI227700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50519[] = {SGI227702, SGI227703, SGI227704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227705, SGI227705, SGI0, SGI0, SGI0, SGI227706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50520[] = {SGI227707, SGI227708, SGI227709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227710, SGI0, SGI0, SGI0, SGI227711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50521[] = {SGI227712, SGI0, SGI227713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227717, SGI227718, SGI227718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50522[] = {SGI227714, SGI0, SGI227715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227720, SGI227721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50523[] = {SGI227716, SGI0, SGI227719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227725, SGI227726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50524[] = {SGI227722, SGI227723, SGI227724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227727, SGI0, SGI0, SGI0, SGI227728, SGI227728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50525[] = {SGI227729, SGI227730, SGI227731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227732, SGI227732, SGI0, SGI0, SGI0, SGI227733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50526[] = {SGI227734, SGI0, SGI227735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227736, SGI227737, SGI227737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50527[] = {SGI227738, SGI0, SGI227739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227740, SGI227741, SGI227741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50528[] = {SGI227742, SGI0, SGI227743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227746, SGI227747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50529[] = {SGI227744, SGI0, SGI227745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227748, SGI227749, SGI227749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50530[] = {SGI227750, SGI0, SGI227751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227752, SGI227753, SGI227753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50531[] = {SGI0, SGI0, SGI227754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227755, SGI227756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50532[] = {SGI227757, SGI0, SGI227758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227759, SGI227760, SGI227760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50533[] = {SGI227761, SGI227762, SGI227763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227764, SGI0, SGI0, SGI0, SGI227765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227766, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50534[] = {SGI227767, SGI227768, SGI227769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227770, SGI0, SGI0, SGI0, SGI227771, SGI227771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50535[] = {SGI227772, SGI0, SGI227773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227774, SGI227776, SGI227776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50536[] = {SGI227775, SGI227777, SGI227778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227779, SGI0, SGI0, SGI0, SGI227780, SGI227780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50537[] = {SGI227781, SGI0, SGI227782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227783, SGI227784, SGI227784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50538[] = {SGI227785, SGI0, SGI227786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227787, SGI227788, SGI227788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50539[] = {SGI227789, SGI227790, SGI227791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227793, SGI0, SGI0, SGI0, SGI227794, SGI227794, SGI0, SGI227797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50540[] = {SGI0, SGI0, SGI227792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227795, SGI227796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50541[] = {SGI227798, SGI227799, SGI227800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227801, SGI0, SGI0, SGI0, SGI227802, SGI227802, SGI0, SGI227803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50542[] = {SGI227804, SGI227805, SGI227806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227807, SGI0, SGI0, SGI0, SGI227808, SGI227808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50543[] = {SGI227809, SGI227810, SGI227811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227812, SGI227812, SGI0, SGI0, SGI0, SGI0, SGI227813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50544[] = {SGI227025, SGI0, SGI227027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227030, SGI227031, SGI227031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50545[] = {SGI227814, SGI0, SGI227815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227816, SGI227817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50546[] = {SGI227818, SGI0, SGI227819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227820, SGI227821, SGI227821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50547[] = {SGI227822, SGI0, SGI227823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227824, SGI227825, SGI227825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50548[] = {SGI227826, SGI0, SGI227827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227831, SGI227832, SGI227832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50549[] = {SGI227828, SGI227829, SGI227830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227833, SGI0, SGI0, SGI0, SGI227834, SGI227834, SGI0, SGI0, SGI0, SGI227835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50550[] = {SGI227836, SGI227837, SGI227838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227839, SGI0, SGI0, SGI0, SGI227840, SGI227840, SGI0, SGI0, SGI0, SGI227841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50551[] = {SGI227842, SGI227843, SGI227844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227846, SGI227846, SGI0, SGI0, SGI0, SGI0, SGI227848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50552[] = {SGI227845, SGI0, SGI227847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227849, SGI227850, SGI227850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50553[] = {SGI227851, SGI227852, SGI227853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227854, SGI0, SGI0, SGI0, SGI227856, SGI227856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227858, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50554[] = {SGI227855, SGI0, SGI227857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227859, SGI227860, SGI227860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50555[] = {SGI227861, SGI0, SGI227862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227863, SGI0, SGI0, SGI0, SGI227864, SGI227864, SGI0, SGI0, SGI0, SGI227865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50556[] = {SGI227866, SGI227867, SGI227868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227869, SGI0, SGI0, SGI0, SGI227870, SGI227870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50557[] = {SGI227871, SGI0, SGI227872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227873, SGI0, SGI0, SGI0, SGI227874, SGI227874, SGI0, SGI0, SGI0, SGI227875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50558[] = {SGI227876, SGI0, SGI227877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227878, SGI227879, SGI227879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50559[] = {SGI227880, SGI0, SGI227881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227882, SGI0, SGI0, SGI0, SGI227883, SGI227883, SGI0, SGI0, SGI0, SGI227884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50560[] = {SGI227885, SGI0, SGI227886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227887, SGI227888, SGI227888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50561[] = {SGI227889, SGI0, SGI227890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227891, SGI227892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50562[] = {SGI227893, SGI0, SGI227894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227895, SGI227896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50563[] = {SGI227897, SGI0, SGI227898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227899, SGI227900, SGI227900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50564[] = {SGI0, SGI0, SGI227901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227902, SGI227903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50565[] = {SGI0, SGI0, SGI227904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227905, SGI227906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50566[] = {SGI227907, SGI0, SGI227908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227909, SGI227910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50567[] = {SGI227911, SGI0, SGI227912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227913, SGI227914, SGI227914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50568[] = {SGI227915, SGI0, SGI227916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227919, SGI227920, SGI227920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50569[] = {SGI227917, SGI0, SGI227918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227921, SGI227922, SGI227922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50570[] = {SGI227923, SGI227924, SGI227925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227926, SGI0, SGI0, SGI0, SGI227927, SGI227927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227928, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50571[] = {SGI227929, SGI0, SGI227930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227931, SGI227932, SGI227932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227933, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50572[] = {SGI227934, SGI227935, SGI227936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227937, SGI0, SGI0, SGI0, SGI227938, SGI227938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50573[] = {SGI227939, SGI0, SGI227940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227941, SGI227942, SGI227942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50574[] = {SGI227943, SGI227944, SGI227946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227949, SGI0, SGI0, SGI0, SGI227950, SGI227950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227952, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50575[] = {SGI227945, SGI227947, SGI227948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227951, SGI227951, SGI0, SGI0, SGI0, SGI0, SGI227953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50576[] = {SGI227954, SGI0, SGI227955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227956, SGI227957, SGI227957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50577[] = {SGI227958, SGI227959, SGI227960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227961, SGI0, SGI0, SGI0, SGI227962, SGI227962, SGI0, SGI0, SGI0, SGI227963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50578[] = {SGI0, SGI227966, SGI0, SGI227968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50579[] = {SGI227964, SGI227965, SGI227967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227969, SGI0, SGI0, SGI0, SGI227970, SGI227970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50580[] = {SGI227971, SGI0, SGI227972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227973, SGI227974, SGI227974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50581[] = {SGI227976, SGI0, SGI227978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227980, SGI227983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50582[] = {SGI227975, SGI0, SGI227977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227979, SGI227981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50583[] = {SGI227982, SGI0, SGI227984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227985, SGI227986, SGI227986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50584[] = {SGI227987, SGI0, SGI227988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227989, SGI227990, SGI227990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50585[] = {SGI227991, SGI227992, SGI227993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227994, SGI0, SGI0, SGI0, SGI227995, SGI227995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50586[] = {SGI227996, SGI227997, SGI227998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI227999, SGI0, SGI0, SGI0, SGI228000, SGI228000, SGI0, SGI0, SGI228001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50587[] = {SGI228002, SGI0, SGI228003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228004, SGI228005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50588[] = {SGI228006, SGI0, SGI228007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50589[] = {SGI228008, SGI0, SGI228009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228010, SGI228011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50590[] = {SGI228012, SGI228013, SGI228014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228015, SGI0, SGI0, SGI0, SGI228016, SGI228016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228017, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50591[] = {SGI228018, SGI228019, SGI228020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228021, SGI228021, SGI0, SGI0, SGI0, SGI228022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50592[] = {SGI228023, SGI0, SGI228024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228025, SGI228026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50593[] = {SGI228027, SGI228028, SGI228029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228030, SGI0, SGI0, SGI0, SGI228031, SGI228031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228032, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50594[] = {SGI228033, SGI0, SGI228034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228036, SGI228038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50595[] = {SGI228035, SGI0, SGI228037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228039, SGI228040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50596[] = {SGI228041, SGI0, SGI228042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228043, SGI228044, SGI228044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50597[] = {SGI228045, SGI0, SGI228046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228047, SGI228048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50598[] = {SGI228049, SGI0, SGI228050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228051, SGI0, SGI0, SGI0, SGI228052, SGI228052, SGI0, SGI0, SGI0, SGI228053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50599[] = {SGI228054, SGI228055, SGI228056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228057, SGI0, SGI0, SGI0, SGI228058, SGI228058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228059, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50600[] = {SGI228060, SGI228061, SGI228062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228063, SGI0, SGI0, SGI0, SGI228064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50601[] = {SGI228065, SGI0, SGI228066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228067, SGI228068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50602[] = {SGI228069, SGI0, SGI228070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228071, SGI228072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50603[] = {SGI228073, SGI0, SGI228074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228077, SGI228078, SGI228078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50604[] = {SGI228075, SGI0, SGI228076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228079, SGI228080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50605[] = {SGI228081, SGI228082, SGI228083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228084, SGI0, SGI0, SGI0, SGI228085, SGI228085, SGI0, SGI228086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50606[] = {SGI228087, SGI228088, SGI228089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228090, SGI228090, SGI0, SGI0, SGI0, SGI228091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50607[] = {SGI228092, SGI0, SGI228093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228094, SGI228095, SGI228095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50608[] = {SGI228096, SGI228097, SGI228098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228099, SGI228099, SGI0, SGI0, SGI0, SGI228100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50609[] = {SGI228101, SGI0, SGI228102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228103, SGI228104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50610[] = {SGI228105, SGI228106, SGI228107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228108, SGI228108, SGI0, SGI0, SGI0, SGI228109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50611[] = {SGI228110, SGI0, SGI228111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228112, SGI228113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50612[] = {SGI228114, SGI0, SGI228115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228116, SGI228117, SGI228117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50613[] = {SGI228118, SGI0, SGI228119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228120, SGI228121, SGI228121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50614[] = {SGI228122, SGI0, SGI228123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228124, SGI228125, SGI228125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50615[] = {SGI228126, SGI0, SGI228127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228128, SGI228129, SGI228129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50616[] = {SGI228130, SGI0, SGI228131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228132, SGI228133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50617[] = {SGI228134, SGI228135, SGI228136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228137, SGI228137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228138, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50618[] = {SGI228139, SGI228140, SGI228141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228143, SGI0, SGI0, SGI0, SGI228145, SGI228145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50619[] = {SGI228142, SGI228144, SGI228146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228147, SGI0, SGI0, SGI0, SGI228148, SGI228148, SGI228149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50620[] = {SGI228150, SGI228151, SGI228152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228153, SGI228153, SGI0, SGI0, SGI0, SGI228154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50621[] = {SGI228155, SGI0, SGI228156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228157, SGI228159, SGI228159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50622[] = {SGI228158, SGI228160, SGI228161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228162, SGI228162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228163, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50623[] = {SGI228164, SGI0, SGI228165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228168, SGI228169, SGI228169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50624[] = {SGI228166, SGI0, SGI228167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228170, SGI228171, SGI228171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50625[] = {SGI228172, SGI0, SGI228173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228176, SGI228177, SGI228177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50626[] = {SGI228174, SGI0, SGI228175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228178, SGI228179, SGI228179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50627[] = {SGI228180, SGI0, SGI228181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228182, SGI228183, SGI228183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50628[] = {SGI228184, SGI0, SGI228185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228186, SGI228187, SGI228187, SGI0, SGI0, SGI0, SGI0, SGI228188, SGI0, NULL};
const Segment * const SG50629[] = {SGI228189, SGI228190, SGI228191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228195, SGI228195, SGI0, SGI0, SGI228196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50630[] = {SGI228197, SGI0, SGI228198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228199, SGI228200, SGI228200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50631[] = {SGI228201, SGI228202, SGI228203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228204, SGI0, SGI0, SGI0, SGI228205, SGI228205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50632[] = {SGI228206, SGI0, SGI228207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228208, SGI228209, SGI228209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50633[] = {SGI228210, SGI228211, SGI228212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228213, SGI0, SGI0, SGI0, SGI228214, SGI228214, SGI0, SGI228215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50634[] = {SGI228216, SGI228217, SGI228218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228219, SGI0, SGI0, SGI0, SGI228220, SGI228220, SGI0, SGI228222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50635[] = {SGI228221, SGI228223, SGI228224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228225, SGI0, SGI0, SGI0, SGI228226, SGI228226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50636[] = {SGI228227, SGI0, SGI228228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228229, SGI228230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50637[] = {SGI228231, SGI228232, SGI228233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228234, SGI0, SGI0, SGI0, SGI228235, SGI228235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228236, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50638[] = {SGI228237, SGI228238, SGI228239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228240, SGI0, SGI0, SGI0, SGI228241, SGI228241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228243, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50639[] = {SGI228242, SGI0, SGI228244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228245, SGI228246, SGI228246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50640[] = {SGI228247, SGI0, SGI228248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228249, SGI228250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50641[] = {SGI228251, SGI0, SGI228252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228253, SGI228254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50642[] = {SGI228255, SGI0, SGI228256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228257, SGI228258, SGI228258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50643[] = {SGI228259, SGI0, SGI228260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228261, SGI228262, SGI228262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50644[] = {SGI228263, SGI228264, SGI228265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228266, SGI228266, SGI0, SGI0, SGI0, SGI0, SGI228267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50645[] = {SGI228268, SGI0, SGI228269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228270, SGI228271, SGI228271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50646[] = {SGI228272, SGI228273, SGI228274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228275, SGI0, SGI0, SGI0, SGI228276, SGI228276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50647[] = {SGI228277, SGI0, SGI228278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228279, SGI0, SGI0, SGI0, SGI228280, SGI228280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228281, NULL};
const Segment * const SG50648[] = {SGI228282, SGI0, SGI228283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228285, SGI0, SGI0, SGI0, SGI228287, SGI228287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228288, NULL};
const Segment * const SG50649[] = {SGI228284, SGI0, SGI228286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228289, SGI228290, SGI228290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228291, SGI0, SGI0, NULL};
const Segment * const SG50650[] = {SGI228292, SGI0, SGI228293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228294, SGI228296, SGI228296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50651[] = {SGI228295, SGI0, SGI228297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228298, SGI228299, SGI228299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50652[] = {SGI228300, SGI0, SGI228301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228302, SGI228303, SGI228303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50653[] = {SGI228304, SGI0, SGI228305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228306, SGI228307, SGI228307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50654[] = {SGI228308, SGI228309, SGI228310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228311, SGI0, SGI0, SGI0, SGI228312, SGI228312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50655[] = {SGI228313, SGI228314, SGI228315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228316, SGI228316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228317, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50656[] = {SGI228318, SGI0, SGI228319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228320, SGI228322, SGI228322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50657[] = {SGI228321, SGI0, SGI228323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228324, SGI228325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50658[] = {SGI228326, SGI0, SGI228327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228328, SGI228329, SGI228329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50659[] = {SGI228330, SGI228331, SGI228332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228333, SGI0, SGI0, SGI0, SGI228334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50660[] = {SGI228335, SGI0, SGI228336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228337, SGI228338, SGI228338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50661[] = {SGI228339, SGI0, SGI228340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228341, SGI228342, SGI228342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50662[] = {SGI228343, SGI228344, SGI228345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228346, SGI0, SGI0, SGI0, SGI228347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50663[] = {SGI228348, SGI228349, SGI228350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228351, SGI0, SGI0, SGI0, SGI228352, SGI0, SGI0, SGI228354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50664[] = {SGI228353, SGI0, SGI228355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228356, SGI228357, SGI228357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50665[] = {SGI228358, SGI228359, SGI228360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228361, SGI0, SGI0, SGI0, SGI228362, SGI0, SGI0, SGI228363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50666[] = {SGI228364, SGI228365, SGI228366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228367, SGI228367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228368, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50667[] = {SGI228369, SGI0, SGI228370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228371, SGI228372, SGI228372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50668[] = {SGI228373, SGI0, SGI228374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228375, SGI228376, SGI228376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50669[] = {SGI228377, SGI0, SGI228378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228379, SGI228380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50670[] = {SGI228381, SGI0, SGI228382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228383, SGI228384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50671[] = {SGI228385, SGI0, SGI228386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228387, SGI228388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50672[] = {SGI228389, SGI228390, SGI228391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228392, SGI0, SGI0, SGI0, SGI228393, SGI228393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228394, NULL};
const Segment * const SG50673[] = {SGI228395, SGI228396, SGI228397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228398, SGI0, SGI0, SGI0, SGI228399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50674[] = {SGI228400, SGI0, SGI228401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228402, SGI228403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50675[] = {SGI228404, SGI228405, SGI228406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228407, SGI0, SGI0, SGI0, SGI228408, SGI228408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50676[] = {SGI228409, SGI0, SGI228410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228411, SGI228412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50677[] = {SGI228413, SGI0, SGI228414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228415, SGI228416, SGI228416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50678[] = {SGI228417, SGI0, SGI228418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228419, SGI228420, SGI228420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228421, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50679[] = {SGI228422, SGI228423, SGI228424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228425, SGI0, SGI0, SGI0, SGI228426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50680[] = {SGI228427, SGI228428, SGI228429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228430, SGI228430, SGI0, SGI0, SGI0, SGI0, SGI228431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50681[] = {SGI228432, SGI228433, SGI228434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228435, SGI228435, SGI0, SGI0, SGI0, SGI0, SGI228436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50682[] = {SGI228437, SGI228438, SGI228439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228440, SGI0, SGI0, SGI0, SGI228441, SGI228441, SGI0, SGI0, SGI0, SGI228442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50683[] = {SGI228443, SGI228444, SGI228445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228446, SGI0, SGI0, SGI0, SGI228447, SGI228447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228448, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50684[] = {SGI228449, SGI228450, SGI228451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228452, SGI0, SGI0, SGI0, SGI228453, SGI228453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228454, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50685[] = {SGI228455, SGI228456, SGI228457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228458, SGI0, SGI0, SGI0, SGI228459, SGI228459, SGI228461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50686[] = {SGI0, SGI0, SGI228460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228462, SGI228462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50687[] = {SGI228463, SGI228464, SGI228465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228466, SGI0, SGI0, SGI0, SGI228467, SGI228467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50688[] = {SGI228468, SGI228469, SGI228470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228471, SGI228471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228472, NULL};
const Segment * const SG50689[] = {SGI228473, SGI228474, SGI228475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228476, SGI228476, SGI0, SGI0, SGI228477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50690[] = {SGI228478, SGI0, SGI228479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228480, SGI228481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50691[] = {SGI228482, SGI228483, SGI228484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228485, SGI0, SGI0, SGI0, SGI228486, SGI228486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228487, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50692[] = {SGI228488, SGI0, SGI228489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228491, SGI228492, SGI228492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50693[] = {SGI228490, SGI0, SGI228493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228494, SGI228495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50694[] = {SGI228496, SGI0, SGI228497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228498, SGI228499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50695[] = {SGI228500, SGI228501, SGI228502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228503, SGI0, SGI0, SGI0, SGI228504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50696[] = {SGI228505, SGI0, SGI228506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228507, SGI0, SGI0, SGI0, SGI228508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50697[] = {SGI228509, SGI228510, SGI228511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228512, SGI0, SGI0, SGI0, SGI228513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50698[] = {SGI228514, SGI228515, SGI228516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228517, SGI0, SGI0, SGI0, SGI228518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50699[] = {SGI228519, SGI0, SGI228520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50700[] = {SGI228521, SGI228522, SGI228523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228524, SGI0, SGI0, SGI0, SGI228525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50701[] = {SGI228526, SGI228527, SGI228528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228529, SGI0, SGI0, SGI0, SGI228530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50702[] = {SGI228192, SGI228193, SGI228194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228531, SGI228531, SGI0, SGI0, SGI0, SGI228532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50703[] = {SGI228533, SGI0, SGI228534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228535, SGI228536, SGI228536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228537, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50704[] = {SGI228538, SGI0, SGI228539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228540, SGI228541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50705[] = {SGI228542, SGI0, SGI228543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228544, SGI228545, SGI228545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50706[] = {SGI228546, SGI228547, SGI228548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228549, SGI228549, SGI0, SGI0, SGI0, SGI228550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50707[] = {SGI228551, SGI0, SGI228552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228553, SGI228554, SGI228554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50708[] = {SGI228555, SGI0, SGI228556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228557, SGI228558, SGI228558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228559, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50709[] = {SGI228560, SGI0, SGI228561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228562, SGI228563, SGI228563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50710[] = {SGI228564, SGI0, SGI228565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228566, SGI228567, SGI228567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50711[] = {SGI228568, SGI0, SGI228569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228570, SGI228571, SGI228571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50712[] = {SGI228572, SGI0, SGI228573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228574, SGI228575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50713[] = {SGI228576, SGI0, SGI228577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228579, SGI228580, SGI228580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50714[] = {SGI228578, SGI228581, SGI228582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228583, SGI0, SGI0, SGI0, SGI228584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50715[] = {SGI228585, SGI0, SGI228586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50716[] = {SGI228587, SGI0, SGI228588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228589, SGI228590, SGI228590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50717[] = {SGI228591, SGI228592, SGI228593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228594, SGI0, SGI0, SGI0, SGI228595, SGI228595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50718[] = {SGI228596, SGI0, SGI228597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228598, SGI228599, SGI228599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50719[] = {SGI228600, SGI228601, SGI228602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228603, SGI0, SGI0, SGI0, SGI228604, SGI228604, SGI228605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50720[] = {SGI228606, SGI228607, SGI228608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228609, SGI0, SGI0, SGI0, SGI228610, SGI228610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228611, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50721[] = {SGI228612, SGI228613, SGI228614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228615, SGI0, SGI0, SGI0, SGI228616, SGI228616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228617, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50722[] = {SGI228618, SGI228619, SGI228620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228621, SGI0, SGI228622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50723[] = {SGI228623, SGI0, SGI228624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228625, SGI228626, SGI228626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50724[] = {SGI228627, SGI228628, SGI228629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228630, SGI0, SGI228631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50725[] = {SGI228632, SGI228633, SGI228634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228635, SGI0, SGI0, SGI0, SGI228636, SGI228636, SGI0, SGI0, SGI228637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50726[] = {SGI228638, SGI228639, SGI228640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228641, SGI0, SGI0, SGI0, SGI228642, SGI228642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228644, SGI0, NULL};
const Segment * const SG50727[] = {SGI228645, SGI228646, SGI228647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228648, SGI0, SGI0, SGI228649, SGI228650, SGI228650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50728[] = {SGI228651, SGI0, SGI228652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228653, SGI228654, SGI228654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50729[] = {SGI228655, SGI228656, SGI228657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228658, SGI0, SGI0, SGI0, SGI228659, SGI228659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228660, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50730[] = {SGI228661, SGI0, SGI228662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228663, SGI228664, SGI228664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50731[] = {SGI228665, SGI228666, SGI228667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228668, SGI228668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228669, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50732[] = {SGI228670, SGI0, SGI228671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228672, SGI228673, SGI228673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50733[] = {SGI228674, SGI0, SGI228675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228676, SGI228677, SGI228677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50734[] = {SGI228678, SGI0, SGI228679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228680, SGI228681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50735[] = {SGI228682, SGI0, SGI228683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228684, SGI228685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50736[] = {SGI228686, SGI0, SGI228687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228688, SGI228689, SGI228689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50737[] = {SGI228690, SGI0, SGI228691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228692, SGI228693, SGI228693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50738[] = {SGI228694, SGI228695, SGI228696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228697, SGI0, SGI0, SGI0, SGI228698, SGI228698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50739[] = {SGI228699, SGI228700, SGI228701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228702, SGI0, SGI0, SGI0, SGI228703, SGI228703, SGI0, SGI228707, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50740[] = {SGI228704, SGI0, SGI228705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50741[] = {SGI228706, SGI228708, SGI228709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228710, SGI0, SGI0, SGI0, SGI228711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50742[] = {SGI228713, SGI228714, SGI228715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228716, SGI0, SGI0, SGI0, SGI228717, SGI228717, SGI0, SGI228718, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50743[] = {SGI228719, SGI228720, SGI228721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228724, SGI0, SGI0, SGI0, SGI228725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50744[] = {SGI228722, SGI0, SGI228723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228726, SGI228727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50745[] = {SGI228730, SGI0, SGI228732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228735, SGI228736, SGI228736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50746[] = {SGI228729, SGI228731, SGI228733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228734, SGI0, SGI0, SGI0, SGI228737, SGI228737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50747[] = {SGI228739, SGI228740, SGI228741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228742, SGI0, SGI0, SGI0, SGI228745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50748[] = {SGI228743, SGI0, SGI228744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228747, SGI228748, SGI228748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50749[] = {SGI228749, SGI228750, SGI228751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228752, SGI0, SGI0, SGI0, SGI228753, SGI228753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228754, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50750[] = {SGI228755, SGI0, SGI228756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228757, SGI228758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50751[] = {SGI228759, SGI0, SGI228760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228761, SGI228762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228765, SGI0, SGI0, NULL};
const Segment * const SG50752[] = {SGI228763, SGI228764, SGI228766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228767, SGI0, SGI0, SGI0, SGI228768, SGI228768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50753[] = {SGI228769, SGI228770, SGI228771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228772, SGI228772, SGI0, SGI0, SGI0, SGI228773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50754[] = {SGI228774, SGI0, SGI228775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228776, SGI228777, SGI228777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50755[] = {SGI228778, SGI228779, SGI228780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228784, SGI0, SGI0, SGI0, SGI228786, SGI228786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228788, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50756[] = {SGI228782, SGI0, SGI228785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228789, SGI228791, SGI228791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50757[] = {SGI228781, SGI0, SGI228783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228787, SGI228790, SGI228790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50758[] = {SGI228792, SGI228793, SGI228794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228797, SGI0, SGI0, SGI0, SGI228799, SGI228799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228801, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50759[] = {SGI228795, SGI228796, SGI228798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228800, SGI0, SGI0, SGI0, SGI228802, SGI228802, SGI0, SGI0, SGI228803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50760[] = {SGI228804, SGI0, SGI228805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228806, SGI228807, SGI228807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50761[] = {SGI228808, SGI0, SGI228810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228812, SGI228813, SGI228813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50762[] = {SGI228809, SGI0, SGI228811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228814, SGI228815, SGI228815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50763[] = {SGI228816, SGI0, SGI228818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228820, SGI228822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50764[] = {SGI228817, SGI0, SGI228819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228821, SGI228823, SGI228823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50765[] = {SGI228824, SGI0, SGI228825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228826, SGI228827, SGI228827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50766[] = {SGI228828, SGI0, SGI228829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228832, SGI228833, SGI228833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50767[] = {SGI228830, SGI0, SGI228831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228835, SGI228836, SGI228836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50768[] = {SGI228834, SGI0, SGI228837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228838, SGI228839, SGI228839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50769[] = {SGI228840, SGI0, SGI228841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228842, SGI0, SGI0, SGI0, SGI228843, SGI228843, SGI0, SGI0, SGI228844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50770[] = {SGI228845, SGI228846, SGI228847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228848, SGI0, SGI0, SGI0, SGI228849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50771[] = {SGI228850, SGI0, SGI228851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228852, SGI228854, SGI228854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50772[] = {SGI228853, SGI0, SGI228855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228856, SGI228858, SGI228858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50773[] = {SGI228857, SGI228859, SGI228860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228861, SGI228861, SGI0, SGI0, SGI0, SGI0, SGI228862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50774[] = {SGI228863, SGI0, SGI228864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228865, SGI228867, SGI228867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50775[] = {SGI228866, SGI0, SGI228868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228869, SGI228870, SGI228870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50776[] = {SGI228871, SGI0, SGI228872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228873, SGI228874, SGI228874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50777[] = {SGI228875, SGI0, SGI228876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228877, SGI228878, SGI228878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50778[] = {SGI228879, SGI0, SGI228880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228881, SGI0, SGI0, SGI0, SGI228883, SGI228883, SGI0, SGI0, SGI0, SGI228885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50779[] = {SGI228882, SGI0, SGI228884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228886, SGI228887, SGI228887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50780[] = {SGI228888, SGI0, SGI228889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228892, SGI228893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50781[] = {SGI228890, SGI0, SGI228891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228894, SGI228895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50782[] = {SGI228896, SGI0, SGI228897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228898, SGI228899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50783[] = {SGI228900, SGI0, SGI228901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228902, SGI228903, SGI228903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50784[] = {SGI228904, SGI0, SGI228905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228907, SGI228909, SGI228909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50785[] = {SGI228906, SGI0, SGI228908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228910, SGI228911, SGI228911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50786[] = {SGI228912, SGI228913, SGI228914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228918, SGI228918, SGI0, SGI0, SGI0, SGI228919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50787[] = {SGI228915, SGI228916, SGI228917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228920, SGI0, SGI0, SGI0, SGI228921, SGI228921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228922, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50788[] = {SGI228923, SGI228924, SGI228925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228926, SGI228926, SGI0, SGI0, SGI0, SGI228927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50789[] = {SGI228928, SGI228929, SGI228930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228931, SGI0, SGI0, SGI0, SGI228932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50790[] = {SGI228933, SGI0, SGI228934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228935, SGI228936, SGI228936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228937, SGI0, SGI0, NULL};
const Segment * const SG50791[] = {SGI228938, SGI228939, SGI228940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228941, SGI0, SGI0, SGI0, SGI228942, SGI228942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228943, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50792[] = {SGI228944, SGI228945, SGI228946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228947, SGI0, SGI0, SGI0, SGI228948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228949, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50793[] = {SGI228950, SGI0, SGI228951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228952, SGI228953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50794[] = {SGI228954, SGI0, SGI228955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228956, SGI228957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50795[] = {SGI228958, SGI0, SGI228959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228960, SGI228961, SGI228961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50796[] = {SGI228962, SGI0, SGI228963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228964, SGI228965, SGI228965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50797[] = {SGI228966, SGI0, SGI228967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228968, SGI228969, SGI228969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50798[] = {SGI228970, SGI0, SGI228971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228975, SGI228976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50799[] = {SGI228972, SGI228973, SGI228974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228977, SGI228977, SGI0, SGI0, SGI228978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50800[] = {SGI228979, SGI228980, SGI228981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228982, SGI0, SGI0, SGI0, SGI228983, SGI228983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228984, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50801[] = {SGI228985, SGI228986, SGI228987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228990, SGI228990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228991, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50802[] = {SGI228988, SGI0, SGI228989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228992, SGI228993, SGI228993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50803[] = {SGI228994, SGI0, SGI228995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI228996, SGI228997, SGI228997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50804[] = {SGI228998, SGI228999, SGI229000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229001, SGI0, SGI0, SGI0, SGI229002, SGI229002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229003, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50805[] = {SGI229004, SGI0, SGI229005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229006, SGI229007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50806[] = {SGI229008, SGI0, SGI229009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229010, SGI229011, SGI229011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50807[] = {SGI229012, SGI229013, SGI229014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229015, SGI0, SGI0, SGI0, SGI229016, SGI229016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50808[] = {SGI229018, SGI229019, SGI229020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229021, SGI0, SGI0, SGI0, SGI229023, SGI229023, SGI0, SGI229025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50809[] = {SGI229022, SGI0, SGI229024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229026, SGI229027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50810[] = {SGI229028, SGI229029, SGI229030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229032, SGI0, SGI0, SGI0, SGI229034, SGI229034, SGI0, SGI229035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50811[] = {SGI229031, SGI0, SGI229033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229036, SGI229037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50812[] = {SGI229038, SGI229039, SGI229040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229041, SGI229041, SGI0, SGI0, SGI0, SGI229042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50813[] = {SGI229043, SGI229044, SGI229045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229046, SGI229046, SGI0, SGI0, SGI0, SGI229047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50814[] = {SGI229048, SGI0, SGI229049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229050, SGI229051, SGI229051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50815[] = {SGI229052, SGI0, SGI229053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229054, SGI229055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50816[] = {SGI229056, SGI0, SGI229057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229058, SGI229059, SGI229059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50817[] = {SGI229060, SGI0, SGI229061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229062, SGI229064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50818[] = {SGI229063, SGI0, SGI229065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229066, SGI229067, SGI229067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50819[] = {SGI229068, SGI0, SGI229069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229070, SGI229071, SGI229071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50820[] = {SGI229072, SGI0, SGI229073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229074, SGI229075, SGI229075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50821[] = {SGI229076, SGI0, SGI229077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229078, SGI229079, SGI229079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50822[] = {SGI229080, SGI0, SGI229081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229082, SGI229083, SGI229083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50823[] = {SGI229084, SGI0, SGI229085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229086, SGI229087, SGI229087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50824[] = {SGI229088, SGI0, SGI229089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229090, SGI229091, SGI229091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50825[] = {SGI229092, SGI0, SGI229093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229094, SGI229095, SGI229095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50826[] = {SGI229096, SGI229097, SGI229098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229099, SGI229099, SGI0, SGI0, SGI0, SGI0, SGI229100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50827[] = {SGI229101, SGI0, SGI229102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229103, SGI229104, SGI229104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50828[] = {SGI229105, SGI0, SGI229106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229107, SGI229108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50829[] = {SGI229109, SGI229110, SGI229111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229112, SGI229112, SGI0, SGI0, SGI0, SGI0, SGI229113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50830[] = {SGI229114, SGI0, SGI229115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229116, SGI229117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50831[] = {SGI229118, SGI229119, SGI229120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229121, SGI0, SGI0, SGI0, SGI229122, SGI229122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50832[] = {SGI229123, SGI0, SGI229124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229126, SGI229127, SGI229127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50833[] = {SGI229128, SGI0, SGI229129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229130, SGI229131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50834[] = {SGI229132, SGI229133, SGI229134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229135, SGI229135, SGI0, SGI0, SGI0, SGI229136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50835[] = {SGI229137, SGI0, SGI229138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229139, SGI229140, SGI229140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50836[] = {SGI229141, SGI229142, SGI229143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229144, SGI0, SGI0, SGI0, SGI229145, SGI229146, SGI0, SGI0, SGI0, SGI229147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50837[] = {SGI229148, SGI0, SGI229149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229150, SGI229151, SGI229151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50838[] = {SGI229152, SGI0, SGI229153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229154, SGI229155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50839[] = {SGI229156, SGI229157, SGI229158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229159, SGI0, SGI0, SGI0, SGI229160, SGI229161, SGI0, SGI0, SGI0, SGI229162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50840[] = {SGI229163, SGI0, SGI229164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229165, SGI229166, SGI229166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50841[] = {SGI229167, SGI0, SGI229168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229169, SGI229170, SGI229170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50842[] = {SGI229171, SGI0, SGI229172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229173, SGI229174, SGI229174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50843[] = {SGI229175, SGI0, SGI229176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229177, SGI229178, SGI229178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50844[] = {SGI229179, SGI0, SGI229180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229181, SGI229182, SGI229182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50845[] = {SGI229183, SGI0, SGI229184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229185, SGI229186, SGI229186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50846[] = {SGI229187, SGI229188, SGI229189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229190, SGI229190, SGI0, SGI0, SGI0, SGI229191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50847[] = {SGI229192, SGI0, SGI229193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229194, SGI229195, SGI229195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50848[] = {SGI229196, SGI0, SGI229197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229198, SGI229199, SGI229199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50849[] = {SGI229200, SGI0, SGI229201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229202, SGI0, SGI0, SGI0, SGI229203, SGI229203, SGI0, SGI0, SGI0, SGI0, SGI229204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50850[] = {SGI229205, SGI0, SGI229206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229207, SGI229208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50851[] = {SGI229209, SGI0, SGI229210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229213, SGI229215, SGI229215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50852[] = {SGI229211, SGI229212, SGI229214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229216, SGI0, SGI0, SGI0, SGI229217, SGI229217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229218, SGI0, SGI0, NULL};
const Segment * const SG50853[] = {SGI229219, SGI229220, SGI229221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229223, SGI0, SGI0, SGI0, SGI229226, SGI229226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50854[] = {SGI229222, SGI229224, SGI229225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229227, SGI0, SGI0, SGI0, SGI229228, SGI229228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229229, SGI0, SGI0, NULL};
const Segment * const SG50855[] = {SGI229230, SGI229231, SGI229232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229233, SGI0, SGI0, SGI0, SGI229234, SGI229234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229235, SGI0, SGI0, NULL};
const Segment * const SG50856[] = {SGI229236, SGI0, SGI229237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229240, SGI229242, SGI229242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50857[] = {SGI229238, SGI229239, SGI229241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229243, SGI0, SGI0, SGI0, SGI229244, SGI229244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229245, SGI0, SGI0, NULL};
const Segment * const SG50858[] = {SGI229246, SGI0, SGI229247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229249, SGI229250, SGI229250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50859[] = {SGI229248, SGI0, SGI229251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229252, SGI229253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50860[] = {SGI229254, SGI229255, SGI229256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229257, SGI0, SGI0, SGI0, SGI229258, SGI0, SGI229259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50861[] = {SGI229260, SGI0, SGI229261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229262, SGI229263, SGI229263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50862[] = {SGI229264, SGI0, SGI229265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229266, SGI229267, SGI229267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50863[] = {SGI229268, SGI0, SGI229269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229270, SGI229271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50864[] = {SGI229272, SGI0, SGI229273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229274, SGI229275, SGI229275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50865[] = {SGI229276, SGI0, SGI229277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229278, SGI229279, SGI229279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229280, SGI0, SGI0, NULL};
const Segment * const SG50866[] = {SGI229281, SGI229282, SGI229283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229284, SGI0, SGI0, SGI0, SGI229285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50867[] = {SGI229286, SGI0, SGI229287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229288, SGI229289, SGI229289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229290, SGI0, SGI0, NULL};
const Segment * const SG50868[] = {SGI229291, SGI0, SGI229292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229293, SGI229294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50869[] = {SGI229295, SGI0, SGI229296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229297, SGI229298, SGI229298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50870[] = {SGI229299, SGI0, SGI229300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229301, SGI229302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50871[] = {SGI229303, SGI0, SGI229304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229305, SGI229306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50872[] = {SGI229307, SGI0, SGI229308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229309, SGI229310, SGI229310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50873[] = {SGI229311, SGI0, SGI229312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229316, SGI229317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50874[] = {SGI229313, SGI229314, SGI229315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229318, SGI0, SGI0, SGI0, SGI229319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50875[] = {SGI229320, SGI0, SGI229321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229322, SGI229323, SGI229323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50876[] = {SGI229324, SGI229325, SGI229326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229327, SGI0, SGI0, SGI0, SGI229328, SGI229328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50877[] = {SGI229329, SGI229330, SGI229331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229332, SGI0, SGI0, SGI0, SGI229333, SGI229333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50878[] = {SGI229334, SGI229335, SGI229336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229337, SGI0, SGI0, SGI0, SGI229339, SGI229339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50879[] = {SGI229338, SGI229340, SGI229341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229342, SGI0, SGI0, SGI0, SGI229343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50880[] = {SGI229344, SGI0, SGI229345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229346, SGI229347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50881[] = {SGI229348, SGI0, SGI229349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229350, SGI229351, SGI229351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50882[] = {SGI229352, SGI0, SGI229354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229359, SGI229361, SGI229361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50883[] = {SGI229353, SGI0, SGI229355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229358, SGI229360, SGI229360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50884[] = {SGI229356, SGI0, SGI229357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229362, SGI229363, SGI229363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50885[] = {SGI229364, SGI0, SGI229365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229366, SGI0, SGI0, SGI0, SGI229367, SGI229367, SGI0, SGI0, SGI0, SGI229368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50886[] = {SGI229369, SGI229370, SGI229371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229372, SGI0, SGI0, SGI0, SGI229373, SGI229373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229374, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50887[] = {SGI0, SGI229375, SGI0, SGI229376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50888[] = {SGI229377, SGI0, SGI229378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229379, SGI229380, SGI229380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50889[] = {SGI229381, SGI0, SGI229382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229383, SGI229384, SGI229384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50890[] = {SGI229385, SGI0, SGI229386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229387, SGI229388, SGI229388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50891[] = {SGI229389, SGI0, SGI229390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229391, SGI0, SGI0, SGI0, SGI229392, SGI229392, SGI0, SGI0, SGI0, SGI229393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50892[] = {SGI229394, SGI0, SGI229395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229396, SGI0, SGI0, SGI0, SGI229397, SGI229397, SGI0, SGI0, SGI0, SGI229398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50893[] = {SGI229399, SGI0, SGI229400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229404, SGI229405, SGI229405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50894[] = {SGI229401, SGI229402, SGI229403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229406, SGI0, SGI0, SGI0, SGI229407, SGI229407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50895[] = {SGI229408, SGI0, SGI229409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229410, SGI229411, SGI229411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50896[] = {SGI229412, SGI0, SGI229413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229414, SGI0, SGI0, SGI0, SGI229415, SGI229415, SGI0, SGI0, SGI0, SGI0, SGI229416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50897[] = {SGI229417, SGI0, SGI229418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229419, SGI229420, SGI229420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50898[] = {SGI229422, SGI229424, SGI229425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229427, SGI0, SGI0, SGI0, SGI229429, SGI229429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50899[] = {SGI229421, SGI0, SGI229423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229426, SGI229428, SGI229428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50900[] = {SGI229430, SGI0, SGI229431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229432, SGI229433, SGI229433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50901[] = {SGI229434, SGI0, SGI229435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229436, SGI229437, SGI229437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229438, SGI0, SGI0, NULL};
const Segment * const SG50902[] = {SGI229439, SGI229440, SGI229441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229442, SGI229442, SGI0, SGI0, SGI0, SGI229443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50903[] = {SGI229444, SGI0, SGI229445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229446, SGI229447, SGI229447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50904[] = {SGI229448, SGI0, SGI229449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229450, SGI229451, SGI229451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50905[] = {SGI229452, SGI0, SGI229453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229454, SGI229455, SGI229455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50906[] = {SGI229456, SGI0, SGI229457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229458, SGI229459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50907[] = {SGI229460, SGI0, SGI229461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229462, SGI229463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50908[] = {SGI229464, SGI0, SGI229465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229468, SGI0, SGI0, SGI0, SGI229469, SGI229469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229472, NULL};
const Segment * const SG50909[] = {SGI229466, SGI0, SGI229467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229470, SGI229471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50910[] = {SGI229473, SGI0, SGI229474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229475, SGI229476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50911[] = {SGI229477, SGI229478, SGI229479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229480, SGI0, SGI0, SGI0, SGI229481, SGI229481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229482, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50912[] = {SGI229483, SGI229484, SGI229485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229486, SGI229486, SGI0, SGI0, SGI0, SGI229487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50913[] = {SGI229488, SGI0, SGI229489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229490, SGI229491, SGI229491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50914[] = {SGI229492, SGI0, SGI229493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229494, SGI229495, SGI229495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50915[] = {SGI229496, SGI0, SGI229497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229498, SGI229499, SGI229499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50916[] = {SGI229500, SGI0, SGI229501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229502, SGI0, SGI0, SGI0, SGI229503, SGI229503, SGI0, SGI0, SGI0, SGI229504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50917[] = {SGI229505, SGI229506, SGI229507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229508, SGI229508, SGI0, SGI0, SGI0, SGI229509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50918[] = {SGI229510, SGI0, SGI229511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229512, SGI229513, SGI229513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50919[] = {SGI229514, SGI0, SGI229515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229516, SGI0, SGI0, SGI0, SGI229517, SGI229517, SGI0, SGI0, SGI0, SGI229518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50920[] = {SGI229519, SGI0, SGI229520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229521, SGI229522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50921[] = {SGI229523, SGI0, SGI229524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229525, SGI229526, SGI229526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50922[] = {SGI229527, SGI0, SGI229528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229529, SGI229530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50923[] = {SGI229531, SGI0, SGI229532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229533, SGI0, SGI0, SGI0, SGI229534, SGI229534, SGI0, SGI0, SGI0, SGI229535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50924[] = {SGI0, SGI0, SGI229536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229537, SGI229538, SGI229538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50925[] = {SGI229539, SGI229540, SGI229541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229542, SGI0, SGI0, SGI0, SGI229543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229544, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50926[] = {SGI229545, SGI229546, SGI229547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229548, SGI229548, SGI0, SGI0, SGI0, SGI229549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50927[] = {SGI229550, SGI0, SGI229551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229552, SGI229553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50928[] = {SGI229554, SGI229555, SGI229556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229557, SGI0, SGI0, SGI0, SGI229558, SGI229558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50929[] = {SGI229559, SGI0, SGI229560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229561, SGI229562, SGI229562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50930[] = {SGI229563, SGI229564, SGI229565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229566, SGI0, SGI0, SGI0, SGI229567, SGI229567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50931[] = {SGI229568, SGI229569, SGI229570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229571, SGI0, SGI0, SGI0, SGI229572, SGI229572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50932[] = {SGI229573, SGI0, SGI229574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229575, SGI229576, SGI229576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229577, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50933[] = {SGI229578, SGI229579, SGI229580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229581, SGI0, SGI0, SGI0, SGI229582, SGI229582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50934[] = {SGI229583, SGI0, SGI229584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229585, SGI229586, SGI229586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50935[] = {SGI229587, SGI0, SGI229588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229589, SGI229590, SGI229590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50936[] = {SGI229591, SGI0, SGI229592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229593, SGI229594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50937[] = {SGI229595, SGI0, SGI229596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229599, SGI229600, SGI229600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50938[] = {SGI229597, SGI0, SGI229598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229601, SGI229602, SGI229602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50939[] = {SGI229603, SGI0, SGI229604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229605, SGI229606, SGI229606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50940[] = {SGI229607, SGI0, SGI229608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229609, SGI229610, SGI229610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50941[] = {SGI229611, SGI0, SGI229613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229615, SGI229617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50942[] = {SGI229612, SGI0, SGI229614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229616, SGI229618, SGI229618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50943[] = {SGI229619, SGI0, SGI229620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229621, SGI229622, SGI229622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50944[] = {SGI229623, SGI229624, SGI229625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229628, SGI0, SGI0, SGI0, SGI229631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50945[] = {SGI229626, SGI0, SGI229627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229633, SGI229634, SGI229634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229636, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50946[] = {SGI229629, SGI229630, SGI229632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229635, SGI229635, SGI0, SGI0, SGI0, SGI0, SGI229637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50947[] = {SGI229638, SGI0, SGI229639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229640, SGI229641, SGI229641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229642, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50948[] = {SGI229643, SGI0, SGI229644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229645, SGI229646, SGI229646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229647, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50949[] = {SGI229648, SGI229649, SGI229650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229651, SGI0, SGI0, SGI0, SGI229652, SGI229652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50950[] = {SGI229653, SGI0, SGI229654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229655, SGI0, SGI0, SGI0, SGI229656, SGI229656, SGI0, SGI0, SGI0, SGI229657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50951[] = {SGI229658, SGI229659, SGI229660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229662, SGI0, SGI0, SGI0, SGI229665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50952[] = {SGI229661, SGI229663, SGI229664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229666, SGI0, SGI0, SGI0, SGI229667, SGI229667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50953[] = {SGI229668, SGI0, SGI229669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229670, SGI229672, SGI229672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50954[] = {SGI229671, SGI229673, SGI229674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229675, SGI0, SGI0, SGI0, SGI229676, SGI229676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50955[] = {SGI229677, SGI0, SGI229678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229679, SGI229680, SGI229680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50956[] = {SGI229683, SGI0, SGI229685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229688, SGI229689, SGI229689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50957[] = {SGI229681, SGI229682, SGI229684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229686, SGI0, SGI0, SGI0, SGI229687, SGI229687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229690, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50958[] = {SGI229691, SGI229692, SGI229693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229694, SGI0, SGI0, SGI0, SGI229695, SGI229695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229696, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50959[] = {SGI229697, SGI0, SGI229698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229699, SGI229700, SGI229700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50960[] = {SGI229701, SGI0, SGI229702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229703, SGI229704, SGI229704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50961[] = {SGI229705, SGI229706, SGI229707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229708, SGI0, SGI0, SGI0, SGI229709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50962[] = {SGI229710, SGI229711, SGI229712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229713, SGI229713, SGI0, SGI0, SGI0, SGI229714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50963[] = {SGI229715, SGI229716, SGI229717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229718, SGI229718, SGI0, SGI0, SGI0, SGI229719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50964[] = {SGI229720, SGI229721, SGI229722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229723, SGI229723, SGI0, SGI0, SGI0, SGI229724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50965[] = {SGI229725, SGI0, SGI229726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229727, SGI229728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50966[] = {SGI229729, SGI229730, SGI229731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229732, SGI229732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229733, NULL};
const Segment * const SG50967[] = {SGI229734, SGI0, SGI229735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229736, SGI0, SGI0, SGI0, SGI229737, SGI229737, SGI0, SGI0, SGI0, SGI229738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50968[] = {SGI229739, SGI229740, SGI229741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229742, SGI229742, SGI0, SGI0, SGI0, SGI229743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50969[] = {SGI229744, SGI229745, SGI229746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229747, SGI229747, SGI0, SGI0, SGI0, SGI229748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50970[] = {SGI229749, SGI0, SGI229750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229751, SGI229752, SGI229752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50971[] = {SGI229753, SGI229754, SGI229755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229756, SGI0, SGI0, SGI0, SGI229757, SGI229757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50972[] = {SGI229758, SGI0, SGI229759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229760, SGI229761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50973[] = {SGI229762, SGI0, SGI229763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229764, SGI229765, SGI229765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50974[] = {SGI229766, SGI0, SGI229767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229768, SGI229769, SGI229769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50975[] = {SGI229770, SGI229771, SGI229772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229773, SGI229773, SGI0, SGI0, SGI229774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50976[] = {SGI229775, SGI0, SGI229776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229777, SGI229778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50977[] = {SGI229779, SGI0, SGI229780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229782, SGI229784, SGI229784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50978[] = {SGI229781, SGI0, SGI229783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229785, SGI229786, SGI229786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50979[] = {SGI229788, SGI0, SGI229790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229793, SGI229794, SGI229794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50980[] = {SGI229787, SGI229789, SGI229791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229792, SGI0, SGI0, SGI0, SGI229795, SGI229795, SGI0, SGI229796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50981[] = {SGI229797, SGI229798, SGI229799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229800, SGI0, SGI0, SGI0, SGI229801, SGI229801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229802, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50982[] = {SGI229803, SGI229804, SGI229805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229809, SGI0, SGI0, SGI0, SGI229810, SGI229810, SGI0, SGI0, SGI0, SGI0, SGI229812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50983[] = {SGI229806, SGI229807, SGI229808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229811, SGI229811, SGI0, SGI0, SGI0, SGI229813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50984[] = {SGI229814, SGI229815, SGI229816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229817, SGI0, SGI0, SGI0, SGI229818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229819, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50985[] = {SGI229820, SGI0, SGI229821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229824, SGI229825, SGI229825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229827, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50986[] = {SGI229822, SGI0, SGI229823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229826, SGI229828, SGI229828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50987[] = {SGI229829, SGI229830, SGI229832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229835, SGI0, SGI0, SGI0, SGI229836, SGI229836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50988[] = {SGI229831, SGI229833, SGI229834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229837, SGI229837, SGI0, SGI0, SGI0, SGI0, SGI229838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50989[] = {SGI229839, SGI229840, SGI229841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229842, SGI0, SGI0, SGI0, SGI229843, SGI229843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50990[] = {SGI229844, SGI0, SGI229845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229846, SGI229847, SGI229847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50991[] = {SGI229848, SGI0, SGI229849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229850, SGI229851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50992[] = {SGI229852, SGI0, SGI229853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229854, SGI0, SGI0, SGI0, SGI229855, SGI229855, SGI0, SGI0, SGI0, SGI0, SGI229856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50993[] = {SGI229857, SGI229858, SGI229859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229860, SGI0, SGI0, SGI0, SGI229861, SGI229861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229862, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50994[] = {SGI229863, SGI0, SGI229864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229865, SGI229866, SGI229866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50995[] = {SGI229867, SGI0, SGI229868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229869, SGI229870, SGI229870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50996[] = {SGI229871, SGI229872, SGI229873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229874, SGI229874, SGI0, SGI0, SGI0, SGI0, SGI229875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50997[] = {SGI229876, SGI229877, SGI229878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229879, SGI229879, SGI0, SGI0, SGI0, SGI0, SGI229880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50998[] = {SGI229881, SGI229882, SGI229883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229886, SGI229886, SGI0, SGI0, SGI0, SGI229888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG50999[] = {SGI229884, SGI0, SGI229885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229887, SGI229889, SGI229889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51000[] = {SGI229894, SGI229895, SGI229896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229897, SGI0, SGI0, SGI0, SGI229898, SGI229898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51001[] = {SGI229899, SGI229900, SGI229901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229902, SGI229902, SGI0, SGI0, SGI0, SGI229903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51002[] = {SGI229904, SGI229905, SGI229906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229909, SGI229909, SGI0, SGI0, SGI0, SGI0, SGI229911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51003[] = {SGI229907, SGI229908, SGI229910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229912, SGI229912, SGI0, SGI0, SGI0, SGI229913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51004[] = {SGI229914, SGI0, SGI229915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229917, SGI229918, SGI229918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51005[] = {SGI229916, SGI0, SGI229919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229920, SGI229921, SGI229921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51006[] = {SGI229922, SGI0, SGI229923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229924, SGI229925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51007[] = {SGI229926, SGI0, SGI229927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229930, SGI229931, SGI229931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51008[] = {SGI229928, SGI0, SGI229929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229932, SGI229933, SGI229933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51009[] = {SGI229934, SGI0, SGI229935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229937, SGI229939, SGI229939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229940, SGI0, SGI0, NULL};
const Segment * const SG51010[] = {SGI229936, SGI0, SGI229938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229941, SGI229942, SGI229942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51011[] = {SGI229943, SGI229944, SGI229945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229946, SGI229946, SGI0, SGI0, SGI0, SGI0, SGI229947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51012[] = {SGI229948, SGI0, SGI229949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229950, SGI229951, SGI229951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51013[] = {SGI229952, SGI229953, SGI229954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229955, SGI229955, SGI0, SGI0, SGI0, SGI229956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51014[] = {SGI229957, SGI0, SGI229958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229959, SGI229960, SGI229960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51015[] = {SGI229961, SGI0, SGI229962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229963, SGI229964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51016[] = {SGI229965, SGI0, SGI229966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229967, SGI229968, SGI229968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51017[] = {SGI229969, SGI0, SGI229970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229971, SGI229972, SGI229972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51018[] = {SGI229974, SGI0, SGI229976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229977, SGI229979, SGI229979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51019[] = {SGI229973, SGI0, SGI229975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229978, SGI229980, SGI229980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51020[] = {SGI229981, SGI229982, SGI229983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229984, SGI0, SGI0, SGI0, SGI229985, SGI229985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51021[] = {SGI229986, SGI229987, SGI229988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229989, SGI0, SGI0, SGI0, SGI229990, SGI229990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51022[] = {SGI229992, SGI0, SGI229993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229994, SGI229995, SGI229995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51023[] = {SGI229996, SGI229997, SGI229998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229999, SGI229999, SGI0, SGI0, SGI230000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51024[] = {SGI230001, SGI230002, SGI230003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230004, SGI230004, SGI0, SGI0, SGI230005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51025[] = {SGI230006, SGI230007, SGI230008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230009, SGI0, SGI0, SGI0, SGI230010, SGI230010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230011, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51026[] = {SGI230012, SGI230013, SGI230014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230015, SGI230015, SGI0, SGI0, SGI0, SGI0, SGI230016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51027[] = {SGI230017, SGI0, SGI230018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230022, SGI230023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230026, SGI0, SGI0, NULL};
const Segment * const SG51028[] = {SGI230019, SGI230020, SGI230021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230024, SGI0, SGI0, SGI0, SGI230025, SGI230025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51029[] = {SGI229890, SGI0, SGI229891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI229892, SGI229893, SGI229893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51030[] = {SGI230027, SGI0, SGI230028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230029, SGI0, SGI0, SGI0, SGI230030, SGI230030, SGI0, SGI0, SGI0, SGI230031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51031[] = {SGI230032, SGI230033, SGI230034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230037, SGI0, SGI0, SGI0, SGI230038, SGI230038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230039, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51032[] = {SGI230035, SGI0, SGI230036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230040, SGI230041, SGI230041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51033[] = {SGI0, SGI0, SGI230042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230043, SGI0, SGI0, SGI0, SGI230044, SGI230044, SGI0, SGI0, SGI0, SGI230046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51034[] = {SGI230045, SGI230048, SGI230049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230052, SGI230052, SGI0, SGI0, SGI0, SGI0, SGI230054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51035[] = {SGI230047, SGI0, SGI230050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230051, SGI230053, SGI230053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51036[] = {SGI230056, SGI0, SGI230057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230059, SGI230060, SGI230060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51037[] = {SGI230055, SGI0, SGI230058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230061, SGI230062, SGI230062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230063, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51038[] = {SGI230064, SGI230065, SGI230066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230067, SGI230067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230068, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51039[] = {SGI230069, SGI0, SGI230070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230071, SGI230072, SGI230072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51040[] = {SGI230073, SGI0, SGI230074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230078, SGI230079, SGI230079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51041[] = {SGI230075, SGI230076, SGI230077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230080, SGI0, SGI0, SGI0, SGI230081, SGI230081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51042[] = {SGI230082, SGI0, SGI230084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230087, SGI230088, SGI230088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51043[] = {SGI230083, SGI230085, SGI230086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230089, SGI230089, SGI0, SGI0, SGI0, SGI0, SGI230090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51044[] = {SGI230091, SGI0, SGI230092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230093, SGI230094, SGI230094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51045[] = {SGI230095, SGI0, SGI230096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230097, SGI230098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51046[] = {SGI230099, SGI0, SGI230101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230104, SGI230105, SGI230105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51047[] = {SGI230100, SGI230102, SGI230103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230106, SGI230106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230107, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51048[] = {SGI230108, SGI0, SGI230109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230110, SGI0, SGI0, SGI0, SGI230111, SGI230111, SGI0, SGI0, SGI0, SGI230112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51049[] = {SGI230113, SGI0, SGI230114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230115, SGI230116, SGI230116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51050[] = {SGI230117, SGI230118, SGI230119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230120, SGI0, SGI230121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51051[] = {SGI230122, SGI230123, SGI230124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230125, SGI0, SGI230126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51052[] = {SGI230127, SGI230128, SGI230129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230130, SGI0, SGI230131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51053[] = {SGI230132, SGI230133, SGI230134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230135, SGI0, SGI0, SGI0, SGI230136, SGI0, SGI0, SGI230137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51054[] = {SGI230138, SGI0, SGI230139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230140, SGI230141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51055[] = {SGI230142, SGI0, SGI230143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230144, SGI230145, SGI230145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51056[] = {SGI230146, SGI0, SGI230147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230148, SGI230149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51057[] = {SGI230150, SGI0, SGI230151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230152, SGI230153, SGI230153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51058[] = {SGI230154, SGI0, SGI230155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230156, SGI230157, SGI230157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51059[] = {SGI230158, SGI0, SGI230159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230160, SGI230161, SGI230161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51060[] = {SGI230162, SGI0, SGI230163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230164, SGI230165, SGI230165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51061[] = {SGI230166, SGI0, SGI230167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230168, SGI230169, SGI230169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51062[] = {SGI230170, SGI0, SGI230171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230172, SGI230173, SGI230173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51063[] = {SGI230174, SGI0, SGI230175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230176, SGI230177, SGI230177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51064[] = {SGI230178, SGI0, SGI230179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230180, SGI230181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51065[] = {SGI230182, SGI0, SGI230183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230186, SGI230187, SGI230187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51066[] = {SGI230184, SGI0, SGI230185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230188, SGI230189, SGI230189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51067[] = {SGI230190, SGI230191, SGI230192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230193, SGI0, SGI0, SGI0, SGI230194, SGI230194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230195, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51068[] = {SGI230196, SGI0, SGI230197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230198, SGI230199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51069[] = {SGI230200, SGI230201, SGI230202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230203, SGI0, SGI0, SGI0, SGI230204, SGI230204, SGI0, SGI0, SGI230205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51070[] = {SGI230206, SGI0, SGI230207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230208, SGI230209, SGI230209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51071[] = {SGI230210, SGI230211, SGI230212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230213, SGI230213, SGI0, SGI0, SGI0, SGI0, SGI230215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51072[] = {SGI230214, SGI0, SGI230216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230217, SGI0, SGI0, SGI0, SGI230218, SGI230218, SGI0, SGI0, SGI230219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51073[] = {SGI230220, SGI0, SGI230221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230222, SGI230223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51074[] = {SGI230224, SGI230225, SGI230226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230227, SGI0, SGI0, SGI0, SGI230228, SGI230228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51075[] = {SGI230229, SGI0, SGI230231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230235, SGI230236, SGI230236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51076[] = {SGI230230, SGI230232, SGI230233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230234, SGI0, SGI0, SGI0, SGI230237, SGI230237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51077[] = {SGI230238, SGI0, SGI230239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230242, SGI230243, SGI230243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51078[] = {SGI230240, SGI0, SGI230241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230244, SGI230245, SGI230245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51079[] = {SGI230246, SGI0, SGI230247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230248, SGI230250, SGI230250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51080[] = {SGI230249, SGI230251, SGI230252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230253, SGI230253, SGI0, SGI0, SGI0, SGI0, SGI230254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51081[] = {SGI230255, SGI230256, SGI230257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230258, SGI0, SGI0, SGI0, SGI230259, SGI230259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230260, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51082[] = {SGI230261, SGI0, SGI230262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230263, SGI230264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51083[] = {SGI230265, SGI0, SGI230266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230269, SGI230270, SGI230270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51084[] = {SGI230267, SGI0, SGI230268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230271, SGI230272, SGI230272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51085[] = {SGI230273, SGI0, SGI230274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230275, SGI230276, SGI230276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51086[] = {SGI230277, SGI0, SGI230278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230279, SGI230280, SGI230280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230281, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51087[] = {SGI230282, SGI0, SGI230283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230286, SGI230287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51088[] = {SGI230284, SGI0, SGI230285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230288, SGI230289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51089[] = {SGI230290, SGI230291, SGI230292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230295, SGI0, SGI0, SGI0, SGI230296, SGI230296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230298, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51090[] = {SGI230293, SGI0, SGI230294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230297, SGI230299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51091[] = {SGI230300, SGI230301, SGI230302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230303, SGI0, SGI0, SGI0, SGI230304, SGI230304, SGI0, SGI230305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51092[] = {SGI230306, SGI0, SGI230307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230308, SGI230309, SGI230309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230313, SGI0, SGI0, NULL};
const Segment * const SG51093[] = {SGI230310, SGI230311, SGI230312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230314, SGI0, SGI0, SGI0, SGI230315, SGI230315, SGI0, SGI230316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51094[] = {SGI230317, SGI0, SGI230318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230319, SGI230320, SGI230320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51095[] = {SGI230321, SGI230322, SGI230323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230324, SGI0, SGI0, SGI0, SGI230325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51096[] = {SGI230326, SGI230327, SGI230328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230329, SGI230329, SGI0, SGI0, SGI230330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51097[] = {SGI230331, SGI0, SGI230332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230333, SGI230334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51098[] = {SGI230335, SGI0, SGI230336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230337, SGI0, SGI0, SGI0, SGI230338, SGI230338, SGI0, SGI0, SGI0, SGI230339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51099[] = {SGI230340, SGI230341, SGI230342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230343, SGI0, SGI0, SGI0, SGI230344, SGI230344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230345, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51100[] = {SGI230346, SGI230347, SGI230348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230349, SGI0, SGI0, SGI0, SGI230350, SGI230350, SGI0, SGI230351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51101[] = {SGI230352, SGI230353, SGI230354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230355, SGI0, SGI0, SGI0, SGI230356, SGI0, SGI0, SGI230357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51102[] = {SGI230358, SGI0, SGI230359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230360, SGI230361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51103[] = {SGI230362, SGI0, SGI230363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230364, SGI230366, SGI230366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51104[] = {SGI230365, SGI230367, SGI230368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230369, SGI0, SGI0, SGI0, SGI230370, SGI230371, SGI0, SGI0, SGI0, SGI230372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51105[] = {SGI230373, SGI230374, SGI230375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230376, SGI230376, SGI0, SGI0, SGI0, SGI230377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51106[] = {SGI230378, SGI230379, SGI230380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230381, SGI0, SGI0, SGI0, SGI230382, SGI230382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230385, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51107[] = {SGI230383, SGI0, SGI230384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51108[] = {SGI230388, SGI230390, SGI230391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51109[] = {SGI230386, SGI230387, SGI230389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230392, SGI0, SGI0, SGI0, SGI230393, SGI230393, SGI0, SGI230394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51110[] = {SGI230397, SGI230399, SGI230400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230401, SGI0, SGI0, SGI0, SGI230403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51111[] = {SGI230395, SGI230396, SGI230398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230402, SGI230402, SGI0, SGI0, SGI0, SGI230404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51112[] = {SGI230405, SGI0, SGI230406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230407, SGI230408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230409, SGI0, SGI0, NULL};
const Segment * const SG51113[] = {SGI230410, SGI230411, SGI230412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230413, SGI0, SGI0, SGI0, SGI230414, SGI230414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51114[] = {SGI230416, SGI0, SGI230417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230418, SGI230419, SGI230419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51115[] = {SGI230420, SGI230421, SGI230422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230423, SGI0, SGI0, SGI0, SGI230424, SGI230424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230425, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51116[] = {SGI230426, SGI0, SGI230427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230428, SGI230429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51117[] = {SGI230430, SGI230431, SGI230432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230433, SGI0, SGI0, SGI0, SGI230435, SGI230435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51118[] = {SGI230434, SGI0, SGI230436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230438, SGI230439, SGI230439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51119[] = {SGI230440, SGI230441, SGI230442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230444, SGI0, SGI0, SGI0, SGI230446, SGI230446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230447, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51120[] = {SGI230443, SGI0, SGI230445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230448, SGI230449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51121[] = {SGI230450, SGI230451, SGI230452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230453, SGI230453, SGI0, SGI0, SGI0, SGI230454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51122[] = {SGI230455, SGI230456, SGI230457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230458, SGI230458, SGI0, SGI0, SGI0, SGI230459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51123[] = {SGI230460, SGI230461, SGI230462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230463, SGI230463, SGI0, SGI0, SGI0, SGI230464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51124[] = {SGI230465, SGI0, SGI230466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230467, SGI230468, SGI230468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51125[] = {SGI230469, SGI0, SGI230470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230471, SGI230472, SGI230472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51126[] = {SGI230473, SGI0, SGI230474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230475, SGI230476, SGI230476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51127[] = {SGI230477, SGI0, SGI230478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230479, SGI230480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51128[] = {SGI230481, SGI0, SGI230482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230483, SGI0, SGI0, SGI0, SGI230484, SGI230484, SGI0, SGI0, SGI0, SGI230485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51129[] = {SGI230486, SGI230487, SGI230488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230489, SGI0, SGI0, SGI0, SGI230490, SGI230490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51130[] = {SGI230491, SGI230492, SGI230493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230494, SGI0, SGI0, SGI0, SGI230495, SGI230495, SGI0, SGI0, SGI0, SGI230496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51131[] = {SGI230497, SGI0, SGI230498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230499, SGI230500, SGI230500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51132[] = {SGI230501, SGI230502, SGI230503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230504, SGI0, SGI0, SGI0, SGI230505, SGI230505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230506, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51133[] = {SGI230507, SGI230508, SGI230509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230510, SGI0, SGI0, SGI0, SGI230511, SGI230511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51134[] = {SGI230513, SGI0, SGI230514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230515, SGI230516, SGI230516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230517, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51135[] = {SGI230518, SGI230519, SGI230520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230521, SGI0, SGI0, SGI0, SGI230522, SGI230522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51136[] = {SGI230523, SGI0, SGI230524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230525, SGI230526, SGI230526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230527, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51137[] = {SGI230528, SGI0, SGI230529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230530, SGI230531, SGI230531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51138[] = {SGI230532, SGI0, SGI230534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230536, SGI230538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51139[] = {SGI230533, SGI0, SGI230535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230537, SGI230539, SGI230539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51140[] = {SGI230540, SGI230541, SGI230542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230543, SGI0, SGI0, SGI0, SGI230544, SGI230544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230545, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51141[] = {SGI230546, SGI230547, SGI230548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230549, SGI230549, SGI0, SGI0, SGI0, SGI230550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51142[] = {SGI230551, SGI230552, SGI230553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230554, SGI230554, SGI0, SGI0, SGI0, SGI230555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51143[] = {SGI230556, SGI0, SGI230557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230558, SGI230559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51144[] = {SGI230560, SGI0, SGI230561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230562, SGI230563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51145[] = {SGI230564, SGI0, SGI230565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230568, SGI230569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51146[] = {SGI230566, SGI0, SGI230567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230570, SGI230571, SGI230571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51147[] = {SGI230572, SGI230573, SGI230574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230579, SGI230579, SGI0, SGI0, SGI0, SGI0, SGI230581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51148[] = {SGI230575, SGI0, SGI230576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230580, SGI230583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51149[] = {SGI230577, SGI0, SGI230578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230585, SGI230587, SGI230587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51150[] = {SGI230582, SGI230584, SGI230586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230588, SGI230588, SGI0, SGI0, SGI0, SGI230589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51151[] = {SGI230590, SGI230591, SGI230592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230593, SGI0, SGI0, SGI0, SGI230594, SGI230594, SGI0, SGI230595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51152[] = {SGI230596, SGI230597, SGI230598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230599, SGI0, SGI0, SGI0, SGI230600, SGI230600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51153[] = {SGI230601, SGI0, SGI230602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230605, SGI230606, SGI230606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51154[] = {SGI230603, SGI0, SGI230604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230607, SGI230608, SGI230608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51155[] = {SGI230609, SGI230610, SGI230611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230612, SGI230612, SGI0, SGI0, SGI0, SGI230613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51156[] = {SGI230614, SGI0, SGI230615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230618, SGI230619, SGI230619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51157[] = {SGI230620, SGI0, SGI230621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230622, SGI230623, SGI230623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51158[] = {SGI230624, SGI0, SGI230625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230626, SGI230627, SGI230627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51159[] = {SGI230628, SGI0, SGI230629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230630, SGI230631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51160[] = {SGI230632, SGI230633, SGI230634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230635, SGI0, SGI0, SGI0, SGI230636, SGI230636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230637, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51161[] = {SGI230638, SGI0, SGI230639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230640, SGI230641, SGI230641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51162[] = {SGI230642, SGI0, SGI230643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230644, SGI230645, SGI230645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51163[] = {SGI230646, SGI0, SGI230647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230648, SGI230649, SGI230649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51164[] = {SGI230650, SGI0, SGI230651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230652, SGI230653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51165[] = {SGI230654, SGI0, SGI230655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230656, SGI230657, SGI230657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230658, SGI0, SGI0, NULL};
const Segment * const SG51166[] = {SGI230659, SGI0, SGI230660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230661, SGI230662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51167[] = {SGI230663, SGI0, SGI230664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230665, SGI230666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51168[] = {SGI230667, SGI0, SGI230668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230669, SGI230670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51169[] = {SGI230671, SGI0, SGI230672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230673, SGI230674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51170[] = {SGI230675, SGI230676, SGI230677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230678, SGI0, SGI0, SGI0, SGI230679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51171[] = {SGI230680, SGI0, SGI230681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51172[] = {SGI230682, SGI0, SGI230683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230684, SGI230685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51173[] = {SGI230686, SGI0, SGI230687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230692, SGI230693, SGI230693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51174[] = {SGI230689, SGI0, SGI230691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230695, SGI230697, SGI230697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51175[] = {SGI230688, SGI0, SGI230690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230694, SGI230696, SGI230696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51176[] = {SGI230698, SGI230699, SGI230700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230701, SGI0, SGI0, SGI0, SGI230702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51177[] = {SGI230704, SGI0, SGI230705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230706, SGI230707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51178[] = {SGI230708, SGI230709, SGI230710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230711, SGI0, SGI0, SGI0, SGI230712, SGI230712, SGI230713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51179[] = {SGI230714, SGI230715, SGI230716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230717, SGI0, SGI0, SGI0, SGI230719, SGI230719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51180[] = {SGI230718, SGI230720, SGI230721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230723, SGI230723, SGI0, SGI0, SGI230724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51181[] = {SGI230725, SGI0, SGI230726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230727, SGI230728, SGI230728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230729, SGI0, SGI0, NULL};
const Segment * const SG51182[] = {SGI230730, SGI0, SGI230731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230732, SGI0, SGI0, SGI0, SGI230733, SGI230733, SGI0, SGI0, SGI0, SGI230734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51183[] = {SGI230735, SGI0, SGI230736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230737, SGI230738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51184[] = {SGI230739, SGI0, SGI230740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230741, SGI0, SGI0, SGI0, SGI230742, SGI230742, SGI0, SGI0, SGI0, SGI230743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51185[] = {SGI230744, SGI0, SGI230745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230746, SGI0, SGI0, SGI0, SGI230747, SGI230747, SGI0, SGI0, SGI0, SGI230748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51186[] = {SGI230749, SGI0, SGI230750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230751, SGI230752, SGI230752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51187[] = {SGI230616, SGI0, SGI230617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230753, SGI230754, SGI230754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51188[] = {SGI230755, SGI0, SGI230756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230757, SGI230758, SGI230758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51189[] = {SGI230759, SGI230760, SGI230761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230765, SGI0, SGI0, SGI0, SGI230766, SGI230766, SGI0, SGI230767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51190[] = {SGI230762, SGI230763, SGI230764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230768, SGI0, SGI0, SGI0, SGI230769, SGI230769, SGI0, SGI0, SGI230770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51191[] = {SGI230771, SGI230772, SGI230773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230774, SGI0, SGI0, SGI0, SGI230775, SGI230775, SGI0, SGI0, NULL};
const Segment * const SG51192[] = {SGI230776, SGI230777, SGI230778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230781, SGI0, SGI0, SGI0, SGI230783, SGI230783, SGI0, SGI0, SGI230784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51193[] = {SGI230779, SGI230780, SGI230782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230785, SGI230785, SGI0, SGI0, SGI0, SGI0, SGI230786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51194[] = {SGI230787, SGI230788, SGI230789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230790, SGI0, SGI0, SGI0, SGI230791, SGI230792, SGI0, SGI0, SGI0, SGI0, SGI230793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51195[] = {SGI230794, SGI0, SGI230795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230796, SGI0, SGI0, SGI0, SGI230797, SGI230797, SGI0, SGI0, SGI0, SGI0, SGI230798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51196[] = {SGI230799, SGI230800, SGI230801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230802, SGI0, SGI0, SGI0, SGI230803, SGI230803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51197[] = {SGI230804, SGI0, SGI230805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230807, SGI230808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230810, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51198[] = {SGI230806, SGI0, SGI230809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230811, SGI230812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51199[] = {SGI230813, SGI0, SGI230814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230815, SGI230816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230817, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51200[] = {SGI230818, SGI0, SGI230819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230823, SGI230824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51201[] = {SGI230820, SGI230821, SGI230822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230825, SGI0, SGI0, SGI0, SGI230826, SGI230826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230827, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51202[] = {SGI230828, SGI0, SGI230829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230830, SGI230831, SGI230831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51203[] = {SGI230832, SGI0, SGI230833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230834, SGI230835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51204[] = {SGI0, SGI0, SGI230836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230837, SGI0, SGI0, SGI0, SGI230838, SGI230838, SGI0, SGI0, SGI0, SGI230839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51205[] = {SGI230840, SGI0, SGI230841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230842, SGI230843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51206[] = {SGI230844, SGI0, SGI230845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230846, SGI230847, SGI230847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51207[] = {SGI230848, SGI0, SGI230849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230850, SGI230851, SGI230851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51208[] = {SGI230852, SGI0, SGI230853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230854, SGI230855, SGI230855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51209[] = {SGI230856, SGI0, SGI230857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230860, SGI230861, SGI230861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51210[] = {SGI230858, SGI0, SGI230859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230862, SGI230863, SGI230863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51211[] = {SGI230864, SGI0, SGI230865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230866, SGI230867, SGI230867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51212[] = {SGI230868, SGI0, SGI230869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230870, SGI230871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51213[] = {SGI230872, SGI230873, SGI230874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230877, SGI0, SGI0, SGI0, SGI230878, SGI230878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230881, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51214[] = {SGI230875, SGI0, SGI230876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230879, SGI230880, SGI230880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51215[] = {SGI230882, SGI0, SGI230883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230884, SGI230885, SGI230885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51216[] = {SGI230886, SGI0, SGI230887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230890, SGI230891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51217[] = {SGI230888, SGI0, SGI230889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230892, SGI230893, SGI230893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51218[] = {SGI230894, SGI230895, SGI230896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230897, SGI0, SGI0, SGI0, SGI230898, SGI230898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230899, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51219[] = {SGI230902, SGI0, SGI230905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51220[] = {SGI230900, SGI230901, SGI230904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230908, SGI0, SGI0, SGI0, SGI230910, SGI230910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230912, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51221[] = {SGI230903, SGI230906, SGI230907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230909, SGI0, SGI0, SGI0, SGI230911, SGI230911, SGI0, SGI0, SGI0, SGI0, SGI230913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51222[] = {SGI230914, SGI230915, SGI230916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230917, SGI0, SGI0, SGI0, SGI230918, SGI230918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51223[] = {SGI230919, SGI0, SGI230920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230921, SGI230922, SGI230922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51224[] = {SGI230923, SGI0, SGI230924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230925, SGI230926, SGI230926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51225[] = {SGI230927, SGI0, SGI230928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230929, SGI230930, SGI230930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51226[] = {SGI230931, SGI0, SGI230932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230933, SGI230935, SGI230935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51227[] = {SGI230934, SGI0, SGI230936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230939, SGI230941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51228[] = {SGI230937, SGI230938, SGI230940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230942, SGI0, SGI0, SGI0, SGI230943, SGI230943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230944, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51229[] = {SGI230945, SGI230946, SGI230947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230948, SGI0, SGI0, SGI0, SGI230949, SGI230949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230950, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51230[] = {SGI230951, SGI230952, SGI230953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230954, SGI0, SGI0, SGI0, SGI230955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51231[] = {SGI230956, SGI230957, SGI230958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230960, SGI0, SGI0, SGI0, SGI230962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51232[] = {SGI230959, SGI230961, SGI230963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230964, SGI0, SGI0, SGI0, SGI230965, SGI230965, SGI230966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51233[] = {SGI230967, SGI0, SGI230968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230969, SGI230970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51234[] = {SGI230971, SGI230972, SGI230973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230974, SGI0, SGI0, SGI0, SGI230975, SGI230975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51235[] = {SGI230976, SGI230977, SGI230978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230980, SGI0, SGI0, SGI0, SGI230982, SGI230982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230983, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51236[] = {SGI230979, SGI0, SGI230981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230984, SGI230985, SGI230985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51237[] = {SGI230986, SGI0, SGI230987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230988, SGI230989, SGI230989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51238[] = {SGI230990, SGI0, SGI230991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230992, SGI0, SGI0, SGI0, SGI230993, SGI230993, SGI0, SGI0, SGI0, SGI230994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51239[] = {SGI230995, SGI0, SGI230996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI230999, SGI231000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51240[] = {SGI230997, SGI0, SGI230998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231001, SGI231002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51241[] = {SGI231003, SGI0, SGI231004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231005, SGI231007, SGI231007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51242[] = {SGI231006, SGI0, SGI231008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231009, SGI231010, SGI231010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51243[] = {SGI231011, SGI0, SGI231012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231014, SGI231016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231017, SGI0, SGI0, NULL};
const Segment * const SG51244[] = {SGI231013, SGI0, SGI231015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231018, SGI231019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51245[] = {SGI231020, SGI231021, SGI231022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231023, SGI0, SGI0, SGI0, SGI231024, SGI231024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231025, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51246[] = {SGI231026, SGI0, SGI231027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231028, SGI231029, SGI231029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51247[] = {SGI231030, SGI0, SGI231031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231032, SGI231033, SGI231033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51248[] = {SGI231034, SGI0, SGI231035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231036, SGI231037, SGI231037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51249[] = {SGI231038, SGI0, SGI231039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231040, SGI231041, SGI231041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51250[] = {SGI231042, SGI231043, SGI231044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231045, SGI231045, SGI0, SGI0, SGI0, SGI231046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51251[] = {SGI231047, SGI231048, SGI231049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231050, SGI0, SGI0, SGI0, SGI231051, SGI231051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51252[] = {SGI231052, SGI0, SGI231053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231054, SGI231055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231056, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51253[] = {SGI231057, SGI0, SGI231058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231059, SGI231060, SGI231060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51254[] = {SGI231061, SGI0, SGI231062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231063, SGI231064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51255[] = {SGI231065, SGI0, SGI231066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231067, SGI231068, SGI231068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51256[] = {SGI231071, SGI0, SGI231073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231074, SGI231075, SGI231075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51257[] = {SGI231069, SGI231070, SGI231072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231076, SGI231076, SGI0, SGI0, SGI231077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51258[] = {SGI231078, SGI0, SGI231079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231080, SGI231081, SGI231081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51259[] = {SGI231082, SGI0, SGI231083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231086, SGI231087, SGI231087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51260[] = {SGI231084, SGI0, SGI231085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231088, SGI231089, SGI231089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231090, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51261[] = {SGI231091, SGI0, SGI231092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231093, SGI231094, SGI231094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51262[] = {SGI231095, SGI0, SGI231096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231097, SGI231098, SGI231098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51263[] = {SGI231099, SGI0, SGI231100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231101, SGI231102, SGI231102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51264[] = {SGI231103, SGI0, SGI231104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231105, SGI231106, SGI231106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51265[] = {SGI231107, SGI0, SGI231108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231109, SGI231110, SGI231110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51266[] = {SGI231111, SGI0, SGI231112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231113, SGI231114, SGI231114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231115, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51267[] = {SGI231116, SGI0, SGI231117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231118, SGI231119, SGI231119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51268[] = {SGI0, SGI0, SGI231120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231122, SGI231124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51269[] = {SGI231121, SGI0, SGI231123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231126, SGI231127, SGI231127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51270[] = {SGI231125, SGI0, SGI231128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231129, SGI0, SGI0, SGI0, SGI231130, SGI231130, SGI0, SGI0, SGI0, SGI0, SGI231131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51271[] = {SGI231132, SGI231133, SGI231134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231137, SGI0, SGI0, SGI0, SGI231139, SGI231139, SGI0, SGI231141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51272[] = {SGI231135, SGI231136, SGI231138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231140, SGI0, SGI0, SGI0, SGI231142, SGI231142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51273[] = {SGI231143, SGI0, SGI231144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231145, SGI231146, SGI231146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51274[] = {SGI231147, SGI0, SGI231148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231149, SGI231150, SGI231150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51275[] = {SGI231151, SGI0, SGI231152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231153, SGI231154, SGI231154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51276[] = {SGI231155, SGI231156, SGI231157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231158, SGI231158, SGI0, SGI0, SGI0, SGI231159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51277[] = {SGI231160, SGI0, SGI231161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231162, SGI231163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51278[] = {SGI231164, SGI231165, SGI231166, SGI231167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231170, SGI0, SGI0, SGI0, SGI231171, SGI231172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51279[] = {SGI231168, SGI0, SGI231169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231173, SGI231174, SGI231174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51280[] = {SGI231175, SGI231176, SGI231177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231178, SGI0, SGI0, SGI0, SGI231179, SGI231179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51281[] = {SGI231180, SGI231181, SGI231182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231183, SGI231183, SGI0, SGI0, SGI0, SGI0, SGI231184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51282[] = {SGI231185, SGI231186, SGI231187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231188, SGI231188, SGI0, SGI0, SGI0, SGI0, SGI231189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51283[] = {SGI231190, SGI0, SGI231191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231192, SGI231193, SGI231193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51284[] = {SGI231194, SGI231195, SGI231196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231197, SGI0, SGI0, SGI0, SGI231198, SGI231198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51285[] = {SGI231199, SGI231200, SGI231201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231202, SGI0, SGI0, SGI0, SGI231203, SGI231203, SGI0, SGI231204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51286[] = {SGI231205, SGI231206, SGI231207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231208, SGI231208, SGI0, SGI0, SGI0, SGI231209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51287[] = {SGI231210, SGI0, SGI231211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231212, SGI231213, SGI231213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51288[] = {SGI231214, SGI0, SGI231215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231216, SGI231217, SGI231217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51289[] = {SGI231218, SGI0, SGI231219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231220, SGI231221, SGI231221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51290[] = {SGI231222, SGI0, SGI231223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231224, SGI231225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231226, SGI0, SGI0, NULL};
const Segment * const SG51291[] = {SGI231227, SGI0, SGI231228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231229, SGI231231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51292[] = {SGI231230, SGI0, SGI231232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231233, SGI231234, SGI231234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51293[] = {SGI231235, SGI231236, SGI231237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231238, SGI231238, SGI0, SGI0, SGI0, SGI231240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51294[] = {SGI231239, SGI0, SGI231241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231242, SGI231243, SGI231243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51295[] = {SGI231244, SGI231245, SGI231246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231247, SGI231247, SGI0, SGI0, SGI0, SGI231248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51296[] = {SGI231249, SGI231250, SGI231251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231252, SGI0, SGI0, SGI0, SGI231253, SGI231253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231254, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51297[] = {SGI231255, SGI231256, SGI231257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231258, SGI231258, SGI0, SGI0, SGI0, SGI231259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51298[] = {SGI231260, SGI0, SGI231261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231262, SGI231263, SGI231263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231264, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51299[] = {SGI231265, SGI231266, SGI231267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231268, SGI231268, SGI0, SGI0, SGI0, SGI231269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51300[] = {SGI231270, SGI0, SGI231271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231272, SGI231273, SGI231273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51301[] = {SGI231274, SGI0, SGI231275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231276, SGI231277, SGI231277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51302[] = {SGI231278, SGI0, SGI231279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231280, SGI231281, SGI231281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51303[] = {SGI231282, SGI231283, SGI231284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231285, SGI0, SGI0, SGI0, SGI231286, SGI231286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231287, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51304[] = {SGI231288, SGI231289, SGI231290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231291, SGI0, SGI0, SGI0, SGI231292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231293, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51305[] = {SGI231294, SGI231295, SGI231296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231297, SGI0, SGI0, SGI0, SGI231298, SGI231298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51306[] = {SGI231299, SGI231300, SGI231301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231302, SGI231302, SGI0, SGI0, SGI0, SGI231303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51307[] = {SGI231304, SGI0, SGI231305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231306, SGI0, SGI0, SGI0, SGI231307, SGI231307, SGI0, SGI0, SGI231308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51308[] = {SGI231309, SGI0, SGI231310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231311, SGI231312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231313, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51309[] = {SGI231314, SGI0, SGI231315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231316, SGI231317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51310[] = {SGI231318, SGI0, SGI231319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231320, SGI231321, SGI231321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51311[] = {SGI231323, SGI231325, SGI231327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231328, SGI0, SGI0, SGI0, SGI231330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51312[] = {SGI231322, SGI231324, SGI231326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231329, SGI231329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231331, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51313[] = {SGI231332, SGI0, SGI231333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231334, SGI231335, SGI231335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51314[] = {SGI231336, SGI231337, SGI231338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231339, SGI231339, SGI0, SGI0, SGI0, SGI231340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51315[] = {SGI231341, SGI0, SGI231342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231343, SGI231344, SGI231344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51316[] = {SGI231345, SGI231346, SGI231347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231348, SGI231348, SGI0, SGI0, SGI0, SGI231349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51317[] = {SGI231350, SGI0, SGI231351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231352, SGI231353, SGI231353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51318[] = {SGI231354, SGI0, SGI231355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231356, SGI0, SGI0, SGI0, SGI231357, SGI231357, SGI0, SGI0, SGI0, SGI0, SGI231358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51319[] = {SGI231359, SGI231360, SGI231361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231362, SGI231362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231363, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51320[] = {SGI231364, SGI231365, SGI231366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231367, SGI231367, SGI0, SGI0, SGI0, SGI231368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51321[] = {SGI231369, SGI231370, SGI231371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231373, SGI231373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231375, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51322[] = {SGI0, SGI231372, SGI231374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231376, SGI0, SGI0, SGI0, SGI231377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51323[] = {SGI231378, SGI231379, SGI231380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231381, SGI0, SGI0, SGI0, SGI231382, SGI231382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51324[] = {SGI231383, SGI231384, SGI231385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231386, SGI0, SGI0, SGI0, SGI231387, SGI231387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51325[] = {SGI231388, SGI0, SGI231390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231393, SGI231395, SGI231395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51326[] = {SGI231389, SGI231391, SGI231392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231394, SGI0, SGI0, SGI0, SGI231396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51327[] = {SGI231397, SGI231398, SGI231399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231400, SGI0, SGI0, SGI0, SGI231401, SGI231401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231402, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51328[] = {SGI231403, SGI231404, SGI231405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231407, SGI0, SGI0, SGI0, SGI231409, SGI231409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231410, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51329[] = {SGI231406, SGI0, SGI231408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231411, SGI231412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51330[] = {SGI231413, SGI0, SGI231414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231415, SGI231416, SGI231416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51331[] = {SGI231417, SGI0, SGI231418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231422, SGI231423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51332[] = {SGI231419, SGI231420, SGI231421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231424, SGI0, SGI0, SGI0, SGI231425, SGI231426, SGI0, SGI0, SGI0, SGI231427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51333[] = {SGI231428, SGI231429, SGI231430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231431, SGI0, SGI0, SGI0, SGI231432, SGI231432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51334[] = {SGI231433, SGI0, SGI231434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231435, SGI231436, SGI231436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51335[] = {SGI231437, SGI0, SGI231438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231439, SGI231440, SGI231440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51336[] = {SGI231441, SGI231442, SGI231443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231444, SGI0, SGI0, SGI0, SGI231446, SGI231446, SGI0, SGI0, SGI0, SGI0, SGI231450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51337[] = {SGI231445, SGI0, SGI231448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231452, SGI231454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51338[] = {SGI231447, SGI231449, SGI231451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231453, SGI0, SGI0, SGI0, SGI231455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51339[] = {SGI231456, SGI231457, SGI231458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231461, SGI231461, SGI0, SGI0, SGI0, SGI231462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51340[] = {SGI231459, SGI0, SGI231460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231463, SGI231464, SGI231464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231465, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51341[] = {SGI231466, SGI231467, SGI231468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231469, SGI231469, SGI0, SGI0, SGI231470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51342[] = {SGI231471, SGI0, SGI231472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51343[] = {SGI231473, SGI0, SGI231474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231475, SGI231476, SGI231476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51344[] = {SGI231477, SGI0, SGI231478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231479, SGI231480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51345[] = {SGI231482, SGI0, SGI231483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231485, SGI231487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51346[] = {SGI231481, SGI0, SGI231484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231486, SGI231488, SGI231488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51347[] = {SGI231489, SGI231490, SGI231491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231492, SGI0, SGI0, SGI0, SGI231493, SGI231493, SGI0, SGI231494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51348[] = {SGI231495, SGI0, SGI231496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231497, SGI231498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51349[] = {SGI231499, SGI0, SGI231500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231501, SGI231502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51350[] = {SGI231503, SGI231504, SGI231505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231506, SGI0, SGI0, SGI0, SGI231507, SGI231507, SGI0, SGI231508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51351[] = {SGI231509, SGI231510, SGI231511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231512, SGI0, SGI0, SGI0, SGI231513, SGI231513, SGI0, SGI0, SGI231514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51352[] = {SGI231515, SGI0, SGI231516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231517, SGI231518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51353[] = {SGI231519, SGI231520, SGI231521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231522, SGI0, SGI0, SGI0, SGI231523, SGI0, SGI0, SGI0, SGI231524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51354[] = {SGI231525, SGI231526, SGI231527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231528, SGI0, SGI0, SGI0, SGI231529, SGI231529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51355[] = {SGI231530, SGI231531, SGI231532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231533, SGI0, SGI0, SGI0, SGI231534, SGI231534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51356[] = {SGI231535, SGI0, SGI231536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231537, SGI231538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51357[] = {SGI231539, SGI0, SGI231540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231541, SGI231542, SGI231542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51358[] = {SGI231543, SGI0, SGI231544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231545, SGI231546, SGI231546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51359[] = {SGI231547, SGI0, SGI231548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231549, SGI231550, SGI231550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51360[] = {SGI231551, SGI231552, SGI231553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231554, SGI0, SGI0, SGI0, SGI231555, SGI231555, SGI0, SGI231556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51361[] = {SGI231557, SGI0, SGI231558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231559, SGI231560, SGI231560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51362[] = {SGI231561, SGI231562, SGI231563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231564, SGI0, SGI0, SGI0, SGI231565, SGI231565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51363[] = {SGI231566, SGI231567, SGI231568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231569, SGI0, SGI0, SGI0, SGI231570, SGI231570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51364[] = {SGI231571, SGI0, SGI231572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231573, SGI231574, SGI231574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51365[] = {SGI231575, SGI0, SGI231576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231577, SGI231578, SGI231578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51366[] = {SGI231579, SGI0, SGI231580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231581, SGI231582, SGI231582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51367[] = {SGI231583, SGI231584, SGI231585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231586, SGI0, SGI0, SGI0, SGI231587, SGI231587, SGI0, SGI0, SGI0, SGI231588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51368[] = {SGI231589, SGI231590, SGI231591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231592, SGI0, SGI0, SGI0, SGI231593, SGI231593, SGI0, SGI0, SGI0, SGI0, SGI231594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51369[] = {SGI231595, SGI231596, SGI231597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231598, SGI0, SGI0, SGI0, SGI231599, SGI231599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231600, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51370[] = {SGI231601, SGI231602, SGI231603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231604, SGI231604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231605, NULL};
const Segment * const SG51371[] = {SGI231606, SGI0, SGI231607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231608, SGI231609, SGI231609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231610, SGI0, SGI0, NULL};
const Segment * const SG51372[] = {SGI231611, SGI0, SGI231612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231613, SGI231614, SGI231614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51373[] = {SGI231615, SGI0, SGI231616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231617, SGI231618, SGI231618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51374[] = {SGI231619, SGI0, SGI231620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231621, SGI0, SGI0, SGI0, SGI231622, SGI231622, SGI0, SGI0, SGI0, SGI231624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51375[] = {SGI231623, SGI0, SGI231625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231626, SGI231627, SGI231627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51376[] = {SGI231628, SGI231629, SGI231630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231631, SGI231631, SGI0, SGI0, SGI0, SGI0, SGI231632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51377[] = {SGI231633, SGI231634, SGI231635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231636, SGI0, SGI0, SGI231637, SGI231638, SGI231638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51378[] = {SGI231639, SGI231640, SGI231641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231642, SGI0, SGI0, SGI231643, SGI231644, SGI231644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51379[] = {SGI231645, SGI231646, SGI231647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231648, SGI0, SGI0, SGI0, SGI231649, SGI231649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231650, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51380[] = {SGI231651, SGI0, SGI231652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231653, SGI231654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51381[] = {SGI231655, SGI231656, SGI231657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231658, SGI231658, SGI0, SGI0, SGI0, SGI231659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51382[] = {SGI231660, SGI231661, SGI231662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231663, SGI0, SGI0, SGI0, SGI231664, SGI0, SGI0, SGI0, SGI231665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51383[] = {SGI231666, SGI231667, SGI231668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231669, SGI0, SGI0, SGI0, SGI231670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51384[] = {SGI231671, SGI231672, SGI231673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231674, SGI0, SGI0, SGI0, SGI231675, SGI231675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51385[] = {SGI231676, SGI231677, SGI231678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231679, SGI0, SGI0, SGI0, SGI231680, SGI231680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51386[] = {SGI0, SGI231681, SGI0, SGI231682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51387[] = {SGI231683, SGI0, SGI231684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231685, SGI231686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51388[] = {SGI231687, SGI0, SGI231688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231689, SGI231690, SGI231690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51389[] = {SGI231691, SGI231692, SGI231693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231694, SGI0, SGI0, SGI0, SGI231695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231696, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51390[] = {SGI231697, SGI0, SGI231698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231699, SGI0, SGI0, SGI0, SGI231700, SGI231700, SGI0, SGI0, SGI0, SGI231701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51391[] = {SGI231702, SGI0, SGI231703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231704, SGI231705, SGI231705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51392[] = {SGI231706, SGI0, SGI231707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231708, SGI231709, SGI231709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51393[] = {SGI231710, SGI0, SGI231711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231712, SGI231713, SGI231713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51394[] = {SGI231714, SGI0, SGI231715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231716, SGI231717, SGI231717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51395[] = {SGI231718, SGI0, SGI231719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231720, SGI0, SGI0, SGI0, SGI231721, SGI231721, SGI0, SGI0, SGI0, SGI231722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51396[] = {SGI231723, SGI0, SGI231724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231725, SGI231726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51397[] = {SGI231727, SGI0, SGI231728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231729, SGI231730, SGI231730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51398[] = {SGI231731, SGI231732, SGI231733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231734, SGI0, SGI0, SGI0, SGI231735, SGI231735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51399[] = {SGI0, SGI0, SGI231737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231738, SGI0, SGI0, SGI0, SGI231739, SGI231739, SGI0, SGI0, SGI0, SGI231740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51400[] = {SGI231741, SGI0, SGI231742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231743, SGI231744, SGI231744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51401[] = {SGI231745, SGI0, SGI231746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231747, SGI231748, SGI231748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51402[] = {SGI231749, SGI231750, SGI231751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231752, SGI0, SGI0, SGI0, SGI231753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51403[] = {SGI231754, SGI0, SGI231755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231756, SGI231757, SGI231757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51404[] = {SGI231758, SGI0, SGI231759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231760, SGI231761, SGI231761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51405[] = {SGI231762, SGI0, SGI231763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231766, SGI231767, SGI231767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51406[] = {SGI231764, SGI0, SGI231765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231768, SGI231769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51407[] = {SGI231770, SGI0, SGI231771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231772, SGI231773, SGI231773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231774, SGI0, SGI0, NULL};
const Segment * const SG51408[] = {SGI231775, SGI0, SGI231776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231777, SGI231778, SGI231778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51409[] = {SGI231779, SGI0, SGI231780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231784, SGI231785, SGI231785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51410[] = {SGI231781, SGI231782, SGI231783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231786, SGI0, SGI0, SGI0, SGI231787, SGI231787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51411[] = {SGI231788, SGI231789, SGI231790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231791, SGI0, SGI0, SGI0, SGI231792, SGI231792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51412[] = {SGI231793, SGI231794, SGI231795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231796, SGI0, SGI0, SGI0, SGI231797, SGI231797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231798, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51413[] = {SGI231799, SGI0, SGI231800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231801, SGI231802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51414[] = {SGI231803, SGI231804, SGI231805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231806, SGI0, SGI0, SGI0, SGI231807, SGI231807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231809, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51415[] = {SGI231808, SGI0, SGI231810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231811, SGI0, SGI0, SGI0, SGI231812, SGI231812, SGI0, SGI0, SGI0, SGI231813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51416[] = {SGI231814, SGI0, SGI231815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231816, SGI231817, SGI231817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51417[] = {SGI231818, SGI0, SGI231819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231820, SGI231821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51418[] = {SGI231823, SGI0, SGI231825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231827, SGI231829, SGI231829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51419[] = {SGI231822, SGI0, SGI231824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231826, SGI231828, SGI231828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51420[] = {SGI231830, SGI231831, SGI231832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231833, SGI0, SGI0, SGI0, SGI231834, SGI231834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51421[] = {SGI231835, SGI0, SGI231836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231837, SGI231838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51422[] = {SGI231839, SGI0, SGI231840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231841, SGI231842, SGI231842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231843, SGI0, SGI0, NULL};
const Segment * const SG51423[] = {SGI231844, SGI231845, SGI231846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231847, SGI231847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231848, NULL};
const Segment * const SG51424[] = {SGI231852, SGI0, SGI231853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231855, SGI231856, SGI231856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51425[] = {SGI231849, SGI231850, SGI231851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231854, SGI231854, SGI0, SGI0, SGI0, SGI231857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51426[] = {SGI231858, SGI231859, SGI231860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231861, SGI231861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231862, NULL};
const Segment * const SG51427[] = {SGI231863, SGI0, SGI231864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231866, SGI231867, SGI231867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51428[] = {SGI231865, SGI0, SGI231868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231869, SGI231870, SGI231870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51429[] = {SGI231871, SGI0, SGI231872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231873, SGI231874, SGI231874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51430[] = {SGI231875, SGI231876, SGI231877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231878, SGI0, SGI0, SGI0, SGI231880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51431[] = {SGI231879, SGI0, SGI231881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231883, SGI231885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51432[] = {SGI231882, SGI231884, SGI231886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231887, SGI231887, SGI0, SGI0, SGI0, SGI0, SGI231888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51433[] = {SGI231889, SGI231890, SGI231891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231894, SGI231894, SGI0, SGI0, SGI0, SGI0, SGI231895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51434[] = {SGI231892, SGI0, SGI231893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231896, SGI231897, SGI231897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51435[] = {SGI231898, SGI0, SGI231899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231901, SGI231902, SGI231902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51436[] = {SGI231900, SGI0, SGI231903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231904, SGI231905, SGI231905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51437[] = {SGI231906, SGI0, SGI231907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231908, SGI231910, SGI231910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51438[] = {SGI231909, SGI0, SGI231911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231912, SGI231913, SGI231913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51439[] = {SGI231914, SGI231915, SGI231916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231917, SGI0, SGI0, SGI0, SGI231918, SGI231918, SGI0, SGI231919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51440[] = {SGI231920, SGI231921, SGI231922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231923, SGI0, SGI0, SGI0, SGI231924, SGI231924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231925, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51441[] = {SGI231926, SGI0, SGI231927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231928, SGI231929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231930, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51442[] = {SGI231931, SGI0, SGI231932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231933, SGI231935, SGI231935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51443[] = {SGI231934, SGI0, SGI231936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231937, SGI231938, SGI231938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51444[] = {SGI231939, SGI0, SGI231940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231941, SGI0, SGI0, SGI0, SGI231942, SGI231942, SGI0, SGI0, SGI231943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51445[] = {SGI231944, SGI0, SGI231945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231946, SGI231947, SGI231947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231948, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51446[] = {SGI231949, SGI0, SGI231950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231951, SGI231952, SGI231952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51447[] = {SGI231953, SGI0, SGI231954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231955, SGI231956, SGI231956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51448[] = {SGI231957, SGI0, SGI231958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231959, SGI231960, SGI231960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51449[] = {SGI231961, SGI0, SGI231962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231963, SGI231964, SGI231964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51450[] = {SGI231965, SGI0, SGI231966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231967, SGI231968, SGI231968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51451[] = {SGI231969, SGI0, SGI231970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231971, SGI231972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51452[] = {SGI231973, SGI0, SGI231974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231975, SGI231976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51453[] = {SGI231977, SGI0, SGI231978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231979, SGI231980, SGI231980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51454[] = {SGI231981, SGI0, SGI231982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231983, SGI231984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51455[] = {SGI231985, SGI231986, SGI231987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231988, SGI0, SGI0, SGI0, SGI231989, SGI231989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51456[] = {SGI231990, SGI231991, SGI231992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231993, SGI0, SGI0, SGI0, SGI231994, SGI231994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231995, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51457[] = {SGI231996, SGI0, SGI231997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI231998, SGI231999, SGI231999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51458[] = {SGI232000, SGI232001, SGI232002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232003, SGI0, SGI0, SGI0, SGI232004, SGI232004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232005, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51459[] = {SGI232006, SGI232007, SGI232008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232009, SGI232009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232010, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51460[] = {SGI232011, SGI232012, SGI232013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232014, SGI232014, SGI0, SGI0, SGI0, SGI232015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51461[] = {SGI232016, SGI0, SGI232017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232018, SGI232019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51462[] = {SGI232020, SGI0, SGI232021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232022, SGI232023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51463[] = {SGI232024, SGI232025, SGI232026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232027, SGI0, SGI0, SGI0, SGI232028, SGI232028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232031, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51464[] = {SGI232029, SGI0, SGI232030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232032, SGI232033, SGI232033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51465[] = {SGI232034, SGI0, SGI232035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232036, SGI232037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51466[] = {SGI232038, SGI232039, SGI232040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232041, SGI0, SGI0, SGI0, SGI232042, SGI232042, SGI0, SGI232043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51467[] = {SGI232044, SGI0, SGI232045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232047, SGI232048, SGI232048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51468[] = {SGI232046, SGI0, SGI232049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232050, SGI232051, SGI232051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51469[] = {SGI232052, SGI0, SGI232053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232054, SGI232055, SGI232055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51470[] = {SGI232056, SGI0, SGI232057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232060, SGI232062, SGI232062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51471[] = {SGI232058, SGI232059, SGI232061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232065, SGI232065, SGI0, SGI0, SGI0, SGI232066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51472[] = {SGI232063, SGI0, SGI232064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232067, SGI232068, SGI232068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51473[] = {SGI232069, SGI0, SGI232070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232071, SGI232072, SGI232072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51474[] = {SGI232073, SGI0, SGI232074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232075, SGI232076, SGI232076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51475[] = {SGI232077, SGI232078, SGI232079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232080, SGI0, SGI0, SGI0, SGI232081, SGI232081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232082, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51476[] = {SGI232083, SGI232084, SGI232085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232086, SGI0, SGI0, SGI0, SGI232087, SGI232087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232089, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51477[] = {SGI232088, SGI232090, SGI232091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232092, SGI0, SGI0, SGI0, SGI232093, SGI232093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232094, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51478[] = {SGI232095, SGI232096, SGI232097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232101, SGI0, SGI0, SGI0, SGI232102, SGI232102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232104, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51479[] = {SGI232098, SGI232099, SGI232100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232103, SGI0, SGI0, SGI0, SGI232105, SGI232105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232106, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51480[] = {SGI232107, SGI232108, SGI232109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232110, SGI0, SGI0, SGI0, SGI232111, SGI232111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232112, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51481[] = {SGI232113, SGI0, SGI232114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232115, SGI232116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51482[] = {SGI232117, SGI232118, SGI232119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232121, SGI0, SGI0, SGI0, SGI232124, SGI232124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232125, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51483[] = {SGI232120, SGI232122, SGI232123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232126, SGI0, SGI0, SGI0, SGI232127, SGI232127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232128, SGI0, SGI0, NULL};
const Segment * const SG51484[] = {SGI232129, SGI232131, SGI232133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232135, SGI0, SGI0, SGI0, SGI232137, SGI232137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51485[] = {SGI232130, SGI232132, SGI232134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232139, SGI232139, SGI0, SGI0, SGI0, SGI232141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51486[] = {SGI232136, SGI232138, SGI232140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232142, SGI0, SGI0, SGI0, SGI232143, SGI232143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232144, SGI0, SGI0, NULL};
const Segment * const SG51487[] = {SGI232145, SGI232146, SGI232147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232148, SGI232148, SGI0, SGI0, SGI0, SGI0, SGI232149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51488[] = {SGI232150, SGI0, SGI232151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232154, SGI232156, SGI232156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51489[] = {SGI232152, SGI232153, SGI232155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232157, SGI0, SGI0, SGI0, SGI232158, SGI232158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51490[] = {SGI232159, SGI232160, SGI232161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232162, SGI0, SGI0, SGI0, SGI232163, SGI232163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51491[] = {SGI232164, SGI232165, SGI232166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232167, SGI232167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232168, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51492[] = {SGI232169, SGI0, SGI232170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232171, SGI232172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232173, SGI0, SGI0, NULL};
const Segment * const SG51493[] = {SGI232174, SGI232175, SGI232176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232177, SGI232177, SGI0, SGI0, SGI0, SGI232180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51494[] = {SGI232178, SGI232179, SGI232181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232182, SGI0, SGI0, SGI0, SGI232183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51495[] = {SGI232184, SGI232185, SGI232186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232187, SGI0, SGI0, SGI0, SGI232188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51496[] = {SGI232189, SGI232190, SGI232191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232194, SGI0, SGI0, SGI0, SGI232196, SGI232196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51497[] = {SGI232192, SGI232193, SGI232195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232197, SGI232197, SGI0, SGI0, SGI0, SGI232198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51498[] = {SGI232199, SGI0, SGI232200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232201, SGI232202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51499[] = {SGI232203, SGI0, SGI232204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232205, SGI232206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51500[] = {SGI232207, SGI0, SGI232208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232209, SGI232210, SGI232210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232211, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51501[] = {SGI232212, SGI0, SGI232213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232214, SGI232215, SGI232215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51502[] = {SGI232216, SGI0, SGI232217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232218, SGI232219, SGI232219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51503[] = {SGI232220, SGI0, SGI232221, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51504[] = {SGI232222, SGI232223, SGI232224, SGI232225, SGI232226, SGI232226, SGI0, NULL};
const Segment * const SG51505[] = {SGI232227, SGI232228, SGI232229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232230, SGI0, SGI0, SGI0, SGI232231, SGI232231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51506[] = {SGI232232, SGI0, SGI232233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232234, SGI232235, SGI232235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51507[] = {SGI232236, SGI232237, SGI232238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232239, SGI232239, SGI0, SGI0, SGI0, SGI232240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51508[] = {SGI232241, SGI232242, SGI232243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232244, SGI232244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232245, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51509[] = {SGI232246, SGI232247, SGI232248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232249, SGI0, SGI0, SGI0, SGI232250, SGI232250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51510[] = {SGI232251, SGI232252, SGI232253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232254, SGI232254, SGI0, SGI0, SGI0, SGI232255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51511[] = {SGI232256, SGI232257, SGI232259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232263, SGI232263, SGI0, SGI0, SGI0, SGI232265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51512[] = {SGI232258, SGI232260, SGI232261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232262, SGI0, SGI0, SGI0, SGI232264, SGI232264, SGI0, SGI232266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51513[] = {SGI232267, SGI232268, SGI232269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232270, SGI0, SGI0, SGI0, SGI232272, SGI232272, SGI0, SGI232275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51514[] = {SGI232271, SGI232273, SGI232274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232276, SGI232276, SGI0, SGI0, SGI0, SGI232277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51515[] = {SGI232278, SGI232279, SGI232280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232281, SGI0, SGI0, SGI0, SGI232282, SGI232282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51516[] = {SGI232283, SGI232284, SGI232285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232286, SGI232286, SGI0, SGI0, SGI0, SGI232287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51517[] = {SGI232288, SGI232289, SGI232290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232291, SGI0, SGI0, SGI0, SGI232292, SGI232292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232293, SGI0, SGI0, NULL};
const Segment * const SG51518[] = {SGI232294, SGI0, SGI232295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232296, SGI232297, SGI232297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232298, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51519[] = {SGI232299, SGI232300, SGI232301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232302, SGI0, SGI0, SGI0, SGI232303, SGI232303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51520[] = {SGI232304, SGI232305, SGI232306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232307, SGI0, SGI0, SGI0, SGI232308, SGI232308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51521[] = {SGI232309, SGI232310, SGI232311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232312, SGI0, SGI0, SGI0, SGI232313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51522[] = {SGI232314, SGI0, SGI232315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232316, SGI232317, SGI232317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51523[] = {SGI232318, SGI0, SGI232319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232320, SGI232321, SGI232321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51524[] = {SGI232322, SGI0, SGI232323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232324, SGI232325, SGI232325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51525[] = {SGI232326, SGI232327, SGI232328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232329, SGI0, SGI0, SGI0, SGI232330, SGI232330, SGI0, SGI0, SGI0, SGI232331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51526[] = {SGI232332, SGI232333, SGI232334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232335, SGI0, SGI0, SGI0, SGI232337, SGI0, SGI232340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51527[] = {SGI232336, SGI232338, SGI232339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232341, SGI0, SGI0, SGI0, SGI232342, SGI232342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51528[] = {SGI232344, SGI232345, SGI232346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232347, SGI0, SGI0, SGI0, SGI232348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232349, SGI0, SGI0, NULL};
const Segment * const SG51529[] = {SGI232350, SGI232351, SGI232352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232353, SGI232353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232354, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51530[] = {SGI232355, SGI0, SGI232356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232359, SGI232360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232361, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51531[] = {SGI232357, SGI0, SGI232358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232362, SGI232363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51532[] = {SGI232364, SGI0, SGI232365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232366, SGI232367, SGI232367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51533[] = {SGI232368, SGI232369, SGI232370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232371, SGI232371, SGI0, SGI0, SGI0, SGI232372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51534[] = {SGI232373, SGI0, SGI232374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232375, SGI232376, SGI232376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51535[] = {SGI232377, SGI0, SGI232378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232379, SGI232380, SGI232380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51536[] = {SGI232382, SGI232384, SGI232386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232387, SGI0, SGI0, SGI0, SGI232389, SGI232389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51537[] = {SGI232381, SGI232383, SGI232385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232388, SGI232388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232390, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51538[] = {SGI232391, SGI0, SGI232392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232393, SGI232394, SGI232394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51539[] = {SGI232395, SGI232396, SGI232397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232398, SGI0, SGI0, SGI0, SGI232399, SGI232399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232400, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51540[] = {SGI232401, SGI232402, SGI232403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232404, SGI0, SGI0, SGI0, SGI232405, SGI232405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51541[] = {SGI232406, SGI0, SGI232407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232408, SGI232409, SGI232409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51542[] = {SGI232410, SGI0, SGI232411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232412, SGI232413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51543[] = {SGI232414, SGI0, SGI232415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232416, SGI232417, SGI232417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232418, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51544[] = {SGI232419, SGI0, SGI232420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232421, SGI232422, SGI232422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232423, SGI0, SGI0, NULL};
const Segment * const SG51545[] = {SGI232424, SGI232425, SGI232426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232427, SGI0, SGI0, SGI0, SGI232428, SGI232428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232429, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51546[] = {SGI232430, SGI0, SGI232431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232432, SGI232433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51547[] = {SGI232434, SGI0, SGI232435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232436, SGI232437, SGI232437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51548[] = {SGI232438, SGI0, SGI232439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232440, SGI232441, SGI232441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51549[] = {SGI232442, SGI0, SGI232443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232444, SGI232445, SGI232445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232446, SGI0, SGI0, NULL};
const Segment * const SG51550[] = {SGI232447, SGI232448, SGI232449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232450, SGI232450, SGI0, SGI0, SGI0, SGI0, SGI232451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51551[] = {SGI232452, SGI0, SGI232453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232454, SGI232455, SGI232455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51552[] = {SGI232456, SGI0, SGI232457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232458, SGI232459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51553[] = {SGI232460, SGI232461, SGI232462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232463, SGI232463, SGI0, SGI0, SGI0, SGI232464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51554[] = {SGI232465, SGI0, SGI232466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232467, SGI232468, SGI232468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51555[] = {SGI232469, SGI0, SGI232470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232471, SGI232472, SGI232472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51556[] = {SGI232473, SGI0, SGI232474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232477, SGI232478, SGI232478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232481, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51557[] = {SGI232475, SGI0, SGI232476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232479, SGI232480, SGI232480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51558[] = {SGI232483, SGI0, SGI232485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232487, SGI232489, SGI232489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51559[] = {SGI232482, SGI0, SGI232484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232486, SGI232488, SGI232488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51560[] = {SGI232490, SGI0, SGI232491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232492, SGI232493, SGI232493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51561[] = {SGI232494, SGI0, SGI232495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232496, SGI232497, SGI232497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51562[] = {SGI232498, SGI232499, SGI232500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232501, SGI0, SGI0, SGI0, SGI232502, SGI0, SGI0, SGI232503, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51563[] = {SGI232504, SGI232505, SGI232506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232507, SGI0, SGI0, SGI0, SGI232508, SGI0, SGI0, SGI232509, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51564[] = {SGI232510, SGI0, SGI232512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232515, SGI232516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51565[] = {SGI232511, SGI232513, SGI232514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232517, SGI232517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232518, NULL};
const Segment * const SG51566[] = {SGI232519, SGI0, SGI232520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232521, SGI232522, SGI232522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51567[] = {SGI232523, SGI0, SGI232524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232525, SGI232526, SGI232526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51568[] = {SGI232527, SGI0, SGI232528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232529, SGI232530, SGI232530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51569[] = {SGI232531, SGI0, SGI232532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232533, SGI232534, SGI232534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51570[] = {SGI232535, SGI0, SGI232536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232537, SGI232538, SGI232538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51571[] = {SGI232539, SGI232540, SGI232541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232542, SGI0, SGI0, SGI0, SGI232544, SGI232544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51572[] = {SGI232543, SGI0, SGI232545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232546, SGI232547, SGI232547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51573[] = {SGI232548, SGI232549, SGI232550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232551, SGI0, SGI0, SGI0, SGI232552, SGI232552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232553, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51574[] = {SGI232554, SGI232555, SGI232556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232557, SGI232557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232558, NULL};
const Segment * const SG51575[] = {SGI232559, SGI232560, SGI232561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232562, SGI0, SGI0, SGI0, SGI232563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51576[] = {SGI232564, SGI232565, SGI232566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232567, SGI232567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232568, NULL};
const Segment * const SG51577[] = {SGI232569, SGI232570, SGI232571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232573, SGI0, SGI0, SGI0, SGI232575, SGI232575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232577, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51578[] = {SGI232572, SGI0, SGI232574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232576, SGI232578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51579[] = {SGI232579, SGI232580, SGI232581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232582, SGI0, SGI0, SGI0, SGI232583, SGI232583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51580[] = {SGI232584, SGI0, SGI232585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232586, SGI232587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51581[] = {SGI232588, SGI232589, SGI232590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232591, SGI232591, SGI0, SGI0, SGI0, SGI0, SGI232592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51582[] = {SGI232593, SGI232594, SGI232595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232596, SGI0, SGI0, SGI0, SGI232597, SGI232597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51583[] = {SGI232598, SGI232599, SGI232600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232604, SGI0, SGI0, SGI0, SGI232605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51584[] = {SGI232601, SGI232602, SGI232603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232606, SGI0, SGI0, SGI0, SGI232607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51585[] = {SGI232608, SGI0, SGI232609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232610, SGI232611, SGI232611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51586[] = {SGI232612, SGI232613, SGI232614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232615, SGI0, SGI0, SGI0, SGI232616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51587[] = {SGI232617, SGI0, SGI232618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232619, SGI232620, SGI232620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51588[] = {SGI232621, SGI0, SGI232622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232623, SGI232624, SGI232624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51589[] = {SGI232625, SGI0, SGI232626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232627, SGI0, SGI0, SGI0, SGI232628, SGI232628, SGI0, SGI0, SGI0, SGI232629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51590[] = {SGI232630, SGI232631, SGI232632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232633, SGI0, SGI0, SGI0, SGI232634, SGI232635, SGI0, SGI0, SGI0, SGI0, SGI232636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51591[] = {SGI232639, SGI232641, SGI232642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232645, SGI0, SGI0, SGI0, SGI232646, SGI232646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51592[] = {SGI232637, SGI232638, SGI232640, SGI0, SGI0, SGI0, SGI232643, SGI232644, SGI0, SGI232647, SGI0, SGI0, SGI0, SGI232648, SGI232648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51593[] = {SGI232649, SGI0, SGI232650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232651, SGI232652, SGI232652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51594[] = {SGI232653, SGI0, SGI232654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232655, SGI232656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51595[] = {SGI232657, SGI232658, SGI232659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232660, SGI0, SGI0, SGI0, SGI232661, SGI232661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51596[] = {SGI232662, SGI0, SGI232663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232664, SGI232665, SGI232665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51597[] = {SGI232666, SGI232667, SGI232668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232669, SGI0, SGI0, SGI0, SGI232670, SGI232670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51598[] = {SGI232671, SGI232672, SGI232673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232674, SGI0, SGI0, SGI0, SGI232675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51599[] = {SGI232677, SGI232678, SGI232679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232680, SGI0, SGI0, SGI0, SGI232681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51600[] = {SGI232683, SGI232684, SGI232685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232686, SGI0, SGI0, SGI0, SGI232687, SGI232687, SGI0, SGI232688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51601[] = {SGI232689, SGI232690, SGI232691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232692, SGI0, SGI0, SGI0, SGI232693, SGI232693, SGI0, SGI232694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51602[] = {SGI232695, SGI232696, SGI232697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232698, SGI0, SGI0, SGI0, SGI232699, SGI0, SGI0, SGI232700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51603[] = {SGI232701, SGI232702, SGI232703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232704, SGI0, SGI0, SGI0, SGI232705, SGI232705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51604[] = {SGI232707, SGI0, SGI232708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232709, SGI0, SGI0, SGI0, SGI232710, SGI232710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232711, NULL};
const Segment * const SG51605[] = {SGI232712, SGI232713, SGI232714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232715, SGI232715, SGI0, SGI0, SGI0, SGI232716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51606[] = {SGI232717, SGI0, SGI232718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232719, SGI232720, SGI232720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51607[] = {SGI232721, SGI232722, SGI232723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232724, SGI0, SGI0, SGI0, SGI232725, SGI232725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232726, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51608[] = {SGI232727, SGI0, SGI232728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232729, SGI232730, SGI232730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51609[] = {SGI232731, SGI0, SGI232732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232733, SGI232734, SGI232734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51610[] = {SGI232735, SGI0, SGI232736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232737, SGI232738, SGI232738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51611[] = {SGI232739, SGI232740, SGI232741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232742, SGI0, SGI0, SGI0, SGI232743, SGI232743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232744, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51612[] = {SGI232745, SGI0, SGI232746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232747, SGI232748, SGI232748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51613[] = {SGI232749, SGI232750, SGI232751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232752, SGI0, SGI0, SGI0, SGI232753, SGI232753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51614[] = {SGI232754, SGI232755, SGI232756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232757, SGI0, SGI0, SGI0, SGI232758, SGI232758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51615[] = {SGI232760, SGI0, SGI232761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232762, SGI232763, SGI232763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51616[] = {SGI232764, SGI232765, SGI232766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232767, SGI0, SGI0, SGI0, SGI232768, SGI232768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232769, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51617[] = {SGI232770, SGI232771, SGI232772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232773, SGI0, SGI0, SGI0, SGI232774, SGI232774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232775, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51618[] = {SGI232776, SGI0, SGI232778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232780, SGI232782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51619[] = {SGI232777, SGI0, SGI232779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232781, SGI0, SGI0, SGI0, SGI232783, SGI232783, SGI0, SGI0, SGI0, SGI232784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51620[] = {SGI232785, SGI0, SGI232786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232787, SGI0, SGI0, SGI0, SGI232788, SGI232788, SGI0, SGI0, SGI0, SGI232789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51621[] = {SGI232790, SGI0, SGI232791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232792, SGI232793, SGI232793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232794, SGI0, SGI0, NULL};
const Segment * const SG51622[] = {SGI232795, SGI0, SGI232796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232797, SGI232798, SGI232798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51623[] = {SGI232799, SGI0, SGI232800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232802, SGI232803, SGI232803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232805, SGI0, SGI0, NULL};
const Segment * const SG51624[] = {SGI232801, SGI0, SGI232804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232806, SGI232807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51625[] = {SGI232808, SGI0, SGI232809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232810, SGI232811, SGI232811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51626[] = {SGI232812, SGI0, SGI232813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232814, SGI232815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51627[] = {SGI232816, SGI232817, SGI232818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232819, SGI232819, SGI0, SGI0, SGI0, SGI0, SGI232820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51628[] = {SGI232821, SGI0, SGI232822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232823, SGI232824, SGI232824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51629[] = {SGI232825, SGI0, SGI232826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232827, SGI232828, SGI232828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51630[] = {SGI232829, SGI232830, SGI232831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232832, SGI232832, SGI0, SGI0, SGI0, SGI232833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51631[] = {SGI232834, SGI0, SGI232835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232836, SGI232837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51632[] = {SGI232838, SGI0, SGI232839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232840, SGI232841, SGI232841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51633[] = {SGI232842, SGI0, SGI232843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232844, SGI232845, SGI232845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51634[] = {SGI232846, SGI0, SGI232847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232848, SGI232850, SGI232850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51635[] = {SGI232849, SGI0, SGI232851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232852, SGI232853, SGI232853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232854, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51636[] = {SGI232855, SGI0, SGI232856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232857, SGI232858, SGI232858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51637[] = {SGI232859, SGI0, SGI232860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232861, SGI232862, SGI232862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51638[] = {SGI232863, SGI0, SGI232864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232865, SGI232866, SGI232866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51639[] = {SGI232867, SGI0, SGI232868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232869, SGI232870, SGI232870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51640[] = {SGI232871, SGI232872, SGI232873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232874, SGI0, SGI0, SGI0, SGI232875, SGI232875, SGI0, SGI0, SGI232876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51641[] = {SGI232877, SGI232878, SGI232879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232880, SGI0, SGI0, SGI0, SGI232881, SGI232881, SGI0, SGI232882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51642[] = {SGI232883, SGI0, SGI232884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232885, SGI232886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51643[] = {SGI232887, SGI232888, SGI232889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232890, SGI0, SGI0, SGI0, SGI232891, SGI232891, SGI0, SGI232892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51644[] = {SGI232893, SGI232894, SGI232895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232896, SGI232896, SGI0, SGI0, SGI0, SGI232897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51645[] = {SGI232898, SGI0, SGI232899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232900, SGI232901, SGI232901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51646[] = {SGI232902, SGI0, SGI232904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232907, SGI232909, SGI232909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51647[] = {SGI232903, SGI232905, SGI232906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232908, SGI0, SGI0, SGI0, SGI232910, SGI232910, SGI0, SGI232911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51648[] = {SGI232912, SGI0, SGI232913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232914, SGI232915, SGI232915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51649[] = {SGI232916, SGI0, SGI232917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232918, SGI232919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51650[] = {SGI232920, SGI232921, SGI232922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232923, SGI0, SGI0, SGI0, SGI232924, SGI232924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232925, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51651[] = {SGI232926, SGI0, SGI232927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232928, SGI232929, SGI232929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51652[] = {SGI232930, SGI232931, SGI232932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232933, SGI0, SGI0, SGI0, SGI232934, SGI232934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232935, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51653[] = {SGI232936, SGI0, SGI232937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232938, SGI232939, SGI232939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51654[] = {SGI232940, SGI0, SGI232941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232942, SGI232943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51655[] = {SGI232944, SGI0, SGI232945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232946, SGI232947, SGI232947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51656[] = {SGI232948, SGI0, SGI232951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232953, SGI232955, SGI232955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51657[] = {SGI232949, SGI232950, SGI232952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232954, SGI0, SGI0, SGI0, SGI232956, SGI232956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232957, NULL};
const Segment * const SG51658[] = {SGI232958, SGI232959, SGI232960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232961, SGI0, SGI0, SGI0, SGI232962, SGI232962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232964, SGI0, NULL};
const Segment * const SG51659[] = {SGI232965, SGI232966, SGI232967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232968, SGI0, SGI0, SGI0, SGI232969, SGI232969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232971, SGI0, NULL};
const Segment * const SG51660[] = {SGI232972, SGI232973, SGI232974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232975, SGI0, SGI0, SGI0, SGI232976, SGI232976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51661[] = {SGI232977, SGI0, SGI232978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51662[] = {SGI232979, SGI232980, SGI232981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232982, SGI232982, SGI0, SGI0, SGI0, SGI232983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51663[] = {SGI232984, SGI232985, SGI232986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232987, SGI0, SGI0, SGI0, SGI232988, SGI232988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51664[] = {SGI232989, SGI232990, SGI232991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232992, SGI232992, SGI0, SGI0, SGI0, SGI232993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51665[] = {SGI232994, SGI232995, SGI232996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI232997, SGI0, SGI0, SGI0, SGI232998, SGI232998, SGI0, SGI0, SGI0, SGI232999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51666[] = {SGI233000, SGI0, SGI233001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233002, SGI233003, SGI233003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51667[] = {SGI233004, SGI233005, SGI233006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233007, SGI0, SGI0, SGI0, SGI233008, SGI233008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51668[] = {SGI233009, SGI0, SGI233010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233011, SGI233012, SGI233012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51669[] = {SGI233013, SGI0, SGI233014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233015, SGI233016, SGI233016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51670[] = {SGI233017, SGI0, SGI233018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233020, SGI233021, SGI233021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51671[] = {SGI233019, SGI0, SGI233022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233023, SGI233024, SGI233024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51672[] = {SGI233025, SGI233026, SGI233027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233028, SGI233028, SGI0, SGI0, SGI0, SGI233029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51673[] = {SGI233030, SGI233031, SGI233032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233033, SGI0, SGI0, SGI0, SGI233034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51674[] = {SGI233035, SGI233036, SGI233037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233038, SGI0, SGI0, SGI0, SGI233039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51675[] = {SGI233041, SGI0, SGI233042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233045, SGI0, SGI0, SGI0, SGI233046, SGI233046, SGI0, SGI0, SGI0, SGI233047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51676[] = {SGI233043, SGI0, SGI233044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233048, SGI233049, SGI233049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51677[] = {SGI233050, SGI0, SGI233051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233052, SGI233053, SGI233053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51678[] = {SGI233054, SGI0, SGI233055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233056, SGI233057, SGI233057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51679[] = {SGI233058, SGI233059, SGI233060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233061, SGI0, SGI0, SGI0, SGI233062, SGI233062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51680[] = {SGI233063, SGI0, SGI233064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233065, SGI233066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51681[] = {SGI233067, SGI0, SGI233068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233069, SGI233070, SGI233070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51682[] = {SGI233071, SGI233072, SGI233073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233074, SGI233074, SGI0, SGI0, SGI0, SGI233075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51683[] = {SGI233076, SGI0, SGI233077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233078, SGI233079, SGI233079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51684[] = {SGI233080, SGI0, SGI233081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233082, SGI233083, SGI233083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233084, SGI0, SGI0, NULL};
const Segment * const SG51685[] = {SGI233085, SGI233086, SGI233087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233088, SGI0, SGI0, SGI0, SGI233089, SGI233089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233090, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51686[] = {SGI233091, SGI0, SGI233092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233093, SGI233094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51687[] = {SGI233095, SGI0, SGI233096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233097, SGI233098, SGI233098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51688[] = {SGI233099, SGI0, SGI233100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233101, SGI233102, SGI233102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51689[] = {SGI233103, SGI0, SGI233104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233105, SGI233106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51690[] = {SGI233107, SGI0, SGI233108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233109, SGI233110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51691[] = {SGI233111, SGI233112, SGI233113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233114, SGI233114, SGI0, SGI0, SGI0, SGI233115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51692[] = {SGI233116, SGI0, SGI233117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233118, SGI0, SGI0, SGI0, SGI233119, SGI233119, SGI0, SGI0, SGI0, SGI233120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51693[] = {SGI233121, SGI233122, SGI233123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233126, SGI0, SGI0, SGI0, SGI233127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51694[] = {SGI233124, SGI0, SGI233125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233128, SGI233129, SGI233129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51695[] = {SGI233130, SGI233131, SGI233132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233133, SGI233133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233134, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51696[] = {SGI233135, SGI233136, SGI233137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233138, SGI0, SGI0, SGI0, SGI233139, SGI233139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233140, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51697[] = {SGI233141, SGI233142, SGI233143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233144, SGI233144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233145, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51698[] = {SGI233146, SGI233147, SGI233148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233149, SGI0, SGI0, SGI0, SGI233150, SGI233150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233151, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51699[] = {SGI233152, SGI233153, SGI233154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233155, SGI233155, SGI0, SGI0, SGI0, SGI233156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51700[] = {SGI233157, SGI233158, SGI233159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233160, SGI0, SGI0, SGI0, SGI233161, SGI233161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233162, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51701[] = {SGI233163, SGI233164, SGI233165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233166, SGI0, SGI0, SGI0, SGI233167, SGI233167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51702[] = {SGI233168, SGI0, SGI233169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233170, SGI233171, SGI233171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51703[] = {SGI233172, SGI0, SGI233173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233174, SGI233175, SGI233175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51704[] = {SGI233176, SGI233177, SGI233178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233179, SGI0, SGI0, SGI0, SGI233180, SGI233180, SGI0, SGI0, SGI233181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51705[] = {SGI233182, SGI233183, SGI233184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233185, SGI0, SGI0, SGI0, SGI233186, SGI233186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51706[] = {SGI233187, SGI233188, SGI233189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233190, SGI0, SGI0, SGI0, SGI233191, SGI233191, SGI0, SGI0, SGI233192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51707[] = {SGI233193, SGI233194, SGI233195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233196, SGI0, SGI0, SGI0, SGI233197, SGI233197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51708[] = {SGI233198, SGI233199, SGI233200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233201, SGI0, SGI0, SGI0, SGI233202, SGI233202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51709[] = {SGI233203, SGI0, SGI233204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233208, SGI233209, SGI233209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51710[] = {SGI233205, SGI233206, SGI233207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233210, SGI0, SGI0, SGI0, SGI233211, SGI233211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233212, SGI0, SGI0, NULL};
const Segment * const SG51711[] = {SGI233213, SGI233214, SGI233215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233216, SGI0, SGI0, SGI0, SGI233217, SGI233217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233218, SGI0, SGI0, NULL};
const Segment * const SG51712[] = {SGI233219, SGI0, SGI233220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233221, SGI233222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51713[] = {SGI233223, SGI233224, SGI233225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233226, SGI0, SGI0, SGI0, SGI233227, SGI233227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51714[] = {SGI233229, SGI0, SGI233230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233231, SGI0, SGI0, SGI0, SGI233232, SGI233232, SGI0, SGI0, SGI0, SGI233233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51715[] = {SGI233234, SGI0, SGI233235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233236, SGI233237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51716[] = {SGI233238, SGI0, SGI233239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233243, SGI233244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51717[] = {SGI233240, SGI233241, SGI233242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233245, SGI233245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233246, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51718[] = {SGI233247, SGI0, SGI233248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233251, SGI233253, SGI233253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51719[] = {SGI233249, SGI233250, SGI233252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233254, SGI0, SGI0, SGI0, SGI233255, SGI233255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233256, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51720[] = {SGI233257, SGI0, SGI233258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233259, SGI233260, SGI233260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51721[] = {SGI233262, SGI0, SGI233264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233265, SGI233267, SGI233267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51722[] = {SGI233261, SGI0, SGI233263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233266, SGI233268, SGI233268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51723[] = {SGI233269, SGI0, SGI233270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233273, SGI233274, SGI233274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51724[] = {SGI233271, SGI0, SGI233272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233275, SGI233276, SGI233276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51725[] = {SGI233277, SGI233278, SGI233279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233280, SGI0, SGI0, SGI0, SGI233281, SGI233281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51726[] = {SGI233282, SGI0, SGI233284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233285, SGI233286, SGI233286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51727[] = {SGI233287, SGI0, SGI233288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233289, SGI233291, SGI233291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51728[] = {SGI233290, SGI0, SGI233292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233293, SGI233294, SGI233294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233295, SGI0, SGI0, NULL};
const Segment * const SG51729[] = {SGI233296, SGI233298, SGI233299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233303, SGI233303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233305, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51730[] = {SGI233297, SGI233300, SGI233301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233302, SGI0, SGI0, SGI0, SGI233304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51731[] = {SGI233307, SGI0, SGI233308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233309, SGI233310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233311, SGI0, NULL};
const Segment * const SG51732[] = {SGI233312, SGI0, SGI233313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233314, SGI233315, SGI233315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51733[] = {SGI233316, SGI233317, SGI233318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233319, SGI0, SGI0, SGI0, SGI233320, SGI233320, SGI0, SGI0, SGI0, SGI0, SGI233321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51734[] = {SGI233322, SGI233323, SGI233324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233325, SGI233325, SGI0, SGI0, SGI0, SGI0, SGI233327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51735[] = {SGI233326, SGI233328, SGI233329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233330, SGI233330, SGI0, SGI0, SGI0, SGI233331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51736[] = {SGI233332, SGI0, SGI233333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233334, SGI233335, SGI233335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51737[] = {SGI233336, SGI233337, SGI233338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233339, SGI0, SGI0, SGI0, SGI233340, SGI233340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51738[] = {SGI233341, SGI0, SGI233343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233345, SGI0, SGI0, SGI0, SGI233346, SGI233346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51739[] = {SGI233342, SGI0, SGI233344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233347, SGI233348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51740[] = {SGI233349, SGI0, SGI233350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233351, SGI233352, SGI233352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51741[] = {SGI233353, SGI0, SGI233354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233355, SGI233356, SGI233356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51742[] = {SGI233357, SGI233358, SGI233359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233360, SGI233360, SGI0, SGI0, SGI0, SGI233361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51743[] = {SGI233362, SGI233363, SGI233364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233365, SGI233365, SGI0, SGI0, SGI0, SGI233366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51744[] = {SGI233367, SGI0, SGI233368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233369, SGI233370, SGI233370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233371, SGI0, SGI0, NULL};
const Segment * const SG51745[] = {SGI233372, SGI0, SGI233373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233374, SGI233375, SGI233375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233376, SGI0, SGI0, NULL};
const Segment * const SG51746[] = {SGI233377, SGI0, SGI233378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233379, SGI233380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233381, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51747[] = {SGI233382, SGI233383, SGI233384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233385, SGI0, SGI0, SGI0, SGI233386, SGI233387, SGI0, SGI0, SGI233388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51748[] = {SGI233389, SGI0, SGI233390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233391, SGI233392, SGI233392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233393, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51749[] = {SGI233394, SGI233395, SGI233396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233397, SGI233397, SGI0, SGI0, SGI233398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51750[] = {SGI233399, SGI0, SGI233400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233401, SGI233402, SGI233402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51751[] = {SGI233403, SGI0, SGI233404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233405, SGI233406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51752[] = {SGI233407, SGI233408, SGI233409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233410, SGI233410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233411, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51753[] = {SGI233412, SGI0, SGI233413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233414, SGI233415, SGI233415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51754[] = {SGI233416, SGI0, SGI233417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233418, SGI233419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51755[] = {SGI233420, SGI0, SGI233421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233422, SGI233423, SGI233423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51756[] = {SGI233424, SGI0, SGI233425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233426, SGI233427, SGI233427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51757[] = {SGI233428, SGI0, SGI233429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233430, SGI0, SGI0, SGI0, SGI233431, SGI233431, SGI0, SGI0, SGI0, SGI0, SGI233432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51758[] = {SGI233433, SGI0, SGI233434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233435, SGI233436, SGI233436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51759[] = {SGI233437, SGI0, SGI233438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233439, SGI233440, SGI233440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51760[] = {SGI233441, SGI233442, SGI233443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233444, SGI0, SGI0, SGI0, SGI233445, SGI233445, SGI0, SGI233446, SGI0, SGI0, SGI0, SGI0, SGI233447, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51761[] = {SGI233448, SGI0, SGI233449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233453, SGI0, SGI0, SGI0, SGI233454, SGI233454, SGI0, SGI0, SGI0, SGI0, SGI233456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51762[] = {SGI233450, SGI233451, SGI233452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233455, SGI0, SGI0, SGI0, SGI233457, SGI233457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51763[] = {SGI233458, SGI233459, SGI233460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233461, SGI0, SGI0, SGI0, SGI233462, SGI233462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51764[] = {SGI233463, SGI233464, SGI233465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233466, SGI0, SGI0, SGI0, SGI233467, SGI233467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51765[] = {SGI233468, SGI233469, SGI233470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233471, SGI0, SGI0, SGI0, SGI233472, SGI233472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51766[] = {SGI233473, SGI233474, SGI233475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233476, SGI233476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233477, NULL};
const Segment * const SG51767[] = {SGI233478, SGI0, SGI233479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233482, SGI233484, SGI233484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51768[] = {SGI233480, SGI233481, SGI233483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233485, SGI0, SGI0, SGI0, SGI233486, SGI233486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51769[] = {SGI233487, SGI233488, SGI233489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233490, SGI0, SGI0, SGI0, SGI233491, SGI233491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51770[] = {SGI233492, SGI0, SGI233493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233494, SGI233495, SGI233495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233496, SGI0, SGI0, NULL};
const Segment * const SG51771[] = {SGI233497, SGI0, SGI233498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233501, SGI233502, SGI233502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51772[] = {SGI233499, SGI0, SGI233500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233503, SGI233504, SGI233504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51773[] = {SGI233505, SGI233506, SGI233507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233508, SGI0, SGI0, SGI0, SGI233509, SGI233509, SGI0, SGI233510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51774[] = {SGI233511, SGI0, SGI233512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233513, SGI233514, SGI233514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51775[] = {SGI233515, SGI0, SGI233516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233517, SGI0, SGI0, SGI0, SGI233518, SGI233518, SGI0, SGI0, SGI0, SGI233519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51776[] = {SGI233520, SGI233521, SGI233522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233523, SGI0, SGI0, SGI0, SGI233524, SGI0, SGI0, SGI233526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51777[] = {SGI233525, SGI233527, SGI233528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233529, SGI0, SGI0, SGI0, SGI233530, SGI233530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51778[] = {SGI233531, SGI233532, SGI233533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233534, SGI0, SGI0, SGI0, SGI233535, SGI233535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51779[] = {SGI233536, SGI0, SGI233537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233538, SGI233539, SGI233539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51780[] = {SGI233540, SGI233541, SGI233542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233543, SGI0, SGI0, SGI0, SGI233544, SGI233544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51781[] = {SGI233545, SGI233546, SGI233547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233548, SGI0, SGI0, SGI0, SGI233549, SGI233549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51782[] = {SGI233550, SGI0, SGI233551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233552, SGI0, SGI0, SGI0, SGI233553, SGI233553, SGI0, SGI0, SGI0, SGI233554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51783[] = {SGI233555, SGI0, SGI233557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233559, SGI233560, SGI233560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51784[] = {SGI233556, SGI0, SGI233558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233561, SGI233562, SGI233562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51785[] = {SGI233563, SGI0, SGI233564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233565, SGI233566, SGI233566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51786[] = {SGI233567, SGI0, SGI233568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233569, SGI233570, SGI233570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51787[] = {SGI233571, SGI0, SGI233572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233573, SGI233574, SGI233574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51788[] = {SGI233575, SGI233577, SGI233578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233581, SGI0, SGI0, SGI0, SGI233582, SGI233582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51789[] = {SGI233576, SGI233579, SGI233580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233583, SGI0, SGI0, SGI0, SGI233584, SGI0, SGI233587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51790[] = {SGI233585, SGI0, SGI233586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233588, SGI233589, SGI233589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51791[] = {SGI233590, SGI0, SGI233591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233592, SGI233593, SGI233593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51792[] = {SGI233594, SGI233595, SGI233596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233597, SGI0, SGI0, SGI0, SGI233599, SGI233599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233602, NULL};
const Segment * const SG51793[] = {SGI233598, SGI233600, SGI233601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233603, SGI0, SGI0, SGI0, SGI233604, SGI233604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233605, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51794[] = {SGI233606, SGI233607, SGI233608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233609, SGI0, SGI0, SGI0, SGI233610, SGI233610, SGI0, SGI0, SGI233611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51795[] = {SGI233612, SGI0, SGI233613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233614, SGI0, SGI0, SGI0, SGI233615, SGI233615, SGI0, SGI0, SGI233616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51796[] = {SGI233621, SGI0, SGI233623, SGI0, SGI233624, SGI233624, SGI0, NULL};
const Segment * const SG51797[] = {SGI233617, SGI233618, SGI233619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233620, SGI0, SGI0, SGI0, SGI233622, SGI233622, SGI0, SGI0, SGI233625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51798[] = {SGI233626, SGI0, SGI233627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233628, SGI233629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51799[] = {SGI233630, SGI0, SGI233631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233634, SGI233636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51800[] = {SGI233632, SGI0, SGI233633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233638, SGI233639, SGI233639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51801[] = {SGI233635, SGI0, SGI233637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233640, SGI0, SGI0, SGI0, SGI233641, SGI233641, SGI0, SGI0, SGI0, SGI233642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51802[] = {SGI233643, SGI0, SGI233644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233645, SGI0, SGI0, SGI0, SGI233646, SGI233646, SGI0, SGI0, SGI0, SGI233647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51803[] = {SGI233648, SGI233649, SGI233650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233651, SGI0, SGI0, SGI0, SGI233652, SGI233652, SGI0, SGI233653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51804[] = {SGI233656, SGI0, SGI233658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233660, SGI233662, SGI233662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51805[] = {SGI233654, SGI233655, SGI233657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233659, SGI0, SGI0, SGI0, SGI233661, SGI233661, SGI0, SGI233663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51806[] = {SGI233664, SGI233665, SGI233666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233667, SGI0, SGI0, SGI0, SGI233668, SGI233668, SGI0, SGI233669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51807[] = {SGI233670, SGI233671, SGI233672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233673, SGI0, SGI0, SGI0, SGI233674, SGI0, SGI0, SGI0, SGI233675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51808[] = {SGI233676, SGI0, SGI233677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233678, SGI233679, SGI233679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51809[] = {SGI233680, SGI0, SGI233681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233682, SGI0, SGI0, SGI0, SGI233683, SGI233683, SGI0, SGI0, SGI233684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51810[] = {SGI233685, SGI0, SGI233686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233689, SGI233690, SGI233690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51811[] = {SGI233687, SGI0, SGI233688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233691, SGI233692, SGI233692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51812[] = {SGI233693, SGI233694, SGI233695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233696, SGI0, SGI0, SGI0, SGI233697, SGI233697, SGI0, SGI0, SGI233700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51813[] = {SGI233698, SGI233699, SGI233701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233702, SGI0, SGI0, SGI0, SGI233703, SGI233703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233704, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51814[] = {SGI233705, SGI0, SGI233706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233707, SGI233708, SGI233708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51815[] = {SGI233709, SGI233710, SGI233711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233712, SGI0, SGI0, SGI0, SGI233713, SGI233713, SGI0, SGI233715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51816[] = {SGI233714, SGI0, SGI233716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233717, SGI233718, SGI233718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51817[] = {SGI233719, SGI233720, SGI233721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233722, SGI233722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233724, NULL};
const Segment * const SG51818[] = {SGI233723, SGI0, SGI233725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233726, SGI233727, SGI233727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51819[] = {SGI233728, SGI233729, SGI233730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233731, SGI233731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233732, NULL};
const Segment * const SG51820[] = {SGI233733, SGI0, SGI233734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233735, SGI233736, SGI233736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51821[] = {SGI233737, SGI233738, SGI233739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233741, SGI0, SGI0, SGI0, SGI233743, SGI233743, SGI0, SGI0, SGI0, SGI233744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51822[] = {SGI233740, SGI0, SGI233742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233745, SGI233746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51823[] = {SGI233747, SGI0, SGI233748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233749, SGI233750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51824[] = {SGI233752, SGI0, SGI233754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233756, SGI233757, SGI233757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51825[] = {SGI233751, SGI0, SGI233753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233755, SGI233758, SGI233758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51826[] = {SGI233759, SGI233760, SGI233761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233762, SGI233762, SGI0, SGI0, SGI0, SGI233763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51827[] = {SGI233764, SGI0, SGI233765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233766, SGI233767, SGI233767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51828[] = {SGI233768, SGI0, SGI233769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233770, SGI233771, SGI233771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51829[] = {SGI233772, SGI0, SGI233773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233774, SGI233775, SGI233775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51830[] = {SGI233776, SGI0, SGI233777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233778, SGI233779, SGI233779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233780, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51831[] = {SGI233781, SGI233782, SGI233783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233784, SGI0, SGI0, SGI0, SGI233785, SGI233785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233786, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51832[] = {SGI233787, SGI0, SGI233788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233789, SGI233790, SGI233790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233791, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51833[] = {SGI233792, SGI0, SGI233793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233794, SGI233795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51834[] = {SGI233796, SGI0, SGI233797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233798, SGI233799, SGI233799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233800, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51835[] = {SGI233801, SGI0, SGI233803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233805, SGI233806, SGI233806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51836[] = {SGI233802, SGI0, SGI233804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233807, SGI233808, SGI233808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233809, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51837[] = {SGI233810, SGI0, SGI233811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233812, SGI233813, SGI233813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51838[] = {SGI233814, SGI0, SGI233815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233816, SGI233817, SGI233817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51839[] = {SGI233818, SGI233819, SGI233820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233821, SGI0, SGI0, SGI0, SGI233822, SGI233822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233823, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51840[] = {SGI233824, SGI0, SGI233825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233826, SGI233827, SGI233827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51841[] = {SGI233828, SGI0, SGI233829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233830, SGI233831, SGI233831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51842[] = {SGI233832, SGI233833, SGI233834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233835, SGI0, SGI0, SGI0, SGI233836, SGI233836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51843[] = {SGI233837, SGI0, SGI233838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233839, SGI233840, SGI233840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51844[] = {SGI233841, SGI0, SGI233842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233843, SGI233844, SGI233844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51845[] = {SGI233845, SGI233846, SGI233847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233849, SGI0, SGI0, SGI0, SGI233851, SGI233851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51846[] = {SGI233848, SGI0, SGI233850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233853, SGI233854, SGI233854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51847[] = {SGI233855, SGI233856, SGI233857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233859, SGI0, SGI0, SGI0, SGI233861, SGI233861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233862, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51848[] = {SGI233858, SGI0, SGI233860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233863, SGI233864, SGI233864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51849[] = {SGI233865, SGI0, SGI233866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233867, SGI0, SGI0, SGI0, SGI233868, SGI233868, SGI0, SGI0, SGI0, SGI233869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51850[] = {SGI233870, SGI0, SGI233871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233872, SGI233873, SGI233873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51851[] = {SGI233874, SGI0, SGI233875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233876, SGI233877, SGI233877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51852[] = {SGI233878, SGI233879, SGI233880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233881, SGI0, SGI0, SGI0, SGI233882, SGI0, SGI0, SGI233883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51853[] = {SGI233884, SGI233885, SGI233886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233887, SGI0, SGI0, SGI0, SGI233888, SGI233888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233889, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51854[] = {SGI233890, SGI0, SGI233891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233893, SGI233895, SGI233895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51855[] = {SGI233892, SGI233894, SGI233896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233897, SGI0, SGI0, SGI0, SGI233898, SGI0, SGI0, SGI233899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51856[] = {SGI233900, SGI0, SGI233901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233902, SGI233903, SGI233903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51857[] = {SGI233904, SGI0, SGI233905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233906, SGI233907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51858[] = {SGI233908, SGI233909, SGI233910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233911, SGI0, SGI0, SGI0, SGI233912, SGI233912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233913, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51859[] = {SGI233914, SGI233915, SGI233916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233917, SGI0, SGI0, SGI0, SGI233918, SGI233918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233919, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51860[] = {SGI233920, SGI0, SGI233921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233923, SGI233924, SGI233924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233927, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51861[] = {SGI233922, SGI233925, SGI233926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233928, SGI0, SGI0, SGI0, SGI233929, SGI233929, SGI0, SGI0, SGI233930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51862[] = {SGI233931, SGI233932, SGI233933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233934, SGI233934, SGI0, SGI0, SGI0, SGI233935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51863[] = {SGI233936, SGI233937, SGI233938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233939, SGI233939, SGI0, SGI0, SGI233941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51864[] = {SGI233940, SGI0, SGI233942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233943, SGI233944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51865[] = {SGI233945, SGI0, SGI233946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233947, SGI233948, SGI233948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51866[] = {SGI233949, SGI233950, SGI233951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233952, SGI233952, SGI0, SGI0, SGI0, SGI0, SGI233953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51867[] = {SGI233954, SGI0, SGI233955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233956, SGI233957, SGI233957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233958, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51868[] = {SGI233959, SGI0, SGI233960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233966, SGI233967, SGI233967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51869[] = {SGI233961, SGI233962, SGI233964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233968, SGI233968, SGI0, SGI0, SGI0, SGI233971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51870[] = {SGI233963, SGI0, SGI233965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233969, SGI233970, SGI233970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51871[] = {SGI233972, SGI0, SGI233973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233974, SGI233975, SGI233975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51872[] = {SGI233976, SGI0, SGI233977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233978, SGI233979, SGI233979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51873[] = {SGI233980, SGI0, SGI233981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233984, SGI233985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51874[] = {SGI233982, SGI0, SGI233983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233986, SGI233987, SGI233987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51875[] = {SGI233988, SGI233989, SGI233990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233991, SGI0, SGI0, SGI0, SGI233992, SGI233992, SGI0, SGI233994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51876[] = {SGI233993, SGI0, SGI233995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI233996, SGI233997, SGI233997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51877[] = {SGI233998, SGI0, SGI233999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234000, SGI234001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51878[] = {SGI234002, SGI234003, SGI234004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234005, SGI234005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234006, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51879[] = {SGI234007, SGI234008, SGI234009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234010, SGI234010, SGI0, SGI0, SGI0, SGI234011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51880[] = {SGI234012, SGI0, SGI234013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234014, SGI234015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51881[] = {SGI234016, SGI0, SGI234017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234018, SGI234019, SGI234019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51882[] = {SGI234020, SGI234021, SGI234022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234025, SGI234025, SGI0, SGI0, SGI0, SGI234027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51883[] = {SGI234023, SGI0, SGI234024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234026, SGI234028, SGI234028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51884[] = {SGI234029, SGI0, SGI234030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234031, SGI234032, SGI234032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51885[] = {SGI234033, SGI234034, SGI234035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234036, SGI234036, SGI0, SGI0, SGI0, SGI0, SGI234037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51886[] = {SGI234038, SGI234039, SGI234040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234043, SGI234043, SGI0, SGI0, SGI0, SGI0, SGI234045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51887[] = {SGI234041, SGI0, SGI234042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234044, SGI234046, SGI234046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51888[] = {SGI234047, SGI0, SGI234048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234049, SGI234050, SGI234050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51889[] = {SGI234051, SGI0, SGI234053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234056, SGI234057, SGI234057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51890[] = {SGI234052, SGI234054, SGI234055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234058, SGI234058, SGI0, SGI0, SGI0, SGI234059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51891[] = {SGI234060, SGI234061, SGI234062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234063, SGI0, SGI0, SGI0, SGI234064, SGI234064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234065, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51892[] = {SGI234066, SGI234067, SGI234068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234069, SGI234069, SGI0, SGI0, SGI0, SGI234070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51893[] = {SGI234071, SGI234072, SGI234073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234075, SGI234075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234077, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51894[] = {SGI234074, SGI0, SGI234076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234078, SGI234079, SGI234079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51895[] = {SGI234080, SGI234081, SGI234082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234083, SGI234083, SGI0, SGI0, SGI0, SGI234084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51896[] = {SGI234085, SGI234086, SGI234087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234088, SGI234088, SGI0, SGI0, SGI0, SGI234089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51897[] = {SGI234090, SGI234091, SGI234092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234093, SGI234093, SGI0, SGI0, SGI0, SGI234094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51898[] = {SGI234095, SGI234096, SGI234097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234098, SGI234098, SGI0, SGI0, SGI0, SGI234099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51899[] = {SGI234100, SGI0, SGI234101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234102, SGI234103, SGI234103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51900[] = {SGI234104, SGI234105, SGI234106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234107, SGI234107, SGI0, SGI0, SGI0, SGI234108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51901[] = {SGI234109, SGI0, SGI234110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234111, SGI234113, SGI234113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51902[] = {SGI234112, SGI0, SGI234114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234115, SGI234116, SGI234116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51903[] = {SGI234117, SGI0, SGI234118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234119, SGI234120, SGI234120, SGI0, SGI0, NULL};
const Segment * const SG51904[] = {SGI234121, SGI0, SGI234122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234123, SGI234124, SGI234124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51905[] = {SGI234125, SGI234126, SGI234127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234128, SGI0, SGI0, SGI0, SGI234129, SGI234129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234130, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51906[] = {SGI234131, SGI234132, SGI234133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234134, SGI0, SGI0, SGI0, SGI234135, SGI234135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51907[] = {SGI234136, SGI234137, SGI234138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234142, SGI0, SGI0, SGI0, SGI234143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51908[] = {SGI234139, SGI234140, SGI234141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234144, SGI0, SGI0, SGI0, SGI234145, SGI234145, SGI0, SGI234146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51909[] = {SGI234148, SGI234150, SGI234152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234153, SGI0, SGI0, SGI0, SGI234155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51910[] = {SGI234147, SGI234149, SGI234151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234154, SGI0, SGI0, SGI0, SGI234156, SGI234156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234157, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51911[] = {SGI234158, SGI0, SGI234159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234160, SGI234161, SGI234161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51912[] = {SGI234162, SGI234163, SGI234164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234165, SGI234165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234166, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51913[] = {SGI234167, SGI234168, SGI234169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234170, SGI234170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234171, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51914[] = {SGI234172, SGI234173, SGI234174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234175, SGI234175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234176, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51915[] = {SGI234177, SGI234178, SGI234179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234180, SGI234180, SGI0, SGI0, SGI0, SGI234181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51916[] = {SGI234182, SGI0, SGI234183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234184, SGI234185, SGI234185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51917[] = {SGI234186, SGI234187, SGI234188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234189, SGI0, SGI0, SGI0, SGI234190, SGI234190, SGI0, SGI0, SGI0, SGI234191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51918[] = {SGI234192, SGI0, SGI234193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234194, SGI234195, SGI234195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234196, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51919[] = {SGI234197, SGI0, SGI234198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234199, SGI234200, SGI234200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51920[] = {SGI234201, SGI0, SGI234202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234203, SGI234204, SGI234204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51921[] = {SGI234205, SGI234206, SGI234208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234210, SGI234210, SGI0, SGI0, SGI234213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51922[] = {SGI234207, SGI0, SGI234209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234211, SGI234212, SGI234212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51923[] = {SGI234214, SGI234215, SGI234216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234217, SGI0, SGI0, SGI0, SGI234218, SGI234218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51924[] = {SGI234219, SGI234220, SGI234221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234222, SGI0, SGI0, SGI0, SGI234223, SGI234223, SGI234224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51925[] = {SGI234225, SGI0, SGI234226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234227, SGI0, SGI0, SGI0, SGI234228, SGI234228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234229, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51926[] = {SGI234230, SGI234232, SGI234233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234235, SGI0, SGI0, SGI0, SGI234237, SGI234237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51927[] = {SGI234231, SGI0, SGI234234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234236, SGI234238, SGI234238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51928[] = {SGI234239, SGI234240, SGI234241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234242, SGI0, SGI0, SGI0, SGI234243, SGI234243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234245, SGI0, NULL};
const Segment * const SG51929[] = {SGI234246, SGI0, SGI234247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234248, SGI234249, SGI234249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51930[] = {SGI234250, SGI0, SGI234251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234252, SGI234253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51931[] = {SGI234254, SGI0, SGI234255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234256, SGI234257, SGI234257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51932[] = {SGI234258, SGI0, SGI234259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234260, SGI234261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51933[] = {SGI234262, SGI0, SGI234263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234264, SGI234265, SGI234265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51934[] = {SGI234266, SGI0, SGI234267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234268, SGI234269, SGI234269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51935[] = {SGI234270, SGI234271, SGI234272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234273, SGI234273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234274, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51936[] = {SGI234275, SGI234276, SGI234277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234278, SGI234278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234279, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51937[] = {SGI234280, SGI234281, SGI234282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234283, SGI0, SGI0, SGI0, SGI234284, SGI234284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51938[] = {SGI234285, SGI234286, SGI234287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234288, SGI0, SGI0, SGI0, SGI234289, SGI234289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234290, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51939[] = {SGI234291, SGI0, SGI234292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234293, SGI234294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51940[] = {SGI234295, SGI0, SGI234296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234297, SGI234298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51941[] = {SGI234299, SGI0, SGI234300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234301, SGI234302, SGI234302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51942[] = {SGI234303, SGI234304, SGI234305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234306, SGI0, SGI0, SGI0, SGI234307, SGI234307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51943[] = {SGI234308, SGI234309, SGI234310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234311, SGI0, SGI0, SGI0, SGI234312, SGI234312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51944[] = {SGI234314, SGI0, SGI234316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51945[] = {SGI234313, SGI234315, SGI234317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234318, SGI234318, SGI0, SGI0, SGI0, SGI234319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51946[] = {SGI234320, SGI0, SGI234321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234322, SGI234323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51947[] = {SGI234324, SGI0, SGI234325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234326, SGI234327, SGI234327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51948[] = {SGI234328, SGI234329, SGI234330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234331, SGI234331, SGI0, SGI0, SGI0, SGI0, SGI234332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51949[] = {SGI234333, SGI0, SGI234334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234335, SGI234336, SGI234336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51950[] = {SGI234337, SGI234338, SGI234339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234340, SGI234340, SGI0, SGI0, SGI0, SGI234341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51951[] = {SGI234342, SGI0, SGI234344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234346, SGI234348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51952[] = {SGI234343, SGI0, SGI234345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234347, SGI234349, SGI234349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51953[] = {SGI234350, SGI0, SGI234351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234352, SGI234353, SGI234353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51954[] = {SGI234354, SGI0, SGI234355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234356, SGI234357, SGI234357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51955[] = {SGI234358, SGI0, SGI234359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234360, SGI234361, SGI234361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234362, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51956[] = {SGI234363, SGI234364, SGI234365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234366, SGI234366, SGI0, SGI0, SGI0, SGI234367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51957[] = {SGI234368, SGI234369, SGI234370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234371, SGI234371, SGI0, SGI0, SGI0, SGI234372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51958[] = {SGI234373, SGI0, SGI234374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234375, SGI234376, SGI234376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51959[] = {SGI234377, SGI0, SGI234378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234379, SGI234380, SGI234380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51960[] = {SGI234381, SGI0, SGI234382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234383, SGI234384, SGI234384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234385, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51961[] = {SGI234386, SGI234387, SGI234388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234389, SGI234389, SGI0, SGI0, SGI234391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51962[] = {SGI234390, SGI0, SGI234392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234393, SGI234394, SGI234394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234395, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51963[] = {SGI234396, SGI0, SGI234397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234398, SGI234399, SGI234399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234400, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51964[] = {SGI234401, SGI234402, SGI234403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234404, SGI0, SGI0, SGI0, SGI234405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51965[] = {SGI234406, SGI234407, SGI234408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234409, SGI234409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234410, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51966[] = {SGI234411, SGI234412, SGI234413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234414, SGI234414, SGI0, SGI0, SGI234415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51967[] = {SGI234416, SGI0, SGI234417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234418, SGI234419, SGI234419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51968[] = {SGI234420, SGI0, SGI234421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234422, SGI234423, SGI234423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51969[] = {SGI234424, SGI234425, SGI234426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234427, SGI0, SGI0, SGI0, SGI234428, SGI234428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51970[] = {SGI234429, SGI234430, SGI234431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234432, SGI234432, SGI0, SGI0, SGI234433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51971[] = {SGI234434, SGI234435, SGI234436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234437, SGI0, SGI0, SGI0, SGI234438, SGI234438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234439, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51972[] = {SGI234440, SGI234441, SGI234442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234443, SGI0, SGI0, SGI0, SGI234444, SGI234444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234446, SGI0, SGI0, NULL};
const Segment * const SG51973[] = {SGI234445, SGI0, SGI234447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234448, SGI234449, SGI234449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51974[] = {SGI234450, SGI234451, SGI234452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234453, SGI0, SGI0, SGI0, SGI234454, SGI234454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234455, SGI0, SGI0, NULL};
const Segment * const SG51975[] = {SGI234456, SGI0, SGI234457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234459, SGI234460, SGI234460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51976[] = {SGI234458, SGI234461, SGI234462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234464, SGI0, SGI0, SGI0, SGI234466, SGI234466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51977[] = {SGI234463, SGI0, SGI234465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234467, SGI234468, SGI234468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51978[] = {SGI234469, SGI0, SGI234470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234471, SGI234472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51979[] = {SGI234473, SGI0, SGI234474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234477, SGI234478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51980[] = {SGI234475, SGI0, SGI234476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234479, SGI234480, SGI234480, SGI0, SGI0, NULL};
const Segment * const SG51981[] = {SGI234481, SGI0, SGI234482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234483, SGI234484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51982[] = {SGI234485, SGI0, SGI234486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234487, SGI234488, SGI234488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234489, SGI0, SGI0, NULL};
const Segment * const SG51983[] = {SGI234490, SGI0, SGI234491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234492, SGI234493, SGI234493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51984[] = {SGI234494, SGI234495, SGI234496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234499, SGI0, SGI0, SGI0, SGI234501, SGI234501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234503, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51985[] = {SGI234497, SGI234498, SGI234500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234502, SGI0, SGI0, SGI0, SGI234504, SGI234504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51986[] = {SGI234506, SGI0, SGI234507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234508, SGI234509, SGI234509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51987[] = {SGI234511, SGI0, SGI234513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234515, SGI234516, SGI234516, SGI0, SGI0, NULL};
const Segment * const SG51988[] = {SGI234510, SGI234512, SGI234514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234517, SGI234517, SGI0, SGI0, SGI0, SGI0, SGI234518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51989[] = {SGI234519, SGI234520, SGI234521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234522, SGI0, SGI0, SGI0, SGI234523, SGI234523, SGI0, SGI234524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51990[] = {SGI234525, SGI234526, SGI234527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234528, SGI0, SGI0, SGI0, SGI234529, SGI234529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51991[] = {SGI234530, SGI0, SGI234531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234532, SGI234533, SGI234533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234537, SGI0, SGI0, NULL};
const Segment * const SG51992[] = {SGI234534, SGI234535, SGI234536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234538, SGI0, SGI0, SGI0, SGI234539, SGI234539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51993[] = {SGI234540, SGI234541, SGI234542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234543, SGI0, SGI0, SGI0, SGI234544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51994[] = {SGI234546, SGI0, SGI234547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234548, SGI234549, SGI234549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234551, SGI0, SGI0, NULL};
const Segment * const SG51995[] = {SGI234550, SGI234552, SGI234553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234554, SGI0, SGI0, SGI0, SGI234555, SGI234555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51996[] = {SGI234557, SGI234558, SGI234559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234560, SGI0, SGI0, SGI0, SGI234561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51997[] = {SGI234563, SGI234564, SGI234566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234569, SGI0, SGI0, SGI0, SGI234571, SGI234571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51998[] = {SGI234565, SGI234567, SGI234568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234570, SGI0, SGI0, SGI0, SGI234572, SGI234572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG51999[] = {SGI234574, SGI234575, SGI234576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234577, SGI0, SGI0, SGI0, SGI234578, SGI234579, SGI0, SGI0, SGI0, SGI0, SGI234580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52000[] = {SGI234581, SGI0, SGI234582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234583, SGI0, SGI0, SGI0, SGI234584, SGI234584, SGI0, SGI0, SGI0, SGI0, SGI234585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52001[] = {SGI234586, SGI0, SGI234587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234588, SGI234589, SGI234589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52002[] = {SGI234590, SGI234591, SGI234592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234593, SGI0, SGI0, SGI0, SGI234594, SGI234594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234595, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52003[] = {SGI234596, SGI234597, SGI234598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234599, SGI0, SGI0, SGI0, SGI234600, SGI234600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234601, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52004[] = {SGI234605, SGI0, SGI234606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234608, SGI234610, SGI234610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52005[] = {SGI234602, SGI234603, SGI234604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234607, SGI0, SGI0, SGI0, SGI234609, SGI0, SGI234611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52006[] = {SGI234612, SGI0, SGI234613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234614, SGI234615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52007[] = {SGI234616, SGI234617, SGI234618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234619, SGI0, SGI0, SGI0, SGI234620, SGI234620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234621, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52008[] = {SGI234622, SGI234623, SGI234624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234625, SGI0, SGI0, SGI0, SGI234626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234628, SGI0, NULL};
const Segment * const SG52009[] = {SGI234629, SGI234630, SGI234631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234632, SGI0, SGI0, SGI0, SGI234633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234635, SGI0, NULL};
const Segment * const SG52010[] = {SGI234636, SGI0, SGI234637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234638, SGI0, SGI0, SGI0, SGI234639, SGI234639, SGI0, SGI0, SGI0, SGI234640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52011[] = {SGI234641, SGI0, SGI234642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234643, SGI0, SGI0, SGI0, SGI234644, SGI234644, SGI0, SGI0, SGI0, SGI234645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52012[] = {SGI234646, SGI0, SGI234647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234648, SGI234649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52013[] = {SGI234650, SGI234651, SGI234652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234653, SGI0, SGI0, SGI0, SGI234654, SGI234654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52014[] = {SGI234655, SGI0, SGI234656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234657, SGI234658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52015[] = {SGI234659, SGI234660, SGI234661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234662, SGI0, SGI0, SGI0, SGI234663, SGI234663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52016[] = {SGI234664, SGI234665, SGI234666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234667, SGI0, SGI0, SGI0, SGI234668, SGI234668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52017[] = {SGI234669, SGI234670, SGI234671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234672, SGI0, SGI0, SGI0, SGI234673, SGI234673, SGI0, SGI0, SGI234674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52018[] = {SGI234675, SGI234676, SGI234677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234678, SGI0, SGI0, SGI0, SGI234679, SGI234679, SGI0, SGI0, SGI234681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52019[] = {SGI234680, SGI0, SGI234682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234683, SGI234684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52020[] = {SGI234685, SGI0, SGI234686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234687, SGI234688, SGI234688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52021[] = {SGI234689, SGI0, SGI234690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234691, SGI234692, SGI234692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52022[] = {SGI234693, SGI0, SGI234694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234695, SGI234696, SGI234696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52023[] = {SGI234697, SGI234698, SGI234699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234700, SGI0, SGI0, SGI0, SGI234701, SGI234701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234702, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52024[] = {SGI234703, SGI0, SGI234705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234707, SGI234709, SGI234709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52025[] = {SGI234704, SGI0, SGI234706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234708, SGI234710, SGI234710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52026[] = {SGI234711, SGI234712, SGI234713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234714, SGI0, SGI0, SGI0, SGI234715, SGI234715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52027[] = {SGI234717, SGI0, SGI234718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234719, SGI234720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52028[] = {SGI234721, SGI0, SGI234722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234723, SGI234724, SGI234724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52029[] = {SGI234725, SGI234726, SGI234727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234730, SGI0, SGI0, SGI0, SGI234731, SGI0, SGI0, SGI234732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52030[] = {SGI234728, SGI0, SGI234729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234733, SGI0, SGI0, SGI0, SGI234734, SGI234734, SGI0, SGI0, SGI0, SGI234735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52031[] = {SGI234736, SGI234737, SGI234738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234739, SGI0, SGI0, SGI0, SGI234740, SGI234740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52032[] = {SGI234741, SGI234742, SGI234743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234744, SGI0, SGI0, SGI0, SGI234745, SGI234745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52033[] = {SGI234746, SGI0, SGI234747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234748, SGI234749, SGI234749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52034[] = {SGI234750, SGI0, SGI234751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234752, SGI234753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52035[] = {SGI234754, SGI234755, SGI234756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234757, SGI0, SGI0, SGI0, SGI234758, SGI234758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234759, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52036[] = {SGI234760, SGI0, SGI234761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234762, SGI234763, SGI234763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52037[] = {SGI234764, SGI0, SGI234765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234766, SGI234767, SGI234767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52038[] = {SGI234768, SGI0, SGI234769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234770, SGI234771, SGI234771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52039[] = {SGI234772, SGI0, SGI234773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234774, SGI234775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52040[] = {SGI234776, SGI0, SGI234777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234778, SGI234779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52041[] = {SGI234780, SGI0, SGI234781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234782, SGI0, SGI0, SGI0, SGI234783, SGI234783, SGI0, SGI0, SGI0, SGI234784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52042[] = {SGI234785, SGI234786, SGI234787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234788, SGI0, SGI0, SGI0, SGI234789, SGI234789, SGI0, SGI234790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52043[] = {SGI234791, SGI0, SGI234792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234793, SGI234794, SGI234794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52044[] = {SGI234795, SGI0, SGI234796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234797, SGI234798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52045[] = {SGI234799, SGI0, SGI234800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234802, SGI234804, SGI234804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52046[] = {SGI234801, SGI234803, SGI234805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234806, SGI0, SGI0, SGI0, SGI234807, SGI234807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52047[] = {SGI234808, SGI0, SGI234809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234813, SGI234814, SGI234814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52048[] = {SGI234810, SGI234811, SGI234812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234815, SGI0, SGI0, SGI0, SGI234816, SGI234816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52049[] = {SGI234817, SGI0, SGI234818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234821, SGI234822, SGI234822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52050[] = {SGI234819, SGI0, SGI234820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234823, SGI234824, SGI234824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52051[] = {SGI234825, SGI0, SGI234826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234828, SGI234830, SGI234830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52052[] = {SGI234827, SGI0, SGI234829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234831, SGI234832, SGI234832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52053[] = {SGI234833, SGI0, SGI234834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234835, SGI234836, SGI234836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52054[] = {SGI234837, SGI0, SGI234838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234839, SGI234840, SGI234840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52055[] = {SGI234841, SGI0, SGI234842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234843, SGI234844, SGI234844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234845, SGI0, SGI0, NULL};
const Segment * const SG52056[] = {SGI234846, SGI0, SGI234847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234848, SGI234849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234850, SGI0, SGI0, NULL};
const Segment * const SG52057[] = {SGI234851, SGI0, SGI234852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234853, SGI234854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52058[] = {SGI234855, SGI0, SGI234856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234859, SGI234860, SGI234860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234861, SGI0, SGI0, NULL};
const Segment * const SG52059[] = {SGI234857, SGI0, SGI234858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234862, SGI234863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52060[] = {SGI234864, SGI0, SGI234865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234866, SGI234867, SGI234867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52061[] = {SGI234868, SGI234869, SGI234870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234871, SGI0, SGI0, SGI0, SGI234872, SGI234872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52062[] = {SGI234873, SGI234874, SGI234875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234876, SGI234876, SGI0, SGI0, SGI234877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52063[] = {SGI234878, SGI234879, SGI234880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234881, SGI0, SGI0, SGI0, SGI234882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52064[] = {SGI234883, SGI234884, SGI234885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234886, SGI0, SGI0, SGI0, SGI234887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52065[] = {SGI234888, SGI234889, SGI234891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234893, SGI234893, SGI0, SGI0, SGI0, SGI0, SGI234896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52066[] = {SGI234890, SGI0, SGI234892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234894, SGI234895, SGI234895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52067[] = {SGI234897, SGI234898, SGI234899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234900, SGI0, SGI0, SGI0, SGI234903, SGI234903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234904, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52068[] = {SGI234901, SGI0, SGI234902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234905, SGI234906, SGI234906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52069[] = {SGI234907, SGI0, SGI234908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234909, SGI234910, SGI234910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52070[] = {SGI234911, SGI0, SGI234912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234913, SGI234914, SGI234914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52071[] = {SGI234915, SGI0, SGI234916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234917, SGI234918, SGI234918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52072[] = {SGI234919, SGI0, SGI234921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234923, SGI234924, SGI234924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52073[] = {SGI234920, SGI0, SGI234922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234925, SGI234926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52074[] = {SGI234927, SGI0, SGI234928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234929, SGI234930, SGI234930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52075[] = {SGI234931, SGI234932, SGI234934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234937, SGI0, SGI0, SGI0, SGI234938, SGI234938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52076[] = {SGI234933, SGI234935, SGI234936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234939, SGI0, SGI0, SGI0, SGI234940, SGI234940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234941, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52077[] = {SGI234942, SGI0, SGI234943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234944, SGI234945, SGI234945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52078[] = {SGI234946, SGI0, SGI234947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234948, SGI234949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52079[] = {SGI234950, SGI0, SGI234951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234952, SGI234953, SGI234953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52080[] = {SGI234954, SGI0, SGI234955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234956, SGI234957, SGI234957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52081[] = {SGI234958, SGI0, SGI234959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234960, SGI234961, SGI234961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52082[] = {SGI234962, SGI234963, SGI234964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234965, SGI0, SGI0, SGI0, SGI234966, SGI234966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234968, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52083[] = {SGI234967, SGI0, SGI234969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234970, SGI234971, SGI234971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52084[] = {SGI234972, SGI0, SGI234973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234974, SGI234975, SGI234975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52085[] = {SGI234976, SGI234977, SGI234978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234979, SGI0, SGI0, SGI0, SGI234980, SGI234980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234981, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52086[] = {SGI234982, SGI234983, SGI234984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234985, SGI0, SGI0, SGI0, SGI234986, SGI234986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234987, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52087[] = {SGI234988, SGI0, SGI234989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234990, SGI234991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52088[] = {SGI234992, SGI0, SGI234993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234994, SGI234995, SGI234995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52089[] = {SGI234996, SGI234997, SGI234998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI234999, SGI234999, SGI0, SGI0, SGI235000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52090[] = {SGI235001, SGI0, SGI235002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235003, SGI235004, SGI235004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52091[] = {SGI235005, SGI235006, SGI235007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235008, SGI0, SGI0, SGI0, SGI235009, SGI235009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235010, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52092[] = {SGI235011, SGI0, SGI235012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235013, SGI235014, SGI235014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235015, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52093[] = {SGI235016, SGI0, SGI235017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235018, SGI235019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235020, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52094[] = {SGI235023, SGI0, SGI235025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235027, SGI235028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52095[] = {SGI235021, SGI235022, SGI235024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235026, SGI0, SGI0, SGI0, SGI235029, SGI235029, SGI0, SGI0, SGI0, SGI0, SGI235030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52096[] = {SGI235031, SGI0, SGI235032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235033, SGI235034, SGI235034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52097[] = {SGI235035, SGI0, SGI235036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235037, SGI235038, SGI235038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52098[] = {SGI235039, SGI0, SGI235041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235044, SGI235046, SGI235046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52099[] = {SGI235040, SGI235042, SGI235043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235045, SGI0, SGI0, SGI0, SGI235047, SGI235047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235048, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52100[] = {SGI235049, SGI235050, SGI235051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235052, SGI235052, SGI0, SGI0, SGI235053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52101[] = {SGI235054, SGI0, SGI235055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235056, SGI0, SGI0, SGI0, SGI235057, SGI235057, SGI0, SGI0, SGI0, SGI235058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52102[] = {SGI235059, SGI235060, SGI235061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235062, SGI235062, SGI0, SGI0, SGI0, SGI235063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52103[] = {SGI235064, SGI0, SGI235065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235066, SGI235067, SGI235067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52104[] = {SGI235068, SGI235069, SGI235070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235071, SGI0, SGI0, SGI0, SGI235072, SGI235072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235073, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52105[] = {SGI235074, SGI0, SGI235075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235076, SGI235077, SGI235077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52106[] = {SGI235078, SGI235079, SGI235080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235083, SGI235083, SGI0, SGI0, SGI0, SGI235084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52107[] = {SGI235081, SGI0, SGI235082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235085, SGI235086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52108[] = {SGI235087, SGI0, SGI235088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235089, SGI235090, SGI235090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52109[] = {SGI235091, SGI0, SGI235092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235093, SGI0, SGI0, SGI0, SGI235094, SGI235094, SGI0, SGI0, SGI0, SGI235095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52110[] = {SGI235096, SGI235097, SGI235098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235099, SGI235099, SGI0, SGI0, SGI0, SGI235100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52111[] = {SGI235101, SGI0, SGI235102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235103, SGI235104, SGI235104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52112[] = {SGI235105, SGI0, SGI235107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235109, SGI235111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52113[] = {SGI235106, SGI0, SGI235108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235110, SGI235112, SGI235112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52114[] = {SGI235113, SGI0, SGI235114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235115, SGI235116, SGI235116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52115[] = {SGI235117, SGI235118, SGI235119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235120, SGI0, SGI0, SGI0, SGI235121, SGI235121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235122, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52116[] = {SGI235123, SGI235124, SGI235125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235126, SGI0, SGI0, SGI0, SGI235127, SGI235127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52117[] = {SGI235128, SGI235129, SGI235130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235131, SGI0, SGI0, SGI0, SGI235132, SGI235132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235133, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52118[] = {SGI235134, SGI0, SGI235135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235136, SGI235137, SGI235137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52119[] = {SGI235138, SGI0, SGI235139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235140, SGI235141, SGI235141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235142, SGI0, SGI0, NULL};
const Segment * const SG52120[] = {SGI235143, SGI235144, SGI235145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235146, SGI0, SGI0, SGI0, SGI235147, SGI235147, SGI235148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52121[] = {SGI235149, SGI235150, SGI235151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235152, SGI235152, SGI0, SGI0, SGI0, SGI235153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52122[] = {SGI235154, SGI235155, SGI235156, SGI235157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235158, SGI0, SGI0, SGI0, SGI235159, SGI235160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52123[] = {SGI235161, SGI235162, SGI235163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235164, SGI0, SGI0, SGI0, SGI235165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52124[] = {SGI235166, SGI0, SGI235167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235168, SGI0, SGI0, SGI0, SGI235169, SGI235169, SGI0, SGI0, SGI0, SGI235170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52125[] = {SGI235171, SGI0, SGI235172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235173, SGI235174, SGI235174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52126[] = {SGI235175, SGI0, SGI235176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235178, SGI235179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52127[] = {SGI235177, SGI0, SGI235180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235181, SGI235182, SGI235182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52128[] = {SGI235183, SGI235184, SGI235185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235186, SGI235186, SGI0, SGI0, SGI0, SGI235187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52129[] = {SGI235188, SGI235189, SGI235190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235191, SGI0, SGI0, SGI0, SGI235192, SGI235192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52130[] = {SGI235193, SGI235194, SGI235195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235196, SGI235196, SGI0, SGI0, SGI0, SGI235197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52131[] = {SGI235198, SGI235199, SGI235200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235201, SGI235201, SGI0, SGI0, SGI0, SGI235202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52132[] = {SGI235203, SGI0, SGI235204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235205, SGI235206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52133[] = {SGI235208, SGI0, SGI235210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235212, SGI235214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52134[] = {SGI235207, SGI0, SGI235209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235211, SGI235213, SGI235213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52135[] = {SGI235215, SGI235216, SGI235217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235218, SGI0, SGI0, SGI0, SGI235219, SGI235219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235220, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52136[] = {SGI235221, SGI235222, SGI235223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235224, SGI235224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235225, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52137[] = {SGI235226, SGI235227, SGI235228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235229, SGI235229, SGI0, SGI0, SGI0, SGI235230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52138[] = {SGI235231, SGI0, SGI235232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235236, SGI235237, SGI235237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52139[] = {SGI235233, SGI235234, SGI235235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235238, SGI0, SGI0, SGI0, SGI235239, SGI235239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235240, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52140[] = {SGI235241, SGI235242, SGI235243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235244, SGI0, SGI0, SGI0, SGI235245, SGI235245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235246, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52141[] = {SGI235247, SGI0, SGI235248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235249, SGI235250, SGI235250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52142[] = {SGI235251, SGI0, SGI235253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235255, SGI235256, SGI235256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52143[] = {SGI235252, SGI0, SGI235254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235257, SGI235258, SGI235258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52144[] = {SGI235259, SGI0, SGI235260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235261, SGI235262, SGI235262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52145[] = {SGI235263, SGI0, SGI235264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235265, SGI235266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52146[] = {SGI235267, SGI0, SGI235268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235269, SGI235270, SGI235270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52147[] = {SGI235271, SGI0, SGI235272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235273, SGI235274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52148[] = {SGI235275, SGI0, SGI235276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235277, SGI235278, SGI235278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52149[] = {SGI235279, SGI235280, SGI235281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235282, SGI0, SGI0, SGI0, SGI235283, SGI235283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235284, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52150[] = {SGI235285, SGI235286, SGI235287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235288, SGI0, SGI0, SGI0, SGI235289, SGI235289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235290, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52151[] = {SGI235291, SGI235292, SGI235293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235294, SGI235294, SGI0, SGI0, SGI0, SGI235295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52152[] = {SGI235296, SGI0, SGI235297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235298, SGI235299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52153[] = {SGI235300, SGI235301, SGI235302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235303, SGI235303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235304, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52154[] = {SGI235305, SGI0, SGI235306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235309, SGI235311, SGI235311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52155[] = {SGI235307, SGI235308, SGI235310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235312, SGI235312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235313, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52156[] = {SGI235314, SGI0, SGI235315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235319, SGI235320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52157[] = {SGI235316, SGI235317, SGI235318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235321, SGI235321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235322, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52158[] = {SGI235323, SGI0, SGI235324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235325, SGI235326, SGI235326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235327, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52159[] = {SGI235328, SGI0, SGI235329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235330, SGI235331, SGI235331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235332, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52160[] = {SGI235333, SGI235334, SGI235335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235336, SGI0, SGI0, SGI0, SGI235337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52161[] = {SGI235338, SGI235339, SGI235340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235341, SGI0, SGI0, SGI0, SGI235342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235343, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52162[] = {SGI235344, SGI235345, SGI235346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235347, SGI0, SGI0, SGI0, SGI235348, SGI235349, SGI0, SGI0, SGI0, SGI235350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52163[] = {SGI235351, SGI235352, SGI235353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235356, SGI0, SGI0, SGI0, SGI235357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52164[] = {SGI235354, SGI0, SGI235355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235358, SGI235359, SGI235359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52165[] = {SGI235360, SGI0, SGI235361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235362, SGI235363, SGI235363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52166[] = {SGI235364, SGI0, SGI235365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235366, SGI235367, SGI235367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52167[] = {SGI235368, SGI0, SGI235369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235370, SGI235371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52168[] = {SGI235372, SGI235373, SGI235374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235378, SGI0, SGI0, SGI0, SGI235379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52169[] = {SGI235375, SGI235376, SGI235377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235380, SGI235380, SGI0, SGI0, SGI0, SGI235381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52170[] = {SGI235382, SGI0, SGI235383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52171[] = {SGI235385, SGI235387, SGI235389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235390, SGI0, SGI0, SGI0, SGI235392, SGI235392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52172[] = {SGI235384, SGI235386, SGI235388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235391, SGI235391, SGI0, SGI0, SGI0, SGI235393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52173[] = {SGI235394, SGI0, SGI235395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235396, SGI235397, SGI235397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52174[] = {SGI235398, SGI0, SGI235399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235401, SGI235403, SGI235403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52175[] = {SGI235400, SGI235402, SGI235404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235405, SGI235405, SGI0, SGI0, SGI0, SGI235406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52176[] = {SGI235407, SGI235409, SGI235410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235413, SGI235413, SGI0, SGI0, SGI0, SGI235414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52177[] = {SGI235415, SGI0, SGI235416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235417, SGI235418, SGI235418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52178[] = {SGI235419, SGI235420, SGI235421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235422, SGI235422, SGI0, SGI0, SGI0, SGI235423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52179[] = {SGI235424, SGI0, SGI235425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235426, SGI235427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52180[] = {SGI235428, SGI235429, SGI235430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235431, SGI235431, SGI0, SGI0, SGI0, SGI235433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52181[] = {SGI235432, SGI0, SGI235434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235435, SGI235436, SGI235436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52182[] = {SGI235437, SGI235438, SGI235439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235442, SGI235442, SGI0, SGI0, SGI0, SGI235443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52183[] = {SGI235440, SGI0, SGI235441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235444, SGI235445, SGI235445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235446, SGI0, SGI0, NULL};
const Segment * const SG52184[] = {SGI235447, SGI235448, SGI235449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235452, SGI0, SGI0, SGI0, SGI235453, SGI235453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235455, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52185[] = {SGI235450, SGI0, SGI235451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235454, SGI235456, SGI235456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235457, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52186[] = {SGI235458, SGI0, SGI235459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235460, SGI235461, SGI235461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52187[] = {SGI235462, SGI235463, SGI235464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235465, SGI0, SGI0, SGI0, SGI235466, SGI235466, SGI0, SGI0, SGI0, SGI235467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52188[] = {SGI235468, SGI0, SGI235469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235470, SGI235471, SGI235471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52189[] = {SGI235472, SGI0, SGI235473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235474, SGI235475, SGI235475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52190[] = {SGI235476, SGI235477, SGI235478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235479, SGI235479, SGI0, SGI0, SGI0, SGI0, SGI235480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52191[] = {SGI235481, SGI235482, SGI235483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235484, SGI0, SGI0, SGI0, SGI235485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52192[] = {SGI235486, SGI235487, SGI235488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235489, SGI0, SGI0, SGI0, SGI235490, SGI235490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235491, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52193[] = {SGI235492, SGI235493, SGI235494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235495, SGI0, SGI0, SGI0, SGI235496, SGI235496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235497, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52194[] = {SGI235498, SGI0, SGI235499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235500, SGI235501, SGI235501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52195[] = {SGI235502, SGI235503, SGI235504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235505, SGI0, SGI0, SGI0, SGI235506, SGI235506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235507, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52196[] = {SGI235508, SGI0, SGI235509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235510, SGI235511, SGI235511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52197[] = {SGI235512, SGI0, SGI235513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235514, SGI235515, SGI235515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52198[] = {SGI235516, SGI0, SGI235517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235518, SGI235519, SGI235519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52199[] = {SGI235520, SGI0, SGI235521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235522, SGI235523, SGI235523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52200[] = {SGI235524, SGI0, SGI235525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235526, SGI235527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52201[] = {SGI235528, SGI235529, SGI235530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235531, SGI0, SGI0, SGI0, SGI235532, SGI235532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52202[] = {SGI235533, SGI235534, SGI235535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235536, SGI0, SGI0, SGI0, SGI235537, SGI0, SGI0, SGI235538, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52203[] = {SGI235539, SGI0, SGI235540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235541, SGI235542, SGI235542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52204[] = {SGI0, SGI0, SGI235543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235544, SGI0, SGI0, SGI0, SGI235545, SGI235545, SGI0, SGI0, SGI0, SGI235546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52205[] = {SGI235547, SGI0, SGI235548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235549, SGI235550, SGI235550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52206[] = {SGI235551, SGI0, SGI235552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235553, SGI0, SGI0, SGI0, SGI235555, SGI235555, SGI0, SGI0, SGI0, SGI0, SGI235557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52207[] = {SGI235554, SGI0, SGI235556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235558, SGI235559, SGI235559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52208[] = {SGI235560, SGI235561, SGI235562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235563, SGI0, SGI0, SGI0, SGI235566, SGI235567, SGI0, SGI0, SGI0, SGI0, SGI235568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52209[] = {SGI235564, SGI0, SGI235565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235569, SGI235570, SGI235570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52210[] = {SGI235571, SGI235572, SGI235573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235574, SGI0, SGI0, SGI0, SGI235575, SGI235575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235576, NULL};
const Segment * const SG52211[] = {SGI235577, SGI0, SGI235578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235579, SGI0, SGI0, SGI0, SGI235580, SGI235580, SGI0, SGI0, SGI0, SGI235581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52212[] = {SGI235582, SGI0, SGI235583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235584, SGI235585, SGI235585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52213[] = {SGI235586, SGI0, SGI235587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235588, SGI0, SGI0, SGI0, SGI235589, SGI235589, SGI0, SGI0, SGI0, SGI0, SGI235590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52214[] = {SGI235592, SGI0, SGI235595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235596, SGI235598, SGI235598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52215[] = {SGI235591, SGI235593, SGI235594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235597, SGI235597, SGI0, SGI0, SGI0, SGI0, SGI235599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52216[] = {SGI235600, SGI235601, SGI235602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235603, SGI235603, SGI0, SGI0, SGI0, SGI0, SGI235604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52217[] = {SGI235605, SGI235606, SGI235607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235608, SGI0, SGI0, SGI0, SGI235609, SGI235609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52218[] = {SGI235611, SGI0, SGI235613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235615, SGI235617, SGI235617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52219[] = {SGI235612, SGI0, SGI235614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235616, SGI0, SGI0, SGI0, SGI235618, SGI235618, SGI0, SGI0, SGI235619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52220[] = {SGI235620, SGI0, SGI235621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235622, SGI235623, SGI235623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235624, SGI0, SGI0, NULL};
const Segment * const SG52221[] = {SGI235625, SGI0, SGI235626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235627, SGI235628, SGI235628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52222[] = {SGI235629, SGI0, SGI235630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235631, SGI235632, SGI235632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52223[] = {SGI235633, SGI0, SGI235634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235635, SGI235636, SGI235636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52224[] = {SGI235637, SGI235638, SGI235639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235640, SGI0, SGI0, SGI0, SGI235641, SGI235641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235642, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52225[] = {SGI235643, SGI235644, SGI235645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235646, SGI235646, SGI0, SGI0, SGI0, SGI0, SGI235647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52226[] = {SGI235648, SGI235649, SGI235650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235651, SGI0, SGI0, SGI0, SGI235652, SGI235652, SGI0, SGI235653, SGI0, SGI0, SGI0, SGI0, SGI235654, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52227[] = {SGI235655, SGI0, SGI235656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235657, SGI0, SGI0, SGI0, SGI235658, SGI235658, SGI0, SGI0, SGI0, SGI235659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52228[] = {SGI235660, SGI235661, SGI235662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235663, SGI0, SGI0, SGI0, SGI235664, SGI235664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235667, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52229[] = {SGI235665, SGI0, SGI235666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235668, SGI0, SGI0, SGI0, SGI235669, SGI235669, SGI0, SGI0, SGI0, SGI235670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52230[] = {SGI235671, SGI235672, SGI235673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235674, SGI0, SGI0, SGI0, SGI235675, SGI235675, SGI0, SGI0, SGI0, SGI235676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52231[] = {SGI235677, SGI235678, SGI235679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235680, SGI235680, SGI0, SGI0, SGI0, SGI0, SGI235681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52232[] = {SGI235682, SGI0, SGI235683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235684, SGI0, SGI0, SGI0, SGI235685, SGI235685, SGI0, SGI0, SGI0, SGI235686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52233[] = {SGI235687, SGI0, SGI235688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235689, SGI235690, SGI235690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52234[] = {SGI235691, SGI0, SGI235692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235693, SGI235694, SGI235694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52235[] = {SGI235695, SGI0, SGI235696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235697, SGI235698, SGI235698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52236[] = {SGI235699, SGI0, SGI235700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235701, SGI235702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52237[] = {SGI235703, SGI0, SGI235704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235705, SGI235706, SGI235706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52238[] = {SGI235707, SGI235708, SGI235709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235710, SGI235710, SGI0, SGI0, SGI0, SGI0, SGI235711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52239[] = {SGI235712, SGI0, SGI235713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235714, SGI235715, SGI235715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52240[] = {SGI235716, SGI235717, SGI235718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235719, SGI235719, SGI0, SGI0, SGI0, SGI0, SGI235720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52241[] = {SGI235721, SGI0, SGI235722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235723, SGI235724, SGI235724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52242[] = {SGI235725, SGI235726, SGI235727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235728, SGI235728, SGI0, SGI0, SGI0, SGI0, SGI235729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52243[] = {SGI235408, SGI235411, SGI235412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235730, SGI0, SGI0, SGI0, SGI235731, SGI235731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52244[] = {SGI235732, SGI0, SGI235733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235734, SGI235735, SGI235735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52245[] = {SGI235736, SGI0, SGI235737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235738, SGI235739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52246[] = {SGI235740, SGI0, SGI235741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235742, SGI235743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52247[] = {SGI235744, SGI0, SGI235745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235746, SGI235747, SGI235747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52248[] = {SGI235748, SGI0, SGI235749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52249[] = {SGI235750, SGI235751, SGI235752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235753, SGI0, SGI0, SGI0, SGI235754, SGI235754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52250[] = {SGI235755, SGI0, SGI235756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235757, SGI235758, SGI235758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52251[] = {SGI235759, SGI0, SGI235760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235761, SGI235762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52252[] = {SGI235763, SGI0, SGI235764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235765, SGI235766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52253[] = {SGI235767, SGI0, SGI235768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235769, SGI235770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52254[] = {SGI235771, SGI0, SGI235772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235773, SGI235774, SGI235774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52255[] = {SGI235775, SGI0, SGI235776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235777, SGI235778, SGI235778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52256[] = {SGI235779, SGI235780, SGI235781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235782, SGI0, SGI0, SGI0, SGI235783, SGI235783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235784, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52257[] = {SGI235785, SGI235786, SGI235787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235788, SGI0, SGI235789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52258[] = {SGI235790, SGI235791, SGI235792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235793, SGI235793, SGI0, SGI0, SGI235794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52259[] = {SGI235795, SGI235796, SGI235797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235798, SGI0, SGI0, SGI0, SGI235799, SGI235799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52260[] = {SGI235800, SGI0, SGI235801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235802, SGI235803, SGI235803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52261[] = {SGI235804, SGI0, SGI235805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235806, SGI235807, SGI235807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52262[] = {SGI235808, SGI235809, SGI235810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235811, SGI0, SGI0, SGI0, SGI235812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52263[] = {SGI235813, SGI235814, SGI235815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235816, SGI0, SGI0, SGI0, SGI235817, SGI235817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52264[] = {SGI235818, SGI0, SGI235819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235820, SGI235821, SGI235821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52265[] = {SGI235822, SGI235823, SGI235824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235825, SGI0, SGI0, SGI0, SGI235826, SGI235826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52266[] = {SGI235827, SGI0, SGI235828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235829, SGI235830, SGI235830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52267[] = {SGI235831, SGI235832, SGI235833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235834, SGI0, SGI0, SGI0, SGI235835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52268[] = {SGI235836, SGI235837, SGI235838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235839, SGI0, SGI0, SGI0, SGI235840, SGI0, SGI0, SGI235841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52269[] = {SGI235842, SGI0, SGI235843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235844, SGI235845, SGI235845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52270[] = {SGI235846, SGI0, SGI235847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235850, SGI235851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52271[] = {SGI235848, SGI0, SGI235849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235852, SGI235853, SGI235853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52272[] = {SGI235854, SGI0, SGI235855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235856, SGI235857, SGI235857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52273[] = {SGI235858, SGI235859, SGI235860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235861, SGI0, SGI235862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52274[] = {SGI235863, SGI0, SGI235864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235865, SGI235866, SGI235866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52275[] = {SGI235867, SGI235868, SGI235870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235872, SGI0, SGI0, SGI0, SGI235873, SGI0, SGI0, SGI0, SGI0, SGI235875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52276[] = {SGI235869, SGI0, SGI235871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235874, SGI235876, SGI235876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235877, SGI0, SGI0, NULL};
const Segment * const SG52277[] = {SGI235878, SGI0, SGI235879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235880, SGI235881, SGI235881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52278[] = {SGI235882, SGI0, SGI235883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235884, SGI0, SGI0, SGI0, SGI235885, SGI235885, SGI0, SGI0, SGI235886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52279[] = {SGI235887, SGI235888, SGI235889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235890, SGI235890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235891, NULL};
const Segment * const SG52280[] = {SGI235892, SGI235893, SGI235894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235895, SGI0, SGI0, SGI0, SGI235896, SGI235896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52281[] = {SGI235897, SGI0, SGI235898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235901, SGI235902, SGI235902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235904, SGI0, SGI0, NULL};
const Segment * const SG52282[] = {SGI235899, SGI0, SGI235900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235903, SGI235905, SGI235905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52283[] = {SGI235906, SGI0, SGI235907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235908, SGI235909, SGI235909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52284[] = {SGI235910, SGI235911, SGI235912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235913, SGI235913, SGI0, SGI0, SGI0, SGI0, SGI235914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52285[] = {SGI235915, SGI235916, SGI235917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235918, SGI0, SGI0, SGI0, SGI235919, SGI235919, SGI0, SGI235920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52286[] = {SGI235921, SGI235922, SGI235923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235924, SGI235924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235925, NULL};
const Segment * const SG52287[] = {SGI235926, SGI235927, SGI235928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235929, SGI0, SGI0, SGI0, SGI235930, SGI235930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235931, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52288[] = {SGI235932, SGI0, SGI235933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235934, SGI0, SGI0, SGI0, SGI235935, SGI235935, SGI0, SGI0, SGI0, SGI235936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52289[] = {SGI235938, SGI235940, SGI235941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235944, SGI235944, SGI0, SGI0, SGI235946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52290[] = {SGI235937, SGI0, SGI235939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235942, SGI0, SGI0, SGI0, SGI235943, SGI235943, SGI0, SGI0, SGI0, SGI235945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52291[] = {SGI235947, SGI0, SGI235948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235949, SGI235950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52292[] = {SGI235951, SGI235952, SGI235953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235954, SGI235954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235957, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52293[] = {SGI235955, SGI0, SGI235956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235958, SGI235959, SGI235959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52294[] = {SGI235960, SGI0, SGI235961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235962, SGI235963, SGI235963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52295[] = {SGI235964, SGI235965, SGI235966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235969, SGI235969, SGI0, SGI0, SGI235970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52296[] = {SGI235967, SGI0, SGI235968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235971, SGI235972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52297[] = {SGI235973, SGI0, SGI235974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235975, SGI235976, SGI235976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52298[] = {SGI235977, SGI0, SGI235978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235979, SGI235980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52299[] = {SGI235981, SGI0, SGI235982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235983, SGI235984, SGI235984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52300[] = {SGI235985, SGI0, SGI235986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235987, SGI235988, SGI235988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52301[] = {SGI235989, SGI235990, SGI235991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235992, SGI0, SGI0, SGI0, SGI235993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52302[] = {SGI235994, SGI0, SGI235995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI235996, SGI235997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52303[] = {SGI235998, SGI0, SGI235999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236000, SGI236001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236002, SGI0, SGI0, NULL};
const Segment * const SG52304[] = {SGI236003, SGI236004, SGI236005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236006, SGI236006, SGI0, SGI0, SGI236007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52305[] = {SGI236008, SGI0, SGI236009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236010, SGI236011, SGI236011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52306[] = {SGI236012, SGI236013, SGI236014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236015, SGI236015, SGI0, SGI0, SGI236016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52307[] = {SGI236017, SGI236018, SGI236019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236020, SGI0, SGI0, SGI0, SGI236021, SGI236021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52308[] = {SGI236022, SGI236023, SGI236024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236025, SGI0, SGI0, SGI0, SGI236026, SGI236026, SGI0, SGI236027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52309[] = {SGI236028, SGI236029, SGI236030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236031, SGI0, SGI0, SGI0, SGI236032, SGI236032, SGI0, SGI236033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52310[] = {SGI236034, SGI0, SGI236035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236036, SGI236037, SGI236037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52311[] = {SGI236038, SGI0, SGI236039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236040, SGI236041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52312[] = {SGI236042, SGI0, SGI236043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236044, SGI236045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52313[] = {SGI236046, SGI0, SGI236047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236048, SGI236049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52314[] = {SGI236050, SGI0, SGI236051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236054, SGI236056, SGI236056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52315[] = {SGI236052, SGI236053, SGI236055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236057, SGI0, SGI0, SGI0, SGI236058, SGI236058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236060, SGI0, NULL};
const Segment * const SG52316[] = {SGI236061, SGI0, SGI236062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236063, SGI236064, SGI236064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52317[] = {SGI236065, SGI0, SGI236066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236067, SGI236068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52318[] = {SGI236069, SGI0, SGI236070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236071, SGI236072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52319[] = {SGI236073, SGI236074, SGI236075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236079, SGI0, SGI236080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52320[] = {SGI236076, SGI236077, SGI236078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236081, SGI0, SGI0, SGI0, SGI236082, SGI236082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52321[] = {SGI236083, SGI236084, SGI236085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236088, SGI0, SGI236090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52322[] = {SGI236086, SGI236087, SGI236089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236091, SGI0, SGI0, SGI0, SGI236092, SGI236092, SGI0, SGI236095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52323[] = {SGI236093, SGI236094, SGI236096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236097, SGI236097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236098, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52324[] = {SGI236099, SGI236100, SGI236101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236102, SGI0, SGI236103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52325[] = {SGI236104, SGI236105, SGI236106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236107, SGI236107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236108, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52326[] = {SGI236109, SGI236110, SGI236111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236112, SGI0, SGI0, SGI0, SGI236113, SGI236113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236114, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52327[] = {SGI236115, SGI0, SGI236116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236117, SGI236118, SGI236118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52328[] = {SGI236119, SGI0, SGI236120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236121, SGI236122, SGI236122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52329[] = {SGI236123, SGI236124, SGI236125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236126, SGI0, SGI0, SGI0, SGI236127, SGI0, SGI236128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52330[] = {SGI236129, SGI236130, SGI236131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236132, SGI0, SGI0, SGI0, SGI236133, SGI0, SGI0, SGI236134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52331[] = {SGI236135, SGI236136, SGI236137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236138, SGI0, SGI0, SGI0, SGI236140, SGI236140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236142, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52332[] = {SGI236139, SGI0, SGI236141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236143, SGI236144, SGI236144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52333[] = {SGI236145, SGI236146, SGI236147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236148, SGI0, SGI0, SGI0, SGI236149, SGI236149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52334[] = {SGI236151, SGI236152, SGI236153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236154, SGI0, SGI0, SGI0, SGI236155, SGI236155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52335[] = {SGI236157, SGI236158, SGI236159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236160, SGI236160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236161, NULL};
const Segment * const SG52336[] = {SGI236162, SGI236163, SGI236164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236165, SGI0, SGI0, SGI0, SGI236166, SGI236166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52337[] = {SGI236168, SGI236169, SGI236170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236171, SGI0, SGI0, SGI0, SGI236172, SGI236172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52338[] = {SGI236173, SGI0, SGI236174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236175, SGI0, SGI0, SGI0, SGI236176, SGI236176, SGI0, SGI0, SGI0, SGI236177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52339[] = {SGI236178, SGI236179, SGI236180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236181, SGI236181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236182, NULL};
const Segment * const SG52340[] = {SGI236183, SGI236184, SGI236185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236186, SGI236186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236187, NULL};
const Segment * const SG52341[] = {SGI236188, SGI236189, SGI236190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236191, SGI236191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236192, NULL};
const Segment * const SG52342[] = {SGI236193, SGI236194, SGI236195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236196, SGI236196, SGI0, SGI0, SGI0, SGI0, SGI236199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52343[] = {SGI236197, SGI0, SGI236198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236200, SGI236201, SGI236201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52344[] = {SGI236202, SGI0, SGI236203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236205, SGI236207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236208, SGI0, SGI0, NULL};
const Segment * const SG52345[] = {SGI236204, SGI0, SGI236206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236209, SGI236210, SGI236210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52346[] = {SGI236211, SGI236212, SGI236213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236214, SGI0, SGI0, SGI0, SGI236215, SGI236215, SGI0, SGI236216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52347[] = {SGI236217, SGI236218, SGI236219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236220, SGI0, SGI0, SGI0, SGI236221, SGI236221, SGI0, SGI236223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52348[] = {SGI236222, SGI0, SGI236224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236225, SGI236226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236227, SGI0, SGI0, NULL};
const Segment * const SG52349[] = {SGI236228, SGI0, SGI236229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236230, SGI236231, SGI236231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52350[] = {SGI236232, SGI0, SGI236233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236234, SGI236235, SGI236235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52351[] = {SGI236236, SGI236237, SGI236238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236239, SGI0, SGI0, SGI0, SGI236240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52352[] = {SGI236241, SGI0, SGI236242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236243, SGI236244, SGI236244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52353[] = {SGI236245, SGI0, SGI236246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236247, SGI236248, SGI236248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52354[] = {SGI236249, SGI0, SGI236250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236251, SGI236252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52355[] = {SGI236253, SGI0, SGI236254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236255, SGI236256, SGI236256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236257, SGI0, SGI0, NULL};
const Segment * const SG52356[] = {SGI236258, SGI236259, SGI236260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236261, SGI0, SGI0, SGI0, SGI236262, SGI236262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236264, SGI0, NULL};
const Segment * const SG52357[] = {SGI236265, SGI236266, SGI236267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236268, SGI0, SGI0, SGI0, SGI236269, SGI236269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52358[] = {SGI236270, SGI236271, SGI236272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236273, SGI0, SGI0, SGI0, SGI236274, SGI236274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236275, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52359[] = {SGI236276, SGI236277, SGI236278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236279, SGI236279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52360[] = {SGI236280, SGI236281, SGI236282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236283, SGI0, SGI0, SGI0, SGI236284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236285, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52361[] = {SGI236286, SGI236287, SGI236288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236290, SGI236290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236293, NULL};
const Segment * const SG52362[] = {SGI236289, SGI236291, SGI236292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236294, SGI0, SGI0, SGI0, SGI236295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236296, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52363[] = {SGI236297, SGI236298, SGI236299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236300, SGI0, SGI0, SGI0, SGI236301, SGI0, SGI0, SGI236302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52364[] = {SGI236303, SGI0, SGI236304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236308, SGI236309, SGI236309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52365[] = {SGI236305, SGI236306, SGI236307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236310, SGI0, SGI0, SGI0, SGI236311, SGI0, SGI0, SGI236312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52366[] = {SGI236313, SGI236314, SGI236315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236318, SGI0, SGI0, SGI0, SGI236319, SGI0, SGI0, SGI236320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52367[] = {SGI236316, SGI0, SGI236317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236321, SGI236322, SGI236322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52368[] = {SGI236323, SGI236324, SGI236325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236326, SGI0, SGI236327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52369[] = {SGI236328, SGI0, SGI236329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236330, SGI236331, SGI236331, SGI0, SGI0, SGI0, SGI0, SGI236332, SGI0, NULL};
const Segment * const SG52370[] = {SGI236333, SGI0, SGI236334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236335, SGI0, SGI0, SGI0, SGI236336, SGI236336, SGI0, SGI0, SGI0, SGI236337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52371[] = {SGI236338, SGI0, SGI236339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236340, SGI0, SGI0, SGI0, SGI236341, SGI236341, SGI0, SGI0, SGI0, SGI236342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52372[] = {SGI236343, SGI236344, SGI236345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236346, SGI0, SGI0, SGI0, SGI236347, SGI236347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236348, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52373[] = {SGI236349, SGI0, SGI236350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236351, SGI236352, SGI236352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52374[] = {SGI236353, SGI0, SGI236354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236355, SGI236356, SGI236356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52375[] = {SGI236357, SGI0, SGI236358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236359, SGI236360, SGI236360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52376[] = {SGI236361, SGI236362, SGI236363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236364, SGI0, SGI0, SGI0, SGI236365, SGI236365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236367, SGI0, NULL};
const Segment * const SG52377[] = {SGI236368, SGI236369, SGI236370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236371, SGI0, SGI0, SGI0, SGI236372, SGI236372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236374, SGI0, NULL};
const Segment * const SG52378[] = {SGI236375, SGI236376, SGI236377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236378, SGI0, SGI0, SGI0, SGI236379, SGI236379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236380, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52379[] = {SGI236381, SGI236382, SGI236383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236384, SGI0, SGI0, SGI0, SGI236385, SGI236385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52380[] = {SGI236386, SGI0, SGI236387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236388, SGI236389, SGI236389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52381[] = {SGI236390, SGI0, SGI236391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236392, SGI0, SGI0, SGI0, SGI236393, SGI236393, SGI0, SGI0, SGI236394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52382[] = {SGI236395, SGI236396, SGI236397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236398, SGI0, SGI0, SGI0, SGI236399, SGI236399, SGI236400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52383[] = {SGI236401, SGI0, SGI236402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236403, SGI236404, SGI236404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236405, SGI0, SGI0, NULL};
const Segment * const SG52384[] = {SGI236406, SGI236407, SGI236408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236409, SGI0, SGI0, SGI0, SGI236410, SGI236410, SGI0, SGI236411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52385[] = {SGI236412, SGI236413, SGI236414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236415, SGI0, SGI0, SGI0, SGI236416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52386[] = {SGI236417, SGI0, SGI236418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236419, SGI236420, SGI236420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52387[] = {SGI236421, SGI0, SGI236422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236423, SGI236424, SGI236424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52388[] = {SGI236425, SGI0, SGI236426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236427, SGI236428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52389[] = {SGI236429, SGI236430, SGI236431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236432, SGI236432, SGI0, SGI0, SGI236433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52390[] = {SGI236434, SGI236435, SGI236436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236437, SGI236437, SGI0, SGI0, SGI0, SGI0, SGI236438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52391[] = {SGI236439, SGI236440, SGI236441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236442, SGI0, SGI0, SGI0, SGI236443, SGI236443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52392[] = {SGI236444, SGI236445, SGI236446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236447, SGI0, SGI0, SGI0, SGI236448, SGI236449, SGI0, SGI0, SGI0, SGI0, SGI236450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52393[] = {SGI236451, SGI236452, SGI236453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236454, SGI236454, SGI0, SGI0, SGI0, SGI0, SGI236455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52394[] = {SGI236456, SGI236457, SGI236458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236459, SGI0, SGI0, SGI0, SGI236460, SGI236460, SGI0, SGI236461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52395[] = {SGI236462, SGI236463, SGI236464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236465, SGI236465, SGI0, SGI0, SGI0, SGI0, SGI236466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52396[] = {SGI236469, SGI0, SGI236471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236473, SGI236475, SGI236475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52397[] = {SGI236467, SGI236468, SGI236470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236472, SGI0, SGI236474, SGI236474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52398[] = {SGI236476, SGI236477, SGI236478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236479, SGI0, SGI0, SGI0, SGI236480, SGI236480, SGI0, SGI236481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52399[] = {SGI236482, SGI0, SGI236483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236484, SGI236485, SGI236485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52400[] = {SGI236486, SGI0, SGI236487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236488, SGI236489, SGI236489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236490, SGI0, SGI0, NULL};
const Segment * const SG52401[] = {SGI236491, SGI0, SGI236492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236493, SGI236494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52402[] = {SGI236495, SGI0, SGI236496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236497, SGI0, SGI0, SGI0, SGI236498, SGI236498, SGI0, SGI0, SGI0, SGI236499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52403[] = {SGI236500, SGI236501, SGI236502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236503, SGI0, SGI0, SGI0, SGI236504, SGI0, SGI236505, SGI0, SGI0, SGI0, SGI0, SGI236506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52404[] = {SGI236507, SGI236508, SGI236509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236510, SGI0, SGI0, SGI0, SGI236511, SGI0, SGI236512, SGI0, SGI0, SGI0, SGI0, SGI236513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52405[] = {SGI236514, SGI236515, SGI236516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236517, SGI0, SGI0, SGI0, SGI236518, SGI0, SGI236519, SGI0, SGI0, SGI0, SGI0, SGI236520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52406[] = {SGI236521, SGI236522, SGI236523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236524, SGI0, SGI0, SGI0, SGI236525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236526, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52407[] = {SGI236527, SGI0, SGI236528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236529, SGI0, SGI0, SGI0, SGI236530, SGI236530, SGI0, SGI0, SGI0, SGI236531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52408[] = {SGI236532, SGI236533, SGI236534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236535, SGI0, SGI0, SGI0, SGI236536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52409[] = {SGI236538, SGI236539, SGI236540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236541, SGI0, SGI0, SGI0, SGI236542, SGI236542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236544, SGI0, NULL};
const Segment * const SG52410[] = {SGI236545, SGI236546, SGI236547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236548, SGI0, SGI0, SGI0, SGI236549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52411[] = {SGI236551, SGI0, SGI236552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236553, SGI236554, SGI236554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52412[] = {SGI236555, SGI0, SGI236556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236557, SGI236558, SGI236558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236559, SGI0, SGI0, NULL};
const Segment * const SG52413[] = {SGI236560, SGI0, SGI236561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236562, SGI236563, SGI236563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52414[] = {SGI236564, SGI0, SGI236565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236566, SGI236567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52415[] = {SGI236568, SGI0, SGI236569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236570, SGI236571, SGI236571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52416[] = {SGI236572, SGI0, SGI236573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236574, SGI236575, SGI236575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52417[] = {SGI236576, SGI0, SGI236577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236578, SGI236579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52418[] = {SGI236580, SGI0, SGI236581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236582, SGI236583, SGI236583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52419[] = {SGI236584, SGI236585, SGI236586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236587, SGI0, SGI0, SGI0, SGI236588, SGI236588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236589, NULL};
const Segment * const SG52420[] = {SGI236590, SGI0, SGI236591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236592, SGI236593, SGI236593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52421[] = {SGI236594, SGI0, SGI236595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236598, SGI236599, SGI236599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52422[] = {SGI236596, SGI0, SGI236597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236600, SGI236601, SGI236601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52423[] = {SGI236602, SGI236603, SGI236604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236605, SGI0, SGI0, SGI0, SGI236606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52424[] = {SGI236607, SGI0, SGI236608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236609, SGI236610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52425[] = {SGI236611, SGI0, SGI236612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236613, SGI236614, SGI236614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52426[] = {SGI236615, SGI236616, SGI236617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236618, SGI236618, SGI0, SGI0, SGI0, SGI0, SGI236619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52427[] = {SGI236620, SGI236621, SGI236622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236623, SGI236623, SGI0, SGI0, SGI0, SGI0, SGI236624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52428[] = {SGI236625, SGI236626, SGI236627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236628, SGI0, SGI0, SGI0, SGI236629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52429[] = {SGI236630, SGI0, SGI236631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236632, SGI236633, SGI236633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52430[] = {SGI236634, SGI0, SGI236635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236636, SGI236637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52431[] = {SGI236638, SGI0, SGI236639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236640, SGI236641, SGI236641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52432[] = {SGI236642, SGI0, SGI236643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236644, SGI0, SGI0, SGI0, SGI236645, SGI236645, SGI0, SGI0, SGI0, SGI0, SGI236646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52433[] = {SGI236647, SGI0, SGI236648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236649, SGI236650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236651, SGI0, SGI0, NULL};
const Segment * const SG52434[] = {SGI236652, SGI236653, SGI236654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236657, SGI0, SGI0, SGI0, SGI236658, SGI236658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52435[] = {SGI236655, SGI0, SGI236656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236659, SGI236660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236661, SGI0, SGI0, NULL};
const Segment * const SG52436[] = {SGI236662, SGI236663, SGI236664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236665, SGI0, SGI236666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52437[] = {SGI236667, SGI0, SGI236668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236669, SGI236670, SGI236670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52438[] = {SGI236671, SGI236672, SGI236673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236674, SGI236674, SGI0, SGI0, SGI236675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52439[] = {SGI236676, SGI0, SGI236677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236678, SGI236679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52440[] = {SGI236680, SGI236681, SGI236682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236683, SGI236683, SGI0, SGI0, SGI236684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52441[] = {SGI236685, SGI0, SGI236686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236692, SGI236693, SGI236693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52442[] = {SGI236688, SGI0, SGI236691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236694, SGI236696, SGI236696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52443[] = {SGI236687, SGI236689, SGI236690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236695, SGI236695, SGI0, SGI0, SGI236697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52444[] = {SGI236698, SGI0, SGI236699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236702, SGI236703, SGI236703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52445[] = {SGI236700, SGI0, SGI236701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236704, SGI236705, SGI236705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52446[] = {SGI236706, SGI236707, SGI236708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236712, SGI236712, SGI0, SGI0, SGI236713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52447[] = {SGI236709, SGI236710, SGI236711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236714, SGI0, SGI0, SGI0, SGI236715, SGI236715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52448[] = {SGI236716, SGI236717, SGI236718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236721, SGI236721, SGI0, SGI0, SGI0, SGI0, SGI236723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52449[] = {SGI236719, SGI236720, SGI236722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236724, SGI0, SGI0, SGI0, SGI236725, SGI236725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52450[] = {SGI236726, SGI236727, SGI236728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236731, SGI236731, SGI0, SGI0, SGI0, SGI0, SGI236732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52451[] = {SGI236729, SGI0, SGI236730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236733, SGI236734, SGI236734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52452[] = {SGI236735, SGI236736, SGI236737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236738, SGI0, SGI0, SGI0, SGI236739, SGI236739, SGI0, SGI236740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52453[] = {SGI236741, SGI0, SGI236742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236743, SGI236744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52454[] = {SGI236745, SGI0, SGI236746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236747, SGI236748, SGI236748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236749, SGI0, SGI0, NULL};
const Segment * const SG52455[] = {SGI236750, SGI236751, SGI236752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236753, SGI0, SGI0, SGI0, SGI236754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236755, SGI0, SGI236756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52456[] = {SGI236757, SGI236758, SGI236759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236761, SGI0, SGI0, SGI0, SGI236763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52457[] = {SGI236760, SGI0, SGI236762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236765, SGI236766, SGI236766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52458[] = {SGI236767, SGI0, SGI236768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236769, SGI236770, SGI236770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52459[] = {SGI236771, SGI236772, SGI236773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236774, SGI0, SGI0, SGI0, SGI236775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52460[] = {SGI236776, SGI236777, SGI236778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236780, SGI236780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236784, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52461[] = {SGI236779, SGI236781, SGI236782, SGI236783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236785, SGI0, SGI0, SGI0, SGI236786, SGI236787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52462[] = {SGI236794, SGI0, SGI236795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236796, SGI236797, SGI236797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52463[] = {SGI236798, SGI236799, SGI236800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236801, SGI0, SGI0, SGI0, SGI236802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52464[] = {SGI236803, SGI236804, SGI236805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236806, SGI236806, SGI0, SGI0, SGI236807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52465[] = {SGI236808, SGI0, SGI236809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236810, SGI0, SGI0, SGI0, SGI236811, SGI236811, SGI0, SGI0, SGI236812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52466[] = {SGI236813, SGI236814, SGI236815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236816, SGI0, SGI0, SGI0, SGI236817, SGI0, SGI0, SGI236818, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52467[] = {SGI236819, SGI236820, SGI236821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236822, SGI0, SGI0, SGI0, SGI236823, SGI0, SGI0, SGI236824, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52468[] = {SGI236825, SGI236826, SGI236827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236828, SGI0, SGI0, SGI0, SGI236829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52469[] = {SGI236830, SGI0, SGI236831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52470[] = {SGI236832, SGI0, SGI236833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236834, SGI0, SGI0, SGI0, SGI236835, SGI236835, SGI0, SGI0, SGI236836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52471[] = {SGI236837, SGI0, SGI236838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236839, SGI236840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52472[] = {SGI236841, SGI236842, SGI236843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236844, SGI0, SGI0, SGI0, SGI236845, SGI236845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52473[] = {SGI236846, SGI236847, SGI236848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236849, SGI0, SGI0, SGI0, SGI236850, SGI236850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52474[] = {SGI236851, SGI0, SGI236852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236853, SGI236854, SGI236854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52475[] = {SGI236855, SGI0, SGI236856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236857, SGI236858, SGI236858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52476[] = {SGI236859, SGI0, SGI236860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236861, SGI236862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52477[] = {SGI236863, SGI0, SGI236864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236865, SGI236866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52478[] = {SGI236867, SGI0, SGI236868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236869, SGI236870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52479[] = {SGI236871, SGI0, SGI236872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236873, SGI236874, SGI236874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52480[] = {SGI236875, SGI0, SGI236876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236877, SGI236878, SGI236878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52481[] = {SGI236879, SGI0, SGI236880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236881, SGI236882, SGI236882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52482[] = {SGI236883, SGI0, SGI236884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236885, SGI236886, SGI236886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52483[] = {SGI236887, SGI236888, SGI236889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236890, SGI0, SGI0, SGI0, SGI236891, SGI236891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52484[] = {SGI236892, SGI236893, SGI236894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236895, SGI0, SGI0, SGI0, SGI236896, SGI236896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52485[] = {SGI236897, SGI236898, SGI236899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236900, SGI0, SGI0, SGI0, SGI236901, SGI236901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236902, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52486[] = {SGI236903, SGI0, SGI236904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236905, SGI236906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52487[] = {SGI236907, SGI0, SGI236908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236909, SGI236910, SGI236910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52488[] = {SGI236911, SGI236912, SGI236913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236914, SGI0, SGI0, SGI0, SGI236915, SGI236915, SGI0, SGI236916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52489[] = {SGI236917, SGI236918, SGI236919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236920, SGI0, SGI0, SGI0, SGI236921, SGI236921, SGI0, SGI236922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52490[] = {SGI236923, SGI236924, SGI236925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236926, SGI0, SGI0, SGI0, SGI236927, SGI236927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52491[] = {SGI236928, SGI236929, SGI236930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236931, SGI0, SGI0, SGI0, SGI236932, SGI236932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52492[] = {SGI236933, SGI236934, SGI236935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236936, SGI236936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236937, NULL};
const Segment * const SG52493[] = {SGI236938, SGI0, SGI236939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236940, SGI236941, SGI236941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236942, SGI0, SGI0, NULL};
const Segment * const SG52494[] = {SGI236943, SGI0, SGI236944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236945, SGI236946, SGI236946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236947, SGI0, SGI0, NULL};
const Segment * const SG52495[] = {SGI236788, SGI236789, SGI236790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236791, SGI0, SGI0, SGI0, SGI236792, SGI0, SGI0, SGI236793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52496[] = {SGI236948, SGI0, SGI236949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236950, SGI0, SGI0, SGI0, SGI236951, SGI236951, SGI0, SGI0, SGI0, SGI236952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52497[] = {SGI236953, SGI236954, SGI236955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236956, SGI0, SGI0, SGI0, SGI236957, SGI0, SGI236958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52498[] = {SGI236959, SGI236960, SGI236961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236962, SGI0, SGI0, SGI0, SGI236963, SGI0, SGI236964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52499[] = {SGI236965, SGI236966, SGI236967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236968, SGI0, SGI0, SGI0, SGI236969, SGI236969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52500[] = {SGI236970, SGI236971, SGI236972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236973, SGI0, SGI0, SGI0, SGI236974, SGI236974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52501[] = {SGI236975, SGI236976, SGI236977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236978, SGI236978, SGI0, SGI0, SGI0, SGI0, SGI236979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52502[] = {SGI236980, SGI0, SGI236981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236982, SGI0, SGI0, SGI0, SGI236983, SGI236983, SGI0, SGI0, SGI236984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52503[] = {SGI236985, SGI0, SGI236986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236987, SGI236988, SGI236988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52504[] = {SGI236989, SGI236990, SGI236991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236992, SGI236992, SGI0, SGI0, SGI0, SGI0, SGI236993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52505[] = {SGI236994, SGI0, SGI236995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI236996, SGI236997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52506[] = {SGI236998, SGI236999, SGI237000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237001, SGI0, SGI0, SGI0, SGI237002, SGI237002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52507[] = {SGI237003, SGI0, SGI237004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237005, SGI237006, SGI237006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52508[] = {SGI237007, SGI0, SGI237008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237009, SGI0, SGI0, SGI0, SGI237010, SGI237010, SGI0, SGI0, SGI237011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52509[] = {SGI237012, SGI0, SGI237013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237014, SGI237015, SGI237015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52510[] = {SGI237016, SGI0, SGI237017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237018, SGI237019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52511[] = {SGI237020, SGI237021, SGI237022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237023, SGI0, SGI0, SGI0, SGI237024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52512[] = {SGI237026, SGI0, SGI237027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237028, SGI0, SGI0, SGI0, SGI237029, SGI237029, SGI0, SGI0, SGI0, SGI237030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52513[] = {SGI237031, SGI237032, SGI237033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237034, SGI0, SGI0, SGI0, SGI237036, SGI237036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52514[] = {SGI237035, SGI0, SGI237037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237038, SGI0, SGI0, SGI0, SGI237039, SGI237039, SGI0, SGI0, SGI0, SGI237040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52515[] = {SGI237041, SGI237042, SGI237043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237044, SGI237044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52516[] = {SGI237045, SGI237046, SGI237047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237048, SGI0, SGI0, SGI0, SGI237049, SGI0, SGI0, SGI0, SGI237050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52517[] = {SGI237051, SGI237052, SGI237053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237054, SGI237054, SGI0, SGI0, SGI237055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52518[] = {SGI237057, SGI237058, SGI237059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237060, SGI237060, SGI0, SGI0, SGI237061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52519[] = {SGI237056, SGI237062, SGI237063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237064, SGI0, SGI0, SGI0, SGI237065, SGI237065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52520[] = {SGI237066, SGI0, SGI237067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237068, SGI237069, SGI237069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52521[] = {SGI237070, SGI0, SGI237071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237072, SGI237073, SGI237073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52522[] = {SGI237074, SGI0, SGI237075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237076, SGI0, SGI0, SGI0, SGI237077, SGI237077, SGI0, SGI0, SGI0, SGI0, SGI237078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52523[] = {SGI237079, SGI0, SGI237080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237081, SGI237082, SGI237082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237083, SGI0, SGI0, NULL};
const Segment * const SG52524[] = {SGI237084, SGI0, SGI237085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237086, SGI237087, SGI237087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52525[] = {SGI237088, SGI237089, SGI237090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237091, SGI0, SGI0, SGI0, SGI237092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52526[] = {SGI237093, SGI0, SGI237094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237095, SGI237096, SGI237096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52527[] = {SGI237097, SGI0, SGI237098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237099, SGI237100, SGI237100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52528[] = {SGI237101, SGI0, SGI237102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237103, SGI237104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237105, SGI0, SGI0, NULL};
const Segment * const SG52529[] = {SGI237106, SGI237107, SGI237108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237109, SGI0, SGI0, SGI0, SGI237110, SGI237110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52530[] = {SGI237111, SGI0, SGI237112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237113, SGI237114, SGI237114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237115, SGI0, SGI0, NULL};
const Segment * const SG52531[] = {SGI237117, SGI0, SGI237119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237122, SGI237123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52532[] = {SGI237116, SGI237118, SGI237120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237121, SGI0, SGI0, SGI0, SGI237124, SGI237124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52533[] = {SGI237125, SGI237126, SGI237127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237128, SGI0, SGI0, SGI0, SGI237129, SGI237129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52534[] = {SGI237130, SGI0, SGI237131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237132, SGI237133, SGI237133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52535[] = {SGI237134, SGI237135, SGI237136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237137, SGI0, SGI0, SGI0, SGI237138, SGI237138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237139, SGI0, SGI0, NULL};
const Segment * const SG52536[] = {SGI237140, SGI237141, SGI237142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237143, SGI0, SGI0, SGI0, SGI237144, SGI237144, SGI237145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52537[] = {SGI237146, SGI0, SGI237147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237148, SGI237149, SGI237149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52538[] = {SGI237150, SGI237151, SGI237152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237156, SGI0, SGI0, SGI0, SGI237157, SGI237157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52539[] = {SGI237153, SGI237154, SGI237155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237158, SGI237158, SGI0, SGI0, SGI237159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52540[] = {SGI237160, SGI0, SGI237161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237162, SGI237163, SGI237163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237164, SGI0, SGI0, NULL};
const Segment * const SG52541[] = {SGI237165, SGI237166, SGI237167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237168, SGI237168, SGI0, SGI0, SGI0, SGI0, SGI237169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52542[] = {SGI237170, SGI237171, SGI237172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237173, SGI237173, SGI0, SGI0, SGI237174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52543[] = {SGI237175, SGI0, SGI237176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237177, SGI237178, SGI237178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52544[] = {SGI237179, SGI0, SGI237180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237181, SGI0, SGI0, SGI0, SGI237182, SGI237182, SGI0, SGI0, SGI0, SGI237183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52545[] = {SGI237184, SGI237185, SGI237186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237187, SGI0, SGI0, SGI0, SGI237188, SGI237188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237190, SGI0, NULL};
const Segment * const SG52546[] = {SGI237191, SGI237192, SGI237193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237194, SGI0, SGI0, SGI0, SGI237195, SGI237196, SGI0, SGI0, SGI237197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52547[] = {SGI237198, SGI237199, SGI237200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237201, SGI0, SGI0, SGI0, SGI237202, SGI237202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52548[] = {SGI237203, SGI237204, SGI237205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237206, SGI0, SGI0, SGI0, SGI237207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52549[] = {SGI237209, SGI0, SGI237210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52550[] = {SGI237211, SGI237212, SGI237213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237216, SGI0, SGI0, SGI0, SGI237217, SGI0, SGI0, SGI237218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52551[] = {SGI237214, SGI0, SGI237215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237219, SGI237220, SGI237220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52552[] = {SGI237221, SGI237222, SGI237223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237225, SGI0, SGI0, SGI0, SGI237227, SGI0, SGI237228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52553[] = {SGI237224, SGI0, SGI237226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237229, SGI237230, SGI237230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52554[] = {SGI237231, SGI237232, SGI237233, SGI0, SGI0, SGI0, SGI237234, SGI237235, SGI0, SGI237236, SGI0, SGI0, SGI0, SGI237237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52555[] = {SGI237238, SGI0, SGI237239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237240, SGI237241, SGI237241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237243, SGI0, SGI0, NULL};
const Segment * const SG52556[] = {SGI237242, SGI0, SGI237244, SGI0, SGI0, SGI0, SGI237245, SGI237246, SGI0, SGI237247, SGI0, SGI0, SGI0, SGI237248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52557[] = {SGI237249, SGI237250, SGI237251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237252, SGI0, SGI0, SGI0, SGI237253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237254, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52558[] = {SGI237255, SGI0, SGI237256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237257, SGI237258, SGI237258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52559[] = {SGI0, SGI237259, SGI0, SGI237260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52560[] = {SGI0, SGI237261, SGI0, SGI237262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237266, SGI0, SGI0, SGI0, SGI237267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52561[] = {SGI237263, SGI237264, SGI237265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237268, SGI0, SGI0, SGI0, SGI237269, SGI237269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52562[] = {SGI237270, SGI0, SGI237271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237272, SGI237273, SGI237273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52563[] = {SGI237274, SGI237275, SGI237276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237279, SGI0, SGI0, SGI0, SGI237280, SGI237280, SGI0, SGI237282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52564[] = {SGI237277, SGI0, SGI237278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237281, SGI0, SGI0, SGI0, SGI237283, SGI237283, SGI0, SGI0, SGI0, SGI237284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52565[] = {SGI237285, SGI237286, SGI237287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237288, SGI237288, SGI0, SGI0, SGI237289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52566[] = {SGI237290, SGI237291, SGI237292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237293, SGI0, SGI0, SGI0, SGI237294, SGI237294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237295, SGI0, SGI0, NULL};
const Segment * const SG52567[] = {SGI237296, SGI237297, SGI237298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237299, SGI0, SGI0, SGI0, SGI237300, SGI237300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237301, SGI0, SGI0, NULL};
const Segment * const SG52568[] = {SGI237302, SGI0, SGI237303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237304, SGI237305, SGI237305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52569[] = {SGI237306, SGI237307, SGI237308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237309, SGI237309, SGI0, SGI0, SGI237310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52570[] = {SGI237311, SGI237312, SGI237313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237314, SGI0, SGI0, SGI0, SGI237315, SGI237315, SGI0, SGI237316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52571[] = {SGI237317, SGI237318, SGI237319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237320, SGI237320, SGI0, SGI0, SGI0, SGI0, SGI237321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52572[] = {SGI237322, SGI237323, SGI237324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237325, SGI0, SGI0, SGI0, SGI237326, SGI237326, SGI0, SGI237327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52573[] = {SGI237328, SGI237329, SGI237330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237331, SGI0, SGI0, SGI0, SGI237333, SGI237333, SGI0, SGI237335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52574[] = {SGI237332, SGI0, SGI237334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237336, SGI237337, SGI237337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52575[] = {SGI237338, SGI0, SGI237339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237340, SGI237342, SGI237342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52576[] = {SGI237341, SGI0, SGI237343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237344, SGI237345, SGI237345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52577[] = {SGI237346, SGI237347, SGI237348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237349, SGI0, SGI0, SGI0, SGI237350, SGI237350, SGI0, SGI237351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52578[] = {SGI237352, SGI237353, SGI237354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237355, SGI0, SGI0, SGI0, SGI237356, SGI237356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52579[] = {SGI237357, SGI0, SGI237358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237359, SGI237360, SGI237360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52580[] = {SGI237361, SGI237362, SGI237363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237364, SGI0, SGI0, SGI0, SGI237365, SGI237365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237366, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52581[] = {SGI237367, SGI0, SGI237368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237369, SGI237370, SGI237370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52582[] = {SGI237371, SGI0, SGI237372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237373, SGI237374, SGI237374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52583[] = {SGI237375, SGI0, SGI237376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237377, SGI237378, SGI237378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52584[] = {SGI237379, SGI237380, SGI237381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237382, SGI237382, SGI0, SGI0, SGI0, SGI0, SGI237383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52585[] = {SGI237384, SGI0, SGI237385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237386, SGI0, SGI0, SGI0, SGI237387, SGI237387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52586[] = {SGI237388, SGI0, SGI237389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237390, SGI237391, SGI237391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52587[] = {SGI237392, SGI237393, SGI237394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237395, SGI0, SGI0, SGI0, SGI237396, SGI237396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52588[] = {SGI237397, SGI237398, SGI237399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237400, SGI0, SGI0, SGI0, SGI237401, SGI237401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52589[] = {SGI237402, SGI0, SGI237403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237404, SGI237405, SGI237405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52590[] = {SGI237406, SGI237407, SGI237408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237409, SGI0, SGI0, SGI0, SGI237410, SGI237410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237412, SGI0, NULL};
const Segment * const SG52591[] = {SGI237413, SGI0, SGI237414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237415, SGI237416, SGI237416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52592[] = {SGI237417, SGI0, SGI237418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237419, SGI237420, SGI237420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52593[] = {SGI237421, SGI237422, SGI237423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237424, SGI0, SGI0, SGI0, SGI237425, SGI237425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52594[] = {SGI237426, SGI0, SGI237427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237428, SGI0, SGI0, SGI0, SGI237429, SGI237429, SGI0, SGI0, SGI0, SGI237430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52595[] = {SGI237431, SGI0, SGI237432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237433, SGI0, SGI0, SGI0, SGI237434, SGI237434, SGI0, SGI0, SGI0, SGI237435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52596[] = {SGI237436, SGI0, SGI237437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52597[] = {SGI237438, SGI237439, SGI237440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237441, SGI0, SGI0, SGI0, SGI237443, SGI237443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52598[] = {SGI237442, SGI237444, SGI237445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237446, SGI0, SGI0, SGI0, SGI237447, SGI0, SGI0, SGI237448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52599[] = {SGI237449, SGI0, SGI237450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237451, SGI237452, SGI237452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52600[] = {SGI237453, SGI0, SGI237454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237455, SGI237456, SGI237456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52601[] = {SGI237457, SGI237459, SGI237460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237462, SGI0, SGI0, SGI0, SGI237464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52602[] = {SGI237458, SGI0, SGI237461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237463, SGI237465, SGI237465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52603[] = {SGI237466, SGI0, SGI237467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237470, SGI237471, SGI237471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52604[] = {SGI237468, SGI0, SGI237469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237472, SGI237473, SGI237473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52605[] = {SGI237474, SGI237475, SGI237476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237478, SGI0, SGI0, SGI0, SGI237480, SGI0, SGI0, SGI237481, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52606[] = {SGI237477, SGI0, SGI237479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237482, SGI237483, SGI237483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52607[] = {SGI237484, SGI0, SGI237485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237486, SGI237487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237488, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52608[] = {SGI237489, SGI237490, SGI237491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237492, SGI0, SGI0, SGI0, SGI237493, SGI237493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237496, SGI0, NULL};
const Segment * const SG52609[] = {SGI237495, SGI0, SGI237497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237498, SGI237499, SGI237499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52610[] = {SGI237500, SGI0, SGI237501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237502, SGI0, SGI0, SGI0, SGI237503, SGI237503, SGI0, SGI0, SGI237504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52611[] = {SGI237505, SGI0, SGI237506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237507, SGI237508, SGI237508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52612[] = {SGI237509, SGI0, SGI237510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237511, SGI237512, SGI237512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52613[] = {SGI237513, SGI237514, SGI237515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237516, SGI0, SGI0, SGI0, SGI237517, SGI237517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237519, SGI0, NULL};
const Segment * const SG52614[] = {SGI237520, SGI0, SGI237521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237522, SGI237523, SGI237523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52615[] = {SGI237524, SGI0, SGI237525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237526, SGI237527, SGI237527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52616[] = {SGI237528, SGI0, SGI237529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237530, SGI237531, SGI237531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52617[] = {SGI237532, SGI237533, SGI237534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237535, SGI0, SGI0, SGI0, SGI237536, SGI237536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52618[] = {SGI237537, SGI0, SGI237538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237539, SGI237540, SGI237540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52619[] = {SGI237541, SGI0, SGI237543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237545, SGI237547, SGI237547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52620[] = {SGI237542, SGI0, SGI237544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237546, SGI237548, SGI237548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52621[] = {SGI237549, SGI237550, SGI237551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237552, SGI237552, SGI0, SGI0, SGI0, SGI0, SGI237553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52622[] = {SGI237554, SGI0, SGI237555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237559, SGI237560, SGI237560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52623[] = {SGI237556, SGI237557, SGI237558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237561, SGI0, SGI0, SGI0, SGI237562, SGI237562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237563, NULL};
const Segment * const SG52624[] = {SGI237564, SGI237565, SGI237566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237567, SGI0, SGI0, SGI0, SGI237568, SGI237568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237569, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52625[] = {SGI237570, SGI237571, SGI237572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237573, SGI0, SGI0, SGI0, SGI237574, SGI237574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52626[] = {SGI237575, SGI237577, SGI237578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237580, SGI237580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237583, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52627[] = {SGI237576, SGI0, SGI237579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237581, SGI237582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237584, SGI0, NULL};
const Segment * const SG52628[] = {SGI237585, SGI237586, SGI237587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237588, SGI0, SGI0, SGI0, SGI237589, SGI237589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52629[] = {SGI237590, SGI0, SGI237591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237592, SGI237593, SGI237593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52630[] = {SGI237594, SGI0, SGI237595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237596, SGI237597, SGI237597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52631[] = {SGI237598, SGI237599, SGI237600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237601, SGI237601, SGI0, SGI0, SGI0, SGI0, SGI237602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52632[] = {SGI237603, SGI0, SGI237604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237605, SGI237606, SGI237606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52633[] = {SGI237607, SGI237608, SGI237609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237610, SGI237610, SGI0, SGI0, SGI0, SGI0, SGI237611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52634[] = {SGI237612, SGI0, SGI237613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237614, SGI237615, SGI237615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52635[] = {SGI237616, SGI0, SGI237617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237618, SGI237619, SGI237619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52636[] = {SGI237620, SGI0, SGI237621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237622, SGI237623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52637[] = {SGI237624, SGI0, SGI237625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237626, SGI237627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237628, SGI0, SGI0, NULL};
const Segment * const SG52638[] = {SGI237629, SGI0, SGI237630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237631, SGI237632, SGI237632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237633, SGI0, SGI0, NULL};
const Segment * const SG52639[] = {SGI237634, SGI0, SGI237635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237636, SGI237637, SGI237637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52640[] = {SGI237638, SGI0, SGI237639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237640, SGI237641, SGI237641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237642, SGI0, SGI0, NULL};
const Segment * const SG52641[] = {SGI237643, SGI0, SGI237644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237645, SGI237646, SGI237646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52642[] = {SGI237650, SGI237651, SGI237652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237654, SGI0, SGI0, SGI0, SGI237656, SGI237656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52643[] = {SGI237647, SGI237648, SGI237649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237653, SGI0, SGI0, SGI0, SGI237655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237657, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52644[] = {SGI0, SGI237658, SGI237659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237660, SGI0, SGI0, SGI0, SGI0, SGI237661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52645[] = {SGI237662, SGI237663, SGI237664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237665, SGI0, SGI0, SGI0, SGI237666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52646[] = {SGI237668, SGI237669, SGI237670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237671, SGI0, SGI0, SGI0, SGI237672, SGI237672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52647[] = {SGI237673, SGI0, SGI237674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237675, SGI0, SGI0, SGI0, SGI237676, SGI237676, SGI0, SGI0, SGI0, SGI237677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52648[] = {SGI237678, SGI0, SGI237679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237680, SGI237681, SGI237681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237684, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52649[] = {SGI237682, SGI237683, SGI237685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237686, SGI0, SGI0, SGI0, SGI237687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52650[] = {SGI237688, SGI0, SGI237689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237692, SGI237693, SGI237693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52651[] = {SGI0, SGI237690, SGI0, SGI237691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52652[] = {SGI237694, SGI237695, SGI237696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237697, SGI0, SGI0, SGI0, SGI237698, SGI237698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237699, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52653[] = {SGI237700, SGI0, SGI237701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237702, SGI0, SGI0, SGI0, SGI237703, SGI237703, SGI0, SGI0, SGI0, SGI237704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52654[] = {SGI237705, SGI237706, SGI237707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237708, SGI0, SGI0, SGI0, SGI237709, SGI237709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237710, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52655[] = {SGI237711, SGI237712, SGI237713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237714, SGI0, SGI0, SGI0, SGI237715, SGI237715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237716, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52656[] = {SGI237717, SGI0, SGI237718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237719, SGI237720, SGI237720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52657[] = {SGI237721, SGI237722, SGI237723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237724, SGI0, SGI0, SGI0, SGI237725, SGI237725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237726, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52658[] = {SGI237727, SGI237728, SGI237729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237730, SGI237730, SGI0, SGI0, SGI237731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52659[] = {SGI237732, SGI0, SGI237733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237734, SGI237735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52660[] = {SGI237736, SGI0, SGI237737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237739, SGI237741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52661[] = {SGI237738, SGI0, SGI237740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237742, SGI237743, SGI237743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52662[] = {SGI237744, SGI0, SGI237745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237746, SGI0, SGI0, SGI0, SGI237747, SGI237747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237748, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52663[] = {SGI237749, SGI237750, SGI237751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237752, SGI0, SGI0, SGI0, SGI237753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52664[] = {SGI237754, SGI237755, SGI237756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237757, SGI0, SGI0, SGI0, SGI237758, SGI237758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52665[] = {SGI237759, SGI0, SGI237760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237761, SGI237762, SGI237762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52666[] = {SGI237763, SGI0, SGI237764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237767, SGI237769, SGI237769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52667[] = {SGI237765, SGI237766, SGI237768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237771, SGI237771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52668[] = {SGI237770, SGI237772, SGI237773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237774, SGI237774, SGI0, SGI0, SGI237775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52669[] = {SGI237776, SGI0, SGI237777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237781, SGI237782, SGI237782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52670[] = {SGI237778, SGI237779, SGI237780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237783, SGI0, SGI0, SGI0, SGI237786, SGI237786, SGI0, SGI237788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52671[] = {SGI237784, SGI237785, SGI237787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237789, SGI0, SGI0, SGI0, SGI237790, SGI237790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52672[] = {SGI237791, SGI237792, SGI237793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237794, SGI0, SGI0, SGI0, SGI237795, SGI237795, SGI0, SGI237796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52673[] = {SGI237797, SGI0, SGI237798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237801, SGI237802, SGI237802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52674[] = {SGI237803, SGI0, SGI237804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237805, SGI237806, SGI237806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52675[] = {SGI237807, SGI237808, SGI237809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237810, SGI0, SGI0, SGI0, SGI237811, SGI237811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52676[] = {SGI237812, SGI237813, SGI237814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237815, SGI0, SGI0, SGI0, SGI237816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52677[] = {SGI237817, SGI0, SGI237818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237819, SGI237820, SGI237820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52678[] = {SGI237821, SGI0, SGI237822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237823, SGI237824, SGI237824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52679[] = {SGI237825, SGI237826, SGI237827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237828, SGI0, SGI0, SGI0, SGI237829, SGI237829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52680[] = {SGI237830, SGI237831, SGI237832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237833, SGI0, SGI0, SGI0, SGI237834, SGI237834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237835, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52681[] = {SGI237836, SGI0, SGI237837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237838, SGI237839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52682[] = {SGI237840, SGI237841, SGI237842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237843, SGI0, SGI0, SGI0, SGI237844, SGI237844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52683[] = {SGI237845, SGI0, SGI237846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52684[] = {SGI237847, SGI237848, SGI237849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237850, SGI0, SGI0, SGI0, SGI237851, SGI237851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52685[] = {SGI237852, SGI0, SGI237853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237854, SGI0, SGI0, SGI0, SGI237855, SGI237855, SGI0, SGI0, SGI0, SGI237856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52686[] = {SGI237857, SGI0, SGI237858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237859, SGI0, SGI0, SGI0, SGI237860, SGI237860, SGI0, SGI0, SGI0, SGI237861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52687[] = {SGI237862, SGI237863, SGI237864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237865, SGI0, SGI0, SGI0, SGI237866, SGI0, SGI237867, SGI0, NULL};
const Segment * const SG52688[] = {SGI237870, SGI0, SGI237872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237874, SGI237876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52689[] = {SGI237868, SGI237869, SGI237871, SGI0, SGI0, SGI0, SGI237873, SGI0, SGI0, SGI237875, SGI0, SGI0, SGI0, SGI237877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52690[] = {SGI237878, SGI0, SGI237879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237880, SGI0, SGI0, SGI0, SGI237881, SGI237881, SGI0, SGI0, SGI0, SGI237882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52691[] = {SGI237883, SGI237884, SGI237885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237886, SGI0, SGI237887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52692[] = {SGI237888, SGI237889, SGI237890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237891, SGI0, SGI0, SGI0, SGI237892, SGI0, SGI237893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52693[] = {SGI237894, SGI237895, SGI237896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237897, SGI0, SGI0, SGI0, SGI237898, SGI0, SGI0, SGI0, SGI0, SGI237899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52694[] = {SGI237900, SGI237901, SGI237902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237903, SGI237903, SGI0, SGI0, SGI0, SGI237904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52695[] = {SGI237799, SGI0, SGI237800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237905, SGI0, SGI0, SGI0, SGI237906, SGI237906, SGI0, SGI0, SGI0, SGI0, SGI237907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52696[] = {SGI237908, SGI0, SGI237909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237910, SGI237911, SGI237911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52697[] = {SGI237912, SGI237913, SGI237914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237915, SGI237915, SGI0, SGI0, SGI237916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52698[] = {SGI237917, SGI237918, SGI237919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237920, SGI0, SGI0, SGI0, SGI237921, SGI237921, SGI0, SGI0, SGI237922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52699[] = {SGI237923, SGI0, SGI237924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237925, SGI237926, SGI237926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52700[] = {SGI237927, SGI0, SGI237928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237931, SGI237932, SGI237932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52701[] = {SGI237929, SGI0, SGI237930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237933, SGI237934, SGI237934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52702[] = {SGI237935, SGI0, SGI237936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237937, SGI237938, SGI237938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237939, SGI0, SGI0, NULL};
const Segment * const SG52703[] = {SGI237940, SGI0, SGI237941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237942, SGI237943, SGI237943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237944, SGI0, SGI0, NULL};
const Segment * const SG52704[] = {SGI237945, SGI237946, SGI237947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237948, SGI237948, SGI0, SGI0, SGI0, SGI0, SGI237949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52705[] = {SGI237950, SGI0, SGI237951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237952, SGI237953, SGI237953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52706[] = {SGI237954, SGI237955, SGI237956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237957, SGI0, SGI0, SGI0, SGI237958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52707[] = {SGI237959, SGI0, SGI237960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237961, SGI237962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52708[] = {SGI237963, SGI237964, SGI237965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237966, SGI0, SGI0, SGI0, SGI237967, SGI237967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237968, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52709[] = {SGI237969, SGI0, SGI237970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237971, SGI0, SGI0, SGI0, SGI237972, SGI237972, SGI0, SGI0, SGI0, SGI0, SGI237973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52710[] = {SGI237974, SGI237975, SGI237976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237977, SGI0, SGI0, SGI0, SGI237978, SGI237978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52711[] = {SGI237979, SGI0, SGI237980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237984, SGI237985, SGI237985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52712[] = {SGI237981, SGI237982, SGI237983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237986, SGI0, SGI0, SGI0, SGI237987, SGI237987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52713[] = {SGI237988, SGI0, SGI237989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52714[] = {SGI237990, SGI237991, SGI237992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237993, SGI0, SGI0, SGI0, SGI237994, SGI237994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52715[] = {SGI237995, SGI237996, SGI237997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI237998, SGI0, SGI237999, SGI237999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52716[] = {SGI238001, SGI0, SGI238002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238003, SGI238004, SGI238004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52717[] = {SGI238005, SGI0, SGI238006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238007, SGI238008, SGI238008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238009, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52718[] = {SGI238010, SGI0, SGI238011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238012, SGI238013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52719[] = {SGI238014, SGI238015, SGI238016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238017, SGI0, SGI0, SGI0, SGI238018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238020, SGI0, NULL};
const Segment * const SG52720[] = {SGI238021, SGI238022, SGI238023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238024, SGI0, SGI0, SGI0, SGI238025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52721[] = {SGI238027, SGI238028, SGI238029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238030, SGI0, SGI0, SGI0, SGI238031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52722[] = {SGI238032, SGI0, SGI238033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238034, SGI238035, SGI238035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52723[] = {SGI238036, SGI0, SGI238037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238038, SGI0, SGI0, SGI0, SGI238039, SGI238039, SGI0, SGI0, SGI0, SGI238040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52724[] = {SGI238041, SGI0, SGI238042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238043, SGI0, SGI0, SGI0, SGI238044, SGI238044, SGI0, SGI0, SGI0, SGI238045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52725[] = {SGI238046, SGI0, SGI238047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238048, SGI0, SGI0, SGI0, SGI238049, SGI238049, SGI0, SGI0, SGI0, SGI238050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52726[] = {SGI238051, SGI0, SGI238052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238053, SGI238054, SGI238054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52727[] = {SGI238055, SGI0, SGI238056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238057, SGI238058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52728[] = {SGI238059, SGI0, SGI238060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238061, SGI0, SGI0, SGI0, SGI238062, SGI238062, SGI0, SGI0, SGI0, SGI238063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52729[] = {SGI238064, SGI0, SGI238065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238066, SGI238067, SGI238067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52730[] = {SGI238000, SGI238068, SGI238069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238070, SGI238070, SGI0, SGI0, SGI0, SGI0, SGI238071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52731[] = {SGI238072, SGI0, SGI238073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238074, SGI238075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52732[] = {SGI238076, SGI0, SGI238077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238078, SGI238079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52733[] = {SGI238080, SGI0, SGI238081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238082, SGI238083, SGI238083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52734[] = {SGI238084, SGI238085, SGI238086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238087, SGI0, SGI0, SGI0, SGI238088, SGI238088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52735[] = {SGI238089, SGI238090, SGI238091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238092, SGI238092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238093, NULL};
const Segment * const SG52736[] = {SGI238094, SGI0, SGI238095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238096, SGI238097, SGI238097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52737[] = {SGI238098, SGI0, SGI238099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238100, SGI238101, SGI238101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52738[] = {SGI238102, SGI0, SGI238103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238104, SGI238105, SGI238105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52739[] = {SGI238106, SGI0, SGI238107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238108, SGI238109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52740[] = {SGI238110, SGI0, SGI238111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238112, SGI238113, SGI238113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52741[] = {SGI238114, SGI238115, SGI238116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238117, SGI0, SGI0, SGI0, SGI238118, SGI238118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238119, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52742[] = {SGI238120, SGI0, SGI238121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238122, SGI238123, SGI238123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52743[] = {SGI238124, SGI0, SGI238125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238126, SGI238127, SGI238127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52744[] = {SGI238128, SGI0, SGI238129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238130, SGI238131, SGI238131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52745[] = {SGI0, SGI238132, SGI0, SGI238133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52746[] = {SGI238134, SGI238135, SGI238136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238137, SGI0, SGI0, SGI0, SGI238138, SGI238138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52747[] = {SGI238139, SGI238140, SGI238141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238145, SGI238145, SGI0, SGI0, SGI238146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52748[] = {SGI238142, SGI238143, SGI238144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238147, SGI0, SGI0, SGI0, SGI238148, SGI238148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238150, SGI0, NULL};
const Segment * const SG52749[] = {SGI238151, SGI238152, SGI238153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238154, SGI0, SGI0, SGI0, SGI238155, SGI238155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238157, SGI0, NULL};
const Segment * const SG52750[] = {SGI238158, SGI0, SGI238159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238160, SGI238161, SGI238161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52751[] = {SGI238162, SGI0, SGI238163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238164, SGI238165, SGI238165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52752[] = {SGI238166, SGI0, SGI238167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238168, SGI238169, SGI238169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52753[] = {SGI238170, SGI238171, SGI238172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238173, SGI238173, SGI0, SGI0, SGI0, SGI0, SGI238174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52754[] = {SGI238175, SGI238176, SGI238177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238178, SGI0, SGI0, SGI0, SGI238179, SGI238179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238180, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52755[] = {SGI238181, SGI238182, SGI238183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238184, SGI0, SGI0, SGI0, SGI238185, SGI238185, SGI0, SGI238188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52756[] = {SGI238186, SGI238187, SGI238189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238190, SGI238190, SGI0, SGI0, SGI0, SGI0, SGI238191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52757[] = {SGI238192, SGI0, SGI238193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238194, SGI238195, SGI238195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52758[] = {SGI238196, SGI238197, SGI238198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238199, SGI238199, SGI0, SGI0, SGI0, SGI0, SGI238200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52759[] = {SGI238202, SGI238204, SGI238206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238207, SGI0, SGI0, SGI0, SGI238209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52760[] = {SGI238201, SGI238203, SGI238205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238208, SGI238208, SGI0, SGI0, SGI0, SGI0, SGI238210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52761[] = {SGI238211, SGI0, SGI238212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238213, SGI238214, SGI238214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52762[] = {SGI238215, SGI238216, SGI238217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238218, SGI238218, SGI0, SGI0, SGI238219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52763[] = {SGI238220, SGI238221, SGI238222, SGI0, SGI0, SGI0, SGI238223, SGI238224, SGI0, SGI238225, SGI0, SGI0, SGI0, SGI238226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52764[] = {SGI238227, SGI0, SGI238228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238231, SGI238233, SGI238233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52765[] = {SGI238229, SGI238230, SGI238232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238234, SGI0, SGI0, SGI0, SGI238235, SGI238235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238236, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52766[] = {SGI238237, SGI238238, SGI238239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238240, SGI0, SGI0, SGI0, SGI238241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238242, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52767[] = {SGI238243, SGI238244, SGI238245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238246, SGI0, SGI0, SGI0, SGI238248, SGI238248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52768[] = {SGI238247, SGI238249, SGI238250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238251, SGI0, SGI0, SGI0, SGI238252, SGI238252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238253, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52769[] = {SGI238254, SGI0, SGI238255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238256, SGI0, SGI0, SGI0, SGI238257, SGI238257, SGI0, SGI0, SGI0, SGI238258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52770[] = {SGI238259, SGI238260, SGI238261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238262, SGI0, SGI0, SGI0, SGI238264, SGI238264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52771[] = {SGI238263, SGI238265, SGI238266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238267, SGI238267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238268, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52772[] = {SGI238269, SGI0, SGI238270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238271, SGI238272, SGI238272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52773[] = {SGI238278, SGI0, SGI238279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238280, SGI238281, SGI238281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238282, SGI0, SGI0, NULL};
const Segment * const SG52774[] = {SGI238283, SGI238284, SGI238285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238289, SGI238289, SGI0, SGI0, SGI0, SGI0, SGI238290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52775[] = {SGI238286, SGI238287, SGI238288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238291, SGI0, SGI0, SGI0, SGI238292, SGI0, SGI238293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52776[] = {SGI238294, SGI238295, SGI238296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238297, SGI238297, SGI0, SGI0, SGI0, SGI0, SGI238298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52777[] = {SGI238299, SGI0, SGI238300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238301, SGI238302, SGI238302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52778[] = {SGI238303, SGI238304, SGI238305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238306, SGI238306, SGI0, SGI0, SGI0, SGI238307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52779[] = {SGI238273, SGI0, SGI238274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238275, SGI0, SGI0, SGI0, SGI238276, SGI238276, SGI0, SGI0, SGI238277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52780[] = {SGI238308, SGI0, SGI238309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238310, SGI238311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52781[] = {SGI238312, SGI238313, SGI238314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238315, SGI0, SGI0, SGI0, SGI238316, SGI238316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52782[] = {SGI238317, SGI238318, SGI238319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238320, SGI238320, SGI0, SGI0, SGI238321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52783[] = {SGI238322, SGI238323, SGI238324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238325, SGI0, SGI0, SGI0, SGI238326, SGI238326, SGI238327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52784[] = {SGI238328, SGI0, SGI238329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238330, SGI238331, SGI238331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52785[] = {SGI238332, SGI0, SGI238333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238334, SGI238335, SGI238335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52786[] = {SGI238336, SGI238337, SGI238338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238339, SGI0, SGI0, SGI0, SGI238340, SGI238340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52787[] = {SGI238341, SGI238342, SGI238343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238344, SGI0, SGI0, SGI0, SGI238345, SGI238345, SGI0, SGI238346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52788[] = {SGI238347, SGI238348, SGI238349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238353, SGI238353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238355, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52789[] = {SGI238350, SGI238351, SGI238352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238354, SGI0, SGI0, SGI0, SGI238356, SGI238356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238357, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52790[] = {SGI238358, SGI238359, SGI238360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238361, SGI0, SGI0, SGI0, SGI238362, SGI238362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238363, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52791[] = {SGI238364, SGI238365, SGI238366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238369, SGI0, SGI0, SGI0, SGI238370, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52792[] = {SGI238367, SGI0, SGI238368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52793[] = {SGI238371, SGI0, SGI238372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52794[] = {SGI238373, SGI0, SGI238374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238375, SGI238376, SGI238376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238377, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52795[] = {SGI238378, SGI238379, SGI238380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238381, SGI0, SGI0, SGI0, SGI238382, SGI238382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52796[] = {SGI238383, SGI0, SGI238384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238387, SGI238388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52797[] = {SGI238385, SGI0, SGI238386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238389, SGI238390, SGI238390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52798[] = {SGI238391, SGI238392, SGI238393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238394, SGI0, SGI0, SGI0, SGI238395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52799[] = {SGI238396, SGI238397, SGI238398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238399, SGI238399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238400, NULL};
const Segment * const SG52800[] = {SGI238401, SGI0, SGI238402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238403, SGI238404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52801[] = {SGI238405, SGI238406, SGI238407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238408, SGI0, SGI0, SGI0, SGI238409, SGI0, SGI0, SGI238410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52802[] = {SGI238411, SGI0, SGI238412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238413, SGI238414, SGI238414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52803[] = {SGI238415, SGI0, SGI238416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238417, SGI238418, SGI238418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52804[] = {SGI238419, SGI238420, SGI238421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238422, SGI0, SGI0, SGI0, SGI238423, SGI0, SGI0, SGI238424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52805[] = {SGI238425, SGI238426, SGI238427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238428, SGI0, SGI0, SGI0, SGI238429, SGI238429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238430, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52806[] = {SGI238431, SGI238432, SGI238433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238434, SGI0, SGI0, SGI0, SGI238435, SGI238435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238436, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52807[] = {SGI238437, SGI0, SGI238438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238439, SGI238440, SGI238440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238441, SGI0, SGI0, NULL};
const Segment * const SG52808[] = {SGI238442, SGI0, SGI238443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238444, SGI238445, SGI238445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52809[] = {SGI238446, SGI238447, SGI238448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238449, SGI0, SGI0, SGI0, SGI238450, SGI238450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238451, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52810[] = {SGI238452, SGI238453, SGI238454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238455, SGI0, SGI0, SGI0, SGI238456, SGI238456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238457, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52811[] = {SGI238458, SGI238459, SGI238460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238461, SGI238461, SGI0, SGI0, SGI238462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52812[] = {SGI238463, SGI238464, SGI238465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238466, SGI0, SGI0, SGI0, SGI238467, SGI238467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238468, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52813[] = {SGI238469, SGI238470, SGI238471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238472, SGI0, SGI0, SGI0, SGI238473, SGI238473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238474, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52814[] = {SGI238475, SGI238476, SGI238477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238478, SGI238478, SGI0, SGI0, SGI0, SGI0, SGI238479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52815[] = {SGI238480, SGI238481, SGI238482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238483, SGI0, SGI0, SGI0, SGI238484, SGI238484, SGI0, SGI238485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52816[] = {SGI238486, SGI238487, SGI238488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238489, SGI238489, SGI0, SGI0, SGI0, SGI0, SGI238490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52817[] = {SGI238491, SGI238492, SGI238493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238494, SGI0, SGI0, SGI0, SGI238495, SGI238495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52818[] = {SGI238496, SGI0, SGI238497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238498, SGI238499, SGI238499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238500, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52819[] = {SGI238501, SGI0, SGI238502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238503, SGI238504, SGI238504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52820[] = {SGI238505, SGI238506, SGI238507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238509, SGI0, SGI0, SGI0, SGI238511, SGI238511, SGI238512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52821[] = {SGI238508, SGI0, SGI238510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238513, SGI238514, SGI238514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52822[] = {SGI238515, SGI0, SGI238516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238517, SGI238518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52823[] = {SGI238519, SGI0, SGI238520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238521, SGI238522, SGI238522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52824[] = {SGI238523, SGI0, SGI238524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238525, SGI238526, SGI238526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238527, SGI0, SGI0, NULL};
const Segment * const SG52825[] = {SGI238528, SGI238529, SGI238530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238531, SGI0, SGI0, SGI0, SGI238532, SGI238532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238533, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52826[] = {SGI238534, SGI238535, SGI238536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238537, SGI0, SGI0, SGI0, SGI238538, SGI238538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52827[] = {SGI238539, SGI238540, SGI238541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238542, SGI0, SGI0, SGI0, SGI238543, SGI238543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52828[] = {SGI238544, SGI0, SGI238545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238546, SGI238547, SGI238547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52829[] = {SGI238548, SGI0, SGI238549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238550, SGI238551, SGI238551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52830[] = {SGI238552, SGI0, SGI238553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238554, SGI238555, SGI238555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52831[] = {SGI238556, SGI238557, SGI238558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238559, SGI0, SGI0, SGI0, SGI238560, SGI238560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52832[] = {SGI238561, SGI0, SGI238562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238563, SGI238564, SGI238564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52833[] = {SGI238565, SGI0, SGI238566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52834[] = {SGI238567, SGI0, SGI238568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52835[] = {SGI238569, SGI0, SGI238570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238571, SGI238572, SGI238572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52836[] = {SGI238573, SGI0, SGI238574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238575, SGI238576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52837[] = {SGI0, SGI0, SGI238578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238581, SGI238582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52838[] = {SGI238577, SGI238579, SGI238580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238583, SGI0, SGI0, SGI0, SGI238584, SGI0, SGI0, SGI0, SGI238585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52839[] = {SGI238586, SGI238587, SGI238588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238589, SGI238589, SGI0, SGI0, SGI0, SGI238590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52840[] = {SGI238591, SGI238592, SGI238593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238594, SGI238594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238595, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52841[] = {SGI238596, SGI0, SGI238597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238598, SGI238599, SGI238599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52842[] = {SGI238600, SGI0, SGI238601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238602, SGI238603, SGI238603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238604, SGI0, SGI0, NULL};
const Segment * const SG52843[] = {SGI238605, SGI0, SGI238606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238607, SGI238608, SGI238608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52844[] = {SGI238609, SGI0, SGI238610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238611, SGI238612, SGI238612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52845[] = {SGI238613, SGI238614, SGI238615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238616, SGI238616, SGI0, SGI0, SGI0, SGI0, SGI238617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52846[] = {SGI238618, SGI238619, SGI238620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238621, SGI0, SGI0, SGI0, SGI238622, SGI238622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238623, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52847[] = {SGI238624, SGI238625, SGI238626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238627, SGI0, SGI0, SGI0, SGI238628, SGI238628, SGI0, SGI0, SGI238629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52848[] = {SGI238630, SGI238631, SGI238632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238634, SGI0, SGI0, SGI0, SGI238636, SGI238636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52849[] = {SGI238633, SGI0, SGI238635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238637, SGI238638, SGI238638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52850[] = {SGI238639, SGI238640, SGI238641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238643, SGI0, SGI0, SGI0, SGI238645, SGI238645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52851[] = {SGI238642, SGI0, SGI238644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238646, SGI238647, SGI238647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52852[] = {SGI238648, SGI0, SGI238649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238650, SGI238651, SGI238651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52853[] = {SGI238652, SGI238653, SGI238654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238655, SGI0, SGI0, SGI0, SGI238656, SGI238656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52854[] = {SGI238657, SGI0, SGI238658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238659, SGI238660, SGI238660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52855[] = {SGI238661, SGI0, SGI238662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238663, SGI238664, SGI238664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52856[] = {SGI238665, SGI238666, SGI238667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238668, SGI0, SGI0, SGI0, SGI238669, SGI238669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52857[] = {SGI238670, SGI0, SGI238671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238672, SGI238673, SGI238673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52858[] = {SGI238674, SGI0, SGI238675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238676, SGI238677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52859[] = {SGI238678, SGI0, SGI238680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238682, SGI238684, SGI238684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52860[] = {SGI238679, SGI0, SGI238681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238683, SGI238685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52861[] = {SGI238686, SGI238687, SGI238688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238689, SGI0, SGI0, SGI0, SGI238690, SGI238690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238691, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52862[] = {SGI238692, SGI0, SGI238693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238694, SGI238695, SGI238695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52863[] = {SGI238696, SGI238697, SGI238698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238699, SGI0, SGI238700, SGI238700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52864[] = {SGI238701, SGI238702, SGI238703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238704, SGI0, SGI238705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52865[] = {SGI238706, SGI238707, SGI238709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238713, SGI238713, SGI0, SGI0, SGI0, SGI0, SGI238715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52866[] = {SGI238708, SGI238710, SGI238711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238712, SGI0, SGI238714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52867[] = {SGI238716, SGI0, SGI238717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238718, SGI238719, SGI238719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52868[] = {SGI238720, SGI0, SGI238721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238722, SGI238723, SGI238723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52869[] = {SGI238724, SGI238725, SGI238726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238727, SGI0, SGI238728, SGI238728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52870[] = {SGI238729, SGI0, SGI238730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238731, SGI238732, SGI238732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52871[] = {SGI238733, SGI0, SGI238734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238735, SGI0, SGI0, SGI0, SGI238736, SGI238736, SGI0, SGI0, SGI238737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52872[] = {SGI238738, SGI0, SGI238739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238742, SGI238743, SGI238743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52873[] = {SGI238740, SGI0, SGI238741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238744, SGI238745, SGI238745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52874[] = {SGI238746, SGI238747, SGI238748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238749, SGI0, SGI0, SGI0, SGI238750, SGI238750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52875[] = {SGI238751, SGI0, SGI238752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238753, SGI238754, SGI238754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52876[] = {SGI238755, SGI0, SGI238756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238757, SGI238758, SGI238758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52877[] = {SGI238759, SGI0, SGI238760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238761, SGI0, SGI0, SGI0, SGI238762, SGI238762, SGI0, SGI0, SGI238763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52878[] = {SGI238764, SGI0, SGI238765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238766, SGI238767, SGI238767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52879[] = {SGI238768, SGI0, SGI238769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238770, SGI0, SGI0, SGI0, SGI238772, SGI238772, SGI0, SGI0, SGI0, SGI238774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52880[] = {SGI238771, SGI0, SGI238773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238775, SGI238776, SGI238776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52881[] = {SGI238777, SGI0, SGI238778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238779, SGI0, SGI0, SGI0, SGI238780, SGI238780, SGI0, SGI0, SGI0, SGI238781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52882[] = {SGI238782, SGI238783, SGI238784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238785, SGI0, SGI0, SGI0, SGI238786, SGI0, SGI238787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52883[] = {SGI238788, SGI238789, SGI238790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238791, SGI238791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238792, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52884[] = {SGI238793, SGI238794, SGI238795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238796, SGI0, SGI0, SGI0, SGI238797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238798, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52885[] = {SGI238799, SGI0, SGI238800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238803, SGI238804, SGI238804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52886[] = {SGI238801, SGI0, SGI238802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238805, SGI0, SGI0, SGI0, SGI238807, SGI238807, SGI0, SGI0, SGI0, SGI238809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52887[] = {SGI238806, SGI0, SGI238808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238810, SGI0, SGI0, SGI0, SGI238811, SGI238811, SGI0, SGI0, SGI238812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52888[] = {SGI238813, SGI238814, SGI238815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238816, SGI0, SGI0, SGI0, SGI238818, SGI238818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52889[] = {SGI238817, SGI238819, SGI238820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238821, SGI0, SGI0, SGI0, SGI238822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238823, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52890[] = {SGI238824, SGI238825, SGI238826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238827, SGI0, SGI0, SGI0, SGI238828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238829, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52891[] = {SGI238830, SGI238831, SGI238832, SGI0, SGI0, SGI0, SGI238833, SGI0, SGI0, SGI238834, SGI0, SGI0, SGI0, SGI238835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52892[] = {SGI238836, SGI238837, SGI238838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238839, SGI0, SGI0, SGI0, SGI238840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52893[] = {SGI0, SGI238845, SGI0, SGI238846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52894[] = {SGI238842, SGI238843, SGI238844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238847, SGI0, SGI0, SGI0, SGI238848, SGI0, SGI238849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52895[] = {SGI238850, SGI238851, SGI238852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238853, SGI0, SGI0, SGI0, SGI238855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52896[] = {SGI238854, SGI0, SGI238856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238858, SGI238859, SGI238859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52897[] = {SGI238860, SGI238861, SGI238862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238863, SGI238863, SGI0, SGI0, SGI0, SGI238864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52898[] = {SGI238865, SGI238866, SGI238867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238868, SGI0, SGI0, SGI0, SGI238869, SGI238869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238870, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52899[] = {SGI238871, SGI0, SGI238872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238873, SGI238874, SGI238874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52900[] = {SGI238875, SGI0, SGI238876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238877, SGI0, SGI0, SGI0, SGI238878, SGI238878, SGI0, SGI0, SGI0, SGI238879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52901[] = {SGI238880, SGI0, SGI238881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238882, SGI238883, SGI238883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52902[] = {SGI238884, SGI0, SGI238885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238886, SGI238887, SGI238887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52903[] = {SGI238888, SGI0, SGI238889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238893, SGI238894, SGI238894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52904[] = {SGI238890, SGI238891, SGI238892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238895, SGI0, SGI0, SGI0, SGI238896, SGI238896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238897, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52905[] = {SGI238898, SGI0, SGI238899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238900, SGI238901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52906[] = {SGI238902, SGI0, SGI238903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238904, SGI238905, SGI238905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52907[] = {SGI238906, SGI0, SGI238907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238908, SGI238909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238910, SGI0, SGI0, NULL};
const Segment * const SG52908[] = {SGI238911, SGI0, SGI238912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238913, SGI238914, SGI238914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52909[] = {SGI238915, SGI238916, SGI238917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238918, SGI0, SGI0, SGI0, SGI238919, SGI238919, SGI0, SGI238920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52910[] = {SGI238921, SGI0, SGI238922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238923, SGI238924, SGI238924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52911[] = {SGI238925, SGI0, SGI238926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238927, SGI238928, SGI238928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52912[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238929, SGI0, SGI0, SGI238930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52913[] = {SGI238931, SGI0, SGI238932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238933, SGI238934, SGI238934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52914[] = {SGI238935, SGI0, SGI238936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238937, SGI238938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52915[] = {SGI238939, SGI0, SGI238940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238941, SGI238942, SGI238942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238943, SGI0, SGI0, NULL};
const Segment * const SG52916[] = {SGI238944, SGI0, SGI238945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238946, SGI238947, SGI238947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52917[] = {SGI238948, SGI0, SGI238949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238950, SGI238951, SGI238951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52918[] = {SGI238952, SGI238953, SGI238954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238955, SGI0, SGI0, SGI0, SGI238956, SGI238956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52919[] = {SGI238957, SGI238958, SGI238959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238960, SGI0, SGI0, SGI0, SGI238961, SGI238961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52920[] = {SGI238962, SGI0, SGI238963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238964, SGI238965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52921[] = {SGI238966, SGI0, SGI238967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238968, SGI238969, SGI238969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52922[] = {SGI238970, SGI0, SGI238972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238974, SGI238975, SGI238975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52923[] = {SGI238971, SGI0, SGI238973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238976, SGI238977, SGI238977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52924[] = {SGI238978, SGI238979, SGI238980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238981, SGI0, SGI0, SGI0, SGI238982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52925[] = {SGI238983, SGI0, SGI238984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238985, SGI238986, SGI238986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52926[] = {SGI238987, SGI238988, SGI238989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238991, SGI0, SGI0, SGI0, SGI238993, SGI238993, SGI0, SGI238995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52927[] = {SGI238990, SGI238992, SGI238994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238996, SGI0, SGI0, SGI0, SGI238997, SGI238997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI238998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52928[] = {SGI238999, SGI239000, SGI239001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239002, SGI0, SGI0, SGI0, SGI239003, SGI239003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52929[] = {SGI239005, SGI0, SGI239006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239007, SGI239008, SGI239008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52930[] = {SGI239009, SGI0, SGI239010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239011, SGI239012, SGI239012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52931[] = {SGI239013, SGI239014, SGI239015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239016, SGI0, SGI0, SGI0, SGI239017, SGI239017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52932[] = {SGI239018, SGI239019, SGI239020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239024, SGI0, SGI0, SGI0, SGI239025, SGI239025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52933[] = {SGI239021, SGI239022, SGI239023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239026, SGI239026, SGI0, SGI0, SGI239027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52934[] = {SGI239028, SGI0, SGI239029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239030, SGI239031, SGI239031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52935[] = {SGI239032, SGI239033, SGI239034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239035, SGI0, SGI239036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52936[] = {SGI239037, SGI239038, SGI239039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239040, SGI0, SGI0, SGI0, SGI239041, SGI239041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239043, SGI0, NULL};
const Segment * const SG52937[] = {SGI239044, SGI0, SGI239045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239046, SGI239047, SGI239047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52938[] = {SGI239049, SGI0, SGI239051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239053, SGI239054, SGI239054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52939[] = {SGI239048, SGI239050, SGI239052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239055, SGI239055, SGI0, SGI0, SGI239056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52940[] = {SGI239057, SGI0, SGI239058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239059, SGI239060, SGI239060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52941[] = {SGI239061, SGI239062, SGI239063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239064, SGI0, SGI0, SGI0, SGI239065, SGI239065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52942[] = {SGI239066, SGI239067, SGI239068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239069, SGI0, SGI0, SGI0, SGI239070, SGI239070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52943[] = {SGI239071, SGI0, SGI239072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239073, SGI239074, SGI239074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52944[] = {SGI239075, SGI0, SGI239076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239077, SGI239078, SGI239078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52945[] = {SGI239079, SGI0, SGI239080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239081, SGI239082, SGI239082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52946[] = {SGI239083, SGI239084, SGI239085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239086, SGI0, SGI0, SGI0, SGI239087, SGI239087, SGI0, SGI0, SGI239088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52947[] = {SGI239089, SGI0, SGI239090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239091, SGI239092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52948[] = {SGI239093, SGI239094, SGI239095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239096, SGI239096, SGI0, SGI0, SGI239097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52949[] = {SGI239098, SGI0, SGI239099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239100, SGI239101, SGI239101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52950[] = {SGI239102, SGI0, SGI239103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239104, SGI239105, SGI239105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52951[] = {SGI239106, SGI0, SGI239107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239108, SGI239109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52952[] = {SGI239110, SGI239111, SGI239112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239113, SGI0, SGI0, SGI0, SGI239114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52953[] = {SGI239116, SGI0, SGI239117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239118, SGI239119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52954[] = {SGI239120, SGI239121, SGI239122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239123, SGI0, SGI0, SGI0, SGI239124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52955[] = {SGI239125, SGI0, SGI239126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239127, SGI0, SGI0, SGI0, SGI239128, SGI239128, SGI0, SGI0, SGI0, SGI239129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52956[] = {SGI239130, SGI239131, SGI239132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239133, SGI0, SGI0, SGI0, SGI239134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239135, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52957[] = {SGI239136, SGI0, SGI239137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239138, SGI0, SGI0, SGI0, SGI239139, SGI239139, SGI0, SGI0, SGI0, SGI239140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52958[] = {SGI239141, SGI239142, SGI239143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239144, SGI239144, SGI0, SGI0, SGI239145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52959[] = {SGI239146, SGI239147, SGI239148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239149, SGI0, SGI0, SGI0, SGI239150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239151, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52960[] = {SGI239152, SGI239153, SGI239154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239155, SGI0, SGI0, SGI0, SGI239156, SGI239156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239157, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52961[] = {SGI239158, SGI239159, SGI239160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239161, SGI0, SGI0, SGI0, SGI239162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239163, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52962[] = {SGI239164, SGI239165, SGI239166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239167, SGI0, SGI0, SGI0, SGI239169, SGI0, SGI239171, SGI0, SGI0, SGI0, SGI0, SGI239172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52963[] = {SGI239168, SGI0, SGI239170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239173, SGI239174, SGI239174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239175, SGI0, SGI0, NULL};
const Segment * const SG52964[] = {SGI239176, SGI239177, SGI239178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239179, SGI0, SGI0, SGI0, SGI239180, SGI239180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239181, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52965[] = {SGI239182, SGI239183, SGI239184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239185, SGI0, SGI0, SGI0, SGI239186, SGI0, SGI0, SGI0, SGI0, SGI239187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52966[] = {SGI239188, SGI0, SGI239189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239190, SGI0, SGI0, SGI0, SGI239191, SGI239191, SGI0, SGI0, SGI0, SGI239192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52967[] = {SGI239193, SGI239194, SGI239195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239196, SGI0, SGI0, SGI0, SGI239197, SGI239198, SGI0, SGI0, SGI0, SGI239200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52968[] = {SGI239199, SGI239201, SGI239202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239203, SGI239203, SGI0, SGI0, SGI239204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52969[] = {SGI239205, SGI239206, SGI239207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239208, SGI0, SGI0, SGI0, SGI239209, SGI0, SGI0, SGI0, SGI239210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52970[] = {SGI239211, SGI239212, SGI239213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239214, SGI0, SGI0, SGI0, SGI239217, SGI239217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52971[] = {SGI239215, SGI239216, SGI239218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239219, SGI239219, SGI0, SGI0, SGI0, SGI239220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52972[] = {SGI239221, SGI239222, SGI239223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239224, SGI239224, SGI0, SGI0, SGI0, SGI239225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52973[] = {SGI239226, SGI239227, SGI239228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239231, SGI239231, SGI0, SGI0, SGI0, SGI239233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52974[] = {SGI239229, SGI0, SGI239230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239232, SGI239234, SGI239234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52975[] = {SGI239235, SGI0, SGI239236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239237, SGI239238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239239, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52976[] = {SGI239240, SGI239241, SGI239242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239243, SGI239243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239244, NULL};
const Segment * const SG52977[] = {SGI239245, SGI0, SGI239246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239247, SGI239248, SGI239248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239249, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52978[] = {SGI239250, SGI239251, SGI239252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239253, SGI239253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239254, NULL};
const Segment * const SG52979[] = {SGI239255, SGI0, SGI239256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239257, SGI239258, SGI239258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52980[] = {SGI239259, SGI0, SGI239260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239261, SGI239262, SGI239262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52981[] = {SGI239263, SGI239264, SGI239265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239266, SGI0, SGI0, SGI0, SGI239267, SGI0, SGI0, SGI239268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52982[] = {SGI239269, SGI0, SGI239270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239271, SGI239272, SGI239272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52983[] = {SGI239273, SGI0, SGI239274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239275, SGI239276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52984[] = {SGI239277, SGI239278, SGI239279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239280, SGI0, SGI0, SGI0, SGI239281, SGI239281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52985[] = {SGI239282, SGI239283, SGI239284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239285, SGI0, SGI0, SGI0, SGI239286, SGI239286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239287, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52986[] = {SGI239288, SGI239289, SGI239290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239291, SGI0, SGI0, SGI0, SGI239292, SGI239293, SGI0, SGI0, SGI0, SGI0, SGI239294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52987[] = {SGI239295, SGI0, SGI239296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239297, SGI239298, SGI239298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239299, SGI0, SGI0, NULL};
const Segment * const SG52988[] = {SGI239300, SGI239301, SGI239302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239303, SGI0, SGI0, SGI0, SGI239304, SGI239304, SGI0, SGI239305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52989[] = {SGI239306, SGI239307, SGI239308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239309, SGI0, SGI0, SGI0, SGI239310, SGI239310, SGI0, SGI0, SGI0, SGI239311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52990[] = {SGI239312, SGI239313, SGI239314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239315, SGI0, SGI0, SGI0, SGI239316, SGI239316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239317, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52991[] = {SGI239318, SGI239319, SGI239320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239323, SGI0, SGI0, SGI0, SGI239324, SGI239324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52992[] = {SGI239321, SGI0, SGI239322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239325, SGI239326, SGI239326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52993[] = {SGI239327, SGI239328, SGI239329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239330, SGI0, SGI0, SGI0, SGI239332, SGI239332, SGI0, SGI0, SGI0, SGI239335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52994[] = {SGI239331, SGI239333, SGI239334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239336, SGI0, SGI0, SGI0, SGI239337, SGI239337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52995[] = {SGI239338, SGI239339, SGI239340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239341, SGI0, SGI0, SGI0, SGI239342, SGI239342, SGI0, SGI0, SGI0, SGI0, SGI239343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52996[] = {SGI239344, SGI239345, SGI239346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239347, SGI0, SGI0, SGI0, SGI239350, SGI239350, SGI0, SGI0, SGI0, SGI239351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52997[] = {SGI239348, SGI0, SGI239349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239352, SGI239353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52998[] = {SGI239354, SGI0, SGI239355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239356, SGI239357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG52999[] = {SGI239358, SGI0, SGI239359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239360, SGI239361, SGI239361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53000[] = {SGI239362, SGI239363, SGI239364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239365, SGI0, SGI0, SGI0, SGI239366, SGI239366, SGI0, SGI239367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53001[] = {SGI239368, SGI239369, SGI239370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239371, SGI239371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239372, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53002[] = {SGI239373, SGI0, SGI239374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239375, SGI239376, SGI239376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53003[] = {SGI239377, SGI0, SGI239378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239379, SGI239380, SGI239380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53004[] = {SGI239383, SGI0, SGI239385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239387, SGI239389, SGI239389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53005[] = {SGI239381, SGI239382, SGI239384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239386, SGI0, SGI0, SGI0, SGI239388, SGI239390, SGI0, SGI0, SGI0, SGI0, SGI239391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53006[] = {SGI239392, SGI0, SGI239393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239394, SGI239395, SGI239395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53007[] = {SGI239396, SGI0, SGI239397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239398, SGI239399, SGI239399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239400, SGI0, SGI0, NULL};
const Segment * const SG53008[] = {SGI239401, SGI239402, SGI239403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239404, SGI0, SGI0, SGI0, SGI239405, SGI239405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239407, SGI0, NULL};
const Segment * const SG53009[] = {SGI239408, SGI239409, SGI239410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239411, SGI0, SGI0, SGI0, SGI239412, SGI239412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53010[] = {SGI239414, SGI239415, SGI239416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239417, SGI0, SGI0, SGI0, SGI239418, SGI239418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53011[] = {SGI239420, SGI239421, SGI239422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239423, SGI0, SGI0, SGI0, SGI239424, SGI239424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53012[] = {SGI239425, SGI239426, SGI239427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239428, SGI0, SGI0, SGI0, SGI239429, SGI239429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239431, SGI0, NULL};
const Segment * const SG53013[] = {SGI239433, SGI0, SGI239435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239437, SGI239439, SGI239439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53014[] = {SGI239432, SGI0, SGI239434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239436, SGI239438, SGI239438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53015[] = {SGI239440, SGI0, SGI239441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239442, SGI239443, SGI239443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53016[] = {SGI239444, SGI0, SGI239445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239446, SGI0, SGI0, SGI0, SGI239447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239448, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53017[] = {SGI239449, SGI0, SGI239450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239451, SGI239452, SGI239452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53018[] = {SGI239453, SGI239455, SGI239456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239459, SGI0, SGI0, SGI0, SGI239461, SGI239461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53019[] = {SGI239454, SGI239457, SGI239458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239460, SGI0, SGI0, SGI0, SGI239462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239463, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53020[] = {SGI239464, SGI0, SGI239465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239466, SGI239467, SGI239467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53021[] = {SGI239468, SGI239469, SGI239470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239471, SGI239471, SGI0, SGI0, SGI0, SGI0, SGI239472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53022[] = {SGI239473, SGI0, SGI239474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239475, SGI239476, SGI239476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53023[] = {SGI239477, SGI0, SGI239478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239479, SGI239480, SGI239480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53024[] = {SGI239481, SGI0, SGI239482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239483, SGI239484, SGI239484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53025[] = {SGI239485, SGI239486, SGI239487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239488, SGI0, SGI0, SGI0, SGI239489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239490, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53026[] = {SGI239491, SGI0, SGI239492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239493, SGI239494, SGI239494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53027[] = {SGI239495, SGI239496, SGI239497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239498, SGI0, SGI0, SGI0, SGI239499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239500, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53028[] = {SGI239501, SGI239502, SGI239503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239504, SGI0, SGI0, SGI0, SGI239505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53029[] = {SGI239506, SGI0, SGI239507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239508, SGI0, SGI0, SGI0, SGI239509, SGI239509, SGI0, SGI0, SGI0, SGI239510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53030[] = {SGI239511, SGI0, SGI239512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239513, SGI239514, SGI239514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53031[] = {SGI239515, SGI239516, SGI239517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239518, SGI0, SGI0, SGI0, SGI239519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239520, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53032[] = {SGI239521, SGI0, SGI239522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239523, SGI0, SGI0, SGI0, SGI239524, SGI239524, SGI0, SGI0, SGI239526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53033[] = {SGI239525, SGI239527, SGI239528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239530, SGI0, SGI0, SGI0, SGI239532, SGI0, SGI239533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53034[] = {SGI239529, SGI0, SGI239531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239534, SGI239535, SGI239535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239536, SGI0, SGI0, NULL};
const Segment * const SG53035[] = {SGI239537, SGI0, SGI239538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239539, SGI239540, SGI239540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53036[] = {SGI239541, SGI0, SGI239542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239543, SGI239544, SGI239544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239545, SGI0, SGI0, NULL};
const Segment * const SG53037[] = {SGI239546, SGI239547, SGI239548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239549, SGI0, SGI0, SGI0, SGI239550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239551, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53038[] = {SGI239552, SGI0, SGI239553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239554, SGI239555, SGI239555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53039[] = {SGI239556, SGI239557, SGI239558, SGI239559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239561, SGI0, SGI0, SGI0, SGI239563, SGI239564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53040[] = {SGI239560, SGI0, SGI239562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239565, SGI239566, SGI239566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53041[] = {SGI239567, SGI239568, SGI239569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239570, SGI239570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239573, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53042[] = {SGI239571, SGI239572, SGI239574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239575, SGI0, SGI0, SGI0, SGI239576, SGI239576, SGI239577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53043[] = {SGI239578, SGI239579, SGI239580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239581, SGI0, SGI0, SGI0, SGI239582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239583, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53044[] = {SGI239584, SGI239585, SGI239586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239587, SGI239587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239588, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53045[] = {SGI239589, SGI239590, SGI239591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239592, SGI0, SGI0, SGI0, SGI239594, SGI239594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239596, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53046[] = {SGI239593, SGI0, SGI239595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53047[] = {SGI239597, SGI239598, SGI239599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239600, SGI0, SGI0, SGI0, SGI239601, SGI239601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239602, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53048[] = {SGI239603, SGI0, SGI239604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239605, SGI0, SGI0, SGI0, SGI239606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239607, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53049[] = {SGI239608, SGI239609, SGI239610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239614, SGI0, SGI0, SGI0, SGI239615, SGI239615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53050[] = {SGI239611, SGI239612, SGI239613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239616, SGI0, SGI0, SGI0, SGI239617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239618, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53051[] = {SGI239619, SGI0, SGI239620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239621, SGI239622, SGI239622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53052[] = {SGI239623, SGI239624, SGI239625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239626, SGI239626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239627, NULL};
const Segment * const SG53053[] = {SGI239628, SGI239629, SGI239630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239631, SGI0, SGI0, SGI0, SGI239632, SGI239632, SGI0, SGI0, SGI0, SGI0, SGI239633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53054[] = {SGI239634, SGI239635, SGI239636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239637, SGI239637, SGI0, SGI0, SGI0, SGI0, SGI239638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53055[] = {SGI239639, SGI0, SGI239640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239641, SGI0, SGI0, SGI0, SGI239642, SGI239642, SGI0, SGI0, SGI0, SGI239643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53056[] = {SGI239644, SGI239645, SGI239646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239647, SGI0, SGI0, SGI0, SGI239648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53057[] = {SGI239650, SGI239651, SGI239652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239653, SGI239653, SGI0, SGI0, SGI0, SGI239654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53058[] = {SGI239655, SGI0, SGI239656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239657, SGI239658, SGI239658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239659, SGI0, SGI0, NULL};
const Segment * const SG53059[] = {SGI239660, SGI0, SGI239661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239665, SGI239666, SGI239666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53060[] = {SGI239662, SGI239663, SGI239664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239667, SGI0, SGI0, SGI0, SGI239668, SGI0, SGI0, SGI239669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53061[] = {SGI239670, SGI239671, SGI239672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239673, SGI0, SGI0, SGI0, SGI239674, SGI239675, SGI0, SGI0, SGI0, SGI0, SGI239676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53062[] = {SGI239677, SGI239678, SGI239680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239683, SGI0, SGI0, SGI0, SGI239684, SGI0, SGI0, SGI239686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53063[] = {SGI239679, SGI239681, SGI239682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239685, SGI239685, SGI0, SGI0, SGI0, SGI0, SGI239687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53064[] = {SGI239688, SGI239689, SGI239690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239691, SGI0, SGI0, SGI0, SGI239692, SGI0, SGI0, SGI0, SGI0, SGI239693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53065[] = {SGI239694, SGI239695, SGI239696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239697, SGI0, SGI0, SGI0, SGI239698, SGI239698, SGI0, SGI0, SGI0, SGI239699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53066[] = {SGI239700, SGI0, SGI239701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239702, SGI239703, SGI239703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53067[] = {SGI239704, SGI239705, SGI239706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239707, SGI0, SGI0, SGI0, SGI239708, SGI0, SGI0, SGI0, SGI239709, SGI0, SGI0, SGI239710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53068[] = {SGI239711, SGI239712, SGI239713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239714, SGI0, SGI0, SGI0, SGI239715, SGI0, SGI239716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53069[] = {SGI239717, SGI239718, SGI239719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239720, SGI0, SGI0, SGI0, SGI239721, SGI0, SGI0, SGI239722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53070[] = {SGI239723, SGI239724, SGI239726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239729, SGI0, SGI0, SGI0, SGI239730, SGI239730, SGI0, SGI239732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53071[] = {SGI239725, SGI239727, SGI239728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239731, SGI0, SGI0, SGI0, SGI239733, SGI0, SGI0, SGI0, SGI239734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53072[] = {SGI239735, SGI0, SGI239736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239737, SGI239739, SGI239739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53073[] = {SGI239738, SGI239740, SGI239741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239742, SGI0, SGI0, SGI0, SGI239743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239744, NULL};
const Segment * const SG53074[] = {SGI239745, SGI239746, SGI239747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239748, SGI0, SGI0, SGI0, SGI239749, SGI0, SGI0, SGI0, SGI0, SGI239750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53075[] = {SGI239751, SGI0, SGI239752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239753, SGI239754, SGI239754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53076[] = {SGI239755, SGI239756, SGI239757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239758, SGI0, SGI0, SGI0, SGI239759, SGI239759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239761, SGI0, NULL};
const Segment * const SG53077[] = {SGI239762, SGI0, SGI239763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239764, SGI239765, SGI239765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53078[] = {SGI239766, SGI0, SGI239767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239768, SGI239769, SGI239769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53079[] = {SGI239770, SGI0, SGI239771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239772, SGI239773, SGI239773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239774, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53080[] = {SGI239775, SGI0, SGI239776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239777, SGI239778, SGI239778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239779, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53081[] = {SGI239780, SGI0, SGI239781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239782, SGI0, SGI0, SGI0, SGI239783, SGI239783, SGI0, SGI0, SGI239784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53082[] = {SGI239785, SGI239786, SGI239787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239788, SGI239788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239789, NULL};
const Segment * const SG53083[] = {SGI239790, SGI0, SGI239791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239793, SGI239794, SGI239794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53084[] = {SGI239792, SGI0, SGI239795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239796, SGI239797, SGI239797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53085[] = {SGI239798, SGI0, SGI239799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239800, SGI239801, SGI239801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53086[] = {SGI239802, SGI0, SGI239803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239804, SGI239805, SGI239805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53087[] = {SGI239806, SGI239807, SGI239808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239809, SGI0, SGI0, SGI0, SGI239810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239811, NULL};
const Segment * const SG53088[] = {SGI239812, SGI239813, SGI239814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239818, SGI0, SGI0, SGI0, SGI239819, SGI239820, SGI0, SGI0, SGI0, SGI0, SGI239822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53089[] = {SGI239815, SGI239816, SGI239817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239821, SGI0, SGI0, SGI0, SGI239823, SGI239823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53090[] = {SGI239824, SGI0, SGI239825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239826, SGI239827, SGI239827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53091[] = {SGI239828, SGI0, SGI239829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239830, SGI239831, SGI239831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239832, SGI0, SGI0, NULL};
const Segment * const SG53092[] = {SGI239833, SGI239834, SGI239835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239836, SGI0, SGI0, SGI0, SGI239837, SGI239837, SGI0, SGI239838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53093[] = {SGI239839, SGI0, SGI239840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239841, SGI239842, SGI239842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53094[] = {SGI239843, SGI239844, SGI239845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239846, SGI0, SGI0, SGI0, SGI239847, SGI239847, SGI0, SGI0, SGI0, SGI239848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53095[] = {SGI239849, SGI239850, SGI239851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239852, SGI239852, SGI0, SGI0, SGI0, SGI0, SGI239853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53096[] = {SGI239854, SGI239855, SGI239856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239857, SGI0, SGI0, SGI0, SGI239858, SGI239858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239859, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53097[] = {SGI239860, SGI239861, SGI239862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239864, SGI0, SGI0, SGI0, SGI239866, SGI239866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53098[] = {SGI239863, SGI0, SGI239865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239867, SGI239868, SGI239868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53099[] = {SGI239869, SGI239870, SGI239871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239872, SGI0, SGI0, SGI0, SGI239873, SGI239873, SGI0, SGI0, SGI0, SGI239874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53100[] = {SGI239875, SGI0, SGI239876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239877, SGI239878, SGI239878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53101[] = {SGI239879, SGI239880, SGI239881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239882, SGI239882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239883, NULL};
const Segment * const SG53102[] = {SGI239884, SGI0, SGI239885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239886, SGI239887, SGI239887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239888, SGI0, SGI0, NULL};
const Segment * const SG53103[] = {SGI239889, SGI239890, SGI239891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239892, SGI0, SGI0, SGI0, SGI239893, SGI0, SGI0, SGI239894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53104[] = {SGI239895, SGI0, SGI239896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239897, SGI239898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53105[] = {SGI239899, SGI0, SGI239900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239901, SGI239902, SGI239902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53106[] = {SGI239903, SGI239904, SGI239905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239906, SGI0, SGI0, SGI0, SGI239907, SGI239907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53107[] = {SGI239908, SGI0, SGI239909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239910, SGI0, SGI0, SGI0, SGI239911, SGI239911, SGI0, SGI0, SGI0, SGI239912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53108[] = {SGI239913, SGI0, SGI239914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239915, SGI239916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53109[] = {SGI239917, SGI0, SGI239918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239919, SGI239920, SGI239920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53110[] = {SGI239921, SGI0, SGI239922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239923, SGI239924, SGI239924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53111[] = {SGI239925, SGI0, SGI239926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239929, SGI239930, SGI239930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53112[] = {SGI239927, SGI0, SGI239928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239931, SGI239932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53113[] = {SGI239933, SGI0, SGI239934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239935, SGI239936, SGI239936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53114[] = {SGI239938, SGI239939, SGI239940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239941, SGI0, SGI0, SGI0, SGI239942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239943, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53115[] = {SGI239944, SGI239945, SGI239946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239947, SGI0, SGI0, SGI0, SGI239948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239949, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53116[] = {SGI239950, SGI0, SGI239951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239952, SGI0, SGI0, SGI0, SGI239953, SGI239953, SGI0, SGI0, SGI0, SGI239954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53117[] = {SGI0, SGI239955, SGI0, SGI239956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53118[] = {SGI239957, SGI0, SGI239958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53119[] = {SGI239959, SGI239960, SGI239961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239962, SGI0, SGI0, SGI0, SGI239963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239964, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53120[] = {SGI239965, SGI239966, SGI239967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239968, SGI0, SGI0, SGI0, SGI239969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239970, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53121[] = {SGI239971, SGI239972, SGI239973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239974, SGI239974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239975, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53122[] = {SGI239976, SGI239977, SGI239978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239979, SGI0, SGI0, SGI0, SGI239980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239981, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53123[] = {SGI239982, SGI239983, SGI239984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239985, SGI0, SGI0, SGI0, SGI239986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239987, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53124[] = {SGI239988, SGI0, SGI239989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53125[] = {SGI239990, SGI239991, SGI239992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239993, SGI239993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239994, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53126[] = {SGI239995, SGI0, SGI239996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI239997, SGI0, SGI0, SGI0, SGI239998, SGI239998, SGI0, SGI0, SGI0, SGI0, SGI239999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53127[] = {SGI240000, SGI0, SGI240001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240002, SGI0, SGI0, SGI0, SGI240003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240004, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53128[] = {SGI240005, SGI240006, SGI240007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240008, SGI0, SGI0, SGI0, SGI240009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240010, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53129[] = {SGI240011, SGI0, SGI240012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240013, SGI0, SGI0, SGI0, SGI240014, SGI240014, SGI0, SGI0, SGI0, SGI240015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53130[] = {SGI240016, SGI240017, SGI240018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240022, SGI240022, SGI0, SGI0, SGI0, SGI0, SGI240023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53131[] = {SGI240019, SGI240020, SGI240021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240024, SGI0, SGI0, SGI0, SGI240025, SGI0, SGI0, SGI240026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53132[] = {SGI240027, SGI0, SGI240028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240029, SGI240030, SGI240030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53133[] = {SGI240031, SGI240032, SGI240033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240034, SGI0, SGI0, SGI0, SGI240035, SGI0, SGI0, SGI240036, SGI0, SGI0, SGI0, SGI240037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53134[] = {SGI240038, SGI240039, SGI240040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240041, SGI0, SGI0, SGI0, SGI240042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240043, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53135[] = {SGI240044, SGI240045, SGI240046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240047, SGI0, SGI0, SGI0, SGI240048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240049, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53136[] = {SGI240050, SGI0, SGI240051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240052, SGI240053, SGI240053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53137[] = {SGI240054, SGI240055, SGI240056, SGI0, SGI0, SGI0, SGI240057, SGI240058, SGI0, SGI240059, SGI0, SGI0, SGI0, SGI240060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53138[] = {SGI240061, SGI240062, SGI240063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240064, SGI0, SGI0, SGI0, SGI240065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240066, SGI0, SGI0, SGI0, SGI0, SGI240067, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53139[] = {SGI240068, SGI0, SGI240069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240070, SGI240071, SGI240071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53140[] = {SGI240072, SGI240073, SGI240074, SGI240075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240076, SGI0, SGI0, SGI0, SGI240077, SGI240078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53141[] = {SGI240079, SGI240080, SGI240081, SGI240082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240083, SGI0, SGI0, SGI0, SGI240084, SGI240085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53142[] = {SGI239937, SGI0, SGI240086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240087, SGI240088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53143[] = {SGI240089, SGI240090, SGI240091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240092, SGI0, SGI0, SGI0, SGI240093, SGI240093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240094, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53144[] = {SGI240095, SGI0, SGI240096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240097, SGI240098, SGI240098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53145[] = {SGI240099, SGI240100, SGI240101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240102, SGI0, SGI0, SGI0, SGI240103, SGI240103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240104, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53146[] = {SGI240105, SGI0, SGI240106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240107, SGI240108, SGI240108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53147[] = {SGI240109, SGI0, SGI240110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240111, SGI240112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53148[] = {SGI240113, SGI240114, SGI240115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240116, SGI0, SGI0, SGI0, SGI240117, SGI0, SGI0, SGI0, SGI0, SGI240120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53149[] = {SGI240118, SGI0, SGI240119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240121, SGI240122, SGI240122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53150[] = {SGI240123, SGI240124, SGI240125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240128, SGI240128, SGI0, SGI0, SGI0, SGI240129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53151[] = {SGI240126, SGI0, SGI240127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240130, SGI240131, SGI240131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53152[] = {SGI240132, SGI240133, SGI240134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240135, SGI240135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240136, NULL};
const Segment * const SG53153[] = {SGI240137, SGI240138, SGI240139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240140, SGI0, SGI0, SGI0, SGI240141, SGI240141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240142, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53154[] = {SGI240143, SGI0, SGI240145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240147, SGI240148, SGI240148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53155[] = {SGI240144, SGI0, SGI240146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240149, SGI240150, SGI240150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53156[] = {SGI240151, SGI0, SGI240152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240153, SGI240154, SGI240154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53157[] = {SGI240155, SGI0, SGI240156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240157, SGI240158, SGI240158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53158[] = {SGI240159, SGI0, SGI240160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240161, SGI240162, SGI240162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53159[] = {SGI240163, SGI240164, SGI240165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240166, SGI0, SGI0, SGI0, SGI240167, SGI240167, SGI0, SGI0, SGI0, SGI0, SGI240168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53160[] = {SGI240169, SGI240170, SGI240171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240172, SGI240172, SGI0, SGI0, SGI0, SGI240173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53161[] = {SGI240174, SGI0, SGI240175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240176, SGI240177, SGI240177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53162[] = {SGI240178, SGI240179, SGI240180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240181, SGI0, SGI0, SGI0, SGI240182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240183, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53163[] = {SGI240184, SGI240185, SGI240186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240187, SGI0, SGI0, SGI0, SGI240188, SGI240188, SGI240189, SGI0, SGI0, SGI0, SGI0, SGI240190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53164[] = {SGI240191, SGI0, SGI240192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240193, SGI240194, SGI240194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53165[] = {SGI240195, SGI240196, SGI240197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240198, SGI0, SGI0, SGI0, SGI240199, SGI0, SGI0, SGI0, SGI240200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53166[] = {SGI240201, SGI240202, SGI240203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240204, SGI0, SGI0, SGI0, SGI240205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240206, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53167[] = {SGI240207, SGI240208, SGI240209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240210, SGI0, SGI0, SGI0, SGI240211, SGI240212, SGI0, SGI0, SGI0, SGI240213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53168[] = {SGI240214, SGI0, SGI240215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240216, SGI240217, SGI240217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240218, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53169[] = {SGI240219, SGI0, SGI240220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240221, SGI240222, SGI240222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240223, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53170[] = {SGI240224, SGI0, SGI240225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240226, SGI240227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240228, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53171[] = {SGI240229, SGI0, SGI240230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240231, SGI240232, SGI240232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53172[] = {SGI240233, SGI0, SGI240234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240235, SGI240236, SGI240236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53173[] = {SGI240237, SGI0, SGI240238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240239, SGI240240, SGI240240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53174[] = {SGI240241, SGI240242, SGI240243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240244, SGI0, SGI0, SGI0, SGI240245, SGI240245, SGI0, SGI240246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53175[] = {SGI240247, SGI0, SGI240248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240249, SGI240250, SGI240250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53176[] = {SGI240251, SGI240252, SGI240253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240254, SGI240254, SGI0, SGI0, SGI0, SGI0, SGI240255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53177[] = {SGI240256, SGI0, SGI240257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240258, SGI240259, SGI240259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53178[] = {SGI240260, SGI240262, SGI240264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240268, SGI240268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240269, NULL};
const Segment * const SG53179[] = {SGI240261, SGI240263, SGI240265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240266, SGI0, SGI0, SGI0, SGI240267, SGI240267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53180[] = {SGI240270, SGI0, SGI240271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240272, SGI240273, SGI240273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53181[] = {SGI240274, SGI240275, SGI240276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240277, SGI0, SGI0, SGI0, SGI240278, SGI240278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53182[] = {SGI240279, SGI0, SGI240280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240281, SGI240282, SGI240282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53183[] = {SGI240283, SGI0, SGI240284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240285, SGI240286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53184[] = {SGI240287, SGI0, SGI240288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240289, SGI240290, SGI240290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53185[] = {SGI240291, SGI0, SGI240292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240293, SGI240294, SGI240294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53186[] = {SGI240295, SGI0, SGI240296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240297, SGI0, SGI0, SGI0, SGI240298, SGI240298, SGI0, SGI0, SGI240299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53187[] = {SGI240300, SGI0, SGI240301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240302, SGI240303, SGI240303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53188[] = {SGI240304, SGI0, SGI240305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240306, SGI240307, SGI240307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53189[] = {SGI240308, SGI0, SGI240309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240310, SGI240311, SGI240311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53190[] = {SGI240312, SGI0, SGI240313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240314, SGI240315, SGI240315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53191[] = {SGI240316, SGI240317, SGI240318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240319, SGI0, SGI0, SGI0, SGI240320, SGI240320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240321, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53192[] = {SGI240322, SGI240323, SGI240324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240325, SGI0, SGI0, SGI0, SGI240326, SGI240326, SGI0, SGI0, SGI0, SGI240327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53193[] = {SGI240328, SGI240329, SGI240330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240333, SGI240333, SGI0, SGI0, SGI0, SGI0, SGI240335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53194[] = {SGI240331, SGI0, SGI240332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240334, SGI240336, SGI240336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53195[] = {SGI240337, SGI0, SGI240338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240339, SGI0, SGI0, SGI0, SGI240340, SGI240340, SGI0, SGI0, SGI0, SGI0, SGI240341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53196[] = {SGI240342, SGI240343, SGI240344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240345, SGI0, SGI0, SGI0, SGI240346, SGI240346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240347, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53197[] = {SGI240348, SGI240349, SGI240350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240351, SGI0, SGI0, SGI0, SGI240352, SGI240352, SGI0, SGI240353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53198[] = {SGI240354, SGI240355, SGI240356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240357, SGI0, SGI0, SGI0, SGI240358, SGI0, SGI240359, SGI0, SGI0, SGI0, SGI0, SGI240360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53199[] = {SGI240361, SGI240362, SGI240363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240364, SGI0, SGI0, SGI0, SGI240365, SGI240365, SGI0, SGI240366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53200[] = {SGI240367, SGI0, SGI240368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240369, SGI240370, SGI240370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53201[] = {SGI240371, SGI0, SGI240372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240373, SGI240374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53202[] = {SGI240375, SGI0, SGI240376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240377, SGI0, SGI0, SGI0, SGI240378, SGI240378, SGI0, SGI0, SGI240379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53203[] = {SGI240380, SGI0, SGI240381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240382, SGI240383, SGI240383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53204[] = {SGI240384, SGI0, SGI240385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240386, SGI240387, SGI240387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53205[] = {SGI240388, SGI240389, SGI240390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240391, SGI0, SGI0, SGI0, SGI240392, SGI240392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53206[] = {SGI240393, SGI240394, SGI240395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240396, SGI240396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240397, NULL};
const Segment * const SG53207[] = {SGI240398, SGI0, SGI240399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240400, SGI240401, SGI240401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53208[] = {SGI240402, SGI0, SGI240403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240404, SGI240405, SGI240405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53209[] = {SGI240406, SGI240407, SGI240408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240409, SGI240409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240410, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53210[] = {SGI240411, SGI0, SGI240412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240413, SGI240414, SGI240414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53211[] = {SGI240415, SGI0, SGI240416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240417, SGI240418, SGI240418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53212[] = {SGI240419, SGI0, SGI240420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240421, SGI240422, SGI240422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53213[] = {SGI240423, SGI0, SGI240424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240425, SGI240426, SGI240426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240427, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53214[] = {SGI240428, SGI0, SGI240429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240430, SGI240431, SGI240431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240432, SGI0, SGI0, NULL};
const Segment * const SG53215[] = {SGI240433, SGI0, SGI240434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240435, SGI240436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53216[] = {SGI240437, SGI240438, SGI240439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240440, SGI0, SGI0, SGI0, SGI240441, SGI240441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53217[] = {SGI240442, SGI0, SGI240443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240444, SGI240445, SGI240445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53218[] = {SGI240446, SGI240447, SGI240448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240449, SGI0, SGI0, SGI0, SGI240450, SGI240450, SGI0, SGI0, SGI0, SGI0, SGI240451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53219[] = {SGI240452, SGI0, SGI240453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240454, SGI0, SGI0, SGI0, SGI240455, SGI240455, SGI0, SGI0, SGI0, SGI240456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53220[] = {SGI240457, SGI240458, SGI240459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240460, SGI0, SGI0, SGI0, SGI240461, SGI240461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240463, SGI0, NULL};
const Segment * const SG53221[] = {SGI240464, SGI0, SGI240465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240466, SGI240467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53222[] = {SGI240468, SGI240469, SGI240470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240472, SGI0, SGI0, SGI0, SGI240475, SGI240475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240479, SGI0, NULL};
const Segment * const SG53223[] = {SGI240471, SGI240473, SGI240474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240477, SGI0, SGI0, SGI0, SGI240478, SGI240478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240481, SGI0, NULL};
const Segment * const SG53224[] = {SGI240482, SGI0, SGI240483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240484, SGI240485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53225[] = {SGI240486, SGI0, SGI240487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240488, SGI240489, SGI240489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53226[] = {SGI240490, SGI240491, SGI240492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240493, SGI0, SGI0, SGI0, SGI240494, SGI240494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53227[] = {SGI240496, SGI0, SGI240497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240498, SGI240499, SGI240499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53228[] = {SGI240500, SGI240501, SGI240502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240503, SGI0, SGI0, SGI0, SGI240504, SGI240504, SGI0, SGI240505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53229[] = {SGI240506, SGI0, SGI240507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240508, SGI240509, SGI240509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53230[] = {SGI240510, SGI240511, SGI240512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240513, SGI240513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240514, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53231[] = {SGI240515, SGI0, SGI240516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240517, SGI240518, SGI240518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53232[] = {SGI240519, SGI240520, SGI240521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240522, SGI0, SGI0, SGI0, SGI240523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53233[] = {SGI240525, SGI240526, SGI240527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240528, SGI0, SGI0, SGI0, SGI240529, SGI240529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240530, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53234[] = {SGI240531, SGI0, SGI240532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240533, SGI0, SGI0, SGI0, SGI240534, SGI240534, SGI0, SGI0, SGI0, SGI240535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53235[] = {SGI240536, SGI240537, SGI240538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240539, SGI0, SGI0, SGI0, SGI240540, SGI240540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53236[] = {SGI240541, SGI240542, SGI240543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240544, SGI0, SGI0, SGI0, SGI240545, SGI240545, SGI0, SGI240546, SGI0, SGI0, SGI0, SGI0, SGI240547, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53237[] = {SGI240549, SGI0, SGI240552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240554, SGI240555, SGI240555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53238[] = {SGI240548, SGI240550, SGI240551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240553, SGI0, SGI0, SGI0, SGI240556, SGI0, SGI240557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53239[] = {SGI240558, SGI0, SGI240559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240560, SGI240561, SGI240561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53240[] = {SGI240562, SGI0, SGI240563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240566, SGI240568, SGI240568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53241[] = {SGI240564, SGI240565, SGI240567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240569, SGI0, SGI0, SGI0, SGI240570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240571, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53242[] = {SGI240572, SGI0, SGI240573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240574, SGI240575, SGI240575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240579, SGI0, SGI0, NULL};
const Segment * const SG53243[] = {SGI240576, SGI240577, SGI240578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240580, SGI0, SGI240581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53244[] = {SGI240582, SGI240583, SGI240584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240587, SGI240587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240589, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53245[] = {SGI240585, SGI0, SGI240586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240588, SGI240590, SGI240590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53246[] = {SGI240591, SGI240592, SGI240593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240594, SGI0, SGI0, SGI0, SGI240595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240596, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53247[] = {SGI240597, SGI0, SGI240598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240599, SGI240600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53248[] = {SGI240601, SGI240602, SGI240603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240605, SGI0, SGI0, SGI0, SGI240608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53249[] = {SGI240604, SGI240606, SGI240607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240609, SGI0, SGI0, SGI0, SGI240610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240611, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53250[] = {SGI240612, SGI0, SGI240613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240614, SGI240615, SGI240615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53251[] = {SGI240616, SGI240617, SGI240618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240619, SGI0, SGI0, SGI0, SGI240620, SGI240620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240621, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53252[] = {SGI240622, SGI0, SGI240623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240624, SGI0, SGI0, SGI0, SGI240625, SGI240625, SGI0, SGI0, SGI0, SGI240626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53253[] = {SGI240627, SGI240628, SGI240629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240630, SGI240630, SGI0, SGI0, SGI0, SGI0, SGI240631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53254[] = {SGI240632, SGI240633, SGI240634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240635, SGI0, SGI240636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53255[] = {SGI240637, SGI240638, SGI240639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240640, SGI0, SGI0, SGI0, SGI240642, SGI0, SGI240644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53256[] = {SGI240641, SGI240643, SGI240645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240646, SGI0, SGI0, SGI0, SGI240647, SGI240647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53257[] = {SGI240648, SGI240649, SGI240650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240651, SGI0, SGI0, SGI0, SGI240652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53258[] = {SGI240653, SGI240654, SGI240655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240656, SGI0, SGI0, SGI0, SGI240658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53259[] = {SGI240657, SGI240659, SGI240660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240661, SGI0, SGI0, SGI0, SGI240662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53260[] = {SGI240664, SGI240665, SGI240666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240667, SGI0, SGI0, SGI0, SGI240670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240672, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53261[] = {SGI240668, SGI240669, SGI240671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240673, SGI240673, SGI0, SGI0, SGI0, SGI0, SGI240674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53262[] = {SGI240675, SGI240676, SGI240677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240678, SGI0, SGI0, SGI0, SGI240679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53263[] = {SGI240681, SGI0, SGI240682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240684, SGI240685, SGI240685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53264[] = {SGI240683, SGI0, SGI240686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240687, SGI0, SGI0, SGI0, SGI240688, SGI240688, SGI0, SGI0, SGI0, SGI240689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53265[] = {SGI240690, SGI0, SGI240691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240692, SGI240693, SGI240693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53266[] = {SGI240694, SGI240695, SGI240696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240697, SGI0, SGI0, SGI0, SGI240698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240699, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53267[] = {SGI240700, SGI240702, SGI240703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240707, SGI240707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240709, NULL};
const Segment * const SG53268[] = {SGI240701, SGI240704, SGI240705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240706, SGI0, SGI0, SGI0, SGI240708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240710, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53269[] = {SGI240711, SGI240712, SGI240713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240714, SGI0, SGI0, SGI0, SGI240716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53270[] = {SGI240715, SGI0, SGI240717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240718, SGI240719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240720, SGI0, SGI0, NULL};
const Segment * const SG53271[] = {SGI240721, SGI0, SGI240722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240723, SGI240724, SGI240724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53272[] = {SGI240725, SGI240726, SGI240727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240728, SGI0, SGI0, SGI0, SGI240729, SGI0, SGI240730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53273[] = {SGI240731, SGI0, SGI240732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240733, SGI240734, SGI240734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240735, SGI0, SGI0, NULL};
const Segment * const SG53274[] = {SGI240736, SGI240737, SGI240739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240743, SGI0, SGI0, SGI0, SGI240747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240749, SGI0, SGI0, SGI0, SGI0, SGI240751, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53275[] = {SGI240738, SGI240740, SGI240741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240746, SGI0, SGI0, SGI0, SGI240748, SGI240750, SGI0, SGI0, SGI0, SGI0, SGI240753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53276[] = {SGI240742, SGI240744, SGI240745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240752, SGI0, SGI0, SGI0, SGI240754, SGI240754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240756, SGI0, NULL};
const Segment * const SG53277[] = {SGI240757, SGI0, SGI240758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240759, SGI0, SGI0, SGI0, SGI240760, SGI240760, SGI0, SGI0, SGI0, SGI240761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53278[] = {SGI240762, SGI240763, SGI240764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240765, SGI0, SGI0, SGI0, SGI240766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240767, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53279[] = {SGI240768, SGI0, SGI240769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240770, SGI240771, SGI240771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53280[] = {SGI240772, SGI0, SGI240773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53281[] = {SGI0, SGI240774, SGI0, SGI240775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53282[] = {SGI240776, SGI240777, SGI240778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240779, SGI240779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240780, NULL};
const Segment * const SG53283[] = {SGI0, SGI240781, SGI0, SGI240782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53284[] = {SGI240783, SGI240784, SGI240785, SGI0, SGI0, SGI0, SGI240786, SGI0, SGI0, SGI240787, SGI0, SGI0, SGI0, SGI240788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53285[] = {SGI240789, SGI240790, SGI240791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240792, SGI0, SGI0, SGI0, SGI240793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240794, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53286[] = {SGI240795, SGI240796, SGI240797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240798, SGI0, SGI0, SGI0, SGI240799, SGI0, SGI0, SGI240800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53287[] = {SGI240801, SGI240802, SGI240803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240804, SGI0, SGI0, SGI0, SGI240805, SGI0, SGI240806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53288[] = {SGI240807, SGI240808, SGI240809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240810, SGI0, SGI0, SGI0, SGI240811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240812, NULL};
const Segment * const SG53289[] = {SGI240813, SGI240815, SGI240817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240820, SGI0, SGI0, SGI0, SGI240821, SGI0, SGI0, SGI0, SGI240823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53290[] = {SGI240814, SGI240816, SGI240818, SGI240819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240822, SGI0, SGI0, SGI0, SGI240824, SGI240825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53291[] = {SGI240829, SGI0, SGI240830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53292[] = {SGI240826, SGI240827, SGI240828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240831, SGI240831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240832, NULL};
const Segment * const SG53293[] = {SGI240833, SGI0, SGI240834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240835, SGI240836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53294[] = {SGI240837, SGI240838, SGI240839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240840, SGI0, SGI0, SGI0, SGI240841, SGI240841, SGI240842, SGI0, SGI0, SGI0, SGI0, SGI240843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53295[] = {SGI240844, SGI240845, SGI240846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240847, SGI0, SGI0, SGI0, SGI240848, SGI0, SGI240849, SGI0, SGI0, SGI0, SGI0, SGI240850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53296[] = {SGI240851, SGI240852, SGI240853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240854, SGI0, SGI0, SGI0, SGI240855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240856, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53297[] = {SGI240857, SGI240858, SGI240859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240860, SGI0, SGI0, SGI0, SGI240861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240862, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53298[] = {SGI240863, SGI240864, SGI240865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240866, SGI0, SGI0, SGI0, SGI240867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240868, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53299[] = {SGI240869, SGI240870, SGI240871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240872, SGI0, SGI0, SGI0, SGI240873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240874, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53300[] = {SGI240875, SGI240876, SGI240877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240878, SGI0, SGI0, SGI0, SGI240880, SGI240880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53301[] = {SGI240879, SGI240881, SGI240882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240883, SGI0, SGI0, SGI0, SGI240884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240885, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53302[] = {SGI240886, SGI240887, SGI240888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240889, SGI0, SGI0, SGI0, SGI240890, SGI240890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53303[] = {SGI240891, SGI240892, SGI240893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240894, SGI0, SGI0, SGI0, SGI240895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53304[] = {SGI240897, SGI0, SGI240898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240899, SGI240900, SGI240900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53305[] = {SGI240901, SGI0, SGI240902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240903, SGI240904, SGI240904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240905, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53306[] = {SGI240906, SGI0, SGI240907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240908, SGI0, SGI0, SGI0, SGI240909, SGI240909, SGI0, SGI0, SGI0, SGI240910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53307[] = {SGI240911, SGI240912, SGI240913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240914, SGI0, SGI0, SGI0, SGI240915, SGI0, SGI0, SGI0, SGI0, SGI240916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53308[] = {SGI240917, SGI0, SGI240918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53309[] = {SGI240919, SGI240920, SGI240921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240922, SGI240922, SGI0, SGI0, SGI0, SGI240923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53310[] = {SGI240924, SGI240925, SGI240926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240927, SGI0, SGI0, SGI0, SGI240928, SGI0, SGI240929, SGI0, SGI0, SGI0, SGI0, SGI240930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53311[] = {SGI240931, SGI240932, SGI240933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240934, SGI0, SGI0, SGI0, SGI240935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240936, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53312[] = {SGI240937, SGI240938, SGI240939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240940, SGI0, SGI0, SGI0, SGI240941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240942, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53313[] = {SGI240943, SGI240944, SGI240945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240946, SGI0, SGI0, SGI0, SGI240947, SGI240947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240948, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53314[] = {SGI240949, SGI240950, SGI240951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240952, SGI0, SGI0, SGI0, SGI240953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240954, SGI240955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53315[] = {SGI240956, SGI240957, SGI240958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240959, SGI240959, SGI0, SGI0, SGI0, SGI240960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53316[] = {SGI240961, SGI240962, SGI240963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240964, SGI0, SGI0, SGI0, SGI240965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53317[] = {SGI240967, SGI240968, SGI240969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240970, SGI0, SGI0, SGI0, SGI240971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240972, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53318[] = {SGI240973, SGI240974, SGI240975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240976, SGI0, SGI0, SGI0, SGI240977, SGI0, SGI0, SGI0, SGI240978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53319[] = {SGI240979, SGI240980, SGI240981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240982, SGI0, SGI0, SGI0, SGI240983, SGI0, SGI0, SGI0, SGI0, SGI240984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53320[] = {SGI240985, SGI240986, SGI240987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240988, SGI0, SGI0, SGI0, SGI240989, SGI240989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240990, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53321[] = {SGI240991, SGI240992, SGI240993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240994, SGI0, SGI0, SGI0, SGI240995, SGI0, SGI0, SGI0, SGI0, SGI240996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53322[] = {SGI240997, SGI0, SGI240998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI240999, SGI241000, SGI241000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53323[] = {SGI241001, SGI0, SGI241002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53324[] = {SGI241003, SGI241004, SGI241005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241006, SGI0, SGI0, SGI0, SGI241007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241009, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53325[] = {SGI241008, SGI0, SGI241010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241011, SGI241012, SGI241012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53326[] = {SGI241015, SGI241017, SGI241019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241021, SGI0, SGI0, SGI0, SGI241022, SGI241022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53327[] = {SGI241013, SGI241014, SGI241016, SGI241018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241020, SGI0, SGI0, SGI0, SGI241023, SGI241024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53328[] = {SGI241025, SGI241026, SGI241027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241028, SGI0, SGI0, SGI0, SGI241029, SGI0, SGI0, SGI0, SGI241030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53329[] = {SGI241031, SGI0, SGI241032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241033, SGI241034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241035, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53330[] = {SGI241036, SGI241037, SGI241038, SGI241039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241040, SGI0, SGI0, SGI0, SGI241041, SGI241042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53331[] = {SGI241044, SGI241046, SGI241048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241052, SGI241052, SGI0, SGI0, SGI0, SGI241053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53332[] = {SGI241049, SGI0, SGI241051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241054, SGI241055, SGI241055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53333[] = {SGI241056, SGI241057, SGI241058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241059, SGI0, SGI0, SGI0, SGI241060, SGI0, SGI0, SGI0, SGI0, SGI241061, SGI0, SGI241062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53334[] = {SGI241063, SGI241064, SGI241065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241066, SGI0, SGI0, SGI0, SGI241067, SGI0, SGI0, SGI0, SGI0, SGI241068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53335[] = {SGI241069, SGI241070, SGI241071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241072, SGI0, SGI0, SGI0, SGI241073, SGI241073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241074, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53336[] = {SGI241075, SGI0, SGI241076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241077, SGI241078, SGI241078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53337[] = {SGI241079, SGI241080, SGI241081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241082, SGI0, SGI0, SGI0, SGI241083, SGI0, SGI241084, SGI0, SGI0, SGI0, SGI0, SGI241085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53338[] = {SGI241086, SGI0, SGI241087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241091, SGI241092, SGI241092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53339[] = {SGI241088, SGI241089, SGI241090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241093, SGI0, SGI0, SGI0, SGI241094, SGI241094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53340[] = {SGI241095, SGI241096, SGI241097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241098, SGI0, SGI0, SGI0, SGI241099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241100, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53341[] = {SGI241102, SGI241103, SGI241105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241109, SGI0, SGI0, SGI0, SGI241111, SGI241111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241115, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53342[] = {SGI241101, SGI0, SGI241104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241110, SGI0, SGI0, SGI0, SGI241112, SGI241112, SGI0, SGI0, SGI0, SGI241114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53343[] = {SGI241106, SGI241107, SGI241108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241113, SGI0, SGI0, SGI0, SGI241116, SGI241116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53344[] = {SGI241117, SGI241119, SGI241120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241122, SGI0, SGI0, SGI0, SGI241124, SGI241124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241126, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53345[] = {SGI241118, SGI0, SGI241121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241123, SGI0, SGI0, SGI0, SGI241125, SGI241125, SGI0, SGI0, SGI0, SGI241127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53346[] = {SGI241128, SGI0, SGI241129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241130, SGI241131, SGI241131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53347[] = {SGI241132, SGI0, SGI241133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241134, SGI241135, SGI241135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53348[] = {SGI241136, SGI241137, SGI241138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241139, SGI0, SGI0, SGI0, SGI241140, SGI241140, SGI0, SGI241141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53349[] = {SGI241142, SGI241143, SGI241144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241145, SGI0, SGI0, SGI0, SGI241146, SGI0, SGI241147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53350[] = {SGI241148, SGI241149, SGI241150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241151, SGI0, SGI0, SGI0, SGI241152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53351[] = {SGI241154, SGI241155, SGI241156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241157, SGI0, SGI0, SGI0, SGI241158, SGI241158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241160, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53352[] = {SGI241159, SGI241161, SGI241162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241163, SGI0, SGI0, SGI0, SGI241164, SGI0, SGI0, SGI241165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53353[] = {SGI241166, SGI0, SGI241167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241168, SGI241169, SGI241169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53354[] = {SGI241170, SGI241171, SGI241172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241173, SGI0, SGI0, SGI0, SGI241174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241175, SGI0, SGI0, SGI0, SGI241176, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53355[] = {SGI241177, SGI0, SGI241178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241179, SGI241180, SGI241180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53356[] = {SGI241181, SGI0, SGI241182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241183, SGI241184, SGI241184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241185, SGI0, SGI0, NULL};
const Segment * const SG53357[] = {SGI241186, SGI241187, SGI241188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241189, SGI0, SGI0, SGI0, SGI241190, SGI0, SGI241191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53358[] = {SGI241192, SGI241193, SGI241194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241195, SGI0, SGI0, SGI0, SGI241196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241197, NULL};
const Segment * const SG53359[] = {SGI241198, SGI241199, SGI241200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241201, SGI0, SGI0, SGI0, SGI241202, SGI241203, SGI0, SGI0, SGI0, SGI241204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53360[] = {SGI241205, SGI241206, SGI241207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241209, SGI0, SGI0, SGI0, SGI241212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241213, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53361[] = {SGI241208, SGI241210, SGI241211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241214, SGI0, SGI0, SGI0, SGI241215, SGI241216, SGI0, SGI0, SGI0, SGI0, SGI241217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53362[] = {SGI241218, SGI241219, SGI241220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241221, SGI241221, SGI0, SGI0, SGI0, SGI0, SGI241222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53363[] = {SGI241223, SGI241224, SGI241225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241226, SGI0, SGI0, SGI0, SGI241227, SGI241227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53364[] = {SGI241228, SGI241229, SGI241230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241234, SGI0, SGI0, SGI0, SGI241235, SGI0, SGI0, SGI0, SGI241237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53365[] = {SGI241231, SGI241232, SGI241233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241236, SGI0, SGI0, SGI0, SGI241238, SGI241238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241239, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53366[] = {SGI241240, SGI0, SGI241241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241242, SGI241243, SGI241243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53367[] = {SGI241244, SGI241245, SGI241246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241247, SGI0, SGI0, SGI0, SGI241248, SGI241248, SGI0, SGI0, SGI0, SGI241249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53368[] = {SGI241250, SGI241251, SGI241252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241253, SGI0, SGI0, SGI0, SGI241254, SGI241254, SGI0, SGI0, SGI0, SGI241255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53369[] = {SGI0, SGI241256, SGI241257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241258, SGI0, SGI0, SGI0, SGI241259, SGI241259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53370[] = {SGI241260, SGI0, SGI241261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241262, SGI241263, SGI241263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53371[] = {SGI241264, SGI0, SGI241266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241270, SGI241272, SGI241272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53372[] = {SGI241265, SGI0, SGI241267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241271, SGI241273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53373[] = {SGI241268, SGI0, SGI241269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241274, SGI241275, SGI241275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53374[] = {SGI241276, SGI0, SGI241277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241278, SGI241279, SGI241279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53375[] = {SGI241280, SGI241281, SGI241282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241283, SGI0, SGI0, SGI0, SGI241284, SGI241284, SGI0, SGI0, SGI0, SGI241285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53376[] = {SGI241286, SGI0, SGI241287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241289, SGI241290, SGI241290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241291, SGI0, SGI0, NULL};
const Segment * const SG53377[] = {SGI241292, SGI241293, SGI241294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241295, SGI0, SGI0, SGI0, SGI241296, SGI241296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241297, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53378[] = {SGI241298, SGI0, SGI241299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241300, SGI0, SGI0, SGI0, SGI241301, SGI241301, SGI0, SGI0, SGI0, SGI0, SGI241302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53379[] = {SGI241303, SGI0, SGI241304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241305, SGI241306, SGI241306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53380[] = {SGI241307, SGI241308, SGI241309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241310, SGI241310, SGI0, SGI0, SGI0, SGI0, SGI241311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53381[] = {SGI241312, SGI0, SGI241313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241314, SGI241315, SGI241315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241316, SGI0, SGI0, NULL};
const Segment * const SG53382[] = {SGI241317, SGI0, SGI241318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241319, SGI0, SGI0, SGI0, SGI241320, SGI241320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241321, NULL};
const Segment * const SG53383[] = {SGI241322, SGI0, SGI241323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241324, SGI241325, SGI241325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53384[] = {SGI241326, SGI0, SGI241327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241328, SGI241329, SGI241329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241330, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53385[] = {SGI241331, SGI0, SGI241332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241333, SGI241334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241335, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53386[] = {SGI241336, SGI0, SGI241337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241338, SGI241339, SGI241339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241340, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53387[] = {SGI241341, SGI0, SGI241342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241343, SGI241344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53388[] = {SGI241043, SGI241045, SGI241047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241050, SGI0, SGI0, SGI0, SGI241288, SGI241288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53389[] = {SGI241345, SGI241346, SGI241347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241348, SGI0, SGI0, SGI0, SGI241349, SGI241349, SGI0, SGI0, SGI241350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53390[] = {SGI241351, SGI0, SGI241352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241353, SGI241354, SGI241354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53391[] = {SGI241355, SGI0, SGI241356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241357, SGI241358, SGI241358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241359, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53392[] = {SGI241360, SGI0, SGI241361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241362, SGI241363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53393[] = {SGI241364, SGI0, SGI241365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241366, SGI241367, SGI241367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53394[] = {SGI241368, SGI0, SGI241369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53395[] = {SGI241370, SGI241371, SGI241372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241373, SGI0, SGI0, SGI0, SGI241374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53396[] = {SGI241375, SGI0, SGI241376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241377, SGI0, SGI0, SGI0, SGI241378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241379, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53397[] = {SGI241380, SGI241381, SGI241382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241383, SGI0, SGI0, SGI0, SGI241384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241385, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53398[] = {SGI241386, SGI241387, SGI241388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241389, SGI0, SGI0, SGI0, SGI241390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241391, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53399[] = {SGI241392, SGI0, SGI241393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241394, SGI241395, SGI241395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53400[] = {SGI241396, SGI0, SGI241397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241400, SGI241401, SGI241401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53401[] = {SGI241398, SGI0, SGI241399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241402, SGI241403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53402[] = {SGI241404, SGI241405, SGI241406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241407, SGI241407, SGI0, SGI0, SGI0, SGI0, SGI241408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53403[] = {SGI241409, SGI241410, SGI241411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241412, SGI0, SGI0, SGI0, SGI241413, SGI241413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53404[] = {SGI241414, SGI241415, SGI241416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241417, SGI0, SGI0, SGI0, SGI241418, SGI241418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53405[] = {SGI241419, SGI241420, SGI241421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241424, SGI241424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241425, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53406[] = {SGI241422, SGI0, SGI241423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241426, SGI241427, SGI241427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53407[] = {SGI241428, SGI0, SGI241429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241430, SGI241431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53408[] = {SGI241432, SGI241433, SGI241434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241435, SGI0, SGI0, SGI0, SGI241436, SGI241436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241437, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53409[] = {SGI241438, SGI0, SGI241439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241440, SGI241441, SGI241441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53410[] = {SGI241442, SGI0, SGI241443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241444, SGI241445, SGI241445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241446, SGI0, SGI0, NULL};
const Segment * const SG53411[] = {SGI241447, SGI0, SGI241448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241449, SGI241450, SGI241450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53412[] = {SGI241451, SGI241452, SGI241453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241454, SGI241454, SGI0, SGI0, SGI0, SGI0, SGI241455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53413[] = {SGI241456, SGI0, SGI241457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241458, SGI241459, SGI241459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53414[] = {SGI241460, SGI0, SGI241461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241463, SGI241464, SGI241464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53415[] = {SGI241462, SGI0, SGI241465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241466, SGI241467, SGI241467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241468, SGI0, SGI0, NULL};
const Segment * const SG53416[] = {SGI241469, SGI0, SGI241470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241471, SGI241472, SGI241472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53417[] = {SGI241473, SGI0, SGI241474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241475, SGI241476, SGI241476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53418[] = {SGI241477, SGI241478, SGI241479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241480, SGI0, SGI0, SGI0, SGI241481, SGI241481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53419[] = {SGI241482, SGI0, SGI241483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241484, SGI241485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53420[] = {SGI241486, SGI0, SGI241487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241488, SGI0, SGI0, SGI0, SGI241489, SGI241489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241490, NULL};
const Segment * const SG53421[] = {SGI241491, SGI0, SGI241492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241493, SGI241494, SGI241494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53422[] = {SGI241495, SGI241496, SGI241497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241498, SGI0, SGI0, SGI0, SGI241499, SGI241499, SGI241500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53423[] = {SGI241501, SGI241502, SGI241503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241504, SGI0, SGI0, SGI0, SGI241505, SGI241505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53424[] = {SGI241506, SGI0, SGI241507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241508, SGI241509, SGI241509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241510, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53425[] = {SGI241511, SGI241512, SGI241513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241514, SGI0, SGI0, SGI0, SGI241515, SGI241515, SGI0, SGI0, SGI241516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53426[] = {SGI241517, SGI0, SGI241518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241519, SGI241520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53427[] = {SGI241521, SGI241522, SGI241523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241524, SGI241524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241525, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53428[] = {SGI241526, SGI241527, SGI241528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241529, SGI241529, SGI0, SGI0, SGI0, SGI0, SGI241530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53429[] = {SGI241531, SGI241532, SGI241533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241534, SGI0, SGI0, SGI0, SGI241536, SGI241536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241539, SGI0, NULL};
const Segment * const SG53430[] = {SGI241535, SGI0, SGI241537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241540, SGI241541, SGI241541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53431[] = {SGI241542, SGI241543, SGI241544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241545, SGI0, SGI0, SGI0, SGI241546, SGI241546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241548, SGI0, NULL};
const Segment * const SG53432[] = {SGI241549, SGI241550, SGI241551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241552, SGI241552, SGI0, SGI0, SGI0, SGI0, SGI241553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53433[] = {SGI241554, SGI241555, SGI241556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241557, SGI0, SGI0, SGI0, SGI241558, SGI241558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241559, NULL};
const Segment * const SG53434[] = {SGI241560, SGI0, SGI241561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241562, SGI241563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53435[] = {SGI241564, SGI0, SGI241565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241566, SGI241567, SGI241567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53436[] = {SGI241568, SGI0, SGI241569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241570, SGI0, SGI0, SGI0, SGI241571, SGI241571, SGI0, SGI0, SGI241572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53437[] = {SGI241573, SGI241574, SGI241575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241576, SGI0, SGI0, SGI0, SGI241577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241579, SGI0, NULL};
const Segment * const SG53438[] = {SGI241580, SGI241581, SGI241582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241583, SGI0, SGI0, SGI0, SGI241585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53439[] = {SGI241584, SGI241586, SGI241587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241588, SGI0, SGI0, SGI0, SGI241589, SGI241589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241591, SGI0, NULL};
const Segment * const SG53440[] = {SGI241592, SGI0, SGI241593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241594, SGI241595, SGI241595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53441[] = {SGI241596, SGI0, SGI241597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241598, SGI241599, SGI241599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53442[] = {SGI241600, SGI0, SGI241601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241602, SGI241603, SGI241603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53443[] = {SGI241604, SGI0, SGI241605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241606, SGI0, SGI0, SGI0, SGI241607, SGI241607, SGI0, SGI0, SGI241608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53444[] = {SGI241609, SGI241610, SGI241611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241612, SGI0, SGI0, SGI0, SGI241613, SGI241613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53445[] = {SGI241614, SGI241615, SGI241616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241617, SGI241617, SGI0, SGI0, SGI0, SGI0, SGI241618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53446[] = {SGI241619, SGI0, SGI241620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241621, SGI0, SGI0, SGI0, SGI241622, SGI241622, SGI0, SGI0, SGI241623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53447[] = {SGI241624, SGI241625, SGI241626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241627, SGI241627, SGI0, SGI0, SGI0, SGI0, SGI241628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53448[] = {SGI241629, SGI241630, SGI241631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241632, SGI0, SGI0, SGI0, SGI241633, SGI0, SGI0, SGI241634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53449[] = {SGI241635, SGI241636, SGI241637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241638, SGI0, SGI0, SGI0, SGI241639, SGI0, SGI0, SGI241640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53450[] = {SGI241641, SGI0, SGI241642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241643, SGI241644, SGI241644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241645, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53451[] = {SGI241646, SGI0, SGI241647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241648, SGI241649, SGI241649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53452[] = {SGI241650, SGI0, SGI241651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241652, SGI241653, SGI241653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53453[] = {SGI241654, SGI0, SGI241655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241656, SGI241657, SGI241657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53454[] = {SGI241658, SGI0, SGI241659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241662, SGI241663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53455[] = {SGI241660, SGI0, SGI241661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241664, SGI241665, SGI241665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53456[] = {SGI241666, SGI0, SGI241667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241668, SGI241669, SGI241669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53457[] = {SGI241670, SGI0, SGI241671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241672, SGI241673, SGI241673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53458[] = {SGI241674, SGI0, SGI241675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241676, SGI241677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53459[] = {SGI241678, SGI0, SGI241679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241680, SGI241681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53460[] = {SGI241682, SGI241683, SGI241684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241685, SGI0, SGI0, SGI0, SGI241688, SGI241688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241692, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53461[] = {SGI241689, SGI0, SGI241691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241693, SGI241695, SGI241695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53462[] = {SGI241686, SGI241687, SGI241690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241694, SGI0, SGI0, SGI0, SGI241696, SGI0, SGI241697, SGI241698, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53463[] = {SGI241699, SGI241700, SGI241701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241702, SGI0, SGI0, SGI0, SGI241703, SGI0, SGI241704, SGI241705, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53464[] = {SGI241706, SGI241707, SGI241708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241709, SGI241709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241710, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53465[] = {SGI241711, SGI0, SGI241712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241713, SGI241714, SGI241714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53466[] = {SGI241715, SGI0, SGI241716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241717, SGI241718, SGI241718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53467[] = {SGI241719, SGI0, SGI241720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241721, SGI241722, SGI241722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53468[] = {SGI241723, SGI0, SGI241724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241725, SGI241726, SGI241726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53469[] = {SGI241727, SGI0, SGI241728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241729, SGI241730, SGI241730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53470[] = {SGI241731, SGI241732, SGI241733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241734, SGI0, SGI0, SGI0, SGI241735, SGI241735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241736, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53471[] = {SGI241737, SGI241738, SGI241739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241743, SGI0, SGI0, SGI0, SGI241744, SGI0, SGI0, SGI241745, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53472[] = {SGI241740, SGI241741, SGI241742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241746, SGI0, SGI0, SGI0, SGI241747, SGI241747, SGI0, SGI0, SGI0, SGI241748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53473[] = {SGI241749, SGI241750, SGI241751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241752, SGI241752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241753, NULL};
const Segment * const SG53474[] = {SGI241754, SGI241755, SGI241756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241757, SGI0, SGI0, SGI0, SGI241758, SGI241758, SGI0, SGI0, SGI0, SGI241759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53475[] = {SGI241760, SGI0, SGI241761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241762, SGI241763, SGI241763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53476[] = {SGI241764, SGI0, SGI241765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241766, SGI241767, SGI241767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53477[] = {SGI241768, SGI0, SGI241769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241770, SGI241771, SGI241771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53478[] = {SGI241772, SGI0, SGI241773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241774, SGI241775, SGI241775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53479[] = {SGI241776, SGI0, SGI241777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241778, SGI241779, SGI241779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53480[] = {SGI241780, SGI241781, SGI241782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241783, SGI0, SGI0, SGI0, SGI241784, SGI241784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241785, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53481[] = {SGI241786, SGI0, SGI241787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241788, SGI241789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241790, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53482[] = {SGI241791, SGI241792, SGI241793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241795, SGI241795, SGI0, SGI0, SGI0, SGI0, SGI241797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53483[] = {SGI241794, SGI0, SGI241796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241798, SGI241799, SGI241799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53484[] = {SGI241800, SGI0, SGI241801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241802, SGI241803, SGI241803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53485[] = {SGI241804, SGI0, SGI241805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241806, SGI241807, SGI241807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53486[] = {SGI241808, SGI0, SGI241809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241810, SGI241811, SGI241811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241812, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53487[] = {SGI241813, SGI0, SGI241814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241815, SGI241816, SGI241816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53488[] = {SGI241817, SGI0, SGI241818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241819, SGI241820, SGI241820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241821, SGI0, SGI0, NULL};
const Segment * const SG53489[] = {SGI241822, SGI0, SGI241823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241824, SGI241825, SGI241825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53490[] = {SGI241826, SGI0, SGI241827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241828, SGI241829, SGI241829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53491[] = {SGI241830, SGI0, SGI241831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241832, SGI241833, SGI241833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53492[] = {SGI241834, SGI241835, SGI241836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241837, SGI0, SGI0, SGI0, SGI241838, SGI241838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241840, SGI0, NULL};
const Segment * const SG53493[] = {SGI241841, SGI0, SGI241842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241843, SGI241844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53494[] = {SGI0, SGI241845, SGI0, SGI241846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53495[] = {SGI0, SGI241847, SGI0, SGI241848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241849, SGI0, SGI0, SGI0, SGI241850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53496[] = {SGI241851, SGI0, SGI241852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241853, SGI0, SGI0, SGI0, SGI241854, SGI241854, SGI0, SGI0, SGI241855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53497[] = {SGI241856, SGI241857, SGI241858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241859, SGI0, SGI0, SGI0, SGI241860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53498[] = {SGI241862, SGI0, SGI241863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241864, SGI241865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53499[] = {SGI241866, SGI0, SGI241867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241868, SGI241869, SGI241869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53500[] = {SGI241870, SGI0, SGI241871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241872, SGI241873, SGI241873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53501[] = {SGI241874, SGI241875, SGI241876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241877, SGI0, SGI0, SGI0, SGI241878, SGI0, SGI241879, SGI0, SGI0, SGI0, SGI0, SGI241880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53502[] = {SGI241881, SGI0, SGI241882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241883, SGI0, SGI0, SGI0, SGI241884, SGI241884, SGI0, SGI0, SGI241885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53503[] = {SGI241886, SGI0, SGI241887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241888, SGI0, SGI0, SGI0, SGI241889, SGI241889, SGI0, SGI0, SGI241890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53504[] = {SGI241891, SGI241892, SGI241893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241894, SGI0, SGI0, SGI0, SGI241895, SGI241896, SGI0, SGI0, SGI241897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53505[] = {SGI241898, SGI241899, SGI241900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241901, SGI0, SGI0, SGI0, SGI241902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241903, NULL};
const Segment * const SG53506[] = {SGI241904, SGI0, SGI241905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241906, SGI241907, SGI241907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53507[] = {SGI241908, SGI0, SGI241909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241910, SGI241911, SGI241911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53508[] = {SGI241912, SGI0, SGI241913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241914, SGI241915, SGI241915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53509[] = {SGI241916, SGI0, SGI241917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241918, SGI241919, SGI241919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53510[] = {SGI241920, SGI0, SGI241921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241922, SGI241923, SGI241923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53511[] = {SGI241924, SGI0, SGI241925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241926, SGI241927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53512[] = {SGI241928, SGI0, SGI241929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241930, SGI241931, SGI241931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53513[] = {SGI241932, SGI0, SGI241933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241934, SGI241935, SGI241935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53514[] = {SGI241936, SGI0, SGI241937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241938, SGI241939, SGI241939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53515[] = {SGI241940, SGI0, SGI241941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241942, SGI241943, SGI241943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53516[] = {SGI241944, SGI0, SGI241945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241946, SGI241947, SGI241947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53517[] = {SGI241948, SGI0, SGI241949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241950, SGI241951, SGI241951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53518[] = {SGI241952, SGI0, SGI241953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241954, SGI241955, SGI241955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53519[] = {SGI241956, SGI0, SGI241957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241958, SGI241959, SGI241959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53520[] = {SGI241960, SGI0, SGI241961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241962, SGI241963, SGI241963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53521[] = {SGI241964, SGI0, SGI241965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241966, SGI241967, SGI241967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53522[] = {SGI241968, SGI0, SGI241969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241970, SGI241971, SGI241971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53523[] = {SGI241972, SGI241973, SGI241974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241975, SGI0, SGI0, SGI0, SGI241976, SGI241976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53524[] = {SGI241977, SGI241978, SGI241979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241980, SGI0, SGI0, SGI0, SGI241981, SGI241981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241982, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53525[] = {SGI241983, SGI241984, SGI241985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241986, SGI0, SGI0, SGI0, SGI241987, SGI241987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53526[] = {SGI241988, SGI0, SGI241989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241990, SGI241991, SGI241991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53527[] = {SGI241995, SGI0, SGI241996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241999, SGI242000, SGI242000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53528[] = {SGI241992, SGI241993, SGI241994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI241997, SGI0, SGI0, SGI0, SGI241998, SGI241998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242002, SGI0, NULL};
const Segment * const SG53529[] = {SGI242003, SGI242004, SGI242005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242006, SGI0, SGI0, SGI0, SGI242007, SGI242007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242008, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53530[] = {SGI242009, SGI0, SGI242010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242011, SGI242012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53531[] = {SGI242013, SGI242014, SGI242015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242016, SGI242016, SGI0, SGI0, SGI0, SGI0, SGI242017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53532[] = {SGI242018, SGI242019, SGI242020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242021, SGI0, SGI0, SGI0, SGI242022, SGI242022, SGI0, SGI0, SGI0, SGI242023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53533[] = {SGI242024, SGI242025, SGI242026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242027, SGI0, SGI0, SGI0, SGI242028, SGI242029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242030, NULL};
const Segment * const SG53534[] = {SGI242031, SGI0, SGI242032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242036, SGI242037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53535[] = {SGI242033, SGI242034, SGI242035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242038, SGI242038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242039, NULL};
const Segment * const SG53536[] = {SGI242040, SGI242041, SGI242042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242043, SGI0, SGI0, SGI0, SGI242044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242045, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53537[] = {SGI242046, SGI242047, SGI242048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242049, SGI242049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242050, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53538[] = {SGI242051, SGI0, SGI242052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242053, SGI242054, SGI242054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53539[] = {SGI242055, SGI242056, SGI242057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242058, SGI0, SGI0, SGI0, SGI242059, SGI242059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242060, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53540[] = {SGI242061, SGI242062, SGI242063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242064, SGI0, SGI0, SGI0, SGI242065, SGI0, SGI0, SGI0, SGI242066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53541[] = {SGI242067, SGI242068, SGI242069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242070, SGI242070, SGI0, SGI0, SGI242071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53542[] = {SGI242072, SGI242073, SGI242074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242075, SGI0, SGI0, SGI0, SGI242076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242080, NULL};
const Segment * const SG53543[] = {SGI242077, SGI242078, SGI242079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242081, SGI242081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242082, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53544[] = {SGI242083, SGI0, SGI242084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242085, SGI242086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53545[] = {SGI242087, SGI0, SGI242088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242089, SGI242090, SGI242090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53546[] = {SGI242091, SGI242092, SGI242093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242094, SGI0, SGI0, SGI0, SGI242095, SGI242095, SGI0, SGI242096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53547[] = {SGI242097, SGI242098, SGI242099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242100, SGI0, SGI0, SGI0, SGI242101, SGI242101, SGI0, SGI242102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53548[] = {SGI242103, SGI242104, SGI242105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242106, SGI0, SGI0, SGI0, SGI242107, SGI242107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53549[] = {SGI242108, SGI242109, SGI242110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242111, SGI0, SGI0, SGI0, SGI242112, SGI242112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53550[] = {SGI242113, SGI242114, SGI242115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242116, SGI242116, SGI0, SGI0, SGI0, SGI0, SGI242118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53551[] = {SGI242117, SGI242119, SGI242120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242121, SGI242121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242122, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53552[] = {SGI242123, SGI242124, SGI242125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242126, SGI242126, SGI0, SGI0, SGI0, SGI0, SGI242127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53553[] = {SGI242128, SGI242129, SGI242130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242131, SGI0, SGI0, SGI0, SGI242132, SGI242132, SGI242133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53554[] = {SGI242134, SGI242135, SGI242136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242137, SGI242137, SGI0, SGI0, SGI0, SGI0, SGI242138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53555[] = {SGI242139, SGI0, SGI242140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242141, SGI242142, SGI242142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53556[] = {SGI242143, SGI242144, SGI242145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242146, SGI0, SGI0, SGI0, SGI242147, SGI242147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53557[] = {SGI242148, SGI242149, SGI242150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242151, SGI242151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242152, NULL};
const Segment * const SG53558[] = {SGI242153, SGI242154, SGI242155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242156, SGI0, SGI0, SGI0, SGI242157, SGI242157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53559[] = {SGI242158, SGI0, SGI242159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242160, SGI242161, SGI242161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53560[] = {SGI242162, SGI242163, SGI242164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242165, SGI0, SGI0, SGI0, SGI242166, SGI242167, SGI0, SGI0, SGI0, SGI0, SGI242168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53561[] = {SGI242169, SGI0, SGI242170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242171, SGI242172, SGI242172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53562[] = {SGI242173, SGI242174, SGI242175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242176, SGI0, SGI0, SGI0, SGI242177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53563[] = {SGI242179, SGI0, SGI242180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242181, SGI242182, SGI242182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53564[] = {SGI242183, SGI0, SGI242184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242185, SGI242186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53565[] = {SGI242187, SGI0, SGI242188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242189, SGI242190, SGI242190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53566[] = {SGI242191, SGI0, SGI242192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242193, SGI242194, SGI242194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53567[] = {SGI242195, SGI242196, SGI242197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242198, SGI0, SGI0, SGI0, SGI242199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53568[] = {SGI242200, SGI242201, SGI242202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242203, SGI0, SGI0, SGI0, SGI242204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53569[] = {SGI242206, SGI242207, SGI242208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242209, SGI0, SGI0, SGI0, SGI242210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53570[] = {SGI242211, SGI242212, SGI242213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242214, SGI0, SGI0, SGI0, SGI242215, SGI0, SGI0, SGI0, SGI0, SGI242216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53571[] = {SGI242217, SGI242218, SGI242219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242220, SGI0, SGI0, SGI0, SGI242221, SGI0, SGI0, SGI0, SGI0, SGI242222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53572[] = {SGI242223, SGI0, SGI242224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242225, SGI0, SGI0, SGI0, SGI242226, SGI242226, SGI0, SGI0, SGI242227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53573[] = {SGI242228, SGI0, SGI242229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242230, SGI0, SGI0, SGI0, SGI242231, SGI242231, SGI0, SGI0, SGI242232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53574[] = {SGI242233, SGI242234, SGI242235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242236, SGI0, SGI0, SGI0, SGI242237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242240, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53575[] = {SGI242238, SGI0, SGI242239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242241, SGI242242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53576[] = {SGI242243, SGI242244, SGI242245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242246, SGI0, SGI0, SGI0, SGI242247, SGI242247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53577[] = {SGI242248, SGI0, SGI242249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242251, SGI242253, SGI242253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53578[] = {SGI242250, SGI0, SGI242252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242254, SGI0, SGI0, SGI0, SGI242255, SGI242255, SGI0, SGI0, SGI242256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53579[] = {SGI242257, SGI242258, SGI242259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242260, SGI0, SGI0, SGI0, SGI242261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53580[] = {SGI242263, SGI242264, SGI242265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242269, SGI242269, SGI0, SGI0, SGI242270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53581[] = {SGI242266, SGI242267, SGI242268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242271, SGI0, SGI0, SGI0, SGI242272, SGI242272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53582[] = {SGI242273, SGI242274, SGI242275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242276, SGI0, SGI0, SGI0, SGI242277, SGI0, SGI0, SGI0, SGI242278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53583[] = {SGI242279, SGI242280, SGI242281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242282, SGI242282, SGI0, SGI0, SGI242283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53584[] = {SGI242284, SGI0, SGI242285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242286, SGI242287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53585[] = {SGI242288, SGI242289, SGI242290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242291, SGI0, SGI0, SGI0, SGI242292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53586[] = {SGI242293, SGI0, SGI242294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53587[] = {SGI242295, SGI0, SGI242296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242297, SGI242298, SGI242298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53588[] = {SGI242299, SGI242300, SGI242301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242302, SGI0, SGI0, SGI0, SGI242303, SGI0, SGI0, SGI0, SGI0, SGI242304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53589[] = {SGI242305, SGI242306, SGI242307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242308, SGI0, SGI0, SGI0, SGI242309, SGI0, SGI0, SGI0, SGI242310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53590[] = {SGI242311, SGI242312, SGI242313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242314, SGI0, SGI0, SGI0, SGI242315, SGI242315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53591[] = {SGI242316, SGI0, SGI242317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242318, SGI242319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53592[] = {SGI242320, SGI0, SGI242321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242322, SGI242323, SGI242323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53593[] = {SGI242324, SGI0, SGI242325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242326, SGI242327, SGI242327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53594[] = {SGI242328, SGI242329, SGI242330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242331, SGI0, SGI0, SGI0, SGI242332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53595[] = {SGI242334, SGI0, SGI242335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53596[] = {SGI242336, SGI0, SGI242337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242338, SGI242339, SGI242339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53597[] = {SGI242340, SGI0, SGI242341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242342, SGI242343, SGI242343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53598[] = {SGI242344, SGI0, SGI242345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53599[] = {SGI242347, SGI0, SGI242348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242350, SGI242351, SGI242351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53600[] = {SGI242346, SGI0, SGI242349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242352, SGI242353, SGI242353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53601[] = {SGI242354, SGI0, SGI242355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242356, SGI242357, SGI242357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53602[] = {SGI242358, SGI0, SGI242359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242360, SGI242361, SGI242361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53603[] = {SGI242362, SGI242363, SGI242364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242365, SGI0, SGI0, SGI0, SGI242366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242367, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53604[] = {SGI242368, SGI0, SGI242369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242370, SGI242371, SGI242371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242372, SGI0, SGI0, NULL};
const Segment * const SG53605[] = {SGI242373, SGI0, SGI242374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242375, SGI242376, SGI242376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242377, SGI0, SGI0, NULL};
const Segment * const SG53606[] = {SGI242378, SGI0, SGI242379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242380, SGI242381, SGI242381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53607[] = {SGI242382, SGI242383, SGI242384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242385, SGI0, SGI0, SGI0, SGI242386, SGI242386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53608[] = {SGI242387, SGI242388, SGI242389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242390, SGI0, SGI0, SGI0, SGI242391, SGI242391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242392, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53609[] = {SGI242393, SGI242394, SGI242395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242396, SGI0, SGI0, SGI0, SGI242397, SGI242397, SGI0, SGI242400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53610[] = {SGI242398, SGI242399, SGI242401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242402, SGI0, SGI0, SGI0, SGI242403, SGI242403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53611[] = {SGI242404, SGI242405, SGI242406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242407, SGI0, SGI0, SGI0, SGI242408, SGI242408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242409, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53612[] = {SGI242410, SGI0, SGI242411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242412, SGI242413, SGI242413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53613[] = {SGI242414, SGI0, SGI242415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242416, SGI242417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53614[] = {SGI242418, SGI0, SGI242419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242420, SGI242421, SGI242421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53615[] = {SGI242422, SGI0, SGI242423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242424, SGI242425, SGI242425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53616[] = {SGI242426, SGI0, SGI242427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242428, SGI242429, SGI242429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53617[] = {SGI242430, SGI242431, SGI242432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242433, SGI242433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242434, NULL};
const Segment * const SG53618[] = {SGI242435, SGI242436, SGI242437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242438, SGI0, SGI0, SGI0, SGI242439, SGI242439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242440, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53619[] = {SGI242441, SGI242442, SGI242443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242444, SGI0, SGI0, SGI0, SGI242445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242446, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53620[] = {SGI242447, SGI0, SGI242448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242449, SGI242450, SGI242450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53621[] = {SGI242451, SGI242452, SGI242453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242454, SGI0, SGI0, SGI0, SGI242455, SGI242455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242456, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53622[] = {SGI242457, SGI0, SGI242458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242459, SGI242460, SGI242460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53623[] = {SGI242461, SGI0, SGI242462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242463, SGI242464, SGI242464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53624[] = {SGI242465, SGI242466, SGI242467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242468, SGI0, SGI0, SGI0, SGI242469, SGI242469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242470, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53625[] = {SGI242471, SGI0, SGI242472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242473, SGI242474, SGI242474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53626[] = {SGI242475, SGI0, SGI242476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242477, SGI242478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53627[] = {SGI242479, SGI0, SGI242480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242481, SGI242482, SGI242482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53628[] = {SGI242483, SGI0, SGI242484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242485, SGI242486, SGI242486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53629[] = {SGI242487, SGI242488, SGI242489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242490, SGI0, SGI0, SGI0, SGI242491, SGI242491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53630[] = {SGI242492, SGI0, SGI242493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242494, SGI242495, SGI242495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53631[] = {SGI242496, SGI0, SGI242497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242498, SGI0, SGI0, SGI0, SGI242499, SGI242499, SGI0, SGI0, SGI0, SGI0, SGI242500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53632[] = {SGI242501, SGI242502, SGI242503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242504, SGI0, SGI0, SGI0, SGI242505, SGI242505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242506, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53633[] = {SGI242507, SGI242508, SGI242509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242510, SGI242510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242511, NULL};
const Segment * const SG53634[] = {SGI242512, SGI242513, SGI242514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242515, SGI0, SGI0, SGI0, SGI242516, SGI242517, SGI0, SGI0, SGI0, SGI0, SGI242518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53635[] = {SGI242519, SGI0, SGI242520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242521, SGI242522, SGI242522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53636[] = {SGI242523, SGI242524, SGI242525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242526, SGI0, SGI0, SGI0, SGI242527, SGI242527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242528, NULL};
const Segment * const SG53637[] = {SGI242529, SGI0, SGI242530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242531, SGI242532, SGI242532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53638[] = {SGI242533, SGI0, SGI242534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242535, SGI242536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53639[] = {SGI242537, SGI0, SGI242538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242539, SGI242540, SGI242540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53640[] = {SGI242541, SGI0, SGI242542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242543, SGI242544, SGI242544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53641[] = {SGI242545, SGI242546, SGI242547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242548, SGI0, SGI0, SGI0, SGI242549, SGI242550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242551, NULL};
const Segment * const SG53642[] = {SGI242552, SGI0, SGI242553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242554, SGI242555, SGI242555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53643[] = {SGI242556, SGI242557, SGI242558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242559, SGI0, SGI0, SGI0, SGI242560, SGI0, SGI0, SGI0, SGI242561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53644[] = {SGI242562, SGI0, SGI242563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242564, SGI242565, SGI242565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53645[] = {SGI242566, SGI242567, SGI242568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242569, SGI0, SGI0, SGI0, SGI242570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53646[] = {SGI242571, SGI0, SGI242572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242573, SGI242574, SGI242574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53647[] = {SGI242575, SGI0, SGI242576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242577, SGI242578, SGI242578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53648[] = {SGI242579, SGI0, SGI242580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242581, SGI242582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242583, SGI0, NULL};
const Segment * const SG53649[] = {SGI242584, SGI0, SGI242585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242586, SGI242587, SGI242587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53650[] = {SGI242588, SGI0, SGI242589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242590, SGI242591, SGI242591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53651[] = {SGI242592, SGI0, SGI242593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242594, SGI242595, SGI242595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53652[] = {SGI242596, SGI0, SGI242597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242598, SGI242599, SGI242599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53653[] = {SGI242600, SGI242601, SGI242602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242603, SGI0, SGI0, SGI0, SGI242604, SGI242604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242605, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53654[] = {SGI242606, SGI242607, SGI242608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242609, SGI0, SGI0, SGI0, SGI242610, SGI242610, SGI0, SGI0, SGI0, SGI242611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53655[] = {SGI242612, SGI0, SGI242613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242614, SGI242615, SGI242615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53656[] = {SGI242616, SGI0, SGI242617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242618, SGI242619, SGI242619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242620, SGI0, SGI0, NULL};
const Segment * const SG53657[] = {SGI242621, SGI0, SGI242622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242623, SGI242624, SGI242624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53658[] = {SGI242625, SGI242626, SGI242627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242628, SGI242628, SGI0, SGI0, SGI0, SGI0, SGI242629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53659[] = {SGI242630, SGI242631, SGI242632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242633, SGI242633, SGI0, SGI0, SGI0, SGI0, SGI242634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53660[] = {SGI242635, SGI0, SGI242636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242637, SGI242638, SGI242638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53661[] = {SGI242639, SGI242640, SGI242641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242642, SGI0, SGI0, SGI0, SGI242643, SGI242643, SGI242644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53662[] = {SGI242645, SGI242646, SGI242647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242648, SGI0, SGI0, SGI0, SGI242649, SGI242649, SGI0, SGI242650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53663[] = {SGI242651, SGI242652, SGI242653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242654, SGI0, SGI0, SGI0, SGI242655, SGI242655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53664[] = {SGI242656, SGI0, SGI242657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242658, SGI242659, SGI242659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53665[] = {SGI242660, SGI242661, SGI242662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242663, SGI0, SGI0, SGI0, SGI242664, SGI242664, SGI242665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53666[] = {SGI242666, SGI242667, SGI242668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242669, SGI0, SGI0, SGI0, SGI242670, SGI242670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242671, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53667[] = {SGI242672, SGI242673, SGI242674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242675, SGI0, SGI0, SGI0, SGI242676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53668[] = {SGI242677, SGI242678, SGI242679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242680, SGI0, SGI0, SGI0, SGI242681, SGI242681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53669[] = {SGI242682, SGI242683, SGI242684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242685, SGI242685, SGI0, SGI0, SGI0, SGI0, SGI242686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53670[] = {SGI242687, SGI242688, SGI242689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242690, SGI0, SGI0, SGI0, SGI242691, SGI242691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53671[] = {SGI242692, SGI242693, SGI242694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242695, SGI0, SGI0, SGI0, SGI242696, SGI242696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242697, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53672[] = {SGI242698, SGI0, SGI242699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242700, SGI242701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242702, SGI0, SGI0, NULL};
const Segment * const SG53673[] = {SGI242703, SGI0, SGI242704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242705, SGI242706, SGI242706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53674[] = {SGI242707, SGI0, SGI242708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242709, SGI242710, SGI242710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53675[] = {SGI242711, SGI0, SGI242712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242713, SGI242714, SGI242714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53676[] = {SGI242715, SGI0, SGI242716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242717, SGI242718, SGI242718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53677[] = {SGI242719, SGI242720, SGI242721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242722, SGI0, SGI0, SGI0, SGI242723, SGI242723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53678[] = {SGI242724, SGI242725, SGI242726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242727, SGI0, SGI0, SGI0, SGI242728, SGI242728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53679[] = {SGI242729, SGI242730, SGI242731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242732, SGI242732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242733, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53680[] = {SGI242734, SGI0, SGI242735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242736, SGI242737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53681[] = {SGI242738, SGI242739, SGI242740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242741, SGI0, SGI0, SGI0, SGI242742, SGI242742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53682[] = {SGI242743, SGI0, SGI242744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242745, SGI242746, SGI242746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53683[] = {SGI242747, SGI242748, SGI242749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242750, SGI0, SGI0, SGI0, SGI242751, SGI0, SGI242752, SGI0, SGI0, SGI0, SGI0, SGI242753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53684[] = {SGI242754, SGI0, SGI242755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242756, SGI242757, SGI242757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242758, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53685[] = {SGI242759, SGI242760, SGI242761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242762, SGI242762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242763, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53686[] = {SGI242764, SGI242765, SGI242766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242767, SGI0, SGI0, SGI0, SGI242768, SGI242768, SGI0, SGI242769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53687[] = {SGI242770, SGI0, SGI242771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242772, SGI242773, SGI242773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53688[] = {SGI242774, SGI0, SGI242775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242776, SGI242777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53689[] = {SGI242778, SGI242779, SGI242780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242781, SGI0, SGI0, SGI0, SGI242782, SGI0, SGI242783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53690[] = {SGI242784, SGI0, SGI242785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242786, SGI242787, SGI242787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53691[] = {SGI242788, SGI0, SGI242789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242790, SGI242791, SGI242791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53692[] = {SGI242792, SGI242793, SGI242794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242795, SGI0, SGI0, SGI0, SGI242796, SGI0, SGI0, SGI0, SGI0, SGI242797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53693[] = {SGI0, SGI242798, SGI0, SGI242799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53694[] = {SGI0, SGI242800, SGI0, SGI242801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242802, SGI0, SGI0, SGI0, SGI242803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53695[] = {SGI0, SGI242804, SGI0, SGI242805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53696[] = {SGI242806, SGI0, SGI242807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242808, SGI242809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53697[] = {SGI242810, SGI242811, SGI242812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242813, SGI0, SGI0, SGI0, SGI242814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242815, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53698[] = {SGI0, SGI0, SGI242816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242817, SGI242818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53699[] = {SGI242819, SGI242820, SGI242821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242822, SGI0, SGI0, SGI0, SGI242823, SGI0, SGI0, SGI242824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53700[] = {SGI242825, SGI242826, SGI242827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242828, SGI0, SGI0, SGI0, SGI242829, SGI0, SGI0, SGI242830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53701[] = {SGI242831, SGI242832, SGI242833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242834, SGI0, SGI0, SGI0, SGI242835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242836, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53702[] = {SGI242837, SGI242838, SGI242839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242840, SGI0, SGI0, SGI0, SGI242841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242842, NULL};
const Segment * const SG53703[] = {SGI242843, SGI242844, SGI242845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242846, SGI0, SGI0, SGI0, SGI242847, SGI0, SGI242848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53704[] = {SGI242849, SGI242850, SGI242851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242852, SGI0, SGI0, SGI0, SGI242853, SGI242853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242854, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53705[] = {SGI242855, SGI0, SGI242856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242857, SGI242858, SGI242858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53706[] = {SGI242859, SGI0, SGI242860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242861, SGI242862, SGI242862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53707[] = {SGI242863, SGI242864, SGI242865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242866, SGI0, SGI0, SGI0, SGI242867, SGI242867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53708[] = {SGI242868, SGI0, SGI242869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53709[] = {SGI242870, SGI0, SGI242872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242875, SGI242877, SGI242877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53710[] = {SGI242871, SGI0, SGI242873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242874, SGI242876, SGI242876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53711[] = {SGI242878, SGI0, SGI242879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242880, SGI242881, SGI242881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53712[] = {SGI242882, SGI0, SGI242886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242889, SGI242891, SGI242891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53713[] = {SGI242884, SGI0, SGI242887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242890, SGI242893, SGI242893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53714[] = {SGI242883, SGI242885, SGI242888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242892, SGI242892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242894, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53715[] = {SGI242895, SGI0, SGI242896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242897, SGI242898, SGI242898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53716[] = {SGI242899, SGI0, SGI242900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242901, SGI242902, SGI242902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242904, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53717[] = {SGI242903, SGI0, SGI242905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242906, SGI242907, SGI242907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53718[] = {SGI242908, SGI0, SGI242909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242910, SGI242911, SGI242911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242912, SGI0, SGI0, NULL};
const Segment * const SG53719[] = {SGI242913, SGI0, SGI242914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242915, SGI242916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242917, SGI0, SGI0, NULL};
const Segment * const SG53720[] = {SGI242918, SGI0, SGI242919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242920, SGI242921, SGI242921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242922, SGI0, SGI0, NULL};
const Segment * const SG53721[] = {SGI242923, SGI0, SGI242924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242925, SGI242926, SGI242926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53722[] = {SGI242927, SGI0, SGI242928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242929, SGI242930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53723[] = {SGI242931, SGI242932, SGI242933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242934, SGI0, SGI0, SGI0, SGI242935, SGI242935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242936, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53724[] = {SGI242937, SGI242938, SGI242939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242940, SGI242940, SGI0, SGI0, SGI0, SGI0, SGI242941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53725[] = {SGI242942, SGI242943, SGI242944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242945, SGI242945, SGI0, SGI0, SGI0, SGI0, SGI242946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53726[] = {SGI242947, SGI242948, SGI242949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242950, SGI242950, SGI0, SGI0, SGI0, SGI0, SGI242951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53727[] = {SGI242952, SGI242953, SGI242954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242955, SGI0, SGI0, SGI0, SGI242956, SGI242957, SGI0, SGI0, SGI0, SGI0, SGI242958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53728[] = {SGI242959, SGI0, SGI242960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242961, SGI242962, SGI242962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53729[] = {SGI242963, SGI0, SGI242964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242965, SGI242966, SGI242966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53730[] = {SGI242967, SGI0, SGI242968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242969, SGI242970, SGI242970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53731[] = {SGI242971, SGI0, SGI242972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242973, SGI242974, SGI242974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53732[] = {SGI242975, SGI0, SGI242976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242977, SGI242978, SGI242978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53733[] = {SGI242979, SGI0, SGI242980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242981, SGI242982, SGI242982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53734[] = {SGI242983, SGI242984, SGI242985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242986, SGI242986, SGI0, SGI0, SGI0, SGI0, SGI242987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53735[] = {SGI242988, SGI0, SGI242989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242990, SGI242991, SGI242991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53736[] = {SGI242992, SGI0, SGI242993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242994, SGI242995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242996, SGI0, SGI0, NULL};
const Segment * const SG53737[] = {SGI242997, SGI0, SGI242998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI242999, SGI243000, SGI243000, SGI0, SGI0, SGI0, SGI0, SGI243001, SGI0, NULL};
const Segment * const SG53738[] = {SGI243002, SGI243003, SGI243004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243005, SGI0, SGI0, SGI0, SGI243006, SGI243006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243007, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53739[] = {SGI243008, SGI0, SGI243009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243010, SGI243011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53740[] = {SGI243012, SGI0, SGI243013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243014, SGI243015, SGI243015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243016, SGI0, SGI0, NULL};
const Segment * const SG53741[] = {SGI243017, SGI243018, SGI243019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243020, SGI0, SGI0, SGI0, SGI243021, SGI243021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53742[] = {SGI243022, SGI0, SGI243023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243027, SGI243028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53743[] = {SGI243024, SGI243025, SGI243026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243029, SGI243029, SGI0, SGI0, SGI0, SGI0, SGI243030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53744[] = {SGI243031, SGI0, SGI243032, SGI0, SGI243033, SGI243034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53745[] = {SGI243035, SGI0, SGI243036, SGI0, SGI243037, SGI243038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53746[] = {SGI243040, SGI0, SGI243042, SGI0, SGI243044, SGI243045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53747[] = {SGI243039, SGI243041, SGI243043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243046, SGI243046, SGI0, SGI0, SGI0, SGI0, SGI243047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53748[] = {SGI243048, SGI0, SGI243049, SGI0, SGI243050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53749[] = {SGI243051, SGI0, SGI243052, SGI0, SGI243053, SGI243054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53750[] = {SGI243055, SGI0, SGI243056, SGI0, SGI243057, SGI243058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53751[] = {SGI243059, SGI0, SGI243060, SGI0, SGI243061, SGI243062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53752[] = {SGI243063, SGI0, SGI243064, SGI0, SGI243065, SGI243066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53753[] = {SGI243067, SGI0, SGI243068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53754[] = {SGI243069, SGI0, SGI243070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243071, SGI243072, SGI243072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53755[] = {SGI243073, SGI0, SGI243074, SGI0, SGI0, SGI243075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53756[] = {SGI243076, SGI0, SGI243077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53757[] = {SGI243080, SGI0, SGI243081, SGI0, SGI0, SGI243084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53758[] = {SGI243078, SGI0, SGI243079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243082, SGI243083, SGI243083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53759[] = {SGI243085, SGI0, SGI243086, SGI0, SGI243087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53760[] = {SGI243088, SGI0, SGI243089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243090, SGI243091, SGI243091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53761[] = {SGI243092, SGI0, SGI243093, SGI0, SGI0, SGI243094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53762[] = {SGI243095, SGI0, SGI243097, SGI0, SGI243098, SGI243100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53763[] = {SGI243096, SGI0, SGI243099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243101, SGI243102, SGI243102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53764[] = {SGI243103, SGI0, SGI243104, SGI0, SGI243105, SGI243106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53765[] = {SGI243107, SGI0, SGI243108, SGI0, SGI243109, SGI243110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53766[] = {SGI243111, SGI0, SGI243112, SGI0, SGI243113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53767[] = {SGI243114, SGI0, SGI243115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53768[] = {SGI243116, SGI0, SGI243117, SGI0, SGI0, SGI243118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53769[] = {SGI243119, SGI0, SGI243120, SGI0, SGI243121, SGI243122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53770[] = {SGI243123, SGI0, SGI243124, SGI0, SGI243125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53771[] = {SGI243126, SGI0, SGI243127, SGI0, SGI0, SGI243128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53772[] = {SGI243129, SGI0, SGI243130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243131, SGI243132, SGI243132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53773[] = {SGI243133, SGI0, SGI243134, SGI0, SGI243135, SGI243136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53774[] = {SGI243137, SGI0, SGI243138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243139, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53775[] = {SGI243140, SGI0, SGI243141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53776[] = {SGI243142, SGI0, SGI243143, SGI0, SGI243144, SGI243145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53777[] = {SGI243146, SGI0, SGI243147, SGI0, SGI0, SGI243148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53778[] = {SGI243150, SGI0, SGI243152, SGI0, SGI0, SGI243154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53779[] = {SGI243149, SGI243151, SGI243153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243155, SGI243155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243156, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53780[] = {SGI243157, SGI0, SGI243158, SGI0, SGI243159, SGI243160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53781[] = {SGI243161, SGI0, SGI243162, SGI0, SGI0, SGI243163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53782[] = {SGI243164, SGI0, SGI243165, SGI0, SGI0, SGI243166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53783[] = {SGI243169, SGI0, SGI243170, SGI0, SGI243171, SGI243173, SGI243175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53784[] = {SGI243167, SGI0, SGI243168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243172, SGI243174, SGI243174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243176, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53785[] = {SGI243177, SGI0, SGI243178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53786[] = {SGI243179, SGI0, SGI243180, SGI0, SGI243181, SGI243182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53787[] = {SGI243183, SGI0, SGI243184, SGI0, SGI243185, SGI243186, SGI243187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53788[] = {SGI243188, SGI0, SGI243189, SGI0, SGI0, SGI243190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53789[] = {SGI243191, SGI0, SGI243192, SGI0, SGI243193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53790[] = {SGI243194, SGI0, SGI243195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53791[] = {SGI243196, SGI0, SGI243197, SGI0, SGI243198, SGI243199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53792[] = {SGI243200, SGI0, SGI243201, SGI0, SGI0, SGI243202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53793[] = {SGI243203, SGI0, SGI243204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243205, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53794[] = {SGI243206, SGI0, SGI243207, SGI0, SGI243208, SGI243209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53795[] = {SGI243210, SGI0, SGI243211, SGI0, SGI243212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53796[] = {SGI243213, SGI0, SGI0, SGI0, SGI243214, SGI243215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53797[] = {SGI243216, SGI0, SGI243217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53798[] = {SGI243218, SGI0, SGI243219, SGI0, SGI243220, SGI243221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53799[] = {SGI243222, SGI0, SGI243223, SGI0, SGI243224, SGI243225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53800[] = {SGI243226, SGI0, SGI243227, SGI0, SGI243228, SGI243229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53801[] = {SGI243230, SGI243231, SGI243232, SGI0, SGI243233, SGI243234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53802[] = {SGI243235, SGI243236, SGI243237, SGI0, SGI243238, SGI243239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53803[] = {SGI243240, SGI0, SGI0, SGI0, SGI243241, SGI243242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53804[] = {SGI243243, SGI0, SGI243244, SGI0, SGI243245, SGI243246, SGI243247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53805[] = {SGI243248, SGI0, SGI243249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243250, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53806[] = {SGI243251, SGI0, SGI243252, SGI0, SGI243253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53807[] = {SGI243254, SGI0, SGI243255, SGI0, SGI243256, SGI243257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53808[] = {SGI243258, SGI0, SGI243259, SGI0, SGI0, SGI243260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53809[] = {SGI243261, SGI0, SGI243262, SGI0, SGI0, SGI243263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53810[] = {SGI243264, SGI0, SGI243265, SGI0, SGI243266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53811[] = {SGI243267, SGI0, SGI243268, SGI0, SGI243269, SGI243270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53812[] = {SGI243271, SGI0, SGI243272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53813[] = {SGI243273, SGI0, SGI0, SGI0, SGI243274, SGI243275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53814[] = {SGI243276, SGI0, SGI0, SGI0, SGI243277, SGI243278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53815[] = {SGI243279, SGI0, SGI243280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53816[] = {SGI243281, SGI0, SGI243282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53817[] = {SGI243283, SGI0, SGI243284, SGI0, SGI243285, SGI243286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53818[] = {SGI243287, SGI0, SGI243288, SGI0, SGI243289, SGI243290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53819[] = {SGI243291, SGI0, SGI243292, SGI0, SGI243293, SGI243294, SGI243295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53820[] = {SGI243296, SGI0, SGI243297, SGI0, SGI243298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53821[] = {SGI243299, SGI0, SGI243300, SGI0, SGI243301, SGI243302, SGI243303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53822[] = {SGI243304, SGI0, SGI243305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243306, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53823[] = {SGI243307, SGI0, SGI243309, SGI0, SGI243311, SGI243312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53824[] = {SGI243308, SGI0, SGI243310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243313, SGI243314, SGI243314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53825[] = {SGI243315, SGI0, SGI243316, SGI0, SGI243317, SGI243318, SGI243319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53826[] = {SGI243320, SGI0, SGI243321, SGI0, SGI243322, SGI243323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53827[] = {SGI243324, SGI0, SGI243325, SGI0, SGI0, SGI243326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53828[] = {SGI243327, SGI0, SGI243328, SGI0, SGI0, SGI243329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53829[] = {SGI243330, SGI0, SGI243331, SGI0, SGI0, SGI243332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53830[] = {SGI243333, SGI0, SGI243334, SGI0, SGI0, SGI243335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53831[] = {SGI243336, SGI243337, SGI243338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243339, SGI0, SGI0, SGI0, SGI243340, SGI0, SGI0, SGI243341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53832[] = {SGI243342, SGI0, SGI243343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53833[] = {SGI243344, SGI0, SGI243345, SGI0, SGI0, SGI243346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53834[] = {SGI243347, SGI243348, SGI243349, SGI0, SGI243350, SGI243351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53835[] = {SGI243352, SGI0, SGI243353, SGI0, SGI243354, SGI243355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53836[] = {SGI243356, SGI243357, SGI243358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243361, SGI243361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243363, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53837[] = {SGI243359, SGI0, SGI243360, SGI0, SGI243362, SGI243364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53838[] = {SGI243365, SGI0, SGI243366, SGI0, SGI243367, SGI243368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53839[] = {SGI243369, SGI0, SGI243370, SGI0, SGI243371, SGI243372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53840[] = {SGI243373, SGI243374, SGI243375, SGI0, SGI243376, SGI243377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53841[] = {SGI243378, SGI0, SGI243379, SGI0, SGI0, SGI243380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53842[] = {SGI243381, SGI243382, SGI243383, SGI0, SGI243384, SGI243385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53843[] = {SGI243386, SGI243387, SGI243388, SGI0, SGI243389, SGI243390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53844[] = {SGI243391, SGI0, SGI243392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243393, SGI243394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53845[] = {SGI243395, SGI0, SGI243396, SGI0, SGI243397, SGI243398, SGI243399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53846[] = {SGI243400, SGI0, SGI243401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243402, SGI243403, SGI243403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53847[] = {SGI243404, SGI0, SGI243405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243406, SGI243407, SGI243407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53848[] = {SGI243408, SGI243409, SGI243410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243411, SGI243411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53849[] = {SGI243412, SGI0, SGI243413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243414, SGI243415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53850[] = {SGI243416, SGI0, SGI243417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243418, SGI243419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53851[] = {SGI243420, SGI243421, SGI243422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243423, SGI0, SGI0, SGI0, SGI243424, SGI243424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53852[] = {SGI243425, SGI243426, SGI243427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243428, SGI243428, SGI0, SGI0, SGI0, SGI0, SGI243431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53853[] = {SGI243429, SGI0, SGI243430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243432, SGI243433, SGI243433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243434, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53854[] = {SGI243435, SGI0, SGI243436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243437, SGI243438, SGI243438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53855[] = {SGI243439, SGI0, SGI243440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243441, SGI243442, SGI243442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53856[] = {SGI243443, SGI0, SGI243444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243445, SGI243446, SGI243446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53857[] = {SGI243447, SGI0, SGI243448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243449, SGI243450, SGI243450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53858[] = {SGI243451, SGI0, SGI243452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243453, SGI243454, SGI243454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53859[] = {SGI243455, SGI0, SGI243456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243457, SGI243458, SGI243458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53860[] = {SGI243459, SGI243460, SGI243461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243462, SGI0, SGI0, SGI0, SGI243463, SGI243463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53861[] = {SGI243464, SGI0, SGI243465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243466, SGI0, SGI0, SGI0, SGI243467, SGI243467, SGI0, SGI0, SGI0, SGI0, SGI243468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53862[] = {SGI243469, SGI243470, SGI243471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243472, SGI0, SGI0, SGI0, SGI243473, SGI243473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243475, SGI0, NULL};
const Segment * const SG53863[] = {SGI243476, SGI0, SGI243477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243478, SGI243479, SGI243479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53864[] = {SGI243480, SGI0, SGI243481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243482, SGI243483, SGI243483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53865[] = {SGI243484, SGI243485, SGI243486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243487, SGI0, SGI0, SGI0, SGI243488, SGI243488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243489, SGI243490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53866[] = {SGI243491, SGI0, SGI243492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243493, SGI243494, SGI243494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53867[] = {SGI243495, SGI243496, SGI243497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243498, SGI0, SGI0, SGI0, SGI243499, SGI243499, SGI243500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53868[] = {SGI243501, SGI243502, SGI243503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243504, SGI0, SGI0, SGI0, SGI243505, SGI243505, SGI0, SGI0, SGI0, SGI0, SGI243507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53869[] = {SGI0, SGI243506, SGI243508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243509, SGI0, SGI243510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53870[] = {SGI243511, SGI243512, SGI243513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53871[] = {SGI243514, SGI0, SGI243515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243516, SGI243517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53872[] = {SGI243518, SGI243519, SGI243520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243521, SGI0, SGI0, SGI0, SGI243522, SGI243522, SGI0, SGI243523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53873[] = {SGI243524, SGI0, SGI243525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243526, SGI243527, SGI243527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53874[] = {SGI243528, SGI243529, SGI243530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243531, SGI243531, SGI0, SGI0, SGI0, SGI0, SGI243532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53875[] = {SGI243533, SGI243534, SGI243536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243539, SGI0, SGI0, SGI0, SGI243541, SGI243541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243543, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53876[] = {SGI243535, SGI243537, SGI243538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243540, SGI0, SGI0, SGI0, SGI243542, SGI243542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53877[] = {SGI243544, SGI0, SGI243545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243546, SGI243547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243548, SGI0, SGI0, NULL};
const Segment * const SG53878[] = {SGI243549, SGI243550, SGI243551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243552, SGI243552, SGI0, SGI0, SGI0, SGI0, SGI243553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53879[] = {SGI243554, SGI0, SGI243555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53880[] = {SGI243556, SGI243557, SGI243558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243559, SGI0, SGI0, SGI0, SGI243560, SGI243560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53881[] = {SGI243561, SGI243562, SGI243563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243564, SGI243564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243565, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53882[] = {SGI243566, SGI0, SGI243567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243568, SGI243569, SGI243569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53883[] = {SGI243570, SGI0, SGI243571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243572, SGI243573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53884[] = {SGI243574, SGI0, SGI243575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243576, SGI243577, SGI243577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53885[] = {SGI243578, SGI0, SGI243579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243580, SGI243581, SGI243581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53886[] = {SGI243582, SGI0, SGI243583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243584, SGI243585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53887[] = {SGI243586, SGI0, SGI243587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243588, SGI243589, SGI243589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53888[] = {SGI243590, SGI243591, SGI243592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243593, SGI0, SGI0, SGI0, SGI243594, SGI0, SGI243595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53889[] = {SGI243596, SGI0, SGI243597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243598, SGI243599, SGI243599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53890[] = {SGI243600, SGI243601, SGI243602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243603, SGI0, SGI0, SGI0, SGI243604, SGI243604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53891[] = {SGI243605, SGI0, SGI243606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243607, SGI243608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53892[] = {SGI243609, SGI243610, SGI243611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243612, SGI0, SGI0, SGI0, SGI243613, SGI243613, SGI0, SGI0, SGI0, SGI243614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53893[] = {SGI243615, SGI243616, SGI243617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243618, SGI243618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243619, NULL};
const Segment * const SG53894[] = {SGI243620, SGI243621, SGI243622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243623, SGI0, SGI0, SGI0, SGI243624, SGI243624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53895[] = {SGI243625, SGI243626, SGI243627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243628, SGI243628, SGI0, SGI0, SGI0, SGI0, SGI243629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53896[] = {SGI243630, SGI0, SGI243631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243632, SGI243633, SGI243633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243634, SGI0, SGI0, NULL};
const Segment * const SG53897[] = {SGI243635, SGI243636, SGI243637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243638, SGI243638, SGI0, SGI0, SGI0, SGI0, SGI243639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53898[] = {SGI243640, SGI243641, SGI243642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243643, SGI0, SGI0, SGI0, SGI243644, SGI243644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53899[] = {SGI243645, SGI243646, SGI243647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243648, SGI243648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243649, NULL};
const Segment * const SG53900[] = {SGI243650, SGI0, SGI243651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243652, SGI243653, SGI243653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53901[] = {SGI243654, SGI243655, SGI243656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243657, SGI0, SGI0, SGI0, SGI243658, SGI243659, SGI0, SGI0, SGI0, SGI0, SGI243660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53902[] = {SGI243661, SGI243662, SGI243663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243664, SGI243664, SGI0, SGI0, SGI0, SGI0, SGI243665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53903[] = {SGI243666, SGI243667, SGI243668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243669, SGI243669, SGI0, SGI0, SGI0, SGI0, SGI243670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53904[] = {SGI243671, SGI0, SGI243672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243673, SGI243674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53905[] = {SGI243675, SGI243676, SGI243677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243678, SGI243678, SGI0, SGI0, SGI0, SGI0, SGI243679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53906[] = {SGI243680, SGI0, SGI243682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243684, SGI243686, SGI243686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53907[] = {SGI243681, SGI0, SGI243683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243685, SGI0, SGI0, SGI0, SGI243687, SGI243687, SGI0, SGI0, SGI243688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53908[] = {SGI243689, SGI0, SGI243690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243694, SGI243695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53909[] = {SGI243691, SGI243692, SGI243693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243696, SGI0, SGI243697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53910[] = {SGI243698, SGI0, SGI243699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243700, SGI243701, SGI243701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53911[] = {SGI243702, SGI243703, SGI243704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243705, SGI0, SGI0, SGI0, SGI243706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243707, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53912[] = {SGI243708, SGI0, SGI243709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243710, SGI243711, SGI243711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53913[] = {SGI243712, SGI243713, SGI243714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243715, SGI0, SGI0, SGI0, SGI243716, SGI243716, SGI0, SGI243717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53914[] = {SGI243718, SGI243719, SGI243720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243721, SGI0, SGI0, SGI0, SGI243722, SGI243723, SGI0, SGI0, SGI243724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53915[] = {SGI243725, SGI0, SGI243726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243727, SGI243728, SGI243728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53916[] = {SGI243729, SGI243730, SGI243731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243732, SGI0, SGI0, SGI0, SGI243733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243734, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53917[] = {SGI243735, SGI0, SGI243736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243737, SGI0, SGI0, SGI0, SGI243739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53918[] = {SGI243738, SGI0, SGI243740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243741, SGI243742, SGI243742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53919[] = {SGI243743, SGI0, SGI243744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243745, SGI243746, SGI243746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53920[] = {SGI243747, SGI243748, SGI243749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243750, SGI243750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243751, NULL};
const Segment * const SG53921[] = {SGI243752, SGI0, SGI243753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243754, SGI243755, SGI243755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53922[] = {SGI243756, SGI0, SGI243757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243761, SGI243762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53923[] = {SGI243758, SGI243759, SGI243760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243763, SGI0, SGI0, SGI0, SGI243765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243767, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53924[] = {SGI243764, SGI0, SGI243766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243768, SGI0, SGI0, SGI0, SGI243769, SGI243769, SGI0, SGI0, SGI0, SGI0, SGI243770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53925[] = {SGI243771, SGI243772, SGI243773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243774, SGI0, SGI0, SGI0, SGI243775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243776, SGI243777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53926[] = {SGI243778, SGI243779, SGI243780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243781, SGI0, SGI0, SGI0, SGI243782, SGI243782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243783, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53927[] = {SGI243784, SGI0, SGI243785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53928[] = {SGI243786, SGI0, SGI243787, SGI0, SGI243788, SGI243789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53929[] = {SGI243790, SGI0, SGI243791, SGI0, SGI243792, SGI243793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53930[] = {SGI243794, SGI0, SGI243795, SGI0, SGI243796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53931[] = {SGI243797, SGI0, SGI243798, SGI0, SGI243799, SGI243800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53932[] = {SGI243801, SGI0, SGI243802, SGI0, SGI243803, SGI243804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53933[] = {SGI243805, SGI243806, SGI243807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243808, SGI0, SGI0, SGI0, SGI243811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243814, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53934[] = {SGI243809, SGI0, SGI243810, SGI0, SGI243812, SGI243813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53935[] = {SGI243815, SGI0, SGI0, SGI0, SGI243816, SGI243817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53936[] = {SGI243818, SGI0, SGI243819, SGI0, SGI243820, SGI243821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53937[] = {SGI243822, SGI0, SGI243823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53938[] = {SGI243824, SGI0, SGI243825, SGI0, SGI243826, SGI243827, SGI243828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53939[] = {SGI243829, SGI243830, SGI243831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243832, SGI0, SGI0, SGI0, SGI243833, SGI0, SGI243834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53940[] = {SGI243835, SGI0, SGI243836, SGI0, SGI0, SGI243837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53941[] = {SGI243838, SGI0, SGI243839, SGI0, SGI243840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53942[] = {SGI243841, SGI0, SGI243842, SGI0, SGI243843, SGI243844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53943[] = {SGI243845, SGI0, SGI243846, SGI0, SGI0, SGI243847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53944[] = {SGI243848, SGI0, SGI243849, SGI0, SGI0, SGI243850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53945[] = {SGI243851, SGI0, SGI243852, SGI0, SGI243853, SGI243854, SGI243855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53946[] = {SGI243856, SGI0, SGI243857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53947[] = {SGI243858, SGI243859, SGI243860, SGI0, SGI243861, SGI243862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53948[] = {SGI243863, SGI0, SGI243864, SGI0, SGI243865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53949[] = {SGI243866, SGI0, SGI243867, SGI0, SGI243868, SGI243869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53950[] = {SGI243870, SGI0, SGI0, SGI0, SGI243871, SGI243872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53951[] = {SGI243873, SGI0, SGI243874, SGI0, SGI243875, SGI243876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53952[] = {SGI243877, SGI0, SGI243878, SGI0, SGI0, SGI243879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53953[] = {SGI243880, SGI243881, SGI243882, SGI243883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243884, SGI0, SGI0, SGI0, SGI243886, SGI243888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53954[] = {SGI243885, SGI0, SGI243887, SGI0, SGI0, SGI243889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53955[] = {SGI243890, SGI0, SGI243891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53956[] = {SGI243892, SGI0, SGI243893, SGI0, SGI243894, SGI243895, SGI243896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53957[] = {SGI243897, SGI0, SGI243898, SGI0, SGI243899, SGI243900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53958[] = {SGI243901, SGI0, SGI243902, SGI0, SGI243903, SGI243904, SGI243905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53959[] = {SGI243906, SGI0, SGI243907, SGI0, SGI243908, SGI243909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53960[] = {SGI243910, SGI0, SGI243911, SGI0, SGI243912, SGI243913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53961[] = {SGI243914, SGI0, SGI243915, SGI0, SGI243916, SGI243917, SGI243918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53962[] = {SGI243919, SGI243920, SGI243921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243924, SGI0, SGI0, SGI0, SGI243926, SGI243926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243929, NULL};
const Segment * const SG53963[] = {SGI243922, SGI243923, SGI243925, SGI0, SGI243927, SGI243928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53964[] = {SGI243930, SGI0, SGI243931, SGI0, SGI0, SGI243932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53965[] = {SGI243933, SGI0, SGI0, SGI0, SGI243935, SGI243936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53966[] = {SGI243934, SGI0, SGI243937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243938, SGI243939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243940, SGI0, SGI0, NULL};
const Segment * const SG53967[] = {SGI243941, SGI0, SGI243942, SGI0, SGI0, SGI243943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53968[] = {SGI243944, SGI0, SGI243945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53969[] = {SGI243946, SGI0, SGI243947, SGI0, SGI243948, SGI243949, SGI243950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53970[] = {SGI243951, SGI0, SGI243952, SGI0, SGI243953, SGI243955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53971[] = {SGI243954, SGI243956, SGI243957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243958, SGI0, SGI0, SGI0, SGI243959, SGI243959, SGI243960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53972[] = {SGI243961, SGI0, SGI0, SGI0, SGI243962, SGI243963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53973[] = {SGI243964, SGI0, SGI243965, SGI0, SGI243966, SGI243967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53974[] = {SGI243968, SGI0, SGI243969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243970, SGI243971, SGI243971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53975[] = {SGI243972, SGI0, SGI243973, SGI0, SGI243974, SGI243975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53976[] = {SGI243976, SGI0, SGI243977, SGI0, SGI243978, SGI243979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53977[] = {SGI243980, SGI0, SGI243981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243982, SGI243984, SGI243984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53978[] = {SGI243983, SGI0, SGI243985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53979[] = {SGI243987, SGI0, SGI243989, SGI0, SGI243991, SGI243992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53980[] = {SGI243986, SGI243988, SGI243990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI243993, SGI243993, SGI0, SGI0, SGI243994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53981[] = {SGI243995, SGI0, SGI243996, SGI0, SGI243997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53982[] = {SGI244001, SGI0, SGI244002, SGI0, SGI244004, SGI244006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53983[] = {SGI243998, SGI243999, SGI244000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244003, SGI0, SGI0, SGI0, SGI244005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244007, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53984[] = {SGI244008, SGI0, SGI244009, SGI0, SGI244010, SGI244011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53985[] = {SGI244012, SGI0, SGI244013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244014, SGI244015, SGI244015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53986[] = {SGI244016, SGI0, SGI244017, SGI0, SGI244018, SGI244019, SGI244020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53987[] = {SGI244021, SGI0, SGI244022, SGI0, SGI244023, SGI244024, SGI244025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53988[] = {SGI244026, SGI0, SGI244027, SGI0, SGI244028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53989[] = {SGI244029, SGI0, SGI244030, SGI0, SGI0, SGI244031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53990[] = {SGI244034, SGI0, SGI244035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244038, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53991[] = {SGI244032, SGI0, SGI244033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244036, SGI244037, SGI244037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53992[] = {SGI244041, SGI0, SGI244043, SGI0, SGI244044, SGI244045, SGI244047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53993[] = {SGI244039, SGI244040, SGI244042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244046, SGI0, SGI0, SGI0, SGI244048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244049, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53994[] = {SGI244050, SGI0, SGI244051, SGI0, SGI0, SGI244052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53995[] = {SGI244053, SGI0, SGI244054, SGI0, SGI0, SGI244056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53996[] = {SGI244055, SGI0, SGI244057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244058, SGI244059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244060, SGI0, SGI0, NULL};
const Segment * const SG53997[] = {SGI244061, SGI0, SGI244062, SGI0, SGI0, SGI244063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53998[] = {SGI244064, SGI0, SGI244065, SGI0, SGI0, SGI244066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG53999[] = {SGI244067, SGI244068, SGI244069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244070, SGI0, SGI0, SGI0, SGI244071, SGI244071, SGI0, SGI0, SGI244072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54000[] = {SGI244073, SGI0, SGI244074, SGI0, SGI244075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54001[] = {SGI244076, SGI0, SGI244077, SGI0, SGI244078, SGI244079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54002[] = {SGI244080, SGI0, SGI244081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54003[] = {SGI244082, SGI0, SGI244083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244084, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54004[] = {SGI244085, SGI244086, SGI244087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244088, SGI0, SGI0, SGI0, SGI244089, SGI244089, SGI0, SGI244092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54005[] = {SGI244090, SGI0, SGI244091, SGI0, SGI244093, SGI244094, SGI244095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54006[] = {SGI244098, SGI0, SGI244100, SGI0, SGI244101, SGI244102, SGI244103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54007[] = {SGI244096, SGI244097, SGI244099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244104, SGI0, SGI0, SGI0, SGI244105, SGI244105, SGI0, SGI244106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54008[] = {SGI244107, SGI244108, SGI244109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244110, SGI0, SGI0, SGI0, SGI244112, SGI244112, SGI244115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54009[] = {SGI244111, SGI244113, SGI244114, SGI0, SGI244116, SGI244117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54010[] = {SGI244118, SGI0, SGI244119, SGI0, SGI244120, SGI244121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54011[] = {SGI244122, SGI0, SGI244123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244124, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54012[] = {SGI244125, SGI0, SGI244126, SGI0, SGI244127, SGI244128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54013[] = {SGI244129, SGI0, SGI244130, SGI0, SGI244131, SGI244132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54014[] = {SGI244133, SGI0, SGI244134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54015[] = {SGI244135, SGI0, SGI244136, SGI0, SGI0, SGI244137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54016[] = {SGI244141, SGI0, SGI244142, SGI0, SGI244143, SGI244145, SGI244146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54017[] = {SGI244138, SGI244139, SGI244140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244144, SGI0, SGI0, SGI0, SGI244147, SGI0, SGI0, SGI244148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54018[] = {SGI244149, SGI0, SGI244150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244151, SGI244152, SGI244152, SGI0, SGI0, SGI0, SGI0, SGI244153, SGI0, NULL};
const Segment * const SG54019[] = {SGI244154, SGI0, SGI244155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244156, SGI244157, SGI244157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54020[] = {SGI244158, SGI244159, SGI244160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244161, SGI0, SGI0, SGI0, SGI244162, SGI244162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54021[] = {SGI244163, SGI0, SGI244164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244165, SGI244166, SGI244166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244167, SGI0, SGI0, NULL};
const Segment * const SG54022[] = {SGI244168, SGI0, SGI244169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244170, SGI244171, SGI244171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54023[] = {SGI244172, SGI244173, SGI244174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244175, SGI0, SGI0, SGI0, SGI244176, SGI0, SGI0, SGI244177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54024[] = {SGI244178, SGI244179, SGI244180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244181, SGI0, SGI0, SGI0, SGI244182, SGI0, SGI0, SGI244183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54025[] = {SGI244184, SGI244185, SGI244186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244187, SGI244187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244188, NULL};
const Segment * const SG54026[] = {SGI244189, SGI244190, SGI244191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244192, SGI244192, SGI0, SGI0, SGI244193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54027[] = {SGI244194, SGI0, SGI244195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244196, SGI244197, SGI244197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54028[] = {SGI244198, SGI0, SGI244199, SGI0, SGI244200, SGI244201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54029[] = {SGI244202, SGI0, SGI244203, SGI0, SGI244205, SGI244207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54030[] = {SGI244204, SGI0, SGI244206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244208, SGI244209, SGI244209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54031[] = {SGI244210, SGI0, SGI244211, SGI0, SGI244212, SGI244213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54032[] = {SGI244214, SGI0, SGI244215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244219, SGI244221, SGI244221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244224, SGI0, SGI0, NULL};
const Segment * const SG54033[] = {SGI244220, SGI0, SGI244222, SGI0, SGI244223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54034[] = {SGI244216, SGI244217, SGI244218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244225, SGI244225, SGI0, SGI0, SGI244226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54035[] = {SGI244227, SGI0, SGI244228, SGI0, SGI244230, SGI244232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54036[] = {SGI244229, SGI244231, SGI244233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244234, SGI244234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54037[] = {SGI244235, SGI0, SGI244236, SGI0, SGI244237, SGI244238, SGI244239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54038[] = {SGI244240, SGI0, SGI244242, SGI0, SGI244244, SGI244246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54039[] = {SGI244241, SGI244243, SGI244245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244247, SGI0, SGI0, SGI0, SGI244248, SGI0, SGI0, SGI0, SGI0, SGI244249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54040[] = {SGI244250, SGI0, SGI244251, SGI0, SGI0, SGI244252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54041[] = {SGI244253, SGI0, SGI244254, SGI0, SGI244255, SGI244256, SGI244257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54042[] = {SGI244258, SGI0, SGI244259, SGI0, SGI0, SGI244260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54043[] = {SGI244261, SGI0, SGI244262, SGI0, SGI244263, SGI244264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54044[] = {SGI244265, SGI0, SGI244266, SGI0, SGI244267, SGI244268, SGI244269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54045[] = {SGI244273, SGI0, SGI244275, SGI0, SGI244276, SGI244278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54046[] = {SGI244270, SGI244271, SGI244272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244274, SGI0, SGI0, SGI0, SGI244277, SGI0, SGI244279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54047[] = {SGI244280, SGI244281, SGI244282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244284, SGI0, SGI0, SGI0, SGI244286, SGI244288, SGI0, SGI0, SGI0, SGI0, SGI244292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54048[] = {SGI244283, SGI0, SGI244285, SGI0, SGI244287, SGI244289, SGI244291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54049[] = {SGI244290, SGI244293, SGI244294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244295, SGI244295, SGI0, SGI0, SGI244296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54050[] = {SGI244297, SGI0, SGI244298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54051[] = {SGI244299, SGI0, SGI244300, SGI0, SGI244301, SGI244302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54052[] = {SGI244304, SGI0, SGI244307, SGI0, SGI244308, SGI244309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54053[] = {SGI244303, SGI244305, SGI244306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244310, SGI244310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244311, NULL};
const Segment * const SG54054[] = {SGI244312, SGI0, SGI244313, SGI0, SGI244314, SGI244315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54055[] = {SGI244316, SGI0, SGI244317, SGI0, SGI244318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54056[] = {SGI244323, SGI244324, SGI244325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244326, SGI244326, SGI0, SGI0, SGI244327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54057[] = {SGI244328, SGI244329, SGI244330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244331, SGI244331, SGI0, SGI0, SGI0, SGI0, SGI244332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54058[] = {SGI244333, SGI244334, SGI244335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244336, SGI244336, SGI0, SGI0, SGI0, SGI0, SGI244337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54059[] = {SGI244338, SGI244339, SGI244340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244341, SGI244341, SGI0, SGI0, SGI0, SGI0, SGI244342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54060[] = {SGI244343, SGI244344, SGI244345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244346, SGI244346, SGI0, SGI0, SGI0, SGI0, SGI244347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54061[] = {SGI244348, SGI244349, SGI244350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244351, SGI0, SGI0, SGI0, SGI244352, SGI244352, SGI0, SGI244353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54062[] = {SGI244354, SGI244355, SGI244356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244357, SGI0, SGI0, SGI0, SGI244358, SGI244358, SGI0, SGI244359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54063[] = {SGI244360, SGI0, SGI244361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244364, SGI244365, SGI244365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54064[] = {SGI244362, SGI0, SGI244363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244366, SGI244367, SGI244367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54065[] = {SGI244368, SGI0, SGI244369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244370, SGI244371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54066[] = {SGI244372, SGI0, SGI244373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244374, SGI244375, SGI244375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54067[] = {SGI244376, SGI244377, SGI244378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244379, SGI0, SGI0, SGI0, SGI244380, SGI244380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244381, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54068[] = {SGI244382, SGI244383, SGI244384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244385, SGI244385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244386, NULL};
const Segment * const SG54069[] = {SGI244387, SGI0, SGI244388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244389, SGI244390, SGI244390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54070[] = {SGI244391, SGI244392, SGI244393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244394, SGI244394, SGI0, SGI0, SGI0, SGI0, SGI244395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54071[] = {SGI244396, SGI244397, SGI244398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244399, SGI244399, SGI0, SGI0, SGI0, SGI0, SGI244400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54072[] = {SGI244401, SGI0, SGI244402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244403, SGI244404, SGI244404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244405, SGI0, SGI0, NULL};
const Segment * const SG54073[] = {SGI244406, SGI244407, SGI244408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244409, SGI244409, SGI0, SGI0, SGI0, SGI0, SGI244410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54074[] = {SGI244411, SGI244412, SGI244413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244414, SGI0, SGI0, SGI0, SGI244415, SGI244415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244416, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54075[] = {SGI244417, SGI244418, SGI244419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244420, SGI244420, SGI0, SGI0, SGI0, SGI0, SGI244421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54076[] = {SGI244422, SGI244423, SGI244424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244425, SGI244425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244426, NULL};
const Segment * const SG54077[] = {SGI244319, SGI0, SGI244320, SGI0, SGI244321, SGI244322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54078[] = {SGI244427, SGI0, SGI244428, SGI0, SGI244429, SGI244430, SGI244431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54079[] = {SGI244432, SGI0, SGI244433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54080[] = {SGI244434, SGI0, SGI244435, SGI0, SGI0, SGI244436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54081[] = {SGI244437, SGI0, SGI244438, SGI0, SGI244439, SGI244440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54082[] = {SGI244441, SGI0, SGI244442, SGI0, SGI244443, SGI244444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54083[] = {SGI244445, SGI0, SGI244446, SGI0, SGI244447, SGI244448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54084[] = {SGI244449, SGI0, SGI244450, SGI0, SGI244451, SGI244452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54085[] = {SGI244453, SGI0, SGI244454, SGI0, SGI244455, SGI244456, SGI244457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54086[] = {SGI244458, SGI0, SGI244459, SGI0, SGI0, SGI244460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54087[] = {SGI244461, SGI0, SGI244462, SGI0, SGI244463, SGI244464, SGI244465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54088[] = {SGI244466, SGI0, SGI244467, SGI0, SGI244468, SGI244469, SGI244470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54089[] = {SGI244471, SGI0, SGI244472, SGI0, SGI244473, SGI244474, SGI244475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54090[] = {SGI244476, SGI0, SGI244477, SGI0, SGI244478, SGI244479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54091[] = {SGI244480, SGI0, SGI244481, SGI0, SGI244482, SGI244483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54092[] = {SGI244484, SGI0, SGI244485, SGI0, SGI0, SGI244486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54093[] = {SGI244487, SGI0, SGI244488, SGI0, SGI0, SGI244489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54094[] = {SGI244490, SGI0, SGI244491, SGI0, SGI244492, SGI244493, SGI244494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54095[] = {SGI244495, SGI0, SGI244496, SGI0, SGI244497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54096[] = {SGI244498, SGI0, SGI244499, SGI0, SGI244500, SGI244501, SGI244502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54097[] = {SGI244503, SGI0, SGI244504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244505, SGI244507, SGI244507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54098[] = {SGI244506, SGI0, SGI244508, SGI0, SGI0, SGI244509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54099[] = {SGI244510, SGI0, SGI244511, SGI0, SGI244512, SGI244513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54100[] = {SGI244514, SGI0, SGI244515, SGI0, SGI244516, SGI244517, SGI244518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54101[] = {SGI244519, SGI0, SGI244520, SGI0, SGI0, SGI244521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54102[] = {SGI244522, SGI0, SGI244523, SGI0, SGI244524, SGI244525, SGI244526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54103[] = {SGI244527, SGI0, SGI244528, SGI0, SGI244529, SGI244530, SGI244531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54104[] = {SGI244532, SGI0, SGI244533, SGI0, SGI244534, SGI244535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54105[] = {SGI244536, SGI0, SGI244537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244538, SGI244539, SGI244539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54106[] = {SGI244540, SGI0, SGI244541, SGI0, SGI244542, SGI244543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54107[] = {SGI244544, SGI0, SGI244545, SGI0, SGI244546, SGI244547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54108[] = {SGI244548, SGI0, SGI244549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244552, SGI244554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54109[] = {SGI244550, SGI0, SGI244551, SGI0, SGI244553, SGI244555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54110[] = {SGI244556, SGI0, SGI244557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244560, SGI244561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54111[] = {SGI244558, SGI0, SGI244559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54112[] = {SGI244562, SGI0, SGI244563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244564, SGI244565, SGI244565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244568, SGI0, SGI0, NULL};
const Segment * const SG54113[] = {SGI244566, SGI0, SGI244567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54114[] = {SGI244569, SGI0, SGI244570, SGI0, SGI244571, SGI244572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54115[] = {SGI244574, SGI0, SGI244575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244577, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54116[] = {SGI244573, SGI0, SGI244576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244578, SGI244579, SGI244579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244580, SGI0, SGI0, NULL};
const Segment * const SG54117[] = {SGI244581, SGI0, SGI244582, SGI0, SGI244583, SGI244584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54118[] = {SGI244585, SGI0, SGI244586, SGI0, SGI244587, SGI244588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54119[] = {SGI244589, SGI0, SGI244590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244591, SGI244592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244593, SGI0, SGI0, NULL};
const Segment * const SG54120[] = {SGI244594, SGI0, SGI244595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244596, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54121[] = {SGI244598, SGI0, SGI244600, SGI0, SGI244601, SGI244602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54122[] = {SGI244597, SGI0, SGI244599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244603, SGI244604, SGI244604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244605, SGI0, SGI0, NULL};
const Segment * const SG54123[] = {SGI244607, SGI0, SGI244609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54124[] = {SGI244606, SGI244608, SGI244610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244611, SGI0, SGI0, SGI0, SGI244612, SGI244612, SGI0, SGI244613, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54125[] = {SGI244614, SGI0, SGI244615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244616, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54126[] = {SGI244617, SGI0, SGI244618, SGI0, SGI0, SGI244619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54127[] = {SGI244622, SGI0, SGI244623, SGI0, SGI244624, SGI244625, SGI244626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54128[] = {SGI244620, SGI0, SGI244621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244627, SGI244628, SGI244628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54129[] = {SGI244629, SGI0, SGI244630, SGI0, SGI0, SGI244631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54130[] = {SGI244632, SGI0, SGI244633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244634, SGI244635, SGI244635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54131[] = {SGI244636, SGI0, SGI244637, SGI0, SGI0, SGI244638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54132[] = {SGI244639, SGI0, SGI244640, SGI0, SGI0, SGI244641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54133[] = {SGI244642, SGI0, SGI244643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54134[] = {SGI244644, SGI0, SGI244645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244646, SGI244647, SGI244647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54135[] = {SGI244648, SGI0, SGI244649, SGI0, SGI0, SGI244650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54136[] = {SGI244651, SGI0, SGI244652, SGI0, SGI0, SGI244653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54137[] = {SGI244654, SGI0, SGI244655, SGI0, SGI244656, SGI244657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54138[] = {SGI244658, SGI0, SGI244659, SGI0, SGI0, SGI244660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54139[] = {SGI244661, SGI0, SGI244662, SGI0, SGI0, SGI244663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54140[] = {SGI244664, SGI0, SGI244665, SGI0, SGI244666, SGI244667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54141[] = {SGI244668, SGI0, SGI244669, SGI0, SGI0, SGI244670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54142[] = {SGI244671, SGI0, SGI244672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244673, SGI244674, SGI244674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54143[] = {SGI244675, SGI0, SGI244676, SGI0, SGI0, SGI244677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54144[] = {SGI244678, SGI0, SGI244679, SGI0, SGI0, SGI244680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54145[] = {SGI244681, SGI0, SGI244682, SGI0, SGI244683, SGI244684, SGI244685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54146[] = {SGI244686, SGI0, SGI244687, SGI0, SGI244688, SGI244689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54147[] = {SGI244690, SGI0, SGI244691, SGI0, SGI0, SGI244692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54148[] = {SGI244693, SGI0, SGI244694, SGI0, SGI244695, SGI244696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54149[] = {SGI244697, SGI0, SGI244698, SGI0, SGI0, SGI244699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54150[] = {SGI244700, SGI244701, SGI244702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244703, SGI0, SGI0, SGI0, SGI244705, SGI244705, SGI0, SGI244708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54151[] = {SGI244704, SGI0, SGI244706, SGI0, SGI244707, SGI244709, SGI244710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54152[] = {SGI244711, SGI0, SGI244712, SGI0, SGI244713, SGI244714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54153[] = {SGI244715, SGI0, SGI244716, SGI0, SGI244717, SGI244718, SGI244719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54154[] = {SGI244720, SGI0, SGI244721, SGI0, SGI244722, SGI244723, SGI244724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54155[] = {SGI244725, SGI0, SGI0, SGI0, SGI244726, SGI244727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54156[] = {SGI244728, SGI0, SGI244729, SGI0, SGI244730, SGI244731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54157[] = {SGI244732, SGI0, SGI244733, SGI0, SGI244734, SGI0, SGI244735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54158[] = {SGI244736, SGI244737, SGI244738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244739, SGI0, SGI0, SGI0, SGI244740, SGI244740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54159[] = {SGI244741, SGI244742, SGI244743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244744, SGI0, SGI0, SGI0, SGI244745, SGI244745, SGI0, SGI244746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54160[] = {SGI244747, SGI244748, SGI244749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244750, SGI0, SGI0, SGI0, SGI244751, SGI244751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244752, SGI0, SGI0, NULL};
const Segment * const SG54161[] = {SGI244753, SGI0, SGI244754, SGI0, SGI244755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54162[] = {SGI244756, SGI0, SGI244757, SGI0, SGI244758, SGI244759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54163[] = {SGI244760, SGI0, SGI244761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54164[] = {SGI244762, SGI0, SGI244763, SGI0, SGI244764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54165[] = {SGI244765, SGI0, SGI244768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54166[] = {SGI244766, SGI0, SGI244769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244771, SGI244773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54167[] = {SGI244767, SGI0, SGI244770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244772, SGI244774, SGI244774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54168[] = {SGI244775, SGI0, SGI244776, SGI0, SGI244777, SGI244778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54169[] = {SGI244779, SGI0, SGI244780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244781, SGI244782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54170[] = {SGI244783, SGI0, SGI244784, SGI0, SGI244785, SGI244786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54171[] = {SGI244787, SGI0, SGI244788, SGI0, SGI244789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54172[] = {SGI244790, SGI0, SGI244791, SGI0, SGI244792, SGI244793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54173[] = {SGI244794, SGI0, SGI244795, SGI0, SGI244796, SGI244797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54174[] = {SGI244798, SGI0, SGI244799, SGI0, SGI244800, SGI244801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54175[] = {SGI244802, SGI0, SGI244803, SGI0, SGI0, SGI244804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54176[] = {SGI244805, SGI244806, SGI244807, SGI0, SGI244808, SGI244810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54177[] = {SGI244809, SGI244811, SGI244812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244813, SGI0, SGI0, SGI0, SGI244814, SGI244814, SGI244815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54178[] = {SGI244816, SGI0, SGI244817, SGI0, SGI244818, SGI244819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54179[] = {SGI244820, SGI0, SGI244821, SGI0, SGI244822, SGI244823, SGI244824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54180[] = {SGI244825, SGI0, SGI244826, SGI0, SGI244827, SGI244828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54181[] = {SGI244829, SGI0, SGI244830, SGI0, SGI244831, SGI244832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54182[] = {SGI244834, SGI244835, SGI244837, SGI0, SGI244838, SGI244839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54183[] = {SGI244833, SGI0, SGI244836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244840, SGI244841, SGI244841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54184[] = {SGI244842, SGI0, SGI244843, SGI0, SGI244844, SGI244845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54185[] = {SGI244846, SGI0, SGI244847, SGI0, SGI244848, SGI244849, SGI244851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54186[] = {SGI244850, SGI0, SGI244852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244853, SGI244854, SGI244854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54187[] = {SGI244855, SGI0, SGI244856, SGI0, SGI0, SGI244857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54188[] = {SGI244858, SGI0, SGI244859, SGI0, SGI0, SGI244860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54189[] = {SGI244861, SGI0, SGI244862, SGI0, SGI244863, SGI244864, SGI244865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54190[] = {SGI244866, SGI0, SGI244867, SGI0, SGI244868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54191[] = {SGI244869, SGI0, SGI0, SGI0, SGI244870, SGI244871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54192[] = {SGI244872, SGI0, SGI244873, SGI0, SGI244874, SGI244875, SGI244874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54193[] = {SGI244876, SGI0, SGI244877, SGI0, SGI0, SGI244878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54194[] = {SGI244879, SGI0, SGI0, SGI0, SGI244880, SGI244881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54195[] = {SGI244882, SGI0, SGI244883, SGI0, SGI244884, SGI244885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54196[] = {SGI244886, SGI0, SGI244887, SGI0, SGI244888, SGI244889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54197[] = {SGI244890, SGI0, SGI244891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244892, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54198[] = {SGI244893, SGI244894, SGI244895, SGI0, SGI244896, SGI244897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54199[] = {SGI244898, SGI0, SGI244899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244900, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54200[] = {SGI244901, SGI0, SGI244902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54201[] = {SGI244903, SGI0, SGI244904, SGI0, SGI244905, SGI244906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54202[] = {SGI244907, SGI0, SGI244908, SGI0, SGI244909, SGI244910, SGI244911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54203[] = {SGI244912, SGI0, SGI244913, SGI0, SGI244914, SGI244915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54204[] = {SGI244916, SGI244917, SGI244918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244919, SGI0, SGI0, SGI0, SGI244920, SGI244920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54205[] = {SGI244921, SGI0, SGI244922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244923, SGI244924, SGI244924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54206[] = {SGI244925, SGI0, SGI244926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244927, SGI244928, SGI244928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54207[] = {SGI244929, SGI0, SGI244930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244931, SGI244932, SGI244932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54208[] = {SGI244933, SGI244934, SGI244935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244936, SGI244936, SGI0, SGI0, SGI0, SGI0, SGI244937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54209[] = {SGI244938, SGI0, SGI244939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244940, SGI244941, SGI244941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54210[] = {SGI244942, SGI0, SGI244943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244944, SGI244945, SGI244945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54211[] = {SGI244946, SGI244947, SGI244948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244949, SGI0, SGI0, SGI0, SGI244950, SGI244950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54212[] = {SGI244951, SGI0, SGI244952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244953, SGI244954, SGI244954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54213[] = {SGI244955, SGI0, SGI244956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244957, SGI244958, SGI244958, SGI0, SGI244959, NULL};
const Segment * const SG54214[] = {SGI244960, SGI0, SGI244961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244962, SGI244963, SGI244963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54215[] = {SGI244964, SGI0, SGI244965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244966, SGI244967, SGI244967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54216[] = {SGI244968, SGI244970, SGI244971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244973, SGI0, SGI0, SGI0, SGI244974, SGI244974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54217[] = {SGI244969, SGI0, SGI244972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244975, SGI244976, SGI244976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244977, SGI0, SGI0, NULL};
const Segment * const SG54218[] = {SGI244978, SGI0, SGI244979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244980, SGI244981, SGI244981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244982, SGI0, SGI0, NULL};
const Segment * const SG54219[] = {SGI244983, SGI0, SGI244984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244985, SGI244986, SGI244986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54220[] = {SGI244987, SGI0, SGI244988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244989, SGI244990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54221[] = {SGI244991, SGI0, SGI244992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244993, SGI244994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244995, SGI0, SGI0, NULL};
const Segment * const SG54222[] = {SGI244996, SGI0, SGI244997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI244998, SGI244999, SGI244999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54223[] = {SGI245000, SGI245001, SGI245002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245003, SGI0, SGI0, SGI0, SGI245004, SGI245004, SGI0, SGI245005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54224[] = {SGI245006, SGI245007, SGI245008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245009, SGI0, SGI0, SGI0, SGI245010, SGI245010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245011, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54225[] = {SGI245012, SGI245013, SGI245014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245015, SGI0, SGI0, SGI0, SGI245016, SGI245016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245017, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54226[] = {SGI245018, SGI0, SGI245019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245020, SGI245021, SGI245021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54227[] = {SGI245022, SGI0, SGI245023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245024, SGI245025, SGI245025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54228[] = {SGI245026, SGI245027, SGI245028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245029, SGI0, SGI0, SGI0, SGI245030, SGI245030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54229[] = {SGI245031, SGI0, SGI245032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245033, SGI245034, SGI245034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54230[] = {SGI245035, SGI245036, SGI245037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245038, SGI0, SGI0, SGI0, SGI245039, SGI245039, SGI0, SGI0, SGI0, SGI245040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54231[] = {SGI245041, SGI245042, SGI245043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245044, SGI0, SGI0, SGI0, SGI245045, SGI245045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245046, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54232[] = {SGI245047, SGI0, SGI245048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245049, SGI245050, SGI245050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54233[] = {SGI245051, SGI245052, SGI245053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245054, SGI245054, SGI0, SGI0, SGI0, SGI0, SGI245055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54234[] = {SGI245056, SGI0, SGI245057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245058, SGI245059, SGI245059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54235[] = {SGI245060, SGI0, SGI245061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245062, SGI245063, SGI245063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54236[] = {SGI245064, SGI0, SGI245065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245066, SGI245067, SGI245067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54237[] = {SGI245068, SGI0, SGI245069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245070, SGI245071, SGI245071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54238[] = {SGI245072, SGI0, SGI245073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245074, SGI245075, SGI245075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54239[] = {SGI245076, SGI245077, SGI245078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245079, SGI0, SGI0, SGI0, SGI245080, SGI245080, SGI0, SGI245081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54240[] = {SGI245082, SGI0, SGI245083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245084, SGI245085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54241[] = {SGI245086, SGI245087, SGI245088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245089, SGI245089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245090, NULL};
const Segment * const SG54242[] = {SGI245091, SGI245092, SGI245093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245094, SGI245094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245095, NULL};
const Segment * const SG54243[] = {SGI245096, SGI245097, SGI245098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54244[] = {SGI245099, SGI245100, SGI245101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245102, SGI0, SGI0, SGI0, SGI245103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54245[] = {SGI245104, SGI245105, SGI245106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245107, SGI245107, SGI0, SGI0, SGI0, SGI0, SGI245108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54246[] = {SGI245109, SGI0, SGI245110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245111, SGI245112, SGI245112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54247[] = {SGI245113, SGI245114, SGI245115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245116, SGI0, SGI0, SGI0, SGI245117, SGI245117, SGI0, SGI245118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54248[] = {SGI245119, SGI245120, SGI245121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245122, SGI0, SGI0, SGI0, SGI245123, SGI245123, SGI0, SGI245124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54249[] = {SGI245125, SGI245126, SGI245127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245128, SGI245128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245129, NULL};
const Segment * const SG54250[] = {SGI245130, SGI0, SGI245131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245132, SGI245133, SGI245133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245134, SGI0, SGI0, NULL};
const Segment * const SG54251[] = {SGI245135, SGI245136, SGI245137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245138, SGI245138, SGI0, SGI0, SGI0, SGI0, SGI245139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54252[] = {SGI245140, SGI245141, SGI245142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245143, SGI245143, SGI0, SGI0, SGI0, SGI0, SGI245144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54253[] = {SGI245145, SGI0, SGI245146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245147, SGI245148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54254[] = {SGI245149, SGI245150, SGI245151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245152, SGI0, SGI0, SGI0, SGI245153, SGI245154, SGI0, SGI0, SGI0, SGI0, SGI245155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54255[] = {SGI245156, SGI0, SGI245157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245158, SGI245159, SGI245159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54256[] = {SGI245160, SGI245161, SGI245162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245163, SGI0, SGI0, SGI0, SGI245164, SGI245165, SGI0, SGI0, SGI0, SGI0, SGI245166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54257[] = {SGI245167, SGI0, SGI245168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245169, SGI245170, SGI245170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54258[] = {SGI245171, SGI245172, SGI245173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245174, SGI245174, SGI0, SGI0, SGI0, SGI0, SGI245175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54259[] = {SGI245176, SGI0, SGI245177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245178, SGI245179, SGI245179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54260[] = {SGI245180, SGI0, SGI245181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245182, SGI245183, SGI245183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245184, SGI0, SGI0, NULL};
const Segment * const SG54261[] = {SGI245185, SGI0, SGI245186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245187, SGI245188, SGI245188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245189, SGI0, SGI0, NULL};
const Segment * const SG54262[] = {SGI245190, SGI0, SGI245191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245192, SGI245193, SGI245193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245194, SGI0, SGI0, NULL};
const Segment * const SG54263[] = {SGI245195, SGI0, SGI245196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245197, SGI245198, SGI245198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54264[] = {SGI245199, SGI0, SGI245200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245201, SGI245202, SGI245202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54265[] = {SGI245203, SGI0, SGI245204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245205, SGI245206, SGI245206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54266[] = {SGI245207, SGI0, SGI245208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245209, SGI245210, SGI245210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54267[] = {SGI245211, SGI0, SGI245212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245213, SGI245214, SGI245214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54268[] = {SGI245215, SGI245216, SGI245217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245218, SGI0, SGI0, SGI0, SGI245219, SGI245219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54269[] = {SGI245220, SGI0, SGI245221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245222, SGI245223, SGI245223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54270[] = {SGI245224, SGI0, SGI245225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245226, SGI0, SGI0, SGI0, SGI245227, SGI245227, SGI0, SGI0, SGI0, SGI0, SGI245228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54271[] = {SGI245229, SGI245230, SGI245231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245232, SGI0, SGI0, SGI0, SGI245233, SGI245233, SGI0, SGI245234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54272[] = {SGI245235, SGI0, SGI245236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245237, SGI245238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54273[] = {SGI245239, SGI0, SGI245240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245241, SGI245242, SGI245242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54274[] = {SGI245243, SGI0, SGI245244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245245, SGI245246, SGI245246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54275[] = {SGI245247, SGI0, SGI245248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245249, SGI245250, SGI245250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54276[] = {SGI245251, SGI0, SGI245252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245253, SGI245254, SGI245254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54277[] = {SGI245255, SGI0, SGI245256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245257, SGI245258, SGI245258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54278[] = {SGI245259, SGI245260, SGI245261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245262, SGI0, SGI0, SGI0, SGI245263, SGI245263, SGI0, SGI245264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54279[] = {SGI245265, SGI245266, SGI245267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245268, SGI0, SGI0, SGI0, SGI245269, SGI245269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54280[] = {SGI245270, SGI0, SGI245271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245272, SGI245273, SGI245273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54281[] = {SGI245274, SGI0, SGI245275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245276, SGI245277, SGI245277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54282[] = {SGI245278, SGI0, SGI245279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245280, SGI245281, SGI245281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54283[] = {SGI245282, SGI245283, SGI245284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245285, SGI0, SGI0, SGI0, SGI245286, SGI245286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54284[] = {SGI245287, SGI245288, SGI245289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245293, SGI0, SGI0, SGI0, SGI245294, SGI245294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54285[] = {SGI245290, SGI245291, SGI245292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245295, SGI0, SGI0, SGI0, SGI245296, SGI245296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54286[] = {SGI245297, SGI0, SGI245298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245299, SGI245300, SGI245300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54287[] = {SGI245301, SGI0, SGI245302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245303, SGI245304, SGI245304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54288[] = {SGI245305, SGI0, SGI245307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245309, SGI245310, SGI245310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54289[] = {SGI245306, SGI0, SGI245308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245311, SGI245312, SGI245312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245313, SGI0, SGI0, NULL};
const Segment * const SG54290[] = {SGI245314, SGI245315, SGI245316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245317, SGI245317, SGI0, SGI0, SGI0, SGI0, SGI245318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54291[] = {SGI245319, SGI245320, SGI245321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245322, SGI245322, SGI0, SGI0, SGI0, SGI0, SGI245323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54292[] = {SGI245324, SGI245325, SGI245326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245327, SGI0, SGI0, SGI0, SGI245328, SGI245328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54293[] = {SGI245329, SGI0, SGI245330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245331, SGI245332, SGI245332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54294[] = {SGI245333, SGI0, SGI245334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245335, SGI245336, SGI245336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245337, SGI0, SGI0, NULL};
const Segment * const SG54295[] = {SGI245338, SGI0, SGI245339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245340, SGI245341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54296[] = {SGI245342, SGI0, SGI245343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54297[] = {SGI245344, SGI0, SGI245346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245348, SGI245349, SGI245349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54298[] = {SGI245345, SGI0, SGI245347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245350, SGI245351, SGI245351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54299[] = {SGI245352, SGI245353, SGI245354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245355, SGI0, SGI0, SGI0, SGI245356, SGI245356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54300[] = {SGI245357, SGI0, SGI245358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245359, SGI245360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54301[] = {SGI245361, SGI0, SGI245362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245363, SGI245364, SGI245364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54302[] = {SGI245365, SGI245366, SGI245367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245368, SGI0, SGI0, SGI0, SGI245369, SGI245369, SGI0, SGI245370, SGI0, SGI0, SGI0, SGI0, SGI245371, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54303[] = {SGI245372, SGI245373, SGI245374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245375, SGI0, SGI0, SGI0, SGI245376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245377, SGI0, NULL};
const Segment * const SG54304[] = {SGI245378, SGI245379, SGI245380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245381, SGI245381, SGI0, SGI0, SGI0, SGI0, SGI245382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54305[] = {SGI245383, SGI245384, SGI245385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245386, SGI0, SGI0, SGI0, SGI245387, SGI245387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245388, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54306[] = {SGI245389, SGI245390, SGI245391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245392, SGI245392, SGI0, SGI0, SGI0, SGI0, SGI245393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54307[] = {SGI245394, SGI245395, SGI245396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245397, SGI245397, SGI0, SGI0, SGI0, SGI0, SGI245398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54308[] = {SGI245399, SGI245400, SGI245401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245402, SGI0, SGI0, SGI0, SGI245403, SGI245403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245404, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54309[] = {SGI245405, SGI245406, SGI245407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245408, SGI245408, SGI0, SGI0, SGI0, SGI0, SGI245409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54310[] = {SGI245410, SGI245411, SGI245412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245413, SGI245413, SGI0, SGI0, SGI0, SGI0, SGI245414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54311[] = {SGI245415, SGI245416, SGI245417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245418, SGI0, SGI0, SGI0, SGI245419, SGI0, SGI0, SGI245420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54312[] = {SGI245421, SGI0, SGI245422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245423, SGI245424, SGI245424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54313[] = {SGI245425, SGI0, SGI245426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245427, SGI0, SGI0, SGI0, SGI245428, SGI245428, SGI0, SGI0, SGI0, SGI0, SGI245429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54314[] = {SGI245430, SGI0, SGI245431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245432, SGI245433, SGI245433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54315[] = {SGI245434, SGI245435, SGI245436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245437, SGI0, SGI0, SGI0, SGI245438, SGI245438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54316[] = {SGI245439, SGI245440, SGI245441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245442, SGI0, SGI0, SGI0, SGI245443, SGI245443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54317[] = {SGI245444, SGI0, SGI245445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245446, SGI245447, SGI245447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54318[] = {SGI245448, SGI0, SGI245449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245450, SGI245451, SGI245451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54319[] = {SGI245452, SGI245453, SGI245454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245455, SGI0, SGI0, SGI0, SGI245456, SGI245456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54320[] = {SGI245457, SGI245458, SGI245459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245460, SGI0, SGI0, SGI0, SGI245461, SGI0, SGI245462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54321[] = {SGI245463, SGI0, SGI245464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245465, SGI245466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245467, SGI0, SGI0, NULL};
const Segment * const SG54322[] = {SGI245468, SGI0, SGI245469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54323[] = {SGI245470, SGI245471, SGI245472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245473, SGI0, SGI0, SGI0, SGI245474, SGI245474, SGI0, SGI0, SGI0, SGI245475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54324[] = {SGI245476, SGI0, SGI245477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245478, SGI245479, SGI245479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54325[] = {SGI245480, SGI245481, SGI245482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245483, SGI0, SGI0, SGI0, SGI245484, SGI245484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245485, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54326[] = {SGI245486, SGI245487, SGI245488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245489, SGI0, SGI0, SGI0, SGI245490, SGI0, SGI0, SGI245491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54327[] = {SGI245492, SGI0, SGI245493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245494, SGI245495, SGI245495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54328[] = {SGI245496, SGI245497, SGI245498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245499, SGI0, SGI0, SGI0, SGI245500, SGI245500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245501, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54329[] = {SGI245502, SGI245503, SGI245504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245505, SGI0, SGI0, SGI0, SGI245506, SGI245506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54330[] = {SGI245507, SGI245508, SGI245509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245510, SGI0, SGI0, SGI0, SGI245511, SGI245511, SGI0, SGI0, SGI0, SGI0, SGI245512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54331[] = {SGI245513, SGI0, SGI245514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245515, SGI245516, SGI245516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54332[] = {SGI245517, SGI245518, SGI245519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245520, SGI245520, SGI0, SGI0, SGI0, SGI0, SGI245521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54333[] = {SGI245522, SGI0, SGI245523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245524, SGI245525, SGI245525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245526, SGI0, SGI0, NULL};
const Segment * const SG54334[] = {SGI245527, SGI0, SGI245528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245529, SGI245530, SGI245530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245531, SGI0, SGI0, NULL};
const Segment * const SG54335[] = {SGI245532, SGI245533, SGI245534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245535, SGI245535, SGI0, SGI0, SGI0, SGI0, SGI245536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54336[] = {SGI245537, SGI245538, SGI245539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245540, SGI245540, SGI0, SGI0, SGI0, SGI0, SGI245541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54337[] = {SGI245542, SGI0, SGI245543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245544, SGI245545, SGI245545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54338[] = {SGI245546, SGI245547, SGI245548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245549, SGI245549, SGI0, SGI0, SGI0, SGI0, SGI245550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54339[] = {SGI245551, SGI245552, SGI245553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245554, SGI0, SGI0, SGI0, SGI245555, SGI245556, SGI0, SGI0, SGI0, SGI0, SGI245557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54340[] = {SGI245558, SGI0, SGI245559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245560, SGI245561, SGI245561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54341[] = {SGI245562, SGI0, SGI245563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245564, SGI245565, SGI245565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245566, SGI0, SGI0, NULL};
const Segment * const SG54342[] = {SGI245567, SGI245568, SGI245569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245570, SGI245570, SGI0, SGI0, SGI0, SGI0, SGI245571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54343[] = {SGI245572, SGI0, SGI245573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245574, SGI245575, SGI245575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245576, SGI0, SGI0, NULL};
const Segment * const SG54344[] = {SGI245577, SGI0, SGI245578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245579, SGI245580, SGI245580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54345[] = {SGI245582, SGI0, SGI245584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245586, SGI245588, SGI245588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54346[] = {SGI245581, SGI0, SGI245583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245585, SGI245587, SGI245587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54347[] = {SGI245589, SGI0, SGI245590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245591, SGI245592, SGI245592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54348[] = {SGI245593, SGI0, SGI245594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245595, SGI245596, SGI245596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54349[] = {SGI245597, SGI0, SGI245598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245599, SGI245600, SGI245600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245601, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54350[] = {SGI245602, SGI245603, SGI245604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245605, SGI245605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245606, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54351[] = {SGI245607, SGI0, SGI245608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245609, SGI245610, SGI245610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54352[] = {SGI245611, SGI0, SGI245612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245613, SGI245614, SGI245614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54353[] = {SGI245615, SGI245616, SGI245617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245618, SGI0, SGI0, SGI0, SGI245619, SGI245619, SGI0, SGI245620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54354[] = {SGI245621, SGI0, SGI245622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245623, SGI245624, SGI245624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54355[] = {SGI245625, SGI0, SGI245626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245627, SGI245628, SGI245628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54356[] = {SGI245629, SGI0, SGI245630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245631, SGI245632, SGI245632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54357[] = {SGI245633, SGI245634, SGI245635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245636, SGI0, SGI0, SGI0, SGI245637, SGI0, SGI0, SGI0, SGI245638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54358[] = {SGI245639, SGI0, SGI245640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245641, SGI245642, SGI245642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54359[] = {SGI245643, SGI245644, SGI245645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245646, SGI0, SGI0, SGI0, SGI245647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54360[] = {SGI245648, SGI0, SGI245649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245650, SGI0, SGI0, SGI0, SGI245651, SGI245651, SGI0, SGI0, SGI0, SGI0, SGI245652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54361[] = {SGI245653, SGI0, SGI245654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245655, SGI245656, SGI245656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54362[] = {SGI245657, SGI245658, SGI245659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245660, SGI245660, SGI0, SGI0, SGI0, SGI0, SGI245661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54363[] = {SGI245662, SGI245663, SGI245664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245665, SGI0, SGI0, SGI0, SGI245666, SGI245666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54364[] = {SGI245667, SGI245668, SGI245669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245670, SGI0, SGI0, SGI0, SGI245671, SGI245671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54365[] = {SGI245672, SGI0, SGI245673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245674, SGI245675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54366[] = {SGI245676, SGI0, SGI245677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245678, SGI245679, SGI245679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54367[] = {SGI245680, SGI0, SGI245681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245682, SGI245683, SGI245683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54368[] = {SGI245684, SGI245685, SGI245686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245687, SGI0, SGI0, SGI0, SGI245688, SGI245688, SGI0, SGI245691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54369[] = {SGI245689, SGI245690, SGI245692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245693, SGI0, SGI0, SGI0, SGI245694, SGI245694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54370[] = {SGI245695, SGI245696, SGI245697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245698, SGI0, SGI0, SGI0, SGI245699, SGI245699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54371[] = {SGI245700, SGI0, SGI245701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245702, SGI245703, SGI245703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54372[] = {SGI245704, SGI245705, SGI245706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245707, SGI0, SGI0, SGI0, SGI245708, SGI245708, SGI0, SGI0, SGI245709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54373[] = {SGI245710, SGI0, SGI245711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245712, SGI245713, SGI245713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54374[] = {SGI245714, SGI245715, SGI245716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245717, SGI0, SGI0, SGI0, SGI245718, SGI245718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54375[] = {SGI245719, SGI0, SGI245720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245721, SGI245722, SGI245722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54376[] = {SGI245723, SGI245724, SGI245725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245726, SGI245726, SGI0, SGI0, SGI0, SGI0, SGI245727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54377[] = {SGI245728, SGI0, SGI245729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245730, SGI245731, SGI245731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54378[] = {SGI245732, SGI245733, SGI245734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245735, SGI0, SGI0, SGI0, SGI245736, SGI245736, SGI0, SGI0, SGI0, SGI0, SGI245737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54379[] = {SGI245738, SGI245739, SGI245740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245741, SGI0, SGI0, SGI0, SGI245742, SGI245742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245743, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54380[] = {SGI245744, SGI0, SGI245745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245746, SGI245747, SGI245747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54381[] = {SGI245748, SGI245749, SGI245750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245751, SGI245751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245754, NULL};
const Segment * const SG54382[] = {SGI245752, SGI245753, SGI245755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245756, SGI0, SGI0, SGI0, SGI245757, SGI245757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245758, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54383[] = {SGI245759, SGI0, SGI245760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245761, SGI245762, SGI245762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54384[] = {SGI245763, SGI0, SGI245764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245765, SGI245766, SGI245766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54385[] = {SGI245767, SGI245768, SGI245769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245770, SGI0, SGI0, SGI0, SGI245771, SGI245771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54386[] = {SGI245772, SGI0, SGI245773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245774, SGI245775, SGI245775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54387[] = {SGI245776, SGI0, SGI245777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245778, SGI245779, SGI245779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54388[] = {SGI245780, SGI0, SGI245781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245782, SGI245783, SGI245783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54389[] = {SGI245784, SGI0, SGI245785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245786, SGI245787, SGI245787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54390[] = {SGI245788, SGI0, SGI245789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245790, SGI245791, SGI245791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54391[] = {SGI245792, SGI245793, SGI245794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245795, SGI245795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54392[] = {SGI245796, SGI245797, SGI245798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245799, SGI0, SGI0, SGI0, SGI245800, SGI245800, SGI0, SGI245801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54393[] = {SGI245802, SGI0, SGI245803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245804, SGI245805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54394[] = {SGI245806, SGI0, SGI245807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245808, SGI245809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54395[] = {SGI245810, SGI245811, SGI245812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245813, SGI245813, SGI0, SGI0, SGI0, SGI0, SGI245814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54396[] = {SGI245815, SGI245816, SGI245817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245818, SGI245818, SGI0, SGI0, SGI0, SGI0, SGI245819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54397[] = {SGI245820, SGI245821, SGI245822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245823, SGI0, SGI0, SGI0, SGI245824, SGI245824, SGI0, SGI245825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54398[] = {SGI245826, SGI245827, SGI245828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245829, SGI0, SGI0, SGI0, SGI245830, SGI245830, SGI0, SGI245831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54399[] = {SGI245832, SGI0, SGI245833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245834, SGI245835, SGI245835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54400[] = {SGI245836, SGI245837, SGI245838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245839, SGI0, SGI0, SGI0, SGI245840, SGI245840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245841, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54401[] = {SGI245842, SGI245843, SGI245844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245845, SGI0, SGI0, SGI0, SGI245846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54402[] = {SGI245847, SGI0, SGI245848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245849, SGI245850, SGI245850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54403[] = {SGI245851, SGI245852, SGI245853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245854, SGI245854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245855, NULL};
const Segment * const SG54404[] = {SGI245856, SGI245857, SGI245858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245859, SGI0, SGI0, SGI0, SGI245860, SGI245860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54405[] = {SGI245861, SGI0, SGI245862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245863, SGI245864, SGI245864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54406[] = {SGI245865, SGI245866, SGI245867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245868, SGI245868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245869, NULL};
const Segment * const SG54407[] = {SGI245870, SGI245871, SGI245872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245873, SGI245873, SGI0, SGI0, SGI0, SGI0, SGI245874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54408[] = {SGI245875, SGI0, SGI245876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245877, SGI0, SGI0, SGI0, SGI245878, SGI245878, SGI0, SGI0, SGI0, SGI0, SGI245879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54409[] = {SGI245880, SGI245881, SGI245882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245883, SGI0, SGI0, SGI0, SGI245884, SGI245885, SGI0, SGI0, SGI0, SGI0, SGI245886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54410[] = {SGI245887, SGI0, SGI245888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245889, SGI245890, SGI245890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54411[] = {SGI245891, SGI0, SGI245892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245893, SGI245894, SGI245894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54412[] = {SGI245895, SGI245896, SGI245897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245898, SGI0, SGI0, SGI0, SGI245899, SGI245900, SGI0, SGI0, SGI0, SGI0, SGI245901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54413[] = {SGI245902, SGI0, SGI245903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245904, SGI245905, SGI245905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245906, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54414[] = {SGI245907, SGI0, SGI245908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245909, SGI245910, SGI245910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54415[] = {SGI245911, SGI245913, SGI245915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245917, SGI0, SGI0, SGI0, SGI245918, SGI245918, SGI0, SGI245920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54416[] = {SGI245912, SGI245914, SGI245916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245919, SGI245919, SGI0, SGI0, SGI0, SGI0, SGI245921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54417[] = {SGI245922, SGI245923, SGI245924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245925, SGI0, SGI0, SGI0, SGI245926, SGI0, SGI0, SGI245927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54418[] = {SGI245928, SGI245929, SGI245930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245931, SGI245931, SGI0, SGI0, SGI0, SGI0, SGI245932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54419[] = {SGI245933, SGI0, SGI245934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245935, SGI245936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54420[] = {SGI245937, SGI245938, SGI245939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245940, SGI245940, SGI0, SGI0, SGI0, SGI0, SGI245941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54421[] = {SGI245942, SGI0, SGI245943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245944, SGI245945, SGI245945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54422[] = {SGI245946, SGI0, SGI245947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245948, SGI245949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54423[] = {SGI245950, SGI0, SGI245951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245952, SGI245953, SGI245953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54424[] = {SGI245954, SGI0, SGI245955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI245956, SGI245957, SGI245957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54425[] = {SGI245958, SGI0, SGI245959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54426[] = {SGI245960, SGI0, SGI245961, SGI0, SGI245962, SGI245963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54427[] = {SGI245964, SGI0, SGI245965, SGI0, SGI245966, SGI245967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54428[] = {SGI245968, SGI0, SGI245969, SGI0, SGI245970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54429[] = {SGI245971, SGI0, SGI245972, SGI0, SGI245973, SGI245974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54430[] = {SGI245975, SGI0, SGI245976, SGI0, SGI245977, SGI245978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54431[] = {SGI245979, SGI0, SGI245980, SGI0, SGI245981, SGI245982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54432[] = {SGI245983, SGI0, SGI245984, SGI0, SGI245985, SGI245986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54433[] = {SGI245987, SGI0, SGI245988, SGI0, SGI245989, SGI245990, SGI245991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54434[] = {SGI245992, SGI0, SGI245993, SGI0, SGI245994, SGI245995, SGI245996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54435[] = {SGI245997, SGI0, SGI245998, SGI0, SGI0, SGI245999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54436[] = {SGI246000, SGI0, SGI246001, SGI0, SGI0, SGI246002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54437[] = {SGI246003, SGI0, SGI246004, SGI0, SGI246005, SGI246006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54438[] = {SGI246007, SGI0, SGI246008, SGI0, SGI246009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54439[] = {SGI246010, SGI0, SGI246011, SGI0, SGI246012, SGI246013, SGI246014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54440[] = {SGI246015, SGI0, SGI246016, SGI0, SGI246017, SGI246018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54441[] = {SGI246019, SGI0, SGI246020, SGI0, SGI246021, SGI246022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54442[] = {SGI246023, SGI0, SGI246024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246025, SGI246026, SGI246026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54443[] = {SGI246027, SGI0, SGI246028, SGI0, SGI246029, SGI246030, SGI246031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54444[] = {SGI246032, SGI0, SGI246033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54445[] = {SGI246034, SGI0, SGI246035, SGI0, SGI246036, SGI246037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246038, SGI0, NULL};
const Segment * const SG54446[] = {SGI246039, SGI0, SGI246040, SGI0, SGI246041, SGI246042, SGI246043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54447[] = {SGI246044, SGI0, SGI246045, SGI0, SGI246046, SGI246047, SGI246048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54448[] = {SGI246049, SGI0, SGI246050, SGI0, SGI246051, SGI246052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54449[] = {SGI246053, SGI0, SGI246054, SGI0, SGI246055, SGI246056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54450[] = {SGI246057, SGI0, SGI246058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54451[] = {SGI246059, SGI0, SGI246060, SGI0, SGI246061, SGI246062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54452[] = {SGI246063, SGI0, SGI246064, SGI0, SGI246065, SGI246066, SGI246067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54453[] = {SGI246068, SGI0, SGI246069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246070, SGI246072, SGI246072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54454[] = {SGI246071, SGI0, SGI246073, SGI0, SGI246074, SGI246075, SGI246076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54455[] = {SGI246077, SGI0, SGI246078, SGI0, SGI246079, SGI246080, SGI246081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54456[] = {SGI246082, SGI0, SGI246083, SGI0, SGI246084, SGI246085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54457[] = {SGI246086, SGI0, SGI246087, SGI0, SGI246088, SGI246089, SGI246090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54458[] = {SGI246091, SGI0, SGI246092, SGI0, SGI246093, SGI246094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54459[] = {SGI246095, SGI0, SGI246096, SGI0, SGI246097, SGI246098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54460[] = {SGI246099, SGI0, SGI246100, SGI0, SGI246101, SGI246102, SGI246103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54461[] = {SGI246104, SGI0, SGI246105, SGI0, SGI246106, SGI246107, SGI246108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54462[] = {SGI246111, SGI0, SGI246112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54463[] = {SGI246109, SGI0, SGI246110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246113, SGI246114, SGI246114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54464[] = {SGI246115, SGI0, SGI246116, SGI0, SGI246117, SGI246118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54465[] = {SGI246119, SGI0, SGI246120, SGI0, SGI246121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54466[] = {SGI246122, SGI0, SGI246123, SGI0, SGI246124, SGI246125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54467[] = {SGI246126, SGI0, SGI246127, SGI0, SGI246128, SGI246129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54468[] = {SGI246130, SGI0, SGI246131, SGI0, SGI246132, SGI246133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54469[] = {SGI246134, SGI0, SGI246135, SGI0, SGI246136, SGI246137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54470[] = {SGI246138, SGI0, SGI246139, SGI0, SGI246140, SGI246141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54471[] = {SGI246142, SGI0, SGI246143, SGI0, SGI246144, SGI246145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54472[] = {SGI246146, SGI0, SGI246147, SGI0, SGI246148, SGI246149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54473[] = {SGI246150, SGI0, SGI246151, SGI0, SGI246152, SGI246153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54474[] = {SGI246154, SGI0, SGI246155, SGI0, SGI246156, SGI246157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54475[] = {SGI246158, SGI0, SGI246159, SGI0, SGI246160, SGI246161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246162, SGI0, NULL};
const Segment * const SG54476[] = {SGI246163, SGI0, SGI246164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246165, SGI246166, SGI246166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54477[] = {SGI246167, SGI0, SGI246168, SGI0, SGI246169, SGI246170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54478[] = {SGI246171, SGI0, SGI246172, SGI0, SGI0, SGI246173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54479[] = {SGI246174, SGI0, SGI246175, SGI0, SGI0, SGI246176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54480[] = {SGI246177, SGI0, SGI246178, SGI0, SGI0, SGI246179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54481[] = {SGI246180, SGI0, SGI246181, SGI0, SGI0, SGI246182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54482[] = {SGI246183, SGI0, SGI246184, SGI0, SGI246185, SGI246186, SGI246188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54483[] = {SGI246187, SGI246189, SGI246190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246191, SGI246191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54484[] = {SGI246192, SGI0, SGI246193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54485[] = {SGI246194, SGI0, SGI246195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246196, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54486[] = {SGI246197, SGI0, SGI246198, SGI0, SGI246199, SGI246200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54487[] = {SGI246201, SGI0, SGI246202, SGI0, SGI246204, SGI246207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54488[] = {SGI246203, SGI246205, SGI246206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246208, SGI246208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54489[] = {SGI246209, SGI0, SGI246210, SGI0, SGI246211, SGI246212, SGI246213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54490[] = {SGI246214, SGI0, SGI246215, SGI0, SGI246217, SGI246220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54491[] = {SGI246216, SGI246218, SGI246219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246221, SGI0, SGI0, SGI0, SGI246222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246223, SGI0, SGI0, NULL};
const Segment * const SG54492[] = {SGI246224, SGI0, SGI246225, SGI0, SGI246226, SGI246227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54493[] = {SGI246228, SGI0, SGI246229, SGI0, SGI0, SGI246230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54494[] = {SGI246231, SGI0, SGI246232, SGI0, SGI246233, SGI246234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54495[] = {SGI246235, SGI0, SGI246236, SGI0, SGI0, SGI246237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54496[] = {SGI246238, SGI0, SGI246239, SGI0, SGI246240, SGI246241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54497[] = {SGI246242, SGI0, SGI246243, SGI0, SGI246244, SGI246245, SGI246246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54498[] = {SGI246247, SGI0, SGI246248, SGI0, SGI246249, SGI246250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54499[] = {SGI246251, SGI0, SGI246252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246253, SGI246254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246255, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54500[] = {SGI246256, SGI0, SGI246257, SGI0, SGI246258, SGI246259, SGI246260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54501[] = {SGI246261, SGI0, SGI246263, SGI0, SGI246265, SGI246266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54502[] = {SGI246262, SGI0, SGI246264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246267, SGI246268, SGI246268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54503[] = {SGI246269, SGI0, SGI246270, SGI0, SGI246271, SGI246272, SGI246273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54504[] = {SGI246274, SGI0, SGI246275, SGI0, SGI246276, SGI246277, SGI246278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54505[] = {SGI246279, SGI0, SGI246280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246281, SGI246282, SGI246282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54506[] = {SGI246283, SGI0, SGI246284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246285, SGI246286, SGI246286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54507[] = {SGI246287, SGI0, SGI246288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246289, SGI246290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54508[] = {SGI246291, SGI0, SGI246292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246293, SGI246294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54509[] = {SGI246295, SGI0, SGI246296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246297, SGI246298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54510[] = {SGI246299, SGI0, SGI246300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246301, SGI246302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54511[] = {SGI246303, SGI0, SGI246304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246305, SGI246306, SGI246306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54512[] = {SGI246307, SGI0, SGI246308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246309, SGI246310, SGI246310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54513[] = {SGI246311, SGI0, SGI246312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246313, SGI246314, SGI246314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54514[] = {SGI246315, SGI246316, SGI246317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246318, SGI0, SGI0, SGI0, SGI246319, SGI246319, SGI246320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54515[] = {SGI246321, SGI0, SGI246322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246323, SGI246324, SGI246324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54516[] = {SGI246325, SGI0, SGI246326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246327, SGI246328, SGI246328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54517[] = {SGI246329, SGI0, SGI246330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246331, SGI246333, SGI246333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54518[] = {SGI246332, SGI0, SGI246334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246335, SGI246336, SGI246336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54519[] = {SGI246337, SGI0, SGI246338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246339, SGI246340, SGI246340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54520[] = {SGI246341, SGI246342, SGI246343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246344, SGI246344, SGI0, SGI0, SGI0, SGI0, SGI246345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54521[] = {SGI246346, SGI246347, SGI246348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246349, SGI0, SGI0, SGI0, SGI246350, SGI246350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246351, NULL};
const Segment * const SG54522[] = {SGI246352, SGI246353, SGI246354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246355, SGI246355, SGI0, SGI0, SGI0, SGI0, SGI246356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54523[] = {SGI246357, SGI0, SGI246358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246359, SGI246360, SGI246360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54524[] = {SGI246361, SGI0, SGI246362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246363, SGI246364, SGI246364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54525[] = {SGI246365, SGI0, SGI246366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246367, SGI246368, SGI246368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54526[] = {SGI246369, SGI0, SGI246370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246371, SGI246372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246373, SGI0, SGI0, NULL};
const Segment * const SG54527[] = {SGI246374, SGI0, SGI246375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246376, SGI246377, SGI246377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246378, SGI0, SGI0, NULL};
const Segment * const SG54528[] = {SGI246379, SGI0, SGI246380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246381, SGI246382, SGI246382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54529[] = {SGI246383, SGI246384, SGI246385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246386, SGI0, SGI0, SGI0, SGI246387, SGI246387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246388, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54530[] = {SGI246389, SGI0, SGI246390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246391, SGI246392, SGI246392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54531[] = {SGI246393, SGI246394, SGI246395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246396, SGI0, SGI0, SGI0, SGI246397, SGI246397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246398, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54532[] = {SGI246399, SGI0, SGI246400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246401, SGI246402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54533[] = {SGI246403, SGI0, SGI246404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246405, SGI246406, SGI246406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54534[] = {SGI246407, SGI0, SGI246408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246409, SGI246410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54535[] = {SGI246411, SGI246412, SGI246413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246414, SGI0, SGI0, SGI0, SGI246415, SGI246415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54536[] = {SGI246416, SGI0, SGI246417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246418, SGI246419, SGI246419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54537[] = {SGI246420, SGI0, SGI246421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246422, SGI246423, SGI246423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54538[] = {SGI246424, SGI0, SGI246425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246426, SGI246427, SGI246427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54539[] = {SGI246428, SGI246429, SGI246430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246431, SGI0, SGI0, SGI0, SGI246432, SGI246432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246433, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54540[] = {SGI246434, SGI0, SGI246435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246436, SGI246437, SGI246437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54541[] = {SGI246438, SGI246439, SGI246440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246441, SGI0, SGI0, SGI0, SGI246442, SGI246442, SGI0, SGI0, SGI0, SGI246443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54542[] = {SGI246444, SGI246445, SGI246446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246447, SGI0, SGI0, SGI0, SGI246448, SGI246448, SGI0, SGI0, SGI246449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54543[] = {SGI246450, SGI246451, SGI246452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246453, SGI0, SGI0, SGI0, SGI246454, SGI0, SGI246455, SGI246456, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54544[] = {SGI246457, SGI246458, SGI246459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246460, SGI0, SGI0, SGI0, SGI246461, SGI0, SGI246462, SGI246463, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54545[] = {SGI246464, SGI246465, SGI246466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246467, SGI0, SGI0, SGI0, SGI246468, SGI246468, SGI0, SGI246469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54546[] = {SGI246470, SGI246472, SGI246474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246476, SGI246476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54547[] = {SGI246471, SGI246473, SGI246475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246477, SGI246477, SGI0, SGI0, SGI0, SGI0, SGI246478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54548[] = {SGI246479, SGI246480, SGI246481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246482, SGI0, SGI0, SGI0, SGI246483, SGI246483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246484, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54549[] = {SGI246485, SGI0, SGI246486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246487, SGI246488, SGI246488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246489, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54550[] = {SGI246490, SGI246491, SGI246492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246493, SGI246493, SGI0, SGI0, SGI0, SGI0, SGI246494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54551[] = {SGI246495, SGI246496, SGI246497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246498, SGI0, SGI0, SGI0, SGI246499, SGI246499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246500, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54552[] = {SGI246501, SGI0, SGI246502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246503, SGI246504, SGI246504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246505, SGI0, SGI0, NULL};
const Segment * const SG54553[] = {SGI246506, SGI0, SGI246507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246508, SGI246509, SGI246509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54554[] = {SGI246510, SGI246511, SGI246512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246513, SGI0, SGI0, SGI0, SGI246514, SGI246514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54555[] = {SGI246515, SGI246516, SGI246517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246518, SGI0, SGI0, SGI0, SGI246519, SGI246519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246520, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54556[] = {SGI246521, SGI0, SGI246522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246523, SGI246524, SGI246524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54557[] = {SGI246525, SGI246526, SGI246527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246528, SGI246528, SGI0, SGI0, SGI0, SGI0, SGI246529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54558[] = {SGI246530, SGI246531, SGI246532, SGI0, SGI246533, SGI246534, SGI246535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54559[] = {SGI246536, SGI0, SGI246537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246538, SGI0, SGI0, SGI0, SGI246539, SGI246539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246540, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54560[] = {SGI246541, SGI246542, SGI246543, SGI0, SGI246544, SGI246545, SGI246546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54561[] = {SGI246547, SGI0, SGI246548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246549, SGI246551, SGI246551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54562[] = {SGI246550, SGI0, SGI246552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246556, SGI246557, SGI246557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54563[] = {SGI246553, SGI246554, SGI246555, SGI0, SGI246558, SGI246559, SGI246560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54564[] = {SGI246561, SGI246562, SGI246563, SGI0, SGI246564, SGI246565, SGI246566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54565[] = {SGI246567, SGI246568, SGI246569, SGI0, SGI246570, SGI246571, SGI246572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54566[] = {SGI246573, SGI246574, SGI246575, SGI0, SGI246576, SGI246577, SGI246578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54567[] = {SGI246579, SGI246580, SGI246581, SGI0, SGI246582, SGI246583, SGI246584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54568[] = {SGI246585, SGI246586, SGI246587, SGI0, SGI246588, SGI246589, SGI246590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54569[] = {SGI246591, SGI246592, SGI246593, SGI0, SGI246594, SGI246595, SGI246596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54570[] = {SGI246597, SGI246598, SGI246599, SGI0, SGI246600, SGI246601, SGI246603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54571[] = {SGI246602, SGI0, SGI246604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246605, SGI246606, SGI246606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54572[] = {SGI246607, SGI246608, SGI246609, SGI0, SGI246610, SGI246611, SGI246612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54573[] = {SGI246613, SGI0, SGI246614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246615, SGI246616, SGI246616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54574[] = {SGI246617, SGI246618, SGI246619, SGI0, SGI246620, SGI246621, SGI246622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54575[] = {SGI246623, SGI0, SGI246624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246627, SGI246629, SGI246629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54576[] = {SGI246625, SGI246626, SGI246628, SGI0, SGI246630, SGI246631, SGI246632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54577[] = {SGI246633, SGI246634, SGI246635, SGI0, SGI246636, SGI246637, SGI246638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54578[] = {SGI246639, SGI246640, SGI246641, SGI0, SGI246642, SGI246643, SGI246644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54579[] = {SGI246645, SGI246646, SGI246647, SGI0, SGI246648, SGI246649, SGI246650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54580[] = {SGI246651, SGI246652, SGI246653, SGI0, SGI246654, SGI246655, SGI246656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54581[] = {SGI246657, SGI0, SGI246658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246659, SGI246660, SGI246660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246661, SGI0, SGI0, NULL};
const Segment * const SG54582[] = {SGI246662, SGI246663, SGI246664, SGI0, SGI246665, SGI246666, SGI246667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54583[] = {SGI246668, SGI246669, SGI246670, SGI0, SGI246671, SGI246672, SGI246673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54584[] = {SGI246674, SGI246675, SGI246676, SGI0, SGI246677, SGI246678, SGI246679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54585[] = {SGI246680, SGI246681, SGI246682, SGI0, SGI246683, SGI246684, SGI246685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54586[] = {SGI246686, SGI246687, SGI246688, SGI0, SGI246689, SGI246690, SGI246691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54587[] = {SGI246692, SGI246693, SGI246694, SGI0, SGI246695, SGI246697, SGI246698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54588[] = {SGI246696, SGI0, SGI246699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246700, SGI246701, SGI246701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54589[] = {SGI246702, SGI246703, SGI246704, SGI0, SGI246705, SGI246706, SGI246707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54590[] = {SGI246708, SGI246709, SGI246710, SGI0, SGI246711, SGI246712, SGI246713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54591[] = {SGI246714, SGI0, SGI246715, SGI0, SGI0, SGI246716, SGI246717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54592[] = {SGI246718, SGI0, SGI246719, SGI0, SGI0, SGI246720, SGI246721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54593[] = {SGI246722, SGI246723, SGI246724, SGI0, SGI246725, SGI246726, SGI246727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54594[] = {SGI246729, SGI246730, SGI246732, SGI0, SGI246733, SGI246734, SGI246735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54595[] = {SGI246728, SGI0, SGI246731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246736, SGI246737, SGI246737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54596[] = {SGI246738, SGI246739, SGI246740, SGI0, SGI246741, SGI246742, SGI246743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54597[] = {SGI246744, SGI246745, SGI246746, SGI0, SGI246747, SGI246748, SGI246749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54598[] = {SGI246750, SGI246751, SGI246752, SGI0, SGI246753, SGI246754, SGI246755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54599[] = {SGI246756, SGI246757, SGI246758, SGI0, SGI246759, SGI246760, SGI246761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54600[] = {SGI246762, SGI246763, SGI246764, SGI0, SGI246765, SGI246766, SGI246767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54601[] = {SGI246768, SGI246769, SGI246770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246771, SGI0, SGI0, SGI0, SGI246773, SGI246773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246777, SGI0, SGI0, NULL};
const Segment * const SG54602[] = {SGI246772, SGI246774, SGI246775, SGI0, SGI246776, SGI246778, SGI246779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54603[] = {SGI246780, SGI246781, SGI246782, SGI0, SGI246783, SGI246784, SGI246785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54604[] = {SGI246786, SGI246787, SGI246788, SGI0, SGI246789, SGI246790, SGI246791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54605[] = {SGI246792, SGI246793, SGI246794, SGI0, SGI246795, SGI246796, SGI246797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54606[] = {SGI246798, SGI246799, SGI246800, SGI0, SGI246801, SGI246802, SGI246803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54607[] = {SGI246804, SGI246805, SGI246806, SGI0, SGI246807, SGI246808, SGI246809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54608[] = {SGI246810, SGI0, SGI246811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246812, SGI246813, SGI246813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54609[] = {SGI246814, SGI246815, SGI246816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246817, SGI0, SGI0, SGI0, SGI246818, SGI246818, SGI0, SGI0, SGI0, SGI0, SGI246819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54610[] = {SGI246820, SGI0, SGI246821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246822, SGI246823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246824, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54611[] = {SGI246825, SGI0, SGI246826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246827, SGI246828, SGI246828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54612[] = {SGI246829, SGI0, SGI246830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246831, SGI246832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54613[] = {SGI246833, SGI0, SGI246834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246835, SGI246836, SGI246836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54614[] = {SGI246837, SGI0, SGI246838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246839, SGI246840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54615[] = {SGI246841, SGI0, SGI246842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246843, SGI246844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54616[] = {SGI246845, SGI246846, SGI246847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246848, SGI0, SGI0, SGI0, SGI246849, SGI246849, SGI0, SGI246850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54617[] = {SGI246851, SGI0, SGI246852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246853, SGI246854, SGI246854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54618[] = {SGI246855, SGI246856, SGI246857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246858, SGI0, SGI0, SGI0, SGI246859, SGI246859, SGI0, SGI246860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54619[] = {SGI246861, SGI0, SGI246862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246863, SGI246864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54620[] = {SGI246865, SGI0, SGI246866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246867, SGI246868, SGI246868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54621[] = {SGI246869, SGI246870, SGI246871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246872, SGI0, SGI0, SGI0, SGI246873, SGI0, SGI246874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54622[] = {SGI246875, SGI0, SGI246876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246877, SGI246878, SGI246878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54623[] = {SGI246879, SGI0, SGI246880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246881, SGI246882, SGI246882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54624[] = {SGI246883, SGI0, SGI246884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246885, SGI246886, SGI246886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54625[] = {SGI246887, SGI0, SGI246888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246889, SGI246890, SGI246890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246891, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54626[] = {SGI246892, SGI0, SGI246893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246894, SGI246895, SGI246895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54627[] = {SGI246896, SGI0, SGI246897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246900, SGI246901, SGI246901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54628[] = {SGI246898, SGI0, SGI246899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246902, SGI246903, SGI246903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54629[] = {SGI246904, SGI0, SGI246905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246906, SGI246907, SGI246907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54630[] = {SGI246908, SGI246909, SGI246910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246913, SGI0, SGI0, SGI0, SGI246914, SGI246914, SGI0, SGI246915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54631[] = {SGI246911, SGI0, SGI246912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246916, SGI246917, SGI246917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54632[] = {SGI246918, SGI0, SGI246919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246920, SGI246921, SGI246921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54633[] = {SGI246922, SGI0, SGI246923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246924, SGI246925, SGI246925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246926, SGI0, SGI0, NULL};
const Segment * const SG54634[] = {SGI246927, SGI246928, SGI246929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246930, SGI0, SGI0, SGI0, SGI246931, SGI246931, SGI0, SGI0, SGI0, SGI0, SGI246932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54635[] = {SGI246933, SGI0, SGI246934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246935, SGI246936, SGI246936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246937, SGI0, SGI0, NULL};
const Segment * const SG54636[] = {SGI246938, SGI0, SGI246939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246940, SGI246941, SGI246941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54637[] = {SGI246942, SGI0, SGI246943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246944, SGI246945, SGI246945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54638[] = {SGI246946, SGI0, SGI246947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246948, SGI246949, SGI246949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54639[] = {SGI246950, SGI246951, SGI246952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246953, SGI0, SGI0, SGI0, SGI246955, SGI246955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246958, NULL};
const Segment * const SG54640[] = {SGI246954, SGI246956, SGI246957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246959, SGI0, SGI0, SGI0, SGI246960, SGI246960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54641[] = {SGI246961, SGI0, SGI246962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246963, SGI246964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246965, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54642[] = {SGI246966, SGI0, SGI246967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246968, SGI246969, SGI246969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54643[] = {SGI246973, SGI0, SGI246974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246976, SGI246978, SGI246978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54644[] = {SGI246970, SGI246971, SGI246972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246975, SGI0, SGI0, SGI246977, SGI0, SGI246979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54645[] = {SGI246980, SGI246981, SGI246982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246983, SGI246983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246984, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54646[] = {SGI246985, SGI246986, SGI246987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246988, SGI0, SGI0, SGI0, SGI246989, SGI246989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246990, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54647[] = {SGI246991, SGI246992, SGI246993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246994, SGI246994, SGI0, SGI0, SGI0, SGI0, SGI246995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54648[] = {SGI246996, SGI0, SGI246997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI246998, SGI246999, SGI246999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247000, SGI0, SGI0, NULL};
const Segment * const SG54649[] = {SGI247001, SGI0, SGI247002, SGI0, SGI247003, SGI247004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54650[] = {SGI247005, SGI0, SGI247006, SGI0, SGI247007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54651[] = {SGI247008, SGI0, SGI247009, SGI0, SGI247010, SGI247011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54652[] = {SGI247012, SGI0, SGI247013, SGI0, SGI247014, SGI247015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54653[] = {SGI247016, SGI0, SGI247017, SGI0, SGI247018, SGI247019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54654[] = {SGI247020, SGI0, SGI0, SGI0, SGI247021, SGI247022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54655[] = {SGI247023, SGI0, SGI247024, SGI0, SGI247025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54656[] = {SGI247030, SGI0, SGI247031, SGI0, SGI247033, SGI247034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54657[] = {SGI247036, SGI0, SGI247037, SGI0, SGI247038, SGI247039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54658[] = {SGI247040, SGI0, SGI247041, SGI0, SGI247042, SGI247043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54659[] = {SGI247044, SGI0, SGI247045, SGI0, SGI247046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54660[] = {SGI247047, SGI0, SGI247048, SGI0, SGI247049, SGI247050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54661[] = {SGI247051, SGI0, SGI247052, SGI0, SGI247053, SGI247054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54662[] = {SGI247055, SGI0, SGI247056, SGI0, SGI247057, SGI247058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54663[] = {SGI247059, SGI0, SGI247060, SGI0, SGI247061, SGI247062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54664[] = {SGI247063, SGI0, SGI247064, SGI0, SGI247065, SGI247066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54665[] = {SGI247067, SGI0, SGI247068, SGI0, SGI247069, SGI247070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54666[] = {SGI247071, SGI0, SGI247072, SGI0, SGI247073, SGI247074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54667[] = {SGI247075, SGI0, SGI247076, SGI0, SGI247077, SGI247078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54668[] = {SGI247079, SGI0, SGI247080, SGI0, SGI247081, SGI247082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54669[] = {SGI247083, SGI0, SGI247084, SGI0, SGI247085, SGI247086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54670[] = {SGI247087, SGI0, SGI247088, SGI0, SGI247089, SGI247090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54671[] = {SGI247091, SGI0, SGI247092, SGI0, SGI247093, SGI247094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54672[] = {SGI247095, SGI0, SGI247096, SGI0, SGI0, SGI247097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54673[] = {SGI247098, SGI0, SGI247099, SGI0, SGI0, SGI247100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54674[] = {SGI247101, SGI0, SGI247102, SGI0, SGI0, SGI247103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54675[] = {SGI247104, SGI0, SGI247105, SGI0, SGI247106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54676[] = {SGI247107, SGI0, SGI247108, SGI0, SGI247109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54677[] = {SGI247110, SGI0, SGI247111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54678[] = {SGI247112, SGI0, SGI247113, SGI0, SGI247114, SGI247115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54679[] = {SGI247116, SGI0, SGI247117, SGI0, SGI247118, SGI247119, SGI247120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54680[] = {SGI247121, SGI0, SGI247122, SGI0, SGI247123, SGI247124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54681[] = {SGI247125, SGI0, SGI247126, SGI0, SGI247127, SGI247128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54682[] = {SGI247129, SGI0, SGI247130, SGI0, SGI0, SGI247131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54683[] = {SGI247132, SGI0, SGI247133, SGI0, SGI0, SGI247134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54684[] = {SGI247135, SGI0, SGI247136, SGI0, SGI0, SGI247137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54685[] = {SGI247138, SGI0, SGI247139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247140, SGI247141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54686[] = {SGI247142, SGI247143, SGI247144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247145, SGI0, SGI0, SGI0, SGI247146, SGI247146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54687[] = {SGI247147, SGI0, SGI247148, SGI0, SGI247149, SGI247150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54688[] = {SGI247151, SGI0, SGI247152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54689[] = {SGI247153, SGI0, SGI247154, SGI0, SGI247155, SGI247156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54690[] = {SGI247026, SGI247027, SGI247028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247029, SGI0, SGI0, SGI0, SGI247032, SGI247032, SGI0, SGI247035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54691[] = {SGI247159, SGI247160, SGI247161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247162, SGI0, SGI0, SGI0, SGI247163, SGI247163, SGI0, SGI247164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54692[] = {SGI247165, SGI247166, SGI247167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247168, SGI0, SGI0, SGI0, SGI247169, SGI247169, SGI0, SGI247170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54693[] = {SGI247171, SGI247172, SGI247173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247174, SGI0, SGI0, SGI0, SGI247175, SGI247175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54694[] = {SGI247176, SGI0, SGI247177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247178, SGI247179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54695[] = {SGI247157, SGI0, SGI247158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54696[] = {SGI247180, SGI247181, SGI247182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247183, SGI0, SGI0, SGI0, SGI247185, SGI247185, SGI247188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54697[] = {SGI247184, SGI0, SGI247186, SGI0, SGI247187, SGI247189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54698[] = {SGI247190, SGI0, SGI247191, SGI0, SGI247192, SGI247193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54699[] = {SGI247194, SGI0, SGI247196, SGI0, SGI247198, SGI247200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54700[] = {SGI247195, SGI247197, SGI247199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247201, SGI0, SGI0, SGI0, SGI247202, SGI0, SGI247203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54701[] = {SGI247204, SGI0, SGI247205, SGI0, SGI247206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54702[] = {SGI247207, SGI0, SGI247208, SGI0, SGI247209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54703[] = {SGI247210, SGI0, SGI247211, SGI0, SGI247212, SGI247213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54704[] = {SGI247214, SGI0, SGI247215, SGI0, SGI247216, SGI247217, SGI247218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247219, SGI0, NULL};
const Segment * const SG54705[] = {SGI247220, SGI0, SGI247221, SGI0, SGI247222, SGI247223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54706[] = {SGI247224, SGI0, SGI247225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247226, SGI247227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54707[] = {SGI247228, SGI0, SGI247229, SGI0, SGI247230, SGI247231, SGI247232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54708[] = {SGI247233, SGI0, SGI247234, SGI0, SGI247235, SGI247236, SGI247237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54709[] = {SGI247238, SGI0, SGI247239, SGI0, SGI247240, SGI247241, SGI247242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54710[] = {SGI247243, SGI0, SGI247244, SGI0, SGI247246, SGI247247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247249, SGI0, NULL};
const Segment * const SG54711[] = {SGI247245, SGI0, SGI247248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247250, SGI247251, SGI247251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54712[] = {SGI247252, SGI0, SGI247253, SGI0, SGI247254, SGI247255, SGI247256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54713[] = {SGI247257, SGI0, SGI247258, SGI0, SGI0, SGI247259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54714[] = {SGI247260, SGI0, SGI247261, SGI0, SGI247262, SGI247263, SGI247264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247265, SGI0, NULL};
const Segment * const SG54715[] = {SGI247266, SGI0, SGI247267, SGI0, SGI247268, SGI247269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54716[] = {SGI247270, SGI0, SGI247271, SGI0, SGI0, SGI247272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54717[] = {SGI247273, SGI0, SGI247274, SGI0, SGI247275, SGI247276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54718[] = {SGI247277, SGI0, SGI247278, SGI0, SGI247279, SGI247280, SGI247281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54719[] = {SGI247282, SGI0, SGI247283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247284, SGI247285, SGI247285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54720[] = {SGI247286, SGI0, SGI247287, SGI0, SGI247288, SGI247289, SGI247290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54721[] = {SGI247291, SGI0, SGI247292, SGI0, SGI247293, SGI247294, SGI247295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247296, SGI0, NULL};
const Segment * const SG54722[] = {SGI247297, SGI0, SGI247298, SGI0, SGI247299, SGI247300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54723[] = {SGI247301, SGI0, SGI247302, SGI0, SGI247303, SGI247305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54724[] = {SGI247304, SGI0, SGI247306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247310, SGI247311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247312, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54725[] = {SGI247307, SGI247308, SGI247309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247313, SGI247313, SGI0, SGI0, SGI0, SGI0, SGI247315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54726[] = {SGI247314, SGI0, SGI247316, SGI0, SGI247317, SGI247318, SGI247319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54727[] = {SGI247320, SGI0, SGI247321, SGI0, SGI247322, SGI247323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54728[] = {SGI247324, SGI247325, SGI247326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247327, SGI247327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54729[] = {SGI247328, SGI0, SGI247329, SGI0, SGI247330, SGI247331, SGI247332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54730[] = {SGI247333, SGI0, SGI247334, SGI0, SGI247335, SGI247336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54731[] = {SGI247340, SGI0, SGI247341, SGI0, SGI247342, SGI247343, SGI247342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54732[] = {SGI247337, SGI247338, SGI247339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247344, SGI0, SGI0, SGI0, SGI247345, SGI247345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247346, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54733[] = {SGI247347, SGI0, SGI247348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247349, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54734[] = {SGI247350, SGI0, SGI247351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54735[] = {SGI247352, SGI0, SGI247353, SGI0, SGI0, SGI247354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54736[] = {SGI247355, SGI0, SGI247356, SGI0, SGI247357, SGI247358, SGI247357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54737[] = {SGI247359, SGI0, SGI247360, SGI0, SGI247361, SGI247362, SGI247363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54738[] = {SGI247364, SGI247365, SGI247366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247367, SGI0, SGI0, SGI0, SGI247368, SGI247368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54739[] = {SGI247369, SGI247370, SGI247371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247372, SGI247372, SGI0, SGI0, SGI0, SGI0, SGI247373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54740[] = {SGI247374, SGI247375, SGI247376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247377, SGI0, SGI0, SGI0, SGI247378, SGI247378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54741[] = {SGI247379, SGI247380, SGI247381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247382, SGI247382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247383, NULL};
const Segment * const SG54742[] = {SGI247384, SGI247385, SGI247386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247387, SGI247387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247388, NULL};
const Segment * const SG54743[] = {SGI247389, SGI247390, SGI247391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247392, SGI247392, SGI0, SGI0, SGI0, SGI0, SGI247393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54744[] = {SGI247394, SGI247395, SGI247396, SGI0, SGI247397, SGI247398, SGI247399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54745[] = {SGI247400, SGI247401, SGI247402, SGI0, SGI247403, SGI247404, SGI247405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54746[] = {SGI247406, SGI247407, SGI247408, SGI0, SGI247409, SGI247410, SGI247411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54747[] = {SGI247415, SGI247416, SGI247417, SGI0, SGI247418, SGI247419, SGI247420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54748[] = {SGI247421, SGI247422, SGI247423, SGI0, SGI247424, SGI247425, SGI247426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54749[] = {SGI247427, SGI247428, SGI247429, SGI0, SGI247430, SGI247431, SGI247432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54750[] = {SGI247433, SGI247434, SGI247435, SGI0, SGI247436, SGI247437, SGI247438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54751[] = {SGI247439, SGI247440, SGI247441, SGI0, SGI247442, SGI247443, SGI247444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54752[] = {SGI247445, SGI247446, SGI247447, SGI0, SGI247448, SGI247449, SGI247450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54753[] = {SGI247451, SGI247452, SGI247453, SGI0, SGI247454, SGI247455, SGI247456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54754[] = {SGI247457, SGI247458, SGI247459, SGI0, SGI247460, SGI247461, SGI247462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54755[] = {SGI247412, SGI247413, SGI247414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247465, SGI0, SGI0, SGI0, SGI247466, SGI247466, SGI0, SGI247467, SGI0, SGI0, SGI0, SGI0, SGI247468, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54756[] = {SGI247469, SGI247470, SGI247471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247472, SGI0, SGI0, SGI0, SGI247473, SGI247474, SGI0, SGI0, SGI0, SGI0, SGI247475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54757[] = {SGI247476, SGI0, SGI247477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247478, SGI247479, SGI247479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54758[] = {SGI247480, SGI247481, SGI247482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247483, SGI0, SGI0, SGI0, SGI247484, SGI247484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247485, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54759[] = {SGI247486, SGI0, SGI247487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247488, SGI247489, SGI247489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54760[] = {SGI247490, SGI0, SGI247491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247492, SGI247493, SGI247493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54761[] = {SGI247494, SGI0, SGI247495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247496, SGI247497, SGI247497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54762[] = {SGI247498, SGI0, SGI247499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247500, SGI247501, SGI247501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54763[] = {SGI247502, SGI0, SGI247503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247504, SGI247505, SGI247505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54764[] = {SGI247506, SGI0, SGI247507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247508, SGI0, SGI0, SGI0, SGI247509, SGI247509, SGI0, SGI0, SGI0, SGI0, SGI247510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54765[] = {SGI247511, SGI0, SGI247512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247513, SGI247514, SGI247514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54766[] = {SGI247515, SGI247516, SGI247517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247518, SGI247518, SGI0, SGI0, SGI0, SGI0, SGI247519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54767[] = {SGI247520, SGI247521, SGI247522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247523, SGI0, SGI0, SGI0, SGI247524, SGI247524, SGI0, SGI0, SGI0, SGI247525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54768[] = {SGI247526, SGI0, SGI247527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247528, SGI247529, SGI247529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247530, SGI0, SGI0, NULL};
const Segment * const SG54769[] = {SGI247531, SGI0, SGI247532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54770[] = {SGI247533, SGI247534, SGI247535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247536, SGI0, SGI0, SGI0, SGI247537, SGI247538, SGI0, SGI0, SGI0, SGI0, SGI247539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54771[] = {SGI247540, SGI247541, SGI247542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247543, SGI247543, SGI0, SGI0, SGI0, SGI0, SGI247544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54772[] = {SGI247545, SGI247546, SGI247547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247548, SGI0, SGI0, SGI0, SGI247549, SGI247549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54773[] = {SGI247463, SGI247464, SGI247550, SGI0, SGI247551, SGI247552, SGI247553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54774[] = {SGI247554, SGI247555, SGI247556, SGI0, SGI247557, SGI247558, SGI247559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54775[] = {SGI247560, SGI247561, SGI0, SGI0, SGI247562, SGI247563, SGI247564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54776[] = {SGI247565, SGI247566, SGI247567, SGI0, SGI247568, SGI247569, SGI247570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54777[] = {SGI247574, SGI247575, SGI247576, SGI0, SGI247577, SGI247578, SGI247579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54778[] = {SGI247571, SGI247572, SGI247573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247580, SGI247580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247581, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54779[] = {SGI247582, SGI247583, SGI0, SGI0, SGI247584, SGI247585, SGI247586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54780[] = {SGI247587, SGI247588, SGI247589, SGI0, SGI247590, SGI247591, SGI247592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54781[] = {SGI247593, SGI247594, SGI247595, SGI0, SGI247596, SGI247597, SGI247598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54782[] = {SGI247599, SGI247600, SGI247601, SGI0, SGI247602, SGI247603, SGI247604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54783[] = {SGI247605, SGI247606, SGI247607, SGI0, SGI247608, SGI247609, SGI247610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54784[] = {SGI247611, SGI247612, SGI0, SGI0, SGI247613, SGI247614, SGI247615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54785[] = {SGI247616, SGI247617, SGI247618, SGI0, SGI247619, SGI247620, SGI247621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54786[] = {SGI247622, SGI247623, SGI247624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247628, SGI0, SGI0, SGI0, SGI247630, SGI247630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54787[] = {SGI247625, SGI247626, SGI247627, SGI0, SGI247629, SGI247631, SGI247632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54788[] = {SGI247633, SGI247634, SGI0, SGI0, SGI247635, SGI247636, SGI247637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54789[] = {SGI247638, SGI247639, SGI247640, SGI0, SGI247641, SGI247642, SGI247643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54790[] = {SGI247644, SGI247645, SGI247646, SGI0, SGI247647, SGI247648, SGI247649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54791[] = {SGI247650, SGI247651, SGI247652, SGI0, SGI247653, SGI247654, SGI247655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54792[] = {SGI247656, SGI247657, SGI247658, SGI0, SGI247659, SGI247660, SGI247661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54793[] = {SGI247662, SGI247663, SGI247664, SGI0, SGI247665, SGI247666, SGI247667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54794[] = {SGI247668, SGI247669, SGI247670, SGI0, SGI247671, SGI247672, SGI247673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54795[] = {SGI247674, SGI247675, SGI247676, SGI0, SGI247677, SGI247678, SGI247680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54796[] = {SGI247679, SGI0, SGI247681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247682, SGI247683, SGI247683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54797[] = {SGI247687, SGI247688, SGI247689, SGI0, SGI247690, SGI247691, SGI247692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54798[] = {SGI247684, SGI247685, SGI247686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247693, SGI247693, SGI0, SGI0, SGI0, SGI0, SGI247694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54799[] = {SGI247695, SGI247696, SGI247697, SGI0, SGI247698, SGI247699, SGI247700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54800[] = {SGI247701, SGI0, SGI247702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247706, SGI247707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54801[] = {SGI247703, SGI247704, SGI247705, SGI0, SGI247708, SGI247709, SGI247710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54802[] = {SGI247711, SGI247712, SGI247713, SGI0, SGI247714, SGI247715, SGI247716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54803[] = {SGI247717, SGI247718, SGI0, SGI0, SGI247719, SGI247720, SGI247721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54804[] = {SGI247722, SGI0, SGI247723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247724, SGI247725, SGI247725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54805[] = {SGI247726, SGI247727, SGI247728, SGI0, SGI247729, SGI247730, SGI247731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54806[] = {SGI247732, SGI247733, SGI247734, SGI0, SGI247735, SGI247736, SGI247737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54807[] = {SGI247738, SGI247739, SGI247740, SGI0, SGI247741, SGI247742, SGI247743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54808[] = {SGI247744, SGI247745, SGI247746, SGI0, SGI247747, SGI247748, SGI247749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54809[] = {SGI247750, SGI247751, SGI247752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247753, SGI247753, SGI0, SGI0, SGI0, SGI0, SGI247756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54810[] = {SGI247754, SGI247755, SGI0, SGI0, SGI247757, SGI247758, SGI247759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54811[] = {SGI247760, SGI247761, SGI247762, SGI0, SGI247763, SGI247764, SGI247765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54812[] = {SGI247766, SGI247767, SGI247768, SGI0, SGI247769, SGI247771, SGI247772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54813[] = {SGI247770, SGI0, SGI247773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247774, SGI247775, SGI247775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54814[] = {SGI247779, SGI247780, SGI247781, SGI0, SGI247782, SGI247783, SGI247785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54815[] = {SGI247776, SGI247777, SGI247778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247784, SGI247784, SGI0, SGI0, SGI0, SGI0, SGI247786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54816[] = {SGI247787, SGI247788, SGI247789, SGI0, SGI247791, SGI247793, SGI247794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54817[] = {SGI247790, SGI0, SGI247792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247795, SGI247796, SGI247796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54818[] = {SGI247797, SGI247798, SGI247799, SGI0, SGI247800, SGI247801, SGI247802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54819[] = {SGI247803, SGI0, SGI247804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247805, SGI247806, SGI247806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54820[] = {SGI247807, SGI247808, SGI247809, SGI0, SGI247810, SGI247811, SGI247812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54821[] = {SGI247813, SGI247814, SGI247815, SGI0, SGI247816, SGI247817, SGI247818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54822[] = {SGI247819, SGI0, SGI247820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247821, SGI247822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54823[] = {SGI247823, SGI0, SGI247824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247825, SGI247826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54824[] = {SGI247827, SGI0, SGI247828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247829, SGI247830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54825[] = {SGI247831, SGI0, SGI247832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247833, SGI247834, SGI247834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54826[] = {SGI247838, SGI247839, SGI247840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247841, SGI0, SGI0, SGI0, SGI247842, SGI0, SGI247843, SGI247844, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54827[] = {SGI247845, SGI247846, SGI247847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247848, SGI0, SGI0, SGI0, SGI247849, SGI0, SGI247850, SGI247851, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54828[] = {SGI247852, SGI0, SGI247853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247854, SGI247855, SGI247855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54829[] = {SGI247835, SGI0, SGI247836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247837, SGI247856, SGI247856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54830[] = {SGI247857, SGI0, SGI247858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247859, SGI247860, SGI247860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54831[] = {SGI247861, SGI0, SGI247862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247863, SGI247864, SGI247864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54832[] = {SGI247865, SGI247866, SGI247867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247868, SGI0, SGI0, SGI0, SGI247869, SGI247869, SGI0, SGI0, SGI247870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54833[] = {SGI247871, SGI0, SGI247872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI247873, SGI247874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54834[] = {SGI247875, SGI247876, SGI247877, SGI0, SGI247878, SGI247879, SGI247880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54835[] = {SGI247881, SGI247882, SGI247883, SGI0, SGI247884, SGI247885, SGI247886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54836[] = {SGI247887, SGI247888, SGI247889, SGI0, SGI247890, SGI247891, SGI247892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54837[] = {SGI247893, SGI247894, SGI247895, SGI0, SGI247896, SGI247897, SGI247898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54838[] = {SGI247899, SGI247900, SGI247901, SGI0, SGI247902, SGI247903, SGI247904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54839[] = {SGI247905, SGI247906, SGI247907, SGI0, SGI247908, SGI247909, SGI247910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54840[] = {SGI247911, SGI247912, SGI247913, SGI0, SGI247914, SGI247915, SGI247916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54841[] = {SGI247917, SGI247918, SGI247919, SGI0, SGI247920, SGI247921, SGI247922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54842[] = {SGI247923, SGI247924, SGI247925, SGI0, SGI247926, SGI247927, SGI247928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54843[] = {SGI247929, SGI247930, SGI247931, SGI0, SGI247932, SGI247933, SGI247934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54844[] = {SGI247935, SGI247936, SGI247937, SGI0, SGI247938, SGI247939, SGI247940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54845[] = {SGI247941, SGI247942, SGI247943, SGI0, SGI247944, SGI247945, SGI247946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54846[] = {SGI247947, SGI247948, SGI247949, SGI0, SGI247950, SGI247951, SGI247952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54847[] = {SGI247953, SGI247954, SGI247955, SGI0, SGI247956, SGI247957, SGI247958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54848[] = {SGI247959, SGI247960, SGI247961, SGI0, SGI247962, SGI247963, SGI247964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54849[] = {SGI247965, SGI247966, SGI247967, SGI0, SGI247968, SGI247969, SGI247970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54850[] = {SGI247971, SGI247972, SGI247973, SGI0, SGI247974, SGI247975, SGI247976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54851[] = {SGI247977, SGI247978, SGI247979, SGI0, SGI247980, SGI247981, SGI247982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54852[] = {SGI247983, SGI0, SGI247984, SGI0, SGI0, SGI247985, SGI247986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54853[] = {SGI247987, SGI247988, SGI247989, SGI0, SGI247990, SGI247991, SGI247992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54854[] = {SGI247993, SGI247994, SGI247995, SGI0, SGI247996, SGI247997, SGI247998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54855[] = {SGI247999, SGI248000, SGI248001, SGI0, SGI248002, SGI248003, SGI248004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54856[] = {SGI248005, SGI0, SGI248006, SGI0, SGI0, SGI248007, SGI248008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54857[] = {SGI248009, SGI248010, SGI248011, SGI0, SGI248012, SGI248013, SGI248014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54858[] = {SGI248015, SGI248016, SGI248017, SGI0, SGI248018, SGI248019, SGI248020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54859[] = {SGI248021, SGI248022, SGI248023, SGI0, SGI248024, SGI248025, SGI248026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54860[] = {SGI248027, SGI248028, SGI248029, SGI0, SGI248030, SGI248031, SGI248032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54861[] = {SGI248033, SGI248034, SGI248035, SGI0, SGI248036, SGI248037, SGI248038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54862[] = {SGI248039, SGI248040, SGI248041, SGI0, SGI248042, SGI248043, SGI248044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54863[] = {SGI248045, SGI248046, SGI248047, SGI0, SGI248048, SGI248049, SGI248050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54864[] = {SGI248051, SGI248052, SGI248053, SGI0, SGI248054, SGI248055, SGI248056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54865[] = {SGI248057, SGI248058, SGI248059, SGI0, SGI248060, SGI248061, SGI248062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54866[] = {SGI248063, SGI248064, SGI248065, SGI0, SGI248066, SGI248067, SGI248068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54867[] = {SGI248069, SGI248070, SGI248071, SGI0, SGI248072, SGI248073, SGI248074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54868[] = {SGI248075, SGI248076, SGI248077, SGI0, SGI248078, SGI248079, SGI248080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54869[] = {SGI248081, SGI248082, SGI248083, SGI0, SGI248084, SGI248085, SGI248086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54870[] = {SGI248087, SGI248088, SGI248089, SGI0, SGI248090, SGI248091, SGI248092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54871[] = {SGI248093, SGI248094, SGI248095, SGI0, SGI248096, SGI248097, SGI248098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54872[] = {SGI248099, SGI248100, SGI248101, SGI0, SGI248102, SGI248103, SGI248104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54873[] = {SGI248105, SGI0, SGI248106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248107, SGI248108, SGI248108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54874[] = {SGI248109, SGI248110, SGI248111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248112, SGI0, SGI0, SGI0, SGI248113, SGI0, SGI248114, SGI248115, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54875[] = {SGI248116, SGI0, SGI248117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248118, SGI0, SGI0, SGI0, SGI248119, SGI248119, SGI0, SGI0, SGI0, SGI0, SGI248120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54876[] = {SGI248121, SGI0, SGI248122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248123, SGI248124, SGI248124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54877[] = {SGI248125, SGI0, SGI248126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248127, SGI248128, SGI248128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54878[] = {SGI248129, SGI0, SGI248130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248131, SGI248132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54879[] = {SGI248133, SGI0, SGI248134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248135, SGI248136, SGI248136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248137, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54880[] = {SGI248138, SGI0, SGI248139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248140, SGI248141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54881[] = {SGI248142, SGI0, SGI248143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248144, SGI248145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54882[] = {SGI248146, SGI0, SGI248147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248148, SGI248149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248150, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54883[] = {SGI248151, SGI248152, SGI248153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248154, SGI248154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248155, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54884[] = {SGI248156, SGI0, SGI248157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248158, SGI248159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54885[] = {SGI248160, SGI248161, SGI248162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248163, SGI0, SGI0, SGI0, SGI248164, SGI0, SGI248165, SGI248166, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54886[] = {SGI248167, SGI248168, SGI248169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248170, SGI0, SGI0, SGI0, SGI248171, SGI0, SGI248172, SGI248173, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54887[] = {SGI248174, SGI0, SGI248175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248176, SGI248177, SGI248177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54888[] = {SGI248178, SGI248179, SGI248180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248181, SGI0, SGI0, SGI0, SGI248182, SGI248182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54889[] = {SGI248183, SGI248184, SGI248185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248186, SGI248186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248187, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54890[] = {SGI248188, SGI248189, SGI248190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248191, SGI0, SGI0, SGI0, SGI248192, SGI248192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54891[] = {SGI248193, SGI0, SGI248194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248195, SGI248196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54892[] = {SGI248197, SGI0, SGI248198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248199, SGI248200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54893[] = {SGI248201, SGI0, SGI248202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248203, SGI248204, SGI248204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54894[] = {SGI248205, SGI248206, SGI248207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248208, SGI248208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248209, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54895[] = {SGI248210, SGI0, SGI248211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248212, SGI248213, SGI248213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54896[] = {SGI248214, SGI248215, SGI248216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248217, SGI0, SGI0, SGI0, SGI248218, SGI248218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54897[] = {SGI248219, SGI248220, SGI248221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248222, SGI248222, SGI0, SGI0, SGI0, SGI0, SGI248223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54898[] = {SGI248224, SGI0, SGI248225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248226, SGI248227, SGI248227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54899[] = {SGI248228, SGI248229, SGI248230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248231, SGI0, SGI0, SGI0, SGI248232, SGI248232, SGI248233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54900[] = {SGI248234, SGI0, SGI248235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248236, SGI248237, SGI248237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54901[] = {SGI248238, SGI248239, SGI248240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248241, SGI0, SGI0, SGI0, SGI248242, SGI248242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54902[] = {SGI248243, SGI0, SGI248244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248245, SGI248246, SGI248246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54903[] = {SGI248247, SGI0, SGI248248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248249, SGI248250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248251, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54904[] = {SGI248252, SGI0, SGI248253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248254, SGI248255, SGI248255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248256, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54905[] = {SGI248257, SGI0, SGI248258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248259, SGI248260, SGI248260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54906[] = {SGI248261, SGI0, SGI248262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248263, SGI248264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54907[] = {SGI248265, SGI248266, SGI248267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248268, SGI0, SGI0, SGI0, SGI248269, SGI0, SGI248270, SGI248271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54908[] = {SGI248272, SGI0, SGI248273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248274, SGI248275, SGI248275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54909[] = {SGI248276, SGI0, SGI248277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248278, SGI248279, SGI248279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54910[] = {SGI248280, SGI0, SGI248281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248282, SGI248283, SGI248283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54911[] = {SGI248284, SGI0, SGI248285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248286, SGI248287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54912[] = {SGI248288, SGI0, SGI248289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248290, SGI248291, SGI248291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54913[] = {SGI248292, SGI248293, SGI248294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248296, SGI0, SGI0, SGI0, SGI248299, SGI248299, SGI0, SGI248300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54914[] = {SGI248295, SGI248297, SGI248298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248301, SGI0, SGI0, SGI0, SGI248302, SGI0, SGI248303, SGI248304, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54915[] = {SGI248305, SGI0, SGI248306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248307, SGI248308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248309, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54916[] = {SGI248310, SGI0, SGI248311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248312, SGI248313, SGI248313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54917[] = {SGI248314, SGI248315, SGI248316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248317, SGI0, SGI0, SGI0, SGI248318, SGI248318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54918[] = {SGI248319, SGI0, SGI248320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248321, SGI248322, SGI248322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54919[] = {SGI248323, SGI0, SGI248324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248325, SGI248326, SGI248326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54920[] = {SGI248327, SGI0, SGI248328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248329, SGI248330, SGI248330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54921[] = {SGI248331, SGI0, SGI248332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248333, SGI248334, SGI248334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54922[] = {SGI248335, SGI0, SGI248336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248337, SGI248338, SGI248338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248339, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54923[] = {SGI248340, SGI248341, SGI248342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248343, SGI0, SGI0, SGI0, SGI248344, SGI248344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248345, NULL};
const Segment * const SG54924[] = {SGI248346, SGI0, SGI248347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248348, SGI248349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54925[] = {SGI248350, SGI248351, SGI248352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248353, SGI0, SGI0, SGI0, SGI248354, SGI0, SGI0, SGI248355, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54926[] = {SGI248356, SGI0, SGI248357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248358, SGI248359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54927[] = {SGI248360, SGI0, SGI248361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248362, SGI248363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248364, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54928[] = {SGI248365, SGI248366, SGI248367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248368, SGI0, SGI0, SGI0, SGI248369, SGI248369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54929[] = {SGI248370, SGI248371, SGI248372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248373, SGI0, SGI0, SGI0, SGI248374, SGI248374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248375, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54930[] = {SGI248376, SGI0, SGI248377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248378, SGI248379, SGI248379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54931[] = {SGI248380, SGI0, SGI248381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248383, SGI248384, SGI248384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54932[] = {SGI248382, SGI248385, SGI248386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248387, SGI0, SGI0, SGI0, SGI248388, SGI0, SGI248389, SGI248390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54933[] = {SGI248391, SGI248392, SGI248393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248394, SGI0, SGI0, SGI0, SGI248395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248396, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54934[] = {SGI248397, SGI248398, SGI248399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248400, SGI0, SGI0, SGI0, SGI248401, SGI248401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248402, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54935[] = {SGI248403, SGI248404, SGI248405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248406, SGI0, SGI0, SGI0, SGI248407, SGI248407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248408, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54936[] = {SGI248409, SGI248410, SGI248411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248412, SGI0, SGI0, SGI0, SGI248413, SGI248413, SGI248414, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54937[] = {SGI248415, SGI0, SGI248416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248417, SGI248418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54938[] = {SGI248419, SGI248420, SGI248421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248423, SGI0, SGI0, SGI0, SGI248425, SGI248425, SGI0, SGI248426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54939[] = {SGI248422, SGI0, SGI248424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248427, SGI248428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54940[] = {SGI248429, SGI0, SGI248430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248431, SGI248432, SGI248432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54941[] = {SGI248433, SGI0, SGI248434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248435, SGI248436, SGI248436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54942[] = {SGI248437, SGI0, SGI248438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248439, SGI248440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54943[] = {SGI248441, SGI0, SGI248442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248443, SGI248444, SGI248444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54944[] = {SGI248445, SGI0, SGI248446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248447, SGI248448, SGI248448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54945[] = {SGI248449, SGI248450, SGI248451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248452, SGI0, SGI0, SGI0, SGI248453, SGI248453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54946[] = {SGI248454, SGI248455, SGI248456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248457, SGI248457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54947[] = {SGI248458, SGI0, SGI248459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248460, SGI248461, SGI248461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248462, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54948[] = {SGI248463, SGI248464, SGI248465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248466, SGI0, SGI0, SGI0, SGI248467, SGI248467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54949[] = {SGI248468, SGI0, SGI248469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248470, SGI248471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54950[] = {SGI248472, SGI0, SGI248473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248474, SGI248475, SGI248475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54951[] = {SGI248476, SGI0, SGI248477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248478, SGI248479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54952[] = {SGI248480, SGI248481, SGI248482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248483, SGI0, SGI0, SGI0, SGI248484, SGI248484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248485, SGI0, SGI0, NULL};
const Segment * const SG54953[] = {SGI248486, SGI0, SGI248487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248488, SGI248489, SGI248489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54954[] = {SGI248490, SGI0, SGI248491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248492, SGI248493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54955[] = {SGI248494, SGI248495, SGI248496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248497, SGI0, SGI0, SGI0, SGI248498, SGI0, SGI248499, SGI248500, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54956[] = {SGI248501, SGI0, SGI248502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54957[] = {SGI248503, SGI0, SGI248504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248505, SGI248506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54958[] = {SGI248507, SGI0, SGI248508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248509, SGI248510, SGI248510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54959[] = {SGI248511, SGI0, SGI248512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248513, SGI248514, SGI248514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54960[] = {SGI248515, SGI248516, SGI248517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248518, SGI0, SGI0, SGI0, SGI248519, SGI248519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248520, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54961[] = {SGI248521, SGI248522, SGI248523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248524, SGI0, SGI0, SGI0, SGI248525, SGI248525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54962[] = {SGI248526, SGI248527, SGI248528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248529, SGI0, SGI0, SGI0, SGI248530, SGI248530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54963[] = {SGI248531, SGI248532, SGI248533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248534, SGI248534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248535, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54964[] = {SGI248536, SGI0, SGI248537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248538, SGI248539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54965[] = {SGI248540, SGI248541, SGI248542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248543, SGI0, SGI0, SGI0, SGI248544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54966[] = {SGI248545, SGI0, SGI248546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248547, SGI248548, SGI248548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54967[] = {SGI248549, SGI0, SGI248550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54968[] = {SGI248551, SGI0, SGI248552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248553, SGI248554, SGI248554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54969[] = {SGI248555, SGI248556, SGI248557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248558, SGI0, SGI0, SGI0, SGI248559, SGI0, SGI248560, SGI248561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54970[] = {SGI248562, SGI0, SGI248563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248564, SGI248565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54971[] = {SGI248566, SGI248567, SGI248568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248569, SGI0, SGI0, SGI0, SGI248570, SGI248570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54972[] = {SGI248571, SGI248572, SGI248573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248574, SGI0, SGI0, SGI0, SGI248575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248576, SGI0, SGI0, NULL};
const Segment * const SG54973[] = {SGI248577, SGI0, SGI248578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248579, SGI248580, SGI248580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54974[] = {SGI248581, SGI248582, SGI248583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248584, SGI0, SGI0, SGI0, SGI248585, SGI248585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54975[] = {SGI248586, SGI248587, SGI248588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248589, SGI0, SGI0, SGI0, SGI248590, SGI248590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54976[] = {SGI248591, SGI248592, SGI248593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248594, SGI248594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248595, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54977[] = {SGI248596, SGI248597, SGI248598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248599, SGI0, SGI0, SGI0, SGI248600, SGI248600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54978[] = {SGI248601, SGI248602, SGI248603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248604, SGI248604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248605, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54979[] = {SGI248606, SGI248607, SGI248608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248609, SGI0, SGI0, SGI248610, SGI248611, SGI248611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54980[] = {SGI248612, SGI248613, SGI248614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248615, SGI248615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248616, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54981[] = {SGI248617, SGI0, SGI248618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248619, SGI248620, SGI248620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54982[] = {SGI248621, SGI248622, SGI248623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248624, SGI0, SGI0, SGI0, SGI248625, SGI248625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54983[] = {SGI248626, SGI0, SGI248627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248628, SGI248629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54984[] = {SGI248630, SGI248631, SGI248632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248633, SGI0, SGI0, SGI0, SGI248634, SGI248634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54985[] = {SGI248635, SGI0, SGI248636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248637, SGI248638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54986[] = {SGI248639, SGI0, SGI248640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248641, SGI248642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54987[] = {SGI248643, SGI248644, SGI248645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248646, SGI0, SGI0, SGI0, SGI248647, SGI248647, SGI248648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54988[] = {SGI248649, SGI0, SGI248650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54989[] = {SGI248651, SGI0, SGI248652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248653, SGI248654, SGI248654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54990[] = {SGI248655, SGI0, SGI248656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248657, SGI248658, SGI248658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248659, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54991[] = {SGI248660, SGI248661, SGI248662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248663, SGI0, SGI0, SGI0, SGI248664, SGI248664, SGI0, SGI0, SGI248665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54992[] = {SGI248666, SGI248667, SGI248668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248669, SGI0, SGI0, SGI0, SGI248670, SGI248670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54993[] = {SGI248671, SGI0, SGI248672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248673, SGI248674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54994[] = {SGI248675, SGI248676, SGI248677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248678, SGI0, SGI0, SGI0, SGI248679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248680, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54995[] = {SGI248681, SGI248682, SGI248683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248687, SGI0, SGI0, SGI0, SGI248688, SGI0, SGI0, SGI0, SGI0, SGI248689, SGI0, SGI0, NULL};
const Segment * const SG54996[] = {SGI248684, SGI248685, SGI248686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248690, SGI248690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248691, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54997[] = {SGI248692, SGI248693, SGI248694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248695, SGI0, SGI0, SGI0, SGI248696, SGI248696, SGI0, SGI0, SGI0, SGI248697, SGI0, SGI0, NULL};
const Segment * const SG54998[] = {SGI248698, SGI248699, SGI248700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248701, SGI0, SGI0, SGI0, SGI248702, SGI248702, SGI248703, SGI248704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG54999[] = {SGI248705, SGI248706, SGI248707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248708, SGI0, SGI0, SGI0, SGI248709, SGI0, SGI248710, SGI248711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55000[] = {SGI248712, SGI0, SGI248713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248714, SGI248715, SGI248715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55001[] = {SGI248716, SGI248717, SGI248718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248719, SGI0, SGI0, SGI0, SGI248720, SGI0, SGI0, SGI248721, SGI0, SGI248722, SGI0, SGI0, NULL};
const Segment * const SG55002[] = {SGI248723, SGI248724, SGI248725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248726, SGI0, SGI0, SGI0, SGI248727, SGI0, SGI248728, SGI248729, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55003[] = {SGI248730, SGI0, SGI248731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248732, SGI248733, SGI248733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55004[] = {SGI248734, SGI0, SGI248735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248736, SGI248737, SGI248737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248738, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55005[] = {SGI248739, SGI248740, SGI248741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248742, SGI0, SGI0, SGI0, SGI248743, SGI248743, SGI248744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55006[] = {SGI248745, SGI248746, SGI248747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248748, SGI0, SGI0, SGI0, SGI248749, SGI248749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55007[] = {SGI248750, SGI0, SGI248751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248752, SGI248753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55008[] = {SGI248754, SGI248755, SGI248756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248757, SGI0, SGI0, SGI0, SGI248758, SGI248758, SGI0, SGI0, SGI248759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55009[] = {SGI248760, SGI248761, SGI248762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248763, SGI248763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55010[] = {SGI248764, SGI248765, SGI248766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248767, SGI0, SGI0, SGI0, SGI248768, SGI248768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55011[] = {SGI248769, SGI0, SGI248770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248771, SGI248772, SGI248772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55012[] = {SGI248773, SGI0, SGI248774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248775, SGI248776, SGI248776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55013[] = {SGI0, SGI248777, SGI0, SGI248778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55014[] = {SGI248779, SGI0, SGI248780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248781, SGI0, SGI0, SGI0, SGI248782, SGI248782, SGI0, SGI0, SGI0, SGI0, SGI248783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55015[] = {SGI248784, SGI248785, SGI248786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248787, SGI0, SGI0, SGI0, SGI248788, SGI0, SGI248789, SGI248790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55016[] = {SGI248791, SGI248792, SGI248793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248794, SGI0, SGI0, SGI0, SGI248795, SGI0, SGI248796, SGI248797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55017[] = {SGI248798, SGI248799, SGI248800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248801, SGI0, SGI0, SGI0, SGI248802, SGI0, SGI248803, SGI248804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55018[] = {SGI248805, SGI248806, SGI248807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248808, SGI0, SGI0, SGI0, SGI248809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55019[] = {SGI248810, SGI0, SGI248811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248812, SGI248813, SGI248813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55020[] = {SGI248814, SGI0, SGI248815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248816, SGI248817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55021[] = {SGI248818, SGI0, SGI248819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248820, SGI248821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55022[] = {SGI248822, SGI0, SGI248823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248824, SGI248825, SGI248825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55023[] = {SGI248826, SGI248827, SGI248828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248829, SGI0, SGI0, SGI0, SGI248830, SGI0, SGI248831, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55024[] = {SGI0, SGI248832, SGI0, SGI248833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55025[] = {SGI248834, SGI248835, SGI248836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248837, SGI0, SGI0, SGI0, SGI248838, SGI0, SGI0, SGI248839, SGI248840, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55026[] = {SGI248841, SGI248842, SGI248843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248844, SGI248844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248845, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55027[] = {SGI248846, SGI248847, SGI248848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248849, SGI0, SGI0, SGI0, SGI248850, SGI248850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248851, SGI0, SGI0, NULL};
const Segment * const SG55028[] = {SGI248852, SGI0, SGI248853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248854, SGI248855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55029[] = {SGI248856, SGI248857, SGI248858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248859, SGI0, SGI0, SGI0, SGI248860, SGI248860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55030[] = {SGI248861, SGI248862, SGI248863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248864, SGI0, SGI0, SGI0, SGI248865, SGI248865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55031[] = {SGI248866, SGI0, SGI248867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248868, SGI248869, SGI248869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55032[] = {SGI248870, SGI248871, SGI248872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248873, SGI0, SGI0, SGI0, SGI248874, SGI248874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55033[] = {SGI248875, SGI248876, SGI248877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248878, SGI0, SGI0, SGI0, SGI248879, SGI248879, SGI248880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55034[] = {SGI248881, SGI0, SGI248882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248883, SGI248884, SGI248884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55035[] = {SGI248885, SGI0, SGI248886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248887, SGI248888, SGI248888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55036[] = {SGI248889, SGI248890, SGI248891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248892, SGI0, SGI0, SGI0, SGI248893, SGI0, SGI0, SGI248894, SGI248895, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55037[] = {SGI248896, SGI0, SGI248897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248898, SGI248899, SGI248899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55038[] = {SGI248900, SGI0, SGI248901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248902, SGI0, SGI0, SGI0, SGI248903, SGI248903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248904, NULL};
const Segment * const SG55039[] = {SGI248905, SGI248906, SGI248907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248908, SGI0, SGI0, SGI0, SGI248909, SGI0, SGI0, SGI248910, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55040[] = {SGI248911, SGI0, SGI248912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248913, SGI248914, SGI248914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55041[] = {SGI248915, SGI248916, SGI248917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248918, SGI0, SGI0, SGI0, SGI248919, SGI248919, SGI0, SGI0, SGI248920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55042[] = {SGI248921, SGI0, SGI248922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248923, SGI248924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248925, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55043[] = {SGI248926, SGI0, SGI248927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248928, SGI248929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55044[] = {SGI248930, SGI248931, SGI248932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248933, SGI0, SGI0, SGI0, SGI248934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248935, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55045[] = {SGI248936, SGI0, SGI248937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248938, SGI248939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55046[] = {SGI248940, SGI0, SGI248941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248942, SGI248943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55047[] = {SGI248944, SGI248945, SGI248946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248947, SGI0, SGI0, SGI0, SGI248948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248949, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55048[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248950, SGI0, SGI0, SGI248951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55049[] = {SGI248952, SGI0, SGI248953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248954, SGI248955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55050[] = {SGI248956, SGI248957, SGI248958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248959, SGI0, SGI0, SGI0, SGI248960, SGI248960, SGI0, SGI248961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55051[] = {SGI248962, SGI248963, SGI248964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248965, SGI0, SGI0, SGI0, SGI248966, SGI248966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55052[] = {SGI248967, SGI248968, SGI248969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248970, SGI0, SGI0, SGI0, SGI248971, SGI248971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55053[] = {SGI248972, SGI248973, SGI248974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248975, SGI0, SGI0, SGI0, SGI248976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248977, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55054[] = {SGI248978, SGI248979, SGI248980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248981, SGI248981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248982, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55055[] = {SGI248983, SGI248984, SGI248985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248986, SGI0, SGI0, SGI0, SGI248987, SGI0, SGI248988, SGI248989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55056[] = {SGI248990, SGI0, SGI248991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248992, SGI248993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248994, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55057[] = {SGI248995, SGI248996, SGI248997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI248998, SGI248998, SGI0, SGI0, SGI0, SGI0, SGI248999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55058[] = {SGI249000, SGI0, SGI249001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249002, SGI249003, SGI249003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55059[] = {SGI249004, SGI0, SGI249005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249006, SGI249007, SGI249007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55060[] = {SGI249008, SGI0, SGI249009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249010, SGI249011, SGI249011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55061[] = {SGI249012, SGI0, SGI249013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249014, SGI249015, SGI249015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55062[] = {SGI249016, SGI0, SGI249017, SGI0, SGI249018, SGI249019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55063[] = {SGI249020, SGI0, SGI249021, SGI0, SGI249022, SGI249023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55064[] = {SGI249024, SGI0, SGI249025, SGI0, SGI249026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55065[] = {SGI249027, SGI0, SGI249028, SGI0, SGI249029, SGI249030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55066[] = {SGI249031, SGI0, SGI249032, SGI0, SGI249033, SGI249034, SGI249033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55067[] = {SGI249035, SGI0, SGI249036, SGI0, SGI249037, SGI249038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55068[] = {SGI249039, SGI0, SGI249040, SGI0, SGI249041, SGI249042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55069[] = {SGI249043, SGI0, SGI249044, SGI0, SGI249045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55070[] = {SGI249046, SGI0, SGI249047, SGI0, SGI249048, SGI249049, SGI249048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55071[] = {SGI249050, SGI0, SGI249051, SGI0, SGI249052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55072[] = {SGI249053, SGI0, SGI249054, SGI0, SGI249055, SGI249056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55073[] = {SGI249057, SGI0, SGI249058, SGI0, SGI249059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55074[] = {SGI249060, SGI0, SGI249061, SGI0, SGI249062, SGI249063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55075[] = {SGI249064, SGI0, SGI249065, SGI0, SGI249066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55076[] = {SGI249067, SGI0, SGI249068, SGI0, SGI249069, SGI249070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55077[] = {SGI249071, SGI0, SGI249072, SGI0, SGI249073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55078[] = {SGI249074, SGI0, SGI0, SGI0, SGI249075, SGI249076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55079[] = {SGI249077, SGI0, SGI249078, SGI0, SGI249079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55080[] = {SGI249080, SGI0, SGI249081, SGI0, SGI249082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55081[] = {SGI249083, SGI0, SGI249084, SGI0, SGI249085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55082[] = {SGI249086, SGI0, SGI249087, SGI0, SGI249088, SGI249089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55083[] = {SGI249090, SGI0, SGI249091, SGI0, SGI249092, SGI249093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55084[] = {SGI249094, SGI0, SGI249095, SGI0, SGI249096, SGI249097, SGI249096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55085[] = {SGI249098, SGI0, SGI249099, SGI0, SGI249100, SGI249101, SGI249100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55086[] = {SGI249102, SGI0, SGI249103, SGI0, SGI249104, SGI249105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55087[] = {SGI249106, SGI0, SGI249107, SGI0, SGI249108, SGI249109, SGI249108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55088[] = {SGI249110, SGI0, SGI249111, SGI0, SGI249112, SGI249113, SGI249112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55089[] = {SGI249114, SGI0, SGI249115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249116, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55090[] = {SGI249117, SGI0, SGI249118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249119, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55091[] = {SGI249120, SGI0, SGI249121, SGI0, SGI249122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55092[] = {SGI249123, SGI0, SGI249124, SGI0, SGI249125, SGI249126, SGI249125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55093[] = {SGI249127, SGI0, SGI249128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55094[] = {SGI249129, SGI0, SGI249130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55095[] = {SGI249131, SGI0, SGI249132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55096[] = {SGI249133, SGI0, SGI249134, SGI0, SGI249135, SGI249136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55097[] = {SGI249137, SGI0, SGI249138, SGI0, SGI249139, SGI249140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55098[] = {SGI249141, SGI0, SGI249142, SGI0, SGI249143, SGI249144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55099[] = {SGI249145, SGI0, SGI249146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55100[] = {SGI249147, SGI0, SGI249148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55101[] = {SGI249149, SGI0, SGI249150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55102[] = {SGI249151, SGI0, SGI249152, SGI0, SGI249153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55103[] = {SGI249154, SGI249155, SGI249156, SGI0, SGI249157, SGI249158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55104[] = {SGI249159, SGI0, SGI249160, SGI0, SGI249161, SGI249162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55105[] = {SGI249163, SGI0, SGI0, SGI0, SGI249164, SGI249165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55106[] = {SGI249166, SGI0, SGI249167, SGI0, SGI249168, SGI249169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55107[] = {SGI249170, SGI0, SGI249171, SGI0, SGI249172, SGI249173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55108[] = {SGI249174, SGI0, SGI249175, SGI0, SGI249176, SGI249177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55109[] = {SGI249178, SGI0, SGI0, SGI0, SGI249179, SGI249180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55110[] = {SGI249181, SGI0, SGI249182, SGI0, SGI249183, SGI249184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55111[] = {SGI249185, SGI0, SGI249186, SGI0, SGI249187, SGI249188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55112[] = {SGI249189, SGI0, SGI249190, SGI0, SGI249191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55113[] = {SGI249192, SGI0, SGI249193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249194, SGI249195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55114[] = {SGI249196, SGI0, SGI249197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249198, SGI249199, SGI249199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55115[] = {SGI249200, SGI0, SGI249201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249202, SGI249203, SGI249203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55116[] = {SGI249204, SGI249205, SGI249206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249207, SGI0, SGI0, SGI0, SGI249208, SGI249208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249209, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55117[] = {SGI249210, SGI249211, SGI249212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249213, SGI0, SGI0, SGI0, SGI249214, SGI0, SGI0, SGI249215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55118[] = {SGI249216, SGI249217, SGI249218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249219, SGI0, SGI0, SGI0, SGI249220, SGI0, SGI249221, SGI249222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55119[] = {SGI249223, SGI249224, SGI249225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249226, SGI0, SGI0, SGI0, SGI249227, SGI0, SGI249228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55120[] = {SGI249229, SGI249230, SGI249231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249232, SGI0, SGI0, SGI0, SGI249233, SGI0, SGI249234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55121[] = {SGI249235, SGI249236, SGI249237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249238, SGI0, SGI0, SGI0, SGI249239, SGI0, SGI249240, SGI249241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55122[] = {SGI249242, SGI0, SGI249243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249244, SGI249245, SGI249245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55123[] = {SGI249246, SGI249247, SGI249248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249249, SGI0, SGI0, SGI0, SGI249250, SGI249250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249251, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55124[] = {SGI249252, SGI249253, SGI249254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249255, SGI0, SGI0, SGI0, SGI249256, SGI249256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249257, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55125[] = {SGI249258, SGI249259, SGI249260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249261, SGI0, SGI249262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55126[] = {SGI249263, SGI0, SGI249264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249265, SGI249266, SGI249266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55127[] = {SGI249267, SGI249268, SGI249269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249270, SGI0, SGI0, SGI0, SGI249271, SGI249271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55128[] = {SGI249272, SGI249273, SGI249274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249275, SGI0, SGI0, SGI0, SGI249276, SGI249276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249277, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55129[] = {SGI249278, SGI0, SGI249279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249280, SGI249281, SGI249281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55130[] = {SGI249282, SGI0, SGI249283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249284, SGI249285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55131[] = {SGI249286, SGI249287, SGI249288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249289, SGI0, SGI0, SGI0, SGI249290, SGI249290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55132[] = {SGI249291, SGI249292, SGI249293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249294, SGI0, SGI0, SGI0, SGI249295, SGI249295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55133[] = {SGI249296, SGI0, SGI249297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249298, SGI249299, SGI249299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55134[] = {SGI249300, SGI249301, SGI249302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249303, SGI249303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249304, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55135[] = {SGI249305, SGI249306, SGI249307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249308, SGI249308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249309, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55136[] = {SGI249310, SGI0, SGI249311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249312, SGI249313, SGI249313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249314, SGI0, SGI0, NULL};
const Segment * const SG55137[] = {SGI249315, SGI0, SGI249316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249317, SGI249318, SGI249318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55138[] = {SGI249319, SGI249320, SGI249321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249322, SGI0, SGI0, SGI0, SGI249323, SGI0, SGI0, SGI249324, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55139[] = {SGI249325, SGI249326, SGI249327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249328, SGI0, SGI0, SGI0, SGI249329, SGI0, SGI0, SGI249330, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55140[] = {SGI249331, SGI0, SGI249332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249333, SGI0, SGI0, SGI0, SGI249334, SGI249334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249335, NULL};
const Segment * const SG55141[] = {SGI249336, SGI0, SGI249337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249338, SGI0, SGI0, SGI0, SGI249339, SGI249339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249340, NULL};
const Segment * const SG55142[] = {SGI249341, SGI249342, SGI249343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249344, SGI0, SGI0, SGI0, SGI249345, SGI0, SGI0, SGI249346, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55143[] = {SGI249347, SGI249348, SGI249349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249350, SGI0, SGI0, SGI0, SGI249351, SGI0, SGI249352, SGI249353, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55144[] = {SGI249354, SGI0, SGI249355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249356, SGI0, SGI0, SGI0, SGI249357, SGI249357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249358, NULL};
const Segment * const SG55145[] = {SGI249359, SGI249360, SGI249361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249362, SGI0, SGI0, SGI0, SGI249363, SGI0, SGI0, SGI249364, SGI249365, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55146[] = {SGI249366, SGI249367, SGI249368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249369, SGI0, SGI0, SGI0, SGI249370, SGI0, SGI249371, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55147[] = {SGI249372, SGI249373, SGI249374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249375, SGI0, SGI0, SGI0, SGI249376, SGI0, SGI0, SGI249377, SGI249378, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55148[] = {SGI249379, SGI249380, SGI249381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249382, SGI0, SGI0, SGI0, SGI249383, SGI0, SGI249384, SGI249385, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55149[] = {SGI249386, SGI249387, SGI249388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249389, SGI0, SGI0, SGI0, SGI249390, SGI0, SGI0, SGI249391, SGI0, SGI249392, SGI0, SGI0, NULL};
const Segment * const SG55150[] = {SGI249393, SGI0, SGI249394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249395, SGI249396, SGI249396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55151[] = {SGI249397, SGI249398, SGI249399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249400, SGI0, SGI0, SGI0, SGI249401, SGI249401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55152[] = {SGI249402, SGI0, SGI249403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55153[] = {SGI249404, SGI0, SGI249405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249406, SGI249407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249408, SGI0, NULL};
const Segment * const SG55154[] = {SGI249409, SGI0, SGI249410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249411, SGI249412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55155[] = {SGI249413, SGI0, SGI249414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249415, SGI249416, SGI249416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55156[] = {SGI249417, SGI249418, SGI249419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249420, SGI0, SGI0, SGI0, SGI249421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55157[] = {SGI249422, SGI0, SGI249423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249424, SGI249425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55158[] = {SGI249426, SGI0, SGI249427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249428, SGI249429, SGI249429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249430, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55159[] = {SGI249431, SGI249432, SGI249433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249434, SGI0, SGI0, SGI0, SGI249435, SGI0, SGI249436, SGI249437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55160[] = {SGI249438, SGI249439, SGI249440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249441, SGI0, SGI0, SGI0, SGI249442, SGI249442, SGI0, SGI249443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55161[] = {SGI249444, SGI249445, SGI249446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249447, SGI0, SGI0, SGI0, SGI249448, SGI249448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55162[] = {SGI249449, SGI249450, SGI249451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249452, SGI0, SGI0, SGI0, SGI249453, SGI0, SGI0, SGI249454, SGI0, SGI0, SGI0, SGI249455, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55163[] = {SGI249456, SGI249457, SGI249458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249459, SGI0, SGI0, SGI0, SGI249460, SGI249460, SGI0, SGI0, SGI249461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55164[] = {SGI249462, SGI0, SGI249463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249464, SGI249465, SGI249465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55165[] = {SGI249466, SGI0, SGI249467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249468, SGI249469, SGI249469, SGI0, SGI0, SGI0, SGI0, SGI249470, SGI0, NULL};
const Segment * const SG55166[] = {SGI249471, SGI0, SGI249472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249473, SGI249474, SGI249474, SGI0, SGI0, SGI0, SGI0, SGI249475, SGI0, NULL};
const Segment * const SG55167[] = {SGI249476, SGI0, SGI249477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249478, SGI249479, SGI249479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55168[] = {SGI249480, SGI249481, SGI249482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249483, SGI0, SGI0, SGI0, SGI249484, SGI0, SGI249486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55169[] = {SGI249485, SGI0, SGI249487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249488, SGI249489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55170[] = {SGI249490, SGI0, SGI249491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249492, SGI249493, SGI249493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55171[] = {SGI249494, SGI0, SGI249495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249496, SGI249497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55172[] = {SGI249498, SGI249499, SGI249500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249501, SGI0, SGI0, SGI0, SGI249503, SGI249503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55173[] = {SGI249502, SGI249504, SGI249505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249506, SGI0, SGI0, SGI0, SGI249507, SGI0, SGI0, SGI249508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249509, NULL};
const Segment * const SG55174[] = {SGI249510, SGI249511, SGI249512, SGI0, SGI249513, SGI249514, SGI249515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55175[] = {SGI249516, SGI249517, SGI249518, SGI0, SGI249519, SGI249520, SGI249521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55176[] = {SGI249522, SGI249523, SGI249524, SGI0, SGI249525, SGI249526, SGI249528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55177[] = {SGI249527, SGI249529, SGI249530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249531, SGI0, SGI0, SGI0, SGI249532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55178[] = {SGI249533, SGI249534, SGI249535, SGI0, SGI249536, SGI249537, SGI249538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55179[] = {SGI249539, SGI249540, SGI249541, SGI0, SGI249542, SGI249543, SGI249544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55180[] = {SGI249545, SGI249546, SGI0, SGI0, SGI249547, SGI249548, SGI249549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55181[] = {SGI249550, SGI249551, SGI249552, SGI0, SGI249553, SGI249554, SGI249555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55182[] = {SGI249556, SGI249557, SGI249558, SGI0, SGI249559, SGI249560, SGI249561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55183[] = {SGI249562, SGI249563, SGI249564, SGI0, SGI249565, SGI249566, SGI249567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55184[] = {SGI249568, SGI249569, SGI249570, SGI0, SGI249571, SGI249572, SGI249573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55185[] = {SGI249574, SGI249575, SGI249576, SGI0, SGI249577, SGI249578, SGI249579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55186[] = {SGI249580, SGI249581, SGI249582, SGI0, SGI249583, SGI249584, SGI249585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55187[] = {SGI249588, SGI249589, SGI249590, SGI0, SGI249591, SGI249592, SGI249593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55188[] = {SGI249586, SGI0, SGI249587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249594, SGI249595, SGI249595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249596, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55189[] = {SGI249597, SGI249598, SGI249599, SGI0, SGI249600, SGI249601, SGI249602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55190[] = {SGI249603, SGI0, SGI249604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249605, SGI249606, SGI249606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55191[] = {SGI249607, SGI249608, SGI0, SGI0, SGI249609, SGI249610, SGI249611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55192[] = {SGI249612, SGI249613, SGI249614, SGI0, SGI249615, SGI249616, SGI249617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55193[] = {SGI249618, SGI249619, SGI249620, SGI0, SGI249621, SGI249622, SGI249623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55194[] = {SGI249624, SGI249625, SGI249626, SGI0, SGI249627, SGI249628, SGI249629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55195[] = {SGI249630, SGI249631, SGI249632, SGI0, SGI249633, SGI249634, SGI249635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55196[] = {SGI249636, SGI249637, SGI249638, SGI0, SGI249639, SGI249640, SGI249641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55197[] = {SGI249642, SGI249643, SGI249644, SGI0, SGI249645, SGI249646, SGI249647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55198[] = {SGI249648, SGI0, SGI249649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249650, SGI249652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55199[] = {SGI249651, SGI249653, SGI249654, SGI0, SGI249655, SGI249656, SGI249657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55200[] = {SGI249658, SGI249659, SGI249660, SGI0, SGI249661, SGI249662, SGI249663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55201[] = {SGI249664, SGI249665, SGI249666, SGI0, SGI249667, SGI249668, SGI249669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55202[] = {SGI249670, SGI249671, SGI249672, SGI0, SGI249673, SGI249674, SGI249675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55203[] = {SGI249676, SGI249677, SGI249678, SGI0, SGI249679, SGI249680, SGI249681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55204[] = {SGI249682, SGI249683, SGI249684, SGI0, SGI249685, SGI249686, SGI249687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55205[] = {SGI249688, SGI249689, SGI249690, SGI0, SGI249691, SGI249692, SGI249693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55206[] = {SGI249694, SGI249695, SGI249696, SGI0, SGI249697, SGI249698, SGI249699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55207[] = {SGI249700, SGI249701, SGI249702, SGI0, SGI249703, SGI249704, SGI249705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55208[] = {SGI249706, SGI249707, SGI249708, SGI0, SGI249709, SGI249710, SGI249711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55209[] = {SGI249712, SGI249713, SGI249714, SGI0, SGI249715, SGI249716, SGI249717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55210[] = {SGI249718, SGI249719, SGI249720, SGI0, SGI249721, SGI249722, SGI249723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55211[] = {SGI249724, SGI249725, SGI249726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249727, SGI0, SGI0, SGI0, SGI249728, SGI249728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55212[] = {SGI249729, SGI0, SGI249730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249731, SGI249732, SGI249732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55213[] = {SGI249733, SGI0, SGI249734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249735, SGI249736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55214[] = {SGI249737, SGI0, SGI249738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249739, SGI249740, SGI249740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249741, SGI0, SGI0, NULL};
const Segment * const SG55215[] = {SGI249742, SGI0, SGI249743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249744, SGI249745, SGI249745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55216[] = {SGI249746, SGI0, SGI249747, SGI0, SGI249748, SGI249749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55217[] = {SGI249750, SGI0, SGI249751, SGI0, SGI249752, SGI249753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55218[] = {SGI249754, SGI0, SGI249755, SGI0, SGI249756, SGI249757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55219[] = {SGI249758, SGI0, SGI249759, SGI0, SGI249760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55220[] = {SGI249761, SGI0, SGI249762, SGI0, SGI249763, SGI249764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55221[] = {SGI249765, SGI0, SGI249766, SGI0, SGI249767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55222[] = {SGI249768, SGI0, SGI249769, SGI0, SGI249770, SGI249771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55223[] = {SGI249772, SGI0, SGI249773, SGI0, SGI249774, SGI249775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55224[] = {SGI249776, SGI0, SGI249777, SGI0, SGI249778, SGI249779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55225[] = {SGI249780, SGI0, SGI249781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249782, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55226[] = {SGI249783, SGI249784, SGI249785, SGI0, SGI249786, SGI249787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55227[] = {SGI249788, SGI0, SGI249789, SGI0, SGI249790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55228[] = {SGI249791, SGI0, SGI249792, SGI0, SGI249793, SGI249794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55229[] = {SGI249795, SGI0, SGI249796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55230[] = {SGI249797, SGI0, SGI249798, SGI0, SGI249799, SGI249800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55231[] = {SGI249801, SGI249802, SGI249803, SGI0, SGI249804, SGI249805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55232[] = {SGI249806, SGI0, SGI249807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55233[] = {SGI249808, SGI0, SGI249809, SGI0, SGI249810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55234[] = {SGI249811, SGI0, SGI249812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249813, SGI249814, SGI249814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55235[] = {SGI249815, SGI0, SGI249816, SGI0, SGI249817, SGI249818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55236[] = {SGI249819, SGI0, SGI249820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249821, SGI249822, SGI249822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55237[] = {SGI249823, SGI0, SGI249824, SGI0, SGI249825, SGI249826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55238[] = {SGI249827, SGI0, SGI249828, SGI0, SGI249829, SGI249830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55239[] = {SGI249831, SGI0, SGI249832, SGI0, SGI249833, SGI249834, SGI249835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55240[] = {SGI249836, SGI0, SGI249837, SGI0, SGI249838, SGI249839, SGI249840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55241[] = {SGI249841, SGI0, SGI0, SGI0, SGI249842, SGI249843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55242[] = {SGI249844, SGI0, SGI249845, SGI0, SGI249846, SGI249847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55243[] = {SGI249848, SGI0, SGI249849, SGI0, SGI249850, SGI249851, SGI249852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55244[] = {SGI249853, SGI0, SGI249854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249855, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55245[] = {SGI249856, SGI0, SGI249857, SGI0, SGI249858, SGI249859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55246[] = {SGI249860, SGI0, SGI249861, SGI0, SGI249862, SGI249863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55247[] = {SGI249864, SGI249866, SGI249868, SGI0, SGI249869, SGI249870, SGI249871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55248[] = {SGI249865, SGI0, SGI249867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249872, SGI249873, SGI249873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55249[] = {SGI249874, SGI249875, SGI249876, SGI0, SGI249877, SGI249878, SGI249879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55250[] = {SGI249880, SGI249881, SGI249882, SGI0, SGI249883, SGI249884, SGI249885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55251[] = {SGI249886, SGI249887, SGI249888, SGI0, SGI249889, SGI249890, SGI249891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55252[] = {SGI249892, SGI249893, SGI249894, SGI0, SGI249895, SGI249896, SGI249897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55253[] = {SGI249898, SGI249899, SGI0, SGI0, SGI249900, SGI249901, SGI249902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55254[] = {SGI249903, SGI249904, SGI249905, SGI0, SGI249906, SGI249907, SGI249908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55255[] = {SGI249909, SGI249910, SGI249911, SGI0, SGI249912, SGI249913, SGI249915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55256[] = {SGI249914, SGI249916, SGI249917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI249918, SGI0, SGI0, SGI0, SGI249919, SGI249919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55257[] = {SGI249920, SGI249921, SGI249922, SGI0, SGI249923, SGI249924, SGI249925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55258[] = {SGI249926, SGI249927, SGI249928, SGI0, SGI249929, SGI249930, SGI249931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55259[] = {SGI249932, SGI249933, SGI249934, SGI0, SGI249935, SGI249936, SGI249937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55260[] = {SGI249938, SGI249939, SGI249940, SGI0, SGI249941, SGI249942, SGI249943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55261[] = {SGI249944, SGI249945, SGI249946, SGI0, SGI249947, SGI249948, SGI249949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55262[] = {SGI249950, SGI249951, SGI249952, SGI0, SGI249953, SGI249954, SGI249955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55263[] = {SGI249956, SGI249957, SGI249958, SGI0, SGI249959, SGI249960, SGI249961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55264[] = {SGI249962, SGI249963, SGI0, SGI0, SGI249964, SGI249965, SGI249966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55265[] = {SGI249967, SGI249968, SGI249969, SGI0, SGI249970, SGI249971, SGI249972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55266[] = {SGI249973, SGI249974, SGI249975, SGI0, SGI249976, SGI249977, SGI249978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55267[] = {SGI249979, SGI249980, SGI249981, SGI0, SGI249982, SGI249983, SGI249984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55268[] = {SGI249985, SGI249986, SGI249987, SGI0, SGI249988, SGI249989, SGI249990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55269[] = {SGI249991, SGI249992, SGI0, SGI0, SGI249993, SGI249994, SGI249995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55270[] = {SGI249996, SGI249997, SGI249998, SGI0, SGI249999, SGI250000, SGI250001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55271[] = {SGI250002, SGI250003, SGI250004, SGI0, SGI250005, SGI250006, SGI250007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55272[] = {SGI250008, SGI250009, SGI0, SGI0, SGI250010, SGI250011, SGI250012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55273[] = {SGI250013, SGI250014, SGI250015, SGI0, SGI250016, SGI250017, SGI250018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55274[] = {SGI250019, SGI250020, SGI250021, SGI0, SGI250022, SGI250023, SGI250024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55275[] = {SGI250025, SGI250026, SGI250027, SGI0, SGI250028, SGI250029, SGI250030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55276[] = {SGI250031, SGI250032, SGI250033, SGI0, SGI250034, SGI250035, SGI250036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55277[] = {SGI250037, SGI250038, SGI250039, SGI0, SGI250040, SGI250041, SGI250042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55278[] = {SGI250043, SGI250044, SGI250045, SGI0, SGI250046, SGI250047, SGI250048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55279[] = {SGI250049, SGI250050, SGI250051, SGI0, SGI250052, SGI250053, SGI250054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55280[] = {SGI250055, SGI250056, SGI250057, SGI0, SGI250058, SGI250059, SGI250060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55281[] = {SGI250061, SGI250062, SGI250063, SGI0, SGI250064, SGI250065, SGI250066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55282[] = {SGI250067, SGI250068, SGI250069, SGI0, SGI250070, SGI250071, SGI250072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55283[] = {SGI250073, SGI250074, SGI250075, SGI0, SGI250076, SGI250077, SGI250078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55284[] = {SGI250079, SGI250080, SGI250081, SGI0, SGI250082, SGI250083, SGI250084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55285[] = {SGI250085, SGI250086, SGI250087, SGI0, SGI250088, SGI250089, SGI250090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55286[] = {SGI250091, SGI250092, SGI250093, SGI0, SGI250094, SGI250095, SGI250096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55287[] = {SGI250097, SGI250098, SGI250099, SGI0, SGI250100, SGI250101, SGI250102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55288[] = {SGI250103, SGI250104, SGI0, SGI0, SGI250105, SGI250106, SGI250107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55289[] = {SGI250108, SGI250109, SGI250110, SGI0, SGI250111, SGI250112, SGI250113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55290[] = {SGI250114, SGI250115, SGI250116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250117, SGI0, SGI0, SGI0, SGI250118, SGI250118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55291[] = {SGI250119, SGI0, SGI250120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250121, SGI250122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55292[] = {SGI250123, SGI250124, SGI250125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250126, SGI0, SGI0, SGI0, SGI250127, SGI250127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55293[] = {SGI250128, SGI250129, SGI250130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250131, SGI250131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250132, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55294[] = {SGI250133, SGI250134, SGI250135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250139, SGI0, SGI0, SGI0, SGI250140, SGI250140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55295[] = {SGI250136, SGI250137, SGI250138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250141, SGI0, SGI0, SGI0, SGI250142, SGI250142, SGI0, SGI250143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55296[] = {SGI250144, SGI0, SGI250145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250146, SGI250147, SGI250147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55297[] = {SGI250148, SGI0, SGI250149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250152, SGI250153, SGI250153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55298[] = {SGI250150, SGI0, SGI250151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250154, SGI250155, SGI250155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55299[] = {SGI250156, SGI250157, SGI250158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250159, SGI0, SGI0, SGI0, SGI250160, SGI250160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55300[] = {SGI250161, SGI250162, SGI250163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250164, SGI0, SGI0, SGI0, SGI250165, SGI250165, SGI250166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55301[] = {SGI250167, SGI0, SGI250168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250169, SGI250170, SGI250170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250171, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55302[] = {SGI250172, SGI250173, SGI250174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250175, SGI0, SGI0, SGI0, SGI250176, SGI0, SGI250177, SGI250178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55303[] = {SGI250179, SGI250180, SGI250181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250182, SGI0, SGI0, SGI0, SGI250183, SGI250183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55304[] = {SGI250184, SGI0, SGI250185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250186, SGI250187, SGI250187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55305[] = {SGI250188, SGI0, SGI250189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250190, SGI250191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55306[] = {SGI250192, SGI250193, SGI250194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250195, SGI0, SGI0, SGI0, SGI250196, SGI0, SGI0, SGI250197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55307[] = {SGI250198, SGI250199, SGI250200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250201, SGI0, SGI0, SGI0, SGI250202, SGI250202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55308[] = {SGI250203, SGI250204, SGI250205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55309[] = {SGI250206, SGI250207, SGI250208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250209, SGI0, SGI0, SGI0, SGI250210, SGI250210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55310[] = {SGI250211, SGI250212, SGI250213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250214, SGI0, SGI0, SGI0, SGI250215, SGI250216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250217, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55311[] = {SGI250218, SGI0, SGI250219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250220, SGI250221, SGI250221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250222, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55312[] = {SGI250223, SGI250224, SGI250225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250226, SGI0, SGI0, SGI0, SGI250227, SGI0, SGI0, SGI250228, SGI0, SGI0, SGI0, SGI250229, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55313[] = {SGI250230, SGI250231, SGI250232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250233, SGI0, SGI0, SGI0, SGI250234, SGI250234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55314[] = {SGI250235, SGI250236, SGI250237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250238, SGI0, SGI0, SGI0, SGI250239, SGI0, SGI0, SGI250240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250241, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55315[] = {SGI250242, SGI250243, SGI250244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250245, SGI0, SGI0, SGI0, SGI250246, SGI250246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250247, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55316[] = {SGI250248, SGI250249, SGI250250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250251, SGI0, SGI0, SGI0, SGI250252, SGI0, SGI250253, SGI250254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55317[] = {SGI250255, SGI250256, SGI250257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250258, SGI0, SGI0, SGI0, SGI250259, SGI0, SGI0, SGI250260, SGI0, SGI0, SGI0, SGI250261, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55318[] = {SGI250262, SGI0, SGI250263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250264, SGI0, SGI0, SGI0, SGI250265, SGI250265, SGI0, SGI0, SGI0, SGI0, SGI250266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55319[] = {SGI250267, SGI250268, SGI250269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250270, SGI0, SGI0, SGI0, SGI250271, SGI0, SGI250272, SGI250273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55320[] = {SGI250274, SGI250275, SGI250276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250278, SGI0, SGI0, SGI0, SGI250281, SGI0, SGI0, SGI250282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250283, NULL};
const Segment * const SG55321[] = {SGI250277, SGI250279, SGI250280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250284, SGI0, SGI0, SGI0, SGI250285, SGI250285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55322[] = {SGI250286, SGI250287, SGI250288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250289, SGI0, SGI0, SGI0, SGI250290, SGI250291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250292, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55323[] = {SGI250293, SGI0, SGI250294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250295, SGI250296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55324[] = {SGI250297, SGI250298, SGI250299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250300, SGI0, SGI0, SGI0, SGI250301, SGI0, SGI0, SGI250302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250303, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55325[] = {SGI250304, SGI250305, SGI250306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250307, SGI0, SGI0, SGI0, SGI250308, SGI250308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55326[] = {SGI250309, SGI250310, SGI250311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250312, SGI0, SGI0, SGI0, SGI250313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250314, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55327[] = {SGI250315, SGI0, SGI250316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250317, SGI250318, SGI250318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55328[] = {SGI250319, SGI0, SGI250320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250321, SGI250322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250323, SGI0, SGI0, NULL};
const Segment * const SG55329[] = {SGI250324, SGI250325, SGI250326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250327, SGI0, SGI0, SGI0, SGI250328, SGI0, SGI250329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55330[] = {SGI250330, SGI250331, SGI250332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250333, SGI250333, SGI0, SGI0, SGI0, SGI0, SGI250334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55331[] = {SGI250335, SGI250336, SGI250337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250338, SGI0, SGI0, SGI0, SGI250339, SGI250339, SGI250340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55332[] = {SGI250341, SGI250342, SGI250343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250344, SGI0, SGI0, SGI0, SGI250345, SGI250345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250346, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55333[] = {SGI250347, SGI250348, SGI250349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250350, SGI0, SGI0, SGI0, SGI250351, SGI250351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250352, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55334[] = {SGI250353, SGI0, SGI250354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250355, SGI250356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55335[] = {SGI250357, SGI250358, SGI250359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250360, SGI0, SGI0, SGI0, SGI250361, SGI250361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55336[] = {SGI250362, SGI0, SGI250363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250364, SGI250365, SGI250365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55337[] = {SGI250366, SGI0, SGI250367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250368, SGI250369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250370, SGI0, SGI0, NULL};
const Segment * const SG55338[] = {SGI250371, SGI0, SGI250372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250373, SGI250374, SGI250374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250375, SGI0, SGI0, NULL};
const Segment * const SG55339[] = {SGI250376, SGI0, SGI250377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250378, SGI250379, SGI250379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250380, SGI0, SGI0, NULL};
const Segment * const SG55340[] = {SGI250381, SGI0, SGI250382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250383, SGI250384, SGI250384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250385, SGI0, SGI0, NULL};
const Segment * const SG55341[] = {SGI250386, SGI0, SGI250387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250388, SGI250389, SGI250389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250390, SGI0, SGI0, NULL};
const Segment * const SG55342[] = {SGI250391, SGI0, SGI250392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250393, SGI250394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55343[] = {SGI250395, SGI0, SGI250396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250397, SGI250398, SGI250398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55344[] = {SGI250399, SGI0, SGI250400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250401, SGI250402, SGI250402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55345[] = {SGI250403, SGI250404, SGI250405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250406, SGI0, SGI0, SGI250407, SGI250408, SGI250408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55346[] = {SGI250409, SGI0, SGI250410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250411, SGI250412, SGI250412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55347[] = {SGI250413, SGI250414, SGI250415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250416, SGI250416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250417, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55348[] = {SGI250418, SGI0, SGI250419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250420, SGI250421, SGI250421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250422, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55349[] = {SGI250423, SGI0, SGI250424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250425, SGI250426, SGI250426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55350[] = {SGI250427, SGI0, SGI250428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250429, SGI250430, SGI250430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55351[] = {SGI250431, SGI0, SGI250432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250433, SGI250434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55352[] = {SGI250435, SGI250436, SGI250437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250438, SGI250438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55353[] = {SGI250439, SGI0, SGI250440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250441, SGI250442, SGI250442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55354[] = {SGI250443, SGI0, SGI250444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250445, SGI250446, SGI250446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55355[] = {SGI250447, SGI250448, SGI250449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250450, SGI0, SGI0, SGI0, SGI250451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55356[] = {SGI250452, SGI250453, SGI250454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250455, SGI0, SGI0, SGI0, SGI250456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55357[] = {SGI250457, SGI250458, SGI250459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250460, SGI250460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250461, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55358[] = {SGI250462, SGI250463, SGI250464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250465, SGI0, SGI0, SGI0, SGI250466, SGI250466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55359[] = {SGI250467, SGI250468, SGI250469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250470, SGI0, SGI0, SGI0, SGI250471, SGI250471, SGI0, SGI0, SGI250472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55360[] = {SGI250473, SGI250474, SGI250475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250476, SGI250476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250477, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55361[] = {SGI250478, SGI0, SGI250479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250480, SGI250481, SGI250481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55362[] = {SGI250482, SGI250483, SGI250484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250485, SGI0, SGI0, SGI0, SGI250486, SGI250486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55363[] = {SGI250487, SGI0, SGI250488, SGI0, SGI250489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55364[] = {SGI250490, SGI0, SGI250491, SGI0, SGI250492, SGI250493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55365[] = {SGI250494, SGI0, SGI250495, SGI0, SGI250496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55366[] = {SGI250497, SGI0, SGI250498, SGI0, SGI250499, SGI250500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55367[] = {SGI250501, SGI0, SGI250502, SGI0, SGI250503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55368[] = {SGI250504, SGI0, SGI250505, SGI0, SGI250506, SGI250507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55369[] = {SGI250508, SGI0, SGI250509, SGI0, SGI250510, SGI250511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55370[] = {SGI250512, SGI0, SGI250513, SGI0, SGI250514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55371[] = {SGI250515, SGI0, SGI250516, SGI0, SGI250517, SGI250518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55372[] = {SGI250519, SGI0, SGI250520, SGI0, SGI250521, SGI250522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55373[] = {SGI250523, SGI0, SGI250524, SGI0, SGI250525, SGI250526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55374[] = {SGI250527, SGI0, SGI250528, SGI0, SGI250529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55375[] = {SGI250530, SGI0, SGI250531, SGI0, SGI250532, SGI250533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55376[] = {SGI250534, SGI0, SGI250535, SGI0, SGI250536, SGI250537, SGI250536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55377[] = {SGI250538, SGI0, SGI250539, SGI0, SGI250540, SGI250541, SGI250542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55378[] = {SGI250543, SGI0, SGI250544, SGI0, SGI250545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55379[] = {SGI250546, SGI0, SGI250547, SGI0, SGI250548, SGI250549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55380[] = {SGI250550, SGI0, SGI250551, SGI0, SGI250552, SGI250553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55381[] = {SGI250554, SGI0, SGI250555, SGI0, SGI250556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55382[] = {SGI250557, SGI0, SGI250558, SGI0, SGI250559, SGI250560, SGI250559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55383[] = {SGI250561, SGI0, SGI250562, SGI0, SGI250563, SGI250564, SGI250565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250566, SGI0, NULL};
const Segment * const SG55384[] = {SGI250567, SGI0, SGI250568, SGI0, SGI250569, SGI250570, SGI250571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250572, SGI0, NULL};
const Segment * const SG55385[] = {SGI250573, SGI0, SGI250574, SGI0, SGI250575, SGI250576, SGI250577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250578, SGI0, NULL};
const Segment * const SG55386[] = {SGI250579, SGI0, SGI250580, SGI0, SGI250581, SGI250582, SGI250583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55387[] = {SGI250584, SGI0, SGI250585, SGI0, SGI250586, SGI250587, SGI250586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55388[] = {SGI250588, SGI250589, SGI250590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250591, SGI0, SGI0, SGI0, SGI250594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55389[] = {SGI250592, SGI0, SGI250593, SGI0, SGI250595, SGI250596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55390[] = {SGI250597, SGI0, SGI250599, SGI0, SGI250601, SGI250602, SGI250601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55391[] = {SGI250598, SGI0, SGI250600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55392[] = {SGI250603, SGI0, SGI250604, SGI0, SGI250605, SGI250606, SGI250605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55393[] = {SGI250607, SGI0, SGI250608, SGI0, SGI250609, SGI250610, SGI250609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55394[] = {SGI250611, SGI0, SGI250612, SGI0, SGI250613, SGI250614, SGI250615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55395[] = {SGI250616, SGI0, SGI250617, SGI0, SGI250618, SGI250619, SGI250620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55396[] = {SGI250621, SGI0, SGI250622, SGI0, SGI250623, SGI250624, SGI250623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55397[] = {SGI250625, SGI0, SGI250626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55398[] = {SGI250627, SGI0, SGI250628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55399[] = {SGI250629, SGI0, SGI250630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55400[] = {SGI250631, SGI0, SGI250632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55401[] = {SGI250633, SGI0, SGI250634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55402[] = {SGI250635, SGI0, SGI250636, SGI0, SGI250637, SGI250638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55403[] = {SGI250643, SGI0, SGI250644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250645, SGI250646, SGI250646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55404[] = {SGI250639, SGI0, SGI250640, SGI0, SGI250641, SGI250642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55405[] = {SGI250647, SGI0, SGI250648, SGI0, SGI250649, SGI250650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55406[] = {SGI250651, SGI0, SGI250652, SGI0, SGI250653, SGI250654, SGI250653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55407[] = {SGI250655, SGI0, SGI250656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55408[] = {SGI250657, SGI0, SGI250658, SGI0, SGI250659, SGI250660, SGI250659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55409[] = {SGI250661, SGI0, SGI250662, SGI0, SGI250663, SGI250664, SGI250665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250666, SGI0, NULL};
const Segment * const SG55410[] = {SGI250667, SGI0, SGI250668, SGI0, SGI250669, SGI250670, SGI250671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250672, SGI0, NULL};
const Segment * const SG55411[] = {SGI250673, SGI0, SGI250674, SGI0, SGI250675, SGI250676, SGI250677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55412[] = {SGI250678, SGI0, SGI250679, SGI0, SGI250680, SGI250681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55413[] = {SGI250682, SGI0, SGI250683, SGI0, SGI250684, SGI250685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55414[] = {SGI250686, SGI0, SGI250687, SGI0, SGI250688, SGI250689, SGI250690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55415[] = {SGI250691, SGI0, SGI250692, SGI0, SGI250693, SGI250694, SGI250695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250696, SGI0, NULL};
const Segment * const SG55416[] = {SGI250697, SGI0, SGI250698, SGI0, SGI250699, SGI250700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55417[] = {SGI250701, SGI0, SGI250702, SGI0, SGI250703, SGI250704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55418[] = {SGI250705, SGI0, SGI250706, SGI0, SGI250707, SGI250708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55419[] = {SGI250709, SGI0, SGI250710, SGI0, SGI250711, SGI250712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55420[] = {SGI250713, SGI0, SGI250714, SGI0, SGI250715, SGI250716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55421[] = {SGI250717, SGI0, SGI250718, SGI0, SGI250719, SGI250720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55422[] = {SGI250721, SGI0, SGI250722, SGI0, SGI250723, SGI250724, SGI250725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55423[] = {SGI250726, SGI0, SGI250727, SGI0, SGI250728, SGI250729, SGI250730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55424[] = {SGI250731, SGI0, SGI250732, SGI0, SGI250733, SGI250734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55425[] = {SGI250735, SGI0, SGI250736, SGI0, SGI250737, SGI250738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55426[] = {SGI250739, SGI0, SGI250740, SGI0, SGI250741, SGI250742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55427[] = {SGI250743, SGI0, SGI250744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250745, SGI250746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55428[] = {SGI250747, SGI0, SGI250748, SGI0, SGI250749, SGI250750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55429[] = {SGI250751, SGI0, SGI250752, SGI0, SGI250753, SGI250754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55430[] = {SGI250755, SGI0, SGI250756, SGI0, SGI250757, SGI250758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55431[] = {SGI250759, SGI0, SGI250760, SGI0, SGI250761, SGI250762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55432[] = {SGI250763, SGI0, SGI250764, SGI0, SGI250765, SGI250766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55433[] = {SGI250767, SGI0, SGI250768, SGI0, SGI250769, SGI250770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55434[] = {SGI250771, SGI0, SGI250772, SGI0, SGI250773, SGI250774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55435[] = {SGI250775, SGI250776, SGI250777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250778, SGI250778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250779, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55436[] = {SGI250780, SGI250781, SGI250782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250783, SGI250783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250784, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55437[] = {SGI250785, SGI0, SGI250786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250787, SGI250788, SGI250788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55438[] = {SGI250789, SGI0, SGI250790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250791, SGI250792, SGI250792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55439[] = {SGI250793, SGI250794, SGI250795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250796, SGI0, SGI0, SGI0, SGI250797, SGI250797, SGI0, SGI0, SGI250798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55440[] = {SGI250799, SGI0, SGI250800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250801, SGI250802, SGI250802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55441[] = {SGI250803, SGI0, SGI250804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250805, SGI250806, SGI250806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250807, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55442[] = {SGI250808, SGI250809, SGI250810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250811, SGI0, SGI0, SGI0, SGI250812, SGI250812, SGI0, SGI250813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55443[] = {SGI250814, SGI250815, NULL};
const Segment * const SG55444[] = {SGI250816, SGI250817, NULL};
const Segment * const SG55445[] = {SGI250818, SGI250819, NULL};
const Segment * const SG55446[] = {SGI250820, SGI250820, NULL};
const Segment * const SG55447[] = {SGI250821, SGI250821, NULL};
const Segment * const SG55448[] = {SGI250822, SGI250822, NULL};
const Segment * const SG55449[] = {SGI250823, SGI250824, NULL};
const Segment * const SG55450[] = {SGI250825, SGI250826, NULL};
const Segment * const SG55451[] = {SGI250827, SGI250827, NULL};
const Segment * const SG55452[] = {SGI250828, SGI250828, NULL};
const Segment * const SG55453[] = {SGI250829, SGI250830, NULL};
const Segment * const SG55454[] = {SGI250831, SGI250832, NULL};
const Segment * const SG55455[] = {SGI250833, SGI250834, NULL};
const Segment * const SG55456[] = {SGI250835, SGI250836, NULL};
const Segment * const SG55457[] = {SGI250837, SGI250838, NULL};
const Segment * const SG55458[] = {SGI250839, SGI250840, NULL};
const Segment * const SG55459[] = {SGI250841, SGI250842, NULL};
const Segment * const SG55460[] = {SGI250843, SGI250844, NULL};
const Segment * const SG55461[] = {SGI250845, SGI250845, NULL};
const Segment * const SG55462[] = {SGI250846, SGI250847, NULL};
const Segment * const SG55463[] = {SGI250848, SGI250849, NULL};
const Segment * const SG55464[] = {SGI250850, SGI250851, NULL};
const Segment * const SG55465[] = {SGI250852, SGI250852, NULL};
const Segment * const SG55466[] = {SGI250853, SGI250854, NULL};
const Segment * const SG55467[] = {SGI250855, SGI250856, NULL};
const Segment * const SG55468[] = {SGI250857, SGI250858, NULL};
const Segment * const SG55469[] = {SGI250859, SGI250860, NULL};
const Segment * const SG55470[] = {SGI250861, SGI250862, NULL};
const Segment * const SG55471[] = {SGI250863, SGI250864, NULL};
const Segment * const SG55472[] = {SGI250865, SGI250866, NULL};
const Segment * const SG55473[] = {SGI250867, SGI250868, NULL};
const Segment * const SG55474[] = {SGI250869, SGI250869, NULL};
const Segment * const SG55475[] = {SGI250874, SGI250874, NULL};
const Segment * const SG55476[] = {SGI250870, SGI0, SGI250871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250872, SGI250873, SGI250873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55477[] = {SGI250875, SGI250876, NULL};
const Segment * const SG55478[] = {SGI250877, SGI250878, NULL};
const Segment * const SG55479[] = {SGI250879, SGI250879, NULL};
const Segment * const SG55480[] = {SGI250880, SGI250881, NULL};
const Segment * const SG55481[] = {SGI250882, SGI250883, NULL};
const Segment * const SG55482[] = {SGI250884, SGI250885, NULL};
const Segment * const SG55483[] = {SGI250886, SGI250887, NULL};
const Segment * const SG55484[] = {SGI250888, SGI250889, NULL};
const Segment * const SG55485[] = {SGI250890, SGI250890, NULL};
const Segment * const SG55486[] = {SGI250891, SGI250892, NULL};
const Segment * const SG55487[] = {SGI250893, SGI250893, NULL};
const Segment * const SG55488[] = {SGI250894, SGI250895, NULL};
const Segment * const SG55489[] = {SGI250896, SGI250897, NULL};
const Segment * const SG55490[] = {SGI250898, SGI250899, NULL};
const Segment * const SG55491[] = {SGI250900, SGI250901, NULL};
const Segment * const SG55492[] = {SGI250902, SGI250903, NULL};
const Segment * const SG55493[] = {SGI250904, SGI250905, NULL};
const Segment * const SG55494[] = {SGI250906, SGI250907, NULL};
const Segment * const SG55495[] = {SGI250908, SGI250909, NULL};
const Segment * const SG55496[] = {SGI250910, SGI250911, NULL};
const Segment * const SG55497[] = {SGI250912, SGI250913, NULL};
const Segment * const SG55498[] = {SGI250916, SGI250916, NULL};
const Segment * const SG55499[] = {SGI250914, SGI0, SGI250915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250917, SGI250918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI250919, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55500[] = {SGI250920, SGI250921, NULL};
const Segment * const SG55501[] = {SGI250922, SGI250923, NULL};
const Segment * const SG55502[] = {SGI250924, SGI250925, NULL};
const Segment * const SG55503[] = {SGI250926, SGI250927, NULL};
const Segment * const SG55504[] = {SGI250928, SGI250929, NULL};
const Segment * const SG55505[] = {SGI250930, SGI250931, NULL};
const Segment * const SG55506[] = {SGI250932, SGI250932, NULL};
const Segment * const SG55507[] = {SGI250933, SGI250934, NULL};
const Segment * const SG55508[] = {SGI250935, SGI250935, NULL};
const Segment * const SG55509[] = {SGI250936, SGI250937, NULL};
const Segment * const SG55510[] = {SGI250938, SGI250938, NULL};
const Segment * const SG55511[] = {SGI250939, SGI250940, NULL};
const Segment * const SG55512[] = {SGI250941, SGI250942, NULL};
const Segment * const SG55513[] = {SGI250943, SGI250944, NULL};
const Segment * const SG55514[] = {SGI250945, SGI250945, NULL};
const Segment * const SG55515[] = {SGI250946, SGI250947, NULL};
const Segment * const SG55516[] = {SGI250948, SGI250949, NULL};
const Segment * const SG55517[] = {SGI250950, SGI250951, NULL};
const Segment * const SG55518[] = {SGI250952, SGI250953, NULL};
const Segment * const SG55519[] = {SGI250954, SGI250955, NULL};
const Segment * const SG55520[] = {SGI250956, SGI250956, NULL};
const Segment * const SG55521[] = {SGI250957, SGI250957, NULL};
const Segment * const SG55522[] = {SGI250958, SGI250959, NULL};
const Segment * const SG55523[] = {SGI250960, SGI250961, NULL};
const Segment * const SG55524[] = {SGI250962, SGI250963, NULL};
const Segment * const SG55525[] = {SGI250964, SGI250964, NULL};
const Segment * const SG55526[] = {SGI250965, SGI250966, SGI250967, SGI0, SGI250968, SGI250969, SGI250970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55527[] = {SGI250971, SGI250972, SGI250973, SGI0, SGI250974, SGI250975, SGI250976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55528[] = {SGI250977, SGI250978, SGI250979, SGI0, SGI250980, SGI250981, SGI250982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55529[] = {SGI250983, SGI250984, SGI250985, SGI0, SGI250986, SGI250987, SGI250988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55530[] = {SGI250989, SGI250990, SGI250991, SGI0, SGI250992, SGI250993, SGI250994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55531[] = {SGI250995, SGI250996, SGI250997, SGI0, SGI250998, SGI250999, SGI251000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55532[] = {SGI251001, SGI251002, SGI251003, SGI0, SGI251004, SGI251005, SGI251006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55533[] = {SGI251007, SGI251008, SGI251009, SGI0, SGI251010, SGI251011, SGI251012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55534[] = {SGI251013, SGI251014, SGI251015, SGI0, SGI251016, SGI251017, SGI251018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55535[] = {SGI251019, SGI251020, SGI251021, SGI0, SGI251022, SGI251023, SGI251024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55536[] = {SGI251025, SGI251026, SGI251027, SGI0, SGI251028, SGI251029, SGI251030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55537[] = {SGI251031, SGI251032, SGI251033, SGI0, SGI251034, SGI251035, SGI251036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55538[] = {SGI251037, SGI251038, SGI251039, SGI0, SGI251040, SGI251041, SGI251042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55539[] = {SGI251043, SGI251044, SGI251045, SGI0, SGI251046, SGI251047, SGI251048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55540[] = {SGI251049, SGI0, SGI251050, SGI0, SGI0, SGI251051, SGI251052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55541[] = {SGI251053, SGI0, SGI251054, SGI0, SGI251055, SGI251056, SGI251057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55542[] = {SGI251058, SGI251059, SGI251060, SGI0, SGI251061, SGI251062, SGI251063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55543[] = {SGI251064, SGI251065, SGI251066, SGI0, SGI251067, SGI251068, SGI251069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55544[] = {SGI251070, SGI251071, SGI251072, SGI0, SGI251073, SGI251074, SGI251075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55545[] = {SGI251076, SGI251077, SGI251078, SGI0, SGI251079, SGI251080, SGI251081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55546[] = {SGI251082, SGI251083, NULL};
const Segment * const SG55547[] = {SGI251084, SGI251085, NULL};
const Segment * const SG55548[] = {SGI251086, SGI251087, NULL};
const Segment * const SG55549[] = {SGI251088, SGI251089, NULL};
const Segment * const SG55550[] = {SGI251090, SGI251091, NULL};
const Segment * const SG55551[] = {SGI251092, SGI251092, NULL};
const Segment * const SG55552[] = {SGI251093, SGI251094, NULL};
const Segment * const SG55553[] = {SGI251095, SGI251096, NULL};
const Segment * const SG55554[] = {SGI251097, SGI251098, NULL};
const Segment * const SG55555[] = {SGI251099, SGI251099, NULL};
const Segment * const SG55556[] = {SGI251100, SGI251101, NULL};
const Segment * const SG55557[] = {SGI251102, SGI251102, NULL};
const Segment * const SG55558[] = {SGI251103, SGI251103, NULL};
const Segment * const SG55559[] = {SGI251104, SGI251105, NULL};
const Segment * const SG55560[] = {SGI251106, SGI251106, NULL};
const Segment * const SG55561[] = {SGI251107, SGI251108, NULL};
const Segment * const SG55562[] = {SGI251109, SGI251110, NULL};
const Segment * const SG55563[] = {SGI251111, SGI251111, NULL};
const Segment * const SG55564[] = {SGI251112, SGI251112, NULL};
const Segment * const SG55565[] = {SGI251113, SGI251114, NULL};
const Segment * const SG55566[] = {SGI251115, SGI251116, NULL};
const Segment * const SG55567[] = {SGI251117, SGI251118, NULL};
const Segment * const SG55568[] = {SGI251119, SGI251120, NULL};
const Segment * const SG55569[] = {SGI251121, SGI251121, NULL};
const Segment * const SG55570[] = {SGI251122, SGI251122, NULL};
const Segment * const SG55571[] = {SGI251123, SGI251123, NULL};
const Segment * const SG55572[] = {SGI251124, SGI251125, NULL};
const Segment * const SG55573[] = {SGI251126, SGI251127, NULL};
const Segment * const SG55574[] = {SGI251128, SGI251128, NULL};
const Segment * const SG55575[] = {SGI251129, SGI251130, NULL};
const Segment * const SG55576[] = {SGI251131, SGI251131, NULL};
const Segment * const SG55577[] = {SGI251132, SGI251133, NULL};
const Segment * const SG55578[] = {SGI251134, SGI251135, NULL};
const Segment * const SG55579[] = {SGI251136, SGI251137, NULL};
const Segment * const SG55580[] = {SGI251138, SGI251139, NULL};
const Segment * const SG55581[] = {SGI251140, SGI251141, NULL};
const Segment * const SG55582[] = {SGI251142, SGI251143, NULL};
const Segment * const SG55583[] = {SGI251144, SGI251145, NULL};
const Segment * const SG55584[] = {SGI251146, SGI251147, NULL};
const Segment * const SG55585[] = {SGI251148, SGI251149, NULL};
const Segment * const SG55586[] = {SGI251150, SGI251151, NULL};
const Segment * const SG55587[] = {SGI251152, SGI251153, NULL};
const Segment * const SG55588[] = {SGI251154, SGI251154, NULL};
const Segment * const SG55589[] = {SGI251155, SGI251156, NULL};
const Segment * const SG55590[] = {SGI251157, SGI251158, NULL};
const Segment * const SG55591[] = {SGI251159, SGI251160, NULL};
const Segment * const SG55592[] = {SGI251161, SGI251161, NULL};
const Segment * const SG55593[] = {SGI251162, SGI251163, NULL};
const Segment * const SG55594[] = {SGI251164, SGI251165, NULL};
const Segment * const SG55595[] = {SGI251166, SGI251167, NULL};
const Segment * const SG55596[] = {SGI251168, SGI251169, NULL};
const Segment * const SG55597[] = {SGI251170, SGI251171, NULL};
const Segment * const SG55598[] = {SGI251172, SGI251173, NULL};
const Segment * const SG55599[] = {SGI251174, SGI251175, NULL};
const Segment * const SG55600[] = {SGI251176, SGI251177, NULL};
const Segment * const SG55601[] = {SGI251178, SGI251179, NULL};
const Segment * const SG55602[] = {SGI251180, SGI251180, NULL};
const Segment * const SG55603[] = {SGI251181, SGI251182, NULL};
const Segment * const SG55604[] = {SGI251183, SGI251184, NULL};
const Segment * const SG55605[] = {SGI251185, SGI251186, NULL};
const Segment * const SG55606[] = {SGI251187, SGI251188, NULL};
const Segment * const SG55607[] = {SGI251189, SGI251190, NULL};
const Segment * const SG55608[] = {SGI251191, SGI251191, NULL};
const Segment * const SG55609[] = {SGI251192, SGI251193, NULL};
const Segment * const SG55610[] = {SGI251194, SGI251194, NULL};
const Segment * const SG55611[] = {SGI251195, SGI251196, NULL};
const Segment * const SG55612[] = {SGI251197, SGI251198, NULL};
const Segment * const SG55613[] = {SGI251199, SGI251200, NULL};
const Segment * const SG55614[] = {SGI251201, SGI251202, NULL};
const Segment * const SG55615[] = {SGI251203, SGI251203, NULL};
const Segment * const SG55616[] = {SGI251204, SGI251204, NULL};
const Segment * const SG55617[] = {SGI251205, SGI251206, NULL};
const Segment * const SG55618[] = {SGI251207, SGI251208, NULL};
const Segment * const SG55619[] = {SGI251209, SGI251210, NULL};
const Segment * const SG55620[] = {SGI251211, SGI251212, NULL};
const Segment * const SG55621[] = {SGI251213, SGI251214, NULL};
const Segment * const SG55622[] = {SGI251215, SGI251216, NULL};
const Segment * const SG55623[] = {SGI251217, SGI251218, NULL};
const Segment * const SG55624[] = {SGI251219, SGI251220, NULL};
const Segment * const SG55625[] = {SGI251221, SGI251222, NULL};
const Segment * const SG55626[] = {SGI251223, SGI251224, NULL};
const Segment * const SG55627[] = {SGI251225, SGI251226, NULL};
const Segment * const SG55628[] = {SGI251227, SGI251228, NULL};
const Segment * const SG55629[] = {SGI251229, SGI251230, NULL};
const Segment * const SG55630[] = {SGI251231, SGI251232, NULL};
const Segment * const SG55631[] = {SGI251233, SGI251234, NULL};
const Segment * const SG55632[] = {SGI251235, SGI251236, NULL};
const Segment * const SG55633[] = {SGI251237, SGI251238, NULL};
const Segment * const SG55634[] = {SGI251239, SGI251240, NULL};
const Segment * const SG55635[] = {SGI251241, SGI251242, NULL};
const Segment * const SG55636[] = {SGI251245, SGI251246, NULL};
const Segment * const SG55637[] = {SGI251243, SGI251244, SGI251247, SGI0, SGI251248, SGI251249, SGI251250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55638[] = {SGI251251, SGI251252, NULL};
const Segment * const SG55639[] = {SGI251253, SGI251254, SGI251255, SGI0, SGI251256, SGI251257, SGI251258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55640[] = {SGI251259, SGI251260, NULL};
const Segment * const SG55641[] = {SGI251265, SGI251266, NULL};
const Segment * const SG55642[] = {SGI251261, SGI0, SGI251262, SGI0, SGI251263, SGI251264, SGI251267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55643[] = {SGI251268, SGI251268, NULL};
const Segment * const SG55644[] = {SGI251275, SGI251275, NULL};
const Segment * const SG55645[] = {SGI251269, SGI251270, SGI251271, SGI0, SGI251272, SGI251273, SGI251274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55646[] = {SGI251276, SGI251277, NULL};
const Segment * const SG55647[] = {SGI251278, SGI251280, SGI251281, SGI0, SGI251283, SGI251284, SGI251285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55648[] = {SGI251279, SGI0, SGI251282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251286, SGI251287, SGI251287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55649[] = {SGI251288, SGI251289, NULL};
const Segment * const SG55650[] = {SGI251290, SGI251291, SGI251292, SGI0, SGI251293, SGI251294, SGI251295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55651[] = {SGI251296, SGI251297, NULL};
const Segment * const SG55652[] = {SGI251298, SGI251299, SGI251300, SGI0, SGI251301, SGI251302, SGI251303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55653[] = {SGI251304, SGI251305, NULL};
const Segment * const SG55654[] = {SGI251306, SGI251307, NULL};
const Segment * const SG55655[] = {SGI251308, SGI251309, NULL};
const Segment * const SG55656[] = {SGI251310, SGI251311, NULL};
const Segment * const SG55657[] = {SGI251312, SGI251313, NULL};
const Segment * const SG55658[] = {SGI251314, SGI251314, NULL};
const Segment * const SG55659[] = {SGI251315, SGI251316, NULL};
const Segment * const SG55660[] = {SGI251317, SGI251318, NULL};
const Segment * const SG55661[] = {SGI251319, SGI251320, NULL};
const Segment * const SG55662[] = {SGI251321, SGI251322, NULL};
const Segment * const SG55663[] = {SGI251323, SGI251324, NULL};
const Segment * const SG55664[] = {SGI251325, SGI251326, NULL};
const Segment * const SG55665[] = {SGI251327, SGI251327, NULL};
const Segment * const SG55666[] = {SGI251328, SGI251329, NULL};
const Segment * const SG55667[] = {SGI251330, SGI251331, NULL};
const Segment * const SG55668[] = {SGI251332, SGI251333, NULL};
const Segment * const SG55669[] = {SGI251334, SGI251335, NULL};
const Segment * const SG55670[] = {SGI251336, SGI251337, NULL};
const Segment * const SG55671[] = {SGI251338, SGI251339, NULL};
const Segment * const SG55672[] = {SGI251340, SGI251341, NULL};
const Segment * const SG55673[] = {SGI251342, SGI251343, NULL};
const Segment * const SG55674[] = {SGI251344, SGI251345, NULL};
const Segment * const SG55675[] = {SGI251346, SGI251347, NULL};
const Segment * const SG55676[] = {SGI251348, SGI251348, NULL};
const Segment * const SG55677[] = {SGI251349, SGI251350, NULL};
const Segment * const SG55678[] = {SGI251351, SGI251352, NULL};
const Segment * const SG55679[] = {SGI251353, SGI251353, NULL};
const Segment * const SG55680[] = {SGI251354, SGI251355, NULL};
const Segment * const SG55681[] = {SGI251356, SGI251357, NULL};
const Segment * const SG55682[] = {SGI251358, SGI251359, NULL};
const Segment * const SG55683[] = {SGI251360, SGI251360, NULL};
const Segment * const SG55684[] = {SGI251361, SGI251362, NULL};
const Segment * const SG55685[] = {SGI251363, SGI251364, NULL};
const Segment * const SG55686[] = {SGI251365, SGI251366, NULL};
const Segment * const SG55687[] = {SGI251367, SGI251368, NULL};
const Segment * const SG55688[] = {SGI251369, SGI251370, NULL};
const Segment * const SG55689[] = {SGI251371, SGI251371, NULL};
const Segment * const SG55690[] = {SGI251372, SGI251373, NULL};
const Segment * const SG55691[] = {SGI251374, SGI251375, NULL};
const Segment * const SG55692[] = {SGI251376, SGI251377, NULL};
const Segment * const SG55693[] = {SGI251378, SGI251379, NULL};
const Segment * const SG55694[] = {SGI251380, SGI251381, NULL};
const Segment * const SG55695[] = {SGI251382, SGI251383, NULL};
const Segment * const SG55696[] = {SGI251384, SGI251385, NULL};
const Segment * const SG55697[] = {SGI251386, SGI251386, NULL};
const Segment * const SG55698[] = {SGI251387, SGI251388, NULL};
const Segment * const SG55699[] = {SGI251389, SGI251389, NULL};
const Segment * const SG55700[] = {SGI251390, SGI251390, NULL};
const Segment * const SG55701[] = {SGI251391, SGI251392, NULL};
const Segment * const SG55702[] = {SGI251393, SGI251393, NULL};
const Segment * const SG55703[] = {SGI251396, SGI251398, NULL};
const Segment * const SG55704[] = {SGI251394, SGI251395, SGI251397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251399, SGI0, SGI0, SGI0, SGI251400, SGI251400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55705[] = {SGI251401, SGI251402, NULL};
const Segment * const SG55706[] = {SGI251403, SGI251404, NULL};
const Segment * const SG55707[] = {SGI251405, SGI251406, NULL};
const Segment * const SG55708[] = {SGI251407, SGI251407, NULL};
const Segment * const SG55709[] = {SGI251408, SGI251409, NULL};
const Segment * const SG55710[] = {SGI251410, SGI251410, NULL};
const Segment * const SG55711[] = {SGI251411, SGI251411, NULL};
const Segment * const SG55712[] = {SGI251412, SGI251413, NULL};
const Segment * const SG55713[] = {SGI251414, SGI251414, NULL};
const Segment * const SG55714[] = {SGI251415, SGI251416, NULL};
const Segment * const SG55715[] = {SGI251417, SGI251418, SGI251419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251420, SGI251420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251421, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55716[] = {SGI251422, SGI0, SGI251423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251424, SGI251425, SGI251425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55717[] = {SGI251426, SGI251427, SGI251428, SGI0, SGI251429, SGI251430, SGI251431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55718[] = {SGI251432, SGI251433, SGI251434, SGI0, SGI251435, SGI251436, SGI251437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55719[] = {SGI251438, SGI251439, SGI251440, SGI0, SGI251441, SGI251442, SGI251443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55720[] = {SGI251444, SGI251445, SGI251446, SGI0, SGI251447, SGI251448, SGI251449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55721[] = {SGI251450, SGI251451, SGI251452, SGI0, SGI251453, SGI251454, SGI251455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55722[] = {SGI251456, SGI251457, SGI251458, SGI0, SGI251459, SGI251460, SGI251461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55723[] = {SGI251462, SGI251463, SGI251464, SGI0, SGI251465, SGI251466, SGI251467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55724[] = {SGI251468, SGI251469, SGI0, SGI0, SGI251470, SGI251471, SGI251472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55725[] = {SGI251473, SGI251474, SGI251475, SGI0, SGI251476, SGI251477, SGI251478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55726[] = {SGI251479, SGI251480, SGI251481, SGI0, SGI251482, SGI251483, SGI251484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55727[] = {SGI251485, SGI251486, SGI251487, SGI0, SGI251488, SGI251489, SGI251490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55728[] = {SGI251491, SGI251492, SGI251493, SGI0, SGI251494, SGI251495, SGI251496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55729[] = {SGI251498, SGI251500, SGI251502, SGI0, SGI251503, SGI251504, SGI251505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55730[] = {SGI251497, SGI251499, SGI251501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251506, SGI251506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251507, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55731[] = {SGI251508, SGI251509, SGI251510, SGI0, SGI251511, SGI251512, SGI251513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55732[] = {SGI251514, SGI251515, SGI0, SGI0, SGI251516, SGI251517, SGI251518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55733[] = {SGI251519, SGI251520, SGI251521, SGI0, SGI251522, SGI251523, SGI251524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55734[] = {SGI251525, SGI251526, SGI251527, SGI0, SGI251528, SGI251529, SGI251530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55735[] = {SGI251534, SGI251535, SGI251536, SGI0, SGI251537, SGI251538, SGI251540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55736[] = {SGI251531, SGI251532, SGI251533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251539, SGI251539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251541, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55737[] = {SGI251542, SGI251543, SGI0, SGI0, SGI251544, SGI251545, SGI251546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55738[] = {SGI251547, SGI251548, SGI251549, SGI0, SGI251550, SGI251551, SGI251552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55739[] = {SGI251553, SGI251554, SGI251555, SGI0, SGI251556, SGI251557, SGI251558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55740[] = {SGI251559, SGI251560, SGI251561, SGI0, SGI251562, SGI251563, SGI251564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55741[] = {SGI251565, SGI251566, SGI251567, SGI0, SGI251568, SGI251569, SGI251570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55742[] = {SGI251571, SGI251572, SGI251573, SGI0, SGI251574, SGI251575, SGI251576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55743[] = {SGI251577, SGI251578, SGI251579, SGI0, SGI251580, SGI251581, SGI251582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55744[] = {SGI251583, SGI251584, SGI251585, SGI0, SGI251586, SGI251587, SGI251588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55745[] = {SGI251589, SGI251590, SGI251591, SGI0, SGI251592, SGI251593, SGI251594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55746[] = {SGI251595, SGI251596, SGI251597, SGI0, SGI251598, SGI251599, SGI251600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55747[] = {SGI251601, SGI251602, SGI251603, SGI0, SGI251604, SGI251605, SGI251606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55748[] = {SGI251607, SGI251608, SGI251609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251610, SGI251610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251611, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55749[] = {SGI251612, SGI251613, SGI251614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251615, SGI0, SGI0, SGI0, SGI251616, SGI251616, SGI251617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55750[] = {SGI251618, SGI251619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55751[] = {SGI251620, SGI251621, SGI251622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251623, SGI0, SGI0, SGI0, SGI251624, SGI251624, SGI251625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55752[] = {SGI251626, SGI251627, NULL};
const Segment * const SG55753[] = {SGI251628, SGI251629, NULL};
const Segment * const SG55754[] = {SGI251630, SGI251631, NULL};
const Segment * const SG55755[] = {SGI251632, SGI251633, NULL};
const Segment * const SG55756[] = {SGI251634, SGI251635, NULL};
const Segment * const SG55757[] = {SGI251636, SGI251637, NULL};
const Segment * const SG55758[] = {SGI251638, SGI251639, NULL};
const Segment * const SG55759[] = {SGI251640, SGI251641, NULL};
const Segment * const SG55760[] = {SGI251642, SGI251643, NULL};
const Segment * const SG55761[] = {SGI251644, SGI251645, NULL};
const Segment * const SG55762[] = {SGI251646, SGI251647, NULL};
const Segment * const SG55763[] = {SGI251648, SGI251649, NULL};
const Segment * const SG55764[] = {SGI251650, SGI251651, NULL};
const Segment * const SG55765[] = {SGI251652, SGI251652, NULL};
const Segment * const SG55766[] = {SGI251654, SGI251654, NULL};
const Segment * const SG55767[] = {SGI251659, SGI251660, NULL};
const Segment * const SG55768[] = {SGI251653, SGI251655, SGI251656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251657, SGI0, SGI0, SGI0, SGI251658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251661, SGI0, SGI0, NULL};
const Segment * const SG55769[] = {SGI251662, SGI251663, NULL};
const Segment * const SG55770[] = {SGI251664, SGI251665, NULL};
const Segment * const SG55771[] = {SGI251666, SGI251667, NULL};
const Segment * const SG55772[] = {SGI251668, SGI251669, NULL};
const Segment * const SG55773[] = {SGI251670, SGI251670, NULL};
const Segment * const SG55774[] = {SGI251671, SGI251672, NULL};
const Segment * const SG55775[] = {SGI251673, SGI251674, NULL};
const Segment * const SG55776[] = {SGI251675, SGI251676, NULL};
const Segment * const SG55777[] = {SGI251677, SGI251678, NULL};
const Segment * const SG55778[] = {SGI251679, SGI251680, NULL};
const Segment * const SG55779[] = {SGI251682, SGI251683, NULL};
const Segment * const SG55780[] = {SGI251681, SGI0, SGI251684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251685, SGI251686, SGI251686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55781[] = {SGI251687, SGI251688, NULL};
const Segment * const SG55782[] = {SGI251689, SGI251690, NULL};
const Segment * const SG55783[] = {SGI251691, SGI251692, NULL};
const Segment * const SG55784[] = {SGI251693, SGI251693, NULL};
const Segment * const SG55785[] = {SGI251694, SGI251695, NULL};
const Segment * const SG55786[] = {SGI251696, SGI251697, NULL};
const Segment * const SG55787[] = {SGI251698, SGI251699, NULL};
const Segment * const SG55788[] = {SGI251700, SGI251701, NULL};
const Segment * const SG55789[] = {SGI251702, SGI251702, NULL};
const Segment * const SG55790[] = {SGI251703, SGI251704, NULL};
const Segment * const SG55791[] = {SGI251708, SGI251708, NULL};
const Segment * const SG55792[] = {SGI251705, SGI251706, SGI251707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251709, SGI0, SGI0, SGI0, SGI251710, SGI251710, SGI0, SGI251711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55793[] = {SGI251712, SGI251712, NULL};
const Segment * const SG55794[] = {SGI251713, SGI251714, NULL};
const Segment * const SG55795[] = {SGI251715, SGI251716, NULL};
const Segment * const SG55796[] = {SGI251717, SGI251718, NULL};
const Segment * const SG55797[] = {SGI251719, SGI251720, NULL};
const Segment * const SG55798[] = {SGI251721, SGI251721, NULL};
const Segment * const SG55799[] = {SGI251726, SGI251727, NULL};
const Segment * const SG55800[] = {SGI251722, SGI251723, SGI251724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251725, SGI0, SGI0, SGI0, SGI251728, SGI251728, SGI0, SGI251729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55801[] = {SGI251730, SGI251730, NULL};
const Segment * const SG55802[] = {SGI251731, SGI251731, NULL};
const Segment * const SG55803[] = {SGI251732, SGI251732, NULL};
const Segment * const SG55804[] = {SGI251733, SGI251734, NULL};
const Segment * const SG55805[] = {SGI251735, SGI251736, NULL};
const Segment * const SG55806[] = {SGI251737, SGI251738, NULL};
const Segment * const SG55807[] = {SGI251739, SGI251740, NULL};
const Segment * const SG55808[] = {SGI251741, SGI251742, NULL};
const Segment * const SG55809[] = {SGI251743, SGI251744, NULL};
const Segment * const SG55810[] = {SGI251745, SGI251746, NULL};
const Segment * const SG55811[] = {SGI251747, SGI251748, NULL};
const Segment * const SG55812[] = {SGI251749, SGI251750, NULL};
const Segment * const SG55813[] = {SGI251757, SGI251758, NULL};
const Segment * const SG55814[] = {SGI251751, SGI251752, SGI251753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251754, SGI0, SGI0, SGI0, SGI251755, SGI251755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251756, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55815[] = {SGI251759, SGI251760, NULL};
const Segment * const SG55816[] = {SGI251761, SGI251762, NULL};
const Segment * const SG55817[] = {SGI251763, SGI251764, NULL};
const Segment * const SG55818[] = {SGI251765, SGI251766, NULL};
const Segment * const SG55819[] = {SGI251767, SGI251768, NULL};
const Segment * const SG55820[] = {SGI251769, SGI251770, NULL};
const Segment * const SG55821[] = {SGI251771, SGI251772, NULL};
const Segment * const SG55822[] = {SGI251773, SGI251774, NULL};
const Segment * const SG55823[] = {SGI251775, SGI251776, NULL};
const Segment * const SG55824[] = {SGI251777, SGI251778, NULL};
const Segment * const SG55825[] = {SGI251779, SGI251780, NULL};
const Segment * const SG55826[] = {SGI251781, SGI251782, NULL};
const Segment * const SG55827[] = {SGI251783, SGI251784, NULL};
const Segment * const SG55828[] = {SGI251785, SGI251786, NULL};
const Segment * const SG55829[] = {SGI251787, SGI251788, NULL};
const Segment * const SG55830[] = {SGI251789, SGI251790, NULL};
const Segment * const SG55831[] = {SGI251791, SGI251791, NULL};
const Segment * const SG55832[] = {SGI251792, SGI251792, NULL};
const Segment * const SG55833[] = {SGI251793, SGI251793, NULL};
const Segment * const SG55834[] = {SGI251794, SGI251795, NULL};
const Segment * const SG55835[] = {SGI251796, SGI251796, NULL};
const Segment * const SG55836[] = {SGI251797, SGI251798, NULL};
const Segment * const SG55837[] = {SGI251799, SGI251800, NULL};
const Segment * const SG55838[] = {SGI251801, SGI251802, NULL};
const Segment * const SG55839[] = {SGI251803, SGI251804, NULL};
const Segment * const SG55840[] = {SGI251805, SGI251805, NULL};
const Segment * const SG55841[] = {SGI251810, SGI251811, NULL};
const Segment * const SG55842[] = {SGI251806, SGI251807, SGI251808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251809, SGI0, SGI0, SGI0, SGI251812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55843[] = {SGI251813, SGI251814, NULL};
const Segment * const SG55844[] = {SGI251815, SGI251816, NULL};
const Segment * const SG55845[] = {SGI251817, SGI251818, NULL};
const Segment * const SG55846[] = {SGI251819, SGI251820, NULL};
const Segment * const SG55847[] = {SGI251821, SGI251822, NULL};
const Segment * const SG55848[] = {SGI251823, SGI251824, NULL};
const Segment * const SG55849[] = {SGI251825, SGI251826, NULL};
const Segment * const SG55850[] = {SGI251827, SGI251828, NULL};
const Segment * const SG55851[] = {SGI251832, SGI251832, NULL};
const Segment * const SG55852[] = {SGI251829, SGI251830, SGI251831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251833, SGI0, SGI0, SGI0, SGI251834, SGI251834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55853[] = {SGI251835, SGI251836, NULL};
const Segment * const SG55854[] = {SGI251837, SGI251837, NULL};
const Segment * const SG55855[] = {SGI251838, SGI251839, NULL};
const Segment * const SG55856[] = {SGI251840, SGI251841, NULL};
const Segment * const SG55857[] = {SGI251842, SGI251843, NULL};
const Segment * const SG55858[] = {SGI251844, SGI251845, NULL};
const Segment * const SG55859[] = {SGI251846, SGI251847, NULL};
const Segment * const SG55860[] = {SGI251848, SGI251848, NULL};
const Segment * const SG55861[] = {SGI251849, SGI251850, NULL};
const Segment * const SG55862[] = {SGI251851, SGI251851, NULL};
const Segment * const SG55863[] = {SGI251852, SGI251853, NULL};
const Segment * const SG55864[] = {SGI251854, SGI251854, NULL};
const Segment * const SG55865[] = {SGI251855, SGI251855, NULL};
const Segment * const SG55866[] = {SGI251856, SGI251857, NULL};
const Segment * const SG55867[] = {SGI251858, SGI251858, NULL};
const Segment * const SG55868[] = {SGI251859, SGI251860, NULL};
const Segment * const SG55869[] = {SGI251861, SGI251862, NULL};
const Segment * const SG55870[] = {SGI251863, SGI251864, NULL};
const Segment * const SG55871[] = {SGI251866, SGI251867, NULL};
const Segment * const SG55872[] = {SGI251870, SGI251870, NULL};
const Segment * const SG55873[] = {SGI251865, SGI0, SGI251868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251869, SGI251871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55874[] = {SGI251872, SGI251873, NULL};
const Segment * const SG55875[] = {SGI251875, SGI251876, NULL};
const Segment * const SG55876[] = {SGI251880, SGI251880, NULL};
const Segment * const SG55877[] = {SGI251874, SGI251877, SGI251878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251879, SGI0, SGI0, SGI0, SGI251881, SGI251881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55878[] = {SGI251882, SGI251883, NULL};
const Segment * const SG55879[] = {SGI251884, SGI251885, NULL};
const Segment * const SG55880[] = {SGI251886, SGI251887, NULL};
const Segment * const SG55881[] = {SGI251888, SGI251888, NULL};
const Segment * const SG55882[] = {SGI251889, SGI251889, NULL};
const Segment * const SG55883[] = {SGI251890, SGI251891, NULL};
const Segment * const SG55884[] = {SGI251892, SGI251893, NULL};
const Segment * const SG55885[] = {SGI251894, SGI251895, NULL};
const Segment * const SG55886[] = {SGI251896, SGI251897, NULL};
const Segment * const SG55887[] = {SGI251898, SGI251899, NULL};
const Segment * const SG55888[] = {SGI251900, SGI251901, NULL};
const Segment * const SG55889[] = {SGI251902, SGI251903, NULL};
const Segment * const SG55890[] = {SGI251904, SGI251905, NULL};
const Segment * const SG55891[] = {SGI251906, SGI251907, NULL};
const Segment * const SG55892[] = {SGI251908, SGI251909, NULL};
const Segment * const SG55893[] = {SGI251910, SGI251910, NULL};
const Segment * const SG55894[] = {SGI251911, SGI251912, NULL};
const Segment * const SG55895[] = {SGI251913, SGI251914, NULL};
const Segment * const SG55896[] = {SGI251915, SGI251916, NULL};
const Segment * const SG55897[] = {SGI251917, SGI251917, NULL};
const Segment * const SG55898[] = {SGI251918, SGI251919, NULL};
const Segment * const SG55899[] = {SGI251920, SGI251920, NULL};
const Segment * const SG55900[] = {SGI251921, SGI251922, NULL};
const Segment * const SG55901[] = {SGI251923, SGI251923, NULL};
const Segment * const SG55902[] = {SGI251924, SGI251924, NULL};
const Segment * const SG55903[] = {SGI251929, SGI251930, NULL};
const Segment * const SG55904[] = {SGI251925, SGI251926, SGI251927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251928, SGI251928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251931, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55905[] = {SGI251932, SGI251933, NULL};
const Segment * const SG55906[] = {SGI251934, SGI251935, NULL};
const Segment * const SG55907[] = {SGI251939, SGI251941, NULL};
const Segment * const SG55908[] = {SGI251936, SGI0, SGI251937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251938, SGI251940, SGI251940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55909[] = {SGI251942, SGI251943, NULL};
const Segment * const SG55910[] = {SGI251947, SGI251948, NULL};
const Segment * const SG55911[] = {SGI251944, SGI251945, SGI251946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251949, SGI251949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251950, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55912[] = {SGI251951, SGI251952, NULL};
const Segment * const SG55913[] = {SGI251953, SGI251954, NULL};
const Segment * const SG55914[] = {SGI251955, SGI251956, NULL};
const Segment * const SG55915[] = {SGI251957, SGI251958, NULL};
const Segment * const SG55916[] = {SGI251959, SGI251959, NULL};
const Segment * const SG55917[] = {SGI251961, SGI251962, NULL};
const Segment * const SG55918[] = {SGI251960, SGI251963, SGI251964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251965, SGI251965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251966, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55919[] = {SGI251967, SGI251968, NULL};
const Segment * const SG55920[] = {SGI251969, SGI251970, NULL};
const Segment * const SG55921[] = {SGI251971, SGI251972, NULL};
const Segment * const SG55922[] = {SGI251975, SGI251976, NULL};
const Segment * const SG55923[] = {SGI251973, SGI0, SGI251974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251977, SGI251978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55924[] = {SGI251979, SGI251980, NULL};
const Segment * const SG55925[] = {SGI251981, SGI251982, NULL};
const Segment * const SG55926[] = {SGI251983, SGI251984, NULL};
const Segment * const SG55927[] = {SGI251985, SGI251986, NULL};
const Segment * const SG55928[] = {SGI251987, SGI251988, NULL};
const Segment * const SG55929[] = {SGI251989, SGI251990, NULL};
const Segment * const SG55930[] = {SGI251991, SGI251992, NULL};
const Segment * const SG55931[] = {SGI251994, SGI251995, NULL};
const Segment * const SG55932[] = {SGI251993, SGI0, SGI251996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI251997, SGI251998, SGI251998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55933[] = {SGI251999, SGI252000, NULL};
const Segment * const SG55934[] = {SGI252001, SGI252002, NULL};
const Segment * const SG55935[] = {SGI252003, SGI252003, NULL};
const Segment * const SG55936[] = {SGI252004, SGI252005, NULL};
const Segment * const SG55937[] = {SGI252006, SGI252007, NULL};
const Segment * const SG55938[] = {SGI252008, SGI252008, NULL};
const Segment * const SG55939[] = {SGI252009, SGI0, SGI252010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI252011, SGI252012, SGI252012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55940[] = {SGI252013, SGI252014, NULL};
const Segment * const SG55941[] = {SGI252015, SGI252016, NULL};
const Segment * const SG55942[] = {SGI252018, SGI252020, NULL};
const Segment * const SG55943[] = {SGI252017, SGI0, SGI252019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI252021, SGI252022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55944[] = {SGI252023, SGI252024, NULL};
const Segment * const SG55945[] = {SGI252025, SGI252026, NULL};
const Segment * const SG55946[] = {SGI252027, SGI252028, NULL};
const Segment * const SG55947[] = {SGI252029, SGI252030, NULL};
const Segment * const SG55948[] = {SGI252031, SGI252032, NULL};
const Segment * const SG55949[] = {SGI252033, SGI252034, NULL};
const Segment * const SG55950[] = {SGI252035, SGI252035, NULL};
const Segment * const SG55951[] = {SGI252036, SGI252037, NULL};
const Segment * const SG55952[] = {SGI252038, SGI252039, NULL};
const Segment * const SG55953[] = {SGI252040, SGI252041, NULL};
const Segment * const SG55954[] = {SGI252042, SGI252043, NULL};
const Segment * const SG55955[] = {SGI252044, SGI252045, NULL};
const Segment * const SG55956[] = {SGI252046, SGI252047, NULL};
const Segment * const SG55957[] = {SGI252048, SGI252049, NULL};
const Segment * const SG55958[] = {SGI252050, SGI252050, NULL};
const Segment * const SG55959[] = {SGI252051, SGI252052, NULL};
const Segment * const SG55960[] = {SGI252053, SGI252053, NULL};
const Segment * const SG55961[] = {SGI252054, SGI252055, NULL};
const Segment * const SG55962[] = {SGI252056, SGI252057, NULL};
const Segment * const SG55963[] = {SGI252058, SGI252058, NULL};
const Segment * const SG55964[] = {SGI252059, SGI252059, NULL};
const Segment * const SG55965[] = {SGI252060, SGI252061, NULL};
const Segment * const SG55966[] = {SGI252062, SGI252063, NULL};
const Segment * const SG55967[] = {SGI252064, SGI252065, NULL};
const Segment * const SG55968[] = {SGI252066, SGI252066, NULL};
const Segment * const SG55969[] = {SGI252067, SGI252067, NULL};
const Segment * const SG55970[] = {SGI252068, SGI252069, NULL};
const Segment * const SG55971[] = {SGI252070, SGI252070, NULL};
const Segment * const SG55972[] = {SGI252071, SGI252072, NULL};
const Segment * const SG55973[] = {SGI252073, SGI252074, NULL};
const Segment * const SG55974[] = {SGI252075, SGI252076, NULL};
const Segment * const SG55975[] = {SGI252077, SGI252077, NULL};
const Segment * const SG55976[] = {SGI252078, SGI252079, NULL};
const Segment * const SG55977[] = {SGI252080, SGI252081, NULL};
const Segment * const SG55978[] = {SGI252082, SGI252083, NULL};
const Segment * const SG55979[] = {SGI252084, SGI252085, NULL};
const Segment * const SG55980[] = {SGI252086, SGI252087, NULL};
const Segment * const SG55981[] = {SGI252088, SGI252088, NULL};
const Segment * const SG55982[] = {SGI252089, SGI252089, NULL};
const Segment * const SG55983[] = {SGI252090, SGI252091, NULL};
const Segment * const SG55984[] = {SGI252092, SGI252093, NULL};
const Segment * const SG55985[] = {SGI252094, SGI252095, NULL};
const Segment * const SG55986[] = {SGI252096, SGI252096, NULL};
const Segment * const SG55987[] = {SGI252097, SGI252098, NULL};
const Segment * const SG55988[] = {SGI252099, SGI0, SGI252100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI252101, SGI252102, SGI252102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG55989[] = {SGI252103, SGI252104, NULL};
const Segment * const SG55990[] = {SGI252105, SGI252106, NULL};
const Segment * const SG55991[] = {SGI252107, SGI252108, NULL};
const Segment * const SG55992[] = {SGI252109, SGI252110, NULL};
const Segment * const SG55993[] = {SGI252111, SGI252112, NULL};
const Segment * const SG55994[] = {SGI252113, SGI252114, NULL};
const Segment * const SG55995[] = {SGI252115, SGI252116, NULL};
const Segment * const SG55996[] = {SGI252117, SGI252118, NULL};
const Segment * const SG55997[] = {SGI252119, SGI252120, NULL};
const Segment * const SG55998[] = {SGI252121, SGI252122, NULL};
const Segment * const SG55999[] = {SGI252123, SGI252124, NULL};
const Segment * const SG56000[] = {SGI252125, SGI252126, NULL};
const Segment * const SG56001[] = {SGI252127, SGI252128, NULL};
const Segment * const SG56002[] = {SGI252129, SGI252130, NULL};
const Segment * const SG56003[] = {SGI252131, SGI252131, NULL};
const Segment * const SG56004[] = {SGI252132, SGI252133, NULL};
const Segment * const SG56005[] = {SGI252134, SGI252134, NULL};
const Segment * const SG56006[] = {SGI252135, SGI252136, NULL};
const Segment * const SG56007[] = {SGI252137, SGI252138, NULL};
const Segment * const SG56008[] = {SGI252139, SGI252140, NULL};
const Segment * const SG56009[] = {SGI252141, SGI252142, NULL};
const Segment * const SG56010[] = {SGI252143, SGI252144, NULL};
const Segment * const SG56011[] = {SGI252145, SGI252146, NULL};
const Segment * const SG56012[] = {SGI252147, SGI252148, NULL};
const Segment * const SG56013[] = {SGI252149, SGI252149, NULL};
const Segment * const SG56014[] = {SGI252150, SGI252151, NULL};
const Segment * const SG56015[] = {SGI252152, SGI252153, NULL};
const Segment * const SG56016[] = {SGI252154, SGI252155, NULL};
const Segment * const SG56017[] = {SGI252156, SGI252156, NULL};
const Segment * const SG56018[] = {SGI252157, SGI252158, NULL};
const Segment * const SG56019[] = {SGI252159, SGI252160, NULL};
const Segment * const SG56020[] = {SGI252161, SGI252162, NULL};
const Segment * const SG56021[] = {SGI252163, SGI252164, NULL};
const Segment * const SG56022[] = {SGI252165, SGI252166, NULL};
const Segment * const SG56023[] = {SGI252167, SGI252168, NULL};
const Segment * const SG56024[] = {SGI252169, SGI252170, NULL};
const Segment * const SG56025[] = {SGI252171, SGI252172, NULL};
const Segment * const SG56026[] = {SGI252173, SGI252174, NULL};
const Segment * const SG56027[] = {SGI252175, SGI252176, NULL};
const Segment * const SG56028[] = {SGI252177, SGI252178, NULL};
const Segment * const SG56029[] = {SGI252179, SGI252179, NULL};
const Segment * const SG56030[] = {SGI252180, SGI252181, NULL};
const Segment * const SG56031[] = {SGI252182, SGI252183, NULL};
const Segment * const SG56032[] = {SGI252184, SGI252185, NULL};
const Segment * const SG56033[] = {SGI252186, SGI252187, NULL};
const Segment * const SG56034[] = {SGI252188, SGI252189, NULL};
const Segment * const SG56035[] = {SGI252190, SGI252191, NULL};
const Segment * const SG56036[] = {SGI252192, SGI252193, NULL};
const Segment * const SG56037[] = {SGI252194, SGI252195, NULL};
const Segment * const SG56038[] = {SGI252196, SGI252197, NULL};
const Segment * const SG56039[] = {SGI252198, SGI252198, NULL};
const Segment * const SG56040[] = {SGI252199, SGI252200, NULL};
const Segment * const SG56041[] = {SGI252201, SGI252202, NULL};
const Segment * const SG56042[] = {SGI252203, SGI252204, NULL};
const Segment * const SG56043[] = {SGI252205, SGI252205, NULL};
const Segment * const SG56044[] = {SGI252206, SGI252207, NULL};
const Segment * const SG56045[] = {SGI252208, SGI252208, NULL};
const Segment * const SG56046[] = {SGI252209, SGI252210, NULL};
const Segment * const SG56047[] = {SGI252211, SGI252212, NULL};
const Segment * const SG56048[] = {SGI252213, SGI252214, NULL};
const Segment * const SG56049[] = {SGI252215, SGI252216, NULL};
const Segment * const SG56050[] = {SGI252217, SGI252217, NULL};
const Segment * const SG56051[] = {SGI252218, SGI252219, NULL};
const Segment * const SG56052[] = {SGI252220, SGI252221, NULL};
const Segment * const SG56053[] = {SGI252222, SGI252223, NULL};
const Segment * const SG56054[] = {SGI252224, SGI252224, NULL};
const Segment * const SG56055[] = {SGI252225, SGI252226, NULL};
const Segment * const SG56056[] = {SGI252227, SGI252227, NULL};
const Segment * const SG56057[] = {SGI252228, SGI252229, NULL};
const Segment * const SG56058[] = {SGI252230, SGI252230, NULL};
const Segment * const SG56059[] = {SGI252231, SGI252232, NULL};
const Segment * const SG56060[] = {SGI252233, SGI252234, NULL};
const Segment * const SG56061[] = {SGI252235, SGI252236, NULL};
const Segment * const SG56062[] = {SGI252237, SGI252238, NULL};
const Segment * const SG56063[] = {SGI252239, SGI252240, NULL};
const Segment * const SG56064[] = {SGI252241, SGI252241, NULL};
const Segment * const SG56065[] = {SGI252242, SGI252243, NULL};
const Segment * const SG56066[] = {SGI252244, SGI252244, NULL};
const Segment * const SG56067[] = {SGI252245, SGI252246, NULL};
const Segment * const SG56068[] = {SGI252247, SGI252247, NULL};
const Segment * const SG56069[] = {SGI252248, SGI252249, NULL};
const Segment * const SG56070[] = {SGI252250, SGI252251, NULL};
const Segment * const SG56071[] = {SGI252252, SGI252253, NULL};
const Segment * const SG56072[] = {SGI252254, SGI252254, NULL};
const Segment * const SG56073[] = {SGI252255, SGI252256, NULL};
const Segment * const SG56074[] = {SGI252257, SGI252258, NULL};
const Segment * const SG56075[] = {SGI252259, SGI252260, NULL};
const Segment * const SG56076[] = {SGI252261, SGI252262, NULL};
const Segment * const SG56077[] = {SGI252266, SGI252267, NULL};
const Segment * const SG56078[] = {SGI252263, SGI252264, SGI252265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI252268, SGI0, SGI0, SGI0, SGI252269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI252270, NULL};
const Segment * const SG56079[] = {SGI252271, SGI252272, NULL};
const Segment * const SG56080[] = {SGI252273, SGI252274, NULL};
const Segment * const SG56081[] = {SGI252275, SGI252276, NULL};
const Segment * const SG56082[] = {SGI252277, SGI252278, NULL};
const Segment * const SG56083[] = {SGI252279, SGI252280, NULL};
const Segment * const SG56084[] = {SGI252281, SGI252282, NULL};
const Segment * const SG56085[] = {SGI252283, SGI252284, NULL};
const Segment * const SG56086[] = {SGI252285, SGI252286, NULL};
const Segment * const SG56087[] = {SGI252287, SGI252287, NULL};
const Segment * const SG56088[] = {SGI252288, SGI0, SGI252289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI252290, SGI252291, SGI252291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56089[] = {SGI252292, SGI252293, NULL};
const Segment * const SG56090[] = {SGI252294, SGI252295, NULL};
const Segment * const SG56091[] = {SGI252299, SGI252300, NULL};
const Segment * const SG56092[] = {SGI252296, SGI0, SGI252297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI252298, SGI252301, SGI252301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56093[] = {SGI252302, SGI252303, NULL};
const Segment * const SG56094[] = {SGI252305, SGI252307, NULL};
const Segment * const SG56095[] = {SGI252304, SGI0, SGI252306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI252308, SGI252309, SGI252309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56096[] = {SGI252310, SGI252311, NULL};
const Segment * const SG56097[] = {SGI252312, SGI252313, NULL};
const Segment * const SG56098[] = {SGI252316, SGI252317, NULL};
const Segment * const SG56099[] = {SGI252314, SGI0, SGI252315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI252318, SGI252319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56100[] = {SGI252320, SGI252321, NULL};
const Segment * const SG56101[] = {SGI252322, SGI252323, NULL};
const Segment * const SG56102[] = {SGI252324, SGI252325, NULL};
const Segment * const SG56103[] = {SGI252326, SGI252327, NULL};
const Segment * const SG56104[] = {SGI252328, SGI252329, NULL};
const Segment * const SG56105[] = {SGI252330, SGI252331, NULL};
const Segment * const SG56106[] = {SGI252332, SGI252332, NULL};
const Segment * const SG56107[] = {SGI252333, SGI252334, NULL};
const Segment * const SG56108[] = {SGI252335, SGI252336, NULL};
const Segment * const SG56109[] = {SGI252337, SGI252338, NULL};
const Segment * const SG56110[] = {SGI252339, SGI252340, NULL};
const Segment * const SG56111[] = {SGI252341, SGI252342, NULL};
const Segment * const SG56112[] = {SGI252343, SGI252344, NULL};
const Segment * const SG56113[] = {SGI252345, SGI252346, NULL};
const Segment * const SG56114[] = {SGI252347, SGI252347, NULL};
const Segment * const SG56115[] = {SGI252348, SGI252349, NULL};
const Segment * const SG56116[] = {SGI252350, SGI252351, NULL};
const Segment * const SG56117[] = {SGI252352, SGI252353, NULL};
const Segment * const SG56118[] = {SGI252354, SGI252355, NULL};
const Segment * const SG56119[] = {SGI252356, SGI252357, NULL};
const Segment * const SG56120[] = {SGI252358, SGI252359, NULL};
const Segment * const SG56121[] = {SGI252360, SGI252361, NULL};
const Segment * const SG56122[] = {SGI252362, SGI252363, NULL};
const Segment * const SG56123[] = {SGI252364, SGI252365, NULL};
const Segment * const SG56124[] = {SGI252366, SGI252367, NULL};
const Segment * const SG56125[] = {SGI252368, SGI252369, NULL};
const Segment * const SG56126[] = {SGI252370, SGI252371, NULL};
const Segment * const SG56127[] = {SGI252372, SGI252373, NULL};
const Segment * const SG56128[] = {SGI252374, SGI252375, NULL};
const Segment * const SG56129[] = {SGI252376, SGI252377, NULL};
const Segment * const SG56130[] = {SGI252378, SGI252379, NULL};
const Segment * const SG56131[] = {SGI252380, SGI252381, NULL};
const Segment * const SG56132[] = {SGI252382, SGI252383, NULL};
const Segment * const SG56133[] = {SGI252384, SGI252385, NULL};
const Segment * const SG56134[] = {SGI252386, SGI252387, NULL};
const Segment * const SG56135[] = {SGI252388, SGI252389, NULL};
const Segment * const SG56136[] = {SGI252390, SGI252391, NULL};
const Segment * const SG56137[] = {SGI252392, SGI252393, NULL};
const Segment * const SG56138[] = {SGI252394, SGI252395, NULL};
const Segment * const SG56139[] = {SGI252396, SGI252397, NULL};
const Segment * const SG56140[] = {SGI252398, SGI252399, NULL};
const Segment * const SG56141[] = {SGI252400, SGI252400, NULL};
const Segment * const SG56142[] = {SGI252401, SGI252402, NULL};
const Segment * const SG56143[] = {SGI252403, SGI252404, NULL};
const Segment * const SG56144[] = {SGI252405, SGI252406, NULL};
const Segment * const SG56145[] = {SGI252407, SGI252408, NULL};
const Segment * const SG56146[] = {SGI252409, SGI252409, NULL};
const Segment * const SG56147[] = {SGI252410, SGI252411, NULL};
const Segment * const SG56148[] = {SGI252412, SGI252412, NULL};
const Segment * const SG56149[] = {SGI252413, SGI252414, NULL};
const Segment * const SG56150[] = {SGI252415, SGI252415, NULL};
const Segment * const SG56151[] = {SGI252416, SGI252417, NULL};
const Segment * const SG56152[] = {SGI252418, SGI252419, NULL};
const Segment * const SG56153[] = {SGI252420, SGI252420, NULL};
const Segment * const SG56154[] = {SGI252421, SGI252422, NULL};
const Segment * const SG56155[] = {SGI252423, SGI252424, NULL};
const Segment * const SG56156[] = {SGI252425, SGI252426, NULL};
const Segment * const SG56157[] = {SGI252427, SGI252428, NULL};
const Segment * const SG56158[] = {SGI252429, SGI252430, NULL};
const Segment * const SG56159[] = {SGI252431, SGI252432, NULL};
const Segment * const SG56160[] = {SGI252433, SGI252434, NULL};
const Segment * const SG56161[] = {SGI252435, SGI252436, NULL};
const Segment * const SG56162[] = {SGI252437, SGI252437, NULL};
const Segment * const SG56163[] = {SGI252438, SGI252438, NULL};
const Segment * const SG56164[] = {SGI252439, SGI252440, NULL};
const Segment * const SG56165[] = {SGI252441, SGI252442, NULL};
const Segment * const SG56166[] = {SGI252443, SGI252444, NULL};
const Segment * const SG56167[] = {SGI252445, SGI252446, NULL};
const Segment * const SG56168[] = {SGI252447, SGI252448, NULL};
const Segment * const SG56169[] = {SGI252449, SGI252450, NULL};
const Segment * const SG56170[] = {SGI252451, SGI252452, NULL};
const Segment * const SG56171[] = {SGI252453, SGI252454, NULL};
const Segment * const SG56172[] = {SGI252455, SGI252456, NULL};
const Segment * const SG56173[] = {SGI252457, SGI252457, NULL};
const Segment * const SG56174[] = {SGI252458, SGI252459, NULL};
const Segment * const SG56175[] = {SGI252460, SGI252461, NULL};
const Segment * const SG56176[] = {SGI252462, SGI252463, NULL};
const Segment * const SG56177[] = {SGI252464, SGI252465, NULL};
const Segment * const SG56178[] = {SGI252466, SGI252467, NULL};
const Segment * const SG56179[] = {SGI252468, SGI252469, NULL};
const Segment * const SG56180[] = {SGI252470, SGI252471, NULL};
const Segment * const SG56181[] = {SGI252472, SGI252473, NULL};
const Segment * const SG56182[] = {SGI252474, SGI252475, NULL};
const Segment * const SG56183[] = {SGI252476, SGI252477, NULL};
const Segment * const SG56184[] = {SGI252478, SGI252479, NULL};
const Segment * const SG56185[] = {SGI252480, SGI252481, NULL};
const Segment * const SG56186[] = {SGI252482, SGI252483, NULL};
const Segment * const SG56187[] = {SGI252484, SGI252485, NULL};
const Segment * const SG56188[] = {SGI252486, SGI252487, NULL};
const Segment * const SG56189[] = {SGI252488, SGI252489, NULL};
const Segment * const SG56190[] = {SGI252490, SGI252491, NULL};
const Segment * const SG56191[] = {SGI252492, SGI252493, NULL};
const Segment * const SG56192[] = {SGI252494, SGI252495, NULL};
const Segment * const SG56193[] = {SGI252496, SGI252497, NULL};
const Segment * const SG56194[] = {SGI252498, SGI252499, NULL};
const Segment * const SG56195[] = {SGI252500, SGI252501, NULL};
const Segment * const SG56196[] = {SGI252502, SGI252503, NULL};
const Segment * const SG56197[] = {SGI252504, SGI252505, NULL};
const Segment * const SG56198[] = {SGI252506, SGI252506, NULL};
const Segment * const SG56199[] = {SGI252507, SGI252508, NULL};
const Segment * const SG56200[] = {SGI252509, SGI252510, NULL};
const Segment * const SG56201[] = {SGI252511, SGI252512, NULL};
const Segment * const SG56202[] = {SGI252513, SGI252514, NULL};
const Segment * const SG56203[] = {SGI252515, SGI252516, NULL};
const Segment * const SG56204[] = {SGI252517, SGI252518, NULL};
const Segment * const SG56205[] = {SGI252519, SGI252520, NULL};
const Segment * const SG56206[] = {SGI252521, SGI252522, NULL};
const Segment * const SG56207[] = {SGI252523, SGI252524, NULL};
const Segment * const SG56208[] = {SGI252525, SGI252526, NULL};
const Segment * const SG56209[] = {SGI252527, SGI252528, NULL};
const Segment * const SG56210[] = {SGI252529, SGI252530, NULL};
const Segment * const SG56211[] = {SGI252531, SGI252532, NULL};
const Segment * const SG56212[] = {SGI252533, SGI252534, NULL};
const Segment * const SG56213[] = {SGI252535, SGI252536, NULL};
const Segment * const SG56214[] = {SGI252537, SGI252538, NULL};
const Segment * const SG56215[] = {SGI252539, SGI252540, NULL};
const Segment * const SG56216[] = {SGI252541, SGI252541, NULL};
const Segment * const SG56217[] = {SGI252542, SGI252543, NULL};
const Segment * const SG56218[] = {SGI252544, SGI252545, NULL};
const Segment * const SG56219[] = {SGI252546, SGI252546, NULL};
const Segment * const SG56220[] = {SGI252547, SGI252548, NULL};
const Segment * const SG56221[] = {SGI252549, SGI252550, NULL};
const Segment * const SG56222[] = {SGI252551, SGI252552, NULL};
const Segment * const SG56223[] = {SGI252553, SGI252554, NULL};
const Segment * const SG56224[] = {SGI252555, SGI252556, NULL};
const Segment * const SG56225[] = {SGI252557, SGI252558, NULL};
const Segment * const SG56226[] = {SGI252559, SGI252560, NULL};
const Segment * const SG56227[] = {SGI252561, SGI252562, NULL};
const Segment * const SG56228[] = {SGI252563, SGI252564, NULL};
const Segment * const SG56229[] = {SGI252565, SGI252566, NULL};
const Segment * const SG56230[] = {SGI252567, SGI252568, NULL};
const Segment * const SG56231[] = {SGI252569, SGI252570, NULL};
const Segment * const SG56232[] = {SGI252571, SGI252572, NULL};
const Segment * const SG56233[] = {SGI252573, SGI252574, NULL};
const Segment * const SG56234[] = {SGI252575, SGI252575, NULL};
const Segment * const SG56235[] = {SGI252576, SGI252577, NULL};
const Segment * const SG56236[] = {SGI252578, SGI252579, NULL};
const Segment * const SG56237[] = {SGI252580, SGI252580, NULL};
const Segment * const SG56238[] = {SGI252581, SGI252582, NULL};
const Segment * const SG56239[] = {SGI252583, SGI252583, NULL};
const Segment * const SG56240[] = {SGI252584, SGI252584, NULL};
const Segment * const SG56241[] = {SGI252585, SGI252586, NULL};
const Segment * const SG56242[] = {SGI252587, SGI252588, NULL};
const Segment * const SG56243[] = {SGI252589, SGI252589, NULL};
const Segment * const SG56244[] = {SGI252590, SGI252591, NULL};
const Segment * const SG56245[] = {SGI252592, SGI252592, NULL};
const Segment * const SG56246[] = {SGI252593, SGI252594, NULL};
const Segment * const SG56247[] = {SGI252595, SGI252596, NULL};
const Segment * const SG56248[] = {SGI252597, SGI252597, NULL};
const Segment * const SG56249[] = {SGI252598, SGI252599, NULL};
const Segment * const SG56250[] = {SGI252600, SGI252601, NULL};
const Segment * const SG56251[] = {SGI252602, SGI252603, NULL};
const Segment * const SG56252[] = {SGI252604, SGI252605, NULL};
const Segment * const SG56253[] = {SGI252606, SGI252607, NULL};
const Segment * const SG56254[] = {SGI252608, SGI252608, NULL};
const Segment * const SG56255[] = {SGI252609, SGI252610, NULL};
const Segment * const SG56256[] = {SGI252611, SGI252612, NULL};
const Segment * const SG56257[] = {SGI252613, SGI252614, NULL};
const Segment * const SG56258[] = {SGI252615, SGI252616, NULL};
const Segment * const SG56259[] = {SGI252617, SGI252618, NULL};
const Segment * const SG56260[] = {SGI252619, SGI252620, NULL};
const Segment * const SG56261[] = {SGI252621, SGI252621, NULL};
const Segment * const SG56262[] = {SGI252622, SGI252623, NULL};
const Segment * const SG56263[] = {SGI252624, SGI252625, SGI252626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI252627, SGI0, SGI0, SGI0, SGI252628, SGI0, SGI252629, SGI252630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56264[] = {SGI252631, SGI252632, NULL};
const Segment * const SG56265[] = {SGI252633, SGI252634, NULL};
const Segment * const SG56266[] = {SGI252635, SGI252636, NULL};
const Segment * const SG56267[] = {SGI252637, SGI252638, NULL};
const Segment * const SG56268[] = {SGI252639, SGI252639, NULL};
const Segment * const SG56269[] = {SGI252640, SGI252641, NULL};
const Segment * const SG56270[] = {SGI252642, SGI252643, NULL};
const Segment * const SG56271[] = {SGI252644, SGI252645, NULL};
const Segment * const SG56272[] = {SGI252646, SGI252647, NULL};
const Segment * const SG56273[] = {SGI252648, SGI252649, NULL};
const Segment * const SG56274[] = {SGI252650, SGI252651, NULL};
const Segment * const SG56275[] = {SGI252652, SGI252653, NULL};
const Segment * const SG56276[] = {SGI252654, SGI252655, NULL};
const Segment * const SG56277[] = {SGI252656, SGI252657, NULL};
const Segment * const SG56278[] = {SGI252658, SGI252659, NULL};
const Segment * const SG56279[] = {SGI252660, SGI252661, NULL};
const Segment * const SG56280[] = {SGI252662, SGI252663, NULL};
const Segment * const SG56281[] = {SGI252664, SGI252665, NULL};
const Segment * const SG56282[] = {SGI252666, SGI252667, NULL};
const Segment * const SG56283[] = {SGI252668, SGI252669, NULL};
const Segment * const SG56284[] = {SGI252670, SGI252671, NULL};
const Segment * const SG56285[] = {SGI252672, SGI252672, NULL};
const Segment * const SG56286[] = {SGI252673, SGI252674, NULL};
const Segment * const SG56287[] = {SGI252675, SGI252676, NULL};
const Segment * const SG56288[] = {SGI252677, SGI252678, NULL};
const Segment * const SG56289[] = {SGI252679, SGI252680, NULL};
const Segment * const SG56290[] = {SGI252681, SGI252682, NULL};
const Segment * const SG56291[] = {SGI252683, SGI252684, NULL};
const Segment * const SG56292[] = {SGI252685, SGI252686, NULL};
const Segment * const SG56293[] = {SGI252687, SGI252688, NULL};
const Segment * const SG56294[] = {SGI252689, SGI252690, NULL};
const Segment * const SG56295[] = {SGI252691, SGI252692, NULL};
const Segment * const SG56296[] = {SGI252693, SGI252694, NULL};
const Segment * const SG56297[] = {SGI252695, SGI252696, NULL};
const Segment * const SG56298[] = {SGI252697, SGI252698, NULL};
const Segment * const SG56299[] = {SGI252699, SGI252700, NULL};
const Segment * const SG56300[] = {SGI252701, SGI252702, NULL};
const Segment * const SG56301[] = {SGI252703, SGI252704, NULL};
const Segment * const SG56302[] = {SGI252705, SGI252706, NULL};
const Segment * const SG56303[] = {SGI252707, SGI252707, NULL};
const Segment * const SG56304[] = {SGI252708, SGI252709, NULL};
const Segment * const SG56305[] = {SGI252710, SGI252711, NULL};
const Segment * const SG56306[] = {SGI252712, SGI252713, NULL};
const Segment * const SG56307[] = {SGI252714, SGI252714, NULL};
const Segment * const SG56308[] = {SGI252715, SGI252716, NULL};
const Segment * const SG56309[] = {SGI252717, SGI252718, NULL};
const Segment * const SG56310[] = {SGI252719, SGI252720, NULL};
const Segment * const SG56311[] = {SGI252721, SGI252722, NULL};
const Segment * const SG56312[] = {SGI252723, SGI252724, NULL};
const Segment * const SG56313[] = {SGI252725, SGI252726, NULL};
const Segment * const SG56314[] = {SGI252727, SGI252728, NULL};
const Segment * const SG56315[] = {SGI252729, SGI252730, NULL};
const Segment * const SG56316[] = {SGI252731, SGI252732, NULL};
const Segment * const SG56317[] = {SGI252733, SGI252734, NULL};
const Segment * const SG56318[] = {SGI252735, SGI252736, NULL};
const Segment * const SG56319[] = {SGI252737, SGI252738, NULL};
const Segment * const SG56320[] = {SGI252740, SGI252742, NULL};
const Segment * const SG56321[] = {SGI252739, SGI0, SGI252741, SGI0, SGI252743, SGI252744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56322[] = {SGI252746, SGI252747, NULL};
const Segment * const SG56323[] = {SGI252745, SGI0, SGI252748, SGI0, SGI252749, SGI252750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56324[] = {SGI252751, SGI252752, NULL};
const Segment * const SG56325[] = {SGI252756, SGI252757, NULL};
const Segment * const SG56326[] = {SGI252753, SGI0, SGI252754, SGI0, SGI252755, SGI252758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56327[] = {SGI252759, SGI252760, NULL};
const Segment * const SG56328[] = {SGI252762, SGI252763, NULL};
const Segment * const SG56329[] = {SGI252761, SGI0, SGI252764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56330[] = {SGI252765, SGI252766, NULL};
const Segment * const SG56331[] = {SGI252768, SGI252769, NULL};
const Segment * const SG56332[] = {SGI252767, SGI0, SGI252770, SGI0, SGI252771, SGI252772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56333[] = {SGI252773, SGI252773, NULL};
const Segment * const SG56334[] = {SGI252775, SGI252776, NULL};
const Segment * const SG56335[] = {SGI252774, SGI0, SGI252777, SGI0, SGI252778, SGI252779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56336[] = {SGI252780, SGI252780, NULL};
const Segment * const SG56337[] = {SGI252783, SGI252785, NULL};
const Segment * const SG56338[] = {SGI252781, SGI0, SGI252782, SGI0, SGI252784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56339[] = {SGI252788, SGI252789, NULL};
const Segment * const SG56340[] = {SGI252786, SGI0, SGI252787, SGI0, SGI252790, SGI252791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56341[] = {SGI252792, SGI252793, NULL};
const Segment * const SG56342[] = {SGI252794, SGI0, SGI252795, SGI0, SGI252796, SGI252797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56343[] = {SGI252798, SGI252799, NULL};
const Segment * const SG56344[] = {SGI252802, SGI252804, NULL};
const Segment * const SG56345[] = {SGI252800, SGI0, SGI252801, SGI0, SGI252803, SGI252805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56346[] = {SGI252806, SGI252807, NULL};
const Segment * const SG56347[] = {SGI252812, SGI252813, NULL};
const Segment * const SG56348[] = {SGI252808, SGI0, SGI252809, SGI0, SGI252810, SGI252811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56349[] = {SGI252814, SGI252815, NULL};
const Segment * const SG56350[] = {SGI252816, SGI0, SGI252817, SGI0, SGI252818, SGI252819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56351[] = {SGI252820, SGI252821, NULL};
const Segment * const SG56352[] = {SGI252822, SGI0, SGI252823, SGI0, SGI252824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56353[] = {SGI252825, SGI252826, NULL};
const Segment * const SG56354[] = {SGI252827, SGI252828, NULL};
const Segment * const SG56355[] = {SGI252831, SGI252831, NULL};
const Segment * const SG56356[] = {SGI252829, SGI0, SGI252830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56357[] = {SGI252834, SGI252835, NULL};
const Segment * const SG56358[] = {SGI252832, SGI0, SGI252833, SGI0, SGI0, SGI252836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56359[] = {SGI252840, SGI252841, NULL};
const Segment * const SG56360[] = {SGI252837, SGI0, SGI252838, SGI0, SGI252839, SGI252842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56361[] = {SGI252843, SGI252844, NULL};
const Segment * const SG56362[] = {SGI252845, SGI0, SGI252846, SGI0, SGI252847, SGI252848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56363[] = {SGI252849, SGI252850, NULL};
const Segment * const SG56364[] = {SGI252851, SGI0, SGI252852, SGI0, SGI252853, SGI252854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56365[] = {SGI252855, SGI252856, NULL};
const Segment * const SG56366[] = {SGI252857, SGI0, SGI252858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56367[] = {SGI252859, SGI252860, NULL};
const Segment * const SG56368[] = {SGI252861, SGI0, SGI252862, SGI0, SGI252863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56369[] = {SGI252864, SGI0, SGI252865, SGI0, SGI252866, SGI252867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56370[] = {SGI252868, SGI252868, NULL};
const Segment * const SG56371[] = {SGI252872, SGI252873, NULL};
const Segment * const SG56372[] = {SGI252869, SGI0, SGI0, SGI0, SGI252870, SGI252871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56373[] = {SGI252876, SGI252877, NULL};
const Segment * const SG56374[] = {SGI252874, SGI0, SGI252875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56375[] = {SGI252878, SGI252879, NULL};
const Segment * const SG56376[] = {SGI252880, SGI0, SGI252881, SGI0, SGI252882, SGI252883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56377[] = {SGI252884, SGI252885, NULL};
const Segment * const SG56378[] = {SGI252886, SGI252887, SGI252888, SGI0, SGI252889, SGI252890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56379[] = {SGI252891, SGI252892, NULL};
const Segment * const SG56380[] = {SGI252893, SGI0, SGI252894, SGI0, SGI252895, SGI252896, SGI252897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56381[] = {SGI252898, SGI252899, NULL};
const Segment * const SG56382[] = {SGI252902, SGI252904, NULL};
const Segment * const SG56383[] = {SGI252900, SGI0, SGI252901, SGI0, SGI0, SGI252903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56384[] = {SGI252905, SGI252906, NULL};
const Segment * const SG56385[] = {SGI252907, SGI0, SGI252908, SGI0, SGI252909, SGI252910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56386[] = {SGI252911, SGI252912, NULL};
const Segment * const SG56387[] = {SGI252913, SGI0, SGI252914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56388[] = {SGI252915, SGI252915, NULL};
const Segment * const SG56389[] = {SGI252916, SGI0, SGI252917, SGI0, SGI0, SGI252918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56390[] = {SGI252919, SGI252921, NULL};
const Segment * const SG56391[] = {SGI252920, SGI252922, SGI252923, SGI0, SGI252924, SGI252925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56392[] = {SGI252926, SGI252926, NULL};
const Segment * const SG56393[] = {SGI252928, SGI252928, NULL};
const Segment * const SG56394[] = {SGI252933, SGI252934, NULL};
const Segment * const SG56395[] = {SGI252927, SGI252929, SGI252930, SGI0, SGI252931, SGI252932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56396[] = {SGI252935, SGI252936, NULL};
const Segment * const SG56397[] = {SGI252937, SGI252937, NULL};
const Segment * const SG56398[] = {SGI252938, SGI252939, NULL};
const Segment * const SG56399[] = {SGI252940, SGI252940, NULL};
const Segment * const SG56400[] = {SGI252941, SGI252942, NULL};
const Segment * const SG56401[] = {SGI252943, SGI252944, NULL};
const Segment * const SG56402[] = {SGI252945, SGI252945, NULL};
const Segment * const SG56403[] = {SGI252946, SGI252947, NULL};
const Segment * const SG56404[] = {SGI252948, SGI252948, NULL};
const Segment * const SG56405[] = {SGI252949, SGI252950, NULL};
const Segment * const SG56406[] = {SGI252951, SGI252952, NULL};
const Segment * const SG56407[] = {SGI252953, SGI252954, NULL};
const Segment * const SG56408[] = {SGI252955, SGI252956, NULL};
const Segment * const SG56409[] = {SGI252957, SGI252957, NULL};
const Segment * const SG56410[] = {SGI252958, SGI252959, NULL};
const Segment * const SG56411[] = {SGI252960, SGI252961, NULL};
const Segment * const SG56412[] = {SGI252962, SGI252962, NULL};
const Segment * const SG56413[] = {SGI252963, SGI252964, NULL};
const Segment * const SG56414[] = {SGI252965, SGI252966, NULL};
const Segment * const SG56415[] = {SGI252967, SGI252968, NULL};
const Segment * const SG56416[] = {SGI252969, SGI252969, NULL};
const Segment * const SG56417[] = {SGI252970, SGI252971, NULL};
const Segment * const SG56418[] = {SGI252972, SGI252973, NULL};
const Segment * const SG56419[] = {SGI252974, SGI252975, NULL};
const Segment * const SG56420[] = {SGI252976, SGI252977, NULL};
const Segment * const SG56421[] = {SGI252978, SGI252979, NULL};
const Segment * const SG56422[] = {SGI252980, SGI252981, NULL};
const Segment * const SG56423[] = {SGI252982, SGI252983, NULL};
const Segment * const SG56424[] = {SGI252986, SGI252986, NULL};
const Segment * const SG56425[] = {SGI252984, SGI0, SGI252985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI252987, SGI252988, SGI252988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56426[] = {SGI252989, SGI252990, NULL};
const Segment * const SG56427[] = {SGI252991, SGI0, SGI252992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI252993, SGI252994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56428[] = {SGI252995, SGI252996, NULL};
const Segment * const SG56429[] = {SGI252997, SGI252998, NULL};
const Segment * const SG56430[] = {SGI252999, SGI253000, NULL};
const Segment * const SG56431[] = {SGI253001, SGI253002, NULL};
const Segment * const SG56432[] = {SGI253003, SGI253004, NULL};
const Segment * const SG56433[] = {SGI253005, SGI253006, NULL};
const Segment * const SG56434[] = {SGI253007, SGI253008, NULL};
const Segment * const SG56435[] = {SGI253009, SGI253010, NULL};
const Segment * const SG56436[] = {SGI253011, SGI253012, NULL};
const Segment * const SG56437[] = {SGI253013, SGI253013, NULL};
const Segment * const SG56438[] = {SGI253014, SGI253015, NULL};
const Segment * const SG56439[] = {SGI253016, SGI253017, NULL};
const Segment * const SG56440[] = {SGI253018, SGI253019, NULL};
const Segment * const SG56441[] = {SGI253020, SGI253021, NULL};
const Segment * const SG56442[] = {SGI253022, SGI253023, NULL};
const Segment * const SG56443[] = {SGI253024, SGI253025, NULL};
const Segment * const SG56444[] = {SGI253026, SGI253026, NULL};
const Segment * const SG56445[] = {SGI253027, SGI253028, NULL};
const Segment * const SG56446[] = {SGI253029, SGI253029, NULL};
const Segment * const SG56447[] = {SGI253030, SGI253030, NULL};
const Segment * const SG56448[] = {SGI253031, SGI253032, NULL};
const Segment * const SG56449[] = {SGI253033, SGI253033, NULL};
const Segment * const SG56450[] = {SGI253034, SGI253035, NULL};
const Segment * const SG56451[] = {SGI253036, SGI253037, NULL};
const Segment * const SG56452[] = {SGI253038, SGI253039, NULL};
const Segment * const SG56453[] = {SGI253040, SGI253041, NULL};
const Segment * const SG56454[] = {SGI253042, SGI253043, NULL};
const Segment * const SG56455[] = {SGI253044, SGI253045, NULL};
const Segment * const SG56456[] = {SGI253046, SGI253047, NULL};
const Segment * const SG56457[] = {SGI253048, SGI253049, NULL};
const Segment * const SG56458[] = {SGI253050, SGI253050, NULL};
const Segment * const SG56459[] = {SGI253051, SGI253052, NULL};
const Segment * const SG56460[] = {SGI253053, SGI253054, NULL};
const Segment * const SG56461[] = {SGI253055, SGI253056, NULL};
const Segment * const SG56462[] = {SGI253057, SGI253058, NULL};
const Segment * const SG56463[] = {SGI253059, SGI253060, NULL};
const Segment * const SG56464[] = {SGI253061, SGI253062, NULL};
const Segment * const SG56465[] = {SGI253063, SGI253064, NULL};
const Segment * const SG56466[] = {SGI253065, SGI253066, NULL};
const Segment * const SG56467[] = {SGI253067, SGI253068, NULL};
const Segment * const SG56468[] = {SGI253069, SGI253070, NULL};
const Segment * const SG56469[] = {SGI253071, SGI253072, NULL};
const Segment * const SG56470[] = {SGI253073, SGI253074, NULL};
const Segment * const SG56471[] = {SGI253075, SGI253076, NULL};
const Segment * const SG56472[] = {SGI253077, SGI253078, NULL};
const Segment * const SG56473[] = {SGI253079, SGI253080, NULL};
const Segment * const SG56474[] = {SGI253081, SGI253082, NULL};
const Segment * const SG56475[] = {SGI253083, SGI253084, NULL};
const Segment * const SG56476[] = {SGI253085, SGI253086, NULL};
const Segment * const SG56477[] = {SGI253087, SGI253088, NULL};
const Segment * const SG56478[] = {SGI253089, SGI253090, NULL};
const Segment * const SG56479[] = {SGI253091, SGI253092, NULL};
const Segment * const SG56480[] = {SGI253093, SGI253094, NULL};
const Segment * const SG56481[] = {SGI253095, SGI253096, NULL};
const Segment * const SG56482[] = {SGI253097, SGI253098, NULL};
const Segment * const SG56483[] = {SGI253099, SGI253100, NULL};
const Segment * const SG56484[] = {SGI253101, SGI253102, NULL};
const Segment * const SG56485[] = {SGI253103, SGI253104, NULL};
const Segment * const SG56486[] = {SGI253105, SGI253106, NULL};
const Segment * const SG56487[] = {SGI253107, SGI253108, NULL};
const Segment * const SG56488[] = {SGI253109, SGI253110, NULL};
const Segment * const SG56489[] = {SGI253111, SGI253111, NULL};
const Segment * const SG56490[] = {SGI253112, SGI253113, NULL};
const Segment * const SG56491[] = {SGI253114, SGI253114, NULL};
const Segment * const SG56492[] = {SGI253115, SGI253116, NULL};
const Segment * const SG56493[] = {SGI253117, SGI253118, NULL};
const Segment * const SG56494[] = {SGI253119, SGI253119, NULL};
const Segment * const SG56495[] = {SGI253120, SGI253121, NULL};
const Segment * const SG56496[] = {SGI253122, SGI253123, NULL};
const Segment * const SG56497[] = {SGI253124, SGI253125, NULL};
const Segment * const SG56498[] = {SGI253126, SGI253127, NULL};
const Segment * const SG56499[] = {SGI253128, SGI253129, NULL};
const Segment * const SG56500[] = {SGI253130, SGI253131, NULL};
const Segment * const SG56501[] = {SGI253132, SGI253133, NULL};
const Segment * const SG56502[] = {SGI253134, SGI253135, NULL};
const Segment * const SG56503[] = {SGI253136, SGI253137, NULL};
const Segment * const SG56504[] = {SGI253138, SGI253139, NULL};
const Segment * const SG56505[] = {SGI253140, SGI253141, NULL};
const Segment * const SG56506[] = {SGI253142, SGI253143, NULL};
const Segment * const SG56507[] = {SGI253144, SGI253145, NULL};
const Segment * const SG56508[] = {SGI253146, SGI253147, NULL};
const Segment * const SG56509[] = {SGI253148, SGI253148, NULL};
const Segment * const SG56510[] = {SGI253149, SGI253150, NULL};
const Segment * const SG56511[] = {SGI253151, SGI253152, NULL};
const Segment * const SG56512[] = {SGI253153, SGI253154, NULL};
const Segment * const SG56513[] = {SGI253155, SGI253156, NULL};
const Segment * const SG56514[] = {SGI253157, SGI253158, NULL};
const Segment * const SG56515[] = {SGI253159, SGI253160, NULL};
const Segment * const SG56516[] = {SGI253161, SGI253162, NULL};
const Segment * const SG56517[] = {SGI253163, SGI253164, NULL};
const Segment * const SG56518[] = {SGI253165, SGI253166, NULL};
const Segment * const SG56519[] = {SGI253167, SGI253167, NULL};
const Segment * const SG56520[] = {SGI253168, SGI253169, NULL};
const Segment * const SG56521[] = {SGI253170, SGI253171, NULL};
const Segment * const SG56522[] = {SGI253172, SGI253173, NULL};
const Segment * const SG56523[] = {SGI253174, SGI253174, NULL};
const Segment * const SG56524[] = {SGI253175, SGI253176, NULL};
const Segment * const SG56525[] = {SGI253177, SGI253178, NULL};
const Segment * const SG56526[] = {SGI253179, SGI253180, NULL};
const Segment * const SG56527[] = {SGI253181, SGI253182, NULL};
const Segment * const SG56528[] = {SGI253183, SGI253184, NULL};
const Segment * const SG56529[] = {SGI253185, SGI253186, NULL};
const Segment * const SG56530[] = {SGI253187, SGI253188, NULL};
const Segment * const SG56531[] = {SGI253189, SGI253189, NULL};
const Segment * const SG56532[] = {SGI253190, SGI253190, NULL};
const Segment * const SG56533[] = {SGI253191, SGI253192, NULL};
const Segment * const SG56534[] = {SGI253193, SGI253194, NULL};
const Segment * const SG56535[] = {SGI253195, SGI253196, NULL};
const Segment * const SG56536[] = {SGI253197, SGI253198, NULL};
const Segment * const SG56537[] = {SGI253199, SGI253200, NULL};
const Segment * const SG56538[] = {SGI253201, SGI253202, NULL};
const Segment * const SG56539[] = {SGI253203, SGI253204, NULL};
const Segment * const SG56540[] = {SGI253205, SGI253206, NULL};
const Segment * const SG56541[] = {SGI253207, SGI253208, NULL};
const Segment * const SG56542[] = {SGI253209, SGI253210, NULL};
const Segment * const SG56543[] = {SGI253211, SGI253211, NULL};
const Segment * const SG56544[] = {SGI253212, SGI253213, NULL};
const Segment * const SG56545[] = {SGI253214, SGI253215, NULL};
const Segment * const SG56546[] = {SGI253216, SGI253217, NULL};
const Segment * const SG56547[] = {SGI253218, SGI253218, NULL};
const Segment * const SG56548[] = {SGI253219, SGI253220, NULL};
const Segment * const SG56549[] = {SGI253221, SGI253221, NULL};
const Segment * const SG56550[] = {SGI253222, SGI253223, NULL};
const Segment * const SG56551[] = {SGI253224, SGI253225, NULL};
const Segment * const SG56552[] = {SGI253226, SGI253227, NULL};
const Segment * const SG56553[] = {SGI253228, SGI253229, NULL};
const Segment * const SG56554[] = {SGI253230, SGI253231, NULL};
const Segment * const SG56555[] = {SGI253232, SGI253233, NULL};
const Segment * const SG56556[] = {SGI253234, SGI253235, NULL};
const Segment * const SG56557[] = {SGI253236, SGI253237, NULL};
const Segment * const SG56558[] = {SGI253238, SGI253239, NULL};
const Segment * const SG56559[] = {SGI253240, SGI253241, NULL};
const Segment * const SG56560[] = {SGI253242, SGI253242, NULL};
const Segment * const SG56561[] = {SGI253243, SGI253244, NULL};
const Segment * const SG56562[] = {SGI253245, SGI253245, NULL};
const Segment * const SG56563[] = {SGI253246, SGI253247, NULL};
const Segment * const SG56564[] = {SGI253248, SGI253249, NULL};
const Segment * const SG56565[] = {SGI253250, SGI253251, NULL};
const Segment * const SG56566[] = {SGI253252, SGI253253, NULL};
const Segment * const SG56567[] = {SGI253254, SGI253255, NULL};
const Segment * const SG56568[] = {SGI253256, SGI253257, NULL};
const Segment * const SG56569[] = {SGI253258, SGI253259, NULL};
const Segment * const SG56570[] = {SGI253260, SGI253260, NULL};
const Segment * const SG56571[] = {SGI253261, SGI253262, NULL};
const Segment * const SG56572[] = {SGI253263, SGI253263, NULL};
const Segment * const SG56573[] = {SGI253264, SGI253265, NULL};
const Segment * const SG56574[] = {SGI253266, SGI253267, NULL};
const Segment * const SG56575[] = {SGI253268, SGI253269, NULL};
const Segment * const SG56576[] = {SGI253270, SGI253271, NULL};
const Segment * const SG56577[] = {SGI253272, SGI253273, NULL};
const Segment * const SG56578[] = {SGI253274, SGI253275, NULL};
const Segment * const SG56579[] = {SGI253276, SGI253276, NULL};
const Segment * const SG56580[] = {SGI253277, SGI253278, NULL};
const Segment * const SG56581[] = {SGI253279, SGI253280, NULL};
const Segment * const SG56582[] = {SGI253281, SGI253282, NULL};
const Segment * const SG56583[] = {SGI253283, SGI253284, NULL};
const Segment * const SG56584[] = {SGI253285, SGI253286, NULL};
const Segment * const SG56585[] = {SGI253287, SGI253288, NULL};
const Segment * const SG56586[] = {SGI253289, SGI253290, NULL};
const Segment * const SG56587[] = {SGI253291, SGI253292, NULL};
const Segment * const SG56588[] = {SGI253293, SGI253294, NULL};
const Segment * const SG56589[] = {SGI253295, SGI253296, NULL};
const Segment * const SG56590[] = {SGI253297, SGI253297, NULL};
const Segment * const SG56591[] = {SGI253298, SGI253299, NULL};
const Segment * const SG56592[] = {SGI253300, SGI253301, NULL};
const Segment * const SG56593[] = {SGI253302, SGI253303, NULL};
const Segment * const SG56594[] = {SGI253304, SGI253305, NULL};
const Segment * const SG56595[] = {SGI253306, SGI253307, NULL};
const Segment * const SG56596[] = {SGI253308, SGI253309, NULL};
const Segment * const SG56597[] = {SGI253310, SGI253311, NULL};
const Segment * const SG56598[] = {SGI253312, SGI253313, NULL};
const Segment * const SG56599[] = {SGI253314, SGI253315, NULL};
const Segment * const SG56600[] = {SGI253316, SGI253316, NULL};
const Segment * const SG56601[] = {SGI253317, SGI253318, NULL};
const Segment * const SG56602[] = {SGI253319, SGI253320, NULL};
const Segment * const SG56603[] = {SGI253321, SGI253321, NULL};
const Segment * const SG56604[] = {SGI253322, SGI253323, NULL};
const Segment * const SG56605[] = {SGI253324, SGI253325, NULL};
const Segment * const SG56606[] = {SGI253326, SGI253327, NULL};
const Segment * const SG56607[] = {SGI253328, SGI253329, NULL};
const Segment * const SG56608[] = {SGI253330, SGI253331, NULL};
const Segment * const SG56609[] = {SGI253332, SGI253333, NULL};
const Segment * const SG56610[] = {SGI253334, SGI253334, NULL};
const Segment * const SG56611[] = {SGI253335, SGI253336, NULL};
const Segment * const SG56612[] = {SGI253337, SGI253338, NULL};
const Segment * const SG56613[] = {SGI253339, SGI253340, NULL};
const Segment * const SG56614[] = {SGI253341, SGI253342, NULL};
const Segment * const SG56615[] = {SGI253343, SGI253344, NULL};
const Segment * const SG56616[] = {SGI253345, SGI253346, NULL};
const Segment * const SG56617[] = {SGI253347, SGI253348, NULL};
const Segment * const SG56618[] = {SGI253349, SGI253350, NULL};
const Segment * const SG56619[] = {SGI253351, SGI253352, NULL};
const Segment * const SG56620[] = {SGI253353, SGI253354, NULL};
const Segment * const SG56621[] = {SGI253355, SGI253356, NULL};
const Segment * const SG56622[] = {SGI253357, SGI253358, NULL};
const Segment * const SG56623[] = {SGI253359, SGI253359, NULL};
const Segment * const SG56624[] = {SGI253360, SGI253361, NULL};
const Segment * const SG56625[] = {SGI253362, SGI253363, NULL};
const Segment * const SG56626[] = {SGI253364, SGI253365, NULL};
const Segment * const SG56627[] = {SGI253366, SGI253367, NULL};
const Segment * const SG56628[] = {SGI253368, SGI253369, NULL};
const Segment * const SG56629[] = {SGI253370, SGI253371, NULL};
const Segment * const SG56630[] = {SGI253372, SGI253373, NULL};
const Segment * const SG56631[] = {SGI253374, SGI253375, NULL};
const Segment * const SG56632[] = {SGI253376, SGI253376, NULL};
const Segment * const SG56633[] = {SGI253377, SGI253378, NULL};
const Segment * const SG56634[] = {SGI253379, SGI253380, NULL};
const Segment * const SG56635[] = {SGI253381, SGI253382, NULL};
const Segment * const SG56636[] = {SGI253383, SGI253384, NULL};
const Segment * const SG56637[] = {SGI253385, SGI253386, NULL};
const Segment * const SG56638[] = {SGI253387, SGI253388, NULL};
const Segment * const SG56639[] = {SGI253389, SGI253390, NULL};
const Segment * const SG56640[] = {SGI253391, SGI253392, NULL};
const Segment * const SG56641[] = {SGI253393, SGI253394, NULL};
const Segment * const SG56642[] = {SGI253395, SGI253396, NULL};
const Segment * const SG56643[] = {SGI253397, SGI253398, NULL};
const Segment * const SG56644[] = {SGI253399, SGI253400, NULL};
const Segment * const SG56645[] = {SGI253401, SGI253402, NULL};
const Segment * const SG56646[] = {SGI253403, SGI253404, NULL};
const Segment * const SG56647[] = {SGI253405, SGI253406, NULL};
const Segment * const SG56648[] = {SGI253407, SGI253407, NULL};
const Segment * const SG56649[] = {SGI253408, SGI253409, NULL};
const Segment * const SG56650[] = {SGI253410, SGI253411, NULL};
const Segment * const SG56651[] = {SGI253412, SGI253413, NULL};
const Segment * const SG56652[] = {SGI253414, SGI253415, NULL};
const Segment * const SG56653[] = {SGI253416, SGI253417, NULL};
const Segment * const SG56654[] = {SGI253418, SGI253419, NULL};
const Segment * const SG56655[] = {SGI253420, SGI253421, NULL};
const Segment * const SG56656[] = {SGI253422, SGI253422, NULL};
const Segment * const SG56657[] = {SGI253423, SGI253424, NULL};
const Segment * const SG56658[] = {SGI253425, SGI253426, NULL};
const Segment * const SG56659[] = {SGI253427, SGI253428, NULL};
const Segment * const SG56660[] = {SGI253429, SGI253430, NULL};
const Segment * const SG56661[] = {SGI253431, SGI253432, NULL};
const Segment * const SG56662[] = {SGI253433, SGI253434, NULL};
const Segment * const SG56663[] = {SGI253435, SGI253436, NULL};
const Segment * const SG56664[] = {SGI253437, SGI253438, NULL};
const Segment * const SG56665[] = {SGI253439, SGI253440, NULL};
const Segment * const SG56666[] = {SGI253441, SGI253442, NULL};
const Segment * const SG56667[] = {SGI253443, SGI253444, NULL};
const Segment * const SG56668[] = {SGI253445, SGI253446, NULL};
const Segment * const SG56669[] = {SGI253447, SGI253448, NULL};
const Segment * const SG56670[] = {SGI253449, SGI253450, NULL};
const Segment * const SG56671[] = {SGI253451, SGI253452, NULL};
const Segment * const SG56672[] = {SGI253453, SGI253454, NULL};
const Segment * const SG56673[] = {SGI253455, SGI253456, NULL};
const Segment * const SG56674[] = {SGI253457, SGI253457, NULL};
const Segment * const SG56675[] = {SGI253458, SGI253459, NULL};
const Segment * const SG56676[] = {SGI253460, SGI253461, NULL};
const Segment * const SG56677[] = {SGI253462, SGI253463, NULL};
const Segment * const SG56678[] = {SGI253464, SGI253465, NULL};
const Segment * const SG56679[] = {SGI253466, SGI253467, NULL};
const Segment * const SG56680[] = {SGI253468, SGI253469, SGI253470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253471, SGI0, SGI0, SGI0, SGI253472, SGI0, SGI253473, SGI253474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56681[] = {SGI253475, SGI253476, SGI253477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253478, SGI0, SGI0, SGI0, SGI253479, SGI0, SGI253480, SGI253481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56682[] = {SGI253482, SGI253483, SGI253484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253485, SGI0, SGI0, SGI0, SGI253486, SGI0, SGI253487, SGI253488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56683[] = {SGI253489, SGI0, SGI253490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253491, SGI253492, SGI253492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56684[] = {SGI253493, SGI253494, SGI253495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253496, SGI0, SGI0, SGI0, SGI253497, SGI0, SGI253498, SGI253499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56685[] = {SGI253500, SGI253501, SGI253502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253503, SGI0, SGI0, SGI0, SGI253504, SGI0, SGI0, SGI253505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56686[] = {SGI253506, SGI0, SGI253507, SGI0, SGI253508, SGI253509, SGI253510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56687[] = {SGI253511, SGI0, SGI253512, SGI0, SGI253513, SGI253514, SGI253515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56688[] = {SGI253516, SGI0, SGI253517, SGI0, SGI253518, SGI253519, SGI253520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56689[] = {SGI253521, SGI0, SGI253522, SGI0, SGI253523, SGI253524, SGI253525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56690[] = {SGI253526, SGI0, SGI253527, SGI0, SGI253528, SGI253529, SGI253530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56691[] = {SGI253531, SGI0, SGI253532, SGI0, SGI253533, SGI253534, SGI253535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56692[] = {SGI253536, SGI0, SGI253537, SGI0, SGI253538, SGI253539, SGI253540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56693[] = {SGI253541, SGI253542, SGI253543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253544, SGI0, SGI0, SGI0, SGI253545, SGI253545, SGI253546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56694[] = {SGI253547, SGI0, SGI253548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253549, SGI253550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253551, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56695[] = {SGI253552, SGI253553, SGI253554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253555, SGI0, SGI0, SGI0, SGI253556, SGI253556, SGI253557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56696[] = {SGI253558, SGI0, SGI253559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253560, SGI253561, SGI253561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56697[] = {SGI253562, SGI0, SGI253563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56698[] = {SGI253564, SGI253565, SGI253566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253567, SGI0, SGI0, SGI0, SGI253568, SGI253568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56699[] = {SGI253569, SGI0, SGI253570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253571, SGI253572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56700[] = {SGI253573, SGI253574, SGI253575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253576, SGI0, SGI0, SGI0, SGI253577, SGI0, SGI253578, SGI253579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56701[] = {SGI253580, SGI0, SGI253581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253582, SGI253583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56702[] = {SGI253584, SGI0, SGI253585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253586, SGI253587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56703[] = {SGI253588, SGI0, SGI253589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253590, SGI253591, SGI253591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56704[] = {SGI253592, SGI0, SGI253593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253594, SGI253595, SGI253595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253596, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56705[] = {SGI253597, SGI0, SGI253598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253599, SGI253600, SGI253600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253601, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56706[] = {SGI253602, SGI253603, SGI253604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253605, SGI0, SGI0, SGI0, SGI253606, SGI253606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56707[] = {SGI253607, SGI0, SGI253608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253609, SGI253610, SGI253610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253611, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56708[] = {SGI253612, SGI0, SGI253613, SGI0, SGI253614, SGI253615, SGI253616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56709[] = {SGI253617, SGI0, SGI253618, SGI0, SGI253619, SGI253620, SGI253621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56710[] = {SGI253622, SGI0, SGI253623, SGI0, SGI253624, SGI253625, SGI253626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56711[] = {SGI253627, SGI0, SGI253628, SGI0, SGI253629, SGI253630, SGI253631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56712[] = {SGI253632, SGI0, SGI253633, SGI0, SGI253634, SGI253635, SGI253636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56713[] = {SGI253637, SGI253638, SGI253639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253640, SGI0, SGI0, SGI0, SGI253641, SGI0, SGI0, SGI253642, SGI0, SGI0, SGI253643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56714[] = {SGI253644, SGI253645, SGI253646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253647, SGI0, SGI0, SGI0, SGI253648, SGI0, SGI253649, SGI253650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56715[] = {SGI253651, SGI0, SGI253652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253653, SGI253654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56716[] = {SGI253655, SGI253656, SGI253657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253658, SGI0, SGI0, SGI0, SGI253659, SGI253659, SGI253660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56717[] = {SGI253661, SGI0, SGI253662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253663, SGI253664, SGI253664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253665, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56718[] = {SGI253666, SGI253667, SGI253668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253669, SGI0, SGI0, SGI0, SGI253670, SGI0, SGI0, SGI253671, SGI253672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56719[] = {SGI253673, SGI253674, SGI253675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253676, SGI0, SGI0, SGI0, SGI253677, SGI0, SGI0, SGI253678, SGI253679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56720[] = {SGI253680, SGI0, SGI253681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253682, SGI253683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253684, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56721[] = {SGI253685, SGI253686, SGI253687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253688, SGI0, SGI0, SGI0, SGI253689, SGI0, SGI0, SGI253690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56722[] = {SGI253691, SGI0, SGI253692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253693, SGI253694, SGI253694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56723[] = {SGI253695, SGI0, SGI253696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253697, SGI253698, SGI253698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56724[] = {SGI253699, SGI253700, SGI253701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253702, SGI0, SGI0, SGI0, SGI253703, SGI0, SGI0, SGI253704, SGI0, SGI0, SGI0, SGI253705, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56725[] = {SGI253706, SGI0, SGI253707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253708, SGI253709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253710, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56726[] = {SGI253711, SGI253712, SGI253713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253714, SGI0, SGI0, SGI0, SGI253715, SGI0, SGI0, SGI253716, SGI253717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56727[] = {SGI253718, SGI0, SGI253719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253720, SGI253721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253722, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56728[] = {SGI253723, SGI0, SGI253724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253725, SGI0, SGI0, SGI0, SGI253726, SGI253726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253727, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56729[] = {SGI253728, SGI0, SGI253729, SGI0, SGI253730, SGI253731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56730[] = {SGI253734, SGI0, SGI253736, SGI0, SGI253737, SGI253738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56731[] = {SGI253732, SGI253733, SGI253735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253739, SGI0, SGI0, SGI0, SGI253740, SGI0, SGI253741, SGI253742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56732[] = {SGI253743, SGI0, SGI253744, SGI0, SGI253745, SGI253746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56733[] = {SGI253747, SGI0, SGI253748, SGI0, SGI253749, SGI253750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56734[] = {SGI253751, SGI0, SGI253752, SGI0, SGI253753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56735[] = {SGI253754, SGI0, SGI253755, SGI0, SGI253756, SGI253757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56736[] = {SGI253758, SGI0, SGI253759, SGI0, SGI253760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56737[] = {SGI253761, SGI0, SGI253762, SGI0, SGI253763, SGI253764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56738[] = {SGI253765, SGI0, SGI253766, SGI0, SGI253767, SGI253768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56739[] = {SGI253769, SGI0, SGI253770, SGI0, SGI253771, SGI253772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56740[] = {SGI253773, SGI0, SGI253774, SGI0, SGI253775, SGI253776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56741[] = {SGI253777, SGI253778, SGI253779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253781, SGI0, SGI0, SGI0, SGI253783, SGI253783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56742[] = {SGI253780, SGI0, SGI253782, SGI0, SGI253784, SGI253785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56743[] = {SGI253787, SGI0, SGI253789, SGI0, SGI253791, SGI253792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56744[] = {SGI253786, SGI253788, SGI253790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253793, SGI0, SGI0, SGI0, SGI253794, SGI253794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56745[] = {SGI253795, SGI0, SGI253796, SGI0, SGI253797, SGI253798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56746[] = {SGI253799, SGI0, SGI253800, SGI0, SGI253801, SGI253802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56747[] = {SGI253803, SGI253804, SGI253805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253806, SGI0, SGI0, SGI0, SGI253808, SGI0, SGI0, SGI253810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56748[] = {SGI253807, SGI0, SGI253809, SGI0, SGI253811, SGI253812, SGI253813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56749[] = {SGI253814, SGI0, SGI253815, SGI0, SGI253816, SGI253817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56750[] = {SGI253818, SGI0, SGI253819, SGI0, SGI253820, SGI253821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56751[] = {SGI253822, SGI0, SGI253823, SGI0, SGI253824, SGI253825, SGI253826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56752[] = {SGI253827, SGI0, SGI253828, SGI0, SGI0, SGI253829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56753[] = {SGI253830, SGI0, SGI253831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56754[] = {SGI253832, SGI0, SGI253833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56755[] = {SGI253834, SGI0, SGI253835, SGI0, SGI0, SGI253836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56756[] = {SGI253837, SGI0, SGI253838, SGI0, SGI253839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56757[] = {SGI253840, SGI0, SGI253841, SGI0, SGI253842, SGI253843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56758[] = {SGI253844, SGI0, SGI253845, SGI0, SGI253846, SGI253847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56759[] = {SGI253848, SGI0, SGI253849, SGI0, SGI0, SGI253850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56760[] = {SGI253855, SGI0, SGI253857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56761[] = {SGI253851, SGI253852, SGI253853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253854, SGI0, SGI0, SGI0, SGI253856, SGI0, SGI0, SGI253858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56762[] = {SGI253859, SGI0, SGI253860, SGI0, SGI253861, SGI253862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56763[] = {SGI253863, SGI0, SGI253864, SGI0, SGI253865, SGI253866, SGI253867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56764[] = {SGI253868, SGI0, SGI253869, SGI0, SGI253870, SGI253871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56765[] = {SGI253872, SGI0, SGI253873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253874, SGI253875, SGI253875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56766[] = {SGI253876, SGI0, SGI253877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253878, SGI253879, SGI253879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56767[] = {SGI253880, SGI0, SGI253881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253882, SGI253883, SGI253883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56768[] = {SGI253884, SGI253885, SGI253886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253887, SGI0, SGI0, SGI0, SGI253888, SGI253888, SGI0, SGI0, SGI0, SGI0, SGI253889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56769[] = {SGI253890, SGI0, SGI253891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253892, SGI253893, SGI253893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253894, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56770[] = {SGI253895, SGI253896, SGI253897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253898, SGI0, SGI0, SGI0, SGI253899, SGI253899, SGI0, SGI0, SGI253900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56771[] = {SGI253901, SGI0, SGI253902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253903, SGI253904, SGI253904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56772[] = {SGI253905, SGI253906, SGI253907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253908, SGI0, SGI0, SGI0, SGI253909, SGI253909, SGI253910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56773[] = {SGI253911, SGI0, SGI253912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI253913, SGI253914, SGI253914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56774[] = {SGI253915, SGI253916, SGI253917, SGI0, SGI253918, SGI253919, SGI253920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56775[] = {SGI253921, SGI253922, SGI253923, SGI0, SGI253924, SGI253925, SGI253926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56776[] = {SGI253927, SGI253928, SGI253929, SGI0, SGI253930, SGI253931, SGI253932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56777[] = {SGI253933, SGI253934, SGI253935, SGI0, SGI253936, SGI253937, SGI253938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56778[] = {SGI253939, SGI253940, SGI253941, SGI0, SGI253942, SGI253943, SGI253944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56779[] = {SGI253945, SGI253946, SGI253947, SGI0, SGI253948, SGI253949, SGI253950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56780[] = {SGI253951, SGI253952, SGI253953, SGI0, SGI253954, SGI253955, SGI253956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56781[] = {SGI253957, SGI253958, SGI253959, SGI0, SGI253960, SGI253961, SGI253962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56782[] = {SGI253963, SGI253964, SGI253965, SGI0, SGI253966, SGI253967, SGI253968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56783[] = {SGI253969, SGI253970, SGI253971, SGI0, SGI253972, SGI253973, SGI253974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56784[] = {SGI253975, SGI253976, SGI253977, SGI0, SGI253978, SGI253979, SGI253980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56785[] = {SGI253981, SGI253982, SGI253983, SGI0, SGI253984, SGI253985, SGI253986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56786[] = {SGI253987, SGI253988, SGI253989, SGI0, SGI253990, SGI253991, SGI253992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56787[] = {SGI253993, SGI253994, SGI253995, SGI0, SGI253996, SGI253997, SGI253998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56788[] = {SGI253999, SGI254000, SGI254001, SGI0, SGI254002, SGI254003, SGI254004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56789[] = {SGI254005, SGI254006, SGI254007, SGI0, SGI254008, SGI254009, SGI254010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56790[] = {SGI254011, SGI254012, SGI254013, SGI0, SGI254014, SGI254015, SGI254016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56791[] = {SGI254017, SGI254018, SGI254019, SGI0, SGI254020, SGI254021, SGI254022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56792[] = {SGI254023, SGI0, SGI254024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI254027, SGI254029, SGI254029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56793[] = {SGI254025, SGI254026, SGI254028, SGI0, SGI254030, SGI254031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56794[] = {SGI254033, SGI254034, SGI254036, SGI0, SGI254037, SGI254038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56795[] = {SGI254032, SGI0, SGI254035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI254039, SGI254040, SGI254040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56796[] = {SGI254041, SGI254042, SGI254043, SGI0, SGI254044, SGI254045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56797[] = {SGI254046, SGI254047, SGI254048, SGI0, SGI254049, SGI254050, SGI254051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56798[] = {SGI254052, SGI254053, SGI254054, SGI0, SGI254055, SGI254056, SGI254057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56799[] = {SGI254058, SGI254059, SGI254060, SGI0, SGI254061, SGI254062, SGI254063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56800[] = {SGI254064, SGI254065, SGI254066, SGI0, SGI254067, SGI254068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56801[] = {SGI254069, SGI254070, SGI254071, SGI0, SGI254072, SGI254073, SGI254074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56802[] = {SGI254076, SGI254077, SGI254079, SGI0, SGI254080, SGI254081, SGI254082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56803[] = {SGI254075, SGI0, SGI254078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI254083, SGI254084, SGI254084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI254085, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56804[] = {SGI254086, SGI254087, SGI254088, SGI0, SGI254089, SGI254090, SGI254091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56805[] = {SGI254092, SGI254093, SGI254094, SGI0, SGI254095, SGI254096, SGI254097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56806[] = {SGI254098, SGI254099, SGI0, SGI0, SGI254100, SGI254101, SGI254102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56807[] = {SGI254103, SGI254104, SGI254105, SGI0, SGI254106, SGI254107, SGI254108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56808[] = {SGI254109, SGI254110, SGI254111, SGI0, SGI254112, SGI254113, SGI254114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56809[] = {SGI254115, SGI254116, SGI254117, SGI0, SGI254118, SGI254119, SGI254120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56810[] = {SGI254121, SGI0, SGI254122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI254123, SGI254124, SGI254124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI254125, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56811[] = {SGI254126, SGI254127, SGI254128, SGI0, SGI254129, SGI254130, SGI254131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56812[] = {SGI254132, SGI254133, SGI254134, SGI0, SGI254135, SGI254136, SGI254137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56813[] = {SGI254138, SGI254139, SGI254140, SGI0, SGI254141, SGI254142, SGI254143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56814[] = {SGI254144, SGI254145, SGI254146, SGI0, SGI254147, SGI254148, SGI254149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56815[] = {SGI254150, SGI254151, SGI254152, SGI0, SGI254153, SGI254154, SGI254155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56816[] = {SGI254156, SGI254157, SGI254158, SGI0, SGI254159, SGI254160, SGI254161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56817[] = {SGI254162, SGI254163, SGI254164, SGI0, SGI254165, SGI254166, SGI254167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56818[] = {SGI254168, SGI254169, SGI254170, SGI0, SGI254171, SGI254172, SGI254173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56819[] = {SGI254174, SGI254175, SGI254176, SGI0, SGI254177, SGI254178, SGI254179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56820[] = {SGI254180, SGI254181, SGI254182, SGI0, SGI254183, SGI254184, SGI254185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56821[] = {SGI254186, SGI254187, SGI254188, SGI0, SGI254189, SGI254190, SGI254191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56822[] = {SGI254192, SGI254193, SGI254194, SGI0, SGI254195, SGI254196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56823[] = {SGI254197, SGI254198, SGI254199, SGI0, SGI254200, SGI254201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56824[] = {SGI254202, SGI254203, SGI254204, SGI0, SGI254205, SGI254206, SGI254207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56825[] = {SGI254208, SGI254209, SGI254210, SGI0, SGI254211, SGI254212, SGI254213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56826[] = {SGI254214, SGI254215, SGI254216, SGI0, SGI254217, SGI254218, SGI254219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56827[] = {SGI254220, SGI254221, SGI254222, SGI0, SGI254223, SGI254224, SGI254225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56828[] = {SGI254226, SGI254227, SGI254228, SGI0, SGI254229, SGI254230, SGI254231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56829[] = {SGI254232, SGI254233, SGI254234, SGI0, SGI254235, SGI254236, SGI254237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56830[] = {SGI254238, SGI254239, SGI254240, SGI0, SGI254241, SGI254242, SGI254243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56831[] = {SGI254244, SGI254245, SGI254246, SGI0, SGI254247, SGI254248, SGI254249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56832[] = {SGI254250, SGI254251, SGI254252, SGI0, SGI254253, SGI254254, SGI254255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56833[] = {SGI254256, SGI254257, SGI254258, SGI0, SGI254259, SGI254260, SGI254261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56834[] = {SGI254262, SGI254263, SGI254264, SGI0, SGI254265, SGI254266, SGI254267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56835[] = {SGI254268, SGI254269, SGI254270, SGI0, SGI254271, SGI254272, SGI254273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56836[] = {SGI254274, SGI254275, SGI254276, SGI0, SGI254277, SGI254278, SGI254279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56837[] = {SGI254280, SGI254281, SGI254282, SGI0, SGI254283, SGI254284, SGI254285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56838[] = {SGI254286, SGI254287, SGI254288, SGI0, SGI254289, SGI254290, SGI254291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56839[] = {SGI254292, SGI254293, SGI254294, SGI0, SGI254295, SGI254296, SGI254295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56840[] = {SGI254297, SGI254298, SGI254299, SGI0, SGI254300, SGI254301, SGI254300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56841[] = {SGI254302, SGI254303, SGI254304, SGI0, SGI254305, SGI254306, SGI254305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56842[] = {SGI254307, SGI254308, SGI254309, SGI0, SGI254310, SGI254311, SGI254310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56843[] = {SGI254312, SGI254313, SGI254314, SGI0, SGI254315, SGI254316, SGI254315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56844[] = {SGI254317, SGI254318, SGI254319, SGI0, SGI254320, SGI254321, SGI254320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56845[] = {SGI254322, SGI254323, SGI254324, SGI0, SGI254325, SGI254326, SGI254327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56846[] = {SGI254328, SGI254329, SGI254330, SGI0, SGI254331, SGI254332, SGI254331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56847[] = {SGI254333, SGI254334, SGI254335, SGI0, SGI254336, SGI254337, SGI254338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56848[] = {SGI254339, SGI254340, SGI254341, SGI0, SGI254342, SGI254343, SGI254344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56849[] = {SGI254345, SGI254346, SGI254347, SGI0, SGI254348, SGI254349, SGI254348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56850[] = {SGI254350, SGI254351, SGI254352, SGI0, SGI254353, SGI254354, SGI254355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56851[] = {SGI254356, SGI254357, SGI254358, SGI0, SGI254359, SGI254360, SGI254361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56852[] = {SGI254362, SGI254363, SGI254364, SGI0, SGI254365, SGI254366, SGI254365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56853[] = {SGI254367, SGI254368, SGI254369, SGI0, SGI254370, SGI254371, SGI254372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56854[] = {SGI254373, SGI254374, SGI254375, SGI0, SGI254376, SGI254377, SGI254378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56855[] = {SGI254379, SGI254380, SGI254381, SGI0, SGI254382, SGI254383, SGI254382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56856[] = {SGI254384, SGI254385, SGI254386, SGI0, SGI254387, SGI254388, SGI254387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56857[] = {SGI254389, SGI254390, SGI254391, SGI0, SGI254392, SGI254393, SGI254392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56858[] = {SGI254394, SGI254395, SGI254396, SGI0, SGI254397, SGI254398, SGI254399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56859[] = {SGI254400, SGI254401, SGI254402, SGI0, SGI254403, SGI254404, SGI254405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56860[] = {SGI254406, SGI254407, SGI254408, SGI0, SGI254409, SGI254410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56861[] = {SGI254411, SGI254412, SGI254413, SGI0, SGI254414, SGI254415, SGI254416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56862[] = {SGI254417, SGI254418, SGI254419, SGI0, SGI254420, SGI254421, SGI254422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56863[] = {SGI254423, SGI254424, SGI254425, SGI0, SGI254426, SGI254427, SGI254428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56864[] = {SGI254429, SGI254430, SGI254431, SGI0, SGI254432, SGI254433, SGI254434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56865[] = {SGI254435, SGI254436, SGI254437, SGI0, SGI254438, SGI254439, SGI254440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56866[] = {SGI254441, SGI254442, SGI254443, SGI0, SGI254444, SGI254445, SGI254446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56867[] = {SGI254447, SGI254448, SGI254449, SGI0, SGI254450, SGI254451, SGI254452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56868[] = {SGI254453, SGI254454, SGI254455, SGI0, SGI254456, SGI254457, SGI254458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56869[] = {SGI254459, SGI254460, SGI254461, SGI0, SGI254462, SGI254463, SGI254464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56870[] = {SGI254465, SGI254466, SGI254467, SGI0, SGI254468, SGI254469, SGI254470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56871[] = {SGI254471, SGI254472, SGI254473, SGI0, SGI254474, SGI254475, SGI254476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56872[] = {SGI254477, SGI254478, SGI254479, SGI0, SGI254480, SGI254481, SGI254482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56873[] = {SGI254483, SGI254484, SGI254485, SGI0, SGI254486, SGI254487, SGI254488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56874[] = {SGI254489, SGI254490, SGI254491, SGI0, SGI254492, SGI254493, SGI254494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56875[] = {SGI254495, SGI254496, SGI254497, SGI0, SGI254498, SGI254499, SGI254500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56876[] = {SGI254501, SGI254502, SGI254503, SGI0, SGI254504, SGI254505, SGI254506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56877[] = {SGI254507, SGI254508, SGI254509, SGI0, SGI254510, SGI254511, SGI254512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56878[] = {SGI254513, SGI254514, SGI254515, SGI0, SGI254516, SGI254517, SGI254518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56879[] = {SGI254519, SGI254520, SGI254521, SGI0, SGI254522, SGI254523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56880[] = {SGI254524, SGI254525, SGI254526, SGI0, SGI254527, SGI254528, SGI254529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56881[] = {SGI254530, SGI254531, SGI254532, SGI0, SGI254533, SGI254534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56882[] = {SGI254535, SGI254536, SGI254537, SGI0, SGI254538, SGI254539, SGI254540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56883[] = {SGI254541, SGI254542, SGI254543, SGI0, SGI254544, SGI0, SGI254545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56884[] = {SGI254546, SGI254547, SGI254548, SGI0, SGI254549, SGI254550, SGI254551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56885[] = {SGI254552, SGI254553, SGI254554, SGI0, SGI254555, SGI254556, SGI254557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56886[] = {SGI254558, SGI254559, SGI254560, SGI0, SGI254561, SGI254562, SGI254563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56887[] = {SGI254564, SGI254565, SGI254566, SGI0, SGI254567, SGI254568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56888[] = {SGI254569, SGI254570, SGI254571, SGI0, SGI254572, SGI254573, SGI254574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56889[] = {SGI254575, SGI254576, SGI254577, SGI0, SGI254578, SGI254579, SGI254580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56890[] = {SGI254581, SGI254582, SGI254583, SGI0, SGI254584, SGI254585, SGI254586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56891[] = {SGI254587, SGI254588, SGI254589, SGI0, SGI254590, SGI254591, SGI254592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56892[] = {SGI254593, SGI254594, SGI254595, SGI0, SGI254596, SGI254597, SGI254598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56893[] = {SGI254599, SGI254600, SGI254601, SGI0, SGI254602, SGI254603, SGI254604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56894[] = {SGI254605, SGI254606, SGI254607, SGI0, SGI254608, SGI254609, SGI254610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56895[] = {SGI254611, SGI254612, SGI254613, SGI0, SGI254614, SGI254615, SGI254616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56896[] = {SGI254617, SGI254618, SGI254619, SGI0, SGI254620, SGI254621, SGI254622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56897[] = {SGI254623, SGI254624, SGI254625, SGI0, SGI254626, SGI254627, SGI254628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56898[] = {SGI254629, SGI254630, SGI254631, SGI0, SGI254632, SGI254633, SGI254634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56899[] = {SGI254635, SGI254636, SGI254637, SGI0, SGI254638, SGI254639, SGI254640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56900[] = {SGI254641, SGI254642, SGI254643, SGI0, SGI254644, SGI254645, SGI254646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56901[] = {SGI254647, SGI254648, SGI254649, SGI0, SGI254650, SGI254651, SGI254652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56902[] = {SGI254653, SGI254654, SGI254655, SGI0, SGI254656, SGI254657, SGI254658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56903[] = {SGI254659, SGI254660, SGI254661, SGI0, SGI254662, SGI254663, SGI254664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56904[] = {SGI254665, SGI254666, SGI254667, SGI0, SGI254668, SGI254669, SGI254670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56905[] = {SGI254671, SGI254672, SGI254673, SGI0, SGI254674, SGI254675, SGI254676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56906[] = {SGI254677, SGI0, SGI254678, SGI0, SGI254679, SGI254680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56907[] = {SGI254681, SGI0, SGI254682, SGI0, SGI254683, SGI254684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56908[] = {SGI254685, SGI0, SGI254686, SGI0, SGI0, SGI254687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56909[] = {SGI254688, SGI0, SGI254689, SGI0, SGI254690, SGI254691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56910[] = {SGI254692, SGI0, SGI254693, SGI0, SGI254694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56911[] = {SGI254695, SGI0, SGI254696, SGI0, SGI254697, SGI254698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56912[] = {SGI254699, SGI0, SGI254700, SGI0, SGI254701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56913[] = {SGI254702, SGI0, SGI254703, SGI0, SGI254704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56914[] = {SGI254705, SGI0, SGI254706, SGI0, SGI254707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56915[] = {SGI254708, SGI0, SGI254709, SGI0, SGI254710, SGI254711, SGI254710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56916[] = {SGI254712, SGI0, SGI254713, SGI0, SGI254714, SGI254715, SGI254714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56917[] = {SGI254716, SGI0, SGI254717, SGI0, SGI254718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56918[] = {SGI254719, SGI0, SGI254720, SGI0, SGI254721, SGI254722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56919[] = {SGI254723, SGI0, SGI254724, SGI0, SGI254725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56920[] = {SGI254726, SGI0, SGI254727, SGI0, SGI254728, SGI254729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56921[] = {SGI254730, SGI0, SGI254731, SGI0, SGI254732, SGI254733, SGI254732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56922[] = {SGI254734, SGI0, SGI254735, SGI0, SGI254736, SGI254737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56923[] = {SGI254738, SGI0, SGI254739, SGI0, SGI254740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56924[] = {SGI254741, SGI0, SGI254742, SGI0, SGI254743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56925[] = {SGI254744, SGI0, SGI254745, SGI0, SGI254746, SGI254747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56926[] = {SGI254748, SGI0, SGI254749, SGI0, SGI254750, SGI254751, SGI254750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56927[] = {SGI254752, SGI0, SGI254753, SGI0, SGI254754, SGI254755, SGI254754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56928[] = {SGI254756, SGI0, SGI254757, SGI0, SGI254758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56929[] = {SGI254759, SGI0, SGI254760, SGI0, SGI254761, SGI254762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56930[] = {SGI254763, SGI0, SGI254764, SGI0, SGI254765, SGI254766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56931[] = {SGI254767, SGI0, SGI254768, SGI0, SGI254769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56932[] = {SGI254770, SGI0, SGI254771, SGI0, SGI254772, SGI254773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56933[] = {SGI254774, SGI0, SGI254775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI254776, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56934[] = {SGI254777, SGI0, SGI254778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56935[] = {SGI254779, SGI0, SGI254780, SGI0, SGI254781, SGI254782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56936[] = {SGI254783, SGI0, SGI254784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56937[] = {SGI254785, SGI0, SGI254786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56938[] = {SGI254787, SGI0, SGI254788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56939[] = {SGI254789, SGI0, SGI254790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56940[] = {SGI254791, SGI0, SGI254792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56941[] = {SGI254793, SGI0, SGI254794, SGI0, SGI254795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56942[] = {SGI254796, SGI0, SGI254797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56943[] = {SGI254798, SGI0, SGI254799, SGI0, SGI254800, SGI254801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56944[] = {SGI254802, SGI0, SGI254803, SGI0, SGI254804, SGI254805, SGI254806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56945[] = {SGI254807, SGI0, SGI254808, SGI0, SGI254809, SGI254810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56946[] = {SGI254811, SGI0, SGI254812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56947[] = {SGI254813, SGI0, SGI0, SGI0, SGI254814, SGI254815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56948[] = {SGI254816, SGI254817, SGI254818, SGI0, SGI254819, SGI254820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56949[] = {SGI254821, SGI0, SGI0, SGI0, SGI254822, SGI254823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56950[] = {SGI254824, SGI0, SGI254825, SGI0, SGI254826, SGI254827, SGI254828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56951[] = {SGI254829, SGI0, SGI254830, SGI0, SGI254831, SGI254832, SGI254833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56952[] = {SGI254834, SGI0, SGI254835, SGI0, SGI254836, SGI254837, SGI254838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56953[] = {SGI254839, SGI0, SGI254840, SGI0, SGI254841, SGI254842, SGI254843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56954[] = {SGI254844, SGI0, SGI254845, SGI0, SGI254846, SGI254847, SGI254848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56955[] = {SGI254849, SGI0, SGI254850, SGI0, SGI254851, SGI254852, SGI254853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56956[] = {SGI254854, SGI0, SGI254855, SGI0, SGI254856, SGI254857, SGI254858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56957[] = {SGI254859, SGI0, SGI254860, SGI0, SGI254861, SGI254862, SGI254863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56958[] = {SGI254864, SGI0, SGI254865, SGI0, SGI254866, SGI254867, SGI254868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56959[] = {SGI254869, SGI0, SGI254870, SGI0, SGI254871, SGI254872, SGI254873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56960[] = {SGI254874, SGI0, SGI254875, SGI0, SGI254876, SGI254877, SGI254878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG56961[] = {SGI254879, SGI254880, NULL};
const Segment * const SG56962[] = {SGI254881, SGI254882, NULL};
const Segment * const SG56963[] = {SGI254883, SGI254884, NULL};
const Segment * const SG56964[] = {SGI254885, SGI254886, NULL};
const Segment * const SG56965[] = {SGI254887, SGI254888, NULL};
const Segment * const SG56966[] = {SGI254889, SGI254889, NULL};
const Segment * const SG56967[] = {SGI254890, SGI254891, NULL};
const Segment * const SG56968[] = {SGI254892, SGI254892, NULL};
const Segment * const SG56969[] = {SGI254893, SGI254894, NULL};
const Segment * const SG56970[] = {SGI254895, SGI254895, NULL};
const Segment * const SG56971[] = {SGI254896, SGI254896, NULL};
const Segment * const SG56972[] = {SGI254897, SGI254898, NULL};
const Segment * const SG56973[] = {SGI254899, SGI254900, NULL};
const Segment * const SG56974[] = {SGI254901, SGI254902, NULL};
const Segment * const SG56975[] = {SGI254903, SGI254904, NULL};
const Segment * const SG56976[] = {SGI254905, SGI254905, NULL};
const Segment * const SG56977[] = {SGI254906, SGI254907, NULL};
const Segment * const SG56978[] = {SGI254908, SGI254909, NULL};
const Segment * const SG56979[] = {SGI254910, SGI254911, NULL};
const Segment * const SG56980[] = {SGI254912, SGI254913, NULL};
const Segment * const SG56981[] = {SGI254914, SGI254915, NULL};
const Segment * const SG56982[] = {SGI254916, SGI254917, NULL};
const Segment * const SG56983[] = {SGI254918, SGI254919, NULL};
const Segment * const SG56984[] = {SGI254920, SGI254921, NULL};
const Segment * const SG56985[] = {SGI254922, SGI254923, NULL};
const Segment * const SG56986[] = {SGI254924, SGI254925, NULL};
const Segment * const SG56987[] = {SGI254926, SGI254927, NULL};
const Segment * const SG56988[] = {SGI254928, SGI254929, NULL};
const Segment * const SG56989[] = {SGI254930, SGI254931, NULL};
const Segment * const SG56990[] = {SGI254932, SGI254933, NULL};
const Segment * const SG56991[] = {SGI254934, SGI254935, NULL};
const Segment * const SG56992[] = {SGI254936, SGI254937, NULL};
const Segment * const SG56993[] = {SGI254938, SGI254939, NULL};
const Segment * const SG56994[] = {SGI254940, SGI254941, NULL};
const Segment * const SG56995[] = {SGI254942, SGI254942, NULL};
const Segment * const SG56996[] = {SGI254943, SGI254944, NULL};
const Segment * const SG56997[] = {SGI254945, SGI254946, NULL};
const Segment * const SG56998[] = {SGI254947, SGI254947, NULL};
const Segment * const SG56999[] = {SGI254948, SGI254949, NULL};
const Segment * const SG57000[] = {SGI254950, SGI254951, NULL};
const Segment * const SG57001[] = {SGI254952, SGI254953, NULL};
const Segment * const SG57002[] = {SGI254954, SGI254955, NULL};
const Segment * const SG57003[] = {SGI254956, SGI254957, NULL};
const Segment * const SG57004[] = {SGI254958, SGI254958, NULL};
const Segment * const SG57005[] = {SGI254959, SGI254960, NULL};
const Segment * const SG57006[] = {SGI254961, SGI254961, NULL};
const Segment * const SG57007[] = {SGI254962, SGI254963, NULL};
const Segment * const SG57008[] = {SGI254964, SGI254965, NULL};
const Segment * const SG57009[] = {SGI254966, SGI254967, NULL};
const Segment * const SG57010[] = {SGI254968, SGI254969, NULL};
const Segment * const SG57011[] = {SGI254970, SGI254971, NULL};
const Segment * const SG57012[] = {SGI254972, SGI254973, NULL};
const Segment * const SG57013[] = {SGI254974, SGI254974, NULL};
const Segment * const SG57014[] = {SGI254975, SGI254976, NULL};
const Segment * const SG57015[] = {SGI254977, SGI254978, NULL};
const Segment * const SG57016[] = {SGI254979, SGI254980, NULL};
const Segment * const SG57017[] = {SGI254981, SGI254982, NULL};
const Segment * const SG57018[] = {SGI254983, SGI254984, NULL};
const Segment * const SG57019[] = {SGI254985, SGI254986, NULL};
const Segment * const SG57020[] = {SGI254987, SGI254988, NULL};
const Segment * const SG57021[] = {SGI254989, SGI254990, NULL};
const Segment * const SG57022[] = {SGI254991, SGI254992, NULL};
const Segment * const SG57023[] = {SGI254993, SGI254994, NULL};
const Segment * const SG57024[] = {SGI254995, SGI254996, NULL};
const Segment * const SG57025[] = {SGI254997, SGI254998, NULL};
const Segment * const SG57026[] = {SGI254999, SGI255000, NULL};
const Segment * const SG57027[] = {SGI255001, SGI255002, NULL};
const Segment * const SG57028[] = {SGI255003, SGI255004, NULL};
const Segment * const SG57029[] = {SGI255005, SGI255006, NULL};
const Segment * const SG57030[] = {SGI255007, SGI255008, NULL};
const Segment * const SG57031[] = {SGI255009, SGI255010, NULL};
const Segment * const SG57032[] = {SGI255011, SGI255012, NULL};
const Segment * const SG57033[] = {SGI255013, SGI255014, NULL};
const Segment * const SG57034[] = {SGI255015, SGI255016, NULL};
const Segment * const SG57035[] = {SGI255017, SGI255017, NULL};
const Segment * const SG57036[] = {SGI255018, SGI255019, NULL};
const Segment * const SG57037[] = {SGI255020, SGI255021, NULL};
const Segment * const SG57038[] = {SGI255022, SGI255023, NULL};
const Segment * const SG57039[] = {SGI255024, SGI255025, NULL};
const Segment * const SG57040[] = {SGI255026, SGI255026, NULL};
const Segment * const SG57041[] = {SGI255027, SGI255028, NULL};
const Segment * const SG57042[] = {SGI255029, SGI255030, NULL};
const Segment * const SG57043[] = {SGI255031, SGI255031, NULL};
const Segment * const SG57044[] = {SGI255032, SGI255033, NULL};
const Segment * const SG57045[] = {SGI255034, SGI255034, NULL};
const Segment * const SG57046[] = {SGI255035, SGI255035, NULL};
const Segment * const SG57047[] = {SGI255036, SGI255036, NULL};
const Segment * const SG57048[] = {SGI255037, SGI255038, NULL};
const Segment * const SG57049[] = {SGI255039, SGI255040, NULL};
const Segment * const SG57050[] = {SGI255041, SGI255042, NULL};
const Segment * const SG57051[] = {SGI255043, SGI255044, NULL};
const Segment * const SG57052[] = {SGI255045, SGI255046, NULL};
const Segment * const SG57053[] = {SGI255047, SGI255048, NULL};
const Segment * const SG57054[] = {SGI255049, SGI255049, NULL};
const Segment * const SG57055[] = {SGI255050, SGI255051, NULL};
const Segment * const SG57056[] = {SGI255052, SGI255053, NULL};
const Segment * const SG57057[] = {SGI255054, SGI255055, NULL};
const Segment * const SG57058[] = {SGI255056, SGI255057, NULL};
const Segment * const SG57059[] = {SGI255058, SGI255058, NULL};
const Segment * const SG57060[] = {SGI255059, SGI0, SGI255060, SGI0, SGI255061, SGI255062, SGI255063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57061[] = {SGI255064, SGI0, SGI255065, SGI0, SGI255066, SGI255067, SGI255068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57062[] = {SGI255069, SGI0, SGI255070, SGI0, SGI255071, SGI255072, SGI255073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57063[] = {SGI255074, SGI0, SGI255075, SGI0, SGI255076, SGI255077, SGI255078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57064[] = {SGI255079, SGI0, SGI255080, SGI0, SGI255081, SGI255082, SGI255083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57065[] = {SGI255084, SGI0, SGI255085, SGI0, SGI255086, SGI255087, SGI255088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57066[] = {SGI255089, SGI0, SGI255090, SGI0, SGI255091, SGI255092, SGI255093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57067[] = {SGI255094, SGI0, SGI255095, SGI0, SGI255096, SGI255097, SGI255098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57068[] = {SGI255099, SGI0, SGI255100, SGI0, SGI255101, SGI255102, SGI255103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57069[] = {SGI255104, SGI0, SGI255105, SGI0, SGI255106, SGI255107, SGI255108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57070[] = {SGI255109, SGI0, SGI255110, SGI0, SGI255111, SGI255112, SGI255113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57071[] = {SGI255114, SGI0, SGI255115, SGI0, SGI255116, SGI255117, SGI255118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57072[] = {SGI255119, SGI0, SGI255120, SGI0, SGI255121, SGI255122, SGI255123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57073[] = {SGI255124, SGI0, SGI255125, SGI0, SGI255126, SGI255127, SGI255128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57074[] = {SGI255129, SGI0, SGI255130, SGI0, SGI255131, SGI255132, SGI255133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57075[] = {SGI255134, SGI0, SGI255135, SGI0, SGI255136, SGI255137, SGI255138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57076[] = {SGI255139, SGI0, SGI255140, SGI0, SGI255141, SGI255142, SGI255143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57077[] = {SGI255144, SGI0, SGI255145, SGI0, SGI255146, SGI255147, SGI255148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57078[] = {SGI255149, SGI0, SGI255150, SGI0, SGI255151, SGI255152, SGI255153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57079[] = {SGI255154, SGI0, SGI255155, SGI0, SGI255156, SGI255157, SGI255158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57080[] = {SGI255159, SGI0, SGI255160, SGI0, SGI255161, SGI255162, SGI255163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57081[] = {SGI255164, SGI0, SGI255165, SGI0, SGI255166, SGI255167, SGI255168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57082[] = {SGI255169, SGI0, SGI255170, SGI0, SGI255171, SGI255172, SGI255173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57083[] = {SGI255174, SGI0, SGI255175, SGI0, SGI255176, SGI255177, SGI255178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57084[] = {SGI255179, SGI255180, SGI255181, SGI0, SGI255182, SGI255183, SGI255184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57085[] = {SGI255185, SGI255186, SGI255187, SGI0, SGI255188, SGI255189, SGI255190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57086[] = {SGI255191, SGI255192, SGI255193, SGI0, SGI255194, SGI255195, SGI255196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57087[] = {SGI255197, SGI255198, SGI255199, SGI0, SGI255200, SGI255201, SGI255202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57088[] = {SGI255203, SGI255204, SGI255205, SGI0, SGI255206, SGI255207, SGI255208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57089[] = {SGI255209, SGI255210, SGI255211, SGI0, SGI255212, SGI255213, SGI255214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57090[] = {SGI255215, SGI255216, SGI255217, SGI0, SGI255218, SGI255219, SGI255220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57091[] = {SGI255221, SGI255222, SGI255223, SGI0, SGI255224, SGI255225, SGI255226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57092[] = {SGI255227, SGI255228, SGI255229, SGI0, SGI255230, SGI255231, SGI255232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57093[] = {SGI255233, SGI255234, SGI255235, SGI0, SGI255236, SGI255237, SGI255238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57094[] = {SGI255239, SGI255240, SGI255241, SGI0, SGI255242, SGI255243, SGI255244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57095[] = {SGI255245, SGI255246, SGI255247, SGI0, SGI255248, SGI255249, SGI255250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57096[] = {SGI255251, SGI255252, SGI255253, SGI0, SGI255254, SGI255255, SGI255256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57097[] = {SGI255257, SGI255258, SGI255259, SGI0, SGI255260, SGI255261, SGI255262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57098[] = {SGI255263, SGI255264, SGI255265, SGI0, SGI255266, SGI255267, SGI255268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57099[] = {SGI255269, SGI255270, SGI255271, SGI0, SGI255272, SGI255273, SGI255274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57100[] = {SGI255275, SGI255276, SGI255277, SGI0, SGI255278, SGI255279, SGI255280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57101[] = {SGI255281, SGI255282, SGI255283, SGI0, SGI255284, SGI255285, SGI255286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57102[] = {SGI255287, SGI255288, SGI255289, SGI0, SGI255290, SGI255291, SGI255292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57103[] = {SGI255293, SGI255294, SGI255295, SGI0, SGI255296, SGI255297, SGI255298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57104[] = {SGI255299, SGI255300, SGI255301, SGI0, SGI255302, SGI255303, SGI255304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57105[] = {SGI255305, SGI255306, SGI255307, SGI0, SGI255308, SGI255309, SGI255310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57106[] = {SGI255311, SGI255312, SGI255313, SGI0, SGI255314, SGI255315, SGI255316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57107[] = {SGI255317, SGI255318, SGI255319, SGI0, SGI255320, SGI255321, SGI255322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57108[] = {SGI255323, SGI255324, SGI255325, SGI0, SGI255326, SGI255327, SGI255328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57109[] = {SGI255329, SGI255330, SGI255331, SGI0, SGI255332, SGI255333, SGI255334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57110[] = {SGI255335, SGI255336, SGI255337, SGI0, SGI255338, SGI255339, SGI255340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57111[] = {SGI255341, SGI255342, SGI255343, SGI0, SGI255344, SGI255345, SGI255346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57112[] = {SGI255347, SGI255348, SGI255349, SGI0, SGI255350, SGI255351, SGI255352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57113[] = {SGI255353, SGI255354, SGI255355, SGI0, SGI255356, SGI255357, SGI255358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57114[] = {SGI255359, SGI255360, SGI255361, SGI0, SGI255362, SGI255363, SGI255364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57115[] = {SGI255365, SGI255366, SGI255367, SGI0, SGI255368, SGI255369, SGI255370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57116[] = {SGI255371, SGI255372, SGI255373, SGI0, SGI255374, SGI255375, SGI255376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57117[] = {SGI255377, SGI255378, SGI0, SGI0, SGI255379, SGI255380, SGI255381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57118[] = {SGI255382, SGI255383, SGI255384, SGI0, SGI255385, SGI255386, SGI255387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57119[] = {SGI255388, SGI255389, SGI255390, SGI0, SGI255391, SGI255392, SGI255393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57120[] = {SGI255394, SGI255395, SGI255396, SGI0, SGI255397, SGI255398, SGI255399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57121[] = {SGI255400, SGI255401, SGI255402, SGI0, SGI255403, SGI255404, SGI255405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57122[] = {SGI255406, SGI255407, SGI255408, SGI0, SGI255409, SGI255410, SGI255411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57123[] = {SGI255412, SGI255413, SGI255414, SGI0, SGI255415, SGI255416, SGI255417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57124[] = {SGI255418, SGI255419, SGI255420, SGI0, SGI255421, SGI255422, SGI255423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57125[] = {SGI255424, SGI255425, SGI255426, SGI0, SGI255427, SGI255428, SGI255429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57126[] = {SGI255430, SGI255431, SGI255432, SGI0, SGI255433, SGI255434, SGI255435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57127[] = {SGI255436, SGI255437, SGI255438, SGI0, SGI255439, SGI255440, SGI255441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57128[] = {SGI255442, SGI255443, SGI255444, SGI0, SGI255445, SGI255446, SGI255447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57129[] = {SGI255448, SGI255449, SGI255450, SGI0, SGI255451, SGI255452, SGI255453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57130[] = {SGI255454, SGI255455, SGI0, SGI0, SGI255456, SGI255457, SGI255458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57131[] = {SGI255459, SGI0, SGI255460, SGI0, SGI255461, SGI255462, SGI255463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57132[] = {SGI255464, SGI0, SGI255465, SGI0, SGI255466, SGI255467, SGI255468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57133[] = {SGI255469, SGI0, SGI255470, SGI0, SGI255471, SGI255472, SGI255473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57134[] = {SGI255474, SGI0, SGI255475, SGI0, SGI255476, SGI255477, SGI255478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57135[] = {SGI255479, SGI0, SGI255480, SGI0, SGI255481, SGI255482, SGI255483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57136[] = {SGI255484, SGI255485, SGI255486, SGI0, SGI255487, SGI255488, SGI255489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57137[] = {SGI255490, SGI255491, SGI255492, SGI0, SGI255493, SGI255494, SGI255495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57138[] = {SGI255496, SGI255497, SGI255498, SGI0, SGI255499, SGI255500, SGI255501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57139[] = {SGI255502, SGI255503, SGI255504, SGI0, SGI255505, SGI255506, SGI255507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57140[] = {SGI255508, SGI255509, SGI255510, SGI0, SGI255511, SGI255512, SGI255513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57141[] = {SGI255514, SGI255515, SGI255516, SGI0, SGI255517, SGI255518, SGI255519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57142[] = {SGI255520, SGI255521, SGI255522, SGI0, SGI255523, SGI255524, SGI255525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57143[] = {SGI255526, SGI255527, SGI255528, SGI0, SGI255529, SGI255530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57144[] = {SGI255531, SGI255532, SGI255533, SGI0, SGI255534, SGI255535, SGI255536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57145[] = {SGI255537, SGI255538, SGI255539, SGI0, SGI255540, SGI255541, SGI255542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57146[] = {SGI255543, SGI255544, SGI255545, SGI0, SGI255546, SGI255547, SGI255548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57147[] = {SGI255549, SGI255550, SGI255551, SGI0, SGI255552, SGI255553, SGI255554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57148[] = {SGI255555, SGI255556, SGI255557, SGI0, SGI255558, SGI255559, SGI255560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57149[] = {SGI255561, SGI255562, SGI255563, SGI0, SGI255564, SGI255565, SGI255566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57150[] = {SGI255567, SGI255568, SGI255569, SGI0, SGI255570, SGI255571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57151[] = {SGI255572, SGI255573, SGI255574, SGI0, SGI255575, SGI255576, SGI255577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57152[] = {SGI255578, SGI255579, SGI255580, SGI0, SGI255581, SGI255582, SGI255583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57153[] = {SGI255584, SGI255585, SGI255586, SGI0, SGI255587, SGI255588, SGI255589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57154[] = {SGI255590, SGI255591, SGI255592, SGI0, SGI255593, SGI255594, SGI255595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57155[] = {SGI255596, SGI255597, SGI255598, SGI0, SGI255599, SGI255600, SGI255601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57156[] = {SGI255602, SGI255603, SGI255604, SGI0, SGI255605, SGI255606, SGI255607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57157[] = {SGI255608, SGI0, SGI255609, SGI0, SGI255610, SGI255611, SGI255612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57158[] = {SGI255613, SGI0, SGI255614, SGI0, SGI255615, SGI255616, SGI255617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57159[] = {SGI255618, SGI0, SGI255619, SGI0, SGI255620, SGI255621, SGI255622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57160[] = {SGI255623, SGI0, SGI255624, SGI0, SGI255625, SGI255626, SGI255627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57161[] = {SGI255628, SGI0, SGI255629, SGI0, SGI255630, SGI255631, SGI255632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57162[] = {SGI255633, SGI0, SGI255634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57163[] = {SGI255635, SGI0, SGI255636, SGI0, SGI255637, SGI255638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57164[] = {SGI255639, SGI0, SGI255640, SGI0, SGI255641, SGI255642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57165[] = {SGI255643, SGI0, SGI255644, SGI0, SGI255645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57166[] = {SGI255646, SGI0, SGI255647, SGI0, SGI255648, SGI255649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57167[] = {SGI255650, SGI0, SGI255651, SGI0, SGI255652, SGI255653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57168[] = {SGI255654, SGI0, SGI255655, SGI0, SGI0, SGI255656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57169[] = {SGI255657, SGI0, SGI255658, SGI0, SGI255659, SGI255660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57170[] = {SGI255661, SGI0, SGI255662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57171[] = {SGI255663, SGI0, SGI255664, SGI0, SGI255665, SGI255666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57172[] = {SGI255667, SGI0, SGI255668, SGI0, SGI255669, SGI255670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57173[] = {SGI255671, SGI0, SGI255672, SGI0, SGI255673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57174[] = {SGI255674, SGI0, SGI255675, SGI0, SGI255676, SGI255677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57175[] = {SGI255678, SGI0, SGI255679, SGI0, SGI255680, SGI255681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57176[] = {SGI255682, SGI0, SGI0, SGI0, SGI255683, SGI255684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57177[] = {SGI255685, SGI0, SGI255686, SGI0, SGI255687, SGI255688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57178[] = {SGI255689, SGI0, SGI255690, SGI0, SGI255691, SGI255692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57179[] = {SGI255693, SGI255694, SGI255695, SGI0, SGI255696, SGI255697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57180[] = {SGI255698, SGI0, SGI255699, SGI0, SGI0, SGI255700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57181[] = {SGI255701, SGI255702, NULL};
const Segment * const SG57182[] = {SGI255703, SGI0, SGI255704, SGI0, SGI255705, SGI255706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57183[] = {SGI255707, SGI0, SGI255708, SGI0, SGI255709, SGI255710, SGI255711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57184[] = {SGI255712, SGI255713, SGI255714, SGI0, SGI255715, SGI255716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57185[] = {SGI255717, SGI255718, SGI255719, SGI0, SGI255720, SGI255721, SGI255722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57186[] = {SGI255723, SGI0, SGI255724, SGI0, SGI255725, SGI255726, SGI255727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57187[] = {SGI255728, SGI255729, SGI255730, SGI0, SGI255731, SGI255732, SGI255733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57188[] = {SGI255734, SGI255735, SGI255736, SGI0, SGI255737, SGI255738, SGI255739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57189[] = {SGI255740, SGI255741, SGI255742, SGI0, SGI255743, SGI255744, SGI255745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57190[] = {SGI255746, SGI255747, SGI255748, SGI0, SGI255749, SGI255750, SGI255751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57191[] = {SGI255752, SGI255753, SGI255754, SGI0, SGI255755, SGI255756, SGI255757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57192[] = {SGI255758, SGI255759, SGI255760, SGI0, SGI255761, SGI255762, SGI255763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57193[] = {SGI255764, SGI255765, SGI255766, SGI0, SGI255767, SGI255768, SGI255769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57194[] = {SGI255770, SGI255771, SGI255772, SGI0, SGI255773, SGI255774, SGI255775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57195[] = {SGI255776, SGI0, SGI255777, SGI0, SGI255778, SGI255779, SGI255780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57196[] = {SGI255781, SGI255782, SGI255783, SGI0, SGI255784, SGI255785, SGI255786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57197[] = {SGI255787, SGI255788, SGI255789, SGI0, SGI255790, SGI255791, SGI255792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57198[] = {SGI255793, SGI255794, NULL};
const Segment * const SG57199[] = {SGI255795, SGI255796, NULL};
const Segment * const SG57200[] = {SGI255797, SGI255798, NULL};
const Segment * const SG57201[] = {SGI255799, SGI255800, NULL};
const Segment * const SG57202[] = {SGI255801, SGI255802, NULL};
const Segment * const SG57203[] = {SGI255803, SGI255804, NULL};
const Segment * const SG57204[] = {SGI255805, SGI255806, NULL};
const Segment * const SG57205[] = {SGI255807, SGI255807, NULL};
const Segment * const SG57206[] = {SGI255808, SGI255809, NULL};
const Segment * const SG57207[] = {SGI255810, SGI255811, NULL};
const Segment * const SG57208[] = {SGI255812, SGI255813, NULL};
const Segment * const SG57209[] = {SGI255814, SGI255815, NULL};
const Segment * const SG57210[] = {SGI255816, SGI255817, NULL};
const Segment * const SG57211[] = {SGI255818, SGI255819, NULL};
const Segment * const SG57212[] = {SGI255820, SGI255821, NULL};
const Segment * const SG57213[] = {SGI255822, SGI255823, NULL};
const Segment * const SG57214[] = {SGI255824, SGI255825, NULL};
const Segment * const SG57215[] = {SGI255826, SGI255827, NULL};
const Segment * const SG57216[] = {SGI255828, SGI255829, NULL};
const Segment * const SG57217[] = {SGI255830, SGI255831, NULL};
const Segment * const SG57218[] = {SGI255832, SGI255833, NULL};
const Segment * const SG57219[] = {SGI255834, SGI255835, NULL};
const Segment * const SG57220[] = {SGI255836, SGI255837, NULL};
const Segment * const SG57221[] = {SGI255838, SGI255839, NULL};
const Segment * const SG57222[] = {SGI255840, SGI255841, NULL};
const Segment * const SG57223[] = {SGI255842, SGI255843, NULL};
const Segment * const SG57224[] = {SGI255844, SGI255845, NULL};
const Segment * const SG57225[] = {SGI255846, SGI255847, NULL};
const Segment * const SG57226[] = {SGI255848, SGI255849, NULL};
const Segment * const SG57227[] = {SGI255850, SGI255851, NULL};
const Segment * const SG57228[] = {SGI255852, SGI255853, NULL};
const Segment * const SG57229[] = {SGI255854, SGI255855, NULL};
const Segment * const SG57230[] = {SGI255856, SGI255857, NULL};
const Segment * const SG57231[] = {SGI255858, SGI255859, NULL};
const Segment * const SG57232[] = {SGI255860, SGI255861, NULL};
const Segment * const SG57233[] = {SGI255862, SGI255863, NULL};
const Segment * const SG57234[] = {SGI255864, SGI255865, NULL};
const Segment * const SG57235[] = {SGI255866, SGI255867, NULL};
const Segment * const SG57236[] = {SGI255868, SGI255869, NULL};
const Segment * const SG57237[] = {SGI255870, SGI255871, NULL};
const Segment * const SG57238[] = {SGI255872, SGI255873, NULL};
const Segment * const SG57239[] = {SGI255874, SGI255874, NULL};
const Segment * const SG57240[] = {SGI255875, SGI255876, NULL};
const Segment * const SG57241[] = {SGI255877, SGI255877, NULL};
const Segment * const SG57242[] = {SGI255878, SGI255879, NULL};
const Segment * const SG57243[] = {SGI255880, SGI255881, NULL};
const Segment * const SG57244[] = {SGI255882, SGI255883, NULL};
const Segment * const SG57245[] = {SGI255884, SGI255885, NULL};
const Segment * const SG57246[] = {SGI255886, SGI255886, NULL};
const Segment * const SG57247[] = {SGI255887, NULL};
const Segment * const SG57248[] = {SGI255888, NULL};
const Segment * const SG57249[] = {SGI255889, SGI255890, NULL};
const Segment * const SG57250[] = {SGI255891, SGI255892, NULL};
const Segment * const SG57251[] = {SGI255893, SGI255894, NULL};
const Segment * const SG57252[] = {SGI255895, SGI255895, NULL};
const Segment * const SG57253[] = {SGI255896, SGI255896, NULL};
const Segment * const SG57254[] = {SGI255897, SGI255898, NULL};
const Segment * const SG57255[] = {SGI255899, SGI255900, NULL};
const Segment * const SG57256[] = {SGI255901, SGI255902, NULL};
const Segment * const SG57257[] = {SGI255903, SGI255904, NULL};
const Segment * const SG57258[] = {SGI255905, SGI255906, NULL};
const Segment * const SG57259[] = {SGI255907, SGI255908, NULL};
const Segment * const SG57260[] = {SGI255909, SGI255910, NULL};
const Segment * const SG57261[] = {SGI255911, SGI255912, NULL};
const Segment * const SG57262[] = {SGI255913, SGI255914, NULL};
const Segment * const SG57263[] = {SGI255915, SGI255916, NULL};
const Segment * const SG57264[] = {SGI255917, SGI255918, NULL};
const Segment * const SG57265[] = {SGI255919, SGI255920, NULL};
const Segment * const SG57266[] = {SGI255921, SGI255922, NULL};
const Segment * const SG57267[] = {SGI255923, SGI255923, NULL};
const Segment * const SG57268[] = {SGI255924, SGI255925, NULL};
const Segment * const SG57269[] = {SGI255926, SGI255927, NULL};
const Segment * const SG57270[] = {SGI255928, SGI255929, NULL};
const Segment * const SG57271[] = {SGI255930, SGI255931, NULL};
const Segment * const SG57272[] = {SGI255932, SGI255933, NULL};
const Segment * const SG57273[] = {SGI255934, SGI255935, NULL};
const Segment * const SG57274[] = {SGI255936, SGI255937, NULL};
const Segment * const SG57275[] = {SGI255938, SGI255939, NULL};
const Segment * const SG57276[] = {SGI255940, SGI255941, NULL};
const Segment * const SG57277[] = {SGI255942, SGI255943, NULL};
const Segment * const SG57278[] = {SGI255944, SGI255945, NULL};
const Segment * const SG57279[] = {SGI255946, SGI255947, NULL};
const Segment * const SG57280[] = {SGI255948, SGI255949, NULL};
const Segment * const SG57281[] = {SGI255950, SGI255950, NULL};
const Segment * const SG57282[] = {SGI255951, SGI255952, NULL};
const Segment * const SG57283[] = {SGI255953, SGI255954, NULL};
const Segment * const SG57284[] = {SGI255955, SGI255956, NULL};
const Segment * const SG57285[] = {SGI255957, SGI255958, NULL};
const Segment * const SG57286[] = {SGI255959, SGI255960, NULL};
const Segment * const SG57287[] = {SGI255961, SGI255962, NULL};
const Segment * const SG57288[] = {SGI255963, SGI255964, NULL};
const Segment * const SG57289[] = {SGI255965, SGI255966, NULL};
const Segment * const SG57290[] = {SGI255967, SGI255968, NULL};
const Segment * const SG57291[] = {SGI255969, SGI255970, NULL};
const Segment * const SG57292[] = {SGI255971, SGI255972, NULL};
const Segment * const SG57293[] = {SGI255973, SGI255974, NULL};
const Segment * const SG57294[] = {SGI255975, SGI255976, NULL};
const Segment * const SG57295[] = {SGI255977, SGI255978, NULL};
const Segment * const SG57296[] = {SGI255979, SGI255980, NULL};
const Segment * const SG57297[] = {SGI255981, SGI255982, NULL};
const Segment * const SG57298[] = {SGI255983, SGI255984, NULL};
const Segment * const SG57299[] = {SGI255985, SGI255986, NULL};
const Segment * const SG57300[] = {SGI255987, SGI255988, NULL};
const Segment * const SG57301[] = {SGI255989, SGI255990, NULL};
const Segment * const SG57302[] = {SGI255991, SGI255991, NULL};
const Segment * const SG57303[] = {SGI255992, SGI255993, NULL};
const Segment * const SG57304[] = {SGI255994, SGI255995, NULL};
const Segment * const SG57305[] = {SGI255996, SGI255997, NULL};
const Segment * const SG57306[] = {SGI255998, SGI255999, NULL};
const Segment * const SG57307[] = {SGI256000, SGI256001, NULL};
const Segment * const SG57308[] = {SGI256002, SGI256003, NULL};
const Segment * const SG57309[] = {SGI256004, SGI256005, NULL};
const Segment * const SG57310[] = {SGI256006, SGI256007, NULL};
const Segment * const SG57311[] = {SGI256008, SGI256009, NULL};
const Segment * const SG57312[] = {SGI256010, SGI256011, NULL};
const Segment * const SG57313[] = {SGI256012, SGI256013, NULL};
const Segment * const SG57314[] = {SGI256014, SGI256015, NULL};
const Segment * const SG57315[] = {SGI256016, SGI256017, NULL};
const Segment * const SG57316[] = {SGI256018, SGI256018, NULL};
const Segment * const SG57317[] = {SGI256019, SGI256020, NULL};
const Segment * const SG57318[] = {SGI256021, SGI256022, NULL};
const Segment * const SG57319[] = {SGI256023, SGI256023, NULL};
const Segment * const SG57320[] = {SGI256024, SGI256024, NULL};
const Segment * const SG57321[] = {SGI256025, SGI256026, NULL};
const Segment * const SG57322[] = {SGI256027, SGI256028, NULL};
const Segment * const SG57323[] = {SGI256029, SGI256030, NULL};
const Segment * const SG57324[] = {SGI256031, SGI256032, NULL};
const Segment * const SG57325[] = {SGI256033, SGI256034, NULL};
const Segment * const SG57326[] = {SGI256035, SGI256035, NULL};
const Segment * const SG57327[] = {SGI256036, SGI256036, NULL};
const Segment * const SG57328[] = {SGI256037, SGI256038, NULL};
const Segment * const SG57329[] = {SGI256039, SGI256040, NULL};
const Segment * const SG57330[] = {SGI256041, SGI256042, NULL};
const Segment * const SG57331[] = {SGI256043, SGI256043, NULL};
const Segment * const SG57332[] = {SGI256044, SGI256045, NULL};
const Segment * const SG57333[] = {SGI256046, SGI256047, NULL};
const Segment * const SG57334[] = {SGI256048, SGI256048, NULL};
const Segment * const SG57335[] = {SGI256049, SGI256050, NULL};
const Segment * const SG57336[] = {SGI256051, SGI256052, NULL};
const Segment * const SG57337[] = {SGI256053, SGI256054, NULL};
const Segment * const SG57338[] = {SGI256055, SGI256056, NULL};
const Segment * const SG57339[] = {SGI256057, SGI256058, NULL};
const Segment * const SG57340[] = {SGI256059, SGI256060, NULL};
const Segment * const SG57341[] = {SGI256061, SGI256062, NULL};
const Segment * const SG57342[] = {SGI256063, SGI256063, NULL};
const Segment * const SG57343[] = {SGI256064, SGI256065, NULL};
const Segment * const SG57344[] = {SGI256066, SGI256066, NULL};
const Segment * const SG57345[] = {SGI256067, SGI256068, NULL};
const Segment * const SG57346[] = {SGI256069, SGI256070, NULL};
const Segment * const SG57347[] = {SGI256071, SGI256072, NULL};
const Segment * const SG57348[] = {SGI256073, SGI256074, NULL};
const Segment * const SG57349[] = {SGI256075, SGI256076, NULL};
const Segment * const SG57350[] = {SGI256077, SGI256078, NULL};
const Segment * const SG57351[] = {SGI256079, SGI256080, NULL};
const Segment * const SG57352[] = {SGI256081, SGI256082, NULL};
const Segment * const SG57353[] = {SGI256083, SGI256084, NULL};
const Segment * const SG57354[] = {SGI256085, SGI256086, NULL};
const Segment * const SG57355[] = {SGI256087, SGI256088, NULL};
const Segment * const SG57356[] = {SGI256089, SGI256090, NULL};
const Segment * const SG57357[] = {SGI256091, SGI256092, NULL};
const Segment * const SG57358[] = {SGI256093, SGI256094, NULL};
const Segment * const SG57359[] = {SGI256095, SGI256096, NULL};
const Segment * const SG57360[] = {SGI256097, SGI256098, NULL};
const Segment * const SG57361[] = {SGI256099, SGI256100, NULL};
const Segment * const SG57362[] = {SGI256101, SGI256101, NULL};
const Segment * const SG57363[] = {SGI256102, SGI256103, NULL};
const Segment * const SG57364[] = {SGI256104, SGI256104, NULL};
const Segment * const SG57365[] = {SGI256105, SGI256106, NULL};
const Segment * const SG57366[] = {SGI256107, SGI256108, NULL};
const Segment * const SG57367[] = {SGI256109, SGI256110, NULL};
const Segment * const SG57368[] = {SGI256111, SGI256112, NULL};
const Segment * const SG57369[] = {SGI256113, SGI256114, NULL};
const Segment * const SG57370[] = {SGI256115, SGI256116, NULL};
const Segment * const SG57371[] = {SGI256117, SGI256117, NULL};
const Segment * const SG57372[] = {SGI256118, SGI256119, NULL};
const Segment * const SG57373[] = {SGI256120, SGI0, SGI256121, SGI0, SGI256122, SGI256123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57374[] = {SGI256124, SGI0, SGI256125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57375[] = {SGI256126, SGI0, SGI256127, SGI0, SGI256128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57376[] = {SGI256129, SGI0, SGI256130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57377[] = {SGI256131, SGI0, SGI256132, SGI0, SGI256133, SGI256134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57378[] = {SGI256135, SGI0, SGI256136, SGI0, SGI256137, SGI256138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57379[] = {SGI256139, SGI0, SGI256140, SGI0, SGI256141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57380[] = {SGI256142, SGI0, SGI256143, SGI0, SGI256144, SGI256145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57381[] = {SGI256146, SGI256147, SGI256148, SGI0, SGI256149, SGI256150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57382[] = {SGI256151, SGI0, SGI256152, SGI0, SGI256153, SGI256154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57383[] = {SGI256155, SGI0, SGI256156, SGI0, SGI0, SGI256157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57384[] = {SGI256158, SGI0, SGI256159, SGI0, SGI256160, SGI256161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57385[] = {SGI256162, SGI0, SGI256163, SGI0, SGI256164, SGI256165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57386[] = {SGI256166, SGI256167, SGI256168, SGI0, SGI256169, SGI256170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57387[] = {SGI256171, SGI0, SGI0, SGI0, SGI256172, SGI256173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57388[] = {SGI256174, SGI0, SGI256175, SGI0, SGI256176, SGI256177, SGI256178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57389[] = {SGI256179, SGI0, SGI256180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57390[] = {SGI256181, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57391[] = {SGI256182, SGI256183, SGI256184, SGI0, SGI256185, SGI256186, SGI256187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57392[] = {SGI256188, SGI256189, SGI256190, SGI0, SGI256191, SGI256192, SGI256193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57393[] = {SGI256194, SGI256195, SGI256196, SGI0, SGI256197, SGI256198, SGI256199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57394[] = {SGI256200, SGI256201, SGI256202, SGI0, SGI256203, SGI256204, SGI256205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57395[] = {SGI256206, SGI256207, SGI256208, SGI0, SGI256209, SGI256210, SGI256211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57396[] = {SGI256212, SGI256213, SGI256214, SGI0, SGI256215, SGI256216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57397[] = {SGI256217, SGI256218, SGI256219, SGI0, SGI256220, SGI256221, SGI256222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57398[] = {SGI256223, SGI256224, SGI256225, SGI0, SGI256226, SGI256227, SGI256228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57399[] = {SGI256229, SGI256230, SGI256231, SGI0, SGI256232, SGI256233, SGI256234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57400[] = {SGI256235, SGI256236, SGI256237, SGI0, SGI256238, SGI256239, SGI256240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57401[] = {SGI256241, SGI256242, SGI256243, SGI0, SGI256244, SGI256245, SGI256246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57402[] = {SGI256247, SGI256248, SGI256249, SGI0, SGI256250, SGI256251, SGI256252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57403[] = {SGI256253, SGI256254, SGI256255, SGI0, SGI256256, SGI256257, SGI256258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57404[] = {SGI256259, SGI256260, SGI256261, SGI0, SGI256262, SGI256263, SGI256264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57405[] = {SGI256265, SGI256266, SGI256267, SGI0, SGI256268, SGI256269, SGI256270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57406[] = {SGI256271, SGI256272, SGI256273, SGI0, SGI256274, SGI256275, SGI256276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57407[] = {SGI256277, SGI256278, SGI256279, SGI0, SGI256280, SGI0, SGI256280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57408[] = {SGI256281, SGI256282, SGI256283, SGI0, SGI256284, SGI256285, SGI256286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57409[] = {SGI256287, SGI256288, SGI256289, SGI0, SGI256290, SGI256291, SGI256292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57410[] = {SGI256293, SGI256294, SGI256295, SGI0, SGI256296, SGI256297, SGI256298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57411[] = {SGI256299, SGI256300, SGI256301, SGI0, SGI256302, SGI256303, SGI256304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57412[] = {SGI256305, SGI256306, SGI256307, SGI0, SGI256308, SGI256309, SGI256310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57413[] = {SGI256311, SGI0, SGI256312, SGI0, SGI256313, SGI256314, SGI256315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57414[] = {SGI256316, SGI0, SGI256317, SGI0, SGI256318, SGI256319, SGI256320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57415[] = {SGI256321, SGI0, SGI256322, SGI0, SGI256323, SGI256324, SGI256325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57416[] = {SGI256326, SGI0, SGI256327, SGI0, SGI256328, SGI256329, SGI256330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57417[] = {SGI256331, SGI0, SGI256332, SGI0, SGI256333, SGI256334, SGI256335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57418[] = {SGI256336, SGI0, SGI256337, SGI0, SGI256338, SGI256339, SGI256340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57419[] = {SGI256341, SGI0, SGI256342, SGI0, SGI256343, SGI256344, SGI256345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57420[] = {SGI256346, SGI0, SGI256347, SGI0, SGI256348, SGI256349, SGI256350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57421[] = {SGI256351, SGI0, SGI256352, SGI0, SGI256353, SGI256354, SGI256355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57422[] = {SGI256356, SGI256357, SGI256358, SGI0, SGI256359, SGI256360, SGI256361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57423[] = {SGI256362, SGI256363, SGI256364, SGI0, SGI256365, SGI256366, SGI256367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57424[] = {SGI256368, SGI256369, SGI256370, SGI0, SGI256371, SGI256372, SGI256373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57425[] = {SGI256374, SGI256375, SGI256376, SGI0, SGI256377, SGI256378, SGI256379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57426[] = {SGI256380, SGI256381, SGI256382, SGI0, SGI256383, SGI256384, SGI256385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57427[] = {SGI256386, SGI256387, SGI256388, SGI0, SGI256389, SGI256390, SGI256391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57428[] = {SGI256392, SGI256393, SGI256394, SGI0, SGI256395, SGI256396, SGI256397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57429[] = {SGI256398, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57430[] = {SGI256399, SGI256400, SGI0, SGI0, NULL};
const Segment * const SG57431[] = {SGI256401, SGI256402, SGI0, SGI0, NULL};
const Segment * const SG57432[] = {SGI256403, SGI0, SGI256404, SGI0, SGI256405, SGI256406, SGI256407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57433[] = {SGI256408, SGI0, SGI256409, SGI0, SGI256410, SGI256411, SGI256412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57434[] = {SGI256413, SGI0, SGI256414, SGI0, SGI256415, SGI256416, SGI256417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57435[] = {SGI256418, SGI0, SGI256419, SGI0, SGI256420, SGI256421, SGI256422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57436[] = {SGI256423, SGI0, SGI256424, SGI0, SGI256425, SGI256426, SGI256427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57437[] = {SGI256428, SGI0, SGI256429, SGI0, SGI256430, SGI256431, SGI256432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57438[] = {SGI256433, SGI0, SGI256434, SGI0, SGI256435, SGI256436, SGI256437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57439[] = {SGI256438, SGI0, SGI256439, SGI0, SGI256440, SGI256441, SGI256442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57440[] = {SGI256443, SGI256444, SGI256445, SGI0, SGI256446, SGI256447, SGI256448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57441[] = {SGI256449, SGI256450, SGI256451, SGI0, SGI256452, SGI256453, SGI256454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57442[] = {SGI256455, SGI256456, SGI256457, SGI0, SGI256458, SGI256459, SGI256460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57443[] = {SGI256461, SGI256462, SGI256463, SGI0, SGI256464, SGI256465, SGI256466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57444[] = {SGI256467, SGI256468, SGI256469, SGI0, SGI256470, SGI256471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57445[] = {SGI256472, SGI256473, SGI256474, SGI0, SGI256475, SGI256476, SGI256477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57446[] = {SGI256478, SGI256479, SGI256480, SGI0, SGI256481, SGI256482, SGI256483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57447[] = {SGI256484, SGI256485, SGI256486, SGI0, SGI256487, SGI256488, SGI256489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57448[] = {SGI256490, SGI256491, SGI256492, SGI0, SGI256493, SGI0, SGI256494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57449[] = {SGI256495, SGI256496, SGI256497, SGI0, SGI256498, SGI256499, SGI256500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57450[] = {SGI256501, SGI0, SGI256502, SGI0, SGI256503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57451[] = {SGI256504, SGI0, SGI256505, SGI0, SGI256506, SGI256507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57452[] = {SGI256508, SGI0, SGI256509, SGI0, SGI256510, SGI256511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57453[] = {SGI256512, SGI0, SGI256513, SGI0, SGI256514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57454[] = {SGI256515, SGI0, SGI256516, SGI0, SGI256517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57455[] = {SGI256518, SGI0, SGI256519, SGI0, SGI256520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57456[] = {SGI256521, SGI0, SGI256522, SGI0, SGI256523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57457[] = {SGI256524, SGI0, SGI256525, SGI0, SGI256526, SGI256527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57458[] = {SGI256528, SGI0, SGI256529, SGI0, SGI256530, SGI256531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57459[] = {SGI256532, SGI0, SGI256533, SGI0, SGI256534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57460[] = {SGI256535, SGI0, SGI256536, SGI0, SGI256537, SGI256538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57461[] = {SGI256539, SGI0, SGI256540, SGI0, SGI256541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57462[] = {SGI256542, SGI0, SGI256543, SGI0, SGI256544, SGI256545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57463[] = {SGI256546, SGI0, SGI256547, SGI0, SGI256548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57464[] = {SGI256549, SGI0, SGI256550, SGI0, SGI256551, SGI256552, SGI256551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57465[] = {SGI256553, SGI0, SGI256554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57466[] = {SGI256555, SGI0, SGI256556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57467[] = {SGI256557, SGI0, SGI256558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57468[] = {SGI256559, SGI0, SGI256560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57469[] = {SGI256561, SGI0, SGI256562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57470[] = {SGI256563, SGI0, SGI256564, SGI0, SGI256565, SGI256566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57471[] = {SGI256567, SGI0, SGI256568, SGI0, SGI256569, SGI256570, SGI256569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57472[] = {SGI256571, SGI0, SGI256572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57473[] = {SGI256573, SGI0, SGI256574, SGI0, SGI256575, SGI256576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57474[] = {SGI256577, SGI0, SGI256578, SGI0, SGI256579, SGI256580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57475[] = {SGI256581, SGI0, SGI256582, SGI0, SGI256583, SGI256584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57476[] = {SGI256585, SGI0, SGI256586, SGI0, SGI256587, SGI256588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57477[] = {SGI256589, SGI0, SGI256590, SGI0, SGI256591, SGI256592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57478[] = {SGI256593, SGI0, SGI256594, SGI0, SGI256595, SGI256596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57479[] = {SGI256597, SGI0, SGI256598, SGI0, SGI256599, SGI256600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57480[] = {SGI256601, SGI0, SGI256602, SGI0, SGI256603, SGI256604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57481[] = {SGI256605, SGI0, SGI256606, SGI0, SGI256607, SGI256608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57482[] = {SGI256609, SGI0, SGI256610, SGI0, SGI256611, SGI256612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57483[] = {SGI256613, SGI0, SGI256614, SGI0, SGI256615, SGI256616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57484[] = {SGI256617, SGI0, SGI256618, SGI0, SGI256619, SGI256620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57485[] = {SGI256621, SGI0, SGI256622, SGI0, SGI256623, SGI256624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57486[] = {SGI256625, SGI0, SGI256626, SGI0, SGI256627, SGI256628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57487[] = {SGI256629, SGI0, SGI0, SGI0, SGI256630, SGI256631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57488[] = {SGI256632, SGI0, SGI256633, SGI0, SGI256634, SGI256635, SGI256636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57489[] = {SGI256637, SGI0, SGI256638, SGI0, SGI256639, SGI256640, SGI256641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57490[] = {SGI256642, SGI0, SGI256643, SGI0, SGI256644, SGI256645, SGI256646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57491[] = {SGI256647, SGI0, SGI256648, SGI0, SGI256649, SGI256650, SGI256651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57492[] = {SGI256652, SGI0, SGI256653, SGI0, SGI256654, SGI256655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57493[] = {SGI256656, SGI0, SGI256657, SGI0, SGI256658, SGI256659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57494[] = {SGI256660, SGI0, SGI256661, SGI0, SGI256662, SGI256663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57495[] = {SGI256664, SGI0, SGI256665, SGI0, SGI256666, SGI256667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57496[] = {SGI256668, SGI0, SGI256669, SGI0, SGI256670, SGI256671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57497[] = {SGI256672, SGI0, SGI256673, SGI0, SGI256674, SGI256675, SGI256676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI256677, SGI0, NULL};
const Segment * const SG57498[] = {SGI256678, SGI0, SGI256679, SGI0, SGI256680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57499[] = {SGI256681, SGI0, SGI256682, SGI0, SGI256683, SGI256684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57500[] = {SGI256685, SGI0, SGI256686, SGI0, SGI256687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57501[] = {SGI256688, SGI0, SGI256689, SGI0, SGI256690, SGI256691, SGI256692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI256693, SGI0, NULL};
const Segment * const SG57502[] = {SGI256694, SGI0, SGI256695, SGI0, SGI256696, SGI256697, SGI256698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57503[] = {SGI256699, SGI0, SGI256700, SGI0, SGI256701, SGI256702, SGI256703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI256704, SGI0, NULL};
const Segment * const SG57504[] = {SGI256705, SGI0, SGI256706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57505[] = {SGI256707, SGI0, SGI256708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57506[] = {SGI256709, SGI0, SGI256710, SGI0, SGI256711, SGI256712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57507[] = {SGI256713, SGI0, SGI256714, SGI0, SGI256715, SGI256716, SGI256717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI256718, SGI0, NULL};
const Segment * const SG57508[] = {SGI256719, SGI0, SGI256720, SGI0, SGI256721, SGI256722, SGI256723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57509[] = {SGI256724, SGI0, SGI256725, SGI0, SGI256726, SGI256727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57510[] = {SGI256728, SGI0, SGI256729, SGI0, SGI256730, SGI256731, SGI256732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI256733, SGI0, NULL};
const Segment * const SG57511[] = {SGI256734, SGI0, SGI256735, SGI0, SGI256736, SGI256737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI256738, SGI0, NULL};
const Segment * const SG57512[] = {SGI256739, SGI0, SGI256740, SGI0, SGI256741, SGI256742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57513[] = {SGI256743, SGI0, SGI256744, SGI0, SGI256745, SGI256746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57514[] = {SGI256747, SGI0, SGI256748, SGI0, SGI256749, SGI256750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57515[] = {SGI256751, SGI0, SGI256752, SGI0, SGI256753, SGI256754, SGI256755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57516[] = {SGI256756, SGI0, SGI256757, SGI0, SGI256758, SGI256759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57517[] = {SGI256760, SGI256761, SGI256762, SGI0, SGI256763, SGI256764, SGI256765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57518[] = {SGI256766, SGI256767, SGI256768, SGI0, SGI256769, SGI256770, SGI256771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57519[] = {SGI256772, SGI256773, SGI256774, SGI0, SGI256775, SGI256776, SGI256777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57520[] = {SGI256778, SGI256779, SGI256780, SGI0, SGI256781, SGI256782, SGI256783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57521[] = {SGI256784, SGI256785, SGI256786, SGI0, SGI256787, SGI256788, SGI256789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57522[] = {SGI256790, SGI256791, SGI256792, SGI0, SGI256793, SGI256794, SGI256795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57523[] = {SGI256796, SGI256797, SGI256798, SGI0, SGI256799, SGI256800, SGI256801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57524[] = {SGI256802, SGI256803, SGI256804, SGI0, SGI256805, SGI256806, SGI256807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57525[] = {SGI256808, SGI256809, SGI256810, SGI0, SGI256811, SGI256812, SGI256813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57526[] = {SGI256814, SGI256815, SGI256816, SGI0, SGI256817, SGI256818, SGI256819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57527[] = {SGI256820, SGI256821, SGI256822, SGI0, SGI256823, SGI256824, SGI256825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57528[] = {SGI256826, SGI256827, SGI256828, SGI0, SGI256829, SGI256830, SGI256831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57529[] = {SGI256832, SGI256833, SGI256834, SGI0, SGI256835, SGI256836, SGI256837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57530[] = {SGI256838, SGI256839, SGI256840, SGI0, SGI256841, SGI256842, SGI256843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57531[] = {SGI256844, SGI0, SGI256845, SGI0, SGI256846, SGI256847, SGI256848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57532[] = {SGI256849, SGI0, SGI256850, SGI0, SGI256851, SGI256852, SGI256853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57533[] = {SGI256854, SGI0, SGI256855, SGI0, SGI256856, SGI256857, SGI256858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57534[] = {SGI256859, SGI0, SGI256860, SGI0, SGI256861, SGI256862, SGI256863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57535[] = {SGI256864, SGI0, SGI256865, SGI0, SGI256866, SGI256867, SGI256868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57536[] = {SGI256869, SGI0, SGI256870, SGI0, SGI256871, SGI256872, SGI256873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57537[] = {SGI256874, SGI0, SGI256875, SGI0, SGI256876, SGI256877, SGI256878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57538[] = {SGI256879, SGI0, SGI256880, SGI0, SGI256881, SGI256882, SGI256883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57539[] = {SGI256884, SGI0, SGI256885, SGI0, SGI256886, SGI256887, SGI256888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57540[] = {SGI256889, SGI0, SGI256890, SGI0, SGI256891, SGI256892, SGI256893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57541[] = {SGI256894, SGI0, SGI256895, SGI0, SGI256896, SGI256897, SGI256898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57542[] = {SGI256899, SGI0, SGI256900, SGI0, SGI256901, SGI256902, SGI256903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57543[] = {SGI256904, SGI0, SGI256905, SGI0, SGI256906, SGI256907, SGI256908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57544[] = {SGI256909, SGI256910, SGI256911, SGI0, SGI256912, SGI256913, SGI256914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57545[] = {SGI256915, SGI256916, SGI256917, SGI0, SGI256918, SGI256919, SGI256920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57546[] = {SGI256921, SGI256922, SGI256923, SGI0, SGI256924, SGI256925, SGI256926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57547[] = {SGI256927, SGI256928, SGI256929, SGI0, SGI256930, SGI256931, SGI256932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57548[] = {SGI256933, SGI256934, SGI256935, SGI0, SGI256936, SGI256937, SGI256938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57549[] = {SGI256939, SGI256940, SGI256941, SGI0, SGI256942, SGI256943, SGI256944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57550[] = {SGI256945, SGI256946, SGI256947, SGI0, SGI256948, SGI256949, SGI256950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57551[] = {SGI256951, SGI256952, SGI256953, SGI0, SGI256954, SGI256955, SGI256956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57552[] = {SGI256957, SGI256958, SGI256959, SGI0, SGI256960, SGI256961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57553[] = {SGI256962, SGI256963, SGI256964, SGI0, SGI256965, SGI256966, SGI256967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57554[] = {SGI256968, SGI256969, SGI256970, SGI0, SGI256971, SGI256972, SGI256973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57555[] = {SGI256974, SGI256975, SGI0, SGI0, SGI256976, SGI256977, SGI256978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57556[] = {SGI256979, SGI256980, SGI256981, SGI0, SGI256982, SGI256983, SGI256984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57557[] = {SGI256985, SGI256986, SGI256987, SGI0, SGI256988, SGI256989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57558[] = {SGI256990, SGI256991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57559[] = {SGI256992, SGI256993, SGI256994, SGI0, SGI256995, SGI256996, SGI256997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57560[] = {SGI256998, SGI256999, SGI257000, SGI0, SGI257001, SGI257002, SGI257003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57561[] = {SGI257004, SGI257005, SGI257006, SGI0, SGI257007, SGI257008, SGI257009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57562[] = {SGI257010, SGI257011, SGI257012, SGI0, SGI257013, SGI257014, SGI257015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57563[] = {SGI257016, SGI257017, SGI257018, SGI0, SGI257019, SGI257020, SGI257021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57564[] = {SGI257022, SGI257023, SGI257024, SGI0, SGI257025, SGI257026, SGI257027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57565[] = {SGI257028, SGI257029, SGI257030, SGI0, SGI257031, SGI257032, SGI257033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57566[] = {SGI257034, SGI257035, SGI257036, SGI0, SGI257037, SGI257038, SGI257039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57567[] = {SGI257040, SGI257041, SGI257042, SGI0, SGI257043, SGI257044, SGI257045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57568[] = {SGI257046, SGI257047, SGI257048, SGI0, SGI257049, SGI257050, SGI257051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57569[] = {SGI257052, SGI257053, SGI257054, SGI0, SGI257055, SGI257056, SGI257057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57570[] = {SGI257058, SGI0, SGI257059, SGI0, SGI257060, SGI257061, SGI257062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57571[] = {SGI257063, SGI0, SGI257064, SGI0, SGI257065, SGI257066, SGI257067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57572[] = {SGI257068, SGI0, SGI257069, SGI0, SGI257070, SGI257071, SGI257072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57573[] = {SGI257073, SGI0, SGI257074, SGI0, SGI257075, SGI257076, SGI257077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57574[] = {SGI257078, SGI0, SGI257079, SGI0, SGI257080, SGI257081, SGI257082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57575[] = {SGI257083, SGI0, SGI257084, SGI0, SGI257085, SGI257086, SGI257087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57576[] = {SGI257088, SGI257089, SGI257090, SGI0, SGI257091, SGI257092, SGI257093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57577[] = {SGI257094, SGI257095, SGI257096, SGI0, SGI257097, SGI257098, SGI257099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57578[] = {SGI257100, SGI257101, SGI257102, SGI0, SGI257103, SGI257104, SGI257105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57579[] = {SGI257106, SGI0, SGI257107, SGI0, SGI257108, SGI257109, SGI257110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57580[] = {SGI257111, SGI0, SGI257112, SGI0, SGI257113, SGI257114, SGI257115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57581[] = {SGI257116, SGI0, SGI257117, SGI0, SGI257118, SGI257119, SGI257120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57582[] = {SGI257121, SGI257122, SGI257123, SGI0, SGI257124, SGI257125, SGI257126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57583[] = {SGI257127, SGI257128, SGI257129, SGI0, SGI257130, SGI257131, SGI257132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57584[] = {SGI257133, SGI257134, SGI257135, SGI0, SGI257136, SGI257137, SGI257138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57585[] = {SGI257139, SGI257140, SGI257141, SGI0, SGI257142, SGI257143, SGI257144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57586[] = {SGI257145, SGI257146, SGI257147, SGI0, SGI257148, SGI257149, SGI257150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57587[] = {SGI257151, SGI257152, SGI257153, SGI0, SGI257154, SGI257155, SGI257156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57588[] = {SGI257157, SGI257158, SGI257159, SGI0, SGI257160, SGI257161, SGI257162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57589[] = {SGI257163, SGI257164, SGI257165, SGI0, SGI257166, SGI257167, SGI257168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57590[] = {SGI257169, SGI257170, SGI257171, SGI0, SGI257172, SGI257173, SGI257174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57591[] = {SGI257175, SGI257176, SGI257177, SGI0, SGI257178, SGI257179, SGI257180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57592[] = {SGI257181, SGI257182, SGI257183, SGI0, SGI257184, SGI257185, SGI257186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57593[] = {SGI257187, SGI257188, SGI257189, SGI0, SGI257190, SGI257191, SGI257192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57594[] = {SGI257193, SGI257194, SGI257195, SGI0, SGI257196, SGI257197, SGI257198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57595[] = {SGI257199, SGI257200, SGI257201, SGI0, SGI257202, SGI257203, SGI257204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57596[] = {SGI257205, SGI257206, SGI257207, SGI0, SGI257208, SGI257209, SGI257210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57597[] = {SGI257211, SGI257212, SGI257213, SGI0, SGI257214, SGI257215, SGI257216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57598[] = {SGI257217, SGI257218, SGI257219, SGI0, SGI257220, SGI257221, SGI257222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57599[] = {SGI257223, SGI257224, SGI257225, SGI0, SGI257226, SGI257227, SGI257228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57600[] = {SGI257229, SGI257230, SGI257231, SGI0, SGI257232, SGI257233, SGI257234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57601[] = {SGI257235, SGI257236, SGI257237, SGI0, SGI257238, SGI257239, SGI257240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57602[] = {SGI257241, SGI257242, SGI257243, SGI0, SGI257244, SGI257245, SGI257246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57603[] = {SGI257247, SGI257248, SGI257249, SGI0, SGI257250, SGI257251, SGI257252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57604[] = {SGI257253, SGI257254, SGI257255, SGI0, SGI257256, SGI257257, SGI257258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57605[] = {SGI257259, SGI257260, SGI257261, SGI0, SGI257262, SGI257263, SGI257264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57606[] = {SGI257265, SGI257266, SGI257267, SGI0, SGI257268, SGI257269, SGI257270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57607[] = {SGI257271, SGI257272, SGI257273, SGI0, SGI257274, SGI257275, SGI257276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57608[] = {SGI257277, SGI257278, SGI257279, SGI0, SGI257280, SGI257281, SGI257282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57609[] = {SGI257283, SGI257284, SGI257285, SGI0, SGI257286, SGI257287, SGI257288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57610[] = {SGI257289, SGI257290, SGI257291, SGI0, SGI257292, SGI257293, SGI257294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57611[] = {SGI257295, SGI257296, SGI257297, SGI0, SGI257298, SGI257299, SGI257300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57612[] = {SGI257301, SGI257302, SGI257303, SGI0, SGI257304, SGI257305, SGI257306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57613[] = {SGI257307, SGI257308, SGI257309, SGI0, SGI257310, SGI257311, SGI257312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57614[] = {SGI257313, SGI257314, SGI257315, SGI0, SGI257316, SGI257317, SGI257318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57615[] = {SGI257319, SGI257320, SGI257321, SGI0, SGI257322, SGI257323, SGI257324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57616[] = {SGI257325, SGI257326, SGI257327, SGI0, SGI257328, SGI257329, SGI257330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57617[] = {SGI257331, SGI257332, SGI257333, SGI0, SGI257334, SGI257335, SGI257336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57618[] = {SGI257337, SGI257338, SGI257339, SGI0, SGI257340, SGI257341, SGI257342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57619[] = {SGI257343, SGI257344, SGI257345, SGI0, SGI257346, SGI257347, SGI257348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57620[] = {SGI257349, SGI257350, SGI257351, SGI0, SGI257352, SGI257353, SGI257354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57621[] = {SGI257355, SGI257356, SGI0, SGI0, NULL};
const Segment * const SG57622[] = {SGI257357, SGI257358, SGI0, SGI0, NULL};
const Segment * const SG57623[] = {SGI257359, SGI257360, SGI257361, SGI0, SGI257362, SGI257363, SGI257364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57624[] = {SGI257365, SGI257366, SGI257367, SGI0, SGI257368, SGI257369, SGI257370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57625[] = {SGI257371, SGI257372, SGI257373, SGI0, SGI257374, SGI257375, SGI257376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57626[] = {SGI257377, SGI257378, SGI257379, SGI0, SGI257380, SGI257381, SGI257382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57627[] = {SGI257383, SGI257384, SGI257385, SGI0, SGI257386, SGI257387, SGI257388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57628[] = {SGI257389, SGI257390, SGI257391, SGI0, SGI257392, SGI257393, SGI257392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57629[] = {SGI257394, SGI257395, SGI257396, SGI0, SGI257397, SGI257398, SGI257399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57630[] = {SGI257400, SGI257401, SGI257402, SGI0, SGI257403, SGI257404, SGI257405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57631[] = {SGI257406, SGI257407, SGI257408, SGI0, SGI257409, SGI257410, SGI257411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57632[] = {SGI257412, SGI257413, SGI257414, SGI0, SGI257415, SGI257416, SGI257417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57633[] = {SGI257418, SGI0, SGI257419, SGI0, SGI257420, SGI257421, SGI0, SGI0, SGI0, SGI257422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57634[] = {SGI257423, SGI0, SGI257424, SGI0, SGI257425, SGI257426, SGI0, SGI0, SGI0, SGI257427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57635[] = {SGI257428, SGI0, SGI257429, SGI0, SGI257430, SGI257431, SGI0, SGI0, SGI0, SGI257432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57636[] = {SGI257433, SGI0, SGI257434, SGI0, SGI257435, SGI257436, SGI0, SGI0, SGI0, SGI257437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57637[] = {SGI257438, SGI0, SGI257439, SGI0, SGI257440, SGI257441, SGI0, SGI0, SGI0, SGI257442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57638[] = {SGI257443, SGI0, SGI257444, SGI0, SGI257445, SGI257446, SGI0, SGI0, SGI0, SGI257447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57639[] = {SGI257448, SGI0, SGI257449, SGI0, SGI257450, SGI257451, SGI0, SGI0, SGI0, SGI257452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57640[] = {SGI257453, SGI0, SGI257454, SGI0, SGI257455, SGI257456, SGI0, SGI0, SGI0, SGI257457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57641[] = {SGI257458, SGI0, SGI257459, SGI0, SGI257460, SGI257461, SGI0, SGI0, SGI0, SGI257462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57642[] = {SGI257463, SGI0, SGI257464, SGI0, SGI257465, SGI257466, SGI0, SGI0, SGI0, SGI257467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57643[] = {SGI257468, SGI0, SGI257469, SGI0, SGI257470, SGI257471, SGI0, SGI0, SGI0, SGI257472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57644[] = {SGI257473, SGI0, SGI257474, SGI0, SGI257475, SGI257476, SGI0, SGI0, SGI0, SGI257477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57645[] = {SGI257478, SGI257479, SGI0, SGI0, NULL};
const Segment * const SG57646[] = {SGI257480, SGI257481, SGI0, SGI0, NULL};
const Segment * const SG57647[] = {SGI257482, SGI257483, SGI0, SGI0, NULL};
const Segment * const SG57648[] = {SGI257484, SGI257485, SGI0, SGI0, NULL};
const Segment * const SG57649[] = {SGI257486, SGI257487, SGI257488, SGI0, SGI257489, SGI257490, SGI257491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57650[] = {SGI257492, SGI257493, SGI257494, SGI0, SGI257495, SGI257496, SGI257497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57651[] = {SGI257498, SGI257499, SGI257500, SGI0, SGI257501, SGI257502, SGI257503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57652[] = {SGI257504, SGI257505, SGI257506, SGI0, SGI257507, SGI257508, SGI257509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57653[] = {SGI257510, SGI257511, SGI257512, SGI0, SGI257513, SGI257514, SGI257515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57654[] = {SGI257516, SGI257517, SGI257518, SGI0, SGI257519, SGI257520, SGI257521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57655[] = {SGI257522, SGI257523, SGI257524, SGI0, SGI257525, SGI257526, SGI257527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57656[] = {SGI257528, SGI257529, SGI0, SGI0, NULL};
const Segment * const SG57657[] = {SGI257530, SGI0, SGI257531, SGI0, SGI257532, SGI257533, SGI257534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57658[] = {SGI257535, SGI0, SGI257536, SGI0, SGI257537, SGI257538, SGI257539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57659[] = {SGI257540, SGI0, SGI257541, SGI0, SGI257542, SGI257543, SGI257544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57660[] = {SGI257545, SGI0, SGI257546, SGI0, SGI257547, SGI257548, SGI257549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57661[] = {SGI257550, SGI0, SGI257551, SGI0, SGI257552, SGI257553, SGI257554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57662[] = {SGI257555, SGI0, SGI257556, SGI0, SGI257557, SGI257558, SGI257559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57663[] = {SGI257560, SGI0, SGI257561, SGI0, SGI257562, SGI257563, SGI257564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57664[] = {SGI257565, SGI0, SGI257566, SGI0, SGI257567, SGI257568, SGI257569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57665[] = {SGI257570, SGI0, SGI257571, SGI0, SGI257572, SGI257573, SGI257574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57666[] = {SGI257575, SGI0, SGI257576, SGI0, SGI257577, SGI257578, SGI257579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57667[] = {SGI257580, SGI0, SGI257581, SGI0, SGI257582, SGI257583, SGI257584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57668[] = {SGI257585, SGI0, SGI257586, SGI0, SGI257587, SGI257588, SGI257589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57669[] = {SGI257590, SGI0, SGI257591, SGI0, SGI257592, SGI257593, SGI257594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57670[] = {SGI257595, SGI0, SGI257596, SGI0, SGI257597, SGI257598, SGI257599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57671[] = {SGI257600, SGI0, SGI257601, SGI0, SGI257602, SGI257603, SGI257604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57672[] = {SGI257605, SGI0, SGI257606, SGI0, SGI257607, SGI257608, SGI257609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57673[] = {SGI257610, SGI0, SGI257611, SGI0, SGI257612, SGI257613, SGI257614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57674[] = {SGI257615, SGI0, SGI257616, SGI0, SGI257617, SGI257618, SGI257619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57675[] = {SGI257620, SGI0, SGI257621, SGI0, SGI257622, SGI257623, SGI257624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57676[] = {SGI257625, SGI0, SGI257626, SGI0, SGI257627, SGI257628, SGI257629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57677[] = {SGI257630, SGI0, SGI257631, SGI0, SGI257632, SGI257633, SGI257634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57678[] = {SGI257635, SGI0, SGI257636, SGI0, SGI257637, SGI257638, SGI257639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57679[] = {SGI257640, SGI0, SGI257641, SGI0, SGI257642, SGI257643, SGI257644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57680[] = {SGI257645, SGI0, SGI257646, SGI0, SGI257647, SGI257648, SGI257649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57681[] = {SGI257650, SGI0, SGI257651, SGI0, SGI257652, SGI257653, SGI257654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57682[] = {SGI257655, SGI0, SGI257656, SGI0, SGI257657, SGI257658, SGI257659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57683[] = {SGI257660, SGI0, SGI257661, SGI0, SGI257662, SGI257663, SGI257664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57684[] = {SGI257665, SGI0, SGI257666, SGI0, SGI257667, SGI257668, SGI257669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57685[] = {SGI257670, SGI0, SGI257671, SGI0, SGI257672, SGI257673, SGI257674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57686[] = {SGI257675, SGI0, SGI257676, SGI0, SGI257677, SGI257678, SGI257679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57687[] = {SGI257680, SGI0, SGI257681, SGI0, SGI257682, SGI257683, SGI257684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57688[] = {SGI257685, SGI0, SGI257686, SGI0, SGI257687, SGI257688, SGI257689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57689[] = {SGI257690, SGI0, SGI257691, SGI0, SGI257692, SGI257693, SGI257694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57690[] = {SGI257695, SGI0, SGI257696, SGI0, SGI257697, SGI257698, SGI257699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57691[] = {SGI257700, SGI0, SGI257701, SGI0, SGI257702, SGI257703, SGI257704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57692[] = {SGI257705, SGI0, SGI257706, SGI0, SGI257707, SGI257708, SGI257709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57693[] = {SGI257710, SGI0, SGI257711, SGI0, SGI257712, SGI257713, SGI257714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57694[] = {SGI257715, SGI0, SGI257716, SGI0, SGI257717, SGI257718, SGI257719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57695[] = {SGI257720, SGI0, SGI257721, SGI0, SGI257722, SGI257723, SGI257724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57696[] = {SGI257725, SGI0, SGI257726, SGI0, SGI257727, SGI257728, SGI257729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57697[] = {SGI257730, SGI0, SGI257731, SGI0, SGI257732, SGI257733, SGI257734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57698[] = {SGI257735, SGI0, SGI257736, SGI0, SGI257737, SGI257738, SGI257739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57699[] = {SGI257740, SGI0, SGI257741, SGI0, SGI257742, SGI257743, SGI257744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57700[] = {SGI257745, SGI0, SGI257746, SGI0, SGI257747, SGI257748, SGI257749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57701[] = {SGI257750, SGI0, SGI257751, SGI0, SGI257752, SGI257753, SGI257754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57702[] = {SGI257755, SGI0, SGI257756, SGI0, SGI257757, SGI257758, SGI257759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57703[] = {SGI257760, SGI0, SGI257761, SGI0, SGI257762, SGI257763, SGI257764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57704[] = {SGI257765, SGI0, SGI257766, SGI0, SGI257767, SGI257768, SGI257769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57705[] = {SGI257770, SGI0, SGI257771, SGI0, SGI257772, SGI257773, SGI257774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57706[] = {SGI257775, SGI0, SGI257776, SGI0, SGI257777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57707[] = {SGI257778, SGI0, SGI257779, SGI0, SGI257780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57708[] = {SGI257781, SGI0, SGI257782, SGI0, SGI257783, SGI257784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57709[] = {SGI257785, SGI0, SGI257786, SGI0, SGI257787, SGI257788, SGI257787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57710[] = {SGI257789, SGI257790, SGI257791, SGI0, SGI257792, SGI257793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57711[] = {SGI257794, SGI0, SGI257795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57712[] = {SGI257796, SGI0, SGI257797, SGI0, SGI257798, SGI257799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57713[] = {SGI257800, SGI0, SGI0, SGI0, SGI257801, SGI257802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57714[] = {SGI257803, SGI0, SGI257804, SGI0, SGI257805, SGI257806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57715[] = {SGI257807, SGI0, SGI257808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57716[] = {SGI257809, SGI0, SGI257810, SGI0, SGI257811, SGI257812, SGI257813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57717[] = {SGI257814, SGI0, SGI257815, SGI0, SGI257816, SGI257817, SGI257818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57718[] = {SGI257819, SGI0, SGI257820, SGI0, SGI257821, SGI257822, SGI257823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57719[] = {SGI257824, SGI0, SGI257825, SGI0, SGI257826, SGI257827, SGI257828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57720[] = {SGI257829, SGI0, SGI257830, SGI0, SGI257831, SGI257832, SGI257833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57721[] = {SGI257834, NULL};
const Segment * const SG57722[] = {SGI257835, SGI0, SGI257836, SGI0, SGI257837, SGI257838, SGI257839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57723[] = {SGI257840, SGI0, SGI257841, SGI0, SGI257842, SGI257843, SGI257844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57724[] = {SGI257845, SGI0, SGI257846, SGI0, SGI257847, SGI257848, SGI257849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57725[] = {SGI257850, SGI0, SGI257851, SGI0, SGI257852, SGI257853, SGI257854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57726[] = {SGI257855, SGI0, SGI257856, SGI0, SGI257857, SGI257858, SGI257859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57727[] = {SGI257860, SGI0, SGI257861, SGI0, SGI257862, SGI257863, SGI257864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57728[] = {SGI257865, SGI0, SGI257866, SGI0, SGI257867, SGI257868, SGI257869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57729[] = {SGI257870, SGI0, SGI257871, SGI0, SGI257872, SGI257873, SGI257874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57730[] = {SGI257875, SGI0, SGI257876, SGI0, SGI257877, SGI257878, SGI257879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57731[] = {SGI257880, SGI0, SGI257881, SGI0, SGI257882, SGI257883, SGI257884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57732[] = {SGI257885, SGI0, SGI257886, SGI0, SGI257887, SGI257888, SGI257889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57733[] = {SGI257890, SGI0, SGI257891, SGI0, SGI257892, SGI257893, SGI257894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57734[] = {SGI257895, SGI257896, SGI257897, SGI0, SGI257898, SGI257899, SGI257900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57735[] = {SGI257901, SGI257902, SGI257903, SGI0, SGI257904, SGI257905, SGI257906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57736[] = {SGI257907, SGI257908, SGI257909, SGI0, SGI257910, SGI257911, SGI257912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57737[] = {SGI257913, SGI257914, SGI257915, SGI0, SGI257916, SGI257917, SGI257918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57738[] = {SGI257919, SGI257920, SGI257921, SGI0, SGI257922, SGI257923, SGI257924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57739[] = {SGI257925, SGI257926, SGI257927, SGI0, SGI257928, SGI257929, SGI257930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57740[] = {SGI257931, SGI0, SGI257932, SGI0, SGI257933, SGI257934, SGI0, SGI0, SGI0, SGI257935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57741[] = {SGI257936, SGI0, SGI257937, SGI0, SGI257938, SGI257939, SGI0, SGI0, SGI0, SGI257940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57742[] = {SGI257941, SGI0, SGI257942, SGI0, SGI257943, SGI257944, SGI0, SGI0, SGI0, SGI257945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57743[] = {SGI257946, SGI0, SGI257947, SGI0, SGI257948, SGI257949, SGI0, SGI0, SGI0, SGI257950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57744[] = {SGI257951, SGI0, SGI257952, SGI0, SGI257953, SGI257954, SGI0, SGI0, SGI0, SGI257955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57745[] = {SGI257956, SGI0, SGI257957, SGI0, SGI257958, SGI257959, SGI0, SGI0, SGI0, SGI257960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57746[] = {SGI257961, SGI257962, SGI257963, SGI0, SGI257964, SGI257965, SGI257966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57747[] = {SGI257967, SGI257968, SGI257969, SGI0, SGI257970, SGI257971, SGI257972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57748[] = {SGI257973, SGI257974, SGI257975, SGI0, SGI257976, SGI257977, SGI257978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57749[] = {SGI257979, SGI257980, SGI257981, SGI0, SGI257982, SGI257983, SGI257984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57750[] = {SGI257985, SGI257986, NULL};
const Segment * const SG57751[] = {SGI257987, SGI257988, NULL};
const Segment * const SG57752[] = {SGI257989, SGI257990, NULL};
const Segment * const SG57753[] = {SGI257991, SGI257992, NULL};
const Segment * const SG57754[] = {SGI257993, SGI257994, NULL};
const Segment * const SG57755[] = {SGI257995, SGI257996, NULL};
const Segment * const SG57756[] = {SGI257997, SGI257998, NULL};
const Segment * const SG57757[] = {SGI257999, SGI258000, NULL};
const Segment * const SG57758[] = {SGI258001, SGI258002, NULL};
const Segment * const SG57759[] = {SGI258003, SGI258004, NULL};
const Segment * const SG57760[] = {SGI258005, SGI258006, NULL};
const Segment * const SG57761[] = {SGI258007, SGI258008, NULL};
const Segment * const SG57762[] = {SGI258009, SGI258009, NULL};
const Segment * const SG57763[] = {SGI258010, SGI258011, NULL};
const Segment * const SG57764[] = {SGI258012, SGI258013, NULL};
const Segment * const SG57765[] = {SGI258014, SGI258015, NULL};
const Segment * const SG57766[] = {SGI258016, SGI258017, NULL};
const Segment * const SG57767[] = {SGI258018, SGI258019, NULL};
const Segment * const SG57768[] = {SGI258020, SGI258021, NULL};
const Segment * const SG57769[] = {SGI258022, SGI258023, NULL};
const Segment * const SG57770[] = {SGI258024, SGI258025, NULL};
const Segment * const SG57771[] = {SGI258026, SGI258027, NULL};
const Segment * const SG57772[] = {SGI258028, SGI258029, NULL};
const Segment * const SG57773[] = {SGI258030, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57774[] = {SGI258031, SGI0, SGI258032, SGI0, SGI258033, SGI258034, SGI258035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57775[] = {SGI258036, SGI0, SGI258037, SGI0, SGI258038, SGI258039, SGI258040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57776[] = {SGI258041, SGI0, SGI258042, SGI0, SGI258043, SGI258044, SGI258045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57777[] = {SGI258046, SGI0, SGI258047, SGI0, SGI258048, SGI258049, SGI258050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57778[] = {SGI258051, SGI0, SGI258052, SGI0, SGI258053, SGI258054, SGI258055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57779[] = {SGI258056, SGI258057, SGI258058, SGI0, SGI258059, SGI258060, SGI258061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57780[] = {SGI258062, SGI258063, SGI258064, SGI0, SGI258065, SGI258066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57781[] = {SGI258067, SGI258068, SGI258069, SGI0, SGI258070, SGI258071, SGI258072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57782[] = {SGI258073, SGI258074, SGI258075, SGI0, SGI258076, SGI258077, SGI258078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57783[] = {SGI258079, SGI258080, SGI258081, SGI0, SGI258082, SGI258083, SGI258084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57784[] = {SGI258085, SGI258086, SGI258087, SGI0, SGI258088, SGI258089, SGI258090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57785[] = {SGI258091, SGI258092, SGI258093, SGI0, SGI258094, SGI258095, SGI258096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57786[] = {SGI258097, SGI258098, SGI258099, SGI0, SGI258100, SGI258101, SGI258102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57787[] = {SGI258103, SGI258104, SGI258105, SGI0, SGI258106, SGI258107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57788[] = {SGI258108, SGI258109, SGI258110, SGI0, SGI258111, SGI258112, SGI258113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57789[] = {SGI258114, SGI258115, SGI258116, SGI0, SGI258117, SGI258118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57790[] = {SGI258119, SGI258120, SGI258121, SGI0, SGI258122, SGI258123, SGI258124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57791[] = {SGI258125, SGI258126, SGI258127, SGI0, SGI258128, SGI258129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57792[] = {SGI258130, SGI258131, SGI258132, SGI0, SGI258133, SGI258134, SGI258135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57793[] = {SGI258136, SGI258137, SGI258138, SGI0, SGI258139, SGI258140, SGI258141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57794[] = {SGI258142, SGI258143, SGI258144, SGI0, SGI258145, SGI258146, SGI258147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57795[] = {SGI258148, SGI258149, SGI258150, SGI0, SGI258151, SGI258152, SGI258153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57796[] = {SGI258154, SGI258155, NULL};
const Segment * const SG57797[] = {SGI258156, SGI258157, NULL};
const Segment * const SG57798[] = {SGI258158, SGI258159, NULL};
const Segment * const SG57799[] = {SGI258160, SGI258161, NULL};
const Segment * const SG57800[] = {SGI258162, SGI258163, NULL};
const Segment * const SG57801[] = {SGI258164, SGI258165, NULL};
const Segment * const SG57802[] = {SGI258166, SGI258167, NULL};
const Segment * const SG57803[] = {SGI258168, SGI258169, NULL};
const Segment * const SG57804[] = {SGI258170, SGI258171, SGI0, SGI0, NULL};
const Segment * const SG57805[] = {SGI258172, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57806[] = {SGI258173, SGI258174, NULL};
const Segment * const SG57807[] = {SGI258175, SGI258176, NULL};
const Segment * const SG57808[] = {SGI258177, SGI258178, NULL};
const Segment * const SG57809[] = {SGI258179, SGI258180, NULL};
const Segment * const SG57810[] = {SGI258181, SGI258182, NULL};
const Segment * const SG57811[] = {SGI258183, SGI258184, NULL};
const Segment * const SG57812[] = {SGI258185, SGI258186, NULL};
const Segment * const SG57813[] = {SGI258187, SGI258188, NULL};
const Segment * const SG57814[] = {SGI258189, SGI258190, NULL};
const Segment * const SG57815[] = {SGI258191, SGI258192, NULL};
const Segment * const SG57816[] = {SGI258193, SGI258194, NULL};
const Segment * const SG57817[] = {SGI258195, SGI258196, NULL};
const Segment * const SG57818[] = {SGI258197, SGI258198, NULL};
const Segment * const SG57819[] = {SGI258199, SGI258200, NULL};
const Segment * const SG57820[] = {SGI258201, SGI258202, NULL};
const Segment * const SG57821[] = {SGI258203, SGI258204, NULL};
const Segment * const SG57822[] = {SGI258205, SGI258206, NULL};
const Segment * const SG57823[] = {SGI258207, SGI258208, NULL};
const Segment * const SG57824[] = {SGI258209, SGI258210, NULL};
const Segment * const SG57825[] = {SGI258211, SGI258212, NULL};
const Segment * const SG57826[] = {SGI258213, SGI258214, NULL};
const Segment * const SG57827[] = {SGI258215, SGI258216, NULL};
const Segment * const SG57828[] = {SGI258217, SGI258218, NULL};
const Segment * const SG57829[] = {SGI258219, SGI258220, NULL};
const Segment * const SG57830[] = {SGI258221, SGI258222, NULL};
const Segment * const SG57831[] = {SGI258223, SGI258224, NULL};
const Segment * const SG57832[] = {SGI258225, SGI258226, NULL};
const Segment * const SG57833[] = {SGI258227, SGI258228, NULL};
const Segment * const SG57834[] = {SGI258229, SGI258230, NULL};
const Segment * const SG57835[] = {SGI258231, SGI258232, NULL};
const Segment * const SG57836[] = {SGI258233, SGI258234, NULL};
const Segment * const SG57837[] = {SGI258235, SGI258236, NULL};
const Segment * const SG57838[] = {SGI258237, SGI258238, NULL};
const Segment * const SG57839[] = {SGI258239, SGI258240, NULL};
const Segment * const SG57840[] = {SGI258241, SGI258242, NULL};
const Segment * const SG57841[] = {SGI258243, SGI258244, NULL};
const Segment * const SG57842[] = {SGI258245, SGI258246, NULL};
const Segment * const SG57843[] = {SGI258247, SGI258248, NULL};
const Segment * const SG57844[] = {SGI258249, SGI258250, NULL};
const Segment * const SG57845[] = {SGI258251, SGI258252, NULL};
const Segment * const SG57846[] = {SGI258253, SGI258254, NULL};
const Segment * const SG57847[] = {SGI258255, SGI258256, NULL};
const Segment * const SG57848[] = {SGI258257, SGI258258, NULL};
const Segment * const SG57849[] = {SGI258259, SGI258260, NULL};
const Segment * const SG57850[] = {SGI258261, SGI258262, NULL};
const Segment * const SG57851[] = {SGI258263, SGI258264, NULL};
const Segment * const SG57852[] = {SGI258265, SGI258266, NULL};
const Segment * const SG57853[] = {SGI258267, SGI258268, NULL};
const Segment * const SG57854[] = {SGI258269, SGI258270, NULL};
const Segment * const SG57855[] = {SGI258271, SGI258272, NULL};
const Segment * const SG57856[] = {SGI258273, SGI258274, NULL};
const Segment * const SG57857[] = {SGI258275, SGI258276, NULL};
const Segment * const SG57858[] = {SGI258277, SGI258278, NULL};
const Segment * const SG57859[] = {SGI258279, SGI258280, NULL};
const Segment * const SG57860[] = {SGI258281, SGI258282, NULL};
const Segment * const SG57861[] = {SGI258283, SGI258284, NULL};
const Segment * const SG57862[] = {SGI258285, SGI258286, NULL};
const Segment * const SG57863[] = {SGI258287, SGI258288, NULL};
const Segment * const SG57864[] = {SGI258289, SGI258290, NULL};
const Segment * const SG57865[] = {SGI258291, SGI258292, NULL};
const Segment * const SG57866[] = {SGI258293, SGI258294, NULL};
const Segment * const SG57867[] = {SGI258295, SGI258296, NULL};
const Segment * const SG57868[] = {SGI258297, SGI258298, NULL};
const Segment * const SG57869[] = {SGI258299, SGI258300, NULL};
const Segment * const SG57870[] = {SGI258301, SGI258302, NULL};
const Segment * const SG57871[] = {SGI258303, SGI258304, NULL};
const Segment * const SG57872[] = {SGI258305, SGI258306, NULL};
const Segment * const SG57873[] = {SGI258307, SGI258308, NULL};
const Segment * const SG57874[] = {SGI258309, SGI258310, NULL};
const Segment * const SG57875[] = {SGI258311, SGI258312, NULL};
const Segment * const SG57876[] = {SGI258313, SGI258314, NULL};
const Segment * const SG57877[] = {SGI258315, SGI258316, NULL};
const Segment * const SG57878[] = {SGI258317, SGI258318, NULL};
const Segment * const SG57879[] = {SGI258319, SGI258320, NULL};
const Segment * const SG57880[] = {SGI258321, SGI258322, NULL};
const Segment * const SG57881[] = {SGI258323, SGI258324, NULL};
const Segment * const SG57882[] = {SGI258325, SGI258326, NULL};
const Segment * const SG57883[] = {SGI258327, SGI258328, NULL};
const Segment * const SG57884[] = {SGI258329, SGI258330, NULL};
const Segment * const SG57885[] = {SGI258331, SGI258332, NULL};
const Segment * const SG57886[] = {SGI258333, SGI258334, NULL};
const Segment * const SG57887[] = {SGI258335, SGI258336, NULL};
const Segment * const SG57888[] = {SGI258337, SGI258338, NULL};
const Segment * const SG57889[] = {SGI258339, SGI258340, NULL};
const Segment * const SG57890[] = {SGI258341, SGI258342, NULL};
const Segment * const SG57891[] = {SGI258343, SGI258344, NULL};
const Segment * const SG57892[] = {SGI258345, SGI258346, NULL};
const Segment * const SG57893[] = {SGI258347, SGI258348, NULL};
const Segment * const SG57894[] = {SGI258349, SGI258350, NULL};
const Segment * const SG57895[] = {SGI258351, SGI258352, NULL};
const Segment * const SG57896[] = {SGI258353, SGI258354, NULL};
const Segment * const SG57897[] = {SGI258355, SGI258356, NULL};
const Segment * const SG57898[] = {SGI258357, SGI258358, NULL};
const Segment * const SG57899[] = {SGI258359, SGI258360, NULL};
const Segment * const SG57900[] = {SGI258361, SGI258362, NULL};
const Segment * const SG57901[] = {SGI258363, SGI258364, NULL};
const Segment * const SG57902[] = {SGI258365, SGI258366, NULL};
const Segment * const SG57903[] = {SGI258367, SGI258368, NULL};
const Segment * const SG57904[] = {SGI258369, SGI258370, NULL};
const Segment * const SG57905[] = {SGI258371, SGI258372, NULL};
const Segment * const SG57906[] = {SGI258373, SGI258374, NULL};
const Segment * const SG57907[] = {SGI258375, SGI258376, NULL};
const Segment * const SG57908[] = {SGI258377, SGI258378, NULL};
const Segment * const SG57909[] = {SGI258379, SGI0, SGI258380, SGI0, SGI258381, SGI258382, SGI258383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57910[] = {SGI258384, SGI0, SGI258385, SGI0, SGI258386, SGI258387, SGI258388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57911[] = {SGI258389, SGI0, SGI258390, SGI0, SGI258391, SGI258392, SGI258393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57912[] = {SGI258394, SGI0, SGI258395, SGI0, SGI258396, SGI258397, SGI258398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57913[] = {SGI258399, SGI0, SGI258400, SGI0, SGI258401, SGI258402, SGI258403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57914[] = {SGI258404, SGI0, SGI258405, SGI0, SGI258406, SGI258407, SGI258408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57915[] = {SGI258409, SGI0, SGI258410, SGI0, SGI258411, SGI258412, SGI258413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57916[] = {SGI258414, SGI0, SGI258415, SGI0, SGI258416, SGI258417, SGI258418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57917[] = {SGI258419, SGI0, SGI258420, SGI0, SGI258421, SGI258422, SGI258423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57918[] = {SGI258424, SGI0, SGI258425, SGI0, SGI258426, SGI258427, SGI258428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57919[] = {SGI258429, SGI258430, SGI258431, SGI0, SGI258432, SGI258433, SGI258434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57920[] = {SGI258435, SGI258436, SGI258437, SGI0, SGI258438, SGI258439, SGI258440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57921[] = {SGI258441, SGI258442, SGI258443, SGI0, SGI258444, SGI258445, SGI258446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57922[] = {SGI258447, SGI258448, SGI258449, SGI0, SGI258450, SGI258451, SGI258452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57923[] = {SGI258453, SGI258454, SGI258455, SGI0, SGI258456, SGI258457, SGI258458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57924[] = {SGI258459, SGI258460, SGI258461, SGI0, SGI258462, SGI258463, SGI258464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57925[] = {SGI258465, SGI258466, SGI258467, SGI0, SGI258468, SGI258469, SGI258470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57926[] = {SGI258471, SGI258472, SGI258473, SGI0, SGI258474, SGI258475, SGI258476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57927[] = {SGI258477, SGI258478, SGI0, SGI0, NULL};
const Segment * const SG57928[] = {SGI258479, SGI258480, NULL};
const Segment * const SG57929[] = {SGI258481, SGI258482, NULL};
const Segment * const SG57930[] = {SGI258483, SGI258483, NULL};
const Segment * const SG57931[] = {SGI258484, SGI258485, NULL};
const Segment * const SG57932[] = {SGI258486, SGI258487, NULL};
const Segment * const SG57933[] = {SGI258488, SGI258488, NULL};
const Segment * const SG57934[] = {SGI258489, SGI258490, NULL};
const Segment * const SG57935[] = {SGI258491, SGI258492, NULL};
const Segment * const SG57936[] = {SGI258493, SGI258494, NULL};
const Segment * const SG57937[] = {SGI258495, SGI258496, NULL};
const Segment * const SG57938[] = {SGI258497, SGI258497, NULL};
const Segment * const SG57939[] = {SGI258498, SGI258499, NULL};
const Segment * const SG57940[] = {SGI258500, SGI258501, NULL};
const Segment * const SG57941[] = {SGI258502, SGI258503, NULL};
const Segment * const SG57942[] = {SGI258504, SGI258505, NULL};
const Segment * const SG57943[] = {SGI258506, SGI0, SGI258507, SGI0, SGI258508, SGI258509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57944[] = {SGI258510, SGI0, SGI258511, SGI0, SGI258512, SGI258513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57945[] = {SGI258514, SGI0, SGI258515, SGI0, SGI258516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57946[] = {SGI258517, SGI0, SGI258518, SGI0, SGI258519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57947[] = {SGI258520, SGI0, SGI258521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI258522, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57948[] = {SGI258523, SGI0, SGI258524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57949[] = {SGI258525, SGI0, SGI258526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57950[] = {SGI258527, SGI0, SGI258528, SGI0, SGI258529, SGI258530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57951[] = {SGI258531, SGI0, SGI258532, SGI0, SGI258533, SGI258534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57952[] = {SGI258535, SGI0, SGI258536, SGI0, SGI258537, SGI258538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57953[] = {SGI258539, SGI0, SGI258540, SGI0, SGI258541, SGI258542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57954[] = {SGI258543, SGI258544, SGI258545, SGI0, SGI258546, SGI258547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57955[] = {SGI258548, SGI0, SGI258549, SGI0, SGI258550, SGI258551, SGI258550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57956[] = {SGI258552, SGI0, SGI258553, SGI0, SGI258554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57957[] = {SGI258555, SGI0, SGI258556, SGI0, SGI258557, SGI258558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57958[] = {SGI258559, SGI0, SGI258560, SGI0, SGI0, SGI258561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57959[] = {SGI258562, SGI0, SGI258563, SGI0, SGI258564, SGI258565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57960[] = {SGI258566, SGI0, SGI258567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57961[] = {SGI258568, SGI0, SGI258569, SGI0, SGI258570, SGI258571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57962[] = {SGI258572, SGI0, SGI0, SGI0, SGI258573, SGI258574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57963[] = {SGI258575, SGI0, SGI258576, SGI0, SGI258577, SGI258578, SGI258579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57964[] = {SGI258580, SGI0, SGI258581, SGI0, SGI258582, SGI258583, SGI258584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57965[] = {SGI258585, SGI0, SGI258586, SGI0, SGI258587, SGI258588, SGI258589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57966[] = {SGI258590, SGI0, SGI258591, SGI0, SGI258592, SGI258593, SGI258594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57967[] = {SGI258595, SGI0, SGI258596, SGI0, SGI258597, SGI258598, SGI258599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57968[] = {SGI258600, SGI0, SGI258601, SGI0, SGI258602, SGI258603, SGI258604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57969[] = {SGI258605, SGI0, SGI258606, SGI0, SGI258607, SGI258608, SGI258609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57970[] = {SGI258610, SGI0, SGI258611, SGI0, SGI258612, SGI258613, SGI258614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57971[] = {SGI258615, SGI0, SGI258616, SGI0, SGI258617, SGI258618, SGI258619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57972[] = {SGI258620, SGI0, SGI258621, SGI0, SGI258622, SGI258623, SGI258624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57973[] = {SGI258625, SGI0, SGI258626, SGI0, SGI258627, SGI258628, SGI258629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57974[] = {SGI258630, SGI0, SGI258631, SGI0, SGI258632, SGI258633, SGI258634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57975[] = {SGI258635, SGI0, SGI258636, SGI0, SGI258637, SGI258638, SGI258639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57976[] = {SGI258640, SGI0, SGI258641, SGI0, SGI258642, SGI258643, SGI258644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57977[] = {SGI258645, SGI0, SGI258646, SGI0, SGI258647, SGI258648, SGI258649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57978[] = {SGI258650, SGI0, SGI258651, SGI0, SGI258652, SGI258653, SGI258654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57979[] = {SGI258655, SGI258656, SGI0, SGI0, NULL};
const Segment * const SG57980[] = {SGI258657, SGI0, SGI258658, SGI0, SGI258659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57981[] = {SGI258660, SGI0, SGI258661, SGI0, SGI258662, SGI258663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57982[] = {SGI258664, SGI0, SGI258665, SGI0, SGI258666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57983[] = {SGI258667, SGI0, SGI258668, SGI0, SGI258669, SGI258670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57984[] = {SGI258671, SGI0, SGI258672, SGI0, SGI258673, SGI258674, SGI258673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57985[] = {SGI258675, SGI0, SGI258676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57986[] = {SGI258677, SGI0, SGI258678, SGI0, SGI258679, SGI258680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57987[] = {SGI258681, SGI0, SGI258682, SGI0, SGI258683, SGI258684, SGI258685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57988[] = {SGI258686, SGI0, SGI258687, SGI0, SGI258688, SGI258689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57989[] = {SGI258690, SGI0, SGI258691, SGI0, SGI258692, SGI258693, SGI258694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI258695, SGI0, NULL};
const Segment * const SG57990[] = {SGI258696, SGI0, SGI258697, SGI0, SGI258698, SGI258699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57991[] = {SGI258700, SGI0, SGI258701, SGI0, SGI258702, SGI258703, SGI258704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI258705, SGI0, NULL};
const Segment * const SG57992[] = {SGI258706, SGI0, SGI258707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57993[] = {SGI258708, SGI0, SGI258709, SGI0, SGI258710, SGI258711, SGI258712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57994[] = {SGI258713, SGI0, SGI258714, SGI0, SGI258715, SGI258716, SGI258717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57995[] = {SGI258718, SGI0, SGI258719, SGI0, SGI258720, SGI258721, SGI258722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57996[] = {SGI258723, SGI0, SGI258724, SGI0, SGI258725, SGI258726, SGI258727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57997[] = {SGI258728, SGI0, SGI258729, SGI0, SGI258730, SGI258731, SGI258732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57998[] = {SGI258733, SGI0, SGI258734, SGI0, SGI258735, SGI258736, SGI258737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG57999[] = {SGI258738, SGI0, SGI258739, SGI0, SGI258740, SGI258741, SGI258742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58000[] = {SGI258743, SGI0, SGI258744, SGI0, SGI258745, SGI258746, SGI258747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58001[] = {SGI258748, SGI0, SGI258749, SGI0, SGI258750, SGI258751, SGI258752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58002[] = {SGI258753, SGI0, SGI258754, SGI0, SGI258755, SGI258756, SGI258757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58003[] = {SGI258758, SGI0, SGI258759, SGI0, SGI258760, SGI258761, SGI258762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58004[] = {SGI258763, SGI0, SGI258764, SGI0, SGI258765, SGI258766, SGI258767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58005[] = {SGI258768, SGI0, SGI258769, SGI0, SGI258770, SGI258771, SGI258772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58006[] = {SGI258773, SGI0, SGI258774, SGI0, SGI258775, SGI258776, SGI258777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58007[] = {SGI258778, SGI0, SGI258779, SGI0, SGI258780, SGI258781, SGI258782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58008[] = {SGI258783, SGI0, SGI258784, SGI0, SGI258785, SGI258786, SGI258787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58009[] = {SGI258788, SGI0, SGI258789, SGI0, SGI258790, SGI258791, SGI258792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58010[] = {SGI258793, SGI0, SGI258794, SGI0, SGI258795, SGI258796, SGI258797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58011[] = {SGI258798, SGI0, SGI258799, SGI0, SGI258800, SGI258801, SGI258802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58012[] = {SGI258803, SGI0, SGI258804, SGI0, SGI258805, SGI258806, SGI258807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58013[] = {SGI258808, SGI0, SGI258809, SGI0, SGI258810, SGI258811, SGI258812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58014[] = {SGI258813, SGI0, SGI258814, SGI0, SGI258815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58015[] = {SGI258816, SGI0, SGI258817, SGI0, SGI258818, SGI258819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58016[] = {SGI258820, SGI0, SGI258821, SGI0, SGI258822, SGI258823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58017[] = {SGI258824, SGI0, SGI258825, SGI0, SGI258826, SGI258827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58018[] = {SGI258828, SGI0, SGI258829, SGI0, SGI258830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58019[] = {SGI258831, SGI0, SGI258832, SGI0, SGI258833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58020[] = {SGI258834, SGI0, SGI258835, SGI0, SGI258836, SGI258837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58021[] = {SGI258838, SGI0, SGI258839, SGI0, SGI258840, SGI258841, SGI258840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58022[] = {SGI258842, SGI0, SGI258843, SGI0, SGI258844, SGI258845, SGI258844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58023[] = {SGI258846, SGI0, SGI258847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58024[] = {SGI258848, SGI0, SGI258849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58025[] = {SGI258850, SGI0, SGI258851, SGI0, SGI258852, SGI258853, SGI258852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58026[] = {SGI258854, SGI0, SGI258855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58027[] = {SGI258856, SGI0, SGI258857, SGI0, SGI258858, SGI258859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58028[] = {SGI258860, SGI0, SGI258861, SGI0, SGI258862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58029[] = {SGI258863, SGI0, SGI258864, SGI0, SGI258865, SGI258866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58030[] = {SGI258867, SGI0, SGI258868, SGI0, SGI258869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58031[] = {SGI258870, SGI0, SGI258871, SGI0, SGI258872, SGI258873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58032[] = {SGI258874, SGI0, SGI258875, SGI0, SGI258876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58033[] = {SGI258877, SGI0, SGI258878, SGI0, SGI258879, SGI258880, SGI258881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58034[] = {SGI258882, SGI0, SGI258883, SGI0, SGI258884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58035[] = {SGI258885, SGI0, SGI258886, SGI0, SGI258887, SGI258888, SGI258887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58036[] = {SGI258889, SGI0, SGI258890, SGI0, SGI258891, SGI258892, SGI258891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58037[] = {SGI258893, SGI0, SGI258894, SGI0, SGI258895, SGI258896, SGI258895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58038[] = {SGI258897, SGI0, SGI258898, SGI0, SGI258899, SGI258900, SGI258901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58039[] = {SGI258902, SGI0, SGI258903, SGI0, SGI258904, SGI258905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58040[] = {SGI258906, SGI258907, SGI0, SGI0, NULL};
const Segment * const SG58041[] = {SGI258908, SGI258909, SGI0, SGI0, NULL};
const Segment * const SG58042[] = {SGI258910, SGI258911, NULL};
const Segment * const SG58043[] = {SGI258912, SGI258913, NULL};
const Segment * const SG58044[] = {SGI258914, SGI0, SGI258915, SGI0, SGI258916, SGI258917, SGI258918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58045[] = {SGI258919, SGI0, SGI258920, SGI0, SGI258921, SGI258922, SGI258923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58046[] = {SGI258924, SGI0, SGI258925, SGI0, SGI258926, SGI258927, SGI258928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58047[] = {SGI258929, SGI0, SGI258930, SGI0, SGI258931, SGI258932, SGI258933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58048[] = {SGI258934, SGI0, SGI258935, SGI0, SGI258936, SGI258937, SGI258938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58049[] = {SGI258939, SGI0, SGI258940, SGI0, SGI258941, SGI258942, SGI258943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58050[] = {SGI258944, SGI0, SGI258945, SGI0, SGI258946, SGI258947, SGI258948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58051[] = {SGI258949, SGI0, SGI258950, SGI0, SGI258951, SGI258952, SGI258953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58052[] = {SGI258954, SGI0, SGI258955, SGI0, SGI258956, SGI258957, SGI258958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58053[] = {SGI258959, SGI0, SGI258960, SGI0, SGI258961, SGI258962, SGI258963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58054[] = {SGI258964, SGI0, SGI258965, SGI0, SGI258966, SGI258967, SGI258968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58055[] = {SGI258969, SGI0, SGI258970, SGI0, SGI258971, SGI258972, SGI258973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58056[] = {SGI258974, SGI0, SGI258975, SGI0, SGI258976, SGI258977, SGI258978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58057[] = {SGI258979, SGI0, SGI258980, SGI0, SGI258981, SGI258982, SGI258983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58058[] = {SGI258984, SGI0, SGI258985, SGI0, SGI258986, SGI258987, SGI258988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58059[] = {SGI258989, SGI0, SGI258990, SGI0, SGI258991, SGI258992, SGI258993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58060[] = {SGI258994, SGI0, SGI258995, SGI0, SGI258996, SGI258997, SGI0, SGI0, SGI0, SGI258998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58061[] = {SGI258999, SGI0, SGI259000, SGI0, SGI259001, SGI259002, SGI0, SGI0, SGI0, SGI259003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58062[] = {SGI259004, SGI0, SGI259005, SGI0, SGI259006, SGI259007, SGI0, SGI0, SGI0, SGI259008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58063[] = {SGI259009, SGI0, SGI259010, SGI0, SGI259011, SGI259012, SGI0, SGI0, SGI0, SGI259013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58064[] = {SGI259014, SGI0, SGI259015, SGI0, SGI259016, SGI259017, SGI259018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58065[] = {SGI259019, SGI0, SGI259020, SGI0, SGI259021, SGI259022, SGI259023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58066[] = {SGI259024, SGI0, SGI259025, SGI0, SGI259026, SGI259027, SGI259028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58067[] = {SGI259029, SGI259030, SGI0, SGI0, NULL};
const Segment * const SG58068[] = {SGI259031, SGI0, SGI259032, SGI0, SGI259033, SGI259034, SGI259035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58069[] = {SGI259036, SGI0, SGI259037, SGI0, SGI259038, SGI259039, SGI259040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58070[] = {SGI259041, SGI0, SGI259042, SGI0, SGI259043, SGI259044, SGI259045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58071[] = {SGI259046, SGI0, SGI259047, SGI0, SGI259048, SGI259049, SGI259050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58072[] = {SGI259051, SGI0, SGI259052, SGI0, SGI259053, SGI259054, SGI259055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58073[] = {SGI259056, SGI0, SGI259057, SGI0, SGI259058, SGI259059, SGI259060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58074[] = {SGI259061, SGI0, SGI259062, SGI0, SGI259063, SGI259064, SGI259065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58075[] = {SGI259066, SGI0, SGI259067, SGI0, SGI259068, SGI259069, SGI259070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58076[] = {SGI259071, SGI0, SGI259072, SGI0, SGI259073, SGI259074, SGI259075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58077[] = {SGI259076, SGI0, SGI259077, SGI0, SGI259078, SGI259079, SGI259080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58078[] = {SGI259081, SGI0, SGI259082, SGI0, SGI259083, SGI259084, SGI259085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58079[] = {SGI259086, SGI0, SGI259087, SGI0, SGI259088, SGI259089, SGI259090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58080[] = {SGI259091, SGI0, SGI259092, SGI0, SGI259093, SGI259094, SGI259095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58081[] = {SGI259096, SGI0, SGI259097, SGI0, SGI259098, SGI259099, SGI259100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58082[] = {SGI259101, SGI0, SGI259102, SGI0, SGI259103, SGI259104, SGI259105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58083[] = {SGI259106, SGI0, SGI259107, SGI0, SGI259108, SGI259109, SGI259110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58084[] = {SGI259111, SGI0, SGI259112, SGI0, SGI259113, SGI259114, SGI259115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58085[] = {SGI259116, SGI0, SGI259117, SGI0, SGI259118, SGI259119, SGI259120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58086[] = {SGI259121, SGI259122, SGI0, SGI0, NULL};
const Segment * const SG58087[] = {SGI259123, SGI259124, SGI0, SGI0, NULL};
const Segment * const SG58088[] = {SGI259125, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58089[] = {SGI259126, SGI0, SGI259127, SGI0, SGI259128, SGI259129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58090[] = {SGI259130, SGI0, SGI259131, SGI0, SGI259132, SGI259133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58091[] = {SGI259134, SGI0, SGI259135, SGI0, SGI259136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58092[] = {SGI259137, SGI0, SGI259138, SGI0, SGI259139, SGI259140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58093[] = {SGI259141, SGI0, SGI259142, SGI0, SGI259143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58094[] = {SGI259144, SGI0, SGI259145, SGI0, SGI259146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58095[] = {SGI259147, SGI0, SGI259148, SGI0, SGI259149, SGI259150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58096[] = {SGI259151, SGI0, SGI259152, SGI0, SGI259153, SGI259154, SGI259155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58097[] = {SGI259156, SGI0, SGI259157, SGI0, SGI259158, SGI259159, SGI259158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58098[] = {SGI259160, SGI0, SGI259161, SGI0, SGI259162, SGI259163, SGI259162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58099[] = {SGI259164, SGI0, SGI259165, SGI0, SGI259166, SGI259167, SGI259166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58100[] = {SGI259168, SGI0, SGI259169, SGI0, SGI259170, SGI259171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58101[] = {SGI259172, SGI0, SGI259173, SGI0, SGI259174, SGI259175, SGI259176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58102[] = {SGI259177, SGI0, SGI259178, SGI0, SGI259179, SGI259180, SGI259181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58103[] = {SGI259182, SGI0, SGI259183, SGI0, SGI259184, SGI259185, SGI259186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58104[] = {SGI259187, SGI0, SGI259188, SGI0, SGI259189, SGI259190, SGI259191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58105[] = {SGI259192, SGI0, SGI259193, SGI0, SGI259194, SGI259195, SGI259196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58106[] = {SGI259197, SGI0, SGI259198, SGI0, SGI259199, SGI259200, SGI259201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58107[] = {SGI259202, SGI0, SGI259203, SGI0, SGI259204, SGI259205, SGI259206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58108[] = {SGI259207, SGI0, SGI259208, SGI0, SGI259209, SGI259210, SGI259211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58109[] = {SGI259212, SGI0, SGI259213, SGI0, SGI259214, SGI259215, SGI259216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58110[] = {SGI259217, SGI0, SGI259218, SGI0, SGI259219, SGI259220, SGI259221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58111[] = {SGI259222, SGI0, SGI259223, SGI0, SGI259224, SGI259225, SGI259226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58112[] = {SGI259227, SGI0, SGI259228, SGI0, SGI259229, SGI259230, SGI259231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58113[] = {SGI259232, SGI0, SGI259233, SGI0, SGI259234, SGI259235, SGI259236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58114[] = {SGI259237, SGI0, SGI259238, SGI0, SGI259239, SGI259240, SGI259241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58115[] = {SGI259242, SGI0, SGI0, SGI0, SGI259243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58116[] = {SGI259244, SGI0, SGI0, SGI0, SGI259245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58117[] = {SGI259246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58118[] = {SGI259247, SGI0, SGI259248, SGI0, SGI259249, SGI259250, SGI259251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58119[] = {SGI259252, SGI0, SGI259253, SGI0, SGI259254, SGI259255, SGI259256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58120[] = {SGI259257, SGI0, SGI259258, SGI0, SGI259259, SGI259260, SGI259261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58121[] = {SGI259262, SGI0, SGI259263, SGI0, SGI259264, SGI259265, SGI259266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58122[] = {SGI259267, SGI259268, SGI0, SGI0, SGI259269, SGI259270, SGI259271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58123[] = {SGI259272, SGI259273, SGI0, SGI0, SGI259274, SGI259275, SGI259276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58124[] = {SGI259277, SGI259278, SGI0, SGI0, SGI259279, SGI259280, SGI259281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58125[] = {SGI259282, SGI259283, SGI0, SGI0, SGI259284, SGI259285, SGI259286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58126[] = {SGI259287, SGI259288, SGI0, SGI0, SGI259289, SGI259290, SGI259291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58127[] = {SGI259292, SGI259293, SGI0, SGI0, SGI259294, SGI259295, SGI259296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58128[] = {SGI259297, SGI259298, SGI0, SGI0, SGI259299, SGI259300, SGI259301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58129[] = {SGI259302, SGI259303, SGI0, SGI0, SGI259304, SGI259305, SGI259306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58130[] = {SGI259307, SGI259308, SGI0, SGI0, SGI259309, SGI259310, SGI259311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58131[] = {SGI259312, SGI259313, SGI0, SGI0, SGI259314, SGI259315, SGI259316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58132[] = {SGI259317, SGI259318, SGI0, SGI0, SGI259319, SGI259320, SGI259321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58133[] = {SGI259322, SGI259323, SGI0, SGI0, SGI259324, SGI259325, SGI259326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58134[] = {SGI259327, SGI259328, SGI0, SGI0, SGI259329, SGI259330, SGI259331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58135[] = {SGI259332, SGI259333, SGI0, SGI0, SGI259334, SGI259335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58136[] = {SGI259336, SGI259337, SGI0, SGI0, SGI259338, SGI259339, SGI259340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58137[] = {SGI259341, SGI259342, SGI0, SGI0, SGI259343, SGI259344, SGI259345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58138[] = {SGI259346, SGI259347, SGI0, SGI0, SGI259348, SGI259349, SGI259350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58139[] = {SGI259351, SGI259352, SGI0, SGI0, SGI259353, SGI259354, SGI259355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58140[] = {SGI259356, SGI259357, SGI0, SGI0, SGI259358, SGI259359, SGI259360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58141[] = {SGI259361, SGI259362, SGI0, SGI0, SGI259363, SGI259364, SGI259365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58142[] = {SGI259366, SGI259367, SGI0, SGI0, SGI259368, SGI259369, SGI259370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58143[] = {SGI259371, SGI0, SGI259372, SGI0, SGI259373, SGI259374, SGI259375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58144[] = {SGI259376, SGI0, SGI259377, SGI0, SGI259378, SGI259379, SGI259380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58145[] = {SGI259381, SGI0, SGI259382, SGI0, SGI259383, SGI259384, SGI0, SGI0, SGI0, SGI259385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58146[] = {SGI259386, SGI0, SGI259387, SGI0, SGI259388, SGI259389, SGI0, SGI0, SGI0, SGI259390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58147[] = {SGI259391, SGI0, SGI259392, SGI0, SGI259393, SGI259394, SGI0, SGI0, SGI0, SGI259395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58148[] = {SGI259396, SGI0, SGI259397, SGI0, SGI259398, SGI259399, SGI0, SGI0, SGI0, SGI259400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58149[] = {SGI259401, SGI0, SGI259402, SGI0, SGI259403, SGI259404, SGI0, SGI0, SGI0, SGI259405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58150[] = {SGI259406, SGI0, SGI259407, SGI0, SGI259408, SGI259409, SGI0, SGI0, SGI0, SGI259410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58151[] = {SGI259411, SGI0, SGI259412, SGI0, SGI259413, SGI259414, SGI0, SGI0, SGI0, SGI259415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58152[] = {SGI259416, SGI0, SGI259417, SGI0, SGI259418, SGI259419, SGI259420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58153[] = {SGI259421, SGI0, SGI259422, SGI0, SGI259423, SGI259424, SGI259425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58154[] = {SGI259426, SGI0, SGI259427, SGI0, SGI259428, SGI259429, SGI259430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58155[] = {SGI259431, SGI0, SGI259432, SGI0, SGI259433, SGI259434, SGI259435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58156[] = {SGI259436, SGI0, SGI259437, SGI0, SGI259438, SGI259439, SGI259440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58157[] = {SGI259441, SGI0, SGI259442, SGI0, SGI259443, SGI259444, SGI259445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58158[] = {SGI259446, SGI0, SGI259447, SGI0, SGI259448, SGI259449, SGI259450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58159[] = {SGI259451, SGI0, SGI259452, SGI0, SGI259453, SGI259454, SGI0, SGI0, SGI0, SGI259455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58160[] = {SGI259456, SGI0, SGI259457, SGI0, SGI259458, SGI259459, SGI0, SGI0, SGI0, SGI259460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58161[] = {SGI259461, SGI0, SGI259462, SGI0, SGI259463, SGI259464, SGI0, SGI0, SGI0, SGI259465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58162[] = {SGI259466, SGI0, SGI259467, SGI0, SGI259468, SGI259469, SGI0, SGI0, SGI0, SGI259470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58163[] = {SGI259471, SGI0, SGI259472, SGI0, SGI259473, SGI259474, SGI0, SGI0, SGI0, SGI259475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58164[] = {SGI259476, SGI0, SGI259477, SGI0, SGI259478, SGI259479, SGI0, SGI0, SGI0, SGI259480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58165[] = {SGI259481, SGI259482, SGI259483, SGI0, SGI259484, SGI259485, SGI259486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58166[] = {SGI259487, SGI259488, SGI259489, SGI0, SGI259490, SGI259491, SGI259492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58167[] = {SGI259493, SGI259494, SGI259495, SGI0, SGI259496, SGI259497, SGI259498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58168[] = {SGI259499, SGI259500, SGI259501, SGI0, SGI259502, SGI259503, SGI259504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58169[] = {SGI259505, SGI259506, SGI259507, SGI0, SGI259508, SGI259509, SGI259510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58170[] = {SGI259511, SGI259512, SGI259513, SGI0, SGI259514, SGI259515, SGI259516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58171[] = {SGI259517, SGI259518, SGI259519, SGI0, SGI259520, SGI259521, SGI259522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58172[] = {SGI259523, SGI259524, SGI259525, SGI0, SGI259526, SGI259527, SGI259528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58173[] = {SGI259529, SGI0, SGI259530, SGI0, SGI259531, SGI259532, SGI259533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58174[] = {SGI259534, SGI0, SGI259535, SGI0, SGI259536, SGI259537, SGI259538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58175[] = {SGI259539, SGI0, SGI259540, SGI0, SGI259541, SGI259542, SGI259543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58176[] = {SGI259544, SGI0, SGI259545, SGI0, SGI259546, SGI259547, SGI259548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58177[] = {SGI259549, SGI0, SGI259550, SGI0, SGI259551, SGI259552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58178[] = {SGI259553, SGI0, SGI259554, SGI0, SGI259555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58179[] = {SGI259556, SGI0, SGI259557, SGI0, SGI259558, SGI259559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58180[] = {SGI259560, SGI0, SGI259561, SGI0, SGI259562, SGI259563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58181[] = {SGI259564, SGI0, SGI259565, SGI0, SGI259566, SGI259567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58182[] = {SGI259568, SGI0, SGI259569, SGI0, SGI259570, SGI259571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58183[] = {SGI259572, SGI0, SGI259573, SGI0, SGI259574, SGI259575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58184[] = {SGI259576, SGI0, SGI259577, SGI0, SGI259578, SGI259579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58185[] = {SGI259580, SGI0, SGI259581, SGI0, SGI259582, SGI259583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58186[] = {SGI259584, SGI0, SGI259585, SGI0, SGI259586, SGI259587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58187[] = {SGI259588, SGI0, SGI259589, SGI0, SGI259590, SGI259591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58188[] = {SGI259592, SGI0, SGI259593, SGI0, SGI259594, SGI259595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58189[] = {SGI259596, SGI0, SGI259597, SGI0, SGI259598, SGI259599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58190[] = {SGI259600, SGI0, SGI259601, SGI0, SGI259602, SGI259603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58191[] = {SGI259604, SGI0, SGI259605, SGI0, SGI259606, SGI259607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58192[] = {SGI259608, SGI0, SGI259609, SGI0, SGI259610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58193[] = {SGI259611, SGI0, SGI259612, SGI0, SGI259613, SGI259614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58194[] = {SGI259615, SGI0, SGI259616, SGI0, SGI259617, SGI259618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58195[] = {SGI259619, SGI0, SGI259620, SGI0, SGI259621, SGI259622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58196[] = {SGI259623, SGI0, SGI259624, SGI0, SGI259625, SGI259626, SGI259627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58197[] = {SGI259628, SGI0, SGI259629, SGI0, SGI259630, SGI259631, SGI259632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58198[] = {SGI259633, SGI0, SGI259634, SGI0, SGI259635, SGI259636, SGI259637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58199[] = {SGI259638, SGI0, SGI259639, SGI0, SGI259640, SGI259641, SGI259642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58200[] = {SGI259643, SGI0, SGI259644, SGI0, SGI259645, SGI259646, SGI259647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58201[] = {SGI259648, SGI0, SGI259649, SGI0, SGI259650, SGI259651, SGI259652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58202[] = {SGI259653, SGI0, SGI259654, SGI0, SGI259655, SGI259656, SGI0, SGI0, SGI0, SGI259657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58203[] = {SGI259658, SGI0, SGI259659, SGI0, SGI259660, SGI259661, SGI0, SGI0, SGI0, SGI259662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58204[] = {SGI259663, SGI0, SGI259664, SGI0, SGI259665, SGI259666, SGI0, SGI0, SGI0, SGI259667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58205[] = {SGI259668, SGI0, SGI259669, SGI0, SGI259670, SGI259671, SGI0, SGI0, SGI0, SGI259672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58206[] = {SGI259673, SGI0, SGI259674, SGI0, SGI259675, SGI259676, SGI0, SGI0, SGI0, SGI259677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58207[] = {SGI259678, SGI0, SGI259679, SGI0, SGI259680, SGI259681, SGI0, SGI0, SGI0, SGI259682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58208[] = {SGI259683, SGI0, SGI259684, SGI0, SGI259685, SGI259686, SGI0, SGI0, SGI0, SGI259687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58209[] = {SGI259688, SGI0, SGI259689, SGI0, SGI259690, SGI259691, SGI0, SGI0, SGI0, SGI259692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58210[] = {SGI259693, SGI0, SGI259694, SGI0, SGI259695, SGI259696, SGI0, SGI0, SGI0, SGI259697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58211[] = {SGI259698, SGI0, SGI259699, SGI0, SGI259700, SGI259701, SGI0, SGI0, SGI0, SGI259702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58212[] = {SGI259703, SGI259704, SGI259705, SGI0, SGI259706, SGI259707, SGI259708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58213[] = {SGI259709, SGI259710, SGI259711, SGI0, SGI259712, SGI259713, SGI259714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58214[] = {SGI259715, SGI259716, SGI259717, SGI0, SGI259718, SGI259719, SGI259720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58215[] = {SGI259721, SGI259722, SGI259723, SGI0, SGI259724, SGI259725, SGI259726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58216[] = {SGI259727, SGI259728, SGI259729, SGI0, SGI259730, SGI259731, SGI259732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58217[] = {SGI259733, SGI259734, SGI259735, SGI0, SGI259736, SGI259737, SGI259738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58218[] = {SGI259739, SGI259739, NULL};
const Segment * const SG58219[] = {SGI259740, SGI259741, NULL};
const Segment * const SG58220[] = {SGI259742, SGI259743, NULL};
const Segment * const SG58221[] = {SGI259744, SGI259744, NULL};
const Segment * const SG58222[] = {SGI259745, SGI259746, NULL};
const Segment * const SG58223[] = {SGI259747, SGI259748, NULL};
const Segment * const SG58224[] = {SGI259749, SGI259750, NULL};
const Segment * const SG58225[] = {SGI259751, SGI259752, NULL};
const Segment * const SG58226[] = {SGI259753, SGI259754, NULL};
const Segment * const SG58227[] = {SGI259755, SGI259755, NULL};
const Segment * const SG58228[] = {SGI259756, SGI259757, NULL};
const Segment * const SG58229[] = {SGI259758, SGI259758, NULL};
const Segment * const SG58230[] = {SGI259759, SGI259759, NULL};
const Segment * const SG58231[] = {SGI259760, SGI259761, NULL};
const Segment * const SG58232[] = {SGI259762, SGI259763, NULL};
const Segment * const SG58233[] = {SGI259764, SGI259765, NULL};
const Segment * const SG58234[] = {SGI259766, SGI259767, NULL};
const Segment * const SG58235[] = {SGI259768, SGI259769, NULL};
const Segment * const SG58236[] = {SGI259770, SGI259771, NULL};
const Segment * const SG58237[] = {SGI259772, SGI259772, NULL};
const Segment * const SG58238[] = {SGI259773, SGI0, SGI259774, SGI0, SGI259775, SGI259776, SGI259777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58239[] = {SGI259778, SGI259779, NULL};
const Segment * const SG58240[] = {SGI259780, SGI259781, NULL};
const Segment * const SG58241[] = {SGI259782, SGI259783, NULL};
const Segment * const SG58242[] = {SGI259784, SGI259785, NULL};
const Segment * const SG58243[] = {SGI259786, SGI259787, NULL};
const Segment * const SG58244[] = {SGI259788, SGI259789, NULL};
const Segment * const SG58245[] = {SGI259790, SGI259791, NULL};
const Segment * const SG58246[] = {SGI259792, SGI259793, NULL};
const Segment * const SG58247[] = {SGI259794, SGI259795, NULL};
const Segment * const SG58248[] = {SGI259796, SGI259797, NULL};
const Segment * const SG58249[] = {SGI259798, SGI259799, NULL};
const Segment * const SG58250[] = {SGI259800, SGI259801, NULL};
const Segment * const SG58251[] = {SGI259802, SGI259803, NULL};
const Segment * const SG58252[] = {SGI259804, SGI259805, NULL};
const Segment * const SG58253[] = {SGI259806, SGI259807, NULL};
const Segment * const SG58254[] = {SGI259808, SGI259809, NULL};
const Segment * const SG58255[] = {SGI259810, SGI259811, NULL};
const Segment * const SG58256[] = {SGI259812, SGI0, SGI259813, SGI0, SGI259814, SGI259815, SGI259816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58257[] = {SGI259817, SGI0, SGI259818, SGI0, SGI259819, SGI259820, SGI259821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58258[] = {SGI259822, SGI259823, NULL};
const Segment * const SG58259[] = {SGI259824, SGI259825, NULL};
const Segment * const SG58260[] = {SGI259826, SGI0, SGI259827, SGI0, SGI259828, SGI259829, SGI259830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58261[] = {SGI259831, SGI0, SGI259832, SGI0, SGI259833, SGI259834, SGI259835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58262[] = {SGI259836, SGI0, SGI259837, SGI0, SGI259838, SGI259839, SGI259840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58263[] = {SGI259841, SGI0, SGI259842, SGI0, SGI259843, SGI259844, SGI259845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58264[] = {SGI259846, SGI259847, SGI0, SGI0, NULL};
const Segment * const SG58265[] = {SGI259848, SGI259849, SGI0, SGI0, NULL};
const Segment * const SG58266[] = {SGI259850, SGI0, SGI259851, SGI0, SGI259852, SGI259853, SGI0, SGI0, SGI0, SGI259854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58267[] = {SGI259855, SGI0, SGI259856, SGI0, SGI259857, SGI259858, SGI0, SGI0, SGI0, SGI259859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58268[] = {SGI259860, SGI0, SGI259861, SGI0, SGI259862, SGI259863, SGI0, SGI0, SGI0, SGI259864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58269[] = {SGI259865, SGI0, SGI259866, SGI0, SGI259867, SGI259868, SGI0, SGI0, SGI0, SGI259869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58270[] = {SGI259870, SGI0, SGI259871, SGI0, SGI259872, SGI259873, SGI0, SGI0, SGI0, SGI259874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58271[] = {SGI259875, SGI0, SGI259876, SGI0, SGI259877, SGI259878, SGI0, SGI0, SGI0, SGI259879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58272[] = {SGI259880, SGI0, SGI259881, SGI0, SGI259882, SGI259883, SGI0, SGI0, SGI0, SGI259884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58273[] = {SGI259885, SGI0, SGI259886, SGI0, SGI259887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58274[] = {SGI259888, SGI0, SGI259889, SGI0, SGI259890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58275[] = {SGI259891, SGI0, SGI259892, SGI0, SGI259893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58276[] = {SGI259894, SGI0, SGI259895, SGI0, SGI259896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58277[] = {SGI259897, SGI0, SGI259898, SGI0, SGI259899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58278[] = {SGI259900, SGI259901, SGI259902, SGI0, SGI259903, SGI259904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58279[] = {SGI259905, SGI259906, SGI259907, SGI0, SGI259908, SGI259909, SGI259910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58280[] = {SGI259911, SGI259912, SGI259913, SGI0, SGI259914, SGI259915, SGI259916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58281[] = {SGI259917, SGI259918, SGI259919, SGI0, SGI259920, SGI259921, SGI259922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58282[] = {SGI259923, SGI259924, SGI259925, SGI0, SGI259926, SGI259927, SGI259928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58283[] = {SGI259929, SGI259930, SGI259931, SGI0, SGI259932, SGI259933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58284[] = {SGI259934, SGI259935, SGI259936, SGI0, SGI259937, SGI259938, SGI259939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58285[] = {SGI259940, SGI259941, SGI259942, SGI0, SGI259943, SGI259944, SGI259945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58286[] = {SGI259946, SGI259947, SGI259948, SGI0, SGI259949, SGI259950, SGI259951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58287[] = {SGI259952, SGI259953, SGI259954, SGI0, SGI259955, SGI259956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58288[] = {SGI259957, SGI259958, SGI259959, SGI0, SGI259960, SGI259961, SGI259962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58289[] = {SGI259963, SGI259964, SGI259965, SGI0, SGI259966, SGI259967, SGI259968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58290[] = {SGI259969, SGI259970, SGI259971, SGI0, SGI259972, SGI259973, SGI259974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58291[] = {SGI259975, SGI259976, SGI259977, SGI0, SGI259978, SGI259979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58292[] = {SGI259980, SGI259981, SGI259982, SGI0, SGI259983, SGI259984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58293[] = {SGI259985, SGI259986, SGI259987, SGI0, SGI259988, SGI259989, SGI259990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58294[] = {SGI259991, SGI259992, SGI259993, SGI0, SGI259994, SGI259995, SGI259996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58295[] = {SGI259997, SGI259998, SGI259999, SGI0, SGI260000, SGI260001, SGI260002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58296[] = {SGI260003, SGI260004, SGI260005, SGI0, SGI260006, SGI260007, SGI260008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58297[] = {SGI260009, SGI260010, SGI260011, SGI0, SGI260012, SGI260013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58298[] = {SGI260014, SGI260015, SGI260016, SGI0, SGI260017, SGI260018, SGI260019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58299[] = {SGI260020, SGI260021, SGI260022, SGI0, SGI260023, SGI260024, SGI260025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58300[] = {SGI260026, SGI260027, SGI260028, SGI0, SGI260029, SGI260030, SGI260031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58301[] = {SGI260032, SGI260033, SGI260034, SGI0, SGI260035, SGI260036, SGI260037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58302[] = {SGI260038, SGI260039, SGI260040, SGI0, SGI260041, SGI260042, SGI260043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58303[] = {SGI260044, SGI260045, SGI260046, SGI0, SGI260047, SGI260048, SGI260049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58304[] = {SGI260050, SGI260051, SGI260052, SGI0, SGI260053, SGI260054, SGI260055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58305[] = {SGI260056, SGI260057, SGI260058, SGI0, SGI260059, SGI260060, SGI260061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58306[] = {SGI260062, SGI260063, SGI260064, SGI0, SGI260065, SGI260066, SGI260067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58307[] = {SGI260068, SGI0, SGI260069, SGI0, SGI260070, SGI260071, SGI260072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58308[] = {SGI260073, SGI0, SGI260074, SGI0, SGI260075, SGI260076, SGI260077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58309[] = {SGI260078, SGI0, SGI260079, SGI0, SGI260080, SGI260081, SGI260082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58310[] = {SGI260083, SGI0, SGI260084, SGI0, SGI260085, SGI260086, SGI260087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58311[] = {SGI260088, SGI0, SGI260089, SGI0, SGI260090, SGI260091, SGI260092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58312[] = {SGI260093, SGI0, SGI260094, SGI0, SGI260095, SGI260096, SGI260097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58313[] = {SGI260098, SGI0, SGI260099, SGI0, SGI260100, SGI260101, SGI260102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58314[] = {SGI260103, SGI0, SGI260104, SGI0, SGI260105, SGI260106, SGI260107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58315[] = {SGI260108, SGI0, SGI260109, SGI0, SGI260110, SGI260111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58316[] = {SGI260112, SGI0, SGI260113, SGI0, SGI260114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58317[] = {SGI260115, SGI0, SGI260116, SGI0, SGI260117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58318[] = {SGI260118, SGI0, SGI260119, SGI0, SGI260120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58319[] = {SGI260121, SGI0, SGI260122, SGI0, SGI260123, SGI260124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58320[] = {SGI260125, SGI0, SGI260126, SGI0, SGI260127, SGI260128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58321[] = {SGI260129, SGI0, SGI260130, SGI0, SGI260131, SGI260132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58322[] = {SGI260133, SGI0, SGI260134, SGI0, SGI260135, SGI260136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58323[] = {SGI260137, SGI0, SGI260138, SGI0, SGI260139, SGI260140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58324[] = {SGI260141, SGI0, SGI260142, SGI0, SGI260143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58325[] = {SGI260144, SGI0, SGI260145, SGI0, SGI260146, SGI260147, SGI260146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58326[] = {SGI260148, SGI0, SGI260149, SGI0, SGI260150, SGI260151, SGI260150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58327[] = {SGI260152, SGI0, SGI260153, SGI0, SGI260154, SGI260155, SGI260156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58328[] = {SGI260157, SGI0, SGI260158, SGI0, SGI260159, SGI260160, SGI260161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58329[] = {SGI260162, SGI0, SGI260163, SGI0, SGI260164, SGI260165, SGI260166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58330[] = {SGI260167, SGI0, SGI260168, SGI0, SGI260169, SGI260170, SGI260171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58331[] = {SGI260172, SGI0, SGI260173, SGI0, SGI260174, SGI260175, SGI260176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58332[] = {SGI260177, SGI0, SGI260178, SGI0, SGI260179, SGI260180, SGI260181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58333[] = {SGI260182, SGI0, SGI260183, SGI0, SGI260184, SGI260185, SGI260186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58334[] = {SGI260187, SGI0, SGI260188, SGI0, SGI260189, SGI260190, SGI260191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58335[] = {SGI260192, SGI0, SGI260193, SGI0, SGI260194, SGI260195, SGI260196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58336[] = {SGI260197, SGI0, SGI260198, SGI0, SGI260199, SGI260200, SGI260201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58337[] = {SGI260202, SGI0, SGI260203, SGI0, SGI260204, SGI260205, SGI260206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58338[] = {SGI260207, SGI0, SGI260208, SGI0, SGI260209, SGI260210, SGI260211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58339[] = {SGI260212, SGI0, SGI260213, SGI0, SGI260214, SGI260215, SGI260216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58340[] = {SGI260217, SGI0, SGI260218, SGI0, SGI260219, SGI260220, SGI260221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58341[] = {SGI260222, SGI0, SGI260223, SGI0, SGI260224, SGI260225, SGI260226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58342[] = {SGI260227, SGI0, SGI260228, SGI0, SGI260229, SGI260230, SGI260231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58343[] = {SGI260232, SGI0, SGI260233, SGI0, SGI260234, SGI260235, SGI260236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58344[] = {SGI260237, SGI0, SGI260238, SGI0, SGI260239, SGI260240, SGI260241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58345[] = {SGI260242, SGI0, SGI260243, SGI0, SGI260244, SGI260245, SGI260246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58346[] = {SGI260247, SGI0, SGI260248, SGI0, SGI260249, SGI260250, SGI260251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58347[] = {SGI260252, SGI0, SGI260253, SGI0, SGI260254, SGI260255, SGI260256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58348[] = {SGI260257, SGI0, SGI260258, SGI0, SGI260259, SGI260260, SGI260261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58349[] = {SGI260262, SGI0, SGI260263, SGI0, SGI260264, SGI260265, SGI260266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58350[] = {SGI260267, SGI0, SGI260268, SGI0, SGI260269, SGI260270, SGI260271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58351[] = {SGI260272, SGI0, SGI260273, SGI0, SGI260274, SGI260275, SGI260276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58352[] = {SGI260277, SGI0, SGI260278, SGI0, SGI260279, SGI260280, SGI260281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58353[] = {SGI260282, SGI0, SGI260283, SGI0, SGI260284, SGI260285, SGI260286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58354[] = {SGI260287, SGI0, SGI260288, SGI0, SGI260289, SGI260290, SGI260291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58355[] = {SGI260292, SGI0, SGI260293, SGI0, SGI260294, SGI260295, SGI260296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58356[] = {SGI260297, SGI0, SGI260298, SGI0, SGI260299, SGI260300, SGI260301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58357[] = {SGI260302, SGI0, SGI260303, SGI0, SGI260304, SGI260305, SGI260306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58358[] = {SGI260307, SGI0, SGI260308, SGI0, SGI260309, SGI260310, SGI260311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58359[] = {SGI260312, SGI0, SGI260313, SGI0, SGI260314, SGI260315, SGI260316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58360[] = {SGI260317, SGI0, SGI260318, SGI0, SGI260319, SGI260320, SGI260321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58361[] = {SGI260322, SGI0, SGI260323, SGI0, SGI260324, SGI260325, SGI260326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58362[] = {SGI260327, SGI0, SGI260328, SGI0, SGI260329, SGI260330, SGI260331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58363[] = {SGI260332, SGI260333, SGI0, SGI0, NULL};
const Segment * const SG58364[] = {SGI260334, SGI0, SGI260335, SGI0, SGI260336, SGI260337, SGI0, SGI0, SGI0, SGI260338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58365[] = {SGI260339, SGI0, SGI260340, SGI0, SGI260341, SGI260342, SGI0, SGI0, SGI0, SGI260343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58366[] = {SGI260344, SGI0, SGI260345, SGI0, SGI260346, SGI260347, SGI0, SGI0, SGI0, SGI260348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58367[] = {SGI260349, SGI0, SGI260350, SGI0, SGI260351, SGI260352, SGI0, SGI0, SGI0, SGI260353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58368[] = {SGI260354, SGI0, SGI260355, SGI0, SGI260356, SGI260357, SGI0, SGI0, SGI0, SGI260358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58369[] = {SGI260359, SGI0, SGI260360, SGI0, SGI260361, SGI260362, SGI0, SGI0, SGI0, SGI260363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58370[] = {SGI260364, SGI0, SGI260365, SGI0, SGI260366, SGI260367, SGI0, SGI0, SGI0, SGI260368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58371[] = {SGI260369, SGI0, SGI260370, SGI0, SGI260371, SGI260372, SGI0, SGI0, SGI0, SGI260373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58372[] = {SGI260374, SGI0, SGI260375, SGI0, SGI260376, SGI260377, SGI0, SGI0, SGI0, SGI260378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58373[] = {SGI260379, SGI0, SGI260380, SGI0, SGI260381, SGI260382, SGI260383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58374[] = {SGI260384, SGI0, SGI260385, SGI0, SGI260386, SGI260387, SGI260388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58375[] = {SGI260389, SGI0, SGI260390, SGI0, SGI260391, SGI260392, SGI260393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58376[] = {SGI260394, SGI0, SGI260395, SGI0, SGI260396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58377[] = {SGI260397, SGI0, SGI260398, SGI0, SGI260399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58378[] = {SGI260400, SGI0, SGI260401, SGI0, SGI260402, SGI260403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58379[] = {SGI260404, SGI0, SGI260405, SGI0, SGI260406, SGI260407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58380[] = {SGI260408, SGI0, SGI260409, SGI0, SGI260410, SGI260411, SGI260410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58381[] = {SGI260412, SGI0, SGI260413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58382[] = {SGI260414, SGI0, SGI260415, SGI0, SGI260416, SGI260417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58383[] = {SGI260418, SGI0, SGI260419, SGI0, SGI260420, SGI260421, SGI260420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58384[] = {SGI260422, SGI0, SGI260423, SGI0, SGI260424, SGI260425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58385[] = {SGI260426, SGI0, SGI260427, SGI0, SGI260428, SGI260429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58386[] = {SGI260430, SGI260431, SGI0, SGI0, NULL};
const Segment * const SG58387[] = {SGI260432, SGI0, SGI260433, SGI0, SGI260434, SGI260435, SGI260436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58388[] = {SGI260437, SGI0, SGI260438, SGI0, SGI260439, SGI260440, SGI260441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58389[] = {SGI260442, SGI0, SGI260443, SGI0, SGI260444, SGI260445, SGI260446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58390[] = {SGI260447, SGI0, SGI260448, SGI0, SGI260449, SGI260450, SGI260451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58391[] = {SGI260452, SGI0, SGI260453, SGI0, SGI260454, SGI260455, SGI260456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58392[] = {SGI260457, SGI0, SGI260458, SGI0, SGI260459, SGI260460, SGI260461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58393[] = {SGI260462, SGI0, SGI260463, SGI0, SGI260464, SGI260465, SGI260466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58394[] = {SGI260467, SGI260468, SGI0, SGI0, NULL};
const Segment * const SG58395[] = {SGI260469, SGI0, SGI260470, SGI0, SGI260471, SGI260472, SGI260473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58396[] = {SGI260474, SGI0, SGI260475, SGI0, SGI260476, SGI260477, SGI260478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58397[] = {SGI260479, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58398[] = {SGI260480, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58399[] = {SGI260481, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58400[] = {SGI260482, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58401[] = {SGI260483, SGI260484, NULL};
const Segment * const SG58402[] = {SGI260485, SGI260486, NULL};
const Segment * const SG58403[] = {SGI260487, SGI0, SGI260488, SGI0, SGI260489, SGI0, SGI260490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI260491, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58404[] = {SGI260492, SGI0, SGI260493, SGI0, SGI260494, SGI260495, SGI260496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58405[] = {SGI260497, SGI0, SGI260498, SGI0, SGI260499, SGI260500, SGI260501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58406[] = {SGI260502, SGI0, SGI260503, SGI0, SGI260504, SGI260505, SGI260506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58407[] = {SGI260507, SGI260508, SGI260509, SGI0, SGI260510, SGI260511, SGI260512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58408[] = {SGI260513, SGI0, SGI260514, SGI0, SGI260515, SGI260516, SGI260517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58409[] = {SGI260518, SGI0, SGI260519, SGI0, SGI260520, SGI260521, SGI260522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58410[] = {SGI260523, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58411[] = {SGI260524, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58412[] = {SGI260525, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58413[] = {SGI260526, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58414[] = {SGI260527, SGI260528, SGI0, SGI0, NULL};
const Segment * const SG58415[] = {SGI260529, SGI0, SGI260530, SGI0, SGI260531, SGI260532, SGI260533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58416[] = {SGI260534, SGI0, SGI260535, SGI0, SGI260536, SGI260537, SGI260538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58417[] = {SGI260539, SGI0, SGI260540, SGI0, SGI260541, SGI260542, SGI260543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58418[] = {SGI260544, SGI0, SGI260545, SGI0, SGI260546, SGI260547, SGI260548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58419[] = {SGI260549, SGI0, SGI260550, SGI0, SGI260551, SGI260552, SGI0, SGI0, SGI0, SGI260553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58420[] = {SGI260554, SGI0, SGI260555, SGI0, SGI260556, SGI260557, SGI0, SGI0, SGI0, SGI260558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58421[] = {SGI260559, SGI0, SGI260560, SGI0, SGI260561, SGI260562, SGI0, SGI0, SGI0, SGI260563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58422[] = {SGI260564, SGI0, SGI260565, SGI0, SGI260566, SGI260567, SGI0, SGI0, SGI0, SGI260568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58423[] = {SGI260569, SGI260570, NULL};
const Segment * const SG58424[] = {SGI260571, SGI260572, SGI0, SGI0, NULL};
const Segment * const SG58425[] = {SGI260573, SGI260574, NULL};
const Segment * const SG58426[] = {SGI260575, SGI0, SGI260576, SGI0, SGI260577, SGI260578, SGI260579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58427[] = {SGI260580, SGI0, SGI260581, SGI0, SGI260582, SGI260583, SGI260584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58428[] = {SGI260585, SGI0, SGI260586, SGI0, SGI260587, SGI260588, SGI260589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58429[] = {SGI260590, SGI0, SGI260591, SGI0, SGI260592, SGI260593, SGI260594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58430[] = {SGI260595, SGI0, SGI260596, SGI0, SGI260597, SGI260598, SGI260599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58431[] = {SGI260600, SGI0, SGI260601, SGI0, SGI260602, SGI260603, SGI260604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58432[] = {SGI260605, SGI0, SGI260606, SGI0, SGI260607, SGI260608, SGI260609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58433[] = {SGI260610, SGI0, SGI260611, SGI0, SGI260612, SGI260613, SGI260614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58434[] = {SGI260615, SGI260616, NULL};
const Segment * const SG58435[] = {SGI260617, SGI0, SGI260618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58436[] = {SGI260619, SGI0, SGI260620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58437[] = {SGI260621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58438[] = {SGI260622, SGI0, SGI260623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58439[] = {SGI260624, SGI0, SGI260625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58440[] = {SGI260626, SGI0, SGI260627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI260628, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58441[] = {SGI260629, SGI260630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58442[] = {SGI260631, SGI260632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58443[] = {SGI260633, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58444[] = {SGI260634, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58445[] = {SGI260635, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58446[] = {SGI260636, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58447[] = {SGI260637, SGI260638, SGI0, SGI0, NULL};
const Segment * const SG58448[] = {SGI260639, SGI0, SGI260640, SGI0, SGI260641, SGI260642, SGI260643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58449[] = {SGI260644, SGI0, SGI260645, SGI0, SGI260646, SGI260647, SGI260648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58450[] = {SGI260649, SGI0, SGI260650, SGI0, SGI260651, SGI260652, SGI260653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58451[] = {SGI260654, SGI260655, SGI260656, SGI0, SGI260657, SGI260658, SGI260659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58452[] = {SGI260660, SGI260661, SGI260662, SGI0, SGI260663, SGI260664, SGI260665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58453[] = {SGI260666, SGI260667, SGI260668, SGI0, SGI260669, SGI260670, SGI260671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58454[] = {SGI260672, SGI0, SGI260673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58455[] = {SGI260674, SGI0, SGI260675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58456[] = {SGI260676, SGI0, SGI260677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58457[] = {SGI260678, SGI0, SGI260679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58458[] = {SGI260680, SGI0, SGI260681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58459[] = {SGI260682, SGI260683, NULL};
const Segment * const SG58460[] = {SGI260684, SGI260685, NULL};
const Segment * const SG58461[] = {SGI260686, SGI260687, NULL};
const Segment * const SG58462[] = {SGI260688, SGI260689, NULL};
const Segment * const SG58463[] = {SGI260690, SGI260691, NULL};
const Segment * const SG58464[] = {SGI260692, SGI260693, NULL};
const Segment * const SG58465[] = {SGI260694, SGI260695, NULL};
const Segment * const SG58466[] = {SGI260696, SGI260697, NULL};
const Segment * const SG58467[] = {SGI260698, SGI260699, NULL};
const Segment * const SG58468[] = {SGI260700, SGI260701, NULL};
const Segment * const SG58469[] = {SGI260702, SGI260703, NULL};
const Segment * const SG58470[] = {SGI260704, SGI260705, NULL};
const Segment * const SG58471[] = {SGI260706, SGI260707, NULL};
const Segment * const SG58472[] = {SGI260708, SGI260709, NULL};
const Segment * const SG58473[] = {SGI260710, SGI0, SGI260711, SGI0, SGI260712, SGI260713, SGI260714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58474[] = {SGI260715, SGI0, SGI260716, SGI0, SGI260717, SGI260718, SGI260719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58475[] = {SGI260720, SGI0, SGI260721, SGI0, SGI260722, SGI260723, SGI260724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58476[] = {SGI260725, SGI0, SGI260726, SGI0, SGI260727, SGI260728, SGI260729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58477[] = {SGI260730, SGI0, SGI260731, SGI0, SGI260732, SGI260733, SGI260734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58478[] = {SGI260735, SGI260736, SGI260737, SGI0, SGI260738, SGI260739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58479[] = {SGI260740, SGI260741, SGI260742, SGI0, SGI260743, SGI260744, SGI260745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58480[] = {SGI260746, SGI260747, SGI260748, SGI0, SGI260749, SGI260750, SGI260751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58481[] = {SGI260752, SGI260753, SGI260754, SGI0, SGI260755, SGI260756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58482[] = {SGI260757, SGI260758, SGI260759, SGI0, SGI260760, SGI260761, SGI260762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58483[] = {SGI260763, SGI260764, SGI260765, SGI0, SGI260766, SGI260767, SGI260768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58484[] = {SGI260769, SGI0, SGI260770, SGI0, SGI260771, SGI260772, SGI260773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58485[] = {SGI260774, SGI260775, SGI260776, SGI0, SGI260777, SGI260778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58486[] = {SGI260779, SGI260780, SGI260781, SGI0, SGI260782, SGI260783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58487[] = {SGI260784, SGI260785, SGI260786, SGI0, SGI260787, SGI260788, SGI260789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58488[] = {SGI260790, SGI260791, SGI260792, SGI0, SGI260793, SGI260794, SGI260795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58489[] = {SGI260796, SGI0, SGI260797, SGI0, SGI260798, SGI260799, SGI260800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58490[] = {SGI260801, SGI260802, SGI260803, SGI0, SGI260804, SGI260805, SGI260806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58491[] = {SGI260807, SGI0, SGI260808, SGI0, SGI260809, SGI260810, SGI260811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58492[] = {SGI260812, SGI260813, SGI260814, SGI0, SGI260815, SGI260816, SGI260817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58493[] = {SGI260818, SGI260819, SGI260820, SGI0, SGI260821, SGI260822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58494[] = {SGI260823, SGI0, SGI260824, SGI0, SGI260825, SGI260826, SGI260827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58495[] = {SGI260828, SGI260829, SGI260830, SGI0, SGI260831, SGI260832, SGI260833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58496[] = {SGI260834, SGI260835, SGI260836, SGI0, SGI260837, SGI260838, SGI260839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58497[] = {SGI260840, SGI260841, SGI260842, SGI0, SGI260843, SGI260844, SGI260845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58498[] = {SGI260846, SGI0, SGI260847, SGI0, SGI260848, SGI260849, SGI260850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58499[] = {SGI260851, SGI0, SGI260852, SGI0, SGI260853, SGI260854, SGI260855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58500[] = {SGI260856, SGI0, SGI260857, SGI0, SGI260858, SGI260859, SGI260860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58501[] = {SGI260861, SGI0, SGI260862, SGI0, SGI260863, SGI260864, SGI260865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58502[] = {SGI260866, SGI0, SGI260867, SGI0, SGI260868, SGI260869, SGI260870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58503[] = {SGI260871, SGI0, SGI260872, SGI0, SGI260873, SGI260874, SGI260875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58504[] = {SGI260876, SGI0, SGI260877, SGI0, SGI260878, SGI260879, SGI260880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58505[] = {SGI260881, SGI0, SGI260882, SGI0, SGI260883, SGI260884, SGI260885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58506[] = {SGI260886, SGI260887, SGI0, SGI0, SGI260888, SGI260889, SGI260890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58507[] = {SGI260891, SGI260892, SGI0, SGI0, SGI260893, SGI260894, SGI260895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58508[] = {SGI260896, SGI260897, SGI0, SGI0, SGI260898, SGI260899, SGI260900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58509[] = {SGI260901, SGI260902, SGI0, SGI0, SGI260903, SGI260904, SGI260905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58510[] = {SGI260906, SGI260907, SGI0, SGI0, SGI260908, SGI260909, SGI260910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58511[] = {SGI260911, SGI260912, SGI0, SGI0, SGI260913, SGI260914, SGI260915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58512[] = {SGI260916, SGI260917, SGI0, SGI0, SGI260918, SGI260919, SGI260920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58513[] = {SGI260921, SGI0, SGI260922, SGI0, SGI260923, SGI260924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI260925, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58514[] = {SGI260926, SGI0, SGI260927, SGI0, SGI260928, SGI260929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI260930, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58515[] = {SGI260931, SGI0, SGI260932, SGI0, SGI260933, SGI260934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI260935, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58516[] = {SGI260936, SGI0, SGI260937, SGI0, SGI260938, SGI260939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI260940, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58517[] = {SGI260941, SGI0, SGI260942, SGI0, SGI260943, SGI260944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI260945, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58518[] = {SGI260946, SGI0, SGI260947, SGI0, SGI260948, SGI260949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI260950, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58519[] = {SGI260951, SGI0, SGI260952, SGI0, SGI260953, SGI260954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI260955, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58520[] = {SGI260956, SGI0, SGI260957, SGI0, SGI260958, SGI260959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI260960, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58521[] = {SGI260961, SGI0, SGI260962, SGI0, SGI260963, SGI260964, SGI260965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI260966, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58522[] = {SGI260967, SGI0, SGI260968, SGI0, SGI260969, SGI260970, SGI0, SGI0, SGI0, SGI260971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58523[] = {SGI260972, SGI0, SGI260973, SGI0, SGI260974, SGI260975, SGI0, SGI0, SGI0, SGI260976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58524[] = {SGI260977, SGI0, SGI260978, SGI0, SGI260979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58525[] = {SGI260980, SGI0, SGI260981, SGI0, SGI260982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58526[] = {SGI260983, SGI0, SGI260984, SGI0, SGI260985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58527[] = {SGI260986, SGI260987, NULL};
const Segment * const SG58528[] = {SGI260988, SGI0, SGI260989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58529[] = {SGI260990, SGI0, SGI260991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58530[] = {SGI260992, SGI0, SGI260993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58531[] = {SGI260994, SGI260995, SGI260996, SGI0, SGI260997, SGI260998, SGI260997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI260999, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58532[] = {SGI261000, SGI0, SGI261001, SGI0, SGI261002, SGI261003, SGI261004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58533[] = {SGI261005, SGI0, SGI261006, SGI0, SGI261007, SGI261008, SGI261009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58534[] = {SGI261010, SGI0, SGI261011, SGI0, SGI261012, SGI261013, SGI261014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58535[] = {SGI261015, SGI0, SGI261016, SGI0, SGI261017, SGI261018, SGI261019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58536[] = {SGI261020, SGI0, SGI261021, SGI0, SGI261022, SGI261023, SGI0, SGI0, SGI0, SGI261024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58537[] = {SGI261025, SGI0, SGI261026, SGI0, SGI261027, SGI261028, SGI0, SGI0, SGI0, SGI261029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58538[] = {SGI261030, SGI0, SGI261031, SGI0, SGI261032, SGI261033, SGI0, SGI0, SGI0, SGI261034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58539[] = {SGI261035, SGI0, SGI261036, SGI0, SGI261037, SGI261038, SGI0, SGI0, SGI0, SGI261039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58540[] = {SGI261040, SGI261041, NULL};
const Segment * const SG58541[] = {SGI261042, SGI261043, NULL};
const Segment * const SG58542[] = {SGI261044, SGI261045, NULL};
const Segment * const SG58543[] = {SGI261046, SGI0, SGI261047, SGI0, SGI261048, SGI261049, SGI261050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58544[] = {SGI261051, SGI0, SGI261052, SGI0, SGI261053, SGI261054, SGI261055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58545[] = {SGI261056, SGI0, SGI261057, SGI0, SGI261058, SGI261059, SGI261060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58546[] = {SGI261061, SGI261062, SGI0, SGI0, NULL};
const Segment * const SG58547[] = {SGI261063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58548[] = {SGI261064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58549[] = {SGI261065, SGI0, SGI261066, SGI0, SGI261067, SGI261068, SGI261069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58550[] = {SGI261070, SGI0, SGI261071, SGI0, SGI261072, SGI261073, SGI261074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58551[] = {SGI261075, SGI0, SGI261076, SGI0, SGI261077, SGI261078, SGI261079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58552[] = {SGI261080, SGI0, SGI261081, SGI0, SGI261082, SGI261083, SGI0, SGI0, SGI0, SGI261084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58553[] = {SGI261085, SGI0, SGI261086, SGI0, SGI261087, SGI261088, SGI0, SGI0, SGI0, SGI261089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58554[] = {SGI261090, SGI0, SGI261091, SGI0, SGI261092, SGI261093, SGI0, SGI0, SGI0, SGI261094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58555[] = {SGI261095, SGI0, SGI261096, SGI0, SGI261097, SGI261098, SGI0, SGI0, SGI0, SGI261099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58556[] = {SGI261100, SGI0, SGI261101, SGI0, SGI261102, SGI261103, SGI0, SGI0, SGI0, SGI261104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58557[] = {SGI261105, SGI0, SGI261106, SGI0, SGI261107, SGI261108, SGI0, SGI0, SGI0, SGI261109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58558[] = {SGI261110, SGI0, SGI261111, SGI0, SGI261112, SGI261113, SGI0, SGI0, SGI0, SGI261114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58559[] = {SGI261115, SGI0, SGI261116, SGI0, SGI261117, SGI261118, SGI0, SGI0, SGI0, SGI261119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58560[] = {SGI261120, SGI0, SGI261121, SGI0, SGI261122, SGI261123, SGI0, SGI0, SGI0, SGI261124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58561[] = {SGI261125, SGI0, SGI261126, SGI0, SGI261127, SGI261128, SGI261129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58562[] = {SGI261130, SGI0, SGI261131, SGI0, SGI261132, SGI261133, SGI261134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58563[] = {SGI261135, SGI0, SGI261136, SGI0, SGI261137, SGI261138, SGI261139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58564[] = {SGI261140, SGI261141, SGI261142, SGI0, SGI261143, SGI261144, SGI261145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58565[] = {SGI261146, SGI261147, SGI261148, SGI0, SGI261149, SGI261150, SGI261151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58566[] = {SGI261152, SGI261153, SGI261154, SGI0, SGI261155, SGI261156, SGI261157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58567[] = {SGI261158, SGI261159, SGI261160, SGI0, SGI261161, SGI261162, SGI261163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58568[] = {SGI261164, SGI261165, SGI261166, SGI0, SGI261167, SGI261168, SGI261169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58569[] = {SGI261170, SGI261171, SGI261172, SGI0, SGI261173, SGI261174, SGI261175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58570[] = {SGI261176, SGI0, SGI261177, SGI0, SGI261178, SGI261179, SGI261180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58571[] = {SGI261181, SGI0, SGI261182, SGI0, SGI261183, SGI261184, SGI261185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58572[] = {SGI261186, SGI0, SGI261187, SGI0, SGI261188, SGI261189, SGI261190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58573[] = {SGI261191, SGI0, SGI261192, SGI0, SGI261193, SGI261194, SGI261195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58574[] = {SGI261196, SGI261197, NULL};
const Segment * const SG58575[] = {SGI261198, SGI261199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58576[] = {SGI261200, SGI0, SGI261201, SGI0, SGI261202, SGI261203, SGI0, SGI0, SGI0, SGI261204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58577[] = {SGI261205, SGI0, SGI261206, SGI0, SGI261207, SGI261208, SGI0, SGI0, SGI0, SGI261209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58578[] = {SGI261210, SGI0, SGI261211, SGI0, SGI261212, SGI261213, SGI0, SGI0, SGI0, SGI261214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58579[] = {SGI261215, SGI0, SGI261216, SGI0, SGI261217, SGI261218, SGI0, SGI0, SGI0, SGI261219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58580[] = {SGI261220, SGI0, SGI261221, SGI0, SGI261222, SGI261223, SGI0, SGI0, SGI0, SGI261224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58581[] = {SGI261225, SGI0, SGI261226, SGI0, SGI261227, SGI261228, SGI0, SGI0, SGI0, SGI261229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58582[] = {SGI261230, SGI0, SGI261231, SGI0, SGI261232, SGI261233, SGI0, SGI0, SGI0, SGI261234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58583[] = {SGI261235, SGI261236, NULL};
const Segment * const SG58584[] = {SGI261237, SGI261238, NULL};
const Segment * const SG58585[] = {SGI261239, SGI261240, SGI261241, SGI0, SGI261242, SGI261243, SGI261244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58586[] = {SGI261245, SGI261246, SGI261247, SGI0, SGI261248, SGI261249, SGI261250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58587[] = {SGI261251, SGI261252, SGI261253, SGI0, SGI261254, SGI261255, SGI261256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58588[] = {SGI261257, SGI261258, SGI261259, SGI0, SGI261260, SGI261261, SGI261260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58589[] = {SGI261262, SGI261263, SGI261264, SGI0, SGI261265, SGI261266, SGI261265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58590[] = {SGI261267, SGI261268, NULL};
const Segment * const SG58591[] = {SGI261269, SGI261270, NULL};
const Segment * const SG58592[] = {SGI261271, SGI261272, NULL};
const Segment * const SG58593[] = {SGI261273, SGI261274, NULL};
const Segment * const SG58594[] = {SGI261275, SGI261276, NULL};
const Segment * const SG58595[] = {SGI261277, SGI261278, NULL};
const Segment * const SG58596[] = {SGI261279, SGI261280, NULL};
const Segment * const SG58597[] = {SGI261281, SGI261282, NULL};
const Segment * const SG58598[] = {SGI261283, SGI261284, NULL};
const Segment * const SG58599[] = {SGI261285, SGI0, SGI261286, SGI0, SGI261287, SGI261288, SGI261289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58600[] = {SGI261290, SGI261291, NULL};
const Segment * const SG58601[] = {SGI261292, SGI0, SGI261293, SGI0, SGI261294, SGI261295, SGI261296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58602[] = {SGI261297, SGI261298, NULL};
const Segment * const SG58603[] = {SGI261299, SGI0, SGI261300, SGI0, SGI261301, SGI261302, SGI261303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58604[] = {SGI261304, SGI261305, NULL};
const Segment * const SG58605[] = {SGI261306, SGI261307, NULL};
const Segment * const SG58606[] = {SGI261308, SGI261309, NULL};
const Segment * const SG58607[] = {SGI261310, SGI261311, NULL};
const Segment * const SG58608[] = {SGI261312, SGI261313, NULL};
const Segment * const SG58609[] = {SGI261314, SGI261315, NULL};
const Segment * const SG58610[] = {SGI261316, SGI261317, NULL};
const Segment * const SG58611[] = {SGI261318, SGI261319, SGI261320, SGI0, SGI261321, SGI261322, SGI261323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58612[] = {SGI261324, SGI261325, SGI261326, SGI0, SGI261327, SGI261328, SGI261329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58613[] = {SGI261330, SGI0, SGI261331, SGI0, SGI0, SGI261332, SGI261333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58614[] = {SGI261334, SGI261335, SGI261336, SGI0, SGI261337, SGI261338, SGI261339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58615[] = {SGI261340, SGI261341, SGI261342, SGI0, SGI261343, SGI261344, SGI261345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58616[] = {SGI261346, SGI261347, SGI261348, SGI0, SGI261349, SGI261350, SGI261351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58617[] = {SGI261352, SGI261353, SGI261354, SGI0, SGI261355, SGI261356, SGI261357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58618[] = {SGI261358, SGI261359, SGI261360, SGI0, SGI261361, SGI261362, SGI261363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58619[] = {SGI261364, SGI0, SGI261365, SGI0, SGI0, SGI261366, SGI261367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58620[] = {SGI261368, SGI0, SGI261369, SGI0, SGI261370, SGI261371, SGI261372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58621[] = {SGI261373, SGI0, SGI261374, SGI0, SGI261375, SGI261376, SGI261377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58622[] = {SGI261378, SGI0, SGI261379, SGI0, SGI261380, SGI261381, SGI261382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58623[] = {SGI261383, SGI0, SGI261384, SGI0, SGI261385, SGI261386, SGI261387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58624[] = {SGI261388, SGI261389, SGI261390, SGI0, SGI261391, SGI261392, SGI261393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58625[] = {SGI261394, SGI261395, NULL};
const Segment * const SG58626[] = {SGI261396, SGI261397, NULL};
const Segment * const SG58627[] = {SGI261398, SGI0, SGI261399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58628[] = {SGI261400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58629[] = {SGI261401, SGI0, SGI261402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58630[] = {SGI261403, SGI0, SGI261404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58631[] = {SGI261405, SGI0, SGI261406, SGI0, SGI261407, SGI261408, SGI261409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58632[] = {SGI261410, SGI0, SGI261411, SGI0, SGI261412, SGI261413, SGI261414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58633[] = {SGI261415, SGI0, SGI261416, SGI0, SGI261417, SGI261418, SGI261419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58634[] = {SGI261420, SGI0, SGI261421, SGI0, SGI261422, SGI261423, SGI261424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58635[] = {SGI261425, SGI0, SGI261426, SGI0, SGI261427, SGI261428, SGI261429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58636[] = {SGI261430, SGI0, SGI261431, SGI0, SGI261432, SGI261433, SGI261434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58637[] = {SGI261435, SGI0, SGI261436, SGI0, SGI261437, SGI261438, SGI261439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58638[] = {SGI261440, SGI0, SGI261441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58639[] = {SGI261442, SGI0, SGI261443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58640[] = {SGI261444, SGI0, SGI261445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58641[] = {SGI261446, SGI0, SGI261447, SGI0, SGI261448, SGI261449, SGI261450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58642[] = {SGI261451, SGI261452, NULL};
const Segment * const SG58643[] = {SGI261453, SGI261454, NULL};
const Segment * const SG58644[] = {SGI261455, SGI261456, NULL};
const Segment * const SG58645[] = {SGI261457, SGI261458, SGI261459, SGI0, SGI261460, SGI261461, SGI261462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58646[] = {SGI261463, SGI261464, SGI261465, SGI0, SGI261466, SGI261467, SGI261468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58647[] = {SGI261469, SGI261470, SGI261471, SGI0, SGI261472, SGI261473, SGI261474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58648[] = {SGI261475, SGI261476, SGI261477, SGI0, SGI261478, SGI261479, SGI261480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58649[] = {SGI261481, SGI261482, SGI261483, SGI0, SGI261484, SGI261485, SGI261486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58650[] = {SGI261487, SGI261488, SGI261489, SGI0, SGI261490, SGI261491, SGI261492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58651[] = {SGI261493, SGI261494, SGI261495, SGI0, SGI261496, SGI261497, SGI261498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58652[] = {SGI261499, SGI0, SGI261500, SGI0, SGI0, SGI261501, SGI261502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58653[] = {SGI261503, SGI261504, SGI261505, SGI0, SGI261506, SGI261507, SGI261508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58654[] = {SGI261509, SGI261510, SGI261511, SGI0, SGI261512, SGI261513, SGI261514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58655[] = {SGI261515, SGI0, SGI261516, SGI0, SGI261517, SGI261518, SGI261519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58656[] = {SGI261520, SGI261521, SGI261522, SGI0, SGI261523, SGI261524, SGI261525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58657[] = {SGI261526, SGI261527, SGI261528, SGI0, SGI261529, SGI261530, SGI261531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58658[] = {SGI261532, SGI261533, SGI261534, SGI0, SGI261535, SGI261536, SGI261537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58659[] = {SGI261538, SGI261539, SGI261540, SGI0, SGI261541, SGI261542, SGI261543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58660[] = {SGI261544, SGI0, SGI261545, SGI0, SGI261546, SGI261547, SGI261548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58661[] = {SGI261549, SGI0, SGI261550, SGI0, SGI261551, SGI261552, SGI261553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58662[] = {SGI261554, SGI0, SGI261555, SGI0, SGI261556, SGI261557, SGI261558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58663[] = {SGI261559, SGI0, SGI261560, SGI0, SGI261561, SGI261562, SGI261563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58664[] = {SGI261564, SGI0, SGI261565, SGI0, SGI261566, SGI261567, SGI261568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58665[] = {SGI261569, SGI0, SGI261570, SGI0, SGI261571, SGI261572, SGI261573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58666[] = {SGI261574, SGI0, SGI261575, SGI0, SGI261576, SGI261577, SGI261578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58667[] = {SGI261579, SGI0, SGI261580, SGI0, SGI261581, SGI261582, SGI261583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58668[] = {SGI261584, SGI261585, SGI261586, SGI0, SGI261587, SGI261588, SGI261589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58669[] = {SGI261590, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58670[] = {SGI261591, SGI0, SGI261592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58671[] = {SGI261593, SGI0, SGI261594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58672[] = {SGI261595, SGI0, SGI261596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58673[] = {SGI261597, SGI0, SGI261598, SGI0, SGI261599, SGI261600, SGI261601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58674[] = {SGI261602, SGI0, SGI261603, SGI0, SGI261604, SGI261605, SGI261606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58675[] = {SGI261607, SGI0, SGI261608, SGI0, SGI261609, SGI261610, SGI261611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58676[] = {SGI261612, SGI0, SGI261613, SGI0, SGI261614, SGI261615, SGI261616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58677[] = {SGI261617, SGI0, SGI261618, SGI0, SGI261619, SGI261620, SGI0, SGI0, SGI0, SGI261621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58678[] = {SGI261622, SGI0, SGI261623, SGI0, SGI261624, SGI261625, SGI0, SGI0, SGI0, SGI261626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58679[] = {SGI261627, SGI0, SGI261628, SGI0, SGI261629, SGI261630, SGI0, SGI0, SGI0, SGI261631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58680[] = {SGI261632, SGI261633, NULL};
const Segment * const SG58681[] = {SGI261634, SGI261635, NULL};
const Segment * const SG58682[] = {SGI261636, SGI261637, NULL};
const Segment * const SG58683[] = {SGI261638, SGI0, SGI261639, SGI0, SGI261640, SGI261641, SGI261642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58684[] = {SGI261643, SGI0, SGI261644, SGI0, SGI261645, SGI261646, SGI261647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58685[] = {SGI261648, SGI0, SGI261649, SGI0, SGI261650, SGI261651, SGI261652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58686[] = {SGI261653, SGI0, SGI261654, SGI0, SGI261655, SGI261656, SGI261657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58687[] = {SGI261658, SGI0, SGI261659, SGI0, SGI261660, SGI261661, SGI261662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58688[] = {SGI261663, SGI0, SGI261664, SGI0, SGI261665, SGI261666, SGI261667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58689[] = {SGI261668, SGI0, SGI261669, SGI0, SGI261670, SGI261671, SGI261672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58690[] = {SGI261673, SGI261674, NULL};
const Segment * const SG58691[] = {SGI261675, SGI261676, NULL};
const Segment * const SG58692[] = {SGI261677, SGI261678, NULL};
const Segment * const SG58693[] = {SGI261679, SGI261680, SGI0, SGI0, NULL};
const Segment * const SG58694[] = {SGI261681, SGI0, SGI261682, SGI0, SGI261683, SGI261684, SGI261685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58695[] = {SGI261686, SGI0, SGI261687, SGI0, SGI261688, SGI261689, SGI261690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58696[] = {SGI261691, SGI0, SGI261692, SGI0, SGI261693, SGI261694, SGI261695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58697[] = {SGI261696, SGI0, SGI261697, SGI0, SGI261698, SGI261699, SGI261700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58698[] = {SGI261701, SGI0, SGI261702, SGI0, SGI261703, SGI261704, SGI261705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58699[] = {SGI261706, SGI0, SGI261707, SGI0, SGI261708, SGI261709, SGI261710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58700[] = {SGI261711, SGI0, SGI261712, SGI0, SGI261713, SGI261714, SGI261715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58701[] = {SGI261716, SGI0, SGI261717, SGI0, SGI261718, SGI261719, SGI261720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58702[] = {SGI261721, SGI0, SGI261722, SGI0, SGI261723, SGI261724, SGI261725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58703[] = {SGI261726, SGI0, SGI261727, SGI0, SGI261728, SGI261729, SGI261730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58704[] = {SGI261731, SGI261732, SGI261733, SGI0, SGI261734, SGI261735, SGI261736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58705[] = {SGI261737, SGI261738, SGI261739, SGI0, SGI261740, SGI261741, SGI261742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58706[] = {SGI261743, SGI261744, SGI261745, SGI0, SGI261746, SGI261747, SGI261748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58707[] = {SGI261749, SGI0, SGI261750, SGI0, SGI261751, SGI261752, SGI261753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58708[] = {SGI261754, SGI261755, SGI261756, SGI0, SGI261757, SGI261758, SGI261759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58709[] = {SGI261760, SGI261761, SGI261762, SGI0, SGI261763, SGI261764, SGI261765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58710[] = {SGI261766, SGI261767, SGI261768, SGI0, SGI261769, SGI261770, SGI261771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58711[] = {SGI261772, SGI261773, SGI261774, SGI0, SGI261775, SGI261776, SGI261777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58712[] = {SGI261778, SGI261779, SGI261780, SGI0, SGI261781, SGI261782, SGI261783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58713[] = {SGI261784, SGI261785, SGI261786, SGI0, SGI261787, SGI261788, SGI261789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58714[] = {SGI261790, SGI0, SGI261791, SGI0, SGI261792, SGI261793, SGI261794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58715[] = {SGI261795, SGI0, SGI261796, SGI0, SGI261797, SGI261798, SGI261799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58716[] = {SGI261800, SGI0, SGI261801, SGI0, SGI261802, SGI261803, SGI261804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58717[] = {SGI261805, SGI261806, NULL};
const Segment * const SG58718[] = {SGI261807, SGI261808, NULL};
const Segment * const SG58719[] = {SGI261809, SGI261810, SGI261811, SGI0, SGI261812, SGI261813, SGI261814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58720[] = {SGI261815, SGI261816, SGI261817, SGI0, SGI261818, SGI261819, SGI261820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58721[] = {SGI261821, SGI261822, SGI261823, SGI0, SGI261824, SGI261825, SGI261826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58722[] = {SGI261827, SGI261828, SGI261829, SGI0, SGI261830, SGI261831, SGI261832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58723[] = {SGI261833, SGI261834, SGI261835, SGI0, SGI261836, SGI261837, SGI261838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58724[] = {SGI261839, SGI261840, SGI261841, SGI0, SGI261842, SGI261843, SGI261844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58725[] = {SGI261845, SGI261846, SGI261847, SGI0, SGI261848, SGI261849, SGI261850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58726[] = {SGI261851, SGI261852, SGI261853, SGI0, SGI261854, SGI261855, SGI261856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58727[] = {SGI261857, SGI261858, SGI261859, SGI0, SGI261860, SGI261861, SGI261862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58728[] = {SGI261863, SGI261864, SGI261865, SGI0, SGI261866, SGI261867, SGI261868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58729[] = {SGI261869, SGI261870, SGI261871, SGI0, SGI261872, SGI261873, SGI261874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58730[] = {SGI261875, SGI261876, SGI261877, SGI0, SGI261878, SGI261879, SGI261880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58731[] = {SGI261881, SGI261882, SGI261883, SGI0, SGI261884, SGI261885, SGI261886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58732[] = {SGI261887, SGI261888, SGI261889, SGI0, SGI261890, SGI261891, SGI261892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58733[] = {SGI261893, SGI261894, SGI261895, SGI0, SGI261896, SGI261897, SGI261898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58734[] = {SGI261899, SGI261900, SGI261901, SGI0, SGI261902, SGI261903, SGI261904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58735[] = {SGI261905, SGI0, SGI261906, SGI0, SGI0, SGI261907, SGI261908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58736[] = {SGI261909, SGI261910, NULL};
const Segment * const SG58737[] = {SGI261911, SGI261912, NULL};
const Segment * const SG58738[] = {SGI261913, SGI261914, SGI261915, SGI0, SGI261916, SGI261917, SGI261918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58739[] = {SGI261919, SGI261920, SGI261921, SGI0, SGI261922, SGI261923, SGI261924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58740[] = {SGI261925, SGI261926, SGI261927, SGI0, SGI261928, SGI261929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58741[] = {SGI261930, SGI261931, SGI261932, SGI0, SGI261933, SGI261934, SGI261935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58742[] = {SGI261936, SGI261937, SGI261938, SGI0, SGI261939, SGI261940, SGI261941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58743[] = {SGI261942, SGI261943, SGI261944, SGI0, SGI261945, SGI261946, SGI261947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58744[] = {SGI261948, SGI261949, SGI261950, SGI0, SGI261951, SGI261952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58745[] = {SGI261953, SGI0, SGI261954, SGI0, SGI261955, SGI261956, SGI261957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58746[] = {SGI261958, SGI261959, SGI261960, SGI0, SGI261961, SGI261962, SGI261963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58747[] = {SGI261964, SGI261965, SGI261966, SGI0, SGI261967, SGI261968, SGI261969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58748[] = {SGI261970, SGI0, SGI261971, SGI0, SGI261972, SGI261973, SGI261974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58749[] = {SGI261975, SGI0, SGI261976, SGI0, SGI261977, SGI261978, SGI261979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58750[] = {SGI261980, SGI0, SGI261981, SGI0, SGI261982, SGI261983, SGI261984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58751[] = {SGI261985, SGI0, SGI261986, SGI0, SGI261987, SGI261988, SGI261989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58752[] = {SGI261990, SGI0, SGI261991, SGI0, SGI261992, SGI261993, SGI261994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58753[] = {SGI261995, SGI0, SGI261996, SGI0, SGI261997, SGI261998, SGI261999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58754[] = {SGI262000, SGI0, SGI262001, SGI0, SGI262002, SGI262003, SGI262004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58755[] = {SGI262005, SGI262006, SGI262007, SGI0, SGI262008, SGI262009, SGI262010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58756[] = {SGI262011, SGI262012, SGI262013, SGI0, SGI262014, SGI262015, SGI262016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58757[] = {SGI262017, SGI262018, SGI262019, SGI0, SGI262020, SGI262021, SGI262022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58758[] = {SGI262023, SGI262024, SGI262025, SGI0, SGI262026, SGI262027, SGI262028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58759[] = {SGI262029, SGI262030, SGI262031, SGI0, SGI262032, SGI262033, SGI262034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58760[] = {SGI262035, SGI262036, SGI262037, SGI0, SGI262038, SGI262039, SGI262040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58761[] = {SGI262041, SGI262042, SGI262043, SGI0, SGI262044, SGI262045, SGI262046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58762[] = {SGI262047, SGI262048, SGI262049, SGI0, SGI262050, SGI262051, SGI262052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58763[] = {SGI262053, SGI262054, NULL};
const Segment * const SG58764[] = {SGI262055, SGI262056, NULL};
const Segment * const SG58765[] = {SGI262057, SGI262058, NULL};
const Segment * const SG58766[] = {SGI262059, SGI262060, NULL};
const Segment * const SG58767[] = {SGI262061, SGI0, SGI262062, SGI0, SGI262063, SGI262064, SGI262065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58768[] = {SGI262066, SGI0, SGI262067, SGI0, SGI262068, SGI262069, SGI262070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58769[] = {SGI262071, SGI0, SGI262072, SGI0, SGI262073, SGI262074, SGI262075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58770[] = {SGI262076, SGI262076, NULL};
const Segment * const SG58771[] = {SGI262077, SGI262078, NULL};
const Segment * const SG58772[] = {SGI262079, SGI0, SGI262080, SGI0, SGI262081, SGI262082, SGI262083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58773[] = {SGI262084, SGI0, SGI262085, SGI0, SGI262086, SGI262087, SGI262088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58774[] = {SGI262089, SGI0, SGI262090, SGI0, SGI262091, SGI262092, SGI262093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58775[] = {SGI262094, SGI0, SGI262095, SGI0, SGI262096, SGI262097, SGI262098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58776[] = {SGI262099, SGI0, SGI262100, SGI0, SGI262101, SGI262102, SGI262103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58777[] = {SGI262104, SGI0, SGI262105, SGI0, SGI262106, SGI262107, SGI262108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58778[] = {SGI262109, SGI0, SGI262110, SGI0, SGI262111, SGI262112, SGI262113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58779[] = {SGI262114, SGI0, SGI262115, SGI0, SGI262116, SGI262117, SGI262118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58780[] = {SGI262119, SGI0, SGI262120, SGI0, SGI262121, SGI262122, SGI262123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58781[] = {SGI262124, SGI262125, SGI262126, SGI0, SGI262127, SGI262128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58782[] = {SGI262129, SGI262130, SGI262131, SGI0, SGI262132, SGI262133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58783[] = {SGI262134, SGI262135, SGI262136, SGI0, SGI262137, SGI262138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58784[] = {SGI262139, SGI262140, SGI262141, SGI0, SGI262142, SGI262143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58785[] = {SGI262144, SGI262145, SGI262146, SGI0, SGI262147, SGI262148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58786[] = {SGI262149, SGI262150, SGI262151, SGI0, SGI262152, SGI262153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58787[] = {SGI262154, SGI262155, SGI262156, SGI0, SGI262157, SGI262158, SGI262159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58788[] = {SGI262160, SGI262161, SGI262162, SGI0, SGI262163, SGI262164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58789[] = {SGI262165, SGI262166, SGI262167, SGI0, SGI262168, SGI262169, SGI262170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58790[] = {SGI262171, SGI262172, SGI262173, SGI0, SGI262174, SGI262175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58791[] = {SGI262176, SGI262177, SGI262178, SGI0, SGI262179, SGI262180, SGI262181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58792[] = {SGI262182, SGI262183, SGI262184, SGI0, SGI262185, SGI262186, SGI262187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58793[] = {SGI262188, SGI262189, SGI262190, SGI0, SGI262191, SGI262192, SGI262193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58794[] = {SGI262194, SGI0, SGI262195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58795[] = {SGI262196, SGI0, SGI262197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58796[] = {SGI262198, SGI0, SGI262199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58797[] = {SGI262200, SGI0, SGI262201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58798[] = {SGI262202, SGI0, SGI262203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58799[] = {SGI262204, SGI0, SGI262205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58800[] = {SGI262206, SGI262207, SGI262208, SGI0, SGI262209, SGI262210, SGI262211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58801[] = {SGI262212, SGI262213, SGI262214, SGI0, SGI262215, SGI262216, SGI262217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58802[] = {SGI262218, SGI0, SGI262219, SGI0, SGI262220, SGI262221, SGI262222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58803[] = {SGI262223, SGI0, SGI262224, SGI0, SGI262225, SGI262226, SGI262227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58804[] = {SGI262228, SGI0, SGI262229, SGI0, SGI262230, SGI262231, SGI262232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58805[] = {SGI262233, SGI0, SGI262234, SGI0, SGI262235, SGI262236, SGI262237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58806[] = {SGI262238, SGI0, SGI262239, SGI0, SGI262240, SGI262241, SGI262242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58807[] = {SGI262243, SGI262244, NULL};
const Segment * const SG58808[] = {SGI262245, SGI0, SGI262246, SGI0, SGI262247, SGI262248, SGI262249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58809[] = {SGI262250, SGI0, SGI262251, SGI0, SGI262252, SGI262253, SGI262254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58810[] = {SGI262255, SGI0, SGI262256, SGI0, SGI262257, SGI262258, SGI262259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58811[] = {SGI262260, SGI0, SGI262261, SGI0, SGI262262, SGI262263, SGI262264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58812[] = {SGI262265, SGI0, SGI262266, SGI0, SGI262267, SGI262268, SGI262269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58813[] = {SGI262270, SGI262271, SGI262272, SGI0, SGI262273, SGI262274, SGI262275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58814[] = {SGI262276, SGI262277, SGI0, SGI0, SGI262278, SGI262279, SGI262280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58815[] = {SGI262281, SGI262282, SGI262283, SGI0, SGI262284, SGI262285, SGI262286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58816[] = {SGI262287, SGI262288, SGI262289, SGI0, SGI262290, SGI262291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58817[] = {SGI262292, SGI262293, SGI262294, SGI0, SGI262295, SGI262296, SGI262297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58818[] = {SGI262298, SGI262299, SGI262300, SGI0, SGI262301, SGI262302, SGI262303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58819[] = {SGI262304, SGI0, SGI262305, SGI0, SGI262306, SGI262307, SGI262308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58820[] = {SGI262309, SGI0, SGI262310, SGI0, SGI262311, SGI262312, SGI262313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58821[] = {SGI262314, SGI0, SGI262315, SGI0, SGI262316, SGI262317, SGI262318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58822[] = {SGI262319, SGI0, SGI262320, SGI0, SGI262321, SGI262322, SGI262323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58823[] = {SGI262324, SGI0, SGI262325, SGI0, SGI262326, SGI262327, SGI262328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58824[] = {SGI262329, SGI0, SGI262330, SGI0, SGI262331, SGI262332, SGI262333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58825[] = {SGI262334, SGI0, SGI262335, SGI0, SGI262336, SGI262337, SGI262338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58826[] = {SGI262339, SGI0, SGI262340, SGI0, SGI262341, SGI262342, SGI262343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58827[] = {SGI262344, SGI0, SGI262345, SGI0, SGI262346, SGI262347, SGI262348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58828[] = {SGI262349, SGI0, SGI262350, SGI0, SGI262351, SGI262352, SGI262353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58829[] = {SGI262354, SGI0, SGI262355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58830[] = {SGI262356, SGI0, SGI262357, SGI0, SGI262358, SGI262359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI262360, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58831[] = {SGI262361, SGI0, SGI262362, SGI0, SGI262363, SGI262364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI262365, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58832[] = {SGI262366, SGI0, SGI262367, SGI0, SGI262368, SGI262369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI262370, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58833[] = {SGI262371, SGI262372, SGI0, SGI0, NULL};
const Segment * const SG58834[] = {SGI262373, SGI262374, NULL};
const Segment * const SG58835[] = {SGI262375, SGI262376, NULL};
const Segment * const SG58836[] = {SGI262377, SGI262378, NULL};
const Segment * const SG58837[] = {SGI262379, SGI262380, NULL};
const Segment * const SG58838[] = {SGI262381, SGI262382, NULL};
const Segment * const SG58839[] = {SGI262383, SGI262384, NULL};
const Segment * const SG58840[] = {SGI262385, SGI262386, NULL};
const Segment * const SG58841[] = {SGI262387, SGI262388, NULL};
const Segment * const SG58842[] = {SGI262389, SGI262390, NULL};
const Segment * const SG58843[] = {SGI262391, SGI262392, NULL};
const Segment * const SG58844[] = {SGI262393, SGI262394, NULL};
const Segment * const SG58845[] = {SGI262395, SGI262396, NULL};
const Segment * const SG58846[] = {SGI262397, SGI262398, NULL};
const Segment * const SG58847[] = {SGI262399, SGI262400, NULL};
const Segment * const SG58848[] = {SGI262401, SGI262402, NULL};
const Segment * const SG58849[] = {SGI262403, SGI262404, NULL};
const Segment * const SG58850[] = {SGI262405, SGI262406, NULL};
const Segment * const SG58851[] = {SGI262407, SGI262408, NULL};
const Segment * const SG58852[] = {SGI262409, SGI262410, NULL};
const Segment * const SG58853[] = {SGI262411, SGI262412, NULL};
const Segment * const SG58854[] = {SGI262413, SGI262414, NULL};
const Segment * const SG58855[] = {SGI262415, SGI262416, NULL};
const Segment * const SG58856[] = {SGI262417, SGI262418, NULL};
const Segment * const SG58857[] = {SGI262419, SGI262420, NULL};
const Segment * const SG58858[] = {SGI262421, SGI262422, NULL};
const Segment * const SG58859[] = {SGI262423, SGI262424, NULL};
const Segment * const SG58860[] = {SGI262425, SGI262426, NULL};
const Segment * const SG58861[] = {SGI262427, SGI262428, NULL};
const Segment * const SG58862[] = {SGI262429, SGI262430, NULL};
const Segment * const SG58863[] = {SGI262431, SGI262432, NULL};
const Segment * const SG58864[] = {SGI262433, SGI262434, NULL};
const Segment * const SG58865[] = {SGI262435, SGI262436, NULL};
const Segment * const SG58866[] = {SGI262437, SGI262438, NULL};
const Segment * const SG58867[] = {SGI262439, SGI262440, NULL};
const Segment * const SG58868[] = {SGI262441, SGI262442, NULL};
const Segment * const SG58869[] = {SGI262443, SGI262444, NULL};
const Segment * const SG58870[] = {SGI262445, SGI262446, NULL};
const Segment * const SG58871[] = {SGI262447, SGI262448, NULL};
const Segment * const SG58872[] = {SGI262449, SGI262450, NULL};
const Segment * const SG58873[] = {SGI262451, SGI0, SGI262452, SGI0, SGI262453, SGI262454, SGI262455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58874[] = {SGI262456, SGI0, SGI262457, SGI0, SGI262458, SGI262459, SGI262460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58875[] = {SGI262461, SGI0, SGI262462, SGI0, SGI262463, SGI262464, SGI262465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58876[] = {SGI262466, SGI262467, SGI262468, SGI0, SGI262469, SGI262470, SGI262471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58877[] = {SGI262472, SGI262473, SGI262474, SGI0, SGI262475, SGI262476, SGI262477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58878[] = {SGI262478, SGI262479, SGI262480, SGI0, SGI262481, SGI262482, SGI262483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58879[] = {SGI262484, SGI262485, NULL};
const Segment * const SG58880[] = {SGI262486, SGI0, SGI262487, SGI0, SGI262488, SGI262489, SGI262490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58881[] = {SGI262491, SGI0, SGI262492, SGI0, SGI262493, SGI262494, SGI262495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58882[] = {SGI262496, SGI0, SGI262497, SGI0, SGI262498, SGI262499, SGI262500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58883[] = {SGI262501, SGI262502, SGI0, SGI0, NULL};
const Segment * const SG58884[] = {SGI262503, SGI262504, SGI0, SGI0, NULL};
const Segment * const SG58885[] = {SGI262505, SGI262506, SGI262507, SGI0, SGI262508, SGI262509, SGI262510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58886[] = {SGI262511, SGI262512, SGI262513, SGI0, SGI262514, SGI262515, SGI262516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58887[] = {SGI262517, SGI262518, SGI262519, SGI0, SGI262520, SGI262521, SGI262522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58888[] = {SGI262523, SGI262524, SGI262525, SGI0, SGI262526, SGI262527, SGI262528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58889[] = {SGI262529, SGI0, SGI262530, SGI0, SGI262531, SGI262532, SGI262533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58890[] = {SGI262534, SGI0, SGI262535, SGI0, SGI262536, SGI262537, SGI262538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58891[] = {SGI262539, SGI0, SGI262540, SGI0, SGI262541, SGI262542, SGI262543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58892[] = {SGI262544, SGI0, SGI262545, SGI0, SGI262546, SGI262547, SGI262548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58893[] = {SGI262549, SGI262550, SGI262551, SGI0, SGI262552, SGI262553, SGI262554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58894[] = {SGI262555, SGI0, SGI262556, SGI0, SGI262557, SGI262558, SGI262559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58895[] = {SGI262560, SGI0, SGI262561, SGI0, SGI262562, SGI262563, SGI262564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58896[] = {SGI262565, SGI0, SGI262566, SGI0, SGI262567, SGI262568, SGI262569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58897[] = {SGI262570, SGI262571, SGI262572, SGI0, SGI262573, SGI262574, SGI262575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58898[] = {SGI262576, SGI262577, SGI262578, SGI0, SGI262579, SGI262580, SGI262581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58899[] = {SGI262582, SGI0, SGI262583, SGI0, SGI262584, SGI262585, SGI262586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58900[] = {SGI262587, SGI262588, SGI262589, SGI0, SGI262590, SGI262591, SGI262592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58901[] = {SGI262593, SGI262594, SGI262595, SGI0, SGI262596, SGI262597, SGI262598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58902[] = {SGI262599, SGI262600, SGI262601, SGI0, SGI262602, SGI262603, SGI262604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58903[] = {SGI262605, SGI0, SGI262606, SGI0, SGI262607, SGI262608, SGI262609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58904[] = {SGI262610, SGI262611, SGI262612, SGI0, SGI262613, SGI262614, SGI262615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58905[] = {SGI262616, SGI262617, SGI262618, SGI0, SGI262619, SGI262620, SGI262621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58906[] = {SGI262622, SGI0, SGI262623, SGI0, SGI0, SGI262624, SGI262625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58907[] = {SGI262626, SGI262627, SGI262628, SGI0, SGI262629, SGI262630, SGI262631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58908[] = {SGI262632, SGI0, SGI262633, SGI0, SGI262634, SGI262635, SGI262636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58909[] = {SGI262637, SGI0, SGI262638, SGI0, SGI262639, SGI262640, SGI262641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58910[] = {SGI262642, SGI262643, NULL};
const Segment * const SG58911[] = {SGI262644, SGI262645, NULL};
const Segment * const SG58912[] = {SGI262646, SGI0, SGI262647, SGI0, SGI262648, SGI262649, SGI0, SGI0, SGI0, SGI262650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58913[] = {SGI262651, SGI0, SGI262652, SGI0, SGI262653, SGI262654, SGI0, SGI0, SGI0, SGI262655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58914[] = {SGI262656, SGI0, SGI262657, SGI0, SGI262658, SGI262659, SGI0, SGI0, SGI0, SGI262660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58915[] = {SGI262661, SGI0, SGI262662, SGI0, SGI262663, SGI262664, SGI262665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58916[] = {SGI262666, SGI0, SGI262667, SGI0, SGI262668, SGI262669, SGI262670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58917[] = {SGI262671, SGI262672, SGI262673, SGI0, SGI262674, SGI262675, SGI262676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58918[] = {SGI262677, SGI262678, SGI262679, SGI0, SGI262680, SGI262681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58919[] = {SGI262682, SGI262683, SGI262684, SGI0, SGI262685, SGI262686, SGI262687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58920[] = {SGI262688, SGI0, SGI262689, SGI0, SGI262690, SGI262691, SGI262692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58921[] = {SGI262693, SGI262694, NULL};
const Segment * const SG58922[] = {SGI262695, SGI262696, SGI262697, SGI0, SGI262698, SGI262699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58923[] = {SGI262700, SGI262701, SGI262702, SGI0, SGI262703, SGI262704, SGI262705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58924[] = {SGI262706, SGI262707, SGI262708, SGI0, SGI262709, SGI262710, SGI262711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58925[] = {SGI262712, SGI0, SGI262713, SGI0, SGI262714, SGI262715, SGI262716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58926[] = {SGI262717, SGI0, SGI262718, SGI0, SGI262719, SGI262720, SGI262721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58927[] = {SGI262722, SGI262723, SGI262724, SGI0, SGI262725, SGI262726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58928[] = {SGI262727, SGI262728, SGI262729, SGI0, SGI262730, SGI262731, SGI262732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58929[] = {SGI262733, SGI0, SGI262734, SGI0, SGI262735, SGI262736, SGI262737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58930[] = {SGI262738, SGI262739, SGI262740, SGI0, SGI262741, SGI262742, SGI262743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58931[] = {SGI262744, SGI0, SGI262745, SGI0, SGI262746, SGI262747, SGI262748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58932[] = {SGI262749, SGI262750, NULL};
const Segment * const SG58933[] = {SGI262751, SGI0, SGI262752, SGI0, SGI262753, SGI262754, SGI0, SGI0, SGI0, SGI262755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58934[] = {SGI262756, SGI0, SGI262757, SGI0, SGI262758, SGI262759, SGI0, SGI0, SGI0, SGI262760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58935[] = {SGI262761, SGI262762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58936[] = {SGI262763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58937[] = {SGI262764, SGI262765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58938[] = {SGI262766, SGI0, SGI262767, SGI0, SGI262768, SGI262769, SGI262770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58939[] = {SGI262771, SGI0, SGI262772, SGI0, SGI262773, SGI262774, SGI262775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58940[] = {SGI262776, SGI262777, NULL};
const Segment * const SG58941[] = {SGI262778, SGI262779, NULL};
const Segment * const SG58942[] = {SGI262780, SGI0, SGI262781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58943[] = {SGI262782, SGI0, SGI262783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58944[] = {SGI262784, SGI262785, SGI262786, SGI0, SGI262787, SGI262788, SGI262789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58945[] = {SGI262790, SGI262791, SGI262792, SGI0, SGI262793, SGI262794, SGI262795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58946[] = {SGI262796, SGI0, SGI262797, SGI0, SGI262798, SGI262799, SGI262800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58947[] = {SGI262801, SGI0, SGI262802, SGI0, SGI262803, SGI262804, SGI262805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58948[] = {SGI262806, SGI0, SGI262807, SGI0, SGI262808, SGI262809, SGI262810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58949[] = {SGI262811, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58950[] = {SGI262812, SGI262813, SGI0, SGI0, NULL};
const Segment * const SG58951[] = {SGI262814, SGI262815, NULL};
const Segment * const SG58952[] = {SGI262816, SGI262817, SGI262818, SGI0, SGI262819, SGI262820, SGI262821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58953[] = {SGI262822, SGI262823, SGI262824, SGI0, SGI262825, SGI262826, SGI262827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58954[] = {SGI262828, SGI262829, SGI262830, SGI0, SGI262831, SGI262832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58955[] = {SGI262833, SGI262834, SGI262835, SGI0, SGI262836, SGI262837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58956[] = {SGI262838, SGI262839, SGI262840, SGI0, SGI262841, SGI262842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58957[] = {SGI262843, SGI262844, SGI262845, SGI0, SGI262846, SGI262847, SGI262848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58958[] = {SGI262849, SGI262850, SGI262851, SGI0, SGI262852, SGI262853, SGI262854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58959[] = {SGI262855, SGI262856, SGI262857, SGI0, SGI262858, SGI262859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58960[] = {SGI262860, SGI262861, SGI262862, SGI0, SGI262863, SGI262864, SGI262865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58961[] = {SGI262866, SGI262867, SGI262868, SGI0, SGI262869, SGI262870, SGI262871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58962[] = {SGI262872, SGI262873, SGI262874, SGI0, SGI262875, SGI262876, SGI262877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58963[] = {SGI262878, SGI262879, SGI262880, SGI0, SGI262881, SGI262882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58964[] = {SGI262883, SGI262884, SGI262885, SGI0, SGI262886, SGI262887, SGI262888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58965[] = {SGI262889, SGI262890, SGI262891, SGI0, SGI262892, SGI262893, SGI262894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58966[] = {SGI262895, SGI262896, SGI262897, SGI0, SGI262898, SGI262899, SGI262900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58967[] = {SGI262901, SGI262902, SGI262903, SGI0, SGI262904, SGI262905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58968[] = {SGI262906, SGI262907, SGI262908, SGI0, SGI262909, SGI262910, SGI262911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58969[] = {SGI262912, SGI0, SGI262913, SGI0, SGI262914, SGI262915, SGI262916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58970[] = {SGI262917, SGI0, SGI262918, SGI0, SGI262919, SGI262920, SGI262921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58971[] = {SGI262922, SGI0, SGI262923, SGI0, SGI262924, SGI262925, SGI262926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58972[] = {SGI262927, SGI0, SGI262928, SGI0, SGI262929, SGI262930, SGI262931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58973[] = {SGI262932, SGI0, SGI262933, SGI0, SGI262934, SGI262935, SGI262936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58974[] = {SGI262937, SGI0, SGI262938, SGI0, SGI262939, SGI262940, SGI262941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58975[] = {SGI262942, SGI0, SGI262943, SGI0, SGI262944, SGI262945, SGI262946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58976[] = {SGI262947, SGI0, SGI262948, SGI0, SGI262949, SGI262950, SGI262951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58977[] = {SGI262952, SGI262953, SGI262954, SGI0, SGI262955, SGI262956, SGI262957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58978[] = {SGI262958, SGI262959, SGI262960, SGI0, SGI262961, SGI262962, SGI262963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58979[] = {SGI262964, SGI262965, SGI262966, SGI0, SGI262967, SGI262968, SGI262969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58980[] = {SGI262970, SGI262971, SGI262972, SGI0, SGI262973, SGI262974, SGI262975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58981[] = {SGI262976, SGI262977, SGI262978, SGI0, SGI262979, SGI262980, SGI262981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58982[] = {SGI262982, SGI262983, SGI262984, SGI0, SGI262985, SGI262986, SGI262987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58983[] = {SGI262988, SGI262989, SGI262990, SGI0, SGI262991, SGI262992, SGI262993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58984[] = {SGI262994, SGI262995, SGI262996, SGI0, SGI262997, SGI262998, SGI262999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58985[] = {SGI263000, SGI263001, SGI263002, SGI0, SGI263003, SGI263004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58986[] = {SGI263005, SGI263006, SGI263007, SGI0, SGI263008, SGI263009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58987[] = {SGI263010, SGI263011, SGI263012, SGI0, SGI263013, SGI263014, SGI263015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58988[] = {SGI263016, SGI263017, SGI263018, SGI0, SGI263019, SGI263020, SGI263021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58989[] = {SGI263022, SGI263023, SGI263024, SGI0, SGI263025, SGI263026, SGI263027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58990[] = {SGI263028, SGI263029, SGI263030, SGI0, SGI263031, SGI263032, SGI263033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58991[] = {SGI263034, SGI263035, SGI263036, SGI0, SGI263037, SGI263038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58992[] = {SGI263039, SGI263040, SGI263041, SGI0, SGI263042, SGI263043, SGI263044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58993[] = {SGI263045, SGI263046, SGI263047, SGI0, SGI263048, SGI263049, SGI263050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58994[] = {SGI263051, SGI263052, SGI263053, SGI0, SGI263054, SGI263055, SGI263056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58995[] = {SGI263057, SGI263058, SGI263059, SGI0, SGI263060, SGI263061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58996[] = {SGI263062, SGI263063, SGI263064, SGI0, SGI263065, SGI263066, SGI263067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58997[] = {SGI263068, SGI263069, SGI263070, SGI0, SGI263071, SGI263072, SGI263073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58998[] = {SGI263074, SGI263075, SGI263076, SGI0, SGI263077, SGI263078, SGI263079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG58999[] = {SGI263080, SGI263081, SGI263082, SGI0, SGI263083, SGI263084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59000[] = {SGI263085, SGI263086, SGI263087, SGI0, SGI263088, SGI263089, SGI263090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59001[] = {SGI263091, SGI263092, SGI263093, SGI0, SGI263094, SGI263095, SGI263096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59002[] = {SGI263097, SGI263098, SGI263099, SGI0, SGI263100, SGI263101, SGI263102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59003[] = {SGI263103, SGI0, SGI263104, SGI0, SGI263105, SGI263106, SGI263107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59004[] = {SGI263108, SGI0, SGI263109, SGI0, SGI263110, SGI263111, SGI263112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59005[] = {SGI263113, SGI0, SGI263114, SGI0, SGI263115, SGI263116, SGI263117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59006[] = {SGI263118, SGI0, SGI263119, SGI0, SGI263120, SGI263121, SGI263122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59007[] = {SGI263123, SGI263124, NULL};
const Segment * const SG59008[] = {SGI263125, SGI263126, SGI0, SGI0, NULL};
const Segment * const SG59009[] = {SGI263127, NULL};
const Segment * const SG59010[] = {SGI263128, SGI0, SGI263129, SGI0, SGI263130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59011[] = {SGI263131, SGI0, SGI263132, SGI0, SGI263133, SGI263134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59012[] = {SGI263135, SGI0, SGI263136, SGI0, SGI263137, SGI263138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59013[] = {SGI263139, SGI0, SGI263140, SGI0, SGI263141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59014[] = {SGI263142, SGI0, SGI263143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59015[] = {SGI263144, SGI0, SGI263145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59016[] = {SGI263146, SGI0, SGI263147, SGI0, SGI263148, SGI263149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59017[] = {SGI263150, SGI263151, SGI0, SGI0, NULL};
const Segment * const SG59018[] = {SGI263152, SGI263153, SGI0, SGI0, NULL};
const Segment * const SG59019[] = {SGI263154, SGI0, SGI263155, SGI0, SGI263156, SGI263157, SGI263158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59020[] = {SGI263159, SGI0, SGI263160, SGI0, SGI263161, SGI263162, SGI263163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59021[] = {SGI263164, SGI0, SGI263165, SGI0, SGI263166, SGI263167, SGI263168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59022[] = {SGI263169, SGI0, SGI263170, SGI0, SGI263171, SGI263172, SGI263173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59023[] = {SGI263174, SGI0, SGI263175, SGI0, SGI263176, SGI263177, SGI263178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59024[] = {SGI263179, SGI0, SGI263180, SGI0, SGI263181, SGI263182, SGI263183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59025[] = {SGI263184, SGI0, SGI263185, SGI0, SGI263186, SGI263187, SGI263188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59026[] = {SGI263189, SGI0, SGI263190, SGI0, SGI263191, SGI263192, SGI263193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59027[] = {SGI263194, SGI0, SGI263195, SGI0, SGI263196, SGI263197, SGI263198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59028[] = {SGI263199, SGI0, SGI263200, SGI0, SGI263201, SGI263202, SGI263203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59029[] = {SGI263204, SGI0, SGI263205, SGI0, SGI263206, SGI263207, SGI263208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59030[] = {SGI263209, SGI263210, SGI263211, SGI0, SGI263212, SGI263213, SGI263214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59031[] = {SGI263215, SGI263216, SGI263217, SGI0, SGI263218, SGI263219, SGI263220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59032[] = {SGI263221, SGI263222, SGI263223, SGI0, SGI263224, SGI263225, SGI263226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59033[] = {SGI263227, SGI0, SGI263228, SGI0, SGI263229, SGI263230, SGI263231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59034[] = {SGI263232, SGI0, SGI263233, SGI0, SGI263234, SGI263235, SGI263236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59035[] = {SGI263237, SGI263238, SGI263239, SGI0, SGI263240, SGI263241, SGI263242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59036[] = {SGI263243, SGI0, SGI263244, SGI0, SGI263245, SGI263246, SGI0, SGI0, SGI0, SGI263247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59037[] = {SGI263248, SGI0, SGI263249, SGI0, SGI263250, SGI263251, SGI0, SGI0, SGI0, SGI263252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59038[] = {SGI263253, SGI0, SGI263254, SGI0, SGI263255, SGI263256, SGI0, SGI0, SGI0, SGI263257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59039[] = {SGI263258, SGI0, SGI263259, SGI0, SGI263260, SGI263261, SGI0, SGI0, SGI0, SGI263262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59040[] = {SGI263263, SGI0, SGI263264, SGI0, SGI263265, SGI263266, SGI0, SGI0, SGI0, SGI263267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59041[] = {SGI263268, SGI0, SGI263269, SGI0, SGI263270, SGI263271, SGI0, SGI0, SGI0, SGI263272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59042[] = {SGI263273, SGI0, SGI263274, SGI0, SGI263275, SGI263276, SGI263277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59043[] = {SGI263278, SGI0, SGI263279, SGI0, SGI263280, SGI263281, SGI263282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59044[] = {SGI263283, SGI0, SGI263284, SGI0, SGI263285, SGI263286, SGI263287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59045[] = {SGI263288, SGI0, SGI263289, SGI0, SGI263290, SGI263291, SGI263292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59046[] = {SGI263293, SGI0, SGI263294, SGI0, SGI263295, SGI263296, SGI0, SGI0, SGI0, SGI263297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59047[] = {SGI263298, SGI0, SGI263299, SGI0, SGI263300, SGI263301, SGI0, SGI0, SGI0, SGI263302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59048[] = {SGI263303, SGI0, SGI263304, SGI0, SGI263305, SGI263306, SGI0, SGI0, SGI0, SGI263307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59049[] = {SGI263308, SGI0, SGI263309, SGI0, SGI263310, SGI263311, SGI0, SGI0, SGI0, SGI263312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59050[] = {SGI263313, SGI0, SGI263314, SGI0, SGI263315, SGI263316, SGI0, SGI0, SGI0, SGI263317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59051[] = {SGI263318, SGI0, SGI263319, SGI0, SGI263320, SGI263321, SGI263322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59052[] = {SGI263323, SGI0, SGI263324, SGI0, SGI263325, SGI263326, SGI263327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59053[] = {SGI263328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59054[] = {SGI263329, SGI0, SGI0, SGI0, SGI0, SGI263330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59055[] = {SGI263331, SGI263332, SGI0, SGI0, NULL};
const Segment * const SG59056[] = {SGI263333, SGI0, SGI263334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59057[] = {SGI263335, SGI0, SGI263336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59058[] = {SGI263337, SGI0, SGI263338, SGI0, SGI263339, SGI263340, SGI263341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59059[] = {SGI263342, SGI0, SGI263343, SGI0, SGI263344, SGI263345, SGI263346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59060[] = {SGI263347, SGI0, SGI263348, SGI0, SGI263349, SGI263350, SGI263351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59061[] = {SGI263352, SGI0, SGI263353, SGI0, SGI263354, SGI263355, SGI263356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59062[] = {SGI263357, SGI0, SGI263358, SGI0, SGI263359, SGI263360, SGI263361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59063[] = {SGI263362, SGI0, SGI263363, SGI0, SGI263364, SGI263365, SGI263366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59064[] = {SGI263367, SGI0, SGI263368, SGI0, SGI263369, SGI0, SGI263370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263371, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59065[] = {SGI263372, SGI263373, SGI0, SGI0, SGI263374, SGI0, SGI263374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59066[] = {SGI263375, SGI263376, SGI0, SGI0, SGI263377, SGI0, SGI263377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59067[] = {SGI263378, SGI263379, SGI0, SGI0, SGI263380, SGI0, SGI263380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59068[] = {SGI263381, SGI0, SGI263382, SGI0, SGI263383, SGI263384, SGI263385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59069[] = {SGI263386, SGI0, SGI263387, SGI0, SGI263388, SGI263389, SGI263390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59070[] = {SGI263391, SGI0, SGI263392, SGI0, SGI263393, SGI263394, SGI263395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59071[] = {SGI263396, SGI0, SGI263397, SGI0, SGI263398, SGI263399, SGI263400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59072[] = {SGI263401, SGI0, SGI263402, SGI0, SGI263403, SGI263404, SGI263405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59073[] = {SGI263406, SGI0, SGI263407, SGI0, SGI263408, SGI263409, SGI263410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59074[] = {SGI263411, SGI263412, NULL};
const Segment * const SG59075[] = {SGI263413, SGI0, SGI263414, SGI0, SGI263415, SGI263416, SGI263417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59076[] = {SGI263418, SGI0, SGI263419, SGI0, SGI263420, SGI263421, SGI263422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59077[] = {SGI263423, SGI0, SGI263424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59078[] = {SGI263425, SGI0, SGI263426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59079[] = {SGI263427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59080[] = {SGI263428, SGI0, SGI263429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59081[] = {SGI263430, SGI263431, NULL};
const Segment * const SG59082[] = {SGI263432, SGI263433, NULL};
const Segment * const SG59083[] = {SGI263434, SGI263435, NULL};
const Segment * const SG59084[] = {SGI263436, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59085[] = {SGI263437, SGI263437, NULL};
const Segment * const SG59086[] = {SGI263438, SGI263438, NULL};
const Segment * const SG59087[] = {SGI263439, SGI263440, SGI263441, SGI0, SGI263442, SGI263443, SGI263444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59088[] = {SGI263445, SGI263446, SGI263447, SGI0, SGI263448, SGI263449, SGI263450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59089[] = {SGI263451, SGI263452, SGI263453, SGI0, SGI263454, SGI263455, SGI263456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59090[] = {SGI263457, SGI263458, SGI263459, SGI0, SGI263460, SGI263461, SGI263462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59091[] = {SGI263463, SGI0, SGI263464, SGI0, SGI263465, SGI263466, SGI263467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263468, SGI0, NULL};
const Segment * const SG59092[] = {SGI263469, SGI0, SGI263470, SGI0, SGI263471, SGI263472, SGI263473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263474, SGI0, NULL};
const Segment * const SG59093[] = {SGI263475, SGI0, SGI263476, SGI0, SGI263477, SGI263478, SGI263479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263480, SGI0, NULL};
const Segment * const SG59094[] = {SGI263481, SGI0, SGI263482, SGI0, SGI263483, SGI263484, SGI263485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263486, SGI0, NULL};
const Segment * const SG59095[] = {SGI263487, SGI0, SGI263488, SGI0, SGI263489, SGI263490, SGI263491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263492, SGI0, NULL};
const Segment * const SG59096[] = {SGI263493, SGI0, SGI263494, SGI0, SGI263495, SGI263496, SGI263497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263498, SGI0, NULL};
const Segment * const SG59097[] = {SGI263499, SGI0, SGI263500, SGI0, SGI263501, SGI263502, SGI263503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263504, SGI0, NULL};
const Segment * const SG59098[] = {SGI263505, SGI0, SGI263506, SGI0, SGI263507, SGI263508, SGI263509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263510, SGI0, NULL};
const Segment * const SG59099[] = {SGI263511, SGI0, SGI263512, SGI0, SGI263513, SGI263514, SGI263515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263516, SGI0, NULL};
const Segment * const SG59100[] = {SGI263517, SGI0, SGI263518, SGI0, SGI263519, SGI263520, SGI263521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263522, SGI0, NULL};
const Segment * const SG59101[] = {SGI263523, SGI0, SGI263524, SGI0, SGI263525, SGI263526, SGI263527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263528, SGI0, NULL};
const Segment * const SG59102[] = {SGI263529, SGI0, SGI263530, SGI0, SGI263531, SGI263532, SGI263533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263534, SGI0, NULL};
const Segment * const SG59103[] = {SGI263535, SGI0, SGI263536, SGI0, SGI263537, SGI263538, SGI263539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263540, SGI0, NULL};
const Segment * const SG59104[] = {SGI263541, SGI0, SGI263542, SGI0, SGI263543, SGI263544, SGI263545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263546, SGI0, NULL};
const Segment * const SG59105[] = {SGI263547, SGI0, SGI263548, SGI0, SGI263549, SGI263550, SGI263551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263552, SGI0, NULL};
const Segment * const SG59106[] = {SGI263553, SGI0, SGI263554, SGI0, SGI263555, SGI263556, SGI263557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263558, SGI0, NULL};
const Segment * const SG59107[] = {SGI263559, SGI0, SGI263560, SGI0, SGI263561, SGI263562, SGI263563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263564, SGI0, NULL};
const Segment * const SG59108[] = {SGI263565, SGI0, SGI263566, SGI0, SGI263567, SGI263568, SGI263569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263570, SGI0, NULL};
const Segment * const SG59109[] = {SGI263571, SGI0, SGI263572, SGI0, SGI263573, SGI263574, SGI263575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263576, SGI0, NULL};
const Segment * const SG59110[] = {SGI263577, SGI0, SGI263578, SGI0, SGI263579, SGI263580, SGI263581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263582, SGI0, NULL};
const Segment * const SG59111[] = {SGI263583, SGI0, SGI263584, SGI0, SGI263585, SGI263586, SGI263587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263588, SGI0, NULL};
const Segment * const SG59112[] = {SGI263589, SGI263590, SGI263591, SGI0, SGI263592, SGI263593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59113[] = {SGI263594, SGI263595, SGI263596, SGI0, SGI263597, SGI263598, SGI263599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59114[] = {SGI263600, SGI263601, SGI263602, SGI0, SGI263603, SGI263604, SGI263605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59115[] = {SGI263606, SGI263607, SGI263608, SGI0, SGI263609, SGI263610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59116[] = {SGI263611, SGI263612, SGI263613, SGI0, SGI263614, SGI263615, SGI263616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59117[] = {SGI263617, SGI263618, SGI263619, SGI0, SGI263620, SGI263621, SGI263622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59118[] = {SGI263623, SGI0, SGI263624, SGI0, SGI263625, SGI263626, SGI263627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59119[] = {SGI263628, SGI263629, SGI263630, SGI0, SGI263631, SGI263632, SGI263633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59120[] = {SGI263634, SGI0, SGI263635, SGI0, SGI263636, SGI263637, SGI263638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59121[] = {SGI263639, SGI0, SGI263640, SGI0, SGI263641, SGI263642, SGI263643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59122[] = {SGI263644, SGI263645, SGI263646, SGI0, SGI263647, SGI263648, SGI263649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59123[] = {SGI263650, SGI263651, NULL};
const Segment * const SG59124[] = {SGI263652, SGI263653, NULL};
const Segment * const SG59125[] = {SGI263654, SGI0, SGI263655, SGI0, SGI263656, SGI263657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59126[] = {SGI263658, SGI0, SGI263659, SGI0, SGI263660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59127[] = {SGI263661, SGI0, SGI263662, SGI0, SGI263663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59128[] = {SGI263664, SGI0, SGI263665, SGI0, SGI263666, SGI263667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59129[] = {SGI263668, SGI0, SGI263669, SGI0, SGI263670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59130[] = {SGI263671, SGI0, SGI263672, SGI0, SGI263673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59131[] = {SGI263674, SGI0, SGI263675, SGI0, SGI263676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59132[] = {SGI263677, SGI263678, SGI263679, SGI0, SGI263680, SGI263681, SGI263682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59133[] = {SGI263683, SGI263684, SGI263685, SGI0, SGI263686, SGI263687, SGI263688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59134[] = {SGI263689, SGI263690, SGI263691, SGI0, SGI263692, SGI263693, SGI263694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59135[] = {SGI263695, SGI263696, SGI263697, SGI0, SGI263698, SGI263699, SGI263700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59136[] = {SGI263701, SGI263702, SGI263703, SGI0, SGI263704, SGI263705, SGI263706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59137[] = {SGI263707, SGI263708, SGI263709, SGI0, SGI263710, SGI263711, SGI263712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59138[] = {SGI263713, SGI263714, SGI263715, SGI0, SGI263716, SGI263717, SGI263718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59139[] = {SGI263719, SGI263720, SGI263721, SGI0, SGI263722, SGI263723, SGI263724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59140[] = {SGI263725, SGI263726, NULL};
const Segment * const SG59141[] = {SGI263727, SGI0, SGI263728, SGI0, SGI263729, SGI263730, SGI263731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59142[] = {SGI263732, SGI0, SGI263733, SGI0, SGI263734, SGI263735, SGI263736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59143[] = {SGI263737, SGI263738, NULL};
const Segment * const SG59144[] = {SGI263739, SGI0, SGI263740, SGI0, SGI263741, SGI263742, SGI263743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263744, SGI0, NULL};
const Segment * const SG59145[] = {SGI263745, SGI0, SGI263746, SGI0, SGI263747, SGI263748, SGI263749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263750, SGI0, NULL};
const Segment * const SG59146[] = {SGI263751, SGI0, SGI263752, SGI0, SGI263753, SGI263754, SGI263755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263756, SGI0, NULL};
const Segment * const SG59147[] = {SGI263757, SGI0, SGI263758, SGI0, SGI263759, SGI263760, SGI263761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263762, SGI0, NULL};
const Segment * const SG59148[] = {SGI263763, SGI0, SGI263764, SGI0, SGI263765, SGI263766, SGI263767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263768, SGI0, NULL};
const Segment * const SG59149[] = {SGI263769, SGI0, SGI263770, SGI0, SGI263771, SGI263772, SGI263773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263774, SGI0, NULL};
const Segment * const SG59150[] = {SGI263775, SGI0, SGI263776, SGI0, SGI263777, SGI263778, SGI263779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263780, SGI0, NULL};
const Segment * const SG59151[] = {SGI263781, SGI0, SGI263782, SGI0, SGI263783, SGI263784, SGI263785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59152[] = {SGI263786, SGI0, SGI263787, SGI0, SGI263788, SGI263789, SGI263790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59153[] = {SGI263791, SGI0, SGI263792, SGI0, SGI263793, SGI263794, SGI263795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59154[] = {SGI263796, SGI0, SGI263797, SGI0, SGI263798, SGI263799, SGI263800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59155[] = {SGI263801, SGI0, SGI263802, SGI0, SGI263803, SGI263804, SGI263805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59156[] = {SGI263806, SGI0, SGI263807, SGI0, SGI263808, SGI263809, SGI263810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59157[] = {SGI263811, SGI263812, NULL};
const Segment * const SG59158[] = {SGI263813, SGI263814, NULL};
const Segment * const SG59159[] = {SGI263815, SGI263816, NULL};
const Segment * const SG59160[] = {SGI263817, SGI263818, NULL};
const Segment * const SG59161[] = {SGI263819, SGI263820, NULL};
const Segment * const SG59162[] = {SGI263821, SGI263822, NULL};
const Segment * const SG59163[] = {SGI263823, SGI263824, NULL};
const Segment * const SG59164[] = {SGI263825, SGI263826, NULL};
const Segment * const SG59165[] = {SGI263827, SGI263828, NULL};
const Segment * const SG59166[] = {SGI263829, SGI263830, NULL};
const Segment * const SG59167[] = {SGI263831, SGI263832, NULL};
const Segment * const SG59168[] = {SGI263833, SGI263834, NULL};
const Segment * const SG59169[] = {SGI263835, SGI263836, NULL};
const Segment * const SG59170[] = {SGI263837, SGI263838, NULL};
const Segment * const SG59171[] = {SGI263839, SGI263840, NULL};
const Segment * const SG59172[] = {SGI263841, SGI263842, NULL};
const Segment * const SG59173[] = {SGI263843, SGI263844, NULL};
const Segment * const SG59174[] = {SGI263845, SGI263846, NULL};
const Segment * const SG59175[] = {SGI263847, SGI263848, NULL};
const Segment * const SG59176[] = {SGI263849, SGI263850, NULL};
const Segment * const SG59177[] = {SGI263851, SGI263852, NULL};
const Segment * const SG59178[] = {SGI263853, SGI263854, NULL};
const Segment * const SG59179[] = {SGI263855, SGI263856, NULL};
const Segment * const SG59180[] = {SGI263857, SGI263858, NULL};
const Segment * const SG59181[] = {SGI263859, SGI263860, NULL};
const Segment * const SG59182[] = {SGI263861, SGI263862, NULL};
const Segment * const SG59183[] = {SGI263863, SGI263864, NULL};
const Segment * const SG59184[] = {SGI263865, SGI263866, NULL};
const Segment * const SG59185[] = {SGI263867, SGI263868, NULL};
const Segment * const SG59186[] = {SGI263869, SGI263870, NULL};
const Segment * const SG59187[] = {SGI263871, SGI263872, NULL};
const Segment * const SG59188[] = {SGI263873, SGI263874, NULL};
const Segment * const SG59189[] = {SGI263875, SGI263876, NULL};
const Segment * const SG59190[] = {SGI263877, SGI263878, NULL};
const Segment * const SG59191[] = {SGI263879, SGI263880, NULL};
const Segment * const SG59192[] = {SGI263881, SGI263882, NULL};
const Segment * const SG59193[] = {SGI263883, SGI263884, NULL};
const Segment * const SG59194[] = {SGI263885, SGI263886, NULL};
const Segment * const SG59195[] = {SGI263887, SGI263888, NULL};
const Segment * const SG59196[] = {SGI263889, SGI263890, NULL};
const Segment * const SG59197[] = {SGI263891, SGI263892, NULL};
const Segment * const SG59198[] = {SGI263893, SGI263894, NULL};
const Segment * const SG59199[] = {SGI263895, SGI263896, NULL};
const Segment * const SG59200[] = {SGI263897, SGI263898, NULL};
const Segment * const SG59201[] = {SGI263899, SGI263900, NULL};
const Segment * const SG59202[] = {SGI263901, SGI263902, NULL};
const Segment * const SG59203[] = {SGI263903, SGI263904, NULL};
const Segment * const SG59204[] = {SGI263905, SGI263906, NULL};
const Segment * const SG59205[] = {SGI263907, SGI263908, NULL};
const Segment * const SG59206[] = {SGI263909, SGI263910, NULL};
const Segment * const SG59207[] = {SGI263911, SGI263912, SGI0, SGI0, NULL};
const Segment * const SG59208[] = {SGI263913, SGI263914, SGI263915, SGI0, SGI263916, SGI263917, SGI263916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59209[] = {SGI263918, SGI263919, SGI263920, SGI0, SGI263921, SGI263922, SGI263923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59210[] = {SGI263924, SGI263925, SGI263926, SGI0, SGI263927, SGI263928, SGI263927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59211[] = {SGI263929, SGI263929, NULL};
const Segment * const SG59212[] = {SGI263930, SGI0, SGI263931, SGI0, SGI263932, SGI263933, SGI263934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59213[] = {SGI263935, SGI0, SGI263936, SGI0, SGI263937, SGI263938, SGI263939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59214[] = {SGI263940, SGI263941, SGI263942, SGI0, SGI263943, SGI263944, SGI263943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263945, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59215[] = {SGI263946, SGI263947, SGI263948, SGI0, SGI263949, SGI263950, SGI263949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59216[] = {SGI263951, SGI263952, SGI263953, SGI0, SGI263954, SGI263955, SGI263954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59217[] = {SGI263956, SGI263957, SGI263958, SGI0, SGI263959, SGI263960, SGI263959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59218[] = {SGI263961, SGI0, SGI263962, SGI0, SGI263963, SGI263964, SGI0, SGI0, SGI0, SGI263965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59219[] = {SGI263966, SGI263967, NULL};
const Segment * const SG59220[] = {SGI263968, SGI263969, NULL};
const Segment * const SG59221[] = {SGI263970, SGI263971, NULL};
const Segment * const SG59222[] = {SGI263972, SGI263973, NULL};
const Segment * const SG59223[] = {SGI263974, SGI263975, NULL};
const Segment * const SG59224[] = {SGI263976, SGI263977, NULL};
const Segment * const SG59225[] = {SGI263978, SGI263979, NULL};
const Segment * const SG59226[] = {SGI263980, SGI263981, NULL};
const Segment * const SG59227[] = {SGI263982, SGI263983, NULL};
const Segment * const SG59228[] = {SGI263984, SGI263985, SGI0, SGI0, NULL};
const Segment * const SG59229[] = {SGI263986, SGI0, SGI263987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59230[] = {SGI263988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59231[] = {SGI263989, SGI0, SGI263990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59232[] = {SGI263991, SGI0, SGI263992, SGI0, SGI263993, SGI263994, SGI263995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI263996, SGI0, NULL};
const Segment * const SG59233[] = {SGI263997, SGI0, SGI263998, SGI0, SGI263999, SGI264000, SGI264001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264002, SGI0, NULL};
const Segment * const SG59234[] = {SGI264003, SGI0, SGI264004, SGI0, SGI264005, SGI264006, SGI264007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264008, SGI0, NULL};
const Segment * const SG59235[] = {SGI264009, SGI0, SGI264010, SGI0, SGI264011, SGI264012, SGI264013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264014, SGI0, NULL};
const Segment * const SG59236[] = {SGI264015, SGI0, SGI264016, SGI0, SGI264017, SGI264018, SGI264019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59237[] = {SGI264020, SGI0, SGI264021, SGI0, SGI264022, SGI264023, SGI264024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59238[] = {SGI264025, SGI0, SGI264026, SGI0, SGI264027, SGI264028, SGI264029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59239[] = {SGI264030, SGI0, SGI264031, SGI0, SGI264032, SGI264033, SGI264034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59240[] = {SGI264035, SGI264036, SGI0, SGI0, NULL};
const Segment * const SG59241[] = {SGI264037, SGI0, SGI264038, SGI0, SGI264039, SGI264040, SGI264041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59242[] = {SGI264042, SGI0, SGI264043, SGI0, SGI264044, SGI264045, SGI264046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59243[] = {SGI264047, SGI0, SGI264048, SGI0, SGI264049, SGI264050, SGI264051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59244[] = {SGI264052, SGI0, SGI264053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59245[] = {SGI264054, SGI0, SGI264055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59246[] = {SGI264056, SGI0, SGI264057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59247[] = {SGI264058, SGI0, SGI264059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59248[] = {SGI264060, SGI0, SGI264061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59249[] = {SGI264062, SGI0, SGI264063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59250[] = {SGI264064, SGI0, SGI264065, SGI0, SGI264066, SGI264067, SGI264068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264069, SGI0, NULL};
const Segment * const SG59251[] = {SGI264070, SGI0, SGI264071, SGI0, SGI264072, SGI264073, SGI264074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264075, SGI0, NULL};
const Segment * const SG59252[] = {SGI264076, SGI0, SGI264077, SGI0, SGI264078, SGI264079, SGI264080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264081, SGI0, NULL};
const Segment * const SG59253[] = {SGI264082, SGI0, SGI264083, SGI0, SGI264084, SGI264085, SGI264086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264087, SGI0, NULL};
const Segment * const SG59254[] = {SGI264088, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59255[] = {SGI264089, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59256[] = {SGI264090, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59257[] = {SGI264091, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59258[] = {SGI264092, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59259[] = {SGI264093, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59260[] = {SGI264094, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59261[] = {SGI264095, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59262[] = {SGI264096, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59263[] = {SGI264097, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59264[] = {SGI264098, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59265[] = {SGI264099, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59266[] = {SGI264100, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59267[] = {SGI264101, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59268[] = {SGI264102, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59269[] = {SGI264103, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59270[] = {SGI264104, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59271[] = {SGI264105, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59272[] = {SGI264106, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59273[] = {SGI264107, SGI264108, SGI0, SGI0, NULL};
const Segment * const SG59274[] = {SGI264109, SGI0, SGI264110, SGI0, SGI264111, SGI264112, SGI0, SGI0, SGI0, SGI264113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59275[] = {SGI264114, SGI0, SGI264115, SGI0, SGI264116, SGI0, SGI264117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264118, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59276[] = {SGI264119, SGI264120, SGI264121, SGI0, SGI264122, SGI264123, SGI264124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59277[] = {SGI264125, SGI0, SGI264126, SGI0, SGI264127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59278[] = {SGI264128, SGI0, SGI264129, SGI0, SGI264130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59279[] = {SGI264131, SGI0, SGI264132, SGI0, SGI264133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59280[] = {SGI264134, SGI264135, NULL};
const Segment * const SG59281[] = {SGI264136, SGI264137, NULL};
const Segment * const SG59282[] = {SGI264138, SGI264139, NULL};
const Segment * const SG59283[] = {SGI264140, SGI264141, NULL};
const Segment * const SG59284[] = {SGI264142, SGI264143, NULL};
const Segment * const SG59285[] = {SGI264144, SGI264145, NULL};
const Segment * const SG59286[] = {SGI264146, SGI264147, NULL};
const Segment * const SG59287[] = {SGI264148, SGI264149, NULL};
const Segment * const SG59288[] = {SGI264150, SGI264151, NULL};
const Segment * const SG59289[] = {SGI264152, SGI264153, NULL};
const Segment * const SG59290[] = {SGI264156, SGI264157, NULL};
const Segment * const SG59291[] = {SGI264154, SGI0, SGI264155, SGI0, SGI264158, SGI264159, SGI264160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59292[] = {SGI264161, SGI0, SGI264162, SGI0, SGI264163, SGI264164, SGI264165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59293[] = {SGI264166, SGI0, SGI264167, SGI0, SGI264168, SGI264169, SGI264170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264171, SGI0, NULL};
const Segment * const SG59294[] = {SGI264172, SGI0, SGI264173, SGI0, SGI264174, SGI264175, SGI264176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264177, SGI0, NULL};
const Segment * const SG59295[] = {SGI264178, SGI0, SGI264179, SGI0, SGI264180, SGI264181, SGI264182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264183, SGI0, NULL};
const Segment * const SG59296[] = {SGI264184, SGI0, SGI264185, SGI0, SGI264186, SGI264187, SGI264188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264189, SGI0, NULL};
const Segment * const SG59297[] = {SGI264190, SGI0, SGI264191, SGI0, SGI264192, SGI264193, SGI264194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264195, SGI0, NULL};
const Segment * const SG59298[] = {SGI264196, SGI0, SGI264197, SGI0, SGI264198, SGI264199, SGI264200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264201, SGI0, NULL};
const Segment * const SG59299[] = {SGI264202, SGI0, SGI264203, SGI0, SGI264204, SGI264205, SGI264206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59300[] = {SGI264207, SGI0, SGI264208, SGI0, SGI264209, SGI264210, SGI264211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59301[] = {SGI264212, SGI0, SGI264213, SGI0, SGI264214, SGI264215, SGI264216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59302[] = {SGI264217, SGI0, SGI264218, SGI0, SGI264219, SGI264220, SGI264221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59303[] = {SGI264222, SGI0, SGI264223, SGI0, SGI264224, SGI264225, SGI264226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59304[] = {SGI264227, SGI0, SGI264228, SGI0, SGI264229, SGI264230, SGI264231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59305[] = {SGI264232, SGI0, SGI264233, SGI0, SGI264234, SGI264235, SGI264236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59306[] = {SGI264237, SGI0, SGI264238, SGI0, SGI264239, SGI264240, SGI264241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59307[] = {SGI264242, SGI264243, SGI0, SGI0, NULL};
const Segment * const SG59308[] = {SGI264244, SGI264245, NULL};
const Segment * const SG59309[] = {SGI264246, SGI264247, NULL};
const Segment * const SG59310[] = {SGI264248, SGI264249, NULL};
const Segment * const SG59311[] = {SGI264250, SGI264251, NULL};
const Segment * const SG59312[] = {SGI264252, SGI0, SGI264253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59313[] = {SGI264254, SGI0, SGI264255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59314[] = {SGI264256, SGI0, SGI264257, SGI0, SGI264258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59315[] = {SGI264259, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59316[] = {SGI264260, SGI0, SGI264261, SGI0, SGI264262, SGI264263, SGI0, SGI0, SGI0, SGI264264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59317[] = {SGI264265, SGI0, SGI264266, SGI0, SGI264267, SGI264268, SGI0, SGI0, SGI0, SGI264269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59318[] = {SGI264274, SGI264276, NULL};
const Segment * const SG59319[] = {SGI264270, SGI264271, SGI264272, SGI0, SGI264273, SGI264275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59320[] = {SGI264277, SGI264278, SGI264279, SGI0, SGI264280, SGI264281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59321[] = {SGI264282, SGI264283, SGI264284, SGI0, SGI264285, SGI264286, SGI264287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59322[] = {SGI264288, SGI264289, SGI264290, SGI0, SGI264291, SGI264292, SGI264293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59323[] = {SGI264294, SGI264295, SGI264296, SGI0, SGI264297, SGI264298, SGI264299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59324[] = {SGI264300, SGI264301, SGI264302, SGI0, SGI264303, SGI264304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59325[] = {SGI264305, SGI264306, SGI264307, SGI0, SGI264308, SGI264309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59326[] = {SGI264310, SGI264311, SGI264312, SGI0, SGI264313, SGI264314, SGI264315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59327[] = {SGI264316, SGI264317, SGI264318, SGI0, SGI264319, SGI264320, SGI264321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59328[] = {SGI264322, SGI264323, SGI264324, SGI0, SGI264325, SGI264326, SGI264327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59329[] = {SGI264328, SGI264329, SGI264330, SGI0, SGI264331, SGI264332, SGI264333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59330[] = {SGI264334, SGI0, SGI264335, SGI0, SGI264336, SGI264337, SGI0, SGI0, SGI0, SGI264338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59331[] = {SGI264339, SGI0, SGI264340, SGI0, SGI264341, SGI264342, SGI0, SGI0, SGI0, SGI264343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59332[] = {SGI264344, SGI0, SGI264345, SGI0, SGI264346, SGI264347, SGI0, SGI0, SGI0, SGI264348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59333[] = {SGI264349, SGI0, SGI264350, SGI0, SGI264351, SGI264352, SGI0, SGI0, SGI0, SGI264353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59334[] = {SGI264354, SGI0, SGI264355, SGI0, SGI264356, SGI264357, SGI0, SGI0, SGI0, SGI264358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59335[] = {SGI264359, SGI0, SGI264360, SGI0, SGI264361, SGI264362, SGI264363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59336[] = {SGI264364, SGI0, SGI264365, SGI0, SGI264366, SGI264367, SGI264368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59337[] = {SGI264369, SGI0, SGI264370, SGI0, SGI264371, SGI264372, SGI264373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59338[] = {SGI264374, SGI0, SGI264375, SGI0, SGI264376, SGI264377, SGI264378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59339[] = {SGI264379, SGI0, SGI264380, SGI0, SGI264381, SGI264382, SGI264383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59340[] = {SGI264384, SGI0, SGI264385, SGI0, SGI264386, SGI264387, SGI264388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59341[] = {SGI264389, SGI264390, NULL};
const Segment * const SG59342[] = {SGI264391, SGI264392, NULL};
const Segment * const SG59343[] = {SGI264393, SGI264394, NULL};
const Segment * const SG59344[] = {SGI264395, SGI264396, NULL};
const Segment * const SG59345[] = {SGI264397, SGI264398, NULL};
const Segment * const SG59346[] = {SGI264399, SGI264400, NULL};
const Segment * const SG59347[] = {SGI264401, SGI264402, NULL};
const Segment * const SG59348[] = {SGI264403, SGI0, SGI264404, SGI0, SGI264405, SGI264406, SGI264407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59349[] = {SGI264408, SGI0, SGI264409, SGI0, SGI264410, SGI264411, SGI264412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59350[] = {SGI264413, SGI0, SGI264414, SGI0, SGI264415, SGI264416, SGI264417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59351[] = {SGI264418, SGI0, SGI264419, SGI264420, SGI264421, SGI264422, SGI264423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59352[] = {SGI264424, SGI0, SGI264425, SGI264426, SGI264427, SGI264428, SGI264429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59353[] = {SGI264430, SGI0, SGI264431, SGI264432, SGI264433, SGI264434, SGI264435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59354[] = {SGI264436, SGI0, SGI264437, SGI264438, SGI264439, SGI264440, SGI264441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59355[] = {SGI264442, SGI0, SGI264443, SGI264444, SGI264445, SGI264446, SGI264447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59356[] = {SGI264448, SGI0, SGI264449, SGI264450, SGI264451, SGI264452, SGI264453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59357[] = {SGI264457, SGI264459, SGI0, SGI0, NULL};
const Segment * const SG59358[] = {SGI264454, SGI0, SGI264455, SGI264456, SGI264458, SGI264460, SGI264461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59359[] = {SGI264462, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59360[] = {SGI264463, SGI0, SGI264464, SGI264465, SGI264466, SGI264467, SGI264468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59361[] = {SGI264469, SGI264470, SGI0, SGI0, NULL};
const Segment * const SG59362[] = {SGI264471, SGI0, SGI264472, SGI264473, SGI264474, SGI264475, SGI264476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59363[] = {SGI264477, SGI0, SGI264478, SGI264479, SGI264480, SGI264481, SGI264482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59364[] = {SGI264483, SGI0, SGI264484, SGI264485, SGI264486, SGI264487, SGI264488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59365[] = {SGI264489, SGI0, SGI264490, SGI264491, SGI264492, SGI264493, SGI264494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59366[] = {SGI264495, SGI0, SGI264496, SGI264497, SGI264498, SGI264499, SGI264500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59367[] = {SGI264501, SGI0, SGI264502, SGI264503, SGI264504, SGI264505, SGI264506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59368[] = {SGI264507, SGI0, SGI264508, SGI264509, SGI264510, SGI264511, SGI264512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59369[] = {SGI264513, SGI0, SGI264514, SGI0, SGI264515, SGI264516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264517, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59370[] = {SGI264518, SGI0, SGI264519, SGI0, SGI264520, SGI264521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264522, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59371[] = {SGI264523, SGI0, SGI264524, SGI0, SGI264525, SGI264526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264527, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59372[] = {SGI264528, SGI0, SGI264529, SGI0, SGI264530, SGI264531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264532, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59373[] = {SGI264533, SGI0, SGI264534, SGI0, SGI264535, SGI264536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264537, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59374[] = {SGI264538, SGI264539, NULL};
const Segment * const SG59375[] = {SGI264540, SGI264541, NULL};
const Segment * const SG59376[] = {SGI264542, SGI264543, NULL};
const Segment * const SG59377[] = {SGI264544, SGI264545, NULL};
const Segment * const SG59378[] = {SGI264546, SGI264547, NULL};
const Segment * const SG59379[] = {SGI264548, SGI264549, NULL};
const Segment * const SG59380[] = {SGI264555, SGI264556, NULL};
const Segment * const SG59381[] = {SGI264550, SGI0, SGI264551, SGI0, SGI264552, SGI264553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264554, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59382[] = {SGI264557, SGI0, SGI264558, SGI0, SGI264559, SGI264560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264561, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59383[] = {SGI264566, SGI264568, NULL};
const Segment * const SG59384[] = {SGI264562, SGI0, SGI264563, SGI0, SGI264564, SGI264565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264567, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59385[] = {SGI264569, SGI0, SGI264570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59386[] = {SGI264571, SGI0, SGI264572, SGI0, SGI264573, SGI264574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264575, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59387[] = {SGI264576, SGI0, SGI264577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59388[] = {SGI264578, SGI264579, SGI264580, SGI0, SGI264581, SGI264582, SGI264583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59389[] = {SGI264584, SGI264585, SGI264586, SGI0, SGI264587, SGI264588, SGI264589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59390[] = {SGI264590, SGI264591, SGI264592, SGI0, SGI264593, SGI264594, SGI264595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59391[] = {SGI264596, SGI264597, SGI264598, SGI0, SGI264599, SGI264600, SGI264601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59392[] = {SGI264602, SGI264603, SGI264604, SGI0, SGI264605, SGI264606, SGI264607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59393[] = {SGI264608, SGI264609, SGI264610, SGI0, SGI264611, SGI264612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59394[] = {SGI264613, SGI264614, SGI264615, SGI0, SGI264616, SGI264617, SGI264618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59395[] = {SGI264619, SGI0, SGI264620, SGI0, SGI264621, SGI264622, SGI264623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59396[] = {SGI264624, SGI264625, SGI264626, SGI0, SGI264627, SGI264628, SGI264629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59397[] = {SGI264630, SGI264631, SGI264632, SGI0, SGI264633, SGI264634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59398[] = {SGI264635, SGI264636, SGI264637, SGI0, SGI264638, SGI264639, SGI264640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59399[] = {SGI264641, SGI264642, SGI264643, SGI0, SGI264644, SGI264645, SGI264646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59400[] = {SGI264647, SGI264648, SGI264649, SGI0, SGI264650, SGI264651, SGI264652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59401[] = {SGI264653, SGI0, SGI264654, SGI0, SGI264655, SGI264656, SGI264657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59402[] = {SGI264658, SGI0, SGI264659, SGI0, SGI264660, SGI264661, SGI264662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59403[] = {SGI264663, SGI0, SGI264664, SGI0, SGI264665, SGI264666, SGI264667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59404[] = {SGI264668, SGI0, SGI264669, SGI0, SGI264670, SGI264671, SGI264672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59405[] = {SGI264673, SGI264674, SGI0, SGI0, NULL};
const Segment * const SG59406[] = {SGI264675, SGI264676, NULL};
const Segment * const SG59407[] = {SGI264677, SGI264678, NULL};
const Segment * const SG59408[] = {SGI264679, SGI264680, SGI264681, SGI0, SGI264682, SGI264683, SGI264684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59409[] = {SGI264685, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59410[] = {SGI264686, SGI264687, SGI264688, SGI0, SGI264689, SGI264690, SGI264691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59411[] = {SGI264692, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59412[] = {SGI264693, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59413[] = {SGI264694, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59414[] = {SGI264695, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59415[] = {SGI264696, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59416[] = {SGI264697, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59417[] = {SGI264698, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59418[] = {SGI264699, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59419[] = {SGI264700, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59420[] = {SGI264701, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59421[] = {SGI264702, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59422[] = {SGI264703, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59423[] = {SGI264704, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59424[] = {SGI264705, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59425[] = {SGI264706, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59426[] = {SGI264707, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59427[] = {SGI264708, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59428[] = {SGI264709, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59429[] = {SGI264710, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59430[] = {SGI264711, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59431[] = {SGI264712, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59432[] = {SGI264713, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59433[] = {SGI264714, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59434[] = {SGI264715, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59435[] = {SGI264716, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59436[] = {SGI264717, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59437[] = {SGI264718, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59438[] = {SGI264719, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59439[] = {SGI264720, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59440[] = {SGI264721, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59441[] = {SGI264722, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59442[] = {SGI264723, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59443[] = {SGI264724, SGI0, SGI264725, SGI0, SGI264726, SGI264727, SGI0, SGI0, SGI0, SGI264728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59444[] = {SGI264729, SGI0, SGI264730, SGI0, SGI264731, SGI264732, SGI0, SGI0, SGI0, SGI264733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59445[] = {SGI264734, SGI0, SGI264735, SGI0, SGI264736, SGI264737, SGI0, SGI0, SGI0, SGI264738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59446[] = {SGI264739, SGI0, SGI264740, SGI0, SGI264741, SGI264742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59447[] = {SGI264743, SGI0, SGI264744, SGI0, SGI264745, SGI264746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59448[] = {SGI264747, SGI0, SGI264748, SGI0, SGI264749, SGI264750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59449[] = {SGI264751, SGI0, SGI264752, SGI0, SGI264753, SGI264754, SGI264753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59450[] = {SGI264755, SGI0, SGI264756, SGI0, SGI264757, SGI264758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59451[] = {SGI264759, SGI0, SGI264760, SGI0, SGI264761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59452[] = {SGI264762, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59453[] = {SGI264763, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59454[] = {SGI264764, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59455[] = {SGI264765, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59456[] = {SGI264766, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59457[] = {SGI264767, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59458[] = {SGI264768, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59459[] = {SGI264769, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59460[] = {SGI264770, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59461[] = {SGI264771, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59462[] = {SGI264772, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59463[] = {SGI264773, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59464[] = {SGI264774, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59465[] = {SGI264775, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59466[] = {SGI264776, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59467[] = {SGI264777, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59468[] = {SGI264778, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59469[] = {SGI264779, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59470[] = {SGI264780, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59471[] = {SGI264781, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59472[] = {SGI264782, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59473[] = {SGI264783, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59474[] = {SGI264784, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59475[] = {SGI264785, SGI0, SGI264786, SGI0, SGI264787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59476[] = {SGI264788, SGI0, SGI264789, SGI0, SGI264790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59477[] = {SGI264791, SGI264792, SGI264793, SGI0, SGI264794, SGI264795, SGI264796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59478[] = {SGI264797, SGI0, SGI264798, SGI0, SGI264799, SGI264800, SGI264801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59479[] = {SGI264802, SGI0, SGI264803, SGI0, SGI264804, SGI264805, SGI264806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59480[] = {SGI264807, SGI0, SGI264808, SGI0, SGI264809, SGI264810, SGI264811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59481[] = {SGI264812, SGI0, SGI264813, SGI0, SGI264814, SGI264815, SGI264816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59482[] = {SGI264817, SGI0, SGI264818, SGI0, SGI264819, SGI264820, SGI264821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59483[] = {SGI264822, SGI0, SGI264823, SGI0, SGI264824, SGI264825, SGI264826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59484[] = {SGI264827, SGI0, SGI264828, SGI0, SGI264829, SGI264830, SGI264831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59485[] = {SGI264832, SGI0, SGI264833, SGI0, SGI264834, SGI264835, SGI264836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264837, SGI0, NULL};
const Segment * const SG59486[] = {SGI264838, SGI0, SGI264839, SGI0, SGI264840, SGI264841, SGI264842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264843, SGI0, NULL};
const Segment * const SG59487[] = {SGI264844, SGI0, SGI264845, SGI0, SGI264846, SGI264847, SGI264848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264849, SGI0, NULL};
const Segment * const SG59488[] = {SGI264850, SGI0, SGI264851, SGI0, SGI264852, SGI264853, SGI264854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI264855, SGI0, NULL};
const Segment * const SG59489[] = {SGI264856, SGI0, SGI264857, SGI0, SGI264858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59490[] = {SGI264859, SGI0, SGI264860, SGI0, SGI264861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59491[] = {SGI264862, SGI0, SGI264863, SGI0, SGI264864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59492[] = {SGI264865, SGI264866, SGI264867, SGI0, SGI264868, SGI264869, SGI264870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59493[] = {SGI264871, SGI264872, SGI264873, SGI0, SGI264874, SGI264875, SGI264876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59494[] = {SGI264877, SGI264878, SGI264879, SGI0, SGI264880, SGI264881, SGI264882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59495[] = {SGI264883, SGI264884, SGI264885, SGI0, SGI264886, SGI264887, SGI264888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59496[] = {SGI264889, SGI264890, SGI264891, SGI0, SGI264892, SGI264893, SGI264894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59497[] = {SGI264895, SGI264896, SGI264897, SGI0, SGI264898, SGI264899, SGI264900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59498[] = {SGI264901, SGI264902, SGI264903, SGI0, SGI264904, SGI264905, SGI264906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59499[] = {SGI264907, SGI264908, NULL};
const Segment * const SG59500[] = {SGI264909, SGI264910, NULL};
const Segment * const SG59501[] = {SGI264911, SGI0, SGI264912, SGI0, SGI264913, SGI264914, SGI264915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59502[] = {SGI264916, SGI0, SGI264917, SGI0, SGI264918, SGI264919, SGI264920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59503[] = {SGI264921, SGI0, SGI264922, SGI0, SGI264923, SGI264924, SGI264925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59504[] = {SGI264926, SGI0, SGI264927, SGI0, SGI264928, SGI264929, SGI264930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59505[] = {SGI264931, SGI0, SGI264932, SGI0, SGI264933, SGI264934, SGI264935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59506[] = {SGI264936, SGI0, SGI264937, SGI0, SGI264938, SGI264939, SGI264940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59507[] = {SGI264941, SGI0, SGI264942, SGI0, SGI264943, SGI264944, SGI264945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59508[] = {SGI264946, SGI0, SGI264947, SGI0, SGI264948, SGI264949, SGI264950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59509[] = {SGI264951, SGI0, SGI264952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59510[] = {SGI264953, SGI0, SGI264954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59511[] = {SGI264955, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59512[] = {SGI264956, SGI0, SGI264957, SGI0, SGI264958, SGI264959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59513[] = {SGI264960, SGI0, SGI264961, SGI0, SGI264962, SGI264963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59514[] = {SGI264964, SGI0, SGI264965, SGI0, SGI264966, SGI264967, SGI264968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59515[] = {SGI264969, SGI0, SGI264970, SGI0, SGI264971, SGI264972, SGI264973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59516[] = {SGI264974, SGI0, SGI264975, SGI0, SGI264976, SGI264977, SGI264978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59517[] = {SGI264979, SGI0, SGI264980, SGI0, SGI264981, SGI264982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59518[] = {SGI264983, SGI0, SGI264984, SGI0, SGI264985, SGI264986, SGI264987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59519[] = {SGI264988, SGI0, SGI264989, SGI0, SGI264990, SGI264991, SGI0, SGI0, SGI0, SGI264992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59520[] = {SGI264993, SGI0, SGI264994, SGI0, SGI264995, SGI264996, SGI0, SGI0, SGI0, SGI264997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59521[] = {SGI264998, SGI0, SGI264999, SGI0, SGI265000, SGI265001, SGI0, SGI0, SGI0, SGI265002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59522[] = {SGI265003, SGI0, SGI265004, SGI0, SGI265005, SGI265006, SGI0, SGI0, SGI0, SGI265007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59523[] = {SGI265008, SGI0, SGI265009, SGI0, SGI265010, SGI265011, SGI0, SGI0, SGI0, SGI265012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59524[] = {SGI265013, SGI265014, SGI265015, SGI0, SGI265016, SGI265017, SGI265018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59525[] = {SGI265019, SGI265020, SGI265021, SGI0, SGI265022, SGI265023, SGI265024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59526[] = {SGI265025, SGI265026, SGI265027, SGI0, SGI265028, SGI265029, SGI265028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59527[] = {SGI265030, SGI265031, NULL};
const Segment * const SG59528[] = {SGI265032, SGI265033, NULL};
const Segment * const SG59529[] = {SGI265034, SGI0, SGI265035, SGI0, SGI265036, SGI265037, SGI265038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265039, SGI0, NULL};
const Segment * const SG59530[] = {SGI265040, SGI0, SGI265041, SGI0, SGI265042, SGI265043, SGI265044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265045, SGI0, NULL};
const Segment * const SG59531[] = {SGI265046, SGI0, SGI265047, SGI0, SGI265048, SGI265049, SGI265050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265051, SGI0, NULL};
const Segment * const SG59532[] = {SGI265052, SGI0, SGI265053, SGI0, SGI265054, SGI265055, SGI265056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265057, SGI0, NULL};
const Segment * const SG59533[] = {SGI265058, SGI0, SGI265059, SGI0, SGI265060, SGI265061, SGI265062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265063, SGI0, NULL};
const Segment * const SG59534[] = {SGI265064, SGI0, SGI265065, SGI0, SGI265066, SGI265067, SGI265068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265069, SGI0, NULL};
const Segment * const SG59535[] = {SGI265070, SGI0, SGI265071, SGI0, SGI265072, SGI265073, SGI265074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59536[] = {SGI265075, SGI0, SGI265076, SGI0, SGI265077, SGI265078, SGI265079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59537[] = {SGI265080, SGI0, SGI265081, SGI0, SGI265082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59538[] = {SGI265083, SGI0, SGI265084, SGI0, SGI265085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59539[] = {SGI265086, SGI265087, SGI265088, SGI0, SGI265089, SGI265090, SGI265091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59540[] = {SGI265092, SGI265093, SGI265094, SGI0, SGI265095, SGI265096, SGI265097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59541[] = {SGI265098, SGI265099, SGI265100, SGI0, SGI265101, SGI265102, SGI265103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59542[] = {SGI265104, SGI265105, SGI265106, SGI0, SGI265107, SGI265108, SGI265109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59543[] = {SGI265110, SGI265111, SGI265112, SGI0, SGI265113, SGI265114, SGI265115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59544[] = {SGI265116, SGI0, SGI265117, SGI0, SGI265118, SGI265119, SGI265120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59545[] = {SGI265121, SGI0, SGI265122, SGI0, SGI265123, SGI265124, SGI265125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59546[] = {SGI265126, SGI0, SGI265127, SGI0, SGI265128, SGI265129, SGI265130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59547[] = {SGI265131, SGI0, SGI265132, SGI0, SGI265133, SGI265134, SGI265135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59548[] = {SGI265136, SGI0, SGI265137, SGI0, SGI265138, SGI265139, SGI265140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59549[] = {SGI265141, SGI0, SGI265142, SGI0, SGI265143, SGI265144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265145, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59550[] = {SGI265146, SGI0, SGI265147, SGI0, SGI265148, SGI265149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265150, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59551[] = {SGI265151, SGI0, SGI265152, SGI0, SGI265153, SGI265154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265155, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59552[] = {SGI265156, SGI0, SGI265157, SGI0, SGI265158, SGI265159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265160, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59553[] = {SGI265161, SGI0, SGI265162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59554[] = {SGI265163, SGI0, SGI265164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59555[] = {SGI265165, SGI0, SGI265166, SGI0, SGI265167, SGI265168, SGI0, SGI0, SGI0, SGI265169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59556[] = {SGI265170, SGI0, SGI265171, SGI0, SGI265172, SGI265173, SGI0, SGI0, SGI0, SGI265174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59557[] = {SGI265175, SGI0, SGI265176, SGI0, SGI265177, SGI265178, SGI0, SGI0, SGI0, SGI265179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59558[] = {SGI265180, SGI0, SGI265181, SGI0, SGI265182, SGI265183, SGI0, SGI0, SGI0, SGI265184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59559[] = {SGI265185, SGI0, SGI265186, SGI0, SGI265187, SGI265188, SGI0, SGI0, SGI0, SGI265189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59560[] = {SGI265190, SGI0, SGI265191, SGI0, SGI265192, SGI265193, SGI0, SGI0, SGI0, SGI265194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59561[] = {SGI265195, SGI0, SGI265196, SGI0, SGI265197, SGI265198, SGI265199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265200, SGI0, NULL};
const Segment * const SG59562[] = {SGI265201, SGI0, SGI265202, SGI0, SGI265203, SGI265204, SGI265205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265206, SGI0, NULL};
const Segment * const SG59563[] = {SGI265207, SGI265208, SGI265209, SGI0, SGI265210, SGI265211, SGI265212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59564[] = {SGI265213, SGI265214, SGI265215, SGI0, SGI265216, SGI265217, SGI265218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59565[] = {SGI265219, SGI265220, SGI265221, SGI0, SGI265222, SGI265223, SGI265224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59566[] = {SGI265225, SGI265226, SGI265227, SGI0, SGI265228, SGI265229, SGI265230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59567[] = {SGI265231, SGI265232, SGI265233, SGI0, SGI265234, SGI265235, SGI265236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59568[] = {SGI265237, SGI265238, SGI265239, SGI0, SGI265240, SGI265241, SGI265242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59569[] = {SGI265243, SGI265244, SGI265245, SGI0, SGI265246, SGI265247, SGI265248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59570[] = {SGI265249, SGI265250, SGI265251, SGI0, SGI265252, SGI265253, SGI265254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59571[] = {SGI265255, SGI265256, SGI265257, SGI0, SGI265258, SGI265259, SGI265260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59572[] = {SGI265261, SGI265262, SGI265263, SGI0, SGI265264, SGI265265, SGI265266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59573[] = {SGI265267, SGI265268, SGI265269, SGI0, SGI265270, SGI265271, SGI265270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59574[] = {SGI265272, SGI0, SGI265273, SGI0, SGI265274, SGI265275, SGI265276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59575[] = {SGI265277, SGI0, SGI265278, SGI0, SGI265279, SGI265280, SGI265281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59576[] = {SGI265282, SGI0, SGI265283, SGI0, SGI265284, SGI265285, SGI265286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265287, SGI0, NULL};
const Segment * const SG59577[] = {SGI265288, SGI0, SGI265289, SGI0, SGI265290, SGI265291, SGI265292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265293, SGI0, NULL};
const Segment * const SG59578[] = {SGI265294, SGI0, SGI265295, SGI0, SGI265296, SGI265297, SGI265298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265299, SGI0, NULL};
const Segment * const SG59579[] = {SGI265300, SGI0, SGI265301, SGI0, SGI265302, SGI265303, SGI265304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265305, SGI0, NULL};
const Segment * const SG59580[] = {SGI265306, SGI0, SGI265307, SGI0, SGI265308, SGI265309, SGI265310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265311, SGI0, NULL};
const Segment * const SG59581[] = {SGI265312, SGI0, SGI265313, SGI0, SGI265314, SGI265315, SGI265316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265317, SGI0, NULL};
const Segment * const SG59582[] = {SGI265318, SGI0, SGI265319, SGI0, SGI265320, SGI265321, SGI265322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265323, SGI0, NULL};
const Segment * const SG59583[] = {SGI265324, SGI0, SGI265325, SGI0, SGI265326, SGI265327, SGI265328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265329, SGI0, NULL};
const Segment * const SG59584[] = {SGI265330, SGI0, SGI265331, SGI0, SGI265332, SGI265333, SGI265334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265335, SGI0, NULL};
const Segment * const SG59585[] = {SGI265336, SGI0, SGI265337, SGI0, SGI265338, SGI265339, SGI265340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265341, SGI0, NULL};
const Segment * const SG59586[] = {SGI265342, SGI0, SGI265343, SGI0, SGI265344, SGI265345, SGI265346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265347, SGI0, NULL};
const Segment * const SG59587[] = {SGI265348, SGI0, SGI265349, SGI0, SGI265350, SGI265351, SGI265352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265353, SGI0, NULL};
const Segment * const SG59588[] = {SGI265354, SGI0, SGI265355, SGI0, SGI0, SGI265356, SGI265357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59589[] = {SGI265358, SGI265359, SGI265360, SGI0, SGI265361, SGI265362, SGI265363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59590[] = {SGI265364, SGI265365, SGI265366, SGI0, SGI265367, SGI265368, SGI265369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59591[] = {SGI265370, SGI265371, SGI265372, SGI0, SGI265373, SGI265374, SGI265375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59592[] = {SGI265376, SGI265377, SGI265378, SGI0, SGI265379, SGI265380, SGI265381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59593[] = {SGI265382, SGI265383, SGI265384, SGI0, SGI265385, SGI265386, SGI265387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59594[] = {SGI265388, SGI0, SGI265389, SGI0, SGI0, SGI265390, SGI265391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59595[] = {SGI265392, SGI0, SGI265393, SGI0, SGI265394, SGI265395, SGI265396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59596[] = {SGI265397, SGI265398, SGI265399, SGI0, SGI265400, SGI265401, SGI265402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59597[] = {SGI265403, SGI265404, SGI265405, SGI0, SGI265406, SGI265407, SGI265408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59598[] = {SGI265409, SGI265410, SGI265411, SGI0, SGI265412, SGI265413, SGI265414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59599[] = {SGI265415, SGI0, SGI265416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59600[] = {SGI265417, SGI0, SGI265418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59601[] = {SGI265419, SGI0, SGI265420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59602[] = {SGI265421, SGI0, SGI265422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59603[] = {SGI265423, SGI0, SGI265424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59604[] = {SGI265425, SGI0, SGI265426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59605[] = {SGI265427, SGI0, SGI265428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59606[] = {SGI265429, SGI0, SGI265430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59607[] = {SGI265431, SGI0, SGI265432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59608[] = {SGI265433, SGI0, SGI265434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59609[] = {SGI265435, SGI0, SGI265436, SGI0, SGI265437, SGI265438, SGI265439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59610[] = {SGI265440, SGI0, SGI265441, SGI0, SGI265442, SGI265443, SGI265444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59611[] = {SGI265445, SGI0, SGI265446, SGI0, SGI265447, SGI265448, SGI265449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265450, SGI0, NULL};
const Segment * const SG59612[] = {SGI265451, SGI0, SGI265452, SGI0, SGI265453, SGI265454, SGI265455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265456, SGI0, NULL};
const Segment * const SG59613[] = {SGI265457, SGI0, SGI265458, SGI0, SGI265459, SGI265460, SGI265461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265462, SGI0, NULL};
const Segment * const SG59614[] = {SGI265463, SGI265464, SGI0, SGI0, NULL};
const Segment * const SG59615[] = {SGI265465, SGI265466, SGI265467, SGI0, SGI265468, SGI265469, SGI265470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59616[] = {SGI265471, SGI265472, SGI265473, SGI0, SGI265474, SGI265475, SGI265476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59617[] = {SGI265477, SGI265478, SGI265479, SGI0, SGI265480, SGI265481, SGI265482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59618[] = {SGI265483, SGI265484, SGI265485, SGI0, SGI265486, SGI265487, SGI265488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59619[] = {SGI265489, SGI265490, SGI265491, SGI0, SGI265492, SGI265493, SGI265494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59620[] = {SGI265495, SGI265496, SGI265497, SGI0, SGI265498, SGI265499, SGI265500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59621[] = {SGI265501, SGI265502, SGI0, SGI0, NULL};
const Segment * const SG59622[] = {SGI265503, SGI0, SGI265504, SGI0, SGI265505, SGI265506, SGI265507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59623[] = {SGI265508, SGI0, SGI265509, SGI0, SGI265510, SGI265511, SGI265512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59624[] = {SGI265513, SGI0, SGI265514, SGI0, SGI265515, SGI265516, SGI265517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59625[] = {SGI265518, SGI0, SGI265519, SGI0, SGI265520, SGI265521, SGI265522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59626[] = {SGI265523, SGI0, SGI265524, SGI0, SGI265525, SGI265526, SGI265527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59627[] = {SGI265528, SGI0, SGI265529, SGI0, SGI265530, SGI265531, SGI265532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59628[] = {SGI265533, SGI0, SGI265534, SGI0, SGI265535, SGI265536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59629[] = {SGI265537, SGI0, SGI265538, SGI0, SGI265539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59630[] = {SGI265540, SGI0, SGI265541, SGI0, SGI265542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59631[] = {SGI265543, SGI0, SGI265544, SGI0, SGI265545, SGI265546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59632[] = {SGI265547, SGI0, SGI265548, SGI0, SGI265549, SGI265550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59633[] = {SGI265551, SGI265552, NULL};
const Segment * const SG59634[] = {SGI265553, SGI265554, NULL};
const Segment * const SG59635[] = {SGI265555, SGI265556, NULL};
const Segment * const SG59636[] = {SGI265557, SGI265558, NULL};
const Segment * const SG59637[] = {SGI265559, SGI265560, NULL};
const Segment * const SG59638[] = {SGI265561, SGI265562, NULL};
const Segment * const SG59639[] = {SGI265563, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59640[] = {SGI265564, SGI265565, SGI0, SGI0, NULL};
const Segment * const SG59641[] = {SGI265566, SGI0, SGI265567, SGI265568, SGI265569, SGI265570, SGI265571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59642[] = {SGI265572, SGI0, SGI265573, SGI265574, SGI265575, SGI265576, SGI265577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59643[] = {SGI265578, SGI0, SGI265579, SGI265580, SGI265581, SGI265582, SGI265583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59644[] = {SGI265584, SGI0, SGI265585, SGI265586, SGI265587, SGI265588, SGI265589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59645[] = {SGI265590, SGI0, SGI265591, SGI265592, SGI265593, SGI265594, SGI265595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59646[] = {SGI265596, SGI0, SGI265597, SGI265598, SGI265599, SGI265600, SGI265601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59647[] = {SGI265602, SGI0, SGI265603, SGI265604, SGI265605, SGI265606, SGI265607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59648[] = {SGI265608, SGI0, SGI265609, SGI265610, SGI265611, SGI265612, SGI265613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59649[] = {SGI265614, SGI0, SGI265615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59650[] = {SGI265616, SGI0, SGI265617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59651[] = {SGI265618, SGI265619, SGI0, SGI0, NULL};
const Segment * const SG59652[] = {SGI265620, SGI265621, SGI265622, SGI0, SGI265623, SGI265624, SGI265625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59653[] = {SGI265626, SGI265627, SGI265628, SGI0, SGI265629, SGI265630, SGI265631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59654[] = {SGI265632, SGI265633, SGI265634, SGI0, SGI265635, SGI265636, SGI265637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59655[] = {SGI265638, SGI265639, SGI265640, SGI0, SGI265641, SGI265642, SGI265643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59656[] = {SGI265644, SGI265645, SGI265646, SGI0, SGI265647, SGI265648, SGI265649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59657[] = {SGI265650, SGI265651, SGI0, SGI0, SGI265652, SGI265653, SGI265654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59658[] = {SGI265655, SGI265656, SGI0, SGI0, SGI265657, SGI265658, SGI265659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59659[] = {SGI265660, SGI0, SGI265661, SGI0, SGI265662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59660[] = {SGI265663, SGI0, SGI265664, SGI0, SGI265665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59661[] = {SGI265666, SGI0, SGI265667, SGI0, SGI265668, SGI265669, SGI265670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59662[] = {SGI265671, SGI0, SGI265672, SGI0, SGI265673, SGI265674, SGI265675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59663[] = {SGI265676, SGI0, SGI265677, SGI0, SGI265678, SGI265679, SGI265680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265681, SGI0, NULL};
const Segment * const SG59664[] = {SGI265682, SGI0, SGI265683, SGI0, SGI265684, SGI265685, SGI265686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265687, SGI0, NULL};
const Segment * const SG59665[] = {SGI265688, SGI0, SGI265689, SGI0, SGI265690, SGI265691, SGI265692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265693, SGI0, NULL};
const Segment * const SG59666[] = {SGI265694, SGI0, SGI265695, SGI0, SGI265696, SGI265697, SGI265698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265699, SGI0, NULL};
const Segment * const SG59667[] = {SGI265700, SGI0, SGI265701, SGI0, SGI265702, SGI265703, SGI265704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265705, SGI0, NULL};
const Segment * const SG59668[] = {SGI265706, SGI0, SGI265707, SGI0, SGI265708, SGI265709, SGI265710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265711, SGI0, NULL};
const Segment * const SG59669[] = {SGI265712, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59670[] = {SGI265713, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59671[] = {SGI265714, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59672[] = {SGI265715, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59673[] = {SGI265716, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59674[] = {SGI265717, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59675[] = {SGI265718, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59676[] = {SGI265719, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59677[] = {SGI265720, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59678[] = {SGI265721, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59679[] = {SGI265722, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59680[] = {SGI265723, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59681[] = {SGI265724, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59682[] = {SGI265725, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59683[] = {SGI265726, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59684[] = {SGI265727, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59685[] = {SGI265728, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59686[] = {SGI265729, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59687[] = {SGI265730, SGI265731, SGI0, SGI0, NULL};
const Segment * const SG59688[] = {SGI265732, SGI265733, SGI265734, SGI0, SGI265735, SGI265736, SGI265737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265738, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59689[] = {SGI265739, SGI0, SGI265740, SGI0, SGI265741, SGI265742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265743, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59690[] = {SGI265744, SGI0, SGI265745, SGI0, SGI265746, SGI265747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265748, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59691[] = {SGI265749, SGI0, SGI265750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59692[] = {SGI265751, SGI0, SGI265752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59693[] = {SGI265753, SGI0, SGI265754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59694[] = {SGI265755, SGI0, SGI265756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59695[] = {SGI265757, SGI0, SGI265758, SGI0, SGI265759, SGI265760, SGI265761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59696[] = {SGI265762, SGI0, SGI265763, SGI0, SGI265764, SGI265765, SGI265766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59697[] = {SGI265767, SGI0, SGI265768, SGI0, SGI265769, SGI265770, SGI265771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59698[] = {SGI265772, SGI0, SGI265773, SGI0, SGI265774, SGI265775, SGI265776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59699[] = {SGI265777, SGI0, SGI265778, SGI0, SGI265779, SGI265780, SGI265781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59700[] = {SGI265782, SGI265783, SGI265784, SGI0, SGI265785, SGI265786, SGI265785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59701[] = {SGI265787, SGI265788, SGI265789, SGI0, SGI265790, SGI265791, SGI265792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59702[] = {SGI265793, SGI265794, SGI265795, SGI0, SGI265796, SGI265797, SGI265798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59703[] = {SGI265799, SGI265800, SGI265801, SGI0, SGI265802, SGI265803, SGI265804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59704[] = {SGI265805, SGI265806, SGI265807, SGI0, SGI265808, SGI265809, SGI265810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59705[] = {SGI265811, SGI265812, SGI265813, SGI0, SGI265814, SGI265815, SGI265814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59706[] = {SGI265816, SGI0, SGI265817, SGI0, SGI265818, SGI265819, SGI265820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59707[] = {SGI265821, SGI0, SGI265822, SGI0, SGI265823, SGI265824, SGI265825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59708[] = {SGI265826, SGI0, SGI265827, SGI0, SGI265828, SGI265829, SGI265830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59709[] = {SGI265831, SGI0, SGI265832, SGI0, SGI265833, SGI265834, SGI265835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59710[] = {SGI265836, SGI265837, SGI265838, SGI0, SGI265839, SGI265840, SGI265839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265841, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59711[] = {SGI265842, SGI0, SGI265843, SGI0, SGI265844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59712[] = {SGI265845, SGI0, SGI265846, SGI0, SGI265847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59713[] = {SGI265848, SGI0, SGI265849, SGI0, SGI265850, SGI265851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59714[] = {SGI265852, SGI0, SGI265853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59715[] = {SGI265854, SGI0, SGI265855, SGI0, SGI265856, SGI265857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59716[] = {SGI265858, SGI0, SGI265859, SGI0, SGI265860, SGI265861, SGI265862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59717[] = {SGI265863, SGI0, SGI265864, SGI0, SGI265865, SGI265866, SGI265867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59718[] = {SGI265868, SGI0, SGI265869, SGI0, SGI265870, SGI265871, SGI265872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59719[] = {SGI265873, SGI0, SGI265874, SGI0, SGI265875, SGI265876, SGI265877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59720[] = {SGI265878, SGI0, SGI265879, SGI0, SGI265880, SGI265881, SGI265882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59721[] = {SGI265883, SGI0, SGI265884, SGI0, SGI265885, SGI265886, SGI265887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59722[] = {SGI265888, SGI0, SGI265889, SGI0, SGI265890, SGI265891, SGI265892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59723[] = {SGI265893, SGI0, SGI265894, SGI0, SGI265895, SGI265896, SGI265897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59724[] = {SGI265898, SGI0, SGI265899, SGI0, SGI265900, SGI265901, SGI265902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59725[] = {SGI265903, SGI0, SGI265904, SGI0, SGI265905, SGI265906, SGI265907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59726[] = {SGI265908, SGI0, SGI265909, SGI0, SGI265910, SGI0, SGI265911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI265912, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59727[] = {SGI265913, SGI265914, SGI0, SGI0, NULL};
const Segment * const SG59728[] = {SGI265915, SGI0, SGI265916, SGI0, SGI265917, SGI265918, SGI265919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59729[] = {SGI265920, SGI0, SGI265921, SGI0, SGI265922, SGI265923, SGI265924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59730[] = {SGI265925, SGI0, SGI265926, SGI0, SGI265927, SGI265928, SGI265929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59731[] = {SGI265930, SGI265931, SGI265932, SGI0, SGI265933, SGI265934, SGI265935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59732[] = {SGI265936, SGI265937, SGI265938, SGI0, SGI265939, SGI265940, SGI265941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59733[] = {SGI265942, SGI265943, SGI265944, SGI0, SGI265945, SGI265946, SGI265947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59734[] = {SGI265948, SGI265949, SGI265950, SGI0, SGI265951, SGI265952, SGI265953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59735[] = {SGI265954, SGI265955, SGI265956, SGI0, SGI265957, SGI265958, SGI265959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59736[] = {SGI265960, SGI0, SGI265961, SGI0, SGI265962, SGI265963, SGI265964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59737[] = {SGI265965, SGI0, SGI265966, SGI0, SGI265967, SGI265968, SGI265969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59738[] = {SGI265970, SGI265971, NULL};
const Segment * const SG59739[] = {SGI265972, SGI0, SGI265973, SGI0, SGI265974, SGI265975, SGI265976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59740[] = {SGI265977, SGI0, SGI265978, SGI0, SGI265979, SGI265980, SGI265981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59741[] = {SGI265982, SGI265983, SGI0, SGI0, NULL};
const Segment * const SG59742[] = {SGI265984, SGI265985, NULL};
const Segment * const SG59743[] = {SGI265986, SGI265987, NULL};
const Segment * const SG59744[] = {SGI265988, SGI0, SGI265989, SGI0, SGI265990, SGI265991, SGI0, SGI0, SGI0, SGI265992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59745[] = {SGI265993, SGI0, SGI265994, SGI0, SGI265995, SGI265996, SGI0, SGI0, SGI0, SGI265997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59746[] = {SGI265998, SGI0, SGI265999, SGI0, SGI266000, SGI266001, SGI0, SGI0, SGI0, SGI266002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59747[] = {SGI266003, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59748[] = {SGI266004, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59749[] = {SGI266005, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59750[] = {SGI266006, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59751[] = {SGI266007, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59752[] = {SGI266008, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59753[] = {SGI266009, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59754[] = {SGI266010, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59755[] = {SGI266011, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59756[] = {SGI266012, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59757[] = {SGI266013, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59758[] = {SGI266014, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59759[] = {SGI266015, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59760[] = {SGI266016, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59761[] = {SGI266017, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59762[] = {SGI266018, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59763[] = {SGI266019, SGI266020, SGI0, SGI0, NULL};
const Segment * const SG59764[] = {SGI266021, SGI266022, SGI0, SGI0, NULL};
const Segment * const SG59765[] = {SGI266023, SGI266024, SGI0, SGI0, NULL};
const Segment * const SG59766[] = {SGI266025, SGI266026, SGI0, SGI0, NULL};
const Segment * const SG59767[] = {SGI266027, SGI266028, SGI0, SGI0, NULL};
const Segment * const SG59768[] = {SGI266029, SGI0, SGI266030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59769[] = {SGI266031, SGI0, SGI266032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59770[] = {SGI266033, SGI0, SGI266034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59771[] = {SGI266035, SGI0, SGI266036, SGI0, SGI266037, SGI266038, SGI266039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266040, SGI0, NULL};
const Segment * const SG59772[] = {SGI266041, SGI0, SGI266042, SGI0, SGI266043, SGI266044, SGI266045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266046, SGI0, NULL};
const Segment * const SG59773[] = {SGI266047, SGI0, SGI266048, SGI0, SGI266049, SGI266050, SGI266051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266052, SGI0, NULL};
const Segment * const SG59774[] = {SGI266053, SGI0, SGI266054, SGI0, SGI266055, SGI266056, SGI266057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266058, SGI0, NULL};
const Segment * const SG59775[] = {SGI266059, SGI0, SGI266060, SGI0, SGI266061, SGI266062, SGI266063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266064, SGI0, NULL};
const Segment * const SG59776[] = {SGI266065, SGI0, SGI266066, SGI0, SGI266067, SGI266068, SGI266069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266070, SGI0, NULL};
const Segment * const SG59777[] = {SGI266071, SGI0, SGI266072, SGI0, SGI266073, SGI266074, SGI266075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59778[] = {SGI266076, SGI0, SGI266077, SGI0, SGI266078, SGI266079, SGI266080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59779[] = {SGI266081, SGI0, SGI266082, SGI0, SGI266083, SGI266084, SGI266085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59780[] = {SGI266086, SGI266087, SGI266088, SGI0, SGI266089, SGI266090, SGI266091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59781[] = {SGI266092, SGI0, SGI266093, SGI0, SGI266094, SGI266095, SGI266096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59782[] = {SGI266097, SGI266098, SGI266099, SGI0, SGI266100, SGI266101, SGI266102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59783[] = {SGI266103, SGI266104, SGI266105, SGI0, SGI266106, SGI266107, SGI266108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59784[] = {SGI266109, SGI0, SGI266110, SGI0, SGI0, SGI266111, SGI266112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59785[] = {SGI266113, SGI266114, SGI266115, SGI0, SGI266116, SGI266117, SGI266118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59786[] = {SGI266119, SGI0, SGI266120, SGI0, SGI0, SGI266121, SGI266122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59787[] = {SGI266123, SGI266124, SGI266125, SGI0, SGI266126, SGI266127, SGI266128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59788[] = {SGI266129, SGI0, SGI266130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59789[] = {SGI266131, SGI0, SGI266132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59790[] = {SGI266133, SGI266134, SGI266135, SGI0, SGI266136, SGI266137, SGI266136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59791[] = {SGI266138, SGI266139, SGI266140, SGI0, SGI266141, SGI266142, SGI266141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59792[] = {SGI266143, SGI266144, SGI0, SGI0, NULL};
const Segment * const SG59793[] = {SGI266145, SGI266146, SGI266147, SGI0, SGI266148, SGI266149, SGI266150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266151, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59794[] = {SGI266152, SGI0, SGI266153, SGI0, SGI266154, SGI266155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266156, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59795[] = {SGI266157, SGI0, SGI266158, SGI0, SGI266159, SGI266160, SGI0, SGI0, SGI0, SGI266161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59796[] = {SGI266162, SGI0, SGI266163, SGI0, SGI266164, SGI266165, SGI0, SGI0, SGI0, SGI266166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59797[] = {SGI266167, SGI266168, SGI266169, SGI0, SGI266170, SGI266171, SGI266172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59798[] = {SGI266173, SGI0, SGI266174, SGI0, SGI266175, SGI266176, SGI266177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266178, SGI0, NULL};
const Segment * const SG59799[] = {SGI266179, SGI0, SGI266180, SGI0, SGI266181, SGI266182, SGI266183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266184, SGI0, NULL};
const Segment * const SG59800[] = {SGI266185, SGI0, SGI266186, SGI0, SGI266187, SGI266188, SGI266189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266190, SGI0, NULL};
const Segment * const SG59801[] = {SGI266191, SGI0, SGI266192, SGI0, SGI266193, SGI266194, SGI266195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266196, SGI0, NULL};
const Segment * const SG59802[] = {SGI266197, SGI0, SGI266198, SGI0, SGI266199, SGI266200, SGI266201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266202, SGI0, NULL};
const Segment * const SG59803[] = {SGI266203, SGI266204, SGI266205, SGI0, SGI266206, SGI266207, SGI266208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59804[] = {SGI266209, SGI266210, SGI266211, SGI0, SGI266212, SGI266213, SGI266214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59805[] = {SGI266215, SGI266216, SGI266217, SGI0, SGI266218, SGI266219, SGI266220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59806[] = {SGI266221, SGI266222, SGI266223, SGI0, SGI266224, SGI266225, SGI266226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59807[] = {SGI266227, SGI266228, SGI266229, SGI0, SGI266230, SGI266231, SGI266232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59808[] = {SGI266233, SGI266234, SGI266235, SGI0, SGI266236, SGI266237, SGI266238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59809[] = {SGI266239, SGI266240, SGI266241, SGI0, SGI266242, SGI266243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59810[] = {SGI266244, SGI266245, SGI266246, SGI0, SGI266247, SGI266248, SGI266249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59811[] = {SGI266250, SGI266251, SGI266252, SGI0, SGI266253, SGI266254, SGI266255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59812[] = {SGI266256, SGI266257, SGI266258, SGI0, SGI266259, SGI266260, SGI266261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59813[] = {SGI266262, SGI266263, SGI266264, SGI0, SGI266265, SGI266266, SGI266267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59814[] = {SGI266268, SGI266269, SGI266270, SGI0, SGI266271, SGI266272, SGI266273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59815[] = {SGI266274, SGI266275, SGI266276, SGI0, SGI266277, SGI266278, SGI266279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59816[] = {SGI266280, SGI266281, SGI266282, SGI0, SGI266283, SGI266284, SGI266285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59817[] = {SGI266286, SGI266287, SGI266288, SGI0, SGI266289, SGI266290, SGI266291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59818[] = {SGI266292, SGI266293, SGI266294, SGI0, SGI266295, SGI266296, SGI266297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59819[] = {SGI266298, SGI0, SGI266299, SGI0, SGI266300, SGI266301, SGI266302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59820[] = {SGI266303, SGI0, SGI266304, SGI0, SGI266305, SGI266306, SGI266307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59821[] = {SGI266308, SGI0, SGI266309, SGI0, SGI266310, SGI266311, SGI266312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59822[] = {SGI266313, SGI0, SGI266314, SGI0, SGI266315, SGI266316, SGI266317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59823[] = {SGI266318, SGI0, SGI266319, SGI0, SGI266320, SGI266321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266322, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59824[] = {SGI266323, SGI0, SGI266324, SGI0, SGI266325, SGI266326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266327, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59825[] = {SGI266328, SGI266329, NULL};
const Segment * const SG59826[] = {SGI266330, SGI266331, NULL};
const Segment * const SG59827[] = {SGI266332, SGI266333, SGI0, SGI266334, NULL};
const Segment * const SG59828[] = {SGI266335, SGI0, SGI0, SGI0, SGI266336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59829[] = {SGI266337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59830[] = {SGI266338, SGI266339, SGI266340, SGI0, SGI266341, SGI266342, SGI266341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59831[] = {SGI266343, SGI0, SGI266344, SGI0, SGI266345, SGI266346, SGI266347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59832[] = {SGI266348, SGI0, SGI266349, SGI0, SGI266350, SGI266351, SGI266352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59833[] = {SGI266353, SGI0, SGI266354, SGI0, SGI266355, SGI266356, SGI266357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59834[] = {SGI266358, SGI0, SGI266359, SGI0, SGI266360, SGI266361, SGI266362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59835[] = {SGI266363, SGI0, SGI266364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59836[] = {SGI266365, SGI0, SGI266366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59837[] = {SGI266367, SGI266368, NULL};
const Segment * const SG59838[] = {SGI266369, SGI266370, NULL};
const Segment * const SG59839[] = {SGI266371, SGI266372, NULL};
const Segment * const SG59840[] = {SGI266373, SGI266374, NULL};
const Segment * const SG59841[] = {SGI266375, SGI266376, NULL};
const Segment * const SG59842[] = {SGI266377, SGI266378, NULL};
const Segment * const SG59843[] = {SGI266379, SGI266380, SGI0, SGI0, NULL};
const Segment * const SG59844[] = {SGI266381, SGI266382, SGI0, SGI0, NULL};
const Segment * const SG59845[] = {SGI266383, SGI266384, SGI0, SGI0, NULL};
const Segment * const SG59846[] = {SGI266385, SGI0, SGI266386, SGI0, SGI266387, SGI266388, SGI0, SGI0, SGI0, SGI266389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59847[] = {SGI266390, SGI0, SGI266391, SGI0, SGI266392, SGI266393, SGI0, SGI0, SGI0, SGI266394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59848[] = {SGI266395, SGI0, SGI266396, SGI0, SGI266397, SGI266398, SGI0, SGI0, SGI0, SGI266399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59849[] = {SGI266400, SGI266401, NULL};
const Segment * const SG59850[] = {SGI266402, SGI266403, NULL};
const Segment * const SG59851[] = {SGI266404, SGI266405, NULL};
const Segment * const SG59852[] = {SGI266406, SGI266407, NULL};
const Segment * const SG59853[] = {SGI266408, SGI266409, NULL};
const Segment * const SG59854[] = {SGI266410, SGI266411, NULL};
const Segment * const SG59855[] = {SGI266412, SGI266413, NULL};
const Segment * const SG59856[] = {SGI266414, SGI0, SGI266415, SGI0, SGI266416, SGI0, SGI266417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266418, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59857[] = {SGI266419, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59858[] = {SGI266420, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59859[] = {SGI266421, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59860[] = {SGI266422, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59861[] = {SGI266423, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59862[] = {SGI266424, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59863[] = {SGI266425, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59864[] = {SGI266426, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59865[] = {SGI266427, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59866[] = {SGI266428, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59867[] = {SGI266429, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59868[] = {SGI266430, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59869[] = {SGI266431, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59870[] = {SGI266432, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59871[] = {SGI266433, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59872[] = {SGI266434, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59873[] = {SGI266435, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59874[] = {SGI266436, SGI266437, SGI266438, SGI0, SGI266439, SGI266440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59875[] = {SGI266441, SGI266442, SGI266443, SGI0, SGI266444, SGI266445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59876[] = {SGI266446, SGI266447, SGI266448, SGI0, SGI266449, SGI266450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59877[] = {SGI266451, SGI266452, SGI266453, SGI0, SGI266454, SGI266455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59878[] = {SGI266456, SGI266457, SGI266458, SGI0, SGI266459, SGI266460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59879[] = {SGI266461, SGI266462, NULL};
const Segment * const SG59880[] = {SGI266463, SGI266464, NULL};
const Segment * const SG59881[] = {SGI266465, SGI0, SGI266466, SGI0, SGI266467, SGI266468, SGI266469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266470, SGI0, NULL};
const Segment * const SG59882[] = {SGI266471, SGI0, SGI266472, SGI0, SGI266473, SGI266474, SGI266475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266476, SGI0, NULL};
const Segment * const SG59883[] = {SGI266477, SGI0, SGI266478, SGI0, SGI266479, SGI266480, SGI266481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266482, SGI0, NULL};
const Segment * const SG59884[] = {SGI266483, SGI0, SGI266484, SGI0, SGI266485, SGI266486, SGI266487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266488, SGI0, NULL};
const Segment * const SG59885[] = {SGI266489, SGI0, SGI266490, SGI0, SGI266491, SGI266492, SGI266493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266494, SGI0, NULL};
const Segment * const SG59886[] = {SGI266495, SGI0, SGI266496, SGI0, SGI266497, SGI266498, SGI266499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266500, SGI0, NULL};
const Segment * const SG59887[] = {SGI266501, SGI266502, NULL};
const Segment * const SG59888[] = {SGI266503, SGI266504, NULL};
const Segment * const SG59889[] = {SGI266505, SGI0, SGI266506, SGI0, SGI266507, SGI266508, SGI266509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59890[] = {SGI266510, SGI0, SGI266511, SGI0, SGI266512, SGI266513, SGI266514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59891[] = {SGI266515, SGI0, SGI266516, SGI0, SGI266517, SGI266518, SGI266519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59892[] = {SGI266520, SGI0, SGI266521, SGI0, SGI266522, SGI266523, SGI266524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59893[] = {SGI266525, SGI0, SGI266526, SGI0, SGI266527, SGI266528, SGI266529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59894[] = {SGI266530, SGI0, SGI266531, SGI0, SGI266532, SGI266533, SGI266534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59895[] = {SGI266535, SGI266536, NULL};
const Segment * const SG59896[] = {SGI266537, SGI0, SGI266538, SGI0, SGI266539, SGI266540, SGI266541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266542, SGI0, NULL};
const Segment * const SG59897[] = {SGI266543, SGI0, SGI266544, SGI0, SGI266545, SGI266546, SGI266547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266548, SGI0, NULL};
const Segment * const SG59898[] = {SGI266549, SGI0, SGI266550, SGI0, SGI266551, SGI266552, SGI266553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266554, SGI0, NULL};
const Segment * const SG59899[] = {SGI266555, SGI0, SGI266556, SGI0, SGI266557, SGI266558, SGI266559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266560, SGI0, NULL};
const Segment * const SG59900[] = {SGI266561, SGI266562, SGI266563, SGI0, SGI266564, SGI266565, SGI266566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59901[] = {SGI266567, SGI266568, SGI266569, SGI0, SGI266570, SGI266571, SGI266572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59902[] = {SGI266573, SGI266574, SGI266575, SGI0, SGI266576, SGI266577, SGI266578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59903[] = {SGI266579, SGI0, SGI266580, SGI0, SGI266581, SGI266582, SGI266583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59904[] = {SGI266584, SGI0, SGI266585, SGI266586, SGI266587, SGI266588, SGI266589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59905[] = {SGI266590, SGI0, SGI266591, SGI266592, SGI266593, SGI266594, SGI266595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59906[] = {SGI266596, SGI0, SGI266597, SGI266598, SGI266599, SGI266600, SGI266601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59907[] = {SGI266602, SGI0, SGI266603, SGI266604, SGI266605, SGI266606, SGI266607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59908[] = {SGI266608, SGI0, SGI266609, SGI266610, SGI266611, SGI266612, SGI266613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59909[] = {SGI266614, SGI0, SGI266615, SGI266616, SGI266617, SGI266618, SGI266619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59910[] = {SGI266620, SGI0, SGI266621, SGI266622, SGI266623, SGI266624, SGI266625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59911[] = {SGI266626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59912[] = {SGI266627, SGI266628, SGI266629, SGI0, SGI266630, SGI266631, SGI266632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59913[] = {SGI266633, SGI0, SGI266634, SGI0, SGI266635, SGI266636, SGI266637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59914[] = {SGI266638, SGI0, SGI266639, SGI0, SGI266640, SGI266641, SGI266642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59915[] = {SGI266643, SGI266644, SGI0, SGI0, NULL};
const Segment * const SG59916[] = {SGI266645, SGI0, SGI266646, SGI0, SGI266647, SGI266648, SGI266649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59917[] = {SGI266650, SGI0, SGI266651, SGI0, SGI266652, SGI266653, SGI266654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59918[] = {SGI266655, SGI0, SGI266656, SGI0, SGI266657, SGI266658, SGI266659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59919[] = {SGI266660, SGI0, SGI266661, SGI0, SGI266662, SGI266663, SGI266664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59920[] = {SGI266665, SGI0, SGI266666, SGI0, SGI266667, SGI266668, SGI266669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59921[] = {SGI266670, SGI0, SGI266671, SGI0, SGI266672, SGI266673, SGI266674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59922[] = {SGI266675, SGI0, SGI266676, SGI0, SGI266677, SGI266678, SGI266679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59923[] = {SGI266680, SGI0, SGI266681, SGI0, SGI266682, SGI266683, SGI266684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266685, SGI0, NULL};
const Segment * const SG59924[] = {SGI266686, SGI0, SGI266687, SGI0, SGI266688, SGI266689, SGI266690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266691, SGI0, NULL};
const Segment * const SG59925[] = {SGI266692, SGI0, SGI266693, SGI0, SGI266694, SGI266695, SGI266696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266697, SGI0, NULL};
const Segment * const SG59926[] = {SGI266698, SGI0, SGI266699, SGI0, SGI266700, SGI266701, SGI266702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266703, SGI0, NULL};
const Segment * const SG59927[] = {SGI266704, SGI0, SGI266705, SGI0, SGI266706, SGI266707, SGI266708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266709, SGI0, NULL};
const Segment * const SG59928[] = {SGI266710, SGI0, SGI266711, SGI0, SGI266712, SGI266713, SGI266714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266715, SGI0, NULL};
const Segment * const SG59929[] = {SGI266716, SGI0, SGI266717, SGI0, SGI266718, SGI266719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266720, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59930[] = {SGI266721, SGI0, SGI266722, SGI0, SGI266723, SGI266724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266725, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59931[] = {SGI266726, SGI0, SGI266727, SGI0, SGI266728, SGI266729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266730, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59932[] = {SGI266731, SGI0, SGI266732, SGI0, SGI266733, SGI266734, SGI266735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59933[] = {SGI266736, SGI0, SGI266737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59934[] = {SGI266738, SGI0, SGI266739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59935[] = {SGI266740, SGI0, SGI266741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59936[] = {SGI266742, SGI0, SGI266743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59937[] = {SGI266744, SGI0, SGI266745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59938[] = {SGI266746, SGI0, SGI266747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59939[] = {SGI266748, SGI266749, SGI266750, SGI0, SGI266751, SGI266752, SGI266753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59940[] = {SGI266754, SGI266755, SGI266756, SGI0, SGI266757, SGI266758, SGI266759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59941[] = {SGI266760, SGI266761, NULL};
const Segment * const SG59942[] = {SGI266762, SGI0, SGI266763, SGI0, SGI266764, SGI266765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266766, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59943[] = {SGI266767, SGI0, SGI266768, SGI0, SGI266769, SGI266770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266771, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59944[] = {SGI266772, SGI0, SGI266773, SGI0, SGI266774, SGI266775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266776, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59945[] = {SGI266777, SGI0, SGI266778, SGI0, SGI266779, SGI266780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266781, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59946[] = {SGI266782, SGI0, SGI266783, SGI0, SGI266784, SGI266785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266786, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59947[] = {SGI266787, SGI0, SGI266788, SGI0, SGI266789, SGI266790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266791, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59948[] = {SGI266792, SGI0, SGI266793, SGI0, SGI266794, SGI266795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266796, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59949[] = {SGI266797, SGI266798, SGI0, SGI0, NULL};
const Segment * const SG59950[] = {SGI266799, SGI266800, SGI266801, SGI0, SGI266802, SGI266803, SGI266804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59951[] = {SGI266805, SGI266806, SGI266807, SGI0, SGI266808, SGI266809, SGI266810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59952[] = {SGI266811, SGI266812, SGI266813, SGI0, SGI266814, SGI266815, SGI266816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59953[] = {SGI266817, SGI0, SGI266818, SGI0, SGI0, SGI266819, SGI266820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59954[] = {SGI266821, SGI266822, SGI266823, SGI0, SGI266824, SGI266825, SGI266826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59955[] = {SGI266827, SGI266828, SGI266829, SGI0, SGI266830, SGI266831, SGI266832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59956[] = {SGI266833, SGI266834, SGI266835, SGI0, SGI266836, SGI266837, SGI266838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59957[] = {SGI266839, SGI266840, SGI266841, SGI0, SGI266842, SGI266843, SGI266844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59958[] = {SGI266845, SGI266846, SGI266847, SGI0, SGI266848, SGI266849, SGI266850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59959[] = {SGI266851, SGI266852, SGI266853, SGI0, SGI266854, SGI266855, SGI266856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59960[] = {SGI266857, SGI266858, SGI266859, SGI0, SGI266860, SGI266861, SGI266862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59961[] = {SGI266863, SGI0, SGI266864, SGI0, SGI266865, SGI266866, SGI266867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266868, SGI0, NULL};
const Segment * const SG59962[] = {SGI266869, SGI0, SGI266870, SGI0, SGI266871, SGI266872, SGI266873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266874, SGI0, NULL};
const Segment * const SG59963[] = {SGI266875, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59964[] = {SGI266876, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59965[] = {SGI266877, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59966[] = {SGI266878, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59967[] = {SGI266879, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59968[] = {SGI266880, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59969[] = {SGI266881, SGI0, SGI266882, SGI0, SGI266883, SGI266884, SGI266885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266886, SGI0, NULL};
const Segment * const SG59970[] = {SGI266887, SGI0, SGI266888, SGI0, SGI266889, SGI266890, SGI266891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266892, SGI0, NULL};
const Segment * const SG59971[] = {SGI266893, SGI0, SGI266894, SGI0, SGI266895, SGI266896, SGI266897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59972[] = {SGI266898, SGI0, SGI266899, SGI0, SGI266900, SGI266901, SGI266902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59973[] = {SGI266903, SGI0, SGI266904, SGI0, SGI266905, SGI266906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266907, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59974[] = {SGI266908, SGI0, SGI266909, SGI0, SGI266910, SGI266911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266912, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59975[] = {SGI266913, SGI0, SGI266914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59976[] = {SGI266915, SGI0, SGI266916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59977[] = {SGI266917, SGI0, SGI266918, SGI0, SGI266919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59978[] = {SGI266920, SGI266921, SGI266922, SGI0, SGI266923, SGI266924, SGI266925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59979[] = {SGI266926, SGI266927, SGI0, SGI0, NULL};
const Segment * const SG59980[] = {SGI266928, SGI266929, SGI266930, SGI0, SGI266931, SGI266932, SGI266933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59981[] = {SGI266934, SGI266935, SGI266936, SGI0, SGI266937, SGI266938, SGI266939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59982[] = {SGI266940, SGI266941, SGI266942, SGI0, SGI266943, SGI266944, SGI266945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59983[] = {SGI266946, SGI266947, SGI266948, SGI0, SGI266949, SGI266950, SGI266951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59984[] = {SGI266952, SGI266953, SGI266954, SGI0, SGI266955, SGI266956, SGI266957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59985[] = {SGI266958, SGI266959, SGI266960, SGI0, SGI266961, SGI266962, SGI266963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59986[] = {SGI266964, SGI0, SGI266965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59987[] = {SGI266966, SGI0, SGI266967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59988[] = {SGI266968, SGI0, SGI266969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59989[] = {SGI266970, SGI266971, SGI266972, SGI0, SGI266973, SGI266974, SGI266973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59990[] = {SGI266975, SGI0, SGI266976, SGI0, SGI266977, SGI266978, SGI266979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266980, SGI0, NULL};
const Segment * const SG59991[] = {SGI266981, SGI0, SGI266982, SGI0, SGI266983, SGI266984, SGI266985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266986, SGI0, NULL};
const Segment * const SG59992[] = {SGI266987, SGI0, SGI266988, SGI0, SGI266989, SGI266990, SGI266991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266992, SGI0, NULL};
const Segment * const SG59993[] = {SGI266993, SGI0, SGI266994, SGI0, SGI266995, SGI266996, SGI266997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI266998, SGI0, NULL};
const Segment * const SG59994[] = {SGI266999, SGI0, SGI267000, SGI0, SGI267001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG59995[] = {SGI267002, SGI267003, NULL};
const Segment * const SG59996[] = {SGI267004, SGI267005, NULL};
const Segment * const SG59997[] = {SGI267006, SGI267007, NULL};
const Segment * const SG59998[] = {SGI267008, SGI267009, NULL};
const Segment * const SG59999[] = {SGI267010, SGI267011, NULL};
const Segment * const SG60000[] = {SGI267012, SGI0, SGI267013, SGI0, SGI267014, SGI267015, SGI267016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60001[] = {SGI267017, SGI0, SGI267018, SGI0, SGI267019, SGI267020, SGI267021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60002[] = {SGI267022, SGI267023, SGI0, SGI0, NULL};
const Segment * const SG60003[] = {SGI267024, SGI267025, SGI0, SGI0, NULL};
const Segment * const SG60004[] = {SGI267026, SGI0, SGI267027, SGI0, SGI267028, SGI267029, SGI267030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267031, SGI0, NULL};
const Segment * const SG60005[] = {SGI267032, SGI0, SGI267033, SGI0, SGI267034, SGI267035, SGI267036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267037, SGI0, NULL};
const Segment * const SG60006[] = {SGI267038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60007[] = {SGI267039, SGI0, SGI0, SGI0, SGI0, SGI267040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60008[] = {SGI267041, SGI0, SGI267042, SGI0, SGI267043, SGI267044, SGI267045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267046, SGI0, NULL};
const Segment * const SG60009[] = {SGI267047, SGI0, SGI267048, SGI0, SGI267049, SGI267050, SGI267051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267052, SGI0, NULL};
const Segment * const SG60010[] = {SGI267053, SGI0, SGI267054, SGI0, SGI267055, SGI267056, SGI267057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267058, SGI0, NULL};
const Segment * const SG60011[] = {SGI267059, SGI0, SGI267060, SGI0, SGI267061, SGI267062, SGI267063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267064, SGI0, NULL};
const Segment * const SG60012[] = {SGI267065, SGI267066, SGI267067, SGI0, SGI267068, SGI267069, SGI267070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60013[] = {SGI267071, SGI0, SGI267072, SGI0, SGI267073, SGI267074, SGI267075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267076, SGI0, NULL};
const Segment * const SG60014[] = {SGI267077, SGI0, SGI267078, SGI0, SGI267079, SGI267080, SGI267081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267082, SGI0, NULL};
const Segment * const SG60015[] = {SGI267083, SGI0, SGI267084, SGI0, SGI267085, SGI267086, SGI267087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267088, SGI0, NULL};
const Segment * const SG60016[] = {SGI267089, SGI267090, SGI0, SGI0, NULL};
const Segment * const SG60017[] = {SGI267091, SGI267092, SGI0, SGI0, NULL};
const Segment * const SG60018[] = {SGI267093, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60019[] = {SGI267094, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60020[] = {SGI267095, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60021[] = {SGI267096, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60022[] = {SGI267097, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60023[] = {SGI267098, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60024[] = {SGI267099, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60025[] = {SGI267100, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60026[] = {SGI267101, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60027[] = {SGI267102, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60028[] = {SGI267103, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60029[] = {SGI267104, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60030[] = {SGI267105, SGI0, SGI267106, SGI0, SGI267107, SGI267108, SGI267109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267110, SGI0, NULL};
const Segment * const SG60031[] = {SGI267111, SGI0, SGI267112, SGI0, SGI267113, SGI267114, SGI267115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267116, SGI0, NULL};
const Segment * const SG60032[] = {SGI267117, SGI267118, NULL};
const Segment * const SG60033[] = {SGI267119, SGI0, SGI267120, SGI267121, SGI267122, SGI267123, SGI267124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60034[] = {SGI267125, SGI0, SGI267126, SGI267127, SGI267128, SGI267129, SGI267130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60035[] = {SGI267131, SGI0, SGI267132, SGI267133, SGI267134, SGI267135, SGI267136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60036[] = {SGI267137, SGI0, SGI267138, SGI267139, SGI267140, SGI267141, SGI267142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60037[] = {SGI267143, SGI0, SGI267144, SGI267145, SGI267146, SGI267147, SGI267148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60038[] = {SGI267149, SGI0, SGI267150, SGI267151, SGI267152, SGI267153, SGI267154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60039[] = {SGI267155, SGI0, SGI267156, SGI267157, SGI267158, SGI267159, SGI267160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60040[] = {SGI267161, SGI0, SGI267162, SGI267163, SGI267164, SGI267165, SGI267166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60041[] = {SGI267167, SGI0, SGI267168, SGI267169, SGI267170, SGI267171, SGI267172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60042[] = {SGI267173, SGI267174, SGI267175, SGI0, SGI267176, SGI267177, SGI267178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60043[] = {SGI267179, SGI267180, SGI267181, SGI0, SGI267182, SGI267183, SGI267184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60044[] = {SGI267185, SGI0, SGI267186, SGI0, SGI267187, SGI267188, SGI267189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267190, SGI0, NULL};
const Segment * const SG60045[] = {SGI267191, SGI0, SGI267192, SGI0, SGI267193, SGI267194, SGI267195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267196, SGI0, NULL};
const Segment * const SG60046[] = {SGI267197, SGI0, SGI267198, SGI0, SGI267199, SGI267200, SGI267201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267202, SGI0, NULL};
const Segment * const SG60047[] = {SGI267203, SGI0, SGI267204, SGI0, SGI267205, SGI267206, SGI267207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267208, SGI0, NULL};
const Segment * const SG60048[] = {SGI267209, SGI0, SGI267210, SGI0, SGI267211, SGI267212, SGI267213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267214, SGI0, NULL};
const Segment * const SG60049[] = {SGI267215, SGI267216, NULL};
const Segment * const SG60050[] = {SGI267217, SGI0, SGI267218, SGI0, SGI267219, SGI267220, SGI267221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267222, SGI0, NULL};
const Segment * const SG60051[] = {SGI267223, SGI0, SGI267224, SGI0, SGI267225, SGI267226, SGI267227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267228, SGI0, NULL};
const Segment * const SG60052[] = {SGI267229, SGI0, SGI267230, SGI0, SGI267231, SGI267232, SGI267233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267234, SGI0, NULL};
const Segment * const SG60053[] = {SGI267235, SGI0, SGI267236, SGI0, SGI267237, SGI267238, SGI267239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267240, SGI0, NULL};
const Segment * const SG60054[] = {SGI267241, SGI0, SGI267242, SGI0, SGI267243, SGI267244, SGI267245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267246, SGI0, NULL};
const Segment * const SG60055[] = {SGI267247, SGI0, SGI267248, SGI0, SGI267249, SGI267250, SGI267251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267252, SGI0, NULL};
const Segment * const SG60056[] = {SGI267253, SGI0, SGI267254, SGI0, SGI267255, SGI267256, SGI267257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267258, SGI0, NULL};
const Segment * const SG60057[] = {SGI267259, SGI0, SGI267260, SGI0, SGI267261, SGI267262, SGI267263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267264, SGI0, NULL};
const Segment * const SG60058[] = {SGI267265, SGI0, SGI267266, SGI0, SGI267267, SGI267268, SGI267269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267270, SGI0, NULL};
const Segment * const SG60059[] = {SGI267271, SGI0, SGI267272, SGI0, SGI267273, SGI267274, SGI267275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267276, SGI0, NULL};
const Segment * const SG60060[] = {SGI267277, SGI0, SGI267278, SGI0, SGI267279, SGI267280, SGI267281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60061[] = {SGI267282, SGI0, SGI267283, SGI0, SGI267284, SGI267285, SGI267286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60062[] = {SGI267287, SGI0, SGI267288, SGI0, SGI267289, SGI267290, SGI267291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60063[] = {SGI267292, SGI267293, SGI267294, SGI0, SGI267295, SGI267296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60064[] = {SGI267297, SGI267298, SGI267299, SGI0, SGI267300, SGI267301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60065[] = {SGI267302, SGI267303, SGI267304, SGI0, SGI267305, SGI267306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60066[] = {SGI267307, SGI267308, SGI267309, SGI0, SGI267310, SGI267311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60067[] = {SGI267312, SGI267313, SGI267314, SGI0, SGI267315, SGI267316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60068[] = {SGI267317, SGI267318, SGI267319, SGI0, SGI267320, SGI267321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60069[] = {SGI267322, SGI0, SGI267323, SGI267324, SGI267325, SGI267326, SGI267327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60070[] = {SGI267328, SGI0, SGI267329, SGI267330, SGI267331, SGI267332, SGI267333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60071[] = {SGI267334, SGI0, SGI267335, SGI267336, SGI267337, SGI267338, SGI267339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60072[] = {SGI267340, SGI0, SGI267341, SGI267342, SGI267343, SGI267344, SGI267345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60073[] = {SGI267346, SGI0, SGI267347, SGI267348, SGI267349, SGI267350, SGI267351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60074[] = {SGI267352, SGI0, SGI267353, SGI267354, SGI267355, SGI267356, SGI267357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60075[] = {SGI267358, SGI267359, SGI267360, SGI0, SGI267361, SGI267362, SGI267363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60076[] = {SGI267364, SGI267365, SGI267366, SGI0, SGI267367, SGI267368, SGI267369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60077[] = {SGI267370, SGI267371, SGI267372, SGI0, SGI267373, SGI267374, SGI267375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60078[] = {SGI267376, SGI267377, SGI267378, SGI0, SGI267379, SGI267380, SGI267381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60079[] = {SGI267382, SGI0, SGI267383, SGI0, SGI267384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60080[] = {SGI267385, SGI0, SGI267386, SGI0, SGI267387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60081[] = {SGI267388, SGI0, SGI267389, SGI267390, SGI267391, SGI267392, SGI267393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60082[] = {SGI267394, SGI0, SGI267395, SGI267396, SGI267397, SGI267398, SGI267399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60083[] = {SGI267400, SGI0, SGI267401, SGI267402, SGI267403, SGI267404, SGI267405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60084[] = {SGI267406, SGI0, SGI267407, SGI267408, SGI267409, SGI267410, SGI267411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60085[] = {SGI267412, SGI0, SGI267413, SGI267414, SGI267415, SGI267416, SGI267417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60086[] = {SGI267418, SGI0, SGI267419, SGI0, SGI267420, SGI267421, SGI267422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267423, SGI0, NULL};
const Segment * const SG60087[] = {SGI267424, SGI0, SGI267425, SGI0, SGI267426, SGI267427, SGI267428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267429, SGI0, NULL};
const Segment * const SG60088[] = {SGI267430, SGI0, SGI267431, SGI0, SGI267432, SGI267433, SGI267434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267435, SGI0, NULL};
const Segment * const SG60089[] = {SGI267436, SGI0, SGI267437, SGI0, SGI267438, SGI267439, SGI267440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267441, SGI0, NULL};
const Segment * const SG60090[] = {SGI267442, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60091[] = {SGI267443, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60092[] = {SGI267444, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60093[] = {SGI267445, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60094[] = {SGI267446, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60095[] = {SGI267447, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60096[] = {SGI267448, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60097[] = {SGI267449, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60098[] = {SGI267450, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60099[] = {SGI267451, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60100[] = {SGI267452, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60101[] = {SGI267453, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60102[] = {SGI267454, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60103[] = {SGI267455, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60104[] = {SGI267456, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60105[] = {SGI267457, SGI267458, SGI267459, SGI0, SGI267460, SGI267461, SGI267462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267463, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60106[] = {SGI267464, SGI267465, SGI0, SGI0, NULL};
const Segment * const SG60107[] = {SGI267466, SGI267467, SGI267468, SGI0, SGI267469, SGI267470, SGI267471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60108[] = {SGI267472, SGI267473, SGI267474, SGI0, SGI267475, SGI267476, SGI267477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60109[] = {SGI267478, SGI267479, SGI267480, SGI0, SGI267481, SGI267482, SGI267483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60110[] = {SGI267484, SGI267485, SGI267486, SGI0, SGI267487, SGI267488, SGI267489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60111[] = {SGI267490, SGI267491, SGI267492, SGI0, SGI267493, SGI267494, SGI267495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60112[] = {SGI267496, SGI267497, SGI267498, SGI0, SGI267499, SGI267500, SGI267501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60113[] = {SGI267502, SGI267503, SGI267504, SGI0, SGI267505, SGI267506, SGI267507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60114[] = {SGI267508, SGI267509, SGI267510, SGI0, SGI267511, SGI267512, SGI267513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60115[] = {SGI267514, SGI267515, SGI267516, SGI0, SGI267517, SGI267518, SGI267519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60116[] = {SGI267520, SGI267521, NULL};
const Segment * const SG60117[] = {SGI267522, SGI0, SGI267523, SGI0, SGI267524, SGI267525, SGI267526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60118[] = {SGI267527, SGI0, SGI267528, SGI0, SGI267529, SGI267530, SGI267531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60119[] = {SGI267532, SGI0, SGI267533, SGI0, SGI267534, SGI267535, SGI267536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60120[] = {SGI267537, SGI0, SGI267538, SGI0, SGI267539, SGI267540, SGI267541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60121[] = {SGI267542, SGI0, SGI267543, SGI0, SGI267544, SGI267545, SGI267546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267547, SGI0, NULL};
const Segment * const SG60122[] = {SGI267548, SGI0, SGI267549, SGI0, SGI267550, SGI267551, SGI267552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267553, SGI0, NULL};
const Segment * const SG60123[] = {SGI267554, SGI0, SGI267555, SGI0, SGI267556, SGI267557, SGI267558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267559, SGI0, NULL};
const Segment * const SG60124[] = {SGI267560, SGI0, SGI267561, SGI0, SGI267562, SGI267563, SGI267564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267565, SGI0, NULL};
const Segment * const SG60125[] = {SGI267566, SGI267567, SGI0, SGI0, NULL};
const Segment * const SG60126[] = {SGI267568, SGI267569, SGI0, SGI0, NULL};
const Segment * const SG60127[] = {SGI267570, SGI267571, SGI0, SGI0, NULL};
const Segment * const SG60128[] = {SGI267572, SGI0, SGI267573, SGI0, SGI267574, SGI267575, SGI267576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267577, SGI0, NULL};
const Segment * const SG60129[] = {SGI267578, SGI0, SGI267579, SGI0, SGI267580, SGI267581, SGI267582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267583, SGI0, NULL};
const Segment * const SG60130[] = {SGI267584, SGI267585, NULL};
const Segment * const SG60131[] = {SGI267586, SGI267587, NULL};
const Segment * const SG60132[] = {SGI267588, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60133[] = {SGI267589, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60134[] = {SGI267590, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60135[] = {SGI267591, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60136[] = {SGI267592, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60137[] = {SGI267593, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60138[] = {SGI267594, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60139[] = {SGI267595, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60140[] = {SGI267596, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60141[] = {SGI267597, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60142[] = {SGI267598, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60143[] = {SGI267599, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60144[] = {SGI267600, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60145[] = {SGI267601, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60146[] = {SGI267602, SGI267603, SGI267604, SGI0, SGI267605, SGI267606, SGI267607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267608, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60147[] = {SGI267609, SGI267610, NULL};
const Segment * const SG60148[] = {SGI267611, SGI267612, NULL};
const Segment * const SG60149[] = {SGI267613, SGI267614, NULL};
const Segment * const SG60150[] = {SGI267615, SGI0, SGI267616, SGI0, SGI267617, SGI267618, SGI267619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60151[] = {SGI267620, SGI0, SGI267621, SGI0, SGI267622, SGI267623, SGI267624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60152[] = {SGI267625, SGI267626, SGI267627, SGI0, SGI267628, SGI267629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60153[] = {SGI267630, SGI267631, SGI267632, SGI0, SGI267633, SGI267634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60154[] = {SGI267635, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60155[] = {SGI267636, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60156[] = {SGI267637, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60157[] = {SGI267638, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60158[] = {SGI267639, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60159[] = {SGI267640, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60160[] = {SGI267641, SGI267642, NULL};
const Segment * const SG60161[] = {SGI267643, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60162[] = {SGI267644, SGI267645, NULL};
const Segment * const SG60163[] = {SGI267646, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60164[] = {SGI267647, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60165[] = {SGI267648, SGI267649, NULL};
const Segment * const SG60166[] = {SGI267650, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60167[] = {SGI267651, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60168[] = {SGI267652, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60169[] = {SGI267653, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60170[] = {SGI267654, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60171[] = {SGI267655, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60172[] = {SGI267656, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60173[] = {SGI267657, SGI267658, SGI267659, SGI0, SGI267660, SGI267661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60174[] = {SGI267662, SGI267663, SGI267664, SGI0, SGI267665, SGI267666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60175[] = {SGI267667, SGI267668, SGI267669, SGI0, SGI267670, SGI267671, SGI267672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60176[] = {SGI267673, SGI267674, SGI267675, SGI0, SGI267676, SGI267677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60177[] = {SGI267678, SGI0, SGI267679, SGI0, SGI267680, SGI267681, SGI267682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267683, SGI0, NULL};
const Segment * const SG60178[] = {SGI267684, SGI0, SGI267685, SGI0, SGI267686, SGI267687, SGI267688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267689, SGI0, NULL};
const Segment * const SG60179[] = {SGI267690, SGI0, SGI267691, SGI0, SGI267692, SGI267693, SGI267694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267695, SGI0, NULL};
const Segment * const SG60180[] = {SGI267696, SGI0, SGI267697, SGI0, SGI267698, SGI267699, SGI267700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267701, SGI0, NULL};
const Segment * const SG60181[] = {SGI267702, SGI0, SGI267703, SGI0, SGI267704, SGI267705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267706, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60182[] = {SGI267707, SGI0, SGI267708, SGI0, SGI267709, SGI267710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267711, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60183[] = {SGI267712, SGI0, SGI267713, SGI0, SGI267714, SGI267715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267716, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60184[] = {SGI267717, SGI0, SGI267718, SGI0, SGI267719, SGI267720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267721, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60185[] = {SGI267722, SGI0, SGI267723, SGI0, SGI267724, SGI267725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267726, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60186[] = {SGI267727, SGI0, SGI267728, SGI0, SGI267729, SGI267730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267731, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60187[] = {SGI267732, SGI267733, SGI267734, SGI0, SGI267735, SGI267736, SGI267737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60188[] = {SGI267738, SGI267739, SGI267740, SGI0, SGI267741, SGI267742, SGI267743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60189[] = {SGI267744, SGI267745, SGI267746, SGI0, SGI267747, SGI267748, SGI267749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60190[] = {SGI267750, SGI267751, SGI267752, SGI0, SGI267753, SGI267754, SGI267755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60191[] = {SGI267756, SGI0, SGI267757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60192[] = {SGI267758, SGI0, SGI267759, SGI0, SGI267760, SGI267761, SGI267762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60193[] = {SGI267763, SGI0, SGI267764, SGI0, SGI267765, SGI267766, SGI267767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60194[] = {SGI267768, SGI267769, SGI267770, SGI0, SGI267771, SGI267772, SGI267773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267774, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60195[] = {SGI267775, SGI0, SGI267776, SGI0, SGI267777, SGI267778, SGI0, SGI0, SGI0, SGI267779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60196[] = {SGI267780, SGI0, SGI267781, SGI0, SGI267782, SGI267783, SGI0, SGI0, SGI0, SGI267784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60197[] = {SGI267785, SGI0, SGI267786, SGI0, SGI267787, SGI267788, SGI0, SGI0, SGI0, SGI267789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60198[] = {SGI267790, SGI0, SGI267791, SGI0, SGI267792, SGI267793, SGI267794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60199[] = {SGI267795, SGI0, SGI267796, SGI267797, SGI267798, SGI267799, SGI267800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60200[] = {SGI267801, SGI0, SGI267802, SGI267803, SGI267804, SGI267805, SGI267806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60201[] = {SGI267807, SGI0, SGI267808, SGI267809, SGI267810, SGI267811, SGI267812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60202[] = {SGI267813, SGI0, SGI267814, SGI267815, SGI267816, SGI267817, SGI267818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60203[] = {SGI267819, SGI0, SGI267820, SGI267821, SGI267822, SGI267823, SGI267824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60204[] = {SGI267825, SGI0, SGI267826, SGI267827, SGI267828, SGI267829, SGI267830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60205[] = {SGI267831, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60206[] = {SGI267832, SGI0, SGI267833, SGI267834, SGI267835, SGI267836, SGI267837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60207[] = {SGI267838, SGI0, SGI267839, SGI267840, SGI267841, SGI267842, SGI267843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60208[] = {SGI267844, SGI0, SGI267845, SGI267846, SGI267847, SGI267848, SGI267849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60209[] = {SGI267850, SGI0, SGI267851, SGI267852, SGI267853, SGI267854, SGI267855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60210[] = {SGI267856, SGI0, SGI267857, SGI0, SGI267858, SGI267859, SGI267860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267861, SGI0, NULL};
const Segment * const SG60211[] = {SGI267862, SGI0, SGI267863, SGI0, SGI267864, SGI267865, SGI267866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267867, SGI0, NULL};
const Segment * const SG60212[] = {SGI267868, SGI0, SGI267869, SGI0, SGI267870, SGI267871, SGI267872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267873, SGI0, NULL};
const Segment * const SG60213[] = {SGI267874, SGI0, SGI267875, SGI0, SGI267876, SGI267877, SGI267878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267879, SGI0, NULL};
const Segment * const SG60214[] = {SGI267880, SGI0, SGI267881, SGI0, SGI267882, SGI267883, SGI267884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267885, SGI0, NULL};
const Segment * const SG60215[] = {SGI267886, SGI267887, SGI0, SGI0, NULL};
const Segment * const SG60216[] = {SGI267888, SGI0, SGI267889, SGI0, SGI267890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60217[] = {SGI267891, SGI0, SGI267892, SGI0, SGI267893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60218[] = {SGI267894, SGI0, SGI267895, SGI0, SGI267896, SGI267897, SGI267898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267899, SGI0, NULL};
const Segment * const SG60219[] = {SGI267900, SGI0, SGI267901, SGI0, SGI267902, SGI267903, SGI267904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267905, SGI0, NULL};
const Segment * const SG60220[] = {SGI267906, SGI267907, NULL};
const Segment * const SG60221[] = {SGI267908, SGI267909, NULL};
const Segment * const SG60222[] = {SGI267910, SGI267911, SGI267912, SGI0, SGI267913, SGI267914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60223[] = {SGI267915, SGI267916, SGI267917, SGI0, SGI267918, SGI267919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60224[] = {SGI267920, SGI267921, SGI267922, SGI0, SGI267923, SGI267924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60225[] = {SGI267925, SGI267926, SGI267927, SGI0, SGI267928, SGI267929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60226[] = {SGI267930, SGI267931, SGI267932, SGI0, SGI267933, SGI267934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60227[] = {SGI267935, SGI267936, NULL};
const Segment * const SG60228[] = {SGI267937, SGI267938, NULL};
const Segment * const SG60229[] = {SGI267939, SGI267940, NULL};
const Segment * const SG60230[] = {SGI267941, SGI0, SGI267942, SGI0, SGI267943, SGI267944, SGI267945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60231[] = {SGI267946, SGI0, SGI267947, SGI0, SGI267948, SGI267949, SGI267950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60232[] = {SGI267951, SGI0, SGI267952, SGI0, SGI267953, SGI267954, SGI267955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60233[] = {SGI267956, SGI267957, SGI267958, SGI0, SGI267959, SGI267960, SGI267961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI267962, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60234[] = {SGI267963, SGI0, SGI267964, SGI0, SGI267965, SGI267966, SGI267967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60235[] = {SGI267968, SGI0, SGI267969, SGI0, SGI267970, SGI267971, SGI267972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60236[] = {SGI267973, SGI0, SGI267974, SGI0, SGI267975, SGI267976, SGI267977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60237[] = {SGI267978, SGI0, SGI267979, SGI0, SGI267980, SGI267981, SGI267982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60238[] = {SGI267983, SGI0, SGI267984, SGI0, SGI267985, SGI267986, SGI267987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60239[] = {SGI267988, SGI0, SGI267989, SGI0, SGI267990, SGI267991, SGI267992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60240[] = {SGI267993, SGI0, SGI267994, SGI0, SGI267995, SGI267996, SGI267997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60241[] = {SGI267998, SGI0, SGI267999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60242[] = {SGI268000, SGI0, SGI268001, SGI0, SGI268002, SGI268003, SGI268004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268005, SGI0, NULL};
const Segment * const SG60243[] = {SGI268006, SGI0, SGI268007, SGI0, SGI268008, SGI268009, SGI268010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268011, SGI0, NULL};
const Segment * const SG60244[] = {SGI268012, SGI0, SGI268013, SGI0, SGI268014, SGI268015, SGI268016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268017, SGI0, NULL};
const Segment * const SG60245[] = {SGI268018, SGI0, SGI268019, SGI0, SGI268020, SGI268021, SGI268022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268023, SGI0, NULL};
const Segment * const SG60246[] = {SGI268024, SGI0, SGI268025, SGI0, SGI268026, SGI268027, SGI268028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268029, SGI0, NULL};
const Segment * const SG60247[] = {SGI268030, SGI0, SGI268031, SGI0, SGI268032, SGI268033, SGI268034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60248[] = {SGI268035, SGI0, SGI268036, SGI0, SGI268037, SGI268038, SGI268039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60249[] = {SGI268040, SGI0, SGI268041, SGI0, SGI268042, SGI268043, SGI268044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60250[] = {SGI268045, SGI0, SGI268046, SGI0, SGI268047, SGI268048, SGI268049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60251[] = {SGI268050, SGI268051, NULL};
const Segment * const SG60252[] = {SGI268052, SGI268053, NULL};
const Segment * const SG60253[] = {SGI268054, SGI268055, NULL};
const Segment * const SG60254[] = {SGI268056, SGI0, SGI268057, SGI0, SGI268058, SGI268059, SGI268060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268061, SGI0, NULL};
const Segment * const SG60255[] = {SGI268062, SGI0, SGI268063, SGI0, SGI268064, SGI268065, SGI268066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268067, SGI0, NULL};
const Segment * const SG60256[] = {SGI268068, SGI0, SGI268069, SGI0, SGI268070, SGI268071, SGI268072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268073, SGI0, NULL};
const Segment * const SG60257[] = {SGI268074, SGI268075, SGI268076, SGI0, SGI268077, SGI268078, SGI268079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60258[] = {SGI268080, SGI268081, SGI268082, SGI0, SGI268083, SGI268084, SGI268085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60259[] = {SGI268086, SGI268087, SGI268088, SGI0, SGI268089, SGI268090, SGI268091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60260[] = {SGI268092, SGI268093, SGI268094, SGI0, SGI268095, SGI268096, SGI268097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60261[] = {SGI268098, SGI268099, SGI268100, SGI0, SGI268101, SGI268102, SGI268103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60262[] = {SGI268104, SGI268105, SGI268106, SGI0, SGI268107, SGI268108, SGI268109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60263[] = {SGI268110, SGI268111, SGI268112, SGI0, SGI268113, SGI268114, SGI268115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60264[] = {SGI268116, SGI268117, SGI268118, SGI0, SGI268119, SGI268120, SGI268121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60265[] = {SGI268122, SGI0, SGI268123, SGI268124, SGI268125, SGI268126, SGI268127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60266[] = {SGI268128, SGI0, SGI268129, SGI268130, SGI268131, SGI268132, SGI268133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60267[] = {SGI268134, SGI0, SGI268135, SGI268136, SGI268137, SGI268138, SGI268139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60268[] = {SGI268140, SGI268141, NULL};
const Segment * const SG60269[] = {SGI268142, SGI268143, NULL};
const Segment * const SG60270[] = {SGI268144, SGI268145, NULL};
const Segment * const SG60271[] = {SGI268146, SGI268147, NULL};
const Segment * const SG60272[] = {SGI268148, SGI268149, NULL};
const Segment * const SG60273[] = {SGI268150, SGI268151, NULL};
const Segment * const SG60274[] = {SGI268152, SGI268153, NULL};
const Segment * const SG60275[] = {SGI268154, SGI268155, NULL};
const Segment * const SG60276[] = {SGI268156, SGI268157, NULL};
const Segment * const SG60277[] = {SGI268158, SGI268159, SGI0, SGI0, NULL};
const Segment * const SG60278[] = {SGI268160, SGI268161, SGI0, SGI0, NULL};
const Segment * const SG60279[] = {SGI268162, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60280[] = {SGI268163, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60281[] = {SGI268164, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60282[] = {SGI268165, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60283[] = {SGI268166, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60284[] = {SGI268167, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60285[] = {SGI268168, SGI268169, NULL};
const Segment * const SG60286[] = {SGI268170, SGI268171, NULL};
const Segment * const SG60287[] = {SGI268172, SGI268173, NULL};
const Segment * const SG60288[] = {SGI268174, SGI0, SGI268175, SGI0, SGI268176, SGI268177, SGI268178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60289[] = {SGI268179, SGI0, SGI268180, SGI0, SGI268181, SGI268182, SGI268183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60290[] = {SGI268184, SGI0, SGI268185, SGI268186, SGI268187, SGI268188, SGI268189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60291[] = {SGI268190, SGI0, SGI268191, SGI268192, SGI268193, SGI268194, SGI268195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60292[] = {SGI268196, SGI0, SGI268197, SGI268198, SGI268199, SGI268200, SGI268201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60293[] = {SGI268202, SGI0, SGI268203, SGI0, SGI268204, SGI268205, SGI268206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60294[] = {SGI268207, SGI0, SGI268208, SGI0, SGI268209, SGI268210, SGI268211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60295[] = {SGI268212, SGI0, SGI268213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60296[] = {SGI268214, SGI0, SGI268215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60297[] = {SGI268216, SGI0, SGI268217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60298[] = {SGI268218, SGI0, SGI268219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60299[] = {SGI268220, SGI0, SGI268221, SGI0, SGI268222, SGI268223, SGI268224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268225, SGI0, NULL};
const Segment * const SG60300[] = {SGI268226, SGI0, SGI268227, SGI0, SGI268228, SGI268229, SGI268230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268231, SGI0, NULL};
const Segment * const SG60301[] = {SGI268232, SGI268233, SGI0, SGI0, SGI268234, SGI268235, SGI268236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60302[] = {SGI268237, SGI268238, NULL};
const Segment * const SG60303[] = {SGI268239, SGI268240, NULL};
const Segment * const SG60304[] = {SGI268241, SGI268242, NULL};
const Segment * const SG60305[] = {SGI268243, SGI268244, NULL};
const Segment * const SG60306[] = {SGI268245, SGI268246, SGI0, SGI0, NULL};
const Segment * const SG60307[] = {SGI268247, SGI0, SGI268248, SGI0, SGI268249, SGI268250, SGI268251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60308[] = {SGI268252, SGI268253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60309[] = {SGI268254, SGI0, SGI268255, SGI0, SGI268256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60310[] = {SGI268257, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60311[] = {SGI268258, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60312[] = {SGI268259, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60313[] = {SGI268260, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60314[] = {SGI268261, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60315[] = {SGI268262, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60316[] = {SGI268263, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60317[] = {SGI268264, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60318[] = {SGI268265, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60319[] = {SGI268266, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60320[] = {SGI268267, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60321[] = {SGI268268, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60322[] = {SGI268269, SGI268270, NULL};
const Segment * const SG60323[] = {SGI268271, SGI268272, NULL};
const Segment * const SG60324[] = {SGI268273, SGI268274, NULL};
const Segment * const SG60325[] = {SGI268275, SGI268276, NULL};
const Segment * const SG60326[] = {SGI268277, SGI268278, NULL};
const Segment * const SG60327[] = {SGI268279, SGI268280, NULL};
const Segment * const SG60328[] = {SGI268281, SGI268282, NULL};
const Segment * const SG60329[] = {SGI268283, SGI0, SGI268284, SGI268285, SGI268286, SGI268287, SGI268288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60330[] = {SGI268289, SGI0, SGI268290, SGI0, SGI268291, SGI0, SGI268292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268293, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60331[] = {SGI268294, SGI0, SGI268295, SGI0, SGI268296, SGI268297, SGI268298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268299, SGI0, NULL};
const Segment * const SG60332[] = {SGI268300, SGI0, SGI268301, SGI0, SGI268302, SGI268303, SGI268304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268305, SGI0, NULL};
const Segment * const SG60333[] = {SGI268306, SGI0, SGI268307, SGI0, SGI268308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60334[] = {SGI268309, SGI0, SGI268310, SGI0, SGI268311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60335[] = {SGI268312, SGI268313, NULL};
const Segment * const SG60336[] = {SGI268314, SGI268315, NULL};
const Segment * const SG60337[] = {SGI268316, SGI268317, NULL};
const Segment * const SG60338[] = {SGI268318, SGI268319, NULL};
const Segment * const SG60339[] = {SGI268320, SGI268321, NULL};
const Segment * const SG60340[] = {SGI268322, SGI268323, NULL};
const Segment * const SG60341[] = {SGI268324, SGI268325, NULL};
const Segment * const SG60342[] = {SGI268326, SGI0, SGI268327, SGI0, SGI268328, SGI268329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60343[] = {SGI268333, SGI268334, NULL};
const Segment * const SG60344[] = {SGI268330, SGI0, SGI268331, SGI0, SGI268332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60345[] = {SGI268335, SGI268336, NULL};
const Segment * const SG60346[] = {SGI268337, SGI268338, NULL};
const Segment * const SG60347[] = {SGI268339, SGI268340, NULL};
const Segment * const SG60348[] = {SGI268341, SGI268342, NULL};
const Segment * const SG60349[] = {SGI268343, SGI268344, NULL};
const Segment * const SG60350[] = {SGI268345, SGI268346, SGI0, SGI0, SGI0, SGI268347, SGI268348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60351[] = {SGI268349, SGI268350, SGI0, SGI0, SGI268351, SGI268352, SGI268353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60352[] = {SGI268354, SGI0, SGI268355, SGI0, SGI268356, SGI268357, SGI268358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268359, SGI0, NULL};
const Segment * const SG60353[] = {SGI268360, SGI0, SGI268361, SGI0, SGI268362, SGI268363, SGI268364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268365, SGI0, NULL};
const Segment * const SG60354[] = {SGI268366, SGI268367, NULL};
const Segment * const SG60355[] = {SGI268368, SGI268369, NULL};
const Segment * const SG60356[] = {SGI268370, SGI268371, NULL};
const Segment * const SG60357[] = {SGI268372, SGI268373, NULL};
const Segment * const SG60358[] = {SGI268374, SGI268375, NULL};
const Segment * const SG60359[] = {SGI268376, SGI0, SGI268377, SGI0, SGI268378, SGI268379, SGI268380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60360[] = {SGI268381, SGI0, SGI268382, SGI0, SGI268383, SGI268384, SGI268385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60361[] = {SGI268386, SGI0, SGI268387, SGI0, SGI268388, SGI268389, SGI268390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60362[] = {SGI268391, SGI0, SGI268392, SGI0, SGI268393, SGI268394, SGI268395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60363[] = {SGI268396, SGI268397, SGI0, SGI0, NULL};
const Segment * const SG60364[] = {SGI268398, SGI268399, SGI0, SGI0, NULL};
const Segment * const SG60365[] = {SGI268400, SGI0, SGI268401, SGI0, SGI268402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60366[] = {SGI268403, SGI0, SGI268404, SGI0, SGI268405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60367[] = {SGI268406, SGI0, SGI268407, SGI0, SGI268408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60368[] = {SGI268409, SGI0, SGI268410, SGI0, SGI268411, SGI268412, SGI268413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268414, SGI0, NULL};
const Segment * const SG60369[] = {SGI268415, SGI0, SGI268416, SGI0, SGI268417, SGI268418, SGI268419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268420, SGI0, NULL};
const Segment * const SG60370[] = {SGI268421, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60371[] = {SGI268422, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60372[] = {SGI268423, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60373[] = {SGI268424, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60374[] = {SGI268425, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60375[] = {SGI268426, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60376[] = {SGI268427, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60377[] = {SGI268428, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60378[] = {SGI268429, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60379[] = {SGI268430, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60380[] = {SGI268431, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60381[] = {SGI268432, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60382[] = {SGI268433, SGI0, SGI268434, SGI0, SGI268435, SGI268436, SGI268437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268438, SGI0, NULL};
const Segment * const SG60383[] = {SGI268439, SGI0, SGI268440, SGI0, SGI268441, SGI268442, SGI268443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268444, SGI0, NULL};
const Segment * const SG60384[] = {SGI268445, SGI0, SGI268446, SGI0, SGI268447, SGI268448, SGI268449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268450, SGI0, NULL};
const Segment * const SG60385[] = {SGI268451, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60386[] = {SGI268452, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60387[] = {SGI268453, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60388[] = {SGI268454, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60389[] = {SGI268455, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60390[] = {SGI268456, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60391[] = {SGI268457, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60392[] = {SGI268458, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60393[] = {SGI268459, SGI268460, NULL};
const Segment * const SG60394[] = {SGI268461, SGI268462, NULL};
const Segment * const SG60395[] = {SGI268463, SGI268464, NULL};
const Segment * const SG60396[] = {SGI268465, SGI268466, NULL};
const Segment * const SG60397[] = {SGI268467, SGI268468, NULL};
const Segment * const SG60398[] = {SGI268469, SGI268470, NULL};
const Segment * const SG60399[] = {SGI268471, SGI268472, NULL};
const Segment * const SG60400[] = {SGI268473, SGI268474, NULL};
const Segment * const SG60401[] = {SGI268475, SGI268476, NULL};
const Segment * const SG60402[] = {SGI268477, SGI268478, SGI268479, SGI0, SGI268480, SGI268481, SGI268482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60403[] = {SGI268483, SGI268484, SGI268485, SGI0, SGI268486, SGI268487, SGI268488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60404[] = {SGI268489, SGI0, SGI268490, SGI0, SGI268491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60405[] = {SGI268492, SGI0, SGI268493, SGI0, SGI268494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60406[] = {SGI268495, SGI268496, SGI268497, SGI0, SGI268498, SGI268499, SGI268500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60407[] = {SGI268501, SGI268502, NULL};
const Segment * const SG60408[] = {SGI268503, SGI0, SGI268504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60409[] = {SGI268505, SGI0, SGI268506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60410[] = {SGI268507, SGI0, SGI268508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60411[] = {SGI268509, SGI0, SGI268510, SGI268511, SGI268512, SGI268513, SGI268514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60412[] = {SGI268515, SGI0, SGI268516, SGI268517, SGI268518, SGI268519, SGI268520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60413[] = {SGI268521, SGI0, SGI268522, SGI268523, SGI268524, SGI268525, SGI268526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60414[] = {SGI268527, SGI0, SGI268528, SGI268529, SGI268530, SGI268531, SGI268532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60415[] = {SGI268533, SGI268534, NULL};
const Segment * const SG60416[] = {SGI268535, SGI268536, SGI268537, SGI0, SGI268538, SGI268539, SGI268540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60417[] = {SGI268541, SGI268542, SGI268543, SGI0, SGI268544, SGI268545, SGI268546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60418[] = {SGI268547, SGI0, SGI268548, SGI0, SGI268549, SGI268550, SGI268551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268552, SGI0, NULL};
const Segment * const SG60419[] = {SGI268553, SGI0, SGI268554, SGI0, SGI268555, SGI268556, SGI268557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268558, SGI0, NULL};
const Segment * const SG60420[] = {SGI268559, SGI0, SGI268560, SGI0, SGI268561, SGI268562, SGI268563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268564, SGI0, NULL};
const Segment * const SG60421[] = {SGI268565, SGI0, SGI268566, SGI0, SGI268567, SGI268568, SGI268569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268570, SGI0, NULL};
const Segment * const SG60422[] = {SGI268571, SGI268572, NULL};
const Segment * const SG60423[] = {SGI268573, SGI268574, SGI0, SGI0, NULL};
const Segment * const SG60424[] = {SGI268575, SGI268576, SGI268577, SGI0, SGI268578, SGI268579, SGI268580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268581, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60425[] = {SGI268582, SGI268583, SGI0, SGI0, NULL};
const Segment * const SG60426[] = {SGI268584, SGI0, SGI268585, SGI0, SGI268586, SGI268587, SGI268588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60427[] = {SGI268589, SGI0, SGI268590, SGI0, SGI268591, SGI268592, SGI268593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60428[] = {SGI268594, SGI0, SGI268595, SGI0, SGI268596, SGI268597, SGI268598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268599, SGI0, NULL};
const Segment * const SG60429[] = {SGI268600, SGI0, SGI268601, SGI0, SGI268602, SGI268603, SGI268604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268605, SGI0, NULL};
const Segment * const SG60430[] = {SGI268606, SGI0, SGI268607, SGI0, SGI268608, SGI268609, SGI268610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268611, SGI0, NULL};
const Segment * const SG60431[] = {SGI268612, SGI268613, SGI268614, SGI0, SGI268615, SGI268616, SGI268617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60432[] = {SGI268618, SGI268619, SGI268620, SGI0, SGI268621, SGI268622, SGI268623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60433[] = {SGI268624, SGI268625, SGI268626, SGI0, SGI268627, SGI268628, SGI268629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60434[] = {SGI268630, SGI268631, SGI268632, SGI0, SGI268633, SGI268634, SGI268635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60435[] = {SGI268636, SGI268637, SGI268638, SGI0, SGI268639, SGI268640, SGI268641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60436[] = {SGI268642, SGI268643, SGI268644, SGI0, SGI268645, SGI268646, SGI268647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60437[] = {SGI268648, SGI268649, SGI0, SGI0, NULL};
const Segment * const SG60438[] = {SGI268650, SGI0, SGI268651, SGI0, SGI268652, SGI268653, SGI268654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268655, SGI0, NULL};
const Segment * const SG60439[] = {SGI268656, SGI0, SGI268657, SGI0, SGI268658, SGI268659, SGI268660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268661, SGI0, NULL};
const Segment * const SG60440[] = {SGI268662, SGI0, SGI268663, SGI0, SGI268664, SGI268665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268666, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60441[] = {SGI268667, SGI0, SGI268668, SGI0, SGI268669, SGI268670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268671, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60442[] = {SGI268672, SGI268673, SGI268674, SGI0, SGI268675, SGI268676, SGI268677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60443[] = {SGI268678, SGI268679, SGI268680, SGI0, SGI268681, SGI268682, SGI268683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60444[] = {SGI268684, SGI0, SGI268685, SGI0, SGI268686, SGI268687, SGI268688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268689, SGI0, NULL};
const Segment * const SG60445[] = {SGI268690, SGI0, SGI268691, SGI0, SGI268692, SGI268693, SGI268694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268695, SGI0, NULL};
const Segment * const SG60446[] = {SGI268696, SGI0, SGI268697, SGI0, SGI268698, SGI268699, SGI268700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268701, SGI0, NULL};
const Segment * const SG60447[] = {SGI268702, SGI0, SGI268703, SGI268704, SGI268705, SGI268706, SGI268707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60448[] = {SGI268708, SGI0, SGI268709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60449[] = {SGI268710, SGI0, SGI268711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60450[] = {SGI268712, SGI268713, SGI0, SGI0, NULL};
const Segment * const SG60451[] = {SGI268714, SGI268715, SGI0, SGI0, NULL};
const Segment * const SG60452[] = {SGI268716, SGI0, SGI268717, SGI0, SGI268718, SGI268719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60453[] = {SGI268720, SGI0, SGI268721, SGI0, SGI268722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60454[] = {SGI268723, SGI268724, SGI268725, SGI0, SGI268726, SGI268727, SGI268728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60455[] = {SGI268729, SGI268730, SGI268731, SGI0, SGI268732, SGI268733, SGI268734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60456[] = {SGI268735, SGI268736, SGI268737, SGI0, SGI268738, SGI268739, SGI268740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60457[] = {SGI268741, SGI268742, SGI268743, SGI0, SGI268744, SGI268745, SGI268746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60458[] = {SGI268747, SGI0, SGI268748, SGI0, SGI268749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60459[] = {SGI268750, SGI0, SGI268751, SGI0, SGI268752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60460[] = {SGI268753, SGI268754, SGI268755, SGI0, NULL};
const Segment * const SG60461[] = {SGI268756, SGI268757, SGI268758, SGI0, NULL};
const Segment * const SG60462[] = {SGI268759, SGI0, SGI268760, SGI0, SGI268761, SGI268762, SGI268763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268764, SGI0, NULL};
const Segment * const SG60463[] = {SGI268765, SGI0, SGI268766, SGI0, SGI268767, SGI268768, SGI268769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268770, SGI0, NULL};
const Segment * const SG60464[] = {SGI268771, SGI0, SGI268772, SGI0, SGI268773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60465[] = {SGI268774, SGI0, SGI268775, SGI0, SGI268776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60466[] = {SGI268777, SGI268778, SGI0, SGI0, NULL};
const Segment * const SG60467[] = {SGI268779, SGI268780, NULL};
const Segment * const SG60468[] = {SGI268781, SGI268782, NULL};
const Segment * const SG60469[] = {SGI268783, SGI268784, NULL};
const Segment * const SG60470[] = {SGI268785, SGI0, SGI268786, SGI0, SGI268787, SGI268788, SGI268789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268790, SGI0, NULL};
const Segment * const SG60471[] = {SGI268791, SGI0, SGI268792, SGI0, SGI268793, SGI268794, SGI268795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268796, SGI0, NULL};
const Segment * const SG60472[] = {SGI268797, SGI0, SGI268798, SGI0, SGI268799, SGI268800, SGI268801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268802, SGI0, NULL};
const Segment * const SG60473[] = {SGI268803, SGI0, SGI268804, SGI0, SGI268805, SGI268806, SGI268807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268808, SGI0, NULL};
const Segment * const SG60474[] = {SGI268809, SGI0, SGI268810, SGI0, SGI268811, SGI268812, SGI268813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268814, SGI0, NULL};
const Segment * const SG60475[] = {SGI268815, SGI0, SGI268816, SGI0, SGI268817, SGI268818, SGI268819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268820, SGI0, NULL};
const Segment * const SG60476[] = {SGI268822, SGI268823, NULL};
const Segment * const SG60477[] = {SGI268821, SGI268824, SGI268825, SGI0, SGI268826, SGI268827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60478[] = {SGI268828, SGI0, SGI268829, SGI0, SGI268830, SGI268831, SGI268832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60479[] = {SGI268833, SGI0, SGI268834, SGI0, SGI268835, SGI268836, SGI268837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60480[] = {SGI268838, SGI0, SGI268839, SGI0, SGI268840, SGI268841, SGI268842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60481[] = {SGI268843, SGI268844, SGI0, SGI0, NULL};
const Segment * const SG60482[] = {SGI268845, SGI268846, NULL};
const Segment * const SG60483[] = {SGI268847, SGI0, SGI268848, SGI0, SGI268849, SGI268850, SGI268851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268852, SGI0, NULL};
const Segment * const SG60484[] = {SGI268853, SGI0, SGI268854, SGI0, SGI268855, SGI268856, SGI268857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268858, SGI0, NULL};
const Segment * const SG60485[] = {SGI268859, SGI0, SGI268860, SGI0, SGI268861, SGI268862, SGI268863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268864, SGI0, NULL};
const Segment * const SG60486[] = {SGI268865, SGI0, SGI268866, SGI0, SGI268867, SGI268868, SGI268869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268870, SGI0, NULL};
const Segment * const SG60487[] = {SGI268871, SGI268872, NULL};
const Segment * const SG60488[] = {SGI268873, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60489[] = {SGI268874, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60490[] = {SGI268875, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60491[] = {SGI268876, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60492[] = {SGI268877, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60493[] = {SGI268878, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60494[] = {SGI268879, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60495[] = {SGI268880, SGI0, SGI268881, SGI0, SGI268882, SGI268883, SGI268884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268885, SGI0, NULL};
const Segment * const SG60496[] = {SGI268886, SGI0, SGI268887, SGI0, SGI268888, SGI268889, SGI268890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268891, SGI0, NULL};
const Segment * const SG60497[] = {SGI268892, SGI0, SGI268893, SGI0, SGI268894, SGI268895, SGI268896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268897, SGI0, NULL};
const Segment * const SG60498[] = {SGI268898, SGI0, SGI268899, SGI0, SGI268900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60499[] = {SGI268901, SGI0, SGI268902, SGI0, SGI268903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60500[] = {SGI268904, SGI0, SGI268905, SGI0, SGI268906, SGI268907, SGI268908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268909, SGI0, NULL};
const Segment * const SG60501[] = {SGI268910, SGI0, SGI268911, SGI0, SGI268912, SGI268913, SGI268914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268915, SGI0, NULL};
const Segment * const SG60502[] = {SGI268916, SGI0, SGI268917, SGI0, SGI268918, SGI268919, SGI268920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268921, SGI0, NULL};
const Segment * const SG60503[] = {SGI268922, SGI0, SGI268923, SGI0, SGI268924, SGI268925, SGI268926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268927, SGI0, NULL};
const Segment * const SG60504[] = {SGI268928, SGI0, SGI268929, SGI0, SGI268930, SGI268931, SGI268932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60505[] = {SGI268933, SGI0, SGI268934, SGI0, SGI268935, SGI268936, SGI268937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60506[] = {SGI268938, SGI0, SGI268939, SGI0, SGI268940, SGI268941, SGI268942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268943, SGI0, NULL};
const Segment * const SG60507[] = {SGI268944, SGI268945, SGI268946, SGI0, SGI268947, SGI268948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60508[] = {SGI268949, SGI0, SGI268950, SGI0, SGI268951, SGI268952, SGI268953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268954, SGI0, NULL};
const Segment * const SG60509[] = {SGI268955, SGI0, SGI268956, SGI0, SGI268957, SGI268958, SGI268959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268960, SGI0, NULL};
const Segment * const SG60510[] = {SGI268961, SGI0, SGI268962, SGI0, SGI268963, SGI268964, SGI268965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI268966, SGI0, NULL};
const Segment * const SG60511[] = {SGI268967, SGI268968, SGI268969, SGI0, SGI268970, SGI268971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60512[] = {SGI268972, SGI268973, SGI268974, SGI0, SGI268975, SGI268976, SGI268977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60513[] = {SGI268978, SGI268979, SGI268980, SGI0, SGI268981, SGI268982, SGI268983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60514[] = {SGI268984, SGI0, SGI268985, SGI268986, SGI268987, SGI268988, SGI268989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60515[] = {SGI268990, SGI0, SGI268991, SGI268992, SGI268993, SGI268994, SGI268995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60516[] = {SGI268996, SGI0, SGI268997, SGI268998, SGI268999, SGI269000, SGI269001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60517[] = {SGI269002, SGI269003, SGI269004, SGI0, SGI269005, SGI269006, SGI269007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60518[] = {SGI269008, SGI269009, SGI269010, SGI0, SGI269011, SGI269012, SGI269013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60519[] = {SGI269014, SGI0, SGI269015, SGI0, SGI269016, SGI269017, SGI269018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60520[] = {SGI269019, SGI0, SGI269020, SGI0, SGI269021, SGI269022, SGI269023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60521[] = {SGI269024, SGI0, SGI269025, SGI0, SGI269026, SGI269027, SGI269028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269029, SGI0, NULL};
const Segment * const SG60522[] = {SGI269030, SGI0, SGI269031, SGI0, SGI269032, SGI269033, SGI269034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269035, SGI0, NULL};
const Segment * const SG60523[] = {SGI269036, SGI0, SGI269037, SGI0, SGI269038, SGI269039, SGI269040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269041, SGI0, NULL};
const Segment * const SG60524[] = {SGI269042, SGI0, SGI269043, SGI0, SGI269044, SGI269045, SGI269046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269047, SGI0, NULL};
const Segment * const SG60525[] = {SGI269048, SGI0, SGI269049, SGI0, SGI269050, SGI269051, SGI269052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269053, SGI0, NULL};
const Segment * const SG60526[] = {SGI269054, SGI269055, SGI269056, SGI0, SGI269057, SGI269058, SGI269059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60527[] = {SGI269060, SGI0, SGI269061, SGI0, SGI269062, SGI269063, SGI269064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269065, SGI0, NULL};
const Segment * const SG60528[] = {SGI269066, SGI0, SGI269067, SGI0, SGI269068, SGI269069, SGI269070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269071, SGI0, NULL};
const Segment * const SG60529[] = {SGI269072, SGI0, SGI269073, SGI0, SGI269074, SGI269075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269076, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60530[] = {SGI269077, SGI0, SGI269078, SGI0, SGI269079, SGI269080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269081, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60531[] = {SGI269082, SGI269083, SGI269084, SGI0, SGI269085, SGI269086, SGI269087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60532[] = {SGI269088, SGI269089, SGI269090, SGI0, SGI269091, SGI269092, SGI269093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60533[] = {SGI269094, SGI269095, NULL};
const Segment * const SG60534[] = {SGI269096, SGI269097, SGI269098, SGI0, SGI269099, SGI269100, SGI269101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60535[] = {SGI269102, SGI269103, SGI269104, SGI0, SGI269105, SGI269106, SGI269107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60536[] = {SGI269108, SGI269109, SGI269110, SGI0, SGI269111, SGI269112, SGI269113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60537[] = {SGI269114, SGI269115, SGI269116, SGI0, SGI269117, SGI269118, SGI269119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60538[] = {SGI269120, SGI269121, SGI269122, SGI0, SGI269123, SGI269124, SGI269125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60539[] = {SGI269126, SGI0, SGI269127, SGI0, SGI269128, SGI269129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269130, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60540[] = {SGI269131, SGI0, SGI269132, SGI0, SGI269133, SGI269134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269135, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60541[] = {SGI269136, SGI0, SGI269137, SGI0, SGI269138, SGI269139, SGI269140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269141, SGI0, NULL};
const Segment * const SG60542[] = {SGI269142, SGI0, SGI269143, SGI0, SGI269144, SGI269145, SGI269146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60543[] = {SGI269147, SGI0, SGI269148, SGI0, SGI269149, SGI269150, SGI269151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60544[] = {SGI269152, SGI269153, SGI0, SGI0, SGI269154, SGI269155, SGI269156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60545[] = {SGI269157, SGI269158, NULL};
const Segment * const SG60546[] = {SGI269159, SGI269160, NULL};
const Segment * const SG60547[] = {SGI269161, SGI0, SGI269162, SGI0, SGI269163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60548[] = {SGI269164, SGI0, SGI269165, SGI0, SGI269166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60549[] = {SGI269167, SGI269168, NULL};
const Segment * const SG60550[] = {SGI269169, SGI269170, NULL};
const Segment * const SG60551[] = {SGI269171, SGI0, SGI269172, SGI0, SGI269173, SGI269174, SGI269175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60552[] = {SGI269176, SGI269177, NULL};
const Segment * const SG60553[] = {SGI269178, SGI0, SGI269179, SGI0, SGI269180, SGI269181, SGI269182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60554[] = {SGI269183, SGI269184, NULL};
const Segment * const SG60555[] = {SGI269185, SGI0, SGI269186, SGI0, SGI269187, SGI269188, SGI269189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269190, SGI0, NULL};
const Segment * const SG60556[] = {SGI269191, SGI0, SGI269192, SGI0, SGI269193, SGI269194, SGI269195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269196, SGI0, NULL};
const Segment * const SG60557[] = {SGI269197, SGI0, SGI269198, SGI0, SGI269199, SGI269200, SGI269201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269202, SGI0, NULL};
const Segment * const SG60558[] = {SGI269203, SGI0, SGI269204, SGI0, SGI269205, SGI269206, SGI269207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269208, SGI0, NULL};
const Segment * const SG60559[] = {SGI269209, SGI0, SGI269210, SGI0, SGI269211, SGI269212, SGI269213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60560[] = {SGI269214, SGI0, SGI269215, SGI0, SGI269216, SGI269217, SGI269218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60561[] = {SGI269219, SGI0, SGI269220, SGI0, SGI269221, SGI269222, SGI269223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269224, SGI0, NULL};
const Segment * const SG60562[] = {SGI269225, SGI0, SGI269226, SGI0, SGI269227, SGI269228, SGI269229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269230, SGI0, NULL};
const Segment * const SG60563[] = {SGI269231, SGI0, SGI269232, SGI0, SGI269233, SGI269234, SGI269235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269236, SGI0, NULL};
const Segment * const SG60564[] = {SGI269237, SGI269238, NULL};
const Segment * const SG60565[] = {SGI269239, SGI269240, SGI0, SGI0, NULL};
const Segment * const SG60566[] = {SGI269241, SGI269242, SGI269243, SGI0, SGI269244, SGI269245, SGI269246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60567[] = {SGI269247, SGI269248, SGI269249, SGI0, SGI269250, SGI269251, SGI269252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60568[] = {SGI269253, SGI269254, SGI269255, SGI0, SGI269256, SGI269257, SGI269258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60569[] = {SGI269259, SGI269260, SGI269261, SGI0, SGI269262, SGI269263, SGI269264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60570[] = {SGI269265, SGI269266, SGI269267, SGI0, SGI269268, SGI269269, SGI269270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60571[] = {SGI269271, SGI269272, SGI269273, SGI0, SGI269274, SGI269275, SGI269276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60572[] = {SGI269277, SGI269278, SGI269279, SGI0, SGI269280, SGI269281, SGI269282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60573[] = {SGI269283, SGI269284, SGI269285, SGI0, SGI269286, SGI269287, SGI269288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60574[] = {SGI269289, SGI0, SGI269290, SGI0, SGI269291, SGI269292, SGI269293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60575[] = {SGI269294, SGI0, SGI269295, SGI0, SGI269296, SGI269297, SGI269298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60576[] = {SGI269299, SGI0, SGI269300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60577[] = {SGI269301, SGI0, SGI269302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60578[] = {SGI269303, SGI0, SGI269304, SGI0, SGI269305, SGI269306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269307, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60579[] = {SGI269308, SGI0, SGI269309, SGI0, SGI269310, SGI269311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269312, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60580[] = {SGI269313, SGI269314, SGI0, SGI0, NULL};
const Segment * const SG60581[] = {SGI269315, SGI0, SGI269316, SGI0, SGI269317, SGI269318, SGI269319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60582[] = {SGI269320, SGI0, SGI269321, SGI0, SGI269322, SGI269323, SGI269324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60583[] = {SGI269325, SGI269326, SGI269327, SGI0, SGI269328, SGI269329, SGI269330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60584[] = {SGI269331, SGI269332, NULL};
const Segment * const SG60585[] = {SGI269333, SGI269334, NULL};
const Segment * const SG60586[] = {SGI269335, SGI0, SGI269336, SGI0, SGI269337, SGI269338, SGI269339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60587[] = {SGI269340, SGI269341, SGI269342, SGI0, SGI269343, SGI269344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60588[] = {SGI269345, SGI0, SGI269346, SGI269347, SGI269348, SGI269349, SGI269350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60589[] = {SGI269351, SGI0, SGI269352, SGI0, SGI269353, SGI269354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269355, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60590[] = {SGI269356, SGI0, SGI269357, SGI0, SGI269358, SGI269359, SGI269360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269361, SGI0, NULL};
const Segment * const SG60591[] = {SGI269362, SGI0, SGI269363, SGI0, SGI269364, SGI269365, SGI269366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269367, SGI0, NULL};
const Segment * const SG60592[] = {SGI269368, SGI0, SGI269369, SGI0, SGI269370, SGI269371, SGI269372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269373, SGI0, NULL};
const Segment * const SG60593[] = {SGI269374, SGI0, SGI269375, SGI0, SGI269376, SGI269377, SGI269378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269379, SGI0, NULL};
const Segment * const SG60594[] = {SGI269380, SGI0, SGI269381, SGI0, SGI269382, SGI269383, SGI269384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269385, SGI0, NULL};
const Segment * const SG60595[] = {SGI269386, SGI0, SGI269387, SGI0, SGI269388, SGI269389, SGI269390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269391, SGI0, NULL};
const Segment * const SG60596[] = {SGI269392, SGI0, SGI269393, SGI0, SGI269394, SGI269395, SGI269396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60597[] = {SGI269397, SGI269398, NULL};
const Segment * const SG60598[] = {SGI269399, SGI269400, NULL};
const Segment * const SG60599[] = {SGI269401, SGI0, SGI269402, SGI0, SGI269403, SGI269404, SGI269405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269406, SGI0, NULL};
const Segment * const SG60600[] = {SGI269407, SGI0, SGI269408, SGI0, SGI269409, SGI269410, SGI269411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269412, SGI0, NULL};
const Segment * const SG60601[] = {SGI269413, SGI269414, NULL};
const Segment * const SG60602[] = {SGI269415, SGI269416, NULL};
const Segment * const SG60603[] = {SGI269417, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60604[] = {SGI269418, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60605[] = {SGI269419, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60606[] = {SGI269420, SGI0, SGI269421, SGI269422, SGI269423, SGI269424, SGI269425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60607[] = {SGI269426, SGI0, SGI269427, SGI269428, SGI269429, SGI269430, SGI269431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60608[] = {SGI269432, SGI0, SGI269433, SGI269434, SGI269435, SGI269436, SGI269437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60609[] = {SGI269438, SGI269439, NULL};
const Segment * const SG60610[] = {SGI269440, SGI269441, SGI269442, SGI0, SGI269443, SGI269444, SGI269445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60611[] = {SGI269446, SGI269447, SGI269448, SGI0, SGI269449, SGI269450, SGI269451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60612[] = {SGI269452, SGI269453, NULL};
const Segment * const SG60613[] = {SGI269454, SGI269455, NULL};
const Segment * const SG60614[] = {SGI269456, SGI269457, NULL};
const Segment * const SG60615[] = {SGI269458, SGI269459, NULL};
const Segment * const SG60616[] = {SGI269460, SGI0, SGI269461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60617[] = {SGI269462, SGI0, SGI269463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60618[] = {SGI269464, SGI0, SGI269465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60619[] = {SGI269466, SGI0, SGI269467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60620[] = {SGI269468, SGI0, SGI269469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60621[] = {SGI269470, SGI269471, NULL};
const Segment * const SG60622[] = {SGI269472, SGI269473, NULL};
const Segment * const SG60623[] = {SGI269474, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60624[] = {SGI269475, SGI269476, SGI269477, SGI0, SGI269478, SGI269479, SGI269480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60625[] = {SGI269481, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60626[] = {SGI269482, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60627[] = {SGI269483, SGI269484, NULL};
const Segment * const SG60628[] = {SGI269485, SGI269486, NULL};
const Segment * const SG60629[] = {SGI269487, SGI269488, NULL};
const Segment * const SG60630[] = {SGI269489, SGI0, SGI269490, SGI0, SGI269491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60631[] = {SGI269492, SGI0, SGI269493, SGI0, SGI269494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60632[] = {SGI269495, SGI0, SGI269496, SGI269497, SGI269498, SGI269499, SGI269500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60633[] = {SGI269501, SGI0, SGI269502, SGI269503, SGI269504, SGI269505, SGI269506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60634[] = {SGI269507, SGI269508, NULL};
const Segment * const SG60635[] = {SGI269509, SGI269510, NULL};
const Segment * const SG60636[] = {SGI269511, SGI0, SGI269512, SGI0, SGI269513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60637[] = {SGI269514, SGI0, SGI269515, SGI0, SGI269516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60638[] = {SGI269517, SGI0, SGI269518, SGI0, SGI269519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60639[] = {SGI269520, SGI269521, NULL};
const Segment * const SG60640[] = {SGI269522, SGI269523, NULL};
const Segment * const SG60641[] = {SGI269524, SGI0, SGI269525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60642[] = {SGI269526, SGI0, SGI269527, SGI0, SGI269528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60643[] = {SGI269529, SGI269530, NULL};
const Segment * const SG60644[] = {SGI269531, SGI269532, NULL};
const Segment * const SG60645[] = {SGI269533, SGI269534, SGI0, SGI0, NULL};
const Segment * const SG60646[] = {SGI269535, SGI269536, NULL};
const Segment * const SG60647[] = {SGI269537, SGI269538, NULL};
const Segment * const SG60648[] = {SGI269539, SGI0, SGI269540, SGI0, SGI269541, SGI269542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60649[] = {SGI269543, SGI0, SGI269544, SGI0, SGI269545, SGI269546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60650[] = {SGI269547, SGI0, SGI269548, SGI0, SGI269549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60651[] = {SGI269550, SGI0, SGI269551, SGI0, SGI269552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60652[] = {SGI269553, SGI0, SGI269554, SGI0, SGI269555, SGI269556, SGI269557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269558, SGI0, NULL};
const Segment * const SG60653[] = {SGI269559, SGI269560, SGI269561, SGI0, SGI269562, SGI269563, SGI269564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60654[] = {SGI269565, SGI269566, SGI269567, SGI0, SGI269568, SGI269569, SGI269570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60655[] = {SGI269571, SGI269572, NULL};
const Segment * const SG60656[] = {SGI269573, SGI269574, NULL};
const Segment * const SG60657[] = {SGI269575, SGI269576, NULL};
const Segment * const SG60658[] = {SGI269577, SGI0, SGI269578, SGI0, SGI269579, SGI269580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269581, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60659[] = {SGI269582, SGI0, SGI269583, SGI0, SGI269584, SGI269585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269586, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60660[] = {SGI269587, SGI0, SGI269588, SGI0, SGI269589, SGI269590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269591, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60661[] = {SGI269592, SGI0, SGI269593, SGI0, SGI269594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60662[] = {SGI269595, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60663[] = {SGI269596, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60664[] = {SGI269597, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60665[] = {SGI269598, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60666[] = {SGI269599, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60667[] = {SGI269600, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60668[] = {SGI269601, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60669[] = {SGI269602, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60670[] = {SGI269603, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60671[] = {SGI269604, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60672[] = {SGI269605, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60673[] = {SGI269606, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60674[] = {SGI269607, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60675[] = {SGI269608, SGI269609, SGI269610, SGI0, SGI269611, SGI269612, SGI269613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60676[] = {SGI269614, SGI269615, SGI269616, SGI0, SGI269617, SGI269618, SGI269619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60677[] = {SGI269620, SGI269621, SGI269622, SGI0, SGI269623, SGI269624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60678[] = {SGI269625, SGI269626, SGI269627, SGI0, SGI269628, SGI269629, SGI269630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60679[] = {SGI269631, SGI269632, SGI269633, SGI0, SGI269634, SGI269635, SGI269636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269637, SGI0, SGI0, NULL};
const Segment * const SG60680[] = {SGI269638, SGI269639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60681[] = {SGI269640, SGI0, SGI269641, SGI269642, SGI269643, SGI269644, SGI269645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60682[] = {SGI269646, SGI0, SGI269647, SGI269648, SGI269649, SGI269650, SGI269651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60683[] = {SGI269652, SGI0, SGI269653, SGI269654, SGI269655, SGI269656, SGI269657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60684[] = {SGI269658, SGI0, SGI269659, SGI0, SGI269660, SGI269661, SGI269662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269663, SGI0, NULL};
const Segment * const SG60685[] = {SGI269664, SGI0, SGI269665, SGI0, SGI269666, SGI269667, SGI269668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269669, SGI0, NULL};
const Segment * const SG60686[] = {SGI269670, SGI0, SGI269671, SGI0, SGI269672, SGI269673, SGI269674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269675, SGI0, NULL};
const Segment * const SG60687[] = {SGI269676, SGI0, SGI269677, SGI0, SGI269678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60688[] = {SGI269679, SGI0, SGI269680, SGI0, SGI269681, SGI269682, SGI269683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60689[] = {SGI269684, SGI269685, SGI269686, SGI0, SGI269687, SGI269688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60690[] = {SGI269689, SGI269690, NULL};
const Segment * const SG60691[] = {SGI269691, SGI269692, NULL};
const Segment * const SG60692[] = {SGI269693, SGI269694, NULL};
const Segment * const SG60693[] = {SGI269695, SGI269696, NULL};
const Segment * const SG60694[] = {SGI269697, SGI269698, NULL};
const Segment * const SG60695[] = {SGI269699, SGI269700, NULL};
const Segment * const SG60696[] = {SGI269701, SGI269702, SGI269703, SGI0, SGI269704, SGI269705, SGI269706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60697[] = {SGI269707, SGI269708, SGI269709, SGI0, SGI269710, SGI269711, SGI269712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60698[] = {SGI269713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60699[] = {SGI269714, SGI269715, NULL};
const Segment * const SG60700[] = {SGI269716, SGI269717, SGI269718, SGI0, SGI269719, SGI269720, SGI269721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60701[] = {SGI269722, SGI269723, SGI269724, SGI0, SGI269725, SGI269726, SGI269727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60702[] = {SGI269728, SGI269729, NULL};
const Segment * const SG60703[] = {SGI269730, SGI269731, NULL};
const Segment * const SG60704[] = {SGI269732, SGI269733, NULL};
const Segment * const SG60705[] = {SGI269734, SGI0, SGI269735, SGI0, SGI269736, SGI269737, SGI269738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269739, SGI0, NULL};
const Segment * const SG60706[] = {SGI269740, SGI0, SGI269741, SGI0, SGI269742, SGI269743, SGI269744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269745, SGI0, NULL};
const Segment * const SG60707[] = {SGI269746, SGI0, SGI269747, SGI0, SGI269748, SGI269749, SGI269750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269751, SGI0, NULL};
const Segment * const SG60708[] = {SGI269752, SGI0, SGI269753, SGI0, SGI269754, SGI269755, SGI269756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269757, SGI0, NULL};
const Segment * const SG60709[] = {SGI269758, SGI0, SGI269759, SGI0, SGI269760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60710[] = {SGI269761, SGI0, SGI269762, SGI0, SGI269763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60711[] = {SGI269764, SGI0, SGI269765, SGI0, SGI269766, SGI269767, SGI269768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269769, SGI0, NULL};
const Segment * const SG60712[] = {SGI269770, SGI0, SGI269771, SGI0, SGI269772, SGI269773, SGI269774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269775, SGI0, NULL};
const Segment * const SG60713[] = {SGI269776, SGI0, SGI269777, SGI0, SGI269778, SGI269779, SGI269780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269781, SGI0, NULL};
const Segment * const SG60714[] = {SGI269782, SGI0, SGI269783, SGI0, SGI269784, SGI269785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269786, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60715[] = {SGI269787, SGI0, SGI269788, SGI269789, SGI269790, SGI269791, SGI269792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60716[] = {SGI269793, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60717[] = {SGI269794, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60718[] = {SGI269795, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60719[] = {SGI269796, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60720[] = {SGI269797, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60721[] = {SGI269798, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60722[] = {SGI269799, SGI0, SGI269800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60723[] = {SGI269801, SGI269802, NULL};
const Segment * const SG60724[] = {SGI269803, SGI0, SGI269804, SGI0, SGI269805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60725[] = {SGI269806, SGI269807, NULL};
const Segment * const SG60726[] = {SGI269808, SGI269809, NULL};
const Segment * const SG60727[] = {SGI269810, SGI269811, NULL};
const Segment * const SG60728[] = {SGI269812, SGI269813, NULL};
const Segment * const SG60729[] = {SGI269814, SGI269815, NULL};
const Segment * const SG60730[] = {SGI269816, SGI0, SGI269817, SGI0, SGI269818, SGI269819, SGI269820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269821, SGI0, NULL};
const Segment * const SG60731[] = {SGI269822, SGI0, SGI269823, SGI0, SGI269824, SGI269825, SGI269826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269827, SGI0, NULL};
const Segment * const SG60732[] = {SGI269828, SGI0, SGI269829, SGI0, SGI269830, SGI269831, SGI269832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269833, SGI0, NULL};
const Segment * const SG60733[] = {SGI269834, SGI0, SGI269835, SGI0, SGI269836, SGI269837, SGI269838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269839, SGI0, NULL};
const Segment * const SG60734[] = {SGI269840, SGI0, SGI269841, SGI0, SGI269842, SGI269843, SGI269844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269845, SGI0, NULL};
const Segment * const SG60735[] = {SGI269846, SGI269847, SGI0, SGI269848, NULL};
const Segment * const SG60736[] = {SGI269849, SGI269850, SGI0, SGI269851, NULL};
const Segment * const SG60737[] = {SGI269852, SGI269853, SGI269854, SGI0, SGI269855, SGI269856, SGI269857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60738[] = {SGI269858, SGI269859, SGI269860, SGI0, SGI269861, SGI269862, SGI269863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60739[] = {SGI269864, SGI269865, SGI269866, SGI0, SGI269867, SGI269868, SGI269869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60740[] = {SGI269870, SGI269871, SGI269872, SGI0, SGI269873, SGI269874, SGI269875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60741[] = {SGI269876, SGI269877, SGI269878, SGI0, SGI269879, SGI269880, SGI269881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60742[] = {SGI269882, SGI269883, SGI269884, SGI0, SGI269885, SGI269886, SGI269887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60743[] = {SGI269888, SGI269889, SGI269890, SGI0, SGI269891, SGI269892, SGI269893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60744[] = {SGI269894, SGI269895, SGI269896, SGI0, SGI269897, SGI269898, SGI269899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60745[] = {SGI269900, SGI0, SGI269901, SGI0, SGI269902, SGI269903, SGI269904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60746[] = {SGI269905, SGI269906, NULL};
const Segment * const SG60747[] = {SGI269907, SGI269908, NULL};
const Segment * const SG60748[] = {SGI269909, SGI269910, NULL};
const Segment * const SG60749[] = {SGI269911, SGI269912, NULL};
const Segment * const SG60750[] = {SGI269913, SGI269914, NULL};
const Segment * const SG60751[] = {SGI269915, SGI0, SGI269916, SGI0, SGI269917, SGI269918, SGI269919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269920, SGI0, NULL};
const Segment * const SG60752[] = {SGI269921, SGI0, SGI269922, SGI0, SGI269923, SGI269924, SGI269925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269926, SGI0, NULL};
const Segment * const SG60753[] = {SGI269927, SGI0, SGI269928, SGI0, SGI269929, SGI269930, SGI269931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269932, SGI0, NULL};
const Segment * const SG60754[] = {SGI269933, SGI0, SGI269934, SGI0, SGI269935, SGI269936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269937, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60755[] = {SGI269938, SGI0, SGI269939, SGI0, SGI269940, SGI269941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269942, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60756[] = {SGI269943, SGI0, SGI269944, SGI0, SGI269945, SGI269946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269947, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60757[] = {SGI269948, SGI0, SGI269949, SGI0, SGI269950, SGI269951, SGI269952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269953, SGI0, NULL};
const Segment * const SG60758[] = {SGI269954, SGI0, SGI269955, SGI0, SGI269956, SGI269957, SGI269958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60759[] = {SGI269959, SGI0, SGI269960, SGI0, SGI269961, SGI269962, SGI269963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269964, SGI0, NULL};
const Segment * const SG60760[] = {SGI269965, SGI0, SGI269966, SGI0, SGI269967, SGI269968, SGI269969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269970, SGI0, NULL};
const Segment * const SG60761[] = {SGI269971, SGI0, SGI269972, SGI0, SGI269973, SGI269974, SGI269975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269976, SGI0, NULL};
const Segment * const SG60762[] = {SGI269977, SGI0, SGI269978, SGI0, SGI269979, SGI269980, SGI269981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60763[] = {SGI269982, SGI0, SGI269983, SGI0, SGI269984, SGI269985, SGI269986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60764[] = {SGI269987, SGI0, SGI269988, SGI0, SGI269989, SGI269990, SGI269991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI269992, SGI0, NULL};
const Segment * const SG60765[] = {SGI269993, SGI269994, SGI269995, SGI0, SGI269996, SGI269997, SGI269998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60766[] = {SGI269999, SGI270000, SGI270001, SGI0, SGI270002, SGI270003, SGI270004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60767[] = {SGI270005, SGI270006, NULL};
const Segment * const SG60768[] = {SGI270007, SGI270008, NULL};
const Segment * const SG60769[] = {SGI270009, SGI270010, NULL};
const Segment * const SG60770[] = {SGI270011, SGI270012, NULL};
const Segment * const SG60771[] = {SGI270013, SGI270014, NULL};
const Segment * const SG60772[] = {SGI270015, SGI0, SGI270016, SGI0, SGI270017, SGI270018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270019, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60773[] = {SGI270020, SGI0, SGI270021, SGI0, SGI270022, SGI270023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270024, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60774[] = {SGI270025, SGI0, SGI270026, SGI0, SGI270027, SGI270028, SGI270029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270030, SGI0, NULL};
const Segment * const SG60775[] = {SGI270031, SGI0, SGI270032, SGI0, SGI270033, SGI270034, SGI270035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270036, SGI0, NULL};
const Segment * const SG60776[] = {SGI270037, SGI0, SGI270038, SGI0, SGI270039, SGI270040, SGI270041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270042, SGI0, NULL};
const Segment * const SG60777[] = {SGI270043, SGI270044, NULL};
const Segment * const SG60778[] = {SGI270045, SGI270046, NULL};
const Segment * const SG60779[] = {SGI270047, SGI270048, NULL};
const Segment * const SG60780[] = {SGI270049, SGI270050, NULL};
const Segment * const SG60781[] = {SGI270051, SGI0, SGI270052, SGI270053, SGI270054, SGI270055, SGI270056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60782[] = {SGI270057, SGI0, SGI270058, SGI270059, SGI270060, SGI270061, SGI270062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60783[] = {SGI270063, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60784[] = {SGI270064, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60785[] = {SGI270065, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60786[] = {SGI270066, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60787[] = {SGI270067, SGI0, SGI270068, SGI0, SGI270069, SGI270070, SGI270071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270072, SGI0, NULL};
const Segment * const SG60788[] = {SGI270073, SGI0, SGI270074, SGI0, SGI270075, SGI270076, SGI270077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270078, SGI0, NULL};
const Segment * const SG60789[] = {SGI270079, SGI0, SGI270080, SGI0, SGI270081, SGI270082, SGI270083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270084, SGI0, NULL};
const Segment * const SG60790[] = {SGI270085, SGI0, SGI270086, SGI0, SGI270087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60791[] = {SGI270088, SGI0, SGI270089, SGI0, SGI270090, SGI270091, SGI270092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60792[] = {SGI270093, SGI0, SGI270094, SGI0, SGI270095, SGI270096, SGI270097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60793[] = {SGI270098, SGI0, SGI270099, SGI0, SGI270100, SGI270101, SGI270102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60794[] = {SGI270103, SGI0, SGI270104, SGI0, SGI270105, SGI270106, SGI270107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60795[] = {SGI270108, SGI270109, NULL};
const Segment * const SG60796[] = {SGI270110, SGI270111, NULL};
const Segment * const SG60797[] = {SGI270112, SGI270113, NULL};
const Segment * const SG60798[] = {SGI270114, SGI270115, NULL};
const Segment * const SG60799[] = {SGI270120, SGI270122, NULL};
const Segment * const SG60800[] = {SGI270116, SGI0, SGI270117, SGI0, SGI270118, SGI270119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270121, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60801[] = {SGI270125, SGI270126, NULL};
const Segment * const SG60802[] = {SGI270123, SGI0, SGI270124, SGI0, SGI270127, SGI270128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270129, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60803[] = {SGI270130, SGI270131, NULL};
const Segment * const SG60804[] = {SGI270132, SGI270133, NULL};
const Segment * const SG60805[] = {SGI270134, SGI0, SGI270135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60806[] = {SGI270136, SGI0, SGI270137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60807[] = {SGI270138, SGI0, SGI270139, SGI0, SGI270140, SGI270141, SGI270142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270143, SGI0, NULL};
const Segment * const SG60808[] = {SGI270144, SGI0, SGI270145, SGI0, SGI270146, SGI270147, SGI270148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270149, SGI0, NULL};
const Segment * const SG60809[] = {SGI270150, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60810[] = {SGI270151, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60811[] = {SGI270152, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60812[] = {SGI270153, SGI270154, SGI0, SGI0, NULL};
const Segment * const SG60813[] = {SGI270155, SGI270156, SGI0, SGI0, NULL};
const Segment * const SG60814[] = {SGI270157, SGI0, SGI270158, SGI0, SGI270159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60815[] = {SGI270160, SGI0, SGI270161, SGI270162, SGI270163, SGI270164, SGI270165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60816[] = {SGI270166, SGI0, SGI270167, SGI270168, SGI270169, SGI270170, SGI270171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60817[] = {SGI270172, SGI270173, SGI0, SGI270174, NULL};
const Segment * const SG60818[] = {SGI270175, SGI0, SGI270176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60819[] = {SGI270177, SGI0, SGI270178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60820[] = {SGI270179, SGI270180, NULL};
const Segment * const SG60821[] = {SGI270181, SGI270182, NULL};
const Segment * const SG60822[] = {SGI270183, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60823[] = {SGI270184, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60824[] = {SGI270185, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60825[] = {SGI270186, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60826[] = {SGI270187, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60827[] = {SGI270188, SGI270189, NULL};
const Segment * const SG60828[] = {SGI270190, SGI270191, NULL};
const Segment * const SG60829[] = {SGI270192, SGI270193, NULL};
const Segment * const SG60830[] = {SGI270194, SGI270195, SGI270196, SGI0, SGI270197, SGI270198, SGI270199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60831[] = {SGI270200, SGI270201, SGI270202, SGI0, SGI270203, SGI270204, SGI270205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60832[] = {SGI270206, SGI0, SGI270207, SGI270208, SGI270209, SGI270210, SGI270211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60833[] = {SGI270212, SGI270213, NULL};
const Segment * const SG60834[] = {SGI270214, SGI270215, NULL};
const Segment * const SG60835[] = {SGI270216, SGI270217, NULL};
const Segment * const SG60836[] = {SGI270218, SGI270219, NULL};
const Segment * const SG60837[] = {SGI270220, SGI270221, SGI270222, SGI0, SGI270223, SGI270224, SGI270225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60838[] = {SGI270226, SGI0, SGI270227, SGI0, SGI270228, SGI270229, SGI270230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60839[] = {SGI270231, SGI270232, SGI270233, SGI0, SGI270234, SGI270235, SGI270236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60840[] = {SGI270237, SGI0, SGI270238, SGI0, SGI270239, SGI270240, SGI270241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270242, SGI0, NULL};
const Segment * const SG60841[] = {SGI270243, SGI0, SGI270244, SGI0, SGI270245, SGI270246, SGI270247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270248, SGI0, NULL};
const Segment * const SG60842[] = {SGI270249, SGI0, SGI270250, SGI0, SGI270251, SGI270252, SGI270253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270254, SGI0, NULL};
const Segment * const SG60843[] = {SGI270255, SGI0, SGI270256, SGI0, SGI270257, SGI270258, SGI270259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270260, SGI0, NULL};
const Segment * const SG60844[] = {SGI270261, SGI0, SGI270262, SGI0, SGI270263, SGI270264, SGI270265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270266, SGI0, NULL};
const Segment * const SG60845[] = {SGI270267, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60846[] = {SGI270268, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60847[] = {SGI270269, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60848[] = {SGI270270, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60849[] = {SGI270271, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60850[] = {SGI270272, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60851[] = {SGI270273, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60852[] = {SGI270274, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60853[] = {SGI270275, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60854[] = {SGI270276, SGI270277, SGI270278, SGI0, SGI270279, SGI270280, SGI270281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60855[] = {SGI270282, SGI270283, SGI270284, SGI0, SGI270285, SGI270286, SGI270287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60856[] = {SGI270293, SGI270294, NULL};
const Segment * const SG60857[] = {SGI270288, SGI0, SGI270289, SGI0, SGI270290, SGI270291, SGI270292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270295, SGI0, NULL};
const Segment * const SG60858[] = {SGI270299, SGI270301, NULL};
const Segment * const SG60859[] = {SGI270296, SGI0, SGI270297, SGI0, SGI270298, SGI270300, SGI270302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270303, SGI0, NULL};
const Segment * const SG60860[] = {SGI270304, SGI270305, NULL};
const Segment * const SG60861[] = {SGI270306, SGI270307, NULL};
const Segment * const SG60862[] = {SGI270308, SGI0, SGI270309, SGI0, SGI270310, SGI270311, SGI270312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270313, SGI0, NULL};
const Segment * const SG60863[] = {SGI270314, SGI0, SGI270315, SGI0, SGI270316, SGI270317, SGI270318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270319, SGI0, NULL};
const Segment * const SG60864[] = {SGI270320, SGI0, SGI270321, SGI0, SGI270322, SGI270323, SGI270324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270325, SGI0, NULL};
const Segment * const SG60865[] = {SGI270326, SGI0, SGI270327, SGI0, SGI270328, SGI270329, SGI270330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270331, SGI0, NULL};
const Segment * const SG60866[] = {SGI270332, SGI270333, SGI270334, SGI0, SGI270335, SGI270336, SGI270335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60867[] = {SGI270337, SGI0, SGI270338, SGI0, SGI270339, SGI270340, SGI270341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270342, SGI0, NULL};
const Segment * const SG60868[] = {SGI270343, SGI0, SGI270344, SGI0, SGI270345, SGI270346, SGI270347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270348, SGI0, NULL};
const Segment * const SG60869[] = {SGI270349, SGI270350, SGI270351, SGI0, SGI270352, SGI270353, SGI270354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60870[] = {SGI270355, SGI270356, SGI270357, SGI0, SGI270358, SGI270359, SGI270360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60871[] = {SGI270361, SGI270362, SGI270363, SGI0, SGI270364, SGI270365, SGI270366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60872[] = {SGI270367, SGI270368, NULL};
const Segment * const SG60873[] = {SGI270369, SGI270370, SGI270371, SGI0, SGI270372, SGI270373, SGI270374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60874[] = {SGI270375, SGI270376, SGI270377, SGI0, SGI270378, SGI270379, SGI270380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60875[] = {SGI270381, SGI0, SGI270382, SGI0, SGI270383, SGI270384, SGI270385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270386, SGI0, NULL};
const Segment * const SG60876[] = {SGI270387, SGI0, SGI270388, SGI0, SGI270389, SGI270390, SGI270391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270392, SGI0, NULL};
const Segment * const SG60877[] = {SGI270393, SGI270394, NULL};
const Segment * const SG60878[] = {SGI270395, SGI270396, NULL};
const Segment * const SG60879[] = {SGI270397, SGI0, SGI270398, SGI0, SGI270399, SGI270400, SGI270401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270402, SGI0, NULL};
const Segment * const SG60880[] = {SGI270403, SGI0, SGI270404, SGI0, SGI270405, SGI270406, SGI270407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60881[] = {SGI270408, SGI0, SGI270409, SGI0, SGI270410, SGI270411, SGI270412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270413, SGI0, NULL};
const Segment * const SG60882[] = {SGI270414, SGI0, SGI270415, SGI0, SGI270416, SGI270417, SGI270418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270419, SGI0, NULL};
const Segment * const SG60883[] = {SGI270420, SGI270421, NULL};
const Segment * const SG60884[] = {SGI270422, SGI270423, NULL};
const Segment * const SG60885[] = {SGI270424, SGI270425, NULL};
const Segment * const SG60886[] = {SGI270426, SGI270427, NULL};
const Segment * const SG60887[] = {SGI270428, SGI270429, NULL};
const Segment * const SG60888[] = {SGI270430, SGI0, SGI270431, SGI270432, SGI270433, SGI270434, SGI270435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60889[] = {SGI270436, SGI0, SGI270437, SGI270438, SGI270439, SGI270440, SGI270441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60890[] = {SGI270442, SGI270443, NULL};
const Segment * const SG60891[] = {SGI270448, SGI270449, NULL};
const Segment * const SG60892[] = {SGI270444, SGI0, SGI270445, SGI0, SGI270446, SGI270447, SGI0, SGI0, SGI0, SGI270450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60893[] = {SGI270451, SGI0, SGI270452, SGI0, SGI270453, SGI270454, SGI0, SGI0, SGI0, SGI270455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60894[] = {SGI270456, SGI270457, NULL};
const Segment * const SG60895[] = {SGI270458, SGI270459, NULL};
const Segment * const SG60896[] = {SGI270460, SGI270461, NULL};
const Segment * const SG60897[] = {SGI270462, SGI270463, NULL};
const Segment * const SG60898[] = {SGI270464, SGI270465, NULL};
const Segment * const SG60899[] = {SGI270466, SGI270467, NULL};
const Segment * const SG60900[] = {SGI270468, SGI270469, NULL};
const Segment * const SG60901[] = {SGI270470, SGI270471, NULL};
const Segment * const SG60902[] = {SGI270472, SGI0, SGI270473, SGI0, SGI270474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60903[] = {SGI270475, SGI0, SGI270476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60904[] = {SGI270477, SGI0, SGI270478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60905[] = {SGI270479, SGI0, SGI270480, SGI0, SGI270481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60906[] = {SGI270482, SGI0, SGI270483, SGI0, SGI270484, SGI270485, SGI270486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60907[] = {SGI270487, SGI270488, NULL};
const Segment * const SG60908[] = {SGI270489, SGI270490, SGI270491, SGI0, SGI270492, SGI270493, SGI270494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60909[] = {SGI270495, SGI270496, SGI270497, SGI0, SGI270498, SGI270499, SGI270500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60910[] = {SGI270501, SGI270502, SGI270503, SGI0, SGI270504, SGI270505, SGI270506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60911[] = {SGI270507, SGI270508, SGI0, SGI270509, NULL};
const Segment * const SG60912[] = {SGI270510, SGI270511, SGI270512, SGI0, SGI270513, SGI270514, SGI270515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60913[] = {SGI270516, SGI270517, SGI270518, SGI0, SGI270519, SGI270520, SGI270521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60914[] = {SGI270522, SGI270523, SGI270524, SGI0, SGI270525, SGI270526, SGI270527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60915[] = {SGI270528, SGI0, SGI270529, SGI0, SGI270530, SGI270531, SGI270532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60916[] = {SGI270533, SGI0, SGI270534, SGI0, SGI270535, SGI270536, SGI270537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60917[] = {SGI270538, SGI270539, SGI270540, SGI0, SGI270541, SGI270542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60918[] = {SGI270543, SGI270544, SGI270545, SGI0, SGI270546, SGI270547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60919[] = {SGI270548, SGI270549, NULL};
const Segment * const SG60920[] = {SGI270550, SGI270551, NULL};
const Segment * const SG60921[] = {SGI270552, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60922[] = {SGI270553, SGI270554, SGI0, SGI0, NULL};
const Segment * const SG60923[] = {SGI270555, SGI0, SGI270556, SGI0, SGI270557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60924[] = {SGI270558, SGI0, SGI270559, SGI0, SGI270560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60925[] = {SGI270561, SGI270562, SGI270563, SGI0, SGI270564, SGI270565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60926[] = {SGI270566, SGI270567, SGI270568, SGI0, SGI270569, SGI270570, SGI270571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60927[] = {SGI270572, SGI0, SGI270573, SGI0, SGI270574, SGI270575, SGI270576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270577, SGI0, NULL};
const Segment * const SG60928[] = {SGI270578, SGI0, SGI270579, SGI0, SGI270580, SGI270581, SGI270582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270583, SGI0, NULL};
const Segment * const SG60929[] = {SGI270584, SGI0, SGI270585, SGI0, SGI270586, SGI270587, SGI270588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270589, SGI0, NULL};
const Segment * const SG60930[] = {SGI270590, SGI0, SGI270591, SGI0, SGI270592, SGI270593, SGI270594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270595, SGI0, NULL};
const Segment * const SG60931[] = {SGI270596, SGI270597, NULL};
const Segment * const SG60932[] = {SGI270598, SGI270599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60933[] = {SGI270600, SGI270601, SGI0, SGI0, NULL};
const Segment * const SG60934[] = {SGI270602, SGI270603, NULL};
const Segment * const SG60935[] = {SGI270604, SGI270605, NULL};
const Segment * const SG60936[] = {SGI270606, SGI270607, NULL};
const Segment * const SG60937[] = {SGI270608, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60938[] = {SGI270609, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60939[] = {SGI270610, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60940[] = {SGI270611, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60941[] = {SGI270612, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60942[] = {SGI270613, SGI270614, NULL};
const Segment * const SG60943[] = {SGI270615, SGI270616, NULL};
const Segment * const SG60944[] = {SGI270617, SGI270618, NULL};
const Segment * const SG60945[] = {SGI270619, SGI270620, SGI270621, SGI0, SGI270622, SGI270623, SGI270624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60946[] = {SGI270625, SGI270626, SGI270627, SGI0, SGI270628, SGI270629, SGI270630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60947[] = {SGI270631, SGI0, SGI270632, SGI0, SGI270633, SGI270634, SGI270635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270636, SGI0, NULL};
const Segment * const SG60948[] = {SGI270637, SGI0, SGI270638, SGI0, SGI270639, SGI270640, SGI270641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270642, SGI0, NULL};
const Segment * const SG60949[] = {SGI270643, SGI0, SGI270644, SGI0, SGI270645, SGI270646, SGI270647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270648, SGI0, NULL};
const Segment * const SG60950[] = {SGI270649, SGI270650, SGI270651, SGI0, SGI270652, SGI270653, SGI270654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60951[] = {SGI270655, SGI270656, SGI270657, SGI0, SGI270658, SGI270659, SGI270660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60952[] = {SGI270661, SGI270662, NULL};
const Segment * const SG60953[] = {SGI270663, SGI270664, NULL};
const Segment * const SG60954[] = {SGI270665, SGI270666, NULL};
const Segment * const SG60955[] = {SGI270667, SGI270668, NULL};
const Segment * const SG60956[] = {SGI270669, SGI0, SGI270670, SGI0, SGI270671, SGI270672, SGI270673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270674, SGI0, NULL};
const Segment * const SG60957[] = {SGI270675, SGI0, SGI270676, SGI0, SGI270677, SGI270678, SGI270679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270680, SGI0, NULL};
const Segment * const SG60958[] = {SGI270681, SGI0, SGI270682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60959[] = {SGI270683, SGI0, SGI270684, SGI0, SGI270685, SGI270686, SGI270687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270688, SGI0, NULL};
const Segment * const SG60960[] = {SGI270689, SGI270690, SGI0, SGI0, NULL};
const Segment * const SG60961[] = {SGI270691, SGI270692, NULL};
const Segment * const SG60962[] = {SGI270693, SGI0, SGI270694, SGI0, SGI270695, SGI270696, SGI270697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60963[] = {SGI270698, SGI270699, NULL};
const Segment * const SG60964[] = {SGI270700, SGI270701, NULL};
const Segment * const SG60965[] = {SGI270702, SGI270703, NULL};
const Segment * const SG60966[] = {SGI270704, SGI270705, NULL};
const Segment * const SG60967[] = {SGI270706, SGI270707, NULL};
const Segment * const SG60968[] = {SGI270708, SGI270709, NULL};
const Segment * const SG60969[] = {SGI270710, SGI0, SGI270711, SGI0, SGI270712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60970[] = {SGI270713, SGI0, SGI270714, SGI270715, SGI270716, SGI270717, SGI270718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60971[] = {SGI270719, SGI0, SGI270720, SGI270721, SGI270722, SGI270723, SGI270724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60972[] = {SGI270725, SGI0, SGI270726, SGI0, SGI270727, SGI270728, SGI270729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270730, SGI0, NULL};
const Segment * const SG60973[] = {SGI270731, SGI0, SGI270732, SGI0, SGI270733, SGI270734, SGI270735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270736, SGI0, NULL};
const Segment * const SG60974[] = {SGI270737, SGI0, SGI270738, SGI0, SGI270739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60975[] = {SGI270740, SGI0, SGI270741, SGI0, SGI270742, SGI270743, SGI270744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270745, SGI0, NULL};
const Segment * const SG60976[] = {SGI270746, SGI0, SGI270747, SGI0, SGI270748, SGI270749, SGI270750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270751, SGI0, NULL};
const Segment * const SG60977[] = {SGI270752, SGI270753, NULL};
const Segment * const SG60978[] = {SGI270754, SGI270755, NULL};
const Segment * const SG60979[] = {SGI270756, SGI270757, NULL};
const Segment * const SG60980[] = {SGI270758, SGI270759, NULL};
const Segment * const SG60981[] = {SGI270760, SGI0, SGI270761, SGI0, SGI270762, SGI270763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270764, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60982[] = {SGI270765, SGI0, SGI270766, SGI0, SGI270767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60983[] = {SGI270768, SGI270769, SGI270770, SGI0, SGI270771, SGI270772, SGI270773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60984[] = {SGI270774, SGI270775, NULL};
const Segment * const SG60985[] = {SGI270776, SGI270777, NULL};
const Segment * const SG60986[] = {SGI270778, SGI0, SGI270779, SGI270780, SGI270781, SGI270782, SGI270783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60987[] = {SGI270784, SGI0, SGI270785, SGI0, SGI270786, SGI270787, SGI270788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270789, SGI0, NULL};
const Segment * const SG60988[] = {SGI270790, SGI270791, NULL};
const Segment * const SG60989[] = {SGI270792, SGI270793, NULL};
const Segment * const SG60990[] = {SGI270794, SGI270795, NULL};
const Segment * const SG60991[] = {SGI270796, SGI270797, NULL};
const Segment * const SG60992[] = {SGI270798, SGI270799, NULL};
const Segment * const SG60993[] = {SGI270800, SGI270801, SGI270802, SGI0, NULL};
const Segment * const SG60994[] = {SGI270803, SGI0, SGI270804, SGI0, SGI270805, SGI270806, SGI270807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270808, SGI0, NULL};
const Segment * const SG60995[] = {SGI270809, SGI0, SGI270810, SGI0, SGI270811, SGI270812, SGI270813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG60996[] = {SGI270814, SGI270815, NULL};
const Segment * const SG60997[] = {SGI270816, SGI270817, NULL};
const Segment * const SG60998[] = {SGI270818, SGI270818, NULL};
const Segment * const SG60999[] = {SGI270819, SGI270820, NULL};
const Segment * const SG61000[] = {SGI270821, SGI0, SGI270822, SGI0, SGI270823, SGI270824, SGI270825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270826, SGI0, NULL};
const Segment * const SG61001[] = {SGI270827, SGI0, SGI270828, SGI0, SGI270829, SGI270830, SGI270831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270832, SGI0, NULL};
const Segment * const SG61002[] = {SGI270833, SGI270834, SGI0, SGI0, NULL};
const Segment * const SG61003[] = {SGI270835, SGI0, SGI270836, SGI0, SGI270837, SGI270838, SGI270839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61004[] = {SGI270840, SGI0, SGI270841, SGI0, SGI270842, SGI270843, SGI270844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61005[] = {SGI270845, SGI0, SGI270846, SGI0, SGI270847, SGI270848, SGI270849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61006[] = {SGI270850, SGI0, SGI270851, SGI0, SGI270852, SGI270853, SGI270854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61007[] = {SGI270855, SGI0, SGI270856, SGI0, SGI270857, SGI270858, SGI270859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61008[] = {SGI270860, SGI270861, NULL};
const Segment * const SG61009[] = {SGI270862, SGI270863, NULL};
const Segment * const SG61010[] = {SGI270864, SGI270865, NULL};
const Segment * const SG61011[] = {SGI270866, SGI270867, SGI0, SGI270868, NULL};
const Segment * const SG61012[] = {SGI270869, SGI270870, NULL};
const Segment * const SG61013[] = {SGI270871, SGI270872, NULL};
const Segment * const SG61014[] = {SGI270873, SGI0, SGI270874, SGI0, SGI270875, SGI270876, SGI270877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270878, SGI0, NULL};
const Segment * const SG61015[] = {SGI270879, SGI0, SGI270880, SGI0, SGI270881, SGI270882, SGI270883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270884, SGI0, NULL};
const Segment * const SG61016[] = {SGI270885, SGI0, SGI270886, SGI0, SGI270887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61017[] = {SGI270888, SGI270889, SGI270890, SGI0, NULL};
const Segment * const SG61018[] = {SGI270891, SGI270892, SGI270893, SGI0, SGI270894, SGI270895, SGI270896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61019[] = {SGI270897, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61020[] = {SGI270898, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61021[] = {SGI270899, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61022[] = {SGI270900, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61023[] = {SGI270901, SGI0, SGI270902, SGI0, SGI270903, SGI270904, SGI270905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270906, SGI0, NULL};
const Segment * const SG61024[] = {SGI270907, SGI270908, NULL};
const Segment * const SG61025[] = {SGI270909, SGI270910, NULL};
const Segment * const SG61026[] = {SGI270911, SGI0, SGI270912, SGI0, SGI270913, SGI270914, SGI270915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61027[] = {SGI270916, SGI0, SGI270917, SGI0, SGI270918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61028[] = {SGI270919, SGI270920, NULL};
const Segment * const SG61029[] = {SGI270921, SGI270922, NULL};
const Segment * const SG61030[] = {SGI270923, SGI0, SGI270924, SGI0, SGI270925, SGI270926, SGI270927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61031[] = {SGI270928, SGI0, SGI270929, SGI0, SGI270930, SGI270931, SGI270932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270933, SGI0, NULL};
const Segment * const SG61032[] = {SGI270934, SGI270935, NULL};
const Segment * const SG61033[] = {SGI270936, SGI270937, NULL};
const Segment * const SG61034[] = {SGI270938, SGI0, SGI270939, SGI0, SGI270940, SGI270941, SGI270942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61035[] = {SGI270943, SGI270944, NULL};
const Segment * const SG61036[] = {SGI270945, SGI270946, NULL};
const Segment * const SG61037[] = {SGI270947, SGI0, SGI270948, SGI0, SGI270949, SGI270950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270951, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61038[] = {SGI270952, SGI270953, SGI270954, SGI0, SGI270955, SGI270956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61039[] = {SGI270957, SGI0, SGI270958, SGI0, SGI270959, SGI270960, SGI270961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270962, SGI0, NULL};
const Segment * const SG61040[] = {SGI270963, SGI270964, NULL};
const Segment * const SG61041[] = {SGI270965, SGI270966, SGI270967, SGI0, SGI270968, SGI270969, SGI270970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61042[] = {SGI270971, SGI0, SGI270972, SGI0, SGI270973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61043[] = {SGI270974, SGI270975, NULL};
const Segment * const SG61044[] = {SGI270976, SGI270977, NULL};
const Segment * const SG61045[] = {SGI270978, SGI270979, NULL};
const Segment * const SG61046[] = {SGI270980, SGI0, SGI270981, SGI0, SGI270982, SGI270983, SGI270984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI270985, SGI0, NULL};
const Segment * const SG61047[] = {SGI270986, SGI270987, SGI270988, SGI0, SGI270989, SGI270990, SGI270991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61048[] = {SGI270992, SGI0, SGI270993, SGI270994, SGI270995, SGI270996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61049[] = {SGI270997, SGI0, SGI270998, SGI270999, SGI271000, SGI271001, SGI271002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61050[] = {SGI271003, SGI0, SGI271004, SGI271005, SGI271006, SGI271007, SGI271008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61051[] = {SGI271009, SGI0, SGI271010, SGI0, SGI271011, SGI271012, SGI271013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61052[] = {SGI271014, SGI271015, NULL};
const Segment * const SG61053[] = {SGI271016, SGI271017, NULL};
const Segment * const SG61054[] = {SGI271018, SGI271019, SGI271020, SGI0, SGI271021, SGI271022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61055[] = {SGI271023, SGI271024, NULL};
const Segment * const SG61056[] = {SGI271025, SGI0, SGI271026, SGI0, SGI271027, SGI271028, SGI271029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271030, SGI0, NULL};
const Segment * const SG61057[] = {SGI271031, SGI271032, SGI271033, SGI0, SGI271034, SGI271035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61058[] = {SGI271036, SGI0, SGI271037, SGI0, SGI271038, SGI271039, SGI271040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61059[] = {SGI271041, SGI0, SGI271042, SGI0, SGI271043, SGI271044, SGI271045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271046, SGI0, NULL};
const Segment * const SG61060[] = {SGI271047, SGI0, SGI271048, SGI0, SGI271049, SGI271050, SGI271051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61061[] = {SGI271052, SGI0, SGI271053, SGI0, SGI271054, SGI271055, SGI271056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271057, SGI0, NULL};
const Segment * const SG61062[] = {SGI271058, SGI271059, NULL};
const Segment * const SG61063[] = {SGI271060, SGI0, SGI271061, SGI0, SGI271062, SGI271063, SGI271064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61064[] = {SGI271065, SGI0, SGI271066, SGI0, SGI271067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61065[] = {SGI271068, SGI271069, NULL};
const Segment * const SG61066[] = {SGI271070, SGI271071, NULL};
const Segment * const SG61067[] = {SGI271072, SGI0, SGI271073, SGI0, SGI271074, SGI271075, SGI271076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271077, SGI0, NULL};
const Segment * const SG61068[] = {SGI271078, SGI0, SGI271079, SGI0, SGI271080, SGI271081, SGI271082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271083, SGI0, NULL};
const Segment * const SG61069[] = {SGI271084, SGI271085, NULL};
const Segment * const SG61070[] = {SGI271086, SGI271087, NULL};
const Segment * const SG61071[] = {SGI271088, SGI271089, NULL};
const Segment * const SG61072[] = {SGI271090, SGI271091, NULL};
const Segment * const SG61073[] = {SGI271092, SGI0, SGI271093, SGI0, SGI271094, SGI271095, SGI271096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271097, SGI0, NULL};
const Segment * const SG61074[] = {SGI271098, SGI0, SGI271099, SGI0, SGI271100, SGI271101, SGI271102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271103, SGI0, NULL};
const Segment * const SG61075[] = {SGI271104, SGI0, SGI271105, SGI0, SGI271106, SGI271107, SGI271108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271109, SGI0, NULL};
const Segment * const SG61076[] = {SGI271110, SGI0, SGI271111, SGI0, SGI271112, SGI271113, SGI271114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271115, SGI0, NULL};
const Segment * const SG61077[] = {SGI271116, SGI271117, SGI271118, SGI0, SGI271119, SGI271120, SGI271121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61078[] = {SGI271122, SGI271123, NULL};
const Segment * const SG61079[] = {SGI271124, SGI271125, SGI0, SGI0, NULL};
const Segment * const SG61080[] = {SGI271126, SGI0, SGI271127, SGI0, SGI271128, SGI271129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271130, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61081[] = {SGI271131, SGI0, SGI271132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61082[] = {SGI271133, SGI0, SGI271134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61083[] = {SGI271135, SGI0, SGI271136, SGI0, SGI271137, SGI271138, SGI271139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271140, SGI0, NULL};
const Segment * const SG61084[] = {SGI271141, SGI0, SGI271142, SGI0, SGI271143, SGI271144, SGI271145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271146, SGI0, NULL};
const Segment * const SG61085[] = {SGI271147, SGI0, SGI271148, SGI0, SGI271149, SGI271150, SGI271151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271152, SGI0, NULL};
const Segment * const SG61086[] = {SGI271153, SGI0, SGI271154, SGI0, SGI271155, SGI271156, SGI271157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61087[] = {SGI271158, SGI0, SGI271159, SGI0, SGI271160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61088[] = {SGI271161, SGI271162, NULL};
const Segment * const SG61089[] = {SGI271163, SGI271164, SGI271165, SGI0, SGI271166, SGI271167, SGI271168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61090[] = {SGI271169, SGI0, SGI271170, SGI0, SGI271171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61091[] = {SGI271172, SGI271173, SGI0, SGI0, NULL};
const Segment * const SG61092[] = {SGI271174, SGI271175, NULL};
const Segment * const SG61093[] = {SGI271176, SGI271177, NULL};
const Segment * const SG61094[] = {SGI271178, SGI271179, NULL};
const Segment * const SG61095[] = {SGI271180, SGI0, SGI271181, SGI271182, SGI271183, SGI271184, SGI271185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61096[] = {SGI271186, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61097[] = {SGI271187, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61098[] = {SGI271188, SGI271189, SGI0, SGI0, NULL};
const Segment * const SG61099[] = {SGI271190, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61100[] = {SGI271191, SGI271192, NULL};
const Segment * const SG61101[] = {SGI271193, SGI0, SGI271194, SGI0, SGI271195, SGI271196, SGI271197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271198, SGI0, NULL};
const Segment * const SG61102[] = {SGI271199, SGI0, SGI271200, SGI0, SGI271201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61103[] = {SGI271202, SGI271203, SGI0, SGI0, NULL};
const Segment * const SG61104[] = {SGI271204, SGI271205, SGI271206, SGI0, SGI271207, SGI271208, SGI271209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61105[] = {SGI271210, SGI271211, NULL};
const Segment * const SG61106[] = {SGI271212, SGI271213, NULL};
const Segment * const SG61107[] = {SGI271214, SGI0, SGI271215, SGI271216, SGI271217, SGI271218, SGI271219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61108[] = {SGI271220, SGI0, SGI271221, SGI0, SGI271222, SGI271223, SGI271224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271225, SGI0, NULL};
const Segment * const SG61109[] = {SGI271226, SGI0, SGI271227, SGI0, SGI271228, SGI271229, SGI271230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271231, SGI0, NULL};
const Segment * const SG61110[] = {SGI271232, SGI0, SGI271233, SGI0, SGI271234, SGI271235, SGI271236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271237, SGI0, NULL};
const Segment * const SG61111[] = {SGI271238, SGI0, SGI271239, SGI0, SGI271240, SGI271241, SGI271242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271243, SGI0, NULL};
const Segment * const SG61112[] = {SGI271244, SGI0, SGI271245, SGI0, SGI271246, SGI271247, SGI271248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271249, SGI0, NULL};
const Segment * const SG61113[] = {SGI271250, SGI0, SGI271251, SGI0, SGI271252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61114[] = {SGI271253, SGI271254, SGI271255, SGI0, SGI271256, SGI271257, SGI271258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61115[] = {SGI271259, SGI0, SGI271260, SGI0, SGI271261, SGI271262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271263, SGI0, NULL};
const Segment * const SG61116[] = {SGI271264, SGI0, SGI271265, SGI0, SGI271266, SGI271267, SGI271268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271269, SGI0, NULL};
const Segment * const SG61117[] = {SGI271270, SGI0, SGI271271, SGI0, SGI271272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61118[] = {SGI271273, SGI271274, NULL};
const Segment * const SG61119[] = {SGI271275, SGI271276, NULL};
const Segment * const SG61120[] = {SGI271277, SGI271278, SGI271279, SGI0, SGI271280, SGI271281, SGI271282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61121[] = {SGI271283, SGI0, SGI271284, SGI0, SGI271285, SGI271286, SGI271287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271288, SGI0, NULL};
const Segment * const SG61122[] = {SGI271289, SGI271290, NULL};
const Segment * const SG61123[] = {SGI271291, SGI271292, NULL};
const Segment * const SG61124[] = {SGI271293, SGI0, SGI271294, SGI0, SGI271295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61125[] = {SGI271296, SGI271297, SGI271298, SGI0, SGI271299, SGI271300, SGI271301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61126[] = {SGI271302, SGI0, SGI271303, SGI0, SGI271304, SGI271305, SGI271306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61127[] = {SGI271307, SGI271308, SGI271309, SGI0, SGI271310, SGI271311, SGI271312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61128[] = {SGI271313, SGI0, SGI271314, SGI0, SGI271315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61129[] = {SGI271316, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61130[] = {SGI271317, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61131[] = {SGI271318, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61132[] = {SGI271319, SGI271320, SGI0, SGI0, NULL};
const Segment * const SG61133[] = {SGI271321, SGI0, SGI271322, SGI271323, SGI271324, SGI271325, SGI271326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61134[] = {SGI271327, SGI0, SGI271328, SGI271329, SGI271330, SGI271331, SGI271332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61135[] = {SGI271333, SGI271334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61136[] = {SGI271335, SGI0, SGI271336, SGI0, SGI271337, SGI271338, SGI271339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271340, SGI0, NULL};
const Segment * const SG61137[] = {SGI271341, SGI0, SGI271342, SGI271343, SGI271344, SGI271345, SGI271346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61138[] = {SGI271347, SGI271348, NULL};
const Segment * const SG61139[] = {SGI271349, SGI271350, NULL};
const Segment * const SG61140[] = {SGI271351, SGI0, SGI271352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61141[] = {SGI271353, SGI271354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61142[] = {SGI271355, SGI0, SGI271356, SGI271357, SGI271358, SGI271359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61143[] = {SGI271360, SGI0, SGI271361, SGI271362, SGI271363, SGI271364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61144[] = {SGI271365, SGI0, SGI271366, SGI271367, SGI271368, SGI271369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61145[] = {SGI271370, SGI0, SGI271371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61146[] = {SGI271372, SGI0, SGI271373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61147[] = {SGI271374, SGI0, SGI271375, SGI271376, SGI271377, SGI271378, SGI271379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61148[] = {SGI271380, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61149[] = {SGI271381, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61150[] = {SGI271382, SGI271383, NULL};
const Segment * const SG61151[] = {SGI271384, SGI271385, NULL};
const Segment * const SG61152[] = {SGI271386, SGI0, SGI271387, SGI0, SGI271388, SGI271389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271390, SGI0, NULL};
const Segment * const SG61153[] = {SGI271391, SGI0, SGI271392, SGI0, SGI271393, SGI271394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271395, SGI0, NULL};
const Segment * const SG61154[] = {SGI271396, SGI0, SGI271397, SGI0, SGI271398, SGI271399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271400, SGI0, NULL};
const Segment * const SG61155[] = {SGI271401, SGI0, SGI271402, SGI271403, SGI271404, SGI271405, SGI271406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61156[] = {SGI271407, SGI0, SGI271408, SGI0, SGI271409, SGI271410, SGI271411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271412, SGI0, NULL};
const Segment * const SG61157[] = {SGI271413, SGI0, SGI271414, SGI0, SGI271415, SGI271416, SGI271417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271418, SGI0, NULL};
const Segment * const SG61158[] = {SGI271419, SGI0, SGI271420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61159[] = {SGI271421, SGI0, SGI271422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61160[] = {SGI271423, SGI0, SGI271424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61161[] = {SGI271425, SGI271426, SGI271427, SGI0, SGI271428, SGI271429, SGI271430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61162[] = {SGI271431, SGI0, SGI271432, SGI0, SGI271433, SGI271434, SGI271435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271436, SGI0, NULL};
const Segment * const SG61163[] = {SGI271437, SGI271438, SGI271439, SGI0, SGI271440, SGI271441, SGI271442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61164[] = {SGI271443, SGI271444, SGI271445, SGI0, SGI271446, SGI271447, SGI271448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61165[] = {SGI271449, SGI271450, SGI271451, SGI0, SGI271452, SGI271453, SGI271454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61166[] = {SGI271455, SGI271456, SGI271457, SGI0, SGI271458, SGI271459, SGI271460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61167[] = {SGI271461, SGI271462, SGI271463, SGI0, SGI271464, SGI271465, SGI271466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61168[] = {SGI271467, SGI271468, SGI271469, SGI0, SGI271470, SGI271471, SGI271472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61169[] = {SGI271473, SGI271474, SGI271475, SGI0, NULL};
const Segment * const SG61170[] = {SGI271476, SGI271477, NULL};
const Segment * const SG61171[] = {SGI271478, SGI0, SGI271479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61172[] = {SGI271480, SGI0, SGI271481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61173[] = {SGI271482, SGI0, SGI271483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61174[] = {SGI271484, SGI0, SGI271485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61175[] = {SGI271486, SGI0, SGI271487, SGI0, SGI271488, SGI271489, SGI271490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271491, SGI0, NULL};
const Segment * const SG61176[] = {SGI271492, SGI271493, NULL};
const Segment * const SG61177[] = {SGI271494, SGI0, SGI271495, SGI271496, SGI271497, SGI271498, SGI271499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61178[] = {SGI271500, SGI0, SGI271501, SGI0, SGI271502, SGI271503, SGI271504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271505, SGI0, NULL};
const Segment * const SG61179[] = {SGI271506, SGI271507, SGI0, SGI0, NULL};
const Segment * const SG61180[] = {SGI271508, SGI271509, NULL};
const Segment * const SG61181[] = {SGI271510, SGI271511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61182[] = {SGI271512, SGI271513, SGI271514, SGI0, SGI271515, SGI271516, SGI271517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61183[] = {SGI271518, SGI0, SGI271519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61184[] = {SGI271520, SGI0, SGI271521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61185[] = {SGI271522, SGI271523, SGI271524, SGI0, NULL};
const Segment * const SG61186[] = {SGI271525, SGI0, SGI271526, SGI0, SGI271527, SGI271528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271529, SGI0, NULL};
const Segment * const SG61187[] = {SGI271530, SGI0, SGI271531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61188[] = {SGI271532, SGI271533, NULL};
const Segment * const SG61189[] = {SGI271534, SGI0, SGI271535, SGI0, SGI271536, SGI271537, SGI271538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61190[] = {SGI271539, SGI271540, NULL};
const Segment * const SG61191[] = {SGI271541, SGI0, SGI271542, SGI0, SGI271543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61192[] = {SGI271544, SGI0, SGI271545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61193[] = {SGI271546, SGI0, SGI271547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61194[] = {SGI271548, SGI0, SGI271549, SGI271550, SGI271551, SGI271552, SGI271553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61195[] = {SGI271554, SGI0, SGI271555, SGI0, SGI271556, SGI271557, SGI271558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271559, SGI0, NULL};
const Segment * const SG61196[] = {SGI271560, SGI0, SGI271561, SGI0, SGI271562, SGI0, SGI271562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61197[] = {SGI271563, SGI271564, NULL};
const Segment * const SG61198[] = {SGI271565, SGI0, SGI271566, SGI0, SGI271567, SGI271568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61199[] = {SGI271569, SGI0, SGI271570, SGI0, SGI271571, SGI271572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61200[] = {SGI271573, SGI0, SGI271574, SGI0, SGI271575, SGI271576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61201[] = {SGI271577, SGI0, SGI271578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61202[] = {SGI271580, SGI0, SGI271581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61203[] = {SGI271583, SGI0, SGI271584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61204[] = {SGI271585, SGI0, SGI271586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61205[] = {SGI271587, SGI0, SGI271588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61206[] = {SGI271589, SGI271590, NULL};
const Segment * const SG61207[] = {SGI271591, SGI271592, NULL};
const Segment * const SG61208[] = {SGI271593, SGI0, SGI271594, SGI0, SGI271595, SGI271596, SGI271597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271598, SGI0, NULL};
const Segment * const SG61209[] = {SGI271599, SGI271600, SGI0, SGI0, NULL};
const Segment * const SG61210[] = {SGI271601, SGI271602, SGI271603, SGI0, SGI271604, SGI271605, SGI271606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61211[] = {SGI271607, SGI271608, SGI271609, SGI0, SGI271610, SGI271611, SGI271612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61212[] = {SGI271613, SGI271614, SGI271615, SGI0, SGI271616, SGI271617, SGI271618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61213[] = {SGI271619, SGI271620, SGI271621, SGI0, SGI271622, SGI271623, SGI271624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61214[] = {SGI271625, SGI271626, SGI271627, SGI0, SGI271628, SGI271629, SGI271630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61215[] = {SGI271631, SGI271632, NULL};
const Segment * const SG61216[] = {SGI271633, SGI271634, NULL};
const Segment * const SG61217[] = {SGI271635, SGI271636, NULL};
const Segment * const SG61218[] = {SGI271637, SGI0, SGI271638, SGI271639, SGI271640, SGI271641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61219[] = {SGI271642, SGI271643, NULL};
const Segment * const SG61220[] = {SGI271644, SGI0, SGI271645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61221[] = {SGI271646, SGI0, SGI271647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61222[] = {SGI271648, SGI0, SGI271649, SGI0, SGI271650, SGI271651, SGI271652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271653, SGI0, NULL};
const Segment * const SG61223[] = {SGI271654, SGI0, SGI271655, SGI0, SGI0, SGI271656, SGI271657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61224[] = {SGI271658, SGI271659, NULL};
const Segment * const SG61225[] = {SGI271660, SGI271661, NULL};
const Segment * const SG61226[] = {SGI271662, SGI271663, NULL};
const Segment * const SG61227[] = {SGI271664, SGI271665, NULL};
const Segment * const SG61228[] = {SGI271666, SGI0, SGI271667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61229[] = {SGI271669, SGI0, SGI271670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61230[] = {SGI271671, SGI0, SGI271672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61231[] = {SGI271673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61232[] = {SGI271674, SGI0, SGI271675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61233[] = {SGI271677, SGI0, SGI271678, SGI271679, SGI271680, SGI271681, SGI271682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61234[] = {SGI271683, SGI271684, SGI0, SGI0, NULL};
const Segment * const SG61235[] = {SGI271685, SGI271686, SGI271687, SGI0, NULL};
const Segment * const SG61236[] = {SGI271688, SGI271689, NULL};
const Segment * const SG61237[] = {SGI271690, SGI0, SGI271691, SGI0, SGI271692, SGI271693, SGI271694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61238[] = {SGI271695, SGI0, SGI271696, SGI0, SGI271697, SGI271698, SGI271699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61239[] = {SGI271700, SGI271701, NULL};
const Segment * const SG61240[] = {SGI271702, SGI271703, SGI0, SGI0, NULL};
const Segment * const SG61241[] = {SGI271704, SGI271705, NULL};
const Segment * const SG61242[] = {SGI271706, SGI271707, NULL};
const Segment * const SG61243[] = {SGI271708, SGI0, SGI271709, SGI0, SGI0, SGI0, SGI0, SGI271710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61244[] = {SGI271711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61245[] = {SGI271712, SGI0, SGI271713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61246[] = {SGI271714, SGI271715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61247[] = {SGI271716, SGI0, SGI271717, SGI0, SGI0, SGI0, SGI0, SGI271718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61248[] = {SGI271719, SGI0, SGI271720, SGI0, SGI0, SGI0, SGI0, SGI271721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61249[] = {SGI271722, SGI0, SGI271723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61250[] = {SGI271725, SGI0, SGI271726, SGI0, SGI0, SGI0, SGI0, SGI271727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61251[] = {SGI271728, SGI0, SGI271729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61252[] = {SGI271731, SGI0, SGI271732, SGI0, SGI0, SGI0, SGI0, SGI271733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61253[] = {SGI271734, SGI0, SGI271735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61254[] = {SGI271737, SGI0, SGI271738, SGI0, SGI0, SGI0, SGI0, SGI271739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61255[] = {SGI271740, SGI0, SGI271741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61256[] = {SGI271742, SGI0, SGI271743, SGI271744, SGI271745, SGI271746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61257[] = {SGI271747, SGI271748, SGI0, SGI0, NULL};
const Segment * const SG61258[] = {SGI271749, SGI271750, NULL};
const Segment * const SG61259[] = {SGI271751, SGI271752, NULL};
const Segment * const SG61260[] = {SGI271753, SGI271754, SGI271755, SGI0, SGI271756, SGI271757, SGI271758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61261[] = {SGI271759, SGI0, SGI271760, SGI0, SGI271761, SGI271762, SGI0, SGI0, SGI0, SGI271763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61262[] = {SGI271764, SGI0, SGI271765, SGI0, SGI271766, SGI271767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61263[] = {SGI271768, SGI0, SGI271769, SGI0, SGI271770, SGI271771, SGI271772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61264[] = {SGI271773, SGI0, SGI271774, SGI0, SGI271775, SGI271776, SGI271777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61265[] = {SGI271778, SGI0, SGI271779, SGI0, SGI271780, SGI271781, SGI271782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61266[] = {SGI271783, SGI0, SGI271784, SGI0, SGI271785, SGI271786, SGI271787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61267[] = {SGI271788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI271789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61268[] = {SGI271790, SGI271791, NULL};
const Segment * const SG61269[] = {SGI271792, SGI271793, NULL};
const Segment * const SG61270[] = {SGI271794, SGI271795, SGI0, SGI0, NULL};
const Segment * const SG61271[] = {SGI271796, SGI271797, NULL};
const Segment * const SG61272[] = {SGI271798, SGI271799, SGI0, SGI0, NULL};
const Segment * const SG61273[] = {SGI271800, SGI271801, NULL};
const Segment * const SG61274[] = {SGI271802, SGI271803, NULL};
const Segment * const SG61275[] = {SGI271804, SGI271805, NULL};
const Segment * const SG61276[] = {SGI271806, SGI271807, NULL};
const Segment * const SG61277[] = {SGI271808, SGI271809, NULL};
const Segment * const SG61278[] = {SGI271810, SGI271811, NULL};
const Segment * const SG61279[] = {SGI271812, SGI271813, NULL};
const Segment * const SG61280[] = {SGI271814, SGI271815, NULL};
const Segment * const SG61281[] = {SGI271816, SGI271817, SGI0, SGI0, NULL};
const Segment * const SG61282[] = {SGI271818, SGI271819, NULL};
const Segment * const SG61283[] = {SGI271820, SGI271821, NULL};
const Segment * const SG61284[] = {SGI271822, SGI271823, NULL};
const Segment * const SG61285[] = {SGI271824, SGI271825, NULL};
const Segment * const SG61286[] = {SGI271826, SGI271827, NULL};
const Segment * const SG61287[] = {SGI271828, SGI271829, NULL};
const Segment * const SG61288[] = {SGI271830, SGI271831, NULL};
const Segment * const SG61289[] = {SGI271832, SGI271833, NULL};
const Segment * const SG61290[] = {SGI271834, SGI271835, NULL};
const Segment * const SG61291[] = {SGI271836, SGI271837, NULL};
const Segment * const SG61292[] = {SGI271838, SGI271839, NULL};
const Segment * const SG61293[] = {SGI271840, SGI271841, NULL};
const Segment * const SG61294[] = {SGI271842, SGI271843, NULL};
const Segment * const SG61295[] = {SGI271844, SGI271845, NULL};
const Segment * const SG61296[] = {SGI271846, SGI271847, SGI0, SGI0, NULL};
const Segment * const SG61297[] = {SGI271848, SGI271849, NULL};
const Segment * const SG61298[] = {SGI271850, SGI271851, NULL};
const Segment * const SG61299[] = {SGI271852, SGI271853, NULL};
const Segment * const SG61300[] = {SGI271854, SGI271855, NULL};
const Segment * const SG61301[] = {SGI271856, SGI271857, NULL};
const Segment * const SG61302[] = {SGI271858, SGI271859, NULL};
const Segment * const SG61303[] = {SGI271860, SGI271861, SGI0, SGI0, NULL};
const Segment * const SG61304[] = {SGI271862, SGI271863, NULL};
const Segment * const SG61305[] = {SGI271864, SGI271865, SGI271866, SGI0, NULL};
const Segment * const SG61306[] = {SGI271867, SGI271868, SGI271869, SGI0, NULL};
const Segment * const SG61307[] = {SGI271870, SGI271871, SGI0, SGI0, NULL};
const Segment * const SG61308[] = {SGI271872, SGI271873, NULL};
const Segment * const SG61309[] = {SGI271874, SGI271875, SGI0, SGI271876, NULL};
const Segment * const SG61310[] = {SGI271877, SGI271878, SGI271879, SGI0, NULL};
const Segment * const SG61311[] = {SGI271880, SGI271881, NULL};
const Segment * const SG61312[] = {SGI271882, SGI271883, SGI0, SGI0, NULL};
const Segment * const SG61313[] = {SGI271884, SGI271885, SGI0, SGI271886, NULL};
const Segment * const SG61314[] = {SGI271887, SGI271888, NULL};
const Segment * const SG61315[] = {SGI271889, SGI271890, NULL};
const Segment * const SG61316[] = {SGI271891, SGI271892, NULL};
const Segment * const SG61317[] = {SGI271893, SGI271894, SGI0, SGI0, NULL};
const Segment * const SG61318[] = {SGI271895, SGI271896, NULL};
const Segment * const SG61319[] = {SGI271897, SGI271898, SGI0, SGI0, NULL};
const Segment * const SG61320[] = {SGI271899, SGI271900, SGI0, SGI0, NULL};
const Segment * const SG61321[] = {SGI271901, SGI271902, NULL};
const Segment * const SG61322[] = {SGI271903, SGI271904, NULL};
const Segment * const SG61323[] = {SGI271905, SGI271906, SGI0, SGI0, NULL};
const Segment * const SG61324[] = {SGI271907, SGI271908, SGI271909, SGI0, NULL};
const Segment * const SG61325[] = {SGI271910, SGI271911, NULL};
const Segment * const SG61326[] = {SGI271912, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61327[] = {SGI271913, SGI271914, SGI0, SGI0, NULL};
const Segment * const SG61328[] = {SGI271915, SGI271916, SGI0, SGI0, NULL};
const Segment * const SG61329[] = {SGI271917, SGI271918, SGI0, SGI271919, NULL};
const Segment * const SG61330[] = {SGI271920, SGI271921, SGI0, SGI0, NULL};
const Segment * const SG61331[] = {SGI271922, SGI271923, SGI0, SGI0, NULL};
const Segment * const SG61332[] = {SGI271924, SGI271925, SGI0, SGI0, NULL};
const Segment * const SG61333[] = {SGI271926, SGI271927, SGI0, SGI0, NULL};
const Segment * const SG61334[] = {SGI271928, SGI271929, SGI271930, SGI0, NULL};
const Segment * const SG61335[] = {SGI271931, SGI271932, SGI0, SGI0, NULL};
const Segment * const SG61336[] = {SGI271933, SGI271934, SGI271935, SGI0, NULL};
const Segment * const SG61337[] = {SGI271936, SGI271937, SGI271938, SGI0, NULL};
const Segment * const SG61338[] = {SGI271939, SGI271940, SGI271941, SGI0, NULL};
const Segment * const SG61339[] = {SGI271942, SGI271943, SGI271944, SGI0, NULL};
const Segment * const SG61340[] = {SGI271945, SGI271946, NULL};
const Segment * const SG61341[] = {SGI271947, SGI271948, NULL};
const Segment * const SG61342[] = {SGI271949, SGI271950, NULL};
const Segment * const SG61343[] = {SGI271951, SGI271952, SGI271953, SGI0, NULL};
const Segment * const SG61344[] = {SGI271954, SGI271955, SGI271956, SGI0, NULL};
const Segment * const SG61345[] = {SGI271957, SGI271958, NULL};
const Segment * const SG61346[] = {SGI271959, SGI271960, SGI0, SGI0, NULL};
const Segment * const SG61347[] = {SGI271961, SGI271962, SGI271963, SGI0, NULL};
const Segment * const SG61348[] = {SGI271964, SGI271965, SGI0, SGI0, NULL};
const Segment * const SG61349[] = {SGI271966, SGI271967, SGI271968, SGI0, NULL};
const Segment * const SG61350[] = {SGI271969, SGI271970, SGI271971, SGI0, NULL};
const Segment * const SG61351[] = {SGI271972, SGI271973, SGI271974, SGI0, NULL};
const Segment * const SG61352[] = {SGI271975, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61353[] = {SGI271976, SGI271977, SGI0, SGI0, NULL};
const Segment * const SG61354[] = {SGI271978, SGI271979, SGI0, SGI271980, NULL};
const Segment * const SG61355[] = {SGI271981, SGI271982, NULL};
const Segment * const SG61356[] = {SGI271983, SGI271984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61357[] = {SGI271985, SGI271986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61358[] = {SGI271987, SGI271988, SGI271989, SGI0, NULL};
const Segment * const SG61359[] = {SGI271991, SGI271992, SGI0, SGI0, NULL};
const Segment * const SG61360[] = {SGI271990, SGI271993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61361[] = {SGI271994, SGI271995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61362[] = {SGI271996, SGI271997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61363[] = {SGI271998, SGI271999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61364[] = {SGI272000, SGI272001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61365[] = {SGI272002, SGI272003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61366[] = {SGI272004, SGI272005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61367[] = {SGI272006, SGI272007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61368[] = {SGI272008, SGI272009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61369[] = {SGI272010, SGI272011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61370[] = {SGI272012, SGI272013, SGI0, SGI272014, NULL};
const Segment * const SG61371[] = {SGI272015, SGI272016, SGI0, SGI272017, NULL};
const Segment * const SG61372[] = {SGI272018, SGI272019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61373[] = {SGI272020, SGI272021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61374[] = {SGI272022, SGI272023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61375[] = {SGI272024, SGI272025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61376[] = {SGI272026, SGI272027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61377[] = {SGI272028, SGI272029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61378[] = {SGI272030, SGI272031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61379[] = {SGI272032, SGI272033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61380[] = {SGI272034, SGI272035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61381[] = {SGI272036, SGI272037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61382[] = {SGI272038, SGI272039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61383[] = {SGI272040, SGI272041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61384[] = {SGI272042, SGI272043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61385[] = {SGI272044, SGI272045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61386[] = {SGI272046, SGI272047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61387[] = {SGI272048, SGI272049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61388[] = {SGI272050, SGI272051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61389[] = {SGI272052, SGI272053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61390[] = {SGI272054, SGI272055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61391[] = {SGI272056, SGI272057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61392[] = {SGI272058, SGI272059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61393[] = {SGI272060, SGI272061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61394[] = {SGI272062, SGI272063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61395[] = {SGI272064, SGI272065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61396[] = {SGI272066, SGI272067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61397[] = {SGI272068, SGI272069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61398[] = {SGI272070, SGI272071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61399[] = {SGI272072, SGI272073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61400[] = {SGI272074, SGI272075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61401[] = {SGI272076, SGI272077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61402[] = {SGI272078, SGI272079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61403[] = {SGI272080, SGI272081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61404[] = {SGI272082, SGI272083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61405[] = {SGI272084, SGI272085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61406[] = {SGI272086, SGI272087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61407[] = {SGI272088, SGI272089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61408[] = {SGI272090, SGI272091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61409[] = {SGI272092, SGI272093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61410[] = {SGI272094, SGI272095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61411[] = {SGI272096, SGI272097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61412[] = {SGI272098, SGI272099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61413[] = {SGI272100, SGI272101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61414[] = {SGI272102, SGI272103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61415[] = {SGI272104, SGI272105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61416[] = {SGI272106, SGI272107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61417[] = {SGI272108, SGI272109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61418[] = {SGI272110, SGI272111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61419[] = {SGI272112, SGI272113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61420[] = {SGI272114, SGI272115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61421[] = {SGI272116, SGI272117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61422[] = {SGI272118, SGI272119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61423[] = {SGI272120, SGI272121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61424[] = {SGI272122, SGI272123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61425[] = {SGI272124, SGI272125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61426[] = {SGI272126, SGI272127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61427[] = {SGI272128, SGI272129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61428[] = {SGI272130, SGI272131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61429[] = {SGI272132, SGI272133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61430[] = {SGI272134, SGI272135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61431[] = {SGI272136, SGI272137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61432[] = {SGI272138, SGI272139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61433[] = {SGI272140, SGI272141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61434[] = {SGI272142, SGI272143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61435[] = {SGI272144, SGI272145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61436[] = {SGI272146, SGI272147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61437[] = {SGI272148, SGI272149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61438[] = {SGI272150, SGI272151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61439[] = {SGI272152, SGI272153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61440[] = {SGI272154, SGI272155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61441[] = {SGI272156, SGI272157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61442[] = {SGI272158, SGI272159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61443[] = {SGI272160, SGI272161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61444[] = {SGI272162, SGI272163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61445[] = {SGI272164, SGI272165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61446[] = {SGI272166, SGI272167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61447[] = {SGI272168, SGI272169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61448[] = {SGI272170, SGI272171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61449[] = {SGI272172, SGI272173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61450[] = {SGI272174, SGI272175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61451[] = {SGI272176, SGI272177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61452[] = {SGI272178, SGI272179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61453[] = {SGI272180, SGI272181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61454[] = {SGI272182, SGI272183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61455[] = {SGI272184, SGI272185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61456[] = {SGI272186, SGI272187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61457[] = {SGI272188, SGI272189, SGI0, SGI0, NULL};
const Segment * const SG61458[] = {SGI272190, SGI272191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61459[] = {SGI272192, SGI272193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61460[] = {SGI272194, SGI272195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61461[] = {SGI272196, SGI272197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61462[] = {SGI272198, SGI272199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61463[] = {SGI272200, SGI272201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61464[] = {SGI272202, SGI272203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61465[] = {SGI272204, SGI272205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI272206, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61466[] = {SGI272207, SGI272208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61467[] = {SGI272209, SGI272210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61468[] = {SGI272211, SGI272212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61469[] = {SGI272213, SGI272214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61470[] = {SGI272215, SGI272216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61471[] = {SGI272217, SGI272218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61472[] = {SGI272219, SGI272220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61473[] = {SGI272221, SGI272222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61474[] = {SGI272223, SGI272224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61475[] = {SGI272225, SGI272226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61476[] = {SGI272227, SGI272228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61477[] = {SGI272229, SGI272230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61478[] = {SGI272231, SGI272232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61479[] = {SGI272233, SGI272234, SGI0, SGI0, NULL};
const Segment * const SG61480[] = {SGI272235, SGI272236, SGI0, SGI0, NULL};
const Segment * const SG61481[] = {SGI272237, SGI272238, SGI272239, SGI0, NULL};
const Segment * const SG61482[] = {SGI272240, SGI272241, SGI272242, SGI0, NULL};
const Segment * const SG61483[] = {SGI272243, SGI272244, SGI272245, SGI0, NULL};
const Segment * const SG61484[] = {SGI272246, SGI272247, SGI272248, SGI0, NULL};
const Segment * const SG61485[] = {SGI272249, SGI272250, SGI272251, SGI0, NULL};
const Segment * const SG61486[] = {SGI272252, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61487[] = {SGI272253, SGI272254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI272255, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61488[] = {SGI272256, SGI272257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI272258, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61489[] = {SGI272259, SGI272260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61490[] = {SGI272261, SGI272262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61491[] = {SGI272263, SGI272264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61492[] = {SGI272265, SGI272266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61493[] = {SGI272267, SGI272268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61494[] = {SGI272269, SGI272270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61495[] = {SGI272271, SGI272272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61496[] = {SGI272273, SGI272274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61497[] = {SGI272275, SGI272276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61498[] = {SGI272277, SGI272278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61499[] = {SGI272279, SGI272280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61500[] = {SGI272281, SGI272282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61501[] = {SGI272283, SGI272284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61502[] = {SGI272285, SGI272286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61503[] = {SGI272287, SGI272288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61504[] = {SGI272289, SGI272290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61505[] = {SGI272291, SGI272292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61506[] = {SGI272293, SGI272294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61507[] = {SGI272295, SGI272296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61508[] = {SGI272297, SGI272298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61509[] = {SGI272299, SGI272300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61510[] = {SGI272301, SGI272302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61511[] = {SGI272303, SGI272304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61512[] = {SGI272305, SGI272306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61513[] = {SGI272307, SGI272308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61514[] = {SGI272309, SGI272310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61515[] = {SGI272311, SGI272312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61516[] = {SGI272313, SGI272314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61517[] = {SGI272315, SGI272316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61518[] = {SGI272317, SGI272318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61519[] = {SGI272319, SGI272320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61520[] = {SGI272321, SGI272322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61521[] = {SGI272323, SGI272324, SGI0, SGI0, NULL};
const Segment * const SG61522[] = {SGI272325, SGI272326, SGI0, SGI0, NULL};
const Segment * const SG61523[] = {SGI272327, SGI272328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61524[] = {SGI272329, SGI272330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61525[] = {SGI272331, SGI272332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61526[] = {SGI272333, SGI272334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61527[] = {SGI272335, SGI272336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61528[] = {SGI272337, SGI272338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61529[] = {SGI272339, SGI272340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61530[] = {SGI272341, SGI272342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61531[] = {SGI272343, SGI272344, SGI0, SGI0, NULL};
const Segment * const SG61532[] = {SGI272345, SGI272346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61533[] = {SGI272347, SGI272348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61534[] = {SGI272349, SGI272350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61535[] = {SGI272351, SGI272352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61536[] = {SGI272353, SGI272354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61537[] = {SGI272355, SGI272356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61538[] = {SGI272357, SGI272358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61539[] = {SGI272359, SGI272360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61540[] = {SGI272361, SGI272362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61541[] = {SGI272363, SGI272364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61542[] = {SGI272365, SGI272366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61543[] = {SGI272367, SGI272368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61544[] = {SGI272369, SGI272370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61545[] = {SGI272371, SGI272372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61546[] = {SGI272373, SGI272374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61547[] = {SGI272375, SGI272376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61548[] = {SGI272377, SGI272378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61549[] = {SGI272379, SGI272380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61550[] = {SGI272381, SGI272382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61551[] = {SGI272383, SGI272384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61552[] = {SGI272385, SGI272386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61553[] = {SGI272387, SGI272388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61554[] = {SGI272389, SGI272390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61555[] = {SGI272391, SGI272392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61556[] = {SGI272393, SGI272394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61557[] = {SGI272395, SGI272396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61558[] = {SGI272397, SGI272398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61559[] = {SGI272399, SGI272400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61560[] = {SGI272401, SGI272402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61561[] = {SGI272403, SGI272404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61562[] = {SGI272405, SGI272406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61563[] = {SGI272407, SGI272408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61564[] = {SGI272409, SGI272410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61565[] = {SGI272411, SGI272412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61566[] = {SGI272413, SGI272414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61567[] = {SGI272415, SGI272416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61568[] = {SGI272417, SGI272418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61569[] = {SGI272419, SGI272420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61570[] = {SGI272421, SGI272422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61571[] = {SGI272423, SGI272424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61572[] = {SGI272425, SGI272426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61573[] = {SGI272427, SGI272428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61574[] = {SGI272429, SGI272430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61575[] = {SGI272431, SGI272432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61576[] = {SGI272433, SGI272434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61577[] = {SGI272435, SGI272436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61578[] = {SGI272437, SGI272438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61579[] = {SGI272439, SGI272440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61580[] = {SGI272441, SGI272442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61581[] = {SGI272443, SGI272444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61582[] = {SGI272445, SGI272446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61583[] = {SGI272447, SGI272448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61584[] = {SGI272449, SGI272450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61585[] = {SGI272451, SGI272452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61586[] = {SGI272453, SGI272454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61587[] = {SGI272455, SGI272456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61588[] = {SGI272457, SGI272458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61589[] = {SGI272459, SGI272460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61590[] = {SGI272461, SGI272462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61591[] = {SGI272463, SGI272464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61592[] = {SGI272465, SGI272466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61593[] = {SGI272467, SGI272468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61594[] = {SGI272469, SGI272470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61595[] = {SGI272471, SGI272472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61596[] = {SGI272473, SGI272474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61597[] = {SGI272475, SGI272476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61598[] = {SGI272477, SGI272478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61599[] = {SGI272479, SGI272480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61600[] = {SGI272481, SGI272482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61601[] = {SGI272483, SGI272484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61602[] = {SGI272485, SGI272486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61603[] = {SGI272487, SGI272488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61604[] = {SGI272489, SGI272490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61605[] = {SGI272491, SGI272492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61606[] = {SGI272493, SGI272494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61607[] = {SGI272495, SGI272496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61608[] = {SGI272497, SGI272498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61609[] = {SGI272499, SGI272500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61610[] = {SGI272501, SGI272502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61611[] = {SGI272503, SGI272504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61612[] = {SGI272505, SGI272506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61613[] = {SGI272507, SGI272508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61614[] = {SGI272509, SGI272510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61615[] = {SGI272511, SGI272512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61616[] = {SGI272513, SGI272514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61617[] = {SGI272515, SGI272516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61618[] = {SGI272517, SGI272518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61619[] = {SGI272519, SGI272520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61620[] = {SGI272521, SGI272522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61621[] = {SGI272523, SGI272524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61622[] = {SGI272525, SGI272526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61623[] = {SGI272527, SGI272528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61624[] = {SGI272529, SGI272530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61625[] = {SGI272531, SGI272532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61626[] = {SGI272533, SGI272534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61627[] = {SGI272535, SGI272536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61628[] = {SGI272537, SGI272538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61629[] = {SGI272539, SGI272540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61630[] = {SGI272541, SGI272542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61631[] = {SGI272543, SGI272544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61632[] = {SGI272545, SGI272546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61633[] = {SGI272547, SGI272548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61634[] = {SGI272549, SGI272550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61635[] = {SGI272551, SGI272552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61636[] = {SGI272553, SGI272554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61637[] = {SGI272555, SGI272556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61638[] = {SGI272557, SGI272558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61639[] = {SGI272559, SGI272560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61640[] = {SGI272561, SGI272562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61641[] = {SGI272563, SGI272564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61642[] = {SGI272565, SGI272566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61643[] = {SGI272567, SGI272568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61644[] = {SGI272569, SGI272570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61645[] = {SGI272571, SGI272572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61646[] = {SGI272573, SGI272574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61647[] = {SGI272575, SGI272576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61648[] = {SGI272577, SGI272578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61649[] = {SGI272579, SGI272580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI272581, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61650[] = {SGI272582, SGI272583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI272584, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61651[] = {SGI272585, SGI272586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61652[] = {SGI272587, SGI272588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61653[] = {SGI272589, SGI272590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61654[] = {SGI272591, SGI272592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61655[] = {SGI272593, SGI272594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61656[] = {SGI272595, SGI272596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61657[] = {SGI272597, SGI272598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61658[] = {SGI272599, SGI272600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61659[] = {SGI272601, SGI272602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61660[] = {SGI272603, SGI272604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61661[] = {SGI272605, SGI272606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61662[] = {SGI272607, SGI272608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61663[] = {SGI272609, SGI272610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61664[] = {SGI272611, SGI272612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61665[] = {SGI272613, SGI272614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61666[] = {SGI272615, SGI272616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61667[] = {SGI272617, SGI272618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61668[] = {SGI272619, SGI272620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61669[] = {SGI272621, SGI272622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61670[] = {SGI272623, SGI272624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61671[] = {SGI272625, SGI272626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI272627, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61672[] = {SGI272628, SGI272629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61673[] = {SGI272630, SGI272631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61674[] = {SGI272632, SGI272633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61675[] = {SGI272634, SGI272635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61676[] = {SGI272636, SGI272637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61677[] = {SGI272638, SGI272639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61678[] = {SGI272640, SGI272641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61679[] = {SGI272642, SGI272643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61680[] = {SGI272644, SGI272645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61681[] = {SGI272646, SGI272647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61682[] = {SGI272648, SGI272649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61683[] = {SGI272650, SGI272651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61684[] = {SGI272652, SGI272653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61685[] = {SGI272654, SGI272655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61686[] = {SGI272656, SGI272657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61687[] = {SGI272658, SGI272659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61688[] = {SGI272660, SGI272661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61689[] = {SGI272662, SGI272663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61690[] = {SGI272664, SGI272665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61691[] = {SGI272666, SGI272667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61692[] = {SGI272668, SGI272669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61693[] = {SGI272670, SGI272671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61694[] = {SGI272672, SGI272673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61695[] = {SGI272674, SGI272675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61696[] = {SGI272676, SGI272677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61697[] = {SGI272678, SGI272679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61698[] = {SGI272680, SGI272681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61699[] = {SGI272682, SGI272683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61700[] = {SGI272684, SGI272685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61701[] = {SGI272686, SGI272687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61702[] = {SGI272688, SGI272689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61703[] = {SGI272690, SGI272691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61704[] = {SGI272692, SGI272693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61705[] = {SGI272694, SGI272695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61706[] = {SGI272696, SGI272697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61707[] = {SGI272698, SGI272699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61708[] = {SGI272700, SGI272701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61709[] = {SGI272702, SGI272703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61710[] = {SGI272704, SGI272705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61711[] = {SGI272706, SGI272707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61712[] = {SGI272708, SGI272709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61713[] = {SGI272710, SGI272711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61714[] = {SGI272712, SGI272713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61715[] = {SGI272714, SGI272715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61716[] = {SGI272716, SGI272717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61717[] = {SGI272718, SGI272719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61718[] = {SGI272720, SGI272721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61719[] = {SGI272722, SGI272723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61720[] = {SGI272724, SGI272725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61721[] = {SGI272726, SGI272727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61722[] = {SGI272728, SGI272729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61723[] = {SGI272730, SGI272731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61724[] = {SGI272732, SGI272733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61725[] = {SGI272734, SGI272735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61726[] = {SGI272736, SGI272737, NULL};
const Segment * const SG61727[] = {SGI272738, SGI272739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61728[] = {SGI272740, SGI272741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61729[] = {SGI272742, SGI272743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61730[] = {SGI272744, SGI272745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61731[] = {SGI272746, SGI272747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61732[] = {SGI272748, SGI272749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61733[] = {SGI272750, SGI272751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61734[] = {SGI272752, SGI272753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61735[] = {SGI272754, SGI272755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61736[] = {SGI272756, SGI272757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61737[] = {SGI272758, SGI272759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61738[] = {SGI272760, SGI272761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61739[] = {SGI272762, SGI272763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61740[] = {SGI272764, SGI272765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61741[] = {SGI272766, SGI272767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61742[] = {SGI272768, SGI272769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61743[] = {SGI272770, SGI272771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61744[] = {SGI272772, SGI272773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61745[] = {SGI272774, SGI272775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61746[] = {SGI272776, SGI272777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61747[] = {SGI272778, SGI272779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61748[] = {SGI272780, SGI272781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61749[] = {SGI272782, SGI272783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61750[] = {SGI272784, SGI272785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61751[] = {SGI272786, SGI272787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61752[] = {SGI272788, SGI272789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61753[] = {SGI272790, SGI272791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61754[] = {SGI272792, SGI272793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61755[] = {SGI272794, SGI272795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61756[] = {SGI272796, SGI272797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61757[] = {SGI272798, SGI272799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61758[] = {SGI272800, SGI272801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61759[] = {SGI272802, SGI272803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI272804, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61760[] = {SGI272805, SGI272806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61761[] = {SGI272807, SGI272808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61762[] = {SGI272809, SGI272810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61763[] = {SGI272811, SGI272812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61764[] = {SGI272813, SGI272814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61765[] = {SGI272815, SGI272816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61766[] = {SGI272817, SGI272818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61767[] = {SGI272819, SGI272820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61768[] = {SGI272821, SGI272822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61769[] = {SGI272823, SGI272824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61770[] = {SGI272825, SGI272826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61771[] = {SGI272827, SGI272828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61772[] = {SGI272829, SGI272830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61773[] = {SGI272831, SGI272832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61774[] = {SGI272833, SGI272834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61775[] = {SGI272835, SGI272836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61776[] = {SGI272837, SGI272838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61777[] = {SGI272839, SGI272840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61778[] = {SGI272841, SGI272842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61779[] = {SGI272843, SGI272844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61780[] = {SGI272845, SGI272846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61781[] = {SGI272847, SGI272848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61782[] = {SGI272849, SGI272850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61783[] = {SGI272851, SGI272852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61784[] = {SGI272853, SGI272854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61785[] = {SGI272855, SGI272856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61786[] = {SGI272857, SGI272858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61787[] = {SGI272859, SGI272860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61788[] = {SGI272861, SGI272862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61789[] = {SGI272863, SGI272864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61790[] = {SGI272865, SGI272866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61791[] = {SGI272867, SGI272868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61792[] = {SGI272869, SGI272870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61793[] = {SGI272871, SGI272872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61794[] = {SGI272873, SGI272874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61795[] = {SGI272875, SGI272876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61796[] = {SGI272877, SGI272878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61797[] = {SGI272879, SGI272880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61798[] = {SGI0, SGI272881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61799[] = {SGI272882, SGI272883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61800[] = {SGI272884, SGI272885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61801[] = {SGI272886, SGI272887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61802[] = {SGI272888, SGI272889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61803[] = {SGI272890, SGI272891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61804[] = {SGI272892, SGI272893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61805[] = {SGI272894, SGI272895, NULL};
const Segment * const SG61806[] = {SGI272896, SGI272897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61807[] = {SGI272898, SGI272899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61808[] = {SGI272900, SGI272901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61809[] = {SGI272902, SGI272903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61810[] = {SGI272904, SGI272905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61811[] = {SGI272906, SGI272907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61812[] = {SGI272908, SGI272909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61813[] = {SGI272910, SGI272911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61814[] = {SGI272912, SGI272913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61815[] = {SGI272914, SGI272915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61816[] = {SGI272916, SGI272917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61817[] = {SGI272918, SGI272919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61818[] = {SGI272920, SGI272921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61819[] = {SGI272922, SGI272923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61820[] = {SGI272924, SGI272925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61821[] = {SGI272926, SGI272927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61822[] = {SGI272928, SGI272929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61823[] = {SGI272930, SGI272931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61824[] = {SGI272932, SGI272933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61825[] = {SGI272934, SGI272935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61826[] = {SGI272936, SGI272937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61827[] = {SGI272938, SGI272939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61828[] = {SGI272940, SGI272941, NULL};
const Segment * const SG61829[] = {SGI272942, SGI272943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61830[] = {SGI272944, SGI272945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61831[] = {SGI272946, SGI272947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61832[] = {SGI272948, SGI272949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61833[] = {SGI272950, SGI272951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61834[] = {SGI272952, SGI272953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61835[] = {SGI272954, SGI272955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61836[] = {SGI272956, SGI272957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61837[] = {SGI272958, SGI272959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61838[] = {SGI272960, SGI272961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61839[] = {SGI272962, SGI272963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61840[] = {SGI272964, SGI272965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61841[] = {SGI272966, SGI272967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61842[] = {SGI272968, SGI272969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61843[] = {SGI272970, SGI272971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61844[] = {SGI272972, SGI272973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61845[] = {SGI272974, SGI272975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI272976, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61846[] = {SGI272977, SGI272978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61847[] = {SGI272979, SGI272980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61848[] = {SGI272981, SGI272982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61849[] = {SGI272983, SGI272984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61850[] = {SGI272985, SGI272986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61851[] = {SGI272987, SGI272988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61852[] = {SGI272989, SGI272990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61853[] = {SGI272991, SGI272992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61854[] = {SGI272993, SGI272994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61855[] = {SGI272995, SGI272996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI272997, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61856[] = {SGI272998, SGI272999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61857[] = {SGI273000, SGI273001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI273002, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61858[] = {SGI273003, SGI273004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61859[] = {SGI273005, SGI273006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61860[] = {SGI273007, SGI273008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61861[] = {SGI273009, SGI273010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61862[] = {SGI273011, SGI273012, NULL};
const Segment * const SG61863[] = {SGI273013, SGI273014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61864[] = {SGI273015, SGI273016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61865[] = {SGI273017, SGI273018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61866[] = {SGI273019, SGI273020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61867[] = {SGI273021, SGI273022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61868[] = {SGI273023, SGI273024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61869[] = {SGI273025, SGI273026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61870[] = {SGI273027, SGI273028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61871[] = {SGI273029, SGI273030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61872[] = {SGI273031, SGI273032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI273033, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61873[] = {SGI273034, SGI273035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61874[] = {SGI273036, SGI273037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61875[] = {SGI273038, SGI273039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61876[] = {SGI273040, SGI273041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61877[] = {SGI273042, SGI273043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61878[] = {SGI273044, SGI273045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61879[] = {SGI273046, SGI273047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61880[] = {SGI273048, SGI273049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61881[] = {SGI273050, SGI273051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61882[] = {SGI273052, SGI273053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61883[] = {SGI273054, SGI273055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61884[] = {SGI273056, SGI273057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61885[] = {SGI273058, SGI273059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61886[] = {SGI273060, SGI273061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61887[] = {SGI273062, SGI273063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61888[] = {SGI273064, SGI273065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61889[] = {SGI273066, SGI273067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61890[] = {SGI273068, SGI273069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI273070, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61891[] = {SGI273071, SGI273072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI273073, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61892[] = {SGI273074, SGI273075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61893[] = {SGI273076, SGI273077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61894[] = {SGI273078, SGI273079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61895[] = {SGI273080, SGI273081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61896[] = {SGI273082, SGI273083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61897[] = {SGI273084, SGI273085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61898[] = {SGI273086, SGI273087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61899[] = {SGI273088, SGI273089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61900[] = {SGI273090, SGI273091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61901[] = {SGI273092, SGI273093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61902[] = {SGI273094, SGI273095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61903[] = {SGI273096, SGI273097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61904[] = {SGI273098, SGI273099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61905[] = {SGI273100, SGI273101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61906[] = {SGI273102, SGI273103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61907[] = {SGI273104, SGI273105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61908[] = {SGI273106, SGI273107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61909[] = {SGI273108, SGI273109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61910[] = {SGI273110, SGI273111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61911[] = {SGI273112, SGI273113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61912[] = {SGI273114, SGI273115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61913[] = {SGI273116, SGI273117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61914[] = {SGI273118, SGI273119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61915[] = {SGI273120, SGI273121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61916[] = {SGI273122, SGI273123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61917[] = {SGI273124, SGI273125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI273126, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61918[] = {SGI273127, SGI273128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61919[] = {SGI273129, SGI273130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61920[] = {SGI273131, SGI273132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI273133, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61921[] = {SGI273134, SGI273135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61922[] = {SGI273136, SGI273137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61923[] = {SGI273138, SGI273139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61924[] = {SGI273140, SGI273141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61925[] = {SGI273142, SGI273143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61926[] = {SGI273144, SGI273145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61927[] = {SGI273146, SGI273147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61928[] = {SGI273148, SGI273149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61929[] = {SGI273150, SGI273151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61930[] = {SGI273152, SGI273153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61931[] = {SGI273154, SGI273155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61932[] = {SGI273156, SGI273157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61933[] = {SGI273158, SGI273159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61934[] = {SGI273160, SGI273161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61935[] = {SGI273162, SGI273163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI273164, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61936[] = {SGI273165, SGI273166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61937[] = {SGI273167, SGI273168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61938[] = {SGI273169, SGI273170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61939[] = {SGI273171, SGI273172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61940[] = {SGI273173, SGI273174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61941[] = {SGI273175, SGI273176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61942[] = {SGI273177, SGI273178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61943[] = {SGI273179, SGI273180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI273181, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61944[] = {SGI273182, SGI273183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI273184, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61945[] = {SGI273185, SGI273186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI273187, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61946[] = {SGI273188, SGI273189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61947[] = {SGI273190, SGI273191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61948[] = {SGI273192, SGI273193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61949[] = {SGI273194, SGI273195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61950[] = {SGI273196, SGI273197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61951[] = {SGI273198, SGI273199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61952[] = {SGI273200, SGI273201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61953[] = {SGI273202, SGI273203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61954[] = {SGI273204, SGI273205, NULL};
const Segment * const SG61955[] = {SGI273206, SGI273207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61956[] = {SGI273208, SGI273209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61957[] = {SGI273210, SGI273211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61958[] = {SGI273212, SGI273213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61959[] = {SGI273214, SGI273215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61960[] = {SGI273216, SGI273217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61961[] = {SGI273218, SGI273219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61962[] = {SGI273220, SGI273221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61963[] = {SGI273222, SGI273223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61964[] = {SGI273224, SGI273225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61965[] = {SGI273226, SGI273227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61966[] = {SGI273228, SGI273229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61967[] = {SGI273230, SGI273231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61968[] = {SGI273232, SGI273233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61969[] = {SGI273234, SGI273235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61970[] = {SGI273236, SGI273237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61971[] = {SGI273238, SGI273239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61972[] = {SGI273240, SGI273241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61973[] = {SGI273242, SGI273243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61974[] = {SGI273244, SGI273245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61975[] = {SGI273246, SGI273247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61976[] = {SGI273248, SGI273249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61977[] = {SGI273250, SGI273251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61978[] = {SGI273252, SGI273253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61979[] = {SGI273254, SGI273255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61980[] = {SGI273256, SGI273257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61981[] = {SGI273258, SGI273259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61982[] = {SGI273260, SGI273261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61983[] = {SGI273262, SGI273263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61984[] = {SGI273264, SGI273265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61985[] = {SGI273266, SGI273267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61986[] = {SGI273268, SGI273269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61987[] = {SGI273270, SGI273271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61988[] = {SGI273272, SGI273273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61989[] = {SGI273274, SGI273275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61990[] = {SGI273276, SGI273277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61991[] = {SGI273278, SGI273279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61992[] = {SGI273280, SGI273281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61993[] = {SGI273282, SGI273283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61994[] = {SGI273284, SGI273285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61995[] = {SGI273286, SGI273287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61996[] = {SGI273288, SGI273289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61997[] = {SGI273290, SGI273291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61998[] = {SGI273292, SGI273293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG61999[] = {SGI273294, SGI273295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62000[] = {SGI273296, SGI273297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62001[] = {SGI273298, SGI273299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62002[] = {SGI273300, SGI273301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62003[] = {SGI273302, SGI273303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62004[] = {SGI273304, SGI273305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62005[] = {SGI273306, SGI273307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62006[] = {SGI273308, SGI273309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62007[] = {SGI273310, SGI273311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62008[] = {SGI273312, SGI273313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62009[] = {SGI273314, SGI273315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62010[] = {SGI273316, SGI273317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62011[] = {SGI273318, SGI273319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62012[] = {SGI273320, SGI273321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62013[] = {SGI273322, SGI273323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62014[] = {SGI273324, SGI273325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62015[] = {SGI273326, SGI273327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62016[] = {SGI273328, SGI273329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62017[] = {SGI273330, SGI273331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62018[] = {SGI273332, SGI273333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62019[] = {SGI273334, SGI273335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62020[] = {SGI273336, SGI273337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62021[] = {SGI273338, SGI273339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62022[] = {SGI273340, SGI273341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62023[] = {SGI273342, SGI273343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62024[] = {SGI273344, SGI273345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62025[] = {SGI273346, SGI273347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62026[] = {SGI273348, SGI273349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62027[] = {SGI273350, SGI273351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62028[] = {SGI273352, SGI273353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62029[] = {SGI273354, SGI273355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62030[] = {SGI273356, SGI273357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62031[] = {SGI273358, SGI273359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62032[] = {SGI273360, SGI273361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62033[] = {SGI273362, SGI273363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62034[] = {SGI273364, SGI273365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62035[] = {SGI273366, SGI273367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62036[] = {SGI272942, SGI272943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62037[] = {SGI272944, SGI272945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62038[] = {SGI273368, SGI273369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62039[] = {SGI273370, SGI273371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62040[] = {SGI273372, SGI273373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62041[] = {SGI273374, SGI273375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62042[] = {SGI273376, SGI273377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62043[] = {SGI273378, SGI273379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62044[] = {SGI273380, SGI273381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62045[] = {SGI273382, SGI273383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62046[] = {SGI273384, SGI273385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62047[] = {SGI273386, SGI273387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62048[] = {SGI273388, SGI273389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62049[] = {SGI273390, SGI273391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62050[] = {SGI273392, SGI273393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62051[] = {SGI273394, SGI273395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62052[] = {SGI273396, SGI273397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62053[] = {SGI273398, SGI273399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62054[] = {SGI273400, SGI273401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62055[] = {SGI273402, SGI273403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62056[] = {SGI273404, SGI273405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62057[] = {SGI273406, SGI273407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62058[] = {SGI273408, SGI273409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62059[] = {SGI273410, SGI273411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62060[] = {SGI273412, SGI273413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62061[] = {SGI273414, SGI273415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62062[] = {SGI273416, SGI273417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62063[] = {SGI273418, SGI273419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62064[] = {SGI273420, SGI273421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62065[] = {SGI273422, SGI273423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62066[] = {SGI273424, SGI273425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62067[] = {SGI273426, SGI273427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62068[] = {SGI273428, SGI273429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62069[] = {SGI273430, SGI273431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62070[] = {SGI273432, SGI273433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62071[] = {SGI273434, SGI273435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62072[] = {SGI273436, SGI273437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62073[] = {SGI273438, SGI273439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62074[] = {SGI273440, SGI273441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62075[] = {SGI273442, SGI273443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62076[] = {SGI273446, SGI273447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62077[] = {SGI273444, SGI273445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62078[] = {SGI273448, SGI273450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62079[] = {SGI273449, SGI273451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62080[] = {SGI273452, SGI273453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62081[] = {SGI273454, SGI273455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62082[] = {SGI273456, SGI273457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI273458, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62083[] = {SGI273459, SGI273460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62084[] = {SGI273461, SGI273462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62085[] = {SGI273463, SGI273464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62086[] = {SGI273465, SGI273466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62087[] = {SGI273467, SGI273468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62088[] = {SGI273469, SGI273470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62089[] = {SGI273471, SGI273473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62090[] = {SGI273472, SGI273474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62091[] = {SGI273475, SGI273476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62092[] = {SGI273477, SGI273478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62093[] = {SGI273479, SGI273480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62094[] = {SGI273481, SGI273482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62095[] = {SGI273483, SGI273484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62096[] = {SGI273485, SGI273486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62097[] = {SGI273487, SGI273488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62098[] = {SGI273489, SGI273490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62099[] = {SGI273491, SGI273492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62100[] = {SGI273493, SGI273494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62101[] = {SGI273495, SGI273496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62102[] = {SGI273497, SGI273498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62103[] = {SGI273499, SGI273500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62104[] = {SGI273501, SGI273502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62105[] = {SGI273503, SGI273504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62106[] = {SGI273505, SGI273506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62107[] = {SGI273507, SGI273508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62108[] = {SGI273509, SGI273510, NULL};
const Segment * const SG62109[] = {SGI273511, SGI273512, NULL};
const Segment * const SG62110[] = {SGI273513, SGI273514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62111[] = {SGI273515, SGI273516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62112[] = {SGI273517, SGI273518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62113[] = {SGI273519, SGI273520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62114[] = {SGI273521, SGI273522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62115[] = {SGI273523, SGI273524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62116[] = {SGI273525, SGI273526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62117[] = {SGI273527, SGI273528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62118[] = {SGI273529, SGI273530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62119[] = {SGI273531, SGI273532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62120[] = {SGI273533, SGI273534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62121[] = {SGI273535, SGI273536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62122[] = {SGI273537, SGI273538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62123[] = {SGI273539, SGI273540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62124[] = {SGI273541, SGI273542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62125[] = {SGI273543, SGI273544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62126[] = {SGI273545, SGI273546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62127[] = {SGI273547, SGI273548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62128[] = {SGI273549, SGI273550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62129[] = {SGI273551, SGI273552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62130[] = {SGI273553, SGI273554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62131[] = {SGI273555, SGI273556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62132[] = {SGI273557, SGI273558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62133[] = {SGI273559, SGI273560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62134[] = {SGI273561, SGI273562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62135[] = {SGI273563, SGI273564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62136[] = {SGI273565, SGI273566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62137[] = {SGI273567, SGI273568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62138[] = {SGI273569, SGI273570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62139[] = {SGI273571, SGI273572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62140[] = {SGI273573, SGI273574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62141[] = {SGI273575, SGI273576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62142[] = {SGI273577, SGI273578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62143[] = {SGI273579, SGI273580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62144[] = {SGI273581, SGI273582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62145[] = {SGI273583, SGI273584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62146[] = {SGI273585, SGI273586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62147[] = {SGI273587, SGI273588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62148[] = {SGI273589, SGI273590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62149[] = {SGI273591, SGI273592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62150[] = {SGI273593, SGI273594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62151[] = {SGI273595, SGI273596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62152[] = {SGI273597, SGI273598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62153[] = {SGI273599, SGI273600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62154[] = {SGI273601, SGI273602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62155[] = {SGI273603, SGI273604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62156[] = {SGI273605, SGI273606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62157[] = {SGI273607, SGI273608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62158[] = {SGI273609, SGI273610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62159[] = {SGI273611, SGI273612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62160[] = {SGI273613, SGI273614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62161[] = {SGI273615, SGI273616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62162[] = {SGI273617, SGI273618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62163[] = {SGI273619, SGI273620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62164[] = {SGI273621, SGI273622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62165[] = {SGI273623, SGI273624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62166[] = {SGI273625, SGI273626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62167[] = {SGI273627, SGI273628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62168[] = {SGI273629, SGI273630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62169[] = {SGI273631, SGI273632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62170[] = {SGI273633, SGI273634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62171[] = {SGI273635, SGI273636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62172[] = {SGI273637, SGI273638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62173[] = {SGI273639, SGI273640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62174[] = {SGI273641, SGI273642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62175[] = {SGI273643, SGI273644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62176[] = {SGI273645, SGI273646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62177[] = {SGI273647, SGI273648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62178[] = {SGI273649, SGI273650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62179[] = {SGI273651, SGI273652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62180[] = {SGI273653, SGI273654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62181[] = {SGI273655, SGI273656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62182[] = {SGI273657, SGI273658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62183[] = {SGI273659, SGI273660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62184[] = {SGI273661, SGI273662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62185[] = {SGI273663, SGI273664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62186[] = {SGI273665, SGI273666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62187[] = {SGI273667, SGI273668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62188[] = {SGI273669, SGI273670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62189[] = {SGI273671, SGI273672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62190[] = {SGI273673, SGI273674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62191[] = {SGI273675, SGI273676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62192[] = {SGI273677, SGI273678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62193[] = {SGI273679, SGI273680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62194[] = {SGI273681, SGI273682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62195[] = {SGI273683, SGI273684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62196[] = {SGI273685, SGI273686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62197[] = {SGI273687, SGI273688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62198[] = {SGI273689, SGI273690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62199[] = {SGI273691, SGI273692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62200[] = {SGI273693, SGI273694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62201[] = {SGI273695, SGI273696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62202[] = {SGI273697, SGI273698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62203[] = {SGI273699, SGI273700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62204[] = {SGI273701, SGI273702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62205[] = {SGI273703, SGI273704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62206[] = {SGI273705, SGI273706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62207[] = {SGI273707, SGI273708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62208[] = {SGI273709, SGI273710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62209[] = {SGI273711, SGI273712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62210[] = {SGI273713, SGI273714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62211[] = {SGI273715, SGI273716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62212[] = {SGI273717, SGI273718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62213[] = {SGI273719, SGI273720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62214[] = {SGI273721, SGI273722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62215[] = {SGI273723, SGI273724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI273725, SGI0, SGI0, NULL};
const Segment * const SG62216[] = {SGI273726, SGI273727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62217[] = {SGI273728, SGI273729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62218[] = {SGI273730, SGI273731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62219[] = {SGI273732, SGI273733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI273734, SGI0, SGI0, NULL};
const Segment * const SG62220[] = {SGI273735, SGI273736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62221[] = {SGI273737, SGI273738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62222[] = {SGI273739, SGI273740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62223[] = {SGI273741, SGI273742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62224[] = {SGI273743, SGI273744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62225[] = {SGI273745, SGI273746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62226[] = {SGI273747, SGI273748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62227[] = {SGI273749, SGI273750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62228[] = {SGI273751, SGI273752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62229[] = {SGI273753, SGI273754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62230[] = {SGI273755, SGI273756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI273757, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62231[] = {SGI273758, SGI273759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62232[] = {SGI273760, SGI273761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62233[] = {SGI273762, SGI273763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI273764, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62234[] = {SGI273765, SGI273766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62235[] = {SGI273767, SGI273768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62236[] = {SGI273769, SGI273770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62237[] = {SGI273771, SGI273772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62238[] = {SGI273773, SGI273774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62239[] = {SGI273775, SGI273776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62240[] = {SGI273777, SGI273778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62241[] = {SGI273779, SGI273780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62242[] = {SGI273781, SGI273782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62243[] = {SGI273783, SGI273784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62244[] = {SGI273785, SGI273786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62245[] = {SGI273787, SGI273788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62246[] = {SGI273789, SGI273790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62247[] = {SGI273791, SGI273792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62248[] = {SGI273793, SGI273794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62249[] = {SGI273795, SGI273796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62250[] = {SGI273797, SGI273798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62251[] = {SGI273799, SGI273800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62252[] = {SGI273801, SGI273802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62253[] = {SGI273803, SGI273804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62254[] = {SGI273805, SGI273806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62255[] = {SGI273807, SGI273808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62256[] = {SGI273809, SGI273810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62257[] = {SGI273811, SGI273812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62258[] = {SGI273813, SGI273814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62259[] = {SGI273815, SGI273816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62260[] = {SGI273817, SGI273818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62261[] = {SGI273819, SGI273820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62262[] = {SGI273821, SGI273822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62263[] = {SGI273823, SGI273824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62264[] = {SGI273825, SGI273826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62265[] = {SGI273827, SGI273828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62266[] = {SGI273829, SGI273830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62267[] = {SGI273831, SGI273832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62268[] = {SGI273833, SGI273834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62269[] = {SGI273835, SGI273836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62270[] = {SGI273837, SGI273838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62271[] = {SGI273839, SGI273840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62272[] = {SGI273841, SGI273842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62273[] = {SGI273843, SGI273844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62274[] = {SGI273845, SGI273846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62275[] = {SGI273847, SGI273848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62276[] = {SGI273849, SGI273850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62277[] = {SGI273851, SGI273852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62278[] = {SGI273853, SGI273854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62279[] = {SGI273855, SGI273856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62280[] = {SGI273857, SGI273858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62281[] = {SGI273859, SGI273860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62282[] = {SGI273861, SGI273862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62283[] = {SGI273863, SGI273864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62284[] = {SGI273865, SGI273866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62285[] = {SGI273867, SGI273868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62286[] = {SGI273869, SGI273870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62287[] = {SGI273871, SGI273872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62288[] = {SGI273873, SGI273874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62289[] = {SGI273875, SGI273876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62290[] = {SGI273877, SGI273878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62291[] = {SGI273879, SGI273880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62292[] = {SGI273881, SGI273882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62293[] = {SGI273883, SGI273884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62294[] = {SGI273885, SGI273886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62295[] = {SGI273887, SGI273888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62296[] = {SGI273889, SGI273890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62297[] = {SGI273891, SGI273892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62298[] = {SGI273893, SGI273894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62299[] = {SGI273895, SGI273896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62300[] = {SGI273897, SGI273898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62301[] = {SGI273899, SGI273900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62302[] = {SGI273901, SGI273902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62303[] = {SGI273903, SGI273904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62304[] = {SGI273905, SGI273906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62305[] = {SGI273907, SGI273908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62306[] = {SGI273909, SGI273910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI273911, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62307[] = {SGI273912, SGI273913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62308[] = {SGI273914, SGI273915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62309[] = {SGI273916, SGI273917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62310[] = {SGI273918, SGI273919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62311[] = {SGI273920, SGI273921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62312[] = {SGI273922, SGI273923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62313[] = {SGI273924, SGI273925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62314[] = {SGI273926, SGI273927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62315[] = {SGI273928, SGI273929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62316[] = {SGI273930, SGI273931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62317[] = {SGI273932, SGI273933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62318[] = {SGI273934, SGI273935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62319[] = {SGI273936, SGI273937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62320[] = {SGI273938, SGI273939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62321[] = {SGI273940, SGI273941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62322[] = {SGI273942, SGI273943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62323[] = {SGI273944, SGI273945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62324[] = {SGI273946, SGI273947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62325[] = {SGI273950, SGI273951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62326[] = {SGI273948, SGI273949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62327[] = {SGI273952, SGI273954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62328[] = {SGI273953, SGI273955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62329[] = {SGI273956, SGI273957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62330[] = {SGI273958, SGI273959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62331[] = {SGI273960, SGI273961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62332[] = {SGI273962, SGI273963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62333[] = {SGI273964, SGI273965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62334[] = {SGI273966, SGI273967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62335[] = {SGI273968, SGI273969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62336[] = {SGI273970, SGI273971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62337[] = {SGI273972, SGI273973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62338[] = {SGI273975, SGI273977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62339[] = {SGI273974, SGI273976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62340[] = {SGI273978, SGI273979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62341[] = {SGI273980, SGI273981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62342[] = {SGI273982, SGI273983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62343[] = {SGI273984, SGI273985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62344[] = {SGI273986, SGI273987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62345[] = {SGI273988, SGI273989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62346[] = {SGI273991, SGI273993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62347[] = {SGI273990, SGI273992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62348[] = {SGI273994, SGI273995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62349[] = {SGI273996, SGI273997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62350[] = {SGI273998, SGI273999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62351[] = {SGI274000, SGI274001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62352[] = {SGI274002, SGI274003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62353[] = {SGI274004, SGI274005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62354[] = {SGI274006, SGI274007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62355[] = {SGI274008, SGI274009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62356[] = {SGI274010, SGI274011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62357[] = {SGI274012, SGI274013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62358[] = {SGI274014, SGI274015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62359[] = {SGI274016, SGI274017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62360[] = {SGI274018, SGI274019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274020, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62361[] = {SGI274021, SGI274022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62362[] = {SGI274023, SGI274024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62363[] = {SGI274025, SGI274026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62364[] = {SGI274027, SGI274028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62365[] = {SGI274029, SGI274030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62366[] = {SGI274031, SGI274032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62367[] = {SGI274033, SGI274034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62368[] = {SGI274035, SGI274036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62369[] = {SGI274037, SGI274038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62370[] = {SGI274039, SGI274040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62371[] = {SGI274041, SGI274042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274043, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62372[] = {SGI274044, SGI274045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62373[] = {SGI274046, SGI274047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274048, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62374[] = {SGI274049, SGI274050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62375[] = {SGI274051, SGI274052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62376[] = {SGI274053, SGI274054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62377[] = {SGI274055, SGI274056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62378[] = {SGI274057, SGI274058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62379[] = {SGI274059, SGI274060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62380[] = {SGI274061, SGI274062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62381[] = {SGI274063, SGI274064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62382[] = {SGI274065, SGI274066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62383[] = {SGI274067, SGI274068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62384[] = {SGI274069, SGI274070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62385[] = {SGI274071, SGI274072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62386[] = {SGI274073, SGI274074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62387[] = {SGI274075, SGI274076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62388[] = {SGI274077, SGI274078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62389[] = {SGI274079, SGI274080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62390[] = {SGI274081, SGI274082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62391[] = {SGI274083, SGI274084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62392[] = {SGI274085, SGI274086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62393[] = {SGI274087, SGI274088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62394[] = {SGI274089, SGI274090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62395[] = {SGI274091, SGI274092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62396[] = {SGI274093, SGI274094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62397[] = {SGI274095, SGI274096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62398[] = {SGI274097, SGI274098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62399[] = {SGI274099, SGI274100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62400[] = {SGI274101, SGI274102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62401[] = {SGI274103, SGI274104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62402[] = {SGI274106, SGI274107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62403[] = {SGI274105, SGI274108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62404[] = {SGI274109, SGI274110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62405[] = {SGI274111, SGI274112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62406[] = {SGI274113, SGI274114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62407[] = {SGI274115, SGI274116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62408[] = {SGI274117, SGI274118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62409[] = {SGI274119, SGI274120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62410[] = {SGI274121, SGI274122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62411[] = {SGI274123, SGI274124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62412[] = {SGI274125, SGI274126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274127, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62413[] = {SGI274128, SGI274129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62414[] = {SGI274130, SGI274131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62415[] = {SGI274132, SGI274133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62416[] = {SGI274134, SGI274135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62417[] = {SGI274136, SGI274137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62418[] = {SGI274138, SGI274139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62419[] = {SGI274140, SGI274141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62420[] = {SGI274142, SGI274143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62421[] = {SGI274144, SGI274145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62422[] = {SGI274146, SGI274147, NULL};
const Segment * const SG62423[] = {SGI274148, SGI274149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62424[] = {SGI274150, SGI274151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62425[] = {SGI274152, SGI274153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62426[] = {SGI274154, SGI274155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62427[] = {SGI274156, SGI274157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62428[] = {SGI274158, SGI274159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62429[] = {SGI274160, SGI274161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62430[] = {SGI274162, SGI274163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62431[] = {SGI274164, SGI274165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274166, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62432[] = {SGI274167, SGI274168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62433[] = {SGI274169, SGI274170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62434[] = {SGI274171, SGI274172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62435[] = {SGI274173, SGI274174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62436[] = {SGI274175, SGI274176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274177, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62437[] = {SGI274178, SGI274179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274180, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62438[] = {SGI274181, SGI274182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274183, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62439[] = {SGI274184, SGI274185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274186, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62440[] = {SGI274187, SGI274188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274189, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62441[] = {SGI274190, SGI274191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274192, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62442[] = {SGI274193, SGI274194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274195, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62443[] = {SGI274196, SGI274197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274198, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62444[] = {SGI274199, SGI274200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274201, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62445[] = {SGI274202, SGI274203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274204, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62446[] = {SGI274205, SGI274206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274207, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62447[] = {SGI274208, SGI274209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274210, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62448[] = {SGI274211, SGI274212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274213, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62449[] = {SGI274214, SGI274215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274216, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62450[] = {SGI274217, SGI274218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274219, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62451[] = {SGI274220, SGI274221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274222, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62452[] = {SGI274223, SGI274224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274225, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62453[] = {SGI274226, SGI274227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274228, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62454[] = {SGI274229, SGI274230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274231, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62455[] = {SGI274232, SGI274233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274234, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62456[] = {SGI274235, SGI274236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274237, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62457[] = {SGI274238, SGI274239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274240, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62458[] = {SGI274241, SGI274242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274243, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62459[] = {SGI274244, SGI274245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274246, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62460[] = {SGI274247, SGI274248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274249, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62461[] = {SGI274250, SGI274251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274252, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62462[] = {SGI274253, SGI274254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274255, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62463[] = {SGI274256, SGI274257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274258, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62464[] = {SGI274259, SGI274260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274261, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62465[] = {SGI274262, SGI274263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274264, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62466[] = {SGI274265, SGI274266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274267, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62467[] = {SGI274268, SGI274269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274270, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62468[] = {SGI274271, SGI274272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274273, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62469[] = {SGI274274, SGI274275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274276, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62470[] = {SGI274277, SGI274278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274279, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62471[] = {SGI274280, SGI274281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274282, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62472[] = {SGI274283, SGI274284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274285, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62473[] = {SGI274286, SGI274287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274288, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62474[] = {SGI274289, SGI274290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274291, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62475[] = {SGI274292, SGI274293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274294, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62476[] = {SGI274295, SGI274296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274297, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62477[] = {SGI274298, SGI274299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274300, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62478[] = {SGI274301, SGI274302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274303, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62479[] = {SGI274304, SGI274305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274306, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62480[] = {SGI274307, SGI274308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274309, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62481[] = {SGI274310, SGI274311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274312, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62482[] = {SGI274313, SGI274314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274315, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62483[] = {SGI274316, SGI274317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274318, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62484[] = {SGI274319, SGI274320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274321, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62485[] = {SGI274322, SGI274323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274324, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62486[] = {SGI274325, SGI274326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274327, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62487[] = {SGI274328, SGI274329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274330, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62488[] = {SGI274331, SGI274332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274333, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62489[] = {SGI274334, SGI274335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274336, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62490[] = {SGI274337, SGI274338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274339, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62491[] = {SGI274340, SGI274341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274342, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62492[] = {SGI274343, SGI274344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274345, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62493[] = {SGI274346, SGI274347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274348, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62494[] = {SGI274349, SGI274350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274351, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62495[] = {SGI274352, SGI274353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274354, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62496[] = {SGI274355, SGI274356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274357, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62497[] = {SGI274358, SGI274359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274360, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62498[] = {SGI274361, SGI274362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274363, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62499[] = {SGI274364, SGI274365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274366, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62500[] = {SGI274367, SGI274368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274369, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62501[] = {SGI274370, SGI274371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274372, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62502[] = {SGI274373, SGI274374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274375, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62503[] = {SGI274376, SGI274377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274378, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62504[] = {SGI274379, SGI274380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274381, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62505[] = {SGI274382, SGI274383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274384, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62506[] = {SGI274385, SGI274386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274387, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62507[] = {SGI274388, SGI274389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274390, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62508[] = {SGI274391, SGI274392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274393, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62509[] = {SGI274394, SGI274395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274396, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62510[] = {SGI274397, SGI274398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274399, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62511[] = {SGI274400, SGI274401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274402, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62512[] = {SGI274403, SGI274404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274405, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62513[] = {SGI274406, SGI274407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274408, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62514[] = {SGI274409, SGI274410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274411, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62515[] = {SGI274412, SGI274413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274414, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62516[] = {SGI274415, SGI274416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274417, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62517[] = {SGI274418, SGI274419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274420, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62518[] = {SGI274421, SGI274422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274423, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62519[] = {SGI274424, SGI274425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274426, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62520[] = {SGI274427, SGI274428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274429, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62521[] = {SGI274430, SGI274431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274432, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62522[] = {SGI274433, SGI274434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274435, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62523[] = {SGI274436, SGI274437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274438, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62524[] = {SGI274439, SGI274440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274441, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62525[] = {SGI274442, SGI274443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274444, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62526[] = {SGI274445, SGI274446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274447, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62527[] = {SGI274448, SGI274449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274450, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62528[] = {SGI274451, SGI274452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274453, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62529[] = {SGI274454, SGI274455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274456, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62530[] = {SGI274457, SGI274458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274459, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62531[] = {SGI274460, SGI274461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274462, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62532[] = {SGI274463, SGI274464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274465, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62533[] = {SGI274466, SGI274467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274468, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62534[] = {SGI274469, SGI274470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274471, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62535[] = {SGI274472, SGI274473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274474, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62536[] = {SGI274475, SGI274476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274477, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62537[] = {SGI274478, SGI274479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274480, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62538[] = {SGI274481, SGI274482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274483, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62539[] = {SGI274484, SGI274485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274486, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62540[] = {SGI274487, SGI274488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274489, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62541[] = {SGI274490, SGI274491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274492, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62542[] = {SGI274493, SGI274494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274495, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62543[] = {SGI274496, SGI274497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274498, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62544[] = {SGI274499, SGI274500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274501, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62545[] = {SGI274502, SGI274503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274504, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62546[] = {SGI274505, SGI274506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274507, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62547[] = {SGI274508, SGI274509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274510, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62548[] = {SGI274511, SGI274512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274513, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62549[] = {SGI274514, SGI274515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274516, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62550[] = {SGI274517, SGI274518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274519, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62551[] = {SGI274520, SGI274521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274522, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62552[] = {SGI274523, SGI274524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274525, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62553[] = {SGI274526, SGI274527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274528, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62554[] = {SGI274529, SGI274530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274531, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62555[] = {SGI274532, SGI274533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274534, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62556[] = {SGI274535, SGI274536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274537, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62557[] = {SGI274538, SGI274539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274540, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62558[] = {SGI274541, SGI274542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274543, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62559[] = {SGI274544, SGI274545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274546, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62560[] = {SGI274547, SGI274548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274549, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62561[] = {SGI274550, SGI274551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274552, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62562[] = {SGI274553, SGI274554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274555, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62563[] = {SGI274556, SGI274557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274558, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62564[] = {SGI274559, SGI274560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274561, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62565[] = {SGI274562, SGI274563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274564, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62566[] = {SGI274565, SGI274566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274567, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62567[] = {SGI274568, SGI274569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274570, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62568[] = {SGI274571, SGI274572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274573, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62569[] = {SGI274574, SGI274575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274576, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62570[] = {SGI274577, SGI274578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274579, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62571[] = {SGI274580, SGI274581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274582, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62572[] = {SGI274583, SGI274584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274585, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62573[] = {SGI274586, SGI274587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274588, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62574[] = {SGI274589, SGI274590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274591, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62575[] = {SGI274592, SGI274593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274594, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62576[] = {SGI274595, SGI274596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274597, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62577[] = {SGI274598, SGI274599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274600, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62578[] = {SGI274601, SGI274602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274603, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62579[] = {SGI274604, SGI274605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274606, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62580[] = {SGI274607, SGI274608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274609, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62581[] = {SGI274610, SGI274611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274612, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62582[] = {SGI274613, SGI274614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274615, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62583[] = {SGI274616, SGI274617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274618, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62584[] = {SGI274619, SGI274620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274621, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62585[] = {SGI274622, SGI274623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274624, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62586[] = {SGI274625, SGI274626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274627, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62587[] = {SGI274628, SGI274629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274630, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62588[] = {SGI274631, SGI274632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274633, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62589[] = {SGI274634, SGI274635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274636, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62590[] = {SGI274637, SGI274638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274639, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62591[] = {SGI274640, SGI274641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274642, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62592[] = {SGI274643, SGI274644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274645, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62593[] = {SGI274646, SGI274647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274648, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62594[] = {SGI274649, SGI274650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274651, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62595[] = {SGI274652, SGI274653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274654, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62596[] = {SGI274655, SGI274656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274657, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62597[] = {SGI274658, SGI274659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274660, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62598[] = {SGI274661, SGI274662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274663, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62599[] = {SGI274664, SGI274665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274666, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62600[] = {SGI274667, SGI274668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274669, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62601[] = {SGI274670, SGI274671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274672, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62602[] = {SGI274673, SGI274674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274675, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62603[] = {SGI274676, SGI274677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274678, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62604[] = {SGI274679, SGI274680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274681, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62605[] = {SGI274682, SGI274683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274684, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62606[] = {SGI274685, SGI274686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274687, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62607[] = {SGI274688, SGI274689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274690, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62608[] = {SGI274691, SGI274692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274693, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62609[] = {SGI274694, SGI274695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274696, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62610[] = {SGI274697, SGI274698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274699, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62611[] = {SGI274700, SGI274701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274702, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62612[] = {SGI274703, SGI274704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274705, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62613[] = {SGI274706, SGI274707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274708, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62614[] = {SGI274709, SGI274710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274711, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62615[] = {SGI274712, SGI274713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274714, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62616[] = {SGI274715, SGI274716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274717, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62617[] = {SGI274718, SGI274719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274720, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62618[] = {SGI274721, SGI274722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274723, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62619[] = {SGI274724, SGI274725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274726, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62620[] = {SGI274727, SGI274728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274729, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62621[] = {SGI274730, SGI274731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274732, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62622[] = {SGI274733, SGI274734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274735, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62623[] = {SGI274736, SGI274737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274738, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62624[] = {SGI274739, SGI274740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274741, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62625[] = {SGI274742, SGI274743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274744, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62626[] = {SGI274745, SGI274746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274747, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62627[] = {SGI274748, SGI274749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274750, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62628[] = {SGI274751, SGI274752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274753, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62629[] = {SGI274754, SGI274755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274756, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62630[] = {SGI274757, SGI274758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274759, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62631[] = {SGI274760, SGI274761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274762, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62632[] = {SGI274763, SGI274764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274765, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62633[] = {SGI274766, SGI274767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274768, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62634[] = {SGI274769, SGI274770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274771, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62635[] = {SGI274772, SGI274773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274774, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62636[] = {SGI274775, SGI274776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274777, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62637[] = {SGI274778, SGI274779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274780, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62638[] = {SGI274781, SGI274782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274783, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62639[] = {SGI274784, SGI274785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274786, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62640[] = {SGI274787, SGI274788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274789, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62641[] = {SGI274790, SGI274791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274792, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62642[] = {SGI274793, SGI274794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274795, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62643[] = {SGI274796, SGI274797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274798, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62644[] = {SGI274799, SGI274800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274801, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62645[] = {SGI274802, SGI274803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274804, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62646[] = {SGI274805, SGI274806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274807, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62647[] = {SGI274808, SGI274809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274810, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62648[] = {SGI274811, SGI274812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274813, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62649[] = {SGI274814, SGI274815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274816, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62650[] = {SGI274817, SGI274818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274819, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62651[] = {SGI274820, SGI274821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274822, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62652[] = {SGI274823, SGI274824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274825, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62653[] = {SGI274826, SGI274827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274828, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62654[] = {SGI274829, SGI274830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274831, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62655[] = {SGI274832, SGI274833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274834, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62656[] = {SGI274835, SGI274836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274837, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62657[] = {SGI274838, SGI274839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274840, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62658[] = {SGI274841, SGI274842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274843, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62659[] = {SGI274844, SGI274845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274846, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62660[] = {SGI274847, SGI274848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274849, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62661[] = {SGI274850, SGI274851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274852, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62662[] = {SGI274853, SGI274854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274855, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62663[] = {SGI274856, SGI274857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274858, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62664[] = {SGI274859, SGI274860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274861, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62665[] = {SGI274862, SGI274863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274864, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62666[] = {SGI274865, SGI274866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274867, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62667[] = {SGI274868, SGI274869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274870, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62668[] = {SGI274871, SGI274872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274873, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62669[] = {SGI274874, SGI274875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274876, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62670[] = {SGI274877, SGI274878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274879, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62671[] = {SGI274880, SGI274881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI274882, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62672[] = {SGI274883, SGI274884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62673[] = {SGI274885, SGI274886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62674[] = {SGI274887, SGI274888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62675[] = {SGI274889, SGI274890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62676[] = {SGI274891, SGI274892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62677[] = {SGI274893, SGI274894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62678[] = {SGI274895, SGI274896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62679[] = {SGI274897, SGI274898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62680[] = {SGI274899, SGI274900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62681[] = {SGI274901, SGI274902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62682[] = {SGI274903, SGI274904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62683[] = {SGI274905, SGI274906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62684[] = {SGI274907, SGI274908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62685[] = {SGI274909, SGI274910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62686[] = {SGI274911, SGI274912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62687[] = {SGI274913, SGI274914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62688[] = {SGI274915, SGI274916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62689[] = {SGI274917, SGI274918, NULL};
const Segment * const SG62690[] = {SGI274919, SGI274920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62691[] = {SGI274921, SGI274922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62692[] = {SGI274923, SGI274924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62693[] = {SGI274925, SGI274926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62694[] = {SGI274927, SGI274928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62695[] = {SGI274929, SGI274930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62696[] = {SGI274931, SGI274932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62697[] = {SGI274933, SGI274934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62698[] = {SGI274935, SGI274936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62699[] = {SGI274937, SGI274938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62700[] = {SGI274939, SGI274940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62701[] = {SGI274941, SGI274942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62702[] = {SGI274943, SGI274944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62703[] = {SGI274945, SGI274946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62704[] = {SGI274947, SGI274948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62705[] = {SGI274949, SGI274950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62706[] = {SGI274951, SGI274952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62707[] = {SGI274953, SGI274954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62708[] = {SGI274955, SGI274956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62709[] = {SGI274957, SGI274958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62710[] = {SGI274959, SGI274960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62711[] = {SGI274961, SGI274962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62712[] = {SGI274963, SGI274964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62713[] = {SGI274965, SGI274966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62714[] = {SGI274967, SGI274968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62715[] = {SGI274969, SGI274970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62716[] = {SGI274971, SGI274973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62717[] = {SGI274972, SGI274974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62718[] = {SGI274975, SGI274977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62719[] = {SGI274976, SGI274978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62720[] = {SGI274979, SGI274980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62721[] = {SGI274981, SGI274982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62722[] = {SGI274983, SGI274984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62723[] = {SGI274985, SGI274986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62724[] = {SGI274987, SGI274988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62725[] = {SGI274989, SGI274990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62726[] = {SGI274991, SGI274992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62727[] = {SGI273631, SGI273632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62728[] = {SGI274993, SGI274994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62729[] = {SGI274995, SGI274996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62730[] = {SGI274997, SGI274998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62731[] = {SGI274999, SGI275000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62732[] = {SGI275001, SGI275002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62733[] = {SGI275003, SGI275004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62734[] = {SGI275005, SGI275006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62735[] = {SGI275007, SGI275008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62736[] = {SGI275009, SGI275010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62737[] = {SGI275011, SGI275012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62738[] = {SGI275013, SGI275014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62739[] = {SGI275015, SGI275016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62740[] = {SGI275017, SGI275018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62741[] = {SGI275019, SGI275020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62742[] = {SGI275021, SGI275022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62743[] = {SGI275023, SGI275024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62744[] = {SGI275025, SGI275026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62745[] = {SGI275027, SGI275028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62746[] = {SGI275029, SGI275030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62747[] = {SGI275031, SGI275032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62748[] = {SGI275033, SGI275034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62749[] = {SGI275035, SGI275036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62750[] = {SGI275037, SGI275038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62751[] = {SGI275039, SGI275040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62752[] = {SGI275041, SGI275042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62753[] = {SGI275043, SGI275044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62754[] = {SGI275045, SGI275046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62755[] = {SGI275047, SGI275048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62756[] = {SGI275049, SGI275050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62757[] = {SGI275051, SGI275052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62758[] = {SGI275053, SGI275054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62759[] = {SGI275055, SGI275056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62760[] = {SGI275057, SGI275058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62761[] = {SGI275059, SGI275060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62762[] = {SGI275061, SGI275062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62763[] = {SGI275063, SGI275064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62764[] = {SGI275065, SGI275066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62765[] = {SGI275067, SGI275068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62766[] = {SGI275069, SGI275070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62767[] = {SGI275071, SGI275072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62768[] = {SGI275073, SGI275074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62769[] = {SGI275075, SGI275076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62770[] = {SGI275077, SGI275078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62771[] = {SGI275079, SGI275080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62772[] = {SGI275081, SGI275082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62773[] = {SGI275083, SGI275084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62774[] = {SGI275085, SGI275086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62775[] = {SGI275087, SGI275088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62776[] = {SGI275089, SGI275090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62777[] = {SGI275091, SGI275092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62778[] = {SGI275093, SGI275094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62779[] = {SGI275095, SGI275096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62780[] = {SGI275097, SGI275098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62781[] = {SGI275099, SGI275100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62782[] = {SGI275101, SGI275102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62783[] = {SGI275103, SGI275104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62784[] = {SGI275105, SGI275106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62785[] = {SGI275107, SGI275108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62786[] = {SGI275109, SGI275110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62787[] = {SGI275111, SGI275112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62788[] = {SGI275113, SGI275114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62789[] = {SGI275115, SGI275116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62790[] = {SGI275117, SGI275118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62791[] = {SGI275119, SGI275120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62792[] = {SGI275121, SGI275122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62793[] = {SGI275123, SGI275124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62794[] = {SGI275127, SGI275128, NULL};
const Segment * const SG62795[] = {SGI275125, SGI275126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62796[] = {SGI275129, SGI275130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62797[] = {SGI275131, SGI275132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62798[] = {SGI275133, SGI275134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62799[] = {SGI275135, SGI275136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62800[] = {SGI275138, SGI275139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62801[] = {SGI275137, SGI275140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62802[] = {SGI275141, SGI275142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62803[] = {SGI275143, SGI275144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62804[] = {SGI275145, SGI275146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62805[] = {SGI275147, SGI275148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62806[] = {SGI275149, SGI275150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62807[] = {SGI275153, SGI275154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62808[] = {SGI275151, SGI275152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62809[] = {SGI275155, SGI275156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62810[] = {SGI275157, SGI275158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62811[] = {SGI275159, SGI275160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62812[] = {SGI275161, SGI275162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62813[] = {SGI275163, SGI275164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62814[] = {SGI275165, SGI275166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62815[] = {SGI275167, SGI275168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62816[] = {SGI275170, SGI275171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62817[] = {SGI275172, SGI275173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62818[] = {SGI275169, SGI275174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62819[] = {SGI275175, SGI275176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62820[] = {SGI275177, SGI275178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62821[] = {SGI275179, SGI275180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62822[] = {SGI275181, SGI275182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62823[] = {SGI275183, SGI275184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62824[] = {SGI275185, SGI275186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62825[] = {SGI275187, SGI275188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62826[] = {SGI275189, SGI275190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62827[] = {SGI275191, SGI275192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62828[] = {SGI275193, SGI275194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62829[] = {SGI275195, SGI275196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62830[] = {SGI275197, SGI275198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62831[] = {SGI275199, SGI275200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62832[] = {SGI275201, SGI275202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62833[] = {SGI275203, SGI275204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62834[] = {SGI275205, SGI275206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62835[] = {SGI275207, SGI275208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62836[] = {SGI275209, SGI275210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62837[] = {SGI275211, SGI275212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62838[] = {SGI275213, SGI275214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62839[] = {SGI275215, SGI275216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62840[] = {SGI275217, SGI275218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62841[] = {SGI275219, SGI275220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62842[] = {SGI275221, SGI275222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62843[] = {SGI275223, SGI275224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62844[] = {SGI275225, SGI275226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62845[] = {SGI275227, SGI275228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62846[] = {SGI275229, SGI275230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62847[] = {SGI275231, SGI275232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62848[] = {SGI275233, SGI275234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62849[] = {SGI275235, SGI275236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62850[] = {SGI275237, SGI275238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62851[] = {SGI275239, SGI275240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62852[] = {SGI275241, SGI275242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62853[] = {SGI275243, SGI275244, NULL};
const Segment * const SG62854[] = {SGI275245, SGI275246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275247, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62855[] = {SGI275248, SGI275249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275250, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62856[] = {SGI275251, SGI275252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275253, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62857[] = {SGI275254, SGI275255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275256, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62858[] = {SGI275257, SGI275258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275259, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62859[] = {SGI275260, SGI275261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275262, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62860[] = {SGI275263, SGI275264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275265, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62861[] = {SGI275266, SGI275267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62862[] = {SGI275268, SGI275269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62863[] = {SGI275270, SGI275271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62864[] = {SGI275272, SGI275273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62865[] = {SGI275274, SGI275275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62866[] = {SGI275276, SGI275277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62867[] = {SGI275278, SGI275279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62868[] = {SGI275280, SGI275281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62869[] = {SGI275282, SGI275283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62870[] = {SGI275284, SGI275285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62871[] = {SGI275286, SGI275287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62872[] = {SGI275288, SGI275289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62873[] = {SGI275290, SGI275291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62874[] = {SGI275292, SGI275293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62875[] = {SGI275294, SGI275295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62876[] = {SGI275296, SGI275297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62877[] = {SGI275298, SGI275299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62878[] = {SGI275300, SGI275301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62879[] = {SGI275302, SGI275303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62880[] = {SGI275304, SGI275305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62881[] = {SGI275306, SGI275307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62882[] = {SGI275308, SGI275309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62883[] = {SGI275310, SGI275311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62884[] = {SGI275312, SGI275313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62885[] = {SGI275314, SGI275315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62886[] = {SGI275316, SGI275317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62887[] = {SGI275318, SGI275319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62888[] = {SGI275320, SGI275321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62889[] = {SGI275322, SGI275323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62890[] = {SGI275324, SGI275325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62891[] = {SGI275326, SGI275327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62892[] = {SGI275328, SGI275329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62893[] = {SGI275330, SGI275331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62894[] = {SGI275332, SGI275333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62895[] = {SGI275334, SGI275335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62896[] = {SGI275336, SGI275337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62897[] = {SGI275338, SGI275339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62898[] = {SGI275340, SGI275341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62899[] = {SGI275342, SGI275343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62900[] = {SGI275344, SGI275345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62901[] = {SGI275346, SGI275347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62902[] = {SGI275348, SGI275349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62903[] = {SGI275350, SGI275351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62904[] = {SGI275352, SGI275353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62905[] = {SGI275354, SGI275355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62906[] = {SGI275356, SGI275357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62907[] = {SGI275358, SGI275359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62908[] = {SGI275360, SGI275361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62909[] = {SGI275362, SGI275363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62910[] = {SGI275364, SGI275365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62911[] = {SGI275366, SGI275367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62912[] = {SGI275368, SGI275369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62913[] = {SGI275370, SGI275371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62914[] = {SGI275372, SGI275373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62915[] = {SGI275374, SGI275375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62916[] = {SGI275376, SGI275377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62917[] = {SGI275378, SGI275379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62918[] = {SGI275380, SGI275381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62919[] = {SGI275382, SGI275383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62920[] = {SGI275384, SGI275385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62921[] = {SGI275386, SGI275387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62922[] = {SGI275388, SGI275389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62923[] = {SGI275390, SGI275391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62924[] = {SGI275392, SGI275393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62925[] = {SGI275394, SGI275395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62926[] = {SGI275396, SGI275397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62927[] = {SGI275398, SGI275399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62928[] = {SGI275400, SGI275401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62929[] = {SGI275402, SGI275403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62930[] = {SGI275404, SGI275405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62931[] = {SGI275406, SGI275407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62932[] = {SGI275408, SGI275409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62933[] = {SGI275410, SGI275411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62934[] = {SGI275412, SGI275413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62935[] = {SGI275414, SGI275415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62936[] = {SGI275416, SGI275417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62937[] = {SGI275418, SGI275419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62938[] = {SGI275420, SGI275421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62939[] = {SGI275422, SGI275423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62940[] = {SGI275424, SGI275425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62941[] = {SGI275426, SGI275427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62942[] = {SGI275428, SGI275429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62943[] = {SGI275430, SGI275431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62944[] = {SGI275432, SGI275433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62945[] = {SGI275434, SGI275435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62946[] = {SGI275436, SGI275437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62947[] = {SGI275438, SGI275439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62948[] = {SGI275440, SGI275441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62949[] = {SGI275442, SGI275443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62950[] = {SGI275444, SGI275445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62951[] = {SGI275446, SGI275447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62952[] = {SGI275448, SGI275449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62953[] = {SGI275450, SGI275451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62954[] = {SGI275452, SGI275453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62955[] = {SGI275454, SGI275455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62956[] = {SGI275456, SGI275457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62957[] = {SGI275458, SGI275459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62958[] = {SGI275460, SGI275461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62959[] = {SGI275462, SGI275463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62960[] = {SGI275464, SGI275465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62961[] = {SGI275466, SGI275467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62962[] = {SGI275468, SGI275469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62963[] = {SGI275470, SGI275471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62964[] = {SGI275472, SGI275473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62965[] = {SGI275474, SGI275475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62966[] = {SGI275476, SGI275477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62967[] = {SGI275478, SGI275479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62968[] = {SGI275480, SGI275481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62969[] = {SGI275482, SGI275483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62970[] = {SGI275485, SGI275486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62971[] = {SGI275487, SGI275488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62972[] = {SGI275489, SGI275490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62973[] = {SGI275491, SGI275492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62974[] = {SGI275484, SGI275493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62975[] = {SGI275494, SGI275495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62976[] = {SGI275498, SGI275499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62977[] = {SGI275500, SGI275501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62978[] = {SGI275496, SGI275497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62979[] = {SGI275502, SGI275503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62980[] = {SGI275504, SGI275505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62981[] = {SGI275506, SGI275507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62982[] = {SGI275508, SGI275509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62983[] = {SGI275510, SGI275511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62984[] = {SGI275512, SGI275513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62985[] = {SGI275514, SGI275515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62986[] = {SGI275516, SGI275517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62987[] = {SGI275518, SGI275519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62988[] = {SGI275520, SGI275521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62989[] = {SGI275522, SGI275523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62990[] = {SGI275524, SGI275525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62991[] = {SGI275526, SGI275527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62992[] = {SGI275528, SGI275529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62993[] = {SGI275530, SGI275531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62994[] = {SGI275532, SGI275533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62995[] = {SGI275534, SGI275535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62996[] = {SGI275536, SGI275537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62997[] = {SGI275538, SGI275539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62998[] = {SGI275540, SGI275541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG62999[] = {SGI275542, SGI275543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63000[] = {SGI275544, SGI275545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63001[] = {SGI275546, SGI275547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63002[] = {SGI275548, SGI275549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63003[] = {SGI275550, SGI275551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63004[] = {SGI275552, SGI275553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63005[] = {SGI275554, SGI275555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275556, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63006[] = {SGI275557, SGI275558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63007[] = {SGI275559, SGI275560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63008[] = {SGI275561, SGI275562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275563, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63009[] = {SGI275564, SGI275565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275566, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63010[] = {SGI275567, SGI275568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63011[] = {SGI275569, SGI275570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275571, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63012[] = {SGI275572, SGI275573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275574, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63013[] = {SGI275575, SGI275576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63014[] = {SGI275577, SGI275578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63015[] = {SGI275579, SGI275580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63016[] = {SGI275581, SGI275582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63017[] = {SGI275583, SGI275584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63018[] = {SGI275585, SGI275586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275587, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63019[] = {SGI275588, SGI275589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275590, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63020[] = {SGI275591, SGI275592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275593, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63021[] = {SGI275594, SGI275595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275596, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63022[] = {SGI275597, SGI275598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63023[] = {SGI275599, SGI275600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63024[] = {SGI275601, SGI275602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63025[] = {SGI275603, SGI275604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63026[] = {SGI275605, SGI275606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63027[] = {SGI275607, SGI275608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63028[] = {SGI275609, SGI275610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63029[] = {SGI275611, SGI275612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275613, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63030[] = {SGI275614, SGI275615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63031[] = {SGI275616, SGI275617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275618, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63032[] = {SGI275619, SGI275620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63033[] = {SGI275621, SGI275622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275623, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63034[] = {SGI275624, SGI275625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63035[] = {SGI275626, SGI275627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63036[] = {SGI275628, SGI275629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63037[] = {SGI275630, SGI275631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63038[] = {SGI275632, SGI275633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63039[] = {SGI275634, SGI275635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63040[] = {SGI275636, SGI275637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275638, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63041[] = {SGI275639, SGI275640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275641, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63042[] = {SGI275642, SGI275643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63043[] = {SGI275644, SGI275645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275646, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63044[] = {SGI275647, SGI275648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275649, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63045[] = {SGI275650, SGI275651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275652, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63046[] = {SGI275653, SGI275654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275655, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63047[] = {SGI275656, SGI275657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275658, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63048[] = {SGI275659, SGI275660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63049[] = {SGI275661, SGI275662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275663, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63050[] = {SGI275664, SGI275665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275666, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63051[] = {SGI275667, SGI275668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63052[] = {SGI275669, SGI275670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275671, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63053[] = {SGI275672, SGI275673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275674, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63054[] = {SGI275675, SGI275676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63055[] = {SGI275677, SGI275678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63056[] = {SGI275679, SGI275680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63057[] = {SGI275681, SGI275682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63058[] = {SGI275683, SGI275684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63059[] = {SGI275685, SGI275686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63060[] = {SGI275687, SGI275688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63061[] = {SGI0, SGI275689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63062[] = {SGI275690, SGI275691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63063[] = {SGI275692, SGI275693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63064[] = {SGI275694, SGI275695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63065[] = {SGI275696, SGI275697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63066[] = {SGI275698, SGI275699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63067[] = {SGI275700, SGI275701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63068[] = {SGI275702, SGI275703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63069[] = {SGI275704, SGI275705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63070[] = {SGI275706, SGI275707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63071[] = {SGI275708, SGI275709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63072[] = {SGI275710, SGI275711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63073[] = {SGI275712, SGI275713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63074[] = {SGI275714, SGI275715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63075[] = {SGI275716, SGI275717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63076[] = {SGI275718, SGI275719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63077[] = {SGI275720, SGI275721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63078[] = {SGI275722, SGI275723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63079[] = {SGI275724, SGI275725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63080[] = {SGI275726, SGI275727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63081[] = {SGI275728, SGI275729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63082[] = {SGI275731, SGI275732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63083[] = {SGI275733, SGI275734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63084[] = {SGI275735, SGI275736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63085[] = {SGI275737, SGI275738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63086[] = {SGI275739, SGI275740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63087[] = {SGI275741, SGI275742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63088[] = {SGI275743, SGI275744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63089[] = {SGI275745, SGI275746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63090[] = {SGI275747, SGI275748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63091[] = {SGI275749, SGI275750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63092[] = {SGI275751, SGI275752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63093[] = {SGI275753, SGI275754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63094[] = {SGI275755, SGI275756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63095[] = {SGI275757, SGI275758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63096[] = {SGI275759, SGI275760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63097[] = {SGI275761, SGI275762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63098[] = {SGI275763, SGI275764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63099[] = {SGI275765, SGI275766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63100[] = {SGI275767, SGI275768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63101[] = {SGI275769, SGI275770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63102[] = {SGI275730, SGI275771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63103[] = {SGI275772, SGI275773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63104[] = {SGI275774, SGI275775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63105[] = {SGI275776, SGI275777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63106[] = {SGI275778, SGI275779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63107[] = {SGI275780, SGI275781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63108[] = {SGI275782, SGI275783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63109[] = {SGI275784, SGI275785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63110[] = {SGI275786, SGI275787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63111[] = {SGI275788, SGI275789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63112[] = {SGI275790, SGI275791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63113[] = {SGI275792, SGI275793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63114[] = {SGI275794, SGI275795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63115[] = {SGI275796, SGI275797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63116[] = {SGI275798, SGI275799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63117[] = {SGI275800, SGI275801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63118[] = {SGI275802, SGI275803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63119[] = {SGI275804, SGI275805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63120[] = {SGI275806, SGI275807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63121[] = {SGI275808, SGI275809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63122[] = {SGI275810, SGI275811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63123[] = {SGI275812, SGI275813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63124[] = {SGI275814, SGI275815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63125[] = {SGI275816, SGI275817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63126[] = {SGI275818, SGI275819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63127[] = {SGI275820, SGI275821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63128[] = {SGI275822, SGI275823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63129[] = {SGI275824, SGI275825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63130[] = {SGI275826, SGI275827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63131[] = {SGI275828, SGI275829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63132[] = {SGI275830, SGI275831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63133[] = {SGI275832, SGI275833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63134[] = {SGI275834, SGI275835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63135[] = {SGI275836, SGI275837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63136[] = {SGI275838, SGI275839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63137[] = {SGI275840, SGI275841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63138[] = {SGI275842, SGI275843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63139[] = {SGI275844, SGI275845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63140[] = {SGI275846, SGI275847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63141[] = {SGI275848, SGI275849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63142[] = {SGI275850, SGI275851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63143[] = {SGI275852, SGI275853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63144[] = {SGI275854, SGI275855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63145[] = {SGI275856, SGI275857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63146[] = {SGI275858, SGI275859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63147[] = {SGI275860, SGI275861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63148[] = {SGI275862, SGI275863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63149[] = {SGI275864, SGI275865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63150[] = {SGI275866, SGI275867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63151[] = {SGI275868, SGI275869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63152[] = {SGI275870, SGI275871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63153[] = {SGI275872, SGI275873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63154[] = {SGI275874, SGI275875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63155[] = {SGI275876, SGI275877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63156[] = {SGI275878, SGI275879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63157[] = {SGI275880, SGI275881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63158[] = {SGI275882, SGI275883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63159[] = {SGI275884, SGI275885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63160[] = {SGI275886, SGI275887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63161[] = {SGI275888, SGI275889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63162[] = {SGI275890, SGI275891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63163[] = {SGI275892, SGI275893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63164[] = {SGI275894, SGI275895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63165[] = {SGI275896, SGI275897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63166[] = {SGI275898, SGI275899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63167[] = {SGI275900, SGI275901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63168[] = {SGI275902, SGI275903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63169[] = {SGI275904, SGI275905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63170[] = {SGI275906, SGI275907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63171[] = {SGI275908, SGI275909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63172[] = {SGI275910, SGI275911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63173[] = {SGI275912, SGI275913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63174[] = {SGI275914, SGI275915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63175[] = {SGI275916, SGI275917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63176[] = {SGI275918, SGI275919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63177[] = {SGI275920, SGI275921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63178[] = {SGI275922, SGI275923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63179[] = {SGI275924, SGI275925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63180[] = {SGI275926, SGI275927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63181[] = {SGI275928, SGI275929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63182[] = {SGI275930, SGI275931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63183[] = {SGI275932, SGI275933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63184[] = {SGI275934, SGI275935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63185[] = {SGI275936, SGI275937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63186[] = {SGI275938, SGI275939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63187[] = {SGI275940, SGI275941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63188[] = {SGI275942, SGI275943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63189[] = {SGI275944, SGI275945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63190[] = {SGI275946, SGI275947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63191[] = {SGI275948, SGI275949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63192[] = {SGI275950, SGI275951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63193[] = {SGI275952, SGI275953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63194[] = {SGI275954, SGI275955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63195[] = {SGI275956, SGI275957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63196[] = {SGI275958, SGI275959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63197[] = {SGI275960, SGI275961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63198[] = {SGI275962, SGI275963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275964, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63199[] = {SGI275965, SGI275966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63200[] = {SGI275967, SGI275968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI275969, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63201[] = {SGI275970, SGI275971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63202[] = {SGI275972, SGI275973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63203[] = {SGI275974, SGI275975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63204[] = {SGI275976, SGI275977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63205[] = {SGI275978, SGI275979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63206[] = {SGI275980, SGI275981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63207[] = {SGI275982, SGI275983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63208[] = {SGI275984, SGI275985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63209[] = {SGI275986, SGI275987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63210[] = {SGI275988, SGI275989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63211[] = {SGI275990, SGI275991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63212[] = {SGI275992, SGI275993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63213[] = {SGI275994, SGI275995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63214[] = {SGI275996, SGI275997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63215[] = {SGI275998, SGI275999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63216[] = {SGI276000, SGI276001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63217[] = {SGI276002, SGI276003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63218[] = {SGI276004, SGI276005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63219[] = {SGI276006, SGI276007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63220[] = {SGI276008, SGI276009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63221[] = {SGI276010, SGI276011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63222[] = {SGI276012, SGI276013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63223[] = {SGI276014, SGI276015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63224[] = {SGI276016, SGI276017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63225[] = {SGI276018, SGI276019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63226[] = {SGI276020, SGI276021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63227[] = {SGI276022, SGI276023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63228[] = {SGI276024, SGI276025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63229[] = {SGI276026, SGI276027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63230[] = {SGI276030, SGI276031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63231[] = {SGI276028, SGI276029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63232[] = {SGI276032, SGI276033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63233[] = {SGI276036, SGI276037, NULL};
const Segment * const SG63234[] = {SGI276034, SGI276035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63235[] = {SGI276038, SGI276039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63236[] = {SGI276040, SGI276041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63237[] = {SGI276042, SGI276043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63238[] = {SGI276044, SGI276045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63239[] = {SGI276046, SGI276047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63240[] = {SGI276048, SGI276049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63241[] = {SGI276050, SGI276051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63242[] = {SGI276052, SGI276053, NULL};
const Segment * const SG63243[] = {SGI276054, SGI276055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63244[] = {SGI276056, SGI276057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63245[] = {SGI276058, SGI276059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63246[] = {SGI276060, SGI276061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63247[] = {SGI276062, SGI276063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63248[] = {SGI276064, SGI276065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63249[] = {SGI276066, SGI276067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63250[] = {SGI276068, SGI276069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63251[] = {SGI276070, SGI276071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63252[] = {SGI276072, SGI276073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63253[] = {SGI276074, SGI276075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63254[] = {SGI276076, SGI276077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63255[] = {SGI276078, SGI276079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63256[] = {SGI276080, SGI276081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63257[] = {SGI276082, SGI276083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63258[] = {SGI276084, SGI276085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63259[] = {SGI276086, SGI276087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63260[] = {SGI276088, SGI276089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63261[] = {SGI276090, SGI276091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63262[] = {SGI276092, SGI276093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63263[] = {SGI276094, SGI276095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63264[] = {SGI276096, SGI276097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63265[] = {SGI276098, SGI276099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63266[] = {SGI276100, SGI276101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63267[] = {SGI276102, SGI276103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63268[] = {SGI276104, SGI276105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63269[] = {SGI276106, SGI276107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63270[] = {SGI276108, SGI276109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63271[] = {SGI276110, SGI276111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63272[] = {SGI276112, SGI276113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63273[] = {SGI276114, SGI276115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63274[] = {SGI276116, SGI276117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63275[] = {SGI276118, SGI276119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63276[] = {SGI276120, SGI276121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63277[] = {SGI276122, SGI276123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63278[] = {SGI276124, SGI276125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63279[] = {SGI276126, SGI276127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63280[] = {SGI276128, SGI276129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63281[] = {SGI276130, SGI276131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63282[] = {SGI276132, SGI276133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63283[] = {SGI276134, SGI276135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63284[] = {SGI276136, SGI276137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63285[] = {SGI276138, SGI276139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63286[] = {SGI276140, SGI276141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63287[] = {SGI276142, SGI276143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63288[] = {SGI276144, SGI276145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63289[] = {SGI276146, SGI276147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63290[] = {SGI276148, SGI276149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63291[] = {SGI276150, SGI276151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63292[] = {SGI276152, SGI276153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63293[] = {SGI276154, SGI276155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63294[] = {SGI276156, SGI276157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63295[] = {SGI276158, SGI276159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63296[] = {SGI276160, SGI276161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63297[] = {SGI276162, SGI276163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63298[] = {SGI276164, SGI276165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63299[] = {SGI276166, SGI276167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63300[] = {SGI276168, SGI276169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63301[] = {SGI276170, SGI276171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63302[] = {SGI276172, SGI276173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63303[] = {SGI276174, SGI276175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63304[] = {SGI276176, SGI276177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63305[] = {SGI276178, SGI276179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63306[] = {SGI276180, SGI276181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63307[] = {SGI276182, SGI276183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63308[] = {SGI276184, SGI276185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63309[] = {SGI276186, SGI276187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63310[] = {SGI276188, SGI276189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63311[] = {SGI276190, SGI276191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63312[] = {SGI276192, SGI276193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63313[] = {SGI276194, SGI276195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63314[] = {SGI276196, SGI276197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63315[] = {SGI276198, SGI276199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63316[] = {SGI276200, SGI276201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63317[] = {SGI276202, SGI276203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63318[] = {SGI276204, SGI276205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63319[] = {SGI276206, SGI276207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63320[] = {SGI276208, SGI276209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63321[] = {SGI276210, SGI276211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63322[] = {SGI276212, SGI276213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63323[] = {SGI276214, SGI276215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63324[] = {SGI276216, SGI276217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63325[] = {SGI276218, SGI276219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63326[] = {SGI276220, SGI276221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63327[] = {SGI276222, SGI276223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63328[] = {SGI276224, SGI276225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63329[] = {SGI276226, SGI276227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63330[] = {SGI276228, SGI276229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63331[] = {SGI276230, SGI276231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63332[] = {SGI276232, SGI276233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63333[] = {SGI276234, SGI276235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63334[] = {SGI276236, SGI276237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63335[] = {SGI276238, SGI276239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63336[] = {SGI276240, SGI276241, NULL};
const Segment * const SG63337[] = {SGI0, SGI276242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63338[] = {SGI276243, SGI276244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63339[] = {SGI276245, SGI276246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63340[] = {SGI276247, SGI276248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63341[] = {SGI276249, SGI276250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63342[] = {SGI276251, SGI276252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63343[] = {SGI276253, SGI276254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63344[] = {SGI276255, SGI276256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63345[] = {SGI276257, SGI276258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63346[] = {SGI276259, SGI276260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63347[] = {SGI276261, SGI276262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63348[] = {SGI276263, SGI276264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63349[] = {SGI276265, SGI276266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63350[] = {SGI276267, SGI276268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63351[] = {SGI276269, SGI276270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63352[] = {SGI276271, SGI276272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63353[] = {SGI276273, SGI276274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63354[] = {SGI276275, SGI276276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63355[] = {SGI276277, SGI276278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63356[] = {SGI276279, SGI276280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63357[] = {SGI276281, SGI276282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63358[] = {SGI276283, SGI276284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63359[] = {SGI276285, SGI276286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63360[] = {SGI276287, SGI276288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63361[] = {SGI276289, SGI276290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63362[] = {SGI276291, SGI276292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63363[] = {SGI276293, SGI276294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63364[] = {SGI276295, SGI276296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63365[] = {SGI276297, SGI276298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63366[] = {SGI276299, SGI276300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63367[] = {SGI276301, SGI276302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63368[] = {SGI276303, SGI276304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63369[] = {SGI276305, SGI276306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63370[] = {SGI276307, SGI276308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63371[] = {SGI276309, SGI276310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63372[] = {SGI276311, SGI276312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63373[] = {SGI276313, SGI276314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63374[] = {SGI276315, SGI276316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63375[] = {SGI276317, SGI276318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63376[] = {SGI276319, SGI276320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63377[] = {SGI276321, SGI276322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63378[] = {SGI276323, SGI276324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63379[] = {SGI276325, SGI276326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63380[] = {SGI276327, SGI276328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63381[] = {SGI276329, SGI276330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63382[] = {SGI276331, SGI276332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63383[] = {SGI276333, SGI276334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63384[] = {SGI276335, SGI276336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63385[] = {SGI276337, SGI276338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63386[] = {SGI276339, SGI276340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63387[] = {SGI276341, SGI276342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63388[] = {SGI276343, SGI276344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63389[] = {SGI276345, SGI276346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63390[] = {SGI276347, SGI276348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63391[] = {SGI276349, SGI276350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63392[] = {SGI276351, SGI276352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63393[] = {SGI276353, SGI276354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63394[] = {SGI276355, SGI276356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63395[] = {SGI276357, SGI276358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63396[] = {SGI276359, SGI276360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63397[] = {SGI276361, SGI276362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276363, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63398[] = {SGI276364, SGI276365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63399[] = {SGI276366, SGI276367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63400[] = {SGI276368, SGI276369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63401[] = {SGI276370, SGI276371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63402[] = {SGI276372, SGI276373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63403[] = {SGI276374, SGI276375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63404[] = {SGI276376, SGI276377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63405[] = {SGI276378, SGI276379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63406[] = {SGI276380, SGI276381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63407[] = {SGI276382, SGI276383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63408[] = {SGI276384, SGI276385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63409[] = {SGI276386, SGI276387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63410[] = {SGI276388, SGI276389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63411[] = {SGI276390, SGI276391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63412[] = {SGI276392, SGI276393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63413[] = {SGI276394, SGI276395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63414[] = {SGI276396, SGI276397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63415[] = {SGI276398, SGI276399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63416[] = {SGI276400, SGI276401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63417[] = {SGI276402, SGI276403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63418[] = {SGI276404, SGI276405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63419[] = {SGI276406, SGI276407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63420[] = {SGI276408, SGI276409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63421[] = {SGI276410, SGI276411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63422[] = {SGI276412, SGI276413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63423[] = {SGI276414, SGI276415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63424[] = {SGI276416, SGI276417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276418, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63425[] = {SGI276419, SGI276420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276421, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63426[] = {SGI276422, SGI276423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276424, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63427[] = {SGI276425, SGI276426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276427, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63428[] = {SGI276428, SGI276429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276430, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63429[] = {SGI276431, SGI276432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276433, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63430[] = {SGI276434, SGI276435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276436, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63431[] = {SGI276437, SGI276438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276439, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63432[] = {SGI276440, SGI276441, NULL};
const Segment * const SG63433[] = {SGI276442, SGI276443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276444, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63434[] = {SGI276445, SGI276446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276447, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63435[] = {SGI276448, SGI276449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276450, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63436[] = {SGI276451, SGI276452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276453, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63437[] = {SGI276454, SGI276455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276456, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63438[] = {SGI276457, SGI276458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276459, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63439[] = {SGI276460, SGI276461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276462, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63440[] = {SGI276463, SGI276464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276465, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63441[] = {SGI276466, SGI276467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276468, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63442[] = {SGI276469, SGI276470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276471, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63443[] = {SGI276472, SGI276473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276474, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63444[] = {SGI276475, SGI276476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276477, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63445[] = {SGI276478, SGI276479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276480, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63446[] = {SGI276481, SGI276482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276483, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63447[] = {SGI276484, SGI276485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276486, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63448[] = {SGI276487, SGI276488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276489, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63449[] = {SGI276490, SGI276491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276492, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63450[] = {SGI276493, SGI276494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276495, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63451[] = {SGI276496, SGI276497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276498, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63452[] = {SGI276499, SGI276500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276501, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63453[] = {SGI276502, SGI276503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276504, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63454[] = {SGI276505, SGI276506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276507, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63455[] = {SGI276508, SGI276509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276510, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63456[] = {SGI276511, SGI276512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276513, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63457[] = {SGI276514, SGI276515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276516, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63458[] = {SGI276517, SGI276518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276519, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63459[] = {SGI276520, SGI276521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276522, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63460[] = {SGI276523, SGI276524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276525, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63461[] = {SGI276526, SGI276527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276528, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63462[] = {SGI276529, SGI276530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276531, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63463[] = {SGI276532, SGI276533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276534, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63464[] = {SGI276535, SGI276536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276537, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63465[] = {SGI276538, SGI276539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276540, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63466[] = {SGI276541, SGI276542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276543, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63467[] = {SGI276544, SGI276545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276546, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63468[] = {SGI276547, SGI276548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276549, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63469[] = {SGI276550, SGI276551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276552, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63470[] = {SGI276553, SGI276554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276555, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63471[] = {SGI276556, SGI276557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276558, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63472[] = {SGI276559, SGI276560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276561, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63473[] = {SGI276562, SGI276563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276564, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63474[] = {SGI276565, SGI276566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276567, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63475[] = {SGI276568, SGI276569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276570, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63476[] = {SGI276571, SGI276572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276573, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63477[] = {SGI276574, SGI276575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276576, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63478[] = {SGI276577, SGI276578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276579, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63479[] = {SGI276580, SGI276581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276582, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63480[] = {SGI276583, SGI276584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276585, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63481[] = {SGI276586, SGI276587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276588, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63482[] = {SGI276589, SGI276590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276591, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63483[] = {SGI276592, SGI276593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276594, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63484[] = {SGI276595, SGI276596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276597, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63485[] = {SGI276598, SGI276599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276600, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63486[] = {SGI276601, SGI276602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276603, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63487[] = {SGI276604, SGI276605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276606, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63488[] = {SGI276607, SGI276608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276609, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63489[] = {SGI276610, SGI276611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276612, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63490[] = {SGI276613, SGI276614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276615, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63491[] = {SGI276616, SGI276617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276618, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63492[] = {SGI276619, SGI276620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276621, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63493[] = {SGI276622, SGI276623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276624, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63494[] = {SGI276625, SGI276626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276627, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63495[] = {SGI276628, SGI276629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276630, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63496[] = {SGI276631, SGI276632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276633, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63497[] = {SGI276634, SGI276635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276636, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63498[] = {SGI276637, SGI276638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276639, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63499[] = {SGI276640, SGI276641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63500[] = {SGI276642, SGI276643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63501[] = {SGI276644, SGI276645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63502[] = {SGI276646, SGI276647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63503[] = {SGI276648, SGI276649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276650, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63504[] = {SGI276651, SGI276652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276653, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63505[] = {SGI276654, SGI276655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63506[] = {SGI276656, SGI276657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63507[] = {SGI276658, SGI276659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63508[] = {SGI276660, SGI276661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63509[] = {SGI276662, SGI276663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63510[] = {SGI276664, SGI276665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63511[] = {SGI276666, SGI276667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63512[] = {SGI276668, SGI276669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63513[] = {SGI276670, SGI276671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63514[] = {SGI276672, SGI276673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63515[] = {SGI276674, SGI276675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63516[] = {SGI276676, SGI276677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63517[] = {SGI276678, SGI276679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63518[] = {SGI276680, SGI276681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63519[] = {SGI276682, SGI276683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63520[] = {SGI0, SGI276684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63521[] = {SGI276685, SGI276686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63522[] = {SGI276687, SGI276688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63523[] = {SGI276689, SGI276690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63524[] = {SGI276691, SGI276692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63525[] = {SGI276693, SGI276694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63526[] = {SGI276695, SGI276696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63527[] = {SGI0, SGI276697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63528[] = {SGI276698, SGI276699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63529[] = {SGI276700, SGI276701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63530[] = {SGI276702, SGI276703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63531[] = {SGI0, SGI276704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63532[] = {SGI0, SGI276705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63533[] = {SGI0, SGI276706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63534[] = {SGI0, SGI276707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63535[] = {SGI276708, SGI276709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63536[] = {SGI0, SGI276710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63537[] = {SGI276711, SGI276712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63538[] = {SGI276713, SGI276714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63539[] = {SGI276715, SGI276716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63540[] = {SGI276717, SGI276718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63541[] = {SGI276719, SGI276720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63542[] = {SGI276721, SGI276722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63543[] = {SGI276723, SGI276724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63544[] = {SGI276725, SGI276726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63545[] = {SGI276727, SGI276728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63546[] = {SGI276729, SGI276730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63547[] = {SGI276731, SGI276732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63548[] = {SGI276733, SGI276734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63549[] = {SGI276735, SGI276736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63550[] = {SGI276737, SGI276738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63551[] = {SGI276739, SGI276740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63552[] = {SGI276741, SGI276742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63553[] = {SGI276743, SGI276744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63554[] = {SGI276745, SGI276746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63555[] = {SGI276747, SGI276748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63556[] = {SGI276749, SGI276750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63557[] = {SGI276751, SGI276752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63558[] = {SGI276753, SGI276754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63559[] = {SGI276755, SGI276756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63560[] = {SGI276757, SGI276758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63561[] = {SGI276760, SGI276761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63562[] = {SGI276763, SGI276764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63563[] = {SGI276766, SGI276767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63564[] = {SGI276769, SGI276770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63565[] = {SGI276772, SGI276773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63566[] = {SGI276775, SGI276776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63567[] = {SGI276778, SGI276779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63568[] = {SGI276781, SGI276782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63569[] = {SGI276784, SGI276785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63570[] = {SGI276786, SGI276787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63571[] = {SGI276788, SGI276789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63572[] = {SGI276790, SGI276791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63573[] = {SGI276792, SGI276793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63574[] = {SGI276794, SGI276795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63575[] = {SGI276796, SGI276797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63576[] = {SGI276798, SGI276799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63577[] = {SGI276800, SGI276801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63578[] = {SGI276802, SGI276803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63579[] = {SGI276804, SGI276805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63580[] = {SGI276806, SGI276807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63581[] = {SGI276808, SGI276809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63582[] = {SGI276810, SGI276811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63583[] = {SGI276812, SGI276813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63584[] = {SGI276814, SGI276815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63585[] = {SGI276816, SGI276817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63586[] = {SGI276818, SGI276819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63587[] = {SGI276820, SGI276821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63588[] = {SGI276822, SGI276823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63589[] = {SGI276824, SGI276825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63590[] = {SGI276826, SGI276827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63591[] = {SGI276828, SGI276829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63592[] = {SGI276830, SGI276831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63593[] = {SGI276832, SGI276833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63594[] = {SGI276834, SGI276835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63595[] = {SGI276836, SGI276837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63596[] = {SGI276838, SGI276839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63597[] = {SGI276840, SGI276841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63598[] = {SGI276842, SGI276843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63599[] = {SGI276844, SGI276845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63600[] = {SGI276846, SGI276847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63601[] = {SGI276848, SGI276849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63602[] = {SGI276850, SGI276851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63603[] = {SGI276852, SGI276853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63604[] = {SGI276854, SGI276855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63605[] = {SGI276856, SGI276857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63606[] = {SGI276858, SGI276859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63607[] = {SGI276860, SGI276861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63608[] = {SGI276862, SGI276863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276864, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63609[] = {SGI276865, SGI276866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63610[] = {SGI276867, SGI276868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63611[] = {SGI276869, SGI276870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63612[] = {SGI276871, SGI276872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63613[] = {SGI276873, SGI276874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63614[] = {SGI276875, SGI276876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63615[] = {SGI276877, SGI276878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63616[] = {SGI276879, SGI276880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63617[] = {SGI276881, SGI276882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63618[] = {SGI276883, SGI276884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63619[] = {SGI276885, SGI276886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63620[] = {SGI276887, SGI276888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63621[] = {SGI276889, SGI276890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63622[] = {SGI276891, SGI276892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63623[] = {SGI276893, SGI276894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63624[] = {SGI276895, SGI276896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63625[] = {SGI276897, SGI276898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63626[] = {SGI276899, SGI276900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63627[] = {SGI276901, SGI276902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63628[] = {SGI276903, SGI276904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63629[] = {SGI276905, SGI276906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63630[] = {SGI276907, SGI276908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63631[] = {SGI276909, SGI276910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63632[] = {SGI276911, SGI276912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63633[] = {SGI276913, SGI276914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63634[] = {SGI276915, SGI276916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63635[] = {SGI276917, SGI276918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63636[] = {SGI276919, SGI276920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63637[] = {SGI276921, SGI276922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63638[] = {SGI276923, SGI276924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63639[] = {SGI276925, SGI276926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63640[] = {SGI276927, SGI276928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63641[] = {SGI276929, SGI276930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63642[] = {SGI276931, SGI276932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63643[] = {SGI276933, SGI276934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63644[] = {SGI276935, SGI276936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63645[] = {SGI276937, SGI276938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63646[] = {SGI276939, SGI276940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63647[] = {SGI276941, SGI276942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63648[] = {SGI276943, SGI276944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276945, SGI0, SGI0, NULL};
const Segment * const SG63649[] = {SGI276946, SGI276947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63650[] = {SGI276948, SGI276949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63651[] = {SGI276950, SGI276951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63652[] = {SGI276952, SGI276953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI276954, SGI0, SGI0, NULL};
const Segment * const SG63653[] = {SGI276955, SGI276956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63654[] = {SGI276957, SGI276958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63655[] = {SGI276959, SGI276960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63656[] = {SGI276961, SGI276962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63657[] = {SGI276963, SGI276964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63658[] = {SGI276965, SGI276966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63659[] = {SGI276967, SGI276968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63660[] = {SGI276969, SGI276970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63661[] = {SGI276971, SGI276972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63662[] = {SGI276973, SGI276974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63663[] = {SGI276975, SGI276976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63664[] = {SGI276977, SGI276978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63665[] = {SGI276979, SGI276980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63666[] = {SGI276981, SGI276982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63667[] = {SGI276983, SGI276984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63668[] = {SGI276985, SGI276986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63669[] = {SGI276987, SGI276988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63670[] = {SGI276989, SGI276990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63671[] = {SGI276991, SGI276992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63672[] = {SGI276993, SGI276994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63673[] = {SGI276995, SGI276996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63674[] = {SGI276997, SGI276998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63675[] = {SGI276999, SGI277000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63676[] = {SGI0, SGI277001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63677[] = {SGI277002, SGI277003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63678[] = {SGI277004, SGI277005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63679[] = {SGI277006, SGI277007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63680[] = {SGI277008, SGI277009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277010, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63681[] = {SGI277011, SGI277012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63682[] = {SGI277013, SGI277014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63683[] = {SGI277015, SGI277016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63684[] = {SGI277017, SGI277018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63685[] = {SGI277019, SGI277020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63686[] = {SGI277021, SGI277022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63687[] = {SGI277023, SGI277024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63688[] = {SGI277025, SGI277026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63689[] = {SGI277027, SGI277028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63690[] = {SGI277029, SGI277030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63691[] = {SGI277031, SGI277032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63692[] = {SGI277033, SGI277034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63693[] = {SGI277035, SGI277036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63694[] = {SGI277037, SGI277038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63695[] = {SGI277039, SGI277040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63696[] = {SGI277041, SGI277042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63697[] = {SGI277043, SGI277044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63698[] = {SGI277045, SGI277046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63699[] = {SGI277047, SGI277048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63700[] = {SGI277049, SGI277050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63701[] = {SGI277051, SGI277052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63702[] = {SGI277053, SGI277054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63703[] = {SGI277055, SGI277056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63704[] = {SGI277057, SGI277058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63705[] = {SGI277059, SGI277060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63706[] = {SGI277061, SGI277062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63707[] = {SGI277063, SGI277064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63708[] = {SGI277065, SGI277066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63709[] = {SGI277067, SGI277068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63710[] = {SGI277069, SGI277070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63711[] = {SGI277071, SGI277072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63712[] = {SGI277073, SGI277074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63713[] = {SGI277075, SGI277076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63714[] = {SGI277077, SGI277078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63715[] = {SGI277079, SGI277080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63716[] = {SGI277081, SGI277082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63717[] = {SGI277083, SGI277084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63718[] = {SGI277085, SGI277086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63719[] = {SGI277087, SGI277088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63720[] = {SGI277089, SGI277090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63721[] = {SGI277091, SGI277092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63722[] = {SGI277093, SGI277094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63723[] = {SGI277095, SGI277096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63724[] = {SGI277097, SGI277098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63725[] = {SGI277099, SGI277100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63726[] = {SGI277101, SGI277102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63727[] = {SGI277103, SGI277104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63728[] = {SGI277105, SGI277106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63729[] = {SGI277107, SGI277108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63730[] = {SGI277109, SGI277110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63731[] = {SGI277111, SGI277112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63732[] = {SGI277113, SGI277114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63733[] = {SGI277115, SGI277116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63734[] = {SGI277117, SGI277118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63735[] = {SGI277119, SGI277120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63736[] = {SGI277121, SGI277122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63737[] = {SGI277123, SGI277124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63738[] = {SGI277125, SGI277126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63739[] = {SGI277127, SGI277128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63740[] = {SGI277129, SGI277130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63741[] = {SGI277131, SGI277132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63742[] = {SGI277133, SGI277134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63743[] = {SGI277135, SGI277136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63744[] = {SGI277137, SGI277138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63745[] = {SGI277139, SGI277140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63746[] = {SGI277141, SGI277142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63747[] = {SGI277143, SGI277144, NULL};
const Segment * const SG63748[] = {SGI277145, SGI277146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63749[] = {SGI277147, SGI277148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63750[] = {SGI277149, SGI277150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63751[] = {SGI277151, SGI277152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63752[] = {SGI277153, SGI277154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63753[] = {SGI277155, SGI277156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63754[] = {SGI277157, SGI277158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63755[] = {SGI277159, SGI277160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63756[] = {SGI277161, SGI277162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63757[] = {SGI277163, SGI277164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63758[] = {SGI277165, SGI277166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63759[] = {SGI277167, SGI277168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63760[] = {SGI277169, SGI277170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63761[] = {SGI277171, SGI277172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63762[] = {SGI277173, SGI277174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63763[] = {SGI277175, SGI277176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63764[] = {SGI277177, SGI277178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63765[] = {SGI277179, SGI277180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63766[] = {SGI277181, SGI277182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63767[] = {SGI277183, SGI277184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63768[] = {SGI277185, SGI277187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63769[] = {SGI277186, SGI277188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63770[] = {SGI277189, SGI277190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63771[] = {SGI277191, SGI277192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63772[] = {SGI277193, SGI277194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63773[] = {SGI277195, SGI277196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63774[] = {SGI277197, SGI277198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63775[] = {SGI277199, SGI277200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63776[] = {SGI277201, SGI277202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63777[] = {SGI277203, SGI277204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63778[] = {SGI277205, SGI277206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63779[] = {SGI277207, SGI277208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63780[] = {SGI277210, SGI277211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63781[] = {SGI277212, SGI277213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63782[] = {SGI277214, SGI277215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63783[] = {SGI277216, SGI277217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63784[] = {SGI277218, SGI277219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63785[] = {SGI277220, SGI277221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63786[] = {SGI277222, SGI277223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63787[] = {SGI277224, SGI277225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63788[] = {SGI277226, SGI277227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277228, SGI0, SGI0, NULL};
const Segment * const SG63789[] = {SGI277229, SGI277230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63790[] = {SGI277231, SGI277232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63791[] = {SGI277233, SGI277234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63792[] = {SGI277235, SGI277236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63793[] = {SGI277237, SGI277238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63794[] = {SGI277239, SGI277240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63795[] = {SGI277241, SGI277242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63796[] = {SGI277243, SGI277244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277245, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63797[] = {SGI277246, SGI277247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63798[] = {SGI277248, SGI277249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277250, SGI0, SGI0, NULL};
const Segment * const SG63799[] = {SGI277251, SGI277252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277253, SGI0, SGI0, NULL};
const Segment * const SG63800[] = {SGI277254, SGI277255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277256, SGI0, SGI0, NULL};
const Segment * const SG63801[] = {SGI277257, SGI277258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277259, SGI0, SGI0, NULL};
const Segment * const SG63802[] = {SGI277260, SGI277261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277262, SGI0, SGI0, NULL};
const Segment * const SG63803[] = {SGI277263, SGI277264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63804[] = {SGI277265, SGI277266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277267, SGI0, SGI0, NULL};
const Segment * const SG63805[] = {SGI277268, SGI277269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277270, SGI0, SGI0, NULL};
const Segment * const SG63806[] = {SGI277209, SGI277271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63807[] = {SGI277272, SGI277273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63808[] = {SGI277274, SGI277275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63809[] = {SGI277276, SGI277277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63810[] = {SGI277278, SGI277279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63811[] = {SGI277280, SGI277281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63812[] = {SGI277282, SGI277283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63813[] = {SGI277284, SGI277285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63814[] = {SGI277286, SGI277287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63815[] = {SGI277288, SGI277289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63816[] = {SGI277290, SGI277291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63817[] = {SGI277292, SGI277293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63818[] = {SGI277294, SGI277295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63819[] = {SGI277296, SGI277297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63820[] = {SGI277298, SGI277299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63821[] = {SGI277300, SGI277301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63822[] = {SGI277302, SGI277303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63823[] = {SGI277304, SGI277305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63824[] = {SGI277306, SGI277307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63825[] = {SGI277308, SGI277309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63826[] = {SGI277310, SGI277311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63827[] = {SGI277312, SGI277313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63828[] = {SGI277314, SGI277315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63829[] = {SGI277316, SGI277317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63830[] = {SGI277318, SGI277319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63831[] = {SGI277320, SGI277321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63832[] = {SGI277322, SGI277323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63833[] = {SGI277324, SGI277325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63834[] = {SGI277326, SGI277327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63835[] = {SGI277328, SGI277329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63836[] = {SGI277330, SGI277331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63837[] = {SGI277332, SGI277333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63838[] = {SGI277334, SGI277335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63839[] = {SGI277336, SGI277337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63840[] = {SGI277338, SGI277339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63841[] = {SGI277340, SGI277341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63842[] = {SGI277342, SGI277343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63843[] = {SGI277344, SGI277345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63844[] = {SGI277346, SGI277347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63845[] = {SGI277348, SGI277349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63846[] = {SGI277350, SGI277351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277352, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63847[] = {SGI277353, SGI277354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63848[] = {SGI277355, SGI277356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63849[] = {SGI277357, SGI277358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63850[] = {SGI277359, SGI277360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63851[] = {SGI277361, SGI277362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63852[] = {SGI277363, SGI277364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63853[] = {SGI277365, SGI277366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63854[] = {SGI277367, SGI277368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63855[] = {SGI277369, SGI277370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63856[] = {SGI277371, SGI277372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63857[] = {SGI277373, SGI277374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63858[] = {SGI277375, SGI277376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63859[] = {SGI277377, SGI277378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63860[] = {SGI277379, SGI277380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63861[] = {SGI277381, SGI277382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63862[] = {SGI277383, SGI277384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63863[] = {SGI277385, SGI277386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63864[] = {SGI277387, SGI277388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63865[] = {SGI277389, SGI277390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63866[] = {SGI277391, SGI277392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63867[] = {SGI277393, SGI277394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63868[] = {SGI277395, SGI277396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63869[] = {SGI277397, SGI277398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63870[] = {SGI277399, SGI277400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63871[] = {SGI277401, SGI277402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63872[] = {SGI277403, SGI277404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277405, SGI0, SGI0, NULL};
const Segment * const SG63873[] = {SGI277406, SGI277407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63874[] = {SGI277408, SGI277409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277410, SGI0, SGI0, NULL};
const Segment * const SG63875[] = {SGI277411, SGI277412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277413, SGI0, SGI0, NULL};
const Segment * const SG63876[] = {SGI277414, SGI277415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277416, SGI0, SGI0, NULL};
const Segment * const SG63877[] = {SGI277417, SGI277418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277419, SGI0, SGI0, NULL};
const Segment * const SG63878[] = {SGI277420, SGI277421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277422, SGI0, SGI0, NULL};
const Segment * const SG63879[] = {SGI277423, SGI277424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277425, SGI0, SGI0, NULL};
const Segment * const SG63880[] = {SGI277426, SGI277427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277428, SGI0, SGI0, NULL};
const Segment * const SG63881[] = {SGI277429, SGI277430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277431, SGI0, SGI0, NULL};
const Segment * const SG63882[] = {SGI277432, SGI277433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277434, SGI0, SGI0, NULL};
const Segment * const SG63883[] = {SGI277435, SGI277436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277437, SGI0, SGI0, NULL};
const Segment * const SG63884[] = {SGI277438, SGI277439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277440, SGI0, SGI0, NULL};
const Segment * const SG63885[] = {SGI277441, SGI277442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277443, SGI0, SGI0, NULL};
const Segment * const SG63886[] = {SGI277444, SGI277445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277446, SGI0, SGI0, NULL};
const Segment * const SG63887[] = {SGI277447, SGI277448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277449, SGI0, SGI0, NULL};
const Segment * const SG63888[] = {SGI277450, SGI277451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277452, SGI0, SGI0, NULL};
const Segment * const SG63889[] = {SGI277453, SGI277454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277455, SGI0, SGI0, NULL};
const Segment * const SG63890[] = {SGI277456, SGI277457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63891[] = {SGI277458, SGI277459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63892[] = {SGI277460, SGI277461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63893[] = {SGI277462, SGI277463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63894[] = {SGI277464, SGI277465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63895[] = {SGI277466, SGI277467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63896[] = {SGI277468, SGI277469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63897[] = {SGI277470, SGI277471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63898[] = {SGI277472, SGI277473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63899[] = {SGI277474, SGI277475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63900[] = {SGI277476, SGI277477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63901[] = {SGI277478, SGI277479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63902[] = {SGI277480, SGI277481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63903[] = {SGI277482, SGI277483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63904[] = {SGI277484, SGI277485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63905[] = {SGI277486, SGI277487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63906[] = {SGI277488, SGI277489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63907[] = {SGI277490, SGI277491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63908[] = {SGI277492, SGI277493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63909[] = {SGI277494, SGI277495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63910[] = {SGI277496, SGI277497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277498, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63911[] = {SGI277499, SGI277500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63912[] = {SGI277501, SGI277502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63913[] = {SGI277503, SGI277504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63914[] = {SGI277505, SGI277506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63915[] = {SGI277507, SGI277508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63916[] = {SGI277509, SGI277510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63917[] = {SGI277511, SGI277512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63918[] = {SGI277513, SGI277514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63919[] = {SGI277515, SGI277516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63920[] = {SGI277517, SGI277518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63921[] = {SGI277519, SGI277520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63922[] = {SGI277521, SGI277522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63923[] = {SGI277523, SGI277524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63924[] = {SGI277525, SGI277526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63925[] = {SGI277527, SGI277529, NULL};
const Segment * const SG63926[] = {SGI277528, SGI277530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63927[] = {SGI277531, SGI277532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63928[] = {SGI277533, SGI277534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63929[] = {SGI277535, SGI277536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63930[] = {SGI277537, SGI277538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63931[] = {SGI277539, SGI277540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63932[] = {SGI277542, SGI277543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63933[] = {SGI277545, SGI277546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63934[] = {SGI277548, SGI277549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63935[] = {SGI277551, SGI277552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63936[] = {SGI277554, SGI277555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63937[] = {SGI277557, SGI277558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63938[] = {SGI277560, SGI277561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63939[] = {SGI277563, SGI277564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63940[] = {SGI277566, SGI277567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63941[] = {SGI277568, SGI277569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63942[] = {SGI277570, SGI277571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63943[] = {SGI277572, SGI277573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63944[] = {SGI277574, SGI277575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63945[] = {SGI277576, SGI277577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63946[] = {SGI277578, SGI277579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63947[] = {SGI277580, SGI277581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63948[] = {SGI277582, SGI277583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63949[] = {SGI277584, SGI277585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63950[] = {SGI277586, SGI277587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63951[] = {SGI277588, SGI277589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63952[] = {SGI277590, SGI277591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63953[] = {SGI277592, SGI277593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63954[] = {SGI277594, SGI277595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63955[] = {SGI277596, SGI277597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63956[] = {SGI277598, SGI277599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63957[] = {SGI277600, SGI277601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63958[] = {SGI277602, SGI277603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63959[] = {SGI277604, SGI277605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63960[] = {SGI277606, SGI277607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63961[] = {SGI277608, SGI277609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63962[] = {SGI277610, SGI277611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63963[] = {SGI277612, SGI277613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63964[] = {SGI277614, SGI277615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63965[] = {SGI277616, SGI277617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63966[] = {SGI277618, SGI277619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63967[] = {SGI277620, SGI277621, NULL};
const Segment * const SG63968[] = {SGI277622, SGI277623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63969[] = {SGI277624, SGI277625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63970[] = {SGI277626, SGI277627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63971[] = {SGI0, SGI277628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63972[] = {SGI277629, SGI277630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63973[] = {SGI277631, SGI277632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63974[] = {SGI277633, SGI277634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63975[] = {SGI277635, SGI277636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63976[] = {SGI277637, SGI277638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63977[] = {SGI277639, SGI277640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63978[] = {SGI277641, SGI277642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63979[] = {SGI277643, SGI277644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63980[] = {SGI277645, SGI277646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63981[] = {SGI277647, SGI277648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63982[] = {SGI277649, SGI277650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277651, SGI0, NULL};
const Segment * const SG63983[] = {SGI277652, SGI277653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277654, SGI0, NULL};
const Segment * const SG63984[] = {SGI277655, SGI277656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63985[] = {SGI277657, SGI277658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63986[] = {SGI277659, SGI277660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63987[] = {SGI277661, SGI277662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63988[] = {SGI277663, SGI277664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63989[] = {SGI277665, SGI277666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63990[] = {SGI277667, SGI277668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63991[] = {SGI277669, SGI277670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63992[] = {SGI277671, SGI277672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63993[] = {SGI277673, SGI277674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63994[] = {SGI277675, SGI277676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63995[] = {SGI277677, SGI277678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63996[] = {SGI277679, SGI277680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63997[] = {SGI277681, SGI277682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63998[] = {SGI277683, SGI277684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG63999[] = {SGI277685, SGI277686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64000[] = {SGI277687, SGI277688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64001[] = {SGI277689, SGI277690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64002[] = {SGI277691, SGI277692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64003[] = {SGI277693, SGI277694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64004[] = {SGI277695, SGI277696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64005[] = {SGI277697, SGI277698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64006[] = {SGI277699, SGI277700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64007[] = {SGI277701, SGI277702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64008[] = {SGI277703, SGI277704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64009[] = {SGI277705, SGI277706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64010[] = {SGI277707, SGI277708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64011[] = {SGI277709, SGI277710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64012[] = {SGI277711, SGI277712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64013[] = {SGI277713, SGI277714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64014[] = {SGI277715, SGI277716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64015[] = {SGI277717, SGI277718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64016[] = {SGI277719, SGI277720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64017[] = {SGI277721, SGI277722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64018[] = {SGI277723, SGI277724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64019[] = {SGI277725, SGI277726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64020[] = {SGI277727, SGI277728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64021[] = {SGI277729, SGI277730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64022[] = {SGI277731, SGI277732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64023[] = {SGI277733, SGI277734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64024[] = {SGI277735, SGI277736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64025[] = {SGI277737, SGI277738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64026[] = {SGI277739, SGI277740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64027[] = {SGI277741, SGI277742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64028[] = {SGI277743, SGI277744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64029[] = {SGI277745, SGI277746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64030[] = {SGI277747, SGI277748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64031[] = {SGI277749, SGI277750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64032[] = {SGI277751, SGI277752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64033[] = {SGI277753, SGI277754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64034[] = {SGI277755, SGI277756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64035[] = {SGI277757, SGI277758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64036[] = {SGI277759, SGI277760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64037[] = {SGI277761, SGI277762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64038[] = {SGI277763, SGI277764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64039[] = {SGI277765, SGI277766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64040[] = {SGI277767, SGI277768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64041[] = {SGI277769, SGI277770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64042[] = {SGI277771, SGI277772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64043[] = {SGI277773, SGI277774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64044[] = {SGI277775, SGI277776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64045[] = {SGI277777, SGI277778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64046[] = {SGI277779, SGI277780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64047[] = {SGI277781, SGI277782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64048[] = {SGI277783, SGI277784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64049[] = {SGI277785, SGI277786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64050[] = {SGI277787, SGI277788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64051[] = {SGI277789, SGI277790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64052[] = {SGI277791, SGI277792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64053[] = {SGI277793, SGI277794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64054[] = {SGI277795, SGI277796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64055[] = {SGI277797, SGI277798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64056[] = {SGI277799, SGI277801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64057[] = {SGI277800, SGI277802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64058[] = {SGI277803, SGI277804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64059[] = {SGI277805, SGI277806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64060[] = {SGI277807, SGI277808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64061[] = {SGI277809, SGI277810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64062[] = {SGI277811, SGI277812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64063[] = {SGI277813, SGI277814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64064[] = {SGI277815, SGI277816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64065[] = {SGI277817, SGI277818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64066[] = {SGI277819, SGI277820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64067[] = {SGI277821, SGI277822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64068[] = {SGI277823, SGI277824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64069[] = {SGI277826, SGI277827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64070[] = {SGI277825, SGI277828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64071[] = {SGI277829, SGI277830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64072[] = {SGI277831, SGI277832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64073[] = {SGI277833, SGI277834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64074[] = {SGI277835, SGI277836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64075[] = {SGI277837, SGI277838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64076[] = {SGI277840, SGI277841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64077[] = {SGI277839, SGI277842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64078[] = {SGI277843, SGI277844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64079[] = {SGI277845, SGI277846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64080[] = {SGI277847, SGI277848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64081[] = {SGI277851, SGI277852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64082[] = {SGI277849, SGI277850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64083[] = {SGI277854, SGI277855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64084[] = {SGI277853, SGI277856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64085[] = {SGI277857, SGI277858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64086[] = {SGI277859, SGI277860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64087[] = {SGI277861, SGI277862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64088[] = {SGI277863, SGI277864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64089[] = {SGI277865, SGI277866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64090[] = {SGI277867, SGI277868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64091[] = {SGI277869, SGI277870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64092[] = {SGI277871, SGI277872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64093[] = {SGI277873, SGI277874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64094[] = {SGI277875, SGI277876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64095[] = {SGI277877, SGI277878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277879, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64096[] = {SGI277880, SGI277881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277882, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64097[] = {SGI277883, SGI277884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277885, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64098[] = {SGI277886, SGI277887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277888, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64099[] = {SGI277889, SGI277890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64100[] = {SGI277891, SGI277892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64101[] = {SGI277893, SGI277894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64102[] = {SGI277895, SGI277896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64103[] = {SGI277897, SGI277898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64104[] = {SGI277899, SGI277900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64105[] = {SGI277901, SGI277902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64106[] = {SGI277903, SGI277904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64107[] = {SGI277905, SGI277906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64108[] = {SGI277907, SGI277908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64109[] = {SGI277909, SGI277910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64110[] = {SGI277911, SGI277912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64111[] = {SGI277913, SGI277914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64112[] = {SGI277915, SGI277916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64113[] = {SGI277917, SGI277918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64114[] = {SGI277919, SGI277920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64115[] = {SGI277921, SGI277922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64116[] = {SGI277923, SGI277924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64117[] = {SGI277925, SGI277926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64118[] = {SGI277927, SGI277928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64119[] = {SGI277929, SGI277930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64120[] = {SGI277931, SGI277932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64121[] = {SGI277933, SGI277934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64122[] = {SGI277935, SGI277936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64123[] = {SGI277937, SGI277938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64124[] = {SGI277939, SGI277940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64125[] = {SGI277941, SGI277942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64126[] = {SGI277943, SGI277944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64127[] = {SGI277945, SGI277946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64128[] = {SGI277947, SGI277948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277949, SGI0, SGI0, NULL};
const Segment * const SG64129[] = {SGI277950, SGI277951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64130[] = {SGI277952, SGI277953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64131[] = {SGI277954, SGI277955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI277956, SGI0, SGI0, NULL};
const Segment * const SG64132[] = {SGI277957, SGI277958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64133[] = {SGI277959, SGI277960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64134[] = {SGI277961, SGI277962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64135[] = {SGI277963, SGI277964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64136[] = {SGI277965, SGI277966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64137[] = {SGI277967, SGI277968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64138[] = {SGI277969, SGI277970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64139[] = {SGI277971, SGI277972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64140[] = {SGI277973, SGI277974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64141[] = {SGI277975, SGI277976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64142[] = {SGI277977, SGI277978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64143[] = {SGI277979, SGI277980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64144[] = {SGI277981, SGI277982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64145[] = {SGI277983, SGI277984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64146[] = {SGI277985, SGI277986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64147[] = {SGI277987, SGI277988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64148[] = {SGI277989, SGI277990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64149[] = {SGI277991, SGI277992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64150[] = {SGI277993, SGI277994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64151[] = {SGI277995, SGI277996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64152[] = {SGI277997, SGI277998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64153[] = {SGI277999, SGI278000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64154[] = {SGI278001, SGI278002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64155[] = {SGI278003, SGI278004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64156[] = {SGI278005, SGI278006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64157[] = {SGI278007, SGI278008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64158[] = {SGI278009, SGI278010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64159[] = {SGI278011, SGI278012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64160[] = {SGI278013, SGI278014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64161[] = {SGI278015, SGI278016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64162[] = {SGI278017, SGI278018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64163[] = {SGI278019, SGI278020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64164[] = {SGI0, SGI278021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64165[] = {SGI278022, SGI278023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64166[] = {SGI278024, SGI278025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64167[] = {SGI278026, SGI278027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64168[] = {SGI278028, SGI278029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64169[] = {SGI278030, SGI278031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278032, SGI0, NULL};
const Segment * const SG64170[] = {SGI278033, SGI278034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278035, SGI0, NULL};
const Segment * const SG64171[] = {SGI278036, SGI278037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278038, SGI0, NULL};
const Segment * const SG64172[] = {SGI278039, SGI278040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278041, SGI0, NULL};
const Segment * const SG64173[] = {SGI278042, SGI278043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64174[] = {SGI278044, SGI278045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64175[] = {SGI278046, SGI278047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64176[] = {SGI278048, SGI278049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64177[] = {SGI278050, SGI278051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64178[] = {SGI278052, SGI278053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64179[] = {SGI278054, SGI278055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64180[] = {SGI278056, SGI278057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64181[] = {SGI278058, SGI278059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64182[] = {SGI278060, SGI278061, NULL};
const Segment * const SG64183[] = {SGI278062, SGI278063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64184[] = {SGI278064, SGI278065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64185[] = {SGI278066, SGI278067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64186[] = {SGI278068, SGI278069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64187[] = {SGI278070, SGI278071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64188[] = {SGI278072, SGI278073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64189[] = {SGI278074, SGI278075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64190[] = {SGI278076, SGI278077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64191[] = {SGI278078, SGI278079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64192[] = {SGI278080, SGI278081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64193[] = {SGI278082, SGI278083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64194[] = {SGI278084, SGI278085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64195[] = {SGI278086, SGI278087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64196[] = {SGI278088, SGI278089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64197[] = {SGI278090, SGI278091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64198[] = {SGI278092, SGI278093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64199[] = {SGI278094, SGI278095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64200[] = {SGI278096, SGI278097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64201[] = {SGI278098, SGI278099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64202[] = {SGI278100, SGI278101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64203[] = {SGI278102, SGI278103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64204[] = {SGI278104, SGI278105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64205[] = {SGI278106, SGI278107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64206[] = {SGI278108, SGI278109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64207[] = {SGI278110, SGI278111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64208[] = {SGI278112, SGI278113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64209[] = {SGI278114, SGI278115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64210[] = {SGI278116, SGI278117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64211[] = {SGI0, SGI278118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64212[] = {SGI278119, SGI278120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64213[] = {SGI278121, SGI278122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64214[] = {SGI278123, SGI278124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64215[] = {SGI278125, SGI278126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64216[] = {SGI278127, SGI278128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64217[] = {SGI278129, SGI278130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64218[] = {SGI278131, SGI278132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64219[] = {SGI278133, SGI278134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64220[] = {SGI278135, SGI278136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64221[] = {SGI278137, SGI278138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64222[] = {SGI278139, SGI278140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64223[] = {SGI278141, SGI278142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64224[] = {SGI278143, SGI278144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64225[] = {SGI278145, SGI278146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64226[] = {SGI278147, SGI278148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64227[] = {SGI278149, SGI278150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64228[] = {SGI278151, SGI278152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64229[] = {SGI278153, SGI278154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64230[] = {SGI278155, SGI278156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64231[] = {SGI278157, SGI278158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64232[] = {SGI278159, SGI278160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64233[] = {SGI278161, SGI278162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64234[] = {SGI278163, SGI278164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64235[] = {SGI278165, SGI278166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64236[] = {SGI278167, SGI278168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64237[] = {SGI278169, SGI278170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64238[] = {SGI278171, SGI278172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64239[] = {SGI278173, SGI278174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64240[] = {SGI278175, SGI278176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64241[] = {SGI278177, SGI278178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64242[] = {SGI278179, SGI278180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64243[] = {SGI278181, SGI278182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64244[] = {SGI278183, SGI278184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64245[] = {SGI278185, SGI278186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64246[] = {SGI278187, SGI278188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64247[] = {SGI278189, SGI278190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64248[] = {SGI278191, SGI278192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64249[] = {SGI278193, SGI278194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64250[] = {SGI278195, SGI278196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64251[] = {SGI278197, SGI278198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64252[] = {SGI278199, SGI278200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64253[] = {SGI278201, SGI278202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64254[] = {SGI278203, SGI278204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64255[] = {SGI278205, SGI278206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64256[] = {SGI278207, SGI278208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64257[] = {SGI278209, SGI278210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64258[] = {SGI278211, SGI278212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64259[] = {SGI278213, SGI278214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64260[] = {SGI278215, SGI278216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64261[] = {SGI278217, SGI278218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64262[] = {SGI278219, SGI278220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64263[] = {SGI278221, SGI278222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64264[] = {SGI278223, SGI278224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64265[] = {SGI278225, SGI278226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64266[] = {SGI278227, SGI278228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64267[] = {SGI278229, SGI278230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64268[] = {SGI278231, SGI278232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64269[] = {SGI278233, SGI278234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64270[] = {SGI278235, SGI278236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64271[] = {SGI278237, SGI278238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64272[] = {SGI278239, SGI278240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64273[] = {SGI278241, SGI278242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64274[] = {SGI278243, SGI278244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64275[] = {SGI278245, SGI278246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64276[] = {SGI278247, SGI278248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64277[] = {SGI278249, SGI278250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64278[] = {SGI278251, SGI278252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64279[] = {SGI278253, SGI278254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64280[] = {SGI278255, SGI278256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64281[] = {SGI278257, SGI278258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64282[] = {SGI278259, SGI278260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64283[] = {SGI278261, SGI278262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64284[] = {SGI278263, SGI278264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64285[] = {SGI278265, SGI278266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64286[] = {SGI278267, SGI278268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64287[] = {SGI278269, SGI278270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64288[] = {SGI278271, SGI278272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64289[] = {SGI278273, SGI278274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64290[] = {SGI278275, SGI278276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64291[] = {SGI278277, SGI278278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64292[] = {SGI278279, SGI278280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64293[] = {SGI278281, SGI278282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64294[] = {SGI278283, SGI278284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64295[] = {SGI278285, SGI278286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64296[] = {SGI278287, SGI278288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64297[] = {SGI278289, SGI278290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64298[] = {SGI278291, SGI278292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64299[] = {SGI278293, SGI278294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64300[] = {SGI278295, SGI278296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64301[] = {SGI278297, SGI278298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64302[] = {SGI278299, SGI278300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64303[] = {SGI278301, SGI278302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64304[] = {SGI278303, SGI278304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64305[] = {SGI278305, SGI278306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64306[] = {SGI278307, SGI278308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64307[] = {SGI278309, SGI278310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64308[] = {SGI278311, SGI278312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64309[] = {SGI278313, SGI278314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64310[] = {SGI278315, SGI278316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64311[] = {SGI278317, SGI278318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64312[] = {SGI278319, SGI278320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64313[] = {SGI278321, SGI278322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64314[] = {SGI278323, SGI278324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64315[] = {SGI278325, SGI278326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64316[] = {SGI278327, SGI278328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64317[] = {SGI278329, SGI278330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64318[] = {SGI278331, SGI278332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64319[] = {SGI278333, SGI278334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64320[] = {SGI278335, SGI278336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64321[] = {SGI278337, SGI278339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64322[] = {SGI278338, SGI278340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64323[] = {SGI278341, SGI278342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64324[] = {SGI278343, SGI278344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64325[] = {SGI278345, SGI278346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64326[] = {SGI278347, SGI278348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64327[] = {SGI278349, SGI278351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64328[] = {SGI278350, SGI278352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64329[] = {SGI278353, SGI278355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64330[] = {SGI278354, SGI278356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64331[] = {SGI278357, SGI278358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64332[] = {SGI278359, SGI278360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64333[] = {SGI278361, SGI278362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64334[] = {SGI278363, SGI278364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64335[] = {SGI278365, SGI278366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64336[] = {SGI278367, SGI278368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64337[] = {SGI278369, SGI278370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64338[] = {SGI278371, SGI278372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64339[] = {SGI278373, SGI278374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64340[] = {SGI278375, SGI278376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64341[] = {SGI278377, SGI278378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64342[] = {SGI278379, SGI278380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64343[] = {SGI278381, SGI278382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64344[] = {SGI278383, SGI278384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64345[] = {SGI278385, SGI278386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64346[] = {SGI278387, SGI278388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64347[] = {SGI278389, SGI278390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64348[] = {SGI278391, SGI278392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64349[] = {SGI278393, SGI278394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64350[] = {SGI278395, SGI278396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64351[] = {SGI278397, SGI278398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64352[] = {SGI278399, SGI278400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64353[] = {SGI278401, SGI278402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64354[] = {SGI278403, SGI278404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64355[] = {SGI278405, SGI278406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64356[] = {SGI278407, SGI278408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64357[] = {SGI278409, SGI278410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64358[] = {SGI278411, SGI278412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64359[] = {SGI278413, SGI278414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64360[] = {SGI278415, SGI278416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64361[] = {SGI278417, SGI278418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64362[] = {SGI278419, SGI278420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64363[] = {SGI278421, SGI278422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64364[] = {SGI278423, SGI278424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64365[] = {SGI278425, SGI278426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64366[] = {SGI278427, SGI278428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64367[] = {SGI278429, SGI278430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64368[] = {SGI278431, SGI278432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64369[] = {SGI278433, SGI278434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64370[] = {SGI278435, SGI278436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64371[] = {SGI278437, SGI278438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64372[] = {SGI278439, SGI278440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64373[] = {SGI278441, SGI278442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64374[] = {SGI278443, SGI278444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64375[] = {SGI278445, SGI278446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64376[] = {SGI278447, SGI278448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64377[] = {SGI278449, SGI278450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64378[] = {SGI278451, SGI278452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64379[] = {SGI278453, SGI278454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64380[] = {SGI278455, SGI278456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64381[] = {SGI278457, SGI278458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64382[] = {SGI278459, SGI278460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64383[] = {SGI278461, SGI278462, NULL};
const Segment * const SG64384[] = {SGI278463, SGI278464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64385[] = {SGI278465, SGI278466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64386[] = {SGI278467, SGI278468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64387[] = {SGI278469, SGI278470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64388[] = {SGI278471, SGI278472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64389[] = {SGI278473, SGI278474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64390[] = {SGI278475, SGI278476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64391[] = {SGI278477, SGI278478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64392[] = {SGI278479, SGI278480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64393[] = {SGI278481, SGI278482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64394[] = {SGI278483, SGI278484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64395[] = {SGI278485, SGI278486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64396[] = {SGI278487, SGI278488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64397[] = {SGI278489, SGI278490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64398[] = {SGI278491, SGI278492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64399[] = {SGI278493, SGI278494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64400[] = {SGI278495, SGI278496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64401[] = {SGI278497, SGI278498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64402[] = {SGI278499, SGI278500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64403[] = {SGI278501, SGI278502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278503, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64404[] = {SGI278504, SGI278505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278506, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64405[] = {SGI278507, SGI278508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278509, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64406[] = {SGI278510, SGI278511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278512, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64407[] = {SGI278513, SGI278514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64408[] = {SGI278515, SGI278516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64409[] = {SGI278517, SGI278518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64410[] = {SGI278519, SGI278520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64411[] = {SGI278521, SGI278522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64412[] = {SGI278523, SGI278524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64413[] = {SGI278525, SGI278526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64414[] = {SGI278527, SGI278528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64415[] = {SGI278529, SGI278530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64416[] = {SGI278531, SGI278532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64417[] = {SGI278533, SGI278534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64418[] = {SGI278535, SGI278536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64419[] = {SGI278537, SGI278538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64420[] = {SGI278539, SGI278540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64421[] = {SGI278541, SGI278542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64422[] = {SGI278543, SGI278544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64423[] = {SGI278545, SGI278546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64424[] = {SGI278547, SGI278548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64425[] = {SGI278549, SGI278550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64426[] = {SGI278551, SGI278552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64427[] = {SGI278553, SGI278554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64428[] = {SGI278555, SGI278556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64429[] = {SGI278557, SGI278558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64430[] = {SGI278559, SGI278560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64431[] = {SGI278561, SGI278562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278563, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64432[] = {SGI278564, SGI278565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278566, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64433[] = {SGI278567, SGI278568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278569, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64434[] = {SGI278570, SGI278571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278572, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64435[] = {SGI278573, SGI278574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278575, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64436[] = {SGI278576, SGI278577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278578, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64437[] = {SGI278579, SGI278580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278581, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64438[] = {SGI278582, SGI278583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64439[] = {SGI278584, SGI278585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64440[] = {SGI278586, SGI278587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64441[] = {SGI278588, SGI278589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64442[] = {SGI278590, SGI278591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64443[] = {SGI278592, SGI278593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64444[] = {SGI278594, SGI278595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64445[] = {SGI278596, SGI278597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64446[] = {SGI278598, SGI278599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64447[] = {SGI278600, SGI278601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64448[] = {SGI278602, SGI278603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64449[] = {SGI278604, SGI278605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64450[] = {SGI278606, SGI278607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64451[] = {SGI278608, SGI278609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64452[] = {SGI278610, SGI278611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64453[] = {SGI278612, SGI278614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64454[] = {SGI0, SGI278613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64455[] = {SGI278615, SGI278616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64456[] = {SGI278617, SGI278618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64457[] = {SGI278619, SGI278620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64458[] = {SGI278621, SGI278622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64459[] = {SGI278623, SGI278624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64460[] = {SGI278625, SGI278626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64461[] = {SGI278627, SGI278628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64462[] = {SGI278629, SGI278630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64463[] = {SGI278631, SGI278632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64464[] = {SGI278633, SGI278634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64465[] = {SGI278635, SGI278636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64466[] = {SGI278637, SGI278638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64467[] = {SGI278639, SGI278640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64468[] = {SGI278641, SGI278642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64469[] = {SGI278643, SGI278644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64470[] = {SGI278645, SGI278646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64471[] = {SGI278647, SGI278648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64472[] = {SGI278649, SGI278650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64473[] = {SGI278651, SGI278652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64474[] = {SGI278653, SGI278654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64475[] = {SGI278655, SGI278656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64476[] = {SGI278657, SGI278658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64477[] = {SGI278659, SGI278660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64478[] = {SGI278661, SGI278662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64479[] = {SGI278663, SGI278664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64480[] = {SGI278665, SGI278666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278667, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64481[] = {SGI278668, SGI278669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278670, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64482[] = {SGI278671, SGI278672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278673, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64483[] = {SGI278674, SGI278675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278676, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64484[] = {SGI278677, SGI278678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278679, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64485[] = {SGI278680, SGI278681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278682, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64486[] = {SGI278683, SGI278684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278685, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64487[] = {SGI278686, SGI278687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64488[] = {SGI0, SGI278688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64489[] = {SGI0, SGI278689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64490[] = {SGI278690, SGI278691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64491[] = {SGI278692, SGI278693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64492[] = {SGI0, SGI278694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64493[] = {SGI278695, SGI278696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64494[] = {SGI0, SGI278697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64495[] = {SGI278698, SGI278699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64496[] = {SGI278700, SGI278701, NULL};
const Segment * const SG64497[] = {SGI278702, SGI278703, SGI0, SGI278704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64498[] = {SGI278705, SGI278706, SGI0, SGI278707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64499[] = {SGI278708, SGI278709, SGI0, SGI278710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64500[] = {SGI278711, SGI278712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278713, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64501[] = {SGI278714, SGI278715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278716, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64502[] = {SGI278717, SGI278718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278719, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64503[] = {SGI278720, SGI278721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278722, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64504[] = {SGI278723, SGI278724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278725, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64505[] = {SGI278726, SGI278727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278728, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64506[] = {SGI278729, SGI278730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278731, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64507[] = {SGI278732, SGI278733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278734, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64508[] = {SGI278735, SGI278736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278737, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64509[] = {SGI278738, SGI278739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278740, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64510[] = {SGI278741, SGI278742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278743, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64511[] = {SGI278744, SGI278745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278746, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64512[] = {SGI278747, SGI278748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278749, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64513[] = {SGI278750, SGI278751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278752, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64514[] = {SGI278753, SGI278754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278755, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64515[] = {SGI278756, SGI278757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278758, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64516[] = {SGI278759, SGI278760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278761, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64517[] = {SGI278762, SGI278763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278764, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64518[] = {SGI278765, SGI278766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278767, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64519[] = {SGI278768, SGI278769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278770, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64520[] = {SGI278771, SGI278772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278773, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64521[] = {SGI278774, SGI278775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278776, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64522[] = {SGI278777, SGI278778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278779, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64523[] = {SGI278780, SGI278781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278782, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64524[] = {SGI278783, SGI278784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278785, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64525[] = {SGI278786, SGI278787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278788, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64526[] = {SGI278789, SGI278790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278791, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64527[] = {SGI278792, SGI278793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278794, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64528[] = {SGI278795, SGI278796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278797, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64529[] = {SGI278798, SGI278799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278800, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64530[] = {SGI278801, SGI278802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278803, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64531[] = {SGI278804, SGI278805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278806, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64532[] = {SGI278807, SGI278808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278809, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64533[] = {SGI278810, SGI278811, NULL};
const Segment * const SG64534[] = {SGI278812, SGI278813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64535[] = {SGI278814, SGI278815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64536[] = {SGI278816, SGI278817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64537[] = {SGI278818, SGI278819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64538[] = {SGI278820, SGI278821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64539[] = {SGI278822, SGI278823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64540[] = {SGI278824, SGI278825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64541[] = {SGI278826, SGI278827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64542[] = {SGI278828, SGI278829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64543[] = {SGI278830, SGI278831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64544[] = {SGI278832, SGI278833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64545[] = {SGI278834, SGI278835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64546[] = {SGI278836, SGI278837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64547[] = {SGI278838, SGI278839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64548[] = {SGI278840, SGI278841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64549[] = {SGI278842, SGI278843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64550[] = {SGI278844, SGI278845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64551[] = {SGI278846, SGI278847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64552[] = {SGI278848, SGI278849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64553[] = {SGI278850, SGI278851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64554[] = {SGI278852, SGI278853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64555[] = {SGI278854, SGI278855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64556[] = {SGI278856, SGI278857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64557[] = {SGI278858, SGI278859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64558[] = {SGI278860, SGI278861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64559[] = {SGI278862, SGI278863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64560[] = {SGI278864, SGI278865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64561[] = {SGI278866, SGI278867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64562[] = {SGI278868, SGI278869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64563[] = {SGI278870, SGI278871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64564[] = {SGI278872, SGI278873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64565[] = {SGI278874, SGI278875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64566[] = {SGI278876, SGI278877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64567[] = {SGI278878, SGI278879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64568[] = {SGI278880, SGI278881, SGI278882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64569[] = {SGI278883, SGI278884, SGI278885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64570[] = {SGI278886, SGI278887, SGI278888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64571[] = {SGI278889, SGI278890, SGI278891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64572[] = {SGI278892, SGI278893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64573[] = {SGI278894, SGI278895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64574[] = {SGI278896, SGI278897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64575[] = {SGI278898, SGI278899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64576[] = {SGI278900, SGI278901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64577[] = {SGI278902, SGI278903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64578[] = {SGI278904, SGI278905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64579[] = {SGI278906, SGI278907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64580[] = {SGI278908, SGI278909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64581[] = {SGI278910, SGI278911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64582[] = {SGI278912, SGI278913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64583[] = {SGI278914, SGI278915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64584[] = {SGI278916, SGI278917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64585[] = {SGI278918, SGI278919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64586[] = {SGI278920, SGI278921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64587[] = {SGI278922, SGI278923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64588[] = {SGI278924, SGI278925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64589[] = {SGI278926, SGI278927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64590[] = {SGI278928, SGI278929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64591[] = {SGI278930, SGI278931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64592[] = {SGI278932, SGI278933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64593[] = {SGI278934, SGI278935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64594[] = {SGI278936, SGI278937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64595[] = {SGI278938, SGI278939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64596[] = {SGI278940, SGI278941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64597[] = {SGI278942, SGI278943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278944, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64598[] = {SGI278945, SGI278946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278947, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64599[] = {SGI278948, SGI278949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278950, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64600[] = {SGI278951, SGI278952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278953, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64601[] = {SGI278954, SGI278955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278956, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64602[] = {SGI278957, SGI278958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278959, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64603[] = {SGI278960, SGI278961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278962, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64604[] = {SGI278963, SGI278964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278965, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64605[] = {SGI278966, SGI278967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278968, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64606[] = {SGI278969, SGI278970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278971, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64607[] = {SGI278972, SGI278973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278974, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64608[] = {SGI278975, SGI278976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278977, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64609[] = {SGI278978, SGI278979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278980, NULL};
const Segment * const SG64610[] = {SGI278981, SGI278982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278983, NULL};
const Segment * const SG64611[] = {SGI278984, SGI278985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278986, NULL};
const Segment * const SG64612[] = {SGI278987, SGI278988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI278989, NULL};
const Segment * const SG64613[] = {SGI278990, SGI278991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64614[] = {SGI278992, SGI278993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64615[] = {SGI278994, SGI278995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64616[] = {SGI278996, SGI278997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64617[] = {SGI278998, SGI278999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64618[] = {SGI279000, SGI279001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64619[] = {SGI279002, SGI279003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64620[] = {SGI279004, SGI279005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64621[] = {SGI279006, SGI279007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64622[] = {SGI279008, SGI279009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64623[] = {SGI279010, SGI279011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64624[] = {SGI279012, SGI279013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64625[] = {SGI279014, SGI279015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64626[] = {SGI279016, SGI279017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64627[] = {SGI279018, SGI279019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64628[] = {SGI279020, SGI279021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64629[] = {SGI279022, SGI279023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64630[] = {SGI279024, SGI279025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64631[] = {SGI279026, SGI279027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279028, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64632[] = {SGI279029, SGI279030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279031, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64633[] = {SGI279032, SGI279033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279034, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64634[] = {SGI279035, SGI279036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279037, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64635[] = {SGI279038, SGI279039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279040, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64636[] = {SGI279041, SGI279042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279043, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64637[] = {SGI279044, SGI279045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279046, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64638[] = {SGI279047, SGI279048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279049, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64639[] = {SGI279050, SGI279051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279052, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64640[] = {SGI279053, SGI279054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279055, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64641[] = {SGI279056, SGI279057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279058, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64642[] = {SGI279059, SGI279060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279061, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64643[] = {SGI279062, SGI279063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279064, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64644[] = {SGI279065, SGI279066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279067, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64645[] = {SGI279068, SGI279069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279070, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64646[] = {SGI279071, SGI279072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279073, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64647[] = {SGI279074, SGI279075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279076, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64648[] = {SGI279077, SGI279078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279079, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64649[] = {SGI279080, SGI279081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279082, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64650[] = {SGI279083, SGI279084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279085, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64651[] = {SGI279086, SGI279087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279088, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64652[] = {SGI279089, SGI279090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279091, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64653[] = {SGI279092, SGI279093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279094, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64654[] = {SGI279095, SGI279096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279097, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64655[] = {SGI279098, SGI279099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279100, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64656[] = {SGI279101, SGI279102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279103, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64657[] = {SGI279104, SGI279105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279106, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64658[] = {SGI279107, SGI279108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279109, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64659[] = {SGI279110, SGI279111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64660[] = {SGI279112, SGI279113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64661[] = {SGI279114, SGI279115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64662[] = {SGI279116, SGI279117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64663[] = {SGI279118, SGI279119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64664[] = {SGI279120, SGI279121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279122, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64665[] = {SGI279123, SGI279124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64666[] = {SGI279125, SGI279126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64667[] = {SGI279127, SGI279128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279129, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64668[] = {SGI279130, SGI279131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64669[] = {SGI279132, SGI279133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64670[] = {SGI279134, SGI279135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64671[] = {SGI279136, SGI279137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64672[] = {SGI279138, SGI279139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64673[] = {SGI279140, SGI279141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64674[] = {SGI279142, SGI279143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64675[] = {SGI279144, SGI279145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64676[] = {SGI279146, SGI279147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279148, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64677[] = {SGI279149, SGI279150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279151, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64678[] = {SGI279152, SGI279153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279154, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64679[] = {SGI279155, SGI279156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279157, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64680[] = {SGI279158, SGI279159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279160, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64681[] = {SGI279161, SGI279162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279163, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64682[] = {SGI279164, SGI279165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279166, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64683[] = {SGI279167, SGI279168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279169, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64684[] = {SGI279170, SGI279171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279172, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64685[] = {SGI279173, SGI279174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279175, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64686[] = {SGI279176, SGI279177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279178, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64687[] = {SGI279179, SGI279180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279181, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64688[] = {SGI279182, SGI279183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64689[] = {SGI279184, SGI279185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64690[] = {SGI279186, SGI279187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64691[] = {SGI279188, SGI279189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64692[] = {SGI279190, SGI279191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64693[] = {SGI279192, SGI279193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64694[] = {SGI279194, SGI279195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279196, SGI0, SGI0, NULL};
const Segment * const SG64695[] = {SGI279197, SGI279198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64696[] = {SGI279199, SGI279200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64697[] = {SGI279201, SGI279202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64698[] = {SGI279203, SGI279204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64699[] = {SGI279205, SGI279206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64700[] = {SGI279207, SGI279208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64701[] = {SGI279209, SGI279210, NULL};
const Segment * const SG64702[] = {SGI279211, SGI279212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64703[] = {SGI279213, SGI279214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64704[] = {SGI279215, SGI279216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64705[] = {SGI279217, SGI279218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64706[] = {SGI279219, SGI279220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64707[] = {SGI279221, SGI279222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64708[] = {SGI279223, SGI279224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279225, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64709[] = {SGI279226, SGI279227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279228, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64710[] = {SGI279229, SGI279230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279231, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64711[] = {SGI279232, SGI279233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279234, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64712[] = {SGI279235, SGI279236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279237, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64713[] = {SGI279238, SGI279239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279240, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64714[] = {SGI279241, SGI279242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279243, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64715[] = {SGI279244, SGI279245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279246, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64716[] = {SGI279247, SGI279248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279249, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64717[] = {SGI279250, SGI279251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279252, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64718[] = {SGI279253, SGI279254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279255, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64719[] = {SGI279256, SGI279257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64720[] = {SGI279258, SGI279259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279260, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64721[] = {SGI279261, SGI279262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64722[] = {SGI279263, SGI279264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279265, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64723[] = {SGI279266, SGI279267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64724[] = {SGI279268, SGI279269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279270, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64725[] = {SGI279271, SGI279272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64726[] = {SGI279273, SGI279274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279275, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64727[] = {SGI279276, SGI279277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64728[] = {SGI279278, SGI279279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279280, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64729[] = {SGI279281, SGI279282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64730[] = {SGI279283, SGI279284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64731[] = {SGI279285, SGI279286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64732[] = {SGI279287, SGI279288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64733[] = {SGI279289, SGI279290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64734[] = {SGI279291, SGI279292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64735[] = {SGI279293, SGI279294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64736[] = {SGI279295, SGI279296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64737[] = {SGI279297, SGI279298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64738[] = {SGI279299, SGI279300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64739[] = {SGI279301, SGI279302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64740[] = {SGI279303, SGI279304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64741[] = {SGI279305, SGI279306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64742[] = {SGI279307, SGI279308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64743[] = {SGI279309, SGI279310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64744[] = {SGI279311, SGI279312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279313, SGI0, SGI0, NULL};
const Segment * const SG64745[] = {SGI279314, SGI279315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279316, SGI0, SGI0, NULL};
const Segment * const SG64746[] = {SGI279317, SGI279318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279319, SGI0, SGI0, NULL};
const Segment * const SG64747[] = {SGI279320, SGI279321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279322, SGI0, SGI0, NULL};
const Segment * const SG64748[] = {SGI279323, SGI279324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279325, SGI0, SGI0, NULL};
const Segment * const SG64749[] = {SGI279326, SGI279327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279328, SGI0, SGI0, NULL};
const Segment * const SG64750[] = {SGI279329, SGI279330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279331, SGI0, SGI0, NULL};
const Segment * const SG64751[] = {SGI279332, SGI279333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279334, SGI0, SGI0, NULL};
const Segment * const SG64752[] = {SGI279335, SGI279336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279337, SGI0, SGI0, NULL};
const Segment * const SG64753[] = {SGI279338, SGI279339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279340, SGI0, SGI0, NULL};
const Segment * const SG64754[] = {SGI279341, SGI279342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64755[] = {SGI279344, SGI279345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64756[] = {SGI279347, SGI279348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64757[] = {SGI279350, SGI279351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64758[] = {SGI279353, SGI279354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64759[] = {SGI279356, SGI279357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64760[] = {SGI279358, SGI279359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64761[] = {SGI279360, SGI279361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64762[] = {SGI279362, SGI279363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64763[] = {SGI279364, SGI279365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64764[] = {SGI279366, SGI279367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64765[] = {SGI279368, SGI279369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64766[] = {SGI279370, SGI279371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64767[] = {SGI279372, SGI279373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64768[] = {SGI279374, SGI279375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64769[] = {SGI279376, SGI279377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64770[] = {SGI279378, SGI279379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64771[] = {SGI279380, SGI279381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64772[] = {SGI279382, SGI279383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64773[] = {SGI279384, SGI279385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64774[] = {SGI279386, SGI279387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64775[] = {SGI279388, SGI279389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64776[] = {SGI279390, SGI279391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64777[] = {SGI279392, SGI279393, SGI0, SGI279394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64778[] = {SGI279395, SGI279396, SGI0, SGI279397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64779[] = {SGI279398, SGI279399, SGI0, SGI279400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64780[] = {SGI279401, SGI279402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64781[] = {SGI279403, SGI279404, SGI0, SGI279405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64782[] = {SGI279406, SGI279407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64783[] = {SGI279408, SGI279409, SGI0, SGI279410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64784[] = {SGI279411, SGI279412, SGI0, SGI279413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64785[] = {SGI279414, SGI279415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279416, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64786[] = {SGI279417, SGI279418, SGI0, SGI279419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64787[] = {SGI279420, SGI279421, SGI0, SGI279422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64788[] = {SGI279423, SGI279424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279425, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64789[] = {SGI279426, SGI279427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279428, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64790[] = {SGI279429, SGI279430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279431, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64791[] = {SGI279432, SGI279433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279434, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64792[] = {SGI279435, SGI279436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279437, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64793[] = {SGI279438, SGI279439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279440, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64794[] = {SGI279441, SGI279442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279443, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64795[] = {SGI279444, SGI279445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279446, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64796[] = {SGI279447, SGI279448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279449, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64797[] = {SGI279450, SGI279451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279452, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64798[] = {SGI279453, SGI279454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279455, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64799[] = {SGI279456, SGI279457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279458, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64800[] = {SGI279459, SGI279460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64801[] = {SGI279461, SGI279462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64802[] = {SGI279463, SGI279464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64803[] = {SGI279465, SGI279466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64804[] = {SGI279467, SGI279468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64805[] = {SGI279469, SGI279470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64806[] = {SGI279471, SGI279472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64807[] = {SGI279473, SGI279474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64808[] = {SGI279475, SGI279476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64809[] = {SGI279477, SGI279478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64810[] = {SGI279479, SGI279480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64811[] = {SGI279481, SGI279482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64812[] = {SGI279483, SGI279484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64813[] = {SGI279485, SGI279486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64814[] = {SGI279487, SGI279488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279489, SGI0, NULL};
const Segment * const SG64815[] = {SGI279490, SGI279491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64816[] = {SGI279492, SGI279493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64817[] = {SGI279494, SGI279495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64818[] = {SGI279496, SGI279497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64819[] = {SGI279498, SGI279499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64820[] = {SGI279500, SGI279501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64821[] = {SGI279502, SGI279503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64822[] = {SGI279504, SGI279505, SGI0, SGI279506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64823[] = {SGI279507, SGI279508, SGI0, SGI279509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64824[] = {SGI279510, SGI279511, SGI0, SGI279512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64825[] = {SGI279513, SGI279514, SGI0, SGI279515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64826[] = {SGI279516, SGI279517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64827[] = {SGI279518, SGI279519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64828[] = {SGI279520, SGI279521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64829[] = {SGI279522, SGI279523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64830[] = {SGI279524, SGI279525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64831[] = {SGI279526, SGI279527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64832[] = {SGI279528, SGI279529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64833[] = {SGI279530, SGI279531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64834[] = {SGI279532, SGI279533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64835[] = {SGI279534, SGI279535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64836[] = {SGI279536, SGI279537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64837[] = {SGI279538, SGI279539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64838[] = {SGI279540, SGI279541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64839[] = {SGI279542, SGI279543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64840[] = {SGI279544, SGI279545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64841[] = {SGI279546, SGI279547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64842[] = {SGI279548, SGI279549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64843[] = {SGI279550, SGI279551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64844[] = {SGI279552, SGI279553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64845[] = {SGI279554, SGI279555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64846[] = {SGI279556, SGI279557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64847[] = {SGI279558, SGI279559, SGI0, SGI279560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64848[] = {SGI279561, SGI279562, SGI0, SGI279563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64849[] = {SGI279564, SGI279565, SGI0, SGI279566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64850[] = {SGI279567, SGI279568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64851[] = {SGI279569, SGI279570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64852[] = {SGI279571, SGI279572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64853[] = {SGI279573, SGI279574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64854[] = {SGI279575, SGI279576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64855[] = {SGI279577, SGI279578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64856[] = {SGI279579, SGI279580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64857[] = {SGI279581, SGI279582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64858[] = {SGI279583, SGI279584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64859[] = {SGI279585, SGI279586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64860[] = {SGI279587, SGI279588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64861[] = {SGI279589, SGI279590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64862[] = {SGI279591, SGI279592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64863[] = {SGI279593, SGI279594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64864[] = {SGI279595, SGI279596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64865[] = {SGI279597, SGI279598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64866[] = {SGI279599, SGI279600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64867[] = {SGI279601, SGI279602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64868[] = {SGI279603, SGI279604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64869[] = {SGI279605, SGI279606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64870[] = {SGI279607, SGI279608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64871[] = {SGI279609, SGI279610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64872[] = {SGI279611, SGI279612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64873[] = {SGI279613, SGI279614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64874[] = {SGI279615, SGI279616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64875[] = {SGI279617, SGI279618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64876[] = {SGI279619, SGI279620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64877[] = {SGI279621, SGI279622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64878[] = {SGI279623, SGI279624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64879[] = {SGI279625, SGI279626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64880[] = {SGI279627, SGI279628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64881[] = {SGI279629, SGI279630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64882[] = {SGI279631, SGI279632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64883[] = {SGI279633, SGI279634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64884[] = {SGI279635, SGI279636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64885[] = {SGI279637, SGI279638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64886[] = {SGI279639, SGI279640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64887[] = {SGI279641, SGI279642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64888[] = {SGI279643, SGI279644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64889[] = {SGI279645, SGI279646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64890[] = {SGI279647, SGI279648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64891[] = {SGI279649, SGI279650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64892[] = {SGI279651, SGI279652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64893[] = {SGI279653, SGI279654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64894[] = {SGI279655, SGI279656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64895[] = {SGI279657, SGI279658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64896[] = {SGI279659, SGI279660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64897[] = {SGI279661, SGI279662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64898[] = {SGI279663, SGI279664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64899[] = {SGI0, SGI279665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64900[] = {SGI279666, SGI279667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64901[] = {SGI279668, SGI279669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64902[] = {SGI279670, SGI279671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64903[] = {SGI279672, SGI279673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64904[] = {SGI279674, SGI279675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64905[] = {SGI279676, SGI279677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64906[] = {SGI279678, SGI279679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64907[] = {SGI279680, SGI279681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64908[] = {SGI279682, SGI279683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64909[] = {SGI279684, SGI279685, SGI0, SGI279686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64910[] = {SGI279687, SGI279688, SGI0, SGI279689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64911[] = {SGI279690, SGI279691, SGI0, SGI279692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64912[] = {SGI279693, SGI279694, SGI0, SGI279695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64913[] = {SGI279696, SGI279697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64914[] = {SGI279698, SGI279699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64915[] = {SGI279700, SGI279701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64916[] = {SGI279702, SGI279703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64917[] = {SGI279704, SGI279705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64918[] = {SGI279706, SGI279707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64919[] = {SGI279708, SGI279709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64920[] = {SGI279710, SGI279711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64921[] = {SGI279712, SGI279713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64922[] = {SGI279714, SGI279715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64923[] = {SGI279716, SGI279717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64924[] = {SGI279718, SGI279719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64925[] = {SGI279720, SGI279721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64926[] = {SGI279722, SGI279723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64927[] = {SGI279724, SGI279725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64928[] = {SGI279726, SGI279727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64929[] = {SGI279728, SGI279729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64930[] = {SGI279730, SGI279731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64931[] = {SGI279732, SGI279733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64932[] = {SGI279734, SGI279735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64933[] = {SGI279736, SGI279737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64934[] = {SGI279738, SGI279739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64935[] = {SGI279741, SGI279742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64936[] = {SGI279744, SGI279745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64937[] = {SGI279747, SGI279748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64938[] = {SGI279750, SGI279751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64939[] = {SGI279753, SGI279754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64940[] = {SGI279756, SGI279757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279758, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64941[] = {SGI279759, SGI279760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279761, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64942[] = {SGI279762, SGI279764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64943[] = {SGI279763, SGI279765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279766, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64944[] = {SGI279767, SGI279768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64945[] = {SGI279769, SGI279770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279771, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64946[] = {SGI279772, SGI279773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64947[] = {SGI279774, SGI279775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279777, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64948[] = {SGI279776, SGI279778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64949[] = {SGI279779, SGI279780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64950[] = {SGI279781, SGI279782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64951[] = {SGI279783, SGI279784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64952[] = {SGI279785, SGI279786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64953[] = {SGI279787, SGI279788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64954[] = {SGI279789, SGI279790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279791, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64955[] = {SGI279792, SGI279793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279794, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64956[] = {SGI279795, SGI279796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279797, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64957[] = {SGI279798, SGI279799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279800, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64958[] = {SGI279801, SGI279802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279803, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64959[] = {SGI279804, SGI279805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279806, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64960[] = {SGI279807, SGI279808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279809, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64961[] = {SGI279810, SGI279811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279812, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64962[] = {SGI279813, SGI279814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279815, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64963[] = {SGI279816, SGI279817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279818, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64964[] = {SGI279819, SGI279820, SGI0, SGI0, SGI0, SGI0, SGI279821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64965[] = {SGI279822, SGI279823, SGI0, SGI0, SGI0, SGI0, SGI279824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64966[] = {SGI279825, SGI279826, SGI0, SGI0, SGI0, SGI0, SGI279827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64967[] = {SGI279828, SGI279829, SGI0, SGI0, SGI0, SGI0, SGI279830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64968[] = {SGI279831, SGI279832, SGI0, SGI0, SGI0, SGI0, SGI279833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64969[] = {SGI279834, SGI279835, SGI0, SGI0, SGI0, SGI0, SGI279836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64970[] = {SGI279837, SGI279838, SGI0, SGI0, SGI0, SGI0, SGI279839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64971[] = {SGI279840, SGI279841, SGI0, SGI0, SGI0, SGI0, SGI279842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64972[] = {SGI279843, SGI279844, SGI0, SGI0, SGI0, SGI0, SGI279845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64973[] = {SGI279846, SGI279847, SGI0, SGI0, SGI0, SGI0, SGI279848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64974[] = {SGI279849, SGI279850, SGI0, SGI0, SGI0, SGI0, SGI279851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64975[] = {SGI279852, SGI279853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64976[] = {SGI279855, SGI279856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64977[] = {SGI279858, SGI279859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64978[] = {SGI279861, SGI279862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64979[] = {SGI279863, SGI279864, SGI0, SGI279865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64980[] = {SGI279866, SGI279867, SGI0, SGI279868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64981[] = {SGI279869, SGI279870, SGI0, SGI279871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64982[] = {SGI279872, SGI279873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64983[] = {SGI279874, SGI279875, SGI0, SGI279876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64984[] = {SGI279877, SGI279878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64985[] = {SGI279879, SGI279880, SGI0, SGI279881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64986[] = {SGI279882, SGI279883, SGI0, SGI279884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64987[] = {SGI279885, SGI279886, SGI0, SGI279887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64988[] = {SGI279888, SGI279889, SGI0, SGI279890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64989[] = {SGI279891, SGI279892, SGI0, SGI279893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64990[] = {SGI279894, SGI279895, SGI0, SGI279896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64991[] = {SGI279897, SGI279898, SGI0, SGI279899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64992[] = {SGI279900, SGI279901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64993[] = {SGI279902, SGI279903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64994[] = {SGI279904, SGI279905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64995[] = {SGI279906, SGI279907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64996[] = {SGI279908, SGI279909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64997[] = {SGI279910, SGI279911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64998[] = {SGI279912, SGI279913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG64999[] = {SGI279914, SGI279915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65000[] = {SGI279916, SGI279917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65001[] = {SGI279918, SGI279919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65002[] = {SGI279920, SGI279921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65003[] = {SGI279922, SGI279923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65004[] = {SGI279924, SGI279925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65005[] = {SGI279926, SGI279927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65006[] = {SGI279928, SGI279929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65007[] = {SGI279930, SGI279931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65008[] = {SGI279932, SGI279933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65009[] = {SGI279934, SGI279935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65010[] = {SGI279936, SGI279937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279938, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65011[] = {SGI279939, SGI279940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279941, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65012[] = {SGI279942, SGI279943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279944, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65013[] = {SGI279945, SGI279946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279947, SGI0, NULL};
const Segment * const SG65014[] = {SGI279948, SGI279949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279950, SGI0, NULL};
const Segment * const SG65015[] = {SGI279951, SGI279952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65016[] = {SGI279953, SGI279954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65017[] = {SGI279955, SGI279956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65018[] = {SGI279957, SGI279958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65019[] = {SGI279959, SGI279960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65020[] = {SGI279961, SGI279962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65021[] = {SGI279963, SGI279964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65022[] = {SGI279965, SGI279966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65023[] = {SGI279967, SGI279968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65024[] = {SGI279969, SGI279970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65025[] = {SGI279971, SGI279972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65026[] = {SGI279973, SGI279974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65027[] = {SGI279975, SGI279976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65028[] = {SGI279977, SGI279978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65029[] = {SGI279979, SGI279980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65030[] = {SGI279981, SGI279982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65031[] = {SGI279983, SGI279984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65032[] = {SGI279985, SGI279986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65033[] = {SGI279987, SGI279988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65034[] = {SGI279989, SGI279990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65035[] = {SGI279991, SGI279992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65036[] = {SGI279993, SGI279994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279995, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65037[] = {SGI279996, SGI279997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI279998, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65038[] = {SGI279999, SGI280000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280001, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65039[] = {SGI280002, SGI280003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280004, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65040[] = {SGI280005, SGI280006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280007, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65041[] = {SGI280008, SGI280009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280010, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65042[] = {SGI280011, SGI280012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65043[] = {SGI280013, SGI280014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65044[] = {SGI280015, SGI280016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65045[] = {SGI280017, SGI280018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65046[] = {SGI280019, SGI280020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65047[] = {SGI280021, SGI280022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65048[] = {SGI0, SGI280023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65049[] = {SGI0, SGI280024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65050[] = {SGI0, SGI280025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65051[] = {SGI0, SGI280026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65052[] = {SGI0, SGI280027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65053[] = {SGI0, SGI280028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65054[] = {SGI0, SGI280029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65055[] = {SGI280030, SGI280031, SGI0, SGI280032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65056[] = {SGI280033, SGI280034, SGI0, SGI280035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65057[] = {SGI280036, SGI280037, SGI0, SGI280038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65058[] = {SGI280039, SGI280040, SGI0, SGI280041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65059[] = {SGI280042, SGI280043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280044, SGI0, SGI0, NULL};
const Segment * const SG65060[] = {SGI280045, SGI280046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280047, SGI0, SGI0, NULL};
const Segment * const SG65061[] = {SGI280048, SGI280049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280050, SGI0, SGI0, NULL};
const Segment * const SG65062[] = {SGI280051, SGI280052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280053, SGI0, SGI0, NULL};
const Segment * const SG65063[] = {SGI280054, SGI280055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280056, SGI0, SGI0, NULL};
const Segment * const SG65064[] = {SGI280057, SGI280058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280059, SGI0, SGI0, NULL};
const Segment * const SG65065[] = {SGI280060, SGI280061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280062, SGI0, SGI0, NULL};
const Segment * const SG65066[] = {SGI280063, SGI280064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280065, SGI0, SGI0, NULL};
const Segment * const SG65067[] = {SGI280066, SGI280067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280068, SGI0, SGI0, NULL};
const Segment * const SG65068[] = {SGI280069, SGI280070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280071, SGI0, SGI0, NULL};
const Segment * const SG65069[] = {SGI280072, SGI280073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280074, SGI0, SGI0, NULL};
const Segment * const SG65070[] = {SGI280075, SGI280076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280077, SGI0, SGI0, NULL};
const Segment * const SG65071[] = {SGI280078, SGI280079, SGI0, SGI280080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65072[] = {SGI280081, SGI280082, SGI0, SGI280083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65073[] = {SGI280084, SGI280085, SGI0, SGI280086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65074[] = {SGI280087, SGI280088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65075[] = {SGI280089, SGI280090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65076[] = {SGI279211, SGI279212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65077[] = {SGI280091, SGI280092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65078[] = {SGI280093, SGI280094, NULL};
const Segment * const SG65079[] = {SGI280095, SGI280096, SGI0, SGI280097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65080[] = {SGI280098, SGI280099, SGI0, SGI280100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65081[] = {SGI280101, SGI280102, SGI0, SGI280103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65082[] = {SGI280104, SGI280105, SGI0, SGI280106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65083[] = {SGI280107, SGI280108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65084[] = {SGI280109, SGI280110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65085[] = {SGI280111, SGI280112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65086[] = {SGI280113, SGI280114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65087[] = {SGI280115, SGI280116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65088[] = {SGI280117, SGI280118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65089[] = {SGI280119, SGI280120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65090[] = {SGI280121, SGI280122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65091[] = {SGI280123, SGI280124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65092[] = {SGI280125, SGI280126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65093[] = {SGI280127, SGI280128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65094[] = {SGI280129, SGI280130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65095[] = {SGI280131, SGI280132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65096[] = {SGI280133, SGI280134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65097[] = {SGI280135, SGI280136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65098[] = {SGI280137, SGI280138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65099[] = {SGI280139, SGI280140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65100[] = {SGI280141, SGI280142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65101[] = {SGI280143, SGI280144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65102[] = {SGI280145, SGI280146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65103[] = {SGI280147, SGI280148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65104[] = {SGI280149, SGI280150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65105[] = {SGI280151, SGI280152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65106[] = {SGI280153, SGI280154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65107[] = {SGI280155, SGI280156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65108[] = {SGI280157, SGI280158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65109[] = {SGI280159, SGI280160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65110[] = {SGI280161, SGI280162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65111[] = {SGI280163, SGI280164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280165, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65112[] = {SGI280166, SGI280167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280168, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65113[] = {SGI280169, SGI280170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65114[] = {SGI280171, SGI280172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65115[] = {SGI280173, SGI280174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65116[] = {SGI280175, SGI280176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65117[] = {SGI280177, SGI280178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65118[] = {SGI280179, SGI280180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65119[] = {SGI280181, SGI280182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65120[] = {SGI280183, SGI280184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65121[] = {SGI280185, SGI280186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65122[] = {SGI280187, SGI280188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65123[] = {SGI280189, SGI280190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65124[] = {SGI280191, SGI280192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280193, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65125[] = {SGI280194, SGI280195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280196, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65126[] = {SGI280197, SGI280198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280199, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65127[] = {SGI280200, SGI280201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65128[] = {SGI280202, SGI280203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280204, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65129[] = {SGI280205, SGI280206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280207, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65130[] = {SGI280208, SGI280209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280210, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65131[] = {SGI280211, SGI280212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280213, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65132[] = {SGI280214, SGI280215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280216, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65133[] = {SGI280217, SGI280218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280219, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65134[] = {SGI280220, SGI280221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280222, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65135[] = {SGI280223, SGI280224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280225, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65136[] = {SGI280226, SGI280227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280228, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65137[] = {SGI280229, SGI280230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280231, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65138[] = {SGI280232, SGI280233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280234, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65139[] = {SGI280235, SGI280236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280237, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65140[] = {SGI280238, SGI280239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280240, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65141[] = {SGI280241, SGI280242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280243, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65142[] = {SGI280244, SGI280245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65143[] = {SGI280246, SGI280247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65144[] = {SGI280248, SGI280249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65145[] = {SGI280250, SGI280251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65146[] = {SGI280252, SGI280253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65147[] = {SGI280254, SGI280255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65148[] = {SGI280256, SGI280257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65149[] = {SGI280258, SGI280259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65150[] = {SGI280260, SGI280261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65151[] = {SGI280262, SGI280263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65152[] = {SGI280264, SGI280265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65153[] = {SGI280266, SGI280267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65154[] = {SGI280268, SGI280269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280270, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65155[] = {SGI280271, SGI280272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280273, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65156[] = {SGI280274, SGI280275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280276, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65157[] = {SGI280277, SGI280278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280279, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65158[] = {SGI280280, SGI280281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280282, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65159[] = {SGI280283, SGI280284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280285, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65160[] = {SGI280286, SGI280287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280288, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65161[] = {SGI280289, SGI280290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280291, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65162[] = {SGI280292, SGI280293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65163[] = {SGI280294, SGI280295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65164[] = {SGI280296, SGI280297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65165[] = {SGI280298, SGI280299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65166[] = {SGI280300, SGI280301, SGI280302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65167[] = {SGI280303, SGI280304, SGI280305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65168[] = {SGI280306, SGI280307, SGI280308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65169[] = {SGI280309, SGI280310, SGI280311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65170[] = {SGI280312, SGI280313, SGI280314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65171[] = {SGI280315, SGI280316, SGI280317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65172[] = {SGI280318, SGI280319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65173[] = {SGI280320, SGI280321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65174[] = {SGI280322, SGI280323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65175[] = {SGI280324, SGI280325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65176[] = {SGI280326, SGI280327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65177[] = {SGI280328, SGI280329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65178[] = {SGI280330, SGI280331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65179[] = {SGI280332, SGI280333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65180[] = {SGI280334, SGI280335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65181[] = {SGI280336, SGI280337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65182[] = {SGI280338, SGI280339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65183[] = {SGI280340, SGI280341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65184[] = {SGI280342, SGI280343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65185[] = {SGI280344, SGI280345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65186[] = {SGI280346, SGI280347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65187[] = {SGI280348, SGI280349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65188[] = {SGI280350, SGI280351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65189[] = {SGI280352, SGI280353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65190[] = {SGI280354, SGI280355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65191[] = {SGI280356, SGI280357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65192[] = {SGI280358, SGI280359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65193[] = {SGI280360, SGI280361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65194[] = {SGI280362, SGI280363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65195[] = {SGI280364, SGI280365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65196[] = {SGI280366, SGI280367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65197[] = {SGI280368, SGI280369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65198[] = {SGI280370, SGI280371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65199[] = {SGI280372, SGI280373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65200[] = {SGI280374, SGI280375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65201[] = {SGI280376, SGI280377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65202[] = {SGI280378, SGI280379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65203[] = {SGI280380, SGI280381, SGI0, SGI280382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65204[] = {SGI280383, SGI280384, SGI0, SGI280385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65205[] = {SGI280386, SGI280387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280388, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65206[] = {SGI280389, SGI280390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280391, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65207[] = {SGI280392, SGI280393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280394, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65208[] = {SGI280395, SGI280396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280397, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65209[] = {SGI280398, SGI280399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280400, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65210[] = {SGI280401, SGI280402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280403, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65211[] = {SGI280404, SGI280405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280406, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65212[] = {SGI280407, SGI280408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280409, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65213[] = {SGI280410, SGI280411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280412, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65214[] = {SGI280413, SGI280414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280415, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65215[] = {SGI280416, SGI280417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280418, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65216[] = {SGI280419, SGI280420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280421, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65217[] = {SGI280422, SGI280423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65218[] = {SGI280424, SGI280425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65219[] = {SGI280426, SGI280427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65220[] = {SGI280428, SGI280429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65221[] = {SGI280430, SGI280431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65222[] = {SGI280432, SGI280433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65223[] = {SGI280434, SGI280435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65224[] = {SGI280436, SGI280437, SGI280438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65225[] = {SGI280439, SGI280440, SGI280441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65226[] = {SGI280442, SGI280443, SGI280444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65227[] = {SGI280445, SGI280446, SGI280447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65228[] = {SGI280448, SGI280449, SGI0, SGI280450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65229[] = {SGI280451, SGI280452, SGI0, SGI280453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65230[] = {SGI280454, SGI280455, SGI0, SGI280456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65231[] = {SGI280457, SGI280458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65232[] = {SGI280459, SGI280460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65233[] = {SGI280461, SGI280462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65234[] = {SGI280463, SGI280464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65235[] = {SGI280465, SGI280466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65236[] = {SGI280467, SGI280468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65237[] = {SGI280469, SGI280470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65238[] = {SGI280471, SGI280472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65239[] = {SGI280473, SGI280474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65240[] = {SGI280475, SGI280476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65241[] = {SGI280477, SGI280478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65242[] = {SGI280479, SGI280480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65243[] = {SGI280481, SGI280482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65244[] = {SGI280483, SGI280484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65245[] = {SGI280485, SGI280486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65246[] = {SGI280487, SGI280488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65247[] = {SGI280489, SGI280490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65248[] = {SGI280491, SGI280492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65249[] = {SGI280493, SGI280494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65250[] = {SGI280495, SGI280496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65251[] = {SGI280497, SGI280498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65252[] = {SGI280499, SGI280500, SGI280501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65253[] = {SGI280502, SGI280503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65254[] = {SGI280504, SGI280505, SGI280506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65255[] = {SGI280507, SGI280508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65256[] = {SGI280509, SGI280510, SGI280511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65257[] = {SGI280512, SGI280513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65258[] = {SGI280514, SGI280515, SGI280516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65259[] = {SGI280517, SGI280518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65260[] = {SGI280519, SGI280520, SGI280521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65261[] = {SGI280522, SGI280523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65262[] = {SGI280524, SGI280525, SGI280526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65263[] = {SGI280527, SGI280528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280529, NULL};
const Segment * const SG65264[] = {SGI280530, SGI280531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280532, NULL};
const Segment * const SG65265[] = {SGI280533, SGI280534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65266[] = {SGI280535, SGI280536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65267[] = {SGI280537, SGI280538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65268[] = {SGI280539, SGI280540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65269[] = {SGI280541, SGI280542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65270[] = {SGI280543, SGI280544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65271[] = {SGI280545, SGI280546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65272[] = {SGI280547, SGI280548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65273[] = {SGI280549, SGI280550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65274[] = {SGI280551, SGI280552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65275[] = {SGI280554, SGI280555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65276[] = {SGI280557, SGI280558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65277[] = {SGI280560, SGI280561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65278[] = {SGI280563, SGI280564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65279[] = {SGI280566, SGI280567, SGI0, SGI280568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65280[] = {SGI280569, SGI280570, SGI0, SGI280571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65281[] = {SGI280572, SGI280573, SGI0, SGI280574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65282[] = {SGI280575, SGI280576, SGI0, SGI280577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65283[] = {SGI280578, SGI280579, SGI0, SGI280580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65284[] = {SGI280581, SGI280582, SGI0, SGI280583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65285[] = {SGI280584, SGI280585, NULL};
const Segment * const SG65286[] = {SGI280586, SGI280587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280588, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65287[] = {SGI280589, SGI280590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280591, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65288[] = {SGI280592, SGI280593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280594, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65289[] = {SGI280595, SGI280596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280597, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65290[] = {SGI280598, SGI280599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280600, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65291[] = {SGI280601, SGI280602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280603, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65292[] = {SGI280604, SGI280605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280606, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65293[] = {SGI280607, SGI280608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280609, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65294[] = {SGI280610, SGI280611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280612, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65295[] = {SGI280613, SGI280614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280615, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65296[] = {SGI280616, SGI280617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65297[] = {SGI280618, SGI280619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65298[] = {SGI280620, SGI280621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65299[] = {SGI280622, SGI280623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65300[] = {SGI280624, SGI280625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65301[] = {SGI280626, SGI280627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65302[] = {SGI280628, SGI280629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65303[] = {SGI280630, SGI280631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65304[] = {SGI280632, SGI280633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65305[] = {SGI280634, SGI280635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65306[] = {SGI280636, SGI280637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65307[] = {SGI280638, SGI280639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65308[] = {SGI280640, SGI280641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280642, SGI0, NULL};
const Segment * const SG65309[] = {SGI280643, SGI280644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65310[] = {SGI280646, SGI280647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65311[] = {SGI280649, SGI280650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280651, SGI0, NULL};
const Segment * const SG65312[] = {SGI280652, SGI280653, SGI280654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65313[] = {SGI280655, SGI280656, SGI280657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65314[] = {SGI280658, SGI280659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280660, SGI0, NULL};
const Segment * const SG65315[] = {SGI280661, SGI280662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65316[] = {SGI280663, SGI280664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280665, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65317[] = {SGI280666, SGI280667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65318[] = {SGI280668, SGI280669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280670, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65319[] = {SGI0, SGI280671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65320[] = {SGI280672, SGI280673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280674, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65321[] = {SGI280675, SGI280676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280677, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65322[] = {SGI280678, SGI280679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280680, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65323[] = {SGI280681, SGI280682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280683, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65324[] = {SGI0, SGI280684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65325[] = {SGI280685, SGI280686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280687, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65326[] = {SGI280688, SGI280689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280690, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65327[] = {SGI280691, SGI280692, SGI280693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65328[] = {SGI280694, SGI280695, SGI280696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65329[] = {SGI280697, SGI280698, SGI280699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65330[] = {SGI280700, SGI280701, SGI280702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65331[] = {SGI280703, SGI280704, SGI280705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65332[] = {SGI280706, SGI280707, SGI280708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65333[] = {SGI280709, SGI280710, SGI280711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65334[] = {SGI280712, SGI280713, SGI280714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65335[] = {SGI280715, SGI280716, SGI280717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65336[] = {SGI280718, SGI280719, SGI280720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65337[] = {SGI280721, SGI280722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280723, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65338[] = {SGI280724, SGI280725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280726, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65339[] = {SGI280727, SGI280728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280729, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65340[] = {SGI280730, SGI280731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280732, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65341[] = {SGI280733, SGI280734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65342[] = {SGI280735, SGI280736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65343[] = {SGI280737, SGI280738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65344[] = {SGI280739, SGI280740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65345[] = {SGI280741, SGI280742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65346[] = {SGI280743, SGI280744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65347[] = {SGI280745, SGI280746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65348[] = {SGI280747, SGI280748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65349[] = {SGI280749, SGI280750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65350[] = {SGI280751, SGI280752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65351[] = {SGI280753, SGI280754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65352[] = {SGI280756, SGI280757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280758, SGI0, NULL};
const Segment * const SG65353[] = {SGI280759, SGI280760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280761, SGI0, NULL};
const Segment * const SG65354[] = {SGI280762, SGI280763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280764, SGI0, NULL};
const Segment * const SG65355[] = {SGI280765, SGI280766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65356[] = {SGI280767, SGI280768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65357[] = {SGI280769, SGI280770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65358[] = {SGI280771, SGI280772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65359[] = {SGI280773, SGI280774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65360[] = {SGI280775, SGI280776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65361[] = {SGI280777, SGI280778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65362[] = {SGI280779, SGI280780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65363[] = {SGI280755, SGI280781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65364[] = {SGI280782, SGI280783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65365[] = {SGI280784, SGI280785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65366[] = {SGI280786, SGI280787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65367[] = {SGI280788, SGI280789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65368[] = {SGI280791, SGI280792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65369[] = {SGI280794, SGI280795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280796, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65370[] = {SGI280797, SGI280798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280799, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65371[] = {SGI280800, SGI280801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280802, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65372[] = {SGI280803, SGI280804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280805, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65373[] = {SGI280806, SGI280807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280808, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65374[] = {SGI280809, SGI280810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280811, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65375[] = {SGI280812, SGI280813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280814, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65376[] = {SGI280815, SGI280816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65377[] = {SGI280817, SGI280818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65378[] = {SGI280819, SGI280820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65379[] = {SGI280821, SGI280822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65380[] = {SGI280823, SGI280824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65381[] = {SGI280825, SGI280826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65382[] = {SGI280827, SGI280828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280829, SGI0, SGI0, NULL};
const Segment * const SG65383[] = {SGI280830, SGI280831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280832, SGI0, SGI0, NULL};
const Segment * const SG65384[] = {SGI280833, SGI280834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280835, SGI0, SGI0, NULL};
const Segment * const SG65385[] = {SGI280836, SGI280837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280838, SGI0, SGI0, NULL};
const Segment * const SG65386[] = {SGI280839, SGI280840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280841, SGI0, SGI0, NULL};
const Segment * const SG65387[] = {SGI280842, SGI280843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280844, SGI0, SGI0, NULL};
const Segment * const SG65388[] = {SGI280845, SGI280846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280847, SGI0, SGI0, NULL};
const Segment * const SG65389[] = {SGI280848, SGI280849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280850, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65390[] = {SGI280851, SGI280852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280853, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65391[] = {SGI280854, SGI280855, SGI0, SGI280856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65392[] = {SGI280857, SGI280858, SGI0, SGI280859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65393[] = {SGI280863, SGI280864, NULL};
const Segment * const SG65394[] = {SGI280860, SGI280861, SGI0, SGI280862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65395[] = {SGI280865, SGI280866, SGI0, SGI280867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65396[] = {SGI280868, SGI280869, SGI0, SGI280870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65397[] = {SGI280871, SGI280872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280873, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65398[] = {SGI280874, SGI280875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280876, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65399[] = {SGI280877, SGI280878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65400[] = {SGI280879, SGI280880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65401[] = {SGI280881, SGI280882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65402[] = {SGI280883, SGI280884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65403[] = {SGI280885, SGI280886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65404[] = {SGI280887, SGI280888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65405[] = {SGI280889, SGI280890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65406[] = {SGI280891, SGI280892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65407[] = {SGI280893, SGI280894, SGI280895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65408[] = {SGI280896, SGI280897, SGI280898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65409[] = {SGI280899, SGI280900, SGI0, SGI280901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65410[] = {SGI280902, SGI280903, SGI0, SGI280904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65411[] = {SGI280905, SGI280906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280907, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65412[] = {SGI280908, SGI280909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI280910, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65413[] = {SGI280911, SGI280912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65414[] = {SGI280913, SGI280914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65415[] = {SGI280915, SGI280916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65416[] = {SGI280917, SGI280918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65417[] = {SGI280919, SGI280920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65418[] = {SGI280921, SGI280922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65419[] = {SGI280923, SGI280924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65420[] = {SGI280925, SGI280926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65421[] = {SGI280927, SGI280928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65422[] = {SGI280929, SGI280930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65423[] = {SGI280931, SGI280932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65424[] = {SGI280933, SGI280934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65425[] = {SGI280935, SGI280936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65426[] = {SGI280937, SGI280938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65427[] = {SGI280939, SGI280940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65428[] = {SGI280941, SGI280942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65429[] = {SGI280943, SGI280944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65430[] = {SGI280945, SGI280946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65431[] = {SGI280947, SGI280948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65432[] = {SGI280949, SGI280950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65433[] = {SGI280951, SGI280952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65434[] = {SGI280953, SGI280954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65435[] = {SGI280955, SGI280956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65436[] = {SGI280957, SGI280958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65437[] = {SGI280959, SGI280960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65438[] = {SGI280961, SGI280962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65439[] = {SGI280963, SGI280964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65440[] = {SGI280965, SGI280966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65441[] = {SGI280967, SGI280968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65442[] = {SGI280969, SGI280970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65443[] = {SGI280971, SGI280972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65444[] = {SGI280973, SGI280974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65445[] = {SGI280975, SGI280976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65446[] = {SGI280977, SGI280978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65447[] = {SGI280979, SGI280980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65448[] = {SGI280981, SGI280982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65449[] = {SGI280983, SGI280984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65450[] = {SGI280985, SGI280986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65451[] = {SGI280987, SGI280988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65452[] = {SGI280989, SGI280990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65453[] = {SGI280991, SGI280992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65454[] = {SGI280993, SGI280994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65455[] = {SGI280995, SGI280996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65456[] = {SGI280997, SGI280998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65457[] = {SGI280999, SGI281000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65458[] = {SGI281001, SGI281002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65459[] = {SGI281003, SGI281004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65460[] = {SGI281005, SGI281006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65461[] = {SGI281007, SGI281008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65462[] = {SGI281009, SGI281010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65463[] = {SGI281011, SGI281012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65464[] = {SGI281013, SGI281014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65465[] = {SGI281015, SGI281016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65466[] = {SGI281017, SGI281018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65467[] = {SGI281019, SGI281020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65468[] = {SGI281021, SGI281022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65469[] = {SGI281023, SGI281024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65470[] = {SGI281025, SGI281026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65471[] = {SGI281027, SGI281028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65472[] = {SGI281029, SGI281030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65473[] = {SGI281031, SGI281032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65474[] = {SGI281033, SGI281034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65475[] = {SGI281035, SGI281036, SGI0, SGI281037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65476[] = {SGI281038, SGI281039, SGI0, SGI281040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65477[] = {SGI281041, SGI281042, SGI0, SGI281043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65478[] = {SGI281044, SGI281045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281046, SGI0, NULL};
const Segment * const SG65479[] = {SGI281047, SGI281048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281049, SGI0, NULL};
const Segment * const SG65480[] = {SGI281050, SGI281051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281052, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65481[] = {SGI281053, SGI281054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281055, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65482[] = {SGI281056, SGI281057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65483[] = {SGI281059, SGI281060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65484[] = {SGI281062, SGI281063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281064, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65485[] = {SGI281065, SGI281066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281067, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65486[] = {SGI281068, SGI281069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281070, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65487[] = {SGI281071, SGI281072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281073, SGI0, SGI0, NULL};
const Segment * const SG65488[] = {SGI281074, SGI281075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281076, SGI0, SGI0, NULL};
const Segment * const SG65489[] = {SGI281077, SGI281078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281079, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65490[] = {SGI281080, SGI281081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281082, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65491[] = {SGI281083, SGI281084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281085, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65492[] = {SGI281086, SGI281087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281088, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65493[] = {SGI281089, SGI281090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281091, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65494[] = {SGI281092, SGI281093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281094, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65495[] = {SGI281095, SGI281096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281097, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65496[] = {SGI281098, SGI281099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65497[] = {SGI281100, SGI281101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65498[] = {SGI281102, SGI281103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65499[] = {SGI281104, SGI281105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65500[] = {SGI281106, SGI281107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65501[] = {SGI281108, SGI281109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65502[] = {SGI281110, SGI281111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65503[] = {SGI281112, SGI281113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65504[] = {SGI281114, SGI281115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65505[] = {SGI281116, SGI281117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65506[] = {SGI281118, SGI281119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65507[] = {SGI281120, SGI281121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281122, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65508[] = {SGI281123, SGI281124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281125, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65509[] = {SGI281126, SGI281127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281128, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65510[] = {SGI281129, SGI281130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281131, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65511[] = {SGI281132, SGI281133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281134, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65512[] = {SGI281135, SGI281136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281137, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65513[] = {SGI281138, SGI281139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281140, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65514[] = {SGI281141, SGI281142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281143, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65515[] = {SGI281144, SGI281145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281146, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65516[] = {SGI281147, SGI281148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65517[] = {SGI281149, SGI281150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65518[] = {SGI281151, SGI281152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65519[] = {SGI281153, SGI281154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65520[] = {SGI281155, SGI281158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65521[] = {SGI281156, SGI281157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65522[] = {SGI281159, SGI281160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65523[] = {SGI281161, SGI281162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65524[] = {SGI281163, SGI281164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65525[] = {SGI281165, SGI281166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65526[] = {SGI281167, SGI281168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65527[] = {SGI281169, SGI281170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65528[] = {SGI281171, SGI281172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65529[] = {SGI281173, SGI281174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65530[] = {SGI281175, SGI281176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65531[] = {SGI281177, SGI281178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65532[] = {SGI281179, SGI281180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65533[] = {SGI281181, SGI281182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65534[] = {SGI281183, SGI281184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65535[] = {SGI281185, SGI281186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65536[] = {SGI281187, SGI281188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65537[] = {SGI281189, SGI281190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65538[] = {SGI281191, SGI281192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65539[] = {SGI281193, SGI281194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65540[] = {SGI281195, SGI281196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65541[] = {SGI281197, SGI281198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65542[] = {SGI281199, SGI281200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65543[] = {SGI281201, SGI281202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65544[] = {SGI281203, SGI281204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281205, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65545[] = {SGI281206, SGI281207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281208, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65546[] = {SGI281209, SGI281210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281211, SGI0, SGI0, NULL};
const Segment * const SG65547[] = {SGI281212, SGI281213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281214, SGI0, SGI0, NULL};
const Segment * const SG65548[] = {SGI281215, SGI281216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281217, SGI0, SGI0, NULL};
const Segment * const SG65549[] = {SGI281218, SGI281219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281220, SGI0, SGI0, NULL};
const Segment * const SG65550[] = {SGI281221, SGI281222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281223, SGI0, SGI0, NULL};
const Segment * const SG65551[] = {SGI281224, SGI281225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281226, SGI0, SGI0, NULL};
const Segment * const SG65552[] = {SGI281227, SGI281228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281229, SGI0, SGI0, NULL};
const Segment * const SG65553[] = {SGI281230, SGI281231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281232, SGI0, SGI0, NULL};
const Segment * const SG65554[] = {SGI281233, SGI281234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281235, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65555[] = {SGI281236, SGI281237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281238, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65556[] = {SGI281239, SGI281240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281241, NULL};
const Segment * const SG65557[] = {SGI281242, SGI281243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281244, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65558[] = {SGI281245, SGI281246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281247, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65559[] = {SGI281248, SGI281249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281250, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65560[] = {SGI281251, SGI281252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281253, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65561[] = {SGI281254, SGI281255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281256, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65562[] = {SGI281257, SGI281258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281259, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65563[] = {SGI281260, SGI281261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281262, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65564[] = {SGI281263, SGI281264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281265, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65565[] = {SGI281266, SGI281267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65566[] = {SGI281268, SGI281269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65567[] = {SGI281270, SGI281271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65568[] = {SGI281272, SGI281273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65569[] = {SGI281274, SGI281275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65570[] = {SGI281276, SGI281277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65571[] = {SGI281278, SGI281279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65572[] = {SGI281280, SGI281281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65573[] = {SGI281282, SGI281283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281284, NULL};
const Segment * const SG65574[] = {SGI281285, SGI281286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281287, NULL};
const Segment * const SG65575[] = {SGI281288, SGI281289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65576[] = {SGI281290, SGI281291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65577[] = {SGI281292, SGI281293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65578[] = {SGI281294, SGI281295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65579[] = {SGI281296, SGI281297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65580[] = {SGI281298, SGI281299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65581[] = {SGI281300, SGI281301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65582[] = {SGI281302, SGI281303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65583[] = {SGI281304, SGI281305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65584[] = {SGI281306, SGI281307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65585[] = {SGI281308, SGI281309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65586[] = {SGI281310, SGI281311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65587[] = {SGI281312, SGI281313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65588[] = {SGI281314, SGI281315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65589[] = {SGI281316, SGI281317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65590[] = {SGI281318, SGI281319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65591[] = {SGI281320, SGI281321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281322, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65592[] = {SGI281323, SGI281324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281325, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65593[] = {SGI281326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65594[] = {SGI281327, SGI281328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65595[] = {SGI281330, SGI281331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65596[] = {SGI281333, SGI281334, SGI0, SGI281335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65597[] = {SGI281336, SGI281337, SGI0, SGI281338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65598[] = {SGI281339, SGI281340, SGI0, SGI281341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65599[] = {SGI281342, SGI281343, SGI0, SGI281344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65600[] = {SGI281345, SGI281346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65601[] = {SGI281347, SGI281348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65602[] = {SGI281349, SGI281350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65603[] = {SGI281351, SGI281352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65604[] = {SGI281353, SGI281354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65605[] = {SGI281355, SGI281356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65606[] = {SGI281357, SGI281358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65607[] = {SGI281360, SGI281361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65608[] = {SGI281363, SGI281364, SGI0, SGI281365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65609[] = {SGI281366, SGI281367, SGI0, SGI281368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65610[] = {SGI281369, SGI281370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281371, NULL};
const Segment * const SG65611[] = {SGI281372, SGI281373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281374, NULL};
const Segment * const SG65612[] = {SGI281375, SGI281376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65613[] = {SGI281377, SGI281378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65614[] = {SGI281379, SGI281380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65615[] = {SGI281381, SGI281382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65616[] = {SGI281383, SGI281384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65617[] = {SGI281385, SGI281386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65618[] = {SGI281387, SGI281388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65619[] = {SGI281389, SGI281390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65620[] = {SGI281391, SGI281392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65621[] = {SGI281393, SGI281394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65622[] = {SGI281395, SGI281396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65623[] = {SGI281397, SGI281398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65624[] = {SGI281400, SGI281401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65625[] = {SGI281403, SGI281404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65626[] = {SGI281405, SGI281406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65627[] = {SGI281407, SGI281408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65628[] = {SGI281409, SGI281410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65629[] = {SGI281411, SGI281412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65630[] = {SGI281413, SGI281414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65631[] = {SGI281415, SGI281416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65632[] = {SGI281417, SGI281418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65633[] = {SGI281419, SGI281420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65634[] = {SGI281421, SGI281422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65635[] = {SGI281423, SGI281424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65636[] = {SGI281425, SGI281426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65637[] = {SGI281427, SGI281428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65638[] = {SGI281429, SGI281430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65639[] = {SGI281431, SGI281432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65640[] = {SGI281434, SGI281435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65641[] = {SGI281437, SGI281438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65642[] = {SGI281440, SGI281441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65643[] = {SGI281443, SGI281444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65644[] = {SGI281445, SGI281446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281447, SGI0, NULL};
const Segment * const SG65645[] = {SGI281448, SGI281449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65646[] = {SGI281450, SGI281451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65647[] = {SGI281452, SGI281453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65648[] = {SGI281454, SGI281455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281456, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65649[] = {SGI281457, SGI281458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281459, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65650[] = {SGI281460, SGI281461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65651[] = {SGI281463, SGI281464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65652[] = {SGI281466, SGI281467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65653[] = {SGI281468, SGI281469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65654[] = {SGI281470, SGI281471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65655[] = {SGI281472, SGI281473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65656[] = {SGI281474, SGI281475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65657[] = {SGI281476, SGI281477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65658[] = {SGI281478, SGI281479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65659[] = {SGI281480, SGI281481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65660[] = {SGI281482, SGI281483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65661[] = {SGI281484, SGI281485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65662[] = {SGI281486, SGI281487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65663[] = {SGI281488, SGI281489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65664[] = {SGI281490, SGI281491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65665[] = {SGI281492, SGI281493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65666[] = {SGI281494, SGI281495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65667[] = {SGI281496, SGI281497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65668[] = {SGI281498, SGI281499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65669[] = {SGI281500, SGI281501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65670[] = {SGI281502, SGI281503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65671[] = {SGI281504, SGI281505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65672[] = {SGI281506, SGI281507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281508, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65673[] = {SGI281509, SGI281510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65674[] = {SGI281511, SGI281512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281513, SGI0, SGI0, NULL};
const Segment * const SG65675[] = {SGI281514, SGI281515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281516, SGI0, SGI0, NULL};
const Segment * const SG65676[] = {SGI281517, SGI281518, SGI0, SGI281519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65677[] = {SGI281520, SGI281521, SGI0, SGI281522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65678[] = {SGI281523, SGI281524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281525, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65679[] = {SGI281526, SGI281527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281528, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65680[] = {SGI281529, SGI281530, SGI0, SGI281531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65681[] = {SGI281532, SGI281533, SGI0, SGI281534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65682[] = {SGI281535, SGI281536, SGI0, SGI281537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65683[] = {SGI281538, SGI281539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65684[] = {SGI281540, SGI281541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281542, SGI0, SGI0, NULL};
const Segment * const SG65685[] = {SGI281543, SGI281544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65686[] = {SGI281545, SGI281546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65687[] = {SGI281547, SGI281548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65688[] = {SGI281549, SGI281550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65689[] = {SGI281551, SGI281552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65690[] = {SGI281553, SGI281554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65691[] = {SGI281555, SGI281556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65692[] = {SGI281557, SGI281558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65693[] = {SGI281559, SGI281560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65694[] = {SGI281561, SGI281562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65695[] = {SGI281563, SGI281564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65696[] = {SGI281565, SGI281566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65697[] = {SGI281567, SGI281568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65698[] = {SGI281569, SGI281570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65699[] = {SGI281571, SGI281572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65700[] = {SGI281573, SGI281574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65701[] = {SGI281575, SGI281576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65702[] = {SGI281577, SGI281578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281579, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65703[] = {SGI281580, SGI281581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65704[] = {SGI281582, SGI281583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281584, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65705[] = {SGI281585, SGI281586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65706[] = {SGI281587, SGI281588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65707[] = {SGI281590, SGI281591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65708[] = {SGI281593, SGI281594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65709[] = {SGI281596, SGI281597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65710[] = {SGI281599, SGI281600, SGI0, SGI281601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65711[] = {SGI281602, SGI281603, SGI0, SGI281604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65712[] = {SGI281605, SGI281606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65713[] = {SGI281607, SGI281608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65714[] = {SGI281610, SGI281611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65715[] = {SGI281613, SGI281614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65716[] = {SGI281615, SGI281616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65717[] = {SGI281617, SGI281618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65718[] = {SGI281619, SGI281620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65719[] = {SGI281621, SGI281622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65720[] = {SGI281623, SGI281624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65721[] = {SGI281625, SGI281626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281627, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65722[] = {SGI281628, SGI281629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65723[] = {SGI281630, SGI281631, SGI0, SGI281632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65724[] = {SGI281633, SGI281634, SGI0, SGI281636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65725[] = {SGI281635, SGI281637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65726[] = {SGI281638, SGI281639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65727[] = {SGI281640, SGI281641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65728[] = {SGI281642, SGI281643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65729[] = {SGI281644, SGI281645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65730[] = {SGI281646, SGI281647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65731[] = {SGI281648, SGI281649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65732[] = {SGI281650, SGI281651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65733[] = {SGI281652, SGI281653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65734[] = {SGI281654, SGI281655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65735[] = {SGI281656, SGI281657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281658, NULL};
const Segment * const SG65736[] = {SGI281659, SGI281660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281661, NULL};
const Segment * const SG65737[] = {SGI281662, SGI281663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65738[] = {SGI281664, SGI281665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281666, NULL};
const Segment * const SG65739[] = {SGI281667, SGI281668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65740[] = {SGI281669, SGI281670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65741[] = {SGI281671, SGI281672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65742[] = {SGI281673, SGI281674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65743[] = {SGI281675, SGI281676, SGI0, SGI281677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65744[] = {SGI281678, SGI281679, SGI0, SGI281680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65745[] = {SGI281681, SGI281682, SGI0, SGI281683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65746[] = {SGI281684, SGI281685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281686, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65747[] = {SGI281687, SGI281688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281689, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65748[] = {SGI281690, SGI281691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281692, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65749[] = {SGI281693, SGI281694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65750[] = {SGI281696, SGI281697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65751[] = {SGI281699, SGI281700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65752[] = {SGI281702, SGI281703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281704, NULL};
const Segment * const SG65753[] = {SGI281705, SGI281706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281707, NULL};
const Segment * const SG65754[] = {SGI281708, SGI281709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65755[] = {SGI281710, SGI281711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65756[] = {SGI281712, SGI281714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65757[] = {SGI281713, SGI281715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65758[] = {SGI281716, SGI281719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65759[] = {SGI281717, SGI281718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65760[] = {SGI281720, SGI281723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65761[] = {SGI281721, SGI281722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65762[] = {SGI281724, SGI281725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65763[] = {SGI281726, SGI281727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65764[] = {SGI281728, SGI281729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65765[] = {SGI281730, SGI281731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65766[] = {SGI281732, SGI281733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65767[] = {SGI281734, SGI281735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65768[] = {SGI281736, SGI281737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65769[] = {SGI281738, SGI281739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65770[] = {SGI281740, SGI281741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65771[] = {SGI281742, SGI281743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65772[] = {SGI281744, SGI281745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65773[] = {SGI281746, SGI281747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65774[] = {SGI281748, SGI281749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65775[] = {SGI281750, SGI281751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65776[] = {SGI281752, SGI281753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65777[] = {SGI281754, SGI281755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65778[] = {SGI281756, SGI281757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65779[] = {SGI281758, SGI281759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65780[] = {SGI281760, SGI281762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65781[] = {SGI281761, SGI281763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65782[] = {SGI281764, SGI281766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65783[] = {SGI281765, SGI281767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65784[] = {SGI281768, SGI281770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65785[] = {SGI281769, SGI281771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65786[] = {SGI281772, SGI281775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65787[] = {SGI281773, SGI281774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65788[] = {SGI281776, SGI281778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65789[] = {SGI281777, SGI281779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65790[] = {SGI281780, SGI281782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65791[] = {SGI281781, SGI281783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65792[] = {SGI281784, SGI281786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65793[] = {SGI281785, SGI281787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65794[] = {SGI281788, SGI281790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65795[] = {SGI281789, SGI281791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65796[] = {SGI281792, SGI281794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65797[] = {SGI281793, SGI281795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65798[] = {SGI281796, SGI281798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65799[] = {SGI281797, SGI281799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65800[] = {SGI281800, SGI281802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65801[] = {SGI281801, SGI281803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65802[] = {SGI281804, SGI281806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65803[] = {SGI281805, SGI281807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65804[] = {SGI281808, SGI281810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65805[] = {SGI281809, SGI281811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65806[] = {SGI281812, SGI281814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65807[] = {SGI281813, SGI281815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65808[] = {SGI281816, SGI281818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65809[] = {SGI281817, SGI281819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65810[] = {SGI281820, SGI281822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65811[] = {SGI281821, SGI281823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65812[] = {SGI281824, SGI281826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65813[] = {SGI281825, SGI281827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65814[] = {SGI281828, SGI281830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65815[] = {SGI281829, SGI281831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65816[] = {SGI281832, SGI281833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65817[] = {SGI281834, SGI281835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65818[] = {SGI281836, SGI281837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65819[] = {SGI281838, SGI281839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65820[] = {SGI281840, SGI281841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65821[] = {SGI281842, SGI281843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65822[] = {SGI281844, SGI281845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65823[] = {SGI281846, SGI281847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65824[] = {SGI281848, SGI281849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65825[] = {SGI281850, SGI281851, SGI0, SGI0, SGI281852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65826[] = {SGI281853, SGI281854, SGI0, SGI0, SGI281855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65827[] = {SGI281856, SGI281857, SGI0, SGI0, SGI281858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65828[] = {SGI281859, SGI281860, SGI0, SGI0, SGI281861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65829[] = {SGI281862, SGI281863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281864, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65830[] = {SGI281865, SGI281866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281867, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65831[] = {SGI281868, SGI281869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65832[] = {SGI281870, SGI281871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281872, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65833[] = {SGI281873, SGI281874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281875, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65834[] = {SGI281876, SGI281877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281878, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65835[] = {SGI281879, SGI281880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65836[] = {SGI281881, SGI281882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65837[] = {SGI281883, SGI281884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65838[] = {SGI281885, SGI281886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65839[] = {SGI281887, SGI281888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65840[] = {SGI281889, SGI281890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281891, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65841[] = {SGI281892, SGI281893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281894, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65842[] = {SGI281895, SGI281896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281897, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65843[] = {SGI281898, SGI281899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281900, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65844[] = {SGI281901, SGI281902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281903, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65845[] = {SGI281904, SGI281905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281906, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65846[] = {SGI281907, SGI281908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65847[] = {SGI281910, SGI281911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65848[] = {SGI281913, SGI281914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65849[] = {SGI281915, SGI281916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65850[] = {SGI281918, SGI281919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65851[] = {SGI281921, SGI281922, SGI0, SGI281923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65852[] = {SGI281924, SGI281925, SGI0, SGI281926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65853[] = {SGI281927, SGI281928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65854[] = {SGI281930, SGI281931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65855[] = {SGI281933, SGI281934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65856[] = {SGI281936, SGI281937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65857[] = {SGI281939, SGI281940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65858[] = {SGI281942, SGI281943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65859[] = {SGI281945, SGI281946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281947, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65860[] = {SGI281948, SGI281949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281950, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65861[] = {SGI281951, SGI281952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281953, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65862[] = {SGI281954, SGI281955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281956, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65863[] = {SGI281957, SGI281958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65864[] = {SGI281959, SGI281960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65865[] = {SGI281961, SGI281962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65866[] = {SGI281963, SGI281964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65867[] = {SGI281965, SGI281966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65868[] = {SGI281967, SGI281968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65869[] = {SGI281969, SGI281970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65870[] = {SGI281971, SGI281972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65871[] = {SGI281973, SGI281974, SGI0, SGI281975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65872[] = {SGI281976, SGI281977, SGI0, SGI281978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65873[] = {SGI281979, SGI281980, SGI0, SGI281981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65874[] = {SGI281982, SGI281983, SGI0, SGI281984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65875[] = {SGI281985, SGI281986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281987, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65876[] = {SGI281988, SGI281989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281990, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65877[] = {SGI281991, SGI281992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65878[] = {SGI281993, SGI281994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281995, SGI0, SGI0, NULL};
const Segment * const SG65879[] = {SGI281996, SGI281997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI281998, NULL};
const Segment * const SG65880[] = {SGI281999, SGI282000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65881[] = {SGI282001, SGI282002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65882[] = {SGI282003, SGI282004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65883[] = {SGI282005, SGI282006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65884[] = {SGI282007, SGI282008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65885[] = {SGI282009, SGI282010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65886[] = {SGI282011, SGI282012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65887[] = {SGI282013, SGI282014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65888[] = {SGI282015, SGI282016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65889[] = {SGI282017, SGI282018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65890[] = {SGI282019, SGI282020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65891[] = {SGI282021, SGI282022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65892[] = {SGI282023, SGI282024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65893[] = {SGI282025, SGI282026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65894[] = {SGI282027, SGI282028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65895[] = {SGI282029, SGI282030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65896[] = {SGI282031, SGI282032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65897[] = {SGI282033, SGI282034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65898[] = {SGI282035, SGI282036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65899[] = {SGI282037, SGI282038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65900[] = {SGI282039, SGI282040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65901[] = {SGI282041, SGI282042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65902[] = {SGI282043, SGI282044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65903[] = {SGI282045, SGI282046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65904[] = {SGI282047, SGI282048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65905[] = {SGI282049, SGI282050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282051, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65906[] = {SGI282052, SGI282053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282054, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65907[] = {SGI282055, SGI282056, SGI0, SGI0, SGI282057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65908[] = {SGI282058, SGI282059, SGI0, SGI0, SGI282060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65909[] = {SGI282061, SGI282062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65910[] = {SGI282063, SGI282064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65911[] = {SGI282065, SGI282066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65912[] = {SGI282067, SGI282068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65913[] = {SGI282069, SGI282070, SGI282071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65914[] = {SGI282072, SGI282073, SGI282074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65915[] = {SGI282075, SGI282076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65916[] = {SGI282077, SGI282078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65917[] = {SGI282079, SGI282080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65918[] = {SGI282081, SGI282082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65919[] = {SGI282083, SGI282084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65920[] = {SGI282085, SGI282086, SGI282087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65921[] = {SGI282088, SGI282089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282090, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65922[] = {SGI282091, SGI282092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282093, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65923[] = {SGI282094, SGI282095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282096, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65924[] = {SGI282097, SGI282098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282099, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65925[] = {SGI282100, SGI282101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282102, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65926[] = {SGI282103, SGI282104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282105, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65927[] = {SGI282106, SGI282107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282108, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65928[] = {SGI282109, SGI282110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282111, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65929[] = {SGI282112, SGI282113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282114, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65930[] = {SGI282115, SGI282116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282117, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65931[] = {SGI282118, SGI282119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65932[] = {SGI282121, SGI282122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65933[] = {SGI282124, SGI282125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65934[] = {SGI282127, SGI282128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282129, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65935[] = {SGI282130, SGI282131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282132, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65936[] = {SGI282133, SGI282134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282135, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65937[] = {SGI282136, SGI282137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282138, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65938[] = {SGI282139, SGI282140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65939[] = {SGI282141, SGI282142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65940[] = {SGI282143, SGI282144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65941[] = {SGI282145, SGI282146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65942[] = {SGI282147, SGI282148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65943[] = {SGI282149, SGI282150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65944[] = {SGI282151, SGI282152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65945[] = {SGI282153, SGI282154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65946[] = {SGI282155, SGI282156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65947[] = {SGI282158, SGI282159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65948[] = {SGI282161, SGI282162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65949[] = {SGI282164, SGI282165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65950[] = {SGI282167, SGI282168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65951[] = {SGI282170, SGI282171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65952[] = {SGI282173, SGI282174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65953[] = {SGI282175, SGI282176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65954[] = {SGI282177, SGI282178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282179, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65955[] = {SGI282180, SGI282181, SGI0, SGI282182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65956[] = {SGI282183, SGI282184, SGI0, SGI282185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65957[] = {SGI282186, SGI282187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65958[] = {SGI282189, SGI282190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG65959[] = {SGI282192, SGI282193, NULL};
const Segment * const SG65960[] = {SGI282194, SGI282195, NULL};
const Segment * const SG65961[] = {SGI282196, SGI282197, NULL};
const Segment * const SG65962[] = {SGI282198, SGI282199, NULL};
const Segment * const SG65963[] = {SGI282200, SGI282201, NULL};
const Segment * const SG65964[] = {SGI282202, SGI282203, NULL};
const Segment * const SG65965[] = {SGI282204, SGI282205, NULL};
const Segment * const SG65966[] = {SGI282206, SGI282207, NULL};
const Segment * const SG65967[] = {SGI282208, SGI282209, NULL};
const Segment * const SG65968[] = {SGI282210, SGI282211, NULL};
const Segment * const SG65969[] = {SGI282212, SGI282213, NULL};
const Segment * const SG65970[] = {SGI282214, SGI282215, NULL};
const Segment * const SG65971[] = {SGI282216, SGI282217, NULL};
const Segment * const SG65972[] = {SGI282218, SGI282219, NULL};
const Segment * const SG65973[] = {SGI282220, SGI282221, NULL};
const Segment * const SG65974[] = {SGI282222, SGI282223, NULL};
const Segment * const SG65975[] = {SGI282224, SGI282225, NULL};
const Segment * const SG65976[] = {SGI282226, SGI282227, NULL};
const Segment * const SG65977[] = {SGI282228, SGI282229, NULL};
const Segment * const SG65978[] = {SGI282230, SGI282231, NULL};
const Segment * const SG65979[] = {SGI282232, SGI282233, NULL};
const Segment * const SG65980[] = {SGI282234, SGI282235, NULL};
const Segment * const SG65981[] = {SGI282236, SGI282237, NULL};
const Segment * const SG65982[] = {SGI282238, SGI282239, NULL};
const Segment * const SG65983[] = {SGI282240, SGI282241, NULL};
const Segment * const SG65984[] = {SGI282242, SGI282243, NULL};
const Segment * const SG65985[] = {SGI282244, SGI282245, NULL};
const Segment * const SG65986[] = {SGI282246, SGI282247, NULL};
const Segment * const SG65987[] = {SGI282248, SGI282249, NULL};
const Segment * const SG65988[] = {SGI282250, SGI282251, NULL};
const Segment * const SG65989[] = {SGI282252, SGI282253, NULL};
const Segment * const SG65990[] = {SGI282254, SGI282255, NULL};
const Segment * const SG65991[] = {SGI282256, SGI282257, NULL};
const Segment * const SG65992[] = {SGI282258, SGI282259, NULL};
const Segment * const SG65993[] = {SGI282260, SGI282261, NULL};
const Segment * const SG65994[] = {SGI282262, SGI282263, NULL};
const Segment * const SG65995[] = {SGI282264, SGI282265, NULL};
const Segment * const SG65996[] = {SGI282266, SGI282267, NULL};
const Segment * const SG65997[] = {SGI282268, SGI282269, NULL};
const Segment * const SG65998[] = {SGI282270, SGI282271, NULL};
const Segment * const SG65999[] = {SGI282272, SGI282273, NULL};
const Segment * const SG66000[] = {SGI282274, SGI282275, NULL};
const Segment * const SG66001[] = {SGI282276, SGI282277, NULL};
const Segment * const SG66002[] = {SGI282278, SGI282279, NULL};
const Segment * const SG66003[] = {SGI282280, SGI282281, NULL};
const Segment * const SG66004[] = {SGI282282, SGI282283, NULL};
const Segment * const SG66005[] = {SGI282284, SGI282285, NULL};
const Segment * const SG66006[] = {SGI282286, SGI282287, NULL};
const Segment * const SG66007[] = {SGI282288, SGI282289, NULL};
const Segment * const SG66008[] = {SGI282290, SGI282291, NULL};
const Segment * const SG66009[] = {SGI282292, SGI282293, NULL};
const Segment * const SG66010[] = {SGI282294, SGI282295, NULL};
const Segment * const SG66011[] = {SGI282296, SGI282297, NULL};
const Segment * const SG66012[] = {SGI282298, SGI282299, NULL};
const Segment * const SG66013[] = {SGI282300, SGI282301, NULL};
const Segment * const SG66014[] = {SGI282302, SGI282303, NULL};
const Segment * const SG66015[] = {SGI282304, SGI282305, NULL};
const Segment * const SG66016[] = {SGI282306, SGI282307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282308, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66017[] = {SGI282309, SGI282310, NULL};
const Segment * const SG66018[] = {SGI282313, SGI282314, NULL};
const Segment * const SG66019[] = {SGI282311, SGI282312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282315, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66020[] = {SGI282316, SGI282317, NULL};
const Segment * const SG66021[] = {SGI282318, SGI282319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282320, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66022[] = {SGI282321, SGI282322, NULL};
const Segment * const SG66023[] = {SGI282323, SGI282324, NULL};
const Segment * const SG66024[] = {SGI282325, SGI282326, NULL};
const Segment * const SG66025[] = {SGI282327, SGI282328, NULL};
const Segment * const SG66026[] = {SGI282329, SGI282330, NULL};
const Segment * const SG66027[] = {SGI282331, SGI282332, NULL};
const Segment * const SG66028[] = {SGI282333, SGI282334, NULL};
const Segment * const SG66029[] = {SGI282335, SGI282336, NULL};
const Segment * const SG66030[] = {SGI282337, SGI282338, NULL};
const Segment * const SG66031[] = {SGI282339, SGI282340, NULL};
const Segment * const SG66032[] = {SGI282341, SGI282342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282343, SGI0, SGI0, NULL};
const Segment * const SG66033[] = {SGI282344, SGI282345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282346, SGI0, SGI0, NULL};
const Segment * const SG66034[] = {SGI282347, SGI282348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282349, SGI0, SGI0, NULL};
const Segment * const SG66035[] = {SGI282350, SGI282351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282352, SGI0, SGI0, NULL};
const Segment * const SG66036[] = {SGI282353, SGI282354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282355, SGI0, SGI0, NULL};
const Segment * const SG66037[] = {SGI282356, SGI282357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282358, SGI0, SGI0, NULL};
const Segment * const SG66038[] = {SGI282359, SGI282360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282361, SGI0, SGI0, NULL};
const Segment * const SG66039[] = {SGI282362, SGI282363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282364, SGI0, SGI0, NULL};
const Segment * const SG66040[] = {SGI282365, SGI282366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282367, SGI0, SGI0, NULL};
const Segment * const SG66041[] = {SGI282368, SGI282369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282370, SGI0, SGI0, NULL};
const Segment * const SG66042[] = {SGI282371, SGI282372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282373, SGI0, SGI0, NULL};
const Segment * const SG66043[] = {SGI282374, SGI282375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66044[] = {SGI282376, SGI282377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66045[] = {SGI282378, SGI282379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66046[] = {SGI282380, SGI282381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282382, NULL};
const Segment * const SG66047[] = {SGI282383, SGI282384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66048[] = {SGI282385, SGI282386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66049[] = {SGI282387, SGI282388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66050[] = {SGI282389, SGI282390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282391, SGI0, SGI0, NULL};
const Segment * const SG66051[] = {SGI282392, SGI282393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282394, SGI0, SGI0, NULL};
const Segment * const SG66052[] = {SGI282395, SGI282396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282397, SGI0, SGI0, NULL};
const Segment * const SG66053[] = {SGI282398, SGI282399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66054[] = {SGI282400, SGI282401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66055[] = {SGI282402, SGI282403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66056[] = {SGI282404, SGI282405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66057[] = {SGI282406, SGI282407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66058[] = {SGI282408, SGI282409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66059[] = {SGI282410, SGI282411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66060[] = {SGI282412, SGI282413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66061[] = {SGI282415, SGI282416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66062[] = {SGI282418, SGI282419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66063[] = {SGI282421, SGI282422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66064[] = {SGI282424, SGI282425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282426, SGI0, SGI0, NULL};
const Segment * const SG66065[] = {SGI282427, SGI282428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282429, SGI0, SGI0, NULL};
const Segment * const SG66066[] = {SGI282430, SGI282431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282432, SGI0, SGI0, NULL};
const Segment * const SG66067[] = {SGI282433, SGI282434, SGI0, SGI282435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66068[] = {SGI282436, SGI282437, SGI0, SGI282438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66069[] = {SGI0, SGI282439, SGI0, SGI282440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66070[] = {SGI282441, SGI282442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282443, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66071[] = {SGI282444, SGI282445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282446, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66072[] = {SGI282447, SGI282448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282449, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66073[] = {SGI282450, SGI282451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282452, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66074[] = {SGI282453, SGI282454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66075[] = {SGI282456, SGI282457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66076[] = {SGI282459, SGI282460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66077[] = {SGI282461, SGI282462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66078[] = {SGI282463, SGI282464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66079[] = {SGI282465, SGI282466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66080[] = {SGI282467, SGI282468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66081[] = {SGI282469, SGI282470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66082[] = {SGI282471, SGI282472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66083[] = {SGI282473, SGI282474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66084[] = {SGI282475, SGI282476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66085[] = {SGI282477, SGI282478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66086[] = {SGI282479, SGI282480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66087[] = {SGI282481, SGI282482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66088[] = {SGI282483, SGI282484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282485, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66089[] = {SGI282486, SGI282487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282488, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66090[] = {SGI282489, SGI282490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66091[] = {SGI282491, SGI282492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66092[] = {SGI282493, SGI282494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66093[] = {SGI282495, SGI282496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66094[] = {SGI282497, SGI282498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66095[] = {SGI282499, SGI282500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66096[] = {SGI282501, SGI282502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66097[] = {SGI282503, SGI282504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66098[] = {SGI282505, SGI282506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66099[] = {SGI282507, SGI282508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66100[] = {SGI282509, SGI282510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66101[] = {SGI282511, SGI282512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66102[] = {SGI282513, SGI282514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66103[] = {SGI282515, SGI282516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66104[] = {SGI282517, SGI282518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66105[] = {SGI282519, SGI282520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66106[] = {SGI282521, SGI282522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66107[] = {SGI282523, SGI282524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66108[] = {SGI282525, SGI282526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66109[] = {SGI282527, SGI282528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66110[] = {SGI282529, SGI282530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66111[] = {SGI282531, SGI282532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66112[] = {SGI282533, SGI282534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66113[] = {SGI282535, SGI282536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66114[] = {SGI282537, SGI282538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66115[] = {SGI282539, SGI282540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66116[] = {SGI282541, SGI282542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66117[] = {SGI282543, SGI282544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66118[] = {SGI282545, SGI282546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66119[] = {SGI282547, SGI282548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66120[] = {SGI282549, SGI282550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66121[] = {SGI282551, SGI282552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66122[] = {SGI282553, SGI282554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66123[] = {SGI282555, SGI282556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66124[] = {SGI282557, SGI282558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66125[] = {SGI282559, SGI282560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66126[] = {SGI282561, SGI282562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66127[] = {SGI282563, SGI282564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66128[] = {SGI282565, SGI282566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66129[] = {SGI282567, SGI282568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66130[] = {SGI282569, SGI282570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66131[] = {SGI282571, SGI282572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66132[] = {SGI282573, SGI282574, SGI0, SGI282575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66133[] = {SGI282576, SGI282577, SGI0, SGI282578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66134[] = {SGI282579, SGI282580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI282581, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66135[] = {SGI282582, SGI282583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66136[] = {SGI282584, SGI282585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66137[] = {SGI282586, SGI282587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66138[] = {SGI282588, SGI282589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66139[] = {SGI282590, SGI282591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66140[] = {SGI282592, SGI282593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66141[] = {SGI282594, SGI282595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66142[] = {SGI282596, SGI282597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66143[] = {SGI282598, SGI282599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66144[] = {SGI282600, SGI282601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66145[] = {SGI282602, SGI282603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66146[] = {SGI282604, SGI282605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66147[] = {SGI282606, SGI282607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66148[] = {SGI282608, SGI282609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66149[] = {SGI282610, SGI282611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66150[] = {SGI282612, SGI282613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66151[] = {SGI282614, SGI282615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66152[] = {SGI282616, SGI282617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66153[] = {SGI282618, SGI282619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66154[] = {SGI282620, SGI282621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66155[] = {SGI282622, SGI282623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66156[] = {SGI282624, SGI282625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66157[] = {SGI282626, SGI282627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66158[] = {SGI282628, SGI282629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66159[] = {SGI282630, SGI282631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66160[] = {SGI282632, SGI282633, SGI0, SGI282634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66161[] = {SGI282635, SGI282636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66162[] = {SGI282637, SGI282638, SGI0, SGI282640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66163[] = {SGI282639, SGI282641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66164[] = {SGI282642, SGI282643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66165[] = {SGI282644, SGI282645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66166[] = {SGI282646, SGI282647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66167[] = {SGI282648, SGI282649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66168[] = {SGI282650, SGI282651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66169[] = {SGI282652, SGI282653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66170[] = {SGI282654, SGI282655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66171[] = {SGI282656, SGI282657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66172[] = {SGI282658, SGI282659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66173[] = {SGI282660, SGI282661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66174[] = {SGI282662, SGI282663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66175[] = {SGI282664, SGI282665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66176[] = {SGI282666, SGI282667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66177[] = {SGI282668, SGI282669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66178[] = {SGI282671, SGI282673, SGI0, SGI0, SGI0, SGI0, SGI282674, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66179[] = {SGI282670, SGI282672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66180[] = {SGI282675, SGI282676, SGI0, SGI0, SGI0, SGI0, SGI282677, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66181[] = {SGI282678, SGI282679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66182[] = {SGI282680, SGI282681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66183[] = {SGI282682, SGI282683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66184[] = {SGI282684, SGI282685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66185[] = {SGI282686, SGI282687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66186[] = {SGI282688, SGI282689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66187[] = {SGI282690, SGI282691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66188[] = {SGI282692, SGI282693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66189[] = {SGI282694, SGI282695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66190[] = {SGI282696, SGI282697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66191[] = {SGI282698, SGI282699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66192[] = {SGI282700, SGI282701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66193[] = {SGI282702, SGI282703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66194[] = {SGI282704, SGI282705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66195[] = {SGI282706, SGI282707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66196[] = {SGI282708, SGI282709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66197[] = {SGI282710, SGI282711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66198[] = {SGI282712, SGI282713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66199[] = {SGI282714, SGI282715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66200[] = {SGI282716, SGI282717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66201[] = {SGI282718, SGI282719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66202[] = {SGI282720, SGI282721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66203[] = {SGI282722, SGI282723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66204[] = {SGI282724, SGI282725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66205[] = {SGI282726, SGI282727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66206[] = {SGI282728, SGI282729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66207[] = {SGI282730, SGI282731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66208[] = {SGI282732, SGI282733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66209[] = {SGI282734, SGI282735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66210[] = {SGI282736, SGI282737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66211[] = {SGI282738, SGI282739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66212[] = {SGI282740, SGI282741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66213[] = {SGI282742, SGI282743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66214[] = {SGI282744, SGI282745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66215[] = {SGI282746, SGI282747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66216[] = {SGI282748, SGI282749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66217[] = {SGI282750, SGI282751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66218[] = {SGI282752, SGI282753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66219[] = {SGI282754, SGI282755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66220[] = {SGI282756, SGI282757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66221[] = {SGI282758, SGI282759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66222[] = {SGI282760, SGI282761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66223[] = {SGI282762, SGI282763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66224[] = {SGI282764, SGI282765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66225[] = {SGI282766, SGI282767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66226[] = {SGI282768, SGI282769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66227[] = {SGI282770, SGI282771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66228[] = {SGI282772, SGI282773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66229[] = {SGI282774, SGI282775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66230[] = {SGI282776, SGI282777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66231[] = {SGI282778, SGI282779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66232[] = {SGI282780, SGI282781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66233[] = {SGI282782, SGI282783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66234[] = {SGI282784, SGI282785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66235[] = {SGI282786, SGI282787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66236[] = {SGI282788, SGI282789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66237[] = {SGI282790, SGI282791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66238[] = {SGI282792, SGI282793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66239[] = {SGI282794, SGI282795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66240[] = {SGI282796, SGI282797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66241[] = {SGI282798, SGI282799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66242[] = {SGI282800, SGI282801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66243[] = {SGI282802, SGI282803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66244[] = {SGI282804, SGI282805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66245[] = {SGI282806, SGI282807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66246[] = {SGI282808, SGI282809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66247[] = {SGI282810, SGI282811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66248[] = {SGI282812, SGI282813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66249[] = {SGI282814, SGI282815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66250[] = {SGI282816, SGI282817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66251[] = {SGI282818, SGI282819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66252[] = {SGI282820, SGI282821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66253[] = {SGI282822, SGI282823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66254[] = {SGI282824, SGI282825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66255[] = {SGI282826, SGI282827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66256[] = {SGI282828, SGI282829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66257[] = {SGI282830, SGI282831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66258[] = {SGI282832, SGI282833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66259[] = {SGI282834, SGI282835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66260[] = {SGI282836, SGI282837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66261[] = {SGI282838, SGI282839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66262[] = {SGI282840, SGI282841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66263[] = {SGI282842, SGI282843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66264[] = {SGI282844, SGI282845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66265[] = {SGI282846, SGI282847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66266[] = {SGI282848, SGI282849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66267[] = {SGI282850, SGI282851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66268[] = {SGI282852, SGI282853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66269[] = {SGI282854, SGI282855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66270[] = {SGI282856, SGI282857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66271[] = {SGI282858, SGI282859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66272[] = {SGI282860, SGI282861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66273[] = {SGI282862, SGI282863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66274[] = {SGI282864, SGI282865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66275[] = {SGI282866, SGI282867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66276[] = {SGI282868, SGI282869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66277[] = {SGI282870, SGI282871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66278[] = {SGI282872, SGI282873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66279[] = {SGI282874, SGI282875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66280[] = {SGI282876, SGI282877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66281[] = {SGI282878, SGI282879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66282[] = {SGI282880, SGI282881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66283[] = {SGI282882, SGI282883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66284[] = {SGI282884, SGI282885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66285[] = {SGI282886, SGI282887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66286[] = {SGI282888, SGI282889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66287[] = {SGI282890, SGI282891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66288[] = {SGI282892, SGI282893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66289[] = {SGI282894, SGI282895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66290[] = {SGI282896, SGI282897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66291[] = {SGI282898, SGI282899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66292[] = {SGI282900, SGI282901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66293[] = {SGI282902, SGI282903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66294[] = {SGI282904, SGI282905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66295[] = {SGI282906, SGI282907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66296[] = {SGI282908, SGI282909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66297[] = {SGI282910, SGI282911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66298[] = {SGI282912, SGI282913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66299[] = {SGI282914, SGI282915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66300[] = {SGI282916, SGI282917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66301[] = {SGI282918, SGI282919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66302[] = {SGI282920, SGI282921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66303[] = {SGI282922, SGI282923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66304[] = {SGI282924, SGI282925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66305[] = {SGI282926, SGI282927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66306[] = {SGI282928, SGI282929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66307[] = {SGI282930, SGI282931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66308[] = {SGI282932, SGI282933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66309[] = {SGI282934, SGI282935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66310[] = {SGI282936, SGI282937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66311[] = {SGI282938, SGI282939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66312[] = {SGI282940, SGI282941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66313[] = {SGI282942, SGI282943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66314[] = {SGI282944, SGI282945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66315[] = {SGI282946, SGI282947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66316[] = {SGI282948, SGI282949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66317[] = {SGI282950, SGI282951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66318[] = {SGI282952, SGI282953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66319[] = {SGI282954, SGI282955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66320[] = {SGI282956, SGI282957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66321[] = {SGI282958, SGI282959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66322[] = {SGI282960, SGI282961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66323[] = {SGI282962, SGI282963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66324[] = {SGI282964, SGI282965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66325[] = {SGI282966, SGI282967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66326[] = {SGI282968, SGI282969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66327[] = {SGI282970, SGI282971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66328[] = {SGI282972, SGI282973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66329[] = {SGI282974, SGI282975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66330[] = {SGI282976, SGI282977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66331[] = {SGI282978, SGI282979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66332[] = {SGI282980, SGI282981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66333[] = {SGI282982, SGI282983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66334[] = {SGI282984, SGI282985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66335[] = {SGI282986, SGI282987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66336[] = {SGI282988, SGI282989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66337[] = {SGI282990, SGI282991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66338[] = {SGI282992, SGI282993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66339[] = {SGI282994, SGI282995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66340[] = {SGI282996, SGI282997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66341[] = {SGI282998, SGI282999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66342[] = {SGI283000, SGI283001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66343[] = {SGI283002, SGI283003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66344[] = {SGI283004, SGI283005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66345[] = {SGI283006, SGI283007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66346[] = {SGI283008, SGI283009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66347[] = {SGI283010, SGI283011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66348[] = {SGI283012, SGI283013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66349[] = {SGI283014, SGI283015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66350[] = {SGI283016, SGI283017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66351[] = {SGI283018, SGI283019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66352[] = {SGI283020, SGI283021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66353[] = {SGI283022, SGI283023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66354[] = {SGI283024, SGI283025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66355[] = {SGI283026, SGI283027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66356[] = {SGI283028, SGI283029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66357[] = {SGI283030, SGI283031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66358[] = {SGI283032, SGI283033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66359[] = {SGI283034, SGI283035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66360[] = {SGI283036, SGI283037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66361[] = {SGI283038, SGI283039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66362[] = {SGI283040, SGI283041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66363[] = {SGI283042, SGI283043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66364[] = {SGI283044, SGI283045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66365[] = {SGI283046, SGI283047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66366[] = {SGI283048, SGI283049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66367[] = {SGI283050, SGI283051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66368[] = {SGI283052, SGI283053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66369[] = {SGI283054, SGI283055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66370[] = {SGI283056, SGI283057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66371[] = {SGI283058, SGI283059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66372[] = {SGI283060, SGI283061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66373[] = {SGI283062, SGI283063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66374[] = {SGI283064, SGI283065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66375[] = {SGI283066, SGI283067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66376[] = {SGI283068, SGI283069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66377[] = {SGI283070, SGI283071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66378[] = {SGI283072, SGI283073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66379[] = {SGI283074, SGI283075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66380[] = {SGI283076, SGI283077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66381[] = {SGI283078, SGI283079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66382[] = {SGI283080, SGI283081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66383[] = {SGI283082, SGI283083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66384[] = {SGI283084, SGI283085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66385[] = {SGI283086, SGI283087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66386[] = {SGI283088, SGI283089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66387[] = {SGI283090, SGI283091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66388[] = {SGI283092, SGI283093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66389[] = {SGI283094, SGI283095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66390[] = {SGI283096, SGI283097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66391[] = {SGI283098, SGI283099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66392[] = {SGI283100, SGI283101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66393[] = {SGI283102, SGI283103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66394[] = {SGI283104, SGI283105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66395[] = {SGI283106, SGI283107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66396[] = {SGI283108, SGI283109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66397[] = {SGI283110, SGI283111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66398[] = {SGI283112, SGI283113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66399[] = {SGI283114, SGI283115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66400[] = {SGI283116, SGI283117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66401[] = {SGI283118, SGI283119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66402[] = {SGI283120, SGI283121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66403[] = {SGI283122, SGI283123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66404[] = {SGI283124, SGI283125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66405[] = {SGI283126, SGI283127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66406[] = {SGI283128, SGI283129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66407[] = {SGI283130, SGI283131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66408[] = {SGI283132, SGI283133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66409[] = {SGI283134, SGI283135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66410[] = {SGI283136, SGI283137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66411[] = {SGI283138, SGI283139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66412[] = {SGI283140, SGI283141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66413[] = {SGI283142, SGI283143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66414[] = {SGI283144, SGI283145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66415[] = {SGI283146, SGI283147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66416[] = {SGI283150, SGI283151, SGI283152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66417[] = {SGI283153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66418[] = {SGI283148, SGI283149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66419[] = {SGI283154, SGI283155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66420[] = {SGI283156, SGI283157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66421[] = {SGI283158, SGI283159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66422[] = {SGI283160, SGI283161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66423[] = {SGI283162, SGI283163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66424[] = {SGI283164, SGI283166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283168, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66425[] = {SGI283165, SGI283167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66426[] = {SGI283169, SGI283170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66427[] = {SGI283171, SGI283172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66428[] = {SGI283173, SGI283174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66429[] = {SGI283175, SGI283176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66430[] = {SGI283177, SGI283179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66431[] = {SGI283178, SGI283180, SGI0, SGI283181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66432[] = {SGI283182, SGI283183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66433[] = {SGI283184, SGI283185, SGI0, SGI283186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66434[] = {SGI283187, SGI283188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66435[] = {SGI283189, SGI283190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66436[] = {SGI283191, SGI283192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66437[] = {SGI283193, SGI283194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66438[] = {SGI283195, SGI283196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66439[] = {SGI283197, SGI283198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66440[] = {SGI283199, SGI283200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66441[] = {SGI283201, SGI283202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66442[] = {SGI283203, SGI283204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66443[] = {SGI283205, SGI283206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66444[] = {SGI283207, SGI283208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66445[] = {SGI283209, SGI283210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66446[] = {SGI283211, SGI283212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66447[] = {SGI283213, SGI283214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66448[] = {SGI283215, SGI283216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66449[] = {SGI283217, SGI283218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66450[] = {SGI283219, SGI283220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66451[] = {SGI283221, SGI283222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66452[] = {SGI283223, SGI283224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66453[] = {SGI283225, SGI283226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66454[] = {SGI283227, SGI283228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66455[] = {SGI283229, SGI283230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66456[] = {SGI283231, SGI283232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66457[] = {SGI283233, SGI283234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66458[] = {SGI283235, SGI283236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66459[] = {SGI283237, SGI283238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66460[] = {SGI283239, SGI283240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66461[] = {SGI283241, SGI283242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66462[] = {SGI283243, SGI283244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66463[] = {SGI283245, SGI283246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66464[] = {SGI283247, SGI283248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66465[] = {SGI283249, SGI283250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66466[] = {SGI283251, SGI283252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66467[] = {SGI283253, SGI283254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66468[] = {SGI283255, SGI283256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66469[] = {SGI283257, SGI283258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66470[] = {SGI283259, SGI283260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66471[] = {SGI283261, SGI283262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66472[] = {SGI283263, SGI283264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66473[] = {SGI283265, SGI283266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66474[] = {SGI283267, SGI283268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66475[] = {SGI283269, SGI283270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66476[] = {SGI283271, SGI283272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66477[] = {SGI283273, SGI283274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66478[] = {SGI283275, SGI283276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66479[] = {SGI283277, SGI283278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66480[] = {SGI283279, SGI283280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66481[] = {SGI283281, SGI283282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283283, SGI0, SGI0, NULL};
const Segment * const SG66482[] = {SGI283284, SGI283285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283286, SGI0, SGI0, NULL};
const Segment * const SG66483[] = {SGI283287, SGI283288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66484[] = {SGI283289, SGI283290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66485[] = {SGI283291, SGI283292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66486[] = {SGI283293, SGI283294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66487[] = {SGI283295, SGI283296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66488[] = {SGI283298, SGI283299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66489[] = {SGI283301, SGI283302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66490[] = {SGI283303, SGI283304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66491[] = {SGI283305, SGI283306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66492[] = {SGI283307, SGI283308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66493[] = {SGI283309, SGI283310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66494[] = {SGI283311, SGI283312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66495[] = {SGI283313, SGI283314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66496[] = {SGI283316, SGI283317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283319, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66497[] = {SGI283315, SGI283318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66498[] = {SGI283320, SGI283321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283322, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66499[] = {SGI283323, SGI283324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66500[] = {SGI283325, SGI283326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66501[] = {SGI283327, SGI283328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66502[] = {SGI283329, SGI283330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66503[] = {SGI283331, SGI283332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66504[] = {SGI283333, SGI283334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66505[] = {SGI283335, SGI283336, SGI0, SGI283337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66506[] = {SGI283338, SGI283339, SGI0, SGI283340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66507[] = {SGI283341, SGI283342, SGI0, SGI283343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66508[] = {SGI283344, SGI283345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66509[] = {SGI283347, SGI283348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66510[] = {SGI283350, SGI283351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66511[] = {SGI283352, SGI283353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66512[] = {SGI283354, SGI283355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283356, SGI0, SGI0, NULL};
const Segment * const SG66513[] = {SGI283357, SGI283358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283359, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66514[] = {SGI283360, SGI283361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283362, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66515[] = {SGI283363, SGI283364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66516[] = {SGI283366, SGI283367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66517[] = {SGI283369, SGI283370, SGI0, SGI283371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66518[] = {SGI283372, SGI283373, SGI0, SGI283374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66519[] = {SGI283375, SGI283376, SGI0, SGI283377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66520[] = {SGI283378, SGI283379, SGI0, SGI283380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66521[] = {SGI283381, SGI283382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283383, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66522[] = {SGI283384, SGI283385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283386, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66523[] = {SGI283387, SGI283388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66524[] = {SGI283390, SGI283391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66525[] = {SGI283393, SGI283394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66526[] = {SGI283396, SGI283397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66527[] = {SGI283399, SGI283400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66528[] = {SGI283402, SGI283403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66529[] = {SGI283404, SGI283405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283406, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66530[] = {SGI283407, SGI283408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283409, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66531[] = {SGI283410, SGI283411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283412, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66532[] = {SGI283413, SGI283414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283415, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66533[] = {SGI283416, SGI283417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66534[] = {SGI283419, SGI283420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66535[] = {SGI283422, SGI283423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66536[] = {SGI283424, SGI283425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66537[] = {SGI283426, SGI283427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66538[] = {SGI283428, SGI283429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66539[] = {SGI283430, SGI283431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66540[] = {SGI283432, SGI283433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66541[] = {SGI283434, SGI283435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66542[] = {SGI283436, SGI283437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66543[] = {SGI283438, SGI283439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66544[] = {SGI283440, SGI283441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283442, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66545[] = {SGI283443, SGI283444, SGI0, SGI0, SGI0, SGI0, SGI283445, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66546[] = {SGI283446, SGI283447, SGI0, SGI283448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66547[] = {SGI283449, SGI283450, SGI0, SGI283451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66548[] = {SGI283452, SGI283453, SGI0, SGI283454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66549[] = {SGI283455, SGI283456, SGI0, SGI283457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66550[] = {SGI283458, SGI283459, SGI0, SGI283460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66551[] = {SGI283461, SGI283462, SGI0, SGI283463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66552[] = {SGI283464, SGI283465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283466, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66553[] = {SGI283467, SGI283468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283469, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66554[] = {SGI283470, SGI283471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283472, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66555[] = {SGI283473, SGI283474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283475, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66556[] = {SGI283476, SGI283477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283478, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66557[] = {SGI283479, SGI283480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66558[] = {SGI283481, SGI283482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66559[] = {SGI283483, SGI283484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66560[] = {SGI283485, SGI283486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66561[] = {SGI283487, SGI283488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283489, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66562[] = {SGI283490, SGI283491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283492, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66563[] = {SGI283493, SGI283494, SGI0, SGI283495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66564[] = {SGI283496, SGI283497, SGI0, SGI283498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66565[] = {SGI283499, SGI283500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283501, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66566[] = {SGI283502, SGI283503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283504, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66567[] = {SGI283505, SGI283506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283507, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66568[] = {SGI283508, SGI283509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283510, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66569[] = {SGI283511, SGI283512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283513, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66570[] = {SGI283514, SGI283515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283516, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66571[] = {SGI283517, SGI283518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66572[] = {SGI283519, SGI283520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66573[] = {SGI283521, SGI283522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66574[] = {SGI283523, SGI283524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283525, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66575[] = {SGI283526, SGI283527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283528, SGI0, SGI0, NULL};
const Segment * const SG66576[] = {SGI283529, SGI283530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283531, SGI0, SGI0, NULL};
const Segment * const SG66577[] = {SGI283532, SGI283533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283534, SGI0, SGI0, NULL};
const Segment * const SG66578[] = {SGI283535, SGI283536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283537, SGI0, SGI0, NULL};
const Segment * const SG66579[] = {SGI283538, SGI283539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283540, SGI0, SGI0, NULL};
const Segment * const SG66580[] = {SGI283541, SGI283542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283543, SGI0, SGI0, NULL};
const Segment * const SG66581[] = {SGI283544, SGI283545, SGI0, SGI283546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66582[] = {SGI283547, SGI283548, SGI0, SGI283549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66583[] = {SGI283550, SGI283551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66584[] = {SGI283552, SGI283553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66585[] = {SGI283554, SGI283555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66586[] = {SGI283556, SGI283557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283558, SGI0, SGI0, NULL};
const Segment * const SG66587[] = {SGI283559, SGI283560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283561, SGI0, SGI0, NULL};
const Segment * const SG66588[] = {SGI283562, SGI283563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66589[] = {SGI283564, SGI283565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66590[] = {SGI283566, SGI283567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66591[] = {SGI283568, SGI283569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66592[] = {SGI283570, SGI283571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66593[] = {SGI283572, SGI283573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66594[] = {SGI283574, SGI283575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66595[] = {SGI283576, SGI283577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66596[] = {SGI283578, SGI283579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283580, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66597[] = {SGI283581, SGI283582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283583, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66598[] = {SGI283584, SGI283585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66599[] = {SGI283587, SGI283588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283589, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66600[] = {SGI283590, SGI283591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283592, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66601[] = {SGI283593, SGI283594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283595, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66602[] = {SGI283596, SGI283597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283598, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66603[] = {SGI283599, SGI283600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283601, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66604[] = {SGI283602, SGI283603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283604, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66605[] = {SGI283605, SGI283606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66606[] = {SGI283607, SGI283608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66607[] = {SGI283609, SGI283610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66608[] = {SGI283611, SGI283612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66609[] = {SGI283613, SGI283614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66610[] = {SGI283615, SGI283616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66611[] = {SGI283617, SGI283618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66612[] = {SGI283619, SGI283620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66613[] = {SGI283622, SGI283623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66614[] = {SGI283624, SGI283625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66615[] = {SGI283627, SGI283628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66616[] = {SGI283629, SGI283630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66617[] = {SGI283631, SGI283632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66618[] = {SGI283633, SGI283634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66619[] = {SGI283635, SGI283636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66620[] = {SGI283637, SGI283638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66621[] = {SGI283639, SGI283640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66622[] = {SGI283641, SGI283642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66623[] = {SGI283643, SGI283644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66624[] = {SGI283645, SGI283646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66625[] = {SGI283647, SGI283648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66626[] = {SGI283649, SGI283650, SGI0, SGI283651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66627[] = {SGI283652, SGI283653, SGI0, SGI283654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66628[] = {SGI283655, SGI283656, SGI0, SGI283657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66629[] = {SGI283658, SGI283659, SGI0, SGI283660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66630[] = {SGI283661, SGI283662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66631[] = {SGI283663, SGI283664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66632[] = {SGI283665, SGI283666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66633[] = {SGI283667, SGI283668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66634[] = {SGI283669, SGI283670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66635[] = {SGI283671, SGI283672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66636[] = {SGI283673, SGI283674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66637[] = {SGI283676, SGI283677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66638[] = {SGI283679, SGI283680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66639[] = {SGI283682, SGI283683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283684, NULL};
const Segment * const SG66640[] = {SGI283685, SGI283686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283687, NULL};
const Segment * const SG66641[] = {SGI283688, SGI283689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283690, NULL};
const Segment * const SG66642[] = {SGI283691, SGI283692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283693, NULL};
const Segment * const SG66643[] = {SGI283694, SGI283695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283696, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66644[] = {SGI283697, SGI283698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283699, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66645[] = {SGI283700, SGI283701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66646[] = {SGI283703, SGI283704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66647[] = {SGI283706, SGI283707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66648[] = {SGI283708, SGI283709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66649[] = {SGI283710, SGI283711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66650[] = {SGI283712, SGI283713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66651[] = {SGI283714, SGI283715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66652[] = {SGI283716, SGI283717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66653[] = {SGI283718, SGI283719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283720, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66654[] = {SGI283721, SGI283722, SGI0, SGI0, SGI283723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66655[] = {SGI283724, SGI283725, SGI0, SGI0, SGI283726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66656[] = {SGI283727, SGI283728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66657[] = {SGI283729, SGI283730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283731, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66658[] = {SGI283732, SGI283733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283734, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66659[] = {SGI283735, SGI283736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283737, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66660[] = {SGI283738, SGI283739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283740, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66661[] = {SGI283741, SGI283742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66662[] = {SGI283743, SGI283744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66663[] = {SGI283745, SGI283746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283747, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66664[] = {SGI283748, SGI283749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283750, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66665[] = {SGI283751, SGI283752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283753, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66666[] = {SGI283754, SGI283755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283756, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66667[] = {SGI283757, SGI283758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283759, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66668[] = {SGI283760, SGI283761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283762, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66669[] = {SGI283763, SGI283764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66670[] = {SGI283765, SGI283766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66671[] = {SGI283767, SGI283768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66672[] = {SGI283769, SGI283770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66673[] = {SGI283771, SGI283772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66674[] = {SGI283773, SGI283774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66675[] = {SGI283775, SGI283776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66676[] = {SGI283777, SGI283778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66677[] = {SGI283779, SGI283780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66678[] = {SGI283781, SGI283782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66679[] = {SGI283783, SGI283784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66680[] = {SGI283785, SGI283786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66681[] = {SGI283787, SGI283788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283789, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66682[] = {SGI283790, SGI283791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283792, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66683[] = {SGI283793, SGI283794, SGI0, SGI283795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66684[] = {SGI283796, SGI283797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283798, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66685[] = {SGI283799, SGI283800, SGI0, SGI283801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66686[] = {SGI283802, SGI283803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283804, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66687[] = {SGI283805, SGI283806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283807, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66688[] = {SGI283808, SGI283809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283810, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66689[] = {SGI283811, SGI283812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283813, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66690[] = {SGI283814, SGI283815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283816, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66691[] = {SGI283817, SGI283818, SGI0, SGI283819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66692[] = {SGI283820, SGI283821, SGI0, SGI283822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66693[] = {SGI283823, SGI283824, SGI0, SGI283825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66694[] = {SGI283826, SGI283827, SGI0, SGI283828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66695[] = {SGI283829, SGI283830, SGI0, SGI283831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66696[] = {SGI283832, SGI283833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66697[] = {SGI283834, SGI283835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66698[] = {SGI283837, SGI283838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66699[] = {SGI283840, SGI283841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66700[] = {SGI283843, SGI283844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283845, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66701[] = {SGI283846, SGI283847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283848, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66702[] = {SGI283849, SGI283850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283851, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66703[] = {SGI283852, SGI283853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283854, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66704[] = {SGI283855, SGI283856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283857, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66705[] = {SGI283858, SGI283859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66706[] = {SGI283861, SGI283862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66707[] = {SGI283864, SGI283865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66708[] = {SGI283866, SGI283867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI283868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66709[] = {SGI283869, SGI283870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66710[] = {SGI283871, SGI283872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66711[] = {SGI283873, SGI283874, SGI0, SGI283875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66712[] = {SGI283876, SGI283877, SGI0, SGI283878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66713[] = {SGI283879, SGI283880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66714[] = {SGI283881, SGI283882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66715[] = {SGI283883, SGI283884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66716[] = {SGI283885, SGI283886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66717[] = {SGI283887, SGI283888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66718[] = {SGI283889, SGI283890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66719[] = {SGI283891, SGI283892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66720[] = {SGI283893, SGI283894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66721[] = {SGI283895, SGI283896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66722[] = {SGI283897, SGI283898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66723[] = {SGI283899, SGI283900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66724[] = {SGI283901, SGI283902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66725[] = {SGI283903, SGI283904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66726[] = {SGI283905, SGI283906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66727[] = {SGI283907, SGI283908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66728[] = {SGI283909, SGI283910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66729[] = {SGI283911, SGI283912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66730[] = {SGI283913, SGI283914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66731[] = {SGI283915, SGI283916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66732[] = {SGI283917, SGI283918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66733[] = {SGI283919, SGI283920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66734[] = {SGI283921, SGI283922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66735[] = {SGI283923, SGI283924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66736[] = {SGI283925, SGI283926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66737[] = {SGI283927, SGI283928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66738[] = {SGI283929, SGI283930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66739[] = {SGI283931, SGI283932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66740[] = {SGI283933, SGI283934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66741[] = {SGI283935, SGI283936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66742[] = {SGI283937, SGI283938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66743[] = {SGI283939, SGI283940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66744[] = {SGI283941, SGI283942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66745[] = {SGI283943, SGI283944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66746[] = {SGI283945, SGI283946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66747[] = {SGI283947, SGI283948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66748[] = {SGI283949, SGI283950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66749[] = {SGI283951, SGI283952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66750[] = {SGI283953, SGI283954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66751[] = {SGI283955, SGI283956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66752[] = {SGI283957, SGI283958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66753[] = {SGI283959, SGI283960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66754[] = {SGI283961, SGI283962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66755[] = {SGI283963, SGI283964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66756[] = {SGI283965, SGI283966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66757[] = {SGI283967, SGI283968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66758[] = {SGI283969, SGI283970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66759[] = {SGI283971, SGI283972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66760[] = {SGI283973, SGI283974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66761[] = {SGI283975, SGI283976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66762[] = {SGI283977, SGI283978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66763[] = {SGI283979, SGI283980, SGI0, SGI283981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66764[] = {SGI283982, SGI283983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66765[] = {SGI283984, SGI283985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66766[] = {SGI283986, SGI283987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66767[] = {SGI283988, SGI283989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66768[] = {SGI283990, SGI283991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66769[] = {SGI283992, SGI283994, SGI0, SGI283996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66770[] = {SGI283993, SGI283995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66771[] = {SGI283997, SGI283998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66772[] = {SGI283999, SGI284000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66773[] = {SGI284001, SGI284002, SGI0, SGI0, SGI0, SGI0, SGI284003, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66774[] = {SGI284004, SGI284005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66775[] = {SGI284006, SGI284007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284008, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66776[] = {SGI284009, SGI284010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284011, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66777[] = {SGI284012, SGI284013, SGI0, SGI0, SGI0, SGI0, SGI284014, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66778[] = {SGI284015, SGI284016, SGI0, SGI284017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66779[] = {SGI284018, SGI284019, SGI0, SGI284020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66780[] = {SGI284021, SGI284022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66781[] = {SGI284023, SGI284024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66782[] = {SGI284025, SGI284026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66783[] = {SGI284027, SGI284028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66784[] = {SGI284029, SGI284030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66785[] = {SGI284031, SGI284032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284033, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66786[] = {SGI284034, SGI284035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284036, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66787[] = {SGI284037, SGI284038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284039, SGI0, NULL};
const Segment * const SG66788[] = {SGI284040, SGI284041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66789[] = {SGI284043, SGI284044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284045, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66790[] = {SGI284046, SGI284047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284048, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66791[] = {SGI284049, SGI284050, SGI0, SGI284051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66792[] = {SGI284052, SGI284053, SGI0, SGI284054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66793[] = {SGI284055, SGI284056, SGI0, SGI0, SGI284057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66794[] = {SGI284058, SGI284059, SGI0, SGI0, SGI284060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66795[] = {SGI284061, SGI284062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66796[] = {SGI284063, SGI284064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66797[] = {SGI284065, SGI284066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66798[] = {SGI284067, SGI284068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66799[] = {SGI284070, SGI284072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66800[] = {SGI284069, SGI284071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66801[] = {SGI284074, SGI284075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66802[] = {SGI284076, SGI284077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66803[] = {SGI284078, SGI284079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66804[] = {SGI284081, SGI284082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66805[] = {SGI284084, SGI284085, SGI0, SGI284086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66806[] = {SGI284087, SGI284088, SGI0, SGI284089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66807[] = {SGI284090, SGI284091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66808[] = {SGI284092, SGI284093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66809[] = {SGI284094, SGI284095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66810[] = {SGI284096, SGI284097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284098, SGI0, SGI0, NULL};
const Segment * const SG66811[] = {SGI284099, SGI284100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284101, SGI0, SGI0, NULL};
const Segment * const SG66812[] = {SGI284102, SGI284103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284104, SGI0, SGI0, NULL};
const Segment * const SG66813[] = {SGI284105, SGI284106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284107, SGI0, SGI0, NULL};
const Segment * const SG66814[] = {SGI284108, SGI284109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284110, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66815[] = {SGI284111, SGI284112, SGI284113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66816[] = {SGI284114, SGI284115, SGI284116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66817[] = {SGI284117, SGI284118, SGI284119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66818[] = {SGI284120, SGI284121, SGI284122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66819[] = {SGI284123, SGI284124, SGI284125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66820[] = {SGI284126, SGI284127, SGI284128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66821[] = {SGI284129, SGI284130, SGI284131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66822[] = {SGI284132, SGI284133, SGI284134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66823[] = {SGI284135, SGI284136, SGI284137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66824[] = {SGI284138, SGI284139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66825[] = {SGI284140, SGI284141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284142, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66826[] = {SGI284143, SGI284144, SGI0, SGI0, SGI0, SGI0, SGI284145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66827[] = {SGI284146, SGI284147, SGI0, SGI0, SGI0, SGI0, SGI284148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66828[] = {SGI284149, SGI284150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284151, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66829[] = {SGI284152, SGI284153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284154, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66830[] = {SGI284155, SGI284156, SGI284157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66831[] = {SGI284158, SGI284159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66832[] = {SGI284160, SGI284161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66833[] = {SGI284162, SGI284163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66834[] = {SGI284164, SGI284165, SGI0, SGI0, SGI284166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66835[] = {SGI284167, SGI284168, SGI0, SGI0, SGI284169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66836[] = {SGI284170, SGI284171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66837[] = {SGI284173, SGI284174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66838[] = {SGI284176, SGI284177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66839[] = {SGI284179, SGI284180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66840[] = {SGI284182, SGI284183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66841[] = {SGI284184, SGI284185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66842[] = {SGI284186, SGI284187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66843[] = {SGI284189, SGI284190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66844[] = {SGI284192, SGI284193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66845[] = {SGI284194, SGI284195, SGI0, SGI0, SGI0, SGI0, SGI284196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66846[] = {SGI284197, SGI284198, SGI0, SGI0, SGI0, SGI0, SGI284199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66847[] = {SGI284200, SGI284201, SGI0, SGI0, SGI284202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66848[] = {SGI284203, SGI284204, SGI0, SGI0, SGI284205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66849[] = {SGI284206, SGI284207, SGI0, SGI0, SGI284208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66850[] = {SGI284209, SGI284210, SGI0, SGI0, SGI284211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66851[] = {SGI284212, SGI284213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66852[] = {SGI284214, SGI284215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66853[] = {SGI284216, SGI284217, SGI284218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66854[] = {SGI284219, SGI284220, SGI284221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66855[] = {SGI284222, SGI284223, SGI0, SGI284224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66856[] = {SGI284225, SGI284226, SGI0, SGI284227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66857[] = {SGI284228, SGI284229, SGI0, SGI284230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66858[] = {SGI284231, SGI284232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66859[] = {SGI284234, SGI284235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66860[] = {SGI284237, SGI284238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66861[] = {SGI284239, SGI284240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66862[] = {SGI284241, SGI284242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66863[] = {SGI284243, SGI284244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66864[] = {SGI284245, SGI284246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66865[] = {SGI284247, SGI284248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284249, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66866[] = {SGI284250, SGI284251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284252, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66867[] = {SGI284253, SGI284254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284255, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66868[] = {SGI284256, SGI284257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284258, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66869[] = {SGI284259, SGI284260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284261, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66870[] = {SGI284262, SGI284263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284264, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66871[] = {SGI284265, SGI284266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284267, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66872[] = {SGI284268, SGI284269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66873[] = {SGI284271, SGI284272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66874[] = {SGI284274, SGI284275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66875[] = {SGI284276, SGI284277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284278, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66876[] = {SGI284279, SGI284280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284281, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66877[] = {SGI284282, SGI284283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66878[] = {SGI284284, SGI284285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66879[] = {SGI284286, SGI284287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66880[] = {SGI284288, SGI284289, SGI0, SGI0, SGI284290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66881[] = {SGI284291, SGI284292, SGI0, SGI0, SGI0, SGI0, SGI284293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66882[] = {SGI284294, SGI284295, SGI0, SGI0, SGI0, SGI0, SGI284296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66883[] = {SGI284297, SGI284298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66884[] = {SGI284300, SGI284301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284302, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66885[] = {SGI284303, SGI284304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66886[] = {SGI284306, SGI284307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66887[] = {SGI284309, SGI284310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284311, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66888[] = {SGI284312, SGI284313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284314, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66889[] = {SGI284315, SGI284316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284317, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66890[] = {SGI284318, SGI284319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66891[] = {SGI284320, SGI284321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66892[] = {SGI284322, SGI284323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66893[] = {SGI284324, SGI284325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66894[] = {SGI284326, SGI284327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66895[] = {SGI284328, SGI284329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66896[] = {SGI284330, SGI284331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66897[] = {SGI284332, SGI284333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66898[] = {SGI284334, SGI284335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66899[] = {SGI284336, SGI284337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66900[] = {SGI284339, SGI284340, SGI0, SGI0, SGI0, SGI0, SGI284341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66901[] = {SGI284342, SGI284343, SGI0, SGI0, SGI0, SGI0, SGI284344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66902[] = {SGI284345, SGI284346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66903[] = {SGI284347, SGI284348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66904[] = {SGI284349, SGI284350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66905[] = {SGI284351, SGI284352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66906[] = {SGI284353, SGI284354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66907[] = {SGI284356, SGI284357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66908[] = {SGI284359, SGI284360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66909[] = {SGI284361, SGI284362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66910[] = {SGI284363, SGI284364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66911[] = {SGI284365, SGI284366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66912[] = {SGI284367, SGI284368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66913[] = {SGI284369, SGI284370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66914[] = {SGI284371, SGI284372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66915[] = {SGI284373, SGI284374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66916[] = {SGI0, SGI284375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66917[] = {SGI284376, SGI284377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66918[] = {SGI284378, SGI284379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284380, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66919[] = {SGI284381, SGI284382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284383, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66920[] = {SGI284384, SGI284385, SGI0, SGI284386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66921[] = {SGI284387, SGI284388, SGI0, SGI284389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66922[] = {SGI284390, SGI284391, SGI0, SGI284392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66923[] = {SGI284393, SGI284394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66924[] = {SGI284396, SGI284397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66925[] = {SGI284399, SGI284400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66926[] = {SGI284401, SGI284402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66927[] = {SGI284403, SGI284404, SGI0, SGI284405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66928[] = {SGI284406, SGI284407, SGI0, SGI284408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66929[] = {SGI284409, SGI284410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66930[] = {SGI284412, SGI284413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284414, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66931[] = {SGI284415, SGI284416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284417, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66932[] = {SGI284418, SGI284419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284420, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66933[] = {SGI284421, SGI284422, SGI0, SGI0, SGI0, SGI0, SGI284423, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66934[] = {SGI284424, SGI284425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284426, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66935[] = {SGI284427, SGI284428, SGI284429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66936[] = {SGI284430, SGI284431, SGI284432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66937[] = {SGI284433, SGI284434, SGI284435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66938[] = {SGI284436, SGI284437, SGI284438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66939[] = {SGI284439, SGI284440, SGI284441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66940[] = {SGI284442, SGI284443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66941[] = {SGI284445, SGI284446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66942[] = {SGI284448, SGI284449, SGI284450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66943[] = {SGI284451, SGI284452, SGI284453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66944[] = {SGI284454, SGI284455, SGI284456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66945[] = {SGI284457, SGI284458, SGI0, SGI0, SGI284459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66946[] = {SGI284460, SGI284461, SGI0, SGI0, SGI284462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66947[] = {SGI284463, SGI284465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66948[] = {SGI284464, SGI284466, SGI0, SGI0, SGI284467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66949[] = {SGI284468, SGI284469, SGI0, SGI0, SGI284470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66950[] = {SGI284471, SGI284472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66951[] = {SGI284474, SGI284475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66952[] = {SGI284476, SGI284477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66953[] = {SGI284479, SGI284480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284481, SGI0, NULL};
const Segment * const SG66954[] = {SGI284482, SGI284483, SGI0, SGI284484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66955[] = {SGI284485, SGI284486, SGI0, SGI284487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66956[] = {SGI284488, SGI284489, SGI0, SGI0, SGI0, SGI0, SGI284490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66957[] = {SGI284491, SGI284492, NULL};
const Segment * const SG66958[] = {SGI284493, SGI284494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66959[] = {SGI284496, SGI284497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66960[] = {SGI284499, SGI284500, SGI0, SGI0, SGI0, SGI0, SGI284501, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66961[] = {SGI284502, SGI284503, SGI0, SGI0, SGI284504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66962[] = {SGI284505, SGI284506, SGI0, SGI0, SGI284507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66963[] = {SGI284508, SGI284509, SGI0, SGI0, SGI284510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66964[] = {SGI284511, SGI284512, SGI0, SGI0, SGI284513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66965[] = {SGI284514, SGI284515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284516, NULL};
const Segment * const SG66966[] = {SGI284517, SGI284518, SGI0, SGI284519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66967[] = {SGI284520, SGI284521, SGI0, SGI284522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66968[] = {SGI284523, SGI284524, SGI0, SGI284525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66969[] = {SGI284526, SGI284527, SGI0, SGI284528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66970[] = {SGI284529, SGI284530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66971[] = {SGI284531, SGI284532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66972[] = {SGI284533, SGI284534, SGI0, SGI0, SGI0, SGI0, SGI284535, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66973[] = {SGI284536, SGI284537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66974[] = {SGI284539, SGI284540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66975[] = {SGI284542, SGI284543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66976[] = {SGI284545, SGI284546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66977[] = {SGI284548, SGI284549, SGI284550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66978[] = {SGI284551, SGI284552, SGI284553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66979[] = {SGI284554, SGI284555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66980[] = {SGI284557, SGI284558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66981[] = {SGI284560, SGI284561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66982[] = {SGI284563, SGI284564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66983[] = {SGI284566, SGI284567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66984[] = {SGI284569, SGI284570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66985[] = {SGI284572, SGI284573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284574, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66986[] = {SGI284575, SGI284576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284577, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66987[] = {SGI284578, SGI284579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66988[] = {SGI284580, SGI284581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66989[] = {SGI284582, SGI284583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66990[] = {SGI284584, SGI284585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66991[] = {SGI284587, SGI284588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66992[] = {SGI284590, SGI284591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284592, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66993[] = {SGI284593, SGI284594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66994[] = {SGI284595, SGI284596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66995[] = {SGI284597, SGI284598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66996[] = {SGI284600, SGI284601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66997[] = {SGI284603, SGI284604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66998[] = {SGI284606, SGI284607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG66999[] = {SGI284609, SGI284610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284611, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67000[] = {SGI284612, SGI284613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284614, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67001[] = {SGI284615, SGI284616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284617, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67002[] = {SGI284618, SGI284619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284620, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67003[] = {SGI284621, SGI284622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284623, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67004[] = {SGI284624, SGI284625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67005[] = {SGI284627, SGI284628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67006[] = {SGI284630, SGI284631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67007[] = {SGI284633, SGI284634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67008[] = {SGI284635, SGI284636, SGI0, SGI284637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67009[] = {SGI284638, SGI284639, SGI0, SGI284640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67010[] = {SGI284641, SGI284642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67011[] = {SGI284644, SGI284645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67012[] = {SGI284647, SGI284648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67013[] = {SGI284650, SGI284651, SGI0, SGI0, SGI0, SGI0, SGI284652, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67014[] = {SGI284653, SGI284654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67015[] = {SGI284656, SGI284657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67016[] = {SGI284658, SGI284659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67017[] = {SGI284660, SGI284661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67018[] = {SGI284662, SGI284663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284664, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67019[] = {SGI284665, SGI284666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284667, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67020[] = {SGI284668, SGI284669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284670, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67021[] = {SGI284671, SGI284672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284673, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67022[] = {SGI284674, SGI284675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67023[] = {SGI284677, SGI284678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67024[] = {SGI284680, SGI284681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67025[] = {SGI284683, SGI284684, SGI0, SGI0, SGI284685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67026[] = {SGI284686, SGI284687, SGI0, SGI0, SGI284688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67027[] = {SGI284689, SGI284690, SGI0, SGI0, SGI0, SGI0, SGI284691, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67028[] = {SGI284692, SGI284693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67029[] = {SGI284695, SGI284696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67030[] = {SGI284698, SGI284699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67031[] = {SGI284701, SGI284702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67032[] = {SGI284704, SGI284705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284706, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67033[] = {SGI284707, SGI284708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284709, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67034[] = {SGI284710, SGI284711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284712, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67035[] = {SGI284713, SGI284714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284715, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67036[] = {SGI284716, SGI284717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284718, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67037[] = {SGI284719, SGI284720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67038[] = {SGI284721, SGI284722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67039[] = {SGI284723, SGI284724, SGI0, SGI0, SGI0, SGI0, SGI284725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67040[] = {SGI284726, SGI284727, SGI0, SGI0, SGI0, SGI0, SGI284728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67041[] = {SGI284729, SGI284730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67042[] = {SGI284732, SGI284733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67043[] = {SGI284735, SGI284736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67044[] = {SGI284738, SGI284739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67045[] = {SGI284741, SGI284742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67046[] = {SGI284743, SGI284744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67047[] = {SGI284745, SGI284746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67048[] = {SGI284747, SGI284748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67049[] = {SGI284749, SGI284750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67050[] = {SGI284751, SGI284752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67051[] = {SGI284753, SGI284754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67052[] = {SGI284755, SGI284756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67053[] = {SGI284757, SGI284758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67054[] = {SGI284759, SGI284760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67055[] = {SGI284761, SGI284762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67056[] = {SGI284763, SGI284764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67057[] = {SGI284765, SGI284766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67058[] = {SGI284768, SGI284769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67059[] = {SGI284771, SGI284772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67060[] = {SGI284774, SGI284775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67061[] = {SGI284777, SGI284778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67062[] = {SGI284780, SGI284781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67063[] = {SGI284783, SGI284784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67064[] = {SGI284786, SGI284787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67065[] = {SGI284789, SGI284790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284791, SGI0, SGI0, NULL};
const Segment * const SG67066[] = {SGI284792, SGI284793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284794, SGI0, SGI0, NULL};
const Segment * const SG67067[] = {SGI284795, SGI284796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67068[] = {SGI284797, SGI284798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67069[] = {SGI284800, SGI284801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67070[] = {SGI284803, SGI284804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284805, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67071[] = {SGI284806, SGI284807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67072[] = {SGI284808, SGI284809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67073[] = {SGI284811, SGI284812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67074[] = {SGI284814, SGI284815, NULL};
const Segment * const SG67075[] = {SGI284816, SGI284817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67076[] = {SGI284819, SGI284820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67077[] = {SGI284822, SGI284823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284824, SGI0, SGI0, NULL};
const Segment * const SG67078[] = {SGI284825, SGI284826, SGI0, SGI0, SGI0, SGI0, SGI284827, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67079[] = {SGI284828, SGI284829, SGI0, SGI0, SGI0, SGI0, SGI284830, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67080[] = {SGI284831, SGI284832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67081[] = {SGI284833, SGI284834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284835, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67082[] = {SGI284836, SGI284837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284838, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67083[] = {SGI284839, SGI284840, SGI284841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67084[] = {SGI284842, SGI284843, SGI284844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67085[] = {SGI284845, SGI284846, SGI284847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67086[] = {SGI284848, SGI284849, SGI0, SGI0, SGI0, SGI0, SGI284850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67087[] = {SGI284851, SGI284852, SGI0, SGI0, SGI0, SGI0, SGI284853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67088[] = {SGI284854, SGI284855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67089[] = {SGI284857, SGI284858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67090[] = {SGI284860, SGI284861, SGI0, SGI0, SGI0, SGI0, SGI284862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67091[] = {SGI284863, SGI284864, SGI284865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67092[] = {SGI284866, SGI284867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284868, SGI0, SGI0, NULL};
const Segment * const SG67093[] = {SGI284869, SGI284870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284871, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67094[] = {SGI284872, SGI284873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284874, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67095[] = {SGI284875, SGI284876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284877, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67096[] = {SGI284878, SGI284879, SGI284880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67097[] = {SGI284881, SGI284882, SGI284883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67098[] = {SGI284884, SGI284885, SGI284886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67099[] = {SGI284887, SGI284888, SGI284889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67100[] = {SGI284890, SGI284891, SGI284892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67101[] = {SGI284893, SGI284894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67102[] = {SGI284896, SGI284897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67103[] = {SGI284899, SGI284900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67104[] = {SGI284902, SGI284903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67105[] = {SGI284905, SGI284906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67106[] = {SGI284908, SGI284909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67107[] = {SGI284911, SGI284912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67108[] = {SGI284914, SGI284915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67109[] = {SGI284917, SGI284918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67110[] = {SGI284920, SGI284921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67111[] = {SGI284922, SGI284923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67112[] = {SGI284924, SGI284925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67113[] = {SGI284927, SGI284928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67114[] = {SGI284930, SGI284931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67115[] = {SGI284933, SGI284934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67116[] = {SGI284936, SGI284937, SGI0, SGI0, SGI284938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67117[] = {SGI284939, SGI284940, SGI284941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67118[] = {SGI284942, SGI284943, SGI284944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67119[] = {SGI284945, SGI284946, SGI284947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67120[] = {SGI284948, SGI284949, SGI284950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67121[] = {SGI284951, SGI284952, SGI284953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67122[] = {SGI284954, SGI284955, SGI284956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67123[] = {SGI284957, SGI284958, SGI284959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67124[] = {SGI284960, SGI284961, SGI284962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67125[] = {SGI284963, SGI284964, SGI284965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67126[] = {SGI284966, SGI284967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67127[] = {SGI284968, SGI284969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67128[] = {SGI284970, SGI284971, SGI0, SGI284972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67129[] = {SGI284973, SGI284974, SGI0, SGI284975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67130[] = {SGI284976, SGI284977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284978, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67131[] = {SGI284979, SGI284980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284981, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67132[] = {SGI284982, SGI284983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67133[] = {SGI284984, SGI284985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI284986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67134[] = {SGI284987, SGI284988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67135[] = {SGI284989, SGI284990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67136[] = {SGI284991, SGI284992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67137[] = {SGI284993, SGI284994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67138[] = {SGI284995, SGI284996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67139[] = {SGI284997, SGI284998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67140[] = {SGI284999, SGI285000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67141[] = {SGI285001, SGI285002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67142[] = {SGI285003, SGI285004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67143[] = {SGI285005, SGI285006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67144[] = {SGI285007, SGI285008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67145[] = {SGI285009, SGI285010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67146[] = {SGI285011, SGI285012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67147[] = {SGI285013, SGI285014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67148[] = {SGI285015, SGI285016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67149[] = {SGI285017, SGI285018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67150[] = {SGI285019, SGI285020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67151[] = {SGI285021, SGI285022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67152[] = {SGI285023, SGI285024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67153[] = {SGI285025, SGI285026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67154[] = {SGI285027, SGI285028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67155[] = {SGI285029, SGI285030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67156[] = {SGI285031, SGI285032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67157[] = {SGI285033, SGI285034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67158[] = {SGI285035, SGI285036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67159[] = {SGI285037, SGI285038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67160[] = {SGI285039, SGI285040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67161[] = {SGI285041, SGI285042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67162[] = {SGI285043, SGI285044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67163[] = {SGI285045, SGI285046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67164[] = {SGI285047, SGI285048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67165[] = {SGI285049, SGI285050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67166[] = {SGI285051, SGI285052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67167[] = {SGI285053, SGI285054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67168[] = {SGI285055, SGI285056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67169[] = {SGI285057, SGI285058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67170[] = {SGI285059, SGI285060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67171[] = {SGI285061, SGI285062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67172[] = {SGI285063, SGI285064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67173[] = {SGI285065, SGI285066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67174[] = {SGI285067, SGI285068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67175[] = {SGI285069, SGI285070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67176[] = {SGI285071, SGI285072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67177[] = {SGI285073, SGI285074, SGI0, SGI285075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67178[] = {SGI285076, SGI285077, SGI0, SGI285078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67179[] = {SGI285079, SGI285080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67180[] = {SGI285082, SGI285083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67181[] = {SGI285085, SGI285086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67182[] = {SGI285087, SGI285088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67183[] = {SGI285090, SGI285091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67184[] = {SGI285093, SGI285094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285095, SGI0, SGI0, NULL};
const Segment * const SG67185[] = {SGI285096, SGI285097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67186[] = {SGI285099, SGI285100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67187[] = {SGI285101, SGI285102, SGI0, SGI0, SGI285103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67188[] = {SGI285104, SGI285105, SGI0, SGI0, SGI285106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67189[] = {SGI285107, SGI285108, NULL};
const Segment * const SG67190[] = {SGI285109, SGI285110, NULL};
const Segment * const SG67191[] = {SGI285111, SGI285112, NULL};
const Segment * const SG67192[] = {SGI285113, SGI285114, SGI285115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67193[] = {SGI285116, SGI285117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67194[] = {SGI285118, SGI285119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67195[] = {SGI285121, SGI285122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67196[] = {SGI285124, SGI285125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67197[] = {SGI285126, SGI285127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67198[] = {SGI285128, SGI285129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67199[] = {SGI285130, SGI285131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67200[] = {SGI285133, SGI285134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67201[] = {SGI285136, SGI285137, SGI0, SGI285138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67202[] = {SGI285139, SGI285140, SGI0, SGI285141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67203[] = {SGI285142, SGI285143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285144, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67204[] = {SGI285145, SGI285146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67205[] = {SGI285147, SGI285148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67206[] = {SGI285149, SGI285150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67207[] = {SGI285151, SGI285152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67208[] = {SGI285153, SGI285154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67209[] = {SGI285155, SGI285156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67210[] = {SGI285157, SGI285158, SGI0, SGI285159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67211[] = {SGI285160, SGI285161, SGI0, SGI285162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67212[] = {SGI285163, SGI285164, NULL};
const Segment * const SG67213[] = {SGI285165, SGI285166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67214[] = {SGI285168, SGI285169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67215[] = {SGI285171, SGI285172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67216[] = {SGI285174, SGI285175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67217[] = {SGI285177, SGI285178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67218[] = {SGI285180, SGI285181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67219[] = {SGI285182, SGI285183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67220[] = {SGI285184, SGI285185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67221[] = {SGI285186, SGI285187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67222[] = {SGI285189, SGI285190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67223[] = {SGI285192, SGI285193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67224[] = {SGI285194, SGI285195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67225[] = {SGI285196, SGI285197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67226[] = {SGI285199, SGI285200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67227[] = {SGI285202, SGI285203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67228[] = {SGI285205, SGI285206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285207, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67229[] = {SGI285208, SGI285209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285210, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67230[] = {SGI285211, SGI285212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67231[] = {SGI285214, SGI285215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67232[] = {SGI285217, SGI285218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67233[] = {SGI285220, SGI285221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67234[] = {SGI285223, SGI285224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67235[] = {SGI285226, SGI285227, SGI0, SGI0, SGI0, SGI0, SGI285228, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67236[] = {SGI285229, SGI285230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67237[] = {SGI285231, SGI285232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285233, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67238[] = {SGI285234, SGI285235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285236, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67239[] = {SGI285237, SGI285238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67240[] = {SGI285239, SGI285240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67241[] = {SGI285241, SGI285242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67242[] = {SGI285243, SGI285244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67243[] = {SGI285245, SGI285246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67244[] = {SGI285247, SGI285248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67245[] = {SGI285250, SGI285251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67246[] = {SGI285253, SGI285254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285255, SGI0, SGI0, NULL};
const Segment * const SG67247[] = {SGI285256, SGI285257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285258, SGI0, SGI0, NULL};
const Segment * const SG67248[] = {SGI285259, SGI285260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67249[] = {SGI285262, SGI285263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67250[] = {SGI285265, SGI285266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67251[] = {SGI285268, SGI285269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67252[] = {SGI285271, SGI285272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67253[] = {SGI285273, SGI285274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67254[] = {SGI285275, SGI285276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67255[] = {SGI285277, SGI285278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67256[] = {SGI285279, SGI285280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67257[] = {SGI285282, SGI285283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67258[] = {SGI285285, SGI285286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285287, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67259[] = {SGI285288, SGI285289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285290, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67260[] = {SGI285291, SGI285292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285293, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67261[] = {SGI285294, SGI285295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67262[] = {SGI285297, SGI285298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67263[] = {SGI285300, SGI285301, SGI0, SGI0, SGI285302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67264[] = {SGI285303, SGI285304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67265[] = {SGI285306, SGI285307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67266[] = {SGI285309, SGI285310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67267[] = {SGI285312, SGI285313, SGI0, SGI285314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67268[] = {SGI285315, SGI285316, SGI0, SGI285317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67269[] = {SGI285318, SGI285319, SGI0, SGI0, SGI285320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67270[] = {SGI285321, SGI285322, SGI0, SGI0, SGI285323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67271[] = {SGI285324, SGI285325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67272[] = {SGI285327, SGI285328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67273[] = {SGI285330, SGI285331, NULL};
const Segment * const SG67274[] = {SGI285332, SGI285333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67275[] = {SGI285335, SGI285336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67276[] = {SGI285337, SGI285338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285339, NULL};
const Segment * const SG67277[] = {SGI285340, SGI285341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67278[] = {SGI285342, SGI285343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67279[] = {SGI285344, SGI285345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67280[] = {SGI285347, SGI285348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67281[] = {SGI285350, SGI285351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67282[] = {SGI285353, SGI285354, SGI285355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67283[] = {SGI285356, SGI285357, SGI285358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67284[] = {SGI285359, SGI285360, SGI285361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67285[] = {SGI285362, SGI285363, SGI0, SGI0, SGI285364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67286[] = {SGI285365, SGI285366, SGI0, SGI0, SGI285367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67287[] = {SGI285368, SGI285369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67288[] = {SGI285370, SGI285371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67289[] = {SGI285372, SGI285373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285374, SGI0, SGI0, NULL};
const Segment * const SG67290[] = {SGI285375, SGI285376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285377, SGI0, SGI0, NULL};
const Segment * const SG67291[] = {SGI285378, SGI285379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67292[] = {SGI285381, SGI285382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67293[] = {SGI285383, SGI285384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67294[] = {SGI285386, SGI285387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67295[] = {SGI285389, SGI285390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285391, NULL};
const Segment * const SG67296[] = {SGI285392, SGI285393, SGI0, SGI0, SGI0, SGI0, SGI285394, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67297[] = {SGI285395, SGI285396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285397, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67298[] = {SGI285398, SGI285399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67299[] = {SGI285400, SGI285401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67300[] = {SGI285402, SGI285403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67301[] = {SGI285404, SGI285405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67302[] = {SGI285406, SGI285407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285408, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67303[] = {SGI285409, SGI285410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285411, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67304[] = {SGI285412, SGI285413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285414, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67305[] = {SGI285415, SGI285416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67306[] = {SGI285417, SGI285418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67307[] = {SGI285419, SGI285420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67308[] = {SGI285421, SGI285422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67309[] = {SGI285423, SGI285424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67310[] = {SGI285425, SGI285426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67311[] = {SGI285427, SGI285428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67312[] = {SGI285429, SGI285430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67313[] = {SGI285431, SGI285432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67314[] = {SGI285433, SGI285434, NULL};
const Segment * const SG67315[] = {SGI285435, SGI285436, NULL};
const Segment * const SG67316[] = {SGI285437, SGI285438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67317[] = {SGI285440, SGI285441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67318[] = {SGI285443, SGI285444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67319[] = {SGI285446, SGI285447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67320[] = {SGI285449, SGI285450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67321[] = {SGI285452, SGI285453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67322[] = {SGI285455, SGI285456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285457, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67323[] = {SGI285458, SGI285459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285460, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67324[] = {SGI285461, SGI285462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67325[] = {SGI285463, SGI285464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67326[] = {SGI285465, SGI285466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67327[] = {SGI285467, SGI285468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67328[] = {SGI285469, SGI285470, NULL};
const Segment * const SG67329[] = {SGI285471, SGI285472, SGI0, SGI285473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67330[] = {SGI285474, SGI285475, SGI0, SGI285476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67331[] = {SGI285477, SGI285478, NULL};
const Segment * const SG67332[] = {SGI285479, SGI285480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67333[] = {SGI285482, SGI285483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285484, SGI0, SGI0, NULL};
const Segment * const SG67334[] = {SGI285485, SGI285486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285487, SGI0, SGI0, NULL};
const Segment * const SG67335[] = {SGI285488, SGI285489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285490, SGI0, SGI0, NULL};
const Segment * const SG67336[] = {SGI285491, SGI285492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67337[] = {SGI285493, SGI285494, SGI0, SGI0, SGI0, SGI0, SGI285495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67338[] = {SGI285496, SGI285497, SGI0, SGI0, SGI0, SGI0, SGI285498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67339[] = {SGI285499, SGI285500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285501, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67340[] = {SGI285502, SGI285503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285504, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67341[] = {SGI285505, SGI285506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285507, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67342[] = {SGI285508, SGI285509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285510, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67343[] = {SGI285511, SGI285512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285513, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67344[] = {SGI285514, SGI285515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285516, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67345[] = {SGI285517, SGI285518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285519, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67346[] = {SGI285520, SGI285521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285522, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67347[] = {SGI285523, SGI285524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285525, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67348[] = {SGI285526, SGI285527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285528, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67349[] = {SGI285529, SGI285530, SGI0, SGI0, SGI285531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67350[] = {SGI285532, SGI285533, SGI0, SGI0, SGI285534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67351[] = {SGI285535, SGI285536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67352[] = {SGI285538, SGI285539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67353[] = {SGI285541, SGI285542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67354[] = {SGI285544, SGI285545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67355[] = {SGI285547, SGI285548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67356[] = {SGI285550, SGI285551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67357[] = {SGI285553, SGI285554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67358[] = {SGI285555, SGI285556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67359[] = {SGI285557, SGI285558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67360[] = {SGI285559, SGI285560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67361[] = {SGI285561, SGI285562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67362[] = {SGI285563, SGI285564, SGI285565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67363[] = {SGI285566, SGI285567, SGI285568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67364[] = {SGI285569, SGI285570, SGI285571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67365[] = {SGI285572, SGI285573, SGI285574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67366[] = {SGI285575, SGI285576, SGI285577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67367[] = {SGI285578, SGI285579, SGI285580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67368[] = {SGI285581, SGI285582, SGI285583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67369[] = {SGI285584, SGI285585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285586, NULL};
const Segment * const SG67370[] = {SGI285587, SGI285588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67371[] = {SGI285589, SGI285590, SGI0, SGI285591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67372[] = {SGI285592, SGI285593, NULL};
const Segment * const SG67373[] = {SGI285594, SGI285595, NULL};
const Segment * const SG67374[] = {SGI285596, SGI285597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67375[] = {SGI285599, SGI285600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67376[] = {SGI285602, SGI285603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67377[] = {SGI285605, SGI285606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67378[] = {SGI285608, SGI285609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67379[] = {SGI285610, SGI285611, SGI0, SGI285612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67380[] = {SGI285613, SGI285614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67381[] = {SGI285616, SGI285617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285618, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67382[] = {SGI285619, SGI285620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285621, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67383[] = {SGI285622, SGI285623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285624, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67384[] = {SGI285625, SGI285626, SGI0, SGI0, SGI0, SGI0, SGI285627, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67385[] = {SGI285628, SGI285629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285630, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67386[] = {SGI285631, SGI285632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285633, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67387[] = {SGI285634, SGI285635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285636, SGI0, SGI0, NULL};
const Segment * const SG67388[] = {SGI285637, SGI285638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285639, SGI0, SGI0, NULL};
const Segment * const SG67389[] = {SGI285640, SGI285641, SGI0, SGI0, SGI285642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67390[] = {SGI285643, SGI285644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285645, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67391[] = {SGI285646, SGI285647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67392[] = {SGI285648, SGI285649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67393[] = {SGI285650, SGI285651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67394[] = {SGI285652, SGI285653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67395[] = {SGI285654, SGI285655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67396[] = {SGI285657, SGI285659, NULL};
const Segment * const SG67397[] = {SGI285656, SGI285658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67398[] = {SGI285660, SGI285661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67399[] = {SGI285662, SGI285663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67400[] = {SGI285664, SGI285665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67401[] = {SGI285666, SGI285667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67402[] = {SGI285668, SGI285669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67403[] = {SGI285670, SGI285671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67404[] = {SGI285672, SGI285673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67405[] = {SGI285674, SGI285675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67406[] = {SGI285676, SGI285677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67407[] = {SGI285678, SGI285679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67408[] = {SGI285680, SGI285681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67409[] = {SGI285682, SGI285683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67410[] = {SGI285684, SGI285685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67411[] = {SGI285686, SGI285687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67412[] = {SGI285688, SGI285689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67413[] = {SGI285690, SGI285691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67414[] = {SGI285692, SGI285693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67415[] = {SGI285694, SGI285695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67416[] = {SGI285696, SGI285697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67417[] = {SGI285698, SGI285699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67418[] = {SGI285700, SGI285701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67419[] = {SGI285702, SGI285703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67420[] = {SGI285704, SGI285705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67421[] = {SGI285706, SGI285707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67422[] = {SGI285708, SGI285709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67423[] = {SGI285710, SGI285711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67424[] = {SGI285712, SGI285713, SGI0, SGI0, SGI0, SGI0, SGI285714, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67425[] = {SGI285715, SGI285716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67426[] = {SGI285718, SGI285719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67427[] = {SGI285721, SGI285722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67428[] = {SGI285723, SGI285724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67429[] = {SGI285725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67430[] = {SGI285726, SGI285727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285728, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67431[] = {SGI285729, SGI285730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285731, NULL};
const Segment * const SG67432[] = {SGI285732, SGI285733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285734, NULL};
const Segment * const SG67433[] = {SGI285735, SGI285736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67434[] = {SGI285738, SGI285739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67435[] = {SGI285741, SGI285742, SGI0, SGI0, SGI285743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67436[] = {SGI285744, SGI285745, SGI0, SGI0, SGI285746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67437[] = {SGI285747, SGI285748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67438[] = {SGI285750, SGI285751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67439[] = {SGI285753, SGI285754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67440[] = {SGI285755, SGI285756, SGI0, SGI0, SGI285757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67441[] = {SGI285758, SGI285759, SGI0, SGI0, SGI285760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67442[] = {SGI285761, SGI285762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67443[] = {SGI285763, SGI285764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67444[] = {SGI285765, SGI285766, NULL};
const Segment * const SG67445[] = {SGI285767, SGI285768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67446[] = {SGI285770, SGI285771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67447[] = {SGI285773, SGI285774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67448[] = {SGI285776, SGI285777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67449[] = {SGI285779, SGI285780, SGI285781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67450[] = {SGI285782, SGI285783, SGI285784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67451[] = {SGI285785, SGI285786, SGI285787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67452[] = {SGI285788, SGI285789, SGI285790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67453[] = {SGI285791, SGI285792, SGI0, SGI285793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67454[] = {SGI285794, SGI285795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67455[] = {SGI285796, SGI285797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67456[] = {SGI285799, SGI285800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67457[] = {SGI285802, SGI285803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67458[] = {SGI285804, SGI285805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67459[] = {SGI285806, SGI285807, NULL};
const Segment * const SG67460[] = {SGI285808, SGI285809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285810, SGI0, NULL};
const Segment * const SG67461[] = {SGI285811, SGI285812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67462[] = {SGI285813, SGI285814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67463[] = {SGI285815, SGI285816, NULL};
const Segment * const SG67464[] = {SGI285817, SGI285818, SGI0, SGI0, SGI0, SGI0, SGI285819, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67465[] = {SGI285820, SGI285821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67466[] = {SGI285822, SGI285823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67467[] = {SGI285825, SGI285826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67468[] = {SGI285828, SGI285829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67469[] = {SGI285831, SGI285832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67470[] = {SGI285833, SGI285834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67471[] = {SGI285835, SGI285836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67472[] = {SGI285837, SGI285838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67473[] = {SGI285839, SGI285840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67474[] = {SGI0, SGI285841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67475[] = {SGI285842, SGI285843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67476[] = {SGI285845, SGI285846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67477[] = {SGI285847, SGI285848, SGI0, SGI285849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67478[] = {SGI285850, SGI285851, SGI0, SGI285852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67479[] = {SGI285853, SGI285854, SGI0, SGI285855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67480[] = {SGI285856, SGI285857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285858, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67481[] = {SGI285859, SGI285860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285861, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67482[] = {SGI285862, SGI285863, SGI0, SGI285864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67483[] = {SGI285865, SGI285866, SGI0, SGI0, SGI0, SGI0, SGI285867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67484[] = {SGI285868, SGI285869, SGI0, SGI0, SGI0, SGI0, SGI285870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67485[] = {SGI285871, SGI285872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285873, SGI0, SGI0, NULL};
const Segment * const SG67486[] = {SGI285874, SGI285875, SGI0, SGI285876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67487[] = {SGI285877, SGI285878, SGI0, SGI285879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67488[] = {SGI285880, SGI285881, SGI0, SGI0, SGI0, SGI0, SGI285882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67489[] = {SGI285883, SGI285884, SGI0, SGI0, SGI0, SGI0, SGI285885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67490[] = {SGI285886, SGI285887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67491[] = {SGI285888, SGI285889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67492[] = {SGI285890, SGI285891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67493[] = {SGI285892, SGI285893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67494[] = {SGI285894, SGI285895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67495[] = {SGI285896, SGI285897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67496[] = {SGI285898, SGI285899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67497[] = {SGI285901, SGI285902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285903, NULL};
const Segment * const SG67498[] = {SGI285904, SGI285905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67499[] = {SGI285906, SGI285907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67500[] = {SGI285908, SGI285909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67501[] = {SGI285911, SGI285912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67502[] = {SGI285914, SGI285915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67503[] = {SGI285917, SGI285918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67504[] = {SGI285920, SGI285921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285922, NULL};
const Segment * const SG67505[] = {SGI285923, SGI285924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67506[] = {SGI285925, SGI285926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67507[] = {SGI285927, SGI285928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285929, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67508[] = {SGI285930, SGI285931, NULL};
const Segment * const SG67509[] = {SGI285932, SGI285933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285934, SGI0, SGI0, NULL};
const Segment * const SG67510[] = {SGI285935, SGI285936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67511[] = {SGI285937, SGI285938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67512[] = {SGI285939, SGI285940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67513[] = {SGI285942, SGI285943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67514[] = {SGI285945, SGI285946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67515[] = {SGI285947, SGI285948, SGI0, SGI0, SGI0, SGI0, SGI285949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67516[] = {SGI285950, SGI285951, SGI0, SGI0, SGI0, SGI0, SGI285952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67517[] = {SGI285953, SGI285954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67518[] = {SGI285955, SGI285956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67519[] = {SGI285958, SGI285959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67520[] = {SGI285961, SGI285962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285963, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67521[] = {SGI285964, SGI285965, SGI0, SGI0, SGI285966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67522[] = {SGI285967, SGI285968, SGI0, SGI0, SGI285969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67523[] = {SGI285970, SGI285971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67524[] = {SGI285973, SGI285974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67525[] = {SGI285976, SGI285977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67526[] = {SGI285979, SGI285980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67527[] = {SGI285982, SGI285983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285984, SGI0, SGI0, NULL};
const Segment * const SG67528[] = {SGI285985, SGI285986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67529[] = {SGI285988, SGI285989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67530[] = {SGI285991, SGI285992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67531[] = {SGI285993, SGI285994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67532[] = {SGI285996, SGI285997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI285998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67533[] = {SGI285999, SGI286000, NULL};
const Segment * const SG67534[] = {SGI286001, SGI286002, SGI0, SGI286003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67535[] = {SGI286004, SGI286005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286006, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67536[] = {SGI286007, SGI286008, NULL};
const Segment * const SG67537[] = {SGI286009, SGI286010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286011, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67538[] = {SGI286012, SGI286013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67539[] = {SGI286014, SGI286015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67540[] = {SGI286017, SGI286018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67541[] = {SGI286020, SGI286021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286022, NULL};
const Segment * const SG67542[] = {SGI286023, SGI286024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67543[] = {SGI286025, SGI286026, NULL};
const Segment * const SG67544[] = {SGI286027, SGI286028, SGI0, SGI0, SGI286029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67545[] = {SGI286030, SGI286031, SGI0, SGI0, SGI286032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67546[] = {SGI286033, SGI286034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67547[] = {SGI286035, SGI286036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67548[] = {SGI286037, SGI286038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67549[] = {SGI286040, SGI286041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67550[] = {SGI286043, SGI286044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67551[] = {SGI286046, SGI286047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286048, NULL};
const Segment * const SG67552[] = {SGI286049, SGI286050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286051, SGI0, NULL};
const Segment * const SG67553[] = {SGI286052, SGI286053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67554[] = {SGI286055, SGI286056, NULL};
const Segment * const SG67555[] = {SGI286057, SGI286058, NULL};
const Segment * const SG67556[] = {SGI286059, SGI286060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67557[] = {SGI0, SGI286061, SGI0, SGI286062, SGI0, SGI286063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286064, SGI286065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67558[] = {SGI0, SGI286066, SGI0, SGI286067, SGI286068, SGI286069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286070, SGI286071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67559[] = {SGI286072, SGI286073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286074, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67560[] = {SGI286079, SGI286080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286082, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67561[] = {SGI0, SGI286075, SGI0, SGI286076, SGI286077, SGI286078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286081, SGI286083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67562[] = {SGI0, SGI286084, SGI0, SGI286085, SGI286086, SGI286087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286088, SGI286089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67563[] = {SGI0, SGI286090, SGI0, SGI0, SGI0, SGI286091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286092, SGI286093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67564[] = {SGI0, SGI286094, SGI0, SGI286095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286096, SGI286097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67565[] = {SGI0, SGI286098, SGI0, SGI286099, SGI286100, SGI286101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286102, SGI286103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67566[] = {SGI0, SGI286104, SGI0, SGI286105, SGI286106, SGI286107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286108, SGI286109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67567[] = {SGI0, SGI286110, SGI0, SGI286111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286112, SGI286113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67568[] = {SGI0, SGI286114, SGI0, SGI286115, SGI286116, SGI286116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286117, SGI286118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67569[] = {SGI0, SGI286119, SGI0, SGI286120, SGI286121, SGI286122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286123, SGI286124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67570[] = {SGI286125, SGI286126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67571[] = {SGI286127, SGI286128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67572[] = {SGI286130, SGI286131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67573[] = {SGI286133, SGI286134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286135, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67574[] = {SGI286136, SGI286137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286138, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67575[] = {SGI286139, SGI286140, SGI0, SGI286141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67576[] = {SGI286142, SGI286143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67577[] = {SGI286145, SGI286146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67578[] = {SGI286148, SGI286149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286150, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67579[] = {SGI286151, SGI286152, SGI286153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67580[] = {SGI286154, SGI286155, SGI286156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67581[] = {SGI286157, SGI286158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67582[] = {SGI286160, SGI286161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67583[] = {SGI286163, SGI286164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286165, SGI0, SGI0, NULL};
const Segment * const SG67584[] = {SGI286166, SGI286167, NULL};
const Segment * const SG67585[] = {SGI286168, SGI286169, NULL};
const Segment * const SG67586[] = {SGI286170, SGI286171, NULL};
const Segment * const SG67587[] = {SGI286172, SGI286173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67588[] = {SGI286174, SGI286175, SGI286176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67589[] = {SGI286177, SGI286178, SGI286179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67590[] = {SGI286180, SGI286181, SGI286182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67591[] = {SGI286183, SGI286184, SGI286185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67592[] = {SGI286186, SGI286187, SGI286188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67593[] = {SGI286189, SGI286190, SGI286191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67594[] = {SGI286192, SGI286193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286194, NULL};
const Segment * const SG67595[] = {SGI286195, SGI286196, NULL};
const Segment * const SG67596[] = {SGI286197, SGI286198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286199, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67597[] = {SGI286200, SGI286201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286202, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67598[] = {SGI286203, SGI286204, SGI0, SGI286205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67599[] = {SGI286206, SGI286207, SGI0, SGI286208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67600[] = {SGI286209, SGI286210, NULL};
const Segment * const SG67601[] = {SGI286211, SGI286212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67602[] = {SGI286213, SGI286214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67603[] = {SGI286215, SGI286216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67604[] = {SGI286217, SGI286218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67605[] = {SGI286219, SGI286220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67606[] = {SGI286221, SGI286222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67607[] = {SGI286223, SGI286224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67608[] = {SGI286225, SGI286226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67609[] = {SGI286227, SGI286228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67610[] = {SGI286229, SGI286230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67611[] = {SGI286231, SGI286232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67612[] = {SGI286233, SGI286234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67613[] = {SGI286235, SGI286236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67614[] = {SGI286237, SGI286238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67615[] = {SGI286239, SGI286240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67616[] = {SGI286241, SGI286242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67617[] = {SGI286243, SGI286244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67618[] = {SGI286245, SGI286246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67619[] = {SGI286247, SGI286248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67620[] = {SGI286249, SGI286250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67621[] = {SGI286251, SGI286252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67622[] = {SGI286253, SGI286254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67623[] = {SGI286255, SGI286256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67624[] = {SGI286257, SGI286258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67625[] = {SGI286259, SGI286260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67626[] = {SGI286261, SGI286262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67627[] = {SGI286263, SGI286264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67628[] = {SGI286265, SGI286266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67629[] = {SGI286267, SGI286268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67630[] = {SGI286269, SGI286270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67631[] = {SGI286271, SGI286272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67632[] = {SGI286273, SGI286274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67633[] = {SGI286275, SGI286276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67634[] = {SGI286277, SGI286278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67635[] = {SGI286279, SGI286280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67636[] = {SGI286281, SGI286282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67637[] = {SGI286283, SGI286284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67638[] = {SGI286285, SGI286286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67639[] = {SGI286287, SGI286288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67640[] = {SGI286289, SGI286290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67641[] = {SGI286291, SGI286292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67642[] = {SGI286294, SGI286295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67643[] = {SGI286297, SGI286298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67644[] = {SGI286300, SGI286301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67645[] = {SGI286303, SGI286304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67646[] = {SGI286306, SGI286307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67647[] = {SGI286309, SGI286310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67648[] = {SGI286311, SGI286312, NULL};
const Segment * const SG67649[] = {SGI286313, SGI286314, NULL};
const Segment * const SG67650[] = {SGI286315, SGI286316, NULL};
const Segment * const SG67651[] = {SGI286317, SGI286318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67652[] = {SGI286320, SGI286321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67653[] = {SGI286322, SGI286323, NULL};
const Segment * const SG67654[] = {SGI286324, SGI286325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67655[] = {SGI0, SGI0, SGI0, SGI286326, SGI286327, SGI286328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286329, SGI286330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67656[] = {SGI0, SGI286331, SGI0, SGI286332, SGI286333, SGI286334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286335, SGI286336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67657[] = {SGI0, SGI286337, SGI0, SGI286338, SGI286339, SGI286340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286341, SGI286342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67658[] = {SGI0, SGI286343, SGI0, SGI286344, SGI286345, SGI286346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286347, SGI286348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67659[] = {SGI0, SGI286349, SGI0, SGI286350, SGI286351, SGI286352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286353, SGI286354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67660[] = {SGI0, SGI286355, SGI0, SGI286356, SGI286357, SGI286357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286358, SGI286359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67661[] = {SGI286360, SGI286361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286362, NULL};
const Segment * const SG67662[] = {SGI286363, SGI286364, NULL};
const Segment * const SG67663[] = {SGI286365, SGI286366, NULL};
const Segment * const SG67664[] = {SGI286367, SGI286368, NULL};
const Segment * const SG67665[] = {SGI286369, SGI286370, NULL};
const Segment * const SG67666[] = {SGI286371, SGI286372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67667[] = {SGI286374, SGI286375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67668[] = {SGI286377, SGI286378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67669[] = {SGI286380, SGI286381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67670[] = {SGI286382, SGI286383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286384, SGI0, SGI0, NULL};
const Segment * const SG67671[] = {SGI286385, SGI286386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67672[] = {SGI286387, SGI286388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67673[] = {SGI286389, SGI286390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67674[] = {SGI286391, SGI286392, SGI0, SGI286393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67675[] = {SGI286394, SGI286395, SGI0, SGI286396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67676[] = {SGI286397, SGI286398, SGI0, SGI286399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67677[] = {SGI286400, SGI286401, SGI0, SGI286402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67678[] = {SGI286403, SGI286404, SGI0, SGI286405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67679[] = {SGI286406, SGI286407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286408, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67680[] = {SGI286409, SGI286410, NULL};
const Segment * const SG67681[] = {SGI286411, SGI286412, NULL};
const Segment * const SG67682[] = {SGI286413, SGI286414, NULL};
const Segment * const SG67683[] = {SGI286415, SGI286416, NULL};
const Segment * const SG67684[] = {SGI286417, SGI286418, NULL};
const Segment * const SG67685[] = {SGI286419, SGI286420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286421, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67686[] = {SGI286422, SGI286423, NULL};
const Segment * const SG67687[] = {SGI286424, SGI286425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67688[] = {SGI286426, SGI286427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67689[] = {SGI286428, SGI286429, SGI286430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67690[] = {SGI286431, SGI286432, NULL};
const Segment * const SG67691[] = {SGI286433, SGI286434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67692[] = {SGI286435, SGI286436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67693[] = {SGI286438, SGI286439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67694[] = {SGI286441, SGI286442, NULL};
const Segment * const SG67695[] = {SGI286443, SGI286444, NULL};
const Segment * const SG67696[] = {SGI286445, SGI286446, NULL};
const Segment * const SG67697[] = {SGI0, SGI286447, SGI0, SGI286448, SGI286449, SGI286450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286451, SGI286452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67698[] = {SGI0, SGI286453, SGI0, SGI286454, SGI286455, SGI286456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286457, SGI286458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67699[] = {SGI0, SGI286459, SGI0, SGI286460, SGI286461, SGI286462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286463, SGI286464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67700[] = {SGI0, SGI286465, SGI0, SGI286466, SGI286467, SGI286468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286469, SGI286470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67701[] = {SGI0, SGI286471, SGI0, SGI286472, SGI286473, SGI286474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286475, SGI286476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67702[] = {SGI0, SGI286477, SGI0, SGI286478, SGI0, SGI286479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286480, SGI286481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67703[] = {SGI286482, SGI286483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67704[] = {SGI286485, SGI286486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67705[] = {SGI286488, SGI286489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67706[] = {SGI286491, SGI286492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67707[] = {SGI286494, SGI286495, SGI0, SGI286496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67708[] = {SGI286497, SGI286498, NULL};
const Segment * const SG67709[] = {SGI286499, SGI286500, SGI0, SGI0, SGI286501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67710[] = {SGI286502, SGI286503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67711[] = {SGI286504, SGI286505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67712[] = {SGI286506, SGI286507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67713[] = {SGI286508, SGI286509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67714[] = {SGI286511, SGI286512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67715[] = {SGI286514, SGI286515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67716[] = {SGI286516, SGI286517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67717[] = {SGI286518, SGI286519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286520, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67718[] = {SGI286521, SGI286522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67719[] = {SGI286523, SGI286524, NULL};
const Segment * const SG67720[] = {SGI286525, SGI286526, NULL};
const Segment * const SG67721[] = {SGI286527, SGI286528, NULL};
const Segment * const SG67722[] = {SGI286529, SGI286530, NULL};
const Segment * const SG67723[] = {SGI286531, SGI286532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67724[] = {SGI286535, SGI286536, SGI0, SGI286537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67725[] = {SGI286533, SGI286534, SGI0, SGI0, SGI0, SGI286538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67726[] = {SGI286539, SGI286540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286541, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67727[] = {SGI286542, SGI286543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286544, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67728[] = {SGI286545, SGI286546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286547, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67729[] = {SGI286548, SGI286549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286550, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67730[] = {SGI286551, SGI286552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67731[] = {SGI286553, SGI286554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67732[] = {SGI286555, SGI286556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67733[] = {SGI286558, SGI286559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67734[] = {SGI286560, SGI286561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286562, SGI0, NULL};
const Segment * const SG67735[] = {SGI286563, SGI286564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67736[] = {SGI286566, SGI286567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67737[] = {SGI286569, SGI286570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67738[] = {SGI286572, SGI286573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67739[] = {SGI286575, SGI286576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67740[] = {SGI286577, SGI286578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67741[] = {SGI286579, SGI286580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67742[] = {SGI286582, SGI286583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67743[] = {SGI286584, SGI286585, SGI0, SGI286586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67744[] = {SGI286587, SGI286588, NULL};
const Segment * const SG67745[] = {SGI286589, SGI286590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67746[] = {SGI286591, SGI286592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67747[] = {SGI286594, SGI286596, SGI0, SGI0, SGI0, SGI0, SGI286598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67748[] = {SGI286595, SGI286597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67749[] = {SGI286600, SGI286601, SGI0, SGI0, SGI0, SGI0, SGI286602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67750[] = {SGI286603, SGI286604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67751[] = {SGI286605, SGI286606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67752[] = {SGI286607, SGI286608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67753[] = {SGI286609, SGI286610, SGI286611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67754[] = {SGI286612, SGI286613, NULL};
const Segment * const SG67755[] = {SGI286614, SGI286615, NULL};
const Segment * const SG67756[] = {SGI286616, SGI286617, NULL};
const Segment * const SG67757[] = {SGI286618, SGI286619, NULL};
const Segment * const SG67758[] = {SGI286620, SGI286621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67759[] = {SGI286623, SGI286624, NULL};
const Segment * const SG67760[] = {SGI286625, SGI286626, NULL};
const Segment * const SG67761[] = {SGI286627, SGI286628, NULL};
const Segment * const SG67762[] = {SGI286629, SGI286630, NULL};
const Segment * const SG67763[] = {SGI286631, SGI286632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67764[] = {SGI286633, SGI286634, NULL};
const Segment * const SG67765[] = {SGI286635, SGI286636, SGI0, SGI0, SGI0, SGI0, SGI286637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67766[] = {SGI286638, SGI286639, SGI0, SGI0, SGI0, SGI0, SGI286640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67767[] = {SGI286641, SGI286642, SGI0, SGI0, SGI0, SGI0, SGI286643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67768[] = {SGI286644, SGI286645, SGI0, SGI0, SGI0, SGI0, SGI286646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67769[] = {SGI286647, SGI286648, SGI0, SGI286649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67770[] = {SGI286650, SGI286651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67771[] = {SGI286652, SGI286653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286654, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67772[] = {SGI286655, SGI286656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67773[] = {SGI286658, SGI286659, NULL};
const Segment * const SG67774[] = {SGI286660, SGI286661, SGI0, SGI286662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67775[] = {SGI286663, SGI286664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286665, NULL};
const Segment * const SG67776[] = {SGI286666, SGI286667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67777[] = {SGI286669, SGI286670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67778[] = {SGI286672, SGI286673, NULL};
const Segment * const SG67779[] = {SGI286674, SGI286675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286676, NULL};
const Segment * const SG67780[] = {SGI286677, SGI286678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286679, NULL};
const Segment * const SG67781[] = {SGI286680, SGI286681, SGI0, SGI286682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67782[] = {SGI286683, SGI286684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286685, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67783[] = {SGI286686, SGI286687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286688, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67784[] = {SGI286689, SGI286690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286691, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67785[] = {SGI286692, SGI286693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67786[] = {SGI286694, SGI286695, NULL};
const Segment * const SG67787[] = {SGI286696, SGI286697, NULL};
const Segment * const SG67788[] = {SGI286698, SGI286699, NULL};
const Segment * const SG67789[] = {SGI286700, SGI286701, NULL};
const Segment * const SG67790[] = {SGI286702, SGI286703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67791[] = {SGI286704, SGI286705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67792[] = {SGI0, SGI286706, SGI0, SGI286707, SGI286708, SGI286708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286709, SGI286710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67793[] = {SGI286712, SGI286713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67794[] = {SGI286714, SGI286715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67795[] = {SGI286717, SGI286718, NULL};
const Segment * const SG67796[] = {SGI286719, SGI286720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67797[] = {SGI286721, SGI286722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286723, SGI0, SGI0, NULL};
const Segment * const SG67798[] = {SGI286724, SGI286725, SGI0, SGI286726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67799[] = {SGI286727, SGI286728, SGI0, SGI286729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67800[] = {SGI0, SGI286730, SGI0, SGI286731, SGI286732, SGI286732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286733, SGI286734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67801[] = {SGI0, SGI286736, SGI0, SGI286737, SGI286738, SGI286738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286739, SGI286740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67802[] = {SGI286742, SGI286743, NULL};
const Segment * const SG67803[] = {SGI286744, SGI286745, NULL};
const Segment * const SG67804[] = {SGI286746, SGI286747, NULL};
const Segment * const SG67805[] = {SGI286748, SGI286749, NULL};
const Segment * const SG67806[] = {SGI0, SGI286750, SGI0, SGI286751, SGI286752, SGI286753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286754, SGI286755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67807[] = {SGI0, SGI286756, SGI0, SGI286757, SGI0, SGI286758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286759, SGI286760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67808[] = {SGI0, SGI286761, SGI0, SGI286762, SGI286763, SGI286764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286765, SGI286766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67809[] = {SGI0, SGI286767, SGI0, SGI286768, SGI286769, SGI286770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286771, SGI286772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67810[] = {SGI0, SGI286773, SGI0, SGI286774, SGI286775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286776, SGI286777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67811[] = {SGI0, SGI286778, SGI0, SGI286779, SGI286780, SGI286781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286782, SGI286783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67812[] = {SGI0, SGI286784, SGI0, SGI286785, SGI286786, SGI286787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286788, SGI286789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67813[] = {SGI0, SGI0, SGI0, SGI286790, SGI286791, SGI286792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286794, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67814[] = {SGI0, SGI286795, SGI0, SGI286796, SGI286797, SGI286798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286799, SGI286800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67815[] = {SGI0, SGI0, SGI0, SGI286801, SGI286802, SGI286803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286804, SGI286805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67816[] = {SGI0, SGI286806, SGI0, SGI286807, SGI286808, SGI286809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286810, SGI286811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67817[] = {SGI0, SGI286812, SGI0, SGI286813, SGI286814, SGI286815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286816, SGI286817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67818[] = {SGI0, SGI0, SGI0, SGI286818, SGI286819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67819[] = {SGI0, SGI286821, SGI0, SGI286822, SGI286823, SGI286824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286825, SGI286826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67820[] = {SGI0, SGI286827, SGI0, SGI286828, SGI286829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286830, SGI286831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67821[] = {SGI0, SGI286832, SGI0, SGI286833, SGI286834, SGI286835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286836, SGI286837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67822[] = {SGI0, SGI286838, SGI0, SGI286839, SGI286840, SGI286841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286842, SGI286843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67823[] = {SGI286844, SGI286845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67824[] = {SGI286846, SGI286847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286848, SGI0, SGI0, NULL};
const Segment * const SG67825[] = {SGI286849, SGI286850, NULL};
const Segment * const SG67826[] = {SGI286851, SGI286852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286853, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67827[] = {SGI286854, SGI286855, NULL};
const Segment * const SG67828[] = {SGI286856, SGI286857, NULL};
const Segment * const SG67829[] = {SGI286858, SGI286859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67830[] = {SGI0, SGI286861, SGI0, SGI286862, SGI286863, SGI286864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286865, SGI286866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67831[] = {SGI0, SGI286867, SGI0, SGI286868, SGI286869, SGI286869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286870, SGI286871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67832[] = {SGI0, SGI286873, SGI0, SGI286874, SGI286875, SGI286875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286876, SGI286877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67833[] = {SGI0, SGI286879, SGI0, SGI286880, SGI286881, SGI286882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286883, SGI286884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67834[] = {SGI0, SGI286885, SGI0, SGI286886, SGI286887, SGI286888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286889, SGI286890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67835[] = {SGI286891, SGI286892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67836[] = {SGI0, SGI286893, SGI0, SGI286894, SGI286895, SGI286895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286896, SGI286897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67837[] = {SGI0, SGI286899, SGI0, SGI286900, SGI286901, SGI286902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286903, SGI286904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67838[] = {SGI0, SGI286905, SGI0, SGI286906, SGI286907, SGI286907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286908, SGI286909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67839[] = {SGI0, SGI286911, SGI0, SGI286912, SGI286913, SGI286913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286914, SGI286915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67840[] = {SGI0, SGI286917, SGI0, SGI286918, SGI286919, SGI286919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286920, SGI286921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67841[] = {SGI0, SGI286923, SGI0, SGI286924, SGI286925, SGI286925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286926, SGI286927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67842[] = {SGI0, SGI286929, SGI0, SGI286930, SGI286931, SGI286931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286932, SGI286933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67843[] = {SGI0, SGI286934, SGI0, SGI286935, SGI286936, SGI286936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286937, SGI286938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67844[] = {SGI0, SGI286940, SGI0, SGI286941, SGI286942, SGI286942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286943, SGI286944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67845[] = {SGI0, SGI286946, SGI0, SGI286947, SGI286948, SGI286949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286950, SGI286951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67846[] = {SGI0, SGI286952, SGI0, SGI286953, SGI286954, SGI286954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286955, SGI286956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67847[] = {SGI0, SGI286958, SGI0, SGI286959, SGI286960, SGI286961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286962, SGI286963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67848[] = {SGI286964, SGI286965, NULL};
const Segment * const SG67849[] = {SGI286966, SGI286967, NULL};
const Segment * const SG67850[] = {SGI286968, SGI286969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67851[] = {SGI286970, SGI286971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67852[] = {SGI286972, SGI286973, SGI0, SGI0, SGI0, SGI0, SGI286974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67853[] = {SGI286975, SGI286976, SGI0, SGI0, SGI0, SGI0, SGI286977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67854[] = {SGI286978, SGI286979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67855[] = {SGI286981, SGI286982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67856[] = {SGI286984, SGI286985, NULL};
const Segment * const SG67857[] = {SGI286986, SGI286987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67858[] = {SGI286989, SGI286990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67859[] = {SGI286992, SGI286993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67860[] = {SGI286995, SGI286996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI286997, SGI0, SGI0, NULL};
const Segment * const SG67861[] = {SGI286998, SGI286999, NULL};
const Segment * const SG67862[] = {SGI287000, SGI287001, SGI0, SGI0, SGI287002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67863[] = {SGI287003, SGI287004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67864[] = {SGI287005, SGI287006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67865[] = {SGI287007, SGI287008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287009, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67866[] = {SGI0, SGI287010, SGI0, SGI287011, SGI287012, SGI287012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287013, SGI287014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67867[] = {SGI0, SGI287016, SGI0, SGI287017, SGI287018, SGI287018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287019, SGI287020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67868[] = {SGI287022, SGI287023, NULL};
const Segment * const SG67869[] = {SGI287024, SGI287025, NULL};
const Segment * const SG67870[] = {SGI287026, SGI287027, SGI0, SGI287028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67871[] = {SGI287029, SGI287030, SGI0, SGI287031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67872[] = {SGI0, SGI287032, SGI0, SGI287033, SGI287034, SGI287034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287035, SGI287036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67873[] = {SGI0, SGI287038, SGI0, SGI287039, SGI287040, SGI287040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287041, SGI287042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67874[] = {SGI287044, SGI287045, NULL};
const Segment * const SG67875[] = {SGI287046, SGI287047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287048, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67876[] = {SGI287049, SGI287050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287051, NULL};
const Segment * const SG67877[] = {SGI287052, SGI287053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67878[] = {SGI287055, SGI287056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67879[] = {SGI287058, SGI287059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67880[] = {SGI287060, SGI287061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67881[] = {SGI287062, SGI287063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67882[] = {SGI287064, SGI287065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287066, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67883[] = {SGI287067, SGI287068, NULL};
const Segment * const SG67884[] = {SGI0, SGI287069, SGI0, SGI287070, SGI287071, SGI287071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287072, SGI287073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67885[] = {SGI0, SGI287075, SGI0, SGI287076, SGI287077, SGI287077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287078, SGI287079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67886[] = {SGI287081, SGI287082, NULL};
const Segment * const SG67887[] = {SGI287083, SGI287084, NULL};
const Segment * const SG67888[] = {SGI287085, SGI287086, NULL};
const Segment * const SG67889[] = {SGI287087, SGI287088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287089, SGI0, SGI0, NULL};
const Segment * const SG67890[] = {SGI287090, SGI287091, SGI0, SGI287092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67891[] = {SGI287093, SGI287094, SGI0, SGI287095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67892[] = {SGI0, SGI287096, SGI0, SGI287097, SGI287098, SGI287098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287099, SGI287100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67893[] = {SGI0, SGI287102, SGI0, SGI287103, SGI287104, SGI287104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287105, SGI287106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67894[] = {SGI287108, SGI287109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67895[] = {SGI0, SGI287111, SGI0, SGI287112, SGI287113, SGI287113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287114, SGI287115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67896[] = {SGI287117, SGI287118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287119, SGI0, SGI0, NULL};
const Segment * const SG67897[] = {SGI287120, SGI287121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67898[] = {SGI287123, SGI287124, SGI0, SGI0, SGI287125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67899[] = {SGI287126, SGI287127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67900[] = {SGI287128, SGI287129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67901[] = {SGI287130, SGI287131, NULL};
const Segment * const SG67902[] = {SGI287132, SGI287133, NULL};
const Segment * const SG67903[] = {SGI287134, SGI287135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67904[] = {SGI287137, SGI287138, NULL};
const Segment * const SG67905[] = {SGI287139, SGI287140, NULL};
const Segment * const SG67906[] = {SGI287141, SGI287142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287143, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67907[] = {SGI287144, SGI287145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67908[] = {SGI287147, SGI287148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67909[] = {SGI287150, SGI287151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67910[] = {SGI287153, SGI287154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67911[] = {SGI287156, SGI287157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67912[] = {SGI287158, SGI287159, SGI0, SGI287160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67913[] = {SGI287161, SGI287162, SGI0, SGI287163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67914[] = {SGI287164, SGI287165, SGI0, SGI287166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67915[] = {SGI287167, SGI287168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287169, SGI0, SGI0, NULL};
const Segment * const SG67916[] = {SGI287170, SGI287171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67917[] = {SGI0, SGI287173, SGI0, SGI287174, SGI287175, SGI287175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287176, SGI287177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67918[] = {SGI0, SGI287179, SGI0, SGI287180, SGI287181, SGI287181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287182, SGI287183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67919[] = {SGI287185, SGI287186, SGI0, SGI287187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67920[] = {SGI287188, SGI287189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67921[] = {SGI0, SGI287190, SGI0, SGI287191, SGI287192, SGI287192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287193, SGI287194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67922[] = {SGI287196, SGI287197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67923[] = {SGI287199, SGI287200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287201, SGI0, SGI0, NULL};
const Segment * const SG67924[] = {SGI287202, SGI287203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287204, SGI0, SGI0, NULL};
const Segment * const SG67925[] = {SGI287205, SGI287206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67926[] = {SGI287208, SGI287209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67927[] = {SGI287211, SGI287212, SGI0, SGI0, SGI287213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67928[] = {SGI287214, SGI287215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67929[] = {SGI287217, SGI287218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67930[] = {SGI287220, SGI287221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287222, SGI0, SGI0, NULL};
const Segment * const SG67931[] = {SGI287223, SGI287224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287225, SGI0, SGI0, NULL};
const Segment * const SG67932[] = {SGI287226, SGI287227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67933[] = {SGI287228, SGI287229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67934[] = {SGI287230, SGI287231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287232, SGI0, SGI0, NULL};
const Segment * const SG67935[] = {SGI287233, SGI287234, NULL};
const Segment * const SG67936[] = {SGI287235, SGI287236, NULL};
const Segment * const SG67937[] = {SGI287237, SGI287238, NULL};
const Segment * const SG67938[] = {SGI287239, SGI287240, NULL};
const Segment * const SG67939[] = {SGI0, SGI287241, SGI0, SGI287242, SGI287243, SGI287243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287244, SGI287245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67940[] = {SGI0, SGI287247, SGI0, SGI287248, SGI287249, SGI287249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287250, SGI287251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67941[] = {SGI287253, SGI287254, SGI0, SGI287255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67942[] = {SGI287256, SGI287257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67943[] = {SGI287258, SGI287259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287260, SGI0, SGI0, NULL};
const Segment * const SG67944[] = {SGI287261, SGI287262, NULL};
const Segment * const SG67945[] = {SGI287263, SGI287264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67946[] = {SGI287266, SGI287267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287268, SGI0, SGI0, NULL};
const Segment * const SG67947[] = {SGI287269, SGI287270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287271, SGI0, SGI0, NULL};
const Segment * const SG67948[] = {SGI287272, SGI287273, SGI0, SGI287274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67949[] = {SGI287275, SGI287276, SGI0, SGI287277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67950[] = {SGI287278, SGI287279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67951[] = {SGI287281, SGI287282, SGI0, SGI0, SGI287283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67952[] = {SGI287284, SGI287285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67953[] = {SGI287287, SGI287288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67954[] = {SGI287289, SGI287290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287291, SGI0, SGI0, NULL};
const Segment * const SG67955[] = {SGI287292, SGI287293, SGI0, SGI287294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67956[] = {SGI287295, SGI287296, SGI0, SGI287297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67957[] = {SGI287298, SGI287299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287300, SGI0, SGI0, NULL};
const Segment * const SG67958[] = {SGI287301, SGI287302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67959[] = {SGI287303, SGI287304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287305, SGI0, SGI0, NULL};
const Segment * const SG67960[] = {SGI287306, SGI287307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67961[] = {SGI287309, SGI287310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67962[] = {SGI287311, SGI287312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67963[] = {SGI287313, SGI287314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67964[] = {SGI287315, SGI287316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67965[] = {SGI287317, SGI287318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67966[] = {SGI287319, SGI287320, SGI0, SGI287321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67967[] = {SGI287322, SGI287323, SGI0, SGI287324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67968[] = {SGI287325, SGI287326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287327, SGI0, SGI0, NULL};
const Segment * const SG67969[] = {SGI287328, SGI287329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287330, SGI0, SGI0, NULL};
const Segment * const SG67970[] = {SGI287331, SGI287332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67971[] = {SGI287333, SGI287334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67972[] = {SGI287335, SGI287336, NULL};
const Segment * const SG67973[] = {SGI287337, SGI287338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67974[] = {SGI287339, SGI287340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67975[] = {SGI287341, SGI287342, SGI0, SGI287343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67976[] = {SGI287344, SGI287345, SGI0, SGI287346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67977[] = {SGI287347, SGI287348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67978[] = {SGI0, SGI287349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67979[] = {SGI0, SGI287350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67980[] = {SGI0, SGI287351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67981[] = {SGI0, SGI287352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67982[] = {SGI0, SGI287353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67983[] = {SGI0, SGI287354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67984[] = {SGI0, SGI287355, SGI0, SGI287356, SGI287357, SGI287357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287358, SGI287359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67985[] = {SGI0, SGI287360, SGI0, SGI287361, SGI287362, SGI287362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287363, SGI287364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67986[] = {SGI0, SGI287365, SGI0, SGI287366, SGI287367, SGI287368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287369, SGI287370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67987[] = {SGI0, SGI287371, SGI0, SGI287372, SGI287373, SGI287374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287375, SGI287376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67988[] = {SGI0, SGI287377, SGI0, SGI287378, SGI287379, SGI287379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287380, SGI287381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67989[] = {SGI287383, SGI287384, NULL};
const Segment * const SG67990[] = {SGI0, SGI287385, SGI0, SGI287386, SGI287387, SGI287387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287388, SGI287389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67991[] = {SGI0, SGI287391, SGI0, SGI287392, SGI287393, SGI287394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67992[] = {SGI0, SGI287396, SGI0, SGI287397, SGI287398, SGI287398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287399, SGI287400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67993[] = {SGI0, SGI287402, SGI0, SGI287403, SGI287404, SGI287404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287405, SGI287406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67994[] = {SGI0, SGI287408, SGI0, SGI287409, SGI287410, SGI287410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67995[] = {SGI0, SGI287412, SGI287412, SGI287413, SGI287414, SGI287414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67996[] = {SGI0, SGI0, SGI287416, SGI287417, SGI287418, SGI287418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67997[] = {SGI287419, SGI287420, NULL};
const Segment * const SG67998[] = {SGI0, SGI287421, SGI0, SGI287422, SGI287423, SGI287423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287424, SGI287425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG67999[] = {SGI0, SGI287427, SGI0, SGI287428, SGI287429, SGI287429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287430, SGI287431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68000[] = {SGI0, SGI287433, SGI0, SGI287434, SGI287435, SGI287436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287437, SGI287438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68001[] = {SGI0, SGI287439, SGI0, SGI287440, SGI287441, SGI287441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287442, SGI287443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68002[] = {SGI0, SGI287445, SGI0, SGI287446, SGI287447, SGI287447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287448, SGI287449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68003[] = {SGI0, SGI287451, SGI0, SGI287452, SGI287453, SGI287453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287454, SGI287455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68004[] = {SGI0, SGI287457, SGI0, SGI287458, SGI287459, SGI287459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287460, SGI287461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68005[] = {SGI0, SGI287463, SGI0, SGI287464, SGI287465, SGI287465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287466, SGI287467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68006[] = {SGI0, SGI287469, SGI0, SGI287470, SGI287471, SGI287471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287472, SGI287473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68007[] = {SGI0, SGI287475, SGI0, SGI287476, SGI287477, SGI287477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287478, SGI287479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68008[] = {SGI0, SGI287480, SGI0, SGI287481, SGI287482, SGI287483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287484, SGI287485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68009[] = {SGI287486, SGI287487, NULL};
const Segment * const SG68010[] = {SGI0, SGI287488, SGI0, SGI287489, SGI287490, SGI287491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287492, SGI287493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68011[] = {SGI0, SGI287494, SGI0, SGI287495, SGI287496, SGI287497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287498, SGI287499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68012[] = {SGI0, SGI287500, SGI0, SGI287501, SGI287502, SGI287503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287504, SGI287505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68013[] = {SGI0, SGI287506, SGI0, SGI287507, SGI287508, SGI287509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287510, SGI287511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68014[] = {SGI0, SGI287512, SGI0, SGI287513, SGI287514, SGI287515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287516, SGI287517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68015[] = {SGI0, SGI287518, SGI0, SGI287519, SGI287520, SGI287520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287521, SGI287522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68016[] = {SGI0, SGI287523, SGI0, SGI287524, SGI287525, SGI287526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287527, SGI287528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68017[] = {SGI0, SGI287529, SGI0, SGI287530, SGI287531, SGI287531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287532, SGI287533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68018[] = {SGI287535, SGI287536, NULL};
const Segment * const SG68019[] = {SGI287537, SGI287538, NULL};
const Segment * const SG68020[] = {SGI287539, SGI287540, SGI0, SGI287541, SGI287542, SGI287542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68021[] = {SGI0, SGI287544, SGI287544, SGI287545, SGI287546, SGI287546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68022[] = {SGI0, SGI287548, SGI0, SGI287549, SGI287550, SGI287550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68023[] = {SGI0, SGI287552, SGI0, SGI287553, SGI287554, SGI287554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68024[] = {SGI0, SGI287556, SGI0, SGI287557, SGI287558, SGI287558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68025[] = {SGI0, SGI287560, SGI0, SGI287561, SGI287562, SGI287562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68026[] = {SGI0, SGI287564, SGI0, SGI287565, SGI287566, SGI287566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287567, SGI287568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68027[] = {SGI0, SGI0, SGI0, SGI287570, SGI287571, SGI287571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68028[] = {SGI0, SGI287572, SGI0, SGI287573, SGI287574, SGI287574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68029[] = {SGI0, SGI0, SGI287576, SGI287577, SGI287578, SGI287578, SGI0, SGI0, SGI287579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68030[] = {SGI0, SGI0, SGI287580, SGI287581, SGI287582, SGI287582, SGI0, SGI0, SGI287583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68031[] = {SGI0, SGI287584, SGI0, SGI287585, SGI287586, SGI287586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68032[] = {SGI0, SGI0, SGI0, SGI287588, SGI287589, SGI287589, SGI0, SGI0, SGI287590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287591, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68033[] = {SGI0, SGI287592, SGI0, SGI287593, SGI287594, SGI287594, SGI0, SGI0, SGI287595, SGI0, SGI0, SGI0, SGI287596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68034[] = {SGI0, SGI0, SGI287597, SGI287598, SGI287599, SGI287599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68035[] = {SGI0, SGI287600, SGI0, SGI287601, SGI287602, SGI287602, SGI0, SGI0, SGI287603, SGI0, SGI0, SGI0, SGI287604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68036[] = {SGI0, SGI287605, SGI0, SGI287606, SGI287607, SGI287607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287608, SGI287609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68037[] = {SGI0, SGI0, SGI287610, SGI287611, SGI287612, SGI287612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68038[] = {SGI287613, SGI287614, NULL};
const Segment * const SG68039[] = {SGI0, SGI287615, SGI0, SGI287616, SGI287617, SGI287617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287618, SGI287619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68040[] = {SGI0, SGI287621, NULL};
const Segment * const SG68041[] = {SGI0, SGI287622, SGI0, SGI287623, SGI287624, SGI287624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287625, SGI287626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68042[] = {SGI0, SGI287628, SGI0, SGI287629, SGI287630, SGI287630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287631, SGI287632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68043[] = {SGI0, SGI287634, SGI0, SGI287635, SGI287636, SGI287636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287637, SGI287638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68044[] = {SGI0, SGI287640, SGI0, SGI287641, SGI287642, SGI287642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287643, SGI287644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68045[] = {SGI0, SGI287646, SGI0, SGI287647, SGI287648, SGI287649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68046[] = {SGI0, SGI287651, SGI0, SGI287652, SGI287653, SGI287654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68047[] = {SGI0, SGI287656, SGI0, SGI287657, SGI287658, SGI287659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68048[] = {SGI0, SGI287661, SGI0, SGI287662, SGI287663, SGI287664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68049[] = {SGI0, SGI0, SGI0, SGI287666, SGI287667, SGI287668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68050[] = {SGI0, SGI287670, SGI0, SGI287671, SGI287672, SGI287672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287673, SGI287674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68051[] = {SGI0, SGI287675, SGI0, SGI287676, SGI287677, SGI287677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287678, SGI287679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68052[] = {SGI0, SGI287680, SGI0, SGI287681, SGI287682, SGI287682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287683, SGI287684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68053[] = {SGI0, SGI287685, SGI0, SGI287686, SGI287687, SGI287688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287689, SGI287690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68054[] = {SGI0, SGI287691, SGI0, SGI287692, SGI287693, SGI287693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287694, SGI287695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68055[] = {SGI0, SGI287696, SGI0, SGI287697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287698, SGI287699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68056[] = {SGI0, SGI287700, SGI0, SGI287701, SGI287702, SGI287703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287704, SGI287705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68057[] = {SGI0, SGI287706, SGI0, SGI0, SGI0, SGI287707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287708, SGI287709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68058[] = {SGI0, SGI287710, SGI0, SGI287711, SGI287712, SGI287713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287714, SGI287715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68059[] = {SGI0, SGI287716, SGI0, SGI287717, SGI287718, SGI287719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287720, SGI287721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68060[] = {SGI0, SGI287722, SGI0, SGI287723, SGI287724, SGI287725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287726, SGI287727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68061[] = {SGI0, SGI287728, SGI0, SGI287729, SGI287730, SGI287731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287732, SGI287733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68062[] = {SGI0, SGI287734, SGI0, SGI287735, SGI287736, SGI287737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287738, SGI287739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68063[] = {SGI0, SGI287740, SGI0, SGI287741, SGI287742, SGI287743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287744, SGI287745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68064[] = {SGI0, SGI287746, SGI0, SGI287747, SGI287748, SGI287749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287750, SGI287751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68065[] = {SGI0, SGI287752, SGI0, SGI287753, SGI287754, SGI287755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287756, SGI287757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68066[] = {SGI0, SGI287758, SGI0, SGI287759, SGI287760, SGI287761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287762, SGI287763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68067[] = {SGI0, SGI287764, SGI0, SGI287765, SGI287766, SGI287767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287768, SGI287769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68068[] = {SGI0, SGI287770, SGI0, SGI0, SGI0, SGI287771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287772, SGI287773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68069[] = {SGI0, SGI287774, SGI0, SGI287775, SGI287776, SGI287777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287778, SGI287779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68070[] = {SGI0, SGI287780, SGI0, SGI287781, SGI287782, SGI287782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287783, SGI287784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68071[] = {SGI0, SGI287785, SGI0, SGI287786, SGI287787, SGI287787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287788, SGI287789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68072[] = {SGI0, SGI287790, SGI0, SGI287791, SGI287792, SGI287792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287793, SGI287794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68073[] = {SGI0, SGI287795, SGI0, SGI287796, SGI287797, SGI287798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287799, SGI287800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68074[] = {SGI0, SGI287801, SGI0, SGI287802, SGI287803, SGI287804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287805, SGI287806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68075[] = {SGI0, SGI287807, SGI0, SGI287808, SGI287809, SGI287809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287810, SGI287811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68076[] = {SGI0, SGI287812, SGI0, SGI287813, SGI287814, SGI287814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287815, SGI287816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68077[] = {SGI0, SGI287817, SGI0, SGI287818, SGI287819, SGI287819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287820, SGI287821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68078[] = {SGI0, SGI287822, SGI0, SGI287823, SGI287824, SGI287824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287825, SGI287826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68079[] = {SGI0, SGI287827, SGI0, SGI287828, SGI287829, SGI287830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287831, SGI287832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68080[] = {SGI0, SGI287833, SGI0, SGI287834, SGI287835, SGI287836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287837, SGI287838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68081[] = {SGI0, SGI287839, SGI0, SGI287840, SGI287841, SGI287841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287842, SGI287843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68082[] = {SGI0, SGI287845, SGI0, SGI287846, SGI287847, SGI287847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287848, SGI287849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68083[] = {SGI287851, SGI287852, NULL};
const Segment * const SG68084[] = {SGI0, SGI287853, SGI0, SGI287854, SGI287855, SGI287855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287856, SGI287857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68085[] = {SGI0, SGI287858, SGI0, SGI287859, SGI287860, SGI287860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287861, SGI287862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68086[] = {SGI0, SGI287863, SGI0, SGI287864, SGI287865, SGI287865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287866, SGI287867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68087[] = {SGI0, SGI287868, SGI0, SGI287869, SGI0, SGI287870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287871, SGI287872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68088[] = {SGI0, SGI287873, SGI0, SGI287874, SGI287875, SGI287876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287877, SGI287878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68089[] = {SGI0, SGI287879, SGI0, SGI287880, SGI287881, SGI287882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287883, SGI287884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68090[] = {SGI0, SGI287885, SGI0, SGI287886, SGI287887, SGI287887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287888, SGI287889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68091[] = {SGI0, SGI287891, SGI0, SGI287892, SGI287893, SGI287893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287894, SGI287895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68092[] = {SGI0, SGI287897, SGI0, SGI287898, SGI287899, SGI287899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287900, SGI287901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68093[] = {SGI0, SGI287903, SGI0, SGI287904, SGI287905, SGI287905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287906, SGI287907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68094[] = {SGI0, SGI287909, SGI0, SGI287910, SGI287911, SGI287911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287912, SGI287913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68095[] = {SGI0, SGI287915, SGI0, SGI287916, SGI287917, SGI287917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287918, SGI287919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68096[] = {SGI0, SGI287920, SGI0, SGI287921, SGI287922, SGI287922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287923, SGI287924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68097[] = {SGI0, SGI287925, SGI0, SGI287926, SGI287927, SGI287928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287929, SGI287930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68098[] = {SGI0, SGI287931, SGI0, SGI287932, SGI287933, SGI287934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287935, SGI287936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68099[] = {SGI0, SGI287937, SGI0, SGI287938, SGI287939, SGI287939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287940, SGI287941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68100[] = {SGI0, SGI287942, SGI0, SGI287943, SGI287944, SGI287944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287945, SGI287946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68101[] = {SGI0, SGI287947, SGI0, SGI287948, SGI287949, SGI287950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287951, SGI287952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68102[] = {SGI0, SGI287953, SGI0, SGI287954, SGI287955, SGI287955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287956, SGI287957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68103[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI287958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287959, SGI287960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68104[] = {SGI0, SGI287961, SGI0, SGI287962, SGI287963, SGI287964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68105[] = {SGI0, SGI287966, SGI0, SGI287967, SGI287968, SGI287968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287969, SGI287970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68106[] = {SGI0, SGI287971, SGI0, SGI287972, SGI287973, SGI287973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287974, SGI287975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68107[] = {SGI0, SGI287976, SGI0, SGI287977, SGI287978, SGI287979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287980, SGI287981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68108[] = {SGI0, SGI287982, SGI0, SGI287983, SGI287984, SGI287985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287986, SGI287987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68109[] = {SGI0, SGI287988, NULL};
const Segment * const SG68110[] = {SGI0, SGI287989, SGI0, SGI287990, SGI287991, SGI287992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68111[] = {SGI0, SGI287994, SGI0, SGI287995, SGI287996, SGI287997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI287998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68112[] = {SGI0, SGI287999, SGI0, SGI288000, SGI288001, SGI288001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288002, SGI288003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68113[] = {SGI0, SGI288004, SGI0, SGI288005, SGI288006, SGI288006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288007, SGI288008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68114[] = {SGI0, SGI288009, SGI0, SGI288010, SGI288011, SGI288012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288013, SGI288014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68115[] = {SGI0, SGI288015, SGI0, SGI288016, SGI288017, SGI288017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288018, SGI288019, SGI0, SGI288020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68116[] = {SGI0, SGI288021, SGI0, SGI288022, SGI288023, SGI288023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288024, SGI288025, SGI0, SGI288026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68117[] = {SGI0, SGI288027, SGI0, SGI288028, SGI288029, SGI288030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288031, SGI288032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68118[] = {SGI0, SGI288033, SGI0, SGI288034, SGI288035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288036, SGI288037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68119[] = {SGI0, SGI288038, SGI0, SGI288039, SGI288040, SGI288040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288041, SGI288042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68120[] = {SGI0, SGI288043, SGI0, SGI288044, SGI288045, SGI288045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288046, SGI288047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68121[] = {SGI0, SGI288048, SGI0, SGI288049, SGI288050, SGI288050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288051, SGI288052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68122[] = {SGI0, SGI288053, SGI0, SGI288054, SGI288055, SGI288055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288056, SGI288057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68123[] = {SGI0, SGI288058, SGI0, SGI288059, SGI288060, SGI288061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288062, SGI288063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68124[] = {SGI0, SGI288064, SGI0, SGI288065, SGI0, SGI288066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288067, SGI288068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68125[] = {SGI0, SGI288069, SGI0, SGI288070, SGI288071, SGI288072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288073, SGI288074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68126[] = {SGI0, SGI288075, SGI0, SGI288076, SGI288077, SGI288077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288078, SGI288079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68127[] = {SGI0, SGI288080, SGI0, SGI288081, SGI288082, SGI288082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288083, SGI288084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68128[] = {SGI0, SGI288085, SGI0, SGI288086, SGI288087, SGI288087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288088, SGI288089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68129[] = {SGI0, SGI288090, SGI0, SGI288091, SGI288092, SGI288093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288094, SGI288095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68130[] = {SGI0, SGI288096, SGI0, SGI288097, SGI288098, SGI288098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288099, SGI288100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68131[] = {SGI0, SGI288101, SGI0, SGI288102, SGI288103, SGI288103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288104, SGI288105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68132[] = {SGI0, SGI288106, SGI0, SGI288107, SGI288108, SGI288108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288109, SGI288110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68133[] = {SGI0, SGI288112, SGI0, SGI288113, SGI288114, SGI288115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288116, SGI288117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68134[] = {SGI0, SGI288118, SGI0, SGI288119, SGI288120, SGI288121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288122, SGI288123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68135[] = {SGI0, SGI288124, SGI0, SGI288125, SGI288126, SGI288127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288128, SGI288129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68136[] = {SGI0, SGI288130, SGI0, SGI288131, SGI288132, SGI288133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288134, SGI288135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68137[] = {SGI0, SGI288136, SGI0, SGI288137, SGI288138, SGI288138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288139, SGI288140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68138[] = {SGI0, SGI288141, SGI0, SGI288142, SGI288143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288144, SGI288145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68139[] = {SGI0, SGI288146, SGI0, SGI288147, SGI0, SGI288148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288149, SGI288150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68140[] = {SGI0, SGI288151, SGI0, SGI288152, SGI288153, SGI288153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288154, SGI288155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68141[] = {SGI0, SGI288156, SGI0, SGI288157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288158, SGI288159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68142[] = {SGI0, SGI0, SGI0, SGI288160, SGI288161, SGI288162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288163, SGI288164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68143[] = {SGI0, SGI288165, SGI0, SGI288166, SGI288167, SGI288167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288168, SGI288169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68144[] = {SGI0, SGI288171, SGI0, SGI288172, SGI288173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288174, SGI288175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68145[] = {SGI0, SGI0, SGI0, SGI288176, SGI288177, SGI288178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68146[] = {SGI0, SGI288180, SGI0, SGI288181, SGI288182, SGI288182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288183, SGI288184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68147[] = {SGI0, SGI288185, SGI0, SGI288186, SGI288187, SGI288187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288188, SGI288189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68148[] = {SGI0, SGI288190, SGI0, SGI288191, SGI288192, SGI288192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288193, SGI288194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68149[] = {SGI0, SGI288195, SGI0, SGI288196, SGI288197, SGI288197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288198, SGI288199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68150[] = {SGI0, SGI288200, SGI0, SGI288201, SGI288202, SGI288203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288204, SGI288205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68151[] = {SGI0, SGI288206, SGI0, SGI288207, SGI288208, SGI288208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288209, SGI288210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68152[] = {SGI0, SGI288211, SGI0, SGI288212, SGI288213, SGI288213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288214, SGI288215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68153[] = {SGI0, SGI288217, SGI0, SGI288218, SGI288219, SGI288220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68154[] = {SGI0, SGI288222, SGI0, SGI288223, SGI288224, SGI288224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288225, SGI288226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68155[] = {SGI0, SGI288228, SGI0, SGI288229, SGI288230, SGI288230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288231, SGI288232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68156[] = {SGI0, SGI288233, SGI0, SGI288234, SGI288235, SGI288235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288236, SGI288237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68157[] = {SGI0, SGI288238, SGI0, SGI288239, SGI288240, SGI288240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288241, SGI288242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68158[] = {SGI0, SGI288243, SGI0, SGI288244, SGI288245, SGI288245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288246, SGI288247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68159[] = {SGI0, SGI288248, SGI0, SGI288249, SGI288250, SGI288251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288252, SGI288253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68160[] = {SGI0, SGI288254, SGI0, SGI288255, SGI288256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288257, SGI288258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68161[] = {SGI0, SGI288259, SGI0, SGI288260, SGI288261, SGI288261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288262, SGI288263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68162[] = {SGI0, SGI288264, SGI0, SGI288265, SGI288266, SGI288266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288267, SGI288268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68163[] = {SGI0, SGI288269, SGI0, SGI288270, SGI288271, SGI288271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288272, SGI288273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68164[] = {SGI0, SGI288274, SGI0, SGI288275, SGI288276, SGI288276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288277, SGI288278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68165[] = {SGI0, SGI288279, SGI0, SGI288280, SGI288281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288282, SGI288283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68166[] = {SGI0, SGI288284, SGI0, SGI288285, SGI288286, SGI288286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288287, SGI288288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68167[] = {SGI0, SGI288289, SGI0, SGI288290, SGI288291, SGI288291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288292, SGI288293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68168[] = {SGI0, SGI288295, SGI0, SGI288296, SGI288297, SGI288297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288298, SGI288299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68169[] = {SGI0, SGI288300, SGI0, SGI288301, SGI288302, SGI288302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288303, SGI288304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68170[] = {SGI0, SGI288305, SGI0, SGI288306, SGI288307, SGI288308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288309, SGI288310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68171[] = {SGI0, SGI288311, SGI0, SGI288312, SGI288313, SGI288313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288314, SGI288315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68172[] = {SGI0, SGI288316, SGI0, SGI288317, SGI288318, SGI288318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288319, SGI288320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68173[] = {SGI0, SGI288322, SGI0, SGI288323, SGI288324, SGI288324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288325, SGI288326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68174[] = {SGI0, SGI288328, SGI0, SGI288329, SGI288330, SGI288331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288332, SGI288333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68175[] = {SGI0, SGI0, SGI0, SGI288334, SGI288335, SGI288336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288338, SGI0, SGI0, NULL};
const Segment * const SG68176[] = {SGI0, SGI288339, SGI0, SGI288340, SGI288341, SGI288342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288343, SGI288344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68177[] = {SGI0, SGI288345, SGI0, SGI288346, SGI288347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288348, SGI288349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68178[] = {SGI0, SGI288350, SGI0, SGI288351, SGI288352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288353, SGI288354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68179[] = {SGI0, SGI288355, SGI0, SGI288356, SGI288357, SGI288358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288359, SGI288360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68180[] = {SGI0, SGI288361, SGI0, SGI288362, SGI288363, SGI288364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288365, SGI288366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68181[] = {SGI0, SGI288367, SGI0, SGI288368, SGI288369, SGI288370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288371, SGI288372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68182[] = {SGI0, SGI0, SGI0, SGI288373, SGI288374, SGI288375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288377, SGI0, SGI0, NULL};
const Segment * const SG68183[] = {SGI0, SGI288378, SGI0, SGI288379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288380, SGI288381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68184[] = {SGI0, SGI288382, SGI0, SGI288383, SGI288384, SGI288384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288385, SGI288386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68185[] = {SGI0, SGI288387, SGI0, SGI288388, SGI288389, SGI288390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288391, SGI288392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68186[] = {SGI0, SGI288393, SGI0, SGI288394, SGI288395, SGI288396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288397, SGI288398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68187[] = {SGI0, SGI288399, SGI0, SGI288400, SGI288401, SGI288402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68188[] = {SGI0, SGI288404, SGI0, SGI288405, SGI288406, SGI288406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288407, SGI288408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68189[] = {SGI0, SGI288410, SGI0, SGI288411, SGI288412, SGI288412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288413, SGI288414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68190[] = {SGI0, SGI288416, SGI0, SGI288417, SGI288418, SGI288418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288419, SGI288420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68191[] = {SGI0, SGI288421, SGI0, SGI288422, SGI288423, SGI288423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288424, SGI288425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68192[] = {SGI0, SGI288426, SGI0, SGI288427, SGI288428, SGI288428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288429, SGI288430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68193[] = {SGI0, SGI288431, SGI0, SGI288432, SGI288433, SGI288434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288435, SGI288436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68194[] = {SGI0, SGI288437, NULL};
const Segment * const SG68195[] = {SGI0, SGI288438, SGI0, SGI288439, SGI288440, SGI288440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288441, SGI288442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68196[] = {SGI0, SGI288443, SGI0, SGI288444, SGI288445, SGI288445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288446, SGI288447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68197[] = {SGI0, SGI288448, SGI0, SGI288449, SGI288450, SGI288451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288452, SGI288453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68198[] = {SGI0, SGI288454, SGI0, SGI288455, SGI288456, SGI288456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288457, SGI288458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68199[] = {SGI0, SGI288460, SGI0, SGI288461, SGI288462, SGI288462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288463, SGI288464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68200[] = {SGI0, SGI288466, SGI0, SGI288467, SGI288468, SGI288469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68201[] = {SGI0, SGI288471, SGI0, SGI288472, SGI288473, SGI288473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288474, SGI288475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68202[] = {SGI0, SGI288476, SGI0, SGI288477, SGI288478, SGI288478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288479, SGI288480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68203[] = {SGI0, SGI288481, SGI0, SGI288482, SGI288483, SGI288483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288484, SGI288485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68204[] = {SGI0, SGI288486, SGI0, SGI288487, SGI288488, SGI288489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288490, SGI288491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68205[] = {SGI0, SGI288492, SGI0, SGI288493, SGI288494, SGI288494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288495, SGI288496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68206[] = {SGI0, SGI288497, SGI0, SGI288498, SGI288499, SGI288499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288500, SGI288501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68207[] = {SGI0, SGI288502, SGI0, SGI288503, SGI288504, SGI288505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288506, SGI288507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68208[] = {SGI0, SGI288508, SGI0, SGI288509, SGI288510, SGI288510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68209[] = {SGI0, SGI288513, SGI0, SGI288514, SGI288515, SGI288515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288516, SGI288517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68210[] = {SGI0, SGI288518, SGI0, SGI288519, SGI288520, SGI288520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288521, SGI288522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68211[] = {SGI0, SGI288523, SGI0, SGI288524, SGI288525, SGI288525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288526, SGI288527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68212[] = {SGI0, SGI288529, NULL};
const Segment * const SG68213[] = {SGI0, SGI288530, SGI0, SGI288531, SGI288532, SGI288532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288533, SGI288534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68214[] = {SGI0, SGI288535, SGI0, SGI288536, SGI288537, SGI288537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288538, SGI288539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68215[] = {SGI0, SGI288540, SGI0, SGI288541, SGI288542, SGI288542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288543, SGI288544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68216[] = {SGI0, SGI288545, SGI0, SGI288546, SGI288547, SGI288548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288549, SGI288550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68217[] = {SGI0, SGI288551, SGI0, SGI288552, SGI288553, SGI288553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288554, SGI288555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68218[] = {SGI0, SGI288556, SGI0, SGI288557, SGI288558, SGI288559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288560, SGI288561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68219[] = {SGI0, SGI288562, SGI0, SGI288563, SGI288564, SGI288564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288565, SGI288566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68220[] = {SGI0, SGI288567, SGI0, SGI288568, SGI288569, SGI288569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288570, SGI288571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68221[] = {SGI0, SGI288572, SGI0, SGI288573, SGI288574, SGI288574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288575, SGI288576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68222[] = {SGI0, SGI288577, SGI0, SGI288578, SGI288579, SGI288579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288580, SGI288581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68223[] = {SGI0, SGI288582, SGI0, SGI288583, SGI288584, SGI288585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288586, SGI288587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68224[] = {SGI0, SGI288588, SGI0, SGI288589, SGI288590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288591, SGI288592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68225[] = {SGI0, SGI288593, SGI0, SGI288594, SGI288595, SGI288595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288596, SGI288597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68226[] = {SGI0, SGI288598, SGI0, SGI288599, SGI288600, SGI288600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288601, SGI288602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68227[] = {SGI0, SGI288603, SGI0, SGI288604, SGI288605, SGI288605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288606, SGI288607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68228[] = {SGI0, SGI288608, SGI0, SGI288609, SGI288610, SGI288611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288612, SGI288613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68229[] = {SGI0, SGI288614, SGI0, SGI288615, SGI288616, SGI288617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68230[] = {SGI0, SGI288619, SGI0, SGI288620, SGI288621, SGI288622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68231[] = {SGI0, SGI288624, SGI0, SGI288625, SGI288626, SGI288626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288627, SGI288628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68232[] = {SGI0, SGI288630, SGI0, SGI288631, SGI288632, SGI288632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288633, SGI288634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68233[] = {SGI0, SGI288635, SGI0, SGI288636, SGI288637, SGI288637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288638, SGI288639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68234[] = {SGI0, SGI288640, SGI0, SGI288641, SGI288642, SGI288642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288643, SGI288644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68235[] = {SGI0, SGI288645, SGI0, SGI288646, SGI288647, SGI288648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288649, SGI288650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68236[] = {SGI0, SGI288651, SGI0, SGI288652, SGI288653, SGI288653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288654, SGI288655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68237[] = {SGI0, SGI288656, SGI0, SGI288657, SGI288658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288659, SGI288660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68238[] = {SGI0, SGI288661, SGI0, SGI288662, SGI288663, SGI288663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288664, SGI288665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68239[] = {SGI0, SGI288666, SGI0, SGI288667, SGI288668, SGI288668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288669, SGI288670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68240[] = {SGI0, SGI288671, SGI0, SGI288672, SGI288673, SGI288674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288675, SGI288676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68241[] = {SGI288677, SGI288678, NULL};
const Segment * const SG68242[] = {SGI288679, SGI288680, NULL};
const Segment * const SG68243[] = {SGI288681, SGI288682, NULL};
const Segment * const SG68244[] = {SGI288683, SGI288684, NULL};
const Segment * const SG68245[] = {SGI288685, SGI288686, NULL};
const Segment * const SG68246[] = {SGI288687, SGI288688, NULL};
const Segment * const SG68247[] = {SGI288689, SGI288690, NULL};
const Segment * const SG68248[] = {SGI288691, SGI288692, NULL};
const Segment * const SG68249[] = {SGI288693, SGI288694, NULL};
const Segment * const SG68250[] = {SGI288695, SGI288696, NULL};
const Segment * const SG68251[] = {SGI288697, SGI288698, NULL};
const Segment * const SG68252[] = {SGI288699, SGI288700, NULL};
const Segment * const SG68253[] = {SGI288701, SGI288702, NULL};
const Segment * const SG68254[] = {SGI0, SGI288703, NULL};
const Segment * const SG68255[] = {SGI0, SGI288704, SGI0, SGI288705, SGI288706, SGI288706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288707, SGI288708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68256[] = {SGI0, SGI288710, SGI0, SGI288711, SGI288712, SGI288712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68257[] = {SGI0, SGI288714, SGI288714, SGI288715, SGI288716, SGI288716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68258[] = {SGI0, SGI288718, SGI0, SGI0, SGI0, SGI288719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68259[] = {SGI0, SGI0, SGI0, SGI288721, SGI288722, SGI288722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68260[] = {SGI0, SGI288724, SGI0, SGI288725, SGI288726, SGI288727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68261[] = {SGI0, SGI288729, SGI0, SGI288730, SGI288731, SGI288731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288732, SGI288733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68262[] = {SGI0, SGI288734, SGI0, SGI288735, SGI288736, SGI288736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288737, SGI288738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68263[] = {SGI0, SGI288739, SGI0, SGI288740, SGI288741, SGI288742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288743, SGI288744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68264[] = {SGI0, SGI288745, SGI0, SGI288746, SGI288747, SGI288748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288749, SGI288750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68265[] = {SGI0, SGI288751, SGI0, SGI288752, SGI0, SGI288753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288754, SGI288755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68266[] = {SGI0, SGI288756, SGI0, SGI288757, SGI288758, SGI288758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288759, SGI288760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68267[] = {SGI0, SGI288762, SGI0, SGI288763, SGI288764, SGI288764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288765, SGI288766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68268[] = {SGI0, SGI288767, SGI0, SGI288768, SGI288769, SGI288769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288770, SGI288771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68269[] = {SGI0, SGI288772, SGI0, SGI288773, SGI288774, SGI288774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288775, SGI288776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68270[] = {SGI0, SGI288778, SGI0, SGI288779, SGI288780, SGI288781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288782, SGI288783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68271[] = {SGI0, SGI288784, SGI0, SGI288785, SGI288786, SGI288786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288787, SGI288788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68272[] = {SGI0, SGI288789, SGI0, SGI288790, SGI288791, SGI288791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288792, SGI288793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68273[] = {SGI0, SGI288794, SGI0, SGI288795, SGI288796, SGI288796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288797, SGI288798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68274[] = {SGI0, SGI288799, SGI0, SGI288800, SGI0, SGI288801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288802, SGI288803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68275[] = {SGI0, SGI288804, SGI0, SGI288805, SGI288806, SGI288807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288808, SGI288809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68276[] = {SGI0, SGI288810, SGI0, SGI288811, SGI288812, SGI288813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288814, SGI288815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68277[] = {SGI0, SGI288816, SGI0, SGI288817, SGI288818, SGI288818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288819, SGI288820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68278[] = {SGI0, SGI288821, SGI0, SGI288822, SGI288823, SGI288823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288824, SGI288825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68279[] = {SGI0, SGI288826, SGI0, SGI288827, SGI288828, SGI288828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288829, SGI288830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68280[] = {SGI0, SGI288831, SGI0, SGI288832, SGI288833, SGI288834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68281[] = {SGI0, SGI288836, SGI0, SGI288837, SGI288838, SGI288839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68282[] = {SGI0, SGI288841, SGI0, SGI288842, SGI288843, SGI288843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288844, SGI288845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68283[] = {SGI0, SGI288847, SGI0, SGI288848, SGI288849, SGI288849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68284[] = {SGI0, SGI288851, SGI288851, SGI288852, SGI288853, SGI288853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68285[] = {SGI0, SGI0, SGI288855, SGI288856, SGI288857, SGI288857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68286[] = {SGI0, SGI288858, SGI0, SGI288859, SGI288860, SGI288860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68287[] = {SGI0, SGI288862, SGI288863, SGI288864, SGI288865, SGI288865, SGI0, SGI0, SGI288866, SGI0, SGI0, SGI0, SGI288867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68288[] = {SGI0, SGI288868, SGI0, SGI288869, SGI288870, SGI288870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68289[] = {SGI0, SGI288872, SGI288872, SGI288873, SGI288874, SGI288874, SGI0, SGI0, SGI288875, SGI0, SGI0, SGI0, SGI288876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68290[] = {SGI0, SGI288877, SGI288877, SGI288878, SGI288879, SGI288879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68291[] = {SGI0, SGI288881, SGI0, SGI288882, SGI288883, SGI288883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68292[] = {SGI0, SGI288885, SGI0, SGI288886, SGI288887, SGI288887, SGI0, SGI0, SGI288888, SGI0, SGI0, SGI0, SGI288889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68293[] = {SGI0, SGI288890, SGI0, SGI288891, SGI288892, SGI288892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68294[] = {SGI0, SGI288894, SGI0, SGI288895, SGI288896, SGI288896, SGI0, SGI0, SGI288897, SGI0, SGI0, SGI0, SGI288898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68295[] = {SGI0, SGI288899, SGI0, SGI288900, SGI288901, SGI288901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68296[] = {SGI0, SGI288903, SGI0, SGI288904, SGI288905, SGI288905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288907, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68297[] = {SGI0, SGI288908, SGI0, SGI288909, SGI288910, SGI288910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288911, SGI288912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68298[] = {SGI0, SGI288914, SGI288914, SGI288915, SGI288916, SGI288916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68299[] = {SGI0, SGI288918, SGI0, SGI288919, SGI288920, SGI288920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68300[] = {SGI0, SGI288922, SGI0, SGI288923, SGI288924, SGI288924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68301[] = {SGI0, SGI288926, SGI0, SGI288927, SGI288928, SGI288929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68302[] = {SGI0, SGI0, SGI288931, SGI288932, SGI288933, SGI288933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68303[] = {SGI0, SGI288934, SGI0, SGI288935, SGI288936, SGI288936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288937, SGI288938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68304[] = {SGI0, SGI288939, SGI0, SGI288940, SGI288941, SGI288941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288942, SGI288943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68305[] = {SGI0, SGI288944, SGI0, SGI288945, SGI288946, SGI288946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288947, SGI288948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68306[] = {SGI0, SGI288949, SGI0, SGI288950, SGI288951, SGI288952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288953, SGI288954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68307[] = {SGI0, SGI288955, SGI0, SGI288956, SGI288957, SGI288957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288958, SGI288959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68308[] = {SGI0, SGI288961, SGI0, SGI288962, SGI288963, SGI288964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68309[] = {SGI0, SGI288966, SGI0, SGI288967, SGI288968, SGI288968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288969, SGI288970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68310[] = {SGI0, SGI288972, SGI0, SGI288973, SGI288974, SGI288974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288975, SGI288976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68311[] = {SGI0, SGI288978, SGI0, SGI288979, SGI288980, SGI288980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288981, SGI288982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68312[] = {SGI0, SGI288984, SGI0, SGI288985, SGI288986, SGI288986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288987, SGI288988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68313[] = {SGI0, SGI288990, SGI0, SGI288991, SGI288992, SGI288992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288993, SGI288994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68314[] = {SGI0, SGI288995, SGI0, SGI288996, SGI288997, SGI288997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI288998, SGI288999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68315[] = {SGI0, SGI289000, SGI0, SGI289001, SGI289002, SGI289003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68316[] = {SGI0, SGI289005, SGI0, SGI289006, SGI289007, SGI289007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289008, SGI289009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68317[] = {SGI0, SGI289011, SGI0, SGI289012, SGI289013, SGI289013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289014, SGI289015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68318[] = {SGI0, SGI289016, SGI0, SGI289017, SGI289018, SGI289018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289019, SGI289020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68319[] = {SGI0, SGI289021, SGI0, SGI289022, SGI289023, SGI289024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289025, SGI289026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68320[] = {SGI0, SGI289027, SGI0, SGI289028, SGI289029, SGI289029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289030, SGI289031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68321[] = {SGI0, SGI289033, SGI0, SGI289034, SGI289035, SGI289035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289036, SGI289037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68322[] = {SGI0, SGI289039, SGI0, SGI289040, SGI289041, SGI289041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289042, SGI289043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68323[] = {SGI0, SGI289044, SGI0, SGI289045, SGI289046, SGI289046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289047, SGI289048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68324[] = {SGI0, SGI289049, SGI0, SGI289050, SGI289051, SGI289052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68325[] = {SGI0, SGI289054, SGI0, SGI289055, SGI289056, SGI289057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68326[] = {SGI0, SGI289059, SGI0, SGI289060, SGI289061, SGI289062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68327[] = {SGI0, SGI289064, SGI0, SGI289065, SGI289066, SGI289066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289067, SGI289068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68328[] = {SGI0, SGI289069, SGI0, SGI289070, SGI289071, SGI289071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289072, SGI289073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68329[] = {SGI0, SGI289074, SGI0, SGI289075, SGI289076, SGI289077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289078, SGI289079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68330[] = {SGI0, SGI289080, SGI0, SGI289081, SGI289082, SGI289082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289083, SGI289084, SGI0, SGI289085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68331[] = {SGI0, SGI289086, SGI0, SGI289087, SGI289088, SGI289088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289089, SGI289090, SGI0, SGI289091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68332[] = {SGI0, SGI289092, SGI0, SGI289093, SGI289094, SGI289095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68333[] = {SGI0, SGI289097, SGI0, SGI289098, SGI289099, SGI289100, SGI0, SGI0, SGI0, SGI0, SGI289101, SGI0, SGI289102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68334[] = {SGI0, SGI289103, SGI0, SGI289104, SGI289105, SGI289105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289106, SGI289107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68335[] = {SGI0, SGI289108, SGI0, SGI289109, SGI289110, SGI289110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289111, SGI289112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68336[] = {SGI0, SGI289113, SGI0, SGI289114, SGI289115, SGI289115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289116, SGI289117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68337[] = {SGI0, SGI289119, SGI0, SGI289120, SGI289121, SGI289121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289122, SGI289123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68338[] = {SGI0, SGI289125, SGI0, SGI289126, SGI289127, SGI289128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289129, SGI289130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68339[] = {SGI0, SGI289131, SGI0, SGI289132, SGI289133, SGI289134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68340[] = {SGI0, SGI289136, SGI0, SGI289137, SGI289138, SGI289138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289139, SGI289140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68341[] = {SGI0, SGI289142, SGI0, SGI289143, SGI289144, SGI289144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289145, SGI289146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68342[] = {SGI0, SGI289148, SGI0, SGI289149, SGI289150, SGI289151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68343[] = {SGI0, SGI289153, SGI0, SGI289154, SGI289155, SGI289156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68344[] = {SGI0, SGI289158, SGI0, SGI289159, SGI289160, SGI289160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289161, SGI289162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68345[] = {SGI0, SGI289164, SGI0, SGI289165, SGI289166, SGI289166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289167, SGI289168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68346[] = {SGI0, SGI289170, SGI0, SGI289171, SGI289172, SGI289172, SGI0, SGI0, SGI289173, SGI0, SGI0, SGI0, SGI289174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68347[] = {SGI289175, SGI289176, SGI0, SGI289177, SGI289178, SGI289178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68348[] = {SGI0, SGI289180, SGI0, SGI289181, SGI289182, SGI289182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68349[] = {SGI0, SGI289184, SGI0, SGI289185, SGI289186, SGI289186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68350[] = {SGI0, SGI0, SGI289188, SGI289189, SGI289190, SGI289190, SGI0, SGI0, SGI289191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68351[] = {SGI0, SGI289192, SGI0, SGI289193, SGI289194, SGI289194, SGI0, SGI0, SGI289195, SGI0, SGI0, SGI0, SGI289196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68352[] = {SGI0, SGI289197, SGI0, SGI289198, SGI289199, SGI289199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289200, SGI289201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68353[] = {SGI0, SGI0, SGI0, SGI289202, SGI289203, SGI289203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289204, SGI0, SGI0, NULL};
const Segment * const SG68354[] = {SGI0, SGI0, SGI0, SGI289205, SGI289206, SGI289206, SGI0, SGI0, SGI289207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289208, SGI0, SGI0, NULL};
const Segment * const SG68355[] = {SGI0, SGI289209, SGI0, SGI289210, SGI289211, SGI289212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289213, SGI289214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68356[] = {SGI0, SGI289215, SGI0, SGI289216, SGI289217, SGI289218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68357[] = {SGI0, SGI289220, SGI0, SGI289221, SGI289222, SGI289223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289224, SGI289225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68358[] = {SGI0, SGI0, SGI0, SGI289226, SGI289227, SGI289228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68359[] = {SGI0, SGI289230, SGI0, SGI289231, SGI289232, SGI289232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68360[] = {SGI0, SGI289234, SGI289234, SGI289235, SGI289236, SGI289236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68361[] = {SGI0, SGI289238, SGI0, SGI289239, SGI289240, SGI289240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289241, SGI289242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68362[] = {SGI0, SGI289243, SGI0, SGI289244, SGI289245, SGI289245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289246, SGI289247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68363[] = {SGI0, SGI289248, SGI0, SGI289249, SGI289250, SGI289250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289251, SGI289252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68364[] = {SGI0, SGI289253, SGI0, SGI289254, SGI289255, SGI289255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289256, SGI289257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68365[] = {SGI0, SGI289258, SGI0, SGI289259, SGI289260, SGI289260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289261, SGI289262, SGI0, SGI289263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68366[] = {SGI0, SGI289264, SGI0, SGI289265, SGI289266, SGI289266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289267, SGI289268, SGI0, SGI289269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68367[] = {SGI0, SGI289270, SGI0, SGI289271, SGI289272, SGI289272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68368[] = {SGI0, SGI289275, SGI0, SGI289276, SGI289277, SGI289277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289278, SGI289279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68369[] = {SGI0, SGI289281, SGI0, SGI289282, SGI289283, SGI289283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289284, SGI289285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68370[] = {SGI0, SGI289286, SGI0, SGI289287, SGI289288, SGI289288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289289, SGI289290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68371[] = {SGI0, SGI289291, SGI0, SGI289292, SGI289293, SGI289294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289295, SGI289296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68372[] = {SGI0, SGI289297, SGI0, SGI289298, SGI289299, SGI289299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289300, SGI289301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68373[] = {SGI0, SGI289302, SGI0, SGI289303, SGI289304, SGI289304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289305, SGI289306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68374[] = {SGI0, SGI289307, SGI0, SGI289308, SGI289309, SGI289309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289310, SGI289311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68375[] = {SGI0, SGI289312, SGI0, SGI289313, SGI289314, SGI289314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289315, SGI289316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68376[] = {SGI0, SGI289318, SGI0, SGI289319, SGI289320, SGI289321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289322, SGI289323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68377[] = {SGI0, SGI289324, SGI0, SGI289325, SGI289326, SGI289327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68378[] = {SGI0, SGI289329, SGI0, SGI289330, SGI289331, SGI289332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68379[] = {SGI0, SGI289334, SGI0, SGI289335, SGI289336, SGI289336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289337, SGI289338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68380[] = {SGI0, SGI289340, SGI0, SGI289341, SGI289342, SGI289342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289343, SGI289344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68381[] = {SGI0, SGI289345, SGI0, SGI289346, SGI289347, SGI289347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289348, SGI289349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68382[] = {SGI0, SGI289350, SGI0, SGI289351, SGI289352, SGI289352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68383[] = {SGI0, SGI289354, SGI289354, SGI289355, SGI289356, SGI289356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68384[] = {SGI0, SGI289358, SGI0, SGI289359, SGI289360, SGI289361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68385[] = {SGI0, SGI289363, SGI0, SGI289364, SGI289365, SGI289366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68386[] = {SGI0, SGI289368, SGI0, SGI289369, SGI289370, SGI289370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289371, SGI289372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68387[] = {SGI0, SGI289373, SGI0, SGI289374, SGI289375, SGI289375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289376, SGI289377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68388[] = {SGI0, SGI289378, SGI0, SGI289379, SGI289380, SGI289380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289381, SGI289382, SGI0, SGI289383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68389[] = {SGI0, SGI289384, SGI0, SGI289385, SGI289386, SGI289386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289387, SGI289388, SGI0, SGI289389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68390[] = {SGI0, SGI289390, SGI0, SGI289391, SGI289392, SGI289392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289393, SGI289394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68391[] = {SGI0, SGI289395, SGI0, SGI289396, SGI289397, SGI289397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289398, SGI289399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68392[] = {SGI0, SGI289400, SGI0, SGI289401, SGI289402, SGI289402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289403, SGI289404, SGI0, SGI289405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68393[] = {SGI0, SGI289406, SGI0, SGI289407, SGI289408, SGI289408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289409, SGI289410, SGI0, SGI289411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68394[] = {SGI0, SGI289412, SGI0, SGI289413, SGI289414, SGI289414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289415, SGI289416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68395[] = {SGI0, SGI289417, SGI0, SGI289418, SGI289419, SGI289419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289420, SGI289421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68396[] = {SGI0, SGI289422, SGI0, SGI289423, SGI289424, SGI289424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289425, SGI289426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68397[] = {SGI0, SGI289428, SGI0, SGI289429, SGI289430, SGI289431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68398[] = {SGI0, SGI289433, SGI0, SGI289434, SGI289435, SGI289436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68399[] = {SGI0, SGI289438, SGI0, SGI289439, SGI289440, SGI289441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68400[] = {SGI0, SGI289443, SGI0, SGI289444, SGI289445, SGI289446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68401[] = {SGI0, SGI289448, SGI0, SGI0, SGI0, SGI289449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68402[] = {SGI0, SGI289451, SGI0, SGI289452, SGI289453, SGI289454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68403[] = {SGI0, SGI289456, SGI0, SGI289457, SGI289458, SGI289458, SGI0, SGI0, SGI289459, SGI0, SGI0, SGI0, SGI289460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68404[] = {SGI0, SGI289461, SGI0, SGI289462, SGI289463, SGI289463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68405[] = {SGI0, SGI289465, SGI0, SGI289466, SGI289467, SGI289468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68406[] = {SGI0, SGI289470, SGI0, SGI289471, SGI289472, SGI289473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68407[] = {SGI0, SGI289475, SGI0, SGI289476, SGI289477, SGI289477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289478, SGI289479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68408[] = {SGI0, SGI289481, SGI0, SGI289482, SGI289483, SGI289484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68409[] = {SGI0, SGI289486, SGI0, SGI289487, SGI289488, SGI289488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289489, SGI289490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68410[] = {SGI0, SGI289492, SGI0, SGI289493, SGI289494, SGI289495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68411[] = {SGI0, SGI289497, SGI0, SGI289498, SGI289499, SGI289500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68412[] = {SGI0, SGI289502, SGI0, SGI289503, SGI289504, SGI289504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289505, SGI289506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68413[] = {SGI0, SGI289507, SGI0, SGI289508, SGI289509, SGI289509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289510, SGI289511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68414[] = {SGI0, SGI289512, SGI0, SGI0, SGI0, SGI289513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289514, SGI289515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68415[] = {SGI0, SGI289516, SGI0, SGI289517, SGI289518, SGI289518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289519, SGI289520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68416[] = {SGI0, SGI289521, SGI0, SGI289522, SGI289523, SGI289524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289525, SGI289526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68417[] = {SGI0, SGI289527, SGI0, SGI289528, SGI289529, SGI289529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289530, SGI289531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68418[] = {SGI0, SGI289532, SGI0, SGI289533, SGI289534, SGI289535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289536, SGI289537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68419[] = {SGI0, SGI289538, SGI0, SGI289539, SGI289540, SGI289540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289541, SGI289542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68420[] = {SGI0, SGI289543, SGI0, SGI289544, SGI289545, SGI289546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289547, SGI289548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68421[] = {SGI0, SGI289549, SGI0, SGI289550, SGI289551, SGI289552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289553, SGI289554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68422[] = {SGI0, SGI289555, SGI0, SGI289556, SGI289557, SGI289557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289558, SGI289559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68423[] = {SGI0, SGI289560, SGI289560, SGI289561, SGI289562, SGI289562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68424[] = {SGI0, SGI289564, SGI0, SGI289565, SGI289566, SGI289566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68425[] = {SGI0, SGI289568, SGI0, SGI289569, SGI289570, SGI289570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68426[] = {SGI0, SGI289572, SGI0, SGI289573, SGI289574, SGI289574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68427[] = {SGI0, SGI289576, SGI289576, SGI289577, SGI289578, SGI289578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68428[] = {SGI0, SGI289580, SGI0, SGI289581, SGI289582, SGI289582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289583, SGI289584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68429[] = {SGI0, SGI289585, SGI0, SGI289586, SGI289587, SGI289587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289588, SGI289589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68430[] = {SGI0, SGI289590, SGI0, SGI289591, SGI289592, SGI289593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289594, SGI289595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68431[] = {SGI0, SGI289596, SGI0, SGI289597, SGI289598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289599, SGI289600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68432[] = {SGI0, SGI289601, SGI0, SGI289602, SGI289603, SGI289603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289604, SGI289605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68433[] = {SGI0, SGI289606, SGI0, SGI289607, SGI289608, SGI289608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68434[] = {SGI0, SGI289611, SGI0, SGI289612, SGI289613, SGI289613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289614, SGI289615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68435[] = {SGI0, SGI289617, SGI0, SGI289618, SGI289619, SGI289619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289620, SGI289621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68436[] = {SGI0, SGI289622, SGI0, SGI289623, SGI289624, SGI289624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289625, SGI289626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68437[] = {SGI0, SGI289627, SGI0, SGI289628, SGI289629, SGI289629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289630, SGI289631, SGI0, SGI289632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68438[] = {SGI0, SGI289633, SGI0, SGI289634, SGI289635, SGI289635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289636, SGI289637, SGI0, SGI289638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68439[] = {SGI0, SGI289639, SGI0, SGI289640, SGI289641, SGI289641, SGI289642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289643, SGI0, SGI289644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68440[] = {SGI0, SGI289645, SGI0, SGI289646, SGI289647, SGI289648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68441[] = {SGI0, SGI289650, SGI0, SGI289651, SGI289652, SGI289653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68442[] = {SGI0, SGI289655, SGI0, SGI289656, SGI289657, SGI289657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289658, SGI289659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68443[] = {SGI0, SGI289660, SGI0, SGI289661, SGI289662, SGI289662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289663, SGI289664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68444[] = {SGI0, SGI289665, SGI0, SGI289666, SGI289667, SGI289668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289669, SGI289670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68445[] = {SGI0, SGI289671, SGI0, SGI289672, SGI289673, SGI289673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289674, SGI289675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68446[] = {SGI0, SGI289676, SGI0, SGI289677, SGI289678, SGI289678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289679, SGI289680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68447[] = {SGI0, SGI289681, SGI0, SGI289682, SGI289683, SGI289683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289684, SGI289685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68448[] = {SGI0, SGI289687, SGI0, SGI289688, SGI289689, SGI289690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68449[] = {SGI0, SGI289692, SGI0, SGI289693, SGI289694, SGI289695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68450[] = {SGI0, SGI289697, SGI0, SGI289698, SGI289699, SGI289700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68451[] = {SGI0, SGI289702, SGI0, SGI289703, SGI289704, SGI289704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68452[] = {SGI0, SGI289707, SGI0, SGI289708, SGI289709, SGI289710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289711, SGI289712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68453[] = {SGI0, SGI289713, SGI0, SGI289714, SGI289715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289716, SGI289717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68454[] = {SGI0, SGI289718, SGI0, SGI289719, SGI289720, SGI289721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289722, SGI289723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68455[] = {SGI0, SGI289724, SGI0, SGI289725, SGI289726, SGI289727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289728, SGI289729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68456[] = {SGI0, SGI289730, SGI0, SGI289731, SGI289732, SGI289732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289733, SGI289734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68457[] = {SGI0, SGI289735, SGI0, SGI289736, SGI289737, SGI289737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289738, SGI289739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68458[] = {SGI0, SGI289740, SGI0, SGI289741, SGI289742, SGI289742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289743, SGI289744, SGI0, SGI289745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68459[] = {SGI0, SGI289746, SGI0, SGI289747, SGI289748, SGI289748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289749, SGI289750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68460[] = {SGI0, SGI289751, SGI0, SGI289752, SGI289753, SGI289753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289754, SGI289755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68461[] = {SGI0, SGI289756, SGI0, SGI289757, SGI289758, SGI289759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289760, SGI289761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68462[] = {SGI0, SGI289762, SGI0, SGI289763, SGI289764, SGI289765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68463[] = {SGI0, SGI289767, SGI0, SGI289768, SGI289769, SGI289770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68464[] = {SGI0, SGI289772, SGI0, SGI289773, SGI289774, SGI289775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68465[] = {SGI0, SGI289777, SGI0, SGI289778, SGI289779, SGI289780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68466[] = {SGI0, SGI289782, SGI0, SGI289783, SGI289784, SGI289785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68467[] = {SGI0, SGI0, SGI0, SGI289787, SGI289788, SGI289789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68468[] = {SGI0, SGI0, SGI0, SGI289791, SGI289792, SGI289793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289794, SGI289795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68469[] = {SGI0, SGI0, SGI0, SGI289796, SGI289797, SGI289798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289799, SGI289800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68470[] = {SGI0, SGI0, SGI0, SGI289801, SGI289802, SGI289803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289804, SGI289805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68471[] = {SGI0, SGI0, SGI0, SGI289806, SGI289807, SGI289808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289809, SGI289810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68472[] = {SGI0, SGI0, SGI0, SGI289811, SGI289812, SGI289813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289814, SGI289815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68473[] = {SGI0, SGI0, SGI0, SGI289816, SGI289817, SGI289818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289819, SGI289820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68474[] = {SGI0, SGI0, SGI0, SGI289821, SGI289822, SGI289823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289824, SGI289825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68475[] = {SGI0, SGI0, SGI0, SGI289826, SGI289827, SGI289828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289829, SGI289830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68476[] = {SGI0, SGI0, SGI0, SGI289831, SGI289832, SGI289832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289833, SGI289834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68477[] = {SGI0, SGI0, SGI0, SGI289835, SGI289836, SGI289837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289838, SGI289839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68478[] = {SGI0, SGI289840, SGI0, SGI289841, SGI289842, SGI289843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68479[] = {SGI0, SGI289845, SGI0, SGI289846, SGI289847, SGI289848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68480[] = {SGI0, SGI289850, SGI0, SGI289851, SGI289852, SGI289852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289853, SGI289854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68481[] = {SGI0, SGI289856, SGI0, SGI289857, SGI289858, SGI289858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289859, SGI289860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68482[] = {SGI0, SGI289861, SGI0, SGI289862, SGI289863, SGI289863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289864, SGI289865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68483[] = {SGI0, SGI289866, SGI0, SGI289867, SGI289868, SGI289868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289869, SGI289870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68484[] = {SGI0, SGI289871, SGI0, SGI289872, SGI289873, SGI289874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289875, SGI289876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68485[] = {SGI0, SGI289877, SGI0, SGI289878, SGI289879, SGI289879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289880, SGI289881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68486[] = {SGI0, SGI289882, SGI0, SGI289883, SGI289884, SGI289884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289885, SGI289886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68487[] = {SGI0, SGI289887, SGI0, SGI289888, SGI289889, SGI289890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68488[] = {SGI0, SGI289892, SGI0, SGI289893, SGI289894, SGI289895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68489[] = {SGI0, SGI289897, SGI0, SGI289898, SGI289899, SGI289900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68490[] = {SGI0, SGI289902, SGI0, SGI289903, SGI289904, SGI289905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68491[] = {SGI0, SGI289907, SGI0, SGI289908, SGI289909, SGI289909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289910, SGI289911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68492[] = {SGI0, SGI289912, SGI0, SGI289913, SGI289914, SGI289914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289915, SGI289916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68493[] = {SGI0, SGI289917, SGI0, SGI289918, SGI289919, SGI289920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68494[] = {SGI0, SGI289922, SGI0, SGI289923, SGI289924, SGI289925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289926, SGI289927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68495[] = {SGI0, SGI289928, SGI0, SGI289929, SGI289930, SGI289930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289931, SGI289932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68496[] = {SGI0, SGI289933, SGI0, SGI289934, SGI289935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289936, SGI289937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68497[] = {SGI0, SGI289938, SGI0, SGI289939, SGI289940, SGI289940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289941, SGI289942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68498[] = {SGI0, SGI289943, SGI0, SGI289944, SGI289945, SGI289946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289947, SGI289948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68499[] = {SGI0, SGI289949, SGI0, SGI289950, SGI289951, SGI289952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289953, SGI289954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68500[] = {SGI0, SGI289955, SGI0, SGI289956, SGI289957, SGI289957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289958, SGI289959, SGI0, SGI0, SGI0, SGI289960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68501[] = {SGI0, SGI289961, SGI0, SGI289962, SGI289963, SGI289963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289964, SGI289965, SGI0, SGI0, SGI0, SGI289966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68502[] = {SGI0, SGI289967, SGI0, SGI289968, SGI289969, SGI289970, SGI0, SGI0, SGI0, SGI0, SGI289971, SGI0, SGI289972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68503[] = {SGI0, SGI289973, SGI0, SGI289974, SGI289975, SGI289976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68504[] = {SGI0, SGI289978, SGI0, SGI289979, SGI289980, SGI289981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68505[] = {SGI0, SGI289983, SGI0, SGI289984, SGI289985, SGI289986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68506[] = {SGI0, SGI289988, SGI0, SGI289989, SGI289990, SGI289991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68507[] = {SGI0, SGI289993, SGI0, SGI289994, SGI289995, SGI289996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI289997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68508[] = {SGI0, SGI289998, SGI0, SGI289999, SGI290000, SGI290001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68509[] = {SGI0, SGI290003, SGI0, SGI290004, SGI290005, SGI290006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68510[] = {SGI0, SGI290008, SGI0, SGI290009, SGI290010, SGI290011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68511[] = {SGI0, SGI290013, SGI0, SGI290014, SGI290015, SGI290015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290016, SGI290017, SGI0, SGI290018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68512[] = {SGI0, SGI290019, SGI0, SGI290020, SGI290021, SGI290021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290022, SGI290023, SGI0, SGI290024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68513[] = {SGI0, SGI290025, SGI0, SGI290026, SGI290027, SGI290028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68514[] = {SGI0, SGI290030, SGI0, SGI290031, SGI290032, SGI290033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68515[] = {SGI0, SGI290035, SGI0, SGI290036, SGI290037, SGI290038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68516[] = {SGI0, SGI290040, SGI0, SGI290041, SGI290042, SGI290043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68517[] = {SGI0, SGI290045, SGI0, SGI290046, SGI290047, SGI290048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68518[] = {SGI0, SGI290050, SGI0, SGI290051, SGI290052, SGI290053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68519[] = {SGI0, SGI290055, SGI0, SGI290056, SGI290057, SGI290058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68520[] = {SGI0, SGI290060, SGI0, SGI290061, SGI290062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68521[] = {SGI0, SGI290064, SGI0, SGI290065, SGI290066, SGI290067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68522[] = {SGI0, SGI290069, SGI0, SGI290070, SGI290071, SGI290072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68523[] = {SGI0, SGI290074, SGI0, SGI290075, SGI290076, SGI290076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68524[] = {SGI0, SGI290078, SGI290078, SGI290079, SGI290080, SGI290080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68525[] = {SGI0, SGI290082, SGI0, SGI290083, SGI290084, SGI290085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68526[] = {SGI0, SGI290087, SGI0, SGI290088, SGI290089, SGI290090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68527[] = {SGI0, SGI290092, SGI0, SGI290093, SGI290094, SGI290095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290096, SGI290097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68528[] = {SGI0, SGI290098, SGI0, SGI290099, SGI290100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68529[] = {SGI0, SGI290102, SGI0, SGI290103, SGI290104, SGI290105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68530[] = {SGI0, SGI290107, SGI0, SGI290108, SGI290109, SGI290109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290110, SGI290111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290112, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68531[] = {SGI0, SGI290113, SGI0, SGI290114, SGI290115, SGI290115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290116, SGI290117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290118, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68532[] = {SGI0, SGI290119, SGI0, SGI290120, SGI290121, SGI290122, SGI0, SGI0, SGI0, SGI0, SGI290123, SGI0, SGI290124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68533[] = {SGI0, SGI290125, SGI0, SGI290126, SGI290127, SGI290128, SGI0, SGI0, SGI0, SGI0, SGI290129, SGI0, SGI290130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68534[] = {SGI0, SGI290131, SGI0, SGI290132, SGI290133, SGI290134, SGI0, SGI0, SGI0, SGI0, SGI290135, SGI0, SGI290136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68535[] = {SGI0, SGI290137, SGI0, SGI290138, SGI290139, SGI290140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68536[] = {SGI0, SGI290142, SGI0, SGI290143, SGI290144, SGI290144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290145, SGI290146, SGI0, SGI290147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68537[] = {SGI0, SGI290148, SGI0, SGI290149, SGI290150, SGI290150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290151, SGI290152, SGI0, SGI290153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68538[] = {SGI0, SGI290154, SGI0, SGI290155, SGI290156, SGI290156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68539[] = {SGI0, SGI290158, SGI290158, SGI290159, SGI290160, SGI290160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68540[] = {SGI0, SGI290162, SGI0, SGI290163, SGI290164, SGI290164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68541[] = {SGI0, SGI290166, SGI0, SGI290167, SGI290168, SGI290169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68542[] = {SGI0, SGI290172, SGI0, SGI290173, SGI290174, SGI290175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68543[] = {SGI0, SGI290178, SGI0, SGI290179, SGI290180, SGI290181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290182, SGI290183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68544[] = {SGI0, SGI290184, SGI0, SGI290185, SGI290186, SGI290186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290187, SGI290188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68545[] = {SGI0, SGI290189, SGI0, SGI290190, SGI290191, SGI290191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290192, SGI290193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68546[] = {SGI0, SGI290194, SGI0, SGI290195, SGI290196, SGI290197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290198, SGI290199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68547[] = {SGI0, SGI290200, SGI0, SGI290201, SGI290202, SGI290203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68548[] = {SGI0, SGI290205, SGI0, SGI290206, SGI290207, SGI290208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68549[] = {SGI0, SGI290210, SGI0, SGI290211, SGI290212, SGI290212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290213, SGI290214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68550[] = {SGI0, SGI290216, SGI0, SGI290217, SGI290218, SGI290218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290219, SGI290220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68551[] = {SGI0, SGI290222, SGI0, SGI290223, SGI290224, SGI290224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290225, SGI290226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68552[] = {SGI0, SGI290227, SGI0, SGI290228, SGI290229, SGI290229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290230, SGI290231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68553[] = {SGI0, SGI290232, SGI0, SGI290233, SGI290234, SGI290234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290235, SGI290236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68554[] = {SGI0, SGI290237, SGI0, SGI290238, SGI290239, SGI290239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290240, SGI290241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68555[] = {SGI0, SGI290243, SGI0, SGI290244, SGI290245, SGI290245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290246, SGI290247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68556[] = {SGI0, SGI290249, SGI0, SGI290250, SGI290251, SGI290252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68557[] = {SGI0, SGI290254, SGI0, SGI290255, SGI290256, SGI290257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68558[] = {SGI0, SGI290259, SGI0, SGI290260, SGI290261, SGI290261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290262, SGI290263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68559[] = {SGI0, SGI290264, SGI0, SGI290265, SGI290266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290267, SGI290268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68560[] = {SGI0, SGI290269, SGI0, SGI290270, SGI290271, SGI290272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290273, SGI290274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68561[] = {SGI0, SGI290275, SGI0, SGI290276, SGI290277, SGI290278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290279, SGI290280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68562[] = {SGI0, SGI290281, SGI0, SGI290282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290283, SGI290284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68563[] = {SGI0, SGI290285, SGI0, SGI290286, SGI290287, SGI290287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290288, SGI290289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290290, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68564[] = {SGI0, SGI290291, SGI0, SGI290292, SGI290293, SGI290293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290294, SGI290295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290296, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68565[] = {SGI0, SGI290297, SGI0, SGI290298, SGI290299, SGI290300, SGI0, SGI0, SGI0, SGI0, SGI290301, SGI0, SGI290302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68566[] = {SGI0, SGI290303, SGI0, SGI290304, SGI290305, SGI290306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68567[] = {SGI0, SGI290308, SGI0, SGI290309, SGI290310, SGI290310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290311, SGI290312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68568[] = {SGI0, SGI290313, SGI0, SGI290314, SGI290315, SGI290315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290316, SGI290317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68569[] = {SGI0, SGI290318, SGI0, SGI290319, SGI290320, SGI290320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290321, SGI290322, SGI0, SGI290323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68570[] = {SGI0, SGI290324, SGI0, SGI290325, SGI290326, SGI290326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290327, SGI290328, SGI0, SGI290329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68571[] = {SGI0, SGI290330, SGI0, SGI290331, SGI290332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290333, SGI290334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68572[] = {SGI0, SGI290335, SGI0, SGI290336, SGI290337, SGI290337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290338, SGI290339, SGI0, SGI290340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68573[] = {SGI0, SGI290341, SGI0, SGI290342, SGI290343, SGI290343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290344, SGI290345, SGI0, SGI290346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68574[] = {SGI0, SGI290347, SGI290347, SGI290348, SGI290349, SGI290349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68575[] = {SGI0, SGI290351, SGI0, SGI290352, SGI290353, SGI290353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68576[] = {SGI0, SGI290355, SGI0, SGI290356, SGI290357, SGI290358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290359, SGI290360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68577[] = {SGI0, SGI290361, SGI0, SGI290362, SGI0, SGI290363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290364, SGI290365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68578[] = {SGI0, SGI290366, SGI0, SGI290367, SGI290368, SGI290369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290370, SGI290371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68579[] = {SGI0, SGI290372, SGI0, SGI290373, SGI290374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290375, SGI290376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68580[] = {SGI0, SGI290377, SGI0, SGI290378, SGI290379, SGI290380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290381, SGI290382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68581[] = {SGI0, SGI290383, SGI0, SGI290384, SGI290385, SGI290385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68582[] = {SGI0, SGI290387, SGI290387, SGI290388, SGI290389, SGI290389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68583[] = {SGI0, SGI290391, SGI0, SGI290392, SGI290393, SGI290393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68584[] = {SGI0, SGI290395, SGI290395, SGI290396, SGI290397, SGI290397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68585[] = {SGI0, SGI290399, SGI0, SGI290400, SGI290401, SGI290401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68586[] = {SGI0, SGI290403, SGI0, SGI290404, SGI290405, SGI290405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290406, SGI290407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290408, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68587[] = {SGI0, SGI290409, SGI0, SGI290410, SGI290411, SGI290411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290412, SGI290413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68588[] = {SGI0, SGI290414, SGI0, SGI290415, SGI290416, SGI290416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290417, SGI290418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68589[] = {SGI0, SGI290419, SGI0, SGI290420, SGI290421, SGI290421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290422, SGI290423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68590[] = {SGI0, SGI290424, SGI0, SGI290425, SGI290426, SGI290426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290427, SGI290428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68591[] = {SGI0, SGI290429, SGI0, SGI290430, SGI290431, SGI290431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290432, SGI290433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68592[] = {SGI0, SGI290434, SGI0, SGI290435, SGI290436, SGI290436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68593[] = {SGI0, SGI290438, SGI0, SGI290439, SGI290440, SGI290440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290441, SGI290442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68594[] = {SGI0, SGI290443, SGI0, SGI290444, SGI290445, SGI290445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290446, SGI290447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68595[] = {SGI0, SGI290448, SGI0, SGI290449, SGI290450, SGI290450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290451, SGI290452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68596[] = {SGI0, SGI290454, SGI0, SGI290455, SGI290456, SGI290456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290457, SGI290458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68597[] = {SGI0, SGI290459, SGI0, SGI290460, SGI0, SGI290461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290462, SGI290463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68598[] = {SGI0, SGI290464, SGI0, SGI290465, SGI290466, SGI290467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290468, SGI290469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68599[] = {SGI0, SGI290470, SGI0, SGI290471, SGI290472, SGI290472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290473, SGI290474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68600[] = {SGI0, SGI290475, SGI0, SGI290476, SGI290477, SGI290477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290478, SGI290479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68601[] = {SGI0, SGI290480, SGI0, SGI290481, SGI290482, SGI290483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290484, SGI290485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68602[] = {SGI0, SGI290486, SGI0, SGI290487, SGI290488, SGI290488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290489, SGI290490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68603[] = {SGI0, SGI290491, SGI0, SGI290492, SGI290493, SGI290493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290494, SGI290495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290496, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68604[] = {SGI0, SGI290497, SGI0, SGI290498, SGI290499, SGI290500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290501, SGI290502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68605[] = {SGI0, SGI290503, SGI0, SGI290504, SGI290505, SGI290505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290506, SGI290507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68606[] = {SGI0, SGI290508, SGI0, SGI290509, SGI290510, SGI290510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290511, SGI290512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68607[] = {SGI0, SGI290513, SGI0, SGI290514, SGI290515, SGI290515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290516, SGI290517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68608[] = {SGI0, SGI290518, SGI0, SGI290519, SGI290520, SGI290521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290522, SGI290523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68609[] = {SGI0, SGI290524, SGI0, SGI290525, SGI290526, SGI290526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290527, SGI290528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68610[] = {SGI0, SGI290529, SGI0, SGI290530, SGI290531, SGI290532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290533, SGI290534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68611[] = {SGI0, SGI0, SGI0, SGI290535, SGI290536, SGI290537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290538, SGI290539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68612[] = {SGI0, SGI290540, SGI0, SGI290541, SGI290542, SGI290543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68613[] = {SGI0, SGI290545, SGI0, SGI290546, SGI290547, SGI290547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68614[] = {SGI0, SGI290550, SGI0, SGI290551, SGI290552, SGI290553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68615[] = {SGI0, SGI290556, SGI0, SGI290557, SGI290558, SGI290558, SGI290559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290560, SGI0, SGI290561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68616[] = {SGI0, SGI290562, SGI0, SGI290563, SGI290564, SGI290565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68617[] = {SGI0, SGI290567, SGI0, SGI290568, SGI290569, SGI290569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290570, SGI290571, SGI0, SGI290572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68618[] = {SGI0, SGI290573, SGI0, SGI290574, SGI290575, SGI290575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290576, SGI290577, SGI0, SGI290578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68619[] = {SGI0, SGI290579, SGI0, SGI290580, SGI290581, SGI290581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290582, SGI290583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68620[] = {SGI0, SGI290584, SGI0, SGI290585, SGI290586, SGI290586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290587, SGI290588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68621[] = {SGI0, SGI290589, SGI0, SGI290590, SGI290591, SGI290591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290592, SGI290593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68622[] = {SGI0, SGI290595, SGI0, SGI290596, SGI290597, SGI290597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290598, SGI290599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68623[] = {SGI0, SGI290601, SGI0, SGI290602, SGI290603, SGI290603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290604, SGI290605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68624[] = {SGI0, SGI290607, SGI0, SGI290608, SGI290609, SGI290609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290610, SGI290611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68625[] = {SGI0, SGI290613, SGI0, SGI290614, SGI290615, SGI290615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290616, SGI290617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68626[] = {SGI0, SGI290619, SGI0, SGI290620, SGI290621, SGI290621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290622, SGI290623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68627[] = {SGI0, SGI290625, SGI0, SGI290626, SGI290627, SGI290627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290628, SGI290629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68628[] = {SGI0, SGI290631, SGI0, SGI290632, SGI290633, SGI290633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290634, SGI290635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68629[] = {SGI0, SGI290637, SGI0, SGI290638, SGI290639, SGI290639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290640, SGI290641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68630[] = {SGI0, SGI290643, SGI0, SGI290644, SGI290645, SGI290645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290646, SGI290647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68631[] = {SGI0, SGI290649, SGI0, SGI290650, SGI290651, SGI290652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68632[] = {SGI0, SGI290654, SGI0, SGI290655, SGI290656, SGI290656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290657, SGI290658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68633[] = {SGI0, SGI290660, SGI0, SGI290661, SGI290662, SGI290662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290663, SGI290664, SGI0, SGI290665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68634[] = {SGI0, SGI290666, SGI0, SGI290667, SGI290668, SGI290668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290669, SGI290670, SGI0, SGI290671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68635[] = {SGI0, SGI290672, SGI0, SGI290673, SGI290674, SGI290674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68636[] = {SGI0, SGI290677, SGI0, SGI290678, SGI290679, SGI290679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290680, SGI290681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68637[] = {SGI0, SGI290682, SGI0, SGI290683, SGI290684, SGI290684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290685, SGI290686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68638[] = {SGI0, SGI290687, SGI0, SGI290688, SGI290689, SGI290690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68639[] = {SGI0, SGI290692, SGI0, SGI290693, SGI290694, SGI290694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290695, SGI290696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68640[] = {SGI0, SGI290697, SGI0, SGI290698, SGI290699, SGI290699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290700, SGI290701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68641[] = {SGI0, SGI290702, SGI0, SGI290703, SGI290704, SGI290705, SGI0, SGI0, SGI0, SGI0, SGI290706, SGI0, SGI290707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68642[] = {SGI0, SGI290708, SGI0, SGI290709, SGI290710, SGI290710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290711, SGI290712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68643[] = {SGI0, SGI290713, SGI0, SGI290714, SGI290715, SGI290715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290716, SGI290717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68644[] = {SGI0, SGI290718, SGI0, SGI290719, SGI290720, SGI290720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290721, SGI290722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68645[] = {SGI0, SGI290723, SGI0, SGI290724, SGI290725, SGI290725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290726, SGI290727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68646[] = {SGI0, SGI290728, SGI0, SGI290729, SGI290730, SGI290730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68647[] = {SGI0, SGI290733, SGI0, SGI290734, SGI290735, SGI290735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290736, SGI290737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290738, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68648[] = {SGI0, SGI290739, SGI0, SGI290740, SGI290741, SGI290741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290742, SGI290743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290744, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68649[] = {SGI0, SGI290745, SGI0, SGI290746, SGI290747, SGI290747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290748, SGI290749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68650[] = {SGI0, SGI290750, SGI0, SGI290751, SGI290752, SGI290752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290753, SGI290754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68651[] = {SGI0, SGI290755, SGI0, SGI290756, SGI290757, SGI290758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68652[] = {SGI0, SGI290760, SGI0, SGI290761, SGI290762, SGI290763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68653[] = {SGI0, SGI290765, SGI0, SGI290766, SGI290767, SGI290767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290768, SGI290769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290770, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68654[] = {SGI0, SGI290771, SGI0, SGI290772, SGI290773, SGI290773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290774, SGI290775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290776, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68655[] = {SGI0, SGI290777, SGI0, SGI290778, SGI290779, SGI290779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68656[] = {SGI0, SGI290782, SGI0, SGI290783, SGI290784, SGI290785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68657[] = {SGI0, SGI290787, SGI0, SGI290788, SGI290789, SGI290789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290790, SGI290791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68658[] = {SGI0, SGI290793, SGI0, SGI290794, SGI290795, SGI290795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290796, SGI290797, SGI0, SGI290798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68659[] = {SGI0, SGI290799, SGI0, SGI290800, SGI290801, SGI290801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290802, SGI290803, SGI0, SGI290804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68660[] = {SGI290805, SGI290806, SGI290807, SGI290808, SGI290809, SGI290806, SGI0, NULL};
const Segment * const SG68661[] = {SGI290810, SGI290811, SGI290812, SGI290813, SGI290814, SGI290811, SGI0, NULL};
const Segment * const SG68662[] = {SGI0, SGI290815, SGI0, SGI290816, SGI290817, SGI290817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290818, SGI290819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68663[] = {SGI0, SGI290820, SGI0, SGI290821, SGI290822, SGI290822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290823, SGI290824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68664[] = {SGI0, SGI290825, SGI0, SGI290826, SGI290827, SGI290827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290828, SGI290829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68665[] = {SGI0, SGI290830, SGI0, SGI290831, SGI290832, SGI290832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290833, SGI290834, SGI0, SGI0, SGI0, SGI290835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68666[] = {SGI0, SGI290836, SGI0, SGI290837, SGI290838, SGI290838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290839, SGI290840, SGI0, SGI0, SGI0, SGI290841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68667[] = {SGI0, SGI290842, SGI0, SGI290843, SGI290844, SGI290844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290845, SGI290846, SGI0, SGI0, SGI0, SGI290847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68668[] = {SGI0, SGI290848, SGI0, SGI290849, SGI290850, SGI290851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290852, SGI290853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68669[] = {SGI0, SGI290854, SGI0, SGI290855, SGI290856, SGI290856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290857, SGI290858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68670[] = {SGI0, SGI290859, SGI0, SGI290860, SGI290861, SGI290862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68671[] = {SGI0, SGI290864, SGI0, SGI290865, SGI290866, SGI290866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290867, SGI290868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68672[] = {SGI0, SGI290869, SGI0, SGI290870, SGI290871, SGI290871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290872, SGI290873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68673[] = {SGI0, SGI290874, SGI0, SGI290875, SGI290876, SGI290876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290877, SGI290878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68674[] = {SGI0, SGI290879, SGI0, SGI290880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290881, SGI290882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68675[] = {SGI0, SGI0, SGI0, SGI290883, SGI290884, SGI290885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290886, SGI290887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68676[] = {SGI0, SGI290888, SGI0, SGI290889, SGI290890, SGI290890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290891, SGI290892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68677[] = {SGI0, SGI290893, SGI0, SGI290894, SGI290895, SGI290896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290897, SGI290898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68678[] = {SGI0, SGI290899, SGI290899, SGI290900, SGI290901, SGI290901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68679[] = {SGI0, SGI290903, SGI0, SGI290904, SGI290905, SGI290905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68680[] = {SGI0, SGI0, SGI290907, SGI290908, SGI290909, SGI290909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68681[] = {SGI0, SGI290910, SGI0, SGI290911, SGI290912, SGI290912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68682[] = {SGI0, SGI290914, SGI290914, SGI290915, SGI290916, SGI290916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68683[] = {SGI0, SGI290918, SGI0, SGI290919, SGI290920, SGI290920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68684[] = {SGI0, SGI0, SGI290922, SGI290923, SGI290924, SGI290924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68685[] = {SGI0, SGI290925, SGI0, SGI290926, SGI290927, SGI290927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68686[] = {SGI0, SGI290930, SGI0, SGI290931, SGI290932, SGI290932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290933, SGI290934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290935, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68687[] = {SGI0, SGI290936, SGI0, SGI290937, SGI290938, SGI290938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290939, SGI290940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290941, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68688[] = {SGI0, SGI290942, SGI0, SGI290943, SGI290944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68689[] = {SGI0, SGI0, SGI0, SGI290946, SGI290947, SGI290948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290949, SGI0, SGI0, NULL};
const Segment * const SG68690[] = {SGI0, SGI290950, SGI0, SGI290951, SGI290952, SGI290953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68691[] = {SGI0, SGI290955, SGI0, SGI290956, SGI290957, SGI290958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68692[] = {SGI0, SGI290960, SGI0, SGI290961, SGI290962, SGI290962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290963, SGI290964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290965, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68693[] = {SGI0, SGI290966, SGI0, SGI290967, SGI290968, SGI290969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68694[] = {SGI0, SGI290972, SGI0, SGI290973, SGI290974, SGI290975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68695[] = {SGI0, SGI290977, SGI0, SGI290978, SGI290979, SGI290980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68696[] = {SGI0, SGI290982, SGI0, SGI290983, SGI290984, SGI290985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68697[] = {SGI0, SGI290987, SGI0, SGI290988, SGI290989, SGI290989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290990, SGI290991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68698[] = {SGI0, SGI290992, SGI0, SGI290993, SGI290994, SGI290994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI290995, SGI290996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68699[] = {SGI0, SGI290997, SGI0, SGI290998, SGI290999, SGI291000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291001, SGI291002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68700[] = {SGI0, SGI291003, SGI0, SGI291004, SGI291005, SGI291006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68701[] = {SGI0, SGI291009, SGI0, SGI291010, SGI291011, SGI291012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68702[] = {SGI0, SGI291015, SGI0, SGI291016, SGI291017, SGI291017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291018, SGI291019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291020, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68703[] = {SGI0, SGI291021, SGI0, SGI291022, SGI291023, SGI291023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291024, SGI291025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291026, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68704[] = {SGI0, SGI291027, SGI291027, SGI291028, SGI291029, SGI291029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68705[] = {SGI0, SGI291031, SGI0, SGI291032, SGI291033, SGI291033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68706[] = {SGI0, SGI0, SGI291035, SGI291036, SGI291037, SGI291037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68707[] = {SGI0, SGI291038, SGI0, SGI291039, SGI291040, SGI291040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68708[] = {SGI0, SGI291043, SGI0, SGI291044, SGI291045, SGI291045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291046, SGI291047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68709[] = {SGI0, SGI291049, SGI0, SGI291050, SGI291051, SGI291052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68710[] = {SGI0, SGI291054, SGI0, SGI291055, SGI291056, SGI291057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68711[] = {SGI0, SGI291059, SGI0, SGI291060, SGI291061, SGI291061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68712[] = {SGI0, SGI291064, SGI0, SGI291065, SGI291066, SGI291066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68713[] = {SGI0, SGI291068, SGI0, SGI291069, SGI0, SGI291070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68714[] = {SGI0, SGI291072, SGI291072, SGI291073, SGI291074, SGI291074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68715[] = {SGI0, SGI291076, SGI0, SGI291077, SGI291078, SGI291078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291079, SGI291080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291081, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68716[] = {SGI0, SGI291082, SGI0, SGI291083, SGI291084, SGI291084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291085, SGI291086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291087, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68717[] = {SGI0, SGI291088, SGI0, SGI291089, SGI291090, SGI291091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68718[] = {SGI0, SGI291093, SGI0, SGI291094, SGI291095, SGI291095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291096, SGI291097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291098, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68719[] = {SGI0, SGI291099, SGI0, SGI291100, SGI291101, SGI291101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291102, SGI291103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291104, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68720[] = {SGI0, SGI291105, SGI0, SGI291106, SGI291107, SGI291108, SGI0, SGI0, SGI0, SGI0, SGI291109, SGI0, SGI291110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68721[] = {SGI0, SGI291111, SGI0, SGI291112, SGI291113, SGI291114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68722[] = {SGI0, SGI291116, SGI0, SGI291117, SGI291118, SGI291119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291120, SGI291121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68723[] = {SGI0, SGI291122, SGI0, SGI291123, SGI291124, SGI291125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291126, SGI291127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68724[] = {SGI0, SGI291128, SGI0, SGI291129, SGI291130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291131, SGI291132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68725[] = {SGI0, SGI291133, SGI0, SGI291134, SGI291135, SGI291136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68726[] = {SGI0, SGI291139, SGI0, SGI291140, SGI291141, SGI291142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291143, SGI291144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68727[] = {SGI0, SGI291145, SGI0, SGI291146, SGI291147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291148, SGI291149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68728[] = {SGI0, SGI291150, SGI0, SGI291151, SGI291152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291153, SGI291154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68729[] = {SGI0, SGI291155, SGI0, SGI291156, SGI291157, SGI291158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291159, SGI291160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291161, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68730[] = {SGI0, SGI291162, SGI0, SGI291163, SGI291164, SGI291164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291165, SGI291166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68731[] = {SGI0, SGI291167, SGI0, SGI291168, SGI291169, SGI291169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291170, SGI291171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68732[] = {SGI0, SGI291172, SGI0, SGI291173, SGI291174, SGI291175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68733[] = {SGI0, SGI291177, SGI0, SGI291178, SGI291179, SGI291180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68734[] = {SGI0, SGI291182, SGI0, SGI291183, SGI291184, SGI291185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291186, SGI291187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68735[] = {SGI0, SGI291188, SGI0, SGI291189, SGI291190, SGI291191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68736[] = {SGI0, SGI291193, SGI0, SGI291194, SGI291195, SGI291196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68737[] = {SGI0, SGI291198, NULL};
const Segment * const SG68738[] = {SGI0, SGI291199, NULL};
const Segment * const SG68739[] = {SGI0, SGI291200, NULL};
const Segment * const SG68740[] = {SGI0, SGI291201, NULL};
const Segment * const SG68741[] = {SGI0, SGI291202, NULL};
const Segment * const SG68742[] = {SGI0, SGI291203, NULL};
const Segment * const SG68743[] = {SGI0, SGI291204, NULL};
const Segment * const SG68744[] = {SGI0, SGI291205, NULL};
const Segment * const SG68745[] = {SGI0, SGI291206, NULL};
const Segment * const SG68746[] = {SGI0, SGI291207, NULL};
const Segment * const SG68747[] = {SGI0, SGI291208, NULL};
const Segment * const SG68748[] = {SGI0, SGI291209, NULL};
const Segment * const SG68749[] = {SGI0, SGI291210, NULL};
const Segment * const SG68750[] = {SGI0, SGI291211, NULL};
const Segment * const SG68751[] = {SGI0, SGI291212, NULL};
const Segment * const SG68752[] = {SGI0, SGI291213, NULL};
const Segment * const SG68753[] = {SGI0, SGI291214, NULL};
const Segment * const SG68754[] = {SGI0, SGI291215, NULL};
const Segment * const SG68755[] = {SGI0, SGI291216, NULL};
const Segment * const SG68756[] = {SGI0, SGI291217, SGI0, SGI291218, SGI291219, SGI291219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68757[] = {SGI0, SGI291222, NULL};
const Segment * const SG68758[] = {SGI0, SGI291223, NULL};
const Segment * const SG68759[] = {SGI0, SGI291224, NULL};
const Segment * const SG68760[] = {SGI0, SGI291225, NULL};
const Segment * const SG68761[] = {SGI0, SGI291226, NULL};
const Segment * const SG68762[] = {SGI0, SGI291227, NULL};
const Segment * const SG68763[] = {SGI0, SGI291228, NULL};
const Segment * const SG68764[] = {SGI0, SGI291229, NULL};
const Segment * const SG68765[] = {SGI0, SGI291230, NULL};
const Segment * const SG68766[] = {SGI0, SGI291231, NULL};
const Segment * const SG68767[] = {SGI0, SGI291232, NULL};
const Segment * const SG68768[] = {SGI0, SGI291233, NULL};
const Segment * const SG68769[] = {SGI0, SGI291234, NULL};
const Segment * const SG68770[] = {SGI0, SGI291235, NULL};
const Segment * const SG68771[] = {SGI0, SGI291236, NULL};
const Segment * const SG68772[] = {SGI0, SGI291237, NULL};
const Segment * const SG68773[] = {SGI0, SGI291238, NULL};
const Segment * const SG68774[] = {SGI0, SGI291239, NULL};
const Segment * const SG68775[] = {SGI0, SGI291240, NULL};
const Segment * const SG68776[] = {SGI0, SGI291241, NULL};
const Segment * const SG68777[] = {SGI0, SGI291242, NULL};
const Segment * const SG68778[] = {SGI0, SGI291243, NULL};
const Segment * const SG68779[] = {SGI0, SGI291244, NULL};
const Segment * const SG68780[] = {SGI0, SGI291245, NULL};
const Segment * const SG68781[] = {SGI0, SGI291249, NULL};
const Segment * const SG68782[] = {SGI0, SGI291246, SGI0, SGI291247, SGI291248, SGI291248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291251, SGI0, NULL};
const Segment * const SG68783[] = {SGI0, SGI291252, NULL};
const Segment * const SG68784[] = {SGI0, SGI291253, SGI0, SGI291254, SGI291255, SGI291256, SGI0, SGI0, SGI0, SGI0, SGI291257, SGI0, SGI291258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68785[] = {SGI0, SGI291259, SGI0, SGI291260, SGI291261, SGI291262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68786[] = {SGI0, SGI291265, SGI0, SGI291266, SGI291267, SGI291267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291268, SGI291269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68787[] = {SGI291271, SGI291272, SGI291273, SGI291274, SGI291275, SGI291272, SGI0, NULL};
const Segment * const SG68788[] = {SGI291276, SGI291277, SGI291278, SGI291279, SGI291280, SGI291277, SGI0, NULL};
const Segment * const SG68789[] = {SGI0, SGI291281, SGI0, SGI291282, SGI291283, SGI291284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68790[] = {SGI0, SGI291286, SGI0, SGI291287, SGI291288, SGI291288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291289, SGI291290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68791[] = {SGI0, SGI291291, SGI0, SGI291292, SGI291293, SGI291293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291294, SGI291295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291296, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68792[] = {SGI0, SGI291297, SGI0, SGI291298, SGI291299, SGI291299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291300, SGI291301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68793[] = {SGI0, SGI291302, SGI0, SGI291303, SGI291304, SGI291304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68794[] = {SGI0, SGI291306, SGI0, SGI291307, SGI291308, SGI291308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68795[] = {SGI0, SGI291310, SGI291310, SGI291311, SGI291312, SGI291312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68796[] = {SGI0, SGI291314, SGI0, SGI291315, SGI291316, SGI291316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291317, SGI291318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68797[] = {SGI0, SGI291319, SGI0, SGI291320, SGI291321, SGI291321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291322, SGI291323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68798[] = {SGI0, SGI291324, SGI0, SGI291325, SGI291326, SGI291326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291327, SGI291328, SGI0, SGI0, SGI0, SGI291329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68799[] = {SGI0, SGI291330, SGI0, SGI291331, SGI291332, SGI291332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291333, SGI291334, SGI0, SGI0, SGI0, SGI291335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68800[] = {SGI0, SGI291336, SGI0, SGI291337, SGI291338, SGI291338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291340, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68801[] = {SGI0, SGI291341, SGI0, SGI291342, SGI291343, SGI291344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68802[] = {SGI0, SGI291346, SGI0, SGI291347, SGI291348, SGI291349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291350, SGI291351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68803[] = {SGI0, SGI291352, SGI0, SGI291353, SGI291354, SGI291354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291355, SGI291356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68804[] = {SGI0, SGI0, SGI0, SGI291357, SGI291358, SGI291359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291361, SGI0, SGI0, NULL};
const Segment * const SG68805[] = {SGI0, SGI291362, SGI0, SGI291363, SGI291364, SGI291364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291365, SGI291366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68806[] = {SGI0, SGI291367, SGI0, SGI291368, SGI291369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68807[] = {SGI0, SGI291372, SGI0, SGI291373, SGI291374, SGI291374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291375, SGI291376, SGI0, SGI291377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68808[] = {SGI0, SGI291378, SGI0, SGI291379, SGI291380, SGI291380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291381, SGI291382, SGI0, SGI291383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68809[] = {SGI291384, SGI291385, NULL};
const Segment * const SG68810[] = {SGI0, SGI291386, SGI0, SGI291387, SGI291388, SGI291389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68811[] = {SGI0, SGI291392, SGI0, SGI291393, SGI291394, SGI291395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68812[] = {SGI0, SGI291398, SGI0, SGI291399, SGI291400, SGI291400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291401, SGI291402, SGI0, SGI0, SGI0, SGI291403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68813[] = {SGI0, SGI291404, SGI0, SGI291405, SGI291406, SGI291406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291407, SGI291408, SGI0, SGI0, SGI0, SGI291409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68814[] = {SGI0, SGI291410, SGI0, SGI291411, SGI291412, SGI291412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291413, SGI291414, SGI0, SGI291415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68815[] = {SGI0, SGI291416, SGI0, SGI291417, SGI291418, SGI291418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291419, SGI291420, SGI0, SGI291421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68816[] = {SGI0, SGI291422, SGI0, SGI291423, SGI291424, SGI291425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68817[] = {SGI0, SGI291427, SGI0, SGI291428, SGI291429, SGI291430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68818[] = {SGI0, SGI0, SGI0, SGI291432, SGI291433, SGI291434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68819[] = {SGI0, SGI291436, SGI0, SGI291437, SGI291438, SGI291439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68820[] = {SGI0, SGI291441, SGI0, SGI291442, SGI291443, SGI291443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291444, SGI291445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68821[] = {SGI0, SGI291447, SGI0, SGI291448, SGI291449, SGI291449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291450, SGI291451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68822[] = {SGI0, SGI291453, SGI0, SGI291454, SGI291455, SGI291455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291456, SGI291457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68823[] = {SGI0, SGI291459, SGI0, SGI291460, SGI291461, SGI291461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291462, SGI291463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68824[] = {SGI0, SGI291465, SGI0, SGI291466, SGI291467, SGI291467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291468, SGI291469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68825[] = {SGI0, SGI291471, SGI0, SGI291472, SGI291473, SGI291473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291474, SGI291475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68826[] = {SGI0, SGI291477, SGI0, SGI291478, SGI291479, SGI291479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291480, SGI291481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68827[] = {SGI0, SGI291483, SGI0, SGI291484, SGI291485, SGI291485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291486, SGI291487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68828[] = {SGI0, SGI291489, SGI0, SGI291490, SGI291491, SGI291491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291492, SGI291493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68829[] = {SGI0, SGI291495, SGI0, SGI291496, SGI291497, SGI291497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291498, SGI291499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68830[] = {SGI0, SGI291501, SGI0, SGI291502, SGI291503, SGI291503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291504, SGI291505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291506, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68831[] = {SGI0, SGI291507, SGI0, SGI291508, SGI291509, SGI291509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291510, SGI291511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291512, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68832[] = {SGI0, SGI291513, SGI0, SGI291514, SGI291515, SGI291516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68833[] = {SGI0, SGI291518, SGI0, SGI291519, SGI291520, SGI291520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291521, SGI291522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291523, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68834[] = {SGI0, SGI291524, SGI0, SGI291525, SGI291526, SGI291527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291528, SGI291529, SGI0, SGI291530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68835[] = {SGI0, SGI291531, SGI0, SGI291532, SGI291533, SGI291534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291535, SGI291536, SGI0, SGI291537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68836[] = {SGI0, SGI291538, SGI0, SGI291539, SGI291540, SGI291541, SGI0, SGI0, SGI0, SGI0, SGI291542, SGI0, SGI291543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68837[] = {SGI0, SGI291544, SGI0, SGI291545, SGI291546, SGI291546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291547, SGI291548, SGI0, SGI291549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68838[] = {SGI0, SGI291550, SGI0, SGI291551, SGI291552, SGI291552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291553, SGI291554, SGI0, SGI291555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68839[] = {SGI0, SGI291556, SGI0, SGI291557, SGI291558, SGI291558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291559, SGI291560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68840[] = {SGI0, SGI291562, SGI0, SGI291563, SGI291564, SGI291565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68841[] = {SGI0, SGI291567, SGI291567, SGI291568, SGI291569, SGI291569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68842[] = {SGI0, SGI291571, SGI0, SGI291572, SGI291573, SGI291573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68843[] = {SGI0, SGI291575, SGI0, SGI291576, SGI291577, SGI291577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291578, SGI291579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291580, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68844[] = {SGI0, SGI291581, SGI0, SGI291582, SGI291583, SGI291583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291584, SGI291585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291586, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68845[] = {SGI0, SGI291587, SGI0, SGI291588, SGI291589, SGI291590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68846[] = {SGI0, SGI291592, SGI0, SGI291593, SGI291594, SGI291595, SGI0, SGI0, SGI0, SGI0, SGI291596, SGI0, SGI291597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68847[] = {SGI0, SGI291598, SGI0, SGI291599, SGI291600, SGI291600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291601, SGI291602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68848[] = {SGI0, SGI291604, SGI0, SGI291605, SGI291606, SGI291606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291607, SGI291608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68849[] = {SGI0, SGI291610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291611, SGI291612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68850[] = {SGI0, SGI291613, SGI0, SGI291614, SGI291615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291616, SGI291617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68851[] = {SGI0, SGI291618, SGI0, SGI291619, SGI291620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291621, SGI291622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68852[] = {SGI0, SGI291623, SGI0, SGI291624, SGI291625, SGI291626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68853[] = {SGI0, SGI291628, SGI0, SGI291629, SGI291630, SGI291630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291631, SGI291632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291633, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68854[] = {SGI0, SGI291634, SGI0, SGI291635, SGI291636, SGI291636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291637, SGI291638, SGI0, SGI291639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68855[] = {SGI0, SGI291640, SGI0, SGI291641, SGI291642, SGI291642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291643, SGI291644, SGI0, SGI291645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68856[] = {SGI0, SGI291646, NULL};
const Segment * const SG68857[] = {SGI0, SGI291647, NULL};
const Segment * const SG68858[] = {SGI0, SGI291648, NULL};
const Segment * const SG68859[] = {SGI0, SGI291649, NULL};
const Segment * const SG68860[] = {SGI0, SGI291650, NULL};
const Segment * const SG68861[] = {SGI0, SGI291651, NULL};
const Segment * const SG68862[] = {SGI0, SGI291652, NULL};
const Segment * const SG68863[] = {SGI0, SGI291653, NULL};
const Segment * const SG68864[] = {SGI0, SGI291654, NULL};
const Segment * const SG68865[] = {SGI0, SGI291655, NULL};
const Segment * const SG68866[] = {SGI0, SGI291656, NULL};
const Segment * const SG68867[] = {SGI0, SGI291657, NULL};
const Segment * const SG68868[] = {SGI0, SGI291658, NULL};
const Segment * const SG68869[] = {SGI0, SGI291659, NULL};
const Segment * const SG68870[] = {SGI0, SGI291660, NULL};
const Segment * const SG68871[] = {SGI0, SGI291661, NULL};
const Segment * const SG68872[] = {SGI0, SGI291662, NULL};
const Segment * const SG68873[] = {SGI0, SGI291663, NULL};
const Segment * const SG68874[] = {SGI0, SGI291664, NULL};
const Segment * const SG68875[] = {SGI0, SGI291665, NULL};
const Segment * const SG68876[] = {SGI0, SGI291666, NULL};
const Segment * const SG68877[] = {SGI0, SGI291667, NULL};
const Segment * const SG68878[] = {SGI0, SGI291668, NULL};
const Segment * const SG68879[] = {SGI0, SGI291669, NULL};
const Segment * const SG68880[] = {SGI0, SGI291670, NULL};
const Segment * const SG68881[] = {SGI0, SGI291671, NULL};
const Segment * const SG68882[] = {SGI0, SGI291672, NULL};
const Segment * const SG68883[] = {SGI0, SGI291673, NULL};
const Segment * const SG68884[] = {SGI0, SGI291674, NULL};
const Segment * const SG68885[] = {SGI0, SGI291675, NULL};
const Segment * const SG68886[] = {SGI0, SGI291676, NULL};
const Segment * const SG68887[] = {SGI0, SGI291677, NULL};
const Segment * const SG68888[] = {SGI0, SGI291678, SGI0, SGI291679, SGI291680, SGI291681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68889[] = {SGI0, SGI291683, SGI0, SGI291684, SGI291685, SGI291686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291687, SGI291688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68890[] = {SGI0, SGI291689, SGI0, SGI291690, SGI291691, SGI291692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291693, SGI291694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68891[] = {SGI0, SGI291695, SGI0, SGI291696, SGI291697, SGI291697, SGI291698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291699, SGI0, SGI291700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68892[] = {SGI0, SGI291701, SGI0, SGI291702, SGI291703, SGI291703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291705, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68893[] = {SGI0, SGI291706, SGI0, SGI291707, SGI291708, SGI291709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68894[] = {SGI0, SGI291711, SGI0, SGI291712, SGI291713, SGI291713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291714, SGI291715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68895[] = {SGI0, SGI291716, SGI0, SGI291717, SGI291718, SGI291718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291719, SGI291720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68896[] = {SGI0, SGI291721, SGI0, SGI291722, SGI291723, SGI291724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68897[] = {SGI0, SGI291726, SGI0, SGI291727, SGI291728, SGI291728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291730, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68898[] = {SGI0, SGI291731, SGI0, SGI291732, SGI291733, SGI291733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291734, SGI291735, SGI0, SGI291736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68899[] = {SGI0, SGI291737, SGI0, SGI291738, SGI291739, SGI291739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291740, SGI291741, SGI0, SGI291742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68900[] = {SGI0, SGI291743, SGI0, SGI291744, SGI291745, SGI291745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291746, SGI291747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291748, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68901[] = {SGI0, SGI291749, SGI0, SGI291750, SGI291751, SGI291752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68902[] = {SGI0, SGI291754, SGI0, SGI291755, SGI291756, SGI291756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291757, SGI291758, SGI0, SGI291759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68903[] = {SGI0, SGI291760, SGI0, SGI291761, SGI291762, SGI291762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291763, SGI291764, SGI0, SGI291765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68904[] = {SGI0, SGI291766, SGI0, SGI291767, SGI291768, SGI291768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291769, SGI291770, SGI0, SGI291771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68905[] = {SGI0, SGI291772, SGI0, SGI291773, SGI291774, SGI291774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291775, SGI291776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68906[] = {SGI0, SGI291777, SGI0, SGI291778, SGI291779, SGI291780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291781, SGI291782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68907[] = {SGI0, SGI291783, SGI0, SGI291784, SGI291785, SGI291786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68908[] = {SGI0, SGI291788, SGI0, SGI291789, SGI291790, SGI291790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68909[] = {SGI0, SGI291793, SGI0, SGI291794, SGI291795, SGI291796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291797, SGI291798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68910[] = {SGI0, SGI291799, SGI0, SGI291800, SGI291801, SGI291802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291803, SGI291804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68911[] = {SGI0, SGI291805, SGI0, SGI291806, SGI291807, SGI291807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68912[] = {SGI291810, SGI291811, NULL};
const Segment * const SG68913[] = {SGI291812, SGI291813, NULL};
const Segment * const SG68914[] = {SGI0, SGI291814, SGI0, SGI291815, SGI291816, SGI291816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291818, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68915[] = {SGI0, SGI291819, SGI0, SGI291820, SGI291821, SGI291821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291822, SGI291823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68916[] = {SGI0, SGI291825, SGI0, SGI291826, SGI291827, SGI291827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68917[] = {SGI0, SGI291830, SGI0, SGI291831, SGI291832, SGI291833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68918[] = {SGI0, SGI291835, SGI0, SGI291836, SGI291837, SGI291837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291838, SGI291839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291840, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68919[] = {SGI0, SGI291841, SGI0, SGI291842, SGI291843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68920[] = {SGI0, SGI291846, SGI0, SGI291847, SGI291848, SGI291848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291849, SGI291850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291851, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68921[] = {SGI0, SGI291852, SGI0, SGI291853, SGI291854, SGI291854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291856, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68922[] = {SGI0, SGI291857, SGI0, SGI291858, SGI291859, SGI291859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291860, SGI291861, SGI0, SGI291862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68923[] = {SGI0, SGI291863, SGI0, SGI291864, SGI291865, SGI291865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291866, SGI291867, SGI0, SGI291868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68924[] = {SGI0, SGI291869, SGI0, SGI291870, SGI291871, SGI291872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68925[] = {SGI0, SGI291874, SGI0, SGI291875, SGI291876, SGI291877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68926[] = {SGI0, SGI291879, SGI0, SGI291880, SGI291881, SGI291881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291882, SGI291883, SGI0, SGI291884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68927[] = {SGI0, SGI291885, SGI0, SGI291886, SGI291887, SGI291887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291888, SGI291889, SGI0, SGI291890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68928[] = {SGI0, SGI291891, SGI0, SGI291892, SGI291893, SGI291893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291894, SGI291895, SGI0, SGI291896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68929[] = {SGI0, SGI291897, SGI0, SGI291898, SGI291899, SGI291899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291900, SGI291901, SGI0, SGI291902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68930[] = {SGI0, SGI291903, SGI0, SGI291904, SGI291905, SGI291906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68931[] = {SGI0, SGI291908, SGI0, SGI291909, SGI291910, SGI291911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291912, SGI291913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68932[] = {SGI0, SGI291914, SGI0, SGI291915, SGI291916, SGI291917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68933[] = {SGI0, SGI291919, SGI0, SGI291920, SGI291921, SGI291921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68934[] = {SGI0, SGI291923, SGI291923, SGI291924, SGI291925, SGI291925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68935[] = {SGI0, SGI291927, SGI0, SGI291928, SGI291929, SGI291929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291930, SGI291931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68936[] = {SGI0, SGI291932, SGI0, SGI291933, SGI291934, SGI291934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291935, SGI291936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68937[] = {SGI0, SGI291937, SGI0, SGI291938, SGI291939, SGI291939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291940, SGI291941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68938[] = {SGI0, SGI291942, SGI0, SGI291943, SGI291944, SGI291944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291945, SGI291946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68939[] = {SGI0, SGI291947, SGI0, SGI291948, SGI291949, SGI291949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291950, SGI291951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291952, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68940[] = {SGI0, SGI291953, SGI0, SGI291954, SGI291955, SGI291955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291957, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68941[] = {SGI0, SGI291958, SGI0, SGI291959, SGI0, SGI291960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291961, SGI291962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68942[] = {SGI0, SGI291963, SGI0, SGI291964, SGI291965, SGI291965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291966, SGI291967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68943[] = {SGI0, SGI291968, SGI0, SGI291969, SGI291970, SGI291971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291972, SGI291973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68944[] = {SGI0, SGI291974, SGI0, SGI291975, SGI291976, SGI291976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291977, SGI291978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68945[] = {SGI0, SGI0, SGI0, SGI291979, SGI291980, SGI291981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291982, SGI291983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291984, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68946[] = {SGI0, SGI291985, SGI0, SGI291986, SGI291987, SGI291987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291988, SGI291989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68947[] = {SGI0, SGI291991, SGI0, SGI291992, SGI291993, SGI291994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI291995, SGI291996, SGI0, SGI291997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68948[] = {SGI0, SGI291998, SGI0, SGI291999, SGI292000, SGI292001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68949[] = {SGI0, SGI292003, SGI0, SGI292004, SGI292005, SGI292006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68950[] = {SGI0, SGI292008, SGI0, SGI292009, SGI292010, SGI292010, SGI292011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292012, SGI0, SGI292013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68951[] = {SGI0, SGI292014, SGI0, SGI292015, SGI292016, SGI292016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292017, SGI292018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68952[] = {SGI0, SGI292019, SGI0, SGI292020, SGI292021, SGI292021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68953[] = {SGI0, SGI292024, SGI0, SGI292025, SGI292026, SGI292026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292027, SGI292028, SGI0, SGI292029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68954[] = {SGI0, SGI292030, SGI0, SGI292031, SGI292032, SGI292032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292033, SGI292034, SGI0, SGI292035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68955[] = {SGI0, SGI292036, SGI0, SGI292037, SGI292038, SGI292038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292039, SGI292040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68956[] = {SGI0, SGI292041, SGI0, SGI292042, SGI292043, SGI292044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68957[] = {SGI0, SGI0, SGI0, SGI292046, SGI292047, SGI292048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292049, SGI0, SGI0, NULL};
const Segment * const SG68958[] = {SGI0, SGI292050, SGI0, SGI292051, SGI292052, SGI292052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292053, SGI292054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292055, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68959[] = {SGI0, SGI292056, SGI0, SGI292057, SGI292058, SGI292058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292059, SGI292060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68960[] = {SGI0, SGI292061, SGI0, SGI292062, SGI292063, SGI292063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292064, SGI292065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68961[] = {SGI0, SGI292066, SGI0, SGI292067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292068, SGI292069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68962[] = {SGI0, SGI292070, SGI0, SGI292071, SGI292072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292073, SGI292074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68963[] = {SGI0, SGI292076, SGI0, SGI292077, SGI292078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292079, SGI292080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68964[] = {SGI0, SGI292082, SGI0, SGI292083, SGI292084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292085, SGI292086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68965[] = {SGI0, SGI292088, SGI0, SGI292089, SGI292090, SGI292090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292091, SGI292092, SGI0, SGI292093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68966[] = {SGI0, SGI292094, SGI0, SGI292095, SGI292096, SGI292096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292097, SGI292098, SGI0, SGI292099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68967[] = {SGI0, SGI292100, SGI0, SGI292101, SGI292102, SGI292102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292103, SGI292104, SGI0, SGI292105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68968[] = {SGI0, SGI292106, SGI0, SGI292107, SGI292108, SGI292108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292109, SGI292110, SGI0, SGI292111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68969[] = {SGI0, SGI292112, SGI0, SGI292113, SGI292114, SGI292114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292115, SGI292116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68970[] = {SGI0, SGI292117, SGI0, SGI292118, SGI292119, SGI292119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292120, SGI292121, SGI0, SGI292122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68971[] = {SGI0, SGI292123, SGI0, SGI292124, SGI292125, SGI292125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292126, SGI292127, SGI0, SGI292128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68972[] = {SGI0, SGI292129, SGI0, SGI292130, SGI292131, SGI292131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292133, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68973[] = {SGI0, SGI292134, SGI0, SGI292135, SGI292136, SGI292137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68974[] = {SGI0, SGI292139, SGI0, SGI292140, SGI292141, SGI292142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68975[] = {SGI0, SGI292144, SGI0, SGI292145, SGI292146, SGI292146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292147, SGI292148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68976[] = {SGI0, SGI292149, SGI0, SGI292150, SGI292151, SGI292151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292152, SGI292153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292154, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68977[] = {SGI0, SGI292155, SGI0, SGI292156, SGI292157, SGI292157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68978[] = {SGI0, SGI292159, SGI292159, SGI292160, SGI292161, SGI292161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68979[] = {SGI0, SGI292163, SGI0, SGI292164, SGI292165, SGI292165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68980[] = {SGI0, SGI292167, SGI0, SGI292168, SGI292169, SGI292169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292170, SGI292171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68981[] = {SGI0, SGI292172, SGI0, SGI292173, SGI292174, SGI292175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292176, SGI292177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68982[] = {SGI0, SGI292178, SGI0, SGI292179, SGI292180, SGI292181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292182, SGI292183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68983[] = {SGI0, SGI292184, SGI0, SGI292185, SGI292186, SGI292186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292187, SGI292188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68984[] = {SGI0, SGI292189, SGI0, SGI292190, SGI292191, SGI292191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292192, SGI292193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68985[] = {SGI0, SGI292194, SGI0, SGI292195, SGI292196, SGI292196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292197, SGI292198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68986[] = {SGI292199, SGI292200, NULL};
const Segment * const SG68987[] = {SGI0, SGI292201, SGI0, SGI292202, SGI292203, SGI292203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292204, SGI292205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68988[] = {SGI0, SGI292207, SGI0, SGI292208, SGI292209, SGI292210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292211, SGI292212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68989[] = {SGI0, SGI292213, NULL};
const Segment * const SG68990[] = {SGI0, SGI292214, SGI0, SGI292215, SGI292216, SGI292216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292217, SGI292218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68991[] = {SGI0, SGI292219, SGI0, SGI292220, SGI292221, SGI292222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292223, SGI292224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68992[] = {SGI0, SGI292225, SGI0, SGI292226, SGI292227, SGI292227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68993[] = {SGI0, SGI292229, SGI292229, SGI292230, SGI292231, SGI292231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68994[] = {SGI0, SGI292233, SGI0, SGI292234, SGI292235, SGI292235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292236, SGI292237, SGI0, SGI0, SGI292238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68995[] = {SGI0, SGI292239, SGI0, SGI292240, SGI292241, SGI292241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292243, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68996[] = {SGI0, SGI292244, SGI0, SGI292245, SGI292246, SGI292247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68997[] = {SGI0, SGI292249, SGI0, SGI292250, SGI292251, SGI292251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292252, SGI292253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68998[] = {SGI0, SGI292254, SGI0, SGI292255, SGI292256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG68999[] = {SGI0, SGI292258, SGI0, SGI292259, SGI292260, SGI292261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69000[] = {SGI0, SGI292263, SGI0, SGI292264, SGI292265, SGI292266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69001[] = {SGI0, SGI292268, SGI0, SGI292269, SGI292270, SGI292271, SGI0, SGI0, SGI0, SGI0, SGI292272, SGI0, SGI292273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69002[] = {SGI0, SGI292274, SGI0, SGI292275, SGI292276, SGI292276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292277, SGI292278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69003[] = {SGI0, SGI292279, SGI0, SGI292280, SGI292281, SGI292282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69004[] = {SGI0, SGI292284, SGI0, SGI292285, SGI292286, SGI292287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69005[] = {SGI0, SGI292289, SGI0, SGI292290, SGI292291, SGI292291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292292, SGI292293, SGI0, SGI292294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69006[] = {SGI0, SGI292295, SGI0, SGI292296, SGI292297, SGI292297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292298, SGI292299, SGI0, SGI292300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69007[] = {SGI0, SGI292301, SGI0, SGI292302, SGI292303, SGI292303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292304, SGI292305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69008[] = {SGI0, SGI292306, SGI0, SGI292307, SGI292308, SGI292308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292309, SGI292310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69009[] = {SGI0, SGI292311, NULL};
const Segment * const SG69010[] = {SGI0, SGI292312, SGI0, SGI292313, SGI292314, SGI292314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292315, SGI292316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69011[] = {SGI0, SGI292317, SGI0, SGI292318, SGI292319, SGI292319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292320, SGI292321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69012[] = {SGI0, SGI292322, SGI0, SGI292323, SGI292324, SGI292324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292325, SGI292326, SGI0, SGI292327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69013[] = {SGI0, SGI292328, SGI0, SGI292329, SGI292330, SGI292330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292331, SGI292332, SGI0, SGI292333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69014[] = {SGI0, SGI292334, SGI0, SGI292335, SGI292336, SGI292336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292337, SGI292338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69015[] = {SGI0, SGI292339, SGI0, SGI292340, SGI292341, SGI292342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292343, SGI292344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69016[] = {SGI0, SGI0, SGI0, SGI292345, SGI292346, SGI292347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292349, SGI0, SGI0, NULL};
const Segment * const SG69017[] = {SGI0, SGI292350, SGI0, SGI292351, SGI292352, SGI292352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292354, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69018[] = {SGI0, SGI292355, SGI0, SGI292356, SGI292357, SGI292357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292358, SGI292359, SGI0, SGI0, SGI292360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69019[] = {SGI0, SGI292361, SGI0, SGI292362, SGI292363, SGI292364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292365, SGI292366, SGI0, SGI0, SGI0, SGI292367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69020[] = {SGI0, SGI292368, SGI0, SGI292369, SGI292370, SGI292371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292372, SGI292373, SGI0, SGI0, SGI0, SGI292374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69021[] = {SGI0, SGI292375, SGI0, SGI292376, SGI292377, SGI292378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292379, SGI292380, SGI0, SGI0, SGI0, SGI292381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69022[] = {SGI0, SGI292382, SGI0, SGI292383, SGI292384, SGI292384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292385, SGI292386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69023[] = {SGI0, SGI292387, SGI0, SGI292388, SGI292389, SGI292390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69024[] = {SGI0, SGI292392, SGI0, SGI292393, SGI292394, SGI292394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292395, SGI292396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69025[] = {SGI0, SGI292397, SGI0, SGI292398, SGI292399, SGI292399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292400, SGI292401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69026[] = {SGI0, SGI292402, SGI0, SGI292403, SGI292404, SGI292405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69027[] = {SGI0, SGI292407, SGI0, SGI292408, SGI292409, SGI292409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292410, SGI292411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69028[] = {SGI0, SGI292413, SGI0, SGI292414, SGI292415, SGI292415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292416, SGI292417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69029[] = {SGI0, SGI292419, SGI0, SGI292420, SGI292421, SGI292421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292422, SGI292423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69030[] = {SGI0, SGI292425, SGI0, SGI292426, SGI292427, SGI292427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292428, SGI292429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69031[] = {SGI0, SGI292431, SGI0, SGI292432, SGI292433, SGI292433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292434, SGI292435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69032[] = {SGI0, SGI292437, SGI0, SGI292438, SGI292439, SGI292439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292440, SGI292441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69033[] = {SGI0, SGI292443, SGI0, SGI292444, SGI292445, SGI292445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292446, SGI292447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69034[] = {SGI0, SGI292449, SGI0, SGI292450, SGI292451, SGI292451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292452, SGI292453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69035[] = {SGI0, SGI292455, SGI0, SGI292456, SGI292457, SGI292457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292458, SGI292459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69036[] = {SGI0, SGI292461, SGI0, SGI292462, SGI292463, SGI292463, SGI292464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292465, SGI0, SGI292466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69037[] = {SGI0, SGI292467, SGI0, SGI292468, SGI292469, SGI292469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292470, SGI292471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69038[] = {SGI0, SGI292472, SGI0, SGI292473, SGI292474, SGI292474, SGI292475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292476, SGI0, SGI292477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69039[] = {SGI0, SGI292478, SGI0, SGI292479, SGI292480, SGI292480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292481, SGI292482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69040[] = {SGI0, SGI292483, SGI0, SGI292484, SGI292485, SGI292485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292486, SGI292487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292488, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69041[] = {SGI0, SGI292489, SGI0, SGI292490, SGI292491, SGI292491, SGI0, SGI292492, SGI0, SGI0, SGI0, SGI0, SGI292493, SGI292494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69042[] = {SGI0, SGI292495, SGI0, SGI292496, SGI292497, SGI292497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292498, SGI292499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69043[] = {SGI0, SGI292500, SGI0, SGI292501, SGI292502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292503, SGI292504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69044[] = {SGI0, SGI292505, SGI0, SGI292506, SGI292507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292508, SGI292509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69045[] = {SGI0, SGI292510, SGI0, SGI292511, SGI292512, SGI292512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292513, SGI292514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292515, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69046[] = {SGI0, SGI292516, SGI0, SGI292517, SGI292518, SGI292518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292519, SGI292520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292521, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69047[] = {SGI0, SGI292522, SGI0, SGI292523, SGI292524, SGI292524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292525, SGI292526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69048[] = {SGI0, SGI292527, SGI0, SGI292528, SGI292529, SGI292530, SGI0, SGI0, SGI0, SGI0, SGI292531, SGI0, SGI292532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69049[] = {SGI292533, SGI292534, SGI0, SGI292535, SGI292536, SGI292537, SGI0, SGI0, SGI0, SGI0, SGI292538, SGI0, SGI292539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69050[] = {SGI0, SGI292540, SGI0, SGI292541, SGI292542, SGI292542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292543, SGI292544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292545, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69051[] = {SGI0, SGI292546, SGI0, SGI292547, SGI292548, SGI292548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292549, SGI292550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69052[] = {SGI0, SGI292551, SGI0, SGI292552, SGI292553, SGI292553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292554, SGI292555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69053[] = {SGI0, SGI292556, SGI0, SGI292557, SGI292558, SGI292558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292559, SGI292560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69054[] = {SGI0, SGI292561, SGI0, SGI292562, SGI292563, SGI292564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292565, SGI292566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69055[] = {SGI0, SGI292567, SGI0, SGI292568, SGI292569, SGI292569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292570, SGI292571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292572, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69056[] = {SGI0, SGI292573, SGI0, SGI292574, SGI292575, SGI292575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292576, SGI292577, SGI0, SGI292578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69057[] = {SGI0, SGI292579, SGI0, SGI292580, SGI292581, SGI292581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292582, SGI292583, SGI0, SGI292584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69058[] = {SGI0, SGI292585, SGI0, SGI292586, SGI292587, SGI292587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292588, SGI292589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292590, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69059[] = {SGI0, SGI292591, SGI0, SGI292592, SGI292593, SGI292593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292594, SGI292595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292596, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69060[] = {SGI0, SGI292597, SGI0, SGI292598, SGI292599, SGI292599, SGI0, SGI292600, SGI0, SGI0, SGI0, SGI0, SGI292601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69061[] = {SGI0, SGI292602, SGI0, SGI292603, SGI292604, SGI292605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292606, SGI292607, SGI0, SGI0, SGI0, SGI0, SGI292608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69062[] = {SGI0, SGI292609, SGI0, SGI292610, SGI292611, SGI292612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292613, SGI292614, SGI0, SGI0, SGI0, SGI0, SGI292615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69063[] = {SGI0, SGI292616, SGI0, SGI292617, SGI292618, SGI292618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292619, SGI292620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292621, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69064[] = {SGI292622, SGI292623, NULL};
const Segment * const SG69065[] = {SGI0, SGI292624, SGI0, SGI292625, SGI292626, SGI292627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292629, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69066[] = {SGI0, SGI292630, SGI0, SGI292631, SGI292632, SGI292633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69067[] = {SGI0, SGI292635, SGI0, SGI292636, SGI292637, SGI292638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69068[] = {SGI0, SGI292640, SGI0, SGI292641, SGI292642, SGI292643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69069[] = {SGI0, SGI292645, SGI0, SGI292646, SGI292647, SGI292647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292648, SGI292649, SGI0, SGI292650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69070[] = {SGI0, SGI292651, SGI0, SGI292652, SGI292653, SGI292653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292654, SGI292655, SGI0, SGI292656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69071[] = {SGI0, SGI292657, SGI0, SGI292658, SGI292659, SGI292659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292660, SGI292661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69072[] = {SGI0, SGI292662, SGI0, SGI292663, SGI292664, SGI292664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292665, SGI292666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69073[] = {SGI0, SGI292667, SGI0, SGI292668, SGI292669, SGI292670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69074[] = {SGI0, SGI292672, SGI0, SGI292673, SGI292674, SGI292674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292675, SGI292676, SGI0, SGI292677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69075[] = {SGI0, SGI292678, SGI0, SGI292679, SGI292680, SGI292680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292681, SGI292682, SGI0, SGI292683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69076[] = {SGI0, SGI292684, SGI0, SGI292685, SGI292686, SGI292686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292687, SGI292688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292689, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69077[] = {SGI0, SGI292690, SGI0, SGI292691, SGI292692, SGI292692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292693, SGI292694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292695, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69078[] = {SGI0, SGI292696, SGI0, SGI292697, SGI292698, SGI292699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292700, SGI292701, SGI0, SGI0, SGI0, SGI292702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69079[] = {SGI0, SGI292703, SGI0, SGI292704, SGI292705, SGI292706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292707, SGI292708, SGI0, SGI0, SGI0, SGI292709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69080[] = {SGI0, SGI292710, SGI0, SGI292711, SGI292712, SGI292713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69081[] = {SGI0, SGI292716, SGI0, SGI292717, SGI292718, SGI292719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69082[] = {SGI0, SGI292721, SGI0, SGI292722, SGI292723, SGI292724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292725, SGI292726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69083[] = {SGI0, SGI292727, SGI0, SGI292728, SGI292729, SGI292729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292730, SGI292731, SGI0, SGI292732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69084[] = {SGI0, SGI292733, SGI0, SGI292734, SGI292735, SGI292735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292736, SGI292737, SGI0, SGI292738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69085[] = {SGI0, SGI292739, SGI0, SGI292740, SGI292741, SGI292742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292744, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69086[] = {SGI0, SGI292745, SGI0, SGI292746, SGI292747, SGI292748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292749, SGI292750, SGI0, SGI0, SGI0, SGI0, SGI292751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69087[] = {SGI0, SGI292752, SGI0, SGI292753, SGI292754, SGI292755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292756, SGI292757, SGI0, SGI0, SGI0, SGI0, SGI292758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69088[] = {SGI0, SGI292759, SGI0, SGI292760, SGI292761, SGI292761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292763, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69089[] = {SGI0, SGI292764, SGI0, SGI292765, SGI292766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292767, SGI292768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69090[] = {SGI0, SGI292769, SGI0, SGI292770, SGI292771, SGI292772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292773, SGI292774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69091[] = {SGI0, SGI292776, SGI0, SGI292777, SGI292778, SGI292779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292780, SGI292781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69092[] = {SGI0, SGI292783, SGI0, SGI292784, SGI292785, SGI292786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292787, SGI292788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69093[] = {SGI0, SGI292789, SGI0, SGI292790, SGI292791, SGI292791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292792, SGI292793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69094[] = {SGI0, SGI292795, SGI0, SGI292796, SGI292797, SGI292797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292798, SGI292799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69095[] = {SGI0, SGI292801, SGI0, SGI292802, SGI292803, SGI292803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292804, SGI292805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69096[] = {SGI0, SGI292807, SGI0, SGI292808, SGI292809, SGI292809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292810, SGI292811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69097[] = {SGI0, SGI292813, SGI0, SGI292814, SGI292815, SGI292815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292816, SGI292817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69098[] = {SGI0, SGI292819, SGI0, SGI292820, SGI292821, SGI292821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292822, SGI292823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69099[] = {SGI0, SGI292825, SGI0, SGI292826, SGI292827, SGI292827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292828, SGI292829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69100[] = {SGI0, SGI292831, SGI0, SGI292832, SGI292833, SGI292833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292834, SGI292835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69101[] = {SGI0, SGI292837, SGI0, SGI292838, SGI292839, SGI292839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292840, SGI292841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69102[] = {SGI0, SGI292843, SGI0, SGI292844, SGI292845, SGI292845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292846, SGI292847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69103[] = {SGI0, SGI292849, SGI0, SGI292850, SGI292851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292852, SGI292853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69104[] = {SGI0, SGI292854, SGI0, SGI292855, SGI292856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69105[] = {SGI0, SGI292858, SGI0, SGI292859, SGI292860, SGI292861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69106[] = {SGI0, SGI292863, SGI0, SGI292864, SGI292865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292866, SGI292867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69107[] = {SGI0, SGI292868, SGI0, SGI292869, SGI292870, SGI292870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292872, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69108[] = {SGI0, SGI292873, SGI0, SGI292874, SGI292875, SGI292876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69109[] = {SGI0, SGI292878, SGI0, SGI292879, SGI292880, SGI292881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69110[] = {SGI0, SGI292883, SGI0, SGI292884, SGI292885, SGI292885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292886, SGI292887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292888, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69111[] = {SGI0, SGI292889, SGI0, SGI292890, SGI292891, SGI292891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292892, SGI292893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292894, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69112[] = {SGI0, SGI292895, SGI0, SGI292896, SGI292897, SGI292898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292900, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69113[] = {SGI0, SGI292901, SGI0, SGI292902, SGI292903, SGI292903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292904, SGI292905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69114[] = {SGI0, SGI292907, SGI0, SGI292908, SGI292909, SGI292910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292912, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69115[] = {SGI292913, SGI292914, SGI292915, SGI292916, SGI292917, SGI292914, SGI0, NULL};
const Segment * const SG69116[] = {SGI0, SGI292918, SGI0, SGI292919, SGI292920, SGI292920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292921, SGI292922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292923, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69117[] = {SGI0, SGI292924, SGI0, SGI292925, SGI292926, SGI292927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69118[] = {SGI0, SGI292929, SGI0, SGI292930, SGI292931, SGI292931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292932, SGI292933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292934, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69119[] = {SGI0, SGI292935, SGI0, SGI292936, SGI292937, SGI292937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292938, SGI292939, SGI0, SGI292940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69120[] = {SGI0, SGI292941, SGI0, SGI292942, SGI292943, SGI292943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292944, SGI292945, SGI0, SGI292946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69121[] = {SGI0, SGI292947, SGI0, SGI292948, SGI292949, SGI292950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69122[] = {SGI0, SGI292953, SGI0, SGI292954, SGI292955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69123[] = {SGI0, SGI292958, SGI0, SGI292959, SGI292960, SGI292960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292961, SGI292962, SGI0, SGI292963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69124[] = {SGI0, SGI292964, SGI0, SGI292965, SGI292966, SGI292966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292967, SGI292968, SGI0, SGI292969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69125[] = {SGI0, SGI292970, SGI0, SGI292971, SGI292972, SGI292973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292975, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69126[] = {SGI0, SGI292976, SGI0, SGI292977, SGI292978, SGI292978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292979, SGI292980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69127[] = {SGI0, SGI292982, SGI0, SGI292983, SGI292984, SGI292984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292985, SGI292986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69128[] = {SGI0, SGI292987, SGI0, SGI292988, SGI292989, SGI292989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292990, SGI292991, SGI0, SGI292992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69129[] = {SGI0, SGI292993, SGI0, SGI292994, SGI292995, SGI292995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI292996, SGI292997, SGI0, SGI292998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69130[] = {SGI0, SGI292999, SGI0, SGI293000, SGI293001, SGI293001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293002, SGI293003, SGI0, SGI293004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69131[] = {SGI0, SGI293005, SGI0, SGI293006, SGI293007, SGI293008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69132[] = {SGI0, SGI293010, SGI0, SGI293011, SGI293012, SGI293013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293014, SGI293015, SGI0, SGI0, SGI0, SGI293016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69133[] = {SGI0, SGI293017, SGI0, SGI293018, SGI293019, SGI293019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293020, SGI293021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69134[] = {SGI0, SGI293022, SGI0, SGI293023, SGI293024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293025, SGI293026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69135[] = {SGI0, SGI293027, SGI0, SGI293028, SGI293029, SGI293029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293030, SGI293031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293032, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69136[] = {SGI0, SGI293033, SGI0, SGI293034, SGI293035, SGI293036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69137[] = {SGI0, SGI293038, SGI0, SGI293039, SGI293040, SGI293041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69138[] = {SGI0, SGI293043, SGI0, SGI293044, SGI293045, SGI293046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293047, SGI293048, SGI0, SGI293049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69139[] = {SGI0, SGI293050, SGI0, SGI293051, SGI293052, SGI293052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69140[] = {SGI0, SGI293054, SGI0, SGI293055, SGI293056, SGI293057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293058, SGI293059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69141[] = {SGI0, SGI293060, SGI0, SGI293061, SGI293062, SGI293062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293063, SGI293064, SGI0, SGI0, SGI293065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69142[] = {SGI0, SGI293066, SGI0, SGI293067, SGI293068, SGI293068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293069, SGI293070, SGI0, SGI0, SGI293071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69143[] = {SGI0, SGI293072, SGI0, SGI293073, SGI293074, SGI293074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293075, SGI293076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69144[] = {SGI0, SGI293078, SGI0, SGI293079, SGI293080, SGI293080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293081, SGI293082, SGI0, SGI293083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69145[] = {SGI0, SGI293084, SGI0, SGI293085, SGI293086, SGI293086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293087, SGI293088, SGI0, SGI293089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69146[] = {SGI0, SGI293090, SGI0, SGI293091, SGI293092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293093, SGI293094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69147[] = {SGI0, SGI293095, SGI0, SGI293096, SGI293097, SGI293098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69148[] = {SGI0, SGI293100, SGI0, SGI293101, SGI293102, SGI293103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69149[] = {SGI0, SGI293105, SGI0, SGI293106, SGI293107, SGI293108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293110, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69150[] = {SGI0, SGI293111, SGI0, SGI293112, SGI293113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293114, SGI293115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69151[] = {SGI0, SGI293116, SGI0, SGI293117, SGI293118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293119, SGI293120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69152[] = {SGI0, SGI293121, SGI0, SGI293122, SGI293123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293124, SGI293125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69153[] = {SGI0, SGI293126, SGI0, SGI293127, SGI293128, SGI293129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69154[] = {SGI0, SGI293131, SGI0, SGI293132, SGI293133, SGI293134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69155[] = {SGI0, SGI293136, SGI0, SGI293137, SGI293138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293139, SGI293140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69156[] = {SGI0, SGI293141, SGI0, SGI293142, SGI293143, SGI293144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69157[] = {SGI0, SGI293146, SGI0, SGI293147, SGI293148, SGI293149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293150, SGI293151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69158[] = {SGI0, SGI293152, SGI0, SGI293153, SGI293154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293155, SGI293156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69159[] = {SGI0, SGI293157, SGI0, SGI293158, SGI293159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293160, SGI293161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69160[] = {SGI0, SGI293162, SGI0, SGI293163, SGI293164, SGI293165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293166, SGI293167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69161[] = {SGI0, SGI293168, SGI0, SGI293169, SGI293170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293171, SGI293172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69162[] = {SGI0, SGI293173, SGI0, SGI293174, SGI293175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293176, SGI293177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69163[] = {SGI293178, SGI293179, SGI293180, SGI293181, SGI0, SGI293179, SGI0, NULL};
const Segment * const SG69164[] = {SGI0, SGI293182, SGI0, SGI293183, SGI293184, SGI293185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293187, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69165[] = {SGI0, SGI293188, SGI0, SGI293189, SGI293190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293191, SGI293192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69166[] = {SGI0, SGI293193, SGI0, SGI293194, SGI293195, SGI293196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293198, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69167[] = {SGI0, SGI293199, SGI0, SGI293200, SGI293201, SGI293201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293202, SGI293203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69168[] = {SGI0, SGI293205, SGI0, SGI293206, SGI293207, SGI293208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69169[] = {SGI0, SGI293210, SGI0, SGI293211, SGI293212, SGI293213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69170[] = {SGI0, SGI293215, SGI0, SGI293216, SGI293217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293218, SGI293219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69171[] = {SGI293220, SGI293221, NULL};
const Segment * const SG69172[] = {SGI293222, SGI293223, NULL};
const Segment * const SG69173[] = {SGI0, SGI293224, NULL};
const Segment * const SG69174[] = {SGI0, SGI293225, NULL};
const Segment * const SG69175[] = {SGI293226, SGI293227, NULL};
const Segment * const SG69176[] = {SGI293228, SGI293229, NULL};
const Segment * const SG69177[] = {SGI293230, SGI293231, NULL};
const Segment * const SG69178[] = {SGI0, SGI293232, NULL};
const Segment * const SG69179[] = {SGI0, SGI293233, NULL};
const Segment * const SG69180[] = {SGI0, SGI293234, NULL};
const Segment * const SG69181[] = {SGI0, SGI293235, NULL};
const Segment * const SG69182[] = {SGI0, SGI293236, NULL};
const Segment * const SG69183[] = {SGI0, SGI293237, NULL};
const Segment * const SG69184[] = {SGI0, SGI293238, NULL};
const Segment * const SG69185[] = {SGI0, SGI293239, NULL};
const Segment * const SG69186[] = {SGI293240, SGI293241, NULL};
const Segment * const SG69187[] = {SGI0, SGI293242, NULL};
const Segment * const SG69188[] = {SGI0, SGI293243, NULL};
const Segment * const SG69189[] = {SGI0, SGI293244, NULL};
const Segment * const SG69190[] = {SGI293245, SGI293246, NULL};
const Segment * const SG69191[] = {SGI293247, SGI293248, NULL};
const Segment * const SG69192[] = {SGI0, SGI293249, SGI0, SGI293250, SGI293251, SGI293252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293253, SGI293254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69193[] = {SGI0, SGI293256, SGI0, SGI293257, SGI293258, SGI293258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293259, SGI293260, SGI0, SGI293261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69194[] = {SGI0, SGI293262, SGI0, SGI293263, SGI293264, SGI293264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293265, SGI293266, SGI0, SGI293267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69195[] = {SGI0, SGI293268, SGI0, SGI293269, SGI293270, SGI293271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69196[] = {SGI0, SGI293273, SGI0, SGI293274, SGI293275, SGI293276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69197[] = {SGI0, SGI293278, SGI0, SGI293279, SGI293280, SGI293280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293281, SGI293282, SGI0, SGI293283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69198[] = {SGI0, SGI293284, SGI0, SGI293285, SGI293286, SGI293286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293287, SGI293288, SGI0, SGI293289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69199[] = {SGI0, SGI293290, SGI0, SGI293291, SGI293292, SGI293292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293293, SGI293294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69200[] = {SGI0, SGI293296, SGI0, SGI293297, SGI293298, SGI293298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293299, SGI293300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69201[] = {SGI0, SGI293302, SGI0, SGI293303, SGI293304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293305, SGI293306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69202[] = {SGI0, SGI293307, SGI0, SGI293308, SGI293309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293310, SGI293311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69203[] = {SGI0, SGI293312, SGI0, SGI293313, SGI293314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293315, SGI293316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69204[] = {SGI0, SGI293317, SGI0, SGI293318, SGI293319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293320, SGI293321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69205[] = {SGI0, SGI293322, SGI0, SGI293323, SGI293324, SGI293324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293325, SGI293326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69206[] = {SGI0, SGI293328, SGI0, SGI293329, SGI293330, SGI293331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69207[] = {SGI0, SGI293333, SGI0, SGI293334, SGI293335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69208[] = {SGI0, SGI293338, SGI0, SGI293339, SGI293340, SGI293341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69209[] = {SGI0, SGI293344, SGI0, SGI293345, SGI293346, SGI293347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293349, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69210[] = {SGI0, SGI293350, SGI0, SGI293351, SGI293352, SGI293352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293353, SGI293354, SGI0, SGI0, SGI293355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69211[] = {SGI0, SGI293356, SGI0, SGI293357, SGI293358, SGI293358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293359, SGI293360, SGI0, SGI0, SGI293361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69212[] = {SGI0, SGI293362, SGI0, SGI293363, SGI293364, SGI293365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69213[] = {SGI0, SGI293367, SGI0, SGI293368, SGI293369, SGI293370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293371, SGI293372, SGI0, SGI0, SGI0, SGI293373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69214[] = {SGI0, SGI293374, SGI0, SGI293375, SGI293376, SGI293377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293378, SGI293379, SGI0, SGI0, SGI0, SGI293380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69215[] = {SGI0, SGI293381, SGI0, SGI293382, SGI293383, SGI293384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293385, SGI293386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69216[] = {SGI0, SGI293387, SGI0, SGI293388, SGI293389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293390, SGI293391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69217[] = {SGI0, SGI293392, SGI0, SGI293393, SGI293394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293395, SGI293396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69218[] = {SGI0, SGI293397, SGI0, SGI293398, SGI293399, SGI293400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293401, SGI293402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69219[] = {SGI0, SGI0, SGI0, SGI293404, SGI293405, SGI293406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293407, SGI293408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69220[] = {SGI0, SGI293410, SGI0, SGI293411, SGI293412, SGI293412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293413, SGI293414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69221[] = {SGI0, SGI293416, SGI0, SGI293417, SGI293418, SGI293419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69222[] = {SGI0, SGI293421, SGI0, SGI293422, SGI293423, SGI293424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293426, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69223[] = {SGI0, SGI293427, SGI0, SGI293428, SGI293429, SGI293429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69224[] = {SGI293432, SGI0, SGI0, SGI293433, SGI293434, SGI293435, SGI0, NULL};
const Segment * const SG69225[] = {SGI0, SGI293436, SGI0, SGI293437, SGI293438, SGI293439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293440, SGI293441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69226[] = {SGI0, SGI293442, SGI0, SGI293443, SGI293444, SGI293445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69227[] = {SGI0, SGI293447, SGI0, SGI293448, SGI293449, SGI293450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69228[] = {SGI0, SGI293452, SGI0, SGI293453, SGI293454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293455, SGI293456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69229[] = {SGI0, SGI293457, SGI0, SGI293458, SGI293459, SGI293460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69230[] = {SGI0, SGI293462, SGI0, SGI293463, SGI293464, SGI293464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293465, SGI293466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69231[] = {SGI0, SGI293468, NULL};
const Segment * const SG69232[] = {SGI0, SGI293469, NULL};
const Segment * const SG69233[] = {SGI0, SGI293470, NULL};
const Segment * const SG69234[] = {SGI0, SGI293471, NULL};
const Segment * const SG69235[] = {SGI0, SGI293472, NULL};
const Segment * const SG69236[] = {SGI0, SGI293473, NULL};
const Segment * const SG69237[] = {SGI0, SGI293474, NULL};
const Segment * const SG69238[] = {SGI0, SGI293475, NULL};
const Segment * const SG69239[] = {SGI0, SGI293476, NULL};
const Segment * const SG69240[] = {SGI0, SGI293477, NULL};
const Segment * const SG69241[] = {SGI0, SGI293478, NULL};
const Segment * const SG69242[] = {SGI0, SGI293479, NULL};
const Segment * const SG69243[] = {SGI0, SGI293480, NULL};
const Segment * const SG69244[] = {SGI0, SGI293481, NULL};
const Segment * const SG69245[] = {SGI0, SGI293482, SGI0, SGI293483, SGI293484, SGI293484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293485, SGI293486, SGI0, SGI293487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69246[] = {SGI0, SGI293488, SGI0, SGI293489, SGI293490, SGI293490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293491, SGI293492, SGI0, SGI293493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69247[] = {SGI0, SGI293494, SGI0, SGI293495, SGI293496, SGI293496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293497, SGI293498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293499, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69248[] = {SGI0, SGI293500, SGI0, SGI293501, SGI293502, SGI293503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293505, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69249[] = {SGI0, SGI293506, SGI0, SGI293507, SGI293508, SGI293509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293510, SGI293511, SGI0, SGI0, SGI0, SGI0, SGI293512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69250[] = {SGI0, SGI293513, SGI0, SGI293514, SGI293515, SGI293515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69251[] = {SGI0, SGI293517, SGI0, SGI293518, SGI293519, SGI293520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69252[] = {SGI0, SGI293522, SGI0, SGI293523, SGI293524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293525, SGI293526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69253[] = {SGI0, SGI293527, SGI0, SGI293528, SGI293529, SGI293530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293532, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69254[] = {SGI0, SGI293533, SGI0, SGI293534, SGI293535, SGI293536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293538, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69255[] = {SGI0, SGI293539, SGI0, SGI293540, SGI293541, SGI293542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69256[] = {SGI0, SGI293544, SGI0, SGI293545, SGI293546, SGI293546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293548, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69257[] = {SGI0, SGI293549, SGI0, SGI293550, SGI293551, SGI293551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293552, SGI293553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69258[] = {SGI0, SGI293554, SGI0, SGI293555, SGI293556, SGI293557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69259[] = {SGI0, SGI293559, SGI0, SGI293560, SGI293561, SGI293562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293564, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69260[] = {SGI0, SGI293565, SGI0, SGI293566, SGI293567, SGI293568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293570, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69261[] = {SGI293571, SGI293572, SGI293573, SGI293574, SGI0, SGI293575, SGI0, NULL};
const Segment * const SG69262[] = {SGI0, SGI293576, SGI0, SGI293577, SGI293578, SGI293579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69263[] = {SGI0, SGI293581, SGI0, SGI293582, SGI293583, SGI293584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69264[] = {SGI0, SGI293586, SGI0, SGI293587, SGI293588, SGI293589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293590, SGI293591, SGI0, SGI0, SGI0, SGI0, SGI293592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69265[] = {SGI0, SGI293593, SGI0, SGI293594, SGI293595, SGI293596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69266[] = {SGI0, SGI293599, SGI0, SGI293600, SGI293601, SGI293602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69267[] = {SGI0, SGI293605, SGI0, SGI293606, SGI293607, SGI293608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69268[] = {SGI0, SGI293610, SGI0, SGI293611, SGI293612, SGI293612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293613, SGI293614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293615, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69269[] = {SGI0, SGI293616, SGI0, SGI293617, SGI293618, SGI293619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69270[] = {SGI0, SGI293621, SGI0, SGI293622, SGI293623, SGI293623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293624, SGI293625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69271[] = {SGI0, SGI293626, SGI0, SGI293627, SGI293628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293629, SGI293630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69272[] = {SGI0, SGI293631, SGI0, SGI293632, SGI293633, SGI293634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293635, SGI293636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69273[] = {SGI0, SGI293638, SGI0, SGI293639, SGI293640, SGI293641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69274[] = {SGI0, SGI293643, SGI0, SGI293644, SGI293645, SGI293646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293647, SGI293648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69275[] = {SGI0, SGI293649, SGI0, SGI293650, SGI293651, SGI293652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293653, SGI293654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69276[] = {SGI0, SGI293655, SGI0, SGI293656, SGI293657, SGI293658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293659, SGI293660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69277[] = {SGI0, SGI0, SGI0, SGI293661, SGI293662, SGI293663, SGI0, SGI0, SGI0, SGI0, SGI293664, SGI0, SGI293665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69278[] = {SGI0, SGI293666, SGI0, SGI293667, SGI293668, SGI293668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293669, SGI293670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69279[] = {SGI0, SGI293671, SGI0, SGI293672, SGI293673, SGI293674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293675, SGI293676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69280[] = {SGI0, SGI293677, SGI0, SGI293678, SGI293679, SGI293680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69281[] = {SGI0, SGI293682, SGI0, SGI293683, SGI293684, SGI293685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69282[] = {SGI0, SGI293688, NULL};
const Segment * const SG69283[] = {SGI293689, SGI293690, NULL};
const Segment * const SG69284[] = {SGI0, SGI293691, NULL};
const Segment * const SG69285[] = {SGI0, SGI293692, NULL};
const Segment * const SG69286[] = {SGI0, SGI293693, NULL};
const Segment * const SG69287[] = {SGI0, SGI293694, NULL};
const Segment * const SG69288[] = {SGI0, SGI293695, NULL};
const Segment * const SG69289[] = {SGI0, SGI293696, NULL};
const Segment * const SG69290[] = {SGI0, SGI293697, NULL};
const Segment * const SG69291[] = {SGI293698, SGI293699, NULL};
const Segment * const SG69292[] = {SGI0, SGI293700, NULL};
const Segment * const SG69293[] = {SGI0, SGI293701, NULL};
const Segment * const SG69294[] = {SGI0, SGI293702, NULL};
const Segment * const SG69295[] = {SGI0, SGI293703, NULL};
const Segment * const SG69296[] = {SGI293704, SGI293705, NULL};
const Segment * const SG69297[] = {SGI0, SGI293706, NULL};
const Segment * const SG69298[] = {SGI0, SGI293707, NULL};
const Segment * const SG69299[] = {SGI0, SGI293708, NULL};
const Segment * const SG69300[] = {SGI0, SGI293709, NULL};
const Segment * const SG69301[] = {SGI0, SGI293710, SGI0, SGI293711, SGI293712, SGI293713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69302[] = {SGI0, SGI293715, NULL};
const Segment * const SG69303[] = {SGI0, SGI293716, SGI0, SGI293717, SGI293718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293719, SGI293720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69304[] = {SGI0, SGI293721, SGI0, SGI293722, SGI293723, SGI293724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69305[] = {SGI0, SGI293726, SGI0, SGI293727, SGI293728, SGI293729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293731, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69306[] = {SGI0, SGI293732, SGI0, SGI293733, SGI293734, SGI293735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293736, SGI293737, SGI0, SGI0, SGI0, SGI0, SGI293738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69307[] = {SGI0, SGI293739, SGI0, SGI293740, SGI293741, SGI293742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69308[] = {SGI0, SGI293745, SGI0, SGI293746, SGI293747, SGI293748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69309[] = {SGI0, SGI293750, SGI0, SGI293751, SGI293752, SGI293753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69310[] = {SGI0, SGI293755, SGI0, SGI293756, SGI293757, SGI293757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293758, SGI293759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69311[] = {SGI0, SGI293761, SGI0, SGI293762, SGI293763, SGI293763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293764, SGI293765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69312[] = {SGI0, SGI293767, SGI0, SGI293768, SGI293769, SGI293769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293770, SGI293771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69313[] = {SGI0, SGI293773, SGI0, SGI293774, SGI293775, SGI293775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293776, SGI293777, SGI0, SGI293778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69314[] = {SGI0, SGI293779, SGI0, SGI293780, SGI293781, SGI293781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293782, SGI293783, SGI0, SGI293784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69315[] = {SGI0, SGI293785, SGI0, SGI293786, SGI293787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293788, SGI293789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69316[] = {SGI0, SGI293790, SGI0, SGI293791, SGI293792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293793, SGI293794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69317[] = {SGI0, SGI293795, SGI0, SGI293796, SGI293797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293798, SGI293799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69318[] = {SGI0, SGI293800, SGI0, SGI293801, SGI293802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293803, SGI293804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69319[] = {SGI0, SGI293805, SGI0, SGI293806, SGI293807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293808, SGI293809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69320[] = {SGI0, SGI293810, SGI0, SGI293811, SGI293812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293813, SGI293814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69321[] = {SGI0, SGI293815, SGI0, SGI293816, SGI293817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293818, SGI293819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69322[] = {SGI0, SGI293820, SGI0, SGI293821, SGI293822, SGI293822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293823, SGI293824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69323[] = {SGI0, SGI293825, SGI0, SGI293826, SGI293827, SGI293828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293829, SGI293830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69324[] = {SGI0, SGI293831, SGI0, SGI293832, SGI293833, SGI293834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293835, SGI293836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69325[] = {SGI0, SGI293838, SGI0, SGI293839, SGI293840, SGI293840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293841, SGI293842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69326[] = {SGI0, SGI293844, SGI0, SGI293845, SGI293846, SGI293847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293849, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69327[] = {SGI0, SGI293850, SGI0, SGI293851, SGI293852, SGI293853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69328[] = {SGI0, SGI293855, SGI0, SGI293856, SGI293857, SGI293858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69329[] = {SGI0, SGI293860, SGI0, SGI293861, SGI293862, SGI293863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69330[] = {SGI0, SGI293865, SGI0, SGI293866, SGI293867, SGI293867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293868, SGI293869, SGI0, SGI293870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69331[] = {SGI0, SGI293871, SGI0, SGI293872, SGI293873, SGI293873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293874, SGI293875, SGI0, SGI293876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69332[] = {SGI293877, SGI293878, SGI293879, SGI293880, SGI293881, SGI293878, SGI0, NULL};
const Segment * const SG69333[] = {SGI0, SGI293882, SGI0, SGI293883, SGI293884, SGI293884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293886, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69334[] = {SGI0, SGI293887, SGI0, SGI293888, SGI293889, SGI293889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293890, SGI293891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69335[] = {SGI0, SGI293893, SGI0, SGI293894, SGI293895, SGI293896, SGI0, SGI0, SGI0, SGI0, SGI293897, SGI0, SGI293898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69336[] = {SGI0, SGI293899, SGI0, SGI293900, SGI293901, SGI293902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293903, SGI293904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69337[] = {SGI0, SGI293905, SGI0, SGI293906, SGI293907, SGI293908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293909, SGI293910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69338[] = {SGI0, SGI293911, SGI0, SGI293912, SGI293913, SGI293914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293915, SGI293916, SGI0, SGI0, SGI0, SGI0, SGI293917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69339[] = {SGI0, SGI293918, SGI0, SGI293919, SGI293920, SGI293921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293922, SGI293923, SGI0, SGI0, SGI0, SGI0, SGI293924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69340[] = {SGI0, SGI293925, SGI0, SGI293926, SGI293927, SGI293928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69341[] = {SGI0, SGI293931, SGI0, SGI293932, SGI293933, SGI293933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293935, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69342[] = {SGI0, SGI293936, SGI0, SGI293937, SGI293938, SGI293939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69343[] = {SGI0, SGI293941, SGI0, SGI293942, SGI293943, SGI293944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69344[] = {SGI0, SGI293947, SGI0, SGI293948, SGI293949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69345[] = {SGI0, SGI293952, SGI0, SGI293953, SGI0, SGI293954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69346[] = {SGI0, SGI293956, SGI0, SGI293957, SGI293958, SGI293959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69347[] = {SGI0, SGI293961, SGI0, SGI293962, SGI293963, SGI293964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69348[] = {SGI0, SGI293966, SGI0, SGI293967, SGI293968, SGI293968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293970, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69349[] = {SGI0, SGI293971, SGI0, SGI293972, SGI293973, SGI293973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293974, SGI293975, SGI0, SGI293976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69350[] = {SGI0, SGI293977, SGI0, SGI293978, SGI293979, SGI293979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI293980, SGI293981, SGI0, SGI293982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69351[] = {SGI0, SGI293983, NULL};
const Segment * const SG69352[] = {SGI0, SGI293984, NULL};
const Segment * const SG69353[] = {SGI0, SGI293985, NULL};
const Segment * const SG69354[] = {SGI293986, SGI293987, NULL};
const Segment * const SG69355[] = {SGI0, SGI293988, NULL};
const Segment * const SG69356[] = {SGI0, SGI293989, NULL};
const Segment * const SG69357[] = {SGI0, SGI293990, NULL};
const Segment * const SG69358[] = {SGI0, SGI293991, NULL};
const Segment * const SG69359[] = {SGI0, SGI293992, NULL};
const Segment * const SG69360[] = {SGI0, SGI293993, NULL};
const Segment * const SG69361[] = {SGI0, SGI293994, NULL};
const Segment * const SG69362[] = {SGI0, SGI293995, NULL};
const Segment * const SG69363[] = {SGI0, SGI293996, NULL};
const Segment * const SG69364[] = {SGI0, SGI293997, NULL};
const Segment * const SG69365[] = {SGI0, SGI293998, NULL};
const Segment * const SG69366[] = {SGI0, SGI293999, NULL};
const Segment * const SG69367[] = {SGI0, SGI294000, NULL};
const Segment * const SG69368[] = {SGI0, SGI294001, NULL};
const Segment * const SG69369[] = {SGI294002, SGI294003, NULL};
const Segment * const SG69370[] = {SGI294004, SGI294005, NULL};
const Segment * const SG69371[] = {SGI294006, SGI294007, NULL};
const Segment * const SG69372[] = {SGI294008, SGI294009, NULL};
const Segment * const SG69373[] = {SGI0, SGI294010, NULL};
const Segment * const SG69374[] = {SGI0, SGI294011, NULL};
const Segment * const SG69375[] = {SGI0, SGI294012, NULL};
const Segment * const SG69376[] = {SGI0, SGI294013, NULL};
const Segment * const SG69377[] = {SGI294014, SGI294015, NULL};
const Segment * const SG69378[] = {SGI0, SGI294016, NULL};
const Segment * const SG69379[] = {SGI0, SGI294017, NULL};
const Segment * const SG69380[] = {SGI294018, SGI294019, NULL};
const Segment * const SG69381[] = {SGI0, SGI294020, NULL};
const Segment * const SG69382[] = {SGI0, SGI294021, NULL};
const Segment * const SG69383[] = {SGI0, SGI294022, NULL};
const Segment * const SG69384[] = {SGI0, SGI294023, NULL};
const Segment * const SG69385[] = {SGI0, SGI294024, NULL};
const Segment * const SG69386[] = {SGI0, SGI294025, NULL};
const Segment * const SG69387[] = {SGI0, SGI294026, NULL};
const Segment * const SG69388[] = {SGI0, SGI294027, NULL};
const Segment * const SG69389[] = {SGI0, SGI294028, NULL};
const Segment * const SG69390[] = {SGI0, SGI294029, NULL};
const Segment * const SG69391[] = {SGI0, SGI294030, NULL};
const Segment * const SG69392[] = {SGI0, SGI294031, NULL};
const Segment * const SG69393[] = {SGI0, SGI294032, NULL};
const Segment * const SG69394[] = {SGI0, SGI294033, NULL};
const Segment * const SG69395[] = {SGI294034, SGI294035, NULL};
const Segment * const SG69396[] = {SGI0, SGI294036, NULL};
const Segment * const SG69397[] = {SGI0, SGI294037, NULL};
const Segment * const SG69398[] = {SGI0, SGI294038, SGI0, SGI294039, SGI294040, SGI294040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294041, SGI294042, SGI0, SGI294043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69399[] = {SGI0, SGI294044, SGI0, SGI294045, SGI294046, SGI294046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294047, SGI294048, SGI0, SGI294049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69400[] = {SGI0, SGI294050, SGI0, SGI294051, SGI294052, SGI294053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294054, SGI294055, SGI0, SGI0, SGI0, SGI0, SGI294056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69401[] = {SGI0, SGI294057, SGI0, SGI294058, SGI294059, SGI294060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69402[] = {SGI0, SGI294062, SGI0, SGI294063, SGI294064, SGI294065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69403[] = {SGI0, SGI294067, SGI0, SGI294068, SGI294069, SGI294070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294071, SGI294072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69404[] = {SGI0, SGI294073, SGI0, SGI294074, SGI294075, SGI294075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294076, SGI294077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294078, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69405[] = {SGI0, SGI294079, SGI0, SGI294080, SGI294081, SGI294082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69406[] = {SGI0, SGI294084, SGI0, SGI294085, SGI294086, SGI294087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69407[] = {SGI0, SGI294090, SGI0, SGI294091, SGI294092, SGI294093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294094, SGI294095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69408[] = {SGI0, SGI294096, SGI0, SGI294097, SGI294098, SGI294099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294101, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69409[] = {SGI0, SGI294102, SGI0, SGI294103, SGI294104, SGI294104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69410[] = {SGI0, SGI294106, SGI0, SGI294107, SGI294108, SGI294108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69411[] = {SGI0, SGI294110, SGI0, SGI294111, SGI294112, SGI294112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69412[] = {SGI0, SGI294114, SGI0, SGI294115, SGI294116, SGI294116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294117, SGI294118, SGI0, SGI294119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69413[] = {SGI0, SGI294120, SGI0, SGI294121, SGI294122, SGI294123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69414[] = {SGI0, SGI294125, SGI0, SGI294126, SGI294127, SGI294128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294130, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69415[] = {SGI0, SGI294131, SGI0, SGI294132, SGI294133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294134, SGI294135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69416[] = {SGI0, SGI294136, SGI0, SGI294137, SGI294138, SGI294138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294139, SGI294140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69417[] = {SGI0, SGI294141, SGI0, SGI294142, SGI294143, SGI294144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294145, SGI294146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69418[] = {SGI0, SGI294147, SGI0, SGI294148, SGI294149, SGI294149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294150, SGI294151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294152, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69419[] = {SGI0, SGI294153, SGI0, SGI294154, SGI294155, SGI294156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69420[] = {SGI0, SGI294158, SGI0, SGI294159, SGI294160, SGI294161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294163, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69421[] = {SGI0, SGI294164, SGI0, SGI294165, SGI294166, SGI294166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294167, SGI294168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69422[] = {SGI0, SGI294170, SGI0, SGI294171, SGI294172, SGI294173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294174, SGI294175, SGI0, SGI0, SGI0, SGI0, SGI294176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69423[] = {SGI0, SGI294177, SGI0, SGI294178, SGI294179, SGI294180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294181, SGI294182, SGI0, SGI0, SGI0, SGI0, SGI294183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69424[] = {SGI0, SGI294184, SGI0, SGI294185, SGI294186, SGI294186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294188, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69425[] = {SGI0, SGI294189, SGI0, SGI294190, SGI294191, SGI294191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294192, SGI294193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69426[] = {SGI0, SGI294194, SGI0, SGI294195, SGI294196, SGI294197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294198, SGI294199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69427[] = {SGI0, SGI294200, SGI0, SGI294201, SGI294202, SGI294202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69428[] = {SGI0, SGI294205, SGI0, SGI294206, SGI294207, SGI294207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294208, SGI294209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69429[] = {SGI0, SGI294211, SGI0, SGI294212, SGI294213, SGI294214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69430[] = {SGI0, SGI294216, SGI0, SGI294217, SGI294218, SGI294218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294219, SGI294220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294221, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69431[] = {SGI0, SGI0, SGI0, SGI294222, SGI294223, SGI294223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69432[] = {SGI0, SGI0, SGI0, SGI294225, SGI294226, SGI294226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69433[] = {SGI0, SGI0, SGI0, SGI294228, SGI294229, SGI294229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69434[] = {SGI0, SGI294230, SGI0, SGI294231, SGI294232, SGI294232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294234, SGI0, NULL};
const Segment * const SG69435[] = {SGI0, SGI294235, SGI0, SGI294236, SGI294237, SGI294237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294238, SGI294239, SGI0, SGI0, SGI294240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69436[] = {SGI0, SGI294241, SGI0, SGI294242, SGI294243, SGI294243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294244, SGI294245, SGI0, SGI0, SGI294246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69437[] = {SGI0, SGI294247, SGI0, SGI294248, SGI294249, SGI294250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69438[] = {SGI0, SGI294252, SGI0, SGI294253, SGI294254, SGI294255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69439[] = {SGI0, SGI294257, SGI0, SGI294258, SGI294259, SGI294259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294260, SGI294261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69440[] = {SGI0, SGI294263, SGI0, SGI294264, SGI294265, SGI294265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294266, SGI294267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69441[] = {SGI0, SGI294269, SGI0, SGI294270, SGI294271, SGI294272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69442[] = {SGI0, SGI294274, SGI0, SGI294275, SGI294276, SGI294277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294278, SGI294279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69443[] = {SGI0, SGI294281, SGI0, SGI294282, SGI294283, SGI294284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69444[] = {SGI0, SGI294286, SGI0, SGI294287, SGI294288, SGI294288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294290, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69445[] = {SGI0, SGI294291, SGI0, SGI294292, SGI294293, SGI294293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294294, SGI294295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69446[] = {SGI0, SGI294297, SGI0, SGI294298, SGI294299, SGI294300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69447[] = {SGI0, SGI294302, SGI0, SGI294303, SGI294304, SGI294304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294305, SGI294306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294307, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69448[] = {SGI0, SGI294308, SGI0, SGI294309, SGI294310, SGI294311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294312, SGI294313, SGI0, SGI0, SGI0, SGI0, SGI294314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69449[] = {SGI0, SGI294315, NULL};
const Segment * const SG69450[] = {SGI0, SGI294316, NULL};
const Segment * const SG69451[] = {SGI0, SGI294317, SGI0, SGI294318, SGI294319, SGI294319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294320, SGI294321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294322, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69452[] = {SGI0, SGI294323, NULL};
const Segment * const SG69453[] = {SGI0, SGI294324, NULL};
const Segment * const SG69454[] = {SGI0, SGI294325, NULL};
const Segment * const SG69455[] = {SGI0, SGI294326, NULL};
const Segment * const SG69456[] = {SGI0, SGI294327, NULL};
const Segment * const SG69457[] = {SGI0, SGI294328, NULL};
const Segment * const SG69458[] = {SGI0, SGI294329, NULL};
const Segment * const SG69459[] = {SGI0, SGI294330, NULL};
const Segment * const SG69460[] = {SGI0, SGI294331, NULL};
const Segment * const SG69461[] = {SGI0, SGI294332, NULL};
const Segment * const SG69462[] = {SGI0, SGI294333, NULL};
const Segment * const SG69463[] = {SGI0, SGI294334, NULL};
const Segment * const SG69464[] = {SGI0, SGI294335, NULL};
const Segment * const SG69465[] = {SGI0, SGI294336, SGI0, SGI294337, SGI294338, SGI294339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69466[] = {SGI0, SGI294342, SGI0, SGI294343, SGI294344, SGI294345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69467[] = {SGI0, SGI294347, SGI0, SGI294348, SGI294349, SGI294349, SGI294350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294351, SGI0, SGI294352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69468[] = {SGI0, SGI294353, SGI0, SGI294354, SGI294355, SGI294356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294358, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69469[] = {SGI0, SGI294359, SGI0, SGI294360, SGI294361, SGI294361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294362, SGI294363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69470[] = {SGI0, SGI294365, SGI0, SGI294366, SGI294367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294368, SGI294369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69471[] = {SGI0, SGI294370, SGI0, SGI294371, SGI294372, SGI294373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294374, SGI294375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69472[] = {SGI0, SGI294376, SGI0, SGI294377, SGI294378, SGI294379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69473[] = {SGI0, SGI294381, SGI0, SGI294382, SGI294383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294384, SGI294385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69474[] = {SGI0, SGI0, SGI0, SGI294386, SGI294387, SGI294387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294389, SGI0, SGI0, NULL};
const Segment * const SG69475[] = {SGI0, SGI294390, SGI0, SGI294391, SGI294392, SGI294393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69476[] = {SGI0, SGI294395, SGI0, SGI294396, SGI294397, SGI294397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294398, SGI294399, SGI0, SGI0, SGI294400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69477[] = {SGI0, SGI294401, SGI0, SGI294402, SGI294403, SGI294404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69478[] = {SGI0, SGI294406, SGI0, SGI294407, SGI294408, SGI294409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294410, SGI294411, SGI0, SGI0, SGI0, SGI294412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69479[] = {SGI0, SGI294413, SGI0, SGI294414, SGI294415, SGI294415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294417, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69480[] = {SGI0, SGI294418, SGI0, SGI294419, SGI294420, SGI294420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294421, SGI294422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294423, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69481[] = {SGI0, SGI294424, SGI0, SGI294425, SGI294426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294427, SGI294428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69482[] = {SGI0, SGI294429, SGI0, SGI294430, SGI294431, SGI294432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69483[] = {SGI0, SGI294434, SGI0, SGI294435, SGI294436, SGI294437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294438, SGI294439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69484[] = {SGI0, SGI294440, SGI0, SGI294441, SGI294442, SGI294443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69485[] = {SGI0, SGI294445, SGI0, SGI294446, SGI294447, SGI294447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294448, SGI294449, SGI0, SGI294450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69486[] = {SGI0, SGI294451, SGI0, SGI294452, SGI294453, SGI294453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294454, SGI294455, SGI0, SGI294456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69487[] = {SGI0, SGI294457, SGI0, SGI294458, SGI294459, SGI294459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294460, SGI294461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294462, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69488[] = {SGI0, SGI294463, SGI0, SGI294464, SGI294465, SGI294465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294466, SGI294467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294468, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69489[] = {SGI0, SGI294469, SGI0, SGI294470, SGI294471, SGI294471, SGI0, SGI294472, SGI0, SGI0, SGI0, SGI0, SGI294473, SGI294474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69490[] = {SGI0, SGI294475, SGI0, SGI294476, SGI294477, SGI294478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69491[] = {SGI0, SGI294481, SGI0, SGI294482, SGI294483, SGI294483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69492[] = {SGI0, SGI294485, SGI0, SGI294486, SGI294487, SGI294488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294490, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69493[] = {SGI0, SGI294491, NULL};
const Segment * const SG69494[] = {SGI294492, SGI294493, SGI294494, SGI294495, SGI0, SGI294496, SGI0, NULL};
const Segment * const SG69495[] = {SGI0, SGI294497, SGI0, SGI294498, SGI294499, SGI294500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69496[] = {SGI0, SGI294502, SGI0, SGI294503, SGI294504, SGI294505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294506, SGI294507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69497[] = {SGI0, SGI294508, SGI0, SGI294509, SGI294510, SGI294511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69498[] = {SGI0, SGI294513, SGI0, SGI294514, SGI294515, SGI294515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294516, SGI294517, SGI0, SGI294518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69499[] = {SGI0, SGI294519, SGI0, SGI294520, SGI294521, SGI294521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294522, SGI294523, SGI0, SGI294524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69500[] = {SGI0, SGI294525, SGI0, SGI294526, SGI294527, SGI294528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69501[] = {SGI0, SGI294530, SGI0, SGI294531, SGI294532, SGI294533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294534, SGI294535, SGI0, SGI0, SGI0, SGI0, SGI294536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69502[] = {SGI0, SGI294537, SGI0, SGI294538, SGI294539, SGI294540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69503[] = {SGI0, SGI294542, SGI0, SGI294543, SGI294544, SGI294544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294545, SGI294546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69504[] = {SGI0, SGI294548, SGI0, SGI294549, SGI294550, SGI294550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294552, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69505[] = {SGI0, SGI294553, SGI0, SGI294554, SGI294555, SGI294556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294557, SGI294558, SGI0, SGI0, SGI0, SGI0, SGI294559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69506[] = {SGI0, SGI294560, SGI0, SGI294561, SGI294562, SGI294562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69507[] = {SGI0, SGI294565, SGI0, SGI294566, SGI294567, SGI294567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294568, SGI294569, SGI0, SGI0, SGI0, SGI294570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69508[] = {SGI0, SGI294571, SGI0, SGI294572, SGI294573, SGI294574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69509[] = {SGI0, SGI294576, SGI0, SGI294577, SGI294578, SGI294579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69510[] = {SGI0, SGI294582, SGI0, SGI294583, SGI294584, SGI294585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294586, SGI294587, SGI0, SGI0, SGI0, SGI0, SGI294588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69511[] = {SGI0, SGI294589, NULL};
const Segment * const SG69512[] = {SGI0, SGI294590, NULL};
const Segment * const SG69513[] = {SGI0, SGI294591, NULL};
const Segment * const SG69514[] = {SGI0, SGI294592, NULL};
const Segment * const SG69515[] = {SGI0, SGI294593, NULL};
const Segment * const SG69516[] = {SGI0, SGI294594, NULL};
const Segment * const SG69517[] = {SGI0, SGI294595, NULL};
const Segment * const SG69518[] = {SGI0, SGI294596, NULL};
const Segment * const SG69519[] = {SGI0, SGI294597, NULL};
const Segment * const SG69520[] = {SGI0, SGI294598, NULL};
const Segment * const SG69521[] = {SGI0, SGI294599, NULL};
const Segment * const SG69522[] = {SGI0, SGI294600, NULL};
const Segment * const SG69523[] = {SGI0, SGI294601, NULL};
const Segment * const SG69524[] = {SGI0, SGI294602, NULL};
const Segment * const SG69525[] = {SGI0, SGI294603, NULL};
const Segment * const SG69526[] = {SGI0, SGI0, SGI0, SGI294604, SGI294605, SGI294606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294607, SGI294608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69527[] = {SGI0, SGI0, SGI0, SGI294609, SGI294610, SGI294611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294612, SGI294613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69528[] = {SGI0, SGI294614, SGI0, SGI294615, SGI294616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294617, SGI294618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69529[] = {SGI0, SGI294619, SGI0, SGI294620, SGI294621, SGI294622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294623, SGI294624, SGI0, SGI0, SGI0, SGI0, SGI294625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69530[] = {SGI0, SGI294626, NULL};
const Segment * const SG69531[] = {SGI294627, SGI294628, NULL};
const Segment * const SG69532[] = {SGI294629, SGI294630, NULL};
const Segment * const SG69533[] = {SGI0, SGI294631, NULL};
const Segment * const SG69534[] = {SGI0, SGI294632, NULL};
const Segment * const SG69535[] = {SGI0, SGI294633, NULL};
const Segment * const SG69536[] = {SGI294634, SGI294635, NULL};
const Segment * const SG69537[] = {SGI0, SGI294636, NULL};
const Segment * const SG69538[] = {SGI0, SGI294637, NULL};
const Segment * const SG69539[] = {SGI0, SGI294638, NULL};
const Segment * const SG69540[] = {SGI0, SGI294639, NULL};
const Segment * const SG69541[] = {SGI0, SGI294640, NULL};
const Segment * const SG69542[] = {SGI0, SGI294641, NULL};
const Segment * const SG69543[] = {SGI0, SGI294642, NULL};
const Segment * const SG69544[] = {SGI0, SGI294643, NULL};
const Segment * const SG69545[] = {SGI0, SGI294644, NULL};
const Segment * const SG69546[] = {SGI0, SGI294645, NULL};
const Segment * const SG69547[] = {SGI0, SGI294649, NULL};
const Segment * const SG69548[] = {SGI0, SGI294646, SGI0, SGI294647, SGI294648, SGI294648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294650, SGI294651, SGI0, SGI0, SGI294652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69549[] = {SGI0, SGI294653, SGI0, SGI294654, SGI294655, SGI294655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294656, SGI294657, SGI0, SGI0, SGI294658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69550[] = {SGI0, SGI294659, SGI0, SGI294660, SGI294661, SGI294661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294662, SGI294663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294664, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69551[] = {SGI0, SGI294665, SGI0, SGI294666, SGI294667, SGI294667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294668, SGI294669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294670, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69552[] = {SGI0, SGI294671, SGI0, SGI294672, SGI294673, SGI294674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294675, SGI294676, SGI0, SGI0, SGI0, SGI0, SGI294677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69553[] = {SGI0, SGI294678, SGI0, SGI294679, SGI294680, SGI294680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69554[] = {SGI0, SGI294682, SGI0, SGI294683, SGI294684, SGI294685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294686, SGI294687, SGI0, SGI0, SGI0, SGI294688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69555[] = {SGI0, SGI294689, SGI0, SGI294690, SGI294691, SGI294692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294693, SGI294694, SGI0, SGI0, SGI0, SGI0, SGI294695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69556[] = {SGI0, SGI294696, SGI0, SGI294697, SGI294698, SGI294699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69557[] = {SGI0, SGI294701, SGI0, SGI294702, SGI294703, SGI294703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294704, SGI294705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69558[] = {SGI0, SGI294706, SGI0, SGI294707, SGI294708, SGI294709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294710, SGI294711, SGI0, SGI294712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69559[] = {SGI0, SGI294713, SGI0, SGI294714, SGI294715, SGI294716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69560[] = {SGI0, SGI294718, SGI0, SGI294719, SGI294720, SGI294721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294722, SGI294723, SGI0, SGI0, SGI0, SGI294724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69561[] = {SGI0, SGI294725, SGI0, SGI294726, SGI294727, SGI294728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294729, SGI294730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69562[] = {SGI0, SGI294732, SGI0, SGI294733, SGI294734, SGI294735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294736, SGI294737, SGI0, SGI0, SGI0, SGI0, SGI294738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69563[] = {SGI0, SGI294739, SGI0, SGI294740, SGI294741, SGI294742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294743, SGI294744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69564[] = {SGI0, SGI294746, SGI0, SGI294747, SGI294748, SGI294749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69565[] = {SGI0, SGI294751, SGI0, SGI294752, SGI294753, SGI294754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69566[] = {SGI0, SGI294756, SGI0, SGI294757, SGI294758, SGI294758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294759, SGI294760, SGI0, SGI294761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69567[] = {SGI0, SGI294762, SGI0, SGI294763, SGI294764, SGI294764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294765, SGI294766, SGI0, SGI294767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69568[] = {SGI0, SGI294768, SGI0, SGI294769, SGI294770, SGI294770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294771, SGI294772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294773, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69569[] = {SGI294774, SGI294775, SGI294776, SGI294777, SGI294778, SGI294775, SGI0, NULL};
const Segment * const SG69570[] = {SGI0, SGI294779, SGI0, SGI294780, SGI294781, SGI294781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294782, SGI294783, SGI0, SGI294784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69571[] = {SGI0, SGI294785, SGI0, SGI294786, SGI294787, SGI294787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294788, SGI294789, SGI0, SGI294790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69572[] = {SGI0, SGI294791, SGI0, SGI294792, SGI294793, SGI294794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294795, SGI294796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69573[] = {SGI0, SGI294798, SGI0, SGI294799, SGI294800, SGI294800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294801, SGI294802, SGI0, SGI0, SGI294803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69574[] = {SGI0, SGI294804, SGI0, SGI294805, SGI294806, SGI294807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294808, SGI294809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69575[] = {SGI0, SGI294811, SGI0, SGI294812, SGI294813, SGI294814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69576[] = {SGI0, SGI294816, SGI0, SGI294817, SGI294818, SGI294818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294819, SGI294820, SGI0, SGI294821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69577[] = {SGI0, SGI294822, SGI0, SGI294823, SGI294824, SGI294824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294825, SGI294826, SGI0, SGI294827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69578[] = {SGI0, SGI294828, NULL};
const Segment * const SG69579[] = {SGI0, SGI294829, NULL};
const Segment * const SG69580[] = {SGI0, SGI294830, NULL};
const Segment * const SG69581[] = {SGI0, SGI294831, NULL};
const Segment * const SG69582[] = {SGI0, SGI294832, NULL};
const Segment * const SG69583[] = {SGI0, SGI294833, NULL};
const Segment * const SG69584[] = {SGI0, SGI294834, NULL};
const Segment * const SG69585[] = {SGI0, SGI294835, NULL};
const Segment * const SG69586[] = {SGI0, SGI294836, NULL};
const Segment * const SG69587[] = {SGI0, SGI294837, NULL};
const Segment * const SG69588[] = {SGI0, SGI294838, SGI0, SGI294839, SGI294840, SGI294840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294842, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69589[] = {SGI0, SGI294843, SGI0, SGI294844, SGI294845, SGI294846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294848, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69590[] = {SGI0, SGI294849, NULL};
const Segment * const SG69591[] = {SGI294850, SGI294851, NULL};
const Segment * const SG69592[] = {SGI0, SGI294852, NULL};
const Segment * const SG69593[] = {SGI294853, SGI294854, NULL};
const Segment * const SG69594[] = {SGI0, SGI294855, NULL};
const Segment * const SG69595[] = {SGI0, SGI294856, NULL};
const Segment * const SG69596[] = {SGI0, SGI294857, NULL};
const Segment * const SG69597[] = {SGI0, SGI294858, NULL};
const Segment * const SG69598[] = {SGI294859, NULL};
const Segment * const SG69599[] = {SGI294860, NULL};
const Segment * const SG69600[] = {SGI294861, NULL};
const Segment * const SG69601[] = {SGI294862, NULL};
const Segment * const SG69602[] = {SGI294863, NULL};
const Segment * const SG69603[] = {SGI294864, NULL};
const Segment * const SG69604[] = {SGI294865, NULL};
const Segment * const SG69605[] = {SGI294866, NULL};
const Segment * const SG69606[] = {SGI294867, NULL};
const Segment * const SG69607[] = {SGI294868, NULL};
const Segment * const SG69608[] = {SGI294869, NULL};
const Segment * const SG69609[] = {SGI294870, NULL};
const Segment * const SG69610[] = {SGI294871, NULL};
const Segment * const SG69611[] = {SGI294872, NULL};
const Segment * const SG69612[] = {SGI294873, NULL};
const Segment * const SG69613[] = {SGI294874, NULL};
const Segment * const SG69614[] = {SGI294875, NULL};
const Segment * const SG69615[] = {SGI294876, NULL};
const Segment * const SG69616[] = {SGI294877, NULL};
const Segment * const SG69617[] = {SGI294878, NULL};
const Segment * const SG69618[] = {SGI294879, NULL};
const Segment * const SG69619[] = {SGI294880, NULL};
const Segment * const SG69620[] = {SGI294881, NULL};
const Segment * const SG69621[] = {SGI294882, NULL};
const Segment * const SG69622[] = {SGI294883, NULL};
const Segment * const SG69623[] = {SGI294884, NULL};
const Segment * const SG69624[] = {SGI294885, NULL};
const Segment * const SG69625[] = {SGI0, SGI294886, SGI0, SGI294887, SGI294888, SGI294888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294889, SGI294890, SGI0, SGI294891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69626[] = {SGI0, SGI294892, SGI0, SGI294893, SGI294894, SGI294894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294895, SGI294896, SGI0, SGI0, SGI0, SGI294897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69627[] = {SGI0, SGI294898, SGI0, SGI294899, SGI294900, SGI294901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69628[] = {SGI0, SGI294903, SGI0, SGI294904, SGI294905, SGI294905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294906, SGI294907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69629[] = {SGI0, SGI294909, SGI0, SGI294910, SGI294911, SGI294912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294913, SGI294914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69630[] = {SGI0, SGI294915, SGI0, SGI294916, SGI294917, SGI294917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294918, SGI294919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69631[] = {SGI0, SGI294920, SGI0, SGI294921, SGI294922, SGI294923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69632[] = {SGI0, SGI294925, SGI0, SGI294926, SGI294927, SGI294927, SGI0, SGI0, SGI0, SGI294928, SGI0, SGI0, SGI294929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69633[] = {SGI0, SGI294930, SGI0, SGI294931, SGI294932, SGI294932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294934, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69634[] = {SGI0, SGI0, SGI0, SGI294935, SGI294936, SGI294937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294938, SGI294939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69635[] = {SGI0, SGI0, SGI0, SGI294940, SGI294941, SGI294942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294943, SGI294944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69636[] = {SGI0, SGI0, SGI0, SGI294945, SGI294946, SGI294947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69637[] = {SGI0, SGI294949, NULL};
const Segment * const SG69638[] = {SGI0, SGI294950, NULL};
const Segment * const SG69639[] = {SGI0, SGI294951, NULL};
const Segment * const SG69640[] = {SGI0, SGI294952, NULL};
const Segment * const SG69641[] = {SGI0, SGI294953, NULL};
const Segment * const SG69642[] = {SGI0, SGI294954, NULL};
const Segment * const SG69643[] = {SGI0, SGI294955, NULL};
const Segment * const SG69644[] = {SGI0, SGI294956, NULL};
const Segment * const SG69645[] = {SGI0, SGI294957, NULL};
const Segment * const SG69646[] = {SGI0, SGI294958, NULL};
const Segment * const SG69647[] = {SGI0, SGI294959, NULL};
const Segment * const SG69648[] = {SGI0, SGI294960, NULL};
const Segment * const SG69649[] = {SGI0, SGI294961, NULL};
const Segment * const SG69650[] = {SGI0, SGI294962, NULL};
const Segment * const SG69651[] = {SGI0, SGI294963, NULL};
const Segment * const SG69652[] = {SGI0, SGI294964, NULL};
const Segment * const SG69653[] = {SGI0, SGI294965, NULL};
const Segment * const SG69654[] = {SGI0, SGI294966, NULL};
const Segment * const SG69655[] = {SGI0, SGI294967, NULL};
const Segment * const SG69656[] = {SGI0, SGI294968, NULL};
const Segment * const SG69657[] = {SGI0, SGI294969, NULL};
const Segment * const SG69658[] = {SGI0, SGI294970, NULL};
const Segment * const SG69659[] = {SGI0, SGI294971, NULL};
const Segment * const SG69660[] = {SGI0, SGI294972, NULL};
const Segment * const SG69661[] = {SGI0, SGI294973, NULL};
const Segment * const SG69662[] = {SGI0, SGI294974, NULL};
const Segment * const SG69663[] = {SGI0, SGI294975, NULL};
const Segment * const SG69664[] = {SGI0, SGI294976, NULL};
const Segment * const SG69665[] = {SGI0, SGI294977, NULL};
const Segment * const SG69666[] = {SGI0, SGI294978, SGI0, SGI294979, SGI294980, SGI294981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294982, SGI294983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69667[] = {SGI0, SGI294984, SGI0, SGI294985, SGI294986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294987, SGI294988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69668[] = {SGI0, SGI294989, SGI0, SGI294990, SGI294991, SGI294991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294992, SGI294993, SGI0, SGI294994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69669[] = {SGI0, SGI294995, SGI0, SGI294996, SGI294997, SGI294997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI294998, SGI294999, SGI0, SGI295000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69670[] = {SGI0, SGI295001, SGI0, SGI295002, SGI295003, SGI295003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295004, SGI295005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295006, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69671[] = {SGI0, SGI295007, SGI0, SGI295008, SGI295009, SGI295009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295010, SGI295011, SGI0, SGI0, SGI0, SGI295012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69672[] = {SGI0, SGI295013, SGI0, SGI295014, SGI295015, SGI295016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69673[] = {SGI0, SGI295019, SGI0, SGI295020, SGI295021, SGI295022, SGI0, SGI0, SGI0, SGI0, SGI295023, SGI0, SGI295024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69674[] = {SGI0, SGI295025, SGI0, SGI295026, SGI295027, SGI295028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69675[] = {SGI0, SGI295030, SGI0, SGI295031, SGI295032, SGI295032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295034, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69676[] = {SGI295035, SGI295036, SGI295037, SGI295038, SGI295039, SGI295036, SGI0, NULL};
const Segment * const SG69677[] = {SGI0, SGI295040, SGI0, SGI295041, SGI295042, SGI295043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295045, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69678[] = {SGI0, SGI295046, SGI0, SGI295047, SGI295048, SGI295049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295050, SGI295051, SGI0, SGI0, SGI0, SGI0, SGI295052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69679[] = {SGI0, SGI295053, SGI0, SGI295054, SGI295055, SGI295056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295057, SGI295058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69680[] = {SGI0, SGI295059, SGI0, SGI295060, SGI295061, SGI295061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295062, SGI295063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295064, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69681[] = {SGI0, SGI295065, NULL};
const Segment * const SG69682[] = {SGI0, SGI295066, SGI0, SGI295067, SGI295068, SGI295069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295071, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69683[] = {SGI0, SGI295072, SGI0, SGI295073, SGI295074, SGI295075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295076, SGI295077, SGI0, SGI0, SGI0, SGI295078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69684[] = {SGI295079, SGI295080, SGI295081, SGI295082, SGI295083, SGI295080, SGI0, NULL};
const Segment * const SG69685[] = {SGI0, SGI295084, NULL};
const Segment * const SG69686[] = {SGI0, SGI295085, NULL};
const Segment * const SG69687[] = {SGI295086, SGI295087, NULL};
const Segment * const SG69688[] = {SGI0, SGI295088, NULL};
const Segment * const SG69689[] = {SGI0, SGI295089, NULL};
const Segment * const SG69690[] = {SGI0, SGI295090, NULL};
const Segment * const SG69691[] = {SGI0, SGI295091, NULL};
const Segment * const SG69692[] = {SGI0, SGI295092, NULL};
const Segment * const SG69693[] = {SGI0, SGI295093, NULL};
const Segment * const SG69694[] = {SGI0, SGI295094, NULL};
const Segment * const SG69695[] = {SGI0, SGI295095, NULL};
const Segment * const SG69696[] = {SGI0, SGI295096, NULL};
const Segment * const SG69697[] = {SGI0, SGI295097, NULL};
const Segment * const SG69698[] = {SGI0, SGI295098, NULL};
const Segment * const SG69699[] = {SGI0, SGI295099, NULL};
const Segment * const SG69700[] = {SGI0, SGI295100, NULL};
const Segment * const SG69701[] = {SGI0, SGI295101, NULL};
const Segment * const SG69702[] = {SGI0, SGI295102, NULL};
const Segment * const SG69703[] = {SGI0, SGI295103, NULL};
const Segment * const SG69704[] = {SGI0, SGI295104, NULL};
const Segment * const SG69705[] = {SGI0, SGI295105, NULL};
const Segment * const SG69706[] = {SGI0, SGI295106, NULL};
const Segment * const SG69707[] = {SGI0, SGI295107, NULL};
const Segment * const SG69708[] = {SGI0, SGI295108, NULL};
const Segment * const SG69709[] = {SGI0, SGI295109, NULL};
const Segment * const SG69710[] = {SGI0, SGI295110, NULL};
const Segment * const SG69711[] = {SGI0, SGI295111, NULL};
const Segment * const SG69712[] = {SGI0, SGI295112, NULL};
const Segment * const SG69713[] = {SGI0, SGI295113, NULL};
const Segment * const SG69714[] = {SGI0, SGI295114, SGI0, SGI295115, SGI295116, SGI295116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295117, SGI295118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295119, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69715[] = {SGI0, SGI295120, SGI0, SGI295121, SGI295122, SGI295122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295123, SGI295124, SGI0, SGI295125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69716[] = {SGI0, SGI295126, SGI0, SGI295127, SGI295128, SGI295128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69717[] = {SGI0, SGI295130, SGI0, SGI295131, SGI295132, SGI295133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69718[] = {SGI0, SGI295135, SGI0, SGI295136, SGI295137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295138, SGI295139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69719[] = {SGI0, SGI295140, SGI0, SGI295141, SGI295142, SGI295143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69720[] = {SGI0, SGI295145, SGI0, SGI295146, SGI295147, SGI295147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295148, SGI295149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295150, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69721[] = {SGI0, SGI295151, SGI0, SGI295152, SGI295153, SGI295153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295154, SGI295155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69722[] = {SGI0, SGI295156, SGI0, SGI295157, SGI295158, SGI295159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69723[] = {SGI0, SGI295161, NULL};
const Segment * const SG69724[] = {SGI0, SGI295162, NULL};
const Segment * const SG69725[] = {SGI0, SGI295163, NULL};
const Segment * const SG69726[] = {SGI0, SGI295164, NULL};
const Segment * const SG69727[] = {SGI0, SGI295165, NULL};
const Segment * const SG69728[] = {SGI0, SGI295166, NULL};
const Segment * const SG69729[] = {SGI0, SGI295167, NULL};
const Segment * const SG69730[] = {SGI0, SGI295168, NULL};
const Segment * const SG69731[] = {SGI0, SGI295169, NULL};
const Segment * const SG69732[] = {SGI0, SGI295170, NULL};
const Segment * const SG69733[] = {SGI0, SGI295171, NULL};
const Segment * const SG69734[] = {SGI0, SGI295172, SGI0, SGI295173, SGI295174, SGI295175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295176, SGI295177, SGI0, SGI0, SGI0, SGI0, SGI295178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69735[] = {SGI0, SGI295179, SGI0, SGI295180, SGI295181, SGI295181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295182, SGI295183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295184, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69736[] = {SGI0, SGI295185, SGI0, SGI295186, SGI295187, SGI295188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295189, SGI295190, SGI0, SGI0, SGI0, SGI295191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69737[] = {SGI0, SGI295192, SGI0, SGI295193, SGI295194, SGI295194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295195, SGI295196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69738[] = {SGI0, SGI295197, NULL};
const Segment * const SG69739[] = {SGI0, SGI295198, NULL};
const Segment * const SG69740[] = {SGI0, SGI295199, NULL};
const Segment * const SG69741[] = {SGI0, SGI295200, NULL};
const Segment * const SG69742[] = {SGI0, SGI295201, NULL};
const Segment * const SG69743[] = {SGI0, SGI295202, NULL};
const Segment * const SG69744[] = {SGI295203, SGI295204, NULL};
const Segment * const SG69745[] = {SGI295205, SGI295206, NULL};
const Segment * const SG69746[] = {SGI295207, SGI295208, NULL};
const Segment * const SG69747[] = {SGI0, SGI295209, SGI0, SGI295210, SGI295211, SGI295212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69748[] = {SGI0, SGI295214, SGI0, SGI295215, SGI295216, SGI295217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69749[] = {SGI0, SGI0, SGI0, SGI295219, SGI295220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69750[] = {SGI0, SGI295222, SGI0, SGI295223, SGI295224, SGI295224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295225, SGI295226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295227, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69751[] = {SGI0, SGI295228, SGI0, SGI295229, SGI295230, SGI295231, SGI0, SGI0, SGI0, SGI0, SGI295232, SGI0, SGI295233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69752[] = {SGI0, SGI295234, SGI0, SGI295235, SGI295236, SGI295236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69753[] = {SGI0, SGI295239, NULL};
const Segment * const SG69754[] = {SGI0, SGI295240, NULL};
const Segment * const SG69755[] = {SGI295241, SGI295242, NULL};
const Segment * const SG69756[] = {SGI0, SGI295243, NULL};
const Segment * const SG69757[] = {SGI0, SGI295244, NULL};
const Segment * const SG69758[] = {SGI295245, SGI295246, NULL};
const Segment * const SG69759[] = {SGI0, SGI295247, NULL};
const Segment * const SG69760[] = {SGI0, SGI295248, NULL};
const Segment * const SG69761[] = {SGI0, SGI295249, NULL};
const Segment * const SG69762[] = {SGI0, SGI295250, NULL};
const Segment * const SG69763[] = {SGI0, SGI295251, NULL};
const Segment * const SG69764[] = {SGI0, SGI295252, NULL};
const Segment * const SG69765[] = {SGI0, SGI295253, NULL};
const Segment * const SG69766[] = {SGI0, SGI295254, NULL};
const Segment * const SG69767[] = {SGI0, SGI295255, NULL};
const Segment * const SG69768[] = {SGI0, SGI295256, NULL};
const Segment * const SG69769[] = {SGI0, SGI295257, NULL};
const Segment * const SG69770[] = {SGI0, SGI295258, NULL};
const Segment * const SG69771[] = {SGI0, SGI295259, SGI0, SGI295260, SGI295261, SGI295262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295263, SGI295264, SGI0, SGI0, SGI0, SGI0, SGI295265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69772[] = {SGI0, SGI295266, SGI0, SGI295267, SGI295268, SGI295268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295269, SGI295270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295271, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69773[] = {SGI0, SGI295272, SGI0, SGI295273, SGI295274, SGI295275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69774[] = {SGI0, SGI295277, SGI0, SGI295278, SGI295279, SGI295279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295280, SGI295281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69775[] = {SGI0, SGI295282, SGI0, SGI295283, SGI295284, SGI295285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295286, SGI295287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69776[] = {SGI0, SGI295289, NULL};
const Segment * const SG69777[] = {SGI295290, SGI295291, NULL};
const Segment * const SG69778[] = {SGI0, SGI295292, NULL};
const Segment * const SG69779[] = {SGI0, SGI295293, NULL};
const Segment * const SG69780[] = {SGI0, SGI295294, NULL};
const Segment * const SG69781[] = {SGI0, SGI295295, NULL};
const Segment * const SG69782[] = {SGI0, SGI295296, NULL};
const Segment * const SG69783[] = {SGI0, SGI295297, NULL};
const Segment * const SG69784[] = {SGI0, SGI295298, NULL};
const Segment * const SG69785[] = {SGI0, SGI295299, NULL};
const Segment * const SG69786[] = {SGI0, SGI295300, NULL};
const Segment * const SG69787[] = {SGI0, SGI295301, NULL};
const Segment * const SG69788[] = {SGI0, SGI295302, NULL};
const Segment * const SG69789[] = {SGI0, SGI295303, NULL};
const Segment * const SG69790[] = {SGI0, SGI295304, NULL};
const Segment * const SG69791[] = {SGI0, SGI295305, NULL};
const Segment * const SG69792[] = {SGI0, SGI295306, SGI0, SGI295307, SGI295308, SGI295309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295310, SGI295311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69793[] = {SGI0, SGI295313, SGI0, SGI295314, SGI295315, SGI295316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295317, SGI295318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69794[] = {SGI0, SGI295320, SGI0, SGI295321, SGI295322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69795[] = {SGI295324, SGI295325, NULL};
const Segment * const SG69796[] = {SGI295326, SGI295327, NULL};
const Segment * const SG69797[] = {SGI295328, SGI295329, NULL};
const Segment * const SG69798[] = {SGI295330, SGI295331, NULL};
const Segment * const SG69799[] = {SGI295332, SGI295333, NULL};
const Segment * const SG69800[] = {SGI295334, SGI295335, NULL};
const Segment * const SG69801[] = {SGI295336, SGI295337, NULL};
const Segment * const SG69802[] = {SGI295338, SGI295339, NULL};
const Segment * const SG69803[] = {SGI295340, SGI295341, NULL};
const Segment * const SG69804[] = {SGI295342, SGI295343, NULL};
const Segment * const SG69805[] = {SGI295344, SGI295345, NULL};
const Segment * const SG69806[] = {SGI295346, SGI295347, NULL};
const Segment * const SG69807[] = {SGI295348, SGI295349, NULL};
const Segment * const SG69808[] = {SGI295350, SGI295351, NULL};
const Segment * const SG69809[] = {SGI295352, SGI295353, NULL};
const Segment * const SG69810[] = {SGI295354, SGI295355, NULL};
const Segment * const SG69811[] = {SGI295356, SGI295357, NULL};
const Segment * const SG69812[] = {SGI295358, SGI295359, NULL};
const Segment * const SG69813[] = {SGI295360, SGI295361, NULL};
const Segment * const SG69814[] = {SGI0, SGI295362, SGI0, SGI295363, SGI295364, SGI295364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295365, SGI295366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69815[] = {SGI0, SGI295368, SGI0, SGI295369, SGI295370, SGI295371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69816[] = {SGI0, SGI295373, SGI0, SGI295374, SGI295375, SGI295376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295378, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69817[] = {SGI0, SGI295379, SGI0, SGI295380, SGI295381, SGI295381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295382, SGI295383, SGI0, SGI295384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69818[] = {SGI0, SGI295385, SGI0, SGI295386, SGI295387, SGI295388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295390, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69819[] = {SGI0, SGI295391, SGI0, SGI295392, SGI295393, SGI295394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295396, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69820[] = {SGI0, SGI295397, NULL};
const Segment * const SG69821[] = {SGI0, SGI295398, NULL};
const Segment * const SG69822[] = {SGI0, SGI295399, SGI0, SGI295400, SGI295401, SGI295402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295403, SGI295404, SGI0, SGI0, SGI0, SGI0, SGI295405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69823[] = {SGI0, SGI295406, SGI0, SGI295407, SGI295408, SGI295409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69824[] = {SGI0, SGI295411, SGI0, SGI295412, SGI295413, SGI295414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69825[] = {SGI0, SGI295416, SGI0, SGI295417, SGI295418, SGI295418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295419, SGI295420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69826[] = {SGI0, SGI295422, SGI0, SGI295423, SGI295424, SGI295425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295426, SGI295427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69827[] = {SGI0, SGI295429, SGI0, SGI295430, SGI295431, SGI295432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69828[] = {SGI0, SGI295434, SGI0, SGI295435, SGI295436, SGI295436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295437, SGI295438, SGI0, SGI295439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69829[] = {SGI0, SGI295440, SGI0, SGI295441, SGI295442, SGI295442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295443, SGI295444, SGI0, SGI295445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69830[] = {SGI0, SGI295446, SGI0, SGI295447, SGI295448, SGI295449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69831[] = {SGI0, SGI295451, SGI0, SGI295452, SGI295453, SGI295454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69832[] = {SGI0, SGI295456, NULL};
const Segment * const SG69833[] = {SGI295457, SGI295458, NULL};
const Segment * const SG69834[] = {SGI295459, SGI295460, NULL};
const Segment * const SG69835[] = {SGI0, SGI295461, NULL};
const Segment * const SG69836[] = {SGI295462, SGI295463, NULL};
const Segment * const SG69837[] = {SGI0, SGI295464, SGI0, SGI295465, SGI295466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295467, SGI295468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69838[] = {SGI0, SGI295469, SGI0, SGI295470, SGI295471, SGI295472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295473, SGI295474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69839[] = {SGI0, SGI295476, SGI0, SGI295477, SGI295478, SGI295479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295480, SGI295481, SGI0, SGI0, SGI0, SGI0, SGI295482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69840[] = {SGI0, SGI295483, SGI0, SGI295484, SGI295485, SGI295486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69841[] = {SGI0, SGI295488, SGI0, SGI295489, SGI295490, SGI295490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295491, SGI295492, SGI0, SGI0, SGI295493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69842[] = {SGI0, SGI295494, NULL};
const Segment * const SG69843[] = {SGI295495, SGI295496, NULL};
const Segment * const SG69844[] = {SGI0, SGI295497, NULL};
const Segment * const SG69845[] = {SGI295498, SGI295499, NULL};
const Segment * const SG69846[] = {SGI295500, SGI295501, NULL};
const Segment * const SG69847[] = {SGI0, SGI295502, NULL};
const Segment * const SG69848[] = {SGI0, SGI295503, NULL};
const Segment * const SG69849[] = {SGI0, SGI295504, NULL};
const Segment * const SG69850[] = {SGI0, SGI295505, NULL};
const Segment * const SG69851[] = {SGI0, SGI295506, NULL};
const Segment * const SG69852[] = {SGI0, SGI295507, NULL};
const Segment * const SG69853[] = {SGI295508, SGI295509, SGI295510, SGI295511, SGI0, SGI295509, SGI0, NULL};
const Segment * const SG69854[] = {SGI0, SGI295512, SGI0, SGI295513, SGI295514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295515, SGI295516, SGI0, SGI0, SGI0, SGI0, SGI295517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69855[] = {SGI0, SGI295518, SGI0, SGI295519, SGI295520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295521, SGI295522, SGI0, SGI0, SGI0, SGI0, SGI295523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69856[] = {SGI0, SGI295524, SGI0, SGI295525, SGI295526, SGI295527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295528, SGI295529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69857[] = {SGI0, SGI295530, SGI0, SGI295531, SGI295532, SGI295533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69858[] = {SGI0, SGI295535, SGI0, SGI295536, SGI295537, SGI295538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295539, SGI295540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69859[] = {SGI0, SGI295542, SGI0, SGI295543, SGI295544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295545, SGI295546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69860[] = {SGI0, SGI295547, SGI0, SGI295548, SGI295549, SGI295549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295550, SGI295551, SGI0, SGI295552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69861[] = {SGI0, SGI295553, SGI0, SGI295554, SGI295555, SGI295555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295556, SGI295557, SGI0, SGI295558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69862[] = {SGI0, SGI295559, SGI0, SGI295560, SGI295561, SGI295561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295562, SGI295563, SGI0, SGI295564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69863[] = {SGI0, SGI295565, SGI0, SGI295566, SGI295567, SGI295567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295568, SGI295569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295570, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69864[] = {SGI0, SGI295571, SGI0, SGI295572, SGI295573, SGI295573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295574, SGI295575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295576, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69865[] = {SGI295577, SGI295578, SGI295579, SGI295580, SGI0, SGI295578, SGI0, NULL};
const Segment * const SG69866[] = {SGI295581, SGI295582, SGI295583, SGI295584, SGI0, SGI295582, SGI0, NULL};
const Segment * const SG69867[] = {SGI295585, SGI295586, SGI295587, SGI295588, SGI0, SGI295586, SGI0, NULL};
const Segment * const SG69868[] = {SGI0, SGI295589, SGI0, SGI295590, SGI295591, SGI295592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295594, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69869[] = {SGI0, SGI295595, SGI0, SGI295596, SGI295597, SGI295598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295599, SGI295600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69870[] = {SGI0, SGI295602, SGI0, SGI295603, SGI295604, SGI295604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295605, SGI295606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295607, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69871[] = {SGI295608, SGI295609, SGI295610, SGI295611, SGI295612, SGI295609, SGI0, NULL};
const Segment * const SG69872[] = {SGI0, SGI295613, SGI0, SGI295614, SGI295615, SGI295616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295618, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69873[] = {SGI0, SGI295619, SGI0, SGI295620, SGI295621, SGI295622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295624, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69874[] = {SGI295625, SGI295626, NULL};
const Segment * const SG69875[] = {SGI295627, SGI295628, NULL};
const Segment * const SG69876[] = {SGI295629, SGI295630, NULL};
const Segment * const SG69877[] = {SGI295631, SGI295632, NULL};
const Segment * const SG69878[] = {SGI295633, SGI295634, NULL};
const Segment * const SG69879[] = {SGI295635, SGI295636, NULL};
const Segment * const SG69880[] = {SGI295637, SGI295638, NULL};
const Segment * const SG69881[] = {SGI0, SGI295639, SGI0, SGI295640, SGI295641, SGI295641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295642, SGI295643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69882[] = {SGI0, SGI295644, SGI0, SGI295645, SGI295646, SGI295647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69883[] = {SGI0, SGI295649, SGI0, SGI295650, SGI295651, SGI295652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295653, SGI295654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69884[] = {SGI0, SGI295655, SGI0, SGI295656, SGI295657, SGI295657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295658, SGI295659, SGI0, SGI0, SGI0, SGI295660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69885[] = {SGI0, SGI295661, SGI0, SGI295662, SGI295663, SGI295663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295664, SGI295665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295666, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69886[] = {SGI0, SGI295667, SGI0, SGI295668, SGI295669, SGI295669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295670, SGI295671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295672, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69887[] = {SGI0, SGI295673, SGI0, SGI295674, SGI295675, SGI295676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295678, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69888[] = {SGI0, SGI295679, SGI0, SGI295680, SGI295681, SGI295681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295682, SGI295683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295684, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69889[] = {SGI0, SGI295685, SGI0, SGI295686, SGI295687, SGI295687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295688, SGI295689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295690, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69890[] = {SGI0, SGI295691, SGI0, SGI295692, SGI295693, SGI295693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295694, SGI295695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69891[] = {SGI295697, SGI295698, SGI295699, SGI295700, SGI0, SGI295698, SGI0, NULL};
const Segment * const SG69892[] = {SGI295701, SGI295702, SGI295703, SGI295704, SGI0, SGI295702, SGI0, NULL};
const Segment * const SG69893[] = {SGI0, SGI295705, SGI0, SGI295706, SGI295707, SGI295708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295710, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69894[] = {SGI0, SGI295711, SGI0, SGI295712, SGI295713, SGI295713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295714, SGI295715, SGI0, SGI0, SGI0, SGI295716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69895[] = {SGI0, SGI295717, SGI0, SGI295718, SGI295719, SGI295719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295721, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69896[] = {SGI0, SGI295722, SGI0, SGI295723, SGI295724, SGI295725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295727, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69897[] = {SGI0, SGI295728, SGI0, SGI295729, SGI295730, SGI295730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295731, SGI295732, SGI0, SGI295733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69898[] = {SGI0, SGI295734, SGI0, SGI295735, SGI295736, SGI295736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295737, SGI295738, SGI0, SGI295739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69899[] = {SGI0, SGI295740, SGI0, SGI295741, SGI295742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295743, SGI295744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69900[] = {SGI0, SGI295745, SGI0, SGI295746, SGI295747, SGI295748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69901[] = {SGI0, SGI295750, SGI0, SGI295751, SGI295752, SGI295753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69902[] = {SGI0, SGI295756, SGI0, SGI295757, SGI295758, SGI295758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295759, SGI295760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295761, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69903[] = {SGI0, SGI295762, SGI0, SGI295763, SGI295764, SGI295765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295766, SGI295767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69904[] = {SGI0, SGI295768, SGI0, SGI295769, SGI295770, SGI295771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295772, SGI295773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69905[] = {SGI0, SGI295774, SGI0, SGI295775, SGI295776, SGI295776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295777, SGI295778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69906[] = {SGI0, SGI295779, NULL};
const Segment * const SG69907[] = {SGI0, SGI295780, SGI0, SGI295781, SGI295782, SGI295783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69908[] = {SGI0, SGI295785, SGI0, SGI295786, SGI295787, SGI295788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295789, SGI295790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69909[] = {SGI0, SGI295791, SGI0, SGI295792, SGI295793, SGI295793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295794, SGI295795, SGI0, SGI295796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69910[] = {SGI0, SGI295797, SGI0, SGI295798, SGI295799, SGI295799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295800, SGI295801, SGI0, SGI295802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69911[] = {SGI0, SGI0, SGI0, SGI295803, SGI295804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69912[] = {SGI0, SGI0, SGI0, SGI295805, SGI295806, SGI295807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69913[] = {SGI0, SGI295808, SGI0, SGI295809, SGI295810, SGI295811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295812, SGI295813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69914[] = {SGI0, SGI295814, NULL};
const Segment * const SG69915[] = {SGI0, SGI295815, NULL};
const Segment * const SG69916[] = {SGI0, SGI295816, NULL};
const Segment * const SG69917[] = {SGI0, SGI295817, NULL};
const Segment * const SG69918[] = {SGI0, SGI295818, NULL};
const Segment * const SG69919[] = {SGI0, SGI295819, NULL};
const Segment * const SG69920[] = {SGI0, SGI295820, NULL};
const Segment * const SG69921[] = {SGI0, SGI295821, NULL};
const Segment * const SG69922[] = {SGI0, SGI295822, NULL};
const Segment * const SG69923[] = {SGI0, SGI295823, SGI0, SGI295824, SGI295825, SGI295826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69924[] = {SGI0, SGI295828, NULL};
const Segment * const SG69925[] = {SGI0, SGI295829, NULL};
const Segment * const SG69926[] = {SGI0, SGI295830, NULL};
const Segment * const SG69927[] = {SGI0, SGI295831, NULL};
const Segment * const SG69928[] = {SGI0, SGI295832, NULL};
const Segment * const SG69929[] = {SGI0, SGI295833, NULL};
const Segment * const SG69930[] = {SGI0, SGI295834, NULL};
const Segment * const SG69931[] = {SGI0, SGI295835, NULL};
const Segment * const SG69932[] = {SGI0, SGI295836, NULL};
const Segment * const SG69933[] = {SGI0, SGI295837, NULL};
const Segment * const SG69934[] = {SGI0, SGI295838, NULL};
const Segment * const SG69935[] = {SGI0, SGI295839, NULL};
const Segment * const SG69936[] = {SGI0, SGI295844, NULL};
const Segment * const SG69937[] = {SGI0, SGI295840, SGI0, SGI295841, SGI295842, SGI295843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69938[] = {SGI0, SGI295847, NULL};
const Segment * const SG69939[] = {SGI295848, SGI295849, SGI295850, SGI295851, SGI0, SGI295849, SGI0, NULL};
const Segment * const SG69940[] = {SGI295852, SGI295853, SGI295854, SGI295855, SGI0, SGI295856, SGI0, NULL};
const Segment * const SG69941[] = {SGI0, SGI295857, SGI0, SGI295858, SGI295859, SGI295859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295860, SGI295861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295862, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69942[] = {SGI0, SGI295863, SGI0, SGI295864, SGI295865, SGI295865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295866, SGI295867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69943[] = {SGI295868, SGI0, SGI0, SGI295869, SGI295870, SGI295871, SGI0, NULL};
const Segment * const SG69944[] = {SGI0, SGI295872, SGI0, SGI295873, SGI295874, SGI295874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295875, SGI295876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295877, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69945[] = {SGI0, SGI295878, SGI0, SGI295879, SGI295880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295881, SGI295882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69946[] = {SGI0, SGI295883, SGI0, SGI295884, SGI295885, SGI295886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295887, SGI295888, SGI0, SGI0, SGI0, SGI295889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69947[] = {SGI0, SGI295890, SGI0, SGI295891, SGI295892, SGI295893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295894, SGI295895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69948[] = {SGI295896, SGI295897, NULL};
const Segment * const SG69949[] = {SGI0, SGI295898, NULL};
const Segment * const SG69950[] = {SGI295899, SGI295900, NULL};
const Segment * const SG69951[] = {SGI0, SGI295901, NULL};
const Segment * const SG69952[] = {SGI295902, SGI295903, NULL};
const Segment * const SG69953[] = {SGI0, SGI295904, NULL};
const Segment * const SG69954[] = {SGI0, SGI295905, NULL};
const Segment * const SG69955[] = {SGI0, SGI295906, SGI0, SGI295907, SGI295908, SGI295908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295909, SGI295910, SGI0, SGI295911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69956[] = {SGI0, SGI295912, SGI0, SGI295913, SGI295914, SGI295914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295915, SGI295916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69957[] = {SGI0, SGI295918, SGI0, SGI295919, SGI295920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295921, SGI295922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69958[] = {SGI0, SGI295923, SGI0, SGI295924, SGI295925, SGI295926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295927, SGI295928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69959[] = {SGI0, SGI295929, SGI0, SGI295930, SGI295931, SGI295931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295932, SGI295933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69960[] = {SGI0, SGI295935, SGI0, SGI295936, SGI295937, SGI295938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295939, SGI295940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69961[] = {SGI295941, SGI295942, NULL};
const Segment * const SG69962[] = {SGI295943, SGI295944, NULL};
const Segment * const SG69963[] = {SGI295945, SGI295946, NULL};
const Segment * const SG69964[] = {SGI0, SGI295947, NULL};
const Segment * const SG69965[] = {SGI0, SGI295948, NULL};
const Segment * const SG69966[] = {SGI295949, SGI295950, NULL};
const Segment * const SG69967[] = {SGI0, SGI295951, NULL};
const Segment * const SG69968[] = {SGI0, SGI295952, NULL};
const Segment * const SG69969[] = {SGI295953, SGI295954, NULL};
const Segment * const SG69970[] = {SGI295955, SGI295956, NULL};
const Segment * const SG69971[] = {SGI0, SGI295957, SGI0, SGI295958, SGI295959, SGI295960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69972[] = {SGI0, SGI295962, SGI0, SGI295963, SGI295964, SGI295964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295965, SGI295966, SGI0, SGI295967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69973[] = {SGI0, SGI295968, SGI0, SGI295969, SGI295970, SGI295970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295971, SGI295972, SGI0, SGI295973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69974[] = {SGI0, SGI295974, SGI0, SGI295975, SGI295976, SGI295977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295978, SGI295979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69975[] = {SGI295980, SGI295981, SGI295982, SGI295983, SGI0, SGI295981, SGI0, NULL};
const Segment * const SG69976[] = {SGI0, SGI295984, SGI0, SGI295985, SGI295986, SGI295987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295989, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69977[] = {SGI0, SGI295990, SGI0, SGI295991, SGI295992, SGI295992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295993, SGI295994, SGI0, SGI295995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69978[] = {SGI0, SGI295996, SGI0, SGI295997, SGI295998, SGI295998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI295999, SGI296000, SGI0, SGI296001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69979[] = {SGI296002, SGI296003, NULL};
const Segment * const SG69980[] = {SGI296004, SGI296005, NULL};
const Segment * const SG69981[] = {SGI296006, SGI296007, NULL};
const Segment * const SG69982[] = {SGI0, SGI296008, SGI0, SGI296009, SGI296010, SGI296011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296013, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69983[] = {SGI0, SGI0, SGI0, SGI296014, SGI296015, SGI296016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296018, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69984[] = {SGI0, SGI296019, SGI0, SGI296020, SGI296021, SGI296022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69985[] = {SGI296025, SGI296026, NULL};
const Segment * const SG69986[] = {SGI0, SGI296027, NULL};
const Segment * const SG69987[] = {SGI0, SGI296028, NULL};
const Segment * const SG69988[] = {SGI0, SGI296029, NULL};
const Segment * const SG69989[] = {SGI296030, SGI296031, NULL};
const Segment * const SG69990[] = {SGI0, SGI296032, NULL};
const Segment * const SG69991[] = {SGI0, SGI296033, NULL};
const Segment * const SG69992[] = {SGI0, SGI296034, NULL};
const Segment * const SG69993[] = {SGI0, SGI296035, NULL};
const Segment * const SG69994[] = {SGI0, SGI296036, SGI0, SGI296037, SGI296038, SGI296039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69995[] = {SGI0, SGI296041, SGI0, SGI296042, SGI296043, SGI296043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296044, SGI296045, SGI0, SGI296046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69996[] = {SGI0, SGI296047, SGI0, SGI296048, SGI296049, SGI296049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296050, SGI296051, SGI0, SGI296052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69997[] = {SGI0, SGI296053, SGI0, SGI296054, SGI296055, SGI296056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296058, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG69998[] = {SGI0, SGI296059, NULL};
const Segment * const SG69999[] = {SGI296060, SGI296061, NULL};
const Segment * const SG70000[] = {SGI296062, SGI296063, NULL};
const Segment * const SG70001[] = {SGI0, SGI296064, NULL};
const Segment * const SG70002[] = {SGI296065, SGI296066, NULL};
const Segment * const SG70003[] = {SGI0, SGI296070, NULL};
const Segment * const SG70004[] = {SGI0, SGI296067, SGI0, SGI296068, SGI296069, SGI296069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296071, SGI296072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296073, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70005[] = {SGI296074, SGI296075, SGI296076, SGI296077, SGI296078, SGI296075, SGI0, NULL};
const Segment * const SG70006[] = {SGI296079, SGI296080, NULL};
const Segment * const SG70007[] = {SGI296081, SGI296082, NULL};
const Segment * const SG70008[] = {SGI296083, SGI296084, NULL};
const Segment * const SG70009[] = {SGI296085, SGI296086, NULL};
const Segment * const SG70010[] = {SGI296087, SGI296088, NULL};
const Segment * const SG70011[] = {SGI296089, SGI296090, NULL};
const Segment * const SG70012[] = {SGI296091, SGI296092, NULL};
const Segment * const SG70013[] = {SGI296093, SGI296094, NULL};
const Segment * const SG70014[] = {SGI0, SGI296095, SGI0, SGI296096, SGI296097, SGI296098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70015[] = {SGI0, SGI296100, SGI0, SGI296101, SGI296102, SGI296102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296103, SGI296104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296105, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70016[] = {SGI0, SGI296106, SGI0, SGI296107, SGI296108, SGI296109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70017[] = {SGI0, SGI296112, SGI0, SGI296113, SGI296114, SGI296115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296116, SGI296117, SGI0, SGI0, SGI0, SGI0, SGI296118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70018[] = {SGI0, SGI296119, SGI0, SGI296120, SGI296121, SGI296122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296123, SGI296124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70019[] = {SGI0, SGI296125, SGI0, SGI296126, SGI296127, SGI296128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296129, SGI296130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70020[] = {SGI0, SGI296131, SGI0, SGI296132, SGI296133, SGI296134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296135, SGI296136, SGI0, SGI0, SGI0, SGI0, SGI296137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70021[] = {SGI0, SGI296138, SGI0, SGI296139, SGI296140, SGI296140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296142, SGI0, NULL};
const Segment * const SG70022[] = {SGI0, SGI296143, SGI0, SGI296144, SGI296145, SGI296145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296146, SGI296147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296148, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70023[] = {SGI0, SGI296149, SGI0, SGI296150, SGI296151, SGI296152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70024[] = {SGI0, SGI296154, SGI0, SGI296155, SGI296156, SGI296157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70025[] = {SGI0, SGI296159, SGI0, SGI296160, SGI296161, SGI296162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70026[] = {SGI0, SGI296165, SGI0, SGI296166, SGI296167, SGI296168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296170, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70027[] = {SGI0, SGI296171, SGI0, SGI296172, SGI296173, SGI296174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296176, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70028[] = {SGI0, SGI296177, SGI0, SGI296178, SGI296179, SGI296179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296180, SGI296181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296182, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70029[] = {SGI0, SGI296183, NULL};
const Segment * const SG70030[] = {SGI0, SGI296184, NULL};
const Segment * const SG70031[] = {SGI296185, SGI296186, NULL};
const Segment * const SG70032[] = {SGI296187, SGI296188, SGI296189, SGI296190, SGI0, SGI296191, SGI0, NULL};
const Segment * const SG70033[] = {SGI0, SGI296192, SGI0, SGI296193, SGI296194, SGI296195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70034[] = {SGI0, SGI296198, SGI0, SGI296199, SGI296200, SGI296201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70035[] = {SGI0, SGI296204, NULL};
const Segment * const SG70036[] = {SGI0, SGI296205, SGI0, SGI296206, SGI296207, SGI296208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296210, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70037[] = {SGI0, SGI296211, SGI0, SGI296212, SGI296213, SGI296213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296214, SGI296215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296216, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70038[] = {SGI0, SGI296217, NULL};
const Segment * const SG70039[] = {SGI0, SGI296218, NULL};
const Segment * const SG70040[] = {SGI0, SGI296219, NULL};
const Segment * const SG70041[] = {SGI0, SGI296220, NULL};
const Segment * const SG70042[] = {SGI0, SGI296221, NULL};
const Segment * const SG70043[] = {SGI0, SGI296222, NULL};
const Segment * const SG70044[] = {SGI0, SGI296223, NULL};
const Segment * const SG70045[] = {SGI0, SGI296224, NULL};
const Segment * const SG70046[] = {SGI0, SGI296225, NULL};
const Segment * const SG70047[] = {SGI0, SGI296226, NULL};
const Segment * const SG70048[] = {SGI0, SGI296227, NULL};
const Segment * const SG70049[] = {SGI0, SGI296228, NULL};
const Segment * const SG70050[] = {SGI0, SGI296229, NULL};
const Segment * const SG70051[] = {SGI0, SGI296230, NULL};
const Segment * const SG70052[] = {SGI0, SGI296231, NULL};
const Segment * const SG70053[] = {SGI0, SGI296232, NULL};
const Segment * const SG70054[] = {SGI0, SGI296233, NULL};
const Segment * const SG70055[] = {SGI0, SGI296234, NULL};
const Segment * const SG70056[] = {SGI0, SGI296235, NULL};
const Segment * const SG70057[] = {SGI0, SGI296236, NULL};
const Segment * const SG70058[] = {SGI296237, NULL};
const Segment * const SG70059[] = {SGI0, SGI296238, SGI0, SGI296239, SGI296240, SGI296240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296241, SGI296242, SGI0, SGI296243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70060[] = {SGI0, SGI296244, SGI0, SGI296245, SGI296246, SGI296246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296247, SGI296248, SGI0, SGI296249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70061[] = {SGI0, SGI296250, SGI0, SGI296251, SGI296252, SGI296252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296253, SGI296254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296255, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70062[] = {SGI0, SGI296256, SGI0, SGI296257, SGI296258, SGI296259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70063[] = {SGI0, SGI296261, SGI0, SGI296262, SGI296263, SGI296263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296265, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70064[] = {SGI0, SGI296266, SGI0, SGI296267, SGI296268, SGI296268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70065[] = {SGI0, SGI296271, SGI0, SGI296272, SGI296273, SGI296273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296274, SGI296275, SGI0, SGI0, SGI0, SGI296276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70066[] = {SGI0, SGI296277, NULL};
const Segment * const SG70067[] = {SGI0, SGI296282, NULL};
const Segment * const SG70068[] = {SGI0, SGI296278, SGI0, SGI296279, SGI296280, SGI296280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296281, SGI296283, SGI0, SGI0, SGI0, SGI296284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70069[] = {SGI0, SGI296285, NULL};
const Segment * const SG70070[] = {SGI0, SGI296286, NULL};
const Segment * const SG70071[] = {SGI0, SGI296293, NULL};
const Segment * const SG70072[] = {SGI0, SGI296287, SGI0, SGI296288, SGI296289, SGI296289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296290, SGI296291, SGI0, SGI296292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70073[] = {SGI0, SGI296294, NULL};
const Segment * const SG70074[] = {SGI0, SGI296301, NULL};
const Segment * const SG70075[] = {SGI0, SGI296295, SGI0, SGI296296, SGI296297, SGI296297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296298, SGI296299, SGI0, SGI296300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70076[] = {SGI0, SGI296302, NULL};
const Segment * const SG70077[] = {SGI0, SGI296303, NULL};
const Segment * const SG70078[] = {SGI0, SGI296304, NULL};
const Segment * const SG70079[] = {SGI0, SGI296311, NULL};
const Segment * const SG70080[] = {SGI0, SGI296305, SGI0, SGI296306, SGI296307, SGI296307, SGI296308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296309, SGI0, SGI296310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70081[] = {SGI0, SGI296312, NULL};
const Segment * const SG70082[] = {SGI0, SGI296313, NULL};
const Segment * const SG70083[] = {SGI0, SGI296314, NULL};
const Segment * const SG70084[] = {SGI0, SGI296315, NULL};
const Segment * const SG70085[] = {SGI0, SGI296316, NULL};
const Segment * const SG70086[] = {SGI0, SGI296317, NULL};
const Segment * const SG70087[] = {SGI0, SGI296318, SGI0, SGI296319, SGI296320, SGI296320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296321, SGI296322, SGI0, SGI296323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70088[] = {SGI0, SGI296324, NULL};
const Segment * const SG70089[] = {SGI0, SGI296325, NULL};
const Segment * const SG70090[] = {SGI0, SGI296326, SGI0, SGI296327, SGI296328, SGI296328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296329, SGI296330, SGI0, SGI296331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70091[] = {SGI0, SGI296332, SGI0, SGI296333, SGI296334, SGI296334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296335, SGI296336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296337, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70092[] = {SGI296338, SGI0, SGI0, SGI296339, SGI296340, SGI296341, SGI0, NULL};
const Segment * const SG70093[] = {SGI0, SGI296342, SGI0, SGI296343, SGI296344, SGI296344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296345, SGI296346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296347, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70094[] = {SGI0, SGI296348, SGI0, SGI296349, SGI296350, SGI296350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296351, SGI296352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296353, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70095[] = {SGI0, SGI296354, NULL};
const Segment * const SG70096[] = {SGI0, SGI296355, NULL};
const Segment * const SG70097[] = {SGI0, SGI296356, NULL};
const Segment * const SG70098[] = {SGI0, SGI296357, NULL};
const Segment * const SG70099[] = {SGI0, SGI296358, NULL};
const Segment * const SG70100[] = {SGI0, SGI296359, SGI0, SGI296360, SGI296361, SGI296362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296363, SGI296364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70101[] = {SGI296365, SGI296366, NULL};
const Segment * const SG70102[] = {SGI296367, SGI296368, NULL};
const Segment * const SG70103[] = {SGI0, SGI296369, SGI0, SGI296370, SGI296371, SGI296372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296373, SGI296374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70104[] = {SGI296375, SGI296376, NULL};
const Segment * const SG70105[] = {SGI0, SGI296377, SGI0, SGI296378, SGI296379, SGI296380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70106[] = {SGI0, SGI296382, SGI0, SGI296383, SGI296384, SGI296385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70107[] = {SGI0, SGI296387, SGI0, SGI296388, SGI296389, SGI296389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296390, SGI296391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70108[] = {SGI0, SGI296393, SGI0, SGI296394, SGI296395, SGI296396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296397, SGI296398, SGI0, SGI0, SGI296399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70109[] = {SGI0, SGI296400, SGI0, SGI296401, SGI296402, SGI296403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296404, SGI296405, SGI0, SGI0, SGI0, SGI0, SGI296406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70110[] = {SGI0, SGI296407, SGI0, SGI296408, SGI296409, SGI296410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70111[] = {SGI0, SGI296412, SGI0, SGI296413, SGI296414, SGI296415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70112[] = {SGI0, SGI296417, SGI0, SGI296418, SGI296419, SGI296419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296420, SGI296421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70113[] = {SGI296423, SGI296424, NULL};
const Segment * const SG70114[] = {SGI296425, SGI296426, NULL};
const Segment * const SG70115[] = {SGI296427, SGI296428, NULL};
const Segment * const SG70116[] = {SGI296429, SGI296430, NULL};
const Segment * const SG70117[] = {SGI296431, SGI296432, NULL};
const Segment * const SG70118[] = {SGI296433, SGI296434, NULL};
const Segment * const SG70119[] = {SGI296435, SGI296436, NULL};
const Segment * const SG70120[] = {SGI296437, SGI296438, NULL};
const Segment * const SG70121[] = {SGI296439, SGI296440, NULL};
const Segment * const SG70122[] = {SGI296441, SGI296442, NULL};
const Segment * const SG70123[] = {SGI296443, SGI296444, NULL};
const Segment * const SG70124[] = {SGI0, SGI296445, SGI0, SGI296446, SGI296447, SGI296448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296449, SGI296450, SGI0, SGI0, SGI296451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70125[] = {SGI0, SGI296452, SGI0, SGI296453, SGI296454, SGI296455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70126[] = {SGI296457, SGI296458, NULL};
const Segment * const SG70127[] = {SGI296459, SGI296460, NULL};
const Segment * const SG70128[] = {SGI296461, SGI296462, NULL};
const Segment * const SG70129[] = {SGI296463, SGI296464, NULL};
const Segment * const SG70130[] = {SGI296465, SGI296466, NULL};
const Segment * const SG70131[] = {SGI296467, SGI296468, NULL};
const Segment * const SG70132[] = {SGI296469, SGI296470, NULL};
const Segment * const SG70133[] = {SGI296471, SGI296472, NULL};
const Segment * const SG70134[] = {SGI296473, SGI296474, NULL};
const Segment * const SG70135[] = {SGI296475, SGI296476, NULL};
const Segment * const SG70136[] = {SGI296477, SGI296478, NULL};
const Segment * const SG70137[] = {SGI0, SGI0, SGI0, SGI296479, SGI296480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296481, SGI296482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70138[] = {SGI0, SGI296483, SGI0, SGI296484, SGI296485, SGI296485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296486, SGI296487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70139[] = {SGI0, SGI296488, SGI0, SGI296489, SGI296490, SGI296490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296491, SGI296492, SGI0, SGI296493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70140[] = {SGI0, SGI296494, SGI0, SGI296495, SGI296496, SGI296496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296497, SGI296498, SGI0, SGI296499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70141[] = {SGI0, SGI296500, SGI0, SGI296501, SGI296502, SGI296503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70142[] = {SGI0, SGI296505, SGI0, SGI296506, SGI296507, SGI296508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296509, SGI296510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70143[] = {SGI0, SGI296512, SGI0, SGI296513, SGI296514, SGI296515, SGI0, SGI0, SGI0, SGI0, SGI296516, SGI0, SGI296517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70144[] = {SGI0, SGI296518, SGI0, SGI296519, SGI296520, SGI296521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296522, SGI296523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70145[] = {SGI0, SGI296524, NULL};
const Segment * const SG70146[] = {SGI0, SGI296525, NULL};
const Segment * const SG70147[] = {SGI0, SGI296526, NULL};
const Segment * const SG70148[] = {SGI0, SGI296527, NULL};
const Segment * const SG70149[] = {SGI0, SGI296528, NULL};
const Segment * const SG70150[] = {SGI0, SGI296529, NULL};
const Segment * const SG70151[] = {SGI0, SGI296530, SGI0, SGI296531, SGI296532, SGI296533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70152[] = {SGI0, SGI296535, SGI0, SGI296536, SGI296537, SGI296538, SGI0, SGI0, SGI0, SGI0, SGI296539, SGI0, SGI296540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70153[] = {SGI0, SGI296541, SGI0, SGI296542, SGI296543, SGI296544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296545, SGI296546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70154[] = {SGI0, SGI296547, NULL};
const Segment * const SG70155[] = {SGI0, SGI296548, NULL};
const Segment * const SG70156[] = {SGI0, SGI296549, SGI0, SGI296550, SGI296551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296552, SGI296553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70157[] = {SGI0, SGI296554, SGI0, SGI296555, SGI296556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296557, SGI296558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70158[] = {SGI296559, SGI296560, NULL};
const Segment * const SG70159[] = {SGI0, SGI296561, NULL};
const Segment * const SG70160[] = {SGI296562, SGI296563, NULL};
const Segment * const SG70161[] = {SGI296564, SGI296565, NULL};
const Segment * const SG70162[] = {SGI0, SGI296566, NULL};
const Segment * const SG70163[] = {SGI0, SGI296567, SGI0, SGI296568, SGI296569, SGI296570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70164[] = {SGI0, SGI296572, NULL};
const Segment * const SG70165[] = {SGI0, SGI296573, NULL};
const Segment * const SG70166[] = {SGI0, SGI296574, NULL};
const Segment * const SG70167[] = {SGI0, SGI296575, NULL};
const Segment * const SG70168[] = {SGI0, SGI296576, NULL};
const Segment * const SG70169[] = {SGI0, SGI296577, NULL};
const Segment * const SG70170[] = {SGI0, SGI296578, NULL};
const Segment * const SG70171[] = {SGI0, SGI296579, NULL};
const Segment * const SG70172[] = {SGI0, SGI296580, NULL};
const Segment * const SG70173[] = {SGI0, SGI296581, NULL};
const Segment * const SG70174[] = {SGI0, SGI296582, NULL};
const Segment * const SG70175[] = {SGI296583, SGI296584, SGI296585, SGI296586, SGI0, SGI296587, SGI0, NULL};
const Segment * const SG70176[] = {SGI0, SGI296588, NULL};
const Segment * const SG70177[] = {SGI296589, SGI296590, NULL};
const Segment * const SG70178[] = {SGI296591, SGI296592, NULL};
const Segment * const SG70179[] = {SGI296593, SGI296594, NULL};
const Segment * const SG70180[] = {SGI0, SGI296595, SGI0, SGI296596, SGI296597, SGI296598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70181[] = {SGI0, SGI296600, SGI0, SGI296601, SGI296602, SGI296602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296603, SGI296604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70182[] = {SGI0, SGI296606, SGI0, SGI296607, SGI296608, SGI296608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296609, SGI296610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296611, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70183[] = {SGI0, SGI296612, NULL};
const Segment * const SG70184[] = {SGI0, SGI296613, NULL};
const Segment * const SG70185[] = {SGI296614, SGI296615, NULL};
const Segment * const SG70186[] = {SGI0, SGI296616, NULL};
const Segment * const SG70187[] = {SGI0, SGI296617, NULL};
const Segment * const SG70188[] = {SGI0, SGI296618, NULL};
const Segment * const SG70189[] = {SGI296619, SGI296620, NULL};
const Segment * const SG70190[] = {SGI0, SGI296621, NULL};
const Segment * const SG70191[] = {SGI0, SGI296622, NULL};
const Segment * const SG70192[] = {SGI296623, SGI296624, NULL};
const Segment * const SG70193[] = {SGI0, SGI296625, NULL};
const Segment * const SG70194[] = {SGI0, SGI296626, NULL};
const Segment * const SG70195[] = {SGI296627, SGI296628, NULL};
const Segment * const SG70196[] = {SGI0, SGI296629, SGI0, SGI296630, SGI296631, SGI296631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296632, SGI296633, SGI0, SGI0, SGI0, SGI296634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70197[] = {SGI0, SGI296635, SGI0, SGI296636, SGI296637, SGI296637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296638, SGI296639, SGI0, SGI0, SGI0, SGI296640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70198[] = {SGI0, SGI296641, SGI0, SGI296642, SGI296643, SGI296644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296646, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70199[] = {SGI0, SGI296647, SGI0, SGI296648, SGI296649, SGI296650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70200[] = {SGI0, SGI296652, SGI0, SGI296653, SGI296654, SGI296655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296656, SGI296657, SGI0, SGI0, SGI0, SGI0, SGI296658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70201[] = {SGI0, SGI296659, SGI0, SGI296660, SGI296661, SGI296661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296662, SGI296663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70202[] = {SGI0, SGI296664, SGI0, SGI296665, SGI296666, SGI296666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296667, SGI296668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70203[] = {SGI0, SGI296669, SGI0, SGI296670, SGI296671, SGI296671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296672, SGI296673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70204[] = {SGI296675, SGI296676, NULL};
const Segment * const SG70205[] = {SGI0, SGI296677, SGI0, SGI296678, SGI296679, SGI296680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296682, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70206[] = {SGI0, SGI296683, NULL};
const Segment * const SG70207[] = {SGI0, SGI296684, NULL};
const Segment * const SG70208[] = {SGI0, SGI296685, NULL};
const Segment * const SG70209[] = {SGI0, SGI296686, NULL};
const Segment * const SG70210[] = {SGI0, SGI296687, NULL};
const Segment * const SG70211[] = {SGI0, SGI296688, NULL};
const Segment * const SG70212[] = {SGI0, SGI296689, NULL};
const Segment * const SG70213[] = {SGI0, SGI296690, NULL};
const Segment * const SG70214[] = {SGI0, SGI296691, NULL};
const Segment * const SG70215[] = {SGI0, SGI296692, NULL};
const Segment * const SG70216[] = {SGI0, SGI296693, SGI0, SGI296694, SGI296695, SGI296696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296698, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70217[] = {SGI0, SGI296699, NULL};
const Segment * const SG70218[] = {SGI296700, SGI296701, NULL};
const Segment * const SG70219[] = {SGI296702, SGI296703, NULL};
const Segment * const SG70220[] = {SGI296704, SGI296705, NULL};
const Segment * const SG70221[] = {SGI296706, SGI296707, NULL};
const Segment * const SG70222[] = {SGI296708, SGI296709, NULL};
const Segment * const SG70223[] = {SGI296710, SGI296711, NULL};
const Segment * const SG70224[] = {SGI296712, SGI296713, NULL};
const Segment * const SG70225[] = {SGI296714, SGI296715, NULL};
const Segment * const SG70226[] = {SGI296716, SGI296717, NULL};
const Segment * const SG70227[] = {SGI296718, SGI296719, NULL};
const Segment * const SG70228[] = {SGI0, SGI296720, SGI0, SGI296721, SGI296722, SGI296722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296723, SGI296724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70229[] = {SGI0, SGI296726, SGI0, SGI296727, SGI296728, SGI296729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70230[] = {SGI0, SGI296731, SGI0, SGI296732, SGI296733, SGI296734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296735, SGI296736, SGI0, SGI296737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70231[] = {SGI0, SGI296738, NULL};
const Segment * const SG70232[] = {SGI0, SGI296739, NULL};
const Segment * const SG70233[] = {SGI0, SGI296740, NULL};
const Segment * const SG70234[] = {SGI0, SGI296741, NULL};
const Segment * const SG70235[] = {SGI0, SGI296742, NULL};
const Segment * const SG70236[] = {SGI0, SGI296743, NULL};
const Segment * const SG70237[] = {SGI0, SGI296744, NULL};
const Segment * const SG70238[] = {SGI0, SGI296745, NULL};
const Segment * const SG70239[] = {SGI0, SGI296746, NULL};
const Segment * const SG70240[] = {SGI296747, SGI296748, NULL};
const Segment * const SG70241[] = {SGI296749, SGI296750, NULL};
const Segment * const SG70242[] = {SGI296751, SGI296752, NULL};
const Segment * const SG70243[] = {SGI0, SGI296753, NULL};
const Segment * const SG70244[] = {SGI296754, SGI296755, NULL};
const Segment * const SG70245[] = {SGI0, SGI296756, SGI0, SGI296757, SGI296758, SGI296759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70246[] = {SGI0, SGI296761, NULL};
const Segment * const SG70247[] = {SGI0, SGI296762, NULL};
const Segment * const SG70248[] = {SGI0, SGI296763, NULL};
const Segment * const SG70249[] = {SGI0, SGI296764, SGI0, SGI296765, SGI296766, SGI296767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296769, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70250[] = {SGI0, SGI296770, SGI0, SGI296771, SGI296772, SGI296772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296773, SGI296774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70251[] = {SGI296776, SGI296777, NULL};
const Segment * const SG70252[] = {SGI0, SGI296778, NULL};
const Segment * const SG70253[] = {SGI0, SGI296779, NULL};
const Segment * const SG70254[] = {SGI0, SGI296780, NULL};
const Segment * const SG70255[] = {SGI0, SGI296781, NULL};
const Segment * const SG70256[] = {SGI296782, SGI296783, NULL};
const Segment * const SG70257[] = {SGI296784, SGI296785, NULL};
const Segment * const SG70258[] = {SGI0, SGI296786, NULL};
const Segment * const SG70259[] = {SGI0, SGI296787, SGI0, SGI296788, SGI296789, SGI296790, SGI0, SGI0, SGI0, SGI0, SGI296791, SGI0, SGI296792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70260[] = {SGI296793, SGI296794, NULL};
const Segment * const SG70261[] = {SGI296795, SGI296796, NULL};
const Segment * const SG70262[] = {SGI296797, SGI296798, NULL};
const Segment * const SG70263[] = {SGI0, SGI296799, NULL};
const Segment * const SG70264[] = {SGI0, SGI296800, NULL};
const Segment * const SG70265[] = {SGI0, SGI296801, NULL};
const Segment * const SG70266[] = {SGI0, SGI296802, NULL};
const Segment * const SG70267[] = {SGI0, SGI296803, SGI0, SGI296804, SGI296805, SGI296806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296807, SGI296808, SGI0, SGI0, SGI0, SGI0, SGI296809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70268[] = {SGI0, SGI296810, NULL};
const Segment * const SG70269[] = {SGI0, SGI296811, NULL};
const Segment * const SG70270[] = {SGI296812, SGI296813, SGI296814, SGI296815, SGI0, SGI296816, SGI0, NULL};
const Segment * const SG70271[] = {SGI0, SGI296817, NULL};
const Segment * const SG70272[] = {SGI0, SGI296818, NULL};
const Segment * const SG70273[] = {SGI0, SGI296819, NULL};
const Segment * const SG70274[] = {SGI0, SGI296820, NULL};
const Segment * const SG70275[] = {SGI0, SGI296821, SGI0, SGI296822, SGI296823, SGI296824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70276[] = {SGI0, SGI296827, SGI0, SGI296828, SGI296829, SGI296830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296832, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70277[] = {SGI0, SGI296833, NULL};
const Segment * const SG70278[] = {SGI0, SGI296834, NULL};
const Segment * const SG70279[] = {SGI0, SGI296835, NULL};
const Segment * const SG70280[] = {SGI296836, SGI296837, NULL};
const Segment * const SG70281[] = {SGI0, SGI296838, SGI0, SGI296839, SGI296840, SGI296840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296841, SGI296842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296843, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70282[] = {SGI0, SGI296844, NULL};
const Segment * const SG70283[] = {SGI0, SGI296845, SGI0, SGI296846, SGI296847, SGI296848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296849, SGI296850, SGI0, SGI0, SGI296851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70284[] = {SGI0, SGI296852, SGI0, SGI296853, SGI296854, SGI296854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296855, SGI296856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296857, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70285[] = {SGI0, SGI296858, SGI0, SGI296859, SGI296860, SGI296860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296861, SGI296862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70286[] = {SGI0, SGI296863, SGI0, SGI296864, SGI296865, SGI296865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296866, SGI296867, SGI0, SGI296868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70287[] = {SGI0, SGI296869, SGI0, SGI296870, SGI296871, SGI296872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70288[] = {SGI0, SGI296874, NULL};
const Segment * const SG70289[] = {SGI0, SGI296875, NULL};
const Segment * const SG70290[] = {SGI0, SGI296876, NULL};
const Segment * const SG70291[] = {SGI0, SGI296877, NULL};
const Segment * const SG70292[] = {SGI0, SGI296878, NULL};
const Segment * const SG70293[] = {SGI0, SGI296879, NULL};
const Segment * const SG70294[] = {SGI0, SGI296880, NULL};
const Segment * const SG70295[] = {SGI0, SGI296881, NULL};
const Segment * const SG70296[] = {SGI0, SGI296882, NULL};
const Segment * const SG70297[] = {SGI0, SGI296883, NULL};
const Segment * const SG70298[] = {SGI0, SGI296884, SGI0, SGI296885, SGI296886, SGI296887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296889, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70299[] = {SGI0, SGI296890, NULL};
const Segment * const SG70300[] = {SGI0, SGI296891, SGI0, SGI296892, SGI296893, SGI296894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296896, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70301[] = {SGI0, SGI296897, SGI0, SGI296898, SGI296899, SGI296900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296902, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70302[] = {SGI0, SGI296903, SGI0, SGI296904, SGI296905, SGI296905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296906, SGI296907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70303[] = {SGI0, SGI296909, NULL};
const Segment * const SG70304[] = {SGI0, SGI296910, NULL};
const Segment * const SG70305[] = {SGI0, SGI296911, NULL};
const Segment * const SG70306[] = {SGI0, SGI296912, SGI0, SGI296913, SGI296914, SGI296915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296917, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70307[] = {SGI296918, SGI296919, NULL};
const Segment * const SG70308[] = {SGI296920, SGI296921, NULL};
const Segment * const SG70309[] = {SGI0, SGI296922, SGI0, SGI296923, SGI296924, SGI296925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70310[] = {SGI0, SGI296927, NULL};
const Segment * const SG70311[] = {SGI0, SGI296928, NULL};
const Segment * const SG70312[] = {SGI0, SGI296929, SGI0, SGI296930, SGI296931, SGI296932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70313[] = {SGI0, SGI296935, SGI0, SGI296936, SGI296937, SGI296938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70314[] = {SGI296940, SGI296941, NULL};
const Segment * const SG70315[] = {SGI296942, SGI296943, NULL};
const Segment * const SG70316[] = {SGI296944, SGI296945, NULL};
const Segment * const SG70317[] = {SGI296946, SGI296947, NULL};
const Segment * const SG70318[] = {SGI296948, SGI296949, NULL};
const Segment * const SG70319[] = {SGI296950, SGI296951, NULL};
const Segment * const SG70320[] = {SGI296952, SGI296953, NULL};
const Segment * const SG70321[] = {SGI296954, SGI296955, NULL};
const Segment * const SG70322[] = {SGI296956, SGI296957, NULL};
const Segment * const SG70323[] = {SGI0, SGI296958, SGI0, SGI296959, SGI296960, SGI296961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70324[] = {SGI0, SGI296963, SGI0, SGI296964, SGI296965, SGI296965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296967, NULL};
const Segment * const SG70325[] = {SGI0, SGI296968, SGI0, SGI296969, SGI296970, SGI296971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70326[] = {SGI0, SGI296973, SGI0, SGI296974, SGI296975, SGI296976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70327[] = {SGI0, SGI296979, SGI0, SGI296980, SGI296981, SGI296982, SGI0, SGI0, SGI0, SGI0, SGI296983, SGI0, SGI296984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70328[] = {SGI0, SGI296985, SGI0, SGI296986, SGI296987, SGI296987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI296988, SGI296989, SGI0, SGI0, SGI0, SGI296990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70329[] = {SGI0, SGI296991, SGI0, SGI296992, SGI296993, SGI296994, SGI0, SGI0, SGI0, SGI0, SGI296995, SGI0, SGI296996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70330[] = {SGI0, SGI296997, NULL};
const Segment * const SG70331[] = {SGI0, SGI296998, NULL};
const Segment * const SG70332[] = {SGI0, SGI296999, NULL};
const Segment * const SG70333[] = {SGI0, SGI297000, NULL};
const Segment * const SG70334[] = {SGI0, SGI297001, SGI0, SGI297002, SGI297003, SGI297004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297006, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70335[] = {SGI0, SGI297007, SGI0, SGI297008, SGI297009, SGI297009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297010, SGI297011, SGI0, SGI0, SGI297012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70336[] = {SGI0, SGI297013, SGI0, SGI297014, SGI297015, SGI297016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297017, SGI297018, SGI0, SGI0, SGI297019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70337[] = {SGI0, SGI297020, SGI0, SGI297021, SGI297022, SGI297023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70338[] = {SGI0, SGI297025, SGI0, SGI297026, SGI297027, SGI297027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297028, SGI297029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297030, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70339[] = {SGI0, SGI297031, SGI0, SGI297032, SGI297033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297034, SGI297035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70340[] = {SGI0, SGI297037, SGI0, SGI297038, SGI297039, SGI297039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297040, SGI297041, SGI0, SGI0, SGI0, SGI297042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70341[] = {SGI0, SGI297043, SGI0, SGI297044, SGI297045, SGI297046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70342[] = {SGI0, SGI297048, SGI0, SGI297049, SGI297050, SGI297051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297053, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70343[] = {SGI0, SGI297054, SGI0, SGI297055, SGI297056, SGI297056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297057, SGI297058, SGI0, SGI297059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70344[] = {SGI0, SGI297060, SGI0, SGI297061, SGI297062, SGI297063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70345[] = {SGI0, SGI297065, SGI0, SGI297066, SGI297067, SGI297068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297069, SGI297070, SGI0, SGI0, SGI0, SGI0, SGI297071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70346[] = {SGI0, SGI297072, SGI0, SGI297073, SGI297074, SGI297074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297075, SGI297076, SGI0, SGI297077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70347[] = {SGI0, SGI297078, SGI0, SGI297079, SGI297080, SGI297080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297081, SGI297082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70348[] = {SGI0, SGI297084, NULL};
const Segment * const SG70349[] = {SGI0, SGI297085, SGI0, SGI297086, SGI297087, SGI297088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297090, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70350[] = {SGI0, SGI297091, SGI0, SGI297092, SGI297093, SGI297094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70351[] = {SGI0, SGI297096, NULL};
const Segment * const SG70352[] = {SGI0, SGI297097, NULL};
const Segment * const SG70353[] = {SGI0, SGI297098, NULL};
const Segment * const SG70354[] = {SGI0, SGI297099, NULL};
const Segment * const SG70355[] = {SGI0, SGI297100, NULL};
const Segment * const SG70356[] = {SGI0, SGI297101, NULL};
const Segment * const SG70357[] = {SGI0, SGI297102, NULL};
const Segment * const SG70358[] = {SGI0, SGI297103, SGI0, SGI297104, SGI297105, SGI297105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297106, SGI297107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297108, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70359[] = {SGI0, SGI297109, NULL};
const Segment * const SG70360[] = {SGI0, SGI297110, NULL};
const Segment * const SG70361[] = {SGI297111, SGI297112, NULL};
const Segment * const SG70362[] = {SGI0, SGI297113, NULL};
const Segment * const SG70363[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI297114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297115, SGI297116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70364[] = {SGI0, SGI0, SGI0, SGI297117, SGI297118, SGI297119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297120, SGI297121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70365[] = {SGI0, SGI297122, SGI0, SGI297123, SGI297124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70366[] = {SGI0, SGI297126, SGI0, SGI297127, SGI297128, SGI297129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297130, SGI297131, SGI0, SGI0, SGI0, SGI0, SGI297132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70367[] = {SGI297133, SGI297134, NULL};
const Segment * const SG70368[] = {SGI297135, SGI297136, NULL};
const Segment * const SG70369[] = {SGI0, SGI297137, SGI0, SGI297138, SGI297139, SGI297140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70370[] = {SGI0, SGI297142, NULL};
const Segment * const SG70371[] = {SGI0, SGI297143, NULL};
const Segment * const SG70372[] = {SGI297144, SGI297145, NULL};
const Segment * const SG70373[] = {SGI297146, SGI297147, NULL};
const Segment * const SG70374[] = {SGI297148, SGI297149, NULL};
const Segment * const SG70375[] = {SGI0, SGI297150, SGI0, SGI297151, SGI297152, SGI297153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297154, SGI297155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70376[] = {SGI0, SGI0, SGI0, SGI297156, SGI297157, SGI297158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297159, SGI297160, SGI0, SGI0, SGI297161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70377[] = {SGI0, SGI297162, SGI0, SGI297163, SGI297164, SGI297164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297165, SGI297166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70378[] = {SGI297168, SGI297169, NULL};
const Segment * const SG70379[] = {SGI297170, SGI297171, NULL};
const Segment * const SG70380[] = {SGI297172, SGI297173, NULL};
const Segment * const SG70381[] = {SGI0, SGI297174, NULL};
const Segment * const SG70382[] = {SGI0, SGI297175, NULL};
const Segment * const SG70383[] = {SGI0, SGI297176, NULL};
const Segment * const SG70384[] = {SGI0, SGI297177, SGI0, SGI297178, SGI297179, SGI297180, SGI0, SGI0, SGI0, SGI0, SGI297181, SGI0, SGI297182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70385[] = {SGI0, SGI297183, SGI0, SGI297184, SGI297185, SGI297186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70386[] = {SGI0, SGI297188, SGI0, SGI297189, SGI297190, SGI297191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70387[] = {SGI0, SGI297193, SGI0, SGI297194, SGI297195, SGI297196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70388[] = {SGI0, SGI297199, SGI0, SGI297200, SGI297201, SGI297201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297202, SGI297203, SGI0, SGI0, SGI0, SGI297204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70389[] = {SGI0, SGI297205, SGI0, SGI297206, SGI297207, SGI297207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297208, SGI297209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297210, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70390[] = {SGI297211, SGI297212, NULL};
const Segment * const SG70391[] = {SGI297213, SGI297214, NULL};
const Segment * const SG70392[] = {SGI297215, SGI297216, NULL};
const Segment * const SG70393[] = {SGI297217, SGI297218, NULL};
const Segment * const SG70394[] = {SGI297219, SGI297220, NULL};
const Segment * const SG70395[] = {SGI297221, SGI297222, NULL};
const Segment * const SG70396[] = {SGI297223, SGI297224, NULL};
const Segment * const SG70397[] = {SGI297225, SGI297226, NULL};
const Segment * const SG70398[] = {SGI297227, SGI297228, NULL};
const Segment * const SG70399[] = {SGI0, SGI297229, NULL};
const Segment * const SG70400[] = {SGI297230, SGI297231, NULL};
const Segment * const SG70401[] = {SGI297232, SGI297233, NULL};
const Segment * const SG70402[] = {SGI297234, SGI297235, NULL};
const Segment * const SG70403[] = {SGI297236, SGI297237, NULL};
const Segment * const SG70404[] = {SGI297238, SGI297239, NULL};
const Segment * const SG70405[] = {SGI297240, SGI297241, NULL};
const Segment * const SG70406[] = {SGI0, SGI297242, SGI0, SGI297243, SGI297244, SGI297245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70407[] = {SGI0, SGI297247, NULL};
const Segment * const SG70408[] = {SGI297248, SGI297249, NULL};
const Segment * const SG70409[] = {SGI0, SGI297250, NULL};
const Segment * const SG70410[] = {SGI0, SGI297251, NULL};
const Segment * const SG70411[] = {SGI297252, SGI297253, SGI297254, SGI297255, SGI0, SGI297256, SGI0, NULL};
const Segment * const SG70412[] = {SGI0, SGI297257, NULL};
const Segment * const SG70413[] = {SGI0, SGI297258, NULL};
const Segment * const SG70414[] = {SGI0, SGI297259, NULL};
const Segment * const SG70415[] = {SGI0, SGI297260, NULL};
const Segment * const SG70416[] = {SGI0, SGI297261, SGI0, SGI297262, SGI297263, SGI297263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297264, SGI297265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70417[] = {SGI0, SGI297266, SGI0, SGI297267, SGI297268, SGI297269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297270, SGI297271, SGI0, SGI0, SGI0, SGI0, SGI297272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70418[] = {SGI0, SGI297273, SGI0, SGI297274, SGI297275, SGI297275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297276, SGI297277, SGI0, SGI297278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70419[] = {SGI0, SGI297279, NULL};
const Segment * const SG70420[] = {SGI0, SGI297280, SGI0, SGI297281, SGI297282, SGI297283, SGI0, SGI0, SGI0, SGI0, SGI297284, SGI0, SGI297285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70421[] = {SGI297286, SGI297287, NULL};
const Segment * const SG70422[] = {SGI297288, SGI297289, NULL};
const Segment * const SG70423[] = {SGI297290, SGI297291, NULL};
const Segment * const SG70424[] = {SGI0, SGI297292, SGI0, SGI297293, SGI297294, SGI297295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297297, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70425[] = {SGI0, SGI297298, NULL};
const Segment * const SG70426[] = {SGI297299, SGI297300, NULL};
const Segment * const SG70427[] = {SGI297301, SGI297302, NULL};
const Segment * const SG70428[] = {SGI297303, SGI297304, NULL};
const Segment * const SG70429[] = {SGI297305, SGI297306, NULL};
const Segment * const SG70430[] = {SGI297307, SGI297308, NULL};
const Segment * const SG70431[] = {SGI297309, SGI297310, NULL};
const Segment * const SG70432[] = {SGI297311, SGI297312, NULL};
const Segment * const SG70433[] = {SGI297313, SGI297314, NULL};
const Segment * const SG70434[] = {SGI297315, SGI297316, NULL};
const Segment * const SG70435[] = {SGI0, SGI297317, NULL};
const Segment * const SG70436[] = {SGI0, SGI297318, NULL};
const Segment * const SG70437[] = {SGI0, SGI297319, NULL};
const Segment * const SG70438[] = {SGI0, SGI297320, NULL};
const Segment * const SG70439[] = {SGI0, SGI297321, NULL};
const Segment * const SG70440[] = {SGI0, SGI297322, NULL};
const Segment * const SG70441[] = {SGI0, SGI297323, NULL};
const Segment * const SG70442[] = {SGI0, SGI297324, NULL};
const Segment * const SG70443[] = {SGI0, SGI297325, NULL};
const Segment * const SG70444[] = {SGI0, SGI297326, NULL};
const Segment * const SG70445[] = {SGI0, SGI297327, NULL};
const Segment * const SG70446[] = {SGI0, SGI297328, NULL};
const Segment * const SG70447[] = {SGI0, SGI297329, NULL};
const Segment * const SG70448[] = {SGI0, SGI297330, NULL};
const Segment * const SG70449[] = {SGI0, SGI297331, NULL};
const Segment * const SG70450[] = {SGI0, SGI297332, SGI0, SGI297333, SGI297334, SGI297335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70451[] = {SGI0, SGI297338, SGI0, SGI297339, SGI297340, SGI297341, SGI0, SGI0, SGI0, SGI0, SGI297342, SGI0, SGI297343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70452[] = {SGI0, SGI297344, SGI0, SGI297345, SGI297346, SGI297347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70453[] = {SGI0, SGI297349, SGI0, SGI297350, SGI297351, SGI297352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70454[] = {SGI0, SGI297354, SGI0, SGI297355, SGI297356, SGI297357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70455[] = {SGI297359, SGI297360, SGI297361, SGI297362, SGI0, SGI297363, SGI0, NULL};
const Segment * const SG70456[] = {SGI297364, SGI297365, NULL};
const Segment * const SG70457[] = {SGI0, SGI297366, NULL};
const Segment * const SG70458[] = {SGI0, SGI297367, NULL};
const Segment * const SG70459[] = {SGI0, SGI297368, NULL};
const Segment * const SG70460[] = {SGI0, SGI297369, SGI0, SGI297370, SGI297371, SGI297371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70461[] = {SGI0, SGI297373, NULL};
const Segment * const SG70462[] = {SGI0, SGI297374, NULL};
const Segment * const SG70463[] = {SGI0, SGI297375, NULL};
const Segment * const SG70464[] = {SGI0, SGI297376, NULL};
const Segment * const SG70465[] = {SGI0, SGI297377, NULL};
const Segment * const SG70466[] = {SGI0, SGI297378, SGI0, SGI297379, SGI297380, SGI297381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297382, SGI297383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70467[] = {SGI0, SGI297384, SGI0, SGI297385, SGI297386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297387, SGI297388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70468[] = {SGI0, SGI297389, SGI0, SGI297390, SGI297391, SGI297391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297392, SGI297393, SGI0, SGI0, SGI0, SGI297394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70469[] = {SGI0, SGI297395, SGI0, SGI297396, SGI297397, SGI297397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297398, SGI297399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70470[] = {SGI0, SGI297401, SGI0, SGI297402, SGI297403, SGI297403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297404, SGI297405, SGI0, SGI0, SGI0, SGI297406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70471[] = {SGI0, SGI297407, SGI0, SGI297408, SGI297409, SGI297410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70472[] = {SGI297413, SGI297414, NULL};
const Segment * const SG70473[] = {SGI297415, SGI297416, NULL};
const Segment * const SG70474[] = {SGI0, SGI297417, SGI0, SGI297418, SGI297419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297420, SGI297421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70475[] = {SGI0, SGI297422, SGI0, SGI297423, SGI297424, SGI297425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70476[] = {SGI0, SGI297428, SGI0, SGI297429, SGI297430, SGI297430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297431, SGI297432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297433, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70477[] = {SGI0, SGI297434, SGI0, SGI297435, SGI297436, SGI297436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297438, NULL};
const Segment * const SG70478[] = {SGI297439, SGI297440, NULL};
const Segment * const SG70479[] = {SGI297441, SGI297442, NULL};
const Segment * const SG70480[] = {SGI297443, SGI297444, NULL};
const Segment * const SG70481[] = {SGI297445, SGI297446, NULL};
const Segment * const SG70482[] = {SGI297447, SGI297448, NULL};
const Segment * const SG70483[] = {SGI0, SGI297449, SGI0, SGI297450, SGI297451, SGI297452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297453, SGI297454, SGI0, SGI0, SGI297455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70484[] = {SGI297456, SGI297457, NULL};
const Segment * const SG70485[] = {SGI297458, SGI297459, NULL};
const Segment * const SG70486[] = {SGI297460, SGI297461, NULL};
const Segment * const SG70487[] = {SGI0, SGI297462, SGI0, SGI297463, SGI297464, SGI297465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70488[] = {SGI0, SGI297467, SGI0, SGI297468, SGI297469, SGI297470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297471, SGI297472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70489[] = {SGI0, SGI297473, NULL};
const Segment * const SG70490[] = {SGI297474, SGI297475, NULL};
const Segment * const SG70491[] = {SGI297476, SGI297477, NULL};
const Segment * const SG70492[] = {SGI297478, NULL};
const Segment * const SG70493[] = {SGI0, SGI297479, SGI0, SGI297480, SGI297481, SGI297482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70494[] = {SGI297485, SGI297486, NULL};
const Segment * const SG70495[] = {SGI297487, SGI297488, NULL};
const Segment * const SG70496[] = {SGI0, SGI297489, NULL};
const Segment * const SG70497[] = {SGI0, SGI297490, NULL};
const Segment * const SG70498[] = {SGI0, SGI297491, NULL};
const Segment * const SG70499[] = {SGI0, SGI297492, SGI0, SGI297493, SGI297494, SGI297495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70500[] = {SGI0, SGI297497, SGI0, SGI297498, SGI297499, SGI297500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297501, SGI297502, SGI0, SGI0, SGI297503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70501[] = {SGI297504, SGI297505, NULL};
const Segment * const SG70502[] = {SGI0, SGI297506, NULL};
const Segment * const SG70503[] = {SGI0, SGI297507, NULL};
const Segment * const SG70504[] = {SGI0, SGI297508, NULL};
const Segment * const SG70505[] = {SGI297509, SGI297510, SGI297511, SGI297512, SGI0, SGI297510, SGI0, NULL};
const Segment * const SG70506[] = {SGI0, SGI297513, SGI0, SGI297514, SGI297515, SGI297515, SGI297516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297517, SGI0, SGI297518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70507[] = {SGI0, SGI297519, SGI0, SGI297520, SGI297521, SGI297521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297522, SGI297523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297524, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70508[] = {SGI0, SGI297525, SGI0, SGI297526, SGI297527, SGI297527, SGI297528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297529, SGI0, SGI297530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70509[] = {SGI0, SGI297531, SGI0, SGI297532, SGI297533, SGI297533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70510[] = {SGI0, SGI297536, SGI0, SGI297537, SGI297538, SGI297539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70511[] = {SGI0, SGI297541, SGI0, SGI297542, SGI297543, SGI297543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297544, SGI297545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70512[] = {SGI0, SGI297547, SGI0, SGI297548, SGI297549, SGI297549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297550, SGI297551, SGI0, SGI297552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70513[] = {SGI0, SGI297553, SGI0, SGI297554, SGI297555, SGI297556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297557, SGI297558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70514[] = {SGI0, SGI297560, SGI0, SGI297561, SGI297562, SGI297562, SGI297563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297564, SGI0, SGI297565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70515[] = {SGI0, SGI297566, SGI0, SGI297567, SGI297568, SGI297569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297570, SGI297571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70516[] = {SGI297573, SGI297574, NULL};
const Segment * const SG70517[] = {SGI0, SGI297575, NULL};
const Segment * const SG70518[] = {SGI0, SGI297576, NULL};
const Segment * const SG70519[] = {SGI0, SGI297577, NULL};
const Segment * const SG70520[] = {SGI0, SGI297578, NULL};
const Segment * const SG70521[] = {SGI0, SGI297579, NULL};
const Segment * const SG70522[] = {SGI0, SGI297580, NULL};
const Segment * const SG70523[] = {SGI297581, SGI297582, NULL};
const Segment * const SG70524[] = {SGI297583, SGI297584, NULL};
const Segment * const SG70525[] = {SGI0, SGI297585, SGI0, SGI297586, SGI297587, SGI297588, SGI0, SGI0, SGI0, SGI0, SGI297589, SGI0, SGI297590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70526[] = {SGI0, SGI297591, NULL};
const Segment * const SG70527[] = {SGI0, SGI297592, NULL};
const Segment * const SG70528[] = {SGI0, SGI0, SGI0, SGI297593, SGI297594, SGI297595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297596, SGI297597, SGI0, SGI0, SGI297598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70529[] = {SGI297599, SGI297599, SGI297600, SGI297601, SGI0, SGI297599, SGI0, NULL};
const Segment * const SG70530[] = {SGI0, SGI0, SGI0, SGI297602, SGI297603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70531[] = {SGI0, SGI297605, NULL};
const Segment * const SG70532[] = {SGI0, SGI297606, NULL};
const Segment * const SG70533[] = {SGI0, SGI297607, NULL};
const Segment * const SG70534[] = {SGI0, SGI297608, NULL};
const Segment * const SG70535[] = {SGI0, SGI297612, NULL};
const Segment * const SG70536[] = {SGI0, SGI0, SGI0, SGI297609, SGI297610, SGI297611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297613, SGI297614, SGI0, SGI0, SGI297615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70537[] = {SGI0, SGI297616, SGI0, SGI297617, SGI297618, SGI297619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70538[] = {SGI0, SGI297622, SGI0, SGI297623, SGI297624, SGI297624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297625, SGI0, SGI297626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70539[] = {SGI297627, SGI297628, NULL};
const Segment * const SG70540[] = {SGI297629, SGI297630, NULL};
const Segment * const SG70541[] = {SGI0, SGI297631, NULL};
const Segment * const SG70542[] = {SGI0, SGI297632, NULL};
const Segment * const SG70543[] = {SGI297633, SGI297634, NULL};
const Segment * const SG70544[] = {SGI297635, SGI297636, NULL};
const Segment * const SG70545[] = {SGI0, SGI297637, NULL};
const Segment * const SG70546[] = {SGI0, SGI297638, NULL};
const Segment * const SG70547[] = {SGI0, SGI297639, NULL};
const Segment * const SG70548[] = {SGI0, SGI297640, NULL};
const Segment * const SG70549[] = {SGI0, SGI297641, NULL};
const Segment * const SG70550[] = {SGI0, SGI297642, NULL};
const Segment * const SG70551[] = {SGI297643, SGI297644, SGI297645, SGI297646, SGI0, SGI297647, SGI0, NULL};
const Segment * const SG70552[] = {SGI297648, NULL};
const Segment * const SG70553[] = {SGI297649, SGI297650, NULL};
const Segment * const SG70554[] = {SGI297651, SGI297652, NULL};
const Segment * const SG70555[] = {SGI297653, SGI297654, NULL};
const Segment * const SG70556[] = {SGI0, SGI297655, NULL};
const Segment * const SG70557[] = {SGI0, SGI297656, NULL};
const Segment * const SG70558[] = {SGI297657, SGI297658, NULL};
const Segment * const SG70559[] = {SGI297659, SGI297660, NULL};
const Segment * const SG70560[] = {SGI297661, SGI297662, NULL};
const Segment * const SG70561[] = {SGI297663, SGI297664, NULL};
const Segment * const SG70562[] = {SGI0, SGI297665, NULL};
const Segment * const SG70563[] = {SGI0, SGI297666, NULL};
const Segment * const SG70564[] = {SGI0, SGI297667, NULL};
const Segment * const SG70565[] = {SGI0, SGI297668, NULL};
const Segment * const SG70566[] = {SGI0, SGI297669, NULL};
const Segment * const SG70567[] = {SGI0, SGI297670, NULL};
const Segment * const SG70568[] = {SGI0, SGI297671, NULL};
const Segment * const SG70569[] = {SGI0, SGI297672, NULL};
const Segment * const SG70570[] = {SGI0, SGI297673, NULL};
const Segment * const SG70571[] = {SGI0, SGI297674, NULL};
const Segment * const SG70572[] = {SGI0, SGI297675, NULL};
const Segment * const SG70573[] = {SGI0, SGI297676, NULL};
const Segment * const SG70574[] = {SGI0, SGI297677, NULL};
const Segment * const SG70575[] = {SGI0, SGI297678, NULL};
const Segment * const SG70576[] = {SGI0, SGI297679, NULL};
const Segment * const SG70577[] = {SGI297680, NULL};
const Segment * const SG70578[] = {SGI0, SGI297681, NULL};
const Segment * const SG70579[] = {SGI0, SGI297682, NULL};
const Segment * const SG70580[] = {SGI0, SGI297683, NULL};
const Segment * const SG70581[] = {SGI0, SGI297684, NULL};
const Segment * const SG70582[] = {SGI297685, SGI297686, NULL};
const Segment * const SG70583[] = {SGI297687, SGI297688, NULL};
const Segment * const SG70584[] = {SGI0, SGI297689, NULL};
const Segment * const SG70585[] = {SGI0, SGI297690, SGI0, SGI297691, SGI297692, SGI297692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70586[] = {SGI0, SGI297694, SGI0, SGI297695, SGI0, SGI297696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297697, SGI297698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70587[] = {SGI297699, SGI297700, NULL};
const Segment * const SG70588[] = {SGI297701, SGI297702, NULL};
const Segment * const SG70589[] = {SGI297703, SGI297704, NULL};
const Segment * const SG70590[] = {SGI0, SGI297705, NULL};
const Segment * const SG70591[] = {SGI297706, NULL};
const Segment * const SG70592[] = {SGI0, SGI297707, SGI0, SGI297708, SGI297709, SGI297710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70593[] = {SGI0, SGI297712, NULL};
const Segment * const SG70594[] = {SGI0, SGI297713, NULL};
const Segment * const SG70595[] = {SGI0, SGI297714, NULL};
const Segment * const SG70596[] = {SGI0, SGI297715, NULL};
const Segment * const SG70597[] = {SGI297716, NULL};
const Segment * const SG70598[] = {SGI0, SGI297717, NULL};
const Segment * const SG70599[] = {SGI297718, NULL};
const Segment * const SG70600[] = {SGI297719, NULL};
const Segment * const SG70601[] = {SGI297720, NULL};
const Segment * const SG70602[] = {SGI0, SGI297721, SGI0, SGI297722, SGI297723, SGI297723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70603[] = {SGI0, SGI297725, SGI0, SGI297726, SGI297727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI297728, SGI297729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG70604[] = {SGI0, SGI297730, NULL};
const Segment * const SG70605[] = {SGI297731, SGI297732, NULL};
const Segment * const SG70606[] = {SGI297733, NULL};
const Segment * const SG70607[] = {SGI0, SGI297734, NULL};
const Segment * const SG70608[] = {SGI0, SGI297735, NULL};
const Segment * const SG70609[] = {SGI297736, NULL};
const Segment * const SG70610[] = {SGI297737, NULL};
const Segment * const SG70611[] = {SGI297738, SGI297739, NULL};
const Segment * const SG70612[] = {SGI297740, SGI297741, NULL};
const Segment * const SG70613[] = {SGI297742, SGI297743, NULL};
const Segment * const SG70614[] = {SGI0, SGI297744, NULL};
const Segment * const SG70615[] = {SGI297745, NULL};
const Segment * const SG70616[] = {SGI297746, NULL};
const Segment * const SG70617[] = {SGI297747, NULL};
const Segment * const SG70618[] = {SGI297748, NULL};
const Segment * const SG70619[] = {SGI0, SGI297749, NULL};
const Segment * const SG70620[] = {SGI297750, SGI297751, SGI297752, SGI297753, SGI0, SGI297751, SGI0, NULL};
const Segment * const SG70621[] = {SGI297754, SGI297755, NULL};
const Segment * const SG70622[] = {SGI297756, NULL};
const Segment * const SG70623[] = {SGI297757, SGI297758, NULL};
const Segment * const SG70624[] = {SGI0, SGI297759, NULL};
const Segment * const SG70625[] = {SGI0, SGI297760, NULL};
const Segment * const SG70626[] = {SGI297761, SGI297762, NULL};
const Segment * const SG70627[] = {SGI297763, SGI297764, NULL};
const Segment * const SG70628[] = {SGI297765, SGI297766, NULL};
const Segment * const SG70629[] = {SGI297767, SGI297768, NULL};
const Segment * const SG70630[] = {SGI297769, SGI297770, NULL};
const Segment * const SG70631[] = {SGI0, SGI297771, NULL};
const Segment * const SG70632[] = {SGI0, SGI297772, NULL};
const Segment * const SG70633[] = {SGI0, SGI297773, NULL};
const Segment * const SG70634[] = {SGI0, SGI297774, NULL};
const Segment * const SG70635[] = {SGI0, SGI297775, NULL};
const Segment * const SG70636[] = {SGI0, SGI297776, NULL};
const Segment * const SG70637[] = {SGI0, SGI297777, NULL};
const Segment * const SG70638[] = {SGI0, SGI297778, NULL};
const Segment * const SG70639[] = {SGI0, SGI297779, NULL};
const Segment * const SG70640[] = {SGI297780, SGI297781, NULL};
const Segment * const SG70641[] = {SGI297782, SGI297783, NULL};
const Segment * const SG70642[] = {SGI297784, SGI297785, NULL};
const Segment * const SG70643[] = {SGI0, SGI297786, NULL};
const Segment * const SG70644[] = {SGI0, SGI297787, NULL};
const Segment * const SG70645[] = {SGI0, SGI297788, NULL};
const Segment * const SG70646[] = {SGI0, SGI297789, NULL};
const Segment * const SG70647[] = {SGI297790, NULL};
const Segment * const SG70648[] = {SGI0, SGI297791, NULL};
const Segment * const SG70649[] = {SGI0, SGI297792, NULL};
const Segment * const SG70650[] = {SGI0, SGI297793, NULL};
const Segment * const SG70651[] = {SGI297794, SGI297795, SGI297796, SGI297797, SGI0, SGI297795, SGI0, NULL};
const Segment * const SG70652[] = {SGI297798, SGI297799, NULL};
const Segment * const SG70653[] = {SGI0, SGI297800, NULL};
const Segment * const SG70654[] = {SGI297801, SGI297802, NULL};
const Segment * const SG70655[] = {SGI0, SGI297803, NULL};
const Segment * const SG70656[] = {SGI0, SGI297804, NULL};
const Segment * const SG70657[] = {SGI0, SGI297805, NULL};
const Segment * const SG70658[] = {SGI0, SGI297806, NULL};
const Segment * const SG70659[] = {SGI0, SGI297807, NULL};
const Segment * const SG70660[] = {SGI297808, SGI297809, NULL};
const Segment * const SG70661[] = {SGI297810, SGI297811, NULL};
const Segment * const SG70662[] = {SGI297812, NULL};
const Segment * const SG70663[] = {SGI297813, SGI297814, NULL};
const Segment * const SG70664[] = {SGI0, SGI297815, NULL};
const Segment * const SG70665[] = {SGI297816, SGI297817, NULL};
const Segment * const SG70666[] = {SGI297818, NULL};
const Segment * const SG70667[] = {SGI297819, NULL};
const Segment * const SG70668[] = {SGI297820, NULL};
const Segment * const SG70669[] = {SGI297821, SGI297822, NULL};
const Segment * const SG70670[] = {SGI297823, SGI297824, NULL};
const Segment * const SG70671[] = {SGI297825, SGI297826, NULL};
const Segment * const SG70672[] = {SGI297827, NULL};
const Segment * const SG70673[] = {SGI297828, SGI297829, NULL};
const Segment * const SG70674[] = {SGI297830, SGI297831, NULL};
const Segment * const SG70675[] = {SGI297832, SGI297833, NULL};
const Segment * const SG70676[] = {SGI297834, SGI297835, NULL};
const Segment * const SG70677[] = {SGI297836, NULL};
const Segment * const SG70678[] = {SGI0, SGI297837, NULL};
const Segment * const SG70679[] = {SGI0, SGI297838, NULL};
const Segment * const SG70680[] = {SGI297839, NULL};
const Segment * const SG70681[] = {SGI297840, SGI297841, NULL};
const Segment * const SG70682[] = {SGI297842, NULL};
const Segment * const SG70683[] = {SGI297843, SGI297844, NULL};
const Segment * const SG70684[] = {SGI297845, SGI297846, NULL};
const Segment * const SG70685[] = {SGI297847, SGI297848, NULL};
const Segment * const SG70686[] = {SGI297849, SGI297850, NULL};
const Segment * const SG70687[] = {SGI0, SGI297851, NULL};
const Segment * const SG70688[] = {SGI0, SGI297852, NULL};
const Segment * const SG70689[] = {SGI0, SGI297853, NULL};
const Segment * const SG70690[] = {SGI0, SGI297854, NULL};
const Segment * const SG70691[] = {SGI0, SGI297855, NULL};
const Segment * const SG70692[] = {SGI297856, SGI297857, NULL};
const Segment * const SG70693[] = {SGI297858, SGI297859, NULL};
const Segment * const SG70694[] = {SGI297860, SGI0, SGI0, SGI297861, SGI297862, SGI297863, SGI0, NULL};
const Segment * const SG70695[] = {SGI297864, SGI297865, NULL};
const Segment * const SG70696[] = {SGI297866, NULL};
const Segment * const SG70697[] = {SGI0, SGI297867, NULL};
const Segment * const SG70698[] = {SGI0, SGI297868, NULL};
const Segment * const SG70699[] = {SGI0, SGI297869, NULL};
const Segment * const SG70700[] = {SGI297870, SGI297871, NULL};
const Segment * const SG70701[] = {SGI297872, SGI297873, NULL};
const Segment * const SG70702[] = {SGI297874, SGI297875, NULL};
const Segment * const SG70703[] = {SGI297876, SGI297877, NULL};
const Segment * const SG70704[] = {SGI0, SGI297878, NULL};
const Segment * const SG70705[] = {SGI0, SGI297879, NULL};
const Segment * const SG70706[] = {SGI297880, SGI297881, NULL};
const Segment * const SG70707[] = {SGI0, SGI297882, NULL};
const Segment * const SG70708[] = {SGI0, SGI297883, NULL};
const Segment * const SG70709[] = {SGI0, SGI297884, NULL};
const Segment * const SG70710[] = {SGI297885, NULL};
const Segment * const SG70711[] = {SGI297886, NULL};
const Segment * const SG70712[] = {SGI297887, SGI297888, NULL};
const Segment * const SG70713[] = {SGI297889, NULL};
const Segment * const SG70714[] = {SGI0, SGI297890, NULL};
const Segment * const SG70715[] = {SGI0, SGI297891, NULL};
const Segment * const SG70716[] = {SGI297892, NULL};
const Segment * const SG70717[] = {SGI0, SGI297893, NULL};
const Segment * const SG70718[] = {SGI0, SGI297894, NULL};
const Segment * const SG70719[] = {SGI297895, SGI297896, NULL};
const Segment * const SG70720[] = {SGI297897, SGI297898, NULL};
const Segment * const SG70721[] = {SGI297899, SGI297900, NULL};
const Segment * const SG70722[] = {SGI0, SGI297901, NULL};
const Segment * const SG70723[] = {SGI0, SGI297902, NULL};
const Segment * const SG70724[] = {SGI0, SGI297903, NULL};
const Segment * const SG70725[] = {SGI0, SGI297904, NULL};
const Segment * const SG70726[] = {SGI0, SGI297905, NULL};
const Segment * const SG70727[] = {SGI0, SGI297906, NULL};
const Segment * const SG70728[] = {SGI0, SGI297907, NULL};
const Segment * const SG70729[] = {SGI0, SGI297908, NULL};
const Segment * const SG70730[] = {SGI0, SGI297909, NULL};
const Segment * const SG70731[] = {SGI0, SGI297910, NULL};
const Segment * const SG70732[] = {SGI297911, SGI297912, NULL};
const Segment * const SG70733[] = {SGI297913, NULL};
const Segment * const SG70734[] = {SGI297914, SGI297915, NULL};
const Segment * const SG70735[] = {SGI0, SGI297916, NULL};
const Segment * const SG70736[] = {SGI0, SGI297917, NULL};
const Segment * const SG70737[] = {SGI0, SGI297918, NULL};
const Segment * const SG70738[] = {SGI297919, SGI297920, NULL};
const Segment * const SG70739[] = {SGI0, SGI297921, NULL};
const Segment * const SG70740[] = {SGI297922, NULL};
const Segment * const SG70741[] = {SGI297923, SGI297924, NULL};
const Segment * const SG70742[] = {SGI0, SGI297925, NULL};
const Segment * const SG70743[] = {SGI297926, SGI297927, NULL};
const Segment * const SG70744[] = {SGI297928, SGI297929, NULL};
const Segment * const SG70745[] = {SGI297930, SGI297931, NULL};
const Segment * const SG70746[] = {SGI297932, SGI297933, NULL};
const Segment * const SG70747[] = {SGI0, SGI297934, NULL};
const Segment * const SG70748[] = {SGI0, SGI297935, NULL};
const Segment * const SG70749[] = {SGI0, SGI297936, NULL};
const Segment * const SG70750[] = {SGI0, SGI297937, NULL};
const Segment * const SG70751[] = {SGI0, SGI297938, NULL};
const Segment * const SG70752[] = {SGI0, SGI297939, NULL};
const Segment * const SG70753[] = {SGI0, SGI297940, NULL};
const Segment * const SG70754[] = {SGI0, SGI297941, NULL};
const Segment * const SG70755[] = {SGI297942, SGI297943, NULL};
const Segment * const SG70756[] = {SGI297944, NULL};
const Segment * const SG70757[] = {SGI0, SGI297945, NULL};
const Segment * const SG70758[] = {SGI0, SGI297946, NULL};
const Segment * const SG70759[] = {SGI297947, NULL};
const Segment * const SG70760[] = {SGI297948, NULL};
const Segment * const SG70761[] = {SGI0, SGI297949, NULL};
const Segment * const SG70762[] = {SGI0, SGI297950, NULL};
const Segment * const SG70763[] = {SGI297951, SGI297952, NULL};
const Segment * const SG70764[] = {SGI297953, SGI297954, NULL};
const Segment * const SG70765[] = {SGI0, SGI297955, NULL};
const Segment * const SG70766[] = {SGI0, SGI297956, NULL};
const Segment * const SG70767[] = {SGI0, SGI297957, NULL};
const Segment * const SG70768[] = {SGI297958, NULL};
const Segment * const SG70769[] = {SGI0, SGI297959, NULL};
const Segment * const SG70770[] = {SGI0, SGI297960, NULL};
const Segment * const SG70771[] = {SGI0, SGI297961, NULL};
const Segment * const SG70772[] = {SGI297962, SGI297963, NULL};
const Segment * const SG70773[] = {SGI0, SGI297964, NULL};
const Segment * const SG70774[] = {SGI0, SGI297965, NULL};
const Segment * const SG70775[] = {SGI297966, SGI297967, NULL};
const Segment * const SG70776[] = {SGI0, SGI297968, NULL};
const Segment * const SG70777[] = {SGI0, SGI297969, NULL};
const Segment * const SG70778[] = {SGI297970, SGI297971, NULL};
const Segment * const SG70779[] = {SGI0, SGI297972, NULL};
const Segment * const SG70780[] = {SGI297973, SGI297974, NULL};
const Segment * const SG70781[] = {SGI297975, SGI297976, NULL};
const Segment * const SG70782[] = {SGI297977, NULL};
const Segment * const SG70783[] = {SGI297978, SGI297979, NULL};
const Segment * const SG70784[] = {SGI0, SGI297980, NULL};
const Segment * const SG70785[] = {SGI0, SGI297981, NULL};
const Segment * const SG70786[] = {SGI0, SGI297982, NULL};
const Segment * const SG70787[] = {SGI0, SGI297983, NULL};
const Segment * const SG70788[] = {SGI297984, SGI297985, NULL};
const Segment * const SG70789[] = {SGI0, SGI297986, NULL};
const Segment * const SG70790[] = {SGI0, SGI297987, NULL};
const Segment * const SG70791[] = {SGI0, SGI297988, NULL};
const Segment * const SG70792[] = {SGI297989, NULL};
const Segment * const SG70793[] = {SGI0, SGI297990, NULL};
const Segment * const SG70794[] = {SGI297991, SGI297992, NULL};
const Segment * const SG70795[] = {SGI0, SGI297993, NULL};
const Segment * const SG70796[] = {SGI297994, NULL};
const Segment * const SG70797[] = {SGI0, SGI297995, NULL};
const Segment * const SG70798[] = {SGI0, SGI297996, NULL};
const Segment * const SG70799[] = {SGI0, SGI297997, NULL};
const Segment * const SG70800[] = {SGI0, SGI297998, NULL};
const Segment * const SG70801[] = {SGI0, SGI297999, NULL};
const Segment * const SG70802[] = {SGI0, SGI298000, NULL};
const Segment * const SG70803[] = {SGI298001, SGI298002, NULL};
const Segment * const SG70804[] = {SGI298003, SGI298004, NULL};
const Segment * const SG70805[] = {SGI298005, SGI298006, NULL};
const Segment * const SG70806[] = {SGI298007, SGI298008, NULL};
const Segment * const SG70807[] = {SGI298009, SGI298010, NULL};
const Segment * const SG70808[] = {SGI298011, SGI298012, NULL};
const Segment * const SG70809[] = {SGI298013, SGI298014, NULL};
const Segment * const SG70810[] = {SGI298015, NULL};
const Segment * const SG70811[] = {SGI0, SGI298016, NULL};
const Segment * const SG70812[] = {SGI298017, SGI298018, NULL};
const Segment * const SG70813[] = {SGI298019, SGI298020, NULL};
const Segment * const SG70814[] = {SGI298021, SGI298022, NULL};
const Segment * const SG70815[] = {SGI0, SGI298023, NULL};
const Segment * const SG70816[] = {SGI0, SGI298024, NULL};
const Segment * const SG70817[] = {SGI0, SGI298025, NULL};
const Segment * const SG70818[] = {SGI0, SGI298026, NULL};
const Segment * const SG70819[] = {SGI298027, NULL};
const Segment * const SG70820[] = {SGI298028, SGI298029, NULL};
const Segment * const SG70821[] = {SGI0, SGI298030, NULL};
const Segment * const SG70822[] = {SGI298031, SGI298032, NULL};
const Segment * const SG70823[] = {SGI298033, SGI298034, NULL};
const Segment * const SG70824[] = {SGI298035, SGI298036, NULL};
const Segment * const SG70825[] = {SGI298037, NULL};
const Segment * const SG70826[] = {SGI0, SGI298038, NULL};
const Segment * const SG70827[] = {SGI298039, SGI298040, NULL};
const Segment * const SG70828[] = {SGI0, SGI298041, NULL};
const Segment * const SG70829[] = {SGI0, SGI298042, NULL};
const Segment * const SG70830[] = {SGI0, SGI298043, NULL};
const Segment * const SG70831[] = {SGI298044, SGI298045, NULL};
const Segment * const SG70832[] = {SGI0, SGI298046, NULL};
const Segment * const SG70833[] = {SGI298047, SGI298048, NULL};
const Segment * const SG70834[] = {SGI0, SGI298049, NULL};
const Segment * const SG70835[] = {SGI298050, SGI298051, NULL};
const Segment * const SG70836[] = {SGI0, SGI298052, NULL};
const Segment * const SG70837[] = {SGI0, SGI298053, NULL};
const Segment * const SG70838[] = {SGI0, SGI298054, NULL};
const Segment * const SG70839[] = {SGI0, SGI298055, NULL};
const Segment * const SG70840[] = {SGI0, SGI298056, NULL};
const Segment * const SG70841[] = {SGI0, SGI298057, NULL};
const Segment * const SG70842[] = {SGI298058, SGI298059, NULL};
const Segment * const SG70843[] = {SGI298060, SGI298061, NULL};
const Segment * const SG70844[] = {SGI298062, SGI298063, NULL};
const Segment * const SG70845[] = {SGI298064, SGI298065, NULL};
const Segment * const SG70846[] = {SGI298066, SGI298067, NULL};
const Segment * const SG70847[] = {SGI298068, NULL};
const Segment * const SG70848[] = {SGI0, SGI298069, NULL};
const Segment * const SG70849[] = {SGI0, SGI298070, NULL};
const Segment * const SG70850[] = {SGI0, SGI298071, NULL};
const Segment * const SG70851[] = {SGI298072, SGI298073, NULL};
const Segment * const SG70852[] = {SGI0, SGI298074, NULL};
const Segment * const SG70853[] = {SGI0, SGI298075, NULL};
const Segment * const SG70854[] = {SGI0, SGI298076, NULL};
const Segment * const SG70855[] = {SGI0, SGI298077, NULL};
const Segment * const SG70856[] = {SGI298078, SGI298079, NULL};
const Segment * const SG70857[] = {SGI0, SGI298080, NULL};
const Segment * const SG70858[] = {SGI298081, NULL};
const Segment * const SG70859[] = {SGI0, SGI298082, NULL};
const Segment * const SG70860[] = {SGI0, SGI298083, NULL};
const Segment * const SG70861[] = {SGI0, SGI298084, NULL};
const Segment * const SG70862[] = {SGI0, SGI298085, NULL};
const Segment * const SG70863[] = {SGI298086, NULL};
const Segment * const SG70864[] = {SGI0, SGI298087, NULL};
const Segment * const SG70865[] = {SGI0, SGI298088, NULL};
const Segment * const SG70866[] = {SGI0, SGI298089, NULL};
const Segment * const SG70867[] = {SGI0, SGI298090, NULL};
const Segment * const SG70868[] = {SGI0, SGI298091, NULL};
const Segment * const SG70869[] = {SGI0, SGI298092, NULL};
const Segment * const SG70870[] = {SGI0, SGI298093, NULL};
const Segment * const SG70871[] = {SGI298094, SGI298095, NULL};
const Segment * const SG70872[] = {SGI0, SGI298096, NULL};
const Segment * const SG70873[] = {SGI298097, SGI298098, NULL};
const Segment * const SG70874[] = {SGI0, SGI298099, NULL};
const Segment * const SG70875[] = {SGI0, SGI298100, NULL};
const Segment * const SG70876[] = {SGI0, SGI298101, NULL};
const Segment * const SG70877[] = {SGI0, SGI298102, NULL};
const Segment * const SG70878[] = {SGI298103, SGI298104, NULL};
const Segment * const SG70879[] = {SGI0, SGI298105, NULL};
const Segment * const SG70880[] = {SGI0, SGI298106, NULL};
const Segment * const SG70881[] = {SGI0, SGI298107, NULL};
const Segment * const SG70882[] = {SGI0, SGI298108, NULL};
const Segment * const SG70883[] = {SGI298109, SGI298110, NULL};
const Segment * const SG70884[] = {SGI0, SGI298111, NULL};
const Segment * const SG70885[] = {SGI298112, SGI298113, NULL};
const Segment * const SG70886[] = {SGI0, SGI298114, NULL};
const Segment * const SG70887[] = {SGI0, SGI298115, NULL};
const Segment * const SG70888[] = {SGI0, SGI298116, NULL};
const Segment * const SG70889[] = {SGI298117, NULL};
const Segment * const SG70890[] = {SGI298118, NULL};
const Segment * const SG70891[] = {SGI0, SGI298119, NULL};
const Segment * const SG70892[] = {SGI0, SGI298120, NULL};
const Segment * const SG70893[] = {SGI298121, SGI298122, NULL};
const Segment * const SG70894[] = {SGI0, SGI298123, NULL};
const Segment * const SG70895[] = {SGI0, SGI298124, NULL};
const Segment * const SG70896[] = {SGI0, SGI298125, NULL};
const Segment * const SG70897[] = {SGI0, SGI298126, NULL};
const Segment * const SG70898[] = {SGI0, SGI298127, NULL};
const Segment * const SG70899[] = {SGI0, SGI298128, NULL};
const Segment * const SG70900[] = {SGI0, SGI298129, NULL};
const Segment * const SG70901[] = {SGI298130, SGI298131, NULL};
const Segment * const SG70902[] = {SGI0, SGI298132, NULL};
const Segment * const SG70903[] = {SGI0, SGI298133, NULL};
const Segment * const SG70904[] = {SGI0, SGI298134, NULL};
const Segment * const SG70905[] = {SGI298135, NULL};
const Segment * const SG70906[] = {SGI298136, NULL};
const Segment * const SG70907[] = {SGI298137, SGI298138, NULL};
const Segment * const SG70908[] = {SGI0, SGI298139, NULL};
const Segment * const SG70909[] = {SGI0, SGI298140, NULL};
const Segment * const SG70910[] = {SGI298141, SGI298142, NULL};
const Segment * const SG70911[] = {SGI0, SGI298143, NULL};
const Segment * const SG70912[] = {SGI0, SGI298144, NULL};
const Segment * const SG70913[] = {SGI0, SGI298145, NULL};
const Segment * const SG70914[] = {SGI0, SGI298146, NULL};
const Segment * const SG70915[] = {SGI298147, NULL};
const Segment * const SG70916[] = {SGI298148, NULL};
const Segment * const SG70917[] = {SGI0, SGI298149, NULL};
const Segment * const SG70918[] = {SGI0, SGI298150, NULL};
const Segment * const SG70919[] = {SGI0, SGI298151, NULL};
const Segment * const SG70920[] = {SGI298152, SGI298153, NULL};
const Segment * const SG70921[] = {SGI0, SGI298154, NULL};
const Segment * const SG70922[] = {SGI0, SGI298155, NULL};
const Segment * const SG70923[] = {SGI0, SGI298156, NULL};
const Segment * const SG70924[] = {SGI0, SGI298157, NULL};
const Segment * const SG70925[] = {SGI298158, SGI298159, NULL};
const Segment * const SG70926[] = {SGI0, SGI298160, NULL};
const Segment * const SG70927[] = {SGI0, SGI298161, NULL};
const Segment * const SG70928[] = {SGI0, SGI298162, NULL};
const Segment * const SG70929[] = {SGI0, SGI298163, NULL};
const Segment * const SG70930[] = {SGI0, SGI298164, NULL};
const Segment * const SG70931[] = {SGI298165, NULL};
const Segment * const SG70932[] = {SGI0, SGI298166, NULL};
const Segment * const SG70933[] = {SGI298167, SGI298168, NULL};
const Segment * const SG70934[] = {SGI298169, SGI298170, NULL};
const Segment * const SG70935[] = {SGI0, SGI298171, NULL};
const Segment * const SG70936[] = {SGI298172, SGI298173, NULL};
const Segment * const SG70937[] = {SGI0, SGI298174, NULL};
const Segment * const SG70938[] = {SGI0, SGI298175, NULL};
const Segment * const SG70939[] = {SGI298176, SGI298177, NULL};
const Segment * const SG70940[] = {SGI298178, NULL};
const Segment * const SG70941[] = {SGI298179, NULL};
const Segment * const SG70942[] = {SGI298180, SGI298181, NULL};
const Segment * const SG70943[] = {SGI0, SGI298182, NULL};
const Segment * const SG70944[] = {SGI0, SGI298183, NULL};
const Segment * const SG70945[] = {SGI0, SGI298184, NULL};
const Segment * const SG70946[] = {SGI0, SGI298185, NULL};
const Segment * const SG70947[] = {SGI0, SGI298186, NULL};
const Segment * const SG70948[] = {SGI0, SGI298187, NULL};
const Segment * const SG70949[] = {SGI0, SGI298188, NULL};
const Segment * const SG70950[] = {SGI0, SGI298189, NULL};
const Segment * const SG70951[] = {SGI0, SGI298190, NULL};
const Segment * const SG70952[] = {SGI0, SGI298191, NULL};
const Segment * const SG70953[] = {SGI298192, SGI298193, NULL};
const Segment * const SG70954[] = {SGI0, SGI298194, NULL};
const Segment * const SG70955[] = {SGI298195, SGI298196, NULL};
const Segment * const SG70956[] = {SGI298197, SGI298198, NULL};
const Segment * const SG70957[] = {SGI298199, SGI298200, NULL};
const Segment * const SG70958[] = {SGI298201, NULL};
const Segment * const SG70959[] = {SGI0, SGI298202, NULL};
const Segment * const SG70960[] = {SGI298203, SGI298204, NULL};
const Segment * const SG70961[] = {SGI298205, SGI298206, NULL};
const Segment * const SG70962[] = {SGI298207, SGI298208, NULL};
const Segment * const SG70963[] = {SGI298209, SGI298210, NULL};
const Segment * const SG70964[] = {SGI0, SGI298211, NULL};
const Segment * const SG70965[] = {SGI0, SGI298212, NULL};
const Segment * const SG70966[] = {SGI0, SGI298213, NULL};
const Segment * const SG70967[] = {SGI0, SGI298214, NULL};
const Segment * const SG70968[] = {SGI0, SGI298215, NULL};
const Segment * const SG70969[] = {SGI298216, SGI298217, NULL};
const Segment * const SG70970[] = {SGI0, SGI298218, NULL};
const Segment * const SG70971[] = {SGI298219, SGI298220, NULL};
const Segment * const SG70972[] = {SGI298221, NULL};
const Segment * const SG70973[] = {SGI298222, NULL};
const Segment * const SG70974[] = {SGI0, SGI298223, NULL};
const Segment * const SG70975[] = {SGI0, SGI298224, NULL};
const Segment * const SG70976[] = {SGI0, SGI298225, NULL};
const Segment * const SG70977[] = {SGI0, SGI298226, NULL};
const Segment * const SG70978[] = {SGI0, SGI298227, NULL};
const Segment * const SG70979[] = {SGI0, SGI298228, NULL};
const Segment * const SG70980[] = {SGI0, SGI298229, NULL};
const Segment * const SG70981[] = {SGI298230, SGI298231, NULL};
const Segment * const SG70982[] = {SGI0, SGI298232, NULL};
const Segment * const SG70983[] = {SGI298233, SGI298234, NULL};
const Segment * const SG70984[] = {SGI0, SGI298235, NULL};
const Segment * const SG70985[] = {SGI0, SGI298236, NULL};
const Segment * const SG70986[] = {SGI298237, SGI298238, NULL};
const Segment * const SG70987[] = {SGI0, SGI298239, NULL};
const Segment * const SG70988[] = {SGI298240, SGI298241, NULL};
const Segment * const SG70989[] = {SGI0, SGI298242, NULL};
const Segment * const SG70990[] = {SGI0, SGI298243, NULL};
const Segment * const SG70991[] = {SGI0, SGI298244, NULL};
const Segment * const SG70992[] = {SGI0, SGI298245, NULL};
const Segment * const SG70993[] = {SGI0, SGI298246, NULL};
const Segment * const SG70994[] = {SGI298247, SGI298248, NULL};
const Segment * const SG70995[] = {SGI0, SGI298249, NULL};
const Segment * const SG70996[] = {SGI0, SGI298250, NULL};
const Segment * const SG70997[] = {SGI0, SGI298251, NULL};
const Segment * const SG70998[] = {SGI298252, NULL};
const Segment * const SG70999[] = {SGI0, SGI298253, NULL};
const Segment * const SG71000[] = {SGI298254, NULL};
const Segment * const SG71001[] = {SGI0, SGI298255, NULL};
const Segment * const SG71002[] = {SGI0, SGI298256, NULL};
const Segment * const SG71003[] = {SGI0, SGI298257, NULL};
const Segment * const SG71004[] = {SGI0, SGI298258, NULL};
const Segment * const SG71005[] = {SGI0, SGI298259, NULL};
const Segment * const SG71006[] = {SGI0, SGI298260, NULL};
const Segment * const SG71007[] = {SGI298261, SGI298262, NULL};
const Segment * const SG71008[] = {SGI0, SGI298263, NULL};
const Segment * const SG71009[] = {SGI298264, SGI298265, NULL};
const Segment * const SG71010[] = {SGI0, SGI298266, NULL};
const Segment * const SG71011[] = {SGI298267, NULL};
const Segment * const SG71012[] = {SGI0, SGI298268, NULL};
const Segment * const SG71013[] = {SGI0, SGI298269, NULL};
const Segment * const SG71014[] = {SGI0, SGI298270, NULL};
const Segment * const SG71015[] = {SGI0, SGI298271, NULL};
const Segment * const SG71016[] = {SGI0, SGI298272, NULL};
const Segment * const SG71017[] = {SGI0, SGI298273, NULL};
const Segment * const SG71018[] = {SGI298274, SGI298275, NULL};
const Segment * const SG71019[] = {SGI0, SGI298276, NULL};
const Segment * const SG71020[] = {SGI0, SGI298277, NULL};
const Segment * const SG71021[] = {SGI0, SGI298278, NULL};
const Segment * const SG71022[] = {SGI298279, NULL};
const Segment * const SG71023[] = {SGI0, SGI298280, NULL};
const Segment * const SG71024[] = {SGI0, SGI298281, NULL};
const Segment * const SG71025[] = {SGI0, SGI298282, NULL};
const Segment * const SG71026[] = {SGI0, SGI298283, NULL};
const Segment * const SG71027[] = {SGI0, SGI298284, NULL};
const Segment * const SG71028[] = {SGI0, SGI298285, NULL};
const Segment * const SG71029[] = {SGI298286, SGI298287, NULL};
const Segment * const SG71030[] = {SGI298288, SGI298289, NULL};
const Segment * const SG71031[] = {SGI298290, NULL};
const Segment * const SG71032[] = {SGI0, SGI298291, NULL};
const Segment * const SG71033[] = {SGI298292, NULL};
const Segment * const SG71034[] = {SGI298293, NULL};
const Segment * const SG71035[] = {SGI298294, NULL};
const Segment * const SG71036[] = {SGI298295, NULL};
const Segment * const SG71037[] = {SGI298296, NULL};
const Segment * const SG71038[] = {SGI298297, NULL};
const Segment * const SG71039[] = {SGI298298, NULL};
const Segment * const SG71040[] = {SGI298299, NULL};
const Segment * const SG71041[] = {SGI298300, NULL};
const Segment * const SG71042[] = {SGI298301, NULL};
const Segment * const SG71043[] = {SGI298302, NULL};
const Segment * const SG71044[] = {SGI298303, NULL};
const Segment * const SG71045[] = {SGI298304, NULL};
const Segment * const SG71046[] = {SGI298305, NULL};
const Segment * const SG71047[] = {SGI298306, NULL};
const Segment * const SG71048[] = {SGI298307, NULL};
const Segment * const SG71049[] = {SGI298308, NULL};
const Segment * const SG71050[] = {SGI298309, NULL};
const Segment * const SG71051[] = {SGI298310, NULL};
const Segment * const SG71052[] = {SGI298311, NULL};
const Segment * const SG71053[] = {SGI298312, NULL};
const Segment * const SG71054[] = {SGI298313, NULL};
const Segment * const SG71055[] = {SGI298314, NULL};
const Segment * const SG71056[] = {SGI298315, NULL};
const Segment * const SG71057[] = {SGI298316, NULL};
const Segment * const SG71058[] = {SGI298317, NULL};
const Segment * const SG71059[] = {SGI298318, NULL};
const Segment * const SG71060[] = {SGI298319, NULL};
const Segment * const SG71061[] = {SGI298320, NULL};
const Segment * const SG71062[] = {SGI298321, NULL};
const Segment * const SG71063[] = {SGI298322, NULL};
const Segment * const SG71064[] = {SGI298323, NULL};
const Segment * const SG71065[] = {SGI298324, NULL};
const Segment * const SG71066[] = {SGI298325, NULL};
const Segment * const SG71067[] = {SGI298326, NULL};
const Segment * const SG71068[] = {SGI298327, NULL};
const Segment * const SG71069[] = {SGI298328, NULL};
const Segment * const SG71070[] = {SGI298329, NULL};
const Segment * const SG71071[] = {SGI298330, NULL};
const Segment * const SG71072[] = {SGI298331, NULL};
const Segment * const SG71073[] = {SGI298332, NULL};
const Segment * const SG71074[] = {SGI298333, NULL};
const Segment * const SG71075[] = {SGI298334, NULL};
const Segment * const SG71076[] = {SGI298335, NULL};
const Segment * const SG71077[] = {SGI298336, NULL};
const Segment * const SG71078[] = {SGI298337, NULL};
const Segment * const SG71079[] = {SGI298338, NULL};
const Segment * const SG71080[] = {SGI298339, NULL};
const Segment * const SG71081[] = {SGI298340, NULL};
const Segment * const SG71082[] = {SGI298341, NULL};
const Segment * const SG71083[] = {SGI298342, NULL};
const Segment * const SG71084[] = {SGI298343, NULL};
const Segment * const SG71085[] = {SGI298344, NULL};
const Segment * const SG71086[] = {SGI298345, NULL};
const Segment * const SG71087[] = {SGI298346, SGI0, SGI298347, SGI298348, SGI298349, SGI298350, SGI0, SGI298351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71088[] = {SGI298352, SGI0, SGI298353, SGI298354, SGI298355, SGI298356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298357, SGI0, SGI0, NULL};
const Segment * const SG71089[] = {SGI298358, SGI0, SGI298359, SGI298360, SGI298361, SGI298362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71090[] = {SGI298363, SGI0, SGI298364, SGI298365, SGI298366, SGI298367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298368, SGI0, SGI0, NULL};
const Segment * const SG71091[] = {SGI298369, SGI0, SGI298370, SGI298371, SGI298372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71092[] = {SGI298373, SGI0, SGI298374, SGI298375, SGI298376, SGI298377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298378, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71093[] = {SGI298379, SGI0, SGI0, SGI0, SGI298380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71094[] = {SGI298381, SGI0, SGI298382, SGI298383, SGI298384, SGI298385, SGI0, SGI298386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71095[] = {SGI298387, SGI0, SGI298388, SGI298389, SGI298390, SGI298391, SGI0, SGI298392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298393, SGI0, SGI0, NULL};
const Segment * const SG71096[] = {SGI298394, SGI0, SGI298395, SGI298396, SGI298397, SGI298398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71097[] = {SGI298399, SGI0, SGI298400, SGI298401, SGI298402, SGI298403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298404, SGI0, SGI0, NULL};
const Segment * const SG71098[] = {SGI298405, SGI0, SGI298406, SGI298407, SGI298408, SGI298409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71099[] = {SGI298411, SGI0, SGI298412, SGI298413, SGI298414, SGI298415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298416, SGI0, SGI0, NULL};
const Segment * const SG71100[] = {SGI298417, SGI0, SGI298418, SGI298419, SGI298420, SGI298421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71101[] = {SGI298422, SGI0, SGI298423, SGI298424, SGI298425, SGI298426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298427, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71102[] = {SGI298428, SGI0, SGI298429, SGI298430, SGI298431, SGI298432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298433, SGI298434, SGI0, SGI0, NULL};
const Segment * const SG71103[] = {SGI298435, SGI0, SGI298436, SGI298437, SGI298438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71104[] = {SGI298439, SGI0, SGI298440, SGI298441, SGI298442, SGI298443, SGI0, SGI298444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71105[] = {SGI298445, SGI0, SGI298446, SGI298447, SGI298448, SGI298449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298450, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71106[] = {SGI298451, SGI0, SGI298452, SGI298453, SGI298454, SGI298455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298456, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71107[] = {SGI298457, SGI0, SGI298458, SGI298459, SGI298460, SGI298461, SGI0, SGI0, SGI0, SGI0, SGI298462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298463, SGI0, SGI0, NULL};
const Segment * const SG71108[] = {SGI298464, SGI0, SGI298465, SGI298466, SGI298467, SGI298468, SGI0, SGI0, SGI0, SGI0, SGI298469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71109[] = {SGI298470, SGI0, SGI298471, SGI298472, SGI298473, SGI298474, SGI0, SGI298475, SGI0, SGI0, SGI298476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71110[] = {SGI298477, SGI0, SGI298478, SGI298479, SGI298480, SGI298481, SGI0, SGI0, SGI0, SGI0, SGI298482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298483, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71111[] = {SGI298484, SGI0, SGI298485, SGI298486, SGI298487, SGI298488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71112[] = {SGI298489, SGI0, SGI0, SGI0, SGI298490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71113[] = {SGI298491, SGI0, SGI298492, SGI298493, SGI298494, SGI298495, SGI0, SGI298496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298497, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71114[] = {SGI298498, SGI0, SGI298499, SGI298500, SGI298501, SGI298502, SGI0, SGI298503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298504, SGI0, SGI0, NULL};
const Segment * const SG71115[] = {SGI298505, SGI0, SGI298506, SGI298507, SGI298508, SGI298509, SGI0, SGI298510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298511, SGI0, SGI0, NULL};
const Segment * const SG71116[] = {SGI298512, SGI0, SGI298513, SGI298514, SGI298515, SGI298516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298518, SGI0, SGI0, NULL};
const Segment * const SG71117[] = {SGI298519, SGI0, SGI298520, SGI298521, SGI298522, SGI298523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298524, SGI0, SGI0, NULL};
const Segment * const SG71118[] = {SGI298525, SGI0, SGI298526, SGI298527, SGI298528, SGI298529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298530, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71119[] = {SGI298531, SGI0, SGI298532, SGI298533, SGI298534, SGI298535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71120[] = {SGI298536, SGI0, SGI298537, SGI298538, SGI298539, SGI298540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71121[] = {SGI298541, SGI0, SGI298542, SGI298543, SGI298544, SGI298545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71122[] = {SGI298546, SGI0, SGI298547, SGI298548, SGI298549, SGI298550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71123[] = {SGI298551, SGI0, SGI298552, SGI298553, SGI298554, SGI298555, SGI0, SGI298556, SGI0, SGI0, SGI298557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298558, SGI0, SGI0, NULL};
const Segment * const SG71124[] = {SGI298559, SGI0, SGI298560, SGI298561, SGI298562, SGI298563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298564, SGI0, SGI0, SGI298565, SGI0, SGI0, NULL};
const Segment * const SG71125[] = {SGI298566, SGI0, SGI298567, SGI298568, SGI298569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71126[] = {SGI298570, SGI0, SGI0, SGI0, SGI298571, SGI298572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71127[] = {SGI298573, SGI0, SGI298574, SGI298575, SGI298576, SGI298577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71128[] = {SGI298578, SGI0, SGI298579, SGI298580, SGI298581, SGI298582, SGI0, SGI298583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71129[] = {SGI298584, SGI0, SGI298585, SGI298586, SGI298587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71130[] = {SGI298588, SGI0, SGI298589, SGI298590, SGI298591, SGI298592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298594, SGI0, SGI0, NULL};
const Segment * const SG71131[] = {SGI298595, SGI0, SGI298596, SGI298597, SGI298598, SGI298599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71132[] = {SGI298600, SGI0, SGI298601, SGI298602, SGI298603, SGI298604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71133[] = {SGI298605, SGI0, SGI298606, SGI298607, SGI298608, SGI298609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71134[] = {SGI298610, SGI0, SGI298611, SGI298612, SGI298613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71135[] = {SGI298614, SGI0, SGI298615, SGI298616, SGI298617, SGI298618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298619, SGI0, SGI0, SGI298620, SGI0, SGI0, NULL};
const Segment * const SG71136[] = {SGI298621, SGI0, SGI298622, SGI298623, SGI298624, SGI298625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71137[] = {SGI298627, SGI0, SGI298628, SGI298629, SGI298630, SGI298631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298632, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71138[] = {SGI298633, SGI0, SGI298634, SGI298635, SGI298636, SGI298637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71139[] = {SGI298638, SGI0, SGI298639, SGI298640, SGI298641, SGI298642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71140[] = {SGI298643, SGI0, SGI298644, SGI298645, SGI298646, SGI298647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71141[] = {SGI298648, SGI0, SGI298649, SGI298650, SGI298651, SGI298652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298653, SGI0, SGI0, NULL};
const Segment * const SG71142[] = {SGI298654, SGI0, SGI298655, SGI298656, SGI298657, SGI298658, SGI0, SGI298659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71143[] = {SGI298660, SGI0, SGI298661, SGI298662, SGI298663, SGI298664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298665, SGI298666, SGI0, SGI0, NULL};
const Segment * const SG71144[] = {SGI298667, SGI0, SGI298668, SGI298669, SGI298670, SGI298671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298672, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71145[] = {SGI298673, SGI0, SGI298674, SGI298675, SGI298676, SGI298677, SGI0, SGI298678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71146[] = {SGI298680, SGI0, SGI298681, SGI298682, SGI298683, SGI298684, SGI0, SGI298685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298686, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71147[] = {SGI298687, SGI0, SGI298688, SGI298689, SGI298690, SGI298691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71148[] = {SGI298692, SGI0, SGI298693, SGI298694, SGI298695, SGI298696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298697, SGI0, SGI0, NULL};
const Segment * const SG71149[] = {SGI298698, SGI0, SGI298699, SGI298700, SGI298701, SGI298702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71150[] = {SGI298703, SGI0, SGI298704, SGI298705, SGI298706, SGI298707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71151[] = {SGI298708, SGI0, SGI298709, SGI298710, SGI298711, SGI298712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71152[] = {SGI298713, SGI0, SGI298714, SGI298715, SGI298716, SGI298717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71153[] = {SGI298718, SGI0, SGI298719, SGI298720, SGI298721, SGI298722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71154[] = {SGI298723, SGI0, SGI298724, SGI298725, SGI298726, SGI298727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298728, SGI0, SGI0, NULL};
const Segment * const SG71155[] = {SGI298729, SGI0, SGI298730, SGI298731, SGI298732, SGI298733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71156[] = {SGI298734, SGI0, SGI298735, SGI298736, SGI298737, SGI298738, SGI0, SGI298739, SGI0, SGI0, SGI298740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71157[] = {SGI298741, SGI0, SGI298742, SGI298743, SGI298744, SGI298745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298746, SGI0, SGI0, NULL};
const Segment * const SG71158[] = {SGI298747, SGI0, SGI298748, SGI298749, SGI298750, SGI298751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298752, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71159[] = {SGI298753, SGI0, SGI298754, SGI298755, SGI298756, SGI298757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71160[] = {SGI298758, SGI0, SGI298759, SGI298760, SGI298761, SGI298762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298763, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71161[] = {SGI298764, SGI0, SGI298765, SGI298766, SGI298767, SGI298768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298769, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71162[] = {SGI298770, SGI0, SGI298771, SGI298772, SGI298773, SGI298774, SGI0, SGI298775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298776, SGI0, SGI0, NULL};
const Segment * const SG71163[] = {SGI298777, SGI0, SGI298778, SGI298779, SGI298780, SGI298781, SGI0, SGI0, SGI0, SGI0, SGI298782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298783, SGI0, SGI0, NULL};
const Segment * const SG71164[] = {SGI298784, SGI0, SGI298785, SGI298786, SGI298787, SGI298788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71165[] = {SGI298789, SGI0, SGI298790, SGI298791, SGI298792, SGI298793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71166[] = {SGI298794, SGI0, SGI298795, SGI298796, SGI298797, SGI298798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298799, SGI0, SGI0, NULL};
const Segment * const SG71167[] = {SGI298800, SGI0, SGI298801, SGI298802, SGI298803, SGI298804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298806, SGI0, SGI0, NULL};
const Segment * const SG71168[] = {SGI298807, SGI0, SGI298808, SGI298809, SGI298810, SGI298811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71169[] = {SGI298812, SGI0, SGI298813, SGI298814, SGI298815, SGI298816, SGI0, SGI298817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71170[] = {SGI298818, SGI0, SGI298819, SGI298820, SGI298821, SGI298822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298823, SGI0, SGI0, NULL};
const Segment * const SG71171[] = {SGI298824, SGI0, SGI298825, SGI298826, SGI298827, SGI298828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298829, SGI0, SGI0, NULL};
const Segment * const SG71172[] = {SGI298830, SGI0, SGI298831, SGI298832, SGI298833, SGI298834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298835, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71173[] = {SGI298836, SGI0, SGI298837, SGI298838, SGI298839, SGI298840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71174[] = {SGI298841, SGI0, SGI298842, SGI298843, SGI298844, SGI298845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71175[] = {SGI298846, SGI0, SGI298847, SGI298848, SGI298849, SGI298850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71176[] = {SGI298851, SGI0, SGI298852, SGI298853, SGI298854, SGI298855, SGI0, SGI298856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298857, SGI0, SGI0, NULL};
const Segment * const SG71177[] = {SGI298858, SGI0, SGI298859, SGI298860, SGI298861, SGI298862, SGI0, SGI298863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298864, SGI0, SGI0, NULL};
const Segment * const SG71178[] = {SGI298865, SGI0, SGI298866, SGI298867, SGI298868, SGI298869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71179[] = {SGI298870, SGI0, SGI298871, SGI298872, SGI298873, SGI298874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71180[] = {SGI298876, SGI0, SGI298877, SGI298878, SGI298879, SGI298880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71181[] = {SGI298881, SGI0, SGI298882, SGI298883, SGI298884, SGI298885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71182[] = {SGI298886, SGI0, SGI298887, SGI298888, SGI298889, SGI298890, SGI0, SGI298891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71183[] = {SGI298892, SGI0, SGI298893, SGI298894, SGI298895, SGI298896, SGI0, SGI298897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298898, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71184[] = {SGI298899, SGI0, SGI298900, SGI298901, SGI298902, SGI298903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298904, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71185[] = {SGI298905, SGI0, SGI298906, SGI298907, SGI298908, SGI298909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71186[] = {SGI298910, SGI0, SGI298911, SGI298912, SGI298913, SGI298914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71187[] = {SGI298915, SGI0, SGI298916, SGI298917, SGI298918, SGI298919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71188[] = {SGI298920, SGI0, SGI298921, SGI298922, SGI298923, SGI298924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298925, SGI0, SGI0, SGI298926, SGI0, SGI0, NULL};
const Segment * const SG71189[] = {SGI298927, SGI0, SGI298928, SGI298929, SGI298930, SGI298931, SGI0, SGI298932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298933, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71190[] = {SGI298934, SGI0, SGI298935, SGI298936, SGI298937, SGI298938, SGI0, SGI298939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71191[] = {SGI0, SGI0, SGI298940, SGI298941, SGI298942, SGI298943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71192[] = {SGI298944, SGI0, SGI298945, SGI298946, SGI298947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71193[] = {SGI298948, SGI0, SGI298949, SGI298950, SGI298951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71194[] = {SGI298952, SGI0, SGI298953, SGI298954, SGI298955, SGI298956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298957, SGI0, SGI0, NULL};
const Segment * const SG71195[] = {SGI298958, SGI0, SGI298959, SGI298960, SGI298961, SGI298962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298963, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71196[] = {SGI298964, SGI0, SGI298965, SGI298966, SGI298967, SGI298968, SGI0, SGI298969, SGI0, SGI0, SGI298970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298971, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71197[] = {SGI298972, SGI0, SGI298973, SGI298974, SGI298975, SGI298976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298977, SGI0, SGI0, NULL};
const Segment * const SG71198[] = {SGI298978, SGI0, SGI298979, SGI298980, SGI298981, SGI298982, SGI0, SGI298983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298984, SGI0, SGI0, NULL};
const Segment * const SG71199[] = {SGI298985, SGI0, SGI298986, SGI298987, SGI298988, SGI298989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298990, SGI0, NULL};
const Segment * const SG71200[] = {SGI298991, SGI0, SGI298992, SGI298993, SGI298994, SGI298995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI298996, SGI298997, SGI0, NULL};
const Segment * const SG71201[] = {SGI298998, SGI0, SGI298999, SGI299000, SGI299001, SGI299002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71202[] = {SGI299003, SGI0, SGI299004, SGI299005, SGI299006, SGI299007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71203[] = {SGI299008, SGI0, SGI299009, SGI299010, SGI299011, SGI299012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71204[] = {SGI299013, SGI0, SGI299014, SGI299015, SGI299016, SGI299017, SGI0, SGI299018, SGI0, SGI0, SGI299019, SGI0, SGI0, SGI299020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71205[] = {SGI299021, SGI0, SGI299022, SGI299023, SGI299024, SGI299025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299026, SGI0, NULL};
const Segment * const SG71206[] = {SGI299027, SGI0, SGI299028, SGI299029, SGI299030, SGI299031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299032, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71207[] = {SGI0, SGI0, SGI299033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71208[] = {SGI299034, SGI0, SGI299035, SGI299036, SGI299037, SGI299038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299039, SGI299040, SGI0, NULL};
const Segment * const SG71209[] = {SGI299041, SGI0, SGI299042, SGI299043, SGI299044, SGI299045, SGI0, SGI299046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71210[] = {SGI299048, SGI0, SGI299049, SGI299050, SGI299051, SGI299052, SGI0, SGI299053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71211[] = {SGI299055, SGI0, SGI299056, SGI299057, SGI299058, SGI299059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71212[] = {SGI299061, SGI0, SGI299062, SGI299063, SGI299064, SGI299065, SGI0, SGI299066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299067, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71213[] = {SGI299068, SGI0, SGI299069, SGI299070, SGI299071, SGI299072, SGI0, SGI299073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299075, SGI0, SGI0, NULL};
const Segment * const SG71214[] = {SGI299076, SGI0, SGI299077, SGI299078, SGI299079, SGI299080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299081, SGI0, SGI0, NULL};
const Segment * const SG71215[] = {SGI299082, SGI0, SGI299083, SGI299084, SGI299085, SGI299086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299087, SGI0, SGI0, NULL};
const Segment * const SG71216[] = {SGI299088, SGI0, SGI299089, SGI299090, SGI299091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71217[] = {SGI299092, SGI0, SGI0, SGI0, SGI299093, SGI299094, SGI0, SGI299095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71218[] = {SGI299096, SGI0, SGI299097, SGI299098, SGI299099, SGI299100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71219[] = {SGI299101, SGI0, SGI299102, SGI299103, SGI299104, SGI299105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71220[] = {SGI299106, SGI0, SGI299107, SGI299108, SGI299109, SGI299110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71221[] = {SGI299112, SGI0, SGI299113, SGI299114, SGI299115, SGI299116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299118, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71222[] = {SGI299119, SGI0, SGI299120, SGI299121, SGI299122, SGI299123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299124, SGI0, SGI0, NULL};
const Segment * const SG71223[] = {SGI299125, SGI0, SGI299126, SGI299127, SGI299128, SGI299129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299130, SGI0, SGI0, NULL};
const Segment * const SG71224[] = {SGI299131, SGI0, SGI299132, SGI299133, SGI299134, SGI299135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299136, SGI0, SGI0, NULL};
const Segment * const SG71225[] = {SGI299137, SGI0, SGI299138, SGI299139, SGI299140, SGI299141, SGI0, SGI299142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299143, SGI299144, SGI0, SGI0, NULL};
const Segment * const SG71226[] = {SGI299145, SGI0, SGI299146, SGI299147, SGI299148, SGI299149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71227[] = {SGI299150, SGI0, SGI299151, SGI299152, SGI299153, SGI299154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71228[] = {SGI299155, SGI0, SGI299156, SGI299157, SGI299158, SGI299159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71229[] = {SGI299160, SGI0, SGI299161, SGI299162, SGI299163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71230[] = {SGI299164, SGI0, SGI299165, SGI299166, SGI299167, SGI299168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71231[] = {SGI299169, SGI0, SGI299170, SGI299171, SGI299172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71232[] = {SGI299173, SGI0, SGI299174, SGI299175, SGI299176, SGI299177, SGI0, SGI299178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71233[] = {SGI299179, SGI0, SGI299180, SGI299181, SGI299182, SGI299183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299184, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71234[] = {SGI299185, SGI0, SGI299186, SGI299187, SGI299188, SGI299189, SGI0, SGI299190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299191, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71235[] = {SGI299192, SGI0, SGI299193, SGI299194, SGI299195, SGI299196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71236[] = {SGI299198, SGI0, SGI299199, SGI299200, SGI299201, SGI299202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71237[] = {SGI299203, SGI0, SGI299204, SGI299205, SGI299206, SGI299207, SGI0, SGI299208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299209, SGI0, SGI299210, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71238[] = {SGI299211, SGI0, SGI299212, SGI299213, SGI299214, SGI299215, SGI0, SGI299216, SGI0, SGI0, SGI299217, SGI0, SGI0, SGI299218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71239[] = {SGI299219, SGI0, SGI299220, SGI299221, SGI299222, SGI299223, SGI0, SGI299224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71240[] = {SGI299226, SGI0, SGI299227, SGI299228, SGI299229, SGI299230, SGI0, SGI299231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299232, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71241[] = {SGI299233, SGI0, SGI299234, SGI299235, SGI299236, SGI299237, SGI0, SGI299238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299240, SGI0, SGI0, NULL};
const Segment * const SG71242[] = {SGI299241, SGI0, SGI299242, SGI299243, SGI299244, SGI299245, SGI0, SGI299246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71243[] = {SGI299247, SGI0, SGI299248, SGI299249, SGI299250, SGI299251, SGI0, SGI299252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299253, SGI0, SGI0, SGI299254, SGI0, SGI0, NULL};
const Segment * const SG71244[] = {SGI299255, SGI0, SGI0, SGI0, SGI299256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71245[] = {SGI299257, SGI0, SGI299258, SGI299259, SGI299260, SGI299261, SGI0, SGI299262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299263, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71246[] = {SGI299264, SGI0, SGI299265, SGI299266, SGI299267, SGI299268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299269, SGI299270, SGI0, SGI0, NULL};
const Segment * const SG71247[] = {SGI299271, SGI0, SGI299272, SGI299273, SGI299274, SGI299275, SGI0, SGI299276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71248[] = {SGI299277, SGI0, SGI299278, SGI299279, SGI299280, SGI299281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299282, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71249[] = {SGI299283, SGI0, SGI299284, SGI299285, SGI299286, SGI299287, SGI0, SGI299288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299289, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71250[] = {SGI299290, SGI0, SGI299291, SGI299292, SGI299293, SGI299294, SGI0, SGI299295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299296, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71251[] = {SGI299297, SGI0, SGI299298, SGI299299, SGI299300, SGI299301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299302, SGI0, SGI0, NULL};
const Segment * const SG71252[] = {SGI299303, SGI0, SGI299304, SGI299305, SGI299306, SGI299307, SGI0, SGI299308, SGI0, SGI0, SGI299309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299310, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71253[] = {SGI299311, SGI0, SGI299312, SGI299313, SGI299314, SGI299315, SGI0, SGI299316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299317, SGI0, SGI299318, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71254[] = {SGI299319, SGI0, SGI299320, SGI299321, SGI299322, SGI299323, SGI0, SGI299324, SGI0, SGI0, SGI299325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299326, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71255[] = {SGI299327, SGI0, SGI299328, SGI299329, SGI299330, SGI299331, SGI0, SGI299332, SGI0, SGI0, SGI299333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71256[] = {SGI299334, SGI0, SGI299335, SGI299336, SGI299337, SGI299338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299340, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71257[] = {SGI299341, SGI0, SGI299342, SGI299343, SGI299344, SGI299345, SGI0, SGI299346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299347, SGI0, SGI299348, SGI299349, SGI0, SGI0, NULL};
const Segment * const SG71258[] = {SGI299350, SGI0, SGI299351, SGI299352, SGI299353, SGI299354, SGI0, SGI299355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299356, SGI299357, SGI0, SGI0, NULL};
const Segment * const SG71259[] = {SGI299358, SGI0, SGI299359, SGI299360, SGI299361, SGI299362, SGI0, SGI299363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299364, SGI299365, SGI0, SGI0, NULL};
const Segment * const SG71260[] = {SGI299366, SGI0, SGI299367, SGI299368, SGI299369, SGI299370, SGI0, SGI299371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299372, SGI0, SGI0, NULL};
const Segment * const SG71261[] = {SGI299373, SGI0, SGI299374, SGI299375, SGI299376, SGI299377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299378, SGI0, SGI299379, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71262[] = {SGI299380, SGI0, SGI299381, SGI299382, SGI299383, SGI299384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299386, SGI0, SGI0, NULL};
const Segment * const SG71263[] = {SGI299387, SGI0, SGI299388, SGI299389, SGI299390, SGI299391, SGI0, SGI299392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71264[] = {SGI299393, SGI0, SGI299394, SGI0, SGI299395, SGI299396, SGI0, SGI299397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71265[] = {SGI299399, SGI0, SGI299400, SGI299401, SGI299402, SGI299403, SGI0, SGI299404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71266[] = {SGI299405, SGI0, SGI299406, SGI299407, SGI299408, SGI299409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299410, SGI0, SGI0, NULL};
const Segment * const SG71267[] = {SGI299411, SGI0, SGI299412, SGI299413, SGI299414, SGI299415, SGI0, SGI299416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299417, SGI0, SGI0, NULL};
const Segment * const SG71268[] = {SGI299418, SGI0, SGI299419, SGI299420, SGI299421, SGI299422, SGI0, SGI299423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71269[] = {SGI299424, SGI0, SGI299425, SGI299426, SGI299427, SGI299428, SGI0, SGI299429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71270[] = {SGI299431, SGI0, SGI299432, SGI299433, SGI299434, SGI299435, SGI0, SGI0, SGI0, SGI0, SGI299436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71271[] = {SGI299437, SGI0, SGI299438, SGI299439, SGI299440, SGI299441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299442, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71272[] = {SGI299443, SGI0, SGI299444, SGI299445, SGI299446, SGI299447, SGI0, SGI299448, SGI0, SGI0, SGI299449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299450, SGI0, SGI299451, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71273[] = {SGI299452, SGI0, SGI299453, SGI299454, SGI299455, SGI299456, SGI0, SGI299457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299458, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71274[] = {SGI299459, SGI0, SGI299460, SGI299461, SGI299462, SGI299463, SGI0, SGI299464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299465, SGI0, SGI0, NULL};
const Segment * const SG71275[] = {SGI299466, SGI0, SGI299467, SGI299468, SGI299469, SGI299470, SGI0, SGI299471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71276[] = {SGI299472, SGI0, SGI299473, SGI299474, SGI299475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71277[] = {SGI299476, SGI0, SGI299477, SGI299478, SGI299479, SGI299480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71278[] = {SGI299481, SGI0, SGI299482, SGI299483, SGI299484, SGI299485, SGI0, SGI0, SGI0, SGI0, SGI299486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299487, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71279[] = {SGI299488, SGI0, SGI299489, SGI299490, SGI299491, SGI299492, SGI0, SGI299493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299494, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71280[] = {SGI299495, SGI0, SGI299496, SGI299497, SGI299498, SGI299499, SGI0, SGI299500, SGI0, SGI0, SGI299501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299502, SGI0, SGI0, NULL};
const Segment * const SG71281[] = {SGI299503, SGI0, SGI299504, SGI299505, SGI299506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71282[] = {SGI299507, SGI0, SGI299508, SGI299509, SGI299510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71283[] = {SGI299511, SGI0, SGI299512, SGI299513, SGI299514, SGI299515, SGI0, SGI299516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299517, SGI0, SGI0, SGI299518, SGI0, SGI0, NULL};
const Segment * const SG71284[] = {SGI299519, SGI0, SGI299520, SGI299521, SGI299522, SGI299523, SGI0, SGI299524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299526, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71285[] = {SGI299527, SGI0, SGI299528, SGI299529, SGI299530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71286[] = {SGI299531, SGI0, SGI299532, SGI299533, SGI299534, SGI299535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71287[] = {SGI299536, SGI0, SGI299537, SGI299538, SGI299539, SGI299540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71288[] = {SGI299542, SGI0, SGI299543, SGI299544, SGI299545, SGI299546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71289[] = {SGI299547, SGI0, SGI299548, SGI299549, SGI299550, SGI299551, SGI0, SGI299552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71290[] = {SGI299554, SGI0, SGI299555, SGI299556, SGI299557, SGI299558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299559, SGI299560, SGI0, SGI0, NULL};
const Segment * const SG71291[] = {SGI299561, SGI0, SGI299562, SGI299563, SGI299564, SGI299565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299567, SGI0, SGI0, NULL};
const Segment * const SG71292[] = {SGI299568, SGI0, SGI299569, SGI299570, SGI299571, SGI299572, SGI0, SGI299573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299574, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71293[] = {SGI299575, SGI0, SGI299576, SGI299577, SGI299578, SGI299579, SGI0, SGI299580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299581, SGI0, SGI0, NULL};
const Segment * const SG71294[] = {SGI299582, SGI0, SGI299583, SGI299584, SGI299585, SGI299586, SGI0, SGI299587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71295[] = {SGI299588, SGI0, SGI299589, SGI299590, SGI299591, SGI299592, SGI0, SGI299593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71296[] = {SGI299595, SGI0, SGI299596, SGI299597, SGI299598, SGI299599, SGI0, SGI299600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299601, SGI0, SGI0, NULL};
const Segment * const SG71297[] = {SGI299602, SGI0, SGI299603, SGI299604, SGI299605, SGI299606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71298[] = {SGI299608, SGI0, SGI299609, SGI299610, SGI299611, SGI299612, SGI0, SGI299613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299614, SGI0, SGI0, NULL};
const Segment * const SG71299[] = {SGI299615, SGI0, SGI299616, SGI299617, SGI299618, SGI299619, SGI0, SGI299620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299621, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71300[] = {SGI299622, SGI0, SGI299623, SGI299624, SGI299625, SGI299626, SGI0, SGI299627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299628, SGI0, SGI0, NULL};
const Segment * const SG71301[] = {SGI299629, SGI0, SGI299630, SGI299631, SGI299632, SGI299633, SGI0, SGI0, SGI0, SGI0, SGI299634, SGI0, SGI0, SGI299635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71302[] = {SGI299636, SGI0, SGI299637, SGI299638, SGI299639, SGI299640, SGI0, SGI299641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71303[] = {SGI299642, SGI0, SGI299643, SGI299644, SGI299645, SGI299646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299647, SGI0, SGI0, SGI299648, SGI0, SGI0, NULL};
const Segment * const SG71304[] = {SGI299649, SGI0, SGI299650, SGI299651, SGI299652, SGI299653, SGI0, SGI299654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299655, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71305[] = {SGI299656, SGI0, SGI299657, SGI299658, SGI299659, SGI299660, SGI0, SGI299661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71306[] = {SGI299662, SGI0, SGI299663, SGI299664, SGI299665, SGI299666, SGI0, SGI299667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299668, SGI299669, SGI0, SGI0, NULL};
const Segment * const SG71307[] = {SGI299670, SGI0, SGI299671, SGI299672, SGI299673, SGI299674, SGI0, SGI299675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71308[] = {SGI299676, SGI0, SGI299677, SGI299678, SGI299679, SGI299680, SGI0, SGI299681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299682, SGI0, SGI0, NULL};
const Segment * const SG71309[] = {SGI299683, SGI0, SGI299684, SGI299685, SGI299686, SGI299687, SGI0, SGI299688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71310[] = {SGI299689, SGI0, SGI299690, SGI299691, SGI299692, SGI299693, SGI0, SGI299694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299695, SGI299696, SGI0, SGI0, NULL};
const Segment * const SG71311[] = {SGI299697, SGI0, SGI299698, SGI299699, SGI299700, SGI299701, SGI0, SGI0, SGI0, SGI0, SGI299702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299703, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71312[] = {SGI299704, SGI0, SGI299705, SGI299706, SGI299707, SGI299708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299709, SGI0, SGI0, SGI299710, SGI0, SGI0, NULL};
const Segment * const SG71313[] = {SGI299711, SGI0, SGI299712, SGI299713, SGI299714, SGI299715, SGI0, SGI299716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299717, SGI299718, SGI0, SGI0, NULL};
const Segment * const SG71314[] = {SGI299719, SGI0, SGI299720, SGI299721, SGI299722, SGI299723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299724, SGI0, SGI299725, SGI299726, SGI0, SGI0, NULL};
const Segment * const SG71315[] = {SGI299727, SGI0, SGI299728, SGI299729, SGI299730, SGI299731, SGI0, SGI299732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299733, SGI0, SGI0, NULL};
const Segment * const SG71316[] = {SGI299734, SGI0, SGI299735, SGI299736, SGI299737, SGI299738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299739, SGI0, SGI0, NULL};
const Segment * const SG71317[] = {SGI299740, SGI0, SGI299741, SGI299742, SGI299743, SGI299744, SGI0, SGI299745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71318[] = {SGI299747, SGI0, SGI299748, SGI299749, SGI299750, SGI299751, SGI0, SGI299752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299753, SGI0, SGI299754, SGI299755, SGI0, SGI0, NULL};
const Segment * const SG71319[] = {SGI299756, SGI0, SGI299757, SGI299758, SGI299759, SGI299760, SGI0, SGI299761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299762, SGI0, SGI299763, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71320[] = {SGI299764, SGI0, SGI299765, SGI299766, SGI299767, SGI299768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299769, SGI0, SGI0, NULL};
const Segment * const SG71321[] = {SGI299770, SGI0, SGI299771, SGI299772, SGI299773, SGI299774, SGI0, SGI299775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71322[] = {SGI299776, SGI0, SGI299777, SGI299778, SGI299779, SGI0, SGI0, SGI299780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71323[] = {SGI299781, SGI0, SGI299782, SGI299783, SGI299784, SGI299785, SGI0, SGI299786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299788, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71324[] = {SGI299789, SGI0, SGI299790, SGI299791, SGI299792, SGI299793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71325[] = {SGI299794, SGI0, SGI299795, SGI299796, SGI299797, SGI299798, SGI0, SGI299799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71326[] = {SGI299800, SGI0, SGI299801, SGI299802, SGI299803, SGI299804, SGI0, SGI299805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299806, SGI0, SGI0, NULL};
const Segment * const SG71327[] = {SGI299807, SGI0, SGI299808, SGI299809, SGI299810, SGI299811, SGI0, SGI299812, SGI0, SGI0, SGI299813, SGI0, SGI0, SGI299814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299815, SGI0, SGI0, NULL};
const Segment * const SG71328[] = {SGI299816, SGI0, SGI299817, SGI299818, SGI299819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71329[] = {SGI299820, SGI0, SGI299821, SGI299822, SGI299823, SGI299824, SGI0, SGI299825, SGI0, SGI0, SGI299826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299827, SGI299828, SGI0, SGI0, NULL};
const Segment * const SG71330[] = {SGI299829, SGI0, SGI299830, SGI299831, SGI299832, SGI299833, SGI0, SGI299834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299835, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71331[] = {SGI299836, SGI0, SGI299837, SGI299838, SGI299839, SGI299840, SGI0, SGI299841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71332[] = {SGI299843, SGI0, SGI299844, SGI299845, SGI299846, SGI299847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71333[] = {SGI299848, SGI0, SGI299849, SGI299850, SGI299851, SGI299852, SGI0, SGI299853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299854, SGI299855, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71334[] = {SGI299856, SGI0, SGI299857, SGI299858, SGI299859, SGI299860, SGI0, SGI299861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299863, SGI0, SGI0, NULL};
const Segment * const SG71335[] = {SGI299864, SGI0, SGI299865, SGI299866, SGI299867, SGI299868, SGI0, SGI299869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299870, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71336[] = {SGI299871, SGI0, SGI299872, SGI299873, SGI299874, SGI299875, SGI0, SGI299876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299878, SGI0, SGI0, NULL};
const Segment * const SG71337[] = {SGI299879, SGI0, SGI299880, SGI299881, SGI299882, SGI299883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299885, SGI0, SGI0, NULL};
const Segment * const SG71338[] = {SGI299886, SGI0, SGI299887, SGI299888, SGI299889, SGI299890, SGI0, SGI299891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71339[] = {SGI299893, SGI0, SGI299894, SGI299895, SGI299896, SGI299897, SGI0, SGI299898, SGI0, SGI0, SGI299899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71340[] = {SGI299900, SGI0, SGI299901, SGI299902, SGI299903, SGI299904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71341[] = {SGI299905, SGI0, SGI299906, SGI299907, SGI299908, SGI299909, SGI0, SGI299910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299912, SGI0, SGI0, NULL};
const Segment * const SG71342[] = {SGI299913, SGI0, SGI299914, SGI299915, SGI299916, SGI299917, SGI0, SGI299918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71343[] = {SGI299920, SGI0, SGI299921, SGI299922, SGI299923, SGI299924, SGI0, SGI299925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299926, SGI0, SGI0, SGI299927, SGI0, SGI0, NULL};
const Segment * const SG71344[] = {SGI299928, SGI0, SGI299929, SGI299930, SGI299931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71345[] = {SGI299932, SGI0, SGI299933, SGI299934, SGI299935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71346[] = {SGI299936, SGI0, SGI0, SGI0, SGI299937, SGI299938, SGI0, SGI299939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71347[] = {SGI299941, SGI0, SGI299942, SGI299943, SGI299944, SGI299945, SGI0, SGI299946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299947, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71348[] = {SGI299948, SGI0, SGI299949, SGI299950, SGI299951, SGI299952, SGI0, SGI299953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71349[] = {SGI299955, SGI0, SGI299956, SGI299957, SGI299958, SGI299959, SGI0, SGI299960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299962, SGI0, SGI0, NULL};
const Segment * const SG71350[] = {SGI299963, SGI0, SGI299964, SGI299965, SGI299966, SGI299967, SGI0, SGI299968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299969, SGI299970, SGI0, SGI0, NULL};
const Segment * const SG71351[] = {SGI299971, SGI0, SGI299972, SGI299973, SGI299974, SGI299975, SGI0, SGI299976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299977, SGI0, NULL};
const Segment * const SG71352[] = {SGI299978, SGI0, SGI299979, SGI299980, SGI299981, SGI299982, SGI0, SGI299983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299985, SGI0, NULL};
const Segment * const SG71353[] = {SGI299986, SGI0, SGI299987, SGI299988, SGI299989, SGI299990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71354[] = {SGI299992, SGI0, SGI299993, SGI299994, SGI299995, SGI299996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI299997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71355[] = {SGI299998, SGI0, SGI299999, SGI300000, SGI300001, SGI300002, SGI0, SGI300003, SGI0, SGI0, SGI300004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300005, SGI0, SGI0, SGI300006, SGI0, SGI0, NULL};
const Segment * const SG71356[] = {SGI300007, SGI0, SGI300008, SGI300009, SGI300010, SGI300011, SGI0, SGI300012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300013, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71357[] = {SGI300014, SGI0, SGI300015, SGI300016, SGI300017, SGI300018, SGI0, SGI0, SGI0, SGI0, SGI300019, SGI0, SGI0, SGI300020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300021, SGI0, SGI0, NULL};
const Segment * const SG71358[] = {SGI300022, SGI0, SGI300023, SGI300024, SGI300025, SGI300026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300027, SGI0, NULL};
const Segment * const SG71359[] = {SGI300028, SGI0, SGI300029, SGI300030, SGI300031, SGI300032, SGI0, SGI300033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300034, SGI0, NULL};
const Segment * const SG71360[] = {SGI300035, SGI0, SGI300036, SGI300037, SGI300038, SGI300039, SGI0, SGI300040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300042, SGI0, NULL};
const Segment * const SG71361[] = {SGI300043, SGI0, SGI300044, SGI300045, SGI300046, SGI300047, SGI0, SGI300048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71362[] = {SGI300050, SGI0, SGI300051, SGI300052, SGI300053, SGI300054, SGI0, SGI300055, SGI0, SGI0, SGI300056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71363[] = {SGI300057, SGI0, SGI300058, SGI300059, SGI300060, SGI300061, SGI0, SGI300062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300063, SGI300064, SGI0, SGI0, NULL};
const Segment * const SG71364[] = {SGI300065, SGI0, SGI300066, SGI300067, SGI300068, SGI300069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300070, SGI0, SGI0, NULL};
const Segment * const SG71365[] = {SGI300071, SGI0, SGI300072, SGI300073, SGI300074, SGI300075, SGI0, SGI300076, SGI0, SGI0, SGI300077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300078, SGI0, SGI300079, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71366[] = {SGI300080, SGI0, SGI300081, SGI300082, SGI300083, SGI300084, SGI0, SGI300085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71367[] = {SGI300087, SGI0, SGI300088, SGI300089, SGI300090, SGI0, SGI0, SGI300091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300092, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71368[] = {SGI300093, SGI0, SGI300094, SGI300095, SGI300096, SGI300097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300098, SGI0, SGI0, NULL};
const Segment * const SG71369[] = {SGI300099, SGI0, SGI300100, SGI300101, SGI300102, SGI300103, SGI0, SGI300104, SGI0, SGI0, SGI300105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300106, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71370[] = {SGI300107, SGI0, SGI300108, SGI300109, SGI300110, SGI300111, SGI0, SGI300112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71371[] = {SGI300114, SGI0, SGI300115, SGI300116, SGI300117, SGI300118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300119, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71372[] = {SGI300120, SGI0, SGI300121, SGI300122, SGI300123, SGI300124, SGI0, SGI0, SGI0, SGI0, SGI300125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300126, SGI0, SGI300127, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71373[] = {SGI300128, SGI0, SGI300129, SGI300130, SGI300131, SGI300132, SGI0, SGI300133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300134, SGI0, SGI0, NULL};
const Segment * const SG71374[] = {SGI300135, SGI0, SGI300136, SGI300137, SGI300138, SGI300139, SGI0, SGI300140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71375[] = {SGI300141, SGI0, SGI300142, SGI300143, SGI300144, SGI300145, SGI0, SGI300146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300147, SGI0, SGI0, NULL};
const Segment * const SG71376[] = {SGI300148, SGI0, SGI300149, SGI300150, SGI300151, SGI300152, SGI0, SGI300153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300154, SGI0, SGI300155, SGI0, NULL};
const Segment * const SG71377[] = {SGI0, SGI0, SGI300156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71378[] = {SGI300158, SGI0, SGI300159, SGI300160, SGI300161, SGI300162, SGI0, SGI300163, SGI0, SGI0, SGI300164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300165, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71379[] = {SGI300166, SGI0, SGI300167, SGI300168, SGI300169, SGI300170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71380[] = {SGI300171, SGI0, SGI300172, SGI300173, SGI300174, SGI300175, SGI0, SGI300176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71381[] = {SGI300178, SGI0, SGI300179, SGI300180, SGI300181, SGI300182, SGI0, SGI300183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300184, SGI0, SGI0, SGI300185, SGI0, SGI0, NULL};
const Segment * const SG71382[] = {SGI300186, SGI0, SGI300187, SGI300188, SGI300189, SGI300190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300191, SGI0, SGI0, NULL};
const Segment * const SG71383[] = {SGI300192, SGI0, SGI300193, SGI300194, SGI300195, SGI300196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300197, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71384[] = {SGI300198, SGI0, SGI300199, SGI300200, SGI300201, SGI300202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300204, SGI0, SGI0, NULL};
const Segment * const SG71385[] = {SGI300205, SGI0, SGI300206, SGI300207, SGI300208, SGI300209, SGI0, SGI300210, SGI0, SGI0, SGI300211, SGI0, SGI0, SGI300212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300213, SGI0, SGI0, NULL};
const Segment * const SG71386[] = {SGI300214, SGI0, SGI300215, SGI300216, SGI300217, SGI300218, SGI0, SGI300219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300220, SGI300221, SGI0, SGI0, NULL};
const Segment * const SG71387[] = {SGI300222, SGI0, SGI300223, SGI300224, SGI300225, SGI300226, SGI0, SGI300227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300228, SGI0, SGI300229, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71388[] = {SGI300230, SGI0, SGI300231, SGI300232, SGI300233, SGI300234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300236, SGI0, SGI0, NULL};
const Segment * const SG71389[] = {SGI300237, SGI0, SGI300238, SGI300239, SGI300240, SGI300241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71390[] = {SGI300243, SGI0, SGI300244, SGI300245, SGI300246, SGI300247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71391[] = {SGI300249, SGI0, SGI300250, SGI300251, SGI300252, SGI300253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300254, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71392[] = {SGI300255, SGI0, SGI300256, SGI300257, SGI300258, SGI300259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300260, SGI0, SGI0, NULL};
const Segment * const SG71393[] = {SGI300261, SGI0, SGI300262, SGI300263, SGI300264, SGI300265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300266, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71394[] = {SGI300267, SGI0, SGI300268, SGI300269, SGI300270, SGI300271, SGI0, SGI300272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71395[] = {SGI300273, SGI0, SGI300274, SGI300275, SGI300276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71396[] = {SGI300277, SGI0, SGI300278, SGI300279, SGI300280, SGI300281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71397[] = {SGI300282, SGI0, SGI300283, SGI300284, SGI300285, SGI300286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71398[] = {SGI300287, SGI0, SGI300288, SGI300289, SGI300290, SGI300291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300292, SGI0, SGI0, NULL};
const Segment * const SG71399[] = {SGI300293, SGI0, SGI300294, SGI300295, SGI300296, SGI300297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71400[] = {SGI300299, SGI0, SGI300300, SGI300301, SGI300302, SGI300303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300304, SGI0, SGI0, NULL};
const Segment * const SG71401[] = {SGI300305, SGI0, SGI300306, SGI300307, SGI300308, SGI300309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300310, SGI0, SGI0, NULL};
const Segment * const SG71402[] = {SGI300311, SGI0, SGI300312, SGI300313, SGI300314, SGI300315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300316, SGI0, SGI0, NULL};
const Segment * const SG71403[] = {SGI300317, SGI0, SGI300318, SGI300319, SGI300320, SGI300321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300322, SGI0, SGI0, NULL};
const Segment * const SG71404[] = {SGI300323, SGI0, SGI300324, SGI300325, SGI300326, SGI300327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300328, SGI0, SGI0, NULL};
const Segment * const SG71405[] = {SGI300329, SGI0, SGI300330, SGI300331, SGI300332, SGI300333, SGI0, SGI300334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300335, SGI0, SGI0, NULL};
const Segment * const SG71406[] = {SGI300336, SGI0, SGI300337, SGI300338, SGI300339, SGI300340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300341, SGI0, SGI0, NULL};
const Segment * const SG71407[] = {SGI300342, SGI0, SGI0, SGI0, SGI300343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71408[] = {SGI300344, SGI0, SGI300345, SGI300346, SGI300347, SGI300348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300350, SGI0, SGI0, NULL};
const Segment * const SG71409[] = {SGI300351, SGI0, SGI300352, SGI300353, SGI300354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71410[] = {SGI300355, SGI0, SGI300356, SGI300357, SGI300358, SGI300359, SGI0, SGI300360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71411[] = {SGI300361, SGI0, SGI300362, SGI300363, SGI300364, SGI300365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71412[] = {SGI300367, SGI0, SGI300368, SGI300369, SGI300370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71413[] = {SGI300371, SGI0, SGI300372, SGI300373, SGI300374, SGI300375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300376, SGI0, SGI0, NULL};
const Segment * const SG71414[] = {SGI300377, SGI0, SGI300378, SGI300379, SGI300380, SGI300381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300382, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71415[] = {SGI300383, SGI0, SGI300384, SGI300385, SGI300386, SGI300387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300388, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71416[] = {SGI300389, SGI0, SGI300390, SGI300391, SGI300392, SGI300393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71417[] = {SGI300394, SGI0, SGI300395, SGI300396, SGI300397, SGI300398, SGI0, SGI300399, SGI0, SGI0, SGI300400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71418[] = {SGI300401, SGI0, SGI300402, SGI300403, SGI300404, SGI300405, SGI0, SGI0, SGI0, SGI0, SGI300406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71419[] = {SGI300407, SGI0, SGI300408, SGI300409, SGI300410, SGI300411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71420[] = {SGI300412, SGI0, SGI300413, SGI300414, SGI300415, SGI300416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71421[] = {SGI300417, SGI0, SGI300418, SGI300419, SGI300420, SGI300421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71422[] = {SGI300422, SGI0, SGI300423, SGI300424, SGI300425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71423[] = {SGI300426, SGI0, SGI300427, SGI300428, SGI300429, SGI300430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71424[] = {SGI300431, SGI0, SGI300432, SGI300433, SGI300434, SGI300435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300436, SGI300437, SGI0, SGI0, NULL};
const Segment * const SG71425[] = {SGI300438, SGI0, SGI300439, SGI300440, SGI300441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71426[] = {SGI300442, SGI0, SGI300443, SGI300444, SGI300445, SGI300446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71427[] = {SGI300447, SGI0, SGI300448, SGI300449, SGI300450, SGI300451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71428[] = {SGI300452, SGI0, SGI300453, SGI300454, SGI300455, SGI300456, SGI0, SGI300457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300458, SGI0, SGI0, NULL};
const Segment * const SG71429[] = {SGI300459, SGI0, SGI300460, SGI300461, SGI300462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71430[] = {SGI300463, SGI0, SGI300464, SGI300465, SGI300466, SGI300467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300468, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71431[] = {SGI300469, SGI0, SGI300470, SGI300471, SGI300472, SGI300473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71432[] = {SGI300474, SGI0, SGI300475, SGI300476, SGI300477, SGI300478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71433[] = {SGI300479, SGI0, SGI300480, SGI300481, SGI300482, SGI300483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71434[] = {SGI300484, SGI0, SGI300485, SGI300486, SGI300487, SGI300488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71435[] = {SGI300489, SGI0, SGI300490, SGI300491, SGI300492, SGI300493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300494, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71436[] = {SGI300495, SGI0, SGI300496, SGI300497, SGI300498, SGI300499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71437[] = {SGI300500, SGI0, SGI300501, SGI300502, SGI300503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71438[] = {SGI300504, SGI0, SGI300505, SGI300506, SGI300507, SGI300508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71439[] = {SGI300509, SGI0, SGI300510, SGI300511, SGI300512, SGI300513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300514, SGI0, SGI0, NULL};
const Segment * const SG71440[] = {SGI300515, SGI0, SGI300516, SGI300517, SGI300518, SGI300519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71441[] = {SGI300520, SGI0, SGI300521, SGI300522, SGI300523, SGI300524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71442[] = {SGI300525, SGI0, SGI300526, SGI300527, SGI300528, SGI300529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71443[] = {SGI300530, SGI0, SGI300531, SGI300532, SGI300533, SGI300534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71444[] = {SGI300535, SGI0, SGI300536, SGI300537, SGI300538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71445[] = {SGI300539, SGI0, SGI300540, SGI300541, SGI300542, SGI300543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71446[] = {SGI300544, SGI0, SGI300545, SGI300546, SGI300547, SGI300548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71447[] = {SGI300549, SGI0, SGI300550, SGI300551, SGI300552, SGI300553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300554, SGI0, SGI0, SGI300555, SGI0, SGI0, NULL};
const Segment * const SG71448[] = {SGI300556, SGI0, SGI300557, SGI300558, SGI300559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71449[] = {SGI300560, SGI0, SGI300561, SGI300562, SGI300563, SGI300564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300565, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71450[] = {SGI300566, SGI0, SGI300567, SGI300568, SGI300569, SGI300570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71451[] = {SGI300572, SGI0, SGI300573, SGI300574, SGI300575, SGI300576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71452[] = {SGI300577, SGI0, SGI300578, SGI300579, SGI300580, SGI300581, SGI0, SGI300582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71453[] = {SGI300583, SGI0, SGI300584, SGI300585, SGI300586, SGI300587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71454[] = {SGI300588, SGI0, SGI300589, SGI300590, SGI300591, SGI300592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300594, SGI0, SGI0, NULL};
const Segment * const SG71455[] = {SGI300595, SGI0, SGI300596, SGI300597, SGI300598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300599, SGI0, SGI0, NULL};
const Segment * const SG71456[] = {SGI300600, SGI0, SGI300601, SGI300602, SGI300603, SGI300604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300605, SGI0, SGI0, NULL};
const Segment * const SG71457[] = {SGI300606, SGI0, SGI0, SGI0, SGI300607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71458[] = {SGI300608, SGI0, SGI300609, SGI300610, SGI300611, SGI300612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71459[] = {SGI300613, SGI0, SGI300614, SGI300615, SGI300616, SGI300617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71460[] = {SGI300619, SGI0, SGI300620, SGI300621, SGI300622, SGI300623, SGI0, SGI300624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300625, SGI0, SGI0, NULL};
const Segment * const SG71461[] = {SGI300626, SGI0, SGI300627, SGI300628, SGI300629, SGI300630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300631, SGI0, SGI0, SGI300632, SGI0, SGI0, NULL};
const Segment * const SG71462[] = {SGI300633, SGI0, SGI300634, SGI300635, SGI300636, SGI300637, SGI0, SGI300638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71463[] = {SGI300639, SGI0, SGI300640, SGI300641, SGI300642, SGI300643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71464[] = {SGI300644, SGI0, SGI300645, SGI300646, SGI300647, SGI300648, SGI0, SGI0, SGI0, SGI0, SGI300649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300650, SGI0, SGI0, NULL};
const Segment * const SG71465[] = {SGI300651, SGI0, SGI300652, SGI300653, SGI300654, SGI300655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300656, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71466[] = {SGI300657, SGI0, SGI300658, SGI300659, SGI300660, SGI300661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300662, SGI0, SGI0, NULL};
const Segment * const SG71467[] = {SGI300663, SGI0, SGI300664, SGI300665, SGI300666, SGI300667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300668, SGI300669, SGI0, SGI0, NULL};
const Segment * const SG71468[] = {SGI300670, SGI0, SGI300671, SGI300672, SGI300673, SGI300674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71469[] = {SGI300675, SGI0, SGI300676, SGI300677, SGI300678, SGI300679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71470[] = {SGI300680, SGI0, SGI300681, SGI300682, SGI300683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71471[] = {SGI300684, SGI0, SGI300685, SGI300686, SGI300687, SGI300688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300689, SGI0, SGI0, NULL};
const Segment * const SG71472[] = {SGI300690, SGI0, SGI300691, SGI300692, SGI300693, SGI300694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300695, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71473[] = {SGI300696, SGI0, SGI300697, SGI300698, SGI300699, SGI300700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300702, SGI0, SGI0, NULL};
const Segment * const SG71474[] = {SGI300703, SGI0, SGI300704, SGI300705, SGI300706, SGI300707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300708, SGI0, SGI0, SGI300709, SGI0, SGI0, NULL};
const Segment * const SG71475[] = {SGI300710, SGI0, SGI300711, SGI300712, SGI300713, SGI300714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300715, SGI0, SGI0, NULL};
const Segment * const SG71476[] = {SGI300716, SGI0, SGI300717, SGI300718, SGI300719, SGI300720, SGI0, SGI300721, SGI0, SGI0, SGI300722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300723, SGI0, SGI0, NULL};
const Segment * const SG71477[] = {SGI300724, SGI0, SGI300725, SGI300726, SGI300727, SGI300728, SGI0, SGI300729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300730, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71478[] = {SGI300731, SGI0, SGI300732, SGI300733, SGI300734, SGI300735, SGI0, SGI300736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300737, SGI0, SGI0, NULL};
const Segment * const SG71479[] = {SGI300738, SGI0, SGI300739, SGI300740, SGI300741, SGI300742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300743, SGI0, SGI0, SGI300744, SGI0, SGI0, NULL};
const Segment * const SG71480[] = {SGI300745, SGI0, SGI300746, SGI300747, SGI300748, SGI300749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71481[] = {SGI300750, SGI0, SGI300751, SGI300752, SGI300753, SGI300754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300755, SGI0, SGI0, NULL};
const Segment * const SG71482[] = {SGI300756, SGI0, SGI300757, SGI300758, SGI300759, SGI300760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71483[] = {SGI300761, SGI0, SGI300762, SGI300763, SGI300764, SGI300765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71484[] = {SGI300767, SGI0, SGI300768, SGI300769, SGI300770, SGI300771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71485[] = {SGI300772, SGI0, SGI300773, SGI300774, SGI300775, SGI300776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300777, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71486[] = {SGI300778, SGI0, SGI300779, SGI300780, SGI300781, SGI300782, SGI0, SGI300783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300784, SGI0, SGI0, NULL};
const Segment * const SG71487[] = {SGI300785, SGI0, SGI0, SGI0, SGI300786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71488[] = {SGI300787, SGI0, SGI300788, SGI300789, SGI300790, SGI300791, SGI0, SGI300792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300793, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71489[] = {SGI300794, SGI0, SGI300795, SGI300796, SGI300797, SGI300798, SGI0, SGI300799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71490[] = {SGI300800, SGI0, SGI300801, SGI300802, SGI300803, SGI300804, SGI0, SGI0, SGI0, SGI0, SGI300805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300806, SGI0, SGI0, NULL};
const Segment * const SG71491[] = {SGI300807, SGI0, SGI300808, SGI300809, SGI300810, SGI300811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71492[] = {SGI300812, SGI0, SGI300813, SGI300814, SGI300815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71493[] = {SGI300816, SGI0, SGI300817, SGI300818, SGI300819, SGI300820, SGI0, SGI300821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71494[] = {SGI300823, SGI0, SGI300824, SGI300825, SGI300826, SGI300827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71495[] = {SGI300828, SGI0, SGI300829, SGI300830, SGI300831, SGI300832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300833, SGI0, NULL};
const Segment * const SG71496[] = {SGI300834, SGI0, SGI300835, SGI300836, SGI300837, SGI300838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71497[] = {SGI300839, SGI0, SGI300840, SGI300841, SGI300842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71498[] = {SGI300843, SGI0, SGI300844, SGI300845, SGI300846, SGI300847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300848, SGI300849, SGI0, NULL};
const Segment * const SG71499[] = {SGI300850, SGI0, SGI300851, SGI300852, SGI300853, SGI300854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300855, SGI0, NULL};
const Segment * const SG71500[] = {SGI300856, SGI0, SGI300857, SGI300858, SGI300859, SGI300860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300861, SGI0, SGI0, NULL};
const Segment * const SG71501[] = {SGI300862, SGI0, SGI300863, SGI300864, SGI300865, SGI300866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300867, SGI0, SGI0, NULL};
const Segment * const SG71502[] = {SGI300868, SGI0, SGI300869, SGI300870, SGI300871, SGI300872, SGI0, SGI300873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300874, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71503[] = {SGI300875, SGI0, SGI300876, SGI300877, SGI300878, SGI300879, SGI0, SGI0, SGI0, SGI0, SGI300880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300881, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71504[] = {SGI300882, SGI0, SGI300883, SGI300884, SGI300885, SGI300886, SGI0, SGI300887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300888, SGI0, SGI0, NULL};
const Segment * const SG71505[] = {SGI300889, SGI0, SGI300890, SGI300891, SGI300892, SGI300893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71506[] = {SGI300894, SGI0, SGI300895, SGI300896, SGI300897, SGI300898, SGI0, SGI300899, SGI0, SGI0, SGI300900, SGI0, SGI0, SGI300901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71507[] = {SGI300902, SGI0, SGI300903, SGI300904, SGI300905, SGI300906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300907, SGI0, SGI0, SGI300908, SGI0, SGI0, NULL};
const Segment * const SG71508[] = {SGI300909, SGI0, SGI300910, SGI300911, SGI300912, SGI300913, SGI0, SGI300914, SGI0, SGI0, SGI300915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300916, SGI0, SGI0, NULL};
const Segment * const SG71509[] = {SGI300917, SGI0, SGI300918, SGI300919, SGI300920, SGI300921, SGI0, SGI300922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300924, SGI0, SGI0, NULL};
const Segment * const SG71510[] = {SGI300925, SGI0, SGI300926, SGI300927, SGI300928, SGI300929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71511[] = {SGI300930, SGI0, SGI300931, SGI300932, SGI300933, SGI300934, SGI0, SGI300935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71512[] = {SGI300936, SGI0, SGI300937, SGI300938, SGI300939, SGI300940, SGI0, SGI300941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300943, SGI0, SGI0, NULL};
const Segment * const SG71513[] = {SGI300944, SGI0, SGI300945, SGI300946, SGI300947, SGI300948, SGI0, SGI300949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300950, SGI0, SGI0, NULL};
const Segment * const SG71514[] = {SGI300951, SGI0, SGI300952, SGI300953, SGI300954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71515[] = {SGI300955, SGI0, SGI300956, SGI300957, SGI300958, SGI300959, SGI0, SGI300960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300961, SGI0, SGI0, NULL};
const Segment * const SG71516[] = {SGI300962, SGI0, SGI300963, SGI300964, SGI300965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71517[] = {SGI300966, SGI0, SGI300967, SGI300968, SGI300969, SGI300970, SGI0, SGI300971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300972, SGI0, SGI0, NULL};
const Segment * const SG71518[] = {SGI300973, SGI0, SGI300974, SGI300975, SGI300976, SGI300977, SGI0, SGI300978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71519[] = {SGI300980, SGI0, SGI300981, SGI300982, SGI300983, SGI300984, SGI0, SGI0, SGI0, SGI0, SGI300985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71520[] = {SGI300986, SGI0, SGI300987, SGI300988, SGI300989, SGI300990, SGI0, SGI300991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71521[] = {SGI300992, SGI0, SGI300993, SGI300994, SGI300995, SGI300996, SGI0, SGI300997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI300998, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71522[] = {SGI300999, SGI0, SGI301000, SGI301001, SGI301002, SGI301003, SGI0, SGI301004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301005, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71523[] = {SGI301006, SGI0, SGI301007, SGI301008, SGI301009, SGI301010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71524[] = {SGI301012, SGI0, SGI301013, SGI301014, SGI301015, SGI301016, SGI0, SGI301017, SGI0, SGI0, SGI301018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71525[] = {SGI301019, SGI0, SGI301020, SGI301021, SGI301022, SGI301023, SGI0, SGI301024, SGI0, SGI0, SGI301025, SGI0, SGI0, SGI301026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71526[] = {SGI301027, SGI0, SGI301028, SGI301029, SGI301030, SGI301031, SGI0, SGI301032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71527[] = {SGI301033, SGI0, SGI301034, SGI301035, SGI301036, SGI301037, SGI0, SGI301038, SGI0, SGI0, SGI301039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301040, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71528[] = {SGI301041, SGI0, SGI301042, SGI301043, SGI301044, SGI301045, SGI0, SGI301046, SGI0, SGI0, SGI301047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301048, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71529[] = {SGI301049, SGI0, SGI301050, SGI301051, SGI301052, SGI301053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301054, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71530[] = {SGI301055, SGI0, SGI301056, SGI301057, SGI301058, SGI301059, SGI0, SGI301060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301061, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71531[] = {SGI301062, SGI0, SGI301063, SGI301064, SGI301065, SGI301066, SGI0, SGI301067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301068, SGI0, SGI301069, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71532[] = {SGI301070, SGI0, SGI301071, SGI301072, SGI301073, SGI301074, SGI0, SGI301075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71533[] = {SGI301077, SGI0, SGI301078, SGI301079, SGI301080, SGI301081, SGI0, SGI301082, SGI0, SGI0, SGI301083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301084, SGI0, SGI301085, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71534[] = {SGI301086, SGI0, SGI301087, SGI301088, SGI301089, SGI301090, SGI0, SGI301091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301092, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71535[] = {SGI301093, SGI0, SGI301094, SGI301095, SGI301096, SGI301097, SGI0, SGI301098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301099, SGI0, SGI301100, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71536[] = {SGI301101, SGI0, SGI301102, SGI301103, SGI301104, SGI301105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71537[] = {SGI301106, SGI0, SGI301107, SGI301108, SGI301109, SGI301110, SGI0, SGI301111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301112, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71538[] = {SGI301113, SGI0, SGI301114, SGI301115, SGI301116, SGI301117, SGI0, SGI301118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71539[] = {SGI301120, SGI0, SGI301121, SGI301122, SGI301123, SGI301124, SGI0, SGI301125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71540[] = {SGI301126, SGI0, SGI301127, SGI301128, SGI301129, SGI301130, SGI0, SGI301131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301133, SGI0, SGI0, NULL};
const Segment * const SG71541[] = {SGI301134, SGI0, SGI301135, SGI301136, SGI301137, SGI301138, SGI0, SGI301139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301140, SGI0, SGI0, SGI301141, SGI0, SGI0, NULL};
const Segment * const SG71542[] = {SGI301142, SGI0, SGI301143, SGI301144, SGI301145, SGI301146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71543[] = {SGI301147, SGI0, SGI301148, SGI301149, SGI301150, SGI301151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301152, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71544[] = {SGI301153, SGI0, SGI301154, SGI301155, SGI301156, SGI301157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301158, SGI0, SGI0, NULL};
const Segment * const SG71545[] = {SGI301159, SGI0, SGI301160, SGI301161, SGI301162, SGI301163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301164, SGI0, SGI0, SGI301165, SGI0, SGI0, NULL};
const Segment * const SG71546[] = {SGI301166, SGI0, SGI301167, SGI301168, SGI301169, SGI301170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71547[] = {SGI301172, SGI0, SGI301173, SGI301174, SGI301175, SGI301176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301177, SGI301178, SGI0, SGI0, NULL};
const Segment * const SG71548[] = {SGI301179, SGI0, SGI301180, SGI301181, SGI301182, SGI301183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71549[] = {SGI301185, SGI0, SGI301186, SGI301187, SGI301188, SGI301189, SGI0, SGI301190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71550[] = {SGI301192, SGI0, SGI301193, SGI301194, SGI301195, SGI301196, SGI0, SGI301197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71551[] = {SGI301199, SGI0, SGI301200, SGI301201, SGI301202, SGI301203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301205, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71552[] = {SGI301206, SGI0, SGI301207, SGI301208, SGI301209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71553[] = {SGI301210, SGI0, SGI301211, SGI301212, SGI301213, SGI301214, SGI0, SGI301215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301216, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71554[] = {SGI301217, SGI0, SGI301218, SGI301219, SGI301220, SGI301221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71555[] = {SGI301223, SGI0, SGI301224, SGI301225, SGI301226, SGI301227, SGI0, SGI301228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71556[] = {SGI301229, SGI0, SGI301230, SGI301231, SGI301232, SGI301233, SGI0, SGI301234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301235, SGI301236, SGI0, SGI0, NULL};
const Segment * const SG71557[] = {SGI301237, SGI0, SGI301238, SGI301239, SGI301240, SGI301241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301242, SGI0, SGI301243, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71558[] = {SGI301244, SGI0, SGI301245, SGI301246, SGI301247, SGI301248, SGI0, SGI301249, SGI0, SGI0, SGI301250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301251, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71559[] = {SGI301252, SGI0, SGI301253, SGI301254, SGI301255, SGI301256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301257, SGI301258, SGI0, SGI0, NULL};
const Segment * const SG71560[] = {SGI301259, SGI0, SGI301260, SGI301261, SGI301262, SGI301263, SGI0, SGI301264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301265, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71561[] = {SGI301266, SGI0, SGI301267, SGI301268, SGI301269, SGI301270, SGI0, SGI0, SGI0, SGI0, SGI301271, SGI0, SGI0, SGI301272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71562[] = {SGI301273, SGI0, SGI301274, SGI301275, SGI301276, SGI301277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301278, SGI0, SGI0, NULL};
const Segment * const SG71563[] = {SGI301279, SGI0, SGI301280, SGI301281, SGI301282, SGI301283, SGI0, SGI301284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71564[] = {SGI301285, SGI0, SGI301286, SGI301287, SGI301288, SGI301289, SGI0, SGI301290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71565[] = {SGI301292, SGI0, SGI301293, SGI301294, SGI301295, SGI301296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71566[] = {SGI301297, SGI0, SGI301298, SGI301299, SGI301300, SGI301301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301303, SGI0, SGI0, NULL};
const Segment * const SG71567[] = {SGI301304, SGI0, SGI301305, SGI301306, SGI301307, SGI301308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71568[] = {SGI301310, SGI0, SGI301311, SGI301312, SGI301313, SGI301314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301316, SGI0, SGI0, NULL};
const Segment * const SG71569[] = {SGI301317, SGI0, SGI301318, SGI301319, SGI301320, SGI301321, SGI0, SGI301322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301323, SGI0, SGI0, NULL};
const Segment * const SG71570[] = {SGI301324, SGI0, SGI301325, SGI301326, SGI301327, SGI301328, SGI0, SGI301329, SGI0, SGI0, SGI301330, SGI0, SGI0, SGI301331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71571[] = {SGI301332, SGI0, SGI301333, SGI301334, SGI301335, SGI301336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71572[] = {SGI301337, SGI0, SGI301338, SGI301339, SGI301340, SGI301341, SGI0, SGI301342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71573[] = {SGI301343, SGI0, SGI301344, SGI301345, SGI301346, SGI301347, SGI0, SGI301348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71574[] = {SGI301349, SGI0, SGI301350, SGI301351, SGI301352, SGI301353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71575[] = {SGI301354, SGI0, SGI301355, SGI0, SGI301356, SGI301357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301358, SGI0, SGI0, NULL};
const Segment * const SG71576[] = {SGI301359, SGI0, SGI301360, SGI301361, SGI301362, SGI301363, SGI0, SGI301364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301365, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71577[] = {SGI301366, SGI0, SGI301367, SGI301368, SGI301369, SGI301370, SGI0, SGI301371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301372, SGI0, NULL};
const Segment * const SG71578[] = {SGI301373, SGI0, SGI301374, SGI301375, SGI301376, SGI301377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71579[] = {SGI301378, SGI0, SGI301379, SGI301380, SGI301381, SGI301382, SGI0, SGI301383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71580[] = {SGI301384, SGI0, SGI301385, SGI301386, SGI301387, SGI301388, SGI0, SGI301389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301391, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71581[] = {SGI301392, SGI0, SGI301393, SGI301394, SGI301395, SGI301396, SGI0, SGI301397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71582[] = {SGI301399, SGI0, SGI301400, SGI301401, SGI301402, SGI301403, SGI0, SGI301404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71583[] = {SGI301406, SGI0, SGI301407, SGI301408, SGI301409, SGI301410, SGI0, SGI301411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301412, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71584[] = {SGI301413, SGI0, SGI301414, SGI301415, SGI301416, SGI301417, SGI0, SGI301418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301419, SGI301420, SGI0, SGI0, NULL};
const Segment * const SG71585[] = {SGI301421, SGI0, SGI301422, SGI301423, SGI301424, SGI301425, SGI0, SGI301426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301427, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71586[] = {SGI301428, SGI0, SGI301429, SGI301430, SGI301431, SGI301432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301433, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71587[] = {SGI301434, SGI0, SGI301435, SGI301436, SGI301437, SGI301438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301439, SGI0, SGI0, NULL};
const Segment * const SG71588[] = {SGI301440, SGI0, SGI301441, SGI301442, SGI301443, SGI301444, SGI0, SGI301445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71589[] = {SGI301446, SGI0, SGI301447, SGI301448, SGI301449, SGI301450, SGI0, SGI301451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301452, SGI0, SGI301453, SGI301454, SGI0, SGI0, NULL};
const Segment * const SG71590[] = {SGI301455, SGI0, SGI301456, SGI301457, SGI301458, SGI301459, SGI0, SGI301460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71591[] = {SGI301461, SGI0, SGI301462, SGI301463, SGI301464, SGI301465, SGI0, SGI301466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71592[] = {SGI301468, SGI0, SGI301469, SGI301470, SGI301471, SGI301472, SGI0, SGI301473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301474, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71593[] = {SGI301475, SGI0, SGI301476, SGI301477, SGI301478, SGI301479, SGI0, SGI301480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71594[] = {SGI301481, SGI0, SGI301482, SGI301483, SGI301484, SGI301485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71595[] = {SGI301486, SGI0, SGI301487, SGI301488, SGI301489, SGI301490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301491, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71596[] = {SGI301492, SGI0, SGI301493, SGI301494, SGI301495, SGI301496, SGI0, SGI301497, SGI0, SGI0, SGI301498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71597[] = {SGI301499, SGI0, SGI301500, SGI301501, SGI301502, SGI301503, SGI0, SGI301504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71598[] = {SGI301505, SGI0, SGI301506, SGI301507, SGI301508, SGI301509, SGI0, SGI0, SGI0, SGI0, SGI301510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301511, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71599[] = {SGI301512, SGI0, SGI301513, SGI301514, SGI301515, SGI301516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71600[] = {SGI301518, SGI0, SGI301519, SGI301520, SGI301521, SGI301522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301523, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71601[] = {SGI301524, SGI0, SGI301525, SGI301526, SGI301527, SGI301528, SGI0, SGI301529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71602[] = {SGI301531, SGI0, SGI301532, SGI301533, SGI301534, SGI301535, SGI0, SGI301536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301537, SGI0, NULL};
const Segment * const SG71603[] = {SGI301538, SGI0, SGI301539, SGI301540, SGI301541, SGI301542, SGI0, SGI301543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301544, SGI0, SGI0, SGI301545, SGI0, SGI0, NULL};
const Segment * const SG71604[] = {SGI301546, SGI0, SGI301547, SGI301548, SGI301549, SGI301550, SGI0, SGI301551, SGI0, SGI0, SGI301552, SGI0, SGI0, SGI301553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71605[] = {SGI301554, SGI0, SGI301555, SGI301556, SGI301557, SGI301558, SGI0, SGI301559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301560, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71606[] = {SGI301561, SGI0, SGI301562, SGI301563, SGI301564, SGI301565, SGI0, SGI0, SGI0, SGI0, SGI301566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301567, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71607[] = {SGI301568, SGI0, SGI301569, SGI301570, SGI301571, SGI301572, SGI0, SGI301573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301575, SGI0, NULL};
const Segment * const SG71608[] = {SGI301576, SGI0, SGI301577, SGI301578, SGI301579, SGI301580, SGI0, SGI301581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301583, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71609[] = {SGI301584, SGI0, SGI301585, SGI0, SGI301586, SGI301587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71610[] = {SGI301588, SGI0, SGI301589, SGI301590, SGI301591, SGI301592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301593, SGI0, NULL};
const Segment * const SG71611[] = {SGI301594, SGI0, SGI301595, SGI301596, SGI301597, SGI301598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71612[] = {SGI301599, SGI0, SGI301600, SGI301601, SGI301602, SGI301603, SGI0, SGI301604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71613[] = {SGI301605, SGI0, SGI301606, SGI301607, SGI301608, SGI301609, SGI0, SGI301610, SGI0, SGI0, SGI301611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71614[] = {SGI301612, SGI0, SGI301613, SGI301614, SGI301615, SGI301616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71615[] = {SGI301617, SGI0, SGI301618, SGI301619, SGI301620, SGI301621, SGI0, SGI301622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301623, SGI0, SGI0, NULL};
const Segment * const SG71616[] = {SGI301624, SGI0, SGI0, SGI0, SGI301625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71617[] = {SGI301626, SGI0, SGI301627, SGI301628, SGI301629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71618[] = {SGI301630, SGI0, SGI301631, SGI301632, SGI301633, SGI301634, SGI0, SGI301635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71619[] = {SGI301636, SGI0, SGI301637, SGI301638, SGI301639, SGI301640, SGI0, SGI301641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301642, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71620[] = {SGI301643, SGI0, SGI301644, SGI301645, SGI301646, SGI301647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71621[] = {SGI301649, SGI0, SGI301650, SGI301651, SGI301652, SGI301653, SGI0, SGI301654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301655, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71622[] = {SGI301656, SGI0, SGI301657, SGI301658, SGI301659, SGI301660, SGI0, SGI301661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301663, SGI0, SGI0, NULL};
const Segment * const SG71623[] = {SGI301664, SGI0, SGI301665, SGI301666, SGI301667, SGI301668, SGI0, SGI301669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71624[] = {SGI301670, SGI0, SGI301671, SGI301672, SGI301673, SGI301674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301675, SGI0, SGI0, NULL};
const Segment * const SG71625[] = {SGI301676, SGI0, SGI301677, SGI301678, SGI301679, SGI301680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71626[] = {SGI301681, SGI0, SGI301682, SGI301683, SGI301684, SGI301685, SGI0, SGI301686, SGI0, SGI0, SGI301687, SGI0, SGI0, SGI301688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71627[] = {SGI301689, SGI0, SGI301690, SGI301691, SGI301692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71628[] = {SGI301693, SGI0, SGI301694, SGI301695, SGI301696, SGI301697, SGI0, SGI301698, SGI0, SGI0, SGI301699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71629[] = {SGI301700, SGI0, SGI301701, SGI301702, SGI301703, SGI301704, SGI0, SGI0, SGI0, SGI0, SGI301705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71630[] = {SGI301706, SGI0, SGI301707, SGI301708, SGI301709, SGI301710, SGI0, SGI301711, SGI0, SGI0, SGI301712, SGI0, SGI0, SGI301713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71631[] = {SGI0, SGI0, SGI301714, SGI301715, SGI301716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71632[] = {SGI301717, SGI0, SGI301718, SGI301719, SGI301720, SGI301721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301722, SGI0, SGI0, NULL};
const Segment * const SG71633[] = {SGI301723, SGI0, SGI301724, SGI301725, SGI301726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71634[] = {SGI301727, SGI0, SGI301728, SGI301729, SGI301730, SGI301731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301732, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71635[] = {SGI301733, SGI0, SGI301734, SGI301735, SGI301736, SGI301737, SGI0, SGI301738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301739, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71636[] = {SGI301740, SGI0, SGI301741, SGI301742, SGI301743, SGI301744, SGI0, SGI301745, SGI0, SGI0, SGI301746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301747, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71637[] = {SGI301748, SGI0, SGI301749, SGI301750, SGI301751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71638[] = {SGI301752, SGI0, SGI301753, SGI301754, SGI301755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71639[] = {SGI301756, SGI0, SGI301757, SGI301758, SGI301759, SGI301760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71640[] = {SGI301762, SGI0, SGI301763, SGI301764, SGI301765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71641[] = {SGI301766, SGI0, SGI301767, SGI301768, SGI301769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71642[] = {SGI301770, SGI0, SGI301771, SGI301772, SGI301773, SGI301774, SGI0, SGI301775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301776, SGI0, SGI0, NULL};
const Segment * const SG71643[] = {SGI301777, SGI0, SGI301778, SGI301779, SGI301780, SGI301781, SGI0, SGI0, SGI0, SGI0, SGI301782, SGI0, SGI0, SGI301783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71644[] = {SGI301784, SGI0, SGI301785, SGI301786, SGI301787, SGI301788, SGI0, SGI301789, SGI0, SGI0, SGI301790, SGI0, SGI0, SGI301791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301792, SGI0, SGI0, NULL};
const Segment * const SG71645[] = {SGI301793, SGI0, SGI301794, SGI301795, SGI301796, SGI301797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301798, SGI0, SGI301799, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71646[] = {SGI301800, SGI0, SGI301801, SGI301802, SGI301803, SGI301804, SGI0, SGI0, SGI0, SGI0, SGI301805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301806, SGI0, SGI0, NULL};
const Segment * const SG71647[] = {SGI301807, SGI0, SGI301808, SGI301809, SGI301810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71648[] = {SGI301811, SGI0, SGI301812, SGI301813, SGI301814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71649[] = {SGI301815, SGI0, SGI301816, SGI301817, SGI301818, SGI301819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301820, SGI0, SGI0, NULL};
const Segment * const SG71650[] = {SGI301821, SGI0, SGI301822, SGI301823, SGI301824, SGI301825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301826, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71651[] = {SGI301827, SGI0, SGI301828, SGI301829, SGI301830, SGI301831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71652[] = {SGI301832, SGI0, SGI0, SGI0, SGI301833, SGI301834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71653[] = {SGI301835, SGI0, SGI301836, SGI301837, SGI301838, SGI301839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71654[] = {SGI301841, SGI0, SGI301842, SGI301843, SGI301844, SGI301845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71655[] = {SGI301846, SGI0, SGI301847, SGI301848, SGI301849, SGI301850, SGI0, SGI301851, SGI0, SGI0, SGI301852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301853, SGI0, SGI0, NULL};
const Segment * const SG71656[] = {SGI301854, SGI0, SGI301855, SGI301856, SGI301857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71657[] = {SGI301858, SGI0, SGI301859, SGI301860, SGI301861, SGI301862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301864, SGI0, SGI0, NULL};
const Segment * const SG71658[] = {SGI301865, SGI0, SGI301866, SGI301867, SGI301868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71659[] = {SGI301869, SGI0, SGI301870, SGI301871, SGI301872, SGI301873, SGI0, SGI301874, SGI0, SGI0, SGI301875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301876, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71660[] = {SGI301877, SGI0, SGI301878, SGI301879, SGI301880, SGI301881, SGI0, SGI301882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71661[] = {SGI301884, SGI0, SGI301885, SGI301886, SGI301887, SGI301888, SGI0, SGI301889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71662[] = {SGI301891, SGI0, SGI301892, SGI301893, SGI301894, SGI301895, SGI0, SGI301896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301897, SGI0, SGI0, NULL};
const Segment * const SG71663[] = {SGI301898, SGI0, SGI301899, SGI301900, SGI301901, SGI301902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301903, SGI0, SGI0, NULL};
const Segment * const SG71664[] = {SGI301904, SGI0, SGI301905, SGI301906, SGI301907, SGI301908, SGI0, SGI301909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301910, SGI0, SGI0, SGI301911, SGI0, SGI0, NULL};
const Segment * const SG71665[] = {SGI301912, SGI0, SGI301913, SGI301914, SGI301915, SGI301916, SGI0, SGI301917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301919, SGI0, SGI0, NULL};
const Segment * const SG71666[] = {SGI301920, SGI0, SGI301921, SGI301922, SGI301923, SGI301924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301926, SGI0, SGI0, NULL};
const Segment * const SG71667[] = {SGI301927, SGI0, SGI301928, SGI301929, SGI301930, SGI301931, SGI0, SGI301932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301933, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71668[] = {SGI301934, SGI0, SGI301935, SGI301936, SGI301937, SGI301938, SGI0, SGI301939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301940, SGI0, SGI301941, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71669[] = {SGI301942, SGI0, SGI301943, SGI301944, SGI301945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71670[] = {SGI301946, SGI0, SGI301947, SGI301948, SGI301949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71671[] = {SGI301950, SGI0, SGI301951, SGI301952, SGI301953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71672[] = {SGI301954, SGI0, SGI301955, SGI301956, SGI301957, SGI301958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71673[] = {SGI301960, SGI0, SGI301961, SGI301962, SGI301963, SGI301964, SGI0, SGI301965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301966, SGI301967, SGI0, SGI0, NULL};
const Segment * const SG71674[] = {SGI301968, SGI0, SGI301969, SGI301970, SGI301971, SGI301972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301973, SGI0, SGI0, NULL};
const Segment * const SG71675[] = {SGI301974, SGI0, SGI301975, SGI301976, SGI301977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71676[] = {SGI301978, SGI0, SGI301979, SGI301980, SGI301981, SGI301982, SGI0, SGI301983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301985, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71677[] = {SGI301986, SGI0, SGI301987, SGI301988, SGI301989, SGI301990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301991, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71678[] = {SGI301992, SGI0, SGI301993, SGI301994, SGI301995, SGI301996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI301997, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71679[] = {SGI301998, SGI0, SGI301999, SGI302000, SGI302001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71680[] = {SGI302002, SGI0, SGI302003, SGI302004, SGI302005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71681[] = {SGI302006, SGI0, SGI302007, SGI302008, SGI302009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71682[] = {SGI302010, SGI0, SGI302011, SGI302012, SGI302013, SGI302014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302015, SGI0, SGI0, NULL};
const Segment * const SG71683[] = {SGI302016, SGI0, SGI302017, SGI302018, SGI302019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71684[] = {SGI302020, SGI0, SGI302021, SGI302022, SGI302023, SGI302024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71685[] = {SGI302025, SGI0, SGI302026, SGI302027, SGI302028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71686[] = {SGI302029, SGI0, SGI302030, SGI302031, SGI302032, SGI302033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302034, SGI0, SGI0, SGI302035, SGI0, SGI0, NULL};
const Segment * const SG71687[] = {SGI302036, SGI0, SGI302037, SGI302038, SGI302039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71688[] = {SGI302040, SGI0, SGI302041, SGI302042, SGI302043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71689[] = {SGI302044, SGI0, SGI302045, SGI302046, SGI302047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71690[] = {SGI302048, SGI0, SGI302049, SGI302050, SGI302051, SGI302052, SGI0, SGI302053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71691[] = {SGI302054, SGI0, SGI302055, SGI302056, SGI302057, SGI302058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71692[] = {SGI302059, SGI0, SGI302060, SGI302061, SGI302062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71693[] = {SGI302063, SGI0, SGI302064, SGI302065, SGI302066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71694[] = {SGI302067, SGI0, SGI302068, SGI302069, SGI302070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71695[] = {SGI302071, SGI0, SGI302072, SGI302073, SGI302074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71696[] = {SGI302075, SGI0, SGI302076, SGI302077, SGI302078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71697[] = {SGI302079, SGI0, SGI302080, SGI302081, SGI302082, SGI302083, SGI0, SGI302084, SGI0, SGI0, SGI302085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302086, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71698[] = {SGI302087, SGI0, SGI302088, SGI302089, SGI302090, SGI302091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302092, SGI0, SGI0, NULL};
const Segment * const SG71699[] = {SGI302093, SGI0, SGI302094, SGI302095, SGI302096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71700[] = {SGI302097, SGI0, SGI302098, SGI302099, SGI302100, SGI302101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302102, SGI0, SGI0, NULL};
const Segment * const SG71701[] = {SGI302103, SGI0, SGI302104, SGI302105, SGI302106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71702[] = {SGI302107, SGI0, SGI302108, SGI302109, SGI302110, SGI302111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302112, SGI0, SGI0, NULL};
const Segment * const SG71703[] = {SGI302113, SGI0, SGI302114, SGI302115, SGI302116, SGI302117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302119, SGI0, SGI0, NULL};
const Segment * const SG71704[] = {SGI302120, SGI0, SGI302121, SGI302122, SGI302123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71705[] = {SGI302124, SGI0, SGI302125, SGI302126, SGI302127, SGI302128, SGI0, SGI302129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71706[] = {SGI302130, SGI0, SGI302131, SGI302132, SGI302133, SGI302134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71707[] = {SGI302136, SGI0, SGI302137, SGI302138, SGI302139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71708[] = {SGI302140, SGI0, SGI302141, SGI302142, SGI302143, SGI302144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71709[] = {SGI302145, SGI0, SGI302146, SGI302147, SGI302148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71710[] = {SGI302149, SGI0, SGI302150, SGI302151, SGI302152, SGI302153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71711[] = {SGI302154, SGI0, SGI302155, SGI302156, SGI302157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71712[] = {SGI302158, SGI0, SGI302159, SGI302160, SGI302161, SGI302162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71713[] = {SGI302163, SGI0, SGI302164, SGI302165, SGI302166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71714[] = {SGI302167, SGI0, SGI302168, SGI302169, SGI302170, SGI302171, SGI0, SGI302172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71715[] = {SGI302173, SGI0, SGI302174, SGI302175, SGI302176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302177, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71716[] = {SGI302178, SGI0, SGI302179, SGI302180, SGI302181, SGI302182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71717[] = {SGI302183, SGI0, SGI302184, SGI302185, SGI302186, SGI302187, SGI0, SGI302188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302189, SGI0, SGI0, NULL};
const Segment * const SG71718[] = {SGI302190, SGI0, SGI302191, SGI302192, SGI302193, SGI302194, SGI0, SGI302195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71719[] = {SGI302197, SGI0, SGI302198, SGI302199, SGI302200, SGI302201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302202, SGI0, SGI0, NULL};
const Segment * const SG71720[] = {SGI302203, SGI0, SGI302204, SGI302205, SGI302206, SGI302207, SGI0, SGI302208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302209, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71721[] = {SGI302210, SGI0, SGI302211, SGI302212, SGI302213, SGI302214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71722[] = {SGI302216, SGI0, SGI0, SGI0, SGI302217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71723[] = {SGI302218, SGI0, SGI302219, SGI302220, SGI302221, SGI302222, SGI0, SGI302223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302224, SGI0, SGI0, NULL};
const Segment * const SG71724[] = {SGI302225, SGI0, SGI302226, SGI302227, SGI302228, SGI302229, SGI0, SGI302230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302231, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71725[] = {SGI302232, SGI0, SGI302233, SGI302234, SGI302235, SGI302236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71726[] = {SGI302237, SGI0, SGI302238, SGI302239, SGI302240, SGI302241, SGI0, SGI302242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302243, SGI0, SGI302244, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71727[] = {SGI302245, SGI0, SGI302246, SGI302247, SGI302248, SGI302249, SGI0, SGI302250, SGI0, SGI0, SGI302251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71728[] = {SGI302252, SGI0, SGI302253, SGI302254, SGI302255, SGI302256, SGI0, SGI302257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71729[] = {SGI302259, SGI0, SGI302260, SGI302261, SGI302262, SGI302263, SGI0, SGI302264, SGI0, SGI0, SGI302265, SGI0, SGI0, SGI302266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71730[] = {SGI302267, SGI0, SGI302268, SGI302269, SGI302270, SGI302271, SGI0, SGI302272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71731[] = {SGI302273, SGI0, SGI302274, SGI302275, SGI302276, SGI302277, SGI0, SGI0, SGI0, SGI0, SGI302278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71732[] = {SGI302279, SGI0, SGI302280, SGI302281, SGI302282, SGI302283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71733[] = {SGI302284, SGI0, SGI302285, SGI302286, SGI302287, SGI302288, SGI0, SGI302289, SGI0, SGI0, SGI302290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302291, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71734[] = {SGI302292, SGI0, SGI302293, SGI302294, SGI302295, SGI302296, SGI0, SGI302297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71735[] = {SGI302298, SGI0, SGI302299, SGI302300, SGI302301, SGI302302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71736[] = {SGI302303, SGI0, SGI302304, SGI302305, SGI302306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71737[] = {SGI302307, SGI0, SGI302308, SGI302309, SGI302310, SGI302311, SGI0, SGI302312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71738[] = {SGI302313, SGI0, SGI302314, SGI302315, SGI302316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71739[] = {SGI302317, SGI0, SGI302318, SGI302319, SGI302320, SGI302321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302322, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71740[] = {SGI302323, SGI0, SGI302324, SGI302325, SGI302326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71741[] = {SGI302327, SGI0, SGI302328, SGI302329, SGI302330, SGI302331, SGI0, SGI302332, SGI0, SGI0, SGI302333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302334, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71742[] = {SGI302335, SGI0, SGI302336, SGI302337, SGI302338, SGI302339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302340, SGI0, SGI0, NULL};
const Segment * const SG71743[] = {SGI302341, SGI0, SGI302342, SGI302343, SGI302344, SGI302345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71744[] = {SGI302346, SGI0, SGI302347, SGI302348, SGI302349, SGI302350, SGI0, SGI302351, SGI0, SGI0, SGI302352, SGI0, SGI0, SGI302353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71745[] = {SGI302354, SGI0, SGI302355, SGI302356, SGI302357, SGI302358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71746[] = {SGI302359, SGI0, SGI0, SGI0, SGI302360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71747[] = {SGI302361, SGI0, SGI302362, SGI0, SGI302363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71748[] = {SGI302364, SGI0, SGI302365, SGI302366, SGI302367, SGI302368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71749[] = {SGI302369, SGI0, SGI302370, SGI302371, SGI302372, SGI302373, SGI0, SGI302374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302375, SGI0, SGI0, NULL};
const Segment * const SG71750[] = {SGI302376, SGI0, SGI302377, SGI302378, SGI302379, SGI302380, SGI0, SGI302381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302382, SGI302383, SGI0, SGI0, NULL};
const Segment * const SG71751[] = {SGI302384, SGI0, SGI302385, SGI302386, SGI302387, SGI302388, SGI0, SGI302389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71752[] = {SGI302390, SGI0, SGI302391, SGI302392, SGI302393, SGI302394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71753[] = {SGI302395, SGI0, SGI302396, SGI302397, SGI302398, SGI302399, SGI0, SGI302400, SGI0, SGI0, SGI302401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302402, SGI0, SGI0, NULL};
const Segment * const SG71754[] = {SGI302403, SGI0, SGI302404, SGI302405, SGI302406, SGI302407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71755[] = {SGI302408, SGI0, SGI302409, SGI302410, SGI302411, SGI302412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302414, SGI0, SGI0, NULL};
const Segment * const SG71756[] = {SGI302415, SGI0, SGI0, SGI0, SGI302416, SGI302417, SGI0, SGI302418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71757[] = {SGI302419, SGI0, SGI302420, SGI302421, SGI302422, SGI302423, SGI0, SGI302424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71758[] = {SGI302425, SGI0, SGI302426, SGI302427, SGI302428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71759[] = {SGI302429, SGI0, SGI302430, SGI302431, SGI302432, SGI302433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302434, SGI0, SGI302435, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71760[] = {SGI302436, SGI0, SGI302437, SGI302438, SGI302439, SGI302440, SGI0, SGI302441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71761[] = {SGI302443, SGI0, SGI302444, SGI302445, SGI302446, SGI302447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302448, SGI0, SGI0, NULL};
const Segment * const SG71762[] = {SGI302449, SGI0, SGI302450, SGI302451, SGI302452, SGI302453, SGI0, SGI302454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71763[] = {SGI302456, SGI0, SGI302457, SGI302458, SGI302459, SGI302460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302461, SGI0, SGI0, NULL};
const Segment * const SG71764[] = {SGI302462, SGI0, SGI302463, SGI302464, SGI302465, SGI302466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71765[] = {SGI302468, SGI0, SGI302469, SGI302470, SGI302471, SGI302472, SGI0, SGI302473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302475, SGI0, SGI0, NULL};
const Segment * const SG71766[] = {SGI302476, SGI0, SGI302477, SGI302478, SGI302479, SGI302480, SGI0, SGI302481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302483, SGI0, SGI0, NULL};
const Segment * const SG71767[] = {SGI302484, SGI0, SGI302485, SGI302486, SGI302487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71768[] = {SGI302488, SGI0, SGI302489, SGI302490, SGI302491, SGI302492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302493, SGI302494, SGI0, SGI0, NULL};
const Segment * const SG71769[] = {SGI302495, SGI0, SGI302496, SGI302497, SGI302498, SGI302499, SGI0, SGI302500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302501, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71770[] = {SGI302502, SGI0, SGI302503, SGI302504, SGI302505, SGI302506, SGI0, SGI302507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302508, SGI0, SGI0, SGI302509, SGI0, SGI0, NULL};
const Segment * const SG71771[] = {SGI302510, SGI0, SGI302511, SGI302512, SGI302513, SGI302514, SGI0, SGI302515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302517, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71772[] = {SGI302518, SGI0, SGI302519, SGI302520, SGI302521, SGI302522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302523, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71773[] = {SGI302524, SGI0, SGI302525, SGI302526, SGI302527, SGI302528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71774[] = {SGI302529, SGI0, SGI302530, SGI302531, SGI302532, SGI302533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302535, SGI0, SGI0, NULL};
const Segment * const SG71775[] = {SGI302536, SGI0, SGI302537, SGI302538, SGI302539, SGI302540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302541, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71776[] = {SGI302542, SGI0, SGI302543, SGI302544, SGI302545, SGI302546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302547, SGI0, SGI0, SGI302548, SGI0, SGI0, NULL};
const Segment * const SG71777[] = {SGI302549, SGI0, SGI302550, SGI302551, SGI302552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71778[] = {SGI302553, SGI0, SGI302554, SGI302555, SGI302556, SGI302557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71779[] = {SGI302558, SGI0, SGI302559, SGI302560, SGI302561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71780[] = {SGI302562, SGI0, SGI302563, SGI302564, SGI302565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71781[] = {SGI302566, SGI0, SGI302567, SGI302568, SGI302569, SGI302570, SGI0, SGI302571, SGI0, SGI0, SGI302572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302573, SGI0, SGI302574, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71782[] = {SGI302575, SGI0, SGI302576, SGI302577, SGI302578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71783[] = {SGI302579, SGI0, SGI302580, SGI302581, SGI302582, SGI302583, SGI0, SGI0, SGI0, SGI0, SGI302584, SGI0, SGI0, SGI302585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71784[] = {SGI302586, SGI0, SGI302587, SGI302588, SGI302589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71785[] = {SGI302590, SGI0, SGI302591, SGI302592, SGI302593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71786[] = {SGI302594, SGI0, SGI302595, SGI302596, SGI302597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71787[] = {SGI302598, SGI0, SGI302599, SGI302600, SGI302601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71788[] = {SGI302602, SGI0, SGI302603, SGI302604, SGI302605, SGI302606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71789[] = {SGI302608, SGI0, SGI302609, SGI302610, SGI302611, SGI0, SGI0, SGI302612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302613, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71790[] = {SGI302614, SGI0, SGI302615, SGI302616, SGI302617, SGI302618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71791[] = {SGI302619, SGI0, SGI302620, SGI302621, SGI302622, SGI302623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71792[] = {SGI302625, SGI0, SGI0, SGI0, SGI302626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71793[] = {SGI302627, SGI0, SGI302628, SGI302629, SGI302630, SGI302631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302632, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71794[] = {SGI302633, SGI0, SGI302634, SGI302635, SGI302636, SGI302637, SGI0, SGI302638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71795[] = {SGI302639, SGI0, SGI302640, SGI302641, SGI302642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71796[] = {SGI302643, SGI0, SGI0, SGI0, SGI302644, SGI302645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71797[] = {SGI302647, SGI0, SGI302648, SGI302649, SGI302650, SGI302651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71798[] = {SGI302652, SGI0, SGI302653, SGI302654, SGI302655, SGI302656, SGI0, SGI302657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71799[] = {SGI302659, SGI0, SGI302660, SGI302661, SGI302662, SGI302663, SGI0, SGI302664, SGI0, SGI0, SGI302665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302666, SGI0, SGI302667, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71800[] = {SGI302668, SGI0, SGI302669, SGI302670, SGI302671, SGI302672, SGI0, SGI302673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302674, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71801[] = {SGI302675, SGI0, SGI302676, SGI302677, SGI302678, SGI302679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71802[] = {SGI302680, SGI0, SGI302681, SGI302682, SGI302683, SGI302684, SGI0, SGI302685, SGI0, SGI0, SGI302686, SGI0, SGI0, SGI302687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71803[] = {SGI302688, SGI0, SGI302689, SGI302690, SGI302691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71804[] = {SGI302692, SGI0, SGI302693, SGI302694, SGI302695, SGI302696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71805[] = {SGI302698, SGI0, SGI302699, SGI302700, SGI302701, SGI302702, SGI0, SGI0, SGI0, SGI0, SGI302703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71806[] = {SGI302704, SGI0, SGI0, SGI0, SGI302705, SGI302706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71807[] = {SGI302707, SGI0, SGI302708, SGI302709, SGI302710, SGI302711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71808[] = {SGI302712, SGI0, SGI302713, SGI302714, SGI302715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71809[] = {SGI302716, SGI0, SGI302717, SGI302718, SGI302719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71810[] = {SGI302720, SGI0, SGI302721, SGI0, SGI302722, SGI302723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71811[] = {SGI302724, SGI0, SGI302725, SGI302726, SGI302727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71812[] = {SGI302728, SGI0, SGI302729, SGI302730, SGI302731, SGI302732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71813[] = {SGI302734, SGI0, SGI302735, SGI302736, SGI302737, SGI302738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302739, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71814[] = {SGI302740, SGI0, SGI302741, SGI302742, SGI302743, SGI302744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71815[] = {SGI302746, SGI0, SGI302747, SGI302748, SGI302749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71816[] = {SGI302750, SGI0, SGI302751, SGI302752, SGI302753, SGI302754, SGI0, SGI302755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71817[] = {SGI302756, SGI0, SGI302757, SGI302758, SGI302759, SGI302760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302761, SGI0, SGI0, NULL};
const Segment * const SG71818[] = {SGI302762, SGI0, SGI302763, SGI302764, SGI302765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71819[] = {SGI302766, SGI0, SGI302767, SGI302768, SGI302769, SGI302770, SGI0, SGI302771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302772, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71820[] = {SGI302773, SGI0, SGI302774, SGI302775, SGI302776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71821[] = {SGI302778, SGI0, SGI302779, SGI302780, SGI302781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302782, SGI0, SGI0, NULL};
const Segment * const SG71822[] = {SGI302783, SGI0, SGI302784, SGI302785, SGI302786, SGI302787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302788, SGI0, SGI0, NULL};
const Segment * const SG71823[] = {SGI302789, SGI0, SGI0, SGI0, SGI302790, SGI302791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302792, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71824[] = {SGI302793, SGI0, SGI302794, SGI302795, SGI302796, SGI302797, SGI0, SGI302798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71825[] = {SGI302800, SGI0, SGI302801, SGI302802, SGI302803, SGI302804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71826[] = {SGI302806, SGI0, SGI302807, SGI302808, SGI302809, SGI302810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71827[] = {SGI302811, SGI0, SGI302812, SGI302813, SGI302814, SGI302815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71828[] = {SGI302816, SGI0, SGI302817, SGI302818, SGI302819, SGI302820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71829[] = {SGI302822, SGI0, SGI302823, SGI302824, SGI302825, SGI302826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71830[] = {SGI302827, SGI0, SGI302828, SGI302829, SGI302830, SGI302831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71831[] = {SGI302832, SGI0, SGI302833, SGI302834, SGI302835, SGI302836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71832[] = {SGI302837, SGI0, SGI302838, SGI302839, SGI302840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71833[] = {SGI302841, SGI0, SGI302842, SGI302843, SGI302844, SGI302845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302846, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71834[] = {SGI302847, SGI0, SGI302848, SGI302849, SGI302850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71835[] = {SGI302852, SGI0, SGI302853, SGI302854, SGI302855, SGI302856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71836[] = {SGI302857, SGI0, SGI0, SGI0, SGI302858, SGI302859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71837[] = {SGI302861, SGI0, SGI302862, SGI0, SGI302863, SGI302864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71838[] = {SGI302866, SGI0, SGI302867, SGI302868, SGI302869, SGI302870, SGI0, SGI302871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302872, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71839[] = {SGI302873, SGI0, SGI302874, SGI302875, SGI302876, SGI302877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71840[] = {SGI302879, SGI0, SGI302880, SGI302881, SGI302882, SGI302883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71841[] = {SGI302885, SGI0, SGI0, SGI0, SGI302886, SGI302887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71842[] = {SGI302888, SGI0, SGI302889, SGI302890, SGI302891, SGI302892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302893, SGI0, SGI0, NULL};
const Segment * const SG71843[] = {SGI302894, SGI0, SGI302895, SGI302896, SGI302897, SGI302898, SGI0, SGI302899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302900, SGI0, SGI302901, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71844[] = {SGI302902, SGI0, SGI0, SGI0, SGI302903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71845[] = {SGI302904, SGI0, SGI302905, SGI302906, SGI302907, SGI302908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71846[] = {SGI302909, SGI0, SGI302910, SGI302911, SGI302912, SGI302913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302914, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71847[] = {SGI302915, SGI0, SGI302916, SGI302917, SGI302918, SGI302919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71848[] = {SGI302921, SGI0, SGI302922, SGI302923, SGI302924, SGI302925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71849[] = {SGI302927, SGI0, SGI302928, SGI302929, SGI302930, SGI302931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71850[] = {SGI302933, SGI0, SGI302934, SGI302935, SGI302936, SGI302937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71851[] = {SGI302938, SGI0, SGI302939, SGI302940, SGI302941, SGI302942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302943, SGI0, SGI0, NULL};
const Segment * const SG71852[] = {SGI302944, SGI0, SGI302945, SGI302946, SGI302947, SGI302948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302950, SGI0, SGI0, NULL};
const Segment * const SG71853[] = {SGI302951, SGI0, SGI302952, SGI302953, SGI302954, SGI302955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71854[] = {SGI302956, SGI0, SGI0, SGI0, SGI302957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71855[] = {SGI302958, SGI0, SGI302959, SGI302960, SGI302961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71856[] = {SGI302962, SGI0, SGI302963, SGI302964, SGI302965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71857[] = {SGI302966, SGI0, SGI302967, SGI302968, SGI302969, SGI302970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302971, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71858[] = {SGI302972, SGI0, SGI302973, SGI302974, SGI302975, SGI302976, SGI0, SGI302977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302978, SGI0, SGI0, NULL};
const Segment * const SG71859[] = {SGI302979, SGI0, SGI302980, SGI0, SGI302981, SGI302982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302983, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71860[] = {SGI302984, SGI0, SGI302985, SGI302986, SGI302987, SGI302988, SGI0, SGI302989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302991, SGI0, SGI0, NULL};
const Segment * const SG71861[] = {SGI302992, SGI0, SGI302993, SGI302994, SGI302995, SGI302996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI302997, SGI0, NULL};
const Segment * const SG71862[] = {SGI302998, SGI0, SGI302999, SGI303000, SGI303001, SGI303002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303004, SGI0, NULL};
const Segment * const SG71863[] = {SGI303005, SGI0, SGI303006, SGI303007, SGI303008, SGI303009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71864[] = {SGI303011, SGI0, SGI303012, SGI303013, SGI303014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71865[] = {SGI303015, SGI0, SGI303016, SGI303017, SGI303018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71866[] = {SGI303019, SGI0, SGI303020, SGI303021, SGI303022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71867[] = {SGI303023, SGI0, SGI303024, SGI303025, SGI303026, SGI303027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71868[] = {SGI303028, SGI0, SGI303029, SGI303030, SGI303031, SGI303032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71869[] = {SGI303034, SGI0, SGI303035, SGI303036, SGI303037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71870[] = {SGI303038, SGI0, SGI303039, SGI303040, SGI303041, SGI303042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303044, SGI0, NULL};
const Segment * const SG71871[] = {SGI303045, SGI0, SGI303046, SGI303047, SGI303048, SGI303049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303050, SGI0, NULL};
const Segment * const SG71872[] = {SGI303051, SGI0, SGI303052, SGI303053, SGI303054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71873[] = {SGI303055, SGI0, SGI303056, SGI303057, SGI303058, SGI303059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71874[] = {SGI303061, SGI0, SGI303062, SGI303063, SGI303064, SGI303065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303066, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71875[] = {SGI303067, SGI0, SGI303068, SGI303069, SGI303070, SGI303071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303072, SGI0, SGI0, NULL};
const Segment * const SG71876[] = {SGI303073, SGI0, SGI303074, SGI303075, SGI303076, SGI303077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303079, SGI0, SGI0, NULL};
const Segment * const SG71877[] = {SGI303080, SGI0, SGI303081, SGI303082, SGI303083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71878[] = {SGI303084, SGI0, SGI303085, SGI0, SGI303086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71879[] = {SGI303087, SGI0, SGI303088, SGI303089, SGI303090, SGI303091, SGI0, SGI303092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71880[] = {SGI303094, SGI0, SGI303095, SGI303096, SGI303097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71881[] = {SGI303098, SGI0, SGI303099, SGI303100, SGI303101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71882[] = {SGI303102, SGI0, SGI303103, SGI303104, SGI303105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71883[] = {SGI303106, SGI0, SGI303107, SGI303108, SGI303109, SGI303110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303112, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71884[] = {SGI303113, SGI0, SGI303114, SGI303115, SGI303116, SGI303117, SGI0, SGI303118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71885[] = {SGI303119, SGI0, SGI0, SGI0, SGI303120, SGI303121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71886[] = {SGI303123, SGI0, SGI303124, SGI303125, SGI303126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71887[] = {SGI303127, SGI0, SGI303128, SGI303129, SGI303130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71888[] = {SGI303131, SGI0, SGI303132, SGI303133, SGI303134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71889[] = {SGI303135, SGI0, SGI303136, SGI303137, SGI303138, SGI303139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303140, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71890[] = {SGI303141, SGI0, SGI303142, SGI303143, SGI303144, SGI303145, SGI0, SGI0, SGI0, SGI0, SGI303146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303147, SGI0, SGI303148, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71891[] = {SGI303149, SGI0, SGI303150, SGI303151, SGI303152, SGI303153, SGI0, SGI0, SGI0, SGI0, SGI303154, SGI0, SGI0, SGI303155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71892[] = {SGI303156, SGI0, SGI303157, SGI303158, SGI303159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71893[] = {SGI303161, SGI0, SGI303162, SGI303163, SGI303164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71894[] = {SGI303165, SGI0, SGI303166, SGI303167, SGI303168, SGI303169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71895[] = {SGI303170, SGI0, SGI303171, SGI303172, SGI303173, SGI303174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71896[] = {SGI303176, SGI0, SGI303177, SGI303178, SGI303179, SGI303180, SGI0, SGI303181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71897[] = {SGI303182, SGI0, SGI303183, SGI303184, SGI303185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71898[] = {SGI303186, SGI0, SGI303187, SGI303188, SGI303189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71899[] = {SGI303190, SGI0, SGI303191, SGI303192, SGI303193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71900[] = {SGI303194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71901[] = {SGI303195, SGI0, SGI303196, SGI303197, SGI303198, SGI303199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71902[] = {SGI303201, SGI0, SGI303202, SGI303203, SGI303204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71903[] = {SGI303205, SGI0, SGI303206, SGI303207, SGI303208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71904[] = {SGI303209, SGI0, SGI303210, SGI303211, SGI303212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71905[] = {SGI303213, SGI0, SGI303214, SGI303215, SGI303216, SGI303217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71906[] = {SGI303218, SGI0, SGI303219, SGI303220, SGI303221, SGI303222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303223, SGI0, SGI0, NULL};
const Segment * const SG71907[] = {SGI303224, SGI0, SGI303225, SGI303226, SGI303227, SGI303228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303229, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71908[] = {SGI303230, SGI0, SGI303231, SGI303232, SGI303233, SGI303234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71909[] = {SGI303235, SGI0, SGI303236, SGI303237, SGI303238, SGI303239, SGI0, SGI303240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303241, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71910[] = {SGI303242, SGI0, SGI303243, SGI303244, SGI303245, SGI303246, SGI0, SGI303247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71911[] = {SGI303248, SGI0, SGI303249, SGI303250, SGI303251, SGI303252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71912[] = {SGI303254, SGI0, SGI303255, SGI303256, SGI303257, SGI303258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71913[] = {SGI303259, SGI0, SGI303260, SGI303261, SGI303262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71914[] = {SGI303263, SGI0, SGI303264, SGI303265, SGI303266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71915[] = {SGI303267, SGI0, SGI303268, SGI303269, SGI303270, SGI303271, SGI0, SGI303272, SGI0, SGI0, SGI303273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71916[] = {SGI303274, SGI0, SGI303275, SGI303276, SGI303277, SGI303278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71917[] = {SGI303279, SGI0, SGI303280, SGI303281, SGI303282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71918[] = {SGI303283, SGI0, SGI303284, SGI303285, SGI303286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71919[] = {SGI303287, SGI0, SGI303288, SGI303289, SGI303290, SGI303291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71920[] = {SGI303292, SGI0, SGI303293, SGI303294, SGI303295, SGI303296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71921[] = {SGI303297, SGI0, SGI303298, SGI303299, SGI303300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71922[] = {SGI303301, SGI0, SGI303302, SGI303303, SGI303304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71923[] = {SGI303305, SGI0, SGI303306, SGI303307, SGI303308, SGI303309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71924[] = {SGI303310, SGI0, SGI303311, SGI303312, SGI303313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71925[] = {SGI303314, SGI0, SGI303315, SGI303316, SGI303317, SGI303318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303319, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71926[] = {SGI303320, SGI0, SGI303321, SGI303322, SGI303323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71927[] = {SGI303324, SGI0, SGI303325, SGI303326, SGI303327, SGI303328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303329, SGI0, SGI0, NULL};
const Segment * const SG71928[] = {SGI303330, SGI0, SGI303331, SGI303332, SGI303333, SGI303334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71929[] = {SGI303335, SGI0, SGI303336, SGI303337, SGI303338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71930[] = {SGI303339, SGI0, SGI303340, SGI303341, SGI303342, SGI303343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303344, SGI0, SGI0, NULL};
const Segment * const SG71931[] = {SGI303345, SGI0, SGI303346, SGI303347, SGI303348, SGI303349, SGI0, SGI303350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303351, SGI0, SGI0, NULL};
const Segment * const SG71932[] = {SGI303352, SGI0, SGI303353, SGI303354, SGI303355, SGI303356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303357, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71933[] = {SGI303358, SGI0, SGI303359, SGI303360, SGI303361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71934[] = {SGI303362, SGI0, SGI303363, SGI303364, SGI303365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71935[] = {SGI303366, SGI0, SGI303367, SGI303368, SGI303369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71936[] = {SGI303370, SGI0, SGI303371, SGI303372, SGI303373, SGI303374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71937[] = {SGI303375, SGI0, SGI303376, SGI303377, SGI303378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303379, SGI0, SGI0, NULL};
const Segment * const SG71938[] = {SGI303380, SGI0, SGI303381, SGI303382, SGI303383, SGI303384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303385, SGI0, SGI0, NULL};
const Segment * const SG71939[] = {SGI303386, SGI0, SGI303387, SGI303388, SGI303389, SGI303390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303391, SGI0, SGI0, NULL};
const Segment * const SG71940[] = {SGI303392, SGI0, SGI303393, SGI303394, SGI303395, SGI303396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303397, SGI0, SGI0, SGI303398, SGI0, SGI0, NULL};
const Segment * const SG71941[] = {SGI303399, SGI0, SGI303400, SGI303401, SGI303402, SGI303403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71942[] = {SGI303404, SGI0, SGI303405, SGI303406, SGI303407, SGI303408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303410, SGI0, SGI0, NULL};
const Segment * const SG71943[] = {SGI303411, SGI0, SGI303412, SGI303413, SGI303414, SGI303415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303416, SGI0, SGI0, NULL};
const Segment * const SG71944[] = {SGI303417, SGI0, SGI303418, SGI303419, SGI303420, SGI303421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303422, SGI0, SGI0, NULL};
const Segment * const SG71945[] = {SGI303423, SGI0, SGI303424, SGI303425, SGI303426, SGI303427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303428, SGI0, SGI0, NULL};
const Segment * const SG71946[] = {SGI303429, SGI0, SGI303430, SGI303431, SGI303432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71947[] = {SGI303433, SGI0, SGI303434, SGI303435, SGI303436, SGI303437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71948[] = {SGI303438, SGI0, SGI303439, SGI303440, SGI303441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71949[] = {SGI303442, SGI0, SGI303443, SGI303444, SGI303445, SGI303446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303447, SGI0, SGI0, NULL};
const Segment * const SG71950[] = {SGI303448, SGI0, SGI303449, SGI303450, SGI303451, SGI303452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303453, SGI303454, SGI0, SGI0, NULL};
const Segment * const SG71951[] = {SGI303455, SGI0, SGI303456, SGI303457, SGI303458, SGI303459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303460, SGI0, SGI0, NULL};
const Segment * const SG71952[] = {SGI303461, SGI0, SGI303462, SGI303463, SGI303464, SGI303465, SGI0, SGI303466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303467, SGI0, SGI0, NULL};
const Segment * const SG71953[] = {SGI303468, SGI0, SGI0, SGI0, SGI303469, SGI303470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71954[] = {SGI303471, SGI0, SGI303472, SGI303473, SGI303474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71955[] = {SGI303475, SGI0, SGI303475, SGI303476, SGI303477, SGI303478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71956[] = {SGI303479, SGI0, SGI303480, SGI303481, SGI303482, SGI303483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71957[] = {SGI303484, SGI0, SGI303485, SGI303486, SGI303487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71958[] = {SGI303488, SGI0, SGI303489, SGI303490, SGI303491, SGI303492, SGI0, SGI0, SGI0, SGI0, SGI303493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303494, SGI0, SGI0, NULL};
const Segment * const SG71959[] = {SGI303495, SGI0, SGI303496, SGI0, SGI303497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71960[] = {SGI303498, SGI0, SGI303499, SGI303500, SGI303501, SGI303502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303503, SGI0, SGI0, NULL};
const Segment * const SG71961[] = {SGI303504, SGI0, SGI303505, SGI303506, SGI303507, SGI303508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303509, SGI0, SGI0, NULL};
const Segment * const SG71962[] = {SGI303510, SGI0, SGI303511, SGI303512, SGI303513, SGI303514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303515, SGI0, SGI0, NULL};
const Segment * const SG71963[] = {SGI303516, SGI0, SGI0, SGI0, SGI303517, SGI303518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71964[] = {SGI303519, SGI0, SGI303520, SGI303521, SGI303522, SGI303523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303524, SGI0, SGI0, NULL};
const Segment * const SG71965[] = {SGI303525, SGI0, SGI303526, SGI303527, SGI303528, SGI303529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303530, SGI0, SGI0, NULL};
const Segment * const SG71966[] = {SGI303531, SGI0, SGI303532, SGI303533, SGI303534, SGI303535, SGI0, SGI303536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303537, SGI0, SGI0, NULL};
const Segment * const SG71967[] = {SGI303538, SGI0, SGI303539, SGI303540, SGI303541, SGI303542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303543, SGI0, SGI0, NULL};
const Segment * const SG71968[] = {SGI303544, SGI0, SGI0, SGI0, SGI303545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71969[] = {SGI303546, SGI0, SGI303547, SGI303548, SGI303549, SGI303550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303551, SGI0, SGI0, NULL};
const Segment * const SG71970[] = {SGI303552, SGI0, SGI303553, SGI303554, SGI303555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71971[] = {SGI303556, SGI0, SGI303557, SGI303558, SGI303559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71972[] = {SGI303560, SGI0, SGI303561, SGI303562, SGI303563, SGI303564, SGI0, SGI303565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71973[] = {SGI303566, SGI0, SGI303567, SGI303568, SGI303569, SGI303570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303571, SGI0, SGI0, NULL};
const Segment * const SG71974[] = {SGI303572, SGI0, SGI303573, SGI303574, SGI303575, SGI303576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303578, SGI0, SGI0, NULL};
const Segment * const SG71975[] = {SGI303579, SGI0, SGI303580, SGI303581, SGI303582, SGI303583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303584, SGI0, SGI0, NULL};
const Segment * const SG71976[] = {SGI303585, SGI0, SGI303586, SGI303587, SGI303588, SGI303589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303590, SGI0, SGI0, NULL};
const Segment * const SG71977[] = {SGI303591, SGI0, SGI303592, SGI303593, SGI303594, SGI303595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303596, SGI0, SGI0, NULL};
const Segment * const SG71978[] = {SGI303597, SGI0, SGI303598, SGI303599, SGI303600, SGI303601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303602, SGI303603, SGI0, SGI0, NULL};
const Segment * const SG71979[] = {SGI303604, SGI0, SGI303605, SGI303606, SGI303607, SGI303608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71980[] = {SGI0, SGI0, SGI303610, SGI303611, SGI303612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303613, SGI0, SGI0, NULL};
const Segment * const SG71981[] = {SGI303614, SGI0, SGI0, SGI0, SGI303615, SGI303616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303617, SGI0, SGI0, NULL};
const Segment * const SG71982[] = {SGI303618, SGI0, SGI303619, SGI303620, SGI303621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71983[] = {SGI303622, SGI0, SGI303623, SGI303624, SGI303625, SGI303626, SGI0, SGI303627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303628, SGI0, SGI0, NULL};
const Segment * const SG71984[] = {SGI303629, SGI0, SGI303630, SGI303631, SGI303632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71985[] = {SGI303633, SGI0, SGI303634, SGI303635, SGI303636, SGI303637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71986[] = {SGI303638, SGI0, SGI303639, SGI303640, SGI303641, SGI303642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303643, SGI0, SGI0, NULL};
const Segment * const SG71987[] = {SGI303644, SGI0, SGI303645, SGI303646, SGI303647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303648, SGI0, SGI0, NULL};
const Segment * const SG71988[] = {SGI303649, SGI0, SGI303650, SGI303651, SGI303652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303653, SGI0, SGI0, NULL};
const Segment * const SG71989[] = {SGI303654, SGI0, SGI303655, SGI303656, SGI303657, SGI303658, SGI0, SGI303659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71990[] = {SGI303661, SGI0, SGI303662, SGI303663, SGI303664, SGI303665, SGI0, SGI303666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303667, SGI0, SGI0, NULL};
const Segment * const SG71991[] = {SGI303668, SGI0, SGI303669, SGI303670, SGI303671, SGI303672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303673, SGI0, SGI0, NULL};
const Segment * const SG71992[] = {SGI303674, SGI0, SGI303675, SGI303676, SGI303677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303678, SGI0, SGI0, NULL};
const Segment * const SG71993[] = {SGI303679, SGI0, SGI303680, SGI303681, SGI303682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71994[] = {SGI303683, SGI0, SGI303684, SGI303685, SGI303686, SGI303687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71995[] = {SGI303688, SGI0, SGI303689, SGI303690, SGI303691, SGI303692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303693, SGI0, SGI0, NULL};
const Segment * const SG71996[] = {SGI303694, SGI0, SGI303695, SGI303696, SGI303697, SGI303698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303699, SGI0, SGI303700, SGI0, SGI0, NULL};
const Segment * const SG71997[] = {SGI303701, SGI0, SGI303702, SGI303703, SGI303704, SGI303705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG71998[] = {SGI303706, SGI0, SGI303707, SGI303708, SGI303709, SGI303710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303711, SGI0, SGI0, NULL};
const Segment * const SG71999[] = {SGI303712, SGI0, SGI303713, SGI303714, SGI303715, SGI303716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303717, SGI0, SGI0, NULL};
const Segment * const SG72000[] = {SGI303718, SGI0, SGI303719, SGI303720, SGI303721, SGI303722, SGI0, SGI303723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72001[] = {SGI303724, SGI0, SGI303725, SGI303726, SGI303727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72002[] = {SGI303729, SGI0, SGI303730, SGI303731, SGI303732, SGI303733, SGI0, SGI303734, SGI0, SGI0, SGI303735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303736, SGI0, SGI0, NULL};
const Segment * const SG72003[] = {SGI303737, SGI0, SGI303738, SGI303739, SGI303740, SGI303741, SGI0, SGI303742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303743, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72004[] = {SGI303744, SGI0, SGI303745, SGI303746, SGI303747, SGI303748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303749, SGI0, SGI0, NULL};
const Segment * const SG72005[] = {SGI303750, SGI0, SGI303751, SGI303752, SGI303753, SGI303754, SGI0, SGI0, SGI0, SGI0, SGI303755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303756, SGI0, SGI0, NULL};
const Segment * const SG72006[] = {SGI303757, SGI0, SGI303758, SGI303759, SGI303760, SGI303761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303762, SGI0, SGI0, NULL};
const Segment * const SG72007[] = {SGI303763, SGI0, SGI303764, SGI303765, SGI303766, SGI303767, SGI0, SGI303768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303769, SGI0, SGI0, NULL};
const Segment * const SG72008[] = {SGI303770, SGI0, SGI303771, SGI303772, SGI303773, SGI303774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303775, SGI0, SGI0, NULL};
const Segment * const SG72009[] = {SGI303776, SGI0, SGI303777, SGI303778, SGI303779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303780, SGI0, SGI0, NULL};
const Segment * const SG72010[] = {SGI303781, SGI0, SGI303782, SGI303783, SGI303784, SGI303785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72011[] = {SGI303787, SGI0, SGI303788, SGI303789, SGI303790, SGI303791, SGI0, SGI303792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303793, SGI0, SGI0, NULL};
const Segment * const SG72012[] = {SGI303794, SGI0, SGI303795, SGI303796, SGI303797, SGI303798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303799, SGI0, SGI0, NULL};
const Segment * const SG72013[] = {SGI303800, SGI0, SGI303801, SGI303802, SGI303803, SGI303804, SGI0, SGI303805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72014[] = {SGI303806, SGI0, SGI303807, SGI303808, SGI303809, SGI303810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303811, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72015[] = {SGI303812, SGI0, SGI303813, SGI303814, SGI303815, SGI303816, SGI0, SGI303817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303818, SGI0, SGI0, NULL};
const Segment * const SG72016[] = {SGI303819, SGI0, SGI303820, SGI303821, SGI303822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72017[] = {SGI303823, SGI0, SGI303824, SGI303825, SGI303826, SGI303827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72018[] = {SGI303828, SGI0, SGI303829, SGI303830, SGI303831, SGI303832, SGI0, SGI303833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303834, SGI303835, SGI0, SGI0, NULL};
const Segment * const SG72019[] = {SGI303836, SGI0, SGI303837, SGI303838, SGI303839, SGI303840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303841, SGI0, SGI0, NULL};
const Segment * const SG72020[] = {SGI303842, SGI0, SGI303843, SGI303844, SGI303845, SGI303846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72021[] = {SGI303847, SGI0, SGI303848, SGI303849, SGI303850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72022[] = {SGI303851, SGI0, SGI303852, SGI303853, SGI303854, SGI303855, SGI0, SGI303856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72023[] = {SGI303857, SGI0, SGI303858, SGI303859, SGI303860, SGI303861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303862, SGI303863, SGI0, NULL};
const Segment * const SG72024[] = {SGI303864, SGI0, SGI303865, SGI303866, SGI303867, SGI303868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303869, SGI0, SGI0, NULL};
const Segment * const SG72025[] = {SGI303870, SGI0, SGI303871, SGI303872, SGI303873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72026[] = {SGI303874, SGI0, SGI303875, SGI303876, SGI303877, SGI303878, SGI0, SGI303879, SGI0, SGI0, SGI303880, SGI0, SGI0, SGI303881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72027[] = {SGI303882, SGI0, SGI303883, SGI303884, SGI303885, SGI303886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303887, SGI303888, SGI0, NULL};
const Segment * const SG72028[] = {SGI303889, SGI0, SGI303890, SGI303891, SGI303892, SGI303893, SGI0, SGI303894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72029[] = {SGI303896, SGI0, SGI303897, SGI303898, SGI303899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72030[] = {SGI303900, SGI0, SGI303901, SGI303902, SGI303903, SGI303904, SGI0, SGI303905, SGI0, SGI0, SGI303906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72031[] = {SGI303907, SGI0, SGI303908, SGI303909, SGI303910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72032[] = {SGI303911, SGI0, SGI303911, SGI303912, SGI303913, SGI303914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303915, SGI0, SGI0, NULL};
const Segment * const SG72033[] = {SGI303916, SGI0, SGI303917, SGI303918, SGI303919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303920, SGI0, SGI0, NULL};
const Segment * const SG72034[] = {SGI303921, SGI0, SGI303922, SGI303923, SGI303924, SGI303925, SGI0, SGI303926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303927, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72035[] = {SGI303928, SGI0, SGI303929, SGI303930, SGI303931, SGI303932, SGI0, SGI303933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303935, SGI0, SGI0, NULL};
const Segment * const SG72036[] = {SGI303936, SGI0, SGI303937, SGI303938, SGI303939, SGI303940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72037[] = {SGI303941, SGI0, SGI303942, SGI303943, SGI303944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72038[] = {SGI303945, SGI0, SGI303946, SGI0, SGI303947, SGI303948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72039[] = {SGI303949, SGI0, SGI303950, SGI303951, SGI303952, SGI303953, SGI0, SGI303954, SGI0, SGI0, SGI303955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303956, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72040[] = {SGI303957, SGI0, SGI303958, SGI303959, SGI303960, SGI303961, SGI0, SGI303962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303963, SGI0, SGI0, NULL};
const Segment * const SG72041[] = {SGI303964, SGI0, SGI303965, SGI303966, SGI303967, SGI303968, SGI0, SGI0, SGI0, SGI0, SGI303969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72042[] = {SGI303970, SGI0, SGI303971, SGI303972, SGI303973, SGI303974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303976, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72043[] = {SGI303977, SGI0, SGI303978, SGI303979, SGI303980, SGI303981, SGI0, SGI303982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303983, SGI303984, SGI0, SGI0, NULL};
const Segment * const SG72044[] = {SGI303985, SGI0, SGI303986, SGI303987, SGI303988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72045[] = {SGI303989, SGI0, SGI303990, SGI303991, SGI303992, SGI303993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI303994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72046[] = {SGI303995, SGI0, SGI303996, SGI303997, SGI303998, SGI303999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72047[] = {SGI304000, SGI0, SGI304001, SGI304002, SGI304003, SGI304004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72048[] = {SGI304005, SGI0, SGI304006, SGI304007, SGI304008, SGI304009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304010, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72049[] = {SGI304011, SGI0, SGI304012, SGI304013, SGI304014, SGI304015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304016, SGI0, SGI0, NULL};
const Segment * const SG72050[] = {SGI304017, SGI0, SGI304018, SGI304019, SGI304020, SGI304021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72051[] = {SGI304022, SGI0, SGI0, SGI0, SGI304023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72052[] = {SGI304024, SGI0, SGI304025, SGI304026, SGI304027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72053[] = {SGI304028, SGI0, SGI304029, SGI304030, SGI304031, SGI304032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304033, SGI0, SGI0, NULL};
const Segment * const SG72054[] = {SGI304034, SGI0, SGI304035, SGI304036, SGI304037, SGI304038, SGI0, SGI304039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72055[] = {SGI304041, SGI0, SGI304042, SGI304043, SGI304044, SGI304045, SGI0, SGI304046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72056[] = {SGI304047, SGI0, SGI304048, SGI304049, SGI304050, SGI304051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72057[] = {SGI304053, SGI0, SGI304054, SGI304055, SGI304056, SGI304057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72058[] = {SGI304058, SGI0, SGI304059, SGI304060, SGI304061, SGI304062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304064, SGI0, NULL};
const Segment * const SG72059[] = {SGI304065, SGI0, SGI304066, SGI304067, SGI304068, SGI304069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304070, SGI0, NULL};
const Segment * const SG72060[] = {SGI304071, SGI0, SGI304072, SGI304073, SGI304074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72061[] = {SGI304075, SGI0, SGI304076, SGI304077, SGI304078, SGI304079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304080, SGI0, NULL};
const Segment * const SG72062[] = {SGI304081, SGI0, SGI304082, SGI304083, SGI304084, SGI304085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304087, SGI0, NULL};
const Segment * const SG72063[] = {SGI304088, SGI0, SGI304089, SGI304090, SGI304091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72064[] = {SGI304092, SGI0, SGI304093, SGI304094, SGI304095, SGI304096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72065[] = {SGI304097, SGI0, SGI304098, SGI304099, SGI304100, SGI304101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72066[] = {SGI304103, SGI0, SGI304104, SGI304105, SGI304106, SGI304107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72067[] = {SGI304109, SGI0, SGI304110, SGI304111, SGI304112, SGI304113, SGI0, SGI304114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304115, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72068[] = {SGI304116, SGI0, SGI304117, SGI304118, SGI304119, SGI304120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72069[] = {SGI304121, SGI0, SGI304122, SGI304123, SGI304124, SGI304125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72070[] = {SGI304127, SGI0, SGI304128, SGI304129, SGI304130, SGI304131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304132, SGI0, SGI304133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72071[] = {SGI304134, SGI0, SGI0, SGI0, SGI304135, SGI304136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72072[] = {SGI304138, SGI0, SGI304139, SGI304140, SGI304141, SGI304142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304143, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72073[] = {SGI304144, SGI0, SGI304145, SGI304146, SGI304147, SGI304148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72074[] = {SGI304150, SGI0, SGI304151, SGI304152, SGI304153, SGI304154, SGI0, SGI304155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72075[] = {SGI304156, SGI0, SGI304157, SGI304158, SGI304159, SGI304160, SGI0, SGI304161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304162, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72076[] = {SGI304163, SGI0, SGI304164, SGI304165, SGI304166, SGI304167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72077[] = {SGI304169, SGI0, SGI304170, SGI304171, SGI304172, SGI304173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72078[] = {SGI304174, SGI0, SGI304175, SGI304176, SGI304177, SGI304178, SGI0, SGI304179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304180, SGI0, SGI304181, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72079[] = {SGI304182, SGI0, SGI304183, SGI304184, SGI304185, SGI304186, SGI0, SGI304187, SGI0, SGI0, SGI304188, SGI0, SGI0, SGI304189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72080[] = {SGI304190, SGI0, SGI304191, SGI304192, SGI304193, SGI304194, SGI0, SGI304195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72081[] = {SGI304197, SGI0, SGI0, SGI0, SGI304198, SGI304199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72082[] = {SGI304200, SGI0, SGI304201, SGI304202, SGI304203, SGI304204, SGI0, SGI304205, SGI0, SGI0, SGI304206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304207, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72083[] = {SGI304208, SGI0, SGI304209, SGI304210, SGI304211, SGI304212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72084[] = {SGI304213, SGI0, SGI304214, SGI0, SGI304215, SGI304216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72085[] = {SGI304217, SGI0, SGI304218, SGI304219, SGI304220, SGI304221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72086[] = {SGI304223, SGI0, SGI304224, SGI304225, SGI304226, SGI304227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72087[] = {SGI304229, SGI304230, SGI304231, SGI304232, SGI304233, SGI304234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72088[] = {SGI304236, SGI304237, SGI304238, SGI304239, SGI304240, SGI304241, SGI0, SGI304242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304243, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72089[] = {SGI304244, SGI304245, SGI304246, SGI304247, SGI304248, SGI304249, SGI0, SGI304250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304251, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72090[] = {SGI304252, SGI304253, SGI304254, SGI304255, SGI304256, SGI304257, SGI0, SGI304258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72091[] = {SGI304260, SGI304261, SGI304262, SGI304263, SGI304264, SGI304265, SGI0, SGI304266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72092[] = {SGI304267, SGI304268, SGI304269, SGI304270, SGI304271, SGI304272, SGI0, SGI304273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304274, SGI0, SGI304275, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72093[] = {SGI304276, SGI304277, SGI304278, SGI304279, SGI304280, SGI304281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72094[] = {SGI304282, SGI304283, SGI304284, SGI304285, SGI304286, SGI304287, SGI0, SGI304288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72095[] = {SGI304290, SGI304291, SGI304292, SGI304293, SGI304294, SGI304295, SGI0, SGI304296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72096[] = {SGI304297, SGI304298, SGI304299, SGI304300, SGI304301, SGI304302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304303, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72097[] = {SGI304304, SGI304305, SGI304306, SGI304307, SGI304308, SGI304309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72098[] = {SGI304311, SGI304312, SGI304313, SGI304314, SGI304315, SGI304316, SGI0, SGI304317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72099[] = {SGI304318, SGI304319, SGI304320, SGI304321, SGI304322, SGI304323, SGI0, SGI304324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72100[] = {SGI304325, SGI304326, SGI304327, SGI304328, SGI304329, SGI304330, SGI0, SGI304331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72101[] = {SGI304332, SGI304333, SGI304334, SGI304335, SGI304336, SGI304337, SGI0, SGI304338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72102[] = {SGI304340, SGI304341, SGI304342, SGI304343, SGI304344, SGI304345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72103[] = {SGI304347, SGI304348, SGI304349, SGI304350, SGI304351, SGI304352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72104[] = {SGI304354, SGI304355, SGI304356, SGI304357, SGI304358, SGI304359, SGI0, SGI304360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304361, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72105[] = {SGI304362, SGI304363, SGI304364, SGI304365, SGI304366, SGI304367, SGI0, SGI304368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304369, SGI0, SGI304370, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72106[] = {SGI304371, SGI304372, SGI304373, SGI304374, SGI304375, SGI304376, SGI0, SGI304377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72107[] = {SGI304378, SGI304379, SGI304380, SGI304381, SGI304382, SGI304383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72108[] = {SGI304384, SGI304385, SGI304386, SGI304387, SGI304388, SGI304389, SGI0, SGI304390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72109[] = {SGI304392, SGI304393, SGI304394, SGI304395, SGI304396, SGI304397, SGI0, SGI304398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304399, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72110[] = {SGI304400, SGI304401, SGI304402, SGI304403, SGI304404, SGI304405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304406, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72111[] = {SGI304407, SGI304408, SGI304409, SGI304410, SGI304411, SGI304412, SGI0, SGI304413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304414, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72112[] = {SGI304415, SGI304416, SGI304417, SGI304418, SGI304419, SGI304420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72113[] = {SGI304422, SGI304423, SGI304424, SGI304425, SGI304426, SGI304427, SGI0, SGI304428, SGI0, SGI0, SGI304429, SGI0, SGI0, SGI304430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72114[] = {SGI304431, SGI304432, SGI304433, SGI304434, SGI304435, SGI304436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72115[] = {SGI304438, SGI304439, SGI304440, SGI304441, SGI304442, SGI304443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72116[] = {SGI304444, SGI304445, SGI304446, SGI304447, SGI304448, SGI304449, SGI0, SGI304450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72117[] = {SGI304452, SGI304453, SGI304454, SGI304455, SGI304456, SGI304457, SGI0, SGI304458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304459, SGI0, SGI304460, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72118[] = {SGI304461, SGI304462, SGI304463, SGI304464, SGI304465, SGI304466, SGI0, SGI304467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72119[] = {SGI304469, SGI304470, SGI304471, SGI304472, SGI304473, SGI304474, SGI0, SGI304475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72120[] = {SGI304476, SGI304477, SGI304478, SGI304479, SGI304480, SGI304481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72121[] = {SGI304483, SGI304484, SGI304485, SGI304486, SGI304487, SGI304488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72122[] = {SGI304489, SGI304490, SGI304491, SGI304492, SGI304493, SGI304494, SGI0, SGI304495, SGI0, SGI0, SGI304496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304497, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72123[] = {SGI304498, SGI304499, SGI304500, SGI304501, SGI304502, SGI304503, SGI0, SGI304504, SGI0, SGI0, SGI304505, SGI0, SGI0, SGI304506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72124[] = {SGI304507, SGI304508, SGI304509, SGI304510, SGI304511, SGI304512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72125[] = {SGI304513, SGI304514, SGI304515, SGI304516, SGI304517, SGI304518, SGI0, SGI304519, SGI0, SGI0, SGI304520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304521, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72126[] = {SGI304522, SGI304523, SGI304524, SGI304525, SGI304526, SGI304527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304528, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72127[] = {SGI304529, SGI304530, SGI304531, SGI304532, SGI304533, SGI304534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72128[] = {SGI304536, SGI304537, SGI304538, SGI304539, SGI304540, SGI304541, SGI0, SGI304542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304543, SGI0, SGI304544, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72129[] = {SGI304545, SGI304546, SGI304547, SGI304548, SGI304549, SGI304550, SGI0, SGI304551, SGI0, SGI0, SGI304552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72130[] = {SGI304553, SGI304554, SGI304555, SGI304556, SGI304557, SGI304558, SGI0, SGI304559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72131[] = {SGI304560, SGI304561, SGI304562, SGI304563, SGI304564, SGI304565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304566, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72132[] = {SGI304567, SGI304568, SGI304569, SGI304570, SGI304571, SGI304572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304573, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72133[] = {SGI304574, SGI304575, SGI304576, SGI304577, SGI304578, SGI304579, SGI0, SGI304580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72134[] = {SGI304581, SGI304582, SGI304583, SGI304584, SGI304585, SGI304586, SGI0, SGI304587, SGI0, SGI0, SGI304588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304589, SGI0, SGI304590, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72135[] = {SGI304591, SGI304592, SGI304593, SGI304594, SGI304595, SGI304596, SGI0, SGI304597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72136[] = {SGI304598, SGI304599, SGI304600, SGI304601, SGI304602, SGI304603, SGI0, SGI304604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304605, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72137[] = {SGI304606, SGI304607, SGI304608, SGI304609, SGI304610, SGI304611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304612, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72138[] = {SGI304613, SGI304614, SGI304615, SGI304616, SGI304617, SGI304618, SGI0, SGI304619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72139[] = {SGI304620, SGI304621, SGI304622, SGI304623, SGI304624, SGI304625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72140[] = {SGI304626, SGI304627, SGI304628, SGI304629, SGI304630, SGI304631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72141[] = {SGI304632, SGI304633, SGI304634, SGI304635, SGI304636, SGI304637, SGI0, SGI304638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304639, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72142[] = {SGI304640, SGI304641, SGI304642, SGI304643, SGI304644, SGI304645, SGI0, SGI304646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72143[] = {SGI304647, SGI304648, SGI304649, SGI304650, SGI304651, SGI304652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304653, SGI0, SGI304654, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72144[] = {SGI304655, SGI304656, SGI304657, SGI304658, SGI304659, SGI304660, SGI0, SGI304661, SGI0, SGI0, SGI304662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304663, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72145[] = {SGI304664, SGI304665, SGI304666, SGI304667, SGI304668, SGI304669, SGI0, SGI304670, SGI0, SGI0, SGI304671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304672, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72146[] = {SGI304673, SGI304674, SGI304675, SGI304676, SGI304677, SGI304678, SGI0, SGI304679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72147[] = {SGI304681, SGI304682, SGI304683, SGI304684, SGI304685, SGI304686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72148[] = {SGI304687, SGI304688, SGI304689, SGI304690, SGI304691, SGI304692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72149[] = {SGI304694, SGI304695, SGI304696, SGI304697, SGI304698, SGI304699, SGI0, SGI304700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304701, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72150[] = {SGI304702, SGI304703, SGI304704, SGI304705, SGI304706, SGI304707, SGI0, SGI304708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304709, SGI0, SGI0, SGI0, SGI304710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72151[] = {SGI304711, SGI304712, SGI304713, SGI304714, SGI304715, SGI304716, SGI0, SGI304717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72152[] = {SGI304719, SGI304720, SGI304721, SGI304722, SGI304723, SGI304724, SGI0, SGI304725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304726, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72153[] = {SGI304727, SGI304728, SGI304729, SGI304730, SGI304731, SGI304732, SGI0, SGI304733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72154[] = {SGI304735, SGI304736, SGI304737, SGI304738, SGI304739, SGI304740, SGI0, SGI304741, SGI0, SGI0, SGI304742, SGI0, SGI0, SGI304743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72155[] = {SGI304744, SGI304745, SGI304746, SGI304747, SGI304748, SGI304749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304750, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72156[] = {SGI304751, SGI304752, SGI304753, SGI304754, SGI304755, SGI304756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304757, SGI0, SGI304758, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72157[] = {SGI304759, SGI304760, SGI304761, SGI304762, SGI304763, SGI304764, SGI0, SGI304765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72158[] = {SGI304766, SGI304767, SGI304768, SGI304769, SGI304770, SGI304771, SGI0, SGI304772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72159[] = {SGI304774, SGI304775, SGI304776, SGI304777, SGI304778, SGI304779, SGI0, SGI304780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304781, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72160[] = {SGI304782, SGI304783, SGI304784, SGI304785, SGI304786, SGI304787, SGI0, SGI304788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72161[] = {SGI304789, SGI304790, SGI304791, SGI304792, SGI304793, SGI304794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72162[] = {SGI304796, SGI304797, SGI304798, SGI304799, SGI304800, SGI304801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72163[] = {SGI304803, SGI304804, SGI304805, SGI304806, SGI304807, SGI304808, SGI0, SGI304809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304810, SGI0, SGI304811, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72164[] = {SGI304812, SGI304813, SGI304814, SGI304815, SGI304816, SGI304817, SGI0, SGI304818, SGI0, SGI0, SGI304819, SGI0, SGI0, SGI304820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72165[] = {SGI304821, SGI304822, SGI304823, SGI304824, SGI304825, SGI304826, SGI0, SGI304827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304828, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72166[] = {SGI304829, SGI304830, SGI304831, SGI304832, SGI304833, SGI304834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72167[] = {SGI304835, SGI304836, SGI304837, SGI304838, SGI304839, SGI304840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72168[] = {SGI304841, SGI304842, SGI304843, SGI304844, SGI304845, SGI304846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304847, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72169[] = {SGI304848, SGI304849, SGI304850, SGI304851, SGI304852, SGI304853, SGI0, SGI304854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72170[] = {SGI304856, SGI304857, SGI304858, SGI304859, SGI304860, SGI304861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304862, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72171[] = {SGI304863, SGI304864, SGI304865, SGI304866, SGI304867, SGI304868, SGI0, SGI304869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72172[] = {SGI304871, SGI304872, SGI304873, SGI304874, SGI304875, SGI304876, SGI0, SGI304877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72173[] = {SGI304879, SGI304880, SGI304881, SGI304882, SGI304883, SGI304884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72174[] = {SGI304886, SGI304887, SGI304888, SGI304889, SGI304890, SGI304891, SGI0, SGI304892, SGI0, SGI0, SGI304893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72175[] = {SGI304894, SGI304895, SGI304896, SGI304897, SGI304898, SGI304899, SGI0, SGI304900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72176[] = {SGI304902, SGI304903, SGI304904, SGI304905, SGI304906, SGI304907, SGI0, SGI304908, SGI0, SGI0, SGI304909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304910, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72177[] = {SGI304911, SGI304912, SGI304913, SGI304914, SGI304915, SGI304916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304917, SGI0, SGI304918, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72178[] = {SGI304919, SGI304920, SGI304921, SGI304922, SGI304923, SGI304924, SGI0, SGI304925, SGI0, SGI0, SGI304926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304927, SGI0, SGI304928, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72179[] = {SGI304929, SGI304930, SGI304931, SGI304932, SGI304933, SGI304934, SGI0, SGI304935, SGI0, SGI0, SGI304936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72180[] = {SGI304937, SGI304938, SGI304939, SGI304940, SGI304941, SGI304942, SGI0, SGI304943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72181[] = {SGI304945, SGI304946, SGI304947, SGI304948, SGI304949, SGI304950, SGI0, SGI304951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72182[] = {SGI304952, SGI304953, SGI304954, SGI304955, SGI304956, SGI304957, SGI0, SGI304958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304959, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72183[] = {SGI304960, SGI304961, SGI304962, SGI304963, SGI304964, SGI304965, SGI0, SGI304966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72184[] = {SGI304967, SGI304968, SGI304969, SGI304970, SGI304971, SGI304972, SGI0, SGI304973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304974, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72185[] = {SGI304975, SGI304976, SGI304977, SGI304978, SGI304979, SGI304980, SGI0, SGI304981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72186[] = {SGI304982, SGI304983, SGI304984, SGI304985, SGI304986, SGI304987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304988, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72187[] = {SGI304989, SGI304990, SGI304991, SGI304992, SGI304993, SGI304994, SGI0, SGI304995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI304996, SGI0, SGI0, NULL};
const Segment * const SG72188[] = {SGI304997, SGI304998, SGI304999, SGI305000, SGI305001, SGI305002, SGI0, SGI305003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305004, SGI305005, SGI0, SGI0, NULL};
const Segment * const SG72189[] = {SGI305006, SGI305007, SGI305008, SGI305009, SGI305010, SGI305011, SGI0, SGI305012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305013, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72190[] = {SGI305014, SGI305015, SGI305016, SGI305017, SGI305018, SGI305019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72191[] = {SGI305020, SGI305021, SGI305022, SGI305023, SGI305024, SGI305025, SGI0, SGI305026, SGI0, SGI0, SGI305027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305028, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72192[] = {SGI305029, SGI305030, SGI305031, SGI305032, SGI305033, SGI305034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72193[] = {SGI305036, SGI305037, SGI305038, SGI305039, SGI305040, SGI305041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72194[] = {SGI305043, SGI305044, SGI305045, SGI305046, SGI305047, SGI305048, SGI0, SGI305049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72195[] = {SGI305050, SGI305051, SGI305052, SGI305053, SGI305054, SGI305055, SGI0, SGI305056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72196[] = {SGI305058, SGI305059, SGI305060, SGI305061, SGI305062, SGI305063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305064, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72197[] = {SGI305065, SGI305066, SGI305067, SGI305068, SGI305069, SGI305070, SGI0, SGI305071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305072, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72198[] = {SGI305073, SGI305074, SGI305075, SGI305076, SGI305077, SGI305078, SGI0, SGI305079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305080, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72199[] = {SGI305081, SGI305082, SGI305083, SGI305084, SGI305085, SGI305086, SGI0, SGI305087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305088, SGI0, SGI305089, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72200[] = {SGI305090, SGI305091, SGI305092, SGI305093, SGI305094, SGI305095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72201[] = {SGI305096, SGI305097, SGI305098, SGI305099, SGI305100, SGI305101, SGI0, SGI305102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72202[] = {SGI305104, SGI305105, SGI305106, SGI305107, SGI305108, SGI305109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72203[] = {SGI305110, SGI305111, SGI305112, SGI305113, SGI305114, SGI305115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305116, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72204[] = {SGI305117, SGI305118, SGI305119, SGI305120, SGI305121, SGI305122, SGI0, SGI305123, SGI0, SGI0, SGI305124, SGI0, SGI0, SGI305125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72205[] = {SGI305126, SGI305127, SGI305128, SGI305129, SGI305130, SGI305131, SGI0, SGI305132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72206[] = {SGI305134, SGI305135, SGI305136, SGI305137, SGI305138, SGI305139, SGI0, SGI305140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305141, SGI0, SGI0, NULL};
const Segment * const SG72207[] = {SGI305142, SGI305143, SGI305144, SGI305145, SGI305146, SGI305147, SGI0, SGI305148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305149, SGI305150, SGI0, SGI0, NULL};
const Segment * const SG72208[] = {SGI305151, SGI305152, SGI305153, SGI305154, SGI305155, SGI305156, SGI0, SGI305157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72209[] = {SGI305159, SGI305160, SGI305161, SGI305162, SGI305163, SGI305164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72210[] = {SGI305166, SGI305167, SGI305168, SGI305169, SGI305170, SGI305171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305172, SGI0, SGI305173, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72211[] = {SGI305174, SGI305175, SGI305176, SGI305177, SGI305178, SGI305179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72212[] = {SGI305181, SGI305182, SGI305183, SGI305184, SGI305185, SGI305186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305187, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72213[] = {SGI305188, SGI305189, SGI305190, SGI305191, SGI305192, SGI305193, SGI0, SGI305194, SGI0, SGI0, SGI305195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305196, SGI0, SGI305197, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72214[] = {SGI305198, SGI305199, SGI305200, SGI305201, SGI305202, SGI305203, SGI0, SGI305204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72215[] = {SGI305206, SGI305207, SGI305208, SGI305209, SGI305210, SGI305211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305212, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72216[] = {SGI305213, SGI305214, SGI305215, SGI305216, SGI305217, SGI305218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72217[] = {SGI305219, SGI305220, SGI305221, SGI305222, SGI305223, SGI305224, SGI0, SGI305225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305226, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72218[] = {SGI305227, SGI305228, SGI305229, SGI305230, SGI305231, SGI305232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72219[] = {SGI305234, SGI305235, SGI305236, SGI305237, SGI305238, SGI305239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72220[] = {SGI305241, SGI305242, SGI305243, SGI305244, SGI305245, SGI305246, SGI0, SGI305247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72221[] = {SGI305248, SGI305249, SGI305250, SGI305251, SGI305252, SGI305253, SGI0, SGI305254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72222[] = {SGI305256, SGI305257, SGI305258, SGI305259, SGI305260, SGI305261, SGI0, SGI305262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72223[] = {SGI305263, SGI305264, SGI305265, SGI305266, SGI305267, SGI305268, SGI0, SGI305269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72224[] = {SGI305271, SGI305272, SGI305273, SGI305274, SGI305275, SGI305276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305277, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72225[] = {SGI305278, SGI305279, SGI305280, SGI305281, SGI305282, SGI305283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72226[] = {SGI305284, SGI305285, SGI305286, SGI305287, SGI305288, SGI305289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305290, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72227[] = {SGI305291, SGI305292, SGI305293, SGI305294, SGI305295, SGI305296, SGI0, SGI305297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72228[] = {SGI305299, SGI305300, SGI305301, SGI305302, SGI305303, SGI305304, SGI0, SGI305305, SGI0, SGI0, SGI305306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72229[] = {SGI305307, SGI305308, SGI305309, SGI305310, SGI305311, SGI305312, SGI0, SGI305313, SGI0, SGI0, SGI305314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305315, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72230[] = {SGI305316, SGI305317, SGI305318, SGI305319, SGI305320, SGI305321, SGI0, SGI305322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72231[] = {SGI305323, SGI305324, SGI305325, SGI305326, SGI305327, SGI305328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72232[] = {SGI305330, SGI305331, SGI305332, SGI305333, SGI305334, SGI305335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305336, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72233[] = {SGI305337, SGI305338, SGI305339, SGI305340, SGI305341, SGI305342, SGI0, SGI305343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72234[] = {SGI305344, SGI305345, SGI305346, SGI305347, SGI305348, SGI305349, SGI0, SGI305350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72235[] = {SGI305352, SGI305353, SGI305354, SGI305355, SGI305356, SGI305357, SGI0, SGI305358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72236[] = {SGI305360, SGI305361, SGI305362, SGI305363, SGI305364, SGI305365, SGI0, SGI305366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72237[] = {SGI305368, SGI305369, SGI305370, SGI305371, SGI305372, SGI305373, SGI0, SGI305374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72238[] = {SGI305375, SGI305376, SGI305377, SGI305378, SGI305379, SGI305380, SGI0, SGI305381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305382, SGI0, SGI0, SGI0, SGI305383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72239[] = {SGI305384, SGI305385, SGI305386, SGI305387, SGI305388, SGI305389, SGI0, SGI305390, SGI0, SGI0, SGI305391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305392, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72240[] = {SGI305393, SGI305394, SGI305395, SGI305396, SGI305397, SGI305398, SGI0, SGI305399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305400, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72241[] = {SGI305401, SGI305402, SGI305403, SGI305404, SGI305405, SGI305406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72242[] = {SGI305408, SGI305409, SGI305410, SGI305411, SGI305412, SGI305413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305415, SGI0, SGI0, NULL};
const Segment * const SG72243[] = {SGI305416, SGI305417, SGI305418, SGI305419, SGI305420, SGI305421, SGI0, SGI305422, SGI0, SGI0, SGI305423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72244[] = {SGI305424, SGI305425, SGI305426, SGI305427, SGI305428, SGI305429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72245[] = {SGI305431, SGI305432, SGI305433, SGI305434, SGI305435, SGI305436, SGI0, SGI305437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305438, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72246[] = {SGI305439, SGI305440, SGI305441, SGI305442, SGI305443, SGI305444, SGI0, SGI305445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305446, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72247[] = {SGI305447, SGI305448, SGI305449, SGI305450, SGI305451, SGI305452, SGI0, SGI305453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305454, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72248[] = {SGI305455, SGI305456, SGI305457, SGI305458, SGI305459, SGI305460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72249[] = {SGI305462, SGI305463, SGI305464, SGI305465, SGI305466, SGI305467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72250[] = {SGI305469, SGI305470, SGI305471, SGI305472, SGI305473, SGI305474, SGI0, SGI305475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72251[] = {SGI305477, SGI305478, SGI305479, SGI305480, SGI305481, SGI305482, SGI0, SGI305483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72252[] = {SGI305484, SGI305485, SGI305486, SGI305487, SGI305488, SGI305489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72253[] = {SGI305491, SGI305492, SGI305493, SGI305494, SGI305495, SGI305496, SGI0, SGI305497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72254[] = {SGI305498, SGI305499, SGI305500, SGI305501, SGI305502, SGI305503, SGI0, SGI305504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305505, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72255[] = {SGI305506, SGI305507, SGI305508, SGI305509, SGI305510, SGI305511, SGI0, SGI305512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72256[] = {SGI305513, SGI305514, SGI305515, SGI305516, SGI305517, SGI305518, SGI0, SGI305519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305520, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72257[] = {SGI305521, SGI305522, SGI305523, SGI305524, SGI305525, SGI305526, SGI0, SGI305527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72258[] = {SGI305529, SGI305530, SGI305531, SGI305532, SGI305533, SGI305534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72259[] = {SGI305535, SGI305536, SGI305537, SGI305538, SGI305539, SGI305540, SGI0, SGI305541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72260[] = {SGI305542, SGI305543, SGI305544, SGI305545, SGI305546, SGI305547, SGI0, SGI305548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305549, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72261[] = {SGI305550, SGI305551, SGI305552, SGI305553, SGI305554, SGI305555, SGI0, SGI305556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305557, SGI305558, SGI0, SGI0, NULL};
const Segment * const SG72262[] = {SGI305559, SGI305560, SGI305561, SGI305562, SGI305563, SGI305564, SGI0, SGI305565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72263[] = {SGI305567, SGI305568, SGI305569, SGI305570, SGI305571, SGI305572, SGI0, SGI305573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305574, SGI0, SGI305575, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72264[] = {SGI305576, SGI305577, SGI305578, SGI305579, SGI305580, SGI305581, SGI0, SGI305582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305583, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72265[] = {SGI305584, SGI305585, SGI305586, SGI305587, SGI305588, SGI305589, SGI0, SGI305590, SGI0, SGI0, SGI305591, SGI0, SGI0, SGI305592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72266[] = {SGI305593, SGI305594, SGI305595, SGI305596, SGI305597, SGI305598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305599, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72267[] = {SGI305600, SGI305601, SGI305602, SGI305603, SGI305604, SGI305605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72268[] = {SGI305606, SGI305607, SGI305608, SGI305609, SGI305610, SGI305611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72269[] = {SGI305612, SGI305613, SGI305614, SGI305615, SGI305616, SGI305617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72270[] = {SGI305619, SGI305620, SGI305621, SGI305622, SGI305623, SGI305624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72271[] = {SGI305626, SGI305627, SGI305628, SGI305629, SGI305630, SGI305631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72272[] = {SGI305633, SGI305634, SGI305635, SGI305636, SGI305637, SGI305638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72273[] = {SGI305639, SGI305640, SGI305641, SGI305642, SGI305643, SGI305644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305645, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72274[] = {SGI305646, SGI305647, SGI305648, SGI305649, SGI305650, SGI305651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72275[] = {SGI305653, SGI305654, SGI305655, SGI305656, SGI305657, SGI305658, SGI0, SGI305659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72276[] = {SGI305660, SGI305661, SGI305662, SGI305663, SGI305664, SGI305665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72277[] = {SGI305666, SGI305667, SGI305668, SGI305669, SGI305670, SGI305671, SGI0, SGI305672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72278[] = {SGI305673, SGI305674, SGI305675, SGI305676, SGI305677, SGI305678, SGI0, SGI305679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305680, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72279[] = {SGI305681, SGI305682, SGI305683, SGI305684, SGI305685, SGI305686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305687, SGI0, SGI305688, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72280[] = {SGI305689, SGI305690, SGI305691, SGI305692, SGI305693, SGI305694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305695, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72281[] = {SGI305696, SGI305697, SGI305698, SGI305699, SGI305700, SGI305701, SGI0, SGI305702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72282[] = {SGI305704, SGI305705, SGI305706, SGI305707, SGI305708, SGI305709, SGI0, SGI305710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72283[] = {SGI305712, SGI305713, SGI305714, SGI305715, SGI305716, SGI305717, SGI0, SGI305718, SGI0, SGI0, SGI305719, SGI0, SGI0, SGI305720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72284[] = {SGI305721, SGI305722, SGI305723, SGI305724, SGI305725, SGI305726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72285[] = {SGI305728, SGI305729, SGI305730, SGI305731, SGI305732, SGI305733, SGI0, SGI305734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305735, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72286[] = {SGI305736, SGI305737, SGI305738, SGI305739, SGI305740, SGI305741, SGI0, SGI305742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305743, SGI0, SGI305744, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72287[] = {SGI305745, SGI305746, SGI305747, SGI305748, SGI305749, SGI305750, SGI0, SGI305751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72288[] = {SGI305752, SGI305753, SGI305754, SGI305755, SGI305756, SGI305757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72289[] = {SGI305759, SGI305760, SGI305761, SGI305762, SGI305763, SGI305764, SGI0, SGI305765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305766, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72290[] = {SGI305767, SGI305768, SGI305769, SGI305770, SGI305771, SGI305772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72291[] = {SGI305774, SGI305775, SGI305776, SGI305777, SGI305778, SGI305779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72292[] = {SGI305780, SGI305781, SGI305782, SGI305783, SGI305784, SGI305785, SGI0, SGI305786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72293[] = {SGI305788, SGI305789, SGI305790, SGI305791, SGI305792, SGI305793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305794, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72294[] = {SGI305795, SGI305796, SGI305797, SGI305798, SGI305799, SGI305800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72295[] = {SGI305801, SGI305802, SGI305803, SGI305804, SGI305805, SGI305806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305807, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72296[] = {SGI305808, SGI305809, SGI305810, SGI305811, SGI305812, SGI305813, SGI0, SGI305814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72297[] = {SGI305816, SGI305817, SGI305818, SGI305819, SGI305820, SGI305821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305822, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72298[] = {SGI305823, SGI305824, SGI305825, SGI305826, SGI305827, SGI305828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72299[] = {SGI305829, SGI305830, SGI305831, SGI305832, SGI305833, SGI305834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305835, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72300[] = {SGI305836, SGI305837, SGI305838, SGI305839, SGI305840, SGI305841, SGI0, SGI305842, SGI0, SGI0, SGI305843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305844, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72301[] = {SGI305845, SGI305846, SGI305847, SGI305848, SGI305849, SGI305850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305851, SGI0, SGI305852, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72302[] = {SGI305853, SGI305854, SGI305855, SGI305856, SGI305857, SGI305858, SGI0, SGI305859, SGI0, SGI0, SGI305860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72303[] = {SGI305861, SGI305862, SGI305863, SGI305864, SGI305865, SGI305866, SGI0, SGI305867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72304[] = {SGI305868, SGI305869, SGI305870, SGI305871, SGI305872, SGI305873, SGI0, SGI305874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72305[] = {SGI305875, SGI305876, SGI305877, SGI305878, SGI305879, SGI305880, SGI0, SGI305881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72306[] = {SGI305882, SGI0, SGI305883, SGI305884, SGI305885, SGI305886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72307[] = {SGI305887, SGI0, SGI305888, SGI305889, SGI305890, SGI305891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72308[] = {SGI305893, SGI0, SGI0, SGI0, SGI305894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72309[] = {SGI305895, SGI0, SGI305896, SGI305897, SGI305898, SGI305899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305900, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72310[] = {SGI305901, SGI0, SGI0, SGI0, SGI305902, SGI305903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72311[] = {SGI305904, SGI0, SGI305905, SGI305906, SGI305907, SGI305908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305909, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72312[] = {SGI305910, SGI0, SGI0, SGI0, SGI305911, SGI305912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72313[] = {SGI305914, SGI0, SGI305915, SGI305916, SGI305917, SGI305918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72314[] = {SGI305919, SGI0, SGI305920, SGI305921, SGI305922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72315[] = {SGI305923, SGI0, SGI305924, SGI305925, SGI305926, SGI305927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72316[] = {SGI305928, SGI0, SGI305929, SGI305930, SGI305931, SGI305932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72317[] = {SGI305934, SGI0, SGI305935, SGI305936, SGI305937, SGI305938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72318[] = {SGI305939, SGI0, SGI305940, SGI305941, SGI305942, SGI305943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72319[] = {SGI305944, SGI0, SGI305945, SGI305946, SGI305947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72320[] = {SGI305948, SGI0, SGI305949, SGI305950, SGI305951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72321[] = {SGI305952, SGI0, SGI305953, SGI305954, SGI305955, SGI305956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72322[] = {SGI305957, SGI0, SGI305958, SGI305959, SGI305960, SGI305961, SGI0, SGI305962, SGI0, SGI0, SGI305963, SGI0, SGI0, SGI305964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72323[] = {SGI305965, SGI0, SGI305966, SGI305967, SGI305968, SGI305969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72324[] = {SGI305971, SGI0, SGI305972, SGI305973, SGI305974, SGI305975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305976, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72325[] = {SGI305977, SGI0, SGI305978, SGI305979, SGI305980, SGI305981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72326[] = {SGI305982, SGI0, SGI305983, SGI305984, SGI305985, SGI305986, SGI0, SGI305987, SGI0, SGI0, SGI305988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI305989, SGI0, SGI305990, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72327[] = {SGI305991, SGI0, SGI305992, SGI305993, SGI305994, SGI305995, SGI0, SGI305996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72328[] = {SGI305997, SGI0, SGI305998, SGI305999, SGI306000, SGI306001, SGI0, SGI306002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72329[] = {SGI306004, SGI0, SGI306005, SGI306006, SGI306007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72330[] = {SGI306008, SGI0, SGI306009, SGI306010, SGI306011, SGI306012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72331[] = {SGI306013, SGI0, SGI306014, SGI306015, SGI306016, SGI306017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72332[] = {SGI306018, SGI0, SGI306019, SGI306020, SGI306021, SGI306022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306023, SGI0, SGI0, NULL};
const Segment * const SG72333[] = {SGI306024, SGI0, SGI306025, SGI306026, SGI306027, SGI306028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306029, SGI0, SGI0, NULL};
const Segment * const SG72334[] = {SGI306030, SGI0, SGI306031, SGI306032, SGI306033, SGI306034, SGI0, SGI306035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306036, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72335[] = {SGI306037, SGI0, SGI306038, SGI306039, SGI306040, SGI306041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72336[] = {SGI306042, SGI0, SGI306043, SGI306044, SGI306045, SGI306046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72337[] = {SGI306048, SGI0, SGI306049, SGI306050, SGI306051, SGI306052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306054, SGI0, SGI0, NULL};
const Segment * const SG72338[] = {SGI306055, SGI0, SGI306056, SGI306057, SGI306058, SGI306059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72339[] = {SGI306061, SGI0, SGI306062, SGI306063, SGI306064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72340[] = {SGI306065, SGI0, SGI306066, SGI306067, SGI306068, SGI306069, SGI0, SGI306070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72341[] = {SGI306072, SGI0, SGI306073, SGI306074, SGI306075, SGI306076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306077, SGI0, NULL};
const Segment * const SG72342[] = {SGI306078, SGI0, SGI306079, SGI306080, SGI306081, SGI306082, SGI0, SGI306083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72343[] = {SGI306084, SGI0, SGI306085, SGI306086, SGI306087, SGI306088, SGI0, SGI306089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72344[] = {SGI306091, SGI0, SGI306092, SGI306093, SGI306094, SGI306095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306097, SGI0, SGI0, NULL};
const Segment * const SG72345[] = {SGI306098, SGI0, SGI306099, SGI306100, SGI306101, SGI306102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306103, SGI0, NULL};
const Segment * const SG72346[] = {SGI306104, SGI0, SGI306105, SGI306106, SGI306107, SGI306108, SGI0, SGI306109, SGI0, SGI0, SGI306110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72347[] = {SGI306111, SGI0, SGI306112, SGI306113, SGI306114, SGI306115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306116, SGI0, SGI0, NULL};
const Segment * const SG72348[] = {SGI306117, SGI0, SGI306118, SGI306119, SGI306120, SGI306121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72349[] = {SGI306122, SGI0, SGI306123, SGI306124, SGI306125, SGI306126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306127, SGI0, SGI0, NULL};
const Segment * const SG72350[] = {SGI306128, SGI0, SGI306129, SGI306130, SGI306131, SGI306132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72351[] = {SGI306134, SGI0, SGI306135, SGI306136, SGI306137, SGI306138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306139, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72352[] = {SGI306140, SGI0, SGI306141, SGI306142, SGI306143, SGI306144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72353[] = {SGI306145, SGI0, SGI0, SGI0, SGI306146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72354[] = {SGI306147, SGI0, SGI306148, SGI306149, SGI306150, SGI306151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306152, SGI0, SGI0, NULL};
const Segment * const SG72355[] = {SGI306153, SGI0, SGI306154, SGI306155, SGI306156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72356[] = {SGI306157, SGI0, SGI306158, SGI306159, SGI306160, SGI306161, SGI0, SGI306162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72357[] = {SGI306163, SGI0, SGI306164, SGI306165, SGI306166, SGI306167, SGI0, SGI0, SGI0, SGI0, SGI306168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72358[] = {SGI306169, SGI0, SGI306170, SGI306171, SGI306172, SGI306173, SGI0, SGI306174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72359[] = {SGI306176, SGI0, SGI306177, SGI306178, SGI306179, SGI306180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306182, SGI0, NULL};
const Segment * const SG72360[] = {SGI306183, SGI0, SGI306184, SGI306185, SGI306186, SGI306187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306188, SGI0, NULL};
const Segment * const SG72361[] = {SGI306189, SGI0, SGI306190, SGI306191, SGI306192, SGI306193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306195, SGI0, NULL};
const Segment * const SG72362[] = {SGI306196, SGI0, SGI306197, SGI306198, SGI306199, SGI306200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72363[] = {SGI306201, SGI0, SGI306202, SGI306203, SGI306204, SGI306205, SGI0, SGI306206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306207, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72364[] = {SGI306208, SGI0, SGI306209, SGI306210, SGI306211, SGI306212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72365[] = {SGI306214, SGI0, SGI306215, SGI306216, SGI306217, SGI306218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72366[] = {SGI306220, SGI0, SGI306221, SGI306222, SGI306223, SGI306224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306226, SGI0, SGI0, NULL};
const Segment * const SG72367[] = {SGI306227, SGI0, SGI306228, SGI306229, SGI306230, SGI306231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306233, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72368[] = {SGI306234, SGI0, SGI306235, SGI306236, SGI306237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72369[] = {SGI306238, SGI0, SGI306239, SGI306240, SGI306241, SGI306242, SGI0, SGI306243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306244, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72370[] = {SGI306245, SGI0, SGI306246, SGI306247, SGI306248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72371[] = {SGI306250, SGI0, SGI306251, SGI306252, SGI306253, SGI306254, SGI0, SGI0, SGI0, SGI0, SGI306255, SGI0, SGI0, SGI306256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72372[] = {SGI306257, SGI0, SGI306258, SGI306259, SGI306260, SGI306261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72373[] = {SGI306263, SGI0, SGI306264, SGI306265, SGI306266, SGI306267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72374[] = {SGI306268, SGI0, SGI306269, SGI306270, SGI306271, SGI306272, SGI0, SGI306273, SGI0, SGI0, SGI306274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72375[] = {SGI306275, SGI0, SGI306276, SGI306277, SGI306278, SGI306279, SGI0, SGI306280, SGI0, SGI0, SGI306281, SGI0, SGI0, SGI306282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72376[] = {SGI306283, SGI0, SGI306284, SGI306285, SGI306286, SGI306287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306288, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72377[] = {SGI306289, SGI0, SGI306290, SGI306291, SGI306292, SGI306293, SGI0, SGI0, SGI0, SGI0, SGI306294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72378[] = {SGI306295, SGI0, SGI306296, SGI306297, SGI306298, SGI306299, SGI0, SGI0, SGI0, SGI0, SGI306300, SGI0, SGI0, SGI306301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72379[] = {SGI306302, SGI0, SGI306303, SGI306304, SGI306305, SGI306306, SGI0, SGI306307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72380[] = {SGI306309, SGI0, SGI306310, SGI306311, SGI306312, SGI306313, SGI0, SGI306314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72381[] = {SGI306315, SGI0, SGI306316, SGI306317, SGI306318, SGI306319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306320, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72382[] = {SGI306321, SGI0, SGI306322, SGI306323, SGI306324, SGI306325, SGI0, SGI0, SGI0, SGI0, SGI306326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306327, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72383[] = {SGI306328, SGI0, SGI306329, SGI306330, SGI306331, SGI306332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72384[] = {SGI306334, SGI0, SGI306335, SGI306336, SGI306337, SGI306338, SGI0, SGI306339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306340, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72385[] = {SGI306341, SGI0, SGI306342, SGI306343, SGI306344, SGI306345, SGI0, SGI306346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72386[] = {SGI306348, SGI0, SGI306349, SGI306350, SGI306351, SGI306352, SGI0, SGI306353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306354, SGI0, SGI306355, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72387[] = {SGI306356, SGI0, SGI306357, SGI306358, SGI306359, SGI306360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72388[] = {SGI306362, SGI0, SGI306363, SGI306364, SGI306365, SGI306366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72389[] = {SGI306368, SGI0, SGI306369, SGI306370, SGI306371, SGI306372, SGI0, SGI306373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306374, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72390[] = {SGI306375, SGI0, SGI306376, SGI306377, SGI306378, SGI306379, SGI0, SGI306380, SGI0, SGI0, SGI306381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306382, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72391[] = {SGI306383, SGI0, SGI306384, SGI306385, SGI306386, SGI306387, SGI0, SGI306388, SGI0, SGI0, SGI306389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306390, SGI0, SGI306391, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72392[] = {SGI306392, SGI0, SGI306393, SGI306394, SGI306395, SGI306396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306398, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72393[] = {SGI306399, SGI0, SGI306400, SGI306401, SGI306402, SGI306403, SGI0, SGI0, SGI0, SGI0, SGI306404, SGI0, SGI0, SGI306405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306406, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72394[] = {SGI306407, SGI0, SGI306408, SGI306409, SGI306410, SGI306411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306412, SGI0, SGI306413, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72395[] = {SGI306414, SGI0, SGI306415, SGI306416, SGI306417, SGI306418, SGI0, SGI306419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306421, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72396[] = {SGI306422, SGI0, SGI306423, SGI306424, SGI306425, SGI306426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72397[] = {SGI306427, SGI0, SGI306428, SGI306429, SGI306430, SGI306431, SGI0, SGI306432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72398[] = {SGI306434, SGI0, SGI306435, SGI306436, SGI306437, SGI306438, SGI0, SGI306439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306440, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72399[] = {SGI306441, SGI0, SGI306442, SGI306443, SGI306444, SGI306445, SGI0, SGI306446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72400[] = {SGI306447, SGI0, SGI306448, SGI306449, SGI306450, SGI306451, SGI0, SGI306452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306453, SGI0, SGI306454, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72401[] = {SGI306455, SGI0, SGI306456, SGI306457, SGI306458, SGI306459, SGI0, SGI306460, SGI0, SGI0, SGI306461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306462, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72402[] = {SGI306463, SGI0, SGI306464, SGI306465, SGI306466, SGI306467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72403[] = {SGI306468, SGI0, SGI306469, SGI306470, SGI306471, SGI306472, SGI0, SGI0, SGI0, SGI0, SGI306473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306474, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72404[] = {SGI306475, SGI0, SGI306476, SGI306477, SGI306478, SGI306479, SGI0, SGI0, SGI0, SGI0, SGI306480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306481, SGI0, SGI306482, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72405[] = {SGI306483, SGI0, SGI306484, SGI306485, SGI306486, SGI306487, SGI0, SGI306488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306489, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72406[] = {SGI306490, SGI0, SGI306491, SGI306492, SGI306493, SGI306494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72407[] = {SGI306496, SGI0, SGI306497, SGI306498, SGI306499, SGI306500, SGI306501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72408[] = {SGI306503, SGI0, SGI306504, SGI306505, SGI306506, SGI306507, SGI306508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306510, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72409[] = {SGI306511, SGI0, SGI306512, SGI306513, SGI306514, SGI306515, SGI306516, SGI306517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72410[] = {SGI306519, SGI0, SGI306520, SGI306521, SGI306522, SGI306523, SGI306524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72411[] = {SGI306527, SGI0, SGI306528, SGI306529, SGI306530, SGI306531, SGI306532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72412[] = {SGI306534, SGI0, SGI306535, SGI306536, SGI306537, SGI306538, SGI306539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306541, SGI0, SGI0, NULL};
const Segment * const SG72413[] = {SGI306542, SGI0, SGI306543, SGI306544, SGI306545, SGI306546, SGI306547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72414[] = {SGI306549, SGI0, SGI306550, SGI306551, SGI306552, SGI306553, SGI306554, SGI306555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306557, SGI0, SGI0, NULL};
const Segment * const SG72415[] = {SGI306558, SGI0, SGI306559, SGI306560, SGI306561, SGI306562, SGI306563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72416[] = {SGI306565, SGI0, SGI306566, SGI306567, SGI306568, SGI306569, SGI306570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72417[] = {SGI306572, SGI0, SGI306573, SGI306574, SGI306575, SGI306576, SGI306577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306579, SGI0, SGI0, NULL};
const Segment * const SG72418[] = {SGI306580, SGI0, SGI306581, SGI306582, SGI306583, SGI306584, SGI306585, SGI306586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72419[] = {SGI306588, SGI306589, SGI306590, SGI306591, SGI306592, SGI306593, SGI306594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72420[] = {SGI306596, SGI0, SGI306597, SGI306598, SGI306599, SGI306600, SGI306601, SGI0, SGI0, SGI0, SGI306602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72421[] = {SGI306604, SGI0, SGI306605, SGI306606, SGI306607, SGI306608, SGI306609, SGI306610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306612, SGI0, SGI0, NULL};
const Segment * const SG72422[] = {SGI306613, SGI0, SGI306614, SGI306615, SGI306616, SGI306617, SGI306618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72423[] = {SGI306621, SGI0, SGI306622, SGI306623, SGI306624, SGI306625, SGI306626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306628, SGI0, SGI0, NULL};
const Segment * const SG72424[] = {SGI306629, SGI0, SGI306630, SGI306631, SGI306632, SGI306633, SGI306634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306636, SGI0, SGI0, NULL};
const Segment * const SG72425[] = {SGI306637, SGI0, SGI306638, SGI306639, SGI306640, SGI306641, SGI306642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306644, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72426[] = {SGI306645, SGI0, SGI306646, SGI306647, SGI306648, SGI306649, SGI306650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306652, SGI0, SGI0, NULL};
const Segment * const SG72427[] = {SGI306653, SGI0, SGI0, SGI0, SGI306654, SGI306655, SGI0, SGI306656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72428[] = {SGI306658, SGI0, SGI0, SGI0, SGI306659, SGI306660, SGI0, SGI306661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72429[] = {SGI306662, SGI0, SGI306663, SGI0, SGI306664, SGI306665, SGI0, SGI306666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306668, SGI0, SGI0, NULL};
const Segment * const SG72430[] = {SGI306669, SGI0, SGI0, SGI0, SGI306670, SGI306671, SGI0, SGI306672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306673, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72431[] = {SGI306674, SGI0, SGI0, SGI0, SGI306675, SGI306676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72432[] = {SGI306677, SGI0, SGI0, SGI0, SGI306678, SGI306679, SGI0, SGI306680, SGI0, SGI0, SGI306681, SGI0, SGI0, SGI306682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72433[] = {SGI306683, SGI0, SGI0, SGI0, SGI306684, SGI306685, SGI0, SGI306686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72434[] = {SGI306687, SGI0, SGI0, SGI0, SGI306688, SGI306689, SGI0, SGI306690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306691, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72435[] = {SGI306692, SGI0, SGI0, SGI0, SGI306693, SGI306694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72436[] = {SGI306696, SGI0, SGI0, SGI0, SGI306697, SGI306698, SGI0, SGI306699, SGI0, SGI0, SGI306700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72437[] = {SGI306701, SGI0, SGI306702, SGI0, SGI306703, SGI306704, SGI0, SGI306705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306706, SGI0, SGI0, NULL};
const Segment * const SG72438[] = {SGI306707, SGI0, SGI306708, SGI0, SGI306709, SGI306710, SGI0, SGI306711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306712, SGI306713, SGI0, SGI0, NULL};
const Segment * const SG72439[] = {SGI306714, SGI0, SGI0, SGI0, SGI306715, SGI306716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306717, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72440[] = {SGI306718, SGI0, SGI0, SGI0, SGI306719, SGI306720, SGI0, SGI306721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306722, SGI0, SGI0, SGI0, SGI306723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72441[] = {SGI306724, SGI0, SGI0, SGI0, SGI306725, SGI306726, SGI0, SGI306727, SGI0, SGI0, SGI306728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306729, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72442[] = {SGI306730, SGI0, SGI0, SGI0, SGI306731, SGI306732, SGI0, SGI0, SGI0, SGI0, SGI306733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72443[] = {SGI306734, SGI0, SGI0, SGI0, SGI306735, SGI306736, SGI0, SGI306737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72444[] = {SGI306739, SGI0, SGI0, SGI0, SGI306740, SGI306741, SGI0, SGI0, SGI0, SGI0, SGI306742, SGI0, SGI0, SGI306743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72445[] = {SGI306744, SGI0, SGI0, SGI0, SGI306745, SGI306746, SGI0, SGI306747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72446[] = {SGI306749, SGI0, SGI0, SGI0, SGI306750, SGI306751, SGI0, SGI306752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72447[] = {SGI306753, SGI0, SGI0, SGI0, SGI306754, SGI306755, SGI0, SGI306756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72448[] = {SGI306758, SGI0, SGI0, SGI0, SGI306759, SGI306760, SGI0, SGI306761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72449[] = {SGI306762, SGI0, SGI306763, SGI306764, SGI306765, SGI306766, SGI306767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72450[] = {SGI306769, SGI0, SGI306770, SGI306771, SGI306772, SGI306773, SGI306774, SGI306775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72451[] = {SGI306777, SGI0, SGI306778, SGI306779, SGI306780, SGI306781, SGI306782, SGI0, SGI0, SGI0, SGI306783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72452[] = {SGI306785, SGI0, SGI306786, SGI306787, SGI306788, SGI306789, SGI306790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72453[] = {SGI306792, SGI0, SGI306793, SGI306794, SGI306795, SGI306796, SGI306797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306799, SGI0, SGI0, NULL};
const Segment * const SG72454[] = {SGI306800, SGI0, SGI306801, SGI306802, SGI306803, SGI306804, SGI306805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72455[] = {SGI306807, SGI0, SGI306808, SGI306809, SGI306810, SGI306811, SGI306812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306814, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72456[] = {SGI306815, SGI0, SGI306816, SGI306817, SGI306818, SGI306819, SGI306820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306822, SGI0, SGI0, NULL};
const Segment * const SG72457[] = {SGI306823, SGI0, SGI306824, SGI306825, SGI306826, SGI306827, SGI306828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72458[] = {SGI306831, SGI0, SGI306832, SGI306833, SGI306834, SGI306835, SGI306836, SGI306837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306839, SGI0, SGI0, NULL};
const Segment * const SG72459[] = {SGI306840, SGI0, SGI306841, SGI306842, SGI306843, SGI306844, SGI306845, SGI306846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72460[] = {SGI306848, SGI0, SGI306849, SGI306850, SGI306851, SGI306852, SGI306853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72461[] = {SGI306855, SGI0, SGI306856, SGI306857, SGI306858, SGI306859, SGI306860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72462[] = {SGI306862, SGI0, SGI306863, SGI306864, SGI306865, SGI306866, SGI306867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306869, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72463[] = {SGI306870, SGI306871, SGI306872, SGI306873, SGI306874, SGI306875, SGI306876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72464[] = {SGI306878, SGI0, SGI306879, SGI306880, SGI306881, SGI306882, SGI306883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306885, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72465[] = {SGI306886, SGI0, SGI306887, SGI306888, SGI306889, SGI306890, SGI306891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72466[] = {SGI306893, SGI0, SGI306894, SGI306895, SGI306896, SGI306897, SGI306898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72467[] = {SGI306900, SGI0, SGI306901, SGI306902, SGI306903, SGI306904, SGI306905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72468[] = {SGI306907, SGI0, SGI306908, SGI306909, SGI306910, SGI306911, SGI306912, SGI0, SGI0, SGI0, SGI306913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72469[] = {SGI306915, SGI0, SGI306916, SGI306917, SGI306918, SGI306919, SGI306920, SGI306921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72470[] = {SGI306923, SGI0, SGI306924, SGI306925, SGI306926, SGI306927, SGI306928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72471[] = {SGI306930, SGI0, SGI306931, SGI306932, SGI306933, SGI306934, SGI306935, SGI306936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72472[] = {SGI306938, SGI0, SGI306939, SGI306940, SGI306941, SGI306942, SGI306943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72473[] = {SGI306946, SGI0, SGI306947, SGI306948, SGI306949, SGI306950, SGI306951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306953, SGI0, SGI0, NULL};
const Segment * const SG72474[] = {SGI306954, SGI0, SGI306955, SGI306956, SGI306957, SGI306958, SGI306959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72475[] = {SGI306961, SGI0, SGI306962, SGI306963, SGI306964, SGI306965, SGI306966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306968, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72476[] = {SGI306969, SGI0, SGI306970, SGI306971, SGI306972, SGI306973, SGI306974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306976, SGI0, SGI0, NULL};
const Segment * const SG72477[] = {SGI306977, SGI0, SGI306978, SGI306979, SGI306980, SGI306981, SGI306982, SGI306983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306985, SGI0, SGI0, NULL};
const Segment * const SG72478[] = {SGI306986, SGI0, SGI306987, SGI306988, SGI306989, SGI306990, SGI306991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI306993, SGI0, SGI0, NULL};
const Segment * const SG72479[] = {SGI306994, SGI0, SGI286350, SGI286351, SGI306995, SGI286352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72480[] = {SGI306996, SGI0, SGI306997, SGI306998, SGI306999, SGI307000, SGI307001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72481[] = {SGI307003, SGI0, SGI307004, SGI307005, SGI307006, SGI307007, SGI307008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72482[] = {SGI307010, SGI0, SGI307011, SGI307012, SGI307013, SGI307014, SGI307015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307017, SGI0, SGI0, NULL};
const Segment * const SG72483[] = {SGI307018, SGI0, SGI307019, SGI307020, SGI307021, SGI307022, SGI307023, SGI307024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72484[] = {SGI307026, SGI0, SGI307027, SGI307028, SGI307029, SGI307030, SGI307031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72485[] = {SGI307033, SGI0, SGI307034, SGI307035, SGI307036, SGI307037, SGI307038, SGI307039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72486[] = {SGI307041, SGI0, SGI307042, SGI307043, SGI307044, SGI307045, SGI307046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307048, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72487[] = {SGI307049, SGI0, SGI307050, SGI307051, SGI307052, SGI307053, SGI307054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72488[] = {SGI307056, SGI0, SGI307057, SGI307058, SGI307059, SGI307060, SGI307061, SGI307062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307064, SGI0, SGI0, NULL};
const Segment * const SG72489[] = {SGI307065, SGI0, SGI307066, SGI307067, SGI307068, SGI307069, SGI307070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72490[] = {SGI307073, SGI0, SGI307074, SGI307075, SGI307076, SGI307077, SGI307078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307080, SGI0, SGI0, NULL};
const Segment * const SG72491[] = {SGI307081, SGI0, SGI307082, SGI307083, SGI307084, SGI307085, SGI307086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307088, SGI0, SGI0, NULL};
const Segment * const SG72492[] = {SGI307089, SGI0, SGI307090, SGI307091, SGI307092, SGI307093, SGI307094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72493[] = {SGI307096, SGI0, SGI307097, SGI307098, SGI307099, SGI307100, SGI307101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307103, SGI0, SGI0, NULL};
const Segment * const SG72494[] = {SGI307104, SGI0, SGI307105, SGI307106, SGI307107, SGI307108, SGI307109, SGI0, SGI0, SGI0, SGI307110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72495[] = {SGI307112, SGI0, SGI307113, SGI307114, SGI307115, SGI307116, SGI307117, SGI307118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307120, SGI0, SGI0, NULL};
const Segment * const SG72496[] = {SGI307121, SGI0, SGI307122, SGI307123, SGI307124, SGI307125, SGI307126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307128, SGI0, SGI0, NULL};
const Segment * const SG72497[] = {SGI307129, SGI0, SGI307130, SGI307131, SGI307132, SGI307133, SGI307134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307136, SGI0, SGI0, NULL};
const Segment * const SG72498[] = {SGI307137, SGI0, SGI307138, SGI307139, SGI307140, SGI307141, SGI307142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72499[] = {SGI307144, SGI0, SGI307145, SGI307146, SGI307147, SGI307148, SGI307149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72500[] = {SGI307151, SGI0, SGI307152, SGI307153, SGI307154, SGI307155, SGI307156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72501[] = {SGI307158, SGI0, SGI307159, SGI307160, SGI307161, SGI307162, SGI307163, SGI307164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72502[] = {SGI307166, SGI0, SGI307167, SGI307168, SGI307169, SGI307170, SGI307171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72503[] = {SGI307174, SGI0, SGI307175, SGI307176, SGI307177, SGI307178, SGI307179, SGI307180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72504[] = {SGI307182, SGI0, SGI307183, SGI307184, SGI307185, SGI307186, SGI307187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307189, SGI0, SGI0, NULL};
const Segment * const SG72505[] = {SGI307190, SGI0, SGI307191, SGI307192, SGI307193, SGI307194, SGI307195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307197, SGI0, SGI0, NULL};
const Segment * const SG72506[] = {SGI307198, SGI0, SGI307199, SGI307200, SGI307201, SGI307202, SGI307203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307205, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72507[] = {SGI307206, SGI0, SGI307207, SGI307208, SGI307209, SGI307210, SGI307211, SGI307212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307214, SGI0, SGI0, NULL};
const Segment * const SG72508[] = {SGI307215, SGI0, SGI307216, SGI307217, SGI307218, SGI307219, SGI307220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72509[] = {SGI307222, SGI0, SGI307223, SGI307224, SGI307225, SGI307226, SGI307227, SGI0, SGI0, SGI0, SGI307228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72510[] = {SGI307230, SGI0, SGI307231, SGI307232, SGI307233, SGI307234, SGI307235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307237, SGI0, SGI0, NULL};
const Segment * const SG72511[] = {SGI307238, SGI0, SGI307239, SGI307240, SGI307241, SGI307242, SGI307243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72512[] = {SGI307245, SGI0, SGI307246, SGI307247, SGI307248, SGI307249, SGI307250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307252, SGI0, SGI0, NULL};
const Segment * const SG72513[] = {SGI307253, SGI0, SGI307254, SGI307255, SGI307256, SGI307257, SGI307258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307260, SGI0, SGI0, NULL};
const Segment * const SG72514[] = {SGI307261, SGI0, SGI307262, SGI307263, SGI307264, SGI307265, SGI307266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307269, SGI0, SGI0, NULL};
const Segment * const SG72515[] = {SGI307270, SGI0, SGI307271, SGI307272, SGI307273, SGI307274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307276, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72516[] = {SGI307277, SGI0, SGI307278, SGI307279, SGI307280, SGI307281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72517[] = {SGI307283, SGI0, SGI307284, SGI307285, SGI307286, SGI307287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307288, SGI307289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72518[] = {SGI307290, SGI0, SGI307291, SGI307292, SGI307293, SGI307294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307295, SGI307296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307297, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72519[] = {SGI307298, SGI0, SGI307299, SGI307300, SGI307301, SGI307302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307304, SGI0, SGI307305, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72520[] = {SGI307306, SGI0, SGI307307, SGI307308, SGI307309, SGI307310, SGI307311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72521[] = {SGI307313, SGI0, SGI307314, SGI307315, SGI307316, SGI307317, SGI307318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72522[] = {SGI307320, SGI0, SGI307321, SGI307322, SGI307323, SGI307324, SGI307325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307327, SGI0, SGI0, NULL};
const Segment * const SG72523[] = {SGI307328, SGI0, SGI307329, SGI307330, SGI307331, SGI307332, SGI307333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72524[] = {SGI307335, SGI0, SGI307336, SGI307337, SGI307338, SGI307339, SGI307340, SGI307341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72525[] = {SGI307343, SGI0, SGI307344, SGI307345, SGI307346, SGI307347, SGI307348, SGI307349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307351, SGI0, SGI0, NULL};
const Segment * const SG72526[] = {SGI307352, SGI0, SGI307353, SGI307354, SGI307355, SGI307356, SGI307357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307359, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72527[] = {SGI307360, SGI0, SGI307361, SGI307362, SGI307363, SGI307364, SGI307365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72528[] = {SGI307367, SGI0, SGI307368, SGI307369, SGI307370, SGI307371, SGI307372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307374, SGI0, SGI0, NULL};
const Segment * const SG72529[] = {SGI307375, SGI0, SGI307376, SGI307377, SGI307378, SGI307379, SGI307380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72530[] = {SGI307382, SGI0, SGI307383, SGI307384, SGI307385, SGI307386, SGI307387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72531[] = {SGI307390, SGI0, SGI307391, SGI307392, SGI307393, SGI307394, SGI307395, SGI307396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72532[] = {SGI307398, SGI0, SGI307399, SGI307400, SGI307401, SGI307402, SGI307403, SGI0, SGI0, SGI0, SGI307404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72533[] = {SGI307406, SGI0, SGI307407, SGI307408, SGI307409, SGI307410, SGI307411, SGI307412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72534[] = {SGI307414, SGI0, SGI307415, SGI307416, SGI307417, SGI307418, SGI307419, SGI307420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72535[] = {SGI307423, SGI0, SGI307424, SGI307425, SGI307426, SGI307427, SGI307428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72536[] = {SGI307431, SGI0, SGI307432, SGI307433, SGI307434, SGI307435, SGI307436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72537[] = {SGI307439, SGI0, SGI307440, SGI307441, SGI307442, SGI307443, SGI307444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72538[] = {SGI307446, SGI0, SGI307447, SGI307448, SGI307449, SGI307450, SGI307451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72539[] = {SGI307454, SGI0, SGI307455, SGI307456, SGI307457, SGI307458, SGI307459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72540[] = {SGI307461, SGI0, SGI307462, SGI307463, SGI307464, SGI307465, SGI307466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72541[] = {SGI307468, SGI0, SGI307469, SGI307470, SGI307471, SGI307472, SGI307473, SGI307474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72542[] = {SGI307476, SGI0, SGI307477, SGI307478, SGI307479, SGI307480, SGI307481, SGI307482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72543[] = {SGI307485, SGI0, SGI307486, SGI307487, SGI307488, SGI307489, SGI307490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72544[] = {SGI307493, SGI0, SGI307494, SGI307495, SGI307496, SGI307497, SGI307498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307500, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72545[] = {SGI307501, SGI0, SGI307502, SGI307503, SGI307504, SGI307505, SGI307506, SGI307507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307509, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72546[] = {SGI307510, SGI0, SGI307511, SGI307512, SGI307513, SGI307514, SGI307515, SGI307516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307518, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72547[] = {SGI307519, SGI0, SGI307520, SGI307521, SGI307522, SGI307523, SGI307524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72548[] = {SGI307526, SGI307527, SGI307528, SGI307529, SGI307530, SGI307531, SGI307532, SGI307533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72549[] = {SGI307536, SGI307537, SGI307538, SGI307539, SGI307540, SGI307541, SGI307542, SGI307543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72550[] = {SGI307546, SGI0, SGI307547, SGI307548, SGI307549, SGI307550, SGI307551, SGI307552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307553, SGI0, SGI0, SGI0, SGI307554, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72551[] = {SGI307555, SGI0, SGI307556, SGI307557, SGI307558, SGI307559, SGI307560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72552[] = {SGI307563, SGI307564, SGI307565, SGI307566, SGI307567, SGI307568, SGI307569, SGI307570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72553[] = {SGI307573, SGI0, SGI307574, SGI307575, SGI307576, SGI307577, SGI307578, SGI307579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72554[] = {SGI307581, SGI0, SGI307582, SGI307583, SGI307584, SGI307585, SGI307586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72555[] = {SGI307589, SGI0, SGI307590, SGI307591, SGI307592, SGI307593, SGI307594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72556[] = {SGI307597, SGI0, SGI307598, SGI307599, SGI307600, SGI307601, SGI307602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72557[] = {SGI307605, SGI307606, SGI307607, SGI307608, SGI307609, SGI307610, SGI307611, SGI307612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72558[] = {SGI307614, SGI0, SGI307615, SGI307616, SGI307617, SGI307618, SGI307619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72559[] = {SGI307621, SGI0, SGI307622, SGI307623, SGI307624, SGI307625, SGI307626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307628, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72560[] = {SGI307629, SGI0, SGI307630, SGI307631, SGI307632, SGI307633, SGI307634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72561[] = {SGI307637, SGI0, SGI307638, SGI307639, SGI307640, SGI307641, SGI307642, SGI307643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72562[] = {SGI307646, SGI0, SGI307647, SGI307648, SGI307649, SGI307650, SGI307651, SGI307652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72563[] = {SGI307654, SGI0, SGI307655, SGI307656, SGI307657, SGI307658, SGI307659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72564[] = {SGI307661, SGI0, SGI307662, SGI307663, SGI307664, SGI307665, SGI307666, SGI307667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72565[] = {SGI307670, SGI0, SGI307671, SGI307672, SGI307673, SGI307674, SGI307675, SGI307676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72566[] = {SGI307678, SGI0, SGI307679, SGI307680, SGI307681, SGI307682, SGI307683, SGI307684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307685, SGI0, SGI0, SGI0, SGI307686, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72567[] = {SGI307687, SGI0, SGI307688, SGI307689, SGI307690, SGI307691, SGI307692, SGI307693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307695, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72568[] = {SGI307696, SGI307697, SGI307698, SGI307699, SGI307700, SGI307701, SGI307702, SGI307703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72569[] = {SGI307706, SGI0, SGI307707, SGI307708, SGI307709, SGI307710, SGI307711, SGI307712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307714, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72570[] = {SGI307715, SGI0, SGI307716, SGI307717, SGI307718, SGI307719, SGI307720, SGI307721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72571[] = {SGI307723, SGI0, SGI307724, SGI307725, SGI307726, SGI307727, SGI307728, SGI307729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307730, SGI0, SGI0, SGI0, SGI307731, SGI0, SGI307732, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72572[] = {SGI307733, SGI0, SGI307734, SGI307735, SGI307736, SGI307737, SGI307738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72573[] = {SGI307740, SGI0, SGI307741, SGI307742, SGI307743, SGI307744, SGI307745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72574[] = {SGI307747, SGI0, SGI307748, SGI307749, SGI307750, SGI307751, SGI307752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307753, SGI0, SGI0, SGI0, SGI307754, SGI0, SGI307755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72575[] = {SGI307756, SGI307757, SGI307758, SGI307759, SGI307760, SGI307761, SGI307762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72576[] = {SGI307765, SGI0, SGI307766, SGI307767, SGI307768, SGI307769, SGI307770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307771, SGI0, SGI0, SGI0, SGI307772, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72577[] = {SGI307773, SGI307774, SGI307775, SGI307776, SGI307777, SGI307778, SGI307779, SGI307780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72578[] = {SGI307783, SGI0, SGI307784, SGI307785, SGI307786, SGI307787, SGI307788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72579[] = {SGI307791, SGI0, SGI286460, SGI286461, SGI307792, SGI286462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307794, SGI0, SGI0, NULL};
const Segment * const SG72580[] = {SGI307795, SGI0, SGI307796, SGI307797, SGI307798, SGI307799, SGI307800, SGI307801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72581[] = {SGI307804, SGI0, SGI307805, SGI307806, SGI307807, SGI307808, SGI307809, SGI307810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72582[] = {SGI307813, SGI0, SGI307814, SGI307815, SGI307816, SGI307817, SGI307818, SGI307819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307820, SGI0, SGI0, SGI0, SGI307821, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72583[] = {SGI307822, SGI0, SGI307823, SGI307824, SGI307825, SGI307826, SGI307827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72584[] = {SGI307829, SGI0, SGI307830, SGI307831, SGI307832, SGI307833, SGI307834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72585[] = {SGI307837, SGI0, SGI307838, SGI307839, SGI307840, SGI307841, SGI307842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72586[] = {SGI307845, SGI0, SGI307846, SGI307847, SGI307848, SGI307849, SGI307850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72587[] = {SGI307852, SGI0, SGI307853, SGI307854, SGI307855, SGI307856, SGI307857, SGI307858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72588[] = {SGI307860, SGI0, SGI307861, SGI307862, SGI307863, SGI307864, SGI307865, SGI307866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72589[] = {SGI307868, SGI0, SGI307869, SGI307870, SGI307871, SGI307872, SGI307873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72590[] = {SGI307876, SGI0, SGI307877, SGI307878, SGI307879, SGI307880, SGI307881, SGI307882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307884, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72591[] = {SGI307885, SGI0, SGI307886, SGI307887, SGI307888, SGI307889, SGI307890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72592[] = {SGI307892, SGI0, SGI307893, SGI307894, SGI307895, SGI307896, SGI307897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307899, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72593[] = {SGI307900, SGI0, SGI307901, SGI307902, SGI307903, SGI307904, SGI307905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72594[] = {SGI307908, SGI0, SGI307909, SGI307910, SGI307911, SGI307912, SGI307913, SGI307914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307916, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72595[] = {SGI307917, SGI0, SGI307918, SGI307919, SGI307920, SGI307921, SGI307922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72596[] = {SGI307924, SGI307925, SGI307926, SGI307927, SGI307928, SGI307929, SGI307930, SGI307931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72597[] = {SGI307934, SGI0, SGI307935, SGI307936, SGI307937, SGI307938, SGI307939, SGI307940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307941, SGI0, SGI0, SGI0, SGI307942, SGI0, SGI307943, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72598[] = {SGI307944, SGI307945, SGI307946, SGI307947, SGI307948, SGI307949, SGI307950, SGI307951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72599[] = {SGI307954, SGI0, SGI307955, SGI307956, SGI307957, SGI307958, SGI307959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72600[] = {SGI307962, SGI0, SGI307963, SGI307964, SGI307965, SGI307966, SGI307967, SGI307968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72601[] = {SGI307970, SGI0, SGI307971, SGI307972, SGI307973, SGI307974, SGI307975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307977, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72602[] = {SGI307978, SGI0, SGI307979, SGI307980, SGI307981, SGI307982, SGI307983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72603[] = {SGI307986, SGI0, SGI307987, SGI307988, SGI307989, SGI307990, SGI307991, SGI307992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI307994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72604[] = {SGI307995, SGI0, SGI307996, SGI307997, SGI307998, SGI307999, SGI308000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72605[] = {SGI308002, SGI0, SGI308003, SGI308004, SGI308005, SGI308006, SGI308007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72606[] = {SGI308009, SGI0, SGI308010, SGI308011, SGI308012, SGI308013, SGI308014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72607[] = {SGI308017, SGI0, SGI308018, SGI308019, SGI308020, SGI308021, SGI308022, SGI308023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72608[] = {SGI308025, SGI0, SGI308026, SGI308027, SGI308028, SGI308029, SGI308030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72609[] = {SGI308033, SGI0, SGI308034, SGI308035, SGI308036, SGI308037, SGI308038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72610[] = {SGI308040, SGI0, SGI308041, SGI308042, SGI308043, SGI308044, SGI308045, SGI308046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308048, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72611[] = {SGI308049, SGI0, SGI308050, SGI308051, SGI308052, SGI308053, SGI308054, SGI308055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308057, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72612[] = {SGI308058, SGI0, SGI308059, SGI308060, SGI308061, SGI308062, SGI308063, SGI308064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72613[] = {SGI308067, SGI0, SGI308068, SGI308069, SGI308070, SGI308071, SGI308072, SGI308073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308074, SGI0, SGI0, SGI0, SGI308075, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72614[] = {SGI308076, SGI0, SGI308077, SGI308078, SGI308079, SGI308080, SGI308081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72615[] = {SGI308083, SGI0, SGI308084, SGI308085, SGI308086, SGI308087, SGI308088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308089, SGI0, SGI0, SGI0, SGI308090, SGI0, SGI308091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72616[] = {SGI308092, SGI0, SGI308093, SGI308094, SGI308095, SGI308096, SGI308097, SGI308098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308099, SGI0, SGI0, SGI0, SGI308100, SGI0, SGI308101, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72617[] = {SGI308102, SGI308103, SGI308104, SGI308105, SGI308106, SGI308107, SGI308108, SGI308109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72618[] = {SGI308112, SGI0, SGI308113, SGI308114, SGI308115, SGI308116, SGI308117, SGI308118, SGI0, SGI0, SGI308119, SGI0, SGI0, SGI308120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72619[] = {SGI308122, SGI0, SGI308123, SGI308124, SGI308125, SGI308126, SGI308127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308128, SGI0, SGI0, SGI0, SGI308129, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72620[] = {SGI308130, SGI308131, SGI308132, SGI308133, SGI308134, SGI308135, SGI308136, SGI308137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72621[] = {SGI308139, SGI308140, SGI308141, SGI308142, SGI308143, SGI308144, SGI308145, SGI308146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72622[] = {SGI308148, SGI308149, SGI308150, SGI308151, SGI308152, SGI308153, SGI308154, SGI308155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72623[] = {SGI308158, SGI308159, SGI308160, SGI308161, SGI308162, SGI308163, SGI308164, SGI308165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72624[] = {SGI308167, SGI308168, SGI308169, SGI308170, SGI308171, SGI308172, SGI308173, SGI308174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72625[] = {SGI308177, SGI308178, SGI308179, SGI308180, SGI308181, SGI308182, SGI308183, SGI308184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72626[] = {SGI308187, SGI0, SGI308188, SGI308189, SGI308190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72627[] = {SGI308191, SGI0, SGI308192, SGI308193, SGI308194, SGI308195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308196, SGI0, SGI0, NULL};
const Segment * const SG72628[] = {SGI308197, SGI0, SGI308198, SGI308199, SGI308200, SGI308201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72629[] = {SGI308202, SGI0, SGI308203, SGI308204, SGI308205, SGI308206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72630[] = {SGI308207, SGI0, SGI308208, SGI308209, SGI308210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72631[] = {SGI308211, SGI0, SGI308212, SGI308213, SGI308214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72632[] = {SGI308215, SGI0, SGI308216, SGI308217, SGI308218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72633[] = {SGI308219, SGI0, SGI308220, SGI308221, SGI308222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72634[] = {SGI308223, SGI0, SGI308224, SGI308225, SGI308226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72635[] = {SGI308227, SGI0, SGI0, SGI0, SGI308228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72636[] = {SGI308229, SGI0, SGI308230, SGI308231, SGI308232, SGI308233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72637[] = {SGI308235, SGI0, SGI308236, SGI308237, SGI308238, SGI308239, SGI0, SGI308240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72638[] = {SGI308241, SGI0, SGI308242, SGI308243, SGI308244, SGI308245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72639[] = {SGI308246, SGI0, SGI308247, SGI308248, SGI308249, SGI308250, SGI0, SGI308251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72640[] = {SGI308253, NULL};
const Segment * const SG72641[] = {SGI308254, NULL};
const Segment * const SG72642[] = {SGI308255, NULL};
const Segment * const SG72643[] = {SGI308256, SGI0, SGI308257, SGI308258, SGI308259, SGI308260, SGI308261, SGI308262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308263, SGI0, SGI0, SGI0, SGI308264, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72644[] = {SGI308265, SGI0, SGI308266, SGI308267, SGI308268, SGI308269, SGI308270, SGI308271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72645[] = {SGI308274, SGI0, SGI308275, SGI308276, SGI308277, SGI308278, SGI308279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72646[] = {SGI308281, SGI0, SGI308282, SGI308283, SGI308284, SGI308285, SGI308286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72647[] = {SGI308289, SGI0, SGI308290, SGI308291, SGI308292, SGI308293, SGI308294, SGI308295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308297, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72648[] = {SGI308298, SGI0, SGI308299, SGI308300, SGI308301, SGI308302, SGI308303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72649[] = {SGI308306, SGI0, SGI308307, SGI308308, SGI308309, SGI308310, SGI308311, SGI308312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72650[] = {SGI308314, SGI0, SGI308315, SGI308316, SGI308317, SGI308318, SGI308319, SGI308320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72651[] = {SGI308322, SGI0, SGI308323, SGI308324, SGI308325, SGI308326, SGI308327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72652[] = {SGI308330, SGI0, SGI308331, SGI308332, SGI308333, SGI308334, SGI308335, SGI308336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308338, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72653[] = {SGI308339, SGI0, SGI308340, SGI308341, SGI308342, SGI308343, SGI308344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72654[] = {SGI308346, SGI0, SGI308347, SGI308348, SGI308349, SGI308350, SGI308351, SGI308352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72655[] = {SGI308355, SGI0, SGI308356, SGI308357, SGI308358, SGI308359, SGI308360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308361, SGI0, SGI0, SGI0, SGI308362, SGI0, SGI308363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72656[] = {SGI308364, SGI0, SGI308365, SGI308366, SGI308367, SGI308368, SGI308369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72657[] = {SGI308372, SGI0, SGI308373, SGI308374, SGI308375, SGI308376, SGI308377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308379, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72658[] = {SGI308380, SGI0, SGI308381, SGI308382, SGI308383, SGI308384, SGI308385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72659[] = {SGI308387, SGI0, SGI308388, SGI308389, SGI308390, SGI308391, SGI308392, SGI308393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72660[] = {SGI308396, SGI0, SGI308397, SGI308398, SGI308399, SGI308400, SGI308401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308402, SGI0, SGI308403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72661[] = {SGI308404, SGI0, SGI308405, SGI308406, SGI308407, SGI308408, SGI308409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72662[] = {SGI308411, SGI308412, SGI308413, SGI308414, SGI308415, SGI308416, SGI308417, SGI308418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72663[] = {SGI308421, SGI0, SGI308422, SGI308423, SGI308424, SGI308425, SGI308426, SGI308427, SGI0, SGI0, SGI308428, SGI0, SGI0, SGI308429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72664[] = {SGI308431, SGI308432, SGI308433, SGI308434, SGI308435, SGI308436, SGI308437, SGI308438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72665[] = {SGI308441, NULL};
const Segment * const SG72666[] = {SGI308442, SGI0, SGI308443, SGI308444, SGI308445, SGI308446, SGI308447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72667[] = {SGI308449, SGI0, SGI308450, SGI308451, SGI308452, SGI308453, SGI308454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72668[] = {SGI308456, SGI0, SGI308457, SGI308458, SGI308459, SGI308460, SGI308461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308463, SGI0, SGI0, NULL};
const Segment * const SG72669[] = {SGI308464, SGI0, SGI308465, SGI308466, SGI308467, SGI308468, SGI308469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72670[] = {SGI308471, SGI0, SGI308472, SGI308473, SGI308474, SGI308475, SGI308476, SGI308477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72671[] = {SGI308479, SGI0, SGI308480, SGI308481, SGI308482, SGI308483, SGI308484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308486, SGI0, SGI0, NULL};
const Segment * const SG72672[] = {SGI308487, SGI0, SGI308488, SGI308489, SGI308490, SGI308491, SGI308492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308494, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72673[] = {SGI308495, SGI0, SGI308496, SGI308497, SGI308498, SGI308499, SGI308500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308502, SGI0, SGI0, NULL};
const Segment * const SG72674[] = {SGI308503, SGI0, SGI308504, SGI308505, SGI308506, SGI308507, SGI308508, SGI308509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72675[] = {SGI308511, SGI0, SGI308512, SGI308513, SGI308514, SGI308515, SGI308516, SGI308517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308519, SGI0, SGI0, NULL};
const Segment * const SG72676[] = {SGI308520, SGI0, SGI308521, SGI308522, SGI308523, SGI308524, SGI308525, SGI0, SGI0, SGI0, SGI308526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72677[] = {SGI308528, SGI0, SGI308529, SGI308530, SGI308531, SGI308532, SGI308533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72678[] = {SGI308536, SGI0, SGI308537, SGI308538, SGI308539, SGI308540, SGI308541, SGI308542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308544, SGI0, SGI0, NULL};
const Segment * const SG72679[] = {SGI308545, SGI0, SGI308546, SGI308547, SGI308548, SGI308549, SGI308550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72680[] = {SGI308552, SGI0, SGI308553, SGI308554, SGI308555, SGI308556, SGI308557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72681[] = {SGI308559, SGI308560, SGI308561, SGI308562, SGI308563, SGI308564, SGI308565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72682[] = {SGI308567, SGI0, SGI308568, SGI308569, SGI308570, SGI308571, SGI308572, SGI308573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72683[] = {SGI308576, SGI0, SGI308577, SGI308578, SGI308579, SGI308580, SGI308581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72684[] = {SGI308583, SGI0, SGI308584, SGI308585, SGI308586, SGI308587, SGI308588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72685[] = {SGI308590, SGI0, SGI308591, SGI308592, SGI308593, SGI308594, SGI308595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72686[] = {SGI308598, SGI0, SGI308599, SGI308600, SGI308601, SGI308602, SGI308603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72687[] = {SGI308606, SGI0, SGI308607, SGI308608, SGI308609, SGI308610, SGI308611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72688[] = {SGI308614, SGI0, SGI308615, SGI308616, SGI308617, SGI308618, SGI308619, SGI308620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308622, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72689[] = {SGI308623, SGI0, SGI308624, SGI308625, SGI308626, SGI308627, SGI308628, SGI308629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72690[] = {SGI308631, SGI0, SGI308632, SGI308633, SGI308634, SGI308635, SGI308636, SGI308637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72691[] = {SGI308640, SGI0, SGI308641, SGI308642, SGI308643, SGI308644, SGI308645, SGI308646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72692[] = {SGI308648, SGI0, SGI308649, SGI308650, SGI308651, SGI308652, SGI308653, SGI308654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308655, SGI0, SGI0, SGI0, SGI308656, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72693[] = {SGI308657, SGI0, SGI308658, SGI308659, SGI308660, SGI308661, SGI308662, SGI308663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308665, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72694[] = {SGI308666, SGI0, SGI308667, SGI308668, SGI308669, SGI308670, SGI308671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72695[] = {SGI308673, SGI0, SGI308674, SGI308675, SGI308676, SGI308677, SGI308678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72696[] = {SGI308681, SGI0, SGI308682, SGI308683, SGI308684, SGI308685, SGI308686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308688, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72697[] = {SGI308689, SGI0, SGI308690, SGI308691, SGI308692, SGI308693, SGI308694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72698[] = {SGI308696, SGI308697, SGI308698, SGI308699, SGI308700, SGI308701, SGI308702, SGI308703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72699[] = {SGI308706, SGI0, SGI308707, SGI308708, SGI308709, SGI308710, SGI308711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308713, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72700[] = {SGI308714, SGI308715, SGI308716, SGI308717, SGI308718, SGI308719, SGI308720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72701[] = {SGI308722, SGI308723, SGI308724, SGI308725, SGI308726, SGI308727, SGI308728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308730, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72702[] = {SGI308731, SGI0, SGI308732, SGI308733, SGI308734, SGI308735, SGI308736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72703[] = {SGI308738, SGI0, SGI308739, SGI308740, SGI308741, SGI308742, SGI308743, SGI308744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72704[] = {SGI308746, SGI0, SGI308747, SGI308748, SGI308749, SGI308750, SGI308751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72705[] = {SGI308754, SGI0, SGI308755, SGI308756, SGI308757, SGI308758, SGI308759, SGI308760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308762, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72706[] = {SGI308763, SGI0, SGI308764, SGI308765, SGI308766, SGI308767, SGI308768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72707[] = {SGI308770, SGI0, SGI308771, SGI308772, SGI308773, SGI308774, SGI308775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72708[] = {SGI308778, SGI0, SGI308779, SGI308780, SGI308781, SGI308782, SGI308783, SGI308784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72709[] = {SGI308787, SGI0, SGI308788, SGI308789, SGI308790, SGI308791, SGI308792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72710[] = {SGI308794, SGI0, SGI308795, SGI308796, SGI308797, SGI308798, SGI308799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72711[] = {SGI308802, SGI0, SGI308803, SGI308804, SGI308805, SGI308806, SGI308807, SGI308808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72712[] = {SGI308811, SGI0, SGI308812, SGI308813, SGI308814, SGI308815, SGI308816, SGI308817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72713[] = {SGI308819, SGI0, SGI308820, SGI308821, SGI308822, SGI308823, SGI308824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72714[] = {SGI308827, SGI0, SGI308828, SGI308829, SGI308830, SGI308831, SGI308832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308834, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72715[] = {SGI308835, SGI0, SGI308836, SGI308837, SGI308838, SGI308839, SGI308840, SGI308841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308843, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72716[] = {SGI308844, SGI0, SGI308845, SGI308846, SGI308847, SGI308848, SGI308849, SGI308850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308851, SGI0, SGI0, SGI0, SGI308852, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72717[] = {SGI308853, SGI0, SGI308854, SGI308855, SGI308856, SGI308857, SGI308858, SGI308859, SGI0, SGI0, SGI308860, SGI0, SGI0, SGI308861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72718[] = {SGI308863, SGI308864, SGI308865, SGI308866, SGI308867, SGI308868, SGI308869, SGI308870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72719[] = {SGI308873, SGI0, SGI308874, SGI308875, SGI308876, SGI308877, SGI308878, SGI308879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308881, SGI0, SGI0, NULL};
const Segment * const SG72720[] = {SGI308882, SGI0, SGI308883, SGI308884, SGI308885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72721[] = {SGI308886, SGI0, SGI308887, SGI308888, SGI308889, SGI308890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72722[] = {SGI308891, SGI0, SGI308892, SGI308893, SGI308894, SGI308895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72723[] = {SGI308896, SGI0, SGI308897, SGI308898, SGI308899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72724[] = {SGI308900, SGI0, SGI308901, SGI308902, SGI308903, SGI308904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72725[] = {SGI308906, SGI0, SGI308907, SGI308908, SGI308909, SGI308910, SGI0, SGI308911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72726[] = {SGI308912, SGI0, SGI308913, SGI308914, SGI308915, SGI308916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72727[] = {SGI308918, SGI0, SGI308919, SGI308920, SGI308921, SGI308922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308923, SGI0, SGI0, NULL};
const Segment * const SG72728[] = {SGI308924, SGI0, SGI308925, SGI308926, SGI308927, SGI308928, SGI0, SGI308929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72729[] = {SGI308931, SGI0, SGI308932, SGI308933, SGI308934, SGI308935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308936, SGI0, SGI0, NULL};
const Segment * const SG72730[] = {SGI308937, SGI0, SGI308938, SGI308939, SGI308940, SGI308941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308942, SGI0, SGI0, NULL};
const Segment * const SG72731[] = {SGI308943, SGI0, SGI308944, SGI308945, SGI308946, SGI308947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308948, SGI0, SGI0, NULL};
const Segment * const SG72732[] = {SGI308949, SGI0, SGI308950, SGI308951, SGI308952, SGI308953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72733[] = {SGI308954, SGI0, SGI308955, SGI308956, SGI308957, SGI308958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308959, SGI0, SGI0, NULL};
const Segment * const SG72734[] = {SGI308960, SGI0, SGI308961, SGI308962, SGI308963, SGI308964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308965, SGI0, SGI0, NULL};
const Segment * const SG72735[] = {SGI308966, SGI0, SGI308967, SGI308968, SGI308969, SGI308970, SGI0, SGI308971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72736[] = {SGI308972, SGI0, SGI308973, SGI308974, SGI308975, SGI308976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72737[] = {SGI308977, SGI0, SGI308978, SGI308979, SGI308980, SGI308981, SGI0, SGI308982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308983, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72738[] = {SGI308984, SGI0, SGI308985, SGI308986, SGI308987, SGI308988, SGI0, SGI308989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72739[] = {SGI308990, SGI0, SGI308991, SGI308992, SGI308993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72740[] = {SGI308994, SGI0, SGI308995, SGI308996, SGI308997, SGI308998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI308999, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72741[] = {SGI309000, SGI0, SGI309001, SGI309002, SGI309003, SGI309002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72742[] = {SGI309004, SGI0, SGI309005, SGI309006, SGI309007, SGI309008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72743[] = {SGI309009, SGI0, SGI309010, SGI309011, SGI309012, SGI309013, SGI0, SGI0, SGI0, SGI0, SGI309014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72744[] = {SGI309015, SGI0, SGI309016, SGI309017, SGI309018, SGI309019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309020, SGI0, SGI0, NULL};
const Segment * const SG72745[] = {SGI309021, SGI0, SGI309022, SGI309023, SGI309024, SGI309025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309026, SGI0, SGI0, NULL};
const Segment * const SG72746[] = {SGI309027, SGI0, SGI309028, SGI309029, SGI309030, SGI309031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309032, SGI0, SGI0, NULL};
const Segment * const SG72747[] = {SGI309033, SGI0, SGI309034, SGI309035, SGI309036, SGI309037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309038, SGI0, SGI0, NULL};
const Segment * const SG72748[] = {SGI309039, SGI0, SGI309040, SGI309041, SGI309042, SGI309043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309044, SGI0, SGI0, NULL};
const Segment * const SG72749[] = {SGI309045, SGI0, SGI309046, SGI309047, SGI309048, SGI309049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309050, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72750[] = {SGI309051, SGI309052, SGI309053, SGI309054, SGI309055, SGI309056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72751[] = {SGI309057, SGI309058, SGI309059, SGI309060, SGI309061, SGI309062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72752[] = {SGI309063, SGI309064, SGI309065, SGI309066, SGI309067, SGI309068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72753[] = {SGI309069, SGI309070, SGI309071, SGI309072, SGI309073, SGI309074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72754[] = {SGI309075, SGI309076, SGI309077, SGI309078, SGI309079, SGI309080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72755[] = {SGI309082, SGI309083, SGI309084, SGI309085, SGI309086, SGI309087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72756[] = {SGI309089, SGI309090, SGI309091, SGI309092, SGI309093, SGI309094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72757[] = {SGI309095, SGI309096, SGI309097, SGI309098, SGI309099, SGI309100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72758[] = {SGI309101, SGI309102, SGI309103, SGI309104, SGI309105, SGI309106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72759[] = {SGI309107, SGI309108, SGI309109, SGI309110, SGI309111, SGI309112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72760[] = {SGI309113, SGI309114, SGI309115, SGI309116, SGI309117, SGI309118, SGI0, SGI309119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72761[] = {SGI309120, SGI309121, SGI309122, SGI309123, SGI309124, SGI309125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72762[] = {SGI309126, SGI309127, SGI309128, SGI309129, SGI309130, SGI309131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72763[] = {SGI309133, SGI309134, SGI309135, SGI309136, SGI309137, SGI309138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72764[] = {SGI309140, SGI309141, SGI309142, SGI309143, SGI309144, SGI309145, SGI0, SGI309146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72765[] = {SGI309147, SGI309148, SGI309149, SGI309150, SGI309151, SGI309152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72766[] = {SGI309153, SGI309154, SGI309155, SGI309156, SGI309157, SGI309158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309159, SGI0, SGI0, NULL};
const Segment * const SG72767[] = {SGI309160, SGI309161, SGI309162, SGI309163, SGI309164, SGI309165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72768[] = {SGI309166, SGI309167, SGI309168, SGI309169, SGI309170, SGI309171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72769[] = {SGI309172, SGI309173, SGI309174, SGI309175, SGI309176, SGI309177, SGI0, SGI309178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72770[] = {SGI309179, SGI309180, SGI309181, SGI309182, SGI309183, SGI309184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72771[] = {SGI309186, SGI309187, SGI309188, SGI309189, SGI309190, SGI309191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72772[] = {SGI309192, SGI309193, SGI309194, SGI309195, SGI309196, SGI309197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72773[] = {SGI309198, SGI309199, SGI309200, SGI309201, SGI309202, SGI309203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72774[] = {SGI309204, SGI309205, SGI309206, SGI309207, SGI309208, SGI309209, SGI0, SGI309210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72775[] = {SGI309211, SGI309212, SGI309213, SGI309214, SGI309215, SGI309216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72776[] = {SGI309218, SGI309219, SGI309220, SGI309221, SGI309222, SGI309223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309224, SGI0, SGI0, NULL};
const Segment * const SG72777[] = {SGI309225, SGI309226, SGI309227, SGI309228, SGI309229, SGI309230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72778[] = {SGI309231, SGI309232, SGI309233, SGI309234, SGI309235, SGI309236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309237, SGI0, SGI0, NULL};
const Segment * const SG72779[] = {SGI309238, SGI309239, SGI309240, SGI309241, SGI309242, SGI309243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72780[] = {SGI309244, SGI309245, SGI309246, SGI309247, SGI309248, SGI309249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72781[] = {SGI309251, SGI309252, SGI309253, SGI309254, SGI309255, SGI309256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72782[] = {SGI309257, SGI309258, SGI309259, SGI309260, SGI309261, SGI309262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72783[] = {SGI309263, SGI309264, SGI309265, SGI309266, SGI309267, SGI309268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72784[] = {SGI309269, SGI309270, SGI309271, SGI309272, SGI309273, SGI309274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72785[] = {SGI309275, SGI0, SGI309276, SGI309277, SGI309278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72786[] = {SGI309279, SGI0, SGI309280, SGI309281, SGI309282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72787[] = {SGI309283, SGI0, SGI309284, SGI309285, SGI309286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72788[] = {SGI309287, SGI0, SGI309288, SGI309289, SGI309290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72789[] = {SGI309291, SGI0, SGI309292, SGI309293, SGI309294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72790[] = {SGI309295, SGI0, SGI309296, SGI309297, SGI309298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72791[] = {SGI309299, SGI0, SGI309300, SGI309301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72792[] = {SGI309302, SGI0, SGI309303, SGI309304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309305, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72793[] = {SGI309306, SGI0, SGI309307, SGI309308, SGI309309, SGI309310, SGI0, SGI0, SGI0, SGI0, SGI309311, SGI0, SGI309312, SGI309313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309314, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72794[] = {SGI309315, SGI0, SGI309316, SGI309317, SGI309318, SGI309319, SGI0, SGI309320, SGI0, SGI0, SGI0, SGI0, SGI309321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309322, SGI0, SGI309323, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72795[] = {SGI309324, SGI0, SGI309325, SGI309326, SGI309327, SGI309328, SGI0, SGI309329, SGI0, SGI0, SGI0, SGI0, SGI309330, SGI309331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309332, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72796[] = {SGI309333, SGI0, SGI309334, SGI309335, SGI309336, SGI309337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309338, SGI309339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309340, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72797[] = {SGI309341, SGI0, SGI309342, SGI309343, SGI309344, SGI309345, SGI0, SGI0, SGI0, SGI0, SGI309346, SGI0, SGI309347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309348, SGI0, SGI309349, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72798[] = {SGI309350, SGI0, SGI309351, SGI309352, SGI309353, SGI309354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309355, SGI309356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72799[] = {SGI309357, SGI0, SGI309358, SGI309359, SGI309360, SGI309361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309363, SGI0, SGI309364, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72800[] = {SGI309365, SGI0, SGI309366, SGI309367, SGI309368, SGI309369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309371, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72801[] = {SGI309372, SGI0, SGI309373, SGI309374, SGI309375, SGI309376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309378, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72802[] = {SGI309379, SGI0, SGI309380, SGI309381, SGI309382, SGI309383, SGI0, SGI309384, SGI0, SGI0, SGI309385, SGI0, SGI309386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309387, SGI0, SGI309388, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72803[] = {SGI309389, SGI0, SGI309390, SGI309391, SGI309392, SGI309393, SGI0, SGI0, SGI0, SGI0, SGI309394, SGI0, SGI309395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309396, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72804[] = {SGI309397, SGI0, SGI309398, SGI309399, SGI309400, SGI309401, SGI0, SGI309402, SGI0, SGI0, SGI0, SGI0, SGI309403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309404, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72805[] = {SGI309405, SGI0, SGI309406, SGI309407, SGI309408, SGI309409, SGI0, SGI309410, SGI0, SGI0, SGI309411, SGI0, SGI309412, SGI309413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309414, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72806[] = {SGI309415, SGI0, SGI309416, SGI309417, SGI309418, SGI309419, SGI309420, SGI309421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72807[] = {SGI309424, SGI0, SGI309425, SGI309426, SGI309427, SGI309428, SGI309429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72808[] = {SGI309431, SGI0, SGI309432, SGI309433, SGI309434, SGI309435, SGI309436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72809[] = {SGI309439, SGI0, SGI309440, SGI309441, SGI309442, SGI309443, SGI309444, SGI309445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309447, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72810[] = {SGI309448, SGI0, SGI309449, SGI309450, SGI309451, SGI309452, SGI309453, SGI309454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72811[] = {SGI309456, SGI0, SGI309457, SGI309458, SGI309459, SGI309460, SGI309461, SGI309462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72812[] = {SGI309465, SGI0, SGI309466, SGI309467, SGI309468, SGI309469, SGI309470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72813[] = {SGI309473, SGI0, SGI309474, SGI309475, SGI309476, SGI309477, SGI309478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309480, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72814[] = {SGI309481, SGI0, SGI309482, SGI309483, SGI309484, SGI309485, SGI309486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72815[] = {SGI309488, SGI0, SGI309489, SGI309490, SGI309491, SGI309492, SGI309493, SGI309494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72816[] = {SGI309496, SGI0, SGI309497, SGI309498, SGI309499, SGI309500, SGI309501, SGI309502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309503, SGI0, SGI0, SGI0, SGI309504, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72817[] = {SGI309505, SGI0, SGI309506, SGI309507, SGI309508, SGI309509, SGI309510, SGI309511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309513, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72818[] = {SGI309514, SGI0, SGI309515, SGI309516, SGI309517, SGI309518, SGI309519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72819[] = {SGI309521, SGI0, SGI309522, SGI309523, SGI309524, SGI309525, SGI309526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309527, SGI0, SGI0, SGI0, SGI309528, SGI0, SGI309529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72820[] = {SGI309530, SGI0, SGI309531, SGI309532, SGI309533, SGI309534, SGI309535, SGI309536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309537, SGI0, SGI0, SGI0, SGI309538, SGI0, SGI309539, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72821[] = {SGI309540, SGI0, SGI309541, SGI309542, SGI309543, SGI309544, SGI309545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72822[] = {SGI309548, SGI0, SGI309549, SGI309550, SGI309551, SGI309552, SGI309553, SGI309554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72823[] = {SGI309557, SGI0, SGI309558, SGI309559, SGI309560, SGI309561, SGI309562, SGI309563, SGI0, SGI0, SGI309564, SGI0, SGI0, SGI309565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72824[] = {SGI309567, SGI0, SGI309568, SGI309569, SGI309570, SGI309571, SGI309572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72825[] = {SGI309575, SGI0, SGI309576, SGI309577, SGI309578, SGI309579, SGI309580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72826[] = {SGI309582, SGI0, SGI309583, SGI309584, SGI309585, SGI309586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72827[] = {SGI309587, SGI0, SGI309588, SGI309589, SGI309590, SGI309591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309592, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72828[] = {SGI309593, SGI0, SGI309594, SGI309595, SGI309596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72829[] = {SGI309597, SGI0, SGI309598, SGI309599, SGI309600, SGI309601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72830[] = {SGI309603, SGI0, SGI309604, SGI309605, SGI309606, SGI309607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72831[] = {SGI309608, SGI0, SGI309609, SGI309610, SGI309611, SGI309612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72832[] = {SGI309613, SGI0, SGI309614, SGI309615, SGI309616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72833[] = {SGI309617, SGI0, SGI309618, SGI309619, SGI309620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72834[] = {SGI309621, SGI0, SGI309622, SGI309623, SGI309624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72835[] = {SGI309625, SGI0, SGI309626, SGI309627, SGI309628, SGI309629, SGI0, SGI0, SGI0, SGI0, SGI309630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72836[] = {SGI309631, SGI0, SGI309632, SGI309633, SGI309634, SGI309635, SGI0, SGI309636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72837[] = {SGI309638, SGI0, SGI309639, SGI309640, SGI309641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72838[] = {SGI309642, SGI0, SGI309643, SGI309644, SGI309645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72839[] = {SGI309646, SGI0, SGI309647, SGI309648, SGI309649, SGI309650, SGI0, SGI309651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309652, SGI0, SGI309653, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72840[] = {SGI309654, SGI0, SGI309655, SGI309656, SGI309657, SGI309658, SGI0, SGI309659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309660, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72841[] = {SGI309661, SGI0, SGI309662, SGI309663, SGI309664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72842[] = {SGI309665, SGI0, SGI309666, SGI309667, SGI309668, SGI309669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309671, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72843[] = {SGI309672, SGI0, SGI309673, SGI309674, SGI309675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72844[] = {SGI309676, SGI0, SGI309677, SGI309678, SGI309679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72845[] = {SGI309680, SGI0, SGI309681, SGI309682, SGI309683, SGI309684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72846[] = {SGI309686, SGI0, SGI309687, SGI309688, SGI309689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72847[] = {SGI309690, SGI0, SGI309691, SGI309692, SGI309693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72848[] = {SGI309694, SGI0, SGI309695, SGI309696, SGI309697, SGI309698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72849[] = {SGI309699, SGI0, SGI309700, SGI309701, SGI309702, SGI309703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309704, SGI0, SGI309705, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72850[] = {SGI309706, SGI0, SGI309707, SGI309708, SGI309709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72851[] = {SGI309710, SGI0, SGI309711, SGI309712, SGI309713, SGI309714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72852[] = {SGI309715, SGI0, SGI309716, SGI309717, SGI309718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72853[] = {SGI309719, SGI0, SGI309720, SGI309721, SGI309722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72854[] = {SGI309723, SGI0, SGI309724, SGI309725, SGI309726, SGI309727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72855[] = {SGI309729, SGI0, SGI309730, SGI309731, SGI309732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72856[] = {SGI309733, SGI309734, SGI309735, SGI309736, SGI309737, SGI309738, SGI0, SGI0, SGI0, SGI0, SGI309739, SGI0, SGI0, SGI309740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72857[] = {SGI309741, SGI309742, SGI309743, SGI309744, SGI309745, SGI309746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72858[] = {SGI309748, SGI309749, SGI309750, SGI309751, SGI309752, SGI309753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309754, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72859[] = {SGI309755, SGI309756, SGI309757, SGI309758, SGI309759, SGI309760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72860[] = {SGI309762, SGI309763, SGI309764, SGI309765, SGI309766, SGI309767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309769, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72861[] = {SGI309770, SGI309771, SGI309772, SGI309773, SGI309774, SGI309775, SGI0, SGI0, SGI0, SGI0, SGI309776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309777, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72862[] = {SGI309778, SGI309779, SGI309780, SGI309781, SGI309782, SGI309783, SGI0, SGI0, SGI0, SGI0, SGI309784, SGI0, SGI0, SGI309785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309786, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72863[] = {SGI309787, SGI309788, SGI309789, SGI309790, SGI309791, SGI309792, SGI0, SGI0, SGI0, SGI0, SGI309793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309794, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72864[] = {SGI309795, SGI309796, SGI309797, SGI309798, SGI309799, SGI309800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309801, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72865[] = {SGI309802, SGI309803, SGI309804, SGI309805, SGI309806, SGI309807, SGI0, SGI0, SGI0, SGI0, SGI309808, SGI0, SGI0, SGI309809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309810, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72866[] = {SGI309811, SGI309812, SGI309813, SGI309814, SGI309815, SGI309816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72867[] = {SGI309818, SGI309819, SGI309820, SGI309821, SGI309822, SGI309823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309824, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72868[] = {SGI309825, SGI309826, SGI309827, SGI309828, SGI309829, SGI309830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72869[] = {SGI309832, SGI309833, SGI309834, SGI309835, SGI309836, SGI309837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309839, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72870[] = {SGI309840, SGI309841, SGI309842, SGI309843, SGI309844, SGI309845, SGI0, SGI0, SGI0, SGI0, SGI309846, SGI0, SGI0, SGI309847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72871[] = {SGI309848, SGI309849, SGI309850, SGI309851, SGI309852, SGI309853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309854, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72872[] = {SGI309855, SGI309856, SGI309857, SGI309858, SGI309859, SGI309860, SGI0, SGI0, SGI0, SGI0, SGI309861, SGI0, SGI0, SGI309862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72873[] = {SGI309863, SGI309864, SGI309865, SGI309866, SGI309867, SGI309868, SGI0, SGI0, SGI0, SGI0, SGI309869, SGI0, SGI0, SGI309870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72874[] = {SGI309871, SGI309872, SGI309873, SGI309874, SGI309875, SGI309876, SGI0, SGI0, SGI0, SGI0, SGI309877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309878, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72875[] = {SGI309879, SGI309880, SGI309881, SGI309882, SGI309883, SGI309884, SGI0, SGI309885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72876[] = {SGI309887, SGI309888, SGI309889, SGI309890, SGI309891, SGI309892, SGI0, SGI309893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309894, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72877[] = {SGI309895, SGI309896, SGI309897, SGI309898, SGI309899, SGI309900, SGI0, SGI309901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309902, SGI0, SGI309903, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72878[] = {SGI309904, SGI309905, SGI309906, SGI309907, SGI309908, SGI309909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72879[] = {SGI309910, SGI309911, SGI309912, SGI309913, SGI309914, SGI309915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309917, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72880[] = {SGI309918, SGI309919, SGI309920, SGI309921, SGI309922, SGI309923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309925, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72881[] = {SGI309926, SGI309927, SGI309928, SGI309929, SGI309930, SGI309931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72882[] = {SGI309932, SGI309933, SGI309934, SGI309935, SGI309936, SGI309937, SGI0, SGI0, SGI0, SGI0, SGI309938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309939, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72883[] = {SGI309940, SGI309941, SGI309942, SGI309943, SGI309944, SGI309945, SGI0, SGI0, SGI0, SGI0, SGI309946, SGI0, SGI0, SGI309947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309948, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72884[] = {SGI309949, SGI309950, SGI309951, SGI309952, SGI309953, SGI309954, SGI0, SGI309955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309956, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72885[] = {SGI309957, SGI309958, SGI309959, SGI309960, SGI309961, SGI309962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72886[] = {SGI309964, SGI309965, SGI309966, SGI309967, SGI309968, SGI309969, SGI0, SGI309970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72887[] = {SGI309972, SGI309973, SGI309974, SGI309975, SGI309976, SGI309977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72888[] = {SGI309979, SGI309980, SGI309981, SGI309982, SGI309983, SGI309984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309985, SGI0, SGI309986, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72889[] = {SGI309987, SGI309988, SGI309989, SGI309990, SGI309991, SGI309992, SGI0, SGI309993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI309994, SGI0, SGI309995, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72890[] = {SGI309996, SGI309997, SGI309998, SGI309999, SGI310000, SGI310001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72891[] = {SGI310003, SGI310004, SGI310005, SGI310006, SGI310007, SGI310008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72892[] = {SGI310010, SGI310011, SGI310012, SGI310013, SGI310014, SGI310015, SGI0, SGI0, SGI0, SGI0, SGI310016, SGI0, SGI0, SGI310017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72893[] = {SGI310018, SGI310019, SGI310020, SGI310021, SGI310022, SGI310023, SGI0, SGI310024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72894[] = {SGI310026, SGI310027, SGI310028, SGI310029, SGI310030, SGI310031, SGI0, SGI310032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72895[] = {SGI310034, SGI310035, SGI310036, SGI310037, SGI310038, SGI310039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310040, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72896[] = {SGI310041, SGI310042, SGI310043, SGI310044, SGI310045, SGI310046, SGI0, SGI0, SGI0, SGI0, SGI310047, SGI0, SGI0, SGI310048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72897[] = {SGI310049, SGI310050, SGI310051, SGI310052, SGI310053, SGI310054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310056, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72898[] = {SGI310057, SGI310058, SGI310059, SGI310060, SGI310061, SGI310062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72899[] = {SGI310064, SGI310065, SGI310066, SGI310067, SGI310068, SGI310069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310070, SGI0, SGI310071, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72900[] = {SGI310072, SGI310073, SGI310074, SGI310075, SGI310076, SGI310077, SGI0, SGI310078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72901[] = {SGI310080, SGI310081, SGI310082, SGI310083, SGI310084, SGI310085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72902[] = {SGI310086, SGI310087, SGI310088, SGI310089, SGI310090, SGI310091, SGI0, SGI0, SGI0, SGI0, SGI310092, SGI0, SGI0, SGI310093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72903[] = {SGI310094, SGI310095, SGI310096, SGI310097, SGI310098, SGI310099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310101, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72904[] = {SGI310102, SGI310103, SGI310104, SGI310105, SGI310106, SGI310107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72905[] = {SGI310108, SGI310109, SGI310110, SGI310111, SGI310112, SGI310113, SGI0, SGI310114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310115, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72906[] = {SGI310116, SGI310117, SGI310118, SGI310119, SGI310120, SGI310121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310122, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72907[] = {SGI310123, SGI310124, SGI310125, SGI310126, SGI310127, SGI310128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72908[] = {SGI310130, SGI310131, SGI310132, SGI310133, SGI310134, SGI310135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72909[] = {SGI310137, SGI310138, SGI310139, SGI310140, SGI310141, SGI310142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310143, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72910[] = {SGI310144, SGI310145, SGI310146, SGI310147, SGI310148, SGI310149, SGI0, SGI0, SGI0, SGI0, SGI310150, SGI0, SGI0, SGI310151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72911[] = {SGI310152, SGI310153, SGI310154, SGI310155, SGI310156, SGI310157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310158, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72912[] = {SGI310159, SGI310160, SGI310161, SGI310162, SGI310163, SGI310164, SGI0, SGI310165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72913[] = {SGI310167, SGI310168, SGI310169, SGI310170, SGI310171, SGI310172, SGI0, SGI310173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310174, SGI0, SGI310175, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72914[] = {SGI310176, SGI310177, SGI310178, SGI310179, SGI310180, SGI310181, SGI0, SGI310182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310183, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72915[] = {SGI310184, SGI310185, SGI310186, SGI310187, SGI310188, SGI310189, SGI0, SGI310190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72916[] = {SGI310192, SGI310193, SGI310194, SGI310195, SGI310196, SGI310197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310199, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72917[] = {SGI310200, SGI310201, SGI310202, SGI310203, SGI310204, SGI310205, SGI0, SGI0, SGI0, SGI0, SGI310206, SGI0, SGI0, SGI310207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310208, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72918[] = {SGI310209, SGI310210, SGI310211, SGI310212, SGI310213, SGI310214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310216, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72919[] = {SGI310217, SGI310218, SGI310219, SGI310220, SGI310221, SGI310222, SGI0, SGI310223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310224, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72920[] = {SGI310225, SGI310226, SGI310227, SGI310228, SGI310229, SGI310230, SGI0, SGI0, SGI0, SGI0, SGI310231, SGI0, SGI0, SGI310232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72921[] = {SGI310233, SGI310234, SGI310235, SGI310236, SGI310237, SGI310238, SGI0, SGI0, SGI0, SGI0, SGI310239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310240, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72922[] = {SGI310241, SGI310242, SGI310243, SGI310244, SGI310245, SGI310246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72923[] = {SGI310247, SGI310248, SGI310249, SGI310250, SGI310251, SGI310252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310253, SGI0, SGI310254, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72924[] = {SGI310255, SGI310256, SGI310257, SGI310258, SGI310259, SGI310260, SGI0, SGI310261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72925[] = {SGI310262, SGI310263, SGI310264, SGI310265, SGI310266, SGI310267, SGI0, SGI0, SGI0, SGI0, SGI310268, SGI0, SGI0, SGI310269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310270, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72926[] = {SGI310271, SGI310272, SGI310273, SGI310274, SGI310275, SGI310276, SGI0, SGI310277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72927[] = {SGI310279, SGI310280, SGI310281, SGI310282, SGI310283, SGI310284, SGI0, SGI310285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310286, SGI0, SGI310287, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72928[] = {SGI310288, SGI310289, SGI310290, SGI310291, SGI310292, SGI310293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310294, SGI0, SGI310295, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72929[] = {SGI310296, SGI310297, SGI310298, SGI310299, SGI310300, SGI310301, SGI0, SGI310302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310303, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72930[] = {SGI310304, SGI310305, SGI310306, SGI310307, SGI310308, SGI310309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72931[] = {SGI310310, SGI310311, SGI310312, SGI310313, SGI310314, SGI310315, SGI0, SGI310316, SGI0, SGI0, SGI310317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310318, SGI0, SGI310319, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72932[] = {SGI310320, SGI310321, SGI310322, SGI310323, SGI310324, SGI310325, SGI0, SGI0, SGI0, SGI0, SGI310326, SGI0, SGI0, SGI310327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72933[] = {SGI310328, SGI310329, SGI310330, SGI310331, SGI310332, SGI310333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310335, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72934[] = {SGI310336, SGI310337, SGI310338, SGI310339, SGI310340, SGI310341, SGI0, SGI0, SGI0, SGI0, SGI310342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310343, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72935[] = {SGI310344, SGI310345, SGI310346, SGI310347, SGI310348, SGI310349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310350, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72936[] = {SGI310351, SGI310352, SGI310353, SGI310354, SGI310355, SGI310356, SGI0, SGI310357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310359, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72937[] = {SGI310360, SGI310361, SGI310362, SGI310363, SGI310364, SGI310365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72938[] = {SGI310367, SGI310368, SGI310369, SGI310370, SGI310371, SGI310372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310373, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72939[] = {SGI310374, SGI310375, SGI310376, SGI310377, SGI310378, SGI310379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310381, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72940[] = {SGI310382, SGI310383, SGI310384, SGI310385, SGI310386, SGI310387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310388, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72941[] = {SGI310389, SGI310390, SGI310391, SGI310392, SGI310393, SGI310394, SGI0, SGI310395, SGI0, SGI0, SGI310396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310397, SGI0, SGI310398, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72942[] = {SGI310399, SGI310400, SGI310401, SGI310402, SGI310403, SGI310404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72943[] = {SGI310406, SGI310407, SGI310408, SGI310409, SGI310410, SGI310411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72944[] = {SGI310413, SGI310414, SGI310415, SGI310416, SGI310417, SGI310418, SGI0, SGI310419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310420, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72945[] = {SGI310421, SGI310422, SGI310423, SGI310424, SGI310425, SGI310426, SGI0, SGI310427, SGI0, SGI0, SGI310428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310429, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72946[] = {SGI310430, SGI310431, SGI310432, SGI310433, SGI310434, SGI310435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310436, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72947[] = {SGI310437, SGI310438, SGI310439, SGI310440, SGI310441, SGI310442, SGI0, SGI310443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72948[] = {SGI310444, SGI310445, SGI310446, SGI310447, SGI310448, SGI310449, SGI0, SGI0, SGI0, SGI0, SGI310450, SGI0, SGI0, SGI310451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310452, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72949[] = {SGI310453, SGI310454, SGI310455, SGI310456, SGI310457, SGI310458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72950[] = {SGI310460, SGI310461, SGI310462, SGI310463, SGI310464, SGI310465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310466, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72951[] = {SGI310467, SGI310468, SGI310469, SGI310470, SGI310471, SGI310472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310474, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72952[] = {SGI310475, SGI310476, SGI310477, SGI310478, SGI310479, SGI310480, SGI0, SGI310481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72953[] = {SGI310483, SGI310484, SGI310485, SGI310486, SGI310487, SGI310488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72954[] = {SGI310489, SGI310490, SGI310491, SGI310492, SGI310493, SGI310494, SGI0, SGI310495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310496, SGI0, SGI310497, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72955[] = {SGI310498, SGI310499, SGI310500, SGI310501, SGI310502, SGI310503, SGI0, SGI0, SGI0, SGI0, SGI310504, SGI0, SGI0, SGI310505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72956[] = {SGI310506, SGI310507, SGI310508, SGI310509, SGI310510, SGI310511, SGI0, SGI310512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310513, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72957[] = {SGI310514, SGI310515, SGI310516, SGI310517, SGI310518, SGI310519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310520, SGI0, SGI310521, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72958[] = {SGI310522, SGI310523, SGI310524, SGI310525, SGI310526, SGI310527, SGI0, SGI0, SGI0, SGI0, SGI310528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310529, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72959[] = {SGI310530, SGI310531, SGI310532, SGI310533, SGI310534, SGI310535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310537, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72960[] = {SGI310538, SGI310539, SGI310540, SGI310541, SGI310542, SGI310543, SGI0, SGI310544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310545, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72961[] = {SGI310546, SGI310547, SGI310548, SGI310549, SGI310550, SGI310551, SGI0, SGI310552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310553, SGI0, SGI310554, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72962[] = {SGI310555, SGI310556, SGI310557, SGI310558, SGI310559, SGI310560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310561, SGI0, SGI310562, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72963[] = {SGI310563, SGI310564, SGI310565, SGI310566, SGI310567, SGI310568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72964[] = {SGI310569, SGI310570, SGI310571, SGI310572, SGI310573, SGI310574, SGI0, SGI0, SGI0, SGI0, SGI310575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310576, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72965[] = {SGI310577, SGI310578, SGI310579, SGI310580, SGI310581, SGI310582, SGI0, SGI310583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72966[] = {SGI310585, SGI310586, SGI310587, SGI310588, SGI310589, SGI310590, SGI0, SGI0, SGI0, SGI0, SGI310591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310592, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72967[] = {SGI310593, SGI310594, SGI310595, SGI310596, SGI310597, SGI310598, SGI0, SGI0, SGI0, SGI0, SGI310599, SGI0, SGI0, SGI310600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72968[] = {SGI310601, SGI310602, SGI310603, SGI310604, SGI310605, SGI310606, SGI0, SGI0, SGI0, SGI0, SGI310607, SGI0, SGI0, SGI310608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310609, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72969[] = {SGI310610, SGI310611, SGI310612, SGI310613, SGI310614, SGI310615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310617, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72970[] = {SGI310618, SGI310619, SGI310620, SGI310621, SGI310622, SGI310623, SGI0, SGI310624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310625, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72971[] = {SGI310626, SGI310627, SGI310628, SGI310629, SGI310630, SGI310631, SGI0, SGI0, SGI0, SGI0, SGI310632, SGI0, SGI0, SGI310633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310634, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72972[] = {SGI310635, SGI310636, SGI310637, SGI310638, SGI310639, SGI310640, SGI0, SGI310641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310642, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72973[] = {SGI310643, SGI310644, SGI310645, SGI310646, SGI310647, SGI310648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310649, SGI0, SGI310650, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72974[] = {SGI310651, SGI310652, SGI310653, SGI310654, SGI310655, SGI310656, SGI0, SGI310657, SGI0, SGI0, SGI310658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310659, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72975[] = {SGI310660, SGI310661, SGI310662, SGI310663, SGI310664, SGI310665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310666, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72976[] = {SGI310667, SGI310668, SGI310669, SGI310670, SGI310671, SGI310672, SGI0, SGI0, SGI0, SGI0, SGI310673, SGI0, SGI0, SGI310674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310675, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72977[] = {SGI310676, SGI310677, SGI310678, SGI310679, SGI310680, SGI310681, SGI0, SGI0, SGI0, SGI0, SGI310682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310683, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72978[] = {SGI310684, SGI310685, SGI310686, SGI310687, SGI310688, SGI310689, SGI0, SGI310690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310691, SGI0, SGI310692, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72979[] = {SGI310693, SGI310694, SGI310695, SGI310696, SGI310697, SGI310698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72980[] = {SGI310700, SGI310701, SGI310702, SGI310703, SGI310704, SGI310705, SGI0, SGI310706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310708, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72981[] = {SGI310709, SGI310710, SGI310711, SGI310712, SGI310713, SGI310714, SGI0, SGI310715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72982[] = {SGI310716, SGI310717, SGI310718, SGI310719, SGI310720, SGI310721, SGI0, SGI310722, SGI0, SGI0, SGI310723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310724, SGI0, SGI310725, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72983[] = {SGI310726, SGI310727, SGI310728, SGI310729, SGI310730, SGI310731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310732, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72984[] = {SGI310733, SGI310734, SGI310735, SGI310736, SGI310737, SGI310738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72985[] = {SGI310740, SGI310741, SGI310742, SGI310743, SGI310744, SGI310745, SGI0, SGI310746, SGI0, SGI0, SGI310747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310748, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72986[] = {SGI310749, SGI310750, SGI310751, SGI310752, SGI310753, SGI310754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310755, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72987[] = {SGI310756, SGI310757, SGI310758, SGI310759, SGI310760, SGI310761, SGI0, SGI310762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310764, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72988[] = {SGI310765, SGI310766, SGI310767, SGI310768, SGI310769, SGI310770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72989[] = {SGI310772, SGI310773, SGI310774, SGI310775, SGI310776, SGI310777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72990[] = {SGI310779, SGI310780, SGI310781, SGI310782, SGI310783, SGI310784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72991[] = {SGI310785, SGI310786, SGI310787, SGI310788, SGI310789, SGI310790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72992[] = {SGI310792, SGI310793, SGI310794, SGI310795, SGI310796, SGI310797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310798, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72993[] = {SGI310799, SGI310800, SGI310801, SGI310802, SGI310803, SGI310804, SGI0, SGI310805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310806, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72994[] = {SGI310807, SGI310808, SGI310809, SGI310810, SGI310811, SGI310812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310813, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72995[] = {SGI310814, SGI310815, SGI310816, SGI310817, SGI310818, SGI310819, SGI0, SGI310820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72996[] = {SGI310822, SGI310823, SGI310824, SGI310825, SGI310826, SGI310827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72997[] = {SGI310829, SGI310830, SGI310831, SGI310832, SGI310833, SGI310834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310835, SGI0, SGI310836, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72998[] = {SGI310837, SGI310838, SGI310839, SGI310840, SGI310841, SGI310842, SGI0, SGI0, SGI0, SGI0, SGI310843, SGI0, SGI0, SGI310844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG72999[] = {SGI310845, SGI310846, SGI310847, SGI310848, SGI310849, SGI310850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310852, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73000[] = {SGI310853, SGI310854, SGI310855, SGI310856, SGI310857, SGI310858, SGI0, SGI310859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310860, SGI0, SGI310861, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73001[] = {SGI310862, SGI310863, SGI310864, SGI310865, SGI310866, SGI310867, SGI0, SGI310868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310869, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73002[] = {SGI310870, SGI310871, SGI310872, SGI310873, SGI310874, SGI310875, SGI0, SGI0, SGI0, SGI0, SGI310876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310877, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73003[] = {SGI310878, SGI310879, SGI310880, SGI310881, SGI310882, SGI310883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310884, SGI0, SGI310885, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73004[] = {SGI310886, SGI310887, SGI310888, SGI310889, SGI310890, SGI310891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310893, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73005[] = {SGI310894, SGI310895, SGI310896, SGI310897, SGI310898, SGI310899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310901, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73006[] = {SGI310902, SGI310903, SGI310904, SGI310905, SGI310906, SGI310907, SGI0, SGI310908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73007[] = {SGI310910, SGI310911, SGI310912, SGI310913, SGI310914, SGI310915, SGI0, SGI310916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310917, SGI0, SGI310918, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73008[] = {SGI310919, SGI310920, SGI310921, SGI310922, SGI310923, SGI310924, SGI0, SGI310925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310926, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73009[] = {SGI310927, SGI310928, SGI310929, SGI310930, SGI310931, SGI310932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310933, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73010[] = {SGI310934, SGI310935, SGI310936, SGI310937, SGI310938, SGI310939, SGI0, SGI0, SGI0, SGI0, SGI310940, SGI0, SGI0, SGI310941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310942, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73011[] = {SGI310943, SGI310944, SGI310945, SGI310946, SGI310947, SGI310948, SGI0, SGI310949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73012[] = {SGI310950, SGI310951, SGI310952, SGI310953, SGI310954, SGI310955, SGI0, SGI0, SGI0, SGI0, SGI310956, SGI0, SGI0, SGI310957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73013[] = {SGI310958, SGI310959, SGI310960, SGI310961, SGI310962, SGI310963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73014[] = {SGI310965, SGI310966, SGI310967, SGI310968, SGI310969, SGI310970, SGI0, SGI0, SGI0, SGI0, SGI310971, SGI0, SGI0, SGI310972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310973, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73015[] = {SGI310974, SGI310975, SGI310976, SGI310977, SGI310978, SGI310979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73016[] = {SGI310980, SGI310981, SGI310982, SGI310983, SGI310984, SGI310985, SGI0, SGI310986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310988, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73017[] = {SGI310989, SGI310990, SGI310991, SGI310992, SGI310993, SGI310994, SGI0, SGI0, SGI0, SGI0, SGI310995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI310996, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73018[] = {SGI310997, SGI310998, SGI310999, SGI311000, SGI311001, SGI311002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311003, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73019[] = {SGI311004, SGI311005, SGI311006, SGI311007, SGI311008, SGI311009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311010, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73020[] = {SGI311011, SGI311012, SGI311013, SGI311014, SGI311015, SGI311016, SGI0, SGI311017, SGI0, SGI0, SGI311018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311019, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73021[] = {SGI311020, SGI311021, SGI311022, SGI311023, SGI311024, SGI311025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73022[] = {SGI311027, SGI311028, SGI311029, SGI311030, SGI311031, SGI311032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73023[] = {SGI311034, SGI311035, SGI311036, SGI311037, SGI311038, SGI311039, SGI0, SGI311040, SGI0, SGI0, SGI311041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311042, SGI0, SGI311043, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73024[] = {SGI311044, SGI311045, SGI311046, SGI311047, SGI311048, SGI311049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311051, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73025[] = {SGI311052, SGI311053, SGI311054, SGI311055, SGI311056, SGI311057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73026[] = {SGI311059, SGI311060, SGI311061, SGI311062, SGI311063, SGI311064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311066, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73027[] = {SGI311067, SGI311068, SGI311069, SGI311070, SGI311071, SGI311072, SGI0, SGI0, SGI0, SGI0, SGI311073, SGI0, SGI0, SGI311074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311075, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73028[] = {SGI311076, SGI311077, SGI311078, SGI311079, SGI311080, SGI311081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73029[] = {SGI311083, SGI311084, SGI311085, SGI311086, SGI311087, SGI311088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311089, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73030[] = {SGI311090, SGI311091, SGI311092, SGI311093, SGI311094, SGI311095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311097, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73031[] = {SGI311098, SGI311099, SGI311100, SGI311101, SGI311102, SGI311103, SGI0, SGI311104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73032[] = {SGI311106, SGI311107, SGI311108, SGI311109, SGI311110, SGI311111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73033[] = {SGI311112, SGI311113, SGI311114, SGI311115, SGI311116, SGI311117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311118, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73034[] = {SGI311119, SGI311120, SGI311121, SGI311122, SGI311123, SGI311124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311125, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73035[] = {SGI311126, SGI311127, SGI311128, SGI311129, SGI311130, SGI311131, SGI0, SGI311132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311133, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73036[] = {SGI311134, SGI311135, SGI311136, SGI311137, SGI311138, SGI311139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311141, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73037[] = {SGI311142, SGI311143, SGI311144, SGI311145, SGI311146, SGI311147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73038[] = {SGI311149, SGI311150, SGI311151, SGI311152, SGI311153, SGI311154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311155, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73039[] = {SGI311156, SGI311157, SGI311158, SGI311159, SGI311160, SGI311161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73040[] = {SGI311163, SGI311164, SGI311165, SGI311166, SGI311167, SGI311168, SGI0, SGI0, SGI0, SGI0, SGI311169, SGI0, SGI0, SGI311170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73041[] = {SGI311171, SGI311172, SGI311173, SGI311174, SGI311175, SGI311176, SGI0, SGI311177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311178, SGI0, SGI311179, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73042[] = {SGI311180, SGI311181, SGI311182, SGI311183, SGI311184, SGI311185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311186, SGI0, SGI311187, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73043[] = {SGI311188, SGI311189, SGI311190, SGI311191, SGI311192, SGI311193, SGI0, SGI0, SGI0, SGI0, SGI311194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311195, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73044[] = {SGI311196, SGI311197, SGI311198, SGI311199, SGI311200, SGI311201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311202, SGI0, SGI311203, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73045[] = {SGI311204, SGI0, SGI311205, SGI311206, SGI311207, SGI311208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73046[] = {SGI311210, SGI0, SGI311211, SGI311212, SGI311213, SGI311214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73047[] = {SGI311216, SGI0, SGI311217, SGI311218, SGI311219, SGI311220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73048[] = {SGI311222, SGI0, SGI311223, SGI311224, SGI311225, SGI311226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311228, SGI0, SGI0, NULL};
const Segment * const SG73049[] = {SGI311229, SGI0, SGI311230, SGI311231, SGI311232, SGI311233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311235, SGI0, SGI0, NULL};
const Segment * const SG73050[] = {SGI311236, SGI0, SGI311237, SGI311238, SGI311239, SGI311240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311242, SGI0, SGI0, NULL};
const Segment * const SG73051[] = {SGI311243, SGI0, SGI311244, SGI311245, SGI311246, SGI311247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73052[] = {SGI311249, SGI0, SGI311250, SGI311251, SGI311252, SGI311253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311255, SGI0, SGI0, NULL};
const Segment * const SG73053[] = {SGI311256, SGI0, SGI311257, SGI311258, SGI311259, SGI311260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311262, SGI0, SGI0, NULL};
const Segment * const SG73054[] = {SGI311263, SGI0, SGI311264, SGI311265, SGI311266, SGI311267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73055[] = {SGI311269, SGI0, SGI311270, SGI311271, SGI311272, SGI311273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73056[] = {SGI311275, SGI0, SGI311276, SGI311277, SGI311278, SGI311279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311281, SGI0, SGI0, NULL};
const Segment * const SG73057[] = {SGI311282, SGI0, SGI311283, SGI311284, SGI311285, SGI311286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311288, SGI0, SGI0, NULL};
const Segment * const SG73058[] = {SGI311289, SGI0, SGI311290, SGI311291, SGI311292, SGI311293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311295, SGI0, SGI0, NULL};
const Segment * const SG73059[] = {SGI311296, SGI0, SGI311297, SGI311298, SGI311299, SGI311300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73060[] = {SGI311302, SGI0, SGI311303, SGI311304, SGI311305, SGI311306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73061[] = {SGI311308, SGI0, SGI311309, SGI311310, SGI311311, SGI311312, SGI0, SGI311313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311315, SGI0, SGI0, NULL};
const Segment * const SG73062[] = {SGI311316, SGI0, SGI311317, SGI311318, SGI311319, SGI311320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311322, SGI0, SGI0, NULL};
const Segment * const SG73063[] = {SGI311323, SGI0, SGI311324, SGI311325, SGI311326, SGI311327, SGI0, SGI311328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73064[] = {SGI311330, SGI0, SGI311331, SGI311332, SGI311333, SGI311334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311336, SGI0, SGI0, NULL};
const Segment * const SG73065[] = {SGI311337, SGI0, SGI311338, SGI311339, SGI311340, SGI311341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73066[] = {SGI311343, SGI0, SGI311344, SGI311345, SGI311346, SGI311347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311348, SGI0, SGI311349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73067[] = {SGI311350, SGI0, SGI311351, SGI311352, SGI311353, SGI311354, SGI0, SGI311355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73068[] = {SGI311357, SGI0, SGI311358, SGI311359, SGI311360, SGI311361, SGI0, SGI311362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73069[] = {SGI311364, SGI0, SGI311365, SGI311366, SGI311367, SGI311368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73070[] = {SGI311370, SGI0, SGI311371, SGI311372, SGI311373, SGI311374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73071[] = {SGI311376, SGI0, SGI311377, SGI311378, SGI311379, SGI311380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73072[] = {SGI311382, SGI0, SGI311383, SGI311384, SGI311385, SGI311386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311388, SGI0, SGI0, NULL};
const Segment * const SG73073[] = {SGI311389, SGI0, SGI311390, SGI311391, SGI311392, SGI311393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311394, SGI0, SGI311395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73074[] = {SGI311396, SGI0, SGI311397, SGI311398, SGI311399, SGI311400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73075[] = {SGI311402, SGI0, SGI311403, SGI311404, SGI311405, SGI311406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73076[] = {SGI311408, SGI0, SGI311409, SGI311410, SGI311411, SGI311412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73077[] = {SGI311414, SGI0, SGI311415, SGI311416, SGI311417, SGI311418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73078[] = {SGI311420, SGI0, SGI311421, SGI311422, SGI311423, SGI311424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73079[] = {SGI311426, SGI0, SGI311427, SGI311428, SGI311429, SGI311430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73080[] = {SGI311432, SGI0, SGI311433, SGI311434, SGI311435, SGI311436, SGI0, SGI311437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73081[] = {SGI311439, SGI0, SGI311440, SGI311441, SGI311442, SGI311443, SGI0, SGI311444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311446, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73082[] = {SGI311447, SGI0, SGI311448, SGI311449, SGI311450, SGI311451, SGI0, SGI311452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311453, SGI0, SGI311454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73083[] = {SGI311455, SGI0, SGI311456, SGI311457, SGI311458, SGI311459, SGI0, SGI311460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311461, SGI0, SGI311462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73084[] = {SGI311463, SGI0, SGI311464, SGI311465, SGI311466, SGI311467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311468, SGI0, SGI311469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311470, SGI0, SGI0, NULL};
const Segment * const SG73085[] = {SGI311471, SGI0, SGI311472, SGI311473, SGI311474, SGI311475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311477, SGI0, SGI0, NULL};
const Segment * const SG73086[] = {SGI311478, SGI0, SGI311479, SGI311480, SGI311481, SGI311482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73087[] = {SGI311484, SGI0, SGI311485, SGI311486, SGI311487, SGI311488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311489, SGI0, SGI311490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73088[] = {SGI311491, SGI0, SGI311492, SGI0, SGI311493, SGI311494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73089[] = {SGI311496, SGI0, SGI311497, SGI311498, SGI311499, SGI311500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73090[] = {SGI311502, SGI0, SGI311503, SGI311504, SGI311505, SGI311506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311507, SGI0, SGI311508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73091[] = {SGI311509, SGI0, SGI311510, SGI311511, SGI311512, SGI311513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311515, SGI0, SGI0, NULL};
const Segment * const SG73092[] = {SGI311516, SGI0, SGI311517, SGI311518, SGI311519, SGI311520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311521, SGI0, SGI311522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73093[] = {SGI311523, SGI0, SGI311524, SGI311525, SGI311526, SGI311527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311528, SGI0, SGI311529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311530, SGI0, SGI0, NULL};
const Segment * const SG73094[] = {SGI311531, SGI0, SGI311532, SGI311533, SGI311534, SGI311535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311537, SGI0, SGI0, NULL};
const Segment * const SG73095[] = {SGI311538, SGI0, SGI311539, SGI311540, SGI311541, SGI311542, SGI0, SGI311543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311545, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73096[] = {SGI311546, SGI0, SGI311547, SGI311548, SGI311549, SGI311550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73097[] = {SGI311552, SGI0, SGI311553, SGI311554, SGI311555, SGI311556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311557, SGI0, SGI311558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73098[] = {SGI311559, SGI0, SGI311560, SGI311561, SGI311562, SGI311563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311564, SGI0, SGI311565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73099[] = {SGI311566, SGI0, SGI311567, SGI311568, SGI311569, SGI311570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311572, SGI0, SGI0, NULL};
const Segment * const SG73100[] = {SGI0, SGI0, SGI311573, SGI0, SGI311574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311576, SGI0, SGI0, NULL};
const Segment * const SG73101[] = {SGI311577, SGI0, SGI311578, SGI311579, SGI311580, SGI311581, SGI0, SGI311582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311583, SGI0, SGI311584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73102[] = {SGI311585, SGI0, SGI311586, SGI311587, SGI311588, SGI311589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311590, SGI0, SGI311591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311592, SGI0, SGI0, NULL};
const Segment * const SG73103[] = {SGI311593, SGI0, SGI311594, SGI311595, SGI311596, SGI311597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73104[] = {SGI311599, SGI0, SGI311600, SGI311601, SGI311602, SGI311603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311604, SGI0, SGI311605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73105[] = {SGI311606, SGI0, SGI311607, SGI311608, SGI311609, SGI311610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311612, SGI0, SGI0, NULL};
const Segment * const SG73106[] = {SGI311613, SGI0, SGI311614, SGI311615, SGI311616, SGI311617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311619, SGI0, SGI0, NULL};
const Segment * const SG73107[] = {SGI311620, SGI0, SGI311621, SGI311622, SGI311623, SGI311624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311625, SGI0, SGI311626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73108[] = {SGI311627, SGI0, SGI311628, SGI311629, SGI311630, SGI311631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311632, SGI0, SGI311633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73109[] = {SGI311634, SGI0, SGI311635, SGI311636, SGI311637, SGI311638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311640, SGI0, SGI0, NULL};
const Segment * const SG73110[] = {SGI311641, SGI0, SGI311642, SGI311643, SGI311644, SGI311645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311647, SGI0, SGI0, NULL};
const Segment * const SG73111[] = {SGI311648, SGI0, SGI311649, SGI311650, SGI311651, SGI311652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311653, SGI0, SGI311654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73112[] = {SGI311655, SGI0, SGI311656, SGI311657, SGI311658, SGI311659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73113[] = {SGI311661, SGI0, SGI311662, SGI311663, SGI311664, SGI311665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311666, SGI0, SGI311667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73114[] = {SGI311668, SGI0, SGI311669, SGI311670, SGI311671, SGI311672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311673, SGI0, SGI311674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73115[] = {SGI311675, SGI0, SGI311676, SGI311677, SGI311678, SGI311679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311681, SGI0, SGI0, NULL};
const Segment * const SG73116[] = {SGI311682, SGI0, SGI311683, SGI311684, SGI311685, SGI311686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73117[] = {SGI311688, SGI0, SGI311689, SGI311690, SGI311691, SGI311692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73118[] = {SGI311694, SGI0, SGI311695, SGI311696, SGI311697, SGI311698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73119[] = {SGI311700, SGI0, SGI311701, SGI311702, SGI311703, SGI311704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73120[] = {SGI311706, SGI0, SGI311707, SGI311708, SGI311709, SGI311710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311711, SGI0, SGI311712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73121[] = {SGI311713, SGI0, SGI311714, SGI311715, SGI311716, SGI311717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73122[] = {SGI311719, SGI0, SGI311720, SGI311721, SGI311722, SGI311723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311725, SGI0, SGI0, NULL};
const Segment * const SG73123[] = {SGI311726, SGI0, SGI311727, SGI311728, SGI311729, SGI311730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311731, SGI0, SGI311732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73124[] = {SGI311733, SGI0, SGI311734, SGI311735, SGI311736, SGI311737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73125[] = {SGI311739, SGI311740, NULL};
const Segment * const SG73126[] = {SGI311741, SGI311742, SGI311743, SGI311744, SGI311745, SGI311746, SGI0, SGI311747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311748, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73127[] = {SGI311749, SGI311750, SGI311751, SGI311752, SGI311753, SGI311754, SGI0, SGI311755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73128[] = {SGI311757, SGI311758, SGI311759, SGI311760, SGI311761, SGI311762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73129[] = {SGI311764, SGI311765, SGI311766, SGI311767, SGI311768, SGI311769, SGI0, SGI311770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73130[] = {SGI311772, SGI311773, SGI311774, SGI311775, SGI311776, SGI311777, SGI0, SGI311778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73131[] = {SGI311780, SGI311781, SGI311782, SGI311783, SGI311784, SGI311785, SGI0, SGI311786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311787, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73132[] = {SGI311788, SGI311789, SGI311790, SGI311791, SGI311792, SGI311793, SGI0, SGI311794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311795, SGI0, SGI311796, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73133[] = {SGI311797, SGI311798, SGI311799, SGI311800, SGI311801, SGI311802, SGI0, SGI311803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311804, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73134[] = {SGI311805, SGI311806, SGI311807, SGI311808, SGI311809, SGI311810, SGI0, SGI311811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311812, SGI0, SGI311813, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73135[] = {SGI311814, SGI311815, SGI311816, SGI311817, SGI311818, SGI311819, SGI0, SGI311820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73136[] = {SGI311822, SGI311823, SGI311824, SGI311825, SGI311826, SGI311827, SGI0, SGI311828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73137[] = {SGI311829, SGI311830, SGI311831, SGI311832, SGI311833, SGI311834, SGI0, SGI311835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73138[] = {SGI311837, SGI311838, SGI311839, SGI311840, SGI311841, SGI311842, SGI0, SGI311843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311844, SGI0, SGI311845, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73139[] = {SGI311846, SGI311847, SGI311848, SGI311849, SGI311850, SGI311851, SGI0, SGI311852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311853, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73140[] = {SGI311854, SGI311855, SGI311856, SGI311857, SGI311858, SGI311859, SGI0, SGI311860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73141[] = {SGI311861, SGI311862, SGI311863, SGI311864, SGI311865, SGI311866, SGI0, SGI311867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311868, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73142[] = {SGI311869, SGI311870, SGI311871, SGI311872, SGI311873, SGI311874, SGI0, SGI311875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73143[] = {SGI311876, SGI311877, SGI311878, SGI311879, SGI311880, SGI311881, SGI0, SGI311882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73144[] = {SGI311883, SGI311884, SGI311885, SGI311886, SGI311887, SGI311888, SGI0, SGI311889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73145[] = {SGI311891, SGI311892, SGI311893, SGI311894, SGI311895, SGI311896, SGI0, SGI311897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73146[] = {SGI311898, SGI311899, SGI311900, SGI311901, SGI311902, SGI311903, SGI0, SGI311904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73147[] = {SGI311906, SGI311907, SGI311908, SGI311909, SGI311910, SGI311911, SGI0, SGI311912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311913, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73148[] = {SGI311914, SGI311915, SGI311916, SGI311917, SGI311918, SGI311919, SGI0, SGI311920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73149[] = {SGI311922, SGI311923, SGI311924, SGI311925, SGI311926, SGI311927, SGI0, SGI311928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311929, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73150[] = {SGI311930, SGI311931, SGI311932, SGI311933, SGI311934, SGI311935, SGI0, SGI311936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311937, SGI0, SGI311938, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73151[] = {SGI311939, SGI311940, SGI311941, SGI311942, SGI311943, SGI311944, SGI0, SGI311945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311946, SGI0, SGI311947, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73152[] = {SGI311948, SGI311949, SGI311950, SGI311951, SGI311952, SGI311953, SGI0, SGI311954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311955, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73153[] = {SGI311956, SGI311957, SGI311958, SGI311959, SGI311960, SGI311961, SGI0, SGI311962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311963, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73154[] = {SGI311964, SGI311965, SGI311966, SGI311967, SGI311968, SGI311969, SGI0, SGI311970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311971, SGI0, SGI311972, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73155[] = {SGI311973, SGI311974, SGI311975, SGI311976, SGI311977, SGI311978, SGI0, SGI311979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311980, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73156[] = {SGI311981, SGI311982, SGI311983, SGI311984, SGI311985, SGI311986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311987, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73157[] = {SGI311988, SGI311989, SGI311990, SGI311991, SGI311992, SGI311993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI311994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73158[] = {SGI311995, SGI311996, SGI311997, SGI311998, SGI311999, SGI312000, SGI0, SGI312001, SGI0, SGI0, SGI312002, SGI0, SGI0, SGI312003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73159[] = {SGI312004, SGI312005, SGI312006, SGI312007, SGI312008, SGI312009, SGI0, SGI312010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73160[] = {SGI312011, SGI312012, SGI312013, SGI312014, SGI312015, SGI312016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73161[] = {SGI312018, SGI312019, SGI312020, SGI312021, SGI312022, SGI312023, SGI0, SGI312024, SGI0, SGI0, SGI312025, SGI0, SGI0, SGI312026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73162[] = {SGI312027, SGI312028, SGI312029, SGI312030, SGI312031, SGI312032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73163[] = {SGI312034, SGI312035, SGI312036, SGI312037, SGI312038, SGI312039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312040, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73164[] = {SGI312041, SGI312042, SGI312043, SGI312044, SGI312045, SGI312046, SGI0, SGI312047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312048, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73165[] = {SGI312049, SGI312050, SGI312051, SGI312052, SGI312053, SGI312054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73166[] = {SGI312056, SGI312057, SGI312058, SGI312059, SGI312060, SGI312061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73167[] = {SGI312063, SGI312064, SGI312065, SGI312066, SGI312067, SGI312068, SGI0, SGI312069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312070, SGI0, SGI312071, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73168[] = {SGI312072, SGI312073, SGI312074, SGI312075, SGI312076, SGI312077, SGI0, SGI312078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73169[] = {SGI312080, SGI312081, SGI312082, SGI312083, SGI312084, SGI312085, SGI0, SGI312086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73170[] = {SGI312087, SGI312088, SGI312089, SGI312090, SGI312091, SGI312092, SGI0, SGI312093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73171[] = {SGI312095, SGI312096, SGI312097, SGI312098, SGI312099, SGI312100, SGI0, SGI312101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312102, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73172[] = {SGI312103, SGI312104, SGI312105, SGI312106, SGI312107, SGI312108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73173[] = {SGI312110, SGI312111, SGI312112, SGI312113, SGI312114, SGI312115, SGI0, SGI312116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312117, SGI0, SGI312118, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73174[] = {SGI312119, SGI312120, SGI312121, SGI312122, SGI312123, SGI312124, SGI0, SGI312125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312126, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73175[] = {SGI312127, SGI312128, SGI312129, SGI312130, SGI312131, SGI312132, SGI0, SGI312133, SGI0, SGI0, SGI312134, SGI0, SGI0, SGI312135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73176[] = {SGI312136, SGI312137, SGI312138, SGI312139, SGI312140, SGI312141, SGI0, SGI312142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73177[] = {SGI312143, SGI312144, SGI312145, SGI312146, SGI312147, SGI312148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73178[] = {SGI312150, SGI312151, SGI312152, SGI312153, SGI312154, SGI312155, SGI0, SGI312156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312157, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73179[] = {SGI312158, SGI312159, SGI312160, SGI312161, SGI312162, SGI312163, SGI0, SGI312164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312165, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73180[] = {SGI312166, SGI312167, SGI312168, SGI312169, SGI312170, SGI312171, SGI0, SGI312172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73181[] = {SGI312174, SGI312175, SGI312176, SGI312177, SGI312178, SGI312179, SGI0, SGI312180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73182[] = {SGI312182, SGI312183, SGI312184, SGI312185, SGI312186, SGI312187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73183[] = {SGI312189, SGI312190, SGI312191, SGI312192, SGI312193, SGI312194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73184[] = {SGI312196, SGI312197, SGI312198, SGI312199, SGI312200, SGI312201, SGI0, SGI312202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312203, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73185[] = {SGI312204, SGI312205, SGI312206, SGI312207, SGI312208, SGI312209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312210, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73186[] = {SGI312211, SGI312212, SGI312213, SGI312214, SGI312215, SGI312216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73187[] = {SGI312218, SGI312219, SGI312220, SGI312221, SGI312222, SGI312223, SGI0, SGI312224, SGI0, SGI0, SGI312225, SGI0, SGI0, SGI312226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73188[] = {SGI312227, SGI312228, SGI312229, SGI312230, SGI312231, SGI312232, SGI0, SGI312233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312234, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73189[] = {SGI312235, SGI312236, SGI312237, SGI312238, SGI312239, SGI312240, SGI0, SGI312241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312242, SGI0, SGI312243, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73190[] = {SGI312244, SGI312245, SGI312246, SGI312247, SGI312248, SGI312249, SGI0, SGI312250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73191[] = {SGI312251, SGI312252, SGI312253, SGI312254, SGI312255, SGI312256, SGI0, SGI312257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73192[] = {SGI312259, SGI312260, SGI312261, SGI312262, SGI312263, SGI312264, SGI0, SGI312265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73193[] = {SGI312267, SGI312268, SGI312269, SGI312270, SGI312271, SGI312272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73194[] = {SGI312274, SGI312275, SGI312276, SGI312277, SGI312278, SGI312279, SGI0, SGI312280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73195[] = {SGI312282, SGI312283, SGI312284, SGI312285, SGI312286, SGI312287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73196[] = {SGI312289, SGI312290, SGI312291, SGI312292, SGI312293, SGI312294, SGI0, SGI312295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312296, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73197[] = {SGI312297, SGI312298, SGI312299, SGI312300, SGI312301, SGI312302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73198[] = {SGI312304, SGI312305, SGI312306, SGI312307, SGI312308, SGI312309, SGI0, SGI312310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312311, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73199[] = {SGI312312, SGI312313, SGI312314, SGI312315, SGI312316, SGI312317, SGI0, SGI312318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312319, SGI0, SGI312320, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73200[] = {SGI312321, SGI312322, SGI312323, SGI312324, SGI312325, SGI312326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312327, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73201[] = {SGI312328, SGI312329, SGI312330, SGI312331, SGI312332, SGI312333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73202[] = {SGI312335, SGI312336, SGI312337, SGI312338, SGI312339, SGI312340, SGI0, SGI312341, SGI0, SGI0, SGI312342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312343, SGI0, SGI312344, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73203[] = {SGI312345, SGI312346, SGI312347, SGI312348, SGI312349, SGI312350, SGI0, SGI312351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73204[] = {SGI312353, SGI312354, SGI312355, SGI312356, SGI312357, SGI312358, SGI0, SGI312359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73205[] = {SGI312360, SGI312361, SGI312362, SGI312363, SGI312364, SGI312365, SGI0, SGI312366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73206[] = {SGI312368, SGI312369, SGI312370, SGI312371, SGI312372, SGI312373, SGI0, SGI312374, SGI0, SGI0, SGI312375, SGI0, SGI0, SGI312376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73207[] = {SGI312377, SGI312378, SGI312379, SGI312380, SGI312381, SGI312382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73208[] = {SGI312384, SGI312385, SGI312386, SGI312387, SGI312388, SGI312389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312390, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73209[] = {SGI312391, SGI312392, SGI312393, SGI312394, SGI312395, SGI312396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73210[] = {SGI312398, SGI312399, SGI312400, SGI312401, SGI312402, SGI312403, SGI0, SGI312404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312405, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73211[] = {SGI312406, SGI312407, SGI312408, SGI312409, SGI312410, SGI312411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73212[] = {SGI312412, SGI312413, SGI312414, SGI312415, SGI312416, SGI312417, SGI0, SGI312418, SGI0, SGI0, SGI312419, SGI0, SGI0, SGI312420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73213[] = {SGI312421, SGI312422, SGI312423, SGI312424, SGI312425, SGI312426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312427, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73214[] = {SGI312428, SGI312429, SGI312430, SGI312431, SGI312432, SGI312433, SGI0, SGI312434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73215[] = {SGI312436, SGI312437, SGI312438, SGI312439, SGI312440, SGI312441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73216[] = {SGI312443, SGI312444, SGI312445, SGI312446, SGI312447, SGI312448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312449, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73217[] = {SGI312450, SGI312451, SGI312452, SGI312453, SGI312454, SGI312455, SGI0, SGI312456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73218[] = {SGI312458, SGI312459, SGI312460, SGI312461, SGI312462, SGI312463, SGI0, SGI312464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312465, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73219[] = {SGI312466, SGI312467, SGI312468, SGI312469, SGI312470, SGI312471, SGI0, SGI312472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73220[] = {SGI312473, SGI312474, SGI312475, SGI312476, SGI312477, SGI312478, SGI0, SGI312479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312480, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73221[] = {SGI312481, SGI312482, SGI312483, SGI312484, SGI312485, SGI312486, SGI0, SGI312487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73222[] = {SGI312489, SGI312490, SGI312491, SGI312492, SGI312493, SGI312494, SGI0, SGI312495, SGI0, SGI0, SGI312496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312497, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73223[] = {SGI312498, SGI312499, SGI312500, SGI312501, SGI312502, SGI312503, SGI0, SGI312504, SGI0, SGI0, SGI312505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312506, SGI0, SGI312507, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73224[] = {SGI312508, SGI312509, SGI312510, SGI312511, SGI312512, SGI312513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73225[] = {SGI312514, SGI312515, SGI312516, SGI312517, SGI312518, SGI312519, SGI0, SGI312520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312521, SGI0, SGI312522, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73226[] = {SGI312523, SGI312524, SGI312525, SGI312526, SGI312527, SGI312528, SGI0, SGI312529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312530, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73227[] = {SGI312531, SGI312532, SGI312533, SGI312534, SGI312535, SGI312536, SGI0, SGI312537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312538, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73228[] = {SGI312539, SGI312540, SGI312541, SGI312542, SGI312543, SGI312544, SGI0, SGI312545, SGI0, SGI0, SGI312546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312547, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73229[] = {SGI312548, SGI312549, SGI312550, SGI312551, SGI312552, SGI312553, SGI0, SGI312554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73230[] = {SGI312556, SGI312557, SGI312558, SGI312559, SGI312560, SGI312561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73231[] = {SGI312563, SGI312564, SGI312565, SGI312566, SGI312567, SGI312568, SGI0, SGI312569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312570, SGI0, SGI312571, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73232[] = {SGI312572, SGI312573, SGI312574, SGI312575, SGI312576, SGI312577, SGI0, SGI312578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73233[] = {SGI312579, SGI0, SGI312580, SGI312581, SGI312582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73234[] = {SGI312583, SGI0, SGI312584, SGI312585, SGI312586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73235[] = {SGI312587, SGI0, SGI312588, SGI312589, SGI312590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73236[] = {SGI0, SGI0, SGI312591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312592, SGI0, SGI0, NULL};
const Segment * const SG73237[] = {SGI312593, SGI0, SGI0, SGI0, SGI312594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73238[] = {SGI0, SGI0, SGI312595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73239[] = {SGI312596, SGI0, SGI312597, SGI312598, SGI312599, SGI312600, SGI312601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73240[] = {SGI312604, SGI0, SGI312605, SGI312606, SGI312607, SGI312608, SGI312609, SGI312610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73241[] = {SGI312612, SGI0, SGI312613, SGI312614, SGI312615, SGI312616, SGI312617, SGI312618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73242[] = {SGI312620, SGI0, SGI312621, SGI312622, SGI312623, SGI312624, SGI312625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73243[] = {SGI312627, SGI0, SGI312628, SGI312629, SGI312630, SGI312631, SGI312632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73244[] = {SGI312634, SGI0, SGI312635, SGI312636, SGI312637, SGI312638, SGI312639, SGI312640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312642, SGI0, SGI0, NULL};
const Segment * const SG73245[] = {SGI312643, SGI0, SGI312644, SGI312645, SGI312646, SGI312647, SGI312648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73246[] = {SGI312650, SGI0, SGI312651, SGI312652, SGI312653, SGI312654, SGI312655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73247[] = {SGI312658, SGI0, SGI312659, SGI312660, SGI312661, SGI312662, SGI312663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312665, SGI0, SGI0, NULL};
const Segment * const SG73248[] = {SGI312666, SGI0, SGI312667, SGI312668, SGI312669, SGI312670, SGI312671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73249[] = {SGI312673, SGI0, SGI312674, SGI312675, SGI312676, SGI312677, SGI312678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73250[] = {SGI312680, SGI0, SGI312681, SGI312682, SGI312683, SGI312684, SGI312685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312687, SGI0, SGI0, NULL};
const Segment * const SG73251[] = {SGI312688, SGI0, SGI312689, SGI312690, SGI312691, SGI312692, SGI312693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312695, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73252[] = {SGI312696, SGI0, SGI312697, SGI312698, SGI312699, SGI312700, SGI312701, SGI0, SGI0, SGI0, SGI312702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73253[] = {SGI312704, SGI312705, SGI312706, SGI312707, SGI312708, SGI312709, SGI0, SGI312710, SGI0, SGI0, SGI312711, SGI0, SGI0, SGI312712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73254[] = {SGI312713, SGI312714, SGI312715, SGI312716, SGI312717, SGI312718, SGI0, SGI312719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73255[] = {SGI312721, SGI312722, SGI312723, SGI312724, SGI312725, SGI312726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312727, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73256[] = {SGI312728, SGI312729, SGI312730, SGI312731, SGI312732, SGI312733, SGI0, SGI0, SGI0, SGI0, SGI312734, SGI0, SGI0, SGI312735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73257[] = {SGI312736, SGI312737, SGI312738, SGI312739, SGI312740, SGI312741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73258[] = {SGI312743, SGI312744, SGI312745, SGI312746, SGI312747, SGI312748, SGI0, SGI0, SGI0, SGI0, SGI312749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312750, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73259[] = {SGI312751, SGI312752, SGI312753, SGI312754, SGI312755, SGI312756, SGI0, SGI0, SGI0, SGI0, SGI312757, SGI0, SGI0, SGI312758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312759, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73260[] = {SGI312760, SGI312761, SGI312762, SGI312763, SGI312764, SGI312765, SGI0, SGI312766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312767, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73261[] = {SGI312768, SGI312769, SGI312770, SGI312771, SGI312772, SGI312773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73262[] = {SGI312774, SGI312775, SGI312776, SGI312777, SGI312778, SGI312779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73263[] = {SGI312781, SGI312782, SGI312783, SGI312784, SGI312785, SGI312786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312788, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73264[] = {SGI312789, SGI312790, SGI312791, SGI312792, SGI312793, SGI312794, SGI0, SGI312795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312796, SGI0, SGI312797, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73265[] = {SGI312798, SGI312799, SGI312800, SGI312801, SGI312802, SGI312803, SGI0, SGI312804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73266[] = {SGI312806, SGI312807, SGI312808, SGI312809, SGI312810, SGI312811, SGI0, SGI312812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73267[] = {SGI312813, SGI312814, SGI312815, SGI312816, SGI312817, SGI312818, SGI0, SGI0, SGI0, SGI0, SGI312819, SGI0, SGI0, SGI312820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73268[] = {SGI312821, SGI312822, SGI312823, SGI312824, SGI312825, SGI312826, SGI0, SGI312827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312828, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73269[] = {SGI312829, SGI312830, SGI312831, SGI312832, SGI312833, SGI312834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73270[] = {SGI312836, SGI312837, SGI312838, SGI312839, SGI312840, SGI312841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73271[] = {SGI312843, SGI312844, SGI312845, SGI312846, SGI312847, SGI312848, SGI0, SGI312849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73272[] = {SGI312851, SGI312852, SGI312853, SGI312854, SGI312855, SGI312856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312857, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73273[] = {SGI312858, SGI312859, SGI312860, SGI312861, SGI312862, SGI312863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73274[] = {SGI312864, SGI312865, SGI312866, SGI312867, SGI312868, SGI312869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312871, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73275[] = {SGI312872, SGI312873, SGI312874, SGI312875, SGI312876, SGI312877, SGI0, SGI312878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312879, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73276[] = {SGI312880, SGI312881, SGI312882, SGI312883, SGI312884, SGI312885, SGI0, SGI312886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312887, SGI0, SGI312888, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73277[] = {SGI312889, SGI312890, SGI312891, SGI312892, SGI312893, SGI312894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312895, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73278[] = {SGI312896, SGI312897, SGI312898, SGI312899, SGI312900, SGI312901, SGI0, SGI312902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73279[] = {SGI312903, SGI312904, SGI312905, SGI312906, SGI312907, SGI312908, SGI0, SGI312909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312910, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73280[] = {SGI312911, SGI312912, SGI312913, SGI312914, SGI312915, SGI312916, SGI0, SGI312917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312919, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73281[] = {SGI312920, SGI312921, SGI312922, SGI312923, SGI312924, SGI312925, SGI0, SGI0, SGI0, SGI0, SGI312926, SGI0, SGI0, SGI312927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73282[] = {SGI312928, SGI312929, SGI312930, SGI312931, SGI312932, SGI312933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73283[] = {SGI312934, SGI312935, SGI312936, SGI312937, SGI312938, SGI312939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73284[] = {SGI312941, SGI312942, SGI312943, SGI312944, SGI312945, SGI312946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73285[] = {SGI312948, SGI312949, SGI312950, SGI312951, SGI312952, SGI312953, SGI0, SGI0, SGI0, SGI0, SGI312954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312955, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73286[] = {SGI312956, SGI312957, SGI312958, SGI312959, SGI312960, SGI312961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73287[] = {SGI312963, SGI312964, SGI312965, SGI312966, SGI312967, SGI312968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312969, SGI0, SGI312970, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73288[] = {SGI312971, SGI312972, SGI312973, SGI312974, SGI312975, SGI312976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312977, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73289[] = {SGI312978, SGI312979, SGI312980, SGI312981, SGI312982, SGI312983, SGI0, SGI312984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73290[] = {SGI312986, SGI312987, SGI312988, SGI312989, SGI312990, SGI312991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73291[] = {SGI312993, SGI312994, SGI312995, SGI312996, SGI312997, SGI312998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI312999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313000, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73292[] = {SGI313001, SGI313002, SGI313003, SGI313004, SGI313005, SGI313006, SGI0, SGI313007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313008, SGI0, SGI313009, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73293[] = {SGI313010, SGI313011, SGI313012, SGI313013, SGI313014, SGI313015, SGI0, SGI313016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313017, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73294[] = {SGI313018, SGI313019, SGI313020, SGI313021, SGI313022, SGI313023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73295[] = {SGI313024, SGI313025, SGI313026, SGI313027, SGI313028, SGI313029, SGI0, SGI313030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313032, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73296[] = {SGI313033, SGI313034, SGI313035, SGI313036, SGI313037, SGI313038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73297[] = {SGI313040, SGI313041, SGI313042, SGI313043, SGI313044, SGI313045, SGI0, SGI0, SGI0, SGI0, SGI313046, SGI0, SGI0, SGI313047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73298[] = {SGI313048, SGI313049, SGI313050, SGI313051, SGI313052, SGI313053, SGI0, SGI313054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73299[] = {SGI313055, SGI313056, SGI313057, SGI313058, SGI313059, SGI313060, SGI0, SGI313061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313062, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73300[] = {SGI313063, SGI313064, SGI313065, SGI313066, SGI313067, SGI313068, SGI0, SGI0, SGI0, SGI0, SGI313069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313070, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73301[] = {SGI313071, SGI313072, SGI313073, SGI313074, SGI313075, SGI313076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73302[] = {SGI313078, SGI313079, SGI313080, SGI313081, SGI313082, SGI313083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313084, SGI0, SGI313085, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73303[] = {SGI313086, SGI313087, SGI313088, SGI313089, SGI313090, SGI313091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313093, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73304[] = {SGI313094, SGI313095, SGI313096, SGI313097, SGI313098, SGI313099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313100, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73305[] = {SGI313101, SGI313102, SGI313103, SGI313104, SGI313105, SGI313106, SGI0, SGI313107, SGI0, SGI0, SGI313108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313109, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73306[] = {SGI313110, SGI313111, SGI313112, SGI313113, SGI313114, SGI313115, SGI0, SGI313116, SGI0, SGI0, SGI313117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313118, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73307[] = {SGI313119, SGI313120, SGI313121, SGI313122, SGI313123, SGI313124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313125, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73308[] = {SGI313126, SGI313127, SGI313128, SGI313129, SGI313130, SGI313131, SGI0, SGI313132, SGI0, SGI0, SGI313133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313134, SGI0, SGI313135, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73309[] = {SGI313136, SGI313137, SGI313138, SGI313139, SGI313140, SGI313141, SGI0, SGI0, SGI0, SGI0, SGI313142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73310[] = {SGI313143, SGI313144, SGI313145, SGI313146, SGI313147, SGI313148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313149, SGI0, SGI313150, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73311[] = {SGI313151, SGI313152, SGI313153, SGI313154, SGI313155, SGI313156, SGI0, SGI313157, SGI0, SGI0, SGI313158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313159, SGI0, SGI313160, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73312[] = {SGI313161, SGI313162, SGI313163, SGI313164, SGI313165, SGI313166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313168, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73313[] = {SGI313169, SGI313170, SGI313171, SGI313172, SGI313173, SGI313174, SGI0, SGI0, SGI0, SGI0, SGI313175, SGI0, SGI0, SGI313176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313177, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73314[] = {SGI313178, SGI313179, SGI313180, SGI313181, SGI313182, SGI313183, SGI0, SGI313184, SGI0, SGI0, SGI313185, SGI0, SGI0, SGI313186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73315[] = {SGI313187, SGI313188, SGI313189, SGI313190, SGI313191, SGI313192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73316[] = {SGI313194, SGI313195, SGI313196, SGI313197, SGI313198, SGI313199, SGI0, SGI313200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313201, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73317[] = {SGI313202, SGI313203, SGI313204, SGI313205, SGI313206, SGI313207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313208, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73318[] = {SGI313209, SGI313210, SGI313211, SGI313212, SGI313213, SGI313214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313216, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73319[] = {SGI313217, SGI313218, SGI313219, SGI313220, SGI313221, SGI313222, SGI0, SGI313223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73320[] = {SGI313225, SGI313226, SGI313227, SGI313228, SGI313229, SGI313230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73321[] = {SGI313231, SGI313232, SGI313233, SGI313234, SGI313235, SGI313236, SGI0, SGI313237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313238, SGI0, SGI313239, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73322[] = {SGI313240, SGI313241, SGI313242, SGI313243, SGI313244, SGI313245, SGI0, SGI0, SGI0, SGI0, SGI313246, SGI0, SGI0, SGI313247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73323[] = {SGI313248, SGI313249, SGI313250, SGI313251, SGI313252, SGI313253, SGI0, SGI313254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73324[] = {SGI313255, SGI313256, SGI313257, SGI313258, SGI313259, SGI313260, SGI0, SGI313261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73325[] = {SGI313263, SGI313264, SGI313265, SGI313266, SGI313267, SGI313268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73326[] = {SGI313270, SGI313271, SGI313272, SGI313273, SGI313274, SGI313275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313277, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73327[] = {SGI313278, SGI313279, SGI313280, SGI313281, SGI313282, SGI313283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313284, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73328[] = {SGI313285, SGI313286, SGI313287, SGI313288, SGI313289, SGI313290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73329[] = {SGI313291, SGI313292, SGI313293, SGI313294, SGI313295, SGI313296, SGI0, SGI313297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313298, SGI0, SGI313299, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73330[] = {SGI313300, SGI313301, SGI313302, SGI313303, SGI313304, SGI313305, SGI0, SGI0, SGI0, SGI0, SGI313306, SGI0, SGI0, SGI313307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73331[] = {SGI313308, SGI313309, SGI313310, SGI313311, SGI313312, SGI313313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73332[] = {SGI313315, SGI313316, SGI313317, SGI313318, SGI313319, SGI313320, SGI0, SGI313321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313323, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73333[] = {SGI313324, SGI313325, SGI313326, SGI313327, SGI313328, SGI313329, SGI0, SGI313330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313331, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73334[] = {SGI313332, SGI313333, SGI313334, SGI313335, SGI313336, SGI313337, SGI0, SGI313338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73335[] = {SGI313339, SGI313340, SGI313341, SGI313342, SGI313343, SGI313344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313345, SGI0, SGI313346, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73336[] = {SGI313347, SGI0, SGI313348, SGI313349, SGI313350, SGI313351, SGI313352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73337[] = {SGI313354, SGI0, SGI313355, SGI313356, SGI313357, SGI313358, SGI313359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73338[] = {SGI313361, SGI0, SGI313362, SGI313363, SGI313364, SGI313365, SGI313366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73339[] = {SGI313368, SGI0, SGI313369, SGI313370, SGI313371, SGI313372, SGI313373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313375, SGI0, SGI0, NULL};
const Segment * const SG73340[] = {SGI313376, SGI0, SGI313377, SGI313378, SGI313379, SGI313380, SGI313381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313383, SGI0, SGI0, NULL};
const Segment * const SG73341[] = {SGI313384, SGI0, SGI313385, SGI313386, SGI313387, SGI313388, SGI313389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73342[] = {SGI313392, SGI0, SGI313393, SGI313394, SGI313395, SGI313396, SGI313397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313399, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73343[] = {SGI313400, SGI0, SGI313401, SGI313402, SGI313403, SGI313404, SGI313405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73344[] = {SGI313407, SGI0, SGI313408, SGI313409, SGI313410, SGI313411, SGI313412, SGI313413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73345[] = {SGI313415, SGI0, SGI313416, SGI313417, SGI313418, SGI313419, SGI313420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313422, SGI0, SGI0, NULL};
const Segment * const SG73346[] = {SGI313423, SGI0, SGI313424, SGI313425, SGI313426, SGI313427, SGI313428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73347[] = {SGI313430, SGI0, SGI313431, SGI313432, SGI313433, SGI313434, SGI313435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313437, SGI0, SGI0, NULL};
const Segment * const SG73348[] = {SGI313438, SGI0, SGI313439, SGI313440, SGI313441, SGI313442, SGI313443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313445, SGI0, SGI0, NULL};
const Segment * const SG73349[] = {SGI313446, SGI313447, NULL};
const Segment * const SG73350[] = {SGI0, SGI0, SGI313448, SGI313449, SGI313450, SGI313451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73351[] = {SGI0, SGI0, SGI313452, SGI313453, SGI313454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73352[] = {SGI313455, SGI0, SGI313456, SGI313457, SGI313458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73353[] = {SGI313459, SGI313460, NULL};
const Segment * const SG73354[] = {SGI0, SGI0, SGI313461, SGI313462, SGI313463, SGI313464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73355[] = {SGI313465, SGI0, SGI313466, SGI313467, SGI313468, SGI313469, SGI313470, SGI313471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73356[] = {SGI313473, SGI0, SGI313474, SGI313475, SGI313476, SGI313477, SGI313478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73357[] = {SGI313480, SGI0, SGI313481, SGI313482, SGI313483, SGI313484, SGI313485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73358[] = {SGI313488, SGI0, SGI313489, SGI313490, SGI313491, SGI313492, SGI313493, SGI313494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73359[] = {SGI313497, SGI0, SGI313498, SGI313499, SGI313500, SGI313501, SGI313502, SGI313503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73360[] = {SGI313505, SGI0, SGI313506, SGI313507, SGI313508, SGI313509, SGI313510, SGI313511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73361[] = {SGI313514, SGI0, SGI313515, SGI313516, SGI313517, SGI313518, SGI313519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73362[] = {SGI313521, SGI0, SGI313522, SGI313523, SGI313524, SGI313525, SGI313526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73363[] = {SGI313528, SGI0, SGI313529, SGI313530, SGI313531, SGI313532, SGI313533, SGI313534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73364[] = {SGI313537, SGI0, SGI313538, SGI313539, SGI313540, SGI313541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73365[] = {SGI313542, SGI0, SGI313543, SGI313544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313545, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73366[] = {SGI313546, SGI0, SGI313547, SGI313548, SGI313549, SGI313550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73367[] = {SGI313551, SGI0, SGI313552, SGI313553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313554, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73368[] = {SGI313555, SGI0, SGI313556, SGI313557, SGI313558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73369[] = {SGI313559, SGI0, SGI313560, SGI313561, SGI313562, SGI313563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73370[] = {SGI313564, SGI0, SGI313565, SGI313566, SGI313567, SGI313568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73371[] = {SGI313569, SGI0, SGI313570, SGI313571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73372[] = {SGI313572, SGI0, SGI313573, SGI313574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73373[] = {SGI313575, SGI0, SGI313576, SGI313577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73374[] = {SGI313579, SGI0, SGI313580, SGI313581, SGI313582, SGI313583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73375[] = {SGI313584, SGI0, SGI313585, SGI313586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313587, SGI313588, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73376[] = {SGI313589, SGI0, SGI313590, SGI313591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313592, SGI0, SGI0, NULL};
const Segment * const SG73377[] = {SGI313593, SGI0, SGI313594, SGI313595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313597, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73378[] = {SGI313598, SGI0, SGI313599, SGI313600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313601, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73379[] = {SGI313602, SGI0, SGI313603, SGI313604, SGI313605, SGI313606, SGI313607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73380[] = {SGI313609, SGI0, SGI313610, SGI313611, SGI313612, SGI313613, SGI313614, SGI313615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73381[] = {SGI313617, SGI0, SGI313618, SGI313619, SGI313620, SGI313621, SGI313622, SGI313623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73382[] = {SGI313626, SGI0, SGI313627, SGI313628, SGI313629, SGI313630, SGI313631, SGI313632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73383[] = {SGI313635, SGI0, SGI313636, SGI313637, SGI313638, SGI313639, SGI313640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73384[] = {SGI313642, SGI0, SGI313643, SGI313644, SGI313645, SGI313646, SGI313647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73385[] = {SGI313650, SGI0, SGI313651, SGI313652, SGI313653, SGI313654, SGI313655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73386[] = {SGI313657, SGI0, SGI313658, SGI313659, SGI313660, SGI313661, SGI313662, SGI313663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73387[] = {SGI313665, SGI0, SGI313666, SGI313667, SGI313668, SGI313669, SGI313670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313672, SGI0, SGI0, NULL};
const Segment * const SG73388[] = {SGI313673, SGI0, SGI313674, SGI313675, SGI313676, SGI313677, SGI313678, SGI313679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73389[] = {SGI313682, SGI0, SGI313683, SGI313684, SGI313685, SGI313686, SGI313687, SGI313688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313690, SGI0, SGI0, NULL};
const Segment * const SG73390[] = {SGI313691, SGI0, SGI313692, SGI313693, SGI313694, SGI313695, SGI313696, SGI313697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313699, SGI0, SGI0, NULL};
const Segment * const SG73391[] = {SGI313700, SGI0, SGI313701, SGI313702, SGI313703, SGI313704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73392[] = {SGI313706, SGI0, SGI313707, SGI313708, SGI313709, SGI313710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313711, SGI313712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73393[] = {SGI313713, SGI0, SGI313714, SGI313715, SGI313716, SGI313717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313718, SGI313719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73394[] = {SGI313720, SGI0, SGI313721, SGI313722, SGI313723, SGI313724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73395[] = {SGI313726, SGI0, SGI313727, SGI313728, SGI313729, SGI313730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313731, SGI313732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313733, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73396[] = {SGI313734, SGI0, SGI313735, SGI313736, SGI313737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73397[] = {SGI313738, SGI0, SGI313739, SGI313740, SGI313741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73398[] = {SGI313742, SGI0, SGI313743, SGI313744, SGI313745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73399[] = {SGI313746, SGI0, SGI313747, SGI313748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313749, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73400[] = {SGI313750, SGI0, SGI313751, SGI313752, SGI313753, SGI313754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73401[] = {SGI313755, SGI0, SGI313756, SGI313757, SGI313758, SGI313759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73402[] = {SGI313760, SGI0, SGI313760, SGI313761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73403[] = {SGI313762, SGI0, SGI313762, SGI313763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73404[] = {SGI313765, SGI0, SGI313765, SGI313766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313767, SGI313768, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73405[] = {SGI313769, SGI0, SGI313769, SGI313770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313771, SGI0, SGI0, NULL};
const Segment * const SG73406[] = {SGI313772, SGI0, SGI313772, SGI313773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313774, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73407[] = {SGI313775, SGI0, SGI313775, SGI313776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313778, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73408[] = {SGI313779, SGI0, SGI313779, SGI313780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313781, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73409[] = {SGI313782, SGI313782, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73410[] = {SGI313783, SGI0, SGI313784, SGI313785, SGI313786, SGI313787, SGI313788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73411[] = {SGI313790, SGI0, SGI313791, SGI313792, SGI313793, SGI313794, SGI313795, SGI313796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73412[] = {SGI313799, SGI0, SGI313800, SGI313801, SGI313802, SGI313803, SGI313804, SGI313805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73413[] = {SGI313807, SGI0, SGI313808, SGI313809, SGI313810, SGI313811, SGI313812, SGI313813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73414[] = {SGI313815, SGI0, SGI313816, SGI313817, SGI313818, SGI313819, SGI313820, SGI313821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73415[] = {SGI313824, SGI0, SGI313825, SGI313826, SGI313827, SGI313828, SGI313829, SGI313830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313832, SGI0, SGI0, NULL};
const Segment * const SG73416[] = {SGI313833, SGI0, SGI313834, SGI313835, SGI313836, SGI313837, SGI313838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73417[] = {SGI313840, SGI0, SGI313841, SGI313842, SGI313843, SGI313844, SGI313845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73418[] = {SGI313848, SGI0, SGI313849, SGI313850, SGI313851, SGI313852, SGI313853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73419[] = {SGI313855, SGI0, SGI313856, SGI313857, SGI313858, SGI313859, SGI313860, SGI313861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73420[] = {SGI313864, SGI0, SGI313865, SGI313866, SGI313867, SGI313868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313870, SGI0, SGI0, SGI0, SGI313871, SGI0, SGI0, NULL};
const Segment * const SG73421[] = {SGI313872, SGI0, SGI313873, SGI313874, SGI313875, SGI313876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313878, SGI0, SGI0, SGI0, SGI313879, SGI0, SGI0, NULL};
const Segment * const SG73422[] = {SGI313880, SGI0, SGI313881, SGI313882, SGI313883, SGI313884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73423[] = {SGI313887, SGI0, SGI313888, SGI313889, SGI313890, SGI313891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73424[] = {SGI313894, SGI0, SGI313895, SGI313896, SGI313897, SGI313898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313900, SGI0, SGI0, SGI0, SGI0, SGI313901, SGI0, SGI0, NULL};
const Segment * const SG73425[] = {SGI313902, SGI0, SGI313903, SGI313904, SGI313905, SGI313906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313908, SGI0, SGI0, SGI0, SGI0, SGI313909, SGI0, SGI0, NULL};
const Segment * const SG73426[] = {SGI313910, SGI0, SGI313911, SGI313912, SGI313913, SGI313914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313916, SGI0, SGI0, SGI0, SGI313917, SGI0, SGI0, NULL};
const Segment * const SG73427[] = {SGI313918, SGI0, SGI313919, SGI313920, SGI313921, SGI313922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313924, SGI0, SGI0, SGI0, SGI0, SGI313925, SGI0, SGI0, NULL};
const Segment * const SG73428[] = {SGI313926, SGI0, SGI313927, SGI313928, SGI313929, SGI313930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73429[] = {SGI313933, SGI0, SGI313934, SGI313935, SGI313936, SGI313937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73430[] = {SGI313940, SGI0, SGI313941, SGI313942, SGI313943, SGI313944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313946, SGI0, SGI0, SGI0, SGI313947, SGI0, SGI0, NULL};
const Segment * const SG73431[] = {SGI313948, SGI0, SGI313949, SGI313950, SGI313951, SGI313952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73432[] = {SGI313955, SGI0, SGI313956, SGI313957, SGI313958, SGI313959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73433[] = {SGI313962, SGI0, SGI313963, SGI313964, SGI313965, SGI313966, SGI0, SGI313967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73434[] = {SGI313970, SGI0, SGI313971, SGI313972, SGI313973, SGI313974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313975, SGI0, SGI313976, SGI0, SGI0, SGI0, SGI0, SGI313977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73435[] = {SGI313978, SGI0, SGI313979, SGI313980, SGI313981, SGI313982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73436[] = {SGI313985, SGI0, SGI313986, SGI313987, SGI313988, SGI313989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313991, SGI0, SGI0, SGI0, SGI313992, SGI0, SGI0, NULL};
const Segment * const SG73437[] = {SGI313993, SGI0, SGI313994, SGI313995, SGI313996, SGI313997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI313999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73438[] = {SGI314000, SGI0, SGI314001, SGI314002, SGI314003, SGI314004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73439[] = {SGI314007, SGI0, SGI314008, SGI314009, SGI314010, SGI314011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314013, SGI0, SGI0, SGI0, SGI314014, SGI0, SGI0, NULL};
const Segment * const SG73440[] = {SGI314015, SGI0, SGI314016, SGI314017, SGI314018, SGI314019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73441[] = {SGI314022, SGI0, SGI314023, SGI314024, SGI314025, SGI314026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314028, SGI0, SGI0, SGI0, SGI0, SGI314029, SGI0, SGI0, NULL};
const Segment * const SG73442[] = {SGI314030, SGI0, SGI314031, SGI314032, SGI314033, SGI314034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314036, SGI0, SGI0, SGI0, SGI314037, SGI0, SGI0, NULL};
const Segment * const SG73443[] = {SGI314038, SGI0, SGI314039, SGI314040, SGI314041, SGI314042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314044, SGI0, SGI0, SGI0, SGI0, SGI314045, SGI0, SGI0, NULL};
const Segment * const SG73444[] = {SGI314046, SGI0, SGI314047, SGI314048, SGI314049, SGI314050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314052, SGI0, SGI0, SGI0, SGI0, SGI314053, SGI0, SGI0, NULL};
const Segment * const SG73445[] = {SGI314054, SGI0, SGI314055, SGI314056, SGI314057, SGI314058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73446[] = {SGI314061, SGI0, SGI314062, SGI314063, SGI314064, SGI314065, SGI0, SGI314066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73447[] = {SGI314069, SGI0, SGI314070, SGI314071, SGI314072, SGI314073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314075, SGI0, SGI0, SGI0, SGI314076, SGI0, SGI0, NULL};
const Segment * const SG73448[] = {SGI314077, SGI0, SGI314078, SGI314079, SGI314080, SGI314081, SGI0, SGI314082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73449[] = {SGI314085, SGI0, SGI314086, SGI314087, SGI314088, SGI314089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314091, SGI0, SGI0, SGI0, SGI0, SGI314092, SGI0, SGI0, NULL};
const Segment * const SG73450[] = {SGI314093, SGI0, SGI314094, SGI314095, SGI314096, SGI314097, SGI314098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73451[] = {SGI314100, SGI0, SGI314101, SGI314102, SGI314103, SGI314104, SGI314105, SGI314106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73452[] = {SGI314109, SGI0, SGI314110, SGI314111, SGI314112, SGI314113, SGI314114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73453[] = {SGI314117, SGI0, SGI314118, SGI314119, SGI314120, SGI314121, SGI314122, SGI314123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73454[] = {SGI314125, SGI0, SGI314126, SGI314127, SGI314128, SGI314129, SGI314130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73455[] = {SGI314132, SGI0, SGI314133, SGI314134, SGI314135, SGI314136, SGI314137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73456[] = {SGI314139, SGI0, SGI314140, SGI314141, SGI314142, SGI314143, SGI314144, SGI314145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73457[] = {SGI314147, SGI0, SGI314148, SGI314149, SGI314150, SGI314151, SGI314152, SGI314153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73458[] = {SGI314156, SGI0, SGI314157, SGI314158, SGI314159, SGI314160, SGI314161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314163, SGI0, SGI0, NULL};
const Segment * const SG73459[] = {SGI314164, SGI0, SGI314165, SGI314166, SGI314167, SGI314168, SGI314169, SGI314170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73460[] = {SGI314173, SGI0, SGI314174, SGI314175, SGI314176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73461[] = {SGI314177, SGI0, SGI314178, SGI314179, SGI314180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314181, SGI0, SGI0, NULL};
const Segment * const SG73462[] = {SGI314182, SGI0, SGI314183, SGI314184, SGI314185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314186, SGI0, SGI0, NULL};
const Segment * const SG73463[] = {SGI314187, SGI0, SGI314188, SGI314189, SGI314190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73464[] = {SGI314191, SGI0, SGI314192, SGI314193, SGI314194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314195, SGI0, SGI0, NULL};
const Segment * const SG73465[] = {SGI314196, SGI0, SGI314197, SGI314198, SGI314199, SGI0, SGI0, SGI314200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314202, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73466[] = {SGI314203, SGI0, SGI314204, SGI314205, SGI314206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314207, SGI0, SGI0, NULL};
const Segment * const SG73467[] = {SGI314208, SGI0, SGI314209, SGI314210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314211, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73468[] = {SGI314212, SGI0, SGI314213, SGI314214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73469[] = {SGI314215, SGI0, SGI314216, SGI314217, SGI314218, SGI314219, SGI314220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73470[] = {SGI314222, SGI0, SGI314223, SGI314224, SGI314225, SGI314226, SGI314227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73471[] = {SGI314229, SGI0, SGI314230, SGI314231, SGI314232, SGI314233, SGI314234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73472[] = {SGI314236, SGI0, SGI314237, SGI314238, SGI314239, SGI314240, SGI314241, SGI314242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73473[] = {SGI314244, SGI0, SGI314245, SGI314246, SGI314247, SGI314248, SGI314249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73474[] = {SGI314252, SGI0, SGI314253, SGI314254, SGI314255, SGI314256, SGI314257, SGI314258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73475[] = {SGI314260, SGI0, SGI314261, SGI314262, SGI314263, SGI314264, SGI314265, SGI314266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73476[] = {SGI314269, SGI0, SGI314270, SGI314271, SGI314272, SGI314273, SGI314274, SGI314275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73477[] = {SGI314278, SGI0, SGI314279, SGI314280, SGI314281, SGI314282, SGI314283, SGI314284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73478[] = {SGI314287, SGI0, SGI314288, SGI314289, SGI314290, SGI314291, SGI314292, SGI314293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314295, SGI0, SGI0, NULL};
const Segment * const SG73479[] = {SGI314296, SGI0, SGI314297, SGI314298, SGI314299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73480[] = {SGI314300, SGI0, SGI314301, SGI314302, SGI314303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314304, SGI0, SGI0, NULL};
const Segment * const SG73481[] = {SGI314305, SGI0, SGI314306, SGI314307, SGI314308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73482[] = {SGI314309, SGI0, SGI314310, SGI314311, SGI314312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314313, SGI0, SGI0, NULL};
const Segment * const SG73483[] = {SGI314314, SGI0, SGI314315, SGI314316, SGI314317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314318, SGI0, SGI0, NULL};
const Segment * const SG73484[] = {SGI314319, SGI0, SGI314320, SGI314321, SGI314322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314323, SGI0, SGI0, NULL};
const Segment * const SG73485[] = {SGI314324, SGI0, SGI314325, SGI314326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314327, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73486[] = {SGI314328, SGI0, SGI314329, SGI314330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73487[] = {SGI314331, SGI0, SGI314332, SGI314333, SGI314334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73488[] = {SGI314335, SGI0, SGI314336, SGI314337, SGI314338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73489[] = {SGI314339, SGI0, SGI314340, SGI314341, SGI314342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73490[] = {SGI314343, SGI0, SGI314344, SGI314345, SGI314346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73491[] = {SGI314347, SGI0, SGI314348, SGI0, SGI314349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73492[] = {SGI314350, SGI0, SGI314351, SGI314352, SGI314353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73493[] = {SGI314354, SGI0, SGI314355, SGI314356, SGI314357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73494[] = {SGI314358, SGI0, SGI314359, SGI0, SGI314360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73495[] = {SGI314361, SGI0, SGI314362, SGI314363, SGI314364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73496[] = {SGI314365, SGI0, SGI314366, SGI314367, SGI314368, SGI0, SGI0, SGI0, SGI0, SGI314369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73497[] = {SGI314370, SGI0, SGI314371, SGI314372, SGI314373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73498[] = {SGI314374, SGI0, SGI314374, SGI314375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73499[] = {SGI314376, SGI0, SGI314376, SGI314377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314378, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73500[] = {SGI314379, SGI0, SGI314380, SGI314381, SGI314382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73501[] = {SGI0, SGI314384, NULL};
const Segment * const SG73502[] = {SGI314385, SGI0, SGI314386, SGI314387, SGI314388, SGI314389, SGI314390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73503[] = {SGI314392, SGI0, SGI314393, SGI314394, SGI314395, SGI314396, SGI314397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73504[] = {SGI314400, SGI0, SGI314401, SGI314402, SGI314403, SGI314404, SGI314405, SGI314406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73505[] = {SGI314408, SGI0, SGI314409, SGI314410, SGI314411, SGI314412, SGI314413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73506[] = {SGI314415, SGI0, SGI314416, SGI314417, SGI314418, SGI314419, SGI314420, SGI314421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73507[] = {SGI314423, SGI0, SGI314424, SGI314425, SGI314426, SGI314427, SGI314428, SGI314429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73508[] = {SGI314432, SGI0, SGI314433, SGI314434, SGI314435, SGI314436, SGI314437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73509[] = {SGI314439, SGI0, SGI314440, SGI314441, SGI314442, SGI314443, SGI314444, SGI314445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73510[] = {SGI314448, SGI0, SGI314449, SGI314450, SGI314451, SGI314452, SGI314453, SGI314454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73511[] = {SGI314457, SGI0, SGI314458, SGI314459, SGI314460, SGI314461, SGI314462, SGI314463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314465, SGI0, SGI0, NULL};
const Segment * const SG73512[] = {SGI314466, SGI0, SGI314467, SGI314468, SGI314469, SGI314470, SGI314471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73513[] = {SGI314473, SGI0, SGI314474, SGI314475, SGI314476, SGI314477, SGI314478, SGI314479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73514[] = {SGI314481, SGI0, SGI314482, SGI314483, SGI314484, SGI314485, SGI314486, SGI314487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73515[] = {SGI314489, SGI0, SGI314490, SGI314491, SGI314492, SGI314493, SGI314494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73516[] = {SGI314496, SGI0, SGI314497, SGI314498, SGI314499, SGI314500, SGI314501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73517[] = {SGI314504, SGI0, SGI314505, SGI314506, SGI314507, SGI314508, SGI314509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73518[] = {SGI314511, SGI0, SGI314512, SGI314513, SGI314514, SGI314515, SGI314516, SGI314517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73519[] = {SGI314520, SGI0, SGI314521, SGI314522, SGI314523, SGI314524, SGI314525, SGI314526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73520[] = {SGI314529, SGI0, SGI314530, SGI314531, SGI314532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73521[] = {SGI314533, SGI0, SGI314534, SGI314535, SGI314536, SGI314537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73522[] = {SGI314538, SGI0, SGI314539, SGI314540, SGI314541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73523[] = {SGI314542, SGI0, SGI314543, SGI314544, SGI314545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314546, SGI0, SGI0, NULL};
const Segment * const SG73524[] = {SGI314547, SGI0, SGI314548, SGI314549, SGI314550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73525[] = {SGI314551, SGI0, SGI314552, SGI314553, SGI314554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73526[] = {SGI314555, NULL};
const Segment * const SG73527[] = {SGI314556, NULL};
const Segment * const SG73528[] = {SGI314557, NULL};
const Segment * const SG73529[] = {SGI314558, SGI314559, SGI0, SGI0, SGI0, SGI314560, SGI0, NULL};
const Segment * const SG73530[] = {SGI314561, SGI314561, SGI0, SGI0, SGI0, SGI314562, SGI0, NULL};
const Segment * const SG73531[] = {SGI314563, SGI0, SGI314564, SGI314565, SGI314566, SGI314567, SGI314568, SGI314569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73532[] = {SGI314571, SGI0, SGI314572, SGI314573, SGI314574, SGI314575, SGI314576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73533[] = {SGI314579, SGI0, SGI314580, SGI314581, SGI314582, SGI314583, SGI314584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73534[] = {SGI314586, SGI0, SGI314587, SGI314588, SGI314589, SGI314590, SGI314591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73535[] = {SGI314594, SGI0, SGI314595, SGI314596, SGI314597, SGI314598, SGI314599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73536[] = {SGI314601, SGI0, SGI314602, SGI314603, SGI314604, SGI314605, SGI314606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314608, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73537[] = {SGI314609, SGI0, SGI314610, SGI314611, SGI314612, SGI314613, SGI314614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73538[] = {SGI314616, SGI0, SGI314617, SGI314618, SGI314619, SGI314620, SGI314621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73539[] = {SGI314624, SGI0, SGI314625, SGI314626, SGI314627, SGI314628, SGI314629, SGI314630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73540[] = {SGI314632, SGI0, SGI314633, SGI314634, SGI314635, SGI314636, SGI314637, SGI314638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73541[] = {SGI314641, SGI0, SGI314642, SGI314643, SGI314644, SGI314645, SGI314646, SGI314647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314650, SGI0, SGI0, NULL};
const Segment * const SG73542[] = {SGI314651, SGI0, SGI314652, SGI314653, SGI314654, SGI314655, SGI314656, SGI314657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314659, SGI0, SGI0, NULL};
const Segment * const SG73543[] = {SGI314660, SGI0, SGI314661, SGI314662, SGI314663, SGI314664, SGI314665, SGI314666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314668, SGI314669, SGI0, SGI0, NULL};
const Segment * const SG73544[] = {SGI314670, SGI0, SGI314671, SGI314672, SGI314673, SGI314674, SGI314675, SGI314676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314677, SGI0, SGI0, SGI0, SGI314678, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73545[] = {SGI314679, SGI0, SGI314680, SGI314681, SGI314682, SGI314683, SGI314684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73546[] = {SGI314687, SGI0, SGI314688, SGI314689, SGI314690, SGI314691, SGI314692, SGI314693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314695, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73547[] = {SGI314696, SGI0, SGI314697, SGI314698, SGI314699, SGI314700, SGI314701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73548[] = {SGI314703, SGI0, SGI314704, SGI314705, SGI314706, SGI314707, SGI314708, SGI314709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73549[] = {SGI314712, SGI0, SGI314713, SGI314714, SGI314715, SGI314716, SGI314717, SGI314718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314720, SGI0, SGI0, NULL};
const Segment * const SG73550[] = {SGI314721, SGI0, SGI314722, SGI314723, SGI314724, SGI314725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73551[] = {SGI314726, SGI0, SGI314727, SGI314728, SGI314729, SGI314730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314731, SGI0, SGI0, NULL};
const Segment * const SG73552[] = {SGI314732, SGI0, SGI314733, SGI314734, SGI314735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73553[] = {SGI314736, SGI0, SGI314737, SGI314738, SGI314739, SGI314740, SGI0, SGI314741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73554[] = {SGI314742, SGI0, SGI314743, SGI314744, SGI314745, SGI314746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73555[] = {SGI314747, SGI0, SGI314748, SGI314749, SGI314750, SGI314751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314752, SGI0, SGI0, NULL};
const Segment * const SG73556[] = {SGI314753, SGI0, SGI314754, SGI314755, SGI314756, SGI314757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73557[] = {SGI314759, SGI0, SGI314760, SGI314761, SGI314762, SGI314763, SGI0, SGI314764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73558[] = {SGI314766, SGI0, SGI314767, SGI314768, SGI314769, SGI314770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73559[] = {SGI314771, SGI0, SGI314772, SGI314773, SGI314774, SGI314775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314776, SGI0, SGI0, NULL};
const Segment * const SG73560[] = {SGI314777, SGI0, SGI314778, SGI314779, SGI314780, SGI314781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314782, SGI0, SGI0, NULL};
const Segment * const SG73561[] = {SGI314783, SGI314783, SGI0, SGI0, SGI0, SGI314784, SGI0, NULL};
const Segment * const SG73562[] = {SGI314785, SGI0, SGI314786, SGI314787, SGI314788, SGI314789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73563[] = {SGI314791, SGI0, SGI314792, SGI314793, SGI314794, SGI314795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314796, SGI0, SGI0, SGI0, SGI314797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73564[] = {SGI314798, SGI0, SGI314799, SGI314800, SGI314801, SGI314802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314804, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73565[] = {SGI314805, SGI0, SGI314806, SGI314807, SGI314808, SGI314809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73566[] = {SGI314811, SGI0, SGI314812, SGI314813, SGI314814, SGI314815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73567[] = {SGI314817, SGI0, SGI314818, SGI314819, SGI314820, SGI314821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314823, SGI0, SGI0, NULL};
const Segment * const SG73568[] = {SGI314824, SGI0, SGI314825, SGI314826, SGI314827, SGI314828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73569[] = {SGI314830, SGI0, SGI314831, SGI314832, SGI314833, SGI314834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73570[] = {SGI314836, SGI0, SGI314837, SGI314838, SGI314839, SGI314840, SGI0, SGI314841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314842, SGI0, SGI0, SGI0, SGI314843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73571[] = {SGI314844, SGI0, SGI314845, SGI314846, SGI314847, SGI314848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314850, SGI0, SGI0, NULL};
const Segment * const SG73572[] = {SGI314851, SGI0, SGI314852, SGI314853, SGI314854, SGI314855, SGI0, SGI314856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73573[] = {SGI314858, SGI0, SGI314859, SGI314860, SGI314861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73574[] = {SGI314862, SGI0, SGI314863, SGI314864, SGI314865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73575[] = {SGI314866, SGI0, SGI314867, SGI314868, SGI314869, SGI314870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73576[] = {SGI314871, SGI0, SGI314872, SGI314873, SGI314874, SGI314875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73577[] = {SGI314876, SGI0, SGI314877, SGI314878, SGI314879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73578[] = {SGI314880, SGI0, SGI314881, SGI314882, SGI314883, SGI314884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73579[] = {SGI314885, SGI0, SGI314886, SGI314887, SGI314888, SGI314889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314890, SGI0, SGI0, NULL};
const Segment * const SG73580[] = {SGI314891, SGI0, SGI314892, SGI314893, SGI314894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73581[] = {SGI314895, SGI0, SGI314896, SGI314897, SGI314898, SGI314899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73582[] = {SGI314900, SGI0, SGI0, SGI0, SGI314901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73583[] = {SGI314902, SGI0, SGI314902, SGI314903, SGI314904, SGI314905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73584[] = {SGI0, SGI314906, NULL};
const Segment * const SG73585[] = {SGI314907, SGI0, SGI314908, SGI314909, SGI314910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73586[] = {SGI314911, SGI0, SGI314912, SGI0, SGI314913, SGI314914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73587[] = {SGI314915, SGI0, SGI314916, SGI314917, SGI314918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73588[] = {SGI314919, SGI0, SGI314920, SGI314921, SGI314922, SGI314923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73589[] = {SGI314924, SGI0, SGI314925, SGI314926, SGI314927, SGI314928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314929, SGI0, SGI0, NULL};
const Segment * const SG73590[] = {SGI314930, SGI0, SGI314931, SGI314932, SGI314933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73591[] = {SGI314934, SGI0, SGI314935, SGI314936, SGI314937, SGI314938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73592[] = {SGI314939, SGI0, SGI314940, SGI314941, SGI314942, SGI314943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73593[] = {SGI314944, SGI0, SGI314945, SGI314946, SGI314947, SGI314948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314949, SGI0, SGI0, NULL};
const Segment * const SG73594[] = {SGI314950, SGI0, SGI314951, SGI314952, SGI314953, SGI314954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314955, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73595[] = {SGI314956, SGI0, SGI314957, SGI314958, SGI314959, SGI314960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI314961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73596[] = {SGI314962, SGI0, SGI314963, SGI314964, SGI314965, SGI314966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73597[] = {SGI314967, SGI0, SGI314968, SGI314969, SGI314970, SGI314971, SGI0, SGI314972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73598[] = {SGI314973, SGI0, SGI314974, SGI314975, SGI314976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73599[] = {SGI314977, NULL};
const Segment * const SG73600[] = {SGI314978, NULL};
const Segment * const SG73601[] = {SGI314979, NULL};
const Segment * const SG73602[] = {SGI314980, NULL};
const Segment * const SG73603[] = {SGI314981, NULL};
const Segment * const SG73604[] = {SGI314982, NULL};
const Segment * const SG73605[] = {SGI314983, NULL};
const Segment * const SG73606[] = {SGI314984, NULL};
const Segment * const SG73607[] = {SGI314985, NULL};
const Segment * const SG73608[] = {SGI314986, NULL};
const Segment * const SG73609[] = {SGI314987, NULL};
const Segment * const SG73610[] = {SGI314988, NULL};
const Segment * const SG73611[] = {SGI314989, NULL};
const Segment * const SG73612[] = {SGI314990, NULL};
const Segment * const SG73613[] = {SGI314991, NULL};
const Segment * const SG73614[] = {SGI314992, NULL};
const Segment * const SG73615[] = {SGI314993, NULL};
const Segment * const SG73616[] = {SGI314994, NULL};
const Segment * const SG73617[] = {SGI314995, NULL};
const Segment * const SG73618[] = {SGI314996, NULL};
const Segment * const SG73619[] = {SGI314997, NULL};
const Segment * const SG73620[] = {SGI314998, NULL};
const Segment * const SG73621[] = {SGI314999, NULL};
const Segment * const SG73622[] = {SGI315000, NULL};
const Segment * const SG73623[] = {SGI315001, NULL};
const Segment * const SG73624[] = {SGI315002, NULL};
const Segment * const SG73625[] = {SGI315003, NULL};
const Segment * const SG73626[] = {SGI315004, NULL};
const Segment * const SG73627[] = {SGI315005, NULL};
const Segment * const SG73628[] = {SGI315006, NULL};
const Segment * const SG73629[] = {SGI315007, NULL};
const Segment * const SG73630[] = {SGI315008, NULL};
const Segment * const SG73631[] = {SGI315009, NULL};
const Segment * const SG73632[] = {SGI315010, NULL};
const Segment * const SG73633[] = {SGI315011, NULL};
const Segment * const SG73634[] = {SGI315012, NULL};
const Segment * const SG73635[] = {SGI315013, NULL};
const Segment * const SG73636[] = {SGI315014, NULL};
const Segment * const SG73637[] = {SGI315015, NULL};
const Segment * const SG73638[] = {SGI315016, NULL};
const Segment * const SG73639[] = {SGI315017, NULL};
const Segment * const SG73640[] = {SGI315018, NULL};
const Segment * const SG73641[] = {SGI315019, NULL};
const Segment * const SG73642[] = {SGI315020, NULL};
const Segment * const SG73643[] = {SGI315021, NULL};
const Segment * const SG73644[] = {SGI315022, NULL};
const Segment * const SG73645[] = {SGI315023, NULL};
const Segment * const SG73646[] = {SGI315024, NULL};
const Segment * const SG73647[] = {SGI315025, NULL};
const Segment * const SG73648[] = {SGI315026, NULL};
const Segment * const SG73649[] = {SGI315027, NULL};
const Segment * const SG73650[] = {SGI315028, NULL};
const Segment * const SG73651[] = {SGI315029, NULL};
const Segment * const SG73652[] = {SGI315030, NULL};
const Segment * const SG73653[] = {SGI315031, NULL};
const Segment * const SG73654[] = {SGI315032, NULL};
const Segment * const SG73655[] = {SGI315033, NULL};
const Segment * const SG73656[] = {SGI315034, NULL};
const Segment * const SG73657[] = {SGI315035, NULL};
const Segment * const SG73658[] = {SGI315036, NULL};
const Segment * const SG73659[] = {SGI315037, NULL};
const Segment * const SG73660[] = {SGI315038, NULL};
const Segment * const SG73661[] = {SGI315039, NULL};
const Segment * const SG73662[] = {SGI315040, NULL};
const Segment * const SG73663[] = {SGI315041, NULL};
const Segment * const SG73664[] = {SGI315042, NULL};
const Segment * const SG73665[] = {SGI315043, NULL};
const Segment * const SG73666[] = {SGI315044, NULL};
const Segment * const SG73667[] = {SGI315045, NULL};
const Segment * const SG73668[] = {SGI315046, NULL};
const Segment * const SG73669[] = {SGI315047, NULL};
const Segment * const SG73670[] = {SGI315048, NULL};
const Segment * const SG73671[] = {SGI315049, NULL};
const Segment * const SG73672[] = {SGI315050, NULL};
const Segment * const SG73673[] = {SGI315051, NULL};
const Segment * const SG73674[] = {SGI315052, NULL};
const Segment * const SG73675[] = {SGI315053, NULL};
const Segment * const SG73676[] = {SGI315054, NULL};
const Segment * const SG73677[] = {SGI315055, NULL};
const Segment * const SG73678[] = {SGI315056, NULL};
const Segment * const SG73679[] = {SGI315057, NULL};
const Segment * const SG73680[] = {SGI315058, NULL};
const Segment * const SG73681[] = {SGI315059, NULL};
const Segment * const SG73682[] = {SGI315060, SGI0, SGI315061, SGI315062, SGI315063, SGI315064, SGI0, SGI315065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73683[] = {SGI315067, SGI0, SGI315068, SGI315069, SGI315070, SGI315071, SGI0, SGI315072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315073, SGI0, SGI0, SGI0, SGI315074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73684[] = {SGI315075, SGI0, SGI315076, SGI315077, SGI315078, SGI315079, SGI0, SGI315080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315082, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73685[] = {SGI315083, SGI0, SGI315084, SGI315085, SGI315086, SGI315087, SGI0, SGI315088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73686[] = {SGI315090, SGI0, SGI315091, SGI315092, SGI315093, SGI315094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315095, SGI0, SGI0, SGI0, SGI315096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73687[] = {SGI315097, SGI0, SGI315098, SGI315099, SGI315100, SGI315101, SGI0, SGI315102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315103, SGI0, SGI0, SGI0, SGI315104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73688[] = {SGI315105, SGI0, SGI315106, SGI315107, SGI315108, SGI315109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73689[] = {SGI315111, SGI0, SGI315112, SGI315113, SGI315114, SGI315115, SGI0, SGI315116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315118, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73690[] = {SGI315119, SGI0, SGI315120, SGI315121, SGI315122, SGI315123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73691[] = {SGI315125, SGI0, SGI315126, SGI315127, SGI315128, SGI315129, SGI0, SGI315130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315132, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73692[] = {SGI315133, SGI0, SGI315134, SGI315135, SGI315136, SGI315137, SGI0, SGI315138, SGI0, SGI0, SGI315139, SGI0, SGI0, SGI315140, SGI0, SGI0, SGI0, SGI315141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73693[] = {SGI315142, SGI0, SGI315143, SGI315144, SGI315145, SGI315146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73694[] = {SGI315148, SGI0, SGI315149, SGI315150, SGI315151, SGI315152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315153, SGI0, SGI0, SGI0, SGI315154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73695[] = {SGI315155, SGI0, SGI315156, SGI315157, SGI315158, SGI315159, SGI0, SGI315160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315162, SGI0, SGI315163, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73696[] = {SGI315164, SGI0, SGI315165, SGI315166, SGI315167, SGI315168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73697[] = {SGI315170, SGI0, SGI315171, SGI315172, SGI315173, SGI315174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73698[] = {SGI315176, SGI0, SGI315177, SGI315178, SGI315179, SGI315180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315182, SGI0, SGI0, NULL};
const Segment * const SG73699[] = {SGI315183, SGI0, SGI315184, SGI315185, SGI315186, SGI315187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73700[] = {SGI315189, SGI0, SGI315190, SGI315191, SGI315192, SGI315193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73701[] = {SGI315195, SGI0, SGI315196, SGI315197, SGI315198, SGI315199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73702[] = {SGI315201, SGI0, SGI315202, SGI315203, SGI315204, SGI315205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315207, SGI0, SGI0, NULL};
const Segment * const SG73703[] = {SGI315208, SGI0, SGI315209, SGI315210, SGI315211, SGI315212, SGI0, SGI315213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73704[] = {SGI315215, SGI0, SGI315216, SGI315217, SGI315218, SGI315219, SGI0, SGI315220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315222, SGI0, SGI0, NULL};
const Segment * const SG73705[] = {SGI315223, SGI0, SGI315224, SGI315225, SGI315226, SGI315227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73706[] = {SGI315229, SGI0, SGI315230, SGI315231, SGI315232, SGI315233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73707[] = {SGI315235, SGI0, SGI315236, SGI315237, SGI315238, SGI315239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315241, SGI0, SGI0, NULL};
const Segment * const SG73708[] = {SGI315242, SGI0, SGI315243, SGI315244, SGI315245, SGI315246, SGI0, SGI315247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315248, SGI0, SGI0, SGI0, SGI315249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73709[] = {SGI315250, SGI0, SGI315251, SGI315252, SGI315253, SGI315254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73710[] = {SGI315256, SGI0, SGI315257, SGI315258, SGI315259, SGI315260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315262, SGI0, SGI0, NULL};
const Segment * const SG73711[] = {SGI315263, SGI0, SGI315264, SGI315265, SGI315266, SGI315267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315269, SGI0, SGI0, NULL};
const Segment * const SG73712[] = {SGI315270, SGI0, SGI315271, SGI315272, SGI315273, SGI315274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315276, SGI0, SGI0, NULL};
const Segment * const SG73713[] = {SGI315277, SGI0, SGI315278, SGI315279, SGI315280, SGI315281, SGI0, SGI315282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73714[] = {SGI315284, SGI0, SGI315285, SGI315286, SGI315287, SGI315288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73715[] = {SGI315290, SGI315291, NULL};
const Segment * const SG73716[] = {SGI315292, SGI0, SGI315293, SGI315294, SGI315295, SGI315296, SGI315297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73717[] = {SGI315299, SGI0, SGI315300, SGI315301, SGI315302, SGI315303, SGI315304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73718[] = {SGI315307, SGI0, SGI315308, SGI315309, SGI315310, SGI315311, SGI315312, SGI315313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73719[] = {SGI315315, SGI0, SGI315316, SGI315317, SGI315318, SGI315319, SGI315320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73720[] = {SGI315322, SGI0, SGI315323, SGI315324, SGI315325, SGI315326, SGI315327, SGI315328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73721[] = {SGI315331, SGI0, SGI315332, SGI315333, SGI315334, SGI315335, SGI315336, SGI315337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73722[] = {SGI315340, SGI0, SGI315341, SGI315342, SGI315343, SGI315344, SGI315345, SGI315346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73723[] = {SGI315348, SGI0, SGI315349, SGI315350, SGI315351, SGI315352, SGI315353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73724[] = {SGI315355, SGI0, SGI315356, SGI315357, SGI315358, SGI315359, SGI315360, SGI315361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315363, SGI0, SGI0, NULL};
const Segment * const SG73725[] = {SGI315364, NULL};
const Segment * const SG73726[] = {SGI315365, NULL};
const Segment * const SG73727[] = {SGI315366, NULL};
const Segment * const SG73728[] = {SGI315367, SGI0, SGI315368, SGI315369, SGI315370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73729[] = {SGI315371, SGI0, SGI315372, SGI315373, SGI315374, SGI315375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73730[] = {SGI315376, SGI0, SGI315377, SGI315378, SGI315379, SGI315380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73731[] = {SGI315381, SGI0, SGI0, SGI0, SGI315382, SGI315383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73732[] = {SGI315385, SGI0, SGI0, SGI0, SGI315386, SGI315387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73733[] = {SGI315388, SGI0, SGI0, SGI0, SGI315389, SGI315390, SGI0, SGI0, SGI0, SGI0, SGI315391, SGI0, SGI0, SGI315392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73734[] = {SGI315393, SGI0, SGI0, SGI0, SGI315394, SGI315395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315397, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73735[] = {SGI315398, SGI0, SGI0, SGI0, SGI315399, SGI315400, SGI0, SGI315401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73736[] = {SGI315403, SGI0, SGI0, SGI0, SGI315404, SGI315405, SGI0, SGI315406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315408, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73737[] = {SGI315409, SGI0, SGI0, SGI0, SGI315410, SGI315411, SGI0, SGI315412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73738[] = {SGI315413, SGI0, SGI0, SGI0, SGI315414, SGI315415, SGI0, SGI0, SGI0, SGI0, SGI315416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73739[] = {SGI315417, SGI0, SGI0, SGI0, SGI315418, SGI315419, SGI0, SGI315420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315421, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73740[] = {SGI315422, SGI0, SGI0, SGI0, SGI315423, SGI315424, SGI0, SGI315425, SGI0, SGI0, SGI315426, SGI0, SGI0, SGI315427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73741[] = {SGI315428, SGI0, SGI0, SGI0, SGI315429, SGI315430, SGI0, SGI315431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73742[] = {SGI315432, SGI0, SGI0, SGI0, SGI315433, SGI315434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73743[] = {SGI315436, SGI0, SGI0, SGI0, SGI315437, SGI315438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315439, SGI0, SGI0, SGI0, SGI315440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73744[] = {SGI315441, SGI0, SGI0, SGI0, SGI315442, SGI315443, SGI0, SGI315444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315445, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73745[] = {SGI315446, SGI0, SGI0, SGI0, SGI315447, SGI315448, SGI0, SGI315449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73746[] = {SGI315451, SGI0, SGI0, SGI0, SGI315452, SGI315453, SGI0, SGI0, SGI0, SGI0, SGI315454, SGI0, SGI0, SGI315455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315456, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73747[] = {SGI315457, SGI0, SGI0, SGI0, SGI315458, SGI315459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315460, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73748[] = {SGI315461, SGI0, SGI0, SGI0, SGI315462, SGI315463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73749[] = {SGI315465, SGI0, SGI0, SGI0, SGI315466, SGI315467, SGI0, SGI315468, SGI0, SGI0, SGI315469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73750[] = {SGI315470, SGI0, SGI315471, SGI315472, SGI315473, SGI315474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315475, SGI0, SGI0, SGI315476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73751[] = {SGI315477, SGI0, SGI315478, SGI315479, SGI315480, SGI315481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73752[] = {SGI315483, SGI0, SGI315484, SGI315485, SGI315486, SGI315487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315488, SGI0, SGI0, SGI315489, SGI315490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73753[] = {SGI315491, SGI0, SGI315492, SGI315493, SGI315494, SGI315495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315496, SGI315497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73754[] = {SGI315498, SGI0, SGI315499, SGI315500, SGI315501, SGI315502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315503, SGI0, SGI0, SGI315504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73755[] = {SGI315505, SGI0, SGI315506, SGI315507, SGI315508, SGI315509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315510, SGI0, SGI0, SGI315511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73756[] = {SGI315512, SGI0, SGI315513, SGI315514, SGI315515, SGI315516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73757[] = {SGI315518, SGI0, SGI315519, SGI315520, SGI315521, SGI315522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315523, SGI0, SGI0, SGI315524, SGI315525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73758[] = {SGI315526, SGI0, SGI315527, SGI315528, SGI315529, SGI315530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315531, SGI315532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73759[] = {SGI315533, SGI0, SGI315534, SGI315535, SGI315536, SGI315537, SGI0, SGI315538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73760[] = {SGI315540, SGI0, SGI0, SGI0, SGI315541, SGI315542, SGI0, SGI315543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73761[] = {SGI315544, SGI0, SGI0, SGI0, SGI315545, SGI315546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73762[] = {SGI315547, SGI0, SGI0, SGI0, SGI315548, SGI315549, SGI0, SGI315550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73763[] = {SGI315552, SGI0, SGI315553, SGI0, SGI315554, SGI315555, SGI0, SGI315556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315557, SGI0, SGI0, NULL};
const Segment * const SG73764[] = {SGI315558, SGI0, SGI0, SGI0, SGI315559, SGI315560, SGI0, SGI315561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73765[] = {SGI315563, SGI0, SGI0, SGI0, SGI315564, SGI315565, SGI0, SGI315566, SGI0, SGI0, SGI315567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73766[] = {SGI315568, SGI0, SGI0, SGI0, SGI315569, SGI315570, SGI0, SGI0, SGI0, SGI0, SGI315571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73767[] = {SGI315572, SGI0, SGI0, SGI0, SGI315573, SGI315574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73768[] = {SGI315576, SGI0, SGI0, SGI0, SGI315577, SGI315578, SGI0, SGI315579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315580, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73769[] = {SGI315581, SGI0, SGI315582, SGI315583, SGI315584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73770[] = {SGI315585, SGI0, SGI315586, SGI315587, SGI315588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73771[] = {SGI315589, SGI315590, NULL};
const Segment * const SG73772[] = {SGI0, SGI315591, NULL};
const Segment * const SG73773[] = {SGI315592, SGI0, SGI315593, SGI315594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73774[] = {SGI315595, SGI0, SGI315596, SGI315597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73775[] = {SGI315598, SGI0, SGI315599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73776[] = {SGI0, SGI0, SGI315600, SGI315601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73777[] = {SGI315602, SGI0, SGI315603, SGI315604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73778[] = {SGI315605, SGI0, SGI315606, SGI315607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73779[] = {SGI315608, SGI0, SGI315609, SGI315610, SGI315611, SGI315612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73780[] = {SGI315614, SGI0, SGI315615, SGI315616, SGI315617, SGI315618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315619, SGI0, SGI0, SGI0, SGI315620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73781[] = {SGI315621, SGI0, SGI315622, SGI315623, SGI315624, SGI315625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315627, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73782[] = {SGI315628, SGI315629, SGI0, SGI0, SGI0, SGI315630, SGI0, NULL};
const Segment * const SG73783[] = {SGI315631, SGI315631, SGI0, SGI0, SGI0, SGI315632, SGI0, NULL};
const Segment * const SG73784[] = {SGI315633, SGI315634, SGI0, SGI0, SGI0, SGI315635, SGI0, NULL};
const Segment * const SG73785[] = {SGI315636, SGI0, SGI315637, SGI315638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315639, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73786[] = {SGI315640, SGI0, SGI315641, SGI315642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315643, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73787[] = {SGI315644, SGI0, SGI315645, SGI315646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73788[] = {SGI315647, SGI0, SGI315648, SGI315649, SGI315650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73789[] = {SGI315651, SGI0, SGI315652, SGI315653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73790[] = {SGI315654, SGI0, SGI315655, SGI315656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315657, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73791[] = {SGI315658, SGI0, SGI315659, SGI315660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315661, SGI0, SGI0, NULL};
const Segment * const SG73792[] = {SGI315662, SGI0, SGI315663, SGI315664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315666, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73793[] = {SGI315667, SGI0, SGI315668, SGI315669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315670, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73794[] = {SGI315671, SGI0, SGI315672, SGI315673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73795[] = {SGI315675, SGI0, SGI315676, SGI315677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315678, SGI315679, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73796[] = {SGI315680, SGI0, SGI315681, SGI315682, SGI315683, SGI315684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315685, SGI315686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73797[] = {SGI315687, SGI0, SGI315688, SGI315689, SGI315690, SGI315691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315693, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73798[] = {SGI315694, SGI0, SGI315695, SGI315696, SGI315697, SGI315698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73799[] = {SGI315700, SGI0, SGI315701, SGI315702, SGI315703, SGI315704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315705, SGI315706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315707, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73800[] = {SGI315708, SGI0, SGI315709, SGI315710, SGI315711, SGI315712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315714, SGI0, SGI315715, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73801[] = {SGI315716, SGI0, SGI315717, SGI315718, SGI315719, SGI315720, SGI315721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73802[] = {SGI315723, SGI0, SGI315724, SGI315725, SGI315726, SGI315727, SGI315728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73803[] = {SGI315730, SGI0, SGI315731, SGI315732, SGI315733, SGI315734, SGI315735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315737, SGI0, SGI0, NULL};
const Segment * const SG73804[] = {SGI315738, SGI0, SGI315739, SGI315740, SGI315741, SGI0, SGI315742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73805[] = {SGI315744, SGI0, SGI315745, SGI315746, SGI315747, SGI315748, SGI315749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315751, SGI0, SGI0, NULL};
const Segment * const SG73806[] = {SGI315752, SGI0, SGI315753, SGI315754, SGI315755, SGI315756, SGI315757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73807[] = {SGI315759, SGI0, SGI315760, SGI315761, SGI315762, SGI315763, SGI315764, SGI0, SGI0, SGI0, SGI315765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73808[] = {SGI315767, SGI0, SGI315768, SGI315769, SGI315770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73809[] = {SGI315771, SGI0, SGI315772, SGI315773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315774, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73810[] = {SGI315775, SGI0, SGI315775, SGI315776, SGI315777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73811[] = {SGI315778, SGI0, SGI315779, SGI315780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73812[] = {SGI315781, SGI0, SGI315781, SGI315782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315783, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73813[] = {SGI315784, SGI0, SGI315785, SGI315786, SGI315787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73814[] = {SGI315788, SGI0, SGI315789, SGI315790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73815[] = {SGI315791, SGI0, SGI315792, SGI315793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73816[] = {SGI315795, SGI0, SGI315796, SGI315797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315798, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73817[] = {SGI315799, SGI0, SGI315800, SGI315801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315802, SGI0, SGI0, NULL};
const Segment * const SG73818[] = {SGI315803, SGI0, SGI315804, SGI315805, SGI315806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73819[] = {SGI315807, SGI0, SGI315808, SGI315809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315810, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73820[] = {SGI315811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73821[] = {SGI315812, SGI0, SGI315813, SGI315814, SGI315815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73822[] = {SGI315816, SGI0, SGI315817, SGI315818, SGI315819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73823[] = {SGI315820, SGI0, SGI315821, SGI0, SGI315822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73824[] = {SGI315823, SGI0, SGI315824, SGI315825, SGI315826, SGI315827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315828, SGI0, SGI0, SGI315829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73825[] = {SGI315830, SGI0, SGI315831, SGI315832, SGI315833, SGI315834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73826[] = {SGI315836, SGI0, SGI315837, SGI315838, SGI315839, SGI315840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315841, SGI0, SGI0, SGI315842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73827[] = {SGI315843, SGI0, SGI315844, SGI315845, SGI315846, SGI315847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73828[] = {SGI315849, SGI0, SGI315850, SGI315851, SGI315852, SGI315853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315854, SGI315855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73829[] = {SGI315856, SGI0, SGI315857, SGI315858, SGI315859, SGI315860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315861, SGI0, SGI0, SGI315862, SGI315863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73830[] = {SGI315864, SGI0, SGI315865, SGI315866, SGI315867, SGI315868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315869, SGI315870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73831[] = {SGI315871, NULL};
const Segment * const SG73832[] = {SGI315872, SGI0, SGI315873, SGI315874, SGI315875, SGI315876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315877, SGI0, SGI0, SGI315878, SGI315879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73833[] = {SGI315880, NULL};
const Segment * const SG73834[] = {SGI315881, SGI0, SGI315882, SGI315883, SGI315884, SGI315885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315886, SGI0, SGI0, SGI315887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73835[] = {SGI315888, SGI315889, SGI315890, SGI315891, SGI315892, SGI315893, SGI0, SGI315894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73836[] = {SGI315895, SGI315896, SGI315897, SGI315898, SGI315899, SGI315900, SGI0, SGI315901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73837[] = {SGI315902, SGI315903, SGI315904, SGI315905, SGI315906, SGI315907, SGI0, SGI315908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73838[] = {SGI315909, SGI315910, SGI315911, SGI315912, SGI315913, SGI315914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73839[] = {SGI315915, SGI315916, SGI315917, SGI315918, SGI315919, SGI315920, SGI0, SGI315921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73840[] = {SGI315923, SGI315924, SGI315925, SGI315926, SGI315927, SGI315928, SGI0, SGI315929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73841[] = {SGI315931, SGI315932, SGI315933, SGI315934, SGI315935, SGI315936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73842[] = {SGI315937, SGI315938, SGI315939, SGI315940, SGI315941, SGI315942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73843[] = {SGI315944, SGI315945, SGI315946, SGI315947, SGI315948, SGI315949, SGI0, SGI315950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315951, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73844[] = {SGI315952, SGI315953, SGI315954, SGI315955, SGI315956, SGI315957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73845[] = {SGI315959, SGI315960, SGI315961, SGI315962, SGI315963, SGI315964, SGI0, SGI315965, SGI0, SGI0, SGI315966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73846[] = {SGI315967, SGI315968, SGI315969, SGI315970, SGI315971, SGI315972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73847[] = {SGI315973, SGI315974, SGI315975, SGI315976, SGI315977, SGI315978, SGI0, SGI315979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73848[] = {SGI315980, SGI315981, SGI315982, SGI315983, SGI315984, SGI315985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73849[] = {SGI315986, SGI315987, SGI315988, SGI315989, SGI315990, SGI315991, SGI0, SGI315992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI315993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73850[] = {SGI315994, SGI315995, SGI315996, SGI315997, SGI315998, SGI315999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73851[] = {SGI316001, SGI316002, SGI316003, SGI316004, SGI316005, SGI316006, SGI0, SGI316007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73852[] = {SGI316008, SGI316009, SGI316010, SGI316011, SGI316012, SGI316013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73853[] = {SGI316014, SGI316015, SGI316016, SGI316017, SGI316018, SGI316019, SGI0, SGI316020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73854[] = {SGI316022, SGI316023, SGI316024, SGI316025, SGI316026, SGI316027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73855[] = {SGI316028, SGI316029, SGI316030, SGI316031, SGI316032, SGI316033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73856[] = {SGI316035, SGI316036, SGI316037, SGI316038, SGI316039, SGI316040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73857[] = {SGI316042, SGI316043, SGI316044, SGI316045, SGI316046, SGI316047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73858[] = {SGI316048, SGI316049, SGI316050, SGI316051, SGI316052, SGI316053, SGI0, SGI316054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316055, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73859[] = {SGI316056, SGI316057, SGI316058, SGI316059, SGI316060, SGI316061, SGI0, SGI316062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73860[] = {SGI316064, SGI316065, SGI316066, SGI316067, SGI316068, SGI316069, SGI0, SGI316070, SGI0, SGI0, SGI316071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73861[] = {SGI316072, SGI316073, SGI316074, SGI316075, SGI316076, SGI316077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316078, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73862[] = {SGI316079, SGI316080, SGI316081, SGI316082, SGI316083, SGI316084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73863[] = {SGI316086, SGI316087, SGI316088, SGI316089, SGI316090, SGI316091, SGI0, SGI316092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73864[] = {SGI316093, SGI316094, SGI316095, SGI316096, SGI316097, SGI316098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73865[] = {SGI316099, SGI316100, SGI316101, SGI316102, SGI316103, SGI316104, SGI0, SGI316105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73866[] = {SGI316106, SGI316107, SGI316108, SGI316109, SGI316110, SGI316111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73867[] = {SGI316112, SGI316113, SGI316114, SGI316115, SGI316116, SGI316117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73868[] = {SGI316119, SGI316120, SGI316121, SGI316122, SGI316123, SGI316124, SGI0, SGI316125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73869[] = {SGI316127, SGI0, SGI316128, SGI316129, SGI316130, SGI316131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73870[] = {SGI316133, SGI0, SGI316134, SGI316135, SGI316136, SGI316137, SGI0, SGI316138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73871[] = {SGI316140, SGI0, SGI316141, SGI316142, SGI316143, SGI316144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316146, SGI0, SGI0, NULL};
const Segment * const SG73872[] = {SGI316147, SGI0, SGI316148, SGI316149, SGI316150, SGI316151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73873[] = {SGI316153, SGI0, SGI316154, SGI316155, SGI316156, SGI316157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316159, SGI0, SGI0, NULL};
const Segment * const SG73874[] = {SGI316160, SGI0, SGI316161, SGI316162, SGI316163, SGI316164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73875[] = {SGI316166, SGI0, SGI316167, SGI316168, SGI316169, SGI316170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73876[] = {SGI316172, SGI0, SGI316173, SGI316174, SGI316175, SGI316176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316178, SGI0, SGI0, NULL};
const Segment * const SG73877[] = {SGI316179, SGI0, SGI316180, SGI316181, SGI316182, SGI316183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73878[] = {SGI316185, SGI0, SGI316186, SGI316187, SGI316188, SGI316189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73879[] = {SGI316191, SGI0, SGI316192, SGI316193, SGI316194, SGI316195, SGI0, SGI316196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316198, SGI0, SGI0, NULL};
const Segment * const SG73880[] = {SGI316199, SGI0, SGI316200, SGI316201, SGI316202, SGI316203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73881[] = {SGI316205, SGI0, SGI316206, SGI316207, SGI316208, SGI316209, SGI0, SGI316210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316212, SGI0, SGI0, NULL};
const Segment * const SG73882[] = {SGI316213, SGI0, SGI316214, SGI316215, SGI316216, SGI316217, SGI0, SGI316218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73883[] = {SGI316220, SGI316221, NULL};
const Segment * const SG73884[] = {SGI316222, SGI0, SGI316223, SGI0, SGI316224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73885[] = {SGI316225, SGI0, SGI316226, SGI0, SGI316227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73886[] = {SGI0, SGI0, SGI316228, SGI316229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316230, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73887[] = {SGI316231, SGI0, SGI0, SGI0, SGI316232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73888[] = {SGI316233, SGI0, SGI316234, SGI316235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73889[] = {SGI316236, SGI0, SGI316237, SGI316238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73890[] = {SGI316239, SGI0, SGI316240, SGI316241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316242, SGI316243, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73891[] = {SGI316244, SGI0, SGI316245, SGI316246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316247, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73892[] = {SGI316248, SGI0, SGI316249, SGI316250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316251, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73893[] = {SGI316252, SGI0, SGI316253, SGI316254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316256, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73894[] = {SGI316257, SGI0, SGI316258, SGI316259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316260, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73895[] = {SGI316261, SGI0, SGI316262, SGI316263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316265, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73896[] = {SGI316266, SGI0, SGI316267, SGI316268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316269, SGI0, SGI0, NULL};
const Segment * const SG73897[] = {SGI316270, SGI0, SGI316271, SGI316272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316273, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73898[] = {SGI316274, SGI0, SGI316275, SGI316276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73899[] = {SGI316278, SGI0, SGI316279, SGI316280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316281, SGI0, SGI0, NULL};
const Segment * const SG73900[] = {SGI316282, SGI0, SGI316283, SGI316284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73901[] = {SGI316286, SGI0, SGI316287, SGI316288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316289, SGI316290, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73902[] = {SGI0, SGI0, SGI316291, SGI316292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73903[] = {SGI316293, SGI0, SGI316294, SGI316295, SGI316296, SGI316297, SGI316298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316300, SGI0, SGI0, NULL};
const Segment * const SG73904[] = {SGI316301, SGI0, SGI316302, SGI316303, SGI316304, SGI316305, SGI316306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73905[] = {SGI316308, SGI0, SGI316309, SGI316310, SGI316311, SGI316312, SGI316313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73906[] = {SGI316315, SGI0, SGI316316, SGI316317, SGI316318, SGI316319, SGI316320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73907[] = {SGI316322, SGI0, SGI316323, SGI316324, SGI316325, SGI316326, SGI316327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316329, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73908[] = {SGI316330, SGI0, SGI316331, SGI316332, SGI316333, SGI316334, SGI316335, SGI316336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73909[] = {SGI316338, SGI0, SGI316339, SGI316340, SGI316341, SGI316342, SGI316343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73910[] = {SGI316346, SGI0, SGI316347, SGI316348, SGI316349, SGI316350, SGI316351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316353, SGI0, SGI0, NULL};
const Segment * const SG73911[] = {SGI316354, SGI0, SGI316355, SGI316356, SGI316357, SGI316358, SGI316359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73912[] = {SGI316361, SGI0, SGI316362, SGI316363, SGI316364, SGI316365, SGI316366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73913[] = {SGI316368, SGI0, SGI316369, SGI316370, SGI316371, SGI316372, SGI316373, SGI316374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316376, SGI0, SGI0, NULL};
const Segment * const SG73914[] = {SGI316377, SGI0, SGI316378, SGI316379, SGI316380, SGI316381, SGI316382, SGI316383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73915[] = {SGI316385, SGI316385, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73916[] = {SGI316386, SGI0, SGI316387, SGI316388, SGI316389, SGI316390, SGI316391, SGI316392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73917[] = {SGI316394, SGI0, SGI316395, SGI316396, SGI316397, SGI316398, SGI316399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316401, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73918[] = {SGI316402, SGI0, SGI316403, SGI316404, SGI316405, SGI316406, SGI316407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73919[] = {SGI316409, SGI0, SGI316410, SGI316411, SGI316412, SGI316413, SGI316414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73920[] = {SGI316417, SGI0, SGI316418, SGI316419, SGI316420, SGI316421, SGI316422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316424, SGI0, SGI0, NULL};
const Segment * const SG73921[] = {SGI316425, SGI0, SGI316426, SGI316427, SGI316428, SGI316429, SGI316430, SGI316431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316433, SGI0, SGI0, NULL};
const Segment * const SG73922[] = {SGI316434, SGI0, SGI316435, SGI316436, SGI316437, SGI316438, SGI316439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73923[] = {SGI316441, SGI0, SGI316442, SGI316443, SGI316444, SGI316445, SGI316446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316448, SGI0, SGI0, NULL};
const Segment * const SG73924[] = {SGI316449, SGI0, SGI316450, SGI316451, SGI316452, SGI316453, SGI316454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73925[] = {SGI316456, SGI0, SGI316457, SGI316458, SGI316459, SGI316460, SGI316461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73926[] = {SGI316463, SGI316464, SGI316465, SGI316466, SGI316467, SGI316468, SGI316469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73927[] = {SGI316471, SGI0, SGI316472, SGI316473, SGI316474, SGI316475, SGI316476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73928[] = {SGI316478, SGI0, SGI316479, SGI316480, SGI316481, SGI316482, SGI316483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73929[] = {SGI316485, SGI0, SGI316486, SGI316487, SGI316488, SGI316489, SGI316490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316492, SGI0, SGI0, NULL};
const Segment * const SG73930[] = {SGI316493, SGI0, SGI316494, SGI316495, SGI316496, SGI316497, SGI316498, SGI316499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73931[] = {SGI316501, SGI0, SGI316502, SGI316503, SGI316504, SGI316505, SGI316506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316508, SGI0, SGI0, NULL};
const Segment * const SG73932[] = {SGI316509, SGI0, SGI316510, SGI316511, SGI316512, SGI316513, SGI316514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73933[] = {SGI316516, SGI0, SGI316517, SGI316518, SGI316519, SGI316520, SGI316521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73934[] = {SGI316523, SGI0, SGI316524, SGI316525, SGI316526, SGI316527, SGI316528, SGI316529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316531, SGI0, SGI0, NULL};
const Segment * const SG73935[] = {SGI316532, SGI0, SGI316533, SGI316534, SGI316535, SGI316536, SGI316537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73936[] = {SGI316539, SGI0, SGI316540, SGI316541, SGI316542, SGI316543, SGI316544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316546, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73937[] = {SGI316547, SGI0, SGI316548, SGI316549, SGI316550, SGI316551, SGI316552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73938[] = {SGI316554, SGI0, SGI316555, SGI316556, SGI316557, SGI316558, SGI316559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316561, SGI0, SGI0, NULL};
const Segment * const SG73939[] = {SGI316562, SGI0, SGI316563, SGI316564, SGI316565, SGI316566, SGI316567, SGI316568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73940[] = {SGI316570, SGI0, SGI316571, SGI316572, SGI316573, SGI316574, SGI316575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73941[] = {SGI316577, SGI0, SGI316578, SGI316579, SGI316580, SGI316581, SGI316582, SGI316583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316585, SGI0, SGI0, NULL};
const Segment * const SG73942[] = {SGI316586, SGI0, SGI316587, SGI316588, SGI316589, SGI316590, SGI316591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316593, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73943[] = {SGI316594, SGI0, SGI316595, SGI316596, SGI316597, SGI316598, SGI316599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73944[] = {SGI316601, SGI0, SGI316602, SGI316603, SGI316604, SGI316605, SGI316606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316608, SGI0, SGI0, NULL};
const Segment * const SG73945[] = {SGI316609, SGI0, SGI316610, SGI316611, SGI316612, SGI316613, SGI316614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73946[] = {SGI316616, SGI0, SGI316617, SGI316618, SGI316619, SGI316620, SGI316621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73947[] = {SGI316624, SGI0, SGI316625, SGI316626, SGI316627, SGI316628, SGI316629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73948[] = {SGI316631, SGI0, SGI316632, SGI316633, SGI316634, SGI316635, SGI316636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73949[] = {SGI316638, SGI0, SGI316639, SGI316640, SGI316641, SGI316642, SGI316643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316645, SGI0, SGI0, NULL};
const Segment * const SG73950[] = {SGI316646, SGI0, SGI316647, SGI316648, SGI316649, SGI316650, SGI316651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73951[] = {SGI316654, SGI316655, NULL};
const Segment * const SG73952[] = {SGI316656, SGI0, SGI316657, SGI316658, SGI316659, SGI316660, SGI316661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73953[] = {SGI316663, SGI0, SGI316664, SGI316665, SGI316666, SGI316667, SGI316668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316670, SGI0, SGI0, NULL};
const Segment * const SG73954[] = {SGI316671, SGI0, SGI316672, SGI316673, SGI316674, SGI316675, SGI316676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316678, SGI0, SGI0, NULL};
const Segment * const SG73955[] = {SGI316679, SGI0, SGI316680, SGI316681, SGI316682, SGI316683, SGI316684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73956[] = {SGI316687, SGI0, SGI316688, SGI316689, SGI316690, SGI316691, SGI316692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73957[] = {SGI316694, SGI0, SGI316695, SGI316696, SGI316697, SGI316698, SGI316699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316701, SGI0, SGI0, NULL};
const Segment * const SG73958[] = {SGI316702, SGI0, SGI316703, SGI316704, SGI316705, SGI316706, SGI0, SGI316707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73959[] = {SGI316708, SGI0, SGI316709, SGI316710, SGI316711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73960[] = {SGI316712, SGI0, SGI316713, SGI316714, SGI316715, SGI316716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73961[] = {SGI316718, SGI0, SGI316719, SGI316720, SGI316721, SGI316722, SGI0, SGI0, SGI0, SGI0, SGI316723, SGI0, SGI0, SGI316724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73962[] = {SGI316725, SGI0, SGI316726, SGI316727, SGI316728, SGI316729, SGI0, SGI316730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73963[] = {SGI316732, SGI0, SGI316733, SGI316734, SGI316735, SGI316736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73964[] = {SGI316737, SGI0, SGI316738, SGI316739, SGI316740, SGI316741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316743, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73965[] = {SGI316744, SGI0, SGI316745, SGI316746, SGI316747, SGI316748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73966[] = {SGI316749, SGI0, SGI316750, SGI316751, SGI316752, SGI316753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316754, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73967[] = {SGI316755, SGI0, SGI316756, SGI316757, SGI316758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73968[] = {SGI316760, SGI0, SGI316761, SGI316762, SGI316763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73969[] = {SGI316765, SGI0, SGI316766, SGI316767, SGI316768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316769, SGI0, SGI316770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73970[] = {SGI316771, SGI0, SGI316772, SGI316773, SGI316774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73971[] = {SGI316776, SGI0, SGI316777, SGI316778, SGI316779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73972[] = {SGI316781, SGI0, SGI316782, SGI316783, SGI316784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73973[] = {SGI316786, SGI0, SGI316787, SGI316788, SGI316789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73974[] = {SGI316791, SGI0, SGI316792, SGI316793, SGI316794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73975[] = {SGI316796, SGI0, SGI316797, SGI316798, SGI316799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73976[] = {SGI316801, SGI0, SGI316802, SGI316803, SGI316804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316806, SGI0, SGI0, NULL};
const Segment * const SG73977[] = {SGI316807, SGI0, SGI316808, SGI316809, SGI316810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316812, SGI0, SGI0, NULL};
const Segment * const SG73978[] = {SGI316813, SGI0, SGI316814, SGI316815, SGI316816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73979[] = {SGI316818, SGI0, SGI316819, SGI316820, SGI316821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316822, SGI0, SGI316823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73980[] = {SGI316824, SGI0, SGI316825, SGI316826, SGI316827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316829, SGI0, SGI0, NULL};
const Segment * const SG73981[] = {SGI316830, SGI0, SGI316831, SGI316832, SGI316833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73982[] = {SGI316835, SGI0, SGI316836, SGI316837, SGI316838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73983[] = {SGI316840, SGI0, SGI316841, SGI316842, SGI316843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316844, SGI0, SGI316845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73984[] = {SGI316846, SGI0, SGI316847, SGI316848, SGI316849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316851, SGI0, SGI0, NULL};
const Segment * const SG73985[] = {SGI316852, SGI0, SGI316853, SGI316854, SGI316855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316856, SGI0, SGI316857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73986[] = {SGI316858, SGI0, SGI316859, SGI316860, SGI316861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316862, SGI0, SGI316863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73987[] = {SGI316864, SGI0, SGI316865, SGI316866, SGI316867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316868, SGI0, SGI316869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73988[] = {SGI316870, SGI0, SGI316871, SGI316872, SGI316873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316875, SGI0, SGI0, NULL};
const Segment * const SG73989[] = {SGI316876, SGI0, SGI316877, SGI316878, SGI316879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316881, SGI0, SGI0, NULL};
const Segment * const SG73990[] = {SGI316882, SGI0, SGI316883, SGI316884, SGI316885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG73991[] = {SGI316887, SGI0, SGI316888, SGI316889, SGI316890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316891, SGI0, SGI316892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316893, SGI0, SGI0, NULL};
const Segment * const SG73992[] = {SGI316894, SGI0, SGI316895, SGI316896, SGI316897, SGI316898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316899, SGI0, NULL};
const Segment * const SG73993[] = {SGI316900, SGI0, SGI316901, SGI316902, SGI316903, SGI316904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316906, SGI0, NULL};
const Segment * const SG73994[] = {SGI316907, SGI0, SGI316908, SGI316909, SGI316910, SGI316911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316912, SGI0, NULL};
const Segment * const SG73995[] = {SGI316913, SGI0, SGI316914, SGI316915, SGI316916, SGI316917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316919, SGI0, NULL};
const Segment * const SG73996[] = {SGI316920, SGI0, SGI316921, SGI316922, SGI316923, SGI316924, SGI0, SGI316925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316926, SGI0, NULL};
const Segment * const SG73997[] = {SGI316927, SGI0, SGI316928, SGI316929, SGI316930, SGI316931, SGI0, SGI316932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316934, SGI0, NULL};
const Segment * const SG73998[] = {SGI316935, SGI0, SGI316936, SGI316937, SGI316938, SGI316939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316940, SGI0, NULL};
const Segment * const SG73999[] = {SGI316941, SGI0, SGI316942, SGI316943, SGI316944, SGI316945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316947, SGI0, NULL};
const Segment * const SG74000[] = {SGI316948, SGI0, SGI316949, SGI316950, SGI316951, SGI316952, SGI0, SGI316953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316954, SGI0, SGI316955, SGI0, NULL};
const Segment * const SG74001[] = {SGI316956, SGI0, SGI316957, SGI316958, SGI316959, SGI316960, SGI0, SGI316961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316963, SGI0, NULL};
const Segment * const SG74002[] = {SGI316964, SGI0, SGI316965, SGI316966, SGI316967, SGI316968, SGI0, SGI316969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316970, SGI0, NULL};
const Segment * const SG74003[] = {SGI316971, SGI0, SGI316972, SGI316973, SGI316974, SGI316975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316976, SGI0, NULL};
const Segment * const SG74004[] = {SGI0, SGI316977, NULL};
const Segment * const SG74005[] = {SGI316978, SGI316978, SGI0, SGI316979, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74006[] = {SGI316980, SGI316980, SGI0, SGI316981, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74007[] = {SGI316982, SGI0, SGI316983, SGI316984, SGI316985, SGI316986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316987, SGI0, SGI316988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74008[] = {SGI316990, SGI0, SGI316991, SGI316992, SGI316993, SGI316994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI316996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74009[] = {SGI316997, SGI0, SGI316998, SGI316999, SGI317000, SGI317001, SGI0, SGI317002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317003, SGI0, SGI317004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74010[] = {SGI317006, SGI0, SGI317007, SGI317008, SGI317009, SGI317010, SGI0, SGI317011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74011[] = {SGI317014, SGI0, SGI317015, SGI317016, SGI317017, SGI317018, SGI0, SGI317019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317021, SGI0, SGI0, SGI0, SGI317022, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74012[] = {SGI317023, SGI0, SGI317024, SGI317025, SGI317026, SGI317027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74013[] = {SGI317030, SGI0, SGI317031, SGI317032, SGI317033, SGI317034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317035, SGI0, SGI317036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74014[] = {SGI317038, SGI0, SGI317039, SGI317040, SGI317041, SGI317042, SGI0, SGI317043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74015[] = {SGI317046, SGI0, SGI317047, SGI317048, SGI317049, SGI317050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317051, SGI0, SGI317052, SGI0, SGI317053, SGI0, SGI0, SGI0, SGI317054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74016[] = {SGI317055, SGI0, SGI317056, SGI317057, SGI317058, SGI317059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317060, SGI0, SGI317061, SGI0, SGI0, SGI0, SGI317062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74017[] = {SGI317063, SGI0, SGI317064, SGI317065, SGI317066, SGI317067, SGI0, SGI317068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317069, SGI0, SGI317070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74018[] = {SGI317072, SGI0, SGI317073, SGI317074, SGI317075, SGI317076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317077, SGI0, SGI317078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74019[] = {SGI317080, SGI0, SGI317081, SGI317082, SGI317083, SGI317084, SGI0, SGI317085, SGI0, SGI0, SGI317086, SGI0, SGI0, SGI317087, SGI0, SGI317088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74020[] = {SGI317090, SGI0, SGI317091, SGI317092, SGI317093, SGI317094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74021[] = {SGI317097, SGI0, SGI317098, SGI317099, SGI317100, SGI317101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317102, SGI0, SGI317103, SGI0, SGI317104, SGI0, SGI0, SGI0, SGI317105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74022[] = {SGI317106, SGI0, SGI317107, SGI317108, SGI317109, SGI317110, SGI0, SGI317111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317113, SGI0, SGI0, SGI0, SGI317114, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74023[] = {SGI317115, SGI317115, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74024[] = {SGI317116, SGI317116, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74025[] = {SGI317117, SGI317117, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74026[] = {SGI317118, SGI317119, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74027[] = {SGI317120, SGI317121, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74028[] = {SGI317122, SGI0, SGI317123, SGI317124, SGI317125, SGI317126, SGI317127, SGI317128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74029[] = {SGI317130, NULL};
const Segment * const SG74030[] = {SGI317131, SGI0, SGI317132, SGI317133, SGI317134, SGI317135, SGI317136, SGI317137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317139, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74031[] = {SGI317140, SGI0, SGI317141, SGI317142, SGI317143, SGI317144, SGI317145, SGI317146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74032[] = {SGI317148, SGI0, SGI317149, SGI317150, SGI317151, SGI317152, SGI317153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74033[] = {SGI317155, SGI0, SGI317156, SGI317157, SGI317158, SGI317159, SGI317160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74034[] = {SGI317163, SGI0, SGI317164, SGI317165, SGI317166, SGI317167, SGI317168, SGI317169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74035[] = {SGI317172, SGI0, SGI317173, SGI317174, SGI317175, SGI317176, SGI317177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74036[] = {SGI317179, SGI0, SGI317180, SGI317181, SGI317182, SGI317183, SGI317184, SGI317185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317187, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74037[] = {SGI317188, SGI0, SGI317189, SGI317190, SGI317191, SGI317192, SGI317193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317195, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74038[] = {SGI317196, SGI0, SGI317197, SGI317198, SGI317199, SGI317200, SGI317201, SGI317202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74039[] = {SGI317205, SGI0, SGI317206, SGI317207, SGI317208, SGI317209, SGI317210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74040[] = {SGI317213, SGI317214, SGI317215, SGI317216, SGI317217, SGI317218, SGI317219, SGI317220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74041[] = {SGI317223, SGI0, SGI317224, SGI317225, SGI317226, SGI317227, SGI317228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74042[] = {SGI0, SGI317231, NULL};
const Segment * const SG74043[] = {SGI317232, SGI0, SGI317233, SGI317234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74044[] = {SGI317235, SGI0, SGI317236, SGI317237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74045[] = {SGI317238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74046[] = {SGI317239, SGI0, SGI317240, SGI317241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74047[] = {SGI317242, SGI0, SGI317243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74048[] = {SGI317244, SGI0, SGI317245, SGI317246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317247, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74049[] = {SGI317248, SGI0, SGI317249, SGI317250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317251, SGI0, SGI317252, SGI0, SGI0, NULL};
const Segment * const SG74050[] = {SGI317253, SGI0, SGI0, SGI0, SGI317254, SGI317255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74051[] = {SGI317256, SGI0, SGI0, SGI0, SGI317257, SGI317258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317259, SGI0, SGI0, NULL};
const Segment * const SG74052[] = {SGI317260, SGI0, SGI0, SGI0, SGI317261, SGI317262, SGI0, SGI317263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74053[] = {SGI317264, SGI0, SGI317265, SGI0, SGI317266, SGI317267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317268, SGI0, SGI0, NULL};
const Segment * const SG74054[] = {SGI317269, SGI0, SGI0, SGI0, SGI317270, SGI317271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317272, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74055[] = {SGI317273, SGI0, SGI0, SGI0, SGI317274, SGI317275, SGI0, SGI0, SGI0, SGI0, SGI317276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74056[] = {SGI317277, SGI0, SGI0, SGI0, SGI317278, SGI317279, SGI0, SGI317280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317281, SGI0, SGI0, NULL};
const Segment * const SG74057[] = {SGI317282, SGI0, SGI317283, SGI317284, SGI317285, SGI317286, SGI0, SGI317287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74058[] = {SGI317290, SGI0, SGI317291, SGI317292, SGI317293, SGI317294, SGI0, SGI317295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74059[] = {SGI317298, SGI0, SGI317299, SGI317300, SGI317301, SGI317302, SGI0, SGI317303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317305, SGI0, SGI0, SGI0, SGI0, SGI317306, SGI0, SGI0, NULL};
const Segment * const SG74060[] = {SGI317307, SGI0, SGI317308, SGI317309, SGI317310, SGI317311, SGI0, SGI317312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317314, SGI0, SGI0, SGI0, SGI317315, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74061[] = {SGI317316, SGI0, SGI317317, SGI317318, SGI317319, SGI317320, SGI0, SGI317321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317322, SGI0, SGI317323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74062[] = {SGI317325, SGI0, SGI317326, SGI317327, SGI317328, SGI317329, SGI0, SGI317330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317331, SGI0, SGI317332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74063[] = {SGI317334, SGI0, SGI317335, SGI317336, SGI317337, SGI317338, SGI0, SGI317339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74064[] = {SGI317342, SGI0, SGI317343, SGI317344, SGI317345, SGI317346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74065[] = {SGI317349, SGI0, SGI317350, SGI317351, SGI317352, SGI317353, SGI0, SGI317354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317356, SGI0, SGI0, SGI317357, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74066[] = {SGI317358, SGI0, SGI317359, SGI317360, SGI317361, SGI317362, SGI0, SGI317363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317365, SGI0, SGI0, SGI317366, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74067[] = {SGI317367, SGI0, SGI317368, SGI317369, SGI317370, SGI317371, SGI0, SGI317372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317373, SGI0, SGI317374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74068[] = {SGI317376, SGI0, SGI317377, SGI317378, SGI317379, SGI317380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317381, SGI0, SGI317382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74069[] = {SGI317384, SGI0, SGI317385, SGI317386, SGI317387, SGI317388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317390, SGI0, SGI0, SGI0, SGI317391, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74070[] = {SGI317392, SGI0, SGI317393, SGI317394, SGI317395, SGI317396, SGI0, SGI317397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317399, SGI0, SGI0, SGI0, SGI317400, SGI317401, SGI0, SGI0, NULL};
const Segment * const SG74071[] = {SGI317402, SGI0, SGI317403, SGI317404, SGI317405, SGI317406, SGI0, SGI317407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317409, SGI0, SGI0, SGI0, SGI317410, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74072[] = {SGI317411, SGI0, SGI317412, SGI317413, SGI317414, SGI317415, SGI0, SGI317416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317417, SGI0, SGI317418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74073[] = {SGI317420, SGI0, SGI317421, SGI317422, SGI317423, SGI317424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317425, SGI0, SGI317426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74074[] = {SGI317428, SGI0, SGI317429, SGI317430, SGI317431, SGI317432, SGI317433, SGI317434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317435, SGI0, SGI317436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74075[] = {SGI317438, SGI0, SGI317439, SGI317440, SGI317441, SGI317442, SGI0, SGI317443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74076[] = {SGI317446, SGI0, SGI317447, SGI317448, SGI317449, SGI317450, SGI0, SGI317451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317452, SGI0, SGI317453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74077[] = {SGI317455, SGI0, SGI317456, SGI317457, SGI317458, SGI317459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74078[] = {SGI317462, SGI0, SGI317463, SGI317464, SGI317465, SGI317466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74079[] = {SGI317469, SGI0, SGI317470, SGI317471, SGI317472, SGI317473, SGI0, SGI317474, SGI0, SGI0, SGI317475, SGI0, SGI0, SGI317476, SGI0, SGI317477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74080[] = {SGI317479, SGI0, SGI317480, SGI317481, SGI317482, SGI317483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74081[] = {SGI317486, SGI0, SGI317487, SGI317488, SGI317489, SGI317490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317491, SGI0, SGI317492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74082[] = {SGI317494, SGI0, SGI317495, SGI317496, SGI317497, SGI317498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317499, SGI0, SGI317500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74083[] = {SGI317502, SGI0, SGI317503, SGI317504, SGI317505, SGI317506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317508, SGI0, SGI0, SGI0, SGI317509, SGI0, SGI0, NULL};
const Segment * const SG74084[] = {SGI317510, SGI0, SGI317511, SGI317512, SGI317513, SGI317514, SGI0, SGI317515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317516, SGI0, SGI317517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317518, SGI0, SGI0, SGI0, SGI0, SGI317519, SGI0, SGI0, NULL};
const Segment * const SG74085[] = {SGI317520, SGI0, SGI317521, SGI317522, SGI317523, SGI317524, SGI0, SGI317525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74086[] = {SGI317528, SGI0, SGI317529, SGI317530, SGI317531, SGI317532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317533, SGI0, SGI317534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74087[] = {SGI317536, SGI0, SGI317537, SGI317538, SGI317539, SGI317540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74088[] = {SGI317543, SGI0, SGI317544, SGI317545, SGI317546, SGI317547, SGI317548, SGI317549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74089[] = {SGI317551, SGI0, SGI317552, SGI317553, SGI317554, SGI317555, SGI317556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317558, SGI0, SGI0, NULL};
const Segment * const SG74090[] = {SGI317559, SGI0, SGI317560, SGI317561, SGI317562, SGI317563, SGI317564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74091[] = {SGI317566, SGI0, SGI317567, SGI317568, SGI317569, SGI317570, SGI317571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74092[] = {SGI317573, SGI0, SGI317574, SGI317575, SGI317576, SGI317577, SGI317578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74093[] = {SGI317580, SGI0, SGI317581, SGI317582, SGI317583, SGI317584, SGI317585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317587, SGI0, SGI0, NULL};
const Segment * const SG74094[] = {SGI317588, SGI0, SGI317589, SGI317590, SGI317591, SGI317592, SGI317593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74095[] = {SGI317595, SGI0, SGI317596, SGI317597, SGI317598, SGI317599, SGI317600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74096[] = {SGI317602, SGI0, SGI317603, SGI317604, SGI317605, SGI317606, SGI317607, SGI317608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74097[] = {SGI317611, SGI0, SGI317612, SGI317613, SGI317614, SGI317615, SGI317616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74098[] = {SGI317618, SGI0, SGI317619, SGI317620, SGI317621, SGI317622, SGI317623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74099[] = {SGI317626, SGI0, SGI317627, SGI317628, SGI317629, SGI317630, SGI317631, SGI317632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74100[] = {SGI317634, SGI0, SGI317635, SGI317636, SGI317637, SGI317638, SGI317639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74101[] = {SGI317642, SGI0, SGI317643, SGI317644, SGI317645, SGI317646, SGI317647, SGI317648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74102[] = {SGI317650, SGI0, SGI317651, SGI317652, SGI317653, SGI317654, SGI317655, SGI317656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74103[] = {SGI317658, SGI0, SGI317659, SGI317660, SGI317661, SGI317662, SGI317663, SGI317664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317666, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74104[] = {SGI317667, SGI0, SGI317668, SGI317669, SGI317670, SGI317671, SGI317672, SGI317673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74105[] = {SGI317675, SGI0, SGI317676, SGI317677, SGI317678, SGI317679, SGI0, SGI317680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74106[] = {SGI317682, SGI0, SGI317683, SGI317684, SGI317685, SGI317686, SGI0, SGI317687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317688, SGI0, SGI0, SGI0, SGI317689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74107[] = {SGI317690, SGI0, SGI317691, SGI317692, SGI317693, SGI317694, SGI0, SGI317695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74108[] = {SGI317697, SGI0, SGI317698, SGI317699, SGI317700, SGI317701, SGI0, SGI317702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317703, SGI0, SGI0, SGI0, SGI317704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74109[] = {SGI317705, SGI0, SGI317706, SGI317707, SGI317708, SGI317709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74110[] = {SGI317711, SGI0, SGI317712, SGI317713, SGI317714, SGI317715, SGI0, SGI317716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317718, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74111[] = {SGI317719, SGI0, SGI317720, SGI317721, SGI317722, SGI317723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74112[] = {SGI317725, SGI0, SGI317726, SGI317727, SGI317728, SGI317729, SGI0, SGI317730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317732, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74113[] = {SGI317733, SGI0, SGI317734, SGI317735, SGI317736, SGI317737, SGI0, SGI317738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317740, SGI0, SGI317741, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74114[] = {SGI317742, SGI0, SGI317743, SGI317744, SGI317745, SGI317746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74115[] = {SGI317748, SGI0, SGI317749, SGI317750, SGI317751, SGI317752, SGI0, SGI317753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317755, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74116[] = {SGI317756, SGI0, SGI317757, SGI317758, SGI317759, SGI317760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317761, SGI0, SGI0, SGI0, SGI317762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74117[] = {SGI317763, SGI0, SGI317764, SGI317765, SGI317766, SGI317767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317769, SGI0, SGI0, NULL};
const Segment * const SG74118[] = {SGI317770, SGI0, SGI317771, SGI317772, SGI317773, SGI317774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317776, SGI0, SGI0, NULL};
const Segment * const SG74119[] = {SGI317777, SGI0, SGI317778, SGI317779, SGI317780, SGI317781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74120[] = {SGI317783, SGI0, SGI317784, SGI317785, SGI317786, SGI317787, SGI0, SGI317788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317790, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74121[] = {SGI317791, SGI0, SGI317792, SGI317793, SGI317794, SGI317795, SGI0, SGI317796, SGI0, SGI0, SGI317797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317799, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74122[] = {SGI317800, SGI0, SGI317801, SGI317802, SGI317803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317804, SGI0, SGI0, NULL};
const Segment * const SG74123[] = {SGI317805, SGI0, SGI317806, SGI317807, SGI317808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74124[] = {SGI317809, SGI0, SGI317810, SGI317811, SGI317812, SGI317813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74125[] = {SGI317814, SGI0, SGI317815, SGI317816, SGI317817, SGI317818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317819, SGI0, SGI0, NULL};
const Segment * const SG74126[] = {SGI317820, SGI0, SGI317821, SGI317822, SGI317823, SGI317824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74127[] = {SGI317825, SGI0, SGI317826, SGI317827, SGI317828, SGI317829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317830, SGI0, SGI0, NULL};
const Segment * const SG74128[] = {SGI317831, SGI0, SGI317832, SGI317833, SGI317834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74129[] = {SGI317836, SGI0, SGI317837, SGI317838, SGI317839, SGI0, SGI0, SGI317840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74130[] = {SGI317841, SGI0, SGI317842, SGI317843, SGI317844, SGI317845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317846, SGI0, SGI0, NULL};
const Segment * const SG74131[] = {SGI317847, SGI0, SGI317848, SGI317849, SGI317850, SGI0, SGI0, SGI317851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74132[] = {SGI317853, SGI0, SGI317854, SGI317855, SGI317856, SGI317857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317858, SGI0, SGI0, NULL};
const Segment * const SG74133[] = {SGI317859, SGI0, SGI317860, SGI317861, SGI317862, SGI317863, SGI0, SGI317864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74134[] = {SGI317866, SGI0, SGI317867, SGI317868, SGI317869, SGI317870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74135[] = {SGI317872, SGI0, SGI317873, SGI317874, SGI317875, SGI317876, SGI0, SGI317877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317879, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74136[] = {SGI317880, SGI0, SGI317881, SGI317882, SGI317883, SGI317884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317886, SGI0, SGI0, NULL};
const Segment * const SG74137[] = {SGI317887, SGI0, SGI317888, SGI317889, SGI317890, SGI317891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317893, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74138[] = {SGI317894, SGI0, SGI317895, SGI317896, SGI317897, SGI317898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317900, SGI0, SGI0, NULL};
const Segment * const SG74139[] = {SGI317901, SGI0, SGI317902, SGI317903, SGI317904, SGI317905, SGI0, SGI317906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74140[] = {SGI317908, SGI0, SGI317909, SGI317910, SGI317911, SGI317912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317914, SGI0, SGI0, NULL};
const Segment * const SG74141[] = {SGI317915, SGI0, SGI317916, SGI317917, SGI317918, SGI317919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317920, SGI0, SGI317921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74142[] = {SGI317922, SGI0, SGI317923, SGI317924, SGI317925, SGI317926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317928, SGI0, SGI0, NULL};
const Segment * const SG74143[] = {SGI317929, SGI0, SGI317930, SGI317931, SGI317932, SGI317933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74144[] = {SGI317935, SGI0, SGI317936, SGI317937, SGI317938, SGI317939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317941, SGI0, SGI0, NULL};
const Segment * const SG74145[] = {SGI317942, SGI0, SGI317943, SGI317944, SGI317945, SGI317946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74146[] = {SGI317948, SGI0, SGI317949, SGI317950, SGI317951, SGI317952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74147[] = {SGI317954, SGI0, SGI317955, SGI317956, SGI317957, SGI317958, SGI0, SGI317959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74148[] = {SGI317961, SGI0, SGI317962, SGI317963, SGI317964, SGI317965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74149[] = {SGI317967, SGI0, SGI317968, SGI317969, SGI317970, SGI317971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74150[] = {SGI317973, SGI0, SGI317974, SGI317975, SGI317976, SGI317977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74151[] = {SGI317979, SGI0, SGI317980, SGI317981, SGI317982, SGI317983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74152[] = {SGI317985, SGI0, SGI317986, SGI317987, SGI317988, SGI317989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317990, SGI0, SGI317991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74153[] = {SGI317992, SGI0, SGI317993, SGI317994, SGI317995, SGI317996, SGI0, SGI317997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI317998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74154[] = {SGI317999, SGI0, SGI318000, SGI318001, SGI318002, SGI318003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74155[] = {SGI318005, SGI0, SGI318006, SGI318007, SGI318008, SGI318009, SGI0, SGI318010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318011, SGI0, SGI318012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74156[] = {SGI318013, SGI0, SGI318014, SGI318015, SGI318016, SGI318017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74157[] = {SGI318019, SGI0, SGI318020, SGI318021, SGI318022, SGI318023, SGI0, SGI318024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318025, SGI0, SGI318026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74158[] = {SGI318027, SGI0, SGI318028, SGI318029, SGI318030, SGI318031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318032, SGI0, SGI318033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74159[] = {SGI318034, SGI0, SGI318035, SGI318036, SGI318037, SGI318038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318039, SGI0, SGI318040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74160[] = {SGI318041, SGI0, SGI318042, SGI318043, SGI318044, SGI318045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74161[] = {SGI318047, SGI0, SGI318048, SGI318049, SGI318050, SGI318051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318052, SGI0, SGI318053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74162[] = {SGI318054, SGI0, SGI318055, SGI318056, SGI318057, SGI318058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318060, SGI0, SGI0, NULL};
const Segment * const SG74163[] = {SGI318061, SGI0, SGI318062, SGI318063, SGI318064, SGI318065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318066, SGI0, SGI318067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74164[] = {SGI318068, SGI0, SGI318069, SGI318070, SGI318071, SGI318072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318074, SGI0, SGI0, NULL};
const Segment * const SG74165[] = {SGI318075, SGI0, SGI318076, SGI318077, SGI318078, SGI318079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318081, SGI0, SGI0, NULL};
const Segment * const SG74166[] = {SGI318082, SGI0, SGI318083, SGI318084, SGI318085, SGI318086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74167[] = {SGI318088, SGI0, SGI318089, SGI318090, SGI318091, SGI318092, SGI0, SGI318093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74168[] = {SGI318095, SGI0, SGI318096, SGI318097, SGI318098, SGI318099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318101, SGI0, SGI0, NULL};
const Segment * const SG74169[] = {SGI318102, SGI0, SGI318103, SGI318104, SGI318105, SGI318106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318108, SGI0, SGI0, NULL};
const Segment * const SG74170[] = {SGI318109, SGI0, SGI318110, SGI318111, SGI318112, SGI318113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74171[] = {SGI318115, SGI0, SGI318116, SGI318117, SGI318118, SGI318119, SGI0, SGI318120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318121, SGI0, SGI318122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74172[] = {SGI318123, SGI0, SGI318124, SGI318125, SGI318126, SGI318127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74173[] = {SGI318129, SGI0, SGI318130, SGI318131, SGI318132, SGI318133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74174[] = {SGI318135, SGI0, SGI318136, SGI318137, SGI318138, SGI318139, SGI0, SGI318140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318141, SGI0, SGI318142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74175[] = {SGI318143, SGI0, SGI318144, SGI318145, SGI318146, SGI318147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318148, SGI0, SGI318149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74176[] = {SGI318150, SGI0, SGI318151, SGI318152, SGI318153, SGI318154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318156, SGI0, SGI0, NULL};
const Segment * const SG74177[] = {SGI318157, SGI0, SGI318158, SGI318159, SGI318160, SGI318161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318162, SGI0, SGI318163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74178[] = {SGI318164, SGI0, SGI318165, SGI318166, SGI318167, SGI318168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74179[] = {SGI318170, SGI0, SGI318171, SGI318172, SGI318173, SGI318174, SGI0, SGI318175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318176, SGI0, SGI318177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74180[] = {SGI318178, SGI0, SGI318179, SGI318180, SGI318181, SGI318182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318183, SGI0, SGI318184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74181[] = {SGI318185, SGI0, SGI318186, SGI318187, SGI318188, SGI318189, SGI0, SGI318190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318191, SGI0, SGI318192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74182[] = {SGI318193, SGI318193, SGI318194, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74183[] = {SGI318195, SGI0, SGI318196, SGI0, SGI318197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74184[] = {SGI318198, SGI0, SGI318199, SGI318200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318201, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74185[] = {SGI318202, SGI0, SGI318203, SGI318204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318205, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74186[] = {SGI318206, SGI0, SGI318207, SGI318208, SGI318209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74187[] = {SGI318210, SGI0, SGI318211, SGI318212, SGI318213, SGI318214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74188[] = {SGI318215, SGI0, SGI318216, SGI318217, SGI318218, SGI318219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318220, SGI0, SGI0, NULL};
const Segment * const SG74189[] = {SGI318221, SGI0, SGI318222, SGI318223, SGI318224, SGI318225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74190[] = {SGI318227, SGI0, SGI318228, SGI318229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318230, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74191[] = {SGI318231, SGI0, SGI318232, SGI318233, SGI318234, SGI318235, SGI0, SGI318236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74192[] = {SGI318237, SGI0, SGI318238, SGI318239, SGI318240, SGI318241, SGI0, SGI318242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74193[] = {SGI318244, SGI0, SGI318245, SGI318246, SGI318247, SGI318248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74194[] = {SGI318250, SGI0, SGI318251, SGI318252, SGI318253, SGI318254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74195[] = {SGI318256, SGI0, SGI318257, SGI318258, SGI318259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74196[] = {SGI318260, SGI0, SGI318261, SGI318262, SGI318263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74197[] = {SGI318264, SGI0, SGI318265, SGI318266, SGI318267, SGI318268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74198[] = {SGI318269, SGI0, SGI318269, SGI318270, SGI318271, SGI318272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74199[] = {SGI318273, SGI0, SGI318274, SGI318275, SGI318276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74200[] = {SGI318277, SGI0, SGI318278, SGI318279, SGI318280, SGI318281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74201[] = {SGI318282, SGI0, SGI318283, SGI318284, SGI318285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74202[] = {SGI318286, SGI0, SGI318286, SGI318287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318288, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74203[] = {SGI318289, SGI0, SGI318290, SGI318291, SGI318292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74204[] = {SGI318293, SGI0, SGI318294, SGI318295, SGI318296, SGI318297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74205[] = {SGI318298, SGI0, SGI318299, SGI318300, SGI318301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74206[] = {SGI318302, SGI0, SGI318303, SGI318304, SGI318305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74207[] = {SGI318306, SGI0, SGI318307, SGI318308, SGI318309, SGI318310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74208[] = {SGI318311, SGI0, SGI318312, SGI318313, SGI318314, SGI318315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74209[] = {SGI318316, SGI0, SGI318317, SGI318318, SGI318319, SGI318320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318321, SGI0, SGI0, NULL};
const Segment * const SG74210[] = {SGI318322, SGI0, SGI318323, SGI318324, SGI318325, SGI318326, SGI0, SGI318327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74211[] = {SGI318329, SGI0, SGI318330, SGI318331, SGI318332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74212[] = {SGI318333, SGI0, SGI318334, SGI318335, SGI318336, SGI318337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74213[] = {SGI318338, SGI0, SGI318339, SGI318340, SGI318341, SGI318342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74214[] = {SGI318343, SGI0, SGI318344, SGI318345, SGI318346, SGI318347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318348, SGI0, SGI0, NULL};
const Segment * const SG74215[] = {SGI318349, SGI0, SGI318350, SGI318351, SGI318352, SGI318353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318354, SGI0, SGI0, NULL};
const Segment * const SG74216[] = {SGI318355, SGI0, SGI318356, SGI318357, SGI318358, SGI318359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318360, SGI0, SGI0, NULL};
const Segment * const SG74217[] = {SGI318361, SGI0, SGI318362, SGI318363, SGI318364, SGI318365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318366, SGI0, SGI0, NULL};
const Segment * const SG74218[] = {SGI318367, SGI0, SGI318368, SGI318369, SGI318370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74219[] = {SGI318371, SGI0, SGI318372, SGI318373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74220[] = {SGI318374, SGI0, SGI318375, SGI318376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318377, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74221[] = {SGI318378, SGI0, SGI318379, SGI318380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74222[] = {SGI318381, SGI0, SGI318382, SGI318383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318384, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74223[] = {SGI318385, SGI0, SGI318386, SGI318387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74224[] = {SGI318389, SGI0, SGI318390, SGI318391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318392, SGI0, SGI0, NULL};
const Segment * const SG74225[] = {SGI318393, SGI0, SGI318394, SGI318395, SGI318396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74226[] = {SGI318397, SGI0, SGI318398, SGI318399, SGI318400, SGI318401, SGI0, SGI0, SGI318402, SGI0, SGI0, SGI0, SGI0, SGI318403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74227[] = {SGI318404, SGI0, SGI318405, SGI318406, SGI318407, SGI318408, SGI0, SGI0, SGI318409, SGI0, SGI0, SGI0, SGI0, SGI318410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74228[] = {SGI318411, SGI0, SGI318412, SGI318413, SGI318414, SGI318415, SGI0, SGI0, SGI318416, SGI0, SGI0, SGI0, SGI0, SGI318417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74229[] = {SGI318418, SGI0, SGI318419, SGI318420, SGI318421, SGI318422, SGI0, SGI0, SGI318423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74230[] = {SGI318424, SGI318424, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74231[] = {SGI318425, SGI0, SGI318426, SGI318427, SGI318428, SGI318429, SGI0, SGI318430, SGI318431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74232[] = {SGI318432, SGI0, SGI318433, SGI318434, SGI318435, SGI318436, SGI0, SGI0, SGI318437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74233[] = {SGI318438, SGI318438, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74234[] = {SGI318439, SGI0, SGI318440, SGI318441, SGI318442, SGI318443, SGI0, SGI0, SGI318444, SGI0, SGI0, SGI0, SGI0, SGI318445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74235[] = {SGI318446, SGI0, SGI318447, SGI318448, SGI318449, SGI318450, SGI0, SGI0, SGI318451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74236[] = {SGI318452, SGI0, SGI318453, SGI318454, SGI318455, SGI318456, SGI0, SGI318457, SGI318458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318459, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74237[] = {SGI318460, SGI0, SGI318461, SGI318462, SGI318463, SGI318464, SGI0, SGI0, SGI318465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74238[] = {SGI318466, SGI0, SGI318467, SGI318468, SGI318469, SGI318470, SGI0, SGI318471, SGI318472, SGI0, SGI0, SGI0, SGI0, SGI318473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74239[] = {SGI318474, SGI0, SGI318475, SGI318476, SGI318477, SGI318478, SGI0, SGI318479, SGI318480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74240[] = {SGI318481, SGI0, SGI318482, SGI318483, SGI318484, SGI318485, SGI0, SGI0, SGI318486, SGI0, SGI0, SGI0, SGI0, SGI318487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74241[] = {SGI318488, SGI0, SGI318489, SGI318490, SGI318491, SGI318492, SGI0, SGI0, SGI318493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74242[] = {SGI318494, SGI0, SGI318495, SGI318496, SGI318497, SGI318498, SGI0, SGI0, SGI318499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74243[] = {SGI318500, SGI0, SGI318501, SGI318502, SGI318503, SGI318504, SGI0, SGI318505, SGI318506, SGI0, SGI0, SGI0, SGI0, SGI318507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74244[] = {SGI318508, SGI0, SGI318509, SGI318510, SGI318511, SGI318512, SGI0, SGI0, SGI318513, SGI0, SGI0, SGI0, SGI0, SGI318514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74245[] = {SGI318515, SGI0, SGI318516, SGI318517, SGI318518, SGI318519, SGI0, SGI318520, SGI318521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74246[] = {SGI318522, SGI0, SGI318523, SGI318524, SGI318525, SGI318526, SGI0, SGI0, SGI318527, SGI0, SGI0, SGI0, SGI0, SGI318528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74247[] = {SGI318529, SGI0, SGI318530, SGI318531, SGI318532, SGI318533, SGI0, SGI318534, SGI318535, SGI0, SGI0, SGI0, SGI0, SGI318536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74248[] = {SGI318537, SGI0, SGI318538, SGI318539, SGI318540, SGI318541, SGI0, SGI0, SGI318542, SGI0, SGI0, SGI0, SGI0, SGI318543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74249[] = {SGI318544, SGI0, SGI318545, SGI318546, SGI318547, SGI318548, SGI0, SGI0, SGI318549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74250[] = {SGI318550, SGI0, SGI318551, SGI318552, SGI318553, SGI318554, SGI0, SGI0, SGI318555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74251[] = {SGI318556, SGI0, SGI318557, SGI318558, SGI318559, SGI318560, SGI0, SGI318561, SGI318562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74252[] = {SGI318563, SGI0, SGI318564, SGI318565, SGI318566, SGI318567, SGI0, SGI318568, SGI318569, SGI0, SGI0, SGI0, SGI0, SGI318570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74253[] = {SGI318571, SGI0, SGI318572, SGI318573, SGI318574, SGI318575, SGI0, SGI0, SGI318576, SGI0, SGI0, SGI0, SGI0, SGI318577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74254[] = {SGI318578, SGI0, SGI318579, SGI318580, SGI318581, SGI318582, SGI0, SGI0, SGI318583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74255[] = {SGI318584, SGI0, SGI318585, SGI318586, SGI318587, SGI318588, SGI0, SGI0, SGI318589, SGI0, SGI0, SGI0, SGI0, SGI318590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74256[] = {SGI318591, SGI0, SGI318592, SGI318593, SGI318594, SGI318595, SGI0, SGI0, SGI318596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74257[] = {SGI318597, SGI0, SGI318598, SGI318599, SGI318600, SGI318601, SGI0, SGI0, SGI318602, SGI0, SGI0, SGI0, SGI0, SGI318603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74258[] = {SGI318604, SGI0, SGI318605, SGI318606, SGI318607, SGI318608, SGI0, SGI318609, SGI318610, SGI0, SGI0, SGI0, SGI0, SGI318611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74259[] = {SGI318612, SGI0, SGI318613, SGI318614, SGI318615, SGI318616, SGI0, SGI318617, SGI318618, SGI0, SGI0, SGI0, SGI0, SGI318619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74260[] = {SGI318620, SGI0, SGI318621, SGI318622, SGI318623, SGI318624, SGI0, SGI318625, SGI318626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74261[] = {SGI318627, SGI0, SGI318628, SGI318629, SGI318630, SGI318631, SGI0, SGI318632, SGI318633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74262[] = {SGI318634, SGI0, SGI318635, SGI318636, SGI318637, SGI318638, SGI0, SGI318639, SGI318640, SGI0, SGI0, SGI0, SGI0, SGI318641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74263[] = {SGI318642, SGI0, SGI318643, SGI318644, SGI318645, SGI318646, SGI0, SGI318647, SGI318648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74264[] = {SGI318649, SGI0, SGI318650, SGI318651, SGI318652, SGI318653, SGI0, SGI0, SGI318654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74265[] = {SGI318655, SGI0, SGI318656, SGI318657, SGI318658, SGI318659, SGI0, SGI0, SGI318660, SGI0, SGI0, SGI0, SGI0, SGI318661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74266[] = {SGI318662, SGI0, SGI318663, SGI318664, SGI318665, SGI318666, SGI0, SGI318667, SGI318668, SGI0, SGI0, SGI0, SGI0, SGI318669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74267[] = {SGI318670, SGI0, SGI318671, SGI318672, SGI318673, SGI318674, SGI0, SGI318675, SGI318676, SGI0, SGI0, SGI0, SGI0, SGI318677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74268[] = {SGI318678, SGI0, SGI318679, SGI318680, SGI318681, SGI318682, SGI0, SGI318683, SGI318684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74269[] = {SGI318685, SGI0, SGI318686, SGI318687, SGI318688, SGI318689, SGI0, SGI0, SGI318690, SGI0, SGI0, SGI0, SGI0, SGI318691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74270[] = {SGI318692, SGI0, SGI318693, SGI318694, SGI318695, SGI318696, SGI0, SGI0, SGI318697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74271[] = {SGI318698, SGI0, SGI318699, SGI318700, SGI318701, SGI318702, SGI0, SGI0, SGI318703, SGI0, SGI0, SGI0, SGI0, SGI318704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74272[] = {SGI318705, SGI0, SGI318706, SGI318707, SGI318708, SGI318709, SGI0, SGI0, SGI318710, SGI0, SGI0, SGI0, SGI0, SGI318711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74273[] = {SGI318712, SGI0, SGI318713, SGI318714, SGI318715, SGI318716, SGI0, SGI0, SGI318717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74274[] = {SGI318718, SGI0, SGI318719, SGI318720, SGI318721, SGI318722, SGI0, SGI0, SGI318723, SGI0, SGI0, SGI0, SGI0, SGI318724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74275[] = {SGI318725, SGI0, SGI318726, SGI318727, SGI318728, SGI318729, SGI0, SGI0, SGI318730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74276[] = {SGI318731, SGI0, SGI318732, SGI318733, SGI318734, SGI318735, SGI0, SGI318736, SGI318737, SGI0, SGI0, SGI0, SGI0, SGI318738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74277[] = {SGI318739, SGI0, SGI318740, SGI318741, SGI318742, SGI318743, SGI0, SGI0, SGI318744, SGI0, SGI0, SGI0, SGI0, SGI318745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74278[] = {SGI318746, SGI0, SGI318747, SGI318748, SGI318749, SGI318750, SGI0, SGI0, SGI318751, SGI0, SGI0, SGI0, SGI0, SGI318752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74279[] = {SGI318753, SGI0, SGI318754, SGI318755, SGI318756, SGI318757, SGI0, SGI0, SGI318758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74280[] = {SGI318759, SGI0, SGI318760, SGI318761, SGI318762, SGI318763, SGI0, SGI318764, SGI318765, SGI0, SGI0, SGI0, SGI0, SGI318766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74281[] = {SGI318767, SGI0, SGI318768, SGI318769, SGI318770, SGI318771, SGI0, SGI0, SGI318772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74282[] = {SGI318773, SGI0, SGI318774, SGI318775, SGI318776, SGI318777, SGI0, SGI0, SGI318778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74283[] = {SGI318779, SGI0, SGI318780, SGI318781, SGI318782, SGI318783, SGI0, SGI0, SGI318784, SGI0, SGI0, SGI0, SGI0, SGI318785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74284[] = {SGI318786, SGI0, SGI318787, SGI318788, SGI318789, SGI318790, SGI0, SGI0, SGI318791, SGI0, SGI0, SGI0, SGI0, SGI318792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74285[] = {SGI318793, SGI0, SGI318794, SGI318795, SGI318796, SGI318797, SGI0, SGI0, SGI318798, SGI0, SGI0, SGI0, SGI0, SGI318799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74286[] = {SGI318800, SGI0, SGI318801, SGI318802, SGI318803, SGI318804, SGI0, SGI0, SGI318805, SGI0, SGI0, SGI0, SGI0, SGI318806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74287[] = {SGI318807, SGI0, SGI318808, SGI318809, SGI318810, SGI318811, SGI0, SGI0, SGI318812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74288[] = {SGI318813, SGI0, SGI318814, SGI318815, SGI318816, SGI318817, SGI0, SGI0, SGI318818, SGI0, SGI0, SGI0, SGI0, SGI318819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74289[] = {SGI318820, SGI0, SGI318821, SGI318822, SGI318823, SGI318824, SGI0, SGI0, SGI318825, SGI0, SGI0, SGI0, SGI0, SGI318826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74290[] = {SGI318827, SGI0, SGI318828, SGI318829, SGI318830, SGI318831, SGI0, SGI0, SGI318832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74291[] = {SGI318833, SGI0, SGI318834, SGI318835, SGI318836, SGI318837, SGI0, SGI0, SGI318838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74292[] = {SGI318839, SGI0, SGI318840, SGI318841, SGI318842, SGI318843, SGI0, SGI0, SGI318844, SGI0, SGI0, SGI0, SGI0, SGI318845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74293[] = {SGI318846, SGI0, SGI318847, SGI318848, SGI318849, SGI318850, SGI0, SGI0, SGI318851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74294[] = {SGI318852, SGI318852, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74295[] = {SGI318853, SGI318853, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74296[] = {SGI318854, SGI318854, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74297[] = {SGI318855, SGI0, SGI318856, SGI318857, SGI318858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74298[] = {SGI318859, SGI0, SGI318860, SGI318861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318862, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74299[] = {SGI318863, SGI0, SGI318863, SGI318864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318865, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74300[] = {SGI318866, SGI0, SGI318867, SGI318868, SGI318869, SGI318870, SGI318871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74301[] = {SGI318872, SGI0, SGI318873, SGI318874, SGI318875, SGI318876, SGI318877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318878, SGI0, SGI0, NULL};
const Segment * const SG74302[] = {SGI318879, SGI0, SGI318880, SGI318881, SGI318882, SGI318883, SGI318884, SGI318885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74303[] = {SGI318886, SGI0, SGI318887, SGI318888, SGI318889, SGI318890, SGI318891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318892, SGI0, SGI0, NULL};
const Segment * const SG74304[] = {SGI318893, SGI0, SGI318894, SGI318895, SGI318896, SGI318897, SGI318898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74305[] = {SGI318899, SGI0, SGI318900, SGI318901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318902, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74306[] = {SGI318903, SGI0, SGI318904, SGI318905, SGI318906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74307[] = {SGI318907, SGI0, SGI318908, SGI318909, SGI318910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74308[] = {SGI318911, SGI0, SGI318912, SGI318913, SGI318914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74309[] = {SGI318915, SGI0, SGI318916, SGI318917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318918, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74310[] = {SGI318919, SGI0, SGI318920, SGI318921, SGI318922, SGI318923, SGI0, SGI318924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74311[] = {SGI318925, SGI0, SGI318926, SGI318927, SGI318928, SGI318929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74312[] = {SGI318930, SGI0, SGI318931, SGI318932, SGI318933, SGI318934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74313[] = {SGI318935, SGI0, SGI318936, SGI318937, SGI318938, SGI318939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74314[] = {SGI318940, SGI0, SGI318941, SGI318942, SGI318943, SGI318944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74315[] = {SGI318945, SGI0, SGI318946, SGI318947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74316[] = {SGI318948, SGI0, SGI318949, SGI318950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74317[] = {SGI318951, SGI0, SGI318952, SGI318953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318954, SGI0, SGI0, NULL};
const Segment * const SG74318[] = {SGI318955, SGI0, SGI318956, SGI318957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318958, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74319[] = {SGI318959, SGI0, SGI318960, SGI318961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318963, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74320[] = {SGI318964, SGI0, SGI318965, SGI318966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74321[] = {SGI318968, SGI0, SGI318969, SGI318970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318971, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74322[] = {SGI318972, SGI0, SGI318973, SGI318974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318975, SGI318976, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74323[] = {SGI318977, SGI0, SGI318978, SGI318979, SGI318980, SGI318981, SGI318982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74324[] = {SGI318985, SGI0, SGI318986, SGI318987, SGI318988, SGI318989, SGI318990, SGI318991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI318993, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74325[] = {SGI318994, SGI0, SGI318995, SGI318996, SGI318997, SGI318998, SGI318999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74326[] = {SGI319001, SGI0, SGI319002, SGI319003, SGI319004, SGI319005, SGI319006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74327[] = {SGI319009, SGI0, SGI319010, SGI319011, SGI319012, SGI319013, SGI319014, SGI319015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74328[] = {SGI319017, SGI0, SGI319018, SGI319019, SGI319020, SGI319021, SGI319022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74329[] = {SGI319024, SGI0, SGI319025, SGI319026, SGI319027, SGI319028, SGI319029, SGI319030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74330[] = {SGI319033, SGI0, SGI319034, SGI319035, SGI319036, SGI319037, SGI319038, SGI319039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74331[] = {SGI319041, SGI0, SGI319042, SGI319043, SGI319044, SGI319045, SGI319046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319048, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74332[] = {SGI319049, SGI0, SGI319050, SGI319051, SGI319052, SGI319053, SGI319054, SGI319055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74333[] = {SGI319058, SGI0, SGI319059, SGI319060, SGI319061, SGI319062, SGI319063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74334[] = {SGI319065, SGI319066, SGI319067, SGI319068, SGI319069, SGI319070, SGI319071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74335[] = {SGI319073, SGI0, SGI319074, SGI319075, SGI319076, SGI319077, SGI319078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319080, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74336[] = {SGI319081, SGI319082, SGI319083, SGI319084, SGI319085, SGI319086, SGI319087, SGI319088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74337[] = {SGI319091, SGI0, SGI319092, SGI319093, SGI319094, SGI319095, SGI319096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74338[] = {SGI319099, SGI319100, SGI319101, SGI319102, SGI319103, SGI319104, SGI319105, SGI319106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74339[] = {SGI319109, SGI0, SGI319110, SGI319111, SGI319112, SGI319113, SGI319114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74340[] = {SGI319116, SGI0, SGI319117, SGI319118, SGI319119, SGI319120, SGI319121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74341[] = {SGI319123, SGI0, SGI319124, SGI319125, SGI319126, SGI319127, SGI319128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74342[] = {SGI319130, SGI0, SGI319131, SGI319132, SGI319133, SGI319134, SGI319135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319137, SGI0, SGI0, NULL};
const Segment * const SG74343[] = {SGI319138, SGI0, SGI319139, SGI319140, SGI319141, SGI319142, SGI319143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74344[] = {SGI319145, SGI0, SGI319146, SGI319147, SGI319148, SGI319149, SGI319150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74345[] = {SGI319152, SGI0, SGI319153, SGI319154, SGI319155, SGI319156, SGI319157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74346[] = {SGI319159, SGI0, SGI319160, SGI319161, SGI319162, SGI319163, SGI319164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74347[] = {SGI319167, SGI0, SGI319168, SGI319169, SGI319170, SGI319171, SGI319172, SGI319173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74348[] = {SGI319175, SGI0, SGI319176, SGI319177, SGI319178, SGI319179, SGI319180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74349[] = {SGI319182, SGI0, SGI319183, SGI319184, SGI319185, SGI319186, SGI319187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319189, SGI0, SGI0, NULL};
const Segment * const SG74350[] = {SGI319190, SGI0, SGI319191, SGI319192, SGI319193, SGI319194, SGI319195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319197, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74351[] = {SGI319198, SGI319198, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74352[] = {SGI319199, SGI0, SGI319200, SGI319201, SGI319202, SGI319203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319204, SGI0, SGI0, NULL};
const Segment * const SG74353[] = {SGI319205, SGI0, SGI319206, SGI319207, SGI319208, SGI319209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74354[] = {SGI319210, SGI0, SGI319211, SGI319212, SGI319213, SGI319214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319215, SGI0, SGI0, NULL};
const Segment * const SG74355[] = {SGI319216, SGI0, SGI319217, SGI319218, SGI319219, SGI319220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74356[] = {SGI319221, SGI0, SGI319222, SGI319223, SGI319224, SGI319225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319226, SGI0, SGI0, NULL};
const Segment * const SG74357[] = {SGI319227, SGI0, SGI319228, SGI319229, SGI319230, SGI319231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319232, SGI0, SGI0, NULL};
const Segment * const SG74358[] = {SGI319233, SGI0, SGI319234, SGI319235, SGI319236, SGI319237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319238, SGI0, SGI0, SGI0, SGI319239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74359[] = {SGI319240, SGI0, SGI319241, SGI319242, SGI319243, SGI319244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74360[] = {SGI319246, SGI0, SGI319247, SGI319248, SGI319249, SGI319250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319252, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74361[] = {SGI0, SGI0, SGI319253, SGI319254, SGI319255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74362[] = {SGI319256, SGI319256, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74363[] = {SGI319257, SGI319257, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74364[] = {SGI319258, SGI0, SGI319259, SGI319260, SGI319261, SGI319262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74365[] = {SGI319264, SGI0, SGI319265, SGI319266, SGI319267, SGI319268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319269, SGI0, SGI0, SGI0, SGI319270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74366[] = {SGI319271, SGI0, SGI319272, SGI319273, SGI319274, SGI319275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319277, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74367[] = {SGI319278, SGI0, SGI319279, SGI319280, SGI319281, SGI319282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74368[] = {SGI319284, SGI0, SGI319285, SGI319286, SGI319287, SGI319288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74369[] = {SGI319290, SGI0, SGI319291, SGI319292, SGI319293, SGI319294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319296, SGI0, SGI0, NULL};
const Segment * const SG74370[] = {SGI319297, SGI0, SGI319298, SGI319299, SGI319300, SGI319301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319303, SGI0, SGI0, NULL};
const Segment * const SG74371[] = {SGI319304, SGI0, SGI319305, SGI319306, SGI319307, SGI319308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74372[] = {SGI319310, SGI0, SGI319311, SGI319312, SGI319313, SGI319314, SGI319315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74373[] = {SGI319317, SGI0, SGI319318, SGI319319, SGI319320, SGI319321, SGI319322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74374[] = {SGI319324, SGI0, SGI319325, SGI319326, SGI319327, SGI319328, SGI319329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74375[] = {SGI319331, SGI0, SGI319332, SGI319333, SGI319334, SGI319335, SGI319336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319338, SGI0, SGI0, NULL};
const Segment * const SG74376[] = {SGI319339, SGI0, SGI319340, SGI319341, SGI319342, SGI319343, SGI319344, SGI319345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319347, SGI0, SGI0, NULL};
const Segment * const SG74377[] = {SGI319348, SGI319349, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74378[] = {SGI319350, SGI319350, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74379[] = {SGI319351, SGI319352, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74380[] = {SGI319353, SGI319354, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74381[] = {SGI319355, SGI319355, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74382[] = {SGI319356, SGI0, SGI319357, SGI319358, SGI319359, SGI319360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74383[] = {SGI319361, SGI0, SGI319362, SGI319363, SGI319364, SGI319365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319366, SGI0, NULL};
const Segment * const SG74384[] = {SGI319367, SGI0, SGI319368, SGI319369, SGI319370, SGI319371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319372, SGI0, NULL};
const Segment * const SG74385[] = {SGI319373, SGI0, SGI319374, SGI319375, SGI319376, SGI319377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319378, SGI319379, SGI0, NULL};
const Segment * const SG74386[] = {SGI319380, SGI0, SGI319381, SGI319382, SGI319383, SGI319384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319385, SGI319386, SGI0, NULL};
const Segment * const SG74387[] = {SGI319387, SGI0, SGI319388, SGI319389, SGI319390, SGI319391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319392, SGI0, NULL};
const Segment * const SG74388[] = {SGI319393, SGI0, SGI319394, SGI319395, SGI319396, SGI319397, SGI0, SGI319398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319399, SGI0, NULL};
const Segment * const SG74389[] = {SGI319400, SGI0, SGI319401, SGI319402, SGI319403, SGI319404, SGI0, SGI319405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319406, SGI0, NULL};
const Segment * const SG74390[] = {SGI319407, SGI0, SGI319408, SGI319409, SGI319410, SGI319411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319412, SGI319413, SGI0, NULL};
const Segment * const SG74391[] = {SGI319414, SGI0, SGI319415, SGI319416, SGI319417, SGI319418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319420, SGI0, NULL};
const Segment * const SG74392[] = {SGI319421, SGI0, SGI319422, SGI319423, SGI319424, SGI319425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319427, SGI0, NULL};
const Segment * const SG74393[] = {SGI319428, SGI0, SGI319429, SGI319430, SGI319431, SGI319432, SGI0, SGI319433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319435, SGI0, NULL};
const Segment * const SG74394[] = {SGI319436, SGI0, SGI319437, SGI319438, SGI319439, SGI319440, SGI0, SGI319441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319443, SGI0, NULL};
const Segment * const SG74395[] = {SGI319444, SGI0, SGI319445, SGI319446, SGI319447, SGI319448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319450, SGI0, NULL};
const Segment * const SG74396[] = {SGI319451, SGI0, SGI319452, SGI319453, SGI319454, SGI319455, SGI0, SGI319456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319457, SGI0, SGI319458, SGI0, NULL};
const Segment * const SG74397[] = {SGI319459, SGI0, SGI319460, SGI319461, SGI319462, SGI319463, SGI0, SGI319464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319466, SGI0, NULL};
const Segment * const SG74398[] = {SGI319467, SGI0, SGI319468, SGI319469, SGI319470, SGI319471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319473, SGI0, NULL};
const Segment * const SG74399[] = {SGI319474, SGI0, SGI319475, SGI319476, SGI319477, SGI319478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319480, SGI0, NULL};
const Segment * const SG74400[] = {SGI319481, SGI0, SGI319482, SGI319483, SGI319484, SGI319485, SGI0, SGI319486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319487, SGI0, SGI319488, SGI0, NULL};
const Segment * const SG74401[] = {SGI319489, SGI0, SGI319490, SGI319491, SGI319492, SGI319493, SGI0, SGI319494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319495, SGI0, NULL};
const Segment * const SG74402[] = {SGI319496, SGI0, SGI319497, SGI319498, SGI319499, SGI319500, SGI0, SGI319501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319503, SGI0, NULL};
const Segment * const SG74403[] = {SGI319504, SGI0, SGI319505, SGI319506, SGI319507, SGI319508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319509, SGI0, NULL};
const Segment * const SG74404[] = {SGI319510, SGI0, SGI319511, SGI319512, SGI319513, SGI319514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319516, SGI0, NULL};
const Segment * const SG74405[] = {SGI319517, SGI0, SGI319518, SGI319519, SGI319520, SGI319521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319523, SGI0, NULL};
const Segment * const SG74406[] = {SGI319524, SGI0, SGI319525, SGI319526, SGI319527, SGI319528, SGI0, SGI319529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319531, SGI0, NULL};
const Segment * const SG74407[] = {SGI319532, SGI0, SGI319533, SGI319534, SGI319535, SGI319536, SGI0, SGI319537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319538, SGI0, NULL};
const Segment * const SG74408[] = {SGI319539, SGI0, SGI319540, SGI319541, SGI319542, SGI319543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319544, SGI0, NULL};
const Segment * const SG74409[] = {SGI319545, SGI0, SGI319546, SGI319547, SGI319548, SGI319549, SGI0, SGI319550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319551, SGI319552, SGI0, NULL};
const Segment * const SG74410[] = {SGI319553, SGI0, SGI319554, SGI319555, SGI319556, SGI319557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319558, SGI0, NULL};
const Segment * const SG74411[] = {SGI319559, SGI0, SGI319560, SGI319561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74412[] = {SGI319563, SGI0, SGI319564, SGI319565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74413[] = {SGI319567, SGI0, SGI319568, SGI319569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319571, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74414[] = {SGI319572, SGI0, SGI319573, SGI319574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319576, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74415[] = {SGI319577, SGI0, SGI319578, SGI319579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319581, SGI0, SGI0, NULL};
const Segment * const SG74416[] = {SGI319582, SGI0, SGI319583, SGI319584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319585, SGI0, SGI0, SGI0, SGI319586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74417[] = {SGI319587, SGI0, SGI319588, SGI319589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319591, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74418[] = {SGI319592, SGI0, SGI319593, SGI319594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319596, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74419[] = {SGI319597, SGI0, SGI319598, SGI319599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319601, SGI0, SGI0, NULL};
const Segment * const SG74420[] = {SGI319602, SGI0, SGI319603, SGI319604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319605, SGI0, SGI0, SGI0, SGI319606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74421[] = {SGI319607, SGI0, SGI319608, SGI319609, SGI319610, SGI319611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74422[] = {SGI319612, SGI0, SGI319613, SGI319614, SGI319615, SGI319616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74423[] = {SGI319617, SGI0, SGI319618, SGI319619, SGI319620, SGI319621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74424[] = {SGI319623, SGI0, SGI319624, SGI319625, SGI319626, SGI319627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319628, SGI0, NULL};
const Segment * const SG74425[] = {SGI319629, SGI0, SGI319630, SGI319631, SGI319632, SGI319633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319634, SGI319635, SGI0, NULL};
const Segment * const SG74426[] = {SGI319636, SGI0, SGI319637, SGI319638, SGI319639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319640, SGI0, NULL};
const Segment * const SG74427[] = {SGI319641, SGI0, SGI319642, SGI319643, SGI319644, SGI319645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319646, SGI319647, SGI0, NULL};
const Segment * const SG74428[] = {SGI319648, SGI0, SGI319649, SGI319650, SGI319651, SGI319652, SGI0, SGI319653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319654, SGI0, NULL};
const Segment * const SG74429[] = {SGI319655, SGI0, SGI319656, SGI319657, SGI319658, SGI319659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319660, SGI319661, SGI0, NULL};
const Segment * const SG74430[] = {SGI319662, SGI319662, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74431[] = {SGI319663, SGI0, SGI319664, SGI319665, SGI319666, SGI319667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74432[] = {SGI319669, NULL};
const Segment * const SG74433[] = {SGI319670, SGI0, SGI319671, SGI319672, SGI319673, SGI319674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319676, SGI0, SGI0, NULL};
const Segment * const SG74434[] = {SGI319677, SGI0, SGI319678, SGI319679, SGI319680, SGI319681, SGI0, SGI319682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319684, SGI0, SGI0, NULL};
const Segment * const SG74435[] = {SGI319685, SGI0, SGI319686, SGI319687, SGI319688, SGI319689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74436[] = {SGI319691, SGI0, SGI319692, SGI319693, SGI319694, SGI319695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74437[] = {SGI319697, SGI0, SGI319698, SGI319699, SGI319700, SGI319701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319703, SGI0, SGI0, NULL};
const Segment * const SG74438[] = {SGI319704, SGI0, SGI319705, SGI319706, SGI319707, SGI319708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319710, SGI0, SGI0, NULL};
const Segment * const SG74439[] = {SGI319711, SGI0, SGI319712, SGI319713, SGI319714, SGI319715, SGI0, SGI319716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319718, SGI0, SGI0, NULL};
const Segment * const SG74440[] = {SGI319719, SGI0, SGI0, SGI0, SGI319720, SGI319721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319723, SGI0, SGI0, NULL};
const Segment * const SG74441[] = {SGI319724, SGI0, SGI319725, SGI319726, SGI319727, SGI319728, SGI0, SGI319729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319730, SGI0, SGI0, SGI0, SGI319731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74442[] = {SGI319732, SGI0, SGI319733, SGI319734, SGI319735, SGI319736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319738, SGI0, SGI0, NULL};
const Segment * const SG74443[] = {SGI319739, SGI0, SGI319740, SGI319741, SGI319742, SGI319743, SGI0, SGI319744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74444[] = {SGI319746, SGI319746, SGI0, SGI0, SGI0, SGI319747, SGI0, NULL};
const Segment * const SG74445[] = {SGI319748, SGI319748, SGI0, SGI0, SGI0, SGI319749, SGI0, NULL};
const Segment * const SG74446[] = {SGI319750, SGI0, SGI319751, SGI319752, SGI319753, SGI319754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319756, SGI0, SGI0, NULL};
const Segment * const SG74447[] = {SGI319757, SGI0, SGI319758, SGI319759, SGI319760, SGI319761, SGI0, SGI319762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319763, SGI0, SGI319764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74448[] = {SGI319765, SGI0, SGI319766, SGI319767, SGI319768, SGI319769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319771, SGI0, SGI0, NULL};
const Segment * const SG74449[] = {SGI319772, SGI0, SGI319773, SGI319774, SGI319775, SGI319776, SGI0, SGI319777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319779, SGI0, SGI0, NULL};
const Segment * const SG74450[] = {SGI319780, SGI0, SGI319781, SGI319782, SGI319783, SGI319784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74451[] = {SGI319786, SGI0, SGI319787, SGI319788, SGI319789, SGI319790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74452[] = {SGI319792, SGI0, SGI319793, SGI319794, SGI319795, SGI319796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74453[] = {SGI319798, SGI0, SGI319799, SGI319800, SGI319801, SGI319802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74454[] = {SGI319804, SGI0, SGI319805, SGI319806, SGI319807, SGI319808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74455[] = {SGI319810, SGI0, SGI319811, SGI319812, SGI319813, SGI319814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74456[] = {SGI319816, SGI0, SGI319817, SGI319818, SGI319819, SGI319820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319822, SGI0, SGI0, NULL};
const Segment * const SG74457[] = {SGI319823, SGI0, SGI319824, SGI319825, SGI319826, SGI319827, SGI0, SGI319828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319829, SGI0, SGI319830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74458[] = {SGI319831, SGI0, SGI319832, SGI319833, SGI319834, SGI319835, SGI0, SGI319836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74459[] = {SGI319838, SGI0, SGI319839, SGI319840, SGI319841, SGI319842, SGI0, SGI319843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74460[] = {SGI319845, SGI0, SGI319846, SGI319847, SGI319848, SGI319849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74461[] = {SGI319851, SGI0, SGI319852, SGI319853, SGI319854, SGI319855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319856, SGI0, SGI319857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74462[] = {SGI319858, SGI0, SGI319859, SGI319860, SGI319861, SGI319862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319863, SGI0, SGI319864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74463[] = {SGI319865, SGI0, SGI319866, SGI319867, SGI319868, SGI319869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319871, SGI0, SGI0, NULL};
const Segment * const SG74464[] = {SGI319872, SGI0, SGI319873, SGI319874, SGI319875, SGI319876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74465[] = {SGI319878, SGI0, SGI319879, SGI319880, SGI319881, SGI319882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319883, SGI0, SGI319884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74466[] = {SGI319885, SGI0, SGI319886, SGI319887, SGI319888, SGI319889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319891, SGI0, SGI0, NULL};
const Segment * const SG74467[] = {SGI319892, SGI0, SGI319893, SGI319894, SGI319895, SGI319896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74468[] = {SGI319898, SGI0, SGI319899, SGI319900, SGI319901, SGI319902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319904, SGI0, SGI0, NULL};
const Segment * const SG74469[] = {SGI319905, SGI0, SGI319906, SGI319907, SGI319908, SGI319909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319910, SGI0, SGI319911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74470[] = {SGI319912, SGI0, SGI319913, SGI319914, SGI319915, SGI319916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74471[] = {SGI319918, SGI0, SGI319919, SGI319920, SGI319921, SGI319922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319923, SGI0, SGI319924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74472[] = {SGI319925, SGI0, SGI319926, SGI319927, SGI319928, SGI319929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74473[] = {SGI319931, SGI0, SGI319932, SGI319933, SGI319934, SGI319935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319937, SGI0, SGI0, NULL};
const Segment * const SG74474[] = {SGI319938, SGI0, SGI319939, SGI319940, SGI319941, SGI319942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319943, SGI0, SGI319944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74475[] = {SGI319945, SGI0, SGI319946, SGI319947, SGI319948, SGI319949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74476[] = {SGI319951, SGI0, SGI319952, SGI319953, SGI319954, SGI319955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319956, SGI0, SGI319957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74477[] = {SGI319958, SGI0, SGI319959, SGI319960, SGI319961, SGI319962, SGI319963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74478[] = {SGI319965, SGI0, SGI319966, SGI319967, SGI319968, SGI319969, SGI319970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74479[] = {SGI319973, SGI0, SGI319974, SGI319975, SGI319976, SGI319977, SGI319978, SGI319979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74480[] = {SGI319981, SGI0, SGI319982, SGI319983, SGI319984, SGI319985, SGI319986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74481[] = {SGI319988, SGI0, SGI319989, SGI319990, SGI319991, SGI319992, SGI319993, SGI319994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI319996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74482[] = {SGI319997, SGI319998, SGI319999, SGI320000, SGI320001, SGI320002, SGI320003, SGI320004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74483[] = {SGI320007, SGI0, SGI320008, SGI320009, SGI320010, SGI320011, SGI320012, SGI320013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320015, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74484[] = {SGI320016, SGI0, SGI320017, SGI320018, SGI320019, SGI320020, SGI320021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74485[] = {SGI320024, SGI0, SGI320025, SGI320026, SGI320027, SGI320028, SGI320029, SGI320030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74486[] = {SGI320032, SGI0, SGI320033, SGI320034, SGI320035, SGI320036, SGI320037, SGI320038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74487[] = {SGI320041, SGI0, SGI320042, SGI320043, SGI320044, SGI320045, SGI320046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74488[] = {SGI320049, SGI320050, SGI320051, SGI320052, SGI320053, SGI320054, SGI320055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74489[] = {SGI320058, SGI0, SGI320059, SGI320060, SGI320061, SGI320062, SGI320063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74490[] = {SGI320066, SGI0, SGI320067, SGI320068, SGI320069, SGI320070, SGI320071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74491[] = {SGI320074, SGI0, SGI320075, SGI320076, SGI320077, SGI320078, SGI320079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74492[] = {SGI320081, SGI0, SGI320082, SGI320083, SGI320084, SGI320085, SGI320086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74493[] = {SGI320088, SGI0, SGI320089, SGI320090, SGI320091, SGI320092, SGI320093, SGI320094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74494[] = {SGI320097, SGI0, SGI320098, SGI320099, SGI320100, SGI320101, SGI320102, SGI320103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74495[] = {SGI320105, SGI0, SGI320106, SGI320107, SGI320108, SGI320109, SGI320110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74496[] = {SGI320113, SGI0, SGI320114, SGI320115, SGI320116, SGI320117, SGI320118, SGI320119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74497[] = {SGI320121, SGI0, SGI320122, SGI320123, SGI320124, SGI320125, SGI320126, SGI320127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74498[] = {SGI320130, SGI0, SGI320131, SGI320132, SGI320133, SGI320134, SGI320135, SGI320136, SGI0, SGI0, SGI320137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74499[] = {SGI320139, SGI0, SGI320140, SGI320141, SGI320142, SGI320143, SGI320144, SGI320145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320147, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74500[] = {SGI320148, SGI320149, NULL};
const Segment * const SG74501[] = {SGI320150, SGI0, SGI320151, SGI320152, SGI320153, SGI320154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74502[] = {SGI320156, SGI0, SGI320157, SGI320158, SGI320159, SGI320160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74503[] = {SGI320162, SGI0, SGI320163, SGI320164, SGI320165, SGI320166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74504[] = {SGI320168, SGI0, SGI320169, SGI320170, SGI320171, SGI320172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74505[] = {SGI320174, SGI0, SGI320175, SGI320176, SGI320177, SGI320178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74506[] = {SGI320180, SGI0, SGI320181, SGI320182, SGI320183, SGI320184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74507[] = {SGI320186, SGI0, SGI320187, SGI320188, SGI320189, SGI320190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74508[] = {SGI320192, SGI0, SGI320193, SGI320194, SGI320195, SGI320196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74509[] = {SGI320198, SGI0, SGI320199, SGI320200, SGI320201, SGI320202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320203, SGI0, SGI320204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74510[] = {SGI320205, SGI0, SGI320206, SGI320207, SGI320208, SGI320209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74511[] = {SGI320211, SGI0, SGI320212, SGI320213, SGI320214, SGI320215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320217, SGI0, SGI0, NULL};
const Segment * const SG74512[] = {SGI320218, SGI0, SGI320219, SGI320220, SGI320221, SGI320222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320223, SGI0, SGI320224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74513[] = {SGI320225, SGI0, SGI320226, SGI320227, SGI320228, SGI320229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320230, SGI0, SGI320231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74514[] = {SGI320232, SGI0, SGI320233, SGI320234, SGI320235, SGI320236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320237, SGI0, SGI320238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74515[] = {SGI320239, SGI0, SGI320240, SGI320241, SGI320242, SGI320243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320245, SGI0, SGI0, NULL};
const Segment * const SG74516[] = {SGI320246, SGI0, SGI320247, SGI320248, SGI320249, SGI320250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320252, SGI0, SGI0, NULL};
const Segment * const SG74517[] = {SGI320253, SGI0, SGI320254, SGI320255, SGI320256, SGI320257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320258, SGI0, SGI320259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74518[] = {SGI320260, SGI0, SGI320261, SGI320262, SGI320263, SGI320264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74519[] = {SGI320266, SGI0, SGI320267, SGI320268, SGI320269, SGI320270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320272, SGI0, SGI0, NULL};
const Segment * const SG74520[] = {SGI320273, SGI0, SGI320274, SGI320275, SGI320276, SGI320277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74521[] = {SGI320279, SGI0, SGI320280, SGI320281, SGI320282, SGI320283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320284, SGI0, SGI320285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74522[] = {SGI320286, SGI0, SGI320287, SGI320288, SGI320289, SGI320290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74523[] = {SGI320292, SGI0, SGI320293, SGI320294, SGI320295, SGI320296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320297, SGI0, NULL};
const Segment * const SG74524[] = {SGI320298, SGI0, SGI320299, SGI320300, SGI320301, SGI320302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320303, SGI320304, SGI0, NULL};
const Segment * const SG74525[] = {SGI320305, SGI0, SGI320306, SGI320307, SGI320308, SGI320309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320310, SGI320311, SGI0, NULL};
const Segment * const SG74526[] = {SGI320312, SGI0, SGI320313, SGI320314, SGI320315, SGI320316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320317, SGI0, NULL};
const Segment * const SG74527[] = {SGI320318, SGI0, SGI320319, SGI320320, SGI320321, SGI320322, SGI320323, SGI320324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74528[] = {SGI320327, SGI0, SGI320328, SGI320329, SGI320330, SGI320331, SGI320332, SGI320333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320335, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74529[] = {SGI320336, SGI0, SGI320337, SGI320338, SGI320339, SGI320340, SGI320341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74530[] = {SGI320344, SGI0, SGI320345, SGI320346, SGI320347, SGI320348, SGI320349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74531[] = {SGI320351, SGI0, SGI320352, SGI320353, SGI320354, SGI320355, SGI320356, SGI320357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74532[] = {SGI320359, SGI0, SGI320360, SGI320361, SGI320362, SGI320363, SGI320364, SGI320365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74533[] = {SGI320368, SGI0, SGI320369, SGI320370, SGI320371, SGI320372, SGI320373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74534[] = {SGI320376, SGI0, SGI320377, SGI320378, SGI320379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74535[] = {SGI320381, SGI0, SGI320382, SGI320383, SGI320384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74536[] = {SGI320386, SGI0, SGI320387, SGI320388, SGI320389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320391, SGI0, SGI0, NULL};
const Segment * const SG74537[] = {SGI320392, SGI0, SGI320393, SGI320394, SGI320395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74538[] = {SGI320397, SGI0, SGI320398, SGI320399, SGI320400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74539[] = {SGI320402, SGI0, SGI320403, SGI320404, SGI320405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320407, SGI0, SGI0, NULL};
const Segment * const SG74540[] = {SGI320408, SGI320408, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74541[] = {SGI320409, SGI0, SGI320410, SGI320411, SGI320412, SGI320413, SGI320414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74542[] = {SGI320416, SGI0, SGI320417, SGI320418, SGI320419, SGI320420, SGI320421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74543[] = {SGI320424, SGI0, SGI320425, SGI320426, SGI320427, SGI320428, SGI320429, SGI320430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74544[] = {SGI320433, SGI0, SGI320434, SGI320435, SGI320436, SGI320437, SGI320438, SGI320439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74545[] = {SGI320441, SGI0, SGI320442, SGI320443, SGI320444, SGI320445, SGI320446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74546[] = {SGI320448, SGI0, SGI320449, SGI320450, SGI320451, SGI320452, SGI320453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74547[] = {SGI320456, SGI0, SGI320457, SGI320458, SGI320459, SGI320460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74548[] = {SGI320461, SGI0, SGI320462, SGI320463, SGI320464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74549[] = {SGI320465, SGI0, SGI320466, SGI320467, SGI320468, SGI320469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74550[] = {SGI320470, SGI0, SGI320471, SGI320472, SGI320473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74551[] = {SGI320474, SGI0, SGI320475, SGI320476, SGI320477, SGI320478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74552[] = {SGI320479, SGI0, SGI320480, SGI320481, SGI320482, SGI320483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74553[] = {SGI320484, SGI0, SGI320485, SGI320486, SGI320487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74554[] = {SGI320488, SGI0, SGI320489, SGI320490, SGI320491, SGI320492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74555[] = {SGI320493, SGI0, SGI320494, SGI320495, SGI320496, SGI320497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74556[] = {SGI320498, SGI0, SGI320498, SGI320499, SGI320500, SGI320501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74557[] = {SGI320502, SGI0, SGI320502, SGI320503, SGI320504, SGI320505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74558[] = {SGI320506, SGI0, SGI320507, SGI320508, SGI320509, SGI320510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74559[] = {SGI320511, SGI0, SGI320512, SGI320513, SGI320514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74560[] = {SGI320516, SGI0, SGI320517, SGI320518, SGI320519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74561[] = {SGI320521, SGI0, SGI320522, SGI320523, SGI320524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320526, SGI0, SGI0, NULL};
const Segment * const SG74562[] = {SGI320527, SGI0, SGI320528, SGI320529, SGI320530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74563[] = {SGI320532, SGI0, SGI320533, SGI320534, SGI320535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74564[] = {SGI320537, SGI0, SGI320538, SGI320539, SGI320540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320542, SGI0, SGI0, NULL};
const Segment * const SG74565[] = {SGI320543, SGI0, SGI320544, SGI320545, SGI320546, SGI320547, SGI320548, SGI320549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74566[] = {SGI320551, SGI0, SGI320552, SGI320553, SGI320554, SGI320555, SGI320556, SGI320557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74567[] = {SGI320559, SGI0, SGI320560, SGI320561, SGI320562, SGI320563, SGI320564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74568[] = {SGI320567, SGI0, SGI320568, SGI320569, SGI320570, SGI320571, SGI320572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74569[] = {SGI320574, SGI0, SGI320575, SGI320576, SGI320577, SGI320578, SGI320579, SGI320580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74570[] = {SGI320583, SGI0, SGI320584, SGI320585, SGI320586, SGI320587, SGI320588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74571[] = {SGI320590, SGI0, SGI320591, SGI320592, SGI320593, SGI320594, SGI320595, SGI320596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320598, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74572[] = {SGI320599, SGI0, SGI320600, SGI320601, SGI320602, SGI320603, SGI320604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74573[] = {SGI320607, SGI0, SGI320608, SGI320609, SGI320610, SGI320611, SGI320612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74574[] = {SGI320614, SGI0, SGI320615, SGI320616, SGI320617, SGI320618, SGI320619, SGI320620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74575[] = {SGI320623, SGI0, SGI320624, SGI320625, SGI320626, SGI320627, SGI320628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74576[] = {SGI320630, SGI0, SGI320631, SGI320632, SGI320633, SGI320634, SGI320635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74577[] = {SGI320638, SGI0, SGI320639, SGI320640, SGI320641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74578[] = {SGI320642, SGI0, SGI320643, SGI320644, SGI320645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74579[] = {SGI320646, SGI0, SGI320647, SGI320648, SGI320649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320650, SGI0, SGI0, NULL};
const Segment * const SG74580[] = {SGI320651, SGI0, SGI320652, SGI320653, SGI320654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320655, SGI0, SGI0, NULL};
const Segment * const SG74581[] = {SGI320656, SGI0, SGI320657, SGI320658, SGI320659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74582[] = {SGI320660, SGI0, SGI320661, SGI320662, SGI320663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320664, SGI0, SGI0, NULL};
const Segment * const SG74583[] = {SGI320665, SGI0, SGI320666, SGI320667, SGI320668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320669, SGI0, SGI0, NULL};
const Segment * const SG74584[] = {SGI320670, SGI0, SGI320671, SGI320672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320673, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74585[] = {SGI320674, SGI0, SGI320675, SGI320676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74586[] = {SGI320677, SGI0, SGI320678, SGI320679, SGI320680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74587[] = {SGI320681, SGI0, SGI320681, SGI320682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320683, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74588[] = {SGI320684, SGI0, SGI320685, SGI320686, SGI320687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74589[] = {SGI320688, SGI320688, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74590[] = {SGI320689, SGI0, SGI320690, SGI320691, SGI320692, SGI320693, SGI320694, SGI320695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74591[] = {SGI320698, SGI0, SGI320699, SGI320700, SGI320701, SGI320702, SGI320703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74592[] = {SGI320706, SGI0, SGI320707, SGI320708, SGI320709, SGI320710, SGI320711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74593[] = {SGI320713, SGI0, SGI320714, SGI320715, SGI320716, SGI320717, SGI320718, SGI320719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74594[] = {SGI320721, SGI0, SGI320722, SGI320723, SGI320724, SGI320725, SGI320726, SGI320727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320729, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74595[] = {SGI320730, SGI0, SGI320731, SGI320732, SGI320733, SGI320734, SGI320735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74596[] = {SGI320738, SGI0, SGI320739, SGI320740, SGI320741, SGI320742, SGI320743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320745, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74597[] = {SGI320746, SGI0, SGI320747, SGI320748, SGI320749, SGI320750, SGI320751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74598[] = {SGI320753, SGI0, SGI320754, SGI320755, SGI320756, SGI320757, SGI0, SGI320758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74599[] = {SGI320760, SGI0, SGI320761, SGI320762, SGI320763, SGI320764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320765, SGI0, SGI0, NULL};
const Segment * const SG74600[] = {SGI320766, SGI0, SGI320767, SGI320768, SGI320769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74601[] = {SGI320770, SGI0, SGI320771, SGI320772, SGI320773, SGI320774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74602[] = {SGI320775, SGI0, SGI320776, SGI320777, SGI320778, SGI320779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74603[] = {SGI320780, SGI0, SGI320781, SGI320782, SGI320783, SGI320784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74604[] = {SGI320786, SGI0, SGI320787, SGI320788, SGI320789, SGI320790, SGI0, SGI320791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74605[] = {SGI320792, SGI0, SGI320793, SGI320794, SGI320795, SGI320796, SGI0, SGI320797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320798, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74606[] = {SGI320799, SGI0, SGI320800, SGI320801, SGI320802, SGI320803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320804, SGI0, SGI0, NULL};
const Segment * const SG74607[] = {SGI320805, SGI0, SGI320806, SGI320807, SGI320808, SGI320809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320810, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74608[] = {SGI320811, SGI0, SGI320812, SGI320813, SGI320814, SGI320815, SGI0, SGI320816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320817, SGI0, SGI0, NULL};
const Segment * const SG74609[] = {SGI320818, SGI0, SGI320819, SGI320820, SGI320821, SGI320822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74610[] = {SGI320824, SGI0, SGI320825, SGI320826, SGI320827, SGI320828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74611[] = {SGI320829, SGI320829, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74612[] = {SGI320830, SGI0, SGI320831, SGI320832, SGI320833, SGI320834, SGI320835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74613[] = {SGI320837, SGI0, SGI320838, SGI320839, SGI320840, SGI320841, SGI320842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74614[] = {SGI320845, SGI0, SGI320846, SGI320847, SGI320848, SGI320849, SGI320850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74615[] = {SGI320853, SGI0, SGI320854, SGI320855, SGI320856, SGI320857, SGI320858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74616[] = {SGI320861, SGI0, SGI320862, SGI320863, SGI320864, SGI320865, SGI320866, SGI320867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74617[] = {SGI320870, SGI0, SGI320871, SGI320872, SGI320873, SGI320874, SGI320875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74618[] = {SGI320878, SGI0, SGI320879, SGI320880, SGI320881, SGI320882, SGI320883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74619[] = {SGI320885, SGI0, SGI320886, SGI320887, SGI320888, SGI320889, SGI320890, SGI320891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74620[] = {SGI320894, SGI0, SGI320895, SGI320896, SGI320897, SGI320898, SGI320899, SGI320900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74621[] = {SGI320902, SGI0, SGI320903, SGI320904, SGI320905, SGI320906, SGI320907, SGI320908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320910, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74622[] = {SGI320911, SGI0, SGI320912, SGI320913, SGI320914, SGI320915, SGI320916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74623[] = {SGI320919, SGI0, SGI320920, SGI320921, SGI320922, SGI320923, SGI320924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74624[] = {SGI320926, SGI0, SGI320927, SGI320928, SGI320929, SGI320930, SGI320931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74625[] = {SGI320933, SGI0, SGI320934, SGI320935, SGI320936, SGI320937, SGI320938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320940, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74626[] = {SGI320941, SGI0, SGI320942, SGI320943, SGI320944, SGI320945, SGI320946, SGI320947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320948, SGI0, SGI0, SGI0, SGI320949, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74627[] = {SGI320950, SGI0, SGI320951, SGI320952, SGI320953, SGI320954, SGI320955, SGI320956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74628[] = {SGI320958, SGI0, SGI320959, SGI320960, SGI320961, SGI320962, SGI320963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74629[] = {SGI320966, SGI0, SGI320967, SGI320968, SGI320969, SGI320970, SGI320971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74630[] = {SGI320973, SGI320973, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74631[] = {SGI320974, SGI320975, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74632[] = {SGI320976, SGI320977, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74633[] = {SGI320978, NULL};
const Segment * const SG74634[] = {SGI320979, NULL};
const Segment * const SG74635[] = {SGI320980, NULL};
const Segment * const SG74636[] = {SGI320981, NULL};
const Segment * const SG74637[] = {SGI320982, NULL};
const Segment * const SG74638[] = {SGI320983, NULL};
const Segment * const SG74639[] = {SGI320984, NULL};
const Segment * const SG74640[] = {SGI320985, NULL};
const Segment * const SG74641[] = {SGI320986, NULL};
const Segment * const SG74642[] = {SGI320987, SGI0, SGI320988, SGI320989, SGI320990, SGI320991, SGI320992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI320994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74643[] = {SGI320995, SGI0, SGI320996, SGI320997, SGI320998, SGI320999, SGI321000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74644[] = {SGI321002, SGI0, SGI321003, SGI321004, SGI321005, SGI321006, SGI321007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74645[] = {SGI321010, SGI0, SGI321011, SGI321012, SGI321013, SGI321014, SGI321015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74646[] = {SGI321017, SGI0, SGI321018, SGI321019, SGI321020, SGI321021, SGI321022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74647[] = {SGI321025, SGI0, SGI321026, SGI321027, SGI321028, SGI321029, SGI321030, SGI321031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74648[] = {SGI321033, SGI0, SGI321034, SGI321035, SGI321036, SGI321037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321038, SGI321039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321040, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74649[] = {SGI321041, SGI0, SGI321042, SGI321043, SGI321044, SGI321045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321046, SGI321047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321048, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74650[] = {SGI321049, SGI0, SGI321050, SGI321051, SGI321052, SGI321053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321055, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74651[] = {SGI321056, SGI0, SGI321057, SGI321058, SGI321059, SGI321060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321062, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74652[] = {SGI321063, NULL};
const Segment * const SG74653[] = {SGI321064, NULL};
const Segment * const SG74654[] = {SGI321065, NULL};
const Segment * const SG74655[] = {SGI321066, NULL};
const Segment * const SG74656[] = {SGI321067, SGI321068, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74657[] = {SGI321069, SGI321069, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74658[] = {SGI321070, NULL};
const Segment * const SG74659[] = {SGI321071, SGI0, SGI321072, SGI321073, SGI321074, SGI321075, SGI321076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74660[] = {SGI321078, SGI0, SGI321079, SGI321080, SGI321081, SGI321082, SGI321083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74661[] = {SGI321086, SGI0, SGI321087, SGI321088, SGI321089, SGI321090, SGI321091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74662[] = {SGI321093, SGI0, SGI321094, SGI321095, SGI321096, SGI321097, SGI321098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74663[] = {SGI321100, SGI0, SGI321101, SGI321102, SGI321103, SGI321104, SGI321105, SGI321106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74664[] = {SGI321108, SGI0, SGI321109, SGI321110, SGI321111, SGI321112, SGI321113, SGI321114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74665[] = {SGI321116, NULL};
const Segment * const SG74666[] = {SGI321117, SGI321117, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74667[] = {SGI321118, SGI321118, SGI0, SGI0, SGI0, SGI321119, SGI0, NULL};
const Segment * const SG74668[] = {SGI321120, SGI321120, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74669[] = {SGI321121, NULL};
const Segment * const SG74670[] = {SGI321122, SGI321122, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74671[] = {SGI0, SGI321123, NULL};
const Segment * const SG74672[] = {SGI321124, SGI0, SGI321125, SGI321126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74673[] = {SGI321127, SGI0, SGI321128, SGI321129, SGI0, SGI0, SGI0, SGI321130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321131, SGI0, SGI0, NULL};
const Segment * const SG74674[] = {SGI321132, SGI0, SGI321133, SGI321134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321135, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74675[] = {SGI321136, SGI321136, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74676[] = {SGI321137, SGI0, SGI321138, SGI321139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74677[] = {SGI321140, SGI0, SGI321141, SGI321142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321143, SGI0, SGI321144, SGI0, SGI0, NULL};
const Segment * const SG74678[] = {SGI321145, SGI321146, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74679[] = {SGI321147, SGI321147, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74680[] = {SGI321148, SGI321148, SGI0, SGI0, SGI0, SGI321149, SGI0, NULL};
const Segment * const SG74681[] = {SGI321150, SGI0, SGI321151, SGI321152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74682[] = {SGI321154, SGI0, SGI321155, SGI321156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321158, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74683[] = {SGI321159, SGI0, SGI321160, SGI321161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321162, SGI0, SGI0, SGI0, SGI321163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74684[] = {SGI321164, SGI0, SGI321165, SGI321166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321168, SGI0, SGI0, NULL};
const Segment * const SG74685[] = {SGI321169, SGI0, SGI321170, SGI321171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321173, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74686[] = {SGI321174, SGI0, SGI321175, SGI321176, SGI321177, SGI321178, SGI321179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74687[] = {SGI321181, SGI0, SGI321182, SGI321183, SGI321184, SGI321185, SGI321186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74688[] = {SGI321188, SGI0, SGI321189, SGI321190, SGI321191, SGI321192, SGI321193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321195, SGI0, SGI0, NULL};
const Segment * const SG74689[] = {SGI321196, SGI0, SGI321197, SGI321198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74690[] = {SGI321200, SGI0, SGI321201, SGI321202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321203, SGI0, SGI0, SGI0, SGI321204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74691[] = {SGI321205, SGI0, SGI321206, SGI321207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321209, SGI0, SGI0, NULL};
const Segment * const SG74692[] = {SGI321210, SGI0, SGI321211, SGI321212, SGI0, SGI321213, SGI0, SGI0, SGI0, SGI321214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74693[] = {SGI321215, SGI0, SGI321216, SGI321217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321219, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74694[] = {SGI321220, SGI0, SGI321221, SGI321222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321224, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74695[] = {SGI321225, SGI0, SGI321226, SGI321227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74696[] = {SGI321229, SGI0, SGI321230, SGI321231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321233, SGI0, SGI0, NULL};
const Segment * const SG74697[] = {SGI321234, SGI0, SGI321235, SGI321236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321237, SGI0, SGI0, SGI0, SGI321238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74698[] = {SGI321239, SGI0, SGI321240, SGI321241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321243, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74699[] = {SGI321244, SGI0, SGI321245, SGI321246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321248, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74700[] = {SGI321249, SGI0, SGI321249, SGI321250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74701[] = {SGI321252, SGI0, SGI321252, SGI321253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321255, SGI0, SGI0, NULL};
const Segment * const SG74702[] = {SGI321256, SGI0, SGI321256, SGI321257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321258, SGI0, SGI0, SGI0, SGI321259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74703[] = {SGI321260, SGI0, SGI321260, SGI321261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321263, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74704[] = {SGI321264, SGI0, SGI321264, SGI321265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321267, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74705[] = {SGI321268, SGI0, SGI321269, SGI321270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74706[] = {SGI321272, SGI0, SGI321273, SGI321274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321276, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74707[] = {SGI321277, SGI0, SGI321278, SGI321279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321280, SGI0, SGI0, SGI0, SGI321281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74708[] = {SGI321282, SGI0, SGI321283, SGI321284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321286, SGI0, SGI0, NULL};
const Segment * const SG74709[] = {SGI321287, SGI0, SGI321288, SGI321289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321291, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74710[] = {SGI321292, SGI0, SGI321293, SGI0, SGI321294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74711[] = {SGI321295, SGI0, SGI321296, SGI321297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321298, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74712[] = {SGI321299, SGI321299, SGI0, SGI0, SGI0, SGI321300, SGI0, NULL};
const Segment * const SG74713[] = {SGI321301, SGI0, SGI321302, SGI321303, SGI321304, SGI321305, SGI0, SGI321306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321308, SGI0, NULL};
const Segment * const SG74714[] = {SGI321309, SGI0, SGI321310, SGI321311, SGI321312, SGI321313, SGI0, SGI321314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321315, SGI0, NULL};
const Segment * const SG74715[] = {SGI321316, SGI0, SGI321317, SGI321318, SGI321319, SGI321320, SGI0, SGI321321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321322, SGI0, SGI321323, SGI0, NULL};
const Segment * const SG74716[] = {SGI321324, SGI0, SGI321325, SGI321326, SGI321327, SGI321328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321329, SGI0, NULL};
const Segment * const SG74717[] = {SGI321330, SGI0, SGI321331, SGI321332, SGI321333, SGI321334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321335, SGI0, NULL};
const Segment * const SG74718[] = {SGI321336, SGI0, SGI321337, SGI321338, SGI321339, SGI321340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321341, SGI321342, SGI0, NULL};
const Segment * const SG74719[] = {SGI321343, SGI0, SGI321344, SGI321345, SGI321346, SGI321347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321349, SGI0, NULL};
const Segment * const SG74720[] = {SGI321350, SGI0, SGI321351, SGI321352, SGI321353, SGI321354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321355, SGI321356, SGI0, NULL};
const Segment * const SG74721[] = {SGI321357, SGI0, SGI321358, SGI321359, SGI321360, SGI321361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321362, SGI0, NULL};
const Segment * const SG74722[] = {SGI321363, SGI0, SGI321364, SGI321365, SGI321366, SGI321367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321369, SGI0, NULL};
const Segment * const SG74723[] = {SGI321370, SGI0, SGI321371, SGI321372, SGI321373, SGI321374, SGI0, SGI321375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321376, SGI0, NULL};
const Segment * const SG74724[] = {SGI321377, SGI0, SGI321378, SGI321379, SGI321380, SGI321381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321383, SGI0, NULL};
const Segment * const SG74725[] = {SGI321384, SGI0, SGI321385, SGI321386, SGI321387, SGI321388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321389, SGI0, NULL};
const Segment * const SG74726[] = {SGI321390, SGI0, SGI321391, SGI321392, SGI321393, SGI321394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321396, SGI0, NULL};
const Segment * const SG74727[] = {SGI321397, SGI0, SGI321398, SGI321399, SGI321400, SGI321401, SGI0, SGI321402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321404, SGI0, NULL};
const Segment * const SG74728[] = {SGI321405, SGI0, SGI321406, SGI321407, SGI321408, SGI321409, SGI0, SGI321410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321411, SGI0, NULL};
const Segment * const SG74729[] = {SGI321412, SGI0, SGI321413, SGI321414, SGI321415, SGI321416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321417, SGI321418, SGI0, NULL};
const Segment * const SG74730[] = {SGI321419, SGI0, SGI321420, SGI321421, SGI321422, SGI321423, SGI0, SGI321424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321426, SGI0, NULL};
const Segment * const SG74731[] = {SGI321427, SGI0, SGI321428, SGI321429, SGI321430, SGI321431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321432, SGI0, NULL};
const Segment * const SG74732[] = {SGI321433, SGI0, SGI321434, SGI321435, SGI321436, SGI321437, SGI0, SGI321438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321439, SGI0, NULL};
const Segment * const SG74733[] = {SGI321440, SGI0, SGI321441, SGI321442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74734[] = {SGI321444, SGI0, SGI321445, SGI321446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74735[] = {SGI321448, SGI0, SGI321449, SGI321450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321451, SGI0, SGI0, SGI0, SGI321452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74736[] = {SGI321453, SGI0, SGI321454, SGI321455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321457, SGI0, SGI0, NULL};
const Segment * const SG74737[] = {SGI321458, SGI0, SGI321459, SGI321460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321462, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74738[] = {SGI321463, SGI0, SGI321464, SGI321465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321467, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74739[] = {SGI321468, SGI0, SGI321469, SGI321470, SGI321471, SGI321472, SGI321473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74740[] = {SGI321476, SGI0, SGI321477, SGI321478, SGI321479, SGI321480, SGI321481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74741[] = {SGI321483, SGI0, SGI321484, SGI321485, SGI321486, SGI321487, SGI321488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74742[] = {SGI321491, SGI0, SGI321492, SGI321493, SGI321494, SGI321495, SGI321496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74743[] = {SGI321499, SGI321500, NULL};
const Segment * const SG74744[] = {SGI321501, SGI321502, NULL};
const Segment * const SG74745[] = {SGI321503, SGI321504, NULL};
const Segment * const SG74746[] = {SGI321505, SGI321506, NULL};
const Segment * const SG74747[] = {SGI321507, SGI0, SGI321508, SGI321509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74748[] = {SGI321511, SGI321511, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74749[] = {SGI321512, SGI0, SGI321513, SGI321514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321515, SGI0, SGI0, SGI0, SGI321516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74750[] = {SGI321517, SGI0, SGI321518, SGI321519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321521, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74751[] = {SGI321522, SGI0, SGI321523, SGI321524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321526, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74752[] = {SGI321527, SGI0, SGI321528, SGI321529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321531, SGI0, SGI0, NULL};
const Segment * const SG74753[] = {SGI321532, SGI0, SGI321532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74754[] = {SGI321533, SGI0, SGI321534, SGI321535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74755[] = {SGI321536, SGI0, SGI321536, SGI321537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74756[] = {SGI321538, SGI0, SGI321539, SGI321540, SGI321541, SGI321542, SGI321543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74757[] = {SGI321545, SGI0, SGI321546, SGI321547, SGI321548, SGI321549, SGI321550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321552, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74758[] = {SGI321553, SGI0, SGI321554, SGI321555, SGI321556, SGI321557, SGI321558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321560, SGI0, SGI0, NULL};
const Segment * const SG74759[] = {SGI321561, SGI0, SGI321562, SGI321563, SGI321564, SGI321565, SGI321566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74760[] = {SGI321568, SGI0, SGI321569, SGI321570, SGI321571, SGI321572, SGI321573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321575, SGI0, SGI0, NULL};
const Segment * const SG74761[] = {SGI321576, SGI0, SGI0, SGI0, SGI321577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74762[] = {SGI321578, SGI0, SGI321579, SGI321580, SGI321581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74763[] = {SGI321582, SGI0, SGI321583, SGI0, SGI321584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74764[] = {SGI321585, SGI0, SGI321585, SGI321586, SGI321587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74765[] = {SGI321588, SGI0, SGI321588, SGI321589, SGI321590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74766[] = {SGI321591, SGI0, SGI321592, SGI321593, SGI321594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74767[] = {SGI321595, SGI0, SGI321596, SGI321597, SGI321598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74768[] = {SGI321599, SGI0, SGI321600, SGI321601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74769[] = {SGI321602, SGI0, SGI321603, SGI321604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321605, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74770[] = {SGI321606, SGI321607, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74771[] = {SGI321608, SGI321608, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74772[] = {SGI321609, SGI321610, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74773[] = {SGI321611, SGI0, SGI321612, SGI321613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74774[] = {SGI321615, SGI321616, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74775[] = {SGI321618, SGI321618, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74776[] = {SGI321617, SGI0, SGI321619, SGI321620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321622, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74777[] = {SGI321623, SGI321623, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74778[] = {SGI321624, SGI0, SGI321625, SGI321626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321627, SGI0, SGI0, SGI0, SGI321628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74779[] = {SGI321629, SGI0, SGI321630, SGI321631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321633, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74780[] = {SGI321634, SGI0, SGI321635, SGI321636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321638, SGI0, SGI0, NULL};
const Segment * const SG74781[] = {SGI321639, SGI321640, SGI0, SGI321641, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74782[] = {SGI321642, SGI321642, SGI0, SGI321643, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74783[] = {SGI321644, SGI321644, SGI0, SGI321645, SGI0, SGI0, SGI321646, NULL};
const Segment * const SG74784[] = {SGI321647, SGI321647, SGI0, SGI321648, SGI0, SGI0, SGI321649, NULL};
const Segment * const SG74785[] = {SGI321650, SGI321650, SGI0, SGI321651, SGI0, SGI0, SGI321652, NULL};
const Segment * const SG74786[] = {SGI321653, SGI321653, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74787[] = {SGI321654, SGI321654, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74788[] = {SGI321655, SGI321655, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74789[] = {SGI321656, SGI321657, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74790[] = {SGI321658, SGI0, SGI321659, SGI321660, SGI321661, SGI321662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74791[] = {SGI321664, SGI0, SGI321665, SGI321666, SGI321667, SGI321668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74792[] = {SGI321670, SGI0, SGI321671, SGI321672, SGI321673, SGI321674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74793[] = {SGI321676, SGI0, SGI321677, SGI321678, SGI321679, SGI321680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74794[] = {SGI321682, SGI0, SGI321683, SGI321684, SGI321685, SGI321686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74795[] = {SGI321688, SGI0, SGI321689, SGI321690, SGI321691, SGI321692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74796[] = {SGI321694, SGI0, SGI321695, SGI321696, SGI321697, SGI321698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321699, SGI0, SGI321700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74797[] = {SGI321701, SGI0, SGI321702, SGI321703, SGI321704, SGI321705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74798[] = {SGI321707, SGI0, SGI321708, SGI321709, SGI321710, SGI321711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321713, SGI0, SGI0, NULL};
const Segment * const SG74799[] = {SGI321714, SGI0, SGI321715, SGI321716, SGI321717, SGI321718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321719, SGI0, SGI321720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74800[] = {SGI321721, SGI0, SGI321722, SGI321723, SGI321724, SGI321725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74801[] = {SGI321727, SGI0, SGI321728, SGI321729, SGI321730, SGI321731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321733, SGI0, SGI0, NULL};
const Segment * const SG74802[] = {SGI321734, SGI0, SGI321735, SGI321736, SGI321737, SGI321738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321740, SGI0, SGI0, NULL};
const Segment * const SG74803[] = {SGI321741, SGI0, SGI321742, SGI321743, SGI321744, SGI321745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74804[] = {SGI321747, SGI0, SGI321748, SGI321749, SGI321750, SGI321751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321752, SGI0, SGI321753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74805[] = {SGI321754, SGI0, SGI321755, SGI321756, SGI321757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321758, SGI0, SGI321759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74806[] = {SGI321760, SGI0, SGI321761, SGI321762, SGI321763, SGI321764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321765, SGI0, SGI321766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74807[] = {SGI321767, SGI0, SGI321768, SGI321769, SGI321770, SGI321771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321773, SGI0, SGI0, NULL};
const Segment * const SG74808[] = {SGI321775, SGI321775, SGI0, SGI0, SGI321778, SGI0, NULL};
const Segment * const SG74809[] = {SGI321774, SGI0, SGI321776, SGI321777, SGI321779, SGI321780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74810[] = {SGI321782, SGI0, SGI321783, SGI321784, SGI321785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321786, SGI0, SGI321787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74811[] = {SGI321788, SGI321789, SGI0, SGI321790, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74812[] = {SGI321791, SGI321792, SGI0, SGI321793, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74813[] = {SGI321794, SGI321795, SGI0, SGI321796, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74814[] = {SGI321797, SGI321797, SGI0, SGI321798, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74815[] = {SGI321799, SGI321799, SGI0, SGI321800, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74816[] = {SGI321801, SGI321802, SGI0, SGI321803, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74817[] = {SGI321804, SGI321805, SGI0, SGI321806, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74818[] = {SGI321807, SGI321807, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74819[] = {SGI321808, SGI321808, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74820[] = {SGI321809, NULL};
const Segment * const SG74821[] = {SGI321810, NULL};
const Segment * const SG74822[] = {SGI321811, NULL};
const Segment * const SG74823[] = {SGI321812, NULL};
const Segment * const SG74824[] = {SGI321813, NULL};
const Segment * const SG74825[] = {SGI321814, NULL};
const Segment * const SG74826[] = {SGI321815, NULL};
const Segment * const SG74827[] = {SGI321816, NULL};
const Segment * const SG74828[] = {SGI321817, NULL};
const Segment * const SG74829[] = {SGI321818, SGI321818, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74830[] = {SGI321819, SGI321819, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74831[] = {SGI321820, NULL};
const Segment * const SG74832[] = {SGI321822, NULL};
const Segment * const SG74833[] = {SGI321821, SGI321821, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74834[] = {SGI321823, SGI0, SGI321824, SGI321825, SGI321826, SGI321827, SGI321828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321830, SGI0, SGI0, NULL};
const Segment * const SG74835[] = {SGI321831, SGI0, SGI321832, SGI321833, SGI321834, SGI321835, SGI321836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74836[] = {SGI321838, SGI0, SGI321839, SGI321840, SGI321841, SGI321842, SGI321843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74837[] = {SGI321845, SGI0, SGI321846, SGI321847, SGI321848, SGI321849, SGI321850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74838[] = {SGI321852, SGI0, SGI321853, SGI321854, SGI321855, SGI321856, SGI321857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74839[] = {SGI321859, SGI0, SGI321860, SGI321861, SGI321862, SGI321863, SGI321864, SGI321865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74840[] = {SGI321867, SGI0, SGI321868, SGI321869, SGI321870, SGI321871, SGI321872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74841[] = {SGI321874, SGI0, SGI321875, SGI321876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321877, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74842[] = {SGI321878, SGI0, SGI321879, SGI321880, SGI321881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74843[] = {SGI321882, SGI0, SGI321883, SGI321884, SGI321885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74844[] = {SGI321886, SGI0, SGI321887, SGI321888, SGI321889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74845[] = {SGI321890, SGI0, SGI321891, SGI321892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74846[] = {SGI321893, SGI0, SGI321894, SGI321895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74847[] = {SGI321897, SGI0, SGI321898, SGI321899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321900, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74848[] = {SGI321901, SGI0, SGI321902, SGI321903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321904, SGI0, SGI0, NULL};
const Segment * const SG74849[] = {SGI321905, SGI0, SGI321906, SGI321907, SGI321908, SGI321909, SGI0, SGI321910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74850[] = {SGI321913, SGI0, SGI321914, SGI321915, SGI321916, SGI321917, SGI0, SGI321918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321920, SGI0, SGI0, SGI0, SGI321921, SGI0, SGI0, NULL};
const Segment * const SG74851[] = {SGI321922, SGI0, SGI321923, SGI321924, SGI321925, SGI321926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74852[] = {SGI321929, SGI0, SGI321930, SGI321931, SGI321932, SGI321933, SGI0, SGI321934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74853[] = {SGI321937, SGI0, SGI321938, SGI321939, SGI321940, SGI321941, SGI0, SGI321942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321944, SGI0, SGI321945, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74854[] = {SGI321946, SGI0, SGI321947, SGI321948, SGI321949, SGI321950, SGI0, SGI0, SGI0, SGI0, SGI321951, SGI0, SGI0, SGI0, SGI0, SGI321952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74855[] = {SGI321954, SGI0, SGI321955, SGI321956, SGI321957, SGI321958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321960, SGI0, SGI0, SGI0, SGI321961, SGI0, SGI0, NULL};
const Segment * const SG74856[] = {SGI321962, SGI0, SGI321963, SGI321964, SGI321965, SGI321966, SGI0, SGI321967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321968, SGI0, SGI321969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74857[] = {SGI321971, SGI0, SGI321972, SGI321973, SGI321974, SGI321975, SGI0, SGI321976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74858[] = {SGI321979, SGI0, SGI321980, SGI321981, SGI321982, SGI321983, SGI0, SGI321984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321985, SGI0, SGI321986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74859[] = {SGI321988, SGI0, SGI321989, SGI321990, SGI321991, SGI321992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI321994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74860[] = {SGI321995, SGI321995, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74861[] = {SGI321996, SGI321996, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74862[] = {SGI321997, SGI321997, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74863[] = {SGI321998, SGI321999, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74864[] = {SGI322000, SGI322000, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74865[] = {SGI322001, SGI0, SGI322002, SGI322003, SGI322004, SGI322005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74866[] = {SGI322007, SGI0, SGI322008, SGI322009, SGI322010, SGI322011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74867[] = {SGI322013, SGI0, SGI322014, SGI322015, SGI322016, SGI322017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74868[] = {SGI322019, SGI0, SGI322020, SGI322021, SGI322022, SGI322023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74869[] = {SGI322025, SGI0, SGI322026, SGI322027, SGI322028, SGI322029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74870[] = {SGI322031, SGI0, SGI322032, SGI322033, SGI322034, SGI322035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74871[] = {SGI322037, SGI0, SGI322038, SGI322039, SGI322040, SGI322041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322043, SGI0, SGI0, NULL};
const Segment * const SG74872[] = {SGI322044, SGI0, SGI322045, SGI322046, SGI322047, SGI322048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322050, SGI0, SGI0, NULL};
const Segment * const SG74873[] = {SGI322051, SGI0, SGI322052, SGI322053, SGI322054, SGI322055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322057, SGI0, SGI0, NULL};
const Segment * const SG74874[] = {SGI322058, SGI0, SGI322059, SGI322060, SGI322061, SGI322062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74875[] = {SGI322064, SGI0, SGI322065, SGI322066, SGI322067, SGI322068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74876[] = {SGI322070, SGI0, SGI322071, SGI322072, SGI322073, SGI322074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322076, SGI0, SGI0, NULL};
const Segment * const SG74877[] = {SGI322077, SGI0, SGI322078, SGI322079, SGI322080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74878[] = {SGI322081, SGI0, SGI322082, SGI322083, SGI322084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74879[] = {SGI322085, SGI322085, SGI0, SGI322086, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74880[] = {SGI322087, SGI322088, SGI0, SGI322089, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74881[] = {SGI322090, SGI0, SGI322091, SGI322092, SGI322093, SGI322094, SGI322095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74882[] = {SGI322097, SGI0, SGI322098, SGI322099, SGI322100, SGI322101, SGI322102, SGI322103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74883[] = {SGI322105, SGI0, SGI322106, SGI322107, SGI322108, SGI322109, SGI322110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74884[] = {SGI322113, SGI0, SGI322114, SGI322115, SGI322116, SGI322117, SGI322118, SGI322119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322121, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74885[] = {SGI322122, SGI0, SGI322123, SGI322124, SGI322125, SGI322126, SGI322127, SGI322128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74886[] = {SGI322131, SGI0, SGI322132, SGI322133, SGI322134, SGI322135, SGI322136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74887[] = {SGI322138, SGI0, SGI322139, SGI322140, SGI322141, SGI322142, SGI322143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74888[] = {SGI322145, SGI0, SGI322146, SGI322147, SGI322148, SGI322149, SGI322150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74889[] = {SGI322153, SGI322153, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74890[] = {SGI322154, NULL};
const Segment * const SG74891[] = {SGI322155, NULL};
const Segment * const SG74892[] = {SGI322158, SGI322158, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74893[] = {SGI322156, SGI0, SGI322157, SGI322159, SGI322160, SGI322161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322162, SGI0, SGI0, SGI322163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74894[] = {SGI322164, SGI0, SGI322165, SGI322166, SGI322167, SGI322168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74895[] = {SGI322170, SGI0, SGI322171, SGI322172, SGI322173, SGI322174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322175, SGI322176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74896[] = {SGI322177, SGI322178, SGI0, SGI322179, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74897[] = {SGI322180, SGI322181, SGI0, SGI322182, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74898[] = {SGI322183, SGI322183, SGI0, SGI322184, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74899[] = {SGI322185, SGI0, SGI322186, SGI322187, SGI322188, SGI322189, SGI0, SGI322190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322192, SGI0, SGI0, NULL};
const Segment * const SG74900[] = {SGI322193, SGI0, SGI322194, SGI322195, SGI322196, SGI322197, SGI0, SGI322198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74901[] = {SGI322200, SGI0, SGI322201, SGI322202, SGI322203, SGI322204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74902[] = {SGI322206, SGI0, SGI322207, SGI322208, SGI322209, SGI322210, SGI0, SGI322211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74903[] = {SGI322213, SGI0, SGI322214, SGI322215, SGI322216, SGI322217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74904[] = {SGI322219, SGI0, SGI322220, SGI322221, SGI322222, SGI322223, SGI0, SGI322224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322225, SGI0, SGI0, SGI0, SGI322226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74905[] = {SGI322227, SGI0, SGI322228, SGI322229, SGI322230, SGI322231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74906[] = {SGI322233, SGI0, SGI322234, SGI322235, SGI322236, SGI322237, SGI0, SGI322238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322240, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74907[] = {SGI322241, SGI0, SGI322242, SGI322243, SGI322244, SGI322245, SGI322246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74908[] = {SGI322249, SGI0, SGI322250, SGI322251, SGI322252, SGI322253, SGI322254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74909[] = {SGI322256, SGI0, SGI322257, SGI322258, SGI322259, SGI322260, SGI322261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74910[] = {SGI322264, SGI0, SGI322265, SGI322266, SGI322267, SGI322268, SGI322269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74911[] = {SGI322272, SGI322272, SGI0, SGI0, SGI322273, SGI0, NULL};
const Segment * const SG74912[] = {SGI322274, SGI322274, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74913[] = {SGI322275, SGI322275, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74914[] = {SGI322276, SGI0, SGI322277, SGI322278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74915[] = {SGI322279, SGI0, SGI322280, SGI322281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322282, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74916[] = {SGI322283, SGI0, SGI322284, SGI322285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322286, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74917[] = {SGI322287, SGI0, SGI322288, SGI322289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322290, SGI0, SGI322291, SGI0, SGI0, NULL};
const Segment * const SG74918[] = {SGI322292, SGI0, SGI322293, SGI322294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322295, SGI0, SGI322296, SGI0, SGI0, NULL};
const Segment * const SG74919[] = {SGI322297, SGI322297, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74920[] = {SGI322298, SGI322298, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74921[] = {SGI322299, NULL};
const Segment * const SG74922[] = {SGI322300, NULL};
const Segment * const SG74923[] = {SGI322301, NULL};
const Segment * const SG74924[] = {SGI322302, NULL};
const Segment * const SG74925[] = {SGI322303, NULL};
const Segment * const SG74926[] = {SGI322304, SGI322304, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74927[] = {SGI322305, NULL};
const Segment * const SG74928[] = {SGI322306, NULL};
const Segment * const SG74929[] = {SGI322307, NULL};
const Segment * const SG74930[] = {SGI322308, NULL};
const Segment * const SG74931[] = {SGI322309, NULL};
const Segment * const SG74932[] = {SGI322310, NULL};
const Segment * const SG74933[] = {SGI322311, SGI0, SGI322312, SGI322313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322314, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74934[] = {SGI322315, SGI0, SGI322316, SGI322317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322318, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74935[] = {SGI322319, SGI0, SGI322320, SGI322321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74936[] = {SGI322322, SGI0, SGI322323, SGI322324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322325, SGI322326, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74937[] = {SGI322327, SGI0, SGI322328, SGI322329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322331, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74938[] = {SGI322332, SGI0, SGI322333, SGI322334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322335, SGI0, SGI0, NULL};
const Segment * const SG74939[] = {SGI322336, SGI0, SGI322337, SGI322338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322339, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74940[] = {SGI322340, SGI0, SGI322341, SGI322342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74941[] = {SGI322344, SGI322345, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74942[] = {SGI322346, SGI322346, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74943[] = {SGI322347, SGI322347, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74944[] = {SGI322348, SGI322349, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74945[] = {SGI322350, SGI0, SGI322351, SGI322352, SGI322353, SGI322354, SGI322355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74946[] = {SGI322357, SGI0, SGI322358, SGI322359, SGI322360, SGI322361, SGI322362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74947[] = {SGI322365, SGI0, SGI322366, SGI322367, SGI322368, SGI322369, SGI322370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74948[] = {SGI322373, SGI0, SGI322374, SGI322375, SGI322376, SGI322377, SGI322378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74949[] = {SGI322381, SGI0, SGI322382, SGI322383, SGI322384, SGI322385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322386, SGI0, SGI0, SGI322387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74950[] = {SGI322388, SGI0, SGI322389, SGI322390, SGI322391, SGI322392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74951[] = {SGI322394, SGI0, SGI322395, SGI322396, SGI322397, SGI322398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322399, SGI0, SGI0, SGI322400, SGI322401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74952[] = {SGI322402, SGI0, SGI322403, SGI322404, SGI322405, SGI322406, SGI0, SGI0, SGI322407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74953[] = {SGI322408, SGI0, SGI322409, SGI322410, SGI322411, SGI322412, SGI0, SGI0, SGI322413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74954[] = {SGI322414, SGI0, SGI322415, SGI322416, SGI322417, SGI322418, SGI0, SGI0, SGI322419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74955[] = {SGI322420, SGI0, SGI322421, SGI322422, SGI322423, SGI322424, SGI0, SGI0, SGI322425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322426, SGI0, SGI0, NULL};
const Segment * const SG74956[] = {SGI322427, SGI0, SGI322428, SGI322429, SGI322430, SGI322431, SGI0, SGI0, SGI322432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74957[] = {SGI322433, SGI0, SGI322434, SGI322435, SGI322436, SGI322437, SGI0, SGI0, SGI322438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74958[] = {SGI322439, SGI0, SGI322440, SGI322441, SGI322442, SGI322443, SGI0, SGI0, SGI322444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322445, SGI0, SGI0, NULL};
const Segment * const SG74959[] = {SGI322446, SGI0, SGI322447, SGI322448, SGI322449, SGI322450, SGI0, SGI0, SGI322451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74960[] = {SGI322452, SGI0, SGI322453, SGI322454, SGI322455, SGI322456, SGI0, SGI0, SGI322457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74961[] = {SGI322458, SGI0, SGI322459, SGI322460, SGI322461, SGI322462, SGI0, SGI0, SGI322463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74962[] = {SGI322464, SGI0, SGI322465, SGI322466, SGI322467, SGI322468, SGI0, SGI0, SGI322469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74963[] = {SGI322470, SGI0, SGI322471, SGI322472, SGI322473, SGI322474, SGI0, SGI0, SGI322475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74964[] = {SGI322476, SGI0, SGI322477, SGI322478, SGI322479, SGI322480, SGI0, SGI0, SGI322481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74965[] = {SGI322482, SGI0, SGI322483, SGI322484, SGI322485, SGI322486, SGI0, SGI0, SGI322487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74966[] = {SGI322488, SGI0, SGI322489, SGI322490, SGI322491, SGI322492, SGI0, SGI0, SGI322493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74967[] = {SGI322494, SGI0, SGI322495, SGI322496, SGI322497, SGI322498, SGI0, SGI0, SGI322499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74968[] = {SGI322500, SGI0, SGI322501, SGI322502, SGI322503, SGI322504, SGI0, SGI0, SGI322505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74969[] = {SGI322506, SGI0, SGI322507, SGI322508, SGI322509, SGI322510, SGI0, SGI0, SGI322511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74970[] = {SGI322512, SGI0, SGI322513, SGI322514, SGI322515, SGI322516, SGI0, SGI0, SGI322517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74971[] = {SGI322518, SGI0, SGI322519, SGI322520, SGI322521, SGI322522, SGI0, SGI0, SGI322523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74972[] = {SGI322524, SGI0, SGI322525, SGI322526, SGI322527, SGI322528, SGI0, SGI0, SGI322529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74973[] = {SGI322530, SGI0, SGI322531, SGI322532, SGI322533, SGI322534, SGI0, SGI0, SGI322535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74974[] = {SGI322536, SGI0, SGI322537, SGI322538, SGI322539, SGI322540, SGI0, SGI0, SGI322541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74975[] = {SGI322542, NULL};
const Segment * const SG74976[] = {SGI322543, NULL};
const Segment * const SG74977[] = {SGI322544, NULL};
const Segment * const SG74978[] = {SGI322545, NULL};
const Segment * const SG74979[] = {SGI322546, NULL};
const Segment * const SG74980[] = {SGI322547, NULL};
const Segment * const SG74981[] = {SGI322548, NULL};
const Segment * const SG74982[] = {SGI322549, NULL};
const Segment * const SG74983[] = {SGI322550, NULL};
const Segment * const SG74984[] = {SGI322551, NULL};
const Segment * const SG74985[] = {SGI322552, NULL};
const Segment * const SG74986[] = {SGI322553, SGI0, SGI322554, SGI322555, SGI322556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74987[] = {SGI322557, SGI0, SGI322557, SGI322558, SGI322559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74988[] = {SGI322560, SGI0, SGI322561, SGI322562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322563, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74989[] = {SGI322564, SGI322565, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74990[] = {SGI322566, SGI322567, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74991[] = {SGI322568, SGI322568, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74992[] = {SGI322569, SGI322570, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74993[] = {SGI322571, SGI0, SGI322572, SGI322573, SGI322574, SGI322575, SGI322576, SGI322577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74994[] = {SGI322579, SGI0, SGI322580, SGI322581, SGI322582, SGI322583, SGI322584, SGI322585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74995[] = {SGI322586, SGI0, SGI322587, SGI322588, SGI322589, SGI322590, SGI322591, SGI322592, SGI0, SGI0, SGI322593, SGI0, SGI0, SGI322594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74996[] = {SGI322595, SGI0, SGI322596, SGI322597, SGI322598, SGI322599, SGI322600, SGI322601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322602, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74997[] = {SGI322603, SGI0, SGI322604, SGI322605, SGI322606, SGI322607, SGI322608, SGI322609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74998[] = {SGI322611, SGI0, SGI322612, SGI322613, SGI322614, SGI322615, SGI322616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG74999[] = {SGI322619, SGI0, SGI322620, SGI322621, SGI322622, SGI322623, SGI322624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75000[] = {SGI322626, SGI0, SGI322627, SGI322628, SGI322629, SGI322630, SGI322631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75001[] = {SGI322634, SGI0, SGI322635, SGI322636, SGI322637, SGI322638, SGI322639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75002[] = {SGI322642, SGI322642, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75003[] = {SGI322643, SGI0, SGI322644, SGI322645, SGI322646, SGI322647, SGI0, SGI0, SGI322648, SGI0, SGI0, SGI0, SGI0, SGI322649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75004[] = {SGI322650, SGI0, SGI322651, SGI322652, SGI322653, SGI322654, SGI0, SGI0, SGI322655, SGI0, SGI0, SGI0, SGI0, SGI322656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75005[] = {SGI322657, SGI0, SGI322658, SGI322659, SGI322660, SGI322661, SGI0, SGI0, SGI322662, SGI0, SGI0, SGI0, SGI0, SGI322663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75006[] = {SGI322664, SGI0, SGI322665, SGI322666, SGI322667, SGI322668, SGI0, SGI0, SGI322669, SGI0, SGI0, SGI0, SGI0, SGI322670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75007[] = {SGI322671, SGI0, SGI322672, SGI322673, SGI322674, SGI322675, SGI0, SGI0, SGI322676, SGI0, SGI0, SGI0, SGI0, SGI322677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75008[] = {SGI322678, SGI0, SGI322679, SGI322680, SGI322681, SGI322682, SGI0, SGI0, SGI322683, SGI0, SGI0, SGI0, SGI0, SGI322684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75009[] = {SGI322685, SGI0, SGI322686, SGI322687, SGI322688, SGI322689, SGI0, SGI0, SGI322690, SGI0, SGI0, SGI0, SGI0, SGI322691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75010[] = {SGI322692, SGI0, SGI322693, SGI322694, SGI322695, SGI322696, SGI0, SGI0, SGI322697, SGI0, SGI0, SGI0, SGI0, SGI322698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75011[] = {SGI322699, SGI322700, SGI0, SGI0, SGI0, SGI322701, SGI0, NULL};
const Segment * const SG75012[] = {SGI322702, SGI322702, SGI0, SGI0, SGI0, SGI322703, SGI0, NULL};
const Segment * const SG75013[] = {SGI322704, SGI322704, SGI0, SGI0, SGI322705, SGI0, NULL};
const Segment * const SG75014[] = {SGI322706, SGI322706, SGI0, SGI0, SGI322707, SGI0, NULL};
const Segment * const SG75015[] = {SGI322708, SGI322709, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75016[] = {SGI322711, SGI322712, SGI322713, NULL};
const Segment * const SG75017[] = {SGI322710, SGI322710, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75018[] = {SGI322714, SGI322714, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75019[] = {SGI322715, SGI322715, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75020[] = {SGI322716, SGI0, SGI322717, SGI322718, SGI322719, SGI322720, SGI322721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75021[] = {SGI322723, SGI0, SGI322724, SGI322725, SGI322726, SGI322727, SGI322728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75022[] = {SGI322731, SGI0, SGI322732, SGI322733, SGI322734, SGI322735, SGI322736, SGI322737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75023[] = {SGI322740, SGI0, SGI322741, SGI322742, SGI322743, SGI322744, SGI322745, SGI322746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75024[] = {SGI322748, SGI0, SGI322749, SGI322750, SGI322751, SGI322752, SGI322753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75025[] = {SGI322756, SGI0, SGI322757, SGI322758, SGI322759, SGI322760, SGI322761, SGI322762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75026[] = {SGI322765, SGI0, SGI322766, SGI322767, SGI322768, SGI322769, SGI322770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75027[] = {SGI322772, SGI0, SGI322773, SGI322774, SGI322775, SGI322776, SGI322777, SGI322778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75028[] = {SGI322780, SGI0, SGI322781, SGI322782, SGI322783, SGI322784, SGI322785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75029[] = {SGI322787, SGI0, SGI322788, SGI322789, SGI322790, SGI322791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322792, SGI0, SGI0, SGI322793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75030[] = {SGI322794, SGI0, SGI322795, SGI322796, SGI322797, SGI322798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75031[] = {SGI322800, SGI0, SGI322801, SGI322802, SGI322803, SGI322804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322805, SGI0, SGI0, SGI322806, SGI322807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75032[] = {SGI322808, SGI0, SGI322809, SGI322810, SGI322811, SGI322812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75033[] = {SGI322815, SGI0, SGI322816, SGI322817, SGI322818, SGI322819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75034[] = {SGI322822, SGI0, SGI322823, SGI322824, SGI322825, SGI322826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322828, SGI0, SGI0, SGI0, SGI0, SGI322829, SGI0, SGI0, NULL};
const Segment * const SG75035[] = {SGI322830, SGI322830, SGI0, SGI0, SGI322831, SGI0, NULL};
const Segment * const SG75036[] = {SGI322832, SGI322832, SGI0, SGI0, SGI322833, SGI0, NULL};
const Segment * const SG75037[] = {SGI322834, SGI0, SGI322835, SGI322836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322837, SGI322838, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75038[] = {SGI322839, NULL};
const Segment * const SG75039[] = {SGI322840, NULL};
const Segment * const SG75040[] = {SGI322841, NULL};
const Segment * const SG75041[] = {SGI322842, NULL};
const Segment * const SG75042[] = {SGI322843, NULL};
const Segment * const SG75043[] = {SGI322844, SGI322844, SGI0, SGI0, SGI322845, SGI0, NULL};
const Segment * const SG75044[] = {SGI322846, SGI322847, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75045[] = {SGI322848, SGI322848, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75046[] = {SGI322849, SGI0, SGI322850, SGI322851, SGI322852, SGI322853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322854, SGI0, NULL};
const Segment * const SG75047[] = {SGI322855, SGI0, SGI322856, SGI322857, SGI322858, SGI322859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322861, SGI0, NULL};
const Segment * const SG75048[] = {SGI322862, SGI0, SGI322863, SGI322864, SGI322865, SGI322866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322868, SGI0, NULL};
const Segment * const SG75049[] = {SGI322869, SGI322869, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75050[] = {SGI322870, SGI0, SGI322871, SGI322872, SGI322873, SGI322874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75051[] = {SGI322876, SGI0, SGI322877, SGI322878, SGI322879, SGI322880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75052[] = {SGI322881, SGI0, SGI322882, SGI322883, SGI322884, SGI322885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322886, SGI0, SGI0, NULL};
const Segment * const SG75053[] = {SGI322887, SGI0, SGI322888, SGI322889, SGI322890, SGI322891, SGI0, SGI322892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75054[] = {SGI322893, SGI0, SGI322894, SGI322895, SGI322896, SGI322897, SGI0, SGI322898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75055[] = {SGI322900, SGI0, SGI322901, SGI322902, SGI322903, SGI322904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322905, SGI0, SGI322906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75056[] = {SGI322908, SGI0, SGI322909, SGI322910, SGI322911, SGI322912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322913, SGI0, SGI322914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75057[] = {SGI322916, SGI0, SGI322917, SGI322918, SGI322919, SGI322920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75058[] = {SGI322923, SGI0, SGI322924, SGI322925, SGI322926, SGI322927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75059[] = {SGI322930, SGI0, SGI322931, SGI322932, SGI322933, SGI322934, SGI0, SGI322935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322936, SGI0, SGI322937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75060[] = {SGI322939, SGI0, SGI322940, SGI322941, SGI322942, SGI322943, SGI0, SGI322944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI322946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75061[] = {SGI322947, SGI0, SGI322948, SGI322949, SGI322950, SGI322951, SGI0, SGI0, SGI322952, SGI0, SGI0, SGI0, SGI0, SGI322953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75062[] = {SGI322954, SGI0, SGI322955, SGI322956, SGI322957, SGI322958, SGI0, SGI0, SGI322959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75063[] = {SGI322960, SGI0, SGI322961, SGI322962, SGI322963, SGI322964, SGI0, SGI0, SGI322965, SGI0, SGI0, SGI0, SGI0, SGI322966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75064[] = {SGI322967, SGI0, SGI322968, SGI322969, SGI322970, SGI322971, SGI0, SGI0, SGI322972, SGI0, SGI0, SGI0, SGI0, SGI322973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75065[] = {SGI322974, SGI0, SGI322975, SGI322976, SGI322977, SGI322978, SGI0, SGI0, SGI322979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75066[] = {SGI322980, SGI0, SGI322981, SGI322982, SGI322983, SGI322984, SGI0, SGI0, SGI322985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75067[] = {SGI322986, SGI0, SGI322987, SGI322988, SGI322989, SGI322990, SGI0, SGI0, SGI322991, SGI0, SGI0, SGI0, SGI0, SGI322992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75068[] = {SGI322993, SGI0, SGI322994, SGI322995, SGI322996, SGI322997, SGI0, SGI0, SGI322998, SGI0, SGI0, SGI0, SGI0, SGI322999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75069[] = {SGI323000, SGI0, SGI323001, SGI323002, SGI323003, SGI323004, SGI0, SGI0, SGI323005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75070[] = {SGI323006, SGI0, SGI323007, SGI323008, SGI323009, SGI323010, SGI0, SGI0, SGI323011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75071[] = {SGI323012, SGI0, SGI323013, SGI323014, SGI0, SGI323015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75072[] = {SGI323016, SGI0, SGI323017, SGI323018, SGI323019, SGI323020, SGI323021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75073[] = {SGI323023, SGI0, SGI323024, SGI323025, SGI323026, SGI323027, SGI323028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75074[] = {SGI323031, SGI0, SGI323032, SGI323033, SGI323034, SGI323035, SGI323036, SGI323037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75075[] = {SGI323039, SGI0, SGI323040, SGI323041, SGI323042, SGI323043, SGI323044, SGI323045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75076[] = {SGI323048, SGI0, SGI323049, SGI323050, SGI323051, SGI323052, SGI323053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75077[] = {SGI323055, SGI0, SGI323056, SGI323057, SGI323058, SGI323059, SGI323060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75078[] = {SGI323062, SGI0, SGI323063, SGI323064, SGI323065, SGI323066, SGI323067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75079[] = {SGI323070, SGI323071, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75080[] = {SGI323072, SGI323072, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75081[] = {SGI323073, SGI323073, SGI0, SGI0, SGI0, SGI0, SGI323074, NULL};
const Segment * const SG75082[] = {SGI323075, SGI0, SGI323076, SGI323077, SGI323078, SGI323079, SGI0, SGI323080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75083[] = {SGI323082, SGI0, SGI323083, SGI323084, SGI323085, SGI323086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75084[] = {SGI323088, SGI0, SGI323089, SGI323090, SGI323091, SGI323092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75085[] = {SGI323094, SGI0, SGI323095, SGI323096, SGI323097, SGI323098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323100, SGI0, SGI0, NULL};
const Segment * const SG75086[] = {SGI323101, SGI0, SGI323102, SGI323103, SGI323104, SGI323105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75087[] = {SGI323107, SGI0, SGI323108, SGI323109, SGI323110, SGI323111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323113, SGI0, SGI0, NULL};
const Segment * const SG75088[] = {SGI323114, SGI0, SGI323115, SGI323116, SGI323117, SGI323118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75089[] = {SGI323120, SGI323121, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75090[] = {SGI323122, SGI323122, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75091[] = {SGI323123, SGI323123, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75092[] = {SGI323124, SGI0, SGI323125, NULL};
const Segment * const SG75093[] = {SGI323126, SGI0, SGI323127, NULL};
const Segment * const SG75094[] = {SGI323128, SGI0, SGI323129, NULL};
const Segment * const SG75095[] = {SGI323130, SGI0, SGI323131, NULL};
const Segment * const SG75096[] = {SGI323132, SGI0, SGI323133, SGI323134, SGI323135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75097[] = {SGI323136, SGI0, SGI323137, SGI323138, SGI323139, SGI323140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323141, SGI0, SGI0, NULL};
const Segment * const SG75098[] = {SGI323142, SGI0, SGI323143, SGI323144, SGI323145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75099[] = {SGI323146, SGI0, SGI323147, SGI323148, SGI323149, SGI323150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75100[] = {SGI323151, SGI0, SGI323152, SGI323153, SGI323154, SGI323155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75101[] = {SGI323156, SGI0, SGI323157, SGI323158, SGI323159, SGI323160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323161, SGI0, SGI0, NULL};
const Segment * const SG75102[] = {SGI323162, SGI0, SGI323163, SGI323164, SGI323165, SGI323166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75103[] = {SGI323168, SGI0, SGI323169, SGI323170, SGI323171, SGI323172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75104[] = {SGI323173, NULL};
const Segment * const SG75105[] = {SGI323174, NULL};
const Segment * const SG75106[] = {SGI323175, SGI323176, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75107[] = {SGI323177, SGI323177, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75108[] = {SGI323178, SGI323179, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75109[] = {SGI323180, SGI0, SGI323181, SGI323182, SGI323183, SGI323184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323185, SGI0, SGI0, SGI0, SGI323186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75110[] = {SGI323187, SGI0, SGI323188, SGI323189, SGI323190, SGI323191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75111[] = {SGI323193, SGI0, SGI323194, SGI323195, SGI323196, SGI323197, SGI0, SGI323198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323199, SGI0, SGI0, SGI0, SGI323200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75112[] = {SGI323201, SGI0, SGI323202, SGI323203, SGI323204, SGI323205, SGI0, SGI323206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75113[] = {SGI323208, SGI0, SGI323209, SGI323210, SGI323211, SGI323212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323213, SGI0, SGI0, SGI0, SGI323214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323215, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75114[] = {SGI323216, SGI323216, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75115[] = {SGI323217, SGI323217, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75116[] = {SGI323218, SGI0, SGI323219, SGI323220, SGI323221, SGI323222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323223, SGI0, NULL};
const Segment * const SG75117[] = {SGI323224, SGI0, SGI323225, SGI323226, SGI323227, SGI323228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323229, SGI323230, SGI0, NULL};
const Segment * const SG75118[] = {SGI323231, SGI323231, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75119[] = {SGI323232, SGI0, SGI323233, SGI323234, SGI323235, SGI323236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75120[] = {SGI323238, SGI0, SGI323239, SGI323240, SGI323241, SGI323242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75121[] = {SGI323244, SGI0, SGI323245, SGI323246, SGI323247, SGI323248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323250, SGI0, SGI0, NULL};
const Segment * const SG75122[] = {SGI323251, SGI0, SGI323252, SGI323253, SGI323254, SGI323255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323257, SGI0, SGI0, NULL};
const Segment * const SG75123[] = {SGI323258, SGI0, SGI323259, SGI323260, SGI323261, SGI323262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75124[] = {SGI323264, SGI0, SGI323265, SGI323266, SGI323267, SGI323268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75125[] = {SGI323270, SGI323270, SGI0, SGI0, SGI0, SGI323271, SGI0, NULL};
const Segment * const SG75126[] = {SGI323272, SGI0, SGI0, SGI0, SGI323273, SGI323274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75127[] = {SGI323275, SGI0, SGI323276, SGI323277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75128[] = {SGI323278, SGI0, SGI323278, SGI323279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75129[] = {SGI323280, SGI0, SGI323280, SGI323281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323282, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75130[] = {SGI323283, SGI0, SGI323283, SGI323284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323285, SGI0, SGI323286, SGI0, SGI0, NULL};
const Segment * const SG75131[] = {SGI323287, SGI323287, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75132[] = {SGI323288, SGI323288, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75133[] = {SGI323289, SGI0, SGI323290, SGI323291, SGI323292, SGI323293, SGI0, SGI0, SGI323294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75134[] = {SGI323295, SGI0, SGI323296, SGI323297, SGI323298, SGI323299, SGI0, SGI0, SGI323300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75135[] = {SGI323301, SGI0, SGI323302, SGI323303, SGI323304, SGI323305, SGI0, SGI0, SGI323306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75136[] = {SGI323307, SGI0, SGI323308, SGI323309, SGI323310, SGI323311, SGI0, SGI0, SGI323312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75137[] = {SGI323313, SGI0, SGI323314, SGI323315, SGI323316, SGI323317, SGI0, SGI323318, SGI323319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75138[] = {SGI323320, SGI0, SGI323321, SGI323322, SGI323323, SGI323324, SGI0, SGI0, SGI323325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75139[] = {SGI323326, SGI0, SGI323327, SGI323328, SGI323329, SGI323330, SGI0, SGI0, SGI323331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75140[] = {SGI323332, SGI0, SGI323333, SGI323334, SGI323335, SGI323336, SGI0, SGI0, SGI323337, SGI0, SGI0, SGI0, SGI0, SGI323338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75141[] = {SGI323339, SGI0, SGI323340, SGI323341, SGI323342, SGI323343, SGI0, SGI323344, SGI323345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75142[] = {SGI323346, SGI0, SGI323347, SGI323348, SGI323349, SGI323350, SGI0, SGI0, SGI323351, SGI0, SGI0, SGI0, SGI0, SGI323352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75143[] = {SGI323353, SGI0, SGI323354, SGI323355, SGI323356, SGI323357, SGI0, SGI0, SGI323358, SGI0, SGI0, SGI0, SGI0, SGI323359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75144[] = {SGI323360, SGI0, SGI323361, SGI323362, SGI323363, SGI323364, SGI0, SGI0, SGI323365, SGI0, SGI0, SGI0, SGI0, SGI323366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75145[] = {SGI323367, SGI0, SGI323368, SGI323369, SGI323370, SGI323371, SGI0, SGI0, SGI323372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75146[] = {SGI323373, SGI0, SGI323374, SGI323375, SGI323376, SGI323377, SGI0, SGI0, SGI323378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75147[] = {SGI323379, SGI0, SGI323380, SGI323381, SGI323382, SGI323383, SGI0, SGI0, SGI323384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75148[] = {SGI323385, SGI0, SGI323386, SGI323387, SGI323388, SGI323389, SGI0, SGI0, SGI323390, SGI0, SGI0, SGI0, SGI0, SGI323391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75149[] = {SGI323392, SGI0, SGI323393, SGI323394, SGI323395, SGI323396, SGI0, SGI0, SGI323397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75150[] = {SGI323398, SGI0, SGI323399, SGI323400, SGI323401, SGI323402, SGI0, SGI0, SGI323403, SGI0, SGI0, SGI0, SGI0, SGI323404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75151[] = {SGI323405, SGI0, SGI323406, SGI323407, SGI323408, SGI323409, SGI0, SGI0, SGI323410, SGI0, SGI0, SGI0, SGI0, SGI323411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75152[] = {SGI323412, SGI0, SGI323413, SGI323414, SGI323415, SGI323416, SGI0, SGI0, SGI323417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75153[] = {SGI323418, SGI0, SGI323419, SGI323420, SGI323421, SGI323422, SGI0, SGI0, SGI323423, SGI0, SGI0, SGI0, SGI0, SGI323424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75154[] = {SGI323425, SGI0, SGI323426, SGI323427, SGI323428, SGI323429, SGI0, SGI0, SGI323430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75155[] = {SGI323431, SGI0, SGI323432, SGI323433, SGI323434, SGI323435, SGI0, SGI0, SGI323436, SGI0, SGI0, SGI0, SGI0, SGI323437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75156[] = {SGI323438, SGI323439, NULL};
const Segment * const SG75157[] = {SGI323446, SGI323447, NULL};
const Segment * const SG75158[] = {SGI323440, SGI0, SGI323441, SGI323442, SGI323443, SGI323444, SGI0, SGI0, SGI323445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75159[] = {SGI323448, SGI323449, NULL};
const Segment * const SG75160[] = {SGI323450, SGI0, SGI323451, SGI323452, SGI323453, SGI323454, SGI0, SGI0, SGI323455, SGI0, SGI0, SGI0, SGI0, SGI323456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75161[] = {SGI323457, SGI323458, NULL};
const Segment * const SG75162[] = {SGI323459, SGI0, SGI323460, SGI323461, SGI323462, SGI323463, SGI0, SGI0, SGI323464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75163[] = {SGI323465, SGI0, SGI323466, SGI323467, SGI323468, SGI323469, SGI0, SGI323470, SGI323471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75164[] = {SGI323472, SGI0, SGI323473, SGI323474, SGI323475, SGI323476, SGI0, SGI0, SGI323477, SGI0, SGI0, SGI0, SGI0, SGI323478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75165[] = {SGI323479, SGI0, SGI323480, SGI323481, SGI323482, SGI323483, SGI0, SGI0, SGI323484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75166[] = {SGI323485, SGI323485, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75167[] = {SGI323486, SGI323486, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75168[] = {SGI323487, SGI323487, SGI0, SGI0, SGI323488, SGI0, NULL};
const Segment * const SG75169[] = {SGI323489, SGI323489, SGI0, SGI0, SGI323490, SGI0, NULL};
const Segment * const SG75170[] = {SGI323491, SGI323491, SGI0, SGI0, SGI323492, SGI0, NULL};
const Segment * const SG75171[] = {SGI323493, SGI323493, SGI0, SGI0, SGI323494, SGI0, NULL};
const Segment * const SG75172[] = {SGI323495, SGI0, SGI323496, SGI323497, SGI323498, SGI323499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75173[] = {SGI323501, SGI0, SGI323502, SGI323503, SGI323504, SGI323505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75174[] = {SGI323507, SGI0, SGI323508, SGI323509, SGI323510, SGI323511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323513, SGI0, SGI0, NULL};
const Segment * const SG75175[] = {SGI323514, SGI0, SGI323515, SGI323516, SGI323517, SGI323518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323520, SGI0, SGI0, NULL};
const Segment * const SG75176[] = {SGI323521, SGI0, SGI323522, SGI323523, SGI323524, SGI323525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75177[] = {SGI323527, SGI0, SGI323528, SGI323529, SGI323530, SGI323531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75178[] = {SGI323533, SGI0, SGI323534, SGI323535, SGI323536, SGI323537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75179[] = {SGI323539, SGI0, SGI323540, SGI323541, SGI323542, SGI323543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75180[] = {SGI323545, SGI0, SGI323546, SGI323547, SGI323548, SGI323549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323551, SGI0, SGI0, NULL};
const Segment * const SG75181[] = {SGI323552, SGI0, SGI323553, SGI323554, SGI323555, SGI323556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323558, SGI0, SGI0, NULL};
const Segment * const SG75182[] = {SGI323559, SGI0, SGI323560, SGI323561, SGI323562, SGI323563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75183[] = {SGI323565, SGI0, SGI323566, SGI323567, SGI323568, SGI323569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75184[] = {SGI323571, SGI323571, SGI0, SGI0, SGI323572, SGI0, SGI0, NULL};
const Segment * const SG75185[] = {SGI323573, NULL};
const Segment * const SG75186[] = {SGI323574, NULL};
const Segment * const SG75187[] = {SGI323575, NULL};
const Segment * const SG75188[] = {SGI323576, NULL};
const Segment * const SG75189[] = {SGI323577, NULL};
const Segment * const SG75190[] = {SGI323578, NULL};
const Segment * const SG75191[] = {SGI323579, NULL};
const Segment * const SG75192[] = {SGI323580, NULL};
const Segment * const SG75193[] = {SGI323581, SGI0, SGI323582, SGI323583, SGI323584, SGI323585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75194[] = {SGI323587, SGI0, SGI323588, SGI323589, SGI323590, SGI323591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323592, SGI0, SGI0, SGI323593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75195[] = {SGI323594, SGI0, SGI323595, SGI323596, SGI323597, SGI323598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75196[] = {SGI323600, SGI0, SGI323601, SGI323602, SGI323603, SGI323604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75197[] = {SGI323606, SGI0, SGI323607, SGI323608, SGI323609, SGI323610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75198[] = {SGI323612, SGI0, SGI323613, SGI323614, SGI323615, SGI323616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323618, SGI0, SGI0, NULL};
const Segment * const SG75199[] = {SGI323619, SGI0, SGI323620, SGI323621, SGI323622, SGI323623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323625, SGI0, SGI0, NULL};
const Segment * const SG75200[] = {SGI323626, SGI0, SGI323627, SGI323628, SGI323629, SGI323630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75201[] = {SGI323632, SGI323632, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75202[] = {SGI323633, SGI323633, SGI0, SGI0, SGI0, SGI0, SGI323634, NULL};
const Segment * const SG75203[] = {SGI323635, SGI323635, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75204[] = {SGI323636, NULL};
const Segment * const SG75205[] = {SGI323637, NULL};
const Segment * const SG75206[] = {SGI323638, SGI323638, SGI0, SGI0, SGI323639, SGI0, NULL};
const Segment * const SG75207[] = {SGI323640, SGI323640, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75208[] = {SGI323641, SGI0, SGI323642, SGI323643, SGI323644, SGI323645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323646, SGI0, SGI323647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75209[] = {SGI323649, SGI0, SGI323650, SGI323651, SGI323652, SGI323653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75210[] = {SGI323656, SGI0, SGI323657, SGI323658, SGI323659, SGI323660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75211[] = {SGI323663, SGI0, SGI323664, SGI323665, SGI323666, SGI323667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323668, SGI0, SGI323669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75212[] = {SGI323671, SGI323671, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75213[] = {SGI323672, SGI323672, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75214[] = {SGI323673, NULL};
const Segment * const SG75215[] = {SGI323674, SGI0, SGI323675, SGI323676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75216[] = {SGI323677, SGI0, SGI323678, SGI323679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75217[] = {SGI323680, SGI0, SGI323681, SGI323682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323683, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75218[] = {SGI323684, SGI0, SGI323685, SGI323686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323687, SGI0, SGI323688, SGI0, SGI0, NULL};
const Segment * const SG75219[] = {SGI323689, SGI323689, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75220[] = {SGI323690, NULL};
const Segment * const SG75221[] = {SGI323691, NULL};
const Segment * const SG75222[] = {SGI323692, SGI323692, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75223[] = {SGI323693, SGI323693, SGI0, SGI0, SGI0, SGI0, SGI323694, NULL};
const Segment * const SG75224[] = {SGI323695, SGI0, SGI323696, SGI323697, SGI323698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323699, SGI0, SGI0, NULL};
const Segment * const SG75225[] = {SGI323700, SGI0, SGI323701, SGI323702, SGI323703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75226[] = {SGI323704, SGI0, SGI323705, SGI323706, SGI323707, SGI0, SGI0, SGI323708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75227[] = {SGI323709, SGI0, SGI323710, SGI323711, SGI323712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75228[] = {SGI323714, SGI0, SGI323715, SGI323716, SGI323717, SGI0, SGI0, SGI323718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75229[] = {SGI323720, SGI323720, SGI0, SGI0, SGI323721, SGI0, NULL};
const Segment * const SG75230[] = {SGI323722, SGI323722, SGI0, SGI0, SGI323723, SGI0, NULL};
const Segment * const SG75231[] = {SGI323724, SGI323725, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75232[] = {SGI323726, NULL};
const Segment * const SG75233[] = {SGI323727, SGI323727, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75234[] = {SGI323728, SGI0, SGI323729, NULL};
const Segment * const SG75235[] = {SGI323730, SGI323731, SGI323732, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75236[] = {SGI323733, SGI323734, SGI323735, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75237[] = {SGI323736, SGI323737, SGI323738, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75238[] = {SGI323739, SGI323739, SGI323740, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75239[] = {SGI323741, SGI323742, SGI323743, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75240[] = {SGI323744, SGI0, SGI323745, SGI323746, SGI323747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75241[] = {SGI323748, SGI0, SGI323749, SGI323750, SGI323751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75242[] = {SGI323752, SGI0, SGI323753, SGI323754, SGI323755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75243[] = {SGI323757, SGI323757, SGI0, SGI0, SGI323760, SGI0, NULL};
const Segment * const SG75244[] = {SGI323756, SGI0, SGI323758, SGI323759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323761, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75245[] = {SGI323762, SGI323762, SGI0, SGI0, SGI323763, SGI0, SGI0, NULL};
const Segment * const SG75246[] = {SGI323764, NULL};
const Segment * const SG75247[] = {SGI323770, NULL};
const Segment * const SG75248[] = {SGI323765, SGI0, SGI323766, SGI323767, SGI323768, SGI323769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323771, SGI0, NULL};
const Segment * const SG75249[] = {SGI323772, NULL};
const Segment * const SG75250[] = {SGI323773, SGI0, SGI323774, SGI323775, SGI323776, SGI323777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323779, SGI0, NULL};
const Segment * const SG75251[] = {SGI323780, NULL};
const Segment * const SG75252[] = {SGI323781, NULL};
const Segment * const SG75253[] = {SGI323788, NULL};
const Segment * const SG75254[] = {SGI323782, SGI0, SGI323783, SGI323784, SGI323785, SGI323786, SGI0, SGI323787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323789, SGI0, NULL};
const Segment * const SG75255[] = {SGI323790, SGI0, SGI323791, SGI323792, SGI323793, SGI323794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323796, SGI0, NULL};
const Segment * const SG75256[] = {SGI323797, SGI0, SGI323798, SGI323799, SGI323800, SGI323801, SGI0, SGI323802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323804, SGI0, NULL};
const Segment * const SG75257[] = {SGI323805, SGI0, SGI323806, SGI323807, SGI323808, SGI323809, SGI0, SGI323810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323805, SGI0, NULL};
const Segment * const SG75258[] = {SGI323811, SGI0, SGI323812, SGI323813, SGI323814, SGI323815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75259[] = {SGI323817, SGI0, SGI323818, SGI323819, SGI323820, SGI323821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75260[] = {SGI323822, SGI0, SGI323823, SGI323824, SGI323825, SGI323826, SGI0, SGI323827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75261[] = {SGI323831, SGI323832, SGI0, SGI0, SGI323833, SGI0, NULL};
const Segment * const SG75262[] = {SGI323834, SGI323834, SGI0, SGI0, SGI323835, SGI0, NULL};
const Segment * const SG75263[] = {SGI323829, SGI0, SGI323830, SGI323836, SGI323837, SGI323838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323839, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75264[] = {SGI323840, SGI0, SGI323841, SGI323842, SGI323843, SGI323844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323846, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75265[] = {SGI323847, SGI323847, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75266[] = {SGI323848, SGI323848, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75267[] = {SGI323849, SGI323849, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75268[] = {SGI323850, SGI323851, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75269[] = {SGI323852, SGI323852, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75270[] = {SGI323853, SGI323853, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75271[] = {SGI323854, NULL};
const Segment * const SG75272[] = {SGI323855, SGI323855, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75273[] = {SGI323856, SGI0, SGI323857, SGI323858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75274[] = {SGI323859, SGI0, SGI323860, SGI323861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323862, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75275[] = {SGI323863, SGI0, SGI323864, SGI323865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75276[] = {SGI323866, SGI0, SGI323867, SGI323868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323869, SGI0, SGI323870, SGI0, SGI0, NULL};
const Segment * const SG75277[] = {SGI323871, SGI323871, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75278[] = {SGI323872, SGI0, SGI323873, SGI323874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323875, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75279[] = {SGI323876, SGI0, SGI323877, SGI323878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323879, SGI0, SGI323880, SGI0, SGI0, NULL};
const Segment * const SG75280[] = {SGI323881, SGI323881, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75281[] = {SGI323882, NULL};
const Segment * const SG75282[] = {SGI323883, NULL};
const Segment * const SG75283[] = {SGI323884, SGI0, SGI323885, SGI323886, SGI323887, SGI323888, SGI0, SGI323889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323890, SGI0, NULL};
const Segment * const SG75284[] = {SGI323891, SGI0, SGI323892, SGI323893, SGI323894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323895, SGI0, NULL};
const Segment * const SG75285[] = {SGI323896, NULL};
const Segment * const SG75286[] = {SGI323897, SGI0, SGI323898, SGI323899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323900, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75287[] = {SGI323901, SGI0, SGI323902, SGI323903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323904, SGI0, SGI323905, SGI0, SGI0, NULL};
const Segment * const SG75288[] = {SGI323906, SGI0, SGI323907, SGI323908, SGI323909, SGI323910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323911, SGI0, SGI323912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75289[] = {SGI323914, SGI0, SGI323915, SGI323916, SGI323917, SGI323918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323919, SGI0, SGI323920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75290[] = {SGI323922, SGI0, SGI323923, SGI323924, SGI323925, SGI323926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75291[] = {SGI323929, SGI0, SGI323930, SGI323931, SGI323932, SGI323933, SGI0, SGI323934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323935, SGI0, SGI323936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75292[] = {SGI323938, SGI0, SGI323939, SGI323940, SGI323941, SGI323942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75293[] = {SGI323945, SGI0, SGI323946, SGI323947, SGI323948, SGI323949, SGI0, SGI323950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75294[] = {SGI323953, SGI0, SGI323954, SGI323955, SGI323956, SGI323957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323958, SGI0, SGI323959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323960, SGI0, SGI0, SGI0, SGI323961, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75295[] = {SGI323962, SGI323962, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75296[] = {SGI323963, SGI0, SGI323964, NULL};
const Segment * const SG75297[] = {SGI323965, SGI0, SGI323966, NULL};
const Segment * const SG75298[] = {SGI323967, SGI0, SGI323968, NULL};
const Segment * const SG75299[] = {SGI323969, SGI0, SGI0, NULL};
const Segment * const SG75300[] = {SGI323970, SGI0, SGI0, NULL};
const Segment * const SG75301[] = {SGI323971, SGI0, SGI323972, NULL};
const Segment * const SG75302[] = {SGI323973, SGI0, SGI0, NULL};
const Segment * const SG75303[] = {SGI323974, SGI323974, SGI0, SGI0, SGI0, SGI323975, SGI0, NULL};
const Segment * const SG75304[] = {SGI323976, SGI323976, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75305[] = {SGI323977, NULL};
const Segment * const SG75306[] = {SGI323978, NULL};
const Segment * const SG75307[] = {SGI323979, SGI0, SGI323980, SGI323981, SGI323982, SGI323983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323984, SGI323985, SGI0, SGI0, SGI0, SGI323986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75308[] = {SGI323987, SGI0, SGI323988, SGI323989, SGI323990, SGI323991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323992, SGI0, SGI0, SGI0, SGI0, SGI323993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI323994, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75309[] = {SGI323995, SGI0, SGI323996, SGI323997, SGI323998, SGI323999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324000, SGI0, SGI0, SGI0, SGI0, SGI324001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75310[] = {SGI324002, SGI324003, NULL};
const Segment * const SG75311[] = {SGI324004, SGI0, SGI324005, SGI324006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324007, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75312[] = {SGI324008, SGI0, SGI324009, SGI324010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324011, SGI0, SGI324012, SGI0, SGI0, NULL};
const Segment * const SG75313[] = {SGI324013, SGI0, SGI324014, SGI324015, SGI324016, SGI324017, SGI0, SGI324018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324019, SGI0, NULL};
const Segment * const SG75314[] = {SGI324020, SGI0, SGI324021, SGI324022, SGI324023, SGI324024, SGI0, SGI324025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324027, SGI0, NULL};
const Segment * const SG75315[] = {SGI324028, SGI0, SGI324029, SGI324030, SGI324031, SGI324032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324033, SGI0, NULL};
const Segment * const SG75316[] = {SGI324034, SGI0, SGI324035, SGI324036, SGI324037, SGI324038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324040, SGI0, NULL};
const Segment * const SG75317[] = {SGI324041, SGI0, SGI324042, SGI324043, SGI324044, SGI324045, SGI0, SGI324046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324047, SGI0, NULL};
const Segment * const SG75318[] = {SGI324048, SGI0, SGI324049, SGI324050, SGI324051, SGI324052, SGI0, SGI324053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324054, SGI0, SGI324055, SGI0, NULL};
const Segment * const SG75319[] = {SGI324056, SGI0, SGI324057, SGI324058, SGI324059, SGI324060, SGI0, SGI324061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324062, SGI0, NULL};
const Segment * const SG75320[] = {SGI324063, SGI0, SGI324064, SGI324065, SGI324066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75321[] = {SGI324067, SGI0, SGI324068, SGI324069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75322[] = {SGI324070, SGI0, SGI324071, SGI324072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324073, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75323[] = {SGI324074, SGI324075, SGI0, SGI324076, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75324[] = {SGI324077, SGI324078, SGI0, SGI324079, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75325[] = {SGI324080, SGI324081, SGI0, SGI324082, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75326[] = {SGI324083, SGI324084, SGI0, SGI324085, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75327[] = {SGI324086, SGI324087, SGI0, SGI324088, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75328[] = {SGI324089, SGI324089, SGI0, SGI324090, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75329[] = {SGI324091, SGI324092, SGI0, SGI324093, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75330[] = {SGI324094, SGI324095, SGI0, SGI324096, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75331[] = {SGI324097, SGI324098, SGI0, SGI324099, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75332[] = {SGI324100, SGI324101, SGI0, SGI324102, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75333[] = {SGI0, SGI324103, NULL};
const Segment * const SG75334[] = {SGI324104, SGI0, SGI324105, NULL};
const Segment * const SG75335[] = {SGI324106, SGI0, SGI0, NULL};
const Segment * const SG75336[] = {SGI324107, SGI0, SGI324108, NULL};
const Segment * const SG75337[] = {SGI324109, SGI0, SGI324110, NULL};
const Segment * const SG75338[] = {SGI324111, SGI0, SGI324112, NULL};
const Segment * const SG75339[] = {SGI324113, SGI0, SGI324114, NULL};
const Segment * const SG75340[] = {SGI324115, SGI0, SGI324116, NULL};
const Segment * const SG75341[] = {SGI324117, SGI0, SGI324118, NULL};
const Segment * const SG75342[] = {SGI324119, SGI0, SGI324120, NULL};
const Segment * const SG75343[] = {SGI324121, SGI0, SGI324122, NULL};
const Segment * const SG75344[] = {SGI324123, SGI324123, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75345[] = {SGI324124, SGI0, SGI324125, NULL};
const Segment * const SG75346[] = {SGI324126, SGI0, SGI324127, NULL};
const Segment * const SG75347[] = {SGI324128, SGI324129, NULL};
const Segment * const SG75348[] = {SGI324130, SGI324131, NULL};
const Segment * const SG75349[] = {SGI324132, SGI324133, NULL};
const Segment * const SG75350[] = {SGI324134, SGI324134, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75351[] = {SGI324135, SGI324135, SGI0, SGI324136, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75352[] = {SGI324137, SGI324138, SGI0, SGI324139, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75353[] = {SGI324140, SGI0, SGI324141, SGI324142, SGI324143, SGI324144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324146, SGI0, SGI0, SGI0, SGI0, SGI324147, SGI0, SGI0, NULL};
const Segment * const SG75354[] = {SGI324148, SGI0, SGI324149, SGI324150, SGI324151, SGI324152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75355[] = {SGI324155, SGI0, SGI324156, SGI324157, SGI324158, SGI324159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324161, SGI0, SGI0, SGI0, SGI0, SGI324162, SGI0, SGI0, NULL};
const Segment * const SG75356[] = {SGI324163, SGI324163, SGI0, SGI0, SGI0, SGI0, SGI324164, NULL};
const Segment * const SG75357[] = {SGI324165, NULL};
const Segment * const SG75358[] = {SGI324166, NULL};
const Segment * const SG75359[] = {SGI324167, NULL};
const Segment * const SG75360[] = {SGI324168, SGI324169, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75361[] = {SGI324170, SGI324170, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75362[] = {SGI324171, SGI324171, SGI0, SGI0, SGI0, SGI0, SGI324172, NULL};
const Segment * const SG75363[] = {SGI324173, SGI324173, SGI0, SGI0, SGI0, SGI0, SGI324174, NULL};
const Segment * const SG75364[] = {SGI324175, NULL};
const Segment * const SG75365[] = {SGI324176, NULL};
const Segment * const SG75366[] = {SGI324177, SGI324177, SGI0, SGI0, SGI324178, SGI0, NULL};
const Segment * const SG75367[] = {SGI324179, SGI324179, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75368[] = {SGI324180, SGI324180, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75369[] = {SGI324181, SGI324181, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75370[] = {SGI324182, SGI324182, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75371[] = {SGI324183, NULL};
const Segment * const SG75372[] = {SGI324184, NULL};
const Segment * const SG75373[] = {SGI324185, NULL};
const Segment * const SG75374[] = {SGI324186, NULL};
const Segment * const SG75375[] = {SGI324187, NULL};
const Segment * const SG75376[] = {SGI324188, SGI324188, SGI0, SGI0, SGI324189, SGI0, NULL};
const Segment * const SG75377[] = {SGI324190, NULL};
const Segment * const SG75378[] = {SGI324191, NULL};
const Segment * const SG75379[] = {SGI324192, NULL};
const Segment * const SG75380[] = {SGI324193, NULL};
const Segment * const SG75381[] = {SGI324194, NULL};
const Segment * const SG75382[] = {SGI324195, NULL};
const Segment * const SG75383[] = {SGI324196, NULL};
const Segment * const SG75384[] = {SGI324197, SGI324197, SGI0, SGI0, SGI0, SGI324198, SGI0, NULL};
const Segment * const SG75385[] = {SGI324199, SGI0, SGI324200, NULL};
const Segment * const SG75386[] = {SGI324201, SGI0, SGI324202, NULL};
const Segment * const SG75387[] = {SGI324203, SGI0, SGI324204, NULL};
const Segment * const SG75388[] = {SGI324205, SGI0, SGI324206, NULL};
const Segment * const SG75389[] = {SGI324207, SGI0, SGI324208, NULL};
const Segment * const SG75390[] = {SGI324209, SGI0, SGI324210, NULL};
const Segment * const SG75391[] = {SGI324211, SGI0, SGI324212, NULL};
const Segment * const SG75392[] = {SGI324213, SGI0, SGI324214, NULL};
const Segment * const SG75393[] = {SGI324215, SGI0, SGI324216, NULL};
const Segment * const SG75394[] = {SGI324217, SGI0, SGI324218, NULL};
const Segment * const SG75395[] = {SGI324219, SGI0, SGI324220, NULL};
const Segment * const SG75396[] = {SGI324221, SGI0, SGI324222, NULL};
const Segment * const SG75397[] = {SGI324223, SGI0, SGI324224, NULL};
const Segment * const SG75398[] = {SGI324225, SGI0, SGI324226, NULL};
const Segment * const SG75399[] = {SGI324227, SGI0, SGI324228, NULL};
const Segment * const SG75400[] = {SGI324229, SGI0, SGI324230, NULL};
const Segment * const SG75401[] = {SGI324231, SGI0, SGI324232, NULL};
const Segment * const SG75402[] = {SGI324233, SGI0, SGI324234, SGI324235, SGI0, SGI324236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75403[] = {SGI324237, SGI0, SGI324238, SGI324239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324240, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75404[] = {SGI324241, SGI0, SGI324242, SGI324243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324244, SGI324245, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75405[] = {SGI324246, SGI0, SGI324247, SGI324248, SGI324249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75406[] = {SGI324250, SGI0, SGI324251, SGI324252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324253, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75407[] = {SGI324254, SGI0, SGI324255, SGI324256, SGI324257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75408[] = {SGI324258, SGI0, SGI324259, SGI324260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324261, SGI324262, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75409[] = {SGI324263, SGI0, SGI324264, NULL};
const Segment * const SG75410[] = {SGI324265, SGI324266, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75411[] = {SGI324267, SGI324268, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75412[] = {SGI324269, SGI324269, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75413[] = {SGI324270, SGI324270, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75414[] = {SGI324271, SGI324271, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75415[] = {SGI324272, SGI0, SGI324273, SGI324274, SGI324275, SGI324276, SGI0, SGI324277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324278, SGI0, NULL};
const Segment * const SG75416[] = {SGI324279, SGI0, SGI324280, SGI324281, SGI324282, SGI324283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324284, SGI324279, SGI0, NULL};
const Segment * const SG75417[] = {SGI324285, SGI0, SGI324286, SGI324287, SGI324288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324289, SGI0, NULL};
const Segment * const SG75418[] = {SGI324290, SGI0, SGI324291, SGI324292, SGI324293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324294, SGI0, NULL};
const Segment * const SG75419[] = {SGI324295, NULL};
const Segment * const SG75420[] = {SGI324296, SGI324296, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75421[] = {SGI324297, NULL};
const Segment * const SG75422[] = {SGI324298, SGI0, SGI324299, SGI324300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324301, SGI324302, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75423[] = {SGI324303, SGI324303, SGI0, SGI0, SGI0, SGI0, SGI324304, NULL};
const Segment * const SG75424[] = {SGI324305, SGI324305, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75425[] = {SGI324306, SGI324306, SGI0, SGI0, SGI324307, SGI0, NULL};
const Segment * const SG75426[] = {SGI324308, SGI0, SGI324309, SGI324310, SGI324311, SGI324312, SGI0, SGI0, SGI324313, SGI0, SGI0, SGI0, SGI0, SGI324314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75427[] = {SGI324315, SGI0, SGI324316, SGI324317, SGI324318, SGI324319, SGI0, SGI0, SGI324320, SGI0, SGI0, SGI0, SGI0, SGI324321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75428[] = {SGI324322, SGI0, SGI324323, SGI324324, SGI324325, SGI324326, SGI0, SGI0, SGI324327, SGI0, SGI0, SGI0, SGI0, SGI324328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75429[] = {SGI324329, SGI0, SGI324330, SGI324331, SGI324332, SGI324333, SGI0, SGI0, SGI324334, SGI0, SGI0, SGI0, SGI0, SGI324335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75430[] = {SGI324336, SGI0, SGI324337, SGI324338, SGI324339, SGI324340, SGI0, SGI0, SGI324341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75431[] = {SGI324342, SGI0, SGI324343, SGI324344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75432[] = {SGI324345, NULL};
const Segment * const SG75433[] = {SGI324346, NULL};
const Segment * const SG75434[] = {SGI324347, SGI0, SGI324348, SGI324349, SGI324350, SGI324351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75435[] = {SGI324353, SGI0, SGI324354, SGI324355, SGI324356, SGI324357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324358, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75436[] = {SGI324359, SGI0, SGI324360, SGI324361, SGI324362, SGI324363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75437[] = {SGI324364, SGI0, SGI324365, SGI324366, SGI324367, SGI324368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75438[] = {SGI324370, SGI324370, SGI0, SGI0, SGI0, SGI0, SGI324371, NULL};
const Segment * const SG75439[] = {SGI324372, SGI324372, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75440[] = {SGI324373, SGI324373, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75441[] = {SGI324374, SGI324374, SGI0, SGI0, SGI324375, SGI0, NULL};
const Segment * const SG75442[] = {SGI324376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75443[] = {SGI324377, NULL};
const Segment * const SG75444[] = {SGI324378, SGI324378, SGI324379, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75445[] = {SGI324380, SGI324381, SGI324382, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75446[] = {SGI324383, SGI324383, SGI324384, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75447[] = {SGI324385, SGI324385, SGI0, SGI0, SGI0, SGI0, SGI324386, NULL};
const Segment * const SG75448[] = {SGI324387, SGI0, SGI324388, SGI324389, SGI324390, SGI324391, SGI0, SGI0, SGI324392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75449[] = {SGI324393, SGI0, SGI324394, SGI324395, SGI324396, SGI0, SGI0, SGI0, SGI324397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75450[] = {SGI324398, SGI0, SGI324399, SGI324400, SGI324401, SGI324402, SGI0, SGI0, SGI324403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75451[] = {SGI324404, SGI0, SGI324405, SGI324406, SGI324407, SGI324408, SGI0, SGI0, SGI324409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75452[] = {SGI324410, SGI0, SGI324411, SGI324412, SGI324413, SGI324414, SGI0, SGI0, SGI324415, SGI0, SGI0, SGI0, SGI0, SGI324416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75453[] = {SGI324417, SGI0, SGI324418, SGI324419, SGI324420, SGI324421, SGI0, SGI0, SGI324422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75454[] = {SGI324423, SGI0, SGI324424, SGI324425, SGI324426, SGI324427, SGI0, SGI0, SGI324428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324429, SGI0, SGI0, NULL};
const Segment * const SG75455[] = {SGI324430, SGI0, SGI324431, SGI324432, SGI324433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75456[] = {SGI324434, SGI0, SGI324435, SGI324436, SGI324437, SGI324438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75457[] = {SGI324439, SGI0, SGI324440, SGI324441, SGI324442, SGI324443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324444, SGI0, SGI0, NULL};
const Segment * const SG75458[] = {SGI324445, SGI0, SGI324446, SGI324447, SGI324448, SGI324449, SGI0, SGI324450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75459[] = {SGI324451, SGI0, SGI324452, SGI324453, SGI324454, SGI324455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75460[] = {SGI324457, SGI0, SGI324458, SGI324459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324460, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75461[] = {SGI324461, SGI0, SGI324462, SGI324463, SGI324464, SGI324465, SGI0, SGI324466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75462[] = {SGI324468, SGI324469, SGI0, SGI324470, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75463[] = {SGI324471, SGI324472, SGI0, SGI324473, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75464[] = {SGI324474, SGI324474, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75465[] = {SGI324475, SGI324475, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75466[] = {SGI324476, SGI324476, SGI0, SGI0, SGI0, SGI0, SGI324477, NULL};
const Segment * const SG75467[] = {SGI324478, SGI0, SGI324479, NULL};
const Segment * const SG75468[] = {SGI324480, SGI324480, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75469[] = {SGI324481, SGI324481, SGI0, SGI0, SGI0, SGI0, SGI324482, NULL};
const Segment * const SG75470[] = {SGI324483, SGI0, SGI324484, SGI324485, SGI324486, SGI324487, SGI324488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75471[] = {SGI324490, SGI0, SGI324491, SGI324492, SGI324493, SGI324494, SGI324495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75472[] = {SGI324498, SGI0, SGI324499, SGI324500, SGI324501, SGI324502, SGI324503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75473[] = {SGI324505, SGI0, SGI324506, SGI324507, SGI324508, SGI324509, SGI324510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75474[] = {SGI324512, SGI0, SGI324513, SGI324514, SGI324515, SGI324516, SGI324517, SGI324518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324520, SGI0, SGI0, NULL};
const Segment * const SG75475[] = {SGI324521, SGI0, SGI324522, SGI324523, SGI324524, SGI324525, SGI324526, SGI324527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75476[] = {SGI324529, SGI0, SGI324530, SGI324531, SGI324532, SGI324533, SGI324534, SGI324535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75477[] = {SGI324537, SGI324537, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75478[] = {SGI324538, NULL};
const Segment * const SG75479[] = {SGI324539, SGI0, SGI324540, SGI324541, SGI324542, SGI324543, SGI324544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324545, SGI324546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75480[] = {SGI324548, NULL};
const Segment * const SG75481[] = {SGI324549, SGI324549, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75482[] = {SGI324550, SGI324550, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75483[] = {SGI324551, SGI324551, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75484[] = {SGI324552, SGI324552, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75485[] = {SGI324553, SGI324553, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75486[] = {SGI324554, SGI324554, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75487[] = {SGI324555, SGI0, SGI324556, SGI324557, SGI324558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75488[] = {SGI324559, SGI0, SGI324560, SGI324561, SGI324562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75489[] = {SGI324563, SGI0, SGI324564, SGI324565, SGI324566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75490[] = {SGI324567, SGI0, SGI324568, SGI324569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324570, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75491[] = {SGI324571, SGI0, SGI0, NULL};
const Segment * const SG75492[] = {SGI324572, SGI0, SGI0, NULL};
const Segment * const SG75493[] = {SGI324573, SGI0, SGI0, NULL};
const Segment * const SG75494[] = {SGI324574, SGI0, SGI0, NULL};
const Segment * const SG75495[] = {SGI324575, SGI0, SGI0, NULL};
const Segment * const SG75496[] = {SGI324576, SGI0, SGI0, NULL};
const Segment * const SG75497[] = {SGI324577, SGI324577, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75498[] = {SGI324578, NULL};
const Segment * const SG75499[] = {SGI324579, SGI324579, SGI0, SGI0, SGI0, SGI0, SGI324580, NULL};
const Segment * const SG75500[] = {SGI324581, SGI0, SGI324582, SGI324583, SGI324584, SGI324585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75501[] = {SGI324586, SGI0, SGI324587, SGI324588, SGI324589, SGI324590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75502[] = {SGI324592, SGI0, SGI324593, SGI324594, SGI324595, SGI324596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75503[] = {SGI324598, SGI324598, SGI0, SGI0, SGI0, SGI0, SGI324599, NULL};
const Segment * const SG75504[] = {SGI324600, SGI0, SGI324601, SGI324602, SGI324603, SGI324604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324605, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75505[] = {SGI324606, SGI0, SGI324607, SGI324608, SGI324609, SGI324610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324611, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75506[] = {SGI324612, SGI324612, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75507[] = {SGI324613, SGI0, SGI324614, NULL};
const Segment * const SG75508[] = {SGI324615, SGI324616, SGI324617, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75509[] = {SGI324618, SGI324618, SGI324619, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75510[] = {SGI324620, SGI324620, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75511[] = {SGI324621, SGI324621, SGI0, SGI0, SGI0, SGI324622, SGI0, NULL};
const Segment * const SG75512[] = {SGI324623, SGI324623, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75513[] = {SGI324624, SGI324624, SGI0, SGI0, SGI0, SGI0, SGI324625, NULL};
const Segment * const SG75514[] = {SGI324626, SGI324626, SGI0, SGI0, SGI324627, SGI0, NULL};
const Segment * const SG75515[] = {SGI324628, NULL};
const Segment * const SG75516[] = {SGI324629, SGI324629, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75517[] = {SGI324630, SGI0, SGI324631, SGI324632, SGI324633, SGI324634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324635, SGI324636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75518[] = {SGI324637, SGI0, SGI324638, SGI324639, SGI324640, SGI324641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324642, SGI324643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75519[] = {SGI324644, SGI324644, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75520[] = {SGI324645, SGI324645, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75521[] = {SGI324646, SGI324646, SGI0, SGI324647, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75522[] = {SGI324648, NULL};
const Segment * const SG75523[] = {SGI324649, NULL};
const Segment * const SG75524[] = {SGI324650, NULL};
const Segment * const SG75525[] = {SGI324651, NULL};
const Segment * const SG75526[] = {SGI324652, NULL};
const Segment * const SG75527[] = {SGI324653, NULL};
const Segment * const SG75528[] = {SGI324654, NULL};
const Segment * const SG75529[] = {SGI324655, NULL};
const Segment * const SG75530[] = {SGI324656, NULL};
const Segment * const SG75531[] = {SGI324657, NULL};
const Segment * const SG75532[] = {SGI324658, NULL};
const Segment * const SG75533[] = {SGI324659, NULL};
const Segment * const SG75534[] = {SGI324660, NULL};
const Segment * const SG75535[] = {SGI324661, NULL};
const Segment * const SG75536[] = {SGI324662, NULL};
const Segment * const SG75537[] = {SGI324663, NULL};
const Segment * const SG75538[] = {SGI324664, NULL};
const Segment * const SG75539[] = {SGI324665, NULL};
const Segment * const SG75540[] = {SGI324666, NULL};
const Segment * const SG75541[] = {SGI324667, SGI324667, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75542[] = {SGI324668, SGI324668, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75543[] = {SGI324669, SGI0, SGI324670, SGI324671, SGI324672, SGI324673, SGI324674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75544[] = {SGI324676, SGI0, SGI324677, SGI324678, SGI324679, SGI324680, SGI324681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75545[] = {SGI324684, SGI0, SGI324685, SGI324686, SGI324687, SGI324688, SGI324689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75546[] = {SGI324692, SGI324692, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75547[] = {SGI324693, SGI324693, SGI0, SGI0, SGI0, SGI0, SGI324694, NULL};
const Segment * const SG75548[] = {SGI324695, SGI324695, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75549[] = {SGI324696, NULL};
const Segment * const SG75550[] = {SGI324697, SGI324698, SGI324699, NULL};
const Segment * const SG75551[] = {SGI324700, SGI324701, SGI324702, NULL};
const Segment * const SG75552[] = {SGI324703, SGI324704, SGI324705, NULL};
const Segment * const SG75553[] = {SGI324706, SGI324707, SGI324708, NULL};
const Segment * const SG75554[] = {SGI324709, SGI324710, SGI324711, NULL};
const Segment * const SG75555[] = {SGI324712, SGI324713, SGI324714, NULL};
const Segment * const SG75556[] = {SGI324715, SGI324716, SGI324717, NULL};
const Segment * const SG75557[] = {SGI324718, SGI324719, SGI324720, NULL};
const Segment * const SG75558[] = {SGI324721, SGI324721, SGI324722, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75559[] = {SGI324723, SGI324723, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75560[] = {SGI324724, SGI0, SGI324725, SGI324726, SGI324727, SGI324728, SGI0, SGI324729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75561[] = {SGI324730, SGI0, SGI324731, SGI324732, SGI324733, SGI324734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75562[] = {SGI324735, SGI0, SGI324736, SGI324737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324738, SGI324739, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75563[] = {SGI324740, SGI0, SGI324741, SGI324742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324744, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75564[] = {SGI324745, SGI0, SGI324746, SGI324747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324748, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75565[] = {SGI324749, SGI324749, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75566[] = {SGI324750, NULL};
const Segment * const SG75567[] = {SGI324751, SGI0, SGI324752, SGI324753, SGI324754, SGI324755, SGI324756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75568[] = {SGI324758, SGI0, SGI324759, SGI324760, SGI324761, SGI324762, SGI324763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75569[] = {SGI324765, SGI0, SGI324766, SGI324767, SGI324768, SGI324769, SGI324770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324772, SGI0, SGI0, NULL};
const Segment * const SG75570[] = {SGI324773, NULL};
const Segment * const SG75571[] = {SGI324774, SGI324774, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75572[] = {SGI324775, NULL};
const Segment * const SG75573[] = {SGI324776, SGI324776, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75574[] = {SGI324777, SGI324777, SGI0, SGI0, SGI0, SGI0, SGI324778, NULL};
const Segment * const SG75575[] = {SGI324779, SGI324779, SGI0, SGI0, SGI0, SGI0, SGI324780, NULL};
const Segment * const SG75576[] = {SGI324781, SGI324781, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75577[] = {SGI324782, SGI324782, SGI0, SGI0, SGI0, SGI0, SGI324783, NULL};
const Segment * const SG75578[] = {SGI324784, SGI324784, SGI0, SGI0, SGI0, SGI0, SGI324785, NULL};
const Segment * const SG75579[] = {SGI324786, SGI0, SGI324787, SGI324788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324789, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75580[] = {SGI324790, SGI0, SGI324791, SGI324792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324793, SGI324794, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75581[] = {SGI324795, SGI0, SGI324796, SGI324797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324799, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75582[] = {SGI324800, SGI324800, SGI0, SGI0, SGI0, SGI0, SGI324801, NULL};
const Segment * const SG75583[] = {SGI324802, SGI0, SGI324803, SGI324804, SGI324805, SGI324806, SGI0, SGI0, SGI324807, SGI0, SGI0, SGI0, SGI0, SGI324808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75584[] = {SGI324809, SGI0, SGI324810, SGI324811, SGI324812, SGI324813, SGI0, SGI0, SGI324814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75585[] = {SGI324815, SGI0, SGI324816, SGI324817, SGI324818, SGI324819, SGI0, SGI0, SGI324820, SGI0, SGI0, SGI0, SGI0, SGI324821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75586[] = {SGI324822, SGI324822, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75587[] = {SGI324823, SGI0, SGI324824, SGI324825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75588[] = {SGI324826, SGI0, SGI324827, SGI324828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75589[] = {SGI324829, SGI0, SGI324830, SGI324831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324832, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75590[] = {SGI324833, SGI0, SGI324834, SGI324835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324836, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75591[] = {SGI324837, SGI0, SGI0, NULL};
const Segment * const SG75592[] = {SGI324838, SGI0, SGI0, NULL};
const Segment * const SG75593[] = {SGI324839, SGI0, SGI0, NULL};
const Segment * const SG75594[] = {SGI324840, SGI0, SGI0, NULL};
const Segment * const SG75595[] = {SGI324841, NULL};
const Segment * const SG75596[] = {SGI324842, NULL};
const Segment * const SG75597[] = {SGI324843, NULL};
const Segment * const SG75598[] = {SGI324844, NULL};
const Segment * const SG75599[] = {SGI324845, NULL};
const Segment * const SG75600[] = {SGI324846, NULL};
const Segment * const SG75601[] = {SGI324847, NULL};
const Segment * const SG75602[] = {SGI324848, SGI324848, SGI0, SGI0, SGI324849, SGI0, SGI0, NULL};
const Segment * const SG75603[] = {SGI324850, SGI324850, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75604[] = {SGI324851, SGI324851, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75605[] = {SGI324852, SGI324852, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75606[] = {SGI324853, SGI0, SGI0, NULL};
const Segment * const SG75607[] = {SGI324854, SGI324854, SGI0, SGI0, SGI324855, SGI0, NULL};
const Segment * const SG75608[] = {SGI324856, SGI324857, NULL};
const Segment * const SG75609[] = {SGI324858, SGI324859, NULL};
const Segment * const SG75610[] = {SGI324860, SGI324860, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75611[] = {SGI324861, SGI324861, SGI0, SGI0, SGI0, SGI0, SGI324862, NULL};
const Segment * const SG75612[] = {SGI324863, SGI0, SGI324863, SGI324864, SGI324865, SGI324866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75613[] = {SGI324867, SGI0, SGI324868, NULL};
const Segment * const SG75614[] = {SGI324869, SGI0, SGI324871, NULL};
const Segment * const SG75615[] = {SGI324870, SGI0, SGI324872, SGI324873, SGI324874, SGI324875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75616[] = {SGI324876, SGI0, SGI324877, NULL};
const Segment * const SG75617[] = {SGI324878, SGI0, SGI324879, SGI324880, SGI324881, SGI324882, SGI324883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324884, SGI324885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75618[] = {SGI324887, SGI0, SGI324888, SGI324889, SGI324890, SGI324891, SGI324892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75619[] = {SGI324895, SGI324895, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75620[] = {SGI324896, SGI324896, SGI0, SGI0, SGI324897, SGI0, NULL};
const Segment * const SG75621[] = {SGI324898, SGI0, SGI324899, SGI324900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324901, SGI324902, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75622[] = {SGI324903, SGI0, SGI324904, SGI324905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324906, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75623[] = {SGI324907, SGI0, SGI324908, SGI324909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324911, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75624[] = {SGI324912, NULL};
const Segment * const SG75625[] = {SGI324913, SGI324913, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75626[] = {SGI324914, SGI0, SGI324915, SGI324916, SGI324917, SGI324918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75627[] = {SGI324920, SGI0, SGI324921, SGI324922, SGI0, SGI324923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324924, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75628[] = {SGI324925, SGI0, SGI324926, SGI324927, SGI0, SGI324928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324929, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75629[] = {SGI324930, SGI0, SGI324931, SGI324932, SGI0, SGI324933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324934, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75630[] = {SGI324935, SGI0, SGI324936, SGI324937, SGI0, SGI324938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324939, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75631[] = {SGI324940, SGI0, SGI324941, SGI324942, SGI0, SGI324943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324944, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75632[] = {SGI324945, SGI324945, SGI0, SGI0, SGI0, SGI0, SGI324949, NULL};
const Segment * const SG75633[] = {SGI324946, SGI0, SGI324947, SGI324948, SGI0, SGI324950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324951, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75634[] = {SGI324952, SGI0, SGI324953, SGI324954, SGI0, SGI324955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324956, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75635[] = {SGI324957, SGI324957, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75636[] = {SGI324958, NULL};
const Segment * const SG75637[] = {SGI324959, NULL};
const Segment * const SG75638[] = {SGI324960, NULL};
const Segment * const SG75639[] = {SGI324961, SGI324961, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75640[] = {SGI324962, SGI324962, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75641[] = {SGI324963, SGI324963, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75642[] = {SGI324964, SGI324965, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75643[] = {SGI324966, SGI324966, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75644[] = {SGI324967, SGI0, SGI324967, SGI324968, SGI324969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75645[] = {SGI324970, SGI324970, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75646[] = {SGI324971, SGI0, SGI324971, SGI324972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324973, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75647[] = {SGI324974, SGI0, SGI324974, SGI324975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI324976, SGI324977, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75648[] = {SGI324978, NULL};
const Segment * const SG75649[] = {SGI324979, SGI0, SGI324980, SGI324981, SGI324982, SGI324983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75650[] = {SGI324984, SGI0, SGI324985, SGI324986, SGI324987, SGI324988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75651[] = {SGI324989, SGI0, SGI324989, SGI324990, SGI324991, SGI324992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75652[] = {SGI324993, SGI0, SGI324994, SGI324995, SGI324996, SGI324997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75653[] = {SGI324998, SGI324998, SGI0, SGI0, SGI0, SGI0, SGI324999, NULL};
const Segment * const SG75654[] = {SGI325000, SGI325001, NULL};
const Segment * const SG75655[] = {SGI325002, SGI325003, SGI325004, NULL};
const Segment * const SG75656[] = {SGI325005, SGI0, SGI325006, SGI325007, SGI325008, SGI325009, SGI325010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75657[] = {SGI325012, SGI0, SGI325013, SGI325014, SGI325015, SGI325016, SGI325017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75658[] = {SGI325020, SGI0, SGI325021, SGI325022, SGI325023, SGI325024, SGI325025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75659[] = {SGI325028, SGI0, SGI325029, SGI325030, SGI325031, SGI325032, SGI325033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75660[] = {SGI325035, SGI0, SGI325036, SGI325037, SGI325038, SGI325039, SGI325040, SGI325041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75661[] = {SGI325044, SGI0, SGI325045, SGI325046, SGI325047, SGI325048, SGI325049, SGI325050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75662[] = {SGI325052, SGI0, SGI325053, SGI325054, SGI325055, SGI325056, SGI325057, SGI325058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325060, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75663[] = {SGI325061, SGI325061, SGI0, SGI0, SGI0, SGI0, SGI325062, NULL};
const Segment * const SG75664[] = {SGI325063, SGI325063, SGI0, SGI0, SGI325064, SGI0, NULL};
const Segment * const SG75665[] = {SGI325065, SGI325066, NULL};
const Segment * const SG75666[] = {SGI325067, SGI325068, NULL};
const Segment * const SG75667[] = {SGI325069, SGI0, SGI0, NULL};
const Segment * const SG75668[] = {SGI325070, SGI0, SGI0, NULL};
const Segment * const SG75669[] = {SGI325071, SGI0, SGI325072, SGI325073, SGI325074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75670[] = {SGI325075, SGI0, SGI325076, SGI325077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325078, SGI325079, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75671[] = {SGI325080, SGI0, SGI325081, SGI325082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325083, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75672[] = {SGI325084, NULL};
const Segment * const SG75673[] = {SGI325085, SGI325085, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75674[] = {SGI325086, SGI325086, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75675[] = {SGI325087, SGI325088, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75676[] = {SGI325089, SGI0, SGI325090, SGI325091, SGI325092, SGI325093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75677[] = {SGI325094, SGI0, SGI325095, SGI325096, SGI325097, SGI325098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75678[] = {SGI325099, SGI0, SGI325100, SGI325101, SGI325102, SGI325103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325104, SGI0, SGI0, NULL};
const Segment * const SG75679[] = {SGI325105, SGI0, SGI325106, SGI325107, SGI325108, SGI325109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325110, SGI0, SGI0, NULL};
const Segment * const SG75680[] = {SGI325111, SGI0, SGI325112, SGI325113, SGI325114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75681[] = {SGI325115, SGI0, SGI325116, SGI325117, SGI325118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75682[] = {SGI325120, SGI0, SGI325121, SGI325122, SGI325123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325124, SGI0, SGI0, NULL};
const Segment * const SG75683[] = {SGI325125, SGI325126, SGI325127, NULL};
const Segment * const SG75684[] = {SGI325128, SGI325129, SGI325130, NULL};
const Segment * const SG75685[] = {SGI325131, SGI325132, SGI325133, NULL};
const Segment * const SG75686[] = {SGI325134, SGI325135, SGI325136, NULL};
const Segment * const SG75687[] = {SGI325137, SGI325138, SGI325139, NULL};
const Segment * const SG75688[] = {SGI325140, SGI325141, SGI325142, NULL};
const Segment * const SG75689[] = {SGI325143, SGI325143, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75690[] = {SGI325144, SGI325144, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75691[] = {SGI325145, SGI0, SGI325146, SGI325147, SGI325148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75692[] = {SGI325149, SGI0, SGI325150, SGI325151, SGI325152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75693[] = {SGI325153, SGI0, SGI325154, SGI325155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325156, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75694[] = {SGI325157, SGI325158, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75695[] = {SGI325159, SGI325159, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75696[] = {SGI325160, SGI325160, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75697[] = {SGI325161, SGI325161, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75698[] = {SGI325162, SGI0, SGI325163, SGI325164, SGI325165, SGI325166, SGI325167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75699[] = {SGI325170, SGI0, SGI325171, SGI325172, SGI325173, SGI325174, SGI325175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75700[] = {SGI325177, SGI0, SGI325178, SGI325179, SGI325180, SGI325181, SGI325182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75701[] = {SGI325184, SGI325185, NULL};
const Segment * const SG75702[] = {SGI325186, NULL};
const Segment * const SG75703[] = {SGI325187, NULL};
const Segment * const SG75704[] = {SGI325188, SGI325188, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75705[] = {SGI325189, SGI325189, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75706[] = {SGI325190, SGI0, SGI325191, SGI325192, SGI325193, SGI325194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325195, SGI0, NULL};
const Segment * const SG75707[] = {SGI325196, SGI0, SGI325197, SGI325198, SGI325199, SGI325200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325196, SGI0, NULL};
const Segment * const SG75708[] = {SGI325201, SGI0, SGI325202, SGI325203, SGI325204, SGI325205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325206, SGI325207, SGI0, NULL};
const Segment * const SG75709[] = {SGI325208, SGI0, SGI325209, SGI325210, SGI325211, SGI325212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325213, SGI325214, SGI0, NULL};
const Segment * const SG75710[] = {SGI325215, NULL};
const Segment * const SG75711[] = {SGI325216, NULL};
const Segment * const SG75712[] = {SGI325217, SGI325218, NULL};
const Segment * const SG75713[] = {SGI325219, SGI325219, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75714[] = {SGI325220, SGI325221, NULL};
const Segment * const SG75715[] = {SGI325222, SGI325223, NULL};
const Segment * const SG75716[] = {SGI325224, NULL};
const Segment * const SG75717[] = {SGI325225, SGI325226, NULL};
const Segment * const SG75718[] = {SGI325227, NULL};
const Segment * const SG75719[] = {SGI325228, SGI325228, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75720[] = {SGI325229, SGI325229, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75721[] = {SGI325230, SGI325230, SGI0, SGI0, SGI325231, SGI0, NULL};
const Segment * const SG75722[] = {SGI325232, SGI0, SGI0, NULL};
const Segment * const SG75723[] = {SGI325233, SGI0, SGI0, NULL};
const Segment * const SG75724[] = {SGI325234, SGI0, SGI0, NULL};
const Segment * const SG75725[] = {SGI325235, SGI0, SGI0, NULL};
const Segment * const SG75726[] = {SGI325236, SGI0, SGI0, NULL};
const Segment * const SG75727[] = {SGI325237, SGI0, SGI0, NULL};
const Segment * const SG75728[] = {SGI325238, SGI0, SGI0, NULL};
const Segment * const SG75729[] = {SGI325239, SGI0, SGI0, NULL};
const Segment * const SG75730[] = {SGI325240, SGI0, SGI325241, SGI325242, SGI325243, SGI325244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325245, SGI325246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325247, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75731[] = {SGI325248, SGI0, SGI325249, SGI325250, SGI325251, SGI325252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325254, SGI0, SGI325255, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75732[] = {SGI325256, SGI325257, NULL};
const Segment * const SG75733[] = {SGI325258, SGI325259, NULL};
const Segment * const SG75734[] = {SGI325260, SGI325261, NULL};
const Segment * const SG75735[] = {SGI325262, SGI325262, SGI0, SGI0, SGI0, SGI0, SGI325263, NULL};
const Segment * const SG75736[] = {SGI325264, SGI325264, SGI0, SGI0, SGI0, SGI325265, SGI0, NULL};
const Segment * const SG75737[] = {SGI325266, NULL};
const Segment * const SG75738[] = {SGI325268, NULL};
const Segment * const SG75739[] = {SGI325267, SGI325267, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75740[] = {SGI325269, SGI0, SGI325270, SGI325271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325272, SGI325273, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75741[] = {SGI325274, SGI0, SGI325275, SGI325276, SGI325277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75742[] = {SGI325278, SGI0, SGI325279, SGI325280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325281, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75743[] = {SGI325282, SGI325282, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75744[] = {SGI325283, NULL};
const Segment * const SG75745[] = {SGI325284, SGI325284, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75746[] = {SGI325285, SGI0, SGI325286, NULL};
const Segment * const SG75747[] = {SGI325287, SGI325287, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75748[] = {SGI325288, SGI0, SGI325289, NULL};
const Segment * const SG75749[] = {SGI325290, SGI325290, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75750[] = {SGI325292, SGI0, SGI325293, NULL};
const Segment * const SG75751[] = {SGI325291, SGI325291, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75752[] = {SGI325294, SGI0, SGI325295, NULL};
const Segment * const SG75753[] = {SGI325296, SGI325297, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75754[] = {SGI325298, SGI325298, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75755[] = {SGI325299, SGI325299, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75756[] = {SGI325300, SGI325300, SGI0, SGI0, SGI0, SGI0, SGI325301, NULL};
const Segment * const SG75757[] = {SGI325302, NULL};
const Segment * const SG75758[] = {SGI325303, SGI325303, SGI0, SGI0, SGI0, SGI0, SGI325304, NULL};
const Segment * const SG75759[] = {SGI325305, NULL};
const Segment * const SG75760[] = {SGI325306, SGI325307, NULL};
const Segment * const SG75761[] = {SGI325308, SGI325308, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75762[] = {SGI325309, SGI325309, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75763[] = {SGI325310, SGI325310, SGI0, SGI0, SGI0, SGI0, SGI325311, NULL};
const Segment * const SG75764[] = {SGI325312, SGI0, SGI325313, SGI325314, SGI325315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75765[] = {SGI325316, SGI0, SGI325317, SGI325318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325319, SGI325320, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75766[] = {SGI325321, SGI0, SGI325322, SGI325323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325324, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75767[] = {SGI325325, SGI325326, SGI325327, NULL};
const Segment * const SG75768[] = {SGI325328, NULL};
const Segment * const SG75769[] = {SGI325329, NULL};
const Segment * const SG75770[] = {SGI325330, NULL};
const Segment * const SG75771[] = {SGI325331, SGI325331, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75772[] = {SGI325332, SGI325333, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75773[] = {SGI325334, SGI325334, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75774[] = {SGI325335, NULL};
const Segment * const SG75775[] = {SGI325336, SGI325337, NULL};
const Segment * const SG75776[] = {SGI325338, SGI325338, SGI0, SGI0, SGI0, SGI0, SGI325339, NULL};
const Segment * const SG75777[] = {SGI325340, SGI325341, NULL};
const Segment * const SG75778[] = {SGI325342, SGI325342, SGI0, SGI0, SGI0, SGI0, SGI325343, NULL};
const Segment * const SG75779[] = {SGI325344, SGI325344, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75780[] = {SGI325345, NULL};
const Segment * const SG75781[] = {SGI325346, NULL};
const Segment * const SG75782[] = {SGI325347, NULL};
const Segment * const SG75783[] = {SGI325348, NULL};
const Segment * const SG75784[] = {SGI325349, NULL};
const Segment * const SG75785[] = {SGI325350, NULL};
const Segment * const SG75786[] = {SGI325351, NULL};
const Segment * const SG75787[] = {SGI325352, NULL};
const Segment * const SG75788[] = {SGI325353, NULL};
const Segment * const SG75789[] = {SGI325354, NULL};
const Segment * const SG75790[] = {SGI325355, NULL};
const Segment * const SG75791[] = {SGI325356, SGI0, SGI325357, SGI325358, SGI325359, SGI325360, SGI0, SGI0, SGI325361, SGI0, SGI0, SGI0, SGI0, SGI325362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75792[] = {SGI325363, SGI0, SGI325364, SGI325365, SGI325366, SGI325367, SGI0, SGI0, SGI325368, SGI0, SGI0, SGI0, SGI0, SGI325369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75793[] = {SGI325370, SGI0, SGI325371, SGI325372, SGI325373, SGI325374, SGI0, SGI0, SGI325375, SGI0, SGI0, SGI0, SGI0, SGI325376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75794[] = {SGI325377, SGI0, SGI325378, SGI325379, SGI325380, SGI325381, SGI0, SGI0, SGI325382, SGI0, SGI0, SGI0, SGI0, SGI325383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75795[] = {SGI325384, NULL};
const Segment * const SG75796[] = {SGI325385, SGI325385, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75797[] = {SGI325386, SGI325387, NULL};
const Segment * const SG75798[] = {SGI325388, SGI325388, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75799[] = {SGI325389, SGI0, SGI325390, SGI325391, SGI325392, SGI325393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75800[] = {SGI325396, SGI0, SGI325397, SGI325398, SGI325399, SGI325400, SGI0, SGI325401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75801[] = {SGI325404, SGI0, SGI325405, SGI325406, SGI325407, SGI325408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325409, SGI0, SGI325410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75802[] = {SGI325412, SGI0, SGI325413, SGI325414, SGI325415, SGI325416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75803[] = {SGI325419, SGI0, SGI325420, SGI325421, SGI325422, SGI325423, SGI0, SGI325424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325425, SGI0, NULL};
const Segment * const SG75804[] = {SGI325426, SGI325426, SGI0, SGI0, SGI0, SGI0, SGI325427, NULL};
const Segment * const SG75805[] = {SGI325428, SGI325428, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75806[] = {SGI325429, SGI325429, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75807[] = {SGI325430, NULL};
const Segment * const SG75808[] = {SGI325431, SGI325431, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75809[] = {SGI325432, SGI325433, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75810[] = {SGI325434, SGI325434, SGI0, SGI0, SGI0, SGI0, SGI325435, NULL};
const Segment * const SG75811[] = {SGI325436, SGI325437, NULL};
const Segment * const SG75812[] = {SGI325438, SGI325439, NULL};
const Segment * const SG75813[] = {SGI325440, SGI0, SGI325441, SGI325442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325443, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75814[] = {SGI325444, SGI0, SGI325445, SGI325446, SGI325447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75815[] = {SGI325448, SGI0, SGI325449, SGI325450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325451, SGI325452, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75816[] = {SGI325453, SGI325453, SGI0, SGI0, SGI325454, SGI0, NULL};
const Segment * const SG75817[] = {SGI325455, SGI325455, SGI0, SGI0, SGI325456, SGI0, NULL};
const Segment * const SG75818[] = {SGI325457, SGI325457, SGI0, SGI0, SGI0, SGI325458, SGI0, NULL};
const Segment * const SG75819[] = {SGI325459, SGI325460, NULL};
const Segment * const SG75820[] = {SGI325461, SGI325461, SGI0, SGI0, SGI325462, SGI0, NULL};
const Segment * const SG75821[] = {SGI325463, NULL};
const Segment * const SG75822[] = {SGI325464, NULL};
const Segment * const SG75823[] = {SGI325465, SGI325465, SGI0, SGI0, SGI325466, SGI0, NULL};
const Segment * const SG75824[] = {SGI325467, SGI0, SGI325468, SGI325469, SGI325470, SGI325471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75825[] = {SGI325472, SGI0, SGI325473, SGI325474, SGI325475, SGI325476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75826[] = {SGI325477, SGI0, SGI325478, SGI325479, SGI325480, SGI325481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75827[] = {SGI325482, SGI0, SGI325483, SGI325484, SGI325485, SGI325486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75828[] = {SGI325487, SGI325487, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75829[] = {SGI325488, SGI325488, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75830[] = {SGI325489, SGI325490, SGI325491, NULL};
const Segment * const SG75831[] = {SGI325492, SGI325493, SGI325494, NULL};
const Segment * const SG75832[] = {SGI325495, SGI325495, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75833[] = {SGI325496, SGI325496, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75834[] = {SGI325497, NULL};
const Segment * const SG75835[] = {SGI325498, SGI0, SGI0, NULL};
const Segment * const SG75836[] = {SGI325499, SGI0, SGI0, NULL};
const Segment * const SG75837[] = {SGI325500, SGI0, SGI0, NULL};
const Segment * const SG75838[] = {SGI325501, SGI0, SGI0, NULL};
const Segment * const SG75839[] = {SGI325502, SGI0, SGI325503, SGI325504, SGI325505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75840[] = {SGI325506, SGI0, SGI325507, SGI325508, SGI325509, SGI325510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75841[] = {SGI325511, SGI0, SGI325512, SGI325513, SGI325514, SGI325515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75842[] = {SGI325516, SGI0, SGI325517, SGI325518, SGI325519, SGI325520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75843[] = {SGI325521, SGI0, SGI325522, SGI325523, SGI325524, SGI325525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75844[] = {SGI325526, SGI0, SGI325527, SGI325528, SGI325529, SGI325530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325531, SGI0, SGI0, NULL};
const Segment * const SG75845[] = {SGI325532, SGI0, SGI325533, SGI325534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325535, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75846[] = {SGI325536, SGI0, SGI325537, SGI325538, SGI325539, SGI325540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75847[] = {SGI325542, SGI325542, SGI0, SGI0, SGI0, SGI0, SGI325543, NULL};
const Segment * const SG75848[] = {SGI325544, SGI325544, SGI0, SGI0, SGI0, SGI0, SGI325545, NULL};
const Segment * const SG75849[] = {SGI325546, SGI0, SGI325547, SGI325548, SGI325549, SGI325550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75850[] = {SGI325551, SGI0, SGI325552, SGI325553, SGI325554, SGI325555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75851[] = {SGI325556, SGI325556, SGI0, SGI0, SGI325557, SGI0, NULL};
const Segment * const SG75852[] = {SGI325558, SGI325558, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75853[] = {SGI325559, SGI325559, SGI325560, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75854[] = {SGI325561, SGI325561, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75855[] = {SGI325562, NULL};
const Segment * const SG75856[] = {SGI325563, SGI0, SGI325564, NULL};
const Segment * const SG75857[] = {SGI325565, SGI325565, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75858[] = {SGI325566, SGI325566, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75859[] = {SGI325567, SGI325567, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75860[] = {SGI325568, SGI325568, SGI0, SGI0, SGI0, SGI0, SGI325569, NULL};
const Segment * const SG75861[] = {SGI325570, SGI0, SGI325571, SGI325572, SGI325573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75862[] = {SGI325574, NULL};
const Segment * const SG75863[] = {SGI325575, SGI0, SGI325575, SGI325576, SGI325577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75864[] = {SGI325578, SGI0, SGI325578, SGI325579, SGI325580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75865[] = {SGI325581, SGI0, SGI325582, SGI325583, SGI325584, SGI325585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75866[] = {SGI325586, SGI0, SGI0, NULL};
const Segment * const SG75867[] = {SGI325587, SGI0, SGI325588, SGI325589, SGI325590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75868[] = {SGI325591, SGI0, SGI325592, SGI325593, SGI325594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75869[] = {SGI325595, SGI0, SGI325596, SGI325597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325598, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75870[] = {SGI325599, SGI0, SGI325600, NULL};
const Segment * const SG75871[] = {SGI325601, SGI0, SGI325602, NULL};
const Segment * const SG75872[] = {SGI325603, SGI0, SGI325604, NULL};
const Segment * const SG75873[] = {SGI325605, SGI0, SGI0, NULL};
const Segment * const SG75874[] = {SGI325606, SGI0, SGI0, NULL};
const Segment * const SG75875[] = {SGI325607, SGI0, SGI325608, NULL};
const Segment * const SG75876[] = {SGI325609, SGI0, SGI0, NULL};
const Segment * const SG75877[] = {SGI325610, SGI0, SGI0, NULL};
const Segment * const SG75878[] = {SGI325611, SGI0, SGI325612, NULL};
const Segment * const SG75879[] = {SGI325613, SGI0, SGI325614, NULL};
const Segment * const SG75880[] = {SGI325615, SGI0, SGI325616, NULL};
const Segment * const SG75881[] = {SGI325617, SGI325617, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75882[] = {SGI325618, SGI0, SGI325619, SGI325620, SGI0, SGI325621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325622, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75883[] = {SGI325623, SGI325623, SGI0, SGI0, SGI0, SGI0, SGI325624, NULL};
const Segment * const SG75884[] = {SGI325625, SGI0, SGI325626, SGI325627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75885[] = {SGI325628, SGI0, SGI325629, SGI325630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75886[] = {SGI325631, SGI0, SGI325632, SGI325633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75887[] = {SGI325634, SGI325634, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75888[] = {SGI325635, SGI325635, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75889[] = {SGI325636, SGI0, SGI325637, SGI325638, SGI0, SGI325639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325640, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75890[] = {SGI325641, SGI0, SGI325642, SGI325643, SGI0, SGI325644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325645, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75891[] = {SGI325646, SGI0, SGI325647, NULL};
const Segment * const SG75892[] = {SGI325648, SGI0, SGI0, NULL};
const Segment * const SG75893[] = {SGI325649, NULL};
const Segment * const SG75894[] = {SGI325650, SGI0, SGI0, NULL};
const Segment * const SG75895[] = {SGI325651, SGI0, SGI325652, NULL};
const Segment * const SG75896[] = {SGI325653, SGI0, SGI0, NULL};
const Segment * const SG75897[] = {SGI325654, SGI0, SGI0, NULL};
const Segment * const SG75898[] = {SGI325655, SGI0, SGI325656, SGI325657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75899[] = {SGI325658, SGI0, SGI325659, SGI325660, SGI325661, SGI325662, SGI0, SGI0, SGI325663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75900[] = {SGI325664, NULL};
const Segment * const SG75901[] = {SGI325665, SGI0, SGI325666, SGI325667, SGI325668, SGI325669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75902[] = {SGI325671, SGI0, SGI325672, NULL};
const Segment * const SG75903[] = {SGI325670, SGI0, SGI325673, SGI325674, SGI325675, SGI325676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325677, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75904[] = {SGI325678, SGI0, SGI325679, NULL};
const Segment * const SG75905[] = {SGI325680, SGI0, SGI325681, SGI325682, SGI325683, SGI325684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75906[] = {SGI325686, SGI0, SGI325687, SGI325688, SGI325689, SGI325690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75907[] = {SGI325691, SGI325691, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75908[] = {SGI325692, SGI325692, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75909[] = {SGI325693, NULL};
const Segment * const SG75910[] = {SGI325694, SGI0, SGI325695, SGI325696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75911[] = {SGI325697, SGI325697, SGI0, SGI0, SGI325698, SGI0, NULL};
const Segment * const SG75912[] = {SGI325699, SGI325699, SGI0, SGI0, SGI325700, SGI0, NULL};
const Segment * const SG75913[] = {SGI325701, SGI325701, SGI0, SGI0, SGI325702, SGI0, NULL};
const Segment * const SG75914[] = {SGI325703, SGI325703, SGI0, SGI0, SGI325704, SGI0, NULL};
const Segment * const SG75915[] = {SGI325705, SGI325705, SGI0, SGI0, SGI325706, SGI0, NULL};
const Segment * const SG75916[] = {SGI325707, SGI325707, SGI0, SGI0, SGI325708, SGI0, NULL};
const Segment * const SG75917[] = {SGI325709, SGI325709, SGI0, SGI0, SGI325710, SGI0, NULL};
const Segment * const SG75918[] = {SGI325711, SGI325711, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75919[] = {SGI325712, SGI325712, SGI0, SGI0, SGI0, SGI0, SGI325713, NULL};
const Segment * const SG75920[] = {SGI325714, NULL};
const Segment * const SG75921[] = {SGI325715, SGI325715, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75922[] = {SGI325716, SGI325716, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75923[] = {SGI325717, SGI0, SGI325718, SGI325719, SGI325720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75924[] = {SGI325721, SGI0, SGI325722, SGI325723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325724, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75925[] = {SGI325725, NULL};
const Segment * const SG75926[] = {SGI325726, SGI325726, SGI0, SGI0, SGI325727, SGI0, SGI0, NULL};
const Segment * const SG75927[] = {SGI325728, NULL};
const Segment * const SG75928[] = {SGI325729, NULL};
const Segment * const SG75929[] = {SGI325730, SGI0, SGI325731, SGI325732, SGI325733, SGI325734, SGI325735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75930[] = {SGI325737, SGI0, SGI325738, SGI325739, SGI325740, SGI325741, SGI325742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75931[] = {SGI325745, SGI0, SGI325746, SGI325747, SGI325748, SGI325749, SGI325750, SGI325751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75932[] = {SGI325753, SGI0, SGI325754, SGI325755, SGI325756, SGI325757, SGI325758, SGI325759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75933[] = {SGI325761, SGI0, SGI325762, SGI325763, SGI325764, SGI325765, SGI325766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75934[] = {SGI325768, SGI0, SGI325769, SGI325770, SGI325771, SGI325772, SGI325773, SGI325774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75935[] = {SGI325777, SGI0, SGI325778, SGI325779, SGI325780, SGI325781, SGI325782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75936[] = {SGI325784, SGI0, SGI325785, SGI325786, SGI325787, SGI325788, SGI325789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75937[] = {SGI325792, SGI0, SGI0, NULL};
const Segment * const SG75938[] = {SGI325793, SGI0, SGI0, NULL};
const Segment * const SG75939[] = {SGI325794, SGI325794, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75940[] = {SGI325795, SGI0, SGI325796, SGI325797, SGI325798, SGI325799, SGI0, SGI0, SGI325800, SGI0, SGI0, SGI0, SGI0, SGI325801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75941[] = {SGI325802, SGI0, SGI325803, SGI325804, SGI325805, SGI325806, SGI0, SGI0, SGI325807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75942[] = {SGI325808, SGI0, SGI325809, SGI325810, SGI325811, SGI325812, SGI0, SGI0, SGI325813, SGI0, SGI0, SGI0, SGI0, SGI325814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75943[] = {SGI325815, SGI0, SGI325816, SGI325817, SGI325818, SGI325819, SGI0, SGI0, SGI325820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75944[] = {SGI325821, SGI0, SGI325822, SGI325823, SGI325824, SGI325825, SGI0, SGI0, SGI325826, SGI0, SGI0, SGI0, SGI0, SGI325827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75945[] = {SGI325828, SGI0, SGI325829, SGI325830, SGI325831, SGI325832, SGI0, SGI0, SGI325833, SGI0, SGI0, SGI0, SGI0, SGI325834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75946[] = {SGI325835, SGI0, SGI325836, SGI325837, SGI325838, SGI325839, SGI0, SGI0, SGI325840, SGI0, SGI0, SGI0, SGI0, SGI325841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75947[] = {SGI325842, NULL};
const Segment * const SG75948[] = {SGI325843, NULL};
const Segment * const SG75949[] = {SGI325844, NULL};
const Segment * const SG75950[] = {SGI325845, NULL};
const Segment * const SG75951[] = {SGI325846, SGI0, SGI325847, NULL};
const Segment * const SG75952[] = {SGI325848, SGI0, SGI325849, NULL};
const Segment * const SG75953[] = {SGI325850, NULL};
const Segment * const SG75954[] = {SGI325851, SGI0, SGI325852, SGI325853, SGI325854, SGI325855, SGI325856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325858, SGI0, SGI0, NULL};
const Segment * const SG75955[] = {SGI325859, SGI0, SGI325860, SGI325861, SGI325862, SGI325863, SGI325864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75956[] = {SGI325868, SGI325868, SGI0, SGI0, SGI0, SGI0, SGI325874, NULL};
const Segment * const SG75957[] = {SGI325866, SGI0, SGI325867, SGI325869, SGI325870, SGI325871, SGI325872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75958[] = {SGI325876, SGI325876, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75959[] = {SGI325877, SGI0, SGI325878, NULL};
const Segment * const SG75960[] = {SGI325884, SGI0, SGI0, NULL};
const Segment * const SG75961[] = {SGI325879, SGI0, SGI325880, SGI325881, SGI325882, SGI325883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325885, SGI0, SGI325886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75962[] = {SGI325887, SGI0, SGI325888, SGI325889, SGI325890, SGI325891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325893, SGI0, SGI0, NULL};
const Segment * const SG75963[] = {SGI325894, SGI0, SGI325895, SGI325896, SGI325897, SGI325898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75964[] = {SGI325900, SGI0, SGI325901, SGI325902, SGI325903, SGI325904, SGI0, SGI325905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75965[] = {SGI325907, SGI325907, SGI0, SGI325908, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75966[] = {SGI325909, SGI0, SGI325910, SGI325911, SGI325912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325914, SGI0, SGI0, NULL};
const Segment * const SG75967[] = {SGI325915, SGI0, SGI325916, SGI325917, SGI325918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75968[] = {SGI325920, SGI0, SGI325921, SGI325922, SGI325923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75969[] = {SGI325925, SGI0, SGI325926, SGI325927, SGI325928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75970[] = {SGI325930, SGI0, SGI325931, SGI325932, SGI325933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325935, SGI0, SGI0, NULL};
const Segment * const SG75971[] = {SGI325936, SGI0, SGI325937, SGI325938, SGI325939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75972[] = {SGI325941, SGI325941, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75973[] = {SGI325942, SGI0, SGI325943, SGI325944, SGI325945, SGI325946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325947, SGI0, SGI0, SGI0, SGI325948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75974[] = {SGI325949, SGI0, SGI325950, SGI325951, SGI325952, SGI325953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75975[] = {SGI325955, SGI0, SGI325956, SGI325957, SGI325958, SGI325959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325961, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75976[] = {SGI325962, SGI325963, SGI325964, NULL};
const Segment * const SG75977[] = {SGI325965, SGI325966, SGI325967, NULL};
const Segment * const SG75978[] = {SGI325968, NULL};
const Segment * const SG75979[] = {SGI325969, NULL};
const Segment * const SG75980[] = {SGI325970, SGI325970, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75981[] = {SGI325971, SGI325971, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75982[] = {SGI0, SGI0, SGI325972, SGI325973, SGI325974, SGI325975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75983[] = {SGI325976, SGI0, SGI325977, SGI325978, SGI325979, SGI325980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75984[] = {SGI325981, SGI0, SGI325982, SGI325983, SGI325984, SGI325985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75985[] = {SGI325986, SGI325986, SGI0, SGI0, SGI0, SGI0, SGI325987, NULL};
const Segment * const SG75986[] = {SGI325988, SGI325988, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75987[] = {SGI325989, SGI325989, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75988[] = {SGI325990, SGI325990, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75989[] = {SGI325991, NULL};
const Segment * const SG75990[] = {SGI325992, SGI0, SGI325993, SGI325994, SGI325995, SGI325996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325997, SGI0, SGI325998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI325999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75991[] = {SGI326000, SGI0, SGI326001, SGI326002, SGI326003, SGI326004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75992[] = {SGI326005, SGI326005, SGI0, SGI0, SGI0, SGI0, SGI326008, NULL};
const Segment * const SG75993[] = {SGI326009, NULL};
const Segment * const SG75994[] = {SGI326010, SGI0, SGI326011, SGI326012, SGI326013, SGI326014, SGI326015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75995[] = {SGI326016, NULL};
const Segment * const SG75996[] = {SGI326017, SGI0, SGI326018, SGI326019, SGI326020, SGI326021, SGI326022, SGI326023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75997[] = {SGI326024, NULL};
const Segment * const SG75998[] = {SGI326025, SGI326025, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG75999[] = {SGI326026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76000[] = {SGI326028, SGI0, SGI326029, SGI326030, SGI326031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76001[] = {SGI326032, SGI0, SGI326032, SGI326033, SGI326034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76002[] = {SGI326035, SGI0, SGI326036, SGI326037, SGI326038, SGI326039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76003[] = {SGI326040, SGI326040, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76004[] = {SGI326041, SGI326041, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76005[] = {SGI326042, SGI0, SGI0, NULL};
const Segment * const SG76006[] = {SGI326043, SGI326043, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76007[] = {SGI326044, SGI0, SGI326045, NULL};
const Segment * const SG76008[] = {SGI326046, NULL};
const Segment * const SG76009[] = {SGI326047, SGI326047, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76010[] = {SGI326048, SGI0, SGI326049, SGI326050, SGI326051, SGI326052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326053, SGI0, SGI326054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76011[] = {SGI326056, SGI0, SGI326057, SGI326058, SGI326059, SGI326060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326061, SGI0, SGI326062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76012[] = {SGI326064, SGI0, SGI326065, SGI326066, SGI326067, SGI326068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76013[] = {SGI326071, NULL};
const Segment * const SG76014[] = {SGI326072, SGI0, SGI326073, SGI326074, SGI326075, SGI326076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326077, SGI0, NULL};
const Segment * const SG76015[] = {SGI326078, SGI326078, SGI326079, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76016[] = {SGI326080, SGI0, SGI326081, SGI326082, SGI326083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76017[] = {SGI326084, SGI0, SGI326085, SGI326086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326087, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76018[] = {SGI326088, SGI0, SGI326089, SGI326090, SGI326091, SGI326092, SGI326093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326094, SGI326095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76019[] = {SGI326097, NULL};
const Segment * const SG76020[] = {SGI326098, SGI326098, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76021[] = {SGI326099, SGI326100, SGI326101, NULL};
const Segment * const SG76022[] = {SGI326102, SGI326103, SGI326104, NULL};
const Segment * const SG76023[] = {SGI326105, SGI326106, SGI326107, NULL};
const Segment * const SG76024[] = {SGI326108, SGI326109, SGI326110, NULL};
const Segment * const SG76025[] = {SGI326111, SGI326112, SGI326113, NULL};
const Segment * const SG76026[] = {SGI326114, SGI326115, SGI326116, NULL};
const Segment * const SG76027[] = {SGI326117, SGI326118, SGI326119, NULL};
const Segment * const SG76028[] = {SGI326120, SGI326121, SGI326122, NULL};
const Segment * const SG76029[] = {SGI326123, SGI326124, SGI326125, NULL};
const Segment * const SG76030[] = {SGI326126, SGI326127, SGI326128, NULL};
const Segment * const SG76031[] = {SGI326129, SGI0, SGI326130, SGI326131, SGI326132, SGI326133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326135, SGI0, NULL};
const Segment * const SG76032[] = {SGI326136, SGI0, SGI326137, SGI326138, SGI326139, SGI326140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326141, SGI326142, SGI0, NULL};
const Segment * const SG76033[] = {SGI326143, SGI0, SGI326144, SGI326145, SGI326146, SGI326147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326148, SGI0, NULL};
const Segment * const SG76034[] = {SGI326152, NULL};
const Segment * const SG76035[] = {SGI326149, SGI0, SGI326150, SGI326151, SGI326153, SGI326154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326156, SGI0, NULL};
const Segment * const SG76036[] = {SGI326157, SGI0, SGI326158, SGI326159, SGI326160, SGI326161, SGI0, SGI326162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326163, SGI0, SGI326164, SGI0, NULL};
const Segment * const SG76037[] = {SGI326165, SGI326165, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76038[] = {SGI326166, SGI326166, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76039[] = {SGI326167, SGI0, SGI326168, SGI326169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326170, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76040[] = {SGI326171, SGI0, SGI326172, SGI326173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326174, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76041[] = {SGI326175, SGI0, SGI326176, NULL};
const Segment * const SG76042[] = {SGI326177, SGI0, SGI326178, NULL};
const Segment * const SG76043[] = {SGI326179, SGI326179, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76044[] = {SGI326180, SGI0, SGI326181, SGI326182, SGI326183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76045[] = {SGI326184, SGI0, SGI326185, SGI326186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326187, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76046[] = {SGI326188, SGI326189, SGI326190, NULL};
const Segment * const SG76047[] = {SGI326191, SGI326192, SGI326193, NULL};
const Segment * const SG76048[] = {SGI326194, SGI326194, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76049[] = {SGI326195, SGI326195, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76050[] = {SGI326196, SGI326196, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76051[] = {SGI326197, SGI326197, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76052[] = {SGI326198, SGI326198, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76053[] = {SGI326199, SGI326199, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76054[] = {SGI326200, NULL};
const Segment * const SG76055[] = {SGI326201, NULL};
const Segment * const SG76056[] = {SGI326202, NULL};
const Segment * const SG76057[] = {SGI326203, NULL};
const Segment * const SG76058[] = {SGI326204, NULL};
const Segment * const SG76059[] = {SGI326205, NULL};
const Segment * const SG76060[] = {SGI326206, NULL};
const Segment * const SG76061[] = {SGI326207, NULL};
const Segment * const SG76062[] = {SGI326208, NULL};
const Segment * const SG76063[] = {SGI326209, NULL};
const Segment * const SG76064[] = {SGI326210, SGI326210, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76065[] = {SGI326211, NULL};
const Segment * const SG76066[] = {SGI326212, SGI326212, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76067[] = {SGI326213, SGI0, SGI326214, SGI326215, SGI326216, SGI326217, SGI326218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326220, SGI0, SGI0, NULL};
const Segment * const SG76068[] = {SGI326221, SGI0, SGI326222, SGI326223, SGI326224, SGI326225, SGI326226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76069[] = {SGI326228, SGI326228, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76070[] = {SGI326229, SGI326229, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76071[] = {SGI326230, NULL};
const Segment * const SG76072[] = {SGI326231, NULL};
const Segment * const SG76073[] = {SGI326232, SGI326232, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76074[] = {SGI326233, SGI326233, SGI0, SGI0, SGI0, SGI326234, SGI0, NULL};
const Segment * const SG76075[] = {SGI326235, SGI326235, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76076[] = {SGI326236, SGI326236, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76077[] = {SGI326237, SGI326237, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76078[] = {SGI326238, SGI326238, SGI0, SGI0, SGI326239, SGI0, NULL};
const Segment * const SG76079[] = {SGI326240, SGI326240, SGI0, SGI0, SGI0, SGI0, SGI326241, NULL};
const Segment * const SG76080[] = {SGI326242, SGI0, SGI326243, SGI326244, SGI326245, SGI326246, SGI0, SGI326247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76081[] = {SGI326248, SGI0, SGI326249, SGI326250, SGI326251, SGI326252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76082[] = {SGI326253, SGI0, SGI326254, SGI326255, SGI326256, SGI326257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76083[] = {SGI0, SGI0, SGI326258, SGI326259, SGI326260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76084[] = {SGI326261, SGI0, SGI326262, SGI326263, SGI326264, SGI326265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76085[] = {SGI326266, SGI0, SGI326267, SGI326268, SGI326269, SGI326270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76086[] = {SGI326271, SGI0, SGI326272, SGI326273, SGI326274, SGI326275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76087[] = {SGI326278, NULL};
const Segment * const SG76088[] = {SGI326279, SGI0, SGI0, NULL};
const Segment * const SG76089[] = {SGI326280, SGI0, SGI326281, SGI326282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326283, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76090[] = {SGI326284, SGI0, SGI326285, SGI326286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326287, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76091[] = {SGI326288, SGI326288, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76092[] = {SGI326289, SGI326290, SGI326291, NULL};
const Segment * const SG76093[] = {SGI326292, SGI326292, SGI0, SGI0, SGI0, SGI0, SGI326293, NULL};
const Segment * const SG76094[] = {SGI326294, SGI326294, SGI0, SGI0, SGI0, SGI0, SGI326295, NULL};
const Segment * const SG76095[] = {SGI326296, SGI326296, SGI0, SGI0, SGI0, SGI0, SGI326297, NULL};
const Segment * const SG76096[] = {SGI326298, SGI326298, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76097[] = {SGI326299, SGI326299, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76098[] = {SGI326300, SGI326300, SGI0, SGI0, SGI326301, SGI0, NULL};
const Segment * const SG76099[] = {SGI326302, SGI326302, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76100[] = {SGI326303, SGI326303, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76101[] = {SGI326304, SGI326305, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76102[] = {SGI326306, SGI326306, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76103[] = {SGI326307, SGI326307, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76104[] = {SGI326308, SGI326308, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76105[] = {SGI326309, SGI326309, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76106[] = {SGI326310, SGI0, SGI326311, NULL};
const Segment * const SG76107[] = {SGI326312, SGI0, SGI326313, NULL};
const Segment * const SG76108[] = {SGI326314, SGI0, SGI326315, NULL};
const Segment * const SG76109[] = {SGI326316, SGI0, SGI326317, SGI326318, SGI326319, SGI326320, SGI0, SGI326321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326322, SGI0, NULL};
const Segment * const SG76110[] = {SGI326323, SGI0, SGI326324, SGI326325, SGI326326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326327, SGI0, NULL};
const Segment * const SG76111[] = {SGI326328, SGI326328, SGI0, SGI0, SGI0, SGI0, SGI326329, NULL};
const Segment * const SG76112[] = {SGI326330, SGI326330, SGI0, SGI0, SGI0, SGI0, SGI326331, NULL};
const Segment * const SG76113[] = {SGI326332, NULL};
const Segment * const SG76114[] = {SGI326333, NULL};
const Segment * const SG76115[] = {SGI326334, NULL};
const Segment * const SG76116[] = {SGI326335, NULL};
const Segment * const SG76117[] = {SGI326336, NULL};
const Segment * const SG76118[] = {SGI326337, SGI0, SGI326338, SGI326339, SGI326340, SGI326341, SGI0, SGI0, SGI326342, SGI0, SGI0, SGI0, SGI0, SGI326343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76119[] = {SGI326344, SGI0, SGI326345, SGI326346, SGI326347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76120[] = {SGI326348, SGI0, SGI326349, SGI326350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326351, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76121[] = {SGI326352, SGI0, SGI326353, SGI326354, SGI326355, SGI326356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76122[] = {SGI326357, SGI326357, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76123[] = {SGI326358, NULL};
const Segment * const SG76124[] = {SGI326359, SGI326359, SGI0, SGI0, SGI0, SGI0, SGI326360, NULL};
const Segment * const SG76125[] = {SGI326361, SGI326361, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76126[] = {SGI326362, SGI326362, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76127[] = {SGI326363, SGI0, SGI326364, SGI326365, SGI326366, SGI326367, SGI0, SGI326368, SGI326369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76128[] = {SGI326370, SGI0, SGI326371, SGI326372, SGI326373, SGI326374, SGI0, SGI326375, SGI326376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76129[] = {SGI326377, SGI0, SGI326378, SGI326379, SGI326380, SGI326381, SGI0, SGI0, SGI326382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76130[] = {SGI326383, SGI0, SGI0, NULL};
const Segment * const SG76131[] = {SGI326384, SGI0, SGI326385, SGI326386, SGI326387, SGI326388, SGI0, SGI0, SGI326389, SGI0, SGI0, SGI0, SGI0, SGI326390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76132[] = {SGI326391, SGI0, SGI326392, SGI326393, SGI326394, SGI326395, SGI0, SGI0, SGI326396, SGI0, SGI0, SGI0, SGI0, SGI326397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76133[] = {SGI326398, SGI0, SGI326399, SGI326400, SGI326401, SGI326402, SGI0, SGI0, SGI326403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76134[] = {SGI326404, SGI0, SGI326405, SGI326406, SGI326407, SGI326408, SGI0, SGI0, SGI326409, SGI0, SGI0, SGI0, SGI0, SGI326410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76135[] = {SGI326411, SGI0, SGI326412, SGI326413, SGI326414, SGI326415, SGI0, SGI0, SGI326416, SGI0, SGI0, SGI0, SGI0, SGI326417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76136[] = {SGI326418, SGI0, SGI326419, SGI326420, SGI326421, SGI326422, SGI0, SGI0, SGI326423, SGI0, SGI0, SGI0, SGI0, SGI326424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76137[] = {SGI326425, SGI0, SGI326426, SGI326427, SGI326428, SGI326429, SGI0, SGI0, SGI326430, SGI0, SGI0, SGI0, SGI0, SGI326431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76138[] = {SGI326432, SGI0, SGI326433, SGI326434, SGI326435, SGI326436, SGI0, SGI0, SGI326437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76139[] = {SGI326438, SGI0, SGI326439, SGI326440, SGI326441, SGI326442, SGI0, SGI0, SGI326443, SGI0, SGI0, SGI0, SGI0, SGI326444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76140[] = {SGI326445, SGI0, SGI326446, SGI326447, SGI326448, SGI326449, SGI0, SGI0, SGI326450, SGI0, SGI0, SGI0, SGI0, SGI326451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76141[] = {SGI326452, SGI0, SGI326453, SGI326454, SGI326455, SGI326456, SGI0, SGI0, SGI326457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76142[] = {SGI326458, SGI0, SGI326459, SGI326460, SGI326461, SGI326462, SGI0, SGI0, SGI326463, SGI0, SGI0, SGI0, SGI0, SGI326464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76143[] = {SGI326465, SGI0, SGI326466, SGI326467, SGI326468, SGI326469, SGI0, SGI0, SGI326470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76144[] = {SGI326471, SGI0, SGI326472, SGI326473, SGI326474, SGI326475, SGI0, SGI0, SGI326476, SGI0, SGI0, SGI0, SGI0, SGI326477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76145[] = {SGI326478, SGI0, SGI326479, SGI326480, SGI326481, SGI326482, SGI0, SGI0, SGI326483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76146[] = {SGI326484, SGI0, SGI326485, SGI326486, SGI326487, SGI326488, SGI0, SGI0, SGI326489, SGI0, SGI0, SGI0, SGI0, SGI326490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76147[] = {SGI326491, SGI0, SGI326492, SGI326493, SGI326494, SGI326495, SGI0, SGI0, SGI326496, SGI0, SGI0, SGI0, SGI0, SGI326497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76148[] = {SGI326498, SGI0, SGI326499, SGI326500, SGI326501, SGI326502, SGI0, SGI0, SGI326503, SGI0, SGI0, SGI0, SGI0, SGI326504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76149[] = {SGI326505, SGI0, SGI326506, SGI326507, SGI326508, SGI326509, SGI0, SGI0, SGI326510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76150[] = {SGI326511, SGI0, SGI326512, SGI326513, SGI326514, SGI326515, SGI0, SGI0, SGI326516, SGI0, SGI0, SGI0, SGI0, SGI326517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76151[] = {SGI326518, SGI0, SGI0, NULL};
const Segment * const SG76152[] = {SGI326519, SGI0, SGI326520, SGI326521, SGI326522, SGI326523, SGI0, SGI0, SGI326524, SGI0, SGI0, SGI0, SGI0, SGI326525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76153[] = {SGI326526, SGI0, SGI326527, SGI326528, SGI326529, SGI326530, SGI0, SGI0, SGI326531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76154[] = {SGI326532, SGI0, SGI326533, SGI326534, SGI326535, SGI326536, SGI0, SGI0, SGI326537, SGI0, SGI0, SGI0, SGI0, SGI326538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76155[] = {SGI326539, SGI0, SGI326540, SGI326541, SGI326542, SGI326543, SGI0, SGI0, SGI326544, SGI0, SGI0, SGI0, SGI0, SGI326545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76156[] = {SGI326546, SGI0, SGI326547, SGI326548, SGI326549, SGI326550, SGI0, SGI0, SGI326551, SGI0, SGI0, SGI0, SGI0, SGI326552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76157[] = {SGI326553, SGI0, SGI326554, SGI326555, SGI326556, SGI326557, SGI0, SGI0, SGI326558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76158[] = {SGI326559, SGI0, SGI326560, SGI326561, SGI326562, SGI326563, SGI326564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326565, SGI326566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76159[] = {SGI326568, SGI326568, SGI0, SGI0, SGI0, SGI326569, SGI0, NULL};
const Segment * const SG76160[] = {SGI326570, SGI326570, SGI0, SGI0, SGI0, SGI0, SGI326571, NULL};
const Segment * const SG76161[] = {SGI326572, SGI326572, SGI0, SGI0, SGI326573, SGI0, NULL};
const Segment * const SG76162[] = {SGI326574, SGI326574, SGI0, SGI0, SGI326575, SGI0, NULL};
const Segment * const SG76163[] = {SGI326576, SGI326576, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76164[] = {SGI326577, SGI326577, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76165[] = {SGI326578, SGI326579, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76166[] = {SGI326580, SGI326580, SGI0, SGI0, SGI326581, SGI0, SGI0, NULL};
const Segment * const SG76167[] = {SGI326582, SGI0, SGI326583, NULL};
const Segment * const SG76168[] = {SGI326589, SGI0, SGI326590, NULL};
const Segment * const SG76169[] = {SGI326584, SGI0, SGI326585, SGI326586, SGI326587, SGI326588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76170[] = {SGI326591, SGI0, SGI326592, NULL};
const Segment * const SG76171[] = {SGI326593, SGI0, SGI326594, NULL};
const Segment * const SG76172[] = {SGI326595, SGI0, SGI326596, NULL};
const Segment * const SG76173[] = {SGI326597, SGI0, SGI326598, SGI326599, SGI326600, SGI326601, SGI326602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326603, SGI326604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76174[] = {SGI326606, SGI326606, SGI0, SGI0, SGI326607, SGI0, NULL};
const Segment * const SG76175[] = {SGI326608, SGI326608, SGI0, SGI0, SGI0, SGI0, SGI326609, NULL};
const Segment * const SG76176[] = {SGI326610, SGI326610, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76177[] = {SGI326611, SGI326611, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76178[] = {SGI326612, SGI326612, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76179[] = {SGI326613, SGI326613, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76180[] = {SGI326614, SGI326615, SGI326616, NULL};
const Segment * const SG76181[] = {SGI326617, SGI326617, SGI0, SGI0, SGI0, SGI0, SGI326618, NULL};
const Segment * const SG76182[] = {SGI326619, SGI326619, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76183[] = {SGI326620, SGI326620, SGI0, SGI326621, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76184[] = {SGI326622, NULL};
const Segment * const SG76185[] = {SGI326623, SGI326623, SGI0, SGI326624, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76186[] = {SGI326625, SGI326625, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76187[] = {SGI326626, SGI326626, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76188[] = {SGI326627, SGI0, SGI326628, SGI326629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326630, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76189[] = {SGI326631, NULL};
const Segment * const SG76190[] = {SGI326632, SGI0, SGI326633, SGI326634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76191[] = {SGI326635, SGI0, SGI326636, SGI326637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326638, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76192[] = {SGI326639, NULL};
const Segment * const SG76193[] = {SGI326640, SGI0, SGI326641, SGI326642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76194[] = {SGI326643, SGI0, SGI326644, SGI326645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76195[] = {SGI326646, SGI326646, SGI0, SGI0, SGI0, SGI0, SGI326647, NULL};
const Segment * const SG76196[] = {SGI326648, NULL};
const Segment * const SG76197[] = {SGI326649, SGI326649, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76198[] = {SGI326650, SGI326650, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76199[] = {SGI326651, SGI326651, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76200[] = {SGI326652, SGI326653, SGI326654, NULL};
const Segment * const SG76201[] = {SGI326655, SGI326656, SGI326657, NULL};
const Segment * const SG76202[] = {SGI326658, SGI326659, SGI326660, NULL};
const Segment * const SG76203[] = {SGI326661, SGI326662, SGI326663, NULL};
const Segment * const SG76204[] = {SGI326664, SGI326665, SGI326666, NULL};
const Segment * const SG76205[] = {SGI326667, SGI326668, SGI326669, NULL};
const Segment * const SG76206[] = {SGI326670, SGI326671, SGI326672, NULL};
const Segment * const SG76207[] = {SGI326673, SGI326673, SGI0, SGI0, SGI0, SGI0, SGI326674, NULL};
const Segment * const SG76208[] = {SGI326675, SGI326675, SGI0, SGI0, SGI0, SGI0, SGI326676, NULL};
const Segment * const SG76209[] = {SGI326677, SGI0, SGI326678, SGI326679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326680, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76210[] = {SGI326681, SGI0, SGI326682, SGI326683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326684, SGI326685, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76211[] = {SGI326686, SGI326686, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76212[] = {SGI326687, SGI326687, SGI0, SGI0, SGI0, SGI0, SGI326688, NULL};
const Segment * const SG76213[] = {SGI326689, SGI326690, SGI326691, NULL};
const Segment * const SG76214[] = {SGI326692, SGI0, SGI326693, SGI326694, SGI326695, SGI326696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326698, SGI0, NULL};
const Segment * const SG76215[] = {SGI326699, SGI0, SGI326700, SGI326701, SGI326702, SGI326703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326704, SGI0, NULL};
const Segment * const SG76216[] = {SGI326705, SGI0, SGI326706, SGI326707, SGI326708, SGI326709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326710, SGI326711, SGI0, NULL};
const Segment * const SG76217[] = {SGI326712, SGI0, SGI326713, SGI326714, SGI326715, SGI326716, SGI0, SGI326717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326719, SGI0, NULL};
const Segment * const SG76218[] = {SGI326720, NULL};
const Segment * const SG76219[] = {SGI326721, NULL};
const Segment * const SG76220[] = {SGI326722, SGI326722, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76221[] = {SGI326723, SGI326723, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76222[] = {SGI326724, SGI326724, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76223[] = {SGI326725, SGI0, SGI326726, SGI326727, SGI326728, SGI326729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76224[] = {SGI326730, SGI326730, SGI0, SGI0, SGI0, SGI0, SGI326731, NULL};
const Segment * const SG76225[] = {SGI326732, SGI0, SGI326733, SGI326734, SGI326735, SGI326736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326737, SGI0, NULL};
const Segment * const SG76226[] = {SGI326738, SGI0, SGI326739, SGI326740, SGI326741, SGI326742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326743, SGI326744, SGI0, NULL};
const Segment * const SG76227[] = {SGI326745, SGI0, SGI326745, SGI326746, SGI326747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76228[] = {SGI326748, SGI0, SGI326749, SGI326750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326751, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76229[] = {SGI326752, SGI326752, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76230[] = {SGI326753, SGI326753, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76231[] = {SGI326754, NULL};
const Segment * const SG76232[] = {SGI326755, SGI326755, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76233[] = {SGI326756, SGI0, SGI326757, SGI326758, SGI326759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76234[] = {SGI326760, SGI0, SGI326760, SGI326761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76235[] = {SGI326762, SGI326762, SGI0, SGI0, SGI0, SGI0, SGI326763, NULL};
const Segment * const SG76236[] = {SGI326764, SGI0, SGI326765, SGI326766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76237[] = {SGI326767, SGI326767, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76238[] = {SGI326768, SGI0, SGI326769, NULL};
const Segment * const SG76239[] = {SGI326770, SGI0, SGI326771, NULL};
const Segment * const SG76240[] = {SGI326772, SGI0, SGI326773, NULL};
const Segment * const SG76241[] = {SGI326774, SGI0, SGI326775, NULL};
const Segment * const SG76242[] = {SGI326776, SGI0, SGI326777, NULL};
const Segment * const SG76243[] = {SGI326778, SGI0, SGI326779, NULL};
const Segment * const SG76244[] = {SGI326780, SGI0, SGI326781, NULL};
const Segment * const SG76245[] = {SGI326782, SGI0, SGI326783, NULL};
const Segment * const SG76246[] = {SGI326784, SGI0, SGI326785, NULL};
const Segment * const SG76247[] = {SGI326786, SGI0, SGI326787, NULL};
const Segment * const SG76248[] = {SGI326788, SGI0, SGI326789, NULL};
const Segment * const SG76249[] = {SGI326790, SGI0, SGI326791, NULL};
const Segment * const SG76250[] = {SGI326792, SGI0, SGI326793, NULL};
const Segment * const SG76251[] = {SGI326794, SGI0, SGI326795, NULL};
const Segment * const SG76252[] = {SGI326796, NULL};
const Segment * const SG76253[] = {SGI326797, NULL};
const Segment * const SG76254[] = {SGI326798, SGI326798, SGI0, SGI0, SGI0, SGI0, SGI326799, NULL};
const Segment * const SG76255[] = {SGI326800, SGI0, SGI326801, SGI326802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326803, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76256[] = {SGI326804, SGI326805, SGI0, NULL};
const Segment * const SG76257[] = {SGI326806, SGI0, SGI326807, SGI326808, SGI326809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76258[] = {SGI326810, SGI0, SGI326811, SGI326812, SGI326813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76259[] = {SGI326814, SGI0, SGI326815, SGI326816, SGI326817, SGI326818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326819, SGI326820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326821, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76260[] = {SGI326822, SGI0, SGI326823, SGI326824, SGI326825, SGI326826, SGI326827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76261[] = {SGI326829, SGI0, SGI326830, SGI326831, SGI326832, SGI326833, SGI326834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76262[] = {SGI326836, SGI0, SGI326837, SGI326838, SGI326839, SGI326840, SGI326841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76263[] = {SGI326843, SGI0, SGI326844, SGI326845, SGI326846, SGI326847, SGI326848, SGI326849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76264[] = {SGI326851, SGI0, SGI326852, SGI326853, SGI326854, SGI326855, SGI326856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76265[] = {SGI326859, SGI326859, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76266[] = {SGI326860, SGI326860, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76267[] = {SGI326861, SGI0, SGI326862, SGI326863, SGI326864, SGI326865, SGI326866, SGI326867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76268[] = {SGI326869, SGI0, SGI326870, SGI326871, SGI326872, SGI326873, SGI326874, SGI326875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76269[] = {SGI326878, NULL};
const Segment * const SG76270[] = {SGI326879, NULL};
const Segment * const SG76271[] = {SGI326880, SGI326880, SGI0, SGI0, SGI0, SGI0, SGI326881, NULL};
const Segment * const SG76272[] = {SGI326882, NULL};
const Segment * const SG76273[] = {SGI326883, NULL};
const Segment * const SG76274[] = {SGI326884, SGI326885, SGI326886, NULL};
const Segment * const SG76275[] = {SGI326887, SGI326888, SGI326889, NULL};
const Segment * const SG76276[] = {SGI326890, SGI326891, SGI326892, NULL};
const Segment * const SG76277[] = {SGI326893, SGI0, SGI326894, SGI326895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326896, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76278[] = {SGI326897, SGI0, SGI326898, SGI326899, SGI326900, SGI326901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76279[] = {SGI326902, SGI0, SGI326903, SGI326904, SGI326905, SGI326906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76280[] = {SGI326907, SGI326907, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76281[] = {SGI326908, SGI326909, SGI326910, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76282[] = {SGI326911, SGI326911, SGI326912, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76283[] = {SGI326913, SGI326913, SGI326914, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76284[] = {SGI326915, SGI326915, SGI0, SGI0, SGI0, SGI0, SGI326916, NULL};
const Segment * const SG76285[] = {SGI326917, SGI326917, SGI0, SGI0, SGI0, SGI0, SGI326918, NULL};
const Segment * const SG76286[] = {SGI326919, SGI326920, SGI326921, NULL};
const Segment * const SG76287[] = {SGI326922, SGI326923, SGI326924, NULL};
const Segment * const SG76288[] = {SGI326925, SGI326925, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76289[] = {SGI326926, SGI326926, SGI0, SGI0, SGI0, SGI0, SGI326927, NULL};
const Segment * const SG76290[] = {SGI326928, SGI0, SGI326929, SGI326930, SGI326931, SGI326932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76291[] = {SGI326933, SGI326933, SGI0, SGI0, SGI326934, SGI0, NULL};
const Segment * const SG76292[] = {SGI326935, NULL};
const Segment * const SG76293[] = {SGI326936, SGI326937, SGI326938, NULL};
const Segment * const SG76294[] = {SGI326939, SGI0, SGI326940, SGI326941, SGI326942, SGI326943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76295[] = {SGI326944, SGI326945, SGI326946, NULL};
const Segment * const SG76296[] = {SGI326947, SGI326948, SGI326949, NULL};
const Segment * const SG76297[] = {SGI326950, SGI326951, SGI326952, NULL};
const Segment * const SG76298[] = {SGI326953, SGI0, SGI326954, SGI326955, SGI326956, SGI326957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76299[] = {SGI326958, SGI0, SGI326959, SGI326960, SGI326961, SGI326962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76300[] = {SGI326963, SGI0, SGI326964, NULL};
const Segment * const SG76301[] = {SGI326965, SGI326965, SGI0, SGI0, SGI326966, SGI0, NULL};
const Segment * const SG76302[] = {SGI326967, NULL};
const Segment * const SG76303[] = {SGI326968, SGI326968, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76304[] = {SGI326969, SGI326969, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76305[] = {SGI326970, SGI326970, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76306[] = {SGI326971, SGI0, SGI326972, SGI326973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76307[] = {SGI326974, SGI0, SGI326975, SGI326976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI326977, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76308[] = {SGI326978, SGI0, SGI326979, SGI326980, SGI326981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76309[] = {SGI326982, SGI0, SGI0, NULL};
const Segment * const SG76310[] = {SGI326983, SGI326983, SGI0, SGI0, SGI0, SGI0, SGI326984, NULL};
const Segment * const SG76311[] = {SGI326985, SGI0, SGI0, NULL};
const Segment * const SG76312[] = {SGI326986, SGI0, SGI326987, SGI326988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76313[] = {SGI326989, SGI326989, SGI0, SGI0, SGI326990, SGI0, NULL};
const Segment * const SG76314[] = {SGI326991, SGI326991, SGI0, SGI0, SGI0, SGI0, SGI326992, NULL};
const Segment * const SG76315[] = {SGI326993, SGI0, SGI326994, NULL};
const Segment * const SG76316[] = {SGI326995, SGI0, SGI326996, NULL};
const Segment * const SG76317[] = {SGI326997, SGI0, SGI326998, NULL};
const Segment * const SG76318[] = {SGI326999, SGI0, SGI327000, NULL};
const Segment * const SG76319[] = {SGI327001, SGI0, SGI327002, NULL};
const Segment * const SG76320[] = {SGI327003, SGI0, SGI327004, NULL};
const Segment * const SG76321[] = {SGI327005, SGI0, SGI327006, NULL};
const Segment * const SG76322[] = {SGI327007, SGI0, SGI327008, NULL};
const Segment * const SG76323[] = {SGI327009, SGI0, SGI327010, NULL};
const Segment * const SG76324[] = {SGI327011, SGI0, SGI327012, NULL};
const Segment * const SG76325[] = {SGI327013, SGI0, SGI327014, NULL};
const Segment * const SG76326[] = {SGI327015, SGI0, SGI327016, NULL};
const Segment * const SG76327[] = {SGI327017, SGI327017, SGI0, SGI0, SGI0, SGI0, SGI327018, NULL};
const Segment * const SG76328[] = {SGI327019, SGI327019, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76329[] = {SGI327020, SGI327020, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76330[] = {SGI327021, SGI0, SGI327022, SGI327023, SGI327024, SGI327025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327026, SGI327027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76331[] = {SGI327028, SGI0, SGI327029, SGI327030, SGI327031, SGI327032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327034, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76332[] = {SGI327035, SGI327035, SGI0, SGI0, SGI0, SGI0, SGI327036, NULL};
const Segment * const SG76333[] = {SGI327037, SGI0, SGI0, NULL};
const Segment * const SG76334[] = {SGI327038, SGI327038, SGI0, SGI0, SGI0, SGI0, SGI327039, NULL};
const Segment * const SG76335[] = {SGI327040, SGI0, SGI327041, SGI327042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76336[] = {SGI327043, SGI0, SGI327044, SGI327045, SGI327046, SGI327047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327048, SGI0, NULL};
const Segment * const SG76337[] = {SGI327049, SGI0, SGI327050, SGI327051, SGI327052, SGI327053, SGI0, SGI327054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327055, SGI0, NULL};
const Segment * const SG76338[] = {SGI327056, SGI0, SGI327057, SGI327058, SGI327059, SGI327060, SGI0, SGI327061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327063, SGI0, NULL};
const Segment * const SG76339[] = {SGI327064, SGI0, SGI327065, SGI327066, SGI327067, SGI327068, SGI0, SGI327069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327070, SGI0, SGI327071, SGI0, NULL};
const Segment * const SG76340[] = {SGI327072, SGI0, SGI327073, SGI327074, SGI327075, SGI327076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327078, SGI0, NULL};
const Segment * const SG76341[] = {SGI327079, SGI0, SGI327080, SGI327081, SGI327082, SGI327083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327085, SGI0, NULL};
const Segment * const SG76342[] = {SGI327086, SGI0, SGI327087, SGI327088, SGI327089, SGI327090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327091, SGI0, NULL};
const Segment * const SG76343[] = {SGI327092, SGI0, SGI327093, SGI327094, SGI327095, SGI327096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327097, SGI327098, SGI0, NULL};
const Segment * const SG76344[] = {SGI327099, SGI0, SGI327100, SGI327101, SGI327102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327103, SGI0, NULL};
const Segment * const SG76345[] = {SGI327104, SGI0, SGI327105, SGI327106, SGI327107, SGI327108, SGI327109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76346[] = {SGI327111, SGI0, SGI327112, SGI327113, SGI327114, SGI327115, SGI327116, SGI327117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76347[] = {SGI327119, SGI0, SGI327120, SGI327121, SGI327122, SGI327123, SGI327124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76348[] = {SGI327126, SGI0, SGI327127, SGI327128, SGI327129, SGI327130, SGI327131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76349[] = {SGI327134, SGI0, SGI327135, SGI327136, SGI327137, SGI327138, SGI327139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76350[] = {SGI327141, SGI0, SGI327142, SGI327143, SGI327144, SGI327145, SGI327146, SGI327147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76351[] = {SGI327149, SGI327149, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76352[] = {SGI327150, SGI327150, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76353[] = {SGI327151, SGI327151, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76354[] = {SGI327152, SGI327152, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76355[] = {SGI327153, SGI0, SGI327154, SGI327155, SGI327156, SGI327157, SGI0, SGI0, SGI327158, SGI0, SGI0, SGI0, SGI0, SGI327159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76356[] = {SGI327160, SGI0, SGI327161, SGI327162, SGI327163, SGI327164, SGI0, SGI0, SGI327165, SGI0, SGI0, SGI0, SGI0, SGI327166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76357[] = {SGI327167, SGI0, SGI327168, SGI327169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327170, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76358[] = {SGI327171, SGI327171, SGI0, SGI0, SGI0, SGI0, SGI327172, NULL};
const Segment * const SG76359[] = {SGI327173, SGI327173, SGI0, SGI0, SGI0, SGI0, SGI327174, NULL};
const Segment * const SG76360[] = {SGI327175, NULL};
const Segment * const SG76361[] = {SGI327176, SGI327177, SGI327178, SGI327179, SGI327180, SGI327181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327183, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76362[] = {SGI327184, SGI327185, SGI327186, SGI327187, SGI327188, SGI327189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327191, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76363[] = {SGI327192, SGI327192, SGI0, SGI0, SGI0, SGI0, SGI327193, NULL};
const Segment * const SG76364[] = {SGI327194, SGI0, SGI0, NULL};
const Segment * const SG76365[] = {SGI327195, SGI327195, SGI0, SGI0, SGI0, SGI0, SGI327196, NULL};
const Segment * const SG76366[] = {SGI327197, SGI327197, SGI0, SGI0, SGI0, SGI0, SGI327198, NULL};
const Segment * const SG76367[] = {SGI327199, SGI0, SGI327200, SGI327201, SGI327202, SGI327203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327204, SGI327205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327206, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76368[] = {SGI327207, SGI327207, SGI0, SGI0, SGI0, SGI0, SGI327208, NULL};
const Segment * const SG76369[] = {SGI327209, SGI0, SGI327210, SGI327211, SGI327212, SGI327213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76370[] = {SGI327215, NULL};
const Segment * const SG76371[] = {SGI327216, SGI0, SGI327217, SGI327218, SGI327219, SGI327220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327221, SGI327222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76372[] = {SGI327223, SGI0, SGI327224, SGI327225, SGI327226, SGI327227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327229, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76373[] = {SGI327230, NULL};
const Segment * const SG76374[] = {SGI327231, SGI0, SGI0, NULL};
const Segment * const SG76375[] = {SGI327232, SGI0, SGI0, NULL};
const Segment * const SG76376[] = {SGI327233, SGI0, SGI0, NULL};
const Segment * const SG76377[] = {SGI327234, SGI327234, SGI0, SGI0, SGI0, SGI0, SGI327235, NULL};
const Segment * const SG76378[] = {SGI327236, SGI327236, SGI0, SGI0, SGI0, SGI0, SGI327237, NULL};
const Segment * const SG76379[] = {SGI327238, SGI327238, SGI0, SGI0, SGI0, SGI0, SGI327239, NULL};
const Segment * const SG76380[] = {SGI327240, NULL};
const Segment * const SG76381[] = {SGI327241, NULL};
const Segment * const SG76382[] = {SGI327242, SGI327242, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76383[] = {SGI327243, SGI327243, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76384[] = {SGI327244, SGI327244, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76385[] = {SGI327245, SGI327246, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76386[] = {SGI327247, SGI327247, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76387[] = {SGI327248, SGI327249, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76388[] = {SGI327250, SGI327251, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76389[] = {SGI327252, SGI327252, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76390[] = {SGI327253, SGI327253, SGI0, SGI0, SGI327254, SGI0, SGI0, NULL};
const Segment * const SG76391[] = {SGI327255, SGI327255, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76392[] = {SGI327256, SGI0, SGI327257, SGI327258, SGI327259, SGI327260, SGI327261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76393[] = {SGI327264, SGI0, SGI327265, SGI327266, SGI327267, SGI327268, SGI327269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76394[] = {SGI327271, SGI0, SGI327272, SGI327273, SGI327274, SGI327275, SGI327276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76395[] = {SGI327278, SGI0, SGI327279, SGI327280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327281, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76396[] = {SGI327282, SGI327282, SGI0, SGI327283, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76397[] = {SGI327284, SGI327284, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76398[] = {SGI327285, SGI0, SGI327286, SGI327287, SGI327288, SGI0, SGI0, SGI0, SGI327289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76399[] = {SGI327290, SGI0, SGI327291, SGI327292, SGI327293, SGI327294, SGI0, SGI0, SGI327295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76400[] = {SGI327296, SGI0, SGI327297, SGI327298, SGI327299, SGI327300, SGI0, SGI0, SGI327301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76401[] = {SGI327302, SGI0, SGI327303, SGI327304, SGI327305, SGI327306, SGI0, SGI0, SGI327307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76402[] = {SGI327308, SGI0, SGI327309, SGI327310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327311, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76403[] = {SGI327312, SGI0, SGI327313, SGI327314, SGI327315, SGI327316, SGI0, SGI327317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327318, SGI0, SGI327319, SGI0, NULL};
const Segment * const SG76404[] = {SGI327320, SGI0, SGI327321, SGI327322, SGI327323, SGI327324, SGI0, SGI327325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327327, SGI0, NULL};
const Segment * const SG76405[] = {SGI327328, SGI0, SGI327329, SGI327330, SGI327331, SGI327332, SGI0, SGI327333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327334, SGI0, NULL};
const Segment * const SG76406[] = {SGI327335, SGI0, SGI327336, SGI327337, SGI327338, SGI327339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327340, SGI0, NULL};
const Segment * const SG76407[] = {SGI327341, SGI0, SGI327342, SGI327343, SGI327344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76408[] = {SGI327345, SGI327345, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76409[] = {SGI327346, SGI327346, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76410[] = {SGI327347, SGI0, SGI327348, SGI327349, SGI327350, SGI327351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76411[] = {SGI327352, SGI327352, SGI0, SGI327353, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76412[] = {SGI327354, SGI327354, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76413[] = {SGI327355, SGI327356, SGI327357, NULL};
const Segment * const SG76414[] = {SGI327359, SGI327360, SGI327361, NULL};
const Segment * const SG76415[] = {SGI327358, SGI327358, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76416[] = {SGI327362, SGI327363, SGI327364, NULL};
const Segment * const SG76417[] = {SGI327365, SGI327365, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76418[] = {SGI327366, SGI327366, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76419[] = {SGI327367, SGI327367, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76420[] = {SGI327368, SGI327368, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76421[] = {SGI327369, SGI327369, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76422[] = {SGI327370, SGI327370, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76423[] = {SGI327371, SGI327371, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76424[] = {SGI327372, NULL};
const Segment * const SG76425[] = {SGI327373, NULL};
const Segment * const SG76426[] = {SGI327374, SGI327375, SGI327376, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76427[] = {SGI327377, SGI327377, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76428[] = {SGI327378, SGI327378, SGI0, SGI327379, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76429[] = {SGI327380, SGI0, SGI327381, SGI327382, SGI327383, SGI327384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76430[] = {SGI327385, SGI0, SGI327386, SGI327387, SGI327388, SGI327389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76431[] = {SGI327391, SGI0, SGI327392, SGI327393, SGI327394, SGI327395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76432[] = {SGI327397, SGI0, SGI327398, SGI327399, SGI327400, SGI327401, SGI0, SGI327402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76433[] = {SGI327404, SGI327404, SGI327405, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76434[] = {SGI327406, SGI327406, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76435[] = {SGI327407, SGI327407, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76436[] = {SGI327408, SGI327408, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76437[] = {SGI327409, SGI327409, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76438[] = {SGI327410, SGI327410, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76439[] = {SGI327411, SGI327412, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76440[] = {SGI327413, SGI0, SGI0, NULL};
const Segment * const SG76441[] = {SGI327414, SGI0, SGI0, NULL};
const Segment * const SG76442[] = {SGI327415, SGI327415, SGI327416, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76443[] = {SGI327417, SGI327417, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76444[] = {SGI327418, SGI327418, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76445[] = {SGI327419, SGI327419, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76446[] = {SGI327420, SGI327420, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76447[] = {SGI327421, SGI0, SGI327422, NULL};
const Segment * const SG76448[] = {SGI327423, SGI327423, SGI0, SGI0, SGI327424, SGI0, SGI0, NULL};
const Segment * const SG76449[] = {SGI327425, SGI327425, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76450[] = {SGI327426, SGI327426, SGI0, SGI0, SGI327427, SGI0, SGI0, NULL};
const Segment * const SG76451[] = {SGI327428, SGI327428, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76452[] = {SGI327429, SGI327429, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76453[] = {SGI327430, SGI327430, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76454[] = {SGI327431, SGI327431, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76455[] = {SGI327432, SGI327432, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76456[] = {SGI327433, SGI327433, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76457[] = {SGI327434, SGI0, SGI327435, SGI327436, SGI327437, SGI327438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76458[] = {SGI327439, SGI0, SGI0, NULL};
const Segment * const SG76459[] = {SGI327440, SGI327440, SGI0, SGI0, SGI327441, SGI0, NULL};
const Segment * const SG76460[] = {SGI327442, SGI327442, SGI327443, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76461[] = {SGI327444, SGI327444, SGI0, SGI0, SGI327445, SGI0, SGI0, NULL};
const Segment * const SG76462[] = {SGI327446, SGI327446, SGI0, SGI0, SGI327447, SGI0, NULL};
const Segment * const SG76463[] = {SGI327448, SGI327448, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76464[] = {SGI327449, SGI327449, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76465[] = {SGI327450, SGI327450, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76466[] = {SGI327451, SGI327451, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76467[] = {SGI327452, SGI327452, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76468[] = {SGI327454, SGI327454, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76469[] = {SGI327453, SGI0, SGI327455, SGI327456, SGI327457, SGI327458, SGI0, SGI0, SGI327459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76470[] = {SGI327460, SGI0, SGI327461, NULL};
const Segment * const SG76471[] = {SGI327462, SGI0, SGI327463, SGI327464, SGI327465, SGI327466, SGI0, SGI327467, SGI327468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76472[] = {SGI327469, SGI0, SGI327470, SGI327471, SGI327472, SGI327473, SGI0, SGI0, SGI327474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76473[] = {SGI327475, SGI327475, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76474[] = {SGI327476, SGI0, SGI327476, SGI327477, SGI327478, SGI327479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76475[] = {SGI327480, NULL};
const Segment * const SG76476[] = {SGI327481, NULL};
const Segment * const SG76477[] = {SGI327482, SGI327482, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76478[] = {SGI327483, SGI327483, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76479[] = {SGI327484, SGI327485, SGI0, NULL};
const Segment * const SG76480[] = {SGI327486, SGI327487, SGI327488, NULL};
const Segment * const SG76481[] = {SGI327489, SGI327489, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76482[] = {SGI327490, SGI327490, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76483[] = {SGI327491, SGI327491, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76484[] = {SGI327492, SGI327492, SGI0, SGI0, SGI327493, SGI0, SGI0, NULL};
const Segment * const SG76485[] = {SGI327494, SGI0, SGI327495, SGI327496, SGI327497, SGI327498, SGI327499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76486[] = {SGI327501, SGI0, SGI327502, SGI327503, SGI327504, SGI327505, SGI327506, SGI327507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76487[] = {SGI327509, SGI0, SGI327510, SGI327511, SGI327512, SGI327513, SGI327514, SGI327515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76488[] = {SGI327517, NULL};
const Segment * const SG76489[] = {SGI327518, SGI327518, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76490[] = {SGI327519, SGI327519, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76491[] = {SGI327520, SGI327520, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76492[] = {SGI327521, SGI327521, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76493[] = {SGI327522, SGI327522, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76494[] = {SGI327528, SGI327528, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76495[] = {SGI327523, SGI0, SGI327524, SGI327525, SGI327526, SGI327527, SGI327529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76496[] = {SGI327531, SGI0, SGI327532, SGI327533, SGI327534, SGI327535, SGI327536, SGI327537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76497[] = {SGI327539, SGI327539, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76498[] = {SGI327540, SGI327540, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76499[] = {SGI327541, SGI327541, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76500[] = {SGI327542, SGI327542, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76501[] = {SGI327543, SGI327543, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76502[] = {SGI327544, SGI327544, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76503[] = {SGI327545, SGI327545, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76504[] = {SGI327546, SGI327546, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76505[] = {SGI327547, SGI327547, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76506[] = {SGI327548, SGI327548, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76507[] = {SGI327549, SGI0, SGI327550, SGI327551, SGI327552, SGI327553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76508[] = {SGI327554, SGI327554, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76509[] = {SGI327555, SGI0, SGI327556, NULL};
const Segment * const SG76510[] = {SGI327557, SGI0, SGI327558, NULL};
const Segment * const SG76511[] = {SGI327559, SGI0, SGI327560, NULL};
const Segment * const SG76512[] = {SGI327561, SGI0, SGI327562, NULL};
const Segment * const SG76513[] = {SGI327563, SGI0, SGI327564, NULL};
const Segment * const SG76514[] = {SGI327565, SGI0, SGI327566, NULL};
const Segment * const SG76515[] = {SGI327567, SGI327567, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76516[] = {SGI327568, SGI0, SGI327569, NULL};
const Segment * const SG76517[] = {SGI327570, SGI0, SGI327571, NULL};
const Segment * const SG76518[] = {SGI327572, SGI327572, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76519[] = {SGI327573, SGI327573, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76520[] = {SGI327574, SGI327575, SGI327576, NULL};
const Segment * const SG76521[] = {SGI327577, SGI327578, SGI327579, NULL};
const Segment * const SG76522[] = {SGI327580, SGI327580, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76523[] = {SGI327581, SGI0, SGI327582, SGI327583, SGI327584, SGI327585, SGI327586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76524[] = {SGI327588, SGI0, SGI327589, SGI327590, SGI327591, SGI327592, SGI327593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76525[] = {SGI327595, SGI327595, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76526[] = {SGI327596, SGI327597, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76527[] = {SGI327598, SGI327598, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76528[] = {SGI327599, SGI327599, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76529[] = {SGI327600, SGI327600, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76530[] = {SGI327601, SGI327602, SGI327603, NULL};
const Segment * const SG76531[] = {SGI327604, SGI327604, SGI0, SGI0, SGI327605, SGI0, NULL};
const Segment * const SG76532[] = {SGI327606, SGI327606, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76533[] = {SGI327607, SGI327607, SGI0, SGI0, SGI327608, SGI0, SGI0, NULL};
const Segment * const SG76534[] = {SGI327609, SGI327609, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76535[] = {SGI327610, SGI327610, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76536[] = {SGI327611, SGI327611, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76537[] = {SGI327612, SGI327612, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76538[] = {SGI327613, SGI327613, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76539[] = {SGI327614, SGI327614, SGI0, SGI0, SGI327615, SGI0, NULL};
const Segment * const SG76540[] = {SGI327616, SGI327616, SGI0, SGI0, SGI327617, SGI0, SGI0, NULL};
const Segment * const SG76541[] = {SGI327618, SGI327618, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76542[] = {SGI327619, SGI327619, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76543[] = {SGI327620, SGI327620, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76544[] = {SGI327621, SGI327621, SGI0, SGI0, SGI327622, SGI0, SGI0, NULL};
const Segment * const SG76545[] = {SGI327623, NULL};
const Segment * const SG76546[] = {SGI327624, NULL};
const Segment * const SG76547[] = {SGI327625, SGI0, SGI0, NULL};
const Segment * const SG76548[] = {SGI327626, SGI0, SGI0, NULL};
const Segment * const SG76549[] = {SGI327627, SGI0, SGI327628, SGI327629, SGI327630, SGI327631, SGI327632, SGI327633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76550[] = {SGI327635, SGI0, SGI327636, SGI327637, SGI327638, SGI327639, SGI327640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76551[] = {SGI327642, SGI0, SGI327643, SGI327644, SGI327645, SGI327646, SGI327647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76552[] = {SGI327649, SGI0, SGI327650, SGI327651, SGI327652, SGI327653, SGI327654, SGI327655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76553[] = {SGI327657, SGI0, SGI327658, SGI327659, SGI327660, SGI327661, SGI327662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76554[] = {SGI327664, SGI0, SGI327665, SGI327666, SGI327667, SGI327668, SGI327669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76555[] = {SGI327672, SGI0, SGI327673, NULL};
const Segment * const SG76556[] = {SGI327674, SGI0, SGI0, NULL};
const Segment * const SG76557[] = {SGI327675, SGI327675, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76558[] = {SGI327676, SGI327676, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76559[] = {SGI327677, SGI327677, SGI0, SGI0, SGI327678, SGI0, SGI0, NULL};
const Segment * const SG76560[] = {SGI327679, SGI327679, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76561[] = {SGI327680, SGI327680, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76562[] = {SGI327681, SGI0, SGI327682, SGI327683, SGI327684, SGI327685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76563[] = {SGI327687, SGI0, SGI0, NULL};
const Segment * const SG76564[] = {SGI327688, SGI327688, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76565[] = {SGI327689, SGI327690, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76566[] = {SGI327691, NULL};
const Segment * const SG76567[] = {SGI327692, SGI327692, SGI0, SGI0, SGI327693, SGI0, SGI0, NULL};
const Segment * const SG76568[] = {SGI327694, SGI327694, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76569[] = {SGI327695, SGI327695, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76570[] = {SGI327696, SGI327696, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76571[] = {SGI327697, SGI0, SGI327698, SGI327699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76572[] = {SGI327700, SGI327701, SGI327702, NULL};
const Segment * const SG76573[] = {SGI327703, NULL};
const Segment * const SG76574[] = {SGI327704, SGI327704, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76575[] = {SGI327705, SGI327705, SGI0, SGI0, SGI327706, SGI0, SGI0, NULL};
const Segment * const SG76576[] = {SGI327707, SGI327707, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76577[] = {SGI327708, SGI327708, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76578[] = {SGI327709, SGI327709, SGI0, SGI0, SGI327710, SGI0, NULL};
const Segment * const SG76579[] = {SGI327711, SGI327712, SGI327713, NULL};
const Segment * const SG76580[] = {SGI327714, SGI327714, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76581[] = {SGI327715, SGI327715, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76582[] = {SGI327716, SGI327716, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76583[] = {SGI327717, SGI327717, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76584[] = {SGI327718, SGI327718, SGI0, SGI0, SGI327719, SGI0, SGI0, NULL};
const Segment * const SG76585[] = {SGI327720, SGI327720, SGI0, SGI0, SGI327721, SGI0, SGI0, NULL};
const Segment * const SG76586[] = {SGI327722, SGI327722, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76587[] = {SGI327723, NULL};
const Segment * const SG76588[] = {SGI327724, SGI327724, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76589[] = {SGI327725, SGI327725, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76590[] = {SGI327726, NULL};
const Segment * const SG76591[] = {SGI327727, SGI327727, SGI0, SGI0, SGI327728, SGI0, SGI0, NULL};
const Segment * const SG76592[] = {SGI327729, SGI327729, SGI0, SGI0, SGI327730, SGI0, SGI0, NULL};
const Segment * const SG76593[] = {SGI327731, SGI327731, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76594[] = {SGI327732, SGI327732, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76595[] = {SGI327733, SGI327733, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76596[] = {SGI327734, SGI327734, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76597[] = {SGI327735, SGI327736, SGI327737, NULL};
const Segment * const SG76598[] = {SGI327738, SGI327739, SGI327740, NULL};
const Segment * const SG76599[] = {SGI327741, SGI327742, SGI327743, NULL};
const Segment * const SG76600[] = {SGI327744, SGI327745, SGI327746, NULL};
const Segment * const SG76601[] = {SGI327747, SGI327748, SGI327749, NULL};
const Segment * const SG76602[] = {SGI327750, SGI327750, SGI0, SGI0, SGI327751, SGI0, SGI0, NULL};
const Segment * const SG76603[] = {SGI327752, SGI327752, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76604[] = {SGI327753, SGI327753, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76605[] = {SGI327754, SGI327754, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76606[] = {SGI327755, SGI327755, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76607[] = {SGI327756, SGI327756, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76608[] = {SGI327757, SGI0, SGI327758, SGI327759, SGI0, SGI327760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327761, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76609[] = {SGI327762, SGI0, SGI327763, NULL};
const Segment * const SG76610[] = {SGI327764, SGI0, SGI327765, NULL};
const Segment * const SG76611[] = {SGI327766, NULL};
const Segment * const SG76612[] = {SGI327767, SGI327767, SGI0, SGI0, SGI327768, SGI0, NULL};
const Segment * const SG76613[] = {SGI327769, SGI0, SGI327770, SGI327771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327772, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76614[] = {SGI327773, NULL};
const Segment * const SG76615[] = {SGI327774, NULL};
const Segment * const SG76616[] = {SGI327775, SGI0, SGI327776, SGI327777, SGI327778, SGI327779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327780, SGI327781, SGI0, NULL};
const Segment * const SG76617[] = {SGI327782, SGI0, SGI327783, SGI327784, SGI327785, SGI327786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327787, SGI0, NULL};
const Segment * const SG76618[] = {SGI327788, SGI327788, SGI0, SGI0, SGI327789, SGI0, NULL};
const Segment * const SG76619[] = {SGI327790, NULL};
const Segment * const SG76620[] = {SGI327791, NULL};
const Segment * const SG76621[] = {SGI327792, NULL};
const Segment * const SG76622[] = {SGI327793, NULL};
const Segment * const SG76623[] = {SGI327794, NULL};
const Segment * const SG76624[] = {SGI327795, SGI0, SGI327796, SGI327797, SGI327798, SGI327799, SGI0, SGI0, SGI327800, SGI0, SGI0, SGI0, SGI0, SGI327801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76625[] = {SGI327802, SGI0, SGI327803, SGI327804, SGI327805, SGI327806, SGI0, SGI0, SGI327807, SGI0, SGI0, SGI0, SGI0, SGI327808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76626[] = {SGI327809, SGI327809, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76627[] = {SGI327810, SGI0, SGI327811, NULL};
const Segment * const SG76628[] = {SGI327812, SGI0, SGI327813, NULL};
const Segment * const SG76629[] = {SGI327814, NULL};
const Segment * const SG76630[] = {SGI327815, SGI0, SGI327816, SGI327817, SGI327818, SGI327819, SGI0, SGI327820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327822, SGI0, NULL};
const Segment * const SG76631[] = {SGI327823, SGI0, SGI327824, SGI327825, SGI327826, SGI327827, SGI0, SGI327828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327829, SGI0, NULL};
const Segment * const SG76632[] = {SGI327830, NULL};
const Segment * const SG76633[] = {SGI327831, NULL};
const Segment * const SG76634[] = {SGI327832, NULL};
const Segment * const SG76635[] = {SGI327833, NULL};
const Segment * const SG76636[] = {SGI327834, NULL};
const Segment * const SG76637[] = {SGI327835, NULL};
const Segment * const SG76638[] = {SGI327836, NULL};
const Segment * const SG76639[] = {SGI327837, NULL};
const Segment * const SG76640[] = {SGI327838, SGI327839, SGI327840, NULL};
const Segment * const SG76641[] = {SGI327841, SGI327842, SGI327843, NULL};
const Segment * const SG76642[] = {SGI327844, SGI0, SGI327845, NULL};
const Segment * const SG76643[] = {SGI327846, SGI327847, SGI327848, NULL};
const Segment * const SG76644[] = {SGI327849, NULL};
const Segment * const SG76645[] = {SGI327850, SGI0, SGI327851, SGI327852, SGI327853, SGI327854, SGI0, SGI327855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76646[] = {SGI327858, SGI0, SGI327859, SGI327860, SGI327861, SGI327862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76647[] = {SGI327865, NULL};
const Segment * const SG76648[] = {SGI327866, NULL};
const Segment * const SG76649[] = {SGI327867, SGI0, SGI327868, SGI327869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76650[] = {SGI327870, SGI327871, SGI327872, NULL};
const Segment * const SG76651[] = {SGI327873, SGI0, SGI327874, SGI327875, SGI327876, SGI327877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76652[] = {SGI327879, NULL};
const Segment * const SG76653[] = {SGI327880, SGI0, SGI327881, SGI327882, SGI327883, SGI327884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327886, SGI0, SGI0, SGI0, SGI0, SGI327887, SGI0, SGI0, NULL};
const Segment * const SG76654[] = {SGI327888, NULL};
const Segment * const SG76655[] = {SGI327889, SGI327889, SGI0, SGI0, SGI327890, SGI0, NULL};
const Segment * const SG76656[] = {SGI327891, SGI327892, SGI327893, NULL};
const Segment * const SG76657[] = {SGI327894, SGI0, SGI327895, SGI327896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76658[] = {SGI327897, SGI0, SGI327898, SGI327899, SGI327900, SGI327901, SGI0, SGI327902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327903, NULL};
const Segment * const SG76659[] = {SGI327904, SGI0, SGI327905, SGI327906, SGI327907, SGI327908, SGI0, SGI0, SGI327909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76660[] = {SGI327910, SGI0, SGI327911, SGI327912, SGI327913, SGI327914, SGI0, SGI0, SGI327915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76661[] = {SGI327916, NULL};
const Segment * const SG76662[] = {SGI327917, SGI0, SGI0, NULL};
const Segment * const SG76663[] = {SGI327918, NULL};
const Segment * const SG76664[] = {SGI327919, NULL};
const Segment * const SG76665[] = {SGI327920, NULL};
const Segment * const SG76666[] = {SGI327921, NULL};
const Segment * const SG76667[] = {SGI327922, NULL};
const Segment * const SG76668[] = {SGI327923, NULL};
const Segment * const SG76669[] = {SGI327924, NULL};
const Segment * const SG76670[] = {SGI327925, NULL};
const Segment * const SG76671[] = {SGI327926, NULL};
const Segment * const SG76672[] = {SGI327927, NULL};
const Segment * const SG76673[] = {SGI327928, NULL};
const Segment * const SG76674[] = {SGI327929, SGI0, SGI327930, SGI327931, SGI327932, SGI327933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76675[] = {SGI327935, NULL};
const Segment * const SG76676[] = {SGI327936, NULL};
const Segment * const SG76677[] = {SGI327937, NULL};
const Segment * const SG76678[] = {SGI327938, NULL};
const Segment * const SG76679[] = {SGI327939, NULL};
const Segment * const SG76680[] = {SGI327940, NULL};
const Segment * const SG76681[] = {SGI327941, SGI0, SGI0, NULL};
const Segment * const SG76682[] = {SGI327942, SGI0, SGI0, NULL};
const Segment * const SG76683[] = {SGI327943, NULL};
const Segment * const SG76684[] = {SGI327944, NULL};
const Segment * const SG76685[] = {SGI327945, SGI0, SGI0, NULL};
const Segment * const SG76686[] = {SGI327946, SGI0, SGI327947, SGI327948, SGI327949, SGI327950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327951, SGI327952, SGI0, NULL};
const Segment * const SG76687[] = {SGI327953, SGI0, SGI327954, SGI327955, SGI327956, SGI327957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327958, SGI0, NULL};
const Segment * const SG76688[] = {SGI327959, NULL};
const Segment * const SG76689[] = {SGI327960, SGI0, SGI327961, SGI327962, SGI327963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76690[] = {SGI327965, SGI0, SGI327966, SGI327967, SGI327968, SGI327969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327971, SGI0, NULL};
const Segment * const SG76691[] = {SGI327972, SGI0, SGI327973, SGI327974, SGI327975, SGI327976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI327977, SGI0, NULL};
const Segment * const SG76692[] = {SGI327978, NULL};
const Segment * const SG76693[] = {SGI327979, NULL};
const Segment * const SG76694[] = {SGI327980, NULL};
const Segment * const SG76695[] = {SGI327981, NULL};
const Segment * const SG76696[] = {SGI327982, NULL};
const Segment * const SG76697[] = {SGI327983, NULL};
const Segment * const SG76698[] = {SGI327984, NULL};
const Segment * const SG76699[] = {SGI327985, SGI327985, SGI0, SGI0, SGI327986, SGI0, NULL};
const Segment * const SG76700[] = {SGI327987, SGI0, SGI0, NULL};
const Segment * const SG76701[] = {SGI327988, SGI0, SGI0, NULL};
const Segment * const SG76702[] = {SGI327989, NULL};
const Segment * const SG76703[] = {SGI327990, NULL};
const Segment * const SG76704[] = {SGI327991, SGI0, SGI327992, SGI327993, SGI327994, SGI327995, SGI0, SGI327996, SGI327997, SGI0, SGI0, SGI0, SGI0, SGI327998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76705[] = {SGI327999, NULL};
const Segment * const SG76706[] = {SGI328000, NULL};
const Segment * const SG76707[] = {SGI328001, NULL};
const Segment * const SG76708[] = {SGI328002, NULL};
const Segment * const SG76709[] = {SGI328003, NULL};
const Segment * const SG76710[] = {SGI328004, SGI0, SGI328005, SGI328006, SGI328007, SGI328008, SGI328009, SGI328010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76711[] = {SGI328012, SGI0, SGI328013, SGI328014, SGI328015, SGI328016, SGI328017, SGI328018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76712[] = {SGI328020, SGI0, SGI328021, SGI328022, SGI328023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328024, SGI0, NULL};
const Segment * const SG76713[] = {SGI328025, SGI0, SGI0, NULL};
const Segment * const SG76714[] = {SGI328026, SGI0, SGI328027, SGI328028, SGI328029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76715[] = {SGI328031, SGI0, SGI328032, SGI328033, SGI328034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76716[] = {SGI328036, SGI0, SGI328037, SGI328038, SGI328039, SGI328040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328041, SGI328042, SGI0, NULL};
const Segment * const SG76717[] = {SGI328043, SGI0, SGI328044, SGI328045, SGI328046, SGI328047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328048, SGI0, NULL};
const Segment * const SG76718[] = {SGI328049, SGI0, SGI328050, SGI328051, SGI328052, SGI328053, SGI328054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328055, SGI328056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76719[] = {SGI328058, NULL};
const Segment * const SG76720[] = {SGI328059, NULL};
const Segment * const SG76721[] = {SGI328060, NULL};
const Segment * const SG76722[] = {SGI328061, NULL};
const Segment * const SG76723[] = {SGI328062, SGI0, SGI328063, SGI328064, SGI328065, SGI328066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76724[] = {SGI328068, SGI0, SGI328069, SGI328070, SGI328071, SGI328072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328073, SGI0, SGI328074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76725[] = {SGI328075, NULL};
const Segment * const SG76726[] = {SGI328076, SGI328076, SGI0, SGI0, SGI328077, SGI0, NULL};
const Segment * const SG76727[] = {SGI328078, SGI0, SGI0, NULL};
const Segment * const SG76728[] = {SGI328079, SGI0, SGI0, SGI0, SGI328080, SGI328081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76729[] = {SGI328082, NULL};
const Segment * const SG76730[] = {SGI328083, SGI328083, SGI0, SGI0, SGI328084, SGI0, NULL};
const Segment * const SG76731[] = {SGI328085, SGI0, SGI328086, SGI328087, SGI328088, SGI328089, SGI0, SGI0, SGI328090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328091, SGI0, SGI0, NULL};
const Segment * const SG76732[] = {SGI328092, SGI0, SGI328093, SGI328094, SGI328095, SGI328096, SGI0, SGI0, SGI328097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76733[] = {SGI328098, SGI328098, SGI0, SGI0, SGI328099, SGI0, NULL};
const Segment * const SG76734[] = {SGI328100, SGI0, SGI328101, SGI328102, SGI328103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328105, SGI0, NULL};
const Segment * const SG76735[] = {SGI328106, SGI0, SGI328107, SGI328108, SGI328109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328111, SGI0, NULL};
const Segment * const SG76736[] = {SGI328112, SGI0, SGI328113, SGI328114, SGI328115, SGI328116, SGI328117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76737[] = {SGI328120, SGI0, SGI328121, SGI328122, SGI328123, SGI328124, SGI328125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76738[] = {SGI328127, SGI0, SGI328128, SGI328129, SGI328130, SGI328131, SGI0, SGI328132, SGI328133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76739[] = {SGI328134, SGI0, SGI0, NULL};
const Segment * const SG76740[] = {SGI328135, SGI0, SGI0, NULL};
const Segment * const SG76741[] = {SGI328136, SGI0, SGI328137, SGI328138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328139, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76742[] = {SGI328140, SGI0, SGI0, NULL};
const Segment * const SG76743[] = {SGI328141, SGI0, SGI0, NULL};
const Segment * const SG76744[] = {SGI328142, SGI0, SGI0, NULL};
const Segment * const SG76745[] = {SGI328143, SGI0, SGI0, NULL};
const Segment * const SG76746[] = {SGI328144, SGI328145, SGI328146, NULL};
const Segment * const SG76747[] = {SGI328147, SGI328148, SGI328149, NULL};
const Segment * const SG76748[] = {SGI328150, SGI0, SGI328151, SGI328152, SGI328153, SGI328154, SGI328155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76749[] = {SGI328157, SGI0, SGI328158, SGI328159, SGI328160, SGI328161, SGI328162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76750[] = {SGI328165, SGI328165, SGI0, SGI0, SGI328166, SGI0, NULL};
const Segment * const SG76751[] = {SGI328167, SGI0, SGI328168, NULL};
const Segment * const SG76752[] = {SGI328169, SGI0, SGI328170, NULL};
const Segment * const SG76753[] = {SGI328171, SGI0, SGI0, NULL};
const Segment * const SG76754[] = {SGI328172, SGI0, SGI0, NULL};
const Segment * const SG76755[] = {SGI328173, SGI0, SGI328174, SGI328175, SGI328176, SGI0, SGI0, SGI0, SGI328177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76756[] = {SGI328178, SGI328178, SGI0, SGI0, SGI328179, SGI0, NULL};
const Segment * const SG76757[] = {SGI328180, SGI328180, SGI0, SGI0, SGI328181, SGI0, NULL};
const Segment * const SG76758[] = {SGI328182, SGI328182, SGI0, SGI0, SGI328183, SGI0, NULL};
const Segment * const SG76759[] = {SGI328184, SGI0, SGI328185, SGI328186, SGI328187, SGI328188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76760[] = {SGI328191, SGI328191, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76761[] = {SGI328192, SGI328193, SGI328194, NULL};
const Segment * const SG76762[] = {SGI328195, SGI328196, SGI328197, NULL};
const Segment * const SG76763[] = {SGI328198, SGI328198, SGI0, SGI0, SGI328199, SGI0, NULL};
const Segment * const SG76764[] = {SGI328200, SGI328200, SGI0, SGI0, SGI328201, SGI0, NULL};
const Segment * const SG76765[] = {SGI328202, SGI328202, SGI0, SGI0, SGI328203, SGI0, NULL};
const Segment * const SG76766[] = {SGI328204, SGI0, SGI328205, SGI328206, SGI328207, SGI328208, SGI0, SGI328209, SGI328210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76767[] = {SGI328211, SGI0, SGI328212, SGI328213, SGI328214, SGI328215, SGI0, SGI0, SGI328216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76768[] = {SGI328217, SGI328217, SGI0, SGI0, SGI328218, SGI0, NULL};
const Segment * const SG76769[] = {SGI328219, SGI328219, SGI0, SGI0, SGI328220, SGI0, NULL};
const Segment * const SG76770[] = {SGI328221, SGI0, SGI328222, SGI328223, SGI328224, SGI328225, SGI0, SGI0, SGI328226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76771[] = {SGI328227, SGI328228, SGI328229, NULL};
const Segment * const SG76772[] = {SGI328230, SGI0, SGI328231, NULL};
const Segment * const SG76773[] = {SGI328232, SGI0, SGI328233, NULL};
const Segment * const SG76774[] = {SGI328234, SGI0, SGI328235, NULL};
const Segment * const SG76775[] = {SGI328236, SGI0, SGI328237, NULL};
const Segment * const SG76776[] = {SGI328238, SGI0, SGI328239, NULL};
const Segment * const SG76777[] = {SGI328240, SGI0, SGI328241, NULL};
const Segment * const SG76778[] = {SGI328242, SGI328242, SGI0, SGI0, SGI328243, SGI0, NULL};
const Segment * const SG76779[] = {SGI328244, SGI328244, SGI0, SGI0, SGI328245, SGI0, NULL};
const Segment * const SG76780[] = {SGI328246, SGI328246, SGI0, SGI0, SGI328247, SGI0, NULL};
const Segment * const SG76781[] = {SGI328248, SGI328248, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76782[] = {SGI328249, SGI0, SGI328250, SGI328251, SGI328252, SGI328253, SGI328254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76783[] = {SGI328257, SGI0, SGI328258, SGI328259, SGI328260, SGI328261, SGI328262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76784[] = {SGI328264, SGI0, SGI328265, SGI328266, SGI328267, SGI328268, SGI328269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76785[] = {SGI328272, SGI0, SGI328273, SGI328274, SGI328275, SGI328276, SGI328277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76786[] = {SGI328279, SGI0, SGI328280, SGI328281, SGI328282, SGI328283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76787[] = {SGI328285, SGI0, SGI328286, SGI328287, SGI328288, SGI328289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76788[] = {SGI328291, SGI0, SGI328292, SGI328293, SGI328294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76789[] = {SGI328295, SGI328295, SGI0, SGI0, SGI328296, SGI0, NULL};
const Segment * const SG76790[] = {SGI328297, SGI0, SGI0, NULL};
const Segment * const SG76791[] = {SGI328298, SGI0, SGI0, NULL};
const Segment * const SG76792[] = {SGI328299, SGI0, SGI0, NULL};
const Segment * const SG76793[] = {SGI328300, SGI0, SGI0, NULL};
const Segment * const SG76794[] = {SGI328301, SGI0, SGI328302, SGI328303, SGI328304, SGI328305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328306, SGI328307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76795[] = {SGI328308, SGI0, SGI328309, SGI328310, SGI328311, SGI328312, SGI0, SGI328313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328314, SGI0, NULL};
const Segment * const SG76796[] = {SGI328315, SGI0, SGI328316, SGI328317, SGI328318, SGI328319, SGI0, SGI328320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328322, SGI0, NULL};
const Segment * const SG76797[] = {SGI328323, SGI0, SGI328324, NULL};
const Segment * const SG76798[] = {SGI328325, SGI0, SGI328326, NULL};
const Segment * const SG76799[] = {SGI328327, SGI0, SGI328328, NULL};
const Segment * const SG76800[] = {SGI328329, SGI0, SGI328330, NULL};
const Segment * const SG76801[] = {SGI328331, SGI0, SGI328332, NULL};
const Segment * const SG76802[] = {SGI328333, SGI0, SGI328334, NULL};
const Segment * const SG76803[] = {SGI328335, SGI328335, SGI0, SGI0, SGI328336, SGI0, NULL};
const Segment * const SG76804[] = {SGI328337, SGI328337, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76805[] = {SGI328338, SGI0, SGI0, NULL};
const Segment * const SG76806[] = {SGI328339, SGI328339, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76807[] = {SGI328340, SGI328340, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76808[] = {SGI328341, SGI0, SGI328342, SGI328343, SGI328344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76809[] = {SGI328345, SGI0, SGI328346, SGI328347, SGI328348, SGI328349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328350, SGI328351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328352, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76810[] = {SGI328353, SGI0, SGI328354, SGI328355, SGI328356, SGI328357, SGI328358, SGI328359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76811[] = {SGI328361, SGI328361, SGI0, SGI0, SGI328362, SGI0, NULL};
const Segment * const SG76812[] = {SGI328363, SGI328363, SGI0, SGI0, SGI328364, SGI0, NULL};
const Segment * const SG76813[] = {SGI328365, SGI328365, SGI0, SGI0, SGI328366, SGI0, NULL};
const Segment * const SG76814[] = {SGI328367, SGI328367, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76815[] = {SGI328368, SGI328369, SGI328370, NULL};
const Segment * const SG76816[] = {SGI328371, SGI328371, SGI0, SGI0, SGI328372, SGI0, NULL};
const Segment * const SG76817[] = {SGI328373, SGI0, SGI0, NULL};
const Segment * const SG76818[] = {SGI328374, SGI0, SGI0, NULL};
const Segment * const SG76819[] = {SGI328375, SGI0, SGI0, NULL};
const Segment * const SG76820[] = {SGI328376, SGI0, SGI328377, NULL};
const Segment * const SG76821[] = {SGI328378, SGI0, SGI0, NULL};
const Segment * const SG76822[] = {SGI328379, SGI0, SGI0, NULL};
const Segment * const SG76823[] = {SGI328380, SGI328381, SGI328382, NULL};
const Segment * const SG76824[] = {SGI328383, SGI0, SGI0, NULL};
const Segment * const SG76825[] = {SGI328384, SGI328385, SGI328386, NULL};
const Segment * const SG76826[] = {SGI328387, SGI0, SGI328388, SGI328389, SGI328390, SGI328391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328392, SGI0, NULL};
const Segment * const SG76827[] = {SGI328393, SGI328394, SGI328395, NULL};
const Segment * const SG76828[] = {SGI328396, SGI0, SGI0, NULL};
const Segment * const SG76829[] = {SGI328397, SGI0, SGI0, NULL};
const Segment * const SG76830[] = {SGI328398, SGI0, SGI0, NULL};
const Segment * const SG76831[] = {SGI328399, SGI0, SGI0, NULL};
const Segment * const SG76832[] = {SGI328400, SGI0, SGI0, NULL};
const Segment * const SG76833[] = {SGI328401, SGI0, SGI0, NULL};
const Segment * const SG76834[] = {SGI328402, SGI328403, SGI0, NULL};
const Segment * const SG76835[] = {SGI328404, SGI0, SGI328405, SGI328406, SGI328407, SGI328408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328409, SGI0, NULL};
const Segment * const SG76836[] = {SGI328410, SGI328410, SGI0, SGI0, SGI328411, SGI0, NULL};
const Segment * const SG76837[] = {SGI328412, SGI328412, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76838[] = {SGI328413, SGI328413, SGI0, SGI0, SGI328414, SGI0, NULL};
const Segment * const SG76839[] = {SGI328415, SGI328415, SGI0, SGI0, SGI328416, SGI0, NULL};
const Segment * const SG76840[] = {SGI328417, SGI328418, SGI328419, NULL};
const Segment * const SG76841[] = {SGI328420, SGI0, SGI328421, NULL};
const Segment * const SG76842[] = {SGI328422, SGI0, SGI328423, NULL};
const Segment * const SG76843[] = {SGI328424, SGI0, SGI328425, NULL};
const Segment * const SG76844[] = {SGI328426, SGI0, SGI0, NULL};
const Segment * const SG76845[] = {SGI328427, SGI0, SGI0, NULL};
const Segment * const SG76846[] = {SGI328428, SGI0, SGI0, NULL};
const Segment * const SG76847[] = {SGI328429, SGI0, SGI328430, NULL};
const Segment * const SG76848[] = {SGI328431, SGI0, SGI328432, NULL};
const Segment * const SG76849[] = {SGI328433, SGI0, SGI0, NULL};
const Segment * const SG76850[] = {SGI328434, SGI328435, SGI328436, NULL};
const Segment * const SG76851[] = {SGI328437, SGI0, SGI328438, SGI328439, SGI328440, SGI328441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76852[] = {SGI328443, SGI328444, SGI328445, NULL};
const Segment * const SG76853[] = {SGI328446, SGI328447, SGI328448, NULL};
const Segment * const SG76854[] = {SGI328449, SGI0, SGI328450, SGI328451, SGI328452, SGI328453, SGI0, SGI0, SGI328454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328455, SGI0, SGI0, NULL};
const Segment * const SG76855[] = {SGI328456, SGI0, SGI0, NULL};
const Segment * const SG76856[] = {SGI328457, SGI0, SGI328458, SGI328459, SGI328460, SGI328461, SGI0, SGI328462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328463, SGI0, NULL};
const Segment * const SG76857[] = {SGI328464, SGI0, SGI328465, SGI328466, SGI328467, SGI328468, SGI328469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76858[] = {SGI328472, SGI0, SGI328473, SGI328474, SGI328475, SGI328476, SGI328477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76859[] = {SGI328479, SGI0, SGI0, NULL};
const Segment * const SG76860[] = {SGI328480, SGI328481, SGI328482, NULL};
const Segment * const SG76861[] = {SGI328483, SGI0, SGI0, NULL};
const Segment * const SG76862[] = {SGI328484, SGI0, SGI0, NULL};
const Segment * const SG76863[] = {SGI328485, SGI0, SGI328486, NULL};
const Segment * const SG76864[] = {SGI328487, SGI0, SGI0, NULL};
const Segment * const SG76865[] = {SGI328488, SGI0, SGI0, NULL};
const Segment * const SG76866[] = {SGI328489, SGI0, SGI0, NULL};
const Segment * const SG76867[] = {SGI328490, SGI328491, SGI328492, NULL};
const Segment * const SG76868[] = {SGI328493, SGI328494, SGI328495, NULL};
const Segment * const SG76869[] = {SGI328496, SGI0, SGI0, NULL};
const Segment * const SG76870[] = {SGI328497, SGI328498, SGI328499, NULL};
const Segment * const SG76871[] = {SGI328500, SGI0, SGI328501, SGI328502, SGI328503, SGI328504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328505, SGI328506, SGI0, NULL};
const Segment * const SG76872[] = {SGI328507, SGI0, SGI328508, SGI328509, SGI328510, SGI328511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328512, SGI0, NULL};
const Segment * const SG76873[] = {SGI328513, SGI328514, SGI328515, NULL};
const Segment * const SG76874[] = {SGI328516, SGI328517, SGI328518, NULL};
const Segment * const SG76875[] = {SGI328519, SGI0, SGI328520, SGI328521, SGI328522, SGI328523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76876[] = {SGI328524, SGI0, SGI328525, NULL};
const Segment * const SG76877[] = {SGI328526, SGI0, SGI328527, NULL};
const Segment * const SG76878[] = {SGI328528, SGI328529, SGI328530, NULL};
const Segment * const SG76879[] = {SGI328531, SGI328532, SGI0, NULL};
const Segment * const SG76880[] = {SGI328533, SGI328534, SGI328535, NULL};
const Segment * const SG76881[] = {SGI328536, SGI0, SGI0, NULL};
const Segment * const SG76882[] = {SGI328537, SGI0, SGI0, NULL};
const Segment * const SG76883[] = {SGI328538, SGI0, SGI328539, NULL};
const Segment * const SG76884[] = {SGI328540, SGI0, SGI0, NULL};
const Segment * const SG76885[] = {SGI328541, SGI0, SGI0, NULL};
const Segment * const SG76886[] = {SGI328542, SGI0, SGI0, NULL};
const Segment * const SG76887[] = {SGI328543, SGI0, SGI0, NULL};
const Segment * const SG76888[] = {SGI328544, SGI328545, SGI328546, NULL};
const Segment * const SG76889[] = {SGI328547, SGI0, SGI0, NULL};
const Segment * const SG76890[] = {SGI328548, SGI0, SGI0, NULL};
const Segment * const SG76891[] = {SGI328549, SGI0, SGI328550, NULL};
const Segment * const SG76892[] = {SGI328551, SGI0, SGI0, NULL};
const Segment * const SG76893[] = {SGI328552, SGI0, SGI0, NULL};
const Segment * const SG76894[] = {SGI328553, SGI328554, SGI328555, NULL};
const Segment * const SG76895[] = {SGI328556, SGI0, SGI0, NULL};
const Segment * const SG76896[] = {SGI328557, SGI0, SGI328558, SGI328559, SGI328560, SGI328561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328563, SGI0, NULL};
const Segment * const SG76897[] = {SGI328564, SGI0, SGI0, NULL};
const Segment * const SG76898[] = {SGI328565, SGI0, SGI0, NULL};
const Segment * const SG76899[] = {SGI328566, SGI328567, SGI328568, NULL};
const Segment * const SG76900[] = {SGI328569, SGI0, SGI0, NULL};
const Segment * const SG76901[] = {SGI328570, SGI328571, SGI328572, NULL};
const Segment * const SG76902[] = {SGI328573, SGI0, SGI0, NULL};
const Segment * const SG76903[] = {SGI328574, SGI328575, SGI328576, NULL};
const Segment * const SG76904[] = {SGI328577, SGI328578, SGI328579, NULL};
const Segment * const SG76905[] = {SGI328580, SGI0, SGI328581, NULL};
const Segment * const SG76906[] = {SGI328582, SGI0, SGI328583, NULL};
const Segment * const SG76907[] = {SGI328584, SGI0, SGI0, NULL};
const Segment * const SG76908[] = {SGI328585, SGI0, SGI328586, NULL};
const Segment * const SG76909[] = {SGI328587, SGI0, SGI0, NULL};
const Segment * const SG76910[] = {SGI328588, SGI0, SGI328589, NULL};
const Segment * const SG76911[] = {SGI328590, SGI0, SGI328591, NULL};
const Segment * const SG76912[] = {SGI328592, SGI0, SGI0, NULL};
const Segment * const SG76913[] = {SGI328593, SGI0, SGI0, NULL};
const Segment * const SG76914[] = {SGI328594, SGI0, SGI0, NULL};
const Segment * const SG76915[] = {SGI328595, SGI0, SGI328596, SGI328597, SGI328598, SGI328599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328600, SGI328601, SGI0, NULL};
const Segment * const SG76916[] = {SGI328602, SGI0, SGI0, NULL};
const Segment * const SG76917[] = {SGI328603, SGI0, SGI328604, NULL};
const Segment * const SG76918[] = {SGI328605, SGI0, SGI328606, SGI328607, SGI328608, SGI328609, SGI0, SGI0, SGI328610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76919[] = {SGI328611, SGI0, SGI328612, SGI328613, SGI328614, SGI328615, SGI0, SGI0, SGI328616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76920[] = {SGI328617, SGI0, SGI328618, SGI328619, SGI328620, SGI328621, SGI0, SGI0, SGI328622, SGI0, SGI0, SGI0, SGI0, SGI328623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76921[] = {SGI328624, SGI0, SGI328625, SGI328626, SGI328627, SGI328628, SGI328629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76922[] = {SGI328632, SGI0, SGI328633, SGI328634, SGI328635, SGI328636, SGI328637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76923[] = {SGI328639, SGI0, SGI328640, SGI328641, SGI328642, SGI328643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328644, SGI328645, SGI0, NULL};
const Segment * const SG76924[] = {SGI328646, SGI0, SGI328647, SGI328648, SGI328649, SGI328650, SGI328651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76925[] = {SGI328654, SGI0, SGI328655, SGI328656, SGI328657, SGI328658, SGI328659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76926[] = {SGI328661, SGI0, SGI328662, SGI328663, SGI328664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76927[] = {SGI328665, SGI0, SGI328666, SGI328667, SGI328668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76928[] = {SGI328669, SGI0, SGI328670, SGI328671, SGI328672, SGI328673, SGI328674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76929[] = {SGI328677, SGI0, SGI328678, SGI328679, SGI328680, SGI328681, SGI328682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76930[] = {SGI328684, SGI0, SGI328685, SGI328686, SGI328687, SGI328688, SGI328689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76931[] = {SGI328692, SGI0, SGI328693, SGI328694, SGI328695, SGI328696, SGI328697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76932[] = {SGI328699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76933[] = {SGI328700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76934[] = {SGI328701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76935[] = {SGI328702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76936[] = {SGI328703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76937[] = {SGI328704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76938[] = {SGI328705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76939[] = {SGI328706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76940[] = {SGI328707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76941[] = {SGI328708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76942[] = {SGI328709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76943[] = {SGI328710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76944[] = {SGI328711, SGI328712, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76945[] = {SGI0, SGI328713, SGI328714, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76946[] = {SGI328715, SGI328716, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76947[] = {SGI0, SGI328717, SGI328718, SGI0, SGI0, SGI0, SGI328719, NULL};
const Segment * const SG76948[] = {SGI0, SGI328720, SGI328721, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76949[] = {SGI0, SGI328722, SGI328723, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76950[] = {SGI0, SGI328724, SGI328725, SGI0, SGI0, SGI0, SGI328726, NULL};
const Segment * const SG76951[] = {SGI0, SGI328727, SGI328728, SGI0, SGI0, SGI0, SGI328729, NULL};
const Segment * const SG76952[] = {SGI0, SGI328730, SGI328731, SGI0, SGI0, SGI0, SGI328732, NULL};
const Segment * const SG76953[] = {SGI328733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328734, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76954[] = {SGI328735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76955[] = {SGI328736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76956[] = {SGI328737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76957[] = {SGI328738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76958[] = {SGI328739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76959[] = {SGI328740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76960[] = {SGI328741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76961[] = {SGI328742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76962[] = {SGI0, SGI328743, SGI328744, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76963[] = {SGI0, SGI328745, SGI328746, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76964[] = {SGI0, SGI328747, SGI328748, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76965[] = {SGI0, SGI328749, SGI328750, SGI0, SGI0, SGI0, SGI328751, NULL};
const Segment * const SG76966[] = {SGI0, SGI328752, SGI328753, SGI0, SGI0, SGI0, SGI328754, NULL};
const Segment * const SG76967[] = {SGI0, SGI328755, SGI328756, SGI0, SGI0, SGI0, SGI328757, NULL};
const Segment * const SG76968[] = {SGI0, SGI328758, SGI328759, SGI0, SGI0, SGI0, SGI328760, NULL};
const Segment * const SG76969[] = {SGI0, SGI328761, SGI328762, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76970[] = {SGI0, SGI328763, SGI328764, SGI0, SGI0, SGI0, SGI328765, NULL};
const Segment * const SG76971[] = {SGI0, SGI328766, SGI328767, SGI0, SGI0, SGI0, SGI328768, NULL};
const Segment * const SG76972[] = {SGI0, SGI328769, SGI328770, SGI0, SGI0, SGI0, SGI328771, NULL};
const Segment * const SG76973[] = {SGI0, SGI328772, SGI328773, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76974[] = {SGI0, SGI328774, SGI328775, SGI0, SGI0, SGI0, SGI328776, NULL};
const Segment * const SG76975[] = {SGI328777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328778, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76976[] = {SGI328779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76977[] = {SGI0, SGI328780, SGI328781, SGI0, SGI0, SGI0, SGI328782, NULL};
const Segment * const SG76978[] = {SGI0, SGI328783, SGI328784, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG76979[] = {SGI0, SGI328785, SGI328786, SGI0, SGI0, SGI0, SGI328787, NULL};
const Segment * const SG76980[] = {SGI328788, SGI328789, SGI0, SGI0, NULL};
const Segment * const SG76981[] = {SGI328790, SGI328791, SGI328792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328792, SGI0, NULL};
const Segment * const SG76982[] = {SGI328793, SGI328794, SGI328795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328796, SGI0, NULL};
const Segment * const SG76983[] = {SGI328797, SGI328798, SGI328799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328800, SGI0, NULL};
const Segment * const SG76984[] = {SGI328801, SGI328802, SGI328803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328804, SGI0, NULL};
const Segment * const SG76985[] = {SGI328805, SGI328806, SGI328807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328808, SGI0, NULL};
const Segment * const SG76986[] = {SGI328809, SGI328810, SGI328811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328812, SGI0, NULL};
const Segment * const SG76987[] = {SGI328813, SGI328814, SGI328815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328816, SGI0, NULL};
const Segment * const SG76988[] = {SGI328817, SGI328818, SGI328819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328820, SGI0, NULL};
const Segment * const SG76989[] = {SGI328821, SGI328822, SGI328823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328824, SGI0, NULL};
const Segment * const SG76990[] = {SGI328825, SGI328826, SGI328827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328828, SGI0, NULL};
const Segment * const SG76991[] = {SGI328829, SGI328830, SGI328831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328832, SGI0, NULL};
const Segment * const SG76992[] = {SGI328833, SGI328834, SGI328835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328836, SGI0, NULL};
const Segment * const SG76993[] = {SGI328837, SGI328838, SGI328839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328840, SGI0, NULL};
const Segment * const SG76994[] = {SGI328841, SGI328842, SGI328843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328844, SGI0, NULL};
const Segment * const SG76995[] = {SGI328845, SGI328846, SGI328847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328848, SGI0, NULL};
const Segment * const SG76996[] = {SGI328849, SGI328850, SGI328851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328852, SGI0, NULL};
const Segment * const SG76997[] = {SGI328853, SGI328854, SGI328855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328856, SGI0, NULL};
const Segment * const SG76998[] = {SGI328857, SGI328858, SGI328859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328860, SGI0, NULL};
const Segment * const SG76999[] = {SGI328861, SGI328862, SGI328863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328864, SGI0, NULL};
const Segment * const SG77000[] = {SGI328865, SGI328866, SGI328867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328868, SGI0, NULL};
const Segment * const SG77001[] = {SGI328869, SGI328870, SGI328871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328872, SGI0, NULL};
const Segment * const SG77002[] = {SGI328873, SGI328874, SGI328875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328876, SGI0, NULL};
const Segment * const SG77003[] = {SGI328877, SGI328878, SGI328879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328880, SGI0, NULL};
const Segment * const SG77004[] = {SGI328881, SGI328882, SGI328883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328884, SGI0, NULL};
const Segment * const SG77005[] = {SGI328885, SGI328886, SGI328887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328888, SGI0, NULL};
const Segment * const SG77006[] = {SGI328889, SGI328890, SGI328891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328892, SGI0, NULL};
const Segment * const SG77007[] = {SGI328893, SGI328894, SGI328895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328896, SGI0, NULL};
const Segment * const SG77008[] = {SGI328897, SGI328898, SGI328899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328900, SGI0, NULL};
const Segment * const SG77009[] = {SGI328901, SGI328902, SGI328903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328904, SGI0, NULL};
const Segment * const SG77010[] = {SGI328905, SGI328906, SGI328907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328908, SGI0, NULL};
const Segment * const SG77011[] = {SGI328909, SGI328910, SGI328911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328912, SGI0, NULL};
const Segment * const SG77012[] = {SGI328913, SGI328914, SGI328915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328916, SGI0, NULL};
const Segment * const SG77013[] = {SGI328917, SGI328918, SGI328919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328920, SGI0, NULL};
const Segment * const SG77014[] = {SGI328921, SGI328922, SGI328923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328924, SGI0, NULL};
const Segment * const SG77015[] = {SGI328925, SGI328926, SGI328927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328928, SGI0, NULL};
const Segment * const SG77016[] = {SGI328929, SGI328930, SGI328931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328932, SGI0, NULL};
const Segment * const SG77017[] = {SGI328933, SGI328934, SGI328935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328936, SGI0, NULL};
const Segment * const SG77018[] = {SGI328937, SGI328938, SGI328939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328940, SGI0, NULL};
const Segment * const SG77019[] = {SGI328941, SGI328942, SGI328943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328944, SGI0, NULL};
const Segment * const SG77020[] = {SGI328945, SGI328946, SGI328947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328948, SGI0, NULL};
const Segment * const SG77021[] = {SGI328949, SGI328950, SGI328951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328952, SGI0, NULL};
const Segment * const SG77022[] = {SGI328953, SGI328954, SGI328955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328956, SGI0, NULL};
const Segment * const SG77023[] = {SGI328957, SGI328958, SGI328959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328960, SGI0, NULL};
const Segment * const SG77024[] = {SGI328961, SGI328962, SGI328963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328964, SGI0, NULL};
const Segment * const SG77025[] = {SGI328965, SGI328966, SGI328967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328968, SGI0, NULL};
const Segment * const SG77026[] = {SGI328969, SGI328970, SGI328971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328972, SGI0, NULL};
const Segment * const SG77027[] = {SGI328973, SGI328974, SGI328975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328976, SGI0, NULL};
const Segment * const SG77028[] = {SGI328977, SGI328978, SGI328979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328980, SGI0, NULL};
const Segment * const SG77029[] = {SGI328981, SGI328982, SGI328983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328984, SGI0, NULL};
const Segment * const SG77030[] = {SGI328985, SGI328986, SGI328987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328987, SGI0, NULL};
const Segment * const SG77031[] = {SGI328988, SGI328989, SGI328990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328991, SGI0, NULL};
const Segment * const SG77032[] = {SGI328992, SGI328993, SGI328994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328995, SGI0, NULL};
const Segment * const SG77033[] = {SGI328996, SGI328997, SGI328998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI328999, SGI0, NULL};
const Segment * const SG77034[] = {SGI329000, SGI329001, SGI329002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329003, SGI0, NULL};
const Segment * const SG77035[] = {SGI329004, SGI329005, SGI329006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329007, SGI0, NULL};
const Segment * const SG77036[] = {SGI329008, SGI329009, SGI329010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329011, SGI0, NULL};
const Segment * const SG77037[] = {SGI329012, SGI329013, SGI329014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329015, SGI0, NULL};
const Segment * const SG77038[] = {SGI329016, SGI329017, SGI329018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329019, SGI0, NULL};
const Segment * const SG77039[] = {SGI329020, SGI329021, SGI329022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329023, SGI0, NULL};
const Segment * const SG77040[] = {SGI329024, SGI329025, SGI329026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329027, SGI0, NULL};
const Segment * const SG77041[] = {SGI329028, SGI329029, SGI329030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329031, SGI0, NULL};
const Segment * const SG77042[] = {SGI329032, SGI329033, SGI329034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329035, SGI0, NULL};
const Segment * const SG77043[] = {SGI329036, SGI329037, SGI329038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329039, SGI0, NULL};
const Segment * const SG77044[] = {SGI329040, SGI329041, SGI329042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329043, SGI0, NULL};
const Segment * const SG77045[] = {SGI329044, SGI329045, SGI329046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329047, SGI0, NULL};
const Segment * const SG77046[] = {SGI329048, SGI329049, SGI329050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329051, SGI0, NULL};
const Segment * const SG77047[] = {SGI329052, SGI329053, SGI329054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329055, SGI0, NULL};
const Segment * const SG77048[] = {SGI329056, SGI329057, SGI329058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329059, SGI0, NULL};
const Segment * const SG77049[] = {SGI329060, SGI329061, SGI329062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329063, SGI0, NULL};
const Segment * const SG77050[] = {SGI329064, SGI329065, SGI329066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329067, SGI0, NULL};
const Segment * const SG77051[] = {SGI329068, SGI329069, SGI329070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329071, SGI0, NULL};
const Segment * const SG77052[] = {SGI329072, SGI329073, SGI329074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329075, SGI0, NULL};
const Segment * const SG77053[] = {SGI329076, SGI329077, SGI329078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329079, SGI0, NULL};
const Segment * const SG77054[] = {SGI329080, SGI329081, SGI329082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329083, SGI0, NULL};
const Segment * const SG77055[] = {SGI329084, SGI329085, SGI329086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329087, SGI0, NULL};
const Segment * const SG77056[] = {SGI329088, SGI329089, SGI329090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329091, SGI0, NULL};
const Segment * const SG77057[] = {SGI329092, SGI329093, SGI329094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329095, SGI0, NULL};
const Segment * const SG77058[] = {SGI329096, SGI329097, SGI329098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329099, SGI0, NULL};
const Segment * const SG77059[] = {SGI329100, SGI329101, SGI329102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329103, SGI0, NULL};
const Segment * const SG77060[] = {SGI329104, SGI329105, SGI329106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329107, SGI0, NULL};
const Segment * const SG77061[] = {SGI329108, SGI329109, SGI329110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329111, SGI0, NULL};
const Segment * const SG77062[] = {SGI329112, SGI329113, SGI329114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329115, SGI0, NULL};
const Segment * const SG77063[] = {SGI329116, SGI329117, SGI329118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329119, SGI0, NULL};
const Segment * const SG77064[] = {SGI329120, SGI329121, SGI329122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329123, SGI0, NULL};
const Segment * const SG77065[] = {SGI329124, SGI329125, SGI329126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329127, SGI0, NULL};
const Segment * const SG77066[] = {SGI329128, SGI329129, SGI329130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329131, SGI0, NULL};
const Segment * const SG77067[] = {SGI329132, SGI329133, SGI329134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329135, SGI0, NULL};
const Segment * const SG77068[] = {SGI329136, SGI329137, SGI329138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329139, SGI0, NULL};
const Segment * const SG77069[] = {SGI329140, SGI329141, SGI329142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329143, SGI0, NULL};
const Segment * const SG77070[] = {SGI329144, SGI329145, SGI329146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329147, SGI0, NULL};
const Segment * const SG77071[] = {SGI329148, SGI329149, SGI329150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329151, SGI0, NULL};
const Segment * const SG77072[] = {SGI329152, SGI329153, SGI329154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329155, SGI0, NULL};
const Segment * const SG77073[] = {SGI329156, SGI329157, SGI329158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329159, SGI0, NULL};
const Segment * const SG77074[] = {SGI329160, SGI329161, SGI329162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329163, SGI0, NULL};
const Segment * const SG77075[] = {SGI329164, SGI329165, SGI329166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329167, SGI0, NULL};
const Segment * const SG77076[] = {SGI329168, SGI329169, SGI329170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329171, SGI0, NULL};
const Segment * const SG77077[] = {SGI329172, SGI329173, SGI329174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329175, SGI0, NULL};
const Segment * const SG77078[] = {SGI329176, SGI329177, SGI329178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329179, SGI0, NULL};
const Segment * const SG77079[] = {SGI329180, SGI329181, SGI329182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329183, SGI0, NULL};
const Segment * const SG77080[] = {SGI329184, SGI329185, SGI329186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329187, SGI0, NULL};
const Segment * const SG77081[] = {SGI329188, SGI329189, SGI329190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329191, SGI0, NULL};
const Segment * const SG77082[] = {SGI329192, SGI329193, SGI329194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329195, SGI0, NULL};
const Segment * const SG77083[] = {SGI329196, SGI329197, SGI329198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329199, SGI0, NULL};
const Segment * const SG77084[] = {SGI329200, SGI329201, SGI329202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329203, SGI0, NULL};
const Segment * const SG77085[] = {SGI329204, SGI329205, SGI329206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329207, SGI0, NULL};
const Segment * const SG77086[] = {SGI329208, SGI329209, SGI329210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329211, SGI0, NULL};
const Segment * const SG77087[] = {SGI329212, SGI329213, SGI329214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329215, SGI0, NULL};
const Segment * const SG77088[] = {SGI329216, SGI329217, SGI329218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329219, SGI0, NULL};
const Segment * const SG77089[] = {SGI329220, SGI329221, SGI329222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329223, SGI0, NULL};
const Segment * const SG77090[] = {SGI329224, SGI329225, SGI329226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329227, SGI0, NULL};
const Segment * const SG77091[] = {SGI329228, SGI329229, SGI329230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329231, SGI0, NULL};
const Segment * const SG77092[] = {SGI329232, SGI329233, SGI329234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329235, SGI0, NULL};
const Segment * const SG77093[] = {SGI329236, SGI329237, SGI329238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329239, SGI0, NULL};
const Segment * const SG77094[] = {SGI329240, SGI329241, SGI329242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329243, SGI0, NULL};
const Segment * const SG77095[] = {SGI329244, SGI329245, SGI329246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329247, SGI0, NULL};
const Segment * const SG77096[] = {SGI329248, SGI329249, SGI329250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329251, SGI0, NULL};
const Segment * const SG77097[] = {SGI329252, SGI329253, SGI329254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329255, SGI0, NULL};
const Segment * const SG77098[] = {SGI329256, SGI329257, SGI329258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329259, SGI0, NULL};
const Segment * const SG77099[] = {SGI329260, SGI329261, SGI329262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329263, SGI0, NULL};
const Segment * const SG77100[] = {SGI329264, SGI329265, SGI329266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329267, SGI0, NULL};
const Segment * const SG77101[] = {SGI329268, SGI329269, SGI329270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329271, SGI0, NULL};
const Segment * const SG77102[] = {SGI329272, SGI329273, SGI329274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329275, SGI0, NULL};
const Segment * const SG77103[] = {SGI329276, SGI329277, SGI329278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329279, SGI0, NULL};
const Segment * const SG77104[] = {SGI329280, SGI329281, SGI329282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329282, SGI0, NULL};
const Segment * const SG77105[] = {SGI329283, SGI329284, SGI329285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329286, SGI0, NULL};
const Segment * const SG77106[] = {SGI329287, SGI329288, SGI329289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329290, SGI0, NULL};
const Segment * const SG77107[] = {SGI329291, SGI329292, SGI329293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329294, SGI0, NULL};
const Segment * const SG77108[] = {SGI329295, SGI329296, SGI329297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329298, SGI0, NULL};
const Segment * const SG77109[] = {SGI329299, SGI329300, SGI329301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329302, SGI0, NULL};
const Segment * const SG77110[] = {SGI329303, SGI329304, SGI329305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329306, SGI0, NULL};
const Segment * const SG77111[] = {SGI329307, SGI329308, SGI329309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329310, SGI0, NULL};
const Segment * const SG77112[] = {SGI329311, SGI329312, SGI329313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329314, SGI0, NULL};
const Segment * const SG77113[] = {SGI329315, SGI329316, SGI329317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329318, SGI0, NULL};
const Segment * const SG77114[] = {SGI329319, SGI329320, SGI329321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329322, SGI0, NULL};
const Segment * const SG77115[] = {SGI329323, SGI329324, SGI329325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329326, SGI0, NULL};
const Segment * const SG77116[] = {SGI329327, SGI329328, SGI329329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329330, SGI0, NULL};
const Segment * const SG77117[] = {SGI329331, SGI329332, SGI329333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329334, SGI0, NULL};
const Segment * const SG77118[] = {SGI329335, SGI329336, SGI329337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329338, SGI0, NULL};
const Segment * const SG77119[] = {SGI329339, SGI329340, SGI329341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329342, SGI0, NULL};
const Segment * const SG77120[] = {SGI329343, SGI329344, SGI329345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329346, SGI0, NULL};
const Segment * const SG77121[] = {SGI329347, SGI329348, SGI329349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329350, SGI0, NULL};
const Segment * const SG77122[] = {SGI329351, SGI329352, SGI329353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329354, SGI0, NULL};
const Segment * const SG77123[] = {SGI329355, SGI329356, SGI329357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329358, SGI0, NULL};
const Segment * const SG77124[] = {SGI329359, SGI329360, SGI329361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329362, SGI0, NULL};
const Segment * const SG77125[] = {SGI329363, SGI329364, SGI329365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329366, SGI0, NULL};
const Segment * const SG77126[] = {SGI329367, SGI329368, SGI329369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329370, SGI0, NULL};
const Segment * const SG77127[] = {SGI329371, SGI329372, SGI329373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329374, SGI0, NULL};
const Segment * const SG77128[] = {SGI329375, SGI329376, SGI329377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329378, SGI0, NULL};
const Segment * const SG77129[] = {SGI329379, SGI329380, SGI329381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329382, SGI0, NULL};
const Segment * const SG77130[] = {SGI329383, SGI329384, SGI329385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329386, SGI0, NULL};
const Segment * const SG77131[] = {SGI329387, SGI329388, SGI329389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329390, SGI0, NULL};
const Segment * const SG77132[] = {SGI329391, SGI329392, SGI329393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329394, SGI0, NULL};
const Segment * const SG77133[] = {SGI329395, SGI329396, SGI329397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329398, SGI0, NULL};
const Segment * const SG77134[] = {SGI329399, SGI329400, SGI329401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329402, SGI0, NULL};
const Segment * const SG77135[] = {SGI329403, SGI329404, SGI329405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329406, SGI0, NULL};
const Segment * const SG77136[] = {SGI329407, SGI329408, SGI329409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329410, SGI0, NULL};
const Segment * const SG77137[] = {SGI329411, SGI329412, SGI329413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329414, SGI0, NULL};
const Segment * const SG77138[] = {SGI329415, SGI329416, SGI329417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329418, SGI0, NULL};
const Segment * const SG77139[] = {SGI329419, SGI329420, SGI329421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329422, SGI0, NULL};
const Segment * const SG77140[] = {SGI329423, SGI329424, SGI329425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329426, SGI0, NULL};
const Segment * const SG77141[] = {SGI329427, SGI329428, SGI329429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329430, SGI0, NULL};
const Segment * const SG77142[] = {SGI329431, SGI329432, SGI329433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329434, SGI0, NULL};
const Segment * const SG77143[] = {SGI329435, SGI329436, SGI329437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329438, SGI0, NULL};
const Segment * const SG77144[] = {SGI329439, SGI329440, SGI329441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329442, SGI0, NULL};
const Segment * const SG77145[] = {SGI329443, SGI329444, SGI329445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329446, SGI0, NULL};
const Segment * const SG77146[] = {SGI329447, SGI329448, SGI329449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329450, SGI0, NULL};
const Segment * const SG77147[] = {SGI329451, SGI329452, SGI329453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329454, SGI0, NULL};
const Segment * const SG77148[] = {SGI329455, SGI329456, SGI329457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329458, SGI0, NULL};
const Segment * const SG77149[] = {SGI329459, SGI329460, SGI329461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329462, SGI0, NULL};
const Segment * const SG77150[] = {SGI329463, SGI329464, SGI329465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329466, SGI0, NULL};
const Segment * const SG77151[] = {SGI329467, SGI329468, SGI329469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329470, SGI0, NULL};
const Segment * const SG77152[] = {SGI329471, SGI329472, SGI329473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329474, SGI0, NULL};
const Segment * const SG77153[] = {SGI329475, SGI329476, SGI329477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329478, SGI0, NULL};
const Segment * const SG77154[] = {SGI329479, SGI329480, SGI329481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329482, SGI0, NULL};
const Segment * const SG77155[] = {SGI329483, SGI329484, SGI329485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329486, SGI0, NULL};
const Segment * const SG77156[] = {SGI329487, SGI329488, SGI329489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329490, SGI0, NULL};
const Segment * const SG77157[] = {SGI329491, SGI329492, SGI329493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329494, SGI0, NULL};
const Segment * const SG77158[] = {SGI329495, SGI329496, SGI329497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329498, SGI0, NULL};
const Segment * const SG77159[] = {SGI329499, SGI329500, SGI329501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329502, SGI0, NULL};
const Segment * const SG77160[] = {SGI329503, SGI329504, SGI329505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329506, SGI0, NULL};
const Segment * const SG77161[] = {SGI329507, SGI329508, SGI329509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329510, SGI0, NULL};
const Segment * const SG77162[] = {SGI329511, SGI329512, SGI0, SGI0, NULL};
const Segment * const SG77163[] = {SGI329513, SGI329514, SGI329515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329515, SGI0, NULL};
const Segment * const SG77164[] = {SGI329516, SGI329517, SGI329518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329519, SGI0, NULL};
const Segment * const SG77165[] = {SGI329520, SGI329521, SGI329522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329523, SGI0, NULL};
const Segment * const SG77166[] = {SGI329524, SGI329525, SGI329526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329527, SGI0, NULL};
const Segment * const SG77167[] = {SGI329528, SGI329529, SGI329530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329531, SGI0, NULL};
const Segment * const SG77168[] = {SGI329532, SGI329533, SGI329534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329535, SGI0, NULL};
const Segment * const SG77169[] = {SGI329536, SGI329537, SGI329538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329539, SGI0, NULL};
const Segment * const SG77170[] = {SGI329540, SGI329541, SGI329542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329543, SGI0, NULL};
const Segment * const SG77171[] = {SGI329544, SGI329545, SGI329546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329547, SGI0, NULL};
const Segment * const SG77172[] = {SGI329548, SGI329549, SGI329550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329551, SGI0, NULL};
const Segment * const SG77173[] = {SGI329552, SGI329553, SGI329554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329555, SGI0, NULL};
const Segment * const SG77174[] = {SGI329556, SGI329557, SGI329558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329559, SGI0, NULL};
const Segment * const SG77175[] = {SGI329561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77176[] = {SGI329560, SGI329562, SGI329563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329564, SGI0, NULL};
const Segment * const SG77177[] = {SGI329565, SGI329566, SGI329567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329568, SGI0, NULL};
const Segment * const SG77178[] = {SGI329569, SGI329570, SGI329571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329572, SGI0, NULL};
const Segment * const SG77179[] = {SGI329573, SGI329574, SGI329575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329576, SGI0, NULL};
const Segment * const SG77180[] = {SGI329577, SGI329578, SGI329579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329580, SGI0, NULL};
const Segment * const SG77181[] = {SGI329581, SGI329582, SGI329583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329584, SGI0, NULL};
const Segment * const SG77182[] = {SGI329585, SGI329586, SGI329587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329588, SGI0, NULL};
const Segment * const SG77183[] = {SGI329589, SGI329590, SGI329591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329592, SGI0, NULL};
const Segment * const SG77184[] = {SGI329593, SGI329594, SGI329595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329596, SGI0, NULL};
const Segment * const SG77185[] = {SGI329597, SGI329598, SGI329599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329600, SGI0, NULL};
const Segment * const SG77186[] = {SGI329601, SGI329602, SGI329603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329604, SGI0, NULL};
const Segment * const SG77187[] = {SGI329605, SGI329606, SGI329607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329608, SGI0, NULL};
const Segment * const SG77188[] = {SGI329609, SGI329610, SGI329611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329612, SGI0, NULL};
const Segment * const SG77189[] = {SGI329613, SGI329614, SGI329615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329616, SGI0, NULL};
const Segment * const SG77190[] = {SGI329617, SGI329618, SGI329619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329620, SGI0, NULL};
const Segment * const SG77191[] = {SGI329621, SGI329622, SGI329623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329624, SGI0, NULL};
const Segment * const SG77192[] = {SGI329625, SGI329626, SGI329627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329628, SGI0, NULL};
const Segment * const SG77193[] = {SGI329629, SGI329630, SGI329631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329632, SGI0, NULL};
const Segment * const SG77194[] = {SGI329633, SGI329634, SGI329635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329636, SGI0, NULL};
const Segment * const SG77195[] = {SGI329637, SGI329638, SGI329639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329640, SGI0, NULL};
const Segment * const SG77196[] = {SGI329641, SGI329642, SGI329643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329644, SGI0, NULL};
const Segment * const SG77197[] = {SGI329645, SGI329646, SGI329647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329648, SGI0, NULL};
const Segment * const SG77198[] = {SGI329649, SGI329650, SGI329651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329652, SGI0, NULL};
const Segment * const SG77199[] = {SGI329653, SGI329654, SGI329655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329656, SGI0, NULL};
const Segment * const SG77200[] = {SGI329657, SGI329658, SGI329659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329660, SGI0, NULL};
const Segment * const SG77201[] = {SGI329661, SGI329662, SGI329663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329664, SGI0, NULL};
const Segment * const SG77202[] = {SGI329665, SGI329666, SGI329667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329668, SGI0, NULL};
const Segment * const SG77203[] = {SGI329669, SGI329670, SGI329671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329672, SGI0, NULL};
const Segment * const SG77204[] = {SGI329673, SGI329674, SGI329675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329676, SGI0, NULL};
const Segment * const SG77205[] = {SGI329677, SGI329678, SGI329679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329680, SGI0, NULL};
const Segment * const SG77206[] = {SGI329681, SGI329682, SGI329683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329684, SGI0, NULL};
const Segment * const SG77207[] = {SGI329685, SGI329686, SGI329687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329688, SGI0, NULL};
const Segment * const SG77208[] = {SGI329689, SGI329690, SGI329691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329692, SGI0, NULL};
const Segment * const SG77209[] = {SGI329693, SGI329694, SGI329695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329696, SGI0, NULL};
const Segment * const SG77210[] = {SGI329697, SGI329698, SGI329699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329700, SGI0, NULL};
const Segment * const SG77211[] = {SGI329701, SGI329702, SGI329703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329704, SGI0, NULL};
const Segment * const SG77212[] = {SGI329705, SGI329706, SGI329707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329708, SGI0, NULL};
const Segment * const SG77213[] = {SGI329709, SGI329710, SGI329711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329712, SGI0, NULL};
const Segment * const SG77214[] = {SGI329713, SGI329714, SGI329715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329716, SGI0, NULL};
const Segment * const SG77215[] = {SGI329717, SGI329718, SGI329719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329720, SGI0, NULL};
const Segment * const SG77216[] = {SGI329721, SGI329722, SGI329723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329724, SGI0, NULL};
const Segment * const SG77217[] = {SGI329725, SGI329726, SGI329727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329728, SGI0, NULL};
const Segment * const SG77218[] = {SGI329729, SGI329730, SGI329731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329732, SGI0, NULL};
const Segment * const SG77219[] = {SGI329733, SGI329734, SGI329735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329736, SGI0, NULL};
const Segment * const SG77220[] = {SGI329737, SGI329738, SGI329739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329740, SGI0, NULL};
const Segment * const SG77221[] = {SGI329741, SGI329742, SGI329743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329744, SGI0, NULL};
const Segment * const SG77222[] = {SGI329745, SGI329746, SGI329747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329748, SGI0, NULL};
const Segment * const SG77223[] = {SGI329749, SGI329750, SGI329751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329752, SGI0, NULL};
const Segment * const SG77224[] = {SGI329753, SGI329754, SGI329755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329756, SGI0, NULL};
const Segment * const SG77225[] = {SGI329757, SGI329758, SGI329759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329760, SGI0, NULL};
const Segment * const SG77226[] = {SGI329761, SGI329762, SGI329763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329764, SGI0, NULL};
const Segment * const SG77227[] = {SGI329765, SGI329766, SGI329767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329768, SGI0, NULL};
const Segment * const SG77228[] = {SGI329769, SGI329770, SGI329771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329772, SGI0, NULL};
const Segment * const SG77229[] = {SGI329773, SGI329774, SGI329775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329776, SGI0, NULL};
const Segment * const SG77230[] = {SGI329777, SGI329778, SGI329779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329780, SGI0, NULL};
const Segment * const SG77231[] = {SGI329781, SGI329782, SGI329783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329784, SGI0, NULL};
const Segment * const SG77232[] = {SGI329785, SGI329786, SGI329787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329788, SGI0, NULL};
const Segment * const SG77233[] = {SGI329789, SGI329790, SGI329791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329792, SGI0, NULL};
const Segment * const SG77234[] = {SGI329793, SGI329794, SGI329795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329796, SGI0, NULL};
const Segment * const SG77235[] = {SGI329797, SGI329798, SGI329799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329800, SGI0, NULL};
const Segment * const SG77236[] = {SGI329801, SGI329802, SGI329803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329804, SGI0, NULL};
const Segment * const SG77237[] = {SGI329805, SGI329806, SGI329807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329808, SGI0, NULL};
const Segment * const SG77238[] = {SGI329809, SGI329810, SGI329811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329812, SGI0, NULL};
const Segment * const SG77239[] = {SGI329813, SGI329814, SGI329815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329816, SGI0, NULL};
const Segment * const SG77240[] = {SGI329817, SGI329818, SGI329819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329820, SGI0, NULL};
const Segment * const SG77241[] = {SGI329821, SGI329822, SGI329823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329824, SGI0, NULL};
const Segment * const SG77242[] = {SGI329825, SGI329826, SGI329827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329828, SGI0, NULL};
const Segment * const SG77243[] = {SGI329829, SGI329830, SGI329831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329832, SGI0, NULL};
const Segment * const SG77244[] = {SGI329833, SGI329834, SGI329835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329836, SGI0, NULL};
const Segment * const SG77245[] = {SGI329837, SGI329838, SGI329839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329840, SGI0, NULL};
const Segment * const SG77246[] = {SGI329841, SGI329842, SGI329843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329844, SGI0, NULL};
const Segment * const SG77247[] = {SGI329845, SGI329846, SGI329847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329848, SGI0, NULL};
const Segment * const SG77248[] = {SGI329849, SGI329850, SGI329851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329852, SGI0, NULL};
const Segment * const SG77249[] = {SGI329853, SGI329854, SGI329855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329856, SGI0, NULL};
const Segment * const SG77250[] = {SGI329857, SGI329858, SGI329859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329859, SGI0, NULL};
const Segment * const SG77251[] = {SGI329860, SGI329861, SGI329862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329863, SGI0, NULL};
const Segment * const SG77252[] = {SGI329864, SGI329865, SGI329866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329867, SGI0, NULL};
const Segment * const SG77253[] = {SGI329868, SGI329869, SGI329870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329871, SGI0, NULL};
const Segment * const SG77254[] = {SGI329872, SGI329873, SGI329874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329875, SGI0, NULL};
const Segment * const SG77255[] = {SGI329876, SGI329877, SGI329878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329879, SGI0, NULL};
const Segment * const SG77256[] = {SGI329880, SGI329881, SGI329882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329883, SGI0, NULL};
const Segment * const SG77257[] = {SGI329884, SGI329885, SGI329886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329887, SGI0, NULL};
const Segment * const SG77258[] = {SGI329888, SGI329889, SGI329890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329891, SGI0, NULL};
const Segment * const SG77259[] = {SGI329892, SGI329893, SGI329894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329895, SGI0, NULL};
const Segment * const SG77260[] = {SGI329896, SGI329897, SGI329898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329899, SGI0, NULL};
const Segment * const SG77261[] = {SGI329900, SGI329901, SGI329902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329903, SGI0, NULL};
const Segment * const SG77262[] = {SGI329904, SGI329905, SGI329906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329907, SGI0, NULL};
const Segment * const SG77263[] = {SGI329908, SGI329909, SGI329910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329911, SGI0, NULL};
const Segment * const SG77264[] = {SGI329912, SGI329913, SGI329914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329915, SGI0, NULL};
const Segment * const SG77265[] = {SGI329916, SGI329917, SGI329918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329919, SGI0, NULL};
const Segment * const SG77266[] = {SGI329920, SGI329921, SGI329922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329923, SGI0, NULL};
const Segment * const SG77267[] = {SGI329924, SGI329925, SGI329926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329927, SGI0, NULL};
const Segment * const SG77268[] = {SGI329928, SGI329929, SGI329930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329931, SGI0, NULL};
const Segment * const SG77269[] = {SGI329932, SGI329933, SGI329934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329935, SGI0, NULL};
const Segment * const SG77270[] = {SGI329936, SGI329937, SGI329938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329939, SGI0, NULL};
const Segment * const SG77271[] = {SGI329940, SGI329941, SGI329942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329943, SGI0, NULL};
const Segment * const SG77272[] = {SGI329944, SGI329945, SGI329946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329947, SGI0, NULL};
const Segment * const SG77273[] = {SGI329948, SGI329949, SGI329950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329951, SGI0, NULL};
const Segment * const SG77274[] = {SGI329952, SGI329953, SGI329954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329955, SGI0, NULL};
const Segment * const SG77275[] = {SGI329956, SGI329957, SGI329958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329959, SGI0, NULL};
const Segment * const SG77276[] = {SGI329960, SGI329961, SGI329962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329963, SGI0, NULL};
const Segment * const SG77277[] = {SGI329964, SGI329965, SGI329966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329967, SGI0, NULL};
const Segment * const SG77278[] = {SGI329968, SGI329969, SGI329970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329971, SGI0, NULL};
const Segment * const SG77279[] = {SGI329972, SGI329973, SGI329974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329975, SGI0, NULL};
const Segment * const SG77280[] = {SGI329976, SGI329977, SGI329978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329979, SGI0, NULL};
const Segment * const SG77281[] = {SGI329980, SGI329981, SGI329982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329983, SGI0, NULL};
const Segment * const SG77282[] = {SGI329984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77283[] = {SGI329985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77284[] = {SGI329986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77285[] = {SGI329987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77286[] = {SGI329988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77287[] = {SGI329989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77288[] = {SGI329990, SGI329991, SGI0, SGI0, NULL};
const Segment * const SG77289[] = {SGI329992, SGI329993, SGI0, SGI0, NULL};
const Segment * const SG77290[] = {SGI329994, SGI329995, SGI329996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI329996, SGI0, NULL};
const Segment * const SG77291[] = {SGI329997, SGI329998, SGI329999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330000, SGI0, NULL};
const Segment * const SG77292[] = {SGI330001, SGI330002, SGI330003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330004, SGI0, NULL};
const Segment * const SG77293[] = {SGI330005, SGI330006, SGI330007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330008, SGI0, NULL};
const Segment * const SG77294[] = {SGI330009, SGI330010, SGI330011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330012, SGI0, NULL};
const Segment * const SG77295[] = {SGI330013, SGI330014, SGI330015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330016, SGI0, NULL};
const Segment * const SG77296[] = {SGI330017, SGI330018, SGI330019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330020, SGI0, NULL};
const Segment * const SG77297[] = {SGI330021, SGI330022, SGI330023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330024, SGI0, NULL};
const Segment * const SG77298[] = {SGI330025, SGI330026, SGI330027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330028, SGI0, NULL};
const Segment * const SG77299[] = {SGI330029, SGI330030, SGI330031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330032, SGI0, NULL};
const Segment * const SG77300[] = {SGI330033, SGI330034, SGI330035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330036, SGI0, NULL};
const Segment * const SG77301[] = {SGI330037, SGI330038, SGI330039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330040, SGI0, NULL};
const Segment * const SG77302[] = {SGI330041, SGI330042, SGI330043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330044, SGI0, NULL};
const Segment * const SG77303[] = {SGI330045, SGI330046, SGI330047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330048, SGI0, NULL};
const Segment * const SG77304[] = {SGI330049, SGI330050, SGI330051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330052, SGI0, NULL};
const Segment * const SG77305[] = {SGI330053, SGI330054, SGI330055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330056, SGI0, NULL};
const Segment * const SG77306[] = {SGI330057, SGI330058, SGI330059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330060, SGI0, NULL};
const Segment * const SG77307[] = {SGI330061, SGI330062, SGI330063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330064, SGI0, NULL};
const Segment * const SG77308[] = {SGI330065, SGI330066, SGI330067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330068, SGI0, NULL};
const Segment * const SG77309[] = {SGI330069, SGI330070, SGI330071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330072, SGI0, NULL};
const Segment * const SG77310[] = {SGI330073, SGI330074, SGI330075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330076, SGI0, NULL};
const Segment * const SG77311[] = {SGI330077, SGI330078, SGI330079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330080, SGI0, NULL};
const Segment * const SG77312[] = {SGI330081, SGI330082, SGI330083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330084, SGI0, NULL};
const Segment * const SG77313[] = {SGI330085, SGI330086, SGI330087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330088, SGI0, NULL};
const Segment * const SG77314[] = {SGI330089, SGI330090, SGI330091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330092, SGI0, NULL};
const Segment * const SG77315[] = {SGI330093, SGI330094, SGI330095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330096, SGI0, NULL};
const Segment * const SG77316[] = {SGI330097, SGI330098, SGI330099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330100, SGI0, NULL};
const Segment * const SG77317[] = {SGI330101, SGI330102, SGI330103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330104, SGI0, NULL};
const Segment * const SG77318[] = {SGI330105, SGI330106, SGI330107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330108, SGI0, NULL};
const Segment * const SG77319[] = {SGI330109, SGI330110, SGI330111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330112, SGI0, NULL};
const Segment * const SG77320[] = {SGI330113, SGI330114, SGI330115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330116, SGI0, NULL};
const Segment * const SG77321[] = {SGI330117, SGI330118, SGI330119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330120, SGI0, NULL};
const Segment * const SG77322[] = {SGI330121, SGI330122, SGI330123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330124, SGI0, NULL};
const Segment * const SG77323[] = {SGI330125, SGI330126, SGI330127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330128, SGI0, NULL};
const Segment * const SG77324[] = {SGI330129, SGI330130, SGI330131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330132, SGI0, NULL};
const Segment * const SG77325[] = {SGI330133, SGI330134, SGI330135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330136, SGI0, NULL};
const Segment * const SG77326[] = {SGI330137, SGI330138, SGI330139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330140, SGI0, NULL};
const Segment * const SG77327[] = {SGI330141, SGI330142, SGI330143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330144, SGI0, NULL};
const Segment * const SG77328[] = {SGI330145, SGI330146, SGI330147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330148, SGI0, NULL};
const Segment * const SG77329[] = {SGI330149, SGI330150, SGI330151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330152, SGI0, NULL};
const Segment * const SG77330[] = {SGI330153, SGI330154, SGI330155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330156, SGI0, NULL};
const Segment * const SG77331[] = {SGI330157, SGI330158, SGI330159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330160, SGI0, NULL};
const Segment * const SG77332[] = {SGI330161, SGI330162, SGI330163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330164, SGI0, NULL};
const Segment * const SG77333[] = {SGI330165, SGI330166, SGI330167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330168, SGI0, NULL};
const Segment * const SG77334[] = {SGI330169, SGI330170, SGI330171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330172, SGI0, NULL};
const Segment * const SG77335[] = {SGI330173, SGI330174, SGI330175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330176, SGI0, NULL};
const Segment * const SG77336[] = {SGI330177, SGI330178, SGI330179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330180, SGI0, NULL};
const Segment * const SG77337[] = {SGI330181, SGI330182, SGI330183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330184, SGI0, NULL};
const Segment * const SG77338[] = {SGI330185, SGI330186, SGI330187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330188, SGI0, NULL};
const Segment * const SG77339[] = {SGI330189, SGI330190, SGI330191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330192, SGI0, NULL};
const Segment * const SG77340[] = {SGI330193, SGI330194, SGI330195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330196, SGI0, NULL};
const Segment * const SG77341[] = {SGI330197, SGI330198, SGI330199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330200, SGI0, NULL};
const Segment * const SG77342[] = {SGI330201, SGI330202, SGI330203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330204, SGI0, NULL};
const Segment * const SG77343[] = {SGI330205, SGI330206, SGI330207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330208, SGI0, NULL};
const Segment * const SG77344[] = {SGI330209, SGI330210, SGI330211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330212, SGI0, NULL};
const Segment * const SG77345[] = {SGI330213, SGI330214, SGI330215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330216, SGI0, NULL};
const Segment * const SG77346[] = {SGI330217, SGI330218, SGI330219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330220, SGI0, NULL};
const Segment * const SG77347[] = {SGI330221, SGI330222, SGI330223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330224, SGI0, NULL};
const Segment * const SG77348[] = {SGI330225, SGI330226, SGI330227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330228, SGI0, NULL};
const Segment * const SG77349[] = {SGI330229, SGI330230, SGI330231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330232, SGI0, NULL};
const Segment * const SG77350[] = {SGI330233, SGI330234, SGI330235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330236, SGI0, NULL};
const Segment * const SG77351[] = {SGI330237, SGI330238, SGI330239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330240, SGI0, NULL};
const Segment * const SG77352[] = {SGI330241, SGI330242, SGI330243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330244, SGI0, NULL};
const Segment * const SG77353[] = {SGI330245, SGI330246, SGI330247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330248, SGI0, NULL};
const Segment * const SG77354[] = {SGI330249, SGI330250, SGI330251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330252, SGI0, NULL};
const Segment * const SG77355[] = {SGI330253, SGI330254, SGI330255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330256, SGI0, NULL};
const Segment * const SG77356[] = {SGI330257, SGI330258, SGI330259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330260, SGI0, NULL};
const Segment * const SG77357[] = {SGI330261, SGI330262, SGI330263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330264, SGI0, NULL};
const Segment * const SG77358[] = {SGI330265, SGI330266, SGI330267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330268, SGI0, NULL};
const Segment * const SG77359[] = {SGI330269, SGI330270, SGI330271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330272, SGI0, NULL};
const Segment * const SG77360[] = {SGI330273, SGI330274, SGI330275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330276, SGI0, NULL};
const Segment * const SG77361[] = {SGI330277, SGI330278, SGI330279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330280, SGI0, NULL};
const Segment * const SG77362[] = {SGI330281, SGI330282, SGI330283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330284, SGI0, NULL};
const Segment * const SG77363[] = {SGI330285, SGI330286, SGI330287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330288, SGI0, NULL};
const Segment * const SG77364[] = {SGI330289, SGI330290, SGI330291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330292, SGI0, NULL};
const Segment * const SG77365[] = {SGI330293, SGI330294, SGI330295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330296, SGI0, NULL};
const Segment * const SG77366[] = {SGI330297, SGI330298, SGI330299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330300, SGI0, NULL};
const Segment * const SG77367[] = {SGI330301, SGI330302, SGI330303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330304, SGI0, NULL};
const Segment * const SG77368[] = {SGI330305, SGI330306, SGI330307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330308, SGI0, NULL};
const Segment * const SG77369[] = {SGI330309, SGI330310, SGI330311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330312, SGI0, NULL};
const Segment * const SG77370[] = {SGI330313, SGI330314, SGI330315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330316, SGI0, NULL};
const Segment * const SG77371[] = {SGI330317, SGI330318, SGI330319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330320, SGI0, NULL};
const Segment * const SG77372[] = {SGI330321, SGI330322, SGI330323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330324, SGI0, NULL};
const Segment * const SG77373[] = {SGI330325, SGI330326, SGI330327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330328, SGI0, NULL};
const Segment * const SG77374[] = {SGI330329, SGI330330, SGI330331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330332, SGI0, NULL};
const Segment * const SG77375[] = {SGI330333, SGI330334, SGI330335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330336, SGI0, NULL};
const Segment * const SG77376[] = {SGI330337, SGI330338, SGI330339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330340, SGI0, NULL};
const Segment * const SG77377[] = {SGI330341, SGI330342, SGI330343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330344, SGI0, NULL};
const Segment * const SG77378[] = {SGI330345, SGI330346, SGI330347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330348, SGI0, NULL};
const Segment * const SG77379[] = {SGI330349, SGI330350, SGI330351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330352, SGI0, NULL};
const Segment * const SG77380[] = {SGI330353, SGI330354, SGI330355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330356, SGI0, NULL};
const Segment * const SG77381[] = {SGI330357, SGI330358, SGI330359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330360, SGI0, NULL};
const Segment * const SG77382[] = {SGI330361, SGI330362, SGI330363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330364, SGI0, NULL};
const Segment * const SG77383[] = {SGI330365, SGI330366, SGI330367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330368, SGI0, NULL};
const Segment * const SG77384[] = {SGI330369, SGI330370, SGI330371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330372, SGI0, NULL};
const Segment * const SG77385[] = {SGI330373, SGI330374, SGI330375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330376, SGI0, NULL};
const Segment * const SG77386[] = {SGI330377, SGI330378, SGI330379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330380, SGI0, NULL};
const Segment * const SG77387[] = {SGI330381, SGI330382, SGI330383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330384, SGI0, NULL};
const Segment * const SG77388[] = {SGI330385, SGI330386, SGI330387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330388, SGI0, NULL};
const Segment * const SG77389[] = {SGI330389, SGI330390, SGI330391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330392, SGI0, NULL};
const Segment * const SG77390[] = {SGI330393, SGI330394, SGI330395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330396, SGI0, NULL};
const Segment * const SG77391[] = {SGI330397, SGI330398, SGI330399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330400, SGI0, NULL};
const Segment * const SG77392[] = {SGI330401, SGI330402, SGI330403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330404, SGI0, NULL};
const Segment * const SG77393[] = {SGI330405, SGI330406, SGI330407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330408, SGI0, NULL};
const Segment * const SG77394[] = {SGI330409, SGI330410, SGI330411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330412, SGI0, NULL};
const Segment * const SG77395[] = {SGI330413, SGI330414, SGI330415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330416, SGI0, NULL};
const Segment * const SG77396[] = {SGI330417, SGI330418, SGI330419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330420, SGI0, NULL};
const Segment * const SG77397[] = {SGI330421, SGI330422, SGI330423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330424, SGI0, NULL};
const Segment * const SG77398[] = {SGI330425, SGI330426, SGI330427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330428, SGI0, NULL};
const Segment * const SG77399[] = {SGI330429, SGI330430, SGI330431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330432, SGI0, NULL};
const Segment * const SG77400[] = {SGI330433, SGI330434, SGI330435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330436, SGI0, NULL};
const Segment * const SG77401[] = {SGI330437, SGI330438, SGI330439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330440, SGI0, NULL};
const Segment * const SG77402[] = {SGI330441, SGI330442, SGI330443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330444, SGI0, NULL};
const Segment * const SG77403[] = {SGI330445, SGI330446, SGI330447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330448, SGI0, NULL};
const Segment * const SG77404[] = {SGI330449, SGI330450, SGI330451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330452, SGI0, NULL};
const Segment * const SG77405[] = {SGI330453, SGI330454, SGI330455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330456, SGI0, NULL};
const Segment * const SG77406[] = {SGI330457, SGI330458, SGI330459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330460, SGI0, NULL};
const Segment * const SG77407[] = {SGI330461, SGI330462, SGI330463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330464, SGI0, NULL};
const Segment * const SG77408[] = {SGI330465, SGI330466, SGI330467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330468, SGI0, NULL};
const Segment * const SG77409[] = {SGI330469, SGI330470, SGI330471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330472, SGI0, NULL};
const Segment * const SG77410[] = {SGI330473, SGI330474, SGI330475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330476, SGI0, NULL};
const Segment * const SG77411[] = {SGI330477, SGI330478, SGI330479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330480, SGI0, NULL};
const Segment * const SG77412[] = {SGI330481, SGI330482, SGI330483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330484, SGI0, NULL};
const Segment * const SG77413[] = {SGI330485, SGI330486, SGI330487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330488, SGI0, NULL};
const Segment * const SG77414[] = {SGI330489, SGI330490, SGI330491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330492, SGI0, NULL};
const Segment * const SG77415[] = {SGI330493, SGI330494, SGI330495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330496, SGI0, NULL};
const Segment * const SG77416[] = {SGI330497, SGI330498, SGI330499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330500, SGI0, NULL};
const Segment * const SG77417[] = {SGI330501, SGI330502, SGI330503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330504, SGI0, NULL};
const Segment * const SG77418[] = {SGI330505, SGI330506, SGI330507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330508, SGI0, NULL};
const Segment * const SG77419[] = {SGI330509, SGI330510, SGI330511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330512, SGI0, NULL};
const Segment * const SG77420[] = {SGI330513, SGI330514, SGI330515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330516, SGI0, NULL};
const Segment * const SG77421[] = {SGI330517, SGI330518, SGI330519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330520, SGI0, NULL};
const Segment * const SG77422[] = {SGI0, SGI330521, SGI330523, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77423[] = {SGI330522, SGI330524, SGI330525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330526, SGI0, NULL};
const Segment * const SG77424[] = {SGI0, SGI330527, SGI330528, SGI0, SGI0, SGI0, SGI330531, NULL};
const Segment * const SG77425[] = {SGI330529, SGI330530, SGI330532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330533, SGI0, NULL};
const Segment * const SG77426[] = {SGI0, SGI330534, SGI330535, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77427[] = {SGI330536, SGI330537, SGI330538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330539, SGI0, NULL};
const Segment * const SG77428[] = {SGI0, SGI330540, SGI330541, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77429[] = {SGI330542, SGI330543, SGI330544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330545, SGI0, NULL};
const Segment * const SG77430[] = {SGI0, SGI330546, SGI330547, SGI0, SGI0, SGI0, SGI330548, NULL};
const Segment * const SG77431[] = {SGI0, SGI330550, SGI330552, SGI0, SGI0, SGI0, SGI330554, NULL};
const Segment * const SG77432[] = {SGI330549, SGI330551, SGI330553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330555, SGI0, NULL};
const Segment * const SG77433[] = {SGI330556, SGI330557, SGI330558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330559, SGI0, NULL};
const Segment * const SG77434[] = {SGI330560, SGI330561, SGI330562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330563, SGI0, NULL};
const Segment * const SG77435[] = {SGI330564, SGI330565, SGI330566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330567, SGI0, NULL};
const Segment * const SG77436[] = {SGI330568, SGI330569, SGI330570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330571, SGI0, NULL};
const Segment * const SG77437[] = {SGI330572, SGI330573, SGI330574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330575, SGI0, NULL};
const Segment * const SG77438[] = {SGI330576, SGI330577, SGI330578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330579, SGI0, NULL};
const Segment * const SG77439[] = {SGI330580, SGI330581, SGI330582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330583, SGI0, NULL};
const Segment * const SG77440[] = {SGI330584, SGI330585, SGI330586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330587, SGI0, NULL};
const Segment * const SG77441[] = {SGI330588, SGI330589, SGI330590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330591, SGI0, NULL};
const Segment * const SG77442[] = {SGI330592, SGI330593, SGI330594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330595, SGI0, NULL};
const Segment * const SG77443[] = {SGI330596, SGI330597, SGI330598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330599, SGI0, NULL};
const Segment * const SG77444[] = {SGI330600, SGI330601, SGI330602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330603, SGI0, NULL};
const Segment * const SG77445[] = {SGI330604, SGI330605, SGI330606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330607, SGI0, NULL};
const Segment * const SG77446[] = {SGI330608, SGI330609, SGI330610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330611, SGI0, NULL};
const Segment * const SG77447[] = {SGI330612, SGI330613, SGI330614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330615, SGI0, NULL};
const Segment * const SG77448[] = {SGI330616, SGI330617, SGI330618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330619, SGI0, NULL};
const Segment * const SG77449[] = {SGI330620, SGI330621, SGI330622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330623, SGI0, NULL};
const Segment * const SG77450[] = {SGI330624, SGI330625, SGI330626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330627, SGI0, NULL};
const Segment * const SG77451[] = {SGI330628, SGI330629, SGI330630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330631, SGI0, NULL};
const Segment * const SG77452[] = {SGI330632, SGI330633, SGI330634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330635, SGI0, NULL};
const Segment * const SG77453[] = {SGI330636, SGI330637, SGI330638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330638, SGI0, NULL};
const Segment * const SG77454[] = {SGI330639, SGI330640, SGI330641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330642, SGI0, NULL};
const Segment * const SG77455[] = {SGI330643, SGI330644, SGI330645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330646, SGI0, NULL};
const Segment * const SG77456[] = {SGI330647, SGI330648, SGI330649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330650, SGI0, NULL};
const Segment * const SG77457[] = {SGI330653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77458[] = {SGI330654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77459[] = {SGI330655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77460[] = {SGI330656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77461[] = {SGI330651, SGI330652, SGI330657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330658, SGI0, NULL};
const Segment * const SG77462[] = {SGI330659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77463[] = {SGI330660, SGI330661, SGI330662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330663, SGI0, NULL};
const Segment * const SG77464[] = {SGI330664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77465[] = {SGI330665, SGI330666, SGI330667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330668, SGI0, NULL};
const Segment * const SG77466[] = {SGI330669, SGI330670, SGI330671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330672, SGI0, NULL};
const Segment * const SG77467[] = {SGI330673, SGI330674, SGI330675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330676, SGI0, NULL};
const Segment * const SG77468[] = {SGI330677, SGI330678, SGI330679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330680, SGI0, NULL};
const Segment * const SG77469[] = {SGI330681, SGI330682, SGI330683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330684, SGI0, NULL};
const Segment * const SG77470[] = {SGI330685, SGI330686, SGI330687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330688, SGI0, NULL};
const Segment * const SG77471[] = {SGI330689, SGI330690, SGI330691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330692, SGI0, NULL};
const Segment * const SG77472[] = {SGI330693, SGI330694, SGI330695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330696, SGI0, NULL};
const Segment * const SG77473[] = {SGI330697, SGI330698, SGI330699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330700, SGI0, NULL};
const Segment * const SG77474[] = {SGI330701, SGI330702, SGI330703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330704, SGI0, NULL};
const Segment * const SG77475[] = {SGI330705, SGI330706, SGI330707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330708, SGI0, NULL};
const Segment * const SG77476[] = {SGI330709, SGI330710, SGI330711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330712, SGI0, NULL};
const Segment * const SG77477[] = {SGI330713, SGI330714, SGI330715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330716, SGI0, NULL};
const Segment * const SG77478[] = {SGI330717, SGI330718, SGI330719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330720, SGI0, NULL};
const Segment * const SG77479[] = {SGI330721, SGI330722, SGI330723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330724, SGI0, NULL};
const Segment * const SG77480[] = {SGI330725, SGI330726, SGI330727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330728, SGI0, NULL};
const Segment * const SG77481[] = {SGI330729, SGI330730, SGI330731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330732, SGI0, NULL};
const Segment * const SG77482[] = {SGI330733, SGI330734, SGI330735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330736, SGI0, NULL};
const Segment * const SG77483[] = {SGI330737, SGI330738, SGI330739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330740, SGI0, NULL};
const Segment * const SG77484[] = {SGI330741, SGI330742, SGI330743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330744, SGI0, NULL};
const Segment * const SG77485[] = {SGI330745, SGI330746, SGI330747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330748, SGI0, NULL};
const Segment * const SG77486[] = {SGI330749, SGI330750, SGI330751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330752, SGI0, NULL};
const Segment * const SG77487[] = {SGI330753, SGI330754, SGI330755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330756, SGI0, NULL};
const Segment * const SG77488[] = {SGI330757, SGI330758, SGI330759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330760, SGI0, NULL};
const Segment * const SG77489[] = {SGI330761, SGI330762, SGI330763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330764, SGI0, NULL};
const Segment * const SG77490[] = {SGI330765, SGI330766, SGI330767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330768, SGI0, NULL};
const Segment * const SG77491[] = {SGI330769, SGI330770, SGI330771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330772, SGI0, NULL};
const Segment * const SG77492[] = {SGI330773, SGI330774, SGI330775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330776, SGI0, NULL};
const Segment * const SG77493[] = {SGI330777, SGI330778, SGI330779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330780, SGI0, NULL};
const Segment * const SG77494[] = {SGI330781, SGI330782, SGI330783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330784, SGI0, NULL};
const Segment * const SG77495[] = {SGI330785, SGI330786, SGI330787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330788, SGI0, NULL};
const Segment * const SG77496[] = {SGI330789, SGI330790, SGI330791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330792, SGI0, NULL};
const Segment * const SG77497[] = {SGI330793, SGI330794, SGI330795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330796, SGI0, NULL};
const Segment * const SG77498[] = {SGI330797, SGI330798, SGI330799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330800, SGI0, NULL};
const Segment * const SG77499[] = {SGI330801, SGI330802, SGI330803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330804, SGI0, NULL};
const Segment * const SG77500[] = {SGI330805, SGI330806, SGI330807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330808, SGI0, NULL};
const Segment * const SG77501[] = {SGI330809, SGI330810, SGI330811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330812, SGI0, NULL};
const Segment * const SG77502[] = {SGI330813, SGI330814, SGI330815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330816, SGI0, NULL};
const Segment * const SG77503[] = {SGI330817, SGI330818, SGI330819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330820, SGI0, NULL};
const Segment * const SG77504[] = {SGI330821, SGI330822, SGI330823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330824, SGI0, NULL};
const Segment * const SG77505[] = {SGI330825, SGI330826, SGI330827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330828, SGI0, NULL};
const Segment * const SG77506[] = {SGI330829, SGI330830, SGI330831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330832, SGI0, NULL};
const Segment * const SG77507[] = {SGI330833, SGI330834, SGI330835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330836, SGI0, NULL};
const Segment * const SG77508[] = {SGI330837, SGI330838, SGI330839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330840, SGI0, NULL};
const Segment * const SG77509[] = {SGI330841, SGI330842, SGI330843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330844, SGI0, NULL};
const Segment * const SG77510[] = {SGI330845, SGI330846, SGI330847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330848, SGI0, NULL};
const Segment * const SG77511[] = {SGI330849, SGI330850, SGI330851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330852, SGI0, NULL};
const Segment * const SG77512[] = {SGI330853, SGI330854, SGI330855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330856, SGI0, NULL};
const Segment * const SG77513[] = {SGI330857, SGI330858, SGI330859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330860, SGI0, NULL};
const Segment * const SG77514[] = {SGI330861, SGI330862, SGI330863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330864, SGI0, NULL};
const Segment * const SG77515[] = {SGI330865, SGI330866, SGI330867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330868, SGI0, NULL};
const Segment * const SG77516[] = {SGI330869, SGI330870, SGI330871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330872, SGI0, NULL};
const Segment * const SG77517[] = {SGI330873, SGI330874, SGI330875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330876, SGI0, NULL};
const Segment * const SG77518[] = {SGI330877, SGI330878, SGI330879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330880, SGI0, NULL};
const Segment * const SG77519[] = {SGI330881, SGI330882, SGI330883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330884, SGI0, NULL};
const Segment * const SG77520[] = {SGI330885, SGI330886, SGI330887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330888, SGI0, NULL};
const Segment * const SG77521[] = {SGI330889, SGI330890, SGI330891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330892, SGI0, NULL};
const Segment * const SG77522[] = {SGI330893, SGI330894, SGI330895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330896, SGI0, NULL};
const Segment * const SG77523[] = {SGI330897, SGI330898, SGI330899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330900, SGI0, NULL};
const Segment * const SG77524[] = {SGI330901, SGI330902, SGI330903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330904, SGI0, NULL};
const Segment * const SG77525[] = {SGI330905, SGI330906, SGI330907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330908, SGI0, NULL};
const Segment * const SG77526[] = {SGI330909, SGI330910, SGI330911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330912, SGI0, NULL};
const Segment * const SG77527[] = {SGI330913, SGI330914, SGI330915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330916, SGI0, NULL};
const Segment * const SG77528[] = {SGI330917, SGI330918, SGI330919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330920, SGI0, NULL};
const Segment * const SG77529[] = {SGI330921, SGI330922, SGI330923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330924, SGI0, NULL};
const Segment * const SG77530[] = {SGI330925, SGI330926, SGI330927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330928, SGI0, NULL};
const Segment * const SG77531[] = {SGI330929, SGI330930, SGI330931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330932, SGI0, NULL};
const Segment * const SG77532[] = {SGI330933, SGI330934, SGI330935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330936, SGI0, NULL};
const Segment * const SG77533[] = {SGI330937, SGI330938, SGI330939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330940, SGI0, NULL};
const Segment * const SG77534[] = {SGI330941, SGI330942, SGI330943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330944, SGI0, NULL};
const Segment * const SG77535[] = {SGI330945, SGI330946, SGI330947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330948, SGI0, NULL};
const Segment * const SG77536[] = {SGI330949, SGI330950, SGI330951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330952, SGI0, NULL};
const Segment * const SG77537[] = {SGI330953, SGI330954, SGI330955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330956, SGI0, NULL};
const Segment * const SG77538[] = {SGI330957, SGI330958, SGI330959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330960, SGI0, NULL};
const Segment * const SG77539[] = {SGI330961, SGI330962, SGI330963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330964, SGI0, NULL};
const Segment * const SG77540[] = {SGI330965, SGI330966, SGI330967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330968, SGI0, NULL};
const Segment * const SG77541[] = {SGI330969, SGI330970, SGI330971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330972, SGI0, NULL};
const Segment * const SG77542[] = {SGI330973, SGI330974, SGI330975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330976, SGI0, NULL};
const Segment * const SG77543[] = {SGI330977, SGI330978, SGI330979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330980, SGI0, NULL};
const Segment * const SG77544[] = {SGI330981, SGI330982, SGI330983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330984, SGI0, NULL};
const Segment * const SG77545[] = {SGI330985, SGI330986, SGI330987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330988, SGI0, NULL};
const Segment * const SG77546[] = {SGI330989, SGI330990, SGI330991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330992, SGI0, NULL};
const Segment * const SG77547[] = {SGI330993, SGI330994, SGI330995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI330996, SGI0, NULL};
const Segment * const SG77548[] = {SGI330997, SGI330998, SGI330999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331000, SGI0, NULL};
const Segment * const SG77549[] = {SGI331001, SGI331002, SGI331003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331004, SGI0, NULL};
const Segment * const SG77550[] = {SGI331005, SGI331006, SGI331007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331008, SGI0, NULL};
const Segment * const SG77551[] = {SGI331009, SGI331010, SGI331011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331012, SGI0, NULL};
const Segment * const SG77552[] = {SGI331013, SGI331014, SGI331015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331016, SGI0, NULL};
const Segment * const SG77553[] = {SGI331017, SGI331018, SGI331019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331020, SGI0, NULL};
const Segment * const SG77554[] = {SGI331021, SGI331022, SGI331023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331024, SGI0, NULL};
const Segment * const SG77555[] = {SGI331025, SGI331026, SGI331027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331028, SGI0, NULL};
const Segment * const SG77556[] = {SGI331029, SGI331030, SGI331031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331032, SGI0, NULL};
const Segment * const SG77557[] = {SGI331033, SGI331034, SGI331035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331036, SGI0, NULL};
const Segment * const SG77558[] = {SGI331037, SGI331038, SGI331039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331040, SGI0, NULL};
const Segment * const SG77559[] = {SGI331041, SGI331042, SGI331043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331044, SGI0, NULL};
const Segment * const SG77560[] = {SGI331045, SGI331046, SGI331047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331048, SGI0, NULL};
const Segment * const SG77561[] = {SGI331049, SGI331050, SGI331051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331052, SGI0, NULL};
const Segment * const SG77562[] = {SGI331053, SGI331054, SGI331055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331056, SGI0, NULL};
const Segment * const SG77563[] = {SGI331057, SGI331058, SGI331059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331060, SGI0, NULL};
const Segment * const SG77564[] = {SGI331061, SGI331062, SGI331063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331064, SGI0, NULL};
const Segment * const SG77565[] = {SGI331065, SGI331066, SGI331067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331068, SGI0, NULL};
const Segment * const SG77566[] = {SGI331069, SGI331070, SGI331071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331072, SGI0, NULL};
const Segment * const SG77567[] = {SGI331073, SGI331074, SGI331075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331076, SGI0, NULL};
const Segment * const SG77568[] = {SGI331077, SGI331078, SGI331079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331080, SGI0, NULL};
const Segment * const SG77569[] = {SGI331081, SGI331082, SGI331083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331084, SGI0, NULL};
const Segment * const SG77570[] = {SGI331085, SGI331086, SGI331087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331088, SGI0, NULL};
const Segment * const SG77571[] = {SGI331089, SGI331090, SGI331091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331092, SGI0, NULL};
const Segment * const SG77572[] = {SGI331093, SGI331094, SGI331095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331096, SGI0, NULL};
const Segment * const SG77573[] = {SGI331097, SGI331098, SGI0, SGI0, NULL};
const Segment * const SG77574[] = {SGI331099, SGI331100, SGI0, SGI0, NULL};
const Segment * const SG77575[] = {SGI331101, SGI331102, SGI331103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77576[] = {SGI331104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77577[] = {SGI331105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77578[] = {SGI331106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77579[] = {SGI331107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77580[] = {SGI331108, SGI331109, SGI0, SGI0, NULL};
const Segment * const SG77581[] = {SGI331110, SGI331111, SGI331112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331112, SGI0, NULL};
const Segment * const SG77582[] = {SGI331113, SGI331114, SGI331115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331116, SGI0, NULL};
const Segment * const SG77583[] = {SGI331117, SGI331118, SGI331119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331120, SGI0, NULL};
const Segment * const SG77584[] = {SGI331121, SGI331122, SGI331123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331124, SGI0, NULL};
const Segment * const SG77585[] = {SGI331125, SGI331126, SGI331127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331128, SGI0, NULL};
const Segment * const SG77586[] = {SGI331129, SGI331130, SGI331131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331132, SGI0, NULL};
const Segment * const SG77587[] = {SGI331133, SGI331134, SGI331135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331136, SGI0, NULL};
const Segment * const SG77588[] = {SGI331137, SGI331138, SGI331139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331140, SGI0, NULL};
const Segment * const SG77589[] = {SGI331141, SGI331142, SGI331143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331144, SGI0, NULL};
const Segment * const SG77590[] = {SGI331145, SGI331146, SGI331147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331148, SGI0, NULL};
const Segment * const SG77591[] = {SGI331149, SGI331150, SGI331151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331152, SGI0, NULL};
const Segment * const SG77592[] = {SGI331153, SGI331154, SGI331155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331156, SGI0, NULL};
const Segment * const SG77593[] = {SGI331157, SGI331158, SGI331159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331160, SGI0, NULL};
const Segment * const SG77594[] = {SGI331161, SGI331162, SGI331163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331164, SGI0, NULL};
const Segment * const SG77595[] = {SGI331165, SGI331166, SGI331167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331168, SGI0, NULL};
const Segment * const SG77596[] = {SGI331169, SGI331170, SGI331171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331172, SGI0, NULL};
const Segment * const SG77597[] = {SGI331173, SGI331174, SGI331175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331176, SGI0, NULL};
const Segment * const SG77598[] = {SGI331177, SGI331178, SGI331179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331180, SGI0, NULL};
const Segment * const SG77599[] = {SGI331181, SGI331182, SGI331183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331184, SGI0, NULL};
const Segment * const SG77600[] = {SGI331185, SGI331186, SGI331187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331188, SGI0, NULL};
const Segment * const SG77601[] = {SGI331189, SGI331190, SGI331191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331192, SGI0, NULL};
const Segment * const SG77602[] = {SGI331193, SGI331194, SGI331195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331196, SGI0, NULL};
const Segment * const SG77603[] = {SGI331197, SGI331198, SGI331199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331200, SGI0, NULL};
const Segment * const SG77604[] = {SGI331201, SGI331202, SGI331203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331204, SGI0, NULL};
const Segment * const SG77605[] = {SGI331205, SGI331206, SGI331207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331208, SGI0, NULL};
const Segment * const SG77606[] = {SGI331209, SGI331210, SGI331211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331212, SGI0, NULL};
const Segment * const SG77607[] = {SGI331213, SGI331214, SGI331215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331216, SGI0, NULL};
const Segment * const SG77608[] = {SGI331217, SGI331218, SGI331219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331220, SGI0, NULL};
const Segment * const SG77609[] = {SGI331221, SGI331222, SGI331223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331224, SGI0, NULL};
const Segment * const SG77610[] = {SGI331225, SGI331226, SGI331227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331228, SGI0, NULL};
const Segment * const SG77611[] = {SGI331229, SGI331230, SGI331231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331232, SGI0, NULL};
const Segment * const SG77612[] = {SGI331233, SGI331234, SGI331235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331236, SGI0, NULL};
const Segment * const SG77613[] = {SGI331237, SGI331238, SGI331239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331240, SGI0, NULL};
const Segment * const SG77614[] = {SGI331241, SGI331242, SGI331243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331244, SGI0, NULL};
const Segment * const SG77615[] = {SGI331245, SGI331246, SGI331247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331248, SGI0, NULL};
const Segment * const SG77616[] = {SGI331249, SGI331250, SGI331251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331252, SGI0, NULL};
const Segment * const SG77617[] = {SGI331253, SGI331254, SGI331255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331256, SGI0, NULL};
const Segment * const SG77618[] = {SGI331257, SGI331258, SGI331259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331260, SGI0, NULL};
const Segment * const SG77619[] = {SGI331261, SGI331262, SGI331263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331264, SGI0, NULL};
const Segment * const SG77620[] = {SGI331265, SGI331266, SGI331267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331268, SGI0, NULL};
const Segment * const SG77621[] = {SGI331269, SGI331270, SGI331271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331272, SGI0, NULL};
const Segment * const SG77622[] = {SGI331273, SGI331274, SGI331275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331276, SGI0, NULL};
const Segment * const SG77623[] = {SGI331277, SGI331278, SGI331279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331280, SGI0, NULL};
const Segment * const SG77624[] = {SGI331281, SGI331282, SGI331283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331284, SGI0, NULL};
const Segment * const SG77625[] = {SGI331285, SGI331286, SGI331287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331288, SGI0, NULL};
const Segment * const SG77626[] = {SGI331289, SGI331290, SGI331291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331292, SGI0, NULL};
const Segment * const SG77627[] = {SGI331293, SGI331294, SGI331295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331296, SGI0, NULL};
const Segment * const SG77628[] = {SGI331297, SGI331298, SGI331299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331300, SGI0, NULL};
const Segment * const SG77629[] = {SGI331301, SGI331302, SGI331303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331304, SGI0, NULL};
const Segment * const SG77630[] = {SGI331305, SGI331306, SGI331307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331308, SGI0, NULL};
const Segment * const SG77631[] = {SGI331309, SGI331310, SGI331311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331312, SGI0, NULL};
const Segment * const SG77632[] = {SGI331313, SGI331314, SGI331315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331316, SGI0, NULL};
const Segment * const SG77633[] = {SGI331317, SGI331318, SGI331319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331320, SGI0, NULL};
const Segment * const SG77634[] = {SGI331321, SGI331322, SGI331323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331324, SGI0, NULL};
const Segment * const SG77635[] = {SGI331325, SGI331326, SGI331327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331328, SGI0, NULL};
const Segment * const SG77636[] = {SGI331329, SGI331330, SGI331331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331332, SGI0, NULL};
const Segment * const SG77637[] = {SGI331333, SGI331334, SGI331335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331336, SGI0, NULL};
const Segment * const SG77638[] = {SGI331337, SGI331338, SGI331339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331340, SGI0, NULL};
const Segment * const SG77639[] = {SGI331341, SGI331342, SGI331343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331344, SGI0, NULL};
const Segment * const SG77640[] = {SGI331345, SGI331346, SGI331347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331348, SGI0, NULL};
const Segment * const SG77641[] = {SGI331349, SGI331350, SGI331351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331352, SGI0, NULL};
const Segment * const SG77642[] = {SGI331353, SGI331354, SGI331355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331356, SGI0, NULL};
const Segment * const SG77643[] = {SGI331357, SGI331358, SGI331359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331360, SGI0, NULL};
const Segment * const SG77644[] = {SGI331361, SGI331362, SGI331363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331364, SGI0, NULL};
const Segment * const SG77645[] = {SGI331365, SGI331366, SGI331367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331368, SGI0, NULL};
const Segment * const SG77646[] = {SGI331369, SGI331370, SGI331371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331372, SGI0, NULL};
const Segment * const SG77647[] = {SGI331373, SGI331374, SGI331375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331376, SGI0, NULL};
const Segment * const SG77648[] = {SGI331377, SGI331378, SGI331379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331380, SGI0, NULL};
const Segment * const SG77649[] = {SGI331381, SGI331382, SGI331383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331384, SGI0, NULL};
const Segment * const SG77650[] = {SGI331385, SGI331386, SGI331387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331388, SGI0, NULL};
const Segment * const SG77651[] = {SGI331389, SGI331390, SGI331391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331392, SGI0, NULL};
const Segment * const SG77652[] = {SGI331393, SGI331394, SGI331395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331396, SGI0, NULL};
const Segment * const SG77653[] = {SGI331397, SGI331398, SGI331399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331400, SGI0, NULL};
const Segment * const SG77654[] = {SGI331401, SGI331402, SGI331403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331404, SGI0, NULL};
const Segment * const SG77655[] = {SGI331405, SGI331406, SGI331407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331408, SGI0, NULL};
const Segment * const SG77656[] = {SGI331409, SGI331410, SGI331411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331412, SGI0, NULL};
const Segment * const SG77657[] = {SGI331413, SGI331414, SGI331415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331416, SGI0, NULL};
const Segment * const SG77658[] = {SGI331417, SGI331418, SGI331419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331420, SGI0, NULL};
const Segment * const SG77659[] = {SGI331421, SGI331422, SGI331423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331424, SGI0, NULL};
const Segment * const SG77660[] = {SGI331425, SGI331426, SGI331427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331428, SGI0, NULL};
const Segment * const SG77661[] = {SGI331429, SGI331430, SGI331431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331432, SGI0, NULL};
const Segment * const SG77662[] = {SGI331433, SGI331434, SGI331435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331436, SGI0, NULL};
const Segment * const SG77663[] = {SGI331437, SGI331438, SGI331439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331440, SGI0, NULL};
const Segment * const SG77664[] = {SGI331441, SGI331442, SGI331443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331444, SGI0, NULL};
const Segment * const SG77665[] = {SGI331445, SGI331446, SGI331447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331448, SGI0, NULL};
const Segment * const SG77666[] = {SGI331449, SGI331450, SGI331451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331452, SGI0, NULL};
const Segment * const SG77667[] = {SGI331453, SGI331454, SGI331455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331456, SGI0, NULL};
const Segment * const SG77668[] = {SGI331457, SGI331458, SGI331459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331460, SGI0, NULL};
const Segment * const SG77669[] = {SGI331461, SGI331462, SGI331463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331464, SGI0, NULL};
const Segment * const SG77670[] = {SGI331465, SGI331466, SGI331467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331468, SGI0, NULL};
const Segment * const SG77671[] = {SGI331469, SGI331470, SGI331471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331472, SGI0, NULL};
const Segment * const SG77672[] = {SGI331473, SGI331474, SGI331475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331476, SGI0, NULL};
const Segment * const SG77673[] = {SGI331477, SGI331478, SGI331479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331480, SGI0, NULL};
const Segment * const SG77674[] = {SGI331481, SGI331482, SGI331483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331484, SGI0, NULL};
const Segment * const SG77675[] = {SGI331485, SGI331486, SGI331487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331488, SGI0, NULL};
const Segment * const SG77676[] = {SGI331489, SGI331490, SGI331491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331492, SGI0, NULL};
const Segment * const SG77677[] = {SGI331493, SGI331494, SGI331495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331496, SGI0, NULL};
const Segment * const SG77678[] = {SGI331497, SGI331498, SGI331499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331500, SGI0, NULL};
const Segment * const SG77679[] = {SGI331501, SGI331502, SGI331503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331504, SGI0, NULL};
const Segment * const SG77680[] = {SGI331505, SGI331506, SGI331507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331508, SGI0, NULL};
const Segment * const SG77681[] = {SGI331509, SGI331511, SGI331512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331513, SGI0, NULL};
const Segment * const SG77682[] = {SGI331510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331514, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77683[] = {SGI331515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77684[] = {SGI331516, SGI331517, SGI331518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331519, SGI0, NULL};
const Segment * const SG77685[] = {SGI331520, SGI331521, SGI331522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331524, SGI0, NULL};
const Segment * const SG77686[] = {SGI331523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77687[] = {SGI331525, SGI331526, SGI331527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331528, SGI0, NULL};
const Segment * const SG77688[] = {SGI331529, SGI331530, SGI331531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331532, SGI0, NULL};
const Segment * const SG77689[] = {SGI331533, SGI331534, SGI331535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331536, SGI0, NULL};
const Segment * const SG77690[] = {SGI331537, SGI331538, SGI331539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331540, SGI0, NULL};
const Segment * const SG77691[] = {SGI331541, SGI331542, SGI331543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331544, SGI0, NULL};
const Segment * const SG77692[] = {SGI331545, SGI331546, SGI331547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331548, SGI0, NULL};
const Segment * const SG77693[] = {SGI331549, SGI331550, SGI331551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331552, SGI0, NULL};
const Segment * const SG77694[] = {SGI331553, SGI331554, SGI331555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331556, SGI0, NULL};
const Segment * const SG77695[] = {SGI331557, SGI331558, SGI331559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331560, SGI0, NULL};
const Segment * const SG77696[] = {SGI331561, SGI331562, SGI331563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331564, SGI0, NULL};
const Segment * const SG77697[] = {SGI331565, SGI331566, SGI331567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331568, SGI0, NULL};
const Segment * const SG77698[] = {SGI331569, SGI331570, SGI331571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331572, SGI0, NULL};
const Segment * const SG77699[] = {SGI331573, SGI331574, SGI331575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331576, SGI0, NULL};
const Segment * const SG77700[] = {SGI331577, SGI331578, SGI331579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331580, SGI0, NULL};
const Segment * const SG77701[] = {SGI331581, SGI331582, SGI331583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331584, SGI0, NULL};
const Segment * const SG77702[] = {SGI331585, SGI331586, SGI331587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331588, SGI0, NULL};
const Segment * const SG77703[] = {SGI331589, SGI331590, SGI331591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331592, SGI0, NULL};
const Segment * const SG77704[] = {SGI331593, SGI331594, SGI331595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331596, SGI0, NULL};
const Segment * const SG77705[] = {SGI331597, SGI331598, SGI331599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331600, SGI0, NULL};
const Segment * const SG77706[] = {SGI331601, SGI331602, SGI331603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331604, SGI0, NULL};
const Segment * const SG77707[] = {SGI331605, SGI331606, SGI331607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331608, SGI0, NULL};
const Segment * const SG77708[] = {SGI331609, SGI331610, SGI331611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331612, SGI0, NULL};
const Segment * const SG77709[] = {SGI331613, SGI331614, SGI331615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331616, SGI0, NULL};
const Segment * const SG77710[] = {SGI331617, SGI331618, SGI331619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331620, SGI0, NULL};
const Segment * const SG77711[] = {SGI331621, SGI331622, SGI331623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331624, SGI0, NULL};
const Segment * const SG77712[] = {SGI331625, SGI331626, SGI331627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331628, SGI0, NULL};
const Segment * const SG77713[] = {SGI331629, SGI331630, SGI331631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331632, SGI0, NULL};
const Segment * const SG77714[] = {SGI331633, SGI331634, SGI331635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331636, SGI0, NULL};
const Segment * const SG77715[] = {SGI331637, SGI331638, SGI331639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331640, SGI0, NULL};
const Segment * const SG77716[] = {SGI331641, SGI331642, SGI331643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331643, SGI0, NULL};
const Segment * const SG77717[] = {SGI331644, SGI331645, SGI331646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331647, SGI0, NULL};
const Segment * const SG77718[] = {SGI331648, SGI331649, SGI331650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331651, SGI0, NULL};
const Segment * const SG77719[] = {SGI331652, SGI331653, SGI331654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331655, SGI0, NULL};
const Segment * const SG77720[] = {SGI331656, SGI331657, SGI331658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331659, SGI0, NULL};
const Segment * const SG77721[] = {SGI331660, SGI331661, SGI331662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331663, SGI0, NULL};
const Segment * const SG77722[] = {SGI331664, SGI331665, SGI331666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331667, SGI0, NULL};
const Segment * const SG77723[] = {SGI331668, SGI331669, SGI331670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331671, SGI0, NULL};
const Segment * const SG77724[] = {SGI331672, SGI331673, SGI331674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331675, SGI0, NULL};
const Segment * const SG77725[] = {SGI331676, SGI331677, SGI331678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331679, SGI0, NULL};
const Segment * const SG77726[] = {SGI331680, SGI331681, SGI331682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331683, SGI0, NULL};
const Segment * const SG77727[] = {SGI331684, SGI331685, SGI331686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331687, SGI0, NULL};
const Segment * const SG77728[] = {SGI331688, SGI331689, SGI331690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331691, SGI0, NULL};
const Segment * const SG77729[] = {SGI331692, SGI331693, SGI331694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331695, SGI0, NULL};
const Segment * const SG77730[] = {SGI331696, SGI331697, SGI331698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331699, SGI0, NULL};
const Segment * const SG77731[] = {SGI331700, SGI331701, SGI331702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331703, SGI0, NULL};
const Segment * const SG77732[] = {SGI331704, SGI331705, SGI331706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331707, SGI0, NULL};
const Segment * const SG77733[] = {SGI331708, SGI331709, SGI331710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331711, SGI0, NULL};
const Segment * const SG77734[] = {SGI331712, SGI331713, SGI331714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331715, SGI0, NULL};
const Segment * const SG77735[] = {SGI331716, SGI331717, SGI331718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331719, SGI0, NULL};
const Segment * const SG77736[] = {SGI331720, SGI331721, SGI331722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331723, SGI0, NULL};
const Segment * const SG77737[] = {SGI331724, SGI331725, SGI331726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331727, SGI0, NULL};
const Segment * const SG77738[] = {SGI331728, SGI331729, SGI331730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331731, SGI0, NULL};
const Segment * const SG77739[] = {SGI331732, SGI331733, SGI331734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331735, SGI0, NULL};
const Segment * const SG77740[] = {SGI331736, SGI331737, SGI331738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331739, SGI0, NULL};
const Segment * const SG77741[] = {SGI331740, SGI331741, SGI331742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331743, SGI0, NULL};
const Segment * const SG77742[] = {SGI331744, SGI331745, SGI331746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331747, SGI0, NULL};
const Segment * const SG77743[] = {SGI331748, SGI331749, SGI331750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331751, SGI0, NULL};
const Segment * const SG77744[] = {SGI331752, SGI331753, SGI331754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331755, SGI0, NULL};
const Segment * const SG77745[] = {SGI331756, SGI331757, SGI331758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331759, SGI0, NULL};
const Segment * const SG77746[] = {SGI331760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77747[] = {SGI331761, SGI331762, SGI331763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331764, SGI0, NULL};
const Segment * const SG77748[] = {SGI331765, SGI331766, SGI331767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331768, SGI0, NULL};
const Segment * const SG77749[] = {SGI331769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77750[] = {SGI331770, SGI331771, SGI331772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331773, SGI0, NULL};
const Segment * const SG77751[] = {SGI331774, SGI331775, SGI331776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331777, SGI0, NULL};
const Segment * const SG77752[] = {SGI331778, SGI331779, SGI331780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331781, SGI0, NULL};
const Segment * const SG77753[] = {SGI331782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77754[] = {SGI331783, SGI331784, SGI331785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331786, SGI0, NULL};
const Segment * const SG77755[] = {SGI331787, SGI331788, SGI331789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331790, SGI0, NULL};
const Segment * const SG77756[] = {SGI331791, SGI331792, SGI331793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331794, SGI0, NULL};
const Segment * const SG77757[] = {SGI331795, SGI331796, SGI331797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331798, SGI0, NULL};
const Segment * const SG77758[] = {SGI331799, SGI331800, SGI331801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331802, SGI0, NULL};
const Segment * const SG77759[] = {SGI331803, SGI331804, SGI331805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331806, SGI0, NULL};
const Segment * const SG77760[] = {SGI331807, SGI331808, SGI331809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331810, SGI0, NULL};
const Segment * const SG77761[] = {SGI331811, SGI331812, SGI331813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331814, SGI0, NULL};
const Segment * const SG77762[] = {SGI331815, SGI331816, SGI331817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331818, SGI0, NULL};
const Segment * const SG77763[] = {SGI331819, SGI331820, SGI331821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331822, SGI0, NULL};
const Segment * const SG77764[] = {SGI331823, SGI331824, SGI331825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331826, SGI0, NULL};
const Segment * const SG77765[] = {SGI331827, SGI331828, SGI331829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331830, SGI0, NULL};
const Segment * const SG77766[] = {SGI331831, SGI331832, SGI331833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331834, SGI0, NULL};
const Segment * const SG77767[] = {SGI331835, SGI331836, SGI331837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331838, SGI0, NULL};
const Segment * const SG77768[] = {SGI331839, SGI331840, SGI331841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331842, SGI0, NULL};
const Segment * const SG77769[] = {SGI331843, SGI331844, SGI331845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331846, SGI0, NULL};
const Segment * const SG77770[] = {SGI331847, SGI331848, SGI331849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331850, SGI0, NULL};
const Segment * const SG77771[] = {SGI331851, SGI331852, SGI331853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331854, SGI0, NULL};
const Segment * const SG77772[] = {SGI331855, SGI331856, SGI331857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331858, SGI0, NULL};
const Segment * const SG77773[] = {SGI331859, SGI331860, SGI331861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331862, SGI0, NULL};
const Segment * const SG77774[] = {SGI331863, SGI331864, SGI331865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331866, SGI0, NULL};
const Segment * const SG77775[] = {SGI331867, SGI331868, SGI331869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331870, SGI0, NULL};
const Segment * const SG77776[] = {SGI331871, SGI331872, SGI331873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331874, SGI0, NULL};
const Segment * const SG77777[] = {SGI331875, SGI331876, SGI331877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331878, SGI0, NULL};
const Segment * const SG77778[] = {SGI331879, SGI331880, SGI331881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331882, SGI0, NULL};
const Segment * const SG77779[] = {SGI331883, SGI331884, SGI331885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331886, SGI0, NULL};
const Segment * const SG77780[] = {SGI331887, SGI331888, SGI331889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331890, SGI0, NULL};
const Segment * const SG77781[] = {SGI331891, SGI331892, SGI331893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331894, SGI0, NULL};
const Segment * const SG77782[] = {SGI331895, SGI331896, SGI331897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331898, SGI0, NULL};
const Segment * const SG77783[] = {SGI331899, SGI331900, SGI331901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331902, SGI0, NULL};
const Segment * const SG77784[] = {SGI331903, SGI331904, SGI331905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331906, SGI0, NULL};
const Segment * const SG77785[] = {SGI331907, SGI331908, SGI331909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331910, SGI0, NULL};
const Segment * const SG77786[] = {SGI331911, SGI331912, SGI331913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331914, SGI0, NULL};
const Segment * const SG77787[] = {SGI331915, SGI331916, SGI331917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331918, SGI0, NULL};
const Segment * const SG77788[] = {SGI331919, SGI331920, SGI331921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331922, SGI0, NULL};
const Segment * const SG77789[] = {SGI331923, SGI331924, SGI331925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331926, SGI0, NULL};
const Segment * const SG77790[] = {SGI331927, SGI331928, SGI331929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331930, SGI0, NULL};
const Segment * const SG77791[] = {SGI331931, SGI331932, SGI331933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331934, SGI0, NULL};
const Segment * const SG77792[] = {SGI331935, SGI331936, SGI331937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331938, SGI0, NULL};
const Segment * const SG77793[] = {SGI331939, SGI331940, SGI331941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331942, SGI0, NULL};
const Segment * const SG77794[] = {SGI331943, SGI331944, SGI331945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331946, SGI0, NULL};
const Segment * const SG77795[] = {SGI331947, SGI331948, SGI331949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331950, SGI0, NULL};
const Segment * const SG77796[] = {SGI331951, SGI331952, SGI331953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331954, SGI0, NULL};
const Segment * const SG77797[] = {SGI331955, SGI331956, SGI331957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331958, SGI0, NULL};
const Segment * const SG77798[] = {SGI331959, SGI331960, SGI331961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331962, SGI0, NULL};
const Segment * const SG77799[] = {SGI331963, SGI331964, SGI331965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331966, SGI0, NULL};
const Segment * const SG77800[] = {SGI331967, SGI331968, SGI331969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331970, SGI0, NULL};
const Segment * const SG77801[] = {SGI331971, SGI331972, SGI331973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331974, SGI0, NULL};
const Segment * const SG77802[] = {SGI331975, SGI331976, SGI331977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331978, SGI0, NULL};
const Segment * const SG77803[] = {SGI331979, SGI331980, SGI331981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331982, SGI0, NULL};
const Segment * const SG77804[] = {SGI331983, SGI331984, SGI331985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331986, SGI0, NULL};
const Segment * const SG77805[] = {SGI331987, SGI331988, SGI331989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331990, SGI0, NULL};
const Segment * const SG77806[] = {SGI331991, SGI331992, SGI331993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331994, SGI0, NULL};
const Segment * const SG77807[] = {SGI331995, SGI331996, SGI331997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI331998, SGI0, NULL};
const Segment * const SG77808[] = {SGI331999, SGI332000, SGI332001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332002, SGI0, NULL};
const Segment * const SG77809[] = {SGI332003, SGI332004, SGI332005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332006, SGI0, NULL};
const Segment * const SG77810[] = {SGI332007, SGI332008, SGI332009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332010, SGI0, NULL};
const Segment * const SG77811[] = {SGI332011, SGI332012, SGI332013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332014, SGI0, NULL};
const Segment * const SG77812[] = {SGI332015, SGI332016, SGI332017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332018, SGI0, NULL};
const Segment * const SG77813[] = {SGI332019, SGI332020, SGI332021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332022, SGI0, NULL};
const Segment * const SG77814[] = {SGI332023, SGI332024, SGI332025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332026, SGI0, NULL};
const Segment * const SG77815[] = {SGI332027, SGI332028, SGI332029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332030, SGI0, NULL};
const Segment * const SG77816[] = {SGI332031, SGI332032, SGI332033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332034, SGI0, NULL};
const Segment * const SG77817[] = {SGI332035, SGI332036, SGI332037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332038, SGI0, NULL};
const Segment * const SG77818[] = {SGI332039, SGI332040, SGI332041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332042, SGI0, NULL};
const Segment * const SG77819[] = {SGI332043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77820[] = {SGI332044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77821[] = {SGI332045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG77822[] = {SGI332046, SGI332047, SGI332048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332048, SGI0, NULL};
const Segment * const SG77823[] = {SGI332049, SGI332050, SGI332051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332052, SGI0, NULL};
const Segment * const SG77824[] = {SGI332053, SGI332054, SGI332055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332056, SGI0, NULL};
const Segment * const SG77825[] = {SGI332057, SGI332058, SGI332059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332060, SGI0, NULL};
const Segment * const SG77826[] = {SGI332061, SGI332062, SGI332063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332063, SGI0, NULL};
const Segment * const SG77827[] = {SGI332064, SGI332065, SGI332066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332067, SGI0, NULL};
const Segment * const SG77828[] = {SGI332068, SGI332069, SGI332070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332071, SGI0, NULL};
const Segment * const SG77829[] = {SGI332072, SGI332073, SGI332074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332075, SGI0, NULL};
const Segment * const SG77830[] = {SGI332076, SGI332077, SGI332078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332079, SGI0, NULL};
const Segment * const SG77831[] = {SGI332080, SGI332081, SGI332082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332083, SGI0, NULL};
const Segment * const SG77832[] = {SGI332084, SGI332085, SGI332086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332087, SGI0, NULL};
const Segment * const SG77833[] = {SGI332088, SGI332089, SGI332090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332091, SGI0, NULL};
const Segment * const SG77834[] = {SGI332092, SGI332093, SGI332094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332095, SGI0, NULL};
const Segment * const SG77835[] = {SGI332096, SGI332097, SGI332098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332099, SGI0, NULL};
const Segment * const SG77836[] = {SGI332100, SGI332101, SGI332102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332103, SGI0, NULL};
const Segment * const SG77837[] = {SGI332104, SGI332105, SGI332106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332107, SGI0, NULL};
const Segment * const SG77838[] = {SGI332108, SGI332109, SGI332110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332111, SGI0, NULL};
const Segment * const SG77839[] = {SGI332112, SGI332113, SGI332114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332115, SGI0, NULL};
const Segment * const SG77840[] = {SGI332116, SGI332117, SGI332118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332119, SGI0, NULL};
const Segment * const SG77841[] = {SGI332120, SGI332121, SGI332122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332123, SGI0, NULL};
const Segment * const SG77842[] = {SGI332124, SGI332125, SGI332126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332127, SGI0, NULL};
const Segment * const SG77843[] = {SGI332128, SGI332129, SGI332130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332131, SGI0, NULL};
const Segment * const SG77844[] = {SGI332132, SGI332133, SGI332134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332135, SGI0, NULL};
const Segment * const SG77845[] = {SGI332136, SGI332137, SGI332138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332139, SGI0, NULL};
const Segment * const SG77846[] = {SGI332140, SGI332141, SGI332142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332143, SGI0, NULL};
const Segment * const SG77847[] = {SGI332144, SGI332145, SGI332146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332147, SGI0, NULL};
const Segment * const SG77848[] = {SGI332148, SGI332149, SGI332150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332151, SGI0, NULL};
const Segment * const SG77849[] = {SGI332152, SGI332153, SGI332154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332155, SGI0, NULL};
const Segment * const SG77850[] = {SGI332156, SGI332157, SGI332158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332159, SGI0, NULL};
const Segment * const SG77851[] = {SGI332160, SGI332161, SGI332162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332163, SGI0, NULL};
const Segment * const SG77852[] = {SGI332164, SGI332165, SGI332166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332167, SGI0, NULL};
const Segment * const SG77853[] = {SGI332168, SGI332169, SGI332170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332171, SGI0, NULL};
const Segment * const SG77854[] = {SGI332172, SGI332173, SGI332174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332175, SGI0, NULL};
const Segment * const SG77855[] = {SGI332176, SGI332177, SGI332178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332179, SGI0, NULL};
const Segment * const SG77856[] = {SGI332180, SGI332181, SGI332182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332183, SGI0, NULL};
const Segment * const SG77857[] = {SGI332184, SGI332185, SGI332186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332187, SGI0, NULL};
const Segment * const SG77858[] = {SGI332188, SGI332189, SGI332190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332191, SGI0, NULL};
const Segment * const SG77859[] = {SGI332192, SGI332193, SGI332194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332195, SGI0, NULL};
const Segment * const SG77860[] = {SGI332196, SGI332197, SGI332198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332199, SGI0, NULL};
const Segment * const SG77861[] = {SGI332200, SGI332201, SGI332202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332203, SGI0, NULL};
const Segment * const SG77862[] = {SGI332204, SGI332205, SGI332206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332207, SGI0, NULL};
const Segment * const SG77863[] = {SGI332208, SGI332209, SGI332210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332211, SGI0, NULL};
const Segment * const SG77864[] = {SGI332212, SGI332213, SGI332214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332215, SGI0, NULL};
const Segment * const SG77865[] = {SGI332216, SGI332217, SGI332218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332219, SGI0, NULL};
const Segment * const SG77866[] = {SGI332220, SGI332221, SGI332222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332223, SGI0, NULL};
const Segment * const SG77867[] = {SGI332224, SGI332225, SGI332226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332227, SGI0, NULL};
const Segment * const SG77868[] = {SGI332228, SGI332229, SGI332230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332231, SGI0, NULL};
const Segment * const SG77869[] = {SGI332232, SGI332233, SGI332234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332235, SGI0, NULL};
const Segment * const SG77870[] = {SGI332236, SGI332237, SGI332238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332239, SGI0, NULL};
const Segment * const SG77871[] = {SGI332240, SGI332241, SGI332242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332243, SGI0, NULL};
const Segment * const SG77872[] = {SGI332244, SGI332245, SGI332246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332247, SGI0, NULL};
const Segment * const SG77873[] = {SGI332248, SGI332249, SGI332250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332251, SGI0, NULL};
const Segment * const SG77874[] = {SGI332252, SGI332253, SGI332254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332255, SGI0, NULL};
const Segment * const SG77875[] = {SGI332256, SGI332257, SGI332258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332259, SGI0, NULL};
const Segment * const SG77876[] = {SGI332260, SGI332261, SGI332262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332263, SGI0, NULL};
const Segment * const SG77877[] = {SGI332264, SGI332265, SGI332266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332267, SGI0, NULL};
const Segment * const SG77878[] = {SGI332268, SGI332269, SGI332270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332271, SGI0, NULL};
const Segment * const SG77879[] = {SGI332272, SGI332273, SGI332274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332275, SGI0, NULL};
const Segment * const SG77880[] = {SGI332276, SGI332277, SGI332278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332279, SGI0, NULL};
const Segment * const SG77881[] = {SGI332280, SGI332281, SGI332282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332283, SGI0, NULL};
const Segment * const SG77882[] = {SGI332284, SGI332285, SGI332286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332287, SGI0, NULL};
const Segment * const SG77883[] = {SGI332288, SGI332289, SGI332290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332291, SGI0, NULL};
const Segment * const SG77884[] = {SGI332292, SGI332293, SGI332294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332295, SGI0, NULL};
const Segment * const SG77885[] = {SGI332296, SGI332297, SGI332298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332299, SGI0, NULL};
const Segment * const SG77886[] = {SGI332300, SGI332301, SGI332302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332302, SGI0, NULL};
const Segment * const SG77887[] = {SGI332303, SGI332304, SGI332305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332306, SGI0, NULL};
const Segment * const SG77888[] = {SGI332307, SGI332308, SGI332309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332310, SGI0, NULL};
const Segment * const SG77889[] = {SGI332311, SGI332312, SGI332313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332314, SGI0, NULL};
const Segment * const SG77890[] = {SGI332315, SGI332316, SGI332317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332318, SGI0, NULL};
const Segment * const SG77891[] = {SGI332319, SGI332320, SGI332321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332322, SGI0, NULL};
const Segment * const SG77892[] = {SGI332323, SGI332324, SGI332325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332326, SGI0, NULL};
const Segment * const SG77893[] = {SGI332327, SGI332328, SGI332329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332330, SGI0, NULL};
const Segment * const SG77894[] = {SGI332331, SGI332332, SGI332333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332334, SGI0, NULL};
const Segment * const SG77895[] = {SGI332335, SGI332336, SGI332337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332338, SGI0, NULL};
const Segment * const SG77896[] = {SGI332339, SGI332340, SGI332341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332342, SGI0, NULL};
const Segment * const SG77897[] = {SGI332343, SGI332344, SGI332345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332346, SGI0, NULL};
const Segment * const SG77898[] = {SGI332347, SGI332348, SGI332349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332350, SGI0, NULL};
const Segment * const SG77899[] = {SGI332351, SGI332352, SGI332353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332354, SGI0, NULL};
const Segment * const SG77900[] = {SGI332355, SGI332356, SGI332357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332358, SGI0, NULL};
const Segment * const SG77901[] = {SGI332359, SGI332360, SGI332361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332362, SGI0, NULL};
const Segment * const SG77902[] = {SGI332363, SGI332364, SGI332365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332366, SGI0, NULL};
const Segment * const SG77903[] = {SGI332367, SGI332368, SGI332369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332370, SGI0, NULL};
const Segment * const SG77904[] = {SGI332371, SGI332372, SGI332373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332374, SGI0, NULL};
const Segment * const SG77905[] = {SGI332375, SGI332376, SGI332377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332377, SGI0, NULL};
const Segment * const SG77906[] = {SGI332378, SGI332379, SGI332380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332381, SGI0, NULL};
const Segment * const SG77907[] = {SGI332382, SGI332383, SGI332384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332385, SGI0, NULL};
const Segment * const SG77908[] = {SGI332386, SGI332387, SGI332388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332389, SGI0, NULL};
const Segment * const SG77909[] = {SGI332390, SGI332391, SGI332392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332393, SGI0, NULL};
const Segment * const SG77910[] = {SGI332394, SGI332395, SGI332396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332397, SGI0, NULL};
const Segment * const SG77911[] = {SGI332398, SGI332399, SGI332400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332401, SGI0, NULL};
const Segment * const SG77912[] = {SGI332402, SGI332403, SGI332404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332405, SGI0, NULL};
const Segment * const SG77913[] = {SGI332406, SGI332407, SGI332408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332409, SGI0, NULL};
const Segment * const SG77914[] = {SGI332410, SGI332411, SGI332412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332413, SGI0, NULL};
const Segment * const SG77915[] = {SGI332414, SGI332415, SGI332416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332417, SGI0, NULL};
const Segment * const SG77916[] = {SGI332418, SGI332419, SGI332420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332421, SGI0, NULL};
const Segment * const SG77917[] = {SGI332422, SGI332423, SGI332424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332425, SGI0, NULL};
const Segment * const SG77918[] = {SGI332426, SGI332427, SGI332428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332429, SGI0, NULL};
const Segment * const SG77919[] = {SGI332430, SGI332431, SGI332432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332433, SGI0, NULL};
const Segment * const SG77920[] = {SGI332434, SGI332435, SGI332436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332437, SGI0, NULL};
const Segment * const SG77921[] = {SGI332438, SGI332439, SGI332440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332441, SGI0, NULL};
const Segment * const SG77922[] = {SGI332442, SGI332443, SGI332444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332445, SGI0, NULL};
const Segment * const SG77923[] = {SGI332446, SGI332447, SGI332448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332449, SGI0, NULL};
const Segment * const SG77924[] = {SGI332450, SGI332451, SGI332452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332453, SGI0, NULL};
const Segment * const SG77925[] = {SGI332454, SGI332455, SGI332456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332457, SGI0, NULL};
const Segment * const SG77926[] = {SGI332458, SGI332459, SGI332460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332461, SGI0, NULL};
const Segment * const SG77927[] = {SGI332462, SGI332463, SGI332464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332465, SGI0, NULL};
const Segment * const SG77928[] = {SGI332466, SGI332467, SGI332468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332469, SGI0, NULL};
const Segment * const SG77929[] = {SGI332470, SGI332471, SGI332472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332473, SGI0, NULL};
const Segment * const SG77930[] = {SGI332474, SGI332475, SGI332476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332477, SGI0, NULL};
const Segment * const SG77931[] = {SGI332478, SGI332479, SGI332480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332481, SGI0, NULL};
const Segment * const SG77932[] = {SGI332482, SGI332483, SGI332484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332485, SGI0, NULL};
const Segment * const SG77933[] = {SGI332486, SGI332487, SGI332488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332489, SGI0, NULL};
const Segment * const SG77934[] = {SGI332490, SGI332491, SGI332492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332493, SGI0, NULL};
const Segment * const SG77935[] = {SGI332494, SGI332495, SGI332496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332497, SGI0, NULL};
const Segment * const SG77936[] = {SGI332498, SGI332499, SGI332500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332501, SGI0, NULL};
const Segment * const SG77937[] = {SGI332502, SGI332503, SGI332504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332505, SGI0, NULL};
const Segment * const SG77938[] = {SGI332506, SGI332507, SGI332508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332509, SGI0, NULL};
const Segment * const SG77939[] = {SGI332510, SGI332511, SGI332512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332513, SGI0, NULL};
const Segment * const SG77940[] = {SGI332514, SGI332515, SGI332516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332517, SGI0, NULL};
const Segment * const SG77941[] = {SGI332518, SGI332519, SGI332520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332521, SGI0, NULL};
const Segment * const SG77942[] = {SGI332522, SGI332523, SGI332524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332525, SGI0, NULL};
const Segment * const SG77943[] = {SGI332526, SGI332527, SGI332528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332529, SGI0, NULL};
const Segment * const SG77944[] = {SGI332530, SGI332531, SGI332532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332533, SGI0, NULL};
const Segment * const SG77945[] = {SGI332534, SGI332535, SGI332536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332537, SGI0, NULL};
const Segment * const SG77946[] = {SGI332538, SGI332539, SGI332540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332541, SGI0, NULL};
const Segment * const SG77947[] = {SGI332542, SGI332543, SGI332544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332545, SGI0, NULL};
const Segment * const SG77948[] = {SGI332546, SGI332547, SGI332548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332549, SGI0, NULL};
const Segment * const SG77949[] = {SGI332550, SGI332551, SGI332552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332553, SGI0, NULL};
const Segment * const SG77950[] = {SGI332554, SGI332555, SGI332556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332557, SGI0, NULL};
const Segment * const SG77951[] = {SGI332558, SGI332559, SGI332560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332561, SGI0, NULL};
const Segment * const SG77952[] = {SGI332562, SGI332563, SGI332564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332565, SGI0, NULL};
const Segment * const SG77953[] = {SGI332566, SGI332567, SGI332568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332569, SGI0, NULL};
const Segment * const SG77954[] = {SGI332570, SGI332571, SGI332572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332573, SGI0, NULL};
const Segment * const SG77955[] = {SGI332574, SGI332575, SGI332576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332577, SGI0, NULL};
const Segment * const SG77956[] = {SGI332578, SGI332579, SGI332580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332581, SGI0, NULL};
const Segment * const SG77957[] = {SGI332582, SGI332583, SGI332584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332585, SGI0, NULL};
const Segment * const SG77958[] = {SGI332586, SGI332587, SGI332588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332589, SGI0, NULL};
const Segment * const SG77959[] = {SGI332590, SGI332591, SGI332592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332593, SGI0, NULL};
const Segment * const SG77960[] = {SGI332594, SGI332595, SGI332596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332597, SGI0, NULL};
const Segment * const SG77961[] = {SGI332598, SGI332599, SGI332600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332601, SGI0, NULL};
const Segment * const SG77962[] = {SGI332602, SGI332603, SGI332604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332605, SGI0, NULL};
const Segment * const SG77963[] = {SGI332606, SGI332607, SGI332608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332609, SGI0, NULL};
const Segment * const SG77964[] = {SGI332610, SGI332611, SGI332612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332613, SGI0, NULL};
const Segment * const SG77965[] = {SGI332614, SGI332615, SGI332616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332617, SGI0, NULL};
const Segment * const SG77966[] = {SGI332618, SGI332619, SGI332620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332621, SGI0, NULL};
const Segment * const SG77967[] = {SGI332622, SGI332623, SGI332624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332625, SGI0, NULL};
const Segment * const SG77968[] = {SGI332626, SGI332627, SGI332628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332629, SGI0, NULL};
const Segment * const SG77969[] = {SGI332630, SGI332631, SGI332632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332633, SGI0, NULL};
const Segment * const SG77970[] = {SGI332634, SGI332635, SGI332636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332637, SGI0, NULL};
const Segment * const SG77971[] = {SGI332638, SGI332639, SGI332640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332641, SGI0, NULL};
const Segment * const SG77972[] = {SGI332642, SGI332643, SGI332644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332645, SGI0, NULL};
const Segment * const SG77973[] = {SGI332646, SGI332647, SGI332648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332649, SGI0, NULL};
const Segment * const SG77974[] = {SGI332650, SGI332651, SGI332652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332653, SGI0, NULL};
const Segment * const SG77975[] = {SGI332654, SGI332655, SGI332656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332657, SGI0, NULL};
const Segment * const SG77976[] = {SGI332658, SGI332659, SGI332660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332661, SGI0, NULL};
const Segment * const SG77977[] = {SGI332662, SGI332663, SGI332664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332665, SGI0, NULL};
const Segment * const SG77978[] = {SGI332666, SGI332667, SGI332668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332669, SGI0, NULL};
const Segment * const SG77979[] = {SGI332670, SGI332671, SGI332672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332673, SGI0, NULL};
const Segment * const SG77980[] = {SGI332674, SGI332675, SGI332676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332677, SGI0, NULL};
const Segment * const SG77981[] = {SGI332678, SGI332679, SGI332680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332681, SGI0, NULL};
const Segment * const SG77982[] = {SGI332682, SGI332683, SGI332684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332685, SGI0, NULL};
const Segment * const SG77983[] = {SGI332686, SGI332687, SGI332688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332688, SGI0, NULL};
const Segment * const SG77984[] = {SGI332689, SGI332690, SGI332691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332691, SGI0, NULL};
const Segment * const SG77985[] = {SGI332692, SGI332693, SGI332694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332695, SGI0, NULL};
const Segment * const SG77986[] = {SGI332696, SGI332697, SGI332698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332699, SGI0, NULL};
const Segment * const SG77987[] = {SGI332700, SGI332701, SGI332702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332703, SGI0, NULL};
const Segment * const SG77988[] = {SGI332704, SGI332705, SGI332706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332707, SGI0, NULL};
const Segment * const SG77989[] = {SGI332708, SGI332709, SGI332710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332711, SGI0, NULL};
const Segment * const SG77990[] = {SGI332712, SGI332713, SGI332714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332715, SGI0, NULL};
const Segment * const SG77991[] = {SGI332716, SGI332717, SGI332718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332719, SGI0, NULL};
const Segment * const SG77992[] = {SGI332720, SGI332721, SGI332722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332723, SGI0, NULL};
const Segment * const SG77993[] = {SGI332724, SGI332725, SGI332726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332727, SGI0, NULL};
const Segment * const SG77994[] = {SGI332728, SGI332729, SGI332730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332731, SGI0, NULL};
const Segment * const SG77995[] = {SGI332732, SGI332733, SGI332734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332735, SGI0, NULL};
const Segment * const SG77996[] = {SGI332736, SGI332737, SGI332738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332739, SGI0, NULL};
const Segment * const SG77997[] = {SGI332740, SGI332741, SGI332742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332743, SGI0, NULL};
const Segment * const SG77998[] = {SGI332744, SGI332745, SGI332746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332747, SGI0, NULL};
const Segment * const SG77999[] = {SGI332748, SGI332749, SGI332750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332751, SGI0, NULL};
const Segment * const SG78000[] = {SGI332752, SGI332753, SGI332754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332755, SGI0, NULL};
const Segment * const SG78001[] = {SGI332756, SGI332757, SGI332758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332759, SGI0, NULL};
const Segment * const SG78002[] = {SGI332760, SGI332761, SGI332762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332763, SGI0, NULL};
const Segment * const SG78003[] = {SGI332764, SGI332765, SGI332766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332767, SGI0, NULL};
const Segment * const SG78004[] = {SGI332768, SGI332769, SGI332770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332771, SGI0, NULL};
const Segment * const SG78005[] = {SGI332772, SGI332773, SGI332774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332775, SGI0, NULL};
const Segment * const SG78006[] = {SGI332776, SGI332777, SGI332778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332779, SGI0, NULL};
const Segment * const SG78007[] = {SGI332780, SGI332781, SGI332782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332783, SGI0, NULL};
const Segment * const SG78008[] = {SGI332784, SGI332785, SGI332786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332787, SGI0, NULL};
const Segment * const SG78009[] = {SGI332788, SGI332789, SGI332790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332791, SGI0, NULL};
const Segment * const SG78010[] = {SGI332792, SGI332793, SGI332794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332795, SGI0, NULL};
const Segment * const SG78011[] = {SGI332796, SGI332797, SGI332798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332799, SGI0, NULL};
const Segment * const SG78012[] = {SGI332800, SGI332801, SGI332802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332803, SGI0, NULL};
const Segment * const SG78013[] = {SGI332804, SGI332805, SGI332806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332807, SGI0, NULL};
const Segment * const SG78014[] = {SGI332808, SGI332809, SGI332810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332811, SGI0, NULL};
const Segment * const SG78015[] = {SGI332812, SGI332813, SGI332814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332815, SGI0, NULL};
const Segment * const SG78016[] = {SGI332816, SGI332817, SGI332818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332819, SGI0, NULL};
const Segment * const SG78017[] = {SGI332820, SGI332821, SGI332822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332823, SGI0, NULL};
const Segment * const SG78018[] = {SGI332824, SGI332825, SGI332826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332827, SGI0, NULL};
const Segment * const SG78019[] = {SGI332828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78020[] = {SGI332829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78021[] = {SGI332830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78022[] = {SGI332831, SGI332832, SGI0, SGI0, NULL};
const Segment * const SG78023[] = {SGI332833, SGI332834, SGI0, SGI0, NULL};
const Segment * const SG78024[] = {SGI0, SGI332835, SGI332836, SGI0, SGI0, SGI0, SGI332837, NULL};
const Segment * const SG78025[] = {SGI0, SGI332838, SGI332839, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78026[] = {SGI0, SGI332840, SGI332841, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78027[] = {SGI0, SGI332842, SGI332843, SGI0, SGI0, SGI0, SGI332844, NULL};
const Segment * const SG78028[] = {SGI0, SGI332845, SGI332846, SGI0, SGI0, SGI0, SGI332847, NULL};
const Segment * const SG78029[] = {SGI332848, SGI332849, SGI332850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332851, SGI0, NULL};
const Segment * const SG78030[] = {SGI332852, SGI332853, SGI332854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332854, SGI0, NULL};
const Segment * const SG78031[] = {SGI332855, SGI332856, SGI332857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332858, SGI0, NULL};
const Segment * const SG78032[] = {SGI332859, SGI332860, SGI332861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332862, SGI0, NULL};
const Segment * const SG78033[] = {SGI332863, SGI332864, SGI332865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332866, SGI0, NULL};
const Segment * const SG78034[] = {SGI332867, SGI332868, SGI332869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332870, SGI0, NULL};
const Segment * const SG78035[] = {SGI332871, SGI332872, SGI332873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332874, SGI0, NULL};
const Segment * const SG78036[] = {SGI332875, SGI332876, SGI332877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332878, SGI0, NULL};
const Segment * const SG78037[] = {SGI332879, SGI332880, SGI332881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332882, SGI0, NULL};
const Segment * const SG78038[] = {SGI332883, SGI332884, SGI332885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332886, SGI0, NULL};
const Segment * const SG78039[] = {SGI332887, SGI332888, SGI332889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332890, SGI0, NULL};
const Segment * const SG78040[] = {SGI332891, SGI332892, SGI332893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332894, SGI0, NULL};
const Segment * const SG78041[] = {SGI332895, SGI332896, SGI332897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332898, SGI0, NULL};
const Segment * const SG78042[] = {SGI332899, SGI332900, SGI332901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332902, SGI0, NULL};
const Segment * const SG78043[] = {SGI332903, SGI332904, SGI332905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332906, SGI0, NULL};
const Segment * const SG78044[] = {SGI332907, SGI332908, SGI332909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332910, SGI0, NULL};
const Segment * const SG78045[] = {SGI332911, SGI332912, SGI332913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332914, SGI0, NULL};
const Segment * const SG78046[] = {SGI332915, SGI332916, SGI332917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332918, SGI0, NULL};
const Segment * const SG78047[] = {SGI332919, SGI332920, SGI332921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332922, SGI0, NULL};
const Segment * const SG78048[] = {SGI332923, SGI332924, SGI332925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332926, SGI0, NULL};
const Segment * const SG78049[] = {SGI332927, SGI332928, SGI332929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332930, SGI0, NULL};
const Segment * const SG78050[] = {SGI332931, SGI332932, SGI332933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332934, SGI0, NULL};
const Segment * const SG78051[] = {SGI332935, SGI332936, SGI332937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332938, SGI0, NULL};
const Segment * const SG78052[] = {SGI332939, SGI332940, SGI332941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332942, SGI0, NULL};
const Segment * const SG78053[] = {SGI332943, SGI332944, SGI332945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332946, SGI0, NULL};
const Segment * const SG78054[] = {SGI332947, SGI332948, SGI332949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332950, SGI0, NULL};
const Segment * const SG78055[] = {SGI332951, SGI332952, SGI332953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332954, SGI0, NULL};
const Segment * const SG78056[] = {SGI332955, SGI332956, SGI332957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332958, SGI0, NULL};
const Segment * const SG78057[] = {SGI332959, SGI332960, SGI332961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332962, SGI0, NULL};
const Segment * const SG78058[] = {SGI332963, SGI332964, SGI332965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332966, SGI0, NULL};
const Segment * const SG78059[] = {SGI332967, SGI332968, SGI332969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332970, SGI0, NULL};
const Segment * const SG78060[] = {SGI332971, SGI332972, SGI332973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332974, SGI0, NULL};
const Segment * const SG78061[] = {SGI332975, SGI332976, SGI332977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332978, SGI0, NULL};
const Segment * const SG78062[] = {SGI332979, SGI332980, SGI332981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332982, SGI0, NULL};
const Segment * const SG78063[] = {SGI332983, SGI332984, SGI332985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332986, SGI0, NULL};
const Segment * const SG78064[] = {SGI332987, SGI332988, SGI332989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332990, SGI0, NULL};
const Segment * const SG78065[] = {SGI332991, SGI332992, SGI332993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332994, SGI0, NULL};
const Segment * const SG78066[] = {SGI332995, SGI332996, SGI332997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI332998, SGI0, NULL};
const Segment * const SG78067[] = {SGI332999, SGI333000, SGI333001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333002, SGI0, NULL};
const Segment * const SG78068[] = {SGI333003, SGI333004, SGI333005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333006, SGI0, NULL};
const Segment * const SG78069[] = {SGI333007, SGI333008, SGI333009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333010, SGI0, NULL};
const Segment * const SG78070[] = {SGI333011, SGI333012, SGI333013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333014, SGI0, NULL};
const Segment * const SG78071[] = {SGI333015, SGI333016, SGI333017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333018, SGI0, NULL};
const Segment * const SG78072[] = {SGI333019, SGI333020, SGI333021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333022, SGI0, NULL};
const Segment * const SG78073[] = {SGI333023, SGI333024, SGI333025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333026, SGI0, NULL};
const Segment * const SG78074[] = {SGI333027, SGI333028, SGI333029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333030, SGI0, NULL};
const Segment * const SG78075[] = {SGI333031, SGI333032, SGI333033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333034, SGI0, NULL};
const Segment * const SG78076[] = {SGI333035, SGI333036, SGI333037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333038, SGI0, NULL};
const Segment * const SG78077[] = {SGI333039, SGI333040, SGI333041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333042, SGI0, NULL};
const Segment * const SG78078[] = {SGI333043, SGI333044, SGI333045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333046, SGI0, NULL};
const Segment * const SG78079[] = {SGI333047, SGI333048, SGI333049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333050, SGI0, NULL};
const Segment * const SG78080[] = {SGI333051, SGI333052, SGI333053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333054, SGI0, NULL};
const Segment * const SG78081[] = {SGI333055, SGI333056, SGI333057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333058, SGI0, NULL};
const Segment * const SG78082[] = {SGI333059, SGI333060, SGI333061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333062, SGI0, NULL};
const Segment * const SG78083[] = {SGI333063, SGI333064, SGI333065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333066, SGI0, NULL};
const Segment * const SG78084[] = {SGI333067, SGI333068, SGI333069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333070, SGI0, NULL};
const Segment * const SG78085[] = {SGI333071, SGI333072, SGI333073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333074, SGI0, NULL};
const Segment * const SG78086[] = {SGI333075, SGI333076, SGI333077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333078, SGI0, NULL};
const Segment * const SG78087[] = {SGI333079, SGI333080, SGI333081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333082, SGI0, NULL};
const Segment * const SG78088[] = {SGI333083, SGI333084, SGI333085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333086, SGI0, NULL};
const Segment * const SG78089[] = {SGI333087, SGI333088, SGI333089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333090, SGI0, NULL};
const Segment * const SG78090[] = {SGI333091, SGI333092, SGI333093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333094, SGI0, NULL};
const Segment * const SG78091[] = {SGI333095, SGI333096, SGI333097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333098, SGI0, NULL};
const Segment * const SG78092[] = {SGI333099, SGI333100, SGI333101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333102, SGI0, NULL};
const Segment * const SG78093[] = {SGI333103, SGI333104, SGI333105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333106, SGI0, NULL};
const Segment * const SG78094[] = {SGI333107, SGI333108, SGI333109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333110, SGI0, NULL};
const Segment * const SG78095[] = {SGI333111, SGI333112, SGI333113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333114, SGI0, NULL};
const Segment * const SG78096[] = {SGI333115, SGI333116, SGI333117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333118, SGI0, NULL};
const Segment * const SG78097[] = {SGI333119, SGI333120, SGI333121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333122, SGI0, NULL};
const Segment * const SG78098[] = {SGI333123, SGI333124, SGI333125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333126, SGI0, NULL};
const Segment * const SG78099[] = {SGI333127, SGI333128, SGI333129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333130, SGI0, NULL};
const Segment * const SG78100[] = {SGI333131, SGI333132, SGI333133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333134, SGI0, NULL};
const Segment * const SG78101[] = {SGI333135, SGI333136, SGI333137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333138, SGI0, NULL};
const Segment * const SG78102[] = {SGI333139, SGI333140, SGI333141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333142, SGI0, NULL};
const Segment * const SG78103[] = {SGI333143, SGI333144, SGI333145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333146, SGI0, NULL};
const Segment * const SG78104[] = {SGI333147, SGI333148, SGI333149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333150, SGI0, NULL};
const Segment * const SG78105[] = {SGI333151, SGI333152, SGI333153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333154, SGI0, NULL};
const Segment * const SG78106[] = {SGI333155, SGI333156, SGI333157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333158, SGI0, NULL};
const Segment * const SG78107[] = {SGI333159, SGI333160, SGI333161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333162, SGI0, NULL};
const Segment * const SG78108[] = {SGI333163, SGI333164, SGI333165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333166, SGI0, NULL};
const Segment * const SG78109[] = {SGI333167, SGI333168, SGI333169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333170, SGI0, NULL};
const Segment * const SG78110[] = {SGI333171, SGI333172, SGI333173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333174, SGI0, NULL};
const Segment * const SG78111[] = {SGI333175, SGI333176, SGI333177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333178, SGI0, NULL};
const Segment * const SG78112[] = {SGI333179, SGI333180, SGI333181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333182, SGI0, NULL};
const Segment * const SG78113[] = {SGI333183, SGI333184, SGI333185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333186, SGI0, NULL};
const Segment * const SG78114[] = {SGI333187, SGI333188, SGI333189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333190, SGI0, NULL};
const Segment * const SG78115[] = {SGI333191, SGI333192, SGI333193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333194, SGI0, NULL};
const Segment * const SG78116[] = {SGI333195, SGI333196, SGI333197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333198, SGI0, NULL};
const Segment * const SG78117[] = {SGI333199, SGI333200, SGI333201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333202, SGI0, NULL};
const Segment * const SG78118[] = {SGI333203, SGI333204, SGI333205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333206, SGI0, NULL};
const Segment * const SG78119[] = {SGI333207, SGI333208, SGI333209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333210, SGI0, NULL};
const Segment * const SG78120[] = {SGI333211, SGI333212, SGI333213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333214, SGI0, NULL};
const Segment * const SG78121[] = {SGI333215, SGI333216, SGI333217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333218, SGI0, NULL};
const Segment * const SG78122[] = {SGI333219, SGI333220, SGI333221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333222, SGI0, NULL};
const Segment * const SG78123[] = {SGI333223, SGI333224, SGI333225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333226, SGI0, NULL};
const Segment * const SG78124[] = {SGI333227, SGI333228, SGI333229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333230, SGI0, NULL};
const Segment * const SG78125[] = {SGI333231, SGI333232, SGI333233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333234, SGI0, NULL};
const Segment * const SG78126[] = {SGI333235, SGI333236, SGI333237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333238, SGI0, NULL};
const Segment * const SG78127[] = {SGI333239, SGI333240, SGI333241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333242, SGI0, NULL};
const Segment * const SG78128[] = {SGI333243, SGI333244, SGI333245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333246, SGI0, NULL};
const Segment * const SG78129[] = {SGI333247, SGI333248, SGI333249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333250, SGI0, NULL};
const Segment * const SG78130[] = {SGI333251, SGI333252, SGI333253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333254, SGI0, NULL};
const Segment * const SG78131[] = {SGI333255, SGI333256, SGI333257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333258, SGI0, NULL};
const Segment * const SG78132[] = {SGI333259, SGI333260, SGI333261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333262, SGI0, NULL};
const Segment * const SG78133[] = {SGI333263, SGI333264, SGI333265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333266, SGI0, NULL};
const Segment * const SG78134[] = {SGI333267, SGI333268, SGI333269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333270, SGI0, NULL};
const Segment * const SG78135[] = {SGI333271, SGI333272, SGI333273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333274, SGI0, NULL};
const Segment * const SG78136[] = {SGI333275, SGI333276, SGI333277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333278, SGI0, NULL};
const Segment * const SG78137[] = {SGI333279, SGI333280, SGI333281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333282, SGI0, NULL};
const Segment * const SG78138[] = {SGI333283, SGI333284, SGI333285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333286, SGI0, NULL};
const Segment * const SG78139[] = {SGI333287, SGI333288, SGI333289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333290, SGI0, NULL};
const Segment * const SG78140[] = {SGI333291, SGI333292, SGI333293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333294, SGI0, NULL};
const Segment * const SG78141[] = {SGI333295, SGI333296, SGI333297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333298, SGI0, NULL};
const Segment * const SG78142[] = {SGI333299, SGI333300, SGI333301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333302, SGI0, NULL};
const Segment * const SG78143[] = {SGI333303, SGI333304, SGI333305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333306, SGI0, NULL};
const Segment * const SG78144[] = {SGI333307, SGI333308, SGI333309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333310, SGI0, NULL};
const Segment * const SG78145[] = {SGI333311, SGI333312, SGI333313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333314, SGI0, NULL};
const Segment * const SG78146[] = {SGI333315, SGI333316, SGI333317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333318, SGI0, NULL};
const Segment * const SG78147[] = {SGI333319, SGI333320, SGI333321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333322, SGI0, NULL};
const Segment * const SG78148[] = {SGI333323, SGI333324, SGI333325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333326, SGI0, NULL};
const Segment * const SG78149[] = {SGI333327, SGI333328, SGI333329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333330, SGI0, NULL};
const Segment * const SG78150[] = {SGI333331, SGI333332, SGI333333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333334, SGI0, NULL};
const Segment * const SG78151[] = {SGI333335, SGI333336, SGI333337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333338, SGI0, NULL};
const Segment * const SG78152[] = {SGI333339, SGI333340, SGI333341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333342, SGI0, NULL};
const Segment * const SG78153[] = {SGI333343, SGI333344, SGI333345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333346, SGI0, NULL};
const Segment * const SG78154[] = {SGI333347, SGI333348, SGI333349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333350, SGI0, NULL};
const Segment * const SG78155[] = {SGI333351, SGI333352, SGI333353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333354, SGI0, NULL};
const Segment * const SG78156[] = {SGI333355, SGI333356, SGI333357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333358, SGI0, NULL};
const Segment * const SG78157[] = {SGI333359, SGI333360, SGI333361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333362, SGI0, NULL};
const Segment * const SG78158[] = {SGI333363, SGI333364, SGI333365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333366, SGI0, NULL};
const Segment * const SG78159[] = {SGI333367, SGI333368, SGI333369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333370, SGI0, NULL};
const Segment * const SG78160[] = {SGI333371, SGI333372, SGI333373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333374, SGI0, NULL};
const Segment * const SG78161[] = {SGI333375, SGI333376, SGI333377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333378, SGI0, NULL};
const Segment * const SG78162[] = {SGI333379, SGI333380, SGI333381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333382, SGI0, NULL};
const Segment * const SG78163[] = {SGI333383, SGI333384, SGI333385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333386, SGI0, NULL};
const Segment * const SG78164[] = {SGI333387, SGI333388, SGI333389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333390, SGI0, NULL};
const Segment * const SG78165[] = {SGI333391, SGI333392, SGI333393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333394, SGI0, NULL};
const Segment * const SG78166[] = {SGI333395, SGI333396, SGI333397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333398, SGI0, NULL};
const Segment * const SG78167[] = {SGI333399, SGI333400, SGI333401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333402, SGI0, NULL};
const Segment * const SG78168[] = {SGI333403, SGI333404, SGI333405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333406, SGI0, NULL};
const Segment * const SG78169[] = {SGI333407, SGI333408, SGI333409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333410, SGI0, NULL};
const Segment * const SG78170[] = {SGI333411, SGI333412, SGI333413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333414, SGI0, NULL};
const Segment * const SG78171[] = {SGI333415, SGI333416, SGI333417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333418, SGI0, NULL};
const Segment * const SG78172[] = {SGI333419, SGI333420, SGI333421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333422, SGI0, NULL};
const Segment * const SG78173[] = {SGI333423, SGI333424, SGI333425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333426, SGI0, NULL};
const Segment * const SG78174[] = {SGI333427, SGI333428, SGI333429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333430, SGI0, NULL};
const Segment * const SG78175[] = {SGI333431, SGI333432, SGI333433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333434, SGI0, NULL};
const Segment * const SG78176[] = {SGI333435, SGI333436, SGI333437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333438, SGI0, NULL};
const Segment * const SG78177[] = {SGI333439, SGI333440, SGI333441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333442, SGI0, NULL};
const Segment * const SG78178[] = {SGI333443, SGI333444, SGI333445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333446, SGI0, NULL};
const Segment * const SG78179[] = {SGI333447, SGI333448, SGI333449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333450, SGI0, NULL};
const Segment * const SG78180[] = {SGI333451, SGI333452, SGI333453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333454, SGI0, NULL};
const Segment * const SG78181[] = {SGI333455, SGI333456, SGI333457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333458, SGI0, NULL};
const Segment * const SG78182[] = {SGI333459, SGI333460, SGI333461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333462, SGI0, NULL};
const Segment * const SG78183[] = {SGI333463, SGI333464, SGI333465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333465, SGI0, NULL};
const Segment * const SG78184[] = {SGI333466, SGI333467, SGI333468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333469, SGI0, NULL};
const Segment * const SG78185[] = {SGI333470, SGI333471, SGI333472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333473, SGI0, NULL};
const Segment * const SG78186[] = {SGI333476, SGI333478, SGI0, SGI0, NULL};
const Segment * const SG78187[] = {SGI333474, SGI333475, SGI333477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333479, SGI0, NULL};
const Segment * const SG78188[] = {SGI333480, SGI333481, SGI333482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333483, SGI0, NULL};
const Segment * const SG78189[] = {SGI333484, SGI333485, SGI333486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333487, SGI0, NULL};
const Segment * const SG78190[] = {SGI333488, SGI333489, SGI333490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333491, SGI0, NULL};
const Segment * const SG78191[] = {SGI333492, SGI333493, SGI333494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333495, SGI0, NULL};
const Segment * const SG78192[] = {SGI333496, SGI333497, SGI333498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333499, SGI0, NULL};
const Segment * const SG78193[] = {SGI333500, SGI333501, SGI333502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333503, SGI0, NULL};
const Segment * const SG78194[] = {SGI333504, SGI333505, SGI333506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333507, SGI0, NULL};
const Segment * const SG78195[] = {SGI333508, SGI333509, SGI333510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333511, SGI0, NULL};
const Segment * const SG78196[] = {SGI333512, SGI333513, SGI333514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333515, SGI0, NULL};
const Segment * const SG78197[] = {SGI333516, SGI333517, SGI333518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333519, SGI0, NULL};
const Segment * const SG78198[] = {SGI333520, SGI333521, SGI333522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333522, SGI0, NULL};
const Segment * const SG78199[] = {SGI333523, SGI333524, SGI333525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333526, SGI0, NULL};
const Segment * const SG78200[] = {SGI333527, SGI333528, SGI333529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333530, SGI0, NULL};
const Segment * const SG78201[] = {SGI333531, SGI333532, SGI333533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333534, SGI0, NULL};
const Segment * const SG78202[] = {SGI333535, SGI333536, SGI333537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333538, SGI0, NULL};
const Segment * const SG78203[] = {SGI333539, SGI333540, SGI333541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333542, SGI0, NULL};
const Segment * const SG78204[] = {SGI333543, SGI333544, SGI333545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333546, SGI0, NULL};
const Segment * const SG78205[] = {SGI333547, SGI333548, SGI333549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333550, SGI0, NULL};
const Segment * const SG78206[] = {SGI333551, SGI333552, SGI333553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333554, SGI0, NULL};
const Segment * const SG78207[] = {SGI333555, SGI333556, SGI333557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333558, SGI0, NULL};
const Segment * const SG78208[] = {SGI333559, SGI333560, SGI333561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333562, SGI0, NULL};
const Segment * const SG78209[] = {SGI333563, SGI333564, SGI333565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333566, SGI0, NULL};
const Segment * const SG78210[] = {SGI333567, SGI333568, SGI333569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333570, SGI0, NULL};
const Segment * const SG78211[] = {SGI333571, SGI333572, SGI333573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333574, SGI0, NULL};
const Segment * const SG78212[] = {SGI333575, SGI333576, SGI333577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333578, SGI0, NULL};
const Segment * const SG78213[] = {SGI333579, SGI333580, SGI333581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333582, SGI0, NULL};
const Segment * const SG78214[] = {SGI333583, SGI333584, SGI333585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333586, SGI0, NULL};
const Segment * const SG78215[] = {SGI333587, SGI333588, SGI333589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333590, SGI0, NULL};
const Segment * const SG78216[] = {SGI333591, SGI333592, SGI333593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333594, SGI0, NULL};
const Segment * const SG78217[] = {SGI333595, SGI333596, SGI333597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333598, SGI0, NULL};
const Segment * const SG78218[] = {SGI333599, SGI333600, SGI333601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333602, SGI0, NULL};
const Segment * const SG78219[] = {SGI333603, SGI333604, SGI333605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333606, SGI0, NULL};
const Segment * const SG78220[] = {SGI333607, SGI333608, SGI333609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333610, SGI0, NULL};
const Segment * const SG78221[] = {SGI333611, SGI333612, SGI333613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333614, SGI0, NULL};
const Segment * const SG78222[] = {SGI333615, SGI333616, SGI333617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333618, SGI0, NULL};
const Segment * const SG78223[] = {SGI333619, SGI333620, SGI333621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333622, SGI0, NULL};
const Segment * const SG78224[] = {SGI333623, SGI333624, SGI333625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333626, SGI0, NULL};
const Segment * const SG78225[] = {SGI333627, SGI333628, SGI333629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333630, SGI0, NULL};
const Segment * const SG78226[] = {SGI333631, SGI333632, SGI333633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333634, SGI0, NULL};
const Segment * const SG78227[] = {SGI333635, SGI333636, SGI333637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333638, SGI0, NULL};
const Segment * const SG78228[] = {SGI333639, SGI333640, SGI333641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333642, SGI0, NULL};
const Segment * const SG78229[] = {SGI333643, SGI333644, SGI333645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333646, SGI0, NULL};
const Segment * const SG78230[] = {SGI333647, SGI333648, SGI333649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333650, SGI0, NULL};
const Segment * const SG78231[] = {SGI333651, SGI333652, SGI333653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333654, SGI0, NULL};
const Segment * const SG78232[] = {SGI333655, SGI333656, SGI333657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333658, SGI0, NULL};
const Segment * const SG78233[] = {SGI333659, SGI333660, SGI333661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333662, SGI0, NULL};
const Segment * const SG78234[] = {SGI333663, SGI333664, SGI333665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333666, SGI0, NULL};
const Segment * const SG78235[] = {SGI333667, SGI333668, SGI333669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333670, SGI0, NULL};
const Segment * const SG78236[] = {SGI333671, SGI333672, SGI333673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333674, SGI0, NULL};
const Segment * const SG78237[] = {SGI333675, SGI333676, SGI333677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333678, SGI0, NULL};
const Segment * const SG78238[] = {SGI333679, SGI333680, SGI333681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333682, SGI0, NULL};
const Segment * const SG78239[] = {SGI333683, SGI333684, SGI333685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333686, SGI0, NULL};
const Segment * const SG78240[] = {SGI333687, SGI333688, SGI333689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333690, SGI0, NULL};
const Segment * const SG78241[] = {SGI333691, SGI333692, SGI333693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333694, SGI0, NULL};
const Segment * const SG78242[] = {SGI333695, SGI333696, SGI333697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333698, SGI0, NULL};
const Segment * const SG78243[] = {SGI333699, SGI333700, SGI333701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333702, SGI0, NULL};
const Segment * const SG78244[] = {SGI333703, SGI333704, SGI333705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333706, SGI0, NULL};
const Segment * const SG78245[] = {SGI333707, SGI333708, SGI333709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333710, SGI0, NULL};
const Segment * const SG78246[] = {SGI333711, SGI333712, SGI333713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333714, SGI0, NULL};
const Segment * const SG78247[] = {SGI333715, SGI333716, SGI333717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333718, SGI0, NULL};
const Segment * const SG78248[] = {SGI333719, SGI333720, SGI333721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333722, SGI0, NULL};
const Segment * const SG78249[] = {SGI333723, SGI333724, SGI333725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333726, SGI0, NULL};
const Segment * const SG78250[] = {SGI333727, SGI333728, SGI333729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333730, SGI0, NULL};
const Segment * const SG78251[] = {SGI333731, SGI333732, SGI333733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333734, SGI0, NULL};
const Segment * const SG78252[] = {SGI333735, SGI333736, SGI333737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333738, SGI0, NULL};
const Segment * const SG78253[] = {SGI333739, SGI333740, SGI333741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333742, SGI0, NULL};
const Segment * const SG78254[] = {SGI333743, SGI333744, SGI333745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333746, SGI0, NULL};
const Segment * const SG78255[] = {SGI333747, SGI333748, SGI333749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333750, SGI0, NULL};
const Segment * const SG78256[] = {SGI333751, SGI333752, SGI333753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333754, SGI0, NULL};
const Segment * const SG78257[] = {SGI333755, SGI333756, SGI333757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333758, SGI0, NULL};
const Segment * const SG78258[] = {SGI333759, SGI333760, SGI333761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333762, SGI0, NULL};
const Segment * const SG78259[] = {SGI333763, SGI333764, SGI333765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333766, SGI0, NULL};
const Segment * const SG78260[] = {SGI333767, SGI333768, SGI333769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333770, SGI0, NULL};
const Segment * const SG78261[] = {SGI333771, SGI333772, SGI333773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333774, SGI0, NULL};
const Segment * const SG78262[] = {SGI333775, SGI333776, SGI333777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333778, SGI0, NULL};
const Segment * const SG78263[] = {SGI333779, SGI333780, SGI333781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333782, SGI0, NULL};
const Segment * const SG78264[] = {SGI333783, SGI333784, SGI333785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333786, SGI0, NULL};
const Segment * const SG78265[] = {SGI333787, SGI333788, SGI333789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333790, SGI0, NULL};
const Segment * const SG78266[] = {SGI333791, SGI333792, SGI333793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333794, SGI0, NULL};
const Segment * const SG78267[] = {SGI333795, SGI333796, SGI333797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333798, SGI0, NULL};
const Segment * const SG78268[] = {SGI333799, SGI333800, SGI333801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333802, SGI0, NULL};
const Segment * const SG78269[] = {SGI333803, SGI333804, SGI333805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333806, SGI0, NULL};
const Segment * const SG78270[] = {SGI333807, SGI333808, SGI333809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333810, SGI0, NULL};
const Segment * const SG78271[] = {SGI333811, SGI333812, SGI333813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333814, SGI0, NULL};
const Segment * const SG78272[] = {SGI333815, SGI333816, SGI333817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333817, SGI0, NULL};
const Segment * const SG78273[] = {SGI333818, SGI333819, SGI333820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333821, SGI0, NULL};
const Segment * const SG78274[] = {SGI333822, SGI333823, SGI333824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333825, SGI0, NULL};
const Segment * const SG78275[] = {SGI333826, SGI333827, SGI333828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333829, SGI0, NULL};
const Segment * const SG78276[] = {SGI333830, SGI333831, SGI333832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333833, SGI0, NULL};
const Segment * const SG78277[] = {SGI333834, SGI333835, SGI333836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333837, SGI0, NULL};
const Segment * const SG78278[] = {SGI333838, SGI333839, SGI333840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333841, SGI0, NULL};
const Segment * const SG78279[] = {SGI333842, SGI333843, SGI333844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333845, SGI0, NULL};
const Segment * const SG78280[] = {SGI333846, SGI333847, SGI333848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333849, SGI0, NULL};
const Segment * const SG78281[] = {SGI333850, SGI333851, SGI333852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333853, SGI0, NULL};
const Segment * const SG78282[] = {SGI333854, SGI333855, SGI333856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333857, SGI0, NULL};
const Segment * const SG78283[] = {SGI333858, SGI333859, SGI333860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333861, SGI0, NULL};
const Segment * const SG78284[] = {SGI333862, SGI333863, SGI333864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333865, SGI0, NULL};
const Segment * const SG78285[] = {SGI333866, SGI333867, SGI333868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333869, SGI0, NULL};
const Segment * const SG78286[] = {SGI333870, SGI333871, SGI333872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333873, SGI0, NULL};
const Segment * const SG78287[] = {SGI333874, SGI333875, SGI333876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333877, SGI0, NULL};
const Segment * const SG78288[] = {SGI333878, SGI333879, SGI333880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333881, SGI0, NULL};
const Segment * const SG78289[] = {SGI333882, SGI333883, SGI333884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333885, SGI0, NULL};
const Segment * const SG78290[] = {SGI333886, SGI333887, SGI333888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333889, SGI0, NULL};
const Segment * const SG78291[] = {SGI333890, SGI333891, SGI333892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333893, SGI0, NULL};
const Segment * const SG78292[] = {SGI333894, SGI333895, SGI333896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333897, SGI0, NULL};
const Segment * const SG78293[] = {SGI333898, SGI333899, SGI333900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333901, SGI0, NULL};
const Segment * const SG78294[] = {SGI333902, SGI333903, SGI333904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333905, SGI0, NULL};
const Segment * const SG78295[] = {SGI333906, SGI333907, SGI333908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333909, SGI0, NULL};
const Segment * const SG78296[] = {SGI333910, SGI333911, SGI333912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333913, SGI0, NULL};
const Segment * const SG78297[] = {SGI333914, SGI333915, SGI333916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333917, SGI0, NULL};
const Segment * const SG78298[] = {SGI333918, SGI333919, SGI333920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333921, SGI0, NULL};
const Segment * const SG78299[] = {SGI333922, SGI333923, SGI333924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333925, SGI0, NULL};
const Segment * const SG78300[] = {SGI333926, SGI333927, SGI333928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333929, SGI0, NULL};
const Segment * const SG78301[] = {SGI333930, SGI333931, SGI333932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333933, SGI0, NULL};
const Segment * const SG78302[] = {SGI333934, SGI333935, SGI333936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333937, SGI0, NULL};
const Segment * const SG78303[] = {SGI333938, SGI333939, SGI333940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333941, SGI0, NULL};
const Segment * const SG78304[] = {SGI333942, SGI333943, SGI333944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333945, SGI0, NULL};
const Segment * const SG78305[] = {SGI333946, SGI333947, SGI333948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333949, SGI0, NULL};
const Segment * const SG78306[] = {SGI333950, SGI333951, SGI333952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333953, SGI0, NULL};
const Segment * const SG78307[] = {SGI333954, SGI333955, SGI333956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333957, SGI0, NULL};
const Segment * const SG78308[] = {SGI333958, SGI333959, SGI333960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333961, SGI0, NULL};
const Segment * const SG78309[] = {SGI333962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78310[] = {SGI333963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78311[] = {SGI333964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78312[] = {SGI333965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78313[] = {SGI333966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78314[] = {SGI333967, SGI333968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78315[] = {SGI333969, SGI333970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78316[] = {SGI333971, SGI333972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78317[] = {SGI333973, SGI333974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78318[] = {SGI333975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78319[] = {SGI333976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78320[] = {SGI333977, SGI333978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78321[] = {SGI333979, SGI333980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78322[] = {SGI333981, SGI333982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78323[] = {SGI333983, SGI333984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78324[] = {SGI333985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78325[] = {SGI333986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78326[] = {SGI333987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78327[] = {SGI333988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78328[] = {SGI333989, SGI333990, SGI333991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333991, SGI0, NULL};
const Segment * const SG78329[] = {SGI333992, SGI333993, SGI333994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333995, SGI0, NULL};
const Segment * const SG78330[] = {SGI333996, SGI333997, SGI333998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI333999, SGI0, NULL};
const Segment * const SG78331[] = {SGI334000, SGI334001, SGI334002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334003, SGI0, NULL};
const Segment * const SG78332[] = {SGI334004, SGI334005, SGI334006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334007, SGI0, NULL};
const Segment * const SG78333[] = {SGI334008, SGI334009, SGI334010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334011, SGI0, NULL};
const Segment * const SG78334[] = {SGI334012, SGI334013, SGI334014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334015, SGI0, NULL};
const Segment * const SG78335[] = {SGI334016, SGI334017, SGI334018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334019, SGI0, NULL};
const Segment * const SG78336[] = {SGI334020, SGI334021, SGI334022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334023, SGI0, NULL};
const Segment * const SG78337[] = {SGI334024, SGI334025, SGI334026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334027, SGI0, NULL};
const Segment * const SG78338[] = {SGI334028, SGI334029, SGI334030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334031, SGI0, NULL};
const Segment * const SG78339[] = {SGI334032, SGI334033, SGI334034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334035, SGI0, NULL};
const Segment * const SG78340[] = {SGI334036, SGI334037, SGI334038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334039, SGI0, NULL};
const Segment * const SG78341[] = {SGI334040, SGI334041, SGI334042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334043, SGI0, NULL};
const Segment * const SG78342[] = {SGI334044, SGI334045, SGI334046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334047, SGI0, NULL};
const Segment * const SG78343[] = {SGI334048, SGI334049, SGI334050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334051, SGI0, NULL};
const Segment * const SG78344[] = {SGI334052, SGI334053, SGI334054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334055, SGI0, NULL};
const Segment * const SG78345[] = {SGI334056, SGI334057, SGI334058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334059, SGI0, NULL};
const Segment * const SG78346[] = {SGI334060, SGI334061, SGI334062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334063, SGI0, NULL};
const Segment * const SG78347[] = {SGI334064, SGI334065, SGI334066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334067, SGI0, NULL};
const Segment * const SG78348[] = {SGI334068, SGI334069, SGI334070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334071, SGI0, NULL};
const Segment * const SG78349[] = {SGI334072, SGI334073, SGI334074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334075, SGI0, NULL};
const Segment * const SG78350[] = {SGI334076, SGI334077, SGI334078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334079, SGI0, NULL};
const Segment * const SG78351[] = {SGI334080, SGI334081, SGI334082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334083, SGI0, NULL};
const Segment * const SG78352[] = {SGI334084, SGI334085, SGI334086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334087, SGI0, NULL};
const Segment * const SG78353[] = {SGI334088, SGI334089, SGI334090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334091, SGI0, NULL};
const Segment * const SG78354[] = {SGI334092, SGI334093, SGI334094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334095, SGI0, NULL};
const Segment * const SG78355[] = {SGI334096, SGI334097, SGI334098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334099, SGI0, NULL};
const Segment * const SG78356[] = {SGI334100, SGI334101, SGI334102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334103, SGI0, NULL};
const Segment * const SG78357[] = {SGI334104, SGI334105, SGI334106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334107, SGI0, NULL};
const Segment * const SG78358[] = {SGI334108, SGI334109, SGI334110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334111, SGI0, NULL};
const Segment * const SG78359[] = {SGI334112, SGI334113, SGI334114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334115, SGI0, NULL};
const Segment * const SG78360[] = {SGI334116, SGI334117, SGI334118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334119, SGI0, NULL};
const Segment * const SG78361[] = {SGI334120, SGI334122, SGI334123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334124, SGI0, NULL};
const Segment * const SG78362[] = {SGI334121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78363[] = {SGI334126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78364[] = {SGI334125, SGI334127, SGI334128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334129, SGI0, NULL};
const Segment * const SG78365[] = {SGI334131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78366[] = {SGI334130, SGI334132, SGI334133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334134, SGI0, NULL};
const Segment * const SG78367[] = {SGI334135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78368[] = {SGI334136, SGI334137, SGI334138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334139, SGI0, NULL};
const Segment * const SG78369[] = {SGI334140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78370[] = {SGI334141, SGI334142, SGI334143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334144, SGI0, NULL};
const Segment * const SG78371[] = {SGI334145, SGI334146, SGI334147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334148, SGI0, NULL};
const Segment * const SG78372[] = {SGI334149, SGI334150, SGI334151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334152, SGI0, NULL};
const Segment * const SG78373[] = {SGI334153, SGI334154, SGI334155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334156, SGI0, NULL};
const Segment * const SG78374[] = {SGI334157, SGI334158, SGI334159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334160, SGI0, NULL};
const Segment * const SG78375[] = {SGI334161, SGI334162, SGI334163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334164, SGI0, NULL};
const Segment * const SG78376[] = {SGI334165, SGI334166, SGI334167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334168, SGI0, NULL};
const Segment * const SG78377[] = {SGI334169, SGI334170, SGI334171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334172, SGI0, NULL};
const Segment * const SG78378[] = {SGI334173, SGI334174, SGI334175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334176, SGI0, NULL};
const Segment * const SG78379[] = {SGI334177, SGI334178, SGI334179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334180, SGI0, NULL};
const Segment * const SG78380[] = {SGI334181, SGI334182, SGI334183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334184, SGI0, NULL};
const Segment * const SG78381[] = {SGI334185, SGI334186, SGI334187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334188, SGI0, NULL};
const Segment * const SG78382[] = {SGI334189, SGI334190, SGI334191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334192, SGI0, NULL};
const Segment * const SG78383[] = {SGI334193, SGI334194, SGI334195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334196, SGI0, NULL};
const Segment * const SG78384[] = {SGI334197, SGI334198, SGI334199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334200, SGI0, NULL};
const Segment * const SG78385[] = {SGI334201, SGI334202, SGI334203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334204, SGI0, NULL};
const Segment * const SG78386[] = {SGI334205, SGI334206, SGI334207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334208, SGI0, NULL};
const Segment * const SG78387[] = {SGI334209, SGI334210, SGI334211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334212, SGI0, NULL};
const Segment * const SG78388[] = {SGI334213, SGI334214, SGI334215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334216, SGI0, NULL};
const Segment * const SG78389[] = {SGI334217, SGI334218, SGI334219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334220, SGI0, NULL};
const Segment * const SG78390[] = {SGI334221, SGI334222, SGI334223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334224, SGI0, NULL};
const Segment * const SG78391[] = {SGI334225, SGI334226, SGI334227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334228, SGI0, NULL};
const Segment * const SG78392[] = {SGI334229, SGI334230, SGI334231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334232, SGI0, NULL};
const Segment * const SG78393[] = {SGI334233, SGI334234, SGI334235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334236, SGI0, NULL};
const Segment * const SG78394[] = {SGI334237, SGI334238, SGI334239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334240, SGI0, NULL};
const Segment * const SG78395[] = {SGI334241, SGI334242, SGI334243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334244, SGI0, NULL};
const Segment * const SG78396[] = {SGI334245, SGI334246, SGI334247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334248, SGI0, NULL};
const Segment * const SG78397[] = {SGI334249, SGI334250, SGI334251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334252, SGI0, NULL};
const Segment * const SG78398[] = {SGI334253, SGI334254, SGI334255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334256, SGI0, NULL};
const Segment * const SG78399[] = {SGI334257, SGI334258, SGI334259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334260, SGI0, NULL};
const Segment * const SG78400[] = {SGI334261, SGI334262, SGI334263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334264, SGI0, NULL};
const Segment * const SG78401[] = {SGI334265, SGI334266, SGI334267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334268, SGI0, NULL};
const Segment * const SG78402[] = {SGI334269, SGI334270, SGI334271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334272, SGI0, NULL};
const Segment * const SG78403[] = {SGI334273, SGI334274, SGI334275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334276, SGI0, NULL};
const Segment * const SG78404[] = {SGI334277, SGI334278, SGI334279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334280, SGI0, NULL};
const Segment * const SG78405[] = {SGI334281, SGI334282, SGI334283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334284, SGI0, NULL};
const Segment * const SG78406[] = {SGI334285, SGI334286, SGI334287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334288, SGI0, NULL};
const Segment * const SG78407[] = {SGI334289, SGI334290, SGI334291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334292, SGI0, NULL};
const Segment * const SG78408[] = {SGI334293, SGI334294, SGI334295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334296, SGI0, NULL};
const Segment * const SG78409[] = {SGI334297, SGI334298, SGI334299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334300, SGI0, NULL};
const Segment * const SG78410[] = {SGI334301, SGI334302, SGI334303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334304, SGI0, NULL};
const Segment * const SG78411[] = {SGI334305, SGI334306, SGI334307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334308, SGI0, NULL};
const Segment * const SG78412[] = {SGI334309, SGI334310, SGI334311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334312, SGI0, NULL};
const Segment * const SG78413[] = {SGI334313, SGI334314, SGI334315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334316, SGI0, NULL};
const Segment * const SG78414[] = {SGI334317, SGI334318, SGI334319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334320, SGI0, NULL};
const Segment * const SG78415[] = {SGI334321, SGI334322, SGI334323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334324, SGI0, NULL};
const Segment * const SG78416[] = {SGI334325, SGI334326, SGI334327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334328, SGI0, NULL};
const Segment * const SG78417[] = {SGI334329, SGI334330, SGI334331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334332, SGI0, NULL};
const Segment * const SG78418[] = {SGI334333, SGI334334, SGI334335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334336, SGI0, NULL};
const Segment * const SG78419[] = {SGI334337, SGI334338, SGI334339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334340, SGI0, NULL};
const Segment * const SG78420[] = {SGI334341, SGI334342, SGI334343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334344, SGI0, NULL};
const Segment * const SG78421[] = {SGI334345, SGI334346, SGI334347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334348, SGI0, NULL};
const Segment * const SG78422[] = {SGI334349, SGI334350, SGI334351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334352, SGI0, NULL};
const Segment * const SG78423[] = {SGI334353, SGI334354, SGI334355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334356, SGI0, NULL};
const Segment * const SG78424[] = {SGI334357, SGI334358, SGI334359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334360, SGI0, NULL};
const Segment * const SG78425[] = {SGI334361, SGI334362, SGI334363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334364, SGI0, NULL};
const Segment * const SG78426[] = {SGI334365, SGI334366, SGI334367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334368, SGI0, NULL};
const Segment * const SG78427[] = {SGI334369, SGI334370, SGI334371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334372, SGI0, NULL};
const Segment * const SG78428[] = {SGI334373, SGI334374, SGI334375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334376, SGI0, NULL};
const Segment * const SG78429[] = {SGI334377, SGI334378, SGI334379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334380, SGI0, NULL};
const Segment * const SG78430[] = {SGI334381, SGI334382, SGI334383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334384, SGI0, NULL};
const Segment * const SG78431[] = {SGI334385, SGI334386, SGI334387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334388, SGI0, NULL};
const Segment * const SG78432[] = {SGI334389, SGI334390, SGI334391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334392, SGI0, NULL};
const Segment * const SG78433[] = {SGI334393, SGI334394, SGI334395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334396, SGI0, NULL};
const Segment * const SG78434[] = {SGI334397, SGI334398, SGI334399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334400, SGI0, NULL};
const Segment * const SG78435[] = {SGI334401, SGI334402, SGI334403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334404, SGI0, NULL};
const Segment * const SG78436[] = {SGI334405, SGI334406, SGI334407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334408, SGI0, NULL};
const Segment * const SG78437[] = {SGI334409, SGI334410, SGI334411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334412, SGI0, NULL};
const Segment * const SG78438[] = {SGI334413, SGI334414, SGI334415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334416, SGI0, NULL};
const Segment * const SG78439[] = {SGI334417, SGI334418, SGI334419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334420, SGI0, NULL};
const Segment * const SG78440[] = {SGI334421, SGI334422, SGI334423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334424, SGI0, NULL};
const Segment * const SG78441[] = {SGI334425, SGI334426, SGI334427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334428, SGI0, NULL};
const Segment * const SG78442[] = {SGI334429, SGI334430, SGI334431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334432, SGI0, NULL};
const Segment * const SG78443[] = {SGI334433, SGI334434, SGI334435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334436, SGI0, NULL};
const Segment * const SG78444[] = {SGI334437, SGI334438, SGI334439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334440, SGI0, NULL};
const Segment * const SG78445[] = {SGI334441, SGI334442, SGI334443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334444, SGI0, NULL};
const Segment * const SG78446[] = {SGI334445, SGI334446, SGI334447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334448, SGI0, NULL};
const Segment * const SG78447[] = {SGI334449, SGI334450, SGI334451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334452, SGI0, NULL};
const Segment * const SG78448[] = {SGI334453, SGI334454, SGI334455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334456, SGI0, NULL};
const Segment * const SG78449[] = {SGI334457, SGI334458, SGI334459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334460, SGI0, NULL};
const Segment * const SG78450[] = {SGI334461, SGI334462, SGI334463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334464, SGI0, NULL};
const Segment * const SG78451[] = {SGI334465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78452[] = {SGI334466, SGI334467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78453[] = {SGI334468, SGI334469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78454[] = {SGI334470, SGI334471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78455[] = {SGI334472, SGI334473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78456[] = {SGI334474, SGI334475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78457[] = {SGI334476, SGI334477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78458[] = {SGI334478, SGI334479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78459[] = {SGI334480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78460[] = {SGI0, SGI334481, SGI334482, SGI0, SGI0, SGI0, SGI334483, NULL};
const Segment * const SG78461[] = {SGI0, SGI334484, SGI334485, SGI0, SGI0, SGI0, SGI334486, NULL};
const Segment * const SG78462[] = {SGI334487, SGI0, SGI0, SGI334488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78463[] = {SGI334489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334490, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78464[] = {SGI334491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334492, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78465[] = {SGI334493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334494, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78466[] = {SGI334495, SGI0, SGI0, SGI334496, SGI0, SGI0, SGI0, SGI0, SGI334497, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78467[] = {SGI334498, SGI334499, SGI0, SGI0, NULL};
const Segment * const SG78468[] = {SGI334500, SGI334501, SGI0, SGI0, NULL};
const Segment * const SG78469[] = {SGI334502, SGI334503, SGI0, SGI0, NULL};
const Segment * const SG78470[] = {SGI334504, SGI334505, SGI0, SGI0, NULL};
const Segment * const SG78471[] = {SGI334506, SGI334507, SGI0, SGI0, NULL};
const Segment * const SG78472[] = {SGI334508, SGI334509, SGI0, SGI0, NULL};
const Segment * const SG78473[] = {SGI334510, SGI334511, SGI0, SGI0, NULL};
const Segment * const SG78474[] = {SGI334512, SGI334513, SGI0, SGI0, NULL};
const Segment * const SG78475[] = {SGI334514, SGI334515, SGI0, SGI0, NULL};
const Segment * const SG78476[] = {SGI334516, SGI334517, SGI0, SGI0, NULL};
const Segment * const SG78477[] = {SGI334518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334519, SGI0, NULL};
const Segment * const SG78478[] = {SGI334520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78479[] = {SGI334521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78480[] = {SGI334522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78481[] = {SGI334523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78482[] = {SGI334524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78483[] = {SGI334525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78484[] = {SGI0, SGI334526, SGI334527, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78485[] = {SGI0, SGI334528, SGI334529, SGI0, SGI0, SGI0, SGI334530, NULL};
const Segment * const SG78486[] = {SGI0, SGI334531, SGI334532, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78487[] = {SGI0, SGI334533, SGI334534, SGI0, SGI0, SGI0, SGI334535, NULL};
const Segment * const SG78488[] = {SGI0, SGI334536, SGI334537, SGI0, SGI0, SGI0, SGI334538, NULL};
const Segment * const SG78489[] = {SGI0, SGI334539, SGI334540, SGI0, SGI0, SGI0, SGI334541, NULL};
const Segment * const SG78490[] = {SGI0, SGI334542, SGI334543, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78491[] = {SGI0, SGI334544, SGI334545, SGI0, SGI0, SGI0, SGI334546, NULL};
const Segment * const SG78492[] = {SGI0, SGI334547, SGI334548, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78493[] = {SGI0, SGI334549, SGI334550, SGI0, SGI0, SGI0, SGI334551, NULL};
const Segment * const SG78494[] = {SGI0, SGI334552, SGI334553, SGI0, SGI0, SGI0, SGI334554, NULL};
const Segment * const SG78495[] = {SGI334555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78496[] = {SGI334556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78497[] = {SGI334557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78498[] = {SGI334558, SGI334559, SGI0, SGI0, NULL};
const Segment * const SG78499[] = {SGI334560, SGI334561, SGI0, SGI0, NULL};
const Segment * const SG78500[] = {SGI334562, SGI334563, SGI0, SGI0, NULL};
const Segment * const SG78501[] = {SGI334564, SGI334565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78502[] = {SGI334566, SGI334567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78503[] = {SGI334568, SGI334569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78504[] = {SGI334570, SGI334571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78505[] = {SGI334572, SGI334573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78506[] = {SGI334574, SGI334575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78507[] = {SGI334576, SGI334577, SGI0, SGI0, NULL};
const Segment * const SG78508[] = {SGI334578, SGI0, SGI334579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334579, SGI0, NULL};
const Segment * const SG78509[] = {SGI334580, SGI0, SGI334581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334582, SGI0, NULL};
const Segment * const SG78510[] = {SGI334583, SGI0, SGI334584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334584, SGI0, NULL};
const Segment * const SG78511[] = {SGI334585, SGI0, SGI334586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334586, SGI0, NULL};
const Segment * const SG78512[] = {SGI334587, SGI0, SGI334588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334588, SGI0, NULL};
const Segment * const SG78513[] = {SGI334589, SGI0, SGI334590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334590, SGI0, NULL};
const Segment * const SG78514[] = {SGI334591, SGI0, SGI334592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334592, SGI0, NULL};
const Segment * const SG78515[] = {SGI334593, SGI0, SGI334594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334594, SGI0, NULL};
const Segment * const SG78516[] = {SGI334595, SGI0, SGI334596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334596, SGI0, NULL};
const Segment * const SG78517[] = {SGI334597, SGI0, SGI334598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334598, SGI0, NULL};
const Segment * const SG78518[] = {SGI334599, SGI0, SGI334600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334600, SGI0, NULL};
const Segment * const SG78519[] = {SGI334601, SGI0, SGI334602, SGI334602, SGI0, SGI0, SGI0, SGI0, SGI334602, SGI0, NULL};
const Segment * const SG78520[] = {SGI334603, SGI0, SGI334604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334604, SGI0, NULL};
const Segment * const SG78521[] = {SGI334605, SGI334606, SGI334607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334607, SGI0, NULL};
const Segment * const SG78522[] = {SGI334608, SGI334609, SGI334610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334610, SGI0, NULL};
const Segment * const SG78523[] = {SGI334611, SGI334612, SGI334613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334613, SGI0, NULL};
const Segment * const SG78524[] = {SGI334614, SGI334615, SGI334616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334616, SGI0, NULL};
const Segment * const SG78525[] = {SGI334617, SGI334618, SGI334619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334619, SGI0, NULL};
const Segment * const SG78526[] = {SGI334620, SGI334621, SGI334622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334622, SGI0, NULL};
const Segment * const SG78527[] = {SGI334623, SGI334624, SGI334625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334625, SGI0, NULL};
const Segment * const SG78528[] = {SGI334626, SGI334627, SGI334628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334628, SGI0, NULL};
const Segment * const SG78529[] = {SGI334629, SGI0, SGI334630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334630, SGI0, NULL};
const Segment * const SG78530[] = {SGI334631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334632, SGI0, NULL};
const Segment * const SG78531[] = {SGI334633, SGI334634, SGI0, SGI0, NULL};
const Segment * const SG78532[] = {SGI334635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78533[] = {SGI334636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78534[] = {SGI334637, SGI334638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78535[] = {SGI334639, SGI334640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78536[] = {SGI334641, SGI334642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78537[] = {SGI334643, SGI334644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78538[] = {SGI334645, SGI334646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78539[] = {SGI334647, SGI334648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78540[] = {SGI334649, SGI334650, SGI0, SGI0, NULL};
const Segment * const SG78541[] = {SGI334651, SGI334652, SGI0, SGI0, NULL};
const Segment * const SG78542[] = {SGI334653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78543[] = {SGI334654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78544[] = {SGI334655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78545[] = {SGI334656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78546[] = {SGI334657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78547[] = {SGI334658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78548[] = {SGI334659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78549[] = {SGI334660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78550[] = {SGI334661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78551[] = {SGI334662, SGI334663, SGI0, SGI0, NULL};
const Segment * const SG78552[] = {SGI334664, SGI334665, SGI0, SGI0, NULL};
const Segment * const SG78553[] = {SGI334666, SGI334667, SGI0, SGI0, NULL};
const Segment * const SG78554[] = {SGI334668, SGI334669, SGI0, SGI0, NULL};
const Segment * const SG78555[] = {SGI334670, SGI334671, SGI0, SGI0, NULL};
const Segment * const SG78556[] = {SGI334672, SGI334673, SGI0, SGI0, NULL};
const Segment * const SG78557[] = {SGI334675, SGI334677, SGI0, SGI0, NULL};
const Segment * const SG78558[] = {SGI334674, SGI0, SGI334676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334676, SGI0, NULL};
const Segment * const SG78559[] = {SGI334678, SGI334679, SGI0, SGI0, NULL};
const Segment * const SG78560[] = {SGI334680, SGI0, SGI334681, SGI334681, SGI0, SGI0, SGI0, SGI0, SGI334681, SGI0, NULL};
const Segment * const SG78561[] = {SGI334682, SGI334683, SGI0, SGI0, NULL};
const Segment * const SG78562[] = {SGI334684, SGI0, SGI334685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334685, SGI0, NULL};
const Segment * const SG78563[] = {SGI334686, SGI0, SGI334687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334687, SGI0, NULL};
const Segment * const SG78564[] = {SGI334688, SGI0, SGI334689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334689, SGI0, NULL};
const Segment * const SG78565[] = {SGI334690, SGI0, SGI334691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334691, SGI0, NULL};
const Segment * const SG78566[] = {SGI334692, SGI0, SGI334693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78567[] = {SGI334695, SGI334697, SGI0, SGI0, NULL};
const Segment * const SG78568[] = {SGI334694, SGI0, SGI334696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334696, SGI0, NULL};
const Segment * const SG78569[] = {SGI334698, SGI0, SGI334699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334700, SGI0, NULL};
const Segment * const SG78570[] = {SGI334701, SGI334702, SGI0, SGI0, NULL};
const Segment * const SG78571[] = {SGI334703, SGI334704, SGI0, SGI0, NULL};
const Segment * const SG78572[] = {SGI334705, SGI334706, SGI0, SGI0, NULL};
const Segment * const SG78573[] = {SGI334707, SGI334708, SGI0, SGI0, NULL};
const Segment * const SG78574[] = {SGI334709, SGI334710, SGI0, SGI0, NULL};
const Segment * const SG78575[] = {SGI334711, SGI334712, SGI0, SGI0, NULL};
const Segment * const SG78576[] = {SGI334713, SGI334714, SGI0, SGI0, NULL};
const Segment * const SG78577[] = {SGI334715, SGI334716, SGI0, SGI0, NULL};
const Segment * const SG78578[] = {SGI334717, SGI334718, SGI0, SGI0, NULL};
const Segment * const SG78579[] = {SGI334719, SGI334720, SGI0, SGI0, NULL};
const Segment * const SG78580[] = {SGI334721, SGI334722, SGI0, SGI0, NULL};
const Segment * const SG78581[] = {SGI334723, SGI334724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78582[] = {SGI334725, SGI334726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78583[] = {SGI334727, SGI334728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78584[] = {SGI334729, SGI334730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78585[] = {SGI334731, SGI334732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78586[] = {SGI334733, SGI334734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78587[] = {SGI334735, SGI334736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78588[] = {SGI334737, SGI334738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78589[] = {SGI334739, SGI334740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78590[] = {SGI334741, SGI334742, SGI0, SGI0, NULL};
const Segment * const SG78591[] = {SGI334743, SGI334744, SGI334745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334745, SGI0, NULL};
const Segment * const SG78592[] = {SGI334746, SGI334747, SGI334748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334749, SGI0, NULL};
const Segment * const SG78593[] = {SGI334750, SGI334751, SGI334752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334753, SGI0, NULL};
const Segment * const SG78594[] = {SGI334754, SGI334755, SGI334756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334757, SGI0, NULL};
const Segment * const SG78595[] = {SGI334758, SGI334759, SGI334760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334761, SGI0, NULL};
const Segment * const SG78596[] = {SGI334762, SGI334763, SGI334764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334765, SGI0, NULL};
const Segment * const SG78597[] = {SGI334766, SGI334767, SGI334768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334769, SGI0, NULL};
const Segment * const SG78598[] = {SGI334770, SGI334771, SGI334772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334773, SGI0, NULL};
const Segment * const SG78599[] = {SGI334774, SGI334775, SGI334776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334777, SGI0, NULL};
const Segment * const SG78600[] = {SGI334778, SGI334779, SGI334780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334781, SGI0, NULL};
const Segment * const SG78601[] = {SGI334782, SGI334783, SGI334784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334785, SGI0, NULL};
const Segment * const SG78602[] = {SGI334786, SGI334787, SGI334788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334789, SGI0, NULL};
const Segment * const SG78603[] = {SGI334790, SGI334791, SGI334792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334793, SGI0, NULL};
const Segment * const SG78604[] = {SGI334794, SGI334795, SGI334796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334797, SGI0, NULL};
const Segment * const SG78605[] = {SGI334798, SGI334799, SGI334800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334801, SGI0, NULL};
const Segment * const SG78606[] = {SGI334802, SGI334803, SGI334804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334805, SGI0, NULL};
const Segment * const SG78607[] = {SGI334806, SGI334807, SGI334808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334809, SGI0, NULL};
const Segment * const SG78608[] = {SGI334810, SGI334811, SGI334812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334813, SGI0, NULL};
const Segment * const SG78609[] = {SGI334814, SGI334815, SGI334816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334817, SGI0, NULL};
const Segment * const SG78610[] = {SGI334818, SGI334819, SGI334820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334821, SGI0, NULL};
const Segment * const SG78611[] = {SGI334822, SGI334823, SGI334824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334825, SGI0, NULL};
const Segment * const SG78612[] = {SGI334826, SGI334827, SGI334828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334829, SGI0, NULL};
const Segment * const SG78613[] = {SGI334830, SGI334831, SGI334832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334833, SGI0, NULL};
const Segment * const SG78614[] = {SGI334834, SGI334835, SGI334836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334837, SGI0, NULL};
const Segment * const SG78615[] = {SGI334838, SGI334839, SGI334840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334841, SGI0, NULL};
const Segment * const SG78616[] = {SGI334842, SGI334843, SGI334844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334845, SGI0, NULL};
const Segment * const SG78617[] = {SGI334846, SGI334847, SGI334848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334849, SGI0, NULL};
const Segment * const SG78618[] = {SGI334850, SGI334851, SGI334852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334853, SGI0, NULL};
const Segment * const SG78619[] = {SGI334854, SGI334855, SGI334856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334857, SGI0, NULL};
const Segment * const SG78620[] = {SGI334858, SGI334859, SGI334860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334861, SGI0, NULL};
const Segment * const SG78621[] = {SGI334862, SGI334863, SGI334864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334865, SGI0, NULL};
const Segment * const SG78622[] = {SGI334866, SGI334867, SGI334868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334869, SGI0, NULL};
const Segment * const SG78623[] = {SGI334870, SGI334871, SGI334872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334873, SGI0, NULL};
const Segment * const SG78624[] = {SGI334874, SGI334875, SGI334876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334877, SGI0, NULL};
const Segment * const SG78625[] = {SGI334878, SGI334879, SGI334880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334881, SGI0, NULL};
const Segment * const SG78626[] = {SGI334882, SGI334883, SGI334884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334885, SGI0, NULL};
const Segment * const SG78627[] = {SGI334886, SGI334887, SGI334888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334889, SGI0, NULL};
const Segment * const SG78628[] = {SGI334890, SGI334891, SGI334892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334893, SGI0, NULL};
const Segment * const SG78629[] = {SGI334894, SGI334895, SGI334896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334897, SGI0, NULL};
const Segment * const SG78630[] = {SGI334898, SGI334899, SGI334900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334901, SGI0, NULL};
const Segment * const SG78631[] = {SGI334902, SGI334903, SGI334904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334905, SGI0, NULL};
const Segment * const SG78632[] = {SGI334906, SGI334907, SGI334908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334909, SGI0, NULL};
const Segment * const SG78633[] = {SGI334910, SGI334911, SGI334912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334913, SGI0, NULL};
const Segment * const SG78634[] = {SGI334914, SGI334915, SGI334916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334917, SGI0, NULL};
const Segment * const SG78635[] = {SGI334918, SGI334919, SGI334920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334921, SGI0, NULL};
const Segment * const SG78636[] = {SGI334922, SGI334923, SGI334924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334925, SGI0, NULL};
const Segment * const SG78637[] = {SGI334926, SGI334927, SGI334928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334929, SGI0, NULL};
const Segment * const SG78638[] = {SGI334930, SGI334931, SGI334932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334933, SGI0, NULL};
const Segment * const SG78639[] = {SGI334934, SGI334935, SGI334936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334937, SGI0, NULL};
const Segment * const SG78640[] = {SGI334938, SGI334939, SGI334940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334941, SGI0, NULL};
const Segment * const SG78641[] = {SGI334942, SGI334943, SGI334944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334945, SGI0, NULL};
const Segment * const SG78642[] = {SGI334946, SGI334947, SGI334948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334949, SGI0, NULL};
const Segment * const SG78643[] = {SGI334950, SGI334951, SGI334952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334953, SGI0, NULL};
const Segment * const SG78644[] = {SGI334954, SGI334955, SGI334956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334957, SGI0, NULL};
const Segment * const SG78645[] = {SGI334958, SGI334959, SGI334960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334961, SGI0, NULL};
const Segment * const SG78646[] = {SGI334962, SGI334963, SGI334964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334965, SGI0, NULL};
const Segment * const SG78647[] = {SGI334966, SGI334967, SGI334968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334969, SGI0, NULL};
const Segment * const SG78648[] = {SGI334970, SGI334971, SGI334972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334973, SGI0, NULL};
const Segment * const SG78649[] = {SGI334974, SGI334975, SGI334976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334977, SGI0, NULL};
const Segment * const SG78650[] = {SGI334978, SGI334979, SGI334980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334981, SGI0, NULL};
const Segment * const SG78651[] = {SGI334982, SGI334983, SGI334984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334985, SGI0, NULL};
const Segment * const SG78652[] = {SGI334986, SGI334987, SGI334988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334989, SGI0, NULL};
const Segment * const SG78653[] = {SGI334990, SGI334991, SGI334992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334993, SGI0, NULL};
const Segment * const SG78654[] = {SGI334994, SGI334995, SGI334996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI334997, SGI0, NULL};
const Segment * const SG78655[] = {SGI334998, SGI334999, SGI335000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335001, SGI0, NULL};
const Segment * const SG78656[] = {SGI335002, SGI335003, SGI335004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335005, SGI0, NULL};
const Segment * const SG78657[] = {SGI335006, SGI335007, SGI335008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335009, SGI0, NULL};
const Segment * const SG78658[] = {SGI335010, SGI335011, SGI335012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335013, SGI0, NULL};
const Segment * const SG78659[] = {SGI335014, SGI335015, SGI335016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335017, SGI0, NULL};
const Segment * const SG78660[] = {SGI335018, SGI335019, SGI335020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335021, SGI0, NULL};
const Segment * const SG78661[] = {SGI335022, SGI335023, SGI335024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335025, SGI0, NULL};
const Segment * const SG78662[] = {SGI335026, SGI335027, SGI335028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335029, SGI0, NULL};
const Segment * const SG78663[] = {SGI335030, SGI335031, SGI335032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335033, SGI0, NULL};
const Segment * const SG78664[] = {SGI335034, SGI335035, SGI335036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335037, SGI0, NULL};
const Segment * const SG78665[] = {SGI335038, SGI335039, SGI335040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335041, SGI0, NULL};
const Segment * const SG78666[] = {SGI335042, SGI335043, SGI335044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335045, SGI0, NULL};
const Segment * const SG78667[] = {SGI335046, SGI335047, SGI335048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335049, SGI0, NULL};
const Segment * const SG78668[] = {SGI335050, SGI335051, SGI335052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335053, SGI0, NULL};
const Segment * const SG78669[] = {SGI335054, SGI335055, SGI335056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335057, SGI0, NULL};
const Segment * const SG78670[] = {SGI335058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78671[] = {SGI335059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78672[] = {SGI335060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78673[] = {SGI335061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78674[] = {SGI335062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78675[] = {SGI335063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78676[] = {SGI335064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78677[] = {SGI335065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78678[] = {SGI335066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78679[] = {SGI335067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78680[] = {SGI335068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78681[] = {SGI335069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335070, SGI0, NULL};
const Segment * const SG78682[] = {SGI335071, SGI335072, SGI0, SGI0, NULL};
const Segment * const SG78683[] = {SGI335073, SGI335074, SGI335075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335076, SGI0, NULL};
const Segment * const SG78684[] = {SGI335077, SGI335078, SGI335079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335080, SGI0, NULL};
const Segment * const SG78685[] = {SGI335081, SGI335082, SGI335083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335084, SGI0, NULL};
const Segment * const SG78686[] = {SGI335085, SGI335086, SGI335087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335088, SGI0, NULL};
const Segment * const SG78687[] = {SGI335089, SGI335090, SGI335091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335092, SGI0, NULL};
const Segment * const SG78688[] = {SGI335093, SGI335094, SGI335095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335096, SGI0, NULL};
const Segment * const SG78689[] = {SGI335097, SGI335098, SGI335099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335099, SGI0, NULL};
const Segment * const SG78690[] = {SGI335100, SGI335101, SGI335102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335103, SGI0, NULL};
const Segment * const SG78691[] = {SGI335104, SGI335105, SGI335106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335107, SGI0, NULL};
const Segment * const SG78692[] = {SGI335108, SGI335109, SGI335110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335111, SGI0, NULL};
const Segment * const SG78693[] = {SGI335112, SGI335113, SGI335114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335115, SGI0, NULL};
const Segment * const SG78694[] = {SGI335116, SGI335117, SGI335118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335119, SGI0, NULL};
const Segment * const SG78695[] = {SGI335120, SGI335121, SGI335122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335123, SGI0, NULL};
const Segment * const SG78696[] = {SGI335124, SGI335125, SGI335126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335127, SGI0, NULL};
const Segment * const SG78697[] = {SGI335128, SGI335129, SGI335130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335131, SGI0, NULL};
const Segment * const SG78698[] = {SGI335132, SGI335133, SGI335134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335135, SGI0, NULL};
const Segment * const SG78699[] = {SGI335136, SGI335137, SGI335138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335139, SGI0, NULL};
const Segment * const SG78700[] = {SGI335140, SGI335141, SGI335142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335143, SGI0, NULL};
const Segment * const SG78701[] = {SGI335144, SGI335145, SGI335146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335147, SGI0, NULL};
const Segment * const SG78702[] = {SGI335148, SGI335149, SGI335150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335151, SGI0, NULL};
const Segment * const SG78703[] = {SGI335152, SGI335153, SGI335154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335155, SGI0, NULL};
const Segment * const SG78704[] = {SGI335156, SGI335157, SGI335158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335159, SGI0, NULL};
const Segment * const SG78705[] = {SGI335160, SGI335161, SGI335162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335163, SGI0, NULL};
const Segment * const SG78706[] = {SGI335164, SGI335165, SGI335166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335167, SGI0, NULL};
const Segment * const SG78707[] = {SGI335168, SGI335169, SGI335170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335171, SGI0, NULL};
const Segment * const SG78708[] = {SGI335172, SGI335173, SGI335174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335175, SGI0, NULL};
const Segment * const SG78709[] = {SGI335176, SGI335177, SGI335178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335179, SGI0, NULL};
const Segment * const SG78710[] = {SGI335180, SGI335181, SGI335182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335183, SGI0, NULL};
const Segment * const SG78711[] = {SGI335184, SGI335185, SGI335186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335187, SGI0, NULL};
const Segment * const SG78712[] = {SGI335188, SGI335189, SGI335190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335191, SGI0, NULL};
const Segment * const SG78713[] = {SGI335192, SGI335193, SGI335194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335195, SGI0, NULL};
const Segment * const SG78714[] = {SGI335196, SGI335197, SGI335198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335199, SGI0, NULL};
const Segment * const SG78715[] = {SGI335200, SGI335201, SGI335202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335203, SGI0, NULL};
const Segment * const SG78716[] = {SGI335204, SGI335205, SGI335206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335207, SGI0, NULL};
const Segment * const SG78717[] = {SGI335208, SGI335209, SGI335210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335211, SGI0, NULL};
const Segment * const SG78718[] = {SGI335212, SGI335213, SGI335214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335215, SGI0, NULL};
const Segment * const SG78719[] = {SGI335216, SGI335217, SGI335218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335219, SGI0, NULL};
const Segment * const SG78720[] = {SGI335220, SGI335221, SGI335222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335223, SGI0, NULL};
const Segment * const SG78721[] = {SGI335224, SGI335225, SGI335226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335227, SGI0, NULL};
const Segment * const SG78722[] = {SGI335228, SGI335229, SGI335230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335231, SGI0, NULL};
const Segment * const SG78723[] = {SGI335232, SGI335233, SGI335234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335235, SGI0, NULL};
const Segment * const SG78724[] = {SGI335236, SGI335237, SGI335238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335239, SGI0, NULL};
const Segment * const SG78725[] = {SGI335240, SGI335241, SGI335242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335243, SGI0, NULL};
const Segment * const SG78726[] = {SGI335244, SGI335245, SGI335246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335247, SGI0, NULL};
const Segment * const SG78727[] = {SGI335248, SGI335249, SGI335250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335251, SGI0, NULL};
const Segment * const SG78728[] = {SGI335252, SGI335253, SGI335254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335255, SGI0, NULL};
const Segment * const SG78729[] = {SGI335256, SGI335257, SGI335258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335259, SGI0, NULL};
const Segment * const SG78730[] = {SGI335260, SGI335261, SGI335262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335263, SGI0, NULL};
const Segment * const SG78731[] = {SGI335264, SGI335265, SGI335266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335267, SGI0, NULL};
const Segment * const SG78732[] = {SGI335268, SGI335269, SGI335270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335271, SGI0, NULL};
const Segment * const SG78733[] = {SGI335272, SGI335273, SGI335274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335274, SGI0, NULL};
const Segment * const SG78734[] = {SGI335275, SGI335276, SGI335277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335278, SGI0, NULL};
const Segment * const SG78735[] = {SGI335279, SGI335280, SGI335281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335282, SGI0, NULL};
const Segment * const SG78736[] = {SGI335283, SGI335284, SGI335285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335286, SGI0, NULL};
const Segment * const SG78737[] = {SGI335287, SGI335288, SGI335289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335290, SGI0, NULL};
const Segment * const SG78738[] = {SGI335291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78739[] = {SGI335292, SGI335293, SGI335294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335295, SGI0, NULL};
const Segment * const SG78740[] = {SGI335296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78741[] = {SGI335297, SGI335298, SGI335299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335300, SGI0, NULL};
const Segment * const SG78742[] = {SGI335301, SGI335302, SGI335303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335304, SGI0, NULL};
const Segment * const SG78743[] = {SGI335305, SGI335306, SGI335307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335308, SGI0, NULL};
const Segment * const SG78744[] = {SGI335309, SGI335310, SGI335311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335312, SGI0, NULL};
const Segment * const SG78745[] = {SGI335313, SGI335314, SGI335315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335316, SGI0, NULL};
const Segment * const SG78746[] = {SGI335317, SGI335318, SGI335319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335320, SGI0, NULL};
const Segment * const SG78747[] = {SGI335321, SGI335322, SGI335323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335324, SGI0, NULL};
const Segment * const SG78748[] = {SGI335325, SGI335326, SGI335327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335328, SGI0, NULL};
const Segment * const SG78749[] = {SGI335329, SGI335330, SGI335331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335332, SGI0, NULL};
const Segment * const SG78750[] = {SGI335333, SGI335334, SGI335335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335336, SGI0, NULL};
const Segment * const SG78751[] = {SGI335337, SGI335338, SGI335339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335340, SGI0, NULL};
const Segment * const SG78752[] = {SGI335341, SGI335342, SGI335343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335344, SGI0, NULL};
const Segment * const SG78753[] = {SGI335345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78754[] = {SGI335346, SGI335347, SGI335348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335350, SGI0, NULL};
const Segment * const SG78755[] = {SGI335349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78756[] = {SGI335351, SGI335353, SGI335354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335355, SGI0, NULL};
const Segment * const SG78757[] = {SGI335352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78758[] = {SGI335356, SGI335357, SGI335358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335359, SGI0, NULL};
const Segment * const SG78759[] = {SGI335360, SGI335361, SGI335362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335363, SGI0, NULL};
const Segment * const SG78760[] = {SGI335364, SGI335365, SGI335366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335367, SGI0, NULL};
const Segment * const SG78761[] = {SGI335368, SGI335369, SGI335370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335371, SGI0, NULL};
const Segment * const SG78762[] = {SGI335372, SGI335373, SGI335374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335375, SGI0, NULL};
const Segment * const SG78763[] = {SGI335376, SGI335377, SGI335378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335379, SGI0, NULL};
const Segment * const SG78764[] = {SGI335380, SGI335381, SGI335382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335383, SGI0, NULL};
const Segment * const SG78765[] = {SGI335384, SGI335385, SGI335386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335387, SGI0, NULL};
const Segment * const SG78766[] = {SGI335388, SGI335389, SGI335390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335391, SGI0, NULL};
const Segment * const SG78767[] = {SGI335392, SGI335393, SGI335394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335395, SGI0, NULL};
const Segment * const SG78768[] = {SGI335396, SGI335397, SGI335398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335399, SGI0, NULL};
const Segment * const SG78769[] = {SGI335400, SGI335401, SGI335402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335403, SGI0, NULL};
const Segment * const SG78770[] = {SGI335404, SGI335405, SGI335406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335407, SGI0, NULL};
const Segment * const SG78771[] = {SGI335408, SGI335409, SGI335410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335411, SGI0, NULL};
const Segment * const SG78772[] = {SGI335412, SGI335413, SGI335414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335415, SGI0, NULL};
const Segment * const SG78773[] = {SGI335416, SGI335417, SGI335418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335419, SGI0, NULL};
const Segment * const SG78774[] = {SGI335420, SGI335421, SGI335422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335423, SGI0, NULL};
const Segment * const SG78775[] = {SGI335424, SGI335425, SGI335426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335427, SGI0, NULL};
const Segment * const SG78776[] = {SGI335428, SGI335429, SGI335430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335431, SGI0, NULL};
const Segment * const SG78777[] = {SGI335432, SGI335433, SGI335434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335435, SGI0, NULL};
const Segment * const SG78778[] = {SGI335436, SGI335437, SGI335438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335439, SGI0, NULL};
const Segment * const SG78779[] = {SGI335440, SGI335441, SGI335442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335443, SGI0, NULL};
const Segment * const SG78780[] = {SGI335444, SGI335445, SGI335446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335447, SGI0, NULL};
const Segment * const SG78781[] = {SGI335448, SGI335449, SGI335450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335451, SGI0, NULL};
const Segment * const SG78782[] = {SGI335452, SGI335453, SGI335454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335455, SGI0, NULL};
const Segment * const SG78783[] = {SGI335456, SGI335457, SGI335458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335459, SGI0, NULL};
const Segment * const SG78784[] = {SGI335460, SGI335461, SGI335462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335463, SGI0, NULL};
const Segment * const SG78785[] = {SGI335464, SGI335465, SGI335466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335467, SGI0, NULL};
const Segment * const SG78786[] = {SGI335468, SGI335469, SGI335470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335471, SGI0, NULL};
const Segment * const SG78787[] = {SGI335472, SGI335473, SGI335474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335475, SGI0, NULL};
const Segment * const SG78788[] = {SGI335476, SGI335477, SGI335478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335479, SGI0, NULL};
const Segment * const SG78789[] = {SGI335480, SGI335481, SGI335482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335483, SGI0, NULL};
const Segment * const SG78790[] = {SGI335484, SGI335485, SGI335486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335487, SGI0, NULL};
const Segment * const SG78791[] = {SGI335488, SGI335489, SGI335490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335491, SGI0, NULL};
const Segment * const SG78792[] = {SGI335492, SGI335493, SGI335494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335495, SGI0, NULL};
const Segment * const SG78793[] = {SGI335496, SGI335497, SGI335498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335499, SGI0, NULL};
const Segment * const SG78794[] = {SGI335500, SGI335501, SGI335502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335503, SGI0, NULL};
const Segment * const SG78795[] = {SGI335504, SGI335505, SGI335506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335507, SGI0, NULL};
const Segment * const SG78796[] = {SGI335508, SGI335509, SGI335510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335511, SGI0, NULL};
const Segment * const SG78797[] = {SGI335512, SGI335513, SGI335514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335515, SGI0, NULL};
const Segment * const SG78798[] = {SGI335516, SGI335517, SGI335518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335519, SGI0, NULL};
const Segment * const SG78799[] = {SGI335520, SGI335521, SGI335522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335523, SGI0, NULL};
const Segment * const SG78800[] = {SGI335524, SGI335525, SGI335526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335527, SGI0, NULL};
const Segment * const SG78801[] = {SGI335528, SGI335529, SGI335530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335531, SGI0, NULL};
const Segment * const SG78802[] = {SGI335532, SGI335533, SGI335534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335535, SGI0, NULL};
const Segment * const SG78803[] = {SGI335536, SGI335537, SGI335538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335539, SGI0, NULL};
const Segment * const SG78804[] = {SGI335540, SGI335541, SGI335542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335543, SGI0, NULL};
const Segment * const SG78805[] = {SGI335544, SGI335545, SGI335546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335547, SGI0, NULL};
const Segment * const SG78806[] = {SGI335548, SGI335549, SGI335550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335551, SGI0, NULL};
const Segment * const SG78807[] = {SGI335552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335553, SGI0, NULL};
const Segment * const SG78808[] = {SGI335554, SGI335555, SGI335556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335557, SGI0, NULL};
const Segment * const SG78809[] = {SGI335558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335560, SGI0, NULL};
const Segment * const SG78810[] = {SGI335559, SGI335561, SGI335562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335563, SGI0, NULL};
const Segment * const SG78811[] = {SGI335564, SGI335565, SGI335566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335567, SGI0, NULL};
const Segment * const SG78812[] = {SGI335568, SGI335569, SGI335570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335571, SGI0, NULL};
const Segment * const SG78813[] = {SGI335572, SGI335573, SGI335574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335575, SGI0, NULL};
const Segment * const SG78814[] = {SGI335576, SGI335577, SGI335578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335579, SGI0, NULL};
const Segment * const SG78815[] = {SGI335580, SGI335581, SGI335582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335583, SGI0, NULL};
const Segment * const SG78816[] = {SGI335584, SGI335585, SGI335586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335587, SGI0, NULL};
const Segment * const SG78817[] = {SGI335588, SGI335589, SGI335590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335591, SGI0, NULL};
const Segment * const SG78818[] = {SGI335592, SGI335593, SGI335594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335595, SGI0, NULL};
const Segment * const SG78819[] = {SGI335596, SGI335597, SGI335598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335599, SGI0, NULL};
const Segment * const SG78820[] = {SGI335600, SGI335601, SGI335602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335603, SGI0, NULL};
const Segment * const SG78821[] = {SGI335604, SGI335605, SGI335606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335607, SGI0, NULL};
const Segment * const SG78822[] = {SGI335608, SGI335609, SGI335610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335611, SGI0, NULL};
const Segment * const SG78823[] = {SGI335612, SGI335613, SGI335614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335615, SGI0, NULL};
const Segment * const SG78824[] = {SGI335616, SGI335617, SGI335618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335619, SGI0, NULL};
const Segment * const SG78825[] = {SGI335620, SGI335621, SGI335622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335623, SGI0, NULL};
const Segment * const SG78826[] = {SGI335624, SGI335625, SGI335626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335627, SGI0, NULL};
const Segment * const SG78827[] = {SGI335628, SGI335629, SGI335630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335631, SGI0, NULL};
const Segment * const SG78828[] = {SGI335632, SGI335633, SGI335634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335635, SGI0, NULL};
const Segment * const SG78829[] = {SGI335636, SGI335637, SGI335638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335639, SGI0, NULL};
const Segment * const SG78830[] = {SGI335640, SGI335641, SGI335642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335643, SGI0, NULL};
const Segment * const SG78831[] = {SGI335644, SGI335645, SGI335646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335647, SGI0, NULL};
const Segment * const SG78832[] = {SGI335648, SGI335649, SGI335650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335651, SGI0, NULL};
const Segment * const SG78833[] = {SGI335652, SGI335653, SGI335654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335655, SGI0, NULL};
const Segment * const SG78834[] = {SGI335656, SGI335657, SGI335658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335659, SGI0, NULL};
const Segment * const SG78835[] = {SGI335660, SGI335661, SGI335662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335663, SGI0, NULL};
const Segment * const SG78836[] = {SGI335664, SGI335665, SGI335666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335667, SGI0, NULL};
const Segment * const SG78837[] = {SGI335668, SGI335669, SGI335670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335671, SGI0, NULL};
const Segment * const SG78838[] = {SGI335672, SGI335673, SGI335674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335675, SGI0, NULL};
const Segment * const SG78839[] = {SGI335676, SGI335677, SGI335678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335679, SGI0, NULL};
const Segment * const SG78840[] = {SGI335680, SGI335681, SGI335682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335683, SGI0, NULL};
const Segment * const SG78841[] = {SGI335684, SGI335685, SGI335686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335686, SGI0, NULL};
const Segment * const SG78842[] = {SGI335687, SGI335688, SGI335689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335690, SGI0, NULL};
const Segment * const SG78843[] = {SGI335691, SGI335692, SGI335693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335694, SGI0, NULL};
const Segment * const SG78844[] = {SGI335695, SGI335696, SGI335697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335698, SGI0, NULL};
const Segment * const SG78845[] = {SGI335699, SGI335700, SGI335701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335702, SGI0, NULL};
const Segment * const SG78846[] = {SGI335703, SGI335704, SGI335705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335706, SGI0, NULL};
const Segment * const SG78847[] = {SGI335707, SGI335708, SGI335709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335710, SGI0, NULL};
const Segment * const SG78848[] = {SGI335711, SGI335712, SGI335713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335714, SGI0, NULL};
const Segment * const SG78849[] = {SGI335715, SGI335716, SGI335717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335718, SGI0, NULL};
const Segment * const SG78850[] = {SGI335719, SGI335720, SGI335721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335722, SGI0, NULL};
const Segment * const SG78851[] = {SGI335723, SGI335724, SGI335725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335726, SGI0, NULL};
const Segment * const SG78852[] = {SGI335727, SGI335728, SGI335729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335730, SGI0, NULL};
const Segment * const SG78853[] = {SGI335731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335732, SGI0, NULL};
const Segment * const SG78854[] = {SGI335733, SGI335734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78855[] = {SGI335735, SGI335736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78856[] = {SGI335737, SGI335738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78857[] = {SGI335739, SGI335740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78858[] = {SGI335741, SGI335742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78859[] = {SGI335743, SGI335744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78860[] = {SGI335745, SGI335746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78861[] = {SGI335747, SGI335748, SGI0, SGI0, NULL};
const Segment * const SG78862[] = {SGI335749, SGI335750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78863[] = {SGI335751, SGI335752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78864[] = {SGI335753, SGI335754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78865[] = {SGI335755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78866[] = {SGI335756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78867[] = {SGI335757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78868[] = {SGI335758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78869[] = {SGI335759, SGI335760, SGI335761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335762, SGI0, NULL};
const Segment * const SG78870[] = {SGI335763, SGI335764, SGI335765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335766, SGI0, NULL};
const Segment * const SG78871[] = {SGI335767, SGI335768, SGI335769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335770, SGI0, NULL};
const Segment * const SG78872[] = {SGI335771, SGI335772, SGI335773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335774, SGI0, NULL};
const Segment * const SG78873[] = {SGI335775, SGI335776, SGI335777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335777, SGI0, NULL};
const Segment * const SG78874[] = {SGI335778, SGI335779, SGI335780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335781, SGI0, NULL};
const Segment * const SG78875[] = {SGI335782, SGI335783, SGI335784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335785, SGI0, NULL};
const Segment * const SG78876[] = {SGI335786, SGI335787, SGI335788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335789, SGI0, NULL};
const Segment * const SG78877[] = {SGI335790, SGI335791, SGI335792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335793, SGI0, NULL};
const Segment * const SG78878[] = {SGI335794, SGI335795, SGI335796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335797, SGI0, NULL};
const Segment * const SG78879[] = {SGI335798, SGI335799, SGI335800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335801, SGI0, NULL};
const Segment * const SG78880[] = {SGI335802, SGI335803, SGI335804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335805, SGI0, NULL};
const Segment * const SG78881[] = {SGI335806, SGI335807, SGI335808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335809, SGI0, NULL};
const Segment * const SG78882[] = {SGI335810, SGI335811, SGI335812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335813, SGI0, NULL};
const Segment * const SG78883[] = {SGI335814, SGI335815, SGI335816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335817, SGI0, NULL};
const Segment * const SG78884[] = {SGI335818, SGI335819, SGI335820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335820, SGI0, NULL};
const Segment * const SG78885[] = {SGI335821, SGI335822, SGI335823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335824, SGI0, NULL};
const Segment * const SG78886[] = {SGI335825, SGI335826, SGI335827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335828, SGI0, NULL};
const Segment * const SG78887[] = {SGI335829, SGI335830, SGI335831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335832, SGI0, NULL};
const Segment * const SG78888[] = {SGI335833, SGI335834, SGI335835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335836, SGI0, NULL};
const Segment * const SG78889[] = {SGI335837, SGI335838, SGI335839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335840, SGI0, NULL};
const Segment * const SG78890[] = {SGI335841, SGI335842, SGI335843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335844, SGI0, NULL};
const Segment * const SG78891[] = {SGI335845, SGI335846, SGI335847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335848, SGI0, NULL};
const Segment * const SG78892[] = {SGI335849, SGI335850, SGI335851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335852, SGI0, NULL};
const Segment * const SG78893[] = {SGI335853, SGI335854, SGI335855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335856, SGI0, NULL};
const Segment * const SG78894[] = {SGI335857, SGI335858, SGI335859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335860, SGI0, NULL};
const Segment * const SG78895[] = {SGI335861, SGI335862, SGI335863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335864, SGI0, NULL};
const Segment * const SG78896[] = {SGI335865, SGI335866, SGI335867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335868, SGI0, NULL};
const Segment * const SG78897[] = {SGI335869, SGI335870, SGI335871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335872, SGI0, NULL};
const Segment * const SG78898[] = {SGI335873, SGI335874, SGI335875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335876, SGI0, NULL};
const Segment * const SG78899[] = {SGI335877, SGI335878, SGI335879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335880, SGI0, NULL};
const Segment * const SG78900[] = {SGI335881, SGI335882, SGI335883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335884, SGI0, NULL};
const Segment * const SG78901[] = {SGI335885, SGI335886, SGI335887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335888, SGI0, NULL};
const Segment * const SG78902[] = {SGI335889, SGI335890, SGI335891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335892, SGI0, NULL};
const Segment * const SG78903[] = {SGI335893, SGI335894, SGI335895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335896, SGI0, NULL};
const Segment * const SG78904[] = {SGI335897, SGI335898, SGI335899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335900, SGI0, NULL};
const Segment * const SG78905[] = {SGI335901, SGI335902, SGI335903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335904, SGI0, NULL};
const Segment * const SG78906[] = {SGI335905, SGI335906, SGI335907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335908, SGI0, NULL};
const Segment * const SG78907[] = {SGI335909, SGI335910, SGI335911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335912, SGI0, NULL};
const Segment * const SG78908[] = {SGI335913, SGI335914, SGI335915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335916, SGI0, NULL};
const Segment * const SG78909[] = {SGI335917, SGI335918, SGI335919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335920, SGI0, NULL};
const Segment * const SG78910[] = {SGI335921, SGI335922, SGI335923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335924, SGI0, NULL};
const Segment * const SG78911[] = {SGI335925, SGI335926, SGI335927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335928, SGI0, NULL};
const Segment * const SG78912[] = {SGI335929, SGI335930, SGI335931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335932, SGI0, NULL};
const Segment * const SG78913[] = {SGI335933, SGI335934, SGI335935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335936, SGI0, NULL};
const Segment * const SG78914[] = {SGI335937, SGI335938, SGI335939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335940, SGI0, NULL};
const Segment * const SG78915[] = {SGI335941, SGI335942, SGI335943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335944, SGI0, NULL};
const Segment * const SG78916[] = {SGI335945, SGI335946, SGI335947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335948, SGI0, NULL};
const Segment * const SG78917[] = {SGI335949, SGI335950, SGI335951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335952, SGI0, NULL};
const Segment * const SG78918[] = {SGI335953, SGI335954, SGI335955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335956, SGI0, NULL};
const Segment * const SG78919[] = {SGI335957, SGI335958, SGI335959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335960, SGI0, NULL};
const Segment * const SG78920[] = {SGI335961, SGI335962, SGI335963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335964, SGI0, NULL};
const Segment * const SG78921[] = {SGI335965, SGI335966, SGI335967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335968, SGI0, NULL};
const Segment * const SG78922[] = {SGI335969, SGI335970, SGI335971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335972, SGI0, NULL};
const Segment * const SG78923[] = {SGI335973, SGI335974, SGI335975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335976, SGI0, NULL};
const Segment * const SG78924[] = {SGI335977, SGI335978, SGI335979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335980, SGI0, NULL};
const Segment * const SG78925[] = {SGI335981, SGI335982, SGI335983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335984, SGI0, NULL};
const Segment * const SG78926[] = {SGI335985, SGI335986, SGI335987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335988, SGI0, NULL};
const Segment * const SG78927[] = {SGI335989, SGI335990, SGI335991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335992, SGI0, NULL};
const Segment * const SG78928[] = {SGI335993, SGI335994, SGI335995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI335996, SGI0, NULL};
const Segment * const SG78929[] = {SGI335997, SGI335998, SGI335999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336000, SGI0, NULL};
const Segment * const SG78930[] = {SGI336001, SGI336002, SGI336003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336004, SGI0, NULL};
const Segment * const SG78931[] = {SGI336005, SGI336006, SGI336007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336008, SGI0, NULL};
const Segment * const SG78932[] = {SGI336009, SGI336010, SGI336011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336012, SGI0, NULL};
const Segment * const SG78933[] = {SGI336013, SGI336014, SGI336015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336016, SGI0, NULL};
const Segment * const SG78934[] = {SGI336017, SGI336018, SGI336019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336020, SGI0, NULL};
const Segment * const SG78935[] = {SGI336021, SGI336022, SGI336023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336024, SGI0, NULL};
const Segment * const SG78936[] = {SGI336025, SGI336026, SGI336027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336028, SGI0, NULL};
const Segment * const SG78937[] = {SGI336029, SGI336030, SGI336031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336032, SGI0, NULL};
const Segment * const SG78938[] = {SGI336033, SGI336034, SGI336035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336036, SGI0, NULL};
const Segment * const SG78939[] = {SGI336037, SGI336038, SGI336039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336040, SGI0, NULL};
const Segment * const SG78940[] = {SGI336041, SGI336042, SGI336043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336044, SGI0, NULL};
const Segment * const SG78941[] = {SGI336045, SGI336046, SGI336047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336048, SGI0, NULL};
const Segment * const SG78942[] = {SGI336049, SGI336050, SGI336051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336052, SGI0, NULL};
const Segment * const SG78943[] = {SGI336053, SGI336054, SGI336055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336056, SGI0, NULL};
const Segment * const SG78944[] = {SGI336057, SGI336058, SGI336059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336060, SGI0, NULL};
const Segment * const SG78945[] = {SGI336061, SGI336062, SGI336063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336064, SGI0, NULL};
const Segment * const SG78946[] = {SGI336065, SGI336066, SGI336067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336068, SGI0, NULL};
const Segment * const SG78947[] = {SGI336069, SGI336070, SGI336071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336072, SGI0, NULL};
const Segment * const SG78948[] = {SGI336073, SGI336074, SGI336075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336076, SGI0, NULL};
const Segment * const SG78949[] = {SGI336077, SGI336078, SGI336079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336080, SGI0, NULL};
const Segment * const SG78950[] = {SGI336081, SGI336082, SGI336083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336084, SGI0, NULL};
const Segment * const SG78951[] = {SGI336085, SGI336086, SGI336087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336088, SGI0, NULL};
const Segment * const SG78952[] = {SGI336089, SGI336090, SGI336091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336092, SGI0, NULL};
const Segment * const SG78953[] = {SGI336093, SGI336094, SGI336095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336096, SGI0, NULL};
const Segment * const SG78954[] = {SGI336097, SGI336098, SGI336099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336100, SGI0, NULL};
const Segment * const SG78955[] = {SGI336101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336102, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78956[] = {SGI336103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336104, SGI0, NULL};
const Segment * const SG78957[] = {SGI336105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336106, SGI0, NULL};
const Segment * const SG78958[] = {SGI336107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78959[] = {SGI336108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78960[] = {SGI336109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78961[] = {SGI336110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78962[] = {SGI336111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78963[] = {SGI336112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78964[] = {SGI336113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336114, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78965[] = {SGI336115, SGI336116, SGI0, SGI0, NULL};
const Segment * const SG78966[] = {SGI336117, SGI336118, SGI0, SGI0, NULL};
const Segment * const SG78967[] = {SGI336119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336120, SGI0, NULL};
const Segment * const SG78968[] = {SGI336121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336122, SGI0, NULL};
const Segment * const SG78969[] = {SGI336123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78970[] = {SGI336124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78971[] = {SGI336125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78972[] = {SGI336126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78973[] = {SGI336127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78974[] = {SGI336128, SGI336129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78975[] = {SGI336130, SGI336131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78976[] = {SGI336132, SGI336133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78977[] = {SGI336134, SGI336135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78978[] = {SGI336136, SGI336137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78979[] = {SGI336138, SGI336139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78980[] = {SGI336140, SGI336141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78981[] = {SGI336142, SGI336143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78982[] = {SGI336144, SGI336145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78983[] = {SGI336146, SGI336147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78984[] = {SGI336148, SGI336149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78985[] = {SGI336150, SGI336151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78986[] = {SGI336152, SGI336153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78987[] = {SGI336154, SGI336155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78988[] = {SGI336156, SGI336157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78989[] = {SGI336158, SGI336159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78990[] = {SGI336160, SGI336161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78991[] = {SGI336162, SGI336163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78992[] = {SGI336164, SGI336165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78993[] = {SGI336166, SGI336167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78994[] = {SGI336168, SGI0, SGI336169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336169, SGI0, NULL};
const Segment * const SG78995[] = {SGI336170, SGI0, SGI336171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336171, SGI0, NULL};
const Segment * const SG78996[] = {SGI336172, SGI0, SGI336173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336174, SGI0, NULL};
const Segment * const SG78997[] = {SGI336175, SGI0, SGI336176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336176, SGI0, NULL};
const Segment * const SG78998[] = {SGI336177, SGI0, SGI336178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG78999[] = {SGI336179, SGI0, SGI336180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336180, SGI0, NULL};
const Segment * const SG79000[] = {SGI336181, SGI0, SGI336182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336182, SGI0, NULL};
const Segment * const SG79001[] = {SGI336183, SGI0, SGI336184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336184, SGI0, NULL};
const Segment * const SG79002[] = {SGI336185, SGI0, SGI336186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336186, SGI0, NULL};
const Segment * const SG79003[] = {SGI336187, SGI0, SGI336188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336188, SGI0, NULL};
const Segment * const SG79004[] = {SGI336189, SGI0, SGI336190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336190, SGI0, NULL};
const Segment * const SG79005[] = {SGI336191, SGI336192, SGI336193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336193, SGI0, NULL};
const Segment * const SG79006[] = {SGI336194, SGI336195, SGI336196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336196, SGI0, NULL};
const Segment * const SG79007[] = {SGI336197, SGI336198, SGI336199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336199, SGI0, NULL};
const Segment * const SG79008[] = {SGI336200, SGI336201, SGI336202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336202, SGI0, NULL};
const Segment * const SG79009[] = {SGI336203, SGI336204, SGI336205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336205, SGI0, NULL};
const Segment * const SG79010[] = {SGI336206, SGI336207, SGI336208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336208, SGI0, NULL};
const Segment * const SG79011[] = {SGI336209, SGI336210, SGI336211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336211, SGI0, NULL};
const Segment * const SG79012[] = {SGI336212, SGI336213, SGI336214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336214, SGI0, NULL};
const Segment * const SG79013[] = {SGI336215, SGI0, SGI336216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336216, SGI0, NULL};
const Segment * const SG79014[] = {SGI336217, SGI0, SGI336218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336218, SGI0, NULL};
const Segment * const SG79015[] = {SGI336219, SGI0, SGI336220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336220, SGI0, NULL};
const Segment * const SG79016[] = {SGI336221, SGI0, SGI336222, SGI336222, SGI0, SGI0, SGI0, SGI0, SGI336222, SGI0, NULL};
const Segment * const SG79017[] = {SGI336223, SGI0, SGI336224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336224, SGI0, NULL};
const Segment * const SG79018[] = {SGI336225, SGI336226, SGI0, SGI0, NULL};
const Segment * const SG79019[] = {SGI336227, SGI336228, SGI336229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336229, SGI0, NULL};
const Segment * const SG79020[] = {SGI336230, SGI336231, SGI336232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336233, SGI0, NULL};
const Segment * const SG79021[] = {SGI336234, SGI336235, SGI336236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336237, SGI0, NULL};
const Segment * const SG79022[] = {SGI336238, SGI336239, SGI336240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336241, SGI0, NULL};
const Segment * const SG79023[] = {SGI336242, SGI336243, SGI336244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336245, SGI0, NULL};
const Segment * const SG79024[] = {SGI336246, SGI336247, SGI336248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336249, SGI0, NULL};
const Segment * const SG79025[] = {SGI336250, SGI336251, SGI336252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336253, SGI0, NULL};
const Segment * const SG79026[] = {SGI336254, SGI336255, SGI336256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336257, SGI0, NULL};
const Segment * const SG79027[] = {SGI336258, SGI336259, SGI336260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336261, SGI0, NULL};
const Segment * const SG79028[] = {SGI336262, SGI336263, SGI336264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336265, SGI0, NULL};
const Segment * const SG79029[] = {SGI336266, SGI336267, SGI336268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336269, SGI0, NULL};
const Segment * const SG79030[] = {SGI336270, SGI336271, SGI336272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336273, SGI0, NULL};
const Segment * const SG79031[] = {SGI336274, SGI336275, SGI336276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336277, SGI0, NULL};
const Segment * const SG79032[] = {SGI336278, SGI336279, SGI336280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336281, SGI0, NULL};
const Segment * const SG79033[] = {SGI336282, SGI336283, SGI336284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336285, SGI0, NULL};
const Segment * const SG79034[] = {SGI336286, SGI336287, SGI336288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336289, SGI0, NULL};
const Segment * const SG79035[] = {SGI336290, SGI336291, SGI336292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336293, SGI0, NULL};
const Segment * const SG79036[] = {SGI336294, SGI336295, SGI336296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336297, SGI0, NULL};
const Segment * const SG79037[] = {SGI336298, SGI336299, SGI336300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336301, SGI0, NULL};
const Segment * const SG79038[] = {SGI336302, SGI336303, SGI336304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336305, SGI0, NULL};
const Segment * const SG79039[] = {SGI336306, SGI336307, SGI336308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336309, SGI0, NULL};
const Segment * const SG79040[] = {SGI336310, SGI336311, SGI336312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336313, SGI0, NULL};
const Segment * const SG79041[] = {SGI336314, SGI336315, SGI336316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336317, SGI0, NULL};
const Segment * const SG79042[] = {SGI336318, SGI336319, SGI336320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336321, SGI0, NULL};
const Segment * const SG79043[] = {SGI336322, SGI336323, SGI336324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336325, SGI0, NULL};
const Segment * const SG79044[] = {SGI336326, SGI336327, SGI336328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336329, SGI0, NULL};
const Segment * const SG79045[] = {SGI336330, SGI336331, SGI336332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336333, SGI0, NULL};
const Segment * const SG79046[] = {SGI336334, SGI336335, SGI336336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336337, SGI0, NULL};
const Segment * const SG79047[] = {SGI336338, SGI336339, SGI336340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336341, SGI0, NULL};
const Segment * const SG79048[] = {SGI336342, SGI336343, SGI336344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336345, SGI0, NULL};
const Segment * const SG79049[] = {SGI336346, SGI336347, SGI336348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336349, SGI0, NULL};
const Segment * const SG79050[] = {SGI336350, SGI336351, SGI336352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336353, SGI0, NULL};
const Segment * const SG79051[] = {SGI336354, SGI336355, SGI336356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336357, SGI0, NULL};
const Segment * const SG79052[] = {SGI336358, SGI336359, SGI336360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336361, SGI0, NULL};
const Segment * const SG79053[] = {SGI336362, SGI336363, SGI336364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336365, SGI0, NULL};
const Segment * const SG79054[] = {SGI336366, SGI336367, SGI336368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336369, SGI0, NULL};
const Segment * const SG79055[] = {SGI336370, SGI336371, SGI336372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336373, SGI0, NULL};
const Segment * const SG79056[] = {SGI336374, SGI336375, SGI336376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336377, SGI0, NULL};
const Segment * const SG79057[] = {SGI336378, SGI336379, SGI336380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336381, SGI0, NULL};
const Segment * const SG79058[] = {SGI336382, SGI336383, SGI336384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336385, SGI0, NULL};
const Segment * const SG79059[] = {SGI336386, SGI336387, SGI336388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336389, SGI0, NULL};
const Segment * const SG79060[] = {SGI336390, SGI336391, SGI336392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336392, SGI0, NULL};
const Segment * const SG79061[] = {SGI336393, SGI336394, SGI336395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336396, SGI0, NULL};
const Segment * const SG79062[] = {SGI336397, SGI336398, SGI336399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336400, SGI0, NULL};
const Segment * const SG79063[] = {SGI336401, SGI336402, SGI336403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336404, SGI0, NULL};
const Segment * const SG79064[] = {SGI336405, SGI336406, SGI336407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336408, SGI0, NULL};
const Segment * const SG79065[] = {SGI336409, SGI336410, SGI336411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336412, SGI0, NULL};
const Segment * const SG79066[] = {SGI336413, SGI336414, SGI336415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336416, SGI0, NULL};
const Segment * const SG79067[] = {SGI336417, SGI336418, SGI336419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336420, SGI0, NULL};
const Segment * const SG79068[] = {SGI336421, SGI336422, SGI336423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336424, SGI0, NULL};
const Segment * const SG79069[] = {SGI336425, SGI336426, SGI336427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336428, SGI0, NULL};
const Segment * const SG79070[] = {SGI336429, SGI336430, SGI336431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336432, SGI0, NULL};
const Segment * const SG79071[] = {SGI336433, SGI336434, SGI336435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336436, SGI0, NULL};
const Segment * const SG79072[] = {SGI336437, SGI336438, SGI336439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336440, SGI0, NULL};
const Segment * const SG79073[] = {SGI336441, SGI336442, SGI336443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336444, SGI0, NULL};
const Segment * const SG79074[] = {SGI336445, SGI336446, SGI336447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336448, SGI0, NULL};
const Segment * const SG79075[] = {SGI336449, SGI336450, SGI336451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336452, SGI0, NULL};
const Segment * const SG79076[] = {SGI336453, SGI336454, SGI336455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336456, SGI0, NULL};
const Segment * const SG79077[] = {SGI336457, SGI336458, SGI336459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336460, SGI0, NULL};
const Segment * const SG79078[] = {SGI336461, SGI336462, SGI336463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336464, SGI0, NULL};
const Segment * const SG79079[] = {SGI336465, SGI336466, SGI336467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336468, SGI0, NULL};
const Segment * const SG79080[] = {SGI336469, SGI336470, SGI336471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336472, SGI0, NULL};
const Segment * const SG79081[] = {SGI336473, SGI336474, SGI336475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336476, SGI0, NULL};
const Segment * const SG79082[] = {SGI336477, SGI336478, SGI336479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336480, SGI0, NULL};
const Segment * const SG79083[] = {SGI336481, SGI336482, SGI336483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336484, SGI0, NULL};
const Segment * const SG79084[] = {SGI336485, SGI336486, SGI336487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336488, SGI0, NULL};
const Segment * const SG79085[] = {SGI336489, SGI336490, SGI336491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336492, SGI0, NULL};
const Segment * const SG79086[] = {SGI336493, SGI336494, SGI336495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336496, SGI0, NULL};
const Segment * const SG79087[] = {SGI336497, SGI336498, SGI336499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336500, SGI0, NULL};
const Segment * const SG79088[] = {SGI336501, SGI336502, SGI336503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336504, SGI0, NULL};
const Segment * const SG79089[] = {SGI336505, SGI336506, SGI336507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336508, SGI0, NULL};
const Segment * const SG79090[] = {SGI336509, SGI336510, SGI336511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336512, SGI0, NULL};
const Segment * const SG79091[] = {SGI336513, SGI336514, SGI336515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336516, SGI0, NULL};
const Segment * const SG79092[] = {SGI336517, SGI336518, SGI336519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336520, SGI0, NULL};
const Segment * const SG79093[] = {SGI336521, SGI336522, SGI336523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336524, SGI0, NULL};
const Segment * const SG79094[] = {SGI336525, SGI336526, SGI336527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336528, SGI0, NULL};
const Segment * const SG79095[] = {SGI336529, SGI336530, SGI336531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336532, SGI0, NULL};
const Segment * const SG79096[] = {SGI336533, SGI336534, SGI336535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336536, SGI0, NULL};
const Segment * const SG79097[] = {SGI336537, SGI336538, SGI336539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336540, SGI0, NULL};
const Segment * const SG79098[] = {SGI336541, SGI336542, SGI336543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336544, SGI0, NULL};
const Segment * const SG79099[] = {SGI336545, SGI336546, SGI336547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336548, SGI0, NULL};
const Segment * const SG79100[] = {SGI336551, SGI0, SGI0, SGI336552, SGI0, SGI0, SGI0, SGI0, SGI336553, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79101[] = {SGI336554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336555, SGI0, NULL};
const Segment * const SG79102[] = {SGI336556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336557, SGI0, NULL};
const Segment * const SG79103[] = {SGI336549, SGI336550, SGI336558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336559, SGI0, NULL};
const Segment * const SG79104[] = {SGI336560, SGI336561, SGI336562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336563, SGI0, NULL};
const Segment * const SG79105[] = {SGI336564, SGI336565, SGI336566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336567, SGI0, NULL};
const Segment * const SG79106[] = {SGI336568, SGI336569, SGI336570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336571, SGI0, NULL};
const Segment * const SG79107[] = {SGI336572, SGI336573, SGI336574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336575, SGI0, NULL};
const Segment * const SG79108[] = {SGI336576, SGI336577, SGI336578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336579, SGI0, NULL};
const Segment * const SG79109[] = {SGI336580, SGI336581, SGI336582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336583, SGI0, NULL};
const Segment * const SG79110[] = {SGI336584, SGI336585, SGI336586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336587, SGI0, NULL};
const Segment * const SG79111[] = {SGI336588, SGI336589, SGI336590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336591, SGI0, NULL};
const Segment * const SG79112[] = {SGI336592, SGI336593, SGI336594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336595, SGI0, NULL};
const Segment * const SG79113[] = {SGI336596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79114[] = {SGI336597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79115[] = {SGI336598, SGI336599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79116[] = {SGI336600, SGI336601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79117[] = {SGI336602, SGI336603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79118[] = {SGI336604, SGI336605, SGI0, SGI0, NULL};
const Segment * const SG79119[] = {SGI336606, SGI336607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79120[] = {SGI336608, SGI336609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79121[] = {SGI336610, SGI336611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79122[] = {SGI336612, SGI336613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79123[] = {SGI336614, SGI336615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79124[] = {SGI336616, SGI336617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79125[] = {SGI0, SGI336618, SGI336619, SGI0, SGI0, SGI0, SGI336620, NULL};
const Segment * const SG79126[] = {SGI0, SGI336621, SGI336622, SGI0, SGI0, SGI0, SGI336623, NULL};
const Segment * const SG79127[] = {SGI336624, SGI336625, SGI336626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336627, SGI0, NULL};
const Segment * const SG79128[] = {SGI336628, SGI336629, SGI336630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336631, SGI0, NULL};
const Segment * const SG79129[] = {SGI336632, SGI336633, SGI336634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336635, SGI0, NULL};
const Segment * const SG79130[] = {SGI336636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79131[] = {SGI336637, SGI336638, SGI336639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336640, SGI0, NULL};
const Segment * const SG79132[] = {SGI336641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79133[] = {SGI336642, SGI336643, SGI336644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336645, SGI0, NULL};
const Segment * const SG79134[] = {SGI336646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79135[] = {SGI336647, SGI336648, SGI336649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336650, SGI0, NULL};
const Segment * const SG79136[] = {SGI336651, SGI336652, SGI336653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336654, SGI0, NULL};
const Segment * const SG79137[] = {SGI336655, SGI336656, SGI336657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336658, SGI0, NULL};
const Segment * const SG79138[] = {SGI336659, SGI336660, SGI336661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336662, SGI0, NULL};
const Segment * const SG79139[] = {SGI336663, SGI336664, SGI336665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336666, SGI0, NULL};
const Segment * const SG79140[] = {SGI336667, SGI336668, SGI336669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336670, SGI0, NULL};
const Segment * const SG79141[] = {SGI336671, SGI336672, SGI336673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336674, SGI0, NULL};
const Segment * const SG79142[] = {SGI336675, SGI336676, SGI336677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336678, SGI0, NULL};
const Segment * const SG79143[] = {SGI336679, SGI336680, SGI336681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336682, SGI0, NULL};
const Segment * const SG79144[] = {SGI336683, SGI336684, SGI336685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336686, SGI0, NULL};
const Segment * const SG79145[] = {SGI336687, SGI336688, SGI336689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336690, SGI0, NULL};
const Segment * const SG79146[] = {SGI336691, SGI336692, SGI336693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336693, SGI0, NULL};
const Segment * const SG79147[] = {SGI336694, SGI336695, SGI336696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336697, SGI0, NULL};
const Segment * const SG79148[] = {SGI336698, SGI336699, SGI336700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336701, SGI0, NULL};
const Segment * const SG79149[] = {SGI336702, SGI336703, SGI336704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336705, SGI0, NULL};
const Segment * const SG79150[] = {SGI336706, SGI336707, SGI336708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336709, SGI0, NULL};
const Segment * const SG79151[] = {SGI336710, SGI336711, SGI336712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336713, SGI0, NULL};
const Segment * const SG79152[] = {SGI336714, SGI336715, SGI336716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336717, SGI0, NULL};
const Segment * const SG79153[] = {SGI336718, SGI336719, SGI336720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336721, SGI0, NULL};
const Segment * const SG79154[] = {SGI336722, SGI336723, SGI336724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336724, SGI0, NULL};
const Segment * const SG79155[] = {SGI336725, SGI336726, SGI336727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336728, SGI0, NULL};
const Segment * const SG79156[] = {SGI336729, SGI336730, SGI336731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336732, SGI0, NULL};
const Segment * const SG79157[] = {SGI336733, SGI336734, SGI336735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336736, SGI0, NULL};
const Segment * const SG79158[] = {SGI336737, SGI336738, SGI336739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336740, SGI0, NULL};
const Segment * const SG79159[] = {SGI336741, SGI336742, SGI336743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336744, SGI0, NULL};
const Segment * const SG79160[] = {SGI336745, SGI336746, SGI336747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336748, SGI0, NULL};
const Segment * const SG79161[] = {SGI336749, SGI336750, SGI336751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336752, SGI0, NULL};
const Segment * const SG79162[] = {SGI336753, SGI336754, SGI336755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336756, SGI0, NULL};
const Segment * const SG79163[] = {SGI336757, SGI336758, SGI336759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336760, SGI0, NULL};
const Segment * const SG79164[] = {SGI336761, SGI336762, SGI336763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336764, SGI0, NULL};
const Segment * const SG79165[] = {SGI336765, SGI336766, SGI336767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336768, SGI0, NULL};
const Segment * const SG79166[] = {SGI336769, SGI336770, SGI336771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336772, SGI0, NULL};
const Segment * const SG79167[] = {SGI336773, SGI336774, SGI336775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336776, SGI0, NULL};
const Segment * const SG79168[] = {SGI336777, SGI336778, SGI336779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336780, SGI0, NULL};
const Segment * const SG79169[] = {SGI336781, SGI336782, SGI336783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336784, SGI0, NULL};
const Segment * const SG79170[] = {SGI336785, SGI336786, SGI336787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336788, SGI0, NULL};
const Segment * const SG79171[] = {SGI336789, SGI336790, SGI336791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336792, SGI0, NULL};
const Segment * const SG79172[] = {SGI336793, SGI336794, SGI336795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336796, SGI0, NULL};
const Segment * const SG79173[] = {SGI336797, SGI336798, SGI336799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336800, SGI0, NULL};
const Segment * const SG79174[] = {SGI336801, SGI336802, SGI336803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336804, SGI0, NULL};
const Segment * const SG79175[] = {SGI336805, SGI336806, SGI336807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336808, SGI0, NULL};
const Segment * const SG79176[] = {SGI336809, SGI336810, SGI336811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336812, SGI0, NULL};
const Segment * const SG79177[] = {SGI336813, SGI336814, SGI336815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336816, SGI0, NULL};
const Segment * const SG79178[] = {SGI336817, SGI336818, SGI336819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336820, SGI0, NULL};
const Segment * const SG79179[] = {SGI336821, SGI336822, SGI336823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336824, SGI0, NULL};
const Segment * const SG79180[] = {SGI336825, SGI336826, SGI336827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336828, SGI0, NULL};
const Segment * const SG79181[] = {SGI336829, SGI336830, SGI336831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336832, SGI0, NULL};
const Segment * const SG79182[] = {SGI336833, SGI336834, SGI336835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336836, SGI0, NULL};
const Segment * const SG79183[] = {SGI336837, SGI336838, SGI336839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336840, SGI0, NULL};
const Segment * const SG79184[] = {SGI336841, SGI336842, SGI336843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336844, SGI0, NULL};
const Segment * const SG79185[] = {SGI336845, SGI336846, SGI336847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336848, SGI0, NULL};
const Segment * const SG79186[] = {SGI336849, SGI336850, SGI336851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336852, SGI0, NULL};
const Segment * const SG79187[] = {SGI336853, SGI336854, SGI336855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336856, SGI0, NULL};
const Segment * const SG79188[] = {SGI336857, SGI336858, SGI336859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336860, SGI0, NULL};
const Segment * const SG79189[] = {SGI336861, SGI336862, SGI336863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336864, SGI0, NULL};
const Segment * const SG79190[] = {SGI336865, SGI336866, SGI336867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336868, SGI0, NULL};
const Segment * const SG79191[] = {SGI336869, SGI336870, SGI336871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336872, SGI0, NULL};
const Segment * const SG79192[] = {SGI336873, SGI336874, SGI336875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336876, SGI0, NULL};
const Segment * const SG79193[] = {SGI336877, SGI336878, SGI336879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336880, SGI0, NULL};
const Segment * const SG79194[] = {SGI336881, SGI336882, SGI336883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336884, SGI0, NULL};
const Segment * const SG79195[] = {SGI336888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79196[] = {SGI336889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79197[] = {SGI336890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79198[] = {SGI336891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79199[] = {SGI336892, SGI0, SGI0, SGI336893, SGI0, SGI0, SGI0, SGI0, SGI336894, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79200[] = {SGI336895, SGI0, SGI0, SGI336896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79201[] = {SGI336897, SGI0, SGI0, SGI336898, SGI0, SGI0, SGI0, SGI0, SGI336899, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79202[] = {SGI336885, SGI336886, SGI336887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336900, SGI0, NULL};
const Segment * const SG79203[] = {SGI336901, SGI336902, SGI336903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336904, SGI0, NULL};
const Segment * const SG79204[] = {SGI336905, SGI336906, SGI336907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336908, SGI0, NULL};
const Segment * const SG79205[] = {SGI336909, SGI336910, SGI336911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336912, SGI0, NULL};
const Segment * const SG79206[] = {SGI336913, SGI336914, SGI336915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336916, SGI0, NULL};
const Segment * const SG79207[] = {SGI336917, SGI336918, SGI336919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336920, SGI0, NULL};
const Segment * const SG79208[] = {SGI336921, SGI336922, SGI336923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336924, SGI0, NULL};
const Segment * const SG79209[] = {SGI336925, SGI336926, SGI336927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336928, SGI0, NULL};
const Segment * const SG79210[] = {SGI336929, SGI336930, SGI336931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336932, SGI0, NULL};
const Segment * const SG79211[] = {SGI336933, SGI336934, SGI336935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336936, SGI0, NULL};
const Segment * const SG79212[] = {SGI336937, SGI336938, SGI336939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336940, SGI0, NULL};
const Segment * const SG79213[] = {SGI336941, SGI336942, SGI336943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336944, SGI0, NULL};
const Segment * const SG79214[] = {SGI336945, SGI336946, SGI336947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336948, SGI0, NULL};
const Segment * const SG79215[] = {SGI336949, SGI336950, SGI336951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336952, SGI0, NULL};
const Segment * const SG79216[] = {SGI336953, SGI336954, SGI336955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336956, SGI0, NULL};
const Segment * const SG79217[] = {SGI336957, SGI336958, SGI336959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336960, SGI0, NULL};
const Segment * const SG79218[] = {SGI336961, SGI336962, SGI336963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336964, SGI0, NULL};
const Segment * const SG79219[] = {SGI336965, SGI336966, SGI336967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336968, SGI0, NULL};
const Segment * const SG79220[] = {SGI336969, SGI336970, SGI336971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336972, SGI0, NULL};
const Segment * const SG79221[] = {SGI336973, SGI336974, SGI336975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336976, SGI0, NULL};
const Segment * const SG79222[] = {SGI336977, SGI336978, SGI336979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336980, SGI0, NULL};
const Segment * const SG79223[] = {SGI336981, SGI336982, SGI336983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336984, SGI0, NULL};
const Segment * const SG79224[] = {SGI336985, SGI336986, SGI336987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336988, SGI0, NULL};
const Segment * const SG79225[] = {SGI336989, SGI336990, SGI336991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336992, SGI0, NULL};
const Segment * const SG79226[] = {SGI336993, SGI336994, SGI336995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI336996, SGI0, NULL};
const Segment * const SG79227[] = {SGI336997, SGI336998, SGI336999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337000, SGI0, NULL};
const Segment * const SG79228[] = {SGI337001, SGI337002, SGI337003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337004, SGI0, NULL};
const Segment * const SG79229[] = {SGI337005, SGI337006, SGI337007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337008, SGI0, NULL};
const Segment * const SG79230[] = {SGI337009, SGI337010, SGI337011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337012, SGI0, NULL};
const Segment * const SG79231[] = {SGI337013, SGI337014, SGI337015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337016, SGI0, NULL};
const Segment * const SG79232[] = {SGI337017, SGI337018, SGI337019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337020, SGI0, NULL};
const Segment * const SG79233[] = {SGI337021, SGI337022, SGI337023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337024, SGI0, NULL};
const Segment * const SG79234[] = {SGI337025, SGI337026, SGI337027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337028, SGI0, NULL};
const Segment * const SG79235[] = {SGI337029, SGI337030, SGI337031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337032, SGI0, NULL};
const Segment * const SG79236[] = {SGI337033, SGI337034, SGI337035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337036, SGI0, NULL};
const Segment * const SG79237[] = {SGI337037, SGI337038, SGI337039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337040, SGI0, NULL};
const Segment * const SG79238[] = {SGI337041, SGI337042, SGI337043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337044, SGI0, NULL};
const Segment * const SG79239[] = {SGI337045, SGI337046, SGI337047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337048, SGI0, NULL};
const Segment * const SG79240[] = {SGI337049, SGI337050, SGI337051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337052, SGI0, NULL};
const Segment * const SG79241[] = {SGI337053, SGI337054, SGI337055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337056, SGI0, NULL};
const Segment * const SG79242[] = {SGI337057, SGI337058, SGI337059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337060, SGI0, NULL};
const Segment * const SG79243[] = {SGI337061, SGI337062, SGI337063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337064, SGI0, NULL};
const Segment * const SG79244[] = {SGI337065, SGI337066, SGI337067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337068, SGI0, NULL};
const Segment * const SG79245[] = {SGI337069, SGI337070, SGI337071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337072, SGI0, NULL};
const Segment * const SG79246[] = {SGI337073, SGI337074, SGI337075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337076, SGI0, NULL};
const Segment * const SG79247[] = {SGI337077, SGI337078, SGI337079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337080, SGI0, NULL};
const Segment * const SG79248[] = {SGI337081, SGI337082, SGI337083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337084, SGI0, NULL};
const Segment * const SG79249[] = {SGI337085, SGI337086, SGI337087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337088, SGI0, NULL};
const Segment * const SG79250[] = {SGI337089, SGI337090, SGI337091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337092, SGI0, NULL};
const Segment * const SG79251[] = {SGI337093, SGI337094, SGI337095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337096, SGI0, NULL};
const Segment * const SG79252[] = {SGI337097, SGI337098, SGI337099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337100, SGI0, NULL};
const Segment * const SG79253[] = {SGI337101, SGI337102, SGI337103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337104, SGI0, NULL};
const Segment * const SG79254[] = {SGI337105, SGI337106, SGI337107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337108, SGI0, NULL};
const Segment * const SG79255[] = {SGI337109, SGI337110, SGI337111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337112, SGI0, NULL};
const Segment * const SG79256[] = {SGI337113, SGI337114, SGI337115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337116, SGI0, NULL};
const Segment * const SG79257[] = {SGI337117, SGI337118, SGI337119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337120, SGI0, NULL};
const Segment * const SG79258[] = {SGI337121, SGI337122, SGI337123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337124, SGI0, NULL};
const Segment * const SG79259[] = {SGI337125, SGI337126, SGI337127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337128, SGI0, NULL};
const Segment * const SG79260[] = {SGI337129, SGI337130, SGI337131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337132, SGI0, NULL};
const Segment * const SG79261[] = {SGI337133, SGI337134, SGI337135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337136, SGI0, NULL};
const Segment * const SG79262[] = {SGI337137, SGI337138, SGI337139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337140, SGI0, NULL};
const Segment * const SG79263[] = {SGI337141, SGI337142, SGI337143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337144, SGI0, NULL};
const Segment * const SG79264[] = {SGI337145, SGI337146, SGI337147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337148, SGI0, NULL};
const Segment * const SG79265[] = {SGI337149, SGI337150, SGI337151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337152, SGI0, NULL};
const Segment * const SG79266[] = {SGI337153, SGI337154, SGI337155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337156, SGI0, NULL};
const Segment * const SG79267[] = {SGI337157, SGI337158, SGI337159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337160, SGI0, NULL};
const Segment * const SG79268[] = {SGI337161, SGI337162, SGI337163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337164, SGI0, NULL};
const Segment * const SG79269[] = {SGI337165, SGI337166, SGI337167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337168, SGI0, NULL};
const Segment * const SG79270[] = {SGI337169, SGI337170, SGI337171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337172, SGI0, NULL};
const Segment * const SG79271[] = {SGI337173, SGI337174, SGI337175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337176, SGI0, NULL};
const Segment * const SG79272[] = {SGI337177, SGI337178, SGI337179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337180, SGI0, NULL};
const Segment * const SG79273[] = {SGI337181, SGI337182, SGI337183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337184, SGI0, NULL};
const Segment * const SG79274[] = {SGI337185, SGI337186, SGI337187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337188, SGI0, NULL};
const Segment * const SG79275[] = {SGI337189, SGI337190, SGI337191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337192, SGI0, NULL};
const Segment * const SG79276[] = {SGI337193, SGI337194, SGI337195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337196, SGI0, NULL};
const Segment * const SG79277[] = {SGI337197, SGI337198, SGI337199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337200, SGI0, NULL};
const Segment * const SG79278[] = {SGI337201, SGI337202, SGI337203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337204, SGI0, NULL};
const Segment * const SG79279[] = {SGI337205, SGI337206, SGI337207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337208, SGI0, NULL};
const Segment * const SG79280[] = {SGI337209, SGI337210, SGI337211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337212, SGI0, NULL};
const Segment * const SG79281[] = {SGI337213, SGI337214, SGI337215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337216, SGI0, NULL};
const Segment * const SG79282[] = {SGI337217, SGI337218, SGI337219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337220, SGI0, NULL};
const Segment * const SG79283[] = {SGI337221, SGI337222, SGI337223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337224, SGI0, NULL};
const Segment * const SG79284[] = {SGI337225, SGI337226, SGI337227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337228, SGI0, NULL};
const Segment * const SG79285[] = {SGI337229, SGI337230, SGI337231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337231, SGI0, NULL};
const Segment * const SG79286[] = {SGI337232, SGI337233, SGI337234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337234, SGI0, NULL};
const Segment * const SG79287[] = {SGI337235, SGI337236, SGI337237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337237, SGI0, NULL};
const Segment * const SG79288[] = {SGI337238, SGI337239, SGI337240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337241, SGI0, NULL};
const Segment * const SG79289[] = {SGI337242, SGI337243, SGI337244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337244, SGI0, NULL};
const Segment * const SG79290[] = {SGI337245, SGI337246, SGI337247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337248, SGI0, NULL};
const Segment * const SG79291[] = {SGI337249, SGI337250, SGI337251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337252, SGI0, NULL};
const Segment * const SG79292[] = {SGI337253, SGI337254, SGI337255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337256, SGI0, NULL};
const Segment * const SG79293[] = {SGI337257, SGI337258, SGI337259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337260, SGI0, NULL};
const Segment * const SG79294[] = {SGI337261, SGI337262, SGI337263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337264, SGI0, NULL};
const Segment * const SG79295[] = {SGI337265, SGI337266, SGI337267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337268, SGI0, NULL};
const Segment * const SG79296[] = {SGI337269, SGI337270, SGI337271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337272, SGI0, NULL};
const Segment * const SG79297[] = {SGI337273, SGI337274, SGI337275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337275, SGI0, NULL};
const Segment * const SG79298[] = {SGI337276, SGI337277, SGI337278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337278, SGI0, NULL};
const Segment * const SG79299[] = {SGI337279, SGI337280, SGI337281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337282, SGI0, NULL};
const Segment * const SG79300[] = {SGI337283, SGI337284, SGI337285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337286, SGI0, NULL};
const Segment * const SG79301[] = {SGI337287, SGI337288, SGI337289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337290, SGI0, NULL};
const Segment * const SG79302[] = {SGI337291, SGI337292, SGI337293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337294, SGI0, NULL};
const Segment * const SG79303[] = {SGI337295, SGI337296, SGI337297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337298, SGI0, NULL};
const Segment * const SG79304[] = {SGI337299, SGI337300, SGI337301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337302, SGI0, NULL};
const Segment * const SG79305[] = {SGI337303, SGI337304, SGI337305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337306, SGI0, NULL};
const Segment * const SG79306[] = {SGI337307, SGI337308, SGI337309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337310, SGI0, NULL};
const Segment * const SG79307[] = {SGI337311, SGI337312, SGI337313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337313, SGI0, NULL};
const Segment * const SG79308[] = {SGI337314, SGI337315, SGI337316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337316, SGI0, NULL};
const Segment * const SG79309[] = {SGI337317, SGI337318, SGI337319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337320, SGI0, NULL};
const Segment * const SG79310[] = {SGI337321, SGI337322, SGI337323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337324, SGI0, NULL};
const Segment * const SG79311[] = {SGI337325, SGI337326, SGI337327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337328, SGI0, NULL};
const Segment * const SG79312[] = {SGI337329, SGI337330, SGI337331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337332, SGI0, NULL};
const Segment * const SG79313[] = {SGI337333, SGI337334, SGI337335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337336, SGI0, NULL};
const Segment * const SG79314[] = {SGI337337, SGI337338, SGI337339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337340, SGI0, NULL};
const Segment * const SG79315[] = {SGI337341, SGI337342, SGI337343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337344, SGI0, NULL};
const Segment * const SG79316[] = {SGI337345, SGI337346, SGI337347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337348, SGI0, NULL};
const Segment * const SG79317[] = {SGI337349, SGI337350, SGI337351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337352, SGI0, NULL};
const Segment * const SG79318[] = {SGI337353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337356, SGI0, NULL};
const Segment * const SG79319[] = {SGI337354, SGI337355, SGI337357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337358, SGI0, NULL};
const Segment * const SG79320[] = {SGI337359, SGI337360, SGI337361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337362, SGI0, NULL};
const Segment * const SG79321[] = {SGI337363, SGI337364, SGI337365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337366, SGI0, NULL};
const Segment * const SG79322[] = {SGI337367, SGI337368, SGI337369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337370, SGI0, NULL};
const Segment * const SG79323[] = {SGI337371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337372, SGI0, NULL};
const Segment * const SG79324[] = {SGI337373, SGI337374, SGI337375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337376, SGI0, NULL};
const Segment * const SG79325[] = {SGI337377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337378, SGI0, NULL};
const Segment * const SG79326[] = {SGI337379, SGI337380, SGI337381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337382, SGI0, NULL};
const Segment * const SG79327[] = {SGI337383, SGI337384, SGI337385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337386, SGI0, NULL};
const Segment * const SG79328[] = {SGI337387, SGI337388, SGI337389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337390, SGI0, NULL};
const Segment * const SG79329[] = {SGI337391, SGI337391, SGI337392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337392, SGI0, NULL};
const Segment * const SG79330[] = {SGI337393, SGI337394, SGI337395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337396, SGI0, NULL};
const Segment * const SG79331[] = {SGI337397, SGI337398, SGI337399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337400, SGI0, NULL};
const Segment * const SG79332[] = {SGI337401, SGI337402, SGI337403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337404, SGI0, NULL};
const Segment * const SG79333[] = {SGI337405, SGI337406, SGI337407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337408, SGI0, NULL};
const Segment * const SG79334[] = {SGI337409, SGI337410, SGI337411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337412, SGI0, NULL};
const Segment * const SG79335[] = {SGI337413, SGI337414, SGI337415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337415, SGI0, NULL};
const Segment * const SG79336[] = {SGI337416, SGI337417, SGI337418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337419, SGI0, NULL};
const Segment * const SG79337[] = {SGI337420, SGI337421, SGI337422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337423, SGI0, NULL};
const Segment * const SG79338[] = {SGI337424, SGI337425, SGI337426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337426, SGI0, NULL};
const Segment * const SG79339[] = {SGI337427, SGI337428, SGI337429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337430, SGI0, NULL};
const Segment * const SG79340[] = {SGI337431, SGI337432, SGI337433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337434, SGI0, NULL};
const Segment * const SG79341[] = {SGI337435, SGI337436, SGI337437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337438, SGI0, NULL};
const Segment * const SG79342[] = {SGI337439, SGI337440, SGI337441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337441, SGI0, NULL};
const Segment * const SG79343[] = {SGI337442, SGI337443, SGI337444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337445, SGI0, NULL};
const Segment * const SG79344[] = {SGI337446, SGI337447, SGI337448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337449, SGI0, NULL};
const Segment * const SG79345[] = {SGI337450, SGI337451, SGI337452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337452, SGI0, NULL};
const Segment * const SG79346[] = {SGI337453, SGI337454, SGI337455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337456, SGI0, NULL};
const Segment * const SG79347[] = {SGI337457, SGI337458, SGI337459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337460, SGI0, NULL};
const Segment * const SG79348[] = {SGI337461, SGI337462, SGI337463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337464, SGI0, NULL};
const Segment * const SG79349[] = {SGI337465, SGI337466, SGI337467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337468, SGI0, NULL};
const Segment * const SG79350[] = {SGI337469, SGI337470, SGI337471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337472, SGI0, NULL};
const Segment * const SG79351[] = {SGI337473, SGI337474, SGI337475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337476, SGI0, NULL};
const Segment * const SG79352[] = {SGI337477, SGI337478, SGI337479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337480, SGI0, NULL};
const Segment * const SG79353[] = {SGI337481, SGI337482, SGI337483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337484, SGI0, NULL};
const Segment * const SG79354[] = {SGI337485, SGI337486, SGI337487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337488, SGI0, NULL};
const Segment * const SG79355[] = {SGI337489, SGI337490, SGI337491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337492, SGI0, NULL};
const Segment * const SG79356[] = {SGI337493, SGI337494, SGI337495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337496, SGI0, NULL};
const Segment * const SG79357[] = {SGI337497, SGI337498, SGI337499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337500, SGI0, NULL};
const Segment * const SG79358[] = {SGI337501, SGI337502, SGI337503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337504, SGI0, NULL};
const Segment * const SG79359[] = {SGI337505, SGI337506, SGI337507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337507, SGI0, NULL};
const Segment * const SG79360[] = {SGI337508, SGI337509, SGI337510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337511, SGI0, NULL};
const Segment * const SG79361[] = {SGI337512, SGI337513, SGI337514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337514, SGI0, NULL};
const Segment * const SG79362[] = {SGI337515, SGI337516, SGI337517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337518, SGI0, NULL};
const Segment * const SG79363[] = {SGI337519, SGI337520, SGI337521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337522, SGI0, NULL};
const Segment * const SG79364[] = {SGI337523, SGI337524, SGI337525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337526, SGI0, NULL};
const Segment * const SG79365[] = {SGI337527, SGI337528, SGI337529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337530, SGI0, NULL};
const Segment * const SG79366[] = {SGI337531, SGI337532, SGI337533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337534, SGI0, NULL};
const Segment * const SG79367[] = {SGI337535, SGI337536, SGI337537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337538, SGI0, NULL};
const Segment * const SG79368[] = {SGI337539, SGI337540, SGI337541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337542, SGI0, NULL};
const Segment * const SG79369[] = {SGI337543, SGI337544, SGI337545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337546, SGI0, NULL};
const Segment * const SG79370[] = {SGI337547, SGI337548, SGI337549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337550, SGI0, NULL};
const Segment * const SG79371[] = {SGI337551, SGI337552, SGI337553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337554, SGI0, NULL};
const Segment * const SG79372[] = {SGI337555, SGI337556, SGI337557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337558, SGI0, NULL};
const Segment * const SG79373[] = {SGI337559, SGI337560, SGI337561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337562, SGI0, NULL};
const Segment * const SG79374[] = {SGI337563, SGI337564, SGI337565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337566, SGI0, NULL};
const Segment * const SG79375[] = {SGI337567, SGI337568, SGI337569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337570, SGI0, NULL};
const Segment * const SG79376[] = {SGI337571, SGI337572, SGI337573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337574, SGI0, NULL};
const Segment * const SG79377[] = {SGI337575, SGI337576, SGI337577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337578, SGI0, NULL};
const Segment * const SG79378[] = {SGI337579, SGI337580, SGI337581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337582, SGI0, NULL};
const Segment * const SG79379[] = {SGI337583, SGI337584, SGI337585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337586, SGI0, NULL};
const Segment * const SG79380[] = {SGI337587, SGI337588, SGI337589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337590, SGI0, NULL};
const Segment * const SG79381[] = {SGI337591, SGI337592, SGI337593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337594, SGI0, NULL};
const Segment * const SG79382[] = {SGI337595, SGI337596, SGI337597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337598, SGI0, NULL};
const Segment * const SG79383[] = {SGI337599, SGI337600, SGI337601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337602, SGI0, NULL};
const Segment * const SG79384[] = {SGI337603, SGI337604, SGI337605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337606, SGI0, NULL};
const Segment * const SG79385[] = {SGI337607, SGI337608, SGI337609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337610, SGI0, NULL};
const Segment * const SG79386[] = {SGI337611, SGI337612, SGI337613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337614, SGI0, NULL};
const Segment * const SG79387[] = {SGI337615, SGI337616, SGI337617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337618, SGI0, NULL};
const Segment * const SG79388[] = {SGI337619, SGI337620, SGI337621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337622, SGI0, NULL};
const Segment * const SG79389[] = {SGI337623, SGI337624, SGI337625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337626, SGI0, NULL};
const Segment * const SG79390[] = {SGI337627, SGI337628, SGI337629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337630, SGI0, NULL};
const Segment * const SG79391[] = {SGI337631, SGI337632, SGI337633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337634, SGI0, NULL};
const Segment * const SG79392[] = {SGI337635, SGI337636, SGI337637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337638, SGI0, NULL};
const Segment * const SG79393[] = {SGI337639, SGI337640, SGI337641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337642, SGI0, NULL};
const Segment * const SG79394[] = {SGI337643, SGI337644, SGI337645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337646, SGI0, NULL};
const Segment * const SG79395[] = {SGI337647, SGI337648, SGI337649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337650, SGI0, NULL};
const Segment * const SG79396[] = {SGI337651, SGI337652, SGI337653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337654, SGI0, NULL};
const Segment * const SG79397[] = {SGI337655, SGI337656, SGI337657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337658, SGI0, NULL};
const Segment * const SG79398[] = {SGI337659, SGI337660, SGI337661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337662, SGI0, NULL};
const Segment * const SG79399[] = {SGI337663, SGI337664, SGI337665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337666, SGI0, NULL};
const Segment * const SG79400[] = {SGI337667, SGI337668, SGI337669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337670, SGI0, NULL};
const Segment * const SG79401[] = {SGI337671, SGI337672, SGI337673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337674, SGI0, NULL};
const Segment * const SG79402[] = {SGI337675, SGI337676, SGI337677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337678, SGI0, NULL};
const Segment * const SG79403[] = {SGI337679, SGI337680, SGI337681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337682, SGI0, NULL};
const Segment * const SG79404[] = {SGI337683, SGI337684, SGI337685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337686, SGI0, NULL};
const Segment * const SG79405[] = {SGI337687, SGI337688, SGI337689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337690, SGI0, NULL};
const Segment * const SG79406[] = {SGI337691, SGI337692, SGI337693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337694, SGI0, NULL};
const Segment * const SG79407[] = {SGI337695, SGI337696, SGI337697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337698, SGI0, NULL};
const Segment * const SG79408[] = {SGI337699, SGI337700, SGI337701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337702, SGI0, NULL};
const Segment * const SG79409[] = {SGI337703, SGI337704, SGI337705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337706, SGI0, NULL};
const Segment * const SG79410[] = {SGI337707, SGI337708, SGI337709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337710, SGI0, NULL};
const Segment * const SG79411[] = {SGI337711, SGI337712, SGI337713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337714, SGI0, NULL};
const Segment * const SG79412[] = {SGI337715, SGI337716, SGI337717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337718, SGI0, NULL};
const Segment * const SG79413[] = {SGI337719, SGI337720, SGI337721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337722, SGI0, NULL};
const Segment * const SG79414[] = {SGI337723, SGI337724, SGI337725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337726, SGI0, NULL};
const Segment * const SG79415[] = {SGI337727, SGI337728, SGI337729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337730, SGI0, NULL};
const Segment * const SG79416[] = {SGI337731, SGI337732, SGI337733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337734, SGI0, NULL};
const Segment * const SG79417[] = {SGI337735, SGI337736, SGI337737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337738, SGI0, NULL};
const Segment * const SG79418[] = {SGI337739, SGI337740, SGI337741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337742, SGI0, NULL};
const Segment * const SG79419[] = {SGI337743, SGI337744, SGI337745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337746, SGI0, NULL};
const Segment * const SG79420[] = {SGI337747, SGI337748, SGI337749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337750, SGI0, NULL};
const Segment * const SG79421[] = {SGI337751, SGI337752, SGI337753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337754, SGI0, NULL};
const Segment * const SG79422[] = {SGI337755, SGI337756, SGI337757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337758, SGI0, NULL};
const Segment * const SG79423[] = {SGI337759, SGI337760, SGI337761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337762, SGI0, NULL};
const Segment * const SG79424[] = {SGI337763, SGI337764, SGI337765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337766, SGI0, NULL};
const Segment * const SG79425[] = {SGI337767, SGI337768, SGI337769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337770, SGI0, NULL};
const Segment * const SG79426[] = {SGI337771, SGI337772, SGI337773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337774, SGI0, NULL};
const Segment * const SG79427[] = {SGI337775, SGI337776, SGI337777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337778, SGI0, NULL};
const Segment * const SG79428[] = {SGI337779, SGI337780, SGI337781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337782, SGI0, NULL};
const Segment * const SG79429[] = {SGI337783, SGI337784, SGI337785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337786, SGI0, NULL};
const Segment * const SG79430[] = {SGI337787, SGI337788, SGI337789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337790, SGI0, NULL};
const Segment * const SG79431[] = {SGI337791, SGI337792, SGI337793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337794, SGI0, NULL};
const Segment * const SG79432[] = {SGI337795, SGI337796, SGI337797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337798, SGI0, NULL};
const Segment * const SG79433[] = {SGI337799, SGI337800, SGI337801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337802, SGI0, NULL};
const Segment * const SG79434[] = {SGI337803, SGI337804, SGI337805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337806, SGI0, NULL};
const Segment * const SG79435[] = {SGI337807, SGI337808, SGI337809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337810, SGI0, NULL};
const Segment * const SG79436[] = {SGI337811, SGI337812, SGI337813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337814, SGI0, NULL};
const Segment * const SG79437[] = {SGI337815, SGI337816, SGI337817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337818, SGI0, NULL};
const Segment * const SG79438[] = {SGI337819, SGI337820, SGI337821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337822, SGI0, NULL};
const Segment * const SG79439[] = {SGI337823, SGI337824, SGI337825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337826, SGI0, NULL};
const Segment * const SG79440[] = {SGI337827, SGI337828, SGI337829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337830, SGI0, NULL};
const Segment * const SG79441[] = {SGI337831, SGI337832, SGI337833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337834, SGI0, NULL};
const Segment * const SG79442[] = {SGI337835, SGI337836, SGI337837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337838, SGI0, NULL};
const Segment * const SG79443[] = {SGI337839, SGI337840, SGI337841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337842, SGI0, NULL};
const Segment * const SG79444[] = {SGI337843, SGI337844, SGI337845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337846, SGI0, NULL};
const Segment * const SG79445[] = {SGI337847, SGI337848, SGI337849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337850, SGI0, NULL};
const Segment * const SG79446[] = {SGI337851, SGI337852, SGI337853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337854, SGI0, NULL};
const Segment * const SG79447[] = {SGI337855, SGI337856, SGI337857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337858, SGI0, NULL};
const Segment * const SG79448[] = {SGI337859, SGI337860, SGI337861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337862, SGI0, NULL};
const Segment * const SG79449[] = {SGI337863, SGI337864, SGI337865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337866, SGI0, NULL};
const Segment * const SG79450[] = {SGI337867, SGI337868, SGI337869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337870, SGI0, NULL};
const Segment * const SG79451[] = {SGI337871, SGI337872, SGI337873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337874, SGI0, NULL};
const Segment * const SG79452[] = {SGI337875, SGI337876, SGI337877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337878, SGI0, NULL};
const Segment * const SG79453[] = {SGI337879, SGI337880, SGI337881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337882, SGI0, NULL};
const Segment * const SG79454[] = {SGI337883, SGI337884, SGI337885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337886, SGI0, NULL};
const Segment * const SG79455[] = {SGI337887, SGI337888, SGI337889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337890, SGI0, NULL};
const Segment * const SG79456[] = {SGI337891, SGI337892, SGI337893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337894, SGI0, NULL};
const Segment * const SG79457[] = {SGI337895, SGI337896, SGI337897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337898, SGI0, NULL};
const Segment * const SG79458[] = {SGI337899, SGI337900, SGI337901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337902, SGI0, NULL};
const Segment * const SG79459[] = {SGI337903, SGI337904, SGI337905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337906, SGI0, NULL};
const Segment * const SG79460[] = {SGI337907, SGI337908, SGI337909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337910, SGI0, NULL};
const Segment * const SG79461[] = {SGI337911, SGI337912, SGI337913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337914, SGI0, NULL};
const Segment * const SG79462[] = {SGI337915, SGI337916, SGI337917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337918, SGI0, NULL};
const Segment * const SG79463[] = {SGI337919, SGI337920, SGI337921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337922, SGI0, NULL};
const Segment * const SG79464[] = {SGI337923, SGI337924, SGI337925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337926, SGI0, NULL};
const Segment * const SG79465[] = {SGI337927, SGI337928, SGI337929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337930, SGI0, NULL};
const Segment * const SG79466[] = {SGI337931, SGI337932, SGI337933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337934, SGI0, NULL};
const Segment * const SG79467[] = {SGI337935, SGI337936, SGI337937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337938, SGI0, NULL};
const Segment * const SG79468[] = {SGI337939, SGI0, SGI0, SGI337940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79469[] = {SGI337941, SGI337942, SGI337943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337944, SGI0, NULL};
const Segment * const SG79470[] = {SGI337945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337946, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79471[] = {SGI337947, SGI337948, SGI337949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337950, SGI0, NULL};
const Segment * const SG79472[] = {SGI337951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337952, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79473[] = {SGI337953, SGI337954, SGI337955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337956, SGI0, NULL};
const Segment * const SG79474[] = {SGI337957, SGI0, SGI0, SGI337958, SGI0, SGI0, SGI0, SGI0, SGI337959, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79475[] = {SGI337960, SGI337961, SGI337962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337963, SGI0, NULL};
const Segment * const SG79476[] = {SGI337964, SGI337965, SGI337966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337967, SGI0, NULL};
const Segment * const SG79477[] = {SGI337968, SGI337969, SGI337970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337971, SGI0, NULL};
const Segment * const SG79478[] = {SGI337972, SGI337973, SGI337974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337975, SGI0, NULL};
const Segment * const SG79479[] = {SGI337976, SGI337977, SGI337978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337979, SGI0, NULL};
const Segment * const SG79480[] = {SGI337980, SGI337981, SGI337982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337983, SGI0, NULL};
const Segment * const SG79481[] = {SGI337984, SGI337985, SGI337986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337987, SGI0, NULL};
const Segment * const SG79482[] = {SGI337988, SGI337989, SGI337990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337991, SGI0, NULL};
const Segment * const SG79483[] = {SGI337992, SGI337993, SGI337994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337995, SGI0, NULL};
const Segment * const SG79484[] = {SGI337996, SGI337997, SGI337998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI337998, SGI0, NULL};
const Segment * const SG79485[] = {SGI337999, SGI338000, SGI338001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338001, SGI0, NULL};
const Segment * const SG79486[] = {SGI338002, SGI338003, SGI338004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338004, SGI0, NULL};
const Segment * const SG79487[] = {SGI338005, SGI338006, SGI338007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338008, SGI0, NULL};
const Segment * const SG79488[] = {SGI338009, SGI338010, SGI338011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338012, SGI0, NULL};
const Segment * const SG79489[] = {SGI338013, SGI338014, SGI338015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338016, SGI0, NULL};
const Segment * const SG79490[] = {SGI338017, SGI338018, SGI338019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338020, SGI0, NULL};
const Segment * const SG79491[] = {SGI338021, SGI338022, SGI338023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338024, SGI0, NULL};
const Segment * const SG79492[] = {SGI338025, SGI338026, SGI338027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338027, SGI0, NULL};
const Segment * const SG79493[] = {SGI338028, SGI338029, SGI338030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338031, SGI0, NULL};
const Segment * const SG79494[] = {SGI338032, SGI338033, SGI338034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338035, SGI0, NULL};
const Segment * const SG79495[] = {SGI338036, SGI338037, SGI338038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338039, SGI0, NULL};
const Segment * const SG79496[] = {SGI338040, SGI338041, SGI338042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338043, SGI0, NULL};
const Segment * const SG79497[] = {SGI338044, SGI338045, SGI338046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338047, SGI0, NULL};
const Segment * const SG79498[] = {SGI338048, SGI338049, SGI338050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338051, SGI0, NULL};
const Segment * const SG79499[] = {SGI338052, SGI338053, SGI338054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338055, SGI0, NULL};
const Segment * const SG79500[] = {SGI338056, SGI338057, SGI338058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338059, SGI0, NULL};
const Segment * const SG79501[] = {SGI338060, SGI338061, SGI338062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338063, SGI0, NULL};
const Segment * const SG79502[] = {SGI338064, SGI338065, SGI338066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338067, SGI0, NULL};
const Segment * const SG79503[] = {SGI338068, SGI338069, SGI338070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338071, SGI0, NULL};
const Segment * const SG79504[] = {SGI338072, SGI338073, SGI338074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338075, SGI0, NULL};
const Segment * const SG79505[] = {SGI338076, SGI338077, SGI338078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338079, SGI0, NULL};
const Segment * const SG79506[] = {SGI338080, SGI338081, SGI338082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338083, SGI0, NULL};
const Segment * const SG79507[] = {SGI338084, SGI338085, SGI338086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338087, SGI0, NULL};
const Segment * const SG79508[] = {SGI338088, SGI338089, SGI338090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338091, SGI0, NULL};
const Segment * const SG79509[] = {SGI338092, SGI338093, SGI338094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338095, SGI0, NULL};
const Segment * const SG79510[] = {SGI338096, SGI338097, SGI338098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338099, SGI0, NULL};
const Segment * const SG79511[] = {SGI338100, SGI338101, SGI338102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338103, SGI0, NULL};
const Segment * const SG79512[] = {SGI338104, SGI338105, SGI0, SGI0, NULL};
const Segment * const SG79513[] = {SGI338106, SGI338107, SGI338108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338109, SGI0, NULL};
const Segment * const SG79514[] = {SGI338110, SGI338111, SGI338112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338113, SGI0, NULL};
const Segment * const SG79515[] = {SGI338114, SGI338115, SGI338116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338117, SGI0, NULL};
const Segment * const SG79516[] = {SGI338118, SGI338119, SGI338120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338121, SGI0, NULL};
const Segment * const SG79517[] = {SGI338122, SGI338123, SGI338124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338125, SGI0, NULL};
const Segment * const SG79518[] = {SGI338126, SGI338127, SGI338128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338129, SGI0, NULL};
const Segment * const SG79519[] = {SGI338130, SGI338131, SGI338132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338133, SGI0, NULL};
const Segment * const SG79520[] = {SGI338134, SGI338134, SGI338135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338135, SGI0, NULL};
const Segment * const SG79521[] = {SGI338136, SGI338137, SGI0, SGI0, NULL};
const Segment * const SG79522[] = {SGI338138, SGI338139, SGI338140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338141, SGI0, NULL};
const Segment * const SG79523[] = {SGI338142, SGI338143, SGI338144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338145, SGI0, NULL};
const Segment * const SG79524[] = {SGI338146, SGI338147, SGI0, SGI0, NULL};
const Segment * const SG79525[] = {SGI338148, SGI338149, SGI338150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338151, SGI0, NULL};
const Segment * const SG79526[] = {SGI338152, SGI338153, SGI0, SGI0, NULL};
const Segment * const SG79527[] = {SGI338154, SGI338155, SGI338156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338157, SGI0, NULL};
const Segment * const SG79528[] = {SGI338158, SGI338159, SGI338160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338161, SGI0, NULL};
const Segment * const SG79529[] = {SGI338162, SGI338163, SGI338164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338165, SGI0, NULL};
const Segment * const SG79530[] = {SGI338166, SGI338167, SGI338168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338169, SGI0, NULL};
const Segment * const SG79531[] = {SGI338170, SGI338171, SGI338172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338173, SGI0, NULL};
const Segment * const SG79532[] = {SGI338174, SGI338175, SGI0, SGI0, NULL};
const Segment * const SG79533[] = {SGI338176, SGI338177, SGI338178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338179, SGI0, NULL};
const Segment * const SG79534[] = {SGI338181, SGI338183, SGI0, SGI0, NULL};
const Segment * const SG79535[] = {SGI338180, SGI338182, SGI338184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338185, SGI0, NULL};
const Segment * const SG79536[] = {SGI338186, SGI338187, SGI338188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338189, SGI0, NULL};
const Segment * const SG79537[] = {SGI338190, SGI338191, SGI338192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338193, SGI0, NULL};
const Segment * const SG79538[] = {SGI338194, SGI338195, SGI338196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338197, SGI0, NULL};
const Segment * const SG79539[] = {SGI338198, SGI338199, SGI338200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338201, SGI0, NULL};
const Segment * const SG79540[] = {SGI338202, SGI338203, SGI338204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338205, SGI0, NULL};
const Segment * const SG79541[] = {SGI338206, SGI338207, SGI338208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338209, SGI0, NULL};
const Segment * const SG79542[] = {SGI338210, SGI338211, SGI338212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338213, SGI0, NULL};
const Segment * const SG79543[] = {SGI338214, SGI338215, SGI338216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338217, SGI0, NULL};
const Segment * const SG79544[] = {SGI338218, SGI338219, SGI338220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338221, SGI0, NULL};
const Segment * const SG79545[] = {SGI338222, SGI338223, SGI338224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338225, SGI0, NULL};
const Segment * const SG79546[] = {SGI338226, SGI338227, SGI338228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338229, SGI0, NULL};
const Segment * const SG79547[] = {SGI338230, SGI338231, SGI338232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338233, SGI0, NULL};
const Segment * const SG79548[] = {SGI338234, SGI338235, SGI338236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338237, SGI0, NULL};
const Segment * const SG79549[] = {SGI338238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79550[] = {SGI338239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79551[] = {SGI338240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79552[] = {SGI338241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79553[] = {SGI338242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338243, SGI0, NULL};
const Segment * const SG79554[] = {SGI338244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338245, SGI0, NULL};
const Segment * const SG79555[] = {SGI338246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79556[] = {SGI338247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338248, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79557[] = {SGI338249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338250, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79558[] = {SGI338251, SGI0, SGI0, SGI338252, SGI0, SGI0, SGI0, SGI0, SGI338253, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79559[] = {SGI338254, SGI0, SGI0, SGI338255, SGI0, SGI0, SGI0, SGI0, SGI338256, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79560[] = {SGI338257, SGI0, SGI338258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338258, SGI0, NULL};
const Segment * const SG79561[] = {SGI338259, SGI0, SGI338260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338260, SGI0, NULL};
const Segment * const SG79562[] = {SGI338261, SGI0, SGI338262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338262, SGI0, NULL};
const Segment * const SG79563[] = {SGI338263, SGI0, SGI338264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338264, SGI0, NULL};
const Segment * const SG79564[] = {SGI338265, SGI0, SGI338266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338267, SGI0, NULL};
const Segment * const SG79565[] = {SGI338268, SGI0, SGI338269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338269, SGI0, NULL};
const Segment * const SG79566[] = {SGI338270, SGI0, SGI338271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338271, SGI0, NULL};
const Segment * const SG79567[] = {SGI338272, SGI0, SGI338273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338273, SGI0, NULL};
const Segment * const SG79568[] = {SGI338274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338277, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79569[] = {SGI338275, SGI0, SGI338276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338278, SGI0, NULL};
const Segment * const SG79570[] = {SGI338280, SGI0, SGI338281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338283, SGI0, NULL};
const Segment * const SG79571[] = {SGI338279, SGI0, SGI0, SGI338282, SGI0, SGI0, SGI0, SGI0, SGI338284, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79572[] = {SGI338285, SGI0, SGI338286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338286, SGI0, NULL};
const Segment * const SG79573[] = {SGI338287, SGI0, SGI0, SGI338288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79574[] = {SGI338289, SGI0, SGI338290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338292, SGI0, NULL};
const Segment * const SG79575[] = {SGI338291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338293, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79576[] = {SGI338294, SGI0, SGI338296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338297, SGI0, NULL};
const Segment * const SG79577[] = {SGI338295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338298, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79578[] = {SGI338299, SGI0, SGI338300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338301, SGI0, NULL};
const Segment * const SG79579[] = {SGI338302, SGI0, SGI338303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338303, SGI0, NULL};
const Segment * const SG79580[] = {SGI338304, SGI0, SGI338305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338306, SGI0, NULL};
const Segment * const SG79581[] = {SGI338307, SGI0, SGI338308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338309, SGI0, NULL};
const Segment * const SG79582[] = {SGI338310, SGI0, SGI338311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338311, SGI0, NULL};
const Segment * const SG79583[] = {SGI338312, SGI0, SGI338313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338314, SGI0, NULL};
const Segment * const SG79584[] = {SGI338315, SGI0, SGI338316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338317, SGI0, NULL};
const Segment * const SG79585[] = {SGI338318, SGI0, SGI338319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338320, SGI0, NULL};
const Segment * const SG79586[] = {SGI338321, SGI0, SGI338322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338323, SGI0, NULL};
const Segment * const SG79587[] = {SGI338324, SGI0, SGI338325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338326, SGI0, NULL};
const Segment * const SG79588[] = {SGI338327, SGI0, SGI338328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338329, SGI0, NULL};
const Segment * const SG79589[] = {SGI338330, SGI0, SGI338331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338332, SGI0, NULL};
const Segment * const SG79590[] = {SGI338333, SGI0, SGI338334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338335, SGI0, NULL};
const Segment * const SG79591[] = {SGI338336, SGI0, SGI338337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338338, SGI0, NULL};
const Segment * const SG79592[] = {SGI338339, SGI0, SGI338340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338341, SGI0, NULL};
const Segment * const SG79593[] = {SGI338342, SGI0, SGI338343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338344, SGI0, NULL};
const Segment * const SG79594[] = {SGI338345, SGI0, SGI338346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338347, SGI0, NULL};
const Segment * const SG79595[] = {SGI338348, SGI0, SGI338349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338350, SGI0, NULL};
const Segment * const SG79596[] = {SGI338351, SGI0, SGI338352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338353, SGI0, NULL};
const Segment * const SG79597[] = {SGI338354, SGI0, SGI338355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338356, SGI0, NULL};
const Segment * const SG79598[] = {SGI338357, SGI0, SGI338358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338359, SGI0, NULL};
const Segment * const SG79599[] = {SGI338360, SGI0, SGI338361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338362, SGI0, NULL};
const Segment * const SG79600[] = {SGI338363, SGI0, SGI338364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338365, SGI0, NULL};
const Segment * const SG79601[] = {SGI338366, SGI0, SGI338367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338368, SGI0, NULL};
const Segment * const SG79602[] = {SGI338369, SGI0, SGI338370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338371, SGI0, NULL};
const Segment * const SG79603[] = {SGI338372, SGI0, SGI338373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338374, SGI0, NULL};
const Segment * const SG79604[] = {SGI338375, SGI0, SGI338376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338377, SGI0, NULL};
const Segment * const SG79605[] = {SGI338378, SGI0, SGI338379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338380, SGI0, NULL};
const Segment * const SG79606[] = {SGI338381, SGI0, SGI338382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338383, SGI0, NULL};
const Segment * const SG79607[] = {SGI338384, SGI0, SGI338385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338386, SGI0, NULL};
const Segment * const SG79608[] = {SGI338387, SGI0, SGI338388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338389, SGI0, NULL};
const Segment * const SG79609[] = {SGI338390, SGI0, SGI338391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338392, SGI0, NULL};
const Segment * const SG79610[] = {SGI338393, SGI0, SGI338394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338395, SGI0, NULL};
const Segment * const SG79611[] = {SGI338396, SGI0, SGI338397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338398, SGI0, NULL};
const Segment * const SG79612[] = {SGI338399, SGI0, SGI338400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338401, SGI0, NULL};
const Segment * const SG79613[] = {SGI338402, SGI0, SGI338403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338404, SGI0, NULL};
const Segment * const SG79614[] = {SGI338405, SGI0, SGI338406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338407, SGI0, NULL};
const Segment * const SG79615[] = {SGI338408, SGI0, SGI338409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338410, SGI0, NULL};
const Segment * const SG79616[] = {SGI338411, SGI0, SGI338412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338413, SGI0, NULL};
const Segment * const SG79617[] = {SGI338414, SGI0, SGI338415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338416, SGI0, NULL};
const Segment * const SG79618[] = {SGI338417, SGI0, SGI338418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338419, SGI0, NULL};
const Segment * const SG79619[] = {SGI338420, SGI0, SGI338421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338422, SGI0, NULL};
const Segment * const SG79620[] = {SGI338423, SGI0, SGI338424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338425, SGI0, NULL};
const Segment * const SG79621[] = {SGI338426, SGI0, SGI338427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338428, SGI0, NULL};
const Segment * const SG79622[] = {SGI338429, SGI0, SGI338430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338431, SGI0, NULL};
const Segment * const SG79623[] = {SGI338432, SGI0, SGI338433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338434, SGI0, NULL};
const Segment * const SG79624[] = {SGI338435, SGI0, SGI338436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338437, SGI0, NULL};
const Segment * const SG79625[] = {SGI338438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338439, SGI0, NULL};
const Segment * const SG79626[] = {SGI338440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79627[] = {SGI338441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79628[] = {SGI338442, SGI0, SGI0, SGI338443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79629[] = {SGI338444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338445, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79630[] = {SGI338446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338447, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79631[] = {SGI338448, SGI338449, SGI338450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338451, SGI0, NULL};
const Segment * const SG79632[] = {SGI338452, SGI338453, SGI338454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338455, SGI0, NULL};
const Segment * const SG79633[] = {SGI338456, SGI338457, SGI338458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338458, SGI0, NULL};
const Segment * const SG79634[] = {SGI338459, SGI338460, SGI338461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338461, SGI0, NULL};
const Segment * const SG79635[] = {SGI338462, SGI338463, SGI338464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338465, SGI0, NULL};
const Segment * const SG79636[] = {SGI338466, SGI338467, SGI338468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338469, SGI0, NULL};
const Segment * const SG79637[] = {SGI338470, SGI338471, SGI338472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338473, SGI0, NULL};
const Segment * const SG79638[] = {SGI338474, SGI338475, SGI338476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338477, SGI0, NULL};
const Segment * const SG79639[] = {SGI338478, SGI338479, SGI338480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338481, SGI0, NULL};
const Segment * const SG79640[] = {SGI338482, SGI338483, SGI338484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338485, SGI0, NULL};
const Segment * const SG79641[] = {SGI338486, SGI338487, SGI338488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338489, SGI0, NULL};
const Segment * const SG79642[] = {SGI338490, SGI338491, SGI338492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338493, SGI0, NULL};
const Segment * const SG79643[] = {SGI338494, SGI338495, SGI338496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338497, SGI0, NULL};
const Segment * const SG79644[] = {SGI338498, SGI338499, SGI338500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338501, SGI0, NULL};
const Segment * const SG79645[] = {SGI338502, SGI338503, SGI338504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338505, SGI0, NULL};
const Segment * const SG79646[] = {SGI338506, SGI338507, SGI338508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338509, SGI0, NULL};
const Segment * const SG79647[] = {SGI338510, SGI338511, SGI338512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338513, SGI0, NULL};
const Segment * const SG79648[] = {SGI338514, SGI338515, SGI338516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338517, SGI0, NULL};
const Segment * const SG79649[] = {SGI338518, SGI338519, SGI338520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338521, SGI0, NULL};
const Segment * const SG79650[] = {SGI338522, SGI338523, SGI338524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338525, SGI0, NULL};
const Segment * const SG79651[] = {SGI338526, SGI338527, SGI338528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338529, SGI0, NULL};
const Segment * const SG79652[] = {SGI338530, SGI338531, SGI338532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338533, SGI0, NULL};
const Segment * const SG79653[] = {SGI338534, SGI338535, SGI338536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338537, SGI0, NULL};
const Segment * const SG79654[] = {SGI338538, SGI338539, SGI338540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338541, SGI0, NULL};
const Segment * const SG79655[] = {SGI338542, SGI338543, SGI338544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338545, SGI0, NULL};
const Segment * const SG79656[] = {SGI338546, SGI338547, SGI338548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338549, SGI0, NULL};
const Segment * const SG79657[] = {SGI338550, SGI338551, SGI338552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338553, SGI0, NULL};
const Segment * const SG79658[] = {SGI338554, SGI338555, SGI338556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338557, SGI0, NULL};
const Segment * const SG79659[] = {SGI338558, SGI338559, SGI338560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338561, SGI0, NULL};
const Segment * const SG79660[] = {SGI338562, SGI338563, SGI338564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338565, SGI0, NULL};
const Segment * const SG79661[] = {SGI338566, SGI338567, SGI338568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338569, SGI0, NULL};
const Segment * const SG79662[] = {SGI338570, SGI338571, SGI338572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338573, SGI0, NULL};
const Segment * const SG79663[] = {SGI338574, SGI338575, SGI338576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338577, SGI0, NULL};
const Segment * const SG79664[] = {SGI338578, SGI338579, SGI338580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338581, SGI0, NULL};
const Segment * const SG79665[] = {SGI338582, SGI338583, SGI338584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338585, SGI0, NULL};
const Segment * const SG79666[] = {SGI338586, SGI338587, SGI338588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338589, SGI0, NULL};
const Segment * const SG79667[] = {SGI338590, SGI338591, SGI338592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338593, SGI0, NULL};
const Segment * const SG79668[] = {SGI338594, SGI338595, SGI338596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338597, SGI0, NULL};
const Segment * const SG79669[] = {SGI338598, SGI338599, SGI338600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338601, SGI0, NULL};
const Segment * const SG79670[] = {SGI338602, SGI338603, SGI338604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338605, SGI0, NULL};
const Segment * const SG79671[] = {SGI338606, SGI338607, SGI338608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338609, SGI0, NULL};
const Segment * const SG79672[] = {SGI338610, SGI338611, SGI338612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338613, SGI0, NULL};
const Segment * const SG79673[] = {SGI338614, SGI338615, SGI338616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338617, SGI0, NULL};
const Segment * const SG79674[] = {SGI338618, SGI338619, SGI338620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338621, SGI0, NULL};
const Segment * const SG79675[] = {SGI338622, SGI338623, SGI338624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338625, SGI0, NULL};
const Segment * const SG79676[] = {SGI338626, SGI338627, SGI338628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338629, SGI0, NULL};
const Segment * const SG79677[] = {SGI338630, SGI338631, SGI338632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338633, SGI0, NULL};
const Segment * const SG79678[] = {SGI338634, SGI338635, SGI338636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338637, SGI0, NULL};
const Segment * const SG79679[] = {SGI338638, SGI338639, SGI338640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338641, SGI0, NULL};
const Segment * const SG79680[] = {SGI338642, SGI338643, SGI338644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338645, SGI0, NULL};
const Segment * const SG79681[] = {SGI338646, SGI338647, SGI338648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338649, SGI0, NULL};
const Segment * const SG79682[] = {SGI338650, SGI338651, SGI338652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338653, SGI0, NULL};
const Segment * const SG79683[] = {SGI338654, SGI338655, SGI338656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338657, SGI0, NULL};
const Segment * const SG79684[] = {SGI338658, SGI338659, SGI338660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338661, SGI0, NULL};
const Segment * const SG79685[] = {SGI338662, SGI338663, SGI338664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338665, SGI0, NULL};
const Segment * const SG79686[] = {SGI338666, SGI338667, SGI338668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338669, SGI0, NULL};
const Segment * const SG79687[] = {SGI338670, SGI338671, SGI338672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338673, SGI0, NULL};
const Segment * const SG79688[] = {SGI338674, SGI338675, SGI338676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338677, SGI0, NULL};
const Segment * const SG79689[] = {SGI338678, SGI338679, SGI338680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338681, SGI0, NULL};
const Segment * const SG79690[] = {SGI338682, SGI338683, SGI338684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338685, SGI0, NULL};
const Segment * const SG79691[] = {SGI338686, SGI338687, SGI338688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338689, SGI0, NULL};
const Segment * const SG79692[] = {SGI338690, SGI338691, SGI338692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338693, SGI0, NULL};
const Segment * const SG79693[] = {SGI338694, SGI338695, SGI338696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338697, SGI0, NULL};
const Segment * const SG79694[] = {SGI338698, SGI338699, SGI338700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338701, SGI0, NULL};
const Segment * const SG79695[] = {SGI338702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79696[] = {SGI338703, SGI338704, SGI338705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338706, SGI0, NULL};
const Segment * const SG79697[] = {SGI338707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79698[] = {SGI338708, SGI338709, SGI338710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338711, SGI0, NULL};
const Segment * const SG79699[] = {SGI338712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79700[] = {SGI338713, SGI338714, SGI338715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338716, SGI0, NULL};
const Segment * const SG79701[] = {SGI338717, SGI338718, SGI338719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338720, SGI0, NULL};
const Segment * const SG79702[] = {SGI338721, SGI338722, SGI338723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338724, SGI0, NULL};
const Segment * const SG79703[] = {SGI338725, SGI338726, SGI338727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338728, SGI0, NULL};
const Segment * const SG79704[] = {SGI338729, SGI338730, SGI338731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338732, SGI0, NULL};
const Segment * const SG79705[] = {SGI338733, SGI338734, SGI338735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338736, SGI0, NULL};
const Segment * const SG79706[] = {SGI338737, SGI338738, SGI338739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338740, SGI0, NULL};
const Segment * const SG79707[] = {SGI338741, SGI338742, SGI338743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338744, SGI0, NULL};
const Segment * const SG79708[] = {SGI338745, SGI338746, SGI338747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338748, SGI0, NULL};
const Segment * const SG79709[] = {SGI338749, SGI338750, SGI338751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338752, SGI0, NULL};
const Segment * const SG79710[] = {SGI338753, SGI338754, SGI338755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338756, SGI0, NULL};
const Segment * const SG79711[] = {SGI338757, SGI338758, SGI338759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338760, SGI0, NULL};
const Segment * const SG79712[] = {SGI338761, SGI338762, SGI338763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338764, SGI0, NULL};
const Segment * const SG79713[] = {SGI338765, SGI338766, SGI338767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338768, SGI0, NULL};
const Segment * const SG79714[] = {SGI338769, SGI338770, SGI338771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338772, SGI0, NULL};
const Segment * const SG79715[] = {SGI338773, SGI338774, SGI338775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338776, SGI0, NULL};
const Segment * const SG79716[] = {SGI338777, SGI338778, SGI338779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338780, SGI0, NULL};
const Segment * const SG79717[] = {SGI338781, SGI338782, SGI338783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338784, SGI0, NULL};
const Segment * const SG79718[] = {SGI338785, SGI338786, SGI338787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338788, SGI0, NULL};
const Segment * const SG79719[] = {SGI338789, SGI338790, SGI338791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338792, SGI0, NULL};
const Segment * const SG79720[] = {SGI338793, SGI338794, SGI338795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338796, SGI0, NULL};
const Segment * const SG79721[] = {SGI338797, SGI338798, SGI338799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338800, SGI0, NULL};
const Segment * const SG79722[] = {SGI338801, SGI338802, SGI338803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338804, SGI0, NULL};
const Segment * const SG79723[] = {SGI338805, SGI338806, SGI338807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338808, SGI0, NULL};
const Segment * const SG79724[] = {SGI338809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338810, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79725[] = {SGI0, SGI338811, SGI338812, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79726[] = {SGI0, SGI338813, SGI338814, SGI0, SGI0, SGI0, SGI338815, NULL};
const Segment * const SG79727[] = {SGI338816, SGI0, SGI0, SGI338817, SGI0, SGI0, SGI0, SGI0, SGI338818, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79728[] = {SGI338819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338820, SGI0, NULL};
const Segment * const SG79729[] = {SGI338821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338822, SGI0, NULL};
const Segment * const SG79730[] = {SGI338823, SGI338824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79731[] = {SGI338825, SGI338826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79732[] = {SGI338827, SGI338828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79733[] = {SGI338829, SGI338830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79734[] = {SGI338831, SGI338832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79735[] = {SGI338833, SGI338834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79736[] = {SGI338835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79737[] = {SGI338836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79738[] = {SGI338837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79739[] = {SGI338838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338839, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79740[] = {SGI338840, SGI338841, SGI338842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338843, SGI0, NULL};
const Segment * const SG79741[] = {SGI338844, SGI338845, SGI338846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338846, SGI0, NULL};
const Segment * const SG79742[] = {SGI338847, SGI338848, SGI338849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338850, SGI0, NULL};
const Segment * const SG79743[] = {SGI338851, SGI338852, SGI338853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338854, SGI0, NULL};
const Segment * const SG79744[] = {SGI338855, SGI338856, SGI338857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338858, SGI0, NULL};
const Segment * const SG79745[] = {SGI338859, SGI0, SGI0, SGI338860, SGI0, SGI0, SGI0, SGI0, SGI338861, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79746[] = {SGI338862, SGI338863, SGI338864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338865, SGI0, NULL};
const Segment * const SG79747[] = {SGI338866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338868, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79748[] = {SGI338867, SGI338869, SGI338870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338871, SGI0, NULL};
const Segment * const SG79749[] = {SGI338872, SGI338873, SGI338874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338875, SGI0, NULL};
const Segment * const SG79750[] = {SGI338876, SGI338877, SGI338878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338879, SGI0, NULL};
const Segment * const SG79751[] = {SGI338880, SGI338881, SGI338882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338883, SGI0, NULL};
const Segment * const SG79752[] = {SGI338884, SGI338885, SGI338886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338887, SGI0, NULL};
const Segment * const SG79753[] = {SGI338888, SGI338889, SGI338890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338891, SGI0, NULL};
const Segment * const SG79754[] = {SGI338892, SGI338893, SGI338894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338895, SGI0, NULL};
const Segment * const SG79755[] = {SGI338896, SGI338897, SGI338898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338899, SGI0, NULL};
const Segment * const SG79756[] = {SGI338900, SGI338901, SGI338902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338903, SGI0, NULL};
const Segment * const SG79757[] = {SGI338904, SGI338905, SGI338906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338907, SGI0, NULL};
const Segment * const SG79758[] = {SGI338908, SGI338909, SGI338910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338911, SGI0, NULL};
const Segment * const SG79759[] = {SGI338912, SGI338913, SGI338914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338915, SGI0, NULL};
const Segment * const SG79760[] = {SGI338916, SGI338917, SGI338918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338919, SGI0, NULL};
const Segment * const SG79761[] = {SGI338920, SGI338921, SGI338922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338922, SGI0, NULL};
const Segment * const SG79762[] = {SGI338923, SGI338924, SGI338925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338926, SGI0, NULL};
const Segment * const SG79763[] = {SGI338927, SGI338928, SGI338929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338930, SGI0, NULL};
const Segment * const SG79764[] = {SGI338931, SGI338932, SGI338933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338934, SGI0, NULL};
const Segment * const SG79765[] = {SGI338935, SGI338936, SGI338937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338938, SGI0, NULL};
const Segment * const SG79766[] = {SGI338939, SGI338940, SGI338941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338942, SGI0, NULL};
const Segment * const SG79767[] = {SGI338943, SGI338944, SGI338945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338946, SGI0, NULL};
const Segment * const SG79768[] = {SGI338947, SGI338948, SGI338949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338950, SGI0, NULL};
const Segment * const SG79769[] = {SGI338951, SGI338952, SGI338953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338954, SGI0, NULL};
const Segment * const SG79770[] = {SGI338955, SGI338956, SGI338957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338958, SGI0, NULL};
const Segment * const SG79771[] = {SGI338959, SGI338960, SGI338961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338962, SGI0, NULL};
const Segment * const SG79772[] = {SGI338963, SGI338964, SGI338965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338966, SGI0, NULL};
const Segment * const SG79773[] = {SGI338967, SGI338968, SGI338969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338970, SGI0, NULL};
const Segment * const SG79774[] = {SGI338971, SGI338972, SGI338973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338974, SGI0, NULL};
const Segment * const SG79775[] = {SGI338975, SGI338976, SGI338977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338978, SGI0, NULL};
const Segment * const SG79776[] = {SGI338979, SGI338980, SGI338981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338982, SGI0, NULL};
const Segment * const SG79777[] = {SGI338983, SGI338984, SGI338985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338986, SGI0, NULL};
const Segment * const SG79778[] = {SGI338987, SGI338988, SGI338989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338990, SGI0, NULL};
const Segment * const SG79779[] = {SGI338991, SGI338992, SGI338993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338994, SGI0, NULL};
const Segment * const SG79780[] = {SGI338995, SGI338996, SGI338997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI338998, SGI0, NULL};
const Segment * const SG79781[] = {SGI338999, SGI339000, SGI339001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339002, SGI0, NULL};
const Segment * const SG79782[] = {SGI339003, SGI339004, SGI339005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339006, SGI0, NULL};
const Segment * const SG79783[] = {SGI339007, SGI339008, SGI339009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339010, SGI0, NULL};
const Segment * const SG79784[] = {SGI339011, SGI339012, SGI339013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339014, SGI0, NULL};
const Segment * const SG79785[] = {SGI339015, SGI339016, SGI339017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339018, SGI0, NULL};
const Segment * const SG79786[] = {SGI339019, SGI339020, SGI339021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339022, SGI0, NULL};
const Segment * const SG79787[] = {SGI339023, SGI0, SGI0, SGI339024, SGI0, SGI0, SGI0, SGI0, SGI339025, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79788[] = {SGI339026, SGI339027, SGI339028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339029, SGI0, NULL};
const Segment * const SG79789[] = {SGI339030, SGI0, SGI0, SGI339031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79790[] = {SGI339032, SGI339033, SGI339034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339035, SGI0, NULL};
const Segment * const SG79791[] = {SGI339036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339037, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79792[] = {SGI339038, SGI339039, SGI339040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339041, SGI0, NULL};
const Segment * const SG79793[] = {SGI339042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339043, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79794[] = {SGI339044, SGI339045, SGI339046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339047, SGI0, NULL};
const Segment * const SG79795[] = {SGI339048, SGI339049, SGI339050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339051, SGI0, NULL};
const Segment * const SG79796[] = {SGI339052, SGI339053, SGI339054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339055, SGI0, NULL};
const Segment * const SG79797[] = {SGI339056, SGI339057, SGI339058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339059, SGI0, NULL};
const Segment * const SG79798[] = {SGI339060, SGI339061, SGI339062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339063, SGI0, NULL};
const Segment * const SG79799[] = {SGI339064, SGI339065, SGI339066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339067, SGI0, NULL};
const Segment * const SG79800[] = {SGI339068, SGI339069, SGI339070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339071, SGI0, NULL};
const Segment * const SG79801[] = {SGI339072, SGI339073, SGI339074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339075, SGI0, NULL};
const Segment * const SG79802[] = {SGI339076, SGI339077, SGI339078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339079, SGI0, NULL};
const Segment * const SG79803[] = {SGI339080, SGI339081, SGI339082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339083, SGI0, NULL};
const Segment * const SG79804[] = {SGI339084, SGI339085, SGI339086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339087, SGI0, NULL};
const Segment * const SG79805[] = {SGI339088, SGI339089, SGI339090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339091, SGI0, NULL};
const Segment * const SG79806[] = {SGI339092, SGI339093, SGI339094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339095, SGI0, NULL};
const Segment * const SG79807[] = {SGI339096, SGI339097, SGI339098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339099, SGI0, NULL};
const Segment * const SG79808[] = {SGI339100, SGI339101, SGI339102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339103, SGI0, NULL};
const Segment * const SG79809[] = {SGI339104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79810[] = {SGI339105, SGI339106, SGI339107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339108, SGI0, NULL};
const Segment * const SG79811[] = {SGI339109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79812[] = {SGI339110, SGI339111, SGI339112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339113, SGI0, NULL};
const Segment * const SG79813[] = {SGI339114, SGI339115, SGI339116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339117, SGI0, NULL};
const Segment * const SG79814[] = {SGI339118, SGI339119, SGI339120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339121, SGI0, NULL};
const Segment * const SG79815[] = {SGI339122, SGI339123, SGI339124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339125, SGI0, NULL};
const Segment * const SG79816[] = {SGI339126, SGI339127, SGI339128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339129, SGI0, NULL};
const Segment * const SG79817[] = {SGI339130, SGI339131, SGI339132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339133, SGI0, NULL};
const Segment * const SG79818[] = {SGI339134, SGI339135, SGI339136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339137, SGI0, NULL};
const Segment * const SG79819[] = {SGI339138, SGI339139, SGI339140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339141, SGI0, NULL};
const Segment * const SG79820[] = {SGI339142, SGI339143, SGI339144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339145, SGI0, NULL};
const Segment * const SG79821[] = {SGI339146, SGI339147, SGI339148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339149, SGI0, NULL};
const Segment * const SG79822[] = {SGI339150, SGI339151, SGI339152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339153, SGI0, NULL};
const Segment * const SG79823[] = {SGI339154, SGI339155, SGI339156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339157, SGI0, NULL};
const Segment * const SG79824[] = {SGI339158, SGI339159, SGI339160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339161, SGI0, NULL};
const Segment * const SG79825[] = {SGI339162, SGI339163, SGI339164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339165, SGI0, NULL};
const Segment * const SG79826[] = {SGI339166, SGI339167, SGI339168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339169, SGI0, NULL};
const Segment * const SG79827[] = {SGI339170, SGI339171, SGI339172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339173, SGI0, NULL};
const Segment * const SG79828[] = {SGI339174, SGI339175, SGI339176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339177, SGI0, NULL};
const Segment * const SG79829[] = {SGI339178, SGI339179, SGI339180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339181, SGI0, NULL};
const Segment * const SG79830[] = {SGI339182, SGI339183, SGI339184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339185, SGI0, NULL};
const Segment * const SG79831[] = {SGI339186, SGI339187, SGI339188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339189, SGI0, NULL};
const Segment * const SG79832[] = {SGI339190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339191, SGI0, NULL};
const Segment * const SG79833[] = {SGI339192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339193, SGI0, NULL};
const Segment * const SG79834[] = {SGI339194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79835[] = {SGI339195, SGI339196, SGI0, SGI0, NULL};
const Segment * const SG79836[] = {SGI339197, SGI339198, SGI339199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339200, SGI0, NULL};
const Segment * const SG79837[] = {SGI339201, SGI339202, SGI339203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339204, SGI0, NULL};
const Segment * const SG79838[] = {SGI339205, SGI339206, SGI339207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339208, SGI0, NULL};
const Segment * const SG79839[] = {SGI339209, SGI339210, SGI339211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339212, SGI0, NULL};
const Segment * const SG79840[] = {SGI339213, SGI339214, SGI339215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339216, SGI0, NULL};
const Segment * const SG79841[] = {SGI339217, SGI339218, SGI339219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339220, SGI0, NULL};
const Segment * const SG79842[] = {SGI339221, SGI339222, SGI339223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339224, SGI0, NULL};
const Segment * const SG79843[] = {SGI339225, SGI339226, SGI339227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339228, SGI0, NULL};
const Segment * const SG79844[] = {SGI339229, SGI339230, SGI339231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339232, SGI0, NULL};
const Segment * const SG79845[] = {SGI339233, SGI339234, SGI339235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339236, SGI0, NULL};
const Segment * const SG79846[] = {SGI339237, SGI339238, SGI339239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339240, SGI0, NULL};
const Segment * const SG79847[] = {SGI339241, SGI339242, SGI339243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339244, SGI0, NULL};
const Segment * const SG79848[] = {SGI339245, SGI339246, SGI339247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339248, SGI0, NULL};
const Segment * const SG79849[] = {SGI339249, SGI339250, SGI339251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339252, SGI0, NULL};
const Segment * const SG79850[] = {SGI339253, SGI339254, SGI339255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339256, SGI0, NULL};
const Segment * const SG79851[] = {SGI339257, SGI339258, SGI339259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339260, SGI0, NULL};
const Segment * const SG79852[] = {SGI339261, SGI339262, SGI339263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339264, SGI0, NULL};
const Segment * const SG79853[] = {SGI339265, SGI339266, SGI339267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339268, SGI0, NULL};
const Segment * const SG79854[] = {SGI339269, SGI339270, SGI339271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339272, SGI0, NULL};
const Segment * const SG79855[] = {SGI339273, SGI339274, SGI339275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339276, SGI0, NULL};
const Segment * const SG79856[] = {SGI339277, SGI339278, SGI339279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339280, SGI0, NULL};
const Segment * const SG79857[] = {SGI339281, SGI339282, SGI339283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339284, SGI0, NULL};
const Segment * const SG79858[] = {SGI339285, SGI339286, SGI339287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339288, SGI0, NULL};
const Segment * const SG79859[] = {SGI339289, SGI339290, SGI339291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339292, SGI0, NULL};
const Segment * const SG79860[] = {SGI339293, SGI339294, SGI339295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339296, SGI0, NULL};
const Segment * const SG79861[] = {SGI339297, SGI339298, SGI339299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339300, SGI0, NULL};
const Segment * const SG79862[] = {SGI339301, SGI339302, SGI339303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339304, SGI0, NULL};
const Segment * const SG79863[] = {SGI339305, SGI339306, SGI339307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339308, SGI0, NULL};
const Segment * const SG79864[] = {SGI339309, SGI339310, SGI339311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339312, SGI0, NULL};
const Segment * const SG79865[] = {SGI339313, SGI339314, SGI339315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339316, SGI0, NULL};
const Segment * const SG79866[] = {SGI339317, SGI339318, SGI339319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339320, SGI0, NULL};
const Segment * const SG79867[] = {SGI339321, SGI339322, SGI339323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339324, SGI0, NULL};
const Segment * const SG79868[] = {SGI339325, SGI339326, SGI339327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339328, SGI0, NULL};
const Segment * const SG79869[] = {SGI339329, SGI339330, SGI339331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339332, SGI0, NULL};
const Segment * const SG79870[] = {SGI339333, SGI339334, SGI339335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339336, SGI0, NULL};
const Segment * const SG79871[] = {SGI339337, SGI339338, SGI339339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339340, SGI0, NULL};
const Segment * const SG79872[] = {SGI339341, SGI339342, SGI339343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339344, SGI0, NULL};
const Segment * const SG79873[] = {SGI339345, SGI339346, SGI339347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339348, SGI0, NULL};
const Segment * const SG79874[] = {SGI339349, SGI339350, SGI339351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339352, SGI0, NULL};
const Segment * const SG79875[] = {SGI339353, SGI339354, SGI339355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339356, SGI0, NULL};
const Segment * const SG79876[] = {SGI339357, SGI339358, SGI339359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339360, SGI0, NULL};
const Segment * const SG79877[] = {SGI339361, SGI339362, SGI339363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339364, SGI0, NULL};
const Segment * const SG79878[] = {SGI339365, SGI339366, SGI339367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339368, SGI0, NULL};
const Segment * const SG79879[] = {SGI339369, SGI339370, SGI339371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339372, SGI0, NULL};
const Segment * const SG79880[] = {SGI339373, SGI339374, SGI339375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339376, SGI0, NULL};
const Segment * const SG79881[] = {SGI339377, SGI339378, SGI339379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339380, SGI0, NULL};
const Segment * const SG79882[] = {SGI339381, SGI339382, SGI339383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339384, SGI0, NULL};
const Segment * const SG79883[] = {SGI339385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79884[] = {SGI339386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79885[] = {SGI339387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79886[] = {SGI339388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339389, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79887[] = {SGI339390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339391, SGI0, NULL};
const Segment * const SG79888[] = {SGI339392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339393, SGI0, NULL};
const Segment * const SG79889[] = {SGI339394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339395, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79890[] = {SGI339396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339397, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79891[] = {SGI339398, SGI0, SGI0, SGI339399, SGI0, SGI0, SGI0, SGI0, SGI339400, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79892[] = {SGI339401, SGI0, SGI0, SGI339402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79893[] = {SGI339403, SGI0, SGI0, SGI339404, SGI0, SGI0, SGI0, SGI0, SGI339405, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79894[] = {SGI339406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79895[] = {SGI339407, SGI339408, SGI0, SGI0, NULL};
const Segment * const SG79896[] = {SGI339409, SGI0, SGI0, SGI339410, SGI0, SGI0, SGI0, SGI0, SGI339411, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79897[] = {SGI339412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339413, SGI0, NULL};
const Segment * const SG79898[] = {SGI339414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339415, SGI0, NULL};
const Segment * const SG79899[] = {SGI339416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339417, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79900[] = {SGI339418, SGI0, SGI0, SGI339419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79901[] = {SGI339420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339421, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79902[] = {SGI339422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339423, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79903[] = {SGI339424, SGI0, SGI0, SGI339425, SGI0, SGI0, SGI0, SGI0, SGI339426, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79904[] = {SGI339427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339428, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79905[] = {SGI339429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339430, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79906[] = {SGI339431, SGI339432, SGI0, SGI0, NULL};
const Segment * const SG79907[] = {SGI339433, SGI339434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79908[] = {SGI339435, SGI339436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79909[] = {SGI339437, SGI339438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79910[] = {SGI339439, SGI339440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79911[] = {SGI339441, SGI339442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79912[] = {SGI339444, SGI339446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79913[] = {SGI339443, SGI0, SGI0, SGI339445, SGI0, SGI0, SGI0, SGI0, SGI339447, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79914[] = {SGI339448, SGI339449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79915[] = {SGI339450, SGI339451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79916[] = {SGI339452, SGI339453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79917[] = {SGI339454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79918[] = {SGI339455, SGI339456, SGI0, SGI0, NULL};
const Segment * const SG79919[] = {SGI339457, SGI339458, SGI0, SGI0, NULL};
const Segment * const SG79920[] = {SGI339459, SGI0, SGI339460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339460, SGI0, NULL};
const Segment * const SG79921[] = {SGI339461, SGI0, SGI339462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339462, SGI0, NULL};
const Segment * const SG79922[] = {SGI339463, SGI0, SGI339464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339464, SGI0, NULL};
const Segment * const SG79923[] = {SGI339465, SGI0, SGI339466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339466, SGI0, NULL};
const Segment * const SG79924[] = {SGI339467, SGI0, SGI339468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339468, SGI0, NULL};
const Segment * const SG79925[] = {SGI339469, SGI0, SGI339470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339470, SGI0, NULL};
const Segment * const SG79926[] = {SGI339471, SGI0, SGI339472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339472, SGI0, NULL};
const Segment * const SG79927[] = {SGI339473, SGI0, SGI339474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339474, SGI0, NULL};
const Segment * const SG79928[] = {SGI339475, SGI0, SGI339476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339476, SGI0, NULL};
const Segment * const SG79929[] = {SGI339477, SGI0, SGI339478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339478, SGI0, NULL};
const Segment * const SG79930[] = {SGI339479, SGI0, SGI339480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339480, SGI0, NULL};
const Segment * const SG79931[] = {SGI339481, SGI0, SGI339482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339482, SGI0, NULL};
const Segment * const SG79932[] = {SGI339483, SGI0, SGI339484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339485, SGI0, NULL};
const Segment * const SG79933[] = {SGI339486, SGI0, SGI339487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339487, SGI0, NULL};
const Segment * const SG79934[] = {SGI339488, SGI339489, SGI339490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339490, SGI0, NULL};
const Segment * const SG79935[] = {SGI339491, SGI339492, SGI339493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339493, SGI0, NULL};
const Segment * const SG79936[] = {SGI339494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339495, SGI0, NULL};
const Segment * const SG79937[] = {SGI339496, SGI0, SGI339497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339498, SGI0, NULL};
const Segment * const SG79938[] = {SGI339499, SGI0, SGI339500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339501, SGI0, NULL};
const Segment * const SG79939[] = {SGI339502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79940[] = {SGI339503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79941[] = {SGI339504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79942[] = {SGI339505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79943[] = {SGI339506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79944[] = {SGI339507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339508, SGI0, NULL};
const Segment * const SG79945[] = {SGI339509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339510, SGI0, NULL};
const Segment * const SG79946[] = {SGI0, SGI339511, SGI339512, SGI0, SGI0, SGI0, SGI339513, NULL};
const Segment * const SG79947[] = {SGI0, SGI339514, SGI339515, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79948[] = {SGI0, SGI339516, SGI339517, SGI0, SGI0, SGI0, SGI339518, NULL};
const Segment * const SG79949[] = {SGI339519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339520, SGI0, NULL};
const Segment * const SG79950[] = {SGI339521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339522, SGI0, NULL};
const Segment * const SG79951[] = {SGI339523, SGI339524, SGI0, SGI0, NULL};
const Segment * const SG79952[] = {SGI339525, SGI0, SGI0, SGI339526, SGI0, SGI0, SGI0, SGI0, SGI339527, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79953[] = {SGI339528, SGI0, SGI0, SGI339529, SGI0, SGI0, SGI0, SGI0, SGI339530, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79954[] = {SGI339531, SGI0, SGI0, SGI339532, SGI0, SGI0, SGI0, SGI0, SGI339533, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79955[] = {SGI339534, SGI0, SGI0, SGI339535, SGI0, SGI0, SGI0, SGI0, SGI339536, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79956[] = {SGI339537, SGI339538, SGI339539, SGI0, SGI0, SGI0, SGI0, SGI339540, SGI0, SGI0, NULL};
const Segment * const SG79957[] = {SGI339541, SGI0, SGI0, SGI339542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79958[] = {SGI339543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339544, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79959[] = {SGI339545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339546, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79960[] = {SGI339547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339548, SGI0, NULL};
const Segment * const SG79961[] = {SGI339549, SGI339550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79962[] = {SGI339551, SGI339552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79963[] = {SGI339553, SGI339554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79964[] = {SGI339555, SGI339556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79965[] = {SGI339557, SGI339558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79966[] = {SGI339559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79967[] = {SGI339560, SGI0, SGI0, SGI339561, SGI0, SGI0, SGI0, SGI0, SGI339562, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79968[] = {SGI339563, SGI0, SGI339564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79969[] = {SGI339565, SGI0, SGI0, SGI339566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79970[] = {SGI339567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339568, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79971[] = {SGI339569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339570, SGI0, NULL};
const Segment * const SG79972[] = {SGI339571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339572, SGI0, NULL};
const Segment * const SG79973[] = {SGI339573, SGI0, SGI0, SGI339574, SGI0, SGI0, SGI0, SGI0, SGI339575, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79974[] = {SGI339576, SGI0, SGI0, SGI339577, SGI0, SGI0, SGI0, SGI0, SGI339578, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79975[] = {SGI339579, SGI0, SGI0, SGI339580, SGI0, SGI0, SGI0, SGI0, SGI339581, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79976[] = {SGI339582, SGI0, SGI0, SGI339583, SGI0, SGI0, SGI0, SGI0, SGI339584, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79977[] = {SGI339585, SGI339586, SGI339587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339587, SGI0, NULL};
const Segment * const SG79978[] = {SGI339588, SGI339589, SGI339590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339591, SGI0, NULL};
const Segment * const SG79979[] = {SGI339592, SGI339593, SGI339594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339595, SGI0, NULL};
const Segment * const SG79980[] = {SGI339596, SGI339597, SGI339598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339599, SGI0, NULL};
const Segment * const SG79981[] = {SGI339600, SGI339601, SGI339602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339603, SGI0, NULL};
const Segment * const SG79982[] = {SGI339604, SGI339605, SGI339606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339607, SGI0, NULL};
const Segment * const SG79983[] = {SGI339608, SGI339609, SGI339610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339611, SGI0, NULL};
const Segment * const SG79984[] = {SGI339612, SGI339613, SGI339614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339615, SGI0, NULL};
const Segment * const SG79985[] = {SGI339616, SGI339617, SGI339618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339619, SGI0, NULL};
const Segment * const SG79986[] = {SGI339620, SGI339621, SGI339622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339623, SGI0, NULL};
const Segment * const SG79987[] = {SGI339624, SGI339625, SGI339626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339627, SGI0, NULL};
const Segment * const SG79988[] = {SGI339628, SGI339629, SGI339630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339631, SGI0, NULL};
const Segment * const SG79989[] = {SGI339632, SGI339633, SGI339634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339635, SGI0, NULL};
const Segment * const SG79990[] = {SGI339636, SGI339637, SGI339638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339639, SGI0, NULL};
const Segment * const SG79991[] = {SGI339640, SGI339641, SGI339642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339643, SGI0, NULL};
const Segment * const SG79992[] = {SGI339644, SGI339645, SGI339646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339647, SGI0, NULL};
const Segment * const SG79993[] = {SGI339648, SGI0, SGI0, SGI339649, SGI0, SGI0, SGI0, SGI0, SGI339650, SGI0, SGI0, SGI0, NULL};
const Segment * const SG79994[] = {SGI339651, SGI339652, SGI339653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339654, SGI0, NULL};
const Segment * const SG79995[] = {SGI339655, SGI339656, SGI339657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339658, SGI0, NULL};
const Segment * const SG79996[] = {SGI339659, SGI339660, SGI339661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339662, SGI0, NULL};
const Segment * const SG79997[] = {SGI339663, SGI339664, SGI339665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339666, SGI0, NULL};
const Segment * const SG79998[] = {SGI339667, SGI339668, SGI339669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339670, SGI0, NULL};
const Segment * const SG79999[] = {SGI339671, SGI339672, SGI339673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339674, SGI0, NULL};
const Segment * const SG80000[] = {SGI339675, SGI339676, SGI339677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339678, SGI0, NULL};
const Segment * const SG80001[] = {SGI339679, SGI339680, SGI339681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339682, SGI0, NULL};
const Segment * const SG80002[] = {SGI339683, SGI339684, SGI339685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339686, SGI0, NULL};
const Segment * const SG80003[] = {SGI339687, SGI339688, SGI339689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339690, SGI0, NULL};
const Segment * const SG80004[] = {SGI339691, SGI339692, SGI339693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339694, SGI0, NULL};
const Segment * const SG80005[] = {SGI339695, SGI339696, SGI339697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339698, SGI0, NULL};
const Segment * const SG80006[] = {SGI339699, SGI339700, SGI339701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339702, SGI0, NULL};
const Segment * const SG80007[] = {SGI339703, SGI339704, SGI339705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339706, SGI0, NULL};
const Segment * const SG80008[] = {SGI339707, SGI339708, SGI339709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339710, SGI0, NULL};
const Segment * const SG80009[] = {SGI339711, SGI339712, SGI339713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339714, SGI0, NULL};
const Segment * const SG80010[] = {SGI339715, SGI339716, SGI339717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339718, SGI0, NULL};
const Segment * const SG80011[] = {SGI339719, SGI339720, SGI339721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339722, SGI0, NULL};
const Segment * const SG80012[] = {SGI339723, SGI339724, SGI339725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339726, SGI0, NULL};
const Segment * const SG80013[] = {SGI339727, SGI339728, SGI339729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339730, SGI0, NULL};
const Segment * const SG80014[] = {SGI339731, SGI339732, SGI339733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339734, SGI0, NULL};
const Segment * const SG80015[] = {SGI339735, SGI339736, SGI339737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339738, SGI0, NULL};
const Segment * const SG80016[] = {SGI339739, SGI339740, SGI339741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339742, SGI0, NULL};
const Segment * const SG80017[] = {SGI339743, SGI339744, SGI339745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339746, SGI0, NULL};
const Segment * const SG80018[] = {SGI339747, SGI339748, SGI339749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339750, SGI0, NULL};
const Segment * const SG80019[] = {SGI339751, SGI339752, SGI339753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339754, SGI0, NULL};
const Segment * const SG80020[] = {SGI339755, SGI339756, SGI339757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339758, SGI0, NULL};
const Segment * const SG80021[] = {SGI339759, SGI339760, SGI339761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339762, SGI0, NULL};
const Segment * const SG80022[] = {SGI339763, SGI339764, SGI339765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339766, SGI0, NULL};
const Segment * const SG80023[] = {SGI339767, SGI339768, SGI339769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339770, SGI0, NULL};
const Segment * const SG80024[] = {SGI339771, SGI339772, SGI339773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339774, SGI0, NULL};
const Segment * const SG80025[] = {SGI339775, SGI339776, SGI339777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339778, SGI0, NULL};
const Segment * const SG80026[] = {SGI339779, SGI0, SGI339780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80027[] = {SGI339781, SGI339782, SGI339783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339784, SGI0, NULL};
const Segment * const SG80028[] = {SGI339785, SGI0, SGI0, SGI339786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80029[] = {SGI339787, SGI339788, SGI339789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339790, SGI0, NULL};
const Segment * const SG80030[] = {SGI339791, SGI0, SGI339792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80031[] = {SGI339793, SGI339794, SGI339795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339796, SGI0, NULL};
const Segment * const SG80032[] = {SGI339797, SGI339798, SGI339799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339800, SGI0, NULL};
const Segment * const SG80033[] = {SGI339801, SGI339802, SGI339803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339804, SGI0, NULL};
const Segment * const SG80034[] = {SGI339805, SGI339806, SGI339807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339808, SGI0, NULL};
const Segment * const SG80035[] = {SGI339809, SGI339810, SGI339811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339812, SGI0, NULL};
const Segment * const SG80036[] = {SGI339813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339815, SGI0, NULL};
const Segment * const SG80037[] = {SGI339814, SGI339816, SGI339817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339818, SGI0, NULL};
const Segment * const SG80038[] = {SGI339819, SGI339820, SGI339821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339822, SGI0, NULL};
const Segment * const SG80039[] = {SGI339823, SGI339824, SGI339825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339826, SGI0, NULL};
const Segment * const SG80040[] = {SGI339827, SGI339828, SGI339829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339830, SGI0, NULL};
const Segment * const SG80041[] = {SGI339831, SGI339832, SGI339833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339834, SGI0, NULL};
const Segment * const SG80042[] = {SGI339835, SGI339836, SGI339837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339838, SGI0, NULL};
const Segment * const SG80043[] = {SGI339839, SGI339839, SGI339840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339840, SGI0, NULL};
const Segment * const SG80044[] = {SGI339841, SGI339842, SGI339843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339844, SGI0, NULL};
const Segment * const SG80045[] = {SGI339845, SGI339846, SGI339847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339848, SGI0, NULL};
const Segment * const SG80046[] = {SGI339849, SGI339850, SGI339851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339852, SGI0, NULL};
const Segment * const SG80047[] = {SGI339853, SGI339854, SGI339855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339856, SGI0, NULL};
const Segment * const SG80048[] = {SGI339857, SGI339858, SGI339859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339860, SGI0, NULL};
const Segment * const SG80049[] = {SGI339861, SGI339862, SGI339863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339864, SGI0, NULL};
const Segment * const SG80050[] = {SGI339865, SGI339866, SGI339867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339868, SGI0, NULL};
const Segment * const SG80051[] = {SGI339869, SGI339870, SGI339871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339872, SGI0, NULL};
const Segment * const SG80052[] = {SGI339873, SGI339874, SGI339875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339876, SGI0, NULL};
const Segment * const SG80053[] = {SGI339877, SGI339878, SGI339879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339880, SGI0, NULL};
const Segment * const SG80054[] = {SGI339881, SGI339882, SGI339883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339884, SGI0, NULL};
const Segment * const SG80055[] = {SGI339885, SGI339886, SGI339887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339888, SGI0, NULL};
const Segment * const SG80056[] = {SGI339889, SGI339890, SGI339891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339892, SGI0, NULL};
const Segment * const SG80057[] = {SGI339893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339894, SGI0, NULL};
const Segment * const SG80058[] = {SGI339895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339896, SGI0, NULL};
const Segment * const SG80059[] = {SGI339897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339898, SGI0, NULL};
const Segment * const SG80060[] = {SGI339899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339900, SGI0, NULL};
const Segment * const SG80061[] = {SGI339901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339902, SGI0, NULL};
const Segment * const SG80062[] = {SGI339903, SGI339904, SGI0, SGI0, NULL};
const Segment * const SG80063[] = {SGI339905, SGI339906, SGI339907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339907, SGI0, NULL};
const Segment * const SG80064[] = {SGI339908, SGI339909, SGI339910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339911, SGI0, NULL};
const Segment * const SG80065[] = {SGI339912, SGI339913, SGI339914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339915, SGI0, NULL};
const Segment * const SG80066[] = {SGI339916, SGI339917, SGI339918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339919, SGI0, NULL};
const Segment * const SG80067[] = {SGI339920, SGI339921, SGI339922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339923, SGI0, NULL};
const Segment * const SG80068[] = {SGI339924, SGI339925, SGI339926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339927, SGI0, NULL};
const Segment * const SG80069[] = {SGI339928, SGI339929, SGI339930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339931, SGI0, NULL};
const Segment * const SG80070[] = {SGI339932, SGI339933, SGI339934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339935, SGI0, NULL};
const Segment * const SG80071[] = {SGI339936, SGI339937, SGI339938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339939, SGI0, NULL};
const Segment * const SG80072[] = {SGI339940, SGI339941, SGI339942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339943, SGI0, NULL};
const Segment * const SG80073[] = {SGI339944, SGI339945, SGI339946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339947, SGI0, NULL};
const Segment * const SG80074[] = {SGI339948, SGI339949, SGI339950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339951, SGI0, NULL};
const Segment * const SG80075[] = {SGI339952, SGI339953, SGI339954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339955, SGI0, NULL};
const Segment * const SG80076[] = {SGI339956, SGI339957, SGI339958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339959, SGI0, NULL};
const Segment * const SG80077[] = {SGI339960, SGI339961, SGI339962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339963, SGI0, NULL};
const Segment * const SG80078[] = {SGI339964, SGI339965, SGI339966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339967, SGI0, NULL};
const Segment * const SG80079[] = {SGI339968, SGI339969, SGI339970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339971, SGI0, NULL};
const Segment * const SG80080[] = {SGI339972, SGI339973, SGI339974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339975, SGI0, NULL};
const Segment * const SG80081[] = {SGI339976, SGI339977, SGI339978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339979, SGI0, NULL};
const Segment * const SG80082[] = {SGI339980, SGI339981, SGI339982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339983, SGI0, NULL};
const Segment * const SG80083[] = {SGI339984, SGI339985, SGI339986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339987, SGI0, NULL};
const Segment * const SG80084[] = {SGI339988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339989, SGI0, NULL};
const Segment * const SG80085[] = {SGI339990, SGI339991, SGI339992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339993, SGI0, NULL};
const Segment * const SG80086[] = {SGI339994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339995, SGI0, NULL};
const Segment * const SG80087[] = {SGI339996, SGI339997, SGI339998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI339999, SGI0, NULL};
const Segment * const SG80088[] = {SGI340000, SGI340001, SGI340002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340003, SGI0, NULL};
const Segment * const SG80089[] = {SGI340004, SGI340005, SGI340006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340007, SGI0, NULL};
const Segment * const SG80090[] = {SGI340008, SGI340009, SGI340010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340011, SGI0, NULL};
const Segment * const SG80091[] = {SGI340012, SGI340013, SGI340014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340015, SGI0, NULL};
const Segment * const SG80092[] = {SGI340016, SGI340017, SGI340018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340019, SGI0, NULL};
const Segment * const SG80093[] = {SGI340020, SGI340021, SGI340022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340023, SGI0, NULL};
const Segment * const SG80094[] = {SGI340024, SGI340025, SGI340026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340027, SGI0, NULL};
const Segment * const SG80095[] = {SGI340028, SGI340029, SGI340030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340031, SGI0, NULL};
const Segment * const SG80096[] = {SGI340032, SGI340033, SGI340034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340035, SGI0, NULL};
const Segment * const SG80097[] = {SGI340036, SGI340037, SGI340038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340039, SGI0, NULL};
const Segment * const SG80098[] = {SGI340040, SGI340041, SGI340042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340043, SGI0, NULL};
const Segment * const SG80099[] = {SGI340044, SGI340045, SGI340046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340047, SGI0, NULL};
const Segment * const SG80100[] = {SGI340048, SGI340049, SGI340050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340051, SGI0, NULL};
const Segment * const SG80101[] = {SGI340052, SGI340053, SGI340054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340055, SGI0, NULL};
const Segment * const SG80102[] = {SGI340056, SGI340057, SGI340058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340059, SGI0, NULL};
const Segment * const SG80103[] = {SGI340060, SGI340061, SGI340062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340063, SGI0, NULL};
const Segment * const SG80104[] = {SGI340064, SGI340065, SGI340066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340067, SGI0, NULL};
const Segment * const SG80105[] = {SGI340068, SGI340069, SGI340070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340071, SGI0, NULL};
const Segment * const SG80106[] = {SGI340072, SGI340073, SGI340074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340075, SGI0, NULL};
const Segment * const SG80107[] = {SGI340076, SGI340077, SGI340078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340079, SGI0, NULL};
const Segment * const SG80108[] = {SGI340080, SGI340081, SGI340082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340083, SGI0, NULL};
const Segment * const SG80109[] = {SGI340084, SGI340085, SGI340086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340087, SGI0, NULL};
const Segment * const SG80110[] = {SGI340088, SGI340089, SGI340090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340091, SGI0, NULL};
const Segment * const SG80111[] = {SGI340092, SGI340093, SGI340094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340095, SGI0, NULL};
const Segment * const SG80112[] = {SGI340096, SGI340097, SGI340098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340099, SGI0, NULL};
const Segment * const SG80113[] = {SGI340100, SGI340101, SGI340102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340103, SGI0, NULL};
const Segment * const SG80114[] = {SGI340104, SGI340105, SGI340106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340107, SGI0, NULL};
const Segment * const SG80115[] = {SGI340108, SGI340109, SGI340110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340111, SGI0, NULL};
const Segment * const SG80116[] = {SGI340112, SGI340113, SGI340114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340115, SGI0, NULL};
const Segment * const SG80117[] = {SGI340116, SGI340117, SGI340118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340119, SGI0, NULL};
const Segment * const SG80118[] = {SGI340120, SGI340121, SGI340122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340123, SGI0, NULL};
const Segment * const SG80119[] = {SGI340124, SGI340125, SGI340126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340127, SGI0, NULL};
const Segment * const SG80120[] = {SGI340128, SGI340129, SGI340130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340131, SGI0, NULL};
const Segment * const SG80121[] = {SGI340132, SGI340133, SGI340134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340135, SGI0, NULL};
const Segment * const SG80122[] = {SGI340136, SGI340137, SGI340138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340139, SGI0, NULL};
const Segment * const SG80123[] = {SGI340140, SGI340141, SGI340142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340143, SGI0, NULL};
const Segment * const SG80124[] = {SGI340144, SGI340145, SGI340146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340147, SGI0, NULL};
const Segment * const SG80125[] = {SGI340148, SGI340149, SGI340150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340151, SGI0, NULL};
const Segment * const SG80126[] = {SGI340152, SGI340153, SGI340154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340155, SGI0, NULL};
const Segment * const SG80127[] = {SGI340156, SGI340157, SGI340158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340159, SGI0, NULL};
const Segment * const SG80128[] = {SGI340160, SGI340161, SGI340162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340163, SGI0, NULL};
const Segment * const SG80129[] = {SGI340164, SGI340165, SGI340166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340167, SGI0, NULL};
const Segment * const SG80130[] = {SGI340168, SGI340169, SGI340170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340171, SGI0, NULL};
const Segment * const SG80131[] = {SGI340172, SGI340173, SGI340174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340175, SGI0, NULL};
const Segment * const SG80132[] = {SGI340176, SGI340177, SGI340178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340179, SGI0, NULL};
const Segment * const SG80133[] = {SGI340180, SGI340181, SGI340182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340183, SGI0, NULL};
const Segment * const SG80134[] = {SGI340184, SGI340185, SGI340186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340187, SGI0, NULL};
const Segment * const SG80135[] = {SGI340188, SGI340189, SGI340190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340191, SGI0, NULL};
const Segment * const SG80136[] = {SGI340192, SGI340193, SGI340194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340195, SGI0, NULL};
const Segment * const SG80137[] = {SGI340196, SGI340197, SGI340198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340199, SGI0, NULL};
const Segment * const SG80138[] = {SGI340200, SGI340201, SGI340202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340203, SGI0, NULL};
const Segment * const SG80139[] = {SGI340204, SGI340205, SGI340206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340207, SGI0, NULL};
const Segment * const SG80140[] = {SGI340208, SGI340209, SGI340210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340211, SGI0, NULL};
const Segment * const SG80141[] = {SGI340212, SGI340213, SGI340214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340215, SGI0, NULL};
const Segment * const SG80142[] = {SGI340216, SGI340217, SGI340218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340219, SGI0, NULL};
const Segment * const SG80143[] = {SGI340220, SGI340221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80144[] = {SGI340222, SGI340223, SGI340224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340225, SGI0, NULL};
const Segment * const SG80145[] = {SGI340226, SGI340228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80146[] = {SGI340227, SGI340229, SGI340230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340231, SGI0, NULL};
const Segment * const SG80147[] = {SGI340232, SGI340234, SGI340235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340237, SGI0, NULL};
const Segment * const SG80148[] = {SGI340233, SGI340236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80149[] = {SGI340238, SGI340239, SGI340240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340241, SGI0, NULL};
const Segment * const SG80150[] = {SGI340242, SGI340243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80151[] = {SGI340244, SGI340245, SGI340246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340247, SGI0, NULL};
const Segment * const SG80152[] = {SGI340248, SGI340249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80153[] = {SGI340250, SGI340251, SGI340252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340253, SGI0, NULL};
const Segment * const SG80154[] = {SGI340254, SGI340255, SGI340256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340257, SGI0, NULL};
const Segment * const SG80155[] = {SGI340258, SGI340259, SGI340260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340261, SGI0, NULL};
const Segment * const SG80156[] = {SGI340262, SGI340263, SGI340264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340265, SGI0, NULL};
const Segment * const SG80157[] = {SGI340266, SGI340267, SGI340268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340269, SGI0, NULL};
const Segment * const SG80158[] = {SGI340270, SGI340271, SGI340272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340273, SGI0, NULL};
const Segment * const SG80159[] = {SGI340274, SGI340275, SGI340276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340277, SGI0, NULL};
const Segment * const SG80160[] = {SGI340278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80161[] = {SGI340279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80162[] = {SGI340280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80163[] = {SGI340281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80164[] = {SGI340282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80165[] = {SGI340283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80166[] = {SGI340284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80167[] = {SGI340285, SGI340286, SGI340287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340287, SGI0, NULL};
const Segment * const SG80168[] = {SGI340288, SGI340289, SGI340290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340291, SGI0, NULL};
const Segment * const SG80169[] = {SGI340292, SGI340293, SGI340294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340295, SGI0, NULL};
const Segment * const SG80170[] = {SGI340296, SGI340297, SGI340298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340299, SGI0, NULL};
const Segment * const SG80171[] = {SGI340300, SGI340301, SGI340302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340303, SGI0, NULL};
const Segment * const SG80172[] = {SGI340304, SGI340305, SGI340306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340307, SGI0, NULL};
const Segment * const SG80173[] = {SGI340308, SGI340309, SGI340310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340311, SGI0, NULL};
const Segment * const SG80174[] = {SGI340312, SGI340313, SGI340314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340315, SGI0, NULL};
const Segment * const SG80175[] = {SGI340316, SGI340317, SGI340318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340319, SGI0, NULL};
const Segment * const SG80176[] = {SGI340320, SGI340321, SGI340322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340323, SGI0, NULL};
const Segment * const SG80177[] = {SGI340324, SGI340325, SGI340326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340327, SGI0, NULL};
const Segment * const SG80178[] = {SGI340328, SGI340329, SGI340330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340331, SGI0, NULL};
const Segment * const SG80179[] = {SGI340332, SGI340333, SGI340334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340335, SGI0, NULL};
const Segment * const SG80180[] = {SGI340336, SGI340337, SGI340338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340339, SGI0, NULL};
const Segment * const SG80181[] = {SGI340340, SGI340341, SGI340342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340343, SGI0, NULL};
const Segment * const SG80182[] = {SGI340344, SGI340345, SGI340346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340347, SGI0, NULL};
const Segment * const SG80183[] = {SGI340348, SGI340349, SGI340350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340351, SGI0, NULL};
const Segment * const SG80184[] = {SGI340352, SGI340353, SGI340354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340355, SGI0, NULL};
const Segment * const SG80185[] = {SGI340356, SGI340357, SGI340358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340359, SGI0, NULL};
const Segment * const SG80186[] = {SGI340360, SGI340361, SGI340362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340363, SGI0, NULL};
const Segment * const SG80187[] = {SGI340364, SGI340365, SGI340366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340367, SGI0, NULL};
const Segment * const SG80188[] = {SGI340368, SGI340369, SGI340370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340371, SGI0, NULL};
const Segment * const SG80189[] = {SGI340372, SGI340373, SGI340374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340375, SGI0, NULL};
const Segment * const SG80190[] = {SGI340376, SGI340377, SGI340378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340379, SGI0, NULL};
const Segment * const SG80191[] = {SGI340380, SGI340380, SGI340381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340381, SGI0, NULL};
const Segment * const SG80192[] = {SGI340382, SGI340383, SGI340384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340385, SGI0, NULL};
const Segment * const SG80193[] = {SGI340386, SGI340387, SGI340388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340389, SGI0, NULL};
const Segment * const SG80194[] = {SGI340390, SGI340391, SGI340392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340393, SGI0, NULL};
const Segment * const SG80195[] = {SGI340394, SGI340395, SGI340396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340397, SGI0, NULL};
const Segment * const SG80196[] = {SGI340398, SGI340399, SGI340400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340401, SGI0, NULL};
const Segment * const SG80197[] = {SGI340402, SGI340403, SGI340404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340405, SGI0, NULL};
const Segment * const SG80198[] = {SGI340406, SGI340407, SGI340408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340409, SGI0, NULL};
const Segment * const SG80199[] = {SGI340410, SGI340411, SGI340412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340413, SGI0, NULL};
const Segment * const SG80200[] = {SGI340414, SGI340415, SGI340416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340417, SGI0, NULL};
const Segment * const SG80201[] = {SGI340418, SGI340419, SGI340420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340421, SGI0, NULL};
const Segment * const SG80202[] = {SGI340422, SGI340423, SGI340424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340425, SGI0, NULL};
const Segment * const SG80203[] = {SGI340426, SGI340427, SGI340428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340429, SGI0, NULL};
const Segment * const SG80204[] = {SGI340430, SGI340431, SGI340432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340433, SGI0, NULL};
const Segment * const SG80205[] = {SGI340434, SGI340435, SGI340436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340437, SGI0, NULL};
const Segment * const SG80206[] = {SGI340438, SGI340439, SGI340440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340441, SGI0, NULL};
const Segment * const SG80207[] = {SGI340442, SGI0, SGI0, SGI340443, SGI0, SGI0, SGI0, SGI0, SGI340444, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80208[] = {SGI340445, SGI0, SGI0, SGI340446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80209[] = {SGI340447, SGI0, SGI0, SGI340448, SGI0, SGI0, SGI0, SGI0, SGI340449, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80210[] = {SGI340450, SGI0, SGI0, SGI340451, SGI0, SGI0, SGI0, SGI0, SGI340452, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80211[] = {SGI340453, SGI0, SGI0, SGI340454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80212[] = {SGI340455, SGI0, SGI0, SGI340456, SGI0, SGI0, SGI0, SGI0, SGI340457, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80213[] = {SGI340458, SGI0, SGI0, SGI340459, SGI0, SGI0, SGI0, SGI0, SGI340460, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80214[] = {SGI340461, SGI0, SGI0, SGI340462, SGI0, SGI0, SGI0, SGI0, SGI340463, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80215[] = {SGI340464, SGI0, SGI0, SGI340465, SGI0, SGI0, SGI0, SGI0, SGI340466, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80216[] = {SGI340467, SGI0, SGI0, SGI340468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80217[] = {SGI340469, SGI0, SGI0, SGI340470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80218[] = {SGI340471, SGI0, SGI0, SGI340472, SGI0, SGI0, SGI0, SGI0, SGI340473, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80219[] = {SGI340474, SGI0, SGI0, SGI340475, SGI0, SGI0, SGI0, SGI0, SGI340476, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80220[] = {SGI340477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80221[] = {SGI340478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80222[] = {SGI340479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80223[] = {SGI340480, SGI340481, SGI340482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340482, SGI0, NULL};
const Segment * const SG80224[] = {SGI340483, SGI340484, SGI340485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340486, SGI0, NULL};
const Segment * const SG80225[] = {SGI340487, SGI340488, SGI340489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340490, SGI0, NULL};
const Segment * const SG80226[] = {SGI340491, SGI340492, SGI340493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340494, SGI0, NULL};
const Segment * const SG80227[] = {SGI340495, SGI340496, SGI340497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340498, SGI0, NULL};
const Segment * const SG80228[] = {SGI340499, SGI340500, SGI340501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340501, SGI0, NULL};
const Segment * const SG80229[] = {SGI340502, SGI340503, SGI340504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340505, SGI0, NULL};
const Segment * const SG80230[] = {SGI340506, SGI340507, SGI340508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340509, SGI0, NULL};
const Segment * const SG80231[] = {SGI340510, SGI340511, SGI340512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340513, SGI0, NULL};
const Segment * const SG80232[] = {SGI340514, SGI340515, SGI340516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340517, SGI0, NULL};
const Segment * const SG80233[] = {SGI340518, SGI340519, SGI340520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340521, SGI0, NULL};
const Segment * const SG80234[] = {SGI340522, SGI340523, SGI340524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340525, SGI0, NULL};
const Segment * const SG80235[] = {SGI340526, SGI340527, SGI340528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340529, SGI0, NULL};
const Segment * const SG80236[] = {SGI340530, SGI340531, SGI340532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340533, SGI0, NULL};
const Segment * const SG80237[] = {SGI340534, SGI340535, SGI340536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340536, SGI0, NULL};
const Segment * const SG80238[] = {SGI340537, SGI340538, SGI340539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340540, SGI0, NULL};
const Segment * const SG80239[] = {SGI340541, SGI340542, SGI340543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340544, SGI0, NULL};
const Segment * const SG80240[] = {SGI340545, SGI340546, SGI340547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340548, SGI0, NULL};
const Segment * const SG80241[] = {SGI340549, SGI340550, SGI340551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340552, SGI0, NULL};
const Segment * const SG80242[] = {SGI340553, SGI340554, SGI340555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340556, SGI0, NULL};
const Segment * const SG80243[] = {SGI340557, SGI340558, SGI340559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340560, SGI0, NULL};
const Segment * const SG80244[] = {SGI340561, SGI340562, SGI340563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340563, SGI0, NULL};
const Segment * const SG80245[] = {SGI340564, SGI340565, SGI340566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340567, SGI0, NULL};
const Segment * const SG80246[] = {SGI340568, SGI340569, SGI340570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340571, SGI0, NULL};
const Segment * const SG80247[] = {SGI340572, SGI340573, SGI340574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340575, SGI0, NULL};
const Segment * const SG80248[] = {SGI340576, SGI340577, SGI340578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340579, SGI0, NULL};
const Segment * const SG80249[] = {SGI340580, SGI340581, SGI340582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340583, SGI0, NULL};
const Segment * const SG80250[] = {SGI340584, SGI340585, SGI340586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340587, SGI0, NULL};
const Segment * const SG80251[] = {SGI340588, SGI340589, SGI340590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340591, SGI0, NULL};
const Segment * const SG80252[] = {SGI340592, SGI340593, SGI340594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340595, SGI0, NULL};
const Segment * const SG80253[] = {SGI340596, SGI340597, SGI340598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340599, SGI0, NULL};
const Segment * const SG80254[] = {SGI340600, SGI340601, SGI340602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340603, SGI0, NULL};
const Segment * const SG80255[] = {SGI340604, SGI340605, SGI340606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340607, SGI0, NULL};
const Segment * const SG80256[] = {SGI340608, SGI340609, SGI340610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340611, SGI0, NULL};
const Segment * const SG80257[] = {SGI340612, SGI340613, SGI340614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340615, SGI0, NULL};
const Segment * const SG80258[] = {SGI340616, SGI340617, SGI340618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340619, SGI0, NULL};
const Segment * const SG80259[] = {SGI340620, SGI340621, SGI340622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340623, SGI0, NULL};
const Segment * const SG80260[] = {SGI340624, SGI340625, SGI340626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340627, SGI0, NULL};
const Segment * const SG80261[] = {SGI340628, SGI340629, SGI340630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340631, SGI0, NULL};
const Segment * const SG80262[] = {SGI340632, SGI340633, SGI340634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340635, SGI0, NULL};
const Segment * const SG80263[] = {SGI340636, SGI340637, SGI340638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340639, SGI0, NULL};
const Segment * const SG80264[] = {SGI340640, SGI340641, SGI340642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340643, SGI0, NULL};
const Segment * const SG80265[] = {SGI340644, SGI340645, SGI340646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340647, SGI0, NULL};
const Segment * const SG80266[] = {SGI340648, SGI340649, SGI340650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340651, SGI0, NULL};
const Segment * const SG80267[] = {SGI340652, SGI340653, SGI340654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340655, SGI0, NULL};
const Segment * const SG80268[] = {SGI340656, SGI340657, SGI340658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340659, SGI0, NULL};
const Segment * const SG80269[] = {SGI340660, SGI340661, SGI340662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340663, SGI0, NULL};
const Segment * const SG80270[] = {SGI340664, SGI340665, SGI340666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340667, SGI0, NULL};
const Segment * const SG80271[] = {SGI340668, SGI340669, SGI340670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340671, SGI0, NULL};
const Segment * const SG80272[] = {SGI340672, SGI340673, SGI340674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340675, SGI0, NULL};
const Segment * const SG80273[] = {SGI340676, SGI340677, SGI340678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340679, SGI0, NULL};
const Segment * const SG80274[] = {SGI340680, SGI340681, SGI340682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340683, SGI0, NULL};
const Segment * const SG80275[] = {SGI340684, SGI340685, SGI340686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340687, SGI0, NULL};
const Segment * const SG80276[] = {SGI340688, SGI340689, SGI340690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340691, SGI0, NULL};
const Segment * const SG80277[] = {SGI340692, SGI340693, SGI340694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340695, SGI0, NULL};
const Segment * const SG80278[] = {SGI340696, SGI340697, SGI340698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340699, SGI0, NULL};
const Segment * const SG80279[] = {SGI340700, SGI340701, SGI340702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340703, SGI0, NULL};
const Segment * const SG80280[] = {SGI340704, SGI340705, SGI340706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340707, SGI0, NULL};
const Segment * const SG80281[] = {SGI340708, SGI340709, SGI340710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340711, SGI0, NULL};
const Segment * const SG80282[] = {SGI340712, SGI340713, SGI340714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340715, SGI0, NULL};
const Segment * const SG80283[] = {SGI340716, SGI340717, SGI340718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340719, SGI0, NULL};
const Segment * const SG80284[] = {SGI340720, SGI340721, SGI340722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340723, SGI0, NULL};
const Segment * const SG80285[] = {SGI340724, SGI340725, SGI340726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340727, SGI0, NULL};
const Segment * const SG80286[] = {SGI340728, SGI340729, SGI340730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340731, SGI0, NULL};
const Segment * const SG80287[] = {SGI340732, SGI340733, SGI340734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340735, SGI0, NULL};
const Segment * const SG80288[] = {SGI340736, SGI340737, SGI340738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340739, SGI0, NULL};
const Segment * const SG80289[] = {SGI340740, SGI340741, SGI340742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340743, SGI0, NULL};
const Segment * const SG80290[] = {SGI340744, SGI340745, SGI340746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340747, SGI0, NULL};
const Segment * const SG80291[] = {SGI340748, SGI340749, SGI340750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340751, SGI0, NULL};
const Segment * const SG80292[] = {SGI340752, SGI340753, SGI340754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340755, SGI0, NULL};
const Segment * const SG80293[] = {SGI340756, SGI340757, SGI340758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340759, SGI0, NULL};
const Segment * const SG80294[] = {SGI340760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80295[] = {SGI340761, SGI340762, SGI340763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340764, SGI0, NULL};
const Segment * const SG80296[] = {SGI340765, SGI340767, SGI340768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340769, SGI0, NULL};
const Segment * const SG80297[] = {SGI340766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80298[] = {SGI340770, SGI340771, SGI340772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340774, SGI0, NULL};
const Segment * const SG80299[] = {SGI340773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80300[] = {SGI340775, SGI340776, SGI340777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340778, SGI0, NULL};
const Segment * const SG80301[] = {SGI340779, SGI340780, SGI340781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340782, SGI0, NULL};
const Segment * const SG80302[] = {SGI340783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80303[] = {SGI340784, SGI340785, SGI340786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340787, SGI0, NULL};
const Segment * const SG80304[] = {SGI340788, SGI340789, SGI340790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340791, SGI0, NULL};
const Segment * const SG80305[] = {SGI340792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80306[] = {SGI340793, SGI340794, SGI340795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340796, SGI0, NULL};
const Segment * const SG80307[] = {SGI340797, SGI340798, SGI340799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340800, SGI0, NULL};
const Segment * const SG80308[] = {SGI340801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80309[] = {SGI340802, SGI340803, SGI340804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340805, SGI0, NULL};
const Segment * const SG80310[] = {SGI340806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80311[] = {SGI340807, SGI340808, SGI340809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340810, SGI0, NULL};
const Segment * const SG80312[] = {SGI340811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80313[] = {SGI340812, SGI340813, SGI340814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340815, SGI0, NULL};
const Segment * const SG80314[] = {SGI340816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80315[] = {SGI340817, SGI340818, SGI340819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340820, SGI0, NULL};
const Segment * const SG80316[] = {SGI340821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80317[] = {SGI340822, SGI340823, SGI340824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340825, SGI0, NULL};
const Segment * const SG80318[] = {SGI340826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80319[] = {SGI340827, SGI340828, SGI340829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340830, SGI0, NULL};
const Segment * const SG80320[] = {SGI340831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80321[] = {SGI340832, SGI340833, SGI340834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340835, SGI0, NULL};
const Segment * const SG80322[] = {SGI340836, SGI340837, SGI340838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340840, SGI0, NULL};
const Segment * const SG80323[] = {SGI340839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80324[] = {SGI340841, SGI340842, SGI340843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340844, SGI0, NULL};
const Segment * const SG80325[] = {SGI340845, SGI340846, SGI340847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340848, SGI0, NULL};
const Segment * const SG80326[] = {SGI340849, SGI340850, SGI340851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340852, SGI0, NULL};
const Segment * const SG80327[] = {SGI340853, SGI340854, SGI340855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340856, SGI0, NULL};
const Segment * const SG80328[] = {SGI340857, SGI340858, SGI340859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340860, SGI0, NULL};
const Segment * const SG80329[] = {SGI340861, SGI340862, SGI340863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340864, SGI0, NULL};
const Segment * const SG80330[] = {SGI340865, SGI340866, SGI340867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340868, SGI0, NULL};
const Segment * const SG80331[] = {SGI340869, SGI340870, SGI340871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340872, SGI0, NULL};
const Segment * const SG80332[] = {SGI340873, SGI340874, SGI340875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340876, SGI0, NULL};
const Segment * const SG80333[] = {SGI340877, SGI340878, SGI340879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340880, SGI0, NULL};
const Segment * const SG80334[] = {SGI340881, SGI340882, SGI340883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340884, SGI0, NULL};
const Segment * const SG80335[] = {SGI340885, SGI0, SGI0, SGI340886, SGI0, SGI0, SGI0, SGI0, SGI340887, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80336[] = {SGI340888, SGI0, SGI0, SGI340889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80337[] = {SGI340890, SGI0, SGI0, SGI340891, SGI0, SGI0, SGI0, SGI0, SGI340892, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80338[] = {SGI340893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340894, SGI0, NULL};
const Segment * const SG80339[] = {SGI340895, SGI340896, SGI340897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340897, SGI0, NULL};
const Segment * const SG80340[] = {SGI340898, SGI340899, SGI340901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340902, SGI0, NULL};
const Segment * const SG80341[] = {SGI340900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340903, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80342[] = {SGI340904, SGI340905, SGI340906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340907, SGI0, NULL};
const Segment * const SG80343[] = {SGI340908, SGI340909, SGI340910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340911, SGI0, NULL};
const Segment * const SG80344[] = {SGI340912, SGI340913, SGI340914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340915, SGI0, NULL};
const Segment * const SG80345[] = {SGI340916, SGI340917, SGI340918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340919, SGI0, NULL};
const Segment * const SG80346[] = {SGI340920, SGI340921, SGI340922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340923, SGI0, NULL};
const Segment * const SG80347[] = {SGI340924, SGI340925, SGI340926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340927, SGI0, NULL};
const Segment * const SG80348[] = {SGI340928, SGI340929, SGI340930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340931, SGI0, NULL};
const Segment * const SG80349[] = {SGI340932, SGI340933, SGI340934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340935, SGI0, NULL};
const Segment * const SG80350[] = {SGI340936, SGI340937, SGI340938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340939, SGI0, NULL};
const Segment * const SG80351[] = {SGI340940, SGI340941, SGI340942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340943, SGI0, NULL};
const Segment * const SG80352[] = {SGI340944, SGI340945, SGI340946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340947, SGI0, NULL};
const Segment * const SG80353[] = {SGI340948, SGI340949, SGI340950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340951, SGI0, NULL};
const Segment * const SG80354[] = {SGI340952, SGI340953, SGI340954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340955, SGI0, NULL};
const Segment * const SG80355[] = {SGI340956, SGI340957, SGI340958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340959, SGI0, NULL};
const Segment * const SG80356[] = {SGI340960, SGI340961, SGI340962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340963, SGI0, NULL};
const Segment * const SG80357[] = {SGI340964, SGI340965, SGI340966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340967, SGI0, NULL};
const Segment * const SG80358[] = {SGI340968, SGI340969, SGI340970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340971, SGI0, NULL};
const Segment * const SG80359[] = {SGI340972, SGI340973, SGI340974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340975, SGI0, NULL};
const Segment * const SG80360[] = {SGI340976, SGI340977, SGI340978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340979, SGI0, NULL};
const Segment * const SG80361[] = {SGI340980, SGI340981, SGI340982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340983, SGI0, NULL};
const Segment * const SG80362[] = {SGI340984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340985, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80363[] = {SGI340986, SGI340987, SGI340988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340989, SGI0, NULL};
const Segment * const SG80364[] = {SGI340990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340991, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80365[] = {SGI340992, SGI340993, SGI340994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI340995, SGI0, NULL};
const Segment * const SG80366[] = {SGI340996, SGI0, SGI0, SGI340997, SGI0, SGI0, SGI0, SGI0, SGI340998, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80367[] = {SGI340999, SGI341000, SGI341001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341002, SGI0, NULL};
const Segment * const SG80368[] = {SGI341003, SGI341004, SGI341005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341006, SGI0, NULL};
const Segment * const SG80369[] = {SGI341007, SGI341008, SGI341009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341010, SGI0, NULL};
const Segment * const SG80370[] = {SGI341011, SGI341012, SGI341013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341014, SGI0, NULL};
const Segment * const SG80371[] = {SGI341015, SGI341016, SGI341017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341018, SGI0, NULL};
const Segment * const SG80372[] = {SGI341019, SGI341020, SGI341021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341022, SGI0, NULL};
const Segment * const SG80373[] = {SGI341023, SGI341024, SGI341025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341026, SGI0, NULL};
const Segment * const SG80374[] = {SGI341027, SGI341028, SGI341029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341030, SGI0, NULL};
const Segment * const SG80375[] = {SGI341031, SGI341032, SGI341033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341034, SGI0, NULL};
const Segment * const SG80376[] = {SGI341035, SGI341036, SGI341037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341038, SGI0, NULL};
const Segment * const SG80377[] = {SGI341039, SGI341040, SGI341041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341042, SGI0, NULL};
const Segment * const SG80378[] = {SGI341043, SGI341044, SGI341045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341046, SGI0, NULL};
const Segment * const SG80379[] = {SGI341047, SGI341048, SGI341049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341050, SGI0, NULL};
const Segment * const SG80380[] = {SGI341051, SGI341052, SGI341053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341054, SGI0, NULL};
const Segment * const SG80381[] = {SGI341055, SGI341056, SGI341057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341058, SGI0, NULL};
const Segment * const SG80382[] = {SGI341059, SGI341060, SGI341061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341062, SGI0, NULL};
const Segment * const SG80383[] = {SGI341063, SGI341064, SGI341065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341066, SGI0, NULL};
const Segment * const SG80384[] = {SGI341067, SGI341068, SGI341069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341070, SGI0, NULL};
const Segment * const SG80385[] = {SGI341071, SGI341072, SGI341073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341074, SGI0, NULL};
const Segment * const SG80386[] = {SGI341075, SGI341076, SGI341077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341078, SGI0, NULL};
const Segment * const SG80387[] = {SGI341079, SGI341080, SGI341081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341082, SGI0, NULL};
const Segment * const SG80388[] = {SGI341083, SGI341084, SGI341085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341086, SGI0, NULL};
const Segment * const SG80389[] = {SGI341087, SGI341088, SGI341089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341090, SGI0, NULL};
const Segment * const SG80390[] = {SGI341091, SGI341092, SGI341093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341094, SGI0, NULL};
const Segment * const SG80391[] = {SGI341095, SGI341096, SGI341097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341098, SGI0, NULL};
const Segment * const SG80392[] = {SGI341099, SGI341100, SGI341101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341102, SGI0, NULL};
const Segment * const SG80393[] = {SGI341103, SGI341104, SGI341105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341106, SGI0, NULL};
const Segment * const SG80394[] = {SGI341107, SGI341108, SGI341109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341110, SGI0, NULL};
const Segment * const SG80395[] = {SGI341111, SGI341112, SGI341113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341114, SGI0, NULL};
const Segment * const SG80396[] = {SGI341115, SGI341116, SGI341117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341118, SGI0, NULL};
const Segment * const SG80397[] = {SGI341119, SGI341120, SGI341121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341122, SGI0, NULL};
const Segment * const SG80398[] = {SGI341123, SGI341124, SGI341125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341126, SGI0, NULL};
const Segment * const SG80399[] = {SGI341127, SGI341128, SGI341129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341130, SGI0, NULL};
const Segment * const SG80400[] = {SGI341131, SGI341132, SGI341133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341134, SGI0, NULL};
const Segment * const SG80401[] = {SGI341135, SGI341136, SGI341137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341138, SGI0, NULL};
const Segment * const SG80402[] = {SGI341139, SGI341140, SGI341141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341142, SGI0, NULL};
const Segment * const SG80403[] = {SGI341143, SGI341144, SGI341145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341146, SGI0, NULL};
const Segment * const SG80404[] = {SGI341147, SGI341148, SGI341149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341150, SGI0, NULL};
const Segment * const SG80405[] = {SGI341151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341152, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80406[] = {SGI341153, SGI0, SGI0, SGI341154, SGI0, SGI0, SGI0, SGI0, SGI341155, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80407[] = {SGI341156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341157, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80408[] = {SGI341158, SGI0, SGI0, SGI341159, SGI0, SGI0, SGI0, SGI0, SGI341160, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80409[] = {SGI341161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80410[] = {SGI341162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341163, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80411[] = {SGI341164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341165, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80412[] = {SGI341166, SGI0, SGI0, SGI341167, SGI0, SGI0, SGI0, SGI0, SGI341168, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80413[] = {SGI341169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341170, SGI0, NULL};
const Segment * const SG80414[] = {SGI341171, SGI341172, SGI0, SGI0, NULL};
const Segment * const SG80415[] = {SGI341173, SGI341174, SGI0, SGI0, NULL};
const Segment * const SG80416[] = {SGI341175, SGI341176, SGI0, SGI0, NULL};
const Segment * const SG80417[] = {SGI341177, SGI341178, SGI0, SGI0, NULL};
const Segment * const SG80418[] = {SGI341179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341180, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80419[] = {SGI341181, SGI0, SGI341182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80420[] = {SGI341183, SGI0, SGI0, SGI341184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80421[] = {SGI341185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341186, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80422[] = {SGI341187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341188, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80423[] = {SGI341189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341190, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80424[] = {SGI341191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341192, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80425[] = {SGI341193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341194, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80426[] = {SGI341195, SGI0, SGI0, SGI341196, SGI0, SGI0, SGI0, SGI0, SGI341197, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80427[] = {SGI341198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80428[] = {SGI341199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80429[] = {SGI341200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80430[] = {SGI341201, SGI0, SGI0, SGI341202, SGI0, SGI0, SGI0, SGI0, SGI341203, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80431[] = {SGI341204, SGI0, SGI0, SGI341205, SGI0, SGI0, SGI0, SGI0, SGI341206, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80432[] = {SGI341207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341208, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80433[] = {SGI341209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341210, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80434[] = {SGI341211, SGI0, SGI0, SGI341212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80435[] = {SGI341213, SGI0, SGI0, SGI341214, SGI0, SGI0, SGI0, SGI0, SGI341215, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80436[] = {SGI341216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341217, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80437[] = {SGI341218, SGI0, SGI0, SGI341219, SGI0, SGI0, SGI0, SGI0, SGI341220, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80438[] = {SGI341221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80439[] = {SGI341222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80440[] = {SGI341223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80441[] = {SGI341224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80442[] = {SGI341225, SGI341226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80443[] = {SGI341227, SGI341228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80444[] = {SGI341229, SGI341230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80445[] = {SGI341231, SGI341232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80446[] = {SGI341233, SGI341234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80447[] = {SGI341235, SGI341236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80448[] = {SGI341237, SGI341238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80449[] = {SGI341239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341240, SGI0, NULL};
const Segment * const SG80450[] = {SGI341241, SGI0, SGI341242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80451[] = {SGI341243, SGI0, SGI0, SGI341244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80452[] = {SGI341245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80453[] = {SGI341246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80454[] = {SGI341247, SGI341248, SGI0, SGI0, NULL};
const Segment * const SG80455[] = {SGI341249, SGI341250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80456[] = {SGI341251, SGI341252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80457[] = {SGI341253, SGI341254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80458[] = {SGI0, SGI341255, SGI341256, SGI0, SGI0, SGI0, SGI341257, NULL};
const Segment * const SG80459[] = {SGI0, SGI341258, SGI341259, SGI0, SGI0, SGI0, SGI341260, NULL};
const Segment * const SG80460[] = {SGI341261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341262, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80461[] = {SGI341263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341264, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80462[] = {SGI341265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80463[] = {SGI341266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80464[] = {SGI341267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80465[] = {SGI341268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80466[] = {SGI341269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80467[] = {SGI341270, SGI0, SGI0, SGI341271, SGI0, SGI0, SGI0, SGI0, SGI341272, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80468[] = {SGI341273, SGI0, SGI0, SGI341274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80469[] = {SGI341275, SGI0, SGI0, SGI341276, SGI0, SGI0, SGI0, SGI0, SGI341277, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80470[] = {SGI341278, SGI0, SGI0, SGI341279, SGI0, SGI0, SGI0, SGI0, SGI341280, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80471[] = {SGI341281, SGI0, SGI0, SGI341282, SGI0, SGI0, SGI0, SGI0, SGI341283, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80472[] = {SGI341284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341285, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80473[] = {SGI341286, SGI0, SGI0, SGI341287, SGI0, SGI0, SGI0, SGI0, SGI341288, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80474[] = {SGI341289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341290, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80475[] = {SGI341291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80476[] = {SGI341292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80477[] = {SGI341293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80478[] = {SGI341294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80479[] = {SGI341295, SGI0, SGI0, SGI341296, SGI0, SGI0, SGI0, SGI0, SGI341297, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80480[] = {SGI341298, SGI0, SGI0, SGI341299, SGI0, SGI0, SGI0, SGI0, SGI341300, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80481[] = {SGI341301, SGI0, SGI0, SGI341302, SGI0, SGI0, SGI0, SGI0, SGI341303, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80482[] = {SGI341304, SGI0, SGI0, SGI341305, SGI0, SGI0, SGI0, SGI0, SGI341306, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80483[] = {SGI341307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80484[] = {SGI341308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80485[] = {SGI341309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341310, SGI0, NULL};
const Segment * const SG80486[] = {SGI341311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341312, SGI0, NULL};
const Segment * const SG80487[] = {SGI341313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341314, SGI0, NULL};
const Segment * const SG80488[] = {SGI341315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341316, SGI0, NULL};
const Segment * const SG80489[] = {SGI341317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341318, SGI0, NULL};
const Segment * const SG80490[] = {SGI341319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341320, SGI0, NULL};
const Segment * const SG80491[] = {SGI341321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341322, SGI0, NULL};
const Segment * const SG80492[] = {SGI341323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341324, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80493[] = {SGI341325, SGI0, SGI0, SGI341326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80494[] = {SGI341327, SGI0, SGI0, SGI341328, SGI0, SGI0, SGI0, SGI0, SGI341329, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80495[] = {SGI341330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341331, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80496[] = {SGI341332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341333, SGI0, NULL};
const Segment * const SG80497[] = {SGI341334, SGI341335, SGI0, SGI0, NULL};
const Segment * const SG80498[] = {SGI341336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341337, SGI0, NULL};
const Segment * const SG80499[] = {SGI341338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341339, SGI0, NULL};
const Segment * const SG80500[] = {SGI341340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341341, SGI0, NULL};
const Segment * const SG80501[] = {SGI341343, SGI341344, SGI341345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341346, SGI0, NULL};
const Segment * const SG80502[] = {SGI341347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80503[] = {SGI341348, SGI341349, SGI341350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341350, SGI0, NULL};
const Segment * const SG80504[] = {SGI341351, SGI0, SGI0, SGI341352, SGI0, SGI0, SGI0, SGI0, SGI341356, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80505[] = {SGI341353, SGI341354, SGI341355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341357, SGI0, NULL};
const Segment * const SG80506[] = {SGI341358, SGI341359, SGI341360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341361, SGI0, NULL};
const Segment * const SG80507[] = {SGI341362, SGI341363, SGI341364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341365, SGI0, NULL};
const Segment * const SG80508[] = {SGI341366, SGI341367, SGI341368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341369, SGI0, NULL};
const Segment * const SG80509[] = {SGI341370, SGI341371, SGI341372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341373, SGI0, NULL};
const Segment * const SG80510[] = {SGI341374, SGI341375, SGI341376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341377, SGI0, NULL};
const Segment * const SG80511[] = {SGI341378, SGI341379, SGI341380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341381, SGI0, NULL};
const Segment * const SG80512[] = {SGI341382, SGI341383, SGI341384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341385, SGI0, NULL};
const Segment * const SG80513[] = {SGI341386, SGI341387, SGI341388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341389, SGI0, NULL};
const Segment * const SG80514[] = {SGI341390, SGI341391, SGI341392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341393, SGI0, NULL};
const Segment * const SG80515[] = {SGI341394, SGI341395, SGI341396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341397, SGI0, NULL};
const Segment * const SG80516[] = {SGI341398, SGI341399, SGI341400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341401, SGI0, NULL};
const Segment * const SG80517[] = {SGI341402, SGI341403, SGI341404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341405, SGI0, NULL};
const Segment * const SG80518[] = {SGI341406, SGI341407, SGI341408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341409, SGI0, NULL};
const Segment * const SG80519[] = {SGI341410, SGI341411, SGI341412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341413, SGI0, NULL};
const Segment * const SG80520[] = {SGI341414, SGI341415, SGI341416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341417, SGI0, NULL};
const Segment * const SG80521[] = {SGI341418, SGI341419, SGI341420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341421, SGI0, NULL};
const Segment * const SG80522[] = {SGI341422, SGI341423, SGI341424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341425, SGI0, NULL};
const Segment * const SG80523[] = {SGI341426, SGI341427, SGI341428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341429, SGI0, NULL};
const Segment * const SG80524[] = {SGI341430, SGI341431, SGI341432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341433, SGI0, NULL};
const Segment * const SG80525[] = {SGI341434, SGI341435, SGI341436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341437, SGI0, NULL};
const Segment * const SG80526[] = {SGI341438, SGI341439, SGI341440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341441, SGI0, NULL};
const Segment * const SG80527[] = {SGI341442, SGI341443, SGI341444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341445, SGI0, NULL};
const Segment * const SG80528[] = {SGI341446, SGI341447, SGI341448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341449, SGI0, NULL};
const Segment * const SG80529[] = {SGI341450, SGI341451, SGI341452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341453, SGI0, NULL};
const Segment * const SG80530[] = {SGI341454, SGI341455, SGI341456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341457, SGI0, NULL};
const Segment * const SG80531[] = {SGI341458, SGI341459, SGI341460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341461, SGI0, NULL};
const Segment * const SG80532[] = {SGI341462, SGI341463, SGI341464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341465, SGI0, NULL};
const Segment * const SG80533[] = {SGI341466, SGI341467, SGI341468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341469, SGI0, NULL};
const Segment * const SG80534[] = {SGI341470, SGI341471, SGI341472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341473, SGI0, NULL};
const Segment * const SG80535[] = {SGI341474, SGI341475, SGI341476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341477, SGI0, NULL};
const Segment * const SG80536[] = {SGI341478, SGI341479, SGI341480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341481, SGI0, NULL};
const Segment * const SG80537[] = {SGI341482, SGI341483, SGI341484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341485, SGI0, NULL};
const Segment * const SG80538[] = {SGI341486, SGI341487, SGI341488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341489, SGI0, NULL};
const Segment * const SG80539[] = {SGI341490, SGI341491, SGI341492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341493, SGI0, NULL};
const Segment * const SG80540[] = {SGI341494, SGI341495, SGI341496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341497, SGI0, NULL};
const Segment * const SG80541[] = {SGI341498, SGI341499, SGI341500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341501, SGI0, NULL};
const Segment * const SG80542[] = {SGI341502, SGI341503, SGI341504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341505, SGI0, NULL};
const Segment * const SG80543[] = {SGI341506, SGI341507, SGI341508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341509, SGI0, NULL};
const Segment * const SG80544[] = {SGI341510, SGI341511, SGI341512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341513, SGI0, NULL};
const Segment * const SG80545[] = {SGI341514, SGI341515, SGI341516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341517, SGI0, NULL};
const Segment * const SG80546[] = {SGI341518, SGI341519, SGI341520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341521, SGI0, NULL};
const Segment * const SG80547[] = {SGI341522, SGI341523, SGI341524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341525, SGI0, NULL};
const Segment * const SG80548[] = {SGI341526, SGI341527, SGI341528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341529, SGI0, NULL};
const Segment * const SG80549[] = {SGI341530, SGI341531, SGI341532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341533, SGI0, NULL};
const Segment * const SG80550[] = {SGI341534, SGI341535, SGI341536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341537, SGI0, NULL};
const Segment * const SG80551[] = {SGI341538, SGI341539, SGI341540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341541, SGI0, NULL};
const Segment * const SG80552[] = {SGI341542, SGI341543, SGI341544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341545, SGI0, NULL};
const Segment * const SG80553[] = {SGI341546, SGI341547, SGI341548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341549, SGI0, NULL};
const Segment * const SG80554[] = {SGI341550, SGI341551, SGI341552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341553, SGI0, NULL};
const Segment * const SG80555[] = {SGI341554, SGI341555, SGI341556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341557, SGI0, NULL};
const Segment * const SG80556[] = {SGI341558, SGI341559, SGI341560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341561, SGI0, NULL};
const Segment * const SG80557[] = {SGI341562, SGI341563, SGI341564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341565, SGI0, NULL};
const Segment * const SG80558[] = {SGI341566, SGI341567, SGI341568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341569, SGI0, NULL};
const Segment * const SG80559[] = {SGI341570, SGI341571, SGI341572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341573, SGI0, NULL};
const Segment * const SG80560[] = {SGI341574, SGI341575, SGI341576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341577, SGI0, NULL};
const Segment * const SG80561[] = {SGI341578, SGI341579, SGI341580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341581, SGI0, NULL};
const Segment * const SG80562[] = {SGI341582, SGI341583, SGI341584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341585, SGI0, NULL};
const Segment * const SG80563[] = {SGI341586, SGI341587, SGI341588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341589, SGI0, NULL};
const Segment * const SG80564[] = {SGI341590, SGI341591, SGI341592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341593, SGI0, NULL};
const Segment * const SG80565[] = {SGI341594, SGI341595, SGI341596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341597, SGI0, NULL};
const Segment * const SG80566[] = {SGI341598, SGI341599, SGI341600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341601, SGI0, NULL};
const Segment * const SG80567[] = {SGI341602, SGI341603, SGI341604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341605, SGI0, NULL};
const Segment * const SG80568[] = {SGI341607, SGI341609, SGI341610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341611, SGI0, NULL};
const Segment * const SG80569[] = {SGI341606, SGI0, SGI0, SGI341608, SGI0, SGI0, SGI0, SGI0, SGI341612, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80570[] = {SGI341613, SGI341614, SGI341615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341616, SGI0, NULL};
const Segment * const SG80571[] = {SGI341617, SGI0, SGI0, SGI341618, SGI0, SGI0, SGI0, SGI0, SGI341619, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80572[] = {SGI341620, SGI341621, SGI341622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341623, SGI0, NULL};
const Segment * const SG80573[] = {SGI341624, SGI341625, SGI341626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341627, SGI0, NULL};
const Segment * const SG80574[] = {SGI341628, SGI341629, SGI341630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341631, SGI0, NULL};
const Segment * const SG80575[] = {SGI341632, SGI341633, SGI341634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341635, SGI0, NULL};
const Segment * const SG80576[] = {SGI341636, SGI341637, SGI341638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341639, SGI0, NULL};
const Segment * const SG80577[] = {SGI341640, SGI341641, SGI341642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341643, SGI0, NULL};
const Segment * const SG80578[] = {SGI341644, SGI0, SGI0, SGI341645, SGI0, SGI0, SGI0, SGI0, SGI341646, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80579[] = {SGI341647, SGI0, SGI0, SGI341648, SGI0, SGI0, SGI0, SGI0, SGI341649, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80580[] = {SGI341650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341651, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80581[] = {SGI341342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341652, SGI0, NULL};
const Segment * const SG80582[] = {SGI341653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341654, SGI0, NULL};
const Segment * const SG80583[] = {SGI341655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341656, SGI0, NULL};
const Segment * const SG80584[] = {SGI341657, SGI341658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80585[] = {SGI341659, SGI341660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80586[] = {SGI341661, SGI341662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80587[] = {SGI341663, SGI341664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80588[] = {SGI341665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341666, SGI0, NULL};
const Segment * const SG80589[] = {SGI341667, SGI341668, SGI341669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341670, SGI0, NULL};
const Segment * const SG80590[] = {SGI341671, SGI341672, SGI341673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341673, SGI0, NULL};
const Segment * const SG80591[] = {SGI341674, SGI341675, SGI341676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341677, SGI0, NULL};
const Segment * const SG80592[] = {SGI341678, SGI341679, SGI341680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341681, SGI0, NULL};
const Segment * const SG80593[] = {SGI341682, SGI341683, SGI341684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341684, SGI0, NULL};
const Segment * const SG80594[] = {SGI341685, SGI341686, SGI341687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341688, SGI0, NULL};
const Segment * const SG80595[] = {SGI341689, SGI341690, SGI341691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341692, SGI0, NULL};
const Segment * const SG80596[] = {SGI341693, SGI341694, SGI341695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341696, SGI0, NULL};
const Segment * const SG80597[] = {SGI341697, SGI341698, SGI341699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341700, SGI0, NULL};
const Segment * const SG80598[] = {SGI341701, SGI341702, SGI341703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341704, SGI0, NULL};
const Segment * const SG80599[] = {SGI341705, SGI341706, SGI341707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341708, SGI0, NULL};
const Segment * const SG80600[] = {SGI341709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341710, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80601[] = {SGI341711, SGI341712, SGI341713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341714, SGI0, NULL};
const Segment * const SG80602[] = {SGI341715, SGI0, SGI0, SGI341716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80603[] = {SGI341717, SGI341718, SGI341719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341720, SGI0, NULL};
const Segment * const SG80604[] = {SGI341721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341723, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80605[] = {SGI341722, SGI341724, SGI341725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341726, SGI0, NULL};
const Segment * const SG80606[] = {SGI341727, SGI0, SGI0, SGI341728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80607[] = {SGI341729, SGI341730, SGI341731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341732, SGI0, NULL};
const Segment * const SG80608[] = {SGI341733, SGI341735, SGI341736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341737, SGI0, NULL};
const Segment * const SG80609[] = {SGI341734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341738, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80610[] = {SGI341739, SGI341740, SGI341741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341742, SGI0, NULL};
const Segment * const SG80611[] = {SGI341743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341744, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80612[] = {SGI341745, SGI341746, SGI341747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341748, SGI0, NULL};
const Segment * const SG80613[] = {SGI341749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341750, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80614[] = {SGI341751, SGI341752, SGI341753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341754, SGI0, NULL};
const Segment * const SG80615[] = {SGI341755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341756, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80616[] = {SGI341757, SGI341758, SGI341759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341760, SGI0, NULL};
const Segment * const SG80617[] = {SGI341761, SGI341762, SGI341763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341764, SGI0, NULL};
const Segment * const SG80618[] = {SGI341765, SGI341766, SGI341767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341768, SGI0, NULL};
const Segment * const SG80619[] = {SGI341769, SGI341770, SGI341771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341772, SGI0, NULL};
const Segment * const SG80620[] = {SGI341773, SGI0, SGI341774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341775, SGI0, NULL};
const Segment * const SG80621[] = {SGI341776, SGI341777, SGI341778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341779, SGI0, NULL};
const Segment * const SG80622[] = {SGI341780, SGI341781, SGI341782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341783, SGI0, NULL};
const Segment * const SG80623[] = {SGI341784, SGI341785, SGI341786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341787, SGI0, NULL};
const Segment * const SG80624[] = {SGI341788, SGI341789, SGI341790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341791, SGI0, NULL};
const Segment * const SG80625[] = {SGI341792, SGI341793, SGI341794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341795, SGI0, NULL};
const Segment * const SG80626[] = {SGI341796, SGI341797, SGI341798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341799, SGI0, NULL};
const Segment * const SG80627[] = {SGI341800, SGI341801, SGI341802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341803, SGI0, NULL};
const Segment * const SG80628[] = {SGI341804, SGI341805, SGI341806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341807, SGI0, NULL};
const Segment * const SG80629[] = {SGI341808, SGI341809, SGI341810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341811, SGI0, NULL};
const Segment * const SG80630[] = {SGI341812, SGI341813, SGI341814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341815, SGI0, NULL};
const Segment * const SG80631[] = {SGI341816, SGI341817, SGI341818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341819, SGI0, NULL};
const Segment * const SG80632[] = {SGI341820, SGI341821, SGI341822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341823, SGI0, NULL};
const Segment * const SG80633[] = {SGI341824, SGI341825, SGI341826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341827, SGI0, NULL};
const Segment * const SG80634[] = {SGI341828, SGI341829, SGI341830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341831, SGI0, NULL};
const Segment * const SG80635[] = {SGI341832, SGI341833, SGI341834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341835, SGI0, NULL};
const Segment * const SG80636[] = {SGI341836, SGI341837, SGI341838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341839, SGI0, NULL};
const Segment * const SG80637[] = {SGI341840, SGI341841, SGI341842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341843, SGI0, NULL};
const Segment * const SG80638[] = {SGI341844, SGI341845, SGI341846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341847, SGI0, NULL};
const Segment * const SG80639[] = {SGI341848, SGI341849, SGI341850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341851, SGI0, NULL};
const Segment * const SG80640[] = {SGI341852, SGI341853, SGI341854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341855, SGI0, NULL};
const Segment * const SG80641[] = {SGI341856, SGI341857, SGI341858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341859, SGI0, NULL};
const Segment * const SG80642[] = {SGI341860, SGI341861, SGI341862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341863, SGI0, NULL};
const Segment * const SG80643[] = {SGI341864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80644[] = {SGI341865, SGI341866, SGI341867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341868, SGI0, NULL};
const Segment * const SG80645[] = {SGI341869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80646[] = {SGI341870, SGI341871, SGI341872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341873, SGI0, NULL};
const Segment * const SG80647[] = {SGI341875, SGI341876, SGI341877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341878, SGI0, NULL};
const Segment * const SG80648[] = {SGI341874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80649[] = {SGI341879, SGI341880, SGI341881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341882, SGI0, NULL};
const Segment * const SG80650[] = {SGI341883, SGI341884, SGI341885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341886, SGI0, NULL};
const Segment * const SG80651[] = {SGI341887, SGI341888, SGI341889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341890, SGI0, NULL};
const Segment * const SG80652[] = {SGI341891, SGI341892, SGI341893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341894, SGI0, NULL};
const Segment * const SG80653[] = {SGI341895, SGI341896, SGI341897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341898, SGI0, NULL};
const Segment * const SG80654[] = {SGI341899, SGI0, SGI341900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341900, SGI0, NULL};
const Segment * const SG80655[] = {SGI341901, SGI341902, SGI341903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341904, SGI0, NULL};
const Segment * const SG80656[] = {SGI341905, SGI341906, SGI341907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341908, SGI0, NULL};
const Segment * const SG80657[] = {SGI341909, SGI341910, SGI341911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341912, SGI0, NULL};
const Segment * const SG80658[] = {SGI341913, SGI341914, SGI341915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341916, SGI0, NULL};
const Segment * const SG80659[] = {SGI341917, SGI341918, SGI341919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341920, SGI0, NULL};
const Segment * const SG80660[] = {SGI341921, SGI341922, SGI341923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341924, SGI0, NULL};
const Segment * const SG80661[] = {SGI341925, SGI341926, SGI341927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341928, SGI0, NULL};
const Segment * const SG80662[] = {SGI341929, SGI341930, SGI341931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341932, SGI0, NULL};
const Segment * const SG80663[] = {SGI341933, SGI341934, SGI341935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341936, SGI0, NULL};
const Segment * const SG80664[] = {SGI341937, SGI341938, SGI341939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341940, SGI0, NULL};
const Segment * const SG80665[] = {SGI341941, SGI341942, SGI341943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341944, SGI0, NULL};
const Segment * const SG80666[] = {SGI341945, SGI341946, SGI341947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341948, SGI0, NULL};
const Segment * const SG80667[] = {SGI341949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341951, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80668[] = {SGI341950, SGI341952, SGI341953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341954, SGI0, NULL};
const Segment * const SG80669[] = {SGI341955, SGI341956, SGI341957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341958, SGI0, NULL};
const Segment * const SG80670[] = {SGI341959, SGI341960, SGI341961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341962, SGI0, NULL};
const Segment * const SG80671[] = {SGI341963, SGI341964, SGI341965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341966, SGI0, NULL};
const Segment * const SG80672[] = {SGI341967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341970, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80673[] = {SGI341968, SGI341969, SGI341971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341972, SGI0, NULL};
const Segment * const SG80674[] = {SGI341973, SGI341974, SGI341975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341976, SGI0, NULL};
const Segment * const SG80675[] = {SGI341977, SGI341978, SGI341979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341980, SGI0, NULL};
const Segment * const SG80676[] = {SGI341981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341986, SGI0, NULL};
const Segment * const SG80677[] = {SGI341982, SGI341983, SGI341984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341985, SGI0, NULL};
const Segment * const SG80678[] = {SGI341987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341991, SGI0, NULL};
const Segment * const SG80679[] = {SGI341988, SGI341989, SGI341990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI341992, SGI0, NULL};
const Segment * const SG80680[] = {SGI0, SGI341993, SGI341994, SGI0, SGI0, SGI0, SGI341995, NULL};
const Segment * const SG80681[] = {SGI341996, SGI341997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80682[] = {SGI0, SGI341998, SGI341999, SGI0, SGI0, SGI0, SGI342000, NULL};
const Segment * const SG80683[] = {SGI342001, SGI342002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80684[] = {SGI342003, SGI342004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80685[] = {SGI342005, SGI342006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80686[] = {SGI342007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80687[] = {SGI342008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80688[] = {SGI342009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342010, SGI0, NULL};
const Segment * const SG80689[] = {SGI342011, SGI342012, SGI0, SGI0, NULL};
const Segment * const SG80690[] = {SGI342013, SGI0, SGI0, SGI342014, SGI0, SGI0, SGI0, SGI0, SGI342015, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80691[] = {SGI342016, SGI0, SGI342017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342017, SGI0, NULL};
const Segment * const SG80692[] = {SGI342018, SGI0, SGI342019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342019, SGI0, NULL};
const Segment * const SG80693[] = {SGI342020, SGI0, SGI342021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342021, SGI0, NULL};
const Segment * const SG80694[] = {SGI342022, SGI0, SGI342023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342023, SGI0, NULL};
const Segment * const SG80695[] = {SGI342024, SGI0, SGI342025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342025, SGI0, NULL};
const Segment * const SG80696[] = {SGI342026, SGI0, SGI342027, SGI342027, SGI0, SGI0, SGI0, SGI0, SGI342027, SGI0, NULL};
const Segment * const SG80697[] = {SGI342028, SGI0, SGI342029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342029, SGI0, NULL};
const Segment * const SG80698[] = {SGI342030, SGI0, SGI342031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342032, SGI0, NULL};
const Segment * const SG80699[] = {SGI342033, SGI0, SGI342034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342035, SGI0, NULL};
const Segment * const SG80700[] = {SGI342036, SGI0, SGI342037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342038, SGI0, NULL};
const Segment * const SG80701[] = {SGI342039, SGI0, SGI342040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342040, SGI0, NULL};
const Segment * const SG80702[] = {SGI342041, SGI0, SGI342042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342042, SGI0, NULL};
const Segment * const SG80703[] = {SGI342043, SGI0, SGI342044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342045, SGI0, NULL};
const Segment * const SG80704[] = {SGI342046, SGI342047, SGI342048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80705[] = {SGI342049, SGI342050, SGI342051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80706[] = {SGI342052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80707[] = {SGI342053, SGI342054, SGI342055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80708[] = {SGI342056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80709[] = {SGI342057, SGI342058, SGI342059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80710[] = {SGI342060, SGI342061, SGI342062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80711[] = {SGI342063, SGI0, SGI0, SGI342064, SGI0, SGI0, SGI0, SGI0, SGI342065, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80712[] = {SGI342066, SGI0, SGI0, SGI342067, SGI0, SGI0, SGI0, SGI0, SGI342068, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80713[] = {SGI342069, SGI0, SGI0, SGI342070, SGI0, SGI0, SGI0, SGI0, SGI342071, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80714[] = {SGI342072, SGI0, SGI0, SGI342073, SGI0, SGI0, SGI0, SGI0, SGI342074, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80715[] = {SGI342075, SGI0, SGI0, SGI342076, SGI0, SGI0, SGI0, SGI0, SGI342077, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80716[] = {SGI342078, SGI0, SGI0, SGI342079, SGI0, SGI0, SGI0, SGI0, SGI342080, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80717[] = {SGI342081, SGI0, SGI0, SGI342082, SGI0, SGI0, SGI0, SGI0, SGI342083, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80718[] = {SGI342084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80719[] = {SGI342085, SGI342086, SGI0, SGI0, NULL};
const Segment * const SG80720[] = {SGI342087, SGI0, SGI0, SGI342088, SGI0, SGI0, SGI0, SGI0, SGI342089, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80721[] = {SGI342090, SGI0, SGI0, SGI342091, SGI0, SGI0, SGI0, SGI0, SGI342092, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80722[] = {SGI342093, SGI0, SGI0, SGI342094, SGI0, SGI0, SGI0, SGI0, SGI342095, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80723[] = {SGI342096, SGI0, SGI0, SGI342097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80724[] = {SGI342098, SGI342099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80725[] = {SGI342100, SGI342101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80726[] = {SGI342102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80727[] = {SGI342103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80728[] = {SGI342104, SGI342105, SGI342106, SGI342107, NULL};
const Segment * const SG80729[] = {SGI0, SGI342108, SGI342109, SGI0, SGI0, SGI0, SGI342110, NULL};
const Segment * const SG80730[] = {SGI342111, SGI342112, SGI342113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342114, SGI0, NULL};
const Segment * const SG80731[] = {SGI342115, SGI342116, SGI342117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342118, SGI0, NULL};
const Segment * const SG80732[] = {SGI342119, SGI342120, SGI342121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342122, SGI0, NULL};
const Segment * const SG80733[] = {SGI342123, SGI342124, SGI342125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342126, SGI0, NULL};
const Segment * const SG80734[] = {SGI342127, SGI342128, SGI342129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342130, SGI0, NULL};
const Segment * const SG80735[] = {SGI342131, SGI342132, SGI342133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342134, SGI0, NULL};
const Segment * const SG80736[] = {SGI342135, SGI342136, SGI342137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342138, SGI0, NULL};
const Segment * const SG80737[] = {SGI342139, SGI342140, SGI342141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342142, SGI0, NULL};
const Segment * const SG80738[] = {SGI342143, SGI342144, SGI342145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342146, SGI0, NULL};
const Segment * const SG80739[] = {SGI342147, SGI342148, SGI342149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342150, SGI0, NULL};
const Segment * const SG80740[] = {SGI342151, SGI342152, SGI342153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342154, SGI0, NULL};
const Segment * const SG80741[] = {SGI342155, SGI342156, SGI342157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342158, SGI0, NULL};
const Segment * const SG80742[] = {SGI342159, SGI342160, SGI342161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342162, SGI0, NULL};
const Segment * const SG80743[] = {SGI342163, SGI342164, SGI342165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342166, SGI0, NULL};
const Segment * const SG80744[] = {SGI342167, SGI342168, SGI342169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342170, SGI0, NULL};
const Segment * const SG80745[] = {SGI342171, SGI342172, SGI342173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342174, SGI0, NULL};
const Segment * const SG80746[] = {SGI342175, SGI342176, SGI342177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342178, SGI0, NULL};
const Segment * const SG80747[] = {SGI342179, SGI342180, SGI342181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342182, SGI0, NULL};
const Segment * const SG80748[] = {SGI342183, SGI342184, SGI342185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342186, SGI0, NULL};
const Segment * const SG80749[] = {SGI342187, SGI342188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80750[] = {SGI342189, SGI342190, SGI342191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342192, SGI0, NULL};
const Segment * const SG80751[] = {SGI342193, SGI342194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80752[] = {SGI342195, SGI342196, SGI342197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342198, SGI0, NULL};
const Segment * const SG80753[] = {SGI342199, SGI342200, SGI342201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342202, SGI0, NULL};
const Segment * const SG80754[] = {SGI342203, SGI342204, SGI342205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342206, SGI0, NULL};
const Segment * const SG80755[] = {SGI342207, SGI342208, SGI342209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342210, SGI0, NULL};
const Segment * const SG80756[] = {SGI342211, SGI342212, SGI342213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342213, SGI0, NULL};
const Segment * const SG80757[] = {SGI342214, SGI342215, SGI342216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342217, SGI0, NULL};
const Segment * const SG80758[] = {SGI342218, SGI342219, SGI342220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342221, SGI0, NULL};
const Segment * const SG80759[] = {SGI342222, SGI342223, SGI342224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342225, SGI0, NULL};
const Segment * const SG80760[] = {SGI342226, SGI342227, SGI342228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342229, SGI0, NULL};
const Segment * const SG80761[] = {SGI342230, SGI342231, SGI342232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342233, SGI0, NULL};
const Segment * const SG80762[] = {SGI342234, SGI342235, SGI342236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342237, SGI0, NULL};
const Segment * const SG80763[] = {SGI342238, SGI342239, SGI342240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342241, SGI0, NULL};
const Segment * const SG80764[] = {SGI342242, SGI342243, SGI342244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342245, SGI0, NULL};
const Segment * const SG80765[] = {SGI342246, SGI342247, SGI342248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342249, SGI0, NULL};
const Segment * const SG80766[] = {SGI342250, SGI342251, SGI342252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342253, SGI0, NULL};
const Segment * const SG80767[] = {SGI342254, SGI342255, SGI342256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342257, SGI0, NULL};
const Segment * const SG80768[] = {SGI342258, SGI342259, SGI342260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342261, SGI0, NULL};
const Segment * const SG80769[] = {SGI342262, SGI342263, SGI342264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342265, SGI0, NULL};
const Segment * const SG80770[] = {SGI342266, SGI342267, SGI342268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342269, SGI0, NULL};
const Segment * const SG80771[] = {SGI342270, SGI342271, SGI342272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342273, SGI0, NULL};
const Segment * const SG80772[] = {SGI342274, SGI342275, SGI342276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342277, SGI0, NULL};
const Segment * const SG80773[] = {SGI342278, SGI342279, SGI342280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342281, SGI0, NULL};
const Segment * const SG80774[] = {SGI342282, SGI342283, SGI342284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342285, SGI0, NULL};
const Segment * const SG80775[] = {SGI342286, SGI342287, SGI342288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342289, SGI0, NULL};
const Segment * const SG80776[] = {SGI342290, SGI342291, SGI342292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342293, SGI0, NULL};
const Segment * const SG80777[] = {SGI342294, SGI342295, SGI342296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342297, SGI0, NULL};
const Segment * const SG80778[] = {SGI342298, SGI342299, SGI342300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342301, SGI0, NULL};
const Segment * const SG80779[] = {SGI342302, SGI342303, SGI342304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342305, SGI0, NULL};
const Segment * const SG80780[] = {SGI342306, SGI342307, SGI342308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342309, SGI0, NULL};
const Segment * const SG80781[] = {SGI342310, SGI342311, SGI342312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342313, SGI0, NULL};
const Segment * const SG80782[] = {SGI342314, SGI342315, SGI342316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342317, SGI0, NULL};
const Segment * const SG80783[] = {SGI342318, SGI342319, SGI342320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342321, SGI0, NULL};
const Segment * const SG80784[] = {SGI342322, SGI342323, SGI342324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342325, SGI0, NULL};
const Segment * const SG80785[] = {SGI342326, SGI342327, SGI342328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342329, SGI0, NULL};
const Segment * const SG80786[] = {SGI342330, SGI342331, SGI342332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342333, SGI0, NULL};
const Segment * const SG80787[] = {SGI342334, SGI342335, SGI342336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342337, SGI0, NULL};
const Segment * const SG80788[] = {SGI342338, SGI342339, SGI342340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342341, SGI0, NULL};
const Segment * const SG80789[] = {SGI342342, SGI342343, SGI342344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342345, SGI0, NULL};
const Segment * const SG80790[] = {SGI342346, SGI342347, SGI342348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342349, SGI0, NULL};
const Segment * const SG80791[] = {SGI342350, SGI342351, SGI342352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342353, SGI0, NULL};
const Segment * const SG80792[] = {SGI342354, SGI342355, SGI342356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342357, SGI0, NULL};
const Segment * const SG80793[] = {SGI342358, SGI342359, SGI342360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342360, SGI0, NULL};
const Segment * const SG80794[] = {SGI342361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342362, SGI0, NULL};
const Segment * const SG80795[] = {SGI342363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342364, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80796[] = {SGI342365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342366, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80797[] = {SGI342367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342368, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80798[] = {SGI342369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342370, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80799[] = {SGI342371, SGI0, SGI0, SGI342372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80800[] = {SGI342373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342374, SGI0, NULL};
const Segment * const SG80801[] = {SGI342375, SGI0, SGI0, SGI342376, SGI0, SGI0, SGI0, SGI0, SGI342377, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80802[] = {SGI342378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342379, SGI0, NULL};
const Segment * const SG80803[] = {SGI342380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342381, SGI0, NULL};
const Segment * const SG80804[] = {SGI342382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342383, SGI0, NULL};
const Segment * const SG80805[] = {SGI342384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342385, SGI0, NULL};
const Segment * const SG80806[] = {SGI342386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342387, SGI0, NULL};
const Segment * const SG80807[] = {SGI342388, SGI0, SGI0, SGI342389, SGI0, SGI0, SGI0, SGI0, SGI342390, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80808[] = {SGI342391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342392, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80809[] = {SGI342393, SGI0, SGI0, SGI342394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80810[] = {SGI342395, SGI342396, SGI0, SGI0, NULL};
const Segment * const SG80811[] = {SGI342397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342398, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80812[] = {SGI342399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342400, SGI0, NULL};
const Segment * const SG80813[] = {SGI342401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342402, SGI0, NULL};
const Segment * const SG80814[] = {SGI342403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342404, SGI0, NULL};
const Segment * const SG80815[] = {SGI342405, SGI342406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80816[] = {SGI342407, SGI342408, SGI0, SGI0, NULL};
const Segment * const SG80817[] = {SGI342409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342410, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80818[] = {SGI342411, SGI0, SGI0, SGI342412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80819[] = {SGI342413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342416, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80820[] = {SGI342414, SGI342415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80821[] = {SGI342417, SGI342418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80822[] = {SGI342419, SGI0, SGI0, SGI342420, SGI0, SGI0, SGI0, SGI0, SGI342421, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80823[] = {SGI342422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342423, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80824[] = {SGI342424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342425, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80825[] = {SGI342426, SGI0, SGI0, SGI342427, SGI0, SGI0, SGI0, SGI0, SGI342428, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80826[] = {SGI342429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342430, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80827[] = {SGI342431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342432, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80828[] = {SGI342433, SGI0, SGI0, SGI342434, SGI0, SGI0, SGI0, SGI0, SGI342435, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80829[] = {SGI342436, SGI0, SGI0, SGI342437, SGI0, SGI0, SGI0, SGI0, SGI342438, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80830[] = {SGI342439, SGI342440, SGI342441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342441, SGI0, NULL};
const Segment * const SG80831[] = {SGI342442, SGI342443, SGI342444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342445, SGI0, NULL};
const Segment * const SG80832[] = {SGI342446, SGI342447, SGI342448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342448, SGI0, NULL};
const Segment * const SG80833[] = {SGI342449, SGI342450, SGI342451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342452, SGI0, NULL};
const Segment * const SG80834[] = {SGI342453, SGI342454, SGI342455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342456, SGI0, NULL};
const Segment * const SG80835[] = {SGI342457, SGI342458, SGI342459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342460, SGI0, NULL};
const Segment * const SG80836[] = {SGI342461, SGI342462, SGI342463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342464, SGI0, NULL};
const Segment * const SG80837[] = {SGI342465, SGI342466, SGI342467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342468, SGI0, NULL};
const Segment * const SG80838[] = {SGI342469, SGI342470, SGI342471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342472, SGI0, NULL};
const Segment * const SG80839[] = {SGI342473, SGI342474, SGI342475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342476, SGI0, NULL};
const Segment * const SG80840[] = {SGI342477, SGI342478, SGI342479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342480, SGI0, NULL};
const Segment * const SG80841[] = {SGI342481, SGI342482, SGI342483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342484, SGI0, NULL};
const Segment * const SG80842[] = {SGI342485, SGI342486, SGI342487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342488, SGI0, NULL};
const Segment * const SG80843[] = {SGI342489, SGI342490, SGI342491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342492, SGI0, NULL};
const Segment * const SG80844[] = {SGI342493, SGI342494, SGI342495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342496, SGI0, NULL};
const Segment * const SG80845[] = {SGI342497, SGI342498, SGI342499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342500, SGI0, NULL};
const Segment * const SG80846[] = {SGI342501, SGI342502, SGI342503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342504, SGI0, NULL};
const Segment * const SG80847[] = {SGI342505, SGI342506, SGI342507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342508, SGI0, NULL};
const Segment * const SG80848[] = {SGI342509, SGI342510, SGI342511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342513, SGI0, NULL};
const Segment * const SG80849[] = {SGI342512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342514, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80850[] = {SGI342515, SGI342516, SGI342517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342518, SGI0, NULL};
const Segment * const SG80851[] = {SGI342519, SGI342520, SGI342521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342522, SGI0, NULL};
const Segment * const SG80852[] = {SGI342523, SGI342524, SGI342525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342526, SGI0, NULL};
const Segment * const SG80853[] = {SGI342527, SGI342528, SGI342529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342530, SGI0, NULL};
const Segment * const SG80854[] = {SGI342531, SGI342532, SGI342533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342534, SGI0, NULL};
const Segment * const SG80855[] = {SGI342535, SGI342536, SGI342537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342538, SGI0, NULL};
const Segment * const SG80856[] = {SGI342539, SGI342540, SGI342541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342542, SGI0, NULL};
const Segment * const SG80857[] = {SGI342543, SGI342544, SGI342545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342546, SGI0, NULL};
const Segment * const SG80858[] = {SGI342547, SGI342548, SGI342549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342550, SGI0, NULL};
const Segment * const SG80859[] = {SGI342551, SGI342552, SGI342553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342554, SGI0, NULL};
const Segment * const SG80860[] = {SGI342555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342556, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80861[] = {SGI342557, SGI342558, SGI342559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342560, SGI0, NULL};
const Segment * const SG80862[] = {SGI342561, SGI342562, SGI342563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342564, SGI0, NULL};
const Segment * const SG80863[] = {SGI342565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342568, SGI0, NULL};
const Segment * const SG80864[] = {SGI342566, SGI342567, SGI342569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342570, SGI0, NULL};
const Segment * const SG80865[] = {SGI342571, SGI342572, SGI342573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342574, SGI0, NULL};
const Segment * const SG80866[] = {SGI342575, SGI342576, SGI342577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342578, SGI0, NULL};
const Segment * const SG80867[] = {SGI342579, SGI342580, SGI342581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342582, SGI0, NULL};
const Segment * const SG80868[] = {SGI342583, SGI342584, SGI342585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342586, SGI0, NULL};
const Segment * const SG80869[] = {SGI342587, SGI342588, SGI342589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342590, SGI0, NULL};
const Segment * const SG80870[] = {SGI342591, SGI342592, SGI342593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342594, SGI0, NULL};
const Segment * const SG80871[] = {SGI342595, SGI342596, SGI342597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342598, SGI0, NULL};
const Segment * const SG80872[] = {SGI342599, SGI342600, SGI342601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342602, SGI0, NULL};
const Segment * const SG80873[] = {SGI342603, SGI342604, SGI342605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342606, SGI0, NULL};
const Segment * const SG80874[] = {SGI342607, SGI342608, SGI342609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342610, SGI0, NULL};
const Segment * const SG80875[] = {SGI342611, SGI342612, SGI342613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342614, SGI0, NULL};
const Segment * const SG80876[] = {SGI342615, SGI342616, SGI342617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342618, SGI0, NULL};
const Segment * const SG80877[] = {SGI342619, SGI342620, SGI342621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342622, SGI0, NULL};
const Segment * const SG80878[] = {SGI342623, SGI342624, SGI342625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342626, SGI0, NULL};
const Segment * const SG80879[] = {SGI342627, SGI342628, SGI342629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342630, SGI0, NULL};
const Segment * const SG80880[] = {SGI342631, SGI342632, SGI0, SGI0, NULL};
const Segment * const SG80881[] = {SGI342633, SGI342634, SGI342635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342636, SGI0, NULL};
const Segment * const SG80882[] = {SGI342637, SGI342638, SGI342639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342640, SGI0, NULL};
const Segment * const SG80883[] = {SGI342641, SGI342642, SGI342643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342644, SGI0, NULL};
const Segment * const SG80884[] = {SGI342645, SGI342646, SGI342647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342648, SGI0, NULL};
const Segment * const SG80885[] = {SGI342649, SGI342650, SGI342651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342652, SGI0, NULL};
const Segment * const SG80886[] = {SGI342653, SGI342654, SGI342655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342656, SGI0, NULL};
const Segment * const SG80887[] = {SGI342657, SGI342658, SGI342659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342660, SGI0, NULL};
const Segment * const SG80888[] = {SGI342661, SGI342662, SGI342663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342664, SGI0, NULL};
const Segment * const SG80889[] = {SGI342665, SGI342666, SGI342667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342668, SGI0, NULL};
const Segment * const SG80890[] = {SGI342669, SGI342670, SGI342671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342672, SGI0, NULL};
const Segment * const SG80891[] = {SGI342673, SGI342674, SGI342675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342676, SGI0, NULL};
const Segment * const SG80892[] = {SGI342677, SGI342678, SGI342679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342680, SGI0, NULL};
const Segment * const SG80893[] = {SGI342681, SGI342682, SGI342683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342684, SGI0, NULL};
const Segment * const SG80894[] = {SGI342685, SGI342686, SGI342687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342688, SGI0, NULL};
const Segment * const SG80895[] = {SGI342689, SGI342690, SGI342691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342692, SGI0, NULL};
const Segment * const SG80896[] = {SGI342693, SGI342694, SGI342695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342696, SGI0, NULL};
const Segment * const SG80897[] = {SGI342697, SGI342698, SGI342699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342700, SGI0, NULL};
const Segment * const SG80898[] = {SGI342701, SGI342702, SGI342703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342704, SGI0, NULL};
const Segment * const SG80899[] = {SGI342705, SGI342706, SGI342707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342708, SGI0, NULL};
const Segment * const SG80900[] = {SGI342709, SGI342710, SGI342711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342712, SGI0, NULL};
const Segment * const SG80901[] = {SGI342713, SGI342714, SGI342715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342716, SGI0, NULL};
const Segment * const SG80902[] = {SGI342717, SGI342718, SGI342719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342720, SGI0, NULL};
const Segment * const SG80903[] = {SGI342721, SGI342722, SGI342723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342724, SGI0, NULL};
const Segment * const SG80904[] = {SGI342725, SGI342726, SGI342727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342728, SGI0, NULL};
const Segment * const SG80905[] = {SGI342729, SGI342730, SGI342731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342732, SGI0, NULL};
const Segment * const SG80906[] = {SGI342733, SGI342734, SGI342735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342736, SGI0, NULL};
const Segment * const SG80907[] = {SGI342737, SGI342738, SGI342739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342740, SGI0, NULL};
const Segment * const SG80908[] = {SGI342741, SGI342742, SGI342743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342744, SGI0, NULL};
const Segment * const SG80909[] = {SGI342745, SGI342746, SGI342747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342748, SGI0, NULL};
const Segment * const SG80910[] = {SGI342749, SGI342750, SGI342751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342752, SGI0, NULL};
const Segment * const SG80911[] = {SGI342753, SGI342754, SGI342755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342756, SGI0, NULL};
const Segment * const SG80912[] = {SGI342757, SGI342758, SGI342759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342760, SGI0, NULL};
const Segment * const SG80913[] = {SGI342761, SGI342762, SGI342763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342764, SGI0, NULL};
const Segment * const SG80914[] = {SGI342765, SGI342766, SGI342767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342768, SGI0, NULL};
const Segment * const SG80915[] = {SGI342769, SGI342770, SGI342771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342772, SGI0, NULL};
const Segment * const SG80916[] = {SGI342773, SGI342774, SGI342775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342776, SGI0, NULL};
const Segment * const SG80917[] = {SGI342777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80918[] = {SGI342778, SGI342779, SGI342780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342781, SGI0, NULL};
const Segment * const SG80919[] = {SGI342782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80920[] = {SGI342783, SGI342784, SGI342785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342786, SGI0, NULL};
const Segment * const SG80921[] = {SGI342787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80922[] = {SGI342788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80923[] = {SGI342789, SGI342790, SGI0, SGI0, NULL};
const Segment * const SG80924[] = {SGI342791, SGI0, SGI0, SGI342792, SGI0, SGI0, SGI0, SGI0, SGI342793, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80925[] = {SGI342794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342795, SGI0, NULL};
const Segment * const SG80926[] = {SGI342796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342797, SGI0, NULL};
const Segment * const SG80927[] = {SGI342798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342799, SGI0, NULL};
const Segment * const SG80928[] = {SGI342800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342801, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80929[] = {SGI342802, SGI342803, SGI0, SGI0, NULL};
const Segment * const SG80930[] = {SGI342804, SGI342805, SGI0, SGI0, NULL};
const Segment * const SG80931[] = {SGI342806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342807, SGI0, NULL};
const Segment * const SG80932[] = {SGI342808, SGI342809, SGI0, SGI0, NULL};
const Segment * const SG80933[] = {SGI342810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342811, SGI0, NULL};
const Segment * const SG80934[] = {SGI342812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342813, SGI0, NULL};
const Segment * const SG80935[] = {SGI342814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342815, SGI0, NULL};
const Segment * const SG80936[] = {SGI342816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342817, SGI0, NULL};
const Segment * const SG80937[] = {SGI342818, SGI342819, SGI0, SGI0, NULL};
const Segment * const SG80938[] = {SGI342820, SGI342821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80939[] = {SGI342822, SGI342823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80940[] = {SGI342824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342825, SGI0, NULL};
const Segment * const SG80941[] = {SGI342826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342827, SGI0, NULL};
const Segment * const SG80942[] = {SGI342828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342829, SGI0, NULL};
const Segment * const SG80943[] = {SGI342830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342831, SGI0, NULL};
const Segment * const SG80944[] = {SGI342832, SGI342833, SGI342834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342834, SGI0, NULL};
const Segment * const SG80945[] = {SGI342835, SGI342836, SGI342837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342838, SGI0, NULL};
const Segment * const SG80946[] = {SGI342839, SGI342840, SGI342841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342842, SGI0, NULL};
const Segment * const SG80947[] = {SGI342843, SGI342844, SGI342845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342846, SGI0, NULL};
const Segment * const SG80948[] = {SGI342847, SGI342848, SGI342849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342850, SGI0, NULL};
const Segment * const SG80949[] = {SGI342851, SGI342852, SGI342853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342854, SGI0, NULL};
const Segment * const SG80950[] = {SGI342855, SGI342856, SGI342857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342858, SGI0, NULL};
const Segment * const SG80951[] = {SGI342859, SGI342860, SGI342861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342862, SGI0, NULL};
const Segment * const SG80952[] = {SGI342863, SGI342864, SGI342865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342866, SGI0, NULL};
const Segment * const SG80953[] = {SGI342867, SGI342868, SGI342869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342870, SGI0, NULL};
const Segment * const SG80954[] = {SGI342871, SGI342872, SGI342873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342874, SGI0, NULL};
const Segment * const SG80955[] = {SGI342875, SGI342876, SGI342877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342878, SGI0, NULL};
const Segment * const SG80956[] = {SGI342879, SGI342880, SGI342881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342882, SGI0, NULL};
const Segment * const SG80957[] = {SGI342883, SGI342884, SGI342885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342886, SGI0, NULL};
const Segment * const SG80958[] = {SGI342887, SGI342888, SGI342889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342890, SGI0, NULL};
const Segment * const SG80959[] = {SGI342891, SGI342892, SGI342893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342894, SGI0, NULL};
const Segment * const SG80960[] = {SGI342895, SGI342896, SGI342897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342898, SGI0, NULL};
const Segment * const SG80961[] = {SGI342899, SGI342900, SGI342901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342902, SGI0, NULL};
const Segment * const SG80962[] = {SGI342903, SGI342904, SGI342905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342906, SGI0, NULL};
const Segment * const SG80963[] = {SGI342907, SGI342908, SGI342909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342910, SGI0, NULL};
const Segment * const SG80964[] = {SGI342911, SGI342912, SGI342913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342914, SGI0, NULL};
const Segment * const SG80965[] = {SGI342915, SGI342916, SGI342917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342918, SGI0, NULL};
const Segment * const SG80966[] = {SGI342919, SGI342920, SGI342921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342922, SGI0, NULL};
const Segment * const SG80967[] = {SGI342923, SGI342924, SGI342925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342926, SGI0, NULL};
const Segment * const SG80968[] = {SGI342927, SGI342928, SGI342929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342930, SGI0, NULL};
const Segment * const SG80969[] = {SGI342931, SGI342932, SGI342933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342934, SGI0, NULL};
const Segment * const SG80970[] = {SGI342935, SGI342936, SGI342937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342938, SGI0, NULL};
const Segment * const SG80971[] = {SGI342939, SGI342940, SGI342941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342942, SGI0, NULL};
const Segment * const SG80972[] = {SGI342943, SGI342944, SGI342945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342946, SGI0, NULL};
const Segment * const SG80973[] = {SGI342947, SGI342948, SGI342949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342950, SGI0, NULL};
const Segment * const SG80974[] = {SGI342951, SGI342952, SGI342953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342954, SGI0, NULL};
const Segment * const SG80975[] = {SGI342955, SGI342956, SGI342957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342958, SGI0, NULL};
const Segment * const SG80976[] = {SGI342959, SGI342960, SGI342961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342962, SGI0, NULL};
const Segment * const SG80977[] = {SGI342963, SGI342964, SGI342965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342966, SGI0, NULL};
const Segment * const SG80978[] = {SGI342967, SGI342968, SGI342969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342970, SGI0, NULL};
const Segment * const SG80979[] = {SGI342971, SGI342972, SGI342973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342974, SGI0, NULL};
const Segment * const SG80980[] = {SGI342975, SGI342976, SGI342977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342978, SGI0, NULL};
const Segment * const SG80981[] = {SGI342979, SGI342980, SGI342981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342982, SGI0, NULL};
const Segment * const SG80982[] = {SGI342983, SGI342984, SGI342985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342986, SGI0, NULL};
const Segment * const SG80983[] = {SGI342987, SGI342988, SGI342989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342990, SGI0, NULL};
const Segment * const SG80984[] = {SGI342991, SGI342992, SGI342993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342994, SGI0, NULL};
const Segment * const SG80985[] = {SGI342995, SGI342996, SGI342997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI342998, SGI0, NULL};
const Segment * const SG80986[] = {SGI342999, SGI343000, SGI343001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343002, SGI0, NULL};
const Segment * const SG80987[] = {SGI343003, SGI343004, SGI343005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343006, SGI0, NULL};
const Segment * const SG80988[] = {SGI343007, SGI343008, SGI343009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343010, SGI0, NULL};
const Segment * const SG80989[] = {SGI343011, SGI343012, SGI343013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343014, SGI0, NULL};
const Segment * const SG80990[] = {SGI343015, SGI343016, SGI343017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343018, SGI0, NULL};
const Segment * const SG80991[] = {SGI343019, SGI343020, SGI343021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343022, SGI0, NULL};
const Segment * const SG80992[] = {SGI343023, SGI343024, SGI343025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343026, SGI0, NULL};
const Segment * const SG80993[] = {SGI343027, SGI343028, SGI343029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343030, SGI0, NULL};
const Segment * const SG80994[] = {SGI343031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343032, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80995[] = {SGI343033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343034, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80996[] = {SGI343035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343036, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80997[] = {SGI343037, SGI343038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80998[] = {SGI343039, SGI0, SGI0, SGI343040, SGI0, SGI0, SGI0, SGI0, SGI343041, SGI0, SGI0, SGI0, NULL};
const Segment * const SG80999[] = {SGI343042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343043, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81000[] = {SGI343044, SGI343045, SGI0, SGI0, NULL};
const Segment * const SG81001[] = {SGI343046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343048, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81002[] = {SGI343047, SGI343049, SGI0, SGI0, NULL};
const Segment * const SG81003[] = {SGI343050, SGI0, SGI0, SGI343051, SGI0, SGI0, SGI0, SGI0, SGI343052, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81004[] = {SGI343053, SGI0, SGI0, SGI343054, SGI0, SGI0, SGI0, SGI0, SGI343055, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81005[] = {SGI343056, SGI0, SGI0, SGI343057, SGI0, SGI0, SGI0, SGI0, SGI343058, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81006[] = {SGI343059, SGI343060, SGI343061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343061, SGI0, NULL};
const Segment * const SG81007[] = {SGI343062, SGI343063, SGI343064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343065, SGI0, NULL};
const Segment * const SG81008[] = {SGI343066, SGI343067, SGI343068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343069, SGI0, NULL};
const Segment * const SG81009[] = {SGI343070, SGI343071, SGI343072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343073, SGI0, NULL};
const Segment * const SG81010[] = {SGI343074, SGI343075, SGI343076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343077, SGI0, NULL};
const Segment * const SG81011[] = {SGI343078, SGI343079, SGI343080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343081, SGI0, NULL};
const Segment * const SG81012[] = {SGI343082, SGI343083, SGI343084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343085, SGI0, NULL};
const Segment * const SG81013[] = {SGI343086, SGI343087, SGI343088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343089, SGI0, NULL};
const Segment * const SG81014[] = {SGI343090, SGI343091, SGI343092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343093, SGI0, NULL};
const Segment * const SG81015[] = {SGI343094, SGI343095, SGI343096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343097, SGI0, NULL};
const Segment * const SG81016[] = {SGI343098, SGI343099, SGI343100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343101, SGI0, NULL};
const Segment * const SG81017[] = {SGI343102, SGI343103, SGI343104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343105, SGI0, NULL};
const Segment * const SG81018[] = {SGI343106, SGI343107, SGI343108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343109, SGI0, NULL};
const Segment * const SG81019[] = {SGI343110, SGI343111, SGI343112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343113, SGI0, NULL};
const Segment * const SG81020[] = {SGI343114, SGI343115, SGI343116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343117, SGI0, NULL};
const Segment * const SG81021[] = {SGI343118, SGI343119, SGI343120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343121, SGI0, NULL};
const Segment * const SG81022[] = {SGI343122, SGI343123, SGI343124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343125, SGI0, NULL};
const Segment * const SG81023[] = {SGI343126, SGI343127, SGI343128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343129, SGI0, NULL};
const Segment * const SG81024[] = {SGI343130, SGI343131, SGI343132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343133, SGI0, NULL};
const Segment * const SG81025[] = {SGI343134, SGI343135, SGI343136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343137, SGI0, NULL};
const Segment * const SG81026[] = {SGI343138, SGI343139, SGI343140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343141, SGI0, NULL};
const Segment * const SG81027[] = {SGI343142, SGI343143, SGI343144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343145, SGI0, NULL};
const Segment * const SG81028[] = {SGI343146, SGI343147, SGI343148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343149, SGI0, NULL};
const Segment * const SG81029[] = {SGI343150, SGI343151, SGI343152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343152, SGI0, NULL};
const Segment * const SG81030[] = {SGI343153, SGI343154, SGI343155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343156, SGI0, NULL};
const Segment * const SG81031[] = {SGI343157, SGI343158, SGI343159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343160, SGI0, NULL};
const Segment * const SG81032[] = {SGI343161, SGI343162, SGI343163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343164, SGI0, NULL};
const Segment * const SG81033[] = {SGI343165, SGI343166, SGI343167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343168, SGI0, NULL};
const Segment * const SG81034[] = {SGI343169, SGI343170, SGI343171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343172, SGI0, NULL};
const Segment * const SG81035[] = {SGI343173, SGI343174, SGI343175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343176, SGI0, NULL};
const Segment * const SG81036[] = {SGI343177, SGI343178, SGI343179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343180, SGI0, NULL};
const Segment * const SG81037[] = {SGI343181, SGI343182, SGI343183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343184, SGI0, NULL};
const Segment * const SG81038[] = {SGI343185, SGI343186, SGI343187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343188, SGI0, NULL};
const Segment * const SG81039[] = {SGI343189, SGI343190, SGI343191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343192, SGI0, NULL};
const Segment * const SG81040[] = {SGI343193, SGI343194, SGI343195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343196, SGI0, NULL};
const Segment * const SG81041[] = {SGI343197, SGI343198, SGI343199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343200, SGI0, NULL};
const Segment * const SG81042[] = {SGI343201, SGI343202, SGI343203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343204, SGI0, NULL};
const Segment * const SG81043[] = {SGI343205, SGI343206, SGI343207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343208, SGI0, NULL};
const Segment * const SG81044[] = {SGI343209, SGI343210, SGI343211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343212, SGI0, NULL};
const Segment * const SG81045[] = {SGI343213, SGI343214, SGI343215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343216, SGI0, NULL};
const Segment * const SG81046[] = {SGI343217, SGI343218, SGI343219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343220, SGI0, NULL};
const Segment * const SG81047[] = {SGI343221, SGI343222, SGI343223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343224, SGI0, NULL};
const Segment * const SG81048[] = {SGI343225, SGI343226, SGI343227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343228, SGI0, NULL};
const Segment * const SG81049[] = {SGI343229, SGI343230, SGI343231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343232, SGI0, NULL};
const Segment * const SG81050[] = {SGI343233, SGI343234, SGI343235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343236, SGI0, NULL};
const Segment * const SG81051[] = {SGI343237, SGI343238, SGI343239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343240, SGI0, NULL};
const Segment * const SG81052[] = {SGI343241, SGI343242, SGI343243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343244, SGI0, NULL};
const Segment * const SG81053[] = {SGI343245, SGI343246, SGI343247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343248, SGI0, NULL};
const Segment * const SG81054[] = {SGI343249, SGI343250, SGI343251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343252, SGI0, NULL};
const Segment * const SG81055[] = {SGI343253, SGI343254, SGI343255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343256, SGI0, NULL};
const Segment * const SG81056[] = {SGI343257, SGI343258, SGI343259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343260, SGI0, NULL};
const Segment * const SG81057[] = {SGI343261, SGI343262, SGI343263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343264, SGI0, NULL};
const Segment * const SG81058[] = {SGI343265, SGI343266, SGI343267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343268, SGI0, NULL};
const Segment * const SG81059[] = {SGI343269, SGI343270, SGI343271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343272, SGI0, NULL};
const Segment * const SG81060[] = {SGI343273, SGI343274, SGI343275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343276, SGI0, NULL};
const Segment * const SG81061[] = {SGI343277, SGI343278, SGI343279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343280, SGI0, NULL};
const Segment * const SG81062[] = {SGI343281, SGI343282, SGI343283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343284, SGI0, NULL};
const Segment * const SG81063[] = {SGI343285, SGI343286, SGI343287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343288, SGI0, NULL};
const Segment * const SG81064[] = {SGI343289, SGI343290, SGI343291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343292, SGI0, NULL};
const Segment * const SG81065[] = {SGI343293, SGI343294, SGI343295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343296, SGI0, NULL};
const Segment * const SG81066[] = {SGI343297, SGI343298, SGI343299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343300, SGI0, NULL};
const Segment * const SG81067[] = {SGI343301, SGI343302, SGI343303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343304, SGI0, NULL};
const Segment * const SG81068[] = {SGI343305, SGI343306, SGI343307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343308, SGI0, NULL};
const Segment * const SG81069[] = {SGI343309, SGI0, SGI0, SGI343310, SGI0, SGI0, SGI0, SGI0, SGI343314, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81070[] = {SGI343311, SGI343312, SGI343313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343315, SGI0, NULL};
const Segment * const SG81071[] = {SGI343316, SGI343317, SGI343318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343319, SGI0, NULL};
const Segment * const SG81072[] = {SGI343320, SGI343321, SGI343322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343323, SGI0, NULL};
const Segment * const SG81073[] = {SGI343324, SGI343325, SGI343326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343327, SGI0, NULL};
const Segment * const SG81074[] = {SGI343328, SGI343329, SGI343330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343331, SGI0, NULL};
const Segment * const SG81075[] = {SGI343332, SGI0, SGI0, SGI343333, SGI0, SGI0, SGI0, SGI0, SGI343335, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81076[] = {SGI343334, SGI343336, SGI343337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343338, SGI0, NULL};
const Segment * const SG81077[] = {SGI343339, SGI343340, SGI343341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343342, SGI0, NULL};
const Segment * const SG81078[] = {SGI343343, SGI343344, SGI343345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343346, SGI0, NULL};
const Segment * const SG81079[] = {SGI343347, SGI343348, SGI343349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343351, SGI0, NULL};
const Segment * const SG81080[] = {SGI343350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343352, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81081[] = {SGI343353, SGI343354, SGI343355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343356, SGI0, NULL};
const Segment * const SG81082[] = {SGI343357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343358, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81083[] = {SGI343359, SGI343360, SGI343361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343362, SGI0, NULL};
const Segment * const SG81084[] = {SGI343363, SGI343364, SGI343365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343366, SGI0, NULL};
const Segment * const SG81085[] = {SGI343367, SGI343368, SGI343369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343370, SGI0, NULL};
const Segment * const SG81086[] = {SGI343371, SGI343372, SGI343373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343374, SGI0, NULL};
const Segment * const SG81087[] = {SGI343375, SGI343376, SGI343377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343378, SGI0, NULL};
const Segment * const SG81088[] = {SGI343379, SGI343380, SGI343381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343381, SGI0, NULL};
const Segment * const SG81089[] = {SGI343382, SGI343383, SGI343384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81090[] = {SGI343385, SGI343386, SGI343387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81091[] = {SGI343388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343389, SGI0, NULL};
const Segment * const SG81092[] = {SGI343390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343391, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81093[] = {SGI343392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81094[] = {SGI343393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81095[] = {SGI343394, SGI343395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81096[] = {SGI343396, SGI343397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81097[] = {SGI343398, SGI343399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81098[] = {SGI343400, SGI343401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81099[] = {SGI343402, SGI343403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81100[] = {SGI343404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343405, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81101[] = {SGI343406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81102[] = {SGI343407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81103[] = {SGI343408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343409, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81104[] = {SGI343410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81105[] = {SGI343411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81106[] = {SGI343412, SGI0, SGI0, SGI343413, SGI0, SGI0, SGI0, SGI0, SGI343414, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81107[] = {SGI343415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81108[] = {SGI343416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81109[] = {SGI343417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343418, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81110[] = {SGI343419, SGI343420, SGI0, SGI0, NULL};
const Segment * const SG81111[] = {SGI343421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343422, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81112[] = {SGI343423, SGI343424, SGI343425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343425, SGI0, NULL};
const Segment * const SG81113[] = {SGI343426, SGI343427, SGI343428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343429, SGI0, NULL};
const Segment * const SG81114[] = {SGI343430, SGI343431, SGI343432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343433, SGI0, NULL};
const Segment * const SG81115[] = {SGI343434, SGI343435, SGI343436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343437, SGI0, NULL};
const Segment * const SG81116[] = {SGI343438, SGI343439, SGI343440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343441, SGI0, NULL};
const Segment * const SG81117[] = {SGI343442, SGI343443, SGI343444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343445, SGI0, NULL};
const Segment * const SG81118[] = {SGI343446, SGI343447, SGI343448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343449, SGI0, NULL};
const Segment * const SG81119[] = {SGI343450, SGI343451, SGI343452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343453, SGI0, NULL};
const Segment * const SG81120[] = {SGI343454, SGI0, SGI0, SGI343455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81121[] = {SGI343456, SGI343457, SGI343458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343459, SGI0, NULL};
const Segment * const SG81122[] = {SGI343460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343463, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81123[] = {SGI343461, SGI343462, SGI343464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343465, SGI0, NULL};
const Segment * const SG81124[] = {SGI343466, SGI343467, SGI343468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343469, SGI0, NULL};
const Segment * const SG81125[] = {SGI343470, SGI343471, SGI343472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343473, SGI0, NULL};
const Segment * const SG81126[] = {SGI343474, SGI343475, SGI343476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343477, SGI0, NULL};
const Segment * const SG81127[] = {SGI343478, SGI343479, SGI343480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343481, SGI0, NULL};
const Segment * const SG81128[] = {SGI343482, SGI343483, SGI343484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343485, SGI0, NULL};
const Segment * const SG81129[] = {SGI343487, SGI343488, SGI343489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343491, SGI0, NULL};
const Segment * const SG81130[] = {SGI343486, SGI0, SGI0, SGI343490, SGI0, SGI0, SGI0, SGI0, SGI343492, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81131[] = {SGI343493, SGI343494, SGI343495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343496, SGI0, NULL};
const Segment * const SG81132[] = {SGI343497, SGI343498, SGI343499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343500, SGI0, NULL};
const Segment * const SG81133[] = {SGI343501, SGI343502, SGI343503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343504, SGI0, NULL};
const Segment * const SG81134[] = {SGI343505, SGI343506, SGI343507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343508, SGI0, NULL};
const Segment * const SG81135[] = {SGI343509, SGI343510, SGI343511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343512, SGI0, NULL};
const Segment * const SG81136[] = {SGI343513, SGI343514, SGI343515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343516, SGI0, NULL};
const Segment * const SG81137[] = {SGI343517, SGI343518, SGI343519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343520, SGI0, NULL};
const Segment * const SG81138[] = {SGI343521, SGI343522, SGI343523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343524, SGI0, NULL};
const Segment * const SG81139[] = {SGI343525, SGI343526, SGI343527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343528, SGI0, NULL};
const Segment * const SG81140[] = {SGI343529, SGI343530, SGI343531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343532, SGI0, NULL};
const Segment * const SG81141[] = {SGI343533, SGI343534, SGI343535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343536, SGI0, NULL};
const Segment * const SG81142[] = {SGI343537, SGI343538, SGI343539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343540, SGI0, NULL};
const Segment * const SG81143[] = {SGI343541, SGI343542, SGI343543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343544, SGI0, NULL};
const Segment * const SG81144[] = {SGI343545, SGI343546, SGI343547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343548, SGI0, NULL};
const Segment * const SG81145[] = {SGI343549, SGI343550, SGI343551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343552, SGI0, NULL};
const Segment * const SG81146[] = {SGI343553, SGI343554, SGI343555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343556, SGI0, NULL};
const Segment * const SG81147[] = {SGI343557, SGI343558, SGI343559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343560, SGI0, NULL};
const Segment * const SG81148[] = {SGI343561, SGI343562, SGI343563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343564, SGI0, NULL};
const Segment * const SG81149[] = {SGI343565, SGI343566, SGI343567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343568, SGI0, NULL};
const Segment * const SG81150[] = {SGI343569, SGI343570, SGI343571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343572, SGI0, NULL};
const Segment * const SG81151[] = {SGI343573, SGI343574, SGI343575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343576, SGI0, NULL};
const Segment * const SG81152[] = {SGI343577, SGI343578, SGI343579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343580, SGI0, NULL};
const Segment * const SG81153[] = {SGI343581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81154[] = {SGI343582, SGI343583, SGI343584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343585, SGI0, NULL};
const Segment * const SG81155[] = {SGI343586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81156[] = {SGI343587, SGI343588, SGI343589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343590, SGI0, NULL};
const Segment * const SG81157[] = {SGI343591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81158[] = {SGI343592, SGI343593, SGI343594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343595, SGI0, NULL};
const Segment * const SG81159[] = {SGI343596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81160[] = {SGI343597, SGI343598, SGI343599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343600, SGI0, NULL};
const Segment * const SG81161[] = {SGI343601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81162[] = {SGI343602, SGI343603, SGI343604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343605, SGI0, NULL};
const Segment * const SG81163[] = {SGI343606, SGI343607, SGI343608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343609, SGI0, NULL};
const Segment * const SG81164[] = {SGI343610, SGI343611, SGI343612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343613, SGI0, NULL};
const Segment * const SG81165[] = {SGI343614, SGI343615, SGI343616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343617, SGI0, NULL};
const Segment * const SG81166[] = {SGI343618, SGI343619, SGI343620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343621, SGI0, NULL};
const Segment * const SG81167[] = {SGI343622, SGI343623, SGI343624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343625, SGI0, NULL};
const Segment * const SG81168[] = {SGI343626, SGI343627, SGI343628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343629, SGI0, NULL};
const Segment * const SG81169[] = {SGI343630, SGI343631, SGI343632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343633, SGI0, NULL};
const Segment * const SG81170[] = {SGI343634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81171[] = {SGI343635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343636, SGI0, NULL};
const Segment * const SG81172[] = {SGI343637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343638, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81173[] = {SGI343639, SGI0, SGI0, SGI343640, SGI0, SGI0, SGI0, SGI0, SGI343641, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81174[] = {SGI343642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343643, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81175[] = {SGI343644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81176[] = {SGI343645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81177[] = {SGI343646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343647, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81178[] = {SGI0, SGI343648, SGI343649, SGI0, SGI0, SGI0, SGI343650, NULL};
const Segment * const SG81179[] = {SGI343651, SGI0, SGI0, SGI343652, SGI0, SGI0, SGI0, SGI0, SGI343653, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81180[] = {SGI343654, SGI0, SGI0, SGI343655, SGI0, SGI0, SGI0, SGI0, SGI343656, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81181[] = {SGI343657, SGI0, SGI0, SGI343658, SGI0, SGI0, SGI0, SGI0, SGI343659, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81182[] = {SGI343660, SGI0, SGI0, SGI343661, SGI0, SGI0, SGI0, SGI0, SGI343662, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81183[] = {SGI343663, SGI343664, SGI0, SGI0, NULL};
const Segment * const SG81184[] = {SGI343665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81185[] = {SGI343666, SGI343667, SGI343668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343668, SGI0, NULL};
const Segment * const SG81186[] = {SGI343669, SGI343670, SGI343671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343672, SGI0, NULL};
const Segment * const SG81187[] = {SGI343673, SGI343674, SGI343675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343675, SGI0, NULL};
const Segment * const SG81188[] = {SGI343676, SGI343677, SGI343678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343679, SGI0, NULL};
const Segment * const SG81189[] = {SGI343680, SGI343681, SGI343682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343683, SGI0, NULL};
const Segment * const SG81190[] = {SGI343684, SGI343685, SGI343686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343687, SGI0, NULL};
const Segment * const SG81191[] = {SGI343688, SGI343689, SGI343690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343691, SGI0, NULL};
const Segment * const SG81192[] = {SGI343692, SGI343693, SGI0, SGI0, NULL};
const Segment * const SG81193[] = {SGI343694, SGI343695, SGI343696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343697, SGI0, NULL};
const Segment * const SG81194[] = {SGI343698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81195[] = {SGI343699, SGI343700, SGI343701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343702, SGI0, NULL};
const Segment * const SG81196[] = {SGI343703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81197[] = {SGI343704, SGI343705, SGI343706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343707, SGI0, NULL};
const Segment * const SG81198[] = {SGI343708, SGI343709, SGI343710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343711, SGI0, NULL};
const Segment * const SG81199[] = {SGI343712, SGI343713, SGI343714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343715, SGI0, NULL};
const Segment * const SG81200[] = {SGI343716, SGI343717, SGI343718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343719, SGI0, NULL};
const Segment * const SG81201[] = {SGI343720, SGI343721, SGI343722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343723, SGI0, NULL};
const Segment * const SG81202[] = {SGI343724, SGI0, SGI0, SGI343725, SGI0, SGI0, SGI0, SGI0, SGI343726, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81203[] = {SGI343727, SGI343728, SGI343729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343730, SGI0, NULL};
const Segment * const SG81204[] = {SGI343731, SGI0, SGI0, SGI343732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81205[] = {SGI343733, SGI343734, SGI343736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343738, SGI0, NULL};
const Segment * const SG81206[] = {SGI343735, SGI0, SGI0, SGI343737, SGI0, SGI0, SGI0, SGI0, SGI343739, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81207[] = {SGI343740, SGI343741, SGI343742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343743, SGI0, NULL};
const Segment * const SG81208[] = {SGI343744, SGI343745, SGI343746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343747, SGI0, NULL};
const Segment * const SG81209[] = {SGI343748, SGI343749, SGI343750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343751, SGI0, NULL};
const Segment * const SG81210[] = {SGI343752, SGI343753, SGI343754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343755, SGI0, NULL};
const Segment * const SG81211[] = {SGI343756, SGI343757, SGI343758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343759, SGI0, NULL};
const Segment * const SG81212[] = {SGI343760, SGI343761, SGI343762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343763, SGI0, NULL};
const Segment * const SG81213[] = {SGI343764, SGI343765, SGI343766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343767, SGI0, NULL};
const Segment * const SG81214[] = {SGI343768, SGI343769, SGI343770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343771, SGI0, NULL};
const Segment * const SG81215[] = {SGI343772, SGI343773, SGI343774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343775, SGI0, NULL};
const Segment * const SG81216[] = {SGI343776, SGI343777, SGI343778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343779, SGI0, NULL};
const Segment * const SG81217[] = {SGI343780, SGI343781, SGI343782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343783, SGI0, NULL};
const Segment * const SG81218[] = {SGI343784, SGI343785, SGI343786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343787, SGI0, NULL};
const Segment * const SG81219[] = {SGI343788, SGI343789, SGI343790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343791, SGI0, NULL};
const Segment * const SG81220[] = {SGI343792, SGI343793, SGI343794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343795, SGI0, NULL};
const Segment * const SG81221[] = {SGI343796, SGI343797, SGI343798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343799, SGI0, NULL};
const Segment * const SG81222[] = {SGI343800, SGI343801, SGI343802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343803, SGI0, NULL};
const Segment * const SG81223[] = {SGI343804, SGI343806, SGI343807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343808, SGI0, NULL};
const Segment * const SG81224[] = {SGI343805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81225[] = {SGI343809, SGI343810, SGI343811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343812, SGI0, NULL};
const Segment * const SG81226[] = {SGI343813, SGI343814, SGI343815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343816, SGI0, NULL};
const Segment * const SG81227[] = {SGI343817, SGI343818, SGI343819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343820, SGI0, NULL};
const Segment * const SG81228[] = {SGI343821, SGI343822, SGI343823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343824, SGI0, NULL};
const Segment * const SG81229[] = {SGI343825, SGI343826, SGI343827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343828, SGI0, NULL};
const Segment * const SG81230[] = {SGI343829, SGI343830, SGI343831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343832, SGI0, NULL};
const Segment * const SG81231[] = {SGI343833, SGI343834, SGI343835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343836, SGI0, NULL};
const Segment * const SG81232[] = {SGI343837, SGI343838, SGI343839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343840, SGI0, NULL};
const Segment * const SG81233[] = {SGI343841, SGI343842, SGI343843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343844, SGI0, NULL};
const Segment * const SG81234[] = {SGI343845, SGI343846, SGI343847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343848, SGI0, NULL};
const Segment * const SG81235[] = {SGI343849, SGI0, SGI0, SGI343850, SGI0, SGI0, SGI0, SGI0, SGI343851, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81236[] = {SGI343852, SGI0, SGI0, SGI343853, SGI0, SGI0, SGI0, SGI0, SGI343854, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81237[] = {SGI343855, SGI0, SGI0, SGI343856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81238[] = {SGI343857, SGI343858, SGI343859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343860, SGI0, NULL};
const Segment * const SG81239[] = {SGI343861, SGI343862, SGI343863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343864, SGI0, NULL};
const Segment * const SG81240[] = {SGI343865, SGI343866, SGI343867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343868, SGI0, NULL};
const Segment * const SG81241[] = {SGI343869, SGI343870, SGI343871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343872, SGI0, NULL};
const Segment * const SG81242[] = {SGI343873, SGI343874, SGI343875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343876, SGI0, NULL};
const Segment * const SG81243[] = {SGI343877, SGI343878, SGI343879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343880, SGI0, NULL};
const Segment * const SG81244[] = {SGI343881, SGI343882, SGI343883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343884, SGI0, NULL};
const Segment * const SG81245[] = {SGI343885, SGI343886, SGI343887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343888, SGI0, NULL};
const Segment * const SG81246[] = {SGI343889, SGI343890, SGI343891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343892, SGI0, NULL};
const Segment * const SG81247[] = {SGI343893, SGI343894, SGI343895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343896, SGI0, NULL};
const Segment * const SG81248[] = {SGI343897, SGI343898, SGI343899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343900, SGI0, NULL};
const Segment * const SG81249[] = {SGI343901, SGI343902, SGI343903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343904, SGI0, NULL};
const Segment * const SG81250[] = {SGI343905, SGI343906, SGI343907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343908, SGI0, NULL};
const Segment * const SG81251[] = {SGI343909, SGI343910, SGI343911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343912, SGI0, NULL};
const Segment * const SG81252[] = {SGI343913, SGI343914, SGI343915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343916, SGI0, NULL};
const Segment * const SG81253[] = {SGI343917, SGI343918, SGI343919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343920, SGI0, NULL};
const Segment * const SG81254[] = {SGI343921, SGI343922, SGI343923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343924, SGI0, NULL};
const Segment * const SG81255[] = {SGI343925, SGI343926, SGI343927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343928, SGI0, NULL};
const Segment * const SG81256[] = {SGI343929, SGI343930, SGI343931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343932, SGI0, NULL};
const Segment * const SG81257[] = {SGI343933, SGI343934, SGI343935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343936, SGI0, NULL};
const Segment * const SG81258[] = {SGI343937, SGI343938, SGI343939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343940, SGI0, NULL};
const Segment * const SG81259[] = {SGI343941, SGI343942, SGI343943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343944, SGI0, NULL};
const Segment * const SG81260[] = {SGI343945, SGI343946, SGI343947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343948, SGI0, NULL};
const Segment * const SG81261[] = {SGI343949, SGI343950, SGI343951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343952, SGI0, NULL};
const Segment * const SG81262[] = {SGI343953, SGI343954, SGI343955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343956, SGI0, NULL};
const Segment * const SG81263[] = {SGI343957, SGI343958, SGI343959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343960, SGI0, NULL};
const Segment * const SG81264[] = {SGI343961, SGI343962, SGI343963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343964, SGI0, NULL};
const Segment * const SG81265[] = {SGI343965, SGI343966, SGI343967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343968, SGI0, NULL};
const Segment * const SG81266[] = {SGI343969, SGI343970, SGI343971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343972, SGI0, NULL};
const Segment * const SG81267[] = {SGI343973, SGI343974, SGI343975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343976, SGI0, NULL};
const Segment * const SG81268[] = {SGI343977, SGI343978, SGI343979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343980, SGI0, NULL};
const Segment * const SG81269[] = {SGI343981, SGI343982, SGI343983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343984, SGI0, NULL};
const Segment * const SG81270[] = {SGI343985, SGI343986, SGI343987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343988, SGI0, NULL};
const Segment * const SG81271[] = {SGI343989, SGI343990, SGI343991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343992, SGI0, NULL};
const Segment * const SG81272[] = {SGI343993, SGI343994, SGI343995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343995, SGI0, NULL};
const Segment * const SG81273[] = {SGI343996, SGI343997, SGI343998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI343999, SGI0, NULL};
const Segment * const SG81274[] = {SGI344000, SGI344001, SGI344002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344003, SGI0, NULL};
const Segment * const SG81275[] = {SGI344004, SGI344005, SGI344006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344007, SGI0, NULL};
const Segment * const SG81276[] = {SGI344008, SGI344009, SGI344010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344011, SGI0, NULL};
const Segment * const SG81277[] = {SGI344012, SGI344013, SGI344014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344015, SGI0, NULL};
const Segment * const SG81278[] = {SGI344016, SGI344017, SGI344018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344019, SGI0, NULL};
const Segment * const SG81279[] = {SGI344020, SGI344021, SGI344022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344023, SGI0, NULL};
const Segment * const SG81280[] = {SGI344024, SGI344025, SGI344026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344027, SGI0, NULL};
const Segment * const SG81281[] = {SGI344028, SGI344029, SGI344030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344031, SGI0, NULL};
const Segment * const SG81282[] = {SGI344032, SGI344033, SGI344034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344035, SGI0, NULL};
const Segment * const SG81283[] = {SGI344036, SGI344037, SGI344038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344039, SGI0, NULL};
const Segment * const SG81284[] = {SGI344040, SGI344041, SGI344042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344043, SGI0, NULL};
const Segment * const SG81285[] = {SGI344044, SGI344045, SGI344046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344047, SGI0, NULL};
const Segment * const SG81286[] = {SGI344048, SGI344049, SGI344050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344051, SGI0, NULL};
const Segment * const SG81287[] = {SGI344052, SGI344053, SGI344054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344055, SGI0, NULL};
const Segment * const SG81288[] = {SGI344056, SGI344057, SGI344058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344059, SGI0, NULL};
const Segment * const SG81289[] = {SGI344060, SGI344061, SGI344062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344063, SGI0, NULL};
const Segment * const SG81290[] = {SGI344064, SGI344065, SGI344066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344067, SGI0, NULL};
const Segment * const SG81291[] = {SGI344068, SGI344069, SGI344070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344071, SGI0, NULL};
const Segment * const SG81292[] = {SGI344072, SGI344073, SGI344074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344075, SGI0, NULL};
const Segment * const SG81293[] = {SGI344076, SGI344077, SGI344078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344079, SGI0, NULL};
const Segment * const SG81294[] = {SGI344080, SGI344081, SGI344082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344083, SGI0, NULL};
const Segment * const SG81295[] = {SGI344084, SGI344085, SGI344086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344087, SGI0, NULL};
const Segment * const SG81296[] = {SGI344088, SGI344089, SGI344090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344091, SGI0, NULL};
const Segment * const SG81297[] = {SGI344092, SGI344093, SGI344094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344095, SGI0, NULL};
const Segment * const SG81298[] = {SGI344096, SGI344097, SGI344098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344099, SGI0, NULL};
const Segment * const SG81299[] = {SGI344101, SGI344103, SGI344104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344105, SGI0, NULL};
const Segment * const SG81300[] = {SGI344100, SGI0, SGI0, SGI344102, SGI0, SGI0, SGI0, SGI0, SGI344106, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81301[] = {SGI344107, SGI344108, SGI344109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344110, SGI0, NULL};
const Segment * const SG81302[] = {SGI344111, SGI344112, SGI344113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344114, SGI0, NULL};
const Segment * const SG81303[] = {SGI344115, SGI344116, SGI344117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344118, SGI0, NULL};
const Segment * const SG81304[] = {SGI344119, SGI344120, SGI344121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344122, SGI0, NULL};
const Segment * const SG81305[] = {SGI344123, SGI344124, SGI344125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344126, SGI0, NULL};
const Segment * const SG81306[] = {SGI344127, SGI344128, SGI344129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344130, SGI0, NULL};
const Segment * const SG81307[] = {SGI344131, SGI344132, SGI344133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344134, SGI0, NULL};
const Segment * const SG81308[] = {SGI344135, SGI344136, SGI344137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344138, SGI0, NULL};
const Segment * const SG81309[] = {SGI344139, SGI344140, SGI344141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344142, SGI0, NULL};
const Segment * const SG81310[] = {SGI344143, SGI344144, SGI344145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344146, SGI0, NULL};
const Segment * const SG81311[] = {SGI344147, SGI344148, SGI344149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344150, SGI0, NULL};
const Segment * const SG81312[] = {SGI344151, SGI0, SGI0, SGI344152, SGI0, SGI0, SGI0, SGI0, SGI344155, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81313[] = {SGI344153, SGI344154, SGI344156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344157, SGI0, NULL};
const Segment * const SG81314[] = {SGI344158, SGI0, SGI0, SGI344159, SGI0, SGI0, SGI0, SGI0, SGI344162, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81315[] = {SGI344160, SGI344161, SGI344163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344164, SGI0, NULL};
const Segment * const SG81316[] = {SGI344165, SGI0, SGI0, SGI344166, SGI0, SGI0, SGI0, SGI0, SGI344167, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81317[] = {SGI344168, SGI344169, SGI344170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344171, SGI0, NULL};
const Segment * const SG81318[] = {SGI344172, SGI0, SGI0, SGI344173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81319[] = {SGI344174, SGI344175, SGI344176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344177, SGI0, NULL};
const Segment * const SG81320[] = {SGI344178, SGI0, SGI0, SGI344179, SGI0, SGI0, SGI0, SGI0, SGI344180, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81321[] = {SGI344181, SGI344182, SGI344183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344184, SGI0, NULL};
const Segment * const SG81322[] = {SGI344185, SGI344186, SGI344187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344188, SGI0, NULL};
const Segment * const SG81323[] = {SGI344189, SGI344190, SGI344191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344192, SGI0, NULL};
const Segment * const SG81324[] = {SGI344193, SGI344194, SGI344195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344196, SGI0, NULL};
const Segment * const SG81325[] = {SGI344197, SGI344198, SGI344199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344200, SGI0, NULL};
const Segment * const SG81326[] = {SGI344201, SGI344202, SGI344203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344204, SGI0, NULL};
const Segment * const SG81327[] = {SGI344205, SGI344206, SGI344207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344208, SGI0, NULL};
const Segment * const SG81328[] = {SGI344209, SGI344210, SGI344211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344212, SGI0, NULL};
const Segment * const SG81329[] = {SGI344213, SGI344214, SGI344215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344216, SGI0, NULL};
const Segment * const SG81330[] = {SGI344217, SGI344218, SGI344219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344220, SGI0, NULL};
const Segment * const SG81331[] = {SGI344221, SGI344222, SGI344223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344224, SGI0, NULL};
const Segment * const SG81332[] = {SGI344225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81333[] = {SGI344226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81334[] = {SGI344227, SGI344228, SGI344229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344230, SGI0, NULL};
const Segment * const SG81335[] = {SGI344231, SGI344232, SGI344233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344234, SGI0, NULL};
const Segment * const SG81336[] = {SGI344235, SGI344236, SGI344237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344238, SGI0, NULL};
const Segment * const SG81337[] = {SGI344239, SGI344240, SGI344241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344242, SGI0, NULL};
const Segment * const SG81338[] = {SGI344243, SGI344244, SGI344245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344246, SGI0, NULL};
const Segment * const SG81339[] = {SGI344247, SGI344248, SGI344249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344249, SGI0, NULL};
const Segment * const SG81340[] = {SGI344250, SGI344251, SGI344252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344253, SGI0, NULL};
const Segment * const SG81341[] = {SGI344254, SGI344255, SGI344256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344257, SGI0, NULL};
const Segment * const SG81342[] = {SGI344258, SGI344259, SGI344260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344261, SGI0, NULL};
const Segment * const SG81343[] = {SGI344262, SGI344263, SGI344264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344265, SGI0, NULL};
const Segment * const SG81344[] = {SGI344266, SGI344267, SGI344268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344269, SGI0, NULL};
const Segment * const SG81345[] = {SGI344270, SGI344271, SGI344272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344273, SGI0, NULL};
const Segment * const SG81346[] = {SGI344274, SGI344275, SGI344276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344277, SGI0, NULL};
const Segment * const SG81347[] = {SGI344278, SGI344279, SGI344280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344281, SGI0, NULL};
const Segment * const SG81348[] = {SGI344282, SGI344283, SGI344284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344285, SGI0, NULL};
const Segment * const SG81349[] = {SGI344286, SGI344287, SGI344288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344289, SGI0, NULL};
const Segment * const SG81350[] = {SGI344290, SGI344291, SGI344292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344293, SGI0, NULL};
const Segment * const SG81351[] = {SGI344294, SGI344295, SGI344296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344297, SGI0, NULL};
const Segment * const SG81352[] = {SGI344298, SGI344299, SGI344300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344301, SGI0, NULL};
const Segment * const SG81353[] = {SGI344302, SGI344303, SGI344304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344305, SGI0, NULL};
const Segment * const SG81354[] = {SGI344306, SGI344307, SGI344308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344309, SGI0, NULL};
const Segment * const SG81355[] = {SGI344310, SGI344311, SGI344312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344313, SGI0, NULL};
const Segment * const SG81356[] = {SGI344314, SGI344315, SGI344316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344317, SGI0, NULL};
const Segment * const SG81357[] = {SGI344318, SGI344319, SGI344320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344321, SGI0, NULL};
const Segment * const SG81358[] = {SGI344322, SGI344323, SGI344324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344325, SGI0, NULL};
const Segment * const SG81359[] = {SGI344326, SGI344327, SGI344328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344329, SGI0, NULL};
const Segment * const SG81360[] = {SGI344330, SGI344331, SGI344332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344333, SGI0, NULL};
const Segment * const SG81361[] = {SGI344334, SGI344335, SGI344336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344337, SGI0, NULL};
const Segment * const SG81362[] = {SGI344338, SGI344339, SGI344340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344341, SGI0, NULL};
const Segment * const SG81363[] = {SGI344342, SGI344343, SGI344344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344345, SGI0, NULL};
const Segment * const SG81364[] = {SGI344346, SGI344347, SGI344348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344349, SGI0, NULL};
const Segment * const SG81365[] = {SGI344350, SGI344351, SGI344352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344353, SGI0, NULL};
const Segment * const SG81366[] = {SGI344354, SGI344355, SGI344356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344357, SGI0, NULL};
const Segment * const SG81367[] = {SGI344358, SGI344359, SGI344360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344361, SGI0, NULL};
const Segment * const SG81368[] = {SGI344362, SGI344363, SGI344364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344365, SGI0, NULL};
const Segment * const SG81369[] = {SGI344366, SGI344367, SGI344368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344369, SGI0, NULL};
const Segment * const SG81370[] = {SGI344370, SGI344371, SGI344372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344373, SGI0, NULL};
const Segment * const SG81371[] = {SGI344374, SGI344375, SGI344376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344377, SGI0, NULL};
const Segment * const SG81372[] = {SGI344378, SGI344379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81373[] = {SGI344380, SGI344381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81374[] = {SGI344382, SGI344383, SGI344384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344385, SGI0, NULL};
const Segment * const SG81375[] = {SGI344386, SGI344387, SGI344388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344389, SGI0, NULL};
const Segment * const SG81376[] = {SGI344390, SGI344391, SGI344392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344393, SGI0, NULL};
const Segment * const SG81377[] = {SGI344394, SGI344395, SGI344396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344397, SGI0, NULL};
const Segment * const SG81378[] = {SGI344398, SGI344399, SGI344400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344401, SGI0, NULL};
const Segment * const SG81379[] = {SGI344402, SGI344403, SGI344404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344405, SGI0, NULL};
const Segment * const SG81380[] = {SGI344406, SGI344407, SGI344408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344409, SGI0, NULL};
const Segment * const SG81381[] = {SGI344410, SGI344411, SGI344412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344413, SGI0, NULL};
const Segment * const SG81382[] = {SGI344414, SGI344415, SGI344416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344417, SGI0, NULL};
const Segment * const SG81383[] = {SGI344418, SGI344419, SGI344420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344421, SGI0, NULL};
const Segment * const SG81384[] = {SGI344422, SGI344423, SGI344424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344425, SGI0, NULL};
const Segment * const SG81385[] = {SGI344426, SGI344427, SGI344428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344429, SGI0, NULL};
const Segment * const SG81386[] = {SGI344430, SGI0, SGI0, SGI344431, SGI0, SGI0, SGI0, SGI0, SGI344434, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81387[] = {SGI344432, SGI344433, SGI344435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344436, SGI0, NULL};
const Segment * const SG81388[] = {SGI344438, SGI344439, SGI344441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344442, SGI0, NULL};
const Segment * const SG81389[] = {SGI344437, SGI0, SGI0, SGI344440, SGI0, SGI0, SGI0, SGI0, SGI344443, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81390[] = {SGI344444, SGI344445, SGI344446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344447, SGI0, NULL};
const Segment * const SG81391[] = {SGI344448, SGI344449, SGI344450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344451, SGI0, NULL};
const Segment * const SG81392[] = {SGI344452, SGI344453, SGI344454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344455, SGI0, NULL};
const Segment * const SG81393[] = {SGI344456, SGI0, SGI344457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81394[] = {SGI344458, SGI0, SGI0, SGI344459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81395[] = {SGI344460, SGI0, SGI344461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81396[] = {SGI344462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344463, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81397[] = {SGI344464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81398[] = {SGI344466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81399[] = {SGI344465, SGI344467, SGI344468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344469, SGI0, NULL};
const Segment * const SG81400[] = {SGI344470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81401[] = {SGI344471, SGI344472, SGI344473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344474, SGI0, NULL};
const Segment * const SG81402[] = {SGI344475, SGI344476, SGI344477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344477, SGI0, NULL};
const Segment * const SG81403[] = {SGI344478, SGI344479, SGI344480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344481, SGI0, NULL};
const Segment * const SG81404[] = {SGI344482, SGI344483, SGI344484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344485, SGI0, NULL};
const Segment * const SG81405[] = {SGI344486, SGI344487, SGI344488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344489, SGI0, NULL};
const Segment * const SG81406[] = {SGI344490, SGI344491, SGI344492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344493, SGI0, NULL};
const Segment * const SG81407[] = {SGI344494, SGI344495, SGI344496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344497, SGI0, NULL};
const Segment * const SG81408[] = {SGI344498, SGI344499, SGI344500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344501, SGI0, NULL};
const Segment * const SG81409[] = {SGI344502, SGI344503, SGI344504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344505, SGI0, NULL};
const Segment * const SG81410[] = {SGI344506, SGI344507, SGI344508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344509, SGI0, NULL};
const Segment * const SG81411[] = {SGI344510, SGI344511, SGI344512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344513, SGI0, NULL};
const Segment * const SG81412[] = {SGI344514, SGI344515, SGI344516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344517, SGI0, NULL};
const Segment * const SG81413[] = {SGI344518, SGI344519, SGI344520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344521, SGI0, NULL};
const Segment * const SG81414[] = {SGI344522, SGI344523, SGI344524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344525, SGI0, NULL};
const Segment * const SG81415[] = {SGI344526, SGI344527, SGI344528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344529, SGI0, NULL};
const Segment * const SG81416[] = {SGI344531, SGI344533, SGI344534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344535, SGI0, NULL};
const Segment * const SG81417[] = {SGI344530, SGI0, SGI0, SGI344532, SGI0, SGI0, SGI0, SGI0, SGI344536, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81418[] = {SGI344537, SGI344538, SGI344539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344540, SGI0, NULL};
const Segment * const SG81419[] = {SGI344541, SGI344542, SGI344543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344544, SGI0, NULL};
const Segment * const SG81420[] = {SGI344545, SGI344546, SGI344547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344548, SGI0, NULL};
const Segment * const SG81421[] = {SGI344549, SGI344550, SGI344551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344552, SGI0, NULL};
const Segment * const SG81422[] = {SGI344553, SGI344554, SGI344555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344556, SGI0, NULL};
const Segment * const SG81423[] = {SGI344557, SGI344558, SGI344559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344560, SGI0, NULL};
const Segment * const SG81424[] = {SGI344561, SGI344562, SGI344563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344564, SGI0, NULL};
const Segment * const SG81425[] = {SGI344565, SGI344566, SGI344567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344568, SGI0, NULL};
const Segment * const SG81426[] = {SGI344569, SGI344570, SGI344571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344572, SGI0, NULL};
const Segment * const SG81427[] = {SGI344573, SGI344574, SGI344575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344576, SGI0, NULL};
const Segment * const SG81428[] = {SGI344577, SGI344578, SGI344579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344580, SGI0, NULL};
const Segment * const SG81429[] = {SGI344581, SGI344582, SGI344583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344584, SGI0, NULL};
const Segment * const SG81430[] = {SGI344585, SGI344586, SGI344587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344588, SGI0, NULL};
const Segment * const SG81431[] = {SGI344589, SGI344590, SGI344591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344592, SGI0, NULL};
const Segment * const SG81432[] = {SGI344593, SGI344594, SGI344595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344596, SGI0, NULL};
const Segment * const SG81433[] = {SGI344597, SGI344598, SGI344599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344600, SGI0, NULL};
const Segment * const SG81434[] = {SGI344601, SGI344602, SGI344603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344604, SGI0, NULL};
const Segment * const SG81435[] = {SGI344605, SGI344606, SGI344607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344608, SGI0, NULL};
const Segment * const SG81436[] = {SGI344609, SGI344610, SGI344611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344612, SGI0, NULL};
const Segment * const SG81437[] = {SGI344613, SGI344614, SGI0, SGI0, NULL};
const Segment * const SG81438[] = {SGI344615, SGI344616, SGI344617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344618, SGI0, NULL};
const Segment * const SG81439[] = {SGI344619, SGI344620, SGI344621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344622, SGI0, NULL};
const Segment * const SG81440[] = {SGI344623, SGI344624, SGI344625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344626, SGI0, NULL};
const Segment * const SG81441[] = {SGI344627, SGI344628, SGI344629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344630, SGI0, NULL};
const Segment * const SG81442[] = {SGI344631, SGI344632, SGI344633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344634, SGI0, NULL};
const Segment * const SG81443[] = {SGI344635, SGI344636, SGI344637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344638, SGI0, NULL};
const Segment * const SG81444[] = {SGI344639, SGI344640, SGI344641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344642, SGI0, NULL};
const Segment * const SG81445[] = {SGI344643, SGI344644, SGI344645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344645, SGI0, NULL};
const Segment * const SG81446[] = {SGI344646, SGI344647, SGI344648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344649, SGI0, NULL};
const Segment * const SG81447[] = {SGI344650, SGI344651, SGI344652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344653, SGI0, NULL};
const Segment * const SG81448[] = {SGI344654, SGI344655, SGI344656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344657, SGI0, NULL};
const Segment * const SG81449[] = {SGI344658, SGI344659, SGI344660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344661, SGI0, NULL};
const Segment * const SG81450[] = {SGI344662, SGI344663, SGI344664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344665, SGI0, NULL};
const Segment * const SG81451[] = {SGI344666, SGI344667, SGI344668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344669, SGI0, NULL};
const Segment * const SG81452[] = {SGI344670, SGI344671, SGI344672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344673, SGI0, NULL};
const Segment * const SG81453[] = {SGI344674, SGI344675, SGI344676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344677, SGI0, NULL};
const Segment * const SG81454[] = {SGI344678, SGI344679, SGI344680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344681, SGI0, NULL};
const Segment * const SG81455[] = {SGI344682, SGI344683, SGI344684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344685, SGI0, NULL};
const Segment * const SG81456[] = {SGI344686, SGI344687, SGI344688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344689, SGI0, NULL};
const Segment * const SG81457[] = {SGI344690, SGI344691, SGI344692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344693, SGI0, NULL};
const Segment * const SG81458[] = {SGI344694, SGI344695, SGI344696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344697, SGI0, NULL};
const Segment * const SG81459[] = {SGI344698, SGI344699, SGI344700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344701, SGI0, NULL};
const Segment * const SG81460[] = {SGI344702, SGI344703, SGI344704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344705, SGI0, NULL};
const Segment * const SG81461[] = {SGI344706, SGI344707, SGI344708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344709, SGI0, NULL};
const Segment * const SG81462[] = {SGI344710, SGI344711, SGI344712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344713, SGI0, NULL};
const Segment * const SG81463[] = {SGI344714, SGI344715, SGI344716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344717, SGI0, NULL};
const Segment * const SG81464[] = {SGI344718, SGI344719, SGI344720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344721, SGI0, NULL};
const Segment * const SG81465[] = {SGI344722, SGI344723, SGI344724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344725, SGI0, NULL};
const Segment * const SG81466[] = {SGI344726, SGI344727, SGI344728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344729, SGI0, NULL};
const Segment * const SG81467[] = {SGI344730, SGI344731, SGI344732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344733, SGI0, NULL};
const Segment * const SG81468[] = {SGI344734, SGI344735, SGI344736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344737, SGI0, NULL};
const Segment * const SG81469[] = {SGI344738, SGI344739, SGI344740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344741, SGI0, NULL};
const Segment * const SG81470[] = {SGI344742, SGI344743, SGI344744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344745, SGI0, NULL};
const Segment * const SG81471[] = {SGI344746, SGI344747, SGI344748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344749, SGI0, NULL};
const Segment * const SG81472[] = {SGI344750, SGI344751, SGI344752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344753, SGI0, NULL};
const Segment * const SG81473[] = {SGI344754, SGI344755, SGI344756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344757, SGI0, NULL};
const Segment * const SG81474[] = {SGI344758, SGI344759, SGI344760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344761, SGI0, NULL};
const Segment * const SG81475[] = {SGI344762, SGI344763, SGI344764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344765, SGI0, NULL};
const Segment * const SG81476[] = {SGI344766, SGI344767, SGI344768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344769, SGI0, NULL};
const Segment * const SG81477[] = {SGI344770, SGI344771, SGI344772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344773, SGI0, NULL};
const Segment * const SG81478[] = {SGI344774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344775, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81479[] = {SGI344776, SGI344777, SGI344778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344779, SGI0, NULL};
const Segment * const SG81480[] = {SGI344780, SGI344781, SGI344782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344783, SGI0, NULL};
const Segment * const SG81481[] = {SGI344784, SGI344785, SGI344786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344787, SGI0, NULL};
const Segment * const SG81482[] = {SGI344788, SGI344789, SGI344790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344791, SGI0, NULL};
const Segment * const SG81483[] = {SGI344792, SGI344793, SGI344794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344795, SGI0, NULL};
const Segment * const SG81484[] = {SGI344796, SGI344797, SGI344798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344799, SGI0, NULL};
const Segment * const SG81485[] = {SGI344800, SGI344801, SGI344802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344803, SGI0, NULL};
const Segment * const SG81486[] = {SGI344804, SGI344805, SGI344806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344807, SGI0, NULL};
const Segment * const SG81487[] = {SGI344808, SGI344809, SGI344810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344811, SGI0, NULL};
const Segment * const SG81488[] = {SGI344812, SGI344813, SGI344814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344815, SGI0, NULL};
const Segment * const SG81489[] = {SGI344816, SGI344817, SGI344818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344819, SGI0, NULL};
const Segment * const SG81490[] = {SGI344820, SGI344821, SGI344822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344823, SGI0, NULL};
const Segment * const SG81491[] = {SGI344824, SGI344825, SGI344826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344827, SGI0, NULL};
const Segment * const SG81492[] = {SGI344828, SGI344829, SGI344830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344831, SGI0, NULL};
const Segment * const SG81493[] = {SGI344832, SGI344833, SGI344834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344835, SGI0, NULL};
const Segment * const SG81494[] = {SGI344836, SGI344837, SGI344838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344839, SGI0, NULL};
const Segment * const SG81495[] = {SGI344840, SGI344841, SGI344842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344843, SGI0, NULL};
const Segment * const SG81496[] = {SGI344844, SGI344845, SGI344846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344847, SGI0, NULL};
const Segment * const SG81497[] = {SGI344848, SGI344849, SGI344850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344851, SGI0, NULL};
const Segment * const SG81498[] = {SGI344852, SGI344853, SGI344854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344855, SGI0, NULL};
const Segment * const SG81499[] = {SGI344856, SGI344857, SGI344858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344859, SGI0, NULL};
const Segment * const SG81500[] = {SGI344860, SGI344861, SGI344862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344863, SGI0, NULL};
const Segment * const SG81501[] = {SGI344864, SGI344865, SGI344866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344867, SGI0, NULL};
const Segment * const SG81502[] = {SGI344868, SGI344869, SGI344870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344871, SGI0, NULL};
const Segment * const SG81503[] = {SGI344872, SGI344873, SGI344874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344875, SGI0, NULL};
const Segment * const SG81504[] = {SGI344876, SGI344877, SGI344878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344879, SGI0, NULL};
const Segment * const SG81505[] = {SGI344880, SGI344881, SGI344882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344883, SGI0, NULL};
const Segment * const SG81506[] = {SGI344884, SGI344885, SGI344886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344887, SGI0, NULL};
const Segment * const SG81507[] = {SGI344888, SGI344889, SGI344890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344891, SGI0, NULL};
const Segment * const SG81508[] = {SGI344892, SGI344893, SGI344894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344895, SGI0, NULL};
const Segment * const SG81509[] = {SGI344896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81510[] = {SGI344897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81511[] = {SGI344898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81512[] = {SGI344899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81513[] = {SGI344900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81514[] = {SGI344901, SGI0, SGI0, SGI344902, SGI0, SGI0, SGI0, SGI0, SGI344903, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81515[] = {SGI344904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81516[] = {SGI344905, SGI344906, SGI344907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344907, SGI0, NULL};
const Segment * const SG81517[] = {SGI344908, SGI344909, SGI344910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344910, SGI0, NULL};
const Segment * const SG81518[] = {SGI344911, SGI344912, SGI344913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344914, SGI0, NULL};
const Segment * const SG81519[] = {SGI344915, SGI344916, SGI344917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344918, SGI0, NULL};
const Segment * const SG81520[] = {SGI344919, SGI344920, SGI344921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344922, SGI0, NULL};
const Segment * const SG81521[] = {SGI344923, SGI344924, SGI344925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344926, SGI0, NULL};
const Segment * const SG81522[] = {SGI344927, SGI344928, SGI344929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344930, SGI0, NULL};
const Segment * const SG81523[] = {SGI344931, SGI344932, SGI344933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344934, SGI0, NULL};
const Segment * const SG81524[] = {SGI344935, SGI344936, SGI344937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344938, SGI0, NULL};
const Segment * const SG81525[] = {SGI344939, SGI344940, SGI344941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344942, SGI0, NULL};
const Segment * const SG81526[] = {SGI344943, SGI344944, SGI344945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344946, SGI0, NULL};
const Segment * const SG81527[] = {SGI344947, SGI344948, SGI344949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344950, SGI0, NULL};
const Segment * const SG81528[] = {SGI344951, SGI344952, SGI344953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344954, SGI0, NULL};
const Segment * const SG81529[] = {SGI344955, SGI344956, SGI344957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344958, SGI0, NULL};
const Segment * const SG81530[] = {SGI344959, SGI344960, SGI344961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344962, SGI0, NULL};
const Segment * const SG81531[] = {SGI344963, SGI344964, SGI344965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344966, SGI0, NULL};
const Segment * const SG81532[] = {SGI344967, SGI344968, SGI344969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344970, SGI0, NULL};
const Segment * const SG81533[] = {SGI344971, SGI344972, SGI344973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344974, SGI0, NULL};
const Segment * const SG81534[] = {SGI344975, SGI344976, SGI344977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344978, SGI0, NULL};
const Segment * const SG81535[] = {SGI344979, SGI344980, SGI344981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344982, SGI0, NULL};
const Segment * const SG81536[] = {SGI344983, SGI344984, SGI344985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344986, SGI0, NULL};
const Segment * const SG81537[] = {SGI344987, SGI344988, SGI344989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344990, SGI0, NULL};
const Segment * const SG81538[] = {SGI344991, SGI344992, SGI344993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344994, SGI0, NULL};
const Segment * const SG81539[] = {SGI344995, SGI344996, SGI344997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI344998, SGI0, NULL};
const Segment * const SG81540[] = {SGI344999, SGI345000, SGI345001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345002, SGI0, NULL};
const Segment * const SG81541[] = {SGI345003, SGI345004, SGI345005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345006, SGI0, NULL};
const Segment * const SG81542[] = {SGI345007, SGI345008, SGI345009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345010, SGI0, NULL};
const Segment * const SG81543[] = {SGI345011, SGI345012, SGI345013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345014, SGI0, NULL};
const Segment * const SG81544[] = {SGI345015, SGI345016, SGI345017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345018, SGI0, NULL};
const Segment * const SG81545[] = {SGI345019, SGI345020, SGI345021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345022, SGI0, NULL};
const Segment * const SG81546[] = {SGI345023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345024, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81547[] = {SGI345025, SGI345026, SGI345027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345028, SGI0, NULL};
const Segment * const SG81548[] = {SGI345029, SGI345030, SGI345031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345032, SGI0, NULL};
const Segment * const SG81549[] = {SGI345033, SGI345034, SGI345035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345036, SGI0, NULL};
const Segment * const SG81550[] = {SGI345037, SGI345038, SGI345039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345040, SGI0, NULL};
const Segment * const SG81551[] = {SGI345041, SGI345042, SGI345043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345044, SGI0, NULL};
const Segment * const SG81552[] = {SGI345045, SGI345046, SGI345047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345048, SGI0, NULL};
const Segment * const SG81553[] = {SGI345049, SGI345050, SGI345051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345052, SGI0, NULL};
const Segment * const SG81554[] = {SGI345053, SGI345054, SGI345055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345056, SGI0, NULL};
const Segment * const SG81555[] = {SGI345057, SGI345058, SGI345059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345060, SGI0, NULL};
const Segment * const SG81556[] = {SGI345061, SGI345062, SGI345063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345064, SGI0, NULL};
const Segment * const SG81557[] = {SGI345065, SGI345066, SGI345067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345068, SGI0, NULL};
const Segment * const SG81558[] = {SGI345069, SGI345070, SGI345071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345072, SGI0, NULL};
const Segment * const SG81559[] = {SGI345073, SGI345074, SGI345075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345076, SGI0, NULL};
const Segment * const SG81560[] = {SGI345077, SGI0, SGI0, SGI345078, SGI0, SGI0, SGI0, SGI0, SGI345080, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81561[] = {SGI345079, SGI345081, SGI345082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345083, SGI0, NULL};
const Segment * const SG81562[] = {SGI345085, SGI345086, SGI345088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345089, SGI0, NULL};
const Segment * const SG81563[] = {SGI345084, SGI0, SGI0, SGI345087, SGI0, SGI0, SGI0, SGI0, SGI345090, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81564[] = {SGI345091, SGI345092, SGI345094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345096, SGI0, NULL};
const Segment * const SG81565[] = {SGI345093, SGI0, SGI0, SGI345095, SGI0, SGI0, SGI0, SGI0, SGI345097, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81566[] = {SGI345098, SGI345099, SGI345100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345101, SGI0, NULL};
const Segment * const SG81567[] = {SGI345102, SGI345103, SGI345104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345105, SGI0, NULL};
const Segment * const SG81568[] = {SGI345106, SGI345107, SGI345108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345109, SGI0, NULL};
const Segment * const SG81569[] = {SGI345110, SGI345111, SGI345112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345113, SGI0, NULL};
const Segment * const SG81570[] = {SGI345114, SGI345115, SGI345116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345117, SGI0, NULL};
const Segment * const SG81571[] = {SGI345118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81572[] = {SGI345119, SGI345120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81573[] = {SGI345121, SGI345122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81574[] = {SGI345123, SGI345124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81575[] = {SGI345125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345126, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81576[] = {SGI0, SGI345127, SGI345128, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81577[] = {SGI0, SGI345129, SGI345130, SGI0, SGI0, SGI0, SGI345131, NULL};
const Segment * const SG81578[] = {SGI345132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345133, SGI0, NULL};
const Segment * const SG81579[] = {SGI345134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345135, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81580[] = {SGI345136, SGI345137, SGI0, SGI0, NULL};
const Segment * const SG81581[] = {SGI345138, SGI0, SGI0, SGI345139, SGI0, SGI0, SGI0, SGI0, SGI345140, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81582[] = {SGI345141, SGI0, SGI0, SGI345142, SGI0, SGI0, SGI0, SGI0, SGI345143, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81583[] = {SGI345144, SGI0, SGI0, SGI345145, SGI0, SGI0, SGI0, SGI0, SGI345146, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81584[] = {SGI345147, SGI0, SGI0, SGI345148, SGI0, SGI345149, SGI0, SGI0, SGI345150, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81585[] = {SGI345151, SGI345152, SGI345153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345153, SGI0, NULL};
const Segment * const SG81586[] = {SGI345154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81587[] = {SGI345155, SGI345156, SGI345157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345157, SGI0, NULL};
const Segment * const SG81588[] = {SGI345158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81589[] = {SGI345159, SGI345160, SGI345161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345161, SGI0, NULL};
const Segment * const SG81590[] = {SGI345162, SGI0, SGI345163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345163, SGI0, NULL};
const Segment * const SG81591[] = {SGI345164, SGI345165, SGI345166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345167, SGI0, NULL};
const Segment * const SG81592[] = {SGI345168, SGI345169, SGI345170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345171, SGI0, NULL};
const Segment * const SG81593[] = {SGI345172, SGI345173, SGI345174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345175, SGI0, NULL};
const Segment * const SG81594[] = {SGI345176, SGI345177, SGI345178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345179, SGI0, NULL};
const Segment * const SG81595[] = {SGI345180, SGI345181, SGI345182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345183, SGI0, NULL};
const Segment * const SG81596[] = {SGI345184, SGI345185, SGI345186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345186, SGI0, NULL};
const Segment * const SG81597[] = {SGI345187, SGI345188, SGI345189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345190, SGI0, NULL};
const Segment * const SG81598[] = {SGI345191, SGI345192, SGI345193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345194, SGI0, NULL};
const Segment * const SG81599[] = {SGI345195, SGI345196, SGI345197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345198, SGI0, NULL};
const Segment * const SG81600[] = {SGI345199, SGI345200, SGI345201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345202, SGI0, NULL};
const Segment * const SG81601[] = {SGI345203, SGI345204, SGI345205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345206, SGI0, NULL};
const Segment * const SG81602[] = {SGI345207, SGI345208, SGI345209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345210, SGI0, NULL};
const Segment * const SG81603[] = {SGI345211, SGI345212, SGI345213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345214, SGI0, NULL};
const Segment * const SG81604[] = {SGI345215, SGI345216, SGI345217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345218, SGI0, NULL};
const Segment * const SG81605[] = {SGI345219, SGI345220, SGI345221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345222, SGI0, NULL};
const Segment * const SG81606[] = {SGI345223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81607[] = {SGI345224, SGI345225, SGI345226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345227, SGI0, NULL};
const Segment * const SG81608[] = {SGI345228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81609[] = {SGI345229, SGI345230, SGI345231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345232, SGI0, NULL};
const Segment * const SG81610[] = {SGI345233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81611[] = {SGI345234, SGI345235, SGI345236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345237, SGI0, NULL};
const Segment * const SG81612[] = {SGI345238, SGI345239, SGI345240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345241, SGI0, NULL};
const Segment * const SG81613[] = {SGI345242, SGI345243, SGI345244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345245, SGI0, NULL};
const Segment * const SG81614[] = {SGI345246, SGI345247, SGI345248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345249, SGI0, NULL};
const Segment * const SG81615[] = {SGI345250, SGI345251, SGI345252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345253, SGI0, NULL};
const Segment * const SG81616[] = {SGI345254, SGI345255, SGI345256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345257, SGI0, NULL};
const Segment * const SG81617[] = {SGI345258, SGI345259, SGI345260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345261, SGI0, NULL};
const Segment * const SG81618[] = {SGI345262, SGI345262, SGI345263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345263, SGI0, NULL};
const Segment * const SG81619[] = {SGI345264, SGI345265, SGI345266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345266, SGI0, NULL};
const Segment * const SG81620[] = {SGI345267, SGI345268, SGI345269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345270, SGI0, NULL};
const Segment * const SG81621[] = {SGI345271, SGI345272, SGI345273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345274, SGI0, NULL};
const Segment * const SG81622[] = {SGI345275, SGI345276, SGI345277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345278, SGI0, NULL};
const Segment * const SG81623[] = {SGI345279, SGI345280, SGI345281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345282, SGI0, NULL};
const Segment * const SG81624[] = {SGI345283, SGI345284, SGI345285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345286, SGI0, NULL};
const Segment * const SG81625[] = {SGI345287, SGI345288, SGI345289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345290, SGI0, NULL};
const Segment * const SG81626[] = {SGI345291, SGI345292, SGI345293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345294, SGI0, NULL};
const Segment * const SG81627[] = {SGI345295, SGI345296, SGI345297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345298, SGI0, NULL};
const Segment * const SG81628[] = {SGI345299, SGI345300, SGI345301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345302, SGI0, NULL};
const Segment * const SG81629[] = {SGI345303, SGI0, SGI345304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345304, SGI0, NULL};
const Segment * const SG81630[] = {SGI345305, SGI345306, SGI345307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345308, SGI0, NULL};
const Segment * const SG81631[] = {SGI345309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81632[] = {SGI345310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81633[] = {SGI345311, SGI0, SGI0, SGI345312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81634[] = {SGI345313, SGI0, SGI0, SGI345314, SGI0, SGI0, SGI0, SGI0, SGI345315, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81635[] = {SGI345316, SGI0, SGI0, SGI345317, SGI0, SGI0, SGI0, SGI0, SGI345318, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81636[] = {SGI345319, SGI0, SGI0, SGI345320, SGI0, SGI0, SGI0, SGI0, SGI345321, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81637[] = {SGI345322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345323, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81638[] = {SGI345324, SGI0, SGI0, SGI345325, SGI0, SGI0, SGI0, SGI0, SGI345326, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81639[] = {SGI345327, SGI0, SGI0, SGI345328, SGI0, SGI0, SGI0, SGI0, SGI345329, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81640[] = {SGI345330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345331, SGI0, NULL};
const Segment * const SG81641[] = {SGI345332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345333, SGI0, NULL};
const Segment * const SG81642[] = {SGI345334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345335, SGI0, NULL};
const Segment * const SG81643[] = {SGI345336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345337, SGI0, NULL};
const Segment * const SG81644[] = {SGI345338, SGI0, SGI0, SGI345339, SGI0, SGI0, SGI0, SGI0, SGI345340, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81645[] = {SGI345341, SGI0, SGI345342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81646[] = {SGI345343, SGI0, SGI345344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81647[] = {SGI345345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345346, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81648[] = {SGI345347, SGI0, SGI0, SGI345348, SGI0, SGI0, SGI0, SGI0, SGI345349, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81649[] = {SGI345350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345351, SGI0, NULL};
const Segment * const SG81650[] = {SGI345352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345353, SGI0, NULL};
const Segment * const SG81651[] = {SGI345354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345355, SGI0, NULL};
const Segment * const SG81652[] = {SGI345356, SGI0, SGI0, SGI345357, SGI0, SGI0, SGI0, SGI0, SGI345358, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81653[] = {SGI345359, SGI0, SGI0, SGI345360, SGI0, SGI0, SGI0, SGI0, SGI345361, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81654[] = {SGI345362, SGI0, SGI0, SGI345363, SGI0, SGI0, SGI0, SGI0, SGI345364, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81655[] = {SGI345365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345366, SGI0, NULL};
const Segment * const SG81656[] = {SGI345367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345368, SGI0, NULL};
const Segment * const SG81657[] = {SGI345369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345370, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81658[] = {SGI345371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345372, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81659[] = {SGI345373, SGI0, SGI0, SGI345374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81660[] = {SGI345375, SGI0, SGI0, SGI345376, SGI0, SGI0, SGI0, SGI0, SGI345377, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81661[] = {SGI345378, SGI0, SGI0, SGI345379, SGI0, SGI0, SGI0, SGI0, SGI345380, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81662[] = {SGI345381, SGI0, SGI0, SGI345382, SGI0, SGI0, SGI0, SGI0, SGI345383, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81663[] = {SGI0, SGI345384, SGI345385, SGI0, SGI0, SGI0, SGI345386, NULL};
const Segment * const SG81664[] = {SGI345387, SGI0, SGI0, SGI345388, SGI0, SGI0, SGI0, SGI0, SGI345389, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81665[] = {SGI345390, SGI0, SGI345391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81666[] = {SGI345392, SGI0, SGI0, SGI345393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81667[] = {SGI345394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345395, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81668[] = {SGI345396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345397, SGI0, NULL};
const Segment * const SG81669[] = {SGI345398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345399, SGI0, NULL};
const Segment * const SG81670[] = {SGI345400, SGI345401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81671[] = {SGI345402, SGI345403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81672[] = {SGI345404, SGI345405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81673[] = {SGI345406, SGI345407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81674[] = {SGI345408, SGI345409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81675[] = {SGI345410, SGI345411, SGI0, SGI0, NULL};
const Segment * const SG81676[] = {SGI345412, SGI345413, SGI0, SGI0, NULL};
const Segment * const SG81677[] = {SGI345414, SGI345415, SGI0, SGI0, NULL};
const Segment * const SG81678[] = {SGI345416, SGI345417, SGI0, SGI0, NULL};
const Segment * const SG81679[] = {SGI345418, SGI345419, SGI0, SGI0, NULL};
const Segment * const SG81680[] = {SGI345420, SGI345421, SGI0, SGI0, NULL};
const Segment * const SG81681[] = {SGI345422, SGI345423, SGI0, SGI0, NULL};
const Segment * const SG81682[] = {SGI345424, SGI345425, SGI0, SGI0, NULL};
const Segment * const SG81683[] = {SGI345426, SGI345427, SGI0, SGI0, NULL};
const Segment * const SG81684[] = {SGI345428, SGI345429, SGI0, SGI0, NULL};
const Segment * const SG81685[] = {SGI345430, SGI345431, SGI0, SGI0, NULL};
const Segment * const SG81686[] = {SGI345432, SGI345433, SGI0, SGI0, NULL};
const Segment * const SG81687[] = {SGI345434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345435, SGI0, NULL};
const Segment * const SG81688[] = {SGI345436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345437, SGI0, NULL};
const Segment * const SG81689[] = {SGI345438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345439, SGI0, NULL};
const Segment * const SG81690[] = {SGI345440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345441, SGI0, NULL};
const Segment * const SG81691[] = {SGI345442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345443, SGI0, NULL};
const Segment * const SG81692[] = {SGI345444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345445, SGI0, NULL};
const Segment * const SG81693[] = {SGI345446, SGI345447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81694[] = {SGI345448, SGI345449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81695[] = {SGI345450, SGI345451, SGI345452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345453, SGI0, NULL};
const Segment * const SG81696[] = {SGI345454, SGI345455, SGI345456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345456, SGI0, NULL};
const Segment * const SG81697[] = {SGI345457, SGI345458, SGI345459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345460, SGI0, NULL};
const Segment * const SG81698[] = {SGI345461, SGI345462, SGI345463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345464, SGI0, NULL};
const Segment * const SG81699[] = {SGI345465, SGI0, SGI0, SGI345466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81700[] = {SGI345467, SGI345468, SGI345469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345470, SGI0, NULL};
const Segment * const SG81701[] = {SGI345471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345472, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81702[] = {SGI345473, SGI345474, SGI345475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345476, SGI0, NULL};
const Segment * const SG81703[] = {SGI345477, SGI345478, SGI345479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345480, SGI0, NULL};
const Segment * const SG81704[] = {SGI345481, SGI345482, SGI345483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345484, SGI0, NULL};
const Segment * const SG81705[] = {SGI345485, SGI345486, SGI345487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345488, SGI0, NULL};
const Segment * const SG81706[] = {SGI345489, SGI345490, SGI345491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345492, SGI0, NULL};
const Segment * const SG81707[] = {SGI345493, SGI345494, SGI345495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345496, SGI0, NULL};
const Segment * const SG81708[] = {SGI345497, SGI345498, SGI345499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345500, SGI0, NULL};
const Segment * const SG81709[] = {SGI345501, SGI345502, SGI345503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345504, SGI0, NULL};
const Segment * const SG81710[] = {SGI345505, SGI345506, SGI345507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345508, SGI0, NULL};
const Segment * const SG81711[] = {SGI345509, SGI345510, SGI345511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345512, SGI0, NULL};
const Segment * const SG81712[] = {SGI345513, SGI345514, SGI345515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345516, SGI0, NULL};
const Segment * const SG81713[] = {SGI345517, SGI345518, SGI345519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345520, SGI0, NULL};
const Segment * const SG81714[] = {SGI345521, SGI345522, SGI345523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345524, SGI0, NULL};
const Segment * const SG81715[] = {SGI345525, SGI345526, SGI345527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345528, SGI0, NULL};
const Segment * const SG81716[] = {SGI345529, SGI345530, SGI345531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345532, SGI0, NULL};
const Segment * const SG81717[] = {SGI345533, SGI345534, SGI345535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345536, SGI0, NULL};
const Segment * const SG81718[] = {SGI345537, SGI345538, SGI345539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345540, SGI0, NULL};
const Segment * const SG81719[] = {SGI345541, SGI345542, SGI345543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345544, SGI0, NULL};
const Segment * const SG81720[] = {SGI345545, SGI345546, SGI345547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345548, SGI0, NULL};
const Segment * const SG81721[] = {SGI345549, SGI345550, SGI345551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345552, SGI0, NULL};
const Segment * const SG81722[] = {SGI345553, SGI345554, SGI345555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345556, SGI0, NULL};
const Segment * const SG81723[] = {SGI345557, SGI345558, SGI345559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345560, SGI0, NULL};
const Segment * const SG81724[] = {SGI345561, SGI345562, SGI345563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345564, SGI0, NULL};
const Segment * const SG81725[] = {SGI345565, SGI345566, SGI345567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345568, SGI0, NULL};
const Segment * const SG81726[] = {SGI345569, SGI345570, SGI345571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345572, SGI0, NULL};
const Segment * const SG81727[] = {SGI345573, SGI345574, SGI345575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345576, SGI0, NULL};
const Segment * const SG81728[] = {SGI345577, SGI345578, SGI345579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345580, SGI0, NULL};
const Segment * const SG81729[] = {SGI345581, SGI345582, SGI345583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345584, SGI0, NULL};
const Segment * const SG81730[] = {SGI345585, SGI345586, SGI345587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345588, SGI0, NULL};
const Segment * const SG81731[] = {SGI345589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345590, SGI0, NULL};
const Segment * const SG81732[] = {SGI345591, SGI345592, SGI345593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345594, SGI0, NULL};
const Segment * const SG81733[] = {SGI345595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345598, SGI0, NULL};
const Segment * const SG81734[] = {SGI345596, SGI345597, SGI345599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345600, SGI0, NULL};
const Segment * const SG81735[] = {SGI345601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345602, SGI0, NULL};
const Segment * const SG81736[] = {SGI345603, SGI345604, SGI345605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345606, SGI0, NULL};
const Segment * const SG81737[] = {SGI345607, SGI345608, SGI345609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345610, SGI0, NULL};
const Segment * const SG81738[] = {SGI345611, SGI345612, SGI345613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345614, SGI0, NULL};
const Segment * const SG81739[] = {SGI345615, SGI345616, SGI345617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345617, SGI0, NULL};
const Segment * const SG81740[] = {SGI345618, SGI345619, SGI345620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345621, SGI0, NULL};
const Segment * const SG81741[] = {SGI345622, SGI345623, SGI345624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345625, SGI0, NULL};
const Segment * const SG81742[] = {SGI345626, SGI345627, SGI345628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345629, SGI0, NULL};
const Segment * const SG81743[] = {SGI345630, SGI345631, SGI345632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345633, SGI0, NULL};
const Segment * const SG81744[] = {SGI345634, SGI345635, SGI345636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345636, SGI0, NULL};
const Segment * const SG81745[] = {SGI345637, SGI345638, SGI345639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345640, SGI0, NULL};
const Segment * const SG81746[] = {SGI345641, SGI345642, SGI345643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345644, SGI0, NULL};
const Segment * const SG81747[] = {SGI345645, SGI345646, SGI345647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345648, SGI0, NULL};
const Segment * const SG81748[] = {SGI345649, SGI345650, SGI345651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345652, SGI0, NULL};
const Segment * const SG81749[] = {SGI345653, SGI345654, SGI345655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345656, SGI0, NULL};
const Segment * const SG81750[] = {SGI345657, SGI345658, SGI345659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345660, SGI0, NULL};
const Segment * const SG81751[] = {SGI345661, SGI345662, SGI345663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345664, SGI0, NULL};
const Segment * const SG81752[] = {SGI345665, SGI345666, SGI345667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345668, SGI0, NULL};
const Segment * const SG81753[] = {SGI345669, SGI345670, SGI345671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345672, SGI0, NULL};
const Segment * const SG81754[] = {SGI345673, SGI345674, SGI345675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345676, SGI0, NULL};
const Segment * const SG81755[] = {SGI345677, SGI345678, SGI345679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345680, SGI0, NULL};
const Segment * const SG81756[] = {SGI345681, SGI345682, SGI345683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345684, SGI0, NULL};
const Segment * const SG81757[] = {SGI345685, SGI345686, SGI345687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345688, SGI0, NULL};
const Segment * const SG81758[] = {SGI345689, SGI345690, SGI345691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345692, SGI0, NULL};
const Segment * const SG81759[] = {SGI345693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345694, SGI0, NULL};
const Segment * const SG81760[] = {SGI345695, SGI345696, SGI345697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345698, SGI0, NULL};
const Segment * const SG81761[] = {SGI345699, SGI345700, SGI345701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345702, SGI0, NULL};
const Segment * const SG81762[] = {SGI345703, SGI345704, SGI345705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345706, SGI0, NULL};
const Segment * const SG81763[] = {SGI345707, SGI345708, SGI345709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345710, SGI0, NULL};
const Segment * const SG81764[] = {SGI345711, SGI345712, SGI345713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345714, SGI0, NULL};
const Segment * const SG81765[] = {SGI345715, SGI345716, SGI345717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345718, SGI0, NULL};
const Segment * const SG81766[] = {SGI345719, SGI345720, SGI345721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345722, SGI0, NULL};
const Segment * const SG81767[] = {SGI345723, SGI345724, SGI345725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345726, SGI0, NULL};
const Segment * const SG81768[] = {SGI345727, SGI345728, SGI345729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345730, SGI0, NULL};
const Segment * const SG81769[] = {SGI345731, SGI345732, SGI345733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345734, SGI0, NULL};
const Segment * const SG81770[] = {SGI345735, SGI345736, SGI345737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345738, SGI0, NULL};
const Segment * const SG81771[] = {SGI345739, SGI345740, SGI345741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345742, SGI0, NULL};
const Segment * const SG81772[] = {SGI345743, SGI345744, SGI345745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345746, SGI0, NULL};
const Segment * const SG81773[] = {SGI345747, SGI345748, SGI345749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345750, SGI0, NULL};
const Segment * const SG81774[] = {SGI345751, SGI345752, SGI345753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345754, SGI0, NULL};
const Segment * const SG81775[] = {SGI345755, SGI345756, SGI345757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345758, SGI0, NULL};
const Segment * const SG81776[] = {SGI345759, SGI345760, SGI345761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345762, SGI0, NULL};
const Segment * const SG81777[] = {SGI345763, SGI345764, SGI345765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345766, SGI0, NULL};
const Segment * const SG81778[] = {SGI345767, SGI345768, SGI345769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345770, SGI0, NULL};
const Segment * const SG81779[] = {SGI345771, SGI345772, SGI345773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345774, SGI0, NULL};
const Segment * const SG81780[] = {SGI345775, SGI345776, SGI345777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345778, SGI0, NULL};
const Segment * const SG81781[] = {SGI345779, SGI345780, SGI345781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345782, SGI0, NULL};
const Segment * const SG81782[] = {SGI345783, SGI345784, SGI345785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345786, SGI0, NULL};
const Segment * const SG81783[] = {SGI345787, SGI345788, SGI345789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345790, SGI0, NULL};
const Segment * const SG81784[] = {SGI345791, SGI345792, SGI345793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345794, SGI0, NULL};
const Segment * const SG81785[] = {SGI345795, SGI345796, SGI345797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345798, SGI0, NULL};
const Segment * const SG81786[] = {SGI345799, SGI345800, SGI345801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345802, SGI0, NULL};
const Segment * const SG81787[] = {SGI345803, SGI345804, SGI345805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345806, SGI0, NULL};
const Segment * const SG81788[] = {SGI345807, SGI345808, SGI345809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345809, SGI0, NULL};
const Segment * const SG81789[] = {SGI345810, SGI345811, SGI345812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345813, SGI0, NULL};
const Segment * const SG81790[] = {SGI345814, SGI345815, SGI345816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345817, SGI0, NULL};
const Segment * const SG81791[] = {SGI345818, SGI345819, SGI345820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345821, SGI0, NULL};
const Segment * const SG81792[] = {SGI345822, SGI345823, SGI345824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345825, SGI0, NULL};
const Segment * const SG81793[] = {SGI345826, SGI345827, SGI345828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345829, SGI0, NULL};
const Segment * const SG81794[] = {SGI345830, SGI345831, SGI345832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345833, SGI0, NULL};
const Segment * const SG81795[] = {SGI345834, SGI345835, SGI345836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345837, SGI0, NULL};
const Segment * const SG81796[] = {SGI345838, SGI345839, SGI345840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345841, SGI0, NULL};
const Segment * const SG81797[] = {SGI345842, SGI345843, SGI345844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345845, SGI0, NULL};
const Segment * const SG81798[] = {SGI345846, SGI345847, SGI345848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345849, SGI0, NULL};
const Segment * const SG81799[] = {SGI345850, SGI345851, SGI345852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345853, SGI0, NULL};
const Segment * const SG81800[] = {SGI345854, SGI345855, SGI345856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345857, SGI0, NULL};
const Segment * const SG81801[] = {SGI345858, SGI345859, SGI345860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345860, SGI0, NULL};
const Segment * const SG81802[] = {SGI345861, SGI345862, SGI345863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345864, SGI0, NULL};
const Segment * const SG81803[] = {SGI345865, SGI345866, SGI345867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345868, SGI0, NULL};
const Segment * const SG81804[] = {SGI345869, SGI345870, SGI345871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345872, SGI0, NULL};
const Segment * const SG81805[] = {SGI345873, SGI345874, SGI345875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345876, SGI0, NULL};
const Segment * const SG81806[] = {SGI345877, SGI345878, SGI345879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345880, SGI0, NULL};
const Segment * const SG81807[] = {SGI345881, SGI345882, SGI345883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345884, SGI0, NULL};
const Segment * const SG81808[] = {SGI345885, SGI345886, SGI345887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345888, SGI0, NULL};
const Segment * const SG81809[] = {SGI345889, SGI345890, SGI345891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345892, SGI0, NULL};
const Segment * const SG81810[] = {SGI345893, SGI345894, SGI345895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345896, SGI0, NULL};
const Segment * const SG81811[] = {SGI345897, SGI345898, SGI345899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345900, SGI0, NULL};
const Segment * const SG81812[] = {SGI345901, SGI345902, SGI345903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345904, SGI0, NULL};
const Segment * const SG81813[] = {SGI345905, SGI345906, SGI345907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345908, SGI0, NULL};
const Segment * const SG81814[] = {SGI345909, SGI345910, SGI345911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345912, SGI0, NULL};
const Segment * const SG81815[] = {SGI345913, SGI345914, SGI345915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345915, SGI0, NULL};
const Segment * const SG81816[] = {SGI345916, SGI345917, SGI345918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345919, SGI0, NULL};
const Segment * const SG81817[] = {SGI345920, SGI345921, SGI345922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345923, SGI0, NULL};
const Segment * const SG81818[] = {SGI345924, SGI345925, SGI345926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345927, SGI0, NULL};
const Segment * const SG81819[] = {SGI345928, SGI345929, SGI345930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345931, SGI0, NULL};
const Segment * const SG81820[] = {SGI345932, SGI345933, SGI345934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345935, SGI0, NULL};
const Segment * const SG81821[] = {SGI345936, SGI345937, SGI345938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345939, SGI0, NULL};
const Segment * const SG81822[] = {SGI345940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345941, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81823[] = {SGI345942, SGI345943, SGI345944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345944, SGI0, NULL};
const Segment * const SG81824[] = {SGI345945, SGI345946, SGI345947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345947, SGI0, NULL};
const Segment * const SG81825[] = {SGI345948, SGI345949, SGI345950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345951, SGI0, NULL};
const Segment * const SG81826[] = {SGI345952, SGI345953, SGI345954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345955, SGI0, NULL};
const Segment * const SG81827[] = {SGI345956, SGI345957, SGI345958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345959, SGI0, NULL};
const Segment * const SG81828[] = {SGI345960, SGI0, SGI0, SGI345962, SGI0, SGI0, SGI0, SGI0, SGI345965, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81829[] = {SGI345961, SGI345963, SGI345964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345966, SGI0, NULL};
const Segment * const SG81830[] = {SGI345967, SGI0, SGI0, SGI345968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81831[] = {SGI345969, SGI345970, SGI345971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345971, SGI0, NULL};
const Segment * const SG81832[] = {SGI345972, SGI345973, SGI345974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345974, SGI0, NULL};
const Segment * const SG81833[] = {SGI345975, SGI345976, SGI345977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345978, SGI0, NULL};
const Segment * const SG81834[] = {SGI345979, SGI345980, SGI345981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345982, SGI0, NULL};
const Segment * const SG81835[] = {SGI345983, SGI345984, SGI345985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345986, SGI0, NULL};
const Segment * const SG81836[] = {SGI345987, SGI345988, SGI345989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345990, SGI0, NULL};
const Segment * const SG81837[] = {SGI345991, SGI345992, SGI345993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345994, SGI0, NULL};
const Segment * const SG81838[] = {SGI345995, SGI345996, SGI345997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI345998, SGI0, NULL};
const Segment * const SG81839[] = {SGI345999, SGI346000, SGI346001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346002, SGI0, NULL};
const Segment * const SG81840[] = {SGI346003, SGI346004, SGI346005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346006, SGI0, NULL};
const Segment * const SG81841[] = {SGI346007, SGI346008, SGI346009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346010, SGI0, NULL};
const Segment * const SG81842[] = {SGI346011, SGI346012, SGI346013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346014, SGI0, NULL};
const Segment * const SG81843[] = {SGI346015, SGI346016, SGI346017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346018, SGI0, NULL};
const Segment * const SG81844[] = {SGI346019, SGI346020, SGI346021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346022, SGI0, NULL};
const Segment * const SG81845[] = {SGI346023, SGI346024, SGI346025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346026, SGI0, NULL};
const Segment * const SG81846[] = {SGI346027, SGI346028, SGI346029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346030, SGI0, NULL};
const Segment * const SG81847[] = {SGI346031, SGI346032, SGI346033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346034, SGI0, NULL};
const Segment * const SG81848[] = {SGI346035, SGI346036, SGI346037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346038, SGI0, NULL};
const Segment * const SG81849[] = {SGI346039, SGI346040, SGI346041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346042, SGI0, NULL};
const Segment * const SG81850[] = {SGI346043, SGI346044, SGI346045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346046, SGI0, NULL};
const Segment * const SG81851[] = {SGI346047, SGI346048, SGI346049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346050, SGI0, NULL};
const Segment * const SG81852[] = {SGI346051, SGI346052, SGI346053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346054, SGI0, NULL};
const Segment * const SG81853[] = {SGI346055, SGI346056, SGI346057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346058, SGI0, NULL};
const Segment * const SG81854[] = {SGI346059, SGI346060, SGI346061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346062, SGI0, NULL};
const Segment * const SG81855[] = {SGI346063, SGI346064, SGI346065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346066, SGI0, NULL};
const Segment * const SG81856[] = {SGI346067, SGI346068, SGI346069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346070, SGI0, NULL};
const Segment * const SG81857[] = {SGI346071, SGI346072, SGI346073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346074, SGI0, NULL};
const Segment * const SG81858[] = {SGI346075, SGI346076, SGI346077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346078, SGI0, NULL};
const Segment * const SG81859[] = {SGI346079, SGI346080, SGI346081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346082, SGI0, NULL};
const Segment * const SG81860[] = {SGI346083, SGI346084, SGI346085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346086, SGI0, NULL};
const Segment * const SG81861[] = {SGI346087, SGI346088, SGI346089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346090, SGI0, NULL};
const Segment * const SG81862[] = {SGI346091, SGI346092, SGI346093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346094, SGI0, NULL};
const Segment * const SG81863[] = {SGI346095, SGI346096, SGI346097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346098, SGI0, NULL};
const Segment * const SG81864[] = {SGI346099, SGI346100, SGI346101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346102, SGI0, NULL};
const Segment * const SG81865[] = {SGI346103, SGI346104, SGI346105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346106, SGI0, NULL};
const Segment * const SG81866[] = {SGI346107, SGI346108, SGI346109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346110, SGI0, NULL};
const Segment * const SG81867[] = {SGI346111, SGI346112, SGI346113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346114, SGI0, NULL};
const Segment * const SG81868[] = {SGI346115, SGI346116, SGI346117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346118, SGI0, NULL};
const Segment * const SG81869[] = {SGI346119, SGI346120, SGI346121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346122, SGI0, NULL};
const Segment * const SG81870[] = {SGI346123, SGI346124, SGI346125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346126, SGI0, NULL};
const Segment * const SG81871[] = {SGI346127, SGI346128, SGI346129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346130, SGI0, NULL};
const Segment * const SG81872[] = {SGI346131, SGI346132, SGI346133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346134, SGI0, NULL};
const Segment * const SG81873[] = {SGI346135, SGI346136, SGI346137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346138, SGI0, NULL};
const Segment * const SG81874[] = {SGI346139, SGI346140, SGI346141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346142, SGI0, NULL};
const Segment * const SG81875[] = {SGI346143, SGI346144, SGI346145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346146, SGI0, NULL};
const Segment * const SG81876[] = {SGI346147, SGI346148, SGI346149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346150, SGI0, NULL};
const Segment * const SG81877[] = {SGI346151, SGI346152, SGI346153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346154, SGI0, NULL};
const Segment * const SG81878[] = {SGI346155, SGI346156, SGI346157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346158, SGI0, NULL};
const Segment * const SG81879[] = {SGI346159, SGI346160, SGI346161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346162, SGI0, NULL};
const Segment * const SG81880[] = {SGI346163, SGI346164, SGI346165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346166, SGI0, NULL};
const Segment * const SG81881[] = {SGI346167, SGI346168, SGI346169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346170, SGI0, NULL};
const Segment * const SG81882[] = {SGI346171, SGI346172, SGI346173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346174, SGI0, NULL};
const Segment * const SG81883[] = {SGI346175, SGI346176, SGI346177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346178, SGI0, NULL};
const Segment * const SG81884[] = {SGI346179, SGI346180, SGI346181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346182, SGI0, NULL};
const Segment * const SG81885[] = {SGI346183, SGI346184, SGI346185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346186, SGI0, NULL};
const Segment * const SG81886[] = {SGI346187, SGI346188, SGI346189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346190, SGI0, NULL};
const Segment * const SG81887[] = {SGI346191, SGI346192, SGI346193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346193, SGI0, NULL};
const Segment * const SG81888[] = {SGI346194, SGI346195, SGI346196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346196, SGI0, NULL};
const Segment * const SG81889[] = {SGI346197, SGI346198, SGI346199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346199, SGI0, NULL};
const Segment * const SG81890[] = {SGI346200, SGI346201, SGI346202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346202, SGI0, NULL};
const Segment * const SG81891[] = {SGI346203, SGI346204, SGI346205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346205, SGI0, NULL};
const Segment * const SG81892[] = {SGI346206, SGI0, SGI0, SGI346207, SGI0, SGI0, SGI0, SGI0, SGI346208, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81893[] = {SGI346209, SGI346210, SGI346211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346212, SGI0, NULL};
const Segment * const SG81894[] = {SGI346213, SGI346214, SGI346215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346216, SGI0, NULL};
const Segment * const SG81895[] = {SGI346217, SGI346218, SGI346219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346220, SGI0, NULL};
const Segment * const SG81896[] = {SGI346221, SGI346222, SGI346223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346224, SGI0, NULL};
const Segment * const SG81897[] = {SGI346225, SGI346226, SGI346227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346228, SGI0, NULL};
const Segment * const SG81898[] = {SGI346229, SGI346230, SGI346231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346231, SGI0, NULL};
const Segment * const SG81899[] = {SGI346232, SGI346233, SGI346234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346235, SGI0, NULL};
const Segment * const SG81900[] = {SGI346236, SGI346237, SGI346238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346239, SGI0, NULL};
const Segment * const SG81901[] = {SGI346240, SGI346241, SGI346242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346242, SGI0, NULL};
const Segment * const SG81902[] = {SGI346243, SGI346244, SGI346245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346246, SGI0, NULL};
const Segment * const SG81903[] = {SGI346247, SGI346248, SGI346249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346249, SGI0, NULL};
const Segment * const SG81904[] = {SGI346250, SGI346251, SGI346252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346253, SGI0, NULL};
const Segment * const SG81905[] = {SGI346254, SGI346255, SGI346256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346257, SGI0, NULL};
const Segment * const SG81906[] = {SGI346258, SGI346259, SGI346260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346261, SGI0, NULL};
const Segment * const SG81907[] = {SGI346262, SGI346263, SGI346264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346265, SGI0, NULL};
const Segment * const SG81908[] = {SGI346266, SGI346267, SGI346268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346269, SGI0, NULL};
const Segment * const SG81909[] = {SGI346270, SGI346271, SGI346272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346273, SGI0, NULL};
const Segment * const SG81910[] = {SGI346274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346276, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81911[] = {SGI346275, SGI346277, SGI346278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346279, SGI0, NULL};
const Segment * const SG81912[] = {SGI346280, SGI346281, SGI346282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346283, SGI0, NULL};
const Segment * const SG81913[] = {SGI346284, SGI346285, SGI346286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346287, SGI0, NULL};
const Segment * const SG81914[] = {SGI346288, SGI346289, SGI346290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346291, SGI0, NULL};
const Segment * const SG81915[] = {SGI346292, SGI346293, SGI346294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346295, SGI0, NULL};
const Segment * const SG81916[] = {SGI346296, SGI346297, SGI346298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346299, SGI0, NULL};
const Segment * const SG81917[] = {SGI346300, SGI346301, SGI346302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346303, SGI0, NULL};
const Segment * const SG81918[] = {SGI346304, SGI346305, SGI346306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346307, SGI0, NULL};
const Segment * const SG81919[] = {SGI346308, SGI346309, SGI346310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346311, SGI0, NULL};
const Segment * const SG81920[] = {SGI346312, SGI346313, SGI346314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346315, SGI0, NULL};
const Segment * const SG81921[] = {SGI0, SGI346316, SGI346317, SGI0, SGI0, SGI0, SGI346318, NULL};
const Segment * const SG81922[] = {SGI346319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346320, SGI0, NULL};
const Segment * const SG81923[] = {SGI346321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346322, SGI0, NULL};
const Segment * const SG81924[] = {SGI346323, SGI346324, SGI346325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81925[] = {SGI346326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81926[] = {SGI346327, SGI346328, SGI346329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81927[] = {SGI346330, SGI346331, SGI0, SGI0, NULL};
const Segment * const SG81928[] = {SGI346332, SGI0, SGI0, SGI346333, SGI0, SGI0, SGI0, SGI0, SGI346334, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81929[] = {SGI346335, SGI346336, SGI346337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346337, SGI0, NULL};
const Segment * const SG81930[] = {SGI346338, SGI346339, SGI346340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346341, SGI0, NULL};
const Segment * const SG81931[] = {SGI346342, SGI346343, SGI346344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346345, SGI0, NULL};
const Segment * const SG81932[] = {SGI346346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346348, SGI0, NULL};
const Segment * const SG81933[] = {SGI346347, SGI0, SGI0, SGI346349, SGI0, SGI0, SGI0, SGI0, SGI346350, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81934[] = {SGI346351, SGI346352, SGI346353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346354, SGI0, NULL};
const Segment * const SG81935[] = {SGI346355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346356, SGI0, NULL};
const Segment * const SG81936[] = {SGI346357, SGI0, SGI0, SGI346358, SGI0, SGI0, SGI0, SGI0, SGI346360, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81937[] = {SGI346359, SGI346361, SGI346362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346363, SGI0, NULL};
const Segment * const SG81938[] = {SGI346364, SGI0, SGI0, SGI346366, SGI0, SGI0, SGI0, SGI0, SGI346369, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81939[] = {SGI346365, SGI346367, SGI346368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346370, SGI0, NULL};
const Segment * const SG81940[] = {SGI346371, SGI0, SGI0, SGI346372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81941[] = {SGI346373, SGI346374, SGI346375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346376, SGI0, NULL};
const Segment * const SG81942[] = {SGI346377, SGI346378, SGI346379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346380, SGI0, NULL};
const Segment * const SG81943[] = {SGI346381, SGI346382, SGI346383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346384, SGI0, NULL};
const Segment * const SG81944[] = {SGI346385, SGI346386, SGI346387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346388, SGI0, NULL};
const Segment * const SG81945[] = {SGI346389, SGI346390, SGI346391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346392, SGI0, NULL};
const Segment * const SG81946[] = {SGI346393, SGI346394, SGI346395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346396, SGI0, NULL};
const Segment * const SG81947[] = {SGI346397, SGI346398, SGI346399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346400, SGI0, NULL};
const Segment * const SG81948[] = {SGI346401, SGI346402, SGI346403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346404, SGI0, NULL};
const Segment * const SG81949[] = {SGI346405, SGI346406, SGI346407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346408, SGI0, NULL};
const Segment * const SG81950[] = {SGI346409, SGI346410, SGI346411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346412, SGI0, NULL};
const Segment * const SG81951[] = {SGI346413, SGI346414, SGI346415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346416, SGI0, NULL};
const Segment * const SG81952[] = {SGI346417, SGI346418, SGI346419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346420, SGI0, NULL};
const Segment * const SG81953[] = {SGI346421, SGI346422, SGI346423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346424, SGI0, NULL};
const Segment * const SG81954[] = {SGI346425, SGI346426, SGI346427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346428, SGI0, NULL};
const Segment * const SG81955[] = {SGI346429, SGI346430, SGI346431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346432, SGI0, NULL};
const Segment * const SG81956[] = {SGI346433, SGI346434, SGI346435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346436, SGI0, NULL};
const Segment * const SG81957[] = {SGI346437, SGI346438, SGI346439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346440, SGI0, NULL};
const Segment * const SG81958[] = {SGI346441, SGI346442, SGI346443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346444, SGI0, NULL};
const Segment * const SG81959[] = {SGI346445, SGI346446, SGI346447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346448, SGI0, NULL};
const Segment * const SG81960[] = {SGI346449, SGI346450, SGI346451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346452, SGI0, NULL};
const Segment * const SG81961[] = {SGI346453, SGI346454, SGI346455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346456, SGI0, NULL};
const Segment * const SG81962[] = {SGI346457, SGI346458, SGI346459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346460, SGI0, NULL};
const Segment * const SG81963[] = {SGI346461, SGI346462, SGI346463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346464, SGI0, NULL};
const Segment * const SG81964[] = {SGI346465, SGI346466, SGI346467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346468, SGI0, NULL};
const Segment * const SG81965[] = {SGI346469, SGI346470, SGI346471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346472, SGI0, NULL};
const Segment * const SG81966[] = {SGI346473, SGI346474, SGI346475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346476, SGI0, NULL};
const Segment * const SG81967[] = {SGI346477, SGI346478, SGI346479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346480, SGI0, NULL};
const Segment * const SG81968[] = {SGI346481, SGI346482, SGI346483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346484, SGI0, NULL};
const Segment * const SG81969[] = {SGI346485, SGI346486, SGI346487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346488, SGI0, NULL};
const Segment * const SG81970[] = {SGI346489, SGI346490, SGI346491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346492, SGI0, NULL};
const Segment * const SG81971[] = {SGI346493, SGI346494, SGI346495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346497, SGI0, NULL};
const Segment * const SG81972[] = {SGI346496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346498, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81973[] = {SGI346499, SGI346500, SGI346501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346502, SGI0, NULL};
const Segment * const SG81974[] = {SGI346503, SGI346504, SGI346505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346506, SGI0, NULL};
const Segment * const SG81975[] = {SGI346507, SGI346508, SGI346509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346510, SGI0, NULL};
const Segment * const SG81976[] = {SGI346511, SGI346512, SGI346513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81977[] = {SGI346514, SGI346515, SGI346516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346517, SGI0, NULL};
const Segment * const SG81978[] = {SGI346518, SGI346519, SGI346520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346521, SGI0, NULL};
const Segment * const SG81979[] = {SGI346522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81980[] = {SGI346523, SGI346524, SGI346525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346526, SGI0, NULL};
const Segment * const SG81981[] = {SGI346527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81982[] = {SGI346528, SGI346529, SGI346530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346531, SGI0, NULL};
const Segment * const SG81983[] = {SGI346532, SGI346533, SGI346534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346535, SGI0, NULL};
const Segment * const SG81984[] = {SGI346536, SGI346537, SGI346538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346539, SGI0, NULL};
const Segment * const SG81985[] = {SGI346540, SGI346541, SGI346542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346543, SGI0, NULL};
const Segment * const SG81986[] = {SGI346544, SGI346545, SGI346546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346547, SGI0, NULL};
const Segment * const SG81987[] = {SGI346548, SGI346549, SGI346550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346551, SGI0, NULL};
const Segment * const SG81988[] = {SGI346552, SGI346553, SGI346554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346555, SGI0, NULL};
const Segment * const SG81989[] = {SGI346556, SGI346557, SGI346558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346559, SGI0, NULL};
const Segment * const SG81990[] = {SGI346560, SGI346561, SGI346562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346563, SGI0, NULL};
const Segment * const SG81991[] = {SGI346564, SGI346565, SGI346566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346567, SGI0, NULL};
const Segment * const SG81992[] = {SGI346568, SGI346569, SGI346570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346571, SGI0, NULL};
const Segment * const SG81993[] = {SGI346572, SGI346573, SGI346574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346575, SGI0, NULL};
const Segment * const SG81994[] = {SGI346576, SGI346577, SGI346578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346579, SGI0, NULL};
const Segment * const SG81995[] = {SGI346580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81996[] = {SGI346581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG81997[] = {SGI346582, SGI346583, SGI346584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346585, SGI0, NULL};
const Segment * const SG81998[] = {SGI346586, SGI346587, SGI346588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346589, SGI0, NULL};
const Segment * const SG81999[] = {SGI346590, SGI346591, SGI346592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346592, SGI0, NULL};
const Segment * const SG82000[] = {SGI346593, SGI346594, SGI346595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346596, SGI0, NULL};
const Segment * const SG82001[] = {SGI346598, SGI346599, SGI346601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346601, SGI0, NULL};
const Segment * const SG82002[] = {SGI346597, SGI0, SGI0, SGI346600, SGI0, SGI0, SGI0, SGI0, SGI346602, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82003[] = {SGI346603, SGI0, SGI0, SGI346605, SGI0, SGI0, SGI0, SGI0, SGI346608, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82004[] = {SGI346604, SGI346606, SGI346607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346609, SGI0, NULL};
const Segment * const SG82005[] = {SGI346610, SGI346611, SGI346612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346613, SGI0, NULL};
const Segment * const SG82006[] = {SGI346614, SGI346615, SGI346616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346617, SGI0, NULL};
const Segment * const SG82007[] = {SGI346618, SGI346619, SGI346620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346620, SGI0, NULL};
const Segment * const SG82008[] = {SGI346621, SGI346622, SGI346623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346624, SGI0, NULL};
const Segment * const SG82009[] = {SGI346625, SGI346626, SGI346627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346628, SGI0, NULL};
const Segment * const SG82010[] = {SGI346629, SGI346630, SGI346631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346632, SGI0, NULL};
const Segment * const SG82011[] = {SGI346633, SGI346634, SGI346635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346636, SGI0, NULL};
const Segment * const SG82012[] = {SGI346637, SGI346638, SGI346639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346640, SGI0, NULL};
const Segment * const SG82013[] = {SGI346641, SGI346642, SGI346643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346644, SGI0, NULL};
const Segment * const SG82014[] = {SGI346645, SGI346646, SGI346647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346648, SGI0, NULL};
const Segment * const SG82015[] = {SGI346649, SGI346650, SGI346651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346652, SGI0, NULL};
const Segment * const SG82016[] = {SGI346653, SGI346654, SGI346655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346656, SGI0, NULL};
const Segment * const SG82017[] = {SGI346657, SGI346658, SGI346659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346660, SGI0, NULL};
const Segment * const SG82018[] = {SGI346661, SGI346662, SGI346663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346664, SGI0, NULL};
const Segment * const SG82019[] = {SGI346665, SGI346666, SGI346667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346668, SGI0, NULL};
const Segment * const SG82020[] = {SGI346669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346670, SGI0, NULL};
const Segment * const SG82021[] = {SGI346671, SGI346672, SGI346673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346674, SGI0, NULL};
const Segment * const SG82022[] = {SGI346675, SGI346676, SGI346677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346678, SGI0, NULL};
const Segment * const SG82023[] = {SGI346679, SGI346680, SGI346681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346682, SGI0, NULL};
const Segment * const SG82024[] = {SGI346683, SGI346684, SGI346685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346686, SGI0, NULL};
const Segment * const SG82025[] = {SGI346687, SGI346688, SGI346689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346690, SGI0, NULL};
const Segment * const SG82026[] = {SGI346691, SGI346692, SGI346693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346693, SGI0, NULL};
const Segment * const SG82027[] = {SGI346695, SGI346696, SGI346697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346698, SGI0, NULL};
const Segment * const SG82028[] = {SGI346694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346699, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82029[] = {SGI346700, SGI346701, SGI346702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346703, SGI0, NULL};
const Segment * const SG82030[] = {SGI346704, SGI346705, SGI346706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346707, SGI0, NULL};
const Segment * const SG82031[] = {SGI346708, SGI346709, SGI346710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346711, SGI0, NULL};
const Segment * const SG82032[] = {SGI346712, SGI346713, SGI346714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346715, SGI0, NULL};
const Segment * const SG82033[] = {SGI346716, SGI346717, SGI346718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346719, SGI0, NULL};
const Segment * const SG82034[] = {SGI346720, SGI346721, SGI346722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346723, SGI0, NULL};
const Segment * const SG82035[] = {SGI346724, SGI0, SGI346725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346726, SGI0, NULL};
const Segment * const SG82036[] = {SGI346727, SGI346728, SGI346729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346730, SGI0, NULL};
const Segment * const SG82037[] = {SGI346731, SGI346732, SGI346733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346734, SGI0, NULL};
const Segment * const SG82038[] = {SGI346735, SGI346736, SGI346737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346738, SGI0, NULL};
const Segment * const SG82039[] = {SGI346739, SGI346740, SGI346741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346742, SGI0, NULL};
const Segment * const SG82040[] = {SGI346743, SGI346744, SGI346745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346746, SGI0, NULL};
const Segment * const SG82041[] = {SGI346747, SGI346748, SGI346749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346750, SGI0, NULL};
const Segment * const SG82042[] = {SGI346751, SGI346752, SGI346753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346754, SGI0, NULL};
const Segment * const SG82043[] = {SGI346755, SGI346756, SGI346757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346758, SGI0, NULL};
const Segment * const SG82044[] = {SGI346759, SGI346760, SGI346761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346762, SGI0, NULL};
const Segment * const SG82045[] = {SGI346763, SGI346764, SGI0, SGI0, NULL};
const Segment * const SG82046[] = {SGI346765, SGI346766, SGI346767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346768, SGI0, NULL};
const Segment * const SG82047[] = {SGI346769, SGI346770, SGI346771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346772, SGI0, NULL};
const Segment * const SG82048[] = {SGI346773, SGI346774, SGI346775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346777, SGI0, NULL};
const Segment * const SG82049[] = {SGI346776, SGI0, SGI0, SGI346778, SGI0, SGI0, SGI0, SGI0, SGI346779, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82050[] = {SGI346780, SGI346781, SGI346782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346783, SGI0, NULL};
const Segment * const SG82051[] = {SGI346784, SGI346785, SGI346786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346787, SGI0, NULL};
const Segment * const SG82052[] = {SGI346788, SGI346789, SGI346790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346791, SGI0, NULL};
const Segment * const SG82053[] = {SGI346792, SGI346793, SGI346794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346795, SGI0, NULL};
const Segment * const SG82054[] = {SGI346796, SGI346797, SGI346798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346799, SGI0, NULL};
const Segment * const SG82055[] = {SGI346800, SGI346801, SGI346802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346803, SGI0, NULL};
const Segment * const SG82056[] = {SGI346804, SGI346805, SGI346806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346807, SGI0, NULL};
const Segment * const SG82057[] = {SGI346808, SGI346809, SGI346810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346811, SGI0, NULL};
const Segment * const SG82058[] = {SGI346812, SGI346813, SGI346814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346815, SGI0, NULL};
const Segment * const SG82059[] = {SGI346816, SGI346817, SGI346818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346819, SGI0, NULL};
const Segment * const SG82060[] = {SGI346820, SGI346821, SGI346822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346823, SGI0, NULL};
const Segment * const SG82061[] = {SGI346824, SGI346826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82062[] = {SGI346825, SGI346827, SGI346828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346829, SGI0, NULL};
const Segment * const SG82063[] = {SGI346830, SGI346831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82064[] = {SGI346832, SGI346833, SGI346834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346835, SGI0, NULL};
const Segment * const SG82065[] = {SGI346836, SGI346837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82066[] = {SGI346838, SGI346839, SGI346840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346841, SGI0, NULL};
const Segment * const SG82067[] = {SGI346842, SGI346843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82068[] = {SGI346844, SGI346845, SGI346846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346847, SGI0, NULL};
const Segment * const SG82069[] = {SGI346848, SGI346849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82070[] = {SGI346850, SGI346851, SGI346852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346853, SGI0, NULL};
const Segment * const SG82071[] = {SGI346854, SGI346855, SGI346856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346857, SGI0, NULL};
const Segment * const SG82072[] = {SGI346858, SGI346859, SGI346860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346861, SGI0, NULL};
const Segment * const SG82073[] = {SGI346862, SGI346863, SGI346864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346865, SGI0, NULL};
const Segment * const SG82074[] = {SGI346866, SGI346867, SGI346868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346869, SGI0, NULL};
const Segment * const SG82075[] = {SGI346870, SGI346871, SGI346872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346873, SGI0, NULL};
const Segment * const SG82076[] = {SGI346874, SGI346875, SGI346876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346877, SGI0, NULL};
const Segment * const SG82077[] = {SGI346878, SGI346879, SGI346880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346881, SGI0, NULL};
const Segment * const SG82078[] = {SGI346882, SGI346883, SGI346884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346885, SGI0, NULL};
const Segment * const SG82079[] = {SGI346886, SGI346887, SGI346888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346889, SGI0, NULL};
const Segment * const SG82080[] = {SGI346890, SGI346891, SGI346892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346893, SGI0, NULL};
const Segment * const SG82081[] = {SGI346894, SGI346895, SGI346896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346897, SGI0, NULL};
const Segment * const SG82082[] = {SGI346898, SGI346899, SGI346900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346901, SGI0, NULL};
const Segment * const SG82083[] = {SGI346902, SGI0, SGI346903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346903, SGI0, NULL};
const Segment * const SG82084[] = {SGI346904, SGI346905, SGI346906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346907, SGI0, NULL};
const Segment * const SG82085[] = {SGI346908, SGI346909, SGI346910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346911, SGI0, NULL};
const Segment * const SG82086[] = {SGI346912, SGI346913, SGI346914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346915, SGI0, NULL};
const Segment * const SG82087[] = {SGI346916, SGI0, SGI346917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346918, SGI0, NULL};
const Segment * const SG82088[] = {SGI346919, SGI346920, SGI346921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346922, SGI0, NULL};
const Segment * const SG82089[] = {SGI346923, SGI346924, SGI346925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346926, SGI0, NULL};
const Segment * const SG82090[] = {SGI346927, SGI346928, SGI346929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346930, SGI0, NULL};
const Segment * const SG82091[] = {SGI346931, SGI346932, SGI346933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346934, SGI0, NULL};
const Segment * const SG82092[] = {SGI346935, SGI346936, SGI346937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346938, SGI0, NULL};
const Segment * const SG82093[] = {SGI346939, SGI346940, SGI346941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346942, SGI0, NULL};
const Segment * const SG82094[] = {SGI346943, SGI346944, SGI346945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346946, SGI0, NULL};
const Segment * const SG82095[] = {SGI346947, SGI346948, SGI346949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346950, SGI0, NULL};
const Segment * const SG82096[] = {SGI346951, SGI346952, SGI346953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346954, SGI0, NULL};
const Segment * const SG82097[] = {SGI346955, SGI346956, SGI346957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346958, SGI0, NULL};
const Segment * const SG82098[] = {SGI346959, SGI346960, SGI346961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346962, SGI0, NULL};
const Segment * const SG82099[] = {SGI346963, SGI346964, SGI346965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346966, SGI0, NULL};
const Segment * const SG82100[] = {SGI346967, SGI346968, SGI346969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346970, SGI0, NULL};
const Segment * const SG82101[] = {SGI346971, SGI346972, SGI346973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346974, SGI0, NULL};
const Segment * const SG82102[] = {SGI346975, SGI346976, SGI346977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346978, SGI0, NULL};
const Segment * const SG82103[] = {SGI346979, SGI346980, SGI346981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346982, SGI0, NULL};
const Segment * const SG82104[] = {SGI346983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346984, SGI0, NULL};
const Segment * const SG82105[] = {SGI346985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346986, SGI0, NULL};
const Segment * const SG82106[] = {SGI346987, SGI0, SGI0, SGI346988, SGI0, SGI0, SGI0, SGI0, SGI346989, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82107[] = {SGI346990, SGI0, SGI0, SGI346991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82108[] = {SGI346992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346993, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82109[] = {SGI346994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI346995, SGI0, NULL};
const Segment * const SG82110[] = {SGI346996, SGI346997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82111[] = {SGI346998, SGI0, SGI0, SGI346999, SGI0, SGI0, SGI0, SGI0, SGI347000, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82112[] = {SGI347001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347002, SGI0, NULL};
const Segment * const SG82113[] = {SGI347003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347004, SGI0, NULL};
const Segment * const SG82114[] = {SGI347005, SGI347006, SGI347007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347008, SGI0, NULL};
const Segment * const SG82115[] = {SGI347009, SGI347010, SGI347011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347011, SGI0, NULL};
const Segment * const SG82116[] = {SGI347012, SGI347013, SGI347014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347015, SGI0, NULL};
const Segment * const SG82117[] = {SGI347016, SGI347017, SGI347018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347019, SGI0, NULL};
const Segment * const SG82118[] = {SGI347020, SGI347021, SGI347022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347023, SGI0, NULL};
const Segment * const SG82119[] = {SGI347024, SGI347025, SGI347026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347027, SGI0, NULL};
const Segment * const SG82120[] = {SGI347028, SGI347029, SGI347030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347031, SGI0, NULL};
const Segment * const SG82121[] = {SGI347032, SGI347033, SGI347034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347035, SGI0, NULL};
const Segment * const SG82122[] = {SGI347036, SGI347037, SGI347038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347039, SGI0, NULL};
const Segment * const SG82123[] = {SGI347040, SGI347041, SGI347042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347043, SGI0, NULL};
const Segment * const SG82124[] = {SGI347044, SGI347045, SGI347046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347047, SGI0, NULL};
const Segment * const SG82125[] = {SGI347048, SGI347049, SGI347050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347051, SGI0, NULL};
const Segment * const SG82126[] = {SGI347052, SGI347053, SGI347054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347055, SGI0, NULL};
const Segment * const SG82127[] = {SGI347056, SGI347057, SGI347058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347059, SGI0, NULL};
const Segment * const SG82128[] = {SGI347060, SGI347061, SGI347062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347063, SGI0, NULL};
const Segment * const SG82129[] = {SGI347064, SGI347065, SGI347066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347067, SGI0, NULL};
const Segment * const SG82130[] = {SGI347068, SGI347069, SGI347070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347071, SGI0, NULL};
const Segment * const SG82131[] = {SGI347072, SGI347073, SGI347074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347075, SGI0, NULL};
const Segment * const SG82132[] = {SGI347076, SGI347077, SGI347078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347079, SGI0, NULL};
const Segment * const SG82133[] = {SGI347080, SGI347081, SGI347082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347083, SGI0, NULL};
const Segment * const SG82134[] = {SGI347084, SGI347085, SGI347086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347087, SGI0, NULL};
const Segment * const SG82135[] = {SGI347088, SGI347089, SGI347090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347091, SGI0, NULL};
const Segment * const SG82136[] = {SGI347092, SGI347093, SGI347094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347095, SGI0, NULL};
const Segment * const SG82137[] = {SGI347096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347097, SGI0, NULL};
const Segment * const SG82138[] = {SGI347098, SGI347099, SGI347100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347101, SGI0, NULL};
const Segment * const SG82139[] = {SGI347102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347106, SGI0, NULL};
const Segment * const SG82140[] = {SGI347103, SGI347104, SGI347105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347107, SGI0, NULL};
const Segment * const SG82141[] = {SGI347108, SGI347109, SGI347110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347111, SGI0, NULL};
const Segment * const SG82142[] = {SGI347112, SGI347113, SGI347114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347115, SGI0, NULL};
const Segment * const SG82143[] = {SGI347116, SGI347117, SGI347118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347119, SGI0, NULL};
const Segment * const SG82144[] = {SGI347120, SGI347121, SGI347122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347123, SGI0, NULL};
const Segment * const SG82145[] = {SGI347124, SGI347125, SGI347126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347127, SGI0, NULL};
const Segment * const SG82146[] = {SGI347128, SGI347129, SGI347130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347131, SGI0, NULL};
const Segment * const SG82147[] = {SGI347132, SGI347133, SGI347134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347135, SGI0, NULL};
const Segment * const SG82148[] = {SGI347136, SGI347137, SGI347138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347139, SGI0, NULL};
const Segment * const SG82149[] = {SGI347140, SGI347141, SGI347142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347143, SGI0, NULL};
const Segment * const SG82150[] = {SGI347144, SGI347145, SGI347146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347147, SGI0, NULL};
const Segment * const SG82151[] = {SGI347148, SGI347149, SGI347150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347151, SGI0, NULL};
const Segment * const SG82152[] = {SGI347152, SGI347153, SGI347154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347155, SGI0, NULL};
const Segment * const SG82153[] = {SGI347156, SGI347157, SGI347158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347159, SGI0, NULL};
const Segment * const SG82154[] = {SGI347160, SGI347161, SGI347162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347163, SGI0, NULL};
const Segment * const SG82155[] = {SGI347164, SGI347165, SGI347166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347167, SGI0, NULL};
const Segment * const SG82156[] = {SGI347168, SGI347169, SGI347170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347171, SGI0, NULL};
const Segment * const SG82157[] = {SGI347172, SGI347173, SGI347174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347175, SGI0, NULL};
const Segment * const SG82158[] = {SGI347176, SGI347177, SGI0, SGI0, NULL};
const Segment * const SG82159[] = {SGI347178, SGI347179, SGI347180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347181, SGI0, NULL};
const Segment * const SG82160[] = {SGI347182, SGI347183, SGI0, SGI0, NULL};
const Segment * const SG82161[] = {SGI347184, SGI347185, SGI347186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347187, SGI0, NULL};
const Segment * const SG82162[] = {SGI347189, SGI347190, SGI0, SGI0, NULL};
const Segment * const SG82163[] = {SGI347188, SGI347191, SGI347192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347193, SGI0, NULL};
const Segment * const SG82164[] = {SGI347194, SGI347195, SGI0, SGI0, NULL};
const Segment * const SG82165[] = {SGI347196, SGI347197, SGI0, SGI0, NULL};
const Segment * const SG82166[] = {SGI347198, SGI347199, SGI0, SGI0, NULL};
const Segment * const SG82167[] = {SGI347200, SGI347201, SGI0, SGI0, NULL};
const Segment * const SG82168[] = {SGI347202, SGI347203, SGI0, SGI0, NULL};
const Segment * const SG82169[] = {SGI347204, SGI347205, SGI0, SGI0, NULL};
const Segment * const SG82170[] = {SGI347206, SGI347207, SGI0, SGI0, NULL};
const Segment * const SG82171[] = {SGI347208, SGI347209, SGI0, SGI0, NULL};
const Segment * const SG82172[] = {SGI347210, SGI347211, SGI0, SGI0, NULL};
const Segment * const SG82173[] = {SGI347212, SGI347213, SGI0, SGI0, NULL};
const Segment * const SG82174[] = {SGI347214, SGI347215, SGI0, SGI0, NULL};
const Segment * const SG82175[] = {SGI347216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82176[] = {SGI347217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82177[] = {SGI347218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347219, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82178[] = {SGI347220, SGI347221, SGI0, SGI347222, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82179[] = {SGI347223, SGI347224, SGI0, SGI347225, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82180[] = {SGI347226, SGI347227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82181[] = {SGI347228, SGI347229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82182[] = {SGI347230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82183[] = {SGI347231, SGI347232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82184[] = {SGI347233, SGI0, SGI0, SGI347234, SGI0, SGI0, SGI0, SGI0, SGI347235, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82185[] = {SGI347236, SGI0, SGI0, SGI347237, SGI0, SGI0, SGI0, SGI0, SGI347238, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82186[] = {SGI347239, SGI0, SGI0, SGI347240, SGI0, SGI0, SGI0, SGI0, SGI347241, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82187[] = {SGI347242, SGI0, SGI0, SGI347243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82188[] = {SGI347244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82189[] = {SGI347245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82190[] = {SGI347246, SGI0, SGI0, SGI347247, SGI0, SGI0, SGI0, SGI0, SGI347248, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82191[] = {SGI347249, SGI347250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82192[] = {SGI347251, SGI347252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82193[] = {SGI347253, SGI347254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82194[] = {SGI347255, SGI0, SGI0, SGI347256, SGI0, SGI0, SGI0, SGI0, SGI347257, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82195[] = {SGI347258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347259, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82196[] = {SGI347260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82197[] = {SGI347261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82198[] = {SGI347262, SGI0, SGI0, SGI347263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82199[] = {SGI347264, SGI0, SGI347265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82200[] = {SGI347266, SGI0, SGI0, SGI0, SGI347267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82201[] = {SGI347268, SGI0, SGI0, SGI0, SGI347269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82202[] = {SGI347270, SGI0, SGI0, SGI0, SGI347271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82203[] = {SGI347272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347273, SGI0, NULL};
const Segment * const SG82204[] = {SGI347274, SGI0, SGI0, SGI347275, SGI0, SGI0, SGI0, SGI0, SGI347276, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82205[] = {SGI347277, SGI0, SGI0, SGI347278, SGI0, SGI0, SGI0, SGI0, SGI347279, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82206[] = {SGI347280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347281, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82207[] = {SGI347282, SGI347283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82208[] = {SGI347284, SGI347285, SGI0, SGI0, NULL};
const Segment * const SG82209[] = {SGI347286, SGI347287, SGI0, SGI0, NULL};
const Segment * const SG82210[] = {SGI347288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347289, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82211[] = {SGI347290, SGI0, SGI0, SGI347291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82212[] = {SGI347292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347293, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82213[] = {SGI347294, SGI0, SGI0, SGI347295, SGI0, SGI0, SGI0, SGI0, SGI347296, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82214[] = {SGI0, SGI347297, SGI347298, SGI347299, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82215[] = {SGI347300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82216[] = {SGI347301, SGI347302, SGI0, SGI0, NULL};
const Segment * const SG82217[] = {SGI347303, SGI0, SGI0, SGI347304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82218[] = {SGI347305, SGI0, SGI347306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82219[] = {SGI347307, SGI0, SGI347308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82220[] = {SGI347309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347310, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82221[] = {SGI347311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347312, SGI0, NULL};
const Segment * const SG82222[] = {SGI347313, SGI347314, SGI0, SGI0, NULL};
const Segment * const SG82223[] = {SGI347315, SGI347316, SGI0, SGI0, NULL};
const Segment * const SG82224[] = {SGI347317, SGI0, SGI0, SGI347318, SGI0, SGI0, SGI0, SGI0, SGI347319, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82225[] = {SGI347320, SGI0, SGI347321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82226[] = {SGI347322, SGI0, SGI0, SGI347323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82227[] = {SGI347324, SGI0, SGI0, SGI347325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82228[] = {SGI347326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347327, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82229[] = {SGI347328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347329, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82230[] = {SGI347330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347331, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82231[] = {SGI347332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347333, SGI0, NULL};
const Segment * const SG82232[] = {SGI347334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347335, SGI0, NULL};
const Segment * const SG82233[] = {SGI347336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347337, SGI0, NULL};
const Segment * const SG82234[] = {SGI347338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347339, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82235[] = {SGI347340, SGI0, SGI0, SGI347341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82236[] = {SGI347342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347343, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82237[] = {SGI347344, SGI347345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82238[] = {SGI347346, SGI347347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82239[] = {SGI347348, SGI347349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82240[] = {SGI347350, SGI347351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82241[] = {SGI347352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347353, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82242[] = {SGI347354, SGI347355, SGI0, SGI0, NULL};
const Segment * const SG82243[] = {SGI347356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82244[] = {SGI347357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82245[] = {SGI347358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82246[] = {SGI347359, SGI347360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82247[] = {SGI347361, SGI347362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82248[] = {SGI347363, SGI347364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82249[] = {SGI347365, SGI347366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82250[] = {SGI347367, SGI347368, SGI0, SGI0, NULL};
const Segment * const SG82251[] = {SGI347369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347370, SGI0, NULL};
const Segment * const SG82252[] = {SGI347371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347372, SGI0, NULL};
const Segment * const SG82253[] = {SGI347373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347374, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82254[] = {SGI347375, SGI0, SGI0, SGI347376, SGI0, SGI0, SGI0, SGI0, SGI347377, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82255[] = {SGI347378, SGI0, SGI0, SGI347379, SGI0, SGI0, SGI0, SGI0, SGI347380, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82256[] = {SGI347381, SGI0, SGI0, SGI347382, SGI0, SGI0, SGI0, SGI0, SGI347383, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82257[] = {SGI347384, SGI0, SGI0, SGI347385, SGI0, SGI0, SGI0, SGI0, SGI347386, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82258[] = {SGI347387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347388, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82259[] = {SGI347389, SGI0, SGI0, SGI347390, SGI0, SGI0, SGI0, SGI0, SGI347391, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82260[] = {SGI347392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347393, SGI0, NULL};
const Segment * const SG82261[] = {SGI347394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347395, SGI0, NULL};
const Segment * const SG82262[] = {SGI347396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347397, SGI0, NULL};
const Segment * const SG82263[] = {SGI347398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347399, SGI0, NULL};
const Segment * const SG82264[] = {SGI347400, SGI347401, SGI347402, SGI347403, NULL};
const Segment * const SG82265[] = {SGI347404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347405, SGI0, NULL};
const Segment * const SG82266[] = {SGI347406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347407, SGI0, NULL};
const Segment * const SG82267[] = {SGI347408, SGI0, SGI0, SGI347409, SGI0, SGI0, SGI0, SGI0, SGI347410, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82268[] = {SGI347411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347412, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82269[] = {SGI347413, SGI0, SGI347414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82270[] = {SGI347415, SGI0, SGI0, SGI347416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82271[] = {SGI347417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347418, SGI0, NULL};
const Segment * const SG82272[] = {SGI347419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347420, SGI0, NULL};
const Segment * const SG82273[] = {SGI347421, SGI347422, SGI347423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347424, SGI0, NULL};
const Segment * const SG82274[] = {SGI347425, SGI347426, SGI347427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347428, SGI0, NULL};
const Segment * const SG82275[] = {SGI347429, SGI347430, SGI347431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347432, SGI0, NULL};
const Segment * const SG82276[] = {SGI347433, SGI347434, SGI347435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347436, SGI0, NULL};
const Segment * const SG82277[] = {SGI347437, SGI347438, SGI347439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347440, SGI0, NULL};
const Segment * const SG82278[] = {SGI347441, SGI0, SGI347442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82279[] = {SGI347443, SGI347444, SGI347445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347446, SGI0, NULL};
const Segment * const SG82280[] = {SGI347447, SGI0, SGI0, SGI347449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82281[] = {SGI347448, SGI347450, SGI347451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347452, SGI0, NULL};
const Segment * const SG82282[] = {SGI347454, SGI347455, SGI347456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347456, SGI0, NULL};
const Segment * const SG82283[] = {SGI347453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347457, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82284[] = {SGI347458, SGI347459, SGI347460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347461, SGI0, NULL};
const Segment * const SG82285[] = {SGI347462, SGI347463, SGI347464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347465, SGI0, NULL};
const Segment * const SG82286[] = {SGI347466, SGI347467, SGI347468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347469, SGI0, NULL};
const Segment * const SG82287[] = {SGI347470, SGI347471, SGI347472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347473, SGI0, NULL};
const Segment * const SG82288[] = {SGI347474, SGI347475, SGI347476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347477, SGI0, NULL};
const Segment * const SG82289[] = {SGI347478, SGI0, SGI0, SGI347479, SGI0, SGI0, SGI0, SGI0, SGI347483, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82290[] = {SGI347480, SGI347481, SGI347482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347484, SGI0, NULL};
const Segment * const SG82291[] = {SGI347485, SGI347487, SGI347488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347490, SGI0, NULL};
const Segment * const SG82292[] = {SGI347486, SGI0, SGI0, SGI347489, SGI0, SGI0, SGI0, SGI0, SGI347491, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82293[] = {SGI347492, SGI347493, SGI347494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347495, SGI0, NULL};
const Segment * const SG82294[] = {SGI347496, SGI347497, SGI347498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347499, SGI0, NULL};
const Segment * const SG82295[] = {SGI347500, SGI347501, SGI347502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347503, SGI0, NULL};
const Segment * const SG82296[] = {SGI347504, SGI347505, SGI347506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347507, SGI0, NULL};
const Segment * const SG82297[] = {SGI347508, SGI347509, SGI347510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347511, SGI0, NULL};
const Segment * const SG82298[] = {SGI347512, SGI347513, SGI347514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347515, SGI0, NULL};
const Segment * const SG82299[] = {SGI347516, SGI347517, SGI347518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347519, SGI0, NULL};
const Segment * const SG82300[] = {SGI347520, SGI347521, SGI347522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347523, SGI0, NULL};
const Segment * const SG82301[] = {SGI347524, SGI347525, SGI347526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347527, SGI0, NULL};
const Segment * const SG82302[] = {SGI347528, SGI347529, SGI347530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347531, SGI0, NULL};
const Segment * const SG82303[] = {SGI347532, SGI347533, SGI347534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347535, SGI0, NULL};
const Segment * const SG82304[] = {SGI347536, SGI347537, SGI347538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347539, SGI0, NULL};
const Segment * const SG82305[] = {SGI347540, SGI347541, SGI347542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347543, SGI0, NULL};
const Segment * const SG82306[] = {SGI347544, SGI347545, SGI347546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347547, SGI0, NULL};
const Segment * const SG82307[] = {SGI347548, SGI347549, SGI347550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347551, SGI0, NULL};
const Segment * const SG82308[] = {SGI347552, SGI347553, SGI347554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347555, SGI0, NULL};
const Segment * const SG82309[] = {SGI347556, SGI347557, SGI347558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347559, SGI0, NULL};
const Segment * const SG82310[] = {SGI347560, SGI347561, SGI347562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347563, SGI0, NULL};
const Segment * const SG82311[] = {SGI347564, SGI347565, SGI347566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347567, SGI0, NULL};
const Segment * const SG82312[] = {SGI347568, SGI347569, SGI347570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347571, SGI0, NULL};
const Segment * const SG82313[] = {SGI347572, SGI347573, SGI347574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347575, SGI0, NULL};
const Segment * const SG82314[] = {SGI347576, SGI347577, SGI347578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347579, SGI0, NULL};
const Segment * const SG82315[] = {SGI347580, SGI0, SGI0, SGI347581, SGI0, SGI0, SGI0, SGI0, SGI347582, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82316[] = {SGI347583, SGI0, SGI0, SGI347584, SGI0, SGI0, SGI0, SGI0, SGI347585, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82317[] = {SGI347586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82318[] = {SGI347587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82319[] = {SGI347588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347589, SGI0, NULL};
const Segment * const SG82320[] = {SGI347590, SGI347591, SGI347592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347593, SGI0, NULL};
const Segment * const SG82321[] = {SGI347594, SGI347595, SGI347596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347597, SGI0, NULL};
const Segment * const SG82322[] = {SGI347598, SGI347599, SGI347600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347601, SGI0, NULL};
const Segment * const SG82323[] = {SGI347602, SGI347603, SGI347604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347604, SGI0, NULL};
const Segment * const SG82324[] = {SGI347605, SGI347606, SGI347607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347608, SGI0, NULL};
const Segment * const SG82325[] = {SGI347609, SGI347610, SGI347611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347612, SGI0, NULL};
const Segment * const SG82326[] = {SGI347613, SGI347614, SGI347615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347616, SGI0, NULL};
const Segment * const SG82327[] = {SGI347617, SGI347618, SGI347619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347620, SGI0, NULL};
const Segment * const SG82328[] = {SGI347621, SGI347622, SGI347623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347624, SGI0, NULL};
const Segment * const SG82329[] = {SGI347625, SGI347626, SGI347627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347628, SGI0, NULL};
const Segment * const SG82330[] = {SGI347629, SGI347630, SGI347631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347632, SGI0, NULL};
const Segment * const SG82331[] = {SGI347633, SGI347634, SGI347635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347636, SGI0, NULL};
const Segment * const SG82332[] = {SGI347637, SGI347638, SGI347639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347640, SGI0, NULL};
const Segment * const SG82333[] = {SGI347641, SGI347642, SGI347643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347643, SGI0, NULL};
const Segment * const SG82334[] = {SGI347644, SGI347645, SGI347646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347647, SGI0, NULL};
const Segment * const SG82335[] = {SGI347648, SGI347649, SGI347650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347651, SGI0, NULL};
const Segment * const SG82336[] = {SGI347652, SGI347653, SGI347654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347655, SGI0, NULL};
const Segment * const SG82337[] = {SGI347656, SGI347657, SGI347658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347659, SGI0, NULL};
const Segment * const SG82338[] = {SGI347660, SGI347661, SGI347662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347663, SGI0, NULL};
const Segment * const SG82339[] = {SGI347664, SGI347665, SGI347666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347667, SGI0, NULL};
const Segment * const SG82340[] = {SGI347668, SGI347669, SGI347670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347671, SGI0, NULL};
const Segment * const SG82341[] = {SGI347672, SGI347673, SGI347674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347675, SGI0, NULL};
const Segment * const SG82342[] = {SGI347676, SGI347677, SGI347678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347679, SGI0, NULL};
const Segment * const SG82343[] = {SGI347680, SGI347681, SGI347682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347683, SGI0, NULL};
const Segment * const SG82344[] = {SGI347684, SGI347685, SGI347686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347687, SGI0, NULL};
const Segment * const SG82345[] = {SGI347688, SGI347689, SGI347690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347691, SGI0, NULL};
const Segment * const SG82346[] = {SGI347692, SGI347693, SGI347694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347695, SGI0, NULL};
const Segment * const SG82347[] = {SGI347696, SGI347697, SGI347698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347699, SGI0, NULL};
const Segment * const SG82348[] = {SGI347700, SGI347701, SGI347702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347703, SGI0, NULL};
const Segment * const SG82349[] = {SGI347704, SGI347705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82350[] = {SGI347706, SGI347707, SGI347708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347709, SGI0, NULL};
const Segment * const SG82351[] = {SGI347710, SGI347711, SGI347712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347713, SGI0, NULL};
const Segment * const SG82352[] = {SGI347714, SGI347715, SGI347716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347717, SGI0, NULL};
const Segment * const SG82353[] = {SGI347718, SGI347719, SGI347720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347721, SGI0, NULL};
const Segment * const SG82354[] = {SGI347722, SGI347723, SGI347724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347725, SGI0, NULL};
const Segment * const SG82355[] = {SGI347726, SGI347727, SGI347728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347729, SGI0, NULL};
const Segment * const SG82356[] = {SGI347730, SGI347731, SGI347732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347733, SGI0, NULL};
const Segment * const SG82357[] = {SGI347734, SGI347735, SGI347736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347737, SGI0, NULL};
const Segment * const SG82358[] = {SGI347738, SGI347739, SGI347740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347741, SGI0, NULL};
const Segment * const SG82359[] = {SGI347742, SGI347743, SGI347744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347745, SGI0, NULL};
const Segment * const SG82360[] = {SGI347746, SGI347747, SGI347748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347749, SGI0, NULL};
const Segment * const SG82361[] = {SGI347750, SGI347751, SGI347752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347753, SGI0, NULL};
const Segment * const SG82362[] = {SGI347754, SGI347755, SGI347756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347757, SGI0, NULL};
const Segment * const SG82363[] = {SGI347758, SGI347759, SGI347760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347761, SGI0, NULL};
const Segment * const SG82364[] = {SGI347762, SGI347763, SGI347764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347765, SGI0, NULL};
const Segment * const SG82365[] = {SGI347766, SGI347767, SGI347768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347769, SGI0, NULL};
const Segment * const SG82366[] = {SGI347770, SGI347771, SGI347772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347773, SGI0, NULL};
const Segment * const SG82367[] = {SGI347774, SGI347775, SGI347776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347777, SGI0, NULL};
const Segment * const SG82368[] = {SGI347778, SGI347779, SGI347780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347781, SGI0, NULL};
const Segment * const SG82369[] = {SGI347782, SGI347783, SGI347784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347785, SGI0, NULL};
const Segment * const SG82370[] = {SGI347786, SGI347787, SGI347788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347789, SGI0, NULL};
const Segment * const SG82371[] = {SGI347790, SGI347791, SGI347792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347793, SGI0, NULL};
const Segment * const SG82372[] = {SGI347794, SGI347795, SGI347796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347797, SGI0, NULL};
const Segment * const SG82373[] = {SGI347798, SGI347799, SGI347800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347801, SGI0, NULL};
const Segment * const SG82374[] = {SGI347802, SGI347803, SGI347804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347805, SGI0, NULL};
const Segment * const SG82375[] = {SGI347806, SGI347807, SGI347808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347809, SGI0, NULL};
const Segment * const SG82376[] = {SGI347810, SGI347811, SGI347812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347813, SGI0, NULL};
const Segment * const SG82377[] = {SGI347815, SGI347816, SGI347817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347818, SGI0, NULL};
const Segment * const SG82378[] = {SGI347814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347819, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82379[] = {SGI347820, SGI347821, SGI347822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347823, SGI0, NULL};
const Segment * const SG82380[] = {SGI347824, SGI347825, SGI347826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347827, SGI0, NULL};
const Segment * const SG82381[] = {SGI347828, SGI347829, SGI0, SGI0, NULL};
const Segment * const SG82382[] = {SGI347830, SGI347831, SGI347832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347833, SGI0, NULL};
const Segment * const SG82383[] = {SGI347834, SGI347835, SGI347836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347837, SGI0, NULL};
const Segment * const SG82384[] = {SGI347838, SGI347839, SGI347840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347841, SGI0, NULL};
const Segment * const SG82385[] = {SGI347842, SGI347843, SGI347844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347845, SGI0, NULL};
const Segment * const SG82386[] = {SGI347846, SGI347847, SGI347848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347849, SGI0, NULL};
const Segment * const SG82387[] = {SGI347850, SGI347851, SGI347852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347853, SGI0, NULL};
const Segment * const SG82388[] = {SGI347854, SGI347855, SGI347856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347857, SGI0, NULL};
const Segment * const SG82389[] = {SGI347858, SGI347859, SGI347860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347861, SGI0, NULL};
const Segment * const SG82390[] = {SGI347862, SGI347863, SGI347864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347865, SGI0, NULL};
const Segment * const SG82391[] = {SGI347866, SGI347867, SGI347868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347869, SGI0, NULL};
const Segment * const SG82392[] = {SGI347870, SGI347871, SGI347872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347873, SGI0, NULL};
const Segment * const SG82393[] = {SGI347874, SGI347875, SGI347876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347877, SGI0, NULL};
const Segment * const SG82394[] = {SGI347878, SGI347879, SGI347880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347881, SGI0, NULL};
const Segment * const SG82395[] = {SGI347882, SGI347883, SGI347884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347885, SGI0, NULL};
const Segment * const SG82396[] = {SGI347886, SGI347887, SGI347888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347889, SGI0, NULL};
const Segment * const SG82397[] = {SGI347890, SGI347891, SGI347892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347893, SGI0, NULL};
const Segment * const SG82398[] = {SGI347894, SGI347895, SGI347896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347897, SGI0, NULL};
const Segment * const SG82399[] = {SGI347898, SGI0, SGI0, SGI347899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82400[] = {SGI347900, SGI347901, SGI347902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347903, SGI0, NULL};
const Segment * const SG82401[] = {SGI347904, SGI0, SGI347905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82402[] = {SGI347906, SGI347907, SGI347908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347909, SGI0, NULL};
const Segment * const SG82403[] = {SGI347910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347914, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82404[] = {SGI347911, SGI347912, SGI347913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347915, SGI0, NULL};
const Segment * const SG82405[] = {SGI0, SGI347916, SGI347917, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82406[] = {SGI347918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82407[] = {SGI347919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82408[] = {SGI347920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82409[] = {SGI347921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82410[] = {SGI347922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347923, SGI0, NULL};
const Segment * const SG82411[] = {SGI347924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347925, SGI0, NULL};
const Segment * const SG82412[] = {SGI347926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347927, SGI0, NULL};
const Segment * const SG82413[] = {SGI347928, SGI0, SGI0, SGI347929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82414[] = {SGI347930, SGI0, SGI347931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82415[] = {SGI347932, SGI0, SGI0, SGI347933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82416[] = {SGI347934, SGI347935, SGI0, SGI0, NULL};
const Segment * const SG82417[] = {SGI347936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347937, SGI0, NULL};
const Segment * const SG82418[] = {SGI347938, SGI347939, SGI0, SGI0, NULL};
const Segment * const SG82419[] = {SGI347940, SGI347941, SGI0, SGI0, NULL};
const Segment * const SG82420[] = {SGI347942, SGI347943, SGI0, SGI0, NULL};
const Segment * const SG82421[] = {SGI347944, SGI347945, SGI0, SGI0, NULL};
const Segment * const SG82422[] = {SGI347946, SGI347947, SGI0, SGI0, NULL};
const Segment * const SG82423[] = {SGI347948, SGI347949, SGI0, SGI0, NULL};
const Segment * const SG82424[] = {SGI347950, SGI0, SGI0, SGI0, SGI347951, SGI347952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82425[] = {SGI347953, SGI0, SGI0, SGI347954, SGI0, SGI0, SGI0, SGI0, SGI347955, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82426[] = {SGI347956, SGI0, SGI0, SGI347957, SGI0, SGI0, SGI0, SGI0, SGI347958, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82427[] = {SGI347959, SGI0, SGI0, SGI347960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82428[] = {SGI347961, SGI0, SGI0, SGI347962, SGI0, SGI0, SGI0, SGI0, SGI347963, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82429[] = {SGI347964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347965, SGI0, NULL};
const Segment * const SG82430[] = {SGI347966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347967, SGI0, NULL};
const Segment * const SG82431[] = {SGI347968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347969, SGI0, NULL};
const Segment * const SG82432[] = {SGI347970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347971, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82433[] = {SGI347972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82434[] = {SGI347973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82435[] = {SGI347974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82436[] = {SGI347975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82437[] = {SGI347976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82438[] = {SGI347977, SGI0, SGI0, SGI347978, SGI0, SGI0, SGI0, SGI0, SGI347979, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82439[] = {SGI347980, SGI0, SGI0, SGI347981, SGI0, SGI0, SGI0, SGI0, SGI347982, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82440[] = {SGI347983, SGI0, SGI0, SGI347984, SGI0, SGI0, SGI0, SGI0, SGI347985, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82441[] = {SGI347986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347987, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82442[] = {SGI0, SGI347988, SGI347989, SGI0, SGI0, SGI0, SGI347990, NULL};
const Segment * const SG82443[] = {SGI0, SGI347991, SGI347992, SGI0, SGI0, SGI0, SGI347993, NULL};
const Segment * const SG82444[] = {SGI347994, SGI0, SGI347995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82445[] = {SGI347996, SGI0, SGI347997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82446[] = {SGI347998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI347999, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82447[] = {SGI348000, SGI0, SGI348001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348001, SGI0, NULL};
const Segment * const SG82448[] = {SGI348002, SGI0, SGI348003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348003, SGI0, NULL};
const Segment * const SG82449[] = {SGI348004, SGI0, SGI348005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348005, SGI0, NULL};
const Segment * const SG82450[] = {SGI348006, SGI0, SGI348007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348007, SGI0, NULL};
const Segment * const SG82451[] = {SGI348008, SGI0, SGI348009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348009, SGI0, NULL};
const Segment * const SG82452[] = {SGI348010, SGI0, SGI348011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348012, SGI0, NULL};
const Segment * const SG82453[] = {SGI348013, SGI0, SGI348014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348014, SGI0, NULL};
const Segment * const SG82454[] = {SGI348015, SGI0, SGI348016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348016, SGI0, NULL};
const Segment * const SG82455[] = {SGI348017, SGI0, SGI0, SGI348018, SGI0, SGI0, SGI0, SGI0, SGI348019, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82456[] = {SGI348020, SGI0, SGI0, SGI348021, SGI0, SGI0, SGI0, SGI0, SGI348022, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82457[] = {SGI348023, SGI348024, SGI348025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348025, SGI0, NULL};
const Segment * const SG82458[] = {SGI348026, SGI348027, SGI348028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348029, SGI0, NULL};
const Segment * const SG82459[] = {SGI348030, SGI348031, SGI348032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348033, SGI0, NULL};
const Segment * const SG82460[] = {SGI348034, SGI348035, SGI348036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348037, SGI0, NULL};
const Segment * const SG82461[] = {SGI348038, SGI348039, SGI348040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348041, SGI0, NULL};
const Segment * const SG82462[] = {SGI348042, SGI348043, SGI348044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348045, SGI0, NULL};
const Segment * const SG82463[] = {SGI348046, SGI348047, SGI348048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348049, SGI0, NULL};
const Segment * const SG82464[] = {SGI348050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348051, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82465[] = {SGI348052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348053, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82466[] = {SGI348054, SGI0, SGI0, SGI348055, SGI0, SGI0, SGI0, SGI0, SGI348056, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82467[] = {SGI348057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348058, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82468[] = {SGI348059, SGI0, SGI0, SGI348060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82469[] = {SGI348061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348062, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82470[] = {SGI348063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82471[] = {SGI348064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82472[] = {SGI348065, SGI348066, SGI0, SGI0, NULL};
const Segment * const SG82473[] = {SGI348067, SGI348068, SGI0, SGI0, NULL};
const Segment * const SG82474[] = {SGI348069, SGI348070, SGI0, SGI0, NULL};
const Segment * const SG82475[] = {SGI348071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348072, SGI0, NULL};
const Segment * const SG82476[] = {SGI348073, SGI348074, SGI0, SGI0, NULL};
const Segment * const SG82477[] = {SGI348075, SGI348076, SGI0, SGI0, NULL};
const Segment * const SG82478[] = {SGI348077, SGI348078, SGI0, SGI0, NULL};
const Segment * const SG82479[] = {SGI348079, SGI348080, SGI0, SGI0, NULL};
const Segment * const SG82480[] = {SGI348081, SGI0, SGI0, SGI348082, SGI0, SGI0, SGI0, SGI0, SGI348083, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82481[] = {SGI348084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348085, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82482[] = {SGI348086, SGI0, SGI0, SGI348087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82483[] = {SGI348088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348089, SGI0, NULL};
const Segment * const SG82484[] = {SGI348090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348091, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82485[] = {SGI348092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348093, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82486[] = {SGI348094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348095, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82487[] = {SGI348096, SGI0, SGI0, SGI348097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82488[] = {SGI348098, SGI0, SGI348099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82489[] = {SGI348100, SGI348101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82490[] = {SGI348102, SGI348103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82491[] = {SGI348104, SGI0, SGI348105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82492[] = {SGI348106, SGI0, SGI348107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82493[] = {SGI348108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348109, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82494[] = {SGI348110, SGI0, SGI0, SGI348111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82495[] = {SGI348112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348113, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82496[] = {SGI348114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348115, SGI0, NULL};
const Segment * const SG82497[] = {SGI348116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348117, SGI0, NULL};
const Segment * const SG82498[] = {SGI348118, SGI0, SGI348119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82499[] = {SGI348120, SGI0, SGI0, SGI348121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82500[] = {SGI348122, SGI348123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82501[] = {SGI348124, SGI0, SGI348125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82502[] = {SGI348126, SGI348127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82503[] = {SGI348128, SGI348129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82504[] = {SGI348130, SGI348131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82505[] = {SGI348132, SGI348133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82506[] = {SGI348134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82507[] = {SGI348135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82508[] = {SGI348136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82509[] = {SGI348137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82510[] = {SGI348138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82511[] = {SGI348139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82512[] = {SGI348140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82513[] = {SGI348141, SGI0, SGI0, SGI348142, SGI0, SGI0, SGI0, SGI0, SGI348143, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82514[] = {SGI348144, SGI0, SGI0, SGI348145, SGI0, SGI0, SGI0, SGI0, SGI348146, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82515[] = {SGI348147, SGI0, SGI0, SGI348148, SGI0, SGI0, SGI0, SGI0, SGI348149, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82516[] = {SGI348150, SGI0, SGI0, SGI348151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82517[] = {SGI348152, SGI0, SGI0, SGI348153, SGI0, SGI0, SGI0, SGI0, SGI348154, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82518[] = {SGI348155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348156, SGI0, NULL};
const Segment * const SG82519[] = {SGI348157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348158, SGI0, NULL};
const Segment * const SG82520[] = {SGI348159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348160, SGI0, NULL};
const Segment * const SG82521[] = {SGI348161, SGI348162, SGI348163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82522[] = {SGI348164, SGI348165, SGI348166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82523[] = {SGI348167, SGI348168, SGI348169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348170, SGI0, NULL};
const Segment * const SG82524[] = {SGI348171, SGI348172, SGI348173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348173, SGI0, NULL};
const Segment * const SG82525[] = {SGI348174, SGI348175, SGI348176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348177, SGI0, NULL};
const Segment * const SG82526[] = {SGI348178, SGI348179, SGI348180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348181, SGI0, NULL};
const Segment * const SG82527[] = {SGI348182, SGI348183, SGI348184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348185, SGI0, NULL};
const Segment * const SG82528[] = {SGI348186, SGI348187, SGI348188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348189, SGI0, NULL};
const Segment * const SG82529[] = {SGI348190, SGI348191, SGI348192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348193, SGI0, NULL};
const Segment * const SG82530[] = {SGI348194, SGI348195, SGI348196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348197, SGI0, NULL};
const Segment * const SG82531[] = {SGI348198, SGI348199, SGI348200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348201, SGI0, NULL};
const Segment * const SG82532[] = {SGI348202, SGI348203, SGI348204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348205, SGI0, NULL};
const Segment * const SG82533[] = {SGI348206, SGI348207, SGI348208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348209, SGI0, NULL};
const Segment * const SG82534[] = {SGI348210, SGI348211, SGI348212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348213, SGI0, NULL};
const Segment * const SG82535[] = {SGI348214, SGI348215, SGI348216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348217, SGI0, NULL};
const Segment * const SG82536[] = {SGI348218, SGI348219, SGI348220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348221, SGI0, NULL};
const Segment * const SG82537[] = {SGI348222, SGI348223, SGI348224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348225, SGI0, NULL};
const Segment * const SG82538[] = {SGI348226, SGI348227, SGI348228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348229, SGI0, NULL};
const Segment * const SG82539[] = {SGI348230, SGI348231, SGI348232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348233, SGI0, NULL};
const Segment * const SG82540[] = {SGI348234, SGI348235, SGI348236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348237, SGI0, NULL};
const Segment * const SG82541[] = {SGI348238, SGI348239, SGI348240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348241, SGI0, NULL};
const Segment * const SG82542[] = {SGI348242, SGI348243, SGI348244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348245, SGI0, NULL};
const Segment * const SG82543[] = {SGI348246, SGI348247, SGI348248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348249, SGI0, NULL};
const Segment * const SG82544[] = {SGI348250, SGI348251, SGI348252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348253, SGI0, NULL};
const Segment * const SG82545[] = {SGI348254, SGI348255, SGI348256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348256, SGI0, NULL};
const Segment * const SG82546[] = {SGI348257, SGI348258, SGI348259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348260, SGI0, NULL};
const Segment * const SG82547[] = {SGI348261, SGI348262, SGI348263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348264, SGI0, NULL};
const Segment * const SG82548[] = {SGI348265, SGI348266, SGI348267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348268, SGI0, NULL};
const Segment * const SG82549[] = {SGI348269, SGI348270, SGI348271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348272, SGI0, NULL};
const Segment * const SG82550[] = {SGI348273, SGI348274, SGI348275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348276, SGI0, NULL};
const Segment * const SG82551[] = {SGI348277, SGI348278, SGI348279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348280, SGI0, NULL};
const Segment * const SG82552[] = {SGI348281, SGI348282, SGI348283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348284, SGI0, NULL};
const Segment * const SG82553[] = {SGI348285, SGI348286, SGI348287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348288, SGI0, NULL};
const Segment * const SG82554[] = {SGI348289, SGI348290, SGI348291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348292, SGI0, NULL};
const Segment * const SG82555[] = {SGI348293, SGI348294, SGI348295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348296, SGI0, NULL};
const Segment * const SG82556[] = {SGI348297, SGI348298, SGI348299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348300, SGI0, NULL};
const Segment * const SG82557[] = {SGI348301, SGI348302, SGI0, SGI0, NULL};
const Segment * const SG82558[] = {SGI348303, SGI348304, SGI348305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348306, SGI0, NULL};
const Segment * const SG82559[] = {SGI348307, SGI348308, SGI348309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348310, SGI0, NULL};
const Segment * const SG82560[] = {SGI348311, SGI348312, SGI348313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348314, SGI0, NULL};
const Segment * const SG82561[] = {SGI348315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348316, SGI0, NULL};
const Segment * const SG82562[] = {SGI348317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348318, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82563[] = {SGI348319, SGI0, SGI0, SGI348320, SGI0, SGI0, SGI0, SGI0, SGI348321, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82564[] = {SGI348322, SGI0, SGI0, SGI348323, SGI0, SGI0, SGI0, SGI0, SGI348324, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82565[] = {SGI348325, SGI348326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82566[] = {SGI348327, SGI348328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82567[] = {SGI348329, SGI348330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82568[] = {SGI348331, SGI348332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82569[] = {SGI348333, SGI348334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82570[] = {SGI348335, SGI348336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82571[] = {SGI348337, SGI348338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82572[] = {SGI348339, SGI0, SGI0, SGI348340, SGI0, SGI0, SGI0, SGI0, SGI348341, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82573[] = {SGI348342, SGI348343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82574[] = {SGI348344, SGI348345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82575[] = {SGI348346, SGI348347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82576[] = {SGI348348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82577[] = {SGI348349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82578[] = {SGI348350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348351, SGI0, NULL};
const Segment * const SG82579[] = {SGI348352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348353, SGI0, NULL};
const Segment * const SG82580[] = {SGI348354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348355, SGI0, NULL};
const Segment * const SG82581[] = {SGI348356, SGI0, SGI348357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348357, SGI0, NULL};
const Segment * const SG82582[] = {SGI348358, SGI0, SGI348359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348359, SGI0, NULL};
const Segment * const SG82583[] = {SGI348360, SGI0, SGI348361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348361, SGI0, NULL};
const Segment * const SG82584[] = {SGI348362, SGI0, SGI348363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348363, SGI0, NULL};
const Segment * const SG82585[] = {SGI348364, SGI0, SGI348365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348365, SGI0, NULL};
const Segment * const SG82586[] = {SGI348366, SGI0, SGI348367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348367, SGI0, NULL};
const Segment * const SG82587[] = {SGI348368, SGI0, SGI348369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348369, SGI0, NULL};
const Segment * const SG82588[] = {SGI348370, SGI0, SGI348371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348371, SGI0, NULL};
const Segment * const SG82589[] = {SGI348372, SGI348373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82590[] = {SGI348374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82591[] = {SGI348375, SGI348376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82592[] = {SGI348377, SGI348378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82593[] = {SGI348379, SGI348380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82594[] = {SGI348381, SGI348382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82595[] = {SGI348383, SGI348384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82596[] = {SGI348385, SGI0, SGI0, SGI348386, SGI0, SGI0, SGI0, SGI0, SGI348387, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82597[] = {SGI348388, SGI348389, SGI0, SGI0, NULL};
const Segment * const SG82598[] = {SGI348390, SGI348391, SGI0, SGI0, NULL};
const Segment * const SG82599[] = {SGI348392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348393, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82600[] = {SGI348394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82601[] = {SGI348395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82602[] = {SGI348396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82603[] = {SGI348397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348398, SGI0, NULL};
const Segment * const SG82604[] = {SGI348399, SGI0, SGI0, SGI348400, SGI0, SGI0, SGI0, SGI0, SGI348401, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82605[] = {SGI348402, SGI0, SGI0, SGI348403, SGI0, SGI0, SGI0, SGI0, SGI348404, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82606[] = {SGI348405, SGI0, SGI0, SGI348406, SGI0, SGI0, SGI0, SGI0, SGI348407, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82607[] = {SGI348408, SGI0, SGI0, SGI348409, SGI0, SGI0, SGI0, SGI0, SGI348410, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82608[] = {SGI348411, SGI0, SGI0, SGI348412, SGI0, SGI0, SGI0, SGI0, SGI348413, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82609[] = {SGI348414, SGI0, SGI0, SGI348415, SGI0, SGI0, SGI0, SGI0, SGI348416, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82610[] = {SGI348417, SGI348418, SGI348419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348420, SGI0, NULL};
const Segment * const SG82611[] = {SGI348421, SGI348422, SGI348423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348423, SGI0, NULL};
const Segment * const SG82612[] = {SGI348424, SGI348425, SGI348426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348427, SGI0, NULL};
const Segment * const SG82613[] = {SGI348428, SGI348429, SGI348430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348431, SGI0, NULL};
const Segment * const SG82614[] = {SGI348432, SGI348433, SGI348434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348435, SGI0, NULL};
const Segment * const SG82615[] = {SGI348436, SGI348437, SGI348438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348439, SGI0, NULL};
const Segment * const SG82616[] = {SGI348440, SGI348441, SGI348442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348443, SGI0, NULL};
const Segment * const SG82617[] = {SGI348444, SGI0, SGI0, SGI348445, SGI0, SGI0, SGI0, SGI0, SGI348446, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82618[] = {SGI348447, SGI348448, SGI348449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348450, SGI0, NULL};
const Segment * const SG82619[] = {SGI348451, SGI0, SGI348452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82620[] = {SGI348453, SGI348454, SGI348455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348457, SGI0, NULL};
const Segment * const SG82621[] = {SGI348456, SGI0, SGI0, SGI348458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82622[] = {SGI348459, SGI348460, SGI348461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348462, SGI0, NULL};
const Segment * const SG82623[] = {SGI348463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348465, SGI0, NULL};
const Segment * const SG82624[] = {SGI348464, SGI348466, SGI348467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348468, SGI0, NULL};
const Segment * const SG82625[] = {SGI348469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348472, SGI0, NULL};
const Segment * const SG82626[] = {SGI348470, SGI348471, SGI348473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348474, SGI0, NULL};
const Segment * const SG82627[] = {SGI348475, SGI348476, SGI348477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348478, SGI0, NULL};
const Segment * const SG82628[] = {SGI348479, SGI348480, SGI348481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348482, SGI0, NULL};
const Segment * const SG82629[] = {SGI348483, SGI348484, SGI348485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348486, SGI0, NULL};
const Segment * const SG82630[] = {SGI348487, SGI348488, SGI348489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348490, SGI0, NULL};
const Segment * const SG82631[] = {SGI348491, SGI348492, SGI348493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348494, SGI0, NULL};
const Segment * const SG82632[] = {SGI348495, SGI348496, SGI348497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348498, SGI0, NULL};
const Segment * const SG82633[] = {SGI348499, SGI348500, SGI348501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348502, SGI0, NULL};
const Segment * const SG82634[] = {SGI348503, SGI348504, SGI348505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348506, SGI0, NULL};
const Segment * const SG82635[] = {SGI348507, SGI348508, SGI348509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348510, SGI0, NULL};
const Segment * const SG82636[] = {SGI348511, SGI348512, SGI348513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348514, SGI0, NULL};
const Segment * const SG82637[] = {SGI348515, SGI348516, SGI348517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348518, SGI0, NULL};
const Segment * const SG82638[] = {SGI348519, SGI348520, SGI348521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348522, SGI0, NULL};
const Segment * const SG82639[] = {SGI348523, SGI348524, SGI348525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348526, SGI0, NULL};
const Segment * const SG82640[] = {SGI348527, SGI348528, SGI348529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348530, SGI0, NULL};
const Segment * const SG82641[] = {SGI348531, SGI348532, SGI348533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348534, SGI0, NULL};
const Segment * const SG82642[] = {SGI348535, SGI348536, SGI348537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348538, SGI0, NULL};
const Segment * const SG82643[] = {SGI348539, SGI348540, SGI348541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348542, SGI0, NULL};
const Segment * const SG82644[] = {SGI348543, SGI348544, SGI348545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348546, SGI0, NULL};
const Segment * const SG82645[] = {SGI348547, SGI348548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82646[] = {SGI348549, SGI348550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82647[] = {SGI348551, SGI348552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82648[] = {SGI348553, SGI348554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82649[] = {SGI348555, SGI0, SGI0, SGI348556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82650[] = {SGI348557, SGI0, SGI348558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82651[] = {SGI348559, SGI0, SGI348560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82652[] = {SGI348561, SGI0, SGI0, SGI348562, SGI0, SGI0, SGI0, SGI0, SGI348563, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82653[] = {SGI348564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348565, SGI0, NULL};
const Segment * const SG82654[] = {SGI348566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348567, SGI0, NULL};
const Segment * const SG82655[] = {SGI348568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348569, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82656[] = {SGI348570, SGI0, SGI0, SGI348571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82657[] = {SGI348572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348573, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82658[] = {SGI348574, SGI348575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82659[] = {SGI348576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348577, SGI0, NULL};
const Segment * const SG82660[] = {SGI348578, SGI348579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82661[] = {SGI348580, SGI348581, SGI348582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82662[] = {SGI348583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348584, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82663[] = {SGI348585, SGI0, SGI0, SGI348586, SGI0, SGI0, SGI0, SGI0, SGI348587, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82664[] = {SGI348588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82665[] = {SGI348589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82666[] = {SGI348590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82667[] = {SGI348591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82668[] = {SGI348592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82669[] = {SGI348593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82670[] = {SGI348594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348595, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82671[] = {SGI348596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348597, SGI0, NULL};
const Segment * const SG82672[] = {SGI348598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82673[] = {SGI348599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348600, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82674[] = {SGI348601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348602, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82675[] = {SGI348603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348604, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82676[] = {SGI348605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348606, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82677[] = {SGI348607, SGI348608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82678[] = {SGI348609, SGI348610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82679[] = {SGI348611, SGI0, SGI0, SGI348612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82680[] = {SGI348613, SGI0, SGI348614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82681[] = {SGI348615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348616, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82682[] = {SGI348617, SGI348618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82683[] = {SGI348619, SGI348620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82684[] = {SGI348621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82685[] = {SGI348622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348623, SGI0, NULL};
const Segment * const SG82686[] = {SGI348624, SGI348625, SGI348626, SGI0, SGI0, SGI0, SGI0, SGI348627, SGI0, SGI0, NULL};
const Segment * const SG82687[] = {SGI348628, SGI348629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82688[] = {SGI348630, SGI348631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82689[] = {SGI348632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348633, SGI0, NULL};
const Segment * const SG82690[] = {SGI348634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348635, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82691[] = {SGI348636, SGI0, SGI0, SGI348637, SGI0, SGI0, SGI0, SGI0, SGI348638, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82692[] = {SGI348639, SGI0, SGI0, SGI348640, SGI0, SGI0, SGI0, SGI0, SGI348641, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82693[] = {SGI348642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348643, SGI0, NULL};
const Segment * const SG82694[] = {SGI348644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348645, SGI0, NULL};
const Segment * const SG82695[] = {SGI348646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348647, SGI0, NULL};
const Segment * const SG82696[] = {SGI348648, SGI348649, SGI0, SGI0, NULL};
const Segment * const SG82697[] = {SGI348650, SGI348651, SGI0, SGI0, NULL};
const Segment * const SG82698[] = {SGI348652, SGI0, SGI0, SGI348653, SGI0, SGI0, SGI0, SGI0, SGI348654, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82699[] = {SGI348655, SGI0, SGI0, SGI348656, SGI0, SGI0, SGI0, SGI0, SGI348657, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82700[] = {SGI348658, SGI0, SGI0, SGI348659, SGI0, SGI0, SGI0, SGI0, SGI348660, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82701[] = {SGI348661, SGI0, SGI0, SGI348662, SGI0, SGI0, SGI0, SGI0, SGI348663, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82702[] = {SGI348664, SGI0, SGI0, SGI348665, SGI0, SGI0, SGI0, SGI0, SGI348666, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82703[] = {SGI348667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348668, SGI0, NULL};
const Segment * const SG82704[] = {SGI348669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348670, SGI0, NULL};
const Segment * const SG82705[] = {SGI348671, SGI348672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82706[] = {SGI348673, SGI348674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82707[] = {SGI348675, SGI348676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82708[] = {SGI348677, SGI348678, SGI348679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348679, SGI0, NULL};
const Segment * const SG82709[] = {SGI348680, SGI348681, SGI348682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348683, SGI0, NULL};
const Segment * const SG82710[] = {SGI348684, SGI348685, SGI348686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348687, SGI0, NULL};
const Segment * const SG82711[] = {SGI348688, SGI348689, SGI348690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348691, SGI0, NULL};
const Segment * const SG82712[] = {SGI348692, SGI348693, SGI348694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348695, SGI0, NULL};
const Segment * const SG82713[] = {SGI348696, SGI348697, SGI348698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348699, SGI0, NULL};
const Segment * const SG82714[] = {SGI348700, SGI348701, SGI348702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348703, SGI0, NULL};
const Segment * const SG82715[] = {SGI348704, SGI348705, SGI348706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348707, SGI0, NULL};
const Segment * const SG82716[] = {SGI348708, SGI348709, SGI348710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348710, SGI0, NULL};
const Segment * const SG82717[] = {SGI348711, SGI348712, SGI348713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348714, SGI0, NULL};
const Segment * const SG82718[] = {SGI348715, SGI348716, SGI348717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348718, SGI0, NULL};
const Segment * const SG82719[] = {SGI348719, SGI348720, SGI348721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348722, SGI0, NULL};
const Segment * const SG82720[] = {SGI348723, SGI348724, SGI348725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348726, SGI0, NULL};
const Segment * const SG82721[] = {SGI348727, SGI348728, SGI348729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348730, SGI0, NULL};
const Segment * const SG82722[] = {SGI348731, SGI348732, SGI348733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348734, SGI0, NULL};
const Segment * const SG82723[] = {SGI348735, SGI348736, SGI348737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348738, SGI0, NULL};
const Segment * const SG82724[] = {SGI348739, SGI348740, SGI348741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348742, SGI0, NULL};
const Segment * const SG82725[] = {SGI348743, SGI348744, SGI348745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348746, SGI0, NULL};
const Segment * const SG82726[] = {SGI348747, SGI348748, SGI348749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348750, SGI0, NULL};
const Segment * const SG82727[] = {SGI348751, SGI348752, SGI348753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348754, SGI0, NULL};
const Segment * const SG82728[] = {SGI348755, SGI348756, SGI348757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348758, SGI0, NULL};
const Segment * const SG82729[] = {SGI348759, SGI348760, SGI348761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348762, SGI0, NULL};
const Segment * const SG82730[] = {SGI348763, SGI348764, SGI348765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348766, SGI0, NULL};
const Segment * const SG82731[] = {SGI348767, SGI348768, SGI348769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348770, SGI0, NULL};
const Segment * const SG82732[] = {SGI348771, SGI348772, SGI348773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348774, SGI0, NULL};
const Segment * const SG82733[] = {SGI348775, SGI348776, SGI348777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348778, SGI0, NULL};
const Segment * const SG82734[] = {SGI348779, SGI348780, SGI348781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348782, SGI0, NULL};
const Segment * const SG82735[] = {SGI348783, SGI348784, SGI348785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348786, SGI0, NULL};
const Segment * const SG82736[] = {SGI348787, SGI348788, SGI348789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348790, SGI0, NULL};
const Segment * const SG82737[] = {SGI348791, SGI348792, SGI348793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348794, SGI0, NULL};
const Segment * const SG82738[] = {SGI348795, SGI348796, SGI348797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348798, SGI0, NULL};
const Segment * const SG82739[] = {SGI348799, SGI348800, SGI348801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348802, SGI0, NULL};
const Segment * const SG82740[] = {SGI348803, SGI348804, SGI348805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348806, SGI0, NULL};
const Segment * const SG82741[] = {SGI348807, SGI348808, SGI348809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348810, SGI0, NULL};
const Segment * const SG82742[] = {SGI348811, SGI348812, SGI348813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348814, SGI0, NULL};
const Segment * const SG82743[] = {SGI348815, SGI348816, SGI348817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348818, SGI0, NULL};
const Segment * const SG82744[] = {SGI348819, SGI348820, SGI348821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348822, SGI0, NULL};
const Segment * const SG82745[] = {SGI348823, SGI348824, SGI348825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348826, SGI0, NULL};
const Segment * const SG82746[] = {SGI348827, SGI348828, SGI348829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348830, SGI0, NULL};
const Segment * const SG82747[] = {SGI348831, SGI348832, SGI348833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348834, SGI0, NULL};
const Segment * const SG82748[] = {SGI348835, SGI348836, SGI348837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348838, SGI0, NULL};
const Segment * const SG82749[] = {SGI348839, SGI348840, SGI348841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348842, SGI0, NULL};
const Segment * const SG82750[] = {SGI348843, SGI348844, SGI348845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348846, SGI0, NULL};
const Segment * const SG82751[] = {SGI348847, SGI348848, SGI348849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348850, SGI0, NULL};
const Segment * const SG82752[] = {SGI348851, SGI0, SGI348852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82753[] = {SGI348853, SGI348854, SGI348855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348856, SGI0, NULL};
const Segment * const SG82754[] = {SGI348857, SGI348858, SGI348859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348860, SGI0, NULL};
const Segment * const SG82755[] = {SGI348861, SGI348862, SGI348863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348864, SGI0, NULL};
const Segment * const SG82756[] = {SGI348865, SGI348866, SGI348867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348868, SGI0, NULL};
const Segment * const SG82757[] = {SGI348869, SGI348870, SGI348871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348872, SGI0, NULL};
const Segment * const SG82758[] = {SGI348873, SGI348874, SGI348875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348876, SGI0, NULL};
const Segment * const SG82759[] = {SGI348877, SGI348878, SGI348879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348880, SGI0, NULL};
const Segment * const SG82760[] = {SGI348881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348883, SGI0, NULL};
const Segment * const SG82761[] = {SGI348882, SGI348884, SGI348885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348886, SGI0, NULL};
const Segment * const SG82762[] = {SGI348887, SGI0, SGI0, SGI348889, SGI0, SGI0, SGI0, SGI0, SGI348892, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82763[] = {SGI348888, SGI348890, SGI348891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348893, SGI0, NULL};
const Segment * const SG82764[] = {SGI348894, SGI348895, SGI348896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348897, SGI0, NULL};
const Segment * const SG82765[] = {SGI348898, SGI348899, SGI348900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348901, SGI0, NULL};
const Segment * const SG82766[] = {SGI348902, SGI348903, SGI348904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348905, SGI0, NULL};
const Segment * const SG82767[] = {SGI348906, SGI348907, SGI348908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348909, SGI0, NULL};
const Segment * const SG82768[] = {SGI348910, SGI348911, SGI348912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348913, SGI0, NULL};
const Segment * const SG82769[] = {SGI348914, SGI348915, SGI348916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348917, SGI0, NULL};
const Segment * const SG82770[] = {SGI348918, SGI348919, SGI348920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348921, SGI0, NULL};
const Segment * const SG82771[] = {SGI348922, SGI348923, SGI348924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348925, SGI0, NULL};
const Segment * const SG82772[] = {SGI348926, SGI348927, SGI348928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348929, SGI0, NULL};
const Segment * const SG82773[] = {SGI348930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348931, SGI0, NULL};
const Segment * const SG82774[] = {SGI348932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348933, SGI0, NULL};
const Segment * const SG82775[] = {SGI348934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348935, SGI0, NULL};
const Segment * const SG82776[] = {SGI348936, SGI0, SGI348937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82777[] = {SGI348938, SGI0, SGI0, SGI348939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82778[] = {SGI348940, SGI0, SGI0, SGI348941, SGI0, SGI0, SGI0, SGI0, SGI348942, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82779[] = {SGI348943, SGI348944, SGI348945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348946, SGI0, NULL};
const Segment * const SG82780[] = {SGI348947, SGI348948, SGI348949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348949, SGI0, NULL};
const Segment * const SG82781[] = {SGI348950, SGI348951, SGI348952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348953, SGI0, NULL};
const Segment * const SG82782[] = {SGI348954, SGI348955, SGI348956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348957, SGI0, NULL};
const Segment * const SG82783[] = {SGI348958, SGI348959, SGI348960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348961, SGI0, NULL};
const Segment * const SG82784[] = {SGI348962, SGI348963, SGI348964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348965, SGI0, NULL};
const Segment * const SG82785[] = {SGI348966, SGI348967, SGI348968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348969, SGI0, NULL};
const Segment * const SG82786[] = {SGI348970, SGI0, SGI0, SGI348971, SGI0, SGI0, SGI0, SGI0, SGI348972, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82787[] = {SGI348973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82788[] = {SGI348974, SGI0, SGI0, SGI348975, SGI0, SGI0, SGI0, SGI0, SGI348976, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82789[] = {SGI348977, SGI0, SGI348978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348978, SGI0, NULL};
const Segment * const SG82790[] = {SGI348979, SGI0, SGI348980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348980, SGI0, NULL};
const Segment * const SG82791[] = {SGI348981, SGI0, SGI348982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348982, SGI0, NULL};
const Segment * const SG82792[] = {SGI348983, SGI0, SGI348984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348984, SGI0, NULL};
const Segment * const SG82793[] = {SGI348985, SGI0, SGI348986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348987, SGI0, NULL};
const Segment * const SG82794[] = {SGI348988, SGI0, SGI348989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348989, SGI0, NULL};
const Segment * const SG82795[] = {SGI348990, SGI0, SGI348991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348992, SGI0, NULL};
const Segment * const SG82796[] = {SGI348993, SGI0, SGI348994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82797[] = {SGI348995, SGI0, SGI348996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82798[] = {SGI348997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI348998, SGI0, NULL};
const Segment * const SG82799[] = {SGI348999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349000, SGI0, NULL};
const Segment * const SG82800[] = {SGI349001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349002, SGI0, NULL};
const Segment * const SG82801[] = {SGI349003, SGI0, SGI0, SGI349004, SGI0, SGI0, SGI0, SGI0, SGI349005, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82802[] = {SGI349006, SGI0, SGI0, SGI349007, SGI0, SGI0, SGI0, SGI0, SGI349008, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82803[] = {SGI349009, SGI0, SGI0, SGI349010, SGI0, SGI0, SGI0, SGI0, SGI349011, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82804[] = {SGI349012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349013, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82805[] = {SGI349014, SGI0, SGI349015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82806[] = {SGI349016, SGI0, SGI0, SGI349017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82807[] = {SGI349018, SGI349019, SGI0, SGI0, NULL};
const Segment * const SG82808[] = {SGI349020, SGI0, SGI0, SGI349021, SGI0, SGI0, SGI0, SGI0, SGI349022, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82809[] = {SGI349023, SGI0, SGI0, SGI349024, SGI0, SGI0, SGI0, SGI0, SGI349025, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82810[] = {SGI349026, SGI349027, SGI349028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349028, SGI0, NULL};
const Segment * const SG82811[] = {SGI349029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349030, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82812[] = {SGI349031, SGI0, SGI0, SGI349032, SGI0, SGI0, SGI0, SGI0, SGI349033, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82813[] = {SGI349034, SGI0, SGI0, SGI349035, SGI0, SGI0, SGI0, SGI0, SGI349036, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82814[] = {SGI349037, SGI0, SGI0, SGI349038, SGI0, SGI0, SGI0, SGI0, SGI349039, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82815[] = {SGI349040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82816[] = {SGI349041, SGI0, SGI0, SGI349042, SGI0, SGI0, SGI0, SGI0, SGI349043, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82817[] = {SGI349044, SGI0, SGI349045, SGI349045, SGI0, SGI0, SGI0, SGI0, SGI349045, SGI0, NULL};
const Segment * const SG82818[] = {SGI349046, SGI0, SGI349047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349047, SGI0, NULL};
const Segment * const SG82819[] = {SGI349048, SGI0, SGI0, SGI349049, SGI0, SGI0, SGI0, SGI0, SGI349050, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82820[] = {SGI349051, SGI0, SGI0, SGI349052, SGI0, SGI0, SGI0, SGI0, SGI349053, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82821[] = {SGI349054, SGI0, SGI0, SGI349055, SGI0, SGI0, SGI0, SGI0, SGI349056, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82822[] = {SGI349057, SGI0, SGI0, SGI349058, SGI0, SGI0, SGI0, SGI0, SGI349059, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82823[] = {SGI349060, SGI349061, SGI349062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349062, SGI0, NULL};
const Segment * const SG82824[] = {SGI349063, SGI349064, SGI349065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349065, SGI0, NULL};
const Segment * const SG82825[] = {SGI349066, SGI349067, SGI349068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349068, SGI0, NULL};
const Segment * const SG82826[] = {SGI349069, SGI349070, SGI349071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349071, SGI0, NULL};
const Segment * const SG82827[] = {SGI349072, SGI349073, SGI349074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349075, SGI0, NULL};
const Segment * const SG82828[] = {SGI349076, SGI349077, SGI349078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349079, SGI0, NULL};
const Segment * const SG82829[] = {SGI349080, SGI349081, SGI349082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349083, SGI0, NULL};
const Segment * const SG82830[] = {SGI349084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82831[] = {SGI349085, SGI349086, SGI349087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349088, SGI0, NULL};
const Segment * const SG82832[] = {SGI349089, SGI349090, SGI349091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349092, SGI0, NULL};
const Segment * const SG82833[] = {SGI349093, SGI0, SGI0, SGI349094, SGI0, SGI0, SGI0, SGI0, SGI349095, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82834[] = {SGI349096, SGI0, SGI349097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82835[] = {SGI349098, SGI0, SGI349099, SGI349099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82836[] = {SGI349100, SGI0, SGI349101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82837[] = {SGI349102, SGI0, SGI349103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82838[] = {SGI349104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349105, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82839[] = {SGI349106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349107, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82840[] = {SGI349108, SGI0, SGI0, SGI349109, SGI0, SGI0, SGI0, SGI0, SGI349110, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82841[] = {SGI349111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349112, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82842[] = {SGI349113, SGI0, SGI0, SGI349114, SGI0, SGI0, SGI0, SGI0, SGI349115, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82843[] = {SGI349116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349117, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82844[] = {SGI349118, SGI0, SGI0, SGI349119, SGI0, SGI0, SGI0, SGI0, SGI349120, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82845[] = {SGI349121, SGI0, SGI0, SGI349122, SGI0, SGI0, SGI0, SGI0, SGI349123, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82846[] = {SGI349124, SGI0, SGI0, SGI349125, SGI0, SGI0, SGI0, SGI0, SGI349126, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82847[] = {SGI349127, SGI349128, SGI349129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82848[] = {SGI349130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82849[] = {SGI349131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82850[] = {SGI349132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349133, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82851[] = {SGI349134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349135, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82852[] = {SGI349136, SGI0, SGI0, SGI349137, SGI0, SGI0, SGI0, SGI0, SGI349138, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82853[] = {SGI349139, SGI0, SGI0, SGI349140, SGI0, SGI0, SGI0, SGI0, SGI349141, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82854[] = {SGI349142, SGI0, SGI0, SGI349143, SGI0, SGI0, SGI0, SGI0, SGI349144, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82855[] = {SGI349145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349146, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82856[] = {SGI349147, SGI0, SGI0, SGI349148, SGI0, SGI0, SGI0, SGI0, SGI349149, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82857[] = {SGI349150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82858[] = {SGI349151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82859[] = {SGI349152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82860[] = {SGI349153, SGI0, SGI0, SGI349154, SGI0, SGI0, SGI0, SGI0, SGI349155, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82861[] = {SGI349156, SGI0, SGI0, SGI349157, SGI0, SGI0, SGI0, SGI0, SGI349158, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82862[] = {SGI349159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349160, SGI0, NULL};
const Segment * const SG82863[] = {SGI349161, SGI0, SGI349162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82864[] = {SGI349163, SGI349164, SGI0, SGI0, NULL};
const Segment * const SG82865[] = {SGI349165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349166, SGI0, NULL};
const Segment * const SG82866[] = {SGI349167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349168, SGI0, NULL};
const Segment * const SG82867[] = {SGI349169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349170, SGI0, NULL};
const Segment * const SG82868[] = {SGI349171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349172, SGI0, NULL};
const Segment * const SG82869[] = {SGI349173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82870[] = {SGI349174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82871[] = {SGI349175, SGI349177, SGI0, SGI0, NULL};
const Segment * const SG82872[] = {SGI349176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349178, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82873[] = {SGI349179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349180, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82874[] = {SGI349181, SGI0, SGI0, SGI349182, SGI0, SGI0, SGI0, SGI0, SGI349183, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82875[] = {SGI349184, SGI0, SGI0, SGI349185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82876[] = {SGI349186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82877[] = {SGI349187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349188, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82878[] = {SGI349189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349190, SGI0, NULL};
const Segment * const SG82879[] = {SGI349191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349192, SGI0, NULL};
const Segment * const SG82880[] = {SGI349193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349194, SGI0, NULL};
const Segment * const SG82881[] = {SGI349195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349196, SGI0, NULL};
const Segment * const SG82882[] = {SGI349197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349198, SGI0, NULL};
const Segment * const SG82883[] = {SGI349199, SGI0, SGI0, SGI349200, SGI0, SGI0, SGI0, SGI0, SGI349201, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82884[] = {SGI349202, SGI0, SGI0, SGI349203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82885[] = {SGI349204, SGI0, SGI0, SGI349205, SGI0, SGI0, SGI0, SGI0, SGI349206, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82886[] = {SGI349207, SGI0, SGI0, SGI349208, SGI0, SGI0, SGI0, SGI0, SGI349209, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82887[] = {SGI349210, SGI0, SGI0, SGI349211, SGI0, SGI0, SGI0, SGI0, SGI349212, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82888[] = {SGI349213, SGI0, SGI0, SGI349214, SGI0, SGI0, SGI0, SGI0, SGI349215, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82889[] = {SGI349216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349217, SGI0, NULL};
const Segment * const SG82890[] = {SGI349218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349219, SGI0, NULL};
const Segment * const SG82891[] = {SGI349220, SGI349221, SGI0, SGI0, NULL};
const Segment * const SG82892[] = {SGI349222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82893[] = {SGI349223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349224, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82894[] = {SGI349225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349226, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82895[] = {SGI349227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349228, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82896[] = {SGI349229, SGI0, SGI0, SGI349230, SGI0, SGI0, SGI0, SGI0, SGI349231, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82897[] = {SGI349232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349233, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82898[] = {SGI349234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82899[] = {SGI349235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82900[] = {SGI349236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82901[] = {SGI349237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82902[] = {SGI349238, SGI349239, SGI0, SGI0, NULL};
const Segment * const SG82903[] = {SGI349240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349241, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82904[] = {SGI349242, SGI0, SGI0, SGI349243, SGI0, SGI0, SGI0, SGI0, SGI349244, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82905[] = {SGI349245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349246, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82906[] = {SGI349247, SGI0, SGI0, SGI349248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82907[] = {SGI349249, SGI349250, SGI349251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349251, SGI0, NULL};
const Segment * const SG82908[] = {SGI349252, SGI349253, SGI349254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349254, SGI0, NULL};
const Segment * const SG82909[] = {SGI349255, SGI349256, SGI349257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349257, SGI0, NULL};
const Segment * const SG82910[] = {SGI349258, SGI349259, SGI349260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349260, SGI0, NULL};
const Segment * const SG82911[] = {SGI349261, SGI349262, SGI349263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349263, SGI0, NULL};
const Segment * const SG82912[] = {SGI349264, SGI349265, SGI349266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349267, SGI0, NULL};
const Segment * const SG82913[] = {SGI349268, SGI349269, SGI349270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349271, SGI0, NULL};
const Segment * const SG82914[] = {SGI349272, SGI349273, SGI349274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349275, SGI0, NULL};
const Segment * const SG82915[] = {SGI349276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82916[] = {SGI349277, SGI349278, SGI349279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349280, SGI0, NULL};
const Segment * const SG82917[] = {SGI349281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82918[] = {SGI349282, SGI349283, SGI349284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349285, SGI0, NULL};
const Segment * const SG82919[] = {SGI349286, SGI349287, SGI349288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349289, SGI0, NULL};
const Segment * const SG82920[] = {SGI349290, SGI349291, SGI349292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349293, SGI0, NULL};
const Segment * const SG82921[] = {SGI349294, SGI349295, SGI349296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349297, SGI0, NULL};
const Segment * const SG82922[] = {SGI349298, SGI349299, SGI349300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349301, SGI0, NULL};
const Segment * const SG82923[] = {SGI349302, SGI0, SGI349303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82924[] = {SGI349304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349305, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82925[] = {SGI349306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349307, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82926[] = {SGI349308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82927[] = {SGI349309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82928[] = {SGI349310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349311, SGI0, NULL};
const Segment * const SG82929[] = {SGI349312, SGI0, SGI0, SGI349313, SGI0, SGI0, SGI0, SGI0, SGI349314, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82930[] = {SGI349315, SGI349316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82931[] = {SGI349317, SGI349318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82932[] = {SGI349319, SGI0, SGI0, SGI349320, SGI0, SGI0, SGI0, SGI0, SGI349321, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82933[] = {SGI349322, SGI0, SGI0, SGI349323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82934[] = {SGI349324, SGI0, SGI0, SGI349325, SGI0, SGI0, SGI0, SGI0, SGI349326, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82935[] = {SGI349327, SGI0, SGI0, SGI349328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82936[] = {SGI349329, SGI0, SGI0, SGI349330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82937[] = {SGI349331, SGI0, SGI0, SGI349332, SGI0, SGI0, SGI0, SGI0, SGI349333, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82938[] = {SGI349334, SGI0, SGI0, SGI349335, SGI0, SGI0, SGI0, SGI0, SGI349336, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82939[] = {SGI349337, SGI0, SGI0, SGI349338, SGI0, SGI0, SGI0, SGI0, SGI349339, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82940[] = {SGI349340, SGI0, SGI0, SGI349341, SGI0, SGI0, SGI0, SGI0, SGI349342, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82941[] = {SGI349343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349344, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82942[] = {SGI349345, SGI0, SGI0, SGI349346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82943[] = {SGI349347, SGI0, SGI349348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82944[] = {SGI349349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349350, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82945[] = {SGI349351, SGI0, SGI0, SGI349352, SGI0, SGI0, SGI0, SGI0, SGI349353, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82946[] = {SGI349354, SGI349355, SGI349356, SGI0, SGI0, SGI0, SGI0, SGI349357, SGI0, SGI0, NULL};
const Segment * const SG82947[] = {SGI349358, SGI349359, SGI349360, SGI0, SGI0, SGI0, SGI0, SGI349361, SGI0, SGI0, NULL};
const Segment * const SG82948[] = {SGI349362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349363, SGI0, NULL};
const Segment * const SG82949[] = {SGI349364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349365, SGI0, NULL};
const Segment * const SG82950[] = {SGI349366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349367, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82951[] = {SGI349368, SGI0, SGI0, SGI349369, SGI0, SGI0, SGI0, SGI0, SGI349370, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82952[] = {SGI349371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349372, SGI0, NULL};
const Segment * const SG82953[] = {SGI349373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82954[] = {SGI349374, SGI0, SGI349375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82955[] = {SGI349376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349377, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82956[] = {SGI349378, SGI0, SGI0, SGI349379, SGI0, SGI0, SGI0, SGI0, SGI349380, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82957[] = {SGI349381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349382, SGI0, NULL};
const Segment * const SG82958[] = {SGI349383, SGI0, SGI349384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349384, SGI0, NULL};
const Segment * const SG82959[] = {SGI349385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349386, SGI0, NULL};
const Segment * const SG82960[] = {SGI349387, SGI0, SGI0, SGI349388, SGI0, SGI0, SGI0, SGI0, SGI349389, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82961[] = {SGI349390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349391, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82962[] = {SGI349392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349393, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82963[] = {SGI349394, SGI0, SGI0, SGI349395, SGI0, SGI0, SGI0, SGI0, SGI349396, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82964[] = {SGI349397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349398, SGI0, NULL};
const Segment * const SG82965[] = {SGI349399, SGI349400, SGI349401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82966[] = {SGI349402, SGI349403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82967[] = {SGI349404, SGI349405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82968[] = {SGI349406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349407, SGI0, NULL};
const Segment * const SG82969[] = {SGI349408, SGI0, SGI0, SGI349409, SGI0, SGI0, SGI0, SGI0, SGI349412, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82970[] = {SGI349410, SGI349411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82971[] = {SGI349413, SGI349414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82972[] = {SGI349415, SGI349416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82973[] = {SGI349417, SGI0, SGI349418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82974[] = {SGI349420, SGI0, SGI0, SGI349421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82975[] = {SGI349419, SGI349422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82976[] = {SGI349423, SGI349424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82977[] = {SGI349425, SGI349426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82978[] = {SGI349427, SGI349428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82979[] = {SGI349429, SGI349430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82980[] = {SGI349431, SGI349432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82981[] = {SGI349433, SGI349434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82982[] = {SGI349435, SGI0, SGI0, SGI349436, SGI0, SGI0, SGI0, SGI0, SGI349437, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82983[] = {SGI349438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82984[] = {SGI349439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82985[] = {SGI349440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349441, SGI0, NULL};
const Segment * const SG82986[] = {SGI349442, SGI0, SGI0, SGI349443, SGI0, SGI0, SGI0, SGI0, SGI349444, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82987[] = {SGI349445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82988[] = {SGI349446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82989[] = {SGI349447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82990[] = {SGI349448, SGI0, SGI0, SGI349449, SGI0, SGI0, SGI0, SGI0, SGI349450, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82991[] = {SGI349451, SGI0, SGI0, SGI349452, SGI0, SGI0, SGI0, SGI0, SGI349453, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82992[] = {SGI349454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349455, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82993[] = {SGI349456, SGI0, SGI0, SGI349457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82994[] = {SGI349458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349459, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82995[] = {SGI349460, SGI0, SGI0, SGI349461, SGI0, SGI0, SGI0, SGI0, SGI349462, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82996[] = {SGI349463, SGI0, SGI0, SGI349464, SGI0, SGI0, SGI0, SGI0, SGI349465, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82997[] = {SGI349466, SGI0, SGI0, SGI349467, SGI0, SGI0, SGI0, SGI0, SGI349468, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82998[] = {SGI349469, SGI0, SGI0, SGI349470, SGI0, SGI0, SGI0, SGI0, SGI349471, SGI0, SGI0, SGI0, NULL};
const Segment * const SG82999[] = {SGI349472, SGI0, SGI0, SGI349473, SGI0, SGI0, SGI0, SGI0, SGI349474, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83000[] = {SGI349475, SGI0, SGI0, SGI349476, SGI0, SGI0, SGI0, SGI0, SGI349477, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83001[] = {SGI349478, SGI349479, SGI349480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349480, SGI0, NULL};
const Segment * const SG83002[] = {SGI349481, SGI349482, SGI349483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349484, SGI0, NULL};
const Segment * const SG83003[] = {SGI349485, SGI349486, SGI349487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349488, SGI0, NULL};
const Segment * const SG83004[] = {SGI349489, SGI0, SGI0, SGI349490, SGI0, SGI0, SGI0, SGI0, SGI349491, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83005[] = {SGI349492, SGI349493, SGI349494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349495, SGI0, NULL};
const Segment * const SG83006[] = {SGI349496, SGI349497, SGI349498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349499, SGI0, NULL};
const Segment * const SG83007[] = {SGI349500, SGI349501, SGI349502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349503, SGI0, NULL};
const Segment * const SG83008[] = {SGI349504, SGI349505, SGI349506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349507, SGI0, NULL};
const Segment * const SG83009[] = {SGI349508, SGI349509, SGI349510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349511, SGI0, NULL};
const Segment * const SG83010[] = {SGI349512, SGI349513, SGI349514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349515, SGI0, NULL};
const Segment * const SG83011[] = {SGI349516, SGI349517, SGI349518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349519, SGI0, NULL};
const Segment * const SG83012[] = {SGI349520, SGI0, SGI0, SGI349521, SGI0, SGI0, SGI0, SGI0, SGI349522, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83013[] = {SGI349523, SGI349524, SGI349525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349526, SGI0, NULL};
const Segment * const SG83014[] = {SGI349527, SGI349528, SGI349529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349530, SGI0, NULL};
const Segment * const SG83015[] = {SGI349531, SGI349532, SGI0, SGI0, NULL};
const Segment * const SG83016[] = {SGI349533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349534, SGI0, NULL};
const Segment * const SG83017[] = {SGI349535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349536, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83018[] = {SGI349537, SGI0, SGI0, SGI349538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83019[] = {SGI349539, SGI0, SGI349540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83020[] = {SGI349541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349542, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83021[] = {SGI349543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349544, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83022[] = {SGI349545, SGI0, SGI0, SGI349546, SGI0, SGI0, SGI0, SGI0, SGI349547, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83023[] = {SGI349548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349549, SGI0, NULL};
const Segment * const SG83024[] = {SGI349550, SGI0, SGI349551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83025[] = {SGI349552, SGI0, SGI349553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83026[] = {SGI349554, SGI0, SGI349555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83027[] = {SGI349556, SGI0, SGI0, SGI349557, SGI0, SGI0, SGI0, SGI0, SGI349558, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83028[] = {SGI349559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83029[] = {SGI349560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83030[] = {SGI349561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349562, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83031[] = {SGI349563, SGI349564, SGI349565, SGI0, SGI0, SGI0, SGI0, SGI349566, SGI0, SGI0, NULL};
const Segment * const SG83032[] = {SGI349567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83033[] = {SGI349568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83034[] = {SGI349569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83035[] = {SGI349570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349571, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83036[] = {SGI349572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349573, SGI0, NULL};
const Segment * const SG83037[] = {SGI349574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349575, SGI0, NULL};
const Segment * const SG83038[] = {SGI349576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349577, SGI0, NULL};
const Segment * const SG83039[] = {SGI349578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349579, SGI0, NULL};
const Segment * const SG83040[] = {SGI349580, SGI0, SGI0, SGI349581, SGI0, SGI0, SGI0, SGI0, SGI349582, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83041[] = {SGI349583, SGI0, SGI349584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349584, SGI0, NULL};
const Segment * const SG83042[] = {SGI349585, SGI0, SGI349586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349586, SGI0, NULL};
const Segment * const SG83043[] = {SGI349587, SGI0, SGI349588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349588, SGI0, NULL};
const Segment * const SG83044[] = {SGI349589, SGI0, SGI349590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349590, SGI0, NULL};
const Segment * const SG83045[] = {SGI349591, SGI0, SGI349592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349592, SGI0, NULL};
const Segment * const SG83046[] = {SGI349593, SGI0, SGI349594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349595, SGI0, NULL};
const Segment * const SG83047[] = {SGI349596, SGI0, SGI0, SGI349597, SGI0, SGI0, SGI0, SGI0, SGI349598, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83048[] = {SGI349599, SGI349600, SGI349601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349601, SGI0, NULL};
const Segment * const SG83049[] = {SGI349602, SGI349603, SGI349604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349604, SGI0, NULL};
const Segment * const SG83050[] = {SGI349605, SGI0, SGI349606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349606, SGI0, NULL};
const Segment * const SG83051[] = {SGI349607, SGI349608, SGI349609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349610, SGI0, NULL};
const Segment * const SG83052[] = {SGI349611, SGI349612, SGI349613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349614, SGI0, NULL};
const Segment * const SG83053[] = {SGI349615, SGI349616, SGI349617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349618, SGI0, NULL};
const Segment * const SG83054[] = {SGI349619, SGI349620, SGI349621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349623, SGI0, NULL};
const Segment * const SG83055[] = {SGI349622, SGI0, SGI349624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83056[] = {SGI349626, SGI0, SGI0, SGI349628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83057[] = {SGI349625, SGI349627, SGI349629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349630, SGI0, NULL};
const Segment * const SG83058[] = {SGI349631, SGI0, SGI349632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83059[] = {SGI349633, SGI349634, SGI349635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349636, SGI0, NULL};
const Segment * const SG83060[] = {SGI349637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349638, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83061[] = {SGI349639, SGI349640, SGI349641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349642, SGI0, NULL};
const Segment * const SG83062[] = {SGI349643, SGI349644, SGI349645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349646, SGI0, NULL};
const Segment * const SG83063[] = {SGI349647, SGI349648, SGI349649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349650, SGI0, NULL};
const Segment * const SG83064[] = {SGI349651, SGI349652, SGI349653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349654, SGI0, NULL};
const Segment * const SG83065[] = {SGI349655, SGI349655, SGI349656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349656, SGI0, NULL};
const Segment * const SG83066[] = {SGI349657, SGI349658, SGI349659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349660, SGI0, NULL};
const Segment * const SG83067[] = {SGI349661, SGI349662, SGI349663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349664, SGI0, NULL};
const Segment * const SG83068[] = {SGI349665, SGI349666, SGI349667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349668, SGI0, NULL};
const Segment * const SG83069[] = {SGI349669, SGI349670, SGI349671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349672, SGI0, NULL};
const Segment * const SG83070[] = {SGI349673, SGI0, SGI0, SGI349674, SGI0, SGI0, SGI0, SGI0, SGI349675, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83071[] = {SGI349676, SGI0, SGI0, SGI349677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83072[] = {SGI349678, SGI0, SGI349679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83073[] = {SGI349680, SGI0, SGI0, SGI349681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83074[] = {SGI349682, SGI0, SGI349683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83075[] = {SGI349684, SGI0, SGI0, SGI349685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83076[] = {SGI349686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349687, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83077[] = {SGI349688, SGI0, SGI349689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83078[] = {SGI349690, SGI0, SGI0, SGI349691, SGI0, SGI0, SGI0, SGI0, SGI349692, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83079[] = {SGI349693, SGI0, SGI0, SGI349694, SGI0, SGI0, SGI0, SGI0, SGI349695, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83080[] = {SGI349696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349697, SGI0, NULL};
const Segment * const SG83081[] = {SGI349698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349699, SGI0, NULL};
const Segment * const SG83082[] = {SGI349700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349701, SGI0, NULL};
const Segment * const SG83083[] = {SGI349702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349705, SGI0, NULL};
const Segment * const SG83084[] = {SGI349703, SGI0, SGI349704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83085[] = {SGI349706, SGI0, SGI0, SGI349707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83086[] = {SGI349708, SGI0, SGI349709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83087[] = {SGI349710, SGI0, SGI0, SGI349711, SGI0, SGI0, SGI0, SGI0, SGI349712, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83088[] = {SGI349713, SGI0, SGI0, SGI349714, SGI0, SGI0, SGI0, SGI0, SGI349715, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83089[] = {SGI349716, SGI0, SGI0, SGI349717, SGI0, SGI0, SGI0, SGI0, SGI349718, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83090[] = {SGI349719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349720, SGI0, NULL};
const Segment * const SG83091[] = {SGI349721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349722, SGI0, NULL};
const Segment * const SG83092[] = {SGI349723, SGI349724, SGI0, SGI0, NULL};
const Segment * const SG83093[] = {SGI349725, SGI0, SGI349726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83094[] = {SGI349727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349728, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83095[] = {SGI349729, SGI349730, SGI0, SGI349731, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83096[] = {SGI349732, SGI349733, SGI0, SGI0, SGI349734, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83097[] = {SGI349735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349736, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83098[] = {SGI349737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83099[] = {SGI349738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83100[] = {SGI349739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83101[] = {SGI349740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349741, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83102[] = {SGI349742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349743, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83103[] = {SGI349744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349745, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83104[] = {SGI349746, SGI0, SGI0, SGI349747, SGI0, SGI0, SGI0, SGI0, SGI349748, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83105[] = {SGI349749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349750, SGI0, NULL};
const Segment * const SG83106[] = {SGI349751, SGI0, SGI349752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83107[] = {SGI349753, SGI0, SGI0, SGI349754, SGI0, SGI0, SGI0, SGI0, SGI349755, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83108[] = {SGI349756, SGI0, SGI0, SGI349757, SGI0, SGI0, SGI0, SGI0, SGI349758, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83109[] = {SGI349759, SGI0, SGI349760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83110[] = {SGI349761, SGI0, SGI0, SGI349762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83111[] = {SGI349763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349764, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83112[] = {SGI349765, SGI0, SGI349766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83113[] = {SGI349767, SGI0, SGI0, SGI349768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83114[] = {SGI349769, SGI0, SGI0, SGI349770, SGI0, SGI0, SGI0, SGI0, SGI349771, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83115[] = {SGI349772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349773, SGI0, NULL};
const Segment * const SG83116[] = {SGI349774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83117[] = {SGI349775, SGI0, SGI349776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83118[] = {SGI349777, SGI0, SGI349778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83119[] = {SGI349779, SGI0, SGI349780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83120[] = {SGI349781, SGI0, SGI349782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83121[] = {SGI349783, SGI0, SGI349784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83122[] = {SGI349785, SGI0, SGI349786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83123[] = {SGI349787, SGI0, SGI349788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83124[] = {SGI349789, SGI349790, SGI0, SGI0, NULL};
const Segment * const SG83125[] = {SGI349791, SGI0, SGI0, SGI349792, SGI0, SGI0, SGI0, SGI0, SGI349793, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83126[] = {SGI349794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83127[] = {SGI349795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83128[] = {SGI349796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83129[] = {SGI349797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83130[] = {SGI349798, SGI349799, SGI349800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349800, SGI0, NULL};
const Segment * const SG83131[] = {SGI349801, SGI349802, SGI349803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349804, SGI0, NULL};
const Segment * const SG83132[] = {SGI349805, SGI349806, SGI349807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349808, SGI0, NULL};
const Segment * const SG83133[] = {SGI349809, SGI349810, SGI349811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349812, SGI0, NULL};
const Segment * const SG83134[] = {SGI349813, SGI349814, SGI349815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349816, SGI0, NULL};
const Segment * const SG83135[] = {SGI349817, SGI349818, SGI349819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349820, SGI0, NULL};
const Segment * const SG83136[] = {SGI349821, SGI349822, SGI349823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349824, SGI0, NULL};
const Segment * const SG83137[] = {SGI349825, SGI349826, SGI349827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349828, SGI0, NULL};
const Segment * const SG83138[] = {SGI349829, SGI349830, SGI349831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349832, SGI0, NULL};
const Segment * const SG83139[] = {SGI349833, SGI349834, SGI349835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349836, SGI0, NULL};
const Segment * const SG83140[] = {SGI349837, SGI349838, SGI349839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349840, SGI0, NULL};
const Segment * const SG83141[] = {SGI349841, SGI349842, SGI349843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349844, SGI0, NULL};
const Segment * const SG83142[] = {SGI349845, SGI349846, SGI349847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349848, SGI0, NULL};
const Segment * const SG83143[] = {SGI349849, SGI0, SGI0, SGI349850, SGI0, SGI0, SGI0, SGI0, SGI349852, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83144[] = {SGI349851, SGI349853, SGI349854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349855, SGI0, NULL};
const Segment * const SG83145[] = {SGI349856, SGI349857, SGI349858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349859, SGI0, NULL};
const Segment * const SG83146[] = {SGI349860, SGI349861, SGI349862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349863, SGI0, NULL};
const Segment * const SG83147[] = {SGI349864, SGI349865, SGI349866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349867, SGI0, NULL};
const Segment * const SG83148[] = {SGI349868, SGI349869, SGI349870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349871, SGI0, NULL};
const Segment * const SG83149[] = {SGI349872, SGI349873, SGI349874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349875, SGI0, NULL};
const Segment * const SG83150[] = {SGI349876, SGI349877, SGI349878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349879, SGI0, NULL};
const Segment * const SG83151[] = {SGI349880, SGI349881, SGI349882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349883, SGI0, NULL};
const Segment * const SG83152[] = {SGI349884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83153[] = {SGI349885, SGI349886, SGI349887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349888, SGI0, NULL};
const Segment * const SG83154[] = {SGI349889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83155[] = {SGI349890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349891, SGI0, NULL};
const Segment * const SG83156[] = {SGI349892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83157[] = {SGI349893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83158[] = {SGI349894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83159[] = {SGI349895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83160[] = {SGI349896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83161[] = {SGI349897, SGI349898, SGI349899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83162[] = {SGI349900, SGI349901, SGI349902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83163[] = {SGI349903, SGI349904, SGI349905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83164[] = {SGI349906, SGI0, SGI0, SGI349907, SGI0, SGI0, SGI0, SGI0, SGI349908, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83165[] = {SGI349909, SGI0, SGI0, SGI349910, SGI0, SGI0, SGI0, SGI0, SGI349911, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83166[] = {SGI349912, SGI0, SGI349913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83167[] = {SGI349914, SGI0, SGI0, SGI349915, SGI0, SGI0, SGI0, SGI0, SGI349916, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83168[] = {SGI349917, SGI0, SGI0, SGI349918, SGI0, SGI0, SGI0, SGI0, SGI349919, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83169[] = {SGI349920, SGI0, SGI0, SGI349921, SGI0, SGI0, SGI0, SGI0, SGI349922, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83170[] = {SGI349923, SGI0, SGI0, SGI349924, SGI0, SGI0, SGI0, SGI0, SGI349925, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83171[] = {SGI349926, SGI349927, SGI0, SGI0, NULL};
const Segment * const SG83172[] = {SGI349928, SGI0, SGI0, SGI349929, SGI0, SGI0, SGI0, SGI0, SGI349930, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83173[] = {SGI349931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349932, SGI0, NULL};
const Segment * const SG83174[] = {SGI349933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349934, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83175[] = {SGI349935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349936, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83176[] = {SGI349937, SGI0, SGI0, SGI349938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83177[] = {SGI349939, SGI0, SGI349940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83178[] = {SGI349941, SGI0, SGI0, SGI349942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83179[] = {SGI349943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349944, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83180[] = {SGI349945, SGI0, SGI0, SGI349946, SGI0, SGI0, SGI0, SGI0, SGI349947, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83181[] = {SGI349948, SGI349949, SGI0, SGI0, NULL};
const Segment * const SG83182[] = {SGI349950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349951, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83183[] = {SGI349952, SGI349953, SGI0, SGI0, NULL};
const Segment * const SG83184[] = {SGI349954, SGI349955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83185[] = {SGI349956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83186[] = {SGI349957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349958, SGI0, NULL};
const Segment * const SG83187[] = {SGI349959, SGI349961, SGI0, SGI0, NULL};
const Segment * const SG83188[] = {SGI349960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349962, SGI0, NULL};
const Segment * const SG83189[] = {SGI349963, SGI349964, SGI0, SGI0, NULL};
const Segment * const SG83190[] = {SGI349965, SGI349966, SGI0, SGI0, NULL};
const Segment * const SG83191[] = {SGI349967, SGI349968, SGI0, SGI0, NULL};
const Segment * const SG83192[] = {SGI349969, SGI349970, SGI0, SGI0, NULL};
const Segment * const SG83193[] = {SGI349971, SGI349972, SGI0, SGI0, NULL};
const Segment * const SG83194[] = {SGI349973, SGI349974, SGI0, SGI0, NULL};
const Segment * const SG83195[] = {SGI349975, SGI349976, SGI0, SGI0, NULL};
const Segment * const SG83196[] = {SGI349977, SGI349978, SGI0, SGI0, NULL};
const Segment * const SG83197[] = {SGI349979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349980, SGI0, NULL};
const Segment * const SG83198[] = {SGI349981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83199[] = {SGI349982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349983, SGI0, NULL};
const Segment * const SG83200[] = {SGI0, SGI349984, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83201[] = {SGI0, SGI349985, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83202[] = {SGI0, SGI349986, SGI0, SGI0, SGI0, SGI0, SGI349987, NULL};
const Segment * const SG83203[] = {SGI0, SGI349988, SGI0, SGI0, SGI0, SGI0, SGI349989, NULL};
const Segment * const SG83204[] = {SGI0, SGI349990, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83205[] = {SGI0, SGI349991, SGI0, SGI0, SGI0, SGI349992, SGI349993, NULL};
const Segment * const SG83206[] = {SGI349994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI349995, SGI0, NULL};
const Segment * const SG83207[] = {SGI349996, SGI0, SGI0, SGI349997, SGI0, SGI0, SGI0, SGI0, SGI349998, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83208[] = {SGI349999, SGI0, SGI0, SGI350000, SGI0, SGI0, SGI0, SGI0, SGI350001, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83209[] = {SGI350002, SGI0, SGI0, SGI350003, SGI0, SGI0, SGI0, SGI0, SGI350004, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83210[] = {SGI350005, SGI0, SGI0, SGI350006, SGI0, SGI0, SGI0, SGI0, SGI350007, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83211[] = {SGI350008, SGI0, SGI0, SGI350009, SGI0, SGI0, SGI0, SGI0, SGI350010, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83212[] = {SGI350011, SGI0, SGI0, SGI350012, SGI0, SGI0, SGI0, SGI0, SGI350013, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83213[] = {SGI350014, SGI0, SGI0, SGI350015, SGI0, SGI0, SGI0, SGI0, SGI350016, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83214[] = {SGI350017, SGI350018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83215[] = {SGI350019, SGI350020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83216[] = {SGI350021, SGI350022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83217[] = {SGI350023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350024, SGI0, NULL};
const Segment * const SG83218[] = {SGI350025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350026, SGI0, NULL};
const Segment * const SG83219[] = {SGI350027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350028, SGI0, NULL};
const Segment * const SG83220[] = {SGI0, SGI350029, SGI0, SGI0, SGI0, SGI0, SGI350030, NULL};
const Segment * const SG83221[] = {SGI0, SGI350031, SGI0, SGI0, SGI0, SGI350032, SGI350033, NULL};
const Segment * const SG83222[] = {SGI0, SGI350034, SGI0, SGI0, SGI0, SGI350035, SGI350036, NULL};
const Segment * const SG83223[] = {SGI0, SGI350037, SGI0, SGI0, SGI0, SGI350038, SGI350039, NULL};
const Segment * const SG83224[] = {SGI350040, SGI0, SGI0, SGI350041, SGI0, SGI0, SGI0, SGI0, SGI350042, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83225[] = {SGI350043, SGI0, SGI0, SGI350044, SGI0, SGI0, SGI0, SGI0, SGI350045, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83226[] = {SGI350046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350047, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83227[] = {SGI350048, SGI0, SGI350049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350049, SGI0, NULL};
const Segment * const SG83228[] = {SGI350050, SGI0, SGI0, SGI350051, SGI0, SGI0, SGI0, SGI0, SGI350052, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83229[] = {SGI350053, SGI0, SGI0, SGI350054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83230[] = {SGI350055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350056, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83231[] = {SGI350057, SGI350058, SGI350059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83232[] = {SGI350060, SGI350061, SGI350062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83233[] = {SGI350063, SGI0, SGI0, SGI350064, SGI0, SGI0, SGI0, SGI0, SGI350065, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83234[] = {SGI350066, SGI0, SGI0, SGI350067, SGI0, SGI0, SGI0, SGI0, SGI350068, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83235[] = {SGI350069, SGI0, SGI0, SGI350070, SGI0, SGI0, SGI0, SGI0, SGI350071, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83236[] = {SGI350072, SGI0, SGI0, SGI350073, SGI0, SGI0, SGI0, SGI0, SGI350074, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83237[] = {SGI350075, SGI0, SGI0, SGI350076, SGI0, SGI0, SGI0, SGI0, SGI350077, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83238[] = {SGI350078, SGI0, SGI0, SGI350079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83239[] = {SGI350080, SGI0, SGI350081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83240[] = {SGI350082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350083, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83241[] = {SGI350084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350085, SGI0, NULL};
const Segment * const SG83242[] = {SGI350086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350087, SGI0, NULL};
const Segment * const SG83243[] = {SGI350088, SGI0, SGI0, SGI350089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83244[] = {SGI350090, SGI0, SGI0, SGI350091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83245[] = {SGI350092, SGI0, SGI350093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83246[] = {SGI350094, SGI0, SGI350095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83247[] = {SGI350096, SGI0, SGI0, SGI350097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83248[] = {SGI350098, SGI0, SGI350099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83249[] = {SGI350100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83250[] = {SGI350101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350102, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83251[] = {SGI350103, SGI0, SGI0, SGI350104, SGI0, SGI0, SGI0, SGI0, SGI350105, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83252[] = {SGI350106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350107, SGI0, NULL};
const Segment * const SG83253[] = {SGI350108, SGI350109, SGI0, SGI0, NULL};
const Segment * const SG83254[] = {SGI350110, SGI350111, SGI0, SGI0, NULL};
const Segment * const SG83255[] = {SGI350112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350113, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83256[] = {SGI350114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350115, SGI0, NULL};
const Segment * const SG83257[] = {SGI350116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350117, SGI0, NULL};
const Segment * const SG83258[] = {SGI350118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83259[] = {SGI350119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83260[] = {SGI350120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83261[] = {SGI350121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83262[] = {SGI350122, SGI0, SGI350123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83263[] = {SGI350124, SGI0, SGI350125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83264[] = {SGI350126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350127, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83265[] = {SGI350128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350129, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83266[] = {SGI350130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350131, SGI0, NULL};
const Segment * const SG83267[] = {SGI350132, SGI0, SGI0, SGI350133, SGI0, SGI0, SGI0, SGI0, SGI350134, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83268[] = {SGI350135, SGI0, SGI0, SGI350136, SGI0, SGI0, SGI0, SGI0, SGI350137, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83269[] = {SGI350138, SGI0, SGI0, SGI350139, SGI0, SGI0, SGI0, SGI0, SGI350140, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83270[] = {SGI350141, SGI0, SGI350142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83271[] = {SGI350143, SGI0, SGI0, SGI350144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83272[] = {SGI350145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350146, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83273[] = {SGI350147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350148, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83274[] = {SGI350149, SGI0, SGI0, SGI350150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83275[] = {SGI350151, SGI0, SGI350152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83276[] = {SGI350153, SGI0, SGI350154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83277[] = {SGI350155, SGI0, SGI0, SGI350156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83278[] = {SGI350157, SGI0, SGI350158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83279[] = {SGI350159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350160, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83280[] = {SGI350161, SGI0, SGI0, SGI350162, SGI0, SGI0, SGI0, SGI0, SGI350163, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83281[] = {SGI350164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350165, SGI0, NULL};
const Segment * const SG83282[] = {SGI350166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350167, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83283[] = {SGI350168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350169, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83284[] = {SGI350170, SGI0, SGI0, SGI350171, SGI0, SGI0, SGI0, SGI0, SGI350172, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83285[] = {SGI350173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350174, SGI0, NULL};
const Segment * const SG83286[] = {SGI350175, SGI350176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83287[] = {SGI350177, SGI350178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83288[] = {SGI350179, SGI350180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83289[] = {SGI350181, SGI350182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83290[] = {SGI350183, SGI350184, SGI350185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350186, SGI0, NULL};
const Segment * const SG83291[] = {SGI350187, SGI350188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83292[] = {SGI350189, SGI350190, SGI350191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350192, SGI0, NULL};
const Segment * const SG83293[] = {SGI350193, SGI350194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83294[] = {SGI350195, SGI350196, SGI350197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350198, SGI0, NULL};
const Segment * const SG83295[] = {SGI350199, SGI350200, SGI350201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350202, SGI0, NULL};
const Segment * const SG83296[] = {SGI350203, SGI350204, SGI350205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350206, SGI0, NULL};
const Segment * const SG83297[] = {SGI0, SGI350207, SGI350208, SGI0, SGI0, SGI0, SGI350209, NULL};
const Segment * const SG83298[] = {SGI350210, SGI350211, SGI350212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350213, SGI0, NULL};
const Segment * const SG83299[] = {SGI0, SGI350214, SGI350215, SGI350216, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83300[] = {SGI350217, SGI350218, SGI350219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350220, SGI0, NULL};
const Segment * const SG83301[] = {SGI350221, SGI350222, SGI350223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350223, SGI0, NULL};
const Segment * const SG83302[] = {SGI350224, SGI350225, SGI350226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350227, SGI0, NULL};
const Segment * const SG83303[] = {SGI350228, SGI350229, SGI350230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350231, SGI0, NULL};
const Segment * const SG83304[] = {SGI350232, SGI350233, SGI350234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350235, SGI0, NULL};
const Segment * const SG83305[] = {SGI350236, SGI350237, SGI350238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350239, SGI0, NULL};
const Segment * const SG83306[] = {SGI350240, SGI350240, SGI350241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350241, SGI0, NULL};
const Segment * const SG83307[] = {SGI350242, SGI350243, SGI350244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350244, SGI0, NULL};
const Segment * const SG83308[] = {SGI350245, SGI350246, SGI350247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350247, SGI0, NULL};
const Segment * const SG83309[] = {SGI350248, SGI350249, SGI350250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350251, SGI0, NULL};
const Segment * const SG83310[] = {SGI350252, SGI0, SGI350253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350254, SGI0, NULL};
const Segment * const SG83311[] = {SGI350255, SGI350256, SGI350257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350258, SGI0, NULL};
const Segment * const SG83312[] = {SGI350259, SGI350260, SGI350261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350262, SGI0, NULL};
const Segment * const SG83313[] = {SGI350263, SGI350264, SGI350265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350266, SGI0, NULL};
const Segment * const SG83314[] = {SGI350267, SGI350268, SGI350269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350270, SGI0, NULL};
const Segment * const SG83315[] = {SGI350271, SGI350272, SGI350273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350274, SGI0, NULL};
const Segment * const SG83316[] = {SGI350275, SGI0, SGI350276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350276, SGI0, NULL};
const Segment * const SG83317[] = {SGI350277, SGI0, SGI350278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83318[] = {SGI350279, SGI0, SGI350280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83319[] = {SGI350281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350282, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83320[] = {SGI350283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350284, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83321[] = {SGI350285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83322[] = {SGI350286, SGI0, SGI350287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83323[] = {SGI350288, SGI0, SGI350289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83324[] = {SGI350290, SGI0, SGI350291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83325[] = {SGI350292, SGI0, SGI350293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83326[] = {SGI350294, SGI0, SGI350295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83327[] = {SGI350296, SGI0, SGI350297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83328[] = {SGI350298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350299, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83329[] = {SGI350300, SGI350301, SGI0, SGI0, NULL};
const Segment * const SG83330[] = {SGI350302, SGI350303, SGI0, SGI0, NULL};
const Segment * const SG83331[] = {SGI350304, SGI0, SGI0, SGI350305, SGI0, SGI0, SGI0, SGI0, SGI350306, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83332[] = {SGI350307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350308, SGI0, NULL};
const Segment * const SG83333[] = {SGI350309, SGI0, SGI0, SGI350310, SGI0, SGI0, SGI0, SGI0, SGI350313, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83334[] = {SGI0, SGI350311, SGI350312, SGI0, SGI0, SGI0, SGI350314, NULL};
const Segment * const SG83335[] = {SGI350315, SGI0, SGI350316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83336[] = {SGI350317, SGI0, SGI350318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83337[] = {SGI350319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350320, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83338[] = {SGI350321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350322, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83339[] = {SGI350323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350324, SGI0, NULL};
const Segment * const SG83340[] = {SGI350325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83341[] = {SGI350326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83342[] = {SGI350327, SGI350328, SGI0, SGI0, NULL};
const Segment * const SG83343[] = {SGI350329, SGI0, SGI0, SGI350330, SGI0, SGI0, SGI0, SGI0, SGI350331, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83344[] = {SGI350332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350333, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83345[] = {SGI350334, SGI0, SGI0, SGI350335, SGI0, SGI0, SGI0, SGI0, SGI350336, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83346[] = {SGI350337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350338, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83347[] = {SGI350339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350340, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83348[] = {SGI350341, SGI0, SGI0, SGI350342, SGI0, SGI0, SGI0, SGI0, SGI350343, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83349[] = {SGI350344, SGI0, SGI0, SGI350345, SGI0, SGI0, SGI0, SGI0, SGI350346, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83350[] = {SGI350347, SGI0, SGI350348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83351[] = {SGI350349, SGI0, SGI0, SGI350350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83352[] = {SGI350351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350352, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83353[] = {SGI350353, SGI0, SGI0, SGI350354, SGI0, SGI0, SGI0, SGI0, SGI350355, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83354[] = {SGI350356, SGI0, SGI0, SGI350357, SGI0, SGI0, SGI0, SGI0, SGI350358, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83355[] = {SGI350359, SGI0, SGI0, SGI350360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83356[] = {SGI350361, SGI0, SGI350362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83357[] = {SGI350363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350364, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83358[] = {SGI350365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350366, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83359[] = {SGI350367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350368, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83360[] = {SGI350369, SGI0, SGI0, SGI350370, SGI0, SGI0, SGI0, SGI0, SGI350371, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83361[] = {SGI350372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350373, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83362[] = {SGI350374, SGI0, SGI350375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83363[] = {SGI350376, SGI0, SGI0, SGI350377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83364[] = {SGI350378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350379, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83365[] = {SGI350380, SGI350381, SGI350382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83366[] = {SGI350383, SGI350384, SGI350385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83367[] = {SGI350386, SGI350387, SGI350388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350388, SGI0, NULL};
const Segment * const SG83368[] = {SGI350389, SGI350390, SGI350391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350392, SGI0, NULL};
const Segment * const SG83369[] = {SGI350393, SGI350394, SGI0, SGI0, NULL};
const Segment * const SG83370[] = {SGI350395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350396, SGI0, NULL};
const Segment * const SG83371[] = {SGI350397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350398, SGI0, NULL};
const Segment * const SG83372[] = {SGI350399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350400, SGI0, NULL};
const Segment * const SG83373[] = {SGI350401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350402, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83374[] = {SGI350403, SGI0, SGI0, SGI350404, SGI0, SGI0, SGI0, SGI0, SGI350405, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83375[] = {SGI350406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83376[] = {SGI350407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350408, SGI0, NULL};
const Segment * const SG83377[] = {SGI350409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350410, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83378[] = {SGI350411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350412, SGI0, NULL};
const Segment * const SG83379[] = {SGI350413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350414, SGI0, NULL};
const Segment * const SG83380[] = {SGI350415, SGI350416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83381[] = {SGI350417, SGI0, SGI0, SGI350418, SGI0, SGI0, SGI0, SGI0, SGI350419, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83382[] = {SGI350420, SGI0, SGI0, SGI350421, SGI0, SGI0, SGI0, SGI0, SGI350422, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83383[] = {SGI350423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350424, SGI0, NULL};
const Segment * const SG83384[] = {SGI350425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350426, SGI0, NULL};
const Segment * const SG83385[] = {SGI350427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350428, SGI0, NULL};
const Segment * const SG83386[] = {SGI350429, SGI0, SGI0, SGI350430, SGI0, SGI0, SGI0, SGI0, SGI350431, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83387[] = {SGI350432, SGI0, SGI350433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83388[] = {SGI350434, SGI0, SGI350435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83389[] = {SGI350436, SGI350437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83390[] = {SGI350438, SGI350439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83391[] = {SGI350440, SGI350441, SGI350442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350442, SGI0, NULL};
const Segment * const SG83392[] = {SGI350443, SGI350444, SGI350445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350445, SGI0, NULL};
const Segment * const SG83393[] = {SGI350446, SGI350447, SGI350448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350449, SGI0, NULL};
const Segment * const SG83394[] = {SGI350450, SGI0, SGI350451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350451, SGI0, NULL};
const Segment * const SG83395[] = {SGI0, SGI350452, SGI350453, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83396[] = {SGI0, SGI350454, SGI350455, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83397[] = {SGI350456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350457, SGI0, NULL};
const Segment * const SG83398[] = {SGI350458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350459, SGI0, NULL};
const Segment * const SG83399[] = {SGI350460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350461, SGI0, NULL};
const Segment * const SG83400[] = {SGI350462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350463, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83401[] = {SGI350464, SGI350465, SGI350466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83402[] = {SGI350468, SGI0, SGI350469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83403[] = {SGI350467, SGI350470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83404[] = {SGI350471, SGI350472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83405[] = {SGI350473, SGI0, SGI0, SGI350474, SGI0, SGI0, SGI0, SGI0, SGI350475, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83406[] = {SGI350476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83407[] = {SGI350477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83408[] = {SGI350478, SGI350479, SGI350480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350481, SGI0, NULL};
const Segment * const SG83409[] = {SGI350482, SGI350483, SGI350484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350485, SGI0, NULL};
const Segment * const SG83410[] = {SGI350486, SGI350487, SGI350488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350489, SGI0, NULL};
const Segment * const SG83411[] = {SGI350490, SGI350491, SGI350492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350493, SGI0, NULL};
const Segment * const SG83412[] = {SGI350494, SGI350495, SGI350496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350497, SGI0, NULL};
const Segment * const SG83413[] = {SGI350498, SGI350499, SGI350500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350501, SGI0, NULL};
const Segment * const SG83414[] = {SGI350502, SGI350503, SGI350504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350504, SGI0, NULL};
const Segment * const SG83415[] = {SGI350505, SGI350506, SGI350507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350508, SGI0, NULL};
const Segment * const SG83416[] = {SGI350509, SGI350510, SGI350511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350512, SGI0, NULL};
const Segment * const SG83417[] = {SGI350513, SGI350514, SGI350515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350516, SGI0, NULL};
const Segment * const SG83418[] = {SGI350517, SGI0, SGI0, SGI350518, SGI0, SGI0, SGI0, SGI0, SGI350519, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83419[] = {SGI350520, SGI350521, SGI350522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350523, SGI0, NULL};
const Segment * const SG83420[] = {SGI350524, SGI350525, SGI350526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350527, SGI0, NULL};
const Segment * const SG83421[] = {SGI350528, SGI350529, SGI350530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350531, SGI0, NULL};
const Segment * const SG83422[] = {SGI350532, SGI350533, SGI350534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350535, SGI0, NULL};
const Segment * const SG83423[] = {SGI350536, SGI350537, SGI350538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350539, SGI0, NULL};
const Segment * const SG83424[] = {SGI350540, SGI350541, SGI350542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350543, SGI0, NULL};
const Segment * const SG83425[] = {SGI350544, SGI350545, SGI350546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350547, SGI0, NULL};
const Segment * const SG83426[] = {SGI350548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350549, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83427[] = {SGI350550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350551, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83428[] = {SGI350552, SGI350553, SGI0, SGI0, NULL};
const Segment * const SG83429[] = {SGI350554, SGI0, SGI0, SGI350555, SGI0, SGI0, SGI0, SGI0, SGI350556, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83430[] = {SGI350557, SGI350558, SGI0, SGI0, NULL};
const Segment * const SG83431[] = {SGI350559, SGI350560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83432[] = {SGI350561, SGI350562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83433[] = {SGI350563, SGI0, SGI0, SGI350564, SGI0, SGI0, SGI0, SGI0, SGI350565, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83434[] = {SGI350566, SGI0, SGI0, SGI350567, SGI0, SGI0, SGI0, SGI0, SGI350568, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83435[] = {SGI350569, SGI0, SGI350570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83436[] = {SGI350571, SGI0, SGI350572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83437[] = {SGI350573, SGI0, SGI350574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83438[] = {SGI350575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350576, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83439[] = {SGI350577, SGI350578, SGI350579, SGI0, SGI0, SGI0, SGI0, SGI350580, SGI0, SGI0, NULL};
const Segment * const SG83440[] = {SGI350581, SGI0, SGI0, SGI350582, SGI0, SGI0, SGI0, SGI0, SGI350583, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83441[] = {SGI350584, SGI0, SGI0, SGI350585, SGI0, SGI0, SGI0, SGI0, SGI350586, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83442[] = {SGI350587, SGI0, SGI350588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83443[] = {SGI350589, SGI0, SGI0, SGI350590, SGI0, SGI0, SGI0, SGI0, SGI350591, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83444[] = {SGI350592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350593, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83445[] = {SGI350594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83446[] = {SGI350595, SGI0, SGI0, SGI350596, SGI0, SGI0, SGI0, SGI0, SGI350597, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83447[] = {SGI350598, SGI0, SGI0, SGI350599, SGI0, SGI0, SGI0, SGI0, SGI350600, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83448[] = {SGI350601, SGI0, SGI0, SGI350602, SGI0, SGI0, SGI0, SGI0, SGI350603, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83449[] = {SGI350604, SGI0, SGI350605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83450[] = {SGI350606, SGI0, SGI350607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83451[] = {SGI350608, SGI350609, SGI350610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350610, SGI0, NULL};
const Segment * const SG83452[] = {SGI350611, SGI350612, SGI350613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350614, SGI0, NULL};
const Segment * const SG83453[] = {SGI350615, SGI350616, SGI350617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350618, SGI0, NULL};
const Segment * const SG83454[] = {SGI350619, SGI350620, SGI350621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350622, SGI0, NULL};
const Segment * const SG83455[] = {SGI350623, SGI350624, SGI350625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350626, SGI0, NULL};
const Segment * const SG83456[] = {SGI350627, SGI350628, SGI350629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350630, SGI0, NULL};
const Segment * const SG83457[] = {SGI350631, SGI350632, SGI350633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350634, SGI0, NULL};
const Segment * const SG83458[] = {SGI350635, SGI350636, SGI350637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350638, SGI0, NULL};
const Segment * const SG83459[] = {SGI350639, SGI350640, SGI350641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350642, SGI0, NULL};
const Segment * const SG83460[] = {SGI350643, SGI350644, SGI350645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350646, SGI0, NULL};
const Segment * const SG83461[] = {SGI350647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350648, SGI0, NULL};
const Segment * const SG83462[] = {SGI350649, SGI0, SGI0, SGI350650, SGI0, SGI0, SGI0, SGI0, SGI350651, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83463[] = {SGI350652, SGI0, SGI350653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83464[] = {SGI350654, SGI350655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83465[] = {SGI350656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350657, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83466[] = {SGI350658, SGI0, SGI0, SGI350659, SGI0, SGI0, SGI0, SGI0, SGI350660, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83467[] = {SGI350661, SGI0, SGI0, SGI350662, SGI0, SGI0, SGI0, SGI0, SGI350663, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83468[] = {SGI350664, SGI0, SGI350665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83469[] = {SGI350666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350667, SGI0, NULL};
const Segment * const SG83470[] = {SGI350668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350669, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83471[] = {SGI350670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350671, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83472[] = {SGI350672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350673, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83473[] = {SGI350674, SGI0, SGI0, SGI350675, SGI0, SGI0, SGI0, SGI0, SGI350676, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83474[] = {SGI350677, SGI0, SGI350678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83475[] = {SGI350679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350680, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83476[] = {SGI350681, SGI0, SGI350682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83477[] = {SGI350683, SGI0, SGI350684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83478[] = {SGI350685, SGI0, SGI350686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83479[] = {SGI350687, SGI0, SGI350688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83480[] = {SGI350689, SGI0, SGI350690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83481[] = {SGI350691, SGI0, SGI350692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83482[] = {SGI350693, SGI0, SGI350694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83483[] = {SGI350695, SGI0, SGI350696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83484[] = {SGI350697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83485[] = {SGI350698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83486[] = {SGI350699, SGI0, SGI0, SGI350700, SGI0, SGI0, SGI0, SGI0, SGI350701, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83487[] = {SGI350702, SGI0, SGI0, SGI350703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83488[] = {SGI350704, SGI350705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83489[] = {SGI350706, SGI350707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83490[] = {SGI350708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83491[] = {SGI350709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350710, SGI0, NULL};
const Segment * const SG83492[] = {SGI350711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83493[] = {SGI350712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83494[] = {SGI350713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83495[] = {SGI350714, SGI0, SGI0, SGI350715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83496[] = {SGI350716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350717, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83497[] = {SGI350718, SGI0, SGI0, SGI350719, SGI0, SGI0, SGI0, SGI0, SGI350720, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83498[] = {SGI350721, SGI0, SGI0, SGI350722, SGI0, SGI0, SGI0, SGI0, SGI350723, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83499[] = {SGI350724, SGI350725, SGI0, SGI0, NULL};
const Segment * const SG83500[] = {SGI350726, SGI0, SGI350727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83501[] = {SGI350728, SGI350729, SGI350730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350731, SGI0, NULL};
const Segment * const SG83502[] = {SGI350732, SGI350733, SGI350734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350734, SGI0, NULL};
const Segment * const SG83503[] = {SGI350735, SGI350736, SGI350737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350738, SGI0, NULL};
const Segment * const SG83504[] = {SGI350739, SGI350740, SGI350741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350742, SGI0, NULL};
const Segment * const SG83505[] = {SGI350743, SGI350744, SGI350745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350746, SGI0, NULL};
const Segment * const SG83506[] = {SGI350747, SGI350748, SGI350749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350750, SGI0, NULL};
const Segment * const SG83507[] = {SGI350751, SGI0, SGI0, SGI350752, SGI0, SGI0, SGI0, SGI0, SGI350753, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83508[] = {SGI350754, SGI0, SGI0, SGI350755, SGI0, SGI0, SGI0, SGI0, SGI350756, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83509[] = {SGI350757, SGI350758, SGI350759, SGI0, SGI0, SGI0, SGI0, SGI350760, SGI0, SGI0, NULL};
const Segment * const SG83510[] = {SGI350761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350762, SGI0, NULL};
const Segment * const SG83511[] = {SGI350763, SGI350764, SGI0, SGI0, NULL};
const Segment * const SG83512[] = {SGI350765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350766, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83513[] = {SGI350767, SGI350768, SGI0, SGI0, NULL};
const Segment * const SG83514[] = {SGI350769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350770, SGI0, NULL};
const Segment * const SG83515[] = {SGI350771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350772, SGI0, NULL};
const Segment * const SG83516[] = {SGI350773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83517[] = {SGI350774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350775, SGI0, NULL};
const Segment * const SG83518[] = {SGI350776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83519[] = {SGI350777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350778, SGI0, NULL};
const Segment * const SG83520[] = {SGI350779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83521[] = {SGI350780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350781, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83522[] = {SGI350782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83523[] = {SGI350783, SGI0, SGI0, SGI350784, SGI0, SGI0, SGI0, SGI0, SGI350785, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83524[] = {SGI350786, SGI0, SGI350787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83525[] = {SGI350788, SGI0, SGI350789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83526[] = {SGI350790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350791, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83527[] = {SGI350792, SGI0, SGI0, SGI350793, SGI0, SGI0, SGI0, SGI0, SGI350794, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83528[] = {SGI350795, SGI0, SGI0, SGI350796, SGI0, SGI0, SGI0, SGI0, SGI350797, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83529[] = {SGI350798, SGI0, SGI0, SGI350799, SGI0, SGI0, SGI0, SGI0, SGI350800, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83530[] = {SGI350801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350802, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83531[] = {SGI350803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83532[] = {SGI350804, SGI350805, SGI350806, SGI0, SGI0, SGI0, SGI0, SGI350807, SGI0, SGI0, NULL};
const Segment * const SG83533[] = {SGI350808, SGI0, SGI350809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83534[] = {SGI350810, SGI0, SGI0, SGI350811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83535[] = {SGI350812, SGI0, SGI0, SGI350813, SGI0, SGI0, SGI0, SGI0, SGI350814, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83536[] = {SGI350815, SGI0, SGI350816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350816, SGI0, NULL};
const Segment * const SG83537[] = {SGI350817, SGI0, SGI350818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350818, SGI0, NULL};
const Segment * const SG83538[] = {SGI350819, SGI0, SGI350820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350820, SGI0, NULL};
const Segment * const SG83539[] = {SGI350821, SGI0, SGI350822, SGI350822, SGI0, SGI0, SGI0, SGI0, SGI350822, SGI0, NULL};
const Segment * const SG83540[] = {SGI350823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83541[] = {SGI350824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83542[] = {SGI350825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350826, SGI0, NULL};
const Segment * const SG83543[] = {SGI0, SGI350827, SGI350828, SGI350829, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83544[] = {SGI350830, SGI0, SGI0, SGI350831, SGI0, SGI0, SGI0, SGI0, SGI350832, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83545[] = {SGI350833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350834, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83546[] = {SGI350835, SGI0, SGI350836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83547[] = {SGI350837, SGI0, SGI350838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83548[] = {SGI350839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350840, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83549[] = {SGI350841, SGI350842, SGI350843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83550[] = {SGI350844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83551[] = {SGI350845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350846, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83552[] = {SGI350847, SGI0, SGI350848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350848, SGI0, NULL};
const Segment * const SG83553[] = {SGI350849, SGI0, SGI350850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350850, SGI0, NULL};
const Segment * const SG83554[] = {SGI350851, SGI0, SGI350852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350852, SGI0, NULL};
const Segment * const SG83555[] = {SGI350853, SGI0, SGI0, SGI350854, SGI0, SGI0, SGI0, SGI0, SGI350855, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83556[] = {SGI350856, SGI0, SGI0, SGI350857, SGI0, SGI0, SGI0, SGI0, SGI350858, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83557[] = {SGI350859, SGI0, SGI0, SGI350860, SGI0, SGI0, SGI0, SGI0, SGI350861, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83558[] = {SGI350862, SGI0, SGI0, SGI350863, SGI0, SGI0, SGI0, SGI0, SGI350864, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83559[] = {SGI350865, SGI0, SGI0, SGI350866, SGI0, SGI0, SGI0, SGI0, SGI350867, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83560[] = {SGI350868, SGI0, SGI0, SGI350869, SGI0, SGI0, SGI0, SGI0, SGI350870, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83561[] = {SGI350871, SGI0, SGI0, SGI350872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83562[] = {SGI350873, SGI0, SGI350874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83563[] = {SGI350875, SGI0, SGI350876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83564[] = {SGI350877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350878, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83565[] = {SGI350879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350880, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83566[] = {SGI350881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350882, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83567[] = {SGI350883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350884, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83568[] = {SGI350885, SGI0, SGI0, SGI350886, SGI0, SGI0, SGI0, SGI0, SGI350887, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83569[] = {SGI350888, SGI0, SGI0, SGI350889, SGI0, SGI0, SGI0, SGI0, SGI350890, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83570[] = {SGI350891, SGI0, SGI0, SGI350892, SGI0, SGI0, SGI0, SGI0, SGI350893, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83571[] = {SGI350894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83572[] = {SGI350895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83573[] = {SGI350896, SGI0, SGI0, SGI350897, SGI0, SGI0, SGI0, SGI0, SGI350898, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83574[] = {SGI350899, SGI0, SGI0, SGI350900, SGI0, SGI0, SGI0, SGI0, SGI350901, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83575[] = {SGI350902, SGI0, SGI0, SGI350903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83576[] = {SGI350904, SGI350905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83577[] = {SGI350906, SGI350907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83578[] = {SGI350908, SGI350909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83579[] = {SGI350910, SGI0, SGI0, SGI350911, SGI0, SGI0, SGI0, SGI0, SGI350912, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83580[] = {SGI350913, SGI0, SGI0, SGI350914, SGI0, SGI0, SGI0, SGI0, SGI350915, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83581[] = {SGI350916, SGI0, SGI350917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83582[] = {SGI350918, SGI350919, SGI350920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83583[] = {SGI350921, SGI350922, SGI350923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350924, SGI0, NULL};
const Segment * const SG83584[] = {SGI350925, SGI350926, SGI350927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83585[] = {SGI350928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350929, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83586[] = {SGI350930, SGI0, SGI0, SGI350931, SGI0, SGI0, SGI0, SGI0, SGI350932, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83587[] = {SGI350933, SGI0, SGI0, SGI350934, SGI0, SGI0, SGI0, SGI0, SGI350935, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83588[] = {SGI350936, SGI0, SGI0, SGI350937, SGI0, SGI0, SGI0, SGI0, SGI350938, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83589[] = {SGI0, SGI350939, SGI350940, SGI0, SGI0, SGI0, SGI350941, NULL};
const Segment * const SG83590[] = {SGI0, SGI350942, SGI350943, SGI0, SGI0, SGI0, SGI350944, NULL};
const Segment * const SG83591[] = {SGI350945, SGI0, SGI0, SGI350946, SGI0, SGI0, SGI0, SGI0, SGI350947, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83592[] = {SGI350948, SGI0, SGI0, SGI350949, SGI0, SGI0, SGI0, SGI0, SGI350950, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83593[] = {SGI350951, SGI0, SGI0, SGI350952, SGI0, SGI0, SGI0, SGI0, SGI350953, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83594[] = {SGI350954, SGI0, SGI0, SGI350955, SGI0, SGI0, SGI0, SGI0, SGI350956, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83595[] = {SGI350957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83596[] = {SGI350958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350959, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83597[] = {SGI350960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83598[] = {SGI350961, SGI0, SGI0, SGI350962, SGI0, SGI0, SGI0, SGI0, SGI350963, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83599[] = {SGI350964, SGI0, SGI350965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83600[] = {SGI350966, SGI0, SGI0, SGI350967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83601[] = {SGI350968, SGI0, SGI0, SGI350969, SGI0, SGI0, SGI0, SGI0, SGI350970, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83602[] = {SGI350971, SGI0, SGI350972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83603[] = {SGI350973, SGI0, SGI350974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83604[] = {SGI350975, SGI0, SGI0, SGI350976, SGI0, SGI0, SGI0, SGI0, SGI350977, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83605[] = {SGI350978, SGI0, SGI0, SGI350979, SGI0, SGI0, SGI0, SGI0, SGI350980, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83606[] = {SGI350981, SGI0, SGI0, SGI350982, SGI0, SGI0, SGI0, SGI0, SGI350983, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83607[] = {SGI350984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83608[] = {SGI350985, SGI0, SGI0, SGI350986, SGI0, SGI0, SGI0, SGI0, SGI350987, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83609[] = {SGI350988, SGI0, SGI0, SGI350989, SGI0, SGI0, SGI0, SGI0, SGI350990, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83610[] = {SGI350991, SGI0, SGI0, SGI350992, SGI0, SGI0, SGI0, SGI0, SGI350993, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83611[] = {SGI350994, SGI0, SGI0, SGI350995, SGI0, SGI0, SGI0, SGI0, SGI350996, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83612[] = {SGI350997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI350998, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83613[] = {SGI350999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351000, SGI0, NULL};
const Segment * const SG83614[] = {SGI351001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351002, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83615[] = {SGI351003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351004, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83616[] = {SGI351005, SGI0, SGI0, SGI351006, SGI0, SGI0, SGI0, SGI0, SGI351007, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83617[] = {SGI351008, SGI351009, SGI351010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351010, SGI0, NULL};
const Segment * const SG83618[] = {SGI351011, SGI0, SGI351012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83619[] = {SGI351013, SGI0, SGI0, SGI351014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83620[] = {SGI351015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351016, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83621[] = {SGI351017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351018, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83622[] = {SGI351019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351020, SGI0, NULL};
const Segment * const SG83623[] = {SGI351021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351022, SGI0, NULL};
const Segment * const SG83624[] = {SGI351023, SGI0, SGI0, SGI351024, SGI0, SGI0, SGI0, SGI0, SGI351025, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83625[] = {SGI351026, SGI0, SGI351027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83626[] = {SGI351028, SGI351029, SGI351030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83627[] = {SGI351031, SGI351032, SGI351033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83628[] = {SGI351034, SGI0, SGI0, SGI351035, SGI0, SGI0, SGI0, SGI0, SGI351036, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83629[] = {SGI351037, SGI0, SGI0, SGI351038, SGI0, SGI0, SGI0, SGI0, SGI351039, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83630[] = {SGI351040, SGI0, SGI351041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83631[] = {SGI351042, SGI0, SGI351043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83632[] = {SGI351044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351045, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83633[] = {SGI351046, SGI351047, SGI351048, SGI0, SGI0, SGI0, SGI0, SGI351049, SGI0, SGI0, NULL};
const Segment * const SG83634[] = {SGI351050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351051, SGI0, NULL};
const Segment * const SG83635[] = {SGI351052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351053, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83636[] = {SGI351054, SGI0, SGI0, SGI351055, SGI0, SGI0, SGI0, SGI0, SGI351056, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83637[] = {SGI351057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351058, SGI0, NULL};
const Segment * const SG83638[] = {SGI351059, SGI351060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83639[] = {SGI351061, SGI351062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83640[] = {SGI351063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351064, SGI0, NULL};
const Segment * const SG83641[] = {SGI351065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351066, SGI0, NULL};
const Segment * const SG83642[] = {SGI351067, SGI0, SGI0, SGI351068, SGI0, SGI0, SGI0, SGI0, SGI351069, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83643[] = {SGI351070, SGI0, SGI0, SGI351071, SGI0, SGI0, SGI0, SGI0, SGI351072, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83644[] = {SGI351073, SGI0, SGI0, SGI351074, SGI0, SGI0, SGI0, SGI0, SGI351075, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83645[] = {SGI351076, SGI0, SGI0, SGI351077, SGI0, SGI0, SGI0, SGI0, SGI351078, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83646[] = {SGI351079, SGI0, SGI0, SGI351080, SGI0, SGI0, SGI0, SGI0, SGI351081, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83647[] = {SGI351082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351083, SGI0, NULL};
const Segment * const SG83648[] = {SGI351084, SGI351085, SGI351086, SGI0, SGI0, SGI0, SGI0, SGI351087, SGI0, SGI0, NULL};
const Segment * const SG83649[] = {SGI351088, SGI351089, SGI351090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351090, SGI0, NULL};
const Segment * const SG83650[] = {SGI351091, SGI351092, SGI351093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351093, SGI0, NULL};
const Segment * const SG83651[] = {SGI351094, SGI0, SGI0, SGI351095, SGI0, SGI0, SGI0, SGI0, SGI351096, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83652[] = {SGI351097, SGI0, SGI0, SGI351098, SGI0, SGI0, SGI0, SGI0, SGI351099, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83653[] = {SGI351100, SGI0, SGI0, SGI351101, SGI0, SGI0, SGI0, SGI0, SGI351102, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83654[] = {SGI351103, SGI0, SGI351104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83655[] = {SGI351105, SGI0, SGI351106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83656[] = {SGI351107, SGI351108, NULL};
const Segment * const SG83657[] = {SGI351109, SGI351110, NULL};
const Segment * const SG83658[] = {SGI351111, SGI351112, SGI0, SGI0, NULL};
const Segment * const SG83659[] = {SGI351113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83660[] = {SGI351114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83661[] = {SGI351115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83662[] = {SGI0, SGI351116, SGI351117, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83663[] = {SGI351118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351119, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83664[] = {SGI351120, SGI0, SGI0, SGI351121, SGI0, SGI0, SGI0, SGI0, SGI351122, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83665[] = {SGI351123, SGI0, SGI351124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83666[] = {SGI351125, SGI0, SGI0, SGI351126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83667[] = {SGI351127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351128, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83668[] = {SGI351129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83669[] = {SGI351130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351131, SGI0, NULL};
const Segment * const SG83670[] = {SGI351132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351133, SGI0, NULL};
const Segment * const SG83671[] = {SGI351134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351135, SGI0, NULL};
const Segment * const SG83672[] = {SGI351136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351137, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83673[] = {SGI351138, SGI0, SGI0, SGI351139, SGI0, SGI0, SGI0, SGI0, SGI351140, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83674[] = {SGI351141, SGI0, SGI0, SGI351142, SGI0, SGI0, SGI0, SGI0, SGI351143, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83675[] = {SGI351144, SGI351145, NULL};
const Segment * const SG83676[] = {SGI351146, SGI351147, SGI351148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83677[] = {SGI351149, SGI351150, SGI351151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83678[] = {SGI351152, SGI351153, SGI351154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83679[] = {SGI351155, SGI351156, SGI351157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83680[] = {SGI351158, SGI351158, SGI351159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83681[] = {SGI351160, SGI351161, SGI351162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83682[] = {SGI351163, SGI351164, SGI351165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83683[] = {SGI351166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351167, SGI0, NULL};
const Segment * const SG83684[] = {SGI351168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351169, SGI0, NULL};
const Segment * const SG83685[] = {SGI351170, SGI0, SGI0, SGI351171, SGI0, SGI0, SGI0, SGI0, SGI351172, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83686[] = {SGI351173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83687[] = {SGI351174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83688[] = {SGI351175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351176, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83689[] = {SGI351177, SGI351178, SGI351179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351180, SGI0, NULL};
const Segment * const SG83690[] = {SGI351181, SGI351182, SGI351183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351184, SGI0, NULL};
const Segment * const SG83691[] = {SGI351185, SGI351186, SGI351187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351188, SGI0, NULL};
const Segment * const SG83692[] = {SGI351189, SGI351190, SGI351191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351192, SGI0, NULL};
const Segment * const SG83693[] = {SGI351193, SGI351194, SGI351195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351196, SGI0, NULL};
const Segment * const SG83694[] = {SGI351197, SGI351198, SGI351199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351200, SGI0, NULL};
const Segment * const SG83695[] = {SGI351201, SGI351202, SGI351203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351203, SGI0, NULL};
const Segment * const SG83696[] = {SGI351204, SGI351205, SGI351206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351207, SGI0, NULL};
const Segment * const SG83697[] = {SGI351208, SGI351209, SGI351210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351211, SGI0, NULL};
const Segment * const SG83698[] = {SGI351212, SGI351213, SGI351214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351215, SGI0, NULL};
const Segment * const SG83699[] = {SGI351216, SGI351217, SGI351218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351219, SGI0, NULL};
const Segment * const SG83700[] = {SGI351220, SGI351221, SGI351222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351223, SGI0, NULL};
const Segment * const SG83701[] = {SGI351224, SGI351225, SGI351226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351227, SGI0, NULL};
const Segment * const SG83702[] = {SGI351228, SGI351229, SGI351230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351231, SGI0, NULL};
const Segment * const SG83703[] = {SGI351232, SGI351233, SGI351234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351235, SGI0, NULL};
const Segment * const SG83704[] = {SGI351236, SGI351237, SGI351238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351239, SGI0, NULL};
const Segment * const SG83705[] = {SGI351240, SGI351241, SGI351242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351243, SGI0, NULL};
const Segment * const SG83706[] = {SGI351244, SGI351245, SGI351246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351247, SGI0, NULL};
const Segment * const SG83707[] = {SGI351248, SGI0, SGI351249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83708[] = {SGI351250, SGI0, SGI351251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83709[] = {SGI351252, SGI0, SGI351253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351253, SGI0, NULL};
const Segment * const SG83710[] = {SGI351254, SGI351255, SGI351256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83711[] = {SGI351257, SGI351258, SGI351259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83712[] = {SGI351260, SGI351261, SGI351262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351263, NULL};
const Segment * const SG83713[] = {SGI351264, SGI351265, SGI351266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351267, NULL};
const Segment * const SG83714[] = {SGI351268, SGI0, SGI0, SGI351269, SGI0, SGI0, SGI0, SGI0, SGI351270, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83715[] = {SGI351271, SGI0, SGI0, SGI351272, SGI0, SGI0, SGI0, SGI0, SGI351273, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83716[] = {SGI351274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351275, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83717[] = {SGI351276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351277, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83718[] = {SGI351278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351279, SGI0, NULL};
const Segment * const SG83719[] = {SGI351280, SGI0, SGI351281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83720[] = {SGI351282, SGI0, SGI0, SGI351283, SGI0, SGI0, SGI0, SGI0, SGI351284, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83721[] = {SGI351285, SGI0, SGI0, SGI351286, SGI0, SGI0, SGI0, SGI0, SGI351287, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83722[] = {SGI351288, SGI0, SGI351289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83723[] = {SGI351290, SGI0, SGI0, SGI351291, SGI0, SGI0, SGI0, SGI0, SGI351292, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83724[] = {SGI351293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351294, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83725[] = {SGI351295, SGI0, SGI0, SGI351296, SGI0, SGI0, SGI0, SGI0, SGI351297, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83726[] = {SGI351298, SGI351299, SGI351300, SGI0, SGI0, SGI0, SGI0, SGI351301, SGI0, SGI0, NULL};
const Segment * const SG83727[] = {SGI351302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351303, SGI0, NULL};
const Segment * const SG83728[] = {SGI351304, SGI0, SGI0, SGI351305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83729[] = {SGI351306, SGI351307, SGI0, SGI0, NULL};
const Segment * const SG83730[] = {SGI351308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83731[] = {SGI351309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83732[] = {SGI351310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83733[] = {SGI351311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83734[] = {SGI351312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351313, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83735[] = {SGI351314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83736[] = {SGI351315, SGI0, SGI0, SGI351316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83737[] = {SGI351317, SGI0, SGI351318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83738[] = {SGI351319, SGI351320, SGI351321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351321, SGI0, NULL};
const Segment * const SG83739[] = {SGI351322, SGI351323, SGI351324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351325, SGI0, NULL};
const Segment * const SG83740[] = {SGI351326, SGI351327, SGI351328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351329, SGI0, NULL};
const Segment * const SG83741[] = {SGI351330, SGI351331, SGI351332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351333, SGI0, NULL};
const Segment * const SG83742[] = {SGI351334, SGI351335, SGI351336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351337, SGI0, NULL};
const Segment * const SG83743[] = {SGI351338, SGI351339, SGI351340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351341, SGI0, NULL};
const Segment * const SG83744[] = {SGI351342, SGI351343, SGI351344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351345, SGI0, NULL};
const Segment * const SG83745[] = {SGI351346, SGI351347, SGI351348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351349, SGI0, NULL};
const Segment * const SG83746[] = {SGI351350, SGI351351, SGI351352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351353, SGI0, NULL};
const Segment * const SG83747[] = {SGI351356, SGI351357, SGI351358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351359, SGI0, NULL};
const Segment * const SG83748[] = {SGI351354, SGI0, SGI0, SGI351355, SGI0, SGI0, SGI0, SGI0, SGI351360, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83749[] = {SGI351361, SGI351362, SGI351363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351364, SGI0, NULL};
const Segment * const SG83750[] = {SGI351365, SGI351365, SGI351366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351366, SGI0, NULL};
const Segment * const SG83751[] = {SGI351367, SGI0, SGI0, SGI351368, SGI0, SGI0, SGI0, SGI0, SGI351369, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83752[] = {SGI351370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83753[] = {SGI351371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83754[] = {SGI351372, SGI0, SGI0, SGI351373, SGI0, SGI0, SGI0, SGI0, SGI351374, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83755[] = {SGI351375, SGI0, SGI0, SGI351376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83756[] = {SGI351377, SGI0, SGI351378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83757[] = {SGI351379, SGI0, SGI351380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83758[] = {SGI351381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351382, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83759[] = {SGI351383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351384, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83760[] = {SGI351385, SGI0, SGI0, SGI351386, SGI0, SGI0, SGI0, SGI0, SGI351387, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83761[] = {SGI351388, SGI351389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83762[] = {SGI351390, SGI351391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83763[] = {SGI351392, SGI0, SGI0, SGI351393, SGI0, SGI0, SGI0, SGI0, SGI351394, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83764[] = {SGI351395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351396, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83765[] = {SGI351397, SGI0, SGI0, SGI351398, SGI0, SGI0, SGI0, SGI0, SGI351399, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83766[] = {SGI351400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351401, SGI0, NULL};
const Segment * const SG83767[] = {SGI351402, SGI0, SGI351403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83768[] = {SGI351404, SGI351405, SGI351406, SGI0, SGI0, SGI0, SGI0, SGI351407, SGI0, SGI0, NULL};
const Segment * const SG83769[] = {SGI351408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351409, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83770[] = {SGI351410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351411, SGI0, NULL};
const Segment * const SG83771[] = {SGI351412, SGI0, SGI351413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83772[] = {SGI351414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351415, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83773[] = {SGI351416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351417, SGI0, NULL};
const Segment * const SG83774[] = {SGI351418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351419, SGI0, NULL};
const Segment * const SG83775[] = {SGI351420, SGI351421, SGI351422, SGI351423, NULL};
const Segment * const SG83776[] = {SGI351424, SGI0, SGI0, SGI351425, SGI0, SGI0, SGI0, SGI0, SGI351426, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83777[] = {SGI351427, SGI0, SGI0, SGI351428, SGI0, SGI0, SGI0, SGI0, SGI351429, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83778[] = {SGI351430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83779[] = {SGI351431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351432, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83780[] = {SGI351433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351434, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83781[] = {SGI351435, SGI0, SGI0, SGI351436, SGI0, SGI0, SGI0, SGI0, SGI351437, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83782[] = {SGI351438, SGI0, SGI0, SGI351439, SGI0, SGI0, SGI0, SGI0, SGI351440, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83783[] = {SGI351441, SGI0, SGI0, SGI351442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83784[] = {SGI351443, SGI351444, SGI351445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83785[] = {SGI351446, SGI351447, SGI351448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83786[] = {SGI351449, SGI351450, SGI351451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83787[] = {SGI351452, SGI351453, SGI351454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83788[] = {SGI351455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83789[] = {SGI351456, SGI0, SGI0, SGI351457, SGI0, SGI0, SGI0, SGI0, SGI351458, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83790[] = {SGI351459, SGI0, SGI0, SGI351460, SGI0, SGI0, SGI0, SGI0, SGI351461, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83791[] = {SGI351462, SGI351463, SGI351464, SGI0, SGI0, SGI0, SGI0, SGI351465, SGI0, SGI0, NULL};
const Segment * const SG83792[] = {SGI351466, SGI0, SGI351467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83793[] = {SGI0, SGI351468, SGI0, SGI0, NULL};
const Segment * const SG83794[] = {SGI351469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351470, SGI0, NULL};
const Segment * const SG83795[] = {SGI351471, SGI0, SGI0, SGI351472, SGI0, SGI0, SGI0, SGI0, SGI351473, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83796[] = {SGI351474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351475, SGI0, NULL};
const Segment * const SG83797[] = {SGI351476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351477, SGI0, NULL};
const Segment * const SG83798[] = {SGI351478, SGI0, SGI0, SGI351479, SGI0, SGI0, SGI0, SGI0, SGI351480, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83799[] = {SGI351481, SGI0, SGI0, SGI351482, SGI0, SGI0, SGI0, SGI0, SGI351483, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83800[] = {SGI351484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83801[] = {SGI351485, SGI0, SGI351486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83802[] = {SGI351487, SGI0, SGI0, SGI351488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83803[] = {SGI351489, SGI351490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83804[] = {SGI351491, SGI0, SGI0, SGI351492, SGI0, SGI0, SGI0, SGI0, SGI351493, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83805[] = {SGI351494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351495, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83806[] = {SGI351496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83807[] = {SGI351497, SGI0, SGI351498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83808[] = {SGI351499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351500, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83809[] = {SGI351501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351502, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83810[] = {SGI351503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351504, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83811[] = {SGI351505, SGI0, SGI0, SGI351506, SGI0, SGI0, SGI0, SGI0, SGI351507, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83812[] = {SGI351508, SGI0, SGI0, SGI351509, SGI0, SGI0, SGI0, SGI0, SGI351510, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83813[] = {SGI351511, SGI351512, SGI351513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351513, SGI0, NULL};
const Segment * const SG83814[] = {SGI351514, SGI351515, SGI351516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351516, SGI0, NULL};
const Segment * const SG83815[] = {SGI351517, SGI351518, SGI351519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351519, SGI0, NULL};
const Segment * const SG83816[] = {SGI351520, SGI351521, SGI351522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351522, SGI0, NULL};
const Segment * const SG83817[] = {SGI351523, SGI351524, SGI351525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351525, SGI0, NULL};
const Segment * const SG83818[] = {SGI351526, SGI351527, SGI0, SGI0, NULL};
const Segment * const SG83819[] = {SGI351528, SGI351529, SGI0, SGI0, NULL};
const Segment * const SG83820[] = {SGI351530, SGI351531, SGI0, SGI0, NULL};
const Segment * const SG83821[] = {SGI351532, SGI351533, SGI0, SGI0, NULL};
const Segment * const SG83822[] = {SGI351534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351535, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83823[] = {SGI351536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351537, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83824[] = {SGI351538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351539, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83825[] = {SGI351540, SGI0, SGI0, SGI351541, SGI0, SGI0, SGI0, SGI0, SGI351542, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83826[] = {SGI351543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351544, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83827[] = {SGI351545, SGI0, SGI0, SGI351546, SGI0, SGI0, SGI0, SGI0, SGI351547, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83828[] = {SGI351548, SGI0, SGI351549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83829[] = {SGI351550, SGI0, SGI351551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83830[] = {SGI351552, SGI0, SGI351553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83831[] = {SGI351554, SGI0, SGI351555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83832[] = {SGI351556, SGI0, SGI351557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83833[] = {SGI351558, SGI0, SGI351559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83834[] = {SGI351560, SGI0, SGI351561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83835[] = {SGI351562, SGI0, SGI351563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83836[] = {SGI351564, SGI0, SGI351565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83837[] = {SGI351566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351567, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83838[] = {SGI351568, SGI0, SGI0, SGI351569, SGI0, SGI0, SGI0, SGI0, SGI351570, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83839[] = {SGI351571, SGI0, SGI0, SGI351572, SGI0, SGI0, SGI0, SGI0, SGI351573, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83840[] = {SGI351574, SGI0, SGI0, SGI351575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83841[] = {SGI351576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351577, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83842[] = {SGI351578, SGI0, SGI0, SGI351579, SGI0, SGI0, SGI0, SGI0, SGI351580, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83843[] = {SGI351581, SGI0, SGI0, SGI351582, SGI0, SGI0, SGI0, SGI0, SGI351583, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83844[] = {SGI351584, SGI0, SGI0, SGI351585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83845[] = {SGI351586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83846[] = {SGI351587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83847[] = {SGI351588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83848[] = {SGI351589, SGI0, SGI0, SGI351590, SGI0, SGI0, SGI0, SGI0, SGI351591, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83849[] = {SGI351592, SGI351593, SGI351594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351594, SGI0, NULL};
const Segment * const SG83850[] = {SGI351595, SGI351596, SGI351597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351598, SGI0, NULL};
const Segment * const SG83851[] = {SGI351599, SGI351600, SGI351601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351602, SGI0, NULL};
const Segment * const SG83852[] = {SGI351603, SGI351604, SGI351605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351606, SGI0, NULL};
const Segment * const SG83853[] = {SGI351607, SGI351608, SGI351609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351610, SGI0, NULL};
const Segment * const SG83854[] = {SGI351611, SGI351612, SGI351613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351614, SGI0, NULL};
const Segment * const SG83855[] = {SGI351615, SGI351616, SGI351617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351618, SGI0, NULL};
const Segment * const SG83856[] = {SGI351619, SGI351620, SGI351621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351622, SGI0, NULL};
const Segment * const SG83857[] = {SGI0, SGI351623, SGI351624, SGI0, SGI0, SGI0, SGI351626, NULL};
const Segment * const SG83858[] = {SGI351625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351627, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83859[] = {SGI351628, SGI351629, SGI351630, SGI0, SGI0, SGI0, SGI0, SGI351631, SGI0, SGI0, NULL};
const Segment * const SG83860[] = {SGI351632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351633, SGI0, NULL};
const Segment * const SG83861[] = {SGI351634, SGI351635, SGI351636, SGI0, SGI0, SGI0, SGI0, SGI351637, SGI0, SGI0, NULL};
const Segment * const SG83862[] = {SGI351638, SGI0, SGI0, SGI351639, SGI0, SGI0, SGI0, SGI0, SGI351640, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83863[] = {SGI351641, SGI0, SGI0, SGI351642, SGI0, SGI0, SGI0, SGI0, SGI351643, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83864[] = {SGI351644, SGI0, SGI0, SGI351645, SGI0, SGI0, SGI0, SGI0, SGI351646, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83865[] = {SGI351647, SGI0, SGI0, SGI351648, SGI0, SGI0, SGI0, SGI0, SGI351649, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83866[] = {SGI351650, SGI0, SGI0, SGI351651, SGI0, SGI0, SGI0, SGI0, SGI351652, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83867[] = {SGI351653, SGI0, SGI0, SGI351654, SGI0, SGI0, SGI0, SGI0, SGI351655, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83868[] = {SGI351656, SGI0, SGI351657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83869[] = {SGI351658, SGI0, SGI0, SGI351659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83870[] = {SGI351660, SGI0, SGI351661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83871[] = {SGI351662, SGI351663, SGI351664, SGI0, SGI0, SGI0, SGI0, SGI351665, SGI0, SGI0, NULL};
const Segment * const SG83872[] = {SGI351666, SGI351667, SGI351668, SGI0, SGI0, SGI0, SGI0, SGI351669, SGI0, SGI0, NULL};
const Segment * const SG83873[] = {SGI351670, SGI0, SGI351671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351671, SGI0, NULL};
const Segment * const SG83874[] = {SGI351672, SGI0, SGI351673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351673, SGI0, NULL};
const Segment * const SG83875[] = {SGI351674, SGI0, SGI351675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351675, SGI0, NULL};
const Segment * const SG83876[] = {SGI351676, SGI0, SGI351677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351677, SGI0, NULL};
const Segment * const SG83877[] = {SGI351678, SGI0, SGI351679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351679, SGI0, NULL};
const Segment * const SG83878[] = {SGI351680, SGI0, SGI351681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351681, SGI0, NULL};
const Segment * const SG83879[] = {SGI351682, SGI0, SGI351683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351683, SGI0, NULL};
const Segment * const SG83880[] = {SGI351684, SGI0, SGI351685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351685, SGI0, NULL};
const Segment * const SG83881[] = {SGI351686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351687, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83882[] = {SGI351688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351689, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83883[] = {SGI351690, SGI0, SGI351691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83884[] = {SGI351692, SGI0, SGI0, SGI351693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83885[] = {SGI351694, SGI0, SGI0, SGI351695, SGI0, SGI0, SGI0, SGI0, SGI351696, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83886[] = {SGI351697, SGI351698, SGI351699, SGI0, SGI0, SGI0, SGI0, SGI351700, SGI0, SGI0, NULL};
const Segment * const SG83887[] = {SGI351701, SGI0, SGI0, SGI351702, SGI0, SGI0, SGI0, SGI0, SGI351703, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83888[] = {SGI351704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83889[] = {SGI351705, SGI0, SGI351706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83890[] = {SGI351707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351708, SGI0, NULL};
const Segment * const SG83891[] = {SGI351709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351710, SGI0, NULL};
const Segment * const SG83892[] = {SGI351711, SGI0, SGI0, SGI351712, SGI0, SGI0, SGI0, SGI0, SGI351713, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83893[] = {SGI351714, SGI0, SGI351715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83894[] = {SGI351716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351717, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83895[] = {SGI351718, SGI351719, SGI0, SGI0, NULL};
const Segment * const SG83896[] = {SGI351720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351721, SGI0, NULL};
const Segment * const SG83897[] = {SGI351722, SGI0, SGI0, SGI351723, SGI0, SGI0, SGI0, SGI0, SGI351724, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83898[] = {SGI351725, SGI0, SGI0, SGI351726, SGI0, SGI0, SGI0, SGI0, SGI351727, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83899[] = {SGI351728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351729, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83900[] = {SGI351730, SGI0, SGI0, SGI351731, SGI0, SGI0, SGI0, SGI0, SGI351732, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83901[] = {SGI351733, SGI0, SGI351734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83902[] = {SGI351735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351736, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83903[] = {SGI351737, SGI351738, SGI0, SGI351739, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83904[] = {SGI351740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351741, SGI0, NULL};
const Segment * const SG83905[] = {SGI351742, SGI0, SGI351743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83906[] = {SGI351744, SGI351745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83907[] = {SGI351746, SGI351747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83908[] = {SGI351748, SGI351749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83909[] = {SGI351750, SGI0, SGI351751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83910[] = {SGI351752, SGI0, SGI0, SGI351753, SGI0, SGI0, SGI0, SGI0, SGI351754, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83911[] = {SGI0, SGI351755, SGI351756, SGI0, SGI0, SGI0, SGI351757, NULL};
const Segment * const SG83912[] = {SGI351758, SGI0, SGI0, SGI351759, SGI0, SGI0, SGI0, SGI0, SGI351760, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83913[] = {SGI351761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351762, SGI0, NULL};
const Segment * const SG83914[] = {SGI351763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351764, SGI0, NULL};
const Segment * const SG83915[] = {SGI351765, SGI351766, SGI0, SGI0, NULL};
const Segment * const SG83916[] = {SGI351767, SGI351768, SGI0, SGI0, NULL};
const Segment * const SG83917[] = {SGI351769, SGI351770, SGI0, SGI0, NULL};
const Segment * const SG83918[] = {SGI351771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83919[] = {SGI351772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351773, SGI0, NULL};
const Segment * const SG83920[] = {SGI351774, SGI0, SGI0, SGI351775, SGI0, SGI0, SGI0, SGI0, SGI351776, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83921[] = {SGI351777, SGI0, SGI0, SGI351778, SGI0, SGI0, SGI0, SGI0, SGI351779, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83922[] = {SGI351780, SGI0, SGI0, SGI351781, SGI0, SGI0, SGI0, SGI0, SGI351782, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83923[] = {SGI351783, SGI0, SGI0, SGI351784, SGI0, SGI0, SGI0, SGI0, SGI351785, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83924[] = {SGI351786, SGI351787, SGI351788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83925[] = {SGI351789, SGI351790, SGI351791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83926[] = {SGI351792, SGI351793, SGI351794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83927[] = {SGI351796, SGI351797, SGI351799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351799, SGI0, NULL};
const Segment * const SG83928[] = {SGI351795, SGI0, SGI351798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83929[] = {SGI351800, SGI0, SGI351801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83930[] = {SGI351802, SGI0, SGI351803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351803, SGI0, NULL};
const Segment * const SG83931[] = {SGI351804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83932[] = {SGI351805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83933[] = {SGI351806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83934[] = {SGI351807, SGI0, SGI0, SGI351808, SGI0, SGI0, SGI0, SGI0, SGI351809, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83935[] = {SGI351810, SGI0, SGI0, SGI351811, SGI0, SGI0, SGI0, SGI0, SGI351812, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83936[] = {SGI351813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83937[] = {SGI351814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83938[] = {SGI351815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351816, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83939[] = {SGI351817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83940[] = {SGI351818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83941[] = {SGI351819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83942[] = {SGI351820, SGI0, SGI0, SGI351821, SGI0, SGI0, SGI0, SGI0, SGI351822, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83943[] = {SGI351823, SGI0, SGI0, SGI351824, SGI0, SGI0, SGI0, SGI0, SGI351825, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83944[] = {SGI351826, SGI351827, SGI0, SGI0, NULL};
const Segment * const SG83945[] = {SGI351828, SGI0, SGI351829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83946[] = {SGI351830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351831, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83947[] = {SGI351832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351833, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83948[] = {SGI351834, SGI0, SGI0, SGI351835, SGI0, SGI0, SGI0, SGI0, SGI351836, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83949[] = {SGI351837, SGI351838, SGI351839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351839, SGI0, NULL};
const Segment * const SG83950[] = {SGI351840, SGI351841, SGI351842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351843, SGI0, NULL};
const Segment * const SG83951[] = {SGI351844, SGI351845, SGI351846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351847, SGI0, NULL};
const Segment * const SG83952[] = {SGI351848, SGI351849, SGI351850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351851, SGI0, NULL};
const Segment * const SG83953[] = {SGI351852, SGI351853, SGI351854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351855, SGI0, NULL};
const Segment * const SG83954[] = {SGI351856, SGI351857, SGI351858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351859, SGI0, NULL};
const Segment * const SG83955[] = {SGI351860, SGI351861, SGI351862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351863, SGI0, NULL};
const Segment * const SG83956[] = {SGI351864, SGI351865, SGI351866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351867, SGI0, NULL};
const Segment * const SG83957[] = {SGI351868, SGI351869, SGI351870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351871, SGI0, NULL};
const Segment * const SG83958[] = {SGI351872, SGI351873, SGI351874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351875, SGI0, NULL};
const Segment * const SG83959[] = {SGI351876, SGI351877, SGI351878, SGI0, SGI351879, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83960[] = {SGI351880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351881, SGI0, NULL};
const Segment * const SG83961[] = {SGI351882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351883, SGI0, NULL};
const Segment * const SG83962[] = {SGI351884, SGI351885, NULL};
const Segment * const SG83963[] = {SGI351886, SGI351887, NULL};
const Segment * const SG83964[] = {SGI351888, SGI351889, NULL};
const Segment * const SG83965[] = {SGI351890, SGI351891, NULL};
const Segment * const SG83966[] = {SGI351892, SGI351893, SGI351894, SGI0, SGI0, SGI0, SGI0, SGI351895, SGI0, SGI0, NULL};
const Segment * const SG83967[] = {SGI351896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83968[] = {SGI351897, SGI351898, SGI0, SGI0, NULL};
const Segment * const SG83969[] = {SGI351899, SGI351900, SGI0, SGI0, NULL};
const Segment * const SG83970[] = {SGI351901, SGI351902, SGI0, SGI0, NULL};
const Segment * const SG83971[] = {SGI351903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351904, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83972[] = {SGI351905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351906, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83973[] = {SGI351907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351908, SGI0, NULL};
const Segment * const SG83974[] = {SGI351909, SGI0, SGI0, SGI351910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83975[] = {SGI351911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351912, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83976[] = {SGI351913, SGI351914, NULL};
const Segment * const SG83977[] = {SGI351915, SGI351916, NULL};
const Segment * const SG83978[] = {SGI351917, SGI351918, NULL};
const Segment * const SG83979[] = {SGI351919, SGI351920, NULL};
const Segment * const SG83980[] = {SGI351921, SGI0, SGI0, SGI351922, SGI0, SGI0, SGI0, SGI0, SGI351923, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83981[] = {SGI351924, SGI0, SGI351925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83982[] = {SGI351926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351927, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83983[] = {SGI351928, SGI0, SGI351929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83984[] = {SGI351930, SGI0, SGI351931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83985[] = {SGI351932, SGI0, SGI351933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83986[] = {SGI351934, SGI0, SGI0, SGI351935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83987[] = {SGI351936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351937, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83988[] = {SGI351938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83989[] = {SGI351939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83990[] = {SGI351940, SGI0, SGI0, SGI351941, SGI0, SGI0, SGI0, SGI0, SGI351942, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83991[] = {SGI351943, SGI0, SGI0, SGI351944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83992[] = {SGI351945, SGI0, SGI0, SGI351946, SGI0, SGI0, SGI0, SGI0, SGI351947, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83993[] = {SGI351948, SGI0, SGI351949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351949, SGI0, NULL};
const Segment * const SG83994[] = {SGI351950, SGI0, SGI351951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351951, SGI0, NULL};
const Segment * const SG83995[] = {SGI351953, SGI0, SGI351955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351955, SGI0, NULL};
const Segment * const SG83996[] = {SGI351952, SGI351954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83997[] = {SGI351956, SGI0, SGI351957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351957, SGI0, NULL};
const Segment * const SG83998[] = {SGI351958, SGI351959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG83999[] = {SGI351960, SGI0, SGI351961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351961, SGI0, NULL};
const Segment * const SG84000[] = {SGI351962, SGI351963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84001[] = {SGI351964, SGI0, SGI351965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351965, SGI0, NULL};
const Segment * const SG84002[] = {SGI351966, SGI351967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84003[] = {SGI351968, SGI0, SGI351969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351969, SGI0, NULL};
const Segment * const SG84004[] = {SGI351970, SGI0, SGI351971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351971, SGI0, NULL};
const Segment * const SG84005[] = {SGI351972, SGI351973, SGI0, SGI351974, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84006[] = {SGI351975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84007[] = {SGI351976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84008[] = {SGI351977, SGI0, SGI0, SGI351978, SGI0, SGI0, SGI0, SGI0, SGI351979, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84009[] = {SGI351980, SGI0, SGI351981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84010[] = {SGI351982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351983, SGI0, NULL};
const Segment * const SG84011[] = {SGI351984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351985, SGI0, NULL};
const Segment * const SG84012[] = {SGI351986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351987, SGI0, NULL};
const Segment * const SG84013[] = {SGI351988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI351989, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84014[] = {SGI351990, SGI0, SGI0, SGI351991, SGI0, SGI0, SGI0, SGI0, SGI351992, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84015[] = {SGI351993, SGI0, SGI0, SGI351994, SGI0, SGI0, SGI0, SGI0, SGI351995, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84016[] = {SGI351996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84017[] = {SGI351997, SGI0, SGI351998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84018[] = {SGI351999, SGI0, SGI352000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84019[] = {SGI352001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352002, SGI0, NULL};
const Segment * const SG84020[] = {SGI352003, SGI0, SGI0, SGI352004, SGI0, SGI0, SGI0, SGI0, SGI352005, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84021[] = {SGI352006, SGI0, SGI0, SGI352007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84022[] = {SGI352008, SGI0, SGI0, SGI352009, SGI0, SGI0, SGI0, SGI0, SGI352010, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84023[] = {SGI352011, SGI0, SGI0, SGI352012, SGI0, SGI0, SGI0, SGI0, SGI352013, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84024[] = {SGI352014, SGI0, SGI0, SGI352015, SGI0, SGI0, SGI0, SGI0, SGI352016, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84025[] = {SGI352017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352018, SGI0, NULL};
const Segment * const SG84026[] = {SGI352019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352020, SGI0, NULL};
const Segment * const SG84027[] = {SGI352021, SGI352022, SGI352023, SGI0, SGI0, SGI0, SGI0, SGI352024, SGI0, SGI0, NULL};
const Segment * const SG84028[] = {SGI352025, SGI0, SGI0, SGI352026, SGI0, SGI0, SGI0, SGI0, SGI352027, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84029[] = {SGI352028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84030[] = {SGI352029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84031[] = {SGI352030, SGI0, SGI352031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84032[] = {SGI352032, SGI0, SGI352033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84033[] = {SGI352034, SGI0, SGI0, SGI352035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84034[] = {SGI352036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352037, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84035[] = {SGI352038, SGI0, SGI0, SGI352039, SGI0, SGI0, SGI0, SGI0, SGI352040, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84036[] = {SGI352041, SGI0, SGI0, SGI352042, SGI0, SGI0, SGI0, SGI0, SGI352043, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84037[] = {SGI352044, SGI0, SGI0, SGI352045, SGI0, SGI0, SGI0, SGI0, SGI352046, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84038[] = {SGI352047, SGI352048, SGI352049, SGI0, SGI352050, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84039[] = {SGI352051, SGI0, SGI0, SGI352052, SGI0, SGI0, SGI0, SGI0, SGI352053, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84040[] = {SGI352054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84041[] = {SGI352055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352056, SGI0, NULL};
const Segment * const SG84042[] = {SGI352057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352058, SGI0, NULL};
const Segment * const SG84043[] = {SGI352059, SGI0, SGI0, SGI352060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84044[] = {SGI352061, SGI0, SGI352062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84045[] = {SGI0, SGI352063, SGI352064, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84046[] = {SGI352065, SGI0, SGI352066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352066, SGI0, NULL};
const Segment * const SG84047[] = {SGI352067, SGI0, SGI352068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352068, SGI0, NULL};
const Segment * const SG84048[] = {SGI352069, SGI0, SGI352070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352070, SGI0, NULL};
const Segment * const SG84049[] = {SGI352071, SGI0, SGI352072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352072, SGI0, NULL};
const Segment * const SG84050[] = {SGI352073, SGI0, SGI352074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352074, SGI0, NULL};
const Segment * const SG84051[] = {SGI352075, SGI0, SGI352076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352076, SGI0, NULL};
const Segment * const SG84052[] = {SGI352077, SGI0, SGI352078, SGI352078, SGI0, SGI0, SGI0, SGI0, SGI352078, SGI0, NULL};
const Segment * const SG84053[] = {SGI352079, SGI0, SGI352080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352080, SGI0, NULL};
const Segment * const SG84054[] = {SGI352081, SGI352082, SGI352083, SGI0, SGI0, SGI0, SGI0, SGI352084, SGI0, SGI0, NULL};
const Segment * const SG84055[] = {SGI352085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352086, SGI0, NULL};
const Segment * const SG84056[] = {SGI352087, SGI352088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84057[] = {SGI352089, SGI352090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84058[] = {SGI352091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352092, SGI0, NULL};
const Segment * const SG84059[] = {SGI352093, SGI352094, SGI352095, SGI0, SGI0, SGI0, SGI0, SGI352096, SGI0, SGI0, NULL};
const Segment * const SG84060[] = {SGI352097, SGI0, SGI352098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84061[] = {SGI352099, SGI0, SGI352100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84062[] = {SGI352101, SGI0, SGI352102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84063[] = {SGI352103, SGI0, SGI352104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84064[] = {SGI352105, SGI0, SGI352106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84065[] = {SGI352107, SGI0, SGI352108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84066[] = {SGI352109, SGI0, SGI0, SGI352110, SGI0, SGI0, SGI0, SGI0, SGI352111, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84067[] = {SGI352112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352113, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84068[] = {SGI352114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352115, SGI0, NULL};
const Segment * const SG84069[] = {SGI352116, SGI0, SGI0, SGI352117, SGI0, SGI0, SGI0, SGI0, SGI352118, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84070[] = {SGI352119, SGI0, SGI0, SGI352120, SGI0, SGI0, SGI0, SGI0, SGI352121, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84071[] = {SGI352122, SGI0, SGI0, SGI352123, SGI0, SGI0, SGI0, SGI0, SGI352124, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84072[] = {SGI352125, SGI0, SGI0, SGI352126, SGI0, SGI0, SGI0, SGI0, SGI352127, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84073[] = {SGI352128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84074[] = {SGI352129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352130, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84075[] = {SGI352131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84076[] = {SGI352132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352133, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84077[] = {SGI352134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352135, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84078[] = {SGI352136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352137, SGI0, NULL};
const Segment * const SG84079[] = {SGI352138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352139, SGI0, NULL};
const Segment * const SG84080[] = {SGI352140, SGI0, SGI0, SGI352141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84081[] = {SGI352142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352143, SGI0, NULL};
const Segment * const SG84082[] = {SGI352144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352145, SGI0, NULL};
const Segment * const SG84083[] = {SGI352146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352147, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84084[] = {SGI352148, SGI0, SGI0, SGI352149, SGI0, SGI0, SGI0, SGI0, SGI352150, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84085[] = {SGI352151, SGI0, SGI0, SGI352152, SGI0, SGI0, SGI0, SGI0, SGI352153, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84086[] = {SGI352154, SGI0, SGI0, SGI352155, SGI0, SGI0, SGI0, SGI0, SGI352156, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84087[] = {SGI352157, SGI0, SGI352158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84088[] = {SGI352159, SGI0, SGI352160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84089[] = {SGI352161, SGI0, SGI0, SGI352162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84090[] = {SGI352163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352164, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84091[] = {SGI352165, SGI0, SGI0, SGI352166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84092[] = {SGI352167, SGI0, SGI352168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84093[] = {SGI352169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352170, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84094[] = {SGI352171, SGI0, SGI352172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84095[] = {SGI352173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352174, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84096[] = {SGI352175, SGI0, SGI0, SGI352176, SGI0, SGI0, SGI0, SGI0, SGI352177, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84097[] = {SGI352178, SGI0, SGI0, SGI352179, SGI0, SGI0, SGI0, SGI0, SGI352180, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84098[] = {SGI352181, SGI0, SGI0, SGI352182, SGI0, SGI0, SGI0, SGI0, SGI352183, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84099[] = {SGI352184, SGI352185, SGI352186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84100[] = {SGI352187, SGI352188, SGI352189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84101[] = {SGI352190, SGI0, SGI352191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84102[] = {SGI352192, SGI0, SGI352193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84103[] = {SGI352194, SGI352195, SGI352196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352196, SGI0, NULL};
const Segment * const SG84104[] = {SGI352197, SGI352198, SGI352199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352200, SGI0, NULL};
const Segment * const SG84105[] = {SGI352201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352202, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84106[] = {SGI352203, SGI0, SGI0, SGI352204, SGI0, SGI0, SGI0, SGI0, SGI352205, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84107[] = {SGI352206, SGI0, SGI0, SGI352207, SGI0, SGI0, SGI0, SGI0, SGI352208, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84108[] = {SGI352209, SGI0, SGI352210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84109[] = {SGI352211, SGI0, SGI352212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84110[] = {SGI352213, SGI0, SGI352214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84111[] = {SGI352215, SGI0, SGI352216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84112[] = {SGI352217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352218, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84113[] = {SGI352219, SGI0, SGI0, SGI352220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84114[] = {SGI352221, SGI0, SGI352222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84115[] = {SGI352223, SGI0, SGI352224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84116[] = {SGI352225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352226, SGI0, NULL};
const Segment * const SG84117[] = {SGI352227, SGI0, SGI0, SGI352228, SGI0, SGI0, SGI0, SGI0, SGI352229, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84118[] = {SGI352230, SGI0, SGI352231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84119[] = {SGI352232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352233, SGI0, NULL};
const Segment * const SG84120[] = {SGI352234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352235, SGI0, NULL};
const Segment * const SG84121[] = {SGI352236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352237, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84122[] = {SGI352238, SGI0, SGI0, SGI352239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84123[] = {SGI352240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84124[] = {SGI352241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352242, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84125[] = {SGI352243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352244, SGI0, NULL};
const Segment * const SG84126[] = {SGI352245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84127[] = {SGI352246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352247, SGI0, NULL};
const Segment * const SG84128[] = {SGI352248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352249, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84129[] = {SGI352250, SGI0, SGI0, SGI352251, SGI0, SGI0, SGI0, SGI0, SGI352252, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84130[] = {SGI352253, SGI352254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84131[] = {SGI352255, SGI352256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84132[] = {SGI352257, SGI352258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84133[] = {SGI352259, SGI352260, SGI352261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84134[] = {SGI352262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352263, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84135[] = {SGI352264, SGI352265, SGI352266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84136[] = {SGI352267, SGI352268, SGI352269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84137[] = {SGI352270, SGI352271, SGI352272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84138[] = {SGI352273, SGI0, SGI0, SGI352274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84139[] = {SGI352275, SGI352276, SGI352277, SGI0, SGI0, SGI0, SGI0, SGI352278, SGI0, SGI0, NULL};
const Segment * const SG84140[] = {SGI0, SGI352279, SGI352280, SGI352281, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84141[] = {SGI352282, SGI0, SGI352283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84142[] = {SGI352284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352285, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84143[] = {SGI352286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352287, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84144[] = {SGI352288, SGI0, SGI0, SGI352289, SGI0, SGI0, SGI0, SGI0, SGI352290, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84145[] = {SGI352291, SGI352292, SGI352293, SGI0, SGI352294, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84146[] = {SGI352295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84147[] = {SGI352296, SGI0, SGI352297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84148[] = {SGI352298, SGI0, SGI0, SGI352299, SGI0, SGI0, SGI0, SGI0, SGI352300, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84149[] = {SGI352301, SGI0, SGI0, SGI352302, SGI0, SGI0, SGI0, SGI0, SGI352303, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84150[] = {SGI352304, SGI0, SGI0, SGI352305, SGI0, SGI0, SGI0, SGI0, SGI352306, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84151[] = {SGI352307, SGI0, SGI0, SGI352308, SGI0, SGI0, SGI0, SGI0, SGI352309, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84152[] = {SGI352310, SGI0, SGI352311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84153[] = {SGI352312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352313, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84154[] = {SGI352314, SGI0, SGI0, SGI352315, SGI0, SGI0, SGI0, SGI0, SGI352316, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84155[] = {SGI352317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84156[] = {SGI352318, SGI0, SGI352319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84157[] = {SGI352320, SGI0, SGI352321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84158[] = {SGI352322, SGI0, SGI352323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84159[] = {SGI352324, SGI0, SGI0, SGI352325, SGI0, SGI0, SGI0, SGI0, SGI352326, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84160[] = {SGI352327, SGI0, SGI0, SGI352328, SGI0, SGI0, SGI0, SGI0, SGI352329, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84161[] = {SGI352330, SGI0, SGI352331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84162[] = {SGI352332, SGI352333, SGI0, SGI0, NULL};
const Segment * const SG84163[] = {SGI352334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352335, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84164[] = {SGI352336, SGI0, SGI352337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84165[] = {SGI352338, SGI0, SGI352339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84166[] = {SGI352340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352341, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84167[] = {SGI352342, SGI0, SGI352343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84168[] = {SGI352344, SGI0, SGI0, SGI352345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84169[] = {SGI352346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352347, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84170[] = {SGI352348, SGI352349, SGI352350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84171[] = {SGI352351, SGI352352, SGI352353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84172[] = {SGI352354, SGI0, SGI0, SGI352355, SGI0, SGI0, SGI0, SGI0, SGI352356, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84173[] = {SGI352357, SGI0, SGI0, SGI352358, SGI0, SGI0, SGI0, SGI0, SGI352359, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84174[] = {SGI352360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352361, SGI0, NULL};
const Segment * const SG84175[] = {SGI352362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352363, SGI0, NULL};
const Segment * const SG84176[] = {SGI0, SGI352364, SGI352365, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84177[] = {SGI352366, SGI0, SGI352367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84178[] = {SGI352368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352369, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84179[] = {SGI0, SGI352370, SGI352371, SGI0, SGI0, SGI0, SGI352372, NULL};
const Segment * const SG84180[] = {SGI352373, SGI0, SGI0, SGI352374, SGI0, SGI0, SGI0, SGI0, SGI352375, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84181[] = {SGI352376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352377, SGI0, NULL};
const Segment * const SG84182[] = {SGI352378, SGI352379, SGI352380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84183[] = {SGI352381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352382, SGI0, NULL};
const Segment * const SG84184[] = {SGI352383, SGI0, SGI0, SGI352384, SGI0, SGI0, SGI0, SGI0, SGI352385, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84185[] = {SGI352386, SGI352387, SGI352388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352388, SGI0, NULL};
const Segment * const SG84186[] = {SGI352389, SGI352390, SGI352391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352391, SGI0, NULL};
const Segment * const SG84187[] = {SGI352392, SGI352393, SGI352394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352394, SGI0, NULL};
const Segment * const SG84188[] = {SGI352395, SGI352396, SGI352397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352397, SGI0, NULL};
const Segment * const SG84189[] = {SGI352398, SGI0, SGI352399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84190[] = {SGI352400, SGI0, SGI0, SGI352401, SGI0, SGI0, SGI0, SGI0, SGI352402, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84191[] = {SGI352403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84192[] = {SGI352404, SGI0, SGI352405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84193[] = {SGI352406, SGI0, SGI352407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84194[] = {SGI352408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352409, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84195[] = {SGI352410, SGI0, SGI0, SGI352411, SGI0, SGI0, SGI0, SGI0, SGI352412, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84196[] = {SGI352413, SGI352414, SGI352415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84197[] = {SGI352416, SGI352417, SGI352418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84198[] = {SGI352419, SGI352420, SGI352421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84199[] = {SGI352422, SGI0, SGI0, SGI352423, SGI0, SGI0, SGI0, SGI0, SGI352424, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84200[] = {SGI352425, SGI352426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84201[] = {SGI352427, SGI352428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84202[] = {SGI352429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84203[] = {SGI352430, SGI352431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84204[] = {SGI352432, SGI0, SGI352433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84205[] = {SGI352434, SGI0, SGI352435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84206[] = {SGI352436, SGI0, SGI352437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84207[] = {SGI352438, SGI0, SGI352439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84208[] = {SGI352440, SGI0, SGI0, SGI352441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84209[] = {SGI352442, SGI0, SGI0, SGI352443, SGI0, SGI0, SGI0, SGI0, SGI352444, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84210[] = {SGI352445, SGI0, SGI352446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84211[] = {SGI352447, SGI0, SGI0, SGI352448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84212[] = {SGI352449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352450, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84213[] = {SGI0, SGI352451, SGI352452, SGI352453, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84214[] = {SGI352454, SGI352455, SGI352456, SGI0, SGI0, SGI0, SGI0, SGI352457, SGI0, SGI0, NULL};
const Segment * const SG84215[] = {SGI352458, SGI352459, SGI352460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84216[] = {SGI352461, SGI352462, SGI352463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84217[] = {SGI352464, SGI0, SGI352465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84218[] = {SGI352466, SGI352467, SGI352468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84219[] = {SGI352469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352470, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84220[] = {SGI352471, SGI0, SGI0, SGI352472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84221[] = {SGI352473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352474, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84222[] = {SGI352475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84223[] = {SGI352476, SGI0, SGI352477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84224[] = {SGI352478, SGI0, SGI352479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84225[] = {SGI352480, SGI0, SGI352481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84226[] = {SGI352482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352483, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84227[] = {SGI352484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352485, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84228[] = {SGI352486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84229[] = {SGI352487, SGI0, SGI0, SGI352488, SGI0, SGI0, SGI0, SGI0, SGI352489, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84230[] = {SGI352490, SGI0, SGI0, SGI352491, SGI0, SGI0, SGI0, SGI0, SGI352492, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84231[] = {SGI352493, SGI352494, SGI352495, SGI0, SGI0, SGI0, SGI0, SGI352496, SGI0, SGI0, NULL};
const Segment * const SG84232[] = {SGI352497, SGI0, SGI352498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352498, SGI0, NULL};
const Segment * const SG84233[] = {SGI352499, SGI0, SGI352500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352500, SGI0, NULL};
const Segment * const SG84234[] = {SGI352501, SGI0, SGI352502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352502, SGI0, NULL};
const Segment * const SG84235[] = {SGI352503, SGI0, SGI0, SGI352504, SGI0, SGI0, SGI0, SGI0, SGI352505, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84236[] = {SGI352506, SGI0, SGI352507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84237[] = {SGI352508, SGI0, SGI352509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84238[] = {SGI352510, SGI352511, SGI352512, SGI0, SGI352513, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84239[] = {SGI352514, SGI352515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84240[] = {SGI352516, SGI352517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84241[] = {SGI352518, SGI352519, SGI0, SGI352520, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84242[] = {SGI352521, SGI0, SGI352522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84243[] = {SGI352523, SGI0, SGI352524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84244[] = {SGI352525, SGI0, SGI352526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84245[] = {SGI352527, SGI0, SGI352528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84246[] = {SGI352529, SGI0, SGI0, SGI352530, SGI0, SGI0, SGI0, SGI0, SGI352531, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84247[] = {SGI352532, SGI0, SGI0, SGI352533, SGI0, SGI0, SGI0, SGI0, SGI352534, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84248[] = {SGI352535, SGI0, SGI0, SGI352536, SGI0, SGI0, SGI0, SGI0, SGI352537, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84249[] = {SGI352538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84250[] = {SGI352539, SGI0, SGI0, SGI352540, SGI0, SGI0, SGI0, SGI0, SGI352541, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84251[] = {SGI352542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352543, SGI0, NULL};
const Segment * const SG84252[] = {SGI352544, SGI0, SGI0, SGI352545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84253[] = {SGI352546, SGI352547, SGI0, SGI352548, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84254[] = {SGI352549, SGI0, SGI0, SGI352550, SGI0, SGI0, SGI0, SGI0, SGI352551, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84255[] = {SGI352552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352553, SGI0, NULL};
const Segment * const SG84256[] = {SGI352554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84257[] = {SGI352555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84258[] = {SGI352556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84259[] = {SGI352557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84260[] = {SGI352558, SGI0, SGI352559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84261[] = {SGI352560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352561, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84262[] = {SGI352562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352563, SGI0, NULL};
const Segment * const SG84263[] = {SGI352564, SGI0, SGI352565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84264[] = {SGI352566, SGI0, SGI352567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84265[] = {SGI352568, SGI0, SGI352569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84266[] = {SGI352570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84267[] = {SGI352571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84268[] = {SGI352572, SGI0, SGI0, SGI352573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84269[] = {SGI352574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352575, SGI0, NULL};
const Segment * const SG84270[] = {SGI352576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352577, SGI0, NULL};
const Segment * const SG84271[] = {SGI352578, SGI0, SGI0, SGI352579, SGI0, SGI0, SGI0, SGI0, SGI352580, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84272[] = {SGI352581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352582, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84273[] = {SGI352583, SGI0, SGI0, SGI352584, SGI0, SGI0, SGI0, SGI0, SGI352585, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84274[] = {SGI352586, SGI0, SGI352587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84275[] = {SGI352588, SGI0, SGI0, SGI352589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84276[] = {SGI352590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352591, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84277[] = {SGI352592, SGI0, SGI352593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84278[] = {SGI352594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352595, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84279[] = {SGI352596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352597, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84280[] = {SGI352598, SGI0, SGI0, SGI352599, SGI0, SGI0, SGI0, SGI0, SGI352600, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84281[] = {SGI352601, SGI0, SGI0, SGI352602, SGI0, SGI0, SGI0, SGI0, SGI352603, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84282[] = {SGI352604, SGI0, SGI0, SGI352605, SGI0, SGI0, SGI0, SGI0, SGI352606, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84283[] = {SGI352607, SGI0, SGI0, SGI352608, SGI0, SGI0, SGI0, SGI0, SGI352609, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84284[] = {SGI352610, SGI0, SGI0, SGI352611, SGI0, SGI0, SGI0, SGI0, SGI352612, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84285[] = {SGI352613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352614, SGI0, NULL};
const Segment * const SG84286[] = {SGI352615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352616, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84287[] = {SGI352617, SGI0, SGI0, SGI352618, SGI0, SGI0, SGI0, SGI0, SGI352619, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84288[] = {SGI352620, SGI0, SGI352621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84289[] = {SGI352622, SGI0, SGI0, SGI352623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84290[] = {SGI352624, SGI352625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84291[] = {SGI352626, SGI352627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84292[] = {SGI352628, SGI352629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84293[] = {SGI352630, SGI352631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84294[] = {SGI352632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352633, SGI0, NULL};
const Segment * const SG84295[] = {SGI352634, SGI0, SGI352635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84296[] = {SGI352636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352637, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84297[] = {SGI352638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352639, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84298[] = {SGI352640, SGI352641, NULL};
const Segment * const SG84299[] = {SGI352642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352643, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84300[] = {SGI352644, SGI0, SGI0, SGI352645, SGI0, SGI0, SGI0, SGI0, SGI352646, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84301[] = {SGI352647, SGI0, SGI0, SGI352648, SGI0, SGI0, SGI0, SGI0, SGI352649, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84302[] = {SGI352650, SGI0, SGI0, SGI352651, SGI0, SGI0, SGI0, SGI0, SGI352652, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84303[] = {SGI352653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352654, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84304[] = {SGI352655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352656, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84305[] = {SGI352657, SGI0, SGI0, SGI352658, SGI0, SGI0, SGI0, SGI0, SGI352659, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84306[] = {SGI352660, SGI0, SGI0, SGI352661, SGI0, SGI0, SGI0, SGI0, SGI352662, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84307[] = {SGI352663, SGI0, SGI0, SGI352664, SGI0, SGI0, SGI0, SGI0, SGI352665, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84308[] = {SGI352666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84309[] = {SGI352667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352668, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84310[] = {SGI352669, SGI0, SGI352670, SGI352670, SGI0, SGI0, SGI0, SGI0, SGI352670, SGI0, NULL};
const Segment * const SG84311[] = {SGI352671, SGI0, SGI0, SGI352672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84312[] = {SGI352673, SGI0, SGI352674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84313[] = {SGI352675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352676, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84314[] = {SGI352677, SGI0, SGI352678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84315[] = {SGI352679, SGI0, SGI0, SGI352680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84316[] = {SGI352681, SGI352682, SGI352683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84317[] = {SGI352684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352685, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84318[] = {SGI352686, SGI352687, SGI352688, SGI0, SGI0, SGI0, SGI0, SGI352689, SGI0, SGI0, NULL};
const Segment * const SG84319[] = {SGI352690, SGI0, SGI0, SGI352691, SGI0, SGI0, SGI0, SGI0, SGI352692, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84320[] = {SGI352693, SGI0, SGI0, SGI352694, SGI0, SGI0, SGI0, SGI0, SGI352695, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84321[] = {SGI352696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352697, SGI0, NULL};
const Segment * const SG84322[] = {SGI352698, SGI0, SGI0, SGI352699, SGI0, SGI0, SGI0, SGI0, SGI352700, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84323[] = {SGI352701, SGI0, SGI0, SGI352702, SGI0, SGI0, SGI0, SGI0, SGI352703, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84324[] = {SGI352704, SGI0, SGI352705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84325[] = {SGI352706, SGI0, SGI0, SGI352707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84326[] = {SGI352708, SGI0, SGI352709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84327[] = {SGI352710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352711, SGI0, NULL};
const Segment * const SG84328[] = {SGI352712, SGI0, SGI0, SGI352713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84329[] = {SGI352714, SGI0, SGI0, SGI352715, SGI0, SGI0, SGI0, SGI0, SGI352716, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84330[] = {SGI352717, SGI0, SGI0, SGI352718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84331[] = {SGI352719, SGI0, SGI352720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84332[] = {SGI352721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352722, SGI0, NULL};
const Segment * const SG84333[] = {SGI352723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352724, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84334[] = {SGI352725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352726, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84335[] = {SGI352727, SGI352728, SGI0, SGI0, NULL};
const Segment * const SG84336[] = {SGI352729, SGI352730, SGI0, SGI0, NULL};
const Segment * const SG84337[] = {SGI352731, SGI352732, SGI0, SGI0, NULL};
const Segment * const SG84338[] = {SGI352733, SGI0, SGI0, SGI352734, SGI0, SGI0, SGI0, SGI0, SGI352735, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84339[] = {SGI352736, SGI352737, SGI352738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352738, SGI0, NULL};
const Segment * const SG84340[] = {SGI352739, SGI0, SGI0, SGI352740, SGI0, SGI0, SGI0, SGI0, SGI352741, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84341[] = {SGI352742, SGI0, SGI352745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84342[] = {SGI352743, SGI352744, SGI352746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352746, SGI0, NULL};
const Segment * const SG84343[] = {SGI352747, SGI352748, SGI352749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352749, SGI0, NULL};
const Segment * const SG84344[] = {SGI352750, SGI0, SGI352751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84345[] = {SGI352752, SGI0, SGI0, SGI352753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84346[] = {SGI352754, SGI352755, SGI352756, SGI0, SGI0, SGI0, SGI0, SGI352757, SGI0, SGI0, NULL};
const Segment * const SG84347[] = {SGI352758, SGI352759, SGI352760, SGI0, SGI0, SGI0, SGI0, SGI352761, SGI0, SGI0, NULL};
const Segment * const SG84348[] = {SGI352762, SGI0, SGI0, SGI352763, SGI0, SGI0, SGI0, SGI0, SGI352764, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84349[] = {SGI352765, SGI0, SGI0, SGI352766, SGI0, SGI0, SGI0, SGI0, SGI352767, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84350[] = {SGI352768, SGI0, SGI352769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352769, SGI0, NULL};
const Segment * const SG84351[] = {SGI352770, SGI0, SGI352771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352771, SGI0, NULL};
const Segment * const SG84352[] = {SGI352772, SGI0, SGI352773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352773, SGI0, NULL};
const Segment * const SG84353[] = {SGI352774, SGI0, SGI352775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352775, SGI0, NULL};
const Segment * const SG84354[] = {SGI352776, SGI0, SGI352777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352777, SGI0, NULL};
const Segment * const SG84355[] = {SGI352778, SGI0, SGI352779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84356[] = {SGI352780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352781, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84357[] = {SGI352782, SGI0, SGI0, SGI352783, SGI0, SGI0, SGI0, SGI0, SGI352784, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84358[] = {SGI352785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352786, SGI0, NULL};
const Segment * const SG84359[] = {SGI352787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84360[] = {SGI352788, SGI352789, SGI352790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84361[] = {SGI352791, SGI352792, SGI352793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84362[] = {SGI352794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352795, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84363[] = {SGI352796, SGI0, SGI352797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84364[] = {SGI352798, SGI0, SGI0, SGI352799, SGI0, SGI0, SGI0, SGI0, SGI352800, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84365[] = {SGI352801, SGI0, SGI0, SGI352802, SGI0, SGI0, SGI0, SGI0, SGI352803, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84366[] = {SGI352804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84367[] = {SGI352805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352806, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84368[] = {SGI352807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352808, SGI0, NULL};
const Segment * const SG84369[] = {SGI352809, SGI0, SGI352810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84370[] = {SGI352811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352812, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84371[] = {SGI352813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352814, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84372[] = {SGI352815, SGI0, SGI0, SGI352816, SGI0, SGI0, SGI0, SGI0, SGI352817, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84373[] = {SGI352818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352819, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84374[] = {SGI352820, SGI0, SGI0, SGI352821, SGI0, SGI0, SGI0, SGI0, SGI352822, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84375[] = {SGI352823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352824, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84376[] = {SGI352825, SGI0, SGI352826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84377[] = {SGI352827, SGI0, SGI352828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84378[] = {SGI352829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352830, SGI0, NULL};
const Segment * const SG84379[] = {SGI352831, SGI0, SGI0, SGI352832, SGI0, SGI0, SGI0, SGI0, SGI352833, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84380[] = {SGI352834, SGI0, SGI0, SGI352835, SGI0, SGI0, SGI0, SGI0, SGI352836, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84381[] = {SGI352837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84382[] = {SGI352838, SGI0, SGI352839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84383[] = {SGI352840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352841, SGI0, NULL};
const Segment * const SG84384[] = {SGI352842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352843, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84385[] = {SGI352844, SGI0, SGI0, SGI352845, SGI0, SGI0, SGI0, SGI0, SGI352846, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84386[] = {SGI352847, SGI0, SGI0, SGI352848, SGI0, SGI0, SGI0, SGI0, SGI352849, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84387[] = {SGI352850, SGI0, SGI0, SGI352851, SGI0, SGI0, SGI0, SGI0, SGI352852, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84388[] = {SGI352853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352854, SGI0, NULL};
const Segment * const SG84389[] = {SGI352855, SGI0, SGI0, SGI352856, SGI0, SGI0, SGI0, SGI0, SGI352857, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84390[] = {SGI352858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84391[] = {SGI352859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352860, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84392[] = {SGI352861, SGI0, SGI0, SGI352862, SGI0, SGI0, SGI0, SGI0, SGI352863, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84393[] = {SGI352864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84394[] = {SGI352865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352866, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84395[] = {SGI352867, SGI0, SGI0, SGI352868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84396[] = {SGI352869, SGI352870, SGI352871, SGI0, SGI0, SGI0, SGI0, SGI352872, SGI0, SGI0, NULL};
const Segment * const SG84397[] = {SGI352873, SGI0, SGI0, SGI352874, SGI0, SGI0, SGI0, SGI0, SGI352875, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84398[] = {SGI352876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352877, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84399[] = {SGI352878, SGI0, SGI352879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84400[] = {SGI352880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84401[] = {SGI352881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84402[] = {SGI352882, SGI0, SGI0, SGI352883, SGI0, SGI0, SGI0, SGI0, SGI352884, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84403[] = {SGI352885, SGI0, SGI0, SGI352886, SGI0, SGI0, SGI0, SGI0, SGI352887, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84404[] = {SGI352888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84405[] = {SGI352889, SGI0, SGI0, SGI352890, SGI0, SGI0, SGI0, SGI0, SGI352891, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84406[] = {SGI352892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352893, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84407[] = {SGI352894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84408[] = {SGI352895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84409[] = {SGI352896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352897, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84410[] = {SGI352898, SGI0, SGI352899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84411[] = {SGI352900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352901, SGI0, NULL};
const Segment * const SG84412[] = {SGI0, SGI352902, SGI352903, SGI0, SGI0, SGI0, SGI352904, NULL};
const Segment * const SG84413[] = {SGI352905, SGI352906, SGI352907, SGI0, SGI0, SGI0, SGI0, SGI352908, SGI0, SGI0, NULL};
const Segment * const SG84414[] = {SGI352909, SGI0, SGI0, SGI352910, SGI0, SGI0, SGI0, SGI0, SGI352911, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84415[] = {SGI352912, SGI0, SGI352913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84416[] = {SGI352914, SGI0, SGI352915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84417[] = {SGI352916, SGI352917, NULL};
const Segment * const SG84418[] = {SGI352918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352919, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84419[] = {SGI352920, SGI0, SGI0, SGI352921, SGI0, SGI0, SGI0, SGI0, SGI352922, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84420[] = {SGI352923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352924, SGI0, NULL};
const Segment * const SG84421[] = {SGI352925, SGI0, SGI0, SGI352926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84422[] = {SGI352927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352928, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84423[] = {SGI352929, SGI0, SGI0, SGI352930, SGI0, SGI0, SGI0, SGI0, SGI352931, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84424[] = {SGI352932, SGI0, SGI0, SGI352933, SGI0, SGI0, SGI0, SGI0, SGI352934, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84425[] = {SGI352935, SGI0, SGI0, SGI352936, SGI0, SGI0, SGI0, SGI0, SGI352937, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84426[] = {SGI352938, SGI0, SGI0, SGI352939, SGI0, SGI0, SGI0, SGI0, SGI352940, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84427[] = {SGI352941, SGI0, SGI0, SGI352942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84428[] = {SGI352943, SGI0, SGI0, SGI352944, SGI0, SGI0, SGI0, SGI0, SGI352945, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84429[] = {SGI352946, SGI352947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84430[] = {SGI352948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352949, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84431[] = {SGI352950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352951, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84432[] = {SGI352952, SGI352953, SGI352954, SGI352955, NULL};
const Segment * const SG84433[] = {SGI352956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352957, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84434[] = {SGI352958, SGI0, SGI352959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84435[] = {SGI352960, SGI0, SGI352961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84436[] = {SGI352962, SGI0, SGI352963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84437[] = {SGI352964, SGI0, SGI0, SGI352965, SGI0, SGI0, SGI0, SGI0, SGI352966, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84438[] = {SGI352967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352968, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84439[] = {SGI352969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352970, SGI0, NULL};
const Segment * const SG84440[] = {SGI352971, SGI0, SGI0, SGI352972, SGI0, SGI0, SGI0, SGI0, SGI352973, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84441[] = {SGI352974, SGI0, SGI352975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84442[] = {SGI352976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352977, SGI0, NULL};
const Segment * const SG84443[] = {SGI352978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84444[] = {SGI352979, SGI0, SGI352980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84445[] = {SGI352981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352982, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84446[] = {SGI352983, SGI0, SGI0, SGI352984, SGI0, SGI0, SGI0, SGI0, SGI352985, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84447[] = {SGI352986, SGI0, SGI0, SGI352987, SGI0, SGI0, SGI0, SGI0, SGI352988, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84448[] = {SGI352989, SGI352990, SGI352991, SGI0, SGI0, SGI0, SGI0, SGI352992, SGI0, SGI0, NULL};
const Segment * const SG84449[] = {SGI352993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84450[] = {SGI352994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84451[] = {SGI352995, SGI0, SGI352996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84452[] = {SGI352997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI352998, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84453[] = {SGI352999, SGI353000, SGI0, SGI353001, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84454[] = {SGI353002, SGI353003, SGI0, SGI353004, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84455[] = {SGI353005, SGI0, SGI0, SGI353006, SGI0, SGI0, SGI0, SGI0, SGI353007, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84456[] = {SGI353008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353009, SGI0, NULL};
const Segment * const SG84457[] = {SGI353010, SGI0, SGI353011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84458[] = {SGI353012, SGI0, SGI353013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84459[] = {SGI353014, SGI0, SGI353015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84460[] = {SGI353016, SGI0, SGI353017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84461[] = {SGI353018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353019, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84462[] = {SGI353020, SGI0, SGI0, SGI353021, SGI0, SGI0, SGI0, SGI0, SGI353022, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84463[] = {SGI353023, SGI0, SGI0, SGI353024, SGI0, SGI0, SGI0, SGI0, SGI353025, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84464[] = {SGI353026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353027, SGI0, NULL};
const Segment * const SG84465[] = {SGI353028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353029, SGI0, NULL};
const Segment * const SG84466[] = {SGI353030, SGI0, SGI353031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84467[] = {SGI353032, SGI0, SGI353033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84468[] = {SGI353034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353035, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84469[] = {SGI45795, SGI45796, SGI353036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84470[] = {SGI45213, SGI45214, SGI353037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84471[] = {SGI43936, SGI43937, SGI353038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84472[] = {SGI45942, SGI45943, SGI353039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84473[] = {SGI82145, SGI82146, SGI353040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84474[] = {SGI353041, SGI353042, SGI353043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84475[] = {SGI84892, SGI84893, SGI353044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84476[] = {SGI45220, SGI45221, SGI353045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84477[] = {SGI45797, SGI45798, SGI353046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84478[] = {SGI43981, SGI43982, SGI353047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84479[] = {SGI82212, SGI82213, SGI353048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84480[] = {SGI353049, SGI0, SGI0, SGI353050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84481[] = {SGI46071, SGI46072, SGI353051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84482[] = {SGI353052, SGI0, SGI353053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84483[] = {SGI353054, SGI353055, SGI353056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84484[] = {SGI84944, SGI84945, SGI353057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84485[] = {SGI353058, SGI353059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84486[] = {SGI353060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353061, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84487[] = {SGI353062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353063, SGI0, NULL};
const Segment * const SG84488[] = {SGI353064, SGI0, SGI0, SGI353065, SGI0, SGI0, SGI0, SGI0, SGI353066, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84489[] = {SGI353067, SGI0, SGI0, SGI353068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84490[] = {SGI353069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353070, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84491[] = {SGI353071, SGI0, SGI353072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84492[] = {SGI353073, SGI0, SGI0, SGI353074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84493[] = {SGI353075, SGI353076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84494[] = {SGI353077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353078, SGI0, NULL};
const Segment * const SG84495[] = {SGI353079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84496[] = {SGI353080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353081, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84497[] = {SGI353082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353083, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84498[] = {SGI353084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353085, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84499[] = {SGI353086, SGI0, SGI0, SGI353087, SGI0, SGI0, SGI0, SGI0, SGI353088, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84500[] = {SGI353089, SGI353090, SGI353092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84501[] = {SGI353091, SGI0, SGI0, SGI353093, SGI0, SGI0, SGI0, SGI0, SGI353094, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84502[] = {SGI353095, SGI0, SGI0, SGI353096, SGI0, SGI0, SGI0, SGI0, SGI353097, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84503[] = {SGI353098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84504[] = {SGI353099, SGI0, SGI0, SGI353100, SGI0, SGI0, SGI0, SGI0, SGI353101, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84505[] = {SGI353102, SGI353103, SGI353104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353104, SGI0, NULL};
const Segment * const SG84506[] = {SGI353105, SGI0, SGI0, SGI353106, SGI0, SGI0, SGI0, SGI0, SGI353107, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84507[] = {SGI353108, SGI353109, SGI353110, SGI0, SGI0, SGI0, SGI0, SGI353111, SGI0, SGI0, NULL};
const Segment * const SG84508[] = {SGI353112, SGI353113, SGI353114, SGI0, SGI0, SGI0, SGI0, SGI353115, SGI0, SGI0, NULL};
const Segment * const SG84509[] = {SGI353116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353117, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84510[] = {SGI353118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353119, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84511[] = {SGI353120, SGI353121, SGI353122, SGI0, SGI353123, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84512[] = {SGI353124, SGI0, SGI0, SGI353125, SGI0, SGI0, SGI0, SGI0, SGI353126, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84513[] = {SGI353127, SGI0, SGI0, SGI353128, SGI0, SGI0, SGI0, SGI0, SGI353129, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84514[] = {SGI353130, SGI0, SGI0, SGI353131, SGI0, SGI0, SGI0, SGI0, SGI353132, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84515[] = {SGI353133, SGI0, SGI0, SGI353134, SGI0, SGI0, SGI0, SGI0, SGI353135, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84516[] = {SGI353136, SGI0, SGI0, SGI353137, SGI0, SGI0, SGI0, SGI0, SGI353138, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84517[] = {SGI353139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353140, SGI0, NULL};
const Segment * const SG84518[] = {SGI353141, SGI0, SGI0, SGI353142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84519[] = {SGI353143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353144, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84520[] = {SGI353145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353146, SGI0, NULL};
const Segment * const SG84521[] = {SGI353147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353148, SGI0, NULL};
const Segment * const SG84522[] = {SGI353149, SGI0, SGI0, SGI353150, SGI0, SGI0, SGI0, SGI0, SGI353151, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84523[] = {SGI353152, SGI0, SGI353153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84524[] = {SGI353154, SGI0, SGI353155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84525[] = {SGI353156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353157, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84526[] = {SGI353158, SGI0, SGI353159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84527[] = {SGI353160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353161, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84528[] = {SGI353162, SGI0, SGI0, SGI353163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84529[] = {SGI353164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353165, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84530[] = {SGI353166, SGI353167, SGI353168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353168, SGI0, NULL};
const Segment * const SG84531[] = {SGI353169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353170, SGI0, NULL};
const Segment * const SG84532[] = {SGI353171, SGI0, SGI0, SGI353172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84533[] = {SGI353173, SGI0, SGI0, SGI353174, SGI0, SGI0, SGI0, SGI0, SGI353175, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84534[] = {SGI353176, SGI0, SGI0, SGI353177, SGI0, SGI0, SGI0, SGI0, SGI353178, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84535[] = {SGI353179, SGI0, SGI0, SGI353180, SGI0, SGI0, SGI0, SGI0, SGI353181, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84536[] = {SGI353182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353183, SGI0, NULL};
const Segment * const SG84537[] = {SGI353184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353185, SGI0, NULL};
const Segment * const SG84538[] = {SGI353186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353187, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84539[] = {SGI353188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353189, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84540[] = {SGI353190, SGI0, SGI0, SGI353191, SGI0, SGI0, SGI0, SGI0, SGI353192, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84541[] = {SGI353193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353194, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84542[] = {SGI353195, SGI0, SGI0, SGI353196, SGI0, SGI0, SGI0, SGI0, SGI353197, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84543[] = {SGI353198, SGI0, SGI0, SGI353199, SGI0, SGI0, SGI0, SGI0, SGI353200, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84544[] = {SGI353201, SGI0, SGI0, SGI353202, SGI0, SGI0, SGI0, SGI0, SGI353203, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84545[] = {SGI353204, SGI0, SGI353205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84546[] = {SGI353206, SGI0, SGI353207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84547[] = {SGI353208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353209, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84548[] = {SGI353210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353211, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84549[] = {SGI353212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84550[] = {SGI0, SGI353213, SGI353214, SGI353215, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84551[] = {SGI353216, SGI353217, SGI353218, SGI0, SGI0, SGI0, SGI0, SGI353219, SGI0, SGI0, NULL};
const Segment * const SG84552[] = {SGI353220, SGI0, SGI353221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84553[] = {SGI353222, SGI0, SGI353223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84554[] = {SGI0, SGI353224, SGI353225, SGI0, SGI0, SGI0, SGI353226, NULL};
const Segment * const SG84555[] = {SGI353227, SGI353228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84556[] = {SGI353229, SGI0, SGI0, SGI353230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84557[] = {SGI353231, SGI353232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84558[] = {SGI0, SGI353233, SGI353234, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84559[] = {SGI0, SGI353235, SGI353236, SGI0, SGI0, SGI0, SGI353237, NULL};
const Segment * const SG84560[] = {SGI353238, SGI353239, SGI0, SGI0, NULL};
const Segment * const SG84561[] = {SGI353240, SGI353241, SGI0, SGI0, NULL};
const Segment * const SG84562[] = {SGI353242, SGI353243, SGI0, SGI0, NULL};
const Segment * const SG84563[] = {SGI353244, SGI353245, SGI0, SGI0, NULL};
const Segment * const SG84564[] = {SGI353246, SGI353247, SGI0, SGI0, NULL};
const Segment * const SG84565[] = {SGI353248, SGI353249, SGI0, SGI0, NULL};
const Segment * const SG84566[] = {SGI353250, SGI353251, SGI0, SGI0, NULL};
const Segment * const SG84567[] = {SGI353252, SGI353253, SGI0, SGI0, NULL};
const Segment * const SG84568[] = {SGI353254, SGI353255, SGI0, SGI0, NULL};
const Segment * const SG84569[] = {SGI353256, SGI353257, SGI0, SGI0, NULL};
const Segment * const SG84570[] = {SGI353258, SGI353259, SGI0, SGI0, NULL};
const Segment * const SG84571[] = {SGI0, SGI353260, SGI0, SGI0, NULL};
const Segment * const SG84572[] = {SGI353261, SGI0, SGI0, SGI353262, SGI0, SGI0, SGI0, SGI0, SGI353263, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84573[] = {SGI353264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353265, SGI0, NULL};
const Segment * const SG84574[] = {SGI353266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353267, SGI0, NULL};
const Segment * const SG84575[] = {SGI353268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353269, SGI0, NULL};
const Segment * const SG84576[] = {SGI353270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84577[] = {SGI353271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84578[] = {SGI353272, SGI0, SGI353273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84579[] = {SGI353274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353275, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84580[] = {SGI353276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353277, SGI0, NULL};
const Segment * const SG84581[] = {SGI353278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353279, SGI0, NULL};
const Segment * const SG84582[] = {SGI353280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353281, SGI0, NULL};
const Segment * const SG84583[] = {SGI353282, SGI0, SGI0, SGI353283, SGI0, SGI0, SGI0, SGI0, SGI353284, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84584[] = {SGI353285, SGI0, SGI0, SGI353286, SGI0, SGI0, SGI0, SGI0, SGI353287, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84585[] = {SGI353288, SGI0, SGI353289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84586[] = {SGI353290, SGI0, SGI353291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84587[] = {SGI353292, SGI0, SGI353293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84588[] = {SGI353294, SGI0, SGI353295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84589[] = {SGI353296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353297, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84590[] = {SGI353298, SGI0, SGI0, SGI353299, SGI0, SGI0, SGI0, SGI0, SGI353300, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84591[] = {SGI353301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353302, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84592[] = {SGI353303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353304, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84593[] = {SGI353305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353306, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84594[] = {SGI353307, SGI353308, SGI353309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353309, SGI0, NULL};
const Segment * const SG84595[] = {SGI353310, SGI353311, SGI353312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353312, SGI0, NULL};
const Segment * const SG84596[] = {SGI353313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353314, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84597[] = {SGI353315, SGI0, SGI353316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353317, SGI0, NULL};
const Segment * const SG84598[] = {SGI353318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353319, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84599[] = {SGI353320, SGI0, SGI0, SGI353321, SGI0, SGI0, SGI0, SGI0, SGI353322, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84600[] = {SGI353323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84601[] = {SGI353324, SGI0, SGI0, SGI353325, SGI0, SGI0, SGI0, SGI0, SGI353326, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84602[] = {SGI353327, SGI353328, SGI353329, SGI0, SGI0, SGI0, SGI0, SGI353330, SGI0, SGI0, NULL};
const Segment * const SG84603[] = {SGI353331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353332, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84604[] = {SGI353333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353334, SGI0, NULL};
const Segment * const SG84605[] = {SGI353335, SGI0, SGI0, SGI353336, SGI0, SGI0, SGI0, SGI0, SGI353337, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84606[] = {SGI353338, SGI353339, SGI353340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353340, SGI0, NULL};
const Segment * const SG84607[] = {SGI353341, SGI0, SGI0, SGI353342, SGI0, SGI0, SGI0, SGI0, SGI353343, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84608[] = {SGI353344, SGI0, SGI0, SGI353345, SGI0, SGI0, SGI0, SGI0, SGI353346, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84609[] = {SGI353347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353348, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84610[] = {SGI353349, SGI0, SGI0, SGI353350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84611[] = {SGI353351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353352, SGI0, NULL};
const Segment * const SG84612[] = {SGI353353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353354, SGI0, SGI0, NULL};
const Segment * const SG84613[] = {SGI353355, SGI0, SGI0, SGI353356, SGI0, SGI0, SGI0, SGI0, SGI353357, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84614[] = {SGI353358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353359, SGI0, NULL};
const Segment * const SG84615[] = {SGI353360, SGI0, SGI0, SGI353361, SGI0, SGI0, SGI0, SGI0, SGI353362, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84616[] = {SGI353363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353364, SGI0, NULL};
const Segment * const SG84617[] = {SGI353365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353366, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84618[] = {SGI353367, SGI0, SGI0, SGI353368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84619[] = {SGI353369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353370, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84620[] = {SGI353371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353372, SGI0, NULL};
const Segment * const SG84621[] = {SGI353373, SGI0, SGI353374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84622[] = {SGI353375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353376, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84623[] = {SGI353377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353378, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84624[] = {SGI353379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353380, SGI0, NULL};
const Segment * const SG84625[] = {SGI353381, SGI0, SGI0, SGI353382, SGI0, SGI0, SGI0, SGI0, SGI353383, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84626[] = {SGI353384, SGI0, SGI0, SGI353385, SGI0, SGI0, SGI0, SGI0, SGI353386, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84627[] = {SGI0, SGI353387, SGI353388, SGI0, SGI0, SGI0, SGI353389, NULL};
const Segment * const SG84628[] = {SGI353390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353391, SGI0, NULL};
const Segment * const SG84629[] = {SGI353392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353393, SGI0, NULL};
const Segment * const SG84630[] = {SGI353394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353395, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84631[] = {SGI353396, SGI0, SGI0, SGI353397, SGI0, SGI0, SGI0, SGI0, SGI353398, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84632[] = {SGI353399, SGI0, SGI0, SGI353400, SGI0, SGI0, SGI0, SGI0, SGI353401, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84633[] = {SGI353402, SGI0, SGI0, SGI353403, SGI0, SGI0, SGI0, SGI0, SGI353404, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84634[] = {SGI353405, SGI0, SGI0, SGI353406, SGI0, SGI0, SGI0, SGI0, SGI353407, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84635[] = {SGI353408, SGI0, SGI0, SGI353409, SGI0, SGI0, SGI0, SGI0, SGI353410, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84636[] = {SGI353411, SGI0, SGI0, SGI353412, SGI0, SGI0, SGI0, SGI0, SGI353413, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84637[] = {SGI353414, SGI0, SGI0, SGI353415, SGI0, SGI0, SGI0, SGI0, SGI353416, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84638[] = {SGI353417, SGI0, SGI0, SGI353418, SGI0, SGI0, SGI0, SGI0, SGI353419, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84639[] = {SGI353420, SGI0, SGI353421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84640[] = {SGI353422, SGI0, SGI353423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84641[] = {SGI353424, SGI0, SGI353425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84642[] = {SGI353426, SGI0, SGI353427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84643[] = {SGI353428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353429, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84644[] = {SGI353430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353431, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84645[] = {SGI353432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353433, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84646[] = {SGI353434, SGI0, SGI0, SGI353435, SGI0, SGI0, SGI0, SGI0, SGI353436, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84647[] = {SGI0, SGI353437, SGI353438, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84648[] = {SGI353439, SGI0, SGI0, SGI353440, SGI0, SGI0, SGI0, SGI0, SGI353441, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84649[] = {SGI353442, SGI0, SGI0, SGI353443, SGI0, SGI0, SGI0, SGI0, SGI353444, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84650[] = {SGI353445, SGI0, SGI353446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84651[] = {SGI353447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353448, SGI0, SGI0, NULL};
const Segment * const SG84652[] = {SGI353449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353450, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84653[] = {SGI353451, SGI0, SGI0, SGI353452, SGI0, SGI0, SGI0, SGI0, SGI353453, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84654[] = {SGI353454, SGI0, SGI0, SGI353455, SGI0, SGI0, SGI0, SGI0, SGI353456, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84655[] = {SGI353457, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84656[] = {SGI353458, SGI353459, SGI0, SGI0, NULL};
const Segment * const SG84657[] = {SGI353460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353461, SGI0, NULL};
const Segment * const SG84658[] = {SGI353462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353463, SGI0, NULL};
const Segment * const SG84659[] = {SGI353464, SGI0, SGI0, SGI353465, SGI0, SGI0, SGI0, SGI0, SGI353466, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84660[] = {SGI353467, SGI0, SGI0, SGI353468, SGI0, SGI0, SGI0, SGI0, SGI353469, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84661[] = {SGI353470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84662[] = {SGI353471, SGI0, SGI353472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84663[] = {SGI353473, SGI353474, NULL};
const Segment * const SG84664[] = {SGI353475, SGI353476, NULL};
const Segment * const SG84665[] = {SGI353477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353478, SGI0, NULL};
const Segment * const SG84666[] = {SGI353479, SGI353480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84667[] = {SGI353481, SGI353482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84668[] = {SGI353483, SGI353484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84669[] = {SGI353485, SGI353486, SGI353487, SGI0, SGI0, SGI0, SGI0, SGI353488, SGI0, SGI0, NULL};
const Segment * const SG84670[] = {SGI353489, SGI0, SGI0, SGI353490, SGI0, SGI0, SGI0, SGI0, SGI353491, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84671[] = {SGI353492, SGI353493, SGI0, SGI0, NULL};
const Segment * const SG84672[] = {SGI353494, SGI353495, SGI0, SGI0, NULL};
const Segment * const SG84673[] = {SGI353496, SGI353497, SGI0, SGI0, NULL};
const Segment * const SG84674[] = {SGI353500, SGI353501, SGI0, SGI0, NULL};
const Segment * const SG84675[] = {SGI353498, SGI0, SGI0, SGI353499, SGI0, SGI0, SGI0, SGI0, SGI353502, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84676[] = {SGI353503, SGI353504, SGI0, SGI0, NULL};
const Segment * const SG84677[] = {SGI353505, SGI353506, SGI0, SGI0, NULL};
const Segment * const SG84678[] = {SGI353507, SGI353508, SGI0, SGI0, NULL};
const Segment * const SG84679[] = {SGI353509, SGI353510, SGI0, SGI0, NULL};
const Segment * const SG84680[] = {SGI353511, SGI353512, SGI0, SGI0, NULL};
const Segment * const SG84681[] = {SGI353513, SGI353514, SGI0, SGI0, NULL};
const Segment * const SG84682[] = {SGI353515, SGI353516, SGI0, SGI0, NULL};
const Segment * const SG84683[] = {SGI353517, SGI0, SGI353518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84684[] = {SGI353519, SGI0, SGI353520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84685[] = {SGI353521, SGI0, SGI353522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84686[] = {SGI353523, SGI0, SGI353524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84687[] = {SGI353525, SGI0, SGI353526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84688[] = {SGI353527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353528, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84689[] = {SGI353529, SGI353530, SGI353531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84690[] = {SGI353532, SGI0, SGI0, SGI353533, SGI0, SGI0, SGI0, SGI0, SGI353534, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84691[] = {SGI353535, SGI0, SGI0, SGI353536, SGI0, SGI0, SGI0, SGI0, SGI353537, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84692[] = {SGI353538, SGI0, SGI353539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84693[] = {SGI353540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353541, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84694[] = {SGI353542, SGI0, SGI0, SGI353543, SGI0, SGI0, SGI0, SGI0, SGI353544, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84695[] = {SGI353545, SGI0, SGI0, SGI353546, SGI0, SGI0, SGI0, SGI0, SGI353547, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84696[] = {SGI353548, SGI0, SGI0, SGI353549, SGI0, SGI0, SGI0, SGI0, SGI353550, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84697[] = {SGI353551, SGI353552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84698[] = {SGI353553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84699[] = {SGI353554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353555, SGI0, NULL};
const Segment * const SG84700[] = {SGI353556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84701[] = {SGI353557, SGI0, SGI0, SGI353558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84702[] = {SGI353559, SGI0, SGI0, SGI353560, SGI0, SGI0, SGI0, SGI0, SGI353561, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84703[] = {SGI353562, SGI0, SGI0, SGI353563, SGI0, SGI0, SGI0, SGI0, SGI353564, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84704[] = {SGI353565, SGI0, SGI0, SGI353566, SGI0, SGI0, SGI0, SGI0, SGI353567, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84705[] = {SGI0, SGI353568, SGI353569, SGI353570, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84706[] = {SGI353571, SGI0, SGI0, SGI353572, SGI0, SGI0, SGI0, SGI0, SGI353573, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84707[] = {SGI353574, SGI353575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84708[] = {SGI353576, SGI0, SGI0, SGI353577, SGI0, SGI0, SGI0, SGI0, SGI353578, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84709[] = {SGI353579, SGI0, SGI353580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84710[] = {SGI353581, SGI0, SGI353582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84711[] = {SGI353583, SGI0, SGI353584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84712[] = {SGI353585, SGI0, SGI0, SGI353586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84713[] = {SGI353587, SGI0, SGI353588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84714[] = {SGI353589, SGI0, SGI353590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84715[] = {SGI353591, SGI353592, SGI0, SGI353593, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84716[] = {SGI353594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353595, NULL};
const Segment * const SG84717[] = {SGI353596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353597, SGI0, NULL};
const Segment * const SG84718[] = {SGI353598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353599, SGI0, NULL};
const Segment * const SG84719[] = {SGI353600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353601, SGI0, NULL};
const Segment * const SG84720[] = {SGI353602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353603, SGI0, NULL};
const Segment * const SG84721[] = {SGI353604, SGI353604, SGI353605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353605, SGI0, NULL};
const Segment * const SG84722[] = {SGI353606, SGI0, SGI353607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84723[] = {SGI353608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353609, SGI0, NULL};
const Segment * const SG84724[] = {SGI353610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353611, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84725[] = {SGI353612, SGI0, SGI0, SGI353613, SGI0, SGI0, SGI0, SGI0, SGI353614, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84726[] = {SGI353615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353616, SGI0, NULL};
const Segment * const SG84727[] = {SGI353617, SGI0, SGI353618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84728[] = {SGI353619, SGI353620, SGI353621, SGI0, SGI0, SGI0, SGI0, SGI353622, SGI0, SGI0, NULL};
const Segment * const SG84729[] = {SGI353623, SGI0, SGI353624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84730[] = {SGI353625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353626, SGI0, NULL};
const Segment * const SG84731[] = {SGI353627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353628, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84732[] = {SGI353629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353630, SGI0, NULL};
const Segment * const SG84733[] = {SGI353631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84734[] = {SGI353632, SGI0, SGI0, SGI353633, SGI0, SGI0, SGI0, SGI0, SGI353634, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84735[] = {SGI353635, SGI0, SGI0, SGI353636, SGI0, SGI0, SGI0, SGI0, SGI353637, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84736[] = {SGI353638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353639, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84737[] = {SGI353640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84738[] = {SGI353641, SGI0, SGI353642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84739[] = {SGI353643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353644, SGI0, NULL};
const Segment * const SG84740[] = {SGI353645, SGI0, SGI0, SGI353646, SGI0, SGI0, SGI0, SGI0, SGI353647, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84741[] = {SGI353648, SGI0, SGI353649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84742[] = {SGI353650, SGI0, SGI353651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84743[] = {SGI353652, SGI0, SGI0, SGI353653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84744[] = {SGI353654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353655, SGI0, SGI0, NULL};
const Segment * const SG84745[] = {SGI353656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353657, NULL};
const Segment * const SG84746[] = {SGI353658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353659, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84747[] = {SGI353660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353661, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84748[] = {SGI353662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84749[] = {SGI353663, SGI0, SGI353664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84750[] = {SGI353665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353666, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84751[] = {SGI353667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353668, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84752[] = {SGI353669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353670, SGI0, NULL};
const Segment * const SG84753[] = {SGI353671, SGI353672, SGI0, SGI0, NULL};
const Segment * const SG84754[] = {SGI353673, SGI0, SGI353674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84755[] = {SGI353675, SGI353676, SGI353677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84756[] = {SGI353678, SGI353679, SGI353680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84757[] = {SGI353681, SGI353682, SGI353683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84758[] = {SGI353684, SGI0, SGI353685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84759[] = {SGI353686, SGI0, SGI353687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84760[] = {SGI353688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353689, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84761[] = {SGI353690, SGI353691, SGI353692, SGI0, SGI0, SGI0, SGI0, SGI353693, SGI0, SGI0, NULL};
const Segment * const SG84762[] = {SGI353694, SGI353695, SGI353696, SGI0, SGI0, SGI0, SGI0, SGI353697, SGI0, SGI0, NULL};
const Segment * const SG84763[] = {SGI353698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353699, SGI0, NULL};
const Segment * const SG84764[] = {SGI353700, SGI0, SGI0, SGI353701, SGI0, SGI0, SGI0, SGI0, SGI353702, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84765[] = {SGI353703, SGI353704, SGI0, SGI0, NULL};
const Segment * const SG84766[] = {SGI353705, SGI353706, SGI0, SGI0, NULL};
const Segment * const SG84767[] = {SGI353707, SGI353708, SGI353709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353709, SGI0, NULL};
const Segment * const SG84768[] = {SGI353710, SGI0, SGI353711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353711, SGI0, NULL};
const Segment * const SG84769[] = {SGI353712, SGI0, SGI353713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84770[] = {SGI353714, SGI0, SGI353715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84771[] = {SGI353716, SGI0, SGI353717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84772[] = {SGI353718, SGI0, SGI353719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84773[] = {SGI353720, SGI0, SGI353721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84774[] = {SGI353722, SGI0, SGI353723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84775[] = {SGI353724, SGI0, SGI353725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84776[] = {SGI353726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353727, SGI0, NULL};
const Segment * const SG84777[] = {SGI353728, SGI0, SGI353729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84778[] = {SGI353730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353731, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84779[] = {SGI353732, SGI0, SGI353733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84780[] = {SGI353734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353735, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84781[] = {SGI353736, SGI0, SGI0, SGI353737, SGI0, SGI0, SGI0, SGI0, SGI353738, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84782[] = {SGI353739, SGI0, SGI353740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84783[] = {SGI353741, SGI353742, SGI353743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353743, SGI0, NULL};
const Segment * const SG84784[] = {SGI353744, SGI0, SGI0, SGI353745, SGI0, SGI0, SGI0, SGI0, SGI353746, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84785[] = {SGI353747, SGI353748, SGI353749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353749, SGI0, NULL};
const Segment * const SG84786[] = {SGI353750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84787[] = {SGI353751, SGI0, SGI353752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84788[] = {SGI353753, SGI0, SGI0, SGI353754, SGI0, SGI0, SGI0, SGI0, SGI353755, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84789[] = {SGI353756, SGI0, SGI0, SGI353757, SGI0, SGI0, SGI0, SGI0, SGI353758, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84790[] = {SGI353759, SGI0, SGI0, SGI353760, SGI0, SGI0, SGI0, SGI0, SGI353761, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84791[] = {SGI353762, SGI0, SGI0, SGI353763, SGI0, SGI0, SGI0, SGI0, SGI353764, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84792[] = {SGI353765, SGI0, SGI0, SGI353766, SGI0, SGI0, SGI0, SGI0, SGI353767, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84793[] = {SGI353768, SGI0, SGI0, SGI353769, SGI0, SGI0, SGI0, SGI0, SGI353770, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84794[] = {SGI353771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353772, SGI0, NULL};
const Segment * const SG84795[] = {SGI353773, SGI0, SGI0, SGI353774, SGI0, SGI0, SGI0, SGI0, SGI353775, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84796[] = {SGI353776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353777, SGI0, NULL};
const Segment * const SG84797[] = {SGI353778, SGI0, SGI353779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84798[] = {SGI353780, SGI0, SGI353781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84799[] = {SGI353782, SGI0, SGI353783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84800[] = {SGI353784, SGI0, SGI353785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84801[] = {SGI353786, SGI0, SGI353787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84802[] = {SGI353788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353789, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84803[] = {SGI353790, SGI0, SGI0, SGI353791, SGI0, SGI0, SGI0, SGI0, SGI353792, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84804[] = {SGI353793, SGI0, SGI0, SGI353794, SGI0, SGI0, SGI0, SGI0, SGI353795, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84805[] = {SGI353796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353797, SGI0, NULL};
const Segment * const SG84806[] = {SGI353798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353799, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84807[] = {SGI353800, SGI353801, SGI353802, SGI0, SGI0, SGI0, SGI0, SGI353803, SGI0, SGI0, NULL};
const Segment * const SG84808[] = {SGI353804, SGI0, SGI0, SGI353805, SGI0, SGI0, SGI0, SGI0, SGI353806, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84809[] = {SGI353807, SGI0, SGI0, SGI353808, SGI0, SGI0, SGI0, SGI0, SGI353809, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84810[] = {SGI353810, SGI0, SGI0, SGI353811, SGI0, SGI0, SGI0, SGI0, SGI353812, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84811[] = {SGI353813, SGI353814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84812[] = {SGI353815, SGI353816, SGI353817, SGI0, SGI0, SGI0, SGI0, SGI353818, SGI0, SGI0, NULL};
const Segment * const SG84813[] = {SGI353819, SGI0, SGI0, SGI353820, SGI0, SGI0, SGI0, SGI0, SGI353821, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84814[] = {SGI353822, SGI0, SGI0, SGI353823, SGI0, SGI0, SGI0, SGI0, SGI353824, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84815[] = {SGI353825, SGI0, SGI0, SGI353826, SGI0, SGI0, SGI0, SGI0, SGI353827, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84816[] = {SGI353828, SGI0, SGI0, SGI353829, SGI0, SGI0, SGI0, SGI0, SGI353830, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84817[] = {SGI353831, SGI353832, SGI0, SGI0, NULL};
const Segment * const SG84818[] = {SGI353833, SGI353834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84819[] = {SGI353835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353836, SGI0, NULL};
const Segment * const SG84820[] = {SGI353837, SGI0, SGI353838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84821[] = {SGI353839, SGI0, SGI353840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84822[] = {SGI353841, SGI0, SGI353842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84823[] = {SGI353843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353844, SGI0, SGI0, NULL};
const Segment * const SG84824[] = {SGI353845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353846, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84825[] = {SGI353847, SGI0, SGI353848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353848, SGI0, NULL};
const Segment * const SG84826[] = {SGI353849, SGI0, SGI353850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353850, SGI0, NULL};
const Segment * const SG84827[] = {SGI353851, SGI0, SGI353852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353852, SGI0, NULL};
const Segment * const SG84828[] = {SGI353853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353854, SGI0, NULL};
const Segment * const SG84829[] = {SGI353855, SGI0, SGI353856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353856, SGI0, NULL};
const Segment * const SG84830[] = {SGI353857, SGI0, SGI0, SGI353858, SGI0, SGI0, SGI0, SGI0, SGI353859, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84831[] = {SGI353860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84832[] = {SGI353861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84833[] = {SGI353862, SGI0, SGI353863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84834[] = {SGI353864, SGI0, SGI353865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84835[] = {SGI353866, SGI0, SGI353867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84836[] = {SGI353868, SGI0, SGI353869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84837[] = {SGI353870, SGI0, SGI353871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84838[] = {SGI353872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84839[] = {SGI353873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353874, SGI0, NULL};
const Segment * const SG84840[] = {SGI353875, SGI353877, SGI353878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353878, SGI0, NULL};
const Segment * const SG84841[] = {SGI353876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353879, SGI0, NULL};
const Segment * const SG84842[] = {SGI353880, SGI0, SGI353881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84843[] = {SGI353882, SGI0, SGI0, SGI353883, SGI0, SGI0, SGI0, SGI0, SGI353884, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84844[] = {SGI353885, SGI0, SGI353886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84845[] = {SGI353887, SGI0, SGI0, SGI353888, SGI0, SGI0, SGI0, SGI0, SGI353889, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84846[] = {SGI353890, SGI0, SGI0, SGI353891, SGI0, SGI0, SGI0, SGI0, SGI353892, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84847[] = {SGI353893, SGI0, SGI353894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84848[] = {SGI353895, SGI0, SGI353896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84849[] = {SGI353897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353898, SGI0, NULL};
const Segment * const SG84850[] = {SGI353899, SGI0, SGI0, SGI353900, SGI0, SGI0, SGI0, SGI0, SGI353901, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84851[] = {SGI353902, SGI0, SGI0, SGI353903, SGI0, SGI0, SGI0, SGI0, SGI353904, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84852[] = {SGI353905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353906, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84853[] = {SGI353907, SGI0, SGI0, SGI353908, SGI0, SGI0, SGI0, SGI0, SGI353909, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84854[] = {SGI353910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353911, SGI0, NULL};
const Segment * const SG84855[] = {SGI353912, SGI0, SGI0, SGI353913, SGI0, SGI0, SGI0, SGI0, SGI353914, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84856[] = {SGI353915, SGI0, SGI0, SGI353916, SGI0, SGI0, SGI0, SGI0, SGI353917, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84857[] = {SGI353918, SGI353919, SGI0, SGI353920, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84858[] = {SGI353921, SGI0, SGI0, SGI353922, SGI0, SGI0, SGI0, SGI0, SGI353923, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84859[] = {SGI353924, SGI0, SGI0, SGI353925, SGI0, SGI0, SGI0, SGI0, SGI353926, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84860[] = {SGI353927, SGI0, SGI0, SGI353928, SGI0, SGI0, SGI0, SGI0, SGI353929, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84861[] = {SGI353930, SGI0, SGI353931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84862[] = {SGI353932, SGI0, SGI353933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84863[] = {SGI353934, SGI0, SGI353935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84864[] = {SGI353936, SGI0, SGI353937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84865[] = {SGI353938, SGI0, SGI353939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84866[] = {SGI353940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353941, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84867[] = {SGI353942, SGI353943, SGI353944, SGI353945, NULL};
const Segment * const SG84868[] = {SGI353946, SGI0, SGI0, SGI353947, SGI0, SGI0, SGI0, SGI0, SGI353948, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84869[] = {SGI0, SGI353949, SGI353950, SGI353951, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84870[] = {SGI353952, SGI0, SGI0, SGI353953, SGI0, SGI0, SGI0, SGI0, SGI353954, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84871[] = {SGI353955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84872[] = {SGI353956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84873[] = {SGI353957, SGI0, SGI353958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84874[] = {SGI353959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84875[] = {SGI353960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353961, SGI0, NULL};
const Segment * const SG84876[] = {SGI353962, SGI0, SGI353963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84877[] = {SGI353964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353965, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84878[] = {SGI353966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI353967, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84879[] = {SGI353968, SGI0, SGI0, SGI353969, SGI0, SGI0, SGI0, SGI0, SGI353970, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84880[] = {SGI353971, SGI0, SGI353972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84881[] = {SGI353973, SGI353974, SGI353975, SGI0, SGI0, SGI0, SGI0, SGI353976, SGI0, SGI0, NULL};
const Segment * const SG84882[] = {SGI353977, SGI353978, SGI353979, SGI0, SGI0, SGI0, SGI0, SGI353980, SGI0, SGI0, NULL};
const Segment * const SG84883[] = {SGI353981, SGI0, SGI0, SGI353982, SGI0, SGI0, SGI0, SGI0, SGI353983, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84884[] = {SGI353984, SGI353985, SGI353986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84885[] = {SGI353987, SGI353988, SGI353989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84886[] = {SGI353990, SGI353991, SGI353992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84887[] = {SGI353993, SGI353994, SGI353995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84888[] = {SGI353996, SGI353997, SGI353998, SGI0, SGI0, SGI0, SGI0, SGI353999, SGI0, SGI0, NULL};
const Segment * const SG84889[] = {SGI354000, SGI0, SGI354001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84890[] = {SGI354002, SGI0, SGI0, SGI354003, SGI0, SGI0, SGI0, SGI0, SGI354004, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84891[] = {SGI354005, SGI0, SGI0, SGI354006, SGI0, SGI0, SGI0, SGI0, SGI354007, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84892[] = {SGI354008, SGI0, SGI0, SGI354009, SGI0, SGI0, SGI0, SGI0, SGI354010, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84893[] = {SGI354011, SGI0, SGI0, SGI354012, SGI0, SGI0, SGI0, SGI0, SGI354013, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84894[] = {SGI354014, SGI354015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84895[] = {SGI354016, SGI354017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84896[] = {SGI354018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354019, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84897[] = {SGI354020, SGI0, SGI354021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84898[] = {SGI354022, SGI0, SGI354023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84899[] = {SGI354024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354025, SGI0, NULL};
const Segment * const SG84900[] = {SGI354026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354027, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84901[] = {SGI354028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354029, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84902[] = {SGI354030, SGI0, SGI354031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84903[] = {SGI354032, SGI0, SGI354033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84904[] = {SGI354034, SGI0, SGI354035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84905[] = {SGI354036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354037, SGI0, NULL};
const Segment * const SG84906[] = {SGI354038, SGI0, SGI354039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84907[] = {SGI354040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354041, SGI0, NULL};
const Segment * const SG84908[] = {SGI354042, SGI354043, SGI354044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354044, SGI0, NULL};
const Segment * const SG84909[] = {SGI354045, SGI354046, SGI354047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354047, SGI0, NULL};
const Segment * const SG84910[] = {SGI354048, SGI0, SGI354049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84911[] = {SGI354050, SGI0, SGI354051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84912[] = {SGI354052, SGI354053, NULL};
const Segment * const SG84913[] = {SGI354054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354055, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84914[] = {SGI354056, SGI0, SGI0, SGI354057, SGI0, SGI0, SGI0, SGI0, SGI354058, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84915[] = {SGI354059, SGI0, SGI354060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354060, SGI0, NULL};
const Segment * const SG84916[] = {SGI354061, SGI354061, SGI354062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84917[] = {SGI354063, SGI354064, SGI354065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84918[] = {SGI354066, SGI354066, SGI354067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84919[] = {SGI354068, SGI354069, SGI354070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84920[] = {SGI354071, SGI0, SGI0, SGI354072, SGI0, SGI0, SGI0, SGI0, SGI354073, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84921[] = {SGI354074, SGI354074, SGI354075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84922[] = {SGI354076, SGI354077, SGI354078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84923[] = {SGI354079, SGI354080, SGI354081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84924[] = {SGI354082, SGI354083, SGI354084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84925[] = {SGI354085, SGI354085, SGI354086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84926[] = {SGI354087, SGI354088, SGI354089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84927[] = {SGI354090, SGI354091, SGI354092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84928[] = {SGI354093, SGI354094, SGI354095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84929[] = {SGI354096, SGI0, SGI354097, SGI354097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84930[] = {SGI354098, SGI354099, SGI0, SGI0, NULL};
const Segment * const SG84931[] = {SGI354100, SGI354101, SGI0, SGI0, NULL};
const Segment * const SG84932[] = {SGI354102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354103, SGI0, NULL};
const Segment * const SG84933[] = {SGI354104, SGI0, SGI354105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84934[] = {SGI354106, SGI354107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84935[] = {SGI354108, SGI354109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84936[] = {SGI354110, SGI0, SGI354111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84937[] = {SGI354112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354113, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84938[] = {SGI354114, SGI354115, SGI354116, SGI0, SGI0, SGI0, SGI0, SGI354117, SGI0, SGI0, NULL};
const Segment * const SG84939[] = {SGI354118, SGI354119, SGI354120, SGI0, SGI0, SGI0, SGI0, SGI354121, SGI0, SGI0, NULL};
const Segment * const SG84940[] = {SGI354122, SGI354123, SGI354124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84941[] = {SGI354125, SGI354126, SGI354127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84942[] = {SGI354128, SGI354129, SGI354130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84943[] = {SGI354131, SGI354132, SGI354133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84944[] = {SGI354134, SGI354135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84945[] = {SGI354136, SGI0, SGI354137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84946[] = {SGI354138, SGI0, SGI354139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84947[] = {SGI0, SGI354140, NULL};
const Segment * const SG84948[] = {SGI0, SGI354141, NULL};
const Segment * const SG84949[] = {SGI354142, SGI0, SGI354143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84950[] = {SGI354144, SGI0, SGI354145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84951[] = {SGI354146, SGI0, SGI354147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84952[] = {SGI354148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84953[] = {SGI354149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354150, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84954[] = {SGI354151, SGI0, SGI0, SGI354152, SGI0, SGI0, SGI0, SGI0, SGI354153, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84955[] = {SGI354154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354155, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84956[] = {SGI354156, SGI0, SGI354157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84957[] = {SGI354158, SGI0, SGI354159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84958[] = {SGI354160, SGI0, SGI354161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84959[] = {SGI354162, SGI0, SGI0, SGI354163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84960[] = {SGI354164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354165, SGI0, NULL};
const Segment * const SG84961[] = {SGI354166, SGI0, SGI0, SGI354167, SGI0, SGI0, SGI0, SGI0, SGI354168, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84962[] = {SGI354169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354170, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84963[] = {SGI354171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354172, SGI0, NULL};
const Segment * const SG84964[] = {SGI354173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84965[] = {SGI354174, SGI0, SGI354175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84966[] = {SGI354176, SGI0, SGI354177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84967[] = {SGI354178, SGI0, SGI354179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84968[] = {SGI354180, SGI0, SGI354181, SGI354181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84969[] = {SGI354182, SGI0, SGI354183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84970[] = {SGI354184, SGI0, SGI354185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84971[] = {SGI354186, SGI0, SGI354187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84972[] = {SGI354188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354189, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84973[] = {SGI354190, SGI0, SGI354191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84974[] = {SGI354192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354193, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84975[] = {SGI354194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354195, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84976[] = {SGI0, SGI354196, SGI0, SGI0, NULL};
const Segment * const SG84977[] = {SGI354197, SGI354198, SGI354199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84978[] = {SGI354200, SGI354201, SGI354202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84979[] = {SGI354203, SGI0, SGI354204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84980[] = {SGI354205, SGI0, SGI354206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84981[] = {SGI354207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354208, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84982[] = {SGI354209, SGI0, SGI0, SGI354210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84983[] = {SGI354211, SGI0, SGI354212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84984[] = {SGI354213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354214, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84985[] = {SGI354215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354216, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84986[] = {SGI354217, SGI0, SGI0, SGI354218, SGI0, SGI0, SGI0, SGI0, SGI354219, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84987[] = {SGI354220, SGI0, SGI0, SGI354221, SGI0, SGI0, SGI0, SGI0, SGI354222, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84988[] = {SGI354223, SGI0, SGI0, SGI354224, SGI0, SGI0, SGI0, SGI0, SGI354225, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84989[] = {SGI354226, SGI0, SGI0, SGI354227, SGI0, SGI0, SGI0, SGI0, SGI354228, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84990[] = {SGI354229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84991[] = {SGI354230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84992[] = {SGI354231, SGI0, SGI354232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84993[] = {SGI354233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354234, SGI0, NULL};
const Segment * const SG84994[] = {SGI354235, SGI0, SGI354236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84995[] = {SGI354237, SGI0, SGI354238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84996[] = {SGI354239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354240, SGI0, SGI0, NULL};
const Segment * const SG84997[] = {SGI354241, SGI354242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84998[] = {SGI354243, SGI354244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG84999[] = {SGI354245, SGI0, SGI0, SGI354246, SGI0, SGI0, SGI0, SGI0, SGI354247, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85000[] = {SGI0, SGI354248, NULL};
const Segment * const SG85001[] = {SGI354249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85002[] = {SGI354250, SGI0, SGI0, SGI354251, SGI0, SGI0, SGI0, SGI0, SGI354252, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85003[] = {SGI354253, SGI0, SGI0, SGI354254, SGI0, SGI0, SGI0, SGI0, SGI354255, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85004[] = {SGI354256, SGI354257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85005[] = {SGI354258, SGI0, SGI0, SGI354259, SGI0, SGI0, SGI0, SGI0, SGI354260, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85006[] = {SGI354261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85007[] = {SGI354262, SGI0, SGI354263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85008[] = {SGI354264, SGI0, SGI354265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85009[] = {SGI354266, SGI0, SGI354267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85010[] = {SGI354268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354269, SGI0, NULL};
const Segment * const SG85011[] = {SGI354270, SGI0, SGI354271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85012[] = {SGI354272, SGI0, SGI354273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85013[] = {SGI354274, SGI0, SGI354275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85014[] = {SGI354276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354277, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85015[] = {SGI354279, SGI354281, SGI0, SGI0, NULL};
const Segment * const SG85016[] = {SGI354278, SGI0, SGI0, SGI354280, SGI0, SGI0, SGI0, SGI0, SGI354282, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85017[] = {SGI354283, SGI0, SGI0, SGI354284, SGI0, SGI0, SGI0, SGI0, SGI354285, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85018[] = {SGI354286, SGI0, SGI0, SGI354287, SGI0, SGI0, SGI0, SGI0, SGI354288, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85019[] = {SGI354289, SGI0, SGI0, SGI354290, SGI0, SGI0, SGI0, SGI0, SGI354291, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85020[] = {SGI354292, SGI0, SGI0, SGI354293, SGI0, SGI0, SGI0, SGI0, SGI354294, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85021[] = {SGI354295, SGI0, SGI0, SGI354296, SGI0, SGI0, SGI0, SGI0, SGI354297, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85022[] = {SGI354298, SGI0, SGI0, SGI354299, SGI0, SGI0, SGI0, SGI0, SGI354300, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85023[] = {SGI354301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85024[] = {SGI354302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85025[] = {SGI354303, SGI0, SGI354304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85026[] = {SGI354305, SGI354306, SGI0, SGI0, NULL};
const Segment * const SG85027[] = {SGI354307, SGI0, SGI354308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85028[] = {SGI354309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354310, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85029[] = {SGI354311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354312, SGI0, NULL};
const Segment * const SG85030[] = {SGI354313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85031[] = {SGI354314, SGI354315, SGI354316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354316, SGI0, NULL};
const Segment * const SG85032[] = {SGI354317, SGI354318, SGI354319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354319, SGI0, NULL};
const Segment * const SG85033[] = {SGI354320, SGI354321, SGI354322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354322, SGI0, NULL};
const Segment * const SG85034[] = {SGI354323, SGI354324, SGI354325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354325, SGI0, NULL};
const Segment * const SG85035[] = {SGI354326, SGI0, SGI0, SGI354327, SGI0, SGI0, SGI0, SGI0, SGI354328, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85036[] = {SGI354329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354330, SGI0, NULL};
const Segment * const SG85037[] = {SGI354331, SGI0, SGI354332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85038[] = {SGI354333, SGI0, SGI354334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85039[] = {SGI354335, SGI0, SGI354336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85040[] = {SGI354337, SGI0, SGI354338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85041[] = {SGI354339, SGI354340, SGI354342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354342, SGI0, NULL};
const Segment * const SG85042[] = {SGI354341, SGI0, SGI0, SGI354343, SGI0, SGI0, SGI0, SGI0, SGI354344, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85043[] = {SGI354345, SGI354346, SGI354347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354348, SGI0, NULL};
const Segment * const SG85044[] = {SGI354349, SGI354350, SGI354351, SGI0, SGI354352, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85045[] = {SGI354353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85046[] = {SGI354354, SGI0, SGI354355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85047[] = {SGI354356, SGI0, SGI0, SGI354357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85048[] = {SGI354358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354359, SGI0, NULL};
const Segment * const SG85049[] = {SGI354360, SGI354361, SGI0, SGI0, NULL};
const Segment * const SG85050[] = {SGI354362, SGI0, SGI354363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354363, SGI0, NULL};
const Segment * const SG85051[] = {SGI354364, SGI0, SGI354365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354365, SGI0, NULL};
const Segment * const SG85052[] = {SGI354366, SGI0, SGI354367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354367, SGI0, NULL};
const Segment * const SG85053[] = {SGI354368, SGI0, SGI354369, SGI354369, SGI0, SGI0, SGI0, SGI0, SGI354369, SGI0, NULL};
const Segment * const SG85054[] = {SGI354370, SGI354371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85055[] = {SGI354372, SGI0, SGI0, SGI354373, SGI0, SGI0, SGI0, SGI0, SGI354374, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85056[] = {SGI354375, SGI0, SGI0, SGI354376, SGI0, SGI0, SGI0, SGI0, SGI354377, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85057[] = {SGI354378, SGI0, SGI354379, SGI354379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85058[] = {SGI354380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354381, SGI0, NULL};
const Segment * const SG85059[] = {SGI354382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85060[] = {SGI354383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85061[] = {SGI354384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85062[] = {SGI354385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85063[] = {SGI354386, SGI0, SGI354387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85064[] = {SGI354388, SGI0, SGI354389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85065[] = {SGI354390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354391, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85066[] = {SGI354392, SGI0, SGI0, SGI354393, SGI0, SGI0, SGI0, SGI0, SGI354394, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85067[] = {SGI354395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354396, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85068[] = {SGI354397, SGI0, SGI0, SGI354398, SGI0, SGI0, SGI0, SGI0, SGI354399, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85069[] = {SGI354400, SGI0, SGI0, SGI354401, SGI0, SGI0, SGI0, SGI0, SGI354402, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85070[] = {SGI354403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354404, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85071[] = {SGI354405, SGI0, SGI354406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85072[] = {SGI354407, SGI0, SGI354408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85073[] = {SGI354409, SGI0, SGI354410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85074[] = {SGI354411, SGI0, SGI354412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85075[] = {SGI354413, SGI0, SGI354414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85076[] = {SGI354415, SGI0, SGI354416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85077[] = {SGI354417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354418, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85078[] = {SGI354419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354420, SGI0, NULL};
const Segment * const SG85079[] = {SGI354421, SGI0, SGI354422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85080[] = {SGI354423, SGI0, SGI354424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85081[] = {SGI354425, SGI0, SGI354426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85082[] = {SGI354427, SGI0, SGI354428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85083[] = {SGI354429, SGI0, SGI354430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85084[] = {SGI354431, SGI0, SGI354432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85085[] = {SGI354433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354434, SGI0, NULL};
const Segment * const SG85086[] = {SGI44236, SGI44237, SGI354435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85087[] = {SGI354436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354437, SGI0, NULL};
const Segment * const SG85088[] = {SGI354438, SGI354439, SGI354440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85089[] = {SGI12745, SGI12746, SGI354441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85090[] = {SGI354442, SGI354442, SGI354443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354443, SGI0, NULL};
const Segment * const SG85091[] = {SGI354444, SGI354445, SGI354446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354446, SGI0, NULL};
const Segment * const SG85092[] = {SGI354447, SGI354448, SGI354449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85093[] = {SGI354450, SGI354451, SGI354452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85094[] = {SGI354453, SGI354454, SGI354455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85095[] = {SGI354456, SGI354457, SGI354458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85096[] = {SGI354459, SGI354460, SGI354461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85097[] = {SGI354462, SGI354463, SGI354464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354464, SGI0, NULL};
const Segment * const SG85098[] = {SGI354465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354466, SGI0, NULL};
const Segment * const SG85099[] = {SGI354467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354468, SGI0, NULL};
const Segment * const SG85100[] = {SGI354469, SGI0, SGI0, SGI354470, SGI0, SGI0, SGI0, SGI0, SGI354471, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85101[] = {SGI354472, SGI354473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85102[] = {SGI354474, SGI354475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85103[] = {SGI354476, SGI354477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85104[] = {SGI354478, SGI0, SGI354479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85105[] = {SGI354480, SGI0, SGI354481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85106[] = {SGI354482, SGI0, SGI354483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85107[] = {SGI354484, SGI354485, SGI354486, SGI0, SGI0, SGI0, SGI0, SGI354487, SGI0, SGI0, NULL};
const Segment * const SG85108[] = {SGI354488, SGI0, SGI354489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85109[] = {SGI354490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354491, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85110[] = {SGI354492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354493, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85111[] = {SGI354494, SGI0, SGI354495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85112[] = {SGI354496, SGI0, SGI354497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85113[] = {SGI354498, SGI0, SGI354499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85114[] = {SGI354500, SGI0, SGI0, SGI354501, SGI0, SGI0, SGI0, SGI0, SGI354502, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85115[] = {SGI354503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354504, SGI0, NULL};
const Segment * const SG85116[] = {SGI354505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354506, SGI0, NULL};
const Segment * const SG85117[] = {SGI354507, SGI0, SGI0, SGI354508, SGI0, SGI0, SGI0, SGI0, SGI354509, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85118[] = {SGI354510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85119[] = {SGI354511, SGI0, SGI0, SGI354512, SGI0, SGI0, SGI0, SGI0, SGI354513, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85120[] = {SGI354514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354515, SGI0, NULL};
const Segment * const SG85121[] = {SGI354516, SGI0, SGI0, SGI354517, SGI0, SGI0, SGI0, SGI0, SGI354518, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85122[] = {SGI354519, SGI354520, NULL};
const Segment * const SG85123[] = {SGI354521, SGI354522, NULL};
const Segment * const SG85124[] = {SGI354523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85125[] = {SGI354524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85126[] = {SGI354525, SGI0, SGI0, SGI354526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85127[] = {SGI354527, SGI0, SGI0, SGI354528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85128[] = {SGI354529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354530, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85129[] = {SGI354531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354532, SGI0, SGI0, NULL};
const Segment * const SG85130[] = {SGI354533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354534, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85131[] = {SGI354535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354536, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85132[] = {SGI354537, SGI0, SGI0, SGI354538, SGI0, SGI0, SGI0, SGI0, SGI354539, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85133[] = {SGI354540, SGI0, SGI0, SGI354541, SGI0, SGI0, SGI0, SGI0, SGI354542, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85134[] = {SGI354543, SGI0, SGI0, SGI354544, SGI0, SGI0, SGI0, SGI0, SGI354545, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85135[] = {SGI354546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85136[] = {SGI354547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85137[] = {SGI354548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85138[] = {SGI354549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85139[] = {SGI354550, SGI0, SGI354551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85140[] = {SGI354552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354553, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85141[] = {SGI354554, SGI354555, SGI0, SGI354556, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85142[] = {SGI354557, SGI0, SGI0, SGI354558, SGI0, SGI0, SGI0, SGI0, SGI354559, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85143[] = {SGI354560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85144[] = {SGI354561, SGI354562, SGI354563, SGI0, SGI0, SGI354564, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85145[] = {SGI354565, SGI0, SGI0, SGI354566, SGI0, SGI0, SGI0, SGI0, SGI354567, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85146[] = {SGI354568, SGI0, SGI0, SGI354569, SGI0, SGI0, SGI0, SGI0, SGI354570, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85147[] = {SGI354571, SGI0, SGI0, SGI354572, SGI0, SGI0, SGI0, SGI0, SGI354573, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85148[] = {SGI354574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354575, SGI0, NULL};
const Segment * const SG85149[] = {SGI354576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354577, SGI0, NULL};
const Segment * const SG85150[] = {SGI354578, SGI0, SGI0, SGI354579, SGI0, SGI0, SGI0, SGI0, SGI354580, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85151[] = {SGI354581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354582, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85152[] = {SGI354583, SGI0, SGI0, SGI354584, SGI0, SGI0, SGI0, SGI0, SGI354585, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85153[] = {SGI354586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85154[] = {SGI354587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85155[] = {SGI354588, SGI0, SGI354589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85156[] = {SGI354590, SGI0, SGI354591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85157[] = {SGI354592, SGI354593, SGI354594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85158[] = {SGI354595, SGI354596, SGI354597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354597, SGI0, NULL};
const Segment * const SG85159[] = {SGI354598, SGI354599, SGI354600, SGI0, SGI0, SGI0, SGI0, SGI354601, SGI0, SGI0, NULL};
const Segment * const SG85160[] = {SGI354602, SGI354603, SGI354604, SGI0, SGI0, SGI0, SGI0, SGI354605, SGI0, SGI0, NULL};
const Segment * const SG85161[] = {SGI354606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354607, SGI0, SGI0, NULL};
const Segment * const SG85162[] = {SGI354608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354609, SGI0, SGI0, NULL};
const Segment * const SG85163[] = {SGI354610, SGI0, SGI354611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85164[] = {SGI354612, SGI0, SGI354613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85165[] = {SGI354614, SGI0, SGI354615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85166[] = {SGI354616, SGI0, SGI354617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85167[] = {SGI354618, SGI0, SGI354619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85168[] = {SGI354620, SGI0, SGI354621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85169[] = {SGI354622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354623, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85170[] = {SGI354624, SGI0, SGI0, SGI354625, SGI0, SGI0, SGI0, SGI0, SGI354626, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85171[] = {SGI354627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354628, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85172[] = {SGI354629, SGI0, SGI354630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354630, SGI0, NULL};
const Segment * const SG85173[] = {SGI354631, SGI0, SGI354632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354632, SGI0, NULL};
const Segment * const SG85174[] = {SGI354633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85175[] = {SGI354634, SGI0, SGI354635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85176[] = {SGI354636, SGI0, SGI354637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85177[] = {SGI354638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354639, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85178[] = {SGI354640, SGI0, SGI0, SGI354641, SGI0, SGI0, SGI0, SGI0, SGI354642, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85179[] = {SGI354643, SGI0, SGI0, SGI354644, SGI0, SGI0, SGI0, SGI0, SGI354645, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85180[] = {SGI354646, SGI0, SGI0, SGI354647, SGI0, SGI0, SGI0, SGI0, SGI354648, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85181[] = {SGI354649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85182[] = {SGI354650, SGI354651, SGI354652, SGI0, SGI0, SGI0, SGI0, SGI354653, SGI0, SGI0, NULL};
const Segment * const SG85183[] = {SGI354654, SGI354655, SGI354656, SGI0, SGI0, SGI0, SGI0, SGI354657, SGI0, SGI0, NULL};
const Segment * const SG85184[] = {SGI354658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85185[] = {SGI354659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354660, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85186[] = {SGI354661, SGI354662, SGI354663, SGI0, SGI0, SGI0, SGI0, SGI354664, SGI0, SGI0, NULL};
const Segment * const SG85187[] = {SGI354665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354666, SGI0, NULL};
const Segment * const SG85188[] = {SGI354667, SGI0, SGI354668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85189[] = {SGI354669, SGI0, SGI354670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85190[] = {SGI354671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354672, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85191[] = {SGI354673, SGI0, SGI354674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85192[] = {SGI354675, SGI0, SGI354676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85193[] = {SGI354677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85194[] = {SGI354678, SGI0, SGI354679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85195[] = {SGI354680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354681, NULL};
const Segment * const SG85196[] = {SGI354682, SGI0, SGI0, SGI354683, SGI0, SGI0, SGI0, SGI0, SGI354684, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85197[] = {SGI354685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354686, SGI0, NULL};
const Segment * const SG85198[] = {SGI354687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354688, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85199[] = {SGI354689, SGI0, SGI0, SGI354690, SGI0, SGI0, SGI0, SGI0, SGI354691, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85200[] = {SGI354692, SGI354693, SGI0, SGI0, NULL};
const Segment * const SG85201[] = {SGI354694, SGI354695, SGI0, SGI0, NULL};
const Segment * const SG85202[] = {SGI354696, SGI354697, SGI0, SGI0, NULL};
const Segment * const SG85203[] = {SGI354698, SGI354699, SGI0, SGI0, NULL};
const Segment * const SG85204[] = {SGI354700, SGI354701, SGI0, SGI0, NULL};
const Segment * const SG85205[] = {SGI354702, SGI354703, SGI0, SGI0, NULL};
const Segment * const SG85206[] = {SGI354704, SGI354705, SGI354706, SGI0, SGI354707, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85207[] = {SGI354708, SGI354709, SGI354710, SGI0, SGI354711, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85208[] = {SGI354712, SGI0, SGI0, SGI354713, SGI0, SGI0, SGI0, SGI0, SGI354714, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85209[] = {SGI354715, SGI0, SGI0, SGI354716, SGI0, SGI0, SGI0, SGI0, SGI354717, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85210[] = {SGI354718, SGI0, SGI354719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85211[] = {SGI354720, SGI0, SGI354721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85212[] = {SGI354722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354723, SGI0, SGI0, NULL};
const Segment * const SG85213[] = {SGI354724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354725, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85214[] = {SGI354726, SGI0, SGI0, SGI354727, SGI0, SGI0, SGI0, SGI0, SGI354728, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85215[] = {SGI354729, SGI0, SGI0, SGI354730, SGI0, SGI0, SGI0, SGI0, SGI354731, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85216[] = {SGI354732, SGI0, SGI0, SGI354733, SGI0, SGI0, SGI0, SGI0, SGI354734, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85217[] = {SGI354735, SGI0, SGI0, SGI354736, SGI0, SGI0, SGI0, SGI0, SGI354737, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85218[] = {SGI354738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354739, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85219[] = {SGI354740, SGI354741, SGI0, SGI0, NULL};
const Segment * const SG85220[] = {SGI354742, SGI354743, SGI0, SGI0, NULL};
const Segment * const SG85221[] = {SGI354744, SGI354745, SGI0, SGI0, NULL};
const Segment * const SG85222[] = {SGI354746, SGI354747, SGI0, SGI0, NULL};
const Segment * const SG85223[] = {SGI354748, SGI0, SGI354749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85224[] = {SGI354750, SGI0, SGI354751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85225[] = {SGI354752, SGI354753, SGI354754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85226[] = {SGI354755, SGI0, SGI354756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85227[] = {SGI354757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354758, SGI0, NULL};
const Segment * const SG85228[] = {SGI354759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354760, SGI0, NULL};
const Segment * const SG85229[] = {SGI354761, SGI0, SGI0, SGI354762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85230[] = {SGI354763, SGI354764, SGI354765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354766, SGI0, NULL};
const Segment * const SG85231[] = {SGI354767, SGI354768, SGI354769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354769, SGI0, NULL};
const Segment * const SG85232[] = {SGI354770, SGI354771, SGI354772, SGI0, SGI0, SGI0, SGI0, SGI354773, SGI0, SGI0, NULL};
const Segment * const SG85233[] = {SGI354774, SGI0, SGI0, SGI354775, SGI0, SGI0, SGI0, SGI0, SGI354776, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85234[] = {SGI354777, SGI0, SGI0, SGI354778, SGI0, SGI0, SGI0, SGI0, SGI354779, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85235[] = {SGI354780, SGI0, SGI354781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85236[] = {SGI354782, SGI354783, SGI354784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354784, SGI0, NULL};
const Segment * const SG85237[] = {SGI354785, SGI0, SGI0, SGI354786, SGI0, SGI0, SGI0, SGI0, SGI354787, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85238[] = {SGI354788, SGI0, SGI0, SGI354789, SGI0, SGI0, SGI0, SGI0, SGI354790, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85239[] = {SGI354791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85240[] = {SGI354792, SGI0, SGI354793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85241[] = {SGI354794, SGI0, SGI354795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85242[] = {SGI354796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354797, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85243[] = {SGI354798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354799, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85244[] = {SGI354800, SGI0, SGI0, SGI354801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85245[] = {SGI354802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85246[] = {SGI354803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354804, SGI0, NULL};
const Segment * const SG85247[] = {SGI354805, SGI0, SGI0, SGI354806, SGI0, SGI0, SGI0, SGI0, SGI354807, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85248[] = {SGI354808, SGI0, SGI0, SGI354809, SGI0, SGI0, SGI0, SGI0, SGI354810, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85249[] = {SGI354811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85250[] = {SGI354812, SGI0, SGI354813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85251[] = {SGI354814, SGI0, SGI354815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85252[] = {SGI354816, SGI0, SGI354817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85253[] = {SGI354818, SGI0, SGI354819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85254[] = {SGI354820, SGI0, SGI354821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85255[] = {SGI354822, SGI0, SGI354823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85256[] = {SGI354824, SGI0, SGI354825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85257[] = {SGI354826, SGI0, SGI354827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85258[] = {SGI354828, SGI0, SGI354829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85259[] = {SGI354830, SGI0, SGI354831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85260[] = {SGI354832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354833, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85261[] = {SGI354834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354835, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85262[] = {SGI354836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354837, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85263[] = {SGI354838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85264[] = {SGI354839, SGI0, SGI0, SGI354840, SGI0, SGI0, SGI0, SGI0, SGI354841, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85265[] = {SGI354842, SGI0, SGI0, SGI354843, SGI0, SGI0, SGI0, SGI0, SGI354844, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85266[] = {SGI354845, SGI0, SGI0, SGI354846, SGI0, SGI0, SGI0, SGI0, SGI354847, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85267[] = {SGI354848, SGI0, SGI0, SGI354849, SGI0, SGI0, SGI0, SGI0, SGI354850, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85268[] = {SGI354851, SGI0, SGI354852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85269[] = {SGI354853, SGI0, SGI0, SGI354854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85270[] = {SGI354855, SGI0, SGI354856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85271[] = {SGI354857, SGI0, SGI354858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85272[] = {SGI354859, SGI0, SGI354860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85273[] = {SGI354861, SGI0, SGI354862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85274[] = {SGI354863, SGI0, SGI354864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85275[] = {SGI354865, SGI354866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85276[] = {SGI354867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354868, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85277[] = {SGI354869, SGI0, SGI354870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85278[] = {SGI354871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354872, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85279[] = {SGI354873, SGI354874, SGI354875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354875, SGI0, NULL};
const Segment * const SG85280[] = {SGI354876, SGI354877, SGI354878, SGI0, SGI0, SGI0, SGI0, SGI354879, SGI0, SGI0, NULL};
const Segment * const SG85281[] = {SGI354880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354881, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85282[] = {SGI354882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354883, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85283[] = {SGI354884, SGI0, SGI0, SGI354885, SGI0, SGI0, SGI0, SGI0, SGI354886, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85284[] = {SGI354887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354888, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85285[] = {SGI354889, SGI354890, SGI354891, SGI0, SGI0, SGI0, SGI0, SGI354892, SGI0, SGI0, NULL};
const Segment * const SG85286[] = {SGI354893, SGI0, SGI0, SGI354894, SGI0, SGI0, SGI0, SGI0, SGI354895, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85287[] = {SGI354896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354897, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85288[] = {SGI354898, SGI0, SGI354899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85289[] = {SGI354900, SGI0, SGI354901, SGI354901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85290[] = {SGI354902, SGI0, SGI354903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85291[] = {SGI354906, SGI0, SGI354907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85292[] = {SGI354904, SGI0, SGI0, SGI354905, SGI0, SGI0, SGI0, SGI0, SGI354908, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85293[] = {SGI354909, SGI0, SGI354910, SGI354910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85294[] = {SGI354911, SGI0, SGI354913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85295[] = {SGI354912, SGI0, SGI0, SGI354914, SGI0, SGI0, SGI0, SGI0, SGI354915, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85296[] = {SGI354916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354917, SGI0, NULL};
const Segment * const SG85297[] = {SGI354918, SGI354919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85298[] = {SGI354920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85299[] = {SGI354921, SGI0, SGI354922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85300[] = {SGI354923, SGI0, SGI354924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85301[] = {SGI354925, SGI0, SGI354926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85302[] = {SGI354927, SGI0, SGI354928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85303[] = {SGI354929, SGI0, SGI354930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85304[] = {SGI354931, SGI0, SGI0, SGI354932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85305[] = {SGI354933, SGI0, SGI354934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85306[] = {SGI354935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354936, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85307[] = {SGI354937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354938, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85308[] = {SGI354939, SGI0, SGI354940, SGI354940, SGI0, SGI0, SGI0, SGI0, SGI354940, SGI0, NULL};
const Segment * const SG85309[] = {SGI354941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354942, SGI0, NULL};
const Segment * const SG85310[] = {SGI354943, SGI354944, SGI0, SGI354945, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85311[] = {SGI354946, SGI354947, SGI0, SGI354948, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85312[] = {SGI354949, SGI0, SGI0, SGI354950, SGI0, SGI0, SGI0, SGI0, SGI354951, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85313[] = {SGI354952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85314[] = {SGI354953, SGI0, SGI354954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85315[] = {SGI354955, SGI354956, NULL};
const Segment * const SG85316[] = {SGI354957, SGI354958, NULL};
const Segment * const SG85317[] = {SGI354959, SGI0, SGI354960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85318[] = {SGI354961, SGI0, SGI354962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85319[] = {SGI354963, SGI0, SGI354964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85320[] = {SGI354965, SGI0, SGI354966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85321[] = {SGI354967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354968, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85322[] = {SGI354969, SGI0, SGI0, SGI354970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85323[] = {SGI354971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354972, NULL};
const Segment * const SG85324[] = {SGI354973, SGI0, SGI354974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85325[] = {SGI354975, SGI0, SGI0, SGI354976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85326[] = {SGI354977, SGI0, SGI0, SGI354978, SGI0, SGI0, SGI0, SGI0, SGI354979, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85327[] = {SGI354980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354981, SGI0, NULL};
const Segment * const SG85328[] = {SGI354982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354983, SGI0, NULL};
const Segment * const SG85329[] = {SGI354984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354985, SGI0, NULL};
const Segment * const SG85330[] = {SGI354986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354987, SGI0, NULL};
const Segment * const SG85331[] = {SGI354988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354989, SGI0, NULL};
const Segment * const SG85332[] = {SGI354990, SGI354991, NULL};
const Segment * const SG85333[] = {SGI354992, SGI354993, SGI354994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354994, SGI0, NULL};
const Segment * const SG85334[] = {SGI354995, SGI354996, SGI354997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI354997, SGI0, NULL};
const Segment * const SG85335[] = {SGI354998, SGI0, SGI0, SGI354999, SGI0, SGI0, SGI0, SGI0, SGI355000, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85336[] = {SGI355001, SGI0, SGI0, SGI355002, SGI0, SGI0, SGI0, SGI0, SGI355003, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85337[] = {SGI355004, SGI355006, SGI355007, SGI0, SGI0, SGI0, SGI0, SGI355009, SGI0, SGI0, NULL};
const Segment * const SG85338[] = {SGI355005, SGI0, SGI0, SGI355008, SGI0, SGI0, SGI0, SGI0, SGI355010, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85339[] = {SGI355011, SGI0, SGI0, SGI355012, SGI0, SGI0, SGI0, SGI0, SGI355013, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85340[] = {SGI355014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85341[] = {SGI355015, SGI0, SGI355016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85342[] = {SGI355017, SGI0, SGI355018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85343[] = {SGI355019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355020, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85344[] = {SGI355021, SGI0, SGI0, SGI355022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85345[] = {SGI355023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355024, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85346[] = {SGI355025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355026, SGI0, NULL};
const Segment * const SG85347[] = {SGI355027, SGI0, SGI0, SGI355028, SGI0, SGI0, SGI0, SGI0, SGI355029, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85348[] = {SGI355030, SGI0, SGI0, SGI355031, SGI0, SGI0, SGI0, SGI0, SGI355032, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85349[] = {SGI355033, SGI0, SGI0, SGI355034, SGI0, SGI0, SGI0, SGI0, SGI355035, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85350[] = {SGI355036, SGI0, SGI0, SGI355037, SGI0, SGI0, SGI0, SGI0, SGI355038, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85351[] = {SGI355039, SGI0, SGI0, SGI355040, SGI0, SGI0, SGI0, SGI0, SGI355041, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85352[] = {SGI355042, SGI0, SGI0, SGI355043, SGI0, SGI0, SGI0, SGI0, SGI355044, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85353[] = {SGI355045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85354[] = {SGI355046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355047, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85355[] = {SGI355048, SGI0, SGI355049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85356[] = {SGI355050, SGI0, SGI355051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85357[] = {SGI355052, SGI0, SGI355053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85358[] = {SGI355054, SGI0, SGI355055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85359[] = {SGI355056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355057, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85360[] = {SGI355058, SGI0, SGI0, SGI355059, SGI0, SGI0, SGI0, SGI0, SGI355060, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85361[] = {SGI355061, SGI0, SGI0, SGI355062, SGI0, SGI0, SGI0, SGI0, SGI355063, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85362[] = {SGI355064, SGI355065, SGI355066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355066, SGI0, NULL};
const Segment * const SG85363[] = {SGI355067, SGI0, SGI0, SGI355068, SGI0, SGI0, SGI0, SGI0, SGI355069, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85364[] = {SGI0, SGI355070, SGI355071, SGI355072, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85365[] = {SGI355073, SGI0, SGI355074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85366[] = {SGI355075, SGI0, SGI355076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85367[] = {SGI355077, SGI0, SGI355078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85368[] = {SGI355079, SGI0, SGI355080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85369[] = {SGI355081, SGI0, SGI355082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85370[] = {SGI0, SGI355083, SGI355084, SGI0, SGI0, SGI0, SGI355085, NULL};
const Segment * const SG85371[] = {SGI355086, SGI0, SGI355087, SGI0, SGI355089, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85372[] = {SGI0, SGI355088, SGI355090, SGI0, SGI0, SGI0, SGI355091, NULL};
const Segment * const SG85373[] = {SGI355092, SGI0, SGI355093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85374[] = {SGI355094, SGI0, SGI355095, SGI0, SGI355096, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85375[] = {SGI355097, SGI0, SGI355098, SGI0, SGI355099, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85376[] = {SGI355100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85377[] = {SGI355101, SGI0, SGI355102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85378[] = {SGI355103, SGI0, SGI355104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85379[] = {SGI355105, SGI0, SGI355106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85380[] = {SGI355107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355108, NULL};
const Segment * const SG85381[] = {SGI355109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355110, SGI0, NULL};
const Segment * const SG85382[] = {SGI355111, SGI0, SGI0, SGI355112, SGI0, SGI0, SGI0, SGI0, SGI355113, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85383[] = {SGI355114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85384[] = {SGI355115, SGI0, SGI355116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85385[] = {SGI355117, SGI0, SGI0, SGI355118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85386[] = {SGI355119, SGI0, SGI355120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85387[] = {SGI355121, SGI0, SGI355122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85388[] = {SGI355123, SGI0, SGI355124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85389[] = {SGI355125, SGI0, SGI355126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85390[] = {SGI355127, SGI0, SGI355128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85391[] = {SGI355129, SGI0, SGI355130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85392[] = {SGI355131, SGI0, SGI355132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85393[] = {SGI355133, SGI0, SGI355134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85394[] = {SGI355135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355136, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85395[] = {SGI355137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355138, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85396[] = {SGI355139, SGI355140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85397[] = {SGI355141, SGI355142, SGI0, SGI0, SGI355143, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85398[] = {SGI355144, SGI355145, SGI0, SGI0, SGI355146, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85399[] = {SGI355147, SGI0, SGI355148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85400[] = {SGI355149, SGI0, SGI355150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85401[] = {SGI355151, SGI0, SGI355152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85402[] = {SGI355153, SGI0, SGI355154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85403[] = {SGI355155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355156, SGI0, NULL};
const Segment * const SG85404[] = {SGI355157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355158, SGI0, NULL};
const Segment * const SG85405[] = {SGI355159, SGI0, SGI0, SGI355160, SGI0, SGI0, SGI0, SGI0, SGI355161, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85406[] = {SGI355162, SGI0, SGI0, SGI355163, SGI0, SGI0, SGI0, SGI0, SGI355164, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85407[] = {SGI355165, SGI355166, SGI355167, SGI0, SGI0, SGI0, SGI0, SGI355168, SGI0, SGI0, NULL};
const Segment * const SG85408[] = {SGI355169, SGI0, SGI0, SGI355170, SGI0, SGI0, SGI0, SGI0, SGI355171, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85409[] = {SGI355172, SGI0, SGI0, SGI355173, SGI0, SGI0, SGI0, SGI0, SGI355174, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85410[] = {SGI355175, SGI0, SGI355176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85411[] = {SGI355177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85412[] = {SGI355178, SGI355179, NULL};
const Segment * const SG85413[] = {SGI355180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85414[] = {SGI355181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355182, SGI0, NULL};
const Segment * const SG85415[] = {SGI355183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355184, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85416[] = {SGI355185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355186, SGI0, NULL};
const Segment * const SG85417[] = {SGI355187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355188, SGI0, NULL};
const Segment * const SG85418[] = {SGI355189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85419[] = {SGI355190, SGI0, SGI0, SGI355191, SGI0, SGI0, SGI0, SGI0, SGI355192, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85420[] = {SGI355193, SGI0, SGI0, SGI355194, SGI0, SGI0, SGI0, SGI0, SGI355195, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85421[] = {SGI355196, SGI0, SGI0, SGI355197, SGI0, SGI0, SGI0, SGI0, SGI355198, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85422[] = {SGI355199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355200, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85423[] = {SGI355201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85424[] = {SGI355202, SGI0, SGI355203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85425[] = {SGI355204, SGI0, SGI355205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85426[] = {SGI355206, SGI355207, SGI355208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355208, SGI0, NULL};
const Segment * const SG85427[] = {SGI355209, SGI0, SGI355210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355210, SGI0, NULL};
const Segment * const SG85428[] = {SGI355211, SGI355212, SGI355213, SGI0, SGI0, SGI0, SGI0, SGI355214, SGI0, SGI0, NULL};
const Segment * const SG85429[] = {SGI355215, SGI355216, SGI355217, SGI0, SGI0, SGI0, SGI0, SGI355218, SGI0, SGI0, NULL};
const Segment * const SG85430[] = {SGI0, SGI355219, SGI0, SGI0, NULL};
const Segment * const SG85431[] = {SGI355220, SGI355221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85432[] = {SGI355222, SGI0, SGI0, SGI355223, SGI0, SGI0, SGI0, SGI0, SGI355224, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85433[] = {SGI355225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85434[] = {SGI355226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85435[] = {SGI355227, SGI0, SGI355228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85436[] = {SGI355229, SGI0, SGI355230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85437[] = {SGI355231, SGI0, SGI355232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85438[] = {SGI355233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355234, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85439[] = {SGI355235, SGI0, SGI355236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85440[] = {SGI355237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355238, SGI0, NULL};
const Segment * const SG85441[] = {SGI355239, SGI0, SGI0, SGI355240, SGI0, SGI0, SGI0, SGI0, SGI355241, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85442[] = {SGI355242, SGI0, SGI0, SGI355243, SGI0, SGI0, SGI0, SGI0, SGI355244, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85443[] = {SGI355245, SGI0, SGI0, SGI355246, SGI0, SGI0, SGI0, SGI0, SGI355247, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85444[] = {SGI355248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355249, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85445[] = {SGI355250, SGI0, SGI0, SGI355251, SGI0, SGI0, SGI0, SGI0, SGI355252, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85446[] = {SGI355253, SGI0, SGI0, SGI355254, SGI0, SGI0, SGI0, SGI0, SGI355255, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85447[] = {SGI355256, SGI0, SGI0, SGI355257, SGI0, SGI0, SGI0, SGI0, SGI355258, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85448[] = {SGI355259, SGI355260, SGI0, SGI0, NULL};
const Segment * const SG85449[] = {SGI355261, SGI0, SGI355262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85450[] = {SGI355263, SGI0, SGI355264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85451[] = {SGI355265, SGI0, SGI355266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85452[] = {SGI355267, SGI0, SGI0, SGI0, SGI355268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85453[] = {SGI355269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355270, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85454[] = {SGI355271, SGI0, SGI355272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85455[] = {SGI355273, SGI355274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85456[] = {SGI355275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355276, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85457[] = {SGI355277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355278, SGI0, NULL};
const Segment * const SG85458[] = {SGI0, SGI355279, SGI355280, SGI0, SGI355281, SGI0, SGI0, NULL};
const Segment * const SG85459[] = {SGI0, SGI355282, SGI355283, SGI0, SGI355284, SGI0, SGI355285, NULL};
const Segment * const SG85460[] = {SGI0, SGI355286, SGI355287, SGI0, SGI355288, SGI0, SGI355289, NULL};
const Segment * const SG85461[] = {SGI355290, SGI0, SGI0, SGI355291, SGI0, SGI0, SGI0, SGI0, SGI355292, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85462[] = {SGI355293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355294, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85463[] = {SGI355295, SGI0, SGI0, SGI355296, SGI0, SGI0, SGI0, SGI0, SGI355297, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85464[] = {SGI355298, SGI355299, SGI0, SGI0, NULL};
const Segment * const SG85465[] = {SGI355300, SGI355301, SGI0, SGI0, NULL};
const Segment * const SG85466[] = {SGI355302, SGI355303, SGI0, SGI0, NULL};
const Segment * const SG85467[] = {SGI355304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85468[] = {SGI355305, SGI0, SGI355306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85469[] = {SGI355307, SGI0, SGI355308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85470[] = {SGI355309, SGI0, SGI355310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85471[] = {SGI355311, SGI0, SGI355312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85472[] = {SGI355313, SGI0, SGI0, SGI355314, SGI0, SGI0, SGI0, SGI0, SGI355315, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85473[] = {SGI355316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355317, SGI0, NULL};
const Segment * const SG85474[] = {SGI355318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355319, SGI0, NULL};
const Segment * const SG85475[] = {SGI355320, SGI0, SGI355321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85476[] = {SGI355322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85477[] = {SGI355323, SGI0, SGI0, SGI0, SGI355324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85478[] = {SGI355325, SGI355326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85479[] = {SGI355327, SGI355328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85480[] = {SGI355329, SGI0, SGI0, SGI355330, SGI0, SGI0, SGI0, SGI0, SGI355331, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85481[] = {SGI355332, SGI355333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85482[] = {SGI355334, SGI355335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85483[] = {SGI355336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355337, SGI0, NULL};
const Segment * const SG85484[] = {SGI355338, SGI355339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85485[] = {SGI0, SGI355340, SGI355341, SGI0, SGI0, SGI0, SGI355342, NULL};
const Segment * const SG85486[] = {SGI355343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355344, SGI0, NULL};
const Segment * const SG85487[] = {SGI355345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355346, SGI0, NULL};
const Segment * const SG85488[] = {SGI355347, SGI355348, SGI355349, SGI0, SGI0, SGI0, SGI0, SGI355350, SGI0, SGI0, NULL};
const Segment * const SG85489[] = {SGI355351, SGI0, SGI0, SGI355352, SGI0, SGI0, SGI0, SGI0, SGI355353, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85490[] = {SGI355354, SGI355355, SGI0, SGI0, NULL};
const Segment * const SG85491[] = {SGI355356, SGI355357, SGI0, SGI0, NULL};
const Segment * const SG85492[] = {SGI355358, SGI0, SGI355359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85493[] = {SGI355360, SGI0, SGI355361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85494[] = {SGI355362, SGI0, SGI355363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85495[] = {SGI355364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355365, NULL};
const Segment * const SG85496[] = {SGI355366, SGI355367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85497[] = {SGI355368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355369, SGI0, NULL};
const Segment * const SG85498[] = {SGI355370, SGI0, SGI0, SGI355371, SGI0, SGI0, SGI0, SGI0, SGI355372, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85499[] = {SGI355373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355374, SGI0, NULL};
const Segment * const SG85500[] = {SGI355375, SGI0, SGI0, SGI355376, SGI0, SGI0, SGI0, SGI0, SGI355377, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85501[] = {SGI355378, SGI355379, SGI355380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355381, SGI0, NULL};
const Segment * const SG85502[] = {SGI355382, SGI355383, SGI355384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355384, SGI0, NULL};
const Segment * const SG85503[] = {SGI355385, SGI0, SGI0, SGI355386, SGI0, SGI0, SGI0, SGI0, SGI355387, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85504[] = {SGI355388, SGI0, SGI355389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85505[] = {SGI355390, SGI0, SGI355391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85506[] = {SGI355392, SGI0, SGI355393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85507[] = {SGI355394, SGI0, SGI355395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85508[] = {SGI355396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355397, SGI0, NULL};
const Segment * const SG85509[] = {SGI355398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355399, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85510[] = {SGI355400, SGI0, SGI0, SGI355401, SGI0, SGI0, SGI0, SGI0, SGI355402, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85511[] = {SGI355403, SGI0, SGI0, SGI355404, SGI0, SGI0, SGI0, SGI0, SGI355405, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85512[] = {SGI355406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85513[] = {SGI0, SGI355407, SGI355408, SGI355409, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85514[] = {SGI355410, SGI355411, SGI355412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355412, SGI0, NULL};
const Segment * const SG85515[] = {SGI355413, SGI355414, SGI355415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355415, SGI0, NULL};
const Segment * const SG85516[] = {SGI355416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355417, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85517[] = {SGI355418, SGI0, SGI0, SGI355419, SGI0, SGI0, SGI0, SGI0, SGI355420, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85518[] = {SGI355421, SGI0, SGI0, SGI355422, SGI0, SGI0, SGI0, SGI0, SGI355423, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85519[] = {SGI355424, SGI0, SGI355425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355425, SGI0, NULL};
const Segment * const SG85520[] = {SGI355426, SGI355427, SGI355428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85521[] = {SGI355429, SGI355430, SGI355431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85522[] = {SGI355432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355433, SGI0, NULL};
const Segment * const SG85523[] = {SGI355434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85524[] = {SGI355435, SGI0, SGI355436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85525[] = {SGI355437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355438, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85526[] = {SGI355439, SGI0, SGI355440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85527[] = {SGI355441, SGI0, SGI0, SGI355442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85528[] = {SGI355443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355444, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85529[] = {SGI355445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355446, SGI0, SGI0, NULL};
const Segment * const SG85530[] = {SGI355447, SGI0, SGI355448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85531[] = {SGI355449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355450, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85532[] = {SGI355451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355452, SGI0, NULL};
const Segment * const SG85533[] = {SGI355453, SGI0, SGI0, SGI355454, SGI0, SGI0, SGI0, SGI0, SGI355455, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85534[] = {SGI355456, SGI0, SGI0, SGI355457, SGI0, SGI0, SGI0, SGI0, SGI355458, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85535[] = {SGI355459, SGI355460, NULL};
const Segment * const SG85536[] = {SGI355461, SGI0, SGI355462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85537[] = {SGI355463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85538[] = {SGI355464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355465, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85539[] = {SGI355466, SGI0, SGI0, SGI355467, SGI0, SGI0, SGI0, SGI0, SGI355468, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85540[] = {SGI355469, SGI0, SGI355470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85541[] = {SGI355471, SGI0, SGI355472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85542[] = {SGI355473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355474, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85543[] = {SGI355475, SGI355476, SGI355477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355478, SGI0, NULL};
const Segment * const SG85544[] = {SGI355479, SGI355480, SGI355481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355481, SGI0, NULL};
const Segment * const SG85545[] = {SGI355482, SGI355483, SGI355484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355484, SGI0, NULL};
const Segment * const SG85546[] = {SGI355485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355486, NULL};
const Segment * const SG85547[] = {SGI355487, SGI0, SGI355488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355488, SGI0, NULL};
const Segment * const SG85548[] = {SGI355489, SGI0, SGI355490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355490, SGI0, NULL};
const Segment * const SG85549[] = {SGI355491, SGI0, SGI355492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355492, SGI0, NULL};
const Segment * const SG85550[] = {SGI355493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85551[] = {SGI355494, SGI355495, SGI355496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355496, SGI0, NULL};
const Segment * const SG85552[] = {SGI355497, SGI355498, SGI355499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355499, SGI0, NULL};
const Segment * const SG85553[] = {SGI355500, SGI355501, SGI355502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355502, SGI0, NULL};
const Segment * const SG85554[] = {SGI355503, SGI355504, SGI355505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355505, SGI0, NULL};
const Segment * const SG85555[] = {SGI355506, SGI355507, SGI355508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355508, SGI0, NULL};
const Segment * const SG85556[] = {SGI355509, SGI355510, SGI355511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355511, SGI0, NULL};
const Segment * const SG85557[] = {SGI355512, SGI355513, SGI355514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355514, SGI0, NULL};
const Segment * const SG85558[] = {SGI355515, SGI355516, SGI355517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355517, SGI0, NULL};
const Segment * const SG85559[] = {SGI355518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355519, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85560[] = {SGI355520, SGI0, SGI0, SGI355521, SGI0, SGI0, SGI0, SGI0, SGI355522, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85561[] = {SGI355523, SGI355524, SGI355525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85562[] = {SGI355526, SGI355527, SGI355528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85563[] = {SGI355529, SGI355530, SGI355531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85564[] = {SGI355532, SGI355533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85565[] = {SGI355534, SGI0, SGI355535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85566[] = {SGI355536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85567[] = {SGI355537, SGI0, SGI355538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85568[] = {SGI355539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355540, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85569[] = {SGI355541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355542, SGI0, NULL};
const Segment * const SG85570[] = {SGI355543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355544, SGI0, NULL};
const Segment * const SG85571[] = {SGI355545, SGI0, SGI0, SGI355546, SGI0, SGI0, SGI0, SGI0, SGI355547, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85572[] = {SGI355548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355549, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85573[] = {SGI355550, SGI0, SGI0, SGI355551, SGI0, SGI0, SGI0, SGI0, SGI355552, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85574[] = {SGI355553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355554, SGI0, NULL};
const Segment * const SG85575[] = {SGI355555, SGI355556, SGI355557, SGI0, SGI0, SGI0, SGI0, SGI355558, SGI0, SGI0, NULL};
const Segment * const SG85576[] = {SGI355559, SGI0, SGI0, SGI355560, SGI0, SGI0, SGI0, SGI0, SGI355561, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85577[] = {SGI355562, SGI0, SGI355563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85578[] = {SGI355564, SGI0, SGI355565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85579[] = {SGI355566, SGI0, SGI355567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85580[] = {SGI355568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85581[] = {SGI355569, SGI355570, SGI0, SGI0, NULL};
const Segment * const SG85582[] = {SGI355571, SGI0, SGI355572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85583[] = {SGI355573, SGI355574, SGI355575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355575, SGI0, NULL};
const Segment * const SG85584[] = {SGI355576, SGI355577, NULL};
const Segment * const SG85585[] = {SGI355578, SGI355579, SGI355580, SGI0, SGI0, SGI0, SGI0, SGI355581, SGI0, SGI0, NULL};
const Segment * const SG85586[] = {SGI355582, SGI0, SGI355583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85587[] = {SGI355584, SGI0, SGI355585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85588[] = {SGI355586, SGI0, SGI355587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85589[] = {SGI355588, SGI0, SGI355589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85590[] = {SGI355590, SGI0, SGI355591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355591, SGI0, NULL};
const Segment * const SG85591[] = {SGI355592, SGI0, SGI355593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85592[] = {SGI355594, SGI0, SGI355595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355595, SGI0, NULL};
const Segment * const SG85593[] = {SGI355596, SGI0, SGI355597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355597, SGI0, NULL};
const Segment * const SG85594[] = {SGI355598, SGI0, SGI355599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355599, SGI0, NULL};
const Segment * const SG85595[] = {SGI355600, SGI0, SGI0, SGI355601, SGI0, SGI0, SGI0, SGI0, SGI355602, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85596[] = {SGI355603, SGI0, SGI0, SGI355604, SGI0, SGI0, SGI0, SGI0, SGI355605, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85597[] = {SGI355606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355607, SGI0, NULL};
const Segment * const SG85598[] = {SGI355608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355609, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85599[] = {SGI355610, SGI0, SGI0, SGI355611, SGI0, SGI0, SGI0, SGI0, SGI355612, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85600[] = {SGI355613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355614, SGI0, NULL};
const Segment * const SG85601[] = {SGI355615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355616, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85602[] = {SGI355617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355618, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85603[] = {SGI355619, SGI0, SGI0, SGI355620, SGI0, SGI0, SGI0, SGI0, SGI355621, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85604[] = {SGI355622, SGI0, SGI0, SGI355623, SGI0, SGI0, SGI0, SGI0, SGI355624, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85605[] = {SGI355625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85606[] = {SGI355626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85607[] = {SGI355627, SGI0, SGI355628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85608[] = {SGI355629, SGI0, SGI355630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85609[] = {SGI355631, SGI0, SGI355632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85610[] = {SGI355633, SGI0, SGI355634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85611[] = {SGI355635, SGI0, SGI0, SGI355636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85612[] = {SGI355637, SGI0, SGI355638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85613[] = {SGI355639, SGI355640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85614[] = {SGI355641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355642, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85615[] = {SGI355643, SGI355644, SGI0, SGI0, NULL};
const Segment * const SG85616[] = {SGI355645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355646, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85617[] = {SGI355647, SGI0, SGI0, SGI355648, SGI0, SGI0, SGI0, SGI0, SGI355649, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85618[] = {SGI355650, SGI0, SGI0, SGI355651, SGI0, SGI0, SGI0, SGI0, SGI355652, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85619[] = {SGI355653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355654, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85620[] = {SGI355655, SGI0, SGI0, SGI355656, SGI0, SGI0, SGI0, SGI0, SGI355657, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85621[] = {SGI355658, SGI0, SGI0, SGI355659, SGI0, SGI0, SGI0, SGI0, SGI355660, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85622[] = {SGI355661, SGI0, SGI0, SGI355662, SGI0, SGI0, SGI0, SGI0, SGI355663, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85623[] = {SGI355664, SGI0, SGI0, SGI355665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85624[] = {SGI355666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355667, SGI0, NULL};
const Segment * const SG85625[] = {SGI355668, SGI0, SGI355669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85626[] = {SGI355670, SGI0, SGI355671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85627[] = {SGI355672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355673, SGI0, SGI0, NULL};
const Segment * const SG85628[] = {SGI355674, SGI0, SGI355675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85629[] = {SGI355676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355677, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85630[] = {SGI355678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355679, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85631[] = {SGI355680, SGI0, SGI355681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85632[] = {SGI355682, SGI0, SGI355683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85633[] = {SGI355684, SGI355685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85634[] = {SGI355686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355687, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85635[] = {SGI355688, SGI0, SGI355689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85636[] = {SGI355690, SGI355691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85637[] = {SGI355692, SGI0, SGI0, SGI355693, SGI0, SGI0, SGI0, SGI0, SGI355694, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85638[] = {SGI355695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85639[] = {SGI355696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355697, SGI0, NULL};
const Segment * const SG85640[] = {SGI355698, SGI355699, SGI355700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85641[] = {SGI355701, SGI0, SGI0, SGI355702, SGI0, SGI0, SGI0, SGI0, SGI355703, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85642[] = {SGI355704, SGI0, SGI0, SGI355705, SGI0, SGI0, SGI0, SGI0, SGI355706, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85643[] = {SGI355707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355708, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85644[] = {SGI355709, SGI0, SGI0, SGI355710, SGI0, SGI0, SGI0, SGI0, SGI355711, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85645[] = {SGI355712, SGI0, SGI0, SGI355713, SGI0, SGI0, SGI0, SGI0, SGI355714, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85646[] = {SGI355715, SGI0, SGI0, SGI355716, SGI0, SGI0, SGI0, SGI0, SGI355717, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85647[] = {SGI355718, SGI0, SGI0, SGI355719, SGI0, SGI0, SGI0, SGI0, SGI355720, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85648[] = {SGI355721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85649[] = {SGI355722, SGI0, SGI355723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85650[] = {SGI355724, SGI0, SGI355725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85651[] = {SGI355726, SGI0, SGI355727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85652[] = {SGI355728, SGI0, SGI355729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85653[] = {SGI355730, SGI0, SGI355731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85654[] = {SGI355732, SGI355733, SGI355734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85655[] = {SGI355735, SGI0, SGI355736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85656[] = {SGI355737, SGI0, SGI355738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85657[] = {SGI355739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355740, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85658[] = {SGI355741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355742, SGI0, NULL};
const Segment * const SG85659[] = {SGI355743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355744, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85660[] = {SGI355745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355746, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85661[] = {SGI355747, SGI0, SGI355748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85662[] = {SGI355749, SGI0, SGI355750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85663[] = {SGI355751, SGI0, SGI355752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85664[] = {SGI355753, SGI0, SGI355754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85665[] = {SGI355755, SGI0, SGI355756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85666[] = {SGI355757, SGI0, SGI0, SGI355758, SGI0, SGI0, SGI0, SGI0, SGI355759, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85667[] = {SGI355760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355761, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85668[] = {SGI355762, SGI0, SGI0, SGI355763, SGI0, SGI0, SGI0, SGI0, SGI355764, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85669[] = {SGI355765, SGI0, SGI0, SGI355766, SGI0, SGI0, SGI0, SGI0, SGI355767, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85670[] = {SGI355768, SGI355769, SGI355770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85671[] = {SGI355771, SGI355772, SGI355773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85672[] = {SGI355774, SGI355775, SGI355776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85673[] = {SGI355777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355778, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85674[] = {SGI355779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355780, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85675[] = {SGI355781, SGI0, SGI0, SGI355782, SGI0, SGI0, SGI0, SGI0, SGI355783, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85676[] = {SGI355784, SGI0, SGI0, SGI355785, SGI0, SGI0, SGI0, SGI0, SGI355786, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85677[] = {SGI355787, SGI0, SGI355788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85678[] = {SGI355789, SGI355790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85679[] = {SGI355791, SGI0, SGI355792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85680[] = {SGI355793, SGI0, SGI355794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85681[] = {SGI355795, SGI0, SGI355796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85682[] = {SGI355797, SGI0, SGI355798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85683[] = {SGI355799, SGI0, SGI355800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85684[] = {SGI355801, SGI0, SGI355802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85685[] = {SGI355803, SGI0, SGI0, SGI355804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85686[] = {SGI355805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355806, SGI0, SGI0, NULL};
const Segment * const SG85687[] = {SGI355807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355808, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85688[] = {SGI355809, SGI0, SGI355810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85689[] = {SGI355811, SGI355812, SGI0, SGI0, NULL};
const Segment * const SG85690[] = {SGI355813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355814, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85691[] = {SGI355815, SGI0, SGI0, SGI355816, SGI0, SGI0, SGI0, SGI0, SGI355817, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85692[] = {SGI355818, SGI355819, SGI0, SGI355820, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85693[] = {SGI355821, SGI355822, SGI0, SGI355823, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85694[] = {SGI355824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355825, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85695[] = {SGI355826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355827, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85696[] = {SGI355828, SGI0, SGI355829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85697[] = {SGI355830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355831, SGI0, NULL};
const Segment * const SG85698[] = {SGI355832, SGI0, SGI0, SGI355833, SGI0, SGI0, SGI0, SGI0, SGI355834, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85699[] = {SGI355835, SGI0, SGI0, SGI355836, SGI0, SGI0, SGI0, SGI0, SGI355837, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85700[] = {SGI355838, SGI0, SGI0, SGI355839, SGI0, SGI0, SGI0, SGI0, SGI355840, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85701[] = {SGI355841, SGI0, SGI0, SGI355842, SGI0, SGI0, SGI0, SGI0, SGI355843, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85702[] = {SGI355844, SGI0, SGI0, SGI355845, SGI0, SGI0, SGI0, SGI0, SGI355846, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85703[] = {SGI355847, SGI0, SGI355848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85704[] = {SGI355849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355850, SGI0, NULL};
const Segment * const SG85705[] = {SGI355851, SGI0, SGI0, SGI355852, SGI0, SGI0, SGI0, SGI0, SGI355853, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85706[] = {SGI355854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85707[] = {SGI355855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355856, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85708[] = {SGI355857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355858, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85709[] = {SGI355859, SGI0, SGI0, SGI355860, SGI0, SGI0, SGI0, SGI0, SGI355861, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85710[] = {SGI355862, SGI0, SGI0, SGI355863, SGI0, SGI0, SGI0, SGI0, SGI355864, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85711[] = {SGI355865, SGI0, SGI0, SGI355866, SGI0, SGI0, SGI0, SGI0, SGI355867, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85712[] = {SGI355868, SGI0, SGI0, SGI355869, SGI0, SGI0, SGI0, SGI0, SGI355870, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85713[] = {SGI355871, SGI0, SGI0, SGI355872, SGI0, SGI0, SGI0, SGI0, SGI355873, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85714[] = {SGI355874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85715[] = {SGI355875, SGI0, SGI355876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355876, SGI0, NULL};
const Segment * const SG85716[] = {SGI355877, SGI355878, SGI0, SGI0, NULL};
const Segment * const SG85717[] = {SGI355879, SGI355880, SGI0, SGI0, NULL};
const Segment * const SG85718[] = {SGI355881, SGI0, SGI355882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85719[] = {SGI355883, SGI0, SGI355884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85720[] = {SGI355885, SGI0, SGI0, SGI355886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85721[] = {SGI355887, SGI0, SGI355888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85722[] = {SGI355889, SGI0, SGI355890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85723[] = {SGI355891, SGI0, SGI355892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85724[] = {SGI355893, SGI0, SGI355894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85725[] = {SGI355895, SGI0, SGI0, SGI355896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85726[] = {SGI355897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355898, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85727[] = {SGI355899, SGI0, SGI355900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85728[] = {SGI355901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355902, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85729[] = {SGI355903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355904, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85730[] = {SGI355905, SGI0, SGI0, SGI355906, SGI0, SGI0, SGI0, SGI0, SGI355907, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85731[] = {SGI355908, SGI355909, SGI0, SGI0, NULL};
const Segment * const SG85732[] = {SGI355910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85733[] = {SGI355911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355912, SGI0, NULL};
const Segment * const SG85734[] = {SGI355913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85735[] = {SGI355914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355915, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85736[] = {SGI355916, SGI0, SGI355917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85737[] = {SGI355918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85738[] = {SGI355919, SGI0, SGI0, SGI355920, SGI0, SGI0, SGI0, SGI0, SGI355921, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85739[] = {SGI355922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355923, SGI0, NULL};
const Segment * const SG85740[] = {SGI355924, SGI0, SGI0, SGI355925, SGI0, SGI0, SGI0, SGI0, SGI355926, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85741[] = {SGI355927, SGI0, SGI0, SGI355928, SGI0, SGI0, SGI0, SGI0, SGI355929, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85742[] = {SGI355930, SGI355931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85743[] = {SGI355932, SGI0, SGI0, SGI355933, SGI0, SGI0, SGI0, SGI0, SGI355934, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85744[] = {SGI355935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355936, SGI0, NULL};
const Segment * const SG85745[] = {SGI355938, SGI355939, NULL};
const Segment * const SG85746[] = {SGI355937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355940, SGI0, NULL};
const Segment * const SG85747[] = {SGI355941, SGI0, SGI355942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85748[] = {SGI355943, SGI0, SGI355944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85749[] = {SGI355945, SGI0, SGI0, SGI355946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85750[] = {SGI355947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355948, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85751[] = {SGI355949, SGI0, SGI355950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85752[] = {SGI355951, SGI0, SGI355952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85753[] = {SGI355953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355954, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85754[] = {SGI355955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355956, SGI0, SGI0, NULL};
const Segment * const SG85755[] = {SGI355957, SGI0, SGI355958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85756[] = {SGI355959, SGI0, SGI0, SGI355960, SGI0, SGI0, SGI0, SGI0, SGI355961, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85757[] = {SGI355962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355963, SGI0, NULL};
const Segment * const SG85758[] = {SGI355964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355965, SGI0, NULL};
const Segment * const SG85759[] = {SGI355966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355967, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85760[] = {SGI355968, SGI355969, SGI355970, SGI0, SGI0, SGI0, SGI0, SGI355971, SGI0, SGI0, NULL};
const Segment * const SG85761[] = {SGI355972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355973, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85762[] = {SGI355974, SGI355975, SGI355976, SGI0, SGI355977, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85763[] = {SGI355978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355979, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85764[] = {SGI355980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355981, SGI0, NULL};
const Segment * const SG85765[] = {SGI355982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355983, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85766[] = {SGI355984, SGI0, SGI0, SGI355985, SGI0, SGI0, SGI0, SGI0, SGI355986, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85767[] = {SGI355987, SGI0, SGI0, SGI355988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85768[] = {SGI355989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI355993, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85769[] = {SGI355990, SGI355991, SGI355992, SGI0, SGI0, SGI0, SGI0, SGI355994, SGI0, SGI0, NULL};
const Segment * const SG85770[] = {SGI355995, SGI355996, SGI355997, SGI355998, NULL};
const Segment * const SG85771[] = {SGI355999, SGI0, SGI356000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85772[] = {SGI356001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356002, SGI0, NULL};
const Segment * const SG85773[] = {SGI356003, SGI356004, SGI356005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356006, SGI0, NULL};
const Segment * const SG85774[] = {SGI356007, SGI356008, SGI356009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356009, SGI0, NULL};
const Segment * const SG85775[] = {SGI356010, SGI0, SGI0, SGI356011, SGI0, SGI0, SGI0, SGI0, SGI356012, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85776[] = {SGI356013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356014, SGI0, NULL};
const Segment * const SG85777[] = {SGI356015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356016, SGI0, NULL};
const Segment * const SG85778[] = {SGI356017, SGI356018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85779[] = {SGI356019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356020, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85780[] = {SGI356021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356022, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85781[] = {SGI356023, SGI0, SGI0, SGI356024, SGI0, SGI0, SGI0, SGI0, SGI356025, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85782[] = {SGI356026, SGI0, SGI356027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85783[] = {SGI356028, SGI0, SGI356029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85784[] = {SGI356030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85785[] = {SGI356031, SGI0, SGI356032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85786[] = {SGI356033, SGI0, SGI356034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85787[] = {SGI356035, SGI0, SGI356036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85788[] = {SGI356037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356038, NULL};
const Segment * const SG85789[] = {SGI356039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356040, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85790[] = {SGI356041, SGI0, SGI356042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85791[] = {SGI356043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356044, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85792[] = {SGI356045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356046, SGI0, NULL};
const Segment * const SG85793[] = {SGI356047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356048, SGI0, NULL};
const Segment * const SG85794[] = {SGI356049, SGI0, SGI356050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85795[] = {SGI356051, SGI356052, SGI356053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356053, SGI0, NULL};
const Segment * const SG85796[] = {SGI356054, SGI356055, SGI356056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356056, SGI0, NULL};
const Segment * const SG85797[] = {SGI356057, SGI0, SGI0, SGI356058, SGI0, SGI0, SGI0, SGI0, SGI356059, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85798[] = {SGI356060, SGI0, SGI0, SGI356061, SGI0, SGI0, SGI0, SGI0, SGI356062, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85799[] = {SGI356063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356064, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85800[] = {SGI356065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356066, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85801[] = {SGI356067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356068, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85802[] = {SGI356069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356070, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85803[] = {SGI356071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356072, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85804[] = {SGI356073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356074, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85805[] = {SGI356075, SGI356076, SGI0, SGI356077, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85806[] = {SGI356078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356079, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85807[] = {SGI356080, SGI0, SGI0, SGI356081, SGI0, SGI0, SGI0, SGI0, SGI356082, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85808[] = {SGI356083, SGI0, SGI0, SGI356084, SGI0, SGI0, SGI0, SGI0, SGI356085, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85809[] = {SGI356086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356087, SGI0, SGI0, NULL};
const Segment * const SG85810[] = {SGI356088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356089, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85811[] = {SGI356090, SGI356091, SGI0, SGI356092, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85812[] = {SGI356093, SGI356094, SGI0, SGI356095, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85813[] = {SGI356096, SGI0, SGI0, SGI356097, SGI0, SGI0, SGI0, SGI0, SGI356098, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85814[] = {SGI356099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356100, SGI0, NULL};
const Segment * const SG85815[] = {SGI356101, SGI0, SGI0, SGI356102, SGI0, SGI0, SGI0, SGI0, SGI356103, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85816[] = {SGI356104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356105, SGI0, NULL};
const Segment * const SG85817[] = {SGI356106, SGI0, SGI0, SGI356107, SGI0, SGI0, SGI0, SGI0, SGI356108, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85818[] = {SGI356109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356110, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85819[] = {SGI356111, SGI356112, SGI0, SGI356113, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85820[] = {SGI356114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356115, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85821[] = {SGI356116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356117, SGI0, NULL};
const Segment * const SG85822[] = {SGI356118, SGI0, SGI0, SGI356119, SGI0, SGI0, SGI0, SGI0, SGI356120, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85823[] = {SGI356121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356122, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85824[] = {SGI356123, SGI0, SGI0, SGI356124, SGI0, SGI0, SGI0, SGI0, SGI356125, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85825[] = {SGI356126, SGI356127, SGI356128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356128, SGI0, NULL};
const Segment * const SG85826[] = {SGI356129, SGI0, SGI0, SGI356130, SGI0, SGI0, SGI0, SGI0, SGI356131, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85827[] = {SGI356132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356133, SGI0, NULL};
const Segment * const SG85828[] = {SGI356134, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85829[] = {SGI356135, SGI356136, SGI0, SGI0, NULL};
const Segment * const SG85830[] = {SGI356137, SGI356138, SGI0, SGI356139, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85831[] = {SGI356140, SGI356141, SGI356142, SGI0, SGI0, SGI0, SGI0, SGI356143, SGI0, SGI0, NULL};
const Segment * const SG85832[] = {SGI356144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85833[] = {SGI356145, SGI0, SGI356146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85834[] = {SGI356147, SGI0, SGI356148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85835[] = {SGI356149, SGI0, SGI356150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85836[] = {SGI356151, SGI0, SGI356152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85837[] = {SGI356153, SGI0, SGI356154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85838[] = {SGI356155, SGI0, SGI356156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85839[] = {SGI356157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356158, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85840[] = {SGI356159, SGI0, SGI356160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85841[] = {SGI356161, SGI0, SGI356162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85842[] = {SGI356163, SGI0, SGI356164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85843[] = {SGI356165, SGI0, SGI356166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85844[] = {SGI356167, SGI0, SGI356168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85845[] = {SGI356169, SGI0, SGI356170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356171, SGI0, NULL};
const Segment * const SG85846[] = {SGI356172, SGI0, SGI356173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85847[] = {SGI356174, SGI0, SGI356175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356175, SGI0, NULL};
const Segment * const SG85848[] = {SGI356176, SGI0, SGI356177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356177, SGI0, NULL};
const Segment * const SG85849[] = {SGI356178, SGI0, SGI356179, SGI356179, SGI0, SGI0, SGI0, SGI0, SGI356179, SGI0, NULL};
const Segment * const SG85850[] = {SGI356180, SGI0, SGI356181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85851[] = {SGI356182, SGI0, SGI356183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85852[] = {SGI356184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356185, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85853[] = {SGI356186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356187, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85854[] = {SGI356188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356189, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85855[] = {SGI356190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356191, SGI0, NULL};
const Segment * const SG85856[] = {SGI356192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356193, NULL};
const Segment * const SG85857[] = {SGI356194, SGI0, SGI356195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85858[] = {SGI356196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356197, SGI0, NULL};
const Segment * const SG85859[] = {SGI356198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356199, SGI0, SGI0, NULL};
const Segment * const SG85860[] = {SGI356200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356201, SGI0, NULL};
const Segment * const SG85861[] = {SGI356202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356203, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85862[] = {SGI356204, SGI0, SGI0, SGI356205, SGI0, SGI0, SGI0, SGI0, SGI356206, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85863[] = {SGI356207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356208, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85864[] = {SGI356210, SGI356211, SGI356212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85865[] = {SGI356209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356213, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85866[] = {SGI356214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356215, SGI0, NULL};
const Segment * const SG85867[] = {SGI14352, SGI14353, SGI356216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85868[] = {SGI356217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356218, SGI0, NULL};
const Segment * const SG85869[] = {SGI46563, SGI46564, SGI356219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85870[] = {SGI356220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356221, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85871[] = {SGI356222, SGI0, SGI0, SGI356223, SGI0, SGI0, SGI0, SGI0, SGI356224, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85872[] = {SGI356225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356226, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85873[] = {SGI356227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356228, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85874[] = {SGI356229, SGI0, SGI356230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85875[] = {SGI356231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356232, SGI0, NULL};
const Segment * const SG85876[] = {SGI356233, SGI0, SGI0, SGI356234, SGI0, SGI0, SGI0, SGI0, SGI356235, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85877[] = {SGI356236, SGI356237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85878[] = {SGI356238, SGI356239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85879[] = {SGI356240, SGI356241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85880[] = {SGI356242, SGI356243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85881[] = {SGI356244, SGI0, SGI0, SGI356245, SGI0, SGI0, SGI0, SGI0, SGI356246, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85882[] = {SGI356247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356248, SGI0, NULL};
const Segment * const SG85883[] = {SGI356249, SGI0, SGI0, SGI356250, SGI0, SGI0, SGI0, SGI0, SGI356251, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85884[] = {SGI356252, SGI0, SGI0, SGI356253, SGI0, SGI0, SGI0, SGI0, SGI356254, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85885[] = {SGI356255, SGI356256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85886[] = {SGI356257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356258, SGI0, NULL};
const Segment * const SG85887[] = {SGI356259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356260, SGI0, NULL};
const Segment * const SG85888[] = {SGI356261, SGI0, SGI0, SGI356262, SGI0, SGI0, SGI0, SGI0, SGI356263, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85889[] = {SGI356264, SGI0, SGI356265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85890[] = {SGI356266, SGI0, SGI356267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85891[] = {SGI356268, SGI0, SGI356269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85892[] = {SGI356270, SGI0, SGI0, SGI0, SGI356271, SGI356272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85893[] = {SGI356273, SGI0, SGI356274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85894[] = {SGI356275, SGI356276, SGI0, SGI0, SGI356277, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85895[] = {SGI356278, SGI356279, NULL};
const Segment * const SG85896[] = {SGI356280, SGI0, SGI356281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85897[] = {SGI356282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356283, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85898[] = {SGI356284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356285, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85899[] = {SGI356286, SGI0, SGI0, SGI356287, SGI0, SGI0, SGI0, SGI0, SGI356288, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85900[] = {SGI356289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356290, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85901[] = {SGI356291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356292, NULL};
const Segment * const SG85902[] = {SGI356293, SGI0, SGI0, SGI356294, SGI0, SGI0, SGI0, SGI0, SGI356295, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85903[] = {SGI356296, SGI0, SGI356297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85904[] = {SGI356298, SGI0, SGI356299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85905[] = {SGI356300, SGI0, SGI356301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85906[] = {SGI356302, SGI0, SGI356303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85907[] = {SGI356304, SGI0, SGI0, SGI356305, SGI0, SGI0, SGI0, SGI0, SGI356306, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85908[] = {SGI356307, SGI0, SGI356308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85909[] = {SGI356309, SGI0, SGI356310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85910[] = {SGI356311, SGI356312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85911[] = {SGI356313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356314, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85912[] = {SGI356315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356316, SGI0, NULL};
const Segment * const SG85913[] = {SGI356318, SGI356319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85914[] = {SGI356317, SGI0, SGI0, SGI356320, SGI0, SGI0, SGI0, SGI0, SGI356321, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85915[] = {SGI356322, SGI356323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85916[] = {SGI356324, SGI356325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85917[] = {SGI356326, SGI0, SGI356327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85918[] = {SGI356328, SGI0, SGI356329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85919[] = {SGI356330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356331, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85920[] = {SGI356332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85921[] = {SGI356333, SGI0, SGI356334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85922[] = {SGI356335, SGI356336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85923[] = {SGI356337, SGI0, SGI356338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85924[] = {SGI356339, SGI356340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85925[] = {SGI356341, SGI356342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85926[] = {SGI356343, SGI356344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85927[] = {SGI356345, SGI0, SGI356346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85928[] = {SGI356347, SGI0, SGI356348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85929[] = {SGI356349, SGI0, SGI356350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85930[] = {SGI356351, SGI0, SGI356352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85931[] = {SGI356353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356354, NULL};
const Segment * const SG85932[] = {SGI356355, SGI0, SGI356356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85933[] = {SGI356357, SGI356358, SGI356359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85934[] = {SGI356360, SGI356361, SGI356362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85935[] = {SGI356363, SGI356364, SGI356365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85936[] = {SGI356366, SGI0, SGI356367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85937[] = {SGI356368, SGI0, SGI356369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85938[] = {SGI356370, SGI0, SGI356371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85939[] = {SGI356372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356373, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85940[] = {SGI356374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356375, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85941[] = {SGI356376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356377, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85942[] = {SGI356378, SGI356379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85943[] = {SGI356380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85944[] = {SGI356381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85945[] = {SGI356382, SGI356383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85946[] = {SGI356384, SGI0, SGI356385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85947[] = {SGI356386, SGI0, SGI356387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85948[] = {SGI356388, SGI0, SGI356389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85949[] = {SGI356390, SGI0, SGI356391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85950[] = {SGI356392, SGI0, SGI356393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85951[] = {SGI356394, SGI0, SGI356395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85952[] = {SGI356396, SGI0, SGI356397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85953[] = {SGI356398, SGI0, SGI0, SGI356399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85954[] = {SGI356400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356401, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85955[] = {SGI356402, SGI356403, SGI356404, SGI0, SGI0, SGI0, SGI0, SGI356405, SGI0, SGI0, NULL};
const Segment * const SG85956[] = {SGI356406, SGI0, SGI356407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85957[] = {SGI356408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356409, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85958[] = {SGI356410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356411, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85959[] = {SGI356412, SGI0, SGI0, SGI356413, SGI0, SGI0, SGI0, SGI0, SGI356414, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85960[] = {SGI356415, SGI0, SGI356416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85961[] = {SGI356417, SGI356418, SGI356419, SGI0, SGI356420, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85962[] = {SGI356421, SGI356422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85963[] = {SGI356423, SGI0, SGI356424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85964[] = {SGI356425, SGI0, SGI356426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85965[] = {SGI356427, SGI0, SGI356428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85966[] = {SGI356429, SGI0, SGI356430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85967[] = {SGI356431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85968[] = {SGI356432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356433, SGI0, NULL};
const Segment * const SG85969[] = {SGI356434, SGI0, SGI356435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85970[] = {SGI356436, SGI0, SGI356437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85971[] = {SGI356438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356439, SGI0, NULL};
const Segment * const SG85972[] = {SGI356440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356441, NULL};
const Segment * const SG85973[] = {SGI356442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356443, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85974[] = {SGI356444, SGI356445, SGI0, SGI0, NULL};
const Segment * const SG85975[] = {SGI356446, SGI0, SGI0, SGI356447, SGI0, SGI0, SGI0, SGI0, SGI356448, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85976[] = {SGI356449, SGI0, SGI356450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85977[] = {SGI356451, SGI0, SGI356452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85978[] = {SGI356453, SGI0, SGI0, SGI356454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85979[] = {SGI356455, SGI0, SGI356456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85980[] = {SGI356457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85981[] = {SGI356458, SGI0, SGI0, SGI356459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85982[] = {SGI356460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356461, NULL};
const Segment * const SG85983[] = {SGI356462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356463, SGI0, NULL};
const Segment * const SG85984[] = {SGI356464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356465, SGI0, SGI0, NULL};
const Segment * const SG85985[] = {SGI356466, SGI0, SGI356467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85986[] = {SGI356470, SGI356471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85987[] = {SGI356472, SGI356473, SGI356474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85988[] = {SGI356475, SGI356476, SGI356477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85989[] = {SGI356478, SGI0, SGI356479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85990[] = {SGI356480, SGI0, SGI356481, SGI356481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85991[] = {SGI356482, SGI0, SGI356483, SGI356483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85992[] = {SGI356468, SGI0, SGI356469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85993[] = {SGI356484, SGI0, SGI356485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85994[] = {SGI356486, SGI0, SGI356487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85995[] = {SGI356488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356489, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85996[] = {SGI356490, SGI356491, SGI0, SGI356492, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85997[] = {SGI356493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356494, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85998[] = {SGI356495, SGI0, SGI0, SGI356496, SGI0, SGI0, SGI0, SGI0, SGI356497, SGI0, SGI0, SGI0, NULL};
const Segment * const SG85999[] = {SGI356498, SGI0, SGI0, SGI356499, SGI0, SGI0, SGI0, SGI0, SGI356500, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86000[] = {SGI356501, SGI0, SGI356502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86001[] = {SGI356503, SGI0, SGI356504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86002[] = {SGI356505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86003[] = {SGI356506, SGI356507, SGI356508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86004[] = {SGI356509, SGI0, SGI356510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86005[] = {SGI356511, SGI0, SGI356512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86006[] = {SGI356513, SGI0, SGI356514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86007[] = {SGI356515, SGI0, SGI356516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86008[] = {SGI356517, SGI0, SGI356518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86009[] = {SGI356519, SGI0, SGI0, SGI0, SGI356520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86010[] = {SGI356521, SGI0, SGI0, SGI0, SGI356522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86011[] = {SGI356523, SGI356524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86012[] = {SGI356525, SGI356526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86013[] = {SGI356528, SGI356529, SGI356531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86014[] = {SGI356527, SGI0, SGI356530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86015[] = {SGI356532, SGI356533, SGI356534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86016[] = {SGI356535, SGI0, SGI356536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86017[] = {SGI356537, SGI356538, SGI356539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86018[] = {SGI356540, SGI356541, SGI356542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86019[] = {SGI356543, SGI356544, SGI356545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356545, SGI0, NULL};
const Segment * const SG86020[] = {SGI356546, SGI356547, SGI356548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356548, SGI0, NULL};
const Segment * const SG86021[] = {SGI356549, SGI356550, SGI356551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356551, SGI0, NULL};
const Segment * const SG86022[] = {SGI356552, SGI356553, SGI356554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356554, SGI0, NULL};
const Segment * const SG86023[] = {SGI356555, SGI356556, SGI356557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356557, SGI0, NULL};
const Segment * const SG86024[] = {SGI356558, SGI0, SGI356559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86025[] = {SGI356560, SGI356561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86026[] = {SGI356562, SGI356563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86027[] = {SGI356564, SGI0, SGI356565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86028[] = {SGI356566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356567, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86029[] = {SGI356568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356570, SGI0, SGI0, NULL};
const Segment * const SG86030[] = {SGI356569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356571, SGI0, NULL};
const Segment * const SG86031[] = {SGI356572, SGI0, SGI0, SGI356573, SGI0, SGI0, SGI0, SGI0, SGI356574, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86032[] = {SGI356575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86033[] = {SGI356576, SGI356577, SGI356578, SGI356579, NULL};
const Segment * const SG86034[] = {SGI356580, SGI0, SGI356581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86035[] = {SGI356582, SGI0, SGI356583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86036[] = {SGI356584, SGI0, SGI356585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86037[] = {SGI356586, SGI0, SGI0, SGI356587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86038[] = {SGI356588, SGI0, SGI0, SGI356589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86039[] = {SGI49533, SGI49534, SGI356590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86040[] = {SGI49361, SGI49362, SGI356591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86041[] = {SGI50628, SGI50629, SGI356592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86042[] = {SGI48065, SGI48066, SGI356593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86043[] = {SGI356594, SGI0, SGI0, SGI356595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86044[] = {SGI356596, SGI356597, SGI356598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86045[] = {SGI89717, SGI89718, SGI356599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86046[] = {SGI356600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356601, SGI0, SGI0, NULL};
const Segment * const SG86047[] = {SGI356602, SGI0, SGI356603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86048[] = {SGI356604, SGI0, SGI356605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86049[] = {SGI356606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86050[] = {SGI356607, SGI0, SGI356608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86051[] = {SGI356609, SGI356610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86052[] = {SGI356611, SGI356612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86053[] = {SGI356613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86054[] = {SGI356614, SGI0, SGI356615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356615, SGI0, NULL};
const Segment * const SG86055[] = {SGI356616, SGI0, SGI356617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86056[] = {SGI356618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86057[] = {SGI356619, SGI0, SGI0, SGI0, SGI356620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86058[] = {SGI356621, SGI0, SGI356622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86059[] = {SGI356623, SGI0, SGI356624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86060[] = {SGI356625, SGI0, SGI356626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86061[] = {SGI356627, SGI0, SGI0, SGI356628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86062[] = {SGI356629, SGI0, SGI356630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86063[] = {SGI356631, SGI0, SGI356632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86064[] = {SGI0, SGI356633, SGI356634, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86065[] = {SGI356635, SGI0, SGI356636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86066[] = {SGI356637, SGI356638, SGI356639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356639, SGI0, NULL};
const Segment * const SG86067[] = {SGI356640, SGI0, SGI0, SGI356641, SGI0, SGI0, SGI0, SGI0, SGI356642, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86068[] = {SGI356643, SGI0, SGI0, SGI356644, SGI0, SGI0, SGI0, SGI0, SGI356645, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86069[] = {SGI356646, SGI0, SGI0, SGI356647, SGI0, SGI0, SGI0, SGI0, SGI356648, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86070[] = {SGI356649, SGI356650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86071[] = {SGI356651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86072[] = {SGI356652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86073[] = {SGI356653, SGI0, SGI356654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86074[] = {SGI356655, SGI0, SGI356656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86075[] = {SGI356657, SGI0, SGI356658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86076[] = {SGI356659, SGI0, SGI0, SGI356660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86077[] = {SGI356661, SGI0, SGI356662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86078[] = {SGI356663, SGI0, SGI356664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86079[] = {SGI356665, SGI0, SGI356666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86080[] = {SGI356667, SGI0, SGI356668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86081[] = {SGI356669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356670, SGI0, NULL};
const Segment * const SG86082[] = {SGI356671, SGI0, SGI0, SGI356672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86083[] = {SGI356673, SGI0, SGI0, SGI356674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86084[] = {SGI356675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356679, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86085[] = {SGI356676, SGI356677, SGI356678, SGI0, SGI0, SGI0, SGI0, SGI356680, SGI0, SGI0, NULL};
const Segment * const SG86086[] = {SGI356681, SGI356682, SGI356683, SGI0, SGI0, SGI0, SGI0, SGI356684, SGI0, SGI0, NULL};
const Segment * const SG86087[] = {SGI356685, SGI0, SGI356686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86088[] = {SGI356687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356688, SGI0, NULL};
const Segment * const SG86089[] = {SGI356689, SGI356690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86090[] = {SGI356691, SGI0, SGI0, SGI356692, SGI0, SGI0, SGI0, SGI0, SGI356693, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86091[] = {SGI356694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86092[] = {SGI356695, SGI0, SGI0, SGI356696, SGI0, SGI0, SGI0, SGI0, SGI356697, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86093[] = {SGI356698, SGI0, SGI0, SGI356699, SGI0, SGI0, SGI0, SGI0, SGI356700, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86094[] = {SGI356701, SGI356702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86095[] = {SGI356703, SGI356704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86096[] = {SGI356705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86097[] = {SGI356706, SGI0, SGI356707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86098[] = {SGI356708, SGI0, SGI356709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86099[] = {SGI356710, SGI0, SGI356711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86100[] = {SGI356712, SGI0, SGI356713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86101[] = {SGI356714, SGI0, SGI356715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86102[] = {SGI356716, SGI0, SGI356717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86103[] = {SGI356718, SGI0, SGI356719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86104[] = {SGI356720, SGI0, SGI0, SGI0, SGI356721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86105[] = {SGI356722, SGI0, SGI356723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86106[] = {SGI356724, SGI0, SGI356725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86107[] = {SGI356726, SGI356727, SGI0, SGI0, NULL};
const Segment * const SG86108[] = {SGI356728, SGI0, SGI356729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86109[] = {SGI356730, SGI0, SGI356731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86110[] = {SGI356732, SGI0, SGI0, SGI356733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86111[] = {SGI356734, SGI0, SGI0, SGI356735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86112[] = {SGI356736, SGI0, SGI356737, SGI356737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86113[] = {SGI356738, SGI0, SGI356739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86114[] = {SGI356740, SGI0, SGI356741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86115[] = {SGI356742, SGI0, SGI356743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86116[] = {SGI356744, SGI356745, SGI0, SGI0, NULL};
const Segment * const SG86117[] = {SGI356746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356747, SGI0, SGI0, NULL};
const Segment * const SG86118[] = {SGI356748, SGI356749, SGI0, SGI0, NULL};
const Segment * const SG86119[] = {SGI356750, SGI356751, SGI0, SGI0, NULL};
const Segment * const SG86120[] = {SGI356752, SGI356753, SGI0, SGI0, NULL};
const Segment * const SG86121[] = {SGI356754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356755, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86122[] = {SGI356756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86123[] = {SGI356757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86124[] = {SGI356758, SGI0, SGI0, SGI356759, SGI0, SGI0, SGI0, SGI0, SGI356760, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86125[] = {SGI356761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86126[] = {SGI356762, SGI0, SGI356763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86127[] = {SGI356764, SGI0, SGI0, SGI356765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86128[] = {SGI356766, SGI0, SGI0, SGI356767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86129[] = {SGI356768, SGI0, SGI356769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86130[] = {SGI356770, SGI0, SGI356771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86131[] = {SGI356772, SGI0, SGI0, SGI356773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86132[] = {SGI356774, SGI0, SGI356775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86133[] = {SGI356776, SGI0, SGI356777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86134[] = {SGI356778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356779, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86135[] = {SGI356780, SGI0, SGI356781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86136[] = {SGI356782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356783, SGI0, SGI0, NULL};
const Segment * const SG86137[] = {SGI356784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356785, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86138[] = {SGI356786, SGI356787, SGI356788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86139[] = {SGI356789, SGI0, SGI0, SGI356790, SGI0, SGI0, SGI0, SGI0, SGI356791, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86140[] = {SGI356792, SGI0, SGI0, SGI356793, SGI0, SGI0, SGI0, SGI0, SGI356794, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86141[] = {SGI356795, SGI0, SGI356796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86142[] = {SGI356797, SGI0, SGI356798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86143[] = {SGI356799, SGI0, SGI356800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86144[] = {SGI356801, SGI0, SGI356802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86145[] = {SGI356803, SGI356804, SGI356805, SGI0, SGI0, SGI0, SGI0, SGI356806, SGI0, SGI0, NULL};
const Segment * const SG86146[] = {SGI356807, SGI356808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86147[] = {SGI356809, SGI0, SGI0, SGI356810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86148[] = {SGI356811, SGI0, SGI356812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86149[] = {SGI356813, SGI0, SGI0, SGI356814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86150[] = {SGI356815, SGI0, SGI0, SGI356816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86151[] = {SGI356817, SGI0, SGI356818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86152[] = {SGI356819, SGI0, SGI356820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86153[] = {SGI356821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356822, NULL};
const Segment * const SG86154[] = {SGI356823, SGI0, SGI356824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86155[] = {SGI356825, SGI0, SGI0, SGI356826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86156[] = {SGI356827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356828, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86157[] = {SGI356829, SGI0, SGI0, SGI356830, SGI0, SGI0, SGI0, SGI0, SGI356831, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86158[] = {SGI356832, SGI0, SGI0, SGI356833, SGI0, SGI0, SGI0, SGI0, SGI356834, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86159[] = {SGI356835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356836, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86160[] = {SGI356837, SGI0, SGI356838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86161[] = {SGI356839, SGI0, SGI0, SGI0, SGI356840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86162[] = {SGI356841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86163[] = {SGI356842, SGI0, SGI356843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86164[] = {SGI356844, SGI356845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86165[] = {SGI356846, SGI356847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86166[] = {SGI356848, SGI0, SGI0, SGI356849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86167[] = {SGI356850, SGI0, SGI356851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86168[] = {SGI356852, SGI0, SGI0, SGI356853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86169[] = {SGI356854, SGI0, SGI0, SGI356855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86170[] = {SGI356856, SGI0, SGI356857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86171[] = {SGI356858, SGI0, SGI356859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86172[] = {SGI356860, SGI0, SGI356861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86173[] = {SGI356862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86174[] = {SGI356863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86175[] = {SGI356864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356865, NULL};
const Segment * const SG86176[] = {SGI356866, SGI356867, SGI0, SGI0, NULL};
const Segment * const SG86177[] = {SGI356868, SGI356869, SGI0, SGI0, NULL};
const Segment * const SG86178[] = {SGI356870, SGI356871, SGI0, SGI0, NULL};
const Segment * const SG86179[] = {SGI356872, SGI356873, SGI0, SGI0, NULL};
const Segment * const SG86180[] = {SGI356874, SGI0, SGI0, SGI356875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86181[] = {SGI356876, SGI356877, SGI356878, SGI0, SGI0, SGI0, SGI0, SGI356879, SGI0, SGI0, NULL};
const Segment * const SG86182[] = {SGI356880, SGI356881, SGI356882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356882, SGI0, NULL};
const Segment * const SG86183[] = {SGI356883, SGI356884, SGI356885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356885, SGI0, NULL};
const Segment * const SG86184[] = {SGI356886, SGI356887, SGI356888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356888, SGI0, NULL};
const Segment * const SG86185[] = {SGI356889, SGI356890, SGI356891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356891, SGI0, NULL};
const Segment * const SG86186[] = {SGI356892, SGI0, SGI0, SGI356893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86187[] = {SGI356894, SGI0, SGI0, SGI356895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86188[] = {SGI356896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356897, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86189[] = {SGI356898, SGI356899, SGI0, SGI0, NULL};
const Segment * const SG86190[] = {SGI356900, SGI356901, SGI0, SGI0, NULL};
const Segment * const SG86191[] = {SGI356902, SGI0, SGI356903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86192[] = {SGI356904, SGI0, SGI0, SGI356905, SGI0, SGI0, SGI0, SGI0, SGI356906, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86193[] = {SGI356907, SGI356908, SGI356909, SGI0, SGI0, SGI0, SGI0, SGI356910, SGI0, SGI0, NULL};
const Segment * const SG86194[] = {SGI356911, SGI356912, SGI356913, SGI0, SGI0, SGI0, SGI0, SGI356914, SGI0, SGI0, NULL};
const Segment * const SG86195[] = {SGI356915, SGI356916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86196[] = {SGI356917, SGI0, SGI0, SGI356918, SGI0, SGI0, SGI0, SGI0, SGI356919, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86197[] = {SGI356920, SGI0, SGI0, SGI356921, SGI0, SGI0, SGI0, SGI0, SGI356922, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86198[] = {SGI356923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356924, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86199[] = {SGI356925, SGI0, SGI356926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86200[] = {SGI356927, SGI0, SGI356928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86201[] = {SGI356929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86202[] = {SGI356930, SGI0, SGI0, SGI356931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86203[] = {SGI356932, SGI0, SGI356933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86204[] = {SGI356934, SGI0, SGI356935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86205[] = {SGI356936, SGI0, SGI356937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86206[] = {SGI356938, SGI0, SGI356939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86207[] = {SGI356940, SGI0, SGI356941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86208[] = {SGI356942, SGI0, SGI356943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86209[] = {SGI356944, SGI0, SGI0, SGI356945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86210[] = {SGI356946, SGI0, SGI356947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86211[] = {SGI356948, SGI0, SGI0, SGI356949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86212[] = {SGI356950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356951, NULL};
const Segment * const SG86213[] = {SGI356952, SGI0, SGI356953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86214[] = {SGI356954, SGI356955, NULL};
const Segment * const SG86215[] = {SGI356956, SGI0, SGI0, SGI356957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86216[] = {SGI356958, SGI0, SGI356959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86217[] = {SGI356960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356961, NULL};
const Segment * const SG86218[] = {SGI356962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356963, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86219[] = {SGI356964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356965, SGI0, SGI0, NULL};
const Segment * const SG86220[] = {SGI356966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356967, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86221[] = {SGI356968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356969, SGI0, NULL};
const Segment * const SG86222[] = {SGI356970, SGI356971, SGI356972, SGI0, SGI0, SGI0, SGI0, SGI356973, SGI0, SGI0, NULL};
const Segment * const SG86223[] = {SGI356974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356975, SGI0, NULL};
const Segment * const SG86224[] = {SGI356976, SGI356977, SGI356978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI356978, SGI0, NULL};
const Segment * const SG86225[] = {SGI356979, SGI0, SGI0, SGI356980, SGI0, SGI0, SGI0, SGI0, SGI356981, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86226[] = {SGI356982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86227[] = {SGI356983, SGI356984, NULL};
const Segment * const SG86228[] = {SGI356985, SGI0, SGI356986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86229[] = {SGI356987, SGI0, SGI356988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86230[] = {SGI356989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86231[] = {SGI356990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86232[] = {SGI356991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86233[] = {SGI356992, SGI0, SGI356993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86234[] = {SGI356994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86235[] = {SGI356995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86236[] = {SGI356996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86237[] = {SGI356997, SGI0, SGI0, SGI356998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86238[] = {SGI356999, SGI0, SGI357000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86239[] = {SGI357001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357002, SGI0, NULL};
const Segment * const SG86240[] = {SGI357003, SGI0, SGI357004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86241[] = {SGI357005, SGI0, SGI357006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86242[] = {SGI357007, SGI0, SGI357008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86243[] = {SGI357009, SGI0, SGI357010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86244[] = {SGI357011, SGI0, SGI357012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86245[] = {SGI357013, SGI0, SGI0, SGI0, SGI357014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86246[] = {SGI357015, SGI0, SGI357016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86247[] = {SGI357017, SGI0, SGI0, SGI357018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86248[] = {SGI357019, SGI0, SGI0, SGI357020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86249[] = {SGI357021, SGI0, SGI0, SGI357022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86250[] = {SGI357023, SGI357024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86251[] = {SGI357025, SGI0, SGI0, SGI357026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86252[] = {SGI357027, SGI0, SGI0, SGI357028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86253[] = {SGI357029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357030, SGI0, SGI0, NULL};
const Segment * const SG86254[] = {SGI357031, SGI0, SGI357032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86255[] = {SGI357033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357034, NULL};
const Segment * const SG86256[] = {SGI357035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357036, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86257[] = {SGI357037, SGI0, SGI0, SGI357038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86258[] = {SGI357039, SGI0, SGI0, SGI357040, SGI0, SGI0, SGI0, SGI0, SGI357041, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86259[] = {SGI357042, SGI0, SGI0, SGI357043, SGI0, SGI0, SGI0, SGI0, SGI357044, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86260[] = {SGI357045, SGI357046, SGI357047, SGI0, SGI0, SGI0, SGI0, SGI357048, SGI0, SGI0, NULL};
const Segment * const SG86261[] = {SGI357049, SGI357050, SGI357051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86262[] = {SGI357052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357053, SGI0, SGI0, NULL};
const Segment * const SG86263[] = {SGI357054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86264[] = {SGI357055, SGI0, SGI357056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86265[] = {SGI357057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86266[] = {SGI357058, SGI0, SGI0, SGI357059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86267[] = {SGI357060, SGI0, SGI0, SGI357061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86268[] = {SGI357062, SGI0, SGI0, SGI357063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86269[] = {SGI357064, SGI0, SGI0, SGI357065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86270[] = {SGI357066, SGI0, SGI0, SGI357067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86271[] = {SGI357068, SGI0, SGI0, SGI357069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86272[] = {SGI357070, SGI0, SGI357071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86273[] = {SGI357072, SGI0, SGI0, SGI0, SGI357073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86274[] = {SGI357074, SGI0, SGI0, SGI357075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86275[] = {SGI357076, SGI0, SGI357077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86276[] = {SGI357078, SGI0, SGI357079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86277[] = {SGI357080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357081, SGI0, NULL};
const Segment * const SG86278[] = {SGI357082, SGI0, SGI357083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86279[] = {SGI357084, SGI0, SGI0, SGI357085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86280[] = {SGI357086, SGI0, SGI0, SGI357087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86281[] = {SGI357088, SGI357089, SGI357090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86282[] = {SGI357091, SGI357092, SGI357093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86283[] = {SGI357094, SGI357095, SGI357096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86284[] = {SGI357097, SGI357098, SGI357099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86285[] = {SGI357100, SGI357101, SGI357102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86286[] = {SGI0, SGI357103, SGI357104, SGI0, SGI0, SGI0, SGI357105, NULL};
const Segment * const SG86287[] = {SGI357106, SGI357107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86288[] = {SGI357108, SGI0, SGI0, SGI357109, SGI0, SGI0, SGI0, SGI0, SGI357110, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86289[] = {SGI357111, SGI0, SGI357112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86290[] = {SGI357113, SGI0, SGI357114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86291[] = {SGI357115, SGI0, SGI357116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86292[] = {SGI357117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357118, SGI0, NULL};
const Segment * const SG86293[] = {SGI357119, SGI357120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86294[] = {SGI357121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86295[] = {SGI357122, SGI0, SGI357123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86296[] = {SGI357124, SGI0, SGI357125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86297[] = {SGI357126, SGI0, SGI0, SGI357127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86298[] = {SGI357128, SGI0, SGI0, SGI357129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86299[] = {SGI357130, SGI0, SGI357131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86300[] = {SGI357132, SGI0, SGI0, SGI357133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86301[] = {SGI357134, SGI0, SGI357135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86302[] = {SGI357136, SGI0, SGI357137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86303[] = {SGI357138, SGI357139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86304[] = {SGI357140, SGI357141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86305[] = {SGI357142, SGI357143, SGI357144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86306[] = {SGI357145, SGI357146, SGI357147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86307[] = {SGI357148, SGI357149, SGI357150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86308[] = {SGI357151, SGI0, SGI0, SGI0, SGI357154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86309[] = {SGI357152, SGI357153, SGI357155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86310[] = {SGI357156, SGI0, SGI0, SGI357157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86311[] = {SGI357158, SGI0, SGI0, SGI357159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86312[] = {SGI357160, SGI0, SGI0, SGI357161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86313[] = {SGI357162, SGI0, SGI357163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86314[] = {SGI357164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357165, NULL};
const Segment * const SG86315[] = {SGI357166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86316[] = {SGI357167, SGI0, SGI357168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86317[] = {SGI357169, SGI0, SGI0, SGI357170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86318[] = {SGI357171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357172, NULL};
const Segment * const SG86319[] = {SGI357173, SGI0, SGI0, SGI357174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86320[] = {SGI357175, SGI0, SGI0, SGI357176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86321[] = {SGI357177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357178, NULL};
const Segment * const SG86322[] = {SGI357179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357180, SGI0, NULL};
const Segment * const SG86323[] = {SGI357181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357182, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86324[] = {SGI357183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357184, SGI0, NULL};
const Segment * const SG86325[] = {SGI357185, SGI0, SGI0, SGI357186, SGI0, SGI0, SGI0, SGI0, SGI357187, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86326[] = {SGI357188, SGI0, SGI0, SGI357189, SGI0, SGI0, SGI0, SGI0, SGI357190, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86327[] = {SGI357191, SGI357192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86328[] = {SGI357193, SGI0, SGI357194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86329[] = {SGI357195, SGI0, SGI357196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86330[] = {SGI357197, SGI357198, SGI357199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357200, SGI0, NULL};
const Segment * const SG86331[] = {SGI357201, SGI357202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86332[] = {SGI357203, SGI357204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86333[] = {SGI357205, SGI0, SGI357206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86334[] = {SGI357209, SGI0, SGI357210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86335[] = {SGI357207, SGI357208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357211, NULL};
const Segment * const SG86336[] = {SGI357212, SGI357213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86337[] = {SGI357214, SGI357215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86338[] = {SGI357216, SGI357217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86339[] = {SGI357218, SGI0, SGI357219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86340[] = {SGI357220, SGI357221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86341[] = {SGI357222, SGI0, SGI357223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86342[] = {SGI357224, SGI0, SGI357225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86343[] = {SGI357226, SGI0, SGI357227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86344[] = {SGI357228, SGI0, SGI357229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86345[] = {SGI357230, SGI0, SGI0, SGI357231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86346[] = {SGI357232, SGI0, SGI357233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86347[] = {SGI357234, SGI0, SGI0, SGI357235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86348[] = {SGI357236, SGI0, SGI357237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86349[] = {SGI357238, SGI0, SGI0, SGI357239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86350[] = {SGI357240, SGI0, SGI357241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86351[] = {SGI357242, SGI0, SGI357243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86352[] = {SGI357244, SGI0, SGI357245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86353[] = {SGI357246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86354[] = {SGI357247, SGI357248, SGI357249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86355[] = {SGI357250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357251, SGI0, NULL};
const Segment * const SG86356[] = {SGI357252, SGI0, SGI0, SGI357253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86357[] = {SGI357254, SGI0, SGI0, SGI357255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86358[] = {SGI357256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357257, SGI0, SGI0, NULL};
const Segment * const SG86359[] = {SGI357258, SGI357259, SGI357260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357260, SGI0, NULL};
const Segment * const SG86360[] = {SGI357261, SGI0, SGI0, SGI357262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86361[] = {SGI357263, SGI0, SGI0, SGI357264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86362[] = {SGI357265, SGI0, SGI0, SGI357266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86363[] = {SGI357267, SGI0, SGI0, SGI357268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86364[] = {SGI357269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357270, SGI0, NULL};
const Segment * const SG86365[] = {SGI357271, SGI0, SGI357272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86366[] = {SGI357273, SGI0, SGI357274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86367[] = {SGI357275, SGI0, SGI357276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86368[] = {SGI357277, SGI0, SGI357278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86369[] = {SGI357279, SGI0, SGI357280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86370[] = {SGI357281, SGI0, SGI357282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86371[] = {SGI357283, SGI0, SGI0, SGI357284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86372[] = {SGI357285, SGI0, SGI0, SGI357286, SGI0, SGI0, SGI0, SGI0, SGI357287, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86373[] = {SGI357288, SGI357289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86374[] = {SGI357290, SGI0, SGI0, SGI357291, SGI0, SGI0, SGI0, SGI0, SGI357292, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86375[] = {SGI357293, SGI0, SGI357294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86376[] = {SGI357295, SGI0, SGI357296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86377[] = {SGI357297, SGI357298, SGI0, SGI0, NULL};
const Segment * const SG86378[] = {SGI357299, SGI357300, SGI0, SGI0, NULL};
const Segment * const SG86379[] = {SGI357301, SGI357302, SGI0, SGI357303, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86380[] = {SGI357304, SGI0, SGI0, SGI357305, SGI0, SGI0, SGI0, SGI0, SGI357306, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86381[] = {SGI357307, SGI357308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86382[] = {SGI357309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357310, SGI0, NULL};
const Segment * const SG86383[] = {SGI357311, SGI357312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86384[] = {SGI357313, SGI0, SGI0, SGI357314, SGI0, SGI0, SGI0, SGI0, SGI357315, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86385[] = {SGI357316, SGI0, SGI0, SGI357317, SGI0, SGI0, SGI0, SGI0, SGI357318, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86386[] = {SGI357319, SGI0, SGI0, SGI357320, SGI0, SGI0, SGI0, SGI0, SGI357321, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86387[] = {SGI357322, SGI0, SGI357323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86388[] = {SGI357324, SGI0, SGI357325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86389[] = {SGI357326, SGI0, SGI357327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86390[] = {SGI357328, SGI0, SGI357329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86391[] = {SGI357330, SGI0, SGI357331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86392[] = {SGI357332, SGI0, SGI357333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86393[] = {SGI357334, SGI0, SGI357335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86394[] = {SGI357336, SGI0, SGI357337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86395[] = {SGI357338, SGI0, SGI357339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86396[] = {SGI357340, SGI0, SGI357341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86397[] = {SGI357342, SGI0, SGI357343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86398[] = {SGI357344, SGI357345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86399[] = {SGI357346, SGI357347, SGI357348, SGI357349, NULL};
const Segment * const SG86400[] = {SGI357350, SGI0, SGI357351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86401[] = {SGI357352, SGI0, SGI357353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86402[] = {SGI357354, SGI0, SGI357355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86403[] = {SGI357356, SGI357357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86404[] = {SGI357358, SGI357359, SGI0, SGI357360, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86405[] = {SGI357361, SGI0, SGI357362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86406[] = {SGI357363, SGI357364, SGI357365, SGI0, SGI357366, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86407[] = {SGI357367, SGI0, SGI357368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86408[] = {SGI357369, SGI357370, SGI357371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86409[] = {SGI357372, SGI357373, SGI0, SGI0, NULL};
const Segment * const SG86410[] = {SGI357374, SGI0, SGI357375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86411[] = {SGI357376, SGI357377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86412[] = {SGI357378, SGI357379, SGI357380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357380, SGI0, NULL};
const Segment * const SG86413[] = {SGI357381, SGI0, SGI357382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86414[] = {SGI357383, SGI357384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86415[] = {SGI357385, SGI357386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86416[] = {SGI357387, SGI357388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86417[] = {SGI357389, SGI0, SGI357390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86418[] = {SGI357391, SGI357392, SGI357393, SGI0, SGI0, SGI0, SGI0, SGI357394, SGI0, SGI0, NULL};
const Segment * const SG86419[] = {SGI357395, SGI357396, SGI357397, SGI0, SGI0, SGI0, SGI0, SGI357398, SGI0, SGI0, NULL};
const Segment * const SG86420[] = {SGI357399, SGI357400, SGI357401, SGI0, SGI0, SGI0, SGI0, SGI357402, SGI0, SGI0, NULL};
const Segment * const SG86421[] = {SGI357403, SGI357404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86422[] = {SGI357405, SGI357406, SGI357407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86423[] = {SGI357408, SGI0, SGI357409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86424[] = {SGI357410, SGI0, SGI357411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86425[] = {SGI357412, SGI0, SGI357413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86426[] = {SGI357414, SGI0, SGI357415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86427[] = {SGI357416, SGI357417, SGI357418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357418, SGI0, NULL};
const Segment * const SG86428[] = {SGI357419, SGI357420, SGI357421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357421, SGI0, NULL};
const Segment * const SG86429[] = {SGI357422, SGI0, SGI357423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357423, SGI0, NULL};
const Segment * const SG86430[] = {SGI0, SGI357424, SGI357425, SGI0, SGI0, SGI0, SGI357426, NULL};
const Segment * const SG86431[] = {SGI357427, SGI0, SGI0, SGI357428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86432[] = {SGI357429, SGI0, SGI357430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86433[] = {SGI357431, SGI0, SGI0, SGI357432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86434[] = {SGI357433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86435[] = {SGI357434, SGI0, SGI357435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86436[] = {SGI357436, SGI0, SGI0, SGI357437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86437[] = {SGI357438, SGI0, SGI357439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86438[] = {SGI357440, SGI0, SGI0, SGI357441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86439[] = {SGI357442, SGI0, SGI357443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86440[] = {SGI357444, SGI0, SGI357445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86441[] = {SGI357446, SGI0, SGI0, SGI357447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86442[] = {SGI357448, SGI0, SGI357449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86443[] = {SGI357450, SGI0, SGI357451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86444[] = {SGI357452, SGI0, SGI357453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86445[] = {SGI357454, SGI0, SGI357455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86446[] = {SGI357456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86447[] = {SGI357457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86448[] = {SGI357458, SGI0, SGI0, SGI357459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86449[] = {SGI357460, SGI0, SGI0, SGI357461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86450[] = {SGI357462, SGI0, SGI0, SGI357463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86451[] = {SGI357464, SGI0, SGI0, SGI357465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86452[] = {SGI357466, SGI0, SGI357467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86453[] = {SGI357468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86454[] = {SGI357469, SGI0, SGI357470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357471, SGI0, NULL};
const Segment * const SG86455[] = {SGI357472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86456[] = {SGI357473, SGI0, SGI0, SGI357474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86457[] = {SGI357475, SGI0, SGI357476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86458[] = {SGI357477, SGI0, SGI0, SGI357478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86459[] = {SGI357479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357480, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86460[] = {SGI357481, SGI0, SGI0, SGI357482, SGI0, SGI0, SGI0, SGI0, SGI357483, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86461[] = {SGI357484, SGI0, SGI0, SGI357485, SGI0, SGI0, SGI0, SGI0, SGI357486, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86462[] = {SGI357487, SGI0, SGI0, SGI357488, SGI0, SGI0, SGI0, SGI0, SGI357489, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86463[] = {SGI357490, SGI357491, SGI0, SGI357492, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86464[] = {SGI357493, SGI357494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86465[] = {SGI357495, SGI357496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86466[] = {SGI357497, SGI357498, SGI357499, SGI0, SGI357500, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86467[] = {SGI357501, SGI0, SGI357502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86468[] = {SGI357503, SGI357504, SGI357505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86469[] = {SGI357506, SGI357507, SGI357508, SGI0, SGI0, SGI0, SGI0, SGI357509, SGI0, SGI0, NULL};
const Segment * const SG86470[] = {SGI357510, SGI357511, SGI357512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86471[] = {SGI357513, SGI357514, SGI357515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86472[] = {SGI357516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357517, SGI0, NULL};
const Segment * const SG86473[] = {SGI357518, SGI357519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86474[] = {SGI357520, SGI357521, NULL};
const Segment * const SG86475[] = {SGI0, SGI357522, SGI357523, SGI0, SGI0, SGI0, SGI357524, NULL};
const Segment * const SG86476[] = {SGI357525, SGI357526, SGI357527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357527, SGI0, NULL};
const Segment * const SG86477[] = {SGI357528, SGI357529, SGI357530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86478[] = {SGI357531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357532, SGI0, NULL};
const Segment * const SG86479[] = {SGI357533, SGI0, SGI0, SGI357534, SGI0, SGI0, SGI0, SGI0, SGI357535, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86480[] = {SGI357536, SGI0, SGI0, SGI357537, SGI0, SGI0, SGI0, SGI0, SGI357538, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86481[] = {SGI357539, SGI0, SGI0, SGI357540, SGI0, SGI0, SGI0, SGI0, SGI357541, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86482[] = {SGI357542, SGI0, SGI0, SGI357543, SGI0, SGI0, SGI0, SGI0, SGI357544, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86483[] = {SGI357545, SGI0, SGI0, SGI357546, SGI0, SGI0, SGI0, SGI0, SGI357547, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86484[] = {SGI357548, SGI0, SGI0, SGI357549, SGI0, SGI0, SGI0, SGI0, SGI357550, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86485[] = {SGI357551, SGI0, SGI0, SGI357552, SGI0, SGI0, SGI0, SGI0, SGI357553, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86486[] = {SGI357554, SGI0, SGI0, SGI357555, SGI0, SGI0, SGI0, SGI0, SGI357556, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86487[] = {SGI357557, SGI0, SGI0, SGI357558, SGI0, SGI0, SGI0, SGI0, SGI357559, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86488[] = {SGI357560, SGI0, SGI357561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86489[] = {SGI357562, SGI0, SGI357563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86490[] = {SGI357564, SGI357565, SGI0, SGI357566, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86491[] = {SGI357567, SGI357568, SGI0, SGI0, NULL};
const Segment * const SG86492[] = {SGI357569, SGI357570, SGI0, SGI0, NULL};
const Segment * const SG86493[] = {SGI357571, SGI357572, SGI0, SGI0, NULL};
const Segment * const SG86494[] = {SGI357573, SGI357574, SGI0, SGI0, NULL};
const Segment * const SG86495[] = {SGI357575, SGI357576, SGI357577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86496[] = {SGI357578, SGI357579, SGI357580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86497[] = {SGI357581, SGI357582, SGI357583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86498[] = {SGI357584, SGI357585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86499[] = {SGI357586, SGI357587, SGI357588, SGI0, SGI357589, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86500[] = {SGI357590, SGI357591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86501[] = {SGI357592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357593, SGI0, NULL};
const Segment * const SG86502[] = {SGI357594, SGI357595, SGI0, SGI357596, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86503[] = {SGI357597, SGI357598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86504[] = {SGI357599, SGI357600, SGI0, SGI357601, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86505[] = {SGI357602, SGI0, SGI357603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357603, SGI0, NULL};
const Segment * const SG86506[] = {SGI357604, SGI357605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86507[] = {SGI0, SGI357606, SGI357607, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86508[] = {SGI357608, SGI357609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86509[] = {SGI357610, SGI357611, NULL};
const Segment * const SG86510[] = {SGI357612, SGI357613, SGI357614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86511[] = {SGI357615, SGI357616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86512[] = {SGI357617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357618, SGI0, NULL};
const Segment * const SG86513[] = {SGI357619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357620, SGI0, NULL};
const Segment * const SG86514[] = {SGI357621, SGI357622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86515[] = {SGI357623, SGI357624, SGI357625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86516[] = {SGI48171, SGI48172, SGI357626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86517[] = {SGI15308, SGI15309, SGI357627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86518[] = {SGI357628, SGI357629, SGI357630, SGI0, SGI357631, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86519[] = {SGI357632, SGI357633, NULL};
const Segment * const SG86520[] = {SGI357634, SGI357635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86521[] = {SGI357636, SGI357637, SGI357638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86522[] = {SGI357639, SGI357640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86523[] = {SGI357641, SGI357642, SGI0, SGI357643, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86524[] = {SGI357644, SGI357645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86525[] = {SGI357646, SGI357647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86526[] = {SGI357648, SGI357649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86527[] = {SGI357650, SGI357651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86528[] = {SGI357652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357653, SGI0, NULL};
const Segment * const SG86529[] = {SGI357654, SGI357655, SGI0, SGI357656, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86530[] = {SGI357657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357658, SGI0, NULL};
const Segment * const SG86531[] = {SGI357659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357660, SGI0, NULL};
const Segment * const SG86532[] = {SGI357661, SGI357662, NULL};
const Segment * const SG86533[] = {SGI357663, SGI357664, NULL};
const Segment * const SG86534[] = {SGI357665, SGI357666, NULL};
const Segment * const SG86535[] = {SGI357667, SGI357668, NULL};
const Segment * const SG86536[] = {SGI357669, SGI357670, NULL};
const Segment * const SG86537[] = {SGI357671, SGI357672, SGI357673, SGI0, SGI0, SGI0, SGI0, SGI357674, SGI0, SGI0, NULL};
const Segment * const SG86538[] = {SGI357675, SGI357676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86539[] = {SGI357677, SGI357678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86540[] = {SGI357679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357680, SGI0, NULL};
const Segment * const SG86541[] = {SGI357681, SGI357682, SGI357683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357683, SGI0, NULL};
const Segment * const SG86542[] = {SGI357684, SGI357685, SGI357686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357686, SGI0, NULL};
const Segment * const SG86543[] = {SGI357687, SGI357688, SGI357689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357689, SGI0, NULL};
const Segment * const SG86544[] = {SGI357690, SGI357691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86545[] = {SGI357692, SGI357693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86546[] = {SGI357694, SGI357695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86547[] = {SGI357696, SGI357697, SGI357698, SGI357699, NULL};
const Segment * const SG86548[] = {SGI357700, SGI357701, SGI357702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86549[] = {SGI357703, SGI357704, SGI357705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86550[] = {SGI357706, SGI357707, SGI0, SGI357708, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86551[] = {SGI357709, SGI357710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86552[] = {SGI357711, SGI357712, SGI0, SGI357713, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86553[] = {SGI357714, SGI357715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86554[] = {SGI357716, SGI357717, SGI357718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357719, SGI0, NULL};
const Segment * const SG86555[] = {SGI357720, SGI357721, SGI0, SGI0, NULL};
const Segment * const SG86556[] = {SGI357722, SGI357723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86557[] = {SGI357724, SGI357725, SGI357726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357726, SGI0, NULL};
const Segment * const SG86558[] = {SGI357727, SGI357728, SGI357729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357729, SGI0, NULL};
const Segment * const SG86559[] = {SGI357730, SGI0, SGI357731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357732, SGI0, NULL};
const Segment * const SG86560[] = {SGI357733, SGI357734, SGI357735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86561[] = {SGI357736, SGI357737, SGI357738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86562[] = {SGI357739, SGI357740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86563[] = {SGI357741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357742, SGI0, NULL};
const Segment * const SG86564[] = {SGI357743, SGI357744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86565[] = {SGI357745, SGI357746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86566[] = {SGI0, SGI357747, SGI357748, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86567[] = {SGI357749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357750, SGI0, NULL};
const Segment * const SG86568[] = {SGI357751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357752, SGI0, NULL};
const Segment * const SG86569[] = {SGI357753, SGI357754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86570[] = {SGI357755, SGI357756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86571[] = {SGI357757, SGI357758, SGI357759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86572[] = {SGI357760, SGI357761, SGI357762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86573[] = {SGI357763, SGI357764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86574[] = {SGI357765, SGI357766, SGI357767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86575[] = {SGI357768, SGI357769, SGI357770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86576[] = {SGI357771, SGI357772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86577[] = {SGI0, SGI357773, SGI357774, SGI357775, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86578[] = {SGI357776, SGI357777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86579[] = {SGI357778, SGI357779, SGI357780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357781, NULL};
const Segment * const SG86580[] = {SGI357782, SGI357783, SGI357784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86581[] = {SGI357785, SGI357786, SGI357787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357788, NULL};
const Segment * const SG86582[] = {SGI357789, SGI357790, SGI357791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357792, NULL};
const Segment * const SG86583[] = {SGI357793, SGI357794, SGI357795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357796, NULL};
const Segment * const SG86584[] = {SGI357797, SGI357798, SGI357799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357800, NULL};
const Segment * const SG86585[] = {SGI357801, SGI357802, SGI357803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357804, NULL};
const Segment * const SG86586[] = {SGI357805, SGI357806, SGI357807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86587[] = {SGI357808, SGI357809, SGI357810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357811, NULL};
const Segment * const SG86588[] = {SGI357812, SGI0, SGI357813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86589[] = {SGI357814, SGI357815, SGI0, SGI0, NULL};
const Segment * const SG86590[] = {SGI357816, SGI357817, SGI0, SGI0, NULL};
const Segment * const SG86591[] = {SGI357818, SGI357819, SGI357820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86592[] = {SGI48763, SGI48764, SGI357821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86593[] = {SGI15786, SGI15787, SGI357822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86594[] = {SGI0, SGI357823, SGI357824, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86595[] = {SGI357825, SGI357826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86596[] = {SGI357827, SGI357828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86597[] = {SGI357829, SGI357830, SGI357831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357831, SGI0, NULL};
const Segment * const SG86598[] = {SGI357832, SGI357833, SGI357834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357834, SGI0, NULL};
const Segment * const SG86599[] = {SGI357835, SGI357836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86600[] = {SGI357837, SGI357838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86601[] = {SGI357839, SGI357840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86602[] = {SGI357841, SGI357842, NULL};
const Segment * const SG86603[] = {SGI0, SGI357843, NULL};
const Segment * const SG86604[] = {SGI0, SGI357844, SGI357845, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86605[] = {SGI357846, SGI357847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86606[] = {SGI357848, SGI357849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86607[] = {SGI357850, SGI357851, SGI0, SGI357852, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86608[] = {SGI357853, SGI357854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86609[] = {SGI357855, SGI357856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86610[] = {SGI357857, SGI357858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86611[] = {SGI357859, SGI357860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86612[] = {SGI357861, SGI357862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86613[] = {SGI357863, SGI357864, SGI357865, SGI0, SGI0, SGI0, SGI0, SGI357866, SGI0, SGI0, NULL};
const Segment * const SG86614[] = {SGI357867, SGI357868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86615[] = {SGI357869, SGI357870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86616[] = {SGI357871, SGI357872, SGI357873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86617[] = {SGI357874, SGI357875, SGI357876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86618[] = {SGI357877, SGI357878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86619[] = {SGI357879, SGI357880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86620[] = {SGI357881, SGI357882, SGI357883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86621[] = {SGI0, SGI357884, SGI357885, SGI0, SGI0, SGI0, SGI357886, NULL};
const Segment * const SG86622[] = {SGI0, SGI357887, SGI357888, SGI0, SGI0, SGI0, SGI357889, NULL};
const Segment * const SG86623[] = {SGI357890, SGI357891, SGI357892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86624[] = {SGI357893, SGI357894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86625[] = {SGI357895, SGI357896, SGI357897, SGI0, SGI0, SGI0, SGI0, SGI357898, SGI0, SGI0, NULL};
const Segment * const SG86626[] = {SGI357899, SGI357900, SGI357901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86627[] = {SGI357902, SGI357903, SGI357904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86628[] = {SGI357905, SGI357906, SGI0, SGI357907, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86629[] = {SGI357908, SGI357909, SGI357910, SGI0, SGI0, SGI0, SGI0, SGI357911, SGI0, SGI0, NULL};
const Segment * const SG86630[] = {SGI357912, SGI0, SGI357913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86631[] = {SGI357914, SGI0, SGI357915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86632[] = {SGI357916, SGI0, SGI357917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86633[] = {SGI357918, SGI0, SGI357919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86634[] = {SGI357920, SGI357921, SGI357922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86635[] = {SGI357923, SGI357924, SGI357925, SGI0, SGI0, SGI0, SGI0, SGI357926, SGI0, SGI0, NULL};
const Segment * const SG86636[] = {SGI357927, SGI357928, SGI357929, SGI0, SGI0, SGI0, SGI0, SGI357930, SGI0, SGI0, NULL};
const Segment * const SG86637[] = {SGI357931, SGI357932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86638[] = {SGI357933, SGI357934, SGI357935, SGI0, SGI0, SGI0, SGI0, SGI357936, SGI0, SGI0, NULL};
const Segment * const SG86639[] = {SGI0, SGI357937, SGI357938, SGI0, SGI357939, SGI0, SGI357940, NULL};
const Segment * const SG86640[] = {SGI0, SGI357941, SGI357942, SGI0, SGI0, SGI0, SGI357943, NULL};
const Segment * const SG86641[] = {SGI0, SGI357944, SGI357945, SGI0, SGI0, SGI0, SGI357946, NULL};
const Segment * const SG86642[] = {SGI357947, SGI357948, SGI0, SGI357949, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86643[] = {SGI357950, SGI357951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86644[] = {SGI0, SGI357952, SGI357953, SGI357954, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86645[] = {SGI357955, SGI357956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86646[] = {SGI357957, SGI357958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86647[] = {SGI357959, SGI357960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86648[] = {SGI357961, SGI357962, SGI357963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86649[] = {SGI357964, SGI357965, SGI357966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86650[] = {SGI357967, SGI357968, SGI357969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86651[] = {SGI357970, SGI0, SGI357971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86652[] = {SGI357972, SGI0, SGI357973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86653[] = {SGI0, SGI357974, SGI0, SGI0, SGI0, SGI0, SGI357975, NULL};
const Segment * const SG86654[] = {SGI357976, SGI357977, SGI357978, SGI0, SGI357979, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86655[] = {SGI357980, SGI357981, SGI357982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357983, SGI0, NULL};
const Segment * const SG86656[] = {SGI0, SGI357984, SGI357985, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86657[] = {SGI357986, SGI357987, SGI357988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357989, NULL};
const Segment * const SG86658[] = {SGI357990, SGI357991, SGI357992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI357993, NULL};
const Segment * const SG86659[] = {SGI357994, SGI357995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86660[] = {SGI357996, SGI357997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86661[] = {SGI357998, SGI357999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86662[] = {SGI358000, SGI358001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86663[] = {SGI358002, SGI358003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86664[] = {SGI358004, SGI358005, NULL};
const Segment * const SG86665[] = {SGI358006, SGI358007, SGI358008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86666[] = {SGI358009, SGI358010, SGI358011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86667[] = {SGI358012, SGI358013, SGI358014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86668[] = {SGI358015, SGI358016, SGI0, SGI358017, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86669[] = {SGI358018, SGI358019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86670[] = {SGI358020, SGI358021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86671[] = {SGI358022, SGI358023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86672[] = {SGI358024, SGI358025, SGI0, SGI358026, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86673[] = {SGI358027, SGI358028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86674[] = {SGI358029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358030, SGI0, NULL};
const Segment * const SG86675[] = {SGI358031, SGI358032, SGI358033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358033, SGI0, NULL};
const Segment * const SG86676[] = {SGI0, SGI358034, SGI358035, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86677[] = {SGI358036, SGI358037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86678[] = {SGI358038, SGI358039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86679[] = {SGI358040, SGI358041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86680[] = {SGI358042, SGI358043, NULL};
const Segment * const SG86681[] = {SGI358044, SGI358045, NULL};
const Segment * const SG86682[] = {SGI358046, SGI358047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358048, SGI0, NULL};
const Segment * const SG86683[] = {SGI358049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358050, SGI0, NULL};
const Segment * const SG86684[] = {SGI358051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358052, SGI0, NULL};
const Segment * const SG86685[] = {SGI358053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358054, SGI0, NULL};
const Segment * const SG86686[] = {SGI358055, SGI358056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86687[] = {SGI358057, SGI358058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86688[] = {SGI358059, SGI358060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86689[] = {SGI358061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358062, SGI0, NULL};
const Segment * const SG86690[] = {SGI358063, SGI358064, SGI0, SGI0, NULL};
const Segment * const SG86691[] = {SGI358065, SGI358066, SGI0, SGI0, NULL};
const Segment * const SG86692[] = {SGI0, SGI358067, SGI358068, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86693[] = {SGI358069, SGI358070, SGI358071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358072, SGI0, NULL};
const Segment * const SG86694[] = {SGI358073, SGI358074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86695[] = {SGI358075, SGI358076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86696[] = {SGI358077, SGI358078, SGI0, SGI358079, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86697[] = {SGI358080, SGI358081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86698[] = {SGI358082, SGI358083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86699[] = {SGI358084, SGI0, SGI358085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86700[] = {SGI358086, SGI0, SGI358087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358087, SGI0, NULL};
const Segment * const SG86701[] = {SGI358088, SGI358089, SGI358090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86702[] = {SGI358091, SGI358092, SGI358093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86703[] = {SGI358094, SGI358095, SGI358096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86704[] = {SGI358097, SGI358098, SGI0, SGI358099, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86705[] = {SGI358100, SGI358101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86706[] = {SGI358102, SGI358103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86707[] = {SGI358104, SGI358105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86708[] = {SGI358106, SGI358107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86709[] = {SGI358108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358109, SGI0, NULL};
const Segment * const SG86710[] = {SGI358110, SGI358111, SGI358112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86711[] = {SGI358113, SGI0, SGI358114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358114, SGI0, NULL};
const Segment * const SG86712[] = {SGI358115, SGI0, SGI358116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358116, SGI0, NULL};
const Segment * const SG86713[] = {SGI358117, SGI358118, NULL};
const Segment * const SG86714[] = {SGI358119, SGI358120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86715[] = {SGI358121, SGI358122, SGI358123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86716[] = {SGI358124, SGI358125, SGI0, SGI358126, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86717[] = {SGI358127, SGI358128, SGI0, SGI358129, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86718[] = {SGI358130, SGI358131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86719[] = {SGI358132, SGI358133, SGI0, SGI358134, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86720[] = {SGI358135, SGI358136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86721[] = {SGI358137, SGI358138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86722[] = {SGI0, SGI358139, SGI358140, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86723[] = {SGI358141, SGI358142, SGI0, SGI358143, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86724[] = {SGI358144, SGI358145, NULL};
const Segment * const SG86725[] = {SGI358146, SGI0, SGI358147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358147, SGI0, NULL};
const Segment * const SG86726[] = {SGI358148, SGI0, SGI358149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358149, SGI0, NULL};
const Segment * const SG86727[] = {SGI358150, SGI358151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86728[] = {SGI358152, SGI358153, SGI0, SGI358154, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86729[] = {SGI358155, SGI358156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86730[] = {SGI358157, SGI358158, SGI0, SGI358159, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86731[] = {SGI358160, SGI358161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86732[] = {SGI0, SGI358162, SGI358163, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86733[] = {SGI358164, SGI358165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86734[] = {SGI358166, SGI358167, SGI358168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358169, NULL};
const Segment * const SG86735[] = {SGI358170, SGI358171, SGI358172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358172, SGI0, NULL};
const Segment * const SG86736[] = {SGI358173, SGI358174, SGI0, SGI358175, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86737[] = {SGI358176, SGI358177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86738[] = {SGI358178, SGI358179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86739[] = {SGI358180, SGI358181, SGI0, SGI0, NULL};
const Segment * const SG86740[] = {SGI16457, SGI16458, SGI358182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86741[] = {SGI358183, SGI358184, SGI358185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86742[] = {SGI358186, SGI358187, SGI358188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358189, NULL};
const Segment * const SG86743[] = {SGI358190, SGI358191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86744[] = {SGI358192, SGI358193, SGI358194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86745[] = {SGI358195, SGI358196, SGI358197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358198, NULL};
const Segment * const SG86746[] = {SGI358199, SGI358200, SGI358201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358202, NULL};
const Segment * const SG86747[] = {SGI358203, SGI358204, SGI358205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358208, NULL};
const Segment * const SG86748[] = {SGI358206, SGI358207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86749[] = {SGI358209, SGI358210, SGI0, SGI358211, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86750[] = {SGI358212, SGI358213, SGI0, SGI358214, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86751[] = {SGI358215, SGI358216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86752[] = {SGI358217, SGI358218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86753[] = {SGI358219, SGI358220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86754[] = {SGI358221, SGI358222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86755[] = {SGI358223, SGI358224, NULL};
const Segment * const SG86756[] = {SGI358225, SGI358226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86757[] = {SGI358227, SGI358228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86758[] = {SGI358229, SGI358230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86759[] = {SGI358231, SGI358232, SGI358233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358234, SGI0, NULL};
const Segment * const SG86760[] = {SGI358235, SGI358236, SGI358237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358238, NULL};
const Segment * const SG86761[] = {SGI358239, SGI358240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86762[] = {SGI358241, SGI358242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86763[] = {SGI358243, SGI358244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86764[] = {SGI358245, SGI358246, SGI358247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358248, NULL};
const Segment * const SG86765[] = {SGI358249, SGI358250, SGI358251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358252, NULL};
const Segment * const SG86766[] = {SGI358253, SGI358254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86767[] = {SGI358255, SGI358256, SGI358257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358258, NULL};
const Segment * const SG86768[] = {SGI358259, SGI358260, SGI358261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358262, NULL};
const Segment * const SG86769[] = {SGI358263, SGI358264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86770[] = {SGI358265, SGI358266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86771[] = {SGI358267, SGI0, SGI358268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86772[] = {SGI358269, SGI358270, SGI0, SGI358271, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86773[] = {SGI358272, SGI358273, SGI358274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86774[] = {SGI358275, SGI0, SGI358276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358276, SGI0, NULL};
const Segment * const SG86775[] = {SGI358277, SGI0, SGI358278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358278, SGI0, NULL};
const Segment * const SG86776[] = {SGI358279, SGI358280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86777[] = {SGI358281, SGI358282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86778[] = {SGI358283, SGI358284, SGI358285, SGI0, SGI0, SGI0, SGI0, SGI358286, SGI0, SGI0, NULL};
const Segment * const SG86779[] = {SGI358287, SGI358288, SGI358289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86780[] = {SGI358290, SGI0, SGI358291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358292, SGI0, NULL};
const Segment * const SG86781[] = {SGI358293, SGI358294, SGI0, SGI358295, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86782[] = {SGI358296, SGI358297, SGI0, SGI358298, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86783[] = {SGI358299, SGI358300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86784[] = {SGI358301, SGI358302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86785[] = {SGI358303, SGI0, SGI358304, SGI358304, SGI0, SGI0, SGI0, SGI0, SGI358304, SGI0, NULL};
const Segment * const SG86786[] = {SGI358305, SGI358306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86787[] = {SGI358307, SGI358308, SGI358309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86788[] = {SGI358310, SGI358311, SGI0, SGI358312, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86789[] = {SGI358313, SGI358314, SGI0, SGI358315, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86790[] = {SGI358316, SGI358317, SGI0, SGI358318, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86791[] = {SGI358319, SGI358320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86792[] = {SGI0, SGI358321, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86793[] = {SGI358322, SGI0, SGI358323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86794[] = {SGI358324, SGI358325, SGI358326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86795[] = {SGI358327, SGI358328, SGI358329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86796[] = {SGI358330, SGI358331, SGI358332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86797[] = {SGI358333, SGI358334, SGI0, SGI358335, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86798[] = {SGI358336, SGI358337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86799[] = {SGI358338, SGI0, SGI358339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86800[] = {SGI101214, SGI101215, SGI358340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86801[] = {SGI358341, SGI358342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86802[] = {SGI53874, SGI53875, SGI358343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86803[] = {SGI358344, SGI358345, SGI358346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86804[] = {SGI358347, SGI358348, SGI358349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358350, NULL};
const Segment * const SG86805[] = {SGI358351, SGI358352, SGI358353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358354, NULL};
const Segment * const SG86806[] = {SGI0, SGI358355, SGI358356, SGI0, SGI0, SGI0, SGI358357, NULL};
const Segment * const SG86807[] = {SGI0, SGI358358, SGI358359, SGI0, SGI0, SGI0, SGI358360, NULL};
const Segment * const SG86808[] = {SGI358361, SGI358362, SGI0, SGI0, NULL};
const Segment * const SG86809[] = {SGI358363, SGI358364, SGI358365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358366, NULL};
const Segment * const SG86810[] = {SGI358367, SGI358368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86811[] = {SGI358369, SGI358370, SGI358371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358372, SGI0, NULL};
const Segment * const SG86812[] = {SGI358373, SGI358374, SGI358375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86813[] = {SGI358376, SGI0, SGI358377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86814[] = {SGI358378, SGI358379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86815[] = {SGI358380, SGI358381, SGI0, SGI0, SGI358382, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86816[] = {SGI358383, SGI358384, SGI0, SGI358385, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86817[] = {SGI358386, SGI358387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86818[] = {SGI358388, SGI358389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86819[] = {SGI358390, SGI358391, SGI0, SGI0, NULL};
const Segment * const SG86820[] = {SGI358392, SGI358393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86821[] = {SGI358394, SGI358395, SGI0, SGI0, NULL};
const Segment * const SG86822[] = {SGI358396, SGI358397, NULL};
const Segment * const SG86823[] = {SGI0, SGI358398, SGI358399, SGI0, SGI0, SGI0, SGI358400, NULL};
const Segment * const SG86824[] = {SGI358401, SGI358402, SGI0, SGI0, NULL};
const Segment * const SG86825[] = {SGI358403, SGI358404, SGI358405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86826[] = {SGI358406, SGI358407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86827[] = {SGI0, SGI358408, SGI358409, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86828[] = {SGI358410, SGI358411, SGI358412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86829[] = {SGI358413, SGI358414, SGI0, SGI358415, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86830[] = {SGI358416, SGI358417, SGI0, SGI358418, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86831[] = {SGI358419, SGI358420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86832[] = {SGI358421, SGI358422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86833[] = {SGI358423, SGI358424, SGI358425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86834[] = {SGI0, SGI358426, SGI358427, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86835[] = {SGI358428, SGI358429, SGI358430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86836[] = {SGI358431, SGI358432, SGI358433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358433, SGI0, NULL};
const Segment * const SG86837[] = {SGI358434, SGI358435, SGI358436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86838[] = {SGI358437, SGI358438, SGI358439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86839[] = {SGI358440, SGI0, SGI358441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86840[] = {SGI358442, SGI358443, SGI358444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86841[] = {SGI358445, SGI358446, SGI358447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358447, SGI0, NULL};
const Segment * const SG86842[] = {SGI0, SGI358448, SGI358449, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86843[] = {SGI358450, SGI0, SGI358451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86844[] = {SGI358452, SGI0, SGI358453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86845[] = {SGI358454, SGI358455, SGI358456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86846[] = {SGI358457, SGI358458, SGI358459, SGI0, SGI0, SGI0, SGI0, SGI358460, SGI0, SGI0, NULL};
const Segment * const SG86847[] = {SGI358461, SGI358462, SGI0, SGI0, NULL};
const Segment * const SG86848[] = {SGI358463, SGI358464, SGI358465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86849[] = {SGI358466, SGI0, SGI358467, SGI358467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86850[] = {SGI358468, SGI358469, SGI358470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86851[] = {SGI358471, SGI358472, SGI358473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358474, NULL};
const Segment * const SG86852[] = {SGI358475, SGI358476, SGI358477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86853[] = {SGI0, SGI358478, SGI358479, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86854[] = {SGI358480, SGI358481, NULL};
const Segment * const SG86855[] = {SGI358482, SGI358483, SGI358484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358485, NULL};
const Segment * const SG86856[] = {SGI358486, SGI358487, SGI358488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358489, NULL};
const Segment * const SG86857[] = {SGI358490, SGI358491, SGI358492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86858[] = {SGI358493, SGI358494, SGI358495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358496, NULL};
const Segment * const SG86859[] = {SGI358497, SGI358498, SGI358499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86860[] = {SGI358500, SGI0, SGI358501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86861[] = {SGI358502, SGI0, SGI358503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86862[] = {SGI358504, SGI358505, NULL};
const Segment * const SG86863[] = {SGI358506, SGI358507, SGI358508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358509, NULL};
const Segment * const SG86864[] = {SGI358510, SGI358511, SGI358512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86865[] = {SGI358513, SGI358514, SGI0, SGI0, NULL};
const Segment * const SG86866[] = {SGI358515, SGI0, SGI358516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86867[] = {SGI358517, SGI0, SGI358518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358518, SGI0, NULL};
const Segment * const SG86868[] = {SGI358519, SGI358520, SGI358521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358522, NULL};
const Segment * const SG86869[] = {SGI358523, SGI358524, NULL};
const Segment * const SG86870[] = {SGI358525, SGI0, SGI358526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86871[] = {SGI358527, SGI358528, SGI358529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86872[] = {SGI358530, SGI358531, SGI0, SGI0, NULL};
const Segment * const SG86873[] = {SGI358532, SGI358533, SGI358534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358535, NULL};
const Segment * const SG86874[] = {SGI0, SGI358536, SGI358537, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86875[] = {SGI0, SGI358538, SGI358539, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86876[] = {SGI358540, SGI358541, NULL};
const Segment * const SG86877[] = {SGI358542, SGI358543, SGI0, SGI0, NULL};
const Segment * const SG86878[] = {SGI358544, SGI358545, SGI358546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86879[] = {SGI358547, SGI0, SGI358548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358548, SGI0, NULL};
const Segment * const SG86880[] = {SGI358549, SGI0, SGI358550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86881[] = {SGI358551, SGI358552, SGI358553, SGI0, SGI0, SGI0, SGI0, SGI358554, SGI0, SGI0, NULL};
const Segment * const SG86882[] = {SGI358555, SGI358556, SGI358557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358558, SGI0, NULL};
const Segment * const SG86883[] = {SGI358559, SGI358560, SGI358561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358562, NULL};
const Segment * const SG86884[] = {SGI0, SGI358563, SGI358564, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86885[] = {SGI0, SGI358565, SGI358566, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86886[] = {SGI0, SGI358567, SGI358568, SGI0, SGI0, SGI0, SGI358569, NULL};
const Segment * const SG86887[] = {SGI358570, SGI0, SGI358571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86888[] = {SGI358572, SGI358573, SGI358574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358575, NULL};
const Segment * const SG86889[] = {SGI358576, SGI358577, SGI358578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86890[] = {SGI358579, SGI358580, SGI358581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86891[] = {SGI358582, SGI358583, SGI358584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358585, NULL};
const Segment * const SG86892[] = {SGI358586, SGI358587, SGI358588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358589, NULL};
const Segment * const SG86893[] = {SGI0, SGI358590, SGI0, SGI0, SGI358591, SGI0, SGI358592, NULL};
const Segment * const SG86894[] = {SGI358593, SGI358594, SGI358595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86895[] = {SGI358596, SGI358597, SGI358598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358599, NULL};
const Segment * const SG86896[] = {SGI0, SGI358600, SGI358601, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86897[] = {SGI358602, SGI0, SGI358603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358604, SGI0, NULL};
const Segment * const SG86898[] = {SGI0, SGI358605, SGI358606, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86899[] = {SGI358607, SGI0, SGI358608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86900[] = {SGI358609, SGI358610, SGI358611, SGI0, SGI0, SGI0, SGI358612, SGI0, SGI358611, SGI0, NULL};
const Segment * const SG86901[] = {SGI358613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86902[] = {SGI358614, SGI358615, SGI358616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86903[] = {SGI358617, SGI358618, NULL};
const Segment * const SG86904[] = {SGI358619, SGI358620, SGI358621, SGI0, SGI0, SGI0, SGI358622, SGI0, SGI358621, SGI0, NULL};
const Segment * const SG86905[] = {SGI0, SGI358623, SGI358624, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86906[] = {SGI358625, SGI358626, NULL};
const Segment * const SG86907[] = {SGI358627, SGI358628, NULL};
const Segment * const SG86908[] = {SGI358629, SGI358630, SGI0, SGI0, NULL};
const Segment * const SG86909[] = {SGI358631, SGI358632, NULL};
const Segment * const SG86910[] = {SGI358633, SGI358634, SGI358635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358636, NULL};
const Segment * const SG86911[] = {SGI358637, SGI358638, SGI0, SGI0, NULL};
const Segment * const SG86912[] = {SGI358639, SGI358640, SGI0, SGI0, NULL};
const Segment * const SG86913[] = {SGI358641, SGI358642, SGI358643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358643, SGI0, NULL};
const Segment * const SG86914[] = {SGI358644, SGI358645, SGI358646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358647, NULL};
const Segment * const SG86915[] = {SGI358648, SGI358649, SGI358650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358651, NULL};
const Segment * const SG86916[] = {SGI358652, SGI358653, SGI358654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86917[] = {SGI358655, SGI358656, SGI358657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358657, SGI0, NULL};
const Segment * const SG86918[] = {SGI358658, SGI358659, SGI358660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358661, NULL};
const Segment * const SG86919[] = {SGI358662, SGI358663, SGI358664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358665, NULL};
const Segment * const SG86920[] = {SGI358666, SGI358667, SGI358668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86921[] = {SGI358669, SGI358670, SGI0, SGI0, NULL};
const Segment * const SG86922[] = {SGI358671, SGI358672, SGI0, SGI0, NULL};
const Segment * const SG86923[] = {SGI358673, SGI358674, SGI358675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358676, NULL};
const Segment * const SG86924[] = {SGI358677, SGI358678, SGI358679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358679, SGI0, NULL};
const Segment * const SG86925[] = {SGI358680, SGI0, SGI358681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86926[] = {SGI358682, SGI358683, SGI358684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358685, NULL};
const Segment * const SG86927[] = {SGI358686, SGI358687, SGI358688, SGI0, SGI0, SGI0, SGI0, SGI358689, SGI0, SGI0, NULL};
const Segment * const SG86928[] = {SGI358690, SGI358691, SGI358692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86929[] = {SGI358693, SGI358694, NULL};
const Segment * const SG86930[] = {SGI358695, SGI0, SGI358696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86931[] = {SGI358697, SGI0, SGI358698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86932[] = {SGI358699, SGI358700, SGI0, SGI0, NULL};
const Segment * const SG86933[] = {SGI358701, SGI358702, SGI358703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358704, NULL};
const Segment * const SG86934[] = {SGI358705, SGI358706, SGI358707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358708, NULL};
const Segment * const SG86935[] = {SGI0, SGI358709, SGI358710, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86936[] = {SGI358711, SGI358712, SGI358713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358714, NULL};
const Segment * const SG86937[] = {SGI358715, SGI358716, SGI358717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358718, NULL};
const Segment * const SG86938[] = {SGI358719, SGI358720, SGI358721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86939[] = {SGI0, SGI358722, SGI358723, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86940[] = {SGI358724, SGI0, SGI358725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358725, SGI0, NULL};
const Segment * const SG86941[] = {SGI0, SGI358726, SGI358727, SGI358728, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86942[] = {SGI0, SGI358729, NULL};
const Segment * const SG86943[] = {SGI358730, SGI358731, SGI358732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358733, NULL};
const Segment * const SG86944[] = {SGI358734, SGI358735, SGI0, SGI0, NULL};
const Segment * const SG86945[] = {SGI0, SGI358736, NULL};
const Segment * const SG86946[] = {SGI358737, SGI358738, SGI358739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86947[] = {SGI358740, SGI358741, SGI358742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86948[] = {SGI358743, SGI358744, SGI358745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358746, NULL};
const Segment * const SG86949[] = {SGI358747, SGI358748, SGI358749, SGI0, SGI0, SGI0, SGI0, SGI358750, SGI0, SGI0, NULL};
const Segment * const SG86950[] = {SGI358751, SGI358752, NULL};
const Segment * const SG86951[] = {SGI358753, SGI358754, NULL};
const Segment * const SG86952[] = {SGI0, SGI358755, SGI358756, SGI0, SGI0, SGI0, SGI358757, NULL};
const Segment * const SG86953[] = {SGI358758, SGI358759, SGI0, SGI0, NULL};
const Segment * const SG86954[] = {SGI358760, SGI358761, SGI358762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358763, NULL};
const Segment * const SG86955[] = {SGI358764, SGI358765, SGI0, SGI0, NULL};
const Segment * const SG86956[] = {SGI358766, SGI358767, SGI358768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358769, NULL};
const Segment * const SG86957[] = {SGI358770, SGI358771, SGI358772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358773, NULL};
const Segment * const SG86958[] = {SGI358774, SGI358775, SGI358776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358776, SGI0, NULL};
const Segment * const SG86959[] = {SGI358777, SGI358778, SGI358779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358779, SGI0, NULL};
const Segment * const SG86960[] = {SGI358780, SGI358781, SGI358782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86961[] = {SGI358783, SGI358784, SGI0, SGI0, NULL};
const Segment * const SG86962[] = {SGI358785, SGI358786, SGI0, SGI0, NULL};
const Segment * const SG86963[] = {SGI358787, SGI0, SGI358788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358788, SGI0, NULL};
const Segment * const SG86964[] = {SGI0, SGI358789, SGI358790, SGI0, SGI0, SGI0, SGI358791, NULL};
const Segment * const SG86965[] = {SGI0, SGI358792, SGI358793, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86966[] = {SGI358794, SGI358795, SGI358796, SGI0, SGI358797, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86967[] = {SGI358798, SGI0, SGI358799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358800, SGI0, NULL};
const Segment * const SG86968[] = {SGI358801, SGI358802, SGI0, SGI0, NULL};
const Segment * const SG86969[] = {SGI358803, SGI0, SGI358804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86970[] = {SGI358805, SGI358806, SGI0, SGI0, NULL};
const Segment * const SG86971[] = {SGI358807, SGI358808, SGI0, SGI0, NULL};
const Segment * const SG86972[] = {SGI358809, SGI358810, NULL};
const Segment * const SG86973[] = {SGI358811, SGI358812, SGI358813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86974[] = {SGI358814, SGI358815, SGI0, SGI0, NULL};
const Segment * const SG86975[] = {SGI358816, SGI358817, SGI358818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358819, NULL};
const Segment * const SG86976[] = {SGI358820, SGI358821, SGI358822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358823, NULL};
const Segment * const SG86977[] = {SGI358824, SGI0, SGI358825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86978[] = {SGI358826, SGI358827, NULL};
const Segment * const SG86979[] = {SGI358828, SGI358829, SGI358830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86980[] = {SGI358831, SGI358832, SGI358833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358834, NULL};
const Segment * const SG86981[] = {SGI358835, SGI0, SGI358836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86982[] = {SGI358837, SGI358838, SGI358839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86983[] = {SGI358840, SGI358841, SGI358842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358842, SGI0, NULL};
const Segment * const SG86984[] = {SGI358843, SGI358844, SGI358845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358846, NULL};
const Segment * const SG86985[] = {SGI358847, SGI358848, SGI0, SGI0, NULL};
const Segment * const SG86986[] = {SGI358849, SGI358850, SGI358851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358852, NULL};
const Segment * const SG86987[] = {SGI358853, SGI358854, SGI358855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358855, SGI0, NULL};
const Segment * const SG86988[] = {SGI358856, SGI358857, SGI358858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86989[] = {SGI358859, SGI358860, SGI0, SGI0, NULL};
const Segment * const SG86990[] = {SGI358861, SGI358862, NULL};
const Segment * const SG86991[] = {SGI358863, SGI358864, NULL};
const Segment * const SG86992[] = {SGI358865, SGI358866, SGI0, SGI0, NULL};
const Segment * const SG86993[] = {SGI358867, SGI358868, SGI358869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86994[] = {SGI57774, SGI57775, SGI358870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86995[] = {SGI358871, SGI358872, SGI358873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358874, NULL};
const Segment * const SG86996[] = {SGI358875, SGI358876, SGI358877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358877, SGI0, NULL};
const Segment * const SG86997[] = {SGI358878, SGI358879, SGI358880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG86998[] = {SGI358881, SGI358882, SGI358883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358884, NULL};
const Segment * const SG86999[] = {SGI0, SGI358885, SGI358886, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87000[] = {SGI358887, SGI358888, SGI0, SGI0, NULL};
const Segment * const SG87001[] = {SGI358889, SGI0, SGI358890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358890, SGI0, NULL};
const Segment * const SG87002[] = {SGI358891, SGI358892, NULL};
const Segment * const SG87003[] = {SGI358893, SGI358894, SGI358895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358896, NULL};
const Segment * const SG87004[] = {SGI358897, SGI358898, SGI358899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87005[] = {SGI358900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87006[] = {SGI358901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87007[] = {SGI358902, SGI358903, SGI358904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87008[] = {SGI358905, SGI358906, SGI358907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87009[] = {SGI358908, SGI358909, SGI358910, SGI0, SGI0, SGI0, SGI0, SGI358911, SGI0, SGI0, NULL};
const Segment * const SG87010[] = {SGI0, SGI358912, NULL};
const Segment * const SG87011[] = {SGI358913, SGI358914, NULL};
const Segment * const SG87012[] = {SGI358915, SGI358916, SGI0, SGI0, NULL};
const Segment * const SG87013[] = {SGI0, SGI358917, SGI0, SGI0, SGI0, SGI0, SGI358918, NULL};
const Segment * const SG87014[] = {SGI358919, SGI358920, SGI358921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358921, SGI0, NULL};
const Segment * const SG87015[] = {SGI358922, SGI358923, SGI358924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87016[] = {SGI358925, SGI358926, SGI358927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87017[] = {SGI358928, SGI358929, NULL};
const Segment * const SG87018[] = {SGI358930, SGI0, SGI358931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87019[] = {SGI358932, SGI358933, SGI358934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87020[] = {SGI358935, SGI358936, SGI358937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87021[] = {SGI358938, SGI358939, NULL};
const Segment * const SG87022[] = {SGI358940, SGI358941, NULL};
const Segment * const SG87023[] = {SGI0, SGI358942, NULL};
const Segment * const SG87024[] = {SGI358943, SGI358944, NULL};
const Segment * const SG87025[] = {SGI358945, SGI358946, SGI358947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87026[] = {SGI358948, SGI358949, SGI358950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87027[] = {SGI358951, SGI358952, SGI358953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87028[] = {SGI0, SGI358954, NULL};
const Segment * const SG87029[] = {SGI358955, SGI358956, SGI358957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87030[] = {SGI358958, SGI358959, SGI358960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87031[] = {SGI358961, SGI358962, NULL};
const Segment * const SG87032[] = {SGI358963, SGI358964, SGI358965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87033[] = {SGI358966, SGI358967, SGI358968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87034[] = {SGI358969, SGI358970, SGI358971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358972, NULL};
const Segment * const SG87035[] = {SGI358973, SGI0, SGI358974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358974, SGI0, NULL};
const Segment * const SG87036[] = {SGI358975, SGI358976, NULL};
const Segment * const SG87037[] = {SGI358977, SGI358978, SGI358979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358979, SGI0, NULL};
const Segment * const SG87038[] = {SGI358980, SGI358981, SGI358982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358982, SGI0, NULL};
const Segment * const SG87039[] = {SGI358983, SGI358984, SGI358985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI358985, SGI0, NULL};
const Segment * const SG87040[] = {SGI358986, SGI358987, SGI358988, SGI0, SGI358989, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87041[] = {SGI0, SGI358990, NULL};
const Segment * const SG87042[] = {SGI358991, SGI358992, NULL};
const Segment * const SG87043[] = {SGI358993, SGI358994, NULL};
const Segment * const SG87044[] = {SGI358995, SGI358996, NULL};
const Segment * const SG87045[] = {SGI358997, SGI358998, SGI358999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI359000, NULL};
const Segment * const SG87046[] = {SGI359001, SGI359002, SGI359003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI359004, NULL};
const Segment * const SG87047[] = {SGI359005, SGI359006, SGI359007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI359008, NULL};
const Segment * const SG87048[] = {SGI359009, SGI359010, SGI359011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI359012, NULL};
const Segment * const SG87049[] = {SGI359013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87050[] = {SGI359014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87051[] = {SGI359015, SGI359016, NULL};
const Segment * const SG87052[] = {SGI359017, SGI359018, NULL};
const Segment * const SG87053[] = {SGI0, SGI299083, SGI299084, SGI299086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87054[] = {SGI0, SGI300890, SGI300891, SGI300893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87055[] = {SGI0, SGI303306, SGI303307, SGI303309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87056[] = {SGI359019, SGI359020, SGI0, SGI359021, SGI359020, SGI359022, SGI359023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87057[] = {SGI359024, SGI359025, SGI359026, SGI359027, SGI359025, SGI359028, SGI359029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87058[] = {SGI359030, SGI359031, SGI359032, SGI0, SGI359031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87059[] = {SGI359033, SGI359034, SGI359035, SGI359036, SGI359034, SGI359037, SGI359038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87060[] = {SGI359039, SGI359040, SGI359041, SGI359042, SGI359040, SGI359043, SGI359044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87061[] = {SGI359045, SGI359046, SGI359047, SGI359048, SGI359046, SGI359049, SGI359050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87062[] = {SGI359051, SGI359052, SGI359053, SGI359054, SGI359052, SGI359055, SGI359056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87063[] = {SGI359057, SGI359058, SGI359059, SGI359060, SGI359058, SGI359061, SGI359062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87064[] = {SGI359063, SGI359064, SGI359065, SGI359066, SGI359064, SGI359067, SGI359068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87065[] = {SGI359069, SGI359070, SGI359071, SGI359072, SGI359070, SGI359073, SGI359074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87066[] = {SGI359075, SGI359076, SGI359077, SGI359078, SGI359076, SGI359079, SGI359080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87067[] = {SGI359081, SGI359082, SGI359083, SGI359084, SGI359082, SGI359085, SGI359086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87068[] = {SGI359087, SGI359088, SGI359089, SGI359090, SGI359088, SGI359091, SGI359092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87069[] = {SGI359093, SGI359094, SGI359095, SGI359096, SGI359094, SGI359097, SGI359098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87070[] = {SGI359099, SGI359100, SGI359101, SGI359102, SGI359100, SGI359103, SGI359104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87071[] = {SGI359105, SGI359106, SGI359107, SGI359108, SGI359106, SGI359109, SGI359110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87072[] = {SGI359111, SGI359112, SGI359113, SGI359114, SGI359112, SGI359115, SGI359116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87073[] = {SGI359117, SGI359118, SGI359119, SGI359120, SGI359118, SGI359121, SGI359122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87074[] = {SGI359123, SGI359124, SGI359125, SGI359126, SGI359124, SGI359127, SGI359128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87075[] = {SGI359129, SGI359130, SGI359131, SGI359132, SGI359130, SGI359133, SGI359134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87076[] = {SGI359135, SGI359136, SGI359137, SGI359138, SGI359136, SGI359139, SGI359140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87077[] = {SGI359141, SGI359142, SGI359143, SGI359144, SGI359142, SGI359145, SGI359146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87078[] = {SGI359147, SGI359148, SGI359149, SGI359150, SGI359148, SGI359151, SGI359152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87079[] = {SGI359153, SGI359154, SGI359155, SGI359156, SGI359154, SGI359157, SGI359158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87080[] = {SGI359159, SGI359160, SGI359161, SGI359162, SGI359160, SGI359163, SGI359164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87081[] = {SGI359165, SGI359166, SGI359167, SGI359168, SGI359166, SGI359169, SGI359170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87082[] = {SGI359171, SGI359172, SGI359173, SGI359174, SGI359172, SGI359175, SGI359176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87083[] = {SGI359177, SGI359178, SGI359179, SGI359180, SGI359178, SGI359181, SGI359182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87084[] = {SGI359183, SGI359184, SGI359185, SGI359186, SGI359184, SGI359187, SGI359188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87085[] = {SGI359189, SGI359190, SGI359191, SGI359192, SGI359190, SGI359193, SGI359194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87086[] = {SGI359195, SGI359196, SGI359197, SGI359198, SGI359196, SGI359199, SGI359200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87087[] = {SGI359201, SGI359202, SGI359203, SGI359204, SGI359202, SGI359205, SGI359206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87088[] = {SGI359207, SGI359208, SGI359209, SGI359210, SGI359208, SGI359211, SGI359212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87089[] = {SGI359213, SGI359214, SGI359215, SGI359216, SGI359214, SGI359217, SGI359218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87090[] = {SGI359219, SGI359220, SGI359221, SGI359222, SGI359220, SGI359223, SGI359224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87091[] = {SGI359225, SGI359226, SGI359227, SGI359228, SGI359226, SGI359229, SGI359230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87092[] = {SGI359231, SGI359232, SGI359233, SGI359234, SGI359232, SGI359235, SGI359236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87093[] = {SGI359237, SGI359238, SGI359239, SGI359240, SGI359238, SGI359241, SGI359242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87094[] = {SGI359243, SGI359244, SGI359245, SGI359246, SGI359244, SGI359247, SGI359248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87095[] = {SGI359249, SGI359250, SGI359251, SGI359252, SGI359250, SGI359253, SGI359254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87096[] = {SGI359255, SGI359256, SGI359257, SGI359258, SGI359256, SGI359259, SGI359260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87097[] = {SGI359261, SGI359262, SGI359263, SGI359264, SGI359262, SGI359265, SGI359266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87098[] = {SGI359267, SGI359268, SGI359269, SGI359270, SGI359268, SGI359271, SGI359272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87099[] = {SGI359273, SGI359274, SGI359275, SGI359276, SGI359274, SGI359277, SGI359278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87100[] = {SGI359279, SGI359280, SGI359281, SGI359282, SGI359280, SGI359283, SGI359284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87101[] = {SGI359285, SGI359286, SGI359287, SGI359288, SGI359286, SGI359289, SGI359290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87102[] = {SGI359291, SGI359292, SGI359293, SGI359294, SGI359292, SGI359295, SGI359296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87103[] = {SGI359297, SGI359298, SGI359299, SGI359300, SGI359298, SGI359301, SGI359302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87104[] = {SGI359303, SGI359304, SGI359305, SGI359306, SGI359304, SGI359307, SGI359308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87105[] = {SGI359309, SGI359310, SGI359311, SGI359312, SGI359310, SGI359313, SGI359314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87106[] = {SGI359315, SGI359316, SGI359317, SGI359318, SGI359316, SGI359319, SGI359320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87107[] = {SGI359321, SGI359322, SGI359323, SGI359324, SGI359322, SGI359325, SGI359326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87108[] = {SGI359327, SGI359328, SGI359329, SGI359330, SGI359328, SGI359331, SGI359332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87109[] = {SGI359333, SGI359334, SGI359335, SGI359336, SGI359334, SGI359337, SGI359338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87110[] = {SGI359339, SGI359340, SGI359341, SGI359342, SGI359340, SGI359343, SGI359344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87111[] = {SGI359345, SGI359346, SGI359347, SGI359348, SGI359346, SGI359349, SGI359350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87112[] = {SGI359351, SGI359352, SGI359353, SGI359354, SGI359352, SGI359355, SGI359356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87113[] = {SGI359357, SGI359358, SGI359359, SGI359360, SGI359358, SGI359361, SGI359362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87114[] = {SGI359363, SGI359364, SGI359365, SGI359366, SGI359364, SGI359367, SGI359368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87115[] = {SGI359369, SGI359370, SGI359371, SGI359372, SGI359370, SGI359373, SGI359374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87116[] = {SGI359375, SGI359376, SGI359377, SGI359378, SGI359376, SGI359379, SGI359380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87117[] = {SGI359381, SGI359382, SGI359383, SGI359384, SGI359382, SGI359385, SGI359386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87118[] = {SGI359387, SGI359388, SGI359389, SGI359390, SGI359388, SGI359391, SGI359392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87119[] = {SGI359393, SGI359394, SGI359395, SGI359396, SGI359394, SGI359397, SGI359398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87120[] = {SGI359399, SGI359400, SGI359401, SGI359402, SGI359400, SGI359403, SGI359404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87121[] = {SGI359405, SGI359406, SGI359407, SGI359408, SGI359406, SGI359409, SGI359410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87122[] = {SGI359411, SGI359412, SGI359413, SGI359414, SGI359412, SGI359415, SGI359416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87123[] = {SGI359417, SGI359418, SGI359419, SGI359420, SGI359418, SGI359421, SGI359422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87124[] = {SGI359423, SGI359424, SGI359425, SGI359426, SGI359424, SGI359427, SGI359428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87125[] = {SGI359429, SGI359430, SGI359431, SGI359432, SGI359430, SGI359433, SGI359434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87126[] = {SGI359435, SGI359436, SGI359437, SGI359438, SGI359436, SGI359439, SGI359440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87127[] = {SGI359441, SGI359442, SGI359443, SGI359444, SGI359442, SGI359445, SGI359446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87128[] = {SGI359447, SGI359448, SGI359449, SGI359450, SGI359448, SGI359451, SGI359452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87129[] = {SGI359453, SGI359454, SGI359455, SGI359456, SGI359454, SGI359457, SGI359458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87130[] = {SGI359459, SGI359460, SGI359461, SGI359462, SGI359460, SGI359463, SGI359464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87131[] = {SGI359465, SGI359466, SGI359467, SGI359468, SGI359466, SGI359469, SGI359470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87132[] = {SGI359471, SGI359472, SGI359473, SGI359474, SGI359472, SGI359475, SGI359476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87133[] = {SGI359477, SGI359478, SGI359479, SGI359480, SGI359478, SGI359481, SGI359482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87134[] = {SGI359483, SGI359484, SGI359485, SGI359486, SGI359484, SGI359487, SGI359488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87135[] = {SGI359489, SGI359490, SGI359491, SGI359492, SGI359490, SGI359493, SGI359494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87136[] = {SGI359495, SGI359496, SGI359497, SGI359498, SGI359496, SGI359499, SGI359500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87137[] = {SGI359501, SGI359502, SGI359503, SGI359504, SGI359502, SGI359505, SGI359506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87138[] = {SGI359507, SGI359508, SGI359509, SGI359510, SGI359508, SGI359511, SGI359512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87139[] = {SGI359513, SGI359514, SGI359515, SGI359516, SGI359514, SGI359517, SGI359518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87140[] = {SGI359519, SGI359520, SGI359521, SGI359522, SGI359520, SGI359523, SGI359524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87141[] = {SGI359525, SGI359526, SGI359527, SGI359528, SGI359526, SGI359529, SGI359530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87142[] = {SGI359531, SGI359532, SGI359533, SGI359534, SGI359532, SGI359535, SGI359536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87143[] = {SGI359537, SGI359538, SGI359539, SGI359540, SGI359538, SGI359541, SGI359542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87144[] = {SGI359543, SGI359544, SGI359545, SGI359546, SGI359544, SGI359547, SGI359548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87145[] = {SGI359549, SGI359550, SGI359551, SGI359552, SGI359550, SGI359553, SGI359554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87146[] = {SGI359555, SGI359556, SGI359557, SGI359558, SGI359556, SGI359559, SGI359560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87147[] = {SGI359561, SGI359562, SGI359563, SGI359564, SGI359562, SGI359565, SGI359566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87148[] = {SGI359567, SGI359568, SGI359569, SGI359570, SGI359568, SGI359571, SGI359572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87149[] = {SGI359573, SGI359574, SGI359575, SGI359576, SGI359574, SGI359577, SGI359578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87150[] = {SGI359579, SGI359580, SGI359581, SGI359582, SGI359580, SGI359583, SGI359584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87151[] = {SGI359585, SGI359586, SGI359587, SGI359588, SGI359586, SGI359589, SGI359590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87152[] = {SGI359591, SGI359592, SGI359593, SGI359594, SGI359592, SGI359595, SGI359596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87153[] = {SGI359597, SGI359598, SGI359599, SGI359600, SGI359598, SGI359601, SGI359602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87154[] = {SGI359603, SGI359604, SGI359605, SGI359606, SGI359604, SGI359607, SGI359608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87155[] = {SGI359609, SGI359610, SGI359611, SGI359612, SGI359610, SGI359613, SGI359614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87156[] = {SGI359615, SGI359616, SGI359617, SGI359618, SGI359616, SGI359619, SGI359620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87157[] = {SGI359621, SGI359622, SGI359623, SGI359624, SGI359622, SGI359625, SGI359626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87158[] = {SGI359627, SGI359628, SGI359629, SGI359630, SGI359628, SGI359631, SGI359632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87159[] = {SGI359633, SGI359634, SGI359635, SGI359636, SGI359634, SGI359637, SGI359638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87160[] = {SGI359639, SGI359640, SGI359641, SGI359642, SGI359640, SGI359643, SGI359644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87161[] = {SGI359645, SGI359646, SGI359647, SGI359648, SGI359646, SGI359649, SGI359650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87162[] = {SGI359651, SGI359652, SGI359653, SGI359654, SGI359652, SGI359655, SGI359656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87163[] = {SGI359657, SGI359658, SGI359659, SGI359660, SGI359658, SGI359661, SGI359662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87164[] = {SGI359663, SGI359664, SGI359665, SGI359666, SGI359664, SGI359667, SGI359668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87165[] = {SGI359669, SGI359670, SGI359671, SGI359672, SGI359670, SGI359673, SGI359674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87166[] = {SGI359675, SGI359676, SGI359677, SGI359678, SGI359676, SGI359679, SGI359680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87167[] = {SGI359681, SGI359682, SGI359683, SGI359684, SGI359682, SGI359685, SGI359686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87168[] = {SGI359687, SGI359688, SGI359689, SGI359690, SGI359688, SGI359691, SGI359692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87169[] = {SGI359693, SGI359694, SGI359695, SGI359696, SGI359694, SGI359697, SGI359698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87170[] = {SGI359699, SGI359700, SGI359701, SGI359702, SGI359700, SGI359703, SGI359704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87171[] = {SGI359705, SGI359706, SGI359707, SGI359708, SGI359706, SGI359709, SGI359710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87172[] = {SGI359711, SGI359712, SGI359713, SGI359714, SGI0, SGI359715, SGI359716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87173[] = {SGI359717, SGI359718, SGI359719, SGI359720, SGI359718, SGI359721, SGI359722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87174[] = {SGI359723, SGI359724, SGI359725, SGI359726, SGI359724, SGI359727, SGI359728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87175[] = {SGI359729, SGI359730, SGI359731, SGI359732, SGI359730, SGI359733, SGI359734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87176[] = {SGI359735, SGI359736, SGI359737, SGI359738, SGI359736, SGI359739, SGI359740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87177[] = {SGI359741, SGI359742, SGI359743, SGI359744, SGI359742, SGI359745, SGI359746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87178[] = {SGI359747, SGI359748, SGI359749, SGI359750, SGI359748, SGI359751, SGI359752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87179[] = {SGI359753, SGI359754, SGI359755, SGI359756, SGI359754, SGI359757, SGI359758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87180[] = {SGI359759, SGI359760, SGI359761, SGI359762, SGI359760, SGI359763, SGI359764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87181[] = {SGI359765, SGI359766, SGI359767, SGI359768, SGI359766, SGI359769, SGI359770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87182[] = {SGI359771, SGI359772, SGI359773, SGI359774, SGI359772, SGI359775, SGI359776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87183[] = {SGI359777, SGI359778, SGI359779, SGI359780, SGI359778, SGI359781, SGI359782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87184[] = {SGI359783, SGI359784, SGI359785, SGI359786, SGI359784, SGI359787, SGI359788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87185[] = {SGI359789, SGI359790, SGI359791, SGI359792, SGI359790, SGI359793, SGI359794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87186[] = {SGI359795, SGI359796, SGI359797, SGI359798, SGI359796, SGI359799, SGI359800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87187[] = {SGI359801, SGI359802, SGI359803, SGI359804, SGI359802, SGI359805, SGI359806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87188[] = {SGI359807, SGI359808, SGI359809, SGI359810, SGI359808, SGI359811, SGI359812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87189[] = {SGI359813, SGI359814, SGI359815, SGI359816, SGI359814, SGI359817, SGI359818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87190[] = {SGI359819, SGI359820, SGI359821, SGI359822, SGI359820, SGI359823, SGI359824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87191[] = {SGI359825, SGI359826, SGI359827, SGI359828, SGI359826, SGI359829, SGI359830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87192[] = {SGI359831, SGI359832, SGI359833, SGI359834, SGI359832, SGI359835, SGI359836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87193[] = {SGI359837, SGI359838, SGI359839, SGI359840, SGI359838, SGI359841, SGI359842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87194[] = {SGI359843, SGI359844, SGI359845, SGI359846, SGI359844, SGI359847, SGI359848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87195[] = {SGI359849, SGI359850, SGI359851, SGI359852, SGI359850, SGI359853, SGI359854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87196[] = {SGI359855, SGI359856, SGI359857, SGI359858, SGI359856, SGI359859, SGI359860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87197[] = {SGI359861, SGI359862, SGI359863, SGI359864, SGI359862, SGI359865, SGI359866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87198[] = {SGI359867, SGI359868, SGI359869, SGI359870, SGI359868, SGI359871, SGI359872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87199[] = {SGI359873, SGI359874, SGI359875, SGI359876, SGI359874, SGI359877, SGI359878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87200[] = {SGI359879, SGI359880, SGI359881, SGI359882, SGI359880, SGI359883, SGI359884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87201[] = {SGI359885, SGI359886, SGI359887, SGI359888, SGI359886, SGI359889, SGI359890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87202[] = {SGI359891, SGI359892, SGI359893, SGI359894, SGI359892, SGI359895, SGI359896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87203[] = {SGI359897, SGI359898, SGI359899, SGI359900, SGI359898, SGI359901, SGI359902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87204[] = {SGI359903, SGI359904, SGI359905, SGI359906, SGI359904, SGI359907, SGI359908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87205[] = {SGI359909, SGI359910, SGI359911, SGI359912, SGI359910, SGI359913, SGI359914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87206[] = {SGI359915, SGI359916, SGI359917, SGI359918, SGI359916, SGI359919, SGI359920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87207[] = {SGI359921, SGI359922, SGI359923, SGI359924, SGI359922, SGI359925, SGI359926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87208[] = {SGI359927, SGI359928, SGI359929, SGI359930, SGI359928, SGI359931, SGI359932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87209[] = {SGI359933, SGI359934, SGI359935, SGI359936, SGI359934, SGI359937, SGI359938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87210[] = {SGI359939, SGI359940, SGI359941, SGI359942, SGI359940, SGI359943, SGI359944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87211[] = {SGI359945, SGI359946, SGI359947, SGI359948, SGI359946, SGI359949, SGI359950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87212[] = {SGI359951, SGI359952, SGI359953, SGI359954, SGI359952, SGI359955, SGI359956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87213[] = {SGI359957, SGI359958, SGI359959, SGI359960, SGI359958, SGI359961, SGI359962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87214[] = {SGI359963, SGI359964, SGI359965, SGI359966, SGI359964, SGI359967, SGI359968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87215[] = {SGI359969, SGI359970, SGI359971, SGI359972, SGI0, SGI359973, SGI359974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87216[] = {SGI359975, SGI359976, SGI359977, SGI359978, SGI359976, SGI359979, SGI359980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87217[] = {SGI359981, SGI359982, SGI359983, SGI359984, SGI359982, SGI359985, SGI359986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87218[] = {SGI359987, SGI359988, SGI359989, SGI359990, SGI359988, SGI359991, SGI359992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87219[] = {SGI359993, SGI359994, SGI359995, SGI359996, SGI359994, SGI359997, SGI359998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87220[] = {SGI359999, SGI360000, SGI360001, SGI360002, SGI360000, SGI360003, SGI360004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87221[] = {SGI360005, SGI360006, SGI360007, SGI360008, SGI360006, SGI360009, SGI360010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87222[] = {SGI360011, SGI360012, SGI360013, SGI360014, SGI360012, SGI360015, SGI360016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87223[] = {SGI360017, SGI360018, SGI360019, SGI360020, SGI360018, SGI360021, SGI360022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87224[] = {SGI360023, SGI360024, SGI360025, SGI360026, SGI360024, SGI360027, SGI360028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87225[] = {SGI360029, SGI360030, SGI360031, SGI360032, SGI360030, SGI360033, SGI360034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87226[] = {SGI360035, SGI360036, SGI360037, SGI360038, SGI360036, SGI360039, SGI360040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87227[] = {SGI360041, SGI360042, SGI360043, SGI360044, SGI360042, SGI360045, SGI360046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87228[] = {SGI360047, SGI360048, SGI360049, SGI360050, SGI360048, SGI360051, SGI360052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87229[] = {SGI360053, SGI360054, SGI360055, SGI360056, SGI360054, SGI360057, SGI360058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87230[] = {SGI360059, SGI360060, SGI360061, SGI360062, SGI360060, SGI360063, SGI360064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87231[] = {SGI360065, SGI360066, SGI360067, SGI360068, SGI360066, SGI360069, SGI360070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87232[] = {SGI360071, SGI360072, SGI360073, SGI360074, SGI360072, SGI360075, SGI360076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87233[] = {SGI360077, SGI360078, SGI360079, SGI360080, SGI360078, SGI360081, SGI360082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87234[] = {SGI360083, SGI360084, SGI360085, SGI360086, SGI360084, SGI360087, SGI360088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87235[] = {SGI360089, SGI360090, SGI360091, SGI360092, SGI360090, SGI360093, SGI360094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87236[] = {SGI360095, SGI360096, SGI360097, SGI360098, SGI360096, SGI360099, SGI360100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87237[] = {SGI360101, SGI360102, SGI360103, SGI360104, SGI360102, SGI360105, SGI360106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87238[] = {SGI360107, SGI360108, SGI360109, SGI360110, SGI360108, SGI360111, SGI360112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87239[] = {SGI360113, SGI360114, SGI360115, SGI360116, SGI360114, SGI360117, SGI360118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87240[] = {SGI360119, SGI360120, SGI360121, SGI360122, SGI360120, SGI360123, SGI360124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87241[] = {SGI360125, SGI360126, SGI360127, SGI360128, SGI360126, SGI360129, SGI360130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87242[] = {SGI360131, SGI360132, SGI360133, SGI360134, SGI360132, SGI360135, SGI360136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87243[] = {SGI360137, SGI360138, SGI360139, SGI360140, SGI360138, SGI360141, SGI360142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87244[] = {SGI360143, SGI360144, SGI360145, SGI360146, SGI360144, SGI360147, SGI360148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87245[] = {SGI360149, SGI360150, SGI360151, SGI360152, SGI360150, SGI360153, SGI360154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87246[] = {SGI360155, SGI360156, SGI360157, SGI360158, SGI360156, SGI360159, SGI360160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87247[] = {SGI360161, SGI360162, SGI360163, SGI360164, SGI360162, SGI360165, SGI360166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87248[] = {SGI360167, SGI360168, SGI360169, SGI360170, SGI360168, SGI360171, SGI360172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87249[] = {SGI360173, SGI360174, SGI360175, SGI360176, SGI360174, SGI360177, SGI360178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87250[] = {SGI360179, SGI360180, SGI360181, SGI360182, SGI360180, SGI360183, SGI360184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87251[] = {SGI360185, SGI360186, SGI360187, SGI360188, SGI360186, SGI360189, SGI360190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87252[] = {SGI360191, SGI360192, SGI360193, SGI360194, SGI360192, SGI360195, SGI360196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87253[] = {SGI360197, SGI360198, SGI360199, SGI360200, SGI360198, SGI360201, SGI360202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87254[] = {SGI360203, SGI360204, SGI360205, SGI360206, SGI360204, SGI360207, SGI360208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87255[] = {SGI360209, SGI360210, SGI360211, SGI360212, SGI360210, SGI360213, SGI360214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87256[] = {SGI360215, SGI360216, SGI360217, SGI360218, SGI360216, SGI360219, SGI360220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87257[] = {SGI360221, SGI360222, SGI360223, SGI360224, SGI360222, SGI360225, SGI360226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87258[] = {SGI360227, SGI360228, SGI360229, SGI360230, SGI360228, SGI360231, SGI360232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87259[] = {SGI360233, SGI360234, SGI360235, SGI360236, SGI360234, SGI360237, SGI360238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87260[] = {SGI360239, SGI360240, SGI360241, SGI360242, SGI360240, SGI360243, SGI360244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87261[] = {SGI360245, SGI360246, SGI360247, SGI360248, SGI360246, SGI360249, SGI360250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87262[] = {SGI360251, SGI360252, SGI360253, SGI360254, SGI360252, SGI360255, SGI360256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87263[] = {SGI360257, SGI360258, SGI360259, SGI360260, SGI360258, SGI360261, SGI360262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87264[] = {SGI360263, SGI360264, SGI360265, SGI360266, SGI360264, SGI360267, SGI360268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87265[] = {SGI360269, SGI360270, SGI360271, SGI360272, SGI360270, SGI360273, SGI360274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87266[] = {SGI360275, SGI360276, SGI360277, SGI360278, SGI360276, SGI360279, SGI360280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87267[] = {SGI360281, SGI360282, SGI360283, SGI360284, SGI360282, SGI360285, SGI360286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87268[] = {SGI360287, SGI360288, SGI360289, SGI360290, SGI360288, SGI360291, SGI360292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87269[] = {SGI360293, SGI360294, SGI360295, SGI360296, SGI360294, SGI360297, SGI360298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87270[] = {SGI360299, SGI360300, SGI360301, SGI360302, SGI360300, SGI360303, SGI360304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87271[] = {SGI360305, SGI360306, SGI360307, SGI360308, SGI360306, SGI360309, SGI360310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87272[] = {SGI360311, SGI360312, SGI360313, SGI360314, SGI360312, SGI360315, SGI360316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87273[] = {SGI360317, SGI360318, SGI360319, SGI360320, SGI360318, SGI360321, SGI360322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87274[] = {SGI360323, SGI360324, SGI360325, SGI360326, SGI360324, SGI360327, SGI360328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87275[] = {SGI360329, SGI360330, SGI360331, SGI360332, SGI360330, SGI360333, SGI360334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87276[] = {SGI360335, SGI360336, SGI360337, SGI360338, SGI360336, SGI360339, SGI360340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87277[] = {SGI360341, SGI360342, SGI360343, SGI360344, SGI360342, SGI360345, SGI360346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87278[] = {SGI360347, SGI360348, SGI360349, SGI360350, SGI360348, SGI360351, SGI360352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87279[] = {SGI360353, SGI360354, SGI360355, SGI360356, SGI360354, SGI360357, SGI360358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87280[] = {SGI360359, SGI360360, SGI360361, SGI360362, SGI360360, SGI360363, SGI360364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87281[] = {SGI360365, SGI360366, SGI360367, SGI360368, SGI0, SGI360369, SGI360370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87282[] = {SGI360371, SGI360372, SGI360373, SGI360374, SGI360372, SGI360375, SGI360376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87283[] = {SGI360377, SGI360378, SGI360379, SGI360380, SGI360378, SGI360381, SGI360382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87284[] = {SGI360383, SGI360384, SGI360385, SGI360386, SGI360384, SGI360387, SGI360388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87285[] = {SGI360389, SGI360390, SGI360391, SGI360392, SGI360390, SGI360393, SGI360394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87286[] = {SGI360395, SGI360396, SGI360397, SGI360398, SGI360396, SGI360399, SGI360400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87287[] = {SGI360401, SGI360402, SGI360403, SGI360404, SGI360402, SGI360405, SGI360406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87288[] = {SGI360407, SGI360408, SGI360409, SGI360410, SGI360408, SGI360411, SGI360412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87289[] = {SGI360413, SGI360414, SGI360415, SGI360416, SGI360414, SGI360417, SGI360418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87290[] = {SGI360419, SGI360420, SGI360421, SGI360422, SGI360420, SGI360423, SGI360424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87291[] = {SGI360425, SGI360426, SGI360427, SGI360428, SGI360426, SGI360429, SGI360430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87292[] = {SGI360431, SGI360432, SGI360433, SGI360434, SGI360432, SGI360435, SGI360436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87293[] = {SGI360437, SGI360438, SGI360439, SGI360440, SGI360438, SGI360441, SGI360442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87294[] = {SGI360443, SGI360444, SGI360445, SGI360446, SGI360444, SGI360447, SGI360448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87295[] = {SGI360449, SGI360450, SGI360451, SGI360452, SGI360450, SGI360453, SGI360454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87296[] = {SGI360455, SGI360456, SGI360457, SGI360458, SGI360456, SGI360459, SGI360460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87297[] = {SGI360461, SGI360462, SGI360463, SGI360464, SGI360462, SGI360465, SGI360466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87298[] = {SGI360467, SGI360468, SGI360469, SGI360470, SGI360468, SGI360471, SGI360472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87299[] = {SGI360473, SGI360474, SGI360475, SGI360476, SGI360474, SGI360477, SGI360478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87300[] = {SGI360479, SGI360480, SGI360481, SGI360482, SGI360480, SGI360483, SGI360484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87301[] = {SGI360485, SGI360486, SGI360487, SGI360488, SGI360486, SGI360489, SGI360490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87302[] = {SGI360491, SGI360492, SGI360493, SGI360494, SGI360492, SGI360495, SGI360496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87303[] = {SGI360497, SGI360498, SGI360499, SGI360500, SGI360498, SGI360501, SGI360502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87304[] = {SGI360503, SGI360504, SGI360505, SGI360506, SGI360504, SGI360507, SGI360508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87305[] = {SGI360509, SGI360510, SGI360511, SGI360512, SGI360510, SGI360513, SGI360514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87306[] = {SGI360515, SGI360516, SGI360517, SGI360518, SGI360516, SGI360519, SGI360520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87307[] = {SGI360521, SGI360522, SGI360523, SGI360524, SGI360522, SGI360525, SGI360526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87308[] = {SGI360527, SGI360528, SGI360529, SGI360530, SGI360528, SGI360531, SGI360532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87309[] = {SGI360533, SGI360534, SGI360535, SGI360536, SGI360534, SGI360537, SGI360538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87310[] = {SGI360539, SGI360540, SGI360541, SGI360542, SGI360540, SGI360543, SGI360544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87311[] = {SGI360545, SGI360546, SGI360547, SGI360548, SGI0, SGI360549, SGI360550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87312[] = {SGI360551, SGI360552, SGI360553, SGI360554, SGI360552, SGI360555, SGI360556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87313[] = {SGI360557, SGI360558, SGI360559, SGI360560, SGI360558, SGI360561, SGI360562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87314[] = {SGI360563, SGI360564, SGI360565, SGI360566, SGI360564, SGI360567, SGI360568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87315[] = {SGI360569, SGI360570, SGI360571, SGI360572, SGI360570, SGI360573, SGI360574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87316[] = {SGI360575, SGI360576, SGI360577, SGI360578, SGI360576, SGI360579, SGI360580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87317[] = {SGI360581, SGI360582, SGI360583, SGI360584, SGI360582, SGI360585, SGI360586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87318[] = {SGI360587, SGI360588, SGI360589, SGI360590, SGI360588, SGI360591, SGI360592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87319[] = {SGI360593, SGI360594, SGI360595, SGI360596, SGI360594, SGI360597, SGI360598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87320[] = {SGI360599, SGI360600, SGI360601, SGI360602, SGI360600, SGI360603, SGI360604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87321[] = {SGI360605, SGI360606, SGI360607, SGI360608, SGI360606, SGI360609, SGI360610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87322[] = {SGI360611, SGI360612, SGI360613, SGI360614, SGI360612, SGI360615, SGI360616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87323[] = {SGI360617, SGI360618, SGI360619, SGI360620, SGI360618, SGI360621, SGI360622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87324[] = {SGI360623, SGI360624, SGI360625, SGI360626, SGI360624, SGI360627, SGI360628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87325[] = {SGI360629, SGI360630, SGI360631, SGI360632, SGI360630, SGI360633, SGI360634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87326[] = {SGI360635, SGI360636, SGI360637, SGI360638, SGI360636, SGI360639, SGI360640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87327[] = {SGI360641, SGI360642, SGI360643, SGI360644, SGI360642, SGI360645, SGI360646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87328[] = {SGI360647, SGI360648, SGI360649, SGI360650, SGI360648, SGI360651, SGI360652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87329[] = {SGI360653, SGI360654, SGI360655, SGI360656, SGI360654, SGI360657, SGI360658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87330[] = {SGI360659, SGI360660, SGI360661, SGI360662, SGI360660, SGI360663, SGI360664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87331[] = {SGI360665, SGI360666, SGI360667, SGI360668, SGI360666, SGI360669, SGI360670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87332[] = {SGI360671, SGI360672, SGI360673, SGI360674, SGI360672, SGI360675, SGI360676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87333[] = {SGI360677, SGI360678, SGI360679, SGI360680, SGI360678, SGI360681, SGI360682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87334[] = {SGI360683, SGI360684, SGI360685, SGI360686, SGI360684, SGI360687, SGI360688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87335[] = {SGI360689, SGI360690, SGI360691, SGI360692, SGI360690, SGI360693, SGI360694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87336[] = {SGI360695, SGI360696, SGI360697, SGI360698, SGI360696, SGI360699, SGI360700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87337[] = {SGI360701, SGI360702, SGI360703, SGI360704, SGI360702, SGI360705, SGI360706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87338[] = {SGI360707, SGI360708, SGI360709, SGI360710, SGI360708, SGI360711, SGI360712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87339[] = {SGI360713, SGI360714, SGI360715, SGI360716, SGI360714, SGI360717, SGI360718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87340[] = {SGI360719, SGI360720, SGI360721, SGI360722, SGI360720, SGI360723, SGI360724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87341[] = {SGI360725, SGI360726, SGI360727, SGI360728, SGI360726, SGI360729, SGI360730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87342[] = {SGI360731, SGI360732, SGI360733, SGI360734, SGI360732, SGI360735, SGI360736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87343[] = {SGI360737, SGI360738, SGI360739, SGI360740, SGI360738, SGI360741, SGI360742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87344[] = {SGI360743, SGI360744, SGI360745, SGI360746, SGI360744, SGI360747, SGI360748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87345[] = {SGI360749, SGI360750, SGI360751, SGI360752, SGI360750, SGI360753, SGI360754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87346[] = {SGI360755, SGI360756, SGI360757, SGI360758, SGI360756, SGI360759, SGI360760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87347[] = {SGI360761, SGI360762, SGI360763, SGI360764, SGI360762, SGI360765, SGI360766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87348[] = {SGI360767, SGI360768, SGI360769, SGI360770, SGI360768, SGI360771, SGI360772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87349[] = {SGI360773, SGI360774, SGI360775, SGI360776, SGI360774, SGI360777, SGI360778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87350[] = {SGI360779, SGI360780, SGI360781, SGI360782, SGI360780, SGI360783, SGI360784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87351[] = {SGI360785, SGI360786, SGI360787, SGI360788, SGI360786, SGI360789, SGI360790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87352[] = {SGI360791, SGI360792, SGI360793, SGI360794, SGI360792, SGI360795, SGI360796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87353[] = {SGI360797, SGI360798, SGI360799, SGI360800, SGI360798, SGI360801, SGI360802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87354[] = {SGI360803, SGI360804, SGI360805, SGI360806, SGI360804, SGI360807, SGI360808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87355[] = {SGI360809, SGI360810, SGI360811, SGI360812, SGI360810, SGI360813, SGI360814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87356[] = {SGI360815, SGI360816, SGI360817, SGI360818, SGI360816, SGI360819, SGI360820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87357[] = {SGI360821, SGI360822, SGI360823, SGI360824, SGI360822, SGI360825, SGI360826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87358[] = {SGI360827, SGI360828, SGI360829, SGI360830, SGI360828, SGI360831, SGI360832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87359[] = {SGI360833, SGI360834, SGI360835, SGI360836, SGI360834, SGI360837, SGI360838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87360[] = {SGI360839, SGI360840, SGI360841, SGI360842, SGI360840, SGI360843, SGI360844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87361[] = {SGI360845, SGI360846, SGI360847, SGI360848, SGI360846, SGI360849, SGI360850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87362[] = {SGI360851, SGI360852, SGI360853, SGI360854, SGI360852, SGI360855, SGI360856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87363[] = {SGI360857, SGI360858, SGI360859, SGI360860, SGI360858, SGI360861, SGI360862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87364[] = {SGI360863, SGI360864, SGI360865, SGI360866, SGI360864, SGI360867, SGI360868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87365[] = {SGI360869, SGI360870, SGI360871, SGI360872, SGI360870, SGI360873, SGI360874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87366[] = {SGI360875, SGI360876, SGI360877, SGI360878, SGI360876, SGI360879, SGI360880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87367[] = {SGI360881, SGI360882, SGI360883, SGI360884, SGI360882, SGI360885, SGI360886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87368[] = {SGI360887, SGI360888, SGI360889, SGI360890, SGI360888, SGI360891, SGI360892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87369[] = {SGI360893, SGI360894, SGI360895, SGI360896, SGI360894, SGI360897, SGI360898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87370[] = {SGI360899, SGI360900, SGI360901, SGI360902, SGI360900, SGI360903, SGI360904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87371[] = {SGI360905, SGI360906, SGI360907, SGI360908, SGI360906, SGI360909, SGI360910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87372[] = {SGI360911, SGI360912, SGI360913, SGI360914, SGI360912, SGI360915, SGI360916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87373[] = {SGI360917, SGI360918, SGI360919, SGI360920, SGI360918, SGI360921, SGI360922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87374[] = {SGI360923, SGI360924, SGI360925, SGI360926, SGI0, SGI360927, SGI360928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87375[] = {SGI360929, SGI360930, SGI360931, SGI360932, SGI360930, SGI360933, SGI360934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87376[] = {SGI360935, SGI360936, SGI360937, SGI360938, SGI360936, SGI360939, SGI360940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87377[] = {SGI360941, SGI360942, SGI360943, SGI360944, SGI360942, SGI360945, SGI360946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87378[] = {SGI360947, SGI360948, SGI360949, SGI360950, SGI360948, SGI360951, SGI360952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87379[] = {SGI360953, SGI360954, SGI360955, SGI360956, SGI360954, SGI360957, SGI360958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87380[] = {SGI360959, SGI360960, SGI360961, SGI360962, SGI360960, SGI360963, SGI360964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87381[] = {SGI360965, SGI360966, SGI360967, SGI360968, SGI360966, SGI360969, SGI360970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87382[] = {SGI360971, SGI360972, SGI360973, SGI360974, SGI360972, SGI360975, SGI360976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87383[] = {SGI360977, SGI360978, SGI360979, SGI360980, SGI360978, SGI360981, SGI360982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87384[] = {SGI360983, SGI360984, SGI360985, SGI360986, SGI360984, SGI360987, SGI360988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87385[] = {SGI360989, SGI360990, SGI360991, SGI360992, SGI360990, SGI360993, SGI360994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87386[] = {SGI360995, SGI360996, SGI360997, SGI360998, SGI360996, SGI360999, SGI361000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87387[] = {SGI361001, SGI361002, SGI361003, SGI361004, SGI361002, SGI361005, SGI361006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87388[] = {SGI361007, SGI361008, SGI361009, SGI361010, SGI361008, SGI361011, SGI361012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87389[] = {SGI361013, SGI361014, SGI361015, SGI361016, SGI361014, SGI361017, SGI361018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87390[] = {SGI361019, SGI361020, SGI361021, SGI361022, SGI361020, SGI361023, SGI361024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87391[] = {SGI361025, SGI361026, SGI361027, SGI361028, SGI361026, SGI361029, SGI361030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87392[] = {SGI361031, SGI361032, SGI361033, SGI361034, SGI361032, SGI361035, SGI361036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87393[] = {SGI361037, SGI361038, SGI361039, SGI361040, SGI361038, SGI361041, SGI361042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87394[] = {SGI361043, SGI361044, SGI361045, SGI361046, SGI361044, SGI361047, SGI361048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87395[] = {SGI361049, SGI361050, SGI361051, SGI361052, SGI361050, SGI361053, SGI361054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87396[] = {SGI361055, SGI361056, SGI361057, SGI361058, SGI361056, SGI361059, SGI361060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87397[] = {SGI361061, SGI361062, SGI361063, SGI361064, SGI361062, SGI361065, SGI361066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87398[] = {SGI361067, SGI361068, SGI361069, SGI361070, SGI361068, SGI361071, SGI361072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87399[] = {SGI361073, SGI361074, SGI361075, SGI361076, SGI361074, SGI361077, SGI361078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87400[] = {SGI361079, SGI361080, SGI361081, SGI361082, SGI361080, SGI361083, SGI361084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87401[] = {SGI361085, SGI361086, SGI361087, SGI361088, SGI361086, SGI361089, SGI361090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87402[] = {SGI361091, SGI361092, SGI361093, SGI361094, SGI361092, SGI361095, SGI361096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87403[] = {SGI361097, SGI361098, SGI361099, SGI361100, SGI361098, SGI361101, SGI361102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87404[] = {SGI361103, SGI361104, SGI361105, SGI361106, SGI361104, SGI361107, SGI361108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87405[] = {SGI361109, SGI361110, SGI361111, SGI361112, SGI361110, SGI361113, SGI361114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87406[] = {SGI361115, SGI361116, SGI361117, SGI361118, SGI361116, SGI361119, SGI361120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87407[] = {SGI361121, SGI361122, SGI361123, SGI361124, SGI361122, SGI361125, SGI361126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87408[] = {SGI361127, SGI361128, SGI361129, SGI361130, SGI361128, SGI361131, SGI361132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87409[] = {SGI361133, SGI361134, SGI361135, SGI361136, SGI361134, SGI361137, SGI361138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87410[] = {SGI361139, SGI361140, SGI361141, SGI361142, SGI361140, SGI361143, SGI361144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87411[] = {SGI361145, SGI361146, SGI361147, SGI361148, SGI361146, SGI361149, SGI361150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87412[] = {SGI361151, SGI361152, SGI361153, SGI361154, SGI361152, SGI361155, SGI361156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87413[] = {SGI361157, SGI361158, SGI361159, SGI361160, SGI361158, SGI361161, SGI361162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87414[] = {SGI361163, SGI361164, SGI361165, SGI361166, SGI361164, SGI361167, SGI361168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87415[] = {SGI361169, SGI361170, SGI361171, SGI361172, SGI361170, SGI361173, SGI361174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87416[] = {SGI361175, SGI361176, SGI361177, SGI361178, SGI361176, SGI361179, SGI361180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87417[] = {SGI361181, SGI361182, SGI361183, SGI361184, SGI361182, SGI361185, SGI361186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87418[] = {SGI361187, SGI361188, SGI361189, SGI361190, SGI361188, SGI361191, SGI361192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87419[] = {SGI361193, SGI361194, SGI361195, SGI361196, SGI361194, SGI361197, SGI361198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87420[] = {SGI361199, SGI361200, SGI361201, SGI361202, SGI361200, SGI361203, SGI361204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87421[] = {SGI361205, SGI361206, SGI361207, SGI361208, SGI361206, SGI361209, SGI361210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87422[] = {SGI361211, SGI361212, SGI361213, SGI361214, SGI361212, SGI361215, SGI361216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87423[] = {SGI361217, SGI361218, SGI361219, SGI361220, SGI361218, SGI361221, SGI361222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87424[] = {SGI361223, SGI361224, SGI361225, SGI361226, SGI361224, SGI361227, SGI361228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87425[] = {SGI361229, SGI361230, SGI361231, SGI361232, SGI361230, SGI361233, SGI361234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87426[] = {SGI361235, SGI361236, SGI361237, SGI361238, SGI361236, SGI361239, SGI361240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87427[] = {SGI361241, SGI361242, SGI361243, SGI361244, SGI361242, SGI361245, SGI361246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87428[] = {SGI361247, SGI361248, SGI361249, SGI361250, SGI361248, SGI361251, SGI361252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87429[] = {SGI361253, SGI361254, SGI361255, SGI361256, SGI361254, SGI361257, SGI361258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87430[] = {SGI361259, SGI361260, SGI361261, SGI361262, SGI361260, SGI361263, SGI361264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87431[] = {SGI361265, SGI361266, SGI361267, SGI361268, SGI361266, SGI361269, SGI361270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87432[] = {SGI361271, SGI361273, SGI361275, SGI361273, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87433[] = {SGI361272, SGI361274, SGI361276, SGI361277, SGI361274, SGI361278, SGI361279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87434[] = {SGI361280, SGI361281, SGI361282, SGI361281, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87435[] = {SGI361283, SGI361284, SGI361285, SGI361286, SGI361284, SGI361287, SGI361288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87436[] = {SGI361289, SGI361290, SGI361291, SGI361292, SGI361290, SGI361293, SGI361294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87437[] = {SGI361295, SGI361296, SGI361297, SGI361298, SGI361296, SGI361299, SGI361300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87438[] = {SGI361301, SGI361302, SGI361303, SGI361304, SGI361302, SGI361305, SGI361306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87439[] = {SGI361307, SGI361308, SGI361309, SGI361310, SGI361308, SGI361311, SGI361312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87440[] = {SGI361313, SGI361314, SGI361315, SGI361316, SGI361314, SGI361317, SGI361318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87441[] = {SGI361319, SGI361320, SGI361321, SGI361322, SGI361320, SGI361323, SGI361324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87442[] = {SGI361325, SGI361326, SGI361327, SGI361326, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87443[] = {SGI361328, SGI361329, SGI361330, SGI361331, SGI361329, SGI361332, SGI361333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87444[] = {SGI361334, SGI361335, SGI361336, SGI361335, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87445[] = {SGI361337, SGI361338, SGI361339, SGI361340, SGI361338, SGI361341, SGI361342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87446[] = {SGI361343, SGI361344, SGI361345, SGI361344, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87447[] = {SGI361346, SGI361347, SGI361348, SGI361349, SGI0, SGI361350, SGI361351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87448[] = {SGI361352, SGI361353, SGI361354, SGI361353, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87449[] = {SGI361355, SGI361356, SGI361357, SGI361358, SGI361356, SGI361359, SGI361360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87450[] = {SGI361361, SGI361362, SGI361363, SGI361362, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87451[] = {SGI361364, SGI361366, SGI361368, SGI361366, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87452[] = {SGI361365, SGI361367, SGI361369, SGI361370, SGI361367, SGI361371, SGI361372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87453[] = {SGI361373, SGI361374, SGI361375, SGI361376, SGI361374, SGI361377, SGI361378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87454[] = {SGI361379, SGI361380, SGI361381, SGI361382, SGI361380, SGI361383, SGI361384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87455[] = {SGI361385, SGI361386, SGI361387, SGI361388, SGI361386, SGI361389, SGI361390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87456[] = {SGI361391, SGI361392, SGI361393, SGI361394, SGI361392, SGI361395, SGI361396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87457[] = {SGI361397, SGI361398, SGI361399, SGI361400, SGI361398, SGI361401, SGI361402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87458[] = {SGI361403, SGI361404, SGI361405, SGI361406, SGI361404, SGI361407, SGI361408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87459[] = {SGI361409, SGI361410, SGI361411, SGI361412, SGI361410, SGI361413, SGI361414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87460[] = {SGI361415, SGI361416, SGI361417, SGI361418, SGI361416, SGI361419, SGI361420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87461[] = {SGI361421, SGI361422, SGI361423, SGI361424, SGI361422, SGI361425, SGI361426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87462[] = {SGI361427, SGI361428, SGI361429, SGI361430, SGI361428, SGI361431, SGI361432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87463[] = {SGI361433, SGI361434, SGI361435, SGI361434, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87464[] = {SGI361436, SGI361437, SGI361438, SGI361437, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87465[] = {SGI361439, SGI361440, SGI361441, SGI361440, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87466[] = {SGI361442, SGI361443, SGI361444, SGI361443, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87467[] = {SGI361445, SGI361446, SGI361447, SGI361446, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87468[] = {SGI361448, SGI361449, SGI361450, SGI361449, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87469[] = {SGI361451, SGI361452, SGI361453, SGI361452, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87470[] = {SGI361454, SGI361455, SGI361456, SGI361455, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87471[] = {SGI361457, SGI361458, SGI361459, SGI361458, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87472[] = {SGI361460, SGI361461, SGI361462, SGI361461, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87473[] = {SGI361463, SGI361464, SGI361465, SGI361464, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87474[] = {SGI361466, SGI361467, SGI361468, SGI361469, SGI361467, SGI361470, SGI361471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87475[] = {SGI361472, SGI361473, SGI361474, SGI361475, SGI361473, SGI361476, SGI361477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87476[] = {SGI361478, SGI361479, SGI361480, SGI361481, SGI361479, SGI361482, SGI361483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87477[] = {SGI361484, SGI361485, SGI361486, SGI361487, SGI361485, SGI361488, SGI361489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87478[] = {SGI361490, SGI361491, SGI361492, SGI361493, SGI361491, SGI361494, SGI361495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87479[] = {SGI361496, SGI361497, SGI361498, SGI361499, SGI361497, SGI361500, SGI361501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87480[] = {SGI361502, SGI361503, SGI361504, SGI361505, SGI361503, SGI361506, SGI361507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87481[] = {SGI361508, SGI361509, SGI361510, SGI361511, SGI361509, SGI361512, SGI361513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87482[] = {SGI361514, SGI361515, SGI361516, SGI361517, SGI361515, SGI361518, SGI361519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87483[] = {SGI361520, SGI361521, SGI361522, SGI361523, SGI361521, SGI361524, SGI361525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87484[] = {SGI361526, SGI361527, SGI361528, SGI361529, SGI361527, SGI361530, SGI361531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87485[] = {SGI361532, SGI361533, SGI361534, SGI361535, SGI361533, SGI361536, SGI361537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87486[] = {SGI361538, SGI361539, SGI361540, SGI361541, SGI361539, SGI361542, SGI361543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87487[] = {SGI361544, SGI361545, SGI361546, SGI361547, SGI361545, SGI361548, SGI361549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87488[] = {SGI361550, SGI361551, SGI361552, SGI361553, SGI361551, SGI361554, SGI361555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87489[] = {SGI361556, SGI361557, SGI0, SGI361557, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87490[] = {SGI361558, SGI361559, SGI361560, SGI361561, SGI361559, SGI361562, SGI361563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87491[] = {SGI361564, SGI361565, SGI361566, SGI361567, SGI361565, SGI361568, SGI361569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87492[] = {SGI361570, SGI361571, SGI361572, SGI361573, SGI361571, SGI361574, SGI361575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87493[] = {SGI361576, SGI361577, SGI361578, SGI361579, SGI361577, SGI361580, SGI361581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87494[] = {SGI361582, SGI361583, SGI361584, SGI361585, SGI361583, SGI361586, SGI361587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87495[] = {SGI361588, SGI361589, SGI361590, SGI361591, SGI361589, SGI361592, SGI361593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87496[] = {SGI361594, SGI361595, SGI361596, SGI361597, SGI361595, SGI361598, SGI361599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87497[] = {SGI361600, SGI361601, SGI361602, SGI361603, SGI361601, SGI361604, SGI361605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87498[] = {SGI361606, SGI361607, SGI361608, SGI361609, SGI361607, SGI361610, SGI361611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87499[] = {SGI361612, SGI361613, SGI361614, SGI361615, SGI361613, SGI361616, SGI361617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87500[] = {SGI361618, SGI361619, SGI361620, SGI361621, SGI361619, SGI361622, SGI361623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87501[] = {SGI361624, SGI361625, SGI361626, SGI361627, SGI361625, SGI361628, SGI361629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87502[] = {SGI361630, SGI361631, SGI361632, SGI361633, SGI361631, SGI361634, SGI361635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87503[] = {SGI361636, SGI361637, SGI361638, SGI361639, SGI361637, SGI361640, SGI361641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87504[] = {SGI361642, SGI361643, SGI361644, SGI361645, SGI361643, SGI361646, SGI361647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87505[] = {SGI361648, SGI361649, SGI361650, SGI361651, SGI361649, SGI361652, SGI361653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87506[] = {SGI361654, SGI361655, SGI361656, SGI361657, SGI361655, SGI361658, SGI361659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87507[] = {SGI361660, SGI361661, SGI361662, SGI361663, SGI361661, SGI361664, SGI361665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87508[] = {SGI361666, SGI361667, SGI361668, SGI361669, SGI361667, SGI361670, SGI361671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87509[] = {SGI361672, SGI361673, SGI361674, SGI361675, SGI361673, SGI361676, SGI361677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87510[] = {SGI361678, SGI361679, SGI361680, SGI361681, SGI361679, SGI361682, SGI361683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87511[] = {SGI361684, SGI361685, SGI361686, SGI361687, SGI361685, SGI361688, SGI361689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87512[] = {SGI361690, SGI361691, SGI361692, SGI361693, SGI361691, SGI361694, SGI361695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87513[] = {SGI361696, SGI361697, SGI361698, SGI361699, SGI361697, SGI361700, SGI361701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87514[] = {SGI361702, SGI361703, SGI361704, SGI361705, SGI361703, SGI361706, SGI361707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87515[] = {SGI361708, SGI361709, SGI361710, SGI361711, SGI361709, SGI361712, SGI361713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87516[] = {SGI361714, SGI361715, SGI361716, SGI361717, SGI0, SGI361718, SGI361719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87517[] = {SGI361720, SGI361721, SGI361722, SGI361723, SGI361721, SGI361724, SGI361725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87518[] = {SGI361726, SGI361727, SGI361728, SGI361729, SGI361727, SGI361730, SGI361731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87519[] = {SGI361732, SGI361733, SGI361734, SGI361735, SGI361733, SGI361736, SGI361737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87520[] = {SGI361738, SGI361739, SGI361740, SGI361741, SGI361739, SGI361742, SGI361743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87521[] = {SGI361744, SGI361745, SGI361746, SGI361747, SGI361745, SGI361748, SGI361749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87522[] = {SGI361750, SGI361751, SGI361752, SGI361753, SGI361751, SGI361754, SGI361755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87523[] = {SGI361756, SGI361757, SGI361758, SGI361759, SGI361757, SGI361760, SGI361761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87524[] = {SGI361762, SGI361763, SGI361764, SGI361765, SGI361763, SGI361766, SGI361767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87525[] = {SGI361768, SGI361769, SGI361770, SGI361771, SGI361769, SGI361772, SGI361773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87526[] = {SGI361774, SGI361775, SGI361776, SGI361777, SGI361775, SGI361778, SGI361779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87527[] = {SGI361780, SGI361781, SGI361782, SGI361783, SGI361781, SGI361784, SGI361785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87528[] = {SGI361786, SGI361787, SGI361788, SGI361789, SGI361787, SGI361790, SGI361791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87529[] = {SGI361792, SGI361793, SGI361794, SGI361795, SGI361793, SGI361796, SGI361797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87530[] = {SGI361798, SGI361799, SGI361800, SGI361801, SGI361799, SGI361802, SGI361803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87531[] = {SGI361804, SGI361805, SGI361806, SGI361807, SGI361805, SGI361808, SGI361809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87532[] = {SGI361810, SGI361811, SGI361812, SGI361813, SGI361811, SGI361814, SGI361815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87533[] = {SGI361816, SGI361817, SGI361818, SGI361819, SGI361817, SGI361820, SGI361821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87534[] = {SGI361822, SGI361823, SGI361824, SGI361825, SGI361823, SGI361826, SGI361827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87535[] = {SGI361828, SGI361829, SGI361830, SGI361831, SGI361829, SGI361832, SGI361833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87536[] = {SGI361834, SGI361835, SGI361836, SGI361837, SGI361835, SGI361838, SGI361839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87537[] = {SGI361840, SGI361841, SGI361842, SGI361843, SGI361841, SGI361844, SGI361845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87538[] = {SGI361846, SGI361847, SGI361848, SGI361849, SGI361847, SGI361850, SGI361851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87539[] = {SGI361852, SGI361853, SGI361854, SGI361855, SGI361853, SGI361856, SGI361857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87540[] = {SGI361858, SGI361859, SGI361860, SGI361861, SGI361859, SGI361862, SGI361863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87541[] = {SGI361864, SGI361865, SGI361866, SGI361867, SGI361865, SGI361868, SGI361869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87542[] = {SGI361870, SGI361871, SGI361872, SGI361873, SGI361871, SGI361874, SGI361875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87543[] = {SGI361876, SGI361877, SGI361878, SGI361879, SGI361877, SGI361880, SGI361881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87544[] = {SGI361882, SGI361883, SGI361884, SGI361885, SGI361883, SGI361886, SGI361887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87545[] = {SGI361888, SGI361889, SGI361890, SGI361891, SGI361889, SGI361892, SGI361893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87546[] = {SGI361894, SGI361895, SGI361896, SGI361897, SGI361895, SGI361898, SGI361899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87547[] = {SGI361900, SGI361901, SGI361902, SGI361903, SGI361901, SGI361904, SGI361905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87548[] = {SGI361906, SGI361907, SGI361908, SGI361909, SGI361907, SGI361910, SGI361911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87549[] = {SGI361912, SGI361913, SGI361914, SGI361915, SGI361913, SGI361916, SGI361917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87550[] = {SGI361918, SGI361919, SGI361920, SGI361921, SGI361919, SGI361922, SGI361923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87551[] = {SGI361924, SGI361925, SGI361926, SGI361927, SGI361925, SGI361928, SGI361929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87552[] = {SGI361930, SGI361931, SGI361932, SGI361933, SGI361931, SGI361934, SGI361935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87553[] = {SGI361936, SGI361937, SGI361938, SGI361939, SGI361937, SGI361940, SGI361941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87554[] = {SGI361942, SGI361943, SGI361944, SGI361945, SGI361943, SGI361946, SGI361947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87555[] = {SGI361948, SGI361949, SGI361950, SGI361951, SGI0, SGI361952, SGI361953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87556[] = {SGI361954, SGI361955, SGI361956, SGI361957, SGI361955, SGI361958, SGI361959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87557[] = {SGI361960, SGI361961, SGI361962, SGI361963, SGI361961, SGI361964, SGI361965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87558[] = {SGI361966, SGI361967, SGI361968, SGI361969, SGI361967, SGI361970, SGI361971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87559[] = {SGI361972, SGI361973, SGI361974, SGI361975, SGI361973, SGI361976, SGI361977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87560[] = {SGI361978, SGI361979, SGI361980, SGI361981, SGI361979, SGI361982, SGI361983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87561[] = {SGI361984, SGI361985, SGI361986, SGI361987, SGI361985, SGI361988, SGI361989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87562[] = {SGI361990, SGI361991, SGI361992, SGI361993, SGI361991, SGI361994, SGI361995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87563[] = {SGI361996, SGI361997, SGI361998, SGI361999, SGI361997, SGI362000, SGI362001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87564[] = {SGI362002, SGI362003, SGI362004, SGI362005, SGI362003, SGI362006, SGI362007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87565[] = {SGI362008, SGI362009, SGI362010, SGI362011, SGI362009, SGI362012, SGI362013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87566[] = {SGI362014, SGI362015, SGI362016, SGI362017, SGI362015, SGI362018, SGI362019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87567[] = {SGI362020, SGI362021, SGI362022, SGI362023, SGI362021, SGI362024, SGI362025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87568[] = {SGI362026, SGI362027, SGI362028, SGI362029, SGI362027, SGI362030, SGI362031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87569[] = {SGI362032, SGI362033, SGI362034, SGI362035, SGI362033, SGI362036, SGI362037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87570[] = {SGI362038, SGI362039, SGI362040, SGI362041, SGI362039, SGI362042, SGI362043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87571[] = {SGI362044, SGI362045, SGI362046, SGI362047, SGI362045, SGI362048, SGI362049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87572[] = {SGI362050, SGI362051, SGI362052, SGI362053, SGI362051, SGI362054, SGI362055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87573[] = {SGI362056, SGI362057, SGI362058, SGI362059, SGI362057, SGI362060, SGI362061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87574[] = {SGI362062, SGI362063, SGI362064, SGI362065, SGI362063, SGI362066, SGI362067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87575[] = {SGI362068, SGI362069, SGI362070, SGI362071, SGI362069, SGI362072, SGI362073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87576[] = {SGI362074, SGI362075, SGI362076, SGI362077, SGI362075, SGI362078, SGI362079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87577[] = {SGI362080, SGI362081, SGI362082, SGI362083, SGI362081, SGI362084, SGI362085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87578[] = {SGI362086, SGI362087, SGI362088, SGI362089, SGI362087, SGI362090, SGI362091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87579[] = {SGI362092, SGI362093, SGI362094, SGI362095, SGI362093, SGI362096, SGI362097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87580[] = {SGI362098, SGI362099, SGI362100, SGI362101, SGI362099, SGI362102, SGI362103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87581[] = {SGI362104, SGI362105, SGI362106, SGI362107, SGI362105, SGI362108, SGI362109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87582[] = {SGI362110, SGI362111, SGI362112, SGI362113, SGI362111, SGI362114, SGI362115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87583[] = {SGI362116, SGI362117, SGI362118, SGI362119, SGI362117, SGI362120, SGI362121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87584[] = {SGI362122, SGI362123, SGI362124, SGI362125, SGI362123, SGI362126, SGI362127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87585[] = {SGI362128, SGI362129, SGI362130, SGI362131, SGI362129, SGI362132, SGI362133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87586[] = {SGI362134, SGI362135, SGI362136, SGI362137, SGI362135, SGI362138, SGI362139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87587[] = {SGI362140, SGI362141, SGI362142, SGI362143, SGI362141, SGI362144, SGI362145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87588[] = {SGI362146, SGI362147, SGI362148, SGI362149, SGI362147, SGI362150, SGI362151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87589[] = {SGI362152, SGI362153, SGI362154, SGI362155, SGI362153, SGI362156, SGI362157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87590[] = {SGI362158, SGI362159, SGI362160, SGI362161, SGI362159, SGI362162, SGI362163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87591[] = {SGI362164, SGI362165, SGI362166, SGI362167, SGI362165, SGI362168, SGI362169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87592[] = {SGI362170, SGI362171, SGI362172, SGI362173, SGI362171, SGI362174, SGI362175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87593[] = {SGI362176, SGI362177, SGI362178, SGI362179, SGI362177, SGI362180, SGI362181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87594[] = {SGI362182, SGI362183, SGI362184, SGI362185, SGI362183, SGI362186, SGI362187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87595[] = {SGI362188, SGI362189, SGI362190, SGI362191, SGI362189, SGI362192, SGI362193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87596[] = {SGI362194, SGI362195, SGI362196, SGI362197, SGI362195, SGI362198, SGI362199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87597[] = {SGI362200, SGI362201, SGI362202, SGI362203, SGI362201, SGI362204, SGI362205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87598[] = {SGI362206, SGI362207, SGI362208, SGI362209, SGI362207, SGI362210, SGI362211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87599[] = {SGI362212, SGI362213, SGI362214, SGI362215, SGI362213, SGI362216, SGI362217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87600[] = {SGI362218, SGI362219, SGI362220, SGI362221, SGI362219, SGI362222, SGI362223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87601[] = {SGI362224, SGI362225, SGI362226, SGI362227, SGI362225, SGI362228, SGI362229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87602[] = {SGI362230, SGI362231, SGI362232, SGI362233, SGI362231, SGI362234, SGI362235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87603[] = {SGI362236, SGI362237, SGI362238, SGI362239, SGI362237, SGI362240, SGI362241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87604[] = {SGI362242, SGI362243, SGI362244, SGI362245, SGI362243, SGI362246, SGI362247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87605[] = {SGI362248, SGI362249, SGI362250, SGI362251, SGI362249, SGI362252, SGI362253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87606[] = {SGI362254, SGI362255, SGI362256, SGI362257, SGI362255, SGI362258, SGI362259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87607[] = {SGI362260, SGI362261, SGI362262, SGI362263, SGI362261, SGI362264, SGI362265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87608[] = {SGI362266, SGI362267, SGI362268, SGI362269, SGI362267, SGI362270, SGI362271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87609[] = {SGI362272, SGI362273, SGI362274, SGI362275, SGI362273, SGI362276, SGI362277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87610[] = {SGI362278, SGI362279, SGI362280, SGI362281, SGI362279, SGI362282, SGI362283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87611[] = {SGI362284, SGI362285, SGI362286, SGI362287, SGI362285, SGI362288, SGI362289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87612[] = {SGI362290, SGI362291, SGI362292, SGI362293, SGI362291, SGI362294, SGI362295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87613[] = {SGI362296, SGI362297, SGI362298, SGI362299, SGI362297, SGI362300, SGI362301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87614[] = {SGI362302, SGI362303, SGI362304, SGI362305, SGI362303, SGI362306, SGI362307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87615[] = {SGI362308, SGI362309, SGI362310, SGI362311, SGI362309, SGI362312, SGI362313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87616[] = {SGI362314, SGI362315, SGI362316, SGI362317, SGI362315, SGI362318, SGI362319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87617[] = {SGI362320, SGI362321, SGI362322, SGI362323, SGI362321, SGI362324, SGI362325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87618[] = {SGI362326, SGI362327, SGI362328, SGI362329, SGI362327, SGI362330, SGI362331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87619[] = {SGI362332, SGI362333, SGI362334, SGI362335, SGI362333, SGI362336, SGI362337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87620[] = {SGI362338, SGI362339, SGI362340, SGI362341, SGI362339, SGI362342, SGI362343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87621[] = {SGI362344, SGI362345, SGI362346, SGI362347, SGI362345, SGI362348, SGI362349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87622[] = {SGI362350, SGI362351, SGI362352, SGI362353, SGI362351, SGI362354, SGI362355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87623[] = {SGI362356, SGI362357, SGI362358, SGI362359, SGI362357, SGI362360, SGI362361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87624[] = {SGI362362, SGI362363, SGI362364, SGI362365, SGI362363, SGI362366, SGI362367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87625[] = {SGI362368, SGI362369, SGI362370, SGI362371, SGI362369, SGI362372, SGI362373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87626[] = {SGI362374, SGI362375, SGI362376, SGI362377, SGI362375, SGI362378, SGI362379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87627[] = {SGI362380, SGI362381, SGI362382, SGI362383, SGI362381, SGI362384, SGI362385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87628[] = {SGI362386, SGI362387, SGI362388, SGI362389, SGI362387, SGI362390, SGI362391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87629[] = {SGI362392, SGI362393, SGI362394, SGI362395, SGI362393, SGI362396, SGI362397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87630[] = {SGI362398, SGI362399, SGI362400, SGI362401, SGI362399, SGI362402, SGI362403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87631[] = {SGI362404, SGI362405, SGI362406, SGI362407, SGI362405, SGI362408, SGI362409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87632[] = {SGI362410, SGI362411, SGI362412, SGI362413, SGI362411, SGI362414, SGI362415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87633[] = {SGI362416, SGI362417, SGI362418, SGI362419, SGI362417, SGI362420, SGI362421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87634[] = {SGI362422, SGI362423, SGI362424, SGI362425, SGI362423, SGI362426, SGI362427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87635[] = {SGI362428, SGI362429, SGI362430, SGI362431, SGI362429, SGI362432, SGI362433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87636[] = {SGI362434, SGI362435, SGI362436, SGI362437, SGI362435, SGI362438, SGI362439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87637[] = {SGI362440, SGI362441, SGI362442, SGI362443, SGI362441, SGI362444, SGI362445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87638[] = {SGI362446, SGI362447, SGI362448, SGI362449, SGI0, SGI362450, SGI362451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87639[] = {SGI362452, SGI362453, SGI362454, SGI362455, SGI362453, SGI362456, SGI362457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87640[] = {SGI362458, SGI362459, SGI362460, SGI362461, SGI362459, SGI362462, SGI362463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87641[] = {SGI362464, SGI362465, SGI362466, SGI362467, SGI362465, SGI362468, SGI362469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87642[] = {SGI362470, SGI362471, SGI362472, SGI362473, SGI362471, SGI362474, SGI362475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87643[] = {SGI362476, SGI362477, SGI362478, SGI362479, SGI362477, SGI362480, SGI362481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87644[] = {SGI362482, SGI362483, SGI362484, SGI362485, SGI362483, SGI362486, SGI362487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87645[] = {SGI362488, SGI362489, SGI362490, SGI362491, SGI362489, SGI362492, SGI362493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87646[] = {SGI362494, SGI362495, SGI362496, SGI362497, SGI362495, SGI362498, SGI362499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87647[] = {SGI362500, SGI362501, SGI362502, SGI362503, SGI362501, SGI362504, SGI362505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87648[] = {SGI362506, SGI362507, SGI362508, SGI362509, SGI362507, SGI362510, SGI362511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87649[] = {SGI362512, SGI362513, SGI362514, SGI362515, SGI362513, SGI362516, SGI362517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87650[] = {SGI362518, SGI362519, SGI362520, SGI362521, SGI362519, SGI362522, SGI362523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87651[] = {SGI362524, SGI362525, SGI362526, SGI362527, SGI362525, SGI362528, SGI362529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87652[] = {SGI362530, SGI362531, SGI362532, SGI362533, SGI362531, SGI362534, SGI362535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87653[] = {SGI362536, SGI362537, SGI362538, SGI362539, SGI362537, SGI362540, SGI362541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87654[] = {SGI362542, SGI362543, SGI362544, SGI362545, SGI362543, SGI362546, SGI362547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87655[] = {SGI362548, SGI362549, SGI362550, SGI362551, SGI362549, SGI362552, SGI362553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87656[] = {SGI362554, SGI362555, SGI362556, SGI362557, SGI362555, SGI362558, SGI362559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87657[] = {SGI362560, SGI362561, SGI362562, SGI362563, SGI362561, SGI362564, SGI362565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87658[] = {SGI362566, SGI362567, SGI362568, SGI362569, SGI362567, SGI362570, SGI362571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87659[] = {SGI362572, SGI362573, SGI362574, SGI362575, SGI362573, SGI362576, SGI362577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87660[] = {SGI362578, SGI362579, SGI362580, SGI362581, SGI362579, SGI362582, SGI362583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87661[] = {SGI362584, SGI362585, SGI362586, SGI362587, SGI362585, SGI362588, SGI362589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87662[] = {SGI362590, SGI362591, SGI362592, SGI362593, SGI362591, SGI362594, SGI362595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87663[] = {SGI362596, SGI362597, SGI362598, SGI362599, SGI362597, SGI362600, SGI362601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87664[] = {SGI362602, SGI362603, SGI362604, SGI362605, SGI0, SGI362606, SGI362607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87665[] = {SGI362608, SGI362609, SGI362610, SGI362611, SGI362609, SGI362612, SGI362613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87666[] = {SGI362614, SGI362615, SGI362616, SGI362617, SGI362615, SGI362618, SGI362619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87667[] = {SGI362620, SGI362621, SGI362622, SGI362623, SGI362621, SGI362624, SGI362625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87668[] = {SGI362626, SGI362627, SGI362628, SGI362629, SGI362627, SGI362630, SGI362631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87669[] = {SGI362632, SGI362633, SGI362634, SGI362635, SGI362633, SGI362636, SGI362637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87670[] = {SGI362638, SGI362639, SGI362640, SGI362641, SGI362639, SGI362642, SGI362643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87671[] = {SGI362644, SGI362645, SGI362646, SGI362647, SGI0, SGI362648, SGI362649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87672[] = {SGI362650, SGI362651, SGI362652, SGI362653, SGI362651, SGI362654, SGI362655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87673[] = {SGI362656, SGI362657, SGI362658, SGI362659, SGI362657, SGI362660, SGI362661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87674[] = {SGI362662, SGI362663, SGI362664, SGI362665, SGI362663, SGI362666, SGI362667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87675[] = {SGI362668, SGI362669, SGI362670, SGI362671, SGI362669, SGI362672, SGI362673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87676[] = {SGI362674, SGI362675, SGI362676, SGI362677, SGI362675, SGI362678, SGI362679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87677[] = {SGI362680, SGI362681, SGI362682, SGI362683, SGI0, SGI362684, SGI362685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87678[] = {SGI362686, SGI362687, SGI362688, SGI362689, SGI362687, SGI362690, SGI362691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87679[] = {SGI362692, SGI362693, SGI362694, SGI0, SGI362693, SGI0, SGI362695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87680[] = {SGI362696, SGI362697, SGI362698, SGI362699, SGI362697, SGI362700, SGI362701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87681[] = {SGI362702, SGI362703, SGI362704, SGI362705, SGI362703, SGI362706, SGI362707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87682[] = {SGI362708, SGI362709, SGI362710, SGI362711, SGI362709, SGI362712, SGI362713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87683[] = {SGI362714, SGI362715, SGI362716, SGI362717, SGI362715, SGI362718, SGI362719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87684[] = {SGI362720, SGI362721, SGI362722, SGI362723, SGI362721, SGI362724, SGI362725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87685[] = {SGI362726, SGI362727, SGI362728, SGI362729, SGI362727, SGI362730, SGI362731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87686[] = {SGI362732, SGI362733, SGI362734, SGI362735, SGI362733, SGI362736, SGI362737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87687[] = {SGI362738, SGI362739, SGI362740, SGI362741, SGI362739, SGI362742, SGI362743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87688[] = {SGI362744, SGI362745, SGI362746, SGI362747, SGI362745, SGI362748, SGI362749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87689[] = {SGI362750, SGI362751, SGI362752, SGI0, SGI362751, SGI0, SGI362753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87690[] = {SGI362754, SGI362755, SGI362756, SGI362757, SGI362755, SGI362758, SGI362759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87691[] = {SGI362760, SGI362761, SGI362762, SGI362763, SGI0, SGI362764, SGI362765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87692[] = {SGI362766, SGI362767, SGI362768, SGI362769, SGI362767, SGI362770, SGI362771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87693[] = {SGI362772, SGI362773, SGI362774, SGI362775, SGI362773, SGI362776, SGI362777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87694[] = {SGI362778, SGI362779, SGI362780, SGI362781, SGI362779, SGI362782, SGI362783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87695[] = {SGI362784, SGI362785, SGI362786, SGI362787, SGI362785, SGI362788, SGI362789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87696[] = {SGI362790, SGI362791, SGI362792, SGI362793, SGI362791, SGI362794, SGI362795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87697[] = {SGI362796, SGI362797, SGI362798, SGI362799, SGI362797, SGI362800, SGI362801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87698[] = {SGI362802, SGI362803, SGI362804, SGI0, SGI362803, SGI0, SGI362805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87699[] = {SGI362806, SGI362807, SGI362808, SGI0, SGI362807, SGI0, SGI362809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87700[] = {SGI362810, SGI362811, SGI362812, SGI362813, SGI362811, SGI362814, SGI362815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87701[] = {SGI362816, SGI362817, SGI362818, SGI362819, SGI362817, SGI362820, SGI362821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87702[] = {SGI362822, SGI362823, SGI362824, SGI362825, SGI362823, SGI362826, SGI362827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87703[] = {SGI362828, SGI362829, SGI362830, SGI362831, SGI362829, SGI362832, SGI362833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87704[] = {SGI362834, SGI362835, SGI362836, SGI362837, SGI362835, SGI362838, SGI362839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87705[] = {SGI362840, SGI362841, SGI362842, SGI362843, SGI362841, SGI362844, SGI362845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87706[] = {SGI362846, SGI362847, SGI362848, SGI0, SGI362847, SGI0, SGI362849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87707[] = {SGI362850, SGI362851, SGI362852, SGI362853, SGI362851, SGI362854, SGI362855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87708[] = {SGI362856, SGI362857, SGI362858, SGI362859, SGI362857, SGI362860, SGI362861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87709[] = {SGI362862, SGI362863, SGI362864, SGI362865, SGI362863, SGI362866, SGI362867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87710[] = {SGI362868, SGI362869, SGI362870, SGI362871, SGI362869, SGI362872, SGI362873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87711[] = {SGI362874, SGI362875, SGI362876, SGI362877, SGI362875, SGI362878, SGI362879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87712[] = {SGI362880, SGI362881, SGI362882, SGI362883, SGI362881, SGI362884, SGI362885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87713[] = {SGI362886, SGI362887, SGI362888, SGI362889, SGI362887, SGI362890, SGI362891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87714[] = {SGI362892, SGI362893, SGI362894, SGI362895, SGI362893, SGI362896, SGI362897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87715[] = {SGI362898, SGI362899, SGI362900, SGI362901, SGI362899, SGI362902, SGI362903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87716[] = {SGI362904, SGI362905, SGI362906, SGI362907, SGI362905, SGI362908, SGI362909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87717[] = {SGI362910, SGI362911, SGI362912, SGI362913, SGI362911, SGI362914, SGI362915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87718[] = {SGI362916, SGI362917, SGI362918, SGI362919, SGI362917, SGI362920, SGI362921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87719[] = {SGI362922, SGI362923, SGI362924, SGI362925, SGI362923, SGI362926, SGI362927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87720[] = {SGI362928, SGI362929, SGI362930, SGI362931, SGI362929, SGI362932, SGI362933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87721[] = {SGI362934, SGI362935, SGI362936, SGI362937, SGI362935, SGI362938, SGI362939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87722[] = {SGI362940, SGI362941, SGI362942, SGI362943, SGI362941, SGI362944, SGI362945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87723[] = {SGI362946, SGI362947, SGI362948, SGI362949, SGI362947, SGI362950, SGI362951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87724[] = {SGI362952, SGI362953, SGI362954, SGI362955, SGI362953, SGI362956, SGI362957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87725[] = {SGI362958, SGI362959, SGI362960, SGI362961, SGI362959, SGI362962, SGI362963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87726[] = {SGI362964, SGI362965, SGI362966, SGI362967, SGI362965, SGI362968, SGI362969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87727[] = {SGI362970, SGI362971, SGI362972, SGI362973, SGI362971, SGI362974, SGI362975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87728[] = {SGI362976, SGI362977, SGI362978, SGI0, SGI362977, SGI0, SGI362979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87729[] = {SGI362980, SGI362981, SGI362982, SGI362983, SGI362981, SGI362984, SGI362985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87730[] = {SGI362986, SGI362987, SGI362988, SGI362989, SGI362987, SGI362990, SGI362991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87731[] = {SGI362992, SGI362993, SGI362994, SGI362995, SGI362993, SGI362996, SGI362997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87732[] = {SGI362998, SGI362999, SGI363000, SGI363001, SGI362999, SGI363002, SGI363003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87733[] = {SGI363004, SGI363005, SGI363006, SGI363007, SGI363005, SGI363008, SGI363009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87734[] = {SGI363010, SGI363011, SGI363012, SGI363013, SGI363011, SGI363014, SGI363015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87735[] = {SGI363016, SGI363017, SGI363018, SGI363019, SGI363017, SGI363020, SGI363021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87736[] = {SGI363022, SGI363023, SGI363024, SGI0, SGI363023, SGI0, SGI363025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87737[] = {SGI363026, SGI363027, SGI363028, SGI363029, SGI363027, SGI363030, SGI363031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87738[] = {SGI363032, SGI363033, SGI363034, SGI363035, SGI363033, SGI363036, SGI363037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87739[] = {SGI363038, SGI363039, SGI363040, SGI363041, SGI363039, SGI363042, SGI363043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87740[] = {SGI363044, SGI363045, SGI363046, SGI363047, SGI363045, SGI363048, SGI363049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87741[] = {SGI363050, SGI363051, SGI363052, SGI363053, SGI363051, SGI363054, SGI363055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87742[] = {SGI363056, SGI363057, SGI363058, SGI363059, SGI363057, SGI363060, SGI363061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87743[] = {SGI363062, SGI363063, SGI363064, SGI363065, SGI363063, SGI363066, SGI363067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87744[] = {SGI363068, SGI363069, SGI363070, SGI363071, SGI363069, SGI363072, SGI363073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87745[] = {SGI363074, SGI363075, SGI363076, SGI363077, SGI363075, SGI363078, SGI363079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87746[] = {SGI363080, SGI363081, SGI363082, SGI363083, SGI363081, SGI363084, SGI363085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87747[] = {SGI363086, SGI363087, SGI363088, SGI363089, SGI363087, SGI363090, SGI363091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87748[] = {SGI363092, SGI363093, SGI363094, SGI363095, SGI363093, SGI363096, SGI363097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87749[] = {SGI363098, SGI363099, SGI363100, SGI363101, SGI363099, SGI363102, SGI363103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87750[] = {SGI363104, SGI363105, SGI363106, SGI363107, SGI0, SGI363108, SGI363109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87751[] = {SGI363110, SGI363111, SGI363112, SGI363113, SGI363111, SGI363114, SGI363115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87752[] = {SGI363116, SGI363117, SGI363118, SGI363119, SGI363117, SGI363120, SGI363121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87753[] = {SGI363122, SGI363123, SGI363124, SGI363125, SGI363123, SGI363126, SGI363127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87754[] = {SGI363128, SGI363129, SGI363130, SGI363131, SGI363129, SGI363132, SGI363133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87755[] = {SGI363134, SGI363135, SGI363136, SGI363137, SGI363135, SGI363138, SGI363139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87756[] = {SGI363140, SGI363141, SGI363142, SGI363143, SGI363141, SGI363144, SGI363145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87757[] = {SGI363146, SGI363147, SGI363148, SGI363149, SGI363147, SGI363150, SGI363151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87758[] = {SGI363152, SGI363153, SGI363154, SGI363155, SGI363153, SGI363156, SGI363157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87759[] = {SGI363158, SGI363159, SGI363160, SGI363161, SGI363159, SGI363162, SGI363163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87760[] = {SGI363164, SGI363165, SGI363166, SGI363167, SGI363165, SGI363168, SGI363169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87761[] = {SGI363170, SGI363171, SGI363172, SGI363173, SGI363171, SGI363174, SGI363175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87762[] = {SGI363176, SGI363177, SGI363178, SGI363179, SGI363177, SGI363180, SGI363181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87763[] = {SGI363182, SGI363183, SGI363184, SGI363185, SGI363183, SGI363186, SGI363187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87764[] = {SGI363188, SGI363189, SGI363190, SGI363191, SGI363189, SGI363192, SGI363193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87765[] = {SGI363194, SGI363195, SGI363196, SGI363197, SGI363195, SGI363198, SGI363199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87766[] = {SGI363200, SGI363201, SGI363202, SGI363203, SGI363201, SGI363204, SGI363205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87767[] = {SGI363206, SGI363207, SGI363208, SGI363209, SGI363207, SGI363210, SGI363211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87768[] = {SGI363212, SGI363213, SGI363214, SGI363215, SGI363213, SGI363216, SGI363217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87769[] = {SGI363218, SGI363219, SGI363220, SGI363221, SGI363219, SGI363222, SGI363223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87770[] = {SGI363224, SGI363225, SGI363226, SGI363227, SGI363225, SGI363228, SGI363229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87771[] = {SGI363230, SGI363231, SGI363232, SGI363233, SGI363231, SGI363234, SGI363235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87772[] = {SGI363236, SGI363237, SGI363238, SGI363239, SGI363237, SGI363240, SGI363241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87773[] = {SGI363242, SGI363243, SGI363244, SGI363245, SGI363243, SGI363246, SGI363247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87774[] = {SGI363248, SGI363249, SGI363250, SGI363251, SGI363249, SGI363252, SGI363253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87775[] = {SGI363254, SGI363255, SGI363256, SGI363257, SGI363255, SGI363258, SGI363259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87776[] = {SGI363260, SGI363261, SGI363262, SGI363263, SGI363261, SGI363264, SGI363265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87777[] = {SGI363266, SGI363267, SGI363268, SGI363269, SGI363267, SGI363270, SGI363271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87778[] = {SGI363272, SGI363273, SGI363274, SGI363275, SGI363273, SGI363276, SGI363277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87779[] = {SGI363278, SGI363279, SGI363280, SGI363281, SGI363279, SGI363282, SGI363283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87780[] = {SGI363284, SGI363285, SGI363286, SGI363287, SGI363285, SGI363288, SGI363289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87781[] = {SGI363290, SGI363291, SGI363292, SGI363293, SGI363291, SGI363294, SGI363295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87782[] = {SGI363296, SGI363297, SGI363298, SGI363299, SGI363297, SGI363300, SGI363301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87783[] = {SGI363302, SGI363303, SGI363304, SGI363305, SGI363303, SGI363306, SGI363307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87784[] = {SGI363308, SGI363309, SGI363310, SGI363311, SGI363309, SGI363312, SGI363313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87785[] = {SGI363314, SGI363315, SGI363316, SGI363317, SGI363315, SGI363318, SGI363319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87786[] = {SGI363320, SGI363321, SGI363322, SGI363323, SGI363321, SGI363324, SGI363325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87787[] = {SGI363326, SGI363327, SGI363328, SGI363329, SGI363327, SGI363330, SGI363331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87788[] = {SGI363332, SGI363333, SGI363334, SGI363335, SGI363333, SGI363336, SGI363337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87789[] = {SGI363338, SGI363339, SGI363340, SGI363341, SGI363339, SGI363342, SGI363343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87790[] = {SGI363344, SGI363345, SGI363346, SGI363347, SGI363345, SGI363348, SGI363349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87791[] = {SGI363350, SGI363351, SGI363352, SGI363353, SGI363351, SGI363354, SGI363355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87792[] = {SGI363356, SGI363357, SGI363358, SGI363359, SGI363357, SGI363360, SGI363361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87793[] = {SGI363362, SGI363363, SGI363364, SGI363365, SGI363363, SGI363366, SGI363367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87794[] = {SGI363368, SGI363369, SGI363370, SGI363371, SGI363369, SGI363372, SGI363373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87795[] = {SGI363374, SGI363375, SGI363376, SGI363377, SGI363375, SGI363378, SGI363379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87796[] = {SGI363380, SGI363381, SGI363382, SGI363383, SGI363381, SGI363384, SGI363385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87797[] = {SGI363386, SGI363387, SGI363388, SGI363389, SGI363387, SGI363390, SGI363391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87798[] = {SGI363392, SGI363393, SGI363394, SGI363395, SGI363393, SGI363396, SGI363397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87799[] = {SGI363398, SGI363399, SGI363400, SGI363401, SGI363399, SGI363402, SGI363403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87800[] = {SGI363404, SGI363405, SGI363406, SGI363407, SGI363405, SGI363408, SGI363409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87801[] = {SGI363410, SGI363411, SGI363412, SGI363413, SGI363411, SGI363414, SGI363415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87802[] = {SGI363416, SGI363417, SGI363418, SGI363419, SGI363417, SGI363420, SGI363421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87803[] = {SGI363422, SGI363423, SGI363424, SGI363425, SGI363423, SGI363426, SGI363427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87804[] = {SGI363428, SGI363429, SGI363430, SGI363431, SGI363429, SGI363432, SGI363433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87805[] = {SGI363434, SGI363435, SGI363436, SGI363437, SGI363435, SGI363438, SGI363439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87806[] = {SGI363440, SGI363441, SGI363442, SGI363443, SGI363441, SGI363444, SGI363445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87807[] = {SGI363446, SGI363447, SGI363448, SGI363449, SGI363447, SGI363450, SGI363451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87808[] = {SGI363452, SGI363453, SGI363454, SGI363455, SGI363453, SGI363456, SGI363457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87809[] = {SGI363458, SGI363459, SGI363460, SGI363461, SGI363459, SGI363462, SGI363463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87810[] = {SGI363464, SGI363465, SGI363466, SGI363467, SGI363465, SGI363468, SGI363469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87811[] = {SGI363470, SGI363471, SGI363472, SGI363473, SGI363471, SGI363474, SGI363475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87812[] = {SGI363476, SGI363477, SGI363478, SGI363479, SGI363477, SGI363480, SGI363481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87813[] = {SGI363482, SGI363483, SGI363484, SGI363485, SGI363483, SGI363486, SGI363487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87814[] = {SGI363488, SGI363489, SGI363490, SGI363491, SGI363489, SGI363492, SGI363493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87815[] = {SGI363494, SGI363495, SGI363496, SGI363497, SGI363495, SGI363498, SGI363499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87816[] = {SGI363500, SGI363501, SGI363502, SGI363503, SGI363501, SGI363504, SGI363505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87817[] = {SGI363506, SGI363507, SGI363508, SGI363509, SGI363507, SGI363510, SGI363511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87818[] = {SGI363512, SGI363513, SGI363514, SGI363515, SGI363513, SGI363516, SGI363517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87819[] = {SGI363518, SGI363519, SGI363520, SGI363521, SGI363519, SGI363522, SGI363523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87820[] = {SGI363524, SGI363525, SGI363526, SGI363527, SGI363525, SGI363528, SGI363529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87821[] = {SGI363530, SGI363531, SGI363532, SGI363533, SGI363531, SGI363534, SGI363535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87822[] = {SGI363536, SGI363537, SGI363538, SGI363539, SGI363537, SGI363540, SGI363541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87823[] = {SGI363542, SGI363543, SGI363544, SGI363545, SGI363543, SGI363546, SGI363547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87824[] = {SGI363548, SGI363549, SGI363550, SGI363551, SGI363549, SGI363552, SGI363553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87825[] = {SGI363554, SGI363555, SGI363556, SGI363557, SGI363555, SGI363558, SGI363559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87826[] = {SGI363560, SGI363561, SGI363562, SGI363563, SGI363561, SGI363564, SGI363565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87827[] = {SGI363566, SGI363567, SGI363568, SGI363569, SGI363567, SGI363570, SGI363571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87828[] = {SGI363572, SGI363573, SGI363574, SGI363575, SGI363573, SGI363576, SGI363577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87829[] = {SGI363578, SGI363579, SGI363580, SGI363581, SGI363579, SGI363582, SGI363583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87830[] = {SGI363584, SGI363585, SGI363586, SGI363587, SGI363585, SGI363588, SGI363589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87831[] = {SGI363590, SGI363591, SGI363592, SGI363593, SGI363591, SGI363594, SGI363595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87832[] = {SGI363596, SGI363597, SGI363598, SGI363599, SGI363597, SGI363600, SGI363601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87833[] = {SGI363602, SGI363603, SGI363604, SGI363605, SGI363603, SGI363606, SGI363607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87834[] = {SGI363608, SGI363609, SGI363610, SGI363611, SGI363609, SGI363612, SGI363613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87835[] = {SGI363614, SGI363615, SGI363616, SGI363617, SGI363615, SGI363618, SGI363619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87836[] = {SGI363620, SGI363621, SGI363622, SGI363623, SGI363621, SGI363624, SGI363625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87837[] = {SGI363626, SGI363627, SGI363628, SGI363629, SGI363627, SGI363630, SGI363631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87838[] = {SGI363632, SGI363633, SGI363634, SGI363635, SGI363633, SGI363636, SGI363637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87839[] = {SGI363638, SGI363639, SGI363640, SGI363641, SGI363639, SGI363642, SGI363643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87840[] = {SGI363644, SGI363645, SGI363646, SGI363647, SGI363645, SGI363648, SGI363649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87841[] = {SGI363650, SGI363651, SGI363652, SGI363653, SGI363651, SGI363654, SGI363655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87842[] = {SGI363656, SGI363657, SGI363658, SGI363659, SGI363657, SGI363660, SGI363661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87843[] = {SGI363662, SGI363663, SGI363664, SGI363665, SGI363663, SGI363666, SGI363667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87844[] = {SGI363668, SGI363669, SGI363670, SGI363671, SGI363669, SGI363672, SGI363673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87845[] = {SGI363674, SGI363675, SGI363676, SGI363677, SGI363675, SGI363678, SGI363679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87846[] = {SGI363680, SGI363681, SGI363682, SGI363683, SGI363681, SGI363684, SGI363685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87847[] = {SGI363686, SGI363687, SGI363688, SGI363689, SGI363687, SGI363690, SGI363691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87848[] = {SGI363692, SGI363693, SGI363694, SGI363695, SGI363693, SGI363696, SGI363697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87849[] = {SGI363698, SGI363699, SGI363700, SGI363701, SGI363699, SGI363702, SGI363703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87850[] = {SGI363704, SGI363705, SGI363706, SGI363707, SGI363705, SGI363708, SGI363709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87851[] = {SGI363710, SGI363711, SGI363712, SGI363713, SGI363711, SGI363714, SGI363715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87852[] = {SGI363716, SGI363717, SGI363718, SGI363719, SGI363717, SGI363720, SGI363721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87853[] = {SGI363722, SGI363723, SGI363724, SGI363725, SGI363723, SGI363726, SGI363727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87854[] = {SGI363728, SGI363729, SGI363730, SGI363731, SGI363729, SGI363732, SGI363733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87855[] = {SGI363734, SGI363735, SGI363736, SGI363737, SGI0, SGI363738, SGI363739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87856[] = {SGI363740, SGI363741, SGI363742, SGI363743, SGI363741, SGI363744, SGI363745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87857[] = {SGI363746, SGI363747, SGI363748, SGI363749, SGI363747, SGI363750, SGI363751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87858[] = {SGI363752, SGI363753, SGI363754, SGI363755, SGI363753, SGI363756, SGI363757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87859[] = {SGI363758, SGI363759, SGI363760, SGI363761, SGI0, SGI363762, SGI363763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87860[] = {SGI363764, SGI363765, SGI363766, SGI363767, SGI363765, SGI363768, SGI363769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87861[] = {SGI363770, SGI363771, SGI363772, SGI363773, SGI363771, SGI363774, SGI363775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87862[] = {SGI363776, SGI363777, SGI363778, SGI363779, SGI363777, SGI363780, SGI363781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87863[] = {SGI363782, SGI363783, SGI363784, SGI363785, SGI363783, SGI363786, SGI363787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87864[] = {SGI363788, SGI363789, SGI363790, SGI363791, SGI363789, SGI363792, SGI363793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87865[] = {SGI363794, SGI363795, SGI363796, SGI363797, SGI363795, SGI363798, SGI363799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87866[] = {SGI363800, SGI363801, SGI363802, SGI363803, SGI363801, SGI363804, SGI363805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87867[] = {SGI363806, SGI363807, SGI363808, SGI363809, SGI363807, SGI363810, SGI363811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87868[] = {SGI363812, SGI363813, SGI363814, SGI363815, SGI363813, SGI363816, SGI363817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87869[] = {SGI363818, SGI363819, SGI363820, SGI363821, SGI363819, SGI363822, SGI363823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87870[] = {SGI363824, SGI363825, SGI363826, SGI363827, SGI363825, SGI363828, SGI363829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87871[] = {SGI363830, SGI363831, SGI363832, SGI363833, SGI363831, SGI363834, SGI363835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87872[] = {SGI363836, SGI363837, SGI363838, SGI363839, SGI363837, SGI363840, SGI363841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87873[] = {SGI363842, SGI363843, SGI363844, SGI363845, SGI363843, SGI363846, SGI363847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87874[] = {SGI363848, SGI363849, SGI363850, SGI363851, SGI363849, SGI363852, SGI363853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87875[] = {SGI363854, SGI363855, SGI363856, SGI363857, SGI363855, SGI363858, SGI363859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87876[] = {SGI363860, SGI363861, SGI363862, SGI363863, SGI363861, SGI363864, SGI363865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87877[] = {SGI363866, SGI363867, SGI363868, SGI363869, SGI363867, SGI363870, SGI363871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87878[] = {SGI363872, SGI363873, SGI363874, SGI363875, SGI363873, SGI363876, SGI363877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87879[] = {SGI363878, SGI363879, SGI363880, SGI363881, SGI363879, SGI363882, SGI363883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87880[] = {SGI363884, SGI363885, SGI363886, SGI363887, SGI363885, SGI363888, SGI363889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87881[] = {SGI363890, SGI363891, SGI363892, SGI363893, SGI363891, SGI363894, SGI363895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87882[] = {SGI363896, SGI363897, SGI363898, SGI363899, SGI363897, SGI363900, SGI363901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87883[] = {SGI363902, SGI363903, SGI363904, SGI363905, SGI363903, SGI363906, SGI363907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87884[] = {SGI363908, SGI363909, SGI363910, SGI363911, SGI363909, SGI363912, SGI363913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87885[] = {SGI363914, SGI363915, SGI363916, SGI363917, SGI363915, SGI363918, SGI363919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87886[] = {SGI363920, SGI363921, SGI363922, SGI363923, SGI363921, SGI363924, SGI363925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87887[] = {SGI363926, SGI363927, SGI363928, SGI363929, SGI363927, SGI363930, SGI363931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87888[] = {SGI363932, SGI363933, SGI363934, SGI363935, SGI363933, SGI363936, SGI363937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87889[] = {SGI363938, SGI363939, SGI363940, SGI363941, SGI363939, SGI363942, SGI363943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87890[] = {SGI363944, SGI363945, SGI363946, SGI363947, SGI363945, SGI363948, SGI363949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87891[] = {SGI363950, SGI363951, SGI363952, SGI363953, SGI363951, SGI363954, SGI363955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87892[] = {SGI363956, SGI363957, SGI363958, SGI363959, SGI363957, SGI363960, SGI363961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87893[] = {SGI363962, SGI363963, SGI363964, SGI363965, SGI363963, SGI363966, SGI363967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87894[] = {SGI363968, SGI363969, SGI363970, SGI363971, SGI363969, SGI363972, SGI363973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87895[] = {SGI363974, SGI363975, SGI363976, SGI363977, SGI363975, SGI363978, SGI363979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87896[] = {SGI363980, SGI363981, SGI363982, SGI363983, SGI363981, SGI363984, SGI363985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87897[] = {SGI363986, SGI363987, SGI363988, SGI363989, SGI363987, SGI363990, SGI363991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87898[] = {SGI363992, SGI363993, SGI363994, SGI363995, SGI363993, SGI363996, SGI363997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87899[] = {SGI363998, SGI363999, SGI364000, SGI364001, SGI363999, SGI364002, SGI364003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87900[] = {SGI364004, SGI364005, SGI364006, SGI364007, SGI364005, SGI364008, SGI364009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87901[] = {SGI364010, SGI364011, SGI364012, SGI364013, SGI364011, SGI364014, SGI364015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87902[] = {SGI364016, SGI364017, SGI364018, SGI364019, SGI364017, SGI364020, SGI364021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87903[] = {SGI364022, SGI364023, SGI364024, SGI364025, SGI364023, SGI364026, SGI364027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87904[] = {SGI364028, SGI364029, SGI364030, SGI364031, SGI364029, SGI364032, SGI364033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87905[] = {SGI364034, SGI364035, SGI364036, SGI364037, SGI364035, SGI364038, SGI364039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87906[] = {SGI364040, SGI364041, SGI364042, SGI364043, SGI364041, SGI364044, SGI364045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87907[] = {SGI364046, SGI364047, SGI364048, SGI364049, SGI364047, SGI364050, SGI364051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87908[] = {SGI364052, SGI364053, SGI364054, SGI364055, SGI364053, SGI364056, SGI364057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87909[] = {SGI364058, SGI364059, SGI364060, SGI364061, SGI364059, SGI364062, SGI364063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87910[] = {SGI364064, SGI364065, SGI364066, SGI364067, SGI364065, SGI364068, SGI364069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87911[] = {SGI364070, SGI364071, SGI364072, SGI364073, SGI364071, SGI364074, SGI364075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87912[] = {SGI364076, SGI364077, SGI364078, SGI364079, SGI364077, SGI364080, SGI364081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87913[] = {SGI364082, SGI364083, SGI364084, SGI364085, SGI364083, SGI364086, SGI364087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87914[] = {SGI364088, SGI364089, SGI364090, SGI364091, SGI364089, SGI364092, SGI364093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87915[] = {SGI364094, SGI364095, SGI364096, SGI364097, SGI364095, SGI364098, SGI364099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87916[] = {SGI364100, SGI364101, SGI364102, SGI364103, SGI364101, SGI364104, SGI364105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87917[] = {SGI364106, SGI364107, SGI364108, SGI364109, SGI364107, SGI364110, SGI364111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87918[] = {SGI364112, SGI364113, SGI364114, SGI364115, SGI364113, SGI364116, SGI364117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87919[] = {SGI364118, SGI364119, SGI364120, SGI364121, SGI0, SGI364122, SGI364123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87920[] = {SGI364124, SGI364125, SGI364126, SGI364127, SGI364125, SGI364128, SGI364129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87921[] = {SGI364130, SGI364131, SGI364132, SGI364133, SGI364131, SGI364134, SGI364135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87922[] = {SGI364136, SGI364137, SGI364138, SGI364139, SGI364137, SGI364140, SGI364141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87923[] = {SGI364142, SGI364143, SGI364144, SGI364145, SGI364143, SGI364146, SGI364147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87924[] = {SGI364148, SGI364149, SGI364150, SGI364151, SGI364149, SGI364152, SGI364153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87925[] = {SGI364154, SGI364155, SGI364156, SGI364157, SGI364155, SGI364158, SGI364159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87926[] = {SGI364160, SGI364161, SGI364162, SGI364163, SGI364161, SGI364164, SGI364165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87927[] = {SGI364166, SGI364167, SGI364168, SGI364169, SGI364167, SGI364170, SGI364171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87928[] = {SGI364172, SGI364173, SGI364174, SGI364175, SGI364173, SGI364176, SGI364177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87929[] = {SGI364178, SGI364179, SGI364180, SGI364181, SGI364179, SGI364182, SGI364183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87930[] = {SGI364184, SGI364185, SGI364186, SGI364187, SGI364185, SGI364188, SGI364189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87931[] = {SGI364190, SGI364191, SGI364192, SGI364193, SGI364191, SGI364194, SGI364195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87932[] = {SGI364196, SGI364197, SGI364198, SGI364199, SGI364197, SGI364200, SGI364201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87933[] = {SGI364202, SGI364203, SGI364204, SGI364205, SGI364203, SGI364206, SGI364207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87934[] = {SGI364208, SGI364209, SGI364210, SGI364211, SGI364209, SGI364212, SGI364213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87935[] = {SGI364214, SGI364215, SGI364216, SGI364217, SGI364215, SGI364218, SGI364219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87936[] = {SGI364220, SGI364221, SGI364222, SGI364223, SGI364221, SGI364224, SGI364225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87937[] = {SGI364226, SGI364227, SGI364228, SGI364229, SGI364227, SGI364230, SGI364231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87938[] = {SGI364232, SGI364233, SGI364234, SGI364235, SGI364233, SGI364236, SGI364237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87939[] = {SGI364238, SGI364239, SGI364240, SGI364241, SGI364239, SGI364242, SGI364243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87940[] = {SGI364244, SGI364245, SGI364246, SGI364247, SGI364245, SGI364248, SGI364249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87941[] = {SGI364250, SGI364251, SGI364252, SGI364253, SGI364251, SGI364254, SGI364255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87942[] = {SGI364256, SGI364257, SGI364258, SGI364259, SGI364257, SGI364260, SGI364261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87943[] = {SGI364262, SGI364263, SGI364264, SGI364265, SGI364263, SGI364266, SGI364267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87944[] = {SGI364268, SGI364269, SGI364270, SGI364271, SGI364269, SGI364272, SGI364273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87945[] = {SGI364274, SGI364275, SGI364276, SGI364277, SGI364275, SGI364278, SGI364279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87946[] = {SGI364280, SGI364281, SGI364282, SGI364283, SGI364281, SGI364284, SGI364285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87947[] = {SGI364286, SGI364287, SGI364288, SGI364289, SGI364287, SGI364290, SGI364291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87948[] = {SGI364292, SGI364293, SGI364294, SGI364295, SGI364293, SGI364296, SGI364297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87949[] = {SGI364298, SGI364299, SGI364300, SGI364301, SGI364299, SGI364302, SGI364303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87950[] = {SGI364304, SGI364305, SGI364306, SGI364307, SGI364305, SGI364308, SGI364309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87951[] = {SGI364310, SGI364311, SGI364312, SGI364313, SGI364311, SGI364314, SGI364315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87952[] = {SGI364316, SGI364317, SGI364318, SGI364319, SGI364317, SGI364320, SGI364321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87953[] = {SGI364322, SGI364323, SGI364324, SGI364325, SGI364323, SGI364326, SGI364327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87954[] = {SGI364328, SGI364329, SGI364330, SGI364331, SGI364329, SGI364332, SGI364333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87955[] = {SGI364334, SGI364335, SGI364336, SGI364337, SGI364335, SGI364338, SGI364339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87956[] = {SGI364340, SGI364341, SGI364342, SGI364343, SGI364341, SGI364344, SGI364345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87957[] = {SGI364346, SGI364347, SGI364348, SGI364349, SGI364347, SGI364350, SGI364351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87958[] = {SGI364352, SGI364353, SGI364354, SGI364355, SGI364353, SGI364356, SGI364357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87959[] = {SGI364358, SGI364359, SGI364360, SGI0, SGI0, SGI0, SGI364361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87960[] = {SGI364362, SGI364363, SGI364364, SGI364365, SGI364363, SGI364366, SGI364367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87961[] = {SGI364368, SGI364369, SGI364370, SGI364371, SGI364369, SGI364372, SGI364373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87962[] = {SGI364374, SGI364375, SGI364376, SGI364377, SGI364375, SGI364378, SGI364379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87963[] = {SGI364380, SGI364381, SGI364382, SGI364383, SGI364381, SGI364384, SGI364385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87964[] = {SGI364386, SGI364387, SGI364388, SGI364389, SGI364387, SGI364390, SGI364391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87965[] = {SGI364392, SGI364393, SGI364394, SGI364395, SGI364393, SGI364396, SGI364397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87966[] = {SGI364398, SGI364399, SGI364400, SGI364401, SGI364399, SGI364402, SGI364403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87967[] = {SGI364404, SGI364405, SGI364406, SGI364407, SGI364405, SGI364408, SGI364409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87968[] = {SGI364410, SGI364411, SGI364412, SGI364413, SGI364411, SGI364414, SGI364415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87969[] = {SGI364416, SGI364417, SGI364418, SGI364419, SGI364417, SGI364420, SGI364421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87970[] = {SGI364422, SGI364423, SGI364424, SGI364425, SGI364423, SGI364426, SGI364427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87971[] = {SGI364428, SGI364429, SGI364430, SGI364431, SGI364429, SGI364432, SGI364433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87972[] = {SGI364434, SGI364435, SGI364436, SGI364437, SGI364435, SGI364438, SGI364439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87973[] = {SGI364445, SGI364447, SGI364448, SGI364447, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87974[] = {SGI364440, SGI364441, SGI364442, SGI364443, SGI364441, SGI364444, SGI364446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87975[] = {SGI364449, SGI364450, SGI364451, SGI364452, SGI364450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87976[] = {SGI364453, SGI364454, SGI364455, SGI364456, SGI364454, SGI364457, SGI364458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87977[] = {SGI364459, SGI364460, SGI364461, SGI364462, SGI364460, SGI364463, SGI364464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87978[] = {SGI364465, SGI364466, SGI364467, SGI364468, SGI364466, SGI364469, SGI364470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87979[] = {SGI364471, SGI364472, SGI364473, SGI364474, SGI364472, SGI364475, SGI364476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87980[] = {SGI364477, SGI364478, SGI364479, SGI364480, SGI364478, SGI364481, SGI364482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87981[] = {SGI364483, SGI364484, SGI364485, SGI364486, SGI364484, SGI364487, SGI364488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87982[] = {SGI364489, SGI364490, SGI364491, SGI364492, SGI364490, SGI364493, SGI364494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87983[] = {SGI364495, SGI364496, SGI364497, SGI364498, SGI364496, SGI364499, SGI364500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87984[] = {SGI364501, SGI364502, SGI364503, SGI364504, SGI364502, SGI364505, SGI364506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87985[] = {SGI364507, SGI364508, SGI364509, SGI364510, SGI364508, SGI364511, SGI364512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87986[] = {SGI364513, SGI364514, SGI364515, SGI364516, SGI364514, SGI364517, SGI364518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87987[] = {SGI364519, SGI364520, SGI364521, SGI364522, SGI364520, SGI364523, SGI364524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87988[] = {SGI364525, SGI364526, SGI364527, SGI364528, SGI364526, SGI364529, SGI364530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87989[] = {SGI364531, SGI364532, SGI364533, SGI364534, SGI364532, SGI364535, SGI364536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87990[] = {SGI364537, SGI364538, SGI364539, SGI364540, SGI364538, SGI364541, SGI364542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87991[] = {SGI364543, SGI364544, SGI364545, SGI364546, SGI364544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87992[] = {SGI364547, SGI364548, SGI364549, SGI364550, SGI364548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87993[] = {SGI364551, SGI364552, SGI364553, SGI364554, SGI364552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87994[] = {SGI364555, SGI364556, SGI364557, SGI364558, SGI364556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87995[] = {SGI364559, SGI364560, SGI364561, SGI364562, SGI364560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87996[] = {SGI364563, SGI364564, SGI364565, SGI364566, SGI364564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87997[] = {SGI364567, SGI364568, SGI364569, SGI364570, SGI364568, SGI364571, SGI364572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87998[] = {SGI364573, SGI364574, SGI364575, SGI364576, SGI364574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG87999[] = {SGI364577, SGI364578, SGI364579, SGI364580, SGI364578, SGI364581, SGI364582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88000[] = {SGI364583, SGI364584, SGI364585, SGI364586, SGI364584, SGI364587, SGI364588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88001[] = {SGI364589, SGI364590, SGI364591, SGI364592, SGI364590, SGI364593, SGI364594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88002[] = {SGI364595, SGI364596, SGI364597, SGI364598, SGI364596, SGI364599, SGI364600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88003[] = {SGI364601, SGI364602, SGI364603, SGI364604, SGI364602, SGI364605, SGI364606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88004[] = {SGI364607, SGI364608, SGI364609, SGI364610, SGI364608, SGI364611, SGI364612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88005[] = {SGI364613, SGI364614, SGI364615, SGI364616, SGI364614, SGI364617, SGI364618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88006[] = {SGI364619, SGI364620, SGI364621, SGI364622, SGI364620, SGI364623, SGI364624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88007[] = {SGI364625, SGI364626, SGI364627, SGI364628, SGI364626, SGI364629, SGI364630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88008[] = {SGI364631, SGI364632, SGI364633, SGI364634, SGI364632, SGI364635, SGI364636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88009[] = {SGI364637, SGI364638, SGI364639, SGI364640, SGI364638, SGI364641, SGI364642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88010[] = {SGI364643, SGI364644, SGI364645, SGI364646, SGI364644, SGI364647, SGI364648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88011[] = {SGI364649, SGI364650, SGI364651, SGI364652, SGI364650, SGI364653, SGI364654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88012[] = {SGI364655, SGI364656, SGI364657, SGI364658, SGI364656, SGI364659, SGI364660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88013[] = {SGI364661, SGI364662, SGI364663, SGI364664, SGI364662, SGI364665, SGI364666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88014[] = {SGI364667, SGI364668, SGI364669, SGI364670, SGI364668, SGI364671, SGI364672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88015[] = {SGI364673, SGI364674, SGI364675, SGI364676, SGI364674, SGI0, SGI364676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88016[] = {SGI364677, SGI364678, SGI364679, SGI364680, SGI364678, SGI364681, SGI364682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88017[] = {SGI364683, SGI364684, SGI0, SGI364684, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88018[] = {SGI364685, SGI364686, SGI364687, SGI364688, SGI364686, SGI364689, SGI364690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88019[] = {SGI364691, SGI364692, SGI364693, SGI364694, SGI364692, SGI364695, SGI364696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88020[] = {SGI364697, SGI364698, SGI364699, SGI364700, SGI364698, SGI364701, SGI364702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88021[] = {SGI364703, SGI364704, SGI0, SGI364704, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88022[] = {SGI364705, SGI364706, SGI364707, SGI364708, SGI364706, SGI364709, SGI364710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88023[] = {SGI364711, SGI364712, SGI0, SGI364712, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88024[] = {SGI364718, SGI364720, SGI0, SGI364720, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88025[] = {SGI364713, SGI364714, SGI364715, SGI364716, SGI364714, SGI364717, SGI364719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88026[] = {SGI364721, SGI364722, SGI364723, SGI364724, SGI364722, SGI364725, SGI364726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88027[] = {SGI364731, SGI364733, SGI0, SGI364733, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88028[] = {SGI364727, SGI364728, SGI364729, SGI364730, SGI364728, SGI364732, SGI364734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88029[] = {SGI364735, SGI364736, SGI364737, SGI364738, SGI364736, SGI364739, SGI364740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88030[] = {SGI364741, SGI364742, SGI0, SGI364742, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88031[] = {SGI364743, SGI364744, SGI364745, SGI364746, SGI364744, SGI364747, SGI364748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88032[] = {SGI364749, SGI364750, SGI364751, SGI364752, SGI364750, SGI364753, SGI364754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88033[] = {SGI364755, SGI364756, SGI0, SGI364756, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88034[] = {SGI364757, SGI364758, SGI0, SGI364758, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88035[] = {SGI364759, SGI364760, SGI0, SGI364760, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88036[] = {SGI364761, SGI364762, SGI364763, SGI0, SGI364764, SGI0, SGI364765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88037[] = {SGI364766, SGI364767, SGI364768, SGI0, SGI364769, SGI0, SGI364770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88038[] = {SGI364771, SGI364772, SGI364773, SGI0, SGI364774, SGI0, SGI364775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88039[] = {SGI364776, SGI364777, SGI364778, SGI0, SGI364779, SGI0, SGI364780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88040[] = {SGI364781, SGI364782, SGI364783, SGI0, SGI364784, SGI0, SGI364785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88041[] = {SGI364786, SGI364787, SGI364788, SGI0, SGI364789, SGI0, SGI364790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88042[] = {SGI364791, SGI364792, SGI364793, SGI0, SGI364794, SGI0, SGI364795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88043[] = {SGI364796, SGI364797, SGI364798, SGI0, SGI364799, SGI0, SGI364800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88044[] = {SGI364801, SGI364802, SGI364803, SGI0, SGI0, SGI0, SGI364804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88045[] = {SGI364805, SGI364806, SGI364807, SGI0, SGI364808, SGI0, SGI364809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88046[] = {SGI364810, SGI364811, SGI364812, SGI0, SGI0, SGI0, SGI364813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88047[] = {SGI364814, SGI364815, SGI364816, SGI364817, SGI364818, SGI364819, SGI364820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88048[] = {SGI364821, SGI364822, SGI364823, SGI0, SGI364824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88049[] = {SGI364825, SGI364826, SGI364827, SGI0, SGI364828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88050[] = {SGI364829, SGI364830, SGI364831, SGI0, SGI364832, SGI0, SGI364833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88051[] = {SGI364834, SGI364835, SGI364836, SGI0, SGI364837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88052[] = {SGI364838, SGI364839, SGI364840, SGI0, SGI0, SGI0, SGI364841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88053[] = {SGI364842, SGI364843, SGI364844, SGI0, SGI364845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88054[] = {SGI364846, SGI364847, SGI364848, SGI0, SGI364849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88055[] = {SGI364850, SGI364851, SGI364852, SGI0, SGI364853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88056[] = {SGI364854, SGI364855, SGI364856, SGI0, SGI0, SGI0, SGI364857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88057[] = {SGI364858, SGI364859, SGI364860, SGI0, SGI364861, SGI0, SGI364862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88058[] = {SGI364863, SGI364864, SGI364865, SGI364866, SGI364867, SGI364868, SGI364869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88059[] = {SGI364870, SGI364871, SGI364872, SGI0, SGI364873, SGI0, SGI364874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88060[] = {SGI364875, SGI364876, SGI364877, SGI0, SGI0, SGI0, SGI364878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88061[] = {SGI364879, SGI364880, SGI364881, SGI0, SGI364882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88062[] = {SGI364883, SGI364884, SGI364885, SGI0, SGI364886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88063[] = {SGI364887, SGI364888, SGI364889, SGI0, SGI364890, SGI0, SGI364891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88064[] = {SGI364892, SGI364893, SGI364894, SGI0, SGI364895, SGI0, SGI364896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88065[] = {SGI364897, SGI364898, SGI364899, SGI0, SGI364900, SGI0, SGI364901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88066[] = {SGI364902, SGI364903, SGI364904, SGI0, SGI0, SGI0, SGI364905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88067[] = {SGI364906, SGI364907, SGI364908, SGI0, SGI364909, SGI0, SGI364910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88068[] = {SGI364911, SGI364912, SGI364913, SGI364914, SGI364915, SGI364916, SGI364917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88069[] = {SGI364918, SGI364919, SGI364920, SGI0, SGI0, SGI0, SGI364921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88070[] = {SGI364922, SGI364923, SGI364924, SGI364925, SGI364926, SGI364927, SGI364928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88071[] = {SGI364929, SGI364930, SGI364931, SGI0, SGI364932, SGI0, SGI364933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88072[] = {SGI364934, SGI364935, SGI364936, SGI0, SGI0, SGI0, SGI364937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88073[] = {SGI364938, SGI364939, SGI364940, SGI364941, SGI364942, SGI364943, SGI364944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88074[] = {SGI364945, SGI364946, SGI364947, SGI0, SGI364948, SGI0, SGI364949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88075[] = {SGI364950, SGI364951, SGI364952, SGI0, SGI364953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88076[] = {SGI364954, SGI364955, SGI364956, SGI0, SGI364957, SGI0, SGI364958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88077[] = {SGI364959, SGI364960, SGI364961, SGI0, SGI364962, SGI0, SGI364963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88078[] = {SGI364964, SGI364965, SGI364966, SGI0, SGI0, SGI0, SGI364967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88079[] = {SGI364968, SGI364969, SGI364970, SGI0, SGI0, SGI0, SGI364971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88080[] = {SGI364972, SGI364973, SGI364974, SGI364975, SGI364976, SGI364977, SGI364978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88081[] = {SGI364979, SGI364980, SGI364981, SGI364982, SGI364983, SGI364984, SGI364985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88082[] = {SGI364986, SGI364987, SGI364988, SGI0, SGI0, SGI0, SGI364989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88083[] = {SGI364990, SGI364991, SGI364992, SGI364993, SGI364994, SGI364995, SGI364996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88084[] = {SGI364997, SGI364998, SGI364999, SGI365000, SGI365001, SGI365002, SGI365003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88085[] = {SGI365004, SGI365005, SGI365006, SGI0, SGI365007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88086[] = {SGI365008, SGI365009, SGI365010, SGI365011, SGI365012, SGI365013, SGI365014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88087[] = {SGI365015, SGI365016, SGI365017, SGI365018, SGI365019, SGI365020, SGI365021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88088[] = {SGI365022, SGI365023, SGI365024, SGI365025, SGI365026, SGI365027, SGI365028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88089[] = {SGI365029, SGI365030, SGI365031, SGI0, SGI0, SGI0, SGI365032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88090[] = {SGI365033, SGI365034, SGI365035, SGI0, SGI365036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88091[] = {SGI365037, SGI365038, SGI365039, SGI0, SGI365040, SGI0, SGI365041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88092[] = {SGI365042, SGI365043, SGI365044, SGI0, SGI0, SGI0, SGI365045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88093[] = {SGI365046, SGI365047, SGI365048, SGI0, SGI365049, SGI0, SGI365050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88094[] = {SGI365051, SGI365052, SGI365053, SGI0, SGI0, SGI0, SGI365054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88095[] = {SGI365055, SGI365056, SGI365057, SGI0, SGI365058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88096[] = {SGI365059, SGI365060, SGI365061, SGI365062, SGI365063, SGI365064, SGI365065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88097[] = {SGI365066, SGI365067, SGI365068, SGI0, SGI365069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88098[] = {SGI365070, SGI365071, SGI365072, SGI365073, SGI365074, SGI365075, SGI365076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88099[] = {SGI365077, SGI365078, SGI365079, SGI365079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88100[] = {SGI365080, SGI365081, SGI365082, SGI365082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88101[] = {SGI365083, SGI365084, SGI365085, SGI365085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88102[] = {SGI365086, SGI365087, SGI365088, SGI365088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88103[] = {SGI365089, SGI365090, SGI365091, SGI365091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88104[] = {SGI365092, SGI365093, SGI365094, SGI365094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88105[] = {SGI365095, SGI365096, SGI365097, SGI365097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88106[] = {SGI365098, SGI365099, SGI365100, SGI365100, SGI365099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88107[] = {SGI365101, SGI365102, SGI365103, SGI365103, SGI365102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88108[] = {SGI365104, SGI365105, SGI365106, SGI365106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI365107, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88109[] = {SGI365108, SGI365109, SGI365110, SGI365110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88110[] = {SGI365111, SGI365112, SGI365113, SGI365113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI365114, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88111[] = {SGI365115, SGI365116, SGI365117, SGI365117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI365118, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88112[] = {SGI365119, SGI365120, SGI365121, SGI365121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI365122, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88113[] = {SGI365123, SGI365124, SGI365125, SGI365125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88114[] = {SGI365126, SGI365127, SGI365128, SGI365128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88115[] = {SGI365129, SGI365130, SGI365131, SGI365131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI365132, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88116[] = {SGI365133, SGI365134, SGI365135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88117[] = {SGI365136, SGI365137, SGI365138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88118[] = {SGI365139, SGI365140, SGI365141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88119[] = {SGI365142, SGI365143, SGI365144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88120[] = {SGI365145, SGI365146, SGI365147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88121[] = {SGI365148, SGI365149, SGI365150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88122[] = {SGI365151, SGI365152, SGI365153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88123[] = {SGI365154, SGI365155, SGI365156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88124[] = {SGI365157, SGI365158, SGI365159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88125[] = {SGI365160, SGI365161, SGI365162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88126[] = {SGI365163, SGI365164, SGI365165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88127[] = {SGI365166, SGI365167, SGI365168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88128[] = {SGI365169, SGI365170, SGI365171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88129[] = {SGI0, SGI365172, SGI365173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88130[] = {SGI365174, SGI365175, SGI365176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88131[] = {SGI365177, SGI365178, SGI365179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88132[] = {SGI365180, SGI365181, SGI365182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88133[] = {SGI365183, SGI365184, SGI365185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88134[] = {SGI365186, SGI365187, SGI365188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88135[] = {SGI365189, SGI365190, SGI365191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88136[] = {SGI365192, SGI365193, SGI365194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88137[] = {SGI365195, SGI365196, SGI365197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88138[] = {SGI365198, SGI365199, SGI365200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88139[] = {SGI365201, SGI365202, SGI365203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88140[] = {SGI365204, SGI365205, SGI365206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88141[] = {SGI365207, SGI365208, SGI365209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88142[] = {SGI365210, SGI365211, SGI365212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88143[] = {SGI365213, SGI365214, SGI365215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88144[] = {SGI365216, SGI365217, SGI365218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88145[] = {SGI365219, SGI365220, SGI365221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88146[] = {SGI365222, SGI365223, SGI365224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88147[] = {SGI365225, SGI365226, SGI365227, SGI0, SGI0, SGI0, SGI0, SGI365228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88148[] = {SGI365229, SGI365230, SGI365231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88149[] = {SGI365232, SGI365233, SGI365234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88150[] = {SGI365235, SGI365236, SGI365237, SGI0, SGI0, SGI0, SGI0, SGI365238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88151[] = {SGI365239, SGI365240, SGI365241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88152[] = {SGI365242, SGI365243, SGI365244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88153[] = {SGI365245, SGI365246, SGI365247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88154[] = {SGI365248, SGI365249, SGI365250, SGI365251, SGI365252, SGI0, SGI365251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88155[] = {SGI365253, SGI365254, SGI365255, SGI0, SGI365256, SGI0, SGI365257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88156[] = {SGI365258, SGI365259, SGI365260, SGI0, SGI365261, SGI0, SGI365262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88157[] = {SGI365263, SGI365264, SGI365265, SGI0, SGI365266, SGI0, SGI365267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88158[] = {SGI365268, SGI365269, SGI365270, SGI0, SGI365271, SGI0, SGI365272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88159[] = {SGI365273, SGI365274, SGI365275, SGI0, SGI365276, SGI0, SGI365277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88160[] = {SGI365278, SGI365279, SGI365280, SGI0, SGI365281, SGI0, SGI365282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88161[] = {SGI365283, SGI365284, SGI365285, SGI0, SGI365286, SGI0, SGI365287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88162[] = {SGI365288, SGI365289, SGI365290, SGI0, SGI0, SGI0, SGI365291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88163[] = {SGI365292, SGI365293, SGI365294, SGI0, SGI365295, SGI0, SGI365296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88164[] = {SGI365297, SGI365298, SGI365299, SGI0, SGI365300, SGI0, SGI365301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88165[] = {SGI365302, SGI365303, SGI365304, SGI0, SGI365305, SGI0, SGI365306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88166[] = {SGI365307, SGI365308, SGI365309, SGI0, SGI0, SGI0, SGI365310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88167[] = {SGI365311, SGI365312, SGI365313, SGI0, SGI365314, SGI0, SGI365315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88168[] = {SGI365316, SGI365317, SGI365318, SGI0, SGI365319, SGI0, SGI365320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88169[] = {SGI365321, SGI365322, SGI365323, SGI0, SGI0, SGI0, SGI365324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88170[] = {SGI365325, SGI365326, SGI365327, SGI0, SGI0, SGI0, SGI365328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88171[] = {SGI365329, SGI365330, SGI365331, SGI0, SGI365332, SGI0, SGI365333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88172[] = {SGI365334, SGI365335, SGI365336, SGI0, SGI365337, SGI0, SGI365338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88173[] = {SGI365339, SGI365340, SGI365341, SGI0, SGI365342, SGI0, SGI365343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88174[] = {SGI365344, SGI365345, SGI365346, SGI0, SGI0, SGI0, SGI365347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88175[] = {SGI365348, SGI365349, SGI365350, SGI0, SGI0, SGI0, SGI365351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88176[] = {SGI365352, SGI365353, SGI0, SGI0, SGI365354, SGI0, SGI365355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88177[] = {SGI365356, SGI365357, SGI365358, SGI0, SGI365359, SGI0, SGI365360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88178[] = {SGI365361, SGI365362, SGI365363, SGI0, SGI0, SGI0, SGI365364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88179[] = {SGI365365, SGI365366, SGI365367, SGI0, SGI365368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88180[] = {SGI365369, SGI365370, SGI365371, SGI0, SGI365372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88181[] = {SGI365373, SGI365374, SGI365375, SGI0, SGI365376, SGI0, SGI365377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88182[] = {SGI365378, SGI365379, SGI365380, SGI0, SGI365381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88183[] = {SGI365382, SGI365383, SGI365384, SGI0, SGI365385, SGI0, SGI365386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88184[] = {SGI365387, SGI365388, SGI365389, SGI0, SGI0, SGI0, SGI365390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88185[] = {SGI365391, SGI365392, SGI365393, SGI365394, SGI365395, SGI0, SGI365394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88186[] = {SGI365396, SGI365397, SGI365398, SGI0, SGI365399, SGI0, SGI365400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88187[] = {SGI365401, SGI365402, SGI365403, SGI0, SGI365404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88188[] = {SGI365405, SGI365406, SGI365407, SGI0, SGI365408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88189[] = {SGI365409, SGI365410, SGI365411, SGI0, SGI365412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88190[] = {SGI365413, SGI365414, SGI365415, SGI0, SGI365416, SGI0, SGI365417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88191[] = {SGI365418, SGI365419, SGI365420, SGI0, SGI365421, SGI0, SGI365422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88192[] = {SGI365423, SGI365424, SGI365425, SGI365426, SGI365427, SGI365428, SGI365429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88193[] = {SGI65024, SGI365430, SGI365431, SGI0, SGI65023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88194[] = {SGI365432, SGI365433, SGI365434, SGI365433, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88195[] = {SGI365435, SGI365436, SGI365437, SGI365436, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88196[] = {SGI365438, SGI365439, SGI365440, SGI365439, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88197[] = {SGI365441, SGI365442, SGI365443, SGI365442, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88198[] = {SGI365444, SGI365445, SGI365446, SGI365445, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88199[] = {SGI365447, SGI365448, SGI365449, SGI365448, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88200[] = {SGI365450, SGI365451, SGI365452, SGI365451, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88201[] = {SGI365453, SGI365454, SGI365455, SGI365454, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88202[] = {SGI365456, SGI365457, SGI365458, SGI365459, SGI365457, SGI365460, SGI365461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88203[] = {SGI365462, SGI365463, SGI365464, SGI365465, SGI365463, SGI365466, SGI365467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88204[] = {SGI365468, SGI365469, SGI365470, SGI365471, SGI365469, SGI365472, SGI365473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88205[] = {SGI365474, SGI365475, SGI365476, SGI365477, SGI365475, SGI365478, SGI365479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88206[] = {SGI365480, SGI365481, SGI365482, SGI365483, SGI365481, SGI365484, SGI365485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88207[] = {SGI365486, SGI365487, SGI365488, SGI365489, SGI365487, SGI365490, SGI365491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88208[] = {SGI365492, SGI365493, SGI365494, SGI365495, SGI365493, SGI365496, SGI365497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88209[] = {SGI365498, SGI365499, SGI365500, SGI365501, SGI365499, SGI365502, SGI365503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88210[] = {SGI365504, SGI365505, SGI365506, SGI365507, SGI365505, SGI365508, SGI365509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88211[] = {SGI365510, SGI365511, SGI365512, SGI365513, SGI365511, SGI365514, SGI365515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88212[] = {SGI365516, SGI365517, SGI365518, SGI365519, SGI365517, SGI365520, SGI365521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88213[] = {SGI365522, SGI365523, SGI365524, SGI365525, SGI365523, SGI365526, SGI365527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88214[] = {SGI365528, SGI365529, SGI365530, SGI365531, SGI365529, SGI365532, SGI365533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88215[] = {SGI365534, SGI365535, SGI365536, SGI365537, SGI365535, SGI365538, SGI365539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88216[] = {SGI365540, SGI365541, SGI365542, SGI365543, SGI365541, SGI365544, SGI365545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88217[] = {SGI365546, SGI365547, SGI365548, SGI365549, SGI365547, SGI365550, SGI365551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88218[] = {SGI365552, SGI365553, SGI365554, SGI365555, SGI365553, SGI365556, SGI365557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88219[] = {SGI365558, SGI365559, SGI365560, SGI365561, SGI365559, SGI365562, SGI365563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88220[] = {SGI365564, SGI365565, SGI365566, SGI365567, SGI365565, SGI365568, SGI365569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88221[] = {SGI365570, SGI365571, SGI365572, SGI365573, SGI365571, SGI365574, SGI365575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88222[] = {SGI365576, SGI365577, SGI365578, SGI365579, SGI365577, SGI365580, SGI365581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88223[] = {SGI365582, SGI365583, SGI365584, SGI365585, SGI365583, SGI365586, SGI365587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88224[] = {SGI365588, SGI365589, SGI365590, SGI365591, SGI365589, SGI365592, SGI365593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88225[] = {SGI365594, SGI365595, SGI365596, SGI365597, SGI365595, SGI365598, SGI365599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88226[] = {SGI365600, SGI365601, SGI365602, SGI365603, SGI365601, SGI365604, SGI365605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88227[] = {SGI365606, SGI365607, SGI365608, SGI365609, SGI365607, SGI365610, SGI365611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88228[] = {SGI365612, SGI365613, SGI365614, SGI365615, SGI365613, SGI365616, SGI365617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88229[] = {SGI365618, SGI365619, SGI365620, SGI365621, SGI365619, SGI365622, SGI365623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88230[] = {SGI365624, SGI365625, SGI365626, SGI365627, SGI365625, SGI365628, SGI365629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88231[] = {SGI365630, SGI365631, SGI365632, SGI365633, SGI365631, SGI365634, SGI365635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88232[] = {SGI365636, SGI365637, SGI365638, SGI365639, SGI365637, SGI365640, SGI365641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88233[] = {SGI365642, SGI365643, SGI365644, SGI365645, SGI365643, SGI365646, SGI365647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88234[] = {SGI365648, SGI365649, SGI365650, SGI365651, SGI365649, SGI365652, SGI365653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88235[] = {SGI365654, SGI365655, SGI365656, SGI365657, SGI365655, SGI365658, SGI365659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88236[] = {SGI365660, SGI365661, SGI365662, SGI365663, SGI365661, SGI365664, SGI365665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88237[] = {SGI365666, SGI365667, SGI365668, SGI365669, SGI365667, SGI365670, SGI365671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88238[] = {SGI365672, SGI365673, SGI365674, SGI365675, SGI365673, SGI365676, SGI365677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88239[] = {SGI365678, SGI365679, SGI365680, SGI365681, SGI365679, SGI365682, SGI365683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88240[] = {SGI365684, SGI365685, SGI365686, SGI365687, SGI365685, SGI365688, SGI365689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88241[] = {SGI365690, SGI365691, SGI365692, SGI365693, SGI365691, SGI365694, SGI365695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88242[] = {SGI365696, SGI365697, SGI365698, SGI365699, SGI365697, SGI365700, SGI365701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88243[] = {SGI365702, SGI365703, SGI365704, SGI365705, SGI365703, SGI365706, SGI365707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88244[] = {SGI365708, SGI365709, SGI365710, SGI365711, SGI365709, SGI365712, SGI365713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88245[] = {SGI365714, SGI365715, SGI365716, SGI365717, SGI365715, SGI365718, SGI365719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88246[] = {SGI365720, SGI365721, SGI365722, SGI365723, SGI365721, SGI365724, SGI365725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88247[] = {SGI365726, SGI365727, SGI365728, SGI365729, SGI365727, SGI365730, SGI365731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88248[] = {SGI365732, SGI365733, SGI365734, SGI365735, SGI365733, SGI365736, SGI365737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88249[] = {SGI365738, SGI365739, SGI365740, SGI365741, SGI365739, SGI365742, SGI365743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88250[] = {SGI365744, SGI365745, SGI365746, SGI365747, SGI365745, SGI365748, SGI365749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88251[] = {SGI365750, SGI365751, SGI365752, SGI365753, SGI365751, SGI365754, SGI365755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88252[] = {SGI365756, SGI365757, SGI365758, SGI365759, SGI365757, SGI365760, SGI365761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88253[] = {SGI365762, SGI365763, SGI365764, SGI365765, SGI365763, SGI365766, SGI365767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88254[] = {SGI365768, SGI365769, SGI365770, SGI365771, SGI365769, SGI365772, SGI365773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88255[] = {SGI365774, SGI365775, SGI365776, SGI365777, SGI365775, SGI365778, SGI365779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88256[] = {SGI365780, SGI365781, SGI365782, SGI365783, SGI365781, SGI365784, SGI365785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88257[] = {SGI365786, SGI365787, SGI365788, SGI365789, SGI365787, SGI365790, SGI365791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88258[] = {SGI365792, SGI365793, SGI365794, SGI365795, SGI365793, SGI365796, SGI365797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88259[] = {SGI365798, SGI365799, SGI365800, SGI365801, SGI365799, SGI365802, SGI365803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88260[] = {SGI365804, SGI365805, SGI365806, SGI365807, SGI365805, SGI365808, SGI365809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88261[] = {SGI365810, SGI365811, SGI365812, SGI365813, SGI365811, SGI365814, SGI365815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88262[] = {SGI365816, SGI365817, SGI365818, SGI365819, SGI365817, SGI365820, SGI365821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88263[] = {SGI365822, SGI365823, SGI365824, SGI365825, SGI365823, SGI365826, SGI365827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88264[] = {SGI365828, SGI365829, SGI365830, SGI365831, SGI365829, SGI365832, SGI365833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88265[] = {SGI365834, SGI365835, SGI365836, SGI365837, SGI365835, SGI365838, SGI365839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88266[] = {SGI365840, SGI365841, SGI365842, SGI365843, SGI365841, SGI365844, SGI365845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88267[] = {SGI365846, SGI365847, SGI365848, SGI365849, SGI365847, SGI365850, SGI365851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88268[] = {SGI365852, SGI365853, SGI365854, SGI365855, SGI365853, SGI365856, SGI365857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88269[] = {SGI365858, SGI365859, SGI365860, SGI365861, SGI365859, SGI365862, SGI365863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88270[] = {SGI365864, SGI365865, SGI365866, SGI365867, SGI365865, SGI365868, SGI365869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88271[] = {SGI365870, SGI365871, SGI365872, SGI365873, SGI0, SGI365874, SGI365875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88272[] = {SGI365876, SGI365877, SGI365878, SGI365879, SGI365877, SGI365880, SGI365881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88273[] = {SGI365882, SGI365883, SGI365884, SGI365885, SGI365883, SGI365886, SGI365887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88274[] = {SGI365888, SGI365889, SGI365890, SGI365891, SGI365889, SGI365892, SGI365893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88275[] = {SGI365894, SGI365895, SGI365896, SGI365897, SGI365895, SGI365898, SGI365899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88276[] = {SGI365900, SGI365901, SGI365902, SGI365903, SGI365901, SGI365904, SGI365905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88277[] = {SGI365906, SGI365907, SGI365908, SGI365909, SGI365907, SGI365910, SGI365911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88278[] = {SGI365912, SGI365913, SGI365914, SGI365915, SGI365913, SGI365916, SGI365917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88279[] = {SGI365918, SGI365919, SGI365920, SGI365921, SGI365919, SGI365922, SGI365923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88280[] = {SGI365924, SGI365925, SGI365926, SGI365927, SGI365925, SGI365928, SGI365929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88281[] = {SGI365930, SGI365931, SGI365932, SGI365933, SGI365931, SGI365934, SGI365935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88282[] = {SGI365936, SGI365937, SGI365938, SGI365939, SGI365937, SGI365940, SGI365941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88283[] = {SGI365942, SGI365943, SGI365944, SGI365945, SGI365943, SGI365946, SGI365947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88284[] = {SGI365948, SGI365949, SGI365950, SGI365951, SGI365949, SGI365952, SGI365953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88285[] = {SGI365954, SGI365955, SGI365956, SGI365957, SGI365955, SGI365958, SGI365959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88286[] = {SGI365960, SGI365961, SGI365962, SGI365963, SGI365961, SGI365964, SGI365965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88287[] = {SGI365966, SGI365967, SGI365968, SGI365969, SGI365967, SGI365970, SGI365971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88288[] = {SGI365972, SGI365973, SGI365974, SGI365975, SGI365973, SGI365976, SGI365977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88289[] = {SGI365978, SGI365979, SGI365980, SGI365981, SGI365979, SGI365982, SGI365983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88290[] = {SGI365984, SGI365985, SGI365986, SGI365987, SGI365985, SGI365988, SGI365989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88291[] = {SGI365990, SGI365991, SGI365992, SGI365993, SGI365991, SGI365994, SGI365995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88292[] = {SGI365996, SGI365997, SGI365998, SGI365999, SGI365997, SGI366000, SGI366001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88293[] = {SGI366002, SGI366003, SGI366004, SGI366005, SGI366003, SGI366006, SGI366007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88294[] = {SGI366008, SGI366009, SGI366010, SGI366011, SGI366009, SGI366012, SGI366013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88295[] = {SGI366014, SGI366015, SGI366016, SGI366017, SGI366015, SGI366018, SGI366019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88296[] = {SGI366020, SGI366021, SGI366022, SGI366023, SGI366021, SGI366024, SGI366025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88297[] = {SGI366026, SGI366027, SGI366028, SGI366029, SGI366027, SGI366030, SGI366031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88298[] = {SGI366032, SGI366033, SGI366034, SGI366035, SGI366033, SGI366036, SGI366037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88299[] = {SGI366038, SGI366039, SGI366040, SGI366041, SGI366039, SGI366042, SGI366043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88300[] = {SGI366044, SGI366045, SGI366046, SGI366047, SGI366045, SGI366048, SGI366049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88301[] = {SGI366050, SGI366051, SGI366052, SGI366053, SGI366051, SGI366054, SGI366055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88302[] = {SGI366056, SGI366057, SGI366058, SGI366059, SGI366057, SGI366060, SGI366061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88303[] = {SGI366062, SGI366063, SGI366064, SGI366065, SGI366063, SGI366066, SGI366067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88304[] = {SGI366068, SGI366069, SGI366070, SGI366071, SGI366069, SGI366072, SGI366073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88305[] = {SGI366074, SGI366075, SGI366076, SGI366077, SGI366075, SGI366078, SGI366079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88306[] = {SGI366080, SGI366081, SGI366082, SGI366083, SGI366081, SGI366084, SGI366085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88307[] = {SGI366086, SGI366087, SGI366088, SGI366089, SGI366087, SGI366090, SGI366091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88308[] = {SGI366092, SGI366093, SGI366094, SGI366095, SGI366093, SGI366096, SGI366097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88309[] = {SGI366098, SGI366099, SGI366100, SGI366101, SGI366099, SGI366102, SGI366103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88310[] = {SGI366104, SGI366105, SGI366106, SGI366107, SGI366105, SGI366108, SGI366109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88311[] = {SGI366110, SGI366111, SGI366112, SGI366113, SGI366111, SGI366114, SGI366115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88312[] = {SGI366116, SGI366117, SGI366118, SGI366119, SGI366117, SGI366120, SGI366121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88313[] = {SGI366122, SGI366123, SGI366124, SGI366125, SGI366123, SGI366126, SGI366127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88314[] = {SGI366128, SGI366129, SGI366130, SGI366131, SGI366129, SGI366132, SGI366133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88315[] = {SGI366134, SGI366135, SGI366136, SGI366137, SGI366135, SGI366138, SGI366139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88316[] = {SGI366140, SGI366141, SGI366142, SGI366143, SGI366141, SGI366144, SGI366145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88317[] = {SGI366146, SGI366147, SGI366148, SGI366149, SGI366147, SGI366150, SGI366151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88318[] = {SGI366152, SGI366153, SGI366154, SGI366155, SGI366153, SGI366156, SGI366157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88319[] = {SGI366158, SGI366159, SGI366160, SGI366161, SGI366159, SGI366162, SGI366163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88320[] = {SGI366164, SGI366165, SGI366166, SGI366167, SGI366165, SGI366168, SGI366169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88321[] = {SGI366170, SGI366171, SGI366172, SGI366173, SGI366171, SGI366174, SGI366175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88322[] = {SGI366176, SGI366177, SGI366178, SGI366179, SGI366177, SGI366180, SGI366181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88323[] = {SGI366182, SGI366183, SGI366184, SGI366185, SGI366183, SGI366186, SGI366187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88324[] = {SGI366188, SGI366189, SGI366190, SGI366191, SGI366189, SGI366192, SGI366193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88325[] = {SGI366194, SGI366195, SGI366196, SGI366197, SGI366195, SGI366198, SGI366199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88326[] = {SGI366200, SGI366201, SGI366202, SGI366203, SGI366201, SGI366204, SGI366205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88327[] = {SGI366206, SGI366207, SGI366208, SGI366209, SGI366207, SGI366210, SGI366211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88328[] = {SGI366212, SGI366213, SGI366214, SGI366215, SGI0, SGI366216, SGI366217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88329[] = {SGI366218, SGI366219, SGI366220, SGI366221, SGI366219, SGI366222, SGI366223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88330[] = {SGI366224, SGI366225, SGI366226, SGI366227, SGI366225, SGI366228, SGI366229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88331[] = {SGI366230, SGI366231, SGI366232, SGI366233, SGI366231, SGI366234, SGI366235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88332[] = {SGI366236, SGI366237, SGI366238, SGI366239, SGI366237, SGI366240, SGI366241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88333[] = {SGI366242, SGI366243, SGI366244, SGI366245, SGI366243, SGI366246, SGI366247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88334[] = {SGI366248, SGI366249, SGI366250, SGI366251, SGI366249, SGI366252, SGI366253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88335[] = {SGI366254, SGI366255, SGI366256, SGI366257, SGI366255, SGI366258, SGI366259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88336[] = {SGI366260, SGI366261, SGI366262, SGI366263, SGI366261, SGI366264, SGI366265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88337[] = {SGI366266, SGI366267, SGI366268, SGI366269, SGI366267, SGI366270, SGI366271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88338[] = {SGI366272, SGI366273, SGI366274, SGI366275, SGI366273, SGI366276, SGI366277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88339[] = {SGI366278, SGI366279, SGI366280, SGI366281, SGI366279, SGI366282, SGI366283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88340[] = {SGI366284, SGI366285, SGI366286, SGI366287, SGI366285, SGI366288, SGI366289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88341[] = {SGI366290, SGI366291, SGI366292, SGI366293, SGI366291, SGI366294, SGI366295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88342[] = {SGI366296, SGI366297, SGI366298, SGI366299, SGI366297, SGI366300, SGI366301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88343[] = {SGI366302, SGI366303, SGI366304, SGI366305, SGI366303, SGI366306, SGI366307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88344[] = {SGI366308, SGI366309, SGI366310, SGI366311, SGI366309, SGI366312, SGI366313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88345[] = {SGI366314, SGI366315, SGI366316, SGI366317, SGI366315, SGI366318, SGI366319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88346[] = {SGI366320, SGI366321, SGI366322, SGI366323, SGI366321, SGI366324, SGI366325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88347[] = {SGI366326, SGI366327, SGI366328, SGI366329, SGI366327, SGI366330, SGI366331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88348[] = {SGI366332, SGI366333, SGI366334, SGI366335, SGI366333, SGI366336, SGI366337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88349[] = {SGI366338, SGI366339, SGI366340, SGI366341, SGI366339, SGI366342, SGI366343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88350[] = {SGI366344, SGI366345, SGI366346, SGI366347, SGI366345, SGI366348, SGI366349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88351[] = {SGI366350, SGI366351, SGI366352, SGI366353, SGI366351, SGI366354, SGI366355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88352[] = {SGI366356, SGI366357, SGI366358, SGI366359, SGI366357, SGI366360, SGI366361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88353[] = {SGI366362, SGI366363, SGI366364, SGI0, SGI0, SGI0, SGI366365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88354[] = {SGI366366, SGI366367, SGI366368, SGI0, SGI366367, SGI0, SGI366369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88355[] = {SGI366370, SGI366371, SGI366372, SGI0, SGI366371, SGI0, SGI366373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88356[] = {SGI366374, SGI366375, SGI366376, SGI0, SGI366375, SGI0, SGI366377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88357[] = {SGI366378, SGI366379, SGI366380, SGI0, SGI366379, SGI0, SGI366381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88358[] = {SGI366382, SGI366383, SGI366384, SGI0, SGI366383, SGI0, SGI366385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88359[] = {SGI366386, SGI366387, SGI366388, SGI0, SGI366387, SGI0, SGI366389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88360[] = {SGI366390, SGI366391, SGI366392, SGI0, SGI366391, SGI0, SGI366393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88361[] = {SGI366394, SGI366395, SGI366396, SGI366397, SGI366395, SGI366398, SGI366399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88362[] = {SGI366400, SGI366401, SGI366402, SGI366403, SGI366401, SGI366404, SGI366405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88363[] = {SGI366406, SGI366407, SGI366408, SGI366409, SGI366407, SGI366410, SGI366411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88364[] = {SGI366412, SGI366413, SGI366414, SGI366415, SGI366413, SGI366416, SGI366417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88365[] = {SGI366418, SGI366419, SGI366420, SGI366419, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88366[] = {SGI366421, SGI366422, SGI366423, SGI366424, SGI366422, SGI366425, SGI366426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88367[] = {SGI366427, SGI366428, SGI366429, SGI366430, SGI366428, SGI366431, SGI366432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88368[] = {SGI366433, SGI366435, SGI366437, SGI366435, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88369[] = {SGI366434, SGI366436, SGI366438, SGI366439, SGI366436, SGI366440, SGI366441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88370[] = {SGI366442, SGI366443, SGI366444, SGI366445, SGI366443, SGI366446, SGI366447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88371[] = {SGI366448, SGI366449, SGI366450, SGI366451, SGI366449, SGI366452, SGI366453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88372[] = {SGI366454, SGI366455, SGI366456, SGI366457, SGI366455, SGI366458, SGI366459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88373[] = {SGI366460, SGI366461, SGI366462, SGI366463, SGI366461, SGI366464, SGI366465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88374[] = {SGI366466, SGI366467, SGI366468, SGI366469, SGI366467, SGI366470, SGI366471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88375[] = {SGI366472, SGI366473, SGI366474, SGI366475, SGI366473, SGI366476, SGI366477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88376[] = {SGI366478, SGI366479, SGI366480, SGI366481, SGI0, SGI366482, SGI366483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88377[] = {SGI366484, SGI366485, SGI366486, SGI366487, SGI366485, SGI366488, SGI366489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88378[] = {SGI366490, SGI366491, SGI366492, SGI366493, SGI366491, SGI366494, SGI366495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88379[] = {SGI366496, SGI366497, SGI366498, SGI366499, SGI366497, SGI366500, SGI366501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88380[] = {SGI366502, SGI366503, SGI366504, SGI366505, SGI366503, SGI366506, SGI366507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88381[] = {SGI366508, SGI366509, SGI366511, SGI366509, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88382[] = {SGI366510, SGI366512, SGI366513, SGI366514, SGI366512, SGI366515, SGI366516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88383[] = {SGI366517, SGI366518, SGI366519, SGI366520, SGI0, SGI366521, SGI366522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88384[] = {SGI366526, SGI366528, SGI366529, SGI366528, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88385[] = {SGI366523, SGI366524, SGI366525, SGI366527, SGI366524, SGI366530, SGI366531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88386[] = {SGI366532, SGI366533, SGI366534, SGI366535, SGI366533, SGI366536, SGI366537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88387[] = {SGI366538, SGI366539, SGI366540, SGI366541, SGI366539, SGI366542, SGI366543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88388[] = {SGI366544, SGI366545, SGI366546, SGI366547, SGI366545, SGI366548, SGI366549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88389[] = {SGI366550, SGI366551, SGI366552, SGI366553, SGI366551, SGI366554, SGI366555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88390[] = {SGI366556, SGI366557, SGI366558, SGI366559, SGI366557, SGI366560, SGI366561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88391[] = {SGI366562, SGI366563, SGI366564, SGI366565, SGI366563, SGI366566, SGI366567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88392[] = {SGI366568, SGI366569, SGI366570, SGI366571, SGI366569, SGI366572, SGI366573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88393[] = {SGI366574, SGI366575, SGI366576, SGI366577, SGI366575, SGI366578, SGI366579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88394[] = {SGI366580, SGI366581, SGI366582, SGI366583, SGI366581, SGI366584, SGI366585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88395[] = {SGI366586, SGI366587, SGI366588, SGI366589, SGI366587, SGI366590, SGI366591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88396[] = {SGI366592, SGI366593, SGI366594, SGI366595, SGI366593, SGI366596, SGI366597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88397[] = {SGI366598, SGI366599, SGI366600, SGI366601, SGI366599, SGI366602, SGI366603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88398[] = {SGI366604, SGI366605, SGI366606, SGI366607, SGI366605, SGI366608, SGI366609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88399[] = {SGI366610, SGI366611, SGI366612, SGI366613, SGI0, SGI366614, SGI366615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88400[] = {SGI366616, SGI366617, SGI366618, SGI366619, SGI366617, SGI366620, SGI366621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88401[] = {SGI366622, SGI366623, SGI366624, SGI366625, SGI366623, SGI366626, SGI366627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88402[] = {SGI366628, SGI366629, SGI366630, SGI366631, SGI366629, SGI366632, SGI366633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88403[] = {SGI366634, SGI366635, SGI366636, SGI366637, SGI366635, SGI366638, SGI366639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88404[] = {SGI366640, SGI366641, SGI366642, SGI366643, SGI366641, SGI366644, SGI366645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88405[] = {SGI366646, SGI366647, SGI366648, SGI366649, SGI366647, SGI366650, SGI366651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88406[] = {SGI366652, SGI366653, SGI366654, SGI366655, SGI366653, SGI366656, SGI366657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88407[] = {SGI366658, SGI366659, SGI366660, SGI366661, SGI366659, SGI366662, SGI366663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88408[] = {SGI366664, SGI366665, SGI366666, SGI366667, SGI366665, SGI366668, SGI366669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88409[] = {SGI366670, SGI366671, SGI366672, SGI366673, SGI366671, SGI366674, SGI366675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88410[] = {SGI366676, SGI366677, SGI366678, SGI366679, SGI366677, SGI366680, SGI366681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88411[] = {SGI366682, SGI366683, SGI366684, SGI366685, SGI366683, SGI366686, SGI366687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88412[] = {SGI366688, SGI366689, SGI366690, SGI366691, SGI366689, SGI366692, SGI366693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88413[] = {SGI366694, SGI366695, SGI366696, SGI366697, SGI366695, SGI366698, SGI366699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88414[] = {SGI366700, SGI366701, SGI366702, SGI366703, SGI366701, SGI366704, SGI366705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88415[] = {SGI366706, SGI366707, SGI366708, SGI366709, SGI366707, SGI366710, SGI366711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88416[] = {SGI366712, SGI366713, SGI366714, SGI366715, SGI366713, SGI366716, SGI366717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88417[] = {SGI366718, SGI366719, SGI366720, SGI366721, SGI366719, SGI366722, SGI366723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88418[] = {SGI366724, SGI366725, SGI366726, SGI366727, SGI366725, SGI366728, SGI366729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88419[] = {SGI366730, SGI366731, SGI366732, SGI366733, SGI366731, SGI366734, SGI366735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88420[] = {SGI366736, SGI366737, SGI366738, SGI366739, SGI366737, SGI366740, SGI366741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88421[] = {SGI366742, SGI366743, SGI366744, SGI366745, SGI366743, SGI366746, SGI366747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88422[] = {SGI366748, SGI366749, SGI366750, SGI366751, SGI366749, SGI366752, SGI366753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88423[] = {SGI366754, SGI366755, SGI366756, SGI366757, SGI366755, SGI366758, SGI366759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88424[] = {SGI366760, SGI366761, SGI366762, SGI366763, SGI366761, SGI366764, SGI366765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88425[] = {SGI366766, SGI366767, SGI366768, SGI366769, SGI366767, SGI366770, SGI366771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88426[] = {SGI366772, SGI366773, SGI366774, SGI366775, SGI366773, SGI366776, SGI366777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88427[] = {SGI366778, SGI366779, SGI366780, SGI366781, SGI366779, SGI366782, SGI366783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88428[] = {SGI366784, SGI366785, SGI366786, SGI366787, SGI366785, SGI366788, SGI366789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88429[] = {SGI366790, SGI366791, SGI366792, SGI366793, SGI366791, SGI366794, SGI366795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88430[] = {SGI366796, SGI366797, SGI366798, SGI366799, SGI366797, SGI366800, SGI366801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88431[] = {SGI366802, SGI366803, SGI366804, SGI366805, SGI366803, SGI366806, SGI366807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88432[] = {SGI366808, SGI366809, SGI366810, SGI366811, SGI366809, SGI366812, SGI366813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88433[] = {SGI366814, SGI366815, SGI366816, SGI366817, SGI366815, SGI366818, SGI366819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88434[] = {SGI366820, SGI366821, SGI366822, SGI366823, SGI366821, SGI366824, SGI366825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88435[] = {SGI366826, SGI366827, SGI366828, SGI366829, SGI0, SGI366830, SGI366831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88436[] = {SGI366832, SGI366833, SGI366834, SGI366835, SGI366833, SGI366836, SGI366837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88437[] = {SGI366838, SGI366839, SGI366840, SGI366841, SGI366839, SGI366842, SGI366843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88438[] = {SGI366844, SGI366845, SGI366846, SGI366847, SGI366845, SGI366848, SGI366849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88439[] = {SGI366850, SGI366851, SGI366852, SGI366853, SGI366851, SGI366854, SGI366855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88440[] = {SGI366856, SGI366857, SGI366858, SGI366859, SGI366857, SGI366860, SGI366861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88441[] = {SGI366862, SGI366863, SGI366864, SGI366865, SGI366863, SGI366866, SGI366867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88442[] = {SGI366868, SGI366869, SGI366870, SGI366871, SGI366869, SGI366872, SGI366873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88443[] = {SGI366874, SGI366875, SGI366876, SGI366877, SGI366875, SGI366878, SGI366879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88444[] = {SGI366880, SGI366881, SGI366882, SGI366883, SGI366881, SGI366884, SGI366885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88445[] = {SGI366886, SGI366887, SGI366888, SGI366889, SGI366887, SGI366890, SGI366891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88446[] = {SGI366892, SGI366893, SGI366894, SGI366895, SGI366893, SGI366896, SGI366897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88447[] = {SGI366898, SGI366899, SGI366900, SGI366901, SGI366899, SGI366902, SGI366903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88448[] = {SGI366904, SGI366905, SGI366906, SGI366907, SGI366905, SGI366908, SGI366909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88449[] = {SGI366910, SGI366911, SGI366912, SGI366913, SGI366911, SGI366914, SGI366915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88450[] = {SGI366916, SGI366917, SGI366918, SGI366919, SGI366917, SGI366920, SGI366921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88451[] = {SGI366922, SGI366923, SGI366924, SGI366925, SGI366923, SGI366926, SGI366927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88452[] = {SGI366928, SGI366929, SGI366930, SGI366931, SGI366929, SGI366932, SGI366933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88453[] = {SGI366934, SGI366935, SGI366936, SGI366937, SGI366935, SGI366938, SGI366939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88454[] = {SGI366940, SGI366941, SGI366942, SGI366943, SGI366941, SGI366944, SGI366945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88455[] = {SGI366946, SGI366947, SGI366948, SGI366949, SGI366947, SGI366950, SGI366951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88456[] = {SGI366952, SGI366953, SGI366954, SGI366955, SGI366953, SGI366956, SGI366957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88457[] = {SGI366958, SGI366959, SGI366960, SGI366961, SGI366959, SGI366962, SGI366963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88458[] = {SGI366964, SGI366965, SGI366966, SGI366967, SGI366965, SGI366968, SGI366969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88459[] = {SGI366970, SGI366971, SGI366972, SGI366973, SGI366971, SGI366974, SGI366975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88460[] = {SGI366976, SGI366977, SGI366978, SGI366979, SGI366977, SGI366980, SGI366981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88461[] = {SGI366982, SGI366983, SGI366984, SGI366985, SGI366983, SGI366986, SGI366987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88462[] = {SGI366988, SGI366989, SGI366990, SGI366991, SGI366989, SGI366992, SGI366993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88463[] = {SGI366994, SGI366995, SGI366996, SGI366997, SGI366995, SGI366998, SGI366999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88464[] = {SGI367000, SGI367001, SGI367002, SGI367003, SGI367001, SGI367004, SGI367005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88465[] = {SGI367006, SGI367007, SGI367008, SGI367009, SGI367007, SGI367010, SGI367011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88466[] = {SGI367012, SGI367013, SGI367014, SGI367015, SGI367013, SGI367016, SGI367017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88467[] = {SGI367018, SGI367019, SGI367020, SGI367021, SGI367019, SGI367022, SGI367023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88468[] = {SGI367024, SGI367025, SGI367026, SGI367027, SGI367025, SGI367028, SGI367029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88469[] = {SGI367030, SGI367031, SGI367032, SGI367033, SGI367031, SGI367034, SGI367035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88470[] = {SGI367036, SGI367037, SGI367038, SGI367039, SGI367037, SGI367040, SGI367041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88471[] = {SGI367042, SGI367043, SGI367044, SGI367045, SGI367043, SGI367046, SGI367047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88472[] = {SGI367048, SGI367049, SGI367050, SGI367051, SGI367049, SGI367052, SGI367053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88473[] = {SGI367054, SGI367055, SGI367056, SGI367057, SGI367055, SGI367058, SGI367059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88474[] = {SGI367060, SGI367061, SGI367062, SGI367063, SGI367061, SGI367064, SGI367065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88475[] = {SGI367066, SGI367067, SGI367068, SGI367069, SGI367067, SGI367070, SGI367071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88476[] = {SGI367072, SGI367073, SGI367074, SGI367075, SGI367073, SGI367076, SGI367077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88477[] = {SGI367078, SGI367079, SGI367080, SGI367081, SGI367079, SGI367082, SGI367083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88478[] = {SGI367084, SGI367085, SGI367086, SGI367087, SGI367085, SGI367088, SGI367089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88479[] = {SGI367090, SGI367091, SGI367092, SGI367093, SGI367091, SGI367094, SGI367095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88480[] = {SGI367096, SGI367097, SGI367098, SGI367099, SGI367097, SGI367100, SGI367101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88481[] = {SGI367102, SGI367103, SGI367104, SGI367105, SGI367103, SGI367106, SGI367107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88482[] = {SGI367108, SGI367109, SGI367110, SGI367111, SGI367109, SGI367112, SGI367113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88483[] = {SGI367114, SGI367115, SGI367116, SGI367117, SGI367115, SGI367118, SGI367119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88484[] = {SGI367120, SGI367121, SGI367122, SGI367123, SGI367121, SGI367124, SGI367125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88485[] = {SGI367126, SGI367127, SGI367128, SGI367129, SGI367127, SGI367130, SGI367131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88486[] = {SGI367132, SGI367133, SGI367134, SGI367135, SGI367133, SGI367136, SGI367137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88487[] = {SGI367138, SGI367139, SGI367140, SGI367141, SGI367139, SGI367142, SGI367143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88488[] = {SGI367144, SGI367145, SGI367146, SGI367147, SGI367145, SGI367148, SGI367149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88489[] = {SGI367150, SGI367151, SGI367152, SGI367153, SGI367151, SGI367154, SGI367155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88490[] = {SGI367156, SGI367157, SGI367158, SGI367159, SGI367157, SGI367160, SGI367161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88491[] = {SGI367162, SGI367163, SGI367164, SGI367165, SGI367163, SGI367166, SGI367167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88492[] = {SGI367168, SGI367169, SGI367170, SGI367171, SGI367169, SGI367172, SGI367173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88493[] = {SGI367174, SGI367175, SGI367176, SGI367177, SGI367175, SGI367178, SGI367179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88494[] = {SGI367180, SGI367181, SGI367182, SGI367183, SGI367181, SGI367184, SGI367185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88495[] = {SGI367186, SGI367187, SGI367188, SGI367189, SGI367187, SGI367190, SGI367191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88496[] = {SGI367192, SGI367193, SGI367194, SGI367195, SGI367193, SGI367196, SGI367197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88497[] = {SGI367198, SGI367199, SGI367200, SGI367201, SGI367199, SGI367202, SGI367203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88498[] = {SGI367204, SGI367205, SGI367206, SGI367207, SGI367205, SGI367208, SGI367209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88499[] = {SGI367210, SGI367211, SGI367212, SGI367213, SGI367211, SGI367214, SGI367215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88500[] = {SGI367216, SGI367217, SGI367218, SGI367219, SGI367217, SGI367220, SGI367221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88501[] = {SGI367222, SGI367223, SGI367224, SGI367225, SGI367223, SGI367226, SGI367227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88502[] = {SGI367228, SGI367229, SGI367230, SGI367231, SGI367229, SGI367232, SGI367233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88503[] = {SGI367234, SGI367235, SGI367236, SGI367237, SGI367235, SGI367238, SGI367239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88504[] = {SGI367240, SGI367241, SGI367242, SGI367243, SGI367241, SGI367244, SGI367245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88505[] = {SGI367246, SGI367247, SGI367248, SGI367249, SGI367247, SGI367250, SGI367251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88506[] = {SGI367252, SGI367253, SGI367254, SGI0, SGI367253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88507[] = {SGI367255, SGI367256, SGI367257, SGI0, SGI367258, SGI0, SGI367259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88508[] = {SGI367260, SGI367261, SGI367262, SGI0, SGI367263, SGI0, SGI367264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88509[] = {SGI367265, SGI367266, SGI367267, SGI0, SGI367268, SGI0, SGI367269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88510[] = {SGI367270, SGI367271, SGI367272, SGI0, SGI0, SGI0, SGI367273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88511[] = {SGI367274, SGI367275, SGI367276, SGI0, SGI367277, SGI0, SGI367278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88512[] = {SGI367279, SGI367280, SGI367281, SGI0, SGI0, SGI0, SGI367282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88513[] = {SGI367283, SGI367284, SGI367285, SGI0, SGI367286, SGI0, SGI367287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88514[] = {SGI367288, SGI367289, SGI367290, SGI0, SGI367291, SGI0, SGI367292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88515[] = {SGI367293, SGI367294, SGI367295, SGI0, SGI367296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88516[] = {SGI367297, SGI367298, SGI367299, SGI0, SGI367300, SGI0, SGI367301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88517[] = {SGI367302, SGI367303, SGI367304, SGI0, SGI0, SGI0, SGI367305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88518[] = {SGI367306, SGI367307, SGI367308, SGI0, SGI367309, SGI0, SGI367310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88519[] = {SGI367311, SGI367312, SGI367313, SGI0, SGI0, SGI0, SGI367314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88520[] = {SGI367315, SGI367316, SGI367317, SGI0, SGI367318, SGI0, SGI367319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88521[] = {SGI367320, SGI367321, SGI367322, SGI0, SGI0, SGI0, SGI367323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88522[] = {SGI367324, SGI367325, SGI367326, SGI367327, SGI367328, SGI367329, SGI367330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88523[] = {SGI367331, SGI367332, SGI367333, SGI367334, SGI367335, SGI367336, SGI367337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88524[] = {SGI367338, SGI367339, SGI367340, SGI367341, SGI367342, SGI367343, SGI367344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88525[] = {SGI367345, SGI367346, SGI367347, SGI367348, SGI367349, SGI367350, SGI367351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88526[] = {SGI367352, SGI367353, SGI367354, SGI367355, SGI367356, SGI367357, SGI367358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88527[] = {SGI367359, SGI367360, SGI367361, SGI367362, SGI367363, SGI367364, SGI367365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88528[] = {SGI367366, SGI367367, SGI367368, SGI367369, SGI367370, SGI367371, SGI367372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88529[] = {SGI367373, SGI367374, SGI367375, SGI367376, SGI367377, SGI367378, SGI367379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88530[] = {SGI367380, SGI367381, SGI367382, SGI367383, SGI367384, SGI367385, SGI367386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88531[] = {SGI367387, SGI367388, SGI367389, SGI367390, SGI367391, SGI367392, SGI367393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88532[] = {SGI367394, SGI367395, SGI367396, SGI367397, SGI367398, SGI367399, SGI367400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88533[] = {SGI367401, SGI367402, SGI367403, SGI367404, SGI367405, SGI367406, SGI367407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88534[] = {SGI367408, SGI367409, SGI367410, SGI367411, SGI367412, SGI367413, SGI367414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88535[] = {SGI367415, SGI367416, SGI367417, SGI367418, SGI367419, SGI367420, SGI367421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88536[] = {SGI367422, SGI367423, SGI367424, SGI367425, SGI367426, SGI367427, SGI367428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88537[] = {SGI367429, SGI367430, SGI367431, SGI367432, SGI367433, SGI367434, SGI367435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88538[] = {SGI367436, SGI367437, SGI367438, SGI367439, SGI367440, SGI367441, SGI367442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88539[] = {SGI367443, SGI367444, SGI367445, SGI367446, SGI367447, SGI367448, SGI367449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88540[] = {SGI367450, SGI367451, SGI367452, SGI367453, SGI367454, SGI367455, SGI367456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88541[] = {SGI367457, SGI367458, SGI367459, SGI367460, SGI367461, SGI367462, SGI367463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88542[] = {SGI367464, SGI367465, SGI367466, SGI367467, SGI367468, SGI367469, SGI367470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88543[] = {SGI367471, SGI367472, SGI367473, SGI367474, SGI367475, SGI367476, SGI367477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88544[] = {SGI367478, SGI367479, SGI367480, SGI367481, SGI367482, SGI367483, SGI367484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88545[] = {SGI367485, SGI367486, SGI367487, SGI367488, SGI367489, SGI367490, SGI367491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88546[] = {SGI367492, SGI367493, SGI367494, SGI367495, SGI367496, SGI367497, SGI367498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88547[] = {SGI367499, SGI367500, SGI367501, SGI367502, SGI367503, SGI367504, SGI367505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88548[] = {SGI367506, SGI367507, SGI367508, SGI367509, SGI367510, SGI367511, SGI367512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88549[] = {SGI367513, SGI367514, SGI367515, SGI367516, SGI367517, SGI367518, SGI367519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88550[] = {SGI367520, SGI367521, SGI367522, SGI367523, SGI367524, SGI367525, SGI367526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88551[] = {SGI367527, SGI367528, SGI367529, SGI367530, SGI367531, SGI367532, SGI367533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88552[] = {SGI367534, SGI367535, SGI367536, SGI367537, SGI367538, SGI367539, SGI367540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88553[] = {SGI367541, SGI367542, SGI367543, SGI367544, SGI367545, SGI367546, SGI367547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88554[] = {SGI367548, SGI367549, SGI367550, SGI367551, SGI367552, SGI367553, SGI367554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88555[] = {SGI367555, SGI367556, SGI367557, SGI367558, SGI367559, SGI367560, SGI367561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88556[] = {SGI367562, SGI367563, SGI367564, SGI367565, SGI367566, SGI367567, SGI367568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88557[] = {SGI367569, SGI367570, SGI367571, SGI367572, SGI367573, SGI367574, SGI367575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88558[] = {SGI367576, SGI367577, SGI367578, SGI367579, SGI367580, SGI367581, SGI367582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88559[] = {SGI367583, SGI367584, SGI367585, SGI367586, SGI367587, SGI367588, SGI367589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88560[] = {SGI367590, SGI367591, SGI367592, SGI367593, SGI367594, SGI367595, SGI367596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88561[] = {SGI367597, SGI367598, SGI367599, SGI367600, SGI367601, SGI367602, SGI367603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88562[] = {SGI367604, SGI367605, SGI367606, SGI367607, SGI367608, SGI367609, SGI367610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88563[] = {SGI367611, SGI367612, SGI367613, SGI367614, SGI367615, SGI367616, SGI367617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88564[] = {SGI367618, SGI367619, SGI367620, SGI367621, SGI367622, SGI367623, SGI367624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88565[] = {SGI367625, SGI367626, SGI367627, SGI367628, SGI367629, SGI367630, SGI367631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88566[] = {SGI367632, SGI367633, SGI367634, SGI367635, SGI367636, SGI367637, SGI367638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88567[] = {SGI367639, SGI367640, SGI367641, SGI367642, SGI367643, SGI367644, SGI367645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88568[] = {SGI367646, SGI367647, SGI367648, SGI367649, SGI367650, SGI367651, SGI367652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88569[] = {SGI367653, SGI367654, SGI367655, SGI367656, SGI367657, SGI367658, SGI367659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88570[] = {SGI367660, SGI367661, SGI367662, SGI367663, SGI367664, SGI367665, SGI367666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88571[] = {SGI367667, SGI367668, SGI367669, SGI367670, SGI367671, SGI367672, SGI367673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88572[] = {SGI367674, SGI367675, SGI367676, SGI367677, SGI0, SGI367678, SGI367679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88573[] = {SGI367680, SGI367681, SGI367682, SGI367683, SGI367684, SGI367685, SGI367686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88574[] = {SGI367687, SGI367688, SGI367689, SGI367690, SGI367691, SGI367692, SGI367693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88575[] = {SGI367694, SGI367695, SGI367696, SGI367697, SGI367698, SGI367699, SGI367700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88576[] = {SGI367701, SGI367702, SGI367703, SGI367704, SGI367705, SGI367706, SGI367707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88577[] = {SGI367708, SGI367709, SGI367710, SGI367711, SGI367712, SGI367713, SGI367714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88578[] = {SGI367715, SGI367716, SGI367717, SGI367718, SGI367719, SGI367720, SGI367721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88579[] = {SGI367722, SGI367723, SGI367724, SGI367725, SGI367726, SGI367727, SGI367728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88580[] = {SGI367729, SGI367730, SGI367731, SGI367732, SGI367733, SGI367734, SGI367735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88581[] = {SGI367736, SGI367737, SGI367738, SGI367739, SGI367740, SGI367741, SGI367742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88582[] = {SGI367743, SGI367744, SGI367745, SGI367746, SGI367747, SGI367748, SGI367749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88583[] = {SGI367750, SGI367751, SGI367752, SGI367753, SGI367754, SGI367755, SGI367756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88584[] = {SGI367757, SGI367758, SGI367759, SGI367760, SGI367761, SGI367762, SGI367763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88585[] = {SGI367764, SGI367765, SGI367766, SGI367767, SGI367768, SGI367769, SGI367770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88586[] = {SGI367771, SGI367772, SGI367773, SGI367774, SGI367775, SGI367776, SGI367777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88587[] = {SGI367778, SGI367779, SGI367780, SGI367781, SGI367782, SGI367783, SGI367784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88588[] = {SGI367785, SGI367786, SGI367787, SGI367788, SGI367789, SGI367790, SGI367791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88589[] = {SGI367792, SGI367793, SGI367794, SGI367795, SGI367796, SGI367797, SGI367798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88590[] = {SGI367799, SGI367800, SGI367801, SGI367802, SGI367803, SGI367804, SGI367805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88591[] = {SGI367806, SGI367807, SGI367808, SGI367809, SGI367810, SGI367811, SGI367812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88592[] = {SGI367813, SGI367814, SGI367815, SGI367816, SGI367817, SGI367818, SGI367819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88593[] = {SGI367820, SGI367821, SGI367822, SGI367823, SGI367824, SGI367825, SGI367826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88594[] = {SGI367827, SGI367828, SGI367829, SGI367830, SGI367831, SGI367832, SGI367833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88595[] = {SGI367834, SGI367835, SGI367836, SGI367837, SGI367838, SGI367839, SGI367840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88596[] = {SGI367841, SGI367842, SGI367843, SGI367844, SGI367845, SGI367846, SGI367847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88597[] = {SGI367848, SGI367849, SGI367850, SGI367851, SGI367852, SGI367853, SGI367854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88598[] = {SGI367855, SGI367856, SGI367857, SGI367858, SGI367859, SGI367860, SGI367861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88599[] = {SGI367862, SGI367863, SGI367864, SGI367865, SGI367866, SGI367867, SGI367868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88600[] = {SGI367869, SGI367870, SGI367871, SGI367872, SGI367873, SGI367874, SGI367875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88601[] = {SGI367876, SGI367877, SGI367878, SGI367879, SGI367880, SGI367881, SGI367882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88602[] = {SGI367883, SGI367884, SGI367885, SGI367886, SGI367887, SGI367888, SGI367889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88603[] = {SGI367890, SGI367891, SGI367892, SGI367893, SGI367894, SGI367895, SGI367896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88604[] = {SGI367897, SGI367898, SGI367899, SGI367900, SGI367901, SGI367902, SGI367903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88605[] = {SGI367904, SGI367905, SGI367906, SGI367907, SGI367908, SGI367909, SGI367910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88606[] = {SGI367911, SGI367912, SGI367913, SGI367914, SGI367915, SGI367916, SGI367917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88607[] = {SGI367918, SGI367919, SGI367920, SGI367921, SGI367922, SGI367923, SGI367924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88608[] = {SGI367925, SGI367926, SGI367927, SGI367928, SGI367929, SGI367930, SGI367931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88609[] = {SGI367932, SGI367933, SGI367934, SGI367935, SGI367936, SGI367937, SGI367938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88610[] = {SGI367939, SGI367940, SGI367941, SGI0, SGI367942, SGI0, SGI367943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88611[] = {SGI367944, SGI367945, SGI367946, SGI0, SGI367947, SGI0, SGI367948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88612[] = {SGI367949, SGI367950, SGI367951, SGI0, SGI367952, SGI0, SGI367953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88613[] = {SGI367954, SGI367955, SGI367956, SGI0, SGI367957, SGI0, SGI367958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88614[] = {SGI367959, SGI367960, SGI367961, SGI0, SGI367962, SGI0, SGI367963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88615[] = {SGI367964, SGI367965, SGI367966, SGI0, SGI367967, SGI0, SGI367968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88616[] = {SGI367969, SGI367970, SGI367971, SGI367972, SGI367973, SGI367974, SGI367975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88617[] = {SGI367976, SGI367977, SGI367978, SGI367979, SGI367980, SGI367981, SGI367982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88618[] = {SGI367983, SGI367984, SGI367985, SGI367986, SGI367987, SGI367988, SGI367989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88619[] = {SGI367990, SGI367991, SGI367992, SGI367993, SGI367994, SGI367995, SGI367996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88620[] = {SGI367997, SGI367998, SGI367999, SGI368000, SGI368001, SGI368002, SGI368003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88621[] = {SGI368004, SGI368005, SGI368006, SGI368007, SGI368008, SGI368009, SGI368010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88622[] = {SGI368011, SGI368012, SGI368013, SGI368014, SGI368015, SGI368016, SGI368017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88623[] = {SGI368018, SGI368019, SGI368020, SGI368021, SGI368022, SGI368023, SGI368024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88624[] = {SGI368025, SGI368026, SGI368027, SGI368028, SGI368029, SGI368030, SGI368031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88625[] = {SGI368032, SGI368033, SGI368034, SGI368035, SGI368036, SGI368037, SGI368038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88626[] = {SGI368039, SGI368040, SGI368041, SGI368042, SGI368043, SGI368044, SGI368045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88627[] = {SGI368046, SGI368047, SGI368048, SGI368049, SGI368050, SGI368051, SGI368052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88628[] = {SGI368053, SGI368054, SGI368055, SGI368056, SGI368057, SGI368058, SGI368059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88629[] = {SGI368060, SGI368061, SGI368062, SGI368063, SGI368064, SGI368065, SGI368066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88630[] = {SGI368067, SGI368068, SGI368069, SGI368070, SGI368071, SGI368072, SGI368073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88631[] = {SGI368074, SGI368075, SGI368076, SGI368077, SGI368078, SGI368079, SGI368080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88632[] = {SGI368081, SGI368082, SGI368083, SGI368084, SGI368085, SGI368086, SGI368087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88633[] = {SGI368088, SGI368089, SGI368090, SGI368091, SGI368092, SGI368093, SGI368094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88634[] = {SGI368095, SGI368096, SGI368097, SGI368098, SGI368099, SGI368100, SGI368101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88635[] = {SGI368102, SGI368103, SGI368104, SGI368105, SGI368106, SGI368107, SGI368108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88636[] = {SGI368109, SGI368110, SGI368111, SGI368112, SGI368113, SGI368114, SGI368115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88637[] = {SGI368116, SGI368117, SGI368118, SGI368119, SGI368120, SGI368121, SGI368122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88638[] = {SGI368123, SGI368124, SGI368125, SGI368126, SGI368127, SGI368128, SGI368129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88639[] = {SGI368130, SGI368131, SGI368132, SGI368133, SGI368134, SGI368135, SGI368136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88640[] = {SGI368137, SGI368138, SGI368139, SGI368140, SGI368141, SGI368142, SGI368143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88641[] = {SGI368144, SGI368145, SGI368146, SGI368147, SGI368148, SGI368149, SGI368150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88642[] = {SGI368151, SGI368152, SGI368153, SGI368154, SGI368155, SGI368156, SGI368157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88643[] = {SGI368158, SGI368159, SGI0, SGI368160, SGI368161, SGI368162, SGI368163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88644[] = {SGI368164, SGI368165, SGI368166, SGI368167, SGI368168, SGI368169, SGI368170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88645[] = {SGI368171, SGI368172, SGI368173, SGI0, SGI368174, SGI0, SGI368175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88646[] = {SGI368176, SGI368177, SGI368178, SGI368179, SGI368180, SGI368181, SGI368182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88647[] = {SGI368183, SGI368184, SGI368185, SGI368186, SGI368187, SGI368188, SGI368189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88648[] = {SGI368190, SGI368191, SGI368192, SGI368193, SGI0, SGI368194, SGI368195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88649[] = {SGI368196, SGI368197, SGI368198, SGI368199, SGI368200, SGI368201, SGI368202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88650[] = {SGI368203, SGI368204, SGI368205, SGI368206, SGI368207, SGI368208, SGI368209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88651[] = {SGI368210, SGI368211, SGI368212, SGI368213, SGI368214, SGI368215, SGI368216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88652[] = {SGI368217, SGI368218, SGI368219, SGI368220, SGI368221, SGI368222, SGI368223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88653[] = {SGI368224, SGI368225, SGI368226, SGI368227, SGI0, SGI368228, SGI368229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88654[] = {SGI368230, SGI368231, SGI368232, SGI368233, SGI368234, SGI368235, SGI368236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88655[] = {SGI368237, SGI368238, SGI368239, SGI368240, SGI368241, SGI368242, SGI368243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88656[] = {SGI368244, SGI368245, SGI368246, SGI368247, SGI368248, SGI368249, SGI368250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88657[] = {SGI368251, SGI368252, SGI368253, SGI368254, SGI368255, SGI368256, SGI368257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88658[] = {SGI368258, SGI368259, SGI368260, SGI368261, SGI368262, SGI368263, SGI368264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88659[] = {SGI368265, SGI368266, SGI368267, SGI368268, SGI368269, SGI368270, SGI368271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88660[] = {SGI368272, SGI368273, SGI368274, SGI368275, SGI368276, SGI368277, SGI368278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88661[] = {SGI368279, SGI368280, SGI368281, SGI368282, SGI368283, SGI368284, SGI368285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88662[] = {SGI368286, SGI368287, SGI368288, SGI368289, SGI368290, SGI368291, SGI368292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88663[] = {SGI368293, SGI368294, SGI368295, SGI368296, SGI368297, SGI368298, SGI368299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88664[] = {SGI368300, SGI368301, SGI368302, SGI368303, SGI368304, SGI368305, SGI368306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88665[] = {SGI368307, SGI368308, SGI368309, SGI368310, SGI368311, SGI368312, SGI368313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88666[] = {SGI368314, SGI368315, SGI368316, SGI368317, SGI368318, SGI368319, SGI368320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88667[] = {SGI368321, SGI368322, SGI368323, SGI368324, SGI368325, SGI368326, SGI368327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88668[] = {SGI368328, SGI368329, SGI368330, SGI368331, SGI368332, SGI368333, SGI368334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88669[] = {SGI368335, SGI368336, SGI368337, SGI368338, SGI368339, SGI368340, SGI368341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88670[] = {SGI368342, SGI368343, SGI368344, SGI368345, SGI368346, SGI368347, SGI368348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88671[] = {SGI368349, SGI368350, SGI368351, SGI368352, SGI368353, SGI368354, SGI368355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88672[] = {SGI368356, SGI368357, SGI368358, SGI368359, SGI368360, SGI368361, SGI368362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88673[] = {SGI368363, SGI368364, SGI368365, SGI368366, SGI368367, SGI368368, SGI368369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88674[] = {SGI368370, SGI368371, SGI368372, SGI368373, SGI368374, SGI368375, SGI368376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88675[] = {SGI368377, SGI368378, SGI368379, SGI368380, SGI368381, SGI368382, SGI368383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88676[] = {SGI368384, SGI368385, SGI368386, SGI368387, SGI368388, SGI368389, SGI368390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88677[] = {SGI368391, SGI368392, SGI368393, SGI368394, SGI368395, SGI368396, SGI368397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88678[] = {SGI368398, SGI368399, SGI368400, SGI368401, SGI368402, SGI368403, SGI368404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88679[] = {SGI368405, SGI368406, SGI368407, SGI368408, SGI368409, SGI368410, SGI368411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88680[] = {SGI368412, SGI368413, SGI368414, SGI368415, SGI368416, SGI368417, SGI368418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88681[] = {SGI368419, SGI368420, SGI368421, SGI368422, SGI368423, SGI368424, SGI368425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88682[] = {SGI368426, SGI368427, SGI368428, SGI368429, SGI368430, SGI368431, SGI368432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88683[] = {SGI368433, SGI368434, SGI368435, SGI368436, SGI368437, SGI368438, SGI368439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88684[] = {SGI368440, SGI368441, SGI368442, SGI368443, SGI368444, SGI368445, SGI368446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88685[] = {SGI368447, SGI368448, SGI368449, SGI368450, SGI368451, SGI368452, SGI368453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88686[] = {SGI368454, SGI368455, SGI368456, SGI368457, SGI368458, SGI368459, SGI368460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88687[] = {SGI368461, SGI368462, SGI368463, SGI368464, SGI368465, SGI368466, SGI368467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88688[] = {SGI368468, SGI368469, SGI368470, SGI368471, SGI368472, SGI368473, SGI368474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88689[] = {SGI368475, SGI368476, SGI368477, SGI368478, SGI368479, SGI368480, SGI368481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88690[] = {SGI368482, SGI368483, SGI368484, SGI368485, SGI368486, SGI368487, SGI368488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88691[] = {SGI368489, SGI368490, SGI368491, SGI368492, SGI368493, SGI368494, SGI368495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88692[] = {SGI368496, SGI368497, SGI368498, SGI368499, SGI368500, SGI368501, SGI368502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88693[] = {SGI368503, SGI368504, SGI368505, SGI368506, SGI368507, SGI368508, SGI368509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88694[] = {SGI368510, SGI368511, SGI368512, SGI368513, SGI368514, SGI368515, SGI368516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88695[] = {SGI368517, SGI368518, SGI368519, SGI368520, SGI368521, SGI368522, SGI368523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88696[] = {SGI368524, SGI368525, SGI368526, SGI368527, SGI368528, SGI368529, SGI368530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88697[] = {SGI368531, SGI368532, SGI368533, SGI368534, SGI0, SGI368535, SGI368536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88698[] = {SGI368537, SGI368538, SGI368539, SGI368540, SGI368541, SGI368542, SGI368543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88699[] = {SGI368544, SGI368545, SGI368546, SGI368547, SGI368548, SGI368549, SGI368550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88700[] = {SGI368551, SGI368552, SGI368553, SGI368554, SGI368555, SGI368556, SGI368557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88701[] = {SGI368558, SGI368559, SGI368560, SGI368561, SGI368562, SGI368563, SGI368564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88702[] = {SGI368565, SGI368566, SGI368567, SGI368568, SGI368569, SGI368570, SGI368571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88703[] = {SGI368572, SGI368573, SGI368574, SGI368575, SGI368576, SGI368577, SGI368578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88704[] = {SGI368579, SGI368580, SGI368581, SGI368582, SGI368583, SGI368584, SGI368585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88705[] = {SGI368586, SGI368587, SGI368588, SGI0, SGI368589, SGI0, SGI368590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88706[] = {SGI368591, SGI368592, SGI368593, SGI368594, SGI368595, SGI368596, SGI368597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88707[] = {SGI368598, SGI368599, SGI368600, SGI368601, SGI368602, SGI368603, SGI368604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88708[] = {SGI368605, SGI368606, SGI368607, SGI368608, SGI368609, SGI368610, SGI368611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88709[] = {SGI368612, SGI368613, SGI368614, SGI368615, SGI368616, SGI368617, SGI368618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88710[] = {SGI368619, SGI368620, SGI368621, SGI368622, SGI368623, SGI368624, SGI368625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88711[] = {SGI0, SGI368626, SGI368627, SGI368628, SGI368629, SGI368630, SGI368631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88712[] = {SGI368632, SGI368633, SGI368634, SGI368635, SGI368636, SGI368637, SGI368638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88713[] = {SGI368639, SGI368640, SGI368641, SGI368642, SGI368643, SGI368644, SGI368645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88714[] = {SGI368646, SGI368647, SGI368648, SGI368649, SGI368650, SGI368651, SGI368652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88715[] = {SGI368653, SGI368654, SGI368655, SGI368656, SGI368657, SGI368658, SGI368659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88716[] = {SGI368660, SGI368661, SGI368662, SGI368663, SGI368664, SGI368665, SGI368666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88717[] = {SGI368667, SGI368668, SGI368669, SGI368670, SGI368671, SGI368672, SGI368673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88718[] = {SGI368674, SGI368675, SGI368676, SGI368677, SGI368678, SGI368679, SGI368680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88719[] = {SGI368681, SGI368682, SGI368683, SGI368684, SGI368685, SGI368686, SGI368687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88720[] = {SGI368688, SGI368689, SGI368690, SGI368691, SGI368692, SGI368693, SGI368694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88721[] = {SGI368695, SGI368696, SGI368697, SGI368698, SGI368699, SGI368700, SGI368701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88722[] = {SGI368702, SGI368703, SGI368704, SGI368705, SGI368706, SGI368707, SGI368708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88723[] = {SGI368709, SGI368710, SGI368711, SGI368712, SGI368713, SGI368714, SGI368715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88724[] = {SGI368716, SGI368717, SGI368718, SGI368719, SGI368720, SGI368721, SGI368722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88725[] = {SGI368723, SGI368724, SGI368725, SGI368726, SGI368727, SGI368728, SGI368729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88726[] = {SGI368730, SGI368731, SGI368732, SGI368733, SGI0, SGI368734, SGI368735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88727[] = {SGI368736, SGI368737, SGI368738, SGI368739, SGI368740, SGI368741, SGI368742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88728[] = {SGI368743, SGI368744, SGI368745, SGI368746, SGI368747, SGI368748, SGI368749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88729[] = {SGI368750, SGI368751, SGI368752, SGI368753, SGI368754, SGI368755, SGI368756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88730[] = {SGI368757, SGI368758, SGI368759, SGI368760, SGI368761, SGI368762, SGI368763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88731[] = {SGI368764, SGI368765, SGI368766, SGI368767, SGI368768, SGI368769, SGI368770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88732[] = {SGI368771, SGI368772, SGI368773, SGI368774, SGI368775, SGI368776, SGI368777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88733[] = {SGI368778, SGI368779, SGI368780, SGI368781, SGI368782, SGI368783, SGI368784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88734[] = {SGI368785, SGI368786, SGI368787, SGI368788, SGI368789, SGI368790, SGI368791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88735[] = {SGI368792, SGI368793, SGI368794, SGI368795, SGI368796, SGI368797, SGI368798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88736[] = {SGI368799, SGI368800, SGI368801, SGI368802, SGI368803, SGI368804, SGI368805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88737[] = {SGI368806, SGI368807, SGI368808, SGI368809, SGI368810, SGI368811, SGI368812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88738[] = {SGI368813, SGI368814, SGI368815, SGI368816, SGI368817, SGI368818, SGI368819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88739[] = {SGI368820, SGI368821, SGI368822, SGI368823, SGI368824, SGI368825, SGI368826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88740[] = {SGI368827, SGI368828, SGI368829, SGI368830, SGI368831, SGI368832, SGI368833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88741[] = {SGI368834, SGI368835, SGI368836, SGI368837, SGI368838, SGI368839, SGI368840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88742[] = {SGI368841, SGI368842, SGI368843, SGI368844, SGI368845, SGI368846, SGI368847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88743[] = {SGI368848, SGI368849, SGI368850, SGI368851, SGI368852, SGI368853, SGI368854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88744[] = {SGI368855, SGI368856, SGI368857, SGI368858, SGI0, SGI368859, SGI368860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88745[] = {SGI368861, SGI368862, SGI368863, SGI368864, SGI368865, SGI368866, SGI368867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88746[] = {SGI368868, SGI368869, SGI368870, SGI368871, SGI368872, SGI368873, SGI368874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88747[] = {SGI368875, SGI368876, SGI368877, SGI368878, SGI368879, SGI368880, SGI368881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88748[] = {SGI368882, SGI368883, SGI368884, SGI368885, SGI368886, SGI368887, SGI368888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88749[] = {SGI368889, SGI368890, SGI368891, SGI368892, SGI368893, SGI368894, SGI368895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88750[] = {SGI368896, SGI368897, SGI368898, SGI368899, SGI368900, SGI368901, SGI368902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88751[] = {SGI368903, SGI368904, SGI368905, SGI368906, SGI368907, SGI368908, SGI368909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88752[] = {SGI368910, SGI368911, SGI368912, SGI368913, SGI368914, SGI368915, SGI368916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88753[] = {SGI368917, SGI368918, SGI368919, SGI368920, SGI368921, SGI368922, SGI368923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88754[] = {SGI368924, SGI368925, SGI368926, SGI368927, SGI368928, SGI368929, SGI368930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88755[] = {SGI368931, SGI368932, SGI368933, SGI368934, SGI368935, SGI368936, SGI368937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88756[] = {SGI368938, SGI368939, SGI368940, SGI368941, SGI368942, SGI368943, SGI368944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88757[] = {SGI368945, SGI368946, SGI368947, SGI368948, SGI368949, SGI368950, SGI368951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88758[] = {SGI368952, SGI368953, SGI368954, SGI368955, SGI368956, SGI368957, SGI368958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88759[] = {SGI368959, SGI368960, SGI368961, SGI368962, SGI368963, SGI368964, SGI368965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88760[] = {SGI368966, SGI368967, SGI368968, SGI368969, SGI368970, SGI368971, SGI368972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88761[] = {SGI368973, SGI368974, SGI368975, SGI368976, SGI368977, SGI368978, SGI368979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88762[] = {SGI368980, SGI368981, SGI368982, SGI368983, SGI368984, SGI368985, SGI368986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88763[] = {SGI368987, SGI368988, SGI368989, SGI368990, SGI368991, SGI368992, SGI368993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88764[] = {SGI368994, SGI368995, SGI368996, SGI368997, SGI368998, SGI368999, SGI369000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88765[] = {SGI369001, SGI369002, SGI369003, SGI369004, SGI369005, SGI369006, SGI369007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88766[] = {SGI369008, SGI369009, SGI369010, SGI369011, SGI369012, SGI369013, SGI369014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88767[] = {SGI369015, SGI369016, SGI369017, SGI369018, SGI369019, SGI369020, SGI369021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88768[] = {SGI369022, SGI369023, SGI369024, SGI369025, SGI369026, SGI369027, SGI369028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88769[] = {SGI369029, SGI369030, SGI369031, SGI369032, SGI369033, SGI369034, SGI369035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88770[] = {SGI369036, SGI369037, SGI369038, SGI369039, SGI369040, SGI369041, SGI369042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88771[] = {SGI369043, SGI369044, SGI369045, SGI369046, SGI369047, SGI369048, SGI369049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88772[] = {SGI369050, SGI369051, SGI369052, SGI369053, SGI369054, SGI369055, SGI369056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88773[] = {SGI369057, SGI369058, SGI369059, SGI369060, SGI369061, SGI369062, SGI369063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88774[] = {SGI369064, SGI369065, SGI369066, SGI369067, SGI369068, SGI369069, SGI369070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88775[] = {SGI369071, SGI369072, SGI369073, SGI369074, SGI369075, SGI369076, SGI369077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88776[] = {SGI369078, SGI369079, SGI369080, SGI369081, SGI369082, SGI369083, SGI369084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88777[] = {SGI369085, SGI369086, SGI369087, SGI369088, SGI369089, SGI369090, SGI369091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88778[] = {SGI369092, SGI369093, SGI369094, SGI369095, SGI369096, SGI369097, SGI369098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88779[] = {SGI369099, SGI369100, SGI369101, SGI369102, SGI369103, SGI369104, SGI369105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88780[] = {SGI369106, SGI369107, SGI369108, SGI369109, SGI369110, SGI369111, SGI369112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88781[] = {SGI369113, SGI369114, SGI369115, SGI369116, SGI369117, SGI369118, SGI369119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88782[] = {SGI369120, SGI369121, SGI369122, SGI369123, SGI369124, SGI369125, SGI369126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88783[] = {SGI369127, SGI369128, SGI369129, SGI369130, SGI369131, SGI369132, SGI369133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88784[] = {SGI369134, SGI369135, SGI369136, SGI369137, SGI369138, SGI369139, SGI369140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88785[] = {SGI369141, SGI369142, SGI369143, SGI369144, SGI369145, SGI369146, SGI369147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88786[] = {SGI369148, SGI369149, SGI369150, SGI369151, SGI369152, SGI369153, SGI369154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88787[] = {SGI369155, SGI369156, SGI369157, SGI369158, SGI369159, SGI369160, SGI369161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88788[] = {SGI369162, SGI369163, SGI369164, SGI369165, SGI369166, SGI369167, SGI369168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88789[] = {SGI369169, SGI369170, SGI369171, SGI369172, SGI369173, SGI369174, SGI369175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88790[] = {SGI369176, SGI369177, SGI369178, SGI369179, SGI369180, SGI369181, SGI369182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88791[] = {SGI369183, SGI369184, SGI369185, SGI369186, SGI369187, SGI369188, SGI369189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88792[] = {SGI369190, SGI369191, SGI369192, SGI369193, SGI369194, SGI369195, SGI369196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88793[] = {SGI369197, SGI369198, SGI369199, SGI369200, SGI369201, SGI369202, SGI369203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88794[] = {SGI369204, SGI369205, SGI369206, SGI369207, SGI369208, SGI369209, SGI369210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88795[] = {SGI369211, SGI369212, SGI369213, SGI369214, SGI369215, SGI369216, SGI369217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88796[] = {SGI369218, SGI369219, SGI369220, SGI369221, SGI369222, SGI369223, SGI369224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88797[] = {SGI369225, SGI369226, SGI369227, SGI369228, SGI369229, SGI369230, SGI369231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88798[] = {SGI369232, SGI369233, SGI369234, SGI369235, SGI369236, SGI369237, SGI369238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88799[] = {SGI369239, SGI369240, SGI369241, SGI369242, SGI369243, SGI369244, SGI369245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88800[] = {SGI369246, SGI369247, SGI369248, SGI369249, SGI369250, SGI369251, SGI369252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88801[] = {SGI369253, SGI369254, SGI369255, SGI369256, SGI369257, SGI369258, SGI369259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88802[] = {SGI369260, SGI369261, SGI369262, SGI369263, SGI369264, SGI369265, SGI369266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88803[] = {SGI369267, SGI369268, SGI369269, SGI369270, SGI369271, SGI369272, SGI369273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88804[] = {SGI369274, SGI369275, SGI369276, SGI369277, SGI369278, SGI369279, SGI369280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88805[] = {SGI369281, SGI369282, SGI369283, SGI369284, SGI369285, SGI369286, SGI369287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88806[] = {SGI369288, SGI369289, SGI369290, SGI369291, SGI369292, SGI0, SGI369291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88807[] = {SGI369293, SGI369294, SGI369295, SGI369296, SGI369297, SGI369298, SGI369299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88808[] = {SGI369300, SGI369301, SGI369302, SGI369303, SGI369304, SGI369305, SGI369306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88809[] = {SGI369307, SGI369308, SGI369309, SGI369310, SGI369311, SGI369312, SGI369313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88810[] = {SGI369314, SGI369315, SGI369316, SGI369317, SGI369318, SGI369319, SGI369320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88811[] = {SGI369321, SGI369322, SGI369323, SGI369324, SGI369325, SGI369326, SGI369327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88812[] = {SGI369328, SGI369329, SGI369330, SGI369331, SGI369332, SGI369333, SGI369334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88813[] = {SGI369335, SGI369336, SGI369337, SGI369338, SGI369339, SGI369340, SGI369341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88814[] = {SGI369342, SGI369343, SGI369344, SGI369345, SGI369346, SGI369347, SGI369348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88815[] = {SGI369349, SGI369350, SGI369351, SGI369352, SGI369353, SGI369354, SGI369355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88816[] = {SGI369356, SGI369357, SGI369358, SGI369359, SGI369360, SGI369361, SGI369362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88817[] = {SGI369363, SGI369364, SGI369365, SGI369366, SGI369367, SGI369368, SGI369369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88818[] = {SGI369370, SGI369371, SGI369372, SGI369373, SGI369374, SGI0, SGI369373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88819[] = {SGI369375, SGI369376, SGI369377, SGI369378, SGI369379, SGI369380, SGI369381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88820[] = {SGI369382, SGI369383, SGI369384, SGI369385, SGI369386, SGI369387, SGI369388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88821[] = {SGI369389, SGI369390, SGI369391, SGI369392, SGI369393, SGI369394, SGI369395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88822[] = {SGI369396, SGI369397, SGI369398, SGI369399, SGI369400, SGI369401, SGI369402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88823[] = {SGI369403, SGI369404, SGI369405, SGI369406, SGI369407, SGI369408, SGI369409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88824[] = {SGI369410, SGI369411, SGI369412, SGI369413, SGI369414, SGI369415, SGI369416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88825[] = {SGI369417, SGI369418, SGI369419, SGI369420, SGI369421, SGI369422, SGI369423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88826[] = {SGI369424, SGI369425, SGI369426, SGI369427, SGI369428, SGI369429, SGI369430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88827[] = {SGI369431, SGI369432, SGI369433, SGI369434, SGI369435, SGI369436, SGI369437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88828[] = {SGI369438, SGI369439, SGI369440, SGI369441, SGI369442, SGI369443, SGI369444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88829[] = {SGI369445, SGI369446, SGI369447, SGI369448, SGI369449, SGI369450, SGI369451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88830[] = {SGI369452, SGI369453, SGI369454, SGI369455, SGI369456, SGI369457, SGI369458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88831[] = {SGI369459, SGI369460, SGI369461, SGI369462, SGI369463, SGI369464, SGI369465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88832[] = {SGI369466, SGI369467, SGI369468, SGI369469, SGI369470, SGI369471, SGI369472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88833[] = {SGI369473, SGI369474, SGI369475, SGI369476, SGI369477, SGI369478, SGI369479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88834[] = {SGI369480, SGI369481, SGI369482, SGI369483, SGI369484, SGI369485, SGI369486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88835[] = {SGI369487, SGI369488, SGI369489, SGI369490, SGI369491, SGI369492, SGI369493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88836[] = {SGI369494, SGI369495, SGI369496, SGI369497, SGI369498, SGI369499, SGI369500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88837[] = {SGI369501, SGI369502, SGI369503, SGI369504, SGI369505, SGI369506, SGI369507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88838[] = {SGI369508, SGI369509, SGI369510, SGI369511, SGI369512, SGI369513, SGI369514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88839[] = {SGI369515, SGI369516, SGI369517, SGI369518, SGI369519, SGI369520, SGI369521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88840[] = {SGI369522, SGI369523, SGI369524, SGI369525, SGI369526, SGI369527, SGI369528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88841[] = {SGI369529, SGI369530, SGI369531, SGI369532, SGI369533, SGI369534, SGI369535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88842[] = {SGI369536, SGI369537, SGI369538, SGI369539, SGI369540, SGI369541, SGI369542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88843[] = {SGI369543, SGI369544, SGI369545, SGI369546, SGI369547, SGI369548, SGI369549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88844[] = {SGI369550, SGI369551, SGI369552, SGI369553, SGI369554, SGI369555, SGI369556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88845[] = {SGI369557, SGI369558, SGI369559, SGI369560, SGI369561, SGI369562, SGI369563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88846[] = {SGI369564, SGI369565, SGI369566, SGI369567, SGI369568, SGI369569, SGI369570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88847[] = {SGI369571, SGI369572, SGI369573, SGI369574, SGI369575, SGI369576, SGI369577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88848[] = {SGI369578, SGI369579, SGI369580, SGI369581, SGI369582, SGI369583, SGI369584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88849[] = {SGI369585, SGI369586, SGI369587, SGI369588, SGI369589, SGI369590, SGI369591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88850[] = {SGI369592, SGI369593, SGI369594, SGI369595, SGI369596, SGI369597, SGI369598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88851[] = {SGI369599, SGI369600, SGI369601, SGI369602, SGI369603, SGI369604, SGI369605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88852[] = {SGI369606, SGI369607, SGI369608, SGI369609, SGI369610, SGI369611, SGI369612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88853[] = {SGI369613, SGI369614, SGI369615, SGI369616, SGI369617, SGI369618, SGI369619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88854[] = {SGI369620, SGI369621, SGI369622, SGI369623, SGI369624, SGI369625, SGI369626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88855[] = {SGI369627, SGI369628, SGI369629, SGI369630, SGI369631, SGI369632, SGI369633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88856[] = {SGI369634, SGI369635, SGI369636, SGI369637, SGI369638, SGI369639, SGI369640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88857[] = {SGI369641, SGI369642, SGI369643, SGI369644, SGI369645, SGI369646, SGI369647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88858[] = {SGI369648, SGI369649, SGI369650, SGI369651, SGI369652, SGI369653, SGI369654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88859[] = {SGI369655, SGI369656, SGI369657, SGI369658, SGI369659, SGI369660, SGI369661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88860[] = {SGI369662, SGI369663, SGI369664, SGI369665, SGI369666, SGI369667, SGI369668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88861[] = {SGI369669, SGI369670, SGI369671, SGI369672, SGI369673, SGI369674, SGI369675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88862[] = {SGI369676, SGI369677, SGI369678, SGI369679, SGI369680, SGI369681, SGI369682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88863[] = {SGI369683, SGI369684, SGI369685, SGI369686, SGI369687, SGI369688, SGI369689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88864[] = {SGI369690, SGI369691, SGI369692, SGI369693, SGI369694, SGI369695, SGI369696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88865[] = {SGI369697, SGI369698, SGI369699, SGI369700, SGI369701, SGI369702, SGI369703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88866[] = {SGI369704, SGI369705, SGI369706, SGI369707, SGI369708, SGI369709, SGI369710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88867[] = {SGI369711, SGI369712, SGI369713, SGI369714, SGI369715, SGI369716, SGI369717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88868[] = {SGI369718, SGI369719, SGI369720, SGI369721, SGI369722, SGI369723, SGI369724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88869[] = {SGI369725, SGI369726, SGI369727, SGI369728, SGI369729, SGI369730, SGI369731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88870[] = {SGI369732, SGI369733, SGI369734, SGI369735, SGI369736, SGI369737, SGI369738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88871[] = {SGI369739, SGI369740, SGI369741, SGI369742, SGI369743, SGI369744, SGI369745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88872[] = {SGI369746, SGI369747, SGI369748, SGI369749, SGI369750, SGI369751, SGI369752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88873[] = {SGI369753, SGI369754, SGI369755, SGI369756, SGI369757, SGI369758, SGI369759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88874[] = {SGI369760, SGI369761, SGI369762, SGI369763, SGI369764, SGI369765, SGI369766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88875[] = {SGI369767, SGI369768, SGI369769, SGI369770, SGI369771, SGI369772, SGI369773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88876[] = {SGI369774, SGI369775, SGI369776, SGI369777, SGI369778, SGI369779, SGI369780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88877[] = {SGI369781, SGI369782, SGI369783, SGI369784, SGI369785, SGI369786, SGI369787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88878[] = {SGI369788, SGI369789, SGI369790, SGI369791, SGI369792, SGI0, SGI369791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88879[] = {SGI369793, SGI369794, SGI369795, SGI369796, SGI369797, SGI369798, SGI369799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88880[] = {SGI369800, SGI369801, SGI369802, SGI369803, SGI369804, SGI369805, SGI369806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88881[] = {SGI369807, SGI369808, SGI369809, SGI369810, SGI369811, SGI369812, SGI369813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88882[] = {SGI369814, SGI369815, SGI369816, SGI369817, SGI369818, SGI369819, SGI369820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88883[] = {SGI369821, SGI369822, SGI369823, SGI369824, SGI369825, SGI369826, SGI369827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88884[] = {SGI369828, SGI369829, SGI369830, SGI369831, SGI369832, SGI369833, SGI369834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88885[] = {SGI369835, SGI369836, SGI369837, SGI369838, SGI369839, SGI369840, SGI369841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88886[] = {SGI369842, SGI369843, SGI369844, SGI369845, SGI369846, SGI369847, SGI369848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88887[] = {SGI369849, SGI369850, SGI369851, SGI369852, SGI369853, SGI369854, SGI369855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88888[] = {SGI369856, SGI369857, SGI369858, SGI369859, SGI369860, SGI369861, SGI369862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88889[] = {SGI369863, SGI369864, SGI369865, SGI369866, SGI369867, SGI369868, SGI369869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88890[] = {SGI369870, SGI369871, SGI369872, SGI369873, SGI369874, SGI369875, SGI369876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88891[] = {SGI369877, SGI369878, SGI369879, SGI369880, SGI369881, SGI369882, SGI369883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88892[] = {SGI369884, SGI369885, SGI369886, SGI369887, SGI369888, SGI369889, SGI369890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88893[] = {SGI369891, SGI369892, SGI369893, SGI369894, SGI369895, SGI369896, SGI369897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88894[] = {SGI369898, SGI369899, SGI369900, SGI369901, SGI369902, SGI369903, SGI369904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88895[] = {SGI369905, SGI369906, SGI369907, SGI369908, SGI369909, SGI369910, SGI369911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88896[] = {SGI369912, SGI369913, SGI369914, SGI369915, SGI369916, SGI369917, SGI369918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88897[] = {SGI369919, SGI369920, SGI369921, SGI369922, SGI369923, SGI369924, SGI369925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88898[] = {SGI369926, SGI369927, SGI369928, SGI369929, SGI369930, SGI369931, SGI369932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88899[] = {SGI369933, SGI369934, SGI369935, SGI369936, SGI369937, SGI369938, SGI369939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88900[] = {SGI369940, SGI369941, SGI369942, SGI369943, SGI369944, SGI369945, SGI369946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88901[] = {SGI369947, SGI369948, SGI369949, SGI369950, SGI369951, SGI369952, SGI369953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88902[] = {SGI369954, SGI369955, SGI369956, SGI369957, SGI369958, SGI369959, SGI369960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88903[] = {SGI369961, SGI369962, SGI369963, SGI369964, SGI369965, SGI369966, SGI369967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88904[] = {SGI369968, SGI369969, SGI369970, SGI369971, SGI369972, SGI369973, SGI369974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88905[] = {SGI369975, SGI369976, SGI369977, SGI369978, SGI369979, SGI369980, SGI369981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88906[] = {SGI369982, SGI369983, SGI369984, SGI369985, SGI369986, SGI369987, SGI369988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88907[] = {SGI369989, SGI369990, SGI369991, SGI369992, SGI369993, SGI369994, SGI369995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88908[] = {SGI369996, SGI369997, SGI369998, SGI369999, SGI370000, SGI0, SGI369999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88909[] = {SGI370001, SGI370002, SGI370003, SGI370004, SGI370005, SGI370006, SGI370007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88910[] = {SGI370008, SGI370009, SGI370010, SGI370011, SGI370012, SGI370013, SGI370014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88911[] = {SGI370015, SGI370016, SGI370017, SGI370018, SGI370019, SGI370020, SGI370021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88912[] = {SGI370022, SGI370023, SGI370024, SGI370025, SGI370026, SGI370027, SGI370028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88913[] = {SGI370029, SGI370030, SGI370031, SGI370032, SGI370033, SGI370034, SGI370035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88914[] = {SGI370036, SGI370037, SGI370038, SGI370039, SGI370040, SGI370041, SGI370042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88915[] = {SGI370043, SGI370044, SGI370045, SGI370046, SGI370047, SGI370048, SGI370049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88916[] = {SGI370050, SGI370051, SGI370052, SGI370053, SGI370054, SGI370055, SGI370056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88917[] = {SGI370057, SGI370058, SGI370059, SGI370060, SGI370061, SGI370062, SGI370063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88918[] = {SGI370064, SGI370065, SGI370066, SGI370067, SGI370068, SGI370069, SGI370070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88919[] = {SGI370071, SGI370072, SGI370073, SGI370074, SGI370075, SGI0, SGI370074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88920[] = {SGI370076, SGI370077, SGI370078, SGI370079, SGI370080, SGI370081, SGI370082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88921[] = {SGI370083, SGI370084, SGI370085, SGI370086, SGI370087, SGI370088, SGI370089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88922[] = {SGI370090, SGI370091, SGI370092, SGI370093, SGI0, SGI370094, SGI370095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88923[] = {SGI370096, SGI370097, SGI370098, SGI370099, SGI370100, SGI370101, SGI370102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88924[] = {SGI370103, SGI370104, SGI370105, SGI370106, SGI370107, SGI370108, SGI370109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88925[] = {SGI370110, SGI370111, SGI370112, SGI370113, SGI370114, SGI370115, SGI370116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88926[] = {SGI370117, SGI370118, SGI370119, SGI370120, SGI370121, SGI370122, SGI370123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88927[] = {SGI370124, SGI370125, SGI370126, SGI370127, SGI370128, SGI370129, SGI370130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88928[] = {SGI370131, SGI370132, SGI370133, SGI370134, SGI370135, SGI370136, SGI370137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88929[] = {SGI370138, SGI370139, SGI370140, SGI370141, SGI370142, SGI0, SGI370141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88930[] = {SGI370143, SGI370144, SGI370145, SGI370146, SGI370147, SGI370148, SGI370149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88931[] = {SGI370150, SGI370151, SGI370152, SGI370153, SGI370154, SGI370155, SGI370156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88932[] = {SGI370157, SGI370158, SGI370159, SGI370160, SGI370161, SGI370162, SGI370163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88933[] = {SGI370164, SGI370165, SGI370166, SGI370167, SGI370168, SGI370169, SGI370170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88934[] = {SGI370171, SGI370172, SGI370173, SGI370174, SGI370175, SGI370176, SGI370177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88935[] = {SGI370178, SGI370179, SGI370180, SGI370181, SGI370182, SGI370183, SGI370184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88936[] = {SGI370185, SGI370186, SGI370187, SGI370188, SGI370189, SGI370190, SGI370191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88937[] = {SGI370192, SGI370193, SGI370194, SGI370195, SGI370196, SGI370197, SGI370198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88938[] = {SGI370199, SGI370200, SGI370201, SGI370202, SGI370203, SGI370204, SGI370205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88939[] = {SGI370206, SGI370207, SGI370208, SGI370209, SGI370210, SGI370211, SGI370212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88940[] = {SGI370213, SGI370214, SGI370215, SGI370216, SGI370217, SGI370218, SGI370219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88941[] = {SGI370220, SGI370221, SGI370222, SGI370223, SGI370224, SGI370225, SGI370226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88942[] = {SGI370227, SGI370228, SGI370229, SGI370230, SGI370231, SGI370232, SGI370233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88943[] = {SGI370234, SGI370235, SGI370236, SGI370237, SGI370238, SGI370239, SGI370240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88944[] = {SGI370241, SGI370242, SGI370243, SGI370244, SGI370245, SGI370246, SGI370247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88945[] = {SGI370248, SGI370249, SGI370250, SGI370251, SGI370252, SGI370253, SGI370254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88946[] = {SGI370255, SGI370256, SGI370257, SGI370258, SGI370259, SGI370260, SGI370261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88947[] = {SGI370262, SGI370263, SGI370264, SGI370265, SGI370266, SGI370267, SGI370268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88948[] = {SGI370269, SGI370270, SGI370271, SGI370272, SGI370273, SGI370274, SGI370275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88949[] = {SGI370276, SGI370277, SGI370278, SGI370279, SGI370280, SGI370281, SGI370282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88950[] = {SGI370283, SGI370284, SGI370285, SGI370286, SGI370287, SGI370288, SGI370289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88951[] = {SGI370290, SGI370291, SGI370292, SGI370293, SGI370294, SGI370295, SGI370296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88952[] = {SGI370297, SGI370298, SGI370299, SGI370300, SGI370301, SGI370302, SGI370303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88953[] = {SGI370304, SGI370305, SGI370306, SGI370307, SGI370308, SGI370309, SGI370310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88954[] = {SGI370311, SGI370312, SGI370313, SGI370314, SGI370315, SGI370316, SGI370317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88955[] = {SGI370318, SGI370319, SGI370320, SGI370321, SGI370322, SGI370323, SGI370324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88956[] = {SGI370325, SGI370326, SGI370327, SGI370328, SGI370329, SGI370330, SGI370331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88957[] = {SGI370332, SGI370333, SGI370334, SGI370335, SGI370336, SGI370337, SGI370338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88958[] = {SGI370339, SGI370340, SGI370341, SGI370342, SGI370343, SGI0, SGI370342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88959[] = {SGI0, SGI370344, SGI370345, SGI370346, SGI370347, SGI370348, SGI370349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88960[] = {SGI370350, SGI370351, SGI370352, SGI370353, SGI0, SGI0, SGI370353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88961[] = {SGI370354, SGI370355, SGI370356, SGI370357, SGI370358, SGI370359, SGI370360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88962[] = {SGI370361, SGI370362, SGI370363, SGI370364, SGI370365, SGI0, SGI370364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88963[] = {SGI370366, SGI370367, SGI370368, SGI370369, SGI370370, SGI370371, SGI370372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88964[] = {SGI370373, SGI370374, SGI370375, SGI370376, SGI370377, SGI370378, SGI370379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88965[] = {SGI370380, SGI370381, SGI370382, SGI370383, SGI370384, SGI370385, SGI370386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88966[] = {SGI370387, SGI370388, SGI370389, SGI370390, SGI370391, SGI370392, SGI370393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88967[] = {SGI370394, SGI370395, SGI370396, SGI370397, SGI0, SGI370398, SGI370399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88968[] = {SGI370400, SGI370401, SGI370402, SGI370403, SGI370404, SGI370405, SGI370406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88969[] = {SGI370407, SGI370408, SGI370409, SGI370410, SGI370411, SGI370412, SGI370413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88970[] = {SGI370414, SGI370415, SGI370416, SGI370417, SGI370418, SGI370419, SGI370420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88971[] = {SGI370421, SGI370422, SGI370423, SGI370424, SGI370425, SGI370426, SGI370427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88972[] = {SGI370428, SGI370429, SGI370430, SGI370431, SGI370432, SGI370433, SGI370434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88973[] = {SGI370435, SGI370436, SGI370437, SGI370438, SGI370439, SGI370440, SGI370441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88974[] = {SGI370442, SGI370443, SGI370444, SGI370445, SGI370446, SGI370447, SGI370448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88975[] = {SGI370449, SGI370450, SGI370451, SGI370452, SGI370453, SGI370454, SGI370455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88976[] = {SGI370456, SGI370457, SGI370458, SGI370459, SGI370460, SGI370461, SGI370462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88977[] = {SGI370463, SGI370464, SGI370465, SGI370466, SGI370467, SGI370468, SGI370469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88978[] = {SGI370470, SGI370471, SGI370472, SGI0, SGI370473, SGI0, SGI370474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88979[] = {SGI370475, SGI370476, SGI370477, SGI370478, SGI370479, SGI370480, SGI370481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88980[] = {SGI370482, SGI370483, SGI370484, SGI370485, SGI370486, SGI370487, SGI370488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88981[] = {SGI370489, SGI370490, SGI370491, SGI370492, SGI370493, SGI370494, SGI370495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88982[] = {SGI370496, SGI370497, SGI370498, SGI370499, SGI370500, SGI370501, SGI370502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88983[] = {SGI370503, SGI370504, SGI370505, SGI370506, SGI370507, SGI370508, SGI370509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88984[] = {SGI0, SGI370510, SGI370511, SGI370512, SGI370513, SGI370514, SGI370515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88985[] = {SGI370516, SGI370517, SGI0, SGI370518, SGI0, SGI370519, SGI370520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88986[] = {SGI370521, SGI370522, SGI370523, SGI370524, SGI370525, SGI370526, SGI370527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88987[] = {SGI370528, SGI370529, SGI370530, SGI370531, SGI370532, SGI370533, SGI370534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88988[] = {SGI370535, SGI370536, SGI370537, SGI370538, SGI370539, SGI370540, SGI370541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88989[] = {SGI370542, SGI370543, SGI370544, SGI370545, SGI370546, SGI370547, SGI370548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88990[] = {SGI370549, SGI370550, SGI370551, SGI370552, SGI0, SGI370553, SGI370554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88991[] = {SGI370555, SGI370556, SGI370557, SGI0, SGI370558, SGI0, SGI370559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88992[] = {SGI370560, SGI370561, SGI370562, SGI370563, SGI370564, SGI370565, SGI370566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88993[] = {SGI370567, SGI370568, SGI370569, SGI370570, SGI370571, SGI370572, SGI370573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88994[] = {SGI370574, SGI370575, SGI370576, SGI370577, SGI370578, SGI370579, SGI370580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88995[] = {SGI370581, SGI370582, SGI370583, SGI370584, SGI370585, SGI370586, SGI370587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88996[] = {SGI370588, SGI370589, SGI370590, SGI370591, SGI370592, SGI370593, SGI370594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88997[] = {SGI370595, SGI370596, SGI370597, SGI370598, SGI370599, SGI370600, SGI370601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88998[] = {SGI370602, SGI370603, SGI370604, SGI370605, SGI370606, SGI370607, SGI370608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG88999[] = {SGI370609, SGI370610, SGI370611, SGI370612, SGI370613, SGI370614, SGI370615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89000[] = {SGI370616, SGI370617, SGI370618, SGI370619, SGI370620, SGI370621, SGI370622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89001[] = {SGI370623, SGI370624, SGI370625, SGI0, SGI370626, SGI0, SGI370627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89002[] = {SGI370628, SGI370629, SGI370630, SGI370631, SGI370632, SGI370633, SGI370634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89003[] = {SGI370635, SGI370636, SGI370637, SGI370638, SGI370639, SGI370640, SGI370641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89004[] = {SGI370642, SGI370643, SGI370644, SGI370645, SGI370646, SGI370647, SGI370648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89005[] = {SGI370649, SGI370650, SGI370651, SGI370652, SGI370653, SGI370654, SGI370655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89006[] = {SGI370656, SGI370657, SGI370658, SGI370659, SGI370660, SGI370661, SGI370662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89007[] = {SGI370663, SGI370664, SGI370665, SGI370666, SGI370667, SGI370668, SGI370669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89008[] = {SGI370670, SGI370671, SGI370672, SGI370673, SGI370674, SGI370675, SGI370676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89009[] = {SGI370677, SGI370678, SGI370679, SGI370680, SGI370681, SGI370682, SGI370683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89010[] = {SGI370684, SGI370685, SGI370686, SGI370687, SGI370688, SGI370689, SGI370690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89011[] = {SGI370691, SGI370692, SGI370693, SGI370694, SGI370695, SGI370696, SGI370697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89012[] = {SGI370698, SGI370699, SGI370700, SGI370701, SGI370702, SGI370703, SGI370704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89013[] = {SGI370705, SGI370706, SGI370707, SGI370708, SGI370709, SGI370710, SGI370711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89014[] = {SGI370712, SGI370713, SGI370714, SGI370715, SGI370716, SGI370717, SGI370718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89015[] = {SGI370719, SGI370720, SGI370721, SGI370722, SGI370723, SGI370724, SGI370725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89016[] = {SGI370726, SGI370727, SGI370728, SGI370729, SGI370730, SGI370731, SGI370732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89017[] = {SGI370733, SGI370734, SGI370735, SGI370736, SGI370737, SGI370738, SGI370739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89018[] = {SGI370740, SGI370741, SGI370742, SGI370743, SGI370744, SGI370745, SGI370746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89019[] = {SGI370747, SGI370748, SGI370749, SGI370750, SGI370751, SGI370752, SGI370753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89020[] = {SGI370754, SGI370755, SGI370756, SGI370757, SGI370758, SGI370759, SGI370760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89021[] = {SGI370761, SGI370762, SGI370763, SGI370764, SGI370765, SGI370766, SGI370767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89022[] = {SGI370768, SGI370769, SGI370770, SGI370771, SGI370772, SGI370773, SGI370774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89023[] = {SGI370775, SGI370776, SGI370777, SGI370778, SGI370779, SGI370780, SGI370781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89024[] = {SGI370782, SGI370783, SGI370784, SGI370785, SGI370786, SGI370787, SGI370788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89025[] = {SGI370789, SGI370790, SGI370791, SGI370792, SGI370793, SGI370794, SGI370795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89026[] = {SGI370796, SGI370797, SGI370798, SGI370799, SGI370800, SGI370801, SGI370802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89027[] = {SGI370803, SGI370804, SGI370805, SGI370806, SGI0, SGI370807, SGI370808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89028[] = {SGI370809, SGI370810, SGI370811, SGI370812, SGI370813, SGI370814, SGI370815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89029[] = {SGI370816, SGI370817, SGI370818, SGI370819, SGI370820, SGI370821, SGI370822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89030[] = {SGI370823, SGI370824, SGI370825, SGI370826, SGI370827, SGI370828, SGI370829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89031[] = {SGI370830, SGI370831, SGI370832, SGI370833, SGI370834, SGI0, SGI370833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89032[] = {SGI370835, SGI370836, SGI370837, SGI370838, SGI0, SGI370839, SGI370840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89033[] = {SGI370841, SGI370842, SGI370843, SGI370844, SGI370845, SGI370846, SGI370847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89034[] = {SGI370848, SGI370849, SGI370850, SGI370851, SGI370852, SGI370853, SGI370854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89035[] = {SGI370855, SGI370856, SGI370857, SGI370858, SGI370859, SGI370860, SGI370861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89036[] = {SGI370862, SGI370863, SGI370864, SGI370865, SGI370866, SGI370867, SGI370868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89037[] = {SGI370869, SGI370870, SGI370871, SGI370872, SGI370873, SGI370874, SGI370875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89038[] = {SGI370876, SGI370877, SGI370878, SGI370879, SGI370880, SGI370881, SGI370882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89039[] = {SGI370883, SGI370884, SGI370885, SGI370886, SGI370887, SGI370888, SGI370889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89040[] = {SGI370890, SGI370891, SGI370892, SGI370893, SGI370894, SGI370895, SGI370896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89041[] = {SGI370897, SGI370898, SGI370899, SGI370900, SGI370901, SGI370902, SGI370903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89042[] = {SGI370904, SGI370905, SGI370906, SGI370907, SGI370908, SGI370909, SGI370910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89043[] = {SGI370911, SGI370912, SGI370913, SGI370914, SGI370915, SGI370916, SGI370917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89044[] = {SGI370918, SGI370919, SGI370920, SGI370921, SGI370922, SGI370923, SGI370924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89045[] = {SGI370925, SGI370926, SGI370927, SGI370928, SGI370929, SGI370930, SGI370931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89046[] = {SGI370932, SGI370933, SGI370934, SGI370935, SGI370936, SGI370937, SGI370938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89047[] = {SGI370939, SGI370940, SGI370941, SGI370942, SGI370943, SGI370944, SGI370945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89048[] = {SGI370946, SGI370947, SGI370948, SGI370949, SGI370950, SGI0, SGI370949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89049[] = {SGI370951, SGI370952, SGI370953, SGI370954, SGI370955, SGI0, SGI370954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89050[] = {SGI370956, SGI370957, SGI370958, SGI370959, SGI370960, SGI370961, SGI370962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89051[] = {SGI370963, SGI370964, SGI370965, SGI370966, SGI370967, SGI370968, SGI370969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89052[] = {SGI370970, SGI370971, SGI370972, SGI370973, SGI370974, SGI370975, SGI370976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89053[] = {SGI370977, SGI370978, SGI370979, SGI370980, SGI370981, SGI370982, SGI370983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89054[] = {SGI370984, SGI370985, SGI370986, SGI370987, SGI370988, SGI370989, SGI370990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89055[] = {SGI370991, SGI370992, SGI370993, SGI370994, SGI370995, SGI0, SGI370994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89056[] = {SGI370996, SGI370997, SGI370998, SGI370999, SGI371000, SGI371001, SGI371002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89057[] = {SGI371003, SGI371004, SGI371005, SGI371006, SGI371007, SGI371008, SGI371009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89058[] = {SGI371010, SGI371011, SGI371012, SGI371013, SGI371014, SGI371015, SGI371016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89059[] = {SGI371017, SGI371018, SGI371019, SGI371020, SGI371021, SGI371022, SGI371023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89060[] = {SGI371024, SGI371025, SGI371026, SGI371027, SGI371028, SGI371029, SGI371030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89061[] = {SGI371031, SGI371032, SGI371033, SGI371034, SGI371035, SGI371036, SGI371037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89062[] = {SGI371038, SGI371039, SGI371040, SGI371041, SGI371042, SGI371043, SGI371044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89063[] = {SGI371045, SGI371046, SGI371047, SGI0, SGI371048, SGI0, SGI371049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89064[] = {SGI371050, SGI371051, SGI371052, SGI371053, SGI371054, SGI371055, SGI371056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89065[] = {SGI371057, SGI371058, SGI371059, SGI371060, SGI371061, SGI371062, SGI371063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89066[] = {SGI371064, SGI371065, SGI371066, SGI371067, SGI371068, SGI371069, SGI371070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89067[] = {SGI371071, SGI371072, SGI371073, SGI371074, SGI371075, SGI371076, SGI371077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89068[] = {SGI371078, SGI371079, SGI371080, SGI371081, SGI371082, SGI371083, SGI371084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89069[] = {SGI371085, SGI371086, SGI371087, SGI371088, SGI371089, SGI0, SGI371088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89070[] = {SGI371090, SGI371091, SGI371092, SGI371093, SGI371094, SGI371095, SGI371096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89071[] = {SGI371097, SGI371098, SGI371099, SGI371100, SGI371101, SGI371102, SGI371103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89072[] = {SGI371104, SGI371105, SGI371106, SGI371107, SGI371108, SGI371109, SGI371110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89073[] = {SGI371111, SGI371112, SGI371113, SGI371114, SGI371115, SGI371116, SGI371117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89074[] = {SGI371118, SGI371119, SGI371120, SGI371121, SGI371122, SGI371123, SGI371124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89075[] = {SGI371125, SGI371126, SGI371127, SGI371128, SGI371129, SGI371130, SGI371131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89076[] = {SGI371132, SGI371133, SGI371134, SGI371135, SGI371136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89077[] = {SGI371137, SGI371138, SGI371139, SGI371140, SGI371141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89078[] = {SGI371142, SGI371143, SGI371144, SGI371145, SGI371146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89079[] = {SGI371147, SGI371148, SGI371149, SGI371150, SGI371151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89080[] = {SGI371152, SGI371153, SGI371154, SGI371155, SGI371156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89081[] = {SGI371157, SGI371158, SGI371159, SGI371160, SGI371161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89082[] = {SGI371162, SGI371163, SGI371164, SGI371165, SGI371166, SGI371167, SGI371168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89083[] = {SGI371169, SGI371170, SGI371171, SGI371172, SGI371173, SGI371174, SGI371175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89084[] = {SGI371176, SGI371177, SGI371178, SGI371179, SGI371180, SGI371181, SGI371182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89085[] = {SGI371183, SGI371184, SGI371185, SGI371186, SGI371187, SGI371188, SGI371189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89086[] = {SGI371190, SGI371191, SGI371192, SGI371193, SGI371194, SGI371195, SGI371196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89087[] = {SGI371197, SGI371198, SGI371199, SGI371200, SGI371201, SGI371202, SGI371203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89088[] = {SGI371204, SGI371205, SGI371206, SGI371207, SGI371208, SGI371209, SGI371210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89089[] = {SGI371211, SGI371212, SGI371213, SGI0, SGI371214, SGI0, SGI371215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89090[] = {SGI371216, SGI371217, SGI371218, SGI371219, SGI371217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89091[] = {SGI371220, SGI371221, SGI371222, SGI371223, SGI371221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89092[] = {SGI371224, SGI371225, SGI371226, SGI371227, SGI371225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89093[] = {SGI371228, SGI371229, SGI371230, SGI371231, SGI371229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89094[] = {SGI371232, SGI371233, SGI371234, SGI371235, SGI371233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89095[] = {SGI371236, SGI371237, SGI371238, SGI371239, SGI371237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89096[] = {SGI371240, SGI371241, SGI371242, SGI0, SGI371241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89097[] = {SGI371243, SGI371244, SGI371245, SGI0, SGI371244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89098[] = {SGI371246, SGI371247, SGI371248, SGI371248, SGI371247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89099[] = {SGI371249, SGI371250, SGI371251, SGI371251, SGI371250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89100[] = {SGI371252, SGI371253, SGI371254, SGI0, SGI371253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89101[] = {SGI371255, SGI371256, SGI371257, SGI371257, SGI371256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89102[] = {SGI371258, SGI371259, SGI371260, SGI0, SGI371259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89103[] = {SGI371261, SGI371262, SGI371263, SGI371263, SGI371262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89104[] = {SGI371264, SGI371265, SGI371266, SGI0, SGI371265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89105[] = {SGI371267, SGI371268, SGI371269, SGI0, SGI371268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89106[] = {SGI371270, SGI371271, SGI371272, SGI371272, SGI371271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89107[] = {SGI371273, SGI371274, SGI371275, SGI0, SGI371274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89108[] = {SGI371276, SGI371277, SGI371278, SGI371278, SGI371277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89109[] = {SGI371279, SGI371280, SGI371281, SGI0, SGI371280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89110[] = {SGI371282, SGI371283, SGI371284, SGI0, SGI371285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89111[] = {SGI371286, SGI371287, SGI371288, SGI371288, SGI371289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89112[] = {SGI371290, SGI371291, SGI371292, SGI0, SGI371293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89113[] = {SGI371294, SGI371295, SGI371296, SGI371296, SGI371297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89114[] = {SGI371298, SGI371299, SGI371300, SGI371300, SGI371301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89115[] = {SGI371302, SGI371303, SGI371304, SGI0, SGI371305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89116[] = {SGI371306, SGI371307, SGI371308, SGI0, SGI371309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89117[] = {SGI371310, SGI371311, SGI371312, SGI371312, SGI371313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89118[] = {SGI371314, SGI371315, SGI371316, SGI371316, SGI371317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89119[] = {SGI371318, SGI371319, SGI371320, SGI0, SGI371321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89120[] = {SGI371322, SGI371323, SGI371324, SGI0, SGI371325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89121[] = {SGI371326, SGI371327, SGI371328, SGI371328, SGI371329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89122[] = {SGI371330, SGI371331, SGI371332, SGI371333, SGI371334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89123[] = {SGI371335, SGI371336, SGI371337, SGI371338, SGI371339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89124[] = {SGI371340, SGI371341, SGI371342, SGI371343, SGI371344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89125[] = {SGI371345, SGI371346, SGI371347, SGI371348, SGI371349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89126[] = {SGI371350, SGI371351, SGI371352, SGI371353, SGI371354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89127[] = {SGI371355, SGI371356, SGI371357, SGI0, SGI371358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89128[] = {SGI371359, SGI371360, SGI371361, SGI371361, SGI371362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89129[] = {SGI371363, SGI371364, SGI371365, SGI0, SGI371366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89130[] = {SGI371367, SGI371368, SGI371369, SGI371368, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89131[] = {SGI371370, SGI371371, SGI371372, SGI371371, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89132[] = {SGI371373, SGI371374, SGI371375, SGI371376, SGI371377, SGI0, SGI371378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89133[] = {SGI371379, SGI371380, SGI371381, SGI371382, SGI371383, SGI0, SGI371384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89134[] = {SGI371385, SGI371386, SGI371387, SGI371388, SGI371389, SGI0, SGI371390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89135[] = {SGI371391, SGI371392, SGI371393, SGI371394, SGI371395, SGI0, SGI371396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89136[] = {SGI371397, SGI371398, SGI371399, SGI371400, SGI0, SGI0, SGI371401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89137[] = {SGI371402, SGI371403, SGI371404, SGI371405, SGI371406, SGI0, SGI371407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89138[] = {SGI371408, SGI371409, SGI0, SGI371410, SGI371411, SGI0, SGI371412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89139[] = {SGI371413, SGI371414, SGI371415, SGI371416, SGI371417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89140[] = {SGI371418, SGI371419, SGI371420, SGI371419, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89141[] = {SGI371421, SGI371422, SGI371423, SGI371422, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89142[] = {SGI371424, SGI371425, SGI371426, SGI371425, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89143[] = {SGI371427, SGI371428, SGI371429, SGI371428, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89144[] = {SGI371430, SGI371431, SGI371432, SGI371431, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89145[] = {SGI371433, SGI371434, SGI371435, SGI371434, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89146[] = {SGI371436, SGI371437, SGI371438, SGI371437, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89147[] = {SGI371439, SGI371440, SGI371441, SGI371440, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89148[] = {SGI371442, SGI371443, SGI371444, SGI371443, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89149[] = {SGI371445, SGI371446, SGI371447, SGI371446, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89150[] = {SGI371448, SGI371449, SGI371450, SGI371449, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89151[] = {SGI126098, SGI371451, SGI371452, SGI371453, SGI126097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89152[] = {SGI130219, SGI371454, SGI371455, SGI371456, SGI130218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89153[] = {SGI371457, SGI371458, SGI371459, SGI371458, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89154[] = {SGI371460, SGI371461, SGI371462, SGI0, SGI0, SGI0, SGI371463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89155[] = {SGI371464, SGI371465, SGI371466, SGI0, SGI0, SGI0, SGI371467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89156[] = {SGI371468, SGI371469, SGI371470, SGI0, SGI0, SGI0, SGI371471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89157[] = {SGI371472, SGI371473, SGI371474, SGI0, SGI0, SGI0, SGI371475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89158[] = {SGI371476, SGI371477, SGI371478, SGI0, SGI0, SGI0, SGI371479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89159[] = {SGI371480, SGI371481, SGI371482, SGI0, SGI0, SGI0, SGI371483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89160[] = {SGI371484, SGI371485, SGI371486, SGI0, SGI0, SGI0, SGI371487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89161[] = {SGI0, SGI371488, SGI371489, SGI0, SGI0, SGI0, SGI371490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89162[] = {SGI371491, SGI371492, SGI371493, SGI0, SGI0, SGI0, SGI371494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89163[] = {SGI371495, SGI371496, SGI371497, SGI371496, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89164[] = {SGI371498, SGI371499, SGI371500, SGI371499, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89165[] = {SGI371501, SGI371502, SGI371503, SGI371502, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89166[] = {SGI371504, SGI371505, SGI371506, SGI371505, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89167[] = {SGI371507, SGI371508, SGI371509, SGI371508, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89168[] = {SGI371510, SGI371511, SGI0, SGI371511, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89169[] = {SGI371512, SGI371513, SGI0, SGI371513, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89170[] = {SGI371514, SGI371515, SGI0, SGI371515, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89171[] = {SGI371516, SGI371517, SGI0, SGI371517, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89172[] = {SGI371518, SGI371519, SGI0, SGI371520, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89173[] = {SGI371521, SGI371522, SGI0, SGI371523, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89174[] = {SGI371524, SGI371525, SGI0, SGI371525, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89175[] = {SGI371526, SGI371527, SGI371528, SGI371527, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89176[] = {SGI371529, SGI371530, SGI371531, SGI371530, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89177[] = {SGI371532, SGI371533, SGI371534, SGI371533, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89178[] = {SGI371535, SGI371536, SGI371537, SGI371538, SGI371536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89179[] = {SGI371539, SGI371540, SGI371541, SGI371542, SGI371540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89180[] = {SGI371543, SGI371544, SGI371545, SGI371546, SGI371544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89181[] = {SGI371551, SGI371552, SGI371553, SGI371552, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89182[] = {SGI371547, SGI371548, SGI371549, SGI371550, SGI371548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89183[] = {SGI371556, SGI371558, SGI371560, SGI371558, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89184[] = {SGI371554, SGI371555, SGI371557, SGI371559, SGI371555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89185[] = {SGI371561, SGI371562, SGI371563, SGI371562, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89186[] = {SGI371564, SGI371565, SGI371566, SGI371567, SGI371565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89187[] = {SGI371568, SGI371569, SGI371570, SGI371570, SGI371569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89188[] = {SGI371571, SGI371572, SGI371573, SGI371573, SGI371572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89189[] = {SGI371574, SGI371575, SGI371576, SGI0, SGI371575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89190[] = {SGI371577, SGI371578, SGI371579, SGI0, SGI371578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89191[] = {SGI371580, SGI371581, SGI371582, SGI0, SGI371581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89192[] = {SGI371583, SGI371584, SGI371585, SGI371585, SGI371584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89193[] = {SGI371586, SGI371587, SGI371588, SGI371588, SGI371587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89194[] = {SGI371589, SGI371590, SGI371591, SGI0, SGI371590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89195[] = {SGI371592, SGI371593, SGI371594, SGI0, SGI371593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89196[] = {SGI371595, SGI371596, SGI371597, SGI0, SGI371596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89197[] = {SGI371598, SGI371599, SGI371600, SGI371600, SGI371599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89198[] = {SGI371601, SGI371602, SGI371603, SGI371603, SGI371602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89199[] = {SGI371604, SGI371605, SGI371606, SGI371606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI371607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89200[] = {SGI371608, SGI371609, SGI371610, SGI371610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI371611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89201[] = {SGI371612, SGI371613, SGI371614, SGI371614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI371615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89202[] = {SGI371616, SGI371617, SGI0, SGI371617, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89203[] = {SGI371618, SGI371619, SGI0, SGI371619, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89204[] = {SGI371620, SGI371621, SGI0, SGI371621, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89205[] = {SGI371622, SGI371623, SGI0, SGI371623, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89206[] = {SGI371624, SGI371625, SGI0, SGI371625, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89207[] = {SGI371626, SGI371627, SGI0, SGI371627, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89208[] = {SGI371628, SGI371629, SGI371630, SGI371631, SGI371629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89209[] = {SGI371632, SGI371633, SGI371634, SGI371635, SGI371633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89210[] = {SGI371636, SGI371637, SGI371638, SGI371639, SGI371637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89211[] = {SGI371640, SGI371641, SGI371642, SGI371643, SGI371641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89212[] = {SGI371644, SGI371645, SGI371646, SGI371647, SGI371645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89213[] = {SGI371648, SGI371649, SGI371650, SGI371651, SGI371649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89214[] = {SGI371652, SGI371653, SGI371654, SGI371655, SGI371653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89215[] = {SGI371656, SGI371657, SGI371658, SGI371659, SGI371657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89216[] = {SGI371660, SGI371661, SGI371662, SGI371663, SGI371661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89217[] = {SGI371664, SGI371665, SGI371666, SGI371667, SGI371665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89218[] = {SGI371668, SGI371669, SGI371670, SGI371671, SGI371669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89219[] = {SGI371672, SGI371673, SGI371674, SGI371675, SGI371673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89220[] = {SGI371676, SGI371677, SGI371678, SGI371679, SGI371677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89221[] = {SGI371680, SGI371681, SGI371682, SGI371683, SGI371681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89222[] = {SGI371684, SGI371685, SGI371686, SGI371686, SGI371685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89223[] = {SGI371687, SGI371688, SGI371689, SGI371690, SGI371688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89224[] = {SGI371691, SGI371692, SGI371693, SGI371694, SGI371692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89225[] = {SGI371695, SGI371696, SGI371697, SGI371697, SGI371696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89226[] = {SGI371698, SGI371699, SGI371700, SGI0, SGI371699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89227[] = {SGI371701, SGI371702, SGI371703, SGI371703, SGI371702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89228[] = {SGI371704, SGI371705, SGI371706, SGI371706, SGI371705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89229[] = {SGI371707, SGI371708, SGI371709, SGI371709, SGI371708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89230[] = {SGI371710, SGI371711, SGI371712, SGI0, SGI371711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89231[] = {SGI371713, SGI371714, SGI371715, SGI371715, SGI371714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89232[] = {SGI371716, SGI371717, SGI371718, SGI0, SGI371717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89233[] = {SGI371719, SGI371720, SGI371721, SGI0, SGI371720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89234[] = {SGI371722, SGI371723, SGI371724, SGI0, SGI371723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89235[] = {SGI371725, SGI371726, SGI371727, SGI0, SGI371726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89236[] = {SGI371728, SGI371729, SGI371730, SGI0, SGI371729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89237[] = {SGI371731, SGI371732, SGI371733, SGI371733, SGI371732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89238[] = {SGI371734, SGI371735, SGI371736, SGI371737, SGI371735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89239[] = {SGI371738, SGI371739, SGI371740, SGI0, SGI371741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89240[] = {SGI371742, SGI371743, SGI371744, SGI371744, SGI371745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89241[] = {SGI371746, SGI371747, SGI371748, SGI0, SGI371749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89242[] = {SGI371750, SGI371751, SGI371752, SGI371752, SGI371753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89243[] = {SGI371754, SGI371755, SGI371756, SGI371756, SGI371757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89244[] = {SGI371758, SGI371759, SGI371760, SGI0, SGI371761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89245[] = {SGI371762, SGI371763, SGI371764, SGI371764, SGI371765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89246[] = {SGI371766, SGI371767, SGI371768, SGI371768, SGI371769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89247[] = {SGI371770, SGI371771, SGI371772, SGI0, SGI371773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89248[] = {SGI371774, SGI371775, SGI371776, SGI0, SGI371777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89249[] = {SGI371778, SGI371779, SGI371780, SGI371780, SGI371781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89250[] = {SGI371782, SGI371783, SGI371784, SGI0, SGI371785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89251[] = {SGI371786, SGI371787, SGI371788, SGI0, SGI371789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89252[] = {SGI371790, SGI371791, SGI371792, SGI371792, SGI371793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89253[] = {SGI371794, SGI371795, SGI371796, SGI371795, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89254[] = {SGI371797, SGI371798, SGI371799, SGI371798, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89255[] = {SGI371800, SGI371801, SGI371802, SGI371801, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89256[] = {SGI371803, SGI371804, SGI371805, SGI371804, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89257[] = {SGI371806, SGI371807, SGI371808, SGI371807, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89258[] = {SGI371809, SGI371810, SGI371811, SGI371810, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89259[] = {SGI371812, SGI371813, SGI371814, SGI371813, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89260[] = {SGI371815, SGI371816, SGI371817, SGI371816, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89261[] = {SGI371818, SGI371819, SGI371820, SGI371819, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89262[] = {SGI371821, SGI371822, SGI371823, SGI371822, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89263[] = {SGI371824, SGI371825, SGI371826, SGI0, SGI0, SGI0, SGI371827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89264[] = {SGI371828, SGI371829, SGI371830, SGI0, SGI0, SGI0, SGI371831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89265[] = {SGI371832, SGI371833, SGI371834, SGI0, SGI0, SGI0, SGI371835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89266[] = {SGI371836, SGI371837, SGI371838, SGI0, SGI0, SGI0, SGI371839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89267[] = {SGI0, SGI371840, SGI371841, SGI0, SGI0, SGI0, SGI371842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89268[] = {SGI371843, SGI371844, SGI371845, SGI0, SGI0, SGI0, SGI371846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89269[] = {SGI371847, SGI371848, SGI371849, SGI0, SGI0, SGI0, SGI371850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89270[] = {SGI371851, SGI371852, SGI371853, SGI0, SGI0, SGI0, SGI371854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89271[] = {SGI371855, SGI371856, SGI371857, SGI0, SGI0, SGI0, SGI371858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89272[] = {SGI371859, SGI371860, SGI371861, SGI0, SGI0, SGI0, SGI371862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89273[] = {SGI371863, SGI371864, SGI371865, SGI0, SGI0, SGI0, SGI371866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89274[] = {SGI371867, SGI371868, SGI371869, SGI371869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI371870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89275[] = {SGI371871, SGI371872, SGI371873, SGI371873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI371874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89276[] = {SGI371875, SGI371876, SGI371877, SGI371877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI371878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89277[] = {SGI371879, SGI371880, SGI371881, SGI371881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI371882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89278[] = {SGI371883, SGI371884, SGI371885, SGI371885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI371886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89279[] = {SGI371887, SGI371888, SGI371889, SGI371890, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89280[] = {SGI371891, SGI371892, SGI371893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89281[] = {SGI371894, SGI371895, SGI371896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI371897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89282[] = {SGI371898, SGI371899, SGI371900, SGI371901, SGI0, SGI371902, SGI371903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89283[] = {SGI0, SGI371904, SGI371905, SGI371906, SGI0, SGI371907, SGI371908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89284[] = {SGI371909, SGI371910, SGI371911, SGI371912, SGI0, SGI371913, SGI371914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89285[] = {SGI371915, SGI371916, SGI371917, SGI0, SGI0, SGI0, SGI371918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89286[] = {SGI371919, SGI371920, SGI371921, SGI371922, SGI0, SGI371923, SGI371924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89287[] = {SGI371925, SGI371926, SGI371927, SGI371928, SGI0, SGI371929, SGI371930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89288[] = {SGI371931, SGI371932, SGI371933, SGI371934, SGI0, SGI371935, SGI371936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89289[] = {SGI371937, SGI371938, SGI371939, SGI371940, SGI0, SGI371941, SGI371942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89290[] = {SGI371943, SGI371944, SGI371945, SGI0, SGI0, SGI0, SGI371946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89291[] = {SGI371947, SGI371948, SGI371949, SGI371950, SGI0, SGI371951, SGI371952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89292[] = {SGI371953, SGI371954, SGI371955, SGI0, SGI0, SGI0, SGI371956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89293[] = {SGI371957, SGI371958, SGI371959, SGI371960, SGI0, SGI371961, SGI371962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89294[] = {SGI371963, SGI371964, SGI371965, SGI371966, SGI0, SGI371967, SGI371968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89295[] = {SGI0, SGI371969, SGI371970, SGI0, SGI0, SGI0, SGI371971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89296[] = {SGI371972, SGI371973, SGI371974, SGI371975, SGI0, SGI371976, SGI371977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89297[] = {SGI371978, SGI371979, SGI371980, SGI0, SGI0, SGI0, SGI371981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89298[] = {SGI371982, SGI371983, SGI371984, SGI0, SGI0, SGI0, SGI371985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89299[] = {SGI371986, SGI371987, SGI371988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89300[] = {SGI371989, SGI371990, SGI371991, SGI371992, SGI0, SGI371993, SGI371994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89301[] = {SGI371995, SGI371996, SGI371997, SGI0, SGI0, SGI0, SGI371998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89302[] = {SGI371999, SGI372000, SGI372001, SGI372002, SGI0, SGI372003, SGI372004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89303[] = {SGI372005, SGI372006, SGI372007, SGI0, SGI0, SGI0, SGI372008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89304[] = {SGI372009, SGI372010, SGI372011, SGI0, SGI372012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89305[] = {SGI372013, SGI372014, SGI372015, SGI0, SGI372016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89306[] = {SGI372017, SGI372018, SGI372019, SGI372019, SGI372020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89307[] = {SGI372021, SGI372022, SGI372023, SGI372023, SGI372024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89308[] = {SGI372025, SGI372026, SGI372027, SGI372027, SGI372028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89309[] = {SGI372029, SGI372030, SGI372031, SGI0, SGI372032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89310[] = {SGI372033, SGI372034, SGI372035, SGI372035, SGI372036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89311[] = {SGI372037, SGI372038, SGI372039, SGI0, SGI372040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89312[] = {SGI372041, SGI372042, SGI372043, SGI372043, SGI372044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89313[] = {SGI372045, SGI372046, SGI372047, SGI372047, SGI372048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89314[] = {SGI372049, SGI372050, SGI372051, SGI0, SGI372052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89315[] = {SGI372053, SGI372054, SGI372055, SGI372055, SGI372056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89316[] = {SGI372057, SGI372058, SGI372059, SGI0, SGI372060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89317[] = {SGI372061, SGI372062, SGI372063, SGI0, SGI372064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89318[] = {SGI372065, SGI372066, SGI372067, SGI0, SGI372068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89319[] = {SGI372069, SGI372070, SGI372071, SGI372071, SGI372072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89320[] = {SGI372073, SGI372074, SGI372075, SGI0, SGI372076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89321[] = {SGI372077, SGI372078, SGI372079, SGI372079, SGI372080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89322[] = {SGI372081, SGI372082, SGI372083, SGI372083, SGI372084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89323[] = {SGI372085, SGI372086, SGI372087, SGI0, SGI372088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89324[] = {SGI372089, SGI372090, SGI372091, SGI372091, SGI372092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89325[] = {SGI372093, SGI372094, SGI372095, SGI372095, SGI372096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89326[] = {SGI372097, SGI372098, SGI372099, SGI0, SGI372100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89327[] = {SGI372101, SGI372102, SGI372103, SGI0, SGI372104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89328[] = {SGI372105, SGI372106, SGI372107, SGI0, SGI372108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89329[] = {SGI372109, SGI372110, SGI372111, SGI0, SGI372112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89330[] = {SGI372113, SGI372114, SGI372115, SGI372115, SGI372116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89331[] = {SGI372117, SGI372118, SGI372119, SGI372120, SGI372121, SGI372122, SGI372123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89332[] = {SGI372124, SGI372125, SGI372126, SGI372127, SGI372128, SGI372129, SGI372130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89333[] = {SGI372131, SGI372132, SGI372133, SGI372134, SGI372135, SGI372136, SGI372137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89334[] = {SGI372138, SGI372139, SGI372140, SGI372141, SGI372142, SGI372143, SGI372144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89335[] = {SGI372145, SGI372146, SGI372147, SGI372148, SGI372149, SGI372150, SGI372151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89336[] = {SGI372152, SGI372153, SGI372154, SGI372155, SGI372156, SGI372157, SGI372158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89337[] = {SGI372159, SGI372160, SGI372161, SGI372162, SGI372163, SGI372164, SGI372165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89338[] = {SGI372166, SGI372167, SGI372168, SGI372169, SGI372170, SGI372171, SGI372172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89339[] = {SGI372173, SGI372174, SGI372175, SGI372176, SGI372177, SGI372178, SGI372179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89340[] = {SGI372180, SGI372181, SGI372182, SGI372183, SGI372184, SGI372185, SGI372186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89341[] = {SGI372187, SGI372188, SGI372189, SGI372190, SGI372191, SGI372192, SGI372193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89342[] = {SGI372194, SGI372195, SGI372196, SGI372197, SGI372198, SGI372199, SGI372200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89343[] = {SGI372201, SGI372202, SGI372203, SGI372204, SGI372205, SGI372206, SGI372207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89344[] = {SGI372208, SGI372209, SGI372210, SGI372211, SGI372212, SGI372213, SGI372214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89345[] = {SGI372215, SGI372216, SGI372217, SGI372218, SGI372219, SGI372220, SGI372221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89346[] = {SGI372222, SGI372223, SGI372224, SGI372225, SGI372226, SGI372227, SGI372228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89347[] = {SGI372229, SGI372230, SGI372231, SGI372232, SGI372233, SGI372234, SGI372235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89348[] = {SGI372236, SGI372237, SGI372238, SGI372239, SGI372240, SGI372241, SGI372242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89349[] = {SGI372243, SGI372244, SGI372245, SGI372246, SGI372247, SGI372248, SGI372249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89350[] = {SGI372250, SGI372251, SGI372252, SGI372253, SGI372254, SGI372255, SGI372256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89351[] = {SGI372257, SGI372258, SGI372259, SGI372260, SGI372261, SGI372262, SGI372263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89352[] = {SGI372264, SGI372265, SGI372266, SGI372267, SGI372268, SGI372269, SGI372270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89353[] = {SGI372271, SGI372272, SGI372273, SGI372274, SGI372275, SGI372276, SGI372277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89354[] = {SGI372278, SGI372279, SGI372280, SGI372281, SGI372282, SGI372283, SGI372284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89355[] = {SGI372285, SGI372286, SGI372287, SGI372288, SGI372289, SGI372290, SGI372291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89356[] = {SGI372292, SGI372293, SGI372294, SGI372295, SGI372296, SGI372297, SGI372298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89357[] = {SGI372299, SGI372300, SGI372301, SGI372302, SGI372303, SGI372304, SGI372305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89358[] = {SGI372306, SGI372307, SGI372308, SGI372309, SGI372310, SGI372311, SGI372312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89359[] = {SGI372313, SGI372314, SGI372315, SGI372316, SGI372317, SGI372318, SGI372319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89360[] = {SGI372320, SGI372321, SGI372322, SGI372323, SGI372324, SGI372325, SGI372326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89361[] = {SGI372327, SGI372328, SGI372329, SGI372330, SGI372331, SGI372332, SGI372333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89362[] = {SGI372334, SGI372335, SGI372336, SGI372337, SGI372338, SGI372339, SGI372340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89363[] = {SGI372341, SGI372342, SGI372343, SGI372344, SGI372345, SGI372346, SGI372347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89364[] = {SGI372348, SGI372349, SGI372350, SGI372351, SGI372352, SGI372353, SGI372354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89365[] = {SGI372355, SGI372356, SGI372357, SGI372358, SGI372359, SGI372360, SGI372361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89366[] = {SGI372362, SGI372363, SGI372364, SGI372365, SGI372366, SGI372367, SGI372368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89367[] = {SGI372369, SGI372370, SGI372371, SGI372372, SGI372373, SGI372374, SGI372375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89368[] = {SGI372376, SGI372377, SGI372378, SGI372379, SGI372380, SGI372381, SGI372382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89369[] = {SGI372383, SGI372384, SGI372385, SGI372386, SGI372387, SGI372388, SGI372389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89370[] = {SGI372390, SGI372391, SGI372392, SGI372393, SGI372394, SGI372395, SGI372396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89371[] = {SGI372397, SGI372398, SGI372399, SGI372400, SGI372401, SGI372402, SGI372403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89372[] = {SGI372404, SGI372405, SGI372406, SGI372407, SGI372408, SGI372409, SGI372410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89373[] = {SGI372411, SGI372412, SGI372413, SGI372414, SGI372415, SGI372416, SGI372417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89374[] = {SGI372418, SGI372419, SGI372420, SGI372421, SGI372422, SGI372423, SGI372424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89375[] = {SGI372425, SGI372426, SGI372427, SGI372428, SGI372429, SGI372430, SGI372431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89376[] = {SGI372432, SGI372433, SGI372434, SGI372435, SGI372436, SGI0, SGI372437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89377[] = {SGI372438, SGI372439, SGI372440, SGI372441, SGI372442, SGI372443, SGI372444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89378[] = {SGI372445, SGI372446, SGI372447, SGI372448, SGI372449, SGI0, SGI372450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89379[] = {SGI372451, SGI372452, SGI372453, SGI372454, SGI372455, SGI372456, SGI372457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89380[] = {SGI372458, SGI372459, SGI372460, SGI372461, SGI372462, SGI372463, SGI372464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89381[] = {SGI372465, SGI372466, SGI372467, SGI372468, SGI372469, SGI372470, SGI372471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89382[] = {SGI372472, SGI372473, SGI372474, SGI372475, SGI372476, SGI372477, SGI372478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89383[] = {SGI372479, SGI372480, SGI372481, SGI372482, SGI372483, SGI372484, SGI372485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89384[] = {SGI372486, SGI372487, SGI372488, SGI372489, SGI372490, SGI372491, SGI372492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89385[] = {SGI372493, SGI372494, SGI372495, SGI372496, SGI372497, SGI372498, SGI372499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89386[] = {SGI372500, SGI372501, SGI372502, SGI372503, SGI372504, SGI372505, SGI372506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89387[] = {SGI372507, SGI372508, SGI372509, SGI372510, SGI372511, SGI372512, SGI372513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89388[] = {SGI372514, SGI372515, SGI372516, SGI0, SGI372517, SGI0, SGI372518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89389[] = {SGI372519, SGI372520, SGI372521, SGI0, SGI372522, SGI0, SGI372523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89390[] = {SGI372524, SGI372525, SGI372526, SGI372527, SGI372528, SGI372529, SGI372530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89391[] = {SGI372531, SGI372532, SGI372533, SGI372534, SGI372535, SGI0, SGI372536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89392[] = {SGI372537, SGI372538, SGI372539, SGI372540, SGI372541, SGI372542, SGI372543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89393[] = {SGI372544, SGI372545, SGI372546, SGI372547, SGI372548, SGI372549, SGI372550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89394[] = {SGI372551, SGI372552, SGI372553, SGI372554, SGI372555, SGI372556, SGI372557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89395[] = {SGI372558, SGI372559, SGI372560, SGI372561, SGI372562, SGI372563, SGI372564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89396[] = {SGI372565, SGI372566, SGI372567, SGI372568, SGI372569, SGI372570, SGI372571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89397[] = {SGI372572, SGI372573, SGI372574, SGI372575, SGI372576, SGI372577, SGI372578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89398[] = {SGI372579, SGI372580, SGI372581, SGI0, SGI372582, SGI0, SGI372583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89399[] = {SGI372584, SGI372585, SGI372586, SGI372587, SGI372588, SGI372589, SGI372590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89400[] = {SGI372591, SGI372592, SGI372593, SGI372594, SGI372595, SGI372596, SGI372597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89401[] = {SGI372603, SGI372606, SGI372607, SGI372606, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89402[] = {SGI372598, SGI372599, SGI372600, SGI372601, SGI372602, SGI372604, SGI372605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89403[] = {SGI372608, SGI372609, SGI372610, SGI372611, SGI372612, SGI372613, SGI372614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89404[] = {SGI372615, SGI372616, SGI372617, SGI372616, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89405[] = {SGI372618, SGI372619, SGI372620, SGI372621, SGI372622, SGI372623, SGI372624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89406[] = {SGI372625, SGI372626, SGI372627, SGI372628, SGI372629, SGI372630, SGI372631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89407[] = {SGI372632, SGI372633, SGI372634, SGI372635, SGI372636, SGI372637, SGI372638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89408[] = {SGI372639, SGI372640, SGI372641, SGI372642, SGI372643, SGI372644, SGI372645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89409[] = {SGI372646, SGI372647, SGI372649, SGI372647, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89410[] = {SGI372648, SGI372650, SGI372651, SGI372652, SGI372653, SGI372654, SGI372655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89411[] = {SGI372656, SGI372657, SGI372658, SGI372657, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89412[] = {SGI372659, SGI372660, SGI372661, SGI372662, SGI372663, SGI372664, SGI372665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89413[] = {SGI372668, SGI372670, SGI372672, SGI372670, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89414[] = {SGI372666, SGI372667, SGI372669, SGI372671, SGI372673, SGI372674, SGI372675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89415[] = {SGI372676, SGI372677, SGI372678, SGI372679, SGI372680, SGI372681, SGI372682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89416[] = {SGI372683, SGI372684, SGI372685, SGI372686, SGI372687, SGI372688, SGI372689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89417[] = {SGI372690, SGI372691, SGI372692, SGI372691, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89418[] = {SGI372693, SGI372694, SGI372695, SGI372696, SGI372697, SGI372698, SGI372699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89419[] = {SGI372700, SGI372701, SGI372702, SGI372703, SGI372704, SGI372705, SGI372706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89420[] = {SGI372707, SGI372708, SGI372709, SGI372710, SGI372711, SGI372712, SGI372713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89421[] = {SGI372714, SGI372715, SGI372716, SGI372717, SGI372718, SGI372719, SGI372720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89422[] = {SGI372721, SGI372722, SGI372723, SGI372724, SGI372725, SGI372726, SGI372727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89423[] = {SGI372728, SGI372729, SGI372730, SGI372731, SGI372732, SGI372733, SGI372734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89424[] = {SGI372735, SGI372736, SGI372737, SGI372738, SGI372739, SGI372740, SGI372741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89425[] = {SGI372742, SGI372743, SGI372744, SGI372745, SGI372746, SGI372747, SGI372748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89426[] = {SGI372749, SGI372750, SGI372751, SGI372752, SGI372753, SGI372754, SGI372755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89427[] = {SGI372756, SGI372757, SGI372758, SGI372759, SGI372760, SGI372761, SGI372762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89428[] = {SGI372763, SGI372764, SGI372765, SGI372766, SGI372767, SGI372768, SGI372769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89429[] = {SGI372770, SGI372771, SGI372772, SGI372773, SGI372774, SGI372775, SGI372776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89430[] = {SGI372777, SGI372778, SGI372779, SGI372780, SGI372781, SGI372782, SGI372783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89431[] = {SGI372784, SGI372785, SGI372786, SGI372787, SGI372788, SGI372789, SGI372790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89432[] = {SGI372791, SGI372792, SGI372793, SGI372794, SGI372795, SGI372796, SGI372797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89433[] = {SGI372798, SGI372799, SGI372800, SGI372801, SGI372802, SGI372803, SGI372804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89434[] = {SGI372805, SGI372806, SGI372807, SGI372808, SGI372809, SGI372810, SGI372811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89435[] = {SGI372812, SGI372813, SGI372814, SGI372815, SGI372816, SGI372817, SGI372818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89436[] = {SGI372819, SGI372820, SGI372821, SGI372822, SGI372823, SGI372824, SGI372825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89437[] = {SGI372826, SGI372827, SGI372828, SGI372829, SGI372830, SGI372831, SGI372832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89438[] = {SGI372833, SGI372834, SGI372835, SGI372836, SGI372837, SGI372838, SGI372839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89439[] = {SGI372840, SGI372841, SGI372842, SGI372843, SGI372844, SGI372845, SGI372846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89440[] = {SGI372847, SGI372848, SGI372849, SGI372850, SGI372851, SGI372852, SGI372853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89441[] = {SGI372854, SGI372855, SGI372856, SGI372857, SGI372858, SGI372859, SGI372860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89442[] = {SGI372861, SGI372862, SGI372863, SGI372864, SGI372865, SGI372866, SGI372867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89443[] = {SGI372868, SGI372869, SGI372870, SGI372871, SGI372872, SGI372873, SGI372874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89444[] = {SGI372875, SGI372876, SGI372877, SGI372878, SGI372879, SGI372880, SGI372881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89445[] = {SGI372882, SGI372883, SGI372884, SGI372885, SGI372886, SGI372887, SGI372888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89446[] = {SGI372889, SGI372890, SGI372891, SGI372892, SGI372893, SGI372894, SGI372895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89447[] = {SGI372896, SGI372897, SGI372898, SGI372899, SGI372900, SGI372901, SGI372902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89448[] = {SGI372903, SGI372904, SGI372905, SGI372906, SGI372907, SGI372908, SGI372909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89449[] = {SGI372910, SGI372911, SGI372912, SGI372913, SGI372914, SGI372915, SGI372916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89450[] = {SGI372917, SGI372918, SGI372919, SGI372920, SGI372921, SGI372922, SGI372923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89451[] = {SGI372924, SGI372925, SGI372926, SGI372927, SGI372928, SGI372929, SGI372930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89452[] = {SGI372931, SGI372932, SGI372933, SGI372934, SGI372935, SGI372936, SGI372937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89453[] = {SGI372938, SGI372939, SGI372940, SGI372941, SGI372942, SGI372943, SGI372944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89454[] = {SGI372945, SGI372946, SGI372947, SGI372948, SGI372949, SGI372950, SGI372951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89455[] = {SGI372952, SGI372953, SGI372954, SGI372955, SGI372956, SGI372957, SGI372958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89456[] = {SGI372959, SGI372960, SGI372961, SGI372962, SGI372963, SGI372964, SGI372965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89457[] = {SGI372966, SGI372967, SGI372968, SGI372969, SGI372970, SGI372971, SGI372972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89458[] = {SGI372973, SGI372974, SGI372975, SGI372976, SGI372977, SGI372978, SGI372979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89459[] = {SGI372980, SGI372981, SGI372982, SGI372983, SGI372984, SGI372985, SGI372986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89460[] = {SGI372987, SGI372988, SGI372989, SGI372990, SGI372991, SGI372992, SGI372993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89461[] = {SGI372994, SGI372995, SGI372996, SGI372997, SGI372998, SGI372999, SGI373000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89462[] = {SGI373001, SGI373002, SGI373003, SGI373004, SGI373005, SGI373006, SGI373007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89463[] = {SGI373008, SGI373009, SGI373010, SGI373011, SGI373012, SGI373013, SGI373014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89464[] = {SGI373015, SGI373016, SGI373017, SGI373018, SGI373019, SGI373020, SGI373021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89465[] = {SGI373022, SGI373023, SGI373024, SGI373025, SGI373026, SGI373027, SGI373028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89466[] = {SGI373029, SGI373030, SGI373031, SGI373032, SGI373033, SGI373034, SGI373035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89467[] = {SGI373036, SGI373037, SGI373038, SGI373039, SGI373040, SGI373041, SGI373042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89468[] = {SGI373043, SGI373044, SGI373045, SGI373046, SGI373047, SGI373048, SGI373049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89469[] = {SGI373050, SGI373051, SGI373052, SGI373053, SGI373054, SGI373055, SGI373056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89470[] = {SGI373057, SGI373058, SGI373059, SGI373060, SGI373061, SGI373062, SGI373063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89471[] = {SGI373064, SGI373065, SGI373066, SGI373067, SGI373068, SGI373069, SGI373070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89472[] = {SGI373071, SGI373072, SGI373073, SGI373074, SGI373075, SGI373076, SGI373077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89473[] = {SGI373078, SGI373079, SGI373080, SGI373081, SGI373082, SGI373083, SGI373084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89474[] = {SGI373085, SGI373086, SGI373087, SGI373088, SGI373089, SGI373090, SGI373091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89475[] = {SGI373092, SGI373093, SGI373094, SGI373095, SGI373096, SGI373097, SGI373098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89476[] = {SGI373099, SGI373100, SGI373101, SGI373102, SGI373103, SGI373104, SGI373105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89477[] = {SGI373106, SGI373107, SGI373108, SGI373109, SGI373110, SGI373111, SGI373112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89478[] = {SGI373113, SGI373114, SGI373115, SGI373116, SGI373117, SGI373118, SGI373119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89479[] = {SGI373120, SGI373121, SGI373122, SGI373123, SGI373124, SGI373125, SGI373126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89480[] = {SGI373127, SGI373128, SGI373129, SGI373130, SGI373131, SGI373132, SGI373133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89481[] = {SGI373134, SGI373135, SGI373136, SGI0, SGI373137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89482[] = {SGI373138, SGI373139, SGI373140, SGI0, SGI373141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89483[] = {SGI373142, SGI373143, SGI373144, SGI0, SGI373145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89484[] = {SGI373146, SGI373147, SGI373148, SGI0, SGI373149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89485[] = {SGI373150, SGI373151, SGI373152, SGI0, SGI373153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89486[] = {SGI373154, SGI373155, SGI373156, SGI0, SGI373157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89487[] = {SGI373158, SGI373159, SGI373160, SGI0, SGI373161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89488[] = {SGI373162, SGI373163, SGI373164, SGI0, SGI373165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89489[] = {SGI373166, SGI373167, SGI373168, SGI0, SGI373169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89490[] = {SGI373170, SGI373171, SGI373172, SGI0, SGI373173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89491[] = {SGI373174, SGI373175, SGI373176, SGI373175, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89492[] = {SGI373177, SGI373178, SGI373179, SGI373178, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89493[] = {SGI373180, SGI373181, SGI373182, SGI373181, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89494[] = {SGI373183, SGI373184, SGI373185, SGI373184, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89495[] = {SGI373186, SGI373187, SGI373188, SGI373187, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89496[] = {SGI373189, SGI373190, SGI373191, SGI373190, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89497[] = {SGI373192, SGI373193, SGI373194, SGI373193, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89498[] = {SGI373195, SGI373196, SGI373197, SGI373196, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89499[] = {SGI373198, SGI373199, SGI373200, SGI373199, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89500[] = {SGI373201, SGI373202, SGI373203, SGI373202, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89501[] = {SGI373204, SGI373205, SGI373206, SGI373205, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89502[] = {SGI373207, SGI373208, SGI373209, SGI373208, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89503[] = {SGI373210, SGI373211, SGI373212, SGI373211, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89504[] = {SGI373213, SGI373214, SGI373215, SGI373214, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89505[] = {SGI373216, SGI373217, SGI373218, SGI373217, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89506[] = {SGI373219, SGI373220, SGI373221, SGI373220, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89507[] = {SGI373222, SGI373223, SGI373224, SGI373223, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89508[] = {SGI373225, SGI373226, SGI373227, SGI373226, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89509[] = {SGI373228, SGI373229, SGI373230, SGI373229, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89510[] = {SGI373231, SGI373232, SGI373233, SGI373232, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89511[] = {SGI373234, SGI373235, SGI373236, SGI373235, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89512[] = {SGI373237, SGI373238, SGI373239, SGI373238, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89513[] = {SGI373240, SGI373241, SGI373242, SGI373241, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89514[] = {SGI373243, SGI373244, SGI373245, SGI373244, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89515[] = {SGI373246, SGI373247, SGI373248, SGI373247, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89516[] = {SGI373249, SGI373250, SGI373251, SGI373250, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89517[] = {SGI373252, SGI373253, SGI373254, SGI373253, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89518[] = {SGI373255, SGI373256, SGI373257, SGI373256, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89519[] = {SGI373258, SGI373259, SGI373260, SGI373259, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89520[] = {SGI373261, SGI373262, SGI373263, SGI373262, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89521[] = {SGI373264, SGI373265, SGI373266, SGI373265, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89522[] = {SGI373267, SGI373268, SGI373269, SGI373268, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89523[] = {SGI373270, SGI373271, SGI373272, SGI373271, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89524[] = {SGI373273, SGI373274, SGI373275, SGI373274, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89525[] = {SGI373276, SGI373277, SGI373278, SGI373277, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89526[] = {SGI373279, SGI373280, SGI373281, SGI373280, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89527[] = {SGI373282, SGI373283, SGI373284, SGI373283, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89528[] = {SGI373285, SGI373286, SGI373287, SGI373286, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89529[] = {SGI373288, SGI373289, SGI373290, SGI373289, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89530[] = {SGI373291, SGI373292, SGI373293, SGI373292, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89531[] = {SGI373294, SGI373295, SGI373296, SGI373295, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89532[] = {SGI373297, SGI373298, SGI373299, SGI373298, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89533[] = {SGI373300, SGI373301, SGI373302, SGI373301, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89534[] = {SGI373303, SGI373304, SGI373305, SGI373304, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89535[] = {SGI373306, SGI373307, SGI373308, SGI373307, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89536[] = {SGI373309, SGI373310, SGI373311, SGI373310, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89537[] = {SGI373312, SGI373313, SGI373314, SGI373313, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89538[] = {SGI373315, SGI373316, SGI373317, SGI373316, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89539[] = {SGI373318, SGI373319, SGI373320, SGI373319, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89540[] = {SGI373321, SGI373322, SGI373323, SGI373322, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89541[] = {SGI373324, SGI373325, SGI373326, SGI373325, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89542[] = {SGI373327, SGI373328, SGI373329, SGI373328, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89543[] = {SGI373330, SGI373331, SGI373332, SGI373331, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89544[] = {SGI373333, SGI373334, SGI373335, SGI373334, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89545[] = {SGI373336, SGI373337, SGI373338, SGI373337, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89546[] = {SGI373339, SGI373340, SGI373341, SGI373340, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89547[] = {SGI373342, SGI373343, SGI373344, SGI373343, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89548[] = {SGI373345, SGI373346, SGI373347, SGI373346, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89549[] = {SGI373348, SGI373349, SGI373350, SGI373349, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89550[] = {SGI373351, SGI373352, SGI373353, SGI373352, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89551[] = {SGI373354, SGI373355, SGI373356, SGI373355, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89552[] = {SGI373357, SGI373358, SGI373359, SGI373358, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89553[] = {SGI373360, SGI373361, SGI373362, SGI373361, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89554[] = {SGI373363, SGI373364, SGI373365, SGI373364, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89555[] = {SGI373366, SGI373367, SGI373368, SGI373367, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89556[] = {SGI373369, SGI373370, SGI373371, SGI373370, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89557[] = {SGI373372, SGI373373, SGI373374, SGI373373, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89558[] = {SGI373375, SGI373376, SGI373377, SGI373376, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89559[] = {SGI373378, SGI373379, SGI373380, SGI373379, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89560[] = {SGI373381, SGI373382, SGI373383, SGI373382, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89561[] = {SGI373384, SGI373385, SGI373386, SGI373385, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89562[] = {SGI373387, SGI373388, SGI373389, SGI373388, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89563[] = {SGI373390, SGI373391, SGI373392, SGI373391, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89564[] = {SGI373393, SGI373394, SGI373395, SGI373394, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89565[] = {SGI373396, SGI373397, SGI373398, SGI373397, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89566[] = {SGI373399, SGI373400, SGI373401, SGI373400, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89567[] = {SGI373402, SGI373403, SGI373404, SGI373403, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89568[] = {SGI373405, SGI373406, SGI373407, SGI373406, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89569[] = {SGI373408, SGI373409, SGI373410, SGI373409, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89570[] = {SGI373411, SGI373412, SGI373413, SGI373412, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89571[] = {SGI373414, SGI373415, SGI373416, SGI373415, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89572[] = {SGI373417, SGI373418, SGI373419, SGI373418, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89573[] = {SGI373420, SGI373421, SGI373422, SGI373421, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89574[] = {SGI373423, SGI373424, SGI373425, SGI373424, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89575[] = {SGI373426, SGI373427, SGI373428, SGI373427, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89576[] = {SGI373429, SGI373430, SGI373431, SGI373430, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89577[] = {SGI373432, SGI373433, SGI373434, SGI373433, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89578[] = {SGI373435, SGI373436, SGI373437, SGI373436, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89579[] = {SGI373438, SGI373439, SGI373440, SGI373439, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89580[] = {SGI373441, SGI373442, SGI373443, SGI373442, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89581[] = {SGI373444, SGI373445, SGI373446, SGI373445, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89582[] = {SGI373447, SGI373448, SGI373449, SGI373448, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89583[] = {SGI373450, SGI373451, SGI373452, SGI373451, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89584[] = {SGI373453, SGI373454, SGI373455, SGI373454, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89585[] = {SGI373456, SGI373457, SGI373458, SGI373457, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89586[] = {SGI373459, SGI373460, SGI373461, SGI373460, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89587[] = {SGI373462, SGI373463, SGI373464, SGI373463, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89588[] = {SGI373465, SGI373466, SGI373467, SGI373466, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89589[] = {SGI373468, SGI373469, SGI373470, SGI373469, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89590[] = {SGI373471, SGI373472, SGI373473, SGI373472, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89591[] = {SGI373474, SGI373475, SGI373476, SGI373475, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89592[] = {SGI373477, SGI373478, SGI373479, SGI373478, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89593[] = {SGI373480, SGI373481, SGI373482, SGI373481, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89594[] = {SGI373483, SGI373484, SGI373485, SGI373484, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89595[] = {SGI373486, SGI373487, SGI373488, SGI373487, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89596[] = {SGI373489, SGI373490, SGI373491, SGI373490, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89597[] = {SGI373492, SGI373493, SGI373494, SGI373493, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89598[] = {SGI373495, SGI373496, SGI373497, SGI373496, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89599[] = {SGI373498, SGI373499, SGI373500, SGI373499, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89600[] = {SGI373501, SGI373502, SGI373503, SGI373502, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89601[] = {SGI373504, SGI373505, SGI373506, SGI373505, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89602[] = {SGI373507, SGI373508, SGI373509, SGI373508, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89603[] = {SGI373510, SGI373511, SGI373512, SGI373511, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89604[] = {SGI373513, SGI373514, SGI373515, SGI373514, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89605[] = {SGI373516, SGI373517, SGI373518, SGI373517, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89606[] = {SGI373519, SGI373520, SGI373521, SGI373520, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89607[] = {SGI373522, SGI373523, SGI373524, SGI373523, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89608[] = {SGI373525, SGI373526, SGI373527, SGI373526, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89609[] = {SGI373528, SGI373529, SGI373530, SGI373529, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89610[] = {SGI373531, SGI373532, SGI373533, SGI373532, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89611[] = {SGI373534, SGI373535, SGI373536, SGI373535, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89612[] = {SGI373537, SGI373538, SGI373539, SGI373538, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89613[] = {SGI373540, SGI373541, SGI373542, SGI373541, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89614[] = {SGI373543, SGI373544, SGI373545, SGI373544, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89615[] = {SGI373546, SGI373547, SGI373548, SGI373547, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89616[] = {SGI373549, SGI373550, SGI373551, SGI373550, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89617[] = {SGI373552, SGI373553, SGI373554, SGI373553, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89618[] = {SGI373555, SGI373556, SGI373557, SGI373556, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89619[] = {SGI373558, SGI373559, SGI373560, SGI373559, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89620[] = {SGI373561, SGI373562, SGI373563, SGI373562, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89621[] = {SGI373564, SGI373565, SGI373566, SGI373565, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89622[] = {SGI373567, SGI373568, SGI373569, SGI373570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI373571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89623[] = {SGI373572, SGI373573, SGI373574, SGI373575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI373576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89624[] = {SGI373577, SGI373578, SGI373579, SGI373580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI373581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89625[] = {SGI373582, SGI373583, SGI373584, SGI373585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI373586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89626[] = {SGI373587, SGI373588, SGI373589, SGI373590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI373591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89627[] = {SGI373592, SGI373593, SGI373594, SGI373595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI373596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89628[] = {SGI373597, SGI288172, SGI288173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89629[] = {SGI373598, SGI288207, SGI288208, SGI288208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89630[] = {SGI373599, SGI373600, SGI0, SGI373600, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89631[] = {SGI373601, SGI373602, SGI0, SGI373602, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89632[] = {SGI373603, SGI373604, SGI373605, SGI373604, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89633[] = {SGI373606, SGI373607, SGI373608, SGI373607, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89634[] = {SGI373609, SGI373610, SGI373611, SGI373610, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89635[] = {SGI373612, SGI373613, SGI373614, SGI373613, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89636[] = {SGI373615, SGI373616, SGI373617, SGI373616, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89637[] = {SGI373618, SGI373619, SGI373620, SGI373619, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89638[] = {SGI373621, SGI373622, SGI373623, SGI373622, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89639[] = {SGI373624, SGI373625, SGI373626, SGI373625, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89640[] = {SGI373627, SGI373628, SGI373629, SGI373628, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89641[] = {SGI373630, SGI373631, SGI373632, SGI373631, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89642[] = {SGI373633, SGI373634, SGI373635, SGI373634, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89643[] = {SGI373636, SGI373637, SGI373638, SGI373637, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89644[] = {SGI373639, SGI373640, SGI373641, SGI373640, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89645[] = {SGI373642, SGI373643, SGI373644, SGI373643, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89646[] = {SGI373645, SGI373646, SGI373647, SGI373646, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89647[] = {SGI373648, SGI373649, SGI373650, SGI373649, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89648[] = {SGI373651, SGI373652, SGI373653, SGI373652, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89649[] = {SGI373654, SGI373655, SGI373656, SGI373655, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89650[] = {SGI373657, SGI373658, SGI373659, SGI373658, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89651[] = {SGI373660, SGI373661, SGI373662, SGI373661, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89652[] = {SGI373663, SGI373664, SGI373665, SGI373664, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89653[] = {SGI373666, SGI373667, SGI373668, SGI373667, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89654[] = {SGI373669, SGI373670, SGI373671, SGI373670, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89655[] = {SGI373672, SGI373673, SGI373674, SGI373673, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89656[] = {SGI373675, SGI373676, SGI373677, SGI373676, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89657[] = {SGI373678, SGI373679, SGI373680, SGI373679, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89658[] = {SGI373681, SGI373682, SGI373683, SGI373682, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89659[] = {SGI373684, SGI373685, SGI373686, SGI373685, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89660[] = {SGI373687, SGI373688, SGI373689, SGI373688, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89661[] = {SGI373690, SGI373691, SGI373692, SGI373691, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89662[] = {SGI373693, SGI373694, SGI373695, SGI373694, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89663[] = {SGI373696, SGI373697, SGI373698, SGI373697, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89664[] = {SGI373699, SGI373700, SGI373701, SGI373700, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89665[] = {SGI373702, SGI373703, SGI373704, SGI373703, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89666[] = {SGI373705, SGI373706, SGI373707, SGI373706, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89667[] = {SGI373708, SGI373709, SGI373710, SGI373709, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89668[] = {SGI373711, SGI373712, SGI373713, SGI373712, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89669[] = {SGI373714, SGI373715, SGI373716, SGI373715, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89670[] = {SGI373717, SGI373718, SGI373719, SGI373718, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89671[] = {SGI373720, SGI373721, SGI373722, SGI373721, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89672[] = {SGI373723, SGI373724, SGI373725, SGI373724, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89673[] = {SGI373726, SGI373727, SGI373728, SGI373727, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89674[] = {SGI373729, SGI373730, SGI373731, SGI373730, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89675[] = {SGI373732, SGI373733, SGI373734, SGI373733, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89676[] = {SGI373735, SGI373736, SGI373737, SGI373736, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89677[] = {SGI373738, SGI373739, SGI373740, SGI373739, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89678[] = {SGI373741, SGI373742, SGI373743, SGI373742, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89679[] = {SGI373744, SGI373745, SGI373746, SGI373745, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89680[] = {SGI373747, SGI373748, SGI373749, SGI373748, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89681[] = {SGI373750, SGI373751, SGI373752, SGI373751, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89682[] = {SGI373753, SGI373754, SGI373755, SGI373754, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89683[] = {SGI373756, SGI373757, SGI373758, SGI373757, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89684[] = {SGI373759, SGI373760, SGI373761, SGI373760, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89685[] = {SGI373762, SGI373763, SGI373764, SGI373763, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89686[] = {SGI373765, SGI373766, SGI373767, SGI373766, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89687[] = {SGI373768, SGI373769, SGI373770, SGI373769, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89688[] = {SGI373771, SGI373772, SGI373773, SGI373772, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89689[] = {SGI373774, SGI373775, SGI373776, SGI373775, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89690[] = {SGI373777, SGI373778, SGI373779, SGI373778, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89691[] = {SGI373780, SGI373781, SGI373782, SGI373781, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89692[] = {SGI373783, SGI373784, SGI373785, SGI373784, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89693[] = {SGI373786, SGI373787, SGI373788, SGI373787, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89694[] = {SGI373789, SGI373790, SGI373791, SGI373790, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89695[] = {SGI373792, SGI373793, SGI373794, SGI373793, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89696[] = {SGI373795, SGI373796, SGI373797, SGI373796, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89697[] = {SGI373798, SGI373799, SGI373800, SGI373799, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89698[] = {SGI373801, SGI373802, SGI373803, SGI373802, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89699[] = {SGI373804, SGI373805, SGI373806, SGI373805, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89700[] = {SGI373807, SGI373808, SGI373809, SGI373808, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89701[] = {SGI373810, SGI373811, SGI373812, SGI373811, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89702[] = {SGI373813, SGI373814, SGI373815, SGI373814, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89703[] = {SGI373816, SGI373817, SGI373818, SGI373817, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89704[] = {SGI373819, SGI373820, SGI373821, SGI373820, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89705[] = {SGI373822, SGI373823, SGI373824, SGI373823, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89706[] = {SGI373825, SGI373826, SGI373827, SGI373826, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89707[] = {SGI373828, SGI373829, SGI373830, SGI373829, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89708[] = {SGI373831, SGI373832, SGI373833, SGI373832, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89709[] = {SGI373834, SGI373835, SGI373836, SGI373835, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89710[] = {SGI373837, SGI373838, SGI373839, SGI373838, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89711[] = {SGI373840, SGI373841, SGI373842, SGI373841, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89712[] = {SGI373843, SGI373844, SGI373845, SGI373844, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89713[] = {SGI373846, SGI373847, SGI373848, SGI373847, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89714[] = {SGI373849, SGI373850, SGI373851, SGI373850, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89715[] = {SGI373852, SGI373853, SGI373854, SGI373853, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89716[] = {SGI373855, SGI373856, SGI373857, SGI373856, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89717[] = {SGI373858, SGI373859, SGI373860, SGI373859, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89718[] = {SGI373861, SGI373862, SGI373863, SGI373862, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89719[] = {SGI373864, SGI373865, SGI373866, SGI373865, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89720[] = {SGI373867, SGI373868, SGI373869, SGI373868, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89721[] = {SGI373870, SGI373871, SGI373872, SGI373871, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89722[] = {SGI373873, SGI373874, SGI373875, SGI373874, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89723[] = {SGI373876, SGI373877, SGI373878, SGI373877, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89724[] = {SGI373879, SGI373880, SGI373881, SGI373880, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89725[] = {SGI373882, SGI373883, SGI373884, SGI373883, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89726[] = {SGI373885, SGI373886, SGI373887, SGI373886, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89727[] = {SGI373888, SGI373889, SGI373890, SGI373889, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89728[] = {SGI373891, SGI373892, SGI373893, SGI373892, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89729[] = {SGI373894, SGI373895, SGI373896, SGI373897, SGI373898, SGI0, SGI373899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89730[] = {SGI373900, SGI373901, SGI373902, SGI373903, SGI373904, SGI0, SGI373905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89731[] = {SGI373906, SGI373907, SGI373908, SGI373909, SGI373910, SGI0, SGI373911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89732[] = {SGI373912, SGI373913, SGI373914, SGI373915, SGI373916, SGI0, SGI373917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89733[] = {SGI373918, SGI373919, SGI373920, SGI373921, SGI373922, SGI0, SGI373923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89734[] = {SGI373924, SGI373925, SGI373926, SGI373927, SGI373928, SGI0, SGI373929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89735[] = {SGI373930, SGI373931, SGI373932, SGI373933, SGI373934, SGI0, SGI373935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89736[] = {SGI373936, SGI373937, SGI373938, SGI373939, SGI0, SGI0, SGI373940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89737[] = {SGI373941, SGI373942, SGI373943, SGI373944, SGI373945, SGI0, SGI373946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89738[] = {SGI373947, SGI373948, SGI373949, SGI373949, SGI373948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89739[] = {SGI373950, SGI373951, SGI373952, SGI373952, SGI373951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89740[] = {SGI373953, SGI373954, SGI373955, SGI373955, SGI373954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89741[] = {SGI373956, SGI373957, SGI373958, SGI373958, SGI373957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89742[] = {SGI373959, SGI373960, SGI373961, SGI373961, SGI373960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89743[] = {SGI373962, SGI373963, SGI373964, SGI373964, SGI373963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89744[] = {SGI373965, SGI373966, SGI373967, SGI373967, SGI373966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89745[] = {SGI373968, SGI373969, SGI373970, SGI373970, SGI373969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89746[] = {SGI373971, SGI373972, SGI373973, SGI373974, SGI373975, SGI373976, SGI373977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89747[] = {SGI373978, SGI373979, SGI373980, SGI373981, SGI373982, SGI373983, SGI373984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89748[] = {SGI373985, SGI373986, SGI373987, SGI373988, SGI373989, SGI373990, SGI373991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89749[] = {SGI373992, SGI373993, SGI373994, SGI373995, SGI373996, SGI373997, SGI373998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89750[] = {SGI373999, SGI374000, SGI374001, SGI374002, SGI374003, SGI374004, SGI374005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89751[] = {SGI374006, SGI374007, SGI374008, SGI374009, SGI374010, SGI374011, SGI374012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89752[] = {SGI374013, SGI374014, SGI374015, SGI374016, SGI374017, SGI374018, SGI374019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89753[] = {SGI374020, SGI374021, SGI374022, SGI374023, SGI374024, SGI374025, SGI374026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89754[] = {SGI374027, SGI374028, SGI374029, SGI374030, SGI374031, SGI374032, SGI374033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89755[] = {SGI374034, SGI374035, SGI374036, SGI374037, SGI374038, SGI374039, SGI374040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89756[] = {SGI374041, SGI374042, SGI374043, SGI374044, SGI374045, SGI374046, SGI374047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89757[] = {SGI374048, SGI374049, SGI374050, SGI374051, SGI374052, SGI374053, SGI374054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89758[] = {SGI374055, SGI374056, SGI374057, SGI374058, SGI374059, SGI374060, SGI374061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89759[] = {SGI374062, SGI374063, SGI374064, SGI374065, SGI374066, SGI374067, SGI374068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89760[] = {SGI374069, SGI374070, SGI374071, SGI374072, SGI374073, SGI374074, SGI374075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89761[] = {SGI374076, SGI374077, SGI374078, SGI374079, SGI374080, SGI374081, SGI374082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89762[] = {SGI374083, SGI374084, SGI374085, SGI374086, SGI374087, SGI374088, SGI374089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89763[] = {SGI374090, SGI374091, SGI374092, SGI374093, SGI374094, SGI374095, SGI374096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89764[] = {SGI374097, SGI374098, SGI374099, SGI374100, SGI374101, SGI374102, SGI374103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89765[] = {SGI374104, SGI374105, SGI374106, SGI374107, SGI374108, SGI374109, SGI374110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89766[] = {SGI374111, SGI374112, SGI374113, SGI374114, SGI374115, SGI374116, SGI374117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89767[] = {SGI374118, SGI374119, SGI374120, SGI374121, SGI374122, SGI374123, SGI374124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89768[] = {SGI374125, SGI374126, SGI374127, SGI374128, SGI374129, SGI374130, SGI374131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89769[] = {SGI374132, SGI374133, SGI374134, SGI374135, SGI374136, SGI374137, SGI374138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89770[] = {SGI374139, SGI374140, SGI374141, SGI0, SGI374142, SGI0, SGI374143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89771[] = {SGI374144, SGI374145, SGI374146, SGI0, SGI374147, SGI0, SGI374148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89772[] = {SGI374149, SGI374150, SGI374151, SGI0, SGI374152, SGI0, SGI374153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89773[] = {SGI374154, SGI374155, SGI374156, SGI374157, SGI374158, SGI374159, SGI374160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89774[] = {SGI374161, SGI374162, SGI374163, SGI374164, SGI374165, SGI374166, SGI374167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89775[] = {SGI374168, SGI374169, SGI374170, SGI374171, SGI374172, SGI374173, SGI374174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89776[] = {SGI374175, SGI374176, SGI374177, SGI374178, SGI374179, SGI374180, SGI374181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89777[] = {SGI374182, SGI374183, SGI374184, SGI374185, SGI374186, SGI374187, SGI374188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89778[] = {SGI374189, SGI374190, SGI374191, SGI374192, SGI374193, SGI374194, SGI374195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89779[] = {SGI374196, SGI374197, SGI374198, SGI374199, SGI374200, SGI374201, SGI374202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89780[] = {SGI374203, SGI374204, SGI374205, SGI374206, SGI374207, SGI374208, SGI374209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89781[] = {SGI374210, SGI374211, SGI374212, SGI374213, SGI374214, SGI374215, SGI374216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89782[] = {SGI374217, SGI374218, SGI374219, SGI374220, SGI374221, SGI374222, SGI374223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89783[] = {SGI374224, SGI374225, SGI374226, SGI374227, SGI374228, SGI374229, SGI374230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89784[] = {SGI374231, SGI374232, SGI374233, SGI374234, SGI374235, SGI374236, SGI374237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89785[] = {SGI374238, SGI374239, SGI374240, SGI374241, SGI374242, SGI374243, SGI374244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89786[] = {SGI374245, SGI374246, SGI374247, SGI374248, SGI374249, SGI374250, SGI374251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89787[] = {SGI374252, SGI374253, SGI374254, SGI374255, SGI374256, SGI374257, SGI374258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89788[] = {SGI374259, SGI374260, SGI374261, SGI374262, SGI374263, SGI374264, SGI374265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89789[] = {SGI374266, SGI374267, SGI374268, SGI374269, SGI374270, SGI374271, SGI374272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89790[] = {SGI374273, SGI374274, SGI374275, SGI374276, SGI374277, SGI374278, SGI374279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89791[] = {SGI374280, SGI374281, SGI374282, SGI374283, SGI374284, SGI374285, SGI374286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89792[] = {SGI374287, SGI374288, SGI374289, SGI374290, SGI374291, SGI374292, SGI374293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89793[] = {SGI374294, SGI374295, SGI374296, SGI374297, SGI374298, SGI374299, SGI374300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89794[] = {SGI374301, SGI374302, SGI374303, SGI374304, SGI374305, SGI374306, SGI374307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89795[] = {SGI374308, SGI374309, SGI374310, SGI374311, SGI374312, SGI374313, SGI374314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89796[] = {SGI374315, SGI374316, SGI374317, SGI374318, SGI374319, SGI374320, SGI374321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89797[] = {SGI374322, SGI374323, SGI374324, SGI374325, SGI374326, SGI374327, SGI374328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89798[] = {SGI374329, SGI374330, SGI374331, SGI374332, SGI374333, SGI374334, SGI374335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89799[] = {SGI374336, SGI374337, SGI374338, SGI374339, SGI374340, SGI374341, SGI374342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89800[] = {SGI374343, SGI374344, SGI374345, SGI374346, SGI374347, SGI374348, SGI374349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89801[] = {SGI374350, SGI374351, SGI374352, SGI374353, SGI374354, SGI374355, SGI374356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89802[] = {SGI374357, SGI374358, SGI374359, SGI374360, SGI374361, SGI374362, SGI374363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89803[] = {SGI374364, SGI374365, SGI374366, SGI374367, SGI374368, SGI0, SGI374367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89804[] = {SGI374369, SGI374370, SGI374371, SGI374372, SGI374373, SGI374374, SGI374375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89805[] = {SGI374376, SGI374377, SGI374378, SGI374379, SGI374380, SGI374381, SGI374382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89806[] = {SGI374383, SGI374384, SGI374385, SGI374386, SGI374387, SGI374388, SGI374389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89807[] = {SGI374390, SGI374391, SGI374392, SGI374393, SGI374394, SGI374395, SGI374396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89808[] = {SGI374397, SGI374398, SGI374399, SGI374400, SGI374401, SGI374402, SGI374403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89809[] = {SGI374404, SGI374405, SGI374406, SGI374407, SGI374408, SGI374409, SGI374410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89810[] = {SGI374411, SGI374412, SGI374413, SGI374414, SGI374415, SGI374416, SGI374417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89811[] = {SGI374418, SGI374419, SGI374420, SGI374421, SGI374422, SGI374423, SGI374424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89812[] = {SGI374425, SGI374426, SGI374427, SGI374428, SGI374429, SGI374430, SGI374431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89813[] = {SGI374432, SGI374433, SGI374434, SGI374435, SGI374436, SGI374437, SGI374438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89814[] = {SGI374439, SGI374440, SGI374441, SGI374442, SGI374443, SGI374444, SGI374445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89815[] = {SGI374446, SGI374447, SGI374448, SGI374449, SGI374450, SGI374451, SGI374452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89816[] = {SGI374453, SGI374454, SGI374455, SGI374456, SGI374457, SGI374458, SGI374459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89817[] = {SGI374460, SGI374461, SGI374462, SGI374463, SGI374464, SGI374465, SGI374466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89818[] = {SGI374467, SGI374468, SGI374469, SGI374470, SGI374471, SGI374472, SGI374473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89819[] = {SGI374474, SGI374475, SGI374476, SGI374477, SGI374478, SGI374479, SGI374480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89820[] = {SGI0, SGI374481, SGI374482, SGI374483, SGI374484, SGI374485, SGI374486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89821[] = {SGI374487, SGI374488, SGI374489, SGI374490, SGI374491, SGI374492, SGI374493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89822[] = {SGI374494, SGI374495, SGI374496, SGI374497, SGI374498, SGI374499, SGI374500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89823[] = {SGI374501, SGI374502, SGI374503, SGI374504, SGI374505, SGI374506, SGI374507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89824[] = {SGI374508, SGI374509, SGI374510, SGI374511, SGI374512, SGI374513, SGI374514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89825[] = {SGI374515, SGI374516, SGI374517, SGI374518, SGI374519, SGI374520, SGI374521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89826[] = {SGI374522, SGI374523, SGI374524, SGI374525, SGI374526, SGI374527, SGI374528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89827[] = {SGI374529, SGI374530, SGI374531, SGI374532, SGI374533, SGI374534, SGI374535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89828[] = {SGI374536, SGI374537, SGI374538, SGI374539, SGI374540, SGI374541, SGI374542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89829[] = {SGI374543, SGI374544, SGI374545, SGI374546, SGI374547, SGI374548, SGI374549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89830[] = {SGI374550, SGI374551, SGI374552, SGI374553, SGI374554, SGI374555, SGI374556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89831[] = {SGI374557, SGI374558, SGI374559, SGI374560, SGI374561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89832[] = {SGI374562, SGI374563, SGI374564, SGI374565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI374566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89833[] = {SGI374567, SGI374568, SGI374569, SGI374570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI374571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89834[] = {SGI374572, SGI374573, SGI374574, SGI374575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI374576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89835[] = {SGI374577, SGI374578, SGI374579, SGI374580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI374581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89836[] = {SGI374582, SGI374583, SGI374584, SGI374585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI374586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89837[] = {SGI374587, SGI374588, SGI374589, SGI374590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI374591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89838[] = {SGI374592, SGI374593, SGI374594, SGI374595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI374596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89839[] = {SGI374597, SGI374598, SGI374599, SGI374600, SGI374601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89840[] = {SGI374602, SGI374603, SGI374604, SGI374605, SGI374606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89841[] = {SGI374607, SGI374608, SGI374609, SGI374610, SGI374611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89842[] = {SGI374612, SGI374613, SGI374614, SGI374615, SGI374616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89843[] = {SGI374617, SGI374618, SGI374619, SGI374620, SGI374621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89844[] = {SGI374622, SGI374623, SGI0, SGI374623, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89845[] = {SGI374624, SGI374625, SGI374626, SGI374627, SGI374628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89846[] = {SGI374629, SGI374630, SGI0, SGI374630, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89847[] = {SGI374631, SGI374633, SGI0, SGI374633, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89848[] = {SGI374632, SGI374634, SGI374635, SGI374636, SGI374637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89849[] = {SGI374638, SGI374639, SGI0, SGI374639, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89850[] = {SGI374640, SGI374641, SGI374642, SGI374642, SGI374643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89851[] = {SGI374644, SGI374645, SGI0, SGI374645, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89852[] = {SGI374646, SGI374647, SGI374648, SGI0, SGI374649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89853[] = {SGI374650, SGI374651, SGI0, SGI374651, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89854[] = {SGI374652, SGI374653, SGI374654, SGI0, SGI374655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89855[] = {SGI374656, SGI374657, SGI0, SGI374657, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89856[] = {SGI374658, SGI374659, SGI374660, SGI374660, SGI374661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89857[] = {SGI374662, SGI374663, SGI0, SGI374663, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89858[] = {SGI374664, SGI374665, SGI0, SGI374665, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89859[] = {SGI374666, SGI374667, SGI0, SGI374667, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89860[] = {SGI374668, SGI374669, SGI0, SGI374669, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89861[] = {SGI374670, SGI374671, SGI0, SGI374671, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89862[] = {SGI374672, SGI374673, SGI0, SGI374673, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89863[] = {SGI374674, SGI374675, SGI0, SGI374675, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89864[] = {SGI374676, SGI374677, SGI0, SGI374677, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89865[] = {SGI374678, SGI374679, SGI0, SGI374679, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89866[] = {SGI374680, SGI374681, SGI0, SGI374681, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89867[] = {SGI374682, SGI374683, SGI0, SGI374683, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89868[] = {SGI374684, SGI374685, SGI0, SGI374685, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89869[] = {SGI374686, SGI374687, SGI0, SGI374687, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89870[] = {SGI374688, SGI374689, SGI0, SGI374689, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89871[] = {SGI374690, SGI374691, SGI0, SGI374691, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89872[] = {SGI374692, SGI374693, SGI0, SGI374693, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89873[] = {SGI374694, SGI374695, SGI0, SGI374695, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89874[] = {SGI374696, SGI374697, SGI0, SGI374697, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89875[] = {SGI374698, SGI374699, SGI0, SGI374699, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89876[] = {SGI374700, SGI374701, SGI0, SGI374701, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89877[] = {SGI374702, SGI374703, SGI0, SGI374703, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89878[] = {SGI374704, SGI374705, SGI0, SGI374705, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89879[] = {SGI374706, SGI374707, SGI0, SGI374707, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89880[] = {SGI374708, SGI374709, SGI0, SGI374709, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89881[] = {SGI374710, SGI374711, SGI0, SGI374711, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89882[] = {SGI374712, SGI374713, SGI0, SGI374713, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89883[] = {SGI374714, SGI374715, SGI0, SGI374715, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89884[] = {SGI374716, SGI374717, SGI0, SGI374717, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89885[] = {SGI374718, SGI374719, SGI0, SGI374719, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89886[] = {SGI374720, SGI374721, SGI0, SGI374721, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89887[] = {SGI374722, SGI374723, SGI0, SGI374723, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89888[] = {SGI374724, SGI374725, SGI0, SGI374725, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89889[] = {SGI374726, SGI374727, SGI0, SGI374727, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89890[] = {SGI374728, SGI374729, SGI0, SGI374729, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89891[] = {SGI374730, SGI374731, SGI0, SGI374731, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89892[] = {SGI374732, SGI374733, SGI0, SGI374733, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89893[] = {SGI374734, SGI374735, SGI0, SGI374735, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89894[] = {SGI374736, SGI374737, SGI0, SGI374737, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89895[] = {SGI374738, SGI374739, SGI0, SGI374739, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89896[] = {SGI374740, SGI374741, SGI0, SGI374741, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89897[] = {SGI374742, SGI374743, SGI0, SGI374743, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89898[] = {SGI374744, SGI374745, SGI0, SGI374745, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89899[] = {SGI374746, SGI374747, SGI0, SGI374747, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89900[] = {SGI374748, SGI374749, SGI0, SGI374749, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89901[] = {SGI374750, SGI374751, SGI0, SGI374751, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89902[] = {SGI374752, SGI374753, SGI0, SGI374753, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89903[] = {SGI374754, SGI374755, SGI0, SGI374755, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89904[] = {SGI374756, SGI374757, SGI0, SGI374757, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89905[] = {SGI374758, SGI374759, SGI0, SGI374759, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89906[] = {SGI374760, SGI374761, SGI374762, SGI374763, SGI374764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89907[] = {SGI374765, SGI374766, SGI374767, SGI374768, SGI374769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89908[] = {SGI374770, SGI374771, SGI374772, SGI374773, SGI374774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89909[] = {SGI374775, SGI374776, SGI374777, SGI374778, SGI374779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89910[] = {SGI374780, SGI374781, SGI374782, SGI374783, SGI374784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89911[] = {SGI374785, SGI374786, SGI374787, SGI374788, SGI374789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89912[] = {SGI374790, SGI374791, SGI374792, SGI374793, SGI374794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89913[] = {SGI374795, SGI374796, SGI374797, SGI374798, SGI374799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89914[] = {SGI374800, SGI374801, SGI374802, SGI374803, SGI374804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89915[] = {SGI374805, SGI374806, SGI374807, SGI0, SGI374808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89916[] = {SGI374809, SGI374810, SGI374811, SGI0, SGI374812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89917[] = {SGI374813, SGI374814, SGI374815, SGI374816, SGI374817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89918[] = {SGI374818, SGI374819, SGI374820, SGI374821, SGI374822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89919[] = {SGI374823, SGI374824, SGI374825, SGI374826, SGI374827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89920[] = {SGI374828, SGI374829, SGI374830, SGI374831, SGI374832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89921[] = {SGI374833, SGI374834, SGI374835, SGI374836, SGI374837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89922[] = {SGI374838, SGI374839, SGI374840, SGI374841, SGI374842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89923[] = {SGI374843, SGI374844, SGI374845, SGI0, SGI374846, SGI0, SGI374847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89924[] = {SGI374848, SGI374849, SGI374850, SGI0, SGI374851, SGI0, SGI374852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89925[] = {SGI374853, SGI374854, SGI374855, SGI0, SGI374856, SGI0, SGI374857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89926[] = {SGI374858, SGI374859, SGI374860, SGI0, SGI374861, SGI0, SGI374862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89927[] = {SGI374863, SGI374864, SGI374865, SGI0, SGI374866, SGI0, SGI374867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89928[] = {SGI374868, SGI374869, SGI374870, SGI0, SGI374871, SGI0, SGI374872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89929[] = {SGI374873, SGI374874, SGI374875, SGI0, SGI374876, SGI0, SGI374877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89930[] = {SGI374878, SGI374879, SGI374880, SGI0, SGI374881, SGI0, SGI374882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89931[] = {SGI374883, SGI374884, SGI374885, SGI0, SGI374886, SGI0, SGI374887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89932[] = {SGI374888, SGI374889, SGI374890, SGI0, SGI374891, SGI0, SGI374892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89933[] = {SGI374893, SGI374894, SGI374895, SGI0, SGI374896, SGI0, SGI374897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89934[] = {SGI374898, SGI374899, SGI374900, SGI0, SGI374901, SGI0, SGI374902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89935[] = {SGI374903, SGI374904, SGI374905, SGI0, SGI374906, SGI0, SGI374907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89936[] = {SGI374908, SGI374909, SGI374910, SGI0, SGI374911, SGI0, SGI374912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89937[] = {SGI374913, SGI374914, SGI374915, SGI0, SGI374916, SGI0, SGI374917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89938[] = {SGI374918, SGI374919, SGI374920, SGI0, SGI374921, SGI0, SGI374922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89939[] = {SGI374923, SGI374924, SGI374925, SGI0, SGI374926, SGI0, SGI374927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89940[] = {SGI374928, SGI374929, SGI374930, SGI0, SGI374931, SGI0, SGI374932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89941[] = {SGI374933, SGI374934, SGI374935, SGI0, SGI374936, SGI0, SGI374937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89942[] = {SGI374938, SGI374939, SGI374940, SGI0, SGI374941, SGI0, SGI374942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89943[] = {SGI374943, SGI374944, SGI374945, SGI0, SGI374946, SGI0, SGI374947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89944[] = {SGI374948, SGI374949, SGI374950, SGI0, SGI374951, SGI0, SGI374952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89945[] = {SGI374953, SGI374954, SGI374955, SGI0, SGI374956, SGI0, SGI374957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89946[] = {SGI374958, SGI374959, SGI0, SGI374959, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89947[] = {SGI374960, SGI374961, SGI0, SGI374961, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89948[] = {SGI374962, SGI374963, SGI0, SGI374964, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89949[] = {SGI374965, SGI374966, SGI0, SGI374966, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89950[] = {SGI31227, SGI374967, SGI374968, SGI0, SGI31226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89951[] = {SGI374969, SGI374970, SGI374971, SGI374971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI374972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89952[] = {SGI374973, SGI374974, SGI374975, SGI374975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI374976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89953[] = {SGI374977, SGI374978, SGI374979, SGI374979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI374980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89954[] = {SGI374981, SGI374982, SGI374983, SGI374983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI374984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89955[] = {SGI374985, SGI374986, SGI374987, SGI374987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI374988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89956[] = {SGI374989, SGI374990, SGI374991, SGI374992, SGI0, SGI374993, SGI374994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89957[] = {SGI374995, SGI374996, SGI374997, SGI374998, SGI0, SGI374999, SGI375000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89958[] = {SGI375001, SGI375002, SGI375003, SGI375004, SGI0, SGI375005, SGI375006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89959[] = {SGI375007, SGI375008, SGI375009, SGI375010, SGI0, SGI375011, SGI375012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89960[] = {SGI375013, SGI375014, SGI375015, SGI375016, SGI0, SGI375017, SGI375018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89961[] = {SGI375019, SGI375020, SGI375021, SGI375022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89962[] = {SGI375023, SGI375024, SGI375025, SGI375026, SGI0, SGI375027, SGI375028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89963[] = {SGI375029, SGI375030, SGI375031, SGI0, SGI0, SGI0, SGI375032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89964[] = {SGI375033, SGI375034, SGI375035, SGI0, SGI0, SGI0, SGI375036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89965[] = {SGI375037, SGI375038, SGI375039, SGI0, SGI0, SGI0, SGI375040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89966[] = {SGI0, SGI375041, SGI375042, SGI0, SGI0, SGI0, SGI375043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89967[] = {SGI375044, SGI375045, SGI375046, SGI375047, SGI0, SGI375048, SGI375049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89968[] = {SGI375050, SGI375051, SGI375052, SGI375053, SGI0, SGI375054, SGI375055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89969[] = {SGI375056, SGI375057, SGI375058, SGI375059, SGI0, SGI375060, SGI375061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89970[] = {SGI0, SGI375062, SGI375063, SGI375064, SGI0, SGI375065, SGI375066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89971[] = {SGI375067, SGI375068, SGI375069, SGI0, SGI0, SGI0, SGI375070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89972[] = {SGI375071, SGI375072, SGI375073, SGI375074, SGI0, SGI375075, SGI375076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89973[] = {SGI375077, SGI375078, SGI375079, SGI0, SGI0, SGI0, SGI375080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89974[] = {SGI375081, SGI375082, SGI375083, SGI375084, SGI0, SGI375085, SGI375086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89975[] = {SGI375087, SGI375088, SGI375089, SGI375090, SGI0, SGI375091, SGI375092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89976[] = {SGI375093, SGI2985, SGI375094, SGI375095, SGI2985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89977[] = {SGI375096, SGI289098, SGI289099, SGI375097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89978[] = {SGI375098, SGI375099, SGI375100, SGI0, SGI375101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89979[] = {SGI375102, SGI375103, SGI375104, SGI375104, SGI375105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89980[] = {SGI375106, SGI375107, SGI375108, SGI375108, SGI375109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89981[] = {SGI375110, SGI375111, SGI375112, SGI0, SGI375113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89982[] = {SGI375114, SGI375115, SGI375116, SGI375116, SGI375117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89983[] = {SGI375118, SGI375119, SGI375120, SGI375120, SGI375121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89984[] = {SGI375122, SGI375123, SGI375124, SGI375124, SGI375125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89985[] = {SGI375126, SGI375127, SGI375128, SGI0, SGI375129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89986[] = {SGI375130, SGI375131, SGI375132, SGI0, SGI375133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89987[] = {SGI375134, SGI375135, SGI375136, SGI0, SGI375137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89988[] = {SGI375138, SGI375139, SGI375140, SGI375140, SGI375141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89989[] = {SGI375142, SGI375143, SGI375144, SGI0, SGI375145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89990[] = {SGI375146, SGI375147, SGI375148, SGI0, SGI375149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89991[] = {SGI375150, SGI375151, SGI375152, SGI375152, SGI375153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89992[] = {SGI375154, SGI375155, SGI375156, SGI375157, SGI375158, SGI0, SGI375159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89993[] = {SGI375160, SGI375161, SGI375162, SGI375163, SGI375164, SGI0, SGI375165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89994[] = {SGI375166, SGI375167, SGI375168, SGI375169, SGI375170, SGI0, SGI375171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89995[] = {SGI375172, SGI375173, SGI375174, SGI375175, SGI375176, SGI375177, SGI375178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89996[] = {SGI375179, SGI375180, SGI375181, SGI375180, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89997[] = {SGI375182, SGI375183, SGI0, SGI375183, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89998[] = {SGI375184, SGI375185, SGI0, SGI375185, SGI0, SGI0, SGI0, NULL};
const Segment * const SG89999[] = {SGI375186, SGI375187, SGI375188, SGI375189, SGI375190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90000[] = {SGI375191, SGI375192, SGI375193, SGI375194, SGI375195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90001[] = {SGI375196, SGI375197, SGI375198, SGI375199, SGI375200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90002[] = {SGI375201, SGI375202, SGI375203, SGI375204, SGI375205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90003[] = {SGI375206, SGI375207, SGI375208, SGI375209, SGI375210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90004[] = {SGI375211, SGI375212, SGI375213, SGI0, SGI375214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90005[] = {SGI375215, SGI375216, SGI375217, SGI375218, SGI375219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90006[] = {SGI375220, SGI375221, SGI375222, SGI375223, SGI375224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90007[] = {SGI375225, SGI375226, SGI375227, SGI375228, SGI375229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90008[] = {SGI375230, SGI375231, SGI375232, SGI375232, SGI375233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90009[] = {SGI375234, SGI375235, SGI375236, SGI375236, SGI375237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90010[] = {SGI375238, SGI375239, SGI375240, SGI0, SGI375241, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90011[] = {SGI375242, SGI375243, SGI375244, SGI0, SGI375245, SGI0, SGI375246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90012[] = {SGI375247, SGI375248, SGI375249, SGI0, SGI375250, SGI0, SGI375251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90013[] = {SGI375252, SGI375253, SGI375254, SGI0, SGI375255, SGI0, SGI375256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90014[] = {SGI375257, SGI375258, SGI375259, SGI0, SGI375260, SGI0, SGI375261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90015[] = {SGI375262, SGI375263, SGI375264, SGI0, SGI375265, SGI0, SGI375266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90016[] = {SGI375267, SGI375268, SGI375269, SGI0, SGI375270, SGI0, SGI375271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90017[] = {SGI375272, SGI375273, SGI375274, SGI0, SGI375275, SGI0, SGI375276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90018[] = {SGI375277, SGI375278, SGI375279, SGI0, SGI375280, SGI0, SGI375281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90019[] = {SGI375282, SGI375283, SGI375284, SGI0, SGI375285, SGI0, SGI375286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90020[] = {SGI375287, SGI375288, SGI375289, SGI0, SGI375290, SGI0, SGI375291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90021[] = {SGI375292, SGI375293, SGI375294, SGI0, SGI375295, SGI0, SGI375296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90022[] = {SGI375297, SGI375298, SGI375299, SGI0, SGI375300, SGI0, SGI375301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90023[] = {SGI375302, SGI375303, SGI375304, SGI0, SGI375305, SGI0, SGI375306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90024[] = {SGI375307, SGI375308, SGI375309, SGI0, SGI375310, SGI0, SGI375311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90025[] = {SGI375312, SGI375313, SGI375314, SGI0, SGI375315, SGI0, SGI375316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90026[] = {SGI375317, SGI375318, SGI375319, SGI0, SGI375320, SGI0, SGI375321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90027[] = {SGI375322, SGI375323, SGI375324, SGI0, SGI375325, SGI0, SGI375326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90028[] = {SGI375327, SGI375328, SGI375329, SGI0, SGI375330, SGI0, SGI375331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90029[] = {SGI375332, SGI375333, SGI375334, SGI0, SGI375335, SGI0, SGI375336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90030[] = {SGI375337, SGI375338, SGI375339, SGI0, SGI375340, SGI0, SGI375341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90031[] = {SGI375342, SGI375343, SGI375344, SGI0, SGI375345, SGI0, SGI375346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90032[] = {SGI375347, SGI375348, SGI375349, SGI0, SGI375348, SGI0, SGI375350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90033[] = {SGI375351, SGI375352, SGI375353, SGI0, SGI375352, SGI0, SGI375354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90034[] = {SGI375355, SGI375356, SGI375357, SGI0, SGI375356, SGI0, SGI375358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90035[] = {SGI375359, SGI375360, SGI375361, SGI0, SGI375360, SGI0, SGI375362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90036[] = {SGI375363, SGI375364, SGI375365, SGI0, SGI375364, SGI0, SGI375366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90037[] = {SGI375367, SGI375368, SGI375369, SGI0, SGI375368, SGI0, SGI375370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90038[] = {SGI375371, SGI375372, SGI375373, SGI375374, SGI375372, SGI0, SGI375374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90039[] = {SGI375375, SGI375376, SGI375377, SGI375378, SGI375376, SGI375379, SGI375380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90040[] = {SGI375381, SGI375382, SGI375383, SGI375384, SGI375382, SGI375385, SGI375386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90041[] = {SGI375387, SGI375388, SGI375389, SGI0, SGI0, SGI0, SGI375390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90042[] = {SGI375391, SGI375392, SGI375393, SGI375394, SGI375392, SGI375395, SGI375396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90043[] = {SGI375397, SGI375398, SGI375399, SGI375400, SGI375398, SGI375401, SGI375402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90044[] = {SGI375403, SGI375404, SGI375405, SGI375405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI375406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90045[] = {SGI375407, SGI375408, SGI375409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI375410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90046[] = {SGI375411, SGI375412, SGI375413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI375414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90047[] = {SGI375415, SGI375416, SGI0, SGI375416, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90048[] = {SGI375417, SGI375418, SGI0, SGI375418, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90049[] = {SGI375419, SGI375420, SGI375421, SGI375422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI375423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90050[] = {SGI375424, SGI375425, SGI375426, SGI375427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI375428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90051[] = {SGI375429, SGI375430, SGI375431, SGI375432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI375433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90052[] = {SGI375434, SGI375435, SGI375436, SGI375435, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90053[] = {SGI375437, SGI375438, SGI375439, SGI375438, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90054[] = {SGI375440, SGI375441, SGI375442, SGI375441, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90055[] = {SGI375443, SGI375444, SGI375445, SGI375444, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90056[] = {SGI375446, SGI375447, SGI375448, SGI375447, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90057[] = {SGI375449, SGI375450, SGI375451, SGI375450, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90058[] = {SGI375452, SGI375453, SGI375454, SGI375453, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90059[] = {SGI375455, SGI375456, SGI375457, SGI375456, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90060[] = {SGI375458, SGI375459, SGI375460, SGI375459, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90061[] = {SGI375461, SGI375462, SGI375463, SGI375462, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90062[] = {SGI375464, SGI375465, SGI375466, SGI375465, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90063[] = {SGI375467, SGI375468, SGI375469, SGI375468, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90064[] = {SGI375470, SGI375471, SGI375472, SGI375471, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90065[] = {SGI375473, SGI375474, SGI375475, SGI375474, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90066[] = {SGI375476, SGI375477, SGI375478, SGI375477, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90067[] = {SGI375479, SGI375480, SGI375481, SGI375480, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90068[] = {SGI375482, SGI375483, SGI375484, SGI375483, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90069[] = {SGI0, SGI375485, SGI375486, SGI375485, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90070[] = {SGI375487, SGI375488, SGI375489, SGI375490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI375491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90071[] = {SGI375492, SGI375493, SGI375494, SGI375495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI375496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90072[] = {SGI375497, SGI375498, SGI375499, SGI375500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI375501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90073[] = {SGI375502, SGI375503, SGI375504, SGI375505, SGI375506, SGI0, SGI375507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90074[] = {SGI375508, SGI375509, SGI375510, SGI375511, SGI375512, SGI0, SGI375513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90075[] = {SGI375514, SGI375515, SGI375516, SGI375517, SGI375518, SGI0, SGI375519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90076[] = {SGI375520, SGI375521, SGI375522, SGI375523, SGI375524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90077[] = {SGI375525, SGI375526, SGI375527, SGI375526, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90078[] = {SGI375528, SGI375529, SGI375530, SGI375529, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90079[] = {SGI375531, SGI375532, SGI375533, SGI375532, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90080[] = {SGI375534, SGI375535, SGI375536, SGI375535, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90081[] = {SGI375537, SGI375538, SGI375539, SGI375538, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90082[] = {SGI375540, SGI375541, SGI375542, SGI375541, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90083[] = {SGI375543, SGI375544, SGI375545, SGI375544, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90084[] = {SGI375546, SGI375547, SGI375548, SGI375547, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90085[] = {SGI375549, SGI375550, SGI375551, SGI375550, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90086[] = {SGI375552, SGI375553, SGI375554, SGI375553, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90087[] = {SGI375555, SGI375556, SGI375557, SGI375556, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90088[] = {SGI375558, SGI375559, SGI375560, SGI375559, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90089[] = {SGI375561, SGI375562, SGI375563, SGI375562, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90090[] = {SGI375564, SGI375565, SGI375566, SGI375565, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90091[] = {SGI375567, SGI375568, SGI375569, SGI375568, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90092[] = {SGI375570, SGI375571, SGI375572, SGI375571, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90093[] = {SGI375573, SGI375574, SGI375575, SGI375574, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90094[] = {SGI375576, SGI375577, SGI375578, SGI375577, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90095[] = {SGI375579, SGI375580, SGI375581, SGI375580, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90096[] = {SGI375582, SGI375583, SGI375584, SGI375583, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90097[] = {SGI375585, SGI375586, SGI375587, SGI375586, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90098[] = {SGI375588, SGI375589, SGI375590, SGI375589, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90099[] = {SGI375591, SGI375592, SGI375593, SGI375592, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90100[] = {SGI375594, SGI375595, SGI375596, SGI375595, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90101[] = {SGI375597, SGI375598, SGI375599, SGI375598, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90102[] = {SGI375600, SGI375601, SGI375602, SGI375601, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90103[] = {SGI375603, SGI375604, SGI375605, SGI375604, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90104[] = {SGI375606, SGI375607, SGI375608, SGI375607, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90105[] = {SGI375609, SGI375610, SGI375611, SGI375610, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90106[] = {SGI375612, SGI375613, SGI375614, SGI375613, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90107[] = {SGI375615, SGI375616, SGI375617, SGI375616, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90108[] = {SGI375618, SGI375619, SGI375620, SGI375619, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90109[] = {SGI375621, SGI375622, SGI375623, SGI375622, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90110[] = {SGI375624, SGI375625, SGI375626, SGI375625, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90111[] = {SGI375627, SGI375628, SGI375629, SGI375628, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90112[] = {SGI375630, SGI375631, SGI375632, SGI375631, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90113[] = {SGI375633, SGI375634, SGI375635, SGI375634, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90114[] = {SGI375636, SGI375637, SGI375638, SGI375637, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90115[] = {SGI375639, SGI375640, SGI375641, SGI375640, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90116[] = {SGI375642, SGI375643, SGI375644, SGI375643, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90117[] = {SGI375645, SGI375646, SGI375647, SGI375646, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90118[] = {SGI375648, SGI375649, SGI375650, SGI375649, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90119[] = {SGI375651, SGI375652, SGI375653, SGI375652, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90120[] = {SGI375654, SGI375655, SGI375656, SGI375655, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90121[] = {SGI375657, SGI375658, SGI375659, SGI375658, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90122[] = {SGI375660, SGI375661, SGI375662, SGI375661, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90123[] = {SGI375663, SGI375664, SGI375665, SGI375664, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90124[] = {SGI375666, SGI375667, SGI375668, SGI375667, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90125[] = {SGI375669, SGI375670, SGI375671, SGI375670, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90126[] = {SGI375672, SGI375673, SGI375674, SGI375675, SGI375673, SGI375676, SGI375677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90127[] = {SGI375678, SGI375679, SGI375680, SGI375681, SGI375679, SGI375682, SGI375683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90128[] = {SGI375684, SGI375685, SGI375686, SGI375687, SGI375685, SGI375688, SGI375689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90129[] = {SGI5062, SGI5061, SGI375690, SGI375691, SGI5061, SGI375692, SGI375693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90130[] = {SGI375694, SGI375695, SGI375696, SGI375697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90131[] = {SGI375698, SGI375699, SGI375700, SGI375701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90132[] = {SGI375702, SGI375703, SGI375704, SGI375704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90133[] = {SGI375705, SGI375706, SGI375707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90134[] = {SGI375708, SGI375709, SGI375710, SGI375710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90135[] = {SGI375711, SGI375712, SGI375713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90136[] = {SGI375714, SGI375715, SGI375716, SGI375717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90137[] = {SGI375718, SGI375719, SGI375720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90138[] = {SGI375721, SGI375722, SGI375723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90139[] = {SGI375724, SGI375725, SGI375726, SGI375726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90140[] = {SGI375727, SGI375728, SGI375729, SGI375729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90141[] = {SGI375730, SGI375731, SGI375732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90142[] = {SGI375733, SGI375734, SGI375735, SGI375735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90143[] = {SGI375736, SGI375737, SGI375738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90144[] = {SGI375739, SGI375740, SGI375741, SGI375741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90145[] = {SGI375742, SGI375743, SGI375744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90146[] = {SGI375745, SGI375746, SGI375747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90147[] = {SGI375748, SGI375749, SGI375750, SGI375750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90148[] = {SGI375751, SGI375752, SGI375753, SGI375754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90149[] = {SGI375755, SGI375756, SGI375757, SGI375758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90150[] = {SGI375759, SGI375760, SGI375761, SGI375761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90151[] = {SGI0, SGI375762, SGI375763, SGI375763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90152[] = {SGI375764, SGI375765, SGI375766, SGI375766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90153[] = {SGI375767, SGI375768, SGI375769, SGI375769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90154[] = {SGI375770, SGI375771, SGI375772, SGI375772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90155[] = {SGI375773, SGI319357, SGI319358, SGI319360, SGI375774, SGI375775, SGI375776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90156[] = {SGI375777, SGI375778, SGI375779, SGI375780, SGI375781, SGI375782, SGI375783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90157[] = {SGI375784, SGI375785, SGI375786, SGI375787, SGI375788, SGI375789, SGI375790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90158[] = {SGI375791, SGI375792, SGI375793, SGI0, SGI375794, SGI0, SGI375795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90159[] = {SGI375796, SGI375797, SGI375798, SGI0, SGI375799, SGI0, SGI375800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90160[] = {SGI375801, SGI375802, SGI375803, SGI0, SGI375804, SGI0, SGI375805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90161[] = {SGI375806, SGI375807, SGI375808, SGI375809, SGI375810, SGI375811, SGI375812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90162[] = {SGI375813, SGI375814, SGI375815, SGI375816, SGI375817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90163[] = {SGI375818, SGI375819, SGI375820, SGI375821, SGI375822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90164[] = {SGI375823, SGI375824, SGI375825, SGI375826, SGI375827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90165[] = {SGI375828, SGI375829, SGI375830, SGI375831, SGI375832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90166[] = {SGI375833, SGI375834, SGI375835, SGI375836, SGI375837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90167[] = {SGI375838, SGI375839, SGI375840, SGI375841, SGI375842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90168[] = {SGI375843, SGI375844, SGI375845, SGI375846, SGI375847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90169[] = {SGI375848, SGI375849, SGI375850, SGI375851, SGI375852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90170[] = {SGI375853, SGI375854, SGI375855, SGI375856, SGI0, SGI0, SGI375857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90171[] = {SGI375858, SGI375859, SGI375860, SGI375861, SGI0, SGI0, SGI375862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90172[] = {SGI375863, SGI375864, SGI375865, SGI375866, SGI0, SGI0, SGI375867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90173[] = {SGI375868, SGI375869, SGI375870, SGI375871, SGI0, SGI0, SGI375872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90174[] = {SGI375873, SGI375874, SGI375875, SGI375876, SGI0, SGI0, SGI375877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90175[] = {SGI0, SGI375878, SGI375879, SGI375880, SGI0, SGI0, SGI375881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90176[] = {SGI375882, SGI375883, SGI375884, SGI375885, SGI0, SGI0, SGI375886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90177[] = {SGI375887, SGI375888, SGI375889, SGI375890, SGI0, SGI0, SGI375891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90178[] = {SGI375892, SGI375893, SGI375894, SGI375895, SGI0, SGI0, SGI375896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90179[] = {SGI375897, SGI375898, SGI375899, SGI375900, SGI0, SGI0, SGI375901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90180[] = {SGI0, SGI375902, SGI375903, SGI0, SGI375904, SGI0, SGI375905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90181[] = {SGI375906, SGI375907, SGI0, SGI375907, SGI375908, SGI0, SGI0, NULL};
const Segment * const SG90182[] = {SGI375909, SGI375910, SGI0, SGI375910, SGI375911, SGI0, SGI0, NULL};
const Segment * const SG90183[] = {SGI375912, SGI375913, SGI375914, SGI375915, SGI375913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90184[] = {SGI375916, SGI375917, SGI375918, SGI375919, SGI375917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90185[] = {SGI375920, SGI375921, SGI375922, SGI375921, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90186[] = {SGI375923, SGI375924, SGI375925, SGI375924, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90187[] = {SGI375926, SGI375927, SGI375928, SGI375927, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90188[] = {SGI375929, SGI375930, SGI0, SGI375930, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90189[] = {SGI375931, SGI375932, SGI0, SGI375932, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90190[] = {SGI375933, SGI375934, SGI375935, SGI375935, SGI0, SGI0, SGI0, SGI375936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90191[] = {SGI375937, SGI375938, SGI375939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI375940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90192[] = {SGI375941, SGI375942, SGI375943, SGI375944, SGI375942, SGI375945, SGI375946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90193[] = {SGI375947, SGI375948, SGI375949, SGI375950, SGI375948, SGI375951, SGI375952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90194[] = {SGI375953, SGI375954, SGI375955, SGI375956, SGI375954, SGI375957, SGI375958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90195[] = {SGI375959, SGI375960, SGI375961, SGI375962, SGI375960, SGI375963, SGI375964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90196[] = {SGI375965, SGI375966, SGI375967, SGI0, SGI375966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI375968, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90197[] = {SGI375969, SGI375970, SGI375971, SGI0, SGI375970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI375972, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90198[] = {SGI375973, SGI375974, SGI375975, SGI0, SGI375974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI375976, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90199[] = {SGI375977, SGI375978, SGI375979, SGI0, SGI375978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI375980, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90200[] = {SGI375981, SGI375982, SGI375983, SGI0, SGI375982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI375984, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90201[] = {SGI375985, SGI375986, SGI375987, SGI0, SGI375986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI375988, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90202[] = {SGI375989, SGI375990, SGI375991, SGI0, SGI375990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI375992, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90203[] = {SGI375993, SGI375994, SGI375995, SGI375996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI375997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90204[] = {SGI375998, SGI375999, SGI376000, SGI375999, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90205[] = {SGI376001, SGI376002, SGI376003, SGI376002, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90206[] = {SGI376004, SGI376005, SGI376006, SGI376005, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90207[] = {SGI376007, SGI376008, SGI376009, SGI376008, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90208[] = {SGI376010, SGI376011, SGI376012, SGI376011, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90209[] = {SGI376013, SGI376014, SGI376015, SGI376014, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90210[] = {SGI376016, SGI376017, SGI376018, SGI376017, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90211[] = {SGI376019, SGI376020, SGI376021, SGI376020, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90212[] = {SGI376022, SGI376023, SGI376024, SGI376023, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90213[] = {SGI376025, SGI376026, SGI376027, SGI376026, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90214[] = {SGI376028, SGI376029, SGI376030, SGI376029, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90215[] = {SGI376031, SGI376032, SGI376033, SGI376032, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90216[] = {SGI376034, SGI376035, SGI376036, SGI376035, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90217[] = {SGI376037, SGI376038, SGI376039, SGI376038, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90218[] = {SGI376040, SGI376041, SGI376042, SGI376041, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90219[] = {SGI376043, SGI376044, SGI376045, SGI376044, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90220[] = {SGI376046, SGI376047, SGI376048, SGI376047, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90221[] = {SGI376049, SGI376050, SGI376051, SGI376050, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90222[] = {SGI376052, SGI376053, SGI376054, SGI376053, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90223[] = {SGI376055, SGI376056, SGI376057, SGI376056, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90224[] = {SGI376058, SGI376059, SGI376060, SGI376059, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90225[] = {SGI376061, SGI376062, SGI376063, SGI376062, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90226[] = {SGI376064, SGI376065, SGI376066, SGI376065, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90227[] = {SGI376067, SGI376068, SGI376069, SGI376068, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90228[] = {SGI376070, SGI376071, SGI376072, SGI376071, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90229[] = {SGI376073, SGI376074, SGI376075, SGI376074, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90230[] = {SGI376076, SGI376077, SGI376078, SGI376077, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90231[] = {SGI376079, SGI376080, SGI376081, SGI376080, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90232[] = {SGI376082, SGI376083, SGI376084, SGI376083, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90233[] = {SGI376085, SGI376086, SGI376087, SGI376088, SGI376089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90234[] = {SGI376090, SGI376091, SGI376092, SGI376093, SGI376094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90235[] = {SGI376095, SGI376096, SGI376097, SGI376098, SGI376099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90236[] = {SGI376100, SGI376101, SGI376102, SGI376103, SGI376104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90237[] = {SGI376105, SGI376106, SGI376107, SGI376108, SGI376109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90238[] = {SGI376110, SGI376111, SGI376112, SGI376113, SGI376114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90239[] = {SGI376115, SGI376116, SGI376117, SGI376118, SGI376119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90240[] = {SGI376120, SGI376121, SGI376122, SGI0, SGI376123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90241[] = {SGI376124, SGI376125, SGI376126, SGI376127, SGI376128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90242[] = {SGI376129, SGI376130, SGI376131, SGI376132, SGI376133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90243[] = {SGI376134, SGI376135, SGI376136, SGI376137, SGI376138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90244[] = {SGI376139, SGI376140, SGI376141, SGI376142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90245[] = {SGI376144, SGI376145, SGI376146, SGI0, SGI376147, SGI0, SGI376148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90246[] = {SGI376149, SGI376150, SGI376151, SGI0, SGI376152, SGI0, SGI376153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90247[] = {SGI376154, SGI376155, SGI376156, SGI0, SGI376157, SGI0, SGI376158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90248[] = {SGI376159, SGI376160, SGI376161, SGI0, SGI376162, SGI0, SGI376163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90249[] = {SGI376164, SGI376165, SGI376166, SGI0, SGI376167, SGI0, SGI376168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90250[] = {SGI376169, SGI376170, SGI376171, SGI0, SGI376172, SGI0, SGI376173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90251[] = {SGI376174, SGI376175, SGI376176, SGI0, SGI376177, SGI0, SGI376178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90252[] = {SGI376179, SGI376180, SGI376181, SGI0, SGI376182, SGI0, SGI376183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90253[] = {SGI376184, SGI376185, SGI376186, SGI0, SGI376187, SGI0, SGI376188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90254[] = {SGI376189, SGI376190, SGI376191, SGI0, SGI376192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90255[] = {SGI376193, SGI376194, SGI376195, SGI0, SGI376196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90256[] = {SGI376197, SGI376198, SGI376199, SGI376200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90257[] = {SGI376202, SGI376203, SGI376204, SGI376205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90258[] = {SGI376207, SGI376208, SGI376209, SGI376210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90259[] = {SGI376211, SGI376212, SGI376213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90260[] = {SGI376214, SGI376215, SGI376216, SGI376217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90261[] = {SGI376218, SGI376219, SGI376220, SGI376221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90262[] = {SGI376222, SGI376223, SGI376224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90263[] = {SGI376225, SGI376226, SGI376227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90264[] = {SGI376228, SGI376229, SGI376230, SGI376231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90265[] = {SGI376232, SGI376233, SGI376234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90266[] = {SGI376235, SGI376236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90267[] = {SGI376237, SGI376238, SGI376239, SGI376240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90268[] = {SGI376241, SGI376242, SGI376243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90269[] = {SGI376244, SGI376245, SGI376246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90270[] = {SGI376247, SGI376248, SGI376249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90271[] = {SGI376250, SGI376251, SGI376252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90272[] = {SGI376253, SGI376254, SGI376255, SGI376256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90273[] = {SGI376257, SGI376258, SGI376259, SGI376260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90274[] = {SGI376261, SGI376262, SGI376263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90275[] = {SGI376264, SGI376265, SGI376266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90276[] = {SGI376267, SGI376268, SGI376269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90277[] = {SGI376270, SGI376271, SGI376272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90278[] = {SGI376273, SGI376274, SGI376275, SGI376276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90279[] = {SGI376277, SGI376278, SGI376279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90280[] = {SGI376280, SGI376281, SGI376282, SGI376283, SGI376284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90281[] = {SGI376285, SGI376286, SGI376287, SGI376288, SGI376289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90282[] = {SGI376290, SGI376291, SGI376292, SGI376293, SGI376294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90283[] = {SGI376295, SGI376296, SGI376297, SGI376298, SGI376299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90284[] = {SGI376300, SGI376301, SGI376302, SGI376303, SGI376304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90285[] = {SGI376305, SGI376306, SGI376307, SGI376308, SGI376309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90286[] = {SGI376310, SGI376311, SGI376312, SGI376313, SGI376314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90287[] = {SGI376315, SGI376316, SGI0, SGI376316, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90288[] = {SGI376317, SGI376318, SGI0, SGI376318, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90289[] = {SGI376319, SGI376320, SGI0, SGI376320, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90290[] = {SGI376321, SGI376322, SGI0, SGI376322, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90291[] = {SGI376323, SGI376324, SGI0, SGI376324, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90292[] = {SGI376325, SGI376326, SGI0, SGI376326, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90293[] = {SGI376327, SGI376328, SGI0, SGI376328, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90294[] = {SGI376329, SGI376330, SGI376331, SGI376330, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90295[] = {SGI376332, SGI376333, SGI376334, SGI376335, SGI376336, SGI376337, SGI376338, SGI0, SGI0, SGI0, SGI0, SGI376339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90296[] = {SGI376340, SGI376341, SGI376342, SGI376343, SGI376344, SGI376345, SGI376346, SGI0, SGI0, SGI0, SGI0, SGI376347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90297[] = {SGI376348, SGI376349, SGI0, SGI376350, SGI376351, SGI376352, SGI376353, SGI0, SGI0, SGI0, SGI0, SGI376354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90298[] = {SGI376355, SGI376356, SGI376357, SGI376358, SGI376359, SGI376360, SGI376361, SGI0, SGI0, SGI0, SGI0, SGI376362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90299[] = {SGI376363, SGI376364, SGI0, SGI376364, SGI376365, SGI0, SGI0, NULL};
const Segment * const SG90300[] = {SGI376366, SGI376367, SGI0, SGI376367, SGI376368, SGI0, SGI0, NULL};
const Segment * const SG90301[] = {SGI376369, SGI376370, SGI376371, SGI376372, SGI376370, SGI376373, SGI376374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90302[] = {SGI376376, SGI376377, SGI376378, SGI376379, SGI376377, SGI376380, SGI376381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90303[] = {SGI376383, SGI376384, SGI376385, SGI376386, SGI376384, SGI376387, SGI376388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90304[] = {SGI376390, SGI376391, SGI376392, SGI376393, SGI376391, SGI376394, SGI376395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90305[] = {SGI376397, SGI376398, SGI376399, SGI376400, SGI376398, SGI376401, SGI376402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90306[] = {SGI376404, SGI376405, SGI376406, SGI376407, SGI376405, SGI376408, SGI376409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90307[] = {SGI376411, SGI376412, SGI376413, SGI376414, SGI376412, SGI376415, SGI376416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90308[] = {SGI376418, SGI376419, SGI376420, SGI376421, SGI376419, SGI376422, SGI376423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90309[] = {SGI376425, SGI376426, SGI376427, SGI376428, SGI376426, SGI376429, SGI376430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90310[] = {SGI376432, SGI376433, SGI376434, SGI376435, SGI376433, SGI376436, SGI376437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90311[] = {SGI376439, SGI376440, SGI376441, SGI376442, SGI376440, SGI376443, SGI376444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90312[] = {SGI376446, SGI376447, SGI376448, SGI376449, SGI376447, SGI376450, SGI376451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90313[] = {SGI376453, SGI376454, SGI376455, SGI376456, SGI376454, SGI376457, SGI376458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90314[] = {SGI376460, SGI376461, SGI376462, SGI376463, SGI376461, SGI376464, SGI376465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90315[] = {SGI376467, SGI376468, SGI376469, SGI376470, SGI376468, SGI376471, SGI376472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90316[] = {SGI376474, SGI376475, SGI376476, SGI376477, SGI376475, SGI376478, SGI376479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90317[] = {SGI376481, SGI376482, SGI376483, SGI376484, SGI376482, SGI376485, SGI376486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90318[] = {SGI376488, SGI376489, SGI376490, SGI376491, SGI376489, SGI376492, SGI376493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90319[] = {SGI376495, SGI376496, SGI376497, SGI376498, SGI376496, SGI376499, SGI376500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90320[] = {SGI376502, SGI376503, SGI376504, SGI376505, SGI376503, SGI376506, SGI376507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90321[] = {SGI376509, SGI376510, SGI376511, SGI376512, SGI376510, SGI376513, SGI376514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90322[] = {SGI376516, SGI376517, SGI376518, SGI376519, SGI376517, SGI376520, SGI376521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90323[] = {SGI376523, SGI376524, SGI376525, SGI376526, SGI376524, SGI376527, SGI376528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90324[] = {SGI376530, SGI376531, SGI376532, SGI376533, SGI376531, SGI376534, SGI376535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90325[] = {SGI376537, SGI376538, SGI376539, SGI376540, SGI376538, SGI376541, SGI376542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90326[] = {SGI376544, SGI376545, SGI376546, SGI376547, SGI376545, SGI376548, SGI376549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90327[] = {SGI376551, SGI376552, SGI376553, SGI376554, SGI376552, SGI376555, SGI376556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90328[] = {SGI376558, SGI376559, SGI376560, SGI376561, SGI376559, SGI376562, SGI376563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90329[] = {SGI376565, SGI376566, SGI376567, SGI376568, SGI376566, SGI376569, SGI376570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90330[] = {SGI376572, SGI376573, SGI376574, SGI376575, SGI376573, SGI376576, SGI376577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90331[] = {SGI376579, SGI376580, SGI376581, SGI376582, SGI376580, SGI376583, SGI376584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90332[] = {SGI376586, SGI376587, SGI376588, SGI376589, SGI376587, SGI376590, SGI376591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90333[] = {SGI376593, SGI376594, SGI376595, SGI376596, SGI376594, SGI376597, SGI376598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90334[] = {SGI376600, SGI376601, SGI376602, SGI376603, SGI376601, SGI376604, SGI376605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90335[] = {SGI376607, SGI376608, SGI376609, SGI376609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90336[] = {SGI376611, SGI376612, SGI376613, SGI376613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90337[] = {SGI0, SGI376615, SGI376616, SGI376616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90338[] = {SGI376618, SGI376619, SGI376620, SGI376621, SGI376619, SGI376622, SGI376623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90339[] = {SGI376624, SGI376625, SGI376626, SGI376627, SGI376625, SGI376628, SGI376629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90340[] = {SGI376630, SGI376631, SGI376632, SGI376633, SGI376631, SGI376634, SGI376635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90341[] = {SGI376636, SGI376637, SGI376638, SGI376639, SGI376637, SGI376640, SGI376641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90342[] = {SGI376642, SGI376643, SGI0, SGI376643, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90343[] = {SGI376644, SGI376645, SGI376646, SGI376646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90344[] = {SGI376648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90345[] = {SGI376649, SGI376650, SGI376651, SGI376652, SGI376650, SGI376653, SGI376654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90346[] = {SGI376655, SGI376656, SGI376657, SGI376658, SGI376656, SGI376659, SGI376660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90347[] = {SGI376661, SGI8846, SGI376662, SGI376663, SGI8846, SGI376664, SGI376665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90348[] = {SGI376666, SGI376667, SGI376668, SGI376669, SGI376667, SGI376670, SGI376671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90349[] = {SGI376673, SGI376674, SGI376675, SGI376675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90350[] = {SGI376677, SGI376678, SGI376679, SGI376679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90351[] = {SGI376681, SGI376682, SGI376683, SGI376684, SGI376682, SGI376685, SGI376686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90352[] = {SGI376687, SGI376688, SGI376689, SGI376690, SGI376688, SGI376691, SGI376692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90353[] = {SGI376693, SGI376694, SGI376695, SGI376696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90354[] = {SGI376698, SGI376699, SGI376700, SGI376701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90355[] = {SGI376703, SGI376704, SGI376705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90356[] = {SGI376707, SGI376708, SGI376709, SGI376708, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90357[] = {SGI376710, SGI376711, SGI376712, SGI376711, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90358[] = {SGI376713, SGI376714, SGI376715, SGI376714, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90359[] = {SGI376716, SGI376717, SGI376718, SGI376717, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90360[] = {SGI376719, SGI376720, SGI376721, SGI376720, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90361[] = {SGI0, SGI376722, SGI376723, SGI376722, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90362[] = {SGI376724, SGI376725, SGI376726, SGI376725, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90363[] = {SGI376727, SGI376728, SGI376729, SGI376728, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90364[] = {SGI376730, SGI376731, SGI376732, SGI376731, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90365[] = {SGI376733, SGI376734, SGI376735, SGI376734, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90366[] = {SGI376736, SGI376737, SGI376738, SGI376737, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90367[] = {SGI376739, SGI376740, SGI376741, SGI376740, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90368[] = {SGI376742, SGI376743, SGI376744, SGI376743, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90369[] = {SGI376745, SGI376746, SGI376747, SGI376746, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90370[] = {SGI376748, SGI376749, SGI376750, SGI376749, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90371[] = {SGI376751, SGI376752, SGI376753, SGI376752, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90372[] = {SGI376754, SGI376755, SGI376756, SGI376755, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90373[] = {SGI376757, SGI376758, SGI376759, SGI376758, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90374[] = {SGI376760, SGI376761, SGI376762, SGI376761, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90375[] = {SGI376763, SGI376764, SGI376765, SGI376764, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90376[] = {SGI376766, SGI376767, SGI376768, SGI376767, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90377[] = {SGI376769, SGI376770, SGI376771, SGI376770, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90378[] = {SGI376772, SGI376773, SGI376774, SGI376773, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90379[] = {SGI376775, SGI376776, SGI376777, SGI376776, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90380[] = {SGI376778, SGI376779, SGI376780, SGI376779, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90381[] = {SGI376781, SGI376782, SGI376783, SGI376782, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90382[] = {SGI376784, SGI376785, SGI376786, SGI376785, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90383[] = {SGI376787, SGI376788, SGI376789, SGI376788, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90384[] = {SGI376790, SGI376791, SGI376792, SGI376791, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90385[] = {SGI376793, SGI376794, SGI376795, SGI376794, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90386[] = {SGI376796, SGI376797, SGI376798, SGI376797, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90387[] = {SGI376799, SGI376800, SGI376801, SGI376800, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90388[] = {SGI376802, SGI376803, SGI376804, SGI376803, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90389[] = {SGI376805, SGI376806, SGI376807, SGI376806, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90390[] = {SGI376808, SGI376809, SGI376810, SGI376809, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90391[] = {SGI376811, SGI376812, SGI376813, SGI376812, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90392[] = {SGI376814, SGI376815, SGI376816, SGI376815, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90393[] = {SGI376817, SGI376818, SGI376819, SGI376818, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90394[] = {SGI376820, SGI376821, SGI376822, SGI376821, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90395[] = {SGI376823, SGI376824, SGI376825, SGI376824, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90396[] = {SGI376826, SGI376827, SGI376828, SGI376827, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90397[] = {SGI376829, SGI376830, SGI376831, SGI376830, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90398[] = {SGI376832, SGI376833, SGI376834, SGI376833, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90399[] = {SGI376835, SGI376836, SGI376837, SGI376836, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90400[] = {SGI376838, SGI376839, SGI376840, SGI376839, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90401[] = {SGI376841, SGI376842, SGI376843, SGI376843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90402[] = {SGI376845, SGI376846, SGI376847, SGI376847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90403[] = {SGI376849, SGI376850, SGI376851, SGI376852, SGI0, SGI376853, SGI376854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90404[] = {SGI376856, SGI376857, SGI376858, SGI376859, SGI376860, SGI376861, SGI376862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90405[] = {SGI376864, SGI376865, SGI376866, SGI376867, SGI376868, SGI376869, SGI376870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90406[] = {SGI376872, SGI376873, SGI376874, SGI376875, SGI0, SGI376876, SGI376877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90407[] = {SGI376879, SGI376880, SGI376881, SGI376882, SGI376883, SGI376884, SGI376885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90408[] = {SGI376887, SGI376888, SGI376889, SGI376890, SGI376891, SGI376892, SGI376893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90409[] = {SGI376895, SGI376896, SGI376897, SGI376896, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90410[] = {SGI376898, SGI376899, SGI376900, SGI376899, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90411[] = {SGI376901, SGI376902, SGI376903, SGI376902, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90412[] = {SGI376904, SGI376905, SGI376906, SGI376907, SGI376905, SGI376908, SGI376909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90413[] = {SGI376910, SGI376911, SGI376912, SGI376911, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90414[] = {SGI376913, SGI376914, SGI376915, SGI376914, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90415[] = {SGI376916, SGI376917, SGI376918, SGI376917, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90416[] = {SGI376919, SGI376920, SGI376921, SGI376920, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90417[] = {SGI376922, SGI376923, SGI376924, SGI376923, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90418[] = {SGI376925, SGI376926, SGI376927, SGI376926, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90419[] = {SGI376928, SGI376929, SGI376930, SGI376929, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90420[] = {SGI376931, SGI376932, SGI376933, SGI376932, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90421[] = {SGI376934, SGI376935, SGI376936, SGI376935, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90422[] = {SGI376937, SGI376938, SGI376939, SGI376938, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90423[] = {SGI376940, SGI376941, SGI376942, SGI376941, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90424[] = {SGI376943, SGI376944, SGI376945, SGI376944, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90425[] = {SGI376946, SGI376947, SGI376948, SGI376947, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90426[] = {SGI376949, SGI376950, SGI376951, SGI376950, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90427[] = {SGI376952, SGI376953, SGI376954, SGI376953, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90428[] = {SGI376955, SGI376956, SGI376957, SGI376956, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90429[] = {SGI376958, SGI376959, SGI376960, SGI376959, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90430[] = {SGI376961, SGI376962, SGI376963, SGI376962, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90431[] = {SGI376964, SGI376965, SGI376966, SGI376965, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90432[] = {SGI376967, SGI376968, SGI376969, SGI376968, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90433[] = {SGI376970, SGI376971, SGI376972, SGI376971, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90434[] = {SGI376973, SGI376974, SGI376975, SGI376974, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90435[] = {SGI376976, SGI376977, SGI376978, SGI376977, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90436[] = {SGI376979, SGI376980, SGI376981, SGI376980, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90437[] = {SGI376982, SGI376983, SGI376984, SGI376983, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90438[] = {SGI376985, SGI376986, SGI376987, SGI376986, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90439[] = {SGI376988, SGI376989, SGI376990, SGI376989, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90440[] = {SGI376991, SGI376992, SGI376993, SGI376994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI376995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90441[] = {SGI376996, SGI376997, SGI376998, SGI376999, SGI377000, SGI377001, SGI377002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90442[] = {SGI377004, SGI377005, SGI377006, SGI377007, SGI377008, SGI377009, SGI377010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90443[] = {SGI377012, SGI377013, SGI377014, SGI377015, SGI377016, SGI377017, SGI377018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90444[] = {SGI377020, SGI377021, SGI377022, SGI377023, SGI377024, SGI377025, SGI377026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90445[] = {SGI377028, SGI377029, SGI377030, SGI377031, SGI377032, SGI377033, SGI377034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90446[] = {SGI377036, SGI377037, SGI377038, SGI377039, SGI377040, SGI377041, SGI377042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90447[] = {SGI377044, SGI377045, SGI377046, SGI377047, SGI377048, SGI377049, SGI377050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90448[] = {SGI377052, SGI377053, SGI377054, SGI377055, SGI0, SGI377056, SGI377057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90449[] = {SGI377059, SGI377060, SGI377061, SGI377062, SGI377063, SGI377064, SGI377065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90450[] = {SGI377067, SGI377068, SGI377069, SGI377070, SGI377071, SGI377072, SGI377073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90451[] = {SGI377075, SGI377076, SGI377077, SGI377078, SGI377079, SGI377080, SGI377081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90452[] = {SGI377083, SGI377084, SGI377085, SGI377086, SGI0, SGI377087, SGI377088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90453[] = {SGI377090, SGI377091, SGI377092, SGI377093, SGI377094, SGI377095, SGI377096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90454[] = {SGI377098, SGI377099, SGI377100, SGI377101, SGI377102, SGI377103, SGI377104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90455[] = {SGI377106, SGI377107, SGI377108, SGI377109, SGI377110, SGI377111, SGI377112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90456[] = {SGI377114, SGI377115, SGI377116, SGI377117, SGI0, SGI377118, SGI377119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90457[] = {SGI377121, SGI377122, SGI377123, SGI377124, SGI377125, SGI377126, SGI377127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90458[] = {SGI377129, SGI377130, SGI377131, SGI377132, SGI377133, SGI377134, SGI377135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90459[] = {SGI377137, SGI377138, SGI0, SGI377138, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90460[] = {SGI377139, SGI377140, SGI0, SGI377140, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90461[] = {SGI377141, SGI377142, SGI377143, SGI377144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90462[] = {SGI377146, SGI377147, SGI377148, SGI377149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90463[] = {SGI377151, SGI377152, SGI377153, SGI377154, SGI377155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90464[] = {SGI377156, SGI377157, SGI377158, SGI377159, SGI377160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90465[] = {SGI377161, SGI377162, SGI377163, SGI377164, SGI377165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90466[] = {SGI377166, SGI377167, SGI377168, SGI377169, SGI377170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90467[] = {SGI377171, SGI377172, SGI377173, SGI377174, SGI377175, SGI0, SGI377176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90468[] = {SGI377177, SGI377178, SGI377179, SGI377178, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90469[] = {SGI377180, SGI377181, SGI377182, SGI377183, SGI0, SGI377184, SGI377185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90470[] = {SGI377187, SGI377188, SGI377189, SGI377190, SGI377191, SGI377192, SGI377193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90471[] = {SGI377195, SGI377196, SGI377197, SGI377198, SGI377199, SGI377200, SGI377201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90472[] = {SGI377203, SGI377204, SGI377205, SGI377206, SGI377207, SGI377208, SGI377209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90473[] = {SGI377211, SGI377212, SGI377213, SGI377214, SGI377215, SGI377216, SGI377217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90474[] = {SGI377219, SGI377220, SGI377221, SGI377222, SGI0, SGI377223, SGI377224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90475[] = {SGI377226, SGI377227, SGI377228, SGI377229, SGI377230, SGI377231, SGI377232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90476[] = {SGI377234, SGI377235, SGI377236, SGI377237, SGI0, SGI377238, SGI377239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90477[] = {SGI377241, SGI377242, SGI377243, SGI377244, SGI0, SGI377245, SGI377246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90478[] = {SGI377248, SGI377249, SGI377250, SGI377251, SGI377252, SGI377253, SGI377254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90479[] = {SGI377256, SGI377257, SGI377258, SGI377259, SGI377260, SGI377261, SGI377262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90480[] = {SGI377264, SGI377265, SGI377266, SGI377267, SGI0, SGI377268, SGI377269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90481[] = {SGI377271, SGI377272, SGI377273, SGI377274, SGI377275, SGI377276, SGI377277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90482[] = {SGI377279, SGI377280, SGI377281, SGI377282, SGI0, SGI377283, SGI377284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90483[] = {SGI377286, SGI377287, SGI377288, SGI377289, SGI377290, SGI377291, SGI377292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90484[] = {SGI377294, SGI377295, SGI377296, SGI377297, SGI377298, SGI377299, SGI377300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90485[] = {SGI377302, SGI377303, SGI377304, SGI377305, SGI377306, SGI377307, SGI377308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90486[] = {SGI377310, SGI377311, SGI0, SGI377311, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90487[] = {SGI377312, SGI377313, SGI0, SGI377313, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90488[] = {SGI377314, SGI377315, SGI377316, SGI377317, SGI377318, SGI0, SGI377319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90489[] = {SGI377321, SGI377322, SGI377323, SGI377324, SGI377325, SGI0, SGI377326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90490[] = {SGI377328, SGI377329, SGI377330, SGI377331, SGI377332, SGI0, SGI377333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90491[] = {SGI377335, SGI377336, SGI377337, SGI377338, SGI0, SGI0, SGI377339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90492[] = {SGI377341, SGI377342, SGI377343, SGI377344, SGI377345, SGI0, SGI377346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90493[] = {SGI377348, SGI377349, SGI377350, SGI377351, SGI377352, SGI377353, SGI377354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90494[] = {SGI377356, SGI377357, SGI377358, SGI377358, SGI0, SGI0, SGI0, SGI377359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90495[] = {SGI377360, SGI377361, SGI377362, SGI377363, SGI377364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90496[] = {SGI377365, SGI377366, SGI377367, SGI377368, SGI377369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90497[] = {SGI377370, SGI377371, SGI377372, SGI377373, SGI377374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90498[] = {SGI377375, SGI377376, SGI377377, SGI377378, SGI377379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90499[] = {SGI377380, SGI377381, SGI377382, SGI377383, SGI377384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90500[] = {SGI377385, SGI377386, SGI377387, SGI377388, SGI377389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90501[] = {SGI377390, SGI377391, SGI377392, SGI377393, SGI377394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90502[] = {SGI377395, SGI377396, SGI377397, SGI377398, SGI377399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90503[] = {SGI377400, SGI377401, SGI377402, SGI377403, SGI377404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90504[] = {SGI377405, SGI377406, SGI377407, SGI377408, SGI377409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90505[] = {SGI377410, SGI377411, SGI377412, SGI377413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90506[] = {SGI377415, SGI377416, SGI377417, SGI377417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90507[] = {SGI377419, SGI377420, SGI377421, SGI377422, SGI377423, SGI0, SGI377422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90508[] = {SGI377424, SGI377425, SGI377426, SGI377425, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90509[] = {SGI377427, SGI377428, SGI377429, SGI377430, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90510[] = {SGI377431, SGI377432, SGI377433, SGI377434, SGI0, SGI0, SGI0, SGI0, SGI377435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90511[] = {SGI377436, SGI377437, SGI377438, SGI377439, SGI377440, SGI0, SGI377441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90512[] = {SGI377442, SGI377443, SGI377444, SGI377445, SGI377446, SGI377447, SGI377448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90513[] = {SGI377449, SGI377450, SGI377451, SGI377452, SGI377453, SGI377454, SGI377455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90514[] = {SGI377456, SGI377457, SGI377458, SGI377459, SGI377460, SGI377461, SGI377462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90515[] = {SGI377463, SGI377464, SGI377465, SGI377466, SGI377467, SGI377468, SGI377469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90516[] = {SGI377470, SGI377471, SGI377472, SGI377473, SGI377474, SGI377475, SGI377476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90517[] = {SGI377477, SGI377478, SGI377479, SGI377480, SGI377481, SGI377482, SGI377483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90518[] = {SGI377484, SGI377485, SGI377486, SGI377487, SGI377488, SGI377489, SGI377490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90519[] = {SGI377491, SGI377492, SGI377493, SGI377494, SGI377495, SGI377496, SGI377497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90520[] = {SGI377498, SGI377499, SGI377500, SGI377501, SGI377502, SGI377503, SGI377504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90521[] = {SGI377505, SGI377506, SGI377507, SGI377508, SGI377509, SGI377510, SGI377511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90522[] = {SGI377512, SGI377513, SGI377514, SGI377515, SGI377516, SGI377517, SGI377518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90523[] = {SGI377519, SGI377520, SGI377521, SGI377522, SGI377523, SGI377524, SGI377525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90524[] = {SGI377526, SGI377527, SGI377528, SGI377529, SGI377530, SGI377531, SGI377532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90525[] = {SGI377533, SGI377534, SGI377535, SGI377536, SGI377537, SGI377538, SGI377539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90526[] = {SGI377540, SGI377541, SGI377542, SGI377543, SGI377544, SGI0, SGI377543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90527[] = {SGI377545, SGI377546, SGI377547, SGI377548, SGI377549, SGI377550, SGI377551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90528[] = {SGI377552, SGI377553, SGI377554, SGI377555, SGI377556, SGI377557, SGI377558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90529[] = {SGI377559, SGI377560, SGI377561, SGI377562, SGI377563, SGI377564, SGI377565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90530[] = {SGI377566, SGI377567, SGI377568, SGI377568, SGI0, SGI0, SGI0, SGI377569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90531[] = {SGI377570, SGI377571, SGI377572, SGI377572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90532[] = {SGI377573, SGI377574, SGI377575, SGI0, SGI0, SGI0, SGI0, SGI377576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90533[] = {SGI377577, SGI377578, SGI377579, SGI377579, SGI0, SGI0, SGI0, SGI377580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90534[] = {SGI377581, SGI377582, SGI377583, SGI377583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90535[] = {SGI377585, SGI377586, SGI377587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90536[] = {SGI377589, SGI377590, SGI377591, SGI377591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90537[] = {SGI377592, SGI377593, SGI377594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90538[] = {SGI377595, SGI377596, SGI377597, SGI377597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90539[] = {SGI377598, SGI377599, SGI377600, SGI377601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90540[] = {SGI377602, SGI377603, SGI377604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90541[] = {SGI377605, SGI377606, SGI377607, SGI377607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90542[] = {SGI377608, SGI377609, SGI377610, SGI377610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90543[] = {SGI377611, SGI377612, SGI377613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90544[] = {SGI377614, SGI377615, SGI377616, SGI377616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90545[] = {SGI377617, SGI377618, SGI377619, SGI377619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90546[] = {SGI377620, SGI377621, SGI377622, SGI377622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90547[] = {SGI377623, SGI377624, SGI377625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90548[] = {SGI377626, SGI377627, SGI377628, SGI377629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90549[] = {SGI377631, SGI377632, SGI377633, SGI377634, SGI377635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90550[] = {SGI377636, SGI377637, SGI377638, SGI377639, SGI377640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90551[] = {SGI377641, SGI377642, SGI377643, SGI377644, SGI377645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90552[] = {SGI377646, SGI377647, SGI377648, SGI377649, SGI377650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90553[] = {SGI377651, SGI377652, SGI377653, SGI377654, SGI377655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90554[] = {SGI377656, SGI377657, SGI377658, SGI377659, SGI377660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90555[] = {SGI377661, SGI377662, SGI377663, SGI377664, SGI377665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90556[] = {SGI377666, SGI377667, SGI377668, SGI377669, SGI377670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90557[] = {SGI377671, SGI377672, SGI377673, SGI377674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90558[] = {SGI377675, SGI377676, SGI377677, SGI377678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90559[] = {SGI377679, SGI377680, SGI377681, SGI377682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90560[] = {SGI377683, SGI377684, SGI377685, SGI377686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90561[] = {SGI377687, SGI377688, SGI377689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90562[] = {SGI377690, SGI377691, SGI377692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90563[] = {SGI377693, SGI377694, SGI377695, SGI377696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90564[] = {SGI377697, SGI377698, SGI377699, SGI377700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90565[] = {SGI377701, SGI377702, SGI377703, SGI377703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90566[] = {SGI377704, SGI377705, SGI377706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90567[] = {SGI377707, SGI377708, SGI377709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90568[] = {SGI377710, SGI377711, SGI377712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90569[] = {SGI377713, SGI377714, SGI377715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90570[] = {SGI377716, SGI377717, SGI377718, SGI377719, SGI377717, SGI377720, SGI377721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90571[] = {SGI377722, SGI377723, SGI377724, SGI377725, SGI377723, SGI377726, SGI377727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90572[] = {SGI377728, SGI377729, SGI377730, SGI377731, SGI377729, SGI377732, SGI377733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90573[] = {SGI377734, SGI377735, SGI377736, SGI377737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90574[] = {SGI377739, SGI377740, SGI377741, SGI377742, SGI0, SGI0, SGI0, SGI0, SGI377743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90575[] = {SGI377744, SGI377745, SGI377746, SGI377747, SGI377748, SGI377749, SGI377750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90576[] = {SGI377752, SGI377753, SGI377754, SGI377755, SGI377756, SGI377757, SGI377758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90577[] = {SGI377760, SGI377761, SGI377762, SGI377763, SGI377764, SGI377765, SGI377766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90578[] = {SGI377768, SGI377769, SGI377770, SGI377771, SGI377772, SGI377773, SGI377774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90579[] = {SGI377776, SGI377777, SGI377778, SGI377779, SGI377780, SGI377781, SGI377782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90580[] = {SGI377784, SGI377785, SGI377786, SGI377787, SGI377788, SGI377789, SGI377790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90581[] = {SGI377792, SGI377793, SGI377794, SGI377795, SGI377796, SGI377797, SGI377798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90582[] = {SGI377800, SGI377801, SGI377802, SGI377803, SGI0, SGI377804, SGI377805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90583[] = {SGI377807, SGI377808, SGI377809, SGI377810, SGI377811, SGI377812, SGI377813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90584[] = {SGI377815, SGI377816, SGI377817, SGI377818, SGI377819, SGI377820, SGI377821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90585[] = {SGI377823, SGI377824, SGI377825, SGI377826, SGI377827, SGI377828, SGI377829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90586[] = {SGI377831, SGI377832, SGI377833, SGI377834, SGI377835, SGI377836, SGI377837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90587[] = {SGI377839, SGI377840, SGI377841, SGI377842, SGI377843, SGI377844, SGI377845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90588[] = {SGI377847, SGI377848, SGI377849, SGI377850, SGI0, SGI377851, SGI377852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90589[] = {SGI377854, SGI377855, SGI377856, SGI377857, SGI377858, SGI377859, SGI377860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI377861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90590[] = {SGI377862, SGI377863, SGI377864, SGI0, SGI377865, SGI0, SGI377866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90591[] = {SGI377867, SGI377868, SGI377869, SGI0, SGI377870, SGI0, SGI377871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90592[] = {SGI377872, SGI377873, SGI377874, SGI0, SGI377875, SGI0, SGI377876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90593[] = {SGI377877, SGI377878, SGI377879, SGI0, SGI377880, SGI0, SGI377881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90594[] = {SGI377882, SGI377883, SGI377884, SGI0, SGI377885, SGI0, SGI377886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90595[] = {SGI377887, SGI377888, SGI377889, SGI0, SGI377890, SGI0, SGI377891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90596[] = {SGI377892, SGI377893, SGI377894, SGI0, SGI377895, SGI0, SGI377896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90597[] = {SGI377897, SGI377898, SGI377899, SGI0, SGI377900, SGI0, SGI377901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90598[] = {SGI377902, SGI377903, SGI377904, SGI0, SGI377905, SGI0, SGI377906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90599[] = {SGI377907, SGI377908, SGI377909, SGI0, SGI377910, SGI0, SGI377911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90600[] = {SGI377912, SGI377913, SGI377914, SGI0, SGI377915, SGI0, SGI377916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90601[] = {SGI377917, SGI377918, SGI377919, SGI0, SGI377920, SGI0, SGI377921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90602[] = {SGI377922, SGI377923, SGI377924, SGI0, SGI377925, SGI0, SGI377926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90603[] = {SGI377927, SGI377928, SGI377929, SGI0, SGI377930, SGI0, SGI377931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90604[] = {SGI377932, SGI377933, SGI377934, SGI0, SGI377935, SGI0, SGI377936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90605[] = {SGI377937, SGI377938, SGI377939, SGI0, SGI377940, SGI0, SGI377941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90606[] = {SGI377942, SGI377943, SGI377944, SGI377943, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90607[] = {SGI377945, SGI377946, SGI377947, SGI377946, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90608[] = {SGI377948, SGI377949, SGI377950, SGI377949, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90609[] = {SGI377951, SGI377952, SGI377953, SGI377952, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90610[] = {SGI377954, SGI377955, SGI377956, SGI377955, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90611[] = {SGI377957, SGI377958, SGI377959, SGI377958, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90612[] = {SGI377960, SGI377961, SGI377962, SGI0, SGI377963, SGI0, SGI377964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90613[] = {SGI377965, SGI377966, SGI377967, SGI0, SGI377968, SGI0, SGI377969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90614[] = {SGI377970, SGI377971, SGI377972, SGI377973, SGI377974, SGI377975, SGI377976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90615[] = {SGI377977, SGI377978, SGI377979, SGI0, SGI377980, SGI0, SGI377981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90616[] = {SGI377982, SGI377983, SGI377984, SGI0, SGI377985, SGI0, SGI377986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90617[] = {SGI377987, SGI377988, SGI377989, SGI0, SGI377990, SGI0, SGI377991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90618[] = {SGI377992, SGI377993, SGI377994, SGI0, SGI377995, SGI0, SGI377996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90619[] = {SGI377997, SGI377998, SGI377999, SGI378000, SGI378001, SGI378002, SGI378003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90620[] = {SGI378004, SGI378005, SGI378006, SGI0, SGI378007, SGI0, SGI378008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90621[] = {SGI378009, SGI378010, SGI378011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90622[] = {SGI378012, SGI378013, SGI378014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90623[] = {SGI378015, SGI378016, SGI378017, SGI0, SGI378018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90624[] = {SGI378019, SGI378020, SGI378021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90625[] = {SGI378022, SGI378023, SGI378024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90626[] = {SGI378025, SGI378026, SGI378027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90627[] = {SGI378028, SGI378029, SGI378030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90628[] = {SGI378031, SGI378032, SGI378033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90629[] = {SGI378034, SGI378035, SGI378036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90630[] = {SGI378037, SGI378038, SGI378039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90631[] = {SGI378040, SGI378041, SGI378042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90632[] = {SGI378043, SGI378044, SGI378045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90633[] = {SGI378046, SGI378047, SGI378048, SGI378048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90634[] = {SGI378050, SGI378051, SGI378052, SGI378052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90635[] = {SGI378054, SGI378055, SGI378056, SGI378057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90636[] = {SGI378058, SGI378059, SGI378060, SGI378061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90637[] = {SGI378063, SGI378064, SGI378065, SGI378066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90638[] = {SGI378068, SGI378069, SGI378070, SGI378071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90639[] = {SGI378073, SGI378074, SGI378075, SGI378076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90640[] = {SGI378078, SGI378079, SGI378080, SGI378081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90641[] = {SGI378083, SGI378084, SGI378085, SGI378086, SGI0, SGI0, SGI0, SGI0, SGI378087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90642[] = {SGI378088, SGI378089, SGI378090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90643[] = {SGI378092, SGI378093, SGI378094, SGI378093, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90644[] = {SGI378095, SGI378096, SGI378097, SGI378096, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90645[] = {SGI378098, SGI378099, SGI378100, SGI378099, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90646[] = {SGI378101, SGI378102, SGI378103, SGI378103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378104, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90647[] = {SGI378105, SGI378106, SGI378107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90648[] = {SGI378109, SGI378110, SGI378111, SGI378112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378113, SGI0, NULL};
const Segment * const SG90649[] = {SGI378114, SGI378115, SGI378116, SGI378116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90650[] = {SGI378118, SGI378119, SGI378120, SGI378120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90651[] = {SGI378122, SGI378123, SGI378124, SGI378125, SGI378126, SGI378127, SGI378128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90652[] = {SGI378130, SGI378131, SGI378132, SGI378133, SGI378134, SGI378135, SGI378136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90653[] = {SGI378138, SGI378139, SGI378140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90654[] = {SGI378141, SGI378142, SGI378143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90655[] = {SGI378144, SGI378145, SGI378146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90656[] = {SGI378147, SGI378148, SGI378149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90657[] = {SGI378150, SGI378151, SGI378152, SGI378153, SGI0, SGI0, SGI0, SGI0, SGI378154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90658[] = {SGI378155, SGI378156, SGI378157, SGI378157, SGI0, SGI0, SGI0, SGI378158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90659[] = {SGI378159, SGI378160, SGI378161, SGI378161, SGI0, SGI0, SGI0, SGI378162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90660[] = {SGI378163, SGI378164, SGI378165, SGI378165, SGI0, SGI0, SGI0, SGI378166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90661[] = {SGI378167, SGI378168, SGI378169, SGI378170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378171, SGI0, NULL};
const Segment * const SG90662[] = {SGI378172, SGI378173, SGI378174, SGI378175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378176, SGI0, NULL};
const Segment * const SG90663[] = {SGI378177, SGI378178, SGI378179, SGI378180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378181, SGI0, NULL};
const Segment * const SG90664[] = {SGI378182, SGI378183, SGI378184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378185, SGI0, NULL};
const Segment * const SG90665[] = {SGI378186, SGI378187, SGI378188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90666[] = {SGI378190, SGI378191, SGI378192, SGI378192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90667[] = {SGI378194, SGI378195, SGI378196, SGI378196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90668[] = {SGI378198, SGI378199, SGI378200, SGI378201, SGI378202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90669[] = {SGI378203, SGI378204, SGI378205, SGI378206, SGI378207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90670[] = {SGI378208, SGI378209, SGI378210, SGI378211, SGI378212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90671[] = {SGI378213, SGI378214, SGI378215, SGI378216, SGI378217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90672[] = {SGI378218, SGI378219, SGI378220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90673[] = {SGI0, SGI378222, SGI378223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90674[] = {SGI378224, SGI378225, SGI378226, SGI378225, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90675[] = {SGI378227, SGI378228, SGI378229, SGI378228, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90676[] = {SGI378230, SGI378231, SGI378232, SGI378231, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90677[] = {SGI378233, SGI378234, SGI378235, SGI378234, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90678[] = {SGI378236, SGI378237, SGI378238, SGI378237, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90679[] = {SGI378239, SGI378240, SGI378241, SGI378240, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90680[] = {SGI378242, SGI378243, SGI378244, SGI378243, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90681[] = {SGI378245, SGI378246, SGI378247, SGI378246, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90682[] = {SGI378248, SGI378249, SGI378250, SGI378249, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90683[] = {SGI378251, SGI378252, SGI378253, SGI378252, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90684[] = {SGI378254, SGI378255, SGI378256, SGI378255, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90685[] = {SGI378257, SGI378258, SGI378259, SGI378258, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90686[] = {SGI378260, SGI378261, SGI378262, SGI378261, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90687[] = {SGI378263, SGI378264, SGI378265, SGI378264, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90688[] = {SGI378266, SGI378267, SGI378268, SGI378267, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90689[] = {SGI378269, SGI378270, SGI378271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90690[] = {SGI378273, SGI378274, SGI378275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90691[] = {SGI378277, SGI378278, SGI378279, SGI378280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90692[] = {SGI378282, SGI378283, SGI378284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90693[] = {SGI378286, SGI378287, SGI378288, SGI378288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90694[] = {SGI378290, SGI378291, SGI378292, SGI378293, SGI378291, SGI0, SGI378294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90695[] = {SGI378295, SGI378296, SGI378297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90696[] = {SGI378299, SGI378300, SGI378301, SGI0, SGI0, SGI0, SGI378302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90697[] = {SGI378303, SGI378304, SGI378305, SGI0, SGI378306, SGI0, SGI378307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90698[] = {SGI378308, SGI378309, SGI378310, SGI0, SGI0, SGI0, SGI378311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90699[] = {SGI378312, SGI378313, SGI378314, SGI378315, SGI0, SGI378316, SGI378317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90700[] = {SGI378318, SGI378319, SGI378320, SGI378321, SGI378322, SGI378323, SGI378324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90701[] = {SGI378325, SGI378326, SGI378327, SGI0, SGI378328, SGI0, SGI378329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90702[] = {SGI378330, SGI378331, SGI378332, SGI378333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90703[] = {SGI378335, SGI378336, SGI378337, SGI378337, SGI0, SGI0, SGI0, SGI378338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90704[] = {SGI378339, SGI378340, SGI378341, SGI378342, SGI0, SGI378343, SGI378344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378345, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90705[] = {SGI378346, SGI378347, SGI378348, SGI0, SGI0, SGI0, SGI378349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378350, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90706[] = {SGI378351, SGI378352, SGI378353, SGI378354, SGI378352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90707[] = {SGI378355, SGI378356, SGI378357, SGI0, SGI378358, SGI0, SGI378359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90708[] = {SGI378361, SGI378362, SGI378363, SGI0, SGI378364, SGI0, SGI378365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90709[] = {SGI378367, SGI378368, SGI378369, SGI0, SGI378370, SGI0, SGI378371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90710[] = {SGI378373, SGI378374, SGI378375, SGI378376, SGI378377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90711[] = {SGI378378, SGI378379, SGI378380, SGI0, SGI378381, SGI0, SGI378382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90712[] = {SGI378383, SGI378384, SGI378385, SGI378386, SGI378384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90713[] = {SGI378387, SGI378388, SGI378389, SGI378390, SGI378388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90714[] = {SGI378391, SGI378392, SGI378393, SGI378394, SGI378392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90715[] = {SGI378395, SGI378396, SGI378397, SGI378398, SGI378396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90716[] = {SGI378399, SGI378400, SGI378401, SGI378402, SGI378400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90717[] = {SGI378403, SGI378404, SGI378405, SGI378406, SGI378404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90718[] = {SGI378407, SGI378408, SGI378409, SGI378409, SGI378408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90719[] = {SGI378410, SGI378411, SGI378412, SGI378412, SGI378411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90720[] = {SGI378413, SGI378414, SGI378415, SGI378415, SGI378414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90721[] = {SGI378416, SGI378417, SGI378418, SGI0, SGI378419, SGI0, SGI378420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90722[] = {SGI378421, SGI378422, SGI378423, SGI0, SGI378424, SGI0, SGI378425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90723[] = {SGI378426, SGI378427, SGI378428, SGI0, SGI378429, SGI0, SGI378430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90724[] = {SGI378431, SGI378432, SGI378433, SGI0, SGI378434, SGI0, SGI378435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90725[] = {SGI378436, SGI378437, SGI378438, SGI0, SGI378439, SGI0, SGI378440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90726[] = {SGI378441, SGI378442, SGI378443, SGI0, SGI378444, SGI0, SGI378445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90727[] = {SGI378446, SGI378447, SGI378448, SGI0, SGI378449, SGI0, SGI378450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90728[] = {SGI378451, SGI378452, SGI378453, SGI0, SGI378454, SGI0, SGI378455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90729[] = {SGI378456, SGI378457, SGI378458, SGI378458, SGI0, SGI0, SGI0, SGI378459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90730[] = {SGI378460, SGI378461, SGI378462, SGI378462, SGI0, SGI0, SGI0, SGI378463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90731[] = {SGI378464, SGI378465, SGI378466, SGI378466, SGI0, SGI0, SGI0, SGI378467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90732[] = {SGI378468, SGI378469, SGI0, SGI378469, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90733[] = {SGI378470, SGI378471, SGI0, SGI378471, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90734[] = {SGI378472, SGI378473, SGI0, SGI378473, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90735[] = {SGI378474, SGI378475, SGI0, SGI378475, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90736[] = {SGI378476, SGI378477, SGI0, SGI378477, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90737[] = {SGI378478, SGI378479, SGI0, SGI378479, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90738[] = {SGI378480, SGI378481, SGI0, SGI378481, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90739[] = {SGI378482, SGI378483, SGI0, SGI378483, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90740[] = {SGI378484, SGI378485, SGI0, SGI378485, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90741[] = {SGI378486, SGI378487, SGI0, SGI378487, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90742[] = {SGI378488, SGI378489, SGI0, SGI378489, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90743[] = {SGI378490, SGI378491, SGI0, SGI378491, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90744[] = {SGI378492, SGI378493, SGI0, SGI378493, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90745[] = {SGI378494, SGI378495, SGI0, SGI378495, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90746[] = {SGI378496, SGI378497, SGI0, SGI378497, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90747[] = {SGI378498, SGI378499, SGI0, SGI378499, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90748[] = {SGI378500, SGI378501, SGI0, SGI378501, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90749[] = {SGI378502, SGI378503, SGI0, SGI378503, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90750[] = {SGI378504, SGI378505, SGI0, SGI378505, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90751[] = {SGI378506, SGI378507, SGI0, SGI378507, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90752[] = {SGI378508, SGI378509, SGI0, SGI378509, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90753[] = {SGI378510, SGI378511, SGI0, SGI378511, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90754[] = {SGI378512, SGI378513, SGI0, SGI378513, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90755[] = {SGI378514, SGI378515, SGI0, SGI378515, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90756[] = {SGI378516, SGI378517, SGI0, SGI378517, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90757[] = {SGI378518, SGI378519, SGI0, SGI378519, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90758[] = {SGI378520, SGI378521, SGI0, SGI378521, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90759[] = {SGI378522, SGI378523, SGI0, SGI378523, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90760[] = {SGI378524, SGI378525, SGI0, SGI378525, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90761[] = {SGI378526, SGI378527, SGI0, SGI378527, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90762[] = {SGI378528, SGI378529, SGI378530, SGI378529, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90763[] = {SGI378531, SGI378532, SGI378533, SGI378534, SGI378532, SGI378535, SGI378536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90764[] = {SGI378537, SGI378538, SGI378539, SGI378539, SGI0, SGI0, SGI0, SGI378540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90765[] = {SGI378541, SGI378542, SGI378543, SGI378542, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90766[] = {SGI378544, SGI378545, SGI378546, SGI378545, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90767[] = {SGI378547, SGI378548, SGI0, SGI378548, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90768[] = {SGI378549, SGI378550, SGI0, SGI378550, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90769[] = {SGI378551, SGI378552, SGI0, SGI378552, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90770[] = {SGI378553, SGI378554, SGI0, SGI378554, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90771[] = {SGI378555, SGI378556, SGI0, SGI378556, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90772[] = {SGI378557, SGI378558, SGI0, SGI378558, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90773[] = {SGI378559, SGI378560, SGI0, SGI378560, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90774[] = {SGI378561, SGI378562, SGI0, SGI378562, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90775[] = {SGI378563, SGI378564, SGI0, SGI378564, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90776[] = {SGI378565, SGI378566, SGI0, SGI378566, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90777[] = {SGI378567, SGI378568, SGI0, SGI378568, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90778[] = {SGI378569, SGI378570, SGI0, SGI378570, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90779[] = {SGI378571, SGI378572, SGI0, SGI378572, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90780[] = {SGI378573, SGI378574, SGI0, SGI378574, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90781[] = {SGI378575, SGI378576, SGI0, SGI378576, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90782[] = {SGI378577, SGI378578, SGI0, SGI378578, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90783[] = {SGI378579, SGI378580, SGI0, SGI378580, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90784[] = {SGI378581, SGI378582, SGI0, SGI378582, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90785[] = {SGI378583, SGI378584, SGI0, SGI378584, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90786[] = {SGI378585, SGI378586, SGI0, SGI378586, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90787[] = {SGI378587, SGI378588, SGI0, SGI378588, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90788[] = {SGI378589, SGI378590, SGI0, SGI378590, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90789[] = {SGI378591, SGI378592, SGI0, SGI378592, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90790[] = {SGI378593, SGI378594, SGI0, SGI378594, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90791[] = {SGI378595, SGI378596, SGI0, SGI378596, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90792[] = {SGI378597, SGI378598, SGI0, SGI378598, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90793[] = {SGI378599, SGI378600, SGI0, SGI378600, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90794[] = {SGI378601, SGI378602, SGI0, SGI378602, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90795[] = {SGI378603, SGI378604, SGI0, SGI378604, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90796[] = {SGI378605, SGI378606, SGI0, SGI378606, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90797[] = {SGI378607, SGI378608, SGI0, SGI378608, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90798[] = {SGI378609, SGI378610, SGI0, SGI378610, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90799[] = {SGI378611, SGI378612, SGI0, SGI378612, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90800[] = {SGI378613, SGI378614, SGI0, SGI378614, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90801[] = {SGI378615, SGI378616, SGI0, SGI378616, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90802[] = {SGI378617, SGI378618, SGI0, SGI378618, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90803[] = {SGI378619, SGI378620, SGI0, SGI378620, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90804[] = {SGI378621, SGI378622, SGI0, SGI378622, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90805[] = {SGI378623, SGI378624, SGI0, SGI378624, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90806[] = {SGI378625, SGI378626, SGI0, SGI378626, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90807[] = {SGI378627, SGI378628, SGI0, SGI378628, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90808[] = {SGI378629, SGI378630, SGI0, SGI378630, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90809[] = {SGI378631, SGI293388, SGI293389, SGI0, SGI293388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90810[] = {SGI378632, SGI378633, SGI378634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90811[] = {SGI378636, SGI378637, SGI378638, SGI0, SGI378639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90812[] = {SGI378640, SGI378641, SGI378642, SGI378643, SGI378644, SGI378645, SGI378646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378647, NULL};
const Segment * const SG90813[] = {SGI378648, SGI378649, SGI378650, SGI0, SGI378651, SGI0, SGI378652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378653, NULL};
const Segment * const SG90814[] = {SGI378654, SGI378655, SGI378656, SGI378657, SGI378658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90815[] = {SGI378659, SGI378660, SGI378661, SGI378662, SGI378663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90816[] = {SGI378664, SGI378665, SGI378666, SGI378667, SGI378668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90817[] = {SGI378669, SGI378670, SGI378671, SGI378672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90818[] = {SGI378674, SGI378675, SGI378676, SGI378676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90819[] = {SGI378677, SGI378678, SGI378679, SGI378680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90820[] = {SGI378682, SGI378683, SGI378684, SGI0, SGI378685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90821[] = {SGI378686, SGI378687, SGI378688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90822[] = {SGI378690, SGI378691, SGI378692, SGI378693, SGI0, SGI0, SGI0, SGI0, SGI378694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90823[] = {SGI378695, SGI378696, SGI0, SGI378696, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90824[] = {SGI378697, SGI378698, SGI0, SGI378698, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90825[] = {SGI378699, SGI378700, SGI0, SGI378700, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90826[] = {SGI378701, SGI378702, SGI0, SGI378702, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90827[] = {SGI378703, SGI378704, SGI0, SGI378704, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90828[] = {SGI378705, SGI378706, SGI0, SGI378706, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90829[] = {SGI378707, SGI378708, SGI378709, SGI378708, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90830[] = {SGI378710, SGI378711, SGI378712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90831[] = {SGI378714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90832[] = {SGI378715, SGI378716, SGI0, SGI378716, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90833[] = {SGI378717, SGI378718, SGI0, SGI378718, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90834[] = {SGI378719, SGI378720, SGI378721, SGI0, SGI0, SGI0, SGI378722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378723, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90835[] = {SGI378724, SGI378725, SGI378726, SGI0, SGI378727, SGI0, SGI378728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378729, NULL};
const Segment * const SG90836[] = {SGI378730, SGI378731, SGI378732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90837[] = {SGI378734, SGI378735, SGI378736, SGI378736, SGI0, SGI0, SGI0, SGI378737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90838[] = {SGI378738, SGI378739, SGI378740, SGI0, SGI0, SGI0, SGI378741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378742, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90839[] = {SGI378743, SGI378744, SGI378745, SGI378746, SGI0, SGI378747, SGI378748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378749, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90840[] = {SGI378750, SGI378751, SGI378752, SGI0, SGI378753, SGI0, SGI378754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378755, NULL};
const Segment * const SG90841[] = {SGI378756, SGI378757, SGI378758, SGI378758, SGI378759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90842[] = {SGI378760, SGI378761, SGI378762, SGI378762, SGI378763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90843[] = {SGI378764, SGI378765, SGI378766, SGI0, SGI378767, SGI0, SGI378768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90844[] = {SGI378769, SGI378770, SGI378771, SGI0, SGI378772, SGI0, SGI378773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90845[] = {SGI378774, SGI378775, SGI378776, SGI0, SGI378777, SGI0, SGI378778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90846[] = {SGI378779, SGI378780, SGI378781, SGI0, SGI378782, SGI0, SGI378783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90847[] = {SGI378784, SGI378785, SGI378786, SGI0, SGI378787, SGI0, SGI378788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90848[] = {SGI378789, SGI378790, SGI378791, SGI378792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90849[] = {SGI378794, SGI378795, SGI378796, SGI0, SGI378797, SGI0, SGI378798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90850[] = {SGI378799, SGI378800, SGI378801, SGI0, SGI378802, SGI0, SGI378803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90851[] = {SGI378804, SGI378805, SGI378806, SGI0, SGI378807, SGI0, SGI378808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90852[] = {SGI378809, SGI378810, SGI378811, SGI0, SGI378812, SGI0, SGI378813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90853[] = {SGI378814, SGI378815, SGI378816, SGI0, SGI378817, SGI0, SGI378818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90854[] = {SGI378819, SGI378820, SGI378821, SGI378822, SGI378820, SGI378823, SGI378824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90855[] = {SGI378825, SGI378826, SGI378827, SGI378828, SGI378826, SGI378829, SGI378830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90856[] = {SGI378831, SGI378832, SGI378833, SGI378834, SGI378832, SGI378835, SGI378836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90857[] = {SGI378837, SGI378838, SGI378839, SGI378840, SGI378838, SGI378841, SGI378842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90858[] = {SGI378843, SGI378844, SGI378845, SGI378846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378847, SGI0, NULL};
const Segment * const SG90859[] = {SGI0, SGI378848, SGI378849, SGI378850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378851, SGI0, NULL};
const Segment * const SG90860[] = {SGI0, SGI378852, SGI378853, SGI378854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378855, SGI0, NULL};
const Segment * const SG90861[] = {SGI378856, SGI378857, SGI378858, SGI378858, SGI378857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90862[] = {SGI378859, SGI378860, SGI378861, SGI378862, SGI378860, SGI378863, SGI378864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90863[] = {SGI378865, SGI378866, SGI378867, SGI0, SGI378868, SGI0, SGI378869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378870, NULL};
const Segment * const SG90864[] = {SGI378871, SGI378872, SGI378873, SGI378872, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90865[] = {SGI378874, SGI378875, SGI378876, SGI378875, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90866[] = {SGI378877, SGI378878, SGI378879, SGI378878, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90867[] = {SGI378880, SGI378881, SGI378882, SGI378881, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90868[] = {SGI378883, SGI378884, SGI378885, SGI378884, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90869[] = {SGI378886, SGI378887, SGI378888, SGI378889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378890, SGI0, NULL};
const Segment * const SG90870[] = {SGI378891, SGI378892, SGI378893, SGI378894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378895, SGI0, NULL};
const Segment * const SG90871[] = {SGI378896, SGI378897, SGI378898, SGI378899, SGI378897, SGI378900, SGI378901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90872[] = {SGI378902, SGI378903, SGI378904, SGI378905, SGI378903, SGI378906, SGI378907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90873[] = {SGI378908, SGI378909, SGI378910, SGI378911, SGI378909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90874[] = {SGI378912, SGI378913, SGI378914, SGI378915, SGI378913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90875[] = {SGI378916, SGI378917, SGI378918, SGI378918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90876[] = {SGI378920, SGI378921, SGI378922, SGI378922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90877[] = {SGI378924, SGI378925, SGI378926, SGI378927, SGI378925, SGI378928, SGI378929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90878[] = {SGI378930, SGI378931, SGI378932, SGI378933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90879[] = {SGI378935, SGI378936, SGI378937, SGI0, SGI0, SGI0, SGI0, SGI378938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90880[] = {SGI378939, SGI378940, SGI378941, SGI378940, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90881[] = {SGI0, SGI378942, SGI378943, SGI378944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378945, SGI0, NULL};
const Segment * const SG90882[] = {SGI0, SGI378946, SGI378947, SGI378948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378949, SGI0, NULL};
const Segment * const SG90883[] = {SGI378950, SGI378951, SGI378952, SGI378953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378954, SGI0, NULL};
const Segment * const SG90884[] = {SGI378955, SGI378956, SGI378957, SGI378958, SGI378959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90885[] = {SGI378960, SGI378961, SGI378962, SGI378963, SGI378964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90886[] = {SGI378965, SGI378966, SGI378967, SGI378968, SGI378969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90887[] = {SGI378970, SGI378971, SGI378972, SGI378973, SGI378974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90888[] = {SGI378975, SGI378976, SGI378977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI378978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90889[] = {SGI378979, SGI378980, SGI378981, SGI0, SGI0, SGI0, SGI0, SGI378982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90890[] = {SGI378983, SGI378984, SGI378985, SGI378986, SGI378984, SGI378987, SGI378988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90891[] = {SGI378989, SGI378990, SGI378991, SGI378992, SGI378990, SGI378993, SGI378994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90892[] = {SGI378995, SGI378996, SGI378997, SGI378996, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90893[] = {SGI378998, SGI378999, SGI379000, SGI378999, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90894[] = {SGI379001, SGI379002, SGI379003, SGI379002, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90895[] = {SGI379004, SGI379005, SGI379006, SGI379005, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90896[] = {SGI379007, SGI379008, SGI379009, SGI379008, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90897[] = {SGI379010, SGI379011, SGI379012, SGI379011, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90898[] = {SGI379013, SGI379014, SGI379015, SGI379014, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90899[] = {SGI379016, SGI379017, SGI379018, SGI379017, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90900[] = {SGI379019, SGI379020, SGI379021, SGI379020, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90901[] = {SGI379022, SGI379023, SGI379024, SGI379023, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90902[] = {SGI379025, SGI379026, SGI379027, SGI379026, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90903[] = {SGI379028, SGI379029, SGI379030, SGI379029, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90904[] = {SGI379031, SGI379032, SGI379033, SGI0, SGI379032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90905[] = {SGI379034, SGI379035, SGI379036, SGI379037, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90906[] = {SGI379038, SGI379039, SGI379040, SGI379041, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90907[] = {SGI379042, SGI379043, SGI379044, SGI379045, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90908[] = {SGI379046, SGI379047, SGI379048, SGI0, SGI0, SGI0, SGI379049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90909[] = {SGI379050, SGI379051, SGI379052, SGI379053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379054, SGI0, NULL};
const Segment * const SG90910[] = {SGI379055, SGI379056, SGI379057, SGI379058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379059, SGI0, NULL};
const Segment * const SG90911[] = {SGI379060, SGI379061, SGI379062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90912[] = {SGI379064, SGI379065, SGI379066, SGI379066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90913[] = {SGI379068, SGI379069, SGI379070, SGI379071, SGI379069, SGI379072, SGI379073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90914[] = {SGI379074, SGI379075, SGI379076, SGI379077, SGI379075, SGI379078, SGI379079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90915[] = {SGI379080, SGI379081, SGI379082, SGI379083, SGI379081, SGI379084, SGI379085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90916[] = {SGI379086, SGI379087, SGI0, SGI379087, SGI379088, SGI0, SGI0, NULL};
const Segment * const SG90917[] = {SGI379089, SGI379090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379091, SGI0, NULL};
const Segment * const SG90918[] = {SGI379092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379093, SGI0, NULL};
const Segment * const SG90919[] = {SGI379094, SGI379095, SGI379096, SGI0, SGI379097, SGI0, SGI379098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90920[] = {SGI379100, SGI379101, SGI379102, SGI0, SGI379103, SGI0, SGI379104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90921[] = {SGI379106, SGI379107, SGI379108, SGI0, SGI379109, SGI0, SGI379110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90922[] = {SGI379112, SGI379113, SGI379114, SGI0, SGI379115, SGI0, SGI379116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90923[] = {SGI379117, SGI379118, SGI0, SGI379118, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90924[] = {SGI379119, SGI379120, SGI0, SGI379120, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90925[] = {SGI379121, SGI379122, SGI0, SGI379122, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90926[] = {SGI379123, SGI379124, SGI0, SGI379124, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90927[] = {SGI379125, SGI379126, SGI0, SGI379126, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90928[] = {SGI379127, SGI379128, SGI0, SGI379128, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90929[] = {SGI379129, SGI379130, SGI0, SGI379130, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90930[] = {SGI379131, SGI379132, SGI0, SGI379132, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90931[] = {SGI379133, SGI379134, SGI0, SGI379134, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90932[] = {SGI379135, SGI379136, SGI0, SGI379136, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90933[] = {SGI379137, SGI379138, SGI0, SGI379138, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90934[] = {SGI379139, SGI379140, SGI0, SGI379140, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90935[] = {SGI379141, SGI379142, SGI0, SGI379142, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90936[] = {SGI379143, SGI379144, SGI0, SGI379144, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90937[] = {SGI379145, SGI379146, SGI0, SGI379146, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90938[] = {SGI379147, SGI379148, SGI0, SGI379148, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90939[] = {SGI379149, SGI379150, SGI0, SGI379150, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90940[] = {SGI379151, SGI379152, SGI0, SGI379152, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90941[] = {SGI379153, SGI379154, SGI0, SGI379154, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90942[] = {SGI379155, SGI379156, SGI0, SGI379156, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90943[] = {SGI379157, SGI379158, SGI379159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90944[] = {SGI379161, SGI379162, SGI379163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90945[] = {SGI379165, SGI379166, SGI379167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90946[] = {SGI379169, SGI379170, SGI379171, SGI379172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90947[] = {SGI379173, SGI379174, SGI379175, SGI379176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90948[] = {SGI379177, SGI379178, SGI379179, SGI379179, SGI379180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90949[] = {SGI379181, SGI379182, SGI379183, SGI0, SGI379184, SGI0, SGI379185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90950[] = {SGI379187, SGI379188, SGI379189, SGI0, SGI379190, SGI0, SGI379191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90951[] = {SGI379193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379194, SGI0, NULL};
const Segment * const SG90952[] = {SGI379195, SGI379196, SGI379197, SGI379198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379199, SGI0, SGI0, NULL};
const Segment * const SG90953[] = {SGI379200, SGI379201, SGI379202, SGI379203, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90954[] = {SGI379204, SGI379205, SGI379206, SGI379207, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90955[] = {SGI379208, SGI379209, SGI0, SGI379210, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90956[] = {SGI379211, SGI379212, SGI0, SGI379212, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90957[] = {SGI379213, SGI379214, SGI379215, SGI379216, SGI0, SGI0, SGI379217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379218, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90958[] = {SGI379219, SGI379220, SGI379221, SGI379221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90959[] = {SGI379223, SGI379224, SGI379225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90960[] = {SGI379227, SGI379228, SGI379229, SGI379229, SGI0, SGI0, SGI0, SGI379230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90961[] = {SGI379231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379232, SGI0, NULL};
const Segment * const SG90962[] = {SGI379233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379234, SGI0, NULL};
const Segment * const SG90963[] = {SGI379235, SGI379236, SGI379237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90964[] = {SGI379239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379240, SGI0, NULL};
const Segment * const SG90965[] = {SGI379241, SGI379242, SGI0, SGI379242, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90966[] = {SGI379243, SGI379244, SGI0, SGI379244, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90967[] = {SGI379245, SGI379246, SGI0, SGI379246, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90968[] = {SGI379247, SGI379248, SGI0, SGI379248, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90969[] = {SGI379249, SGI379250, SGI0, SGI379250, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90970[] = {SGI379251, SGI379252, SGI0, SGI379252, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90971[] = {SGI379253, SGI379254, SGI0, SGI379254, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90972[] = {SGI379255, SGI379256, SGI0, SGI379256, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90973[] = {SGI379257, SGI379258, SGI0, SGI379258, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90974[] = {SGI379259, SGI379260, SGI0, SGI379260, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90975[] = {SGI379261, SGI379262, SGI0, SGI379262, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90976[] = {SGI379263, SGI379264, SGI0, SGI379264, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90977[] = {SGI379265, SGI379266, SGI0, SGI379266, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90978[] = {SGI379267, SGI379268, SGI0, SGI379268, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90979[] = {SGI379269, SGI379270, SGI0, SGI379270, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90980[] = {SGI379271, SGI379272, SGI0, SGI379272, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90981[] = {SGI379273, SGI379274, SGI0, SGI379274, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90982[] = {SGI379275, SGI379276, SGI0, SGI379276, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90983[] = {SGI379277, SGI379278, SGI0, SGI379278, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90984[] = {SGI379279, SGI379280, SGI0, SGI379280, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90985[] = {SGI379281, SGI379282, SGI0, SGI379282, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90986[] = {SGI379283, SGI379284, SGI0, SGI379284, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90987[] = {SGI379285, SGI379286, SGI0, SGI379286, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90988[] = {SGI379287, SGI379288, SGI0, SGI379288, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90989[] = {SGI379289, SGI379290, SGI0, SGI379290, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90990[] = {SGI379291, SGI379292, SGI0, SGI379292, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90991[] = {SGI379293, SGI379294, SGI0, SGI379294, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90992[] = {SGI379295, SGI379296, SGI379297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90993[] = {SGI379299, SGI379300, SGI379301, SGI379301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90994[] = {SGI379303, SGI379304, SGI379305, SGI379305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90995[] = {SGI0, SGI379307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90996[] = {SGI379308, SGI379309, SGI379310, SGI379310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90997[] = {SGI379312, SGI379313, SGI379314, SGI0, SGI379315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90998[] = {SGI379316, SGI379317, SGI379318, SGI0, SGI379319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG90999[] = {SGI379320, SGI379321, SGI379322, SGI379323, SGI379321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91000[] = {SGI379324, SGI379325, SGI379326, SGI0, SGI0, SGI0, SGI379327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379328, SGI0, SGI0, NULL};
const Segment * const SG91001[] = {SGI379329, SGI379330, SGI379331, SGI0, SGI0, SGI0, SGI379332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379333, SGI0, SGI0, NULL};
const Segment * const SG91002[] = {SGI379334, SGI379335, SGI379336, SGI379337, SGI0, SGI379338, SGI379339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379340, SGI0, SGI0, NULL};
const Segment * const SG91003[] = {SGI379341, SGI379342, SGI379343, SGI0, SGI0, SGI0, SGI379344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379345, SGI0, SGI0, NULL};
const Segment * const SG91004[] = {SGI379346, SGI379347, SGI379348, SGI0, SGI0, SGI0, SGI379349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379350, SGI0, SGI0, NULL};
const Segment * const SG91005[] = {SGI379351, SGI379352, SGI379353, SGI0, SGI0, SGI0, SGI379354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379355, SGI0, SGI0, NULL};
const Segment * const SG91006[] = {SGI379356, SGI379357, SGI379358, SGI379359, SGI0, SGI379360, SGI379361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379362, SGI0, SGI0, NULL};
const Segment * const SG91007[] = {SGI379363, SGI379364, SGI379365, SGI379366, SGI0, SGI379367, SGI379368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379369, SGI0, SGI0, NULL};
const Segment * const SG91008[] = {SGI379370, SGI379371, SGI379372, SGI0, SGI0, SGI0, SGI379373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379374, SGI0, SGI0, NULL};
const Segment * const SG91009[] = {SGI379375, SGI379376, SGI379377, SGI0, SGI0, SGI0, SGI379378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379379, SGI0, SGI0, NULL};
const Segment * const SG91010[] = {SGI379380, SGI379381, SGI379382, SGI379383, SGI0, SGI379384, SGI379385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379386, NULL};
const Segment * const SG91011[] = {SGI379387, SGI379388, SGI0, SGI379388, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91012[] = {SGI379389, SGI379390, SGI379391, SGI0, SGI379392, SGI0, SGI379393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91013[] = {SGI379394, SGI379395, SGI379396, SGI379397, SGI379398, SGI379399, SGI379400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91014[] = {SGI379401, SGI379402, SGI379403, SGI0, SGI379404, SGI0, SGI379405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91015[] = {SGI379406, SGI379407, SGI379408, SGI0, SGI0, SGI0, SGI379409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379410, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91016[] = {SGI379411, SGI379412, SGI379413, SGI379414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91017[] = {SGI379416, SGI379417, SGI379418, SGI379419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379420, SGI0, NULL};
const Segment * const SG91018[] = {SGI379421, SGI379422, SGI379423, SGI379424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379425, SGI0, NULL};
const Segment * const SG91019[] = {SGI379426, SGI379427, SGI379428, SGI379429, SGI0, SGI0, SGI0, SGI0, SGI379430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91020[] = {SGI379431, SGI379432, SGI379433, SGI0, SGI379432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91021[] = {SGI379434, SGI379435, SGI379436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91022[] = {SGI379438, SGI379439, SGI379440, SGI0, SGI0, SGI0, SGI379441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379442, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91023[] = {SGI379443, SGI379444, SGI0, SGI379444, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91024[] = {SGI379445, SGI379446, SGI0, SGI379446, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91025[] = {SGI379447, SGI379448, SGI0, SGI379448, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91026[] = {SGI379449, SGI379450, SGI0, SGI379450, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91027[] = {SGI379451, SGI379452, SGI0, SGI379452, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91028[] = {SGI379453, SGI379454, SGI0, SGI379454, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91029[] = {SGI379455, SGI379456, SGI0, SGI379456, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91030[] = {SGI379457, SGI379458, SGI0, SGI379458, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91031[] = {SGI379459, SGI379460, SGI379461, SGI379462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379463, SGI0, NULL};
const Segment * const SG91032[] = {SGI379464, SGI379465, SGI379466, SGI379467, SGI0, SGI379468, SGI379469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379470, NULL};
const Segment * const SG91033[] = {SGI379471, SGI379472, SGI379473, SGI379473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379474, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91034[] = {SGI379475, SGI379476, SGI379477, SGI0, SGI379478, SGI0, SGI379479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91035[] = {SGI379481, SGI379482, SGI379483, SGI0, SGI379484, SGI0, SGI379485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91036[] = {SGI379487, SGI379488, SGI379489, SGI379490, SGI379488, SGI379491, SGI379492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91037[] = {SGI379493, SGI379494, SGI379495, SGI379496, SGI379494, SGI379497, SGI379498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91038[] = {SGI379499, SGI379500, SGI379501, SGI379502, SGI379500, SGI379503, SGI379504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91039[] = {SGI379505, SGI379506, SGI379507, SGI379508, SGI379506, SGI379509, SGI379510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91040[] = {SGI379511, SGI379512, SGI379513, SGI379514, SGI379512, SGI379515, SGI379516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91041[] = {SGI379517, SGI379518, SGI379519, SGI379520, SGI379518, SGI379521, SGI379522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91042[] = {SGI379523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379524, SGI0, NULL};
const Segment * const SG91043[] = {SGI379525, SGI379526, SGI379527, SGI0, SGI379528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91044[] = {SGI379529, SGI379530, SGI379531, SGI0, SGI379532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91045[] = {SGI379533, SGI379534, SGI379535, SGI379536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379537, SGI0, NULL};
const Segment * const SG91046[] = {SGI0, SGI379538, SGI379539, SGI379539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91047[] = {SGI379540, SGI379541, SGI379542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91048[] = {SGI379544, SGI379545, SGI379546, SGI379547, SGI379548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91049[] = {SGI379549, SGI379550, SGI379551, SGI379552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379553, SGI0, NULL};
const Segment * const SG91050[] = {SGI379554, SGI379555, SGI379556, SGI379557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379558, SGI0, NULL};
const Segment * const SG91051[] = {SGI379559, SGI379560, SGI379561, SGI0, SGI379562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91052[] = {SGI379563, SGI379564, SGI379565, SGI0, SGI379566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91053[] = {SGI379567, SGI379568, SGI379569, SGI379569, SGI379570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91054[] = {SGI379571, SGI379572, SGI379573, SGI379572, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91055[] = {SGI379574, SGI379575, SGI379576, SGI379577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379578, SGI0, NULL};
const Segment * const SG91056[] = {SGI379579, SGI379580, SGI379581, SGI0, SGI0, SGI0, SGI379582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379583, SGI0, SGI0, NULL};
const Segment * const SG91057[] = {SGI379584, SGI379585, SGI379586, SGI0, SGI0, SGI0, SGI379587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379588, SGI0, SGI0, NULL};
const Segment * const SG91058[] = {SGI379589, SGI379590, SGI379591, SGI0, SGI0, SGI0, SGI379592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379593, SGI0, SGI0, NULL};
const Segment * const SG91059[] = {SGI0, SGI379594, SGI379595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91060[] = {SGI0, SGI379597, SGI379598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91061[] = {SGI0, SGI379600, SGI379601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91062[] = {SGI0, SGI379603, SGI379604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91063[] = {SGI379606, SGI379607, SGI379608, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91064[] = {SGI379610, SGI379611, SGI379612, SGI379612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91065[] = {SGI379614, SGI379615, SGI379616, SGI379617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91066[] = {SGI379619, SGI379620, SGI379621, SGI379622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379623, SGI0, NULL};
const Segment * const SG91067[] = {SGI379624, SGI379625, SGI379626, SGI0, SGI379627, SGI0, SGI379628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91068[] = {SGI379629, SGI379630, SGI379631, SGI379632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91069[] = {SGI379634, SGI379635, SGI379636, SGI0, SGI379637, SGI0, SGI379638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91070[] = {SGI379639, SGI379640, SGI379641, SGI0, SGI379642, SGI0, SGI379643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91071[] = {SGI379644, SGI379645, SGI379646, SGI0, SGI379647, SGI0, SGI379648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91072[] = {SGI379650, SGI379651, SGI379652, SGI0, SGI379653, SGI0, SGI379654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379655, NULL};
const Segment * const SG91073[] = {SGI379656, SGI379657, SGI379658, SGI379659, SGI379660, SGI379661, SGI379662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379663, NULL};
const Segment * const SG91074[] = {SGI379664, SGI379665, SGI379666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91075[] = {SGI379668, SGI379669, SGI379670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91076[] = {SGI379672, SGI379673, SGI379674, SGI379675, SGI0, SGI0, SGI0, SGI0, SGI379676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91077[] = {SGI379677, SGI379678, SGI379679, SGI0, SGI379680, SGI0, SGI379681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91078[] = {SGI379683, SGI379684, SGI379685, SGI379684, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91079[] = {SGI379686, SGI379687, SGI0, SGI379687, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91080[] = {SGI379688, SGI379689, SGI0, SGI379689, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91081[] = {SGI379690, SGI379691, SGI0, SGI379692, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91082[] = {SGI379693, SGI379694, SGI0, SGI379694, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91083[] = {SGI379695, SGI379696, SGI379697, SGI0, SGI379698, SGI0, SGI379699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91084[] = {SGI379701, SGI379702, SGI379703, SGI0, SGI379704, SGI0, SGI379705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91085[] = {SGI379707, SGI379708, SGI379709, SGI379710, SGI379711, SGI379712, SGI379713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91086[] = {SGI379714, SGI379715, SGI379716, SGI379717, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91087[] = {SGI379718, SGI379719, SGI379720, SGI379721, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91088[] = {SGI379722, SGI379723, SGI379724, SGI379725, SGI379723, SGI379726, SGI379727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91089[] = {SGI379728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379729, SGI0, NULL};
const Segment * const SG91090[] = {SGI379730, SGI379731, SGI379732, SGI379733, SGI0, SGI0, SGI0, SGI0, SGI379734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91091[] = {SGI379735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379736, SGI0, NULL};
const Segment * const SG91092[] = {SGI379737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379738, SGI0, NULL};
const Segment * const SG91093[] = {SGI379739, SGI379740, SGI379741, SGI379742, SGI379743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91094[] = {SGI379744, SGI379745, SGI379746, SGI379747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91095[] = {SGI379749, SGI379750, SGI379751, SGI379752, SGI379753, SGI379754, SGI379755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379756, NULL};
const Segment * const SG91096[] = {SGI379757, SGI379758, SGI379759, SGI379760, SGI379761, SGI379762, SGI379763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379764, NULL};
const Segment * const SG91097[] = {SGI379765, SGI379766, SGI379767, SGI0, SGI379768, SGI0, SGI379769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379770, NULL};
const Segment * const SG91098[] = {SGI379771, SGI379772, SGI379773, SGI379774, SGI379775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91099[] = {SGI379776, SGI379777, SGI379778, SGI379779, SGI0, SGI0, SGI0, SGI0, SGI379780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91100[] = {SGI379781, SGI379782, SGI379783, SGI0, SGI0, SGI0, SGI379784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91101[] = {SGI379785, SGI379786, SGI379787, SGI379786, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91102[] = {SGI379788, SGI379789, SGI379790, SGI379790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91103[] = {SGI379792, SGI379793, SGI379794, SGI379795, SGI0, SGI0, SGI0, SGI0, SGI379796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91104[] = {SGI379797, SGI379798, SGI379799, SGI379800, SGI0, SGI0, SGI0, SGI0, SGI379801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91105[] = {SGI379802, SGI379803, SGI379804, SGI379805, SGI379806, SGI379807, SGI379808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91106[] = {SGI379810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379811, SGI0, NULL};
const Segment * const SG91107[] = {SGI379812, SGI379813, SGI379814, SGI379815, SGI379813, SGI379816, SGI379817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91108[] = {SGI379818, SGI379819, SGI379820, SGI379820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91109[] = {SGI379822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379823, SGI0, NULL};
const Segment * const SG91110[] = {SGI379824, SGI379825, SGI379826, SGI0, SGI379827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91111[] = {SGI0, SGI379828, SGI379829, SGI379830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379831, SGI0, NULL};
const Segment * const SG91112[] = {SGI379832, SGI379833, SGI379834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91113[] = {SGI379836, SGI379837, SGI379838, SGI0, SGI379839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91114[] = {SGI379840, SGI379841, SGI379842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91115[] = {SGI379844, SGI379845, SGI379846, SGI379846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91116[] = {SGI379848, SGI379849, SGI379850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379851, SGI0, NULL};
const Segment * const SG91117[] = {SGI379852, SGI379853, SGI379854, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91118[] = {SGI379855, SGI379856, SGI379857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379858, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91119[] = {SGI379859, SGI379860, SGI379861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91120[] = {SGI379863, SGI379864, SGI379865, SGI0, SGI379866, SGI0, SGI379867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91121[] = {SGI379869, SGI379870, SGI0, SGI379870, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91122[] = {SGI0, SGI379871, SGI0, SGI379871, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91123[] = {SGI379872, SGI379873, SGI379874, SGI379875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91124[] = {SGI379877, SGI379878, SGI379879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91125[] = {SGI379881, SGI379882, SGI379883, SGI0, SGI0, SGI0, SGI379884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379885, SGI0, SGI0, NULL};
const Segment * const SG91126[] = {SGI379886, SGI379887, SGI379888, SGI0, SGI0, SGI0, SGI379889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379890, SGI0, SGI0, NULL};
const Segment * const SG91127[] = {SGI379891, SGI379892, SGI379893, SGI0, SGI0, SGI0, SGI379894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379895, SGI0, SGI0, NULL};
const Segment * const SG91128[] = {SGI379896, SGI379897, SGI379898, SGI379899, SGI0, SGI0, SGI0, SGI0, SGI379900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91129[] = {SGI379901, SGI379902, SGI0, SGI379902, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91130[] = {SGI379903, SGI379904, SGI0, SGI379904, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91131[] = {SGI379905, SGI379906, SGI379907, SGI379908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91132[] = {SGI379909, SGI379910, SGI379911, SGI379910, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91133[] = {SGI379912, SGI379913, SGI0, SGI379913, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91134[] = {SGI379914, SGI379915, SGI0, SGI379915, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91135[] = {SGI379916, SGI379917, SGI0, SGI379917, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91136[] = {SGI379918, SGI379919, SGI0, SGI379919, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91137[] = {SGI379920, SGI379921, SGI0, SGI379921, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91138[] = {SGI379922, SGI379923, SGI0, SGI379923, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91139[] = {SGI379924, SGI379925, SGI379926, SGI379925, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91140[] = {SGI379927, SGI379928, SGI379929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91141[] = {SGI379931, SGI379932, SGI379933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91142[] = {SGI379935, SGI379936, SGI379937, SGI379938, SGI379936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91143[] = {SGI379939, SGI379940, SGI379941, SGI379942, SGI379940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91144[] = {SGI379943, SGI379944, SGI379945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91145[] = {SGI379946, SGI379947, SGI379948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91146[] = {SGI379949, SGI379950, SGI379951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91147[] = {SGI379952, SGI379953, SGI379954, SGI379954, SGI0, SGI0, SGI0, SGI379955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91148[] = {SGI379956, SGI379957, SGI379958, SGI379959, SGI379960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91149[] = {SGI379961, SGI379962, SGI379963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91150[] = {SGI379965, SGI379966, SGI379967, SGI379968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91151[] = {SGI379970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379971, SGI0, NULL};
const Segment * const SG91152[] = {SGI379972, SGI379973, SGI379974, SGI0, SGI379975, SGI0, SGI379976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91153[] = {SGI379977, SGI379978, SGI379979, SGI0, SGI379980, SGI0, SGI379981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91154[] = {SGI379982, SGI379983, SGI379984, SGI0, SGI379985, SGI0, SGI379986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91155[] = {SGI379988, SGI379989, SGI379990, SGI0, SGI379991, SGI0, SGI379992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI379993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91156[] = {SGI379994, SGI379995, SGI379996, SGI379997, SGI0, SGI0, SGI0, SGI0, SGI379998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91157[] = {SGI379999, SGI380000, SGI380001, SGI380002, SGI0, SGI0, SGI380003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380004, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91158[] = {SGI380005, SGI380006, SGI380007, SGI0, SGI0, SGI0, SGI380008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91159[] = {SGI380009, SGI380010, SGI380011, SGI380012, SGI380013, SGI380014, SGI380015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91160[] = {SGI380016, SGI380017, SGI380018, SGI380019, SGI0, SGI0, SGI0, SGI0, SGI380020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91161[] = {SGI380021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380022, SGI0, NULL};
const Segment * const SG91162[] = {SGI380023, SGI380024, SGI380025, SGI380026, SGI380027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91163[] = {SGI380028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380029, SGI0, NULL};
const Segment * const SG91164[] = {SGI380030, SGI380031, SGI380032, SGI380032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91165[] = {SGI380034, SGI380035, SGI380036, SGI380037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91166[] = {SGI380039, SGI380040, SGI380041, SGI0, SGI380042, SGI0, SGI380043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91167[] = {SGI380045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380046, SGI0, NULL};
const Segment * const SG91168[] = {SGI380047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380048, SGI0, NULL};
const Segment * const SG91169[] = {SGI380049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380050, SGI0, NULL};
const Segment * const SG91170[] = {SGI380051, SGI380052, SGI380053, SGI380054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91171[] = {SGI380056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91172[] = {SGI380057, SGI380058, SGI380059, SGI380059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91173[] = {SGI380061, SGI380062, SGI380063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91174[] = {SGI380065, SGI380066, SGI380067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91175[] = {SGI380069, SGI380070, SGI380071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91176[] = {SGI380073, SGI380074, SGI380075, SGI380076, SGI0, SGI380077, SGI380078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380079, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91177[] = {SGI380080, SGI380081, SGI380082, SGI380083, SGI0, SGI380084, SGI380085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380086, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91178[] = {SGI380087, SGI380088, SGI380089, SGI0, SGI0, SGI0, SGI380090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380091, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91179[] = {SGI380092, SGI380093, SGI380094, SGI380095, SGI380093, SGI380096, SGI380097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91180[] = {SGI380098, SGI380099, SGI380100, SGI380101, SGI380099, SGI380102, SGI380103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91181[] = {SGI380104, SGI380105, SGI380106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91182[] = {SGI380108, SGI380109, SGI380110, SGI380111, SGI380112, SGI380113, SGI380114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91183[] = {SGI380115, SGI380116, SGI380117, SGI380118, SGI380119, SGI380120, SGI380121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91184[] = {SGI380122, SGI380123, SGI380124, SGI380125, SGI380126, SGI380127, SGI380128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91185[] = {SGI380129, SGI380130, SGI0, SGI380130, SGI380131, SGI0, SGI0, NULL};
const Segment * const SG91186[] = {SGI380132, SGI380133, SGI0, SGI380133, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91187[] = {SGI380134, SGI380135, SGI380136, SGI380137, SGI0, SGI0, SGI0, SGI0, SGI380138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91188[] = {SGI380139, SGI380140, SGI380141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91189[] = {SGI380143, SGI380144, SGI380145, SGI0, SGI380146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91190[] = {SGI380147, SGI380148, SGI380149, SGI380150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91191[] = {SGI380152, SGI380153, SGI380154, SGI380155, SGI380153, SGI380156, SGI380157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91192[] = {SGI380158, SGI380159, SGI0, SGI380159, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91193[] = {SGI380160, SGI380161, SGI0, SGI380161, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91194[] = {SGI380162, SGI380163, SGI380164, SGI380164, SGI0, SGI0, SGI0, SGI380165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91195[] = {SGI380166, SGI380167, SGI380168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91196[] = {SGI380169, SGI380170, SGI380171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91197[] = {SGI380173, SGI380174, SGI380175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91198[] = {SGI380177, SGI380178, SGI380179, SGI380180, SGI380178, SGI380181, SGI380182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91199[] = {SGI380183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380184, SGI0, NULL};
const Segment * const SG91200[] = {SGI380185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380186, SGI0, NULL};
const Segment * const SG91201[] = {SGI380187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380188, SGI0, NULL};
const Segment * const SG91202[] = {SGI380189, SGI380190, SGI380191, SGI380192, SGI380190, SGI380193, SGI380194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91203[] = {SGI380195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380196, SGI0, NULL};
const Segment * const SG91204[] = {SGI380197, SGI380198, SGI380199, SGI380198, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91205[] = {SGI380200, SGI380201, SGI380202, SGI380201, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91206[] = {SGI380203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380204, SGI0, NULL};
const Segment * const SG91207[] = {SGI380205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380206, SGI0, NULL};
const Segment * const SG91208[] = {SGI380207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380208, SGI0, NULL};
const Segment * const SG91209[] = {SGI380209, SGI380210, SGI380211, SGI380210, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91210[] = {SGI380212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380213, SGI0, NULL};
const Segment * const SG91211[] = {SGI380214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380215, SGI0, NULL};
const Segment * const SG91212[] = {SGI380216, SGI380217, SGI380218, SGI380217, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91213[] = {SGI380219, SGI380220, SGI380221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91214[] = {SGI380223, SGI380224, SGI380225, SGI0, SGI380226, SGI0, SGI380227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91215[] = {SGI380229, SGI380230, SGI380231, SGI380232, SGI0, SGI0, SGI0, SGI0, SGI380233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91216[] = {SGI380234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380235, SGI0, NULL};
const Segment * const SG91217[] = {SGI380236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380237, SGI0, NULL};
const Segment * const SG91218[] = {SGI380238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380239, SGI0, NULL};
const Segment * const SG91219[] = {SGI380240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380241, SGI0, NULL};
const Segment * const SG91220[] = {SGI380242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380243, SGI0, NULL};
const Segment * const SG91221[] = {SGI380244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380245, SGI0, NULL};
const Segment * const SG91222[] = {SGI380246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380247, SGI0, NULL};
const Segment * const SG91223[] = {SGI380248, SGI380249, SGI380250, SGI0, SGI0, SGI0, SGI380251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380252, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91224[] = {SGI380253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380254, SGI0, NULL};
const Segment * const SG91225[] = {SGI380255, SGI380256, SGI380257, SGI380256, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91226[] = {SGI380258, SGI380259, SGI380260, SGI0, SGI0, SGI0, SGI380261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380262, SGI0, SGI0, NULL};
const Segment * const SG91227[] = {SGI380263, SGI380264, SGI380265, SGI380266, SGI0, SGI380267, SGI380268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91228[] = {SGI380269, SGI380270, SGI380271, SGI380272, SGI0, SGI380273, SGI380274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91229[] = {SGI380275, SGI380276, SGI380277, SGI380278, SGI0, SGI380279, SGI380280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91230[] = {SGI380281, SGI380282, SGI380283, SGI380284, SGI0, SGI0, SGI0, SGI0, SGI380285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91231[] = {SGI380286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380287, SGI0, NULL};
const Segment * const SG91232[] = {SGI380288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380289, SGI0, NULL};
const Segment * const SG91233[] = {SGI380290, SGI380291, SGI380292, SGI0, SGI380293, SGI0, SGI380294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91234[] = {SGI380296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380297, SGI0, NULL};
const Segment * const SG91235[] = {SGI380298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380299, SGI0, NULL};
const Segment * const SG91236[] = {SGI380300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380301, SGI0, NULL};
const Segment * const SG91237[] = {SGI380302, SGI380303, SGI380304, SGI380304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91238[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380306, SGI0, NULL};
const Segment * const SG91239[] = {SGI380307, SGI380308, SGI380309, SGI380310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91240[] = {SGI380312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380313, SGI0, NULL};
const Segment * const SG91241[] = {SGI380314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380315, SGI0, NULL};
const Segment * const SG91242[] = {SGI380316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380317, SGI0, NULL};
const Segment * const SG91243[] = {SGI380318, SGI380319, SGI380320, SGI380321, SGI380322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91244[] = {SGI380323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380324, SGI0, NULL};
const Segment * const SG91245[] = {SGI380325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380326, SGI0, NULL};
const Segment * const SG91246[] = {SGI380327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380328, SGI0, NULL};
const Segment * const SG91247[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380329, SGI0, NULL};
const Segment * const SG91248[] = {SGI380330, SGI380331, SGI380332, SGI380333, SGI0, SGI0, SGI0, SGI0, SGI380334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91249[] = {SGI380335, SGI380336, SGI380337, SGI380338, SGI380339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91250[] = {SGI380340, SGI380341, SGI380342, SGI380343, SGI380344, SGI380345, SGI380346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91251[] = {SGI380347, SGI380348, SGI380349, SGI380350, SGI380351, SGI380352, SGI380353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91252[] = {SGI380354, SGI380355, SGI0, SGI380355, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91253[] = {SGI380356, SGI380357, SGI380358, SGI0, SGI380359, SGI0, SGI380360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91254[] = {SGI380361, SGI380362, SGI380363, SGI380364, SGI0, SGI0, SGI0, SGI0, SGI380365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91255[] = {SGI380366, SGI380367, SGI380368, SGI0, SGI380369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91256[] = {SGI380370, SGI380371, SGI380372, SGI380372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91257[] = {SGI380374, SGI380375, SGI380376, SGI0, SGI0, SGI0, SGI380377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380378, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91258[] = {SGI380379, SGI380380, SGI380381, SGI380381, SGI0, SGI0, SGI0, SGI380382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91259[] = {SGI380383, SGI380384, SGI380385, SGI380386, SGI380387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91260[] = {SGI380388, SGI380389, SGI380390, SGI380391, SGI380392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91261[] = {SGI380393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91262[] = {SGI380394, SGI380395, SGI380396, SGI0, SGI380395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91263[] = {SGI380397, SGI380398, SGI380399, SGI0, SGI380400, SGI0, SGI380401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91264[] = {SGI380403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380404, SGI0, NULL};
const Segment * const SG91265[] = {SGI380405, SGI380406, SGI380407, SGI0, SGI380408, SGI0, SGI380409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91266[] = {SGI380411, SGI380412, SGI0, SGI380412, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91267[] = {SGI380413, SGI380414, SGI0, SGI380414, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91268[] = {SGI380415, SGI380416, SGI0, SGI380416, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91269[] = {SGI380417, SGI380418, SGI380419, SGI380420, SGI0, SGI380421, SGI380422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91270[] = {SGI380423, SGI380424, SGI380425, SGI380426, SGI0, SGI380427, SGI380428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91271[] = {SGI380429, SGI380430, SGI380431, SGI380431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91272[] = {SGI380433, SGI380434, SGI380435, SGI0, SGI380436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91273[] = {SGI380437, SGI380438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380439, SGI0, NULL};
const Segment * const SG91274[] = {SGI380440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380441, SGI0, NULL};
const Segment * const SG91275[] = {SGI380442, SGI380443, SGI380444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91276[] = {SGI380446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380447, SGI0, NULL};
const Segment * const SG91277[] = {SGI380448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380449, SGI0, NULL};
const Segment * const SG91278[] = {SGI380450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380451, SGI0, NULL};
const Segment * const SG91279[] = {SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380452, SGI0, NULL};
const Segment * const SG91280[] = {SGI380453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380454, SGI0, NULL};
const Segment * const SG91281[] = {SGI380455, SGI380456, SGI380457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91282[] = {SGI380459, SGI380460, SGI380461, SGI380460, SGI0, SGI380462, SGI380463, NULL};
const Segment * const SG91283[] = {SGI380464, SGI380465, SGI380466, SGI380465, SGI0, SGI380467, SGI380468, NULL};
const Segment * const SG91284[] = {SGI380469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380470, SGI0, NULL};
const Segment * const SG91285[] = {SGI380471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380472, SGI0, NULL};
const Segment * const SG91286[] = {SGI380473, SGI380474, SGI380475, SGI380475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91287[] = {SGI380477, SGI380478, SGI0, SGI380478, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91288[] = {SGI380479, SGI380480, SGI0, SGI380480, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91289[] = {SGI380481, SGI380482, SGI380483, SGI380484, SGI380482, SGI380485, SGI380486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91290[] = {SGI380487, SGI380488, SGI380489, SGI380488, SGI0, SGI380490, SGI380491, NULL};
const Segment * const SG91291[] = {SGI380492, SGI380493, SGI380494, SGI380493, SGI0, SGI380495, SGI380496, NULL};
const Segment * const SG91292[] = {SGI380497, SGI380498, SGI380499, SGI380500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91293[] = {SGI380502, SGI380503, SGI380504, SGI0, SGI0, SGI0, SGI380505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380506, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91294[] = {SGI380507, SGI380508, SGI380509, SGI0, SGI380510, SGI0, SGI380511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91295[] = {SGI380513, SGI380514, SGI380515, SGI380514, SGI0, SGI380516, SGI380517, NULL};
const Segment * const SG91296[] = {SGI380518, SGI380519, SGI380520, SGI380519, SGI0, SGI380521, SGI380522, NULL};
const Segment * const SG91297[] = {SGI380523, SGI380524, SGI380525, SGI380524, SGI0, SGI380526, SGI380527, NULL};
const Segment * const SG91298[] = {SGI380528, SGI380529, SGI380530, SGI380529, SGI0, SGI380531, SGI380532, NULL};
const Segment * const SG91299[] = {SGI380533, SGI380534, SGI0, SGI380534, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91300[] = {SGI380535, SGI380536, SGI0, SGI380536, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91301[] = {SGI380537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380538, SGI0, NULL};
const Segment * const SG91302[] = {SGI380539, SGI380540, SGI380541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91303[] = {SGI380543, SGI380544, SGI380545, SGI380546, SGI380547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91304[] = {SGI380548, SGI380549, SGI380550, SGI380549, SGI0, SGI380551, SGI380552, NULL};
const Segment * const SG91305[] = {SGI380553, SGI380554, SGI0, SGI380554, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91306[] = {SGI380555, SGI380556, SGI0, SGI380556, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91307[] = {SGI380557, SGI380558, SGI380559, SGI380558, SGI0, SGI380560, SGI380561, NULL};
const Segment * const SG91308[] = {SGI380562, SGI380563, SGI380564, SGI380563, SGI0, SGI380565, SGI380566, NULL};
const Segment * const SG91309[] = {SGI380567, SGI380568, SGI380569, SGI380568, SGI0, SGI380570, SGI380571, NULL};
const Segment * const SG91310[] = {SGI380572, SGI380573, SGI380574, SGI380573, SGI0, SGI380575, SGI380576, NULL};
const Segment * const SG91311[] = {SGI380577, SGI380578, SGI0, SGI380578, SGI0, SGI380579, SGI380580, NULL};
const Segment * const SG91312[] = {SGI380581, SGI380582, SGI0, SGI380582, SGI0, SGI380583, SGI380584, NULL};
const Segment * const SG91313[] = {SGI380585, SGI380586, SGI0, SGI380586, SGI0, SGI380587, SGI380588, NULL};
const Segment * const SG91314[] = {SGI380589, SGI380590, SGI0, SGI380590, SGI0, SGI380591, SGI380592, NULL};
const Segment * const SG91315[] = {SGI380593, SGI380594, SGI0, SGI380594, SGI0, SGI380595, SGI380596, NULL};
const Segment * const SG91316[] = {SGI380597, SGI380598, SGI0, SGI380598, SGI0, SGI380599, SGI380600, NULL};
const Segment * const SG91317[] = {SGI380601, SGI380602, SGI380603, SGI380602, SGI0, SGI380604, SGI380605, NULL};
const Segment * const SG91318[] = {SGI380606, SGI380607, SGI380608, SGI380607, SGI0, SGI380609, SGI380610, NULL};
const Segment * const SG91319[] = {SGI380611, SGI380612, SGI380613, SGI380612, SGI0, SGI380614, SGI380615, NULL};
const Segment * const SG91320[] = {SGI380616, SGI380617, SGI380618, SGI380617, SGI0, SGI380619, SGI380620, NULL};
const Segment * const SG91321[] = {SGI380621, SGI380622, SGI380623, SGI380622, SGI0, SGI380624, SGI380625, NULL};
const Segment * const SG91322[] = {SGI380626, SGI380627, SGI380628, SGI380627, SGI0, SGI380629, SGI380630, NULL};
const Segment * const SG91323[] = {SGI380631, SGI380632, SGI380633, SGI380634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91324[] = {SGI380636, SGI380637, SGI380638, SGI380639, SGI0, SGI0, SGI0, SGI0, SGI380640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91325[] = {SGI380641, SGI380642, SGI380643, SGI0, SGI380644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91326[] = {SGI380645, SGI380646, SGI380647, SGI380648, SGI0, SGI0, SGI0, SGI0, SGI380649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91327[] = {SGI380650, SGI380651, SGI0, SGI380651, SGI0, SGI380652, SGI380653, NULL};
const Segment * const SG91328[] = {SGI380654, SGI380655, SGI0, SGI380655, SGI0, SGI380656, SGI380657, NULL};
const Segment * const SG91329[] = {SGI380658, SGI380659, SGI0, SGI380659, SGI0, SGI380660, SGI380661, NULL};
const Segment * const SG91330[] = {SGI380662, SGI380663, SGI380664, SGI380663, SGI0, SGI380665, SGI380666, NULL};
const Segment * const SG91331[] = {SGI380667, SGI380668, SGI380669, SGI380668, SGI0, SGI380670, SGI380671, NULL};
const Segment * const SG91332[] = {SGI380672, SGI380673, SGI0, SGI380673, SGI0, SGI380674, SGI380675, NULL};
const Segment * const SG91333[] = {SGI380676, SGI380677, SGI0, SGI380677, SGI0, SGI380678, SGI380679, NULL};
const Segment * const SG91334[] = {SGI380680, SGI380681, SGI380682, SGI380681, SGI0, SGI380683, SGI380684, NULL};
const Segment * const SG91335[] = {SGI380685, SGI380686, SGI380687, SGI380686, SGI0, SGI380688, SGI380689, NULL};
const Segment * const SG91336[] = {SGI380690, SGI380691, SGI380692, SGI380691, SGI0, SGI380693, SGI380694, NULL};
const Segment * const SG91337[] = {SGI380695, SGI380696, SGI380697, SGI380696, SGI0, SGI380698, SGI380699, NULL};
const Segment * const SG91338[] = {SGI380700, SGI380701, SGI380702, SGI380701, SGI0, SGI380703, SGI380704, NULL};
const Segment * const SG91339[] = {SGI380705, SGI380706, SGI380707, SGI380706, SGI0, SGI380708, SGI380709, NULL};
const Segment * const SG91340[] = {SGI380710, SGI380711, SGI380712, SGI380711, SGI0, SGI380713, SGI380714, NULL};
const Segment * const SG91341[] = {SGI380715, SGI380716, SGI380717, SGI380716, SGI0, SGI380718, SGI380719, NULL};
const Segment * const SG91342[] = {SGI380720, SGI380721, SGI380722, SGI380723, SGI0, SGI380724, SGI380725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91343[] = {SGI380726, SGI380727, SGI380728, SGI0, SGI380729, SGI0, SGI380730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91344[] = {SGI380731, SGI380732, SGI0, SGI380732, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91345[] = {SGI380733, SGI380734, SGI380735, SGI0, SGI380734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380736, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91346[] = {SGI124631, SGI380737, SGI380738, SGI380739, SGI380740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91347[] = {SGI237877, SGI380741, SGI380742, SGI380743, SGI380744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91348[] = {SGI380745, SGI380746, SGI380747, SGI380748, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91349[] = {SGI380749, SGI380750, SGI0, SGI380750, SGI0, SGI380751, SGI380752, NULL};
const Segment * const SG91350[] = {SGI380753, SGI380754, SGI0, SGI380754, SGI0, SGI380755, SGI380756, NULL};
const Segment * const SG91351[] = {SGI380757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380758, SGI0, NULL};
const Segment * const SG91352[] = {SGI380759, SGI380760, SGI0, SGI380760, SGI0, SGI380761, SGI380762, NULL};
const Segment * const SG91353[] = {SGI380763, SGI380764, SGI0, SGI380764, SGI0, SGI380765, SGI380766, NULL};
const Segment * const SG91354[] = {SGI380767, SGI380768, SGI0, SGI380768, SGI0, SGI380769, SGI380770, NULL};
const Segment * const SG91355[] = {SGI380771, SGI380772, SGI0, SGI380772, SGI0, SGI380773, SGI380774, NULL};
const Segment * const SG91356[] = {SGI380775, SGI380776, SGI0, SGI380776, SGI0, SGI380777, SGI380778, NULL};
const Segment * const SG91357[] = {SGI380779, SGI380780, SGI0, SGI380780, SGI0, SGI380781, SGI380782, NULL};
const Segment * const SG91358[] = {SGI380783, SGI380784, SGI0, SGI380784, SGI0, SGI380785, SGI380786, NULL};
const Segment * const SG91359[] = {SGI380787, SGI380788, SGI0, SGI380788, SGI0, SGI380789, SGI380790, NULL};
const Segment * const SG91360[] = {SGI380791, SGI380792, SGI0, SGI380792, SGI0, SGI380793, SGI380794, NULL};
const Segment * const SG91361[] = {SGI380795, SGI380796, SGI0, SGI380796, SGI0, SGI380797, SGI380798, NULL};
const Segment * const SG91362[] = {SGI380799, SGI380800, SGI0, SGI380800, SGI0, SGI380801, SGI380802, NULL};
const Segment * const SG91363[] = {SGI380803, SGI380804, SGI0, SGI380804, SGI0, SGI380805, SGI380806, NULL};
const Segment * const SG91364[] = {SGI380807, SGI380808, SGI380809, SGI380808, SGI0, SGI380810, SGI380811, NULL};
const Segment * const SG91365[] = {SGI380812, SGI380813, SGI380814, SGI380813, SGI0, SGI380815, SGI380816, NULL};
const Segment * const SG91366[] = {SGI380817, SGI380818, SGI0, SGI380818, SGI0, SGI380819, SGI380820, NULL};
const Segment * const SG91367[] = {SGI380821, SGI380822, SGI380823, SGI380822, SGI0, SGI380824, SGI380825, NULL};
const Segment * const SG91368[] = {SGI380826, SGI380827, SGI0, SGI380827, SGI0, SGI380828, SGI380829, NULL};
const Segment * const SG91369[] = {SGI380830, SGI380831, SGI380832, SGI380831, SGI0, SGI380833, SGI380834, NULL};
const Segment * const SG91370[] = {SGI380835, SGI380836, SGI380837, SGI380836, SGI0, SGI380838, SGI380839, NULL};
const Segment * const SG91371[] = {SGI380840, SGI380841, SGI380842, SGI380841, SGI0, SGI380843, SGI380844, NULL};
const Segment * const SG91372[] = {SGI380845, SGI380846, SGI380847, SGI380846, SGI0, SGI380848, SGI380849, NULL};
const Segment * const SG91373[] = {SGI380850, SGI380851, SGI380852, SGI380851, SGI0, SGI380853, SGI380854, NULL};
const Segment * const SG91374[] = {SGI380855, SGI380856, SGI0, SGI380856, SGI0, SGI380857, SGI380858, NULL};
const Segment * const SG91375[] = {SGI380859, SGI380860, SGI380861, SGI380860, SGI0, SGI380862, SGI380863, NULL};
const Segment * const SG91376[] = {SGI380864, SGI380865, SGI380866, SGI380865, SGI0, SGI380867, SGI380868, NULL};
const Segment * const SG91377[] = {SGI380869, SGI380870, SGI0, SGI380870, SGI0, SGI380871, SGI380872, NULL};
const Segment * const SG91378[] = {SGI380873, SGI380874, SGI0, SGI380874, SGI0, SGI380875, SGI380876, NULL};
const Segment * const SG91379[] = {SGI380877, SGI380878, SGI0, SGI380878, SGI0, SGI380879, SGI380880, NULL};
const Segment * const SG91380[] = {SGI380881, SGI380882, SGI0, SGI380882, SGI0, SGI380883, SGI380884, NULL};
const Segment * const SG91381[] = {SGI380885, SGI380886, SGI0, SGI380886, SGI0, SGI380887, SGI380888, NULL};
const Segment * const SG91382[] = {SGI380889, SGI380890, SGI0, SGI380890, SGI0, SGI380891, SGI380892, NULL};
const Segment * const SG91383[] = {SGI380893, SGI380894, SGI380895, SGI0, SGI380896, SGI0, SGI380897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91384[] = {SGI380899, SGI380900, SGI380901, SGI0, SGI0, SGI0, SGI0, SGI380902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91385[] = {SGI380903, SGI380904, SGI380905, SGI380905, SGI0, SGI0, SGI0, SGI380906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91386[] = {SGI125244, SGI380907, SGI380908, SGI380909, SGI380910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91387[] = {SGI238835, SGI380911, SGI380912, SGI380913, SGI380914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91388[] = {SGI380915, SGI380916, SGI380917, SGI380918, SGI380919, SGI380920, SGI380921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91389[] = {SGI380922, SGI380923, SGI380924, SGI380925, SGI380923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91390[] = {SGI380926, SGI380927, SGI380928, SGI0, SGI380929, SGI0, SGI380930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91391[] = {SGI380932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380933, SGI0, NULL};
const Segment * const SG91392[] = {SGI380934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380935, SGI0, NULL};
const Segment * const SG91393[] = {SGI380936, SGI380937, SGI380938, SGI380939, SGI0, SGI0, SGI0, SGI0, SGI380940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91394[] = {SGI380941, SGI380942, SGI380943, SGI0, SGI380944, SGI0, SGI380945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91395[] = {SGI380947, SGI380948, SGI380949, SGI0, SGI380950, SGI0, SGI380951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91396[] = {SGI380953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380954, SGI0, NULL};
const Segment * const SG91397[] = {SGI380955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380956, SGI0, NULL};
const Segment * const SG91398[] = {SGI380957, SGI380958, SGI380959, SGI380959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91399[] = {SGI380961, SGI380962, SGI380963, SGI0, SGI380964, SGI0, SGI380965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91400[] = {SGI380967, SGI380968, SGI380969, SGI0, SGI0, SGI0, SGI0, SGI380970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91401[] = {SGI240652, SGI297708, SGI297709, SGI297710, SGI240651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91402[] = {SGI240788, SGI380971, SGI380972, SGI380973, SGI380974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91403[] = {SGI380975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380976, SGI0, NULL};
const Segment * const SG91404[] = {SGI380977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380978, SGI380979, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91405[] = {SGI380980, SGI380981, SGI380982, SGI380982, SGI380983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91406[] = {SGI0, SGI380984, SGI380985, SGI380986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91407[] = {SGI380987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380988, SGI0, NULL};
const Segment * const SG91408[] = {SGI380989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380990, SGI0, NULL};
const Segment * const SG91409[] = {SGI380991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380992, SGI0, NULL};
const Segment * const SG91410[] = {SGI380993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380994, SGI0, NULL};
const Segment * const SG91411[] = {SGI380995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380996, SGI0, NULL};
const Segment * const SG91412[] = {SGI380997, SGI380998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI380999, SGI381000, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91413[] = {SGI381001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91414[] = {SGI381002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381003, SGI0, NULL};
const Segment * const SG91415[] = {SGI381004, SGI381005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381006, SGI381007, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91416[] = {SGI381008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91417[] = {SGI381009, SGI0, SGI0, SGI0, SGI0, SGI381010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91418[] = {SGI381011, SGI0, SGI0, SGI0, SGI0, SGI381012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91419[] = {SGI381013, SGI0, SGI0, SGI0, SGI0, SGI381014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91420[] = {SGI381015, SGI0, SGI0, SGI0, SGI0, SGI381016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91421[] = {SGI381017, SGI0, SGI0, SGI0, SGI0, SGI381018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91422[] = {SGI381019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381020, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91423[] = {SGI381021, SGI381022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381023, SGI0, NULL};
const Segment * const SG91424[] = {SGI381024, SGI381025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381026, SGI381027, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91425[] = {SGI381028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381029, SGI0, NULL};
const Segment * const SG91426[] = {SGI381030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91427[] = {SGI381031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91428[] = {SGI381032, SGI0, SGI0, SGI0, SGI0, SGI381033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91429[] = {SGI381034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381035, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91430[] = {SGI381036, SGI381037, SGI0, SGI381038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91431[] = {SGI381039, SGI381040, SGI0, SGI381041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91432[] = {SGI381042, SGI381043, SGI0, SGI381044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91433[] = {SGI381045, SGI0, SGI0, SGI0, SGI381046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91434[] = {SGI381047, SGI0, SGI0, SGI0, SGI381048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91435[] = {SGI381049, SGI0, SGI0, SGI0, SGI381050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91436[] = {SGI381051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381052, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91437[] = {SGI381053, SGI0, SGI0, SGI0, SGI0, SGI381054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91438[] = {SGI381055, SGI0, SGI0, SGI0, SGI0, SGI381056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91439[] = {SGI381057, SGI381058, SGI0, SGI381059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91440[] = {SGI381060, SGI0, SGI0, SGI381061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91441[] = {SGI381062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381063, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91442[] = {SGI381064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381065, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91443[] = {SGI381066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381067, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91444[] = {SGI381068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381069, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91445[] = {SGI381070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381071, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91446[] = {SGI381072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381073, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91447[] = {SGI381074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381075, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91448[] = {SGI381076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381077, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91449[] = {SGI381078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381079, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91450[] = {SGI381080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381081, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91451[] = {SGI381082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381083, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91452[] = {SGI381084, SGI381085, SGI0, SGI381086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91453[] = {SGI381087, SGI381088, SGI0, SGI381089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91454[] = {SGI381090, SGI381091, SGI0, SGI381092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91455[] = {SGI381093, SGI381094, SGI0, SGI381095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91456[] = {SGI381096, SGI381097, SGI0, SGI381098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91457[] = {SGI381099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381100, SGI381101, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91458[] = {SGI381102, SGI0, SGI0, SGI0, SGI0, SGI381103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91459[] = {SGI381104, SGI0, SGI0, SGI381105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91460[] = {SGI381106, SGI0, SGI0, SGI381107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91461[] = {SGI381108, SGI0, SGI0, SGI381109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91462[] = {SGI381110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381111, SGI381112, SGI0, NULL};
const Segment * const SG91463[] = {SGI381113, SGI0, SGI0, SGI0, SGI0, SGI381114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91464[] = {SGI381115, SGI0, SGI0, SGI0, SGI0, SGI381116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91465[] = {SGI381117, SGI0, SGI0, SGI0, SGI0, SGI381118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91466[] = {SGI381119, SGI0, SGI0, SGI0, SGI0, SGI381120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91467[] = {SGI381121, SGI0, SGI0, SGI0, SGI0, SGI381122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91468[] = {SGI381123, SGI381124, SGI0, SGI381125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91469[] = {SGI381126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381127, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91470[] = {SGI381128, SGI381129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381130, SGI381131, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91471[] = {SGI381132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381133, SGI381134, SGI0, NULL};
const Segment * const SG91472[] = {SGI381135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381136, SGI381137, SGI0, NULL};
const Segment * const SG91473[] = {SGI381138, SGI381139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91474[] = {SGI0, SGI381140, SGI0, SGI0, SGI0, SGI381141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91475[] = {SGI381142, SGI0, SGI0, SGI381143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91476[] = {SGI381144, SGI0, SGI0, SGI381145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91477[] = {SGI381146, SGI0, SGI0, SGI381147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91478[] = {SGI381148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381149, SGI381150, SGI0, NULL};
const Segment * const SG91479[] = {SGI381151, SGI0, SGI0, SGI0, SGI0, SGI381152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91480[] = {SGI381153, SGI0, SGI0, SGI0, SGI0, SGI381154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91481[] = {SGI381155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381156, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91482[] = {SGI381157, SGI0, SGI0, SGI0, SGI0, SGI381158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91483[] = {SGI381159, SGI0, SGI0, SGI0, SGI0, SGI381160, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91484[] = {SGI381161, SGI0, SGI0, SGI0, SGI0, SGI381162, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91485[] = {SGI381163, SGI0, SGI0, SGI0, SGI0, SGI381164, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91486[] = {SGI381165, SGI0, SGI0, SGI0, SGI0, SGI381166, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91487[] = {SGI381167, SGI0, SGI0, SGI0, SGI0, SGI381168, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91488[] = {SGI381169, SGI0, SGI0, SGI0, SGI0, SGI381170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91489[] = {SGI381171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91490[] = {SGI381172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91491[] = {SGI381173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381174, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91492[] = {SGI381175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381176, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91493[] = {SGI381177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381178, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91494[] = {SGI381179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381180, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91495[] = {SGI381181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381182, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91496[] = {SGI381183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381184, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91497[] = {SGI381185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381186, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91498[] = {SGI381187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381188, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91499[] = {SGI381189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381190, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91500[] = {SGI381191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381192, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91501[] = {SGI381193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381194, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91502[] = {SGI381195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381196, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91503[] = {SGI381197, SGI0, SGI381198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91504[] = {SGI381199, SGI0, SGI381200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91505[] = {SGI381201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381202, SGI0, NULL};
const Segment * const SG91506[] = {SGI381203, SGI381204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91507[] = {SGI381205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91508[] = {SGI0, SGI381206, SGI0, SGI0, SGI0, SGI381207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91509[] = {SGI0, SGI381208, SGI0, SGI0, SGI0, SGI381209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91510[] = {SGI381210, SGI381211, SGI0, SGI381212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91511[] = {SGI381213, SGI0, SGI0, SGI0, SGI0, SGI381214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91512[] = {SGI381215, SGI0, SGI0, SGI0, SGI0, SGI381216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91513[] = {SGI381217, SGI0, SGI0, SGI0, SGI0, SGI381218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91514[] = {SGI381219, SGI0, SGI0, SGI0, SGI0, SGI381220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91515[] = {SGI381221, SGI0, SGI0, SGI0, SGI0, SGI381222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91516[] = {SGI381223, SGI0, SGI0, SGI0, SGI0, SGI381224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91517[] = {SGI381225, SGI0, SGI0, SGI0, SGI0, SGI381226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91518[] = {SGI381227, SGI0, SGI0, SGI0, SGI0, SGI381228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91519[] = {SGI381229, SGI0, SGI0, SGI0, SGI0, SGI381230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91520[] = {SGI381231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381232, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91521[] = {SGI381233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381234, SGI381235, SGI0, NULL};
const Segment * const SG91522[] = {SGI381236, SGI381237, SGI0, SGI381238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91523[] = {SGI381239, SGI0, SGI381240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91524[] = {SGI381241, SGI0, SGI381242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91525[] = {SGI381243, SGI0, SGI0, SGI0, SGI0, SGI381244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91526[] = {SGI381245, SGI381246, SGI0, SGI381247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91527[] = {SGI381248, SGI0, SGI0, SGI0, SGI0, SGI381249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91528[] = {SGI381250, SGI0, SGI0, SGI0, SGI0, SGI381251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91529[] = {SGI381252, SGI0, SGI0, SGI381253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91530[] = {SGI381254, SGI0, SGI0, SGI381255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91531[] = {SGI381256, SGI381257, SGI0, SGI381258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91532[] = {SGI381259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381260, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91533[] = {SGI381261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381262, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91534[] = {SGI381263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381264, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91535[] = {SGI381265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381266, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91536[] = {SGI381267, SGI381268, SGI0, SGI381269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91537[] = {SGI381270, SGI0, SGI0, SGI0, SGI0, SGI381271, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91538[] = {SGI381272, SGI0, SGI0, SGI0, SGI0, SGI381273, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91539[] = {SGI381274, SGI0, SGI0, SGI0, SGI0, SGI381275, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91540[] = {SGI381276, SGI0, SGI0, SGI0, SGI0, SGI381277, SGI0, SGI381278, SGI0, SGI0, NULL};
const Segment * const SG91541[] = {SGI381279, SGI0, SGI0, SGI0, SGI0, SGI381280, SGI0, SGI381281, SGI381282, SGI0, NULL};
const Segment * const SG91542[] = {SGI381283, SGI0, SGI0, SGI381284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91543[] = {SGI381285, SGI0, SGI0, SGI381286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91544[] = {SGI381287, SGI0, SGI0, SGI0, SGI0, SGI381288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91545[] = {SGI381289, SGI0, SGI0, SGI0, SGI0, SGI381290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91546[] = {SGI381291, SGI0, SGI0, SGI0, SGI0, SGI381292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91547[] = {SGI381293, SGI0, SGI0, SGI0, SGI0, SGI381294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91548[] = {SGI381295, SGI0, SGI0, SGI0, SGI0, SGI381296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91549[] = {SGI381297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91550[] = {SGI381298, SGI0, SGI381299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91551[] = {SGI381300, SGI0, SGI381301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91552[] = {SGI381302, SGI0, SGI381303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91553[] = {SGI381304, SGI381305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381306, SGI381307, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91554[] = {SGI0, SGI381308, SGI0, SGI0, SGI0, SGI381309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91555[] = {SGI381310, SGI0, SGI0, SGI0, SGI0, SGI381311, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91556[] = {SGI381312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91557[] = {SGI381313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91558[] = {SGI381314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91559[] = {SGI381315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91560[] = {SGI381316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91561[] = {SGI381317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91562[] = {SGI381318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91563[] = {SGI381319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91564[] = {SGI381320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91565[] = {SGI381321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91566[] = {SGI381322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91567[] = {SGI381323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91568[] = {SGI381324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91569[] = {SGI381325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91570[] = {SGI381326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91571[] = {SGI381327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91572[] = {SGI381328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91573[] = {SGI381329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91574[] = {SGI381330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91575[] = {SGI381331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91576[] = {SGI381332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91577[] = {SGI381333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91578[] = {SGI381334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91579[] = {SGI381335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91580[] = {SGI381336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91581[] = {SGI381337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381338, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91582[] = {SGI381339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381340, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91583[] = {SGI381341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381342, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91584[] = {SGI381343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381344, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91585[] = {SGI381345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381346, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91586[] = {SGI381347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381348, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91587[] = {SGI381349, SGI0, SGI0, SGI0, SGI0, SGI381350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91588[] = {SGI381351, SGI0, SGI0, SGI0, SGI0, SGI381352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91589[] = {SGI381353, SGI0, SGI0, SGI0, SGI0, SGI381354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91590[] = {SGI381355, SGI0, SGI381356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91591[] = {SGI381357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381358, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91592[] = {SGI381359, SGI0, SGI0, SGI0, SGI0, SGI381360, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91593[] = {SGI381361, SGI0, SGI0, SGI0, SGI0, SGI381362, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91594[] = {SGI381363, SGI0, SGI0, SGI0, SGI0, SGI381364, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91595[] = {SGI381365, SGI0, SGI0, SGI0, SGI0, SGI381366, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91596[] = {SGI381367, SGI0, SGI0, SGI0, SGI0, SGI381368, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91597[] = {SGI381369, SGI0, SGI0, SGI0, SGI0, SGI381370, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91598[] = {SGI381371, SGI0, SGI0, SGI0, SGI0, SGI381372, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91599[] = {SGI381373, SGI0, SGI0, SGI0, SGI0, SGI381374, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91600[] = {SGI381375, SGI0, SGI0, SGI0, SGI0, SGI381376, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91601[] = {SGI381377, SGI0, SGI0, SGI0, SGI0, SGI381378, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91602[] = {SGI381379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381380, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91603[] = {SGI381381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381382, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91604[] = {SGI381383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381384, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91605[] = {SGI381385, SGI0, SGI0, SGI381386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91606[] = {SGI381387, SGI0, SGI0, SGI0, SGI0, SGI381388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91607[] = {SGI381389, SGI0, SGI0, SGI0, SGI0, SGI381390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91608[] = {SGI381391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91609[] = {SGI381392, SGI0, SGI0, SGI0, SGI0, SGI381393, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91610[] = {SGI381394, SGI0, SGI0, SGI0, SGI0, SGI381395, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91611[] = {SGI381396, SGI0, SGI0, SGI0, SGI0, SGI381397, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91612[] = {SGI381398, SGI0, SGI0, SGI0, SGI0, SGI381399, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91613[] = {SGI381400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91614[] = {SGI381401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91615[] = {SGI381402, SGI381403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91616[] = {SGI0, SGI381404, SGI0, SGI0, SGI0, SGI0, SGI381405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91617[] = {SGI381406, SGI0, SGI0, SGI381407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91618[] = {SGI381408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381409, SGI381410, SGI0, NULL};
const Segment * const SG91619[] = {SGI381411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381412, SGI381413, SGI0, NULL};
const Segment * const SG91620[] = {SGI381414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91621[] = {SGI381415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91622[] = {SGI381416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381417, SGI381418, SGI0, NULL};
const Segment * const SG91623[] = {SGI381419, SGI0, SGI0, SGI0, SGI0, SGI381420, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91624[] = {SGI381421, SGI0, SGI0, SGI0, SGI0, SGI381422, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91625[] = {SGI381423, SGI0, SGI0, SGI0, SGI0, SGI381424, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91626[] = {SGI381425, SGI0, SGI0, SGI0, SGI0, SGI381426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91627[] = {SGI381427, SGI0, SGI0, SGI0, SGI0, SGI381428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91628[] = {SGI381429, SGI0, SGI0, SGI0, SGI0, SGI381430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91629[] = {SGI381431, SGI0, SGI0, SGI0, SGI0, SGI381432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91630[] = {SGI381433, SGI0, SGI0, SGI0, SGI0, SGI381434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91631[] = {SGI381435, SGI0, SGI0, SGI0, SGI0, SGI381436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91632[] = {SGI381437, SGI381438, SGI0, SGI381439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91633[] = {SGI381440, SGI0, SGI0, SGI0, SGI381441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91634[] = {SGI381442, SGI381443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381444, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91635[] = {SGI381445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381446, SGI381447, SGI0, NULL};
const Segment * const SG91636[] = {SGI381448, SGI0, SGI0, SGI0, SGI0, SGI381449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91637[] = {SGI381450, SGI0, SGI0, SGI0, SGI0, SGI381451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91638[] = {SGI381452, SGI0, SGI0, SGI0, SGI0, SGI381453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91639[] = {SGI381454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381455, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91640[] = {SGI381456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381457, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91641[] = {SGI381458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381459, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91642[] = {SGI381460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381461, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91643[] = {SGI381462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381463, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91644[] = {SGI381464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381465, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91645[] = {SGI381466, SGI381467, SGI0, SGI381468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91646[] = {SGI381469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381470, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91647[] = {SGI381471, SGI0, SGI0, SGI0, SGI0, SGI381472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91648[] = {SGI381473, SGI0, SGI0, SGI0, SGI0, SGI381474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91649[] = {SGI381475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381476, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91650[] = {SGI381477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381478, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91651[] = {SGI381479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381480, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91652[] = {SGI381481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381482, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91653[] = {SGI0, SGI381483, SGI0, SGI0, SGI0, SGI381484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91654[] = {SGI381485, SGI381486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91655[] = {SGI381487, SGI381488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91656[] = {SGI381489, SGI0, SGI0, SGI0, SGI0, SGI381490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91657[] = {SGI381491, SGI0, SGI0, SGI0, SGI0, SGI381492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91658[] = {SGI381493, SGI0, SGI0, SGI0, SGI0, SGI381494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91659[] = {SGI381495, SGI0, SGI0, SGI381496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91660[] = {SGI381497, SGI0, SGI0, SGI0, SGI0, SGI381498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91661[] = {SGI381499, SGI0, SGI0, SGI0, SGI0, SGI381500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91662[] = {SGI381501, SGI381502, SGI0, SGI381503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91663[] = {SGI381504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91664[] = {SGI381506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91665[] = {SGI381507, SGI0, SGI0, SGI0, SGI0, SGI381508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91666[] = {SGI381509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91667[] = {SGI381510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91668[] = {SGI381511, SGI0, SGI0, SGI0, SGI0, SGI381513, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91669[] = {SGI381512, SGI0, SGI0, SGI0, SGI0, SGI381514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91670[] = {SGI381515, SGI0, SGI0, SGI0, SGI0, SGI381516, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91671[] = {SGI381517, SGI0, SGI0, SGI0, SGI0, SGI381518, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91672[] = {SGI381519, SGI0, SGI0, SGI0, SGI0, SGI381520, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91673[] = {SGI381521, SGI0, SGI0, SGI0, SGI0, SGI381522, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91674[] = {SGI381523, SGI0, SGI0, SGI0, SGI0, SGI381524, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91675[] = {SGI381525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91676[] = {SGI381526, SGI0, SGI0, SGI0, SGI0, SGI381527, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91677[] = {SGI381528, SGI0, SGI0, SGI0, SGI0, SGI381529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91678[] = {SGI381530, SGI381531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91679[] = {SGI381532, SGI0, SGI0, SGI381533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91680[] = {SGI381534, SGI0, SGI0, SGI0, SGI0, SGI381535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91681[] = {SGI381536, SGI0, SGI381537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91682[] = {SGI381538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381539, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91683[] = {SGI381540, SGI0, SGI0, SGI0, SGI381541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91684[] = {SGI381542, SGI381543, SGI0, SGI381544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91685[] = {SGI381545, SGI381546, SGI0, SGI381547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91686[] = {SGI381548, SGI0, SGI0, SGI0, SGI0, SGI381549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91687[] = {SGI381550, SGI0, SGI0, SGI0, SGI0, SGI381551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91688[] = {SGI381552, SGI0, SGI0, SGI0, SGI0, SGI381553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91689[] = {SGI381554, SGI0, SGI0, SGI0, SGI0, SGI381555, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91690[] = {SGI381556, SGI381557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91691[] = {SGI381558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381559, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91692[] = {SGI381560, SGI0, SGI0, SGI0, SGI0, SGI381561, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91693[] = {SGI381562, SGI0, SGI381563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91694[] = {SGI381564, SGI0, SGI381565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91695[] = {SGI0, SGI381566, SGI0, SGI0, SGI0, SGI381567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91696[] = {SGI381568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91697[] = {SGI381569, SGI0, SGI0, SGI0, SGI0, SGI381570, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91698[] = {SGI381571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91699[] = {SGI381572, SGI0, SGI0, SGI0, SGI0, SGI381573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91700[] = {SGI381574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381575, SGI381576, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91701[] = {SGI381577, SGI0, SGI0, SGI381578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91702[] = {SGI381579, SGI0, SGI0, SGI381580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91703[] = {SGI381581, SGI0, SGI0, SGI381582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91704[] = {SGI381583, SGI0, SGI0, SGI381584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91705[] = {SGI381585, SGI0, SGI0, SGI0, SGI0, SGI381586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91706[] = {SGI381587, SGI0, SGI0, SGI0, SGI0, SGI381588, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91707[] = {SGI381589, SGI0, SGI0, SGI0, SGI0, SGI381590, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91708[] = {SGI381591, SGI0, SGI0, SGI0, SGI0, SGI381592, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91709[] = {SGI381593, SGI0, SGI0, SGI0, SGI0, SGI381594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91710[] = {SGI381595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91711[] = {SGI381596, SGI0, SGI0, SGI0, SGI0, SGI381597, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91712[] = {SGI381598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381599, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91713[] = {SGI381600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381601, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91714[] = {SGI381602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91715[] = {SGI381603, SGI0, SGI0, SGI381604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91716[] = {SGI381605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381606, SGI381607, SGI0, NULL};
const Segment * const SG91717[] = {SGI381608, SGI381609, SGI0, SGI381610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91718[] = {SGI381611, SGI0, SGI0, SGI0, SGI0, SGI381612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91719[] = {SGI381613, SGI0, SGI0, SGI0, SGI0, SGI381614, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91720[] = {SGI381615, SGI0, SGI0, SGI0, SGI0, SGI381616, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91721[] = {SGI381617, SGI0, SGI0, SGI0, SGI0, SGI381618, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91722[] = {SGI381619, SGI0, SGI0, SGI0, SGI0, SGI381620, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91723[] = {SGI381621, SGI0, SGI381622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91724[] = {SGI381623, SGI0, SGI0, SGI0, SGI0, SGI381624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91725[] = {SGI381625, SGI0, SGI0, SGI0, SGI0, SGI381626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91726[] = {SGI381627, SGI0, SGI0, SGI0, SGI0, SGI381628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91727[] = {SGI381629, SGI0, SGI0, SGI0, SGI0, SGI381630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91728[] = {SGI381631, SGI0, SGI0, SGI0, SGI0, SGI381632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91729[] = {SGI381634, SGI0, SGI0, SGI381635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91730[] = {SGI381633, SGI0, SGI0, SGI0, SGI0, SGI381636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91731[] = {SGI381637, SGI0, SGI0, SGI381638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91732[] = {SGI381639, SGI0, SGI0, SGI381640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91733[] = {SGI381641, SGI0, SGI0, SGI381642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91734[] = {SGI381643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381644, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91735[] = {SGI381645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381646, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91736[] = {SGI381647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381648, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91737[] = {SGI381649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381650, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91738[] = {SGI381651, SGI0, SGI0, SGI0, SGI0, SGI381652, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91739[] = {SGI381653, SGI0, SGI0, SGI0, SGI0, SGI381654, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91740[] = {SGI381655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381656, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91741[] = {SGI381657, SGI381658, SGI0, SGI381659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91742[] = {SGI381660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91743[] = {SGI381662, SGI0, SGI0, SGI0, SGI0, SGI381663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91744[] = {SGI381664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91745[] = {SGI381666, SGI0, SGI0, SGI0, SGI0, SGI381667, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91746[] = {SGI381668, SGI0, SGI0, SGI0, SGI0, SGI381669, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91747[] = {SGI381670, SGI0, SGI0, SGI0, SGI0, SGI381671, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91748[] = {SGI381672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381673, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91749[] = {SGI381674, SGI0, SGI0, SGI0, SGI0, SGI381675, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91750[] = {SGI381676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91751[] = {SGI381677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91752[] = {SGI381678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381679, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91753[] = {SGI381680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91754[] = {SGI381681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91755[] = {SGI381682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91756[] = {SGI381683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91757[] = {SGI381684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91758[] = {SGI381685, SGI0, SGI0, SGI0, SGI0, SGI381686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91759[] = {SGI381687, SGI0, SGI0, SGI0, SGI0, SGI381688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91760[] = {SGI381689, SGI0, SGI0, SGI0, SGI0, SGI381690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91761[] = {SGI381691, SGI0, SGI0, SGI0, SGI0, SGI381692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91762[] = {SGI381693, SGI0, SGI0, SGI0, SGI0, SGI381694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91763[] = {SGI381695, SGI0, SGI0, SGI0, SGI0, SGI381696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91764[] = {SGI381697, SGI0, SGI0, SGI0, SGI0, SGI381698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91765[] = {SGI381699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91766[] = {SGI381700, SGI0, SGI0, SGI0, SGI0, SGI381701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91767[] = {SGI0, SGI381702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91768[] = {SGI381703, SGI0, SGI0, SGI0, SGI0, SGI381704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91769[] = {SGI381705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91770[] = {SGI0, SGI381706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91771[] = {SGI0, SGI381707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381708, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91772[] = {SGI381709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91773[] = {SGI381710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381711, SGI381712, SGI0, NULL};
const Segment * const SG91774[] = {SGI381713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91775[] = {SGI381715, SGI0, SGI0, SGI0, SGI0, SGI381716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91776[] = {SGI381717, SGI381718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381719, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91777[] = {SGI381720, SGI381721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381722, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91778[] = {SGI0, SGI381723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381724, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91779[] = {SGI381725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381726, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91780[] = {SGI381727, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91781[] = {SGI381729, SGI0, SGI0, SGI381730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91782[] = {SGI381731, SGI0, SGI0, SGI0, SGI0, SGI381732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91783[] = {SGI381733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91784[] = {SGI381734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91785[] = {SGI381735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91786[] = {SGI381736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91787[] = {SGI381737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91788[] = {SGI381738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91789[] = {SGI381739, SGI0, SGI0, SGI0, SGI0, SGI381740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91790[] = {SGI381741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91791[] = {SGI381742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91792[] = {SGI381743, SGI0, SGI0, SGI381744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91793[] = {SGI381745, SGI0, SGI0, SGI381746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91794[] = {SGI381747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91795[] = {SGI381748, SGI0, SGI0, SGI0, SGI0, SGI381749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91796[] = {SGI381750, SGI0, SGI0, SGI0, SGI0, SGI381751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91797[] = {SGI381752, SGI0, SGI0, SGI0, SGI0, SGI381753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91798[] = {SGI381754, SGI0, SGI0, SGI0, SGI0, SGI381755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91799[] = {SGI381756, SGI0, SGI0, SGI0, SGI0, SGI381757, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91800[] = {SGI381758, SGI381759, SGI0, SGI0, SGI0, SGI381760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91801[] = {SGI381761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91802[] = {SGI381762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91803[] = {SGI381763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381764, NULL};
const Segment * const SG91804[] = {SGI381765, SGI0, SGI0, SGI0, SGI0, SGI381766, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91805[] = {SGI381767, SGI0, SGI0, SGI0, SGI0, SGI381768, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91806[] = {SGI381769, SGI0, SGI381770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91807[] = {SGI0, SGI381771, SGI0, SGI0, SGI0, SGI381772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91808[] = {SGI381773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91809[] = {SGI381774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381775, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91810[] = {SGI381776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381777, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91811[] = {SGI381778, SGI0, SGI0, SGI0, SGI0, SGI381779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91812[] = {SGI381780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381781, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91813[] = {SGI381782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91814[] = {SGI381783, SGI0, SGI0, SGI0, SGI0, SGI381784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91815[] = {SGI381785, SGI0, SGI0, SGI0, SGI0, SGI381786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91816[] = {SGI381787, SGI0, SGI0, SGI0, SGI0, SGI381788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91817[] = {SGI381789, SGI381790, SGI0, SGI381791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91818[] = {SGI381792, SGI0, SGI0, SGI0, SGI0, SGI381793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91819[] = {SGI381794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381795, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91820[] = {SGI381796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91821[] = {SGI381797, SGI0, SGI0, SGI0, SGI0, SGI381798, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91822[] = {SGI381799, SGI381800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91823[] = {SGI381801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91824[] = {SGI381802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91825[] = {SGI381803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381804, SGI381805, SGI0, NULL};
const Segment * const SG91826[] = {SGI381806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91827[] = {SGI381808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381809, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91828[] = {SGI381810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91829[] = {SGI0, SGI381811, SGI0, SGI0, SGI0, SGI381812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91830[] = {SGI381813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381814, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91831[] = {SGI381815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381816, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91832[] = {SGI381817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91833[] = {SGI381818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91834[] = {SGI381819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91835[] = {SGI381820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91836[] = {SGI381821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91837[] = {SGI381822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91838[] = {SGI381823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91839[] = {SGI381824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91840[] = {SGI0, SGI381826, SGI0, SGI0, SGI0, SGI381827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91841[] = {SGI0, SGI381828, SGI0, SGI0, SGI0, SGI381829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91842[] = {SGI0, SGI381830, SGI0, SGI0, SGI0, SGI381831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91843[] = {SGI381832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91844[] = {SGI381833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91845[] = {SGI381834, SGI0, SGI0, SGI0, SGI0, SGI381835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91846[] = {SGI381836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91847[] = {SGI381837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91848[] = {SGI381838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381839, NULL};
const Segment * const SG91849[] = {SGI0, SGI381840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91850[] = {SGI0, SGI381841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91851[] = {SGI381842, SGI0, SGI0, SGI0, SGI0, SGI381843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91852[] = {SGI381844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91853[] = {SGI381845, SGI0, SGI0, SGI0, SGI0, SGI381846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91854[] = {SGI381847, SGI0, SGI0, SGI0, SGI0, SGI381848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91855[] = {SGI381849, SGI0, SGI0, SGI0, SGI0, SGI381850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91856[] = {SGI381851, SGI0, SGI0, SGI0, SGI0, SGI381852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91857[] = {SGI381853, SGI0, SGI0, SGI0, SGI0, SGI381854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91858[] = {SGI381855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381856, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91859[] = {SGI381857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381858, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91860[] = {SGI381859, SGI0, SGI0, SGI0, SGI0, SGI381860, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91861[] = {SGI381861, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91862[] = {SGI381863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91863[] = {SGI381864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91864[] = {SGI381865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91865[] = {SGI381867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91866[] = {SGI0, SGI381868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381869, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91867[] = {SGI381870, SGI0, SGI0, SGI0, SGI0, SGI381871, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91868[] = {SGI381872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91869[] = {SGI381873, SGI0, SGI0, SGI0, SGI0, SGI381874, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91870[] = {SGI381875, SGI0, SGI0, SGI0, SGI0, SGI381876, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91871[] = {SGI381877, SGI0, SGI0, SGI0, SGI0, SGI381878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91872[] = {SGI381879, SGI0, SGI0, SGI0, SGI0, SGI381880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91873[] = {SGI381881, SGI0, SGI0, SGI0, SGI0, SGI381882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91874[] = {SGI381883, SGI0, SGI0, SGI0, SGI0, SGI381884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91875[] = {SGI381885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91876[] = {SGI381886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91877[] = {SGI381887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91878[] = {SGI381888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91879[] = {SGI381889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91880[] = {SGI381890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91881[] = {SGI381892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91882[] = {SGI381893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381894, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91883[] = {SGI381895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381896, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91884[] = {SGI381897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381898, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91885[] = {SGI381899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381900, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91886[] = {SGI381901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381902, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91887[] = {SGI381903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381904, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91888[] = {SGI381905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381906, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91889[] = {SGI381907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381908, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91890[] = {SGI0, SGI381909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381910, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91891[] = {SGI381911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381912, SGI0, SGI0, NULL};
const Segment * const SG91892[] = {SGI381913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381914, SGI0, SGI0, NULL};
const Segment * const SG91893[] = {SGI381915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381916, SGI0, SGI381917, NULL};
const Segment * const SG91894[] = {SGI381918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381919, SGI0, SGI0, NULL};
const Segment * const SG91895[] = {SGI381920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381921, SGI0, SGI0, NULL};
const Segment * const SG91896[] = {SGI381922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381923, SGI0, SGI0, NULL};
const Segment * const SG91897[] = {SGI381924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381925, SGI0, SGI0, NULL};
const Segment * const SG91898[] = {SGI0, SGI381926, SGI0, SGI0, SGI0, SGI381927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91899[] = {SGI381928, SGI0, SGI381929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91900[] = {SGI381930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381931, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91901[] = {SGI381932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91902[] = {SGI381933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381934, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91903[] = {SGI381935, SGI0, SGI0, SGI0, SGI0, SGI381936, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91904[] = {SGI381937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91905[] = {SGI381938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91906[] = {SGI381939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91907[] = {SGI381940, SGI0, SGI0, SGI0, SGI0, SGI381941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91908[] = {SGI381942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91909[] = {SGI381943, SGI0, SGI381944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91910[] = {SGI381945, SGI0, SGI381946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91911[] = {SGI381947, SGI0, SGI0, SGI0, SGI0, SGI381948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91912[] = {SGI381949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91913[] = {SGI381950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91914[] = {SGI381951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381952, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91915[] = {SGI381953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381955, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91916[] = {SGI381954, SGI0, SGI0, SGI0, SGI0, SGI381956, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91917[] = {SGI381957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381958, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91918[] = {SGI381959, SGI0, SGI0, SGI0, SGI0, SGI381960, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91919[] = {SGI381961, SGI0, SGI0, SGI0, SGI0, SGI381962, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91920[] = {SGI381963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381964, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91921[] = {SGI381965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381966, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91922[] = {SGI381967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381968, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91923[] = {SGI381969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381970, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91924[] = {SGI381971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381972, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91925[] = {SGI381973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381974, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91926[] = {SGI381975, SGI0, SGI0, SGI0, SGI0, SGI381976, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91927[] = {SGI381977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91928[] = {SGI381978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381979, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91929[] = {SGI381980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381981, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91930[] = {SGI381982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381983, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91931[] = {SGI381984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381985, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91932[] = {SGI381986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381987, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91933[] = {SGI381988, SGI0, SGI0, SGI0, SGI0, SGI381989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91934[] = {SGI381990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381991, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91935[] = {SGI381992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91936[] = {SGI381993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91937[] = {SGI381994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381995, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91938[] = {SGI381996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91939[] = {SGI381997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI381998, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91940[] = {SGI381999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91941[] = {SGI382001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91942[] = {SGI382003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91943[] = {SGI382005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382006, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91944[] = {SGI382007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382008, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91945[] = {SGI382009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382010, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91946[] = {SGI382011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382012, SGI382013, SGI0, NULL};
const Segment * const SG91947[] = {SGI382014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91948[] = {SGI382015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91949[] = {SGI382016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91950[] = {SGI382017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91951[] = {SGI382018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91952[] = {SGI382019, SGI0, SGI0, SGI382020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91953[] = {SGI382021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382022, NULL};
const Segment * const SG91954[] = {SGI382023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382024, NULL};
const Segment * const SG91955[] = {SGI382025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382026, NULL};
const Segment * const SG91956[] = {SGI382027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382028, NULL};
const Segment * const SG91957[] = {SGI382029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382030, NULL};
const Segment * const SG91958[] = {SGI382031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382032, NULL};
const Segment * const SG91959[] = {SGI382033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382034, NULL};
const Segment * const SG91960[] = {SGI382035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382036, NULL};
const Segment * const SG91961[] = {SGI382037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382038, NULL};
const Segment * const SG91962[] = {SGI382039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382040, NULL};
const Segment * const SG91963[] = {SGI382041, SGI0, SGI382042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91964[] = {SGI382043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91965[] = {SGI382044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382045, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91966[] = {SGI382046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91967[] = {SGI382047, SGI0, SGI0, SGI0, SGI0, SGI382048, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91968[] = {SGI382049, SGI0, SGI0, SGI0, SGI0, SGI382050, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91969[] = {SGI382051, SGI0, SGI0, SGI0, SGI0, SGI382052, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91970[] = {SGI382053, SGI0, SGI0, SGI0, SGI0, SGI382054, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91971[] = {SGI382055, SGI0, SGI0, SGI0, SGI0, SGI382056, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91972[] = {SGI382057, SGI0, SGI0, SGI0, SGI0, SGI382058, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91973[] = {SGI382059, SGI0, SGI0, SGI0, SGI0, SGI382060, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91974[] = {SGI382061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91975[] = {SGI382062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91976[] = {SGI382063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91977[] = {SGI382064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91978[] = {SGI382065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91979[] = {SGI382066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91980[] = {SGI382067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91981[] = {SGI382068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91982[] = {SGI382069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91983[] = {SGI382070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91984[] = {SGI382071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91985[] = {SGI382072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91986[] = {SGI382073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91987[] = {SGI382074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91988[] = {SGI382075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91989[] = {SGI382076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91990[] = {SGI382077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91991[] = {SGI382078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91992[] = {SGI382079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91993[] = {SGI382080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91994[] = {SGI382081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382082, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91995[] = {SGI382083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382084, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91996[] = {SGI382085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382086, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91997[] = {SGI382087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382088, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91998[] = {SGI382089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382090, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG91999[] = {SGI382091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382092, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92000[] = {SGI382093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382094, SGI0, SGI0, NULL};
const Segment * const SG92001[] = {SGI382095, SGI0, SGI0, SGI0, SGI0, SGI382096, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92002[] = {SGI382097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92003[] = {SGI382098, SGI0, SGI0, SGI0, SGI0, SGI382099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92004[] = {SGI382100, SGI0, SGI0, SGI0, SGI0, SGI382101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92005[] = {SGI382102, SGI0, SGI0, SGI0, SGI0, SGI382103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92006[] = {SGI382104, SGI0, SGI0, SGI0, SGI0, SGI382105, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92007[] = {SGI382106, SGI0, SGI0, SGI0, SGI0, SGI382107, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92008[] = {SGI382108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382109, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92009[] = {SGI382110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382111, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92010[] = {SGI382112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382113, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92011[] = {SGI382114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382115, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92012[] = {SGI382116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382117, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92013[] = {SGI382118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382119, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92014[] = {SGI382120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382121, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92015[] = {SGI382122, SGI0, SGI382123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92016[] = {SGI382124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382125, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92017[] = {SGI382126, SGI0, SGI0, SGI0, SGI0, SGI382127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92018[] = {SGI382128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382129, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92019[] = {SGI382130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382131, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92020[] = {SGI382132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92021[] = {SGI382133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92022[] = {SGI382134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92023[] = {SGI0, SGI382135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92024[] = {SGI0, SGI382136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382137, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92025[] = {SGI382138, SGI0, SGI0, SGI0, SGI382139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92026[] = {SGI382140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92027[] = {SGI382141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92028[] = {SGI382142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382143, NULL};
const Segment * const SG92029[] = {SGI382144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92030[] = {SGI382145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92031[] = {SGI382146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92032[] = {SGI382147, SGI0, SGI0, SGI0, SGI0, SGI382148, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92033[] = {SGI382149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382150, SGI0, SGI0, NULL};
const Segment * const SG92034[] = {SGI382151, SGI0, SGI0, SGI0, SGI0, SGI382152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92035[] = {SGI382153, SGI0, SGI0, SGI0, SGI0, SGI382154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92036[] = {SGI382155, SGI0, SGI0, SGI0, SGI0, SGI382156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92037[] = {SGI382157, SGI0, SGI0, SGI0, SGI0, SGI382158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92038[] = {SGI382159, SGI0, SGI0, SGI0, SGI0, SGI382160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92039[] = {SGI382161, SGI0, SGI0, SGI0, SGI0, SGI382162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92040[] = {SGI382163, SGI0, SGI0, SGI0, SGI0, SGI382164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92041[] = {SGI382165, SGI0, SGI0, SGI0, SGI0, SGI382166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92042[] = {SGI382167, SGI0, SGI0, SGI0, SGI0, SGI382168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92043[] = {SGI382169, SGI0, SGI0, SGI0, SGI0, SGI382170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92044[] = {SGI382171, SGI382172, SGI0, SGI0, SGI0, SGI382173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92045[] = {SGI382174, SGI0, SGI0, SGI0, SGI0, SGI382175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92046[] = {SGI382176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92047[] = {SGI382177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92048[] = {SGI382178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382179, SGI382180, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92049[] = {SGI382181, SGI0, SGI0, SGI0, SGI0, SGI382182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92050[] = {SGI382183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92051[] = {SGI382185, SGI0, SGI0, SGI0, SGI0, SGI382186, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92052[] = {SGI382187, SGI0, SGI0, SGI0, SGI0, SGI382188, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92053[] = {SGI382189, SGI0, SGI0, SGI0, SGI0, SGI382190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92054[] = {SGI382191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92055[] = {SGI382192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92056[] = {SGI382193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92057[] = {SGI382194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92058[] = {SGI382195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92059[] = {SGI382196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92060[] = {SGI382197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382198, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92061[] = {SGI382199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382200, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92062[] = {SGI382201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92063[] = {SGI382202, SGI0, SGI0, SGI0, SGI0, SGI382203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92064[] = {SGI382204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92065[] = {SGI382205, SGI0, SGI0, SGI0, SGI0, SGI382206, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92066[] = {SGI382207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92067[] = {SGI382208, SGI0, SGI0, SGI0, SGI0, SGI382209, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92068[] = {SGI382210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92069[] = {SGI382212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92070[] = {SGI382214, SGI0, SGI0, SGI0, SGI0, SGI382215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92071[] = {SGI382216, SGI0, SGI382217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92072[] = {SGI382218, SGI0, SGI382219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92073[] = {SGI382220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382221, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92074[] = {SGI382222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382223, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92075[] = {SGI382224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92076[] = {SGI382225, SGI0, SGI0, SGI0, SGI0, SGI382226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92077[] = {SGI382227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92078[] = {SGI382229, SGI0, SGI0, SGI0, SGI0, SGI382230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92079[] = {SGI382231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92080[] = {SGI382233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92081[] = {SGI382235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382236, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92082[] = {SGI382237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382238, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92083[] = {SGI382239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92084[] = {SGI382240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382241, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92085[] = {SGI382242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382243, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92086[] = {SGI382244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92087[] = {SGI382245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92088[] = {SGI382246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92089[] = {SGI382248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92090[] = {SGI382249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92091[] = {SGI382250, SGI0, SGI0, SGI0, SGI0, SGI382251, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92092[] = {SGI382252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382253, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92093[] = {SGI382254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92094[] = {SGI382255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92095[] = {SGI382256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382257, NULL};
const Segment * const SG92096[] = {SGI382258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382259, NULL};
const Segment * const SG92097[] = {SGI382260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382261, NULL};
const Segment * const SG92098[] = {SGI382262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382263, NULL};
const Segment * const SG92099[] = {SGI382264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382265, NULL};
const Segment * const SG92100[] = {SGI382266, SGI382267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92101[] = {SGI0, SGI382268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92102[] = {SGI382269, SGI382270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92103[] = {SGI382271, SGI0, SGI0, SGI0, SGI0, SGI382272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92104[] = {SGI0, SGI382273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382274, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92105[] = {SGI382275, SGI0, SGI0, SGI0, SGI0, SGI382276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92106[] = {SGI382277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92107[] = {SGI382278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92108[] = {SGI382279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92109[] = {SGI382280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92110[] = {SGI382281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92111[] = {SGI382282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92112[] = {SGI382283, SGI0, SGI0, SGI0, SGI0, SGI382284, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92113[] = {SGI382285, SGI0, SGI0, SGI0, SGI0, SGI382286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92114[] = {SGI382287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92115[] = {SGI382288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382289, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92116[] = {SGI382290, SGI0, SGI0, SGI0, SGI0, SGI382291, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92117[] = {SGI382292, SGI0, SGI0, SGI0, SGI0, SGI382293, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92118[] = {SGI382294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92119[] = {SGI0, SGI382295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92120[] = {SGI0, SGI382296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92121[] = {SGI382297, SGI0, SGI0, SGI0, SGI0, SGI382298, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92122[] = {SGI382299, SGI0, SGI0, SGI0, SGI0, SGI382300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92123[] = {SGI382301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92124[] = {SGI382303, SGI382304, SGI0, SGI382305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92125[] = {SGI382306, SGI382307, SGI0, SGI382308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92126[] = {SGI382309, SGI0, SGI382310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92127[] = {SGI382311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92128[] = {SGI382312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92129[] = {SGI382313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92130[] = {SGI382314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92131[] = {SGI382315, SGI0, SGI0, SGI0, SGI0, SGI382316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92132[] = {SGI382317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92133[] = {SGI0, SGI382318, SGI0, SGI0, SGI0, SGI382319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92134[] = {SGI382320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92135[] = {SGI382322, SGI0, SGI382323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92136[] = {SGI382324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382325, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92137[] = {SGI382326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382327, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92138[] = {SGI382328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382329, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92139[] = {SGI382330, SGI0, SGI0, SGI0, SGI0, SGI382331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92140[] = {SGI382332, SGI0, SGI0, SGI0, SGI0, SGI382333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92141[] = {SGI382334, SGI0, SGI0, SGI382335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92142[] = {SGI382336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382337, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92143[] = {SGI382338, SGI0, SGI0, SGI0, SGI0, SGI382339, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92144[] = {SGI382340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92145[] = {SGI382342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382343, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92146[] = {SGI382344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92147[] = {SGI382345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92148[] = {SGI0, SGI382346, SGI0, SGI0, SGI0, SGI382347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92149[] = {SGI382348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92150[] = {SGI382350, SGI0, SGI0, SGI0, SGI0, SGI382351, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92151[] = {SGI382352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382353, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92152[] = {SGI382354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382355, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92153[] = {SGI382356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382357, SGI0, SGI0, NULL};
const Segment * const SG92154[] = {SGI382358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382359, SGI0, SGI0, NULL};
const Segment * const SG92155[] = {SGI382360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92156[] = {SGI0, SGI382361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92157[] = {SGI382362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382363, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92158[] = {SGI382364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382365, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92159[] = {SGI382366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382367, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92160[] = {SGI382368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92161[] = {SGI0, SGI382370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382371, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92162[] = {SGI382372, SGI382373, SGI382374, SGI0, SGI0, SGI382375, NULL};
const Segment * const SG92163[] = {SGI382376, SGI382377, SGI382378, SGI0, SGI0, SGI382379, NULL};
const Segment * const SG92164[] = {SGI382380, SGI382381, SGI382382, SGI0, SGI0, SGI382383, NULL};
const Segment * const SG92165[] = {SGI382384, SGI382385, SGI382386, SGI0, SGI0, SGI382387, NULL};
const Segment * const SG92166[] = {SGI382388, SGI382389, SGI382390, SGI0, SGI0, SGI382391, NULL};
const Segment * const SG92167[] = {SGI382392, SGI382393, SGI382394, SGI0, SGI0, SGI382395, NULL};
const Segment * const SG92168[] = {SGI382396, SGI382397, SGI382398, SGI0, SGI0, SGI382399, NULL};
const Segment * const SG92169[] = {SGI382400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92170[] = {SGI0, SGI382401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92171[] = {SGI382402, SGI0, SGI0, SGI0, SGI0, SGI382403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92172[] = {SGI382404, SGI0, SGI0, SGI0, SGI0, SGI382405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92173[] = {SGI0, SGI382406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92174[] = {SGI382407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382408, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92175[] = {SGI382409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92176[] = {SGI382410, SGI0, SGI0, SGI0, SGI0, SGI382411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92177[] = {SGI382412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92178[] = {SGI382413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92179[] = {SGI382414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382415, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92180[] = {SGI0, SGI382416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382417, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92181[] = {SGI382418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92182[] = {SGI382419, SGI0, SGI0, SGI0, SGI0, SGI382420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92183[] = {SGI382421, SGI0, SGI0, SGI0, SGI0, SGI382422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92184[] = {SGI382423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92185[] = {SGI382424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92186[] = {SGI382425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92187[] = {SGI382426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92188[] = {SGI382427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92189[] = {SGI382428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92190[] = {SGI382430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92191[] = {SGI382431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92192[] = {SGI382433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382434, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92193[] = {SGI382435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92194[] = {SGI382436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92195[] = {SGI382438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382439, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92196[] = {SGI382440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382441, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92197[] = {SGI382442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382443, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92198[] = {SGI382444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382445, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92199[] = {SGI0, SGI382446, SGI0, SGI0, SGI0, SGI382447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92200[] = {SGI0, SGI382448, SGI0, SGI0, SGI0, SGI382449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92201[] = {SGI382450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92202[] = {SGI382451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382452, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92203[] = {SGI382453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92204[] = {SGI382454, SGI0, SGI0, SGI0, SGI0, SGI382455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92205[] = {SGI382456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92206[] = {SGI382458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92207[] = {SGI382460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92208[] = {SGI382462, SGI0, SGI0, SGI0, SGI0, SGI382463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92209[] = {SGI382464, SGI0, SGI0, SGI0, SGI0, SGI382465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92210[] = {SGI382466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382467, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92211[] = {SGI382468, SGI0, SGI0, SGI0, SGI0, SGI382469, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92212[] = {SGI382470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382471, NULL};
const Segment * const SG92213[] = {SGI382472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382473, NULL};
const Segment * const SG92214[] = {SGI382474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382475, NULL};
const Segment * const SG92215[] = {SGI0, SGI382476, SGI0, SGI0, SGI0, SGI382477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92216[] = {SGI0, SGI382478, SGI0, SGI0, SGI0, SGI382479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92217[] = {SGI0, SGI382480, SGI0, SGI0, SGI0, SGI382481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92218[] = {SGI382482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92219[] = {SGI382483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92220[] = {SGI382484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92221[] = {SGI382485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92222[] = {SGI382486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92223[] = {SGI382487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92224[] = {SGI382488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92225[] = {SGI0, SGI382490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92226[] = {SGI382491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92227[] = {SGI382493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382494, SGI382495, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92228[] = {SGI382496, SGI0, SGI0, SGI0, SGI0, SGI382497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92229[] = {SGI0, SGI382498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382499, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92230[] = {SGI0, SGI382500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382501, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92231[] = {SGI382502, SGI0, SGI0, SGI0, SGI0, SGI382503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92232[] = {SGI382504, SGI0, SGI0, SGI0, SGI0, SGI382505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92233[] = {SGI382506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92234[] = {SGI382507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92235[] = {SGI382508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382509, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92236[] = {SGI382510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92237[] = {SGI382512, SGI0, SGI0, SGI0, SGI0, SGI382513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92238[] = {SGI382514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92239[] = {SGI382516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92240[] = {SGI382518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92241[] = {SGI382520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92242[] = {SGI382522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92243[] = {SGI382523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92244[] = {SGI382524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382525, NULL};
const Segment * const SG92245[] = {SGI382526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92246[] = {SGI382528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92247[] = {SGI382529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92248[] = {SGI382530, SGI0, SGI0, SGI0, SGI0, SGI382531, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92249[] = {SGI382532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92250[] = {SGI382534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382535, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92251[] = {SGI382536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92252[] = {SGI382538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92253[] = {SGI382539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92254[] = {SGI382540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92255[] = {SGI382541, SGI0, SGI382542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92256[] = {SGI382543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92257[] = {SGI0, SGI382545, SGI0, SGI0, SGI0, SGI382546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92258[] = {SGI0, SGI382547, SGI0, SGI0, SGI0, SGI382548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92259[] = {SGI382549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92260[] = {SGI382551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92261[] = {SGI382552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92262[] = {SGI382553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92263[] = {SGI0, SGI382554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382555, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92264[] = {SGI382556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92265[] = {SGI382557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382558, SGI382559, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92266[] = {SGI382560, SGI0, SGI0, SGI382561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92267[] = {SGI382562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92268[] = {SGI0, SGI382563, SGI0, SGI0, SGI0, SGI382564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92269[] = {SGI0, SGI382565, SGI0, SGI0, SGI0, SGI382566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92270[] = {SGI382567, SGI0, SGI0, SGI0, SGI0, SGI382568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92271[] = {SGI382569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92272[] = {SGI382570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92273[] = {SGI382571, SGI0, SGI0, SGI0, SGI0, SGI382572, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92274[] = {SGI382573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382574, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92275[] = {SGI376333, SGI382575, SGI376332, SGI376332, SGI376336, NULL};
const Segment * const SG92276[] = {SGI376341, SGI382576, SGI376340, SGI376340, SGI376344, NULL};
const Segment * const SG92277[] = {SGI376349, SGI382577, SGI376348, SGI376348, SGI376351, NULL};
const Segment * const SG92278[] = {SGI376356, SGI382578, SGI382579, SGI376355, SGI376359, NULL};
const Segment * const SG92279[] = {SGI382580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382581, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92280[] = {SGI382582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92281[] = {SGI382583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382584, SGI0, SGI0, NULL};
const Segment * const SG92282[] = {SGI382585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382586, SGI0, SGI0, NULL};
const Segment * const SG92283[] = {SGI382587, SGI0, SGI0, SGI0, SGI0, SGI382588, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92284[] = {SGI382589, SGI0, SGI0, SGI0, SGI0, SGI382590, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92285[] = {SGI382591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92286[] = {SGI382593, SGI0, SGI0, SGI0, SGI0, SGI382594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92287[] = {SGI382595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382596, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92288[] = {SGI0, SGI382597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382598, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92289[] = {SGI382599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92290[] = {SGI382601, SGI0, SGI0, SGI0, SGI0, SGI382602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92291[] = {SGI382603, SGI0, SGI0, SGI0, SGI0, SGI382604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92292[] = {SGI382605, SGI0, SGI0, SGI0, SGI0, SGI382606, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92293[] = {SGI382607, SGI0, SGI0, SGI0, SGI0, SGI382608, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92294[] = {SGI382609, SGI0, SGI0, SGI0, SGI0, SGI382610, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92295[] = {SGI382611, SGI382612, SGI0, SGI382613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92296[] = {SGI382614, SGI382615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92297[] = {SGI382616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382617, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92298[] = {SGI382618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382619, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92299[] = {SGI0, SGI382620, SGI0, SGI0, SGI0, SGI382621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92300[] = {SGI0, SGI382622, SGI0, SGI0, SGI0, SGI382623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92301[] = {SGI0, SGI382624, SGI0, SGI0, SGI0, SGI382625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92302[] = {SGI0, SGI382626, SGI0, SGI0, SGI0, SGI382627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92303[] = {SGI0, SGI382628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92304[] = {SGI0, SGI382629, SGI0, SGI0, SGI0, SGI382630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92305[] = {SGI382631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92306[] = {SGI382633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92307[] = {SGI382635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92308[] = {SGI382637, SGI0, SGI0, SGI0, SGI0, SGI382638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92309[] = {SGI382639, SGI0, SGI0, SGI0, SGI0, SGI382640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92310[] = {SGI382641, SGI0, SGI0, SGI0, SGI0, SGI382642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92311[] = {SGI382643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382644, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92312[] = {SGI382645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92313[] = {SGI382646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92314[] = {SGI382647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92315[] = {SGI382649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92316[] = {SGI382651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92317[] = {SGI382652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92318[] = {SGI382654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92319[] = {SGI382655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382656, SGI382657, SGI0, NULL};
const Segment * const SG92320[] = {SGI382658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92321[] = {SGI382659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92322[] = {SGI382660, SGI0, SGI0, SGI0, SGI0, SGI382661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92323[] = {SGI382662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92324[] = {SGI382663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92325[] = {SGI0, SGI382664, SGI0, SGI0, SGI0, SGI382665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92326[] = {SGI382666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92327[] = {SGI382667, SGI0, SGI0, SGI0, SGI0, SGI382668, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92328[] = {SGI382669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92329[] = {SGI382670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92330[] = {SGI382671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92331[] = {SGI0, SGI382673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92332[] = {SGI382674, SGI382675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92333[] = {SGI382676, SGI0, SGI0, SGI382677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92334[] = {SGI382678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382679, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92335[] = {SGI382680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92336[] = {SGI382682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92337[] = {SGI382684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92338[] = {SGI382685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92339[] = {SGI382687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92340[] = {SGI382689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92341[] = {SGI382691, SGI0, SGI0, SGI0, SGI0, SGI382692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92342[] = {SGI382693, SGI0, SGI0, SGI0, SGI0, SGI382694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92343[] = {SGI382695, SGI382696, SGI0, SGI0, SGI0, SGI382697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92344[] = {SGI382698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382699, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92345[] = {SGI382700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92346[] = {SGI382702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92347[] = {SGI382704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92348[] = {SGI382706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92349[] = {SGI382708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92350[] = {SGI382710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382711, NULL};
const Segment * const SG92351[] = {SGI382712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382713, NULL};
const Segment * const SG92352[] = {SGI382714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382715, NULL};
const Segment * const SG92353[] = {SGI0, SGI382716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92354[] = {SGI382717, SGI0, SGI0, SGI0, SGI0, SGI382718, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92355[] = {SGI382719, SGI0, SGI0, SGI0, SGI0, SGI382720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92356[] = {SGI0, SGI382721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382722, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92357[] = {SGI0, SGI382723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92358[] = {SGI382724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382725, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92359[] = {SGI382726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382727, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92360[] = {SGI382728, SGI0, SGI0, SGI0, SGI0, SGI382729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92361[] = {SGI0, SGI382730, SGI0, SGI0, SGI0, SGI382731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92362[] = {SGI0, SGI382732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92363[] = {SGI382733, SGI382734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92364[] = {SGI382735, SGI382736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382737, SGI382738, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92365[] = {SGI382739, SGI0, SGI0, SGI0, SGI0, SGI382740, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92366[] = {SGI382741, SGI0, SGI0, SGI0, SGI0, SGI382742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92367[] = {SGI382743, SGI0, SGI0, SGI0, SGI0, SGI382744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92368[] = {SGI382745, SGI0, SGI0, SGI0, SGI0, SGI382746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92369[] = {SGI0, SGI382747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92370[] = {SGI382748, SGI0, SGI0, SGI0, SGI0, SGI382749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92371[] = {SGI0, SGI382750, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382751, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92372[] = {SGI382752, SGI0, SGI0, SGI0, SGI0, SGI382753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92373[] = {SGI382754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92374[] = {SGI382755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92375[] = {SGI382756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92376[] = {SGI382757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382758, SGI0, SGI0, NULL};
const Segment * const SG92377[] = {SGI382759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382760, SGI0, SGI0, NULL};
const Segment * const SG92378[] = {SGI382761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382762, SGI382763, SGI0, NULL};
const Segment * const SG92379[] = {SGI382764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92380[] = {SGI382766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382767, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92381[] = {SGI382768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92382[] = {SGI382770, SGI382771, SGI0, SGI382772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92383[] = {SGI382773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92384[] = {SGI382775, SGI0, SGI0, SGI0, SGI0, SGI382776, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92385[] = {SGI382777, SGI0, SGI0, SGI0, SGI0, SGI382778, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92386[] = {SGI382779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92387[] = {SGI382781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92388[] = {SGI382783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382784, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92389[] = {SGI382785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92390[] = {SGI382786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92391[] = {SGI382787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92392[] = {SGI382789, SGI0, SGI0, SGI0, SGI0, SGI382790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92393[] = {SGI382791, SGI0, SGI0, SGI0, SGI0, SGI382792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92394[] = {SGI382793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382794, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92395[] = {SGI0, SGI382795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382796, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92396[] = {SGI382797, SGI382798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92397[] = {SGI382799, SGI0, SGI382800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92398[] = {SGI382801, SGI0, SGI0, SGI0, SGI0, SGI382802, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92399[] = {SGI382803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92400[] = {SGI382804, SGI0, SGI0, SGI0, SGI0, SGI382805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92401[] = {SGI382806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92402[] = {SGI382807, SGI0, SGI382808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92403[] = {SGI382809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92404[] = {SGI382811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382812, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92405[] = {SGI382813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92406[] = {SGI382814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92407[] = {SGI382816, SGI0, SGI0, SGI0, SGI0, SGI382817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92408[] = {SGI382818, SGI0, SGI0, SGI0, SGI0, SGI382819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92409[] = {SGI382820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92410[] = {SGI382821, NULL};
const Segment * const SG92411[] = {SGI382822, SGI0, SGI0, SGI0, SGI0, SGI382823, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92412[] = {SGI382824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92413[] = {SGI382826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92414[] = {SGI382828, SGI0, SGI0, SGI0, SGI0, SGI382829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92415[] = {SGI382830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92416[] = {SGI382831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92417[] = {SGI382832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92418[] = {SGI382833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92419[] = {SGI382834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92420[] = {SGI382836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92421[] = {SGI382838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92422[] = {SGI382840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92423[] = {SGI382841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92424[] = {SGI382842, SGI382843, SGI382844, SGI0, SGI0, SGI382845, NULL};
const Segment * const SG92425[] = {SGI382846, SGI382847, SGI382848, SGI0, SGI0, SGI382849, NULL};
const Segment * const SG92426[] = {SGI382850, SGI382851, SGI382852, SGI0, SGI0, SGI382853, NULL};
const Segment * const SG92427[] = {SGI382854, SGI382855, SGI382856, SGI0, SGI0, SGI382857, NULL};
const Segment * const SG92428[] = {SGI382858, SGI382859, SGI382860, SGI0, SGI0, SGI382861, NULL};
const Segment * const SG92429[] = {SGI382862, SGI382863, SGI382864, SGI0, SGI0, SGI382865, NULL};
const Segment * const SG92430[] = {SGI382866, SGI382867, SGI382868, SGI0, SGI0, SGI382869, NULL};
const Segment * const SG92431[] = {SGI382870, SGI382871, SGI382872, SGI0, SGI0, SGI382873, NULL};
const Segment * const SG92432[] = {SGI382874, SGI382875, SGI382876, SGI0, SGI0, SGI382877, NULL};
const Segment * const SG92433[] = {SGI382878, SGI382879, SGI382880, SGI0, SGI0, SGI382881, NULL};
const Segment * const SG92434[] = {SGI382882, SGI382883, SGI382884, SGI0, SGI0, SGI382885, NULL};
const Segment * const SG92435[] = {SGI382886, SGI382887, SGI382888, SGI0, SGI0, SGI382889, NULL};
const Segment * const SG92436[] = {SGI382890, SGI382891, SGI382892, SGI0, SGI0, SGI382893, NULL};
const Segment * const SG92437[] = {SGI382894, SGI382895, SGI382896, SGI0, SGI0, SGI382897, NULL};
const Segment * const SG92438[] = {SGI382898, SGI382899, SGI382900, SGI0, SGI0, SGI382901, NULL};
const Segment * const SG92439[] = {SGI382902, SGI382903, SGI382904, SGI0, SGI0, SGI382905, NULL};
const Segment * const SG92440[] = {SGI382906, SGI382907, SGI382908, SGI0, SGI0, SGI382909, NULL};
const Segment * const SG92441[] = {SGI382910, SGI382911, SGI382912, SGI0, SGI0, SGI382913, NULL};
const Segment * const SG92442[] = {SGI382914, SGI382915, SGI382916, SGI0, SGI0, SGI382917, NULL};
const Segment * const SG92443[] = {SGI382918, SGI382919, SGI382920, SGI0, SGI0, SGI382921, NULL};
const Segment * const SG92444[] = {SGI382922, SGI382923, SGI382924, SGI0, SGI0, SGI382925, NULL};
const Segment * const SG92445[] = {SGI382926, SGI382927, SGI382928, SGI0, SGI0, SGI382929, NULL};
const Segment * const SG92446[] = {SGI382930, SGI382931, SGI382932, SGI0, SGI0, SGI382933, NULL};
const Segment * const SG92447[] = {SGI382934, SGI382935, SGI382936, SGI0, SGI0, SGI382937, NULL};
const Segment * const SG92448[] = {SGI382938, SGI382939, SGI382940, SGI0, SGI0, SGI382941, NULL};
const Segment * const SG92449[] = {SGI382942, SGI382943, SGI382944, SGI0, SGI0, SGI382945, NULL};
const Segment * const SG92450[] = {SGI382946, SGI382947, SGI382948, SGI0, SGI0, SGI382949, NULL};
const Segment * const SG92451[] = {SGI382950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92452[] = {SGI382952, SGI382953, SGI382954, SGI0, SGI0, SGI382955, NULL};
const Segment * const SG92453[] = {SGI382956, SGI382957, SGI382958, SGI0, SGI0, SGI382959, NULL};
const Segment * const SG92454[] = {SGI382960, SGI0, SGI0, SGI0, SGI0, SGI382961, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92455[] = {SGI382962, SGI0, SGI0, SGI0, SGI0, SGI382963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92456[] = {SGI0, SGI382964, SGI0, SGI0, SGI0, SGI382965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92457[] = {SGI382966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92458[] = {SGI0, SGI382967, SGI0, SGI0, SGI0, SGI382968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92459[] = {SGI0, SGI382969, SGI0, SGI0, SGI0, SGI382970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92460[] = {SGI382971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382972, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92461[] = {SGI382973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92462[] = {SGI382974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92463[] = {SGI382975, SGI382976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92464[] = {SGI382977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382978, NULL};
const Segment * const SG92465[] = {SGI382979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382980, NULL};
const Segment * const SG92466[] = {SGI382981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382982, NULL};
const Segment * const SG92467[] = {SGI382983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92468[] = {SGI382984, SGI0, SGI0, SGI0, SGI0, SGI382985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92469[] = {SGI382986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382987, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92470[] = {SGI0, SGI382988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92471[] = {SGI0, SGI382989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382990, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92472[] = {SGI382991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92473[] = {SGI382993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92474[] = {SGI382995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382996, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92475[] = {SGI382997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI382998, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92476[] = {SGI382999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92477[] = {SGI383001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92478[] = {SGI383003, SGI0, SGI0, SGI383004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92479[] = {SGI383005, SGI383006, SGI383007, SGI0, SGI0, SGI383008, NULL};
const Segment * const SG92480[] = {SGI383009, SGI383010, SGI383011, SGI0, SGI0, SGI383012, NULL};
const Segment * const SG92481[] = {SGI383013, SGI383014, SGI383015, SGI0, SGI0, SGI383016, NULL};
const Segment * const SG92482[] = {SGI383017, SGI383018, SGI383019, SGI0, SGI0, SGI383020, NULL};
const Segment * const SG92483[] = {SGI383021, SGI383022, SGI383023, SGI0, SGI0, SGI383024, NULL};
const Segment * const SG92484[] = {SGI383025, SGI383026, SGI383027, SGI0, SGI0, SGI383028, NULL};
const Segment * const SG92485[] = {SGI383029, SGI383030, SGI383031, SGI0, SGI0, SGI383032, NULL};
const Segment * const SG92486[] = {SGI383033, SGI383034, SGI383035, SGI0, SGI0, SGI383036, NULL};
const Segment * const SG92487[] = {SGI383037, SGI383038, SGI383039, SGI0, SGI0, SGI383040, NULL};
const Segment * const SG92488[] = {SGI383041, SGI383042, SGI383043, SGI0, SGI0, SGI383044, NULL};
const Segment * const SG92489[] = {SGI383045, SGI383046, SGI383047, SGI0, SGI0, SGI383048, NULL};
const Segment * const SG92490[] = {SGI383049, SGI383050, SGI383051, SGI0, SGI0, SGI383052, NULL};
const Segment * const SG92491[] = {SGI383053, SGI383054, SGI383055, SGI0, SGI0, SGI383056, NULL};
const Segment * const SG92492[] = {SGI383057, SGI383058, SGI383059, SGI0, SGI0, SGI383060, NULL};
const Segment * const SG92493[] = {SGI383061, SGI383062, SGI383063, SGI0, SGI0, SGI383064, NULL};
const Segment * const SG92494[] = {SGI383065, SGI383066, SGI383067, SGI0, SGI0, SGI383068, NULL};
const Segment * const SG92495[] = {SGI383069, SGI383070, SGI383071, SGI0, SGI0, SGI383072, NULL};
const Segment * const SG92496[] = {SGI383073, SGI383074, SGI383075, SGI0, SGI0, SGI383076, NULL};
const Segment * const SG92497[] = {SGI383077, SGI383078, SGI383079, SGI0, SGI0, SGI383080, NULL};
const Segment * const SG92498[] = {SGI383081, SGI383082, SGI383083, SGI0, SGI0, SGI383084, NULL};
const Segment * const SG92499[] = {SGI383085, SGI383086, SGI383087, SGI0, SGI0, SGI383088, NULL};
const Segment * const SG92500[] = {SGI383089, SGI383090, SGI383091, SGI0, SGI0, SGI383092, NULL};
const Segment * const SG92501[] = {SGI383093, SGI383094, SGI383095, SGI0, SGI0, SGI383096, NULL};
const Segment * const SG92502[] = {SGI383097, SGI383098, SGI383099, SGI0, SGI0, SGI383100, NULL};
const Segment * const SG92503[] = {SGI383101, SGI383102, SGI383103, SGI0, SGI0, SGI383104, NULL};
const Segment * const SG92504[] = {SGI383105, SGI383106, SGI383107, SGI0, SGI0, SGI383108, NULL};
const Segment * const SG92505[] = {SGI383109, SGI383110, SGI383111, SGI0, SGI0, SGI383112, NULL};
const Segment * const SG92506[] = {SGI383113, SGI383114, SGI383115, SGI0, SGI0, SGI383116, NULL};
const Segment * const SG92507[] = {SGI383117, SGI383118, SGI383119, SGI0, SGI0, SGI383120, NULL};
const Segment * const SG92508[] = {SGI383121, SGI383122, SGI383123, SGI0, SGI0, SGI383124, NULL};
const Segment * const SG92509[] = {SGI383125, SGI383126, SGI383127, SGI0, SGI0, SGI383128, NULL};
const Segment * const SG92510[] = {SGI383129, SGI383130, SGI383131, SGI0, SGI0, SGI383132, NULL};
const Segment * const SG92511[] = {SGI383133, SGI383134, SGI383135, SGI0, SGI0, SGI383136, NULL};
const Segment * const SG92512[] = {SGI383137, SGI383138, SGI383139, SGI0, SGI0, SGI383140, NULL};
const Segment * const SG92513[] = {SGI383141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383142, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92514[] = {SGI383143, SGI383144, SGI383145, SGI0, SGI0, SGI383146, NULL};
const Segment * const SG92515[] = {SGI383147, SGI383148, SGI383149, SGI0, SGI0, SGI383150, NULL};
const Segment * const SG92516[] = {SGI383151, SGI383152, SGI383153, SGI0, SGI0, SGI383154, NULL};
const Segment * const SG92517[] = {SGI383155, SGI383157, SGI383158, SGI0, SGI0, SGI383160, NULL};
const Segment * const SG92518[] = {SGI0, SGI383156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383159, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92519[] = {SGI383161, SGI383162, SGI383163, SGI0, SGI0, SGI383165, NULL};
const Segment * const SG92520[] = {SGI0, SGI383164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383166, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92521[] = {SGI0, SGI383168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92522[] = {SGI383167, SGI383169, SGI383170, SGI0, SGI0, SGI383171, NULL};
const Segment * const SG92523[] = {SGI0, SGI383172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92524[] = {SGI383173, SGI383174, SGI383175, SGI0, SGI0, SGI383176, NULL};
const Segment * const SG92525[] = {SGI383177, SGI383178, SGI383179, SGI0, SGI0, SGI383180, NULL};
const Segment * const SG92526[] = {SGI383181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92527[] = {SGI383182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92528[] = {SGI383184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92529[] = {SGI383186, SGI0, SGI0, SGI0, SGI0, SGI383187, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92530[] = {SGI0, SGI383188, SGI0, SGI0, SGI0, SGI383189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92531[] = {SGI383190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92532[] = {SGI383191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92533[] = {SGI383193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92534[] = {SGI383194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92535[] = {SGI383195, SGI0, SGI0, SGI0, SGI0, SGI383196, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92536[] = {SGI383197, SGI0, SGI0, SGI0, SGI0, SGI383198, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92537[] = {SGI383199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383200, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92538[] = {SGI383201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383202, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92539[] = {SGI383203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92540[] = {SGI383205, SGI0, SGI0, SGI0, SGI0, SGI383206, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92541[] = {SGI383207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92542[] = {SGI383208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92543[] = {SGI383209, SGI383210, SGI0, SGI383211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92544[] = {SGI383212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92545[] = {SGI383214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92546[] = {SGI383216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92547[] = {SGI383218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92548[] = {SGI383220, SGI383221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92549[] = {SGI383222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92550[] = {SGI383223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92551[] = {SGI383224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92552[] = {SGI0, SGI383226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92553[] = {SGI383227, SGI383228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92554[] = {SGI0, SGI383229, SGI0, SGI0, SGI0, SGI0, SGI383230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92555[] = {SGI383231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383232, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92556[] = {SGI383233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92557[] = {SGI383234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92558[] = {SGI383236, SGI0, SGI0, SGI0, SGI0, SGI383237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92559[] = {SGI383238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383239, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92560[] = {SGI383240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383241, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92561[] = {SGI383242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383243, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92562[] = {SGI383244, SGI0, SGI0, SGI0, SGI0, SGI383245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92563[] = {SGI0, SGI383246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383247, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92564[] = {SGI383248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383249, SGI383250, SGI0, NULL};
const Segment * const SG92565[] = {SGI383251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92566[] = {SGI383252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92567[] = {SGI383253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92568[] = {SGI383254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92569[] = {SGI383255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383256, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92570[] = {SGI383257, SGI0, SGI0, SGI0, SGI0, SGI383258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92571[] = {SGI383259, SGI0, SGI0, SGI0, SGI0, SGI383260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92572[] = {SGI383261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92573[] = {SGI383262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92574[] = {SGI383264, SGI0, SGI0, SGI0, SGI0, SGI383265, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92575[] = {SGI383266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383267, NULL};
const Segment * const SG92576[] = {SGI383268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383269, NULL};
const Segment * const SG92577[] = {SGI383270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383271, NULL};
const Segment * const SG92578[] = {SGI383272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383273, NULL};
const Segment * const SG92579[] = {SGI383274, SGI0, SGI0, SGI0, SGI0, SGI383275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92580[] = {SGI383276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383277, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92581[] = {SGI0, SGI383278, SGI0, SGI0, SGI0, SGI383279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92582[] = {SGI0, SGI383280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383281, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92583[] = {SGI383282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92584[] = {SGI383283, SGI0, SGI0, SGI0, SGI0, SGI383284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92585[] = {SGI383285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92586[] = {SGI383286, SGI383287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92587[] = {SGI383288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92588[] = {SGI383289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92589[] = {SGI383291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92590[] = {SGI383292, SGI0, SGI0, SGI0, SGI0, SGI383293, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92591[] = {SGI383294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383295, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92592[] = {SGI383296, SGI0, SGI0, SGI0, SGI0, SGI383297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92593[] = {SGI383298, SGI383299, SGI0, SGI383300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92594[] = {SGI383301, SGI0, SGI0, SGI383302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92595[] = {SGI383303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383304, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92596[] = {SGI0, SGI383305, SGI0, SGI0, SGI0, SGI383306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92597[] = {SGI0, SGI383307, SGI0, SGI0, SGI0, SGI383308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92598[] = {SGI383309, SGI0, SGI0, SGI0, SGI0, SGI383310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92599[] = {SGI383311, SGI0, SGI0, SGI0, SGI0, SGI383312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92600[] = {SGI383313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92601[] = {SGI0, SGI383314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92602[] = {SGI383315, SGI0, SGI0, SGI0, SGI0, SGI383316, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92603[] = {SGI383317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92604[] = {SGI383318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92605[] = {SGI383319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383320, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92606[] = {SGI383321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92607[] = {SGI383323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92608[] = {SGI383325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92609[] = {SGI383326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383327, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92610[] = {SGI383328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383329, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92611[] = {SGI383330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383331, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92612[] = {SGI383332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92613[] = {SGI383333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92614[] = {SGI0, SGI383334, SGI0, SGI0, SGI0, SGI383335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92615[] = {SGI383336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92616[] = {SGI383337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92617[] = {SGI383339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92618[] = {SGI383340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92619[] = {SGI383341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92620[] = {SGI383342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383343, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92621[] = {SGI383344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92622[] = {SGI383345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92623[] = {SGI383346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92624[] = {SGI0, SGI383347, SGI0, SGI0, SGI0, SGI383348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92625[] = {SGI383349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92626[] = {SGI383350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92627[] = {SGI383351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92628[] = {SGI383352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92629[] = {SGI0, SGI383354, SGI0, SGI0, SGI0, SGI383355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92630[] = {SGI0, SGI383356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383357, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92631[] = {SGI0, SGI383358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383359, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92632[] = {SGI383360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383362, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92633[] = {SGI0, SGI383361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383363, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92634[] = {SGI0, SGI383364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383365, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92635[] = {SGI383366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92636[] = {SGI383367, SGI383368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92637[] = {SGI383369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383370, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92638[] = {SGI383371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92639[] = {SGI383372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92640[] = {SGI383373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92641[] = {SGI383374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92642[] = {SGI383375, SGI0, SGI0, SGI0, SGI0, SGI383376, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92643[] = {SGI383377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92644[] = {SGI383378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383379, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92645[] = {SGI0, SGI383380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383381, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92646[] = {SGI383382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92647[] = {SGI383383, SGI0, SGI0, SGI0, SGI0, SGI383384, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92648[] = {SGI383385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383386, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92649[] = {SGI383387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92650[] = {SGI383388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92651[] = {SGI383390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92652[] = {SGI0, SGI383392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92653[] = {SGI383393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92654[] = {SGI383395, SGI383396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383397, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92655[] = {SGI383398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383399, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92656[] = {SGI383400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92657[] = {SGI0, SGI383401, SGI0, SGI0, SGI0, SGI383402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92658[] = {SGI383403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92659[] = {SGI383404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92660[] = {SGI383405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92661[] = {SGI383406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92662[] = {SGI383407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92663[] = {SGI383408, SGI0, SGI0, SGI0, SGI0, SGI383409, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92664[] = {SGI383410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92665[] = {SGI383412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92666[] = {SGI383414, SGI0, SGI0, SGI0, SGI0, SGI383415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92667[] = {SGI383416, SGI0, SGI0, SGI0, SGI0, SGI383417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92668[] = {SGI383418, SGI383419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383420, SGI383421, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92669[] = {SGI383422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92670[] = {SGI383424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92671[] = {SGI0, SGI383425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383426, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92672[] = {SGI383427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383428, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92673[] = {SGI383429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92674[] = {SGI383431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92675[] = {SGI383433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92676[] = {SGI383435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92677[] = {SGI383437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92678[] = {SGI383438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92679[] = {SGI383439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383440, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92680[] = {SGI383441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383442, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92681[] = {SGI383443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383444, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92682[] = {SGI383445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383446, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92683[] = {SGI383447, SGI0, SGI0, SGI0, SGI0, SGI383448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92684[] = {SGI0, SGI383449, SGI0, SGI0, SGI0, SGI383450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92685[] = {SGI383451, SGI383452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92686[] = {SGI0, SGI383453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92687[] = {SGI383454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92688[] = {SGI383456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92689[] = {SGI383458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383459, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92690[] = {SGI383460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92691[] = {SGI383461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92692[] = {SGI383462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92693[] = {SGI383463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92694[] = {SGI383465, SGI0, SGI383466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92695[] = {SGI383467, SGI0, SGI383468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92696[] = {SGI383469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383470, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92697[] = {SGI383471, SGI0, SGI0, SGI0, SGI0, SGI383472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92698[] = {SGI383473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92699[] = {SGI383474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92700[] = {SGI383476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92701[] = {SGI0, SGI383478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383479, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92702[] = {SGI383480, SGI0, SGI0, SGI0, SGI0, SGI383481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92703[] = {SGI383482, SGI0, SGI0, SGI0, SGI0, SGI383483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92704[] = {SGI383484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92705[] = {SGI383485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92706[] = {SGI383486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92707[] = {SGI383488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92708[] = {SGI383490, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383491, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92709[] = {SGI383492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383493, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92710[] = {SGI383494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383495, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92711[] = {SGI0, SGI383496, SGI0, SGI0, SGI0, SGI383497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92712[] = {SGI0, SGI383498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383499, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92713[] = {SGI383500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92714[] = {SGI383501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92715[] = {SGI383502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92716[] = {SGI383503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92717[] = {SGI383504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92718[] = {SGI0, SGI383505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92719[] = {SGI0, SGI383506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92720[] = {SGI383507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92721[] = {SGI383508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92722[] = {SGI383510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383511, SGI0, SGI0, NULL};
const Segment * const SG92723[] = {SGI383512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383513, SGI0, SGI0, NULL};
const Segment * const SG92724[] = {SGI383514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383515, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92725[] = {SGI0, SGI383516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383517, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92726[] = {SGI383518, SGI383519, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92727[] = {SGI0, SGI383520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92728[] = {SGI383521, SGI0, SGI0, SGI0, SGI0, SGI383522, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92729[] = {SGI383523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92730[] = {SGI383524, SGI0, SGI0, SGI0, SGI0, SGI383525, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92731[] = {SGI383526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383527, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92732[] = {SGI383528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383529, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92733[] = {SGI383530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383531, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92734[] = {SGI383532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383533, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92735[] = {SGI383534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383535, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92736[] = {SGI0, SGI383536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92737[] = {SGI383537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383538, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92738[] = {SGI0, SGI383539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383540, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92739[] = {SGI383541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383542, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92740[] = {SGI383543, SGI0, SGI0, SGI0, SGI0, SGI383544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92741[] = {SGI383545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92742[] = {SGI383546, SGI0, SGI0, SGI0, SGI0, SGI383547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92743[] = {SGI383548, SGI0, SGI0, SGI383549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92744[] = {SGI383550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92745[] = {SGI383551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92746[] = {SGI383552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92747[] = {SGI383553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92748[] = {SGI383554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92749[] = {SGI383556, SGI0, SGI0, SGI0, SGI0, SGI383557, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92750[] = {SGI383558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383559, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92751[] = {SGI383560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383561, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92752[] = {SGI383562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92753[] = {SGI383563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383564, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92754[] = {SGI383565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92755[] = {SGI383566, SGI383567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92756[] = {SGI383568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92757[] = {SGI383570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92758[] = {SGI383572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92759[] = {SGI383574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383575, SGI383576, SGI0, NULL};
const Segment * const SG92760[] = {SGI383577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92761[] = {SGI383579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383580, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92762[] = {SGI383581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92763[] = {SGI383582, SGI0, SGI0, SGI0, SGI0, SGI383583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92764[] = {SGI383584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92765[] = {SGI383585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92766[] = {SGI383586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92767[] = {SGI383588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92768[] = {SGI383589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92769[] = {SGI383591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92770[] = {SGI383593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92771[] = {SGI383595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92772[] = {SGI383597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92773[] = {SGI383598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92774[] = {SGI383600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383601, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92775[] = {SGI383602, SGI0, SGI0, SGI0, SGI0, SGI383603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92776[] = {SGI383604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92777[] = {SGI383605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92778[] = {SGI383606, SGI0, SGI383607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92779[] = {SGI383608, SGI0, SGI383609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92780[] = {SGI383610, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92781[] = {SGI0, SGI383611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383612, NULL};
const Segment * const SG92782[] = {SGI383613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92783[] = {SGI383615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92784[] = {SGI383616, SGI0, SGI0, SGI0, SGI0, SGI383617, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92785[] = {SGI383618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92786[] = {SGI383620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92787[] = {SGI383622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383623, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92788[] = {SGI383624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92789[] = {SGI0, SGI383625, SGI0, SGI0, SGI0, SGI383626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92790[] = {SGI0, SGI383627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92791[] = {SGI0, SGI383628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92792[] = {SGI0, SGI383629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383630, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92793[] = {SGI383631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92794[] = {SGI383632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92795[] = {SGI383633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383634, SGI383635, SGI0, NULL};
const Segment * const SG92796[] = {SGI383636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92797[] = {SGI383638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92798[] = {SGI383639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92799[] = {SGI383640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92800[] = {SGI383641, SGI383642, SGI0, SGI383643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92801[] = {SGI383644, SGI0, SGI0, SGI383645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92802[] = {SGI383646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92803[] = {SGI383647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383648, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92804[] = {SGI383649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92805[] = {SGI383650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383651, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92806[] = {SGI383652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92807[] = {SGI383653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92808[] = {SGI383654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383655, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92809[] = {SGI0, SGI383656, SGI0, SGI0, SGI0, SGI383657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92810[] = {SGI383658, SGI0, SGI0, SGI0, SGI0, SGI383659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92811[] = {SGI0, SGI383660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92812[] = {SGI0, SGI383661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92813[] = {SGI383662, SGI0, SGI0, SGI0, SGI0, SGI383663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92814[] = {SGI383664, SGI0, SGI0, SGI0, SGI0, SGI383665, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92815[] = {SGI383666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383667, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92816[] = {SGI0, SGI383668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383669, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92817[] = {SGI383670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383671, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92818[] = {SGI0, SGI383672, SGI0, SGI0, SGI0, SGI383673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92819[] = {SGI383674, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383675, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92820[] = {SGI383676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383677, NULL};
const Segment * const SG92821[] = {SGI383678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383679, NULL};
const Segment * const SG92822[] = {SGI383680, SGI0, SGI0, SGI383681, SGI0, SGI383682, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92823[] = {SGI383683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92824[] = {SGI383684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92825[] = {SGI383685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92826[] = {SGI383686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92827[] = {SGI383688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383689, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92828[] = {SGI383690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92829[] = {SGI383692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92830[] = {SGI383694, SGI0, SGI0, SGI0, SGI0, SGI383695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92831[] = {SGI0, SGI383696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92832[] = {SGI383697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92833[] = {SGI383698, SGI0, SGI0, SGI0, SGI0, SGI383699, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92834[] = {SGI383700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92835[] = {SGI383701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92836[] = {SGI383702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92837[] = {SGI383703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92838[] = {SGI383705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92839[] = {SGI383707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92840[] = {SGI383709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92841[] = {SGI383710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383712, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92842[] = {SGI383713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92843[] = {SGI383715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92844[] = {SGI383717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92845[] = {SGI383719, SGI0, SGI0, SGI0, SGI0, SGI383720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92846[] = {SGI0, SGI383721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92847[] = {SGI0, SGI383722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383723, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92848[] = {SGI0, SGI383724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383725, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92849[] = {SGI0, SGI383726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383727, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92850[] = {SGI383728, SGI383729, SGI383730, SGI0, SGI0, SGI383731, NULL};
const Segment * const SG92851[] = {SGI383732, SGI383733, SGI383734, SGI0, SGI0, SGI383735, NULL};
const Segment * const SG92852[] = {SGI383736, SGI383737, SGI383738, SGI0, SGI0, SGI383739, NULL};
const Segment * const SG92853[] = {SGI383740, SGI383741, SGI383742, SGI0, SGI0, SGI383743, NULL};
const Segment * const SG92854[] = {SGI383744, SGI383745, SGI383746, SGI0, SGI0, SGI383747, NULL};
const Segment * const SG92855[] = {SGI383748, SGI383749, SGI383750, SGI0, SGI0, SGI383751, NULL};
const Segment * const SG92856[] = {SGI383752, SGI383753, SGI383754, SGI0, SGI0, SGI383755, NULL};
const Segment * const SG92857[] = {SGI383756, SGI383757, SGI383758, SGI0, SGI0, SGI383759, NULL};
const Segment * const SG92858[] = {SGI383760, SGI383761, SGI383762, SGI0, SGI0, SGI383763, NULL};
const Segment * const SG92859[] = {SGI383764, SGI383765, SGI383766, SGI0, SGI0, SGI383767, NULL};
const Segment * const SG92860[] = {SGI383768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383769, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92861[] = {SGI383770, SGI383771, SGI383772, SGI0, SGI0, SGI383773, NULL};
const Segment * const SG92862[] = {SGI383774, SGI383775, SGI383776, SGI0, SGI0, SGI383777, NULL};
const Segment * const SG92863[] = {SGI383778, SGI383779, SGI383780, SGI0, SGI0, SGI383781, NULL};
const Segment * const SG92864[] = {SGI383782, SGI0, SGI0, SGI0, SGI0, SGI383783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92865[] = {SGI383784, SGI383785, SGI383786, SGI0, SGI0, SGI383787, NULL};
const Segment * const SG92866[] = {SGI383788, SGI383789, SGI383790, SGI0, SGI0, SGI383791, NULL};
const Segment * const SG92867[] = {SGI383792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92868[] = {SGI383793, SGI383794, SGI383795, SGI0, SGI0, SGI383796, NULL};
const Segment * const SG92869[] = {SGI383797, SGI383798, SGI383799, SGI0, SGI0, SGI383800, NULL};
const Segment * const SG92870[] = {SGI383801, SGI383802, SGI383803, SGI0, SGI0, SGI383804, NULL};
const Segment * const SG92871[] = {SGI383805, SGI383806, SGI383807, SGI0, SGI0, SGI383808, NULL};
const Segment * const SG92872[] = {SGI383809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92873[] = {SGI383810, SGI383811, SGI383812, SGI0, SGI0, SGI383813, NULL};
const Segment * const SG92874[] = {SGI383814, SGI383815, SGI383816, SGI0, SGI0, SGI383817, NULL};
const Segment * const SG92875[] = {SGI383818, SGI383819, SGI383820, SGI0, SGI0, SGI383821, NULL};
const Segment * const SG92876[] = {SGI383822, SGI383823, SGI383824, SGI0, SGI0, SGI383825, NULL};
const Segment * const SG92877[] = {SGI383826, SGI383827, SGI383828, SGI0, SGI0, SGI383829, NULL};
const Segment * const SG92878[] = {SGI383830, SGI383831, SGI383832, SGI0, SGI0, SGI383833, NULL};
const Segment * const SG92879[] = {SGI383834, SGI383835, SGI383836, SGI0, SGI0, SGI383837, NULL};
const Segment * const SG92880[] = {SGI383838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92881[] = {SGI383839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92882[] = {SGI383840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92883[] = {SGI383841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92884[] = {SGI383843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92885[] = {SGI383844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92886[] = {SGI383845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92887[] = {SGI383846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92888[] = {SGI383848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92889[] = {SGI383850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92890[] = {SGI383851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92891[] = {SGI383853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92892[] = {SGI383854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383855, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92893[] = {SGI383856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383857, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92894[] = {SGI383858, SGI0, SGI0, SGI0, SGI0, SGI383859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92895[] = {SGI383860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383861, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92896[] = {SGI383862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383863, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92897[] = {SGI383864, SGI0, SGI0, SGI0, SGI0, SGI383865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92898[] = {SGI383866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92899[] = {SGI0, SGI383867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92900[] = {SGI383868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383869, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92901[] = {SGI0, SGI383870, SGI0, SGI0, SGI0, SGI383871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92902[] = {SGI0, SGI383872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92903[] = {SGI383873, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92904[] = {SGI383874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92905[] = {SGI383875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92906[] = {SGI383876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92907[] = {SGI383877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92908[] = {SGI383878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92909[] = {SGI383879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92910[] = {SGI383880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92911[] = {SGI383881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92912[] = {SGI383883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92913[] = {SGI383884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92914[] = {SGI383885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92915[] = {SGI0, SGI383887, SGI0, SGI0, SGI0, SGI0, SGI383888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92916[] = {SGI383889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92917[] = {SGI383890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383891, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92918[] = {SGI383892, SGI383893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92919[] = {SGI383894, SGI0, SGI0, SGI0, SGI0, SGI383895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92920[] = {SGI383896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383897, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92921[] = {SGI383898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92922[] = {SGI0, SGI383899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92923[] = {SGI383900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92924[] = {SGI383901, SGI0, SGI0, SGI0, SGI0, SGI383902, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92925[] = {SGI383903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383904, NULL};
const Segment * const SG92926[] = {SGI383905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92927[] = {SGI383906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92928[] = {SGI383907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92929[] = {SGI383908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92930[] = {SGI383909, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92931[] = {SGI383910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92932[] = {SGI383911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383912, NULL};
const Segment * const SG92933[] = {SGI383913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92934[] = {SGI383914, SGI0, SGI383915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92935[] = {SGI383916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92936[] = {SGI383917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92937[] = {SGI383919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92938[] = {SGI383921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92939[] = {SGI383922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92940[] = {SGI383923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92941[] = {SGI383924, SGI0, SGI0, SGI383925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92942[] = {SGI383926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92943[] = {SGI383928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92944[] = {SGI383930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92945[] = {SGI383931, SGI383932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92946[] = {SGI383933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383934, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92947[] = {SGI383935, SGI383936, SGI0, SGI383937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92948[] = {SGI383938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383939, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92949[] = {SGI0, SGI383940, SGI0, SGI0, SGI0, SGI383941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92950[] = {SGI0, SGI383942, SGI0, SGI0, SGI0, SGI383943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92951[] = {SGI383944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383945, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92952[] = {SGI383946, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383947, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92953[] = {SGI0, SGI383948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92954[] = {SGI0, SGI383949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92955[] = {SGI0, SGI383950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92956[] = {SGI0, SGI383951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92957[] = {SGI0, SGI383952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92958[] = {SGI0, SGI383953, SGI0, SGI0, SGI0, SGI383954, SGI0, SGI0, SGI383955, SGI383956, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92959[] = {SGI383957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383958, NULL};
const Segment * const SG92960[] = {SGI383959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92961[] = {SGI383960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383961, NULL};
const Segment * const SG92962[] = {SGI383962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92963[] = {SGI383964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92964[] = {SGI383965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92965[] = {SGI383966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92966[] = {SGI383968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92967[] = {SGI383970, SGI0, SGI0, SGI0, SGI0, SGI383971, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92968[] = {SGI383972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92969[] = {SGI383974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383975, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92970[] = {SGI383976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383977, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92971[] = {SGI383978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383979, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92972[] = {SGI383980, SGI0, SGI0, SGI0, SGI0, SGI383981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92973[] = {SGI383982, SGI0, SGI0, SGI0, SGI0, SGI383983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92974[] = {SGI383984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383985, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92975[] = {SGI0, SGI383986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92976[] = {SGI383987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383988, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92977[] = {SGI383989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92978[] = {SGI383990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92979[] = {SGI0, SGI383991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383992, NULL};
const Segment * const SG92980[] = {SGI383993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92981[] = {SGI383994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92982[] = {SGI383995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92983[] = {SGI383997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI383998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92984[] = {SGI383999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92985[] = {SGI384000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92986[] = {SGI384001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384002, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92987[] = {SGI384003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384004, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92988[] = {SGI384005, SGI0, SGI0, SGI0, SGI0, SGI384006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92989[] = {SGI384007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92990[] = {SGI384008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92991[] = {SGI0, SGI384009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92992[] = {SGI384010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92993[] = {SGI0, SGI384011, SGI0, SGI0, SGI0, SGI384012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92994[] = {SGI384013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384014, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92995[] = {SGI384015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384016, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92996[] = {SGI384017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92997[] = {SGI384018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92998[] = {SGI384020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG92999[] = {SGI384022, SGI0, SGI0, SGI0, SGI0, SGI384023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93000[] = {SGI384024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384025, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93001[] = {SGI384026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384027, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93002[] = {SGI384028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384029, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93003[] = {SGI0, SGI384030, SGI0, SGI0, SGI0, SGI384031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93004[] = {SGI384032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384033, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93005[] = {SGI384034, SGI384035, SGI0, SGI384036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93006[] = {SGI384037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384038, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93007[] = {SGI384039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384040, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93008[] = {SGI384041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93009[] = {SGI384042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384043, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93010[] = {SGI384044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384045, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93011[] = {SGI384046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384047, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93012[] = {SGI384048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384049, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93013[] = {SGI384050, SGI0, SGI0, SGI0, SGI0, SGI384051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93014[] = {SGI384052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93015[] = {SGI0, SGI384053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93016[] = {SGI0, SGI384054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93017[] = {SGI384055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93018[] = {SGI384056, SGI384057, SGI384058, SGI0, SGI0, SGI384059, NULL};
const Segment * const SG93019[] = {SGI384060, SGI384061, SGI384062, SGI0, SGI0, SGI384063, NULL};
const Segment * const SG93020[] = {SGI384064, SGI384065, SGI384066, SGI0, SGI0, SGI384067, NULL};
const Segment * const SG93021[] = {SGI384068, SGI384069, SGI384070, SGI0, SGI0, SGI384071, NULL};
const Segment * const SG93022[] = {SGI384072, SGI384073, SGI384074, SGI0, SGI0, SGI384075, NULL};
const Segment * const SG93023[] = {SGI384076, SGI384077, SGI384078, SGI0, SGI0, SGI384079, NULL};
const Segment * const SG93024[] = {SGI384080, SGI384081, SGI384082, SGI0, SGI0, SGI384083, NULL};
const Segment * const SG93025[] = {SGI384084, SGI384085, SGI384086, SGI0, SGI0, SGI384087, NULL};
const Segment * const SG93026[] = {SGI384088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93027[] = {SGI384089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93028[] = {SGI384091, SGI0, SGI0, SGI0, SGI0, SGI384092, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93029[] = {SGI384093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93030[] = {SGI384094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93031[] = {SGI384095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93032[] = {SGI384096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93033[] = {SGI384097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93034[] = {SGI0, SGI384098, SGI0, SGI0, SGI0, SGI384099, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93035[] = {SGI0, SGI384100, SGI0, SGI0, SGI0, SGI384101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93036[] = {SGI384102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93037[] = {SGI384103, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384104, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93038[] = {SGI384105, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93039[] = {SGI384106, SGI0, SGI0, SGI0, SGI0, SGI384107, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93040[] = {SGI384108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93041[] = {SGI384109, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93042[] = {SGI0, SGI384110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93043[] = {SGI384111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93044[] = {SGI0, SGI384112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93045[] = {SGI384113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93046[] = {SGI0, SGI384114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93047[] = {SGI0, SGI384115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93048[] = {SGI384116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384117, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93049[] = {SGI384118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384119, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93050[] = {SGI384120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384121, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93051[] = {SGI0, SGI384122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384123, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93052[] = {SGI384124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384125, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93053[] = {SGI384126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384127, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93054[] = {SGI384128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384129, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93055[] = {SGI384130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93056[] = {SGI384131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93057[] = {SGI384132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93058[] = {SGI384134, SGI0, SGI0, SGI0, SGI0, SGI384135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93059[] = {SGI384136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93060[] = {SGI384137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93061[] = {SGI384138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93062[] = {SGI384139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93063[] = {SGI384140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93064[] = {SGI384141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93065[] = {SGI384142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93066[] = {SGI384143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384144, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93067[] = {SGI384145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384146, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93068[] = {SGI384147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384148, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93069[] = {SGI0, SGI384149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93070[] = {SGI384150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93071[] = {SGI384151, SGI0, SGI0, SGI0, SGI0, SGI384152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93072[] = {SGI384153, SGI0, SGI0, SGI0, SGI0, SGI384154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93073[] = {SGI384155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93074[] = {SGI384156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93075[] = {SGI384157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93076[] = {SGI384159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93077[] = {SGI384161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384162, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93078[] = {SGI0, SGI384163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384164, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93079[] = {SGI384165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93080[] = {SGI384166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384167, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93081[] = {SGI0, SGI384168, SGI0, SGI0, SGI0, SGI384169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93082[] = {SGI384170, SGI384171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93083[] = {SGI384172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93084[] = {SGI384173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93085[] = {SGI0, SGI384174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93086[] = {SGI384175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384176, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93087[] = {SGI384177, SGI0, SGI0, SGI0, SGI0, SGI384178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93088[] = {SGI384179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93089[] = {SGI384180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93090[] = {SGI0, SGI384181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93091[] = {SGI384182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93092[] = {SGI384183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93093[] = {SGI384184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93094[] = {SGI384186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93095[] = {SGI384188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93096[] = {SGI384189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93097[] = {SGI384191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384192, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93098[] = {SGI384193, SGI0, SGI0, SGI0, SGI0, SGI384194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93099[] = {SGI0, SGI384195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93100[] = {SGI384196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93101[] = {SGI384197, SGI0, SGI0, SGI0, SGI0, SGI384198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93102[] = {SGI384199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384200, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93103[] = {SGI384201, SGI0, SGI0, SGI0, SGI0, SGI384202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93104[] = {SGI384203, SGI0, SGI0, SGI0, SGI0, SGI384204, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93105[] = {SGI384205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93106[] = {SGI384206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93107[] = {SGI384207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93108[] = {SGI384208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93109[] = {SGI384209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93110[] = {SGI384210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384211, NULL};
const Segment * const SG93111[] = {SGI384212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384213, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93112[] = {SGI384214, SGI0, SGI0, SGI384215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93113[] = {SGI384216, SGI0, SGI0, SGI384217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93114[] = {SGI384218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93115[] = {SGI384220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93116[] = {SGI384222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93117[] = {SGI384224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93118[] = {SGI384226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93119[] = {SGI384228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93120[] = {SGI384230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384231, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93121[] = {SGI384232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93122[] = {SGI384233, SGI0, SGI0, SGI0, SGI0, SGI384234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93123[] = {SGI0, SGI384235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93124[] = {SGI0, SGI384236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93125[] = {SGI384237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93126[] = {SGI0, SGI384238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93127[] = {SGI0, SGI384239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93128[] = {SGI384240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93129[] = {SGI384241, SGI0, SGI0, SGI0, SGI0, SGI384242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93130[] = {SGI384243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384244, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93131[] = {SGI0, SGI384245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93132[] = {SGI0, SGI384246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93133[] = {SGI0, SGI384247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93134[] = {SGI0, SGI384248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384249, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93135[] = {SGI0, SGI384250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384251, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93136[] = {SGI384252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93137[] = {SGI384253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93138[] = {SGI384255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93139[] = {SGI384257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93140[] = {SGI384258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93141[] = {SGI384260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93142[] = {SGI384262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93143[] = {SGI384264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93144[] = {SGI384266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384267, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93145[] = {SGI384268, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384269, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93146[] = {SGI384270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93147[] = {SGI384271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384272, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93148[] = {SGI384273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384274, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93149[] = {SGI0, SGI384275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93150[] = {SGI0, SGI384276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93151[] = {SGI384277, SGI0, SGI0, SGI0, SGI0, SGI384278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93152[] = {SGI384279, SGI0, SGI0, SGI0, SGI0, SGI384280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93153[] = {SGI384281, SGI0, SGI0, SGI0, SGI0, SGI384282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93154[] = {SGI384283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384284, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93155[] = {SGI384285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93156[] = {SGI384286, SGI0, SGI0, SGI0, SGI0, SGI384287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93157[] = {SGI384288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93158[] = {SGI384289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384290, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93159[] = {SGI384291, SGI0, SGI0, SGI0, SGI0, SGI384292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93160[] = {SGI384293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93161[] = {SGI0, SGI384294, SGI0, SGI0, SGI0, SGI384295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93162[] = {SGI0, SGI384296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384297, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93163[] = {SGI384298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93164[] = {SGI384299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93165[] = {SGI384300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93166[] = {SGI384301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93167[] = {SGI384303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93168[] = {SGI384304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384305, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93169[] = {SGI384306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384307, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93170[] = {SGI384308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384309, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93171[] = {SGI0, SGI384310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93172[] = {SGI384311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93173[] = {SGI384312, SGI0, SGI0, SGI0, SGI0, SGI384313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93174[] = {SGI384314, SGI0, SGI0, SGI0, SGI0, SGI384315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93175[] = {SGI384316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384317, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93176[] = {SGI384318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93177[] = {SGI384319, SGI0, SGI0, SGI0, SGI0, SGI384320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93178[] = {SGI384321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384322, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93179[] = {SGI0, SGI384323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384324, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93180[] = {SGI384325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93181[] = {SGI0, SGI384326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93182[] = {SGI384327, SGI0, SGI0, SGI0, SGI0, SGI384328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93183[] = {SGI384329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384330, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93184[] = {SGI384331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93185[] = {SGI384332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93186[] = {SGI384333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93187[] = {SGI384334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93188[] = {SGI384335, SGI0, SGI384336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93189[] = {SGI384337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384338, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93190[] = {SGI384339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93191[] = {SGI384341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93192[] = {SGI384343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93193[] = {SGI384345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93194[] = {SGI384347, SGI0, SGI0, SGI0, SGI0, SGI384348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93195[] = {SGI384349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93196[] = {SGI384350, SGI0, SGI0, SGI0, SGI0, SGI384351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93197[] = {SGI384352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384353, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93198[] = {SGI384354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93199[] = {SGI384355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384356, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93200[] = {SGI384357, SGI0, SGI0, SGI0, SGI0, SGI384358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93201[] = {SGI0, SGI384359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93202[] = {SGI384360, SGI0, SGI0, SGI0, SGI0, SGI384361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93203[] = {SGI384362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384363, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93204[] = {SGI0, SGI384364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384365, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93205[] = {SGI0, SGI384366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384367, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93206[] = {SGI384368, SGI384369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93207[] = {SGI384370, SGI0, SGI0, SGI0, SGI0, SGI384371, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93208[] = {SGI384372, SGI0, SGI0, SGI0, SGI0, SGI384373, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93209[] = {SGI384374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93210[] = {SGI0, SGI384375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93211[] = {SGI0, SGI384376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93212[] = {SGI384377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384378, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93213[] = {SGI384379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384380, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93214[] = {SGI384381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93215[] = {SGI384382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384383, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93216[] = {SGI384384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384385, SGI0, SGI0, NULL};
const Segment * const SG93217[] = {SGI384386, SGI0, SGI0, SGI0, SGI0, SGI384387, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93218[] = {SGI384388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93219[] = {SGI384390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93220[] = {SGI384392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93221[] = {SGI384393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93222[] = {SGI384395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93223[] = {SGI384396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384397, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93224[] = {SGI384398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384399, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93225[] = {SGI384400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384401, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93226[] = {SGI0, SGI384402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93227[] = {SGI0, SGI384403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93228[] = {SGI384404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384405, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93229[] = {SGI0, SGI384406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384407, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93230[] = {SGI0, SGI384408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384409, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93231[] = {SGI384410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384411, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93232[] = {SGI0, SGI384412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384413, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93233[] = {SGI384414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93234[] = {SGI384415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93235[] = {SGI0, SGI384416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93236[] = {SGI0, SGI384417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93237[] = {SGI384418, SGI0, SGI0, SGI0, SGI0, SGI384419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93238[] = {SGI384420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384421, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93239[] = {SGI384422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93240[] = {SGI384423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93241[] = {SGI0, SGI384424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93242[] = {SGI384425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93243[] = {SGI384426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384427, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93244[] = {SGI384428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93245[] = {SGI384429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93246[] = {SGI384431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93247[] = {SGI384433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93248[] = {SGI384435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93249[] = {SGI384437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93250[] = {SGI384439, SGI384440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93251[] = {SGI384441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384442, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93252[] = {SGI384443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384444, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93253[] = {SGI384445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93254[] = {SGI384446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384447, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93255[] = {SGI0, SGI384448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93256[] = {SGI384449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93257[] = {SGI0, SGI384450, SGI0, SGI0, SGI0, SGI384451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93258[] = {SGI0, SGI384452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93259[] = {SGI0, SGI384453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93260[] = {SGI384454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93261[] = {SGI384455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384456, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93262[] = {SGI384457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384458, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93263[] = {SGI384459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93264[] = {SGI384460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384461, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93265[] = {SGI0, SGI384462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93266[] = {SGI0, SGI384463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93267[] = {SGI384464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93268[] = {SGI0, SGI384465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93269[] = {SGI384466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93270[] = {SGI0, SGI384467, SGI0, SGI0, SGI0, SGI384468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93271[] = {SGI0, SGI384469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93272[] = {SGI384470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93273[] = {SGI384471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384472, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93274[] = {SGI384473, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384474, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93275[] = {SGI384475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384476, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93276[] = {SGI384477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384478, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93277[] = {SGI384479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93278[] = {SGI384481, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93279[] = {SGI384483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384484, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93280[] = {SGI384485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93281[] = {SGI384487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93282[] = {SGI384489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384490, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93283[] = {SGI384491, SGI0, SGI0, SGI0, SGI0, SGI384492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93284[] = {SGI384493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93285[] = {SGI384494, SGI0, SGI0, SGI0, SGI0, SGI384495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93286[] = {SGI0, SGI384496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93287[] = {SGI0, SGI384497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93288[] = {SGI384498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384499, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93289[] = {SGI384500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93290[] = {SGI384501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384502, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93291[] = {SGI384503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93292[] = {SGI0, SGI384504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384505, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93293[] = {SGI384506, SGI384507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384508, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93294[] = {SGI0, SGI384509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384510, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93295[] = {SGI384511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93296[] = {SGI384512, SGI0, SGI0, SGI0, SGI0, SGI384513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93297[] = {SGI384514, SGI0, SGI0, SGI0, SGI0, SGI384515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93298[] = {SGI384516, SGI384517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93299[] = {SGI384518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93300[] = {SGI384519, SGI0, SGI0, SGI0, SGI0, SGI384520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93301[] = {SGI384521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93302[] = {SGI384522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384523, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93303[] = {SGI384524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384525, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93304[] = {SGI384526, SGI0, SGI0, SGI0, SGI0, SGI384527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93305[] = {SGI0, SGI384528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93306[] = {SGI384529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93307[] = {SGI384530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93308[] = {SGI384531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93309[] = {SGI384532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93310[] = {SGI384533, SGI0, SGI0, SGI0, SGI0, SGI384534, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93311[] = {SGI384535, SGI0, SGI0, SGI0, SGI0, SGI384536, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93312[] = {SGI384537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93313[] = {SGI384539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93314[] = {SGI384541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93315[] = {SGI384543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93316[] = {SGI384545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93317[] = {SGI384547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93318[] = {SGI384548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93319[] = {SGI384550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93320[] = {SGI384552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93321[] = {SGI384554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93322[] = {SGI384555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384556, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93323[] = {SGI384557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93324[] = {SGI384558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93325[] = {SGI384559, SGI0, SGI384560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93326[] = {SGI384561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93327[] = {SGI384562, SGI0, SGI0, SGI0, SGI0, SGI384563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93328[] = {SGI384564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93329[] = {SGI384565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384566, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93330[] = {SGI384567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384568, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93331[] = {SGI384569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93332[] = {SGI384570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93333[] = {SGI0, SGI384571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384572, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93334[] = {SGI384573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93335[] = {SGI384574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384575, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93336[] = {SGI384576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93337[] = {SGI0, SGI384577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93338[] = {SGI0, SGI384578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93339[] = {SGI384579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384580, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93340[] = {SGI384581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93341[] = {SGI384582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384583, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93342[] = {SGI384584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384585, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93343[] = {SGI0, SGI384586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93344[] = {SGI384587, SGI0, SGI0, SGI0, SGI0, SGI384588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93345[] = {SGI384589, SGI0, SGI0, SGI0, SGI0, SGI384590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93346[] = {SGI0, SGI384591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93347[] = {SGI0, SGI384592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93348[] = {SGI0, SGI384593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93349[] = {SGI0, SGI384594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93350[] = {SGI0, SGI384595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384596, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93351[] = {SGI384597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93352[] = {SGI0, SGI384598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93353[] = {SGI0, SGI384599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93354[] = {SGI384600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93355[] = {SGI0, SGI384601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93356[] = {SGI384602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384603, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93357[] = {SGI0, SGI384604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384605, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93358[] = {SGI384606, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93359[] = {SGI384607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384608, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93360[] = {SGI384609, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384610, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93361[] = {SGI384611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93362[] = {SGI384612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384613, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93363[] = {SGI384614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93364[] = {SGI384616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93365[] = {SGI384618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93366[] = {SGI384620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93367[] = {SGI384622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93368[] = {SGI384624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93369[] = {SGI384626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93370[] = {SGI384628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93371[] = {SGI384630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384631, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93372[] = {SGI384632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384633, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93373[] = {SGI384634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93374[] = {SGI384635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93375[] = {SGI384636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384637, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93376[] = {SGI384638, SGI384639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93377[] = {SGI384640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384641, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93378[] = {SGI384642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93379[] = {SGI384643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93380[] = {SGI384644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384645, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93381[] = {SGI0, SGI384646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93382[] = {SGI384647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384648, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93383[] = {SGI384649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93384[] = {SGI384650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384651, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93385[] = {SGI0, SGI384652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93386[] = {SGI384653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384654, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93387[] = {SGI384655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93388[] = {SGI384656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93389[] = {SGI384657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384658, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93390[] = {SGI384659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93391[] = {SGI0, SGI384660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93392[] = {SGI384661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93393[] = {SGI384662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384663, SGI0, SGI0, NULL};
const Segment * const SG93394[] = {SGI384664, SGI0, SGI0, SGI0, SGI0, SGI384665, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93395[] = {SGI384666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93396[] = {SGI0, SGI384667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93397[] = {SGI0, SGI384668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93398[] = {SGI384669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384670, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93399[] = {SGI384671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384672, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93400[] = {SGI0, SGI384673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384674, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93401[] = {SGI384675, SGI0, SGI0, SGI0, SGI0, SGI384676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93402[] = {SGI384677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93403[] = {SGI0, SGI384678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93404[] = {SGI0, SGI384679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93405[] = {SGI384680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93406[] = {SGI384681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93407[] = {SGI384682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93408[] = {SGI384683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93409[] = {SGI384684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93410[] = {SGI384685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93411[] = {SGI384686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93412[] = {SGI384687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93413[] = {SGI384688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93414[] = {SGI0, SGI384689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93415[] = {SGI384690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384691, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93416[] = {SGI384692, SGI384693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93417[] = {SGI0, SGI384694, SGI0, SGI0, SGI0, SGI384695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93418[] = {SGI384696, SGI384697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93419[] = {SGI0, SGI384698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93420[] = {SGI0, SGI384699, SGI0, SGI0, SGI0, SGI384700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93421[] = {SGI384701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93422[] = {SGI384702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93423[] = {SGI384703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93424[] = {SGI384704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384705, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93425[] = {SGI384706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93426[] = {SGI384707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93427[] = {SGI0, SGI384708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384709, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93428[] = {SGI384710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93429[] = {SGI0, SGI384711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384712, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93430[] = {SGI384713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93431[] = {SGI0, SGI384714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384715, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93432[] = {SGI384716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384717, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93433[] = {SGI384718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384719, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93434[] = {SGI384720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93435[] = {SGI0, SGI384721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384722, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93436[] = {SGI384723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93437[] = {SGI384724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93438[] = {SGI384725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93439[] = {SGI0, SGI384726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384727, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93440[] = {SGI384728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93441[] = {SGI384729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93442[] = {SGI0, SGI384730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384731, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93443[] = {SGI384732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93444[] = {SGI384733, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93445[] = {SGI384734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93446[] = {SGI384735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93447[] = {SGI384736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93448[] = {SGI384737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93449[] = {SGI384738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93450[] = {SGI384740, SGI0, SGI384741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93451[] = {SGI384742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93452[] = {SGI384743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93453[] = {SGI384744, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384745, SGI0, SGI0, NULL};
const Segment * const SG93454[] = {SGI384746, SGI0, SGI0, SGI384747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93455[] = {SGI384748, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93456[] = {SGI384749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384750, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93457[] = {SGI384751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93458[] = {SGI384753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93459[] = {SGI384754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93460[] = {SGI384755, SGI0, SGI0, SGI0, SGI0, SGI384756, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93461[] = {SGI384757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384758, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93462[] = {SGI384759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93463[] = {SGI384760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384761, NULL};
const Segment * const SG93464[] = {SGI384762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93465[] = {SGI384763, SGI0, SGI0, SGI0, SGI0, SGI384764, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93466[] = {SGI384765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93467[] = {SGI384766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93468[] = {SGI384767, SGI0, SGI0, SGI0, SGI0, SGI384768, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93469[] = {SGI384769, SGI0, SGI0, SGI0, SGI0, SGI384770, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93470[] = {SGI384771, SGI0, SGI0, SGI384772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93471[] = {SGI384773, SGI384774, SGI0, SGI384774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93472[] = {SGI384775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384776, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93473[] = {SGI384777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93474[] = {SGI384778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384779, SGI0, SGI0, NULL};
const Segment * const SG93475[] = {SGI384780, SGI0, SGI0, SGI0, SGI0, SGI384781, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93476[] = {SGI384782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384783, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93477[] = {SGI384784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384785, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93478[] = {SGI384786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93479[] = {SGI384787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384788, SGI384789, SGI0, NULL};
const Segment * const SG93480[] = {SGI384790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384791, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93481[] = {SGI384792, SGI0, SGI0, SGI384793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93482[] = {SGI384794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384795, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93483[] = {SGI384796, SGI384797, SGI384798, SGI0, SGI0, SGI384799, NULL};
const Segment * const SG93484[] = {SGI384800, SGI384801, SGI384802, SGI0, SGI0, SGI384803, NULL};
const Segment * const SG93485[] = {SGI384804, SGI0, SGI0, SGI384805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93486[] = {SGI384806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93487[] = {SGI384807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93488[] = {SGI384808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93489[] = {SGI384809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384810, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93490[] = {SGI384811, SGI384812, SGI384813, SGI0, SGI0, SGI384814, NULL};
const Segment * const SG93491[] = {SGI384815, SGI384816, SGI384817, SGI0, SGI0, SGI384818, NULL};
const Segment * const SG93492[] = {SGI384819, SGI384820, SGI384821, SGI0, SGI0, SGI384822, NULL};
const Segment * const SG93493[] = {SGI384823, SGI384824, SGI384825, SGI0, SGI0, SGI384826, NULL};
const Segment * const SG93494[] = {SGI384827, SGI384828, SGI384829, SGI0, SGI0, SGI384830, NULL};
const Segment * const SG93495[] = {SGI384831, SGI384832, SGI384833, SGI0, SGI0, SGI384834, NULL};
const Segment * const SG93496[] = {SGI384835, SGI0, SGI0, SGI0, SGI0, SGI384836, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93497[] = {SGI384837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93498[] = {SGI384838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93499[] = {SGI384839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93500[] = {SGI384840, SGI0, SGI0, SGI0, SGI0, SGI384841, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93501[] = {SGI384842, SGI0, SGI0, SGI384843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93502[] = {SGI384844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93503[] = {SGI384845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93504[] = {SGI384846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93505[] = {SGI384847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384848, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93506[] = {SGI384849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384850, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93507[] = {SGI384851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384852, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93508[] = {SGI384853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384854, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93509[] = {SGI384855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384856, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93510[] = {SGI384857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384858, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93511[] = {SGI384859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93512[] = {SGI384860, SGI0, SGI0, SGI0, SGI0, SGI384861, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93513[] = {SGI384862, SGI0, SGI384863, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93514[] = {SGI384864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93515[] = {SGI384865, SGI0, SGI0, SGI0, SGI0, SGI384866, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93516[] = {SGI384867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384868, SGI0, SGI0, NULL};
const Segment * const SG93517[] = {SGI384869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384870, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93518[] = {SGI384871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93519[] = {SGI384872, SGI0, SGI0, SGI0, SGI0, SGI384873, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93520[] = {SGI384874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93521[] = {SGI384875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93522[] = {SGI384876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93523[] = {SGI384877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93524[] = {SGI384878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93525[] = {SGI384879, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93526[] = {SGI384880, SGI0, SGI0, SGI0, SGI0, SGI384881, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93527[] = {SGI384882, SGI0, SGI0, SGI0, SGI0, SGI384883, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93528[] = {SGI384884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384885, SGI0, SGI0, NULL};
const Segment * const SG93529[] = {SGI384886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93530[] = {SGI384887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384888, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93531[] = {SGI384889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93532[] = {SGI384890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384891, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93533[] = {SGI384892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93534[] = {SGI384893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384894, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93535[] = {SGI384895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93536[] = {SGI384896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93537[] = {SGI384897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93538[] = {SGI384898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384899, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93539[] = {SGI384900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93540[] = {SGI384901, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93541[] = {SGI384902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93542[] = {SGI384903, SGI384904, SGI384905, SGI0, SGI0, SGI384906, NULL};
const Segment * const SG93543[] = {SGI384907, SGI384908, SGI384909, SGI0, SGI0, SGI384910, NULL};
const Segment * const SG93544[] = {SGI384911, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93545[] = {SGI384912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93546[] = {SGI384913, SGI0, SGI0, SGI0, SGI0, SGI384914, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93547[] = {SGI384915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93548[] = {SGI384916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93549[] = {SGI384917, SGI0, SGI0, SGI0, SGI0, SGI384918, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93550[] = {SGI384919, SGI0, SGI0, SGI384920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93551[] = {SGI384921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93552[] = {SGI384922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93553[] = {SGI384923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93554[] = {SGI384924, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384925, NULL};
const Segment * const SG93555[] = {SGI384926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93556[] = {SGI384927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384928, NULL};
const Segment * const SG93557[] = {SGI384929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93558[] = {SGI384930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93559[] = {SGI384931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384932, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93560[] = {SGI384933, SGI0, SGI0, SGI0, SGI0, SGI384934, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93561[] = {SGI384935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93562[] = {SGI384936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93563[] = {SGI384937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93564[] = {SGI384938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93565[] = {SGI384939, SGI0, SGI0, SGI0, SGI0, SGI384940, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93566[] = {SGI384941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93567[] = {SGI384942, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93568[] = {SGI384943, SGI0, SGI0, SGI0, SGI0, SGI384944, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93569[] = {SGI384945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93570[] = {SGI384946, SGI0, SGI0, SGI384947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93571[] = {SGI384948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93572[] = {SGI384949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93573[] = {SGI384950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93574[] = {SGI384951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384952, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93575[] = {SGI384953, SGI0, SGI0, SGI384954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93576[] = {SGI384955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93577[] = {SGI384956, SGI0, SGI0, SGI0, SGI0, SGI384957, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93578[] = {SGI384958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93579[] = {SGI384959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93580[] = {SGI384960, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384961, NULL};
const Segment * const SG93581[] = {SGI384962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384963, SGI0, SGI0, NULL};
const Segment * const SG93582[] = {SGI384964, SGI0, SGI384965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93583[] = {SGI384966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93584[] = {SGI384967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93585[] = {SGI384968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93586[] = {SGI384969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93587[] = {SGI384970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93588[] = {SGI384971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93589[] = {SGI384972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93590[] = {SGI384973, SGI0, SGI0, SGI0, SGI0, SGI384974, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93591[] = {SGI384975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93592[] = {SGI384976, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93593[] = {SGI384977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93594[] = {SGI384978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384979, NULL};
const Segment * const SG93595[] = {SGI384980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93596[] = {SGI384981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93597[] = {SGI384982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93598[] = {SGI384983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93599[] = {SGI384984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93600[] = {SGI384985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93601[] = {SGI384986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93602[] = {SGI384987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93603[] = {SGI384988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384989, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93604[] = {SGI384990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93605[] = {SGI384991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93606[] = {SGI384992, SGI0, SGI0, SGI0, SGI0, SGI384993, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93607[] = {SGI384994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384995, SGI0, SGI0, NULL};
const Segment * const SG93608[] = {SGI384996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93609[] = {SGI384997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93610[] = {SGI384998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI384999, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93611[] = {SGI385000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI385001, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93612[] = {SGI385002, SGI0, SGI385003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93613[] = {SGI385004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93614[] = {SGI385005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93615[] = {SGI385006, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93616[] = {SGI385007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI385008, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93617[] = {SGI385009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93618[] = {SGI385010, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93619[] = {SGI385011, SGI0, SGI0, SGI385012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93620[] = {SGI385013, SGI0, SGI0, SGI0, SGI0, SGI385014, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93621[] = {SGI385015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI385016, SGI0, SGI0, NULL};
const Segment * const SG93622[] = {SGI385017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI385018, NULL};
const Segment * const SG93623[] = {SGI385019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93624[] = {SGI385020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93625[] = {SGI385021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93626[] = {SGI385022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93627[] = {SGI385023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93628[] = {SGI385024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93629[] = {SGI385025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93630[] = {SGI385026, SGI0, SGI0, SGI0, SGI0, SGI385027, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93631[] = {SGI385028, SGI385029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93632[] = {SGI385030, SGI0, SGI0, SGI0, SGI0, SGI385031, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93633[] = {SGI385032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93634[] = {SGI385033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI385034, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93635[] = {SGI385035, SGI385036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93636[] = {SGI385037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93637[] = {SGI385038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93638[] = {SGI385039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI385040, SGI385041, SGI0, NULL};
const Segment * const SG93639[] = {SGI385042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93640[] = {SGI385043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93641[] = {SGI385044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93642[] = {SGI385045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93643[] = {SGI385046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93644[] = {SGI385047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93645[] = {SGI385048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93646[] = {SGI385049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93647[] = {SGI385050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93648[] = {SGI385051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93649[] = {SGI385052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93650[] = {SGI385053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93651[] = {SGI385054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93652[] = {SGI385055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93653[] = {SGI385056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93654[] = {SGI385057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93655[] = {SGI385058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93656[] = {SGI385059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93657[] = {SGI385060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93658[] = {SGI385061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93659[] = {SGI385062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93660[] = {SGI385063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93661[] = {SGI385064, SGI385065, SGI385066, SGI0, SGI0, SGI385067, NULL};
const Segment * const SG93662[] = {SGI385068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93663[] = {SGI385069, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93664[] = {SGI385070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93665[] = {SGI385071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93666[] = {SGI385072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93667[] = {SGI385073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93668[] = {SGI385074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93669[] = {SGI385075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93670[] = {SGI385076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93671[] = {SGI385077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93672[] = {SGI385078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93673[] = {SGI385079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93674[] = {SGI385080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93675[] = {SGI385081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93676[] = {SGI385082, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93677[] = {SGI385083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93678[] = {SGI385084, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93679[] = {SGI385085, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93680[] = {SGI385086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93681[] = {SGI385087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93682[] = {SGI385088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93683[] = {SGI385089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93684[] = {SGI385090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93685[] = {SGI385091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93686[] = {SGI385092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93687[] = {SGI385093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93688[] = {SGI385094, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93689[] = {SGI385095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93690[] = {SGI385096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93691[] = {SGI385097, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93692[] = {SGI385098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93693[] = {SGI385099, SGI385100, SGI385101, SGI0, SGI0, SGI385102, NULL};
const Segment * const SG93694[] = {SGI385103, SGI385104, SGI385105, SGI0, SGI0, SGI385106, NULL};
const Segment * const SG93695[] = {SGI385107, SGI385108, SGI385109, SGI0, SGI0, SGI385110, NULL};
const Segment * const SG93696[] = {SGI385111, SGI385112, SGI385113, SGI0, SGI0, SGI385114, NULL};
const Segment * const SG93697[] = {SGI385115, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93698[] = {SGI385116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93699[] = {SGI385117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93700[] = {SGI385118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93701[] = {SGI385119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93702[] = {SGI385120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93703[] = {SGI385121, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93704[] = {SGI385122, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93705[] = {SGI385123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93706[] = {SGI385124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93707[] = {SGI385125, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93708[] = {SGI385126, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93709[] = {SGI385127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93710[] = {SGI385128, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93711[] = {SGI385129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93712[] = {SGI385130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93713[] = {SGI385131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93714[] = {SGI385132, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93715[] = {SGI385133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93716[] = {SGI385134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93717[] = {SGI385135, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93718[] = {SGI385136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93719[] = {SGI385137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93720[] = {SGI385138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93721[] = {SGI385139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93722[] = {SGI385140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93723[] = {SGI385141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93724[] = {SGI385142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93725[] = {SGI385143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93726[] = {SGI385144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93727[] = {SGI385145, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93728[] = {SGI385146, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93729[] = {SGI385147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93730[] = {SGI385148, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93731[] = {SGI385149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93732[] = {SGI385150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93733[] = {SGI385151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93734[] = {SGI385152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93735[] = {SGI385153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93736[] = {SGI385154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93737[] = {SGI385155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93738[] = {SGI385156, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93739[] = {SGI385157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93740[] = {SGI385158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93741[] = {SGI385159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93742[] = {SGI385160, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93743[] = {SGI385161, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93744[] = {SGI385162, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93745[] = {SGI385163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93746[] = {SGI385164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93747[] = {SGI385165, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93748[] = {SGI385166, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93749[] = {SGI385167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93750[] = {SGI385168, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93751[] = {SGI385169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93752[] = {SGI385170, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93753[] = {SGI385171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93754[] = {SGI385172, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93755[] = {SGI385173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93756[] = {SGI385174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93757[] = {SGI385175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93758[] = {SGI385176, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93759[] = {SGI385177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93760[] = {SGI385178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93761[] = {SGI385179, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93762[] = {SGI385180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93763[] = {SGI385181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93764[] = {SGI385182, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93765[] = {SGI385183, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93766[] = {SGI385184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93767[] = {SGI385185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93768[] = {SGI385186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93769[] = {SGI385187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93770[] = {SGI385188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93771[] = {SGI385189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93772[] = {SGI385190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93773[] = {SGI385191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93774[] = {SGI385192, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93775[] = {SGI385193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93776[] = {SGI385194, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93777[] = {SGI385195, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93778[] = {SGI385196, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93779[] = {SGI385197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93780[] = {SGI385198, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93781[] = {SGI385199, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93782[] = {SGI385200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93783[] = {SGI385201, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93784[] = {SGI385202, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93785[] = {SGI385203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93786[] = {SGI385204, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93787[] = {SGI385205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93788[] = {SGI385206, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93789[] = {SGI385207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93790[] = {SGI385208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93791[] = {SGI385209, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93792[] = {SGI385210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93793[] = {SGI385211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93794[] = {SGI385212, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93795[] = {SGI385213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93796[] = {SGI385214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93797[] = {SGI385215, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93798[] = {SGI385216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93799[] = {SGI385217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93800[] = {SGI385218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93801[] = {SGI385219, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93802[] = {SGI385220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93803[] = {SGI385221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93804[] = {SGI385222, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93805[] = {SGI385223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93806[] = {SGI385224, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93807[] = {SGI385225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93808[] = {SGI385226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93809[] = {SGI385227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93810[] = {SGI385228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93811[] = {SGI385229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93812[] = {SGI385230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93813[] = {SGI385231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93814[] = {SGI385232, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93815[] = {SGI385233, NULL};
const Segment * const SG93816[] = {SGI385234, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93817[] = {SGI385235, NULL};
const Segment * const SG93818[] = {SGI385236, NULL};
const Segment * const SG93819[] = {SGI385237, NULL};
const Segment * const SG93820[] = {SGI385238, NULL};
const Segment * const SG93821[] = {SGI385239, NULL};
const Segment * const SG93822[] = {SGI385240, NULL};
const Segment * const SG93823[] = {SGI385241, NULL};
const Segment * const SG93824[] = {SGI385242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93825[] = {SGI385243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93826[] = {SGI385244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93827[] = {SGI385245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93828[] = {SGI385246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93829[] = {SGI385247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93830[] = {SGI385248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93831[] = {SGI385249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93832[] = {SGI385250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93833[] = {SGI385251, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93834[] = {SGI385252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93835[] = {SGI385253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93836[] = {SGI385254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93837[] = {SGI385255, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93838[] = {SGI385256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93839[] = {SGI385257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93840[] = {SGI385258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93841[] = {SGI385259, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93842[] = {SGI385260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93843[] = {SGI385261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93844[] = {SGI385262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93845[] = {SGI385263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93846[] = {SGI385264, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93847[] = {SGI385265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93848[] = {SGI385266, NULL};
const Segment * const SG93849[] = {SGI385267, NULL};
const Segment * const SG93850[] = {SGI385268, NULL};
const Segment * const SG93851[] = {SGI385269, NULL};
const Segment * const SG93852[] = {SGI385270, NULL};
const Segment * const SG93853[] = {SGI385271, NULL};
const Segment * const SG93854[] = {SGI385272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93855[] = {SGI385273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93856[] = {SGI385274, NULL};
const Segment * const SG93857[] = {SGI385275, NULL};
const Segment * const SG93858[] = {SGI385276, NULL};
const Segment * const SG93859[] = {SGI385277, NULL};
const Segment * const SG93860[] = {SGI385278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93861[] = {SGI385279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93862[] = {SGI385280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93863[] = {SGI385281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93864[] = {SGI385282, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93865[] = {SGI385283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93866[] = {SGI385284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93867[] = {SGI385285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93868[] = {SGI385286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93869[] = {SGI385287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93870[] = {SGI385288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93871[] = {SGI385289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93872[] = {SGI385290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93873[] = {SGI385291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93874[] = {SGI385292, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93875[] = {SGI385293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93876[] = {SGI385294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93877[] = {SGI385295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93878[] = {SGI385296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93879[] = {SGI385297, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93880[] = {SGI385298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93881[] = {SGI385299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93882[] = {SGI385300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93883[] = {SGI385301, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93884[] = {SGI385302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93885[] = {SGI385303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93886[] = {SGI385304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93887[] = {SGI385305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93888[] = {SGI385306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93889[] = {SGI385307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93890[] = {SGI385308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93891[] = {SGI385309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93892[] = {SGI385310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93893[] = {SGI385311, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93894[] = {SGI385312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93895[] = {SGI385313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93896[] = {SGI385314, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93897[] = {SGI385315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93898[] = {SGI385316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93899[] = {SGI385317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93900[] = {SGI385318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93901[] = {SGI385319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93902[] = {SGI385320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93903[] = {SGI385321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93904[] = {SGI385322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93905[] = {SGI385323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93906[] = {SGI385324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93907[] = {SGI385325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93908[] = {SGI385326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93909[] = {SGI385327, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93910[] = {SGI385328, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93911[] = {SGI385329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93912[] = {SGI385330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93913[] = {SGI385331, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93914[] = {SGI385332, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93915[] = {SGI385333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93916[] = {SGI385334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93917[] = {SGI385335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93918[] = {SGI385336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93919[] = {SGI385337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93920[] = {SGI385338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93921[] = {SGI385339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93922[] = {SGI385340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93923[] = {SGI385341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93924[] = {SGI385342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93925[] = {SGI385343, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93926[] = {SGI385344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93927[] = {SGI385345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93928[] = {SGI385346, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93929[] = {SGI385347, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93930[] = {SGI385348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93931[] = {SGI385349, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93932[] = {SGI385350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93933[] = {SGI385351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93934[] = {SGI385352, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93935[] = {SGI385353, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93936[] = {SGI385354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93937[] = {SGI385355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93938[] = {SGI385356, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93939[] = {SGI385357, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93940[] = {SGI385358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93941[] = {SGI385359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93942[] = {SGI385360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93943[] = {SGI385361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93944[] = {SGI385362, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93945[] = {SGI385363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93946[] = {SGI385364, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93947[] = {SGI385365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93948[] = {SGI385366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93949[] = {SGI385367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93950[] = {SGI385368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93951[] = {SGI385369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93952[] = {SGI385370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93953[] = {SGI385371, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93954[] = {SGI385372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93955[] = {SGI385373, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93956[] = {SGI385374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93957[] = {SGI385375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93958[] = {SGI385376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93959[] = {SGI385377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93960[] = {SGI385378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93961[] = {SGI385379, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93962[] = {SGI385380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93963[] = {SGI385381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93964[] = {SGI385382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93965[] = {SGI385383, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93966[] = {SGI385384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93967[] = {SGI385385, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93968[] = {SGI385386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93969[] = {SGI385387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93970[] = {SGI385388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93971[] = {SGI385389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93972[] = {SGI385390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93973[] = {SGI385391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93974[] = {SGI385392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93975[] = {SGI385393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93976[] = {SGI385394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93977[] = {SGI385395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93978[] = {SGI385396, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93979[] = {SGI385397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93980[] = {SGI385398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93981[] = {SGI385399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93982[] = {SGI385400, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93983[] = {SGI385401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93984[] = {SGI385402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93985[] = {SGI385403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93986[] = {SGI385404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93987[] = {SGI385405, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93988[] = {SGI385406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93989[] = {SGI385407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93990[] = {SGI385408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93991[] = {SGI385409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93992[] = {SGI385410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG93993[] = {SGI385411, NULL};
const Segment * const SG93994[] = {SGI385412, NULL};
const Segment * const SG93995[] = {SGI385413, NULL};
const Segment * const SG93996[] = {SGI385414, NULL};
const Segment * const SG93997[] = {SGI385415, NULL};
const Segment * const SG93998[] = {SGI385416, NULL};
const Segment * const SG93999[] = {SGI385417, NULL};
const Segment * const SG94000[] = {SGI385418, NULL};
const Segment * const SG94001[] = {SGI385419, NULL};
const Segment * const SG94002[] = {SGI385420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94003[] = {SGI385421, NULL};
const Segment * const SG94004[] = {SGI385422, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94005[] = {SGI385423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94006[] = {SGI385424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94007[] = {SGI385425, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94008[] = {SGI385426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94009[] = {SGI385427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94010[] = {SGI385428, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94011[] = {SGI385429, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94012[] = {SGI385430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94013[] = {SGI385431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94014[] = {SGI385432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94015[] = {SGI385433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94016[] = {SGI385434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94017[] = {SGI385435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94018[] = {SGI385436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94019[] = {SGI385437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94020[] = {SGI385438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94021[] = {SGI385439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94022[] = {SGI385440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94023[] = {SGI385441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94024[] = {SGI385442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94025[] = {SGI385443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94026[] = {SGI385444, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94027[] = {SGI385445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94028[] = {SGI385446, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94029[] = {SGI385447, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94030[] = {SGI385448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94031[] = {SGI385449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94032[] = {SGI385450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94033[] = {SGI385451, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94034[] = {SGI385452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94035[] = {SGI385453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94036[] = {SGI385454, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94037[] = {SGI385455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94038[] = {SGI385456, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94039[] = {SGI385457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94040[] = {SGI385458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94041[] = {SGI385459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94042[] = {SGI385460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94043[] = {SGI385461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94044[] = {SGI385462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94045[] = {SGI385463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94046[] = {SGI385464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94047[] = {SGI385465, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94048[] = {SGI385466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94049[] = {SGI385467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94050[] = {SGI385468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94051[] = {SGI385469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94052[] = {SGI385470, NULL};
const Segment * const SG94053[] = {SGI385471, NULL};
const Segment * const SG94054[] = {SGI385472, NULL};
const Segment * const SG94055[] = {SGI385473, NULL};
const Segment * const SG94056[] = {SGI385474, NULL};
const Segment * const SG94057[] = {SGI385475, NULL};
const Segment * const SG94058[] = {SGI385476, NULL};
const Segment * const SG94059[] = {SGI385477, NULL};
const Segment * const SG94060[] = {SGI385478, NULL};
const Segment * const SG94061[] = {SGI385479, NULL};
const Segment * const SG94062[] = {SGI385480, NULL};
const Segment * const SG94063[] = {SGI385481, NULL};
const Segment * const SG94064[] = {SGI385482, NULL};
const Segment * const SG94065[] = {SGI385483, NULL};
const Segment * const SG94066[] = {SGI385484, NULL};
const Segment * const SG94067[] = {SGI385485, NULL};
const Segment * const SG94068[] = {SGI385486, NULL};
const Segment * const SG94069[] = {SGI385487, NULL};
const Segment * const SG94070[] = {SGI385488, NULL};
const Segment * const SG94071[] = {SGI385489, NULL};
const Segment * const SG94072[] = {SGI385490, NULL};
const Segment * const SG94073[] = {SGI385491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94074[] = {SGI385492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94075[] = {SGI385493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94076[] = {SGI385494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94077[] = {SGI385495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94078[] = {SGI385496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94079[] = {SGI385497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94080[] = {SGI385498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94081[] = {SGI385499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94082[] = {SGI385500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94083[] = {SGI385501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94084[] = {SGI385502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94085[] = {SGI385503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94086[] = {SGI385504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94087[] = {SGI385505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94088[] = {SGI385506, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94089[] = {SGI385507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94090[] = {SGI385508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94091[] = {SGI385509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94092[] = {SGI385510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94093[] = {SGI385511, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94094[] = {SGI385512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94095[] = {SGI385513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94096[] = {SGI385514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94097[] = {SGI385515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94098[] = {SGI385516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94099[] = {SGI385517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94100[] = {SGI385518, NULL};
const Segment * const SG94101[] = {SGI385519, NULL};
const Segment * const SG94102[] = {SGI385520, NULL};
const Segment * const SG94103[] = {SGI385521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94104[] = {SGI385522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94105[] = {SGI385523, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94106[] = {SGI385524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94107[] = {SGI385525, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94108[] = {SGI385526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94109[] = {SGI385527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94110[] = {SGI385528, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94111[] = {SGI385529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94112[] = {SGI385530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94113[] = {SGI385531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94114[] = {SGI385532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94115[] = {SGI385533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94116[] = {SGI385534, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94117[] = {SGI385535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94118[] = {SGI385536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94119[] = {SGI385537, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94120[] = {SGI385538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94121[] = {SGI385539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94122[] = {SGI385540, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94123[] = {SGI385541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94124[] = {SGI385542, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94125[] = {SGI385543, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94126[] = {SGI385544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94127[] = {SGI385545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94128[] = {SGI385546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94129[] = {SGI385547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94130[] = {SGI385548, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94131[] = {SGI385549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94132[] = {SGI385550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94133[] = {SGI385551, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94134[] = {SGI385552, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94135[] = {SGI385553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94136[] = {SGI385554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94137[] = {SGI385555, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94138[] = {SGI385556, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94139[] = {SGI385557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94140[] = {SGI385558, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94141[] = {SGI385559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94142[] = {SGI385560, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94143[] = {SGI385561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94144[] = {SGI385562, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94145[] = {SGI385563, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94146[] = {SGI385564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94147[] = {SGI385565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94148[] = {SGI385566, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94149[] = {SGI385567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94150[] = {SGI385568, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94151[] = {SGI385569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94152[] = {SGI385570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94153[] = {SGI385571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94154[] = {SGI385572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94155[] = {SGI385573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94156[] = {SGI385574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94157[] = {SGI385575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94158[] = {SGI385576, NULL};
const Segment * const SG94159[] = {SGI385577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94160[] = {SGI385578, NULL};
const Segment * const SG94161[] = {SGI385579, NULL};
const Segment * const SG94162[] = {SGI385581, NULL};
const Segment * const SG94163[] = {SGI385580, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94164[] = {SGI385582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94165[] = {SGI385583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94166[] = {SGI385584, NULL};
const Segment * const SG94167[] = {SGI385585, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94168[] = {SGI385586, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94169[] = {SGI385587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94170[] = {SGI385588, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94171[] = {SGI385589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94172[] = {SGI385590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94173[] = {SGI385591, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94174[] = {SGI385592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94175[] = {SGI385593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94176[] = {SGI385594, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94177[] = {SGI385595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94178[] = {SGI385596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94179[] = {SGI385597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94180[] = {SGI385598, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94181[] = {SGI385599, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94182[] = {SGI385600, NULL};
const Segment * const SG94183[] = {SGI385601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94184[] = {SGI385602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94185[] = {SGI385603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94186[] = {SGI385604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94187[] = {SGI385605, NULL};
const Segment * const SG94188[] = {SGI385606, NULL};
const Segment * const SG94189[] = {SGI385607, NULL};
const Segment * const SG94190[] = {SGI385608, NULL};
const Segment * const SG94191[] = {SGI385609, NULL};
const Segment * const SG94192[] = {SGI385610, NULL};
const Segment * const SG94193[] = {SGI385611, NULL};
const Segment * const SG94194[] = {SGI385612, NULL};
const Segment * const SG94195[] = {SGI385613, NULL};
const Segment * const SG94196[] = {SGI385614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94197[] = {SGI385615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94198[] = {SGI385616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94199[] = {SGI385617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94200[] = {SGI385618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94201[] = {SGI385619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94202[] = {SGI385620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94203[] = {SGI385621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94204[] = {SGI385622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94205[] = {SGI385623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94206[] = {SGI385624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94207[] = {SGI385625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94208[] = {SGI385626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94209[] = {SGI385627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94210[] = {SGI385628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94211[] = {SGI385629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94212[] = {SGI385630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94213[] = {SGI385631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94214[] = {SGI385632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94215[] = {SGI385633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94216[] = {SGI385634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94217[] = {SGI385635, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94218[] = {SGI385636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94219[] = {SGI385637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94220[] = {SGI385638, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94221[] = {SGI385639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94222[] = {SGI385640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94223[] = {SGI385641, NULL};
const Segment * const SG94224[] = {SGI385642, NULL};
const Segment * const SG94225[] = {SGI385643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94226[] = {SGI385644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94227[] = {SGI385645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94228[] = {SGI385646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94229[] = {SGI385647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94230[] = {SGI385648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94231[] = {SGI385649, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94232[] = {SGI385650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94233[] = {SGI385651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI385652, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94234[] = {SGI385653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI385654, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94235[] = {SGI385655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI385656, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94236[] = {SGI385657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI385658, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94237[] = {SGI385659, NULL};
const Segment * const SG94238[] = {SGI385660, NULL};
const Segment * const SG94239[] = {SGI385661, NULL};
const Segment * const SG94240[] = {SGI385662, NULL};
const Segment * const SG94241[] = {SGI385663, NULL};
const Segment * const SG94242[] = {SGI385664, NULL};
const Segment * const SG94243[] = {SGI385665, NULL};
const Segment * const SG94244[] = {SGI385666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94245[] = {SGI385667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94246[] = {SGI385668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94247[] = {SGI385669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94248[] = {SGI385670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94249[] = {SGI385671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI385672, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94250[] = {SGI385673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI385674, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94251[] = {SGI385675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI385676, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94252[] = {SGI385677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94253[] = {SGI385678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94254[] = {SGI385679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94255[] = {SGI385680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94256[] = {SGI385681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94257[] = {SGI385682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94258[] = {SGI385683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94259[] = {SGI385684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94260[] = {SGI385685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94261[] = {SGI385686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94262[] = {SGI385687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94263[] = {SGI385688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94264[] = {SGI385689, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94265[] = {SGI385690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94266[] = {SGI385691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94267[] = {SGI385692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94268[] = {SGI385693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94269[] = {SGI385694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94270[] = {SGI385695, SGI385696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94271[] = {SGI385697, SGI385698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94272[] = {SGI385699, NULL};
const Segment * const SG94273[] = {SGI385700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94274[] = {SGI385701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94275[] = {SGI385702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94276[] = {SGI385703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94277[] = {SGI385704, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94278[] = {SGI385705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94279[] = {SGI385706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94280[] = {SGI385707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94281[] = {SGI385708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94282[] = {SGI385709, NULL};
const Segment * const SG94283[] = {SGI385710, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94284[] = {SGI385711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94285[] = {SGI385712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94286[] = {SGI385713, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94287[] = {SGI385714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94288[] = {SGI385715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94289[] = {SGI385716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94290[] = {SGI385717, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94291[] = {SGI385718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94292[] = {SGI385719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94293[] = {SGI385720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94294[] = {SGI385721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94295[] = {SGI385722, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94296[] = {SGI385723, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94297[] = {SGI385724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94298[] = {SGI385725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94299[] = {SGI385726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94300[] = {SGI385727, NULL};
const Segment * const SG94301[] = {SGI385728, NULL};
const Segment * const SG94302[] = {SGI385729, NULL};
const Segment * const SG94303[] = {SGI385730, NULL};
const Segment * const SG94304[] = {SGI385731, NULL};
const Segment * const SG94305[] = {SGI385732, NULL};
const Segment * const SG94306[] = {SGI385733, SGI385734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94307[] = {SGI385735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94308[] = {SGI385736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94309[] = {SGI385737, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94310[] = {SGI385738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94311[] = {SGI385739, NULL};
const Segment * const SG94312[] = {SGI385740, SGI385741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94313[] = {SGI385742, SGI385743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94314[] = {SGI385744, SGI385745, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94315[] = {SGI385746, NULL};
const Segment * const SG94316[] = {SGI385747, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94317[] = {SGI385748, NULL};
const Segment * const SG94318[] = {SGI385749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94319[] = {SGI385750, SGI385751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94320[] = {SGI385752, SGI385753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94321[] = {SGI385754, SGI385755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94322[] = {SGI385756, SGI385757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94323[] = {SGI385758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94324[] = {SGI385759, SGI385760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94325[] = {SGI385761, NULL};
const Segment * const SG94326[] = {SGI385762, NULL};
const Segment * const SG94327[] = {SGI385763, NULL};
const Segment * const SG94328[] = {SGI385764, NULL};
const Segment * const SG94329[] = {SGI385765, NULL};
const Segment * const SG94330[] = {SGI385766, NULL};
const Segment * const SG94331[] = {SGI385767, NULL};
const Segment * const SG94332[] = {SGI385768, SGI385769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94333[] = {SGI385770, SGI385771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94334[] = {SGI385772, NULL};
const Segment * const SG94335[] = {SGI385773, SGI385774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94336[] = {SGI385775, SGI385776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94337[] = {SGI385777, SGI385778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94338[] = {SGI385779, SGI385780, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94339[] = {SGI385781, SGI385782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94340[] = {SGI385783, NULL};
const Segment * const SG94341[] = {SGI385784, NULL};
const Segment * const SG94342[] = {SGI385785, NULL};
const Segment * const SG94343[] = {SGI385786, SGI0, SGI0, SGI0, SGI385787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94344[] = {SGI385788, SGI0, SGI0, SGI0, SGI385789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94345[] = {SGI385790, SGI0, SGI0, SGI0, SGI385791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94346[] = {SGI385792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94347[] = {SGI385793, SGI385794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94348[] = {SGI385795, SGI385796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94349[] = {SGI385797, SGI385798, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94350[] = {SGI385799, SGI385800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94351[] = {SGI385801, SGI385802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94352[] = {SGI385803, SGI385804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94353[] = {SGI385805, SGI385806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94354[] = {SGI385807, SGI385808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94355[] = {SGI385809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94356[] = {SGI385810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94357[] = {SGI385811, SGI0, SGI0, SGI0, SGI385812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94358[] = {SGI385813, SGI385814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94359[] = {SGI385815, SGI385816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94360[] = {SGI385817, SGI0, SGI0, SGI0, SGI0, SGI0, SGI385818, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94361[] = {SGI385819, SGI385820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94362[] = {SGI385821, SGI385822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94363[] = {SGI385823, SGI385824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94364[] = {SGI385825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94365[] = {SGI385826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94366[] = {SGI385827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94367[] = {SGI385828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94368[] = {SGI385829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94369[] = {SGI385830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94370[] = {SGI385831, SGI0, SGI385832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94371[] = {SGI385833, SGI385834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94372[] = {SGI385835, SGI385836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94373[] = {SGI385837, SGI385838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94374[] = {SGI385839, SGI385840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94375[] = {SGI385841, SGI385842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94376[] = {SGI385843, SGI385844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94377[] = {SGI385845, SGI0, SGI0, SGI385846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94378[] = {SGI385847, SGI0, SGI0, SGI385848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94379[] = {SGI385849, SGI0, SGI0, SGI385850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94380[] = {SGI385851, SGI0, SGI0, SGI385852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94381[] = {SGI385853, SGI0, SGI0, SGI385854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94382[] = {SGI385855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94383[] = {SGI385856, SGI0, SGI0, SGI0, SGI385857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94384[] = {SGI385858, NULL};
const Segment * const SG94385[] = {SGI385859, NULL};
const Segment * const SG94386[] = {SGI385860, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94387[] = {SGI385861, SGI385862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94388[] = {SGI385863, SGI385864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94389[] = {SGI385865, SGI0, SGI0, SGI0, SGI385866, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94390[] = {SGI385867, SGI0, SGI385868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94391[] = {SGI385869, SGI0, SGI385870, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94392[] = {SGI385871, SGI0, SGI385872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94393[] = {SGI385873, SGI0, SGI385874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94394[] = {SGI385875, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94395[] = {SGI385876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94396[] = {SGI385877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94397[] = {SGI385878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94398[] = {SGI385879, SGI385880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94399[] = {SGI385881, SGI385882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94400[] = {SGI385883, SGI385884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94401[] = {SGI385885, SGI0, SGI0, SGI385886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94402[] = {SGI385887, SGI385888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94403[] = {SGI385889, SGI385890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94404[] = {SGI385891, SGI385892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94405[] = {SGI385893, SGI385894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94406[] = {SGI385895, SGI385896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94407[] = {SGI385897, SGI385898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94408[] = {SGI385899, SGI385900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94409[] = {SGI385901, SGI385902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94410[] = {SGI385903, SGI385904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94411[] = {SGI385905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94412[] = {SGI385906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94413[] = {SGI385907, SGI385908, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94414[] = {SGI385909, SGI385910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94415[] = {SGI385911, SGI385912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94416[] = {SGI385913, SGI385914, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94417[] = {SGI385915, SGI385916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94418[] = {SGI385917, SGI385918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94419[] = {SGI385919, NULL};
const Segment * const SG94420[] = {SGI385920, NULL};
const Segment * const SG94421[] = {SGI385921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94422[] = {SGI385922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94423[] = {SGI385923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94424[] = {SGI385924, SGI385925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94425[] = {SGI385927, NULL};
const Segment * const SG94426[] = {SGI385926, SGI385928, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94427[] = {SGI385929, NULL};
const Segment * const SG94428[] = {SGI385930, NULL};
const Segment * const SG94429[] = {SGI385931, SGI385932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94430[] = {SGI385933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94431[] = {SGI385934, SGI385935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94432[] = {SGI385936, SGI385937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94433[] = {SGI385938, SGI385939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94434[] = {SGI385940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI385941, SGI0, SGI385942, NULL};
const Segment * const SG94435[] = {SGI385943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI385944, SGI0, SGI385945, NULL};
const Segment * const SG94436[] = {SGI385946, SGI385947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94437[] = {SGI385948, SGI385949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94438[] = {SGI385950, SGI0, SGI385951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94439[] = {SGI385952, SGI385953, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94440[] = {SGI385954, SGI385955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94441[] = {SGI385956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94442[] = {SGI385957, SGI385958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94443[] = {SGI385959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI385960, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94444[] = {SGI385961, SGI385962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94445[] = {SGI385963, SGI385964, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94446[] = {SGI385965, SGI385966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94447[] = {SGI385967, SGI0, SGI385968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94448[] = {SGI385969, SGI385970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94449[] = {SGI385971, NULL};
const Segment * const SG94450[] = {SGI385972, SGI0, SGI0, SGI385973, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94451[] = {SGI385974, SGI0, SGI0, SGI385975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94452[] = {SGI385976, SGI0, SGI0, SGI385977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94453[] = {SGI385978, SGI0, SGI0, SGI385979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94454[] = {SGI385980, SGI0, SGI0, SGI385981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94455[] = {SGI385982, SGI385983, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94456[] = {SGI385984, SGI385985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94457[] = {SGI385986, NULL};
const Segment * const SG94458[] = {SGI385987, SGI385988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94459[] = {SGI385989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94460[] = {SGI385990, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94461[] = {SGI385991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94462[] = {SGI385992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94463[] = {SGI385993, SGI385994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94464[] = {SGI385995, SGI385996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94465[] = {SGI385997, SGI385998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94466[] = {SGI385999, SGI386000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94467[] = {SGI386001, SGI0, SGI0, SGI386002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94468[] = {SGI386003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386004, SGI0, SGI0, NULL};
const Segment * const SG94469[] = {SGI386005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386006, SGI0, SGI386007, NULL};
const Segment * const SG94470[] = {SGI386008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386009, SGI0, SGI386010, NULL};
const Segment * const SG94471[] = {SGI386011, SGI0, SGI386012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94472[] = {SGI386013, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94473[] = {SGI386014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94474[] = {SGI386015, SGI386016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94475[] = {SGI386017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94476[] = {SGI386018, SGI386019, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94477[] = {SGI386020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94478[] = {SGI386021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94479[] = {SGI386022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386023, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94480[] = {SGI386024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94481[] = {SGI386025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94482[] = {SGI386026, SGI386027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94483[] = {SGI386028, SGI386029, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94484[] = {SGI386030, NULL};
const Segment * const SG94485[] = {SGI386031, SGI0, SGI386032, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94486[] = {SGI386033, SGI0, SGI386034, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94487[] = {SGI386035, SGI0, SGI386036, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94488[] = {SGI386037, SGI0, SGI386038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94489[] = {SGI386039, NULL};
const Segment * const SG94490[] = {SGI386040, SGI386041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94491[] = {SGI386042, SGI386043, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94492[] = {SGI386044, SGI386045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94493[] = {SGI386046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386047, SGI0, SGI386048, NULL};
const Segment * const SG94494[] = {SGI386049, SGI386050, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94495[] = {SGI386051, SGI0, SGI0, SGI0, SGI386052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94496[] = {SGI386053, SGI386054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94497[] = {SGI386055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94498[] = {SGI386056, SGI0, SGI0, SGI0, SGI386057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94499[] = {SGI386058, SGI0, SGI0, SGI0, SGI386059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94500[] = {SGI386060, SGI386061, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94501[] = {SGI386062, SGI386063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94502[] = {SGI386064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386065, SGI0, SGI386066, NULL};
const Segment * const SG94503[] = {SGI386067, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386068, SGI0, SGI386069, NULL};
const Segment * const SG94504[] = {SGI386070, NULL};
const Segment * const SG94505[] = {SGI386071, SGI386072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94506[] = {SGI386073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386074, SGI0, SGI386075, NULL};
const Segment * const SG94507[] = {SGI386076, SGI386077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94508[] = {SGI386078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386079, SGI0, SGI386080, NULL};
const Segment * const SG94509[] = {SGI386081, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386082, SGI0, SGI386083, NULL};
const Segment * const SG94510[] = {SGI386084, NULL};
const Segment * const SG94511[] = {SGI386085, SGI0, SGI386086, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94512[] = {SGI386087, SGI0, SGI386088, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94513[] = {SGI386089, SGI0, SGI386090, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94514[] = {SGI386091, SGI0, SGI386092, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94515[] = {SGI386093, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386094, SGI0, SGI386095, NULL};
const Segment * const SG94516[] = {SGI386096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386097, SGI0, SGI386098, NULL};
const Segment * const SG94517[] = {SGI386099, SGI0, SGI0, SGI0, SGI386100, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94518[] = {SGI386101, SGI0, SGI0, SGI0, SGI386102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94519[] = {SGI386103, SGI386104, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94520[] = {SGI386105, SGI386106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94521[] = {SGI386107, SGI386108, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94522[] = {SGI386109, SGI386110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94523[] = {SGI386111, NULL};
const Segment * const SG94524[] = {SGI386112, SGI0, SGI0, SGI0, SGI386113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94525[] = {SGI386114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386115, SGI0, SGI386116, NULL};
const Segment * const SG94526[] = {SGI386117, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386118, SGI0, SGI386119, NULL};
const Segment * const SG94527[] = {SGI386120, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386121, SGI0, SGI386122, NULL};
const Segment * const SG94528[] = {SGI386123, SGI386124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94529[] = {SGI386125, NULL};
const Segment * const SG94530[] = {SGI386126, NULL};
const Segment * const SG94531[] = {SGI386127, NULL};
const Segment * const SG94532[] = {SGI386128, NULL};
const Segment * const SG94533[] = {SGI386129, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386130, SGI0, SGI0, NULL};
const Segment * const SG94534[] = {SGI386131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386132, SGI0, SGI386133, NULL};
const Segment * const SG94535[] = {SGI386134, NULL};
const Segment * const SG94536[] = {SGI386135, SGI386136, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94537[] = {SGI386137, SGI386138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94538[] = {SGI386139, SGI386140, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94539[] = {SGI386141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94540[] = {SGI386142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94541[] = {SGI386143, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94542[] = {SGI386144, NULL};
const Segment * const SG94543[] = {SGI386145, NULL};
const Segment * const SG94544[] = {SGI386146, NULL};
const Segment * const SG94545[] = {SGI386147, NULL};
const Segment * const SG94546[] = {SGI386148, NULL};
const Segment * const SG94547[] = {SGI386149, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386150, SGI0, SGI0, NULL};
const Segment * const SG94548[] = {SGI386151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94549[] = {SGI386152, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94550[] = {SGI386153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94551[] = {SGI386154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94552[] = {SGI386155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94553[] = {SGI386156, SGI386157, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94554[] = {SGI386158, SGI386159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94555[] = {SGI386160, NULL};
const Segment * const SG94556[] = {SGI386161, NULL};
const Segment * const SG94557[] = {SGI386162, NULL};
const Segment * const SG94558[] = {SGI386163, NULL};
const Segment * const SG94559[] = {SGI386164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386165, SGI0, SGI0, NULL};
const Segment * const SG94560[] = {SGI386166, NULL};
const Segment * const SG94561[] = {SGI386167, NULL};
const Segment * const SG94562[] = {SGI386168, SGI0, SGI0, SGI386169, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94563[] = {SGI386170, SGI0, SGI0, SGI386171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94564[] = {SGI386172, SGI0, SGI0, SGI386173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94565[] = {SGI386174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386175, SGI0, SGI386176, NULL};
const Segment * const SG94566[] = {SGI386177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386178, SGI0, SGI386179, NULL};
const Segment * const SG94567[] = {SGI386180, NULL};
const Segment * const SG94568[] = {SGI386181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386182, SGI0, SGI0, NULL};
const Segment * const SG94569[] = {SGI386183, NULL};
const Segment * const SG94570[] = {SGI386184, NULL};
const Segment * const SG94571[] = {SGI386185, NULL};
const Segment * const SG94572[] = {SGI386186, SGI386187, NULL};
const Segment * const SG94573[] = {SGI386188, NULL};
const Segment * const SG94574[] = {SGI386189, NULL};
const Segment * const SG94575[] = {SGI386190, NULL};
const Segment * const SG94576[] = {SGI386191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94577[] = {SGI386192, NULL};
const Segment * const SG94578[] = {SGI386193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386194, SGI0, SGI0, NULL};
const Segment * const SG94579[] = {SGI386195, NULL};
const Segment * const SG94580[] = {SGI386196, NULL};
const Segment * const SG94581[] = {SGI386197, NULL};
const Segment * const SG94582[] = {SGI386198, NULL};
const Segment * const SG94583[] = {SGI386199, NULL};
const Segment * const SG94584[] = {SGI386200, NULL};
const Segment * const SG94585[] = {SGI386201, NULL};
const Segment * const SG94586[] = {SGI386202, NULL};
const Segment * const SG94587[] = {SGI386203, NULL};
const Segment * const SG94588[] = {SGI386204, NULL};
const Segment * const SG94589[] = {SGI386205, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94590[] = {SGI386206, NULL};
const Segment * const SG94591[] = {SGI386207, NULL};
const Segment * const SG94592[] = {SGI386208, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94593[] = {SGI386209, SGI386210, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94594[] = {SGI386211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386212, SGI0, SGI0, NULL};
const Segment * const SG94595[] = {SGI386213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386214, SGI0, SGI386215, NULL};
const Segment * const SG94596[] = {SGI386216, NULL};
const Segment * const SG94597[] = {SGI386217, NULL};
const Segment * const SG94598[] = {SGI386218, NULL};
const Segment * const SG94599[] = {SGI386219, NULL};
const Segment * const SG94600[] = {SGI386220, NULL};
const Segment * const SG94601[] = {SGI386221, NULL};
const Segment * const SG94602[] = {SGI386222, NULL};
const Segment * const SG94603[] = {SGI386223, NULL};
const Segment * const SG94604[] = {SGI386224, NULL};
const Segment * const SG94605[] = {SGI386225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386226, SGI0, SGI386227, NULL};
const Segment * const SG94606[] = {SGI386228, NULL};
const Segment * const SG94607[] = {SGI386229, NULL};
const Segment * const SG94608[] = {SGI386230, SGI0, SGI0, SGI386231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94609[] = {SGI386232, SGI386233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94610[] = {SGI386234, NULL};
const Segment * const SG94611[] = {SGI386235, NULL};
const Segment * const SG94612[] = {SGI386236, NULL};
const Segment * const SG94613[] = {SGI386237, SGI386238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94614[] = {SGI386239, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386240, SGI0, NULL};
const Segment * const SG94615[] = {SGI386241, SGI0, SGI0, SGI0, SGI386242, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94616[] = {SGI386243, SGI0, SGI0, SGI0, SGI386244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94617[] = {SGI386245, SGI0, SGI0, SGI0, SGI386246, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94618[] = {SGI386247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386248, SGI0, SGI386249, NULL};
const Segment * const SG94619[] = {SGI386250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386251, SGI0, NULL};
const Segment * const SG94620[] = {SGI386252, SGI386253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94621[] = {SGI386254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94622[] = {SGI386255, SGI386256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94623[] = {SGI386257, SGI0, SGI0, SGI386258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94624[] = {SGI386259, SGI0, SGI0, SGI386260, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94625[] = {SGI386261, SGI386262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94626[] = {SGI386263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386264, SGI0, NULL};
const Segment * const SG94627[] = {SGI386265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386266, SGI0, SGI386267, NULL};
const Segment * const SG94628[] = {SGI386268, SGI386269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94629[] = {SGI386270, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386271, SGI0, NULL};
const Segment * const SG94630[] = {SGI386272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386273, SGI0, SGI386274, NULL};
const Segment * const SG94631[] = {SGI386275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386276, SGI0, SGI0, NULL};
const Segment * const SG94632[] = {SGI386277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386278, SGI0, SGI386279, NULL};
const Segment * const SG94633[] = {SGI386280, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386281, SGI0, SGI386282, NULL};
const Segment * const SG94634[] = {SGI386283, SGI386284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94635[] = {SGI386285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94636[] = {SGI386286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386287, SGI0, SGI386288, NULL};
const Segment * const SG94637[] = {SGI386289, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386290, SGI0, SGI0, NULL};
const Segment * const SG94638[] = {SGI386291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386292, SGI0, NULL};
const Segment * const SG94639[] = {SGI386293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386294, SGI0, SGI0, NULL};
const Segment * const SG94640[] = {SGI386295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386296, SGI0, SGI386297, NULL};
const Segment * const SG94641[] = {SGI386298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386299, SGI0, SGI0, NULL};
const Segment * const SG94642[] = {SGI386300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94643[] = {SGI386301, SGI0, SGI0, SGI386302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94644[] = {SGI386303, SGI0, SGI0, SGI386304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94645[] = {SGI386305, SGI0, SGI0, SGI386306, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94646[] = {SGI386307, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386308, SGI0, NULL};
const Segment * const SG94647[] = {SGI386309, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386310, SGI0, SGI386311, NULL};
const Segment * const SG94648[] = {SGI386312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94649[] = {SGI386313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386314, SGI0, SGI386315, NULL};
const Segment * const SG94650[] = {SGI386316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386317, SGI0, SGI386318, NULL};
const Segment * const SG94651[] = {SGI386319, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94652[] = {SGI386320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386321, SGI0, NULL};
const Segment * const SG94653[] = {SGI386322, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386323, SGI0, NULL};
const Segment * const SG94654[] = {SGI386324, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386325, SGI0, SGI0, NULL};
const Segment * const SG94655[] = {SGI386326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386327, SGI0, SGI0, NULL};
const Segment * const SG94656[] = {SGI386328, SGI386329, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94657[] = {SGI386330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386331, SGI0, SGI386332, NULL};
const Segment * const SG94658[] = {SGI386333, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386334, SGI0, NULL};
const Segment * const SG94659[] = {SGI386335, SGI0, SGI0, SGI0, SGI386336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94660[] = {SGI386337, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386338, SGI0, SGI0, NULL};
const Segment * const SG94661[] = {SGI386339, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386340, SGI0, SGI386341, NULL};
const Segment * const SG94662[] = {SGI386342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386343, SGI0, SGI386344, NULL};
const Segment * const SG94663[] = {SGI386345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386346, SGI0, SGI386347, NULL};
const Segment * const SG94664[] = {SGI386348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386349, SGI0, SGI386350, NULL};
const Segment * const SG94665[] = {SGI386351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386352, SGI0, SGI386353, NULL};
const Segment * const SG94666[] = {SGI386354, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386355, SGI0, SGI386356, NULL};
const Segment * const SG94667[] = {SGI386357, SGI386358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94668[] = {SGI386359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386360, SGI0, NULL};
const Segment * const SG94669[] = {SGI386361, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386362, SGI0, SGI0, NULL};
const Segment * const SG94670[] = {SGI386363, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386364, SGI0, SGI386365, NULL};
const Segment * const SG94671[] = {SGI386366, SGI386367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94672[] = {SGI386368, SGI386369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94673[] = {SGI386370, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386371, SGI0, SGI386372, NULL};
const Segment * const SG94674[] = {SGI386373, SGI0, SGI0, SGI386374, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94675[] = {SGI386375, SGI0, SGI0, SGI386376, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94676[] = {SGI386377, SGI0, SGI0, SGI386378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94677[] = {SGI386379, SGI0, SGI0, SGI386380, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94678[] = {SGI386381, SGI0, SGI0, SGI386382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94679[] = {SGI386383, SGI0, SGI0, SGI386384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94680[] = {SGI386385, SGI0, SGI386386, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94681[] = {SGI386387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386388, SGI0, SGI386389, NULL};
const Segment * const SG94682[] = {SGI386390, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386391, SGI0, SGI0, NULL};
const Segment * const SG94683[] = {SGI386392, SGI0, SGI386393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94684[] = {SGI386394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386395, SGI0, SGI386396, NULL};
const Segment * const SG94685[] = {SGI386397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386398, SGI0, NULL};
const Segment * const SG94686[] = {SGI386399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386400, SGI0, SGI386401, NULL};
const Segment * const SG94687[] = {SGI386402, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386403, SGI0, SGI386404, NULL};
const Segment * const SG94688[] = {SGI386405, SGI386406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94689[] = {SGI386407, SGI386408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94690[] = {SGI386409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94691[] = {SGI386410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386411, SGI0, SGI0, NULL};
const Segment * const SG94692[] = {SGI386412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386413, SGI0, SGI0, NULL};
const Segment * const SG94693[] = {SGI386414, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386415, SGI0, SGI386416, NULL};
const Segment * const SG94694[] = {SGI386417, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94695[] = {SGI386418, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386419, SGI0, NULL};
const Segment * const SG94696[] = {SGI386420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386421, SGI0, SGI386422, NULL};
const Segment * const SG94697[] = {SGI386423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94698[] = {SGI386424, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386425, SGI0, SGI386426, NULL};
const Segment * const SG94699[] = {SGI386427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386428, SGI0, SGI386429, NULL};
const Segment * const SG94700[] = {SGI386430, SGI0, SGI0, SGI386431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94701[] = {SGI386432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94702[] = {SGI386433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386434, SGI0, NULL};
const Segment * const SG94703[] = {SGI386435, SGI0, SGI0, SGI386436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94704[] = {SGI386437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94705[] = {SGI386438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94706[] = {SGI386439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386440, SGI0, SGI386441, NULL};
const Segment * const SG94707[] = {SGI386442, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386443, SGI0, SGI386444, NULL};
const Segment * const SG94708[] = {SGI386445, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386446, SGI0, SGI386447, NULL};
const Segment * const SG94709[] = {SGI386448, SGI386449, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94710[] = {SGI386450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386451, SGI0, SGI0, NULL};
const Segment * const SG94711[] = {SGI386452, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94712[] = {SGI386453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386454, SGI0, NULL};
const Segment * const SG94713[] = {SGI386455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386456, SGI0, SGI0, NULL};
const Segment * const SG94714[] = {SGI386457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386458, SGI0, SGI386459, NULL};
const Segment * const SG94715[] = {SGI386460, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386461, SGI0, SGI386462, NULL};
const Segment * const SG94716[] = {SGI386463, SGI0, SGI0, SGI386464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94717[] = {SGI386465, SGI0, SGI0, SGI386466, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94718[] = {SGI386467, SGI0, SGI0, SGI386468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94719[] = {SGI386469, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94720[] = {SGI386470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94721[] = {SGI386471, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94722[] = {SGI386472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386473, SGI0, SGI386474, NULL};
const Segment * const SG94723[] = {SGI386475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386476, SGI0, SGI0, NULL};
const Segment * const SG94724[] = {SGI386477, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386478, SGI0, SGI386479, NULL};
const Segment * const SG94725[] = {SGI386480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386481, SGI0, SGI386482, NULL};
const Segment * const SG94726[] = {SGI386483, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386484, SGI0, SGI386485, NULL};
const Segment * const SG94727[] = {SGI386486, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94728[] = {SGI386487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386488, SGI0, SGI0, NULL};
const Segment * const SG94729[] = {SGI386489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386490, SGI0, SGI0, NULL};
const Segment * const SG94730[] = {SGI386491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386492, SGI0, SGI386493, NULL};
const Segment * const SG94731[] = {SGI386494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386495, SGI0, NULL};
const Segment * const SG94732[] = {SGI386496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94733[] = {SGI386497, SGI386498, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94734[] = {SGI386499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94735[] = {SGI386500, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94736[] = {SGI386501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386502, SGI0, SGI386503, NULL};
const Segment * const SG94737[] = {SGI386504, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94738[] = {SGI386505, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386506, SGI0, NULL};
const Segment * const SG94739[] = {SGI386507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386508, SGI0, SGI386509, NULL};
const Segment * const SG94740[] = {SGI386510, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386511, SGI0, SGI386512, NULL};
const Segment * const SG94741[] = {SGI386513, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94742[] = {SGI386514, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386515, SGI0, SGI0, NULL};
const Segment * const SG94743[] = {SGI386516, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386517, SGI0, SGI0, NULL};
const Segment * const SG94744[] = {SGI386518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386519, SGI0, NULL};
const Segment * const SG94745[] = {SGI386520, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386521, SGI0, NULL};
const Segment * const SG94746[] = {SGI386522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386523, SGI0, NULL};
const Segment * const SG94747[] = {SGI386524, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386525, SGI0, NULL};
const Segment * const SG94748[] = {SGI386526, SGI0, SGI0, SGI386527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94749[] = {SGI386528, SGI0, SGI0, SGI386529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94750[] = {SGI386530, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94751[] = {SGI386531, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386532, SGI0, NULL};
const Segment * const SG94752[] = {SGI386533, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386534, SGI0, NULL};
const Segment * const SG94753[] = {SGI386535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386536, SGI0, SGI386537, NULL};
const Segment * const SG94754[] = {SGI386538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386539, SGI0, SGI386540, NULL};
const Segment * const SG94755[] = {SGI386541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386542, SGI0, SGI386543, NULL};
const Segment * const SG94756[] = {SGI386544, SGI0, SGI0, SGI386545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94757[] = {SGI386546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94758[] = {SGI386547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386548, SGI0, NULL};
const Segment * const SG94759[] = {SGI386549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386550, SGI0, SGI386551, NULL};
const Segment * const SG94760[] = {SGI386552, SGI386553, NULL};
const Segment * const SG94761[] = {SGI386554, SGI386555, NULL};
const Segment * const SG94762[] = {SGI386556, SGI386557, NULL};
const Segment * const SG94763[] = {SGI386558, SGI386559, NULL};
const Segment * const SG94764[] = {SGI386560, SGI386561, NULL};
const Segment * const SG94765[] = {SGI386562, SGI386563, NULL};
const Segment * const SG94766[] = {SGI386564, SGI386565, NULL};
const Segment * const SG94767[] = {SGI386566, SGI386567, NULL};
const Segment * const SG94768[] = {SGI386568, SGI386569, NULL};
const Segment * const SG94769[] = {SGI386570, SGI386571, NULL};
const Segment * const SG94770[] = {SGI386572, SGI386573, NULL};
const Segment * const SG94771[] = {SGI386574, SGI386575, NULL};
const Segment * const SG94772[] = {SGI386576, SGI386577, NULL};
const Segment * const SG94773[] = {SGI386578, SGI386579, NULL};
const Segment * const SG94774[] = {SGI386580, SGI386581, NULL};
const Segment * const SG94775[] = {SGI386582, SGI386583, NULL};
const Segment * const SG94776[] = {SGI386584, SGI386585, NULL};
const Segment * const SG94777[] = {SGI386586, SGI386587, NULL};
const Segment * const SG94778[] = {SGI386588, SGI386589, NULL};
const Segment * const SG94779[] = {SGI386590, SGI386591, NULL};
const Segment * const SG94780[] = {SGI386592, SGI386593, NULL};
const Segment * const SG94781[] = {SGI386594, SGI386595, NULL};
const Segment * const SG94782[] = {SGI386596, SGI386597, NULL};
const Segment * const SG94783[] = {SGI386598, SGI386599, NULL};
const Segment * const SG94784[] = {SGI386600, SGI386601, NULL};
const Segment * const SG94785[] = {SGI386602, SGI386603, NULL};
const Segment * const SG94786[] = {SGI386604, SGI386605, NULL};
const Segment * const SG94787[] = {SGI386606, SGI386607, NULL};
const Segment * const SG94788[] = {SGI386608, SGI386609, NULL};
const Segment * const SG94789[] = {SGI386610, SGI386611, NULL};
const Segment * const SG94790[] = {SGI386612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386613, SGI0, NULL};
const Segment * const SG94791[] = {SGI386614, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386615, SGI0, NULL};
const Segment * const SG94792[] = {SGI386616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386617, SGI0, SGI0, NULL};
const Segment * const SG94793[] = {SGI386618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386619, SGI0, SGI0, NULL};
const Segment * const SG94794[] = {SGI386620, SGI386621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94795[] = {SGI386622, SGI386623, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94796[] = {SGI386624, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94797[] = {SGI386625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94798[] = {SGI386626, SGI386627, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94799[] = {SGI386628, SGI386629, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94800[] = {SGI386630, SGI386631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94801[] = {SGI386632, SGI386633, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94802[] = {SGI386634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386635, SGI0, NULL};
const Segment * const SG94803[] = {SGI386636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94804[] = {SGI386637, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386638, SGI0, SGI0, NULL};
const Segment * const SG94805[] = {SGI386639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94806[] = {SGI386640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94807[] = {SGI386641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386642, SGI0, SGI0, NULL};
const Segment * const SG94808[] = {SGI386643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94809[] = {SGI386644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386645, SGI0, SGI0, NULL};
const Segment * const SG94810[] = {SGI386646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386647, SGI0, SGI386648, NULL};
const Segment * const SG94811[] = {SGI386649, SGI386650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94812[] = {SGI386651, SGI386652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94813[] = {SGI386653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386654, SGI0, NULL};
const Segment * const SG94814[] = {SGI386655, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94815[] = {SGI386656, SGI386657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94816[] = {SGI386658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94817[] = {SGI386659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94818[] = {SGI386660, SGI0, SGI0, SGI0, SGI0, SGI386661, SGI386662, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94819[] = {SGI386663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94820[] = {SGI386664, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94821[] = {SGI386665, SGI0, SGI0, SGI0, SGI386666, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94822[] = {SGI386667, SGI0, SGI0, SGI0, SGI386668, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94823[] = {SGI386669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386670, SGI0, NULL};
const Segment * const SG94824[] = {SGI386671, SGI0, SGI0, SGI0, SGI0, SGI386672, SGI386673, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94825[] = {SGI386674, SGI386675, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94826[] = {SGI386676, SGI386677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94827[] = {SGI386678, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386679, SGI0, SGI386680, NULL};
const Segment * const SG94828[] = {SGI386681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94829[] = {SGI386682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94830[] = {SGI386683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94831[] = {SGI386684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94832[] = {SGI386685, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386686, SGI0, NULL};
const Segment * const SG94833[] = {SGI386687, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94834[] = {SGI386688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386689, SGI0, SGI386690, NULL};
const Segment * const SG94835[] = {SGI386691, SGI386692, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94836[] = {SGI386693, SGI386694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94837[] = {SGI386695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94838[] = {SGI386696, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94839[] = {SGI386697, SGI0, SGI0, SGI0, SGI0, SGI386698, SGI386699, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94840[] = {SGI386700, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94841[] = {SGI386701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94842[] = {SGI386702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94843[] = {SGI386703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386704, SGI0, SGI386705, NULL};
const Segment * const SG94844[] = {SGI386706, SGI0, SGI0, SGI386707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94845[] = {SGI386708, SGI0, SGI0, SGI386709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94846[] = {SGI386710, SGI0, SGI0, SGI386711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94847[] = {SGI386712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94848[] = {SGI386713, SGI386714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94849[] = {SGI386715, SGI386716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94850[] = {SGI386717, SGI0, SGI0, SGI386718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94851[] = {SGI386719, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94852[] = {SGI386720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386721, SGI0, SGI386722, NULL};
const Segment * const SG94853[] = {SGI386723, SGI0, SGI0, SGI386724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94854[] = {SGI386725, SGI0, SGI0, SGI386726, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94855[] = {SGI386727, SGI0, SGI0, SGI386728, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94856[] = {SGI386729, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94857[] = {SGI386730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386731, SGI0, NULL};
const Segment * const SG94858[] = {SGI386732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386733, SGI0, NULL};
const Segment * const SG94859[] = {SGI386734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386735, SGI0, NULL};
const Segment * const SG94860[] = {SGI386736, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386737, SGI0, SGI386738, NULL};
const Segment * const SG94861[] = {SGI386739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94862[] = {SGI386740, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94863[] = {SGI386741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386742, SGI0, NULL};
const Segment * const SG94864[] = {SGI386743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386744, SGI0, SGI386745, NULL};
const Segment * const SG94865[] = {SGI386746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386747, SGI0, SGI386748, NULL};
const Segment * const SG94866[] = {SGI386749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386750, SGI0, SGI0, NULL};
const Segment * const SG94867[] = {SGI386751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94868[] = {SGI386752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94869[] = {SGI386753, SGI0, SGI0, SGI386754, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94870[] = {SGI386755, SGI0, SGI0, SGI386756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94871[] = {SGI386757, SGI0, SGI0, SGI386758, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94872[] = {SGI386759, SGI0, SGI0, SGI386760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94873[] = {SGI386761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386762, SGI0, NULL};
const Segment * const SG94874[] = {SGI386763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386764, SGI0, NULL};
const Segment * const SG94875[] = {SGI386765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386766, SGI0, SGI0, NULL};
const Segment * const SG94876[] = {SGI386767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386768, SGI0, SGI386769, NULL};
const Segment * const SG94877[] = {SGI386770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94878[] = {SGI386771, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94879[] = {SGI386772, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386773, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94880[] = {SGI386774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386775, SGI0, NULL};
const Segment * const SG94881[] = {SGI386776, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386777, SGI0, SGI0, NULL};
const Segment * const SG94882[] = {SGI386778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94883[] = {SGI386779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386780, SGI0, SGI386781, NULL};
const Segment * const SG94884[] = {SGI386782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94885[] = {SGI386783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386784, SGI0, SGI386785, NULL};
const Segment * const SG94886[] = {SGI386786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94887[] = {SGI386787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94888[] = {SGI386788, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386789, SGI0, NULL};
const Segment * const SG94889[] = {SGI386790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386791, SGI0, SGI0, NULL};
const Segment * const SG94890[] = {SGI386792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386793, SGI0, SGI0, NULL};
const Segment * const SG94891[] = {SGI386794, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386795, SGI0, SGI386796, NULL};
const Segment * const SG94892[] = {SGI386797, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386798, SGI0, SGI386799, NULL};
const Segment * const SG94893[] = {SGI386800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94894[] = {SGI386801, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386802, SGI0, NULL};
const Segment * const SG94895[] = {SGI386803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386804, SGI0, SGI386805, NULL};
const Segment * const SG94896[] = {SGI386806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386807, SGI0, SGI386808, NULL};
const Segment * const SG94897[] = {SGI386809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386810, SGI0, SGI386811, NULL};
const Segment * const SG94898[] = {SGI386812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386813, SGI0, SGI386814, NULL};
const Segment * const SG94899[] = {SGI386815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386816, SGI0, SGI386817, NULL};
const Segment * const SG94900[] = {SGI386818, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386819, SGI0, SGI386820, NULL};
const Segment * const SG94901[] = {SGI386821, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386822, SGI0, SGI386823, NULL};
const Segment * const SG94902[] = {SGI386824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94903[] = {SGI386825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94904[] = {SGI386826, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94905[] = {SGI386827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94906[] = {SGI386828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94907[] = {SGI386829, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386830, SGI0, NULL};
const Segment * const SG94908[] = {SGI386831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386832, SGI0, NULL};
const Segment * const SG94909[] = {SGI386833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386834, SGI0, SGI0, NULL};
const Segment * const SG94910[] = {SGI386835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386836, SGI0, NULL};
const Segment * const SG94911[] = {SGI386837, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386838, SGI0, SGI0, NULL};
const Segment * const SG94912[] = {SGI386839, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386840, SGI0, SGI0, NULL};
const Segment * const SG94913[] = {SGI386841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386842, SGI0, SGI0, NULL};
const Segment * const SG94914[] = {SGI386843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386844, SGI0, NULL};
const Segment * const SG94915[] = {SGI386845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386846, SGI0, SGI0, NULL};
const Segment * const SG94916[] = {SGI386847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386848, SGI0, SGI386849, NULL};
const Segment * const SG94917[] = {SGI386850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94918[] = {SGI386851, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386852, SGI0, SGI386853, NULL};
const Segment * const SG94919[] = {SGI386854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94920[] = {SGI386855, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386856, SGI0, NULL};
const Segment * const SG94921[] = {SGI386857, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386858, SGI0, SGI0, NULL};
const Segment * const SG94922[] = {SGI386859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386860, SGI0, NULL};
const Segment * const SG94923[] = {SGI386861, SGI0, SGI0, SGI0, SGI0, SGI386862, SGI386863, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94924[] = {SGI386864, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94925[] = {SGI386865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386866, SGI0, NULL};
const Segment * const SG94926[] = {SGI386867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386868, SGI0, NULL};
const Segment * const SG94927[] = {SGI386869, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386870, SGI0, NULL};
const Segment * const SG94928[] = {SGI386871, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386872, SGI0, SGI386873, NULL};
const Segment * const SG94929[] = {SGI386874, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386875, SGI0, SGI386876, NULL};
const Segment * const SG94930[] = {SGI386877, SGI386878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94931[] = {SGI386879, SGI0, SGI0, SGI386880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94932[] = {SGI386881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94933[] = {SGI386882, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94934[] = {SGI386883, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386884, SGI0, SGI0, NULL};
const Segment * const SG94935[] = {SGI386885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94936[] = {SGI386886, SGI0, SGI0, SGI386887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94937[] = {SGI386888, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94938[] = {SGI386889, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94939[] = {SGI386890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94940[] = {SGI386891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94941[] = {SGI386892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94942[] = {SGI386893, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94943[] = {SGI386894, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94944[] = {SGI386895, SGI0, SGI0, SGI0, SGI386896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94945[] = {SGI386897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94946[] = {SGI386898, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94947[] = {SGI386899, SGI386900, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94948[] = {SGI386901, SGI0, SGI0, SGI386902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94949[] = {SGI386903, SGI386904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94950[] = {SGI386905, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386906, SGI0, SGI0, NULL};
const Segment * const SG94951[] = {SGI386907, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386908, SGI0, SGI0, NULL};
const Segment * const SG94952[] = {SGI386909, SGI386910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94953[] = {SGI386911, SGI0, SGI0, SGI0, SGI386912, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94954[] = {SGI386913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386914, SGI0, NULL};
const Segment * const SG94955[] = {SGI386915, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94956[] = {SGI386916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94957[] = {SGI386917, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94958[] = {SGI386918, SGI0, SGI0, SGI0, SGI386919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94959[] = {SGI386920, SGI0, SGI0, SGI386921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94960[] = {SGI386922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94961[] = {SGI386923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386924, SGI0, NULL};
const Segment * const SG94962[] = {SGI386925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386926, SGI0, NULL};
const Segment * const SG94963[] = {SGI386927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386928, SGI0, NULL};
const Segment * const SG94964[] = {SGI386929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94965[] = {SGI386930, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94966[] = {SGI386931, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94967[] = {SGI386932, SGI0, SGI0, SGI386933, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94968[] = {SGI386934, SGI0, SGI0, SGI386935, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94969[] = {SGI386936, SGI386937, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94970[] = {SGI386938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94971[] = {SGI386939, SGI0, SGI0, SGI386940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94972[] = {SGI386941, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94973[] = {SGI386942, SGI386943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94974[] = {SGI386944, SGI0, SGI386945, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94975[] = {SGI386946, SGI0, SGI386947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94976[] = {SGI386948, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94977[] = {SGI386949, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94978[] = {SGI386950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94979[] = {SGI386951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94980[] = {SGI386952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94981[] = {SGI386953, SGI0, SGI0, SGI0, SGI386954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94982[] = {SGI386955, SGI0, SGI0, SGI386956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94983[] = {SGI386957, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386958, SGI0, NULL};
const Segment * const SG94984[] = {SGI386959, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386960, SGI0, SGI0, NULL};
const Segment * const SG94985[] = {SGI386961, SGI386962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94986[] = {SGI386963, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94987[] = {SGI386964, SGI386965, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94988[] = {SGI386966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94989[] = {SGI386967, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94990[] = {SGI386968, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94991[] = {SGI386969, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94992[] = {SGI386970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94993[] = {SGI386971, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94994[] = {SGI386972, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94995[] = {SGI386973, SGI0, SGI0, SGI0, SGI386974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94996[] = {SGI386975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94997[] = {SGI386976, SGI0, SGI0, SGI386977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG94998[] = {SGI386978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386979, SGI0, NULL};
const Segment * const SG94999[] = {SGI386980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95000[] = {SGI386981, SGI386982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95001[] = {SGI386983, SGI386984, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95002[] = {SGI386985, SGI386986, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95003[] = {SGI386987, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95004[] = {SGI386988, SGI0, SGI0, SGI0, SGI386989, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95005[] = {SGI386990, SGI0, SGI386991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95006[] = {SGI386992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386993, SGI0, NULL};
const Segment * const SG95007[] = {SGI386994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95008[] = {SGI386995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95009[] = {SGI386996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95010[] = {SGI386997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95011[] = {SGI386998, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI386999, SGI0, NULL};
const Segment * const SG95012[] = {SGI387000, SGI0, SGI0, SGI387001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95013[] = {SGI387002, SGI387003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95014[] = {SGI387004, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95015[] = {SGI387005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95016[] = {SGI387006, SGI387007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95017[] = {SGI387008, SGI387009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95018[] = {SGI387010, SGI387011, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95019[] = {SGI387012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95020[] = {SGI387013, SGI387014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95021[] = {SGI387015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95022[] = {SGI387016, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95023[] = {SGI387017, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95024[] = {SGI387018, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI387019, SGI0, NULL};
const Segment * const SG95025[] = {SGI387020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95026[] = {SGI387021, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95027[] = {SGI387022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95028[] = {SGI387023, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95029[] = {SGI387024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95030[] = {SGI387025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95031[] = {SGI387026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95032[] = {SGI387027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95033[] = {SGI387028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI387029, SGI0, NULL};
const Segment * const SG95034[] = {SGI387030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI387031, SGI0, NULL};
const Segment * const SG95035[] = {SGI387032, SGI0, SGI387033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95036[] = {SGI387034, SGI387035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95037[] = {SGI387036, SGI0, SGI0, SGI387037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95038[] = {SGI387038, SGI387039, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95039[] = {SGI387040, SGI387041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95040[] = {SGI387042, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95041[] = {SGI387043, SGI387044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95042[] = {SGI387045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95043[] = {SGI387046, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95044[] = {SGI387047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95045[] = {SGI387048, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95046[] = {SGI387049, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95047[] = {SGI387050, SGI387051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95048[] = {SGI387052, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95049[] = {SGI387053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95050[] = {SGI387054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95051[] = {SGI387055, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95052[] = {SGI387056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95053[] = {SGI387057, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95054[] = {SGI387058, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95055[] = {SGI387059, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95056[] = {SGI387060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI387061, SGI0, NULL};
const Segment * const SG95057[] = {SGI387062, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI387063, SGI0, NULL};
const Segment * const SG95058[] = {SGI387064, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95059[] = {SGI387065, SGI0, SGI387066, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95060[] = {SGI387067, SGI0, SGI387068, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95061[] = {SGI387069, SGI0, SGI387070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95062[] = {SGI387071, SGI387072, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95063[] = {SGI387073, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95064[] = {SGI387074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI387075, SGI0, NULL};
const Segment * const SG95065[] = {SGI387076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI387077, SGI0, NULL};
const Segment * const SG95066[] = {SGI387078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95067[] = {SGI387079, SGI387080, NULL};
const Segment * const SG95068[] = {SGI387081, SGI387082, NULL};
const Segment * const SG95069[] = {SGI387083, SGI387084, NULL};
const Segment * const SG95070[] = {SGI387085, SGI387086, NULL};
const Segment * const SG95071[] = {SGI387087, SGI387088, NULL};
const Segment * const SG95072[] = {SGI387089, SGI387090, NULL};
const Segment * const SG95073[] = {SGI387091, SGI387092, NULL};
const Segment * const SG95074[] = {SGI387093, SGI387094, NULL};
const Segment * const SG95075[] = {SGI387095, NULL};
const Segment * const SG95076[] = {SGI387096, NULL};
const Segment * const SG95077[] = {SGI387097, NULL};
const Segment * const SG95078[] = {SGI387098, SGI387099, NULL};
const Segment * const SG95079[] = {SGI387100, SGI387101, NULL};
const Segment * const SG95080[] = {SGI387102, SGI387103, NULL};
const Segment * const SG95081[] = {SGI387104, SGI387105, NULL};
const Segment * const SG95082[] = {SGI387106, SGI387107, NULL};
const Segment * const SG95083[] = {SGI387108, NULL};
const Segment * const SG95084[] = {SGI387109, NULL};
const Segment * const SG95085[] = {SGI387110, NULL};
const Segment * const SG95086[] = {SGI387111, NULL};
const Segment * const SG95087[] = {SGI387112, NULL};
const Segment * const SG95088[] = {SGI387113, NULL};
const Segment * const SG95089[] = {SGI387114, NULL};
const Segment * const SG95090[] = {SGI387115, NULL};
const Segment * const SG95091[] = {SGI387116, NULL};
const Segment * const SG95092[] = {SGI387117, NULL};
const Segment * const SG95093[] = {SGI387118, NULL};
const Segment * const SG95094[] = {SGI387119, NULL};
const Segment * const SG95095[] = {SGI387120, NULL};
const Segment * const SG95096[] = {SGI387121, NULL};
const Segment * const SG95097[] = {SGI387122, NULL};
const Segment * const SG95098[] = {SGI387123, NULL};
const Segment * const SG95099[] = {SGI387124, NULL};
const Segment * const SG95100[] = {SGI387125, NULL};
const Segment * const SG95101[] = {SGI387126, NULL};
const Segment * const SG95102[] = {SGI387127, NULL};
const Segment * const SG95103[] = {SGI387128, NULL};
const Segment * const SG95104[] = {SGI387129, NULL};
const Segment * const SG95105[] = {SGI387130, NULL};
const Segment * const SG95106[] = {SGI387131, NULL};
const Segment * const SG95107[] = {SGI387132, NULL};
const Segment * const SG95108[] = {SGI387133, NULL};
const Segment * const SG95109[] = {SGI387134, NULL};
const Segment * const SG95110[] = {SGI387135, NULL};
const Segment * const SG95111[] = {SGI387136, NULL};
const Segment * const SG95112[] = {SGI387137, NULL};
const Segment * const SG95113[] = {SGI387138, NULL};
const Segment * const SG95114[] = {SGI387139, NULL};
const Segment * const SG95115[] = {SGI387140, NULL};
const Segment * const SG95116[] = {SGI387141, NULL};
const Segment * const SG95117[] = {SGI387142, NULL};
const Segment * const SG95118[] = {SGI387143, NULL};
const Segment * const SG95119[] = {SGI387144, NULL};
const Segment * const SG95120[] = {SGI387145, NULL};
const Segment * const SG95121[] = {SGI387146, NULL};
const Segment * const SG95122[] = {SGI387147, NULL};
const Segment * const SG95123[] = {SGI387148, NULL};
const Segment * const SG95124[] = {SGI387149, NULL};
const Segment * const SG95125[] = {SGI387150, NULL};
const Segment * const SG95126[] = {SGI387151, NULL};
const Segment * const SG95127[] = {SGI387152, NULL};
const Segment * const SG95128[] = {SGI387153, NULL};
const Segment * const SG95129[] = {SGI387154, NULL};
const Segment * const SG95130[] = {SGI387155, NULL};
const Segment * const SG95131[] = {SGI387156, NULL};
const Segment * const SG95132[] = {SGI387157, NULL};
const Segment * const SG95133[] = {SGI387158, NULL};
const Segment * const SG95134[] = {SGI387159, NULL};
const Segment * const SG95135[] = {SGI387160, NULL};
const Segment * const SG95136[] = {SGI387161, NULL};
const Segment * const SG95137[] = {SGI387162, NULL};
const Segment * const SG95138[] = {SGI387163, NULL};
const Segment * const SG95139[] = {SGI387164, NULL};
const Segment * const SG95140[] = {SGI387165, NULL};
const Segment * const SG95141[] = {SGI387166, NULL};
const Segment * const SG95142[] = {SGI387167, NULL};
const Segment * const SG95143[] = {SGI387168, NULL};
const Segment * const SG95144[] = {SGI387169, NULL};
const Segment * const SG95145[] = {SGI387170, NULL};
const Segment * const SG95146[] = {SGI387171, NULL};
const Segment * const SG95147[] = {SGI387172, NULL};
const Segment * const SG95148[] = {SGI387173, NULL};
const Segment * const SG95149[] = {SGI387174, NULL};
const Segment * const SG95150[] = {SGI387175, NULL};
const Segment * const SG95151[] = {SGI387176, NULL};
const Segment * const SG95152[] = {SGI387177, NULL};
const Segment * const SG95153[] = {SGI387178, NULL};
const Segment * const SG95154[] = {SGI387179, NULL};
const Segment * const SG95155[] = {SGI387180, NULL};
const Segment * const SG95156[] = {SGI387181, NULL};
const Segment * const SG95157[] = {SGI387182, NULL};
const Segment * const SG95158[] = {SGI387183, NULL};
const Segment * const SG95159[] = {SGI387184, NULL};
const Segment * const SG95160[] = {SGI387185, NULL};
const Segment * const SG95161[] = {SGI387186, NULL};
const Segment * const SG95162[] = {SGI387187, NULL};
const Segment * const SG95163[] = {SGI387188, NULL};
const Segment * const SG95164[] = {SGI387189, NULL};
const Segment * const SG95165[] = {SGI387190, NULL};
const Segment * const SG95166[] = {SGI387191, NULL};
const Segment * const SG95167[] = {SGI387192, NULL};
const Segment * const SG95168[] = {SGI387193, NULL};
const Segment * const SG95169[] = {SGI387194, NULL};
const Segment * const SG95170[] = {SGI387195, NULL};
const Segment * const SG95171[] = {SGI387196, NULL};
const Segment * const SG95172[] = {SGI387197, NULL};
const Segment * const SG95173[] = {SGI387198, NULL};
const Segment * const SG95174[] = {SGI387199, NULL};
const Segment * const SG95175[] = {SGI387200, NULL};
const Segment * const SG95176[] = {SGI387201, NULL};
const Segment * const SG95177[] = {SGI387202, NULL};
const Segment * const SG95178[] = {SGI387203, NULL};
const Segment * const SG95179[] = {SGI387204, NULL};
const Segment * const SG95180[] = {SGI387205, NULL};
const Segment * const SG95181[] = {SGI387206, NULL};
const Segment * const SG95182[] = {SGI387207, NULL};
const Segment * const SG95183[] = {SGI387208, NULL};
const Segment * const SG95184[] = {SGI387209, NULL};
const Segment * const SG95185[] = {SGI387210, NULL};
const Segment * const SG95186[] = {SGI387211, NULL};
const Segment * const SG95187[] = {SGI387212, NULL};
const Segment * const SG95188[] = {SGI387213, NULL};
const Segment * const SG95189[] = {SGI387214, NULL};
const Segment * const SG95190[] = {SGI387215, NULL};
const Segment * const SG95191[] = {SGI387216, NULL};
const Segment * const SG95192[] = {SGI387217, NULL};
const Segment * const SG95193[] = {SGI387218, NULL};
const Segment * const SG95194[] = {SGI387219, NULL};
const Segment * const SG95195[] = {SGI387220, NULL};
const Segment * const SG95196[] = {SGI387221, NULL};
const Segment * const SG95197[] = {SGI387222, NULL};
const Segment * const SG95198[] = {SGI387223, NULL};
const Segment * const SG95199[] = {SGI387224, NULL};
const Segment * const SG95200[] = {SGI387225, NULL};
const Segment * const SG95201[] = {SGI387226, NULL};
const Segment * const SG95202[] = {SGI387227, NULL};
const Segment * const SG95203[] = {SGI387228, SGI387229, SGI387229, SGI387230, NULL};
const Segment * const SG95204[] = {SGI387231, SGI387232, SGI387232, SGI387233, NULL};
const Segment * const SG95205[] = {SGI387234, SGI387235, SGI387235, SGI387236, NULL};
const Segment * const SG95206[] = {SGI387237, NULL};
const Segment * const SG95207[] = {SGI387238, NULL};
const Segment * const SG95208[] = {SGI387239, SGI387240, SGI387240, SGI387241, NULL};
const Segment * const SG95209[] = {SGI387242, NULL};
const Segment * const SG95210[] = {SGI387243, NULL};
const Segment * const SG95211[] = {SGI387244, NULL};
const Segment * const SG95212[] = {SGI387245, NULL};
const Segment * const SG95213[] = {SGI387246, NULL};
const Segment * const SG95214[] = {SGI387247, NULL};
const Segment * const SG95215[] = {SGI387248, NULL};
const Segment * const SG95216[] = {SGI387249, NULL};
const Segment * const SG95217[] = {SGI387250, NULL};
const Segment * const SG95218[] = {SGI387251, NULL};
const Segment * const SG95219[] = {SGI387252, NULL};
const Segment * const SG95220[] = {SGI387253, NULL};
const Segment * const SG95221[] = {SGI387254, NULL};
const Segment * const SG95222[] = {SGI387255, NULL};
const Segment * const SG95223[] = {SGI387256, NULL};
const Segment * const SG95224[] = {SGI387257, NULL};
const Segment * const SG95225[] = {SGI387258, NULL};
const Segment * const SG95226[] = {SGI387259, NULL};
const Segment * const SG95227[] = {SGI387260, NULL};
const Segment * const SG95228[] = {SGI387261, NULL};
const Segment * const SG95229[] = {SGI387262, NULL};
const Segment * const SG95230[] = {SGI387263, NULL};
const Segment * const SG95231[] = {SGI387264, NULL};
const Segment * const SG95232[] = {SGI387265, NULL};
const Segment * const SG95233[] = {SGI387266, NULL};
const Segment * const SG95234[] = {SGI387267, NULL};
const Segment * const SG95235[] = {SGI387268, NULL};
const Segment * const SG95236[] = {SGI387269, NULL};
const Segment * const SG95237[] = {SGI387270, NULL};
const Segment * const SG95238[] = {SGI387271, NULL};
const Segment * const SG95239[] = {SGI387272, NULL};
const Segment * const SG95240[] = {SGI387273, NULL};
const Segment * const SG95241[] = {SGI387274, NULL};
const Segment * const SG95242[] = {SGI387275, NULL};
const Segment * const SG95243[] = {SGI387276, NULL};
const Segment * const SG95244[] = {SGI387277, NULL};
const Segment * const SG95245[] = {SGI387278, NULL};
const Segment * const SG95246[] = {SGI387279, NULL};
const Segment * const SG95247[] = {SGI387280, NULL};
const Segment * const SG95248[] = {SGI387281, NULL};
const Segment * const SG95249[] = {SGI387282, NULL};
const Segment * const SG95250[] = {SGI387283, NULL};
const Segment * const SG95251[] = {SGI387284, NULL};
const Segment * const SG95252[] = {SGI387285, NULL};
const Segment * const SG95253[] = {SGI387286, NULL};
const Segment * const SG95254[] = {SGI387287, NULL};
const Segment * const SG95255[] = {SGI387288, NULL};
const Segment * const SG95256[] = {SGI387289, NULL};
const Segment * const SG95257[] = {SGI387290, NULL};
const Segment * const SG95258[] = {SGI387291, NULL};
const Segment * const SG95259[] = {SGI387292, SGI387293, SGI387293, SGI387294, NULL};
const Segment * const SG95260[] = {SGI387295, SGI387296, SGI387296, SGI387297, NULL};
const Segment * const SG95261[] = {SGI387298, SGI387299, SGI387299, SGI387300, NULL};
const Segment * const SG95262[] = {SGI387301, SGI387302, SGI387302, SGI387303, NULL};
const Segment * const SG95263[] = {SGI387304, SGI387305, SGI387305, SGI387306, NULL};
const Segment * const SG95264[] = {SGI387307, SGI387308, SGI387308, SGI0, NULL};
const Segment * const SG95265[] = {SGI387309, SGI387310, SGI0, SGI0, NULL};
const Segment * const SG95266[] = {SGI387311, SGI387312, SGI0, SGI387313, NULL};
const Segment * const SG95267[] = {SGI387314, SGI387315, SGI0, SGI387316, NULL};
const Segment * const SG95268[] = {SGI387317, SGI387318, SGI0, SGI387319, NULL};
const Segment * const SG95269[] = {SGI387320, SGI387321, SGI0, SGI387322, NULL};
const Segment * const SG95270[] = {SGI387323, SGI387324, SGI0, SGI387325, NULL};
const Segment * const SG95271[] = {SGI387326, SGI387327, SGI0, SGI387328, NULL};
const Segment * const SG95272[] = {SGI387329, SGI387330, SGI0, SGI387331, NULL};
const Segment * const SG95273[] = {SGI387332, SGI387333, SGI0, SGI387334, NULL};
const Segment * const SG95274[] = {SGI387335, SGI387336, SGI0, SGI387337, NULL};
const Segment * const SG95275[] = {SGI387338, SGI387339, SGI0, SGI387340, NULL};
const Segment * const SG95276[] = {SGI387341, SGI387342, SGI0, SGI387343, NULL};
const Segment * const SG95277[] = {SGI387344, SGI387345, SGI0, SGI387346, NULL};
const Segment * const SG95278[] = {SGI387347, SGI387348, SGI0, SGI387349, NULL};
const Segment * const SG95279[] = {SGI387350, SGI387351, SGI0, SGI387352, NULL};
const Segment * const SG95280[] = {SGI387353, SGI387354, SGI0, SGI387355, NULL};
const Segment * const SG95281[] = {SGI387356, SGI387357, SGI0, SGI387358, NULL};
const Segment * const SG95282[] = {SGI387359, SGI387360, SGI0, SGI387361, NULL};
const Segment * const SG95283[] = {SGI387362, SGI387363, SGI0, SGI387364, NULL};
const Segment * const SG95284[] = {SGI387365, SGI387366, SGI0, SGI387367, NULL};
const Segment * const SG95285[] = {SGI387368, SGI387369, SGI0, SGI387370, NULL};
const Segment * const SG95286[] = {SGI387371, SGI387372, SGI0, SGI387373, NULL};
const Segment * const SG95287[] = {SGI387374, SGI387375, SGI0, SGI387376, NULL};
const Segment * const SG95288[] = {SGI387377, SGI387378, SGI0, SGI387379, NULL};
const Segment * const SG95289[] = {SGI387380, NULL};
const Segment * const SG95290[] = {SGI387381, NULL};
const Segment * const SG95291[] = {SGI387382, NULL};
const Segment * const SG95292[] = {SGI387383, NULL};
const Segment * const SG95293[] = {SGI387384, NULL};
const Segment * const SG95294[] = {SGI387385, NULL};
const Segment * const SG95295[] = {SGI387386, NULL};
const Segment * const SG95296[] = {SGI387387, NULL};
const Segment * const SG95297[] = {SGI387388, NULL};
const Segment * const SG95298[] = {SGI387389, NULL};
const Segment * const SG95299[] = {SGI387390, SGI387391, SGI387391, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95300[] = {SGI387392, SGI387393, SGI387393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95301[] = {SGI387394, SGI387395, SGI387395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95302[] = {SGI387396, SGI0, SGI387397, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95303[] = {SGI387398, SGI387399, SGI387399, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95304[] = {SGI387400, SGI0, SGI387401, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95305[] = {SGI387402, SGI387403, SGI387404, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95306[] = {SGI387405, SGI387406, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95307[] = {SGI387407, SGI387408, SGI387409, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95308[] = {SGI387410, NULL};
const Segment * const SG95309[] = {SGI387411, NULL};
const Segment * const SG95310[] = {SGI387412, NULL};
const Segment * const SG95311[] = {SGI387413, NULL};
const Segment * const SG95312[] = {SGI387414, NULL};
const Segment * const SG95313[] = {SGI387415, NULL};
const Segment * const SG95314[] = {SGI387416, NULL};
const Segment * const SG95315[] = {SGI387417, NULL};
const Segment * const SG95316[] = {SGI387418, NULL};
const Segment * const SG95317[] = {SGI387419, NULL};
const Segment * const SG95318[] = {SGI387420, NULL};
const Segment * const SG95319[] = {SGI387421, NULL};
const Segment * const SG95320[] = {SGI387422, NULL};
const Segment * const SG95321[] = {SGI387423, NULL};
const Segment * const SG95322[] = {SGI387424, NULL};
const Segment * const SG95323[] = {SGI387425, NULL};
const Segment * const SG95324[] = {SGI387426, NULL};
const Segment * const SG95325[] = {SGI387427, NULL};
const Segment * const SG95326[] = {SGI387428, NULL};
const Segment * const SG95327[] = {SGI387429, NULL};
const Segment * const SG95328[] = {SGI387430, NULL};
const Segment * const SG95329[] = {SGI387431, NULL};
const Segment * const SG95330[] = {SGI387432, NULL};
const Segment * const SG95331[] = {SGI387433, SGI387434, SGI387435, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI387436, SGI0, SGI0, NULL};
const Segment * const SG95332[] = {SGI387437, NULL};
const Segment * const SG95333[] = {SGI387438, NULL};
const Segment * const SG95334[] = {SGI387439, NULL};
const Segment * const SG95335[] = {SGI387440, NULL};
const Segment * const SG95336[] = {SGI387441, NULL};
const Segment * const SG95337[] = {SGI387442, NULL};
const Segment * const SG95338[] = {SGI387443, NULL};
const Segment * const SG95339[] = {SGI387444, NULL};
const Segment * const SG95340[] = {SGI387445, NULL};
const Segment * const SG95341[] = {SGI387446, NULL};
const Segment * const SG95342[] = {SGI387447, NULL};
const Segment * const SG95343[] = {SGI387448, NULL};
const Segment * const SG95344[] = {SGI387449, NULL};
const Segment * const SG95345[] = {SGI387450, NULL};
const Segment * const SG95346[] = {SGI387451, NULL};
const Segment * const SG95347[] = {SGI387452, NULL};
const Segment * const SG95348[] = {SGI387453, NULL};
const Segment * const SG95349[] = {SGI387454, NULL};
const Segment * const SG95350[] = {SGI387455, NULL};
const Segment * const SG95351[] = {SGI387456, NULL};
const Segment * const SG95352[] = {SGI387457, NULL};
const Segment * const SG95353[] = {SGI387458, NULL};
const Segment * const SG95354[] = {SGI387459, SGI387460, SGI0, SGI387461, NULL};
const Segment * const SG95355[] = {SGI387462, SGI387463, SGI0, SGI387464, NULL};
const Segment * const SG95356[] = {SGI387465, SGI387466, SGI0, SGI387467, NULL};
const Segment * const SG95357[] = {SGI387468, SGI387469, SGI0, SGI387470, NULL};
const Segment * const SG95358[] = {SGI387471, SGI387472, SGI0, SGI387473, NULL};
const Segment * const SG95359[] = {SGI387474, SGI387475, SGI0, SGI387476, NULL};
const Segment * const SG95360[] = {SGI387477, SGI387478, SGI0, SGI387479, NULL};
const Segment * const SG95361[] = {SGI387480, SGI387481, SGI0, SGI387482, NULL};
const Segment * const SG95362[] = {SGI387483, SGI387484, SGI0, SGI387485, NULL};
const Segment * const SG95363[] = {SGI387486, SGI387487, SGI0, SGI387488, NULL};
const Segment * const SG95364[] = {SGI387489, SGI387490, SGI0, SGI387491, NULL};
const Segment * const SG95365[] = {SGI387492, SGI387493, SGI0, SGI387494, NULL};
const Segment * const SG95366[] = {SGI387495, SGI387496, SGI0, SGI387497, NULL};
const Segment * const SG95367[] = {SGI387498, SGI387499, SGI0, SGI387500, NULL};
const Segment * const SG95368[] = {SGI387501, SGI387502, SGI0, SGI387503, NULL};
const Segment * const SG95369[] = {SGI387504, SGI387505, SGI0, SGI387506, NULL};
const Segment * const SG95370[] = {SGI387507, SGI387508, SGI0, SGI387509, NULL};
const Segment * const SG95371[] = {SGI387510, SGI387511, SGI0, SGI387512, NULL};
const Segment * const SG95372[] = {SGI387513, SGI387514, SGI0, SGI387515, NULL};
const Segment * const SG95373[] = {SGI387516, SGI387517, SGI0, SGI387518, NULL};
const Segment * const SG95374[] = {SGI387519, SGI387520, SGI0, SGI387521, NULL};
const Segment * const SG95375[] = {SGI387522, SGI387523, SGI0, SGI387524, NULL};
const Segment * const SG95376[] = {SGI387525, SGI387526, SGI0, SGI387527, NULL};
const Segment * const SG95377[] = {SGI387528, SGI387529, SGI0, SGI387530, NULL};
const Segment * const SG95378[] = {SGI387531, SGI387532, SGI0, SGI387533, NULL};
const Segment * const SG95379[] = {SGI387534, SGI387535, SGI0, SGI387536, NULL};
const Segment * const SG95380[] = {SGI387537, SGI387538, SGI0, SGI387539, NULL};
const Segment * const SG95381[] = {SGI387540, SGI387541, SGI0, SGI387542, NULL};
const Segment * const SG95382[] = {SGI387543, SGI387544, SGI0, SGI387545, NULL};
const Segment * const SG95383[] = {SGI387546, SGI387547, SGI0, SGI0, NULL};
const Segment * const SG95384[] = {SGI387548, SGI387549, SGI0, SGI387550, NULL};
const Segment * const SG95385[] = {SGI387551, SGI387552, SGI0, SGI387553, NULL};
const Segment * const SG95386[] = {SGI387554, SGI387555, SGI0, SGI387556, NULL};
const Segment * const SG95387[] = {SGI387557, SGI387558, SGI0, SGI387559, NULL};
const Segment * const SG95388[] = {SGI387560, SGI387561, SGI0, SGI387562, NULL};
const Segment * const SG95389[] = {SGI387563, SGI387564, SGI0, SGI387565, NULL};
const Segment * const SG95390[] = {SGI387566, SGI387567, SGI0, SGI387568, NULL};
const Segment * const SG95391[] = {SGI387569, NULL};
const Segment * const SG95392[] = {SGI387570, NULL};
const Segment * const SG95393[] = {SGI387571, NULL};
const Segment * const SG95394[] = {SGI387572, NULL};
const Segment * const SG95395[] = {SGI387573, NULL};
const Segment * const SG95396[] = {SGI387574, NULL};
const Segment * const SG95397[] = {SGI387575, NULL};
const Segment * const SG95398[] = {SGI387576, NULL};
const Segment * const SG95399[] = {SGI387577, NULL};
const Segment * const SG95400[] = {SGI387578, NULL};
const Segment * const SG95401[] = {SGI387579, NULL};
const Segment * const SG95402[] = {SGI387580, NULL};
const Segment * const SG95403[] = {SGI387581, NULL};
const Segment * const SG95404[] = {SGI387582, NULL};
const Segment * const SG95405[] = {SGI387583, NULL};
const Segment * const SG95406[] = {SGI387584, NULL};
const Segment * const SG95407[] = {SGI387585, NULL};
const Segment * const SG95408[] = {SGI387586, NULL};
const Segment * const SG95409[] = {SGI387587, NULL};
const Segment * const SG95410[] = {SGI387588, NULL};
const Segment * const SG95411[] = {SGI387589, NULL};
const Segment * const SG95412[] = {SGI387590, NULL};
const Segment * const SG95413[] = {SGI387591, NULL};
const Segment * const SG95414[] = {SGI387592, NULL};
const Segment * const SG95415[] = {SGI387593, NULL};
const Segment * const SG95416[] = {SGI387594, NULL};
const Segment * const SG95417[] = {SGI387595, NULL};
const Segment * const SG95418[] = {SGI387596, NULL};
const Segment * const SG95419[] = {SGI387597, NULL};
const Segment * const SG95420[] = {SGI387598, NULL};
const Segment * const SG95421[] = {SGI387599, NULL};
const Segment * const SG95422[] = {SGI387600, NULL};
const Segment * const SG95423[] = {SGI387601, NULL};
const Segment * const SG95424[] = {SGI387602, NULL};
const Segment * const SG95425[] = {SGI387603, NULL};
const Segment * const SG95426[] = {SGI387604, NULL};
const Segment * const SG95427[] = {SGI387605, NULL};
const Segment * const SG95428[] = {SGI387606, NULL};
const Segment * const SG95429[] = {SGI387607, NULL};
const Segment * const SG95430[] = {SGI387608, NULL};
const Segment * const SG95431[] = {SGI387609, NULL};
const Segment * const SG95432[] = {SGI387610, NULL};
const Segment * const SG95433[] = {SGI387611, NULL};
const Segment * const SG95434[] = {SGI387612, NULL};
const Segment * const SG95435[] = {SGI387613, NULL};
const Segment * const SG95436[] = {SGI387614, NULL};
const Segment * const SG95437[] = {SGI387615, NULL};
const Segment * const SG95438[] = {SGI387616, NULL};
const Segment * const SG95439[] = {SGI387617, NULL};
const Segment * const SG95440[] = {SGI387618, NULL};
const Segment * const SG95441[] = {SGI387619, NULL};
const Segment * const SG95442[] = {SGI387620, NULL};
const Segment * const SG95443[] = {SGI387621, NULL};
const Segment * const SG95444[] = {SGI387622, NULL};
const Segment * const SG95445[] = {SGI387623, NULL};
const Segment * const SG95446[] = {SGI387624, NULL};
const Segment * const SG95447[] = {SGI387625, NULL};
const Segment * const SG95448[] = {SGI387626, NULL};
const Segment * const SG95449[] = {SGI387627, NULL};
const Segment * const SG95450[] = {SGI387628, NULL};
const Segment * const SG95451[] = {SGI387629, NULL};
const Segment * const SG95452[] = {SGI387630, NULL};
const Segment * const SG95453[] = {SGI387631, NULL};
const Segment * const SG95454[] = {SGI387632, NULL};
const Segment * const SG95455[] = {SGI387633, NULL};
const Segment * const SG95456[] = {SGI387634, NULL};
const Segment * const SG95457[] = {SGI387635, NULL};
const Segment * const SG95458[] = {SGI387636, NULL};
const Segment * const SG95459[] = {SGI387637, NULL};
const Segment * const SG95460[] = {SGI387638, NULL};
const Segment * const SG95461[] = {SGI387639, NULL};
const Segment * const SG95462[] = {SGI387640, NULL};
const Segment * const SG95463[] = {SGI387641, NULL};
const Segment * const SG95464[] = {SGI387642, NULL};
const Segment * const SG95465[] = {SGI387643, NULL};
const Segment * const SG95466[] = {SGI387644, NULL};
const Segment * const SG95467[] = {SGI387645, NULL};
const Segment * const SG95468[] = {SGI387646, NULL};
const Segment * const SG95469[] = {SGI387647, NULL};
const Segment * const SG95470[] = {SGI387648, NULL};
const Segment * const SG95471[] = {SGI387649, NULL};
const Segment * const SG95472[] = {SGI387650, NULL};
const Segment * const SG95473[] = {SGI387651, NULL};
const Segment * const SG95474[] = {SGI387652, NULL};
const Segment * const SG95475[] = {SGI387653, NULL};
const Segment * const SG95476[] = {SGI387654, NULL};
const Segment * const SG95477[] = {SGI387655, NULL};
const Segment * const SG95478[] = {SGI387656, NULL};
const Segment * const SG95479[] = {SGI387657, NULL};
const Segment * const SG95480[] = {SGI387658, NULL};
const Segment * const SG95481[] = {SGI387659, NULL};
const Segment * const SG95482[] = {SGI387660, NULL};
const Segment * const SG95483[] = {SGI387661, NULL};
const Segment * const SG95484[] = {SGI387662, NULL};
const Segment * const SG95485[] = {SGI387663, NULL};
const Segment * const SG95486[] = {SGI387664, NULL};
const Segment * const SG95487[] = {SGI387665, NULL};
const Segment * const SG95488[] = {SGI387666, NULL};
const Segment * const SG95489[] = {SGI387667, NULL};
const Segment * const SG95490[] = {SGI387668, NULL};
const Segment * const SG95491[] = {SGI387669, NULL};
const Segment * const SG95492[] = {SGI387670, NULL};
const Segment * const SG95493[] = {SGI387671, NULL};
const Segment * const SG95494[] = {SGI387672, NULL};
const Segment * const SG95495[] = {SGI387673, NULL};
const Segment * const SG95496[] = {SGI387674, NULL};
const Segment * const SG95497[] = {SGI387675, NULL};
const Segment * const SG95498[] = {SGI387676, SGI387677, SGI387677, SGI387678, NULL};
const Segment * const SG95499[] = {SGI387679, NULL};
const Segment * const SG95500[] = {SGI387680, SGI387681, SGI387682, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95501[] = {SGI387683, NULL};
const Segment * const SG95502[] = {SGI387684, NULL};
const Segment * const SG95503[] = {SGI387685, NULL};
const Segment * const SG95504[] = {SGI387686, NULL};
const Segment * const SG95505[] = {SGI387687, NULL};
const Segment * const SG95506[] = {SGI387688, NULL};
const Segment * const SG95507[] = {SGI387689, NULL};
const Segment * const SG95508[] = {SGI387690, NULL};
const Segment * const SG95509[] = {SGI387691, NULL};
const Segment * const SG95510[] = {SGI387692, NULL};
const Segment * const SG95511[] = {SGI387693, NULL};
const Segment * const SG95512[] = {SGI387694, NULL};
const Segment * const SG95513[] = {SGI387695, NULL};
const Segment * const SG95514[] = {SGI387696, NULL};
const Segment * const SG95515[] = {SGI387697, NULL};
const Segment * const SG95516[] = {SGI387698, SGI387699, SGI387700, SGI0, NULL};
const Segment * const SG95517[] = {SGI387701, SGI387702, SGI0, SGI0, NULL};
const Segment * const SG95518[] = {SGI387703, SGI387704, SGI387704, SGI0, NULL};
const Segment * const SG95519[] = {SGI387705, NULL};
const Segment * const SG95520[] = {SGI387706, NULL};
const Segment * const SG95521[] = {SGI387707, NULL};
const Segment * const SG95522[] = {SGI387708, SGI387709, SGI387710, SGI0, NULL};
const Segment * const SG95523[] = {SGI387711, SGI387712, SGI0, SGI387713, NULL};
const Segment * const SG95524[] = {SGI387714, SGI387715, SGI0, SGI387716, NULL};
const Segment * const SG95525[] = {SGI387717, SGI387718, SGI0, SGI387719, NULL};
const Segment * const SG95526[] = {SGI387720, SGI387721, SGI387722, SGI0, NULL};
const Segment * const SG95527[] = {SGI387723, SGI387724, SGI0, SGI387725, NULL};
const Segment * const SG95528[] = {SGI387726, SGI387727, SGI0, SGI387728, NULL};
const Segment * const SG95529[] = {SGI387729, SGI387730, SGI0, SGI387731, NULL};
const Segment * const SG95530[] = {SGI387732, SGI387733, SGI0, SGI387734, NULL};
const Segment * const SG95531[] = {SGI387735, SGI387736, SGI0, SGI0, NULL};
const Segment * const SG95532[] = {SGI387737, SGI387738, SGI0, SGI387739, NULL};
const Segment * const SG95533[] = {SGI387740, SGI387741, SGI0, SGI387742, NULL};
const Segment * const SG95534[] = {SGI387743, SGI387744, SGI0, SGI387745, NULL};
const Segment * const SG95535[] = {SGI387746, SGI387747, SGI0, SGI387748, NULL};
const Segment * const SG95536[] = {SGI387749, SGI387750, SGI0, SGI387751, NULL};
const Segment * const SG95537[] = {SGI387752, SGI387753, SGI0, SGI387754, NULL};
const Segment * const SG95538[] = {SGI387755, SGI387756, SGI0, SGI387757, NULL};
const Segment * const SG95539[] = {SGI387758, SGI387759, SGI387760, SGI0, NULL};
const Segment * const SG95540[] = {SGI387761, SGI387762, SGI0, SGI387763, NULL};
const Segment * const SG95541[] = {SGI387764, SGI387765, SGI0, SGI387766, NULL};
const Segment * const SG95542[] = {SGI387767, SGI387768, SGI0, SGI387769, NULL};
const Segment * const SG95543[] = {SGI387770, SGI387771, SGI0, SGI387772, NULL};
const Segment * const SG95544[] = {SGI387773, SGI387774, SGI0, SGI387775, NULL};
const Segment * const SG95545[] = {SGI387776, NULL};
const Segment * const SG95546[] = {SGI387777, SGI387778, SGI0, SGI387779, NULL};
const Segment * const SG95547[] = {SGI387780, SGI387781, SGI0, SGI387782, NULL};
const Segment * const SG95548[] = {SGI387783, SGI387784, SGI0, SGI387785, NULL};
const Segment * const SG95549[] = {SGI387786, SGI387787, SGI0, SGI387788, NULL};
const Segment * const SG95550[] = {SGI387789, SGI387790, SGI0, SGI387791, NULL};
const Segment * const SG95551[] = {SGI387792, SGI387793, SGI0, SGI387794, NULL};
const Segment * const SG95552[] = {SGI387795, SGI387796, SGI0, SGI387797, NULL};
const Segment * const SG95553[] = {SGI387798, SGI387799, SGI0, SGI387800, NULL};
const Segment * const SG95554[] = {SGI387801, SGI387802, SGI0, SGI387803, NULL};
const Segment * const SG95555[] = {SGI387804, SGI387805, SGI0, SGI387806, NULL};
const Segment * const SG95556[] = {SGI387807, SGI387808, SGI0, SGI387809, NULL};
const Segment * const SG95557[] = {SGI387810, SGI387811, SGI0, SGI387812, NULL};
const Segment * const SG95558[] = {SGI387813, SGI387814, SGI0, SGI387815, NULL};
const Segment * const SG95559[] = {SGI387816, SGI387817, SGI0, SGI387818, NULL};
const Segment * const SG95560[] = {SGI387819, SGI387820, SGI0, SGI387821, NULL};
const Segment * const SG95561[] = {SGI387822, SGI387823, SGI0, SGI387824, NULL};
const Segment * const SG95562[] = {SGI387825, SGI387826, SGI0, SGI387827, NULL};
const Segment * const SG95563[] = {SGI387828, NULL};
const Segment * const SG95564[] = {SGI387829, NULL};
const Segment * const SG95565[] = {SGI387830, NULL};
const Segment * const SG95566[] = {SGI387831, NULL};
const Segment * const SG95567[] = {SGI387832, NULL};
const Segment * const SG95568[] = {SGI387833, NULL};
const Segment * const SG95569[] = {SGI387834, NULL};
const Segment * const SG95570[] = {SGI387835, SGI387836, SGI0, SGI387837, NULL};
const Segment * const SG95571[] = {SGI387838, SGI387839, SGI0, SGI387840, NULL};
const Segment * const SG95572[] = {SGI387841, SGI387842, SGI0, SGI387843, NULL};
const Segment * const SG95573[] = {SGI387844, SGI387845, SGI0, SGI387846, NULL};
const Segment * const SG95574[] = {SGI387847, SGI387848, SGI0, SGI387849, NULL};
const Segment * const SG95575[] = {SGI387850, SGI387851, SGI0, SGI387852, NULL};
const Segment * const SG95576[] = {SGI387853, SGI387854, SGI0, SGI387855, NULL};
const Segment * const SG95577[] = {SGI387856, SGI387857, SGI0, SGI387858, NULL};
const Segment * const SG95578[] = {SGI387859, SGI387860, SGI0, SGI387861, NULL};
const Segment * const SG95579[] = {SGI387862, SGI387863, SGI0, SGI387864, NULL};
const Segment * const SG95580[] = {SGI387865, SGI387866, SGI0, SGI387867, NULL};
const Segment * const SG95581[] = {SGI387868, SGI387869, SGI0, SGI387870, NULL};
const Segment * const SG95582[] = {SGI387871, SGI387872, SGI0, SGI387873, NULL};
const Segment * const SG95583[] = {SGI387874, SGI387875, SGI0, SGI387876, NULL};
const Segment * const SG95584[] = {SGI387877, SGI387878, SGI0, SGI387879, NULL};
const Segment * const SG95585[] = {SGI387880, SGI387881, SGI0, SGI387882, NULL};
const Segment * const SG95586[] = {SGI387883, SGI387884, SGI0, SGI387885, NULL};
const Segment * const SG95587[] = {SGI387886, SGI387887, SGI0, SGI387888, NULL};
const Segment * const SG95588[] = {SGI387889, SGI387890, SGI0, SGI387891, NULL};
const Segment * const SG95589[] = {SGI387892, SGI387893, SGI0, SGI387894, NULL};
const Segment * const SG95590[] = {SGI387895, SGI387896, SGI0, SGI387897, NULL};
const Segment * const SG95591[] = {SGI387898, SGI387899, SGI0, SGI387900, NULL};
const Segment * const SG95592[] = {SGI387901, SGI387902, SGI0, SGI387903, NULL};
const Segment * const SG95593[] = {SGI387904, SGI387905, SGI0, SGI387906, NULL};
const Segment * const SG95594[] = {SGI387907, SGI387908, SGI0, SGI387909, NULL};
const Segment * const SG95595[] = {SGI387910, SGI387911, SGI0, SGI387912, NULL};
const Segment * const SG95596[] = {SGI387913, NULL};
const Segment * const SG95597[] = {SGI387914, NULL};
const Segment * const SG95598[] = {SGI387915, NULL};
const Segment * const SG95599[] = {SGI387916, NULL};
const Segment * const SG95600[] = {SGI387917, NULL};
const Segment * const SG95601[] = {SGI387918, SGI387919, SGI387920, SGI0, NULL};
const Segment * const SG95602[] = {SGI387921, SGI387922, SGI387923, SGI0, NULL};
const Segment * const SG95603[] = {SGI387924, SGI387925, SGI387926, SGI0, NULL};
const Segment * const SG95604[] = {SGI387927, SGI387928, SGI387929, SGI0, NULL};
const Segment * const SG95605[] = {SGI387930, SGI387931, SGI387932, SGI0, NULL};
const Segment * const SG95606[] = {SGI387933, SGI387934, SGI387935, SGI0, NULL};
const Segment * const SG95607[] = {SGI387936, SGI387937, SGI387938, SGI0, NULL};
const Segment * const SG95608[] = {SGI387939, SGI387940, SGI387941, SGI0, NULL};
const Segment * const SG95609[] = {SGI387942, SGI387943, SGI387944, SGI0, NULL};
const Segment * const SG95610[] = {SGI387945, SGI387946, SGI0, SGI0, NULL};
const Segment * const SG95611[] = {SGI387947, SGI387948, SGI0, SGI387949, NULL};
const Segment * const SG95612[] = {SGI387950, SGI387951, SGI387952, SGI387953, NULL};
const Segment * const SG95613[] = {SGI387954, SGI387955, SGI387956, SGI387957, NULL};
const Segment * const SG95614[] = {SGI387958, SGI387959, SGI387960, SGI387961, NULL};
const Segment * const SG95615[] = {SGI387962, SGI387963, SGI0, SGI387964, NULL};
const Segment * const SG95616[] = {SGI387965, SGI387966, SGI387967, SGI387968, NULL};
const Segment * const SG95617[] = {SGI387969, SGI387970, SGI0, SGI387971, NULL};
const Segment * const SG95618[] = {SGI387972, SGI387973, SGI387974, SGI387975, NULL};
const Segment * const SG95619[] = {SGI387976, SGI387977, SGI387978, SGI387979, NULL};
const Segment * const SG95620[] = {SGI387980, SGI387981, SGI0, SGI387982, NULL};
const Segment * const SG95621[] = {SGI387983, SGI387984, SGI0, SGI387985, NULL};
const Segment * const SG95622[] = {SGI387986, SGI387987, SGI0, SGI387988, NULL};
const Segment * const SG95623[] = {SGI387989, SGI387990, SGI387991, SGI387992, NULL};
const Segment * const SG95624[] = {SGI387993, SGI387994, SGI0, SGI387995, NULL};
const Segment * const SG95625[] = {SGI387996, SGI387997, SGI0, SGI387998, NULL};
const Segment * const SG95626[] = {SGI387999, SGI388000, SGI0, SGI388001, NULL};
const Segment * const SG95627[] = {SGI388002, SGI388003, SGI0, SGI388004, NULL};
const Segment * const SG95628[] = {SGI388005, SGI388006, SGI388007, SGI388008, NULL};
const Segment * const SG95629[] = {SGI388009, SGI388010, SGI0, SGI388011, NULL};
const Segment * const SG95630[] = {SGI388012, SGI388013, SGI0, SGI388014, NULL};
const Segment * const SG95631[] = {SGI388015, SGI388016, SGI0, SGI388017, NULL};
const Segment * const SG95632[] = {SGI388018, SGI388019, SGI0, SGI388020, NULL};
const Segment * const SG95633[] = {SGI388021, SGI388022, SGI0, SGI388023, NULL};
const Segment * const SG95634[] = {SGI388024, SGI388025, SGI0, SGI388026, NULL};
const Segment * const SG95635[] = {SGI388027, SGI388028, SGI0, SGI388029, NULL};
const Segment * const SG95636[] = {SGI388030, SGI388031, SGI0, SGI388032, NULL};
const Segment * const SG95637[] = {SGI388033, SGI388034, SGI388035, SGI388036, NULL};
const Segment * const SG95638[] = {SGI388037, SGI388038, SGI0, SGI388039, NULL};
const Segment * const SG95639[] = {SGI388040, SGI388041, SGI0, SGI388042, NULL};
const Segment * const SG95640[] = {SGI388043, SGI388044, SGI0, SGI388045, NULL};
const Segment * const SG95641[] = {SGI388046, SGI388047, SGI0, SGI388048, NULL};
const Segment * const SG95642[] = {SGI388049, SGI388050, SGI0, SGI388051, NULL};
const Segment * const SG95643[] = {SGI388052, SGI388053, SGI0, SGI388054, NULL};
const Segment * const SG95644[] = {SGI388055, SGI388056, SGI0, SGI388057, NULL};
const Segment * const SG95645[] = {SGI388058, SGI388059, SGI0, SGI388060, NULL};
const Segment * const SG95646[] = {SGI388061, SGI388062, SGI388063, SGI388064, NULL};
const Segment * const SG95647[] = {SGI388065, SGI388066, SGI388067, SGI388068, NULL};
const Segment * const SG95648[] = {SGI388069, SGI388070, SGI0, SGI388071, NULL};
const Segment * const SG95649[] = {SGI388072, SGI388073, SGI0, SGI388074, NULL};
const Segment * const SG95650[] = {SGI388075, SGI388076, SGI0, SGI388077, NULL};
const Segment * const SG95651[] = {SGI388078, SGI388079, SGI0, SGI388080, NULL};
const Segment * const SG95652[] = {SGI388081, SGI388082, SGI0, SGI388083, NULL};
const Segment * const SG95653[] = {SGI388084, SGI388085, SGI0, SGI388086, NULL};
const Segment * const SG95654[] = {SGI388087, SGI388088, SGI0, SGI388089, NULL};
const Segment * const SG95655[] = {SGI388090, SGI388091, SGI0, SGI388092, NULL};
const Segment * const SG95656[] = {SGI388093, SGI388094, SGI0, SGI388095, NULL};
const Segment * const SG95657[] = {SGI388096, SGI388097, SGI0, SGI388098, NULL};
const Segment * const SG95658[] = {SGI388099, SGI388100, SGI0, SGI388101, NULL};
const Segment * const SG95659[] = {SGI388102, SGI388103, SGI388104, SGI388105, NULL};
const Segment * const SG95660[] = {SGI388106, SGI388107, SGI0, SGI388108, NULL};
const Segment * const SG95661[] = {SGI388109, SGI388110, SGI0, SGI388111, NULL};
const Segment * const SG95662[] = {SGI388112, NULL};
const Segment * const SG95663[] = {SGI388113, NULL};
const Segment * const SG95664[] = {SGI388114, SGI388115, SGI388115, SGI388116, NULL};
const Segment * const SG95665[] = {SGI388117, SGI388118, SGI388119, SGI0, NULL};
const Segment * const SG95666[] = {SGI388120, SGI388121, SGI388121, SGI0, NULL};
const Segment * const SG95667[] = {SGI388122, SGI388123, SGI0, SGI0, NULL};
const Segment * const SG95668[] = {SGI388124, SGI388125, SGI388125, SGI388126, NULL};
const Segment * const SG95669[] = {SGI388127, SGI388128, SGI388129, SGI388130, NULL};
const Segment * const SG95670[] = {SGI388131, SGI388132, SGI388132, SGI388133, NULL};
const Segment * const SG95671[] = {SGI388134, NULL};
const Segment * const SG95672[] = {SGI388135, SGI388136, SGI388137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95673[] = {SGI388138, SGI388139, SGI388139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95674[] = {SGI388140, SGI388141, SGI388142, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95675[] = {SGI388143, NULL};
const Segment * const SG95676[] = {SGI388144, NULL};
const Segment * const SG95677[] = {SGI388145, NULL};
const Segment * const SG95678[] = {SGI388146, NULL};
const Segment * const SG95679[] = {SGI388147, NULL};
const Segment * const SG95680[] = {SGI388148, SGI388149, SGI388150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95681[] = {SGI388151, SGI388152, SGI388153, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95682[] = {SGI388154, NULL};
const Segment * const SG95683[] = {SGI388155, NULL};
const Segment * const SG95684[] = {SGI388156, SGI388157, SGI388158, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95685[] = {SGI388159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95686[] = {SGI388160, NULL};
const Segment * const SG95687[] = {SGI388161, NULL};
const Segment * const SG95688[] = {SGI388162, SGI388163, SGI388164, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95689[] = {SGI388165, NULL};
const Segment * const SG95690[] = {SGI388166, NULL};
const Segment * const SG95691[] = {SGI388167, NULL};
const Segment * const SG95692[] = {SGI388168, NULL};
const Segment * const SG95693[] = {SGI388169, NULL};
const Segment * const SG95694[] = {SGI388170, NULL};
const Segment * const SG95695[] = {SGI388171, NULL};
const Segment * const SG95696[] = {SGI388172, NULL};
const Segment * const SG95697[] = {SGI388173, SGI388174, SGI388175, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388176, SGI0, SGI0, NULL};
const Segment * const SG95698[] = {SGI388177, SGI388178, SGI388178, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388179, SGI0, SGI0, NULL};
const Segment * const SG95699[] = {SGI388180, SGI388181, SGI388181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388182, SGI0, SGI0, NULL};
const Segment * const SG95700[] = {SGI388183, NULL};
const Segment * const SG95701[] = {SGI388184, NULL};
const Segment * const SG95702[] = {SGI388185, SGI388186, SGI388187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95703[] = {SGI388188, SGI388189, SGI388189, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95704[] = {SGI388190, SGI388191, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95705[] = {SGI388192, NULL};
const Segment * const SG95706[] = {SGI388193, NULL};
const Segment * const SG95707[] = {SGI388194, NULL};
const Segment * const SG95708[] = {SGI388195, NULL};
const Segment * const SG95709[] = {SGI388196, NULL};
const Segment * const SG95710[] = {SGI388197, NULL};
const Segment * const SG95711[] = {SGI388198, SGI388199, SGI388200, SGI388201, NULL};
const Segment * const SG95712[] = {SGI388202, NULL};
const Segment * const SG95713[] = {SGI388203, NULL};
const Segment * const SG95714[] = {SGI388204, NULL};
const Segment * const SG95715[] = {SGI388205, NULL};
const Segment * const SG95716[] = {SGI388206, NULL};
const Segment * const SG95717[] = {SGI388207, NULL};
const Segment * const SG95718[] = {SGI388208, NULL};
const Segment * const SG95719[] = {SGI388209, NULL};
const Segment * const SG95720[] = {SGI388210, NULL};
const Segment * const SG95721[] = {SGI388211, NULL};
const Segment * const SG95722[] = {SGI388212, SGI388213, SGI388214, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95723[] = {SGI388215, NULL};
const Segment * const SG95724[] = {SGI388216, NULL};
const Segment * const SG95725[] = {SGI388217, NULL};
const Segment * const SG95726[] = {SGI388218, NULL};
const Segment * const SG95727[] = {SGI388219, NULL};
const Segment * const SG95728[] = {SGI388220, NULL};
const Segment * const SG95729[] = {SGI388221, SGI388222, SGI388222, SGI388223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95730[] = {SGI388224, SGI388225, SGI388225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95731[] = {SGI388226, NULL};
const Segment * const SG95732[] = {SGI388227, NULL};
const Segment * const SG95733[] = {SGI388228, NULL};
const Segment * const SG95734[] = {SGI388229, NULL};
const Segment * const SG95735[] = {SGI388230, NULL};
const Segment * const SG95736[] = {SGI388231, NULL};
const Segment * const SG95737[] = {SGI388232, NULL};
const Segment * const SG95738[] = {SGI388233, NULL};
const Segment * const SG95739[] = {SGI388234, NULL};
const Segment * const SG95740[] = {SGI388235, NULL};
const Segment * const SG95741[] = {SGI388236, NULL};
const Segment * const SG95742[] = {SGI388237, NULL};
const Segment * const SG95743[] = {SGI388238, SGI388239, SGI388240, SGI0, NULL};
const Segment * const SG95744[] = {SGI388241, SGI388242, SGI388243, SGI0, NULL};
const Segment * const SG95745[] = {SGI388244, SGI388245, SGI388246, SGI0, NULL};
const Segment * const SG95746[] = {SGI388247, SGI388248, SGI388249, SGI0, NULL};
const Segment * const SG95747[] = {SGI388250, SGI388251, SGI388252, SGI0, NULL};
const Segment * const SG95748[] = {SGI388253, SGI388254, SGI388255, SGI0, NULL};
const Segment * const SG95749[] = {SGI388256, SGI388257, SGI0, SGI0, NULL};
const Segment * const SG95750[] = {SGI388258, SGI388259, SGI0, SGI388260, NULL};
const Segment * const SG95751[] = {SGI388261, SGI388262, SGI388263, SGI388264, NULL};
const Segment * const SG95752[] = {SGI388265, SGI388266, SGI0, SGI388267, NULL};
const Segment * const SG95753[] = {SGI388268, SGI388269, SGI0, SGI388270, NULL};
const Segment * const SG95754[] = {SGI388271, SGI388272, SGI388273, SGI0, NULL};
const Segment * const SG95755[] = {SGI388274, SGI388275, SGI0, SGI388276, NULL};
const Segment * const SG95756[] = {SGI388277, SGI388278, SGI0, SGI388279, NULL};
const Segment * const SG95757[] = {SGI388280, SGI388281, SGI0, SGI388282, NULL};
const Segment * const SG95758[] = {SGI388283, SGI388284, SGI0, SGI388285, NULL};
const Segment * const SG95759[] = {SGI388286, SGI388287, SGI0, SGI388288, NULL};
const Segment * const SG95760[] = {SGI388289, SGI388290, SGI388291, SGI388292, NULL};
const Segment * const SG95761[] = {SGI388293, SGI388294, SGI0, SGI388295, NULL};
const Segment * const SG95762[] = {SGI388296, SGI388297, SGI0, SGI388298, NULL};
const Segment * const SG95763[] = {SGI388299, SGI388300, SGI0, SGI388301, NULL};
const Segment * const SG95764[] = {SGI388302, SGI388303, SGI0, SGI388304, NULL};
const Segment * const SG95765[] = {SGI388305, SGI388306, SGI0, SGI388307, NULL};
const Segment * const SG95766[] = {SGI388308, SGI388309, SGI0, SGI388310, NULL};
const Segment * const SG95767[] = {SGI388311, SGI388312, SGI0, SGI388313, NULL};
const Segment * const SG95768[] = {SGI388314, SGI388315, SGI0, SGI388316, NULL};
const Segment * const SG95769[] = {SGI388317, SGI388318, SGI0, SGI388319, NULL};
const Segment * const SG95770[] = {SGI388320, NULL};
const Segment * const SG95771[] = {SGI388321, NULL};
const Segment * const SG95772[] = {SGI388322, NULL};
const Segment * const SG95773[] = {SGI388323, NULL};
const Segment * const SG95774[] = {SGI388324, NULL};
const Segment * const SG95775[] = {SGI388325, NULL};
const Segment * const SG95776[] = {SGI388326, NULL};
const Segment * const SG95777[] = {SGI388327, NULL};
const Segment * const SG95778[] = {SGI388328, NULL};
const Segment * const SG95779[] = {SGI388329, NULL};
const Segment * const SG95780[] = {SGI388330, NULL};
const Segment * const SG95781[] = {SGI388331, NULL};
const Segment * const SG95782[] = {SGI388332, NULL};
const Segment * const SG95783[] = {SGI388333, NULL};
const Segment * const SG95784[] = {SGI388334, NULL};
const Segment * const SG95785[] = {SGI388335, NULL};
const Segment * const SG95786[] = {SGI388336, NULL};
const Segment * const SG95787[] = {SGI388337, NULL};
const Segment * const SG95788[] = {SGI388338, NULL};
const Segment * const SG95789[] = {SGI388339, NULL};
const Segment * const SG95790[] = {SGI388340, NULL};
const Segment * const SG95791[] = {SGI388341, NULL};
const Segment * const SG95792[] = {SGI388342, NULL};
const Segment * const SG95793[] = {SGI388343, NULL};
const Segment * const SG95794[] = {SGI388344, NULL};
const Segment * const SG95795[] = {SGI388345, NULL};
const Segment * const SG95796[] = {SGI388346, NULL};
const Segment * const SG95797[] = {SGI388347, NULL};
const Segment * const SG95798[] = {SGI388348, NULL};
const Segment * const SG95799[] = {SGI388349, NULL};
const Segment * const SG95800[] = {SGI388350, SGI388351, SGI388351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388352, NULL};
const Segment * const SG95801[] = {SGI388353, SGI388354, SGI388354, SGI388355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388356, NULL};
const Segment * const SG95802[] = {SGI388357, SGI388358, SGI388359, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95803[] = {SGI388360, NULL};
const Segment * const SG95804[] = {SGI388361, NULL};
const Segment * const SG95805[] = {SGI388362, NULL};
const Segment * const SG95806[] = {SGI388363, NULL};
const Segment * const SG95807[] = {SGI388364, SGI388365, SGI388366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388367, SGI0, SGI0, NULL};
const Segment * const SG95808[] = {SGI388368, SGI0, SGI0, SGI388369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95809[] = {SGI388370, SGI388371, SGI0, SGI388372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95810[] = {SGI388373, SGI388374, SGI0, SGI388375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95811[] = {SGI388376, SGI388377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95812[] = {SGI388378, NULL};
const Segment * const SG95813[] = {SGI388379, NULL};
const Segment * const SG95814[] = {SGI388380, NULL};
const Segment * const SG95815[] = {SGI388381, NULL};
const Segment * const SG95816[] = {SGI388382, SGI388383, SGI388384, SGI388384, SGI0, NULL};
const Segment * const SG95817[] = {SGI388385, SGI388386, SGI388387, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95818[] = {SGI388388, SGI388389, SGI388389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95819[] = {SGI388390, SGI388391, SGI388391, SGI388392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95820[] = {SGI388393, SGI388394, SGI388394, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388395, NULL};
const Segment * const SG95821[] = {SGI388396, NULL};
const Segment * const SG95822[] = {SGI388397, NULL};
const Segment * const SG95823[] = {SGI388398, NULL};
const Segment * const SG95824[] = {SGI388399, NULL};
const Segment * const SG95825[] = {SGI388400, NULL};
const Segment * const SG95826[] = {SGI388401, NULL};
const Segment * const SG95827[] = {SGI388402, NULL};
const Segment * const SG95828[] = {SGI388403, NULL};
const Segment * const SG95829[] = {SGI388404, NULL};
const Segment * const SG95830[] = {SGI388405, NULL};
const Segment * const SG95831[] = {SGI388406, NULL};
const Segment * const SG95832[] = {SGI388407, NULL};
const Segment * const SG95833[] = {SGI388408, NULL};
const Segment * const SG95834[] = {SGI388409, NULL};
const Segment * const SG95835[] = {SGI388410, NULL};
const Segment * const SG95836[] = {SGI388411, NULL};
const Segment * const SG95837[] = {SGI388412, NULL};
const Segment * const SG95838[] = {SGI388413, NULL};
const Segment * const SG95839[] = {SGI388414, NULL};
const Segment * const SG95840[] = {SGI388415, NULL};
const Segment * const SG95841[] = {SGI388416, NULL};
const Segment * const SG95842[] = {SGI388417, NULL};
const Segment * const SG95843[] = {SGI388418, NULL};
const Segment * const SG95844[] = {SGI388419, NULL};
const Segment * const SG95845[] = {SGI388420, NULL};
const Segment * const SG95846[] = {SGI388421, NULL};
const Segment * const SG95847[] = {SGI388422, SGI388423, SGI388423, SGI0, NULL};
const Segment * const SG95848[] = {SGI388424, SGI388425, SGI0, SGI0, NULL};
const Segment * const SG95849[] = {SGI388426, NULL};
const Segment * const SG95850[] = {SGI388427, NULL};
const Segment * const SG95851[] = {SGI388428, NULL};
const Segment * const SG95852[] = {SGI388429, NULL};
const Segment * const SG95853[] = {SGI388430, NULL};
const Segment * const SG95854[] = {SGI388431, NULL};
const Segment * const SG95855[] = {SGI388432, NULL};
const Segment * const SG95856[] = {SGI388433, NULL};
const Segment * const SG95857[] = {SGI388434, NULL};
const Segment * const SG95858[] = {SGI388435, SGI388436, SGI388437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95859[] = {SGI388439, SGI388440, SGI388440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95860[] = {SGI388442, NULL};
const Segment * const SG95861[] = {SGI388443, NULL};
const Segment * const SG95862[] = {SGI388444, NULL};
const Segment * const SG95863[] = {SGI388445, NULL};
const Segment * const SG95864[] = {SGI388446, NULL};
const Segment * const SG95865[] = {SGI388447, NULL};
const Segment * const SG95866[] = {SGI388448, NULL};
const Segment * const SG95867[] = {SGI388449, NULL};
const Segment * const SG95868[] = {SGI388450, NULL};
const Segment * const SG95869[] = {SGI388451, NULL};
const Segment * const SG95870[] = {SGI388452, NULL};
const Segment * const SG95871[] = {SGI388453, NULL};
const Segment * const SG95872[] = {SGI388454, NULL};
const Segment * const SG95873[] = {SGI388455, NULL};
const Segment * const SG95874[] = {SGI388456, SGI388457, SGI388458, SGI0, NULL};
const Segment * const SG95875[] = {SGI388459, SGI388460, SGI388461, SGI388462, NULL};
const Segment * const SG95876[] = {SGI388463, SGI388464, SGI388465, SGI388466, NULL};
const Segment * const SG95877[] = {SGI388467, SGI388468, SGI388469, SGI388470, NULL};
const Segment * const SG95878[] = {SGI388471, SGI388472, SGI388473, SGI388474, NULL};
const Segment * const SG95879[] = {SGI388475, NULL};
const Segment * const SG95880[] = {SGI388476, NULL};
const Segment * const SG95881[] = {SGI388477, NULL};
const Segment * const SG95882[] = {SGI388478, NULL};
const Segment * const SG95883[] = {SGI388479, SGI388480, SGI0, SGI0, NULL};
const Segment * const SG95884[] = {SGI388481, SGI388482, SGI0, SGI388483, NULL};
const Segment * const SG95885[] = {SGI388484, SGI388485, SGI0, SGI388486, NULL};
const Segment * const SG95886[] = {SGI388487, NULL};
const Segment * const SG95887[] = {SGI388488, NULL};
const Segment * const SG95888[] = {SGI388489, SGI388490, SGI388491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95889[] = {SGI388493, SGI388494, SGI388495, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388496, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95890[] = {SGI388497, NULL};
const Segment * const SG95891[] = {SGI388498, NULL};
const Segment * const SG95892[] = {SGI388499, NULL};
const Segment * const SG95893[] = {SGI388500, NULL};
const Segment * const SG95894[] = {SGI388501, NULL};
const Segment * const SG95895[] = {SGI388502, NULL};
const Segment * const SG95896[] = {SGI388503, NULL};
const Segment * const SG95897[] = {SGI388504, NULL};
const Segment * const SG95898[] = {SGI388505, NULL};
const Segment * const SG95899[] = {SGI388506, NULL};
const Segment * const SG95900[] = {SGI388507, NULL};
const Segment * const SG95901[] = {SGI388508, NULL};
const Segment * const SG95902[] = {SGI388509, NULL};
const Segment * const SG95903[] = {SGI388510, NULL};
const Segment * const SG95904[] = {SGI388511, NULL};
const Segment * const SG95905[] = {SGI388512, NULL};
const Segment * const SG95906[] = {SGI388513, NULL};
const Segment * const SG95907[] = {SGI388514, NULL};
const Segment * const SG95908[] = {SGI388515, NULL};
const Segment * const SG95909[] = {SGI388516, NULL};
const Segment * const SG95910[] = {SGI388517, NULL};
const Segment * const SG95911[] = {SGI388518, NULL};
const Segment * const SG95912[] = {SGI388519, NULL};
const Segment * const SG95913[] = {SGI388520, NULL};
const Segment * const SG95914[] = {SGI388521, NULL};
const Segment * const SG95915[] = {SGI388522, NULL};
const Segment * const SG95916[] = {SGI388523, NULL};
const Segment * const SG95917[] = {SGI388524, NULL};
const Segment * const SG95918[] = {SGI388525, NULL};
const Segment * const SG95919[] = {SGI388526, NULL};
const Segment * const SG95920[] = {SGI388527, NULL};
const Segment * const SG95921[] = {SGI388528, NULL};
const Segment * const SG95922[] = {SGI388529, NULL};
const Segment * const SG95923[] = {SGI388530, NULL};
const Segment * const SG95924[] = {SGI388531, NULL};
const Segment * const SG95925[] = {SGI388532, NULL};
const Segment * const SG95926[] = {SGI388533, NULL};
const Segment * const SG95927[] = {SGI388534, SGI388535, SGI388536, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388537, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95928[] = {SGI388538, SGI388539, SGI388539, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388540, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95929[] = {SGI388541, NULL};
const Segment * const SG95930[] = {SGI388542, SGI0, SGI0, SGI388543, SGI0, NULL};
const Segment * const SG95931[] = {SGI388544, SGI0, SGI0, SGI388545, SGI0, NULL};
const Segment * const SG95932[] = {SGI388546, NULL};
const Segment * const SG95933[] = {SGI388547, NULL};
const Segment * const SG95934[] = {SGI388548, NULL};
const Segment * const SG95935[] = {SGI388549, NULL};
const Segment * const SG95936[] = {SGI388550, NULL};
const Segment * const SG95937[] = {SGI388551, NULL};
const Segment * const SG95938[] = {SGI388552, NULL};
const Segment * const SG95939[] = {SGI388553, NULL};
const Segment * const SG95940[] = {SGI388554, NULL};
const Segment * const SG95941[] = {SGI388555, NULL};
const Segment * const SG95942[] = {SGI388556, NULL};
const Segment * const SG95943[] = {SGI388557, NULL};
const Segment * const SG95944[] = {SGI388558, NULL};
const Segment * const SG95945[] = {SGI388559, NULL};
const Segment * const SG95946[] = {SGI388560, NULL};
const Segment * const SG95947[] = {SGI388561, NULL};
const Segment * const SG95948[] = {SGI388562, NULL};
const Segment * const SG95949[] = {SGI388563, NULL};
const Segment * const SG95950[] = {SGI388564, SGI388565, SGI388565, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388566, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95951[] = {SGI388567, SGI388568, SGI388568, SGI0, SGI0, SGI0, SGI0, SGI388569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95952[] = {SGI388570, SGI388571, SGI388572, SGI0, SGI0, SGI0, SGI0, SGI388573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95953[] = {SGI388574, SGI0, SGI388575, SGI0, SGI0, SGI0, SGI0, SGI388576, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95954[] = {SGI388577, SGI388578, SGI388578, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95955[] = {SGI388579, SGI388580, SGI388581, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95956[] = {SGI388582, SGI388583, SGI388583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388584, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95957[] = {SGI388585, SGI388586, SGI388587, SGI388587, SGI0, NULL};
const Segment * const SG95958[] = {SGI388588, SGI388589, SGI388590, SGI388590, SGI0, NULL};
const Segment * const SG95959[] = {SGI388591, SGI388592, SGI388592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388593, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95960[] = {SGI388594, SGI0, SGI37652, SGI388595, SGI0, NULL};
const Segment * const SG95961[] = {SGI388596, SGI0, SGI37656, SGI388597, SGI0, NULL};
const Segment * const SG95962[] = {SGI388598, SGI0, SGI37660, SGI388599, SGI0, NULL};
const Segment * const SG95963[] = {SGI388600, SGI0, SGI37664, SGI388601, SGI0, NULL};
const Segment * const SG95964[] = {SGI388602, SGI0, SGI37668, SGI388603, SGI0, NULL};
const Segment * const SG95965[] = {SGI388604, SGI0, SGI37672, SGI388605, SGI0, NULL};
const Segment * const SG95966[] = {SGI388606, SGI388607, SGI388607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388608, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95967[] = {SGI388609, SGI388610, SGI0, SGI388611, SGI0, NULL};
const Segment * const SG95968[] = {SGI388612, SGI388613, SGI0, SGI388614, NULL};
const Segment * const SG95969[] = {SGI388615, SGI388616, SGI0, SGI388617, SGI0, NULL};
const Segment * const SG95970[] = {SGI388618, SGI388619, SGI388619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388620, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95971[] = {SGI388621, SGI388622, SGI0, SGI388623, NULL};
const Segment * const SG95972[] = {SGI388624, SGI388625, SGI0, SGI388626, SGI0, NULL};
const Segment * const SG95973[] = {SGI388627, SGI388628, SGI388628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95974[] = {SGI388629, SGI388630, SGI388630, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388631, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95975[] = {SGI388632, SGI388633, SGI0, SGI388634, SGI0, NULL};
const Segment * const SG95976[] = {SGI388635, SGI388636, SGI388636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388637, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95977[] = {SGI388638, SGI388639, SGI388640, SGI388640, SGI0, NULL};
const Segment * const SG95978[] = {SGI388641, SGI388642, SGI388643, SGI388643, SGI0, NULL};
const Segment * const SG95979[] = {SGI388644, SGI388645, SGI388645, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388646, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95980[] = {SGI388647, SGI388648, SGI388648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388649, NULL};
const Segment * const SG95981[] = {SGI388650, SGI388651, SGI388651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95982[] = {SGI388652, SGI388653, SGI388653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388654, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95983[] = {SGI388655, SGI388656, SGI388656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388657, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95984[] = {SGI388658, SGI0, SGI0, SGI388659, SGI0, NULL};
const Segment * const SG95985[] = {SGI388660, SGI0, SGI0, SGI388661, SGI0, NULL};
const Segment * const SG95986[] = {SGI388662, SGI388663, SGI388663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388664, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95987[] = {SGI388665, SGI388666, SGI0, SGI388667, SGI0, NULL};
const Segment * const SG95988[] = {SGI388668, SGI388669, SGI388669, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388670, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95989[] = {SGI388671, SGI388672, SGI0, SGI388673, SGI0, NULL};
const Segment * const SG95990[] = {SGI388674, SGI388675, SGI0, SGI388676, SGI0, NULL};
const Segment * const SG95991[] = {SGI388677, SGI388678, SGI388679, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388680, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95992[] = {SGI388681, SGI388682, SGI388683, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388684, SGI0, SGI0, SGI0, SGI388685, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95993[] = {SGI388686, SGI388687, SGI388688, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388689, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95994[] = {SGI388690, SGI388691, SGI388691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG95995[] = {SGI388692, SGI388693, SGI0, SGI388694, SGI0, NULL};
const Segment * const SG95996[] = {SGI388695, SGI388696, SGI388697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388698, SGI0, SGI0, NULL};
const Segment * const SG95997[] = {SGI388699, SGI388700, SGI388701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388702, SGI0, SGI0, NULL};
const Segment * const SG95998[] = {SGI388703, SGI388704, SGI0, SGI388705, SGI0, NULL};
const Segment * const SG95999[] = {SGI388706, SGI388707, SGI388708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96000[] = {SGI388710, SGI388711, SGI388711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96001[] = {SGI388713, SGI388714, SGI0, SGI388715, SGI0, NULL};
const Segment * const SG96002[] = {SGI388716, SGI0, SGI41131, SGI388717, SGI0, NULL};
const Segment * const SG96003[] = {SGI388718, SGI0, SGI41135, SGI388719, SGI0, NULL};
const Segment * const SG96004[] = {SGI388720, SGI0, SGI41139, SGI388721, SGI0, NULL};
const Segment * const SG96005[] = {SGI388722, SGI0, SGI41143, SGI388723, SGI0, NULL};
const Segment * const SG96006[] = {SGI388724, SGI0, SGI41147, SGI388725, SGI0, NULL};
const Segment * const SG96007[] = {SGI388726, SGI388727, SGI0, SGI388728, SGI0, NULL};
const Segment * const SG96008[] = {SGI388729, SGI388730, SGI0, SGI388731, SGI0, NULL};
const Segment * const SG96009[] = {SGI388732, SGI388733, SGI388733, SGI0, NULL};
const Segment * const SG96010[] = {SGI388734, SGI388735, SGI388736, SGI388736, SGI0, NULL};
const Segment * const SG96011[] = {SGI388737, SGI388738, SGI388738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388739, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96012[] = {SGI388740, SGI388741, SGI388742, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388743, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96013[] = {SGI388744, SGI388745, SGI388746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388747, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96014[] = {SGI388748, SGI388749, SGI388749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388750, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96015[] = {SGI388751, SGI388752, SGI388753, SGI388753, SGI0, NULL};
const Segment * const SG96016[] = {SGI388754, SGI388755, SGI388755, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96017[] = {SGI388757, SGI388758, SGI388759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388760, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96018[] = {SGI388761, SGI388762, SGI0, SGI388763, SGI0, NULL};
const Segment * const SG96019[] = {SGI388764, SGI388765, SGI388765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388766, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96020[] = {SGI388767, SGI388768, SGI388768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388769, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96021[] = {SGI388770, SGI388771, SGI388772, SGI388773, SGI0, NULL};
const Segment * const SG96022[] = {SGI388774, SGI388775, SGI0, SGI388776, SGI0, NULL};
const Segment * const SG96023[] = {SGI388777, SGI388778, SGI0, SGI388779, SGI0, NULL};
const Segment * const SG96024[] = {SGI388780, SGI388781, SGI388782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388783, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96025[] = {SGI388784, SGI388785, SGI388786, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96026[] = {SGI388788, SGI388789, SGI388789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388790, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96027[] = {SGI388791, SGI388792, SGI0, SGI388793, SGI0, NULL};
const Segment * const SG96028[] = {SGI388794, SGI388795, SGI388796, SGI388797, SGI0, NULL};
const Segment * const SG96029[] = {SGI388798, SGI388799, SGI388799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96030[] = {SGI388800, SGI388801, SGI388801, SGI388802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96031[] = {SGI388803, SGI388804, SGI0, SGI388805, SGI0, NULL};
const Segment * const SG96032[] = {SGI388806, SGI388807, SGI388808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388809, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96033[] = {SGI388810, SGI388811, SGI0, SGI388812, SGI0, NULL};
const Segment * const SG96034[] = {SGI388813, SGI388814, SGI388815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388816, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96035[] = {SGI388817, SGI388818, SGI388819, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388820, SGI0, SGI0, SGI0, SGI388821, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96036[] = {SGI388822, SGI388823, SGI388824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388825, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96037[] = {SGI388826, SGI388827, SGI388827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI388828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96038[] = {SGI388829, SGI388830, SGI388830, SGI0, NULL};
const Segment * const SG96039[] = {SGI388831, SGI388832, SGI388832, SGI0, NULL};
const Segment * const SG96040[] = {SGI388833, SGI388834, SGI388835, SGI0, NULL};
const Segment * const SG96041[] = {SGI388836, SGI388837, SGI0, SGI0, NULL};
const Segment * const SG96042[] = {SGI388838, SGI0, SGI388839, SGI0, NULL};
const Segment * const SG96043[] = {SGI388840, SGI0, SGI388841, SGI0, NULL};
const Segment * const SG96044[] = {SGI388842, SGI0, SGI388843, SGI0, NULL};
const Segment * const SG96045[] = {SGI388844, SGI0, SGI388845, SGI0, NULL};
const Segment * const SG96046[] = {SGI388846, SGI0, SGI388847, SGI0, NULL};
const Segment * const SG96047[] = {SGI388848, SGI388849, SGI388850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96048[] = {SGI388851, SGI0, SGI388852, SGI0, NULL};
const Segment * const SG96049[] = {SGI388853, SGI0, SGI388854, SGI0, NULL};
const Segment * const SG96050[] = {SGI388855, SGI0, SGI388856, SGI0, NULL};
const Segment * const SG96051[] = {SGI388857, SGI0, SGI388858, SGI0, NULL};
const Segment * const SG96052[] = {SGI388859, SGI0, SGI388860, SGI0, NULL};
const Segment * const SG96053[] = {SGI388861, SGI0, SGI388862, SGI0, NULL};
const Segment * const SG96054[] = {SGI388863, SGI0, SGI388864, SGI0, NULL};
const Segment * const SG96055[] = {SGI388865, SGI0, SGI388866, SGI0, NULL};
const Segment * const SG96056[] = {SGI388867, SGI0, SGI388868, SGI0, NULL};
const Segment * const SG96057[] = {SGI388869, SGI0, SGI388870, SGI0, NULL};
const Segment * const SG96058[] = {SGI388871, SGI0, SGI388872, SGI0, NULL};
const Segment * const SG96059[] = {SGI388873, SGI0, SGI388874, SGI0, NULL};
const Segment * const SG96060[] = {SGI388875, SGI0, SGI388876, SGI0, NULL};
const Segment * const SG96061[] = {SGI388877, SGI0, SGI388878, SGI0, NULL};
const Segment * const SG96062[] = {SGI388879, SGI0, SGI388880, SGI0, NULL};
const Segment * const SG96063[] = {SGI388881, SGI0, SGI388882, SGI0, NULL};
const Segment * const SG96064[] = {SGI388883, SGI0, SGI388884, SGI0, NULL};
const Segment * const SG96065[] = {SGI388885, SGI0, SGI388886, SGI0, NULL};
const Segment * const SG96066[] = {SGI388887, SGI0, SGI388888, SGI0, NULL};
const Segment * const SG96067[] = {SGI388889, SGI0, SGI388890, SGI0, NULL};
const Segment * const SG96068[] = {SGI388891, SGI0, SGI388892, SGI0, NULL};
const Segment * const SG96069[] = {SGI388893, SGI0, SGI388894, SGI0, NULL};
const Segment * const SG96070[] = {SGI388895, SGI0, SGI388896, SGI0, NULL};
const Segment * const SG96071[] = {SGI388897, SGI0, SGI388898, SGI0, NULL};
const Segment * const SG96072[] = {SGI388899, SGI0, SGI388900, SGI0, NULL};
const Segment * const SG96073[] = {SGI388901, SGI0, SGI388902, SGI0, NULL};
const Segment * const SG96074[] = {SGI388903, SGI0, SGI388904, SGI0, NULL};
const Segment * const SG96075[] = {SGI388905, SGI0, SGI388906, SGI0, NULL};
const Segment * const SG96076[] = {SGI388907, SGI0, SGI388908, SGI0, NULL};
const Segment * const SG96077[] = {SGI388909, SGI0, SGI388910, SGI0, NULL};
const Segment * const SG96078[] = {SGI388911, SGI0, SGI388912, SGI0, NULL};
const Segment * const SG96079[] = {SGI388913, SGI0, SGI388914, SGI0, NULL};
const Segment * const SG96080[] = {SGI388915, SGI0, SGI388916, SGI0, NULL};
const Segment * const SG96081[] = {SGI388917, SGI0, SGI388918, SGI0, NULL};
const Segment * const SG96082[] = {SGI388919, SGI0, SGI388920, SGI0, NULL};
const Segment * const SG96083[] = {SGI388921, SGI0, SGI388922, SGI0, NULL};
const Segment * const SG96084[] = {SGI388923, SGI0, SGI388924, SGI0, NULL};
const Segment * const SG96085[] = {SGI388925, SGI0, SGI388926, SGI0, NULL};
const Segment * const SG96086[] = {SGI388927, SGI0, SGI388928, SGI0, NULL};
const Segment * const SG96087[] = {SGI388929, SGI0, SGI388930, SGI0, NULL};
const Segment * const SG96088[] = {SGI388931, SGI0, SGI388932, SGI0, NULL};
const Segment * const SG96089[] = {SGI388933, SGI0, SGI388934, SGI0, NULL};
const Segment * const SG96090[] = {SGI388935, SGI0, SGI388936, SGI0, NULL};
const Segment * const SG96091[] = {SGI388937, SGI0, SGI388938, SGI0, NULL};
const Segment * const SG96092[] = {SGI388939, SGI0, SGI388940, SGI0, NULL};
const Segment * const SG96093[] = {SGI388941, SGI0, SGI388942, SGI0, NULL};
const Segment * const SG96094[] = {SGI388943, SGI0, SGI388944, SGI0, NULL};
const Segment * const SG96095[] = {SGI388945, SGI0, SGI388946, SGI0, NULL};
const Segment * const SG96096[] = {SGI388947, SGI0, SGI388948, SGI0, NULL};
const Segment * const SG96097[] = {SGI388949, SGI0, SGI388950, SGI0, NULL};
const Segment * const SG96098[] = {SGI388951, SGI0, SGI388952, SGI0, NULL};
const Segment * const SG96099[] = {SGI388953, SGI0, SGI388954, SGI0, NULL};
const Segment * const SG96100[] = {SGI388955, SGI0, SGI388956, SGI0, NULL};
const Segment * const SG96101[] = {SGI388957, SGI0, SGI388958, SGI0, NULL};
const Segment * const SG96102[] = {SGI388959, SGI0, SGI388960, SGI0, NULL};
const Segment * const SG96103[] = {SGI388961, SGI0, SGI388962, SGI0, NULL};
const Segment * const SG96104[] = {SGI388963, SGI0, SGI388964, SGI0, NULL};
const Segment * const SG96105[] = {SGI388965, SGI0, SGI388966, SGI0, NULL};
const Segment * const SG96106[] = {SGI388967, SGI0, SGI388968, SGI0, NULL};
const Segment * const SG96107[] = {SGI388969, SGI0, SGI388970, SGI0, NULL};
const Segment * const SG96108[] = {SGI388971, SGI0, SGI388972, SGI0, NULL};
const Segment * const SG96109[] = {SGI388973, SGI0, SGI388974, SGI0, NULL};
const Segment * const SG96110[] = {SGI388975, SGI0, SGI388976, SGI0, NULL};
const Segment * const SG96111[] = {SGI388977, SGI0, SGI388978, SGI0, NULL};
const Segment * const SG96112[] = {SGI388979, SGI0, SGI388980, SGI0, NULL};
const Segment * const SG96113[] = {SGI388981, SGI0, SGI388982, SGI0, NULL};
const Segment * const SG96114[] = {SGI388983, SGI0, SGI388984, SGI0, NULL};
const Segment * const SG96115[] = {SGI388985, SGI0, SGI388986, SGI0, NULL};
const Segment * const SG96116[] = {SGI388987, SGI0, SGI388988, SGI0, NULL};
const Segment * const SG96117[] = {SGI388989, SGI0, SGI388990, SGI0, NULL};
const Segment * const SG96118[] = {SGI388991, SGI0, SGI388992, SGI0, NULL};
const Segment * const SG96119[] = {SGI388993, SGI0, SGI388994, SGI0, NULL};
const Segment * const SG96120[] = {SGI388995, SGI0, SGI388996, SGI0, NULL};
const Segment * const SG96121[] = {SGI388997, SGI0, SGI388998, SGI0, NULL};
const Segment * const SG96122[] = {SGI388999, SGI0, SGI389000, SGI0, NULL};
const Segment * const SG96123[] = {SGI389001, SGI0, SGI389002, SGI0, NULL};
const Segment * const SG96124[] = {SGI389003, SGI0, SGI389004, SGI0, NULL};
const Segment * const SG96125[] = {SGI389005, SGI0, SGI389006, SGI0, NULL};
const Segment * const SG96126[] = {SGI389007, SGI0, SGI389008, SGI0, NULL};
const Segment * const SG96127[] = {SGI389009, SGI0, SGI389010, SGI0, NULL};
const Segment * const SG96128[] = {SGI389011, SGI0, SGI389012, SGI0, NULL};
const Segment * const SG96129[] = {SGI389013, SGI0, SGI389014, SGI0, NULL};
const Segment * const SG96130[] = {SGI389015, SGI0, SGI389016, SGI0, NULL};
const Segment * const SG96131[] = {SGI389017, SGI0, SGI389018, SGI0, NULL};
const Segment * const SG96132[] = {SGI389019, SGI0, SGI389020, SGI0, NULL};
const Segment * const SG96133[] = {SGI389021, SGI0, SGI389022, SGI0, NULL};
const Segment * const SG96134[] = {SGI389023, SGI0, SGI389024, SGI0, NULL};
const Segment * const SG96135[] = {SGI389025, SGI0, SGI389026, SGI0, NULL};
const Segment * const SG96136[] = {SGI389027, SGI0, SGI389028, SGI0, NULL};
const Segment * const SG96137[] = {SGI389029, SGI0, SGI389030, SGI0, NULL};
const Segment * const SG96138[] = {SGI389031, SGI0, SGI389032, SGI0, NULL};
const Segment * const SG96139[] = {SGI389033, SGI0, SGI389034, SGI0, NULL};
const Segment * const SG96140[] = {SGI389035, SGI0, SGI389036, SGI0, NULL};
const Segment * const SG96141[] = {SGI389037, SGI0, SGI389038, SGI0, NULL};
const Segment * const SG96142[] = {SGI389039, SGI0, SGI389040, SGI0, NULL};
const Segment * const SG96143[] = {SGI389041, SGI0, SGI389042, SGI0, NULL};
const Segment * const SG96144[] = {SGI389043, SGI0, SGI389044, SGI0, NULL};
const Segment * const SG96145[] = {SGI389045, SGI0, SGI389046, SGI0, NULL};
const Segment * const SG96146[] = {SGI389047, SGI0, SGI389048, SGI0, NULL};
const Segment * const SG96147[] = {SGI389049, SGI0, SGI389050, SGI0, NULL};
const Segment * const SG96148[] = {SGI389051, SGI0, SGI389052, SGI0, NULL};
const Segment * const SG96149[] = {SGI389053, SGI0, SGI389054, SGI0, NULL};
const Segment * const SG96150[] = {SGI389055, SGI0, SGI389056, SGI0, NULL};
const Segment * const SG96151[] = {SGI389057, SGI0, SGI389058, SGI0, NULL};
const Segment * const SG96152[] = {SGI389059, SGI0, SGI389060, SGI0, NULL};
const Segment * const SG96153[] = {SGI389061, SGI0, SGI389062, SGI0, NULL};
const Segment * const SG96154[] = {SGI389063, SGI0, SGI389064, SGI0, NULL};
const Segment * const SG96155[] = {SGI389065, SGI0, SGI389066, SGI0, NULL};
const Segment * const SG96156[] = {SGI389067, SGI0, SGI389068, SGI0, NULL};
const Segment * const SG96157[] = {SGI389069, SGI0, SGI389070, SGI0, NULL};
const Segment * const SG96158[] = {SGI389071, SGI0, SGI389072, SGI0, NULL};
const Segment * const SG96159[] = {SGI389073, SGI0, SGI389074, SGI0, NULL};
const Segment * const SG96160[] = {SGI389075, SGI0, SGI389076, SGI0, NULL};
const Segment * const SG96161[] = {SGI389077, SGI0, SGI389078, SGI0, NULL};
const Segment * const SG96162[] = {SGI389079, SGI0, SGI389080, SGI0, NULL};
const Segment * const SG96163[] = {SGI389081, SGI0, SGI389082, SGI0, NULL};
const Segment * const SG96164[] = {SGI389083, SGI0, SGI389084, SGI0, NULL};
const Segment * const SG96165[] = {SGI389085, SGI0, SGI389086, SGI0, NULL};
const Segment * const SG96166[] = {SGI389087, SGI0, SGI389088, SGI0, NULL};
const Segment * const SG96167[] = {SGI389089, SGI0, SGI389090, SGI0, NULL};
const Segment * const SG96168[] = {SGI389091, SGI0, SGI389092, SGI0, NULL};
const Segment * const SG96169[] = {SGI389093, SGI0, SGI389094, SGI0, NULL};
const Segment * const SG96170[] = {SGI389095, SGI0, SGI389096, SGI0, NULL};
const Segment * const SG96171[] = {SGI389097, SGI0, SGI389098, SGI0, NULL};
const Segment * const SG96172[] = {SGI389099, SGI0, SGI389100, SGI0, NULL};
const Segment * const SG96173[] = {SGI389101, SGI0, SGI389102, SGI0, NULL};
const Segment * const SG96174[] = {SGI389103, SGI0, SGI389104, SGI0, NULL};
const Segment * const SG96175[] = {SGI389105, SGI0, SGI389106, SGI0, NULL};
const Segment * const SG96176[] = {SGI389107, SGI0, SGI389108, SGI0, NULL};
const Segment * const SG96177[] = {SGI389109, SGI0, SGI389110, SGI0, NULL};
const Segment * const SG96178[] = {SGI389111, SGI0, SGI389112, SGI0, NULL};
const Segment * const SG96179[] = {SGI389113, SGI0, SGI389114, SGI0, NULL};
const Segment * const SG96180[] = {SGI389115, SGI0, SGI389116, SGI0, NULL};
const Segment * const SG96181[] = {SGI389117, SGI0, SGI389118, SGI0, NULL};
const Segment * const SG96182[] = {SGI389119, SGI0, SGI389120, SGI0, NULL};
const Segment * const SG96183[] = {SGI389121, SGI0, SGI389122, SGI0, NULL};
const Segment * const SG96184[] = {SGI389123, SGI0, SGI389124, SGI0, NULL};
const Segment * const SG96185[] = {SGI389125, SGI0, SGI389126, SGI0, NULL};
const Segment * const SG96186[] = {SGI389127, SGI0, SGI389128, SGI0, NULL};
const Segment * const SG96187[] = {SGI389129, SGI0, SGI389130, SGI0, NULL};
const Segment * const SG96188[] = {SGI389131, SGI0, SGI389132, SGI0, NULL};
const Segment * const SG96189[] = {SGI389133, SGI0, SGI389134, SGI0, NULL};
const Segment * const SG96190[] = {SGI389135, SGI0, SGI389136, SGI0, NULL};
const Segment * const SG96191[] = {SGI389137, SGI0, SGI389138, SGI0, NULL};
const Segment * const SG96192[] = {SGI389139, SGI0, SGI389140, SGI0, NULL};
const Segment * const SG96193[] = {SGI389141, SGI0, SGI389142, SGI0, NULL};
const Segment * const SG96194[] = {SGI389143, SGI0, SGI389144, SGI0, NULL};
const Segment * const SG96195[] = {SGI389145, SGI0, SGI389146, SGI0, NULL};
const Segment * const SG96196[] = {SGI389147, SGI0, SGI389148, SGI0, NULL};
const Segment * const SG96197[] = {SGI389149, SGI0, SGI389150, SGI0, NULL};
const Segment * const SG96198[] = {SGI389151, SGI0, SGI389152, SGI0, NULL};
const Segment * const SG96199[] = {SGI389153, SGI0, SGI389154, SGI0, NULL};
const Segment * const SG96200[] = {SGI389155, SGI0, SGI389156, SGI0, NULL};
const Segment * const SG96201[] = {SGI389157, SGI0, SGI389158, SGI0, NULL};
const Segment * const SG96202[] = {SGI389159, SGI0, SGI389160, SGI0, NULL};
const Segment * const SG96203[] = {SGI389161, SGI0, SGI389162, SGI0, NULL};
const Segment * const SG96204[] = {SGI389163, SGI0, SGI389164, SGI0, NULL};
const Segment * const SG96205[] = {SGI389165, SGI0, SGI389166, SGI0, NULL};
const Segment * const SG96206[] = {SGI389167, SGI0, SGI389168, SGI0, NULL};
const Segment * const SG96207[] = {SGI389169, SGI0, SGI389170, SGI0, NULL};
const Segment * const SG96208[] = {SGI389171, SGI0, SGI389172, SGI0, NULL};
const Segment * const SG96209[] = {SGI389173, SGI0, SGI389174, SGI0, NULL};
const Segment * const SG96210[] = {SGI389175, SGI0, SGI389176, SGI0, NULL};
const Segment * const SG96211[] = {SGI389177, SGI0, SGI389178, SGI0, NULL};
const Segment * const SG96212[] = {SGI389179, SGI0, SGI389180, SGI0, NULL};
const Segment * const SG96213[] = {SGI389181, SGI0, SGI389182, SGI0, NULL};
const Segment * const SG96214[] = {SGI389183, SGI0, SGI389184, SGI0, NULL};
const Segment * const SG96215[] = {SGI389185, SGI0, SGI389186, SGI0, NULL};
const Segment * const SG96216[] = {SGI389187, SGI0, SGI389188, SGI0, NULL};
const Segment * const SG96217[] = {SGI389189, SGI0, SGI389190, SGI0, NULL};
const Segment * const SG96218[] = {SGI389191, SGI0, SGI389192, SGI0, NULL};
const Segment * const SG96219[] = {SGI389193, SGI389194, SGI389195, SGI389196, SGI0, NULL};
const Segment * const SG96220[] = {SGI389197, SGI0, SGI389198, SGI0, NULL};
const Segment * const SG96221[] = {SGI389199, SGI0, SGI389200, SGI0, NULL};
const Segment * const SG96222[] = {SGI389201, SGI0, SGI389202, SGI0, NULL};
const Segment * const SG96223[] = {SGI389203, SGI0, SGI389204, SGI0, NULL};
const Segment * const SG96224[] = {SGI389205, SGI0, SGI389206, SGI0, NULL};
const Segment * const SG96225[] = {SGI389207, SGI0, SGI389208, SGI0, NULL};
const Segment * const SG96226[] = {SGI389209, SGI0, SGI389210, SGI0, NULL};
const Segment * const SG96227[] = {SGI389211, SGI0, SGI389212, SGI0, NULL};
const Segment * const SG96228[] = {SGI389213, SGI0, SGI389214, SGI0, NULL};
const Segment * const SG96229[] = {SGI389215, SGI0, SGI389216, SGI0, NULL};
const Segment * const SG96230[] = {SGI389217, SGI0, SGI389218, SGI0, NULL};
const Segment * const SG96231[] = {SGI389219, SGI0, SGI389220, SGI0, NULL};
const Segment * const SG96232[] = {SGI389221, SGI0, SGI389222, SGI0, NULL};
const Segment * const SG96233[] = {SGI389223, SGI0, SGI389224, SGI0, NULL};
const Segment * const SG96234[] = {SGI389225, SGI0, SGI389226, SGI0, NULL};
const Segment * const SG96235[] = {SGI389227, SGI0, SGI389228, SGI0, NULL};
const Segment * const SG96236[] = {SGI389229, SGI0, SGI389230, SGI0, NULL};
const Segment * const SG96237[] = {SGI389231, SGI0, SGI389232, SGI0, NULL};
const Segment * const SG96238[] = {SGI389233, SGI0, SGI389234, SGI0, NULL};
const Segment * const SG96239[] = {SGI389235, SGI0, SGI389236, SGI0, NULL};
const Segment * const SG96240[] = {SGI389237, SGI0, SGI389238, SGI0, NULL};
const Segment * const SG96241[] = {SGI389239, SGI0, SGI389240, SGI0, NULL};
const Segment * const SG96242[] = {SGI389241, SGI0, SGI389242, SGI0, NULL};
const Segment * const SG96243[] = {SGI389243, SGI0, SGI389244, SGI0, NULL};
const Segment * const SG96244[] = {SGI389245, SGI0, SGI389246, SGI0, NULL};
const Segment * const SG96245[] = {SGI389247, SGI0, SGI389248, SGI0, NULL};
const Segment * const SG96246[] = {SGI389249, SGI0, SGI389250, SGI0, NULL};
const Segment * const SG96247[] = {SGI389251, SGI0, SGI389252, SGI0, NULL};
const Segment * const SG96248[] = {SGI389253, SGI0, SGI389254, SGI0, NULL};
const Segment * const SG96249[] = {SGI389255, SGI0, SGI389256, SGI0, NULL};
const Segment * const SG96250[] = {SGI389257, SGI0, SGI389258, SGI0, NULL};
const Segment * const SG96251[] = {SGI389259, SGI0, SGI389260, SGI0, NULL};
const Segment * const SG96252[] = {SGI389261, SGI0, SGI389262, SGI0, NULL};
const Segment * const SG96253[] = {SGI389263, SGI0, SGI389264, SGI0, NULL};
const Segment * const SG96254[] = {SGI389265, SGI0, SGI389266, SGI0, NULL};
const Segment * const SG96255[] = {SGI389267, SGI0, SGI389268, SGI0, NULL};
const Segment * const SG96256[] = {SGI389269, SGI0, SGI389270, SGI0, NULL};
const Segment * const SG96257[] = {SGI389271, SGI0, SGI389272, SGI0, NULL};
const Segment * const SG96258[] = {SGI389273, SGI0, SGI389274, SGI0, NULL};
const Segment * const SG96259[] = {SGI389275, SGI0, SGI389276, SGI0, NULL};
const Segment * const SG96260[] = {SGI389277, SGI0, SGI389278, SGI0, NULL};
const Segment * const SG96261[] = {SGI389279, SGI0, SGI389280, SGI0, NULL};
const Segment * const SG96262[] = {SGI389281, SGI0, SGI389282, SGI0, NULL};
const Segment * const SG96263[] = {SGI389283, SGI0, SGI389284, SGI0, NULL};
const Segment * const SG96264[] = {SGI389285, SGI0, SGI389286, SGI0, NULL};
const Segment * const SG96265[] = {SGI389287, SGI0, SGI389288, SGI0, NULL};
const Segment * const SG96266[] = {SGI389289, SGI0, SGI389290, SGI0, NULL};
const Segment * const SG96267[] = {SGI389291, SGI0, SGI389292, SGI0, NULL};
const Segment * const SG96268[] = {SGI389293, SGI0, SGI389294, SGI0, NULL};
const Segment * const SG96269[] = {SGI389295, SGI0, SGI389296, SGI0, NULL};
const Segment * const SG96270[] = {SGI389297, SGI0, SGI389298, SGI0, NULL};
const Segment * const SG96271[] = {SGI389299, SGI0, SGI389300, SGI0, NULL};
const Segment * const SG96272[] = {SGI389301, SGI0, SGI389302, SGI0, NULL};
const Segment * const SG96273[] = {SGI389303, SGI0, SGI389304, SGI0, NULL};
const Segment * const SG96274[] = {SGI389305, SGI0, SGI389306, SGI0, NULL};
const Segment * const SG96275[] = {SGI389307, SGI0, SGI389308, SGI0, NULL};
const Segment * const SG96276[] = {SGI389309, SGI0, SGI389310, SGI0, NULL};
const Segment * const SG96277[] = {SGI389311, SGI0, SGI389312, SGI0, NULL};
const Segment * const SG96278[] = {SGI389313, SGI0, SGI389314, SGI0, NULL};
const Segment * const SG96279[] = {SGI389315, SGI0, SGI389316, SGI0, NULL};
const Segment * const SG96280[] = {SGI389317, SGI0, SGI389318, SGI0, NULL};
const Segment * const SG96281[] = {SGI389319, SGI0, SGI389320, SGI0, NULL};
const Segment * const SG96282[] = {SGI389321, SGI0, SGI389322, SGI0, NULL};
const Segment * const SG96283[] = {SGI389323, SGI0, SGI389324, SGI0, NULL};
const Segment * const SG96284[] = {SGI389325, SGI0, SGI389326, SGI0, NULL};
const Segment * const SG96285[] = {SGI389327, SGI0, SGI389328, SGI0, NULL};
const Segment * const SG96286[] = {SGI389329, SGI0, SGI389330, SGI0, NULL};
const Segment * const SG96287[] = {SGI389331, SGI0, SGI389332, SGI0, NULL};
const Segment * const SG96288[] = {SGI389333, SGI0, SGI389334, SGI0, NULL};
const Segment * const SG96289[] = {SGI389335, SGI0, SGI389336, SGI0, NULL};
const Segment * const SG96290[] = {SGI389337, SGI0, SGI389338, SGI0, NULL};
const Segment * const SG96291[] = {SGI389339, SGI0, SGI389340, SGI0, NULL};
const Segment * const SG96292[] = {SGI389341, SGI0, SGI389342, SGI0, NULL};
const Segment * const SG96293[] = {SGI389343, SGI0, SGI389344, SGI0, NULL};
const Segment * const SG96294[] = {SGI389345, SGI0, SGI389346, SGI0, NULL};
const Segment * const SG96295[] = {SGI389347, SGI0, SGI389348, SGI0, NULL};
const Segment * const SG96296[] = {SGI389349, SGI0, SGI389350, SGI0, NULL};
const Segment * const SG96297[] = {SGI389351, SGI0, SGI389352, SGI0, NULL};
const Segment * const SG96298[] = {SGI389353, SGI0, SGI389354, SGI0, NULL};
const Segment * const SG96299[] = {SGI389355, SGI0, SGI389356, SGI0, NULL};
const Segment * const SG96300[] = {SGI389357, SGI0, SGI389358, SGI0, NULL};
const Segment * const SG96301[] = {SGI389359, SGI0, SGI389360, SGI0, NULL};
const Segment * const SG96302[] = {SGI389361, SGI0, SGI389362, SGI0, NULL};
const Segment * const SG96303[] = {SGI389363, SGI0, SGI389364, SGI0, NULL};
const Segment * const SG96304[] = {SGI389365, SGI0, SGI389366, SGI0, NULL};
const Segment * const SG96305[] = {SGI389367, SGI0, SGI389368, SGI0, NULL};
const Segment * const SG96306[] = {SGI389369, SGI0, SGI389370, SGI0, NULL};
const Segment * const SG96307[] = {SGI389371, SGI0, SGI389372, SGI0, NULL};
const Segment * const SG96308[] = {SGI389373, SGI0, SGI389374, SGI0, NULL};
const Segment * const SG96309[] = {SGI389375, SGI0, SGI389376, SGI0, NULL};
const Segment * const SG96310[] = {SGI389377, SGI0, SGI389378, SGI0, NULL};
const Segment * const SG96311[] = {SGI389379, SGI0, SGI389380, SGI0, NULL};
const Segment * const SG96312[] = {SGI389381, SGI0, SGI389382, SGI0, NULL};
const Segment * const SG96313[] = {SGI389383, SGI0, SGI389384, SGI0, NULL};
const Segment * const SG96314[] = {SGI389385, SGI0, SGI389386, SGI0, NULL};
const Segment * const SG96315[] = {SGI389387, SGI0, SGI389388, SGI0, NULL};
const Segment * const SG96316[] = {SGI389389, SGI0, SGI389390, SGI0, NULL};
const Segment * const SG96317[] = {SGI389391, SGI0, SGI389392, SGI0, NULL};
const Segment * const SG96318[] = {SGI389393, SGI0, SGI389394, SGI0, NULL};
const Segment * const SG96319[] = {SGI389395, SGI0, SGI389396, SGI0, NULL};
const Segment * const SG96320[] = {SGI389397, SGI0, SGI389398, SGI0, NULL};
const Segment * const SG96321[] = {SGI389399, SGI0, SGI389400, SGI0, NULL};
const Segment * const SG96322[] = {SGI389401, SGI0, SGI389402, SGI0, NULL};
const Segment * const SG96323[] = {SGI389403, SGI0, SGI389404, SGI0, NULL};
const Segment * const SG96324[] = {SGI389405, SGI0, SGI389406, SGI0, NULL};
const Segment * const SG96325[] = {SGI389407, SGI0, SGI389408, SGI0, NULL};
const Segment * const SG96326[] = {SGI389409, SGI0, SGI389410, SGI0, NULL};
const Segment * const SG96327[] = {SGI389411, SGI0, SGI389412, SGI0, NULL};
const Segment * const SG96328[] = {SGI389413, SGI0, SGI389414, SGI0, NULL};
const Segment * const SG96329[] = {SGI389415, SGI0, SGI389416, SGI0, NULL};
const Segment * const SG96330[] = {SGI389417, SGI0, SGI389418, SGI0, NULL};
const Segment * const SG96331[] = {SGI389419, SGI0, SGI389420, SGI0, NULL};
const Segment * const SG96332[] = {SGI389421, SGI0, SGI389422, SGI0, NULL};
const Segment * const SG96333[] = {SGI389423, SGI0, SGI389424, SGI0, NULL};
const Segment * const SG96334[] = {SGI389425, SGI389426, SGI0, SGI389427, SGI0, NULL};
const Segment * const SG96335[] = {SGI389428, SGI0, SGI389429, SGI0, NULL};
const Segment * const SG96336[] = {SGI389430, SGI0, SGI389431, SGI0, NULL};
const Segment * const SG96337[] = {SGI389432, SGI0, SGI389433, SGI0, NULL};
const Segment * const SG96338[] = {SGI389434, SGI0, SGI389435, SGI0, NULL};
const Segment * const SG96339[] = {SGI389436, SGI0, SGI389437, SGI0, NULL};
const Segment * const SG96340[] = {SGI389438, SGI0, SGI389439, SGI0, NULL};
const Segment * const SG96341[] = {SGI389440, SGI0, SGI389441, SGI0, NULL};
const Segment * const SG96342[] = {SGI389442, SGI0, SGI389443, SGI0, NULL};
const Segment * const SG96343[] = {SGI389444, SGI0, SGI389445, SGI0, NULL};
const Segment * const SG96344[] = {SGI389446, SGI0, SGI389447, SGI0, NULL};
const Segment * const SG96345[] = {SGI389448, SGI0, SGI389449, SGI0, NULL};
const Segment * const SG96346[] = {SGI389450, SGI0, SGI389451, SGI0, NULL};
const Segment * const SG96347[] = {SGI389452, SGI0, SGI389453, SGI0, NULL};
const Segment * const SG96348[] = {SGI389454, SGI0, SGI389455, SGI0, NULL};
const Segment * const SG96349[] = {SGI389456, SGI0, SGI389457, SGI0, NULL};
const Segment * const SG96350[] = {SGI389458, SGI0, SGI389459, SGI0, NULL};
const Segment * const SG96351[] = {SGI389460, SGI0, SGI389461, SGI0, NULL};
const Segment * const SG96352[] = {SGI389462, SGI0, SGI389463, SGI0, NULL};
const Segment * const SG96353[] = {SGI389464, SGI0, SGI389465, SGI0, NULL};
const Segment * const SG96354[] = {SGI389466, SGI0, SGI389467, SGI0, NULL};
const Segment * const SG96355[] = {SGI389468, SGI0, SGI389469, SGI0, NULL};
const Segment * const SG96356[] = {SGI389470, SGI0, SGI389471, SGI0, NULL};
const Segment * const SG96357[] = {SGI389472, SGI0, SGI389473, SGI0, NULL};
const Segment * const SG96358[] = {SGI389474, SGI0, SGI389475, SGI0, NULL};
const Segment * const SG96359[] = {SGI389476, SGI0, SGI389477, SGI0, NULL};
const Segment * const SG96360[] = {SGI389478, SGI0, SGI389479, SGI0, NULL};
const Segment * const SG96361[] = {SGI389480, SGI0, SGI389481, SGI0, NULL};
const Segment * const SG96362[] = {SGI389482, SGI0, SGI389483, SGI0, NULL};
const Segment * const SG96363[] = {SGI389484, SGI0, SGI389485, SGI0, NULL};
const Segment * const SG96364[] = {SGI389486, SGI0, SGI389487, SGI0, NULL};
const Segment * const SG96365[] = {SGI389488, SGI0, SGI389489, SGI0, NULL};
const Segment * const SG96366[] = {SGI389490, SGI0, SGI389491, SGI0, NULL};
const Segment * const SG96367[] = {SGI389492, SGI0, SGI389493, SGI0, NULL};
const Segment * const SG96368[] = {SGI389494, SGI0, SGI389495, SGI0, NULL};
const Segment * const SG96369[] = {SGI389496, SGI0, SGI389497, SGI0, NULL};
const Segment * const SG96370[] = {SGI389498, SGI0, SGI389499, SGI0, NULL};
const Segment * const SG96371[] = {SGI389500, SGI0, SGI389501, SGI0, NULL};
const Segment * const SG96372[] = {SGI389502, SGI0, SGI389503, SGI0, NULL};
const Segment * const SG96373[] = {SGI389504, SGI0, SGI389505, SGI0, NULL};
const Segment * const SG96374[] = {SGI389506, SGI0, SGI389507, SGI0, NULL};
const Segment * const SG96375[] = {SGI389508, SGI0, SGI389509, SGI0, NULL};
const Segment * const SG96376[] = {SGI389510, SGI0, SGI389511, SGI0, NULL};
const Segment * const SG96377[] = {SGI389512, SGI0, SGI389513, SGI0, NULL};
const Segment * const SG96378[] = {SGI389514, SGI0, SGI389515, SGI0, NULL};
const Segment * const SG96379[] = {SGI389516, SGI0, SGI389517, SGI0, NULL};
const Segment * const SG96380[] = {SGI389518, SGI0, SGI389519, SGI0, NULL};
const Segment * const SG96381[] = {SGI389520, SGI0, SGI389521, SGI0, NULL};
const Segment * const SG96382[] = {SGI389522, SGI0, SGI389523, SGI0, NULL};
const Segment * const SG96383[] = {SGI389524, SGI0, SGI389525, SGI0, NULL};
const Segment * const SG96384[] = {SGI389526, SGI0, SGI389527, SGI0, NULL};
const Segment * const SG96385[] = {SGI389528, SGI0, SGI389529, SGI0, NULL};
const Segment * const SG96386[] = {SGI389530, SGI0, SGI389531, SGI0, NULL};
const Segment * const SG96387[] = {SGI389532, SGI0, SGI389533, SGI0, NULL};
const Segment * const SG96388[] = {SGI389534, SGI0, SGI389535, SGI0, NULL};
const Segment * const SG96389[] = {SGI389536, SGI0, SGI389537, SGI0, NULL};
const Segment * const SG96390[] = {SGI389538, SGI0, SGI389539, SGI0, NULL};
const Segment * const SG96391[] = {SGI389540, SGI0, SGI389541, SGI0, NULL};
const Segment * const SG96392[] = {SGI389542, SGI0, SGI389543, SGI0, NULL};
const Segment * const SG96393[] = {SGI389544, SGI0, SGI389545, SGI0, NULL};
const Segment * const SG96394[] = {SGI389546, SGI0, SGI389547, SGI0, NULL};
const Segment * const SG96395[] = {SGI389548, SGI0, SGI389549, SGI0, NULL};
const Segment * const SG96396[] = {SGI389550, SGI0, SGI389551, SGI0, NULL};
const Segment * const SG96397[] = {SGI389552, SGI0, SGI389553, SGI0, NULL};
const Segment * const SG96398[] = {SGI389554, SGI0, SGI389555, SGI0, NULL};
const Segment * const SG96399[] = {SGI389556, SGI0, SGI389557, SGI0, NULL};
const Segment * const SG96400[] = {SGI389558, SGI0, SGI389559, SGI0, NULL};
const Segment * const SG96401[] = {SGI389560, SGI0, SGI389561, SGI0, NULL};
const Segment * const SG96402[] = {SGI389562, SGI0, SGI389563, SGI0, NULL};
const Segment * const SG96403[] = {SGI389564, SGI0, SGI389565, SGI0, NULL};
const Segment * const SG96404[] = {SGI389566, SGI0, SGI389567, SGI0, NULL};
const Segment * const SG96405[] = {SGI389568, SGI0, SGI389569, SGI0, NULL};
const Segment * const SG96406[] = {SGI389570, SGI0, SGI389571, SGI0, NULL};
const Segment * const SG96407[] = {SGI389572, SGI0, SGI389573, SGI0, NULL};
const Segment * const SG96408[] = {SGI389574, SGI0, SGI389575, SGI0, NULL};
const Segment * const SG96409[] = {SGI389576, SGI0, SGI389577, SGI0, NULL};
const Segment * const SG96410[] = {SGI389578, SGI0, SGI389579, SGI0, NULL};
const Segment * const SG96411[] = {SGI389580, SGI0, SGI389581, SGI0, NULL};
const Segment * const SG96412[] = {SGI389582, SGI0, SGI389583, SGI0, NULL};
const Segment * const SG96413[] = {SGI389584, SGI0, SGI389585, SGI0, NULL};
const Segment * const SG96414[] = {SGI389586, SGI0, SGI389587, SGI0, NULL};
const Segment * const SG96415[] = {SGI389588, SGI0, SGI389589, SGI0, NULL};
const Segment * const SG96416[] = {SGI389590, SGI0, SGI389591, SGI0, NULL};
const Segment * const SG96417[] = {SGI389592, SGI0, SGI389593, SGI0, NULL};
const Segment * const SG96418[] = {SGI389594, SGI389595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI389596, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96419[] = {SGI389597, SGI389598, SGI389599, SGI0, SGI0, SGI0, SGI0, SGI389600, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96420[] = {SGI389601, SGI389602, SGI389603, SGI0, SGI0, SGI0, SGI0, SGI389604, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96421[] = {SGI389605, SGI389606, SGI389607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI389608, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96422[] = {SGI389609, SGI389610, SGI389611, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI389612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96423[] = {SGI389613, SGI389614, SGI389615, SGI389616, SGI0, NULL};
const Segment * const SG96424[] = {SGI389617, SGI389618, SGI0, SGI389619, SGI0, NULL};
const Segment * const SG96425[] = {SGI389620, SGI389621, SGI0, SGI389622, SGI0, NULL};
const Segment * const SG96426[] = {SGI389623, SGI389624, SGI389625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI389626, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96427[] = {SGI389627, SGI389628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96428[] = {SGI389629, SGI389630, SGI389631, SGI389631, SGI0, NULL};
const Segment * const SG96429[] = {SGI389632, SGI389633, SGI0, SGI389634, SGI0, NULL};
const Segment * const SG96430[] = {SGI389635, SGI389636, SGI0, SGI389637, SGI0, NULL};
const Segment * const SG96431[] = {SGI389638, SGI389639, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI389640, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96432[] = {SGI389641, SGI389642, SGI389643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI389644, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96433[] = {SGI389645, SGI389646, SGI389646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI389647, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96434[] = {SGI389648, SGI389649, SGI389650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI389651, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96435[] = {SGI389652, SGI389653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI389654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96436[] = {SGI389655, SGI389656, SGI389657, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI389658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96437[] = {SGI389659, SGI389660, SGI389661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96438[] = {SGI389662, SGI389663, SGI389663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96439[] = {SGI389664, SGI389665, SGI0, SGI389666, SGI0, NULL};
const Segment * const SG96440[] = {SGI389667, SGI389668, SGI0, SGI389669, SGI0, NULL};
const Segment * const SG96441[] = {SGI389670, SGI389671, SGI389672, SGI389673, SGI0, NULL};
const Segment * const SG96442[] = {SGI389674, SGI389675, SGI389676, SGI389677, SGI0, NULL};
const Segment * const SG96443[] = {SGI389678, SGI389679, SGI389680, SGI389680, SGI0, NULL};
const Segment * const SG96444[] = {SGI389681, SGI389682, SGI389683, SGI389683, SGI0, NULL};
const Segment * const SG96445[] = {SGI389684, SGI389685, SGI389686, SGI389687, SGI0, NULL};
const Segment * const SG96446[] = {SGI389688, SGI389689, SGI389690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI389691, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96447[] = {SGI389692, SGI389693, SGI389694, SGI389695, SGI0, NULL};
const Segment * const SG96448[] = {SGI389696, SGI389697, SGI389697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI389698, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96449[] = {SGI389699, SGI389700, SGI389701, SGI389702, SGI0, NULL};
const Segment * const SG96450[] = {SGI389703, SGI389704, SGI389704, SGI0, SGI0, SGI0, SGI389705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96451[] = {SGI389706, SGI389707, SGI389708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI389709, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96452[] = {SGI389710, SGI389711, SGI389712, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI389713, SGI0, SGI0, SGI0, SGI389714, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96453[] = {SGI389715, SGI389716, SGI389717, SGI389718, SGI0, NULL};
const Segment * const SG96454[] = {SGI389719, SGI389720, SGI389721, SGI389722, SGI0, NULL};
const Segment * const SG96455[] = {SGI389723, SGI389724, SGI389725, SGI389726, NULL};
const Segment * const SG96456[] = {SGI389727, SGI389728, SGI389729, SGI389730, NULL};
const Segment * const SG96457[] = {SGI389731, SGI389732, SGI389733, SGI389734, NULL};
const Segment * const SG96458[] = {SGI389735, SGI389736, SGI389737, SGI389738, NULL};
const Segment * const SG96459[] = {SGI389739, SGI389740, SGI389741, SGI389742, NULL};
const Segment * const SG96460[] = {SGI389743, SGI389744, SGI389745, SGI389746, NULL};
const Segment * const SG96461[] = {SGI389747, SGI389748, SGI389749, SGI389750, SGI0, NULL};
const Segment * const SG96462[] = {SGI389751, SGI389752, SGI389753, SGI389754, SGI0, NULL};
const Segment * const SG96463[] = {SGI389755, SGI389756, SGI0, SGI389757, SGI0, NULL};
const Segment * const SG96464[] = {SGI389758, SGI389759, SGI389759, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI389760, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96465[] = {SGI389761, SGI389762, SGI389762, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI389763, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96466[] = {SGI389764, SGI389765, SGI389766, SGI0, SGI0, SGI0, SGI389767, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96467[] = {SGI389768, SGI389769, SGI0, SGI389770, SGI0, NULL};
const Segment * const SG96468[] = {SGI389771, SGI389772, SGI0, SGI389773, SGI0, NULL};
const Segment * const SG96469[] = {SGI389774, SGI389775, SGI0, SGI389776, SGI0, NULL};
const Segment * const SG96470[] = {SGI389777, SGI389778, SGI0, SGI389779, SGI0, NULL};
const Segment * const SG96471[] = {SGI389780, SGI389781, SGI389781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI389782, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96472[] = {SGI389783, SGI389784, SGI389785, SGI389786, SGI0, NULL};
const Segment * const SG96473[] = {SGI389787, SGI389788, SGI389789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI389790, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96474[] = {SGI389791, SGI389792, SGI389793, SGI389794, SGI0, NULL};
const Segment * const SG96475[] = {SGI389795, SGI389796, SGI0, SGI389797, SGI0, NULL};
const Segment * const SG96476[] = {SGI389798, SGI389799, SGI389799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI389800, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96477[] = {SGI389801, SGI389802, SGI389803, SGI389803, SGI0, NULL};
const Segment * const SG96478[] = {SGI389804, SGI389805, SGI389806, SGI389807, NULL};
const Segment * const SG96479[] = {SGI389808, SGI389809, SGI389810, SGI389811, NULL};
const Segment * const SG96480[] = {SGI389812, SGI389813, SGI389814, SGI389815, NULL};
const Segment * const SG96481[] = {SGI389816, SGI389817, SGI389818, SGI389819, NULL};
const Segment * const SG96482[] = {SGI389820, SGI389821, SGI0, SGI389822, SGI0, NULL};
const Segment * const SG96483[] = {SGI389823, SGI389824, SGI0, SGI389825, SGI0, NULL};
const Segment * const SG96484[] = {SGI389826, SGI389827, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96485[] = {SGI389828, SGI389829, SGI0, SGI389830, SGI0, NULL};
const Segment * const SG96486[] = {SGI389831, SGI389832, SGI389833, SGI389834, SGI0, NULL};
const Segment * const SG96487[] = {SGI389835, SGI389836, SGI0, SGI389837, SGI0, NULL};
const Segment * const SG96488[] = {SGI389838, SGI389839, SGI0, SGI389840, SGI0, NULL};
const Segment * const SG96489[] = {SGI389841, SGI389842, SGI389842, SGI0, SGI389843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96490[] = {SGI389844, SGI389845, SGI389846, SGI0, SGI389847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96491[] = {SGI389848, SGI389849, SGI389849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI389850, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96492[] = {SGI389851, SGI0, SGI389852, SGI0, NULL};
const Segment * const SG96493[] = {SGI389853, SGI0, SGI389854, SGI0, NULL};
const Segment * const SG96494[] = {SGI389855, SGI0, SGI389856, SGI0, NULL};
const Segment * const SG96495[] = {SGI389857, SGI0, SGI389858, SGI0, NULL};
const Segment * const SG96496[] = {SGI389859, SGI0, SGI389860, SGI0, NULL};
const Segment * const SG96497[] = {SGI389861, SGI0, SGI389862, SGI0, NULL};
const Segment * const SG96498[] = {SGI389863, SGI0, SGI389864, SGI0, NULL};
const Segment * const SG96499[] = {SGI389865, SGI0, SGI389866, SGI0, NULL};
const Segment * const SG96500[] = {SGI389867, SGI0, SGI389868, SGI0, NULL};
const Segment * const SG96501[] = {SGI389869, SGI0, SGI389870, SGI0, NULL};
const Segment * const SG96502[] = {SGI389871, SGI0, SGI389872, SGI0, NULL};
const Segment * const SG96503[] = {SGI389873, SGI0, SGI389874, SGI0, NULL};
const Segment * const SG96504[] = {SGI389875, SGI0, SGI389876, SGI0, NULL};
const Segment * const SG96505[] = {SGI389877, SGI0, SGI389878, SGI0, NULL};
const Segment * const SG96506[] = {SGI389879, SGI0, SGI389880, SGI0, NULL};
const Segment * const SG96507[] = {SGI389881, SGI0, SGI389882, SGI0, NULL};
const Segment * const SG96508[] = {SGI389883, SGI0, SGI389884, SGI0, NULL};
const Segment * const SG96509[] = {SGI389885, SGI0, SGI389886, SGI0, NULL};
const Segment * const SG96510[] = {SGI389887, SGI0, SGI389888, SGI0, NULL};
const Segment * const SG96511[] = {SGI389889, SGI0, SGI389890, SGI0, NULL};
const Segment * const SG96512[] = {SGI389891, SGI0, SGI389892, SGI0, NULL};
const Segment * const SG96513[] = {SGI389893, SGI0, SGI389894, SGI0, NULL};
const Segment * const SG96514[] = {SGI389895, SGI0, SGI389896, SGI0, NULL};
const Segment * const SG96515[] = {SGI389897, SGI0, SGI389898, SGI0, NULL};
const Segment * const SG96516[] = {SGI389899, SGI0, SGI389900, SGI0, NULL};
const Segment * const SG96517[] = {SGI389901, SGI0, SGI389902, SGI0, NULL};
const Segment * const SG96518[] = {SGI389903, SGI389904, SGI389905, SGI389905, SGI0, NULL};
const Segment * const SG96519[] = {SGI389906, SGI0, SGI389907, SGI0, NULL};
const Segment * const SG96520[] = {SGI389908, SGI0, SGI389909, SGI0, NULL};
const Segment * const SG96521[] = {SGI389910, SGI0, SGI389911, SGI0, NULL};
const Segment * const SG96522[] = {SGI389912, SGI0, SGI389913, SGI0, NULL};
const Segment * const SG96523[] = {SGI389914, SGI0, SGI389915, SGI0, NULL};
const Segment * const SG96524[] = {SGI389916, SGI0, SGI389917, SGI0, NULL};
const Segment * const SG96525[] = {SGI389918, SGI0, SGI389919, SGI0, NULL};
const Segment * const SG96526[] = {SGI389920, SGI0, SGI389921, SGI0, NULL};
const Segment * const SG96527[] = {SGI389922, SGI0, SGI389923, SGI0, NULL};
const Segment * const SG96528[] = {SGI389924, SGI0, SGI389925, SGI0, NULL};
const Segment * const SG96529[] = {SGI389926, SGI0, SGI389927, SGI0, NULL};
const Segment * const SG96530[] = {SGI389928, SGI0, SGI389929, SGI0, NULL};
const Segment * const SG96531[] = {SGI389930, SGI0, SGI389931, SGI0, NULL};
const Segment * const SG96532[] = {SGI389932, SGI0, SGI389933, SGI0, NULL};
const Segment * const SG96533[] = {SGI389934, SGI0, SGI389935, SGI0, NULL};
const Segment * const SG96534[] = {SGI389936, SGI0, SGI389937, SGI0, NULL};
const Segment * const SG96535[] = {SGI389938, SGI0, SGI389939, SGI0, NULL};
const Segment * const SG96536[] = {SGI389940, SGI0, SGI389941, SGI0, NULL};
const Segment * const SG96537[] = {SGI389942, SGI0, SGI389943, SGI0, NULL};
const Segment * const SG96538[] = {SGI389944, SGI0, SGI389945, SGI0, NULL};
const Segment * const SG96539[] = {SGI389946, SGI0, SGI389947, SGI0, NULL};
const Segment * const SG96540[] = {SGI389948, SGI0, SGI389949, SGI0, NULL};
const Segment * const SG96541[] = {SGI389950, SGI0, SGI389951, SGI0, NULL};
const Segment * const SG96542[] = {SGI389952, SGI0, SGI389953, SGI0, NULL};
const Segment * const SG96543[] = {SGI389954, SGI0, SGI389955, SGI0, NULL};
const Segment * const SG96544[] = {SGI389956, SGI0, SGI389957, SGI0, NULL};
const Segment * const SG96545[] = {SGI389958, SGI0, SGI389959, SGI0, NULL};
const Segment * const SG96546[] = {SGI389960, SGI0, SGI389961, SGI0, NULL};
const Segment * const SG96547[] = {SGI389962, SGI0, SGI389963, SGI0, NULL};
const Segment * const SG96548[] = {SGI389964, SGI0, SGI389965, SGI0, NULL};
const Segment * const SG96549[] = {SGI389966, SGI0, SGI389967, SGI0, NULL};
const Segment * const SG96550[] = {SGI389968, SGI0, SGI389969, SGI0, NULL};
const Segment * const SG96551[] = {SGI389970, SGI0, SGI389971, SGI0, NULL};
const Segment * const SG96552[] = {SGI389972, SGI0, SGI389973, SGI0, NULL};
const Segment * const SG96553[] = {SGI389974, SGI0, SGI389975, SGI0, NULL};
const Segment * const SG96554[] = {SGI389976, SGI0, SGI389977, SGI0, NULL};
const Segment * const SG96555[] = {SGI389978, SGI0, SGI389979, SGI0, NULL};
const Segment * const SG96556[] = {SGI389980, SGI0, SGI389981, SGI0, NULL};
const Segment * const SG96557[] = {SGI389982, SGI0, SGI389983, SGI0, NULL};
const Segment * const SG96558[] = {SGI389984, SGI0, SGI389985, SGI0, NULL};
const Segment * const SG96559[] = {SGI389986, SGI0, SGI389987, SGI0, NULL};
const Segment * const SG96560[] = {SGI389988, SGI0, SGI389989, SGI0, NULL};
const Segment * const SG96561[] = {SGI389990, SGI0, SGI389991, SGI0, NULL};
const Segment * const SG96562[] = {SGI389992, SGI0, SGI389993, SGI0, NULL};
const Segment * const SG96563[] = {SGI389994, SGI0, SGI389995, SGI0, NULL};
const Segment * const SG96564[] = {SGI389996, SGI0, SGI389997, SGI0, NULL};
const Segment * const SG96565[] = {SGI389998, SGI0, SGI389999, SGI0, NULL};
const Segment * const SG96566[] = {SGI390000, SGI0, SGI390001, SGI0, NULL};
const Segment * const SG96567[] = {SGI390002, SGI0, SGI390003, SGI0, NULL};
const Segment * const SG96568[] = {SGI390004, SGI0, SGI390005, SGI0, NULL};
const Segment * const SG96569[] = {SGI390006, SGI0, SGI390007, SGI0, NULL};
const Segment * const SG96570[] = {SGI390008, SGI0, SGI390009, SGI0, NULL};
const Segment * const SG96571[] = {SGI390010, SGI0, SGI390011, SGI0, NULL};
const Segment * const SG96572[] = {SGI390012, SGI0, SGI390013, SGI0, NULL};
const Segment * const SG96573[] = {SGI390014, SGI0, SGI390015, SGI0, NULL};
const Segment * const SG96574[] = {SGI390016, SGI0, SGI390017, SGI0, NULL};
const Segment * const SG96575[] = {SGI390018, SGI0, SGI390019, SGI0, NULL};
const Segment * const SG96576[] = {SGI390020, SGI0, SGI390021, SGI0, NULL};
const Segment * const SG96577[] = {SGI390022, SGI0, SGI390023, SGI0, NULL};
const Segment * const SG96578[] = {SGI390024, SGI0, SGI390025, SGI0, NULL};
const Segment * const SG96579[] = {SGI390026, SGI0, SGI390027, SGI0, NULL};
const Segment * const SG96580[] = {SGI390028, SGI0, SGI390029, SGI0, NULL};
const Segment * const SG96581[] = {SGI390030, SGI0, SGI390031, SGI0, NULL};
const Segment * const SG96582[] = {SGI390032, SGI0, SGI390033, SGI0, NULL};
const Segment * const SG96583[] = {SGI390034, SGI0, SGI390035, SGI0, NULL};
const Segment * const SG96584[] = {SGI390036, SGI0, SGI390037, SGI0, NULL};
const Segment * const SG96585[] = {SGI390038, SGI0, SGI390039, SGI0, NULL};
const Segment * const SG96586[] = {SGI390040, SGI0, SGI390041, SGI0, NULL};
const Segment * const SG96587[] = {SGI390042, SGI0, SGI390043, SGI0, NULL};
const Segment * const SG96588[] = {SGI390044, SGI0, SGI390045, SGI0, NULL};
const Segment * const SG96589[] = {SGI390046, SGI0, SGI390047, SGI0, NULL};
const Segment * const SG96590[] = {SGI390048, SGI0, SGI390049, SGI0, NULL};
const Segment * const SG96591[] = {SGI390050, SGI0, SGI390051, SGI0, NULL};
const Segment * const SG96592[] = {SGI390052, SGI0, SGI390053, SGI0, NULL};
const Segment * const SG96593[] = {SGI390054, SGI0, SGI390055, SGI0, NULL};
const Segment * const SG96594[] = {SGI390056, SGI0, SGI390057, SGI0, NULL};
const Segment * const SG96595[] = {SGI390058, SGI0, SGI390059, SGI0, NULL};
const Segment * const SG96596[] = {SGI390060, SGI0, SGI390061, SGI0, NULL};
const Segment * const SG96597[] = {SGI390062, SGI0, SGI390063, SGI0, NULL};
const Segment * const SG96598[] = {SGI390064, SGI0, SGI390065, SGI0, NULL};
const Segment * const SG96599[] = {SGI390066, SGI0, SGI390067, SGI0, NULL};
const Segment * const SG96600[] = {SGI390068, SGI0, SGI390069, SGI0, NULL};
const Segment * const SG96601[] = {SGI390070, SGI0, SGI390071, SGI0, NULL};
const Segment * const SG96602[] = {SGI390072, SGI0, SGI390073, SGI0, NULL};
const Segment * const SG96603[] = {SGI390074, SGI0, SGI390075, SGI0, NULL};
const Segment * const SG96604[] = {SGI390076, SGI0, SGI390077, SGI0, NULL};
const Segment * const SG96605[] = {SGI390078, SGI0, SGI390079, SGI0, NULL};
const Segment * const SG96606[] = {SGI390080, SGI0, SGI390081, SGI0, NULL};
const Segment * const SG96607[] = {SGI390082, SGI0, SGI390083, SGI0, NULL};
const Segment * const SG96608[] = {SGI390084, SGI0, SGI390085, SGI0, NULL};
const Segment * const SG96609[] = {SGI390086, SGI0, SGI390087, SGI0, NULL};
const Segment * const SG96610[] = {SGI390088, SGI0, SGI390089, SGI0, NULL};
const Segment * const SG96611[] = {SGI390090, SGI0, SGI390091, SGI0, NULL};
const Segment * const SG96612[] = {SGI390092, SGI0, SGI390093, SGI0, NULL};
const Segment * const SG96613[] = {SGI390094, SGI0, SGI390095, SGI0, NULL};
const Segment * const SG96614[] = {SGI390096, SGI0, SGI390097, SGI0, NULL};
const Segment * const SG96615[] = {SGI390098, SGI0, SGI390099, SGI0, NULL};
const Segment * const SG96616[] = {SGI390100, SGI0, SGI390101, SGI0, NULL};
const Segment * const SG96617[] = {SGI390102, SGI0, SGI390103, SGI0, NULL};
const Segment * const SG96618[] = {SGI390104, SGI0, SGI390105, SGI0, NULL};
const Segment * const SG96619[] = {SGI390106, SGI0, SGI390107, SGI0, NULL};
const Segment * const SG96620[] = {SGI390108, SGI0, SGI390109, SGI0, NULL};
const Segment * const SG96621[] = {SGI390110, SGI0, SGI390111, SGI0, NULL};
const Segment * const SG96622[] = {SGI390112, SGI0, SGI390113, SGI0, NULL};
const Segment * const SG96623[] = {SGI390114, SGI0, SGI390115, SGI0, NULL};
const Segment * const SG96624[] = {SGI390116, SGI0, SGI390117, SGI0, NULL};
const Segment * const SG96625[] = {SGI390118, SGI0, SGI390119, SGI0, NULL};
const Segment * const SG96626[] = {SGI390120, SGI0, SGI390121, SGI0, NULL};
const Segment * const SG96627[] = {SGI390122, SGI0, SGI390123, SGI0, NULL};
const Segment * const SG96628[] = {SGI390124, SGI0, SGI390125, SGI0, NULL};
const Segment * const SG96629[] = {SGI390126, SGI0, SGI390127, SGI0, NULL};
const Segment * const SG96630[] = {SGI390128, SGI0, SGI390129, SGI0, NULL};
const Segment * const SG96631[] = {SGI390130, SGI0, SGI390131, SGI0, NULL};
const Segment * const SG96632[] = {SGI390132, SGI0, SGI390133, SGI0, NULL};
const Segment * const SG96633[] = {SGI390134, SGI0, SGI390135, SGI0, NULL};
const Segment * const SG96634[] = {SGI390136, SGI0, SGI390137, SGI0, NULL};
const Segment * const SG96635[] = {SGI390138, SGI0, SGI390139, SGI0, NULL};
const Segment * const SG96636[] = {SGI390140, SGI0, SGI390141, SGI0, NULL};
const Segment * const SG96637[] = {SGI390142, SGI0, SGI390143, SGI0, NULL};
const Segment * const SG96638[] = {SGI390144, SGI0, SGI390145, SGI0, NULL};
const Segment * const SG96639[] = {SGI390146, SGI0, SGI390147, SGI0, NULL};
const Segment * const SG96640[] = {SGI390148, SGI0, SGI390149, SGI0, NULL};
const Segment * const SG96641[] = {SGI390150, SGI0, SGI390151, SGI0, NULL};
const Segment * const SG96642[] = {SGI390152, SGI0, SGI390153, SGI0, NULL};
const Segment * const SG96643[] = {SGI390154, SGI0, SGI390155, SGI0, NULL};
const Segment * const SG96644[] = {SGI390156, SGI0, SGI390157, SGI0, NULL};
const Segment * const SG96645[] = {SGI390158, SGI0, SGI390159, SGI0, NULL};
const Segment * const SG96646[] = {SGI390160, SGI0, SGI390161, SGI0, NULL};
const Segment * const SG96647[] = {SGI390162, SGI0, SGI390163, SGI0, NULL};
const Segment * const SG96648[] = {SGI390164, SGI0, SGI390165, SGI0, NULL};
const Segment * const SG96649[] = {SGI390166, SGI0, SGI390167, SGI0, NULL};
const Segment * const SG96650[] = {SGI390168, SGI0, SGI390169, SGI0, NULL};
const Segment * const SG96651[] = {SGI390170, SGI0, SGI390171, SGI0, NULL};
const Segment * const SG96652[] = {SGI390172, SGI0, SGI390173, SGI0, NULL};
const Segment * const SG96653[] = {SGI390174, SGI0, SGI390175, SGI0, NULL};
const Segment * const SG96654[] = {SGI390176, SGI0, SGI390177, SGI0, NULL};
const Segment * const SG96655[] = {SGI390178, SGI0, SGI390179, SGI0, NULL};
const Segment * const SG96656[] = {SGI390180, SGI0, SGI390181, SGI0, NULL};
const Segment * const SG96657[] = {SGI390182, SGI0, SGI390183, SGI0, NULL};
const Segment * const SG96658[] = {SGI390184, SGI0, SGI390185, SGI0, NULL};
const Segment * const SG96659[] = {SGI390186, SGI0, SGI390187, SGI0, NULL};
const Segment * const SG96660[] = {SGI390188, SGI0, SGI390189, SGI0, NULL};
const Segment * const SG96661[] = {SGI390190, SGI0, SGI390191, SGI0, NULL};
const Segment * const SG96662[] = {SGI390192, SGI0, SGI390193, SGI0, NULL};
const Segment * const SG96663[] = {SGI390194, SGI0, SGI390195, SGI0, NULL};
const Segment * const SG96664[] = {SGI390196, SGI0, SGI390197, SGI0, NULL};
const Segment * const SG96665[] = {SGI390198, SGI0, SGI390199, SGI0, NULL};
const Segment * const SG96666[] = {SGI390200, SGI0, SGI390201, SGI0, NULL};
const Segment * const SG96667[] = {SGI390202, SGI0, SGI390203, SGI0, NULL};
const Segment * const SG96668[] = {SGI390204, SGI0, SGI390205, SGI0, NULL};
const Segment * const SG96669[] = {SGI390206, SGI0, SGI390207, SGI0, NULL};
const Segment * const SG96670[] = {SGI390208, SGI0, SGI390209, SGI0, NULL};
const Segment * const SG96671[] = {SGI390210, SGI0, SGI390211, SGI0, NULL};
const Segment * const SG96672[] = {SGI390212, SGI0, SGI390213, SGI0, NULL};
const Segment * const SG96673[] = {SGI390214, SGI0, SGI390215, SGI0, NULL};
const Segment * const SG96674[] = {SGI390216, SGI0, SGI390217, SGI0, NULL};
const Segment * const SG96675[] = {SGI390218, SGI0, SGI390219, SGI0, NULL};
const Segment * const SG96676[] = {SGI390220, SGI0, SGI390221, SGI0, NULL};
const Segment * const SG96677[] = {SGI390222, SGI0, SGI390223, SGI0, NULL};
const Segment * const SG96678[] = {SGI390224, SGI0, SGI390225, SGI0, NULL};
const Segment * const SG96679[] = {SGI390226, SGI0, SGI390227, SGI0, NULL};
const Segment * const SG96680[] = {SGI390228, SGI0, SGI390229, SGI0, NULL};
const Segment * const SG96681[] = {SGI390230, SGI0, SGI390231, SGI0, NULL};
const Segment * const SG96682[] = {SGI390232, SGI0, SGI390233, SGI0, NULL};
const Segment * const SG96683[] = {SGI390234, SGI0, SGI390235, SGI0, NULL};
const Segment * const SG96684[] = {SGI390236, SGI0, SGI390237, SGI0, NULL};
const Segment * const SG96685[] = {SGI390238, SGI0, SGI390239, SGI0, NULL};
const Segment * const SG96686[] = {SGI390240, SGI0, SGI390241, SGI0, NULL};
const Segment * const SG96687[] = {SGI390242, SGI0, SGI390243, SGI0, NULL};
const Segment * const SG96688[] = {SGI390244, SGI0, SGI390245, SGI0, NULL};
const Segment * const SG96689[] = {SGI390246, SGI0, SGI390247, SGI0, NULL};
const Segment * const SG96690[] = {SGI390248, SGI0, SGI390249, SGI0, NULL};
const Segment * const SG96691[] = {SGI390250, SGI0, SGI390251, SGI0, NULL};
const Segment * const SG96692[] = {SGI390252, SGI0, SGI390253, SGI0, NULL};
const Segment * const SG96693[] = {SGI390254, SGI0, SGI390255, SGI0, NULL};
const Segment * const SG96694[] = {SGI390256, SGI0, SGI390257, SGI0, NULL};
const Segment * const SG96695[] = {SGI390258, SGI0, SGI390259, SGI0, NULL};
const Segment * const SG96696[] = {SGI390260, SGI0, SGI390261, SGI0, NULL};
const Segment * const SG96697[] = {SGI390262, SGI0, SGI390263, SGI0, NULL};
const Segment * const SG96698[] = {SGI390264, SGI0, SGI390265, SGI0, NULL};
const Segment * const SG96699[] = {SGI390266, SGI0, SGI390267, SGI0, NULL};
const Segment * const SG96700[] = {SGI390268, SGI0, SGI390269, SGI0, NULL};
const Segment * const SG96701[] = {SGI390270, SGI0, SGI390271, SGI0, NULL};
const Segment * const SG96702[] = {SGI390272, SGI0, SGI390273, SGI0, NULL};
const Segment * const SG96703[] = {SGI390274, SGI0, SGI390275, SGI0, NULL};
const Segment * const SG96704[] = {SGI390276, SGI0, SGI390277, SGI0, NULL};
const Segment * const SG96705[] = {SGI390278, SGI0, SGI390279, SGI0, NULL};
const Segment * const SG96706[] = {SGI390280, SGI0, SGI390281, SGI0, NULL};
const Segment * const SG96707[] = {SGI390282, SGI0, SGI390283, SGI0, NULL};
const Segment * const SG96708[] = {SGI390284, SGI0, SGI390285, SGI0, NULL};
const Segment * const SG96709[] = {SGI390286, SGI0, SGI390287, SGI0, NULL};
const Segment * const SG96710[] = {SGI390288, SGI0, SGI390289, SGI0, NULL};
const Segment * const SG96711[] = {SGI390290, SGI0, SGI390291, SGI0, NULL};
const Segment * const SG96712[] = {SGI390292, SGI0, SGI390293, SGI0, NULL};
const Segment * const SG96713[] = {SGI390294, SGI0, SGI390295, SGI0, NULL};
const Segment * const SG96714[] = {SGI390296, SGI0, SGI390297, SGI0, NULL};
const Segment * const SG96715[] = {SGI390298, SGI0, SGI390299, SGI0, NULL};
const Segment * const SG96716[] = {SGI390300, SGI0, SGI390301, SGI0, NULL};
const Segment * const SG96717[] = {SGI390302, SGI0, SGI390303, SGI0, NULL};
const Segment * const SG96718[] = {SGI390304, SGI0, SGI390305, SGI0, NULL};
const Segment * const SG96719[] = {SGI390306, SGI0, SGI390307, SGI0, NULL};
const Segment * const SG96720[] = {SGI390308, SGI0, SGI390309, SGI0, NULL};
const Segment * const SG96721[] = {SGI390310, SGI0, SGI390311, SGI0, NULL};
const Segment * const SG96722[] = {SGI390312, SGI0, SGI390313, SGI0, NULL};
const Segment * const SG96723[] = {SGI390314, SGI0, SGI390315, SGI0, NULL};
const Segment * const SG96724[] = {SGI390316, SGI0, SGI390317, SGI0, NULL};
const Segment * const SG96725[] = {SGI390318, SGI0, SGI390319, SGI0, NULL};
const Segment * const SG96726[] = {SGI390320, SGI0, SGI390321, SGI0, NULL};
const Segment * const SG96727[] = {SGI390322, SGI0, SGI390323, SGI0, NULL};
const Segment * const SG96728[] = {SGI390324, SGI0, SGI390325, SGI0, NULL};
const Segment * const SG96729[] = {SGI390326, SGI0, SGI390327, SGI0, NULL};
const Segment * const SG96730[] = {SGI390328, SGI0, SGI390329, SGI0, NULL};
const Segment * const SG96731[] = {SGI390330, SGI0, SGI390331, SGI0, NULL};
const Segment * const SG96732[] = {SGI390332, SGI0, SGI390333, SGI0, NULL};
const Segment * const SG96733[] = {SGI390334, SGI0, SGI390335, SGI0, NULL};
const Segment * const SG96734[] = {SGI390336, SGI0, SGI390337, SGI0, NULL};
const Segment * const SG96735[] = {SGI390338, SGI0, SGI390339, SGI0, NULL};
const Segment * const SG96736[] = {SGI390340, SGI0, SGI390341, SGI0, NULL};
const Segment * const SG96737[] = {SGI390342, SGI0, SGI390343, SGI0, NULL};
const Segment * const SG96738[] = {SGI390344, SGI0, SGI390345, SGI0, NULL};
const Segment * const SG96739[] = {SGI390346, SGI0, SGI390347, SGI0, NULL};
const Segment * const SG96740[] = {SGI390348, SGI0, SGI390349, SGI0, NULL};
const Segment * const SG96741[] = {SGI390350, SGI0, SGI390351, SGI0, NULL};
const Segment * const SG96742[] = {SGI390352, SGI0, SGI390353, SGI0, NULL};
const Segment * const SG96743[] = {SGI390354, SGI0, SGI390355, SGI0, NULL};
const Segment * const SG96744[] = {SGI390356, SGI0, SGI390357, SGI0, NULL};
const Segment * const SG96745[] = {SGI390358, SGI0, SGI390359, SGI0, NULL};
const Segment * const SG96746[] = {SGI390360, SGI0, SGI390361, SGI0, NULL};
const Segment * const SG96747[] = {SGI390362, SGI0, SGI390363, SGI0, NULL};
const Segment * const SG96748[] = {SGI390364, SGI0, SGI390365, SGI0, NULL};
const Segment * const SG96749[] = {SGI390366, SGI0, SGI390367, SGI0, NULL};
const Segment * const SG96750[] = {SGI390368, SGI0, SGI390369, SGI0, NULL};
const Segment * const SG96751[] = {SGI390370, SGI0, SGI390371, SGI0, NULL};
const Segment * const SG96752[] = {SGI390372, SGI0, SGI390373, SGI0, NULL};
const Segment * const SG96753[] = {SGI390374, SGI0, SGI390375, SGI0, NULL};
const Segment * const SG96754[] = {SGI390376, SGI0, SGI390377, SGI0, NULL};
const Segment * const SG96755[] = {SGI390378, SGI0, SGI390379, SGI0, NULL};
const Segment * const SG96756[] = {SGI390380, SGI0, SGI390381, SGI0, NULL};
const Segment * const SG96757[] = {SGI390382, SGI0, SGI390383, SGI0, NULL};
const Segment * const SG96758[] = {SGI390384, SGI0, SGI390385, SGI0, NULL};
const Segment * const SG96759[] = {SGI390386, SGI0, SGI390387, SGI0, NULL};
const Segment * const SG96760[] = {SGI390388, SGI0, SGI390389, SGI0, NULL};
const Segment * const SG96761[] = {SGI390390, SGI0, SGI390391, SGI0, NULL};
const Segment * const SG96762[] = {SGI390392, SGI0, SGI390393, SGI0, NULL};
const Segment * const SG96763[] = {SGI390394, SGI0, SGI390395, SGI0, NULL};
const Segment * const SG96764[] = {SGI390396, SGI0, SGI390397, SGI0, NULL};
const Segment * const SG96765[] = {SGI390398, SGI0, SGI390399, SGI0, NULL};
const Segment * const SG96766[] = {SGI390400, SGI0, SGI390401, SGI0, NULL};
const Segment * const SG96767[] = {SGI390402, SGI0, SGI390403, SGI0, NULL};
const Segment * const SG96768[] = {SGI390404, SGI0, SGI390405, SGI0, NULL};
const Segment * const SG96769[] = {SGI390406, SGI0, SGI390407, SGI0, NULL};
const Segment * const SG96770[] = {SGI390408, SGI0, SGI390409, SGI0, NULL};
const Segment * const SG96771[] = {SGI390410, SGI390411, SGI390412, SGI0, SGI0, SGI0, SGI0, SGI390413, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96772[] = {SGI390414, SGI390415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96773[] = {SGI390417, SGI390418, SGI390419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390420, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96774[] = {SGI390421, SGI390422, SGI0, SGI390423, SGI0, NULL};
const Segment * const SG96775[] = {SGI390424, SGI390425, SGI390426, SGI390427, SGI0, NULL};
const Segment * const SG96776[] = {SGI390428, SGI390429, SGI0, SGI390430, SGI0, NULL};
const Segment * const SG96777[] = {SGI390431, SGI390432, SGI390433, SGI390434, SGI0, NULL};
const Segment * const SG96778[] = {SGI390435, SGI390436, SGI390437, SGI390437, SGI0, NULL};
const Segment * const SG96779[] = {SGI390438, SGI390439, SGI390440, SGI390441, SGI0, NULL};
const Segment * const SG96780[] = {SGI390442, SGI390443, SGI390444, SGI390444, SGI0, NULL};
const Segment * const SG96781[] = {SGI390445, SGI390446, SGI0, SGI390447, SGI0, NULL};
const Segment * const SG96782[] = {SGI390448, SGI390449, SGI390450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390451, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96783[] = {SGI390452, SGI390453, SGI0, SGI390454, SGI0, SGI0, SGI0, SGI0, SGI390455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96784[] = {SGI390456, SGI390457, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96785[] = {SGI390459, SGI390460, SGI390461, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96786[] = {SGI390463, SGI390464, SGI390464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390465, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96787[] = {SGI390466, SGI390467, SGI390468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390469, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96788[] = {SGI390470, SGI390471, SGI0, SGI390472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390473, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96789[] = {SGI390474, SGI390475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390476, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96790[] = {SGI390477, SGI390478, SGI390479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390480, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96791[] = {SGI390481, SGI390482, SGI390482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390483, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96792[] = {SGI390484, SGI390485, SGI390486, SGI390487, SGI0, NULL};
const Segment * const SG96793[] = {SGI390488, SGI390489, SGI0, SGI390490, SGI0, NULL};
const Segment * const SG96794[] = {SGI390491, SGI390492, SGI390493, SGI390493, SGI0, NULL};
const Segment * const SG96795[] = {SGI390494, SGI390495, SGI390496, SGI390496, SGI0, NULL};
const Segment * const SG96796[] = {SGI390497, SGI390498, SGI390499, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390500, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96797[] = {SGI390501, SGI390502, SGI390503, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390504, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96798[] = {SGI390505, SGI390506, SGI390507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390508, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96799[] = {SGI390509, SGI390510, SGI390511, SGI390512, SGI0, NULL};
const Segment * const SG96800[] = {SGI390513, SGI390514, SGI390515, SGI390515, SGI0, NULL};
const Segment * const SG96801[] = {SGI390516, SGI390517, SGI390518, SGI390519, SGI0, NULL};
const Segment * const SG96802[] = {SGI390520, SGI390521, SGI0, SGI390522, SGI0, NULL};
const Segment * const SG96803[] = {SGI390523, SGI390524, SGI0, SGI390525, SGI0, NULL};
const Segment * const SG96804[] = {SGI390526, SGI390527, SGI390528, SGI390529, SGI0, NULL};
const Segment * const SG96805[] = {SGI390530, SGI390531, SGI0, SGI390532, SGI0, NULL};
const Segment * const SG96806[] = {SGI390533, SGI390534, SGI0, SGI390535, SGI0, NULL};
const Segment * const SG96807[] = {SGI390536, SGI390537, SGI0, SGI390538, SGI0, NULL};
const Segment * const SG96808[] = {SGI390539, SGI390540, SGI390541, SGI390541, SGI0, NULL};
const Segment * const SG96809[] = {SGI390542, SGI390543, SGI390544, SGI0, SGI0, SGI0, SGI0, SGI390545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96810[] = {SGI390546, SGI390547, SGI390548, SGI0, SGI0, SGI0, SGI0, SGI390549, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96811[] = {SGI390550, SGI390551, SGI390552, SGI390553, SGI0, NULL};
const Segment * const SG96812[] = {SGI390554, SGI390555, SGI390556, SGI390557, NULL};
const Segment * const SG96813[] = {SGI390558, SGI390559, SGI0, SGI390560, NULL};
const Segment * const SG96814[] = {SGI390561, SGI390562, SGI0, SGI390563, NULL};
const Segment * const SG96815[] = {SGI390564, SGI390565, SGI0, SGI390566, SGI0, NULL};
const Segment * const SG96816[] = {SGI390567, SGI390568, SGI390569, SGI0, SGI0, SGI0, SGI390570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96817[] = {SGI390571, SGI390572, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390573, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96818[] = {SGI390574, SGI390575, SGI390576, SGI0, SGI0, SGI0, SGI0, SGI390577, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96819[] = {SGI390578, SGI390579, SGI390579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96820[] = {SGI390580, SGI390581, SGI390582, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96821[] = {SGI390583, SGI390584, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390585, NULL};
const Segment * const SG96822[] = {SGI390586, SGI390587, SGI0, SGI390588, SGI0, NULL};
const Segment * const SG96823[] = {SGI390589, SGI390590, SGI0, SGI390591, SGI0, NULL};
const Segment * const SG96824[] = {SGI390592, SGI390593, SGI390594, SGI390595, SGI0, NULL};
const Segment * const SG96825[] = {SGI390596, SGI390597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390598, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96826[] = {SGI390599, SGI390600, SGI390601, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390602, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96827[] = {SGI390603, SGI390604, SGI390605, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390606, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96828[] = {SGI390607, SGI390608, SGI390609, SGI390610, SGI0, NULL};
const Segment * const SG96829[] = {SGI390611, SGI390612, SGI390613, SGI390613, SGI0, NULL};
const Segment * const SG96830[] = {SGI390614, SGI390615, SGI390616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390617, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96831[] = {SGI390618, SGI390619, SGI390619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390620, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96832[] = {SGI390621, SGI390622, SGI390623, SGI390623, SGI0, NULL};
const Segment * const SG96833[] = {SGI390624, SGI390625, SGI390626, SGI390626, SGI0, NULL};
const Segment * const SG96834[] = {SGI390627, SGI390628, SGI390629, SGI390629, SGI0, NULL};
const Segment * const SG96835[] = {SGI390630, SGI390631, SGI390631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390632, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96836[] = {SGI390633, SGI390634, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390635, NULL};
const Segment * const SG96837[] = {SGI390636, SGI390637, SGI390638, SGI390639, SGI0, NULL};
const Segment * const SG96838[] = {SGI390640, SGI390641, SGI390642, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390643, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96839[] = {SGI390644, SGI390645, SGI390646, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390647, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96840[] = {SGI390648, SGI390649, SGI390650, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390651, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96841[] = {SGI390652, SGI390653, SGI390654, SGI390654, SGI0, NULL};
const Segment * const SG96842[] = {SGI390655, SGI390656, SGI0, SGI390657, SGI0, NULL};
const Segment * const SG96843[] = {SGI390658, SGI390659, SGI390659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390660, NULL};
const Segment * const SG96844[] = {SGI390661, SGI390662, SGI390663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390664, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96845[] = {SGI390665, SGI390666, SGI0, SGI390667, SGI0, NULL};
const Segment * const SG96846[] = {SGI390668, SGI390669, SGI0, SGI390670, SGI0, NULL};
const Segment * const SG96847[] = {SGI390671, SGI390672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390673, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96848[] = {SGI390674, SGI390675, SGI390676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390677, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96849[] = {SGI390678, SGI390679, SGI390680, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96850[] = {SGI390682, SGI390683, SGI390684, SGI390684, SGI0, NULL};
const Segment * const SG96851[] = {SGI390685, SGI390686, SGI390687, SGI390688, SGI0, NULL};
const Segment * const SG96852[] = {SGI390689, SGI390690, SGI390691, SGI390692, NULL};
const Segment * const SG96853[] = {SGI390693, SGI390694, SGI0, SGI390695, SGI0, NULL};
const Segment * const SG96854[] = {SGI390696, SGI390697, SGI390698, SGI390699, SGI0, NULL};
const Segment * const SG96855[] = {SGI390700, SGI390701, SGI0, SGI390702, SGI0, SGI0, SGI0, SGI0, SGI390703, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96856[] = {SGI390704, SGI390705, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390706, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96857[] = {SGI390707, SGI390708, SGI390709, SGI390709, SGI0, NULL};
const Segment * const SG96858[] = {SGI390710, SGI390711, SGI390712, SGI390713, SGI0, SGI0, SGI0, SGI390714, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96859[] = {SGI390715, SGI390716, SGI390717, SGI0, SGI0, SGI0, SGI0, SGI390718, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96860[] = {SGI390719, SGI390720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390721, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96861[] = {SGI390722, SGI390723, SGI390724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96862[] = {SGI390726, SGI390727, SGI0, SGI390728, SGI0, NULL};
const Segment * const SG96863[] = {SGI390729, SGI390730, SGI390731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96864[] = {SGI390733, SGI390734, SGI0, SGI390735, SGI0, NULL};
const Segment * const SG96865[] = {SGI390736, SGI390737, SGI390738, SGI390738, SGI0, NULL};
const Segment * const SG96866[] = {SGI390739, SGI390740, SGI390741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390742, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96867[] = {SGI390743, SGI390744, SGI390745, SGI390746, SGI0, NULL};
const Segment * const SG96868[] = {SGI390747, SGI390748, SGI390749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390750, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96869[] = {SGI390751, SGI390752, SGI390752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96870[] = {SGI390753, SGI390754, SGI390755, SGI390756, SGI0, NULL};
const Segment * const SG96871[] = {SGI390757, SGI390758, SGI390759, SGI390760, SGI0, NULL};
const Segment * const SG96872[] = {SGI390761, SGI390762, SGI0, SGI390763, SGI0, NULL};
const Segment * const SG96873[] = {SGI390764, SGI390765, SGI390766, SGI390767, SGI0, NULL};
const Segment * const SG96874[] = {SGI390768, SGI390769, SGI390770, SGI390771, SGI0, NULL};
const Segment * const SG96875[] = {SGI390772, SGI390773, SGI390774, SGI390775, SGI0, NULL};
const Segment * const SG96876[] = {SGI390776, SGI390777, SGI390778, SGI0, SGI0, SGI0, SGI0, SGI390779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96877[] = {SGI390780, SGI390781, SGI390782, SGI390783, SGI0, NULL};
const Segment * const SG96878[] = {SGI390784, SGI390785, SGI390786, SGI390787, SGI0, NULL};
const Segment * const SG96879[] = {SGI390788, SGI390789, SGI390789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390790, NULL};
const Segment * const SG96880[] = {SGI390791, SGI390792, SGI390792, SGI390793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390794, NULL};
const Segment * const SG96881[] = {SGI390795, SGI390796, SGI390797, SGI390798, SGI0, NULL};
const Segment * const SG96882[] = {SGI390799, SGI390800, SGI0, SGI390801, SGI0, NULL};
const Segment * const SG96883[] = {SGI390802, SGI390803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96884[] = {SGI390805, SGI390806, SGI390807, SGI0, SGI0, SGI0, SGI0, SGI390808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96885[] = {SGI390809, SGI390810, SGI390811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390812, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96886[] = {SGI390813, SGI390814, SGI390814, SGI0, SGI0, SGI0, SGI390815, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96887[] = {SGI390816, SGI390817, SGI390818, SGI390818, SGI0, NULL};
const Segment * const SG96888[] = {SGI390819, SGI390820, SGI390821, SGI390821, SGI0, NULL};
const Segment * const SG96889[] = {SGI390822, SGI390823, SGI390824, SGI390825, NULL};
const Segment * const SG96890[] = {SGI390826, SGI390827, SGI390828, SGI390829, NULL};
const Segment * const SG96891[] = {SGI390830, SGI390831, SGI390832, SGI0, SGI0, NULL};
const Segment * const SG96892[] = {SGI390833, SGI390834, SGI390835, SGI390835, SGI0, NULL};
const Segment * const SG96893[] = {SGI390836, SGI390837, SGI390838, SGI390838, SGI0, NULL};
const Segment * const SG96894[] = {SGI390839, SGI390840, SGI390841, SGI0, SGI390842, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390843, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96895[] = {SGI390844, SGI390845, SGI390845, SGI0, SGI390846, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390847, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96896[] = {SGI390848, SGI390849, SGI390850, SGI390851, SGI0, NULL};
const Segment * const SG96897[] = {SGI390852, SGI390853, SGI0, SGI390854, SGI0, NULL};
const Segment * const SG96898[] = {SGI390855, SGI390856, SGI390857, SGI390858, SGI0, NULL};
const Segment * const SG96899[] = {SGI390859, SGI390860, SGI390861, SGI390861, SGI0, NULL};
const Segment * const SG96900[] = {SGI390862, SGI390863, SGI390864, SGI390865, SGI0, NULL};
const Segment * const SG96901[] = {SGI390866, SGI390867, SGI390868, SGI390869, SGI0, NULL};
const Segment * const SG96902[] = {SGI390870, SGI390871, SGI390872, SGI390873, SGI0, NULL};
const Segment * const SG96903[] = {SGI390874, SGI390875, SGI390876, SGI0, SGI0, SGI0, SGI390877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96904[] = {SGI390878, SGI390879, SGI390880, SGI390881, SGI0, NULL};
const Segment * const SG96905[] = {SGI390882, SGI390883, SGI390884, SGI390884, SGI0, NULL};
const Segment * const SG96906[] = {SGI390885, SGI390886, SGI390887, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390888, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96907[] = {SGI390889, SGI390890, SGI390890, SGI0, SGI390891, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390892, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96908[] = {SGI390893, SGI390894, SGI390895, SGI0, SGI390896, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390897, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96909[] = {SGI390898, SGI390899, SGI0, SGI390900, SGI0, NULL};
const Segment * const SG96910[] = {SGI390901, SGI390902, SGI390903, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390904, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96911[] = {SGI390905, SGI390906, SGI390906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390907, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96912[] = {SGI390908, SGI390909, SGI390909, SGI0, NULL};
const Segment * const SG96913[] = {SGI390910, SGI390911, SGI0, SGI390912, SGI0, NULL};
const Segment * const SG96914[] = {SGI390913, SGI390914, SGI390915, SGI390916, SGI0, NULL};
const Segment * const SG96915[] = {SGI390917, SGI390918, SGI390918, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390919, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96916[] = {SGI390920, SGI390921, SGI390921, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390922, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96917[] = {SGI390923, SGI390924, SGI390925, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390926, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96918[] = {SGI390927, SGI390928, SGI390929, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390930, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96919[] = {SGI390931, SGI0, SGI390932, SGI0, NULL};
const Segment * const SG96920[] = {SGI390933, SGI390934, SGI390935, SGI0, SGI0, SGI0, SGI0, SGI390936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96921[] = {SGI390937, SGI390938, SGI390939, SGI390940, SGI0, NULL};
const Segment * const SG96922[] = {SGI390941, SGI390942, SGI390943, SGI390944, SGI0, NULL};
const Segment * const SG96923[] = {SGI390945, SGI390946, SGI390947, SGI390948, SGI0, NULL};
const Segment * const SG96924[] = {SGI390949, SGI390950, SGI390951, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390952, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96925[] = {SGI390953, SGI390954, SGI390955, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390956, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96926[] = {SGI390957, SGI390958, SGI390959, SGI390960, SGI0, NULL};
const Segment * const SG96927[] = {SGI390961, SGI390962, SGI0, SGI390963, SGI0, NULL};
const Segment * const SG96928[] = {SGI390964, SGI390965, SGI390966, SGI390967, SGI0, NULL};
const Segment * const SG96929[] = {SGI390968, SGI390969, SGI390970, SGI390971, SGI0, NULL};
const Segment * const SG96930[] = {SGI390972, SGI390973, SGI390974, SGI390975, NULL};
const Segment * const SG96931[] = {SGI390976, SGI390977, SGI390977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96932[] = {SGI390979, SGI390980, SGI390981, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96933[] = {SGI390983, SGI390984, SGI390985, SGI390985, SGI0, NULL};
const Segment * const SG96934[] = {SGI390986, SGI390987, SGI390988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390989, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96935[] = {SGI390990, SGI390991, SGI390992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI390993, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96936[] = {SGI390994, SGI390995, SGI390996, SGI0, SGI0, SGI390997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96937[] = {SGI390998, SGI390999, SGI391000, SGI0, SGI0, SGI391001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96938[] = {SGI391002, SGI391003, SGI391003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391004, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96939[] = {SGI391005, SGI391006, SGI391007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391008, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96940[] = {SGI391009, SGI391010, SGI391011, SGI391012, SGI0, NULL};
const Segment * const SG96941[] = {SGI391013, SGI391014, SGI391015, SGI391015, SGI0, NULL};
const Segment * const SG96942[] = {SGI391016, SGI391017, SGI391018, SGI391018, SGI0, NULL};
const Segment * const SG96943[] = {SGI391019, SGI391020, SGI391021, SGI0, SGI0, SGI0, SGI0, SGI391022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96944[] = {SGI391023, SGI391024, SGI391024, SGI0, SGI0, SGI0, SGI0, SGI391025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96945[] = {SGI391026, SGI391027, SGI391028, SGI391028, SGI0, NULL};
const Segment * const SG96946[] = {SGI391029, SGI391030, SGI0, SGI391031, SGI0, NULL};
const Segment * const SG96947[] = {SGI391032, SGI391033, SGI391034, SGI391035, SGI0, NULL};
const Segment * const SG96948[] = {SGI391036, SGI391037, SGI391038, SGI391039, SGI0, NULL};
const Segment * const SG96949[] = {SGI391040, SGI391041, SGI391042, SGI391043, SGI0, NULL};
const Segment * const SG96950[] = {SGI391044, SGI391045, SGI391046, SGI391047, SGI0, NULL};
const Segment * const SG96951[] = {SGI391048, SGI391049, SGI391050, SGI391051, SGI0, NULL};
const Segment * const SG96952[] = {SGI391052, SGI391053, SGI391054, SGI391054, SGI0, NULL};
const Segment * const SG96953[] = {SGI391055, SGI391056, SGI0, SGI391057, SGI0, NULL};
const Segment * const SG96954[] = {SGI391058, SGI391059, SGI391060, SGI391061, NULL};
const Segment * const SG96955[] = {SGI391062, SGI391063, SGI391064, SGI391065, NULL};
const Segment * const SG96956[] = {SGI391066, SGI391067, SGI391068, SGI391068, SGI0, NULL};
const Segment * const SG96957[] = {SGI391069, SGI391070, SGI391070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391071, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96958[] = {SGI391072, SGI391073, SGI391074, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96959[] = {SGI391076, SGI391077, SGI391078, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391079, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96960[] = {SGI391080, SGI391081, SGI0, SGI391082, SGI0, NULL};
const Segment * const SG96961[] = {SGI391083, SGI391084, SGI391085, SGI391085, SGI0, NULL};
const Segment * const SG96962[] = {SGI391086, SGI391087, SGI391088, SGI0, NULL};
const Segment * const SG96963[] = {SGI391089, SGI391090, SGI391091, SGI0, NULL};
const Segment * const SG96964[] = {SGI391092, SGI391093, SGI391094, SGI0, NULL};
const Segment * const SG96965[] = {SGI391095, SGI391096, SGI391097, SGI0, NULL};
const Segment * const SG96966[] = {SGI391098, SGI391099, SGI391100, SGI0, NULL};
const Segment * const SG96967[] = {SGI391101, SGI391102, SGI391103, SGI0, NULL};
const Segment * const SG96968[] = {SGI391104, SGI391105, SGI391106, SGI0, SGI0, SGI0, SGI0, SGI391107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96969[] = {SGI391108, SGI391109, SGI391110, SGI391110, SGI0, NULL};
const Segment * const SG96970[] = {SGI391111, SGI391112, SGI391112, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391113, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96971[] = {SGI391114, SGI391115, SGI0, SGI391116, SGI0, NULL};
const Segment * const SG96972[] = {SGI391117, SGI391118, SGI391118, SGI0, SGI391119, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391120, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96973[] = {SGI391121, SGI391122, SGI391123, SGI0, SGI391124, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391125, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96974[] = {SGI391126, SGI391127, SGI391128, SGI391128, SGI0, NULL};
const Segment * const SG96975[] = {SGI391129, SGI0, SGI391130, SGI0, NULL};
const Segment * const SG96976[] = {SGI391131, SGI0, SGI391132, SGI0, NULL};
const Segment * const SG96977[] = {SGI391133, SGI391134, SGI0, SGI391135, SGI0, NULL};
const Segment * const SG96978[] = {SGI391136, SGI391137, SGI391138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391139, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96979[] = {SGI391140, SGI391141, SGI391142, SGI391143, SGI0, NULL};
const Segment * const SG96980[] = {SGI391144, SGI391145, SGI391146, SGI391147, SGI0, NULL};
const Segment * const SG96981[] = {SGI391148, SGI391149, SGI391150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391151, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96982[] = {SGI391152, SGI391153, SGI391154, SGI0, SGI0, SGI0, SGI0, SGI391155, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG96983[] = {SGI391156, SGI391157, SGI391158, SGI391158, SGI0, NULL};
const Segment * const SG96984[] = {SGI391159, SGI391160, SGI391161, SGI0, NULL};
const Segment * const SG96985[] = {SGI391162, SGI391163, SGI391164, SGI0, NULL};
const Segment * const SG96986[] = {SGI391165, SGI391166, SGI391167, SGI0, NULL};
const Segment * const SG96987[] = {SGI391168, SGI391169, SGI391170, SGI0, NULL};
const Segment * const SG96988[] = {SGI391171, SGI391172, SGI391173, SGI0, NULL};
const Segment * const SG96989[] = {SGI391174, SGI391175, SGI391176, SGI0, NULL};
const Segment * const SG96990[] = {SGI391177, SGI391178, SGI391179, SGI0, NULL};
const Segment * const SG96991[] = {SGI391180, SGI391181, SGI391182, SGI0, NULL};
const Segment * const SG96992[] = {SGI391183, SGI391184, SGI391185, SGI0, NULL};
const Segment * const SG96993[] = {SGI391186, SGI391187, SGI391188, SGI0, NULL};
const Segment * const SG96994[] = {SGI391189, SGI391190, SGI391191, SGI0, NULL};
const Segment * const SG96995[] = {SGI391192, SGI391193, SGI391194, SGI0, NULL};
const Segment * const SG96996[] = {SGI391195, SGI391196, SGI391197, SGI0, NULL};
const Segment * const SG96997[] = {SGI391198, SGI391199, SGI391200, SGI0, NULL};
const Segment * const SG96998[] = {SGI391201, SGI391202, SGI391203, SGI0, NULL};
const Segment * const SG96999[] = {SGI391204, SGI391205, SGI391206, SGI0, NULL};
const Segment * const SG97000[] = {SGI391207, SGI391208, SGI391209, SGI0, NULL};
const Segment * const SG97001[] = {SGI391210, SGI391211, SGI391212, SGI0, NULL};
const Segment * const SG97002[] = {SGI391213, SGI391214, SGI391215, SGI0, NULL};
const Segment * const SG97003[] = {SGI391216, SGI391217, SGI391218, SGI0, NULL};
const Segment * const SG97004[] = {SGI391219, SGI391220, SGI391221, SGI0, NULL};
const Segment * const SG97005[] = {SGI391222, SGI391223, SGI391224, SGI0, NULL};
const Segment * const SG97006[] = {SGI391225, SGI391226, SGI391227, SGI0, NULL};
const Segment * const SG97007[] = {SGI391228, SGI391229, SGI391230, SGI0, NULL};
const Segment * const SG97008[] = {SGI391231, SGI391232, SGI391233, SGI0, NULL};
const Segment * const SG97009[] = {SGI391234, SGI391235, SGI391236, SGI0, NULL};
const Segment * const SG97010[] = {SGI391237, SGI391238, SGI391239, SGI0, NULL};
const Segment * const SG97011[] = {SGI391240, SGI391241, SGI391242, SGI0, NULL};
const Segment * const SG97012[] = {SGI391243, SGI391244, SGI391245, SGI0, NULL};
const Segment * const SG97013[] = {SGI391246, SGI391247, SGI391248, SGI0, NULL};
const Segment * const SG97014[] = {SGI391249, SGI391250, SGI391251, SGI0, NULL};
const Segment * const SG97015[] = {SGI391252, SGI391253, SGI391254, SGI0, NULL};
const Segment * const SG97016[] = {SGI391255, SGI391256, SGI391257, SGI0, NULL};
const Segment * const SG97017[] = {SGI391258, SGI391259, SGI391260, SGI0, NULL};
const Segment * const SG97018[] = {SGI391261, SGI391262, SGI391263, SGI0, NULL};
const Segment * const SG97019[] = {SGI391264, SGI391265, SGI391266, SGI0, NULL};
const Segment * const SG97020[] = {SGI391267, SGI391268, SGI391269, SGI0, NULL};
const Segment * const SG97021[] = {SGI391270, SGI391271, SGI391272, SGI0, NULL};
const Segment * const SG97022[] = {SGI391273, SGI391274, SGI391275, SGI0, NULL};
const Segment * const SG97023[] = {SGI391276, SGI391277, SGI391278, SGI0, NULL};
const Segment * const SG97024[] = {SGI391279, SGI391280, SGI391281, SGI0, NULL};
const Segment * const SG97025[] = {SGI391282, SGI391283, SGI391284, SGI0, NULL};
const Segment * const SG97026[] = {SGI391285, SGI391286, SGI391287, SGI0, NULL};
const Segment * const SG97027[] = {SGI391288, SGI391289, SGI391290, SGI0, NULL};
const Segment * const SG97028[] = {SGI391291, SGI391292, SGI391293, SGI0, NULL};
const Segment * const SG97029[] = {SGI391294, SGI391295, SGI391296, SGI0, NULL};
const Segment * const SG97030[] = {SGI391297, SGI391298, SGI391299, SGI0, NULL};
const Segment * const SG97031[] = {SGI391300, SGI391301, SGI391302, SGI0, NULL};
const Segment * const SG97032[] = {SGI391303, SGI391304, SGI391305, SGI0, NULL};
const Segment * const SG97033[] = {SGI391306, SGI391307, SGI391308, SGI0, NULL};
const Segment * const SG97034[] = {SGI391309, SGI391310, SGI391311, SGI0, NULL};
const Segment * const SG97035[] = {SGI391312, SGI391313, SGI391314, SGI0, NULL};
const Segment * const SG97036[] = {SGI391315, SGI391316, SGI391317, SGI0, NULL};
const Segment * const SG97037[] = {SGI391318, SGI391319, SGI391320, SGI0, NULL};
const Segment * const SG97038[] = {SGI391321, SGI391322, SGI391323, SGI0, NULL};
const Segment * const SG97039[] = {SGI391324, SGI391325, SGI391326, SGI0, NULL};
const Segment * const SG97040[] = {SGI391327, SGI391328, SGI391329, SGI0, NULL};
const Segment * const SG97041[] = {SGI391330, SGI391331, SGI391332, SGI0, NULL};
const Segment * const SG97042[] = {SGI391333, SGI391334, SGI391334, SGI0, SGI391335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391336, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97043[] = {SGI391337, SGI391338, SGI391339, SGI391339, SGI0, NULL};
const Segment * const SG97044[] = {SGI391340, SGI391341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391342, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97045[] = {SGI391343, SGI391344, SGI0, SGI391345, SGI0, NULL};
const Segment * const SG97046[] = {SGI391346, SGI391347, SGI391348, SGI0, NULL};
const Segment * const SG97047[] = {SGI391349, SGI391350, SGI391351, SGI0, NULL};
const Segment * const SG97048[] = {SGI391352, SGI391353, SGI391354, SGI0, NULL};
const Segment * const SG97049[] = {SGI391355, SGI391356, SGI391357, SGI0, NULL};
const Segment * const SG97050[] = {SGI391358, SGI391359, SGI391360, SGI0, NULL};
const Segment * const SG97051[] = {SGI391361, SGI391362, SGI391363, SGI0, NULL};
const Segment * const SG97052[] = {SGI391364, SGI391365, SGI391366, SGI0, NULL};
const Segment * const SG97053[] = {SGI391367, SGI391368, SGI391369, SGI0, NULL};
const Segment * const SG97054[] = {SGI391370, SGI391371, SGI391372, SGI0, NULL};
const Segment * const SG97055[] = {SGI391373, SGI391374, SGI391375, SGI0, NULL};
const Segment * const SG97056[] = {SGI391376, SGI391377, SGI391378, SGI0, NULL};
const Segment * const SG97057[] = {SGI391379, SGI391380, SGI391381, SGI0, NULL};
const Segment * const SG97058[] = {SGI391382, SGI391383, SGI391384, SGI0, NULL};
const Segment * const SG97059[] = {SGI391385, SGI391386, SGI391387, SGI0, NULL};
const Segment * const SG97060[] = {SGI391388, SGI391389, SGI391390, SGI0, NULL};
const Segment * const SG97061[] = {SGI391391, SGI391392, SGI391393, SGI0, NULL};
const Segment * const SG97062[] = {SGI391394, SGI391395, SGI391396, SGI0, NULL};
const Segment * const SG97063[] = {SGI391397, SGI391398, SGI391399, SGI0, NULL};
const Segment * const SG97064[] = {SGI391400, SGI391401, SGI391402, SGI0, NULL};
const Segment * const SG97065[] = {SGI391403, SGI391404, SGI391405, SGI0, NULL};
const Segment * const SG97066[] = {SGI391406, SGI391407, SGI391408, SGI0, NULL};
const Segment * const SG97067[] = {SGI391409, SGI391410, SGI391411, SGI0, NULL};
const Segment * const SG97068[] = {SGI391412, SGI391413, SGI391414, SGI0, NULL};
const Segment * const SG97069[] = {SGI391415, SGI391416, SGI391417, SGI0, NULL};
const Segment * const SG97070[] = {SGI391418, SGI391419, SGI391420, SGI0, NULL};
const Segment * const SG97071[] = {SGI391421, SGI391422, SGI391423, SGI0, NULL};
const Segment * const SG97072[] = {SGI391424, SGI391425, SGI391426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97073[] = {SGI391428, SGI391429, SGI391430, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97074[] = {SGI391432, SGI391433, SGI391434, SGI391434, SGI0, NULL};
const Segment * const SG97075[] = {SGI391435, SGI0, SGI391436, SGI0, NULL};
const Segment * const SG97076[] = {SGI391437, SGI0, SGI391438, SGI0, NULL};
const Segment * const SG97077[] = {SGI391439, SGI0, SGI391440, SGI0, NULL};
const Segment * const SG97078[] = {SGI391441, SGI0, SGI391442, SGI0, NULL};
const Segment * const SG97079[] = {SGI391443, SGI0, SGI391444, SGI0, NULL};
const Segment * const SG97080[] = {SGI391445, SGI0, SGI391446, SGI0, NULL};
const Segment * const SG97081[] = {SGI391447, SGI0, SGI391448, SGI0, NULL};
const Segment * const SG97082[] = {SGI391449, SGI0, SGI391450, SGI0, NULL};
const Segment * const SG97083[] = {SGI391451, SGI0, SGI391452, SGI0, NULL};
const Segment * const SG97084[] = {SGI391453, SGI0, SGI391454, SGI0, NULL};
const Segment * const SG97085[] = {SGI391455, SGI0, SGI391456, SGI0, NULL};
const Segment * const SG97086[] = {SGI391457, SGI0, SGI391458, SGI0, NULL};
const Segment * const SG97087[] = {SGI391459, SGI0, SGI391460, SGI0, NULL};
const Segment * const SG97088[] = {SGI391461, SGI0, SGI391462, SGI0, NULL};
const Segment * const SG97089[] = {SGI391463, SGI0, SGI391464, SGI0, NULL};
const Segment * const SG97090[] = {SGI391465, SGI0, SGI391466, SGI0, NULL};
const Segment * const SG97091[] = {SGI391467, SGI0, SGI391468, SGI0, NULL};
const Segment * const SG97092[] = {SGI391469, SGI0, SGI391470, SGI0, NULL};
const Segment * const SG97093[] = {SGI391471, SGI0, SGI391472, SGI0, NULL};
const Segment * const SG97094[] = {SGI391473, SGI0, SGI391474, SGI0, NULL};
const Segment * const SG97095[] = {SGI391475, SGI0, SGI391476, SGI0, NULL};
const Segment * const SG97096[] = {SGI391477, SGI0, SGI391478, SGI0, NULL};
const Segment * const SG97097[] = {SGI391479, SGI0, SGI391480, SGI0, NULL};
const Segment * const SG97098[] = {SGI391481, SGI0, SGI391482, SGI0, NULL};
const Segment * const SG97099[] = {SGI391483, SGI0, SGI391484, SGI0, NULL};
const Segment * const SG97100[] = {SGI391485, SGI0, SGI391486, SGI0, NULL};
const Segment * const SG97101[] = {SGI391487, SGI0, SGI391488, SGI0, NULL};
const Segment * const SG97102[] = {SGI391489, SGI0, SGI391490, SGI0, NULL};
const Segment * const SG97103[] = {SGI391491, SGI0, SGI391492, SGI0, NULL};
const Segment * const SG97104[] = {SGI391493, SGI0, SGI391494, SGI0, NULL};
const Segment * const SG97105[] = {SGI391495, SGI0, SGI391496, SGI0, NULL};
const Segment * const SG97106[] = {SGI391497, SGI0, SGI391498, SGI0, NULL};
const Segment * const SG97107[] = {SGI391499, SGI0, SGI391500, SGI0, NULL};
const Segment * const SG97108[] = {SGI391501, SGI0, SGI391502, SGI0, NULL};
const Segment * const SG97109[] = {SGI391503, SGI0, SGI391504, SGI0, NULL};
const Segment * const SG97110[] = {SGI391505, SGI0, SGI391506, SGI0, NULL};
const Segment * const SG97111[] = {SGI391507, SGI0, SGI391508, SGI0, NULL};
const Segment * const SG97112[] = {SGI391509, SGI0, SGI391510, SGI0, NULL};
const Segment * const SG97113[] = {SGI391511, SGI0, SGI391512, SGI0, NULL};
const Segment * const SG97114[] = {SGI391513, SGI0, SGI391514, SGI0, NULL};
const Segment * const SG97115[] = {SGI391515, SGI0, SGI391516, SGI0, NULL};
const Segment * const SG97116[] = {SGI391517, SGI0, SGI391518, SGI0, NULL};
const Segment * const SG97117[] = {SGI391519, SGI0, SGI391520, SGI0, NULL};
const Segment * const SG97118[] = {SGI391521, SGI0, SGI391522, SGI0, NULL};
const Segment * const SG97119[] = {SGI391523, SGI0, SGI391524, SGI0, NULL};
const Segment * const SG97120[] = {SGI391525, SGI0, SGI391526, SGI0, NULL};
const Segment * const SG97121[] = {SGI391527, SGI0, SGI391528, SGI0, NULL};
const Segment * const SG97122[] = {SGI391529, SGI0, SGI391530, SGI0, NULL};
const Segment * const SG97123[] = {SGI391531, SGI0, SGI391532, SGI0, NULL};
const Segment * const SG97124[] = {SGI391533, SGI0, SGI391534, SGI0, NULL};
const Segment * const SG97125[] = {SGI391535, SGI0, SGI391536, SGI0, NULL};
const Segment * const SG97126[] = {SGI391537, SGI0, SGI391538, SGI0, NULL};
const Segment * const SG97127[] = {SGI391539, SGI0, SGI391540, SGI0, NULL};
const Segment * const SG97128[] = {SGI391541, SGI0, SGI391542, SGI0, NULL};
const Segment * const SG97129[] = {SGI391543, SGI0, SGI391544, SGI0, NULL};
const Segment * const SG97130[] = {SGI391545, SGI0, SGI391546, SGI0, NULL};
const Segment * const SG97131[] = {SGI391547, SGI0, SGI391548, SGI0, NULL};
const Segment * const SG97132[] = {SGI391549, SGI0, SGI391550, SGI0, NULL};
const Segment * const SG97133[] = {SGI391551, SGI0, SGI391552, SGI0, NULL};
const Segment * const SG97134[] = {SGI391553, SGI0, SGI391554, SGI0, NULL};
const Segment * const SG97135[] = {SGI391555, SGI0, SGI391556, SGI0, NULL};
const Segment * const SG97136[] = {SGI391557, SGI0, SGI391558, SGI0, NULL};
const Segment * const SG97137[] = {SGI391559, SGI0, SGI391560, SGI0, NULL};
const Segment * const SG97138[] = {SGI391561, SGI0, SGI391562, SGI0, NULL};
const Segment * const SG97139[] = {SGI391563, SGI0, SGI391564, SGI0, NULL};
const Segment * const SG97140[] = {SGI391565, SGI0, SGI391566, SGI0, NULL};
const Segment * const SG97141[] = {SGI391567, SGI0, SGI391568, SGI0, NULL};
const Segment * const SG97142[] = {SGI391569, SGI0, SGI391570, SGI0, NULL};
const Segment * const SG97143[] = {SGI391571, SGI0, SGI391572, SGI0, NULL};
const Segment * const SG97144[] = {SGI391573, SGI0, SGI391574, SGI0, NULL};
const Segment * const SG97145[] = {SGI391575, SGI0, SGI391576, SGI0, NULL};
const Segment * const SG97146[] = {SGI391577, SGI0, SGI391578, SGI0, NULL};
const Segment * const SG97147[] = {SGI391579, SGI0, SGI391580, SGI0, NULL};
const Segment * const SG97148[] = {SGI391581, SGI0, SGI391582, SGI0, NULL};
const Segment * const SG97149[] = {SGI391583, SGI0, SGI391584, SGI0, NULL};
const Segment * const SG97150[] = {SGI391585, SGI0, SGI391586, SGI0, NULL};
const Segment * const SG97151[] = {SGI391587, SGI0, SGI391588, SGI0, NULL};
const Segment * const SG97152[] = {SGI391589, SGI0, SGI391590, SGI0, NULL};
const Segment * const SG97153[] = {SGI391591, SGI0, SGI391592, SGI0, NULL};
const Segment * const SG97154[] = {SGI391593, SGI0, SGI391594, SGI0, NULL};
const Segment * const SG97155[] = {SGI391595, SGI0, SGI391596, SGI0, NULL};
const Segment * const SG97156[] = {SGI391597, SGI391598, SGI391599, SGI391599, SGI0, NULL};
const Segment * const SG97157[] = {SGI391600, SGI391601, SGI391602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391603, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97158[] = {SGI391604, SGI391605, SGI391606, SGI391607, SGI0, NULL};
const Segment * const SG97159[] = {SGI391608, SGI391609, SGI391610, SGI391610, SGI0, NULL};
const Segment * const SG97160[] = {SGI391611, SGI391612, SGI391613, SGI391614, SGI0, NULL};
const Segment * const SG97161[] = {SGI391615, SGI391616, SGI391617, SGI0, SGI0, SGI0, SGI0, SGI391618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97162[] = {SGI391619, SGI391620, SGI391621, SGI391622, SGI0, NULL};
const Segment * const SG97163[] = {SGI391623, SGI391624, SGI391625, SGI391625, SGI0, NULL};
const Segment * const SG97164[] = {SGI391626, SGI391627, SGI391628, SGI391629, SGI0, NULL};
const Segment * const SG97165[] = {SGI391630, SGI391631, SGI391632, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391633, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97166[] = {SGI391634, SGI391635, SGI391636, SGI0, NULL};
const Segment * const SG97167[] = {SGI391637, SGI391638, SGI391639, SGI0, NULL};
const Segment * const SG97168[] = {SGI391640, SGI391641, SGI391642, SGI0, NULL};
const Segment * const SG97169[] = {SGI391643, SGI391644, SGI391645, SGI0, NULL};
const Segment * const SG97170[] = {SGI391646, SGI391647, SGI391648, SGI0, NULL};
const Segment * const SG97171[] = {SGI391649, SGI391650, SGI391651, SGI0, NULL};
const Segment * const SG97172[] = {SGI391652, SGI391653, SGI391654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391655, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97173[] = {SGI391656, SGI391657, SGI391658, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391659, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97174[] = {SGI391660, SGI391661, SGI0, SGI391662, SGI0, SGI0, SGI0, SGI0, SGI391663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97175[] = {SGI391664, SGI391665, SGI391666, SGI391667, SGI0, NULL};
const Segment * const SG97176[] = {SGI391668, SGI391669, SGI391670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391671, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97177[] = {SGI391672, SGI391673, SGI391674, SGI391675, SGI0, NULL};
const Segment * const SG97178[] = {SGI391676, SGI391677, SGI391678, SGI0, NULL};
const Segment * const SG97179[] = {SGI391679, SGI391680, SGI391681, SGI0, NULL};
const Segment * const SG97180[] = {SGI391682, SGI391683, SGI391684, SGI0, NULL};
const Segment * const SG97181[] = {SGI391685, SGI391686, SGI391687, SGI0, NULL};
const Segment * const SG97182[] = {SGI391688, SGI391689, SGI391690, SGI0, NULL};
const Segment * const SG97183[] = {SGI391691, SGI391692, SGI391693, SGI0, NULL};
const Segment * const SG97184[] = {SGI391694, SGI391695, SGI391696, SGI0, NULL};
const Segment * const SG97185[] = {SGI391697, SGI391698, SGI391699, SGI0, NULL};
const Segment * const SG97186[] = {SGI391700, SGI391701, SGI391702, SGI0, NULL};
const Segment * const SG97187[] = {SGI391703, SGI391704, SGI391705, SGI0, NULL};
const Segment * const SG97188[] = {SGI391706, SGI391707, SGI391708, SGI0, NULL};
const Segment * const SG97189[] = {SGI391709, SGI391710, SGI391711, SGI0, NULL};
const Segment * const SG97190[] = {SGI391712, SGI391713, SGI391714, SGI0, NULL};
const Segment * const SG97191[] = {SGI391715, SGI391716, SGI391717, SGI0, NULL};
const Segment * const SG97192[] = {SGI391718, SGI391719, SGI391720, SGI0, NULL};
const Segment * const SG97193[] = {SGI391721, SGI391722, SGI391723, SGI0, NULL};
const Segment * const SG97194[] = {SGI391724, SGI391725, SGI391726, SGI0, NULL};
const Segment * const SG97195[] = {SGI391727, SGI391728, SGI391729, SGI0, NULL};
const Segment * const SG97196[] = {SGI391730, SGI391731, SGI391732, SGI0, NULL};
const Segment * const SG97197[] = {SGI391733, SGI391734, SGI391735, SGI0, NULL};
const Segment * const SG97198[] = {SGI391736, SGI391737, SGI391738, SGI0, NULL};
const Segment * const SG97199[] = {SGI391739, SGI391740, SGI391741, SGI0, NULL};
const Segment * const SG97200[] = {SGI391742, SGI391743, SGI391744, SGI0, NULL};
const Segment * const SG97201[] = {SGI391745, SGI391746, SGI391747, SGI0, NULL};
const Segment * const SG97202[] = {SGI391748, SGI391749, SGI391750, SGI0, NULL};
const Segment * const SG97203[] = {SGI391751, SGI391752, SGI391753, SGI0, NULL};
const Segment * const SG97204[] = {SGI391754, SGI391755, SGI391756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391757, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97205[] = {SGI391758, SGI391759, SGI391760, SGI0, SGI0, SGI0, SGI391761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97206[] = {SGI391762, SGI391763, SGI391763, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391764, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97207[] = {SGI391765, SGI391766, SGI0, SGI391767, SGI0, NULL};
const Segment * const SG97208[] = {SGI391768, SGI391769, SGI0, SGI391770, SGI0, NULL};
const Segment * const SG97209[] = {SGI391771, SGI391772, SGI391773, SGI391774, SGI0, NULL};
const Segment * const SG97210[] = {SGI391775, SGI391776, SGI391777, SGI391777, SGI0, NULL};
const Segment * const SG97211[] = {SGI391778, SGI0, SGI391779, SGI0, NULL};
const Segment * const SG97212[] = {SGI391780, SGI391781, SGI0, SGI391782, SGI0, NULL};
const Segment * const SG97213[] = {SGI391783, SGI391784, SGI391785, SGI391786, SGI0, NULL};
const Segment * const SG97214[] = {SGI391787, SGI391788, SGI391789, SGI391790, SGI0, NULL};
const Segment * const SG97215[] = {SGI391791, SGI391792, SGI0, SGI391793, NULL};
const Segment * const SG97216[] = {SGI391794, SGI391795, SGI0, SGI391796, SGI0, NULL};
const Segment * const SG97217[] = {SGI391797, SGI391798, SGI391799, SGI391800, NULL};
const Segment * const SG97218[] = {SGI391801, SGI391802, SGI391803, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97219[] = {SGI391805, SGI391806, SGI391806, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391807, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97220[] = {SGI391808, SGI391809, SGI0, SGI391810, SGI0, SGI0, SGI0, SGI0, SGI391811, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97221[] = {SGI391812, SGI391813, SGI391814, SGI0, NULL};
const Segment * const SG97222[] = {SGI391815, SGI391816, SGI391817, SGI0, NULL};
const Segment * const SG97223[] = {SGI391818, SGI391819, SGI391820, SGI0, NULL};
const Segment * const SG97224[] = {SGI391821, SGI391822, SGI391823, SGI0, NULL};
const Segment * const SG97225[] = {SGI391824, SGI391825, SGI391826, SGI0, NULL};
const Segment * const SG97226[] = {SGI391827, SGI391828, SGI391829, SGI0, NULL};
const Segment * const SG97227[] = {SGI391830, SGI391831, SGI391831, SGI0, SGI0, SGI0, SGI391832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97228[] = {SGI391833, SGI391834, SGI391835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391836, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97229[] = {SGI391837, SGI391838, SGI391839, SGI0, NULL};
const Segment * const SG97230[] = {SGI391840, SGI391841, SGI391842, SGI0, NULL};
const Segment * const SG97231[] = {SGI391843, SGI391844, SGI391845, SGI0, NULL};
const Segment * const SG97232[] = {SGI391846, SGI391847, SGI391848, SGI0, NULL};
const Segment * const SG97233[] = {SGI391849, SGI391850, SGI391851, SGI0, NULL};
const Segment * const SG97234[] = {SGI391852, SGI391853, SGI391854, SGI0, NULL};
const Segment * const SG97235[] = {SGI391855, SGI391856, SGI391857, SGI0, NULL};
const Segment * const SG97236[] = {SGI391858, SGI391859, SGI391860, SGI0, NULL};
const Segment * const SG97237[] = {SGI391861, SGI391862, SGI391863, SGI0, NULL};
const Segment * const SG97238[] = {SGI391864, SGI391865, SGI391866, SGI0, NULL};
const Segment * const SG97239[] = {SGI391867, SGI391868, SGI391869, SGI0, NULL};
const Segment * const SG97240[] = {SGI391870, SGI391871, SGI391872, SGI0, NULL};
const Segment * const SG97241[] = {SGI391873, SGI391874, SGI0, SGI391875, SGI0, NULL};
const Segment * const SG97242[] = {SGI391876, SGI0, SGI391877, SGI0, NULL};
const Segment * const SG97243[] = {SGI391878, SGI391879, SGI391880, SGI0, NULL};
const Segment * const SG97244[] = {SGI391881, SGI391882, SGI391883, SGI0, NULL};
const Segment * const SG97245[] = {SGI391884, SGI391885, SGI391886, SGI0, NULL};
const Segment * const SG97246[] = {SGI391887, SGI391888, SGI391889, SGI0, NULL};
const Segment * const SG97247[] = {SGI391890, SGI391891, SGI391892, SGI0, NULL};
const Segment * const SG97248[] = {SGI391893, SGI391894, SGI391895, SGI0, NULL};
const Segment * const SG97249[] = {SGI391896, SGI391897, SGI391898, SGI0, NULL};
const Segment * const SG97250[] = {SGI391899, SGI391900, SGI391901, SGI0, NULL};
const Segment * const SG97251[] = {SGI391902, SGI391903, SGI391904, SGI0, NULL};
const Segment * const SG97252[] = {SGI391905, SGI391906, SGI391907, SGI0, NULL};
const Segment * const SG97253[] = {SGI391908, SGI391909, SGI391910, SGI0, NULL};
const Segment * const SG97254[] = {SGI391911, SGI391912, SGI391913, SGI0, NULL};
const Segment * const SG97255[] = {SGI391914, SGI391915, SGI391916, SGI0, NULL};
const Segment * const SG97256[] = {SGI391917, SGI391918, SGI391919, SGI0, NULL};
const Segment * const SG97257[] = {SGI391920, SGI391921, SGI391922, SGI0, NULL};
const Segment * const SG97258[] = {SGI391923, SGI391924, SGI391925, SGI0, NULL};
const Segment * const SG97259[] = {SGI391926, SGI391927, SGI391928, SGI0, NULL};
const Segment * const SG97260[] = {SGI391929, SGI391930, SGI391931, SGI391932, SGI0, NULL};
const Segment * const SG97261[] = {SGI391933, SGI391934, SGI391935, SGI391936, SGI0, NULL};
const Segment * const SG97262[] = {SGI391937, SGI391938, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391939, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97263[] = {SGI391940, SGI391941, SGI391942, SGI0, SGI0, SGI0, SGI0, SGI391943, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97264[] = {SGI391944, SGI391945, SGI0, SGI391946, SGI0, NULL};
const Segment * const SG97265[] = {SGI391947, SGI0, SGI0, SGI391948, SGI391949, NULL};
const Segment * const SG97266[] = {SGI391950, SGI391951, SGI391951, SGI0, SGI0, SGI0, SGI0, SGI391952, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97267[] = {SGI391953, SGI391954, SGI391955, SGI0, SGI0, SGI0, SGI0, SGI391956, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97268[] = {SGI391957, SGI391958, SGI391959, SGI391959, SGI0, NULL};
const Segment * const SG97269[] = {SGI391960, SGI391961, SGI391962, SGI391962, SGI0, NULL};
const Segment * const SG97270[] = {SGI391963, SGI391964, SGI391965, SGI0, SGI0, SGI0, SGI0, SGI391966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97271[] = {SGI391967, SGI391968, SGI391969, SGI391970, SGI0, NULL};
const Segment * const SG97272[] = {SGI391971, SGI391972, SGI391973, SGI391974, SGI0, NULL};
const Segment * const SG97273[] = {SGI391975, SGI391976, SGI391977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391978, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97274[] = {SGI391979, SGI391980, SGI391981, SGI391981, SGI0, NULL};
const Segment * const SG97275[] = {SGI391982, SGI391983, SGI391984, SGI0, SGI0, SGI0, SGI0, SGI391985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97276[] = {SGI391986, SGI391987, SGI391988, SGI391989, NULL};
const Segment * const SG97277[] = {SGI391990, SGI391991, SGI391991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI391992, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97278[] = {SGI391993, SGI391994, SGI391995, SGI0, SGI0, SGI0, SGI0, SGI391996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97279[] = {SGI391997, SGI391998, SGI391999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392000, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97280[] = {SGI392001, SGI392002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392003, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97281[] = {SGI392004, SGI0, SGI392005, SGI0, NULL};
const Segment * const SG97282[] = {SGI392006, SGI392007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392008, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97283[] = {SGI392009, SGI392010, SGI392011, SGI392012, SGI0, NULL};
const Segment * const SG97284[] = {SGI392013, SGI392014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97285[] = {SGI392016, SGI392017, SGI392018, SGI392019, SGI0, NULL};
const Segment * const SG97286[] = {SGI392020, SGI392021, SGI392022, SGI392023, SGI0, NULL};
const Segment * const SG97287[] = {SGI392024, SGI392025, SGI392026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392027, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97288[] = {SGI392028, SGI392029, SGI0, SGI392030, SGI0, NULL};
const Segment * const SG97289[] = {SGI392031, SGI392032, SGI392033, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392034, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97290[] = {SGI392035, SGI392036, SGI392036, SGI0, SGI0, SGI0, SGI0, SGI392037, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97291[] = {SGI392038, SGI392039, SGI392040, SGI0, SGI0, SGI0, SGI0, SGI392041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97292[] = {SGI392042, SGI392043, SGI392044, SGI0, SGI0, SGI0, SGI0, SGI392045, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97293[] = {SGI392046, SGI392047, SGI392048, SGI392048, SGI392046, NULL};
const Segment * const SG97294[] = {SGI392049, SGI392050, SGI0, SGI392051, SGI0, NULL};
const Segment * const SG97295[] = {SGI392052, SGI392053, SGI392053, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392054, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97296[] = {SGI392055, SGI392056, SGI392056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392057, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97297[] = {SGI392058, SGI392059, SGI392060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392061, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97298[] = {SGI392062, SGI392063, SGI392064, SGI392065, SGI0, NULL};
const Segment * const SG97299[] = {SGI392066, SGI392067, SGI392068, SGI392068, SGI0, NULL};
const Segment * const SG97300[] = {SGI392069, SGI392070, SGI392071, SGI392072, SGI0, NULL};
const Segment * const SG97301[] = {SGI392073, SGI392074, SGI392075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392076, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97302[] = {SGI392077, SGI392078, SGI0, SGI392079, SGI0, SGI0, SGI0, SGI0, SGI392080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97303[] = {SGI392081, SGI0, SGI392082, SGI0, NULL};
const Segment * const SG97304[] = {SGI392083, SGI0, SGI392084, SGI0, NULL};
const Segment * const SG97305[] = {SGI392085, SGI0, SGI392086, SGI0, NULL};
const Segment * const SG97306[] = {SGI392087, SGI0, SGI392088, SGI0, NULL};
const Segment * const SG97307[] = {SGI392089, SGI0, SGI392090, SGI0, NULL};
const Segment * const SG97308[] = {SGI392091, SGI0, SGI392092, SGI0, NULL};
const Segment * const SG97309[] = {SGI392093, SGI0, SGI392094, SGI0, NULL};
const Segment * const SG97310[] = {SGI392095, SGI0, SGI392096, SGI0, NULL};
const Segment * const SG97311[] = {SGI392097, SGI0, SGI392098, SGI0, NULL};
const Segment * const SG97312[] = {SGI392099, SGI0, SGI392100, SGI0, NULL};
const Segment * const SG97313[] = {SGI392101, SGI0, SGI392102, SGI0, NULL};
const Segment * const SG97314[] = {SGI392103, SGI0, SGI392104, SGI0, NULL};
const Segment * const SG97315[] = {SGI392105, SGI0, SGI392106, SGI0, NULL};
const Segment * const SG97316[] = {SGI392107, SGI0, SGI392108, SGI0, NULL};
const Segment * const SG97317[] = {SGI392109, SGI0, SGI392110, SGI0, NULL};
const Segment * const SG97318[] = {SGI392111, SGI0, SGI392112, SGI0, NULL};
const Segment * const SG97319[] = {SGI392113, SGI0, SGI392114, SGI0, NULL};
const Segment * const SG97320[] = {SGI392115, SGI0, SGI392116, SGI0, NULL};
const Segment * const SG97321[] = {SGI392117, SGI0, SGI392118, SGI0, NULL};
const Segment * const SG97322[] = {SGI392119, SGI0, SGI392120, SGI0, NULL};
const Segment * const SG97323[] = {SGI392121, SGI0, SGI392122, SGI0, NULL};
const Segment * const SG97324[] = {SGI392123, SGI0, SGI392124, SGI0, NULL};
const Segment * const SG97325[] = {SGI392125, SGI0, SGI392126, SGI0, NULL};
const Segment * const SG97326[] = {SGI392127, SGI0, SGI392128, SGI0, NULL};
const Segment * const SG97327[] = {SGI392129, SGI0, SGI392130, SGI0, NULL};
const Segment * const SG97328[] = {SGI392131, SGI0, SGI392132, SGI0, NULL};
const Segment * const SG97329[] = {SGI392133, SGI0, SGI392134, SGI0, NULL};
const Segment * const SG97330[] = {SGI392135, SGI0, SGI392136, SGI0, NULL};
const Segment * const SG97331[] = {SGI392137, SGI0, SGI392138, SGI0, NULL};
const Segment * const SG97332[] = {SGI392139, SGI0, SGI392140, SGI0, NULL};
const Segment * const SG97333[] = {SGI392141, SGI0, SGI392142, SGI0, NULL};
const Segment * const SG97334[] = {SGI392143, SGI0, SGI392144, SGI0, NULL};
const Segment * const SG97335[] = {SGI392145, SGI0, SGI392146, SGI0, NULL};
const Segment * const SG97336[] = {SGI392147, SGI0, SGI392148, SGI0, NULL};
const Segment * const SG97337[] = {SGI392149, SGI0, SGI392150, SGI0, NULL};
const Segment * const SG97338[] = {SGI392151, SGI0, SGI392152, SGI0, NULL};
const Segment * const SG97339[] = {SGI392153, SGI0, SGI392154, SGI0, NULL};
const Segment * const SG97340[] = {SGI392155, SGI0, SGI392156, SGI0, NULL};
const Segment * const SG97341[] = {SGI392157, SGI0, SGI392158, SGI0, NULL};
const Segment * const SG97342[] = {SGI392159, SGI0, SGI392160, SGI0, NULL};
const Segment * const SG97343[] = {SGI392161, SGI0, SGI392162, SGI0, NULL};
const Segment * const SG97344[] = {SGI392163, SGI0, SGI392164, SGI0, NULL};
const Segment * const SG97345[] = {SGI392165, SGI0, SGI392166, SGI0, NULL};
const Segment * const SG97346[] = {SGI392167, SGI0, SGI392168, SGI0, NULL};
const Segment * const SG97347[] = {SGI392169, SGI0, SGI392170, SGI0, NULL};
const Segment * const SG97348[] = {SGI392171, SGI0, SGI392172, SGI0, NULL};
const Segment * const SG97349[] = {SGI392173, SGI0, SGI392174, SGI0, NULL};
const Segment * const SG97350[] = {SGI392175, SGI0, SGI392176, SGI0, NULL};
const Segment * const SG97351[] = {SGI392177, SGI392178, SGI392179, SGI392180, NULL};
const Segment * const SG97352[] = {SGI392181, SGI392182, SGI392183, SGI392183, SGI0, NULL};
const Segment * const SG97353[] = {SGI392184, SGI392185, SGI392186, SGI392186, SGI0, NULL};
const Segment * const SG97354[] = {SGI392187, SGI392188, SGI392189, SGI0, NULL};
const Segment * const SG97355[] = {SGI392190, SGI392191, SGI392192, SGI0, NULL};
const Segment * const SG97356[] = {SGI392193, SGI392194, SGI392195, SGI0, NULL};
const Segment * const SG97357[] = {SGI392196, SGI392197, SGI392198, SGI0, NULL};
const Segment * const SG97358[] = {SGI392199, SGI392200, SGI392201, SGI0, NULL};
const Segment * const SG97359[] = {SGI392202, SGI392203, SGI392204, SGI0, NULL};
const Segment * const SG97360[] = {SGI392205, SGI392206, SGI392207, SGI392208, NULL};
const Segment * const SG97361[] = {SGI392209, SGI392210, SGI392211, SGI392212, SGI0, NULL};
const Segment * const SG97362[] = {SGI392213, SGI392214, SGI392215, SGI392216, SGI0, NULL};
const Segment * const SG97363[] = {SGI392217, SGI392218, SGI0, SGI392219, SGI0, NULL};
const Segment * const SG97364[] = {SGI392220, SGI392221, SGI392222, SGI392222, SGI0, NULL};
const Segment * const SG97365[] = {SGI392223, SGI392224, SGI392225, SGI392226, SGI0, NULL};
const Segment * const SG97366[] = {SGI392227, SGI392228, SGI392228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392229, SGI0, NULL};
const Segment * const SG97367[] = {SGI392230, SGI392231, SGI0, SGI392232, SGI0, NULL};
const Segment * const SG97368[] = {SGI392233, SGI392234, SGI392235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392236, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97369[] = {SGI392237, SGI392238, SGI392239, SGI392240, SGI0, NULL};
const Segment * const SG97370[] = {SGI392241, SGI392242, SGI392243, SGI392244, SGI0, NULL};
const Segment * const SG97371[] = {SGI392245, SGI0, SGI392246, SGI0, NULL};
const Segment * const SG97372[] = {SGI392247, SGI392247, SGI392248, SGI392248, SGI0, NULL};
const Segment * const SG97373[] = {SGI392249, SGI392250, SGI392251, SGI0, NULL};
const Segment * const SG97374[] = {SGI392252, SGI392253, SGI392254, SGI0, NULL};
const Segment * const SG97375[] = {SGI392255, SGI392256, SGI392257, SGI0, NULL};
const Segment * const SG97376[] = {SGI392258, SGI392259, SGI392260, SGI0, NULL};
const Segment * const SG97377[] = {SGI392261, SGI392262, SGI392263, SGI0, NULL};
const Segment * const SG97378[] = {SGI392264, SGI392265, SGI392266, SGI0, NULL};
const Segment * const SG97379[] = {SGI392267, SGI392268, SGI392269, SGI392270, SGI0, NULL};
const Segment * const SG97380[] = {SGI392271, SGI392272, SGI392273, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392274, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97381[] = {SGI392275, SGI392276, SGI392277, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392278, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97382[] = {SGI392279, SGI392280, SGI392281, SGI392282, SGI0, NULL};
const Segment * const SG97383[] = {SGI392283, SGI392284, SGI392285, SGI392286, SGI0, NULL};
const Segment * const SG97384[] = {SGI392287, SGI392288, SGI392288, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97385[] = {SGI392289, SGI392290, SGI392291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392292, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97386[] = {SGI392293, SGI392294, SGI392295, SGI392295, SGI0, NULL};
const Segment * const SG97387[] = {SGI392296, SGI392297, SGI392298, SGI392298, SGI0, NULL};
const Segment * const SG97388[] = {SGI392299, SGI392300, SGI392301, SGI392302, SGI0, NULL};
const Segment * const SG97389[] = {SGI392303, SGI392304, SGI392305, SGI392306, SGI0, NULL};
const Segment * const SG97390[] = {SGI392307, SGI392308, SGI0, SGI392309, SGI0, NULL};
const Segment * const SG97391[] = {SGI392310, SGI392311, SGI392311, SGI0, SGI0, SGI0, SGI0, SGI392312, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97392[] = {SGI392313, SGI392314, SGI392315, SGI0, SGI0, SGI0, SGI0, SGI392316, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97393[] = {SGI392317, SGI392318, SGI392319, SGI392320, SGI0, NULL};
const Segment * const SG97394[] = {SGI392321, SGI392322, SGI392323, SGI392324, SGI0, NULL};
const Segment * const SG97395[] = {SGI392325, SGI392326, SGI392326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392327, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97396[] = {SGI392328, SGI392329, SGI0, SGI392330, SGI0, NULL};
const Segment * const SG97397[] = {SGI392331, SGI392332, SGI392333, SGI392333, SGI0, NULL};
const Segment * const SG97398[] = {SGI392334, SGI392335, SGI392336, SGI392337, SGI0, NULL};
const Segment * const SG97399[] = {SGI392338, SGI392339, SGI392340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97400[] = {SGI392342, SGI392343, SGI392344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97401[] = {SGI392346, SGI392347, SGI392348, SGI392348, SGI0, NULL};
const Segment * const SG97402[] = {SGI392349, SGI392350, SGI392351, SGI392352, SGI0, NULL};
const Segment * const SG97403[] = {SGI392353, SGI392354, SGI392355, SGI392356, SGI0, NULL};
const Segment * const SG97404[] = {SGI392357, SGI392358, SGI392359, SGI392360, SGI0, NULL};
const Segment * const SG97405[] = {SGI392361, SGI392362, SGI392363, SGI392363, SGI0, NULL};
const Segment * const SG97406[] = {SGI392364, SGI392365, SGI392366, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392367, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97407[] = {SGI392368, SGI392369, SGI392370, SGI392371, SGI0, NULL};
const Segment * const SG97408[] = {SGI392372, SGI392373, SGI392374, SGI392374, SGI0, NULL};
const Segment * const SG97409[] = {SGI392375, SGI392376, SGI392377, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97410[] = {SGI392379, SGI0, SGI392380, SGI0, NULL};
const Segment * const SG97411[] = {SGI392381, SGI0, SGI392382, SGI0, NULL};
const Segment * const SG97412[] = {SGI392383, SGI0, SGI392384, SGI0, NULL};
const Segment * const SG97413[] = {SGI392385, SGI0, SGI392386, SGI0, NULL};
const Segment * const SG97414[] = {SGI392387, SGI0, SGI392388, SGI0, NULL};
const Segment * const SG97415[] = {SGI392389, SGI0, SGI392390, SGI0, NULL};
const Segment * const SG97416[] = {SGI392391, SGI0, SGI392392, SGI0, NULL};
const Segment * const SG97417[] = {SGI392393, SGI0, SGI392394, SGI0, NULL};
const Segment * const SG97418[] = {SGI392395, SGI0, SGI392396, SGI0, NULL};
const Segment * const SG97419[] = {SGI392397, SGI392398, SGI392399, SGI392400, SGI0, NULL};
const Segment * const SG97420[] = {SGI392401, SGI392402, SGI392403, SGI392403, SGI0, NULL};
const Segment * const SG97421[] = {SGI392404, SGI392405, SGI392406, SGI0, SGI392407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392408, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97422[] = {SGI392409, SGI392410, SGI392410, SGI0, SGI392411, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392412, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97423[] = {SGI392413, SGI392414, SGI392415, SGI0, SGI0, SGI0, SGI0, SGI392416, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97424[] = {SGI392417, SGI392418, SGI392419, SGI392419, SGI0, NULL};
const Segment * const SG97425[] = {SGI392420, SGI392421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392422, NULL};
const Segment * const SG97426[] = {SGI392423, SGI392424, SGI392425, SGI392426, SGI0, NULL};
const Segment * const SG97427[] = {SGI392427, SGI0, SGI392428, SGI0, NULL};
const Segment * const SG97428[] = {SGI392429, SGI0, SGI392430, SGI0, NULL};
const Segment * const SG97429[] = {SGI392431, SGI392432, SGI392432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392433, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97430[] = {SGI392434, SGI392435, SGI392436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392437, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97431[] = {SGI392438, SGI392439, SGI392440, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392441, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97432[] = {SGI392442, SGI392443, SGI392444, SGI0, NULL};
const Segment * const SG97433[] = {SGI392445, SGI392446, SGI392447, SGI0, NULL};
const Segment * const SG97434[] = {SGI392448, SGI392449, SGI392450, SGI0, NULL};
const Segment * const SG97435[] = {SGI392451, SGI392452, SGI392453, SGI0, NULL};
const Segment * const SG97436[] = {SGI392454, SGI392455, SGI392456, SGI0, NULL};
const Segment * const SG97437[] = {SGI392457, SGI392458, SGI392459, SGI0, NULL};
const Segment * const SG97438[] = {SGI392460, SGI392461, SGI392462, SGI392463, SGI0, NULL};
const Segment * const SG97439[] = {SGI392464, SGI392465, SGI392466, SGI392467, SGI0, NULL};
const Segment * const SG97440[] = {SGI392468, SGI392469, SGI0, SGI392470, SGI0, NULL};
const Segment * const SG97441[] = {SGI392471, SGI392472, SGI392473, SGI392473, SGI0, NULL};
const Segment * const SG97442[] = {SGI392474, SGI392475, SGI392476, SGI392476, SGI0, NULL};
const Segment * const SG97443[] = {SGI392477, SGI392478, SGI392479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392480, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97444[] = {SGI392481, SGI392482, SGI392483, SGI392484, SGI0, NULL};
const Segment * const SG97445[] = {SGI392485, SGI392486, SGI392487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392488, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97446[] = {SGI392489, SGI392490, SGI392491, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97447[] = {SGI392492, SGI392493, SGI392493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392494, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97448[] = {SGI392495, SGI392496, SGI392497, SGI392498, NULL};
const Segment * const SG97449[] = {SGI392499, SGI392500, SGI392501, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97450[] = {SGI392503, SGI392504, SGI392505, SGI392506, SGI0, NULL};
const Segment * const SG97451[] = {SGI392507, SGI392508, SGI392509, SGI392510, SGI0, NULL};
const Segment * const SG97452[] = {SGI392511, SGI392512, SGI392513, SGI392514, SGI0, NULL};
const Segment * const SG97453[] = {SGI392515, SGI392516, SGI392517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97454[] = {SGI392519, SGI392520, SGI392521, SGI0, SGI0, SGI0, SGI0, SGI392522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97455[] = {SGI392523, SGI392524, SGI392525, SGI392526, SGI0, NULL};
const Segment * const SG97456[] = {SGI392527, SGI392528, SGI392529, SGI392530, SGI0, NULL};
const Segment * const SG97457[] = {SGI392531, SGI392532, SGI392533, SGI392533, SGI0, NULL};
const Segment * const SG97458[] = {SGI392534, SGI0, SGI392535, SGI0, NULL};
const Segment * const SG97459[] = {SGI392536, SGI392537, SGI392538, SGI392538, SGI0, NULL};
const Segment * const SG97460[] = {SGI392539, SGI392540, SGI392541, SGI392542, SGI0, NULL};
const Segment * const SG97461[] = {SGI392543, SGI392544, SGI392545, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392546, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97462[] = {SGI392547, SGI392548, SGI392549, SGI0, NULL};
const Segment * const SG97463[] = {SGI392550, SGI392551, SGI392552, SGI0, NULL};
const Segment * const SG97464[] = {SGI392553, SGI392554, SGI392555, SGI0, NULL};
const Segment * const SG97465[] = {SGI392556, SGI392557, SGI392558, SGI0, NULL};
const Segment * const SG97466[] = {SGI392559, SGI392560, SGI392561, SGI0, NULL};
const Segment * const SG97467[] = {SGI392562, SGI392563, SGI392564, SGI0, NULL};
const Segment * const SG97468[] = {SGI392565, SGI392566, SGI392567, SGI392568, SGI0, NULL};
const Segment * const SG97469[] = {SGI392569, SGI392570, SGI392571, SGI392572, SGI0, NULL};
const Segment * const SG97470[] = {SGI392573, SGI392574, SGI392575, SGI392576, SGI0, NULL};
const Segment * const SG97471[] = {SGI392577, SGI392578, SGI392579, SGI392580, SGI0, NULL};
const Segment * const SG97472[] = {SGI392581, SGI392582, SGI392583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392584, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97473[] = {SGI392585, SGI0, SGI392586, SGI0, NULL};
const Segment * const SG97474[] = {SGI392587, SGI392588, SGI392589, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392590, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97475[] = {SGI392591, SGI392592, SGI392593, SGI392593, SGI0, NULL};
const Segment * const SG97476[] = {SGI392594, SGI392595, SGI392595, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392596, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97477[] = {SGI392597, SGI392598, SGI392599, SGI0, NULL};
const Segment * const SG97478[] = {SGI392600, SGI392601, SGI392602, SGI0, NULL};
const Segment * const SG97479[] = {SGI392603, SGI392604, SGI392605, SGI0, NULL};
const Segment * const SG97480[] = {SGI392606, SGI392607, SGI392608, SGI0, NULL};
const Segment * const SG97481[] = {SGI392609, SGI392610, SGI392611, SGI0, NULL};
const Segment * const SG97482[] = {SGI392612, SGI392613, SGI392614, SGI0, NULL};
const Segment * const SG97483[] = {SGI392615, SGI392616, SGI392617, SGI392618, SGI0, NULL};
const Segment * const SG97484[] = {SGI392619, SGI392620, SGI392621, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392622, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97485[] = {SGI392623, SGI392624, SGI392625, SGI392626, SGI0, NULL};
const Segment * const SG97486[] = {SGI392627, SGI392628, SGI392629, SGI0, NULL};
const Segment * const SG97487[] = {SGI392630, SGI392631, SGI392632, SGI0, NULL};
const Segment * const SG97488[] = {SGI392633, SGI392634, SGI392635, SGI0, NULL};
const Segment * const SG97489[] = {SGI392636, SGI392637, SGI392638, SGI0, NULL};
const Segment * const SG97490[] = {SGI392639, SGI392640, SGI392641, SGI0, NULL};
const Segment * const SG97491[] = {SGI392642, SGI392643, SGI392644, SGI392645, SGI0, NULL};
const Segment * const SG97492[] = {SGI392646, SGI392647, SGI0, SGI392648, SGI0, NULL};
const Segment * const SG97493[] = {SGI392649, SGI392650, SGI392651, SGI392651, SGI0, NULL};
const Segment * const SG97494[] = {SGI392652, SGI392653, SGI392653, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392654, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97495[] = {SGI392655, SGI392656, SGI392657, SGI392658, SGI0, NULL};
const Segment * const SG97496[] = {SGI392659, SGI392660, SGI392661, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392662, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97497[] = {SGI392663, SGI0, SGI392664, SGI0, NULL};
const Segment * const SG97498[] = {SGI392665, SGI0, SGI392667, SGI0, NULL};
const Segment * const SG97499[] = {SGI392666, SGI392668, SGI392669, SGI392670, SGI0, NULL};
const Segment * const SG97500[] = {SGI392671, SGI0, SGI392672, SGI0, NULL};
const Segment * const SG97501[] = {SGI392673, SGI0, SGI392674, SGI0, NULL};
const Segment * const SG97502[] = {SGI392675, SGI0, SGI392676, SGI0, NULL};
const Segment * const SG97503[] = {SGI392677, SGI0, SGI392678, SGI0, NULL};
const Segment * const SG97504[] = {SGI392679, SGI0, SGI392680, SGI0, NULL};
const Segment * const SG97505[] = {SGI392681, SGI392682, SGI392683, SGI0, SGI392684, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392685, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97506[] = {SGI392686, SGI0, SGI392687, SGI0, NULL};
const Segment * const SG97507[] = {SGI392688, SGI392689, SGI392690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392691, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97508[] = {SGI392692, SGI392693, SGI392694, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392695, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97509[] = {SGI392696, SGI392697, SGI392697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392698, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97510[] = {SGI392699, SGI392700, SGI392701, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97511[] = {SGI392703, SGI392704, SGI392705, SGI392705, SGI0, NULL};
const Segment * const SG97512[] = {SGI392706, SGI392707, SGI392708, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392709, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97513[] = {SGI392710, SGI392711, SGI392712, SGI392713, SGI0, NULL};
const Segment * const SG97514[] = {SGI392714, SGI392715, SGI392715, SGI0, SGI0, SGI0, SGI392716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97515[] = {SGI392717, SGI392718, SGI392719, SGI392720, SGI0, NULL};
const Segment * const SG97516[] = {SGI392721, SGI392722, SGI392723, SGI0, SGI0, SGI0, SGI392724, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97517[] = {SGI392725, SGI392726, SGI392727, SGI392728, SGI0, NULL};
const Segment * const SG97518[] = {SGI392729, SGI392730, SGI392731, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392732, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97519[] = {SGI392733, SGI392734, SGI392734, SGI0, SGI0, SGI0, SGI0, SGI392735, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97520[] = {SGI392736, SGI392737, SGI392738, SGI392738, SGI0, NULL};
const Segment * const SG97521[] = {SGI392739, SGI392740, SGI392741, SGI392741, SGI0, NULL};
const Segment * const SG97522[] = {SGI392742, SGI392743, SGI392744, SGI392745, SGI0, NULL};
const Segment * const SG97523[] = {SGI392746, SGI392747, SGI392748, SGI0, SGI0, SGI0, SGI392749, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97524[] = {SGI392750, SGI392751, SGI392752, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392753, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97525[] = {SGI392754, SGI392755, SGI392756, SGI392757, SGI0, NULL};
const Segment * const SG97526[] = {SGI392758, SGI392759, SGI392760, SGI392761, SGI0, NULL};
const Segment * const SG97527[] = {SGI392762, SGI392763, SGI392764, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392765, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97528[] = {SGI392766, SGI392767, SGI392768, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392769, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97529[] = {SGI392770, SGI392771, SGI0, SGI392772, SGI0, NULL};
const Segment * const SG97530[] = {SGI392773, SGI392774, SGI392775, SGI392775, SGI0, NULL};
const Segment * const SG97531[] = {SGI392776, SGI392777, SGI392778, SGI0, SGI0, SGI0, SGI0, SGI392779, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392780, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97532[] = {SGI392781, SGI392782, SGI392783, SGI0, SGI0, SGI0, SGI392784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97533[] = {SGI392785, SGI392786, SGI392787, SGI392787, SGI0, NULL};
const Segment * const SG97534[] = {SGI392788, SGI392789, SGI392790, SGI392790, SGI0, NULL};
const Segment * const SG97535[] = {SGI392791, SGI392792, SGI392793, SGI0, SGI392794, SGI0, SGI0, SGI392795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97536[] = {SGI392796, SGI0, SGI392797, SGI0, NULL};
const Segment * const SG97537[] = {SGI392798, SGI0, SGI392799, SGI0, NULL};
const Segment * const SG97538[] = {SGI392800, SGI392801, SGI392802, SGI392803, SGI0, NULL};
const Segment * const SG97539[] = {SGI392804, SGI392805, SGI392806, SGI392807, SGI0, NULL};
const Segment * const SG97540[] = {SGI392808, SGI392809, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97541[] = {SGI392811, SGI392812, SGI392813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392814, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97542[] = {SGI392815, SGI0, SGI392816, SGI0, NULL};
const Segment * const SG97543[] = {SGI392817, SGI0, SGI392818, SGI0, NULL};
const Segment * const SG97544[] = {SGI392819, SGI0, SGI392820, SGI0, NULL};
const Segment * const SG97545[] = {SGI392821, SGI392822, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392823, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97546[] = {SGI392824, SGI392825, SGI392826, SGI392826, SGI0, NULL};
const Segment * const SG97547[] = {SGI392827, SGI392828, SGI392829, SGI392829, SGI0, NULL};
const Segment * const SG97548[] = {SGI392830, SGI392830, SGI392831, SGI392831, SGI0, NULL};
const Segment * const SG97549[] = {SGI392832, SGI392833, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97550[] = {SGI392835, SGI0, SGI392836, SGI0, NULL};
const Segment * const SG97551[] = {SGI392837, SGI392838, SGI392839, SGI392840, SGI0, NULL};
const Segment * const SG97552[] = {SGI392841, SGI392842, SGI392843, SGI392843, SGI0, NULL};
const Segment * const SG97553[] = {SGI392844, SGI392845, SGI392846, SGI392846, SGI0, NULL};
const Segment * const SG97554[] = {SGI392847, SGI392848, SGI392849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97555[] = {SGI392851, SGI0, SGI392852, SGI0, NULL};
const Segment * const SG97556[] = {SGI392853, SGI392854, SGI392855, SGI0, SGI0, SGI0, SGI392856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97557[] = {SGI392857, SGI392858, SGI392859, SGI392860, NULL};
const Segment * const SG97558[] = {SGI392861, SGI392862, SGI0, SGI392863, SGI0, NULL};
const Segment * const SG97559[] = {SGI392864, SGI392865, SGI392866, SGI392866, SGI0, NULL};
const Segment * const SG97560[] = {SGI392867, SGI392868, SGI392869, SGI392870, SGI0, NULL};
const Segment * const SG97561[] = {SGI392871, SGI392872, SGI392873, SGI392873, SGI0, NULL};
const Segment * const SG97562[] = {SGI392874, SGI392875, SGI392876, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97563[] = {SGI392878, SGI392879, SGI392880, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392881, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97564[] = {SGI392882, SGI392883, SGI392884, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97565[] = {SGI392885, SGI392886, SGI392886, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97566[] = {SGI392887, SGI392888, SGI392889, SGI392890, SGI0, NULL};
const Segment * const SG97567[] = {SGI392891, SGI392892, SGI392893, SGI392894, SGI0, NULL};
const Segment * const SG97568[] = {SGI392895, SGI392896, SGI392897, SGI392898, SGI0, NULL};
const Segment * const SG97569[] = {SGI392899, SGI392900, SGI392901, SGI392902, SGI0, NULL};
const Segment * const SG97570[] = {SGI392903, SGI392904, SGI0, SGI392905, SGI0, NULL};
const Segment * const SG97571[] = {SGI392906, SGI392907, SGI392908, SGI392908, SGI0, NULL};
const Segment * const SG97572[] = {SGI392909, SGI392910, SGI392911, SGI392911, SGI0, NULL};
const Segment * const SG97573[] = {SGI392912, SGI392913, SGI392914, SGI392914, SGI0, NULL};
const Segment * const SG97574[] = {SGI392915, SGI392916, SGI392917, SGI0, NULL};
const Segment * const SG97575[] = {SGI392918, SGI392919, SGI392920, SGI0, NULL};
const Segment * const SG97576[] = {SGI392921, SGI392922, SGI392923, SGI0, NULL};
const Segment * const SG97577[] = {SGI392924, SGI392925, SGI392926, SGI0, NULL};
const Segment * const SG97578[] = {SGI392927, SGI392928, SGI392929, SGI0, NULL};
const Segment * const SG97579[] = {SGI392930, SGI392931, SGI392932, SGI0, NULL};
const Segment * const SG97580[] = {SGI392933, SGI392934, SGI392935, SGI0, NULL};
const Segment * const SG97581[] = {SGI392936, SGI392937, SGI392938, SGI0, NULL};
const Segment * const SG97582[] = {SGI392939, SGI392940, SGI392941, SGI0, NULL};
const Segment * const SG97583[] = {SGI392942, SGI392943, SGI392944, SGI0, NULL};
const Segment * const SG97584[] = {SGI392945, SGI392946, SGI392947, SGI0, NULL};
const Segment * const SG97585[] = {SGI392948, SGI392949, SGI392950, SGI0, NULL};
const Segment * const SG97586[] = {SGI392951, SGI392952, SGI392953, SGI0, NULL};
const Segment * const SG97587[] = {SGI392954, SGI392955, SGI392956, SGI0, NULL};
const Segment * const SG97588[] = {SGI392957, SGI392958, SGI392959, SGI0, NULL};
const Segment * const SG97589[] = {SGI392960, SGI392961, SGI392962, SGI0, NULL};
const Segment * const SG97590[] = {SGI392963, SGI392964, SGI392965, SGI0, NULL};
const Segment * const SG97591[] = {SGI392966, SGI392967, SGI392968, SGI0, NULL};
const Segment * const SG97592[] = {SGI392969, SGI392970, SGI392971, SGI0, NULL};
const Segment * const SG97593[] = {SGI392972, SGI392973, SGI392974, SGI0, NULL};
const Segment * const SG97594[] = {SGI392975, SGI392976, SGI392977, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97595[] = {SGI392978, SGI392979, SGI392980, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392981, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97596[] = {SGI392982, SGI392983, SGI392984, SGI392985, SGI0, NULL};
const Segment * const SG97597[] = {SGI392986, SGI392987, SGI392988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI392989, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97598[] = {SGI392990, SGI392991, SGI392992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97599[] = {SGI392993, SGI392994, SGI392995, SGI392996, SGI0, NULL};
const Segment * const SG97600[] = {SGI392997, SGI392998, SGI392999, SGI0, SGI0, NULL};
const Segment * const SG97601[] = {SGI393000, SGI393001, SGI393002, SGI393003, SGI0, NULL};
const Segment * const SG97602[] = {SGI393004, SGI393005, SGI393006, SGI0, NULL};
const Segment * const SG97603[] = {SGI393007, SGI393008, SGI393009, SGI0, NULL};
const Segment * const SG97604[] = {SGI393010, SGI393011, SGI393012, SGI0, NULL};
const Segment * const SG97605[] = {SGI393013, SGI393014, SGI393015, SGI0, NULL};
const Segment * const SG97606[] = {SGI393016, SGI393017, SGI393018, SGI0, NULL};
const Segment * const SG97607[] = {SGI393019, SGI393020, SGI393021, SGI0, SGI0, SGI0, SGI393022, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97608[] = {SGI393023, SGI393024, SGI393025, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI393026, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97609[] = {SGI393027, SGI393028, SGI0, SGI393029, SGI0, SGI0, SGI0, SGI0, SGI393030, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97610[] = {SGI393031, SGI393032, SGI393033, SGI0, NULL};
const Segment * const SG97611[] = {SGI393034, SGI393035, SGI393036, SGI0, NULL};
const Segment * const SG97612[] = {SGI393037, SGI393038, SGI393039, SGI0, NULL};
const Segment * const SG97613[] = {SGI393040, SGI393041, SGI393042, SGI0, NULL};
const Segment * const SG97614[] = {SGI393043, SGI393044, SGI393045, SGI0, NULL};
const Segment * const SG97615[] = {SGI393046, SGI393047, SGI393048, SGI0, NULL};
const Segment * const SG97616[] = {SGI393049, SGI393050, SGI393051, SGI393051, SGI0, NULL};
const Segment * const SG97617[] = {SGI393052, SGI393053, SGI393054, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI393055, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97618[] = {SGI393056, SGI393057, SGI393058, SGI393059, SGI0, NULL};
const Segment * const SG97619[] = {SGI393060, SGI393061, SGI393062, SGI0, SGI0, SGI0, SGI393063, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97620[] = {SGI393064, SGI393065, SGI393066, SGI393066, SGI0, NULL};
const Segment * const SG97621[] = {SGI393067, SGI393068, SGI393069, SGI393069, SGI0, NULL};
const Segment * const SG97622[] = {SGI393070, SGI393071, SGI0, SGI393072, SGI0, NULL};
const Segment * const SG97623[] = {SGI393073, SGI393074, SGI393075, SGI0, NULL};
const Segment * const SG97624[] = {SGI393076, SGI393077, SGI393078, SGI0, NULL};
const Segment * const SG97625[] = {SGI393079, SGI393080, SGI393081, SGI0, NULL};
const Segment * const SG97626[] = {SGI393082, SGI393083, SGI393084, SGI0, NULL};
const Segment * const SG97627[] = {SGI393085, SGI393086, SGI393087, SGI0, NULL};
const Segment * const SG97628[] = {SGI393088, SGI393089, SGI393090, SGI0, NULL};
const Segment * const SG97629[] = {SGI393091, SGI393092, SGI393093, SGI0, NULL};
const Segment * const SG97630[] = {SGI393094, SGI393095, SGI393096, SGI0, NULL};
const Segment * const SG97631[] = {SGI393097, SGI393098, SGI393099, SGI0, NULL};
const Segment * const SG97632[] = {SGI393100, SGI393101, SGI393102, SGI0, NULL};
const Segment * const SG97633[] = {SGI393103, SGI393104, SGI393105, SGI0, NULL};
const Segment * const SG97634[] = {SGI393106, SGI393107, SGI393108, SGI0, NULL};
const Segment * const SG97635[] = {SGI393109, SGI393110, SGI393111, SGI0, NULL};
const Segment * const SG97636[] = {SGI393112, SGI393113, SGI393114, SGI0, NULL};
const Segment * const SG97637[] = {SGI393115, SGI393116, SGI393117, SGI0, NULL};
const Segment * const SG97638[] = {SGI393118, SGI393119, SGI393120, SGI0, NULL};
const Segment * const SG97639[] = {SGI393121, SGI393122, SGI393123, SGI0, NULL};
const Segment * const SG97640[] = {SGI393124, SGI393125, SGI393126, SGI0, NULL};
const Segment * const SG97641[] = {SGI393127, SGI393128, SGI393129, SGI0, NULL};
const Segment * const SG97642[] = {SGI393130, SGI393131, SGI393132, SGI0, NULL};
const Segment * const SG97643[] = {SGI393133, SGI393134, SGI393135, SGI0, NULL};
const Segment * const SG97644[] = {SGI393136, SGI393137, SGI393138, SGI0, NULL};
const Segment * const SG97645[] = {SGI393139, SGI393140, SGI393141, SGI0, NULL};
const Segment * const SG97646[] = {SGI393142, SGI393143, SGI393144, SGI0, NULL};
const Segment * const SG97647[] = {SGI393145, SGI393146, SGI393147, SGI0, NULL};
const Segment * const SG97648[] = {SGI393148, SGI393149, SGI393150, SGI0, NULL};
const Segment * const SG97649[] = {SGI393151, SGI393152, SGI393153, SGI0, NULL};
const Segment * const SG97650[] = {SGI393154, SGI393155, SGI393156, SGI0, NULL};
const Segment * const SG97651[] = {SGI393157, SGI393158, SGI393159, SGI0, NULL};
const Segment * const SG97652[] = {SGI393160, SGI0, SGI393161, SGI0, NULL};
const Segment * const SG97653[] = {SGI393162, SGI393163, SGI393164, SGI393165, SGI0, NULL};
const Segment * const SG97654[] = {SGI393166, SGI393167, SGI393168, SGI393169, SGI0, NULL};
const Segment * const SG97655[] = {SGI393170, SGI393171, SGI393172, SGI0, SGI0, SGI0, SGI393173, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97656[] = {SGI393174, SGI393175, SGI393176, SGI393177, SGI0, NULL};
const Segment * const SG97657[] = {SGI393178, SGI393179, SGI393180, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI393181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97658[] = {SGI393182, SGI0, SGI393183, SGI0, NULL};
const Segment * const SG97659[] = {SGI393184, SGI393185, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI393186, NULL};
const Segment * const SG97660[] = {SGI393187, SGI393188, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI393189, NULL};
const Segment * const SG97661[] = {SGI393190, SGI393191, SGI393192, SGI393192, SGI0, NULL};
const Segment * const SG97662[] = {SGI393193, SGI393194, SGI393195, SGI393196, SGI0, NULL};
const Segment * const SG97663[] = {SGI393197, SGI393198, SGI393199, SGI393199, SGI0, NULL};
const Segment * const SG97664[] = {SGI393200, SGI0, SGI393201, SGI0, NULL};
const Segment * const SG97665[] = {SGI393202, SGI393203, SGI393204, SGI0, NULL};
const Segment * const SG97666[] = {SGI393205, SGI393206, SGI393207, SGI0, NULL};
const Segment * const SG97667[] = {SGI393208, SGI393209, SGI393210, SGI0, NULL};
const Segment * const SG97668[] = {SGI393211, SGI393212, SGI393213, SGI0, NULL};
const Segment * const SG97669[] = {SGI393214, SGI393215, SGI393216, SGI393216, SGI0, NULL};
const Segment * const SG97670[] = {SGI393217, SGI393218, SGI393219, SGI393219, SGI0, NULL};
const Segment * const SG97671[] = {SGI393220, SGI393221, SGI393222, SGI393223, SGI0, NULL};
const Segment * const SG97672[] = {SGI393224, SGI393225, SGI393226, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI393227, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97673[] = {SGI393228, SGI393229, SGI393230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI393231, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97674[] = {SGI393232, SGI393233, SGI393234, SGI393234, SGI0, NULL};
const Segment * const SG97675[] = {SGI393235, SGI393236, SGI393237, SGI393238, SGI0, NULL};
const Segment * const SG97676[] = {SGI393239, SGI0, SGI393240, SGI0, NULL};
const Segment * const SG97677[] = {SGI393241, SGI393242, SGI393243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI393244, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97678[] = {SGI393245, SGI393246, SGI393247, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97679[] = {SGI393248, SGI393249, SGI393249, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI393250, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97680[] = {SGI393251, SGI393252, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI393253, NULL};
const Segment * const SG97681[] = {SGI393254, SGI393255, SGI393256, SGI0, SGI0, SGI0, SGI0, SGI0, SGI393257, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG97682[] = {SGI393258, SGI393258, SGI393259, SGI393260, SGI0, NULL};
const Segment * const SG97683[] = {SGI393261, SGI393262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI393263, NULL};
const Segment * const SG97684[] = {SGI393264, SGI393265, SGI393266, SGI393267, SGI0, NULL};
const Segment * const SG97685[] = {SGI393268, SGI0, SGI393269, SGI0, NULL};
const Segment * const SG97686[] = {SGI393270, SGI393271, SGI393272, SGI393272, SGI0, NULL};
const Segment * const SG97687[] = {SGI393273, SGI393274, SGI393275, SGI393275, SGI0, NULL};
const Segment * const SG97688[] = {SGI393276, SGI393277, SGI393278, SGI393279, SGI0, NULL};
const Segment * const SG97689[] = {SGI393280, SGI393281, SGI393282, SGI393283, SGI0, NULL};
const Segment * const SG97690[] = {SGI393284, SGI393285, SGI393286, SGI393286, SGI393284, NULL};
const Segment * const SG97691[] = {SGI393287, SGI393288, SGI393289, SGI393290, SGI0, NULL};
const Segment * const SG97692[] = {SGI393291, SGI393292, SGI393293, SGI393294, SGI0, NULL};
const Segment * const SG97693[] = {SGI393295, SGI393296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI393297, NULL};
const Segment * const SG97694[] = {SGI393298, SGI393299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI393300, NULL};
const Segment * const SG97695[] = {SGI393301, SGI393302, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI393303, NULL};
const Segment * const SG97696[] = {SGI393304, SGI393305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI393306, NULL};
const Segment * const SG97697[] = {SGI393307, SGI393308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI393309, NULL};
const Segment * const SG97698[] = {SGI393310, SGI393311, SGI393312, SGI393313, SGI0, NULL};
const Segment * const SG97699[] = {SGI393314, SGI393314, SGI393315, SGI393315, SGI0, NULL};
const Segment * const SG97700[] = {SGI393316, SGI0, SGI393317, SGI0, NULL};
const Segment * const SG97701[] = {SGI393318, SGI393319, SGI393320, SGI393321, SGI0, NULL};
const Segment * const SG97702[] = {SGI393322, SGI393323, SGI393324, SGI0, NULL};
const Segment * const SG97703[] = {SGI393325, SGI393326, SGI393327, SGI0, NULL};
const Segment * const SG97704[] = {SGI393328, SGI393329, SGI393330, SGI0, NULL};
const Segment * const SG97705[] = {SGI393331, SGI393332, SGI393333, SGI0, NULL};
const Segment * const SG97706[] = {SGI393334, SGI393335, SGI393336, SGI393337, SGI0, NULL};
const Segment * const SG97707[] = {SGI393338, SGI393339, SGI393340, SGI393340, SGI0, NULL};
const Segment * const SG97708[] = {SGI393341, SGI0, SGI393342, SGI0, NULL};
const Segment * const SG97709[] = {SGI393343, SGI0, SGI393344, SGI0, NULL};
const Segment * const SG97710[] = {SGI393345, SGI0, SGI393346, SGI0, NULL};
const Segment * const SG97711[] = {SGI393347, SGI0, SGI393348, SGI0, NULL};
const Segment * const SG97712[] = {SGI393349, SGI0, SGI393350, SGI0, NULL};
const Segment * const SG97713[] = {SGI393351, SGI0, SGI393352, SGI0, NULL};
const Segment * const SG97714[] = {SGI393353, SGI0, SGI393354, SGI0, NULL};
const Segment * const SG97715[] = {SGI393355, SGI0, SGI393356, SGI0, NULL};
const Segment * const SG97716[] = {SGI393357, SGI393358, SGI393359, SGI0, NULL};
const Segment * const SG97717[] = {SGI393360, SGI393361, SGI393362, SGI0, NULL};
const Segment * const SG97718[] = {SGI393363, SGI393364, SGI393365, SGI0, NULL};
const Segment * const SG97719[] = {SGI393366, SGI393367, SGI393368, SGI0, NULL};
const Segment * const SG97720[] = {SGI393369, SGI393370, SGI393371, SGI0, NULL};
const Segment * const SG97721[] = {SGI393372, SGI393372, SGI393373, SGI393374, SGI0, NULL};
const Segment * const SG97722[] = {SGI393375, SGI393375, SGI393376, SGI393377, SGI0, NULL};
const Segment * const SG97723[] = {SGI393378, SGI393379, SGI393380, SGI393381, SGI0, NULL};
const Segment * const SG97724[] = {SGI393382, SGI393383, SGI393384, SGI393385, SGI0, NULL};
const Segment * const SG97725[] = {SGI393386, SGI393387, SGI393388, SGI0, NULL};
const Segment * const SG97726[] = {SGI393389, SGI393390, SGI393391, SGI0, NULL};
const Segment * const SG97727[] = {SGI393392, SGI393393, SGI393394, SGI393394, SGI0, NULL};
const Segment * const SG97728[] = {SGI393395, SGI393395, SGI393396, SGI393397, SGI0, NULL};
const Segment * const SG97729[] = {SGI393398, SGI393399, SGI393400, SGI393400, SGI0, NULL};
const Segment * const SG97730[] = {SGI393401, SGI393402, SGI393403, SGI393404, SGI0, NULL};
const Segment * const SG97731[] = {SGI393405, SGI393405, SGI393406, SGI393406, SGI0, NULL};
const Segment * const SG97732[] = {SGI393407, SGI393408, SGI393409, SGI393410, SGI0, NULL};
const Segment * const SG97733[] = {SGI393411, SGI393412, SGI393413, SGI0, NULL};
const Segment * const SG97734[] = {SGI393414, SGI393415, SGI393416, SGI0, NULL};
const Segment * const SG97735[] = {SGI393417, SGI393418, SGI393419, SGI0, NULL};
const Segment * const SG97736[] = {SGI393420, SGI393421, SGI393422, SGI0, NULL};
const Segment * const SG97737[] = {SGI393423, SGI393424, SGI393425, SGI0, NULL};
const Segment * const SG97738[] = {SGI393426, SGI393427, SGI393428, SGI0, NULL};
const Segment * const SG97739[] = {SGI393429, SGI0, SGI393430, SGI0, NULL};
const Segment * const SG97740[] = {SGI393431, SGI393432, SGI393433, SGI0, NULL};
const Segment * const SG97741[] = {SGI393434, SGI393435, SGI393436, SGI0, NULL};
const Segment * const SG97742[] = {SGI393437, SGI393438, SGI393439, SGI0, NULL};
const Segment * const SG97743[] = {SGI393440, SGI393441, SGI393442, SGI0, NULL};
const Segment * const SG97744[] = {SGI393443, SGI393444, SGI393445, SGI393446, SGI0, NULL};
const Segment * const SG97745[] = {SGI393447, SGI0, NULL};
const Segment * const SG97746[] = {SGI393448, SGI0, NULL};
const Segment * const SG97747[] = {SGI393449, SGI0, NULL};
const Segment * const SG97748[] = {SGI393450, SGI0, NULL};
const Segment * const SG97749[] = {SGI393451, SGI0, NULL};
const Segment * const SG97750[] = {SGI393452, SGI0, NULL};
const Segment * const SG97751[] = {SGI393453, SGI0, NULL};
const Segment * const SG97752[] = {SGI393454, SGI0, NULL};
const Segment * const SG97753[] = {SGI393455, SGI0, NULL};
const Segment * const SG97754[] = {SGI393456, SGI0, NULL};
const Segment * const SG97755[] = {SGI393457, SGI0, NULL};
const Segment * const SG97756[] = {SGI393458, SGI0, NULL};
const Segment * const SG97757[] = {SGI393459, SGI0, NULL};
const Segment * const SG97758[] = {SGI393460, SGI0, NULL};
const Segment * const SG97759[] = {SGI393461, SGI0, NULL};
const Segment * const SG97760[] = {SGI393462, SGI0, NULL};
const Segment * const SG97761[] = {SGI393463, SGI0, NULL};
const Segment * const SG97762[] = {SGI393464, SGI0, NULL};
const Segment * const SG97763[] = {SGI393465, SGI0, NULL};
const Segment * const SG97764[] = {SGI393466, SGI0, NULL};
const Segment * const SG97765[] = {SGI393467, SGI0, NULL};
const Segment * const SG97766[] = {SGI393468, SGI0, SGI393469, SGI0, NULL};
const Segment * const SG97767[] = {SGI393470, SGI0, SGI393471, SGI0, NULL};
const Segment * const SG97768[] = {SGI393472, SGI393473, SGI393474, SGI393475, SGI0, NULL};
const Segment * const SG97769[] = {SGI393476, SGI0, SGI393477, SGI0, NULL};
const Segment * const SG97770[] = {SGI393478, SGI393479, SGI393480, SGI393481, SGI0, NULL};
const Segment * const SG97771[] = {SGI393482, SGI0, SGI393483, SGI0, NULL};
const Segment * const SG97772[] = {SGI393484, SGI0, SGI393485, SGI0, NULL};
const Segment * const SG97773[] = {SGI393486, SGI393487, SGI393488, SGI393489, SGI0, NULL};
const Segment * const SG97774[] = {SGI393490, SGI0, SGI393491, SGI0, NULL};
const Segment * const SG97775[] = {SGI393492, SGI0, SGI393493, SGI0, NULL};
const Segment * const SG97776[] = {SGI393494, SGI0, SGI393495, SGI0, NULL};
const Segment * const SG97777[] = {SGI393496, SGI0, SGI393497, SGI0, NULL};
const Segment * const SG97778[] = {SGI393498, SGI0, SGI393499, SGI0, NULL};
const Segment * const SG97779[] = {SGI393500, SGI0, SGI393501, SGI0, NULL};
const Segment * const SG97780[] = {SGI393502, SGI0, SGI393503, SGI0, NULL};
const Segment * const SG97781[] = {SGI393504, SGI0, SGI393505, SGI0, NULL};
const Segment * const SG97782[] = {SGI393506, SGI0, SGI393507, SGI0, NULL};
const Segment * const SG97783[] = {SGI393508, SGI0, SGI393509, SGI0, NULL};
const Segment * const SG97784[] = {SGI393510, SGI0, SGI393511, SGI0, NULL};
const Segment * const SG97785[] = {SGI393512, SGI0, SGI393513, SGI0, NULL};
const Segment * const SG97786[] = {SGI393514, SGI0, SGI393515, SGI0, NULL};
const Segment * const SG97787[] = {SGI393516, SGI0, SGI393517, SGI0, NULL};
const Segment * const SG97788[] = {SGI393518, SGI0, SGI393519, SGI0, NULL};
const Segment * const SG97789[] = {SGI393520, SGI0, SGI393521, SGI0, NULL};
const Segment * const SG97790[] = {SGI393522, SGI0, SGI393523, SGI0, NULL};
const Segment * const SG97791[] = {SGI393524, SGI0, SGI393525, SGI0, NULL};
const Segment * const SG97792[] = {SGI393526, SGI0, SGI393527, SGI0, NULL};
const Segment * const SG97793[] = {SGI393528, SGI0, SGI393529, SGI0, NULL};
const Segment * const SG97794[] = {SGI393530, SGI393531, SGI393532, SGI0, NULL};
const Segment * const SG97795[] = {SGI393533, SGI393534, SGI393535, SGI0, NULL};
const Segment * const SG97796[] = {SGI393536, SGI393537, SGI393538, SGI0, NULL};
const Segment * const SG97797[] = {SGI393539, SGI0, SGI393540, SGI0, NULL};
const Segment * const SG97798[] = {SGI393541, SGI0, SGI393542, SGI0, NULL};
const Segment * const SG97799[] = {SGI393543, SGI0, SGI393544, SGI0, NULL};
const Segment * const SG97800[] = {SGI393545, SGI0, SGI393546, SGI0, NULL};
const Segment * const SG97801[] = {SGI393547, SGI0, SGI393548, SGI0, NULL};
const Segment * const SG97802[] = {SGI393549, SGI393550, SGI393551, SGI0, NULL};
const Segment * const SG97803[] = {SGI393552, SGI393553, SGI393554, SGI0, NULL};
const Segment * const SG97804[] = {SGI393555, SGI393556, SGI393557, SGI0, NULL};
const Segment * const SG97805[] = {SGI393558, SGI393559, SGI393560, SGI0, NULL};
const Segment * const SG97806[] = {SGI393561, SGI393562, SGI393563, SGI0, NULL};
const Segment * const SG97807[] = {SGI393564, SGI393565, SGI393566, SGI0, NULL};
const Segment * const SG97808[] = {SGI393567, SGI0, SGI393568, SGI0, NULL};
const Segment * const SG97809[] = {SGI393569, SGI393570, SGI393571, SGI0, NULL};
const Segment * const SG97810[] = {SGI393572, SGI393573, SGI393574, SGI0, NULL};
const Segment * const SG97811[] = {SGI393575, SGI393576, SGI393577, SGI0, NULL};
const Segment * const SG97812[] = {SGI393578, SGI393579, SGI393580, SGI0, NULL};
const Segment * const SG97813[] = {SGI393581, SGI393582, SGI393583, SGI0, NULL};
const Segment * const SG97814[] = {SGI393584, SGI393585, SGI393586, SGI0, NULL};
const Segment * const SG97815[] = {SGI393587, SGI393588, SGI393589, SGI0, NULL};
const Segment * const SG97816[] = {SGI393590, SGI0, SGI393591, SGI0, NULL};
const Segment * const SG97817[] = {SGI393592, SGI0, NULL};
const Segment * const SG97818[] = {SGI393593, SGI0, NULL};
const Segment * const SG97819[] = {SGI393594, SGI0, NULL};
const Segment * const SG97820[] = {SGI393595, SGI0, NULL};
const Segment * const SG97821[] = {SGI393596, SGI0, NULL};
const Segment * const SG97822[] = {SGI393597, SGI0, NULL};
const Segment * const SG97823[] = {SGI393598, SGI0, NULL};
const Segment * const SG97824[] = {SGI393599, SGI0, NULL};
const Segment * const SG97825[] = {SGI393600, SGI0, NULL};
const Segment * const SG97826[] = {SGI393601, SGI393602, SGI393603, SGI0, NULL};
const Segment * const SG97827[] = {SGI393604, SGI393605, SGI393606, SGI0, NULL};
const Segment * const SG97828[] = {SGI393607, SGI393608, SGI393609, SGI0, NULL};
const Segment * const SG97829[] = {SGI393610, SGI0, SGI393611, SGI0, NULL};
const Segment * const SG97830[] = {SGI393612, SGI0, SGI393613, SGI0, NULL};
const Segment * const SG97831[] = {SGI393614, SGI0, NULL};
const Segment * const SG97832[] = {SGI393615, SGI0, NULL};
const Segment * const SG97833[] = {SGI393616, SGI0, NULL};
const Segment * const SG97834[] = {SGI393617, SGI0, NULL};
const Segment * const SG97835[] = {SGI393618, SGI0, NULL};
const Segment * const SG97836[] = {SGI393619, SGI0, NULL};
const Segment * const SG97837[] = {SGI393620, SGI0, NULL};
const Segment * const SG97838[] = {SGI393621, SGI0, NULL};
const Segment * const SG97839[] = {SGI393622, SGI0, NULL};
const Segment * const SG97840[] = {SGI393623, SGI0, NULL};
const Segment * const SG97841[] = {SGI393624, SGI0, NULL};
const Segment * const SG97842[] = {SGI393625, SGI0, NULL};
const Segment * const SG97843[] = {SGI393626, SGI0, NULL};
const Segment * const SG97844[] = {SGI393627, SGI393628, SGI393629, SGI0, NULL};
const Segment * const SG97845[] = {SGI393630, SGI393631, SGI393632, SGI0, NULL};
const Segment * const SG97846[] = {SGI393633, SGI393634, SGI393635, SGI0, NULL};
const Segment * const SG97847[] = {SGI393636, SGI393637, SGI393638, SGI0, NULL};
const Segment * const SG97848[] = {SGI393639, SGI393640, SGI393641, SGI0, NULL};
const Segment * const SG97849[] = {SGI393642, SGI393643, SGI393644, SGI0, NULL};
const Segment * const SG97850[] = {SGI393645, SGI393646, SGI393647, SGI0, NULL};
const Segment * const SG97851[] = {SGI393648, SGI0, NULL};
const Segment * const SG97852[] = {SGI393649, SGI0, NULL};
const Segment * const SG97853[] = {SGI393650, SGI0, NULL};
const Segment * const SG97854[] = {SGI393651, SGI0, NULL};
const Segment * const SG97855[] = {SGI393652, SGI0, NULL};
const Segment * const SG97856[] = {SGI393653, SGI0, NULL};
const Segment * const SG97857[] = {SGI393654, SGI0, NULL};
const Segment * const SG97858[] = {SGI393655, SGI0, NULL};
const Segment * const SG97859[] = {SGI393656, SGI0, NULL};
const Segment * const SG97860[] = {SGI393657, SGI0, NULL};
const Segment * const SG97861[] = {SGI393658, SGI0, NULL};
const Segment * const SG97862[] = {SGI393659, SGI0, NULL};
const Segment * const SG97863[] = {SGI393660, SGI0, NULL};
const Segment * const SG97864[] = {SGI393661, SGI0, NULL};
const Segment * const SG97865[] = {SGI393662, SGI0, SGI393663, SGI0, NULL};
const Segment * const SG97866[] = {SGI393664, SGI0, SGI393665, SGI0, NULL};
const Segment * const SG97867[] = {SGI393666, SGI0, SGI393667, SGI0, NULL};
const Segment * const SG97868[] = {SGI393668, SGI0, SGI393669, SGI0, NULL};
const Segment * const SG97869[] = {SGI393670, SGI0, SGI393671, SGI0, NULL};
const Segment * const SG97870[] = {SGI393672, SGI0, SGI393673, SGI0, NULL};
const Segment * const SG97871[] = {SGI393674, SGI393675, SGI393676, SGI0, NULL};
const Segment * const SG97872[] = {SGI393677, SGI393678, SGI393679, SGI0, NULL};
const Segment * const SG97873[] = {SGI393680, SGI393681, SGI393682, SGI0, NULL};
const Segment * const SG97874[] = {SGI393683, SGI393684, SGI393685, SGI0, NULL};
const Segment * const SG97875[] = {SGI393686, SGI393687, SGI393688, SGI0, NULL};
const Segment * const SG97876[] = {SGI393689, SGI393690, SGI393691, SGI0, NULL};
const Segment * const SG97877[] = {SGI393692, SGI0, NULL};
const Segment * const SG97878[] = {SGI393693, SGI0, NULL};
const Segment * const SG97879[] = {SGI393694, SGI0, NULL};
const Segment * const SG97880[] = {SGI393695, SGI0, NULL};
const Segment * const SG97881[] = {SGI393696, SGI0, NULL};
const Segment * const SG97882[] = {SGI393697, SGI0, NULL};
const Segment * const SG97883[] = {SGI393698, SGI0, NULL};
const Segment * const SG97884[] = {SGI393699, SGI0, NULL};
const Segment * const SG97885[] = {SGI393700, SGI0, SGI393701, SGI0, NULL};
const Segment * const SG97886[] = {SGI393702, SGI0, SGI393703, SGI0, NULL};
const Segment * const SG97887[] = {SGI393704, SGI0, SGI393705, SGI0, NULL};
const Segment * const SG97888[] = {SGI393706, SGI0, SGI393707, SGI0, NULL};
const Segment * const SG97889[] = {SGI393708, SGI0, SGI393709, SGI0, NULL};
const Segment * const SG97890[] = {SGI393710, SGI0, SGI393711, SGI0, NULL};
const Segment * const SG97891[] = {SGI393712, SGI0, SGI393713, SGI0, NULL};
const Segment * const SG97892[] = {SGI393714, SGI0, SGI393715, SGI0, NULL};
const Segment * const SG97893[] = {SGI393716, SGI0, SGI393717, SGI0, NULL};
const Segment * const SG97894[] = {SGI393718, SGI393719, SGI393720, SGI0, NULL};
const Segment * const SG97895[] = {SGI393721, SGI393722, SGI393723, SGI0, NULL};
const Segment * const SG97896[] = {SGI393724, SGI393725, SGI393726, SGI0, NULL};
const Segment * const SG97897[] = {SGI393727, SGI393728, SGI393729, SGI0, NULL};
const Segment * const SG97898[] = {SGI393730, SGI393731, SGI393732, SGI0, NULL};
const Segment * const SG97899[] = {SGI393733, SGI0, NULL};
const Segment * const SG97900[] = {SGI393734, SGI0, NULL};
const Segment * const SG97901[] = {SGI393735, SGI0, NULL};
const Segment * const SG97902[] = {SGI393736, SGI0, NULL};
const Segment * const SG97903[] = {SGI393737, SGI0, NULL};
const Segment * const SG97904[] = {SGI393738, SGI0, NULL};
const Segment * const SG97905[] = {SGI393739, SGI0, NULL};
const Segment * const SG97906[] = {SGI393740, SGI0, NULL};
const Segment * const SG97907[] = {SGI393741, SGI0, NULL};
const Segment * const SG97908[] = {SGI393742, SGI0, NULL};
const Segment * const SG97909[] = {SGI393743, SGI0, NULL};
const Segment * const SG97910[] = {SGI393744, SGI0, NULL};
const Segment * const SG97911[] = {SGI393745, SGI0, NULL};
const Segment * const SG97912[] = {SGI393746, SGI0, NULL};
const Segment * const SG97913[] = {SGI393747, SGI0, NULL};
const Segment * const SG97914[] = {SGI393748, SGI0, NULL};
const Segment * const SG97915[] = {SGI393749, SGI0, NULL};
const Segment * const SG97916[] = {SGI393750, SGI0, NULL};
const Segment * const SG97917[] = {SGI393751, SGI0, NULL};
const Segment * const SG97918[] = {SGI393752, SGI0, NULL};
const Segment * const SG97919[] = {SGI393753, SGI0, NULL};
const Segment * const SG97920[] = {SGI393754, SGI0, NULL};
const Segment * const SG97921[] = {SGI393755, SGI0, NULL};
const Segment * const SG97922[] = {SGI393756, SGI0, NULL};
const Segment * const SG97923[] = {SGI393757, SGI0, NULL};
const Segment * const SG97924[] = {SGI393758, SGI0, NULL};
const Segment * const SG97925[] = {SGI393759, SGI0, NULL};
const Segment * const SG97926[] = {SGI393760, SGI0, NULL};
const Segment * const SG97927[] = {SGI393761, SGI0, NULL};
const Segment * const SG97928[] = {SGI393762, SGI0, NULL};
const Segment * const SG97929[] = {SGI393763, SGI0, NULL};
const Segment * const SG97930[] = {SGI393764, SGI0, NULL};
const Segment * const SG97931[] = {SGI393765, SGI0, NULL};
const Segment * const SG97932[] = {SGI393766, SGI0, NULL};
const Segment * const SG97933[] = {SGI393767, SGI0, NULL};
const Segment * const SG97934[] = {SGI393768, SGI0, NULL};
const Segment * const SG97935[] = {SGI393769, SGI0, NULL};
const Segment * const SG97936[] = {SGI393770, SGI0, NULL};
const Segment * const SG97937[] = {SGI393771, SGI0, NULL};
const Segment * const SG97938[] = {SGI393772, SGI0, NULL};
const Segment * const SG97939[] = {SGI393773, SGI0, NULL};
const Segment * const SG97940[] = {SGI393774, SGI0, NULL};
const Segment * const SG97941[] = {SGI393775, SGI0, NULL};
const Segment * const SG97942[] = {SGI393776, SGI0, NULL};
const Segment * const SG97943[] = {SGI393777, SGI0, NULL};
const Segment * const SG97944[] = {SGI393778, SGI0, NULL};
const Segment * const SG97945[] = {SGI393779, SGI0, NULL};
const Segment * const SG97946[] = {SGI393780, SGI0, NULL};
const Segment * const SG97947[] = {SGI393781, SGI0, NULL};
const Segment * const SG97948[] = {SGI393782, SGI0, SGI393783, SGI0, NULL};
const Segment * const SG97949[] = {SGI393784, SGI0, SGI393785, SGI0, NULL};
const Segment * const SG97950[] = {SGI393786, SGI0, SGI393787, SGI0, NULL};
const Segment * const SG97951[] = {SGI393788, SGI0, SGI393789, SGI0, NULL};
const Segment * const SG97952[] = {SGI393790, SGI393791, SGI393792, SGI0, NULL};
const Segment * const SG97953[] = {SGI393793, SGI393794, SGI393795, SGI0, NULL};
const Segment * const SG97954[] = {SGI393796, SGI0, SGI393797, SGI0, NULL};
const Segment * const SG97955[] = {SGI393798, SGI0, SGI393799, SGI0, NULL};
const Segment * const SG97956[] = {SGI393800, SGI0, SGI393801, SGI0, NULL};
const Segment * const SG97957[] = {SGI393802, SGI393803, SGI393804, SGI0, NULL};
const Segment * const SG97958[] = {SGI393805, SGI393806, SGI393807, SGI0, NULL};
const Segment * const SG97959[] = {SGI393808, SGI393809, SGI393810, SGI0, NULL};
const Segment * const SG97960[] = {SGI393811, SGI393812, SGI393813, SGI0, NULL};
const Segment * const SG97961[] = {SGI393814, SGI393815, SGI393816, SGI0, NULL};
const Segment * const SG97962[] = {SGI393817, SGI393818, SGI393819, SGI0, NULL};
const Segment * const SG97963[] = {SGI393820, SGI0, SGI393821, SGI0, NULL};
const Segment * const SG97964[] = {SGI393822, SGI0, SGI393823, SGI0, NULL};
const Segment * const SG97965[] = {SGI393824, SGI0, SGI393825, SGI0, NULL};
const Segment * const SG97966[] = {SGI393826, SGI393827, SGI393828, SGI0, NULL};
const Segment * const SG97967[] = {SGI393829, SGI393830, SGI393831, SGI0, NULL};
const Segment * const SG97968[] = {SGI393832, SGI393833, SGI393834, SGI0, NULL};
const Segment * const SG97969[] = {SGI393835, SGI0, SGI393836, SGI0, NULL};
const Segment * const SG97970[] = {SGI393837, SGI0, SGI393838, SGI393839, NULL};
const Segment * const SG97971[] = {SGI393840, SGI0, SGI393841, SGI0, NULL};
const Segment * const SG97972[] = {SGI393842, SGI0, NULL};
const Segment * const SG97973[] = {SGI393843, SGI0, NULL};
const Segment * const SG97974[] = {SGI393844, SGI0, NULL};
const Segment * const SG97975[] = {SGI393845, SGI0, NULL};
const Segment * const SG97976[] = {SGI393846, SGI0, NULL};
const Segment * const SG97977[] = {SGI393847, SGI0, NULL};
const Segment * const SG97978[] = {SGI393848, SGI0, NULL};
const Segment * const SG97979[] = {SGI393849, SGI0, NULL};
const Segment * const SG97980[] = {SGI393850, SGI0, NULL};
const Segment * const SG97981[] = {SGI393851, SGI0, NULL};
const Segment * const SG97982[] = {SGI393852, SGI0, NULL};
const Segment * const SG97983[] = {SGI393853, SGI0, NULL};
const Segment * const SG97984[] = {SGI393854, SGI0, NULL};
const Segment * const SG97985[] = {SGI393855, SGI0, NULL};
const Segment * const SG97986[] = {SGI393856, SGI0, NULL};
const Segment * const SG97987[] = {SGI393857, SGI0, NULL};
const Segment * const SG97988[] = {SGI393858, SGI0, NULL};
const Segment * const SG97989[] = {SGI393859, SGI0, NULL};
const Segment * const SG97990[] = {SGI393860, SGI0, NULL};
const Segment * const SG97991[] = {SGI393861, SGI0, NULL};
const Segment * const SG97992[] = {SGI393862, SGI0, NULL};
const Segment * const SG97993[] = {SGI393863, SGI0, NULL};
const Segment * const SG97994[] = {SGI393864, SGI0, NULL};
const Segment * const SG97995[] = {SGI393865, SGI0, NULL};
const Segment * const SG97996[] = {SGI393866, SGI0, NULL};
const Segment * const SG97997[] = {SGI393867, SGI0, NULL};
const Segment * const SG97998[] = {SGI393868, SGI0, NULL};
const Segment * const SG97999[] = {SGI393869, SGI0, NULL};
const Segment * const SG98000[] = {SGI393870, SGI0, NULL};
const Segment * const SG98001[] = {SGI393871, SGI0, NULL};
const Segment * const SG98002[] = {SGI393872, SGI0, NULL};
const Segment * const SG98003[] = {SGI393873, SGI0, NULL};
const Segment * const SG98004[] = {SGI393874, SGI0, NULL};
const Segment * const SG98005[] = {SGI393875, SGI0, NULL};
const Segment * const SG98006[] = {SGI393876, SGI0, NULL};
const Segment * const SG98007[] = {SGI393877, SGI0, SGI393878, SGI0, NULL};
const Segment * const SG98008[] = {SGI393879, SGI0, SGI393880, SGI0, NULL};
const Segment * const SG98009[] = {SGI393881, SGI0, SGI393882, SGI393883, NULL};
const Segment * const SG98010[] = {SGI393884, SGI393885, SGI393886, SGI0, NULL};
const Segment * const SG98011[] = {SGI393887, SGI393888, SGI393889, SGI0, NULL};
const Segment * const SG98012[] = {SGI393890, SGI393891, SGI393892, SGI0, NULL};
const Segment * const SG98013[] = {SGI393893, SGI0, SGI393894, SGI0, NULL};
const Segment * const SG98014[] = {SGI393895, SGI393896, NULL};
const Segment * const SG98015[] = {SGI393897, SGI393898, NULL};
const Segment * const SG98016[] = {SGI393899, SGI393900, NULL};
const Segment * const SG98017[] = {SGI393901, SGI393902, NULL};
const Segment * const SG98018[] = {SGI393903, SGI393904, NULL};
const Segment * const SG98019[] = {SGI393905, SGI393906, NULL};
const Segment * const SG98020[] = {SGI393907, SGI393908, NULL};
const Segment * const SG98021[] = {SGI393909, SGI393910, NULL};
const Segment * const SG98022[] = {SGI393911, SGI393912, NULL};
const Segment * const SG98023[] = {SGI393913, SGI393914, NULL};
const Segment * const SG98024[] = {SGI393915, SGI393916, NULL};
const Segment * const SG98025[] = {SGI393917, SGI393918, NULL};
const Segment * const SG98026[] = {SGI393919, SGI393920, NULL};
const Segment * const SG98027[] = {SGI0, SGI393921, NULL};
const Segment * const SG98028[] = {SGI393922, SGI393923, NULL};
const Segment * const SG98029[] = {SGI393924, SGI393925, NULL};
const Segment * const SG98030[] = {SGI393926, SGI393927, NULL};
const Segment * const SG98031[] = {SGI393928, SGI393929, NULL};
const Segment * const SG98032[] = {SGI393930, SGI393931, NULL};
const Segment * const SG98033[] = {SGI393932, SGI393933, NULL};
const Segment * const SG98034[] = {SGI393934, SGI393935, NULL};
const Segment * const SG98035[] = {SGI393936, SGI393937, NULL};
const Segment * const SG98036[] = {SGI393938, SGI393939, NULL};
const Segment * const SG98037[] = {SGI393940, SGI393941, NULL};
const Segment * const SG98038[] = {SGI393942, SGI393943, NULL};
const Segment * const SG98039[] = {SGI393944, SGI393945, NULL};
const Segment * const SG98040[] = {SGI393946, SGI393947, NULL};
const Segment * const SG98041[] = {SGI393948, SGI393949, NULL};
const Segment * const SG98042[] = {SGI393950, SGI393951, NULL};
const Segment * const SG98043[] = {SGI393952, SGI393953, NULL};
const Segment * const SG98044[] = {SGI393954, SGI0, SGI393955, SGI0, NULL};
const Segment * const SG98045[] = {SGI393956, SGI0, NULL};
const Segment * const SG98046[] = {SGI393957, SGI0, NULL};
const Segment * const SG98047[] = {SGI393958, SGI0, NULL};
const Segment * const SG98048[] = {SGI393959, SGI0, NULL};
const Segment * const SG98049[] = {SGI393960, SGI0, NULL};
const Segment * const SG98050[] = {SGI393961, SGI0, NULL};
const Segment * const SG98051[] = {SGI393962, SGI0, NULL};
const Segment * const SG98052[] = {SGI393963, SGI0, NULL};
const Segment * const SG98053[] = {SGI393964, SGI0, NULL};
const Segment * const SG98054[] = {SGI393965, SGI0, NULL};
const Segment * const SG98055[] = {SGI393966, SGI0, NULL};
const Segment * const SG98056[] = {SGI393967, SGI0, NULL};
const Segment * const SG98057[] = {SGI393968, SGI0, NULL};
const Segment * const SG98058[] = {SGI393969, SGI0, NULL};
const Segment * const SG98059[] = {SGI393970, SGI0, NULL};
const Segment * const SG98060[] = {SGI393971, SGI0, NULL};
const Segment * const SG98061[] = {SGI393972, SGI0, NULL};
const Segment * const SG98062[] = {SGI393973, SGI0, NULL};
const Segment * const SG98063[] = {SGI393974, SGI0, NULL};
const Segment * const SG98064[] = {SGI393975, SGI0, NULL};
const Segment * const SG98065[] = {SGI393976, SGI393977, SGI393978, SGI0, NULL};
const Segment * const SG98066[] = {SGI393979, SGI393980, SGI393981, SGI0, NULL};
const Segment * const SG98067[] = {SGI393982, SGI393983, SGI393984, SGI0, NULL};
const Segment * const SG98068[] = {SGI393985, SGI393986, SGI393987, SGI0, NULL};
const Segment * const SG98069[] = {SGI393988, SGI393989, SGI393990, SGI0, NULL};
const Segment * const SG98070[] = {SGI393991, SGI393992, SGI393993, SGI0, NULL};
const Segment * const SG98071[] = {SGI393994, SGI393995, SGI393996, SGI0, NULL};
const Segment * const SG98072[] = {SGI393997, SGI393998, SGI393999, SGI0, NULL};
const Segment * const SG98073[] = {SGI394000, SGI0, SGI394001, SGI0, NULL};
const Segment * const SG98074[] = {SGI394002, SGI394003, SGI394004, SGI0, NULL};
const Segment * const SG98075[] = {SGI394005, SGI394006, SGI394007, SGI0, NULL};
const Segment * const SG98076[] = {SGI394008, SGI394009, SGI394010, SGI0, NULL};
const Segment * const SG98077[] = {SGI394011, SGI0, NULL};
const Segment * const SG98078[] = {SGI394012, SGI0, NULL};
const Segment * const SG98079[] = {SGI394013, SGI0, NULL};
const Segment * const SG98080[] = {SGI394014, SGI0, NULL};
const Segment * const SG98081[] = {SGI394015, SGI0, NULL};
const Segment * const SG98082[] = {SGI394016, SGI0, NULL};
const Segment * const SG98083[] = {SGI394017, SGI0, NULL};
const Segment * const SG98084[] = {SGI394018, SGI0, NULL};
const Segment * const SG98085[] = {SGI394019, SGI0, NULL};
const Segment * const SG98086[] = {SGI394020, SGI0, NULL};
const Segment * const SG98087[] = {SGI394021, SGI0, NULL};
const Segment * const SG98088[] = {SGI394022, SGI0, SGI394023, SGI0, NULL};
const Segment * const SG98089[] = {SGI394024, SGI394025, SGI394026, SGI0, NULL};
const Segment * const SG98090[] = {SGI394027, SGI394028, SGI394029, SGI0, NULL};
const Segment * const SG98091[] = {SGI394030, SGI394031, SGI394032, SGI0, NULL};
const Segment * const SG98092[] = {SGI394033, SGI394034, SGI394035, SGI0, NULL};
const Segment * const SG98093[] = {SGI394036, SGI394037, SGI394038, SGI0, NULL};
const Segment * const SG98094[] = {SGI394039, SGI0, SGI394040, SGI0, NULL};
const Segment * const SG98095[] = {SGI394041, SGI0, SGI394042, SGI0, NULL};
const Segment * const SG98096[] = {SGI394043, SGI0, SGI394044, SGI0, NULL};
const Segment * const SG98097[] = {SGI394045, SGI394046, SGI394047, SGI0, NULL};
const Segment * const SG98098[] = {SGI394048, SGI394049, SGI394050, SGI0, NULL};
const Segment * const SG98099[] = {SGI394051, SGI394052, SGI394053, SGI0, NULL};
const Segment * const SG98100[] = {SGI394054, SGI394055, SGI394056, SGI0, NULL};
const Segment * const SG98101[] = {SGI394057, SGI394058, SGI394059, SGI0, NULL};
const Segment * const SG98102[] = {SGI394060, SGI0, SGI394061, SGI0, NULL};
const Segment * const SG98103[] = {SGI394062, SGI0, SGI394063, SGI0, NULL};
const Segment * const SG98104[] = {SGI394064, SGI0, SGI394065, SGI0, NULL};
const Segment * const SG98105[] = {SGI394066, SGI0, SGI394067, SGI0, NULL};
const Segment * const SG98106[] = {SGI394068, SGI0, SGI394069, SGI0, NULL};
const Segment * const SG98107[] = {SGI394070, SGI0, SGI394071, SGI0, NULL};
const Segment * const SG98108[] = {SGI394072, SGI0, SGI394073, SGI394074, NULL};
const Segment * const SG98109[] = {SGI394075, SGI0, SGI394076, SGI0, NULL};
const Segment * const SG98110[] = {SGI394077, SGI0, SGI394078, SGI0, NULL};
const Segment * const SG98111[] = {SGI394079, SGI0, SGI394080, SGI0, NULL};
const Segment * const SG98112[] = {SGI394081, SGI0, SGI394082, SGI0, NULL};
const Segment * const SG98113[] = {SGI394083, SGI394084, SGI394085, SGI0, NULL};
const Segment * const SG98114[] = {SGI394086, SGI0, SGI394087, SGI0, NULL};
const Segment * const SG98115[] = {SGI394088, SGI0, SGI394089, SGI0, NULL};
const Segment * const SG98116[] = {SGI394090, SGI0, SGI394091, SGI0, NULL};
const Segment * const SG98117[] = {SGI394092, SGI394093, SGI394094, SGI0, NULL};
const Segment * const SG98118[] = {SGI394095, SGI394096, SGI394097, SGI0, NULL};
const Segment * const SG98119[] = {SGI394098, SGI394099, SGI394100, SGI0, NULL};
const Segment * const SG98120[] = {SGI394101, SGI394102, SGI394103, SGI0, NULL};
const Segment * const SG98121[] = {SGI394104, SGI394105, SGI394106, SGI0, NULL};
const Segment * const SG98122[] = {SGI394107, SGI394108, SGI394109, SGI0, NULL};
const Segment * const SG98123[] = {SGI394110, SGI394111, SGI394112, SGI0, NULL};
const Segment * const SG98124[] = {SGI394113, SGI0, SGI394114, SGI0, NULL};
const Segment * const SG98125[] = {SGI394115, SGI0, SGI394116, SGI394117, NULL};
const Segment * const SG98126[] = {SGI394118, SGI0, SGI394119, SGI0, NULL};
const Segment * const SG98127[] = {SGI394120, SGI394121, NULL};
const Segment * const SG98128[] = {SGI394122, SGI394123, NULL};
const Segment * const SG98129[] = {SGI394124, SGI394125, NULL};
const Segment * const SG98130[] = {SGI394126, SGI394127, NULL};
const Segment * const SG98131[] = {SGI394128, SGI394129, NULL};
const Segment * const SG98132[] = {SGI394130, SGI394131, NULL};
const Segment * const SG98133[] = {SGI394132, SGI394133, SGI394134, SGI0, NULL};
const Segment * const SG98134[] = {SGI394135, SGI394136, SGI394137, SGI0, NULL};
const Segment * const SG98135[] = {SGI394138, SGI394139, SGI394140, SGI0, NULL};
const Segment * const SG98136[] = {SGI394141, SGI394142, SGI394143, SGI0, NULL};
const Segment * const SG98137[] = {SGI394144, SGI394145, SGI394146, SGI0, NULL};
const Segment * const SG98138[] = {SGI394147, SGI394148, SGI394149, SGI0, NULL};
const Segment * const SG98139[] = {SGI394150, SGI0, SGI394151, SGI0, NULL};
const Segment * const SG98140[] = {SGI394152, SGI394153, SGI394154, SGI0, NULL};
const Segment * const SG98141[] = {SGI394155, SGI0, SGI394156, SGI0, NULL};
const Segment * const SG98142[] = {SGI394157, SGI0, SGI394158, SGI0, NULL};
const Segment * const SG98143[] = {SGI394159, SGI0, SGI394160, SGI0, NULL};
const Segment * const SG98144[] = {SGI394161, SGI0, SGI394162, SGI0, NULL};
const Segment * const SG98145[] = {SGI394163, SGI394164, SGI394165, SGI0, NULL};
const Segment * const SG98146[] = {SGI394166, SGI0, SGI394167, SGI0, NULL};
const Segment * const SG98147[] = {SGI394168, SGI394169, SGI394170, SGI0, NULL};
const Segment * const SG98148[] = {SGI394171, SGI394172, SGI394173, SGI0, NULL};
const Segment * const SG98149[] = {SGI394174, SGI394175, SGI394176, SGI0, NULL};
const Segment * const SG98150[] = {SGI394177, SGI394178, SGI394179, SGI0, NULL};
const Segment * const SG98151[] = {SGI394180, SGI394181, SGI394182, SGI0, NULL};
const Segment * const SG98152[] = {SGI394183, SGI0, SGI394184, SGI0, NULL};
const Segment * const SG98153[] = {SGI394185, SGI394186, SGI394187, SGI0, NULL};
const Segment * const SG98154[] = {SGI394188, SGI394189, SGI394190, SGI0, NULL};
const Segment * const SG98155[] = {SGI394191, SGI394192, SGI394193, SGI0, NULL};
const Segment * const SG98156[] = {SGI394194, SGI394195, SGI394196, SGI0, NULL};
const Segment * const SG98157[] = {SGI394197, SGI394198, SGI394199, SGI0, NULL};
const Segment * const SG98158[] = {SGI394200, SGI394201, SGI394202, SGI0, NULL};
const Segment * const SG98159[] = {SGI394203, SGI394204, SGI394205, SGI0, NULL};
const Segment * const SG98160[] = {SGI394206, SGI394207, SGI394208, SGI0, NULL};
const Segment * const SG98161[] = {SGI394209, SGI0, SGI394210, SGI0, NULL};
const Segment * const SG98162[] = {SGI394211, SGI394212, SGI394213, SGI0, NULL};
const Segment * const SG98163[] = {SGI394214, SGI394215, SGI394216, SGI0, NULL};
const Segment * const SG98164[] = {SGI394217, SGI394218, SGI394219, SGI0, NULL};
const Segment * const SG98165[] = {SGI394220, SGI394221, SGI394222, SGI0, NULL};
const Segment * const SG98166[] = {SGI394223, SGI394224, SGI394225, SGI0, NULL};
const Segment * const SG98167[] = {SGI394226, SGI394227, SGI394228, SGI0, NULL};
const Segment * const SG98168[] = {SGI394229, SGI0, NULL};
const Segment * const SG98169[] = {SGI394230, SGI0, SGI394231, SGI0, NULL};
const Segment * const SG98170[] = {SGI394232, SGI394233, SGI394234, SGI0, NULL};
const Segment * const SG98171[] = {SGI394235, SGI0, SGI394236, SGI0, NULL};
const Segment * const SG98172[] = {SGI394237, SGI0, SGI394238, SGI0, NULL};
const Segment * const SG98173[] = {SGI394239, SGI0, SGI394240, SGI394241, NULL};
const Segment * const SG98174[] = {SGI394242, SGI0, SGI394243, SGI0, NULL};
const Segment * const SG98175[] = {SGI394244, SGI0, SGI394245, SGI0, NULL};
const Segment * const SG98176[] = {SGI394246, SGI0, NULL};
const Segment * const SG98177[] = {SGI394247, SGI394248, SGI394249, SGI0, NULL};
const Segment * const SG98178[] = {SGI394250, SGI0, SGI394251, SGI394252, NULL};
const Segment * const SG98179[] = {SGI394253, SGI0, SGI394254, SGI394255, NULL};
const Segment * const SG98180[] = {SGI394256, SGI0, SGI394257, SGI394258, NULL};
const Segment * const SG98181[] = {SGI394259, SGI0, SGI394260, SGI394261, NULL};
const Segment * const SG98182[] = {SGI394262, SGI0, SGI394263, SGI394264, NULL};
const Segment * const SG98183[] = {SGI394265, SGI394266, SGI394267, SGI0, NULL};
const Segment * const SG98184[] = {SGI394268, SGI394269, SGI394270, SGI0, NULL};
const Segment * const SG98185[] = {SGI394271, SGI0, SGI394272, SGI394273, NULL};
const Segment * const SG98186[] = {SGI394274, SGI0, NULL};
const Segment * const SG98187[] = {SGI394275, SGI0, NULL};
const Segment * const SG98188[] = {SGI394276, SGI0, NULL};
const Segment * const SG98189[] = {SGI394277, SGI0, NULL};
const Segment * const SG98190[] = {SGI394278, SGI0, NULL};
const Segment * const SG98191[] = {SGI394279, SGI0, NULL};
const Segment * const SG98192[] = {SGI394280, SGI0, NULL};
const Segment * const SG98193[] = {SGI394281, SGI0, NULL};
const Segment * const SG98194[] = {SGI394282, SGI0, NULL};
const Segment * const SG98195[] = {SGI394283, SGI0, NULL};
const Segment * const SG98196[] = {SGI394284, SGI0, NULL};
const Segment * const SG98197[] = {SGI394285, SGI0, NULL};
const Segment * const SG98198[] = {SGI394286, SGI0, NULL};
const Segment * const SG98199[] = {SGI394287, SGI0, NULL};
const Segment * const SG98200[] = {SGI394288, SGI0, NULL};
const Segment * const SG98201[] = {SGI394289, SGI0, NULL};
const Segment * const SG98202[] = {SGI394290, SGI0, NULL};
const Segment * const SG98203[] = {SGI394291, SGI0, NULL};
const Segment * const SG98204[] = {SGI394292, SGI0, NULL};
const Segment * const SG98205[] = {SGI394293, SGI0, NULL};
const Segment * const SG98206[] = {SGI394294, SGI0, NULL};
const Segment * const SG98207[] = {SGI394295, SGI0, NULL};
const Segment * const SG98208[] = {SGI394296, SGI0, NULL};
const Segment * const SG98209[] = {SGI394297, SGI0, NULL};
const Segment * const SG98210[] = {SGI394298, SGI0, NULL};
const Segment * const SG98211[] = {SGI394299, SGI0, NULL};
const Segment * const SG98212[] = {SGI394300, SGI0, NULL};
const Segment * const SG98213[] = {SGI394301, SGI0, NULL};
const Segment * const SG98214[] = {SGI394302, SGI0, NULL};
const Segment * const SG98215[] = {SGI394303, SGI0, NULL};
const Segment * const SG98216[] = {SGI394304, SGI0, NULL};
const Segment * const SG98217[] = {SGI394305, SGI0, NULL};
const Segment * const SG98218[] = {SGI394306, SGI0, NULL};
const Segment * const SG98219[] = {SGI394307, SGI0, NULL};
const Segment * const SG98220[] = {SGI394308, SGI0, NULL};
const Segment * const SG98221[] = {SGI394309, SGI0, NULL};
const Segment * const SG98222[] = {SGI394310, SGI0, NULL};
const Segment * const SG98223[] = {SGI394311, SGI0, NULL};
const Segment * const SG98224[] = {SGI394312, SGI0, NULL};
const Segment * const SG98225[] = {SGI394313, SGI0, NULL};
const Segment * const SG98226[] = {SGI394314, SGI0, NULL};
const Segment * const SG98227[] = {SGI394315, SGI0, NULL};
const Segment * const SG98228[] = {SGI394316, SGI0, NULL};
const Segment * const SG98229[] = {SGI394317, SGI0, NULL};
const Segment * const SG98230[] = {SGI394318, SGI0, NULL};
const Segment * const SG98231[] = {SGI394319, SGI0, NULL};
const Segment * const SG98232[] = {SGI394320, SGI0, NULL};
const Segment * const SG98233[] = {SGI394321, SGI0, NULL};
const Segment * const SG98234[] = {SGI394322, SGI0, NULL};
const Segment * const SG98235[] = {SGI394323, SGI0, NULL};
const Segment * const SG98236[] = {SGI394324, SGI0, NULL};
const Segment * const SG98237[] = {SGI394325, SGI0, NULL};
const Segment * const SG98238[] = {SGI394326, SGI0, NULL};
const Segment * const SG98239[] = {SGI394327, SGI0, NULL};
const Segment * const SG98240[] = {SGI394328, SGI0, NULL};
const Segment * const SG98241[] = {SGI394329, SGI0, NULL};
const Segment * const SG98242[] = {SGI394330, SGI0, NULL};
const Segment * const SG98243[] = {SGI394331, SGI0, NULL};
const Segment * const SG98244[] = {SGI394332, SGI0, NULL};
const Segment * const SG98245[] = {SGI394333, SGI0, NULL};
const Segment * const SG98246[] = {SGI394334, SGI0, NULL};
const Segment * const SG98247[] = {SGI394335, SGI0, NULL};
const Segment * const SG98248[] = {SGI394336, SGI0, NULL};
const Segment * const SG98249[] = {SGI394337, SGI0, NULL};
const Segment * const SG98250[] = {SGI394338, SGI0, NULL};
const Segment * const SG98251[] = {SGI394339, SGI0, NULL};
const Segment * const SG98252[] = {SGI394340, SGI0, NULL};
const Segment * const SG98253[] = {SGI394341, SGI0, NULL};
const Segment * const SG98254[] = {SGI394342, SGI0, NULL};
const Segment * const SG98255[] = {SGI394343, SGI0, NULL};
const Segment * const SG98256[] = {SGI394344, SGI0, NULL};
const Segment * const SG98257[] = {SGI394345, SGI0, NULL};
const Segment * const SG98258[] = {SGI394346, SGI0, NULL};
const Segment * const SG98259[] = {SGI394347, SGI394348, SGI394349, SGI0, NULL};
const Segment * const SG98260[] = {SGI394350, SGI394351, SGI394352, SGI0, NULL};
const Segment * const SG98261[] = {SGI394353, SGI0, NULL};
const Segment * const SG98262[] = {SGI394354, SGI0, NULL};
const Segment * const SG98263[] = {SGI394355, SGI0, NULL};
const Segment * const SG98264[] = {SGI394356, SGI0, NULL};
const Segment * const SG98265[] = {SGI394357, SGI0, NULL};
const Segment * const SG98266[] = {SGI394358, SGI0, NULL};
const Segment * const SG98267[] = {SGI394359, SGI0, NULL};
const Segment * const SG98268[] = {SGI394360, SGI0, NULL};
const Segment * const SG98269[] = {SGI394361, SGI394362, SGI394363, SGI0, NULL};
const Segment * const SG98270[] = {SGI394364, SGI394365, SGI394366, SGI0, NULL};
const Segment * const SG98271[] = {SGI394367, SGI394368, SGI394369, SGI0, NULL};
const Segment * const SG98272[] = {SGI394370, SGI394371, SGI394372, SGI0, NULL};
const Segment * const SG98273[] = {SGI394373, SGI394374, SGI394375, SGI0, NULL};
const Segment * const SG98274[] = {SGI394376, SGI394377, SGI394378, SGI0, NULL};
const Segment * const SG98275[] = {SGI394379, SGI394380, SGI394381, SGI0, NULL};
const Segment * const SG98276[] = {SGI394382, SGI0, NULL};
const Segment * const SG98277[] = {SGI394383, SGI394384, SGI394385, SGI0, NULL};
const Segment * const SG98278[] = {SGI394386, SGI394387, SGI394388, SGI0, NULL};
const Segment * const SG98279[] = {SGI394389, SGI394390, SGI394391, SGI0, NULL};
const Segment * const SG98280[] = {SGI394392, SGI394393, SGI394394, SGI0, NULL};
const Segment * const SG98281[] = {SGI394395, SGI394396, SGI394397, SGI0, NULL};
const Segment * const SG98282[] = {SGI394398, SGI394399, SGI394400, SGI0, NULL};
const Segment * const SG98283[] = {SGI394401, SGI394402, SGI394403, SGI0, NULL};
const Segment * const SG98284[] = {SGI394404, SGI394405, SGI394406, SGI0, NULL};
const Segment * const SG98285[] = {SGI394407, SGI394408, SGI394409, SGI0, NULL};
const Segment * const SG98286[] = {SGI394410, SGI394411, SGI394412, SGI0, SGI394413, NULL};
const Segment * const SG98287[] = {SGI394414, SGI394415, SGI394416, SGI0, SGI394417, NULL};
const Segment * const SG98288[] = {SGI394418, SGI394419, SGI394420, SGI0, SGI394421, NULL};
const Segment * const SG98289[] = {SGI394422, SGI394423, SGI394424, SGI0, SGI394425, NULL};
const Segment * const SG98290[] = {SGI394426, SGI394427, SGI394428, SGI0, SGI394429, NULL};
const Segment * const SG98291[] = {SGI394430, SGI394431, SGI394432, SGI0, SGI394433, NULL};
const Segment * const SG98292[] = {SGI394434, SGI394435, SGI394436, SGI0, SGI394437, NULL};
const Segment * const SG98293[] = {SGI394438, SGI394439, SGI394440, SGI0, SGI394441, NULL};
const Segment * const SG98294[] = {SGI394442, SGI394443, SGI394444, SGI0, SGI394445, NULL};
const Segment * const SG98295[] = {SGI394446, SGI394447, SGI0, SGI0, SGI0, NULL};
const Segment * const SG98296[] = {SGI394448, SGI394449, SGI394450, SGI394451, SGI0, NULL};
const Segment * const SG98297[] = {SGI394452, SGI394453, SGI394454, SGI394455, SGI0, NULL};
const Segment * const SG98298[] = {SGI394456, SGI394457, SGI394458, SGI394459, SGI0, NULL};
const Segment * const SG98299[] = {SGI394460, SGI394461, SGI394462, SGI0, SGI394463, NULL};
const Segment * const SG98300[] = {SGI394464, SGI394465, SGI394466, SGI0, SGI394467, NULL};
const Segment * const SG98301[] = {SGI394468, SGI394469, SGI0, SGI0, SGI0, NULL};
const Segment * const SG98302[] = {SGI394470, SGI394471, SGI394472, SGI0, SGI394473, NULL};
const Segment * const SG98303[] = {SGI394474, SGI394475, SGI394476, SGI0, SGI394477, NULL};
const Segment * const SG98304[] = {SGI394478, SGI394479, SGI394480, SGI0, SGI394481, NULL};
const Segment * const SG98305[] = {SGI394482, SGI394483, SGI0, SGI0, SGI0, NULL};
const Segment * const SG98306[] = {SGI394484, SGI394485, SGI394486, SGI0, SGI394487, NULL};
const Segment * const SG98307[] = {SGI394488, SGI394489, SGI394490, SGI394491, SGI0, NULL};
const Segment * const SG98308[] = {SGI394492, SGI394493, SGI394494, SGI394495, SGI0, NULL};
const Segment * const SG98309[] = {SGI394496, SGI394497, SGI394498, SGI394499, SGI0, NULL};
const Segment * const SG98310[] = {SGI394500, SGI394501, SGI394502, SGI394503, SGI0, NULL};
const Segment * const SG98311[] = {SGI394504, SGI394505, SGI394506, SGI394507, SGI0, NULL};
const Segment * const SG98312[] = {SGI394508, SGI394509, SGI394510, SGI394511, SGI0, NULL};
const Segment * const SG98313[] = {SGI394512, SGI394513, SGI394514, SGI394515, SGI0, NULL};
const Segment * const SG98314[] = {SGI394516, SGI394517, SGI394518, SGI394519, SGI0, NULL};
const Segment * const SG98315[] = {SGI394520, SGI394521, SGI394522, SGI394523, SGI0, NULL};
const Segment * const SG98316[] = {SGI394524, SGI394525, SGI394526, SGI0, SGI394527, NULL};
const Segment * const SG98317[] = {SGI394528, SGI394529, SGI394530, SGI0, SGI394531, NULL};
const Segment * const SG98318[] = {SGI394532, SGI394533, SGI394534, SGI0, SGI394535, NULL};
const Segment * const SG98319[] = {SGI394536, SGI394537, SGI394538, SGI0, SGI394539, NULL};
const Segment * const SG98320[] = {SGI394540, SGI394541, SGI394542, SGI0, SGI394543, NULL};
const Segment * const SG98321[] = {SGI394544, SGI394545, SGI394546, SGI0, SGI394547, NULL};
const Segment * const SG98322[] = {SGI394548, SGI394549, SGI394550, SGI0, SGI394551, NULL};
const Segment * const SG98323[] = {SGI394552, SGI394553, SGI0, SGI0, SGI0, NULL};
const Segment * const SG98324[] = {SGI394554, SGI394555, SGI394556, SGI0, SGI394557, NULL};
const Segment * const SG98325[] = {SGI394558, SGI394559, SGI394560, SGI0, SGI394561, NULL};
const Segment * const SG98326[] = {SGI394562, SGI394563, SGI394564, SGI394565, SGI0, NULL};
const Segment * const SG98327[] = {SGI394566, SGI394567, SGI394568, SGI394569, SGI0, NULL};
const Segment * const SG98328[] = {SGI394570, SGI394571, SGI394572, SGI394573, SGI0, NULL};
const Segment * const SG98329[] = {SGI394574, SGI394575, SGI394576, SGI394577, SGI0, NULL};
const Segment * const SG98330[] = {SGI394578, SGI394579, SGI394580, SGI0, SGI394581, NULL};
const Segment * const SG98331[] = {SGI394582, SGI394583, SGI394584, SGI0, SGI394585, NULL};
const Segment * const SG98332[] = {SGI394586, SGI394587, SGI394588, SGI394589, SGI0, NULL};
const Segment * const SG98333[] = {SGI394590, SGI394591, SGI394592, SGI394593, SGI0, NULL};
const Segment * const SG98334[] = {SGI394594, SGI394595, SGI394596, SGI394597, SGI0, NULL};
const Segment * const SG98335[] = {SGI394598, SGI394599, SGI394600, SGI394601, SGI0, NULL};
const Segment * const SG98336[] = {SGI394602, SGI394603, SGI394604, SGI394605, SGI0, NULL};
const Segment * const SG98337[] = {SGI394606, SGI394607, SGI394608, SGI394609, SGI0, NULL};
const Segment * const SG98338[] = {SGI394610, SGI394611, SGI394612, SGI394613, SGI0, NULL};
const Segment * const SG98339[] = {SGI394614, SGI394615, SGI394616, SGI394617, SGI0, NULL};
const Segment * const SG98340[] = {SGI394618, SGI394619, SGI394620, SGI394621, SGI0, NULL};
const Segment * const SG98341[] = {SGI394622, SGI394623, SGI394624, SGI394625, SGI0, NULL};
const Segment * const SG98342[] = {SGI394626, SGI394627, SGI394628, SGI394629, SGI0, NULL};
const Segment * const SG98343[] = {SGI394630, SGI394631, SGI394632, SGI394633, SGI0, NULL};
const Segment * const SG98344[] = {SGI394634, SGI394635, SGI394636, SGI394637, SGI0, NULL};
const Segment * const SG98345[] = {SGI394638, SGI394639, SGI394640, SGI394641, SGI0, NULL};
const Segment * const SG98346[] = {SGI394642, SGI394643, SGI394644, SGI394645, SGI0, NULL};
const Segment * const SG98347[] = {SGI394646, SGI394647, SGI394648, SGI394649, SGI0, NULL};
const Segment * const SG98348[] = {SGI394650, SGI394651, SGI394652, SGI394653, SGI0, NULL};
const Segment * const SG98349[] = {SGI394654, SGI394655, SGI394656, SGI394657, SGI0, NULL};
const Segment * const SG98350[] = {SGI394658, SGI394659, SGI394660, SGI394661, SGI0, NULL};
const Segment * const SG98351[] = {SGI394662, SGI394663, SGI394664, SGI394665, SGI0, NULL};
const Segment * const SG98352[] = {SGI394666, SGI394667, SGI394668, SGI394669, SGI0, NULL};
const Segment * const SG98353[] = {SGI394670, SGI394671, SGI394672, SGI394673, SGI0, NULL};
const Segment * const SG98354[] = {SGI394674, SGI394675, SGI394676, SGI394677, SGI0, NULL};
const Segment * const SG98355[] = {SGI394678, SGI394679, SGI394680, SGI394681, SGI0, NULL};
const Segment * const SG98356[] = {SGI394682, SGI394683, SGI394684, SGI394685, SGI0, NULL};
const Segment * const SG98357[] = {SGI394686, SGI394687, SGI394688, SGI394689, SGI0, NULL};
const Segment * const SG98358[] = {SGI394690, SGI394691, SGI394692, SGI394693, SGI0, NULL};
const Segment * const SG98359[] = {SGI394694, SGI394695, SGI394696, SGI394697, SGI0, NULL};
const Segment * const SG98360[] = {SGI394698, SGI394699, SGI394700, SGI394701, SGI0, NULL};
const Segment * const SG98361[] = {SGI394702, SGI394703, SGI394704, SGI394705, SGI0, NULL};
const Segment * const SG98362[] = {SGI394706, SGI394707, SGI394708, SGI394709, SGI0, NULL};
const Segment * const SG98363[] = {SGI394710, SGI394711, SGI394712, SGI394713, SGI0, NULL};
const Segment * const SG98364[] = {SGI394714, SGI394715, SGI394716, SGI394717, SGI0, NULL};
const Segment * const SG98365[] = {SGI394718, SGI394719, SGI394720, SGI394721, SGI0, NULL};
const Segment * const SG98366[] = {SGI394722, SGI394723, SGI394724, SGI394725, SGI0, NULL};
const Segment * const SG98367[] = {SGI394726, SGI394727, SGI394728, SGI394729, SGI0, NULL};
const Segment * const SG98368[] = {SGI394730, SGI394731, SGI394732, SGI394733, SGI0, NULL};
const Segment * const SG98369[] = {SGI394734, SGI394735, SGI394736, SGI0, SGI394737, NULL};
const Segment * const SG98370[] = {SGI394738, SGI394739, SGI394740, SGI0, SGI394741, NULL};
const Segment * const SG98371[] = {SGI394742, SGI394743, SGI394744, SGI0, SGI394745, NULL};
const Segment * const SG98372[] = {SGI394746, SGI394747, SGI394748, SGI0, SGI394749, NULL};
const Segment * const SG98373[] = {SGI394750, SGI394751, SGI394752, SGI0, SGI394753, NULL};
const Segment * const SG98374[] = {SGI394754, SGI394755, SGI394756, SGI0, SGI394757, NULL};
const Segment * const SG98375[] = {SGI394758, SGI394759, SGI394760, SGI0, SGI394761, NULL};
const Segment * const SG98376[] = {SGI394762, SGI394763, SGI0, SGI0, SGI0, NULL};
const Segment * const SG98377[] = {SGI394764, SGI394765, SGI394766, SGI394767, SGI0, NULL};
const Segment * const SG98378[] = {SGI394768, SGI394769, SGI394770, SGI394771, SGI0, NULL};
const Segment * const SG98379[] = {SGI394772, SGI394773, SGI394774, SGI394775, SGI0, NULL};
const Segment * const SG98380[] = {SGI394776, SGI394777, SGI394778, SGI394779, SGI0, NULL};
const Segment * const SG98381[] = {SGI394780, SGI394781, SGI394782, SGI394783, SGI0, NULL};
const Segment * const SG98382[] = {SGI394784, SGI394785, SGI394786, SGI394787, SGI0, NULL};
const Segment * const SG98383[] = {SGI394788, SGI394789, SGI394790, SGI394791, SGI0, NULL};
const Segment * const SG98384[] = {SGI394792, SGI394793, SGI394794, SGI394795, SGI0, NULL};
const Segment * const SG98385[] = {SGI394796, SGI394797, SGI394798, SGI394799, SGI0, NULL};
const Segment * const SG98386[] = {SGI394800, SGI394801, SGI394802, SGI394803, SGI0, NULL};
const Segment * const SG98387[] = {SGI394804, SGI394805, SGI394806, SGI0, SGI394807, NULL};
const Segment * const SG98388[] = {SGI394808, SGI394809, SGI394810, SGI0, SGI394811, NULL};
const Segment * const SG98389[] = {SGI394812, SGI394813, SGI394814, SGI0, SGI394815, NULL};
const Segment * const SG98390[] = {SGI394816, SGI394817, SGI394818, SGI0, SGI394819, NULL};
const Segment * const SG98391[] = {SGI394820, SGI394821, SGI394822, SGI0, SGI394823, NULL};
const Segment * const SG98392[] = {SGI394824, SGI394825, SGI394826, SGI394827, SGI0, NULL};
const Segment * const SG98393[] = {SGI394828, SGI394829, SGI394830, SGI394831, SGI0, NULL};
const Segment * const SG98394[] = {SGI394832, SGI394833, SGI394834, SGI394835, SGI0, NULL};
const Segment * const SG98395[] = {SGI394836, SGI394837, SGI394838, SGI394839, SGI0, NULL};
const Segment * const SG98396[] = {SGI394840, SGI394841, SGI394842, SGI0, SGI394843, NULL};
const Segment * const SG98397[] = {SGI394844, SGI394845, SGI394846, SGI0, SGI394847, NULL};
const Segment * const SG98398[] = {SGI394848, SGI394849, SGI394850, SGI394851, SGI0, NULL};
const Segment * const SG98399[] = {SGI394852, SGI394853, SGI394854, SGI394855, SGI0, NULL};
const Segment * const SG98400[] = {SGI394856, SGI394857, SGI394858, SGI394859, SGI0, NULL};
const Segment * const SG98401[] = {SGI394860, SGI394861, SGI394862, SGI394863, SGI0, NULL};
const Segment * const SG98402[] = {SGI394864, SGI0, NULL};
const Segment * const SG98403[] = {SGI394865, SGI394866, SGI394867, SGI394868, SGI0, NULL};
const Segment * const SG98404[] = {SGI394869, SGI394870, SGI394871, SGI394872, SGI0, NULL};
const Segment * const SG98405[] = {SGI394873, SGI394874, SGI394875, SGI394876, SGI0, NULL};
const Segment * const SG98406[] = {SGI394877, SGI394878, SGI394879, SGI394880, SGI0, NULL};
const Segment * const SG98407[] = {SGI394881, SGI394882, SGI394883, SGI394884, SGI0, NULL};
const Segment * const SG98408[] = {SGI394885, SGI394886, SGI394887, SGI394888, SGI0, NULL};
const Segment * const SG98409[] = {SGI394889, SGI394890, SGI394891, SGI394892, SGI0, NULL};
const Segment * const SG98410[] = {SGI394893, SGI394894, SGI394895, SGI394896, SGI0, NULL};
const Segment * const SG98411[] = {SGI394897, SGI394898, SGI394899, SGI394900, SGI0, NULL};
const Segment * const SG98412[] = {SGI394901, SGI394902, SGI394903, SGI394904, SGI0, NULL};
const Segment * const SG98413[] = {SGI394905, SGI394906, SGI394907, SGI394908, SGI0, NULL};
const Segment * const SG98414[] = {SGI394909, SGI394910, SGI394911, SGI394912, SGI0, NULL};
const Segment * const SG98415[] = {SGI394913, SGI394914, SGI394915, SGI394916, SGI0, NULL};
const Segment * const SG98416[] = {SGI394917, SGI394918, SGI394919, SGI394920, SGI0, NULL};
const Segment * const SG98417[] = {SGI394921, SGI394922, SGI394923, SGI394924, SGI0, NULL};
const Segment * const SG98418[] = {SGI394925, SGI394926, SGI394927, SGI394928, SGI0, NULL};
const Segment * const SG98419[] = {SGI394929, SGI394930, SGI394931, SGI394932, SGI0, NULL};
const Segment * const SG98420[] = {SGI394933, SGI394934, SGI394935, SGI394936, SGI0, NULL};
const Segment * const SG98421[] = {SGI394937, SGI394938, SGI394939, SGI394940, SGI0, NULL};
const Segment * const SG98422[] = {SGI394941, SGI394942, SGI394943, SGI394944, SGI0, NULL};
const Segment * const SG98423[] = {SGI394945, SGI394946, SGI394947, SGI394948, SGI0, NULL};
const Segment * const SG98424[] = {SGI394949, SGI394950, SGI394951, SGI0, SGI394952, NULL};
const Segment * const SG98425[] = {SGI394953, SGI394954, SGI394955, SGI394956, SGI0, NULL};
const Segment * const SG98426[] = {SGI394957, SGI394958, SGI394959, SGI394960, SGI0, NULL};
const Segment * const SG98427[] = {SGI394961, SGI394962, SGI394963, SGI394964, SGI0, NULL};
const Segment * const SG98428[] = {SGI394965, SGI394966, SGI394967, SGI394968, SGI0, NULL};
const Segment * const SG98429[] = {SGI394969, SGI394970, SGI394971, SGI394972, SGI0, NULL};
const Segment * const SG98430[] = {SGI394973, SGI394974, SGI394975, SGI394976, SGI0, NULL};
const Segment * const SG98431[] = {SGI394977, SGI394978, SGI394979, SGI394980, SGI0, NULL};
const Segment * const SG98432[] = {SGI394981, SGI394982, SGI394983, SGI394984, SGI0, NULL};
const Segment * const SG98433[] = {SGI394985, SGI394986, SGI394987, SGI0, SGI394988, NULL};
const Segment * const SG98434[] = {SGI394989, SGI394990, SGI394991, SGI394992, SGI0, NULL};
const Segment * const SG98435[] = {SGI394993, SGI394994, SGI394995, SGI0, SGI394996, NULL};
const Segment * const SG98436[] = {SGI394997, SGI394998, SGI394999, SGI0, SGI395000, NULL};
const Segment * const SG98437[] = {SGI395001, SGI395002, SGI395003, SGI0, SGI395004, NULL};
const Segment * const SG98438[] = {SGI395005, SGI395006, SGI395007, SGI0, SGI395008, NULL};
const Segment * const SG98439[] = {SGI395009, SGI395010, SGI395011, SGI0, SGI395012, NULL};
const Segment * const SG98440[] = {SGI395013, SGI395014, SGI395015, SGI395016, SGI0, NULL};
const Segment * const SG98441[] = {SGI395017, SGI395018, SGI395019, SGI395020, SGI0, NULL};
const Segment * const SG98442[] = {SGI395021, SGI395022, SGI395023, SGI395024, SGI0, NULL};
const Segment * const SG98443[] = {SGI395025, SGI395026, SGI395027, SGI395028, SGI0, NULL};
const Segment * const SG98444[] = {SGI395029, SGI395030, SGI395031, SGI0, SGI0, NULL};
const Segment * const SG98445[] = {SGI395032, SGI395033, SGI395034, SGI0, SGI0, NULL};
const Segment * const SG98446[] = {SGI395035, SGI395036, SGI395037, SGI0, SGI0, NULL};
const Segment * const SG98447[] = {SGI395038, SGI395039, SGI395040, SGI0, SGI0, NULL};
const Segment * const SG98448[] = {SGI395041, SGI395042, SGI395043, SGI0, SGI0, NULL};
const Segment * const SG98449[] = {SGI395044, SGI395045, SGI395046, SGI0, SGI0, NULL};
const Segment * const SG98450[] = {SGI395047, SGI395048, SGI395049, SGI395050, SGI0, NULL};
const Segment * const SG98451[] = {SGI395051, SGI395052, SGI395053, SGI395054, SGI0, NULL};
const Segment * const SG98452[] = {SGI395055, SGI395056, SGI395057, SGI0, SGI395058, NULL};
const Segment * const SG98453[] = {SGI395059, SGI395060, SGI395061, SGI395062, SGI0, NULL};
const Segment * const SG98454[] = {SGI395063, SGI395064, SGI395065, SGI395066, SGI0, NULL};
const Segment * const SG98455[] = {SGI395067, SGI395068, SGI395069, SGI395070, SGI0, NULL};
const Segment * const SG98456[] = {SGI395071, SGI395072, SGI395073, SGI395074, SGI0, NULL};
const Segment * const SG98457[] = {SGI395075, SGI395076, SGI395077, SGI395078, SGI0, NULL};
const Segment * const SG98458[] = {SGI395079, SGI395080, SGI395081, SGI395082, SGI0, NULL};
const Segment * const SG98459[] = {SGI395083, SGI395084, SGI395085, SGI395086, SGI0, NULL};
const Segment * const SG98460[] = {SGI395087, SGI395088, SGI395089, SGI395090, SGI0, NULL};
const Segment * const SG98461[] = {SGI395091, SGI395092, SGI395093, SGI395094, SGI0, NULL};
const Segment * const SG98462[] = {SGI395095, SGI395096, SGI395097, SGI395098, SGI0, NULL};
const Segment * const SG98463[] = {SGI395099, SGI395100, SGI395101, SGI395102, SGI0, NULL};
const Segment * const SG98464[] = {SGI395103, SGI395104, SGI395105, SGI395106, SGI0, NULL};
const Segment * const SG98465[] = {SGI395107, SGI395108, SGI395109, SGI395110, SGI0, NULL};
const Segment * const SG98466[] = {SGI395111, SGI395112, SGI395113, SGI395114, SGI0, NULL};
const Segment * const SG98467[] = {SGI395115, SGI395116, SGI395117, SGI395118, SGI0, NULL};
const Segment * const SG98468[] = {SGI395119, SGI395120, SGI395121, SGI395122, SGI0, NULL};
const Segment * const SG98469[] = {SGI395123, SGI395124, SGI395125, SGI395126, SGI0, NULL};
const Segment * const SG98470[] = {SGI395127, SGI395128, SGI395129, SGI0, SGI395130, NULL};
const Segment * const SG98471[] = {SGI395131, SGI395132, SGI395133, SGI0, SGI395134, NULL};
const Segment * const SG98472[] = {SGI395135, SGI395136, SGI395137, SGI0, SGI395138, NULL};
const Segment * const SG98473[] = {SGI395139, SGI395140, SGI395141, SGI395142, SGI0, NULL};
const Segment * const SG98474[] = {SGI395143, NULL};
const Segment * const SG98475[] = {SGI395144, NULL};
const Segment * const SG98476[] = {SGI395145, NULL};
const Segment * const SG98477[] = {SGI395146, NULL};
const Segment * const SG98478[] = {SGI395147, NULL};
const Segment * const SG98479[] = {SGI395148, NULL};
const Segment * const SG98480[] = {SGI395149, NULL};
const Segment * const SG98481[] = {SGI395150, NULL};
const Segment * const SG98482[] = {SGI395151, NULL};
const Segment * const SG98483[] = {SGI395152, NULL};
const Segment * const SG98484[] = {SGI395153, NULL};
const Segment * const SG98485[] = {SGI395154, NULL};
const Segment * const SG98486[] = {SGI395155, NULL};
const Segment * const SG98487[] = {SGI395156, NULL};
const Segment * const SG98488[] = {SGI395157, NULL};
const Segment * const SG98489[] = {SGI395158, NULL};
const Segment * const SG98490[] = {SGI395159, NULL};
const Segment * const SG98491[] = {SGI395160, NULL};
const Segment * const SG98492[] = {SGI395161, NULL};
const Segment * const SG98493[] = {SGI395162, NULL};
const Segment * const SG98494[] = {SGI395163, NULL};
const Segment * const SG98495[] = {SGI395164, NULL};
const Segment * const SG98496[] = {SGI395165, NULL};
const Segment * const SG98497[] = {SGI395166, NULL};
const Segment * const SG98498[] = {SGI395167, NULL};
const Segment * const SG98499[] = {SGI395168, NULL};
const Segment * const SG98500[] = {SGI395169, NULL};
const Segment * const SG98501[] = {SGI395170, NULL};
const Segment * const SG98502[] = {SGI395171, NULL};
const Segment * const SG98503[] = {SGI395172, NULL};
const Segment * const SG98504[] = {SGI395173, NULL};
const Segment * const SG98505[] = {SGI395174, NULL};
const Segment * const SG98506[] = {SGI395175, NULL};
const Segment * const SG98507[] = {SGI395176, NULL};
const Segment * const SG98508[] = {SGI395177, NULL};
const Segment * const SG98509[] = {SGI395178, NULL};
const Segment * const SG98510[] = {SGI395179, NULL};
const Segment * const SG98511[] = {SGI395180, NULL};
const Segment * const SG98512[] = {SGI395181, NULL};
const Segment * const SG98513[] = {SGI395182, NULL};
const Segment * const SG98514[] = {SGI395183, NULL};
const Segment * const SG98515[] = {SGI395184, NULL};
const Segment * const SG98516[] = {SGI395185, NULL};
const Segment * const SG98517[] = {SGI395186, NULL};
const Segment * const SG98518[] = {SGI395187, NULL};
const Segment * const SG98519[] = {SGI395188, NULL};
const Segment * const SG98520[] = {SGI395189, NULL};
const Segment * const SG98521[] = {SGI395190, NULL};
const Segment * const SG98522[] = {SGI395191, NULL};
const Segment * const SG98523[] = {SGI395192, NULL};
const Segment * const SG98524[] = {SGI395193, NULL};
const Segment * const SG98525[] = {SGI395194, NULL};
const Segment * const SG98526[] = {SGI395195, NULL};
const Segment * const SG98527[] = {SGI395196, NULL};
const Segment * const SG98528[] = {SGI395197, NULL};
const Segment * const SG98529[] = {SGI395198, NULL};
const Segment * const SG98530[] = {SGI395199, NULL};
const Segment * const SG98531[] = {SGI395200, NULL};
const Segment * const SG98532[] = {SGI395201, NULL};
const Segment * const SG98533[] = {SGI395202, NULL};
const Segment * const SG98534[] = {SGI395203, NULL};
const Segment * const SG98535[] = {SGI395204, NULL};
const Segment * const SG98536[] = {SGI395205, NULL};
const Segment * const SG98537[] = {SGI395206, NULL};
const Segment * const SG98538[] = {SGI395207, NULL};
const Segment * const SG98539[] = {SGI395208, NULL};
const Segment * const SG98540[] = {SGI395209, NULL};
const Segment * const SG98541[] = {SGI395210, NULL};
const Segment * const SG98542[] = {SGI395211, NULL};
const Segment * const SG98543[] = {SGI395212, NULL};
const Segment * const SG98544[] = {SGI395213, NULL};
const Segment * const SG98545[] = {SGI395214, NULL};
const Segment * const SG98546[] = {SGI395215, NULL};
const Segment * const SG98547[] = {SGI395216, NULL};
const Segment * const SG98548[] = {SGI395217, NULL};
const Segment * const SG98549[] = {SGI395218, NULL};
const Segment * const SG98550[] = {SGI395219, SGI395220, SGI395221, SGI0, SGI0, NULL};
const Segment * const SG98551[] = {SGI395222, SGI395223, SGI395224, SGI395225, SGI0, NULL};
const Segment * const SG98552[] = {SGI395226, SGI395227, SGI395228, SGI395229, SGI0, NULL};
const Segment * const SG98553[] = {SGI395230, SGI395231, SGI395232, SGI395233, SGI0, NULL};
const Segment * const SG98554[] = {SGI395234, SGI395235, SGI395236, SGI395237, SGI0, NULL};
const Segment * const SG98555[] = {SGI395238, SGI395239, SGI395240, SGI395241, SGI0, NULL};
const Segment * const SG98556[] = {SGI395242, SGI395243, SGI395244, SGI395245, SGI0, NULL};
const Segment * const SG98557[] = {SGI395246, SGI395247, SGI395248, SGI395249, SGI0, NULL};
const Segment * const SG98558[] = {SGI395250, SGI395251, SGI395252, SGI0, SGI0, NULL};
const Segment * const SG98559[] = {SGI395253, SGI395254, SGI395255, SGI395256, SGI0, NULL};
const Segment * const SG98560[] = {SGI395257, SGI395258, SGI395259, SGI0, SGI395260, NULL};
const Segment * const SG98561[] = {SGI395261, SGI395262, SGI395263, SGI395264, SGI0, NULL};
const Segment * const SG98562[] = {SGI395265, SGI395266, SGI395267, SGI395268, SGI0, NULL};
const Segment * const SG98563[] = {SGI395269, SGI395270, SGI395271, SGI395272, SGI0, NULL};
const Segment * const SG98564[] = {SGI395273, SGI395274, SGI395275, SGI395276, SGI0, NULL};
const Segment * const SG98565[] = {SGI395277, SGI395278, SGI395279, SGI395280, SGI0, NULL};
const Segment * const SG98566[] = {SGI395281, SGI395282, SGI395283, SGI395284, SGI0, NULL};
const Segment * const SG98567[] = {SGI395285, SGI395286, SGI395287, SGI395288, SGI0, NULL};
const Segment * const SG98568[] = {SGI395289, SGI395290, SGI395291, SGI395292, SGI0, NULL};
const Segment * const SG98569[] = {SGI395293, SGI0, SGI0, NULL};
const Segment * const SG98570[] = {SGI395294, SGI0, SGI0, NULL};
const Segment * const SG98571[] = {SGI395295, SGI0, SGI0, NULL};
const Segment * const SG98572[] = {SGI395296, SGI395297, NULL};
const Segment * const SG98573[] = {SGI395298, SGI395299, NULL};
const Segment * const SG98574[] = {SGI395300, SGI395301, NULL};
const Segment * const SG98575[] = {SGI395302, SGI395303, SGI395304, SGI395305, SGI0, NULL};
const Segment * const SG98576[] = {SGI395306, SGI395307, SGI395308, SGI395309, SGI0, NULL};
const Segment * const SG98577[] = {SGI395310, SGI395311, SGI395312, SGI395313, SGI0, NULL};
const Segment * const SG98578[] = {SGI395314, SGI395315, SGI395316, SGI395317, SGI0, NULL};
const Segment * const SG98579[] = {SGI395318, SGI395319, SGI395320, SGI395321, SGI0, NULL};
const Segment * const SG98580[] = {SGI395322, SGI395323, SGI395324, SGI395325, SGI0, NULL};
const Segment * const SG98581[] = {SGI395326, SGI395327, SGI395328, SGI0, SGI0, NULL};
const Segment * const SG98582[] = {SGI395329, SGI395330, SGI395331, SGI395332, SGI0, NULL};
const Segment * const SG98583[] = {SGI395333, SGI395334, SGI395335, SGI395336, SGI0, NULL};
const Segment * const SG98584[] = {SGI395337, SGI395338, SGI395339, SGI395340, SGI0, NULL};
const Segment * const SG98585[] = {SGI395341, SGI395342, SGI395343, SGI395344, SGI0, NULL};
const Segment * const SG98586[] = {SGI395345, SGI395346, SGI395347, SGI395348, SGI0, NULL};
const Segment * const SG98587[] = {SGI395349, SGI395350, SGI395351, SGI395352, SGI0, NULL};
const Segment * const SG98588[] = {SGI395353, SGI395354, SGI395355, SGI395356, SGI0, NULL};
const Segment * const SG98589[] = {SGI395357, SGI395358, SGI395359, SGI395360, SGI0, NULL};
const Segment * const SG98590[] = {SGI395361, SGI395362, SGI395363, SGI395364, SGI0, NULL};
const Segment * const SG98591[] = {SGI395365, SGI395366, SGI395367, SGI0, SGI0, NULL};
const Segment * const SG98592[] = {SGI395368, SGI395369, SGI395370, SGI395371, SGI0, NULL};
const Segment * const SG98593[] = {SGI395372, SGI395373, SGI395374, SGI0, SGI0, NULL};
const Segment * const SG98594[] = {SGI395375, SGI395376, SGI395377, SGI395378, SGI0, NULL};
const Segment * const SG98595[] = {SGI395379, SGI395380, SGI395381, SGI395382, SGI0, NULL};
const Segment * const SG98596[] = {SGI395383, SGI395384, SGI395385, SGI0, SGI395386, NULL};
const Segment * const SG98597[] = {SGI395387, SGI395388, SGI395389, SGI0, SGI395390, NULL};
const Segment * const SG98598[] = {SGI395391, SGI395392, SGI395393, SGI0, SGI395394, NULL};
const Segment * const SG98599[] = {SGI395395, SGI395396, SGI395397, SGI0, SGI395398, NULL};
const Segment * const SG98600[] = {SGI395399, SGI395400, SGI395401, SGI395402, SGI0, NULL};
const Segment * const SG98601[] = {SGI395403, SGI395404, SGI395405, SGI395406, SGI0, NULL};
const Segment * const SG98602[] = {SGI395407, SGI395408, SGI395409, SGI395410, SGI0, NULL};
const Segment * const SG98603[] = {SGI395411, SGI395412, SGI395413, SGI395414, SGI0, NULL};
const Segment * const SG98604[] = {SGI395415, SGI395416, SGI395417, SGI395418, SGI0, NULL};
const Segment * const SG98605[] = {SGI395419, SGI395420, SGI395421, SGI0, SGI0, NULL};
const Segment * const SG98606[] = {SGI395422, SGI395423, SGI395424, SGI395425, SGI0, NULL};
const Segment * const SG98607[] = {SGI395426, SGI395427, SGI395428, SGI395429, SGI0, NULL};
const Segment * const SG98608[] = {SGI395430, SGI0, SGI395431, SGI395432, SGI0, NULL};
const Segment * const SG98609[] = {SGI395433, SGI395434, SGI395435, SGI395436, SGI0, NULL};
const Segment * const SG98610[] = {SGI395437, SGI395438, SGI395439, SGI395440, SGI0, NULL};
const Segment * const SG98611[] = {SGI395441, SGI395442, SGI395443, SGI395444, SGI0, NULL};
const Segment * const SG98612[] = {SGI395445, SGI395446, SGI395447, SGI395448, SGI0, NULL};
const Segment * const SG98613[] = {SGI395449, SGI395450, SGI395451, SGI395452, SGI0, NULL};
const Segment * const SG98614[] = {SGI395453, SGI395454, SGI395455, SGI395456, SGI0, NULL};
const Segment * const SG98615[] = {SGI395457, SGI395458, SGI395459, SGI395460, SGI0, NULL};
const Segment * const SG98616[] = {SGI395461, SGI395462, SGI395463, SGI395464, SGI0, NULL};
const Segment * const SG98617[] = {SGI395465, SGI395466, SGI395467, SGI395468, SGI0, NULL};
const Segment * const SG98618[] = {SGI395469, SGI395470, SGI395471, SGI395472, SGI0, NULL};
const Segment * const SG98619[] = {SGI395473, SGI395474, SGI395475, SGI395476, SGI0, NULL};
const Segment * const SG98620[] = {SGI395477, SGI395478, SGI395479, SGI395480, SGI0, NULL};
const Segment * const SG98621[] = {SGI395481, SGI395482, SGI395483, SGI395484, SGI0, NULL};
const Segment * const SG98622[] = {SGI395485, SGI395486, SGI395487, SGI395488, SGI0, NULL};
const Segment * const SG98623[] = {SGI395489, SGI395490, SGI395491, SGI395492, SGI0, NULL};
const Segment * const SG98624[] = {SGI395493, SGI395494, SGI395495, SGI395496, SGI0, NULL};
const Segment * const SG98625[] = {SGI395497, SGI395498, SGI395499, SGI395500, SGI0, NULL};
const Segment * const SG98626[] = {SGI395501, SGI395502, SGI395503, SGI395504, SGI0, NULL};
const Segment * const SG98627[] = {SGI395505, SGI395506, SGI395507, SGI395508, SGI0, NULL};
const Segment * const SG98628[] = {SGI395509, SGI395510, SGI395511, SGI395512, SGI0, NULL};
const Segment * const SG98629[] = {SGI395513, SGI395514, SGI395515, SGI395516, SGI0, NULL};
const Segment * const SG98630[] = {SGI395517, SGI395518, SGI395519, SGI395520, SGI0, NULL};
const Segment * const SG98631[] = {SGI395521, SGI395522, SGI395523, SGI395524, SGI0, NULL};
const Segment * const SG98632[] = {SGI395525, SGI395526, SGI395527, SGI395528, SGI0, NULL};
const Segment * const SG98633[] = {SGI395529, SGI395530, SGI395531, SGI395532, SGI0, NULL};
const Segment * const SG98634[] = {SGI395533, SGI395534, SGI395535, SGI395536, SGI0, NULL};
const Segment * const SG98635[] = {SGI395537, SGI395538, SGI395539, SGI395540, SGI0, NULL};
const Segment * const SG98636[] = {SGI395541, SGI395542, SGI395543, SGI0, SGI0, NULL};
const Segment * const SG98637[] = {SGI395544, SGI395545, SGI395546, SGI395547, SGI0, NULL};
const Segment * const SG98638[] = {SGI395548, SGI395549, SGI395550, SGI395551, SGI0, NULL};
const Segment * const SG98639[] = {SGI395552, SGI395553, NULL};
const Segment * const SG98640[] = {SGI395554, SGI395555, NULL};
const Segment * const SG98641[] = {SGI395556, SGI395557, SGI395558, SGI395559, SGI0, NULL};
const Segment * const SG98642[] = {SGI395560, SGI395561, SGI395562, SGI395563, SGI0, NULL};
const Segment * const SG98643[] = {SGI395564, SGI395565, SGI395566, SGI395567, SGI0, NULL};
const Segment * const SG98644[] = {SGI395568, SGI395569, SGI395570, SGI395571, SGI0, NULL};
const Segment * const SG98645[] = {SGI395572, SGI395573, SGI395574, SGI395575, SGI0, NULL};
const Segment * const SG98646[] = {SGI395576, SGI395577, SGI395578, SGI395579, SGI0, NULL};
const Segment * const SG98647[] = {SGI395580, SGI395581, SGI395582, SGI395583, SGI0, NULL};
const Segment * const SG98648[] = {SGI395584, SGI395585, SGI395586, SGI395587, SGI0, NULL};
const Segment * const SG98649[] = {SGI395588, SGI395589, SGI395590, SGI395591, SGI0, NULL};
const Segment * const SG98650[] = {SGI395592, SGI395593, SGI395594, SGI395595, SGI0, NULL};
const Segment * const SG98651[] = {SGI395596, SGI395597, SGI395598, SGI0, SGI0, NULL};
const Segment * const SG98652[] = {SGI395599, SGI395600, SGI395601, SGI395602, SGI0, NULL};
const Segment * const SG98653[] = {SGI395603, SGI395604, NULL};
const Segment * const SG98654[] = {SGI395605, SGI395606, SGI395607, SGI395608, SGI0, NULL};
const Segment * const SG98655[] = {SGI395609, SGI395610, SGI395611, SGI395612, SGI0, NULL};
const Segment * const SG98656[] = {SGI395613, SGI395614, SGI395615, SGI395616, SGI0, NULL};
const Segment * const SG98657[] = {SGI395617, SGI395618, SGI395619, SGI0, SGI0, NULL};
const Segment * const SG98658[] = {SGI395620, SGI395621, SGI395622, SGI395623, SGI0, NULL};
const Segment * const SG98659[] = {SGI395624, SGI395625, SGI395626, SGI395627, SGI0, NULL};
const Segment * const SG98660[] = {SGI395628, SGI395629, SGI395630, SGI395631, SGI0, NULL};
const Segment * const SG98661[] = {SGI395632, SGI395633, SGI395634, SGI395635, SGI0, NULL};
const Segment * const SG98662[] = {SGI395636, SGI395637, SGI395638, SGI395639, SGI0, NULL};
const Segment * const SG98663[] = {SGI395640, SGI395641, SGI395642, SGI0, SGI395643, NULL};
const Segment * const SG98664[] = {SGI395644, SGI395645, SGI395646, SGI0, SGI395647, NULL};
const Segment * const SG98665[] = {SGI395648, SGI395649, SGI395650, SGI0, SGI395651, NULL};
const Segment * const SG98666[] = {SGI395652, SGI0, SGI0, NULL};
const Segment * const SG98667[] = {SGI395653, SGI0, SGI0, NULL};
const Segment * const SG98668[] = {SGI395654, SGI0, SGI0, NULL};
const Segment * const SG98669[] = {SGI395655, SGI0, SGI0, NULL};
const Segment * const SG98670[] = {SGI395656, SGI0, SGI0, NULL};
const Segment * const SG98671[] = {SGI395657, SGI395658, SGI395659, SGI395660, SGI0, NULL};
const Segment * const SG98672[] = {SGI395661, SGI395662, SGI395663, SGI395664, SGI0, NULL};
const Segment * const SG98673[] = {SGI395665, SGI395666, SGI395667, SGI395668, SGI0, NULL};
const Segment * const SG98674[] = {SGI395669, SGI395670, SGI395671, SGI395672, SGI0, NULL};
const Segment * const SG98675[] = {SGI395673, SGI395674, SGI395675, SGI395676, SGI0, NULL};
const Segment * const SG98676[] = {SGI395677, SGI395678, SGI395679, SGI395680, SGI0, NULL};
const Segment * const SG98677[] = {SGI395681, SGI395682, SGI395683, SGI395684, SGI0, NULL};
const Segment * const SG98678[] = {SGI395685, SGI395686, SGI395687, SGI395688, SGI0, NULL};
const Segment * const SG98679[] = {SGI395689, SGI395690, SGI395691, SGI0, SGI0, NULL};
const Segment * const SG98680[] = {SGI395692, SGI395693, SGI395694, SGI395695, SGI0, NULL};
const Segment * const SG98681[] = {SGI395696, SGI395697, SGI395698, SGI0, SGI0, NULL};
const Segment * const SG98682[] = {SGI395699, SGI395700, SGI395701, SGI0, SGI0, NULL};
const Segment * const SG98683[] = {SGI395702, SGI395703, SGI0, SGI0, SGI0, NULL};
const Segment * const SG98684[] = {SGI395704, SGI395705, SGI395706, SGI395707, SGI0, NULL};
const Segment * const SG98685[] = {SGI395708, SGI395709, SGI395710, SGI395711, SGI0, NULL};
const Segment * const SG98686[] = {SGI395712, SGI0, SGI395713, SGI0, SGI0, NULL};
const Segment * const SG98687[] = {SGI395714, SGI395715, SGI395716, SGI395717, SGI0, NULL};
const Segment * const SG98688[] = {SGI395718, SGI395719, SGI395720, SGI395721, SGI0, NULL};
const Segment * const SG98689[] = {SGI395722, SGI395723, SGI395724, SGI0, SGI0, NULL};
const Segment * const SG98690[] = {SGI395725, SGI395726, NULL};
const Segment * const SG98691[] = {SGI395727, SGI395728, SGI395729, SGI395730, SGI0, NULL};
const Segment * const SG98692[] = {SGI395731, SGI395732, SGI395733, SGI395734, SGI0, NULL};
const Segment * const SG98693[] = {SGI395735, SGI395736, SGI395737, SGI395738, SGI0, NULL};
const Segment * const SG98694[] = {SGI395739, SGI395740, SGI395741, SGI395742, SGI0, NULL};
const Segment * const SG98695[] = {SGI395743, SGI395744, SGI395745, SGI0, SGI0, NULL};
const Segment * const SG98696[] = {SGI395746, SGI395747, SGI395748, SGI0, SGI0, NULL};
const Segment * const SG98697[] = {SGI395749, SGI395750, SGI395751, SGI0, SGI0, NULL};
const Segment * const SG98698[] = {SGI395752, SGI395753, SGI395754, SGI395755, SGI0, NULL};
const Segment * const SG98699[] = {SGI395756, SGI395757, SGI395758, SGI395759, SGI0, NULL};
const Segment * const SG98700[] = {SGI395760, SGI395761, SGI395762, SGI395763, SGI0, NULL};
const Segment * const SG98701[] = {SGI395764, SGI395765, SGI395766, SGI395767, SGI0, NULL};
const Segment * const SG98702[] = {SGI395768, SGI395769, SGI395770, SGI395771, SGI0, NULL};
const Segment * const SG98703[] = {SGI395772, SGI395773, SGI395774, SGI0, SGI395775, NULL};
const Segment * const SG98704[] = {SGI395776, SGI0, SGI395777, SGI395778, SGI0, NULL};
const Segment * const SG98705[] = {SGI395779, SGI395780, SGI395781, SGI395782, SGI0, NULL};
const Segment * const SG98706[] = {SGI395783, SGI395784, SGI395785, SGI395786, SGI0, NULL};
const Segment * const SG98707[] = {SGI395787, SGI395788, SGI395789, SGI395790, SGI0, NULL};
const Segment * const SG98708[] = {SGI395791, SGI395792, SGI395793, SGI0, SGI395794, NULL};
const Segment * const SG98709[] = {SGI395795, SGI395796, SGI395797, SGI0, SGI395798, NULL};
const Segment * const SG98710[] = {SGI395799, SGI395800, SGI395801, SGI395802, SGI0, NULL};
const Segment * const SG98711[] = {SGI395803, SGI395804, SGI395805, SGI395806, SGI0, NULL};
const Segment * const SG98712[] = {SGI395807, SGI395808, SGI0, SGI0, SGI0, NULL};
const Segment * const SG98713[] = {SGI395809, SGI395810, SGI395811, SGI395812, SGI0, NULL};
const Segment * const SG98714[] = {SGI395813, SGI395814, SGI395815, SGI395816, SGI0, NULL};
const Segment * const SG98715[] = {SGI395817, SGI395818, SGI395819, SGI0, SGI0, NULL};
const Segment * const SG98716[] = {SGI395820, SGI395821, SGI395822, SGI0, SGI395823, NULL};
const Segment * const SG98717[] = {SGI395824, SGI395825, SGI395826, SGI395827, SGI0, NULL};
const Segment * const SG98718[] = {SGI395828, SGI395829, SGI395830, SGI0, SGI395831, NULL};
const Segment * const SG98719[] = {SGI395832, SGI395833, SGI395834, SGI395835, SGI0, NULL};
const Segment * const SG98720[] = {SGI395836, SGI395837, SGI395838, SGI395839, SGI0, NULL};
const Segment * const SG98721[] = {SGI395840, SGI395841, SGI395842, SGI395843, SGI0, NULL};
const Segment * const SG98722[] = {SGI395844, SGI395845, SGI395846, SGI395847, SGI0, NULL};
const Segment * const SG98723[] = {SGI395848, SGI395849, SGI395850, SGI395851, SGI0, NULL};
const Segment * const SG98724[] = {SGI395852, SGI395853, SGI395854, SGI395855, SGI0, NULL};
const Segment * const SG98725[] = {SGI395856, SGI395857, SGI395858, SGI395859, SGI0, NULL};
const Segment * const SG98726[] = {SGI395860, SGI395861, SGI395862, SGI0, SGI395863, NULL};
const Segment * const SG98727[] = {SGI395864, SGI395865, SGI395866, SGI0, SGI395867, NULL};
const Segment * const SG98728[] = {SGI395868, SGI395869, SGI395870, SGI395871, SGI0, NULL};
const Segment * const SG98729[] = {SGI395872, SGI395873, SGI0, SGI0, SGI0, NULL};
const Segment * const SG98730[] = {SGI395874, SGI395875, SGI395876, SGI395877, SGI0, NULL};
const Segment * const SG98731[] = {SGI395878, SGI395879, SGI395880, SGI395881, SGI0, NULL};
const Segment * const SG98732[] = {SGI395882, SGI395883, SGI395884, SGI395885, SGI0, NULL};
const Segment * const SG98733[] = {SGI395886, SGI395887, SGI395888, SGI395889, SGI0, NULL};
const Segment * const SG98734[] = {SGI395890, SGI395891, SGI395892, SGI0, SGI395893, NULL};
const Segment * const SG98735[] = {SGI395894, SGI395895, SGI395896, SGI395897, SGI0, NULL};
const Segment * const SG98736[] = {SGI395898, SGI0, SGI395899, SGI395900, SGI0, NULL};
const Segment * const SG98737[] = {SGI395901, SGI395902, SGI395903, SGI395904, SGI0, NULL};
const Segment * const SG98738[] = {SGI395905, SGI395906, SGI395907, SGI0, SGI395908, NULL};
const Segment * const SG98739[] = {SGI395909, SGI395910, SGI395911, SGI0, SGI395912, NULL};
const Segment * const SG98740[] = {SGI395913, SGI395914, SGI395915, SGI0, SGI395916, NULL};
const Segment * const SG98741[] = {SGI395917, SGI395918, SGI395919, SGI0, SGI0, NULL};
const Segment * const SG98742[] = {SGI395920, SGI395921, SGI395922, SGI0, SGI0, NULL};
const Segment * const SG98743[] = {SGI395923, SGI395924, SGI395925, SGI395926, SGI0, NULL};
const Segment * const SG98744[] = {SGI395927, SGI395928, SGI395929, SGI395930, SGI0, NULL};
const Segment * const SG98745[] = {SGI395931, SGI395932, SGI395933, SGI0, SGI0, NULL};
const Segment * const SG98746[] = {SGI395934, SGI395935, SGI395936, SGI395937, SGI0, NULL};
const Segment * const SG98747[] = {SGI395938, SGI395939, SGI395940, SGI395941, SGI0, NULL};
const Segment * const SG98748[] = {SGI395942, SGI395943, SGI395944, SGI395945, SGI0, NULL};
const Segment * const SG98749[] = {SGI395946, SGI395947, SGI395948, SGI395949, SGI0, NULL};
const Segment * const SG98750[] = {SGI395950, SGI395951, SGI395952, SGI395953, SGI0, NULL};
const Segment * const SG98751[] = {SGI395954, SGI395955, SGI395956, SGI395957, SGI0, NULL};
const Segment * const SG98752[] = {SGI395958, SGI395959, SGI395960, SGI395961, SGI0, NULL};
const Segment * const SG98753[] = {SGI395962, SGI395963, SGI395964, SGI395965, SGI0, NULL};
const Segment * const SG98754[] = {SGI395966, SGI395967, SGI395968, SGI395969, SGI0, NULL};
const Segment * const SG98755[] = {SGI395970, SGI395971, SGI395972, SGI395973, SGI0, NULL};
const Segment * const SG98756[] = {SGI395974, SGI395975, SGI395976, SGI395977, SGI0, NULL};
const Segment * const SG98757[] = {SGI395978, SGI395979, SGI395980, SGI395981, SGI0, NULL};
const Segment * const SG98758[] = {SGI395982, SGI395983, SGI395984, SGI395985, SGI0, NULL};
const Segment * const SG98759[] = {SGI395986, SGI395987, SGI395988, SGI0, SGI395989, NULL};
const Segment * const SG98760[] = {SGI395990, SGI395991, SGI395992, SGI395993, SGI0, NULL};
const Segment * const SG98761[] = {SGI395994, SGI395995, SGI395996, SGI395997, SGI0, NULL};
const Segment * const SG98762[] = {SGI395998, SGI395999, SGI396000, SGI0, SGI0, NULL};
const Segment * const SG98763[] = {SGI396001, SGI396002, SGI396003, SGI0, SGI0, NULL};
const Segment * const SG98764[] = {SGI396004, SGI396005, SGI396006, SGI396007, SGI0, NULL};
const Segment * const SG98765[] = {SGI396008, SGI396009, SGI396010, SGI396011, SGI0, NULL};
const Segment * const SG98766[] = {SGI396012, SGI396013, SGI396014, SGI0, SGI0, NULL};
const Segment * const SG98767[] = {SGI396015, SGI396016, SGI396017, SGI0, SGI0, NULL};
const Segment * const SG98768[] = {SGI396018, SGI396019, SGI396020, SGI0, SGI0, NULL};
const Segment * const SG98769[] = {SGI396021, SGI396022, SGI396023, SGI396024, SGI0, NULL};
const Segment * const SG98770[] = {SGI396025, SGI396026, SGI396027, SGI396028, SGI0, NULL};
const Segment * const SG98771[] = {SGI396029, SGI396030, SGI396031, SGI396032, SGI0, NULL};
const Segment * const SG98772[] = {SGI396033, SGI396034, SGI396035, SGI0, SGI0, NULL};
const Segment * const SG98773[] = {SGI396036, SGI396037, SGI396038, SGI0, SGI0, NULL};
const Segment * const SG98774[] = {SGI396039, SGI0, SGI396040, SGI0, SGI0, NULL};
const Segment * const SG98775[] = {SGI396041, SGI396042, SGI396043, SGI396044, SGI0, NULL};
const Segment * const SG98776[] = {SGI396045, SGI396046, SGI396047, SGI0, SGI0, NULL};
const Segment * const SG98777[] = {SGI396048, SGI396049, NULL};
const Segment * const SG98778[] = {SGI396050, SGI396051, SGI396052, SGI396053, SGI0, NULL};
const Segment * const SG98779[] = {SGI396054, SGI396055, SGI396056, SGI0, SGI0, NULL};
const Segment * const SG98780[] = {SGI396057, SGI396058, SGI396059, SGI396060, SGI0, NULL};
const Segment * const SG98781[] = {SGI396061, SGI396062, SGI396063, SGI396064, SGI0, NULL};
const Segment * const SG98782[] = {SGI396065, SGI396066, SGI396067, SGI396068, SGI0, NULL};
const Segment * const SG98783[] = {SGI396069, SGI396070, SGI396071, SGI396072, SGI0, NULL};
const Segment * const SG98784[] = {SGI396073, SGI396074, SGI396075, SGI396076, SGI0, NULL};
const Segment * const SG98785[] = {SGI396077, SGI396078, SGI396079, SGI396080, SGI0, NULL};
const Segment * const SG98786[] = {SGI396081, SGI396082, SGI396083, SGI396084, SGI0, NULL};
const Segment * const SG98787[] = {SGI396085, SGI396086, SGI396087, SGI396088, SGI0, NULL};
const Segment * const SG98788[] = {SGI396089, SGI0, SGI0, NULL};
const Segment * const SG98789[] = {SGI396090, SGI396091, SGI396092, SGI396093, SGI0, NULL};
const Segment * const SG98790[] = {SGI396094, SGI396095, NULL};
const Segment * const SG98791[] = {SGI396096, SGI396097, NULL};
const Segment * const SG98792[] = {SGI396098, SGI396099, SGI0, NULL};
const Segment * const SG98793[] = {SGI396100, SGI0, SGI0, NULL};
const Segment * const SG98794[] = {SGI396101, SGI396102, SGI0, NULL};
const Segment * const SG98795[] = {SGI396103, SGI0, SGI0, NULL};
const Segment * const SG98796[] = {SGI396104, SGI0, SGI0, NULL};
const Segment * const SG98797[] = {SGI396105, SGI0, SGI0, NULL};
const Segment * const SG98798[] = {SGI396106, SGI396107, SGI0, NULL};
const Segment * const SG98799[] = {SGI396108, SGI396109, SGI0, NULL};
const Segment * const SG98800[] = {SGI396110, SGI0, SGI0, NULL};
const Segment * const SG98801[] = {SGI396111, SGI396112, SGI0, NULL};
const Segment * const SG98802[] = {SGI396113, SGI396114, NULL};
const Segment * const SG98803[] = {SGI396115, SGI396116, NULL};
const Segment * const SG98804[] = {SGI396117, SGI0, SGI0, NULL};
const Segment * const SG98805[] = {SGI396118, SGI0, SGI0, NULL};
const Segment * const SG98806[] = {SGI396119, SGI0, SGI0, NULL};
const Segment * const SG98807[] = {SGI396120, NULL};
const Segment * const SG98808[] = {SGI396121, SGI0, SGI0, NULL};
const Segment * const SG98809[] = {SGI396122, SGI0, SGI0, NULL};
const Segment * const SG98810[] = {SGI396123, SGI0, SGI0, NULL};
const Segment * const SG98811[] = {SGI396124, SGI0, SGI0, NULL};
const Segment * const SG98812[] = {SGI396125, SGI396126, SGI0, NULL};
const Segment * const SG98813[] = {SGI396127, SGI396128, SGI0, NULL};
const Segment * const SG98814[] = {SGI396129, SGI0, SGI0, NULL};
const Segment * const SG98815[] = {SGI396130, SGI0, SGI0, NULL};
const Segment * const SG98816[] = {SGI396131, NULL};
const Segment * const SG98817[] = {SGI396132, NULL};
const Segment * const SG98818[] = {SGI396133, NULL};
const Segment * const SG98819[] = {SGI396134, NULL};
const Segment * const SG98820[] = {SGI396135, NULL};
const Segment * const SG98821[] = {SGI396136, NULL};
const Segment * const SG98822[] = {SGI396137, NULL};
const Segment * const SG98823[] = {SGI396138, NULL};
const Segment * const SG98824[] = {SGI396139, NULL};
const Segment * const SG98825[] = {SGI396140, SGI0, SGI0, NULL};
const Segment * const SG98826[] = {SGI396141, SGI0, SGI0, NULL};
const Segment * const SG98827[] = {SGI396142, SGI0, SGI0, NULL};
const Segment * const SG98828[] = {SGI396143, SGI0, SGI0, NULL};
const Segment * const SG98829[] = {SGI396144, SGI0, SGI396145, NULL};
const Segment * const SG98830[] = {SGI396146, SGI0, SGI0, NULL};
const Segment * const SG98831[] = {SGI396147, SGI0, SGI0, NULL};
const Segment * const SG98832[] = {SGI396148, SGI0, SGI0, NULL};
const Segment * const SG98833[] = {SGI396149, SGI0, SGI0, NULL};
const Segment * const SG98834[] = {SGI396150, SGI0, SGI0, NULL};
const Segment * const SG98835[] = {SGI396151, SGI0, SGI0, NULL};
const Segment * const SG98836[] = {SGI396152, SGI0, SGI0, NULL};
const Segment * const SG98837[] = {SGI396153, SGI396154, SGI0, NULL};
const Segment * const SG98838[] = {SGI396155, NULL};
const Segment * const SG98839[] = {SGI396156, NULL};
const Segment * const SG98840[] = {SGI396157, NULL};
const Segment * const SG98841[] = {SGI396158, NULL};
const Segment * const SG98842[] = {SGI396159, SGI0, SGI0, NULL};
const Segment * const SG98843[] = {SGI396160, SGI0, SGI0, NULL};
const Segment * const SG98844[] = {SGI396161, NULL};
const Segment * const SG98845[] = {SGI396162, NULL};
const Segment * const SG98846[] = {SGI396163, NULL};
const Segment * const SG98847[] = {SGI396164, NULL};
const Segment * const SG98848[] = {SGI396165, NULL};
const Segment * const SG98849[] = {SGI396166, SGI0, SGI0, NULL};
const Segment * const SG98850[] = {SGI396167, SGI0, SGI0, NULL};
const Segment * const SG98851[] = {SGI396168, SGI396169, NULL};
const Segment * const SG98852[] = {SGI396170, SGI396171, NULL};
const Segment * const SG98853[] = {SGI396172, SGI396173, NULL};
const Segment * const SG98854[] = {SGI396174, SGI396175, NULL};
const Segment * const SG98855[] = {SGI396176, SGI396177, NULL};
const Segment * const SG98856[] = {SGI396178, SGI396179, NULL};
const Segment * const SG98857[] = {SGI396180, SGI396181, NULL};
const Segment * const SG98858[] = {SGI396182, SGI0, NULL};
const Segment * const SG98859[] = {SGI396183, SGI0, SGI0, NULL};
const Segment * const SG98860[] = {SGI396184, NULL};
const Segment * const SG98861[] = {SGI396185, NULL};
const Segment * const SG98862[] = {SGI396186, SGI396187, NULL};
const Segment * const SG98863[] = {SGI396188, SGI396189, NULL};
const Segment * const SG98864[] = {SGI396190, SGI396191, NULL};
const Segment * const SG98865[] = {SGI396192, SGI396193, NULL};
const Segment * const SG98866[] = {SGI396194, SGI396195, NULL};
const Segment * const SG98867[] = {SGI396196, SGI396197, NULL};
const Segment * const SG98868[] = {SGI396198, SGI396199, NULL};
const Segment * const SG98869[] = {SGI396200, SGI396201, NULL};
const Segment * const SG98870[] = {SGI396202, SGI396203, NULL};
const Segment * const SG98871[] = {SGI396204, SGI396205, NULL};
const Segment * const SG98872[] = {SGI396206, SGI396207, NULL};
const Segment * const SG98873[] = {SGI396208, SGI396209, NULL};
const Segment * const SG98874[] = {SGI396210, SGI396211, NULL};
const Segment * const SG98875[] = {SGI396212, SGI396213, NULL};
const Segment * const SG98876[] = {SGI396214, SGI0, NULL};
const Segment * const SG98877[] = {SGI396215, SGI396216, NULL};
const Segment * const SG98878[] = {SGI396217, SGI396218, NULL};
const Segment * const SG98879[] = {SGI396219, SGI0, NULL};
const Segment * const SG98880[] = {SGI396220, SGI396221, NULL};
const Segment * const SG98881[] = {SGI396222, SGI396223, NULL};
const Segment * const SG98882[] = {SGI396224, SGI396225, NULL};
const Segment * const SG98883[] = {SGI396226, NULL};
const Segment * const SG98884[] = {SGI396227, SGI0, SGI0, NULL};
const Segment * const SG98885[] = {SGI396228, NULL};
const Segment * const SG98886[] = {SGI396229, NULL};
const Segment * const SG98887[] = {SGI396230, NULL};
const Segment * const SG98888[] = {SGI396231, SGI396232, NULL};
const Segment * const SG98889[] = {SGI396233, SGI396234, NULL};
const Segment * const SG98890[] = {SGI396235, SGI396236, NULL};
const Segment * const SG98891[] = {SGI396237, SGI396238, NULL};
const Segment * const SG98892[] = {SGI396239, SGI396240, NULL};
const Segment * const SG98893[] = {SGI396241, SGI396242, NULL};
const Segment * const SG98894[] = {SGI396243, SGI0, NULL};
const Segment * const SG98895[] = {SGI396244, SGI396245, NULL};
const Segment * const SG98896[] = {SGI396246, SGI396247, NULL};
const Segment * const SG98897[] = {SGI396248, SGI396249, NULL};
const Segment * const SG98898[] = {SGI396250, SGI396251, NULL};
const Segment * const SG98899[] = {SGI396252, SGI0, SGI0, NULL};
const Segment * const SG98900[] = {SGI396253, SGI0, SGI0, NULL};
const Segment * const SG98901[] = {SGI0, SGI396254, NULL};
const Segment * const SG98902[] = {SGI396255, SGI396256, NULL};
const Segment * const SG98903[] = {SGI396257, SGI396258, NULL};
const Segment * const SG98904[] = {SGI396259, SGI396260, NULL};
const Segment * const SG98905[] = {SGI396261, SGI396262, NULL};
const Segment * const SG98906[] = {SGI396263, SGI396264, NULL};
const Segment * const SG98907[] = {SGI396265, SGI396266, NULL};
const Segment * const SG98908[] = {SGI396267, SGI0, NULL};
const Segment * const SG98909[] = {SGI396268, SGI396269, NULL};
const Segment * const SG98910[] = {SGI396270, SGI0, NULL};
const Segment * const SG98911[] = {SGI396271, SGI0, NULL};
const Segment * const SG98912[] = {SGI396272, SGI396273, NULL};
const Segment * const SG98913[] = {SGI396274, SGI396275, NULL};
const Segment * const SG98914[] = {SGI396276, SGI396277, NULL};
const Segment * const SG98915[] = {SGI396278, SGI396279, NULL};
const Segment * const SG98916[] = {SGI396280, SGI396281, NULL};
const Segment * const SG98917[] = {SGI396282, SGI396283, NULL};
const Segment * const SG98918[] = {SGI396284, SGI396285, NULL};
const Segment * const SG98919[] = {SGI396286, SGI396287, NULL};
const Segment * const SG98920[] = {SGI396288, SGI396289, NULL};
const Segment * const SG98921[] = {SGI396290, SGI396291, NULL};
const Segment * const SG98922[] = {SGI396292, SGI0, NULL};
const Segment * const SG98923[] = {SGI396293, NULL};
const Segment * const SG98924[] = {SGI396294, NULL};
const Segment * const SG98925[] = {SGI396295, NULL};
const Segment * const SG98926[] = {SGI396296, SGI0, SGI0, NULL};
const Segment * const SG98927[] = {SGI396297, NULL};
const Segment * const SG98928[] = {SGI396298, NULL};
const Segment * const SG98929[] = {SGI396299, NULL};
const Segment * const SG98930[] = {SGI396300, NULL};
const Segment * const SG98931[] = {SGI396301, SGI0, NULL};
const Segment * const SG98932[] = {SGI396302, NULL};
const Segment * const SG98933[] = {SGI396303, SGI396304, NULL};
const Segment * const SG98934[] = {SGI396305, SGI396306, NULL};
const Segment * const SG98935[] = {SGI396307, SGI396308, NULL};
const Segment * const SG98936[] = {SGI396309, SGI396310, NULL};
const Segment * const SG98937[] = {SGI396311, SGI396312, NULL};
const Segment * const SG98938[] = {SGI396313, SGI396314, NULL};
const Segment * const SG98939[] = {SGI396315, SGI396316, NULL};
const Segment * const SG98940[] = {SGI396317, SGI396318, NULL};
const Segment * const SG98941[] = {SGI396319, SGI396320, NULL};
const Segment * const SG98942[] = {SGI396321, SGI396322, NULL};
const Segment * const SG98943[] = {SGI396323, SGI396324, NULL};
const Segment * const SG98944[] = {SGI396325, SGI396326, NULL};
const Segment * const SG98945[] = {SGI396327, SGI396328, NULL};
const Segment * const SG98946[] = {SGI396329, SGI396330, NULL};
const Segment * const SG98947[] = {SGI396331, NULL};
const Segment * const SG98948[] = {SGI396332, NULL};
const Segment * const SG98949[] = {SGI396333, SGI396334, NULL};
const Segment * const SG98950[] = {SGI396335, SGI396336, NULL};
const Segment * const SG98951[] = {SGI396337, SGI396338, NULL};
const Segment * const SG98952[] = {SGI396339, SGI396340, NULL};
const Segment * const SG98953[] = {SGI396341, SGI396342, NULL};
const Segment * const SG98954[] = {SGI396343, SGI396344, NULL};
const Segment * const SG98955[] = {SGI396345, SGI396346, NULL};
const Segment * const SG98956[] = {SGI396347, SGI396348, NULL};
const Segment * const SG98957[] = {SGI396349, SGI396350, NULL};
const Segment * const SG98958[] = {SGI396351, SGI0, NULL};
const Segment * const SG98959[] = {SGI396352, SGI396353, NULL};
const Segment * const SG98960[] = {SGI396354, SGI396355, NULL};
const Segment * const SG98961[] = {SGI396356, SGI396357, NULL};
const Segment * const SG98962[] = {SGI396358, NULL};
const Segment * const SG98963[] = {SGI396359, SGI396360, NULL};
const Segment * const SG98964[] = {SGI396361, SGI396362, NULL};
const Segment * const SG98965[] = {SGI396363, SGI396364, NULL};
const Segment * const SG98966[] = {SGI396365, SGI0, SGI396366, NULL};
const Segment * const SG98967[] = {SGI396367, SGI396368, NULL};
const Segment * const SG98968[] = {SGI396369, SGI396370, NULL};
const Segment * const SG98969[] = {SGI396371, SGI396372, NULL};
const Segment * const SG98970[] = {SGI396373, SGI396374, NULL};
const Segment * const SG98971[] = {SGI396375, SGI396376, NULL};
const Segment * const SG98972[] = {SGI396377, SGI0, NULL};
const Segment * const SG98973[] = {SGI396378, NULL};
const Segment * const SG98974[] = {SGI396379, SGI0, SGI0, NULL};
const Segment * const SG98975[] = {SGI396380, NULL};
const Segment * const SG98976[] = {SGI396381, NULL};
const Segment * const SG98977[] = {SGI396382, NULL};
const Segment * const SG98978[] = {SGI396383, NULL};
const Segment * const SG98979[] = {SGI396384, NULL};
const Segment * const SG98980[] = {SGI396385, SGI0, SGI396386, NULL};
const Segment * const SG98981[] = {SGI396387, SGI0, SGI0, NULL};
const Segment * const SG98982[] = {SGI396388, NULL};
const Segment * const SG98983[] = {SGI396389, NULL};
const Segment * const SG98984[] = {SGI396390, NULL};
const Segment * const SG98985[] = {SGI396391, NULL};
const Segment * const SG98986[] = {SGI396392, NULL};
const Segment * const SG98987[] = {SGI396393, NULL};
const Segment * const SG98988[] = {SGI396394, NULL};
const Segment * const SG98989[] = {SGI396395, NULL};
const Segment * const SG98990[] = {SGI396396, NULL};
const Segment * const SG98991[] = {SGI396397, SGI396398, NULL};
const Segment * const SG98992[] = {SGI396399, SGI396400, NULL};
const Segment * const SG98993[] = {SGI396401, NULL};
const Segment * const SG98994[] = {SGI396402, NULL};
const Segment * const SG98995[] = {SGI396403, SGI396404, NULL};
const Segment * const SG98996[] = {SGI396405, SGI396406, NULL};
const Segment * const SG98997[] = {SGI396407, SGI396408, NULL};
const Segment * const SG98998[] = {SGI396409, SGI396410, NULL};
const Segment * const SG98999[] = {SGI396411, SGI396412, NULL};
const Segment * const SG99000[] = {SGI396413, SGI396414, NULL};
const Segment * const SG99001[] = {SGI396415, SGI396416, NULL};
const Segment * const SG99002[] = {SGI396417, SGI0, NULL};
const Segment * const SG99003[] = {SGI396418, SGI0, SGI0, NULL};
const Segment * const SG99004[] = {SGI396419, NULL};
const Segment * const SG99005[] = {SGI396420, SGI396421, NULL};
const Segment * const SG99006[] = {SGI396422, SGI0, NULL};
const Segment * const SG99007[] = {SGI396423, NULL};
const Segment * const SG99008[] = {SGI396424, NULL};
const Segment * const SG99009[] = {SGI396425, NULL};
const Segment * const SG99010[] = {SGI396426, NULL};
const Segment * const SG99011[] = {SGI396427, SGI0, NULL};
const Segment * const SG99012[] = {SGI396428, SGI396429, NULL};
const Segment * const SG99013[] = {SGI396430, SGI0, NULL};
const Segment * const SG99014[] = {SGI396431, SGI0, SGI0, NULL};
const Segment * const SG99015[] = {SGI396432, SGI396433, NULL};
const Segment * const SG99016[] = {SGI396434, SGI0, NULL};
const Segment * const SG99017[] = {SGI396435, NULL};
const Segment * const SG99018[] = {SGI396436, NULL};
const Segment * const SG99019[] = {SGI396437, SGI0, SGI0, NULL};
const Segment * const SG99020[] = {SGI396438, SGI0, SGI396439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396440, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99021[] = {SGI396441, SGI396442, NULL};
const Segment * const SG99022[] = {SGI396443, SGI0, NULL};
const Segment * const SG99023[] = {SGI396444, SGI0, SGI0, NULL};
const Segment * const SG99024[] = {SGI396445, SGI396446, NULL};
const Segment * const SG99025[] = {SGI396447, SGI0, NULL};
const Segment * const SG99026[] = {SGI396448, SGI396449, NULL};
const Segment * const SG99027[] = {SGI396450, SGI396451, NULL};
const Segment * const SG99028[] = {SGI396452, SGI396453, NULL};
const Segment * const SG99029[] = {SGI396454, SGI396455, NULL};
const Segment * const SG99030[] = {SGI396456, SGI396457, NULL};
const Segment * const SG99031[] = {SGI396458, SGI396459, NULL};
const Segment * const SG99032[] = {SGI396460, SGI396461, NULL};
const Segment * const SG99033[] = {SGI396462, SGI396463, NULL};
const Segment * const SG99034[] = {SGI396464, SGI396465, NULL};
const Segment * const SG99035[] = {SGI396466, SGI396467, NULL};
const Segment * const SG99036[] = {SGI396468, SGI396469, NULL};
const Segment * const SG99037[] = {SGI396470, SGI396471, NULL};
const Segment * const SG99038[] = {SGI396472, SGI0, NULL};
const Segment * const SG99039[] = {SGI396473, SGI396474, NULL};
const Segment * const SG99040[] = {SGI396475, SGI396476, NULL};
const Segment * const SG99041[] = {SGI396477, SGI0, NULL};
const Segment * const SG99042[] = {SGI396478, SGI396479, NULL};
const Segment * const SG99043[] = {SGI396480, SGI396481, NULL};
const Segment * const SG99044[] = {SGI396482, SGI396483, NULL};
const Segment * const SG99045[] = {SGI396484, SGI396485, NULL};
const Segment * const SG99046[] = {SGI396486, SGI396487, NULL};
const Segment * const SG99047[] = {SGI396488, SGI0, NULL};
const Segment * const SG99048[] = {SGI396489, SGI396490, NULL};
const Segment * const SG99049[] = {SGI396491, SGI396492, NULL};
const Segment * const SG99050[] = {SGI396493, SGI396494, NULL};
const Segment * const SG99051[] = {SGI396495, SGI396496, NULL};
const Segment * const SG99052[] = {SGI396497, SGI396498, NULL};
const Segment * const SG99053[] = {SGI396499, SGI0, NULL};
const Segment * const SG99054[] = {SGI396500, SGI0, NULL};
const Segment * const SG99055[] = {SGI396501, SGI0, NULL};
const Segment * const SG99056[] = {SGI396502, SGI396503, NULL};
const Segment * const SG99057[] = {SGI396504, NULL};
const Segment * const SG99058[] = {SGI396505, SGI396506, NULL};
const Segment * const SG99059[] = {SGI396507, SGI396508, NULL};
const Segment * const SG99060[] = {SGI396509, SGI396510, NULL};
const Segment * const SG99061[] = {SGI396511, SGI396512, NULL};
const Segment * const SG99062[] = {SGI396513, SGI0, NULL};
const Segment * const SG99063[] = {SGI396514, SGI396515, NULL};
const Segment * const SG99064[] = {SGI396516, SGI0, NULL};
const Segment * const SG99065[] = {SGI396517, SGI396518, NULL};
const Segment * const SG99066[] = {SGI396519, SGI396520, NULL};
const Segment * const SG99067[] = {SGI396521, SGI396522, NULL};
const Segment * const SG99068[] = {SGI396523, SGI396524, NULL};
const Segment * const SG99069[] = {SGI396525, SGI396526, NULL};
const Segment * const SG99070[] = {SGI396527, SGI396528, NULL};
const Segment * const SG99071[] = {SGI396529, SGI396530, NULL};
const Segment * const SG99072[] = {SGI396531, SGI396532, NULL};
const Segment * const SG99073[] = {SGI396533, SGI396534, NULL};
const Segment * const SG99074[] = {SGI396535, SGI396536, NULL};
const Segment * const SG99075[] = {SGI396537, SGI396538, NULL};
const Segment * const SG99076[] = {SGI396539, SGI396540, NULL};
const Segment * const SG99077[] = {SGI396541, SGI396542, NULL};
const Segment * const SG99078[] = {SGI396543, SGI396544, NULL};
const Segment * const SG99079[] = {SGI396545, SGI396546, NULL};
const Segment * const SG99080[] = {SGI396547, NULL};
const Segment * const SG99081[] = {SGI396548, SGI396549, NULL};
const Segment * const SG99082[] = {SGI396550, NULL};
const Segment * const SG99083[] = {SGI396551, NULL};
const Segment * const SG99084[] = {SGI396552, NULL};
const Segment * const SG99085[] = {SGI396553, SGI396554, NULL};
const Segment * const SG99086[] = {SGI396555, SGI396556, NULL};
const Segment * const SG99087[] = {SGI396557, SGI396558, NULL};
const Segment * const SG99088[] = {SGI396559, SGI396560, NULL};
const Segment * const SG99089[] = {SGI396561, SGI396562, NULL};
const Segment * const SG99090[] = {SGI396563, SGI0, SGI396564, NULL};
const Segment * const SG99091[] = {SGI396565, SGI396566, NULL};
const Segment * const SG99092[] = {SGI396567, NULL};
const Segment * const SG99093[] = {SGI396568, NULL};
const Segment * const SG99094[] = {SGI396569, SGI0, SGI0, NULL};
const Segment * const SG99095[] = {SGI396570, SGI396571, NULL};
const Segment * const SG99096[] = {SGI396572, SGI396573, NULL};
const Segment * const SG99097[] = {SGI396574, SGI396575, NULL};
const Segment * const SG99098[] = {SGI396576, SGI396577, NULL};
const Segment * const SG99099[] = {SGI396578, SGI396579, NULL};
const Segment * const SG99100[] = {SGI396580, SGI396581, NULL};
const Segment * const SG99101[] = {SGI396582, SGI0, NULL};
const Segment * const SG99102[] = {SGI396583, SGI0, NULL};
const Segment * const SG99103[] = {SGI396584, SGI396585, NULL};
const Segment * const SG99104[] = {SGI396586, SGI396587, NULL};
const Segment * const SG99105[] = {SGI396588, SGI396589, NULL};
const Segment * const SG99106[] = {SGI396590, SGI396591, NULL};
const Segment * const SG99107[] = {SGI396592, SGI396593, NULL};
const Segment * const SG99108[] = {SGI396594, SGI396595, NULL};
const Segment * const SG99109[] = {SGI396596, SGI396597, NULL};
const Segment * const SG99110[] = {SGI396598, SGI396599, NULL};
const Segment * const SG99111[] = {SGI396600, SGI396601, NULL};
const Segment * const SG99112[] = {SGI396602, SGI396603, NULL};
const Segment * const SG99113[] = {SGI396604, SGI396605, NULL};
const Segment * const SG99114[] = {SGI396606, SGI396607, NULL};
const Segment * const SG99115[] = {SGI396608, SGI396609, NULL};
const Segment * const SG99116[] = {SGI396610, SGI396611, NULL};
const Segment * const SG99117[] = {SGI396612, SGI396613, NULL};
const Segment * const SG99118[] = {SGI396614, SGI396615, NULL};
const Segment * const SG99119[] = {SGI396616, SGI396617, NULL};
const Segment * const SG99120[] = {SGI396618, SGI396619, NULL};
const Segment * const SG99121[] = {SGI396620, SGI396621, NULL};
const Segment * const SG99122[] = {SGI396622, SGI396623, NULL};
const Segment * const SG99123[] = {SGI396624, SGI0, NULL};
const Segment * const SG99124[] = {SGI396625, SGI0, NULL};
const Segment * const SG99125[] = {SGI396626, SGI396627, NULL};
const Segment * const SG99126[] = {SGI396628, SGI396629, NULL};
const Segment * const SG99127[] = {SGI396630, NULL};
const Segment * const SG99128[] = {SGI396631, SGI396632, NULL};
const Segment * const SG99129[] = {SGI396633, SGI0, NULL};
const Segment * const SG99130[] = {SGI396634, SGI396635, NULL};
const Segment * const SG99131[] = {SGI396636, SGI0, SGI0, NULL};
const Segment * const SG99132[] = {SGI396637, NULL};
const Segment * const SG99133[] = {SGI396638, SGI396639, NULL};
const Segment * const SG99134[] = {SGI396640, SGI0, SGI396641, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396642, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99135[] = {SGI396643, SGI396644, NULL};
const Segment * const SG99136[] = {SGI396645, SGI0, NULL};
const Segment * const SG99137[] = {SGI396646, SGI396647, NULL};
const Segment * const SG99138[] = {SGI396648, NULL};
const Segment * const SG99139[] = {SGI396649, SGI396650, NULL};
const Segment * const SG99140[] = {SGI396651, SGI396652, NULL};
const Segment * const SG99141[] = {SGI396653, SGI396654, NULL};
const Segment * const SG99142[] = {SGI396655, SGI396656, NULL};
const Segment * const SG99143[] = {SGI396657, SGI396658, NULL};
const Segment * const SG99144[] = {SGI396659, SGI396660, NULL};
const Segment * const SG99145[] = {SGI396661, SGI396662, NULL};
const Segment * const SG99146[] = {SGI396663, SGI396664, NULL};
const Segment * const SG99147[] = {SGI396665, SGI396666, NULL};
const Segment * const SG99148[] = {SGI396667, SGI396668, NULL};
const Segment * const SG99149[] = {SGI396669, SGI396670, NULL};
const Segment * const SG99150[] = {SGI396671, SGI396672, NULL};
const Segment * const SG99151[] = {SGI396673, SGI396674, NULL};
const Segment * const SG99152[] = {SGI396675, SGI396676, NULL};
const Segment * const SG99153[] = {SGI396677, SGI396678, NULL};
const Segment * const SG99154[] = {SGI396679, SGI396680, NULL};
const Segment * const SG99155[] = {SGI396681, SGI396682, NULL};
const Segment * const SG99156[] = {SGI396683, SGI396684, NULL};
const Segment * const SG99157[] = {SGI396685, SGI396686, NULL};
const Segment * const SG99158[] = {SGI396687, SGI396688, NULL};
const Segment * const SG99159[] = {SGI396689, SGI396690, NULL};
const Segment * const SG99160[] = {SGI396691, SGI396692, NULL};
const Segment * const SG99161[] = {SGI396693, NULL};
const Segment * const SG99162[] = {SGI396694, SGI396695, NULL};
const Segment * const SG99163[] = {SGI396696, SGI396697, NULL};
const Segment * const SG99164[] = {SGI396698, SGI396699, NULL};
const Segment * const SG99165[] = {SGI396700, SGI396701, NULL};
const Segment * const SG99166[] = {SGI396702, SGI396703, NULL};
const Segment * const SG99167[] = {SGI396704, SGI396705, NULL};
const Segment * const SG99168[] = {SGI396706, SGI396707, NULL};
const Segment * const SG99169[] = {SGI396708, SGI396709, NULL};
const Segment * const SG99170[] = {SGI396710, SGI396711, NULL};
const Segment * const SG99171[] = {SGI396712, SGI396713, NULL};
const Segment * const SG99172[] = {SGI396714, SGI396715, NULL};
const Segment * const SG99173[] = {SGI396716, SGI0, NULL};
const Segment * const SG99174[] = {SGI396717, SGI396718, NULL};
const Segment * const SG99175[] = {SGI396719, SGI396720, NULL};
const Segment * const SG99176[] = {SGI396721, SGI396722, NULL};
const Segment * const SG99177[] = {SGI396723, SGI396724, NULL};
const Segment * const SG99178[] = {SGI396725, SGI0, SGI396726, NULL};
const Segment * const SG99179[] = {SGI396727, SGI396728, NULL};
const Segment * const SG99180[] = {SGI396729, SGI396730, NULL};
const Segment * const SG99181[] = {SGI396731, SGI396732, NULL};
const Segment * const SG99182[] = {SGI396733, SGI0, NULL};
const Segment * const SG99183[] = {SGI396734, SGI396735, NULL};
const Segment * const SG99184[] = {SGI396736, SGI396737, NULL};
const Segment * const SG99185[] = {SGI396738, SGI396739, NULL};
const Segment * const SG99186[] = {SGI396740, SGI396741, NULL};
const Segment * const SG99187[] = {SGI396742, SGI396743, NULL};
const Segment * const SG99188[] = {SGI396744, SGI396745, NULL};
const Segment * const SG99189[] = {SGI396746, SGI0, SGI396747, NULL};
const Segment * const SG99190[] = {SGI396748, SGI396749, NULL};
const Segment * const SG99191[] = {SGI396750, SGI396751, NULL};
const Segment * const SG99192[] = {SGI396752, SGI396753, NULL};
const Segment * const SG99193[] = {SGI396754, SGI396755, NULL};
const Segment * const SG99194[] = {SGI396756, SGI396757, NULL};
const Segment * const SG99195[] = {SGI396758, SGI396759, NULL};
const Segment * const SG99196[] = {SGI396760, SGI396761, NULL};
const Segment * const SG99197[] = {SGI396762, SGI396763, NULL};
const Segment * const SG99198[] = {SGI396764, SGI396765, NULL};
const Segment * const SG99199[] = {SGI396766, SGI396767, NULL};
const Segment * const SG99200[] = {SGI396768, NULL};
const Segment * const SG99201[] = {SGI396769, SGI396770, NULL};
const Segment * const SG99202[] = {SGI396771, SGI396772, NULL};
const Segment * const SG99203[] = {SGI396773, SGI0, SGI396774, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396775, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99204[] = {SGI396776, SGI0, SGI396777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396778, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99205[] = {SGI396779, SGI396780, NULL};
const Segment * const SG99206[] = {SGI396781, SGI396782, SGI396783, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396784, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99207[] = {SGI396785, SGI396786, SGI396787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396788, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99208[] = {SGI396789, SGI396790, SGI396791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396792, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99209[] = {SGI396793, SGI396794, SGI396795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396796, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99210[] = {SGI396797, SGI396798, SGI396799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396800, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99211[] = {SGI396801, SGI0, SGI396802, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99212[] = {SGI396803, SGI396804, SGI396805, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396806, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99213[] = {SGI396807, SGI0, SGI396808, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396809, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99214[] = {SGI396810, SGI396811, SGI396812, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396813, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99215[] = {SGI396814, SGI396815, SGI396816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396817, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99216[] = {SGI396818, SGI396819, SGI396820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396821, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99217[] = {SGI396822, SGI396823, SGI396824, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396825, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99218[] = {SGI396826, SGI396827, SGI396828, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396829, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99219[] = {SGI396830, SGI0, SGI396831, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99220[] = {SGI396832, SGI396833, SGI396834, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396835, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99221[] = {SGI396836, SGI396837, SGI396838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396839, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99222[] = {SGI396840, SGI0, SGI396841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396842, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99223[] = {SGI396843, SGI396844, SGI396845, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396846, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99224[] = {SGI396847, SGI396848, SGI396849, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396850, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99225[] = {SGI396851, SGI396852, SGI396853, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396854, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99226[] = {SGI396855, SGI396856, NULL};
const Segment * const SG99227[] = {SGI396857, SGI0, NULL};
const Segment * const SG99228[] = {SGI396858, SGI396859, NULL};
const Segment * const SG99229[] = {SGI396860, SGI396861, NULL};
const Segment * const SG99230[] = {SGI396862, SGI396863, NULL};
const Segment * const SG99231[] = {SGI396864, NULL};
const Segment * const SG99232[] = {SGI396865, SGI396866, NULL};
const Segment * const SG99233[] = {SGI396867, SGI396868, NULL};
const Segment * const SG99234[] = {SGI396869, NULL};
const Segment * const SG99235[] = {SGI396870, NULL};
const Segment * const SG99236[] = {SGI396871, SGI396872, NULL};
const Segment * const SG99237[] = {SGI396873, SGI0, SGI0, NULL};
const Segment * const SG99238[] = {SGI396874, SGI396875, NULL};
const Segment * const SG99239[] = {SGI396876, SGI0, NULL};
const Segment * const SG99240[] = {SGI396877, SGI396878, NULL};
const Segment * const SG99241[] = {SGI396879, SGI396880, NULL};
const Segment * const SG99242[] = {SGI396881, SGI396882, NULL};
const Segment * const SG99243[] = {SGI396883, SGI396884, NULL};
const Segment * const SG99244[] = {SGI396885, SGI396886, NULL};
const Segment * const SG99245[] = {SGI396887, SGI396888, NULL};
const Segment * const SG99246[] = {SGI396889, SGI396890, NULL};
const Segment * const SG99247[] = {SGI396891, SGI0, NULL};
const Segment * const SG99248[] = {SGI396892, SGI396893, NULL};
const Segment * const SG99249[] = {SGI396894, SGI396895, NULL};
const Segment * const SG99250[] = {SGI396896, SGI396897, NULL};
const Segment * const SG99251[] = {SGI396898, SGI396899, NULL};
const Segment * const SG99252[] = {SGI396900, SGI396901, NULL};
const Segment * const SG99253[] = {SGI396902, SGI396903, NULL};
const Segment * const SG99254[] = {SGI396904, SGI396905, NULL};
const Segment * const SG99255[] = {SGI396906, SGI396907, NULL};
const Segment * const SG99256[] = {SGI396908, SGI396909, NULL};
const Segment * const SG99257[] = {SGI396910, SGI0, NULL};
const Segment * const SG99258[] = {SGI396911, SGI396912, NULL};
const Segment * const SG99259[] = {SGI396913, SGI396914, NULL};
const Segment * const SG99260[] = {SGI396915, SGI396916, NULL};
const Segment * const SG99261[] = {SGI396917, NULL};
const Segment * const SG99262[] = {SGI396918, SGI396919, NULL};
const Segment * const SG99263[] = {SGI396920, SGI396921, NULL};
const Segment * const SG99264[] = {SGI396922, SGI0, SGI0, NULL};
const Segment * const SG99265[] = {SGI396923, SGI396924, NULL};
const Segment * const SG99266[] = {SGI396925, SGI396926, NULL};
const Segment * const SG99267[] = {SGI396927, SGI396928, NULL};
const Segment * const SG99268[] = {SGI396929, SGI396930, NULL};
const Segment * const SG99269[] = {SGI396931, SGI396932, NULL};
const Segment * const SG99270[] = {SGI396933, SGI396934, NULL};
const Segment * const SG99271[] = {SGI396935, SGI396936, NULL};
const Segment * const SG99272[] = {SGI396937, SGI396938, NULL};
const Segment * const SG99273[] = {SGI396939, SGI396940, NULL};
const Segment * const SG99274[] = {SGI396941, SGI396942, NULL};
const Segment * const SG99275[] = {SGI396943, SGI396944, NULL};
const Segment * const SG99276[] = {SGI396945, SGI396946, NULL};
const Segment * const SG99277[] = {SGI396947, SGI396948, NULL};
const Segment * const SG99278[] = {SGI396949, SGI396950, NULL};
const Segment * const SG99279[] = {SGI396951, SGI396952, NULL};
const Segment * const SG99280[] = {SGI396953, SGI396954, NULL};
const Segment * const SG99281[] = {SGI396955, SGI396956, NULL};
const Segment * const SG99282[] = {SGI396957, SGI396958, NULL};
const Segment * const SG99283[] = {SGI396959, SGI396960, NULL};
const Segment * const SG99284[] = {SGI396961, SGI396962, NULL};
const Segment * const SG99285[] = {SGI396963, SGI396964, NULL};
const Segment * const SG99286[] = {SGI396965, SGI396966, NULL};
const Segment * const SG99287[] = {SGI396967, SGI396968, NULL};
const Segment * const SG99288[] = {SGI396969, SGI0, NULL};
const Segment * const SG99289[] = {SGI396970, SGI396971, NULL};
const Segment * const SG99290[] = {SGI396972, SGI396973, NULL};
const Segment * const SG99291[] = {SGI396974, SGI396975, NULL};
const Segment * const SG99292[] = {SGI396976, SGI396977, NULL};
const Segment * const SG99293[] = {SGI396978, SGI0, SGI396979, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396980, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99294[] = {SGI396981, SGI396982, NULL};
const Segment * const SG99295[] = {SGI396983, SGI396984, NULL};
const Segment * const SG99296[] = {SGI396985, SGI396986, NULL};
const Segment * const SG99297[] = {SGI396987, SGI396988, NULL};
const Segment * const SG99298[] = {SGI396989, SGI396990, SGI396991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396992, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99299[] = {SGI396993, SGI396994, SGI396995, SGI0, SGI0, SGI0, SGI0, SGI0, SGI396996, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99300[] = {SGI396997, SGI396998, SGI396999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397000, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99301[] = {SGI397001, SGI397002, SGI397003, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397004, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99302[] = {SGI397005, SGI397006, SGI397007, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397008, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99303[] = {SGI397009, SGI397011, NULL};
const Segment * const SG99304[] = {SGI397010, SGI0, SGI397012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99305[] = {SGI397013, SGI0, SGI397014, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397015, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99306[] = {SGI397017, SGI397019, NULL};
const Segment * const SG99307[] = {SGI397016, SGI397018, SGI397020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397021, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99308[] = {SGI397022, SGI397023, SGI397024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397025, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99309[] = {SGI397026, SGI397027, SGI397028, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397029, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99310[] = {SGI397030, SGI397031, NULL};
const Segment * const SG99311[] = {SGI397032, SGI397033, NULL};
const Segment * const SG99312[] = {SGI397034, SGI397035, NULL};
const Segment * const SG99313[] = {SGI397036, SGI397037, NULL};
const Segment * const SG99314[] = {SGI397038, SGI397039, NULL};
const Segment * const SG99315[] = {SGI397040, SGI397041, NULL};
const Segment * const SG99316[] = {SGI397042, SGI397043, NULL};
const Segment * const SG99317[] = {SGI397044, SGI397045, NULL};
const Segment * const SG99318[] = {SGI397046, SGI397047, NULL};
const Segment * const SG99319[] = {SGI397048, SGI0, NULL};
const Segment * const SG99320[] = {SGI397049, SGI397050, NULL};
const Segment * const SG99321[] = {SGI397051, SGI397052, NULL};
const Segment * const SG99322[] = {SGI397053, SGI397054, NULL};
const Segment * const SG99323[] = {SGI397055, SGI397056, NULL};
const Segment * const SG99324[] = {SGI397057, SGI0, NULL};
const Segment * const SG99325[] = {SGI397058, SGI397059, NULL};
const Segment * const SG99326[] = {SGI397060, SGI397061, NULL};
const Segment * const SG99327[] = {SGI397062, SGI397063, NULL};
const Segment * const SG99328[] = {SGI397064, SGI397065, NULL};
const Segment * const SG99329[] = {SGI397066, SGI397067, NULL};
const Segment * const SG99330[] = {SGI397068, SGI397069, NULL};
const Segment * const SG99331[] = {SGI397070, SGI0, SGI0, NULL};
const Segment * const SG99332[] = {SGI397071, SGI397072, NULL};
const Segment * const SG99333[] = {SGI397073, SGI397074, NULL};
const Segment * const SG99334[] = {SGI397075, SGI397076, NULL};
const Segment * const SG99335[] = {SGI397077, SGI397078, NULL};
const Segment * const SG99336[] = {SGI397079, SGI397080, NULL};
const Segment * const SG99337[] = {SGI397081, SGI397082, NULL};
const Segment * const SG99338[] = {SGI397083, SGI397084, NULL};
const Segment * const SG99339[] = {SGI397085, SGI397086, NULL};
const Segment * const SG99340[] = {SGI397087, SGI0, NULL};
const Segment * const SG99341[] = {SGI397088, SGI0, SGI397089, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397090, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99342[] = {SGI397091, SGI397092, NULL};
const Segment * const SG99343[] = {SGI397093, SGI397094, NULL};
const Segment * const SG99344[] = {SGI397095, SGI397096, NULL};
const Segment * const SG99345[] = {SGI397097, SGI397098, NULL};
const Segment * const SG99346[] = {SGI397099, SGI397100, NULL};
const Segment * const SG99347[] = {SGI397101, SGI397102, NULL};
const Segment * const SG99348[] = {SGI397103, SGI397104, NULL};
const Segment * const SG99349[] = {SGI397105, SGI397106, NULL};
const Segment * const SG99350[] = {SGI397107, SGI397108, NULL};
const Segment * const SG99351[] = {SGI397109, SGI397110, NULL};
const Segment * const SG99352[] = {SGI397111, SGI397112, NULL};
const Segment * const SG99353[] = {SGI397113, SGI397114, NULL};
const Segment * const SG99354[] = {SGI397115, SGI397116, NULL};
const Segment * const SG99355[] = {SGI397117, SGI397118, NULL};
const Segment * const SG99356[] = {SGI397119, SGI397120, NULL};
const Segment * const SG99357[] = {SGI397121, SGI397122, NULL};
const Segment * const SG99358[] = {SGI397123, SGI397124, NULL};
const Segment * const SG99359[] = {SGI397125, SGI397126, NULL};
const Segment * const SG99360[] = {SGI397127, SGI0, SGI0, NULL};
const Segment * const SG99361[] = {SGI397128, SGI397129, NULL};
const Segment * const SG99362[] = {SGI397130, SGI397131, NULL};
const Segment * const SG99363[] = {SGI397132, SGI397133, NULL};
const Segment * const SG99364[] = {SGI397134, SGI397135, NULL};
const Segment * const SG99365[] = {SGI397136, SGI397137, NULL};
const Segment * const SG99366[] = {SGI397138, SGI397139, NULL};
const Segment * const SG99367[] = {SGI397140, SGI0, NULL};
const Segment * const SG99368[] = {SGI397141, SGI0, NULL};
const Segment * const SG99369[] = {SGI397142, SGI397143, NULL};
const Segment * const SG99370[] = {SGI397144, SGI397145, NULL};
const Segment * const SG99371[] = {SGI397146, SGI397147, NULL};
const Segment * const SG99372[] = {SGI397148, SGI397149, NULL};
const Segment * const SG99373[] = {SGI397150, SGI397151, NULL};
const Segment * const SG99374[] = {SGI397152, SGI397153, NULL};
const Segment * const SG99375[] = {SGI397154, SGI397155, NULL};
const Segment * const SG99376[] = {SGI397156, SGI397157, NULL};
const Segment * const SG99377[] = {SGI397158, SGI397159, NULL};
const Segment * const SG99378[] = {SGI397160, SGI397161, NULL};
const Segment * const SG99379[] = {SGI397162, SGI397163, NULL};
const Segment * const SG99380[] = {SGI397164, SGI397165, NULL};
const Segment * const SG99381[] = {SGI397166, SGI397167, NULL};
const Segment * const SG99382[] = {SGI397168, SGI397169, NULL};
const Segment * const SG99383[] = {SGI397170, SGI397171, NULL};
const Segment * const SG99384[] = {SGI397172, SGI397173, NULL};
const Segment * const SG99385[] = {SGI397174, SGI397175, NULL};
const Segment * const SG99386[] = {SGI397176, SGI397177, NULL};
const Segment * const SG99387[] = {SGI397178, SGI0, NULL};
const Segment * const SG99388[] = {SGI397179, SGI0, NULL};
const Segment * const SG99389[] = {SGI397180, SGI397181, NULL};
const Segment * const SG99390[] = {SGI397182, SGI397183, NULL};
const Segment * const SG99391[] = {SGI397184, SGI397185, NULL};
const Segment * const SG99392[] = {SGI397186, SGI0, SGI0, NULL};
const Segment * const SG99393[] = {SGI397187, SGI397188, NULL};
const Segment * const SG99394[] = {SGI397189, SGI397190, NULL};
const Segment * const SG99395[] = {SGI397191, SGI397192, NULL};
const Segment * const SG99396[] = {SGI397193, SGI397194, NULL};
const Segment * const SG99397[] = {SGI397195, SGI397196, NULL};
const Segment * const SG99398[] = {SGI397197, SGI397198, NULL};
const Segment * const SG99399[] = {SGI397199, SGI397200, NULL};
const Segment * const SG99400[] = {SGI397201, SGI397202, NULL};
const Segment * const SG99401[] = {SGI397203, SGI397204, NULL};
const Segment * const SG99402[] = {SGI397205, SGI397206, NULL};
const Segment * const SG99403[] = {SGI397207, SGI397208, NULL};
const Segment * const SG99404[] = {SGI397209, SGI397210, NULL};
const Segment * const SG99405[] = {SGI397211, SGI397212, NULL};
const Segment * const SG99406[] = {SGI397213, SGI397214, NULL};
const Segment * const SG99407[] = {SGI397215, SGI397216, NULL};
const Segment * const SG99408[] = {SGI397217, SGI397218, NULL};
const Segment * const SG99409[] = {SGI397219, SGI0, SGI397220, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397221, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99410[] = {SGI397222, SGI397223, NULL};
const Segment * const SG99411[] = {SGI397224, SGI397225, NULL};
const Segment * const SG99412[] = {SGI397226, SGI397227, NULL};
const Segment * const SG99413[] = {SGI397228, SGI397229, NULL};
const Segment * const SG99414[] = {SGI397230, SGI397231, NULL};
const Segment * const SG99415[] = {SGI397232, SGI397233, NULL};
const Segment * const SG99416[] = {SGI397234, SGI397235, NULL};
const Segment * const SG99417[] = {SGI397236, SGI397237, NULL};
const Segment * const SG99418[] = {SGI397238, SGI397239, NULL};
const Segment * const SG99419[] = {SGI397240, SGI397241, NULL};
const Segment * const SG99420[] = {SGI397242, SGI397243, NULL};
const Segment * const SG99421[] = {SGI397244, SGI397245, NULL};
const Segment * const SG99422[] = {SGI397246, SGI397247, NULL};
const Segment * const SG99423[] = {SGI397248, SGI397249, NULL};
const Segment * const SG99424[] = {SGI397250, SGI0, NULL};
const Segment * const SG99425[] = {SGI397251, SGI397252, NULL};
const Segment * const SG99426[] = {SGI397253, SGI397254, NULL};
const Segment * const SG99427[] = {SGI397255, SGI397256, NULL};
const Segment * const SG99428[] = {SGI397257, SGI397258, NULL};
const Segment * const SG99429[] = {SGI397259, SGI397260, NULL};
const Segment * const SG99430[] = {SGI397261, SGI397262, NULL};
const Segment * const SG99431[] = {SGI397263, SGI397264, NULL};
const Segment * const SG99432[] = {SGI397265, SGI0, SGI397266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397267, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99433[] = {SGI397268, SGI397269, NULL};
const Segment * const SG99434[] = {SGI397270, SGI397271, NULL};
const Segment * const SG99435[] = {SGI397272, SGI397273, NULL};
const Segment * const SG99436[] = {SGI397274, SGI397275, NULL};
const Segment * const SG99437[] = {SGI397276, SGI397277, NULL};
const Segment * const SG99438[] = {SGI397278, SGI397279, NULL};
const Segment * const SG99439[] = {SGI397280, SGI397281, NULL};
const Segment * const SG99440[] = {SGI397282, SGI397283, NULL};
const Segment * const SG99441[] = {SGI397284, SGI397285, NULL};
const Segment * const SG99442[] = {SGI397286, SGI397287, NULL};
const Segment * const SG99443[] = {SGI397288, SGI397289, NULL};
const Segment * const SG99444[] = {SGI397290, SGI397291, NULL};
const Segment * const SG99445[] = {SGI397292, SGI0, SGI397293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397294, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99446[] = {SGI397295, SGI0, SGI397296, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397297, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99447[] = {SGI397298, SGI397299, NULL};
const Segment * const SG99448[] = {SGI397300, SGI397301, NULL};
const Segment * const SG99449[] = {SGI397302, SGI397303, NULL};
const Segment * const SG99450[] = {SGI397304, SGI397305, NULL};
const Segment * const SG99451[] = {SGI397306, SGI397307, NULL};
const Segment * const SG99452[] = {SGI397308, SGI397309, NULL};
const Segment * const SG99453[] = {SGI397310, SGI397311, NULL};
const Segment * const SG99454[] = {SGI397312, SGI397313, NULL};
const Segment * const SG99455[] = {SGI397314, SGI0, SGI0, NULL};
const Segment * const SG99456[] = {SGI397315, SGI397316, NULL};
const Segment * const SG99457[] = {SGI397317, SGI397318, NULL};
const Segment * const SG99458[] = {SGI397319, SGI397320, NULL};
const Segment * const SG99459[] = {SGI397321, SGI397322, NULL};
const Segment * const SG99460[] = {SGI397323, SGI397324, NULL};
const Segment * const SG99461[] = {SGI397325, SGI397326, NULL};
const Segment * const SG99462[] = {SGI397327, SGI397328, NULL};
const Segment * const SG99463[] = {SGI397329, SGI397330, NULL};
const Segment * const SG99464[] = {SGI397331, SGI397332, NULL};
const Segment * const SG99465[] = {SGI397333, SGI397334, NULL};
const Segment * const SG99466[] = {SGI397335, SGI397336, NULL};
const Segment * const SG99467[] = {SGI397339, SGI397340, NULL};
const Segment * const SG99468[] = {SGI397337, SGI0, SGI397338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397341, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99469[] = {SGI397342, SGI397343, NULL};
const Segment * const SG99470[] = {SGI397344, SGI397345, NULL};
const Segment * const SG99471[] = {SGI397346, SGI397347, NULL};
const Segment * const SG99472[] = {SGI397348, SGI397349, NULL};
const Segment * const SG99473[] = {SGI397350, SGI397351, NULL};
const Segment * const SG99474[] = {SGI397352, SGI0, NULL};
const Segment * const SG99475[] = {SGI397353, SGI397354, NULL};
const Segment * const SG99476[] = {SGI397355, SGI0, NULL};
const Segment * const SG99477[] = {SGI397356, SGI397357, NULL};
const Segment * const SG99478[] = {SGI397358, SGI397359, NULL};
const Segment * const SG99479[] = {SGI397360, SGI397361, NULL};
const Segment * const SG99480[] = {SGI397362, SGI0, NULL};
const Segment * const SG99481[] = {SGI397363, SGI397364, NULL};
const Segment * const SG99482[] = {SGI397365, SGI397366, NULL};
const Segment * const SG99483[] = {SGI397367, SGI0, SGI397368, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397369, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99484[] = {SGI397370, SGI397371, NULL};
const Segment * const SG99485[] = {SGI397372, SGI397373, NULL};
const Segment * const SG99486[] = {SGI397374, SGI397375, NULL};
const Segment * const SG99487[] = {SGI397376, SGI397377, NULL};
const Segment * const SG99488[] = {SGI397378, SGI397379, NULL};
const Segment * const SG99489[] = {SGI397380, SGI397381, NULL};
const Segment * const SG99490[] = {SGI397382, SGI397383, NULL};
const Segment * const SG99491[] = {SGI397384, SGI397385, NULL};
const Segment * const SG99492[] = {SGI397386, SGI397387, NULL};
const Segment * const SG99493[] = {SGI397388, SGI397389, NULL};
const Segment * const SG99494[] = {SGI397390, SGI397391, NULL};
const Segment * const SG99495[] = {SGI397392, SGI0, NULL};
const Segment * const SG99496[] = {SGI397393, SGI397394, NULL};
const Segment * const SG99497[] = {SGI397395, SGI397396, NULL};
const Segment * const SG99498[] = {SGI397397, SGI397398, NULL};
const Segment * const SG99499[] = {SGI397399, SGI397400, NULL};
const Segment * const SG99500[] = {SGI397401, SGI397402, NULL};
const Segment * const SG99501[] = {SGI397403, SGI397404, NULL};
const Segment * const SG99502[] = {SGI397405, SGI397406, NULL};
const Segment * const SG99503[] = {SGI397407, SGI397408, NULL};
const Segment * const SG99504[] = {SGI397409, SGI397410, NULL};
const Segment * const SG99505[] = {SGI397411, SGI397412, NULL};
const Segment * const SG99506[] = {SGI397413, SGI397414, NULL};
const Segment * const SG99507[] = {SGI397415, SGI397416, NULL};
const Segment * const SG99508[] = {SGI397417, SGI397418, NULL};
const Segment * const SG99509[] = {SGI397419, SGI397420, NULL};
const Segment * const SG99510[] = {SGI397421, SGI397422, NULL};
const Segment * const SG99511[] = {SGI397423, SGI397424, NULL};
const Segment * const SG99512[] = {SGI397425, SGI397426, NULL};
const Segment * const SG99513[] = {SGI397427, SGI397428, NULL};
const Segment * const SG99514[] = {SGI397429, SGI397430, NULL};
const Segment * const SG99515[] = {SGI397431, SGI397432, NULL};
const Segment * const SG99516[] = {SGI397433, SGI397434, NULL};
const Segment * const SG99517[] = {SGI397435, SGI397436, NULL};
const Segment * const SG99518[] = {SGI397437, SGI397438, NULL};
const Segment * const SG99519[] = {SGI397439, SGI397440, NULL};
const Segment * const SG99520[] = {SGI397441, SGI397442, NULL};
const Segment * const SG99521[] = {SGI397443, SGI0, SGI0, NULL};
const Segment * const SG99522[] = {SGI397444, SGI397445, NULL};
const Segment * const SG99523[] = {SGI397446, SGI0, NULL};
const Segment * const SG99524[] = {SGI397447, SGI397448, NULL};
const Segment * const SG99525[] = {SGI397449, SGI397450, NULL};
const Segment * const SG99526[] = {SGI397451, SGI397452, NULL};
const Segment * const SG99527[] = {SGI397453, SGI397454, NULL};
const Segment * const SG99528[] = {SGI397455, SGI397456, NULL};
const Segment * const SG99529[] = {SGI397457, SGI397458, NULL};
const Segment * const SG99530[] = {SGI397459, SGI397460, NULL};
const Segment * const SG99531[] = {SGI397461, SGI397462, NULL};
const Segment * const SG99532[] = {SGI397463, SGI397464, NULL};
const Segment * const SG99533[] = {SGI397465, SGI397466, NULL};
const Segment * const SG99534[] = {SGI397467, SGI397468, NULL};
const Segment * const SG99535[] = {SGI397469, SGI397470, NULL};
const Segment * const SG99536[] = {SGI397471, SGI397472, NULL};
const Segment * const SG99537[] = {SGI397473, SGI397474, NULL};
const Segment * const SG99538[] = {SGI397475, SGI397476, NULL};
const Segment * const SG99539[] = {SGI397477, SGI397478, NULL};
const Segment * const SG99540[] = {SGI397479, SGI397480, NULL};
const Segment * const SG99541[] = {SGI397481, SGI397482, NULL};
const Segment * const SG99542[] = {SGI397483, SGI397484, NULL};
const Segment * const SG99543[] = {SGI397485, SGI397486, NULL};
const Segment * const SG99544[] = {SGI397487, SGI397488, NULL};
const Segment * const SG99545[] = {SGI397489, SGI397490, NULL};
const Segment * const SG99546[] = {SGI397491, SGI397492, NULL};
const Segment * const SG99547[] = {SGI397493, SGI397494, NULL};
const Segment * const SG99548[] = {SGI397495, SGI397496, NULL};
const Segment * const SG99549[] = {SGI397497, SGI397498, NULL};
const Segment * const SG99550[] = {SGI397499, SGI397500, NULL};
const Segment * const SG99551[] = {SGI397501, SGI397502, NULL};
const Segment * const SG99552[] = {SGI397503, SGI397504, NULL};
const Segment * const SG99553[] = {SGI397505, SGI397506, NULL};
const Segment * const SG99554[] = {SGI397507, SGI397508, NULL};
const Segment * const SG99555[] = {SGI397509, SGI397510, NULL};
const Segment * const SG99556[] = {SGI397511, SGI397512, NULL};
const Segment * const SG99557[] = {SGI397513, SGI397514, NULL};
const Segment * const SG99558[] = {SGI397515, SGI397516, NULL};
const Segment * const SG99559[] = {SGI397517, SGI0, NULL};
const Segment * const SG99560[] = {SGI397518, SGI397519, NULL};
const Segment * const SG99561[] = {SGI397520, SGI397521, NULL};
const Segment * const SG99562[] = {SGI397522, SGI397523, NULL};
const Segment * const SG99563[] = {SGI397524, SGI397525, NULL};
const Segment * const SG99564[] = {SGI397526, SGI0, NULL};
const Segment * const SG99565[] = {SGI397527, SGI397528, NULL};
const Segment * const SG99566[] = {SGI397529, SGI397530, NULL};
const Segment * const SG99567[] = {SGI397531, SGI397532, NULL};
const Segment * const SG99568[] = {SGI397533, SGI397534, NULL};
const Segment * const SG99569[] = {SGI397535, SGI397536, NULL};
const Segment * const SG99570[] = {SGI397537, SGI397538, NULL};
const Segment * const SG99571[] = {SGI397539, SGI397540, NULL};
const Segment * const SG99572[] = {SGI397541, SGI397542, NULL};
const Segment * const SG99573[] = {SGI397543, SGI397544, NULL};
const Segment * const SG99574[] = {SGI397545, SGI397546, NULL};
const Segment * const SG99575[] = {SGI397547, SGI397548, NULL};
const Segment * const SG99576[] = {SGI397549, SGI397550, NULL};
const Segment * const SG99577[] = {SGI397551, SGI397552, NULL};
const Segment * const SG99578[] = {SGI397553, SGI397554, NULL};
const Segment * const SG99579[] = {SGI397555, SGI397556, NULL};
const Segment * const SG99580[] = {SGI397557, SGI397558, NULL};
const Segment * const SG99581[] = {SGI397559, SGI397560, NULL};
const Segment * const SG99582[] = {SGI397561, SGI397562, NULL};
const Segment * const SG99583[] = {SGI397563, SGI397564, NULL};
const Segment * const SG99584[] = {SGI397565, SGI397566, NULL};
const Segment * const SG99585[] = {SGI397567, SGI397568, NULL};
const Segment * const SG99586[] = {SGI397569, SGI0, SGI397570, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397571, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99587[] = {SGI397572, SGI397573, NULL};
const Segment * const SG99588[] = {SGI397574, SGI397575, NULL};
const Segment * const SG99589[] = {SGI397576, SGI0, NULL};
const Segment * const SG99590[] = {SGI397577, SGI0, NULL};
const Segment * const SG99591[] = {SGI397578, SGI397579, NULL};
const Segment * const SG99592[] = {SGI397580, SGI397581, NULL};
const Segment * const SG99593[] = {SGI397582, SGI397583, NULL};
const Segment * const SG99594[] = {SGI397584, SGI397585, NULL};
const Segment * const SG99595[] = {SGI397586, SGI397587, NULL};
const Segment * const SG99596[] = {SGI397588, SGI397589, NULL};
const Segment * const SG99597[] = {SGI397590, SGI397591, NULL};
const Segment * const SG99598[] = {SGI397592, SGI0, SGI397593, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397594, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99599[] = {SGI397595, SGI397596, NULL};
const Segment * const SG99600[] = {SGI397597, SGI397598, NULL};
const Segment * const SG99601[] = {SGI397599, SGI397600, NULL};
const Segment * const SG99602[] = {SGI397601, SGI397602, NULL};
const Segment * const SG99603[] = {SGI397603, SGI397604, NULL};
const Segment * const SG99604[] = {SGI397605, SGI397606, NULL};
const Segment * const SG99605[] = {SGI397607, SGI397608, NULL};
const Segment * const SG99606[] = {SGI397609, SGI397610, NULL};
const Segment * const SG99607[] = {SGI397611, SGI397612, NULL};
const Segment * const SG99608[] = {SGI397613, SGI397614, NULL};
const Segment * const SG99609[] = {SGI397615, SGI0, SGI397616, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397617, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99610[] = {SGI397618, SGI0, SGI397619, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397620, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99611[] = {SGI397621, SGI0, SGI397622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397623, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99612[] = {SGI397624, SGI0, SGI397625, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397626, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99613[] = {SGI397627, SGI0, SGI397628, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397629, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99614[] = {SGI397630, SGI397631, NULL};
const Segment * const SG99615[] = {SGI397632, SGI397633, NULL};
const Segment * const SG99616[] = {SGI397634, SGI397635, NULL};
const Segment * const SG99617[] = {SGI397636, SGI397637, NULL};
const Segment * const SG99618[] = {SGI397638, SGI397639, NULL};
const Segment * const SG99619[] = {SGI397640, SGI397641, NULL};
const Segment * const SG99620[] = {SGI397642, SGI397643, NULL};
const Segment * const SG99621[] = {SGI397644, SGI397645, NULL};
const Segment * const SG99622[] = {SGI397646, SGI397647, NULL};
const Segment * const SG99623[] = {SGI397648, SGI397649, NULL};
const Segment * const SG99624[] = {SGI397650, SGI397651, NULL};
const Segment * const SG99625[] = {SGI397652, SGI397653, NULL};
const Segment * const SG99626[] = {SGI397654, SGI397655, NULL};
const Segment * const SG99627[] = {SGI397656, SGI397657, NULL};
const Segment * const SG99628[] = {SGI397658, SGI397659, NULL};
const Segment * const SG99629[] = {SGI397660, SGI397661, NULL};
const Segment * const SG99630[] = {SGI397662, SGI397663, NULL};
const Segment * const SG99631[] = {SGI397664, SGI397665, NULL};
const Segment * const SG99632[] = {SGI397666, SGI397667, NULL};
const Segment * const SG99633[] = {SGI397668, SGI397669, NULL};
const Segment * const SG99634[] = {SGI397670, SGI397671, NULL};
const Segment * const SG99635[] = {SGI397672, SGI397673, NULL};
const Segment * const SG99636[] = {SGI397674, SGI397675, NULL};
const Segment * const SG99637[] = {SGI397676, SGI397677, NULL};
const Segment * const SG99638[] = {SGI397678, SGI397679, NULL};
const Segment * const SG99639[] = {SGI397680, SGI397681, NULL};
const Segment * const SG99640[] = {SGI397682, SGI397683, NULL};
const Segment * const SG99641[] = {SGI397684, SGI397685, NULL};
const Segment * const SG99642[] = {SGI397686, SGI397687, NULL};
const Segment * const SG99643[] = {SGI397688, SGI397689, NULL};
const Segment * const SG99644[] = {SGI397690, SGI397691, NULL};
const Segment * const SG99645[] = {SGI397692, SGI397693, NULL};
const Segment * const SG99646[] = {SGI397694, SGI397695, NULL};
const Segment * const SG99647[] = {SGI397696, SGI397697, NULL};
const Segment * const SG99648[] = {SGI397698, SGI397699, NULL};
const Segment * const SG99649[] = {SGI397700, SGI0, NULL};
const Segment * const SG99650[] = {SGI397701, SGI0, NULL};
const Segment * const SG99651[] = {SGI397702, SGI397703, NULL};
const Segment * const SG99652[] = {SGI397704, SGI0, NULL};
const Segment * const SG99653[] = {SGI397705, SGI397706, NULL};
const Segment * const SG99654[] = {SGI397707, SGI397708, NULL};
const Segment * const SG99655[] = {SGI397709, SGI397710, NULL};
const Segment * const SG99656[] = {SGI397711, SGI397712, NULL};
const Segment * const SG99657[] = {SGI397713, SGI397714, NULL};
const Segment * const SG99658[] = {SGI397715, SGI397716, NULL};
const Segment * const SG99659[] = {SGI397717, SGI397718, NULL};
const Segment * const SG99660[] = {SGI397719, SGI397720, NULL};
const Segment * const SG99661[] = {SGI397721, SGI397722, NULL};
const Segment * const SG99662[] = {SGI397723, SGI397724, NULL};
const Segment * const SG99663[] = {SGI397725, SGI397726, NULL};
const Segment * const SG99664[] = {SGI397727, SGI397728, NULL};
const Segment * const SG99665[] = {SGI397729, SGI397730, NULL};
const Segment * const SG99666[] = {SGI397731, SGI397732, NULL};
const Segment * const SG99667[] = {SGI397733, SGI397734, NULL};
const Segment * const SG99668[] = {SGI397735, SGI397736, NULL};
const Segment * const SG99669[] = {SGI397737, SGI397738, NULL};
const Segment * const SG99670[] = {SGI397739, SGI397740, NULL};
const Segment * const SG99671[] = {SGI397741, SGI397742, NULL};
const Segment * const SG99672[] = {SGI397743, SGI397744, NULL};
const Segment * const SG99673[] = {SGI397745, SGI397746, NULL};
const Segment * const SG99674[] = {SGI397747, SGI397748, NULL};
const Segment * const SG99675[] = {SGI397749, SGI397750, NULL};
const Segment * const SG99676[] = {SGI397751, SGI397752, NULL};
const Segment * const SG99677[] = {SGI397753, SGI397754, NULL};
const Segment * const SG99678[] = {SGI397755, SGI397756, NULL};
const Segment * const SG99679[] = {SGI397757, SGI397758, NULL};
const Segment * const SG99680[] = {SGI397759, SGI397760, NULL};
const Segment * const SG99681[] = {SGI397761, SGI397762, NULL};
const Segment * const SG99682[] = {SGI397763, SGI397764, NULL};
const Segment * const SG99683[] = {SGI397765, SGI397766, NULL};
const Segment * const SG99684[] = {SGI397767, SGI397768, NULL};
const Segment * const SG99685[] = {SGI397769, SGI397770, NULL};
const Segment * const SG99686[] = {SGI397771, SGI397772, NULL};
const Segment * const SG99687[] = {SGI397773, SGI397774, SGI397775, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397776, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99688[] = {SGI397777, SGI0, SGI397778, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99689[] = {SGI397779, SGI397780, SGI397781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397782, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99690[] = {SGI397783, SGI397784, SGI397785, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397786, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99691[] = {SGI397787, SGI397788, SGI397789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397790, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99692[] = {SGI397791, SGI397792, NULL};
const Segment * const SG99693[] = {SGI397793, SGI397794, NULL};
const Segment * const SG99694[] = {SGI397795, SGI397796, NULL};
const Segment * const SG99695[] = {SGI397797, SGI397798, NULL};
const Segment * const SG99696[] = {SGI397799, SGI397800, NULL};
const Segment * const SG99697[] = {SGI397801, SGI0, NULL};
const Segment * const SG99698[] = {SGI397802, SGI397803, NULL};
const Segment * const SG99699[] = {SGI397804, SGI397805, NULL};
const Segment * const SG99700[] = {SGI397806, SGI397807, NULL};
const Segment * const SG99701[] = {SGI397808, SGI397809, NULL};
const Segment * const SG99702[] = {SGI397810, SGI397811, NULL};
const Segment * const SG99703[] = {SGI397812, SGI397813, NULL};
const Segment * const SG99704[] = {SGI397814, SGI397815, NULL};
const Segment * const SG99705[] = {SGI397816, SGI397817, NULL};
const Segment * const SG99706[] = {SGI397818, SGI397819, NULL};
const Segment * const SG99707[] = {SGI397820, SGI397821, NULL};
const Segment * const SG99708[] = {SGI397822, SGI397823, NULL};
const Segment * const SG99709[] = {SGI397824, SGI397825, NULL};
const Segment * const SG99710[] = {SGI397826, SGI397827, NULL};
const Segment * const SG99711[] = {SGI397828, SGI397829, NULL};
const Segment * const SG99712[] = {SGI397830, SGI397831, NULL};
const Segment * const SG99713[] = {SGI397832, SGI397833, NULL};
const Segment * const SG99714[] = {SGI397834, SGI0, SGI397835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397836, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99715[] = {SGI397837, SGI0, SGI397838, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397839, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99716[] = {SGI397840, SGI0, SGI397841, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397842, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99717[] = {SGI397843, SGI0, SGI397844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI397845, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99718[] = {SGI397846, SGI397847, NULL};
const Segment * const SG99719[] = {SGI397848, SGI397849, NULL};
const Segment * const SG99720[] = {SGI397850, SGI397851, NULL};
const Segment * const SG99721[] = {SGI397852, SGI397853, NULL};
const Segment * const SG99722[] = {SGI397854, SGI397855, NULL};
const Segment * const SG99723[] = {SGI397856, SGI397857, NULL};
const Segment * const SG99724[] = {SGI397858, SGI397859, NULL};
const Segment * const SG99725[] = {SGI397860, SGI397861, NULL};
const Segment * const SG99726[] = {SGI397862, SGI397863, NULL};
const Segment * const SG99727[] = {SGI397864, SGI397865, NULL};
const Segment * const SG99728[] = {SGI397866, SGI397867, NULL};
const Segment * const SG99729[] = {SGI397868, SGI397869, NULL};
const Segment * const SG99730[] = {SGI397870, SGI397871, NULL};
const Segment * const SG99731[] = {SGI397872, SGI397873, NULL};
const Segment * const SG99732[] = {SGI397874, SGI397875, NULL};
const Segment * const SG99733[] = {SGI397876, SGI397877, NULL};
const Segment * const SG99734[] = {SGI397878, SGI0, SGI0, NULL};
const Segment * const SG99735[] = {SGI397879, SGI397880, NULL};
const Segment * const SG99736[] = {SGI397881, SGI397882, NULL};
const Segment * const SG99737[] = {SGI397883, SGI397884, NULL};
const Segment * const SG99738[] = {SGI397885, SGI397886, NULL};
const Segment * const SG99739[] = {SGI397887, SGI397888, NULL};
const Segment * const SG99740[] = {SGI397889, SGI397890, NULL};
const Segment * const SG99741[] = {SGI397891, SGI397892, NULL};
const Segment * const SG99742[] = {SGI397893, SGI397894, NULL};
const Segment * const SG99743[] = {SGI397895, SGI0, NULL};
const Segment * const SG99744[] = {SGI397896, SGI397897, NULL};
const Segment * const SG99745[] = {SGI397898, SGI0, NULL};
const Segment * const SG99746[] = {SGI397899, SGI397900, NULL};
const Segment * const SG99747[] = {SGI397901, SGI397902, NULL};
const Segment * const SG99748[] = {SGI397903, SGI397904, NULL};
const Segment * const SG99749[] = {SGI397905, SGI397906, NULL};
const Segment * const SG99750[] = {SGI397907, SGI397908, NULL};
const Segment * const SG99751[] = {SGI397909, SGI397910, NULL};
const Segment * const SG99752[] = {SGI397911, SGI397912, NULL};
const Segment * const SG99753[] = {SGI397913, SGI397914, NULL};
const Segment * const SG99754[] = {SGI397915, SGI397916, NULL};
const Segment * const SG99755[] = {SGI397917, SGI397918, NULL};
const Segment * const SG99756[] = {SGI397919, SGI397920, NULL};
const Segment * const SG99757[] = {SGI397921, SGI397922, NULL};
const Segment * const SG99758[] = {SGI397923, SGI397924, NULL};
const Segment * const SG99759[] = {SGI397925, SGI397926, NULL};
const Segment * const SG99760[] = {SGI397927, SGI397928, NULL};
const Segment * const SG99761[] = {SGI397929, SGI397930, NULL};
const Segment * const SG99762[] = {SGI397931, SGI397932, NULL};
const Segment * const SG99763[] = {SGI397933, SGI397934, NULL};
const Segment * const SG99764[] = {SGI397935, SGI397936, NULL};
const Segment * const SG99765[] = {SGI397937, SGI397938, NULL};
const Segment * const SG99766[] = {SGI397939, SGI397940, NULL};
const Segment * const SG99767[] = {SGI397941, SGI397942, NULL};
const Segment * const SG99768[] = {SGI397943, SGI397944, NULL};
const Segment * const SG99769[] = {SGI397945, SGI397946, NULL};
const Segment * const SG99770[] = {SGI397947, SGI397948, NULL};
const Segment * const SG99771[] = {SGI397949, SGI397950, NULL};
const Segment * const SG99772[] = {SGI397951, SGI397952, NULL};
const Segment * const SG99773[] = {SGI397953, SGI397954, NULL};
const Segment * const SG99774[] = {SGI397955, SGI397956, NULL};
const Segment * const SG99775[] = {SGI397957, SGI397958, NULL};
const Segment * const SG99776[] = {SGI397959, SGI397960, NULL};
const Segment * const SG99777[] = {SGI397961, SGI397962, NULL};
const Segment * const SG99778[] = {SGI397963, SGI397964, NULL};
const Segment * const SG99779[] = {SGI397965, SGI0, NULL};
const Segment * const SG99780[] = {SGI397966, SGI397967, NULL};
const Segment * const SG99781[] = {SGI397968, SGI397969, NULL};
const Segment * const SG99782[] = {SGI397970, SGI0, NULL};
const Segment * const SG99783[] = {SGI397971, SGI397972, NULL};
const Segment * const SG99784[] = {SGI397973, SGI397974, NULL};
const Segment * const SG99785[] = {SGI397975, SGI397976, NULL};
const Segment * const SG99786[] = {SGI397977, SGI397978, NULL};
const Segment * const SG99787[] = {SGI397979, SGI397980, NULL};
const Segment * const SG99788[] = {SGI397981, SGI0, NULL};
const Segment * const SG99789[] = {SGI397982, SGI397983, NULL};
const Segment * const SG99790[] = {SGI397984, SGI397985, NULL};
const Segment * const SG99791[] = {SGI397986, SGI397987, NULL};
const Segment * const SG99792[] = {SGI397988, SGI397989, NULL};
const Segment * const SG99793[] = {SGI397990, SGI397991, NULL};
const Segment * const SG99794[] = {SGI397992, SGI397993, NULL};
const Segment * const SG99795[] = {SGI397994, SGI397995, NULL};
const Segment * const SG99796[] = {SGI397996, SGI397997, NULL};
const Segment * const SG99797[] = {SGI397998, SGI397999, NULL};
const Segment * const SG99798[] = {SGI398000, SGI398001, NULL};
const Segment * const SG99799[] = {SGI398002, SGI398003, NULL};
const Segment * const SG99800[] = {SGI398004, SGI398005, NULL};
const Segment * const SG99801[] = {SGI398006, SGI398007, NULL};
const Segment * const SG99802[] = {SGI398008, SGI398009, NULL};
const Segment * const SG99803[] = {SGI398010, SGI398011, NULL};
const Segment * const SG99804[] = {SGI398012, SGI398013, NULL};
const Segment * const SG99805[] = {SGI398014, SGI398015, NULL};
const Segment * const SG99806[] = {SGI398016, SGI398017, NULL};
const Segment * const SG99807[] = {SGI398018, SGI398019, NULL};
const Segment * const SG99808[] = {SGI398020, SGI398021, NULL};
const Segment * const SG99809[] = {SGI398022, SGI398023, NULL};
const Segment * const SG99810[] = {SGI398024, SGI398025, NULL};
const Segment * const SG99811[] = {SGI398026, SGI398027, NULL};
const Segment * const SG99812[] = {SGI398028, SGI398029, NULL};
const Segment * const SG99813[] = {SGI398030, SGI398031, NULL};
const Segment * const SG99814[] = {SGI398032, SGI398033, NULL};
const Segment * const SG99815[] = {SGI398034, SGI398035, NULL};
const Segment * const SG99816[] = {SGI398036, SGI398037, NULL};
const Segment * const SG99817[] = {SGI398038, SGI398039, NULL};
const Segment * const SG99818[] = {SGI398040, SGI398041, NULL};
const Segment * const SG99819[] = {SGI398042, SGI398043, NULL};
const Segment * const SG99820[] = {SGI398044, SGI398045, NULL};
const Segment * const SG99821[] = {SGI398046, SGI398047, NULL};
const Segment * const SG99822[] = {SGI398048, SGI398049, NULL};
const Segment * const SG99823[] = {SGI398050, SGI398051, NULL};
const Segment * const SG99824[] = {SGI398052, SGI398053, NULL};
const Segment * const SG99825[] = {SGI398054, SGI398055, NULL};
const Segment * const SG99826[] = {SGI398056, SGI398057, NULL};
const Segment * const SG99827[] = {SGI398058, SGI398059, NULL};
const Segment * const SG99828[] = {SGI398060, SGI398061, NULL};
const Segment * const SG99829[] = {SGI398062, SGI398063, NULL};
const Segment * const SG99830[] = {SGI398064, SGI398065, NULL};
const Segment * const SG99831[] = {SGI398066, SGI398067, NULL};
const Segment * const SG99832[] = {SGI398068, SGI398069, NULL};
const Segment * const SG99833[] = {SGI398070, SGI398071, NULL};
const Segment * const SG99834[] = {SGI398072, SGI0, NULL};
const Segment * const SG99835[] = {SGI398073, SGI0, NULL};
const Segment * const SG99836[] = {SGI398074, SGI398075, NULL};
const Segment * const SG99837[] = {SGI398076, SGI0, SGI398077, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398078, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99838[] = {SGI398079, SGI0, SGI398080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398081, SGI0, SGI0, SGI0, NULL};
const Segment * const SG99839[] = {SGI398082, SGI398083, NULL};
const Segment * const SG99840[] = {SGI398084, SGI398085, NULL};
const Segment * const SG99841[] = {SGI398086, SGI398087, NULL};
const Segment * const SG99842[] = {SGI398088, SGI398089, NULL};
const Segment * const SG99843[] = {SGI398090, SGI398091, NULL};
const Segment * const SG99844[] = {SGI398092, SGI398093, NULL};
const Segment * const SG99845[] = {SGI398094, SGI398095, NULL};
const Segment * const SG99846[] = {SGI398096, SGI398097, NULL};
const Segment * const SG99847[] = {SGI398098, SGI398099, NULL};
const Segment * const SG99848[] = {SGI398100, SGI398101, NULL};
const Segment * const SG99849[] = {SGI398102, SGI398103, NULL};
const Segment * const SG99850[] = {SGI398104, SGI398105, NULL};
const Segment * const SG99851[] = {SGI398106, SGI398107, NULL};
const Segment * const SG99852[] = {SGI398108, SGI398109, NULL};
const Segment * const SG99853[] = {SGI398110, SGI398111, NULL};
const Segment * const SG99854[] = {SGI398112, SGI398113, NULL};
const Segment * const SG99855[] = {SGI398114, SGI398115, NULL};
const Segment * const SG99856[] = {SGI398116, SGI398117, NULL};
const Segment * const SG99857[] = {SGI398118, SGI398119, NULL};
const Segment * const SG99858[] = {SGI398120, SGI398121, NULL};
const Segment * const SG99859[] = {SGI398122, SGI398123, NULL};
const Segment * const SG99860[] = {SGI398124, SGI398125, NULL};
const Segment * const SG99861[] = {SGI398126, SGI0, NULL};
const Segment * const SG99862[] = {SGI398127, SGI398128, NULL};
const Segment * const SG99863[] = {SGI398129, SGI398130, NULL};
const Segment * const SG99864[] = {SGI398131, SGI398132, NULL};
const Segment * const SG99865[] = {SGI398133, SGI398134, NULL};
const Segment * const SG99866[] = {SGI398135, SGI398136, NULL};
const Segment * const SG99867[] = {SGI398137, SGI398138, NULL};
const Segment * const SG99868[] = {SGI398139, SGI398140, NULL};
const Segment * const SG99869[] = {SGI398141, SGI398142, NULL};
const Segment * const SG99870[] = {SGI398143, SGI398144, NULL};
const Segment * const SG99871[] = {SGI398145, SGI0, NULL};
const Segment * const SG99872[] = {SGI398146, SGI398147, NULL};
const Segment * const SG99873[] = {SGI398148, SGI398149, NULL};
const Segment * const SG99874[] = {SGI398150, SGI398151, NULL};
const Segment * const SG99875[] = {SGI398152, SGI398153, NULL};
const Segment * const SG99876[] = {SGI398154, SGI398155, NULL};
const Segment * const SG99877[] = {SGI398156, SGI398157, NULL};
const Segment * const SG99878[] = {SGI398158, SGI398159, NULL};
const Segment * const SG99879[] = {SGI398160, SGI398161, NULL};
const Segment * const SG99880[] = {SGI398162, SGI398163, NULL};
const Segment * const SG99881[] = {SGI398164, SGI398165, NULL};
const Segment * const SG99882[] = {SGI398166, SGI398167, NULL};
const Segment * const SG99883[] = {SGI398168, SGI398169, NULL};
const Segment * const SG99884[] = {SGI398170, SGI398171, NULL};
const Segment * const SG99885[] = {SGI398172, SGI398173, NULL};
const Segment * const SG99886[] = {SGI398174, SGI398175, NULL};
const Segment * const SG99887[] = {SGI398176, SGI398177, NULL};
const Segment * const SG99888[] = {SGI398178, SGI398179, NULL};
const Segment * const SG99889[] = {SGI398180, SGI398181, NULL};
const Segment * const SG99890[] = {SGI398182, SGI398183, NULL};
const Segment * const SG99891[] = {SGI398184, SGI398185, NULL};
const Segment * const SG99892[] = {SGI398186, SGI398187, NULL};
const Segment * const SG99893[] = {SGI398188, SGI398189, NULL};
const Segment * const SG99894[] = {SGI398190, SGI398191, NULL};
const Segment * const SG99895[] = {SGI398192, SGI398193, NULL};
const Segment * const SG99896[] = {SGI398194, SGI398195, NULL};
const Segment * const SG99897[] = {SGI398196, SGI398197, NULL};
const Segment * const SG99898[] = {SGI398198, SGI398199, NULL};
const Segment * const SG99899[] = {SGI398200, SGI398201, NULL};
const Segment * const SG99900[] = {SGI398202, SGI398203, NULL};
const Segment * const SG99901[] = {SGI398204, SGI398205, NULL};
const Segment * const SG99902[] = {SGI398206, SGI398207, NULL};
const Segment * const SG99903[] = {SGI398208, SGI398209, NULL};
const Segment * const SG99904[] = {SGI398210, SGI398211, NULL};
const Segment * const SG99905[] = {SGI398212, SGI398213, NULL};
const Segment * const SG99906[] = {SGI398214, SGI398215, NULL};
const Segment * const SG99907[] = {SGI398216, SGI398217, NULL};
const Segment * const SG99908[] = {SGI398218, SGI398219, NULL};
const Segment * const SG99909[] = {SGI398220, SGI398221, NULL};
const Segment * const SG99910[] = {SGI398222, SGI398223, NULL};
const Segment * const SG99911[] = {SGI398224, SGI398225, NULL};
const Segment * const SG99912[] = {SGI398226, SGI398227, NULL};
const Segment * const SG99913[] = {SGI398228, SGI398229, NULL};
const Segment * const SG99914[] = {SGI398230, SGI398231, NULL};
const Segment * const SG99915[] = {SGI398232, SGI398233, NULL};
const Segment * const SG99916[] = {SGI398234, SGI398235, NULL};
const Segment * const SG99917[] = {SGI398236, SGI398237, NULL};
const Segment * const SG99918[] = {SGI398238, SGI398239, NULL};
const Segment * const SG99919[] = {SGI398240, SGI398241, NULL};
const Segment * const SG99920[] = {SGI398242, SGI398243, NULL};
const Segment * const SG99921[] = {SGI398244, SGI398245, NULL};
const Segment * const SG99922[] = {SGI398246, SGI398247, NULL};
const Segment * const SG99923[] = {SGI398248, SGI398249, NULL};
const Segment * const SG99924[] = {SGI398250, SGI398251, NULL};
const Segment * const SG99925[] = {SGI398252, SGI398253, NULL};
const Segment * const SG99926[] = {SGI398254, SGI398255, NULL};
const Segment * const SG99927[] = {SGI398256, SGI398257, NULL};
const Segment * const SG99928[] = {SGI398258, SGI0, NULL};
const Segment * const SG99929[] = {SGI398259, SGI398260, SGI398261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398262, SGI0, SGI0, SGI0, SGI398263, NULL};
const Segment * const SG99930[] = {SGI398264, SGI398265, SGI398266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398267, SGI0, SGI0, SGI0, SGI398268, NULL};
const Segment * const SG99931[] = {SGI398269, SGI398270, SGI398271, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398272, SGI0, SGI0, SGI0, SGI398273, NULL};
const Segment * const SG99932[] = {SGI398274, SGI398275, SGI398276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398277, SGI0, SGI0, SGI0, SGI398278, NULL};
const Segment * const SG99933[] = {SGI398279, SGI0, NULL};
const Segment * const SG99934[] = {SGI398280, SGI398282, NULL};
const Segment * const SG99935[] = {SGI398281, SGI398283, SGI398284, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398285, SGI0, SGI0, SGI0, SGI398286, NULL};
const Segment * const SG99936[] = {SGI398287, SGI398288, NULL};
const Segment * const SG99937[] = {SGI398289, SGI398291, NULL};
const Segment * const SG99938[] = {SGI398290, SGI398292, SGI398293, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398294, SGI0, SGI0, SGI0, SGI398295, NULL};
const Segment * const SG99939[] = {SGI398296, SGI398297, SGI398298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398299, SGI0, SGI0, SGI0, SGI398300, NULL};
const Segment * const SG99940[] = {SGI398301, SGI398302, SGI398303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398304, SGI0, SGI0, SGI0, SGI398305, NULL};
const Segment * const SG99941[] = {SGI398306, SGI398307, SGI398308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398309, SGI0, SGI0, SGI0, SGI398310, NULL};
const Segment * const SG99942[] = {SGI398311, SGI398312, SGI398313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398314, SGI0, SGI0, SGI0, SGI398315, NULL};
const Segment * const SG99943[] = {SGI398316, SGI398317, SGI398318, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398319, SGI0, SGI0, SGI0, SGI398320, NULL};
const Segment * const SG99944[] = {SGI398321, SGI398322, SGI398323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398324, SGI0, SGI0, SGI0, SGI398325, NULL};
const Segment * const SG99945[] = {SGI398326, SGI398327, NULL};
const Segment * const SG99946[] = {SGI398328, SGI398329, NULL};
const Segment * const SG99947[] = {SGI398330, SGI398331, NULL};
const Segment * const SG99948[] = {SGI398332, SGI398333, NULL};
const Segment * const SG99949[] = {SGI398334, SGI398335, NULL};
const Segment * const SG99950[] = {SGI398336, SGI398337, NULL};
const Segment * const SG99951[] = {SGI398338, SGI398339, NULL};
const Segment * const SG99952[] = {SGI398340, SGI398341, NULL};
const Segment * const SG99953[] = {SGI398342, SGI398343, NULL};
const Segment * const SG99954[] = {SGI398344, SGI398345, NULL};
const Segment * const SG99955[] = {SGI398346, SGI398347, NULL};
const Segment * const SG99956[] = {SGI398348, SGI0, NULL};
const Segment * const SG99957[] = {SGI398349, SGI0, NULL};
const Segment * const SG99958[] = {SGI398350, SGI398351, NULL};
const Segment * const SG99959[] = {SGI398352, SGI398353, NULL};
const Segment * const SG99960[] = {SGI398354, SGI398355, NULL};
const Segment * const SG99961[] = {SGI398356, SGI0, NULL};
const Segment * const SG99962[] = {SGI398357, SGI398358, NULL};
const Segment * const SG99963[] = {SGI398359, SGI398360, NULL};
const Segment * const SG99964[] = {SGI398361, SGI398362, NULL};
const Segment * const SG99965[] = {SGI398363, SGI398364, NULL};
const Segment * const SG99966[] = {SGI398365, SGI398366, NULL};
const Segment * const SG99967[] = {SGI398367, SGI398368, NULL};
const Segment * const SG99968[] = {SGI398369, SGI398370, NULL};
const Segment * const SG99969[] = {SGI398371, SGI398372, NULL};
const Segment * const SG99970[] = {SGI398373, SGI398374, NULL};
const Segment * const SG99971[] = {SGI398375, SGI398376, NULL};
const Segment * const SG99972[] = {SGI398377, SGI398378, NULL};
const Segment * const SG99973[] = {SGI398379, SGI398380, NULL};
const Segment * const SG99974[] = {SGI398381, SGI398382, NULL};
const Segment * const SG99975[] = {SGI398383, SGI398384, NULL};
const Segment * const SG99976[] = {SGI398385, SGI398386, NULL};
const Segment * const SG99977[] = {SGI398387, SGI398388, NULL};
const Segment * const SG99978[] = {SGI398389, SGI398390, NULL};
const Segment * const SG99979[] = {SGI398391, SGI398392, NULL};
const Segment * const SG99980[] = {SGI398393, SGI398394, NULL};
const Segment * const SG99981[] = {SGI398395, SGI398396, NULL};
const Segment * const SG99982[] = {SGI398397, SGI398398, NULL};
const Segment * const SG99983[] = {SGI398399, SGI398400, NULL};
const Segment * const SG99984[] = {SGI398401, SGI398402, NULL};
const Segment * const SG99985[] = {SGI398403, SGI398404, NULL};
const Segment * const SG99986[] = {SGI398405, SGI398406, NULL};
const Segment * const SG99987[] = {SGI398407, SGI398408, NULL};
const Segment * const SG99988[] = {SGI398409, SGI398410, NULL};
const Segment * const SG99989[] = {SGI398411, SGI398412, NULL};
const Segment * const SG99990[] = {SGI398413, SGI398414, NULL};
const Segment * const SG99991[] = {SGI398415, SGI398416, NULL};
const Segment * const SG99992[] = {SGI398417, SGI398418, NULL};
const Segment * const SG99993[] = {SGI398419, SGI398420, NULL};
const Segment * const SG99994[] = {SGI398421, SGI398422, NULL};
const Segment * const SG99995[] = {SGI398423, SGI398424, NULL};
const Segment * const SG99996[] = {SGI398425, SGI398426, NULL};
const Segment * const SG99997[] = {SGI398427, SGI398428, NULL};
const Segment * const SG99998[] = {SGI398429, SGI398430, NULL};
const Segment * const SG99999[] = {SGI398431, SGI398432, NULL};
const Segment * const SG100000[] = {SGI398433, SGI398434, NULL};
const Segment * const SG100001[] = {SGI398435, SGI398436, NULL};
const Segment * const SG100002[] = {SGI398437, SGI398438, NULL};
const Segment * const SG100003[] = {SGI398439, SGI398440, NULL};
const Segment * const SG100004[] = {SGI398441, SGI398442, NULL};
const Segment * const SG100005[] = {SGI398443, SGI398444, NULL};
const Segment * const SG100006[] = {SGI398445, SGI398446, NULL};
const Segment * const SG100007[] = {SGI398447, SGI398448, NULL};
const Segment * const SG100008[] = {SGI398449, SGI398450, NULL};
const Segment * const SG100009[] = {SGI398451, SGI0, NULL};
const Segment * const SG100010[] = {SGI398452, SGI0, NULL};
const Segment * const SG100011[] = {SGI398453, SGI398454, NULL};
const Segment * const SG100012[] = {SGI398455, SGI0, NULL};
const Segment * const SG100013[] = {SGI398456, SGI0, NULL};
const Segment * const SG100014[] = {SGI398457, SGI0, NULL};
const Segment * const SG100015[] = {SGI398458, SGI398459, NULL};
const Segment * const SG100016[] = {SGI398460, SGI398461, NULL};
const Segment * const SG100017[] = {SGI398462, SGI398463, NULL};
const Segment * const SG100018[] = {SGI398464, SGI398465, NULL};
const Segment * const SG100019[] = {SGI398466, SGI398467, NULL};
const Segment * const SG100020[] = {SGI398468, SGI398469, NULL};
const Segment * const SG100021[] = {SGI398470, SGI398471, NULL};
const Segment * const SG100022[] = {SGI398472, SGI398473, NULL};
const Segment * const SG100023[] = {SGI398474, SGI398475, NULL};
const Segment * const SG100024[] = {SGI398476, SGI398477, NULL};
const Segment * const SG100025[] = {SGI398478, SGI398479, NULL};
const Segment * const SG100026[] = {SGI398480, SGI398481, NULL};
const Segment * const SG100027[] = {SGI398482, SGI398483, NULL};
const Segment * const SG100028[] = {SGI398484, SGI398485, NULL};
const Segment * const SG100029[] = {SGI398486, SGI398487, NULL};
const Segment * const SG100030[] = {SGI398488, SGI398489, NULL};
const Segment * const SG100031[] = {SGI398490, SGI398491, NULL};
const Segment * const SG100032[] = {SGI398492, SGI398493, NULL};
const Segment * const SG100033[] = {SGI398494, SGI398495, NULL};
const Segment * const SG100034[] = {SGI398496, SGI398497, NULL};
const Segment * const SG100035[] = {SGI398498, SGI398499, NULL};
const Segment * const SG100036[] = {SGI398500, SGI398501, NULL};
const Segment * const SG100037[] = {SGI398502, SGI398503, NULL};
const Segment * const SG100038[] = {SGI398504, SGI398505, NULL};
const Segment * const SG100039[] = {SGI398506, SGI398507, NULL};
const Segment * const SG100040[] = {SGI398508, SGI0, SGI398509, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398510, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100041[] = {SGI398511, SGI0, SGI398512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398513, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100042[] = {SGI398514, SGI0, SGI398515, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398516, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100043[] = {SGI398517, SGI0, SGI398518, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398519, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100044[] = {SGI398520, SGI0, SGI398521, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398522, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100045[] = {SGI398523, SGI398524, NULL};
const Segment * const SG100046[] = {SGI398525, SGI398526, NULL};
const Segment * const SG100047[] = {SGI398527, SGI398528, NULL};
const Segment * const SG100048[] = {SGI398529, SGI398530, NULL};
const Segment * const SG100049[] = {SGI398531, SGI398532, NULL};
const Segment * const SG100050[] = {SGI398533, SGI398534, NULL};
const Segment * const SG100051[] = {SGI398535, SGI398536, NULL};
const Segment * const SG100052[] = {SGI398537, SGI398538, NULL};
const Segment * const SG100053[] = {SGI398539, SGI398540, NULL};
const Segment * const SG100054[] = {SGI398541, SGI398542, NULL};
const Segment * const SG100055[] = {SGI398543, SGI398544, NULL};
const Segment * const SG100056[] = {SGI398545, SGI398546, SGI398547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398548, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100057[] = {SGI398549, SGI0, SGI398550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100058[] = {SGI398551, SGI398552, SGI398553, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398554, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100059[] = {SGI398555, SGI398556, SGI398557, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398558, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100060[] = {SGI398559, SGI398560, SGI398561, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398562, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100061[] = {SGI398563, SGI398564, NULL};
const Segment * const SG100062[] = {SGI398565, SGI398566, SGI398567, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398568, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100063[] = {SGI398569, SGI398570, SGI398571, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398572, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100064[] = {SGI398573, SGI398574, SGI398575, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398576, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100065[] = {SGI398577, SGI398578, SGI398579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398580, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100066[] = {SGI398581, SGI398582, SGI398583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398584, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100067[] = {SGI398585, SGI398586, SGI398587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398588, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100068[] = {SGI398589, SGI0, SGI398590, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100069[] = {SGI398591, SGI0, SGI398592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100070[] = {SGI398593, SGI398594, NULL};
const Segment * const SG100071[] = {SGI398595, SGI398596, NULL};
const Segment * const SG100072[] = {SGI398597, SGI0, NULL};
const Segment * const SG100073[] = {SGI398598, SGI0, NULL};
const Segment * const SG100074[] = {SGI398599, SGI398600, NULL};
const Segment * const SG100075[] = {SGI398601, SGI398602, NULL};
const Segment * const SG100076[] = {SGI398603, SGI398604, NULL};
const Segment * const SG100077[] = {SGI398605, SGI398606, NULL};
const Segment * const SG100078[] = {SGI398607, SGI398608, NULL};
const Segment * const SG100079[] = {SGI398609, SGI398610, NULL};
const Segment * const SG100080[] = {SGI398611, SGI398612, NULL};
const Segment * const SG100081[] = {SGI398613, SGI398614, NULL};
const Segment * const SG100082[] = {SGI398615, SGI398616, NULL};
const Segment * const SG100083[] = {SGI398617, SGI398618, NULL};
const Segment * const SG100084[] = {SGI398619, SGI398620, NULL};
const Segment * const SG100085[] = {SGI398621, SGI398622, NULL};
const Segment * const SG100086[] = {SGI398623, SGI398624, NULL};
const Segment * const SG100087[] = {SGI398625, SGI398626, NULL};
const Segment * const SG100088[] = {SGI398627, SGI398628, NULL};
const Segment * const SG100089[] = {SGI398629, SGI398630, NULL};
const Segment * const SG100090[] = {SGI398631, SGI398632, NULL};
const Segment * const SG100091[] = {SGI398633, SGI398634, NULL};
const Segment * const SG100092[] = {SGI398635, SGI398636, NULL};
const Segment * const SG100093[] = {SGI398637, SGI398638, NULL};
const Segment * const SG100094[] = {SGI398639, SGI398640, NULL};
const Segment * const SG100095[] = {SGI398641, SGI398642, NULL};
const Segment * const SG100096[] = {SGI398643, SGI398644, NULL};
const Segment * const SG100097[] = {SGI398645, SGI398646, NULL};
const Segment * const SG100098[] = {SGI398647, SGI398648, NULL};
const Segment * const SG100099[] = {SGI398649, SGI398650, NULL};
const Segment * const SG100100[] = {SGI398651, SGI398652, NULL};
const Segment * const SG100101[] = {SGI398653, SGI398654, NULL};
const Segment * const SG100102[] = {SGI398655, SGI398656, NULL};
const Segment * const SG100103[] = {SGI398657, SGI398658, NULL};
const Segment * const SG100104[] = {SGI398659, SGI398660, NULL};
const Segment * const SG100105[] = {SGI398661, SGI398662, NULL};
const Segment * const SG100106[] = {SGI398663, SGI398664, NULL};
const Segment * const SG100107[] = {SGI398665, SGI398666, NULL};
const Segment * const SG100108[] = {SGI398667, SGI398668, NULL};
const Segment * const SG100109[] = {SGI398669, SGI398670, NULL};
const Segment * const SG100110[] = {SGI398671, SGI398672, NULL};
const Segment * const SG100111[] = {SGI398673, SGI398674, NULL};
const Segment * const SG100112[] = {SGI398675, SGI0, SGI398676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398677, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100113[] = {SGI398678, SGI398679, NULL};
const Segment * const SG100114[] = {SGI398680, SGI398681, NULL};
const Segment * const SG100115[] = {SGI398682, SGI398683, NULL};
const Segment * const SG100116[] = {SGI398684, SGI398685, NULL};
const Segment * const SG100117[] = {SGI398686, SGI398687, NULL};
const Segment * const SG100118[] = {SGI398688, SGI398689, NULL};
const Segment * const SG100119[] = {SGI398690, SGI398691, NULL};
const Segment * const SG100120[] = {SGI398692, SGI398693, NULL};
const Segment * const SG100121[] = {SGI398694, SGI398695, NULL};
const Segment * const SG100122[] = {SGI398696, SGI398697, NULL};
const Segment * const SG100123[] = {SGI398698, SGI398699, NULL};
const Segment * const SG100124[] = {SGI398700, SGI398701, NULL};
const Segment * const SG100125[] = {SGI398702, SGI398703, NULL};
const Segment * const SG100126[] = {SGI398704, SGI398705, NULL};
const Segment * const SG100127[] = {SGI398706, SGI398707, NULL};
const Segment * const SG100128[] = {SGI398708, SGI398709, NULL};
const Segment * const SG100129[] = {SGI398710, SGI0, NULL};
const Segment * const SG100130[] = {SGI398711, SGI398712, NULL};
const Segment * const SG100131[] = {SGI398713, SGI398714, NULL};
const Segment * const SG100132[] = {SGI398715, SGI0, SGI398716, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398717, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100133[] = {SGI398718, SGI398719, NULL};
const Segment * const SG100134[] = {SGI398720, SGI398721, NULL};
const Segment * const SG100135[] = {SGI398722, SGI398723, NULL};
const Segment * const SG100136[] = {SGI398724, SGI398725, NULL};
const Segment * const SG100137[] = {SGI398726, SGI398727, NULL};
const Segment * const SG100138[] = {SGI398728, SGI398729, NULL};
const Segment * const SG100139[] = {SGI398730, SGI398731, NULL};
const Segment * const SG100140[] = {SGI398732, SGI398733, NULL};
const Segment * const SG100141[] = {SGI398734, SGI398735, NULL};
const Segment * const SG100142[] = {SGI398736, SGI398737, NULL};
const Segment * const SG100143[] = {SGI398738, SGI398739, NULL};
const Segment * const SG100144[] = {SGI398740, SGI398741, NULL};
const Segment * const SG100145[] = {SGI398742, SGI398743, NULL};
const Segment * const SG100146[] = {SGI398744, SGI398745, NULL};
const Segment * const SG100147[] = {SGI398746, SGI398747, NULL};
const Segment * const SG100148[] = {SGI398748, SGI398749, NULL};
const Segment * const SG100149[] = {SGI398750, SGI398751, NULL};
const Segment * const SG100150[] = {SGI398752, SGI398753, NULL};
const Segment * const SG100151[] = {SGI398754, SGI398755, NULL};
const Segment * const SG100152[] = {SGI398756, SGI398757, NULL};
const Segment * const SG100153[] = {SGI398758, SGI398759, NULL};
const Segment * const SG100154[] = {SGI398760, SGI398761, NULL};
const Segment * const SG100155[] = {SGI398762, SGI398763, NULL};
const Segment * const SG100156[] = {SGI398764, SGI398765, NULL};
const Segment * const SG100157[] = {SGI398766, SGI398767, NULL};
const Segment * const SG100158[] = {SGI398768, SGI398769, NULL};
const Segment * const SG100159[] = {SGI398770, SGI398771, NULL};
const Segment * const SG100160[] = {SGI398772, SGI398773, NULL};
const Segment * const SG100161[] = {SGI398774, SGI398775, NULL};
const Segment * const SG100162[] = {SGI398776, SGI398777, NULL};
const Segment * const SG100163[] = {SGI398778, SGI398779, NULL};
const Segment * const SG100164[] = {SGI398780, SGI0, SGI398781, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398782, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100165[] = {SGI398783, SGI0, SGI398784, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398785, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100166[] = {SGI398786, SGI0, SGI398787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398788, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100167[] = {SGI398789, SGI0, SGI398790, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398791, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100168[] = {SGI398792, SGI0, SGI398793, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398794, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100169[] = {SGI398795, SGI0, SGI398796, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398797, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100170[] = {SGI398798, SGI0, SGI398799, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398800, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100171[] = {SGI398801, SGI398802, NULL};
const Segment * const SG100172[] = {SGI398803, SGI398804, NULL};
const Segment * const SG100173[] = {SGI398805, SGI398806, NULL};
const Segment * const SG100174[] = {SGI398807, SGI398808, NULL};
const Segment * const SG100175[] = {SGI398809, SGI398810, NULL};
const Segment * const SG100176[] = {SGI398811, SGI398812, NULL};
const Segment * const SG100177[] = {SGI398813, SGI398814, NULL};
const Segment * const SG100178[] = {SGI398815, SGI398816, NULL};
const Segment * const SG100179[] = {SGI398817, SGI398818, NULL};
const Segment * const SG100180[] = {SGI398819, SGI398820, NULL};
const Segment * const SG100181[] = {SGI398821, SGI398822, NULL};
const Segment * const SG100182[] = {SGI398823, SGI398824, NULL};
const Segment * const SG100183[] = {SGI398825, SGI398826, NULL};
const Segment * const SG100184[] = {SGI398827, SGI398828, NULL};
const Segment * const SG100185[] = {SGI398829, SGI398830, NULL};
const Segment * const SG100186[] = {SGI398831, SGI0, SGI398832, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398833, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100187[] = {SGI398834, SGI398835, NULL};
const Segment * const SG100188[] = {SGI398836, SGI398837, NULL};
const Segment * const SG100189[] = {SGI398838, SGI0, NULL};
const Segment * const SG100190[] = {SGI398839, SGI398840, NULL};
const Segment * const SG100191[] = {SGI398841, SGI398842, NULL};
const Segment * const SG100192[] = {SGI398843, SGI398844, NULL};
const Segment * const SG100193[] = {SGI398845, SGI398846, NULL};
const Segment * const SG100194[] = {SGI398847, SGI398848, NULL};
const Segment * const SG100195[] = {SGI398849, SGI398850, NULL};
const Segment * const SG100196[] = {SGI398851, SGI398852, NULL};
const Segment * const SG100197[] = {SGI398853, SGI398854, NULL};
const Segment * const SG100198[] = {SGI398855, SGI398856, NULL};
const Segment * const SG100199[] = {SGI398857, SGI0, NULL};
const Segment * const SG100200[] = {SGI398858, SGI0, SGI398859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI398860, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100201[] = {SGI398861, SGI398862, NULL};
const Segment * const SG100202[] = {SGI398863, SGI398864, NULL};
const Segment * const SG100203[] = {SGI398865, SGI398866, NULL};
const Segment * const SG100204[] = {SGI398867, SGI398868, NULL};
const Segment * const SG100205[] = {SGI398869, SGI398870, NULL};
const Segment * const SG100206[] = {SGI398871, SGI398872, NULL};
const Segment * const SG100207[] = {SGI398873, SGI398874, NULL};
const Segment * const SG100208[] = {SGI398875, SGI398876, NULL};
const Segment * const SG100209[] = {SGI398877, SGI398878, NULL};
const Segment * const SG100210[] = {SGI398879, SGI398880, NULL};
const Segment * const SG100211[] = {SGI398881, SGI398882, NULL};
const Segment * const SG100212[] = {SGI398883, SGI398884, NULL};
const Segment * const SG100213[] = {SGI398885, SGI398886, NULL};
const Segment * const SG100214[] = {SGI398887, SGI398888, NULL};
const Segment * const SG100215[] = {SGI398889, SGI398890, NULL};
const Segment * const SG100216[] = {SGI398891, SGI398892, NULL};
const Segment * const SG100217[] = {SGI398893, SGI398894, NULL};
const Segment * const SG100218[] = {SGI398895, SGI398896, NULL};
const Segment * const SG100219[] = {SGI398897, SGI398898, NULL};
const Segment * const SG100220[] = {SGI398899, SGI398900, NULL};
const Segment * const SG100221[] = {SGI398901, SGI398902, NULL};
const Segment * const SG100222[] = {SGI398903, SGI398904, NULL};
const Segment * const SG100223[] = {SGI398905, SGI398906, NULL};
const Segment * const SG100224[] = {SGI398907, SGI398908, NULL};
const Segment * const SG100225[] = {SGI398909, SGI398910, NULL};
const Segment * const SG100226[] = {SGI398911, SGI398912, NULL};
const Segment * const SG100227[] = {SGI398913, SGI398914, NULL};
const Segment * const SG100228[] = {SGI398915, SGI398916, NULL};
const Segment * const SG100229[] = {SGI398917, SGI398918, NULL};
const Segment * const SG100230[] = {SGI398919, SGI398920, NULL};
const Segment * const SG100231[] = {SGI398921, SGI398922, NULL};
const Segment * const SG100232[] = {SGI398923, SGI398924, NULL};
const Segment * const SG100233[] = {SGI398925, SGI398926, NULL};
const Segment * const SG100234[] = {SGI398927, SGI398928, NULL};
const Segment * const SG100235[] = {SGI398929, SGI398930, NULL};
const Segment * const SG100236[] = {SGI398931, SGI398932, NULL};
const Segment * const SG100237[] = {SGI398933, SGI398934, NULL};
const Segment * const SG100238[] = {SGI398935, SGI398936, NULL};
const Segment * const SG100239[] = {SGI398937, SGI398938, NULL};
const Segment * const SG100240[] = {SGI398939, SGI398940, NULL};
const Segment * const SG100241[] = {SGI398941, SGI398942, NULL};
const Segment * const SG100242[] = {SGI398943, SGI398944, NULL};
const Segment * const SG100243[] = {SGI398945, SGI398946, NULL};
const Segment * const SG100244[] = {SGI398947, SGI398948, NULL};
const Segment * const SG100245[] = {SGI398949, SGI398950, NULL};
const Segment * const SG100246[] = {SGI398951, SGI398952, NULL};
const Segment * const SG100247[] = {SGI398953, SGI398954, NULL};
const Segment * const SG100248[] = {SGI398955, SGI398956, NULL};
const Segment * const SG100249[] = {SGI398957, SGI398958, NULL};
const Segment * const SG100250[] = {SGI398959, SGI398960, NULL};
const Segment * const SG100251[] = {SGI398961, SGI398962, NULL};
const Segment * const SG100252[] = {SGI398963, SGI398964, NULL};
const Segment * const SG100253[] = {SGI398965, SGI398966, NULL};
const Segment * const SG100254[] = {SGI398967, SGI398968, NULL};
const Segment * const SG100255[] = {SGI398969, SGI398970, NULL};
const Segment * const SG100256[] = {SGI398971, SGI398972, NULL};
const Segment * const SG100257[] = {SGI398973, SGI398974, NULL};
const Segment * const SG100258[] = {SGI398975, SGI0, NULL};
const Segment * const SG100259[] = {SGI398976, SGI398977, NULL};
const Segment * const SG100260[] = {SGI398978, SGI398979, NULL};
const Segment * const SG100261[] = {SGI398980, SGI398981, NULL};
const Segment * const SG100262[] = {SGI398982, SGI398983, NULL};
const Segment * const SG100263[] = {SGI398984, SGI398985, NULL};
const Segment * const SG100264[] = {SGI398986, SGI398987, NULL};
const Segment * const SG100265[] = {SGI398988, SGI398989, NULL};
const Segment * const SG100266[] = {SGI398990, SGI398991, NULL};
const Segment * const SG100267[] = {SGI398992, SGI398993, NULL};
const Segment * const SG100268[] = {SGI398994, SGI398995, NULL};
const Segment * const SG100269[] = {SGI398996, SGI398997, NULL};
const Segment * const SG100270[] = {SGI398998, SGI398999, NULL};
const Segment * const SG100271[] = {SGI399000, SGI399001, NULL};
const Segment * const SG100272[] = {SGI399002, SGI399003, NULL};
const Segment * const SG100273[] = {SGI399004, SGI399005, NULL};
const Segment * const SG100274[] = {SGI399006, SGI399007, NULL};
const Segment * const SG100275[] = {SGI399008, SGI399009, NULL};
const Segment * const SG100276[] = {SGI399010, SGI399011, NULL};
const Segment * const SG100277[] = {SGI399012, SGI399013, NULL};
const Segment * const SG100278[] = {SGI399014, SGI399015, NULL};
const Segment * const SG100279[] = {SGI399016, SGI399017, NULL};
const Segment * const SG100280[] = {SGI399018, SGI399019, NULL};
const Segment * const SG100281[] = {SGI399020, SGI399021, NULL};
const Segment * const SG100282[] = {SGI399022, SGI399023, NULL};
const Segment * const SG100283[] = {SGI399024, SGI399025, NULL};
const Segment * const SG100284[] = {SGI399026, SGI399027, NULL};
const Segment * const SG100285[] = {SGI399028, SGI399029, NULL};
const Segment * const SG100286[] = {SGI399030, SGI399031, NULL};
const Segment * const SG100287[] = {SGI399032, SGI399033, NULL};
const Segment * const SG100288[] = {SGI399034, SGI399035, NULL};
const Segment * const SG100289[] = {SGI399036, SGI399037, NULL};
const Segment * const SG100290[] = {SGI399038, SGI399039, NULL};
const Segment * const SG100291[] = {SGI399040, SGI399041, NULL};
const Segment * const SG100292[] = {SGI399042, SGI399043, NULL};
const Segment * const SG100293[] = {SGI399044, SGI399045, NULL};
const Segment * const SG100294[] = {SGI399046, SGI399047, NULL};
const Segment * const SG100295[] = {SGI399048, SGI399049, NULL};
const Segment * const SG100296[] = {SGI399050, SGI399051, NULL};
const Segment * const SG100297[] = {SGI399052, SGI399053, NULL};
const Segment * const SG100298[] = {SGI399054, SGI399055, NULL};
const Segment * const SG100299[] = {SGI399056, SGI399057, NULL};
const Segment * const SG100300[] = {SGI399058, SGI399059, NULL};
const Segment * const SG100301[] = {SGI399060, SGI399061, NULL};
const Segment * const SG100302[] = {SGI399062, SGI399063, NULL};
const Segment * const SG100303[] = {SGI399064, SGI399065, NULL};
const Segment * const SG100304[] = {SGI399066, SGI399067, NULL};
const Segment * const SG100305[] = {SGI399068, SGI399069, NULL};
const Segment * const SG100306[] = {SGI399070, SGI399071, NULL};
const Segment * const SG100307[] = {SGI399072, SGI399073, NULL};
const Segment * const SG100308[] = {SGI399074, SGI399075, NULL};
const Segment * const SG100309[] = {SGI399076, SGI399077, NULL};
const Segment * const SG100310[] = {SGI399078, SGI399079, NULL};
const Segment * const SG100311[] = {SGI399080, SGI399081, NULL};
const Segment * const SG100312[] = {SGI399082, SGI399083, NULL};
const Segment * const SG100313[] = {SGI399084, SGI399085, NULL};
const Segment * const SG100314[] = {SGI399086, SGI399087, NULL};
const Segment * const SG100315[] = {SGI399088, SGI399089, NULL};
const Segment * const SG100316[] = {SGI399090, SGI399091, NULL};
const Segment * const SG100317[] = {SGI399092, SGI399093, NULL};
const Segment * const SG100318[] = {SGI399094, SGI399095, NULL};
const Segment * const SG100319[] = {SGI399096, SGI399097, NULL};
const Segment * const SG100320[] = {SGI399098, SGI399099, NULL};
const Segment * const SG100321[] = {SGI399100, SGI399101, NULL};
const Segment * const SG100322[] = {SGI399102, SGI399103, NULL};
const Segment * const SG100323[] = {SGI399104, SGI399105, NULL};
const Segment * const SG100324[] = {SGI399106, SGI399107, NULL};
const Segment * const SG100325[] = {SGI399108, SGI399109, NULL};
const Segment * const SG100326[] = {SGI399110, SGI399111, NULL};
const Segment * const SG100327[] = {SGI399112, SGI399113, NULL};
const Segment * const SG100328[] = {SGI399114, SGI399115, NULL};
const Segment * const SG100329[] = {SGI399116, SGI399117, NULL};
const Segment * const SG100330[] = {SGI399118, SGI399119, NULL};
const Segment * const SG100331[] = {SGI399120, SGI399121, NULL};
const Segment * const SG100332[] = {SGI399122, SGI399123, NULL};
const Segment * const SG100333[] = {SGI399124, SGI399125, NULL};
const Segment * const SG100334[] = {SGI399126, SGI399127, NULL};
const Segment * const SG100335[] = {SGI399128, SGI399129, NULL};
const Segment * const SG100336[] = {SGI399130, SGI0, SGI399131, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399132, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100337[] = {SGI399133, SGI0, SGI399134, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399135, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100338[] = {SGI399136, SGI399137, SGI399138, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399139, SGI399140, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100339[] = {SGI399141, SGI399142, NULL};
const Segment * const SG100340[] = {SGI399143, SGI399144, NULL};
const Segment * const SG100341[] = {SGI399145, SGI399146, NULL};
const Segment * const SG100342[] = {SGI399147, SGI399148, NULL};
const Segment * const SG100343[] = {SGI399149, SGI399150, NULL};
const Segment * const SG100344[] = {SGI399151, SGI399152, NULL};
const Segment * const SG100345[] = {SGI399153, SGI399154, NULL};
const Segment * const SG100346[] = {SGI399155, SGI399156, NULL};
const Segment * const SG100347[] = {SGI399157, SGI399158, NULL};
const Segment * const SG100348[] = {SGI399159, SGI0, NULL};
const Segment * const SG100349[] = {SGI399160, SGI399161, NULL};
const Segment * const SG100350[] = {SGI399162, SGI399163, NULL};
const Segment * const SG100351[] = {SGI399164, SGI399165, NULL};
const Segment * const SG100352[] = {SGI399166, SGI399167, NULL};
const Segment * const SG100353[] = {SGI399168, SGI399169, NULL};
const Segment * const SG100354[] = {SGI399170, SGI399171, NULL};
const Segment * const SG100355[] = {SGI399172, SGI399173, NULL};
const Segment * const SG100356[] = {SGI399174, SGI399175, NULL};
const Segment * const SG100357[] = {SGI399176, SGI399177, NULL};
const Segment * const SG100358[] = {SGI399178, SGI399179, NULL};
const Segment * const SG100359[] = {SGI399180, SGI399181, NULL};
const Segment * const SG100360[] = {SGI399182, SGI399183, NULL};
const Segment * const SG100361[] = {SGI399184, SGI399185, NULL};
const Segment * const SG100362[] = {SGI399186, SGI399187, NULL};
const Segment * const SG100363[] = {SGI399188, SGI399189, NULL};
const Segment * const SG100364[] = {SGI399190, SGI399191, NULL};
const Segment * const SG100365[] = {SGI399192, SGI399193, NULL};
const Segment * const SG100366[] = {SGI399194, SGI399195, NULL};
const Segment * const SG100367[] = {SGI399196, SGI0, NULL};
const Segment * const SG100368[] = {SGI399197, SGI399198, NULL};
const Segment * const SG100369[] = {SGI399199, SGI399200, NULL};
const Segment * const SG100370[] = {SGI399201, SGI399202, NULL};
const Segment * const SG100371[] = {SGI399203, SGI399204, NULL};
const Segment * const SG100372[] = {SGI399205, SGI399206, NULL};
const Segment * const SG100373[] = {SGI399207, SGI399208, NULL};
const Segment * const SG100374[] = {SGI399209, SGI399210, NULL};
const Segment * const SG100375[] = {SGI399211, SGI399212, NULL};
const Segment * const SG100376[] = {SGI399213, SGI399214, NULL};
const Segment * const SG100377[] = {SGI399215, SGI399216, NULL};
const Segment * const SG100378[] = {SGI399217, SGI399218, NULL};
const Segment * const SG100379[] = {SGI399219, SGI399220, NULL};
const Segment * const SG100380[] = {SGI399221, SGI399222, NULL};
const Segment * const SG100381[] = {SGI399223, SGI399224, NULL};
const Segment * const SG100382[] = {SGI399225, SGI399226, NULL};
const Segment * const SG100383[] = {SGI399227, SGI399228, NULL};
const Segment * const SG100384[] = {SGI399229, SGI399230, NULL};
const Segment * const SG100385[] = {SGI399231, SGI399232, NULL};
const Segment * const SG100386[] = {SGI399233, SGI399234, NULL};
const Segment * const SG100387[] = {SGI399235, SGI399236, NULL};
const Segment * const SG100388[] = {SGI399237, SGI399238, NULL};
const Segment * const SG100389[] = {SGI399239, SGI399240, NULL};
const Segment * const SG100390[] = {SGI399241, SGI399242, NULL};
const Segment * const SG100391[] = {SGI399243, SGI399244, NULL};
const Segment * const SG100392[] = {SGI399245, SGI399246, NULL};
const Segment * const SG100393[] = {SGI399247, SGI399248, NULL};
const Segment * const SG100394[] = {SGI399249, SGI0, NULL};
const Segment * const SG100395[] = {SGI399250, SGI399251, NULL};
const Segment * const SG100396[] = {SGI399252, SGI399253, NULL};
const Segment * const SG100397[] = {SGI399254, SGI399255, NULL};
const Segment * const SG100398[] = {SGI399256, SGI399257, NULL};
const Segment * const SG100399[] = {SGI399258, SGI399259, NULL};
const Segment * const SG100400[] = {SGI399260, SGI399261, NULL};
const Segment * const SG100401[] = {SGI399262, SGI0, SGI399263, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399264, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100402[] = {SGI399265, SGI0, SGI399266, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399267, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100403[] = {SGI399268, SGI399269, NULL};
const Segment * const SG100404[] = {SGI399270, SGI399271, NULL};
const Segment * const SG100405[] = {SGI399272, SGI399273, NULL};
const Segment * const SG100406[] = {SGI399274, SGI399275, SGI399276, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399277, SGI0, SGI0, SGI0, SGI399278, NULL};
const Segment * const SG100407[] = {SGI399279, SGI399280, SGI399281, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399282, SGI0, SGI0, SGI0, SGI399283, NULL};
const Segment * const SG100408[] = {SGI399284, SGI399285, SGI399286, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399287, SGI0, SGI0, SGI0, SGI399288, NULL};
const Segment * const SG100409[] = {SGI399289, SGI399290, SGI399291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399292, SGI0, SGI0, SGI0, SGI399293, NULL};
const Segment * const SG100410[] = {SGI399294, SGI399295, NULL};
const Segment * const SG100411[] = {SGI399296, SGI399297, SGI399298, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399299, SGI0, SGI0, SGI0, SGI399300, NULL};
const Segment * const SG100412[] = {SGI399301, SGI399302, SGI399303, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399304, SGI0, SGI0, SGI0, SGI399305, NULL};
const Segment * const SG100413[] = {SGI399306, SGI399307, SGI399308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399309, SGI0, SGI0, SGI0, SGI399310, NULL};
const Segment * const SG100414[] = {SGI399311, SGI399312, SGI399313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399314, SGI0, SGI0, SGI0, SGI399315, NULL};
const Segment * const SG100415[] = {SGI399316, SGI399317, NULL};
const Segment * const SG100416[] = {SGI399318, SGI399319, NULL};
const Segment * const SG100417[] = {SGI399320, SGI399321, NULL};
const Segment * const SG100418[] = {SGI399322, SGI399323, NULL};
const Segment * const SG100419[] = {SGI399324, SGI399325, NULL};
const Segment * const SG100420[] = {SGI399326, SGI399327, NULL};
const Segment * const SG100421[] = {SGI399328, SGI399329, NULL};
const Segment * const SG100422[] = {SGI399330, SGI399331, NULL};
const Segment * const SG100423[] = {SGI399332, SGI399333, NULL};
const Segment * const SG100424[] = {SGI399334, SGI399335, NULL};
const Segment * const SG100425[] = {SGI399336, SGI399337, NULL};
const Segment * const SG100426[] = {SGI399338, SGI399339, NULL};
const Segment * const SG100427[] = {SGI399340, SGI399341, NULL};
const Segment * const SG100428[] = {SGI399342, SGI399343, NULL};
const Segment * const SG100429[] = {SGI399344, SGI399345, NULL};
const Segment * const SG100430[] = {SGI399346, SGI399347, NULL};
const Segment * const SG100431[] = {SGI399348, SGI399349, NULL};
const Segment * const SG100432[] = {SGI399350, SGI399351, NULL};
const Segment * const SG100433[] = {SGI399352, SGI399353, NULL};
const Segment * const SG100434[] = {SGI399354, SGI399355, NULL};
const Segment * const SG100435[] = {SGI399356, SGI399357, NULL};
const Segment * const SG100436[] = {SGI399358, SGI399359, NULL};
const Segment * const SG100437[] = {SGI399360, SGI399361, NULL};
const Segment * const SG100438[] = {SGI399362, SGI399363, NULL};
const Segment * const SG100439[] = {SGI399364, SGI399365, NULL};
const Segment * const SG100440[] = {SGI399366, SGI399367, NULL};
const Segment * const SG100441[] = {SGI399368, SGI399369, NULL};
const Segment * const SG100442[] = {SGI399370, SGI399371, NULL};
const Segment * const SG100443[] = {SGI399372, SGI399373, NULL};
const Segment * const SG100444[] = {SGI399374, SGI399375, NULL};
const Segment * const SG100445[] = {SGI399376, SGI399377, NULL};
const Segment * const SG100446[] = {SGI399378, SGI399379, NULL};
const Segment * const SG100447[] = {SGI399380, SGI0, NULL};
const Segment * const SG100448[] = {SGI399381, SGI0, SGI399382, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399383, SGI0, SGI399384, SGI0, NULL};
const Segment * const SG100449[] = {SGI399385, SGI399386, NULL};
const Segment * const SG100450[] = {SGI399387, SGI399388, NULL};
const Segment * const SG100451[] = {SGI399389, SGI399390, NULL};
const Segment * const SG100452[] = {SGI399391, SGI399392, NULL};
const Segment * const SG100453[] = {SGI399393, SGI399394, NULL};
const Segment * const SG100454[] = {SGI399395, SGI399396, NULL};
const Segment * const SG100455[] = {SGI399397, SGI399398, NULL};
const Segment * const SG100456[] = {SGI399399, SGI399400, NULL};
const Segment * const SG100457[] = {SGI399401, SGI399402, NULL};
const Segment * const SG100458[] = {SGI399403, SGI399404, NULL};
const Segment * const SG100459[] = {SGI399405, SGI399406, NULL};
const Segment * const SG100460[] = {SGI399407, SGI399408, NULL};
const Segment * const SG100461[] = {SGI399409, SGI399410, NULL};
const Segment * const SG100462[] = {SGI399411, SGI399412, NULL};
const Segment * const SG100463[] = {SGI399413, SGI399414, NULL};
const Segment * const SG100464[] = {SGI399415, SGI399416, NULL};
const Segment * const SG100465[] = {SGI399417, SGI399418, NULL};
const Segment * const SG100466[] = {SGI399419, SGI399420, SGI399421, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399422, SGI0, SGI0, SGI0, SGI399423, NULL};
const Segment * const SG100467[] = {SGI399424, SGI399425, SGI399426, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399427, SGI0, SGI0, SGI0, SGI399428, NULL};
const Segment * const SG100468[] = {SGI399429, SGI399430, SGI399431, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399432, SGI0, SGI0, SGI0, SGI399433, NULL};
const Segment * const SG100469[] = {SGI399434, SGI399436, NULL};
const Segment * const SG100470[] = {SGI399435, SGI399437, SGI399438, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399439, SGI0, SGI0, SGI0, SGI399440, NULL};
const Segment * const SG100471[] = {SGI399444, SGI399445, NULL};
const Segment * const SG100472[] = {SGI399441, SGI399442, SGI399443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399446, SGI0, SGI0, SGI0, SGI399447, NULL};
const Segment * const SG100473[] = {SGI399448, SGI399449, SGI399450, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399451, SGI0, SGI0, SGI0, SGI399452, NULL};
const Segment * const SG100474[] = {SGI399453, SGI399454, SGI399455, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399456, SGI0, SGI0, SGI0, SGI399457, NULL};
const Segment * const SG100475[] = {SGI399458, SGI399459, NULL};
const Segment * const SG100476[] = {SGI399460, SGI399461, NULL};
const Segment * const SG100477[] = {SGI399462, SGI399463, NULL};
const Segment * const SG100478[] = {SGI399464, SGI399465, NULL};
const Segment * const SG100479[] = {SGI399466, SGI399467, NULL};
const Segment * const SG100480[] = {SGI399468, SGI399469, NULL};
const Segment * const SG100481[] = {SGI399470, SGI399471, NULL};
const Segment * const SG100482[] = {SGI399472, SGI399473, NULL};
const Segment * const SG100483[] = {SGI399474, SGI399475, NULL};
const Segment * const SG100484[] = {SGI399476, SGI399477, NULL};
const Segment * const SG100485[] = {SGI399478, SGI399479, NULL};
const Segment * const SG100486[] = {SGI399480, SGI399481, NULL};
const Segment * const SG100487[] = {SGI399482, SGI0, NULL};
const Segment * const SG100488[] = {SGI399483, SGI399484, NULL};
const Segment * const SG100489[] = {SGI399485, SGI399486, SGI399487, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399488, SGI0, SGI0, SGI0, SGI399489, NULL};
const Segment * const SG100490[] = {SGI399490, SGI399491, SGI399492, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399493, SGI0, SGI0, SGI0, SGI399494, NULL};
const Segment * const SG100491[] = {SGI399495, SGI399496, SGI399497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399498, SGI0, SGI0, SGI0, SGI399499, NULL};
const Segment * const SG100492[] = {SGI399500, SGI399501, SGI399502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399503, SGI0, SGI0, SGI0, SGI399504, NULL};
const Segment * const SG100493[] = {SGI399505, SGI399506, SGI399507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399508, SGI0, SGI0, SGI0, SGI399509, NULL};
const Segment * const SG100494[] = {SGI399510, SGI399511, SGI399512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399513, SGI0, SGI0, SGI0, SGI399514, NULL};
const Segment * const SG100495[] = {SGI399515, SGI399516, SGI399517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399518, SGI0, SGI0, SGI0, SGI399519, NULL};
const Segment * const SG100496[] = {SGI399520, SGI399521, SGI399522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399523, SGI0, SGI0, SGI0, SGI399524, NULL};
const Segment * const SG100497[] = {SGI399525, SGI399526, SGI399527, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399528, SGI0, SGI0, SGI0, SGI399529, NULL};
const Segment * const SG100498[] = {SGI399530, SGI399531, SGI399532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399533, SGI0, SGI0, SGI0, SGI399534, NULL};
const Segment * const SG100499[] = {SGI399535, SGI399536, NULL};
const Segment * const SG100500[] = {SGI399537, SGI399538, NULL};
const Segment * const SG100501[] = {SGI399539, SGI399540, NULL};
const Segment * const SG100502[] = {SGI399541, SGI399542, NULL};
const Segment * const SG100503[] = {SGI399543, SGI399544, NULL};
const Segment * const SG100504[] = {SGI399545, SGI399546, NULL};
const Segment * const SG100505[] = {SGI399547, SGI399548, NULL};
const Segment * const SG100506[] = {SGI399549, SGI399550, NULL};
const Segment * const SG100507[] = {SGI399551, SGI399552, NULL};
const Segment * const SG100508[] = {SGI399553, SGI399554, NULL};
const Segment * const SG100509[] = {SGI399555, SGI399556, NULL};
const Segment * const SG100510[] = {SGI399557, SGI399558, NULL};
const Segment * const SG100511[] = {SGI399559, SGI399560, NULL};
const Segment * const SG100512[] = {SGI399561, SGI399562, NULL};
const Segment * const SG100513[] = {SGI399563, SGI399564, NULL};
const Segment * const SG100514[] = {SGI399565, SGI399566, NULL};
const Segment * const SG100515[] = {SGI399567, SGI399568, NULL};
const Segment * const SG100516[] = {SGI399569, SGI399570, NULL};
const Segment * const SG100517[] = {SGI399571, SGI399572, NULL};
const Segment * const SG100518[] = {SGI399573, SGI399574, NULL};
const Segment * const SG100519[] = {SGI399575, SGI399576, NULL};
const Segment * const SG100520[] = {SGI399577, SGI399578, NULL};
const Segment * const SG100521[] = {SGI399579, SGI399580, NULL};
const Segment * const SG100522[] = {SGI399581, SGI399582, NULL};
const Segment * const SG100523[] = {SGI399583, SGI399584, NULL};
const Segment * const SG100524[] = {SGI399585, SGI399586, NULL};
const Segment * const SG100525[] = {SGI399587, SGI399588, NULL};
const Segment * const SG100526[] = {SGI399589, SGI399590, NULL};
const Segment * const SG100527[] = {SGI399591, SGI399592, NULL};
const Segment * const SG100528[] = {SGI399593, SGI399594, NULL};
const Segment * const SG100529[] = {SGI399595, SGI399596, NULL};
const Segment * const SG100530[] = {SGI399597, SGI399598, NULL};
const Segment * const SG100531[] = {SGI399599, SGI399600, NULL};
const Segment * const SG100532[] = {SGI399601, SGI399602, NULL};
const Segment * const SG100533[] = {SGI399603, SGI399604, NULL};
const Segment * const SG100534[] = {SGI399605, SGI399606, NULL};
const Segment * const SG100535[] = {SGI399607, SGI0, NULL};
const Segment * const SG100536[] = {SGI399608, SGI399609, NULL};
const Segment * const SG100537[] = {SGI399610, SGI399611, NULL};
const Segment * const SG100538[] = {SGI399612, SGI399613, NULL};
const Segment * const SG100539[] = {SGI399614, SGI399615, NULL};
const Segment * const SG100540[] = {SGI399616, SGI399617, NULL};
const Segment * const SG100541[] = {SGI399618, SGI399619, NULL};
const Segment * const SG100542[] = {SGI399620, SGI399621, NULL};
const Segment * const SG100543[] = {SGI399622, SGI399623, NULL};
const Segment * const SG100544[] = {SGI399624, SGI399625, NULL};
const Segment * const SG100545[] = {SGI399626, SGI399627, NULL};
const Segment * const SG100546[] = {SGI399628, SGI399629, NULL};
const Segment * const SG100547[] = {SGI399630, SGI399631, NULL};
const Segment * const SG100548[] = {SGI399632, SGI399633, NULL};
const Segment * const SG100549[] = {SGI399634, SGI399635, NULL};
const Segment * const SG100550[] = {SGI399636, SGI399637, NULL};
const Segment * const SG100551[] = {SGI399638, SGI399639, NULL};
const Segment * const SG100552[] = {SGI399640, SGI399641, NULL};
const Segment * const SG100553[] = {SGI399642, SGI399643, NULL};
const Segment * const SG100554[] = {SGI399644, SGI399645, NULL};
const Segment * const SG100555[] = {SGI399646, SGI399647, NULL};
const Segment * const SG100556[] = {SGI399648, SGI399649, NULL};
const Segment * const SG100557[] = {SGI399650, SGI399651, NULL};
const Segment * const SG100558[] = {SGI399652, SGI399653, NULL};
const Segment * const SG100559[] = {SGI399654, SGI399655, NULL};
const Segment * const SG100560[] = {SGI399656, SGI399657, NULL};
const Segment * const SG100561[] = {SGI399658, SGI399659, NULL};
const Segment * const SG100562[] = {SGI399660, SGI399661, NULL};
const Segment * const SG100563[] = {SGI399662, SGI399663, NULL};
const Segment * const SG100564[] = {SGI399664, SGI399665, NULL};
const Segment * const SG100565[] = {SGI399666, SGI399667, NULL};
const Segment * const SG100566[] = {SGI399668, SGI399669, NULL};
const Segment * const SG100567[] = {SGI399670, SGI399671, NULL};
const Segment * const SG100568[] = {SGI399672, SGI399673, NULL};
const Segment * const SG100569[] = {SGI399674, SGI399675, NULL};
const Segment * const SG100570[] = {SGI399676, SGI399677, NULL};
const Segment * const SG100571[] = {SGI399678, SGI399679, NULL};
const Segment * const SG100572[] = {SGI399680, SGI399681, NULL};
const Segment * const SG100573[] = {SGI399682, SGI399683, NULL};
const Segment * const SG100574[] = {SGI399684, SGI399685, NULL};
const Segment * const SG100575[] = {SGI399686, SGI399687, NULL};
const Segment * const SG100576[] = {SGI399688, SGI399689, NULL};
const Segment * const SG100577[] = {SGI399690, SGI399691, NULL};
const Segment * const SG100578[] = {SGI399692, SGI399693, NULL};
const Segment * const SG100579[] = {SGI399694, SGI399695, NULL};
const Segment * const SG100580[] = {SGI399696, SGI399697, NULL};
const Segment * const SG100581[] = {SGI399698, SGI399699, NULL};
const Segment * const SG100582[] = {SGI399700, SGI399701, NULL};
const Segment * const SG100583[] = {SGI399702, SGI399703, NULL};
const Segment * const SG100584[] = {SGI399704, SGI399705, NULL};
const Segment * const SG100585[] = {SGI399706, SGI399707, NULL};
const Segment * const SG100586[] = {SGI399708, SGI399709, NULL};
const Segment * const SG100587[] = {SGI399710, SGI399711, NULL};
const Segment * const SG100588[] = {SGI399712, SGI399713, NULL};
const Segment * const SG100589[] = {SGI399714, SGI399715, NULL};
const Segment * const SG100590[] = {SGI399716, SGI399717, NULL};
const Segment * const SG100591[] = {SGI399718, SGI399719, NULL};
const Segment * const SG100592[] = {SGI399720, SGI399721, NULL};
const Segment * const SG100593[] = {SGI399722, SGI399723, NULL};
const Segment * const SG100594[] = {SGI399724, SGI399725, NULL};
const Segment * const SG100595[] = {SGI399726, SGI399727, NULL};
const Segment * const SG100596[] = {SGI399728, SGI399729, NULL};
const Segment * const SG100597[] = {SGI399730, SGI399731, NULL};
const Segment * const SG100598[] = {SGI399732, SGI399733, NULL};
const Segment * const SG100599[] = {SGI399734, SGI399735, NULL};
const Segment * const SG100600[] = {SGI399736, SGI399737, NULL};
const Segment * const SG100601[] = {SGI399738, SGI399739, NULL};
const Segment * const SG100602[] = {SGI399740, SGI399741, NULL};
const Segment * const SG100603[] = {SGI399742, SGI399743, NULL};
const Segment * const SG100604[] = {SGI399744, SGI399745, NULL};
const Segment * const SG100605[] = {SGI399746, SGI399747, NULL};
const Segment * const SG100606[] = {SGI399748, SGI399749, NULL};
const Segment * const SG100607[] = {SGI399750, SGI399751, NULL};
const Segment * const SG100608[] = {SGI399752, SGI399753, NULL};
const Segment * const SG100609[] = {SGI399754, SGI399755, NULL};
const Segment * const SG100610[] = {SGI399756, SGI399757, NULL};
const Segment * const SG100611[] = {SGI399758, SGI399759, NULL};
const Segment * const SG100612[] = {SGI399760, SGI399761, NULL};
const Segment * const SG100613[] = {SGI399762, SGI399763, NULL};
const Segment * const SG100614[] = {SGI399764, SGI399765, NULL};
const Segment * const SG100615[] = {SGI399766, SGI399767, NULL};
const Segment * const SG100616[] = {SGI399768, SGI399769, NULL};
const Segment * const SG100617[] = {SGI399770, SGI399771, NULL};
const Segment * const SG100618[] = {SGI399772, SGI399773, NULL};
const Segment * const SG100619[] = {SGI399774, SGI399775, NULL};
const Segment * const SG100620[] = {SGI399776, SGI399777, NULL};
const Segment * const SG100621[] = {SGI399778, SGI399779, NULL};
const Segment * const SG100622[] = {SGI399780, SGI399781, NULL};
const Segment * const SG100623[] = {SGI399782, SGI399783, NULL};
const Segment * const SG100624[] = {SGI399784, SGI399785, NULL};
const Segment * const SG100625[] = {SGI399786, SGI399787, NULL};
const Segment * const SG100626[] = {SGI399788, SGI0, SGI399789, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399790, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100627[] = {SGI399791, SGI0, SGI399792, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399793, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100628[] = {SGI399794, SGI399795, NULL};
const Segment * const SG100629[] = {SGI399796, SGI399797, NULL};
const Segment * const SG100630[] = {SGI399798, SGI399799, NULL};
const Segment * const SG100631[] = {SGI399800, SGI399801, NULL};
const Segment * const SG100632[] = {SGI399802, SGI399803, NULL};
const Segment * const SG100633[] = {SGI399804, SGI399805, NULL};
const Segment * const SG100634[] = {SGI399806, SGI399807, NULL};
const Segment * const SG100635[] = {SGI399808, SGI399809, NULL};
const Segment * const SG100636[] = {SGI399810, SGI399811, NULL};
const Segment * const SG100637[] = {SGI399812, SGI399813, NULL};
const Segment * const SG100638[] = {SGI399814, SGI399815, NULL};
const Segment * const SG100639[] = {SGI399816, SGI399817, NULL};
const Segment * const SG100640[] = {SGI399818, SGI399819, NULL};
const Segment * const SG100641[] = {SGI399820, SGI399821, NULL};
const Segment * const SG100642[] = {SGI399822, SGI399823, NULL};
const Segment * const SG100643[] = {SGI399824, SGI399825, NULL};
const Segment * const SG100644[] = {SGI399826, SGI399827, NULL};
const Segment * const SG100645[] = {SGI399828, SGI399829, NULL};
const Segment * const SG100646[] = {SGI399830, SGI399831, NULL};
const Segment * const SG100647[] = {SGI399832, SGI399833, NULL};
const Segment * const SG100648[] = {SGI399834, SGI399835, NULL};
const Segment * const SG100649[] = {SGI399836, SGI399837, NULL};
const Segment * const SG100650[] = {SGI399838, SGI399839, NULL};
const Segment * const SG100651[] = {SGI399840, SGI399841, NULL};
const Segment * const SG100652[] = {SGI399842, SGI399843, SGI399844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399845, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100653[] = {SGI399846, SGI399847, NULL};
const Segment * const SG100654[] = {SGI399848, SGI399849, SGI399850, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399851, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100655[] = {SGI399855, SGI399856, NULL};
const Segment * const SG100656[] = {SGI399852, SGI399853, SGI399854, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399857, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100657[] = {SGI399858, SGI399860, NULL};
const Segment * const SG100658[] = {SGI399859, SGI399861, SGI399862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399863, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100659[] = {SGI399864, SGI0, SGI399865, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100660[] = {SGI399866, SGI399867, SGI399868, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399869, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100661[] = {SGI399870, SGI399871, SGI399872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399873, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100662[] = {SGI399874, SGI399875, NULL};
const Segment * const SG100663[] = {SGI399876, SGI399877, SGI399878, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399879, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100664[] = {SGI399880, SGI0, SGI399881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100665[] = {SGI399882, SGI399883, NULL};
const Segment * const SG100666[] = {SGI399884, SGI399885, NULL};
const Segment * const SG100667[] = {SGI399886, SGI399887, NULL};
const Segment * const SG100668[] = {SGI399888, SGI399889, NULL};
const Segment * const SG100669[] = {SGI399890, SGI399891, NULL};
const Segment * const SG100670[] = {SGI399892, SGI399893, NULL};
const Segment * const SG100671[] = {SGI399894, SGI399895, NULL};
const Segment * const SG100672[] = {SGI399896, SGI399897, NULL};
const Segment * const SG100673[] = {SGI399898, SGI399899, NULL};
const Segment * const SG100674[] = {SGI399900, SGI399901, NULL};
const Segment * const SG100675[] = {SGI399902, SGI399903, NULL};
const Segment * const SG100676[] = {SGI399904, SGI399905, NULL};
const Segment * const SG100677[] = {SGI399906, SGI399907, NULL};
const Segment * const SG100678[] = {SGI399908, SGI399909, NULL};
const Segment * const SG100679[] = {SGI399910, SGI399911, NULL};
const Segment * const SG100680[] = {SGI399912, SGI399913, NULL};
const Segment * const SG100681[] = {SGI399914, SGI399915, NULL};
const Segment * const SG100682[] = {SGI399916, SGI399917, NULL};
const Segment * const SG100683[] = {SGI399918, SGI399919, NULL};
const Segment * const SG100684[] = {SGI399920, SGI399921, NULL};
const Segment * const SG100685[] = {SGI399922, SGI399923, NULL};
const Segment * const SG100686[] = {SGI399924, SGI399925, NULL};
const Segment * const SG100687[] = {SGI399926, SGI399927, NULL};
const Segment * const SG100688[] = {SGI399928, SGI399929, NULL};
const Segment * const SG100689[] = {SGI399930, SGI399931, NULL};
const Segment * const SG100690[] = {SGI399932, SGI399933, NULL};
const Segment * const SG100691[] = {SGI399934, SGI399935, NULL};
const Segment * const SG100692[] = {SGI399936, SGI399937, NULL};
const Segment * const SG100693[] = {SGI399938, SGI399939, NULL};
const Segment * const SG100694[] = {SGI399940, SGI399941, NULL};
const Segment * const SG100695[] = {SGI399942, SGI399943, NULL};
const Segment * const SG100696[] = {SGI399944, SGI399945, NULL};
const Segment * const SG100697[] = {SGI399946, SGI399947, NULL};
const Segment * const SG100698[] = {SGI399948, SGI399949, NULL};
const Segment * const SG100699[] = {SGI399950, SGI399951, NULL};
const Segment * const SG100700[] = {SGI399952, SGI399953, NULL};
const Segment * const SG100701[] = {SGI399954, SGI399955, NULL};
const Segment * const SG100702[] = {SGI399956, SGI399957, NULL};
const Segment * const SG100703[] = {SGI399958, SGI399959, NULL};
const Segment * const SG100704[] = {SGI399960, SGI399961, NULL};
const Segment * const SG100705[] = {SGI399962, SGI0, NULL};
const Segment * const SG100706[] = {SGI399963, SGI399964, NULL};
const Segment * const SG100707[] = {SGI399965, SGI399966, NULL};
const Segment * const SG100708[] = {SGI399967, SGI399968, NULL};
const Segment * const SG100709[] = {SGI399969, SGI0, SGI399970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399971, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100710[] = {SGI399972, SGI399973, NULL};
const Segment * const SG100711[] = {SGI399974, SGI0, SGI399975, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399976, SGI0, SGI399977, SGI0, NULL};
const Segment * const SG100712[] = {SGI399978, SGI399979, NULL};
const Segment * const SG100713[] = {SGI399980, SGI399981, NULL};
const Segment * const SG100714[] = {SGI399982, SGI399983, NULL};
const Segment * const SG100715[] = {SGI399984, SGI399985, NULL};
const Segment * const SG100716[] = {SGI399986, SGI399987, SGI399988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399989, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100717[] = {SGI399990, SGI399991, SGI399992, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399993, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100718[] = {SGI399994, SGI399995, SGI399996, SGI0, SGI0, SGI0, SGI0, SGI0, SGI399997, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100719[] = {SGI399998, SGI0, SGI399999, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100720[] = {SGI400000, SGI400001, SGI400002, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400003, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100721[] = {SGI400004, SGI400005, NULL};
const Segment * const SG100722[] = {SGI400006, SGI400007, NULL};
const Segment * const SG100723[] = {SGI400008, SGI400009, NULL};
const Segment * const SG100724[] = {SGI400010, SGI400011, NULL};
const Segment * const SG100725[] = {SGI400012, SGI400013, NULL};
const Segment * const SG100726[] = {SGI400014, SGI400015, NULL};
const Segment * const SG100727[] = {SGI400016, SGI400017, NULL};
const Segment * const SG100728[] = {SGI400018, SGI400019, NULL};
const Segment * const SG100729[] = {SGI400020, SGI400021, NULL};
const Segment * const SG100730[] = {SGI0, SGI400022, NULL};
const Segment * const SG100731[] = {SGI400023, SGI0, NULL};
const Segment * const SG100732[] = {SGI400024, SGI0, NULL};
const Segment * const SG100733[] = {SGI400025, SGI400026, NULL};
const Segment * const SG100734[] = {SGI400027, SGI400028, NULL};
const Segment * const SG100735[] = {SGI400029, SGI400030, NULL};
const Segment * const SG100736[] = {SGI400031, SGI400032, NULL};
const Segment * const SG100737[] = {SGI400033, SGI400034, NULL};
const Segment * const SG100738[] = {SGI400035, SGI400036, NULL};
const Segment * const SG100739[] = {SGI400037, SGI400038, NULL};
const Segment * const SG100740[] = {SGI400039, SGI400040, NULL};
const Segment * const SG100741[] = {SGI400041, SGI400042, NULL};
const Segment * const SG100742[] = {SGI400043, SGI400044, NULL};
const Segment * const SG100743[] = {SGI400045, SGI400046, NULL};
const Segment * const SG100744[] = {SGI400047, SGI400048, NULL};
const Segment * const SG100745[] = {SGI400049, SGI400050, NULL};
const Segment * const SG100746[] = {SGI400051, SGI400052, NULL};
const Segment * const SG100747[] = {SGI400053, SGI400054, NULL};
const Segment * const SG100748[] = {SGI400055, SGI400056, NULL};
const Segment * const SG100749[] = {SGI400057, SGI400058, NULL};
const Segment * const SG100750[] = {SGI400059, SGI0, NULL};
const Segment * const SG100751[] = {SGI400060, SGI400061, NULL};
const Segment * const SG100752[] = {SGI400062, SGI400063, NULL};
const Segment * const SG100753[] = {SGI400064, SGI400065, NULL};
const Segment * const SG100754[] = {SGI400066, SGI400067, NULL};
const Segment * const SG100755[] = {SGI400068, SGI0, NULL};
const Segment * const SG100756[] = {SGI400069, SGI400070, NULL};
const Segment * const SG100757[] = {SGI400071, SGI400072, NULL};
const Segment * const SG100758[] = {SGI400073, SGI400074, NULL};
const Segment * const SG100759[] = {SGI400075, SGI400076, NULL};
const Segment * const SG100760[] = {SGI400077, SGI400078, NULL};
const Segment * const SG100761[] = {SGI400079, SGI400080, NULL};
const Segment * const SG100762[] = {SGI400081, SGI400082, NULL};
const Segment * const SG100763[] = {SGI400083, SGI400084, NULL};
const Segment * const SG100764[] = {SGI400085, SGI400086, NULL};
const Segment * const SG100765[] = {SGI400087, SGI400088, NULL};
const Segment * const SG100766[] = {SGI400089, SGI400090, SGI400091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400092, SGI0, SGI400093, SGI0, SGI0, NULL};
const Segment * const SG100767[] = {SGI400094, SGI400095, SGI400096, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400097, SGI0, SGI400098, SGI0, SGI0, NULL};
const Segment * const SG100768[] = {SGI400099, SGI400100, SGI400101, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400102, SGI0, SGI400103, SGI0, SGI0, NULL};
const Segment * const SG100769[] = {SGI400104, SGI400105, SGI400106, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400107, SGI0, SGI400108, SGI0, SGI0, NULL};
const Segment * const SG100770[] = {SGI400109, SGI0, SGI400110, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400111, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100771[] = {SGI400112, SGI0, SGI400113, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400114, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100772[] = {SGI400115, SGI0, SGI400116, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400117, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100773[] = {SGI400118, SGI400119, NULL};
const Segment * const SG100774[] = {SGI400120, SGI400121, NULL};
const Segment * const SG100775[] = {SGI400122, SGI400123, NULL};
const Segment * const SG100776[] = {SGI400124, SGI400125, NULL};
const Segment * const SG100777[] = {SGI400126, SGI400127, NULL};
const Segment * const SG100778[] = {SGI400128, SGI400129, NULL};
const Segment * const SG100779[] = {SGI400130, SGI0, NULL};
const Segment * const SG100780[] = {SGI400131, SGI400132, NULL};
const Segment * const SG100781[] = {SGI400133, SGI400134, NULL};
const Segment * const SG100782[] = {SGI400135, SGI400136, NULL};
const Segment * const SG100783[] = {SGI400137, SGI400138, NULL};
const Segment * const SG100784[] = {SGI400139, SGI400140, NULL};
const Segment * const SG100785[] = {SGI400141, SGI400142, NULL};
const Segment * const SG100786[] = {SGI400143, SGI400144, NULL};
const Segment * const SG100787[] = {SGI400145, SGI400146, NULL};
const Segment * const SG100788[] = {SGI400147, SGI400148, NULL};
const Segment * const SG100789[] = {SGI400149, SGI400150, NULL};
const Segment * const SG100790[] = {SGI400151, SGI400152, NULL};
const Segment * const SG100791[] = {SGI400153, SGI400154, NULL};
const Segment * const SG100792[] = {SGI400155, SGI400156, NULL};
const Segment * const SG100793[] = {SGI400157, SGI400158, NULL};
const Segment * const SG100794[] = {SGI400159, SGI400160, NULL};
const Segment * const SG100795[] = {SGI400161, SGI400162, NULL};
const Segment * const SG100796[] = {SGI400163, SGI400164, NULL};
const Segment * const SG100797[] = {SGI400165, SGI400166, NULL};
const Segment * const SG100798[] = {SGI400167, SGI400168, NULL};
const Segment * const SG100799[] = {SGI400169, SGI400170, NULL};
const Segment * const SG100800[] = {SGI400171, SGI400172, NULL};
const Segment * const SG100801[] = {SGI400173, SGI400174, NULL};
const Segment * const SG100802[] = {SGI400175, SGI400176, NULL};
const Segment * const SG100803[] = {SGI400177, SGI400178, NULL};
const Segment * const SG100804[] = {SGI400179, SGI400180, NULL};
const Segment * const SG100805[] = {SGI400181, SGI400182, NULL};
const Segment * const SG100806[] = {SGI400183, SGI0, SGI400184, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400185, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100807[] = {SGI400186, SGI0, SGI400187, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400188, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100808[] = {SGI400189, SGI0, SGI400190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400191, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100809[] = {SGI400192, SGI400193, NULL};
const Segment * const SG100810[] = {SGI400194, SGI400195, NULL};
const Segment * const SG100811[] = {SGI400196, SGI400197, NULL};
const Segment * const SG100812[] = {SGI400198, SGI400199, NULL};
const Segment * const SG100813[] = {SGI400200, SGI400201, NULL};
const Segment * const SG100814[] = {SGI400202, SGI400203, NULL};
const Segment * const SG100815[] = {SGI400204, SGI0, NULL};
const Segment * const SG100816[] = {SGI400205, SGI400206, NULL};
const Segment * const SG100817[] = {SGI400207, SGI400208, NULL};
const Segment * const SG100818[] = {SGI400209, SGI400210, NULL};
const Segment * const SG100819[] = {SGI400211, SGI400212, NULL};
const Segment * const SG100820[] = {SGI400213, SGI400214, NULL};
const Segment * const SG100821[] = {SGI400215, SGI400216, SGI400217, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400218, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100822[] = {SGI400219, SGI400220, SGI400221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400222, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100823[] = {SGI400223, SGI400224, SGI400225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400226, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100824[] = {SGI400227, SGI400228, SGI400229, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400230, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100825[] = {SGI400231, SGI400232, SGI400233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400234, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100826[] = {SGI400235, SGI400236, SGI400237, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400238, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100827[] = {SGI400239, SGI0, SGI400240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100828[] = {SGI400241, SGI400242, SGI400243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400244, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100829[] = {SGI400245, SGI400246, NULL};
const Segment * const SG100830[] = {SGI400247, SGI400248, NULL};
const Segment * const SG100831[] = {SGI400249, SGI400250, NULL};
const Segment * const SG100832[] = {SGI400251, SGI400252, NULL};
const Segment * const SG100833[] = {SGI400253, SGI0, SGI400254, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400255, SGI0, SGI400256, SGI0, NULL};
const Segment * const SG100834[] = {SGI400257, SGI0, SGI400258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400259, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100835[] = {SGI400260, SGI0, SGI400261, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400262, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100836[] = {SGI400263, SGI400264, NULL};
const Segment * const SG100837[] = {SGI400265, SGI400266, NULL};
const Segment * const SG100838[] = {SGI400267, SGI400268, NULL};
const Segment * const SG100839[] = {SGI400269, SGI400270, NULL};
const Segment * const SG100840[] = {SGI400271, SGI400272, NULL};
const Segment * const SG100841[] = {SGI400273, SGI400274, NULL};
const Segment * const SG100842[] = {SGI400275, SGI400276, NULL};
const Segment * const SG100843[] = {SGI400277, SGI400278, NULL};
const Segment * const SG100844[] = {SGI400279, SGI0, NULL};
const Segment * const SG100845[] = {SGI400280, SGI400281, NULL};
const Segment * const SG100846[] = {SGI400282, SGI0, NULL};
const Segment * const SG100847[] = {SGI400283, SGI400284, SGI400285, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400286, SGI0, SGI0, SGI0, SGI400287, NULL};
const Segment * const SG100848[] = {SGI400288, SGI400289, SGI400290, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400291, SGI0, SGI0, SGI0, SGI400292, NULL};
const Segment * const SG100849[] = {SGI400293, SGI400294, SGI400295, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400296, SGI0, SGI0, SGI0, SGI400297, NULL};
const Segment * const SG100850[] = {SGI400298, SGI400299, SGI400300, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400301, SGI0, SGI0, SGI0, SGI400302, NULL};
const Segment * const SG100851[] = {SGI400303, SGI400304, SGI400305, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400306, SGI0, SGI0, SGI0, SGI400307, NULL};
const Segment * const SG100852[] = {SGI400308, SGI400309, SGI400310, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400311, SGI0, SGI0, SGI0, SGI400312, NULL};
const Segment * const SG100853[] = {SGI400313, SGI400314, SGI400315, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400316, SGI0, SGI0, SGI0, SGI400317, NULL};
const Segment * const SG100854[] = {SGI400318, SGI400319, SGI400320, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400321, SGI0, SGI0, SGI0, SGI400322, NULL};
const Segment * const SG100855[] = {SGI400323, SGI400324, SGI400325, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400326, SGI0, SGI0, SGI0, SGI400327, NULL};
const Segment * const SG100856[] = {SGI400328, SGI400329, SGI400330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400331, SGI0, SGI0, SGI0, SGI400332, NULL};
const Segment * const SG100857[] = {SGI400333, SGI400334, SGI400335, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400336, SGI0, SGI0, SGI0, SGI400337, NULL};
const Segment * const SG100858[] = {SGI400338, SGI400339, SGI400340, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400341, SGI0, SGI0, SGI0, SGI400342, NULL};
const Segment * const SG100859[] = {SGI400343, SGI400344, SGI400345, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400346, SGI0, SGI0, SGI0, SGI400347, NULL};
const Segment * const SG100860[] = {SGI400348, SGI400349, SGI400350, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400351, SGI0, SGI0, SGI0, SGI400352, NULL};
const Segment * const SG100861[] = {SGI400353, SGI400354, SGI400355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400356, SGI0, SGI0, SGI0, SGI400357, NULL};
const Segment * const SG100862[] = {SGI400358, SGI400359, SGI400360, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400361, SGI0, SGI0, SGI0, SGI400362, NULL};
const Segment * const SG100863[] = {SGI400363, SGI400364, SGI400365, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400366, SGI0, SGI0, SGI0, SGI400367, NULL};
const Segment * const SG100864[] = {SGI400368, SGI0, SGI400369, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400370, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100865[] = {SGI400371, SGI0, SGI400372, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400373, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100866[] = {SGI400374, SGI0, SGI400375, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400376, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100867[] = {SGI400377, SGI0, SGI400378, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400379, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100868[] = {SGI400380, SGI0, SGI400381, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400382, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100869[] = {SGI400383, SGI0, SGI400384, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400385, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100870[] = {SGI400386, SGI400387, SGI400388, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400389, SGI0, SGI0, SGI0, SGI400390, NULL};
const Segment * const SG100871[] = {SGI400391, SGI400392, SGI400393, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400394, SGI0, SGI0, SGI0, SGI400395, NULL};
const Segment * const SG100872[] = {SGI400396, SGI400397, SGI400398, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400399, SGI0, SGI0, SGI0, SGI400400, NULL};
const Segment * const SG100873[] = {SGI400401, SGI400402, SGI400403, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400404, SGI0, SGI0, SGI0, SGI400405, NULL};
const Segment * const SG100874[] = {SGI400406, SGI0, SGI400407, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400408, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100875[] = {SGI400409, SGI0, SGI400410, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400411, SGI0, SGI400412, SGI0, NULL};
const Segment * const SG100876[] = {SGI400413, SGI400414, SGI400415, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400416, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100877[] = {SGI400417, SGI400418, SGI400419, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400420, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100878[] = {SGI400421, SGI400422, SGI400423, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400424, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100879[] = {SGI400425, SGI400426, SGI400427, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400428, SGI0, SGI0, SGI0, SGI400429, NULL};
const Segment * const SG100880[] = {SGI400430, SGI400431, SGI400432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400433, SGI0, SGI0, SGI0, SGI400434, NULL};
const Segment * const SG100881[] = {SGI400435, SGI0, SGI400436, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400437, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100882[] = {SGI400438, SGI0, SGI400439, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400440, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100883[] = {SGI400441, SGI400442, SGI400443, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400444, SGI400445, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100884[] = {SGI400446, SGI400447, SGI400448, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400449, SGI400450, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100885[] = {SGI400451, SGI400452, SGI400453, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400454, SGI400455, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100886[] = {SGI400456, SGI400457, SGI400458, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400459, SGI400460, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100887[] = {SGI400461, SGI400462, SGI400463, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400464, SGI400465, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100888[] = {SGI400466, SGI400467, SGI400468, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400469, SGI400470, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100889[] = {SGI400471, SGI0, SGI400472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400473, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100890[] = {SGI400474, SGI0, SGI400475, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400476, SGI0, SGI400477, SGI0, NULL};
const Segment * const SG100891[] = {SGI400478, SGI0, SGI400479, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400480, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100892[] = {SGI400481, SGI0, SGI400482, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400483, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100893[] = {SGI400484, SGI0, SGI400485, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400486, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100894[] = {SGI400487, SGI400488, SGI400489, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400490, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100895[] = {SGI400491, SGI400492, SGI400493, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400494, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100896[] = {SGI400495, SGI400496, SGI400497, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400498, SGI0, SGI0, SGI0, SGI400499, NULL};
const Segment * const SG100897[] = {SGI400500, SGI400501, SGI400502, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400503, SGI0, SGI0, SGI0, SGI400504, NULL};
const Segment * const SG100898[] = {SGI400505, SGI400506, SGI400507, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400508, SGI0, SGI0, SGI0, SGI400509, NULL};
const Segment * const SG100899[] = {SGI400510, SGI400511, SGI400512, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400513, SGI0, SGI0, SGI0, SGI400514, NULL};
const Segment * const SG100900[] = {SGI400515, SGI400516, SGI400517, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400518, SGI0, SGI0, SGI0, SGI400519, NULL};
const Segment * const SG100901[] = {SGI400520, SGI400521, SGI400522, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400523, SGI0, SGI0, SGI0, SGI400524, NULL};
const Segment * const SG100902[] = {SGI400525, SGI0, SGI400526, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400527, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100903[] = {SGI400528, SGI0, SGI400529, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400530, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100904[] = {SGI400531, SGI0, SGI400532, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400533, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100905[] = {SGI400534, SGI0, SGI400535, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400536, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100906[] = {SGI400537, SGI0, SGI400538, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400539, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100907[] = {SGI400540, SGI0, SGI400541, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400542, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100908[] = {SGI400543, SGI0, SGI400544, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400545, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100909[] = {SGI400546, SGI0, SGI400547, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400548, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100910[] = {SGI400549, SGI0, SGI400550, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400551, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100911[] = {SGI400552, SGI400553, SGI400554, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400555, SGI0, SGI0, SGI0, SGI400556, NULL};
const Segment * const SG100912[] = {SGI400557, SGI400558, SGI400559, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400560, SGI0, SGI0, SGI0, SGI400561, NULL};
const Segment * const SG100913[] = {SGI400562, SGI400563, SGI400564, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400565, SGI0, SGI0, SGI0, SGI400566, NULL};
const Segment * const SG100914[] = {SGI400567, SGI400568, SGI400569, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400570, SGI0, SGI0, SGI0, SGI400571, NULL};
const Segment * const SG100915[] = {SGI400572, SGI400573, SGI400574, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400575, SGI0, SGI0, SGI0, SGI400576, NULL};
const Segment * const SG100916[] = {SGI400577, SGI400578, SGI400579, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400580, SGI0, SGI0, SGI0, SGI400581, NULL};
const Segment * const SG100917[] = {SGI400582, SGI0, SGI400583, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400584, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100918[] = {SGI400585, SGI400586, SGI400587, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400588, SGI0, SGI0, SGI0, SGI400589, NULL};
const Segment * const SG100919[] = {SGI400590, SGI400591, SGI400592, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400593, SGI0, SGI0, SGI0, SGI400594, NULL};
const Segment * const SG100920[] = {SGI400595, SGI400596, SGI400597, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400598, SGI0, SGI0, SGI0, SGI400599, NULL};
const Segment * const SG100921[] = {SGI400600, SGI400601, SGI400602, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400603, SGI0, SGI0, SGI0, SGI400604, NULL};
const Segment * const SG100922[] = {SGI400605, SGI400606, SGI400607, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400608, SGI0, SGI0, SGI0, SGI400609, NULL};
const Segment * const SG100923[] = {SGI400610, SGI400611, SGI400612, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400613, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100924[] = {SGI400614, SGI0, SGI400615, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100925[] = {SGI400616, SGI400617, SGI400618, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400619, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100926[] = {SGI400620, SGI400621, SGI400622, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400623, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100927[] = {SGI400624, SGI400625, SGI400626, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400627, SGI0, SGI0, SGI0, SGI400628, NULL};
const Segment * const SG100928[] = {SGI400629, SGI400630, SGI400631, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400632, SGI0, SGI0, SGI0, SGI400633, NULL};
const Segment * const SG100929[] = {SGI400634, SGI400635, SGI400636, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400637, SGI0, SGI0, SGI0, SGI400638, NULL};
const Segment * const SG100930[] = {SGI400639, SGI0, SGI400640, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400641, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100931[] = {SGI400642, SGI400643, SGI400644, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400645, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100932[] = {SGI400646, SGI400647, SGI400648, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400649, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100933[] = {SGI400650, SGI400651, SGI400652, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400653, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100934[] = {SGI400654, SGI400655, SGI400656, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400657, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100935[] = {SGI400658, SGI400659, SGI400660, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400661, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100936[] = {SGI400662, SGI0, SGI400663, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400664, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100937[] = {SGI400665, SGI400666, SGI400667, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100938[] = {SGI400668, SGI400669, SGI400670, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100939[] = {SGI400671, SGI0, SGI400672, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400673, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100940[] = {SGI400674, SGI400675, SGI400676, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400677, SGI0, SGI0, SGI0, SGI400678, NULL};
const Segment * const SG100941[] = {SGI400679, SGI400680, SGI400681, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400682, SGI0, SGI0, SGI0, SGI400683, NULL};
const Segment * const SG100942[] = {SGI400684, SGI400685, SGI400686, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400687, SGI0, SGI0, SGI0, SGI400688, NULL};
const Segment * const SG100943[] = {SGI400689, SGI0, SGI400690, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400691, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100944[] = {SGI400692, SGI0, SGI400693, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400694, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100945[] = {SGI400695, SGI400696, SGI400697, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400698, SGI0, SGI0, SGI0, SGI400699, NULL};
const Segment * const SG100946[] = {SGI400700, SGI400701, SGI400702, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400703, SGI0, SGI0, SGI0, SGI400704, NULL};
const Segment * const SG100947[] = {SGI400705, SGI400706, SGI400707, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400708, SGI0, SGI0, SGI0, SGI400709, NULL};
const Segment * const SG100948[] = {SGI400710, SGI0, SGI400711, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400712, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100949[] = {SGI400713, SGI400714, SGI400715, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400716, SGI0, SGI0, SGI0, SGI400717, NULL};
const Segment * const SG100950[] = {SGI400718, SGI400719, SGI400720, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400721, SGI0, SGI0, SGI0, SGI400722, NULL};
const Segment * const SG100951[] = {SGI400723, SGI400724, SGI400725, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400726, SGI0, SGI0, SGI0, SGI400727, NULL};
const Segment * const SG100952[] = {SGI400728, SGI400729, SGI400730, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400731, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100953[] = {SGI400732, SGI400733, SGI400734, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400735, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100954[] = {SGI400736, SGI400737, SGI400738, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100955[] = {SGI400739, SGI400740, SGI400741, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400742, SGI0, SGI0, SGI0, SGI400743, NULL};
const Segment * const SG100956[] = {SGI400744, SGI400745, SGI400746, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400747, SGI0, SGI0, SGI0, SGI400748, NULL};
const Segment * const SG100957[] = {SGI400749, SGI400750, SGI400751, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400752, SGI0, SGI0, SGI0, SGI400753, NULL};
const Segment * const SG100958[] = {SGI400754, SGI400755, SGI400756, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400757, SGI0, SGI0, SGI0, SGI400758, NULL};
const Segment * const SG100959[] = {SGI400759, SGI400760, SGI400761, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400762, SGI0, SGI0, SGI0, SGI400763, NULL};
const Segment * const SG100960[] = {SGI400764, SGI400765, SGI400766, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400767, SGI0, SGI0, SGI0, SGI400768, NULL};
const Segment * const SG100961[] = {SGI400769, SGI0, SGI400770, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400771, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100962[] = {SGI400772, SGI0, SGI400773, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400774, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100963[] = {SGI400775, SGI400776, SGI400777, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400778, SGI0, SGI0, SGI0, SGI400779, NULL};
const Segment * const SG100964[] = {SGI400780, SGI400781, SGI400782, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400783, SGI0, SGI0, SGI0, SGI400784, NULL};
const Segment * const SG100965[] = {SGI400785, SGI400786, SGI400787, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400788, SGI0, SGI0, SGI0, SGI400789, NULL};
const Segment * const SG100966[] = {SGI400790, SGI0, SGI400791, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400792, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100967[] = {SGI400793, SGI400794, SGI400795, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400796, SGI0, SGI0, SGI0, SGI400797, NULL};
const Segment * const SG100968[] = {SGI400798, SGI400799, SGI400800, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400801, SGI0, SGI0, SGI0, SGI400802, NULL};
const Segment * const SG100969[] = {SGI400803, SGI0, SGI400804, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400805, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100970[] = {SGI400806, SGI0, SGI400807, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400808, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100971[] = {SGI400809, SGI0, SGI400810, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400811, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100972[] = {SGI400812, SGI0, SGI400813, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400814, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100973[] = {SGI400815, SGI0, SGI400816, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400817, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100974[] = {SGI400818, SGI400819, SGI400820, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400821, SGI0, SGI0, SGI0, SGI400822, NULL};
const Segment * const SG100975[] = {SGI400823, SGI400824, SGI400825, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400826, SGI0, SGI0, SGI0, SGI400827, NULL};
const Segment * const SG100976[] = {SGI400828, SGI400829, SGI400830, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400831, SGI0, SGI0, SGI0, SGI400832, NULL};
const Segment * const SG100977[] = {SGI400833, SGI400834, SGI400835, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400836, SGI0, SGI0, SGI0, SGI400837, NULL};
const Segment * const SG100978[] = {SGI400838, SGI400839, SGI400840, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400841, SGI0, SGI0, SGI0, SGI400842, NULL};
const Segment * const SG100979[] = {SGI400843, SGI0, SGI400844, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400845, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100980[] = {SGI400846, SGI400847, SGI400848, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400849, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100981[] = {SGI400850, SGI400851, SGI400852, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400853, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100982[] = {SGI400854, SGI400855, SGI400856, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400857, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100983[] = {SGI400858, SGI0, SGI400859, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400860, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100984[] = {SGI400861, SGI0, SGI400862, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400863, SGI0, SGI400864, SGI0, NULL};
const Segment * const SG100985[] = {SGI400865, SGI400866, SGI400867, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400868, SGI0, SGI400869, SGI0, SGI0, NULL};
const Segment * const SG100986[] = {SGI400870, SGI400871, SGI400872, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400873, SGI0, SGI400874, SGI0, SGI0, NULL};
const Segment * const SG100987[] = {SGI400875, SGI400876, SGI400877, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400878, SGI0, SGI0, SGI0, SGI400879, NULL};
const Segment * const SG100988[] = {SGI400880, SGI0, SGI400881, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400882, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100989[] = {SGI400883, SGI400884, SGI400885, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400886, SGI0, SGI0, SGI0, SGI400887, NULL};
const Segment * const SG100990[] = {SGI400888, SGI400889, SGI400890, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400891, SGI0, SGI0, SGI0, SGI400892, NULL};
const Segment * const SG100991[] = {SGI400893, SGI400894, SGI400895, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400896, SGI0, SGI0, SGI0, SGI400897, NULL};
const Segment * const SG100992[] = {SGI400898, SGI0, SGI400899, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400900, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100993[] = {SGI400901, SGI0, SGI400902, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400903, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100994[] = {SGI400904, SGI400905, SGI400906, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400907, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100995[] = {SGI400908, SGI400909, SGI400910, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400911, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100996[] = {SGI400912, SGI0, SGI400913, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400914, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100997[] = {SGI400915, SGI0, SGI400916, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400917, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100998[] = {SGI400918, SGI400919, SGI400920, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400921, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG100999[] = {SGI400922, SGI0, SGI400923, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400924, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101000[] = {SGI400925, SGI400926, SGI400927, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400928, SGI0, SGI0, SGI0, SGI400929, NULL};
const Segment * const SG101001[] = {SGI400930, SGI400931, SGI400932, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400933, SGI0, SGI0, SGI0, SGI400934, NULL};
const Segment * const SG101002[] = {SGI400935, SGI0, SGI400936, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400937, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101003[] = {SGI400938, SGI400939, SGI400940, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400941, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101004[] = {SGI400942, SGI400943, SGI400944, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400945, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101005[] = {SGI400946, SGI0, SGI400947, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400948, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101006[] = {SGI400949, SGI0, SGI400950, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400951, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101007[] = {SGI400952, SGI400953, SGI400954, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400955, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101008[] = {SGI400956, SGI400957, SGI400958, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400959, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101009[] = {SGI400960, SGI400961, SGI400962, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400963, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101010[] = {SGI400964, SGI400965, SGI400966, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400967, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101011[] = {SGI400968, SGI400969, SGI400970, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400971, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101012[] = {SGI400972, SGI400973, SGI400974, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400975, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101013[] = {SGI400976, SGI400977, SGI400978, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400979, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101014[] = {SGI400980, SGI400981, SGI400982, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400983, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101015[] = {SGI400984, SGI0, SGI400985, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400986, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101016[] = {SGI400987, SGI0, SGI400988, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400989, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101017[] = {SGI400990, SGI0, SGI400991, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400992, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101018[] = {SGI400993, SGI0, SGI400994, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400995, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101019[] = {SGI400996, SGI0, SGI400997, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI400998, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101020[] = {SGI400999, SGI401000, SGI401001, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401002, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101021[] = {SGI401003, SGI401004, SGI401005, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401006, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101022[] = {SGI401007, SGI401008, SGI401009, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101023[] = {SGI401010, SGI401011, SGI401012, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101024[] = {SGI401013, SGI401014, SGI401015, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401016, SGI0, SGI0, SGI0, SGI401017, NULL};
const Segment * const SG101025[] = {SGI401018, SGI401019, SGI401020, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401021, SGI0, SGI0, SGI0, SGI401022, NULL};
const Segment * const SG101026[] = {SGI401023, SGI0, SGI401024, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401025, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101027[] = {SGI401026, SGI0, SGI401027, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401028, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101028[] = {SGI401029, SGI401030, SGI401031, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401032, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101029[] = {SGI401033, SGI401034, SGI401035, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401036, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101030[] = {SGI401037, SGI0, SGI401038, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401039, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101031[] = {SGI401040, SGI0, SGI401041, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401042, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101032[] = {SGI401043, SGI0, SGI401044, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401045, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101033[] = {SGI401046, SGI0, SGI401047, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401048, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101034[] = {SGI401049, SGI401050, SGI401051, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401052, SGI0, SGI0, SGI0, SGI401053, NULL};
const Segment * const SG101035[] = {SGI401054, SGI401055, SGI401056, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401057, SGI0, SGI0, SGI0, SGI401058, NULL};
const Segment * const SG101036[] = {SGI401059, SGI0, SGI401060, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401061, SGI0, SGI401062, SGI0, NULL};
const Segment * const SG101037[] = {SGI401063, SGI401064, SGI401065, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401066, SGI0, SGI0, SGI0, SGI401067, NULL};
const Segment * const SG101038[] = {SGI401068, SGI401069, SGI401070, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401071, SGI0, SGI0, SGI0, SGI401072, NULL};
const Segment * const SG101039[] = {SGI401073, SGI401074, SGI401075, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401076, SGI0, SGI0, SGI0, SGI401077, NULL};
const Segment * const SG101040[] = {SGI401078, SGI401079, SGI401080, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101041[] = {SGI401081, SGI401082, SGI401083, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401084, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101042[] = {SGI401085, SGI401086, SGI401087, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401088, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101043[] = {SGI401089, SGI401090, SGI401091, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401092, SGI0, SGI401093, SGI0, SGI0, NULL};
const Segment * const SG101044[] = {SGI401094, SGI0, SGI401095, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401096, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101045[] = {SGI401097, SGI0, SGI401098, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401099, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101046[] = {SGI401100, SGI401101, SGI401102, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401103, SGI0, SGI0, SGI0, SGI401104, NULL};
const Segment * const SG101047[] = {SGI401105, SGI401106, SGI401107, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401108, SGI0, SGI0, SGI0, SGI401109, NULL};
const Segment * const SG101048[] = {SGI401110, SGI0, SGI401111, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401112, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101049[] = {SGI401113, SGI0, SGI401114, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401115, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101050[] = {SGI401116, SGI401117, SGI401118, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401119, SGI0, SGI0, SGI0, SGI401120, NULL};
const Segment * const SG101051[] = {SGI401121, SGI401122, SGI401123, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401124, SGI0, SGI0, SGI0, SGI401125, NULL};
const Segment * const SG101052[] = {SGI401126, SGI0, SGI401127, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401128, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101053[] = {SGI401129, SGI0, SGI401130, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401131, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101054[] = {SGI401132, SGI0, SGI401133, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401134, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101055[] = {SGI401135, SGI401136, SGI401137, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401138, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101056[] = {SGI401139, SGI401140, SGI401141, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401142, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101057[] = {SGI401143, SGI0, SGI401144, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401145, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101058[] = {SGI401146, SGI0, SGI401147, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401148, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101059[] = {SGI401149, SGI0, SGI401150, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401151, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101060[] = {SGI401152, SGI401153, SGI401154, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401155, SGI0, SGI0, SGI0, SGI401156, NULL};
const Segment * const SG101061[] = {SGI401157, SGI401158, SGI401159, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401160, SGI0, SGI0, SGI0, SGI401161, NULL};
const Segment * const SG101062[] = {SGI401162, SGI0, SGI401163, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401164, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101063[] = {SGI401165, SGI401166, SGI401167, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401168, SGI0, SGI0, SGI0, SGI401169, NULL};
const Segment * const SG101064[] = {SGI401170, SGI0, SGI401171, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401172, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101065[] = {SGI401173, SGI0, SGI401174, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401175, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101066[] = {SGI401176, SGI0, SGI401177, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401178, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101067[] = {SGI401179, SGI401180, SGI401181, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401182, SGI0, SGI0, SGI0, SGI401183, NULL};
const Segment * const SG101068[] = {SGI401184, SGI401185, SGI401186, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401187, SGI401188, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101069[] = {SGI401189, SGI0, SGI401190, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401191, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101070[] = {SGI401192, SGI0, SGI401193, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401194, SGI0, SGI401195, SGI0, NULL};
const Segment * const SG101071[] = {SGI401196, SGI0, SGI401197, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401198, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101072[] = {SGI401199, SGI0, SGI401200, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401201, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101073[] = {SGI401202, SGI0, SGI401203, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401204, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101074[] = {SGI401205, SGI401206, SGI401207, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401208, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101075[] = {SGI401209, SGI401210, SGI401211, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401212, SGI0, SGI0, SGI0, SGI0, SGI401213, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101076[] = {SGI401214, SGI401215, SGI401216, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401217, SGI0, SGI0, SGI0, SGI0, SGI401218, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101077[] = {SGI401219, SGI401220, SGI401221, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401222, SGI0, SGI0, SGI0, SGI0, SGI401223, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101078[] = {SGI0, SGI401224, SGI401225, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101079[] = {SGI401226, SGI401227, SGI401228, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401229, SGI0, SGI0, SGI0, SGI0, SGI401230, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101080[] = {SGI401231, SGI401232, SGI401233, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401234, SGI0, SGI0, SGI0, SGI0, SGI401235, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101081[] = {SGI401236, SGI401237, SGI401238, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401239, SGI0, SGI0, SGI0, SGI0, SGI401240, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101082[] = {SGI401241, SGI401242, SGI401243, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401244, SGI0, SGI0, SGI0, SGI0, SGI401245, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101083[] = {SGI401246, SGI401247, SGI401248, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401249, SGI0, SGI0, SGI0, SGI0, SGI401250, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101084[] = {SGI401251, SGI401252, SGI401253, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401254, SGI0, SGI0, SGI0, SGI401255, NULL};
const Segment * const SG101085[] = {SGI401256, SGI401257, SGI401258, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401259, SGI0, SGI0, SGI0, SGI401260, NULL};
const Segment * const SG101086[] = {SGI401261, SGI0, SGI401262, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401263, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101087[] = {SGI401264, SGI0, SGI401265, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401266, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101088[] = {SGI401267, SGI401268, SGI401269, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401270, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101089[] = {SGI401271, SGI0, SGI401272, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401273, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101090[] = {SGI401274, SGI0, SGI401275, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401276, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101091[] = {SGI401277, SGI401278, SGI401279, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401280, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101092[] = {SGI401281, SGI401282, SGI401283, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401284, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101093[] = {SGI401285, SGI401286, SGI401287, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401288, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101094[] = {SGI401289, SGI401290, SGI401291, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401292, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101095[] = {SGI401293, SGI0, SGI401294, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401295, SGI0, SGI401296, SGI0, NULL};
const Segment * const SG101096[] = {SGI401297, SGI401298, SGI401299, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401300, SGI0, SGI0, SGI0, SGI401301, NULL};
const Segment * const SG101097[] = {SGI401302, SGI401303, SGI401304, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401305, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101098[] = {SGI401306, SGI401307, SGI401308, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401309, SGI0, SGI0, SGI0, SGI401310, NULL};
const Segment * const SG101099[] = {SGI401311, SGI401312, SGI401313, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401314, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101100[] = {SGI401315, SGI401316, SGI401317, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401318, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101101[] = {SGI401319, SGI401320, SGI401321, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101102[] = {SGI401322, SGI0, SGI401323, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401324, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101103[] = {SGI401325, SGI0, SGI401326, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401327, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101104[] = {SGI401328, SGI401329, SGI401330, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401331, SGI0, SGI0, SGI0, SGI401332, NULL};
const Segment * const SG101105[] = {SGI401333, SGI0, SGI401334, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401335, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101106[] = {SGI401336, SGI401337, SGI401338, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401339, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101107[] = {SGI401340, SGI0, SGI401341, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401342, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101108[] = {SGI401343, SGI0, SGI401344, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401345, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101109[] = {SGI401346, SGI401347, SGI401348, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401349, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101110[] = {SGI401350, SGI0, SGI401351, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401352, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101111[] = {SGI401353, SGI401354, SGI401355, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401356, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101112[] = {SGI401357, SGI0, SGI401358, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401359, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101113[] = {SGI401360, SGI401361, NULL};
const Segment * const SG101114[] = {SGI401362, SGI401363, NULL};
const Segment * const SG101115[] = {SGI401364, SGI401365, NULL};
const Segment * const SG101116[] = {SGI401366, SGI401367, NULL};
const Segment * const SG101117[] = {SGI401368, SGI401369, NULL};
const Segment * const SG101118[] = {SGI401370, SGI401371, NULL};
const Segment * const SG101119[] = {SGI401372, SGI401373, NULL};
const Segment * const SG101120[] = {SGI401374, SGI401375, NULL};
const Segment * const SG101121[] = {SGI401376, SGI401377, NULL};
const Segment * const SG101122[] = {SGI401378, SGI401379, NULL};
const Segment * const SG101123[] = {SGI401380, SGI401381, NULL};
const Segment * const SG101124[] = {SGI401382, SGI401383, NULL};
const Segment * const SG101125[] = {SGI401384, SGI401385, NULL};
const Segment * const SG101126[] = {SGI401386, SGI401387, NULL};
const Segment * const SG101127[] = {SGI401388, SGI0, SGI401389, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401390, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101128[] = {SGI401391, SGI0, SGI401392, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401393, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101129[] = {SGI401394, SGI0, SGI401395, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401396, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101130[] = {SGI401397, SGI401398, NULL};
const Segment * const SG101131[] = {SGI401399, SGI401400, NULL};
const Segment * const SG101132[] = {SGI401401, SGI401402, NULL};
const Segment * const SG101133[] = {SGI401403, SGI401404, NULL};
const Segment * const SG101134[] = {SGI401405, SGI401406, NULL};
const Segment * const SG101135[] = {SGI401407, SGI401408, NULL};
const Segment * const SG101136[] = {SGI401409, SGI401410, NULL};
const Segment * const SG101137[] = {SGI401411, SGI401412, NULL};
const Segment * const SG101138[] = {SGI401413, SGI401414, NULL};
const Segment * const SG101139[] = {SGI401415, SGI401416, NULL};
const Segment * const SG101140[] = {SGI401417, SGI401418, NULL};
const Segment * const SG101141[] = {SGI401419, SGI401420, NULL};
const Segment * const SG101142[] = {SGI401421, SGI401422, NULL};
const Segment * const SG101143[] = {SGI401423, SGI401424, NULL};
const Segment * const SG101144[] = {SGI401425, SGI401426, NULL};
const Segment * const SG101145[] = {SGI401427, SGI401428, NULL};
const Segment * const SG101146[] = {SGI401429, SGI401430, NULL};
const Segment * const SG101147[] = {SGI401431, SGI0, SGI401432, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401433, SGI0, SGI0, SGI0, SGI401434, SGI401434, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401435, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101148[] = {SGI401436, SGI401437, NULL};
const Segment * const SG101149[] = {SGI401438, SGI401439, NULL};
const Segment * const SG101150[] = {SGI401440, SGI401441, NULL};
const Segment * const SG101151[] = {SGI401442, SGI401443, NULL};
const Segment * const SG101152[] = {SGI401444, SGI401445, NULL};
const Segment * const SG101153[] = {SGI401446, SGI401447, NULL};
const Segment * const SG101154[] = {SGI401448, SGI401449, NULL};
const Segment * const SG101155[] = {SGI401450, SGI401451, NULL};
const Segment * const SG101156[] = {SGI401452, SGI401453, NULL};
const Segment * const SG101157[] = {SGI401454, SGI401455, NULL};
const Segment * const SG101158[] = {SGI401456, SGI401457, NULL};
const Segment * const SG101159[] = {SGI401458, SGI0, SGI401459, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101160[] = {SGI401460, SGI401461, SGI401462, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401463, SGI0, SGI0, SGI0, SGI401464, SGI401464, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101161[] = {SGI401465, SGI401466, SGI401467, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101162[] = {SGI401468, SGI401469, SGI401470, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI401471, SGI0, SGI0, SGI0, SGI401472, SGI401472, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, SGI0, NULL};
const Segment * const SG101163[] = {SGI401473, SGI401474, NULL};
const Segment * const SG101164[] = {SGI401475, SGI401476, NULL};
const Segment * const SG101165[] = {SGI401477, SGI401478, NULL};
const Segment * const SG101166[] = {SGI401479, SGI401480, NULL};
const Segment * const SG101167[] = {SGI401481, SGI401482, NULL};
const Segment * const SG101168[] = {SGI401483, SGI401484, NULL};
const Segment * const SG101169[] = {SGI401485, SGI401486, NULL};
const Segment * const SG101170[] = {SGI401487, SGI401488, NULL};
const Segment * const SG101171[] = {SGI401489, SGI401490, NULL};
const Segment * const SG101172[] = {SGI401491, SGI401492, NULL};
const Segment * const SG101173[] = {SGI401493, SGI401494, NULL};
const Segment * const SG101174[] = {SGI401495, SGI401496, NULL};
const Segment * const SG101175[] = {SGI401497, SGI401498, NULL};
const Segment * const SG101176[] = {SGI401499, SGI401500, NULL};
const Segment * const SG101177[] = {SGI401501, SGI401502, NULL};
const Segment * const SG101178[] = {SGI401503, SGI401504, NULL};
const Segment * const SG101179[] = {SGI401505, SGI401506, NULL};
const Segment * const SG101180[] = {SGI401507, SGI401508, NULL};
const Segment * const SG101181[] = {SGI401509, SGI401510, NULL};
const Segment * const SG101182[] = {SGI401511, SGI401512, NULL};
const Segment * const SG101183[] = {SGI401513, SGI401514, NULL};
const Segment * const SG101184[] = {SGI401515, SGI401516, NULL};
const Segment * const SG101185[] = {SGI401517, SGI401518, NULL};
const Segment * const SG101186[] = {SGI401519, SGI401520, NULL};


const Segmentdata DAlcatel[] = {
	{SG1802, &UA2404102},
	{SG1803, &UA2768842},
	{SG2292, &UA20577},
	{SG2293, &UA20590},
	{SG2294, &UA52441},
	{SG2295, &UA62847},
	{SG2296, &UA64337},
	{SG2588, &UA2348219},
	{SG2638, &UA24661},
	{SG2639, &UA30897},
	{SG2640, &UA31249},
	{SG2641, &UA46659},
	{SG2642, &UA64082},
	{SG2661, &UA44118},
	{SG2662, &UA55188},
	{SG2663, &UA63160},
	{SG2717, &UA20574},
	{SG2718, &UA24521},
	{SG2719, &UA41390},
	{SG2720, &UA54157},
	{SG2721, &UA63177},
	{SG2723, &UA2362410},
	{SG2773, &UA2997101},
	{SG2774, &UA3009905},
	{SG2815, &UA8588},
	{SG2816, &UA26942},
	{SG2817, &UA31371},
	{SG2818, &UA64413},
	{SG2942, &UA54212},
	{SG2943, &UA57949},
	{SG2944, &UA59112},
	{SG2945, &UA63194},
	{SG2946, &UA63506},
	{SG2992, &UA16191},
	{SG2993, &UA24090},
	{SG2994, &UA53725},
	{SG2995, &UA62809},
	{SG3082, &UA82310},
	{SG3088, &UA56366},
	{SG3089, &UA57047},
	{SG3090, &UA62512},
	{SG3091, &UA63705},
	{SG3097, &UA63741},
	{SG3098, &UA2370893},
	{SG3110, &UA2580248},
	{SG3158, &UA21666},
	{SG3159, &UA51702},
	{SG3160, &UA55262},
	{SG3177, &UA21998},
	{SG3178, &UA47688},
	{SG3179, &UA60565},
	{SG3180, &UA60566},
	{SG3181, &UA61132},
	{SG3217, &UA9137},
	{SG3218, &UA31498},
	{SG3219, &UA64438},
	{SG3223, &UA2386227},
	{SG3224, &UA2420567},
	{SG3225, &UA2578447},
	{SG3249, &UA2660191},
	{SG3270, &UA24515},
	{SG3271, &UA30164},
	{SG3272, &UA58661},
	{SG3273, &UA63221},
	{SG3338, &UA3018271},
	{SG3339, &UA3021548},
	{SG3380, &UA59494},
	{SG3404, &UA3023572},
	{SG3405, &UA3055661},
	{SG3443, &UA3060253},
	{SG3470, &UA2502839},
	{SG3471, &UA2595332},
	{SG3530, &UA20578},
	{SG3544, &UA28415},
	{SG3546, &UA89164},
	{SG3568, &UA51677},
	{SG3607, &UA45816},
	{SG3608, &UA59619},
	{SG3609, &UA61804},
	{SG3617, &UA2956282},
	{SG3618, &UA3002719},
	{SG3622, &UA3125965},
	{SG3633, &UA3061518},
	{SG3638, &UA3058421},
	{SG3639, &UA3126060},
	{SG3663, &UA2963585},
	{SG3673, &UA53943},
	{SG3681, &UA21376},
	{SG3684, &UA2878993},
	{SG3686, &UA2994161},
	{SG3687, &UA3121205},
	{SG3703, &UA33383},
	{SG3704, &UA61345},
	{SG3719, &UA2793141},
	{SG3729, &UA3014412},
	{SG3733, &UA20444},
	{SG3734, &UA88649},
	{SG3735, &UA88833},
	{SG3736, &UA88862},
	{SG3737, &UA89119},
	{SG3738, &UA90320},
	{SG3767, &UA11600},
	{SG3768, &UA20158},
	{SG3769, &UA26758},
	{SG3770, &UA55610},
	{SG3784, &UA30193},
	{SG3785, &UA46649},
	{SG3866, &UA20403},
	{SG3867, &UA49096},
	{SG3868, &UA2593533},
	{SG3874, &UA2854761},
	{SG3901, &UA3126131},
	{SG3918, &UA3107715},
	{SG3928, &UA20398},
	{SG3929, &UA24712},
	{SG3933, &UA2990385},
	{SG3948, &UA64600},
	{SG3950, &UA2720981},
	{SG3964, &UA22626},
	{SG3965, &UA60567},
	{SG3966, &UA90190},
	{SG3990, &UA21369},
	{SG3993, &UA22401},
	{SG4007, &UA22217},
	{SG4008, &UA49397},
	{SG4009, &UA88740},
	{SG4010, &UA64631},
	{SG4011, &UA88424},
	{SG4022, &UA33159},
	{SG4023, &UA49715},
	{SG4024, &UA54133},
	{SG4025, &UA54449},
	{SG4029, &UA20354},
	{SG4030, &UA61057},
	{SG4031, &UA63277},
	{SG4044, &UA21739},
	{SG4050, &UA82259},
	{SG4057, &UA21659},
	{SG4058, &UA22376},
	{SG4059, &UA33000},
	{SG4062, &UA49196},
	{SG4063, &UA49324},
	{SG4064, &UA61109},
	{SG4065, &UA82270},
	{SG4075, &UA22861},
	{SG4076, &UA90174},
	{SG4077, &UA24086},
	{SG4078, &UA49801},
	{SG4079, &UA60631},
	{SG4080, &UA60731},
	{SG4087, &UA23191},
	{SG4088, &UA23901},
	{SG4091, &UA49179},
	{SG4092, &UA49412},
	{SG4094, &UA60880},
	{NULL}};
const Device *DAmoi[] = {
	&UA54503,
	&UA1599,
	&UA17201,
	&UA24235,
	&UA24236,
	&UA39021,
	&UA63693,
	&UA63694,
	NULL};
const Segmentdata DAndroidVersion2[] = {
	{SG1284, &UA69022},
	{SG1290, &UA2018508},
	{SG1291, &UA2018841},
	{SG2076, &UA50081},
	{SG2077, &UA62232},
	{SG2078, &UA62929},
	{SG2079, &UA68183},
	{SG2080, &UA540100},
	{SG2290, &UA163806},
	{SG2291, &UA219621},
	{SG2449, &UA53579},
	{SG2450, &UA69070},
	{SG2521, &UA342036},
	{SG2796, &UA761430},
	{SG2910, &UA50433},
	{SG2911, &UA89771},
	{SG2912, &UA797088},
	{SG2913, &UA3069052},
	{SG3059, &UA67656},
	{SG3449, &UA67419},
	{SG3450, &UA67830},
	{SG3451, &UA646485},
	{SG3452, &UA1851549},
	{NULL}};
const Segmentdata DAndroidVersion1[] = {
	{SG1140, &UA5921},
	{SG1141, &UA30535},
	{SG1142, &UA30553},
	{SG1143, &UA30570},
	{SG1144, &UA30603},
	{SG1145, &UA36526},
	{SG1146, &UA70985},
	{SG1175, &UA109436},
	{SG1176, &UA250816},
	{SG1177, &UA782529},
	{SG1188, &UA11052},
	{SG1189, &UA15075},
	{SG1190, &UA15106},
	{SG1191, &UA49084},
	{SG1192, &UA72198},
	{SG1240, &UA31667},
	{SG1241, &UA31670},
	{SG1243, &UA31683},
	{SG1244, &UA37509},
	{SG1346, &UA21314},
	{SG1347, &UA21320},
	{SG1348, &UA21336},
	{SG1349, &UA21337},
	{SG1350, &UA21339},
	{SG1351, &UA44397},
	{SG1352, &UA54792},
	{SG1353, &UA89740},
	{SG1423, &UA7035},
	{SG1424, &UA7036},
	{SG1425, &UA7095},
	{SG1426, &UA7102},
	{SG1427, &UA7108},
	{SG1428, &UA27622},
	{SG1429, &UA54945},
	{SG1430, &UA88941},
	{SG1431, &UA2018673},
	{SG1441, &UA7244},
	{SG1443, &UA7252},
	{SG1445, &UA7254},
	{SG1447, &UA25476},
	{SG1449, &UA25480},
	{SG1451, &UA25483},
	{SG1453, &UA25484},
	{SG1454, &UA25486},
	{SG1455, &UA63002},
	{SG1477, &UA6403},
	{SG1478, &UA9238},
	{SG1479, &UA10086},
	{SG1480, &UA32160},
	{SG1481, &UA43619},
	{SG1482, &UA50935},
	{SG1483, &UA54085},
	{SG1484, &UA64262},
	{SG1485, &UA64533},
	{SG1486, &UA64723},
	{SG1598, &UA50178},
	{SG1599, &UA50371},
	{SG1601, &UA50438},
	{SG1602, &UA52793},
	{SG1604, &UA52806},
	{SG1606, &UA53566},
	{SG1608, &UA61781},
	{SG1610, &UA62294},
	{SG1612, &UA67146},
	{SG1614, &UA67613},
	{SG1615, &UA68375},
	{SG1616, &UA2018123},
	{SG1629, &UA25927},
	{SG1630, &UA53743},
	{SG1631, &UA2775358},
	{SG1847, &UA50072},
	{SG1848, &UA50179},
	{SG1849, &UA50233},
	{SG1850, &UA50241},
	{SG1851, &UA50249},
	{SG1852, &UA56081},
	{SG1853, &UA443337},
	{SG1949, &UA50214},
	{SG1950, &UA67683},
	{SG2033, &UA12307},
	{SG2034, &UA23554},
	{SG2035, &UA23557},
	{SG2036, &UA23807},
	{SG2037, &UA24638},
	{SG2038, &UA28973},
	{SG2039, &UA50068},
	{SG2040, &UA66418},
	{SG2042, &UA66422},
	{SG2044, &UA88465},
	{SG2045, &UA524316},
	{SG2046, &UA735573},
	{SG2047, &UA791949},
	{SG2048, &UA2018885},
	{SG2049, &UA2028590},
	{SG2050, &UA3117781},
	{SG2051, &UA22983},
	{SG2052, &UA50206},
	{SG2053, &UA52790},
	{SG2054, &UA56029},
	{SG2055, &UA2018332},
	{SG2056, &UA2019441},
	{SG2057, &UA3117693},
	{SG2069, &UA6073},
	{SG2070, &UA31031},
	{SG2071, &UA446859},
	{SG2072, &UA498790},
	{SG2073, &UA2018297},
	{SG2074, &UA2018687},
	{SG2075, &UA2932918},
	{SG2092, &UA1182},
	{SG2093, &UA17077},
	{SG2094, &UA17086},
	{SG2095, &UA17092},
	{SG2096, &UA17104},
	{SG2097, &UA17109},
	{SG2098, &UA35618},
	{SG2099, &UA35619},
	{SG2100, &UA62547},
	{SG2101, &UA72597},
	{SG2102, &UA281398},
	{SG2103, &UA3098421},
	{SG2231, &UA50343},
	{SG2232, &UA67185},
	{SG2247, &UA67702},
	{SG2248, &UA67813},
	{SG2249, &UA363864},
	{SG2250, &UA2020809},
	{SG2251, &UA2021358},
	{SG2252, &UA2022623},
	{SG2297, &UA50203},
	{SG2298, &UA56143},
	{SG2299, &UA67438},
	{SG2300, &UA140308},
	{SG2305, &UA2018106},
	{SG2308, &UA62426},
	{SG2309, &UA67725},
	{SG2314, &UA50458},
	{SG2332, &UA28454},
	{SG2333, &UA28460},
	{SG2335, &UA28474},
	{SG2337, &UA28478},
	{SG2338, &UA45943},
	{SG2339, &UA45946},
	{SG2347, &UA50201},
	{SG2348, &UA88910},
	{SG2349, &UA2021588},
	{SG2350, &UA2031153},
	{SG2360, &UA67451},
	{SG2361, &UA68569},
	{SG2362, &UA68668},
	{SG2363, &UA389059},
	{SG2364, &UA508827},
	{SG2365, &UA655581},
	{SG2366, &UA695004},
	{SG2367, &UA715465},
	{SG2368, &UA765930},
	{SG2369, &UA1941695},
	{SG2370, &UA2023129},
	{SG2371, &UA2026679},
	{SG2376, &UA30370},
	{SG2377, &UA67672},
	{SG2378, &UA89017},
	{SG2398, &UA5471},
	{SG2399, &UA5481},
	{SG2400, &UA7180},
	{SG2401, &UA29538},
	{SG2402, &UA64693},
	{SG2428, &UA50392},
	{SG2441, &UA3570},
	{SG2442, &UA3573},
	{SG2443, &UA3581},
	{SG2444, &UA3587},
	{SG2445, &UA37970},
	{SG2446, &UA37973},
	{SG2447, &UA69531},
	{SG2448, &UA69545},
	{SG2476, &UA88996},
	{SG2477, &UA460274},
	{SG2478, &UA477974},
	{SG2489, &UA50353},
	{SG2541, &UA68114},
	{SG2542, &UA241676},
	{SG2543, &UA1876932},
	{SG2646, &UA12695},
	{SG2647, &UA29101},
	{SG2664, &UA67765},
	{SG2665, &UA68416},
	{SG2666, &UA687577},
	{SG2667, &UA2021825},
	{SG2694, &UA158246},
	{SG2697, &UA18466},
	{SG2698, &UA56008},
	{SG2699, &UA67348},
	{SG2700, &UA67552},
	{SG2701, &UA88850},
	{SG2702, &UA445242},
	{SG2703, &UA512892},
	{SG2704, &UA561730},
	{SG2705, &UA660674},
	{SG2706, &UA729754},
	{SG2707, &UA1868494},
	{SG2708, &UA1898418},
	{SG2709, &UA2018585},
	{SG2710, &UA2018833},
	{SG2711, &UA2807380},
	{SG2712, &UA3042182},
	{SG2713, &UA3042284},
	{SG2722, &UA50352},
	{SG2725, &UA89708},
	{SG2726, &UA94074},
	{SG2727, &UA7605},
	{SG2728, &UA8932},
	{SG2814, &UA870786},
	{SG2822, &UA50106},
	{SG2823, &UA68058},
	{SG2824, &UA68174},
	{SG2873, &UA50452},
	{SG2874, &UA89509},
	{SG2889, &UA93968},
	{SG2897, &UA50069},
	{SG2898, &UA67698},
	{SG2899, &UA68197},
	{SG2900, &UA88558},
	{SG2901, &UA1884548},
	{SG2902, &UA3118595},
	{SG2908, &UA68372},
	{SG2909, &UA191575},
	{SG2967, &UA6250},
	{SG2968, &UA25467},
	{SG2969, &UA25468},
	{SG2970, &UA31297},
	{SG2971, &UA50082},
	{SG2972, &UA50296},
	{SG2973, &UA2520518},
	{SG2987, &UA5475},
	{SG2988, &UA29535},
	{SG2989, &UA46194},
	{SG3006, &UA50407},
	{SG3007, &UA2018601},
	{SG3014, &UA50367},
	{SG3022, &UA56347},
	{SG3023, &UA64279},
	{SG3024, &UA88822},
	{SG3025, &UA3117828},
	{SG3027, &UA67678},
	{SG3033, &UA66955},
	{SG3034, &UA67362},
	{SG3035, &UA122766},
	{SG3036, &UA485022},
	{SG3037, &UA501442},
	{SG3038, &UA2018083},
	{SG3078, &UA54639},
	{SG3083, &UA50288},
	{SG3084, &UA89792},
	{SG3099, &UA89889},
	{SG3100, &UA90338},
	{SG3101, &UA484122},
	{SG3102, &UA50341},
	{SG3103, &UA88667},
	{SG3104, &UA2018113},
	{SG3105, &UA50149},
	{SG3106, &UA61989},
	{SG3107, &UA2721882},
	{SG3108, &UA2801418},
	{SG3125, &UA14832},
	{SG3145, &UA788668},
	{SG3147, &UA67826},
	{SG3148, &UA88609},
	{SG3149, &UA88620},
	{SG3150, &UA479415},
	{SG3154, &UA50229},
	{SG3193, &UA17084},
	{SG3214, &UA10103},
	{SG3215, &UA10105},
	{SG3222, &UA66983},
	{SG3231, &UA67028},
	{SG3232, &UA67248},
	{SG3233, &UA67369},
	{SG3234, &UA67806},
	{SG3235, &UA68201},
	{SG3236, &UA90745},
	{SG3237, &UA421737},
	{SG3239, &UA2018341},
	{SG3242, &UA2018871},
	{SG3251, &UA32183},
	{SG3252, &UA32717},
	{SG3256, &UA44221},
	{SG3280, &UA67688},
	{SG3284, &UA2237960},
	{SG3293, &UA32088},
	{SG3298, &UA9427},
	{SG3302, &UA53937},
	{SG3340, &UA50326},
	{SG3341, &UA52801},
	{SG3342, &UA67080},
	{SG3401, &UA4791},
	{SG3402, &UA27766},
	{SG3462, &UA54103},
	{SG3478, &UA56049},
	{SG3481, &UA50310},
	{SG3482, &UA673384},
	{SG3503, &UA656149},
	{SG3504, &UA893816},
	{SG3505, &UA3116749},
	{SG3515, &UA50204},
	{SG3516, &UA54731},
	{SG3517, &UA118957},
	{SG3540, &UA780052},
	{SG3596, &UA5474},
	{SG3610, &UA2028365},
	{SG3611, &UA2395942},
	{SG3667, &UA67344},
	{SG3694, &UA67865},
	{SG3695, &UA90744},
	{SG3696, &UA443833},
	{SG3697, &UA1936933},
	{SG3709, &UA40779},
	{SG3713, &UA2019231},
	{SG3716, &UA66925},
	{SG3791, &UA2019329},
	{SG3808, &UA42859},
	{SG3809, &UA50107},
	{SG3827, &UA1850505},
	{SG3864, &UA67313},
	{SG3869, &UA56791},
	{SG3875, &UA2019394},
	{SG3885, &UA67592},
	{SG3886, &UA2020489},
	{SG3904, &UA2492774},
	{SG3920, &UA68055},
	{SG4012, &UA500092},
	{SG4013, &UA491718},
	{SG4014, &UA508195},
	{SG4015, &UA2018365},
	{SG4042, &UA503997},
	{SG4061, &UA2509},
	{SG4095, &UA3117533},
	{SG4103, &UA1872185},
	{SG4107, &UA2020266},
	{SG4123, &UA35835},
	{SG4124, &UA2034205},
	{SG4125, &UA414028},
	{SG4143, &UA50242},
	{SG4144, &UA50271},
	{SG4146, &UA2018187},
	{SG4147, &UA2024502},
	{SG4149, &UA50074},
	{SG4162, &UA2019183},
	{SG4181, &UA2018196},
	{SG4186, &UA30809},
	{SG4190, &UA31166},
	{SG4193, &UA50330},
	{SG4199, &UA67799},
	{SG4202, &UA2020375},
	{SG4204, &UA14282},
	{SG4205, &UA17075},
	{SG4206, &UA89693},
	{SG4212, &UA51700},
	{SG4220, &UA27764},
	{SG4223, &UA50064},
	{SG4243, &UA69603},
	{SG4249, &UA33111},
	{SG4253, &UA88911},
	{SG4259, &UA2022980},
	{SG4278, &UA19951},
	{SG4316, &UA347104},
	{SG4320, &UA2018632},
	{SG4352, &UA49803},
	{SG4360, &UA73218},
	{SG4364, &UA88787},
	{SG4367, &UA88943},
	{SG4368, &UA89696},
	{SG4369, &UA89752},
	{SG4380, &UA2019501},
	{SG4405, &UA2616407},
	{NULL}};
const Segmentdata DAndroidVersion2_2_1[] = {
	{SG11, &UA11207},
	{SG12, &UA14656},
	{SG19, &UA32132},
	{SG23, &UA61816},
	{SG27, &UA140727},
	{SG29, &UA154469},
	{SG31, &UA155768},
	{SG33, &UA160440},
	{SG38, &UA754088},
	{SG40, &UA2018253},
	{SG91, &UA7461},
	{SG92, &UA7494},
	{SG93, &UA7501},
	{SG94, &UA51763},
	{SG95, &UA64246},
	{SG96, &UA159572},
	{SG97, &UA169714},
	{SG98, &UA644719},
	{SG99, &UA2033690},
	{SG100, &UA8528},
	{SG101, &UA14293},
	{SG102, &UA18073},
	{SG103, &UA392080},
	{SG104, &UA1926324},
	{SG105, &UA2022332},
	{SG106, &UA2028230},
	{SG107, &UA2607631},
	{SG108, &UA2610230},
	{SG109, &UA2611263},
	{SG110, &UA2614172},
	{SG111, &UA2615731},
	{SG112, &UA2617644},
	{SG113, &UA2618624},
	{SG114, &UA2618822},
	{SG115, &UA2620059},
	{SG116, &UA2622196},
	{SG117, &UA2624698},
	{SG118, &UA2626216},
	{SG119, &UA2635809},
	{SG237, &UA11403},
	{SG238, &UA11443},
	{SG239, &UA11471},
	{SG240, &UA16334},
	{SG241, &UA16338},
	{SG242, &UA16340},
	{SG243, &UA26596},
	{SG244, &UA40253},
	{SG245, &UA41041},
	{SG246, &UA62858},
	{SG247, &UA90484},
	{SG248, &UA173147},
	{SG249, &UA541439},
	{SG250, &UA708648},
	{SG267, &UA24001},
	{SG268, &UA42455},
	{SG269, &UA42456},
	{SG270, &UA42471},
	{SG271, &UA58338},
	{SG272, &UA60926},
	{SG273, &UA64730},
	{SG274, &UA181432},
	{SG275, &UA397328},
	{SG276, &UA450355},
	{SG277, &UA1894041},
	{SG278, &UA2019401},
	{SG279, &UA2019673},
	{SG280, &UA2027607},
	{SG282, &UA7907},
	{SG283, &UA7968},
	{SG284, &UA7969},
	{SG285, &UA7976},
	{SG286, &UA8009},
	{SG287, &UA9509},
	{SG288, &UA9533},
	{SG289, &UA10480},
	{SG290, &UA10486},
	{SG291, &UA10490},
	{SG292, &UA10506},
	{SG293, &UA10510},
	{SG294, &UA10557},
	{SG295, &UA10580},
	{SG296, &UA10575},
	{SG297, &UA11638},
	{SG298, &UA39297},
	{SG299, &UA52101},
	{SG300, &UA52119},
	{SG301, &UA52135},
	{SG302, &UA52136},
	{SG303, &UA52633},
	{SG304, &UA52669},
	{SG305, &UA52717},
	{SG306, &UA376468},
	{SG307, &UA695461},
	{SG308, &UA1936487},
	{SG309, &UA2021668},
	{SG310, &UA2022170},
	{SG311, &UA2027104},
	{SG312, &UA2314532},
	{SG313, &UA19561},
	{SG314, &UA22996},
	{SG315, &UA24095},
	{SG316, &UA24289},
	{SG317, &UA61149},
	{SG318, &UA90375},
	{SG319, &UA189800},
	{SG320, &UA450857},
	{SG321, &UA681566},
	{SG322, &UA736063},
	{SG323, &UA1969291},
	{SG324, &UA2019625},
	{SG325, &UA2019701},
	{SG333, &UA57937},
	{SG334, &UA59867},
	{SG335, &UA161384},
	{SG336, &UA167113},
	{SG337, &UA168366},
	{SG338, &UA299166},
	{SG339, &UA301176},
	{SG340, &UA362192},
	{SG341, &UA393458},
	{SG342, &UA420311},
	{SG343, &UA435044},
	{SG344, &UA450258},
	{SG345, &UA458649},
	{SG346, &UA458740},
	{SG347, &UA491459},
	{SG348, &UA505668},
	{SG349, &UA509344},
	{SG350, &UA523354},
	{SG351, &UA525469},
	{SG352, &UA536304},
	{SG353, &UA542117},
	{SG354, &UA547860},
	{SG355, &UA552759},
	{SG356, &UA555553},
	{SG357, &UA557790},
	{SG358, &UA558093},
	{SG359, &UA561533},
	{SG360, &UA562219},
	{SG361, &UA575107},
	{SG362, &UA632119},
	{SG363, &UA641053},
	{SG364, &UA651048},
	{SG365, &UA656076},
	{SG366, &UA661917},
	{SG367, &UA675045},
	{SG368, &UA677703},
	{SG369, &UA678265},
	{SG370, &UA683706},
	{SG371, &UA686871},
	{SG372, &UA688242},
	{SG373, &UA696426},
	{SG374, &UA699477},
	{SG375, &UA724599},
	{SG376, &UA735861},
	{SG377, &UA736758},
	{SG378, &UA739051},
	{SG379, &UA740316},
	{SG380, &UA740850},
	{SG381, &UA750332},
	{SG382, &UA754514},
	{SG383, &UA755650},
	{SG384, &UA763738},
	{SG385, &UA771328},
	{SG386, &UA772364},
	{SG387, &UA775496},
	{SG388, &UA817830},
	{SG389, &UA825627},
	{SG390, &UA851298},
	{SG391, &UA910261},
	{SG392, &UA1862725},
	{SG393, &UA1880774},
	{SG394, &UA1883563},
	{SG395, &UA1883843},
	{SG396, &UA1889287},
	{SG397, &UA1898092},
	{SG398, &UA1898542},
	{SG399, &UA1903811},
	{SG400, &UA1905771},
	{SG401, &UA1934361},
	{SG402, &UA1940455},
	{SG403, &UA1941387},
	{SG404, &UA1951540},
	{SG405, &UA1979179},
	{SG406, &UA2018124},
	{SG407, &UA2018250},
	{SG408, &UA2018310},
	{SG409, &UA2018547},
	{SG410, &UA2018616},
	{SG411, &UA2018628},
	{SG412, &UA2019207},
	{SG413, &UA2019211},
	{SG414, &UA2019284},
	{SG415, &UA2019302},
	{SG416, &UA2019430},
	{SG417, &UA2019541},
	{SG418, &UA2019597},
	{SG419, &UA2019721},
	{SG420, &UA2020039},
	{SG421, &UA2020055},
	{SG422, &UA2020264},
	{SG423, &UA2020273},
	{SG424, &UA2020548},
	{SG425, &UA2020603},
	{SG426, &UA2020763},
	{SG427, &UA2020827},
	{SG428, &UA2020841},
	{SG429, &UA2020879},
	{SG430, &UA2021076},
	{SG431, &UA2021094},
	{SG432, &UA2021600},
	{SG433, &UA2021766},
	{SG434, &UA2021865},
	{SG435, &UA2021880},
	{SG436, &UA2021976},
	{SG437, &UA2022135},
	{SG438, &UA2022256},
	{SG439, &UA2022304},
	{SG440, &UA2022450},
	{SG441, &UA2022649},
	{SG442, &UA2022920},
	{SG443, &UA2023184},
	{SG444, &UA2023353},
	{SG445, &UA2023532},
	{SG446, &UA2023695},
	{SG447, &UA2024177},
	{SG448, &UA2024757},
	{SG449, &UA2025111},
	{SG450, &UA2025117},
	{SG451, &UA2025252},
	{SG453, &UA2025599},
	{SG454, &UA2027279},
	{SG455, &UA2027456},
	{SG456, &UA2029999},
	{SG458, &UA2032427},
	{SG461, &UA2034204},
	{SG463, &UA2034563},
	{SG466, &UA2262150},
	{SG468, &UA2409897},
	{SG469, &UA2410796},
	{SG471, &UA2413619},
	{SG474, &UA2439670},
	{SG479, &UA2479402},
	{SG480, &UA2494343},
	{SG482, &UA2520752},
	{SG483, &UA2542503},
	{SG485, &UA2544380},
	{SG487, &UA2588216},
	{SG489, &UA2588276},
	{SG492, &UA2589467},
	{SG495, &UA2640681},
	{SG497, &UA2640961},
	{SG501, &UA2774956},
	{SG503, &UA2775342},
	{SG505, &UA2786844},
	{SG507, &UA2809352},
	{SG508, &UA2810331},
	{SG510, &UA2812165},
	{SG511, &UA2812963},
	{SG513, &UA2832152},
	{SG514, &UA2863812},
	{SG516, &UA2949302},
	{SG518, &UA3042537},
	{SG519, &UA3098267},
	{SG521, &UA3098903},
	{SG612, &UA6916},
	{SG613, &UA51152},
	{SG614, &UA63610},
	{SG615, &UA20046},
	{SG616, &UA20048},
	{SG617, &UA58559},
	{SG618, &UA2019013},
	{SG619, &UA2023531},
	{SG620, &UA173314},
	{SG621, &UA184927},
	{SG622, &UA188928},
	{SG623, &UA394663},
	{SG624, &UA574814},
	{SG625, &UA632239},
	{SG626, &UA707339},
	{SG627, &UA859428},
	{SG628, &UA1972098},
	{SG629, &UA2018795},
	{SG630, &UA2018884},
	{SG631, &UA2019056},
	{SG632, &UA2019629},
	{SG633, &UA2022203},
	{SG634, &UA2371809},
	{SG635, &UA2379102},
	{SG636, &UA2809312},
	{SG637, &UA2822410},
	{SG638, &UA3042485},
	{SG707, &UA22166},
	{SG708, &UA90546},
	{SG709, &UA192688},
	{SG710, &UA232356},
	{SG711, &UA417663},
	{SG712, &UA534697},
	{SG713, &UA16537},
	{SG714, &UA16538},
	{SG715, &UA16541},
	{SG716, &UA56855},
	{SG717, &UA56867},
	{SG718, &UA76835},
	{SG719, &UA656199},
	{SG720, &UA694061},
	{SG721, &UA2021989},
	{SG722, &UA6560},
	{SG723, &UA6603},
	{SG724, &UA6629},
	{SG725, &UA9314},
	{SG726, &UA32320},
	{SG727, &UA32321},
	{SG728, &UA32323},
	{SG729, &UA32331},
	{SG730, &UA32337},
	{SG731, &UA32341},
	{SG732, &UA32349},
	{SG733, &UA32353},
	{SG734, &UA32354},
	{SG735, &UA32371},
	{SG736, &UA32376},
	{SG737, &UA32446},
	{SG738, &UA32449},
	{SG739, &UA37777},
	{SG740, &UA37785},
	{SG741, &UA44269},
	{SG742, &UA51175},
	{SG743, &UA165295},
	{SG744, &UA684430},
	{SG745, &UA899227},
	{SG746, &UA2018982},
	{SG747, &UA2025366},
	{SG748, &UA2028516},
	{SG749, &UA2034276},
	{SG750, &UA2375695},
	{SG751, &UA2847191},
	{SG758, &UA54266},
	{SG759, &UA57925},
	{SG760, &UA70727},
	{SG761, &UA90483},
	{SG762, &UA188557},
	{SG763, &UA2018397},
	{SG764, &UA2643062},
	{SG765, &UA12872},
	{SG766, &UA34314},
	{SG767, &UA40620},
	{SG768, &UA55942},
	{SG769, &UA90518},
	{SG770, &UA187935},
	{SG771, &UA2020666},
	{SG798, &UA53735},
	{SG800, &UA7463},
	{SG801, &UA11661},
	{SG802, &UA11663},
	{SG803, &UA26894},
	{SG804, &UA26895},
	{SG805, &UA68054},
	{SG806, &UA1861993},
	{SG807, &UA2018564},
	{SG808, &UA2019125},
	{SG830, &UA8720},
	{SG831, &UA10680},
	{SG832, &UA10780},
	{SG833, &UA10787},
	{SG834, &UA14611},
	{SG835, &UA14661},
	{SG836, &UA14682},
	{SG837, &UA14691},
	{SG838, &UA14725},
	{SG839, &UA14756},
	{SG840, &UA39586},
	{SG841, &UA39668},
	{SG842, &UA42603},
	{SG843, &UA54417},
	{SG844, &UA54475},
	{SG845, &UA61750},
	{SG846, &UA165362},
	{SG847, &UA652312},
	{SG848, &UA2019120},
	{SG849, &UA2336421},
	{SG850, &UA23187},
	{SG851, &UA565813},
	{SG852, &UA62640},
	{SG853, &UA64777},
	{SG854, &UA187906},
	{SG855, &UA575569},
	{SG856, &UA814280},
	{SG857, &UA2034155},
	{SG858, &UA2370615},
	{SG883, &UA171098},
	{SG884, &UA173931},
	{SG885, &UA556470},
	{SG886, &UA703468},
	{SG887, &UA1883286},
	{SG888, &UA2018622},
	{SG889, &UA2018874},
	{SG890, &UA2018938},
	{SG907, &UA142494},
	{SG908, &UA142847},
	{SG909, &UA158843},
	{SG910, &UA297306},
	{SG911, &UA302666},
	{SG914, &UA896626},
	{SG932, &UA11337},
	{SG933, &UA11339},
	{SG934, &UA11365},
	{SG935, &UA11366},
	{SG936, &UA11369},
	{SG937, &UA11696},
	{SG938, &UA11697},
	{SG939, &UA11744},
	{SG940, &UA11745},
	{SG941, &UA11746},
	{SG942, &UA11747},
	{SG943, &UA11748},
	{SG944, &UA11749},
	{SG945, &UA11751},
	{SG946, &UA26928},
	{SG947, &UA26932},
	{SG948, &UA40093},
	{SG949, &UA55353},
	{SG950, &UA55406},
	{SG951, &UA55705},
	{SG952, &UA90529},
	{SG953, &UA524056},
	{SG967, &UA24586},
	{SG968, &UA42968},
	{SG969, &UA70070},
	{SG970, &UA71178},
	{SG971, &UA72867},
	{SG972, &UA74427},
	{SG973, &UA207076},
	{SG974, &UA369308},
	{SG975, &UA371309},
	{SG976, &UA633332},
	{SG977, &UA1907228},
	{SG978, &UA2019381},
	{SG979, &UA2022395},
	{SG981, &UA24602},
	{SG982, &UA38126},
	{SG983, &UA51273},
	{SG984, &UA1888260},
	{SG985, &UA2019179},
	{SG986, &UA2022670},
	{SG987, &UA90349},
	{SG988, &UA167940},
	{SG989, &UA301554},
	{SG1021, &UA16785},
	{SG1023, &UA16828},
	{SG1024, &UA16829},
	{SG1026, &UA16835},
	{SG1027, &UA16839},
	{SG1028, &UA16849},
	{SG1029, &UA64810},
	{SG1031, &UA74662},
	{SG1032, &UA174605},
	{SG1033, &UA369305},
	{SG1034, &UA521831},
	{SG1037, &UA2018104},
	{SG1053, &UA68133},
	{SG1054, &UA178773},
	{SG1055, &UA2607755},
	{SG1056, &UA2607805},
	{SG1057, &UA2610683},
	{SG1058, &UA2615848},
	{SG1059, &UA2615868},
	{SG1060, &UA2617205},
	{SG1061, &UA2618014},
	{SG1062, &UA2620356},
	{SG1063, &UA2620938},
	{SG1064, &UA2623378},
	{SG1065, &UA2625890},
	{SG1066, &UA2625940},
	{SG1067, &UA2628278},
	{SG1108, &UA189204},
	{SG1109, &UA302484},
	{SG1110, &UA622910},
	{SG1111, &UA693435},
	{SG1112, &UA910158},
	{SG1113, &UA1881384},
	{SG1114, &UA2043700},
	{SG1115, &UA2043701},
	{SG1116, &UA2043708},
	{SG1117, &UA2043709},
	{SG1118, &UA2043727},
	{SG1119, &UA2043739},
	{SG1120, &UA2043742},
	{SG1121, &UA2043741},
	{SG1123, &UA169210},
	{SG1124, &UA169737},
	{SG1125, &UA532221},
	{SG1126, &UA536849},
	{SG1127, &UA548311},
	{SG1128, &UA1877115},
	{SG1129, &UA2018638},
	{SG1131, &UA48456},
	{SG1132, &UA48458},
	{SG1133, &UA61413},
	{SG1134, &UA61415},
	{SG1135, &UA61422},
	{SG1136, &UA632102},
	{SG1137, &UA196577},
	{SG1138, &UA537154},
	{SG1139, &UA2020093},
	{SG1151, &UA22267},
	{SG1152, &UA24608},
	{SG1153, &UA42794},
	{SG1154, &UA178718},
	{SG1155, &UA758386},
	{SG1156, &UA1967149},
	{SG1157, &UA2462926},
	{SG1193, &UA26442},
	{SG1194, &UA54669},
	{SG1195, &UA291895},
	{SG1196, &UA292902},
	{SG1197, &UA299609},
	{SG1198, &UA491528},
	{SG1199, &UA1875247},
	{SG1200, &UA2023942},
	{SG1201, &UA2600766},
	{SG1216, &UA22383},
	{SG1218, &UA59866},
	{SG1219, &UA59868},
	{SG1220, &UA59871},
	{SG1221, &UA59873},
	{SG1222, &UA59880},
	{SG1223, &UA315100},
	{SG1225, &UA24480},
	{SG1226, &UA41372},
	{SG1227, &UA60607},
	{SG1228, &UA64524},
	{SG1229, &UA90712},
	{SG1230, &UA176035},
	{SG1231, &UA379901},
	{SG1232, &UA551148},
	{SG1233, &UA2306462},
	{SG1234, &UA2544472},
	{SG1235, &UA181495},
	{SG1236, &UA308876},
	{SG1237, &UA2019147},
	{SG1238, &UA16430},
	{SG1239, &UA54637},
	{SG1242, &UA374532},
	{SG1258, &UA177912},
	{SG1259, &UA195227},
	{SG1274, &UA42272},
	{SG1277, &UA21277},
	{SG1278, &UA197080},
	{SG1280, &UA172538},
	{SG1281, &UA175499},
	{SG1282, &UA870090},
	{SG1285, &UA16404},
	{SG1286, &UA40035},
	{SG1287, &UA40036},
	{SG1288, &UA149736},
	{SG1289, &UA300031},
	{SG1323, &UA316195},
	{SG1337, &UA56152},
	{SG1338, &UA67850},
	{SG1339, &UA67854},
	{SG1340, &UA67998},
	{SG1341, &UA68112},
	{SG1342, &UA68150},
	{SG1343, &UA68175},
	{SG1344, &UA159340},
	{SG1368, &UA56079},
	{SG1369, &UA56097},
	{SG1370, &UA67059},
	{SG1371, &UA67086},
	{SG1372, &UA67155},
	{SG1373, &UA67493},
	{SG1374, &UA67595},
	{SG1375, &UA67617},
	{SG1376, &UA67670},
	{SG1377, &UA67723},
	{SG1378, &UA67847},
	{SG1379, &UA68046},
	{SG1380, &UA161686},
	{SG1381, &UA844231},
	{SG1382, &UA16613},
	{SG1383, &UA42580},
	{SG1384, &UA42602},
	{SG1385, &UA49088},
	{SG1386, &UA54416},
	{SG1387, &UA70448},
	{SG1388, &UA76603},
	{SG1389, &UA568620},
	{SG1390, &UA2024638},
	{SG1463, &UA60649},
	{SG1472, &UA60385},
	{SG1473, &UA60570},
	{SG1474, &UA2019749},
	{SG1487, &UA22259},
	{SG1488, &UA182705},
	{SG1489, &UA2018647},
	{SG1490, &UA61442},
	{SG1491, &UA61444},
	{SG1492, &UA635450},
	{SG1524, &UA13246},
	{SG1525, &UA13247},
	{SG1526, &UA20327},
	{SG1527, &UA20329},
	{SG1528, &UA20330},
	{SG1529, &UA44058},
	{SG1530, &UA56513},
	{SG1531, &UA377390},
	{SG1545, &UA28084},
	{SG1546, &UA56070},
	{SG1586, &UA11625},
	{SG1587, &UA40408},
	{SG1588, &UA55614},
	{SG1589, &UA55627},
	{SG1590, &UA55647},
	{SG1591, &UA55652},
	{SG1592, &UA53750},
	{SG1597, &UA56974},
	{SG1600, &UA8755},
	{SG1603, &UA8776},
	{SG1605, &UA8778},
	{SG1607, &UA8789},
	{SG1609, &UA8803},
	{SG1611, &UA8822},
	{SG1613, &UA11905},
	{SG1617, &UA12530},
	{SG1618, &UA12545},
	{SG1619, &UA12546},
	{SG1620, &UA38905},
	{SG1621, &UA52592},
	{SG1622, &UA63281},
	{SG1623, &UA71761},
	{SG1751, &UA30781},
	{SG1753, &UA50579},
	{SG1755, &UA50582},
	{SG1757, &UA50584},
	{SG1759, &UA50587},
	{SG1762, &UA50589},
	{SG1774, &UA15018},
	{SG1775, &UA15020},
	{SG1776, &UA15027},
	{SG1777, &UA15030},
	{SG1778, &UA15038},
	{SG1779, &UA15047},
	{SG1780, &UA63167},
	{SG1781, &UA62985},
	{SG1799, &UA8602},
	{SG1800, &UA55240},
	{SG1801, &UA90469},
	{SG1833, &UA7983},
	{SG1834, &UA8329},
	{SG1835, &UA9508},
	{SG1836, &UA52120},
	{SG1837, &UA55658},
	{SG1838, &UA370329},
	{SG1841, &UA41006},
	{SG1843, &UA58758},
	{SG1863, &UA18077},
	{SG1864, &UA40739},
	{SG1865, &UA57273},
	{SG1866, &UA57279},
	{SG1867, &UA2018436},
	{SG1871, &UA73200},
	{SG1874, &UA159711},
	{SG1876, &UA178769},
	{SG1878, &UA317922},
	{SG1880, &UA389706},
	{SG1883, &UA559884},
	{SG1885, &UA2019428},
	{SG1888, &UA157854},
	{SG1889, &UA68027},
	{SG1890, &UA68207},
	{SG1891, &UA181916},
	{SG1942, &UA17969},
	{SG1943, &UA17970},
	{SG1944, &UA53598},
	{SG1945, &UA53600},
	{SG1946, &UA310575},
	{SG1947, &UA18549},
	{SG1948, &UA692586},
	{SG1951, &UA190297},
	{SG1952, &UA633824},
	{SG1954, &UA175806},
	{SG1955, &UA220990},
	{SG1956, &UA223933},
	{SG1957, &UA2018390},
	{SG1979, &UA9499},
	{SG1980, &UA56064},
	{SG1981, &UA67035},
	{SG1982, &UA67866},
	{SG1983, &UA67971},
	{SG1984, &UA68085},
	{SG1985, &UA68162},
	{SG1986, &UA435635},
	{SG1987, &UA439779},
	{SG1988, &UA657345},
	{SG1989, &UA869497},
	{SG1990, &UA2018286},
	{SG1991, &UA2018694},
	{SG1992, &UA2019150},
	{SG1993, &UA2021324},
	{SG1994, &UA5978},
	{SG1996, &UA5985},
	{SG1997, &UA50572},
	{SG1998, &UA56327},
	{SG1999, &UA377064},
	{SG2000, &UA2019055},
	{SG2001, &UA2019067},
	{SG2002, &UA2019129},
	{SG2003, &UA2019227},
	{SG2004, &UA2019364},
	{SG2005, &UA2019544},
	{SG2017, &UA166460},
	{SG2018, &UA2019127},
	{SG2023, &UA55115},
	{SG2024, &UA55116},
	{SG2025, &UA90293},
	{SG2026, &UA500611},
	{SG2027, &UA60658},
	{SG2028, &UA2020295},
	{SG2104, &UA644144},
	{SG2105, &UA2018446},
	{SG2111, &UA220508},
	{SG2112, &UA226746},
	{SG2113, &UA708032},
	{SG2117, &UA56385},
	{SG2118, &UA70839},
	{SG2129, &UA53748},
	{SG2130, &UA287347},
	{SG2155, &UA167883},
	{SG2156, &UA177565},
	{SG2193, &UA67355},
	{SG2194, &UA302125},
	{SG2195, &UA549272},
	{SG2196, &UA781258},
	{SG2224, &UA176044},
	{SG2225, &UA250342},
	{SG2226, &UA333630},
	{SG2227, &UA626574},
	{SG2228, &UA631820},
	{SG2229, &UA2205790},
	{SG2230, &UA3042969},
	{SG2253, &UA187598},
	{SG2254, &UA2019217},
	{SG2256, &UA19832},
	{SG2257, &UA19869},
	{SG2258, &UA58449},
	{SG2259, &UA58448},
	{SG2260, &UA58451},
	{SG2261, &UA58465},
	{SG2263, &UA58473},
	{SG2266, &UA58474},
	{SG2267, &UA317891},
	{SG2268, &UA175699},
	{SG2269, &UA625182},
	{SG2270, &UA64468},
	{SG2271, &UA160367},
	{SG2275, &UA309205},
	{SG2276, &UA312055},
	{SG2277, &UA314625},
	{SG2278, &UA315912},
	{SG2279, &UA2018873},
	{SG2316, &UA171618},
	{SG2317, &UA2019974},
	{SG2320, &UA20889},
	{SG2321, &UA23578},
	{SG2322, &UA555181},
	{SG2323, &UA147377},
	{SG2324, &UA161905},
	{SG2325, &UA1874256},
	{SG2326, &UA2022244},
	{SG2328, &UA16705},
	{SG2329, &UA18094},
	{SG2330, &UA368678},
	{SG2331, &UA1906818},
	{SG2341, &UA188749},
	{SG2343, &UA193145},
	{SG2344, &UA193542},
	{SG2345, &UA1927225},
	{SG2379, &UA6436},
	{SG2380, &UA32094},
	{SG2381, &UA637292},
	{SG2415, &UA20847},
	{SG2417, &UA23722},
	{SG2420, &UA667400},
	{SG2422, &UA725762},
	{SG2424, &UA2364811},
	{SG2427, &UA630564},
	{SG2452, &UA173366},
	{SG2453, &UA2018581},
	{SG2454, &UA8717},
	{SG2455, &UA18426},
	{SG2456, &UA20553},
	{SG2457, &UA20605},
	{SG2458, &UA28260},
	{SG2459, &UA28309},
	{SG2460, &UA38795},
	{SG2461, &UA166816},
	{SG2462, &UA296704},
	{SG2463, &UA1856948},
	{SG2490, &UA167782},
	{SG2491, &UA2018488},
	{SG2544, &UA10986},
	{SG2545, &UA10989},
	{SG2546, &UA11019},
	{SG2547, &UA15042},
	{SG2548, &UA69622},
	{SG2549, &UA90292},
	{SG2550, &UA159556},
	{SG2558, &UA60980},
	{SG2559, &UA77405},
	{SG2563, &UA166716},
	{SG2564, &UA174607},
	{SG2565, &UA541096},
	{SG2566, &UA2019251},
	{SG2567, &UA20388},
	{SG2570, &UA11498},
	{SG2572, &UA26630},
	{SG2573, &UA55514},
	{SG2592, &UA175465},
	{SG2601, &UA182655},
	{SG2602, &UA197640},
	{SG2603, &UA60948},
	{SG2604, &UA187907},
	{SG2605, &UA542398},
	{SG2606, &UA1933443},
	{SG2607, &UA2030354},
	{SG2608, &UA683773},
	{SG2617, &UA67640},
	{SG2658, &UA16542},
	{SG2659, &UA557607},
	{SG2660, &UA850316},
	{SG2676, &UA162474},
	{SG2677, &UA719519},
	{SG2681, &UA178974},
	{SG2682, &UA302195},
	{SG2683, &UA701238},
	{SG2684, &UA2018639},
	{SG2689, &UA54878},
	{SG2690, &UA704401},
	{SG2691, &UA1875315},
	{SG2692, &UA2028421},
	{SG2693, &UA2543196},
	{SG2737, &UA2018252},
	{SG2751, &UA716521},
	{SG2767, &UA15021},
	{SG2768, &UA55666},
	{SG2769, &UA57929},
	{SG2770, &UA293381},
	{SG2771, &UA304726},
	{SG2772, &UA2018329},
	{SG2775, &UA90556},
	{SG2776, &UA221041},
	{SG2777, &UA246126},
	{SG2778, &UA319909},
	{SG2779, &UA1902405},
	{SG2780, &UA2023588},
	{SG2781, &UA63257},
	{SG2782, &UA90440},
	{SG2783, &UA392309},
	{SG2813, &UA875277},
	{SG2833, &UA2019265},
	{SG2835, &UA653621},
	{SG2836, &UA879115},
	{SG2837, &UA1867620},
	{SG2838, &UA1875604},
	{SG2839, &UA2018610},
	{SG2840, &UA2412499},
	{SG2841, &UA2431285},
	{SG2850, &UA645494},
	{SG2851, &UA673718},
	{SG2852, &UA703566},
	{SG2853, &UA813467},
	{SG2861, &UA170768},
	{SG2862, &UA305105},
	{SG2863, &UA526276},
	{SG2864, &UA2018787},
	{SG2865, &UA2019181},
	{SG2875, &UA67446},
	{SG2876, &UA534871},
	{SG2880, &UA389636},
	{SG2881, &UA548784},
	{SG2882, &UA550008},
	{SG2884, &UA172879},
	{SG2885, &UA656499},
	{SG2894, &UA563477},
	{SG2929, &UA245457},
	{SG2949, &UA171958},
	{SG2950, &UA234761},
	{SG2951, &UA248355},
	{SG2952, &UA311109},
	{SG2953, &UA328559},
	{SG2954, &UA557868},
	{SG2955, &UA687379},
	{SG2956, &UA1912911},
	{SG2961, &UA18407},
	{SG2962, &UA20846},
	{SG2963, &UA57613},
	{SG2974, &UA67707},
	{SG2981, &UA2025296},
	{SG2982, &UA2360261},
	{SG2983, &UA2888950},
	{SG2984, &UA342299},
	{SG2985, &UA2021123},
	{SG2996, &UA167132},
	{SG2997, &UA657611},
	{SG3012, &UA199241},
	{SG3013, &UA2020045},
	{SG3015, &UA145370},
	{SG3016, &UA625783},
	{SG3017, &UA2021470},
	{SG3020, &UA67541},
	{SG3026, &UA23643},
	{SG3045, &UA21509},
	{SG3046, &UA1870253},
	{SG3054, &UA170558},
	{SG3055, &UA447297},
	{SG3056, &UA182801},
	{SG3057, &UA1863532},
	{SG3058, &UA2019218},
	{SG3065, &UA514847},
	{SG3066, &UA2018764},
	{SG3071, &UA540041},
	{SG3072, &UA2018943},
	{SG3073, &UA2472362},
	{SG3079, &UA90394},
	{SG3080, &UA2018251},
	{SG3081, &UA2018497},
	{SG3085, &UA668510},
	{SG3086, &UA2020310},
	{SG3109, &UA396135},
	{SG3114, &UA166171},
	{SG3115, &UA451798},
	{SG3116, &UA183968},
	{SG3117, &UA190690},
	{SG3118, &UA392533},
	{SG3119, &UA448409},
	{SG3120, &UA1852929},
	{SG3121, &UA1882895},
	{SG3122, &UA2019091},
	{SG3123, &UA379134},
	{SG3127, &UA90527},
	{SG3128, &UA250152},
	{SG3129, &UA334695},
	{SG3130, &UA344881},
	{SG3131, &UA561794},
	{SG3132, &UA2024265},
	{SG3141, &UA142447},
	{SG3142, &UA2018164},
	{SG3146, &UA2384468},
	{SG3151, &UA663105},
	{SG3156, &UA52404},
	{SG3157, &UA67612},
	{SG3161, &UA2202908},
	{SG3162, &UA2537916},
	{SG3163, &UA3021047},
	{SG3169, &UA2019389},
	{SG3175, &UA721825},
	{SG3176, &UA879346},
	{SG3187, &UA416874},
	{SG3197, &UA745391},
	{SG3216, &UA760861},
	{SG3220, &UA171147},
	{SG3221, &UA2018895},
	{SG3227, &UA307980},
	{SG3228, &UA2018916},
	{SG3229, &UA2019396},
	{SG3245, &UA2431079},
	{SG3246, &UA640198},
	{SG3247, &UA2018747},
	{SG3250, &UA67695},
	{SG3259, &UA316694},
	{SG3260, &UA1904546},
	{SG3261, &UA170833},
	{SG3262, &UA2018509},
	{SG3263, &UA2018928},
	{SG3268, &UA397209},
	{SG3269, &UA564170},
	{SG3274, &UA768099},
	{SG3276, &UA2708543},
	{SG3300, &UA787579},
	{SG3301, &UA2019000},
	{SG3307, &UA436319},
	{SG3308, &UA2024090},
	{SG3327, &UA855579},
	{SG3328, &UA623200},
	{SG3332, &UA396257},
	{SG3348, &UA19676},
	{SG3350, &UA221358},
	{SG3351, &UA2616045},
	{SG3352, &UA319666},
	{SG3354, &UA2667275},
	{SG3357, &UA561968},
	{SG3358, &UA712314},
	{SG3359, &UA1882274},
	{SG3360, &UA2018746},
	{SG3363, &UA2019059},
	{SG3364, &UA2023139},
	{SG3374, &UA391110},
	{SG3375, &UA763006},
	{SG3377, &UA11611},
	{SG3378, &UA11620},
	{SG3379, &UA2020723},
	{SG3381, &UA395969},
	{SG3382, &UA2622344},
	{SG3397, &UA172390},
	{SG3398, &UA1905909},
	{SG3399, &UA2019226},
	{SG3400, &UA2235200},
	{SG3403, &UA33182},
	{SG3406, &UA2633727},
	{SG3419, &UA2262147},
	{SG3420, &UA900493},
	{SG3431, &UA182032},
	{SG3436, &UA2237667},
	{SG3444, &UA41838},
	{SG3445, &UA650470},
	{SG3446, &UA42356},
	{SG3447, &UA576305},
	{SG3448, &UA672646},
	{SG3453, &UA307196},
	{SG3454, &UA2020270},
	{SG3464, &UA90523},
	{SG3465, &UA450897},
	{SG3466, &UA573941},
	{SG3467, &UA771720},
	{SG3468, &UA2019144},
	{SG3472, &UA2580505},
	{SG3473, &UA3078570},
	{SG3474, &UA151651},
	{SG3475, &UA291620},
	{SG3476, &UA523680},
	{SG3477, &UA2128631},
	{SG3484, &UA2564899},
	{SG3485, &UA3058118},
	{SG3487, &UA734623},
	{SG3488, &UA1907546},
	{SG3500, &UA2262215},
	{SG3501, &UA2031580},
	{SG3502, &UA2867128},
	{SG3506, &UA2708931},
	{SG3524, &UA2262019},
	{SG3526, &UA67479},
	{SG3527, &UA705169},
	{SG3528, &UA2028238},
	{SG3529, &UA2455539},
	{SG3537, &UA2531692},
	{SG3538, &UA3078680},
	{SG3541, &UA2019367},
	{SG3542, &UA2234925},
	{SG3550, &UA2738465},
	{SG3580, &UA2485002},
	{SG3588, &UA844787},
	{SG3592, &UA301994},
	{SG3593, &UA690712},
	{SG3594, &UA2031779},
	{SG3595, &UA2563588},
	{SG3597, &UA2469204},
	{SG3600, &UA2458274},
	{SG3601, &UA90555},
	{SG3602, &UA2019760},
	{SG3604, &UA2617012},
	{SG3605, &UA2626809},
	{SG3606, &UA2636258},
	{SG3612, &UA32936},
	{SG3613, &UA2020696},
	{SG3614, &UA201692},
	{SG3615, &UA2019411},
	{SG3616, &UA2021202},
	{SG3635, &UA11929},
	{SG3636, &UA55901},
	{SG3637, &UA3070381},
	{SG3648, &UA2485535},
	{SG3660, &UA2738093},
	{SG3661, &UA33260},
	{SG3662, &UA42050},
	{SG3664, &UA21638},
	{SG3671, &UA2018957},
	{SG3674, &UA54656},
	{SG3675, &UA78108},
	{SG3682, &UA2026791},
	{SG3688, &UA23439},
	{SG3689, &UA42790},
	{SG3690, &UA2020348},
	{SG3691, &UA534789},
	{SG3692, &UA2018591},
	{SG3693, &UA2889714},
	{SG3714, &UA3065015},
	{SG3715, &UA649707},
	{SG3724, &UA702988},
	{SG3742, &UA2021191},
	{SG3743, &UA2022268},
	{SG3744, &UA2685407},
	{SG3747, &UA2939221},
	{SG3748, &UA3079812},
	{SG3759, &UA694319},
	{SG3762, &UA141234},
	{SG3763, &UA298722},
	{SG3764, &UA3029539},
	{SG3771, &UA2432514},
	{SG3778, &UA2028560},
	{SG3779, &UA2853120},
	{SG3783, &UA58800},
	{SG3786, &UA742898},
	{SG3788, &UA2618681},
	{SG3789, &UA2944986},
	{SG3795, &UA2034522},
	{SG3801, &UA178674},
	{SG3802, &UA2018838},
	{SG3804, &UA2424400},
	{SG3829, &UA2433388},
	{SG3831, &UA2026518},
	{SG3841, &UA2694507},
	{SG3872, &UA2631560},
	{SG3882, &UA51759},
	{SG3889, &UA2621553},
	{SG3890, &UA2635524},
	{SG3891, &UA2635523},
	{SG3893, &UA3093026},
	{SG3899, &UA691524},
	{SG3900, &UA3100081},
	{SG3906, &UA3010251},
	{SG3907, &UA3051113},
	{SG3911, &UA67262},
	{SG3914, &UA2455404},
	{SG3915, &UA3046344},
	{SG3921, &UA306182},
	{SG3922, &UA312089},
	{SG3923, &UA520257},
	{SG3924, &UA2021033},
	{SG3925, &UA1934563},
	{SG3926, &UA2455362},
	{SG3932, &UA2455497},
	{SG3939, &UA2335521},
	{SG3953, &UA3049739},
	{SG3956, &UA2608337},
	{SG3957, &UA2706472},
	{SG3961, &UA3073964},
	{SG3970, &UA3112968},
	{SG3972, &UA67708},
	{SG3973, &UA2018449},
	{SG3974, &UA743072},
	{SG3976, &UA2660352},
	{SG3979, &UA2803205},
	{SG3980, &UA1957270},
	{SG3981, &UA2452319},
	{SG3997, &UA2682939},
	{SG4038, &UA2636081},
	{SG4041, &UA2766513},
	{SG4045, &UA3018481},
	{SG4052, &UA2027078},
	{SG4056, &UA2706687},
	{SG4066, &UA3026248},
	{SG4073, &UA2018289},
	{SG4074, &UA2018672},
	{SG4081, &UA2958446},
	{SG4086, &UA2223260},
	{SG4090, &UA2721132},
	{SG4093, &UA2834557},
	{SG4096, &UA3014899},
	{SG4097, &UA3018397},
	{SG4105, &UA3125746},
	{SG4116, &UA2851302},
	{SG4120, &UA2986201},
	{SG4126, &UA3098362},
	{SG4129, &UA3125505},
	{SG4136, &UA2018790},
	{SG4138, &UA2018318},
	{SG4139, &UA2022003},
	{SG4150, &UA3085558},
	{SG4159, &UA3106526},
	{SG4169, &UA3125812},
	{SG4171, &UA61350},
	{SG4172, &UA2022041},
	{SG4174, &UA303999},
	{SG4176, &UA323837},
	{SG4179, &UA2018264},
	{SG4180, &UA2275909},
	{SG4183, &UA2018256},
	{SG4185, &UA2821890},
	{SG4189, &UA2614006},
	{SG4201, &UA2628164},
	{SG4208, &UA767380},
	{SG4214, &UA2028032},
	{SG4215, &UA2625012},
	{SG4219, &UA67437},
	{SG4221, &UA2021396},
	{SG4225, &UA2629671},
	{SG4235, &UA2020309},
	{SG4238, &UA2028468},
	{SG4240, &UA2033614},
	{SG4241, &UA2615944},
	{SG4250, &UA587186},
	{SG4251, &UA2023607},
	{SG4252, &UA2031801},
	{SG4254, &UA2623272},
	{SG4255, &UA2627101},
	{SG4265, &UA2019782},
	{SG4266, &UA2624299},
	{SG4267, &UA2647624},
	{SG4281, &UA755370},
	{SG4282, &UA2023718},
	{SG4283, &UA2617568},
	{SG4284, &UA2623177},
	{SG4285, &UA2632499},
	{SG4301, &UA2018430},
	{SG4303, &UA2019378},
	{SG4304, &UA2621070},
	{SG4306, &UA2613632},
	{SG4307, &UA2618389},
	{SG4317, &UA609219},
	{SG4332, &UA2632743},
	{SG4340, &UA17111},
	{SG4341, &UA17113},
	{SG4361, &UA697608},
	{SG4362, &UA1948328},
	{SG4363, &UA2019804},
	{SG4386, &UA370461},
	{SG4387, &UA371265},
	{SG4399, &UA2020692},
	{SG4401, &UA2026609},
	{SG4403, &UA2607636},
	{SG4404, &UA2609508},
	{SG4419, &UA39351},
	{SG4441, &UA857481},
	{SG4454, &UA2028424},
	{SG4462, &UA2610567},
	{SG4466, &UA57582},
	{SG4472, &UA368799},
	{SG4474, &UA2018925},
	{SG4477, &UA2024873},
	{SG4478, &UA2026967},
	{SG4484, &UA2614015},
	{SG4490, &UA2634781},
	{SG4491, &UA2634925},
	{SG4521, &UA370672},
	{SG4525, &UA2032064},
	{SG4526, &UA2033304},
	{SG4538, &UA2617865},
	{SG4540, &UA2617908},
	{SG4542, &UA2617976},
	{SG4555, &UA2621157},
	{SG4564, &UA2628804},
	{SG4566, &UA2632830},
	{SG4572, &UA2636732},
	{SG4573, &UA2636845},
	{NULL}};
const Segmentdata DAndroidNonNumeric[] = {
	{SG0, &UA2967672},
	{SG1, &UA3114786},
	{SG2, &UA349835},
	{SG3, &UA370451},
	{SG4, &UA571694},
	{SG5, &UA2020725},
	{SG6, &UA2023606},
	{SG7, &UA2427893},
	{SG8, &UA353150},
	{SG9, &UA2431158},
	{SG10, &UA2708988},
	{SG53, &UA38136},
	{SG54, &UA51749},
	{SG55, &UA51761},
	{SG56, &UA51767},
	{SG57, &UA51780},
	{SG58, &UA51782},
	{SG59, &UA55667},
	{SG60, &UA2018781},
	{SG61, &UA2242283},
	{SG62, &UA2574863},
	{SG63, &UA2661956},
	{SG64, &UA59258},
	{SG65, &UA2619155},
	{SG88, &UA2751912},
	{SG89, &UA890622},
	{SG90, &UA2797855},
	{SG157, &UA2608817},
	{SG175, &UA2618321},
	{SG183, &UA2620737},
	{SG200, &UA2625514},
	{SG235, &UA888152},
	{SG236, &UA2431421},
	{SG263, &UA1940585},
	{SG265, &UA2588110},
	{SG266, &UA2024967},
	{SG281, &UA2027425},
	{SG326, &UA2023996},
	{SG327, &UA2025417},
	{SG330, &UA2024704},
	{SG331, &UA2431784},
	{SG332, &UA2025770},
	{SG452, &UA2642917},
	{SG752, &UA2790853},
	{SG777, &UA33224},
	{SG778, &UA49975},
	{SG779, &UA257799},
	{SG780, &UA351195},
	{SG781, &UA689292},
	{SG782, &UA748206},
	{SG783, &UA2021210},
	{SG797, &UA2797909},
	{SG799, &UA3033106},
	{SG829, &UA3117759},
	{SG1036, &UA1916144},
	{SG1130, &UA2020583},
	{SG1178, &UA2682867},
	{SG1215, &UA2418488},
	{SG1217, &UA2746815},
	{SG1224, &UA2864527},
	{SG1272, &UA2798185},
	{SG1273, &UA3015973},
	{SG1275, &UA2412408},
	{SG1279, &UA2436652},
	{SG1283, &UA2488358},
	{SG1301, &UA67684},
	{SG1302, &UA67713},
	{SG1303, &UA69277},
	{SG1304, &UA69292},
	{SG1305, &UA116591},
	{SG1306, &UA142216},
	{SG1307, &UA153358},
	{SG1308, &UA167563},
	{SG1309, &UA296384},
	{SG1310, &UA301854},
	{SG1311, &UA306009},
	{SG1312, &UA314022},
	{SG1313, &UA498137},
	{SG1314, &UA507557},
	{SG1315, &UA513839},
	{SG1316, &UA518987},
	{SG1317, &UA633443},
	{SG1318, &UA747705},
	{SG1319, &UA800141},
	{SG1320, &UA1858702},
	{SG1321, &UA1878908},
	{SG1322, &UA1902033},
	{SG1345, &UA2235405},
	{SG1475, &UA2219145},
	{SG1846, &UA2032622},
	{SG1934, &UA720172},
	{SG2030, &UA2347817},
	{SG2031, &UA33200},
	{SG2032, &UA2028419},
	{SG2119, &UA89000},
	{SG2128, &UA2739818},
	{SG2159, &UA3115785},
	{SG2255, &UA2471651},
	{SG2272, &UA353811},
	{SG2273, &UA890675},
	{SG2274, &UA2289850},
	{SG2307, &UA2570137},
	{SG2315, &UA2508085},
	{SG2327, &UA2524613},
	{SG2499, &UA178476},
	{SG2500, &UA300110},
	{SG2501, &UA330111},
	{SG2502, &UA389783},
	{SG2503, &UA440003},
	{SG2504, &UA449093},
	{SG2505, &UA449912},
	{SG2506, &UA458728},
	{SG2507, &UA539853},
	{SG2508, &UA691061},
	{SG2509, &UA767703},
	{SG2510, &UA1897882},
	{SG2511, &UA2018487},
	{SG2512, &UA2018664},
	{SG2513, &UA2018829},
	{SG2515, &UA2018935},
	{SG2516, &UA2019005},
	{SG2517, &UA2019171},
	{SG2518, &UA2020037},
	{SG2519, &UA2020113},
	{SG2520, &UA2022565},
	{SG2526, &UA129707},
	{SG2527, &UA641282},
	{SG2528, &UA1899129},
	{SG2529, &UA1948719},
	{SG2530, &UA2020203},
	{SG2531, &UA2021126},
	{SG2532, &UA2463861},
	{SG2533, &UA2723154},
	{SG2534, &UA2732841},
	{SG2535, &UA2743632},
	{SG2536, &UA2744068},
	{SG2538, &UA2745358},
	{SG2539, &UA2764098},
	{SG2540, &UA2980239},
	{SG2551, &UA2984220},
	{SG2552, &UA3012964},
	{SG2553, &UA3043763},
	{SG2554, &UA3057159},
	{SG2555, &UA3074285},
	{SG2556, &UA3100011},
	{SG2557, &UA3108664},
	{SG2574, &UA448971},
	{SG2575, &UA542963},
	{SG2576, &UA549775},
	{SG2577, &UA706856},
	{SG2578, &UA716247},
	{SG2579, &UA747008},
	{SG2580, &UA765587},
	{SG2581, &UA2018415},
	{SG2582, &UA2018725},
	{SG2583, &UA2019136},
	{SG2584, &UA2019847},
	{SG2585, &UA2019904},
	{SG2586, &UA2023512},
	{SG2587, &UA2493691},
	{SG2589, &UA2767328},
	{SG2590, &UA2892776},
	{SG2645, &UA3083962},
	{SG2648, &UA2528126},
	{SG2649, &UA2751594},
	{SG2696, &UA1953349},
	{SG2714, &UA2798457},
	{SG2715, &UA2876262},
	{SG2716, &UA3116359},
	{SG2724, &UA2733879},
	{SG2736, &UA626559},
	{SG2764, &UA2407155},
	{SG2800, &UA224141},
	{SG2801, &UA794566},
	{SG2802, &UA2018759},
	{SG2866, &UA2937272},
	{SG2867, &UA3049774},
	{SG2868, &UA3061514},
	{SG2869, &UA455914},
	{SG2870, &UA614004},
	{SG2871, &UA896482},
	{SG2872, &UA2877035},
	{SG2907, &UA545615},
	{SG2917, &UA171299},
	{SG2919, &UA303645},
	{SG2921, &UA416045},
	{SG2922, &UA524521},
	{SG2923, &UA524761},
	{SG2924, &UA530032},
	{SG2925, &UA657235},
	{SG2926, &UA787023},
	{SG2927, &UA1870021},
	{SG2928, &UA2018643},
	{SG2979, &UA3010288},
	{SG2980, &UA2408125},
	{SG2986, &UA2025496},
	{SG2998, &UA2794029},
	{SG3008, &UA1900451},
	{SG3030, &UA1961175},
	{SG3067, &UA2341776},
	{SG3068, &UA1984421},
	{SG3077, &UA644439},
	{SG3092, &UA2018958},
	{SG3126, &UA2401525},
	{SG3155, &UA2466653},
	{SG3164, &UA2597596},
	{SG3196, &UA640509},
	{SG3202, &UA729050},
	{SG3206, &UA3225},
	{SG3207, &UA18526},
	{SG3208, &UA25866},
	{SG3209, &UA53741},
	{SG3210, &UA65421},
	{SG3211, &UA65431},
	{SG3212, &UA2018444},
	{SG3213, &UA2020735},
	{SG3253, &UA2399250},
	{SG3254, &UA2027718},
	{SG3255, &UA2030078},
	{SG3257, &UA2796329},
	{SG3277, &UA2617593},
	{SG3278, &UA3118480},
	{SG3279, &UA693552},
	{SG3297, &UA44038},
	{SG3306, &UA2886813},
	{SG3309, &UA2466199},
	{SG3345, &UA2520569},
	{SG3346, &UA53784},
	{SG3373, &UA3092193},
	{SG3383, &UA3079233},
	{SG3393, &UA2327873},
	{SG3415, &UA893237},
	{SG3416, &UA2707048},
	{SG3426, &UA2583382},
	{SG3432, &UA2608190},
	{SG3434, &UA2458996},
	{SG3435, &UA2458249},
	{SG3437, &UA2879246},
	{SG3507, &UA2413944},
	{SG3531, &UA16432},
	{SG3532, &UA59811},
	{SG3533, &UA1941737},
	{SG3534, &UA2021488},
	{SG3535, &UA2023792},
	{SG3536, &UA2025947},
	{SG3548, &UA2622470},
	{SG3549, &UA2637438},
	{SG3551, &UA2659129},
	{SG3553, &UA2596018},
	{SG3558, &UA8550},
	{SG3559, &UA8725},
	{SG3561, &UA9982},
	{SG3562, &UA10402},
	{SG3563, &UA10403},
	{SG3564, &UA10405},
	{SG3565, &UA10406},
	{SG3566, &UA10407},
	{SG3567, &UA10409},
	{SG3569, &UA38803},
	{SG3570, &UA44040},
	{SG3571, &UA44216},
	{SG3572, &UA44215},
	{SG3573, &UA52520},
	{SG3574, &UA58663},
	{SG3575, &UA11421},
	{SG3576, &UA2522328},
	{SG3582, &UA2736924},
	{SG3586, &UA2034396},
	{SG3587, &UA2611075},
	{SG3619, &UA448585},
	{SG3620, &UA544119},
	{SG3627, &UA2864105},
	{SG3629, &UA1909906},
	{SG3634, &UA2032587},
	{SG3640, &UA2619577},
	{SG3641, &UA2621539},
	{SG3643, &UA2863727},
	{SG3644, &UA24554},
	{SG3645, &UA33040},
	{SG3646, &UA43294},
	{SG3647, &UA43514},
	{SG3649, &UA59235},
	{SG3651, &UA73016},
	{SG3652, &UA2019778},
	{SG3654, &UA2023110},
	{SG3656, &UA2451158},
	{SG3658, &UA2641501},
	{SG3670, &UA2610958},
	{SG3672, &UA2610209},
	{SG3680, &UA2596212},
	{SG3683, &UA2618094},
	{SG3705, &UA2579808},
	{SG3706, &UA2768500},
	{SG3710, &UA2617855},
	{SG3717, &UA2524135},
	{SG3718, &UA2593889},
	{SG3720, &UA196053},
	{SG3721, &UA2032157},
	{SG3722, &UA2655295},
	{SG3740, &UA2610878},
	{SG3741, &UA2622973},
	{SG3746, &UA2792078},
	{SG3749, &UA2652939},
	{SG3751, &UA2616963},
	{SG3752, &UA2622109},
	{SG3753, &UA295717},
	{SG3757, &UA2623618},
	{SG3758, &UA2632797},
	{SG3760, &UA2613266},
	{SG3765, &UA2622454},
	{SG3766, &UA2617615},
	{SG3772, &UA2615924},
	{SG3773, &UA2622963},
	{SG3774, &UA2622966},
	{SG3776, &UA2623449},
	{SG3787, &UA2657597},
	{SG3800, &UA2607774},
	{SG3803, &UA2996431},
	{SG3805, &UA2031983},
	{SG3807, &UA2610612},
	{SG3814, &UA20240},
	{SG3815, &UA21356},
	{SG3816, &UA21357},
	{SG3817, &UA21358},
	{SG3818, &UA24074},
	{SG3819, &UA32757},
	{SG3825, &UA33223},
	{SG3826, &UA43327},
	{SG3828, &UA2619366},
	{SG3838, &UA2790404},
	{SG3839, &UA2844644},
	{SG3840, &UA6438},
	{SG3843, &UA6439},
	{SG3844, &UA11923},
	{SG3845, &UA37680},
	{SG3846, &UA40583},
	{SG3847, &UA50966},
	{SG3848, &UA55904},
	{SG3849, &UA55906},
	{SG3850, &UA55908},
	{SG3851, &UA55909},
	{SG3852, &UA55910},
	{SG3853, &UA71859},
	{SG3854, &UA369210},
	{SG3855, &UA2022447},
	{SG3856, &UA507140},
	{SG3857, &UA1863272},
	{SG3878, &UA270844},
	{SG3880, &UA2617500},
	{SG3881, &UA2610622},
	{SG3883, &UA58591},
	{SG3888, &UA2607680},
	{SG3895, &UA2615819},
	{SG3897, &UA2624167},
	{SG3898, &UA2624215},
	{SG3908, &UA2618227},
	{SG3909, &UA2621797},
	{SG3934, &UA2554771},
	{SG3935, &UA2637486},
	{SG3940, &UA10232},
	{SG3941, &UA10235},
	{SG3942, &UA10230},
	{SG3943, &UA25701},
	{SG3944, &UA30239},
	{SG3945, &UA30236},
	{SG3946, &UA30241},
	{SG3947, &UA30243},
	{SG3952, &UA2498301},
	{SG3958, &UA301325},
	{SG3960, &UA2474959},
	{SG3971, &UA2617819},
	{SG3975, &UA2889229},
	{SG3977, &UA2625246},
	{SG3978, &UA2995843},
	{SG3991, &UA2890929},
	{SG4001, &UA11162},
	{SG4002, &UA55139},
	{SG4003, &UA23746},
	{SG4004, &UA2496975},
	{SG4017, &UA2021785},
	{SG4027, &UA52047},
	{SG4028, &UA278380},
	{SG4033, &UA2619345},
	{SG4034, &UA2617355},
	{SG4040, &UA3108113},
	{SG4046, &UA2610070},
	{SG4047, &UA2617820},
	{SG4048, &UA2613546},
	{SG4067, &UA2609162},
	{SG4068, &UA2620836},
	{SG4069, &UA2617241},
	{SG4070, &UA2633019},
	{SG4071, &UA2621980},
	{SG4072, &UA2622160},
	{SG4084, &UA2617603},
	{SG4098, &UA2613122},
	{SG4099, &UA2616999},
	{SG4101, &UA2623442},
	{SG4102, &UA2621951},
	{SG4110, &UA2617816},
	{SG4111, &UA2615947},
	{SG4112, &UA2618148},
	{SG4113, &UA2616687},
	{SG4115, &UA2825103},
	{SG4119, &UA2979980},
	{SG4133, &UA2018283},
	{SG4134, &UA2619078},
	{SG4135, &UA2619249},
	{SG4137, &UA2617672},
	{SG4151, &UA6937},
	{SG4152, &UA32289},
	{SG4153, &UA32387},
	{SG4154, &UA1875276},
	{SG4156, &UA32395},
	{SG4157, &UA32411},
	{SG4158, &UA73442},
	{SG4163, &UA2621963},
	{SG4164, &UA2608290},
	{SG4165, &UA2608367},
	{SG4166, &UA2621561},
	{SG4167, &UA2622152},
	{SG4168, &UA2622905},
	{SG4173, &UA2951554},
	{SG4187, &UA66553},
	{SG4188, &UA643155},
	{SG4191, &UA2607880},
	{SG4192, &UA2607983},
	{SG4194, &UA2608030},
	{SG4195, &UA2627971},
	{SG4196, &UA2630893},
	{SG4197, &UA2625731},
	{SG4198, &UA2617334},
	{SG4203, &UA2810805},
	{SG4217, &UA3108222},
	{SG4222, &UA4584},
	{SG4224, &UA36876},
	{SG4231, &UA2608245},
	{SG4233, &UA2617636},
	{SG4236, &UA2866372},
	{SG4242, &UA3125400},
	{SG4245, &UA55989},
	{SG4246, &UA2024379},
	{SG4247, &UA2619539},
	{SG4248, &UA2636115},
	{SG4258, &UA2618253},
	{SG4260, &UA2618674},
	{SG4261, &UA2620655},
	{SG4262, &UA2624259},
	{SG4263, &UA2629428},
	{SG4270, &UA2613545},
	{SG4271, &UA2615713},
	{SG4272, &UA2618438},
	{SG4273, &UA2632417},
	{SG4274, &UA2622519},
	{SG4275, &UA2623092},
	{SG4276, &UA2626054},
	{SG4277, &UA2627587},
	{SG4279, &UA2625662},
	{SG4280, &UA2686630},
	{SG4289, &UA2618701},
	{SG4290, &UA2618744},
	{SG4291, &UA2625015},
	{SG4294, &UA2609450},
	{SG4297, &UA2614375},
	{SG4298, &UA2618968},
	{SG4299, &UA2625338},
	{SG4300, &UA2620582},
	{SG4302, &UA2630714},
	{SG4308, &UA369427},
	{SG4309, &UA2619349},
	{SG4310, &UA2620973},
	{SG4311, &UA2619640},
	{SG4312, &UA2633596},
	{SG4313, &UA2630610},
	{SG4318, &UA651525},
	{SG4319, &UA2021379},
	{SG4321, &UA2618915},
	{SG4322, &UA2618121},
	{SG4323, &UA2624104},
	{SG4324, &UA2622017},
	{SG4325, &UA2622357},
	{SG4326, &UA2631060},
	{SG4333, &UA2609539},
	{SG4334, &UA2619146},
	{SG4335, &UA2609936},
	{SG4336, &UA2621190},
	{SG4337, &UA2611639},
	{SG4338, &UA2619684},
	{SG4339, &UA2634295},
	{SG4343, &UA2607863},
	{SG4344, &UA2618998},
	{SG4345, &UA2625883},
	{SG4346, &UA2628409},
	{SG4347, &UA2618916},
	{SG4348, &UA2614382},
	{SG4349, &UA2615823},
	{SG4350, &UA2619239},
	{SG4351, &UA2619515},
	{SG4353, &UA2618533},
	{SG4354, &UA2635396},
	{SG4355, &UA2619784},
	{SG4356, &UA2621927},
	{SG4357, &UA2630342},
	{SG4358, &UA2625286},
	{SG4359, &UA3084440},
	{SG4371, &UA2625496},
	{SG4372, &UA2619541},
	{SG4373, &UA2617629},
	{SG4374, &UA2618152},
	{SG4375, &UA2619130},
	{SG4376, &UA2628411},
	{SG4377, &UA2628282},
	{SG4379, &UA2631724},
	{SG4382, &UA30242},
	{SG4385, &UA2024001},
	{SG4388, &UA2618248},
	{SG4389, &UA2618271},
	{SG4390, &UA2623354},
	{SG4392, &UA2633110},
	{SG4393, &UA2623207},
	{SG4395, &UA2628431},
	{SG4396, &UA2628410},
	{SG4407, &UA476652},
	{SG4408, &UA2018209},
	{SG4412, &UA2609666},
	{SG4413, &UA2609899},
	{SG4414, &UA2615596},
	{SG4415, &UA2618713},
	{SG4416, &UA2620896},
	{SG4423, &UA2020587},
	{SG4426, &UA2613020},
	{SG4427, &UA2618421},
	{SG4428, &UA2616175},
	{SG4429, &UA2619470},
	{SG4430, &UA2618236},
	{SG4431, &UA2618966},
	{SG4432, &UA2619924},
	{SG4433, &UA2625189},
	{SG4434, &UA2630477},
	{SG4435, &UA2631152},
	{SG4436, &UA2631485},
	{SG4437, &UA2631248},
	{SG4438, &UA2636891},
	{SG4450, &UA2242092},
	{SG4451, &UA2242104},
	{SG4452, &UA2613424},
	{SG4453, &UA2617509},
	{SG4455, &UA2616654},
	{SG4456, &UA2619008},
	{SG4457, &UA2620850},
	{SG4458, &UA2634463},
	{SG4459, &UA2622658},
	{SG4461, &UA2624029},
	{SG4463, &UA2626722},
	{SG4464, &UA40857},
	{SG4465, &UA51183},
	{SG4473, &UA2018743},
	{SG4475, &UA2616651},
	{SG4476, &UA2617536},
	{SG4479, &UA2621674},
	{SG4480, &UA2630412},
	{SG4481, &UA2621448},
	{SG4482, &UA2625285},
	{SG4483, &UA2625544},
	{SG4488, &UA7612},
	{SG4493, &UA67211},
	{SG4494, &UA76139},
	{SG4497, &UA2620945},
	{SG4498, &UA2611410},
	{SG4499, &UA2615579},
	{SG4500, &UA2617206},
	{SG4501, &UA2617374},
	{SG4502, &UA2617511},
	{SG4503, &UA2622445},
	{SG4504, &UA2617575},
	{SG4505, &UA2617934},
	{SG4506, &UA2618849},
	{SG4507, &UA2618955},
	{SG4508, &UA2619049},
	{SG4509, &UA2619971},
	{SG4510, &UA2620053},
	{SG4511, &UA2620932},
	{SG4512, &UA2621473},
	{SG4513, &UA2623777},
	{SG4514, &UA2625741},
	{SG4517, &UA2631220},
	{SG4518, &UA2634724},
	{SG4519, &UA2634859},
	{SG4520, &UA5824},
	{SG4527, &UA36907},
	{SG4528, &UA75299},
	{SG4534, &UA2241486},
	{SG4535, &UA2613853},
	{SG4536, &UA2626558},
	{SG4541, &UA2617212},
	{SG4543, &UA2627975},
	{SG4544, &UA2618416},
	{SG4545, &UA2618531},
	{SG4546, &UA2618558},
	{SG4547, &UA2618780},
	{SG4548, &UA2618972},
	{SG4549, &UA2618991},
	{SG4550, &UA2619573},
	{SG4551, &UA2619882},
	{SG4552, &UA2620946},
	{SG4553, &UA2622377},
	{SG4554, &UA2623025},
	{SG4556, &UA2626042},
	{SG4558, &UA2634471},
	{SG4559, &UA2627107},
	{SG4560, &UA2628417},
	{SG4561, &UA2636294},
	{SG4562, &UA2631515},
	{SG4563, &UA2634875},
	{SG4565, &UA3189},
	{SG4575, &UA2029993},
	{SG4576, &UA2241410},
	{SG4577, &UA2616884},
	{SG4578, &UA2617284},
	{SG4579, &UA2618712},
	{SG4580, &UA2619790},
	{SG4581, &UA2620627},
	{SG4582, &UA2620026},
	{SG4583, &UA2620350},
	{SG4584, &UA2620395},
	{SG4586, &UA2620642},
	{SG4587, &UA2620484},
	{SG4588, &UA2620521},
	{SG4589, &UA2621047},
	{SG4590, &UA2621672},
	{SG4591, &UA2621950},
	{SG4592, &UA2622314},
	{SG4593, &UA2622532},
	{SG4594, &UA2623070},
	{SG4595, &UA2624224},
	{SG4596, &UA2626853},
	{SG4597, &UA2629484},
	{SG4598, &UA2629920},
	{SG4599, &UA2630029},
	{SG4600, &UA2631207},
	{SG4601, &UA2631525},
	{SG4602, &UA2631954},
	{SG4603, &UA2632744},
	{SG4604, &UA2634883},
	{SG4605, &UA2637691},
	{SG4606, &UA2635323},
	{SG4609, &UA8278},
	{SG4610, &UA32203},
	{SG4611, &UA37678},
	{SG4612, &UA40244},
	{SG4613, &UA42148},
	{SG4617, &UA52435},
	{SG4624, &UA67861},
	{SG4625, &UA370052},
	{SG4629, &UA2242159},
	{SG4635, &UA2615232},
	{SG4636, &UA2615321},
	{SG4638, &UA2616631},
	{SG4640, &UA2617083},
	{SG4641, &UA2618204},
	{SG4642, &UA2618530},
	{SG4643, &UA2618888},
	{SG4644, &UA2619309},
	{SG4645, &UA2620783},
	{SG4646, &UA2621002},
	{SG4647, &UA2621403},
	{SG4648, &UA2621987},
	{SG4649, &UA2621993},
	{SG4650, &UA2622631},
	{SG4651, &UA2622716},
	{SG4652, &UA2623477},
	{SG4653, &UA2624464},
	{SG4654, &UA2624725},
	{SG4655, &UA2624751},
	{SG4656, &UA2625336},
	{SG4657, &UA2627123},
	{SG4658, &UA2628032},
	{SG4659, &UA2628109},
	{SG4660, &UA2628122},
	{SG4661, &UA2630061},
	{SG4662, &UA2630250},
	{SG4663, &UA2630316},
	{SG4664, &UA2631179},
	{SG4665, &UA2632026},
	{SG4666, &UA2632328},
	{SG4668, &UA2634857},
	{SG4669, &UA2636363},
	{SG4670, &UA2637365},
	{NULL}};
const Segmentdata DAndroidVersion2_2[] = {
	{SG13, &UA5237},
	{SG14, &UA5247},
	{SG15, &UA12557},
	{SG16, &UA20554},
	{SG17, &UA20863},
	{SG18, &UA21355},
	{SG20, &UA28420},
	{SG21, &UA29763},
	{SG22, &UA29780},
	{SG24, &UA29790},
	{SG25, &UA29810},
	{SG26, &UA29843},
	{SG28, &UA29849},
	{SG30, &UA29841},
	{SG32, &UA32112},
	{SG34, &UA32113},
	{SG35, &UA32122},
	{SG36, &UA32125},
	{SG37, &UA32124},
	{SG39, &UA54006},
	{SG41, &UA63892},
	{SG42, &UA64039},
	{SG43, &UA67583},
	{SG44, &UA122406},
	{SG45, &UA494895},
	{SG46, &UA501433},
	{SG47, &UA572920},
	{SG48, &UA622989},
	{SG49, &UA1965244},
	{SG50, &UA2018156},
	{SG51, &UA2019408},
	{SG52, &UA2686813},
	{SG120, &UA4921},
	{SG121, &UA4947},
	{SG122, &UA7456},
	{SG123, &UA14869},
	{SG124, &UA51799},
	{SG125, &UA63150},
	{SG126, &UA134153},
	{SG127, &UA140777},
	{SG128, &UA159404},
	{SG129, &UA292975},
	{SG130, &UA548459},
	{SG131, &UA791467},
	{SG132, &UA2018533},
	{SG133, &UA2441436},
	{SG134, &UA38499},
	{SG135, &UA40952},
	{SG136, &UA43728},
	{SG137, &UA48981},
	{SG138, &UA53684},
	{SG139, &UA64024},
	{SG140, &UA64136},
	{SG141, &UA67044},
	{SG142, &UA67819},
	{SG143, &UA75312},
	{SG144, &UA157650},
	{SG145, &UA178704},
	{SG146, &UA370762},
	{SG147, &UA370984},
	{SG148, &UA444386},
	{SG149, &UA543119},
	{SG150, &UA1858710},
	{SG151, &UA2018777},
	{SG152, &UA2019201},
	{SG153, &UA2019383},
	{SG154, &UA2026639},
	{SG155, &UA2402572},
	{SG156, &UA2607572},
	{SG158, &UA2607583},
	{SG159, &UA2607593},
	{SG160, &UA2607637},
	{SG161, &UA2607675},
	{SG162, &UA2607682},
	{SG163, &UA2607683},
	{SG164, &UA2607847},
	{SG165, &UA2607913},
	{SG166, &UA2607961},
	{SG167, &UA2607969},
	{SG168, &UA2608331},
	{SG169, &UA2609007},
	{SG170, &UA2609988},
	{SG171, &UA2610318},
	{SG172, &UA2610534},
	{SG173, &UA2610999},
	{SG174, &UA2611606},
	{SG176, &UA2611645},
	{SG177, &UA2614066},
	{SG178, &UA2614634},
	{SG179, &UA2614754},
	{SG180, &UA2614771},
	{SG181, &UA2615256},
	{SG182, &UA2615677},
	{SG184, &UA2615929},
	{SG185, &UA2616299},
	{SG186, &UA2616440},
	{SG187, &UA2617175},
	{SG188, &UA2617215},
	{SG189, &UA2617648},
	{SG190, &UA2617698},
	{SG191, &UA2618073},
	{SG192, &UA2618081},
	{SG193, &UA2618222},
	{SG194, &UA2618368},
	{SG195, &UA2618392},
	{SG196, &UA2618677},
	{SG197, &UA2618696},
	{SG198, &UA2618814},
	{SG199, &UA2618885},
	{SG201, &UA2618977},
	{SG202, &UA2618989},
	{SG203, &UA2619066},
	{SG204, &UA2619722},
	{SG205, &UA2620008},
	{SG206, &UA2620978},
	{SG207, &UA2621289},
	{SG208, &UA2621457},
	{SG209, &UA2621656},
	{SG210, &UA2621705},
	{SG211, &UA2622065},
	{SG212, &UA2622351},
	{SG213, &UA2622492},
	{SG214, &UA2623297},
	{SG215, &UA2623347},
	{SG216, &UA2623774},
	{SG217, &UA2625230},
	{SG218, &UA2626133},
	{SG219, &UA2626677},
	{SG220, &UA2626766},
	{SG221, &UA2626912},
	{SG222, &UA2626919},
	{SG223, &UA2626959},
	{SG224, &UA2627193},
	{SG225, &UA2627557},
	{SG226, &UA2629202},
	{SG227, &UA2629215},
	{SG228, &UA2632506},
	{SG229, &UA2632572},
	{SG230, &UA2632694},
	{SG231, &UA2633451},
	{SG232, &UA2636368},
	{SG233, &UA2636558},
	{SG234, &UA2637863},
	{SG251, &UA26556},
	{SG252, &UA26568},
	{SG253, &UA26588},
	{SG254, &UA26595},
	{SG255, &UA40248},
	{SG256, &UA41042},
	{SG257, &UA48911},
	{SG258, &UA53984},
	{SG259, &UA55493},
	{SG260, &UA64582},
	{SG261, &UA166966},
	{SG262, &UA524261},
	{SG264, &UA2034577},
	{SG328, &UA857872},
	{SG329, &UA2743093},
	{SG457, &UA10337},
	{SG459, &UA43291},
	{SG460, &UA49973},
	{SG462, &UA75505},
	{SG464, &UA167477},
	{SG465, &UA178438},
	{SG467, &UA258097},
	{SG470, &UA301489},
	{SG472, &UA304523},
	{SG473, &UA305141},
	{SG475, &UA312111},
	{SG476, &UA324639},
	{SG477, &UA324644},
	{SG478, &UA331408},
	{SG481, &UA368938},
	{SG484, &UA370603},
	{SG486, &UA388470},
	{SG488, &UA396089},
	{SG490, &UA396127},
	{SG491, &UA396945},
	{SG493, &UA400130},
	{SG494, &UA436260},
	{SG496, &UA458921},
	{SG498, &UA536251},
	{SG499, &UA539757},
	{SG500, &UA543740},
	{SG502, &UA549520},
	{SG504, &UA550313},
	{SG506, &UA551894},
	{SG509, &UA562067},
	{SG512, &UA573296},
	{SG515, &UA638088},
	{SG517, &UA639542},
	{SG520, &UA669189},
	{SG522, &UA670832},
	{SG523, &UA675431},
	{SG524, &UA686187},
	{SG525, &UA692415},
	{SG526, &UA695724},
	{SG527, &UA712125},
	{SG528, &UA715882},
	{SG529, &UA719125},
	{SG530, &UA719899},
	{SG531, &UA721087},
	{SG532, &UA723770},
	{SG533, &UA732981},
	{SG534, &UA734821},
	{SG535, &UA735809},
	{SG536, &UA763399},
	{SG537, &UA824725},
	{SG538, &UA829778},
	{SG539, &UA1870754},
	{SG540, &UA1876827},
	{SG541, &UA1893517},
	{SG542, &UA1893632},
	{SG543, &UA1893825},
	{SG544, &UA1894729},
	{SG545, &UA1903033},
	{SG546, &UA1924054},
	{SG547, &UA1924176},
	{SG548, &UA1931236},
	{SG549, &UA1962261},
	{SG550, &UA1965064},
	{SG551, &UA2018082},
	{SG552, &UA2018240},
	{SG553, &UA2018364},
	{SG554, &UA2018523},
	{SG555, &UA2018702},
	{SG556, &UA2018840},
	{SG557, &UA2018868},
	{SG558, &UA2018892},
	{SG559, &UA2018951},
	{SG560, &UA2018963},
	{SG561, &UA2019177},
	{SG562, &UA2019440},
	{SG563, &UA2019768},
	{SG564, &UA2020097},
	{SG565, &UA2020238},
	{SG566, &UA2020468},
	{SG567, &UA2021065},
	{SG568, &UA2021663},
	{SG569, &UA2022199},
	{SG570, &UA2022235},
	{SG571, &UA2022781},
	{SG572, &UA2022904},
	{SG573, &UA2022931},
	{SG574, &UA2022971},
	{SG575, &UA2026834},
	{SG576, &UA2027384},
	{SG577, &UA2027681},
	{SG578, &UA2027846},
	{SG579, &UA2027961},
	{SG580, &UA2028086},
	{SG581, &UA2029839},
	{SG582, &UA2031474},
	{SG583, &UA2032933},
	{SG584, &UA2033558},
	{SG585, &UA2033939},
	{SG586, &UA2033968},
	{SG587, &UA2256982},
	{SG588, &UA2339920},
	{SG589, &UA2419652},
	{SG590, &UA2466370},
	{SG591, &UA2495626},
	{SG592, &UA2572956},
	{SG593, &UA2589930},
	{SG594, &UA2639850},
	{SG595, &UA2720998},
	{SG596, &UA2742471},
	{SG597, &UA2743302},
	{SG598, &UA2804393},
	{SG599, &UA2818013},
	{SG600, &UA2978151},
	{SG655, &UA40052},
	{SG656, &UA163928},
	{SG657, &UA166914},
	{SG658, &UA175115},
	{SG659, &UA175969},
	{SG660, &UA177589},
	{SG661, &UA242060},
	{SG662, &UA258269},
	{SG663, &UA258597},
	{SG664, &UA299025},
	{SG665, &UA309400},
	{SG666, &UA309793},
	{SG667, &UA321305},
	{SG668, &UA525906},
	{SG669, &UA541859},
	{SG670, &UA624157},
	{SG671, &UA624889},
	{SG672, &UA813356},
	{SG673, &UA1911120},
	{SG675, &UA1992164},
	{SG677, &UA2018189},
	{SG679, &UA2018685},
	{SG684, &UA2019914},
	{SG687, &UA2020382},
	{SG691, &UA2024513},
	{SG692, &UA2031492},
	{SG693, &UA2374895},
	{SG694, &UA2392792},
	{SG695, &UA2393396},
	{SG696, &UA2787040},
	{SG697, &UA2949051},
	{SG698, &UA26972},
	{SG699, &UA26975},
	{SG700, &UA26979},
	{SG701, &UA26984},
	{SG702, &UA30138},
	{SG703, &UA30139},
	{SG704, &UA40533},
	{SG705, &UA63412},
	{SG706, &UA170013},
	{SG753, &UA16536},
	{SG754, &UA6921},
	{SG755, &UA1864649},
	{SG756, &UA2019827},
	{SG757, &UA2440913},
	{SG772, &UA1891715},
	{SG773, &UA2020962},
	{SG774, &UA34311},
	{SG775, &UA55940},
	{SG776, &UA300575},
	{SG809, &UA20},
	{SG810, &UA4936},
	{SG811, &UA7070},
	{SG812, &UA7192},
	{SG813, &UA26887},
	{SG814, &UA32528},
	{SG815, &UA32725},
	{SG816, &UA32731},
	{SG817, &UA37851},
	{SG818, &UA37853},
	{SG819, &UA37862},
	{SG820, &UA37850},
	{SG821, &UA51398},
	{SG822, &UA73541},
	{SG823, &UA74260},
	{SG824, &UA78090},
	{SG825, &UA153505},
	{SG826, &UA448561},
	{SG827, &UA780277},
	{SG828, &UA2021647},
	{SG859, &UA10690},
	{SG860, &UA14637},
	{SG861, &UA14785},
	{SG862, &UA14787},
	{SG863, &UA14791},
	{SG864, &UA44091},
	{SG865, &UA44093},
	{SG866, &UA48938},
	{SG867, &UA61748},
	{SG868, &UA150419},
	{SG869, &UA644392},
	{SG870, &UA2018292},
	{SG871, &UA2033664},
	{SG872, &UA18166},
	{SG873, &UA67319},
	{SG874, &UA67853},
	{SG875, &UA310483},
	{SG876, &UA547306},
	{SG877, &UA555780},
	{SG878, &UA634407},
	{SG879, &UA1881298},
	{SG880, &UA1958724},
	{SG881, &UA2019919},
	{SG882, &UA2409697},
	{SG954, &UA50453},
	{SG955, &UA105435},
	{SG956, &UA107927},
	{SG957, &UA111457},
	{SG958, &UA125431},
	{SG959, &UA125785},
	{SG960, &UA127629},
	{SG961, &UA129892},
	{SG962, &UA268200},
	{SG963, &UA650666},
	{SG964, &UA2018092},
	{SG965, &UA2018480},
	{SG966, &UA2019733},
	{SG980, &UA2019446},
	{SG1002, &UA6495},
	{SG1003, &UA6500},
	{SG1004, &UA6506},
	{SG1005, &UA6508},
	{SG1006, &UA6511},
	{SG1007, &UA6528},
	{SG1008, &UA51065},
	{SG1009, &UA51074},
	{SG1010, &UA51118},
	{SG1011, &UA51119},
	{SG1012, &UA51120},
	{SG1013, &UA51123},
	{SG1014, &UA64128},
	{SG1015, &UA165609},
	{SG1016, &UA299494},
	{SG1017, &UA473848},
	{SG1018, &UA679850},
	{SG1019, &UA2021331},
	{SG1020, &UA90339},
	{SG1022, &UA156291},
	{SG1025, &UA294928},
	{SG1030, &UA2018905},
	{SG1035, &UA16847},
	{SG1038, &UA4320},
	{SG1039, &UA4324},
	{SG1040, &UA4332},
	{SG1041, &UA24851},
	{SG1042, &UA24865},
	{SG1043, &UA24904},
	{SG1044, &UA24977},
	{SG1045, &UA25341},
	{SG1046, &UA45162},
	{SG1047, &UA54800},
	{SG1048, &UA163687},
	{SG1049, &UA784331},
	{SG1050, &UA1958158},
	{SG1051, &UA2018098},
	{SG1052, &UA2027506},
	{SG1068, &UA2631274},
	{SG1122, &UA448474},
	{SG1147, &UA150847},
	{SG1148, &UA165888},
	{SG1149, &UA315071},
	{SG1150, &UA373591},
	{SG1202, &UA54621},
	{SG1203, &UA54670},
	{SG1204, &UA89871},
	{SG1205, &UA109685},
	{SG1206, &UA273462},
	{SG1207, &UA462860},
	{SG1208, &UA481792},
	{SG1209, &UA687404},
	{SG1210, &UA2018069},
	{SG1211, &UA2018142},
	{SG1212, &UA2018279},
	{SG1213, &UA2381303},
	{SG1214, &UA2659089},
	{SG1253, &UA90401},
	{SG1254, &UA143239},
	{SG1255, &UA161431},
	{SG1256, &UA165887},
	{SG1257, &UA2022584},
	{SG1260, &UA23964},
	{SG1261, &UA376768},
	{SG1262, &UA2654152},
	{SG1276, &UA2018830},
	{SG1292, &UA3648},
	{SG1293, &UA10149},
	{SG1294, &UA16405},
	{SG1295, &UA48918},
	{SG1296, &UA62748},
	{SG1297, &UA137313},
	{SG1298, &UA832843},
	{SG1299, &UA1903389},
	{SG1300, &UA2023707},
	{SG1354, &UA50523},
	{SG1355, &UA89937},
	{SG1356, &UA475325},
	{SG1357, &UA2409804},
	{SG1358, &UA24319},
	{SG1359, &UA51879},
	{SG1360, &UA53599},
	{SG1361, &UA70447},
	{SG1362, &UA234131},
	{SG1363, &UA2019775},
	{SG1364, &UA2025922},
	{SG1464, &UA20522},
	{SG1465, &UA23166},
	{SG1466, &UA41358},
	{SG1467, &UA58873},
	{SG1468, &UA64313},
	{SG1469, &UA190121},
	{SG1470, &UA646131},
	{SG1471, &UA3041402},
	{SG1476, &UA2018831},
	{SG1522, &UA42862},
	{SG1523, &UA56509},
	{SG1536, &UA5155},
	{SG1537, &UA5158},
	{SG1538, &UA5159},
	{SG1539, &UA5160},
	{SG1540, &UA5162},
	{SG1541, &UA28075},
	{SG1542, &UA67874},
	{SG1543, &UA140848},
	{SG1544, &UA2020641},
	{SG1547, &UA40054},
	{SG1548, &UA53550},
	{SG1549, &UA64767},
	{SG1550, &UA190103},
	{SG1551, &UA67320},
	{SG1552, &UA90445},
	{SG1553, &UA389906},
	{SG1554, &UA10289},
	{SG1555, &UA68304},
	{SG1556, &UA169449},
	{SG1557, &UA211574},
	{SG1558, &UA2019479},
	{SG1559, &UA11605},
	{SG1560, &UA11609},
	{SG1561, &UA11615},
	{SG1562, &UA11621},
	{SG1563, &UA20167},
	{SG1564, &UA20168},
	{SG1565, &UA26765},
	{SG1566, &UA26770},
	{SG1567, &UA26773},
	{SG1568, &UA26783},
	{SG1569, &UA26784},
	{SG1570, &UA40425},
	{SG1571, &UA52161},
	{SG1572, &UA74358},
	{SG1573, &UA90119},
	{SG1574, &UA448629},
	{SG1575, &UA2018429},
	{SG1576, &UA2019522},
	{SG1577, &UA2811449},
	{SG1578, &UA5115},
	{SG1579, &UA7561},
	{SG1580, &UA16258},
	{SG1581, &UA22080},
	{SG1582, &UA641999},
	{SG1583, &UA655016},
	{SG1584, &UA2021005},
	{SG1585, &UA2026666},
	{SG1593, &UA12449},
	{SG1594, &UA12508},
	{SG1595, &UA12786},
	{SG1596, &UA62827},
	{SG1624, &UA5982},
	{SG1625, &UA11000},
	{SG1626, &UA15034},
	{SG1627, &UA39742},
	{SG1628, &UA62864},
	{SG1645, &UA64315},
	{SG1647, &UA534815},
	{SG1664, &UA151676},
	{SG1666, &UA190355},
	{SG1668, &UA400929},
	{SG1670, &UA434166},
	{SG1671, &UA435084},
	{SG1673, &UA448260},
	{SG1782, &UA7991},
	{SG1783, &UA7994},
	{SG1784, &UA8019},
	{SG1785, &UA8325},
	{SG1786, &UA9505},
	{SG1787, &UA9506},
	{SG1788, &UA9510},
	{SG1789, &UA26829},
	{SG1790, &UA26831},
	{SG1791, &UA38388},
	{SG1792, &UA52106},
	{SG1793, &UA52115},
	{SG1794, &UA52131},
	{SG1795, &UA52155},
	{SG1796, &UA69608},
	{SG1797, &UA74977},
	{SG1798, &UA2019340},
	{SG1804, &UA20302},
	{SG1805, &UA45216},
	{SG1806, &UA53936},
	{SG1807, &UA2018347},
	{SG1808, &UA2804328},
	{SG1842, &UA28079},
	{SG1844, &UA90436},
	{SG1845, &UA151615},
	{SG1854, &UA50502},
	{SG1855, &UA68136},
	{SG1856, &UA68274},
	{SG1857, &UA68273},
	{SG1858, &UA68442},
	{SG1859, &UA90020},
	{SG1860, &UA285329},
	{SG1861, &UA2018150},
	{SG1862, &UA2019475},
	{SG1892, &UA54007},
	{SG1893, &UA184205},
	{SG1894, &UA12924},
	{SG1895, &UA12928},
	{SG1896, &UA12933},
	{SG1897, &UA12934},
	{SG1898, &UA38563},
	{SG1899, &UA56213},
	{SG1900, &UA258716},
	{SG1901, &UA369517},
	{SG1902, &UA376444},
	{SG1903, &UA1905235},
	{SG1953, &UA160876},
	{SG1995, &UA1900118},
	{SG2011, &UA56053},
	{SG2013, &UA67065},
	{SG2029, &UA8947},
	{SG2041, &UA38537},
	{SG2043, &UA659323},
	{SG2058, &UA63275},
	{SG2059, &UA23176},
	{SG2060, &UA60656},
	{SG2061, &UA2019643},
	{SG2062, &UA2019753},
	{SG2063, &UA9140},
	{SG2064, &UA38284},
	{SG2065, &UA204869},
	{SG2120, &UA90307},
	{SG2121, &UA151890},
	{SG2122, &UA154993},
	{SG2123, &UA181504},
	{SG2124, &UA816155},
	{SG2125, &UA880058},
	{SG2126, &UA1875248},
	{SG2127, &UA1878147},
	{SG2157, &UA156661},
	{SG2158, &UA716452},
	{SG2160, &UA8834},
	{SG2161, &UA32034},
	{SG2162, &UA278311},
	{SG2163, &UA2018340},
	{SG2189, &UA20542},
	{SG2190, &UA20543},
	{SG2191, &UA23764},
	{SG2192, &UA2018424},
	{SG2197, &UA157261},
	{SG2198, &UA495028},
	{SG2199, &UA2018387},
	{SG2200, &UA2743554},
	{SG2223, &UA54799},
	{SG2262, &UA67661},
	{SG2264, &UA153848},
	{SG2265, &UA162585},
	{SG2283, &UA117822},
	{SG2284, &UA139498},
	{SG2285, &UA158171},
	{SG2286, &UA302898},
	{SG2287, &UA2018095},
	{SG2288, &UA2018295},
	{SG2289, &UA2536822},
	{SG2301, &UA156452},
	{SG2302, &UA765987},
	{SG2303, &UA2018195},
	{SG2304, &UA49396},
	{SG2310, &UA90269},
	{SG2311, &UA90425},
	{SG2312, &UA510768},
	{SG2313, &UA2018698},
	{SG2318, &UA23802},
	{SG2319, &UA630377},
	{SG2334, &UA180939},
	{SG2336, &UA360255},
	{SG2340, &UA180408},
	{SG2342, &UA2018406},
	{SG2346, &UA116004},
	{SG2351, &UA54353},
	{SG2352, &UA2018573},
	{SG2372, &UA179244},
	{SG2373, &UA458676},
	{SG2374, &UA458810},
	{SG2375, &UA459067},
	{SG2451, &UA41772},
	{SG2464, &UA175512},
	{SG2465, &UA458690},
	{SG2466, &UA458702},
	{SG2467, &UA458812},
	{SG2468, &UA599052},
	{SG2479, &UA40912},
	{SG2480, &UA64021},
	{SG2481, &UA532094},
	{SG2482, &UA67403},
	{SG2483, &UA5216},
	{SG2484, &UA28338},
	{SG2485, &UA53649},
	{SG2486, &UA53657},
	{SG2487, &UA290324},
	{SG2488, &UA2024188},
	{SG2492, &UA18515},
	{SG2493, &UA715612},
	{SG2494, &UA768044},
	{SG2514, &UA24393},
	{SG2522, &UA206019},
	{SG2523, &UA458701},
	{SG2524, &UA458786},
	{SG2537, &UA24246},
	{SG2568, &UA15044},
	{SG2569, &UA555183},
	{SG2571, &UA90558},
	{SG2591, &UA433143},
	{SG2593, &UA728592},
	{SG2643, &UA16968},
	{SG2644, &UA2018901},
	{SG2695, &UA2019901},
	{SG2758, &UA2018414},
	{SG2759, &UA8683},
	{SG2760, &UA8685},
	{SG2761, &UA10329},
	{SG2762, &UA63929},
	{SG2763, &UA399667},
	{SG2765, &UA163828},
	{SG2766, &UA1864615},
	{SG2797, &UA12460},
	{SG2798, &UA33281},
	{SG2799, &UA294938},
	{SG2811, &UA62527},
	{SG2812, &UA2018137},
	{SG2819, &UA53716},
	{SG2820, &UA90519},
	{SG2821, &UA188370},
	{SG2825, &UA49630},
	{SG2854, &UA195622},
	{SG2855, &UA330448},
	{SG2856, &UA370812},
	{SG2857, &UA558348},
	{SG2858, &UA562539},
	{SG2859, &UA2020895},
	{SG2860, &UA2028694},
	{SG2877, &UA53539},
	{SG2878, &UA421774},
	{SG2879, &UA2019339},
	{SG2886, &UA558055},
	{SG2887, &UA173379},
	{SG2888, &UA394752},
	{SG2890, &UA165853},
	{SG2891, &UA508386},
	{SG2892, &UA2019133},
	{SG2893, &UA2948651},
	{SG2906, &UA182021},
	{SG2915, &UA308745},
	{SG2930, &UA67272},
	{SG2931, &UA790206},
	{SG2940, &UA67567},
	{SG2941, &UA2018186},
	{SG2957, &UA249279},
	{SG2958, &UA2023231},
	{SG2975, &UA67304},
	{SG2976, &UA2018803},
	{SG2990, &UA90400},
	{SG2991, &UA677278},
	{SG3009, &UA67410},
	{SG3010, &UA537456},
	{SG3011, &UA32767},
	{SG3019, &UA40082},
	{SG3021, &UA373413},
	{SG3028, &UA167852},
	{SG3029, &UA2019074},
	{SG3032, &UA575233},
	{SG3047, &UA53544},
	{SG3048, &UA58524},
	{SG3049, &UA90534},
	{SG3053, &UA65425},
	{SG3060, &UA67353},
	{SG3061, &UA439969},
	{SG3062, &UA53980},
	{SG3063, &UA1934515},
	{SG3064, &UA2496390},
	{SG3074, &UA10137},
	{SG3075, &UA2018524},
	{SG3076, &UA523332},
	{SG3087, &UA301932},
	{SG3124, &UA309538},
	{SG3140, &UA2659309},
	{SG3143, &UA205035},
	{SG3144, &UA851755},
	{SG3165, &UA358983},
	{SG3166, &UA394673},
	{SG3167, &UA531884},
	{SG3168, &UA2022337},
	{SG3170, &UA119661},
	{SG3184, &UA67481},
	{SG3185, &UA1880484},
	{SG3186, &UA2018426},
	{SG3188, &UA723013},
	{SG3191, &UA62674},
	{SG3192, &UA67654},
	{SG3203, &UA10141},
	{SG3204, &UA60788},
	{SG3205, &UA2203463},
	{SG3230, &UA54473},
	{SG3238, &UA638738},
	{SG3240, &UA779258},
	{SG3241, &UA2019514},
	{SG3243, &UA2025846},
	{SG3248, &UA682245},
	{SG3258, &UA67763},
	{SG3267, &UA11317},
	{SG3275, &UA23015},
	{SG3281, &UA1985819},
	{SG3303, &UA704531},
	{SG3315, &UA48467},
	{SG3316, &UA61468},
	{SG3318, &UA1899880},
	{SG3319, &UA2024011},
	{SG3320, &UA2579968},
	{SG3321, &UA3921},
	{SG3322, &UA2019124},
	{SG3323, &UA32968},
	{SG3331, &UA49682},
	{SG3336, &UA183739},
	{SG3337, &UA2019146},
	{SG3349, &UA793880},
	{SG3353, &UA183721},
	{SG3365, &UA2232940},
	{SG3366, &UA2621231},
	{SG3367, &UA2621439},
	{SG3368, &UA2623077},
	{SG3388, &UA2469280},
	{SG3389, &UA2622600},
	{SG3390, &UA2624282},
	{SG3394, &UA700847},
	{SG3417, &UA2530360},
	{SG3418, &UA2634820},
	{SG3424, &UA420959},
	{SG3425, &UA687308},
	{SG3427, &UA38497},
	{SG3428, &UA560581},
	{SG3429, &UA2616051},
	{SG3430, &UA516563},
	{SG3438, &UA543933},
	{SG3459, &UA855228},
	{SG3460, &UA458989},
	{SG3461, &UA459120},
	{SG3463, &UA646549},
	{SG3479, &UA5470},
	{SG3480, &UA5472},
	{SG3483, &UA363320},
	{SG3486, &UA663680},
	{SG3491, &UA1902455},
	{SG3492, &UA1951001},
	{SG3493, &UA2018371},
	{SG3494, &UA2018541},
	{SG3495, &UA2018802},
	{SG3497, &UA2022223},
	{SG3499, &UA567845},
	{SG3508, &UA274997},
	{SG3509, &UA6486},
	{SG3510, &UA32180},
	{SG3511, &UA32182},
	{SG3512, &UA32188},
	{SG3521, &UA1897224},
	{SG3525, &UA2800231},
	{SG3547, &UA1987946},
	{SG3552, &UA2018742},
	{SG3554, &UA90777},
	{SG3555, &UA258516},
	{SG3556, &UA258519},
	{SG3557, &UA2023496},
	{SG3560, &UA331480},
	{SG3577, &UA393297},
	{SG3578, &UA555817},
	{SG3579, &UA2019192},
	{SG3581, &UA2019263},
	{SG3583, &UA2257121},
	{SG3589, &UA32786},
	{SG3621, &UA3062904},
	{SG3623, &UA67518},
	{SG3628, &UA295314},
	{SG3642, &UA761483},
	{SG3650, &UA2616213},
	{SG3653, &UA2624249},
	{SG3655, &UA2626098},
	{SG3657, &UA2631654},
	{SG3668, &UA40290},
	{SG3669, &UA2020405},
	{SG3685, &UA2481006},
	{SG3700, &UA458739},
	{SG3701, &UA458973},
	{SG3702, &UA857613},
	{SG3707, &UA2737532},
	{SG3708, &UA173462},
	{SG3711, &UA52322},
	{SG3712, &UA2020374},
	{SG3723, &UA10138},
	{SG3726, &UA70915},
	{SG3728, &UA2020708},
	{SG3732, &UA2660089},
	{SG3739, &UA10143},
	{SG3745, &UA458843},
	{SG3754, &UA2792314},
	{SG3755, &UA6616},
	{SG3756, &UA23337},
	{SG3761, &UA2661578},
	{SG3775, &UA2384337},
	{SG3777, &UA53538},
	{SG3790, &UA1890267},
	{SG3792, &UA2618468},
	{SG3793, &UA2623874},
	{SG3794, &UA2625778},
	{SG3798, &UA2944807},
	{SG3812, &UA90077},
	{SG3813, &UA2409818},
	{SG3823, &UA759041},
	{SG3858, &UA2382626},
	{SG3859, &UA2525485},
	{SG3860, &UA2617052},
	{SG3861, &UA2617328},
	{SG3862, &UA2618034},
	{SG3863, &UA2623516},
	{SG3865, &UA2351741},
	{SG3876, &UA2018307},
	{SG3877, &UA718376},
	{SG3892, &UA3124356},
	{SG3896, &UA2020729},
	{SG3905, &UA2026892},
	{SG3930, &UA2609583},
	{SG3931, &UA2634320},
	{SG3937, &UA2999880},
	{SG3938, &UA3056824},
	{SG3949, &UA2581047},
	{SG3954, &UA59230},
	{SG3955, &UA2020064},
	{SG3962, &UA2618708},
	{SG3963, &UA2623260},
	{SG3986, &UA370934},
	{SG3987, &UA2020005},
	{SG4005, &UA1903890},
	{SG4006, &UA2025375},
	{SG4019, &UA2616101},
	{SG4020, &UA2662510},
	{SG4026, &UA2019462},
	{SG4032, &UA2577804},
	{SG4035, &UA2023044},
	{SG4039, &UA2794486},
	{SG4051, &UA1905032},
	{SG4060, &UA2938042},
	{SG4082, &UA2630805},
	{SG4083, &UA2750415},
	{SG4089, &UA2543723},
	{SG4100, &UA3113241},
	{SG4109, &UA2636674},
	{SG4114, &UA3046606},
	{SG4127, &UA2731888},
	{SG4132, &UA3015313},
	{SG4140, &UA33379},
	{SG4141, &UA49209},
	{SG4142, &UA1934100},
	{SG4145, &UA2747051},
	{SG4155, &UA3049518},
	{SG4161, &UA3106921},
	{SG4175, &UA2611876},
	{SG4177, &UA2620263},
	{SG4178, &UA2620315},
	{SG4182, &UA2729188},
	{SG4200, &UA3076772},
	{SG4213, &UA2454862},
	{SG4216, &UA2837178},
	{SG4218, &UA2853264},
	{SG4244, &UA2986268},
	{SG4256, &UA369838},
	{SG4257, &UA701392},
	{SG4268, &UA2029029},
	{SG4269, &UA2632517},
	{SG4286, &UA2619058},
	{SG4287, &UA2624442},
	{SG4288, &UA2625617},
	{SG4292, &UA2775112},
	{SG4295, &UA2610580},
	{SG4296, &UA2618807},
	{SG4305, &UA10139},
	{SG4314, &UA2627310},
	{SG4315, &UA2627862},
	{SG4328, &UA2019713},
	{SG4329, &UA2020678},
	{SG4330, &UA2827414},
	{SG4331, &UA2021456},
	{SG4342, &UA370129},
	{SG4378, &UA2020509},
	{SG4391, &UA459174},
	{SG4394, &UA780113},
	{SG4400, &UA2023939},
	{SG4418, &UA2624420},
	{SG4424, &UA1950441},
	{SG4425, &UA2019139},
	{SG4442, &UA21217},
	{SG4444, &UA78253},
	{SG4445, &UA2622153},
	{SG4446, &UA368805},
	{SG4447, &UA1901863},
	{SG4448, &UA1967999},
	{SG4449, &UA2022424},
	{SG4469, &UA2026336},
	{SG4470, &UA2618038},
	{SG4471, &UA2620943},
	{SG4485, &UA2031554},
	{SG4486, &UA2028500},
	{SG4487, &UA2615830},
	{SG4489, &UA2612789},
	{SG4495, &UA40072},
	{SG4496, &UA565320},
	{SG4516, &UA2029745},
	{SG4531, &UA2025900},
	{SG4532, &UA2395960},
	{SG4557, &UA21432},
	{SG4568, &UA2607801},
	{SG4571, &UA2611229},
	{SG4608, &UA47993},
	{SG4614, &UA2631330},
	{SG4615, &UA2611112},
	{SG4616, &UA2613922},
	{SG4619, &UA2628541},
	{SG4626, &UA67318},
	{SG4627, &UA71968},
	{SG4628, &UA369159},
	{SG4631, &UA2019615},
	{SG4632, &UA2021227},
	{SG4633, &UA2028344},
	{SG4634, &UA2241470},
	{SG4637, &UA2607647},
	{SG4639, &UA2613194},
	{SG4673, &UA37837},
	{SG4674, &UA77882},
	{SG4675, &UA369122},
	{SG4676, &UA369830},
	{SG4678, &UA2021541},
	{SG4679, &UA2023809},
	{SG4681, &UA2607935},
	{SG4685, &UA2618173},
	{SG4686, &UA2618892},
	{SG4687, &UA2634045},
	{NULL}};
const Segmentdata DAndroidVersion2_1[] = {
	{SG66, &UA16772},
	{SG67, &UA19812},
	{SG68, &UA42523},
	{SG69, &UA43920},
	{SG70, &UA43944},
	{SG71, &UA58417},
	{SG72, &UA58430},
	{SG73, &UA58939},
	{SG74, &UA70022},
	{SG75, &UA71849},
	{SG76, &UA76427},
	{SG77, &UA172400},
	{SG78, &UA173592},
	{SG79, &UA179836},
	{SG80, &UA793846},
	{SG81, &UA1897011},
	{SG82, &UA1906633},
	{SG83, &UA2020850},
	{SG84, &UA2023292},
	{SG85, &UA2028740},
	{SG86, &UA2684156},
	{SG87, &UA2720655},
	{SG601, &UA9720},
	{SG602, &UA9786},
	{SG603, &UA19491},
	{SG604, &UA21994},
	{SG605, &UA58138},
	{SG606, &UA58159},
	{SG607, &UA58190},
	{SG608, &UA70212},
	{SG609, &UA189798},
	{SG610, &UA719964},
	{SG611, &UA2022566},
	{SG639, &UA9590},
	{SG640, &UA9601},
	{SG641, &UA9644},
	{SG642, &UA42085},
	{SG643, &UA43766},
	{SG644, &UA44146},
	{SG645, &UA57980},
	{SG646, &UA58048},
	{SG647, &UA58076},
	{SG648, &UA63076},
	{SG649, &UA174261},
	{SG650, &UA2020319},
	{SG651, &UA2021105},
	{SG652, &UA2024758},
	{SG653, &UA2026777},
	{SG654, &UA2032576},
	{SG674, &UA4894},
	{SG676, &UA7473},
	{SG678, &UA14866},
	{SG680, &UA50457},
	{SG681, &UA54215},
	{SG682, &UA56627},
	{SG683, &UA138502},
	{SG685, &UA505776},
	{SG686, &UA2022095},
	{SG688, &UA2022461},
	{SG689, &UA2029630},
	{SG690, &UA2031328},
	{SG784, &UA19660},
	{SG785, &UA30312},
	{SG786, &UA34196},
	{SG787, &UA40591},
	{SG788, &UA42388},
	{SG789, &UA42417},
	{SG790, &UA58277},
	{SG791, &UA58283},
	{SG792, &UA77356},
	{SG793, &UA189229},
	{SG794, &UA769274},
	{SG795, &UA1902318},
	{SG796, &UA2025092},
	{SG891, &UA7903},
	{SG892, &UA7911},
	{SG893, &UA7965},
	{SG894, &UA7966},
	{SG895, &UA7977},
	{SG896, &UA7997},
	{SG897, &UA8012},
	{SG898, &UA8016},
	{SG899, &UA9530},
	{SG900, &UA10471},
	{SG901, &UA10512},
	{SG902, &UA10532},
	{SG903, &UA10484},
	{SG904, &UA10515},
	{SG905, &UA10579},
	{SG906, &UA11637},
	{SG912, &UA12798},
	{SG913, &UA12803},
	{SG915, &UA12806},
	{SG916, &UA13568},
	{SG917, &UA13572},
	{SG918, &UA18687},
	{SG919, &UA26828},
	{SG920, &UA26832},
	{SG921, &UA44277},
	{SG922, &UA52132},
	{SG923, &UA52147},
	{SG924, &UA52644},
	{SG925, &UA52726},
	{SG926, &UA54240},
	{SG927, &UA62990},
	{SG928, &UA62991},
	{SG929, &UA90093},
	{SG930, &UA636036},
	{SG931, &UA2023920},
	{SG990, &UA6515},
	{SG991, &UA32202},
	{SG992, &UA32213},
	{SG993, &UA32216},
	{SG994, &UA32226},
	{SG995, &UA32228},
	{SG996, &UA62489},
	{SG997, &UA71922},
	{SG998, &UA89977},
	{SG999, &UA146796},
	{SG1000, &UA152845},
	{SG1001, &UA2018858},
	{SG1069, &UA11502},
	{SG1070, &UA26635},
	{SG1071, &UA39977},
	{SG1072, &UA53519},
	{SG1073, &UA62383},
	{SG1074, &UA79300},
	{SG1075, &UA162766},
	{SG1076, &UA165234},
	{SG1077, &UA2018266},
	{SG1078, &UA12693},
	{SG1079, &UA12697},
	{SG1080, &UA12702},
	{SG1081, &UA28165},
	{SG1082, &UA28170},
	{SG1083, &UA28244},
	{SG1084, &UA29107},
	{SG1085, &UA29803},
	{SG1086, &UA29807},
	{SG1087, &UA29808},
	{SG1088, &UA29809},
	{SG1089, &UA29813},
	{SG1090, &UA29848},
	{SG1091, &UA31259},
	{SG1092, &UA32114},
	{SG1093, &UA32118},
	{SG1094, &UA32119},
	{SG1095, &UA32120},
	{SG1096, &UA32128},
	{SG1097, &UA37101},
	{SG1098, &UA45692},
	{SG1099, &UA54005},
	{SG1100, &UA64323},
	{SG1101, &UA66546},
	{SG1102, &UA66549},
	{SG1103, &UA66554},
	{SG1104, &UA66544},
	{SG1105, &UA93547},
	{SG1106, &UA2018717},
	{SG1107, &UA2025294},
	{SG1158, &UA4},
	{SG1159, &UA11},
	{SG1160, &UA14},
	{SG1161, &UA7068},
	{SG1162, &UA7084},
	{SG1163, &UA32527},
	{SG1164, &UA32722},
	{SG1165, &UA32723},
	{SG1166, &UA32727},
	{SG1167, &UA32728},
	{SG1168, &UA32729},
	{SG1169, &UA37867},
	{SG1170, &UA37870},
	{SG1171, &UA51306},
	{SG1172, &UA76912},
	{SG1173, &UA90021},
	{SG1174, &UA157340},
	{SG1179, &UA52064},
	{SG1180, &UA53769},
	{SG1181, &UA67311},
	{SG1182, &UA67332},
	{SG1183, &UA67809},
	{SG1184, &UA164840},
	{SG1185, &UA566387},
	{SG1186, &UA648674},
	{SG1187, &UA749854},
	{SG1245, &UA19374},
	{SG1246, &UA26333},
	{SG1247, &UA42162},
	{SG1248, &UA42178},
	{SG1249, &UA53512},
	{SG1250, &UA54545},
	{SG1251, &UA58633},
	{SG1252, &UA705813},
	{SG1263, &UA90066},
	{SG1264, &UA132392},
	{SG1265, &UA139982},
	{SG1266, &UA152149},
	{SG1267, &UA418173},
	{SG1268, &UA538765},
	{SG1269, &UA1879885},
	{SG1270, &UA2018421},
	{SG1271, &UA2018532},
	{SG1324, &UA3109},
	{SG1325, &UA3114},
	{SG1326, &UA3175},
	{SG1327, &UA3200},
	{SG1328, &UA3235},
	{SG1329, &UA30895},
	{SG1330, &UA64017},
	{SG1331, &UA65403},
	{SG1332, &UA66115},
	{SG1333, &UA369924},
	{SG1334, &UA782721},
	{SG1335, &UA821570},
	{SG1336, &UA2023488},
	{SG1365, &UA10297},
	{SG1366, &UA156929},
	{SG1367, &UA2030714},
	{SG1391, &UA10626},
	{SG1392, &UA10627},
	{SG1393, &UA10633},
	{SG1394, &UA10635},
	{SG1395, &UA10640},
	{SG1396, &UA10653},
	{SG1397, &UA10654},
	{SG1398, &UA10655},
	{SG1399, &UA10656},
	{SG1400, &UA10659},
	{SG1401, &UA11849},
	{SG1402, &UA14482},
	{SG1403, &UA14483},
	{SG1404, &UA14489},
	{SG1405, &UA14490},
	{SG1406, &UA14492},
	{SG1407, &UA14494},
	{SG1408, &UA14498},
	{SG1409, &UA14499},
	{SG1410, &UA14504},
	{SG1411, &UA14512},
	{SG1412, &UA14513},
	{SG1413, &UA14519},
	{SG1414, &UA14520},
	{SG1415, &UA14522},
	{SG1416, &UA30151},
	{SG1417, &UA30152},
	{SG1418, &UA52841},
	{SG1419, &UA52850},
	{SG1420, &UA79024},
	{SG1421, &UA89769},
	{SG1422, &UA786602},
	{SG1432, &UA11301},
	{SG1433, &UA14562},
	{SG1434, &UA20595},
	{SG1435, &UA39512},
	{SG1436, &UA39997},
	{SG1437, &UA43003},
	{SG1438, &UA44125},
	{SG1439, &UA44130},
	{SG1440, &UA51686},
	{SG1442, &UA52883},
	{SG1444, &UA64651},
	{SG1446, &UA67974},
	{SG1448, &UA153867},
	{SG1450, &UA1912638},
	{SG1452, &UA2932933},
	{SG1456, &UA46895},
	{SG1457, &UA46904},
	{SG1458, &UA60024},
	{SG1459, &UA369197},
	{SG1460, &UA564423},
	{SG1461, &UA575223},
	{SG1462, &UA2021000},
	{SG1493, &UA7915},
	{SG1494, &UA7980},
	{SG1495, &UA8002},
	{SG1496, &UA8010},
	{SG1497, &UA8023},
	{SG1498, &UA8025},
	{SG1499, &UA7999},
	{SG1500, &UA8326},
	{SG1501, &UA9497},
	{SG1502, &UA9511},
	{SG1503, &UA9514},
	{SG1504, &UA9525},
	{SG1505, &UA9526},
	{SG1506, &UA11640},
	{SG1507, &UA18107},
	{SG1508, &UA26830},
	{SG1509, &UA38397},
	{SG1510, &UA41575},
	{SG1511, &UA52124},
	{SG1512, &UA52129},
	{SG1513, &UA52143},
	{SG1514, &UA53729},
	{SG1515, &UA53730},
	{SG1516, &UA53731},
	{SG1517, &UA53732},
	{SG1518, &UA57291},
	{SG1519, &UA370973},
	{SG1520, &UA551657},
	{SG1521, &UA2020653},
	{SG1532, &UA67003},
	{SG1533, &UA160461},
	{SG1534, &UA234722},
	{SG1535, &UA2019455},
	{SG1632, &UA14514},
	{SG1633, &UA20191},
	{SG1634, &UA24296},
	{SG1635, &UA40938},
	{SG1636, &UA44063},
	{SG1637, &UA53707},
	{SG1638, &UA53706},
	{SG1639, &UA57269},
	{SG1640, &UA67880},
	{SG1641, &UA68641},
	{SG1642, &UA165754},
	{SG1643, &UA166497},
	{SG1644, &UA298381},
	{SG1646, &UA298575},
	{SG1648, &UA299375},
	{SG1649, &UA299753},
	{SG1650, &UA312190},
	{SG1651, &UA354459},
	{SG1652, &UA354464},
	{SG1653, &UA355512},
	{SG1654, &UA370559},
	{SG1655, &UA385275},
	{SG1656, &UA387350},
	{SG1657, &UA390586},
	{SG1658, &UA393508},
	{SG1659, &UA417962},
	{SG1660, &UA432922},
	{SG1661, &UA434926},
	{SG1662, &UA446937},
	{SG1663, &UA503730},
	{SG1665, &UA525243},
	{SG1667, &UA535926},
	{SG1669, &UA540897},
	{SG1672, &UA543116},
	{SG1674, &UA553209},
	{SG1675, &UA559046},
	{SG1676, &UA566770},
	{SG1677, &UA632313},
	{SG1678, &UA654208},
	{SG1679, &UA658992},
	{SG1680, &UA677415},
	{SG1681, &UA681549},
	{SG1682, &UA690213},
	{SG1683, &UA691752},
	{SG1684, &UA692418},
	{SG1685, &UA704295},
	{SG1686, &UA713198},
	{SG1687, &UA713358},
	{SG1688, &UA716386},
	{SG1689, &UA716701},
	{SG1690, &UA721811},
	{SG1691, &UA726463},
	{SG1692, &UA750311},
	{SG1693, &UA762678},
	{SG1694, &UA768067},
	{SG1695, &UA768387},
	{SG1696, &UA770193},
	{SG1697, &UA791948},
	{SG1698, &UA801551},
	{SG1699, &UA811329},
	{SG1700, &UA880292},
	{SG1701, &UA1858705},
	{SG1702, &UA1859564},
	{SG1703, &UA1865233},
	{SG1704, &UA1882183},
	{SG1705, &UA1886246},
	{SG1706, &UA1894130},
	{SG1707, &UA1894806},
	{SG1708, &UA1895544},
	{SG1709, &UA1900441},
	{SG1710, &UA1901089},
	{SG1711, &UA1920412},
	{SG1712, &UA1922235},
	{SG1713, &UA1938729},
	{SG1714, &UA1941477},
	{SG1715, &UA1967010},
	{SG1716, &UA2018100},
	{SG1717, &UA2018116},
	{SG1718, &UA2018218},
	{SG1719, &UA2018223},
	{SG1720, &UA2018306},
	{SG1721, &UA2018469},
	{SG1722, &UA2018604},
	{SG1723, &UA2018646},
	{SG1724, &UA2018740},
	{SG1725, &UA2018864},
	{SG1726, &UA2018954},
	{SG1727, &UA2018974},
	{SG1728, &UA2019002},
	{SG1729, &UA2019012},
	{SG1730, &UA2019078},
	{SG1731, &UA2019101},
	{SG1732, &UA2019156},
	{SG1733, &UA2019180},
	{SG1734, &UA2019199},
	{SG1735, &UA2019235},
	{SG1736, &UA2019215},
	{SG1737, &UA2019242},
	{SG1738, &UA2019274},
	{SG1739, &UA2019365},
	{SG1740, &UA2019484},
	{SG1741, &UA2020112},
	{SG1742, &UA2020222},
	{SG1743, &UA2020275},
	{SG1744, &UA2020338},
	{SG1745, &UA2020552},
	{SG1746, &UA2020961},
	{SG1747, &UA2021445},
	{SG1748, &UA2021503},
	{SG1749, &UA2021885},
	{SG1750, &UA2021964},
	{SG1752, &UA2022808},
	{SG1754, &UA2022898},
	{SG1756, &UA2023107},
	{SG1758, &UA2023149},
	{SG1760, &UA2024089},
	{SG1761, &UA2024360},
	{SG1763, &UA2028299},
	{SG1764, &UA2031965},
	{SG1765, &UA2290580},
	{SG1766, &UA2311563},
	{SG1767, &UA2416472},
	{SG1768, &UA2475516},
	{SG1769, &UA2518713},
	{SG1770, &UA2641172},
	{SG1771, &UA2659144},
	{SG1772, &UA2752801},
	{SG1773, &UA2808007},
	{SG1809, &UA156},
	{SG1810, &UA165},
	{SG1811, &UA167},
	{SG1812, &UA169},
	{SG1813, &UA177},
	{SG1814, &UA178},
	{SG1815, &UA179},
	{SG1816, &UA180},
	{SG1817, &UA3485},
	{SG1818, &UA7218},
	{SG1819, &UA26366},
	{SG1820, &UA37963},
	{SG1821, &UA51425},
	{SG1822, &UA54078},
	{SG1823, &UA54080},
	{SG1824, &UA54081},
	{SG1825, &UA383655},
	{SG1826, &UA417323},
	{SG1827, &UA492151},
	{SG1828, &UA492727},
	{SG1829, &UA1864225},
	{SG1830, &UA2018067},
	{SG1831, &UA2018147},
	{SG1832, &UA2018167},
	{SG1839, &UA89936},
	{SG1840, &UA894020},
	{SG1868, &UA43866},
	{SG1869, &UA58931},
	{SG1870, &UA63777},
	{SG1872, &UA368632},
	{SG1873, &UA368974},
	{SG1875, &UA396527},
	{SG1877, &UA409583},
	{SG1879, &UA497520},
	{SG1881, &UA643323},
	{SG1882, &UA2018598},
	{SG1884, &UA2019830},
	{SG1886, &UA2261699},
	{SG1887, &UA2476167},
	{SG1904, &UA50220},
	{SG1905, &UA50264},
	{SG1906, &UA50322},
	{SG1907, &UA50380},
	{SG1908, &UA50381},
	{SG1909, &UA53560},
	{SG1910, &UA56062},
	{SG1911, &UA56122},
	{SG1912, &UA56320},
	{SG1913, &UA56800},
	{SG1914, &UA67253},
	{SG1915, &UA67415},
	{SG1916, &UA67760},
	{SG1917, &UA478283},
	{SG1918, &UA683804},
	{SG1919, &UA767721},
	{SG1920, &UA819619},
	{SG1921, &UA1844413},
	{SG1922, &UA1861932},
	{SG1923, &UA2018339},
	{SG1924, &UA2020939},
	{SG1925, &UA50281},
	{SG1926, &UA50518},
	{SG1927, &UA53573},
	{SG1928, &UA61321},
	{SG1929, &UA67824},
	{SG1930, &UA133332},
	{SG1931, &UA507843},
	{SG1932, &UA2018254},
	{SG1933, &UA2019530},
	{SG1935, &UA30763},
	{SG1936, &UA30764},
	{SG1937, &UA30769},
	{SG1938, &UA30777},
	{SG1939, &UA30780},
	{SG1940, &UA63406},
	{SG1941, &UA89773},
	{SG1958, &UA7003},
	{SG1959, &UA62808},
	{SG1960, &UA49035},
	{SG1961, &UA49042},
	{SG1962, &UA56847},
	{SG1963, &UA58648},
	{SG1964, &UA722358},
	{SG1965, &UA53554},
	{SG1966, &UA53798},
	{SG1967, &UA634400},
	{SG1968, &UA699357},
	{SG1969, &UA2019095},
	{SG1970, &UA2393013},
	{SG1971, &UA52036},
	{SG1972, &UA55979},
	{SG1973, &UA55983},
	{SG1974, &UA56796},
	{SG1975, &UA67442},
	{SG1976, &UA90308},
	{SG1977, &UA2019314},
	{SG1978, &UA2020132},
	{SG2006, &UA170725},
	{SG2007, &UA377386},
	{SG2008, &UA90429},
	{SG2009, &UA173718},
	{SG2010, &UA545406},
	{SG2012, &UA2018640},
	{SG2014, &UA2018738},
	{SG2015, &UA2019191},
	{SG2016, &UA2019453},
	{SG2019, &UA31980},
	{SG2020, &UA32032},
	{SG2021, &UA50908},
	{SG2022, &UA167156},
	{SG2066, &UA14842},
	{SG2067, &UA1871543},
	{SG2068, &UA2018667},
	{SG2081, &UA31014},
	{SG2082, &UA447404},
	{SG2083, &UA544120},
	{SG2084, &UA652193},
	{SG2085, &UA810638},
	{SG2086, &UA816846},
	{SG2087, &UA1864663},
	{SG2088, &UA13242},
	{SG2089, &UA25966},
	{SG2090, &UA30342},
	{SG2091, &UA44022},
	{SG2106, &UA19716},
	{SG2107, &UA44256},
	{SG2108, &UA44261},
	{SG2109, &UA169743},
	{SG2110, &UA2019793},
	{SG2114, &UA8914},
	{SG2115, &UA538008},
	{SG2116, &UA705173},
	{SG2131, &UA38532},
	{SG2132, &UA500676},
	{SG2133, &UA7987},
	{SG2134, &UA18060},
	{SG2135, &UA18058},
	{SG2136, &UA33225},
	{SG2137, &UA38361},
	{SG2138, &UA41525},
	{SG2139, &UA41526},
	{SG2140, &UA53645},
	{SG2141, &UA53644},
	{SG2142, &UA53646},
	{SG2143, &UA53643},
	{SG2144, &UA53642},
	{SG2145, &UA55268},
	{SG2146, &UA78721},
	{SG2147, &UA90094},
	{SG2148, &UA1892780},
	{SG2149, &UA2022711},
	{SG2150, &UA2029744},
	{SG2151, &UA50508},
	{SG2152, &UA142975},
	{SG2153, &UA655950},
	{SG2154, &UA2496875},
	{SG2164, &UA7806},
	{SG2165, &UA9416},
	{SG2166, &UA26397},
	{SG2167, &UA61976},
	{SG2168, &UA659773},
	{SG2169, &UA63265},
	{SG2170, &UA89992},
	{SG2171, &UA431264},
	{SG2172, &UA9013},
	{SG2173, &UA50157},
	{SG2174, &UA50213},
	{SG2175, &UA67923},
	{SG2176, &UA89187},
	{SG2177, &UA368636},
	{SG2178, &UA368707},
	{SG2179, &UA1951973},
	{SG2180, &UA2018068},
	{SG2181, &UA2658394},
	{SG2182, &UA2658595},
	{SG2183, &UA3058545},
	{SG2184, &UA90201},
	{SG2185, &UA90451},
	{SG2186, &UA140365},
	{SG2187, &UA553356},
	{SG2188, &UA2019204},
	{SG2201, &UA4300},
	{SG2202, &UA4304},
	{SG2203, &UA24905},
	{SG2204, &UA24922},
	{SG2205, &UA24995},
	{SG2206, &UA24998},
	{SG2207, &UA25223},
	{SG2208, &UA25235},
	{SG2209, &UA25237},
	{SG2210, &UA25238},
	{SG2211, &UA25244},
	{SG2212, &UA25246},
	{SG2213, &UA25253},
	{SG2214, &UA25316},
	{SG2215, &UA25320},
	{SG2216, &UA25324},
	{SG2217, &UA26795},
	{SG2218, &UA36594},
	{SG2219, &UA388185},
	{SG2220, &UA659820},
	{SG2221, &UA679827},
	{SG2222, &UA2021452},
	{SG2233, &UA9172},
	{SG2234, &UA9178},
	{SG2235, &UA9179},
	{SG2236, &UA9183},
	{SG2237, &UA9185},
	{SG2238, &UA9187},
	{SG2239, &UA9199},
	{SG2240, &UA9200},
	{SG2241, &UA13080},
	{SG2242, &UA38267},
	{SG2243, &UA51925},
	{SG2244, &UA52756},
	{SG2245, &UA62888},
	{SG2246, &UA2018464},
	{SG2280, &UA38571},
	{SG2281, &UA38572},
	{SG2282, &UA555556},
	{SG2306, &UA43723},
	{SG2353, &UA12894},
	{SG2354, &UA12898},
	{SG2355, &UA34378},
	{SG2356, &UA34379},
	{SG2357, &UA40644},
	{SG2358, &UA55967},
	{SG2359, &UA1865299},
	{SG2382, &UA108996},
	{SG2383, &UA174720},
	{SG2384, &UA268575},
	{SG2385, &UA275589},
	{SG2386, &UA445559},
	{SG2387, &UA2018146},
	{SG2388, &UA43586},
	{SG2389, &UA43593},
	{SG2390, &UA627045},
	{SG2391, &UA667243},
	{SG2392, &UA2020445},
	{SG2393, &UA2385729},
	{SG2394, &UA3070543},
	{SG2395, &UA9138},
	{SG2396, &UA38283},
	{SG2397, &UA541899},
	{SG2403, &UA6610},
	{SG2404, &UA32249},
	{SG2405, &UA32251},
	{SG2406, &UA32265},
	{SG2407, &UA32270},
	{SG2408, &UA32271},
	{SG2409, &UA32272},
	{SG2410, &UA32296},
	{SG2411, &UA32313},
	{SG2412, &UA32317},
	{SG2413, &UA32391},
	{SG2414, &UA37780},
	{SG2416, &UA51173},
	{SG2418, &UA89840},
	{SG2419, &UA132858},
	{SG2421, &UA157732},
	{SG2423, &UA2021637},
	{SG2425, &UA51971},
	{SG2426, &UA1872253},
	{SG2429, &UA7088},
	{SG2430, &UA7092},
	{SG2431, &UA27632},
	{SG2432, &UA27638},
	{SG2433, &UA54947},
	{SG2434, &UA65915},
	{SG2435, &UA65916},
	{SG2436, &UA65917},
	{SG2437, &UA65921},
	{SG2438, &UA71211},
	{SG2439, &UA423653},
	{SG2440, &UA550928},
	{SG2469, &UA158270},
	{SG2470, &UA172571},
	{SG2471, &UA458645},
	{SG2472, &UA458646},
	{SG2473, &UA458681},
	{SG2474, &UA458873},
	{SG2475, &UA633210},
	{SG2495, &UA23698},
	{SG2496, &UA23699},
	{SG2497, &UA62549},
	{SG2498, &UA482188},
	{SG2525, &UA401110},
	{SG2560, &UA232607},
	{SG2561, &UA690353},
	{SG2562, &UA1898812},
	{SG2594, &UA30846},
	{SG2595, &UA31521},
	{SG2596, &UA31522},
	{SG2597, &UA62411},
	{SG2598, &UA285325},
	{SG2599, &UA561899},
	{SG2600, &UA2018565},
	{SG2609, &UA163726},
	{SG2610, &UA228258},
	{SG2611, &UA242211},
	{SG2612, &UA637319},
	{SG2613, &UA51081},
	{SG2614, &UA21300},
	{SG2615, &UA26214},
	{SG2616, &UA40855},
	{SG2618, &UA42511},
	{SG2619, &UA44231},
	{SG2620, &UA56045},
	{SG2621, &UA56779},
	{SG2622, &UA67274},
	{SG2623, &UA67428},
	{SG2624, &UA67556},
	{SG2625, &UA290439},
	{SG2626, &UA301441},
	{SG2627, &UA751344},
	{SG2628, &UA770126},
	{SG2629, &UA1987520},
	{SG2630, &UA2018210},
	{SG2631, &UA2018850},
	{SG2632, &UA2019990},
	{SG2633, &UA2021074},
	{SG2634, &UA2021550},
	{SG2635, &UA2392644},
	{SG2636, &UA44140},
	{SG2637, &UA2019579},
	{SG2650, &UA14362},
	{SG2651, &UA14363},
	{SG2652, &UA40788},
	{SG2653, &UA44344},
	{SG2654, &UA44345},
	{SG2655, &UA56649},
	{SG2656, &UA90423},
	{SG2657, &UA370263},
	{SG2668, &UA23251},
	{SG2669, &UA50208},
	{SG2670, &UA50272},
	{SG2671, &UA50396},
	{SG2672, &UA56102},
	{SG2673, &UA56333},
	{SG2674, &UA68424},
	{SG2675, &UA167680},
	{SG2678, &UA43862},
	{SG2679, &UA43863},
	{SG2680, &UA62625},
	{SG2685, &UA52754},
	{SG2686, &UA55200},
	{SG2687, &UA61735},
	{SG2688, &UA731309},
	{SG2729, &UA191879},
	{SG2730, &UA233757},
	{SG2731, &UA395837},
	{SG2732, &UA5443},
	{SG2733, &UA10194},
	{SG2734, &UA10195},
	{SG2735, &UA2018770},
	{SG2738, &UA67124},
	{SG2739, &UA67164},
	{SG2740, &UA1869583},
	{SG2741, &UA2018161},
	{SG2742, &UA2022683},
	{SG2743, &UA55140},
	{SG2744, &UA2018081},
	{SG2745, &UA6437},
	{SG2746, &UA13640},
	{SG2747, &UA444091},
	{SG2748, &UA21655},
	{SG2749, &UA76144},
	{SG2750, &UA2020202},
	{SG2752, &UA421611},
	{SG2753, &UA844783},
	{SG2754, &UA1966382},
	{SG2755, &UA20320},
	{SG2756, &UA42860},
	{SG2757, &UA2020262},
	{SG2784, &UA49485},
	{SG2785, &UA49484},
	{SG2786, &UA2022453},
	{SG2787, &UA50301},
	{SG2788, &UA89885},
	{SG2789, &UA90288},
	{SG2790, &UA90543},
	{SG2791, &UA169442},
	{SG2792, &UA200785},
	{SG2793, &UA250468},
	{SG2794, &UA632073},
	{SG2795, &UA2018531},
	{SG2803, &UA26393},
	{SG2804, &UA40048},
	{SG2805, &UA2018587},
	{SG2806, &UA12984},
	{SG2807, &UA53703},
	{SG2808, &UA53705},
	{SG2809, &UA2018408},
	{SG2810, &UA2019567},
	{SG2826, &UA7695},
	{SG2827, &UA40937},
	{SG2828, &UA56616},
	{SG2829, &UA2018409},
	{SG2830, &UA49014},
	{SG2831, &UA745566},
	{SG2832, &UA2019758},
	{SG2834, &UA48977},
	{SG2842, &UA25967},
	{SG2843, &UA34796},
	{SG2844, &UA66552},
	{SG2845, &UA89991},
	{SG2846, &UA113629},
	{SG2847, &UA54187},
	{SG2848, &UA56002},
	{SG2849, &UA2018708},
	{SG2883, &UA89949},
	{SG2895, &UA169584},
	{SG2896, &UA704148},
	{SG2903, &UA30765},
	{SG2904, &UA89842},
	{SG2905, &UA274553},
	{SG2914, &UA8290},
	{SG2916, &UA164294},
	{SG2918, &UA695611},
	{SG2920, &UA1891851},
	{SG2932, &UA631373},
	{SG2933, &UA695078},
	{SG2934, &UA2404816},
	{SG2935, &UA12692},
	{SG2936, &UA44023},
	{SG2937, &UA68044},
	{SG2938, &UA287608},
	{SG2939, &UA2022171},
	{SG2947, &UA42545},
	{SG2948, &UA627750},
	{SG2959, &UA144795},
	{SG2960, &UA492173},
	{SG2964, &UA44059},
	{SG2965, &UA78358},
	{SG2966, &UA2018122},
	{SG2977, &UA26627},
	{SG2978, &UA26629},
	{SG2999, &UA69027},
	{SG3000, &UA50519},
	{SG3001, &UA67328},
	{SG3002, &UA67427},
	{SG3003, &UA67536},
	{SG3004, &UA504802},
	{SG3005, &UA1862884},
	{SG3018, &UA837622},
	{SG3031, &UA43982},
	{SG3039, &UA3141},
	{SG3040, &UA50266},
	{SG3041, &UA90076},
	{SG3042, &UA2018064},
	{SG3043, &UA2018185},
	{SG3044, &UA2020880},
	{SG3050, &UA32187},
	{SG3051, &UA32718},
	{SG3052, &UA2019747},
	{SG3069, &UA63935},
	{SG3070, &UA2018076},
	{SG3093, &UA55802},
	{SG3094, &UA90544},
	{SG3095, &UA200575},
	{SG3096, &UA878103},
	{SG3111, &UA48971},
	{SG3112, &UA657518},
	{SG3113, &UA153490},
	{SG3133, &UA16675},
	{SG3134, &UA24223},
	{SG3135, &UA41196},
	{SG3136, &UA49131},
	{SG3137, &UA49134},
	{SG3138, &UA49133},
	{SG3139, &UA90550},
	{SG3152, &UA305146},
	{SG3153, &UA2018455},
	{SG3171, &UA693062},
	{SG3172, &UA2024795},
	{SG3173, &UA62169},
	{SG3174, &UA63834},
	{SG3182, &UA42064},
	{SG3183, &UA57266},
	{SG3189, &UA39872},
	{SG3190, &UA55138},
	{SG3194, &UA2384197},
	{SG3195, &UA2842039},
	{SG3198, &UA38511},
	{SG3199, &UA536925},
	{SG3200, &UA156118},
	{SG3201, &UA2018136},
	{SG3226, &UA54470},
	{SG3244, &UA68575},
	{SG3264, &UA149496},
	{SG3265, &UA2537988},
	{SG3266, &UA3015831},
	{SG3282, &UA5106},
	{SG3283, &UA5110},
	{SG3285, &UA151009},
	{SG3286, &UA640513},
	{SG3287, &UA647199},
	{SG3288, &UA680272},
	{SG3289, &UA703158},
	{SG3290, &UA1865434},
	{SG3291, &UA2018312},
	{SG3292, &UA2018465},
	{SG3294, &UA2019966},
	{SG3295, &UA2493839},
	{SG3296, &UA2685253},
	{SG3299, &UA675701},
	{SG3304, &UA78236},
	{SG3305, &UA379872},
	{SG3310, &UA54614},
	{SG3311, &UA503742},
	{SG3312, &UA48973},
	{SG3313, &UA54296},
	{SG3314, &UA58118},
	{SG3317, &UA49456},
	{SG3324, &UA64837},
	{SG3325, &UA557630},
	{SG3326, &UA2018140},
	{SG3329, &UA63302},
	{SG3330, &UA2018904},
	{SG3333, &UA30442},
	{SG3334, &UA30445},
	{SG3335, &UA2018595},
	{SG3343, &UA162255},
	{SG3344, &UA1905666},
	{SG3347, &UA2233035},
	{SG3355, &UA42381},
	{SG3356, &UA1906101},
	{SG3361, &UA2768788},
	{SG3362, &UA2799738},
	{SG3369, &UA43681},
	{SG3370, &UA857438},
	{SG3371, &UA1920634},
	{SG3372, &UA2742830},
	{SG3376, &UA2033140},
	{SG3384, &UA21324},
	{SG3385, &UA44392},
	{SG3386, &UA44396},
	{SG3387, &UA2469871},
	{SG3391, &UA3916},
	{SG3392, &UA128808},
	{SG3395, &UA281198},
	{SG3396, &UA298509},
	{SG3407, &UA2563379},
	{SG3408, &UA3021158},
	{SG3409, &UA42207},
	{SG3410, &UA1905295},
	{SG3411, &UA287008},
	{SG3412, &UA2018075},
	{SG3413, &UA2018735},
	{SG3414, &UA2022401},
	{SG3421, &UA432574},
	{SG3422, &UA495074},
	{SG3423, &UA2742346},
	{SG3433, &UA157477},
	{SG3439, &UA506285},
	{SG3440, &UA1855979},
	{SG3441, &UA2019096},
	{SG3442, &UA2019237},
	{SG3455, &UA51080},
	{SG3456, &UA90446},
	{SG3457, &UA2018315},
	{SG3458, &UA749176},
	{SG3469, &UA67291},
	{SG3489, &UA156552},
	{SG3490, &UA2018217},
	{SG3496, &UA9420},
	{SG3498, &UA707383},
	{SG3513, &UA157704},
	{SG3514, &UA1900029},
	{SG3518, &UA14287},
	{SG3519, &UA40942},
	{SG3520, &UA49016},
	{SG3522, &UA295468},
	{SG3523, &UA2019208},
	{SG3539, &UA2486404},
	{SG3543, &UA132689},
	{SG3545, &UA2203465},
	{SG3584, &UA2869750},
	{SG3585, &UA67747},
	{SG3590, &UA42119},
	{SG3591, &UA751843},
	{SG3598, &UA57944},
	{SG3599, &UA363397},
	{SG3603, &UA13088},
	{SG3624, &UA3509},
	{SG3625, &UA24085},
	{SG3626, &UA32549},
	{SG3630, &UA1979875},
	{SG3631, &UA2023461},
	{SG3632, &UA2031158},
	{SG3659, &UA56165},
	{SG3665, &UA388508},
	{SG3666, &UA2018458},
	{SG3676, &UA48889},
	{SG3677, &UA560956},
	{SG3678, &UA2020570},
	{SG3679, &UA2020667},
	{SG3698, &UA281562},
	{SG3699, &UA2018514},
	{SG3725, &UA2384334},
	{SG3727, &UA315045},
	{SG3730, &UA792870},
	{SG3731, &UA2765519},
	{SG3750, &UA53780},
	{SG3780, &UA506527},
	{SG3781, &UA639955},
	{SG3782, &UA651915},
	{SG3796, &UA158280},
	{SG3797, &UA616256},
	{SG3799, &UA809469},
	{SG3806, &UA3925},
	{SG3810, &UA641302},
	{SG3811, &UA1871661},
	{SG3820, &UA675854},
	{SG3821, &UA2020046},
	{SG3822, &UA2027194},
	{SG3824, &UA2768847},
	{SG3830, &UA21128},
	{SG3832, &UA2662389},
	{SG3833, &UA3107128},
	{SG3834, &UA50376},
	{SG3835, &UA56135},
	{SG3836, &UA300401},
	{SG3837, &UA2020605},
	{SG3842, &UA2350172},
	{SG3870, &UA447471},
	{SG3871, &UA828866},
	{SG3873, &UA2023462},
	{SG3879, &UA2652589},
	{SG3884, &UA19965},
	{SG3887, &UA2366962},
	{SG3894, &UA2958533},
	{SG3902, &UA2019582},
	{SG3903, &UA2517411},
	{SG3910, &UA2032407},
	{SG3912, &UA2781482},
	{SG3913, &UA2866356},
	{SG3916, &UA687163},
	{SG3917, &UA2855728},
	{SG3919, &UA2652757},
	{SG3927, &UA3109042},
	{SG3936, &UA43909},
	{SG3951, &UA2526667},
	{SG3959, &UA2424641},
	{SG3967, &UA2837443},
	{SG3968, &UA2865853},
	{SG3969, &UA54189},
	{SG3982, &UA16424},
	{SG3983, &UA41072},
	{SG3984, &UA48940},
	{SG3985, &UA56750},
	{SG3988, &UA2366173},
	{SG3989, &UA2662300},
	{SG3992, &UA18446},
	{SG3994, &UA21299},
	{SG3995, &UA53902},
	{SG3996, &UA2022196},
	{SG3998, &UA752884},
	{SG3999, &UA2025072},
	{SG4000, &UA2032343},
	{SG4016, &UA39996},
	{SG4018, &UA2021203},
	{SG4021, &UA2599242},
	{SG4036, &UA1951327},
	{SG4037, &UA2455623},
	{SG4043, &UA3011942},
	{SG4049, &UA508564},
	{SG4053, &UA3013528},
	{SG4054, &UA18173},
	{SG4055, &UA2021265},
	{SG4085, &UA56167},
	{SG4104, &UA2868906},
	{SG4106, &UA2025004},
	{SG4108, &UA2729963},
	{SG4117, &UA2027041},
	{SG4118, &UA2934844},
	{SG4121, &UA2529570},
	{SG4122, &UA2594771},
	{SG4128, &UA18558},
	{SG4130, &UA388951},
	{SG4131, &UA1962496},
	{SG4148, &UA3108315},
	{SG4160, &UA2703174},
	{SG4170, &UA2997491},
	{SG4184, &UA3108924},
	{SG4207, &UA2552851},
	{SG4209, &UA2792452},
	{SG4210, &UA2838746},
	{SG4211, &UA2851732},
	{SG4226, &UA1187},
	{SG4227, &UA1189},
	{SG4228, &UA17090},
	{SG4229, &UA17094},
	{SG4230, &UA38999},
	{SG4232, &UA22178},
	{SG4234, &UA2019841},
	{SG4237, &UA2525068},
	{SG4239, &UA2838441},
	{SG4264, &UA2832734},
	{SG4293, &UA50294},
	{SG4327, &UA2984959},
	{SG4365, &UA2023918},
	{SG4366, &UA2023983},
	{SG4370, &UA2222910},
	{SG4381, &UA2837533},
	{SG4383, &UA2847430},
	{SG4384, &UA2852984},
	{SG4397, &UA2018844},
	{SG4398, &UA2032368},
	{SG4402, &UA2222951},
	{SG4406, &UA2859183},
	{SG4409, &UA8534},
	{SG4410, &UA2627793},
	{SG4411, &UA2630536},
	{SG4417, &UA2018816},
	{SG4420, &UA34332},
	{SG4421, &UA2810837},
	{SG4422, &UA2018093},
	{SG4439, &UA769047},
	{SG4440, &UA1994257},
	{SG4443, &UA2804272},
	{SG4460, &UA3503},
	{SG4467, &UA2018437},
	{SG4468, &UA2018691},
	{SG4492, &UA1904837},
	{SG4515, &UA2026570},
	{SG4522, &UA563026},
	{SG4523, &UA1861174},
	{SG4524, &UA1886194},
	{SG4529, &UA61335},
	{SG4530, &UA2023355},
	{SG4533, &UA705610},
	{SG4537, &UA2609500},
	{SG4539, &UA2617101},
	{SG4567, &UA2338446},
	{SG4569, &UA2612164},
	{SG4570, &UA2614530},
	{SG4574, &UA30811},
	{SG4585, &UA2018178},
	{SG4607, &UA19689},
	{SG4618, &UA2018728},
	{SG4620, &UA2034507},
	{SG4621, &UA2612938},
	{SG4622, &UA2616332},
	{SG4623, &UA2631905},
	{SG4630, &UA2018330},
	{SG4667, &UA76022},
	{SG4671, &UA2022497},
	{SG4672, &UA2024386},
	{SG4677, &UA75108},
	{SG4680, &UA2019028},
	{SG4682, &UA2020392},
	{SG4683, &UA2022090},
	{SG4684, &UA2022984},
	{SG4688, &UA2635403},
	{SG4689, &UA34149},
	{SG4690, &UA38228},
	{SG4691, &UA41577},
	{SG4692, &UA57685},
	{SG4693, &UA58263},
	{SG4694, &UA2241447},
	{SG4695, &UA2611389},
	{SG4696, &UA2614541},
	{SG4697, &UA2619287},
	{SG4698, &UA2623274},
	{NULL}};
const Segmentdata DAndroidVersion2_2_2[] = {
	{SG4703, &UA36939},
	{SG4704, &UA36942},
	{SG4705, &UA359267},
	{SG4706, &UA361242},
	{SG4707, &UA396572},
	{SG4708, &UA398041},
	{SG4709, &UA630302},
	{SG4710, &UA630626},
	{SG4711, &UA649899},
	{SG4712, &UA701724},
	{SG4713, &UA2021381},
	{SG4714, &UA2024383},
	{SG4715, &UA2425697},
	{SG5236, &UA11430},
	{SG5237, &UA11434},
	{SG5238, &UA23677},
	{SG5239, &UA40257},
	{SG5240, &UA40288},
	{SG5241, &UA40295},
	{SG5242, &UA41034},
	{SG5245, &UA56718},
	{SG5246, &UA182029},
	{SG5248, &UA668853},
	{SG5420, &UA47987},
	{SG5425, &UA2020711},
	{SG5554, &UA22353},
	{SG5555, &UA33235},
	{SG5556, &UA46868},
	{SG5558, &UA47450},
	{SG5560, &UA49231},
	{SG5561, &UA54114},
	{SG5562, &UA54401},
	{SG5564, &UA58376},
	{SG5566, &UA58378},
	{SG5569, &UA58380},
	{SG5571, &UA59947},
	{SG5573, &UA59948},
	{SG5574, &UA61060},
	{SG5576, &UA70373},
	{SG5577, &UA71015},
	{SG5579, &UA72666},
	{SG5580, &UA72732},
	{SG5582, &UA79480},
	{SG5583, &UA321454},
	{SG5584, &UA660585},
	{SG5585, &UA755477},
	{SG5586, &UA1948881},
	{SG5587, &UA1979494},
	{SG5588, &UA2019992},
	{SG5589, &UA2020289},
	{SG5590, &UA2026354},
	{SG5591, &UA2933143},
	{SG5728, &UA8007},
	{SG5731, &UA10548},
	{SG5732, &UA10560},
	{SG5733, &UA10561},
	{SG5736, &UA39411},
	{SG5737, &UA39421},
	{SG5740, &UA52116},
	{SG5742, &UA52133},
	{SG5744, &UA52612},
	{SG5745, &UA52626},
	{SG5746, &UA52649},
	{SG5749, &UA52661},
	{SG5751, &UA52664},
	{SG5752, &UA52670},
	{SG5754, &UA52672},
	{SG5755, &UA52725},
	{SG5757, &UA61249},
	{SG5759, &UA70581},
	{SG5761, &UA70638},
	{SG5764, &UA71590},
	{SG5766, &UA395372},
	{SG5769, &UA693981},
	{SG5771, &UA713419},
	{SG5772, &UA721163},
	{SG5773, &UA744383},
	{SG5774, &UA748307},
	{SG5777, &UA851015},
	{SG5779, &UA858094},
	{SG5781, &UA1960500},
	{SG5782, &UA1983269},
	{SG5783, &UA2020406},
	{SG5785, &UA2021344},
	{SG5788, &UA2024913},
	{SG5790, &UA2026829},
	{SG5792, &UA2394721},
	{SG5794, &UA2495465},
	{SG5795, &UA2786837},
	{SG5796, &UA2810153},
	{SG5803, &UA43349},
	{SG5804, &UA48157},
	{SG5805, &UA566730},
	{SG6135, &UA52097},
	{SG6136, &UA79533},
	{SG6137, &UA227282},
	{SG6138, &UA316573},
	{SG6139, &UA332045},
	{SG6140, &UA334402},
	{SG6141, &UA370116},
	{SG6142, &UA370269},
	{SG6143, &UA370971},
	{SG6144, &UA451303},
	{SG6145, &UA565684},
	{SG6146, &UA566294},
	{SG6147, &UA570966},
	{SG6148, &UA571024},
	{SG6149, &UA632416},
	{SG6150, &UA648924},
	{SG6151, &UA675039},
	{SG6152, &UA678504},
	{SG6153, &UA699141},
	{SG6154, &UA712077},
	{SG6155, &UA718727},
	{SG6156, &UA725869},
	{SG6157, &UA732141},
	{SG6158, &UA736786},
	{SG6159, &UA741227},
	{SG6160, &UA754103},
	{SG6161, &UA760183},
	{SG6162, &UA813561},
	{SG6163, &UA849084},
	{SG6164, &UA875224},
	{SG6165, &UA1867060},
	{SG6166, &UA1877274},
	{SG6167, &UA1899173},
	{SG6168, &UA1899368},
	{SG6169, &UA1903543},
	{SG6170, &UA1907223},
	{SG6171, &UA1931208},
	{SG6172, &UA1932547},
	{SG6173, &UA1932876},
	{SG6174, &UA1934107},
	{SG6175, &UA1935064},
	{SG6176, &UA1935964},
	{SG6177, &UA1937188},
	{SG6178, &UA1942497},
	{SG6179, &UA1952075},
	{SG6180, &UA1955978},
	{SG6181, &UA1960170},
	{SG6182, &UA1961186},
	{SG6183, &UA1961949},
	{SG6184, &UA1968269},
	{SG6185, &UA1976155},
	{SG6186, &UA2018800},
	{SG6187, &UA2019234},
	{SG6188, &UA2019445},
	{SG6189, &UA2019696},
	{SG6190, &UA2019697},
	{SG6191, &UA2020021},
	{SG6192, &UA2020575},
	{SG6193, &UA2020930},
	{SG6194, &UA2021414},
	{SG6195, &UA2021601},
	{SG6196, &UA2021818},
	{SG6197, &UA2022019},
	{SG6198, &UA2022031},
	{SG6199, &UA2022187},
	{SG6200, &UA2022209},
	{SG6201, &UA2022346},
	{SG6202, &UA2022354},
	{SG6203, &UA2022520},
	{SG6204, &UA2022554},
	{SG6205, &UA2022738},
	{SG6206, &UA2022832},
	{SG6207, &UA2023015},
	{SG6208, &UA2023084},
	{SG6210, &UA2023213},
	{SG6211, &UA2023293},
	{SG6212, &UA2023912},
	{SG6213, &UA2024132},
	{SG6214, &UA2024210},
	{SG6215, &UA2024493},
	{SG6216, &UA2024579},
	{SG6217, &UA2024896},
	{SG6218, &UA2025620},
	{SG6219, &UA2025687},
	{SG6220, &UA2025882},
	{SG6221, &UA2026624},
	{SG6222, &UA2026634},
	{SG6223, &UA2026861},
	{SG6224, &UA2026983},
	{SG6225, &UA2027088},
	{SG6226, &UA2027773},
	{SG6227, &UA2028170},
	{SG6228, &UA2028745},
	{SG6229, &UA2029552},
	{SG6231, &UA2031682},
	{SG6233, &UA2032433},
	{SG6234, &UA2032745},
	{SG6236, &UA2033079},
	{SG6238, &UA2034635},
	{SG6240, &UA2223172},
	{SG6241, &UA2256901},
	{SG6242, &UA2256954},
	{SG6243, &UA2338967},
	{SG6245, &UA2411675},
	{SG6247, &UA2442470},
	{SG6248, &UA2443432},
	{SG6249, &UA2444594},
	{SG6250, &UA2448658},
	{SG6251, &UA2475667},
	{SG6252, &UA2478499},
	{SG6253, &UA2502482},
	{SG6254, &UA2505635},
	{SG6255, &UA2545823},
	{SG6256, &UA2573376},
	{SG6257, &UA2588966},
	{SG6258, &UA2589761},
	{SG6259, &UA2594779},
	{SG6260, &UA2596416},
	{SG6261, &UA2597531},
	{SG6262, &UA2600649},
	{SG6263, &UA2640033},
	{SG6264, &UA2642288},
	{SG6265, &UA2644259},
	{SG6266, &UA2658480},
	{SG6267, &UA2659093},
	{SG6268, &UA2684983},
	{SG6269, &UA2685707},
	{SG6270, &UA2721474},
	{SG6271, &UA2743235},
	{SG6272, &UA2762405},
	{SG6273, &UA2774998},
	{SG6274, &UA2786556},
	{SG6275, &UA2789493},
	{SG6276, &UA2811927},
	{SG6277, &UA2812979},
	{SG6278, &UA2889085},
	{SG6279, &UA2980346},
	{SG6280, &UA2982972},
	{SG6281, &UA2988623},
	{SG6282, &UA3085207},
	{SG6283, &UA3117234},
	{SG6494, &UA20073},
	{SG6495, &UA20074},
	{SG6496, &UA21644},
	{SG6497, &UA42656},
	{SG6498, &UA49490},
	{SG6499, &UA58550},
	{SG6500, &UA58579},
	{SG6501, &UA61027},
	{SG6502, &UA72728},
	{SG6503, &UA76800},
	{SG6504, &UA314768},
	{SG6505, &UA664434},
	{SG6506, &UA702472},
	{SG6507, &UA777870},
	{SG6508, &UA786430},
	{SG6509, &UA1952549},
	{SG6510, &UA3043984},
	{SG6511, &UA396156},
	{SG6512, &UA451497},
	{SG6513, &UA558925},
	{SG6514, &UA560373},
	{SG6515, &UA626581},
	{SG6516, &UA682978},
	{SG6517, &UA711002},
	{SG6518, &UA725750},
	{SG6520, &UA730933},
	{SG6521, &UA777476},
	{SG6522, &UA1895742},
	{SG6524, &UA1924922},
	{SG6525, &UA2018897},
	{SG6526, &UA2019594},
	{SG6527, &UA2020133},
	{SG6528, &UA2020284},
	{SG6529, &UA2020500},
	{SG6530, &UA2021286},
	{SG6531, &UA2021426},
	{SG6532, &UA2024234},
	{SG6533, &UA2024508},
	{SG6534, &UA2027815},
	{SG6535, &UA2372381},
	{SG6536, &UA2372729},
	{SG6537, &UA2395495},
	{SG6538, &UA2743801},
	{SG6539, &UA2980134},
	{SG6540, &UA40519},
	{SG6541, &UA40524},
	{SG6542, &UA40546},
	{SG6543, &UA55808},
	{SG6544, &UA55812},
	{SG6545, &UA55830},
	{SG6546, &UA395963},
	{SG6547, &UA659960},
	{SG6570, &UA45290},
	{SG6571, &UA49879},
	{SG6573, &UA70297},
	{SG6575, &UA70551},
	{SG6577, &UA368899},
	{SG6579, &UA672990},
	{SG6581, &UA673778},
	{SG6582, &UA737503},
	{SG6583, &UA1994524},
	{SG6584, &UA2020462},
	{SG6585, &UA2021215},
	{SG6586, &UA2189300},
	{SG6592, &UA56864},
	{SG6593, &UA56865},
	{SG6594, &UA56866},
	{SG6595, &UA60395},
	{SG6596, &UA440349},
	{SG6597, &UA668359},
	{SG6598, &UA1941177},
	{SG6599, &UA2022864},
	{SG6667, &UA57927},
	{SG6668, &UA378547},
	{SG6670, &UA2020986},
	{SG6671, &UA2022240},
	{SG6672, &UA2024217},
	{SG6676, &UA40622},
	{SG6680, &UA401103},
	{SG6682, &UA2022631},
	{SG6897, &UA22340},
	{SG6898, &UA46519},
	{SG6899, &UA46543},
	{SG6900, &UA46593},
	{SG6901, &UA46595},
	{SG6902, &UA59827},
	{SG6904, &UA59829},
	{SG6906, &UA59837},
	{SG6908, &UA59821},
	{SG6910, &UA70142},
	{SG6911, &UA225273},
	{SG6912, &UA379544},
	{SG6914, &UA576414},
	{SG6915, &UA623671},
	{SG6916, &UA2020002},
	{SG6917, &UA2022261},
	{SG6920, &UA2642563},
	{SG6921, &UA2807661},
	{SG6957, &UA18152},
	{SG6958, &UA18157},
	{SG6959, &UA22200},
	{SG6960, &UA22201},
	{SG6962, &UA24650},
	{SG6963, &UA24700},
	{SG6965, &UA45309},
	{SG6967, &UA48566},
	{SG6968, &UA49440},
	{SG6971, &UA49680},
	{SG6973, &UA59557},
	{SG6975, &UA190102},
	{SG6977, &UA396563},
	{SG6979, &UA1924258},
	{SG6980, &UA2020101},
	{SG6982, &UA2020942},
	{SG6984, &UA2027638},
	{SG7086, &UA72792},
	{SG7087, &UA638602},
	{SG7088, &UA2019811},
	{SG7089, &UA2023581},
	{SG7090, &UA2025210},
	{SG7136, &UA211507},
	{SG7138, &UA236463},
	{SG7141, &UA685969},
	{SG7144, &UA2019157},
	{SG7145, &UA2019513},
	{SG7146, &UA2143291},
	{SG7147, &UA2495070},
	{SG7171, &UA171942},
	{SG7172, &UA188493},
	{SG7173, &UA11358},
	{SG7174, &UA11360},
	{SG7175, &UA11361},
	{SG7176, &UA11363},
	{SG7177, &UA11693},
	{SG7178, &UA11694},
	{SG7183, &UA11752},
	{SG7184, &UA24224},
	{SG7185, &UA40095},
	{SG7186, &UA40118},
	{SG7187, &UA40119},
	{SG7188, &UA48030},
	{SG7189, &UA48037},
	{SG7191, &UA48042},
	{SG7193, &UA48044},
	{SG7196, &UA48052},
	{SG7198, &UA48053},
	{SG7201, &UA48064},
	{SG7202, &UA48070},
	{SG7203, &UA48072},
	{SG7205, &UA48075},
	{SG7207, &UA48079},
	{SG7209, &UA48080},
	{SG7211, &UA48086},
	{SG7213, &UA55329},
	{SG7214, &UA55330},
	{SG7215, &UA55334},
	{SG7216, &UA55335},
	{SG7217, &UA55338},
	{SG7218, &UA55348},
	{SG7219, &UA55369},
	{SG7220, &UA55374},
	{SG7221, &UA55384},
	{SG7222, &UA55389},
	{SG7223, &UA55399},
	{SG7224, &UA55414},
	{SG7225, &UA55371},
	{SG7226, &UA55425},
	{SG7227, &UA55426},
	{SG7228, &UA55421},
	{SG7229, &UA55710},
	{SG7230, &UA55713},
	{SG7231, &UA60997},
	{SG7232, &UA60998},
	{SG7233, &UA61000},
	{SG7234, &UA61003},
	{SG7235, &UA61004},
	{SG7236, &UA61006},
	{SG7237, &UA61007},
	{SG7238, &UA61010},
	{SG7240, &UA70873},
	{SG7241, &UA70929},
	{SG7242, &UA72287},
	{SG7243, &UA73338},
	{SG7244, &UA77991},
	{SG7245, &UA79122},
	{SG7246, &UA174874},
	{SG7247, &UA371073},
	{SG7248, &UA784300},
	{SG7249, &UA1908705},
	{SG7250, &UA1938602},
	{SG7251, &UA2026442},
	{SG7252, &UA2573315},
	{SG7253, &UA2699672},
	{SG7254, &UA2787428},
	{SG7274, &UA175510},
	{SG7275, &UA178969},
	{SG7276, &UA369531},
	{SG7277, &UA418402},
	{SG7278, &UA435941},
	{SG7279, &UA539744},
	{SG7280, &UA543997},
	{SG7281, &UA681539},
	{SG7282, &UA1878754},
	{SG7283, &UA2474175},
	{SG7389, &UA32793},
	{SG7391, &UA32797},
	{SG7393, &UA32798},
	{SG7395, &UA32800},
	{SG7396, &UA47634},
	{SG7397, &UA48595},
	{SG7399, &UA48602},
	{SG7400, &UA378652},
	{SG7401, &UA828033},
	{SG7438, &UA216218},
	{SG7439, &UA2607923},
	{SG7440, &UA2609510},
	{SG7441, &UA2611044},
	{SG7442, &UA2622391},
	{SG7443, &UA43389},
	{SG7444, &UA43393},
	{SG7446, &UA49827},
	{SG7447, &UA59328},
	{SG7449, &UA59329},
	{SG7450, &UA59335},
	{SG7451, &UA59332},
	{SG7452, &UA249868},
	{SG7454, &UA369017},
	{SG7455, &UA557538},
	{SG7456, &UA2023098},
	{SG7457, &UA2023773},
	{SG7496, &UA22167},
	{SG7497, &UA33221},
	{SG7498, &UA45291},
	{SG7499, &UA45292},
	{SG7501, &UA45294},
	{SG7502, &UA379434},
	{SG7503, &UA2019037},
	{SG7505, &UA2021720},
	{SG7567, &UA191892},
	{SG7568, &UA192446},
	{SG7570, &UA196332},
	{SG7572, &UA532280},
	{SG7573, &UA555770},
	{SG7574, &UA557381},
	{SG7576, &UA557437},
	{SG7577, &UA797604},
	{SG7578, &UA2019492},
	{SG7580, &UA47929},
	{SG7581, &UA47930},
	{SG7582, &UA78318},
	{SG7583, &UA78878},
	{SG7584, &UA575469},
	{SG7585, &UA575537},
	{SG7586, &UA645454},
	{SG7588, &UA646076},
	{SG7590, &UA775186},
	{SG7592, &UA801211},
	{SG7594, &UA820924},
	{SG7596, &UA1940616},
	{SG7598, &UA1965106},
	{SG7600, &UA2020532},
	{SG7602, &UA2475092},
	{SG7603, &UA2644394},
	{SG7610, &UA819990},
	{SG7720, &UA38517},
	{SG7723, &UA678712},
	{SG7778, &UA2031605},
	{SG7780, &UA2034408},
	{SG7794, &UA182608},
	{SG7795, &UA189864},
	{SG7796, &UA236014},
	{SG7797, &UA308013},
	{SG7798, &UA2019293},
	{SG7799, &UA2021916},
	{SG7800, &UA2574460},
	{SG7801, &UA22314},
	{SG7802, &UA46504},
	{SG7804, &UA205495},
	{SG7806, &UA370174},
	{SG7810, &UA33002},
	{SG7811, &UA33004},
	{SG7812, &UA48773},
	{SG7813, &UA70021},
	{SG7814, &UA70767},
	{SG7815, &UA70888},
	{SG7816, &UA79198},
	{SG7817, &UA368765},
	{SG7818, &UA420455},
	{SG7819, &UA645263},
	{SG7821, &UA2574024},
	{SG7886, &UA19546},
	{SG7887, &UA19551},
	{SG7888, &UA42257},
	{SG7889, &UA58222},
	{SG7890, &UA58228},
	{SG7891, &UA58229},
	{SG7892, &UA63802},
	{SG7893, &UA191212},
	{SG7895, &UA225905},
	{SG7896, &UA379005},
	{SG7897, &UA2809321},
	{SG7908, &UA59223},
	{SG7909, &UA666124},
	{SG7914, &UA61113},
	{SG7915, &UA76561},
	{SG7916, &UA78103},
	{SG7917, &UA378520},
	{SG7918, &UA2019495},
	{SG7919, &UA3117796},
	{SG7920, &UA250782},
	{SG7921, &UA654499},
	{SG7923, &UA48704},
	{SG7924, &UA48706},
	{SG7925, &UA49881},
	{SG7926, &UA59318},
	{SG7927, &UA70647},
	{SG7928, &UA90564},
	{SG7929, &UA1895745},
	{SG7931, &UA2020251},
	{SG7933, &UA2023513},
	{SG7934, &UA2023895},
	{SG7935, &UA421688},
	{SG7936, &UA454655},
	{SG7937, &UA1954005},
	{SG7938, &UA2024776},
	{SG7939, &UA2031814},
	{SG7940, &UA2033027},
	{SG7949, &UA236944},
	{SG7950, &UA378730},
	{SG7951, &UA891308},
	{SG7952, &UA2023972},
	{SG8063, &UA61102},
	{SG8064, &UA61257},
	{SG8065, &UA61333},
	{SG8066, &UA61380},
	{SG8067, &UA251136},
	{SG8068, &UA680893},
	{SG8069, &UA24526},
	{SG8070, &UA60722},
	{SG8071, &UA61377},
	{SG8072, &UA551750},
	{SG8073, &UA727850},
	{SG8074, &UA1893475},
	{SG8075, &UA2439987},
	{SG8081, &UA59613},
	{SG8083, &UA78145},
	{SG8084, &UA195727},
	{SG8085, &UA578137},
	{SG8148, &UA40868},
	{SG8149, &UA58781},
	{SG8150, &UA58782},
	{SG8151, &UA2021592},
	{SG8225, &UA680434},
	{SG8226, &UA838624},
	{SG8227, &UA1901242},
	{SG8228, &UA68159},
	{SG8229, &UA686236},
	{SG8231, &UA40410},
	{SG8232, &UA40431},
	{SG8233, &UA55644},
	{SG8234, &UA55648},
	{SG8235, &UA70770},
	{SG8236, &UA71746},
	{SG8237, &UA694834},
	{SG8238, &UA2019190},
	{SG8239, &UA6376},
	{SG8241, &UA56697},
	{SG8243, &UA90710},
	{SG8244, &UA434220},
	{SG8252, &UA33440},
	{SG8253, &UA558873},
	{SG8254, &UA2019377},
	{SG8312, &UA23811},
	{SG8313, &UA23812},
	{SG8314, &UA310643},
	{SG8315, &UA2018586},
	{SG8317, &UA52402},
	{SG8320, &UA561282},
	{SG8368, &UA38378},
	{SG8370, &UA52095},
	{SG8372, &UA52146},
	{SG8373, &UA52159},
	{SG8374, &UA52149},
	{SG8375, &UA55657},
	{SG8376, &UA409144},
	{SG8428, &UA41549},
	{SG8430, &UA876241},
	{SG8432, &UA2019248},
	{SG8433, &UA2020654},
	{SG8435, &UA2022296},
	{SG8436, &UA2022441},
	{SG8481, &UA42855},
	{SG8482, &UA70793},
	{SG8483, &UA76416},
	{SG8484, &UA805291},
	{SG8485, &UA869800},
	{SG8486, &UA895477},
	{SG8510, &UA68208},
	{SG8511, &UA68481},
	{SG8512, &UA68835},
	{SG8513, &UA2019131},
	{SG8547, &UA33213},
	{SG8549, &UA1939935},
	{SG8575, &UA49444},
	{SG8576, &UA49445},
	{SG8577, &UA762653},
	{SG8578, &UA719371},
	{SG8587, &UA57802},
	{SG8588, &UA1894582},
	{SG8618, &UA50021},
	{SG8621, &UA641153},
	{SG8622, &UA1933830},
	{SG8623, &UA2019787},
	{SG8624, &UA2345078},
	{SG8626, &UA2465967},
	{SG8649, &UA178599},
	{SG8651, &UA378267},
	{SG8652, &UA556879},
	{SG8654, &UA2018772},
	{SG8656, &UA2018835},
	{SG8658, &UA2018836},
	{SG8660, &UA2019271},
	{SG8664, &UA90596},
	{SG8665, &UA644410},
	{SG8758, &UA67784},
	{SG8765, &UA2018462},
	{SG8798, &UA224601},
	{SG8799, &UA243755},
	{SG8800, &UA2021300},
	{SG8801, &UA2043778},
	{SG8816, &UA19948},
	{SG8817, &UA24652},
	{SG8818, &UA186229},
	{SG8819, &UA534014},
	{SG8821, &UA175530},
	{SG8822, &UA184198},
	{SG8823, &UA735239},
	{SG8824, &UA2021493},
	{SG8905, &UA192111},
	{SG8906, &UA566943},
	{SG8907, &UA246169},
	{SG8908, &UA1907351},
	{SG8909, &UA2020150},
	{SG8910, &UA2020155},
	{SG8911, &UA2023668},
	{SG8997, &UA61527},
	{SG8998, &UA70061},
	{SG8999, &UA674792},
	{SG9022, &UA194361},
	{SG9023, &UA214316},
	{SG9024, &UA220042},
	{SG9025, &UA601686},
	{SG9026, &UA2018338},
	{SG9038, &UA706142},
	{SG9039, &UA2027452},
	{SG9062, &UA24407},
	{SG9064, &UA40798},
	{SG9083, &UA641269},
	{SG9084, &UA1935114},
	{SG9090, &UA180342},
	{SG9092, &UA2019330},
	{SG9102, &UA6361},
	{SG9103, &UA37551},
	{SG9110, &UA49350},
	{SG9111, &UA49352},
	{SG9112, &UA2019608},
	{SG9134, &UA4134},
	{SG9135, &UA20161},
	{SG9136, &UA2023023},
	{SG9154, &UA48710},
	{SG9156, &UA75023},
	{SG9158, &UA1946895},
	{SG9159, &UA2019899},
	{SG9286, &UA625005},
	{SG9287, &UA643165},
	{SG9306, &UA20334},
	{SG9308, &UA1889165},
	{SG9325, &UA43231},
	{SG9332, &UA70983},
	{SG9333, &UA79571},
	{SG9334, &UA653358},
	{SG9335, &UA671251},
	{SG9336, &UA2021521},
	{SG9337, &UA2023388},
	{SG9338, &UA2686445},
	{SG9364, &UA192217},
	{SG9365, &UA563889},
	{SG9366, &UA1877645},
	{SG9367, &UA2032694},
	{SG9377, &UA20885},
	{SG9378, &UA20887},
	{SG9379, &UA59123},
	{SG9381, &UA59124},
	{SG9382, &UA59125},
	{SG9383, &UA59133},
	{SG9384, &UA59141},
	{SG9385, &UA59139},
	{SG9386, &UA551869},
	{SG9414, &UA18103},
	{SG9415, &UA41570},
	{SG9469, &UA68310},
	{SG9471, &UA643275},
	{SG9504, &UA195095},
	{SG9505, &UA556464},
	{SG9541, &UA799139},
	{SG9581, &UA24004},
	{SG9583, &UA206984},
	{SG9617, &UA23593},
	{SG9619, &UA197127},
	{SG9682, &UA60982},
	{SG9683, &UA651621},
	{SG9684, &UA2023372},
	{SG9691, &UA49283},
	{SG9693, &UA90616},
	{SG9695, &UA370854},
	{SG9696, &UA1902369},
	{SG9716, &UA42871},
	{SG9717, &UA42869},
	{SG9748, &UA195135},
	{SG9749, &UA623359},
	{SG9750, &UA702184},
	{SG9756, &UA694095},
	{SG9757, &UA761132},
	{SG9768, &UA67801},
	{SG9769, &UA434939},
	{SG9841, &UA56862},
	{SG9842, &UA742928},
	{SG9869, &UA40757},
	{SG9870, &UA799295},
	{SG9871, &UA2030882},
	{SG9872, &UA2813339},
	{SG9890, &UA49219},
	{SG9891, &UA758362},
	{SG9892, &UA2812051},
	{SG9896, &UA33365},
	{SG9927, &UA200671},
	{SG9928, &UA250334},
	{SG9929, &UA453499},
	{SG9930, &UA1905761},
	{SG9931, &UA2042574},
	{SG9940, &UA48711},
	{SG9941, &UA669893},
	{SG9942, &UA792742},
	{SG9944, &UA2031146},
	{SG9954, &UA2642196},
	{SG9979, &UA58215},
	{SG9986, &UA210660},
	{SG9987, &UA2019936},
	{SG9988, &UA2020547},
	{SG9993, &UA56761},
	{SG9994, &UA781930},
	{SG9995, &UA32883},
	{SG9996, &UA2019997},
	{SG10000, &UA573497},
	{SG10001, &UA580135},
	{SG10002, &UA622517},
	{SG10005, &UA622529},
	{SG10006, &UA622499},
	{SG10007, &UA2597217},
	{SG10008, &UA2603992},
	{SG10017, &UA599867},
	{SG10032, &UA782949},
	{SG10069, &UA2033062},
	{SG10076, &UA396491},
	{SG10077, &UA712348},
	{SG10078, &UA2029034},
	{SG10079, &UA2337145},
	{SG10080, &UA553535},
	{SG10082, &UA68340},
	{SG10083, &UA2018818},
	{SG10093, &UA395495},
	{SG10099, &UA67674},
	{SG10100, &UA68376},
	{SG10101, &UA2018537},
	{SG10114, &UA24522},
	{SG10115, &UA230097},
	{SG10116, &UA728482},
	{SG10117, &UA1980099},
	{SG10118, &UA2019494},
	{SG10129, &UA567854},
	{SG10161, &UA566578},
	{SG10163, &UA2020774},
	{SG10165, &UA2021973},
	{SG10179, &UA253809},
	{SG10180, &UA379642},
	{SG10181, &UA744005},
	{SG10190, &UA566080},
	{SG10191, &UA569512},
	{SG10192, &UA627414},
	{SG10193, &UA685128},
	{SG10194, &UA436479},
	{SG10195, &UA701205},
	{SG10220, &UA53548},
	{SG10221, &UA76458},
	{SG10222, &UA1937011},
	{SG10225, &UA436577},
	{SG10226, &UA2019968},
	{SG10232, &UA1943042},
	{SG10233, &UA2019857},
	{SG10234, &UA2574029},
	{SG10246, &UA40003},
	{SG10247, &UA699785},
	{SG10248, &UA1937436},
	{SG10249, &UA68248},
	{SG10250, &UA1888227},
	{SG10258, &UA57730},
	{SG10261, &UA646372},
	{SG10267, &UA52401},
	{SG10269, &UA539347},
	{SG10270, &UA2019172},
	{SG10272, &UA180625},
	{SG10273, &UA564977},
	{SG10301, &UA1896861},
	{SG10302, &UA2020554},
	{SG10313, &UA70529},
	{SG10314, &UA715273},
	{SG10315, &UA2033408},
	{SG10335, &UA378661},
	{SG10387, &UA22381},
	{SG10389, &UA721970},
	{SG10404, &UA49424},
	{SG10405, &UA589768},
	{SG10407, &UA683148},
	{SG10463, &UA792241},
	{SG10465, &UA751521},
	{SG10466, &UA895490},
	{SG10471, &UA67905},
	{SG10472, &UA2018763},
	{SG10473, &UA654448},
	{SG10474, &UA741652},
	{SG10475, &UA772344},
	{SG10490, &UA378330},
	{SG10491, &UA2203466},
	{SG10504, &UA400769},
	{SG10505, &UA787304},
	{SG10506, &UA1900115},
	{SG10507, &UA2018567},
	{SG10508, &UA2022105},
	{SG10509, &UA683410},
	{SG10510, &UA2023199},
	{SG10512, &UA663892},
	{SG10513, &UA722126},
	{SG10514, &UA1964122},
	{SG10515, &UA2978573},
	{SG10532, &UA61591},
	{SG10558, &UA2020855},
	{SG10559, &UA763092},
	{SG10560, &UA2019941},
	{SG10561, &UA2026696},
	{SG10572, &UA40130},
	{SG10573, &UA40161},
	{SG10574, &UA40168},
	{SG10575, &UA48379},
	{SG10576, &UA55718},
	{SG10577, &UA61362},
	{SG10578, &UA72780},
	{SG10579, &UA75480},
	{SG10580, &UA2020905},
	{SG10596, &UA557971},
	{SG10599, &UA55313},
	{SG10629, &UA1991717},
	{SG10640, &UA734859},
	{SG10642, &UA2022120},
	{SG10643, &UA2024068},
	{SG10644, &UA2034156},
	{SG10655, &UA673497},
	{SG10663, &UA712577},
	{SG10664, &UA2028730},
	{SG10678, &UA2425804},
	{SG10682, &UA61467},
	{SG10707, &UA545565},
	{SG10709, &UA571309},
	{SG10712, &UA1968178},
	{SG10714, &UA2020753},
	{SG10716, &UA2020876},
	{SG10717, &UA2026072},
	{SG10718, &UA2027833},
	{SG10719, &UA2030008},
	{SG10720, &UA2030501},
	{SG10721, &UA2721199},
	{SG10724, &UA49508},
	{SG10762, &UA447905},
	{SG10787, &UA70718},
	{SG10788, &UA2022692},
	{SG10789, &UA2546114},
	{SG10793, &UA196107},
	{SG10800, &UA1994894},
	{SG10802, &UA49905},
	{SG10803, &UA679819},
	{SG10806, &UA1941514},
	{SG10807, &UA2023339},
	{SG10808, &UA674235},
	{SG10825, &UA572918},
	{SG10826, &UA1983086},
	{SG10827, &UA2034345},
	{SG10838, &UA801674},
	{SG10839, &UA2022257},
	{SG10846, &UA40430},
	{SG10847, &UA55619},
	{SG10848, &UA55645},
	{SG10849, &UA2022312},
	{SG10850, &UA2025950},
	{SG10876, &UA853919},
	{SG10877, &UA2021068},
	{SG10879, &UA205012},
	{SG10880, &UA2019152},
	{SG10892, &UA2440254},
	{SG10897, &UA733057},
	{SG10903, &UA48472},
	{SG10911, &UA1898731},
	{SG10919, &UA2237126},
	{SG10920, &UA49828},
	{SG10931, &UA2431940},
	{SG10937, &UA2020833},
	{SG10939, &UA73267},
	{SG10940, &UA2022023},
	{SG10943, &UA2023316},
	{SG10944, &UA670003},
	{SG10945, &UA2020929},
	{SG10946, &UA2026538},
	{SG10954, &UA525642},
	{SG10955, &UA533228},
	{SG10959, &UA41580},
	{SG10960, &UA57325},
	{SG10961, &UA2812052},
	{SG10962, &UA42677},
	{SG10963, &UA421550},
	{SG10967, &UA552575},
	{SG10969, &UA570671},
	{SG10970, &UA733290},
	{SG10978, &UA49383},
	{SG10988, &UA2424828},
	{SG11009, &UA687840},
	{SG11013, &UA789730},
	{SG11017, &UA22221},
	{SG11018, &UA22222},
	{SG11019, &UA59582},
	{SG11020, &UA59583},
	{SG11021, &UA59584},
	{SG11022, &UA59585},
	{SG11035, &UA2455579},
	{SG11051, &UA2019119},
	{SG11079, &UA540926},
	{SG11101, &UA2034226},
	{SG11108, &UA2033606},
	{SG11138, &UA2495731},
	{SG11139, &UA2983277},
	{SG11143, &UA37311},
	{SG11157, &UA79159},
	{SG11162, &UA2027227},
	{SG11163, &UA2028198},
	{SG11164, &UA2032067},
	{SG11165, &UA2575056},
	{SG11166, &UA2262221},
	{SG11176, &UA628381},
	{SG11177, &UA733405},
	{SG11178, &UA2022962},
	{SG11179, &UA2027376},
	{SG11189, &UA40284},
	{SG11190, &UA833097},
	{SG11193, &UA738439},
	{SG11194, &UA2028280},
	{SG11195, &UA2033536},
	{SG11196, &UA2546523},
	{SG11223, &UA11921},
	{SG11224, &UA2026488},
	{SG11225, &UA2032255},
	{SG11232, &UA2737890},
	{SG11234, &UA61471},
	{SG11235, &UA2023821},
	{SG11238, &UA1922090},
	{SG11248, &UA60904},
	{SG11250, &UA2019084},
	{SG11257, &UA2559101},
	{SG11258, &UA3058143},
	{SG11262, &UA78680},
	{SG11264, &UA2598438},
	{SG11287, &UA2022551},
	{SG11309, &UA60655},
	{SG11314, &UA1905951},
	{SG11315, &UA2020436},
	{SG11316, &UA2026527},
	{SG11318, &UA2699532},
	{SG11320, &UA706081},
	{SG11325, &UA1934264},
	{SG11326, &UA2024870},
	{SG11327, &UA2026765},
	{SG11358, &UA718782},
	{SG11362, &UA2474420},
	{SG11372, &UA706571},
	{SG11390, &UA688379},
	{SG11398, &UA369629},
	{SG11399, &UA2021722},
	{SG11400, &UA2024229},
	{SG11409, &UA1906549},
	{SG11417, &UA2502408},
	{SG11426, &UA2752274},
	{SG11430, &UA2337011},
	{SG11454, &UA2684941},
	{SG11455, &UA2722667},
	{SG11462, &UA49875},
	{SG11472, &UA2351671},
	{SG11473, &UA2457533},
	{SG11482, &UA58799},
	{SG11502, &UA2455042},
	{SG11532, &UA2424642},
	{SG11565, &UA50022},
	{SG11566, &UA2682790},
	{SG11568, &UA2023092},
	{SG11573, &UA2966557},
	{SG11581, &UA2024836},
	{SG11620, &UA1974907},
	{SG11621, &UA2720523},
	{SG11625, &UA2659516},
	{SG11626, &UA2762110},
	{SG11631, &UA49327},
	{SG11638, &UA3010840},
	{SG11648, &UA2019642},
	{SG11651, &UA2459531},
	{SG11658, &UA2032316},
	{SG11696, &UA2598631},
	{SG11705, &UA2032782},
	{SG11716, &UA2020933},
	{SG11726, &UA2455374},
	{SG11737, &UA369991},
	{SG11738, &UA2021514},
	{SG11751, &UA2029936},
	{SG11752, &UA2466164},
	{SG11753, &UA2484711},
	{SG11756, &UA2659633},
	{SG11778, &UA2941393},
	{SG11792, &UA2350909},
	{SG11793, &UA2482701},
	{SG11817, &UA2027944},
	{SG11829, &UA2748758},
	{SG11847, &UA1953444},
	{SG11855, &UA2660648},
	{SG11872, &UA2020142},
	{SG11874, &UA2032962},
	{SG11875, &UA2033716},
	{SG11878, &UA2484312},
	{SG11889, &UA2998736},
	{SG11898, &UA2383778},
	{SG11905, &UA2892560},
	{SG11921, &UA2455069},
	{SG11922, &UA2659220},
	{SG11929, &UA2993872},
	{SG11940, &UA49224},
	{SG11941, &UA70315},
	{SG11945, &UA2019347},
	{SG11950, &UA2863060},
	{SG11972, &UA2730550},
	{SG11975, &UA2860072},
	{SG11977, &UA2999813},
	{SG11988, &UA1901667},
	{SG11991, &UA2400993},
	{SG12014, &UA2023261},
	{SG12017, &UA2833069},
	{SG12018, &UA2814225},
	{SG12019, &UA2825221},
	{SG12031, &UA2031450},
	{SG12033, &UA2395777},
	{SG12043, &UA2027503},
	{SG12044, &UA2028725},
	{SG12052, &UA2340607},
	{SG12053, &UA2624880},
	{SG12057, &UA2631562},
	{SG12058, &UA2820139},
	{SG12075, &UA2338455},
	{SG12080, &UA55961},
	{SG12081, &UA61570},
	{SG12082, &UA2024523},
	{SG12083, &UA2024787},
	{SG12095, &UA2030136},
	{SG12102, &UA2027778},
	{SG12108, &UA369476},
	{SG12110, &UA2026116},
	{SG12114, &UA580176},
	{SG12115, &UA2022458},
	{SG12123, &UA48378},
	{SG12140, &UA2026212},
	{SG12149, &UA21301},
	{SG12152, &UA59231},
	{SG12155, &UA39407},
	{SG12159, &UA879566},
	{SG12162, &UA2024888},
	{SG12163, &UA2027288},
	{SG12164, &UA2030719},
	{SG12184, &UA2028407},
	{SG12185, &UA2618829},
	{SG12194, &UA2637799},
	{SG12207, &UA40414},
	{SG12216, &UA75326},
	{SG12220, &UA370145},
	{SG12222, &UA2030590},
	{NULL}};
const Segmentdata DAndroidVersion2_3_1[] = {
	{SG4701, &UA90531},
	{SG4702, &UA2290164},
	{SG4907, &UA300166},
	{SG4975, &UA192386},
	{SG4976, &UA358968},
	{SG4977, &UA370499},
	{SG4978, &UA370616},
	{SG4979, &UA421797},
	{SG4980, &UA421856},
	{SG4982, &UA437113},
	{SG4985, &UA458775},
	{SG4988, &UA550108},
	{SG4990, &UA567912},
	{SG4991, &UA570889},
	{SG4993, &UA573820},
	{SG4995, &UA578369},
	{SG4997, &UA578600},
	{SG4999, &UA581408},
	{SG5000, &UA634277},
	{SG5001, &UA637813},
	{SG5002, &UA643767},
	{SG5003, &UA662886},
	{SG5004, &UA687532},
	{SG5005, &UA692161},
	{SG5006, &UA700501},
	{SG5007, &UA700601},
	{SG5008, &UA702078},
	{SG5009, &UA702081},
	{SG5010, &UA707388},
	{SG5011, &UA710998},
	{SG5012, &UA711093},
	{SG5013, &UA715924},
	{SG5014, &UA725884},
	{SG5015, &UA730296},
	{SG5016, &UA732354},
	{SG5017, &UA737882},
	{SG5018, &UA759657},
	{SG5019, &UA765546},
	{SG5020, &UA776169},
	{SG5021, &UA897210},
	{SG5022, &UA1879944},
	{SG5023, &UA1900235},
	{SG5024, &UA1901330},
	{SG5025, &UA1906505},
	{SG5026, &UA1908141},
	{SG5027, &UA1931404},
	{SG5028, &UA1938778},
	{SG5029, &UA1949988},
	{SG5030, &UA1955759},
	{SG5031, &UA1963337},
	{SG5032, &UA1969605},
	{SG5033, &UA1987849},
	{SG5034, &UA1991115},
	{SG5035, &UA1994920},
	{SG5036, &UA2018683},
	{SG5037, &UA2019115},
	{SG5038, &UA2019993},
	{SG5039, &UA2020259},
	{SG5040, &UA2020495},
	{SG5041, &UA2020870},
	{SG5042, &UA2021001},
	{SG5043, &UA2021620},
	{SG5044, &UA2021692},
	{SG5045, &UA2021906},
	{SG5046, &UA2021943},
	{SG5047, &UA2022430},
	{SG5048, &UA2022547},
	{SG5049, &UA2022575},
	{SG5050, &UA2022854},
	{SG5051, &UA2023192},
	{SG5052, &UA2023528},
	{SG5053, &UA2023692},
	{SG5061, &UA2025445},
	{SG5062, &UA2025577},
	{SG5063, &UA2025741},
	{SG5064, &UA2026556},
	{SG5065, &UA2026734},
	{SG5066, &UA2027128},
	{SG5067, &UA2027360},
	{SG5068, &UA2027431},
	{SG5069, &UA2027835},
	{SG5070, &UA2028017},
	{SG5071, &UA2029553},
	{SG5072, &UA2030509},
	{SG5073, &UA2031195},
	{SG5074, &UA2031586},
	{SG5075, &UA2031686},
	{SG5076, &UA2032799},
	{SG5077, &UA2032996},
	{SG5078, &UA2033088},
	{SG5079, &UA2033525},
	{SG5080, &UA2034241},
	{SG5081, &UA2034420},
	{SG5082, &UA2257182},
	{SG5085, &UA2411284},
	{SG5086, &UA2412307},
	{SG5088, &UA2474119},
	{SG5090, &UA2519173},
	{SG5091, &UA2548218},
	{SG5092, &UA2589627},
	{SG5093, &UA2591892},
	{SG5094, &UA2643414},
	{SG5095, &UA2652472},
	{SG5096, &UA2659302},
	{SG5097, &UA2682731},
	{SG5098, &UA2682960},
	{SG5105, &UA2758376},
	{SG5118, &UA2808285},
	{SG5119, &UA2809370},
	{SG5120, &UA2810769},
	{SG5122, &UA2815732},
	{SG5125, &UA2839999},
	{SG5131, &UA2979161},
	{SG5136, &UA2982783},
	{SG5142, &UA3084997},
	{SG5148, &UA3117295},
	{SG5183, &UA18221},
	{SG5184, &UA18306},
	{SG5189, &UA70715},
	{SG5257, &UA316467},
	{SG5261, &UA2018345},
	{SG5262, &UA2019054},
	{SG5263, &UA2019751},
	{SG5264, &UA2026325},
	{SG5267, &UA2034440},
	{SG5282, &UA2374285},
	{SG5334, &UA2811434},
	{SG5956, &UA298603},
	{SG6230, &UA2029340},
	{SG6235, &UA2607568},
	{SG6352, &UA41483},
	{SG6354, &UA41505},
	{SG6361, &UA53618},
	{SG6364, &UA57188},
	{SG6519, &UA690655},
	{SG6523, &UA2018211},
	{SG6953, &UA296926},
	{SG6954, &UA301254},
	{SG6955, &UA743403},
	{SG7711, &UA2272383},
	{SG7716, &UA67354},
	{SG7717, &UA1860244},
	{SG7718, &UA2018233},
	{SG7719, &UA2018400},
	{SG7948, &UA57775},
	{SG8152, &UA168229},
	{SG8205, &UA1860891},
	{SG8850, &UA166839},
	{SG8855, &UA387297},
	{SG8857, &UA819282},
	{SG8860, &UA2018478},
	{SG8863, &UA2018599},
	{SG9028, &UA196489},
	{SG9029, &UA349257},
	{SG9030, &UA650148},
	{SG9031, &UA1944050},
	{SG9032, &UA2813211},
	{SG9131, &UA2031535},
	{SG9273, &UA2598100},
	{SG9278, &UA59287},
	{SG9411, &UA250911},
	{SG9412, &UA573590},
	{SG9413, &UA2020096},
	{SG9591, &UA169800},
	{SG9595, &UA658285},
	{SG9596, &UA2018337},
	{SG9597, &UA2018412},
	{SG9598, &UA2021873},
	{SG9600, &UA2027101},
	{SG9613, &UA622908},
	{SG9627, &UA622984},
	{SG9718, &UA198043},
	{SG9720, &UA2018959},
	{SG9813, &UA190447},
	{SG9814, &UA750333},
	{SG9815, &UA1882478},
	{SG9816, &UA2019102},
	{SG9834, &UA636829},
	{SG9835, &UA814871},
	{SG9943, &UA599545},
	{SG9955, &UA2237115},
	{SG9978, &UA369230},
	{SG9980, &UA573522},
	{SG9981, &UA737339},
	{SG9982, &UA1932246},
	{SG9983, &UA2023393},
	{SG9997, &UA753909},
	{SG10009, &UA354761},
	{SG10130, &UA629266},
	{SG10204, &UA771009},
	{SG10205, &UA2023305},
	{SG10206, &UA360307},
	{SG10207, &UA574540},
	{SG10208, &UA650344},
	{SG10283, &UA701420},
	{SG10285, &UA2031444},
	{SG10299, &UA370023},
	{SG10300, &UA682610},
	{SG10316, &UA630270},
	{SG10317, &UA2034281},
	{SG10324, &UA2022194},
	{SG10337, &UA396523},
	{SG10338, &UA2023411},
	{SG10533, &UA578329},
	{SG10534, &UA2028292},
	{SG10588, &UA630668},
	{SG10679, &UA739718},
	{SG10680, &UA1979251},
	{SG10684, &UA654552},
	{SG10783, &UA696801},
	{SG10799, &UA628573},
	{SG10804, &UA721919},
	{SG10871, &UA2022942},
	{SG10895, &UA2022111},
	{SG10957, &UA759837},
	{SG10958, &UA1966047},
	{SG11006, &UA719985},
	{SG11125, &UA699021},
	{SG11133, &UA702996},
	{SG11254, &UA679372},
	{SG11255, &UA742296},
	{SG11319, &UA779907},
	{SG11371, &UA2033316},
	{SG11450, &UA627178},
	{SG11586, &UA725722},
	{SG11599, &UA705706},
	{SG11610, &UA2024275},
	{SG11639, &UA2336305},
	{SG11640, &UA2616605},
	{SG11747, &UA2024077},
	{SG11749, &UA2411627},
	{SG11884, &UA3048318},
	{SG11993, &UA2779979},
	{SG12002, &UA2554714},
	{SG12117, &UA2023271},
	{SG12151, &UA2746948},
	{SG12270, &UA2463206},
	{SG12307, &UA2020082},
	{SG12332, &UA2339055},
	{SG12362, &UA2024687},
	{SG12569, &UA2629863},
	{NULL}};
const Segmentdata DAndroidVersion2_3[] = {
	{SG4699, &UA1970192},
	{SG4700, &UA2807223},
	{SG4798, &UA2030887},
	{SG4935, &UA2018243},
	{SG4943, &UA68822},
	{SG5083, &UA370848},
	{SG5084, &UA458968},
	{SG5087, &UA683703},
	{SG5089, &UA780781},
	{SG5099, &UA1969486},
	{SG5100, &UA2021802},
	{SG5108, &UA2023814},
	{SG5110, &UA2024428},
	{SG5113, &UA2024545},
	{SG5116, &UA2025971},
	{SG5117, &UA2027890},
	{SG5121, &UA2031918},
	{SG5123, &UA2032057},
	{SG5124, &UA2223431},
	{SG5126, &UA2257189},
	{SG5127, &UA2339170},
	{SG5128, &UA2421753},
	{SG5129, &UA2474220},
	{SG5130, &UA2474777},
	{SG5132, &UA2494854},
	{SG5133, &UA2503695},
	{SG5134, &UA2506154},
	{SG5135, &UA2518606},
	{SG5137, &UA2547800},
	{SG5138, &UA2574707},
	{SG5139, &UA2601758},
	{SG5140, &UA2641340},
	{SG5141, &UA2685060},
	{SG5143, &UA2743023},
	{SG5144, &UA2786699},
	{SG5145, &UA2790109},
	{SG5146, &UA2810433},
	{SG5147, &UA2811947},
	{SG5149, &UA2817589},
	{SG5150, &UA2823336},
	{SG5151, &UA2869203},
	{SG5153, &UA2952996},
	{SG5154, &UA2982033},
	{SG5158, &UA3057034},
	{SG5159, &UA3091822},
	{SG5160, &UA3099957},
	{SG5161, &UA3103031},
	{SG5361, &UA421293},
	{SG5364, &UA535343},
	{SG5369, &UA653583},
	{SG5396, &UA1958309},
	{SG5400, &UA2020323},
	{SG5404, &UA2029882},
	{SG5405, &UA2365457},
	{SG5417, &UA2851876},
	{SG6013, &UA842734},
	{SG6018, &UA2031789},
	{SG6319, &UA1943553},
	{SG6394, &UA2934904},
	{SG6642, &UA2026451},
	{SG6643, &UA2474882},
	{SG7097, &UA491477},
	{SG7098, &UA569707},
	{SG7099, &UA2032766},
	{SG7100, &UA2428463},
	{SG7729, &UA67198},
	{SG7733, &UA2018498},
	{SG7734, &UA2018577},
	{SG7922, &UA370868},
	{SG8395, &UA32223},
	{SG8930, &UA2733563},
	{SG9127, &UA59094},
	{SG9142, &UA2030013},
	{SG9305, &UA2698358},
	{SG9618, &UA1996933},
	{SG9623, &UA882716},
	{SG9700, &UA2196739},
	{SG9712, &UA2557052},
	{SG9714, &UA2766053},
	{SG9762, &UA2937699},
	{SG9789, &UA213463},
	{SG9790, &UA1940728},
	{SG9948, &UA458745},
	{SG9949, &UA458766},
	{SG9950, &UA458837},
	{SG9951, &UA459016},
	{SG10066, &UA909272},
	{SG10067, &UA2595580},
	{SG10068, &UA3100227},
	{SG10141, &UA2026043},
	{SG10150, &UA643771},
	{SG10187, &UA684998},
	{SG10196, &UA1996373},
	{SG10287, &UA2197297},
	{SG10293, &UA1962912},
	{SG10294, &UA1994574},
	{SG10323, &UA3015610},
	{SG10327, &UA458864},
	{SG10328, &UA459055},
	{SG10415, &UA1912516},
	{SG10511, &UA2452103},
	{SG10562, &UA2781010},
	{SG10584, &UA2385684},
	{SG10598, &UA2962224},
	{SG10621, &UA2715502},
	{SG10685, &UA900428},
	{SG10694, &UA458715},
	{SG10695, &UA458755},
	{SG10805, &UA2201672},
	{SG10829, &UA670230},
	{SG10840, &UA2230106},
	{SG10844, &UA571939},
	{SG10845, &UA2024684},
	{SG10855, &UA2580151},
	{SG10858, &UA458818},
	{SG10878, &UA2692398},
	{SG10898, &UA2026040},
	{SG10914, &UA724409},
	{SG10992, &UA1948318},
	{SG10993, &UA1949809},
	{SG11014, &UA2771216},
	{SG11025, &UA2032108},
	{SG11069, &UA2879240},
	{SG11094, &UA567327},
	{SG11095, &UA2019832},
	{SG11096, &UA2020508},
	{SG11097, &UA2025967},
	{SG11185, &UA719900},
	{SG11209, &UA883702},
	{SG11229, &UA2031062},
	{SG11240, &UA2781608},
	{SG11260, &UA2032610},
	{SG11306, &UA2840196},
	{SG11476, &UA2733110},
	{SG11588, &UA2991266},
	{SG11741, &UA1963683},
	{SG11776, &UA744306},
	{SG11845, &UA2863673},
	{SG11863, &UA2029641},
	{SG11910, &UA2023710},
	{SG11985, &UA2030009},
	{SG11992, &UA459005},
	{SG12100, &UA2019729},
	{SG12132, &UA2019911},
	{SG12254, &UA2022481},
	{SG12255, &UA3117118},
	{SG12450, &UA2022089},
	{SG12469, &UA2021327},
	{SG12491, &UA438848},
	{SG12520, &UA2026849},
	{NULL}};
const Segmentdata DAndroidVersion2_2_3[] = {
	{SG5600, &UA7498},
	{SG6627, &UA351130},
	{SG6644, &UA2025481},
	{SG6648, &UA2502446},
	{SG6656, &UA2596364},
	{SG6663, &UA2811330},
	{SG6665, &UA2816930},
	{SG7152, &UA1988041},
	{SG7775, &UA369299},
	{SG7776, &UA576299},
	{SG7777, &UA633481},
	{SG7781, &UA662827},
	{SG7782, &UA746472},
	{SG7784, &UA750411},
	{SG7786, &UA786989},
	{SG7787, &UA2021003},
	{SG7789, &UA2021401},
	{SG8143, &UA625925},
	{SG8144, &UA633252},
	{SG8145, &UA2032265},
	{SG8414, &UA625516},
	{SG8415, &UA631666},
	{SG8416, &UA2025357},
	{SG8417, &UA2027190},
	{SG8418, &UA2029312},
	{SG8419, &UA2030632},
	{SG8421, &UA2030669},
	{SG8922, &UA2028728},
	{SG9656, &UA771790},
	{SG9658, &UA2474651},
	{SG9906, &UA599899},
	{SG10177, &UA2025039},
	{SG10182, &UA1959776},
	{SG10183, &UA2029850},
	{SG10344, &UA2023046},
	{SG10388, &UA2021465},
	{SG10687, &UA1954676},
	{SG10689, &UA2814925},
	{SG10801, &UA2238375},
	{SG10817, &UA2031931},
	{SG10820, &UA2546151},
	{SG11181, &UA2028568},
	{SG12331, &UA559249},
	{NULL}};
const Segmentdata DAndroidVersion2_3_2[] = {
	{SG4795, &UA10851},
	{SG4796, &UA16617},
	{SG4797, &UA2018317},
	{SG5152, &UA180924},
	{SG5155, &UA76379},
	{SG5172, &UA2018678},
	{SG5596, &UA18196},
	{SG5597, &UA18215},
	{SG5598, &UA18281},
	{SG5599, &UA18310},
	{SG5601, &UA18319},
	{SG5602, &UA18324},
	{SG5603, &UA18330},
	{SG5604, &UA18290},
	{SG5605, &UA24592},
	{SG5606, &UA48191},
	{SG5607, &UA57399},
	{SG5608, &UA57498},
	{SG5609, &UA59575},
	{SG5611, &UA61197},
	{SG5616, &UA74912},
	{SG5620, &UA189799},
	{SG5621, &UA371303},
	{SG6061, &UA171684},
	{SG6065, &UA389288},
	{SG6067, &UA433615},
	{SG6079, &UA752158},
	{SG6080, &UA753331},
	{SG6086, &UA1863883},
	{SG6089, &UA2018559},
	{SG6101, &UA2948906},
	{SG6435, &UA169749},
	{SG6695, &UA18002},
	{SG6697, &UA24565},
	{SG6698, &UA41433},
	{SG6705, &UA57163},
	{SG6708, &UA57198},
	{SG6709, &UA57217},
	{SG6711, &UA57224},
	{SG6712, &UA90548},
	{SG6864, &UA20746},
	{SG6865, &UA20756},
	{SG6866, &UA20772},
	{SG6867, &UA23503},
	{SG6868, &UA24561},
	{SG6869, &UA59056},
	{SG6870, &UA534850},
	{SG7010, &UA805792},
	{SG7012, &UA2018284},
	{SG7479, &UA169577},
	{SG7480, &UA2603796},
	{SG7910, &UA68466},
	{SG7911, &UA205338},
	{SG7912, &UA253905},
	{SG7913, &UA554286},
	{SG7997, &UA171212},
	{SG8001, &UA64115},
	{SG8004, &UA67525},
	{SG8155, &UA18535},
	{SG8157, &UA41909},
	{SG8158, &UA41919},
	{SG8159, &UA2023601},
	{SG8161, &UA2360090},
	{SG8272, &UA18477},
	{SG8273, &UA57706},
	{SG8274, &UA57710},
	{SG8275, &UA57716},
	{SG8276, &UA2018539},
	{SG8394, &UA60937},
	{SG8439, &UA302859},
	{SG8500, &UA170486},
	{SG9231, &UA169560},
	{SG9235, &UA304436},
	{SG9236, &UA782411},
	{SG9239, &UA1865574},
	{SG9389, &UA20818},
	{SG9390, &UA20819},
	{SG9391, &UA20821},
	{SG9392, &UA90763},
	{SG9393, &UA2018443},
	{SG9406, &UA2026434},
	{SG9424, &UA430065},
	{SG9521, &UA20759},
	{SG9522, &UA73055},
	{SG9566, &UA2237758},
	{SG9575, &UA43370},
	{SG9576, &UA2373860},
	{SG9873, &UA302128},
	{SG9874, &UA304740},
	{SG10394, &UA1891280},
	{SG10628, &UA1933554},
	{SG10636, &UA230022},
	{SG10637, &UA631893},
	{SG10658, &UA67607},
	{SG10819, &UA2018887},
	{SG10956, &UA302882},
	{SG10971, &UA857232},
	{SG11180, &UA917217},
	{SG11182, &UA2019118},
	{SG11249, &UA1861081},
	{SG11615, &UA2018328},
	{SG11736, &UA2019444},
	{SG11947, &UA2018313},
	{SG12091, &UA17122},
	{SG12739, &UA2018278},
	{NULL}};
const Segmentdata DAndroidVersion2_3_3[] = {
	{SG4716, &UA22295},
	{SG4717, &UA23196},
	{SG4718, &UA23266},
	{SG4719, &UA24693},
	{SG4720, &UA46261},
	{SG4721, &UA46331},
	{SG4722, &UA46333},
	{SG4723, &UA46345},
	{SG4724, &UA46399},
	{SG4725, &UA46409},
	{SG4726, &UA46412},
	{SG4727, &UA46430},
	{SG4728, &UA46441},
	{SG4729, &UA47888},
	{SG4730, &UA59771},
	{SG4731, &UA59776},
	{SG4732, &UA59787},
	{SG4733, &UA59796},
	{SG4734, &UA59803},
	{SG4735, &UA70081},
	{SG4736, &UA72692},
	{SG4737, &UA74454},
	{SG4738, &UA195277},
	{SG4739, &UA237718},
	{SG4740, &UA371202},
	{SG4741, &UA395125},
	{SG4742, &UA683577},
	{SG4743, &UA893337},
	{SG4744, &UA1931925},
	{SG4745, &UA1935156},
	{SG4746, &UA1983892},
	{SG4747, &UA2020808},
	{SG4748, &UA2022780},
	{SG4749, &UA2026657},
	{SG4750, &UA2652391},
	{SG4799, &UA24165},
	{SG4800, &UA42324},
	{SG4801, &UA42338},
	{SG4802, &UA42361},
	{SG4803, &UA58241},
	{SG4804, &UA79545},
	{SG4805, &UA211788},
	{SG4806, &UA740845},
	{SG4807, &UA2021659},
	{SG4808, &UA2257377},
	{SG4818, &UA24754},
	{SG4820, &UA33155},
	{SG4821, &UA43410},
	{SG4823, &UA43418},
	{SG4824, &UA43433},
	{SG4826, &UA43438},
	{SG4828, &UA43439},
	{SG4829, &UA43454},
	{SG4831, &UA43461},
	{SG4832, &UA43499},
	{SG4833, &UA43505},
	{SG4834, &UA59349},
	{SG4835, &UA59353},
	{SG4836, &UA59354},
	{SG4837, &UA59364},
	{SG4838, &UA59367},
	{SG4839, &UA59394},
	{SG4841, &UA59465},
	{SG4843, &UA60873},
	{SG4845, &UA70666},
	{SG4846, &UA72962},
	{SG4847, &UA77048},
	{SG4848, &UA193259},
	{SG4849, &UA370010},
	{SG4850, &UA371105},
	{SG4852, &UA558034},
	{SG4854, &UA664385},
	{SG4856, &UA673956},
	{SG4858, &UA731099},
	{SG4859, &UA1966402},
	{SG4860, &UA2019853},
	{SG4861, &UA2022632},
	{SG4862, &UA2030784},
	{SG4863, &UA2590520},
	{SG4880, &UA10835},
	{SG4881, &UA16614},
	{SG4882, &UA20899},
	{SG4883, &UA23645},
	{SG4884, &UA24291},
	{SG4885, &UA24614},
	{SG4886, &UA39563},
	{SG4887, &UA41071},
	{SG4888, &UA52922},
	{SG4889, &UA52963},
	{SG4890, &UA52987},
	{SG4891, &UA53055},
	{SG4892, &UA53068},
	{SG4893, &UA56535},
	{SG4895, &UA56747},
	{SG4897, &UA58514},
	{SG4899, &UA59547},
	{SG4902, &UA73307},
	{SG4904, &UA76866},
	{SG4906, &UA78293},
	{SG4909, &UA179243},
	{SG4911, &UA190147},
	{SG4916, &UA369237},
	{SG4918, &UA535299},
	{SG4920, &UA551679},
	{SG4921, &UA570973},
	{SG4926, &UA714029},
	{SG4929, &UA1987154},
	{SG4933, &UA2030523},
	{SG4934, &UA2933892},
	{SG5156, &UA68742},
	{SG5157, &UA2019264},
	{SG5185, &UA47654},
	{SG5186, &UA60558},
	{SG5187, &UA90783},
	{SG5188, &UA220513},
	{SG5190, &UA680393},
	{SG5191, &UA2033077},
	{SG5192, &UA2307375},
	{SG5193, &UA23819},
	{SG5194, &UA38135},
	{SG5195, &UA40852},
	{SG5196, &UA51769},
	{SG5197, &UA51751},
	{SG5198, &UA53166},
	{SG5199, &UA69626},
	{SG5200, &UA182536},
	{SG5201, &UA436815},
	{SG5202, &UA1895863},
	{SG5203, &UA1992063},
	{SG5206, &UA2019638},
	{SG5209, &UA2025466},
	{SG5211, &UA2787230},
	{SG5214, &UA18614},
	{SG5215, &UA18619},
	{SG5216, &UA18620},
	{SG5217, &UA22345},
	{SG5218, &UA22344},
	{SG5219, &UA23179},
	{SG5220, &UA32779},
	{SG5221, &UA41950},
	{SG5222, &UA42015},
	{SG5223, &UA57815},
	{SG5224, &UA57832},
	{SG5225, &UA57856},
	{SG5226, &UA57866},
	{SG5227, &UA59374},
	{SG5228, &UA77054},
	{SG5229, &UA190356},
	{SG5230, &UA370895},
	{SG5231, &UA686938},
	{SG5232, &UA908546},
	{SG5233, &UA2215830},
	{SG5234, &UA2320106},
	{SG5235, &UA2445865},
	{SG5265, &UA36878},
	{SG5266, &UA47479},
	{SG5268, &UA333644},
	{SG5271, &UA370579},
	{SG5274, &UA377864},
	{SG5278, &UA447133},
	{SG5280, &UA450112},
	{SG5285, &UA557432},
	{SG5288, &UA571281},
	{SG5292, &UA624101},
	{SG5295, &UA639276},
	{SG5298, &UA649727},
	{SG5301, &UA659876},
	{SG5304, &UA667806},
	{SG5308, &UA695397},
	{SG5311, &UA699701},
	{SG5315, &UA702244},
	{SG5317, &UA703271},
	{SG5336, &UA713281},
	{SG5340, &UA745410},
	{SG5342, &UA754704},
	{SG5343, &UA777892},
	{SG5347, &UA822541},
	{SG5349, &UA853935},
	{SG5350, &UA1891919},
	{SG5351, &UA1895725},
	{SG5353, &UA1898089},
	{SG5356, &UA1934501},
	{SG5357, &UA1940587},
	{SG5358, &UA1950980},
	{SG5359, &UA1956923},
	{SG5360, &UA1961822},
	{SG5365, &UA2018812},
	{SG5366, &UA2019047},
	{SG5368, &UA2019053},
	{SG5372, &UA2019970},
	{SG5374, &UA2019973},
	{SG5376, &UA2020020},
	{SG5380, &UA2020116},
	{SG5382, &UA2020198},
	{SG5386, &UA2020796},
	{SG5391, &UA2021047},
	{SG5393, &UA2021435},
	{SG5395, &UA2021573},
	{SG5398, &UA2021866},
	{SG5399, &UA2022107},
	{SG5401, &UA2022302},
	{SG5406, &UA2022892},
	{SG5407, &UA2022950},
	{SG5408, &UA2023156},
	{SG5409, &UA2023246},
	{SG5410, &UA2023600},
	{SG5411, &UA2023775},
	{SG5412, &UA2024998},
	{SG5413, &UA2026132},
	{SG5414, &UA2026210},
	{SG5415, &UA2026471},
	{SG5416, &UA2026535},
	{SG5424, &UA2028588},
	{SG5427, &UA2029154},
	{SG5432, &UA2030329},
	{SG5436, &UA2031958},
	{SG5438, &UA2032105},
	{SG5440, &UA2033012},
	{SG5444, &UA2033652},
	{SG5445, &UA2033992},
	{SG5448, &UA2034537},
	{SG5449, &UA2238801},
	{SG5450, &UA2337717},
	{SG5457, &UA2445561},
	{SG5471, &UA2545861},
	{SG5475, &UA2547069},
	{SG5478, &UA2547879},
	{SG5494, &UA2594005},
	{SG5528, &UA2641108},
	{SG5529, &UA2641348},
	{SG5530, &UA2643241},
	{SG5531, &UA2650934},
	{SG5532, &UA2656026},
	{SG5533, &UA2683374},
	{SG5534, &UA2684697},
	{SG5535, &UA2695534},
	{SG5536, &UA2727149},
	{SG5537, &UA2742670},
	{SG5543, &UA2763754},
	{SG5557, &UA2807591},
	{SG5559, &UA2808548},
	{SG5567, &UA2811049},
	{SG5570, &UA2811307},
	{SG5627, &UA2998137},
	{SG5643, &UA3069733},
	{SG5647, &UA3071124},
	{SG5652, &UA3084937},
	{SG6105, &UA32920},
	{SG6106, &UA41648},
	{SG6107, &UA41664},
	{SG6108, &UA41754},
	{SG6109, &UA41759},
	{SG6110, &UA47105},
	{SG6111, &UA47110},
	{SG6112, &UA47175},
	{SG6113, &UA47188},
	{SG6114, &UA47190},
	{SG6115, &UA57366},
	{SG6116, &UA57369},
	{SG6117, &UA57410},
	{SG6118, &UA57480},
	{SG6119, &UA57500},
	{SG6120, &UA57507},
	{SG6121, &UA57512},
	{SG6122, &UA57535},
	{SG6123, &UA57544},
	{SG6124, &UA60132},
	{SG6125, &UA60134},
	{SG6126, &UA60161},
	{SG6127, &UA71665},
	{SG6128, &UA76348},
	{SG6129, &UA79386},
	{SG6130, &UA90574},
	{SG6131, &UA196186},
	{SG6132, &UA819756},
	{SG6133, &UA2022066},
	{SG6134, &UA2028841},
	{SG6320, &UA32897},
	{SG6321, &UA47550},
	{SG6322, &UA47530},
	{SG6323, &UA47582},
	{SG6324, &UA47583},
	{SG6325, &UA48740},
	{SG6326, &UA60441},
	{SG6327, &UA76713},
	{SG6328, &UA77202},
	{SG6329, &UA351166},
	{SG6330, &UA665981},
	{SG6331, &UA838582},
	{SG6332, &UA1932163},
	{SG6333, &UA2022460},
	{SG6334, &UA2022757},
	{SG6335, &UA74109},
	{SG6336, &UA434029},
	{SG6337, &UA452708},
	{SG6338, &UA628100},
	{SG6339, &UA666726},
	{SG6340, &UA765518},
	{SG6341, &UA1953921},
	{SG6342, &UA2019159},
	{SG6343, &UA2021570},
	{SG6344, &UA2365608},
	{SG6345, &UA2366364},
	{SG6346, &UA2383238},
	{SG6347, &UA2393454},
	{SG6348, &UA2394494},
	{SG6349, &UA2758422},
	{SG6446, &UA20217},
	{SG6447, &UA46965},
	{SG6448, &UA46976},
	{SG6449, &UA46992},
	{SG6450, &UA46993},
	{SG6451, &UA47008},
	{SG6452, &UA47045},
	{SG6453, &UA47047},
	{SG6454, &UA60094},
	{SG6455, &UA70243},
	{SG6456, &UA70818},
	{SG6457, &UA73384},
	{SG6458, &UA378452},
	{SG6459, &UA653817},
	{SG6460, &UA711374},
	{SG6461, &UA1938540},
	{SG6490, &UA61575},
	{SG6491, &UA395079},
	{SG6492, &UA2022555},
	{SG6493, &UA2364262},
	{SG6572, &UA24508},
	{SG6574, &UA24509},
	{SG6576, &UA49851},
	{SG6578, &UA180323},
	{SG6580, &UA196842},
	{SG6587, &UA1886375},
	{SG6588, &UA1892071},
	{SG6589, &UA2020660},
	{SG6590, &UA2022007},
	{SG6591, &UA2024781},
	{SG6631, &UA11695},
	{SG6636, &UA40483},
	{SG6640, &UA55410},
	{SG6641, &UA65908},
	{SG6645, &UA32951},
	{SG6646, &UA46794},
	{SG6647, &UA46811},
	{SG6649, &UA49474},
	{SG6650, &UA59899},
	{SG6651, &UA59903},
	{SG6652, &UA59906},
	{SG6653, &UA59907},
	{SG6654, &UA59914},
	{SG6655, &UA59911},
	{SG6657, &UA70384},
	{SG6658, &UA79425},
	{SG6659, &UA90728},
	{SG6660, &UA195336},
	{SG6661, &UA378822},
	{SG6662, &UA703637},
	{SG6664, &UA1924394},
	{SG6666, &UA2030133},
	{SG6669, &UA46057},
	{SG6673, &UA48523},
	{SG6674, &UA48525},
	{SG6675, &UA59692},
	{SG6677, &UA59723},
	{SG6678, &UA61529},
	{SG6679, &UA61531},
	{SG6681, &UA61532},
	{SG6683, &UA61533},
	{SG6684, &UA61537},
	{SG6685, &UA61538},
	{SG6686, &UA61540},
	{SG6687, &UA61549},
	{SG6688, &UA61550},
	{SG6689, &UA191301},
	{SG6690, &UA196863},
	{SG6691, &UA197243},
	{SG6692, &UA204853},
	{SG6693, &UA210007},
	{SG6694, &UA217997},
	{SG6696, &UA325896},
	{SG6699, &UA738209},
	{SG6702, &UA1962412},
	{SG6704, &UA2018993},
	{SG6715, &UA32766},
	{SG6716, &UA33414},
	{SG6717, &UA38584},
	{SG6718, &UA40772},
	{SG6719, &UA56620},
	{SG6720, &UA56763},
	{SG6721, &UA74468},
	{SG6722, &UA195755},
	{SG6723, &UA713869},
	{SG6724, &UA725649},
	{SG6725, &UA812743},
	{SG6726, &UA2020311},
	{SG6727, &UA2020549},
	{SG6728, &UA2020577},
	{SG6729, &UA2021052},
	{SG6730, &UA2022944},
	{SG6731, &UA2442797},
	{SG6732, &UA2607619},
	{SG6733, &UA2607632},
	{SG6734, &UA2607910},
	{SG6735, &UA2607960},
	{SG6736, &UA2607970},
	{SG6737, &UA2608490},
	{SG6738, &UA2608670},
	{SG6739, &UA2608851},
	{SG6740, &UA2609752},
	{SG6741, &UA2610979},
	{SG6742, &UA2611323},
	{SG6743, &UA2611379},
	{SG6744, &UA2612184},
	{SG6745, &UA2615296},
	{SG6746, &UA2615623},
	{SG6747, &UA2615949},
	{SG6748, &UA2616119},
	{SG6749, &UA2616163},
	{SG6750, &UA2616394},
	{SG6751, &UA2616441},
	{SG6752, &UA2616452},
	{SG6753, &UA2616459},
	{SG6754, &UA2616520},
	{SG6755, &UA2616618},
	{SG6756, &UA2616984},
	{SG6757, &UA2617224},
	{SG6758, &UA2617338},
	{SG6759, &UA2617437},
	{SG6760, &UA2617465},
	{SG6761, &UA2618273},
	{SG6762, &UA2618274},
	{SG6763, &UA2618951},
	{SG6764, &UA2618959},
	{SG6765, &UA2619198},
	{SG6766, &UA2619292},
	{SG6767, &UA2619793},
	{SG6768, &UA2619937},
	{SG6769, &UA2620673},
	{SG6770, &UA2620856},
	{SG6771, &UA2621913},
	{SG6772, &UA2622397},
	{SG6773, &UA2622495},
	{SG6774, &UA2623351},
	{SG6775, &UA2624123},
	{SG6776, &UA2624216},
	{SG6777, &UA2624669},
	{SG6778, &UA2625433},
	{SG6779, &UA2625434},
	{SG6780, &UA2626147},
	{SG6781, &UA2626374},
	{SG6782, &UA2626526},
	{SG6783, &UA2626687},
	{SG6784, &UA2626708},
	{SG6785, &UA2626830},
	{SG6786, &UA2626975},
	{SG6787, &UA2628010},
	{SG6788, &UA2628024},
	{SG6789, &UA2629196},
	{SG6790, &UA2630840},
	{SG6791, &UA2631073},
	{SG6792, &UA2631322},
	{SG6793, &UA2633163},
	{SG6794, &UA2633376},
	{SG6943, &UA61293},
	{SG7000, &UA17984},
	{SG7001, &UA18017},
	{SG7002, &UA18023},
	{SG7003, &UA18035},
	{SG7004, &UA18047},
	{SG7005, &UA32995},
	{SG7006, &UA41417},
	{SG7007, &UA41464},
	{SG7008, &UA41443},
	{SG7009, &UA47198},
	{SG7011, &UA47248},
	{SG7013, &UA57170},
	{SG7014, &UA57181},
	{SG7015, &UA57190},
	{SG7016, &UA57196},
	{SG7017, &UA57215},
	{SG7018, &UA57232},
	{SG7019, &UA57239},
	{SG7020, &UA60209},
	{SG7021, &UA60225},
	{SG7022, &UA60210},
	{SG7023, &UA60228},
	{SG7024, &UA60240},
	{SG7025, &UA60249},
	{SG7026, &UA60272},
	{SG7027, &UA70330},
	{SG7028, &UA74836},
	{SG7029, &UA75460},
	{SG7030, &UA76037},
	{SG7031, &UA196530},
	{SG7032, &UA2020287},
	{SG7033, &UA2022936},
	{SG7034, &UA2030190},
	{SG7035, &UA33084},
	{SG7036, &UA378694},
	{SG7052, &UA76072},
	{SG7053, &UA76074},
	{SG7054, &UA247755},
	{SG7055, &UA371018},
	{SG7056, &UA382237},
	{SG7057, &UA613038},
	{SG7058, &UA631650},
	{SG7059, &UA857810},
	{SG7060, &UA891877},
	{SG7061, &UA925298},
	{SG7062, &UA1904227},
	{SG7063, &UA1954742},
	{SG7064, &UA1961232},
	{SG7065, &UA1969502},
	{SG7066, &UA2020268},
	{SG7067, &UA2021336},
	{SG7068, &UA2022488},
	{SG7069, &UA2025940},
	{SG7070, &UA2026827},
	{SG7071, &UA2028460},
	{SG7072, &UA2029201},
	{SG7073, &UA2029477},
	{SG7074, &UA2029923},
	{SG7075, &UA2030500},
	{SG7076, &UA2030702},
	{SG7077, &UA2030723},
	{SG7078, &UA2235593},
	{SG7079, &UA2238916},
	{SG7080, &UA2402919},
	{SG7081, &UA2480576},
	{SG7082, &UA2480752},
	{SG7083, &UA2526934},
	{SG7084, &UA2554124},
	{SG7085, &UA3127083},
	{SG7102, &UA20765},
	{SG7103, &UA43084},
	{SG7104, &UA43092},
	{SG7105, &UA43119},
	{SG7106, &UA43138},
	{SG7107, &UA48223},
	{SG7108, &UA48242},
	{SG7109, &UA49806},
	{SG7110, &UA58993},
	{SG7112, &UA59015},
	{SG7114, &UA59029},
	{SG7116, &UA59067},
	{SG7118, &UA59049},
	{SG7120, &UA59091},
	{SG7122, &UA61201},
	{SG7124, &UA61207},
	{SG7126, &UA70587},
	{SG7130, &UA316571},
	{SG7148, &UA47465},
	{SG7149, &UA47468},
	{SG7150, &UA47469},
	{SG7151, &UA47480},
	{SG7153, &UA47484},
	{SG7154, &UA47493},
	{SG7155, &UA47496},
	{SG7156, &UA60407},
	{SG7157, &UA60414},
	{SG7158, &UA60418},
	{SG7159, &UA70911},
	{SG7160, &UA71173},
	{SG7161, &UA71325},
	{SG7162, &UA71334},
	{SG7163, &UA72058},
	{SG7164, &UA179420},
	{SG7165, &UA183565},
	{SG7166, &UA193950},
	{SG7167, &UA245069},
	{SG7168, &UA313876},
	{SG7169, &UA576432},
	{SG7170, &UA623318},
	{SG7239, &UA196656},
	{SG7267, &UA177025},
	{SG7268, &UA178990},
	{SG7269, &UA192368},
	{SG7270, &UA206994},
	{SG7271, &UA704846},
	{SG7272, &UA711653},
	{SG7284, &UA11209},
	{SG7285, &UA23750},
	{SG7286, &UA24566},
	{SG7287, &UA45927},
	{SG7288, &UA46381},
	{SG7289, &UA51020},
	{SG7290, &UA53413},
	{SG7291, &UA77631},
	{SG7292, &UA180013},
	{SG7293, &UA188073},
	{SG7294, &UA189262},
	{SG7295, &UA312096},
	{SG7296, &UA417097},
	{SG7299, &UA680280},
	{SG7307, &UA2018491},
	{SG7309, &UA2018570},
	{SG7312, &UA2019260},
	{SG7315, &UA2019714},
	{SG7316, &UA2027036},
	{SG7317, &UA2031196},
	{SG7318, &UA2810303},
	{SG7319, &UA22231},
	{SG7320, &UA45508},
	{SG7321, &UA45513},
	{SG7322, &UA45587},
	{SG7323, &UA45662},
	{SG7324, &UA45677},
	{SG7325, &UA45730},
	{SG7326, &UA49332},
	{SG7327, &UA50027},
	{SG7328, &UA59592},
	{SG7329, &UA59607},
	{SG7330, &UA59595},
	{SG7331, &UA73704},
	{SG7332, &UA74381},
	{SG7333, &UA208934},
	{SG7334, &UA623254},
	{SG7335, &UA1977494},
	{SG7336, &UA2022607},
	{SG7337, &UA2828193},
	{SG7338, &UA43175},
	{SG7339, &UA43180},
	{SG7340, &UA43189},
	{SG7341, &UA43195},
	{SG7342, &UA59116},
	{SG7343, &UA59126},
	{SG7344, &UA59128},
	{SG7345, &UA59136},
	{SG7346, &UA59140},
	{SG7347, &UA59142},
	{SG7348, &UA77499},
	{SG7349, &UA212818},
	{SG7350, &UA42554},
	{SG7351, &UA42557},
	{SG7352, &UA42561},
	{SG7353, &UA42569},
	{SG7354, &UA58452},
	{SG7356, &UA58455},
	{SG7357, &UA58470},
	{SG7359, &UA59718},
	{SG7364, &UA220998},
	{SG7365, &UA1932824},
	{SG7367, &UA2444993},
	{SG7459, &UA19875},
	{SG7460, &UA23646},
	{SG7461, &UA42587},
	{SG7462, &UA42595},
	{SG7463, &UA42599},
	{SG7464, &UA45979},
	{SG7465, &UA48571},
	{SG7466, &UA61586},
	{SG7469, &UA714871},
	{SG7471, &UA2021049},
	{SG7472, &UA2021190},
	{SG7579, &UA1889694},
	{SG7587, &UA23674},
	{SG7589, &UA38818},
	{SG7591, &UA38819},
	{SG7593, &UA38848},
	{SG7595, &UA38924},
	{SG7597, &UA38928},
	{SG7599, &UA39215},
	{SG7601, &UA40572},
	{SG7604, &UA52561},
	{SG7605, &UA898876},
	{SG7606, &UA1996937},
	{SG7607, &UA2337606},
	{SG7608, &UA2476014},
	{SG7609, &UA2505000},
	{SG7611, &UA47423},
	{SG7612, &UA49513},
	{SG7613, &UA60342},
	{SG7614, &UA60345},
	{SG7615, &UA60349},
	{SG7616, &UA60353},
	{SG7617, &UA60378},
	{SG7618, &UA60383},
	{SG7619, &UA71193},
	{SG7620, &UA71856},
	{SG7621, &UA571468},
	{SG7622, &UA680126},
	{SG7623, &UA743061},
	{SG7624, &UA2023687},
	{SG7625, &UA2878035},
	{SG7626, &UA2024167},
	{SG7631, &UA33312},
	{SG7638, &UA42268},
	{SG7639, &UA42271},
	{SG7641, &UA42273},
	{SG7642, &UA42275},
	{SG7643, &UA42278},
	{SG7645, &UA238248},
	{SG7646, &UA545235},
	{SG7647, &UA565590},
	{SG7651, &UA2023006},
	{SG7712, &UA8684},
	{SG7713, &UA304882},
	{SG7714, &UA1868082},
	{SG7715, &UA2849825},
	{SG7831, &UA40204},
	{SG7834, &UA40221},
	{SG7836, &UA40275},
	{SG7838, &UA40294},
	{SG7840, &UA40273},
	{SG7842, &UA41026},
	{SG7844, &UA55474},
	{SG7846, &UA55487},
	{SG7848, &UA57939},
	{SG7850, &UA77845},
	{SG7852, &UA396110},
	{SG7853, &UA622712},
	{SG7854, &UA632913},
	{SG7855, &UA634177},
	{SG7856, &UA651872},
	{SG7857, &UA660649},
	{SG7858, &UA700167},
	{SG7859, &UA714613},
	{SG7860, &UA735941},
	{SG7862, &UA2020054},
	{SG7863, &UA2020534},
	{SG7866, &UA2933190},
	{SG7955, &UA60296},
	{SG7980, &UA58614},
	{SG7981, &UA70333},
	{SG7989, &UA47331},
	{SG7990, &UA223130},
	{SG7991, &UA248664},
	{SG7992, &UA632014},
	{SG7993, &UA644889},
	{SG7994, &UA675641},
	{SG7995, &UA1889213},
	{SG7996, &UA2020067},
	{SG8010, &UA18394},
	{SG8011, &UA41790},
	{SG8012, &UA43169},
	{SG8013, &UA57595},
	{SG8014, &UA57611},
	{SG8015, &UA191783},
	{SG8016, &UA567953},
	{SG8017, &UA1898555},
	{SG8018, &UA1984973},
	{SG8019, &UA2544428},
	{SG8028, &UA48281},
	{SG8030, &UA48284},
	{SG8032, &UA48289},
	{SG8034, &UA48292},
	{SG8036, &UA48299},
	{SG8038, &UA48285},
	{SG8040, &UA61262},
	{SG8042, &UA79454},
	{SG8044, &UA186325},
	{SG8046, &UA246970},
	{SG8050, &UA570930},
	{SG8131, &UA23429},
	{SG8132, &UA33171},
	{SG8133, &UA33172},
	{SG8134, &UA48868},
	{SG8135, &UA59398},
	{SG8136, &UA61400},
	{SG8137, &UA654370},
	{SG8138, &UA1901668},
	{SG8139, &UA2019957},
	{SG8140, &UA2022824},
	{SG8141, &UA2028310},
	{SG8142, &UA2475206},
	{SG8146, &UA59542},
	{SG8147, &UA2019018},
	{SG8153, &UA59858},
	{SG8154, &UA181120},
	{SG8156, &UA684985},
	{SG8196, &UA32986},
	{SG8199, &UA67734},
	{SG8200, &UA180570},
	{SG8201, &UA391537},
	{SG8202, &UA517888},
	{SG8203, &UA2720092},
	{SG8204, &UA41409},
	{SG8255, &UA24403},
	{SG8256, &UA2019049},
	{SG8257, &UA2021216},
	{SG8258, &UA33157},
	{SG8259, &UA69978},
	{SG8260, &UA239653},
	{SG8261, &UA667557},
	{SG8262, &UA2022593},
	{SG8263, &UA2022733},
	{SG8264, &UA2410047},
	{SG8267, &UA24506},
	{SG8268, &UA33250},
	{SG8271, &UA197498},
	{SG8316, &UA22362},
	{SG8318, &UA33057},
	{SG8319, &UA33060},
	{SG8321, &UA45823},
	{SG8322, &UA59623},
	{SG8323, &UA59626},
	{SG8324, &UA59634},
	{SG8325, &UA59635},
	{SG8326, &UA59644},
	{SG8327, &UA59666},
	{SG8328, &UA59642},
	{SG8329, &UA59688},
	{SG8330, &UA59693},
	{SG8331, &UA59705},
	{SG8332, &UA59707},
	{SG8333, &UA59709},
	{SG8334, &UA59714},
	{SG8335, &UA61087},
	{SG8336, &UA61088},
	{SG8337, &UA90779},
	{SG8338, &UA743371},
	{SG8339, &UA1929956},
	{SG8340, &UA2019297},
	{SG8341, &UA2019333},
	{SG8342, &UA2019779},
	{SG8343, &UA47759},
	{SG8344, &UA47765},
	{SG8345, &UA60661},
	{SG8346, &UA70214},
	{SG8347, &UA73838},
	{SG8348, &UA79764},
	{SG8349, &UA251989},
	{SG8350, &UA451498},
	{SG8351, &UA576358},
	{SG8352, &UA577463},
	{SG8353, &UA692006},
	{SG8354, &UA700795},
	{SG8355, &UA2027524},
	{SG8356, &UA2028751},
	{SG8387, &UA33237},
	{SG8388, &UA57776},
	{SG8389, &UA57791},
	{SG8390, &UA73469},
	{SG8391, &UA73619},
	{SG8392, &UA556673},
	{SG8393, &UA754653},
	{SG8396, &UA46847},
	{SG8397, &UA70838},
	{SG8398, &UA660825},
	{SG8399, &UA856505},
	{SG8400, &UA1973855},
	{SG8401, &UA2021199},
	{SG8420, &UA2385799},
	{SG8459, &UA24745},
	{SG8462, &UA180114},
	{SG8463, &UA248619},
	{SG8466, &UA576537},
	{SG8467, &UA704654},
	{SG8489, &UA33251},
	{SG8490, &UA8663},
	{SG8491, &UA38690},
	{SG8492, &UA52467},
	{SG8493, &UA52479},
	{SG8494, &UA52481},
	{SG8496, &UA79305},
	{SG8497, &UA378242},
	{SG8543, &UA61348},
	{SG8544, &UA61349},
	{SG8545, &UA79490},
	{SG8546, &UA79725},
	{SG8548, &UA250019},
	{SG8557, &UA575056},
	{SG8558, &UA744757},
	{SG8559, &UA753384},
	{SG8560, &UA768280},
	{SG8561, &UA1907227},
	{SG8562, &UA18481},
	{SG8563, &UA18482},
	{SG8564, &UA41853},
	{SG8565, &UA48353},
	{SG8566, &UA57720},
	{SG8567, &UA72434},
	{SG8568, &UA706909},
	{SG8569, &UA2025878},
	{SG8570, &UA2033619},
	{SG8603, &UA20381},
	{SG8605, &UA46946},
	{SG8607, &UA60059},
	{SG8609, &UA60068},
	{SG8672, &UA198504},
	{SG8673, &UA393281},
	{SG8674, &UA2018970},
	{SG8675, &UA2445176},
	{SG8677, &UA23020},
	{SG8721, &UA33457},
	{SG8722, &UA47995},
	{SG8723, &UA47997},
	{SG8724, &UA48001},
	{SG8725, &UA60932},
	{SG8726, &UA60934},
	{SG8727, &UA76235},
	{SG8728, &UA191108},
	{SG8729, &UA369608},
	{SG8730, &UA2020349},
	{SG8731, &UA51207},
	{SG8732, &UA51399},
	{SG8733, &UA737160},
	{SG8734, &UA1889460},
	{SG8735, &UA2018976},
	{SG8746, &UA67904},
	{SG8759, &UA557020},
	{SG8769, &UA69508},
	{SG8771, &UA241241},
	{SG8773, &UA2019461},
	{SG8792, &UA22286},
	{SG8793, &UA182399},
	{SG8794, &UA195922},
	{SG8795, &UA1938683},
	{SG8796, &UA2545590},
	{SG8802, &UA22944},
	{SG8803, &UA59684},
	{SG8804, &UA180087},
	{SG8805, &UA2019068},
	{SG8808, &UA2815896},
	{SG8912, &UA34161},
	{SG8913, &UA52706},
	{SG8914, &UA2018881},
	{SG8929, &UA60594},
	{SG8975, &UA33226},
	{SG8976, &UA43544},
	{SG8977, &UA43538},
	{SG8978, &UA59526},
	{SG8979, &UA74703},
	{SG8980, &UA609187},
	{SG8981, &UA626178},
	{SG8983, &UA1966039},
	{SG9085, &UA60741},
	{SG9091, &UA49207},
	{SG9093, &UA49965},
	{SG9094, &UA554393},
	{SG9099, &UA49733},
	{SG9100, &UA1905204},
	{SG9101, &UA2018550},
	{SG9165, &UA42991},
	{SG9166, &UA58886},
	{SG9167, &UA58888},
	{SG9168, &UA58889},
	{SG9169, &UA58892},
	{SG9170, &UA371308},
	{SG9171, &UA565836},
	{SG9172, &UA1914615},
	{SG9173, &UA2019825},
	{SG9174, &UA2021251},
	{SG9175, &UA2023175},
	{SG9176, &UA2413111},
	{SG9177, &UA2496802},
	{SG9196, &UA33252},
	{SG9225, &UA200839},
	{SG9226, &UA309176},
	{SG9227, &UA23270},
	{SG9246, &UA48447},
	{SG9247, &UA48448},
	{SG9248, &UA2019050},
	{SG9297, &UA48412},
	{SG9323, &UA33104},
	{SG9368, &UA176576},
	{SG9369, &UA179328},
	{SG9370, &UA187409},
	{SG9371, &UA306639},
	{SG9372, &UA369842},
	{SG9373, &UA377013},
	{SG9374, &UA1871943},
	{SG9376, &UA32924},
	{SG9398, &UA196480},
	{SG9399, &UA205710},
	{SG9400, &UA378704},
	{SG9401, &UA2018880},
	{SG9402, &UA47701},
	{SG9403, &UA47702},
	{SG9404, &UA70732},
	{SG9405, &UA616491},
	{SG9416, &UA251389},
	{SG9417, &UA638017},
	{SG9421, &UA32748},
	{SG9422, &UA69392},
	{SG9423, &UA616490},
	{SG9425, &UA2020121},
	{SG9426, &UA2027622},
	{SG9427, &UA2439857},
	{SG9428, &UA53164},
	{SG9429, &UA420781},
	{SG9514, &UA204357},
	{SG9515, &UA378767},
	{SG9516, &UA409304},
	{SG9517, &UA2023982},
	{SG9520, &UA568267},
	{SG9523, &UA43170},
	{SG9524, &UA43142},
	{SG9525, &UA43145},
	{SG9526, &UA43150},
	{SG9527, &UA43152},
	{SG9528, &UA59103},
	{SG9529, &UA59104},
	{SG9530, &UA1894227},
	{SG9531, &UA33113},
	{SG9532, &UA657735},
	{SG9542, &UA454641},
	{SG9548, &UA43523},
	{SG9549, &UA43526},
	{SG9550, &UA43529},
	{SG9551, &UA59505},
	{SG9552, &UA59507},
	{SG9553, &UA1896479},
	{SG9554, &UA2019459},
	{SG9589, &UA58832},
	{SG9590, &UA61085},
	{SG9592, &UA429982},
	{SG9593, &UA429998},
	{SG9594, &UA430001},
	{SG9599, &UA430131},
	{SG9601, &UA2023497},
	{SG9602, &UA2024534},
	{SG9603, &UA2027469},
	{SG9604, &UA2658102},
	{SG9605, &UA2658472},
	{SG9606, &UA2849895},
	{SG9609, &UA48421},
	{SG9610, &UA48422},
	{SG9620, &UA2018839},
	{SG9621, &UA181123},
	{SG9622, &UA2018710},
	{SG9624, &UA59009},
	{SG9625, &UA59063},
	{SG9626, &UA77396},
	{SG9633, &UA46823},
	{SG9634, &UA46824},
	{SG9635, &UA46830},
	{SG9636, &UA46834},
	{SG9637, &UA59931},
	{SG9638, &UA59932},
	{SG9639, &UA59933},
	{SG9640, &UA59934},
	{SG9641, &UA70593},
	{SG9642, &UA70641},
	{SG9643, &UA2202581},
	{SG9665, &UA43357},
	{SG9667, &UA43365},
	{SG9668, &UA60641},
	{SG9669, &UA76927},
	{SG9670, &UA78736},
	{SG9673, &UA2021494},
	{SG9701, &UA249382},
	{SG9702, &UA458799},
	{SG9703, &UA458883},
	{SG9704, &UA459042},
	{SG9705, &UA2030643},
	{SG9746, &UA641162},
	{SG9747, &UA59618},
	{SG9759, &UA250071},
	{SG9760, &UA458771},
	{SG9761, &UA458823},
	{SG9772, &UA359560},
	{SG9773, &UA576407},
	{SG9774, &UA609217},
	{SG9775, &UA629645},
	{SG9776, &UA2004009},
	{SG9777, &UA2023722},
	{SG9778, &UA2033848},
	{SG9779, &UA2169097},
	{SG9780, &UA2193495},
	{SG9781, &UA2532593},
	{SG9782, &UA2582454},
	{SG9783, &UA2782698},
	{SG9784, &UA2860897},
	{SG9785, &UA2958530},
	{SG9786, &UA2995345},
	{SG9787, &UA3015660},
	{SG9788, &UA3083091},
	{SG9799, &UA534058},
	{SG9800, &UA544950},
	{SG9808, &UA702862},
	{SG9809, &UA703893},
	{SG9810, &UA2019686},
	{SG9811, &UA2033721},
	{SG9824, &UA33103},
	{SG9825, &UA630806},
	{SG9829, &UA2619278},
	{SG9830, &UA204618},
	{SG9831, &UA458732},
	{SG9832, &UA458988},
	{SG9852, &UA230101},
	{SG9853, &UA318489},
	{SG9854, &UA397168},
	{SG9855, &UA421825},
	{SG9856, &UA623601},
	{SG9857, &UA851909},
	{SG9860, &UA2031883},
	{SG9861, &UA2812616},
	{SG9880, &UA51924},
	{SG9881, &UA421788},
	{SG9887, &UA458816},
	{SG9888, &UA458830},
	{SG9889, &UA458934},
	{SG9924, &UA202839},
	{SG9925, &UA2019790},
	{SG9926, &UA2020857},
	{SG9936, &UA181982},
	{SG9937, &UA2018592},
	{SG9945, &UA2753823},
	{SG9946, &UA2958650},
	{SG9947, &UA3088452},
	{SG9956, &UA202351},
	{SG9957, &UA682784},
	{SG9958, &UA2018761},
	{SG9960, &UA174775},
	{SG9961, &UA523047},
	{SG9962, &UA871416},
	{SG9963, &UA180564},
	{SG9964, &UA311440},
	{SG9965, &UA647248},
	{SG9966, &UA1869592},
	{SG10018, &UA194604},
	{SG10022, &UA194830},
	{SG10023, &UA754075},
	{SG10028, &UA204249},
	{SG10029, &UA554371},
	{SG10030, &UA571247},
	{SG10036, &UA214226},
	{SG10037, &UA225756},
	{SG10038, &UA245605},
	{SG10039, &UA397542},
	{SG10040, &UA625033},
	{SG10041, &UA626261},
	{SG10042, &UA754593},
	{SG10043, &UA2031037},
	{SG10044, &UA2520814},
	{SG10045, &UA2649616},
	{SG10059, &UA49721},
	{SG10060, &UA655447},
	{SG10061, &UA1953106},
	{SG10062, &UA2019846},
	{SG10063, &UA2027152},
	{SG10065, &UA2440535},
	{SG10070, &UA244112},
	{SG10071, &UA458772},
	{SG10072, &UA459007},
	{SG10087, &UA458717},
	{SG10088, &UA458995},
	{SG10089, &UA459040},
	{SG10120, &UA362900},
	{SG10121, &UA369590},
	{SG10122, &UA705684},
	{SG10123, &UA1884147},
	{SG10148, &UA246979},
	{SG10149, &UA2019460},
	{SG10158, &UA223986},
	{SG10159, &UA238616},
	{SG10160, &UA251099},
	{SG10162, &UA331546},
	{SG10164, &UA348577},
	{SG10166, &UA348844},
	{SG10167, &UA1955253},
	{SG10168, &UA2019560},
	{SG10169, &UA2531117},
	{SG10170, &UA2751919},
	{SG10219, &UA187755},
	{SG10243, &UA595668},
	{SG10245, &UA2203476},
	{SG10251, &UA241258},
	{SG10252, &UA458880},
	{SG10253, &UA458950},
	{SG10259, &UA308909},
	{SG10260, &UA2018434},
	{SG10288, &UA537945},
	{SG10295, &UA216043},
	{SG10296, &UA566528},
	{SG10303, &UA2797575},
	{SG10304, &UA3088754},
	{SG10307, &UA33389},
	{SG10311, &UA459024},
	{SG10312, &UA459046},
	{SG10320, &UA213669},
	{SG10321, &UA458688},
	{SG10322, &UA458884},
	{SG10329, &UA38394},
	{SG10332, &UA234678},
	{SG10333, &UA1902932},
	{SG10334, &UA1940457},
	{SG10336, &UA68324},
	{SG10339, &UA173623},
	{SG10340, &UA376451},
	{SG10362, &UA22919},
	{SG10363, &UA22920},
	{SG10364, &UA2021425},
	{SG10365, &UA220049},
	{SG10366, &UA235751},
	{SG10367, &UA247091},
	{SG10368, &UA379980},
	{SG10369, &UA458429},
	{SG10370, &UA622425},
	{SG10371, &UA2031056},
	{SG10372, &UA3070443},
	{SG10395, &UA90751},
	{SG10396, &UA221987},
	{SG10397, &UA236746},
	{SG10398, &UA251385},
	{SG10399, &UA901189},
	{SG10400, &UA2029246},
	{SG10401, &UA2488180},
	{SG10402, &UA2879707},
	{SG10419, &UA197883},
	{SG10420, &UA248870},
	{SG10421, &UA420918},
	{SG10422, &UA451998},
	{SG10423, &UA562258},
	{SG10424, &UA616185},
	{SG10425, &UA626770},
	{SG10426, &UA629112},
	{SG10427, &UA718227},
	{SG10428, &UA1936590},
	{SG10429, &UA2019601},
	{SG10430, &UA2024092},
	{SG10431, &UA2026463},
	{SG10432, &UA3091718},
	{SG10456, &UA191888},
	{SG10457, &UA196562},
	{SG10458, &UA1889966},
	{SG10459, &UA2019517},
	{SG10460, &UA2542822},
	{SG10461, &UA198216},
	{SG10462, &UA2019257},
	{SG10468, &UA458774},
	{SG10469, &UA458795},
	{SG10470, &UA459045},
	{SG10492, &UA880313},
	{SG10500, &UA2784824},
	{SG10503, &UA2793266},
	{SG10516, &UA199997},
	{SG10517, &UA220457},
	{SG10518, &UA248993},
	{SG10519, &UA628574},
	{SG10521, &UA631722},
	{SG10523, &UA1900202},
	{SG10525, &UA2031041},
	{SG10551, &UA197422},
	{SG10553, &UA1922570},
	{SG10586, &UA807535},
	{SG10587, &UA1980869},
	{SG10590, &UA458694},
	{SG10591, &UA458885},
	{SG10620, &UA33451},
	{SG10630, &UA42553},
	{SG10633, &UA452739},
	{SG10634, &UA2023085},
	{SG10666, &UA196020},
	{SG10667, &UA1942720},
	{SG10676, &UA624534},
	{SG10683, &UA3028112},
	{SG10697, &UA2772282},
	{SG10704, &UA2799252},
	{SG10725, &UA50034},
	{SG10727, &UA369061},
	{SG10728, &UA673251},
	{SG10729, &UA458778},
	{SG10730, &UA459119},
	{SG10737, &UA458862},
	{SG10738, &UA458882},
	{SG10740, &UA634192},
	{SG10742, &UA2597795},
	{SG10745, &UA1939711},
	{SG10746, &UA2018404},
	{SG10747, &UA2019178},
	{SG10748, &UA67899},
	{SG10749, &UA2021747},
	{SG10784, &UA2019244},
	{SG10786, &UA2205515},
	{SG10790, &UA47597},
	{SG10791, &UA47599},
	{SG10792, &UA2020315},
	{SG10798, &UA653090},
	{SG10822, &UA2018419},
	{SG10837, &UA2538063},
	{SG10842, &UA397331},
	{SG10843, &UA715136},
	{SG10854, &UA2944548},
	{SG10874, &UA2257458},
	{SG10883, &UA310659},
	{SG10891, &UA1994341},
	{SG10896, &UA175882},
	{SG10918, &UA458784},
	{SG10925, &UA2469777},
	{SG10930, &UA351581},
	{SG10932, &UA458906},
	{SG10933, &UA1967193},
	{SG10952, &UA753712},
	{SG10972, &UA2027563},
	{SG10996, &UA49813},
	{SG10997, &UA2019723},
	{SG10998, &UA859284},
	{SG10999, &UA3117653},
	{SG11003, &UA2426010},
	{SG11023, &UA2030734},
	{SG11034, &UA32945},
	{SG11047, &UA573886},
	{SG11048, &UA2019280},
	{SG11052, &UA624543},
	{SG11060, &UA2018752},
	{SG11067, &UA179830},
	{SG11068, &UA2727783},
	{SG11070, &UA2799370},
	{SG11081, &UA43478},
	{SG11082, &UA304600},
	{SG11083, &UA2019241},
	{SG11113, &UA671893},
	{SG11114, &UA2022150},
	{SG11123, &UA4147},
	{SG11124, &UA3049545},
	{SG11136, &UA180111},
	{SG11142, &UA2702524},
	{SG11150, &UA200229},
	{SG11152, &UA2023998},
	{SG11183, &UA48741},
	{SG11184, &UA49606},
	{SG11186, &UA2993997},
	{SG11187, &UA3079794},
	{SG11191, &UA787437},
	{SG11201, &UA2866458},
	{SG11207, &UA2961364},
	{SG11220, &UA2565295},
	{SG11221, &UA706293},
	{SG11226, &UA2853776},
	{SG11228, &UA738511},
	{SG11247, &UA905972},
	{SG11253, &UA2431150},
	{SG11261, &UA2867554},
	{SG11280, &UA2737926},
	{SG11285, &UA2260236},
	{SG11289, &UA59857},
	{SG11291, &UA310419},
	{SG11332, &UA636338},
	{SG11333, &UA2028555},
	{SG11334, &UA2475249},
	{SG11335, &UA761081},
	{SG11336, &UA2021980},
	{SG11361, &UA644225},
	{SG11373, &UA2832583},
	{SG11376, &UA776268},
	{SG11378, &UA2663425},
	{SG11384, &UA2587292},
	{SG11385, &UA2588294},
	{SG11386, &UA3116818},
	{SG11392, &UA3110401},
	{SG11393, &UA2232830},
	{SG11396, &UA2794662},
	{SG11397, &UA2771452},
	{SG11405, &UA2600282},
	{SG11437, &UA576332},
	{SG11438, &UA791298},
	{SG11457, &UA831729},
	{SG11458, &UA2866770},
	{SG11463, &UA671590},
	{SG11474, &UA2030733},
	{SG11477, &UA2776058},
	{SG11484, &UA2693159},
	{SG11485, &UA2732763},
	{SG11486, &UA2879138},
	{SG11487, &UA3103525},
	{SG11512, &UA2622958},
	{SG11513, &UA2658987},
	{SG11524, &UA2383386},
	{SG11526, &UA2816527},
	{SG11531, &UA2024374},
	{SG11543, &UA3018230},
	{SG11556, &UA2746391},
	{SG11561, &UA3092621},
	{SG11562, &UA2673409},
	{SG11578, &UA309791},
	{SG11582, &UA857065},
	{SG11593, &UA38124},
	{SG11594, &UA1904768},
	{SG11600, &UA45656},
	{SG11601, &UA699585},
	{SG11611, &UA2020105},
	{SG11616, &UA3007802},
	{SG11618, &UA2257906},
	{SG11619, &UA2257388},
	{SG11622, &UA2623884},
	{SG11627, &UA791272},
	{SG11628, &UA2998036},
	{SG11641, &UA2337903},
	{SG11646, &UA2384235},
	{SG11649, &UA2717874},
	{SG11662, &UA2469287},
	{SG11663, &UA2621125},
	{SG11664, &UA2624860},
	{SG11665, &UA2626240},
	{SG11669, &UA2032227},
	{SG11679, &UA836931},
	{SG11680, &UA642880},
	{SG11681, &UA1870787},
	{SG11682, &UA2018385},
	{SG11683, &UA810199},
	{SG11684, &UA3103167},
	{SG11690, &UA2562972},
	{SG11701, &UA2848225},
	{SG11720, &UA2552912},
	{SG11727, &UA2026544},
	{SG11755, &UA3062245},
	{SG11761, &UA2469288},
	{SG11779, &UA2029626},
	{SG11787, &UA2432687},
	{SG11791, &UA2946037},
	{SG11803, &UA2737622},
	{SG11835, &UA2994766},
	{SG11838, &UA2615294},
	{SG11839, &UA2624204},
	{SG11840, &UA2626026},
	{SG11842, &UA2023296},
	{SG11843, &UA1939830},
	{SG11844, &UA2033008},
	{SG11850, &UA2427855},
	{SG11859, &UA2864666},
	{SG11860, &UA68163},
	{SG11868, &UA2031906},
	{SG11873, &UA3003354},
	{SG11886, &UA3007839},
	{SG11887, &UA2504599},
	{SG11888, &UA2737859},
	{SG11897, &UA2660184},
	{SG11899, &UA1979333},
	{SG11900, &UA2410551},
	{SG11907, &UA2469880},
	{SG11928, &UA2558201},
	{SG11942, &UA608919},
	{SG11953, &UA52337},
	{SG11954, &UA2018992},
	{SG11973, &UA2257459},
	{SG11976, &UA2564827},
	{SG12001, &UA2669050},
	{SG12003, &UA195248},
	{SG12004, &UA2554656},
	{SG12020, &UA314214},
	{SG12021, &UA2018890},
	{SG12032, &UA3023348},
	{SG12035, &UA47015},
	{SG12054, &UA2667575},
	{SG12063, &UA2766013},
	{SG12064, &UA2984789},
	{SG12069, &UA3079444},
	{SG12076, &UA552110},
	{SG12077, &UA885842},
	{SG12078, &UA2028942},
	{SG12090, &UA2527596},
	{SG12103, &UA3080072},
	{SG12104, &UA3052036},
	{SG12107, &UA2554666},
	{SG12121, &UA40256},
	{SG12122, &UA1935761},
	{SG12124, &UA634198},
	{SG12125, &UA2028348},
	{SG12126, &UA2027641},
	{SG12130, &UA2616006},
	{SG12131, &UA2619379},
	{SG12157, &UA2025630},
	{SG12171, &UA2019690},
	{SG12173, &UA2020853},
	{SG12186, &UA38996},
	{SG12192, &UA2654395},
	{SG12209, &UA2032838},
	{SG12210, &UA2433065},
	{SG12211, &UA2455376},
	{SG12212, &UA2629871},
	{SG12213, &UA2630599},
	{SG12214, &UA2632003},
	{SG12215, &UA2635106},
	{SG12225, &UA2961363},
	{SG12226, &UA2963638},
	{SG12229, &UA2500482},
	{SG12232, &UA1963979},
	{SG12235, &UA2028790},
	{SG12237, &UA2652698},
	{SG12238, &UA2868246},
	{SG12243, &UA1875219},
	{SG12249, &UA2767687},
	{SG12250, &UA3044728},
	{SG12256, &UA258735},
	{SG12257, &UA1970068},
	{SG12259, &UA2029085},
	{SG12277, &UA2610837},
	{SG12278, &UA2617385},
	{SG12279, &UA2625000},
	{SG12286, &UA3072413},
	{SG12292, &UA1987703},
	{SG12293, &UA2032182},
	{SG12294, &UA2341198},
	{SG12309, &UA1940899},
	{SG12322, &UA2023059},
	{SG12323, &UA2337676},
	{SG12351, &UA2600617},
	{SG12359, &UA59358},
	{SG12360, &UA59468},
	{SG12370, &UA2866676},
	{SG12381, &UA3019701},
	{SG12382, &UA3091639},
	{SG12388, &UA2757571},
	{SG12393, &UA2022879},
	{SG12398, &UA2525676},
	{SG12405, &UA3013823},
	{SG12407, &UA3113526},
	{SG12409, &UA2019175},
	{SG12412, &UA902257},
	{SG12413, &UA2021684},
	{SG12417, &UA2502491},
	{SG12418, &UA2615127},
	{SG12419, &UA2621259},
	{SG12431, &UA2020167},
	{SG12432, &UA2021797},
	{SG12433, &UA2022294},
	{SG12445, &UA2594816},
	{SG12447, &UA2861618},
	{SG12453, &UA49853},
	{SG12459, &UA2991993},
	{SG12465, &UA3106671},
	{SG12467, &UA3118983},
	{SG12473, &UA2384660},
	{SG12493, &UA2443871},
	{SG12517, &UA2028906},
	{SG12523, &UA2633678},
	{SG12524, &UA2637419},
	{SG12543, &UA2957284},
	{SG12547, &UA2957776},
	{SG12548, &UA2957807},
	{SG12561, &UA3001250},
	{SG12571, &UA2991449},
	{SG12575, &UA3014798},
	{SG12576, &UA3048529},
	{SG12579, &UA3102756},
	{SG12589, &UA2026198},
	{SG12590, &UA2026622},
	{SG12596, &UA2866484},
	{SG12599, &UA3108109},
	{SG12607, &UA2026686},
	{SG12608, &UA2627510},
	{SG12612, &UA2857928},
	{SG12625, &UA3086400},
	{SG12626, &UA3089319},
	{SG12646, &UA2574031},
	{SG12651, &UA2843666},
	{SG12653, &UA1907394},
	{SG12654, &UA2023955},
	{SG12655, &UA2025686},
	{SG12656, &UA360157},
	{SG12657, &UA2546454},
	{SG12661, &UA49992},
	{SG12662, &UA78119},
	{SG12663, &UA2023030},
	{SG12664, &UA2024359},
	{SG12665, &UA2613110},
	{SG12666, &UA2620530},
	{SG12668, &UA2621954},
	{SG12675, &UA2609417},
	{SG12676, &UA2633091},
	{SG12677, &UA2619498},
	{SG12678, &UA2622422},
	{SG12679, &UA2629000},
	{SG12680, &UA2630478},
	{SG12681, &UA2635985},
	{SG12685, &UA881271},
	{SG12686, &UA2339683},
	{SG12687, &UA2616887},
	{SG12688, &UA2627043},
	{SG12693, &UA2028823},
	{SG12701, &UA2024948},
	{SG12703, &UA2032208},
	{SG12707, &UA2615423},
	{SG12711, &UA40699},
	{SG12712, &UA2025408},
	{SG12713, &UA2019498},
	{SG12720, &UA2023890},
	{SG12721, &UA2029143},
	{SG12725, &UA2445437},
	{SG12734, &UA804731},
	{SG12738, &UA2817107},
	{SG12744, &UA2027805},
	{SG12745, &UA2029223},
	{SG12752, &UA2033233},
	{SG12754, &UA2623056},
	{SG12782, &UA2021481},
	{SG12793, &UA37284},
	{SG12794, &UA2020522},
	{SG12798, &UA2032445},
	{SG12799, &UA2023715},
	{SG12800, &UA2024083},
	{SG12803, &UA2629434},
	{SG12809, &UA79794},
	{SG12812, &UA2019871},
	{SG12815, &UA2021736},
	{SG12826, &UA2611205},
	{SG12828, &UA2608234},
	{SG12834, &UA2613633},
	{SG12850, &UA76084},
	{SG12852, &UA2019703},
	{SG12853, &UA2023075},
	{SG12854, &UA2027253},
	{SG12857, &UA2608389},
	{SG12858, &UA2609302},
	{SG12867, &UA370432},
	{SG12873, &UA2612731},
	{SG12874, &UA2633866},
	{SG12880, &UA2633328},
	{SG12921, &UA45652},
	{SG12929, &UA72210},
	{SG12930, &UA73148},
	{SG12938, &UA459100},
	{SG12940, &UA2024971},
	{SG12946, &UA2618183},
	{NULL}};
const Segmentdata DAndroidVersion2_3_4[] = {
	{SG4751, &UA32890},
	{SG4752, &UA59749},
	{SG4753, &UA71385},
	{SG4754, &UA219902},
	{SG4755, &UA369437},
	{SG4756, &UA636774},
	{SG4757, &UA641571},
	{SG4758, &UA680540},
	{SG4759, &UA705866},
	{SG4760, &UA751509},
	{SG4761, &UA1967129},
	{SG4762, &UA2021403},
	{SG4763, &UA2022486},
	{SG4764, &UA2026856},
	{SG4765, &UA2032000},
	{SG4766, &UA2033453},
	{SG4767, &UA2640255},
	{SG4768, &UA2645383},
	{SG4769, &UA2821067},
	{SG4809, &UA42368},
	{SG4810, &UA48158},
	{SG4811, &UA49416},
	{SG4812, &UA58262},
	{SG4813, &UA70292},
	{SG4814, &UA76098},
	{SG4815, &UA79572},
	{SG4816, &UA235662},
	{SG4817, &UA240426},
	{SG4819, &UA413015},
	{SG4822, &UA627674},
	{SG4825, &UA645503},
	{SG4827, &UA654354},
	{SG4830, &UA710869},
	{SG4840, &UA903848},
	{SG4842, &UA1937631},
	{SG4844, &UA1973139},
	{SG4851, &UA2364138},
	{SG4853, &UA2463776},
	{SG4855, &UA2693646},
	{SG4857, &UA2821349},
	{SG4864, &UA251823},
	{SG4865, &UA347650},
	{SG4866, &UA643185},
	{SG4868, &UA673849},
	{SG4873, &UA1923163},
	{SG4874, &UA1933452},
	{SG4936, &UA45977},
	{SG4937, &UA46061},
	{SG4938, &UA70271},
	{SG4939, &UA73256},
	{SG4940, &UA202356},
	{SG4945, &UA2019809},
	{SG5162, &UA59574},
	{SG5163, &UA70138},
	{SG5164, &UA395943},
	{SG5165, &UA575429},
	{SG5166, &UA623057},
	{SG5167, &UA701306},
	{SG5168, &UA2021979},
	{SG5169, &UA2022451},
	{SG5170, &UA2033867},
	{SG5171, &UA2215938},
	{SG5204, &UA47675},
	{SG5205, &UA47686},
	{SG5207, &UA70202},
	{SG5208, &UA71688},
	{SG5210, &UA624073},
	{SG5212, &UA2021462},
	{SG5213, &UA2371110},
	{SG5243, &UA204347},
	{SG5244, &UA207222},
	{SG5247, &UA724581},
	{SG5249, &UA1931383},
	{SG5250, &UA2545725},
	{SG5251, &UA3084693},
	{SG5252, &UA70064},
	{SG5253, &UA71061},
	{SG5254, &UA776276},
	{SG5269, &UA32810},
	{SG5270, &UA48638},
	{SG5272, &UA48745},
	{SG5273, &UA49661},
	{SG5275, &UA49667},
	{SG5276, &UA49674},
	{SG5277, &UA70788},
	{SG5279, &UA72573},
	{SG5281, &UA74196},
	{SG5283, &UA75802},
	{SG5284, &UA76395},
	{SG5286, &UA78067},
	{SG5287, &UA78608},
	{SG5289, &UA369507},
	{SG5290, &UA623025},
	{SG5291, &UA623230},
	{SG5293, &UA623464},
	{SG5294, &UA625057},
	{SG5296, &UA625344},
	{SG5297, &UA625414},
	{SG5299, &UA627673},
	{SG5300, &UA628507},
	{SG5302, &UA629618},
	{SG5303, &UA630679},
	{SG5305, &UA635120},
	{SG5306, &UA636850},
	{SG5307, &UA671274},
	{SG5309, &UA674074},
	{SG5310, &UA677724},
	{SG5312, &UA678599},
	{SG5313, &UA683697},
	{SG5314, &UA695615},
	{SG5316, &UA722845},
	{SG5318, &UA725288},
	{SG5319, &UA740062},
	{SG5320, &UA826728},
	{SG5321, &UA842379},
	{SG5322, &UA904642},
	{SG5323, &UA1940167},
	{SG5324, &UA1940730},
	{SG5325, &UA1941357},
	{SG5326, &UA1962458},
	{SG5327, &UA1964322},
	{SG5328, &UA1971854},
	{SG5329, &UA2024942},
	{SG5330, &UA2029749},
	{SG5331, &UA2034202},
	{SG5332, &UA2585365},
	{SG5333, &UA2641708},
	{SG5335, &UA2808954},
	{SG5418, &UA49852},
	{SG5419, &UA70171},
	{SG5421, &UA245456},
	{SG5422, &UA328035},
	{SG5423, &UA332984},
	{SG5426, &UA355673},
	{SG5428, &UA369287},
	{SG5429, &UA369774},
	{SG5430, &UA370029},
	{SG5431, &UA370072},
	{SG5433, &UA430102},
	{SG5434, &UA437167},
	{SG5435, &UA437175},
	{SG5437, &UA552186},
	{SG5439, &UA623529},
	{SG5441, &UA650618},
	{SG5442, &UA651533},
	{SG5443, &UA656311},
	{SG5446, &UA691179},
	{SG5447, &UA699197},
	{SG5451, &UA701320},
	{SG5452, &UA715909},
	{SG5453, &UA722092},
	{SG5454, &UA737336},
	{SG5455, &UA738429},
	{SG5456, &UA762429},
	{SG5458, &UA766125},
	{SG5459, &UA775253},
	{SG5460, &UA788372},
	{SG5461, &UA799574},
	{SG5462, &UA1900413},
	{SG5463, &UA1928428},
	{SG5464, &UA1934131},
	{SG5465, &UA1936864},
	{SG5466, &UA1937930},
	{SG5467, &UA1941384},
	{SG5468, &UA1967783},
	{SG5469, &UA1971094},
	{SG5470, &UA1971894},
	{SG5472, &UA1975825},
	{SG5473, &UA1976175},
	{SG5474, &UA1978919},
	{SG5476, &UA1982957},
	{SG5477, &UA1983765},
	{SG5479, &UA1986920},
	{SG5480, &UA2019407},
	{SG5481, &UA2019478},
	{SG5482, &UA2019716},
	{SG5483, &UA2019826},
	{SG5484, &UA2020071},
	{SG5485, &UA2020248},
	{SG5486, &UA2020254},
	{SG5487, &UA2020302},
	{SG5488, &UA2020842},
	{SG5489, &UA2020846},
	{SG5490, &UA2020875},
	{SG5491, &UA2020900},
	{SG5492, &UA2020978},
	{SG5493, &UA2021010},
	{SG5495, &UA2021726},
	{SG5496, &UA2021935},
	{SG5497, &UA2022314},
	{SG5498, &UA2022315},
	{SG5499, &UA2022521},
	{SG5500, &UA2022645},
	{SG5501, &UA2023266},
	{SG5502, &UA2023279},
	{SG5503, &UA2023638},
	{SG5504, &UA2023883},
	{SG5505, &UA2023985},
	{SG5506, &UA2024002},
	{SG5507, &UA2024179},
	{SG5508, &UA2024338},
	{SG5509, &UA2024470},
	{SG5510, &UA2024528},
	{SG5511, &UA2024872},
	{SG5512, &UA2024883},
	{SG5513, &UA2024966},
	{SG5514, &UA2024975},
	{SG5515, &UA2025386},
	{SG5516, &UA2025389},
	{SG5517, &UA2025514},
	{SG5518, &UA2025622},
	{SG5519, &UA2025650},
	{SG5520, &UA2025718},
	{SG5521, &UA2026095},
	{SG5522, &UA2026123},
	{SG5523, &UA2026343},
	{SG5524, &UA2026676},
	{SG5525, &UA2026752},
	{SG5526, &UA2026979},
	{SG5527, &UA2027672},
	{SG5538, &UA2027682},
	{SG5539, &UA2028008},
	{SG5540, &UA2028066},
	{SG5541, &UA2028462},
	{SG5542, &UA2028539},
	{SG5544, &UA2029616},
	{SG5545, &UA2029755},
	{SG5546, &UA2030823},
	{SG5547, &UA2031449},
	{SG5548, &UA2031644},
	{SG5549, &UA2031961},
	{SG5550, &UA2032034},
	{SG5551, &UA2032230},
	{SG5552, &UA2032532},
	{SG5553, &UA2032632},
	{SG5563, &UA2033660},
	{SG5565, &UA2033799},
	{SG5568, &UA2034084},
	{SG5572, &UA2034228},
	{SG5575, &UA2034392},
	{SG5578, &UA2034545},
	{SG5581, &UA2221412},
	{SG5592, &UA2258446},
	{SG5593, &UA2317724},
	{SG5594, &UA2320330},
	{SG5595, &UA2410720},
	{SG5617, &UA2412757},
	{SG5618, &UA2412782},
	{SG5619, &UA2413401},
	{SG5622, &UA2442799},
	{SG5623, &UA2455886},
	{SG5624, &UA2475676},
	{SG5625, &UA2479690},
	{SG5629, &UA2494565},
	{SG5630, &UA2494994},
	{SG5634, &UA2542834},
	{SG5635, &UA2544096},
	{SG5636, &UA2544400},
	{SG5638, &UA2545654},
	{SG5639, &UA2545685},
	{SG5641, &UA2547625},
	{SG5649, &UA2553488},
	{SG5654, &UA2574391},
	{SG5655, &UA2589451},
	{SG5657, &UA2589883},
	{SG5661, &UA2595711},
	{SG5665, &UA2641449},
	{SG5667, &UA2641516},
	{SG5668, &UA2643332},
	{SG5671, &UA2643796},
	{SG5672, &UA2643857},
	{SG5675, &UA2652266},
	{SG5677, &UA2652332},
	{SG5679, &UA2653869},
	{SG5682, &UA2684383},
	{SG5684, &UA2684632},
	{SG5685, &UA2685020},
	{SG5687, &UA2685883},
	{SG5693, &UA2690956},
	{SG5700, &UA2722378},
	{SG5703, &UA2724463},
	{SG5704, &UA2726944},
	{SG5710, &UA2742803},
	{SG5712, &UA2743711},
	{SG5717, &UA2760605},
	{SG5721, &UA2775183},
	{SG5724, &UA2781494},
	{SG5739, &UA2810413},
	{SG5741, &UA2810520},
	{SG5747, &UA2812009},
	{SG5750, &UA2812557},
	{SG5753, &UA2815060},
	{SG5758, &UA2815829},
	{SG5760, &UA2815912},
	{SG5767, &UA2816515},
	{SG5770, &UA2816592},
	{SG5775, &UA2817136},
	{SG5778, &UA2817164},
	{SG5780, &UA2817269},
	{SG5798, &UA2836768},
	{SG5801, &UA2839776},
	{SG5806, &UA2889081},
	{SG5808, &UA2891757},
	{SG5809, &UA2950082},
	{SG5810, &UA2950713},
	{SG5811, &UA2958218},
	{SG5812, &UA2979442},
	{SG5813, &UA2979482},
	{SG5814, &UA2979740},
	{SG5815, &UA2980354},
	{SG5816, &UA2980837},
	{SG5817, &UA2981278},
	{SG5818, &UA2982979},
	{SG5819, &UA2983344},
	{SG5833, &UA3042237},
	{SG5834, &UA3042293},
	{SG5837, &UA3042858},
	{SG5838, &UA3042985},
	{SG5853, &UA3088505},
	{SG5856, &UA3097794},
	{SG5858, &UA3098360},
	{SG5878, &UA3098655},
	{SG5898, &UA3117440},
	{SG5899, &UA3117524},
	{SG5902, &UA3118739},
	{SG6102, &UA2032863},
	{SG6103, &UA2033832},
	{SG6104, &UA2174877},
	{SG6284, &UA47095},
	{SG6285, &UA47106},
	{SG6286, &UA47144},
	{SG6287, &UA47148},
	{SG6288, &UA49642},
	{SG6289, &UA49633},
	{SG6290, &UA49645},
	{SG6291, &UA49648},
	{SG6292, &UA60194},
	{SG6293, &UA70095},
	{SG6294, &UA70349},
	{SG6295, &UA70371},
	{SG6296, &UA70982},
	{SG6297, &UA74367},
	{SG6298, &UA75483},
	{SG6299, &UA76260},
	{SG6300, &UA252502},
	{SG6301, &UA368893},
	{SG6302, &UA368927},
	{SG6303, &UA369198},
	{SG6304, &UA457081},
	{SG6305, &UA593783},
	{SG6306, &UA624805},
	{SG6307, &UA637574},
	{SG6308, &UA658345},
	{SG6309, &UA659041},
	{SG6310, &UA690937},
	{SG6311, &UA1911093},
	{SG6312, &UA1970059},
	{SG6313, &UA2021384},
	{SG6314, &UA2022813},
	{SG6315, &UA2027210},
	{SG6316, &UA2028837},
	{SG6317, &UA2979075},
	{SG6318, &UA2988778},
	{SG6350, &UA47520},
	{SG6351, &UA47549},
	{SG6353, &UA49769},
	{SG6355, &UA49884},
	{SG6356, &UA70755},
	{SG6357, &UA70768},
	{SG6358, &UA75261},
	{SG6359, &UA76749},
	{SG6360, &UA79448},
	{SG6362, &UA424862},
	{SG6363, &UA454711},
	{SG6365, &UA623841},
	{SG6366, &UA624994},
	{SG6367, &UA627006},
	{SG6368, &UA628727},
	{SG6369, &UA630847},
	{SG6370, &UA634124},
	{SG6371, &UA637949},
	{SG6372, &UA639759},
	{SG6373, &UA656919},
	{SG6374, &UA659141},
	{SG6375, &UA659188},
	{SG6376, &UA681961},
	{SG6377, &UA696484},
	{SG6378, &UA755631},
	{SG6379, &UA856926},
	{SG6380, &UA1963584},
	{SG6381, &UA2021650},
	{SG6382, &UA2022597},
	{SG6383, &UA2022577},
	{SG6384, &UA2032655},
	{SG6385, &UA2357303},
	{SG6386, &UA2441644},
	{SG6387, &UA2641274},
	{SG6388, &UA2733320},
	{SG6389, &UA2758286},
	{SG6390, &UA70994},
	{SG6391, &UA249682},
	{SG6392, &UA368713},
	{SG6393, &UA370416},
	{SG6395, &UA440316},
	{SG6396, &UA540533},
	{SG6397, &UA542800},
	{SG6398, &UA609396},
	{SG6399, &UA625586},
	{SG6400, &UA643949},
	{SG6401, &UA655163},
	{SG6402, &UA666616},
	{SG6403, &UA682181},
	{SG6404, &UA684818},
	{SG6405, &UA693441},
	{SG6406, &UA702147},
	{SG6407, &UA751829},
	{SG6408, &UA754466},
	{SG6409, &UA804976},
	{SG6410, &UA826132},
	{SG6411, &UA837171},
	{SG6412, &UA847572},
	{SG6413, &UA849935},
	{SG6414, &UA891951},
	{SG6415, &UA1938642},
	{SG6416, &UA2004903},
	{SG6417, &UA2021718},
	{SG6418, &UA2021961},
	{SG6419, &UA2022774},
	{SG6420, &UA2023252},
	{SG6421, &UA2023333},
	{SG6422, &UA2023624},
	{SG6423, &UA2024830},
	{SG6424, &UA2026100},
	{SG6425, &UA2026745},
	{SG6426, &UA2027157},
	{SG6427, &UA2029992},
	{SG6428, &UA2031629},
	{SG6429, &UA2033417},
	{SG6430, &UA2034307},
	{SG6431, &UA2233814},
	{SG6432, &UA2319579},
	{SG6433, &UA2343746},
	{SG6434, &UA2352843},
	{SG6436, &UA2375875},
	{SG6437, &UA2378106},
	{SG6438, &UA2380895},
	{SG6439, &UA2393453},
	{SG6440, &UA2412583},
	{SG6441, &UA2764141},
	{SG6442, &UA2782932},
	{SG6443, &UA2808094},
	{SG6444, &UA2840502},
	{SG6445, &UA3044926},
	{SG6548, &UA76089},
	{SG6549, &UA458431},
	{SG6550, &UA576614},
	{SG6551, &UA623666},
	{SG6552, &UA624246},
	{SG6553, &UA625061},
	{SG6554, &UA625216},
	{SG6555, &UA628140},
	{SG6556, &UA629244},
	{SG6557, &UA633660},
	{SG6558, &UA662783},
	{SG6559, &UA665443},
	{SG6560, &UA669890},
	{SG6561, &UA703381},
	{SG6562, &UA708427},
	{SG6563, &UA728579},
	{SG6564, &UA745137},
	{SG6565, &UA761634},
	{SG6566, &UA802572},
	{SG6567, &UA1947321},
	{SG6568, &UA2021161},
	{SG6569, &UA2023731},
	{SG6628, &UA47373},
	{SG6629, &UA252239},
	{SG6630, &UA342466},
	{SG6632, &UA452669},
	{SG6633, &UA633743},
	{SG6634, &UA661241},
	{SG6635, &UA738430},
	{SG6637, &UA764991},
	{SG6638, &UA1965925},
	{SG6639, &UA2019763},
	{SG6700, &UA70806},
	{SG6701, &UA73839},
	{SG6703, &UA199101},
	{SG6706, &UA314548},
	{SG6707, &UA398288},
	{SG6710, &UA710339},
	{SG6795, &UA40091},
	{SG6796, &UA40128},
	{SG6797, &UA40151},
	{SG6798, &UA40159},
	{SG6799, &UA40164},
	{SG6800, &UA40177},
	{SG6801, &UA40485},
	{SG6802, &UA48038},
	{SG6803, &UA48046},
	{SG6804, &UA48050},
	{SG6805, &UA48061},
	{SG6806, &UA48069},
	{SG6807, &UA49186},
	{SG6808, &UA49187},
	{SG6809, &UA55709},
	{SG6810, &UA55717},
	{SG6811, &UA55719},
	{SG6812, &UA55722},
	{SG6813, &UA55723},
	{SG6814, &UA70322},
	{SG6815, &UA72017},
	{SG6816, &UA73665},
	{SG6817, &UA74486},
	{SG6818, &UA370541},
	{SG6819, &UA622442},
	{SG6820, &UA627485},
	{SG6821, &UA629248},
	{SG6822, &UA663111},
	{SG6823, &UA708609},
	{SG6824, &UA722299},
	{SG6825, &UA730049},
	{SG6826, &UA742829},
	{SG6827, &UA751246},
	{SG6828, &UA794113},
	{SG6829, &UA1948521},
	{SG6830, &UA2026675},
	{SG6831, &UA2836076},
	{SG6832, &UA72886},
	{SG6833, &UA73056},
	{SG6834, &UA78285},
	{SG6835, &UA254012},
	{SG6836, &UA354611},
	{SG6837, &UA369401},
	{SG6838, &UA371201},
	{SG6839, &UA453126},
	{SG6840, &UA625821},
	{SG6841, &UA625803},
	{SG6842, &UA628101},
	{SG6843, &UA633473},
	{SG6844, &UA637141},
	{SG6845, &UA641442},
	{SG6846, &UA672467},
	{SG6847, &UA673708},
	{SG6848, &UA674056},
	{SG6849, &UA686840},
	{SG6850, &UA698336},
	{SG6851, &UA706557},
	{SG6852, &UA734234},
	{SG6853, &UA761450},
	{SG6854, &UA830951},
	{SG6855, &UA1958953},
	{SG6856, &UA1979878},
	{SG6857, &UA2021058},
	{SG6858, &UA2022762},
	{SG6859, &UA2025562},
	{SG6860, &UA2027353},
	{SG6861, &UA2028378},
	{SG6862, &UA2543413},
	{SG6863, &UA2888893},
	{SG6883, &UA1917784},
	{SG6884, &UA1940735},
	{SG6885, &UA1951207},
	{SG6886, &UA2027321},
	{SG6887, &UA45825},
	{SG6888, &UA45982},
	{SG6889, &UA45983},
	{SG6890, &UA48507},
	{SG6891, &UA48513},
	{SG6892, &UA48515},
	{SG6893, &UA48517},
	{SG6894, &UA48519},
	{SG6895, &UA48526},
	{SG6896, &UA48528},
	{SG6903, &UA69893},
	{SG6905, &UA69904},
	{SG6907, &UA69911},
	{SG6909, &UA70360},
	{SG6913, &UA75502},
	{SG6918, &UA227938},
	{SG6919, &UA236691},
	{SG6922, &UA354047},
	{SG6923, &UA379536},
	{SG6924, &UA421199},
	{SG6925, &UA567157},
	{SG6926, &UA651744},
	{SG6927, &UA741035},
	{SG6928, &UA1933986},
	{SG6929, &UA1993427},
	{SG6930, &UA2019413},
	{SG6931, &UA2020118},
	{SG6932, &UA2021606},
	{SG6933, &UA2021611},
	{SG6934, &UA2543400},
	{SG6944, &UA2028195},
	{SG6945, &UA2411843},
	{SG6946, &UA2607591},
	{SG6947, &UA2607921},
	{SG6948, &UA2608273},
	{SG6949, &UA2609767},
	{SG6950, &UA2610393},
	{SG6951, &UA2615683},
	{SG6952, &UA2617715},
	{SG6956, &UA2620799},
	{SG6999, &UA2627486},
	{SG7111, &UA47824},
	{SG7113, &UA47825},
	{SG7115, &UA48328},
	{SG7117, &UA48329},
	{SG7119, &UA48333},
	{SG7121, &UA48334},
	{SG7123, &UA48336},
	{SG7125, &UA48337},
	{SG7127, &UA48330},
	{SG7128, &UA48862},
	{SG7129, &UA49986},
	{SG7131, &UA61289},
	{SG7132, &UA61290},
	{SG7133, &UA70088},
	{SG7134, &UA70680},
	{SG7135, &UA71808},
	{SG7137, &UA72473},
	{SG7139, &UA369238},
	{SG7140, &UA396941},
	{SG7142, &UA451505},
	{SG7143, &UA641356},
	{SG7179, &UA49686},
	{SG7180, &UA49688},
	{SG7181, &UA49694},
	{SG7182, &UA49696},
	{SG7190, &UA60273},
	{SG7195, &UA70421},
	{SG7197, &UA72477},
	{SG7200, &UA72834},
	{SG7206, &UA252017},
	{SG7208, &UA379931},
	{SG7210, &UA398611},
	{SG7212, &UA625300},
	{SG7255, &UA626496},
	{SG7256, &UA632236},
	{SG7257, &UA633716},
	{SG7258, &UA656628},
	{SG7259, &UA662124},
	{SG7260, &UA673468},
	{SG7261, &UA675141},
	{SG7262, &UA705496},
	{SG7263, &UA734056},
	{SG7264, &UA1902211},
	{SG7265, &UA1993510},
	{SG7266, &UA2030253},
	{SG7273, &UA48825},
	{SG7355, &UA48221},
	{SG7358, &UA48229},
	{SG7360, &UA48234},
	{SG7361, &UA49864},
	{SG7362, &UA49869},
	{SG7363, &UA49868},
	{SG7366, &UA70998},
	{SG7368, &UA74481},
	{SG7369, &UA251810},
	{SG7370, &UA253985},
	{SG7371, &UA368781},
	{SG7372, &UA428408},
	{SG7373, &UA625620},
	{SG7374, &UA688825},
	{SG7375, &UA751518},
	{SG7376, &UA814158},
	{SG7377, &UA844135},
	{SG7378, &UA2025143},
	{SG7379, &UA2027154},
	{SG7380, &UA2027940},
	{SG7381, &UA2028061},
	{SG7382, &UA2425970},
	{SG7383, &UA324302},
	{SG7384, &UA623004},
	{SG7385, &UA623590},
	{SG7386, &UA673897},
	{SG7387, &UA680551},
	{SG7388, &UA687996},
	{SG7390, &UA691270},
	{SG7392, &UA692229},
	{SG7394, &UA758821},
	{SG7398, &UA1982552},
	{SG7402, &UA2022350},
	{SG7403, &UA2030700},
	{SG7404, &UA2382605},
	{SG7405, &UA3079646},
	{SG7406, &UA48116},
	{SG7407, &UA49551},
	{SG7408, &UA49558},
	{SG7409, &UA77800},
	{SG7410, &UA78038},
	{SG7411, &UA350411},
	{SG7412, &UA368795},
	{SG7413, &UA368878},
	{SG7414, &UA370849},
	{SG7415, &UA630394},
	{SG7416, &UA631750},
	{SG7417, &UA633062},
	{SG7418, &UA633299},
	{SG7419, &UA633731},
	{SG7420, &UA658749},
	{SG7421, &UA662982},
	{SG7422, &UA670306},
	{SG7423, &UA686175},
	{SG7424, &UA705438},
	{SG7425, &UA714738},
	{SG7426, &UA742125},
	{SG7427, &UA779582},
	{SG7428, &UA1976891},
	{SG7429, &UA1992957},
	{SG7430, &UA2029268},
	{SG7431, &UA2743586},
	{SG7445, &UA229384},
	{SG7448, &UA316054},
	{SG7453, &UA451687},
	{SG7458, &UA49256},
	{SG7467, &UA208995},
	{SG7468, &UA231072},
	{SG7470, &UA231271},
	{SG7473, &UA368666},
	{SG7474, &UA561449},
	{SG7475, &UA732137},
	{SG7476, &UA1893636},
	{SG7477, &UA1903186},
	{SG7478, &UA2020163},
	{SG7483, &UA194231},
	{SG7484, &UA194357},
	{SG7485, &UA315276},
	{SG7487, &UA321858},
	{SG7488, &UA327077},
	{SG7490, &UA371275},
	{SG7491, &UA392877},
	{SG7492, &UA419958},
	{SG7493, &UA550824},
	{SG7494, &UA557977},
	{SG7495, &UA637851},
	{SG7500, &UA915540},
	{SG7504, &UA2018693},
	{SG7506, &UA2018949},
	{SG7507, &UA2021380},
	{SG7508, &UA2029985},
	{SG7509, &UA2642764},
	{SG7510, &UA43188},
	{SG7511, &UA43200},
	{SG7512, &UA43201},
	{SG7513, &UA59129},
	{SG7514, &UA59131},
	{SG7515, &UA70848},
	{SG7516, &UA72122},
	{SG7517, &UA76662},
	{SG7518, &UA77847},
	{SG7519, &UA78219},
	{SG7520, &UA78269},
	{SG7521, &UA78435},
	{SG7522, &UA78436},
	{SG7523, &UA250527},
	{SG7524, &UA2019422},
	{SG7525, &UA2813924},
	{SG7526, &UA42567},
	{SG7528, &UA70586},
	{SG7529, &UA70876},
	{SG7531, &UA71130},
	{SG7533, &UA71129},
	{SG7535, &UA71375},
	{SG7537, &UA71516},
	{SG7539, &UA71553},
	{SG7540, &UA72222},
	{SG7542, &UA79167},
	{SG7545, &UA241930},
	{SG7546, &UA369240},
	{SG7548, &UA371112},
	{SG7550, &UA1946893},
	{SG7551, &UA2020109},
	{SG7552, &UA2020137},
	{SG7553, &UA2023294},
	{SG7554, &UA2454051},
	{SG7627, &UA48573},
	{SG7628, &UA69905},
	{SG7629, &UA70392},
	{SG7630, &UA319042},
	{SG7632, &UA321260},
	{SG7658, &UA48748},
	{SG7659, &UA48750},
	{SG7660, &UA49620},
	{SG7661, &UA459172},
	{SG7662, &UA622584},
	{SG7663, &UA622889},
	{SG7664, &UA623734},
	{SG7665, &UA625257},
	{SG7666, &UA700496},
	{SG7667, &UA902967},
	{SG7668, &UA909468},
	{SG7669, &UA910595},
	{SG7670, &UA910705},
	{SG7671, &UA911610},
	{SG7675, &UA225831},
	{SG7678, &UA564046},
	{SG7685, &UA2806711},
	{SG7803, &UA213927},
	{SG7805, &UA223886},
	{SG7807, &UA338505},
	{SG7808, &UA562114},
	{SG7809, &UA571732},
	{SG7820, &UA46215},
	{SG7826, &UA47401},
	{SG7827, &UA49761},
	{SG7828, &UA49764},
	{SG7829, &UA72964},
	{SG7830, &UA75398},
	{SG7832, &UA590243},
	{SG7833, &UA622926},
	{SG7835, &UA630101},
	{SG7837, &UA630539},
	{SG7839, &UA632610},
	{SG7841, &UA649392},
	{SG7843, &UA710703},
	{SG7845, &UA817786},
	{SG7847, &UA852551},
	{SG7849, &UA1948411},
	{SG7851, &UA1964460},
	{SG7869, &UA2507725},
	{SG7870, &UA2512815},
	{SG7871, &UA2558575},
	{SG7872, &UA2559454},
	{SG7873, &UA2642366},
	{SG7874, &UA2764305},
	{SG7875, &UA2807809},
	{SG7876, &UA2813544},
	{SG7877, &UA3117500},
	{SG7878, &UA210151},
	{SG7879, &UA337067},
	{SG7880, &UA369686},
	{SG7881, &UA571327},
	{SG7882, &UA806349},
	{SG7883, &UA2024802},
	{SG7884, &UA2521521},
	{SG7885, &UA3070184},
	{SG7894, &UA68332},
	{SG7898, &UA348469},
	{SG7899, &UA623504},
	{SG7900, &UA626084},
	{SG7901, &UA685245},
	{SG7902, &UA687096},
	{SG7903, &UA787955},
	{SG7904, &UA852401},
	{SG7905, &UA878566},
	{SG7906, &UA2022230},
	{SG7907, &UA2034449},
	{SG7941, &UA24486},
	{SG7942, &UA190780},
	{SG7943, &UA544543},
	{SG7944, &UA786902},
	{SG7945, &UA2659506},
	{SG7956, &UA49211},
	{SG7957, &UA357614},
	{SG7958, &UA369482},
	{SG7959, &UA646302},
	{SG7960, &UA678509},
	{SG7961, &UA719740},
	{SG7962, &UA738605},
	{SG7963, &UA751565},
	{SG7964, &UA754276},
	{SG7965, &UA2033528},
	{SG7966, &UA39416},
	{SG7967, &UA48838},
	{SG7968, &UA48841},
	{SG7969, &UA48843},
	{SG7970, &UA72752},
	{SG7971, &UA625025},
	{SG7972, &UA698868},
	{SG7973, &UA719219},
	{SG7974, &UA726148},
	{SG7975, &UA754687},
	{SG7976, &UA793804},
	{SG7977, &UA2021109},
	{SG7978, &UA2023180},
	{SG7979, &UA2023196},
	{SG7987, &UA68331},
	{SG7988, &UA2018757},
	{SG7998, &UA49896},
	{SG7999, &UA623021},
	{SG8000, &UA631717},
	{SG8002, &UA57116},
	{SG8003, &UA394949},
	{SG8005, &UA397223},
	{SG8006, &UA692362},
	{SG8007, &UA2032081},
	{SG8008, &UA2034481},
	{SG8009, &UA2364161},
	{SG8020, &UA368812},
	{SG8021, &UA369919},
	{SG8022, &UA626497},
	{SG8023, &UA626533},
	{SG8024, &UA640950},
	{SG8025, &UA655750},
	{SG8026, &UA675153},
	{SG8027, &UA744467},
	{SG8029, &UA797571},
	{SG8031, &UA810535},
	{SG8033, &UA864541},
	{SG8035, &UA1975754},
	{SG8037, &UA1977205},
	{SG8039, &UA1987400},
	{SG8041, &UA2023943},
	{SG8043, &UA2032061},
	{SG8045, &UA2033825},
	{SG8047, &UA2034583},
	{SG8048, &UA2233809},
	{SG8049, &UA2528429},
	{SG8051, &UA2578039},
	{SG8052, &UA2933231},
	{SG8086, &UA48012},
	{SG8087, &UA48014},
	{SG8088, &UA48015},
	{SG8089, &UA48016},
	{SG8090, &UA48017},
	{SG8091, &UA48019},
	{SG8092, &UA60964},
	{SG8093, &UA60965},
	{SG8094, &UA60966},
	{SG8095, &UA60968},
	{SG8096, &UA70226},
	{SG8097, &UA70350},
	{SG8098, &UA70378},
	{SG8099, &UA71053},
	{SG8100, &UA71369},
	{SG8101, &UA74123},
	{SG8102, &UA74916},
	{SG8103, &UA77900},
	{SG8104, &UA79307},
	{SG8105, &UA252229},
	{SG8106, &UA252428},
	{SG8107, &UA357880},
	{SG8108, &UA362332},
	{SG8109, &UA605229},
	{SG8110, &UA609306},
	{SG8111, &UA623091},
	{SG8112, &UA623098},
	{SG8113, &UA641467},
	{SG8114, &UA651752},
	{SG8115, &UA676766},
	{SG8116, &UA896127},
	{SG8117, &UA896381},
	{SG8118, &UA901297},
	{SG8119, &UA1905475},
	{SG8120, &UA2020798},
	{SG8121, &UA2028546},
	{SG8122, &UA2201776},
	{SG8123, &UA49320},
	{SG8124, &UA70118},
	{SG8125, &UA624336},
	{SG8126, &UA627326},
	{SG8127, &UA2020057},
	{SG8128, &UA32982},
	{SG8129, &UA409545},
	{SG8130, &UA2032380},
	{SG8160, &UA47291},
	{SG8162, &UA47298},
	{SG8163, &UA47299},
	{SG8164, &UA47302},
	{SG8165, &UA47307},
	{SG8166, &UA47309},
	{SG8168, &UA47311},
	{SG8169, &UA47325},
	{SG8171, &UA60297},
	{SG8173, &UA60303},
	{SG8174, &UA60306},
	{SG8175, &UA70562},
	{SG8177, &UA72232},
	{SG8179, &UA73594},
	{SG8180, &UA74124},
	{SG8181, &UA74136},
	{SG8182, &UA75958},
	{SG8183, &UA451226},
	{SG8184, &UA588530},
	{SG8185, &UA625747},
	{SG8186, &UA651300},
	{SG8187, &UA680767},
	{SG8188, &UA791405},
	{SG8189, &UA1957053},
	{SG8190, &UA1981857},
	{SG8191, &UA1983264},
	{SG8192, &UA1985538},
	{SG8193, &UA1987595},
	{SG8194, &UA1992919},
	{SG8195, &UA2022625},
	{SG8197, &UA2023639},
	{SG8198, &UA2031266},
	{SG8206, &UA41598},
	{SG8207, &UA368630},
	{SG8208, &UA368965},
	{SG8209, &UA369184},
	{SG8210, &UA370770},
	{SG8211, &UA624645},
	{SG8212, &UA654382},
	{SG8213, &UA726689},
	{SG8214, &UA750155},
	{SG8215, &UA795986},
	{SG8216, &UA849619},
	{SG8217, &UA1944739},
	{SG8218, &UA2022466},
	{SG8219, &UA2028925},
	{SG8220, &UA2031778},
	{SG8221, &UA2473589},
	{SG8222, &UA2502290},
	{SG8223, &UA2645548},
	{SG8224, &UA49787},
	{SG8240, &UA625027},
	{SG8242, &UA637336},
	{SG8245, &UA681229},
	{SG8246, &UA2024216},
	{SG8247, &UA2026057},
	{SG8248, &UA2027869},
	{SG8249, &UA2032577},
	{SG8250, &UA2442199},
	{SG8251, &UA2759074},
	{SG8265, &UA71481},
	{SG8266, &UA2977894},
	{SG8277, &UA321413},
	{SG8279, &UA451104},
	{SG8280, &UA609100},
	{SG8281, &UA609273},
	{SG8282, &UA611269},
	{SG8283, &UA623232},
	{SG8284, &UA623422},
	{SG8285, &UA650597},
	{SG8286, &UA773810},
	{SG8287, &UA854613},
	{SG8304, &UA622922},
	{SG8305, &UA625014},
	{SG8306, &UA691172},
	{SG8307, &UA697104},
	{SG8308, &UA725958},
	{SG8309, &UA773252},
	{SG8310, &UA774279},
	{SG8311, &UA2933188},
	{SG8377, &UA221341},
	{SG8378, &UA624935},
	{SG8379, &UA632131},
	{SG8380, &UA783709},
	{SG8381, &UA2807896},
	{SG8382, &UA2982447},
	{SG8402, &UA39721},
	{SG8403, &UA39724},
	{SG8404, &UA39726},
	{SG8405, &UA39749},
	{SG8406, &UA39750},
	{SG8407, &UA39753},
	{SG8408, &UA59678},
	{SG8409, &UA251922},
	{SG8410, &UA680457},
	{SG8411, &UA2032198},
	{SG8412, &UA56959},
	{SG8413, &UA2020746},
	{SG8422, &UA36858},
	{SG8423, &UA36860},
	{SG8424, &UA36862},
	{SG8425, &UA36866},
	{SG8426, &UA36867},
	{SG8427, &UA36871},
	{SG8429, &UA190862},
	{SG8431, &UA247694},
	{SG8434, &UA369216},
	{SG8437, &UA1898309},
	{SG8438, &UA2573622},
	{SG8448, &UA188401},
	{SG8449, &UA190831},
	{SG8450, &UA196516},
	{SG8451, &UA237796},
	{SG8452, &UA2021858},
	{SG8453, &UA2023013},
	{SG8454, &UA2440560},
	{SG8455, &UA57143},
	{SG8456, &UA57145},
	{SG8458, &UA197222},
	{SG8460, &UA203470},
	{SG8461, &UA209210},
	{SG8464, &UA369343},
	{SG8468, &UA681724},
	{SG8469, &UA692532},
	{SG8470, &UA745977},
	{SG8471, &UA769400},
	{SG8472, &UA771426},
	{SG8473, &UA774966},
	{SG8474, &UA775636},
	{SG8475, &UA1905379},
	{SG8476, &UA2019099},
	{SG8477, &UA2020391},
	{SG8478, &UA2020903},
	{SG8479, &UA2031830},
	{SG8480, &UA2355952},
	{SG8514, &UA459194},
	{SG8515, &UA615114},
	{SG8516, &UA623418},
	{SG8517, &UA626608},
	{SG8518, &UA1948885},
	{SG8519, &UA2408357},
	{SG8520, &UA47717},
	{SG8521, &UA47712},
	{SG8522, &UA60606},
	{SG8523, &UA33295},
	{SG8524, &UA420751},
	{SG8550, &UA396968},
	{SG8551, &UA677284},
	{SG8553, &UA689114},
	{SG8554, &UA1941029},
	{SG8555, &UA2020224},
	{SG8556, &UA2021571},
	{SG8593, &UA48382},
	{SG8594, &UA48399},
	{SG8595, &UA48404},
	{SG8596, &UA49604},
	{SG8597, &UA49605},
	{SG8598, &UA234721},
	{SG8599, &UA369290},
	{SG8600, &UA437137},
	{SG8601, &UA437162},
	{SG8602, &UA622444},
	{SG8604, &UA667825},
	{SG8606, &UA702970},
	{SG8608, &UA788381},
	{SG8610, &UA2019382},
	{SG8611, &UA2020944},
	{SG8612, &UA2021302},
	{SG8613, &UA2021782},
	{SG8614, &UA2258589},
	{SG8615, &UA2816785},
	{SG8643, &UA77514},
	{SG8644, &UA647234},
	{SG8645, &UA671867},
	{SG8646, &UA703341},
	{SG8647, &UA710573},
	{SG8648, &UA2024466},
	{SG8650, &UA2033876},
	{SG8676, &UA45890},
	{SG8678, &UA45891},
	{SG8679, &UA45981},
	{SG8680, &UA46134},
	{SG8681, &UA46163},
	{SG8682, &UA46660},
	{SG8683, &UA48126},
	{SG8686, &UA70197},
	{SG8687, &UA76499},
	{SG8688, &UA77031},
	{SG8689, &UA250055},
	{SG8690, &UA251908},
	{SG8691, &UA251994},
	{SG8693, &UA252032},
	{SG8694, &UA252534},
	{SG8695, &UA350210},
	{SG8696, &UA451235},
	{SG8697, &UA576525},
	{SG8698, &UA624781},
	{SG8699, &UA626446},
	{SG8701, &UA630523},
	{SG8702, &UA635509},
	{SG8703, &UA708058},
	{SG8704, &UA759192},
	{SG8705, &UA760285},
	{SG8706, &UA1913935},
	{SG8707, &UA2020464},
	{SG8708, &UA2020776},
	{SG8709, &UA2022880},
	{SG8710, &UA2022900},
	{SG8711, &UA2024173},
	{SG8712, &UA412735},
	{SG8713, &UA427508},
	{SG8714, &UA599843},
	{SG8715, &UA649282},
	{SG8716, &UA656227},
	{SG8717, &UA776286},
	{SG8718, &UA798876},
	{SG8719, &UA852234},
	{SG8720, &UA2024003},
	{SG8736, &UA32838},
	{SG8737, &UA47727},
	{SG8738, &UA47726},
	{SG8739, &UA47744},
	{SG8740, &UA47723},
	{SG8741, &UA49576},
	{SG8742, &UA49577},
	{SG8743, &UA49580},
	{SG8744, &UA49584},
	{SG8745, &UA60621},
	{SG8747, &UA70695},
	{SG8748, &UA73132},
	{SG8749, &UA74523},
	{SG8750, &UA254004},
	{SG8751, &UA368697},
	{SG8752, &UA368703},
	{SG8753, &UA624956},
	{SG8754, &UA636279},
	{SG8755, &UA658219},
	{SG8756, &UA659052},
	{SG8757, &UA709218},
	{SG8760, &UA1945022},
	{SG8761, &UA1966662},
	{SG8762, &UA1972801},
	{SG8763, &UA2019667},
	{SG8764, &UA2021525},
	{SG8766, &UA2022712},
	{SG8767, &UA2024646},
	{SG8768, &UA2512119},
	{SG8770, &UA2662882},
	{SG8772, &UA2730594},
	{SG8774, &UA348362},
	{SG8775, &UA370236},
	{SG8776, &UA622953},
	{SG8777, &UA625377},
	{SG8778, &UA637929},
	{SG8779, &UA662432},
	{SG8780, &UA690043},
	{SG8781, &UA698453},
	{SG8782, &UA728796},
	{SG8783, &UA1987530},
	{SG8785, &UA1990468},
	{SG8788, &UA2034606},
	{SG8806, &UA224647},
	{SG8807, &UA644580},
	{SG8820, &UA69100},
	{SG8849, &UA48691},
	{SG8851, &UA48693},
	{SG8852, &UA48698},
	{SG8853, &UA77924},
	{SG8854, &UA696122},
	{SG8856, &UA1903859},
	{SG8858, &UA1967565},
	{SG8859, &UA2022474},
	{SG8864, &UA2474087},
	{SG8865, &UA370998},
	{SG8877, &UA38685},
	{SG8878, &UA38688},
	{SG8881, &UA38696},
	{SG8883, &UA38702},
	{SG8885, &UA38718},
	{SG8887, &UA38719},
	{SG8890, &UA69774},
	{SG8891, &UA74887},
	{SG8893, &UA213435},
	{SG8895, &UA570773},
	{SG8896, &UA622448},
	{SG8915, &UA48897},
	{SG8916, &UA440700},
	{SG8917, &UA658437},
	{SG8918, &UA846680},
	{SG8919, &UA850987},
	{SG8920, &UA854655},
	{SG8921, &UA862576},
	{SG8923, &UA1992394},
	{SG8924, &UA2841497},
	{SG8925, &UA616492},
	{SG8926, &UA632672},
	{SG8927, &UA655942},
	{SG8928, &UA680353},
	{SG8931, &UA46935},
	{SG8932, &UA49747},
	{SG8933, &UA73564},
	{SG8934, &UA74930},
	{SG8935, &UA603129},
	{SG8936, &UA632830},
	{SG8937, &UA692432},
	{SG8938, &UA704714},
	{SG8939, &UA706033},
	{SG8940, &UA712647},
	{SG8941, &UA835012},
	{SG8942, &UA1939312},
	{SG8943, &UA2024036},
	{SG8944, &UA2024220},
	{SG8989, &UA208023},
	{SG8990, &UA235932},
	{SG8991, &UA2019346},
	{SG8992, &UA2026260},
	{SG8993, &UA2445297},
	{SG8994, &UA2474671},
	{SG8995, &UA49301},
	{SG8996, &UA636718},
	{SG9000, &UA2028726},
	{SG9004, &UA47352},
	{SG9005, &UA47353},
	{SG9006, &UA47358},
	{SG9007, &UA47359},
	{SG9008, &UA47363},
	{SG9009, &UA60326},
	{SG9010, &UA369325},
	{SG9011, &UA378427},
	{SG9012, &UA2019362},
	{SG9013, &UA2019950},
	{SG9027, &UA49969},
	{SG9033, &UA91023},
	{SG9034, &UA370804},
	{SG9035, &UA622380},
	{SG9036, &UA629797},
	{SG9037, &UA697711},
	{SG9045, &UA347137},
	{SG9046, &UA644983},
	{SG9047, &UA2020518},
	{SG9048, &UA2022740},
	{SG9049, &UA654849},
	{SG9050, &UA51406},
	{SG9051, &UA190327},
	{SG9052, &UA195793},
	{SG9053, &UA560209},
	{SG9054, &UA1886988},
	{SG9065, &UA369023},
	{SG9067, &UA623055},
	{SG9069, &UA2029490},
	{SG9071, &UA2031655},
	{SG9073, &UA2467276},
	{SG9074, &UA2734805},
	{SG9079, &UA68368},
	{SG9080, &UA211431},
	{SG9081, &UA58863},
	{SG9082, &UA572758},
	{SG9086, &UA33231},
	{SG9087, &UA625442},
	{SG9088, &UA2020022},
	{SG9089, &UA2588078},
	{SG9095, &UA192857},
	{SG9096, &UA197132},
	{SG9097, &UA558514},
	{SG9098, &UA767013},
	{SG9113, &UA39710},
	{SG9114, &UA39711},
	{SG9115, &UA39714},
	{SG9116, &UA59710},
	{SG9117, &UA69871},
	{SG9118, &UA70300},
	{SG9119, &UA193951},
	{SG9120, &UA234988},
	{SG9121, &UA369269},
	{SG9122, &UA636002},
	{SG9123, &UA2020078},
	{SG9124, &UA2021038},
	{SG9125, &UA2026456},
	{SG9126, &UA2031710},
	{SG9137, &UA33145},
	{SG9138, &UA640761},
	{SG9139, &UA1993195},
	{SG9140, &UA2021182},
	{SG9141, &UA2030237},
	{SG9143, &UA49995},
	{SG9144, &UA640914},
	{SG9145, &UA759725},
	{SG9146, &UA792228},
	{SG9147, &UA861149},
	{SG9148, &UA1907709},
	{SG9149, &UA48840},
	{SG9150, &UA48845},
	{SG9151, &UA70366},
	{SG9152, &UA78839},
	{SG9153, &UA322143},
	{SG9155, &UA787750},
	{SG9157, &UA1907853},
	{SG9160, &UA1975261},
	{SG9161, &UA2018908},
	{SG9162, &UA2020244},
	{SG9163, &UA2020492},
	{SG9164, &UA2022787},
	{SG9178, &UA2758858},
	{SG9179, &UA48357},
	{SG9180, &UA48362},
	{SG9181, &UA61325},
	{SG9182, &UA656437},
	{SG9183, &UA662165},
	{SG9184, &UA692575},
	{SG9185, &UA2020279},
	{SG9186, &UA2031575},
	{SG9187, &UA2809589},
	{SG9197, &UA403357},
	{SG9198, &UA718427},
	{SG9200, &UA767164},
	{SG9202, &UA2020731},
	{SG9203, &UA2032145},
	{SG9207, &UA48383},
	{SG9208, &UA48400},
	{SG9209, &UA49260},
	{SG9210, &UA49293},
	{SG9211, &UA61182},
	{SG9212, &UA72024},
	{SG9213, &UA236722},
	{SG9214, &UA566456},
	{SG9215, &UA622337},
	{SG9216, &UA649458},
	{SG9217, &UA652587},
	{SG9218, &UA663500},
	{SG9219, &UA713424},
	{SG9220, &UA902769},
	{SG9221, &UA2022384},
	{SG9222, &UA2024405},
	{SG9223, &UA2026085},
	{SG9224, &UA2027728},
	{SG9237, &UA246517},
	{SG9238, &UA370702},
	{SG9240, &UA633996},
	{SG9241, &UA1982033},
	{SG9242, &UA1985348},
	{SG9243, &UA2026169},
	{SG9244, &UA2028420},
	{SG9245, &UA2033261},
	{SG9249, &UA49398},
	{SG9250, &UA659175},
	{SG9251, &UA678912},
	{SG9252, &UA683451},
	{SG9253, &UA697816},
	{SG9254, &UA753540},
	{SG9255, &UA825975},
	{SG9256, &UA1903143},
	{SG9257, &UA2742330},
	{SG9258, &UA49878},
	{SG9259, &UA649772},
	{SG9260, &UA627378},
	{SG9261, &UA659892},
	{SG9262, &UA701239},
	{SG9263, &UA778968},
	{SG9264, &UA852213},
	{SG9265, &UA893480},
	{SG9266, &UA895587},
	{SG9267, &UA1993671},
	{SG9268, &UA2024305},
	{SG9269, &UA2029073},
	{SG9270, &UA2684693},
	{SG9271, &UA2725056},
	{SG9272, &UA2820597},
	{SG9275, &UA49190},
	{SG9276, &UA623982},
	{SG9277, &UA2024174},
	{SG9279, &UA194749},
	{SG9280, &UA241922},
	{SG9281, &UA327166},
	{SG9282, &UA1900605},
	{SG9291, &UA47462},
	{SG9292, &UA47482},
	{SG9298, &UA725237},
	{SG9299, &UA2029645},
	{SG9300, &UA346965},
	{SG9301, &UA615349},
	{SG9302, &UA641214},
	{SG9303, &UA658271},
	{SG9304, &UA2408431},
	{SG9307, &UA230577},
	{SG9309, &UA656649},
	{SG9314, &UA191814},
	{SG9315, &UA382189},
	{SG9316, &UA625398},
	{SG9317, &UA838188},
	{SG9318, &UA33435},
	{SG9319, &UA633790},
	{SG9320, &UA2028606},
	{SG9321, &UA2424829},
	{SG9322, &UA2734515},
	{SG9324, &UA43016},
	{SG9326, &UA368817},
	{SG9327, &UA370372},
	{SG9328, &UA653077},
	{SG9329, &UA1940188},
	{SG9330, &UA1966935},
	{SG9331, &UA2022744},
	{SG9339, &UA238557},
	{SG9340, &UA238575},
	{SG9341, &UA350516},
	{SG9342, &UA395913},
	{SG9343, &UA623986},
	{SG9344, &UA630953},
	{SG9345, &UA680415},
	{SG9346, &UA683301},
	{SG9347, &UA698224},
	{SG9348, &UA750676},
	{SG9349, &UA762331},
	{SG9350, &UA1904162},
	{SG9351, &UA2030644},
	{SG9352, &UA2375420},
	{SG9353, &UA2590465},
	{SG9356, &UA33441},
	{SG9357, &UA231617},
	{SG9358, &UA2025709},
	{SG9359, &UA49979},
	{SG9360, &UA356332},
	{SG9361, &UA703392},
	{SG9362, &UA758694},
	{SG9363, &UA789441},
	{SG9375, &UA794659},
	{SG9387, &UA804215},
	{SG9388, &UA829572},
	{SG9394, &UA70195},
	{SG9395, &UA70186},
	{SG9396, &UA637173},
	{SG9407, &UA51874},
	{SG9408, &UA576291},
	{SG9409, &UA646780},
	{SG9410, &UA2814686},
	{SG9418, &UA251884},
	{SG9419, &UA646434},
	{SG9420, &UA2759013},
	{SG9432, &UA74441},
	{SG9433, &UA77323},
	{SG9434, &UA627833},
	{SG9435, &UA652940},
	{SG9436, &UA680788},
	{SG9437, &UA708141},
	{SG9438, &UA739558},
	{SG9439, &UA1946506},
	{SG9440, &UA1950763},
	{SG9441, &UA2022067},
	{SG9442, &UA2022563},
	{SG9443, &UA2023076},
	{SG9444, &UA2392358},
	{SG9446, &UA33217},
	{SG9447, &UA48799},
	{SG9448, &UA78474},
	{SG9449, &UA322123},
	{SG9451, &UA334833},
	{SG9453, &UA639057},
	{SG9454, &UA688706},
	{SG9455, &UA693417},
	{SG9456, &UA720912},
	{SG9457, &UA2024356},
	{SG9458, &UA2544192},
	{SG9459, &UA33453},
	{SG9460, &UA75693},
	{SG9461, &UA75970},
	{SG9462, &UA77208},
	{SG9463, &UA362879},
	{SG9464, &UA576584},
	{SG9465, &UA633631},
	{SG9468, &UA70446},
	{SG9470, &UA70754},
	{SG9472, &UA70858},
	{SG9473, &UA369339},
	{SG9474, &UA574253},
	{SG9475, &UA626246},
	{SG9476, &UA635115},
	{SG9477, &UA653387},
	{SG9478, &UA655252},
	{SG9479, &UA702565},
	{SG9480, &UA720086},
	{SG9481, &UA1962925},
	{SG9482, &UA2022909},
	{SG9483, &UA2024333},
	{SG9484, &UA2025199},
	{SG9485, &UA2025464},
	{SG9498, &UA49543},
	{SG9499, &UA70694},
	{SG9500, &UA623251},
	{SG9501, &UA2024561},
	{SG9502, &UA2030004},
	{SG9533, &UA190692},
	{SG9534, &UA192055},
	{SG9535, &UA315554},
	{SG9536, &UA624890},
	{SG9537, &UA891484},
	{SG9538, &UA2026007},
	{SG9539, &UA48737},
	{SG9540, &UA640711},
	{SG9543, &UA33342},
	{SG9544, &UA220357},
	{SG9545, &UA623060},
	{SG9546, &UA2447620},
	{SG9547, &UA3070059},
	{SG9555, &UA210692},
	{SG9556, &UA235438},
	{SG9558, &UA332065},
	{SG9561, &UA333886},
	{SG9563, &UA451972},
	{SG9565, &UA2019877},
	{SG9577, &UA48261},
	{SG9578, &UA640191},
	{SG9579, &UA754240},
	{SG9580, &UA795772},
	{SG9582, &UA1904992},
	{SG9584, &UA1944193},
	{SG9585, &UA2535654},
	{SG9586, &UA2543865},
	{SG9587, &UA2559590},
	{SG9588, &UA2933279},
	{SG9607, &UA903494},
	{SG9608, &UA1971405},
	{SG9611, &UA48575},
	{SG9612, &UA694942},
	{SG9628, &UA2597362},
	{SG9629, &UA239631},
	{SG9630, &UA683581},
	{SG9644, &UA641463},
	{SG9645, &UA653318},
	{SG9646, &UA663887},
	{SG9647, &UA703595},
	{SG9648, &UA733720},
	{SG9649, &UA788077},
	{SG9650, &UA817054},
	{SG9651, &UA853642},
	{SG9652, &UA1917249},
	{SG9653, &UA2019500},
	{SG9654, &UA2023646},
	{SG9655, &UA2024540},
	{SG9657, &UA2026693},
	{SG9671, &UA253241},
	{SG9672, &UA421776},
	{SG9674, &UA453704},
	{SG9675, &UA625519},
	{SG9676, &UA2004712},
	{SG9677, &UA2005515},
	{SG9678, &UA2206195},
	{SG9679, &UA2206554},
	{SG9680, &UA2352905},
	{SG9681, &UA840385},
	{SG9685, &UA814658},
	{SG9689, &UA576019},
	{SG9690, &UA626440},
	{SG9692, &UA627535},
	{SG9694, &UA680563},
	{SG9697, &UA2002571},
	{SG9698, &UA2018018},
	{SG9699, &UA2033276},
	{SG9719, &UA42905},
	{SG9721, &UA370620},
	{SG9722, &UA430029},
	{SG9723, &UA430079},
	{SG9724, &UA430080},
	{SG9725, &UA430083},
	{SG9726, &UA430120},
	{SG9727, &UA430124},
	{SG9728, &UA647470},
	{SG9729, &UA747411},
	{SG9730, &UA2597337},
	{SG9731, &UA2597658},
	{SG9732, &UA2659280},
	{SG9733, &UA48418},
	{SG9734, &UA48423},
	{SG9735, &UA682382},
	{SG9736, &UA752259},
	{SG9737, &UA2033344},
	{SG9738, &UA2649904},
	{SG9739, &UA641457},
	{SG9740, &UA643660},
	{SG9741, &UA2022297},
	{SG9751, &UA49526},
	{SG9752, &UA452506},
	{SG9753, &UA33269},
	{SG9754, &UA452355},
	{SG9755, &UA59853},
	{SG9758, &UA459182},
	{SG9763, &UA623289},
	{SG9764, &UA640765},
	{SG9765, &UA1946739},
	{SG9766, &UA1950833},
	{SG9767, &UA2775029},
	{SG9791, &UA197405},
	{SG9792, &UA324512},
	{SG9793, &UA2019193},
	{SG9794, &UA628869},
	{SG9795, &UA1939542},
	{SG9796, &UA628619},
	{SG9797, &UA2386780},
	{SG9798, &UA2559356},
	{SG9812, &UA623343},
	{SG9817, &UA253559},
	{SG9818, &UA626134},
	{SG9819, &UA2256905},
	{SG9820, &UA2256919},
	{SG9821, &UA2571969},
	{SG9822, &UA2584108},
	{SG9823, &UA2935417},
	{SG9833, &UA2202577},
	{SG9836, &UA347376},
	{SG9837, &UA643427},
	{SG9838, &UA683479},
	{SG9839, &UA707212},
	{SG9840, &UA1904620},
	{SG9843, &UA436683},
	{SG9844, &UA629594},
	{SG9845, &UA731017},
	{SG9846, &UA736045},
	{SG9847, &UA741002},
	{SG9848, &UA855008},
	{SG9849, &UA1904094},
	{SG9850, &UA1962836},
	{SG9851, &UA2685524},
	{SG9863, &UA632680},
	{SG9864, &UA690910},
	{SG9865, &UA691137},
	{SG9866, &UA2027591},
	{SG9867, &UA2033930},
	{SG9868, &UA2933413},
	{SG9876, &UA49251},
	{SG9882, &UA194725},
	{SG9883, &UA2019198},
	{SG9884, &UA2019327},
	{SG9885, &UA2023323},
	{SG9886, &UA41573},
	{SG9893, &UA352682},
	{SG9894, &UA882276},
	{SG9895, &UA1941664},
	{SG9911, &UA197020},
	{SG9914, &UA229488},
	{SG9918, &UA1882966},
	{SG9920, &UA1883392},
	{SG9921, &UA625770},
	{SG9922, &UA2023188},
	{SG9923, &UA2024146},
	{SG9952, &UA635992},
	{SG9953, &UA1945532},
	{SG9959, &UA2618515},
	{SG9967, &UA49464},
	{SG9968, &UA797527},
	{SG9969, &UA817871},
	{SG9970, &UA2020154},
	{SG9971, &UA2020744},
	{SG9984, &UA650828},
	{SG9985, &UA2588362},
	{SG10014, &UA624932},
	{SG10015, &UA646782},
	{SG10016, &UA2032959},
	{SG10019, &UA242668},
	{SG10020, &UA657353},
	{SG10021, &UA788574},
	{SG10024, &UA55302},
	{SG10025, &UA55310},
	{SG10026, &UA743341},
	{SG10027, &UA2753928},
	{SG10033, &UA183007},
	{SG10034, &UA188783},
	{SG10035, &UA449511},
	{SG10046, &UA211469},
	{SG10047, &UA695605},
	{SG10048, &UA2019600},
	{SG10049, &UA2019772},
	{SG10050, &UA73970},
	{SG10051, &UA2775809},
	{SG10052, &UA395055},
	{SG10053, &UA1901337},
	{SG10054, &UA2402265},
	{SG10055, &UA192033},
	{SG10056, &UA207041},
	{SG10057, &UA314812},
	{SG10058, &UA544057},
	{SG10064, &UA1936903},
	{SG10084, &UA48690},
	{SG10085, &UA59608},
	{SG10086, &UA2019008},
	{SG10119, &UA2022719},
	{SG10124, &UA569915},
	{SG10125, &UA697574},
	{SG10126, &UA1990351},
	{SG10127, &UA2020176},
	{SG10128, &UA2022164},
	{SG10139, &UA351158},
	{SG10140, &UA2022377},
	{SG10171, &UA1943062},
	{SG10201, &UA755792},
	{SG10202, &UA2033498},
	{SG10235, &UA1933486},
	{SG10236, &UA1963744},
	{SG10237, &UA1965526},
	{SG10238, &UA2411960},
	{SG10239, &UA2475826},
	{SG10240, &UA2518643},
	{SG10241, &UA2518661},
	{SG10242, &UA2725032},
	{SG10244, &UA48697},
	{SG10305, &UA2018719},
	{SG10306, &UA2018997},
	{SG10308, &UA420886},
	{SG10309, &UA1900926},
	{SG10310, &UA648756},
	{SG10318, &UA196259},
	{SG10319, &UA225390},
	{SG10345, &UA2722539},
	{SG10346, &UA611324},
	{SG10348, &UA854836},
	{SG10350, &UA2033964},
	{SG10354, &UA76203},
	{SG10355, &UA360224},
	{SG10356, &UA614354},
	{SG10357, &UA741866},
	{SG10358, &UA2191017},
	{SG10359, &UA2982035},
	{SG10381, &UA595218},
	{SG10382, &UA2236475},
	{SG10390, &UA633285},
	{SG10391, &UA1931757},
	{SG10392, &UA1937057},
	{SG10393, &UA2745288},
	{SG10406, &UA625621},
	{SG10408, &UA2022425},
	{SG10417, &UA49501},
	{SG10418, &UA2020352},
	{SG10433, &UA250470},
	{SG10434, &UA348736},
	{SG10435, &UA609468},
	{SG10436, &UA622475},
	{SG10437, &UA648738},
	{SG10438, &UA658693},
	{SG10439, &UA2679624},
	{SG10440, &UA2774304},
	{SG10441, &UA315930},
	{SG10446, &UA77773},
	{SG10447, &UA716802},
	{SG10448, &UA719030},
	{SG10449, &UA1944829},
	{SG10450, &UA2021427},
	{SG10451, &UA2022759},
	{SG10452, &UA2029406},
	{SG10453, &UA2812102},
	{SG10455, &UA195340},
	{SG10483, &UA221039},
	{SG10484, &UA729797},
	{SG10488, &UA49931},
	{SG10489, &UA681170},
	{SG10494, &UA575794},
	{SG10496, &UA251868},
	{SG10497, &UA458457},
	{SG10498, &UA2020434},
	{SG10499, &UA2197243},
	{SG10520, &UA217091},
	{SG10522, &UA242585},
	{SG10524, &UA381698},
	{SG10526, &UA901173},
	{SG10527, &UA2030927},
	{SG10528, &UA2032267},
	{SG10535, &UA1940687},
	{SG10536, &UA79463},
	{SG10537, &UA2021735},
	{SG10538, &UA2806739},
	{SG10539, &UA38364},
	{SG10540, &UA38370},
	{SG10541, &UA2020469},
	{SG10548, &UA224200},
	{SG10554, &UA2020357},
	{SG10555, &UA69091},
	{SG10556, &UA318362},
	{SG10557, &UA678150},
	{SG10565, &UA60286},
	{SG10566, &UA685652},
	{SG10567, &UA2721260},
	{SG10569, &UA368808},
	{SG10570, &UA1951463},
	{SG10571, &UA2021716},
	{SG10582, &UA318135},
	{SG10583, &UA2019142},
	{SG10585, &UA318336},
	{SG10594, &UA844347},
	{SG10595, &UA2029545},
	{SG10597, &UA2721020},
	{SG10600, &UA2032533},
	{SG10603, &UA844069},
	{SG10604, &UA2809473},
	{SG10605, &UA70859},
	{SG10606, &UA338370},
	{SG10607, &UA345831},
	{SG10608, &UA632730},
	{SG10609, &UA685875},
	{SG10610, &UA895049},
	{SG10611, &UA1972209},
	{SG10612, &UA2020007},
	{SG10613, &UA2023777},
	{SG10614, &UA2025114},
	{SG10615, &UA2034529},
	{SG10616, &UA2583853},
	{SG10617, &UA2796623},
	{SG10618, &UA2862161},
	{SG10619, &UA2999727},
	{SG10622, &UA904877},
	{SG10623, &UA2010808},
	{SG10624, &UA2029556},
	{SG10625, &UA2202832},
	{SG10626, &UA2606280},
	{SG10627, &UA2863593},
	{SG10646, &UA1994546},
	{SG10654, &UA705948},
	{SG10661, &UA1906278},
	{SG10665, &UA33016},
	{SG10668, &UA2994499},
	{SG10669, &UA3048181},
	{SG10670, &UA40114},
	{SG10671, &UA74188},
	{SG10672, &UA2020042},
	{SG10673, &UA2567740},
	{SG10674, &UA568057},
	{SG10675, &UA572189},
	{SG10677, &UA2432007},
	{SG10686, &UA350373},
	{SG10722, &UA440176},
	{SG10723, &UA450883},
	{SG10726, &UA332898},
	{SG10731, &UA2026419},
	{SG10739, &UA774999},
	{SG10752, &UA437101},
	{SG10753, &UA2020063},
	{SG10770, &UA349833},
	{SG10771, &UA694423},
	{SG10772, &UA2025482},
	{SG10773, &UA2640152},
	{SG10782, &UA32997},
	{SG10785, &UA59637},
	{SG10809, &UA709790},
	{SG10810, &UA2815911},
	{SG10828, &UA2385633},
	{SG10830, &UA351338},
	{SG10831, &UA422316},
	{SG10832, &UA437099},
	{SG10833, &UA610157},
	{SG10834, &UA676850},
	{SG10835, &UA2714594},
	{SG10836, &UA2784813},
	{SG10841, &UA2797698},
	{SG10851, &UA696833},
	{SG10859, &UA2868379},
	{SG10861, &UA379548},
	{SG10863, &UA2565566},
	{SG10864, &UA2665448},
	{SG10872, &UA68396},
	{SG10873, &UA2537942},
	{SG10888, &UA230579},
	{SG10889, &UA660753},
	{SG10899, &UA2383373},
	{SG10900, &UA2481950},
	{SG10904, &UA627433},
	{SG10905, &UA2023831},
	{SG10907, &UA47598},
	{SG10908, &UA47600},
	{SG10909, &UA2022398},
	{SG10910, &UA2034518},
	{SG10921, &UA315565},
	{SG10922, &UA777794},
	{SG10923, &UA2018801},
	{SG10924, &UA2019277},
	{SG10934, &UA2598800},
	{SG10935, &UA2663513},
	{SG10942, &UA2021088},
	{SG10951, &UA318523},
	{SG10984, &UA628441},
	{SG10985, &UA837144},
	{SG10986, &UA2540561},
	{SG10987, &UA2540577},
	{SG10989, &UA2540621},
	{SG10990, &UA2234486},
	{SG10991, &UA2455113},
	{SG11004, &UA651090},
	{SG11005, &UA2019464},
	{SG11007, &UA2021888},
	{SG11008, &UA2469875},
	{SG11010, &UA68370},
	{SG11011, &UA195612},
	{SG11015, &UA396894},
	{SG11016, &UA2020010},
	{SG11033, &UA194373},
	{SG11036, &UA567322},
	{SG11037, &UA1937279},
	{SG11038, &UA2021854},
	{SG11039, &UA2810990},
	{SG11049, &UA594698},
	{SG11050, &UA2031074},
	{SG11053, &UA632517},
	{SG11054, &UA734060},
	{SG11055, &UA748102},
	{SG11056, &UA1996927},
	{SG11057, &UA2476081},
	{SG11058, &UA2707565},
	{SG11059, &UA3059681},
	{SG11061, &UA409569},
	{SG11062, &UA574450},
	{SG11063, &UA613533},
	{SG11064, &UA631231},
	{SG11065, &UA2024662},
	{SG11066, &UA2716982},
	{SG11071, &UA2870159},
	{SG11084, &UA76632},
	{SG11085, &UA689634},
	{SG11086, &UA2021421},
	{SG11087, &UA2024844},
	{SG11088, &UA2027413},
	{SG11090, &UA535944},
	{SG11091, &UA2285885},
	{SG11092, &UA361186},
	{SG11093, &UA2980609},
	{SG11099, &UA2484448},
	{SG11100, &UA3059595},
	{SG11109, &UA353516},
	{SG11110, &UA610553},
	{SG11111, &UA711433},
	{SG11112, &UA1947366},
	{SG11126, &UA573556},
	{SG11128, &UA667277},
	{SG11129, &UA691020},
	{SG11130, &UA1944367},
	{SG11131, &UA2024670},
	{SG11132, &UA2025768},
	{SG11134, &UA2807825},
	{SG11146, &UA2032778},
	{SG11160, &UA194755},
	{SG11168, &UA696016},
	{SG11170, &UA2223744},
	{SG11173, &UA2808917},
	{SG11205, &UA49812},
	{SG11212, &UA371002},
	{SG11237, &UA702449},
	{SG11243, &UA32998},
	{SG11244, &UA561304},
	{SG11245, &UA813285},
	{SG11246, &UA2019583},
	{SG11256, &UA1939635},
	{SG11259, &UA744486},
	{SG11263, &UA2034250},
	{SG11265, &UA1885755},
	{SG11266, &UA2725321},
	{SG11268, &UA319792},
	{SG11269, &UA2019735},
	{SG11271, &UA2020426},
	{SG11273, &UA2024713},
	{SG11275, &UA2543193},
	{SG11276, &UA2527916},
	{SG11277, &UA2983964},
	{SG11278, &UA556478},
	{SG11286, &UA2784047},
	{SG11293, &UA677148},
	{SG11294, &UA2018766},
	{SG11297, &UA2533366},
	{SG11298, &UA677962},
	{SG11299, &UA1972437},
	{SG11303, &UA1953471},
	{SG11304, &UA1968364},
	{SG11310, &UA409620},
	{SG11311, &UA901320},
	{SG11312, &UA2027110},
	{SG11313, &UA2030925},
	{SG11323, &UA626893},
	{SG11324, &UA2642319},
	{SG11337, &UA70923},
	{SG11345, &UA2234487},
	{SG11350, &UA49225},
	{SG11351, &UA72958},
	{SG11352, &UA666596},
	{SG11353, &UA683087},
	{SG11354, &UA697305},
	{SG11355, &UA757357},
	{SG11357, &UA2024504},
	{SG11359, &UA2998469},
	{SG11360, &UA3006079},
	{SG11363, &UA202270},
	{SG11365, &UA319009},
	{SG11383, &UA2258617},
	{SG11394, &UA2022238},
	{SG11395, &UA2023001},
	{SG11403, &UA1932348},
	{SG11404, &UA2439532},
	{SG11406, &UA718526},
	{SG11408, &UA2713884},
	{SG11410, &UA723696},
	{SG11411, &UA2020921},
	{SG11412, &UA2031469},
	{SG11418, &UA603073},
	{SG11419, &UA669515},
	{SG11420, &UA2387201},
	{SG11421, &UA2878182},
	{SG11422, &UA71760},
	{SG11423, &UA2019939},
	{SG11427, &UA2869282},
	{SG11434, &UA2783285},
	{SG11435, &UA1970909},
	{SG11436, &UA221860},
	{SG11449, &UA2660051},
	{SG11453, &UA1922249},
	{SG11464, &UA369652},
	{SG11465, &UA746962},
	{SG11466, &UA2029642},
	{SG11467, &UA2992965},
	{SG11468, &UA623133},
	{SG11469, &UA646005},
	{SG11470, &UA2879726},
	{SG11480, &UA2023806},
	{SG11481, &UA2023944},
	{SG11483, &UA2435643},
	{SG11495, &UA628537},
	{SG11496, &UA2023176},
	{SG11497, &UA2023964},
	{SG11498, &UA2033911},
	{SG11499, &UA2394637},
	{SG11500, &UA2817941},
	{SG11506, &UA369119},
	{SG11507, &UA624837},
	{SG11508, &UA635417},
	{SG11509, &UA895486},
	{SG11510, &UA2669394},
	{SG11511, &UA2894708},
	{SG11514, &UA2993999},
	{SG11516, &UA2029166},
	{SG11517, &UA2817311},
	{SG11518, &UA322479},
	{SG11519, &UA2018703},
	{SG11520, &UA638300},
	{SG11521, &UA773940},
	{SG11522, &UA820418},
	{SG11523, &UA2026077},
	{SG11525, &UA2032783},
	{SG11527, &UA836391},
	{SG11530, &UA1941333},
	{SG11533, &UA1993971},
	{SG11534, &UA2223079},
	{SG11535, &UA368780},
	{SG11536, &UA2024496},
	{SG11537, &UA2496366},
	{SG11538, &UA2817616},
	{SG11569, &UA2485212},
	{SG11574, &UA661834},
	{SG11575, &UA2233694},
	{SG11580, &UA2235446},
	{SG11583, &UA2716276},
	{SG11584, &UA2958432},
	{SG11589, &UA213303},
	{SG11591, &UA2790646},
	{SG11602, &UA451561},
	{SG11609, &UA2580507},
	{SG11629, &UA794631},
	{SG11630, &UA2022295},
	{SG11633, &UA616467},
	{SG11634, &UA616473},
	{SG11635, &UA616479},
	{SG11642, &UA421844},
	{SG11643, &UA603757},
	{SG11644, &UA660917},
	{SG11645, &UA1959792},
	{SG11653, &UA197288},
	{SG11654, &UA774134},
	{SG11655, &UA2957613},
	{SG11657, &UA2234302},
	{SG11668, &UA2024570},
	{SG11670, &UA665438},
	{SG11671, &UA669126},
	{SG11672, &UA1951784},
	{SG11673, &UA2387324},
	{SG11674, &UA2401653},
	{SG11678, &UA1970941},
	{SG11686, &UA2411292},
	{SG11691, &UA664377},
	{SG11692, &UA2771018},
	{SG11693, &UA2469227},
	{SG11694, &UA2022671},
	{SG11699, &UA2031435},
	{SG11700, &UA2376373},
	{SG11703, &UA2756298},
	{SG11706, &UA2558964},
	{SG11708, &UA783214},
	{SG11709, &UA3009064},
	{SG11719, &UA2384356},
	{SG11721, &UA2258534},
	{SG11728, &UA2530581},
	{SG11733, &UA2019107},
	{SG11739, &UA725576},
	{SG11740, &UA1954073},
	{SG11742, &UA2258608},
	{SG11743, &UA2032336},
	{SG11744, &UA2033593},
	{SG11746, &UA902254},
	{SG11754, &UA1934531},
	{SG11757, &UA3126086},
	{SG11760, &UA2621118},
	{SG11765, &UA2022045},
	{SG11766, &UA72822},
	{SG11767, &UA75739},
	{SG11768, &UA1991333},
	{SG11769, &UA2021715},
	{SG11770, &UA2030392},
	{SG11771, &UA2031799},
	{SG11772, &UA661832},
	{SG11773, &UA2022608},
	{SG11774, &UA2406570},
	{SG11777, &UA1902478},
	{SG11780, &UA355242},
	{SG11781, &UA2220169},
	{SG11788, &UA3063035},
	{SG11798, &UA2025124},
	{SG11800, &UA2637596},
	{SG11806, &UA2411014},
	{SG11809, &UA689337},
	{SG11819, &UA1927882},
	{SG11821, &UA2589277},
	{SG11825, &UA2800569},
	{SG11827, &UA196512},
	{SG11828, &UA539849},
	{SG11848, &UA2400634},
	{SG11849, &UA2632709},
	{SG11851, &UA697870},
	{SG11852, &UA2024256},
	{SG11854, &UA2506561},
	{SG11858, &UA2469283},
	{SG11861, &UA798879},
	{SG11862, &UA1919626},
	{SG11864, &UA2023500},
	{SG11865, &UA2026375},
	{SG11867, &UA2531113},
	{SG11877, &UA2715705},
	{SG11879, &UA367186},
	{SG11880, &UA667116},
	{SG11881, &UA1945293},
	{SG11882, &UA635875},
	{SG11911, &UA911303},
	{SG11912, &UA1983300},
	{SG11913, &UA2344415},
	{SG11914, &UA3061810},
	{SG11915, &UA2026461},
	{SG11916, &UA2026992},
	{SG11917, &UA2758545},
	{SG11918, &UA2421551},
	{SG11920, &UA2565995},
	{SG11923, &UA2033958},
	{SG11924, &UA2234299},
	{SG11925, &UA2773154},
	{SG11927, &UA2545318},
	{SG11932, &UA2426804},
	{SG11933, &UA3061278},
	{SG11937, &UA3121705},
	{SG11944, &UA1961920},
	{SG11951, &UA2944441},
	{SG11952, &UA3056807},
	{SG11957, &UA568122},
	{SG11958, &UA825518},
	{SG11962, &UA575078},
	{SG11963, &UA2021479},
	{SG11964, &UA2023014},
	{SG11965, &UA2024044},
	{SG11967, &UA2701383},
	{SG11968, &UA3048255},
	{SG11969, &UA75293},
	{SG11970, &UA2022912},
	{SG11971, &UA2027358},
	{SG11979, &UA2796890},
	{SG11982, &UA2531432},
	{SG11987, &UA2018666},
	{SG11989, &UA1941866},
	{SG11990, &UA1964295},
	{SG11994, &UA79417},
	{SG11995, &UA2019805},
	{SG12010, &UA2384536},
	{SG12011, &UA3016338},
	{SG12013, &UA2021883},
	{SG12022, &UA743402},
	{SG12023, &UA1921542},
	{SG12029, &UA870169},
	{SG12030, &UA2755501},
	{SG12036, &UA1987699},
	{SG12037, &UA3086717},
	{SG12039, &UA905711},
	{SG12041, &UA2752644},
	{SG12045, &UA2503821},
	{SG12047, &UA688329},
	{SG12048, &UA1943375},
	{SG12049, &UA2033315},
	{SG12055, &UA2025026},
	{SG12059, &UA2032286},
	{SG12066, &UA2637902},
	{SG12070, &UA2032826},
	{SG12071, &UA1940553},
	{SG12073, &UA2353943},
	{SG12074, &UA2668981},
	{SG12079, &UA2025057},
	{SG12085, &UA2872533},
	{SG12086, &UA778013},
	{SG12087, &UA2800240},
	{SG12088, &UA2258496},
	{SG12089, &UA2485208},
	{SG12093, &UA3064717},
	{SG12098, &UA1937256},
	{SG12105, &UA684181},
	{SG12112, &UA2708683},
	{SG12113, &UA2658953},
	{SG12116, &UA855682},
	{SG12128, &UA2964020},
	{SG12134, &UA2018995},
	{SG12136, &UA2705974},
	{SG12139, &UA2943861},
	{SG12141, &UA671739},
	{SG12144, &UA2667126},
	{SG12148, &UA2030320},
	{SG12150, &UA2025703},
	{SG12153, &UA2506923},
	{SG12154, &UA2703971},
	{SG12160, &UA2783695},
	{SG12161, &UA2864952},
	{SG12172, &UA2971887},
	{SG12174, &UA563031},
	{SG12175, &UA2023934},
	{SG12177, &UA2958538},
	{SG12178, &UA2021183},
	{SG12179, &UA2021962},
	{SG12180, &UA2809532},
	{SG12182, &UA2659727},
	{SG12187, &UA1933400},
	{SG12193, &UA2869504},
	{SG12197, &UA570467},
	{SG12198, &UA2258620},
	{SG12201, &UA2793863},
	{SG12203, &UA2962000},
	{SG12217, &UA2580473},
	{SG12218, &UA2599557},
	{SG12219, &UA3086562},
	{SG12221, &UA2799112},
	{SG12227, &UA2384295},
	{SG12228, &UA2882264},
	{SG12233, &UA2542524},
	{SG12236, &UA2018754},
	{SG12245, &UA2558961},
	{SG12262, &UA2955973},
	{SG12265, &UA2781486},
	{SG12266, &UA2781723},
	{SG12267, &UA3024530},
	{SG12271, &UA717743},
	{SG12272, &UA2541950},
	{SG12273, &UA2957452},
	{SG12274, &UA2892605},
	{SG12281, &UA2411324},
	{SG12282, &UA2485537},
	{SG12288, &UA3029607},
	{SG12289, &UA3108361},
	{SG12290, &UA895595},
	{SG12291, &UA2867681},
	{SG12295, &UA2527565},
	{SG12297, &UA2597794},
	{SG12302, &UA2029874},
	{SG12304, &UA2455391},
	{SG12305, &UA2765260},
	{SG12311, &UA2019466},
	{SG12313, &UA684847},
	{SG12316, &UA370649},
	{SG12317, &UA2758207},
	{SG12320, &UA2031765},
	{SG12321, &UA2892617},
	{SG12326, &UA2945657},
	{SG12333, &UA642643},
	{SG12334, &UA2029233},
	{SG12335, &UA2987675},
	{SG12341, &UA2795701},
	{SG12342, &UA2950617},
	{SG12345, &UA1991076},
	{SG12346, &UA2440152},
	{SG12348, &UA2958581},
	{SG12354, &UA2781927},
	{SG12355, &UA2866977},
	{SG12356, &UA2957326},
	{SG12361, &UA2019176},
	{SG12364, &UA662666},
	{SG12373, &UA590304},
	{SG12374, &UA1955171},
	{SG12375, &UA1984259},
	{SG12376, &UA2023642},
	{SG12377, &UA2034257},
	{SG12378, &UA2033582},
	{SG12383, &UA2659891},
	{SG12387, &UA2028575},
	{SG12389, &UA2989910},
	{SG12394, &UA2020141},
	{SG12395, &UA2022100},
	{SG12397, &UA2034047},
	{SG12399, &UA2727970},
	{SG12401, &UA3019019},
	{SG12410, &UA2021690},
	{SG12411, &UA2023073},
	{SG12420, &UA2831033},
	{SG12422, &UA2026385},
	{SG12423, &UA2028338},
	{SG12424, &UA2350034},
	{SG12426, &UA2485817},
	{SG12427, &UA2580668},
	{SG12428, &UA2660329},
	{SG12430, &UA2783878},
	{SG12434, &UA2946166},
	{SG12437, &UA3007263},
	{SG12439, &UA359497},
	{SG12451, &UA2964262},
	{SG12457, &UA3005967},
	{SG12460, &UA2466089},
	{SG12462, &UA2661713},
	{SG12463, &UA2665592},
	{SG12464, &UA2703450},
	{SG12472, &UA2998415},
	{SG12474, &UA3111353},
	{SG12477, &UA561063},
	{SG12478, &UA2027055},
	{SG12479, &UA2028504},
	{SG12480, &UA2032249},
	{SG12481, &UA2029784},
	{SG12482, &UA2350819},
	{SG12483, &UA2371758},
	{SG12484, &UA2431838},
	{SG12489, &UA2867610},
	{SG12498, &UA2654109},
	{SG12511, &UA2027468},
	{SG12512, &UA2043770},
	{SG12513, &UA2415194},
	{SG12514, &UA2480240},
	{SG12515, &UA2502430},
	{SG12516, &UA2598676},
	{SG12521, &UA2705931},
	{SG12522, &UA2727195},
	{SG12525, &UA2775748},
	{SG12530, &UA2962958},
	{SG12544, &UA3079811},
	{SG12565, &UA2027748},
	{SG12566, &UA2699068},
	{SG12567, &UA2997163},
	{SG12568, &UA3079258},
	{SG12572, &UA2020739},
	{SG12574, &UA2022339},
	{SG12577, &UA2525944},
	{SG12578, &UA2750903},
	{SG12583, &UA2998556},
	{SG12586, &UA3016257},
	{SG12588, &UA3110316},
	{SG12593, &UA2028994},
	{SG12594, &UA2029500},
	{SG12597, &UA2867442},
	{SG12600, &UA3109404},
	{SG12603, &UA2167099},
	{SG12604, &UA2445159},
	{SG12605, &UA2502567},
	{SG12606, &UA2661668},
	{SG12610, &UA3015353},
	{SG12618, &UA2360145},
	{SG12620, &UA2552777},
	{SG12622, &UA2730021},
	{SG12624, &UA2764359},
	{SG12627, &UA2858158},
	{SG12628, &UA2865779},
	{SG12629, &UA2998795},
	{SG12630, &UA2999576},
	{SG12644, &UA2705499},
	{SG12645, &UA2733363},
	{SG12648, &UA2850574},
	{SG12650, &UA2863270},
	{SG12652, &UA3049709},
	{SG12658, &UA2028666},
	{SG12660, &UA2629588},
	{SG12671, &UA2957567},
	{SG12673, &UA2999810},
	{SG12674, &UA3049119},
	{SG12682, &UA3103069},
	{SG12683, &UA3108370},
	{SG12684, &UA3114896},
	{SG12689, &UA2223194},
	{SG12690, &UA2577663},
	{SG12691, &UA2594292},
	{SG12692, &UA2748567},
	{SG12700, &UA2985081},
	{SG12702, &UA2995814},
	{SG12705, &UA3012158},
	{SG12708, &UA3019443},
	{SG12714, &UA2029876},
	{SG12716, &UA2339316},
	{SG12718, &UA2352228},
	{SG12722, &UA2706494},
	{SG12723, &UA2726632},
	{SG12726, &UA2744879},
	{SG12729, &UA2863486},
	{SG12732, &UA2997633},
	{SG12743, &UA3127516},
	{SG12746, &UA1919092},
	{SG12747, &UA2034275},
	{SG12756, &UA2983626},
	{SG12758, &UA2998557},
	{SG12759, &UA2999404},
	{SG12761, &UA2999964},
	{SG12764, &UA3049590},
	{SG12766, &UA3100985},
	{SG12775, &UA3125783},
	{SG12776, &UA3127081},
	{SG12777, &UA3127238},
	{SG12778, &UA371226},
	{SG12779, &UA1976965},
	{SG12780, &UA2027014},
	{SG12781, &UA2167291},
	{SG12784, &UA2826246},
	{SG12785, &UA2836704},
	{SG12786, &UA2847894},
	{SG12788, &UA2861747},
	{SG12795, &UA2834109},
	{SG12801, &UA2019447},
	{SG12802, &UA2223434},
	{SG12804, &UA458890},
	{SG12805, &UA2031323},
	{SG12848, &UA2026825},
	{SG12849, &UA2816540},
	{SG12851, &UA369455},
	{SG12855, &UA2627028},
	{SG12856, &UA2630740},
	{SG12863, &UA2022714},
	{SG12864, &UA2023148},
	{SG12865, &UA2338632},
	{SG12870, &UA2028107},
	{SG12871, &UA2032399},
	{SG12872, &UA2816256},
	{SG12875, &UA2019167},
	{SG12876, &UA2022415},
	{SG12877, &UA2025494},
	{SG12881, &UA2021586},
	{SG12882, &UA2023201},
	{SG12885, &UA1957662},
	{SG12886, &UA2020220},
	{SG12890, &UA2026467},
	{SG12927, &UA49453},
	{SG12928, &UA53549},
	{SG12932, &UA2023537},
	{SG12933, &UA2024752},
	{SG12934, &UA2025076},
	{SG12935, &UA2033091},
	{SG12936, &UA2607532},
	{SG12939, &UA1966392},
	{SG12941, &UA2031322},
	{SG12942, &UA2034040},
	{SG12943, &UA2626995},
	{SG12953, &UA2021893},
	{SG12954, &UA2023619},
	{SG12955, &UA2026110},
	{SG12956, &UA2026787},
	{SG12958, &UA2026944},
	{SG12961, &UA2443259},
	{SG12962, &UA2819196},
	{SG12966, &UA776847},
	{SG12967, &UA1991123},
	{SG12968, &UA2025221},
	{SG12969, &UA2027521},
	{SG12970, &UA2633640},
	{SG12971, &UA2634273},
	{SG12972, &UA368722},
	{SG12973, &UA832664},
	{SG12974, &UA1950251},
	{SG12975, &UA1962110},
	{SG12976, &UA2021820},
	{SG12977, &UA2025251},
	{SG12978, &UA2025923},
	{SG12979, &UA2032663},
	{SG12980, &UA2622104},
	{SG13029, &UA90782},
	{SG13030, &UA458575},
	{SG13031, &UA2025594},
	{SG13032, &UA2627061},
	{SG13057, &UA369071},
	{SG13058, &UA2022093},
	{SG13059, &UA2028877},
	{SG13060, &UA2029594},
	{SG13061, &UA2031255},
	{SG13062, &UA2586589},
	{SG13064, &UA2615805},
	{SG13065, &UA2616800},
	{SG13066, &UA2632280},
	{SG13068, &UA369813},
	{SG13069, &UA371078},
	{SG13070, &UA905237},
	{SG13071, &UA2027178},
	{SG13072, &UA2540597},
	{SG13090, &UA3084058},
	{SG13093, &UA3084095},
	{NULL}};
const Segmentdata DAndroidVersion2_3_5[] = {
	{SG4770, &UA71268},
	{SG4771, &UA73532},
	{SG4772, &UA74825},
	{SG4773, &UA77107},
	{SG4774, &UA244330},
	{SG4775, &UA252028},
	{SG4776, &UA347576},
	{SG4777, &UA370805},
	{SG4778, &UA622971},
	{SG4779, &UA622977},
	{SG4780, &UA634708},
	{SG4781, &UA683713},
	{SG4782, &UA717522},
	{SG4783, &UA737822},
	{SG4784, &UA759969},
	{SG4785, &UA907981},
	{SG4786, &UA1914475},
	{SG4787, &UA1987293},
	{SG4788, &UA2022863},
	{SG4789, &UA2027406},
	{SG4790, &UA2032877},
	{SG4791, &UA2033203},
	{SG4792, &UA2473949},
	{SG4793, &UA2642235},
	{SG4794, &UA2979345},
	{SG4867, &UA78479},
	{SG4869, &UA251918},
	{SG4870, &UA351559},
	{SG4871, &UA356599},
	{SG4872, &UA627275},
	{SG4875, &UA673419},
	{SG4876, &UA2019450},
	{SG4877, &UA2026061},
	{SG4878, &UA2027455},
	{SG4879, &UA2256920},
	{SG4894, &UA351685},
	{SG4896, &UA352866},
	{SG4898, &UA352991},
	{SG4900, &UA366344},
	{SG4901, &UA368784},
	{SG4903, &UA380084},
	{SG4905, &UA421924},
	{SG4908, &UA421983},
	{SG4910, &UA422332},
	{SG4912, &UA582205},
	{SG4913, &UA623290},
	{SG4914, &UA628411},
	{SG4915, &UA632232},
	{SG4917, &UA641163},
	{SG4919, &UA663878},
	{SG4922, &UA686835},
	{SG4923, &UA705972},
	{SG4924, &UA737577},
	{SG4925, &UA751952},
	{SG4927, &UA760384},
	{SG4928, &UA786785},
	{SG4930, &UA806493},
	{SG4931, &UA817141},
	{SG4932, &UA819433},
	{SG4941, &UA848013},
	{SG4942, &UA897990},
	{SG4944, &UA898830},
	{SG4946, &UA898932},
	{SG4947, &UA899020},
	{SG4948, &UA911440},
	{SG4949, &UA1950388},
	{SG4950, &UA1963658},
	{SG4951, &UA1974071},
	{SG4952, &UA1979284},
	{SG4953, &UA1991129},
	{SG4954, &UA2024560},
	{SG4955, &UA2025337},
	{SG4956, &UA2025908},
	{SG4957, &UA2027764},
	{SG4958, &UA2030859},
	{SG4959, &UA2032037},
	{SG4960, &UA2362580},
	{SG4961, &UA2495274},
	{SG4962, &UA39628},
	{SG4963, &UA39648},
	{SG4964, &UA69996},
	{SG4965, &UA72065},
	{SG4966, &UA210730},
	{SG4967, &UA347784},
	{SG4968, &UA359565},
	{SG4969, &UA371083},
	{SG4970, &UA401105},
	{SG4971, &UA622686},
	{SG4972, &UA623971},
	{SG4973, &UA625297},
	{SG4974, &UA625664},
	{SG4981, &UA625998},
	{SG4983, &UA636627},
	{SG4984, &UA697047},
	{SG4986, &UA763977},
	{SG4987, &UA1947570},
	{SG4989, &UA1990052},
	{SG4992, &UA2020194},
	{SG4994, &UA2022644},
	{SG4996, &UA2024946},
	{SG4998, &UA2026516},
	{SG5054, &UA2029756},
	{SG5055, &UA2031306},
	{SG5056, &UA2031896},
	{SG5057, &UA2388808},
	{SG5058, &UA2789163},
	{SG5059, &UA2808481},
	{SG5060, &UA3117144},
	{SG5101, &UA69916},
	{SG5102, &UA396967},
	{SG5103, &UA622962},
	{SG5104, &UA642548},
	{SG5106, &UA680363},
	{SG5107, &UA763287},
	{SG5109, &UA1952216},
	{SG5111, &UA2021564},
	{SG5112, &UA2023017},
	{SG5114, &UA2023383},
	{SG5115, &UA2023426},
	{SG5173, &UA77821},
	{SG5174, &UA397397},
	{SG5175, &UA626976},
	{SG5176, &UA631199},
	{SG5177, &UA641112},
	{SG5178, &UA707370},
	{SG5179, &UA730032},
	{SG5180, &UA1895357},
	{SG5181, &UA2020094},
	{SG5182, &UA2419103},
	{SG5255, &UA78402},
	{SG5256, &UA355110},
	{SG5258, &UA633320},
	{SG5259, &UA2215979},
	{SG5260, &UA2811178},
	{SG5337, &UA227934},
	{SG5338, &UA240438},
	{SG5339, &UA339372},
	{SG5341, &UA624318},
	{SG5344, &UA2019622},
	{SG5345, &UA2020448},
	{SG5346, &UA2020893},
	{SG5348, &UA2023121},
	{SG5352, &UA2542519},
	{SG5354, &UA2545909},
	{SG5355, &UA2979872},
	{SG5362, &UA42005},
	{SG5363, &UA57807},
	{SG5367, &UA69930},
	{SG5370, &UA71931},
	{SG5371, &UA72627},
	{SG5373, &UA252248},
	{SG5375, &UA367693},
	{SG5377, &UA379928},
	{SG5378, &UA396176},
	{SG5379, &UA623039},
	{SG5381, &UA624362},
	{SG5383, &UA636793},
	{SG5384, &UA660240},
	{SG5385, &UA684966},
	{SG5387, &UA698200},
	{SG5388, &UA860571},
	{SG5389, &UA895468},
	{SG5390, &UA1935521},
	{SG5392, &UA1986519},
	{SG5394, &UA2021768},
	{SG5397, &UA2362945},
	{SG5402, &UA2494594},
	{SG5403, &UA2742510},
	{SG5610, &UA250933},
	{SG5612, &UA337113},
	{SG5613, &UA338087},
	{SG5614, &UA339394},
	{SG5615, &UA358091},
	{SG5626, &UA368729},
	{SG5628, &UA370433},
	{SG5631, &UA581140},
	{SG5632, &UA655800},
	{SG5633, &UA680613},
	{SG5637, &UA709908},
	{SG5640, &UA718482},
	{SG5642, &UA744079},
	{SG5644, &UA744838},
	{SG5645, &UA748851},
	{SG5646, &UA752069},
	{SG5648, &UA758923},
	{SG5650, &UA763548},
	{SG5651, &UA804169},
	{SG5653, &UA853897},
	{SG5656, &UA1937521},
	{SG5658, &UA1941418},
	{SG5659, &UA1945054},
	{SG5660, &UA1957455},
	{SG5662, &UA1958544},
	{SG5663, &UA1974017},
	{SG5664, &UA1975338},
	{SG5666, &UA1975956},
	{SG5669, &UA1992121},
	{SG5670, &UA1994907},
	{SG5673, &UA2019694},
	{SG5674, &UA2019888},
	{SG5676, &UA2020048},
	{SG5678, &UA2020780},
	{SG5680, &UA2022406},
	{SG5681, &UA2023078},
	{SG5683, &UA2023100},
	{SG5686, &UA2023392},
	{SG5688, &UA2024144},
	{SG5689, &UA2024321},
	{SG5690, &UA2024471},
	{SG5691, &UA2024635},
	{SG5692, &UA2024761},
	{SG5694, &UA2024902},
	{SG5695, &UA2024941},
	{SG5696, &UA2025090},
	{SG5697, &UA2025219},
	{SG5698, &UA2025637},
	{SG5699, &UA2025711},
	{SG5701, &UA2025837},
	{SG5702, &UA2025938},
	{SG5705, &UA2026864},
	{SG5706, &UA2026999},
	{SG5707, &UA2027119},
	{SG5708, &UA2027553},
	{SG5709, &UA2027600},
	{SG5711, &UA2027849},
	{SG5713, &UA2028187},
	{SG5714, &UA2028223},
	{SG5715, &UA2028577},
	{SG5716, &UA2028584},
	{SG5718, &UA2029636},
	{SG5719, &UA2029983},
	{SG5720, &UA2030192},
	{SG5722, &UA2030980},
	{SG5723, &UA2031156},
	{SG5725, &UA2032043},
	{SG5726, &UA2032163},
	{SG5727, &UA2032269},
	{SG5729, &UA2032321},
	{SG5730, &UA2032605},
	{SG5734, &UA2033352},
	{SG5735, &UA2033637},
	{SG5738, &UA2033782},
	{SG5743, &UA2034311},
	{SG5748, &UA2034612},
	{SG5756, &UA2228533},
	{SG5762, &UA2256928},
	{SG5763, &UA2256933},
	{SG5765, &UA2256955},
	{SG5768, &UA2256956},
	{SG5776, &UA2332538},
	{SG5784, &UA2339704},
	{SG5786, &UA2340040},
	{SG5787, &UA2340182},
	{SG5789, &UA2342111},
	{SG5791, &UA2343385},
	{SG5793, &UA2348759},
	{SG5797, &UA2412626},
	{SG5799, &UA2439980},
	{SG5800, &UA2441432},
	{SG5802, &UA2466801},
	{SG5807, &UA2481267},
	{SG5820, &UA2496181},
	{SG5821, &UA2498187},
	{SG5822, &UA2501846},
	{SG5823, &UA2502215},
	{SG5824, &UA2502641},
	{SG5825, &UA2502867},
	{SG5826, &UA2519941},
	{SG5827, &UA2520444},
	{SG5828, &UA2520952},
	{SG5829, &UA2521110},
	{SG5830, &UA2523817},
	{SG5831, &UA2527199},
	{SG5832, &UA2545250},
	{SG5835, &UA2546022},
	{SG5836, &UA2546182},
	{SG5839, &UA2548885},
	{SG5840, &UA2550973},
	{SG5841, &UA2554032},
	{SG5842, &UA2555367},
	{SG5843, &UA2556574},
	{SG5844, &UA2556981},
	{SG5845, &UA2558112},
	{SG5846, &UA2574468},
	{SG5847, &UA2574644},
	{SG5848, &UA2577200},
	{SG5849, &UA2577215},
	{SG5850, &UA2579031},
	{SG5851, &UA2579953},
	{SG5852, &UA2589587},
	{SG5854, &UA2590431},
	{SG5855, &UA2592405},
	{SG5857, &UA2594291},
	{SG5859, &UA2596760},
	{SG5860, &UA2598167},
	{SG5861, &UA2599760},
	{SG5862, &UA2640621},
	{SG5863, &UA2642458},
	{SG5864, &UA2643131},
	{SG5865, &UA2644129},
	{SG5866, &UA2644681},
	{SG5867, &UA2646365},
	{SG5868, &UA2647240},
	{SG5869, &UA2651205},
	{SG5870, &UA2652049},
	{SG5871, &UA2652463},
	{SG5872, &UA2654328},
	{SG5873, &UA2654766},
	{SG5874, &UA2657274},
	{SG5875, &UA2658082},
	{SG5876, &UA2665780},
	{SG5877, &UA2683997},
	{SG5879, &UA2685827},
	{SG5880, &UA2686179},
	{SG5881, &UA2686192},
	{SG5882, &UA2686369},
	{SG5883, &UA2686413},
	{SG5884, &UA2686676},
	{SG5885, &UA2686893},
	{SG5886, &UA2687950},
	{SG5887, &UA2689913},
	{SG5888, &UA2690814},
	{SG5889, &UA2693387},
	{SG5890, &UA2693999},
	{SG5891, &UA2694440},
	{SG5892, &UA2699073},
	{SG5893, &UA2702371},
	{SG5894, &UA2708278},
	{SG5895, &UA2721161},
	{SG5896, &UA2721391},
	{SG5897, &UA2722095},
	{SG5900, &UA2722525},
	{SG5901, &UA2722641},
	{SG5903, &UA2725564},
	{SG5904, &UA2726569},
	{SG5905, &UA2727483},
	{SG5906, &UA2728205},
	{SG5907, &UA2729227},
	{SG5908, &UA2729757},
	{SG5909, &UA2729811},
	{SG5910, &UA2733497},
	{SG5911, &UA2743724},
	{SG5912, &UA2745467},
	{SG5913, &UA2747010},
	{SG5914, &UA2747059},
	{SG5915, &UA2748152},
	{SG5916, &UA2751740},
	{SG5917, &UA2762129},
	{SG5918, &UA2762554},
	{SG5919, &UA2763568},
	{SG5920, &UA2764746},
	{SG5921, &UA2765252},
	{SG5922, &UA2765613},
	{SG5923, &UA2775176},
	{SG5924, &UA2776883},
	{SG5925, &UA2777889},
	{SG5926, &UA2777964},
	{SG5927, &UA2779764},
	{SG5928, &UA2780699},
	{SG5929, &UA2781045},
	{SG5930, &UA2781786},
	{SG5931, &UA2786619},
	{SG5932, &UA2787418},
	{SG5933, &UA2789392},
	{SG5934, &UA2789542},
	{SG5935, &UA2789610},
	{SG5936, &UA2789928},
	{SG5937, &UA2790155},
	{SG5938, &UA2792147},
	{SG5939, &UA2793155},
	{SG5940, &UA2793768},
	{SG5941, &UA2810039},
	{SG5942, &UA2812642},
	{SG5943, &UA2815054},
	{SG5944, &UA2815261},
	{SG5945, &UA2815276},
	{SG5946, &UA2815403},
	{SG5947, &UA2816199},
	{SG5948, &UA2816473},
	{SG5949, &UA2816990},
	{SG5950, &UA2817348},
	{SG5951, &UA2817419},
	{SG5952, &UA2818022},
	{SG5953, &UA2821721},
	{SG5954, &UA2822422},
	{SG5955, &UA2823293},
	{SG5957, &UA2825570},
	{SG5958, &UA2826551},
	{SG5959, &UA2827010},
	{SG5960, &UA2833158},
	{SG5961, &UA2834312},
	{SG5962, &UA2840120},
	{SG5963, &UA2841845},
	{SG5964, &UA2848546},
	{SG5965, &UA2850413},
	{SG5966, &UA2850973},
	{SG5967, &UA2856167},
	{SG5968, &UA2860968},
	{SG5969, &UA2865848},
	{SG5970, &UA2866739},
	{SG5971, &UA2869698},
	{SG5972, &UA2878748},
	{SG5973, &UA2888930},
	{SG5974, &UA2889182},
	{SG5975, &UA2889219},
	{SG5976, &UA2891130},
	{SG5977, &UA2891980},
	{SG5978, &UA2892210},
	{SG5979, &UA2892963},
	{SG5980, &UA2933263},
	{SG5981, &UA2933441},
	{SG5982, &UA2933713},
	{SG5983, &UA2933724},
	{SG5984, &UA2933959},
	{SG5985, &UA2934141},
	{SG5986, &UA2934384},
	{SG5987, &UA2937222},
	{SG5988, &UA2949516},
	{SG5989, &UA2951660},
	{SG5990, &UA2952634},
	{SG5991, &UA2953089},
	{SG5992, &UA2953278},
	{SG5993, &UA2954274},
	{SG5994, &UA2955115},
	{SG5995, &UA2957463},
	{SG5996, &UA2959790},
	{SG5997, &UA2980843},
	{SG5998, &UA2981054},
	{SG5999, &UA2981327},
	{SG6000, &UA2981563},
	{SG6001, &UA2981664},
	{SG6002, &UA2982365},
	{SG6003, &UA2982433},
	{SG6004, &UA2982809},
	{SG6005, &UA2982981},
	{SG6006, &UA2984035},
	{SG6007, &UA2984145},
	{SG6008, &UA2984574},
	{SG6009, &UA2986061},
	{SG6010, &UA2986954},
	{SG6011, &UA2989589},
	{SG6012, &UA2996804},
	{SG6014, &UA2996842},
	{SG6015, &UA2997057},
	{SG6016, &UA2997447},
	{SG6017, &UA2997599},
	{SG6019, &UA2999236},
	{SG6020, &UA3001096},
	{SG6021, &UA3001703},
	{SG6022, &UA3001705},
	{SG6023, &UA3002475},
	{SG6024, &UA3005891},
	{SG6025, &UA3014490},
	{SG6026, &UA3020403},
	{SG6027, &UA3021446},
	{SG6028, &UA3026748},
	{SG6029, &UA3042042},
	{SG6030, &UA3042345},
	{SG6031, &UA3042454},
	{SG6032, &UA3042471},
	{SG6033, &UA3042498},
	{SG6034, &UA3042814},
	{SG6035, &UA3043212},
	{SG6036, &UA3043335},
	{SG6037, &UA3043662},
	{SG6038, &UA3043832},
	{SG6039, &UA3044501},
	{SG6040, &UA3048821},
	{SG6041, &UA3049092},
	{SG6042, &UA3049486},
	{SG6043, &UA3049682},
	{SG6044, &UA3049688},
	{SG6045, &UA3057210},
	{SG6046, &UA3057252},
	{SG6047, &UA3058022},
	{SG6048, &UA3058433},
	{SG6049, &UA3058563},
	{SG6050, &UA3058856},
	{SG6051, &UA3059220},
	{SG6052, &UA3059707},
	{SG6053, &UA3070194},
	{SG6054, &UA3070228},
	{SG6055, &UA3070848},
	{SG6056, &UA3071127},
	{SG6057, &UA3071264},
	{SG6058, &UA3071828},
	{SG6059, &UA3072231},
	{SG6060, &UA3072966},
	{SG6062, &UA3074202},
	{SG6063, &UA3074235},
	{SG6064, &UA3074460},
	{SG6066, &UA3076444},
	{SG6068, &UA3079825},
	{SG6069, &UA3085699},
	{SG6070, &UA3085990},
	{SG6071, &UA3086816},
	{SG6072, &UA3088165},
	{SG6073, &UA3088201},
	{SG6074, &UA3088861},
	{SG6075, &UA3091414},
	{SG6076, &UA3091617},
	{SG6077, &UA3092292},
	{SG6078, &UA3092409},
	{SG6081, &UA3098395},
	{SG6082, &UA3098527},
	{SG6083, &UA3098558},
	{SG6084, &UA3098674},
	{SG6085, &UA3098929},
	{SG6087, &UA3100061},
	{SG6088, &UA3102362},
	{SG6090, &UA3102675},
	{SG6091, &UA3103637},
	{SG6092, &UA3107061},
	{SG6093, &UA3107820},
	{SG6094, &UA3108611},
	{SG6095, &UA3121323},
	{SG6096, &UA3124109},
	{SG6097, &UA3124868},
	{SG6098, &UA3124942},
	{SG6099, &UA3125214},
	{SG6100, &UA3125414},
	{SG6209, &UA73936},
	{SG6232, &UA644305},
	{SG6237, &UA716762},
	{SG6239, &UA1952780},
	{SG6244, &UA2031687},
	{SG6246, &UA2493974},
	{SG6462, &UA719884},
	{SG6463, &UA753950},
	{SG6464, &UA762398},
	{SG6465, &UA1978019},
	{SG6466, &UA2028993},
	{SG6467, &UA2030464},
	{SG6468, &UA2030499},
	{SG6469, &UA2034265},
	{SG6470, &UA2183474},
	{SG6471, &UA2256971},
	{SG6472, &UA2361541},
	{SG6473, &UA2364729},
	{SG6474, &UA2373419},
	{SG6475, &UA2373651},
	{SG6476, &UA2373818},
	{SG6477, &UA2375498},
	{SG6478, &UA2377224},
	{SG6479, &UA2380688},
	{SG6480, &UA2382985},
	{SG6481, &UA2385046},
	{SG6482, &UA2385908},
	{SG6483, &UA2398019},
	{SG6484, &UA2419006},
	{SG6485, &UA2426518},
	{SG6486, &UA2547652},
	{SG6487, &UA2698301},
	{SG6488, &UA2810241},
	{SG6489, &UA2812773},
	{SG6600, &UA623420},
	{SG6601, &UA625641},
	{SG6602, &UA626849},
	{SG6603, &UA627553},
	{SG6604, &UA631732},
	{SG6605, &UA633625},
	{SG6606, &UA644895},
	{SG6607, &UA692381},
	{SG6608, &UA694741},
	{SG6609, &UA700664},
	{SG6610, &UA716789},
	{SG6611, &UA737197},
	{SG6612, &UA841615},
	{SG6613, &UA860550},
	{SG6614, &UA924768},
	{SG6615, &UA1934389},
	{SG6616, &UA1942911},
	{SG6617, &UA1996115},
	{SG6618, &UA1996633},
	{SG6619, &UA2023685},
	{SG6620, &UA2025331},
	{SG6621, &UA2028082},
	{SG6622, &UA2444638},
	{SG6623, &UA2545151},
	{SG6624, &UA2759251},
	{SG6625, &UA2985444},
	{SG6626, &UA3070415},
	{SG6713, &UA76678},
	{SG6714, &UA1939186},
	{SG6871, &UA48588},
	{SG6872, &UA236619},
	{SG6873, &UA368919},
	{SG6874, &UA369050},
	{SG6875, &UA369328},
	{SG6876, &UA622587},
	{SG6877, &UA622941},
	{SG6878, &UA627479},
	{SG6879, &UA653162},
	{SG6880, &UA879738},
	{SG6881, &UA1950326},
	{SG6882, &UA2235793},
	{SG6935, &UA40116},
	{SG6936, &UA70941},
	{SG6937, &UA75158},
	{SG6938, &UA76885},
	{SG6939, &UA224642},
	{SG6940, &UA678831},
	{SG6941, &UA679906},
	{SG6942, &UA2025336},
	{SG6961, &UA73082},
	{SG6964, &UA368880},
	{SG6966, &UA369495},
	{SG6969, &UA452260},
	{SG6970, &UA584235},
	{SG6972, &UA584929},
	{SG6974, &UA624936},
	{SG6976, &UA625591},
	{SG6978, &UA627664},
	{SG6981, &UA657084},
	{SG6983, &UA662291},
	{SG6985, &UA691718},
	{SG6986, &UA699442},
	{SG6987, &UA736514},
	{SG6988, &UA780344},
	{SG6989, &UA1934557},
	{SG6990, &UA1950123},
	{SG6991, &UA1962931},
	{SG6992, &UA2023966},
	{SG6993, &UA2026129},
	{SG6994, &UA2033752},
	{SG6995, &UA2496372},
	{SG6996, &UA2697219},
	{SG6997, &UA2722927},
	{SG6998, &UA2840988},
	{SG7037, &UA46168},
	{SG7038, &UA46173},
	{SG7039, &UA72084},
	{SG7040, &UA72799},
	{SG7041, &UA73138},
	{SG7042, &UA73906},
	{SG7043, &UA582795},
	{SG7044, &UA740284},
	{SG7045, &UA1900766},
	{SG7046, &UA1948975},
	{SG7047, &UA2020626},
	{SG7048, &UA2021172},
	{SG7049, &UA2022042},
	{SG7050, &UA2032816},
	{SG7051, &UA2815875},
	{SG7091, &UA721655},
	{SG7092, &UA2608842},
	{SG7093, &UA2616065},
	{SG7094, &UA2616196},
	{SG7095, &UA2616471},
	{SG7096, &UA2623019},
	{SG7101, &UA2634547},
	{SG7192, &UA249006},
	{SG7194, &UA575337},
	{SG7199, &UA653101},
	{SG7204, &UA2033922},
	{SG7297, &UA622882},
	{SG7298, &UA623949},
	{SG7300, &UA868405},
	{SG7301, &UA911994},
	{SG7302, &UA923959},
	{SG7303, &UA1936427},
	{SG7304, &UA1940356},
	{SG7305, &UA1943303},
	{SG7306, &UA2022058},
	{SG7308, &UA2033222},
	{SG7310, &UA2453301},
	{SG7311, &UA2453981},
	{SG7313, &UA2520515},
	{SG7314, &UA2796086},
	{SG7432, &UA73090},
	{SG7433, &UA366079},
	{SG7434, &UA369332},
	{SG7435, &UA369785},
	{SG7436, &UA653736},
	{SG7437, &UA1946324},
	{SG7481, &UA220038},
	{SG7482, &UA234946},
	{SG7486, &UA851465},
	{SG7489, &UA2019182},
	{SG7527, &UA249849},
	{SG7530, &UA329487},
	{SG7532, &UA354668},
	{SG7534, &UA355381},
	{SG7536, &UA355609},
	{SG7538, &UA369329},
	{SG7541, &UA637636},
	{SG7543, &UA639081},
	{SG7544, &UA644003},
	{SG7547, &UA693738},
	{SG7549, &UA2021066},
	{SG7555, &UA49258},
	{SG7556, &UA50020},
	{SG7557, &UA79282},
	{SG7558, &UA79294},
	{SG7559, &UA79283},
	{SG7560, &UA616494},
	{SG7561, &UA633506},
	{SG7562, &UA2021377},
	{SG7563, &UA3084868},
	{SG7564, &UA242284},
	{SG7565, &UA249425},
	{SG7566, &UA250420},
	{SG7569, &UA572557},
	{SG7571, &UA574159},
	{SG7575, &UA2020817},
	{SG7633, &UA69895},
	{SG7634, &UA72588},
	{SG7635, &UA211461},
	{SG7636, &UA211518},
	{SG7637, &UA219506},
	{SG7640, &UA252073},
	{SG7644, &UA335101},
	{SG7648, &UA655679},
	{SG7649, &UA686408},
	{SG7650, &UA715971},
	{SG7652, &UA738201},
	{SG7653, &UA2019531},
	{SG7654, &UA2019640},
	{SG7655, &UA2020183},
	{SG7656, &UA2025412},
	{SG7657, &UA2519409},
	{SG7672, &UA77922},
	{SG7673, &UA2145076},
	{SG7674, &UA623267},
	{SG7676, &UA641087},
	{SG7677, &UA646543},
	{SG7679, &UA655302},
	{SG7680, &UA678974},
	{SG7681, &UA682258},
	{SG7682, &UA750931},
	{SG7683, &UA799906},
	{SG7684, &UA863645},
	{SG7686, &UA869934},
	{SG7687, &UA876728},
	{SG7688, &UA2021237},
	{SG7689, &UA2684427},
	{SG7690, &UA2832519},
	{SG7691, &UA42578},
	{SG7692, &UA700195},
	{SG7693, &UA851004},
	{SG7694, &UA1919381},
	{SG7695, &UA2023424},
	{SG7696, &UA245262},
	{SG7697, &UA369265},
	{SG7698, &UA369333},
	{SG7699, &UA635148},
	{SG7700, &UA639159},
	{SG7701, &UA723370},
	{SG7702, &UA723442},
	{SG7703, &UA742266},
	{SG7704, &UA760360},
	{SG7705, &UA762989},
	{SG7706, &UA863364},
	{SG7707, &UA1936686},
	{SG7708, &UA2546621},
	{SG7709, &UA2686382},
	{SG7710, &UA3117739},
	{SG7721, &UA70756},
	{SG7722, &UA72930},
	{SG7724, &UA332345},
	{SG7725, &UA453128},
	{SG7726, &UA575067},
	{SG7727, &UA576647},
	{SG7728, &UA776941},
	{SG7730, &UA2021742},
	{SG7731, &UA2022549},
	{SG7732, &UA3102384},
	{SG7735, &UA73116},
	{SG7736, &UA350183},
	{SG7737, &UA351497},
	{SG7738, &UA366016},
	{SG7739, &UA575690},
	{SG7740, &UA579674},
	{SG7741, &UA624353},
	{SG7742, &UA628083},
	{SG7743, &UA632172},
	{SG7744, &UA637400},
	{SG7745, &UA653490},
	{SG7746, &UA659536},
	{SG7747, &UA668200},
	{SG7748, &UA685345},
	{SG7749, &UA686450},
	{SG7750, &UA703260},
	{SG7751, &UA708401},
	{SG7752, &UA710599},
	{SG7753, &UA718486},
	{SG7754, &UA739622},
	{SG7755, &UA751236},
	{SG7756, &UA752766},
	{SG7757, &UA764969},
	{SG7758, &UA767050},
	{SG7759, &UA774737},
	{SG7760, &UA781261},
	{SG7761, &UA786539},
	{SG7762, &UA813551},
	{SG7763, &UA822273},
	{SG7764, &UA824013},
	{SG7765, &UA1932343},
	{SG7766, &UA1939417},
	{SG7767, &UA1942282},
	{SG7768, &UA1945780},
	{SG7769, &UA1953069},
	{SG7770, &UA1992015},
	{SG7771, &UA2022145},
	{SG7772, &UA2022997},
	{SG7773, &UA2024821},
	{SG7774, &UA2024947},
	{SG7779, &UA2030786},
	{SG7783, &UA2033810},
	{SG7785, &UA2364807},
	{SG7788, &UA48749},
	{SG7790, &UA73012},
	{SG7791, &UA77827},
	{SG7792, &UA576289},
	{SG7793, &UA1918252},
	{SG7822, &UA49621},
	{SG7823, &UA622899},
	{SG7824, &UA781507},
	{SG7825, &UA2021399},
	{SG7861, &UA345828},
	{SG7864, &UA717818},
	{SG7865, &UA773533},
	{SG7867, &UA2025362},
	{SG7868, &UA2409654},
	{SG7930, &UA244397},
	{SG7932, &UA250507},
	{SG7946, &UA69268},
	{SG7947, &UA242229},
	{SG7953, &UA584016},
	{SG7954, &UA622943},
	{SG7982, &UA209005},
	{SG7983, &UA228715},
	{SG7984, &UA1895545},
	{SG7985, &UA2659190},
	{SG7986, &UA3097683},
	{SG8053, &UA639473},
	{SG8054, &UA694488},
	{SG8055, &UA708303},
	{SG8056, &UA725765},
	{SG8057, &UA1952819},
	{SG8058, &UA2393649},
	{SG8059, &UA2685559},
	{SG8060, &UA2979836},
	{SG8061, &UA219623},
	{SG8062, &UA2021589},
	{SG8076, &UA623080},
	{SG8077, &UA717781},
	{SG8078, &UA2809650},
	{SG8079, &UA578440},
	{SG8080, &UA664370},
	{SG8082, &UA2022833},
	{SG8167, &UA352711},
	{SG8170, &UA633401},
	{SG8172, &UA671464},
	{SG8176, &UA2022818},
	{SG8178, &UA2034582},
	{SG8230, &UA79247},
	{SG8269, &UA76851},
	{SG8270, &UA78594},
	{SG8278, &UA395738},
	{SG8288, &UA70149},
	{SG8289, &UA70545},
	{SG8290, &UA79088},
	{SG8291, &UA422367},
	{SG8292, &UA569703},
	{SG8293, &UA678764},
	{SG8294, &UA720932},
	{SG8295, &UA1912873},
	{SG8296, &UA1948187},
	{SG8297, &UA1965680},
	{SG8298, &UA2023625},
	{SG8299, &UA2023825},
	{SG8300, &UA2024969},
	{SG8301, &UA2030255},
	{SG8302, &UA2375441},
	{SG8303, &UA2933451},
	{SG8357, &UA623159},
	{SG8358, &UA624877},
	{SG8359, &UA636756},
	{SG8360, &UA658321},
	{SG8361, &UA674327},
	{SG8362, &UA716481},
	{SG8363, &UA802938},
	{SG8364, &UA821257},
	{SG8365, &UA903384},
	{SG8366, &UA1961688},
	{SG8367, &UA2034611},
	{SG8369, &UA329459},
	{SG8371, &UA379152},
	{SG8383, &UA256763},
	{SG8384, &UA623913},
	{SG8385, &UA644062},
	{SG8386, &UA1937212},
	{SG8440, &UA73627},
	{SG8441, &UA640074},
	{SG8442, &UA455359},
	{SG8443, &UA1997071},
	{SG8444, &UA1997684},
	{SG8445, &UA2026700},
	{SG8446, &UA2220898},
	{SG8447, &UA2221630},
	{SG8457, &UA71456},
	{SG8465, &UA2020883},
	{SG8487, &UA73145},
	{SG8488, &UA329495},
	{SG8495, &UA207069},
	{SG8498, &UA631011},
	{SG8499, &UA2020208},
	{SG8501, &UA41403},
	{SG8502, &UA41406},
	{SG8503, &UA41410},
	{SG8504, &UA74024},
	{SG8505, &UA210289},
	{SG8506, &UA220052},
	{SG8507, &UA333623},
	{SG8508, &UA563536},
	{SG8509, &UA800926},
	{SG8525, &UA70158},
	{SG8526, &UA77105},
	{SG8527, &UA78763},
	{SG8528, &UA78901},
	{SG8529, &UA361817},
	{SG8530, &UA368963},
	{SG8531, &UA575539},
	{SG8532, &UA576933},
	{SG8533, &UA622543},
	{SG8534, &UA629133},
	{SG8535, &UA634695},
	{SG8536, &UA661363},
	{SG8537, &UA665382},
	{SG8538, &UA701894},
	{SG8539, &UA780221},
	{SG8540, &UA2022465},
	{SG8541, &UA2023068},
	{SG8542, &UA2023459},
	{SG8552, &UA2684294},
	{SG8571, &UA252009},
	{SG8572, &UA371284},
	{SG8573, &UA693411},
	{SG8574, &UA2256934},
	{SG8579, &UA47711},
	{SG8580, &UA70405},
	{SG8581, &UA71659},
	{SG8582, &UA73332},
	{SG8583, &UA73861},
	{SG8584, &UA74946},
	{SG8585, &UA78782},
	{SG8586, &UA635750},
	{SG8589, &UA2020089},
	{SG8590, &UA2022359},
	{SG8591, &UA257670},
	{SG8592, &UA633899},
	{SG8616, &UA369124},
	{SG8617, &UA369093},
	{SG8619, &UA397779},
	{SG8620, &UA654665},
	{SG8625, &UA722861},
	{SG8627, &UA751033},
	{SG8628, &UA1930011},
	{SG8629, &UA1940106},
	{SG8630, &UA1944902},
	{SG8631, &UA1947037},
	{SG8632, &UA2023112},
	{SG8633, &UA2027551},
	{SG8634, &UA2032611},
	{SG8635, &UA2034360},
	{SG8636, &UA2544104},
	{SG8637, &UA2720619},
	{SG8638, &UA2818034},
	{SG8639, &UA49600},
	{SG8640, &UA247163},
	{SG8641, &UA640815},
	{SG8642, &UA693908},
	{SG8653, &UA369148},
	{SG8655, &UA572642},
	{SG8657, &UA636268},
	{SG8659, &UA822702},
	{SG8661, &UA2385007},
	{SG8662, &UA2463556},
	{SG8663, &UA2508194},
	{SG8666, &UA2656732},
	{SG8667, &UA452921},
	{SG8668, &UA689588},
	{SG8669, &UA711152},
	{SG8670, &UA2336355},
	{SG8671, &UA2810246},
	{SG8684, &UA250424},
	{SG8685, &UA339251},
	{SG8692, &UA2021730},
	{SG8700, &UA212825},
	{SG8784, &UA250308},
	{SG8786, &UA251317},
	{SG8787, &UA616493},
	{SG8789, &UA622339},
	{SG8790, &UA1906325},
	{SG8791, &UA2021491},
	{SG8797, &UA2019414},
	{SG8809, &UA69502},
	{SG8810, &UA349410},
	{SG8811, &UA353621},
	{SG8812, &UA370211},
	{SG8813, &UA564662},
	{SG8814, &UA2019299},
	{SG8815, &UA2019553},
	{SG8825, &UA73277},
	{SG8826, &UA76337},
	{SG8827, &UA359942},
	{SG8828, &UA369189},
	{SG8829, &UA609175},
	{SG8830, &UA629942},
	{SG8831, &UA636963},
	{SG8832, &UA644344},
	{SG8833, &UA646489},
	{SG8834, &UA656000},
	{SG8835, &UA705550},
	{SG8836, &UA707538},
	{SG8837, &UA808616},
	{SG8838, &UA887349},
	{SG8839, &UA2020687},
	{SG8840, &UA2025675},
	{SG8841, &UA2034292},
	{SG8842, &UA2324166},
	{SG8843, &UA2397573},
	{SG8844, &UA2482226},
	{SG8845, &UA2494753},
	{SG8846, &UA2520807},
	{SG8847, &UA2582975},
	{SG8848, &UA2444510},
	{SG8861, &UA48898},
	{SG8862, &UA360159},
	{SG8866, &UA579190},
	{SG8867, &UA622574},
	{SG8868, &UA252238},
	{SG8869, &UA628900},
	{SG8870, &UA1995871},
	{SG8871, &UA2032476},
	{SG8872, &UA2235252},
	{SG8873, &UA2469782},
	{SG8874, &UA2659194},
	{SG8875, &UA2019808},
	{SG8876, &UA452888},
	{SG8879, &UA634071},
	{SG8880, &UA661548},
	{SG8882, &UA694827},
	{SG8884, &UA752524},
	{SG8886, &UA781741},
	{SG8888, &UA870719},
	{SG8889, &UA1996326},
	{SG8892, &UA2025835},
	{SG8894, &UA2029605},
	{SG8897, &UA2029775},
	{SG8898, &UA2033366},
	{SG8899, &UA2033596},
	{SG8900, &UA2222201},
	{SG8901, &UA2441798},
	{SG8902, &UA2789193},
	{SG8903, &UA2789260},
	{SG8904, &UA3098890},
	{SG8945, &UA70648},
	{SG8946, &UA73340},
	{SG8947, &UA251967},
	{SG8948, &UA371320},
	{SG8949, &UA379597},
	{SG8950, &UA575028},
	{SG8951, &UA622955},
	{SG8952, &UA629343},
	{SG8953, &UA631605},
	{SG8954, &UA633575},
	{SG8955, &UA634166},
	{SG8956, &UA644520},
	{SG8957, &UA775229},
	{SG8958, &UA800172},
	{SG8959, &UA800695},
	{SG8960, &UA829364},
	{SG8961, &UA1942372},
	{SG8962, &UA1988637},
	{SG8963, &UA2020143},
	{SG8964, &UA2031767},
	{SG8965, &UA2033418},
	{SG8966, &UA2643255},
	{SG8967, &UA2684666},
	{SG8968, &UA2721394},
	{SG8969, &UA2786479},
	{SG8970, &UA397352},
	{SG8971, &UA643195},
	{SG8972, &UA2024958},
	{SG8973, &UA349263},
	{SG8974, &UA787460},
	{SG8982, &UA77401},
	{SG8984, &UA677302},
	{SG8985, &UA1942969},
	{SG8986, &UA2027328},
	{SG8987, &UA2463017},
	{SG8988, &UA693795},
	{SG9001, &UA421760},
	{SG9002, &UA743013},
	{SG9003, &UA2758868},
	{SG9014, &UA349001},
	{SG9015, &UA351681},
	{SG9016, &UA625196},
	{SG9017, &UA640949},
	{SG9018, &UA698895},
	{SG9019, &UA627308},
	{SG9020, &UA768246},
	{SG9021, &UA1980888},
	{SG9040, &UA74622},
	{SG9041, &UA236647},
	{SG9042, &UA341046},
	{SG9043, &UA669993},
	{SG9044, &UA2019424},
	{SG9055, &UA1901935},
	{SG9056, &UA250337},
	{SG9057, &UA354148},
	{SG9058, &UA355701},
	{SG9059, &UA368849},
	{SG9060, &UA622458},
	{SG9061, &UA635584},
	{SG9063, &UA737590},
	{SG9066, &UA754968},
	{SG9068, &UA806152},
	{SG9070, &UA817559},
	{SG9072, &UA845543},
	{SG9075, &UA70927},
	{SG9076, &UA568094},
	{SG9077, &UA801068},
	{SG9078, &UA2023312},
	{SG9104, &UA235488},
	{SG9105, &UA565773},
	{SG9106, &UA633691},
	{SG9107, &UA902394},
	{SG9108, &UA924896},
	{SG9109, &UA2242338},
	{SG9128, &UA73140},
	{SG9129, &UA209142},
	{SG9130, &UA642856},
	{SG9132, &UA2019491},
	{SG9133, &UA2021277},
	{SG9188, &UA232243},
	{SG9189, &UA2020099},
	{SG9190, &UA2020185},
	{SG9191, &UA2020368},
	{SG9192, &UA701496},
	{SG9193, &UA701880},
	{SG9194, &UA743248},
	{SG9195, &UA803819},
	{SG9199, &UA2024065},
	{SG9201, &UA2027205},
	{SG9204, &UA2823086},
	{SG9205, &UA576662},
	{SG9206, &UA1940584},
	{SG9228, &UA2024566},
	{SG9229, &UA2026795},
	{SG9230, &UA2033083},
	{SG9232, &UA746957},
	{SG9233, &UA1958527},
	{SG9234, &UA2833413},
	{SG9274, &UA2023205},
	{SG9283, &UA635255},
	{SG9284, &UA699249},
	{SG9285, &UA2024252},
	{SG9288, &UA209786},
	{SG9289, &UA572821},
	{SG9290, &UA633188},
	{SG9293, &UA252545},
	{SG9294, &UA359548},
	{SG9295, &UA370623},
	{SG9296, &UA622969},
	{SG9310, &UA1947273},
	{SG9311, &UA2025244},
	{SG9312, &UA2025378},
	{SG9313, &UA2393554},
	{SG9354, &UA329037},
	{SG9355, &UA2640952},
	{SG9380, &UA3056956},
	{SG9397, &UA2021046},
	{SG9430, &UA1969948},
	{SG9431, &UA2339882},
	{SG9445, &UA357279},
	{SG9450, &UA2022639},
	{SG9452, &UA2024645},
	{SG9466, &UA49737},
	{SG9467, &UA628822},
	{SG9486, &UA452907},
	{SG9487, &UA623245},
	{SG9488, &UA681156},
	{SG9489, &UA693221},
	{SG9490, &UA796980},
	{SG9491, &UA799835},
	{SG9492, &UA852718},
	{SG9493, &UA862941},
	{SG9494, &UA895430},
	{SG9495, &UA2023330},
	{SG9496, &UA2028985},
	{SG9497, &UA3117455},
	{SG9503, &UA74695},
	{SG9506, &UA1969609},
	{SG9507, &UA2416755},
	{SG9508, &UA251612},
	{SG9509, &UA665591},
	{SG9510, &UA2809578},
	{SG9511, &UA354831},
	{SG9512, &UA1903921},
	{SG9513, &UA2020017},
	{SG9518, &UA627408},
	{SG9519, &UA923988},
	{SG9557, &UA623908},
	{SG9559, &UA690333},
	{SG9560, &UA874536},
	{SG9562, &UA2554127},
	{SG9564, &UA2933268},
	{SG9567, &UA209017},
	{SG9568, &UA209803},
	{SG9569, &UA241287},
	{SG9570, &UA693815},
	{SG9571, &UA2019692},
	{SG9572, &UA2019803},
	{SG9573, &UA2021045},
	{SG9574, &UA2022284},
	{SG9614, &UA569122},
	{SG9615, &UA753665},
	{SG9616, &UA1909318},
	{SG9631, &UA625258},
	{SG9632, &UA2029457},
	{SG9659, &UA723310},
	{SG9660, &UA1930535},
	{SG9661, &UA2023937},
	{SG9662, &UA2032969},
	{SG9663, &UA2335286},
	{SG9664, &UA2427992},
	{SG9666, &UA2686274},
	{SG9686, &UA251975},
	{SG9687, &UA575300},
	{SG9688, &UA1906707},
	{SG9706, &UA864192},
	{SG9707, &UA1971691},
	{SG9708, &UA1978975},
	{SG9709, &UA2033701},
	{SG9710, &UA2594512},
	{SG9711, &UA2815584},
	{SG9713, &UA2860291},
	{SG9715, &UA73612},
	{SG9742, &UA595682},
	{SG9743, &UA659625},
	{SG9744, &UA684084},
	{SG9745, &UA2024642},
	{SG9770, &UA351026},
	{SG9771, &UA1939052},
	{SG9801, &UA213501},
	{SG9802, &UA568711},
	{SG9803, &UA2019482},
	{SG9804, &UA2020049},
	{SG9805, &UA2020233},
	{SG9806, &UA2020897},
	{SG9807, &UA2409819},
	{SG9826, &UA236666},
	{SG9827, &UA2019374},
	{SG9828, &UA2032772},
	{SG9858, &UA627600},
	{SG9859, &UA2033399},
	{SG9862, &UA2202582},
	{SG9875, &UA851598},
	{SG9877, &UA1937843},
	{SG9878, &UA1941003},
	{SG9879, &UA2028135},
	{SG9897, &UA412584},
	{SG9898, &UA613202},
	{SG9899, &UA625890},
	{SG9900, &UA632075},
	{SG9901, &UA910447},
	{SG9902, &UA2236726},
	{SG9903, &UA2425715},
	{SG9904, &UA2567739},
	{SG9905, &UA2883805},
	{SG9907, &UA49253},
	{SG9908, &UA73543},
	{SG9909, &UA654409},
	{SG9910, &UA672177},
	{SG9912, &UA694695},
	{SG9913, &UA787529},
	{SG9915, &UA1965827},
	{SG9916, &UA1973565},
	{SG9917, &UA2338470},
	{SG9919, &UA2643530},
	{SG9932, &UA209779},
	{SG9933, &UA337612},
	{SG9934, &UA1895103},
	{SG9935, &UA2033136},
	{SG9938, &UA73301},
	{SG9939, &UA1993533},
	{SG9972, &UA209819},
	{SG9973, &UA338044},
	{SG9974, &UA562747},
	{SG9975, &UA567468},
	{SG9976, &UA2020161},
	{SG9977, &UA2020868},
	{SG9989, &UA652623},
	{SG9990, &UA2022828},
	{SG9991, &UA2024526},
	{SG9992, &UA2024668},
	{SG9998, &UA786005},
	{SG9999, &UA1956042},
	{SG10003, &UA642892},
	{SG10004, &UA2020581},
	{SG10010, &UA251244},
	{SG10011, &UA576244},
	{SG10012, &UA2022691},
	{SG10013, &UA2034488},
	{SG10031, &UA2986902},
	{SG10073, &UA2256949},
	{SG10074, &UA2705608},
	{SG10075, &UA3085858},
	{SG10081, &UA2328254},
	{SG10090, &UA235501},
	{SG10091, &UA571472},
	{SG10092, &UA2753816},
	{SG10094, &UA2867353},
	{SG10095, &UA233415},
	{SG10096, &UA562055},
	{SG10097, &UA861092},
	{SG10098, &UA2384624},
	{SG10102, &UA236780},
	{SG10103, &UA2019307},
	{SG10104, &UA77886},
	{SG10105, &UA361889},
	{SG10106, &UA662795},
	{SG10107, &UA694619},
	{SG10108, &UA746442},
	{SG10109, &UA829287},
	{SG10110, &UA1983579},
	{SG10111, &UA1997030},
	{SG10112, &UA2024152},
	{SG10113, &UA2222430},
	{SG10131, &UA79601},
	{SG10132, &UA699864},
	{SG10133, &UA1908802},
	{SG10134, &UA2021652},
	{SG10135, &UA582349},
	{SG10136, &UA640376},
	{SG10137, &UA2023549},
	{SG10138, &UA2027052},
	{SG10142, &UA634218},
	{SG10143, &UA636192},
	{SG10144, &UA901801},
	{SG10145, &UA1979422},
	{SG10146, &UA2027848},
	{SG10147, &UA2889018},
	{SG10151, &UA624897},
	{SG10152, &UA1938460},
	{SG10153, &UA1969947},
	{SG10154, &UA2024812},
	{SG10155, &UA2555628},
	{SG10156, &UA2933926},
	{SG10157, &UA2934090},
	{SG10172, &UA623834},
	{SG10173, &UA1946265},
	{SG10174, &UA2172968},
	{SG10175, &UA2551951},
	{SG10176, &UA2827813},
	{SG10178, &UA630530},
	{SG10184, &UA2431271},
	{SG10185, &UA624515},
	{SG10186, &UA1959007},
	{SG10188, &UA2515246},
	{SG10189, &UA2853770},
	{SG10197, &UA626887},
	{SG10198, &UA737694},
	{SG10199, &UA1941195},
	{SG10200, &UA2026238},
	{SG10203, &UA2416067},
	{SG10209, &UA2027158},
	{SG10210, &UA2024071},
	{SG10211, &UA350971},
	{SG10212, &UA351377},
	{SG10213, &UA363054},
	{SG10214, &UA627653},
	{SG10215, &UA632317},
	{SG10216, &UA1993659},
	{SG10217, &UA2206643},
	{SG10218, &UA2652719},
	{SG10223, &UA630978},
	{SG10224, &UA2025202},
	{SG10227, &UA2189930},
	{SG10228, &UA2500864},
	{SG10229, &UA878789},
	{SG10230, &UA2775660},
	{SG10231, &UA2235822},
	{SG10254, &UA2355666},
	{SG10255, &UA2600783},
	{SG10256, &UA2695193},
	{SG10257, &UA2558399},
	{SG10262, &UA623432},
	{SG10263, &UA731527},
	{SG10264, &UA1901859},
	{SG10265, &UA2552347},
	{SG10266, &UA3086998},
	{SG10268, &UA2440001},
	{SG10271, &UA79638},
	{SG10274, &UA369193},
	{SG10275, &UA630642},
	{SG10276, &UA659724},
	{SG10277, &UA2020559},
	{SG10278, &UA2034394},
	{SG10279, &UA2256921},
	{SG10280, &UA2987898},
	{SG10281, &UA710976},
	{SG10282, &UA847784},
	{SG10284, &UA2023745},
	{SG10286, &UA2025339},
	{SG10289, &UA2256940},
	{SG10290, &UA2375814},
	{SG10291, &UA2375957},
	{SG10292, &UA2776013},
	{SG10297, &UA2709184},
	{SG10298, &UA2469199},
	{SG10325, &UA2558182},
	{SG10326, &UA2956086},
	{SG10330, &UA227576},
	{SG10331, &UA659746},
	{SG10341, &UA625056},
	{SG10342, &UA2024009},
	{SG10343, &UA2524957},
	{SG10347, &UA2421107},
	{SG10349, &UA2507276},
	{SG10351, &UA2581946},
	{SG10352, &UA2747533},
	{SG10353, &UA2996957},
	{SG10360, &UA1960411},
	{SG10361, &UA1960896},
	{SG10373, &UA623729},
	{SG10374, &UA625728},
	{SG10375, &UA627195},
	{SG10376, &UA635305},
	{SG10377, &UA714859},
	{SG10378, &UA1942815},
	{SG10379, &UA2562105},
	{SG10380, &UA2562381},
	{SG10383, &UA212784},
	{SG10384, &UA248876},
	{SG10385, &UA2603083},
	{SG10386, &UA2432255},
	{SG10403, &UA436960},
	{SG10409, &UA355743},
	{SG10410, &UA371271},
	{SG10411, &UA625343},
	{SG10412, &UA899634},
	{SG10413, &UA923831},
	{SG10414, &UA1979425},
	{SG10416, &UA599681},
	{SG10442, &UA2512187},
	{SG10443, &UA2660074},
	{SG10444, &UA2952393},
	{SG10445, &UA2237762},
	{SG10454, &UA2843691},
	{SG10464, &UA576303},
	{SG10467, &UA2658977},
	{SG10476, &UA2019429},
	{SG10477, &UA605699},
	{SG10478, &UA627688},
	{SG10479, &UA640343},
	{SG10480, &UA905085},
	{SG10481, &UA1955536},
	{SG10482, &UA2031996},
	{SG10485, &UA353977},
	{SG10486, &UA768234},
	{SG10487, &UA2031985},
	{SG10493, &UA210694},
	{SG10495, &UA396394},
	{SG10501, &UA571709},
	{SG10502, &UA2020702},
	{SG10529, &UA2027419},
	{SG10530, &UA2032674},
	{SG10531, &UA2744095},
	{SG10542, &UA455129},
	{SG10543, &UA626183},
	{SG10544, &UA897826},
	{SG10545, &UA1993513},
	{SG10546, &UA1996913},
	{SG10547, &UA1997126},
	{SG10549, &UA2030795},
	{SG10550, &UA2230692},
	{SG10552, &UA655729},
	{SG10563, &UA626013},
	{SG10564, &UA757895},
	{SG10568, &UA786790},
	{SG10581, &UA1992354},
	{SG10589, &UA742813},
	{SG10592, &UA73496},
	{SG10593, &UA213510},
	{SG10601, &UA209083},
	{SG10602, &UA2019433},
	{SG10631, &UA2032026},
	{SG10632, &UA2691527},
	{SG10635, &UA3100095},
	{SG10638, &UA728062},
	{SG10639, &UA2362398},
	{SG10641, &UA2600223},
	{SG10645, &UA359363},
	{SG10647, &UA369059},
	{SG10648, &UA379997},
	{SG10649, &UA424761},
	{SG10650, &UA900462},
	{SG10651, &UA900705},
	{SG10652, &UA1990838},
	{SG10653, &UA2033591},
	{SG10656, &UA410632},
	{SG10657, &UA633966},
	{SG10659, &UA904805},
	{SG10660, &UA2032940},
	{SG10662, &UA2960832},
	{SG10681, &UA2032710},
	{SG10688, &UA659722},
	{SG10690, &UA756638},
	{SG10691, &UA905096},
	{SG10692, &UA2032063},
	{SG10693, &UA2032811},
	{SG10696, &UA716712},
	{SG10698, &UA639287},
	{SG10699, &UA3117105},
	{SG10700, &UA2797284},
	{SG10701, &UA629622},
	{SG10702, &UA2023245},
	{SG10703, &UA2667546},
	{SG10705, &UA361824},
	{SG10706, &UA426955},
	{SG10708, &UA613283},
	{SG10710, &UA623054},
	{SG10711, &UA924909},
	{SG10713, &UA2206534},
	{SG10715, &UA2607058},
	{SG10732, &UA727501},
	{SG10733, &UA2558919},
	{SG10734, &UA2764044},
	{SG10735, &UA2952303},
	{SG10736, &UA3044674},
	{SG10741, &UA2487104},
	{SG10743, &UA697045},
	{SG10744, &UA2022877},
	{SG10750, &UA630272},
	{SG10751, &UA1940174},
	{SG10754, &UA2527712},
	{SG10755, &UA2553995},
	{SG10756, &UA2578713},
	{SG10757, &UA2934364},
	{SG10758, &UA567284},
	{SG10759, &UA2683629},
	{SG10760, &UA2722459},
	{SG10761, &UA217093},
	{SG10763, &UA77849},
	{SG10764, &UA1962601},
	{SG10765, &UA2021415},
	{SG10766, &UA2575309},
	{SG10767, &UA2587865},
	{SG10768, &UA3042431},
	{SG10769, &UA329851},
	{SG10774, &UA363786},
	{SG10775, &UA627070},
	{SG10776, &UA629559},
	{SG10777, &UA1967566},
	{SG10778, &UA2031436},
	{SG10779, &UA2532462},
	{SG10780, &UA2892977},
	{SG10781, &UA2477651},
	{SG10794, &UA1959352},
	{SG10795, &UA2032109},
	{SG10796, &UA2033837},
	{SG10797, &UA2543045},
	{SG10811, &UA331414},
	{SG10812, &UA421497},
	{SG10813, &UA569733},
	{SG10814, &UA458411},
	{SG10815, &UA609119},
	{SG10816, &UA626751},
	{SG10818, &UA2032495},
	{SG10821, &UA2563353},
	{SG10823, &UA896863},
	{SG10824, &UA3012466},
	{SG10852, &UA1961379},
	{SG10853, &UA2031695},
	{SG10856, &UA623138},
	{SG10857, &UA797045},
	{SG10860, &UA2527540},
	{SG10862, &UA3065065},
	{SG10865, &UA584383},
	{SG10866, &UA614309},
	{SG10867, &UA627474},
	{SG10868, &UA1995420},
	{SG10869, &UA2231051},
	{SG10870, &UA2669100},
	{SG10875, &UA2401787},
	{SG10881, &UA2753497},
	{SG10882, &UA2783653},
	{SG10884, &UA691417},
	{SG10885, &UA708197},
	{SG10886, &UA1969763},
	{SG10887, &UA2023226},
	{SG10890, &UA2796274},
	{SG10893, &UA900394},
	{SG10894, &UA2753418},
	{SG10901, &UA698480},
	{SG10902, &UA2702404},
	{SG10906, &UA2958002},
	{SG10912, &UA2738278},
	{SG10913, &UA2599640},
	{SG10915, &UA2750068},
	{SG10916, &UA772303},
	{SG10917, &UA2705678},
	{SG10926, &UA626619},
	{SG10927, &UA899116},
	{SG10928, &UA1989706},
	{SG10929, &UA2943815},
	{SG10936, &UA642631},
	{SG10938, &UA2019823},
	{SG10941, &UA2235142},
	{SG10947, &UA2502874},
	{SG10948, &UA2552852},
	{SG10949, &UA2578105},
	{SG10950, &UA2728196},
	{SG10953, &UA2737854},
	{SG10964, &UA330606},
	{SG10965, &UA337994},
	{SG10966, &UA2213216},
	{SG10968, &UA2537940},
	{SG10973, &UA459145},
	{SG10974, &UA459148},
	{SG10975, &UA459150},
	{SG10976, &UA459157},
	{SG10977, &UA1955940},
	{SG10979, &UA571306},
	{SG10980, &UA1901650},
	{SG10981, &UA2019170},
	{SG10982, &UA2020123},
	{SG10983, &UA2021157},
	{SG10994, &UA674205},
	{SG10995, &UA679800},
	{SG11000, &UA2021253},
	{SG11001, &UA585067},
	{SG11002, &UA1994891},
	{SG11012, &UA210704},
	{SG11024, &UA2945606},
	{SG11026, &UA368944},
	{SG11027, &UA454465},
	{SG11028, &UA603563},
	{SG11029, &UA631024},
	{SG11030, &UA1964694},
	{SG11031, &UA2033725},
	{SG11032, &UA2605745},
	{SG11040, &UA2033940},
	{SG11041, &UA2480447},
	{SG11042, &UA2543185},
	{SG11043, &UA2658559},
	{SG11044, &UA2815279},
	{SG11045, &UA2850046},
	{SG11046, &UA2993649},
	{SG11072, &UA2833731},
	{SG11073, &UA2877490},
	{SG11074, &UA3108197},
	{SG11075, &UA2665836},
	{SG11076, &UA2879027},
	{SG11077, &UA3005425},
	{SG11078, &UA3008145},
	{SG11080, &UA738391},
	{SG11089, &UA742611},
	{SG11098, &UA2768402},
	{SG11102, &UA218947},
	{SG11103, &UA830922},
	{SG11104, &UA2027614},
	{SG11105, &UA2028307},
	{SG11106, &UA2520672},
	{SG11107, &UA2544244},
	{SG11115, &UA2815387},
	{SG11116, &UA2937769},
	{SG11117, &UA631920},
	{SG11118, &UA677917},
	{SG11119, &UA1952892},
	{SG11120, &UA1995597},
	{SG11121, &UA2605842},
	{SG11122, &UA685500},
	{SG11127, &UA2024328},
	{SG11135, &UA2532255},
	{SG11137, &UA2850754},
	{SG11140, &UA1982895},
	{SG11141, &UA1995395},
	{SG11144, &UA1977705},
	{SG11145, &UA1959022},
	{SG11147, &UA2026769},
	{SG11148, &UA2415816},
	{SG11149, &UA1994030},
	{SG11151, &UA2026313},
	{SG11153, &UA2028189},
	{SG11154, &UA2032271},
	{SG11155, &UA2817313},
	{SG11156, &UA3117679},
	{SG11158, &UA208933},
	{SG11159, &UA627128},
	{SG11161, &UA2506560},
	{SG11167, &UA370501},
	{SG11169, &UA833072},
	{SG11171, &UA2413277},
	{SG11172, &UA2644480},
	{SG11174, &UA803600},
	{SG11175, &UA3003047},
	{SG11188, &UA2371681},
	{SG11192, &UA2697470},
	{SG11197, &UA734010},
	{SG11198, &UA751741},
	{SG11199, &UA1957944},
	{SG11200, &UA1964768},
	{SG11202, &UA2031766},
	{SG11203, &UA2441738},
	{SG11204, &UA3088518},
	{SG11206, &UA359365},
	{SG11208, &UA368769},
	{SG11210, &UA2021813},
	{SG11211, &UA2518872},
	{SG11213, &UA636864},
	{SG11214, &UA2029267},
	{SG11215, &UA878143},
	{SG11216, &UA2022952},
	{SG11217, &UA216285},
	{SG11218, &UA668270},
	{SG11219, &UA735242},
	{SG11222, &UA2031651},
	{SG11227, &UA2737580},
	{SG11230, &UA622682},
	{SG11231, &UA634876},
	{SG11233, &UA2753734},
	{SG11236, &UA2642489},
	{SG11239, &UA2496187},
	{SG11241, &UA2713454},
	{SG11242, &UA2469389},
	{SG11251, &UA675030},
	{SG11252, &UA2708332},
	{SG11267, &UA2957618},
	{SG11270, &UA1956057},
	{SG11272, &UA2026025},
	{SG11274, &UA3117375},
	{SG11279, &UA1916747},
	{SG11281, &UA661413},
	{SG11282, &UA899145},
	{SG11283, &UA2033247},
	{SG11284, &UA2961340},
	{SG11288, &UA2533209},
	{SG11290, &UA2607510},
	{SG11292, &UA1935893},
	{SG11295, &UA2512725},
	{SG11296, &UA3021199},
	{SG11300, &UA333683},
	{SG11301, &UA2457292},
	{SG11302, &UA2504131},
	{SG11305, &UA220495},
	{SG11307, &UA2714102},
	{SG11308, &UA2256966},
	{SG11317, &UA2026515},
	{SG11321, &UA770210},
	{SG11322, &UA2494710},
	{SG11328, &UA1959546},
	{SG11329, &UA2022519},
	{SG11330, &UA331248},
	{SG11331, &UA2021495},
	{SG11338, &UA605422},
	{SG11339, &UA747568},
	{SG11340, &UA748693},
	{SG11341, &UA2034617},
	{SG11342, &UA2508287},
	{SG11343, &UA2879918},
	{SG11344, &UA369400},
	{SG11346, &UA2023134},
	{SG11347, &UA2754733},
	{SG11348, &UA2796509},
	{SG11349, &UA2596588},
	{SG11356, &UA244011},
	{SG11364, &UA2582117},
	{SG11366, &UA696065},
	{SG11367, &UA1944934},
	{SG11368, &UA2025978},
	{SG11369, &UA2034632},
	{SG11370, &UA3089656},
	{SG11374, &UA2256939},
	{SG11375, &UA2579882},
	{SG11377, &UA2556801},
	{SG11379, &UA2338453},
	{SG11380, &UA2578195},
	{SG11381, &UA2835749},
	{SG11382, &UA2702435},
	{SG11387, &UA2892676},
	{SG11388, &UA2958638},
	{SG11389, &UA2029466},
	{SG11391, &UA2028556},
	{SG11401, &UA2510034},
	{SG11402, &UA3107710},
	{SG11407, &UA3034487},
	{SG11413, &UA247879},
	{SG11414, &UA220040},
	{SG11415, &UA453165},
	{SG11416, &UA630187},
	{SG11424, &UA659668},
	{SG11425, &UA2462050},
	{SG11428, &UA1982160},
	{SG11429, &UA2997549},
	{SG11431, &UA2025921},
	{SG11432, &UA2643478},
	{SG11433, &UA2721812},
	{SG11439, &UA2524682},
	{SG11440, &UA2028653},
	{SG11441, &UA2022869},
	{SG11442, &UA2518610},
	{SG11443, &UA2815674},
	{SG11444, &UA2737554},
	{SG11445, &UA3064895},
	{SG11446, &UA837965},
	{SG11447, &UA2662642},
	{SG11448, &UA3058385},
	{SG11451, &UA724481},
	{SG11452, &UA1902311},
	{SG11456, &UA2707655},
	{SG11459, &UA2019705},
	{SG11460, &UA2944091},
	{SG11461, &UA3016734},
	{SG11471, &UA2205477},
	{SG11475, &UA2668222},
	{SG11478, &UA2602287},
	{SG11479, &UA2019590},
	{SG11488, &UA2293838},
	{SG11489, &UA2827867},
	{SG11490, &UA2661049},
	{SG11491, &UA2704450},
	{SG11492, &UA2863731},
	{SG11493, &UA3105866},
	{SG11494, &UA3120304},
	{SG11501, &UA902205},
	{SG11503, &UA2872153},
	{SG11504, &UA2963686},
	{SG11505, &UA3059898},
	{SG11515, &UA2782857},
	{SG11528, &UA2220365},
	{SG11529, &UA2870354},
	{SG11539, &UA2029760},
	{SG11540, &UA2029774},
	{SG11541, &UA1973063},
	{SG11542, &UA2030530},
	{SG11544, &UA736089},
	{SG11545, &UA1895791},
	{SG11546, &UA331483},
	{SG11547, &UA563309},
	{SG11548, &UA2020209},
	{SG11549, &UA630373},
	{SG11550, &UA2031876},
	{SG11551, &UA761421},
	{SG11552, &UA2021394},
	{SG11553, &UA2024605},
	{SG11554, &UA2412613},
	{SG11555, &UA2787221},
	{SG11557, &UA3061433},
	{SG11558, &UA817011},
	{SG11559, &UA2025320},
	{SG11560, &UA2027661},
	{SG11563, &UA2165722},
	{SG11564, &UA2722491},
	{SG11567, &UA2751477},
	{SG11570, &UA2425277},
	{SG11571, &UA2558461},
	{SG11572, &UA1940621},
	{SG11576, &UA1952991},
	{SG11577, &UA2027865},
	{SG11579, &UA2483519},
	{SG11585, &UA2234222},
	{SG11587, &UA2462056},
	{SG11590, &UA219589},
	{SG11592, &UA638747},
	{SG11595, &UA2336541},
	{SG11596, &UA3017030},
	{SG11597, &UA2531016},
	{SG11598, &UA2660026},
	{SG11603, &UA2022801},
	{SG11604, &UA2579098},
	{SG11605, &UA2025690},
	{SG11606, &UA2029048},
	{SG11607, &UA2412362},
	{SG11608, &UA2759322},
	{SG11612, &UA2458329},
	{SG11613, &UA3094454},
	{SG11614, &UA567891},
	{SG11617, &UA2869320},
	{SG11623, &UA2781919},
	{SG11624, &UA2840814},
	{SG11632, &UA2580828},
	{SG11636, &UA637103},
	{SG11637, &UA2607902},
	{SG11647, &UA2962194},
	{SG11650, &UA668054},
	{SG11652, &UA2584953},
	{SG11656, &UA2216096},
	{SG11659, &UA616470},
	{SG11660, &UA616481},
	{SG11661, &UA616482},
	{SG11666, &UA1950968},
	{SG11667, &UA2999591},
	{SG11675, &UA2672668},
	{SG11676, &UA876261},
	{SG11677, &UA1994081},
	{SG11685, &UA2707883},
	{SG11687, &UA2256907},
	{SG11688, &UA626843},
	{SG11689, &UA2027308},
	{SG11695, &UA2658214},
	{SG11697, &UA1988792},
	{SG11698, &UA2767162},
	{SG11702, &UA2937924},
	{SG11704, &UA2556186},
	{SG11707, &UA2349679},
	{SG11710, &UA2026595},
	{SG11711, &UA2999188},
	{SG11712, &UA804963},
	{SG11713, &UA2658677},
	{SG11714, &UA2764567},
	{SG11715, &UA2852413},
	{SG11717, &UA637189},
	{SG11718, &UA2032405},
	{SG11722, &UA846008},
	{SG11723, &UA2581016},
	{SG11724, &UA2734566},
	{SG11725, &UA2878437},
	{SG11729, &UA2032354},
	{SG11730, &UA2771219},
	{SG11731, &UA2997656},
	{SG11732, &UA2963096},
	{SG11734, &UA1956278},
	{SG11735, &UA2816862},
	{SG11745, &UA3023823},
	{SG11748, &UA2851712},
	{SG11750, &UA2337505},
	{SG11758, &UA2715886},
	{SG11759, &UA2900601},
	{SG11762, &UA336684},
	{SG11763, &UA787403},
	{SG11764, &UA354043},
	{SG11775, &UA2598068},
	{SG11782, &UA2033657},
	{SG11783, &UA600159},
	{SG11784, &UA622486},
	{SG11785, &UA2901214},
	{SG11786, &UA3025941},
	{SG11789, &UA2895907},
	{SG11790, &UA2617054},
	{SG11794, &UA2579296},
	{SG11795, &UA2938028},
	{SG11796, &UA3049928},
	{SG11797, &UA2812075},
	{SG11799, &UA2604670},
	{SG11801, &UA2580265},
	{SG11802, &UA3020559},
	{SG11804, &UA2988927},
	{SG11805, &UA3020724},
	{SG11807, &UA3062408},
	{SG11808, &UA2025587},
	{SG11810, &UA2812407},
	{SG11811, &UA863151},
	{SG11812, &UA2399969},
	{SG11813, &UA2808290},
	{SG11814, &UA1955159},
	{SG11815, &UA2445504},
	{SG11816, &UA2814939},
	{SG11818, &UA360156},
	{SG11820, &UA624068},
	{SG11822, &UA2024464},
	{SG11823, &UA2659676},
	{SG11824, &UA3012678},
	{SG11826, &UA2580617},
	{SG11830, &UA2221470},
	{SG11831, &UA2344703},
	{SG11832, &UA2594213},
	{SG11833, &UA2469226},
	{SG11834, &UA2736513},
	{SG11836, &UA1993866},
	{SG11837, &UA2660354},
	{SG11841, &UA2626363},
	{SG11846, &UA2333497},
	{SG11853, &UA217080},
	{SG11856, &UA678342},
	{SG11857, &UA2782942},
	{SG11866, &UA2528457},
	{SG11869, &UA2877858},
	{SG11870, &UA564077},
	{SG11871, &UA2635659},
	{SG11876, &UA2767188},
	{SG11883, &UA2772697},
	{SG11885, &UA3064703},
	{SG11890, &UA2506191},
	{SG11891, &UA2530079},
	{SG11892, &UA2901174},
	{SG11893, &UA2034258},
	{SG11894, &UA2644240},
	{SG11895, &UA2767908},
	{SG11896, &UA2944510},
	{SG11901, &UA2719410},
	{SG11902, &UA3022376},
	{SG11903, &UA3079130},
	{SG11904, &UA3092982},
	{SG11906, &UA2372271},
	{SG11908, &UA2729788},
	{SG11909, &UA2996573},
	{SG11919, &UA2671451},
	{SG11926, &UA2031251},
	{SG11930, &UA2457541},
	{SG11931, &UA2581622},
	{SG11934, &UA2816224},
	{SG11935, &UA3044357},
	{SG11936, &UA2939593},
	{SG11938, &UA1994728},
	{SG11939, &UA2421674},
	{SG11943, &UA3091924},
	{SG11946, &UA2237850},
	{SG11948, &UA2599617},
	{SG11949, &UA2529115},
	{SG11955, &UA1951037},
	{SG11956, &UA2025898},
	{SG11959, &UA2751545},
	{SG11960, &UA2780777},
	{SG11961, &UA2733814},
	{SG11966, &UA2555972},
	{SG11974, &UA2456137},
	{SG11978, &UA2480393},
	{SG11980, &UA2734648},
	{SG11981, &UA3017456},
	{SG11983, &UA2701370},
	{SG11984, &UA2863269},
	{SG11986, &UA2717754},
	{SG11996, &UA2022646},
	{SG11997, &UA2686150},
	{SG11998, &UA2423542},
	{SG11999, &UA2892353},
	{SG12000, &UA3037096},
	{SG12005, &UA2597873},
	{SG12006, &UA2937961},
	{SG12007, &UA3095128},
	{SG12008, &UA655518},
	{SG12009, &UA1985473},
	{SG12012, &UA903267},
	{SG12015, &UA1944387},
	{SG12016, &UA2999856},
	{SG12024, &UA2597973},
	{SG12025, &UA2599748},
	{SG12026, &UA3058016},
	{SG12027, &UA2658255},
	{SG12028, &UA2838819},
	{SG12034, &UA2028080},
	{SG12038, &UA2708686},
	{SG12040, &UA2420386},
	{SG12042, &UA2751752},
	{SG12046, &UA2936790},
	{SG12050, &UA2703040},
	{SG12051, &UA3058056},
	{SG12056, &UA1993969},
	{SG12060, &UA2422057},
	{SG12061, &UA2669395},
	{SG12062, &UA2869338},
	{SG12065, &UA2026157},
	{SG12067, &UA2033022},
	{SG12068, &UA694049},
	{SG12072, &UA1900568},
	{SG12084, &UA2658175},
	{SG12092, &UA2554904},
	{SG12094, &UA1896490},
	{SG12096, &UA2565709},
	{SG12097, &UA2778866},
	{SG12099, &UA2455416},
	{SG12101, &UA3063831},
	{SG12106, &UA2024289},
	{SG12109, &UA3110374},
	{SG12111, &UA2800433},
	{SG12118, &UA2658800},
	{SG12119, &UA3003416},
	{SG12120, &UA3123407},
	{SG12127, &UA2667378},
	{SG12129, &UA2699270},
	{SG12133, &UA2945143},
	{SG12135, &UA218955},
	{SG12137, &UA1897974},
	{SG12138, &UA2031520},
	{SG12142, &UA569082},
	{SG12143, &UA2019910},
	{SG12145, &UA2034225},
	{SG12146, &UA2712283},
	{SG12147, &UA2963735},
	{SG12156, &UA3006643},
	{SG12158, &UA2943811},
	{SG12165, &UA3059544},
	{SG12166, &UA230558},
	{SG12167, &UA566840},
	{SG12168, &UA2401732},
	{SG12169, &UA421189},
	{SG12170, &UA2021800},
	{SG12176, &UA3113939},
	{SG12181, &UA359525},
	{SG12183, &UA745272},
	{SG12188, &UA649856},
	{SG12189, &UA2028853},
	{SG12190, &UA2026531},
	{SG12191, &UA2667532},
	{SG12195, &UA3002215},
	{SG12196, &UA2667150},
	{SG12199, &UA2420635},
	{SG12200, &UA2892878},
	{SG12202, &UA2972713},
	{SG12204, &UA3108377},
	{SG12205, &UA2029603},
	{SG12206, &UA2814281},
	{SG12208, &UA2867437},
	{SG12223, &UA2030256},
	{SG12224, &UA2937439},
	{SG12230, &UA2655421},
	{SG12231, &UA3059304},
	{SG12234, &UA2781890},
	{SG12239, &UA2402128},
	{SG12240, &UA2660106},
	{SG12241, &UA2794967},
	{SG12242, &UA3090791},
	{SG12244, &UA1947007},
	{SG12246, &UA2031988},
	{SG12247, &UA2337199},
	{SG12248, &UA2842358},
	{SG12251, &UA2790564},
	{SG12252, &UA2848498},
	{SG12253, &UA2879211},
	{SG12258, &UA250605},
	{SG12260, &UA3089895},
	{SG12261, &UA2892685},
	{SG12263, &UA2019655},
	{SG12264, &UA2882550},
	{SG12268, &UA3112545},
	{SG12269, &UA333733},
	{SG12275, &UA2799704},
	{SG12276, &UA3112137},
	{SG12280, &UA2893469},
	{SG12283, &UA2025017},
	{SG12284, &UA2890209},
	{SG12285, &UA2020558},
	{SG12287, &UA2866614},
	{SG12296, &UA2449942},
	{SG12298, &UA2457473},
	{SG12299, &UA2554721},
	{SG12300, &UA2890050},
	{SG12301, &UA3060689},
	{SG12303, &UA2544397},
	{SG12306, &UA2371266},
	{SG12308, &UA2486405},
	{SG12310, &UA2863119},
	{SG12312, &UA2033093},
	{SG12314, &UA2375895},
	{SG12315, &UA2766052},
	{SG12318, &UA2031292},
	{SG12319, &UA2420045},
	{SG12324, &UA2842990},
	{SG12325, &UA3010167},
	{SG12327, &UA3078967},
	{SG12328, &UA3108661},
	{SG12329, &UA368701},
	{SG12330, &UA646625},
	{SG12336, &UA1967306},
	{SG12337, &UA2668584},
	{SG12338, &UA2855383},
	{SG12339, &UA2891625},
	{SG12340, &UA2891784},
	{SG12343, &UA1951016},
	{SG12344, &UA2578309},
	{SG12347, &UA3108605},
	{SG12349, &UA2346283},
	{SG12350, &UA2824637},
	{SG12352, &UA2866576},
	{SG12353, &UA2026247},
	{SG12357, &UA2686400},
	{SG12358, &UA2844716},
	{SG12363, &UA3110553},
	{SG12365, &UA1957109},
	{SG12366, &UA2588468},
	{SG12367, &UA2812782},
	{SG12368, &UA3121225},
	{SG12369, &UA2853438},
	{SG12371, &UA2958561},
	{SG12372, &UA3114554},
	{SG12379, &UA2026240},
	{SG12380, &UA2029725},
	{SG12384, &UA2728009},
	{SG12385, &UA403358},
	{SG12386, &UA831924},
	{SG12390, &UA2026642},
	{SG12391, &UA2957938},
	{SG12392, &UA2256898},
	{SG12396, &UA2934917},
	{SG12400, &UA217066},
	{SG12402, &UA2443988},
	{SG12403, &UA2953344},
	{SG12404, &UA2557914},
	{SG12406, &UA2854757},
	{SG12408, &UA3000329},
	{SG12414, &UA1974633},
	{SG12415, &UA2024814},
	{SG12416, &UA2022996},
	{SG12421, &UA3102938},
	{SG12425, &UA2367408},
	{SG12429, &UA2692237},
	{SG12435, &UA3014333},
	{SG12436, &UA3022059},
	{SG12438, &UA3022496},
	{SG12440, &UA2029935},
	{SG12441, &UA2030676},
	{SG12442, &UA2450183},
	{SG12443, &UA2660999},
	{SG12444, &UA2703972},
	{SG12446, &UA3021057},
	{SG12448, &UA1940944},
	{SG12449, &UA402827},
	{SG12452, &UA2028685},
	{SG12454, &UA2453284},
	{SG12455, &UA3093223},
	{SG12456, &UA2339743},
	{SG12458, &UA2660833},
	{SG12461, &UA2999184},
	{SG12466, &UA369719},
	{SG12468, &UA2033935},
	{SG12470, &UA2588813},
	{SG12471, &UA2859261},
	{SG12475, &UA3004535},
	{SG12476, &UA3006604},
	{SG12485, &UA2339946},
	{SG12486, &UA2506637},
	{SG12487, &UA2655158},
	{SG12488, &UA2659684},
	{SG12490, &UA2892458},
	{SG12492, &UA2935116},
	{SG12494, &UA2957342},
	{SG12495, &UA3023836},
	{SG12496, &UA3092717},
	{SG12497, &UA3109414},
	{SG12499, &UA2019563},
	{SG12500, &UA2019906},
	{SG12501, &UA2032650},
	{SG12502, &UA3049364},
	{SG12503, &UA2032665},
	{SG12504, &UA2033545},
	{SG12505, &UA2529375},
	{SG12506, &UA2663163},
	{SG12507, &UA2701374},
	{SG12508, &UA2958575},
	{SG12509, &UA3109120},
	{SG12510, &UA3116529},
	{SG12518, &UA2043767},
	{SG12519, &UA2043769},
	{SG12526, &UA2695561},
	{SG12527, &UA2706141},
	{SG12529, &UA2768238},
	{SG12555, &UA3003329},
	{SG12559, &UA3014407},
	{SG12564, &UA3060359},
	{SG12570, &UA2866581},
	{SG12573, &UA2998985},
	{SG12580, &UA369712},
	{SG12581, &UA2023128},
	{SG12582, &UA2028945},
	{SG12584, &UA2355178},
	{SG12585, &UA2502533},
	{SG12587, &UA2722705},
	{SG12591, &UA2984837},
	{SG12592, &UA3005018},
	{SG12595, &UA3052166},
	{SG12598, &UA2957031},
	{SG12601, &UA3089523},
	{SG12602, &UA3093112},
	{SG12609, &UA2032313},
	{SG12611, &UA2485536},
	{SG12613, &UA2853378},
	{SG12614, &UA2861543},
	{SG12615, &UA2862223},
	{SG12616, &UA2863209},
	{SG12617, &UA2956314},
	{SG12619, &UA3013497},
	{SG12621, &UA3057454},
	{SG12623, &UA3103201},
	{SG12631, &UA2026863},
	{SG12632, &UA3088884},
	{SG12633, &UA2382918},
	{SG12634, &UA2530132},
	{SG12635, &UA2651110},
	{SG12636, &UA2700528},
	{SG12637, &UA2793849},
	{SG12638, &UA2850370},
	{SG12639, &UA2952457},
	{SG12640, &UA2965098},
	{SG12641, &UA2999092},
	{SG12642, &UA2999646},
	{SG12643, &UA3009584},
	{SG12647, &UA3079460},
	{SG12649, &UA3106120},
	{SG12659, &UA2790470},
	{SG12667, &UA2996048},
	{SG12669, &UA2998906},
	{SG12670, &UA3006172},
	{SG12672, &UA3074004},
	{SG12694, &UA2780778},
	{SG12695, &UA2829107},
	{SG12696, &UA2860377},
	{SG12697, &UA2982026},
	{SG12698, &UA2998626},
	{SG12699, &UA3046916},
	{SG12704, &UA3079723},
	{SG12706, &UA3090323},
	{SG12709, &UA3107942},
	{SG12710, &UA3108252},
	{SG12715, &UA2577716},
	{SG12717, &UA2726358},
	{SG12719, &UA2790460},
	{SG12724, &UA2850575},
	{SG12727, &UA2861290},
	{SG12728, &UA2983818},
	{SG12730, &UA2990012},
	{SG12731, &UA2998494},
	{SG12733, &UA3056507},
	{SG12735, &UA3074856},
	{SG12736, &UA3100034},
	{SG12737, &UA3100075},
	{SG12740, &UA3125622},
	{SG12741, &UA3125755},
	{SG12742, &UA3125826},
	{SG12748, &UA2543861},
	{SG12749, &UA2726913},
	{SG12750, &UA2837585},
	{SG12751, &UA2852097},
	{SG12753, &UA2985548},
	{SG12755, &UA2996028},
	{SG12757, &UA2998870},
	{SG12760, &UA3044836},
	{SG12762, &UA3048398},
	{SG12763, &UA3054418},
	{SG12765, &UA3099189},
	{SG12767, &UA3107642},
	{SG12768, &UA3118345},
	{SG12769, &UA3120484},
	{SG12770, &UA3121526},
	{SG12771, &UA3123926},
	{SG12772, &UA3124502},
	{SG12773, &UA3124952},
	{SG12774, &UA3125581},
	{SG12783, &UA2382619},
	{SG12787, &UA2848132},
	{SG12789, &UA2850550},
	{SG12790, &UA2864704},
	{SG12791, &UA2020497},
	{SG12792, &UA2026479},
	{SG12796, &UA2834562},
	{SG12797, &UA2843387},
	{SG12806, &UA2520855},
	{SG12807, &UA2722547},
	{SG12822, &UA245177},
	{SG12837, &UA2339092},
	{SG12842, &UA2446807},
	{SG12847, &UA2813718},
	{SG12859, &UA2520820},
	{SG12860, &UA2027943},
	{SG12861, &UA2635631},
	{SG12862, &UA2817918},
	{SG12866, &UA2030348},
	{SG12868, &UA2817402},
	{SG12869, &UA2817685},
	{SG12878, &UA2021256},
	{SG12879, &UA2025941},
	{SG12883, &UA2817975},
	{SG12884, &UA3084097},
	{SG12901, &UA1982935},
	{SG12931, &UA369201},
	{SG12937, &UA3084110},
	{SG12947, &UA2025099},
	{SG12948, &UA2025189},
	{SG12949, &UA2026722},
	{SG12950, &UA2607508},
	{SG12963, &UA2026411},
	{SG12964, &UA2026557},
	{SG12965, &UA2033509},
	{SG13008, &UA2607509},
	{SG13010, &UA2607511},
	{SG13033, &UA2026051},
	{SG13034, &UA2024205},
	{SG13035, &UA2024462},
	{SG13036, &UA2031808},
	{SG13037, &UA2033571},
	{SG13063, &UA3084098},
	{SG13067, &UA912231},
	{SG13073, &UA370123},
	{SG13074, &UA582508},
	{SG13075, &UA2023447},
	{SG13076, &UA2030610},
	{SG13077, &UA2607512},
	{SG13078, &UA2607513},
	{SG13079, &UA2607514},
	{SG13080, &UA2607515},
	{NULL}};
const Segmentdata DAndroidVersion2_3_6[] = {
	{SG12528, &UA349124},
	{SG12531, &UA622979},
	{SG12532, &UA623175},
	{SG12533, &UA627389},
	{SG12534, &UA627453},
	{SG12535, &UA628654},
	{SG12536, &UA661886},
	{SG12537, &UA733368},
	{SG12538, &UA738497},
	{SG12539, &UA742238},
	{SG12540, &UA773369},
	{SG12541, &UA909474},
	{SG12542, &UA1960559},
	{SG12545, &UA1976969},
	{SG12546, &UA1981478},
	{SG12549, &UA2021163},
	{SG12550, &UA2021983},
	{SG12551, &UA2022148},
	{SG12552, &UA2023289},
	{SG12553, &UA2024698},
	{SG12554, &UA2025388},
	{SG12556, &UA2031073},
	{SG12557, &UA2167038},
	{SG12558, &UA2257102},
	{SG12560, &UA2366714},
	{SG12562, &UA2950540},
	{SG12563, &UA3118559},
	{SG12808, &UA354673},
	{SG12810, &UA627260},
	{SG12811, &UA630524},
	{SG12813, &UA632308},
	{SG12814, &UA632553},
	{SG12816, &UA653512},
	{SG12817, &UA683058},
	{SG12818, &UA821756},
	{SG12819, &UA1988473},
	{SG12820, &UA2023804},
	{SG12821, &UA2024353},
	{SG12823, &UA2027511},
	{SG12824, &UA2029982},
	{SG12825, &UA2030611},
	{SG12827, &UA2030870},
	{SG12829, &UA2031597},
	{SG12830, &UA2176708},
	{SG12831, &UA2257104},
	{SG12832, &UA2362659},
	{SG12833, &UA2385791},
	{SG12835, &UA2403784},
	{SG12836, &UA2414214},
	{SG12838, &UA2496901},
	{SG12839, &UA2650034},
	{SG12840, &UA2694881},
	{SG12841, &UA2751001},
	{SG12843, &UA2786591},
	{SG12844, &UA2836955},
	{SG12845, &UA2889157},
	{SG12846, &UA2979395},
	{SG12887, &UA348645},
	{SG12888, &UA360495},
	{SG12889, &UA362910},
	{SG12891, &UA622991},
	{SG12892, &UA625397},
	{SG12893, &UA628375},
	{SG12894, &UA630199},
	{SG12895, &UA633649},
	{SG12896, &UA639403},
	{SG12897, &UA648704},
	{SG12898, &UA899772},
	{SG12899, &UA1937832},
	{SG12900, &UA1959358},
	{SG12902, &UA1965478},
	{SG12903, &UA1971213},
	{SG12904, &UA1973017},
	{SG12905, &UA1985511},
	{SG12906, &UA1988175},
	{SG12907, &UA1999899},
	{SG12908, &UA2021342},
	{SG12909, &UA2023416},
	{SG12910, &UA2025116},
	{SG12911, &UA2026691},
	{SG12912, &UA2029546},
	{SG12913, &UA2030858},
	{SG12914, &UA2030908},
	{SG12915, &UA2034034},
	{SG12916, &UA2222926},
	{SG12917, &UA2367448},
	{SG12918, &UA2487236},
	{SG12919, &UA2487938},
	{SG12920, &UA2564262},
	{SG12922, &UA2592761},
	{SG12923, &UA2722907},
	{SG12924, &UA2781732},
	{SG12925, &UA2807328},
	{SG12926, &UA2816151},
	{SG12981, &UA623682},
	{SG12982, &UA625080},
	{SG12983, &UA632142},
	{SG12984, &UA702613},
	{SG12985, &UA765160},
	{SG12986, &UA781747},
	{SG12987, &UA849591},
	{SG12988, &UA902807},
	{SG12989, &UA908911},
	{SG12990, &UA924870},
	{SG12991, &UA1973486},
	{SG12992, &UA1973509},
	{SG12993, &UA1986611},
	{SG12994, &UA1992580},
	{SG12995, &UA1999146},
	{SG12996, &UA2027000},
	{SG12997, &UA2232555},
	{SG12998, &UA2234782},
	{SG12999, &UA2344623},
	{SG13000, &UA2352125},
	{SG13001, &UA2381119},
	{SG13002, &UA2383849},
	{SG13003, &UA2384029},
	{SG13004, &UA2407255},
	{SG13005, &UA2410842},
	{SG13006, &UA2427601},
	{SG13007, &UA2431872},
	{SG13009, &UA2446104},
	{SG13011, &UA2452756},
	{SG13012, &UA2453847},
	{SG13013, &UA2469169},
	{SG13014, &UA2469232},
	{SG13015, &UA2519845},
	{SG13016, &UA2526633},
	{SG13017, &UA2557634},
	{SG13018, &UA2697573},
	{SG13019, &UA2748897},
	{SG13020, &UA2784957},
	{SG13021, &UA2846319},
	{SG13022, &UA2935222},
	{SG13023, &UA2957632},
	{SG13024, &UA2984504},
	{SG13025, &UA2996823},
	{SG13026, &UA3034472},
	{SG13027, &UA3085783},
	{SG13028, &UA3115701},
	{SG13038, &UA355053},
	{SG13039, &UA356211},
	{SG13040, &UA371231},
	{SG13041, &UA625040},
	{SG13042, &UA635651},
	{SG13043, &UA638553},
	{SG13044, &UA655277},
	{SG13045, &UA659148},
	{SG13046, &UA700838},
	{SG13047, &UA1987648},
	{SG13048, &UA2029141},
	{SG13049, &UA2234093},
	{SG13050, &UA2257103},
	{SG13051, &UA2331004},
	{SG13052, &UA2362360},
	{SG13053, &UA2362559},
	{SG13054, &UA2558296},
	{SG13055, &UA2659111},
	{SG13056, &UA2665679},
	{SG13081, &UA603201},
	{SG13082, &UA622439},
	{SG13083, &UA627519},
	{SG13084, &UA677359},
	{SG13085, &UA687305},
	{SG13086, &UA1989688},
	{SG13087, &UA1993055},
	{SG13088, &UA1993274},
	{SG13089, &UA2024218},
	{SG13091, &UA2028191},
	{SG13092, &UA2031464},
	{SG13094, &UA2232877},
	{SG13095, &UA2371354},
	{SG13096, &UA2485244},
	{SG13097, &UA2953185},
	{SG13098, &UA3127262},
	{SG13099, &UA352201},
	{SG13100, &UA355801},
	{SG13101, &UA735929},
	{SG13102, &UA1940765},
	{SG13103, &UA1971046},
	{SG13104, &UA2023564},
	{SG13105, &UA2023628},
	{SG13106, &UA2025067},
	{SG13107, &UA2027089},
	{SG13108, &UA2027729},
	{SG13109, &UA2034055},
	{SG13110, &UA2257117},
	{SG13111, &UA2257130},
	{SG13112, &UA2734471},
	{SG13113, &UA2023833},
	{SG13114, &UA900470},
	{SG13115, &UA1967456},
	{SG13116, &UA1981765},
	{SG13117, &UA1995687},
	{SG13122, &UA2023705},
	{SG13123, &UA2024072},
	{SG13124, &UA2024156},
	{SG13125, &UA2024810},
	{SG13126, &UA2024876},
	{SG13127, &UA2025263},
	{SG13128, &UA2025816},
	{SG13129, &UA2025932},
	{SG13130, &UA2026115},
	{SG13131, &UA2026898},
	{SG13132, &UA2027243},
	{SG13133, &UA2027680},
	{SG13134, &UA2027723},
	{SG13135, &UA2027948},
	{SG13136, &UA2028232},
	{SG13137, &UA2028780},
	{SG13138, &UA2029359},
	{SG13139, &UA2029451},
	{SG13140, &UA2029770},
	{SG13141, &UA2029832},
	{SG13142, &UA2030007},
	{SG13143, &UA2030085},
	{SG13144, &UA2030185},
	{SG13145, &UA2030707},
	{SG13146, &UA2032673},
	{SG13147, &UA2032886},
	{SG13148, &UA2032979},
	{SG13149, &UA2033373},
	{SG13150, &UA2034114},
	{SG13151, &UA2034400},
	{SG13152, &UA2167044},
	{SG13153, &UA2222915},
	{SG13154, &UA2222983},
	{SG13155, &UA2223460},
	{SG13156, &UA2227086},
	{SG13157, &UA2227635},
	{SG13158, &UA2257074},
	{SG13159, &UA2257133},
	{SG13160, &UA2320558},
	{SG13161, &UA2335279},
	{SG13162, &UA2337435},
	{SG13163, &UA2337975},
	{SG13164, &UA2338600},
	{SG13165, &UA2339414},
	{SG13166, &UA2351951},
	{SG13167, &UA2352833},
	{SG13168, &UA2412751},
	{SG13169, &UA2413956},
	{SG13170, &UA2413976},
	{SG13171, &UA2414515},
	{SG13172, &UA2418079},
	{SG13173, &UA2418600},
	{SG13174, &UA2421369},
	{SG13175, &UA2422013},
	{SG13177, &UA2422754},
	{SG13178, &UA2423538},
	{SG13179, &UA2424596},
	{SG13180, &UA2443840},
	{SG13181, &UA2443939},
	{SG13182, &UA2444732},
	{SG13183, &UA2444849},
	{SG13184, &UA2445258},
	{SG13185, &UA2446693},
	{SG13186, &UA2455720},
	{SG13187, &UA2473813},
	{SG13188, &UA2476398},
	{SG13189, &UA2476672},
	{SG13190, &UA2479474},
	{SG13191, &UA2479973},
	{SG13192, &UA2480089},
	{SG13194, &UA2481257},
	{SG13195, &UA2485172},
	{SG13197, &UA2496316},
	{SG13198, &UA2496837},
	{SG13199, &UA2500358},
	{SG13201, &UA2503592},
	{SG13202, &UA2504804},
	{SG13203, &UA2518395},
	{SG13205, &UA2520350},
	{SG13207, &UA2520504},
	{SG13208, &UA2520549},
	{SG13210, &UA2520825},
	{SG13212, &UA2523968},
	{SG13214, &UA2524226},
	{SG13215, &UA2525350},
	{SG13217, &UA2525709},
	{SG13219, &UA2526479},
	{SG13220, &UA2527730},
	{SG13221, &UA2545625},
	{SG13222, &UA2546023},
	{SG13223, &UA2547362},
	{SG13224, &UA2548413},
	{SG13225, &UA2548597},
	{SG13226, &UA2549817},
	{SG13227, &UA2551228},
	{SG13228, &UA2552579},
	{SG13229, &UA2552583},
	{SG13230, &UA2552787},
	{SG13231, &UA2553115},
	{SG13232, &UA2553890},
	{SG13233, &UA2554335},
	{SG13234, &UA2554412},
	{SG13235, &UA2554888},
	{SG13236, &UA2555383},
	{SG13237, &UA2555486},
	{SG13238, &UA2556231},
	{SG13239, &UA2557513},
	{SG13240, &UA2574293},
	{SG13241, &UA2574339},
	{SG13242, &UA2576593},
	{SG13243, &UA2578547},
	{SG13244, &UA2579609},
	{SG13245, &UA2588189},
	{SG13246, &UA2589946},
	{SG13247, &UA2590213},
	{SG13248, &UA2592858},
	{SG13249, &UA2593766},
	{SG13250, &UA2594191},
	{SG13251, &UA2594421},
	{SG13252, &UA2594935},
	{SG13253, &UA2595613},
	{SG13254, &UA2596847},
	{SG13255, &UA2600677},
	{SG13256, &UA2643786},
	{SG13257, &UA2644008},
	{SG13258, &UA2644194},
	{SG13259, &UA2644423},
	{SG13260, &UA2650060},
	{SG13261, &UA2650841},
	{SG13262, &UA2651811},
	{SG13263, &UA2652358},
	{SG13264, &UA2652458},
	{SG13265, &UA2652650},
	{SG13266, &UA2652954},
	{SG13267, &UA2653204},
	{SG13268, &UA2654142},
	{SG13269, &UA2656819},
	{SG13270, &UA2657991},
	{SG13271, &UA2659585},
	{SG13272, &UA2660318},
	{SG13273, &UA2661244},
	{SG13274, &UA2661398},
	{SG13275, &UA2662309},
	{SG13276, &UA2666984},
	{SG13277, &UA2668922},
	{SG13278, &UA2683939},
	{SG13279, &UA2684180},
	{SG13280, &UA2685971},
	{SG13281, &UA2685975},
	{SG13282, &UA2686361},
	{SG13283, &UA2686441},
	{SG13284, &UA2686456},
	{SG13285, &UA2686848},
	{SG13286, &UA2686851},
	{SG13287, &UA2686865},
	{SG13288, &UA2688454},
	{SG13289, &UA2690133},
	{SG13290, &UA2691766},
	{SG13291, &UA2693430},
	{SG13292, &UA2693525},
	{SG13293, &UA2693687},
	{SG13294, &UA2695309},
	{SG13295, &UA2695398},
	{SG13296, &UA2697675},
	{SG13297, &UA2698687},
	{SG13298, &UA2699868},
	{SG13299, &UA2706899},
	{SG13300, &UA2707676},
	{SG13301, &UA2717903},
	{SG13302, &UA2721700},
	{SG13303, &UA2722749},
	{SG13304, &UA2723327},
	{SG13305, &UA2723746},
	{SG13306, &UA2723989},
	{SG13307, &UA2724946},
	{SG13308, &UA2726968},
	{SG13309, &UA2727083},
	{SG13310, &UA2727645},
	{SG13311, &UA2728179},
	{SG13312, &UA2729376},
	{SG13313, &UA2731631},
	{SG13314, &UA2732412},
	{SG13315, &UA2733112},
	{SG13316, &UA2734618},
	{SG13318, &UA2734772},
	{SG13320, &UA2742839},
	{SG13321, &UA2743847},
	{SG13322, &UA2745738},
	{SG13323, &UA2748366},
	{SG13324, &UA2749516},
	{SG13325, &UA2749815},
	{SG13326, &UA2759216},
	{SG13327, &UA2759505},
	{SG13328, &UA2759812},
	{SG13329, &UA2761011},
	{SG13330, &UA2761024},
	{SG13331, &UA2763262},
	{SG13332, &UA2764541},
	{SG13333, &UA2765364},
	{SG13334, &UA2775283},
	{SG13335, &UA2775769},
	{SG13336, &UA2777317},
	{SG13337, &UA2777614},
	{SG13338, &UA2777724},
	{SG13339, &UA2778150},
	{SG13340, &UA2779937},
	{SG13341, &UA2779957},
	{SG13342, &UA2786747},
	{SG13343, &UA2786877},
	{SG13344, &UA2787397},
	{SG13345, &UA2787650},
	{SG13346, &UA2788501},
	{SG13347, &UA2788772},
	{SG13348, &UA2789102},
	{SG13349, &UA2789701},
	{SG13350, &UA2790192},
	{SG13351, &UA2790779},
	{SG13352, &UA2794665},
	{SG13353, &UA2810229},
	{SG13354, &UA2810662},
	{SG13355, &UA2811549},
	{SG13356, &UA2813707},
	{SG13357, &UA2814232},
	{SG13358, &UA2814457},
	{SG13359, &UA2816026},
	{SG13360, &UA2816736},
	{SG13361, &UA2816769},
	{SG13362, &UA2816842},
	{SG13363, &UA2817054},
	{SG13364, &UA2817195},
	{SG13365, &UA2817470},
	{SG13366, &UA2817570},
	{SG13367, &UA2817611},
	{SG13368, &UA2817612},
	{SG13369, &UA2819274},
	{SG13370, &UA2821626},
	{SG13371, &UA2822302},
	{SG13372, &UA2822793},
	{SG13373, &UA2822814},
	{SG13374, &UA2823224},
	{SG13375, &UA2826043},
	{SG13376, &UA2826587},
	{SG13377, &UA2828149},
	{SG13378, &UA2830374},
	{SG13379, &UA2830548},
	{SG13380, &UA2832996},
	{SG13381, &UA2833535},
	{SG13382, &UA2833903},
	{SG13383, &UA2834744},
	{SG13384, &UA2834771},
	{SG13385, &UA2835655},
	{SG13386, &UA2835938},
	{SG13387, &UA2836139},
	{SG13388, &UA2837377},
	{SG13389, &UA2838439},
	{SG13390, &UA2838918},
	{SG13391, &UA2838940},
	{SG13392, &UA2843381},
	{SG13393, &UA2843981},
	{SG13394, &UA2844196},
	{SG13395, &UA2846050},
	{SG13396, &UA2848055},
	{SG13397, &UA2849176},
	{SG13398, &UA2853546},
	{SG13399, &UA2855909},
	{SG13400, &UA2856713},
	{SG13401, &UA2858726},
	{SG13402, &UA2889055},
	{SG13403, &UA2889099},
	{SG13404, &UA2892257},
	{SG13405, &UA2933607},
	{SG13406, &UA2933765},
	{SG13407, &UA2933917},
	{SG13408, &UA2934771},
	{SG13409, &UA2935055},
	{SG13410, &UA2936391},
	{SG13411, &UA2936553},
	{SG13412, &UA2936779},
	{SG13413, &UA2937986},
	{SG13414, &UA2950269},
	{SG13415, &UA2950526},
	{SG13416, &UA2950536},
	{SG13417, &UA2950582},
	{SG13418, &UA2951334},
	{SG13419, &UA2951828},
	{SG13420, &UA2952039},
	{SG13421, &UA2953107},
	{SG13422, &UA2953458},
	{SG13423, &UA2954481},
	{SG13424, &UA2956542},
	{SG13426, &UA2957120},
	{SG13428, &UA2957361},
	{SG13429, &UA2958283},
	{SG13431, &UA2963572},
	{SG13432, &UA2980146},
	{SG13433, &UA2980548},
	{SG13434, &UA2980668},
	{SG13437, &UA2981361},
	{SG13438, &UA2981415},
	{SG13440, &UA2981443},
	{SG13441, &UA2981469},
	{SG13442, &UA2981486},
	{SG13443, &UA2981549},
	{SG13444, &UA2981558},
	{SG13447, &UA2981639},
	{SG13449, &UA2981988},
	{SG13450, &UA2982046},
	{SG13451, &UA2982103},
	{SG13452, &UA2982398},
	{SG13453, &UA2982424},
	{SG13454, &UA2982472},
	{SG13455, &UA2982795},
	{SG13456, &UA2982825},
	{SG13457, &UA2983068},
	{SG13458, &UA2983410},
	{SG13459, &UA2984258},
	{SG13460, &UA2986230},
	{SG13461, &UA2986666},
	{SG13462, &UA2989553},
	{SG13463, &UA2995658},
	{SG13464, &UA2996907},
	{SG13465, &UA2998474},
	{SG13466, &UA2998674},
	{SG13467, &UA2999787},
	{SG13468, &UA3000771},
	{SG13469, &UA3001187},
	{SG13470, &UA3002649},
	{SG13471, &UA3004773},
	{SG13472, &UA3006674},
	{SG13473, &UA3008774},
	{SG13474, &UA3009408},
	{SG13475, &UA3016339},
	{SG13476, &UA3016697},
	{SG13477, &UA3019115},
	{SG13478, &UA3030112},
	{SG13479, &UA3042205},
	{SG13480, &UA3042215},
	{SG13481, &UA3042219},
	{SG13482, &UA3042325},
	{SG13483, &UA3042335},
	{SG13484, &UA3042411},
	{SG13485, &UA3042463},
	{SG13486, &UA3042469},
	{SG13487, &UA3042513},
	{SG13488, &UA3042525},
	{SG13489, &UA3042639},
	{SG13490, &UA3043004},
	{SG13491, &UA3043030},
	{SG13492, &UA3043883},
	{SG13493, &UA3043923},
	{SG13494, &UA3044214},
	{SG13495, &UA3045066},
	{SG13496, &UA3045386},
	{SG13497, &UA3045569},
	{SG13498, &UA3057051},
	{SG13499, &UA3070081},
	{SG13500, &UA3070158},
	{SG13501, &UA3070644},
	{SG13502, &UA3071206},
	{SG13503, &UA3071346},
	{SG13504, &UA3071417},
	{SG13505, &UA3071484},
	{SG13506, &UA3072699},
	{SG13507, &UA3073669},
	{SG13508, &UA3074367},
	{SG13509, &UA3074570},
	{SG13510, &UA3078282},
	{SG13511, &UA3085224},
	{SG13512, &UA3085469},
	{SG13513, &UA3097892},
	{SG13514, &UA3098598},
	{SG13515, &UA3098631},
	{SG13516, &UA3098766},
	{SG13517, &UA3098774},
	{SG13518, &UA3098835},
	{SG13519, &UA3099048},
	{SG13520, &UA3099108},
	{SG13521, &UA3099172},
	{SG13522, &UA3099759},
	{SG13523, &UA3100028},
	{SG13524, &UA3100395},
	{SG13525, &UA3100508},
	{SG13526, &UA3101956},
	{SG13527, &UA3101988},
	{SG13528, &UA3102789},
	{SG13529, &UA3104027},
	{SG13530, &UA3107509},
	{SG13531, &UA3108142},
	{SG13532, &UA3108745},
	{SG13533, &UA3112161},
	{SG13534, &UA3112677},
	{SG13535, &UA3117370},
	{SG13536, &UA3117635},
	{SG13537, &UA3117691},
	{SG13538, &UA3117871},
	{SG13539, &UA3118251},
	{SG13540, &UA3118667},
	{SG13541, &UA3119091},
	{SG13542, &UA3121338},
	{SG13543, &UA3123565},
	{SG13544, &UA3124879},
	{SG13545, &UA3124992},
	{SG13546, &UA3125312},
	{SG13547, &UA3127389},
	{SG13557, &UA358813},
	{SG13558, &UA361477},
	{SG13559, &UA580178},
	{SG13560, &UA589996},
	{SG13561, &UA622973},
	{SG13562, &UA624176},
	{SG13563, &UA624898},
	{SG13564, &UA625023},
	{SG13565, &UA625992},
	{SG13566, &UA630848},
	{SG13567, &UA631959},
	{SG13568, &UA639113},
	{SG13569, &UA641691},
	{SG13570, &UA650795},
	{SG13571, &UA654723},
	{SG13572, &UA655905},
	{SG13573, &UA675507},
	{SG13574, &UA687253},
	{SG13575, &UA702203},
	{SG13576, &UA737979},
	{SG13577, &UA795155},
	{SG13578, &UA840442},
	{SG13579, &UA905216},
	{SG13580, &UA1961076},
	{SG13581, &UA2026532},
	{SG13582, &UA2030118},
	{SG13583, &UA2031318},
	{SG13584, &UA2031654},
	{SG13586, &UA2545486},
	{SG13592, &UA605190},
	{SG13593, &UA631445},
	{SG13594, &UA636044},
	{SG13595, &UA656664},
	{SG13596, &UA675144},
	{SG13597, &UA700307},
	{SG13598, &UA760849},
	{SG13599, &UA764830},
	{SG13600, &UA806167},
	{SG13601, &UA893658},
	{SG13602, &UA901535},
	{SG13603, &UA912342},
	{SG13604, &UA1953613},
	{SG13605, &UA1980118},
	{SG13606, &UA1994668},
	{SG13607, &UA2032444},
	{SG13608, &UA2034222},
	{SG13609, &UA2041378},
	{SG13610, &UA2163521},
	{SG13611, &UA2190388},
	{SG13612, &UA2203068},
	{SG13613, &UA2229905},
	{SG13614, &UA2236058},
	{SG13615, &UA2257070},
	{SG13616, &UA2389484},
	{SG13617, &UA2422211},
	{SG13618, &UA2445630},
	{SG13619, &UA2462003},
	{SG13620, &UA2486892},
	{SG13621, &UA2694809},
	{SG13622, &UA2723287},
	{SG13623, &UA2787081},
	{SG13624, &UA3117829},
	{SG13625, &UA3117955},
	{SG13636, &UA636753},
	{SG13637, &UA780313},
	{SG13638, &UA811083},
	{SG13639, &UA895172},
	{SG13640, &UA901725},
	{SG13641, &UA907008},
	{SG13642, &UA909254},
	{SG13643, &UA1994601},
	{SG13644, &UA2025125},
	{SG13645, &UA2029011},
	{SG13646, &UA2031393},
	{SG13647, &UA2032930},
	{SG13648, &UA2185667},
	{SG13649, &UA2222204},
	{SG13650, &UA2228018},
	{SG13651, &UA2353872},
	{SG13652, &UA2407252},
	{SG13653, &UA2408218},
	{SG13654, &UA2428228},
	{SG13655, &UA2433273},
	{SG13656, &UA2435164},
	{SG13657, &UA2480016},
	{SG13658, &UA2482952},
	{SG13659, &UA3099839},
	{SG13666, &UA2034494},
	{SG13667, &UA2811852},
	{SG13703, &UA893321},
	{SG13704, &UA1954205},
	{SG13705, &UA1959647},
	{SG13706, &UA1967084},
	{SG13707, &UA1988216},
	{SG13708, &UA2029950},
	{SG13709, &UA2230154},
	{SG13710, &UA2347455},
	{SG13711, &UA2360229},
	{SG13712, &UA2373099},
	{SG13713, &UA2388232},
	{SG13714, &UA2434259},
	{SG13715, &UA2494939},
	{SG13716, &UA2536648},
	{SG13717, &UA2579132},
	{SG13770, &UA635431},
	{SG13772, &UA656723},
	{SG13773, &UA657007},
	{SG13775, &UA663057},
	{SG13779, &UA751956},
	{SG13782, &UA757777},
	{SG13783, &UA762339},
	{SG13785, &UA767316},
	{SG13786, &UA795236},
	{SG13788, &UA801241},
	{SG13789, &UA847341},
	{SG13791, &UA853237},
	{SG13792, &UA854360},
	{SG13794, &UA856263},
	{SG13795, &UA871693},
	{SG13797, &UA892372},
	{SG13798, &UA896046},
	{SG13799, &UA899120},
	{SG13800, &UA1966630},
	{SG13801, &UA1967087},
	{SG13802, &UA1975766},
	{SG13803, &UA1984574},
	{SG13804, &UA1993183},
	{SG13805, &UA1994322},
	{SG13806, &UA2023272},
	{SG13807, &UA2025196},
	{SG13808, &UA2025201},
	{SG13809, &UA2025661},
	{SG13810, &UA2026015},
	{SG13811, &UA2027155},
	{SG13812, &UA2027180},
	{SG13813, &UA2027183},
	{SG13814, &UA2029116},
	{SG13815, &UA2029262},
	{SG13817, &UA2030193},
	{SG13818, &UA2032330},
	{SG13819, &UA2032411},
	{SG13820, &UA2032434},
	{SG13821, &UA2032547},
	{SG13822, &UA2032912},
	{SG13823, &UA2033653},
	{SG13824, &UA2033717},
	{SG13825, &UA2034148},
	{SG13826, &UA2202520},
	{SG13827, &UA2222438},
	{SG13828, &UA2223218},
	{SG13829, &UA2234416},
	{SG13830, &UA2257034},
	{SG13832, &UA2257116},
	{SG13833, &UA2340944},
	{SG13834, &UA2346327},
	{SG13836, &UA2352158},
	{SG13837, &UA2361996},
	{SG13838, &UA2362287},
	{SG13839, &UA2365331},
	{SG13843, &UA2366569},
	{SG13844, &UA2372389},
	{SG13845, &UA2375151},
	{SG13846, &UA2375348},
	{SG13847, &UA2376043},
	{SG13848, &UA2376067},
	{SG13849, &UA2378040},
	{SG13859, &UA2378636},
	{SG13861, &UA2380844},
	{SG13862, &UA2381676},
	{SG13863, &UA2381972},
	{SG13864, &UA2383581},
	{SG13865, &UA2383794},
	{SG13867, &UA2383914},
	{SG13868, &UA2384175},
	{SG13869, &UA2384789},
	{SG13870, &UA2387284},
	{SG13871, &UA2387655},
	{SG13872, &UA2395361},
	{SG13877, &UA2395813},
	{SG13878, &UA2395821},
	{SG13879, &UA2395862},
	{SG13880, &UA2397578},
	{SG13881, &UA2398211},
	{SG13882, &UA2400800},
	{SG13883, &UA2402494},
	{SG13884, &UA2403007},
	{SG13885, &UA2404244},
	{SG13886, &UA2417764},
	{SG13887, &UA2424425},
	{SG13888, &UA2428576},
	{SG13890, &UA2441964},
	{SG13891, &UA2444324},
	{SG13893, &UA2458037},
	{SG13894, &UA2467205},
	{SG13895, &UA2481127},
	{SG13896, &UA2496337},
	{SG13897, &UA2503921},
	{SG13898, &UA2545462},
	{SG13899, &UA2546732},
	{SG13900, &UA2555309},
	{SG13901, &UA2577522},
	{SG13902, &UA2588138},
	{SG13903, &UA2594571},
	{SG13904, &UA2602608},
	{SG13905, &UA2653013},
	{SG13906, &UA2687569},
	{SG13907, &UA2723745},
	{SG13909, &UA2729767},
	{SG13910, &UA2743494},
	{SG13911, &UA2744596},
	{SG13912, &UA2747044},
	{SG13913, &UA2758589},
	{SG13914, &UA2766851},
	{SG13915, &UA2777558},
	{SG13916, &UA2782062},
	{SG13918, &UA2786923},
	{SG13919, &UA2809397},
	{SG13920, &UA2811416},
	{SG13921, &UA2826867},
	{SG13922, &UA2837467},
	{SG13923, &UA2838934},
	{SG13924, &UA2933696},
	{SG13925, &UA2933897},
	{SG13926, &UA2949400},
	{SG13927, &UA2979640},
	{SG13928, &UA2981625},
	{SG13929, &UA2982155},
	{SG13930, &UA3008506},
	{SG13931, &UA3043355},
	{SG13932, &UA3103155},
	{SG13933, &UA3117990},
	{SG13934, &UA3118180},
	{SG14080, &UA370797},
	{SG14082, &UA675784},
	{SG14084, &UA763414},
	{SG14087, &UA785669},
	{SG14088, &UA885362},
	{SG14090, &UA904042},
	{SG14092, &UA910211},
	{SG14094, &UA1981402},
	{SG14096, &UA1994024},
	{SG14098, &UA1994337},
	{SG14101, &UA1995316},
	{SG14103, &UA1996244},
	{SG14105, &UA2027873},
	{SG14107, &UA2028996},
	{SG14109, &UA2029444},
	{SG14111, &UA2029503},
	{SG14113, &UA2031706},
	{SG14115, &UA2180614},
	{SG14117, &UA2202844},
	{SG14119, &UA2238800},
	{SG14121, &UA2257142},
	{SG14123, &UA2384387},
	{SG14126, &UA2456597},
	{SG14128, &UA2509692},
	{SG14130, &UA2534747},
	{SG14132, &UA2547137},
	{SG14134, &UA2561466},
	{SG14136, &UA2569312},
	{SG14138, &UA2692289},
	{SG14139, &UA2751235},
	{SG14141, &UA2818064},
	{SG14144, &UA2870206},
	{SG14147, &UA2981972},
	{SG14149, &UA3056577},
	{SG14247, &UA627598},
	{SG14249, &UA636086},
	{SG14250, &UA672831},
	{SG14252, &UA677118},
	{SG14253, &UA703827},
	{SG14255, &UA899194},
	{SG14258, &UA924838},
	{SG14260, &UA1954208},
	{SG14262, &UA1991190},
	{SG14264, &UA2025186},
	{SG14266, &UA2230594},
	{SG14268, &UA2256938},
	{SG14270, &UA2333089},
	{SG14272, &UA2361956},
	{SG14274, &UA2470657},
	{SG14276, &UA2527614},
	{SG14278, &UA2674766},
	{SG14475, &UA586782},
	{SG14477, &UA676832},
	{SG14478, &UA894859},
	{SG14479, &UA902843},
	{SG14480, &UA2028867},
	{SG14482, &UA2413208},
	{SG14483, &UA2545639},
	{SG14484, &UA2660386},
	{SG14781, &UA352967},
	{SG14783, &UA368815},
	{SG14786, &UA625028},
	{SG14788, &UA630094},
	{SG14790, &UA632186},
	{SG14792, &UA661217},
	{SG14794, &UA674981},
	{SG14797, &UA688169},
	{SG14799, &UA893652},
	{SG14800, &UA2031912},
	{SG14801, &UA2356044},
	{SG14802, &UA2511347},
	{SG14803, &UA2994684},
	{SG14889, &UA370111},
	{SG14890, &UA586255},
	{SG14891, &UA625244},
	{SG14892, &UA671631},
	{SG14894, &UA2361423},
	{SG14895, &UA2428312},
	{SG14897, &UA2607825},
	{SG14899, &UA2608637},
	{SG14900, &UA2609224},
	{SG14902, &UA2610252},
	{SG14903, &UA2610291},
	{SG14904, &UA2611469},
	{SG14906, &UA2616983},
	{SG14907, &UA2618664},
	{SG14908, &UA2618858},
	{SG14909, &UA2618942},
	{SG14910, &UA2620380},
	{SG14913, &UA2621068},
	{SG14914, &UA2621323},
	{SG14916, &UA2622235},
	{SG14917, &UA2622387},
	{SG14918, &UA2622501},
	{SG14920, &UA2623125},
	{SG14924, &UA2625611},
	{SG14925, &UA2625729},
	{SG14926, &UA2627247},
	{SG14927, &UA2629232},
	{SG14928, &UA2635201},
	{SG14929, &UA2635215},
	{SG14955, &UA622994},
	{SG14956, &UA626796},
	{SG14957, &UA638865},
	{SG14958, &UA660007},
	{SG14959, &UA730340},
	{SG14960, &UA1957700},
	{SG14961, &UA2033714},
	{SG14962, &UA2218844},
	{SG14963, &UA2518826},
	{SG14992, &UA441536},
	{SG14993, &UA899483},
	{SG14994, &UA1966905},
	{SG14995, &UA2025572},
	{SG14996, &UA2028913},
	{SG14997, &UA2168267},
	{SG14998, &UA2441602},
	{SG14999, &UA368779},
	{SG15000, &UA412422},
	{SG15001, &UA680713},
	{SG15002, &UA711591},
	{SG15003, &UA879991},
	{SG15004, &UA1992361},
	{SG15005, &UA459272},
	{SG15006, &UA459292},
	{SG15007, &UA459299},
	{SG15008, &UA459304},
	{SG15009, &UA2027496},
	{SG15010, &UA2029972},
	{SG15011, &UA2164745},
	{SG15012, &UA2520897},
	{SG15013, &UA2814827},
	{SG15022, &UA2468999},
	{SG15023, &UA900376},
	{SG15024, &UA2373212},
	{SG15025, &UA359103},
	{SG15026, &UA359354},
	{SG15027, &UA363753},
	{SG15028, &UA369556},
	{SG15029, &UA452929},
	{SG15030, &UA580910},
	{SG15031, &UA622536},
	{SG15032, &UA649723},
	{SG15033, &UA658014},
	{SG15034, &UA682976},
	{SG15035, &UA704335},
	{SG15036, &UA737247},
	{SG15037, &UA2023559},
	{SG15038, &UA2418617},
	{SG15039, &UA2789765},
	{SG15040, &UA349345},
	{SG15041, &UA356090},
	{SG15042, &UA356427},
	{SG15043, &UA666018},
	{SG15044, &UA692480},
	{SG15045, &UA2021745},
	{SG15046, &UA2033449},
	{SG15047, &UA2838005},
	{SG15070, &UA580642},
	{SG15071, &UA622924},
	{SG15073, &UA370394},
	{SG15074, &UA900437},
	{SG15075, &UA1959727},
	{SG15076, &UA1960839},
	{SG15077, &UA1964448},
	{SG15078, &UA1976144},
	{SG15079, &UA1995335},
	{SG15080, &UA2030057},
	{SG15081, &UA2475002},
	{SG15082, &UA2643248},
	{SG15083, &UA2684874},
	{SG15084, &UA626062},
	{SG15085, &UA626837},
	{SG15086, &UA657250},
	{SG15087, &UA660594},
	{SG15088, &UA665091},
	{SG15089, &UA745874},
	{SG15090, &UA899427},
	{SG15091, &UA1983219},
	{SG15092, &UA1986452},
	{SG15093, &UA2028643},
	{SG15094, &UA2164121},
	{SG15095, &UA2444633},
	{SG15096, &UA2490011},
	{SG15097, &UA2518664},
	{SG15098, &UA2519925},
	{SG15099, &UA2691480},
	{SG15100, &UA2721835},
	{SG15101, &UA2749943},
	{SG15102, &UA2816628},
	{SG15103, &UA2933506},
	{SG15104, &UA2991336},
	{SG15105, &UA370696},
	{SG15106, &UA428551},
	{SG15107, &UA606630},
	{SG15108, &UA608845},
	{SG15109, &UA622474},
	{SG15110, &UA624971},
	{SG15111, &UA627056},
	{SG15112, &UA658907},
	{SG15113, &UA662643},
	{SG15114, &UA667392},
	{SG15115, &UA669937},
	{SG15116, &UA672695},
	{SG15117, &UA684828},
	{SG15118, &UA729228},
	{SG15119, &UA736566},
	{SG15120, &UA747634},
	{SG15121, &UA785751},
	{SG15122, &UA828330},
	{SG15123, &UA829887},
	{SG15124, &UA833500},
	{SG15125, &UA834859},
	{SG15126, &UA848978},
	{SG15127, &UA853887},
	{SG15128, &UA895035},
	{SG15129, &UA1930977},
	{SG15130, &UA1936728},
	{SG15131, &UA1977191},
	{SG15132, &UA1987399},
	{SG15133, &UA1989717},
	{SG15134, &UA1994438},
	{SG15135, &UA1995447},
	{SG15136, &UA2021461},
	{SG15137, &UA2021710},
	{SG15138, &UA2022035},
	{SG15139, &UA2023899},
	{SG15140, &UA2024242},
	{SG15141, &UA2024262},
	{SG15142, &UA2025468},
	{SG15143, &UA2025780},
	{SG15144, &UA2027774},
	{SG15145, &UA2031838},
	{SG15146, &UA2032162},
	{SG15147, &UA2032572},
	{SG15148, &UA2684927},
	{SG15149, &UA2761486},
	{SG15150, &UA2980312},
	{SG15151, &UA1958293},
	{SG15152, &UA1960843},
	{SG15153, &UA1987276},
	{SG15154, &UA1995553},
	{SG15155, &UA2031132},
	{SG15156, &UA2523025},
	{SG15157, &UA2652966},
	{SG15158, &UA2712697},
	{SG15159, &UA2865625},
	{SG15160, &UA622978},
	{SG15161, &UA626472},
	{SG15162, &UA627480},
	{SG15163, &UA630517},
	{SG15164, &UA725595},
	{SG15172, &UA369256},
	{SG15174, &UA630476},
	{SG15176, &UA633969},
	{SG15179, &UA902903},
	{SG15182, &UA2033868},
	{SG15184, &UA2256976},
	{SG15279, &UA254311},
	{SG15280, &UA350903},
	{SG15281, &UA576037},
	{SG15282, &UA622550},
	{SG15283, &UA623706},
	{SG15284, &UA1997527},
	{SG15285, &UA2022060},
	{SG15286, &UA2024671},
	{SG15287, &UA2031884},
	{SG15288, &UA2439834},
	{SG15289, &UA2570291},
	{SG15290, &UA2576274},
	{SG15291, &UA2588737},
	{SG15293, &UA2668683},
	{SG15294, &UA1965820},
	{SG15295, &UA1995364},
	{SG15296, &UA2012020},
	{SG15299, &UA2029952},
	{SG15300, &UA2180048},
	{SG15301, &UA2204381},
	{SG15302, &UA2256999},
	{SG15303, &UA2348924},
	{SG15304, &UA2381816},
	{SG15305, &UA2382965},
	{SG15306, &UA2413525},
	{SG15307, &UA2421507},
	{SG15308, &UA2746764},
	{SG15309, &UA2838995},
	{SG15310, &UA2982058},
	{SG15311, &UA2983242},
	{SG15377, &UA634154},
	{SG15378, &UA2023414},
	{SG15379, &UA2027287},
	{SG15380, &UA2834499},
	{SG15381, &UA2989156},
	{SG15389, &UA906733},
	{SG15390, &UA909678},
	{SG15391, &UA2023877},
	{SG15392, &UA2043473},
	{SG15393, &UA2432358},
	{SG15394, &UA2600323},
	{SG15398, &UA242593},
	{SG15399, &UA341696},
	{SG15400, &UA603597},
	{SG15401, &UA1906451},
	{SG15402, &UA2020650},
	{SG15403, &UA2020821},
	{SG15490, &UA1995466},
	{SG15491, &UA1998769},
	{SG15492, &UA2029198},
	{SG15493, &UA2030740},
	{SG15494, &UA2032127},
	{SG15495, &UA2033034},
	{SG15496, &UA2033470},
	{SG15497, &UA2042027},
	{SG15498, &UA2549247},
	{SG15499, &UA363505},
	{SG15500, &UA368669},
	{SG15501, &UA370827},
	{SG15502, &UA421956},
	{SG15503, &UA623147},
	{SG15504, &UA625160},
	{SG15505, &UA628719},
	{SG15506, &UA662394},
	{SG15507, &UA677960},
	{SG15508, &UA747286},
	{SG15509, &UA771938},
	{SG15510, &UA788142},
	{SG15511, &UA802215},
	{SG15512, &UA1936919},
	{SG15513, &UA1937437},
	{SG15514, &UA2021458},
	{SG15515, &UA2025819},
	{SG15516, &UA2026403},
	{SG15517, &UA2026704},
	{SG15518, &UA2027345},
	{SG15519, &UA2028091},
	{SG15520, &UA2031295},
	{SG15565, &UA626871},
	{SG15568, &UA633605},
	{SG15569, &UA649569},
	{SG15577, &UA403353},
	{SG15579, &UA607803},
	{SG15581, &UA734376},
	{SG15582, &UA895852},
	{SG15584, &UA896830},
	{SG15586, &UA899931},
	{SG15588, &UA910345},
	{SG15590, &UA924477},
	{SG15592, &UA1961420},
	{SG15594, &UA1995213},
	{SG15596, &UA2026962},
	{SG15598, &UA2030535},
	{SG15600, &UA2032740},
	{SG15601, &UA2032964},
	{SG15602, &UA2034118},
	{SG15604, &UA2200536},
	{SG15606, &UA2225811},
	{SG15608, &UA2256984},
	{SG15610, &UA2564320},
	{SG15611, &UA2825414},
	{SG15614, &UA637157},
	{SG15615, &UA651501},
	{SG15617, &UA711265},
	{SG15619, &UA2024964},
	{SG15620, &UA2026042},
	{SG15621, &UA2026792},
	{SG15624, &UA2410076},
	{SG15626, &UA2747670},
	{SG15627, &UA2787062},
	{SG15735, &UA625900},
	{SG15736, &UA627629},
	{SG15737, &UA660264},
	{SG15738, &UA664378},
	{SG15739, &UA684085},
	{SG15740, &UA700442},
	{SG15741, &UA766235},
	{SG15742, &UA1964975},
	{SG15743, &UA1967930},
	{SG15744, &UA1981928},
	{SG15745, &UA2197110},
	{SG15746, &UA2229581},
	{SG15747, &UA2257114},
	{SG15748, &UA2488406},
	{SG15749, &UA2544531},
	{SG15750, &UA2725994},
	{SG15751, &UA2865444},
	{SG15754, &UA615791},
	{SG15755, &UA624817},
	{SG15756, &UA632767},
	{SG15757, &UA673864},
	{SG15758, &UA2698107},
	{SG15759, &UA2698397},
	{SG15760, &UA3098325},
	{SG15761, &UA360013},
	{SG15762, &UA361170},
	{SG15763, &UA369318},
	{SG15764, &UA622582},
	{SG15765, &UA1938678},
	{SG15775, &UA895053},
	{SG15776, &UA1953941},
	{SG15777, &UA2030613},
	{SG15778, &UA2205732},
	{SG15783, &UA2223310},
	{SG15784, &UA2338392},
	{SG15785, &UA2644306},
	{SG15786, &UA2822894},
	{SG15787, &UA2830961},
	{SG15788, &UA624918},
	{SG15789, &UA625007},
	{SG15790, &UA633725},
	{SG15791, &UA1992073},
	{SG15792, &UA2027300},
	{SG15815, &UA596772},
	{SG15817, &UA623087},
	{SG15820, &UA750440},
	{SG15821, &UA869542},
	{SG15822, &UA1963186},
	{SG15824, &UA1970918},
	{SG15825, &UA2029382},
	{SG15827, &UA2406815},
	{SG15828, &UA2589477},
	{SG15835, &UA597039},
	{SG15837, &UA616152},
	{SG15840, &UA722618},
	{SG15843, &UA766102},
	{SG15846, &UA808064},
	{SG15847, &UA825384},
	{SG15848, &UA842810},
	{SG15850, &UA2933296},
	{SG15890, &UA2423611},
	{SG15891, &UA2426671},
	{SG15936, &UA369228},
	{SG15937, &UA622891},
	{SG15938, &UA634396},
	{SG15939, &UA639310},
	{SG15940, &UA683678},
	{SG15941, &UA691051},
	{SG15942, &UA705845},
	{SG15943, &UA784404},
	{SG15944, &UA2227280},
	{SG15945, &UA2230705},
	{SG15946, &UA2366786},
	{SG15947, &UA2411406},
	{SG15948, &UA2526601},
	{SG15949, &UA2556438},
	{SG15950, &UA2694253},
	{SG15951, &UA2842322},
	{SG15952, &UA2843635},
	{SG15953, &UA3086398},
	{SG15974, &UA244159},
	{SG15975, &UA244472},
	{SG15976, &UA245041},
	{SG15979, &UA679669},
	{SG15982, &UA2446101},
	{SG15983, &UA2758269},
	{SG15984, &UA368936},
	{SG15985, &UA690936},
	{SG15986, &UA821788},
	{SG15987, &UA369161},
	{SG15988, &UA380007},
	{SG15989, &UA398287},
	{SG15990, &UA583608},
	{SG15991, &UA622464},
	{SG15992, &UA623011},
	{SG15993, &UA626102},
	{SG15994, &UA649293},
	{SG15995, &UA1936617},
	{SG15996, &UA1941202},
	{SG15997, &UA2022250},
	{SG15998, &UA2022943},
	{SG15999, &UA2024928},
	{SG16000, &UA2025112},
	{SG16001, &UA2025491},
	{SG16002, &UA2028050},
	{SG16003, &UA2029144},
	{SG16004, &UA2029591},
	{SG16005, &UA2030460},
	{SG16006, &UA2030666},
	{SG16007, &UA2031188},
	{SG16008, &UA2031249},
	{SG16009, &UA2031824},
	{SG16010, &UA2032097},
	{SG16011, &UA2033559},
	{SG16012, &UA2034515},
	{SG16013, &UA623183},
	{SG16014, &UA694176},
	{SG16015, &UA1957039},
	{SG16016, &UA1979224},
	{SG16017, &UA1983093},
	{SG16018, &UA1989318},
	{SG16019, &UA1992392},
	{SG16020, &UA2475978},
	{SG16021, &UA2979892},
	{SG16022, &UA750645},
	{SG16023, &UA2023468},
	{SG16024, &UA369819},
	{SG16025, &UA370533},
	{SG16026, &UA371288},
	{SG16027, &UA622368},
	{SG16028, &UA622945},
	{SG16029, &UA653533},
	{SG16030, &UA701303},
	{SG16031, &UA715376},
	{SG16032, &UA860020},
	{SG16033, &UA886014},
	{SG16034, &UA1906139},
	{SG16035, &UA1940302},
	{SG16036, &UA1952624},
	{SG16037, &UA1953504},
	{SG16038, &UA1955378},
	{SG16039, &UA1993629},
	{SG16040, &UA2022543},
	{SG16042, &UA2022695},
	{SG16044, &UA2031325},
	{SG16046, &UA2033153},
	{SG16048, &UA2033458},
	{SG16050, &UA2811853},
	{SG16052, &UA3070530},
	{SG16054, &UA366198},
	{SG16056, &UA369130},
	{SG16059, &UA623037},
	{SG16061, &UA660995},
	{SG16062, &UA758582},
	{SG16065, &UA2023836},
	{SG16067, &UA2024467},
	{SG16068, &UA2026130},
	{SG16077, &UA368854},
	{SG16079, &UA381715},
	{SG16081, &UA622578},
	{SG16085, &UA2024341},
	{SG16087, &UA2027676},
	{SG16088, &UA655721},
	{SG16089, &UA713080},
	{SG16090, &UA1975911},
	{SG16091, &UA2375811},
	{SG16094, &UA2684005},
	{SG16097, &UA2797063},
	{SG16158, &UA686451},
	{SG16159, &UA711366},
	{SG16160, &UA1983055},
	{SG16161, &UA2029380},
	{SG16162, &UA2588497},
	{SG16163, &UA2684278},
	{SG16164, &UA770741},
	{SG16165, &UA895533},
	{SG16166, &UA896688},
	{SG16167, &UA1986469},
	{SG16168, &UA2030014},
	{SG16169, &UA2347492},
	{SG16170, &UA2546019},
	{SG16171, &UA2830820},
	{SG16252, &UA716728},
	{SG16253, &UA765224},
	{SG16254, &UA895394},
	{SG16255, &UA1958171},
	{SG16256, &UA1979232},
	{SG16257, &UA1992264},
	{SG16258, &UA2023604},
	{SG16259, &UA2027724},
	{SG16260, &UA2933371},
	{SG16284, &UA627227},
	{SG16287, &UA648082},
	{SG16289, &UA750778},
	{SG16290, &UA2028742},
	{SG16292, &UA2222659},
	{SG16293, &UA2684513},
	{SG16294, &UA3106564},
	{SG16300, &UA724918},
	{SG16301, &UA892436},
	{SG16302, &UA2032153},
	{SG16303, &UA601769},
	{SG16304, &UA625035},
	{SG16307, &UA697591},
	{SG16308, &UA896418},
	{SG16309, &UA2030661},
	{SG16310, &UA2375127},
	{SG16311, &UA2446746},
	{SG16312, &UA2545494},
	{SG16313, &UA2546096},
	{SG16314, &UA2816417},
	{SG16315, &UA438883},
	{SG16316, &UA2033060},
	{SG16317, &UA2234604},
	{SG16318, &UA2257151},
	{SG16319, &UA2502274},
	{SG16320, &UA2590145},
	{SG16326, &UA362638},
	{SG16327, &UA363640},
	{SG16328, &UA2032987},
	{SG16329, &UA623016},
	{SG16330, &UA630641},
	{SG16331, &UA868797},
	{SG16332, &UA1993228},
	{SG16333, &UA2702246},
	{SG16334, &UA740859},
	{SG16335, &UA787823},
	{SG16336, &UA2027856},
	{SG16337, &UA2028219},
	{SG16343, &UA1961206},
	{SG16344, &UA1961626},
	{SG16345, &UA2204153},
	{SG16346, &UA2257057},
	{SG16347, &UA2413426},
	{SG16348, &UA2600655},
	{SG16349, &UA2750712},
	{SG16350, &UA2821016},
	{SG16351, &UA3119349},
	{SG16367, &UA350301},
	{SG16368, &UA354715},
	{SG16369, &UA1970550},
	{SG16370, &UA2573548},
	{SG16371, &UA2722070},
	{SG16372, &UA2779687},
	{SG16373, &UA2935039},
	{SG16374, &UA636228},
	{SG16375, &UA2024323},
	{SG16376, &UA748978},
	{SG16377, &UA769512},
	{SG16378, &UA1904221},
	{SG16379, &UA1958064},
	{SG16380, &UA2234645},
	{SG16381, &UA2593660},
	{SG16382, &UA2837906},
	{SG16389, &UA1994996},
	{SG16390, &UA2029479},
	{SG16391, &UA2378696},
	{SG16392, &UA2378859},
	{SG16393, &UA2378890},
	{SG16394, &UA2400197},
	{SG16395, &UA2414714},
	{SG16396, &UA2419806},
	{SG16397, &UA2447398},
	{SG16398, &UA634450},
	{SG16399, &UA2022723},
	{SG16400, &UA2165057},
	{SG16401, &UA2238811},
	{SG16402, &UA2384732},
	{SG16403, &UA2697467},
	{SG16404, &UA576445},
	{SG16405, &UA623201},
	{SG16406, &UA723894},
	{SG16407, &UA740161},
	{SG16408, &UA2545117},
	{SG16409, &UA2812788},
	{SG16410, &UA2204908},
	{SG16411, &UA2242366},
	{SG16412, &UA2986869},
	{SG16414, &UA2595227},
	{SG16416, &UA2372249},
	{SG16417, &UA2504767},
	{SG16439, &UA623085},
	{SG16440, &UA1941675},
	{SG16441, &UA2691546},
	{SG16442, &UA403388},
	{SG16443, &UA1987960},
	{SG16444, &UA1997046},
	{SG16445, &UA2027440},
	{SG16446, &UA2029637},
	{SG16447, &UA2223374},
	{SG16448, &UA2226621},
	{SG16449, &UA2257040},
	{SG16450, &UA2344327},
	{SG16451, &UA2393165},
	{SG16452, &UA2393949},
	{SG16453, &UA2414309},
	{SG16454, &UA2445648},
	{SG16455, &UA2501984},
	{SG16456, &UA2549789},
	{SG16457, &UA2578189},
	{SG16458, &UA2722829},
	{SG16459, &UA2980277},
	{SG16460, &UA2316959},
	{SG16461, &UA2459327},
	{SG16462, &UA2952191},
	{SG16463, &UA2414691},
	{SG16464, &UA459241},
	{SG16465, &UA459240},
	{SG16466, &UA627503},
	{SG16467, &UA899470},
	{SG16468, &UA1996943},
	{SG16469, &UA2032617},
	{SG16470, &UA2170014},
	{SG16471, &UA2257184},
	{SG16472, &UA2388198},
	{SG16473, &UA2419445},
	{SG16474, &UA2448011},
	{SG16475, &UA2574430},
	{SG16476, &UA2701834},
	{SG16477, &UA2727840},
	{SG16478, &UA3042767},
	{SG16479, &UA2332361},
	{SG16481, &UA2483166},
	{SG16483, &UA2859213},
	{SG16484, &UA3092091},
	{SG16491, &UA421986},
	{SG16492, &UA700885},
	{SG16493, &UA701421},
	{SG16494, &UA735875},
	{SG16495, &UA761630},
	{SG16496, &UA1935335},
	{SG16497, &UA1983794},
	{SG16498, &UA1994443},
	{SG16499, &UA1995394},
	{SG16500, &UA2024324},
	{SG16501, &UA2028300},
	{SG16502, &UA2028832},
	{SG16503, &UA2033401},
	{SG16504, &UA2393583},
	{SG16505, &UA2812514},
	{SG16506, &UA370929},
	{SG16507, &UA635581},
	{SG16508, &UA640151},
	{SG16509, &UA651027},
	{SG16510, &UA2022371},
	{SG16511, &UA2197719},
	{SG16512, &UA2762654},
	{SG16513, &UA2775203},
	{SG16514, &UA363327},
	{SG16515, &UA765740},
	{SG16516, &UA2399936},
	{SG16517, &UA2418802},
	{SG16525, &UA626404},
	{SG16526, &UA755219},
	{SG16534, &UA781007},
	{SG16535, &UA2190057},
	{SG16538, &UA2416311},
	{SG16540, &UA2419175},
	{SG16542, &UA2451363},
	{SG16543, &UA2452284},
	{SG16544, &UA2465078},
	{SG16545, &UA2465818},
	{SG16546, &UA369660},
	{SG16547, &UA370498},
	{SG16548, &UA370767},
	{SG16549, &UA2024871},
	{SG16550, &UA2815608},
	{SG16572, &UA625279},
	{SG16573, &UA641812},
	{SG16575, &UA898772},
	{SG16576, &UA1952280},
	{SG16577, &UA2759708},
	{SG16582, &UA785520},
	{SG16583, &UA924295},
	{SG16584, &UA2032086},
	{SG16585, &UA2163472},
	{SG16586, &UA2225902},
	{SG16587, &UA2236482},
	{SG16588, &UA2415361},
	{SG16589, &UA2416891},
	{SG16590, &UA2790038},
	{SG16591, &UA575081},
	{SG16592, &UA643543},
	{SG16593, &UA654621},
	{SG16594, &UA663891},
	{SG16595, &UA669737},
	{SG16596, &UA2025041},
	{SG16597, &UA2030180},
	{SG16598, &UA2034503},
	{SG16599, &UA2809512},
	{SG16622, &UA904380},
	{SG16623, &UA2027912},
	{SG16624, &UA2242223},
	{SG16625, &UA2743509},
	{SG16630, &UA626428},
	{SG16631, &UA632273},
	{SG16632, &UA2759050},
	{SG16666, &UA623731},
	{SG16667, &UA653278},
	{SG16668, &UA2685330},
	{SG16669, &UA2505210},
	{SG16670, &UA2530694},
	{SG16671, &UA2555440},
	{SG16672, &UA2555713},
	{SG16673, &UA2583033},
	{SG16674, &UA2655591},
	{SG16675, &UA2658604},
	{SG16676, &UA2668074},
	{SG16677, &UA2707078},
	{SG16678, &UA2734707},
	{SG16679, &UA2748864},
	{SG16680, &UA2751430},
	{SG16681, &UA2752626},
	{SG16682, &UA2797367},
	{SG16683, &UA2934993},
	{SG16684, &UA2987469},
	{SG16685, &UA2992589},
	{SG16686, &UA3057520},
	{SG16687, &UA2257155},
	{SG16688, &UA2396018},
	{SG16698, &UA456742},
	{SG16699, &UA584290},
	{SG16700, &UA2032327},
	{SG16701, &UA3041646},
	{SG16769, &UA896548},
	{SG16770, &UA2024158},
	{SG16771, &UA2830287},
	{SG16801, &UA578287},
	{SG16802, &UA871245},
	{SG16804, &UA627220},
	{SG16805, &UA1961517},
	{SG16806, &UA2643218},
	{SG16832, &UA352863},
	{SG16833, &UA622553},
	{SG16834, &UA759824},
	{SG16835, &UA2021264},
	{SG16836, &UA2029095},
	{SG16891, &UA909496},
	{SG16892, &UA2027796},
	{SG16893, &UA2853025},
	{SG16894, &UA827227},
	{SG16895, &UA924085},
	{SG16896, &UA2317812},
	{SG16897, &UA2789365},
	{SG16949, &UA438788},
	{SG16952, &UA438789},
	{SG16977, &UA2195950},
	{SG16978, &UA2355407},
	{SG16979, &UA2366761},
	{SG16980, &UA2385443},
	{SG16981, &UA403352},
	{SG16982, &UA663915},
	{SG16983, &UA706720},
	{SG16984, &UA746446},
	{SG16985, &UA1945158},
	{SG16986, &UA1955017},
	{SG16987, &UA2033296},
	{SG16992, &UA630033},
	{SG16993, &UA846840},
	{SG17011, &UA369048},
	{SG17012, &UA727364},
	{SG17013, &UA2024731},
	{SG17022, &UA693767},
	{SG17023, &UA699621},
	{SG17024, &UA359730},
	{SG17025, &UA397719},
	{SG17026, &UA590135},
	{SG17027, &UA611150},
	{SG17028, &UA624726},
	{SG17029, &UA626132},
	{SG17030, &UA633778},
	{SG17031, &UA2027495},
	{SG17032, &UA2028579},
	{SG17034, &UA2027803},
	{SG17035, &UA2481647},
	{SG17070, &UA2428545},
	{SG17071, &UA2435243},
	{SG17072, &UA592278},
	{SG17073, &UA802472},
	{SG17074, &UA1958100},
	{SG17075, &UA1989378},
	{SG17076, &UA2222758},
	{SG17093, &UA354724},
	{SG17094, &UA640363},
	{SG17095, &UA735762},
	{SG17096, &UA1911245},
	{SG17097, &UA1976876},
	{SG17100, &UA2750135},
	{SG17116, &UA1995373},
	{SG17136, &UA1956325},
	{SG17137, &UA2402419},
	{SG17154, &UA2602773},
	{SG17155, &UA2753409},
	{SG17156, &UA2781926},
	{SG17157, &UA3015655},
	{SG17168, &UA627373},
	{SG17169, &UA2032693},
	{SG17179, &UA2023898},
	{SG17180, &UA2466220},
	{SG17181, &UA380540},
	{SG17182, &UA625742},
	{SG17183, &UA713885},
	{SG17215, &UA677613},
	{SG17264, &UA2257126},
	{SG17266, &UA2777977},
	{SG17307, &UA2029716},
	{SG17308, &UA2031867},
	{SG17309, &UA2033148},
	{SG17310, &UA2452600},
	{SG17311, &UA2494500},
	{SG17312, &UA2575582},
	{SG17313, &UA2685775},
	{SG17314, &UA2759042},
	{SG17326, &UA2033985},
	{SG17327, &UA2157932},
	{SG17328, &UA2350640},
	{SG17329, &UA2412574},
	{SG17331, &UA895029},
	{SG17339, &UA2202587},
	{SG17388, &UA899014},
	{SG17389, &UA2543426},
	{SG17390, &UA624651},
	{SG17391, &UA887751},
	{SG17392, &UA623859},
	{SG17393, &UA1951173},
	{SG17394, &UA1957746},
	{SG17395, &UA1962395},
	{SG17396, &UA2024865},
	{SG17397, &UA744128},
	{SG17398, &UA2813116},
	{SG17399, &UA647481},
	{SG17400, &UA1937754},
	{SG17401, &UA2021029},
	{SG17402, &UA2732651},
	{SG17403, &UA2957929},
	{SG17409, &UA1995644},
	{SG17410, &UA2554857},
	{SG17411, &UA2689643},
	{SG17433, &UA2168680},
	{SG17435, &UA2497263},
	{SG17436, &UA2549942},
	{SG17509, &UA361739},
	{SG17510, &UA1982408},
	{SG17511, &UA2614285},
	{SG17512, &UA2615054},
	{SG17513, &UA2615788},
	{SG17514, &UA2616445},
	{SG17515, &UA2623329},
	{SG17516, &UA2626135},
	{SG17517, &UA2635557},
	{SG17534, &UA347057},
	{SG17535, &UA660168},
	{SG17537, &UA663801},
	{SG17539, &UA2423592},
	{SG17574, &UA2257159},
	{SG17575, &UA2466294},
	{SG17576, &UA2553531},
	{SG17577, &UA2777780},
	{SG17578, &UA2793098},
	{SG17579, &UA2837128},
	{SG17580, &UA2934081},
	{SG17581, &UA2984721},
	{SG17586, &UA900539},
	{SG17587, &UA2487271},
	{SG17588, &UA2685344},
	{SG17589, &UA2986356},
	{SG17607, &UA615554},
	{SG17609, &UA660196},
	{SG17611, &UA2022428},
	{SG17613, &UA2024809},
	{SG17655, &UA1949525},
	{SG17656, &UA2547531},
	{SG17682, &UA2029203},
	{SG17683, &UA2386040},
	{SG17684, &UA2554025},
	{SG17685, &UA2844860},
	{SG17686, &UA2772388},
	{SG17687, &UA2796199},
	{SG17688, &UA2957443},
	{SG17693, &UA676788},
	{SG17694, &UA702254},
	{SG17695, &UA2021279},
	{SG17696, &UA678561},
	{SG17697, &UA691862},
	{SG17698, &UA2022318},
	{SG17699, &UA2026483},
	{SG17700, &UA2354253},
	{SG17704, &UA1996631},
	{SG17705, &UA700311},
	{SG17706, &UA851717},
	{SG17707, &UA2587903},
	{SG17813, &UA2230079},
	{SG17814, &UA2423442},
	{SG17815, &UA2534887},
	{SG17862, &UA2480637},
	{SG17863, &UA2725644},
	{SG17864, &UA2421653},
	{SG17865, &UA2552437},
	{SG17866, &UA2735504},
	{SG17867, &UA2987064},
	{SG17868, &UA3008431},
	{SG17873, &UA2234913},
	{SG17874, &UA2524285},
	{SG17875, &UA2982183},
	{SG17887, &UA635322},
	{SG17888, &UA2423735},
	{SG17889, &UA2809123},
	{SG17890, &UA2362365},
	{SG17891, &UA2502931},
	{SG17892, &UA2655076},
	{SG17893, &UA2487061},
	{SG17894, &UA2580379},
	{SG17895, &UA2749803},
	{SG17896, &UA2751936},
	{SG17899, &UA2480079},
	{SG17900, &UA2531008},
	{SG17901, &UA2659294},
	{SG17902, &UA3002362},
	{SG17903, &UA3024938},
	{SG17908, &UA2507912},
	{SG17930, &UA625714},
	{SG17932, &UA852710},
	{SG17935, &UA607041},
	{SG17953, &UA2431428},
	{SG17954, &UA717489},
	{SG17955, &UA759368},
	{SG17956, &UA2034585},
	{SG17957, &UA2372319},
	{SG17997, &UA2027395},
	{SG17999, &UA753579},
	{SG18000, &UA757106},
	{SG18001, &UA2025964},
	{SG18002, &UA2547281},
	{SG18003, &UA2642858},
	{SG18004, &UA2810889},
	{SG18016, &UA671415},
	{SG18017, &UA2023634},
	{SG18026, &UA2205325},
	{SG18027, &UA2554454},
	{SG18028, &UA2315026},
	{SG18029, &UA2366902},
	{SG18030, &UA2476476},
	{SG18031, &UA2593338},
	{SG18032, &UA2652844},
	{SG18068, &UA606400},
	{SG18069, &UA2495351},
	{SG18076, &UA2488370},
	{SG18077, &UA2699811},
	{SG18078, &UA2703859},
	{SG18079, &UA2853487},
	{SG18080, &UA2974746},
	{SG18088, &UA2229872},
	{SG18089, &UA2594239},
	{SG18090, &UA2688681},
	{SG18091, &UA2943476},
	{SG18092, &UA791152},
	{SG18093, &UA2027278},
	{SG18094, &UA2349847},
	{SG18095, &UA2027860},
	{SG18098, &UA631948},
	{SG18099, &UA671980},
	{SG18100, &UA854243},
	{SG18101, &UA2222939},
	{SG18102, &UA2982143},
	{SG18103, &UA2714260},
	{SG18104, &UA2998295},
	{SG18105, &UA3000088},
	{SG18116, &UA2431068},
	{SG18171, &UA1965793},
	{SG18172, &UA2026124},
	{SG18173, &UA660967},
	{SG18174, &UA2025614},
	{SG18185, &UA2230709},
	{SG18186, &UA2578230},
	{SG18194, &UA688026},
	{SG18195, &UA840679},
	{SG18196, &UA2024954},
	{SG18200, &UA2431114},
	{SG18220, &UA790317},
	{SG18221, &UA2029644},
	{SG18222, &UA2227282},
	{SG18223, &UA2809533},
	{SG18236, &UA715404},
	{SG18237, &UA804205},
	{SG18238, &UA1965014},
	{SG18239, &UA2023172},
	{SG18240, &UA2483062},
	{SG18259, &UA2403459},
	{SG18260, &UA2455581},
	{SG18261, &UA2486015},
	{SG18262, &UA2504526},
	{SG18263, &UA2506514},
	{SG18264, &UA2603587},
	{SG18265, &UA2604174},
	{SG18268, &UA2665535},
	{SG18270, &UA2702949},
	{SG18272, &UA2747761},
	{SG18275, &UA2752720},
	{SG18276, &UA2852782},
	{SG18279, &UA2999251},
	{SG18283, &UA896805},
	{SG18298, &UA2237117},
	{SG18306, &UA438888},
	{SG18307, &UA2413837},
	{SG18308, &UA2415122},
	{SG18309, &UA2544665},
	{SG18310, &UA2889069},
	{SG18311, &UA2980887},
	{SG18316, &UA686697},
	{SG18317, &UA1946800},
	{SG18375, &UA750985},
	{SG18376, &UA2576839},
	{SG18377, &UA2722444},
	{SG18378, &UA2027188},
	{SG18443, &UA2732949},
	{SG18446, &UA2026743},
	{SG18451, &UA2237537},
	{SG18461, &UA1959045},
	{SG18462, &UA2758692},
	{SG18472, &UA2431426},
	{SG18488, &UA2489066},
	{SG18489, &UA3020106},
	{SG18497, &UA1974184},
	{SG18498, &UA1990741},
	{SG18499, &UA2025237},
	{SG18500, &UA2257139},
	{SG18501, &UA2372288},
	{SG18502, &UA2376063},
	{SG18503, &UA2786659},
	{SG18504, &UA2236469},
	{SG18505, &UA736527},
	{SG18506, &UA2810159},
	{SG18510, &UA858175},
	{SG18522, &UA625916},
	{SG18523, &UA813460},
	{SG18524, &UA651253},
	{SG18537, &UA587364},
	{SG18538, &UA656586},
	{SG18539, &UA2024434},
	{SG18545, &UA900968},
	{SG18546, &UA2573452},
	{SG18553, &UA2431067},
	{SG18564, &UA784750},
	{SG18565, &UA2445612},
	{SG18566, &UA2787642},
	{SG18573, &UA640798},
	{SG18574, &UA2642352},
	{SG18592, &UA2708141},
	{SG18593, &UA3102717},
	{SG18619, &UA2352584},
	{SG18620, &UA2552425},
	{SG18622, &UA2652787},
	{SG18624, &UA2746446},
	{SG18630, &UA2431179},
	{SG18639, &UA830996},
	{SG18640, &UA2588308},
	{SG18641, &UA1970023},
	{SG18642, &UA2759226},
	{SG18653, &UA2235598},
	{SG18656, &UA641248},
	{SG18657, &UA710579},
	{SG18660, &UA684860},
	{SG18661, &UA2775084},
	{SG18688, &UA2688473},
	{SG18693, &UA2403942},
	{SG18694, &UA3099379},
	{SG18695, &UA2795191},
	{SG18696, &UA2799511},
	{SG18697, &UA2800671},
	{SG18699, &UA2965031},
	{SG18709, &UA2361879},
	{SG18710, &UA2716826},
	{SG18711, &UA2432667},
	{SG18712, &UA2508560},
	{SG18713, &UA2792179},
	{SG18714, &UA3020789},
	{SG18722, &UA1988627},
	{SG18724, &UA2027213},
	{SG18726, &UA2029990},
	{SG18728, &UA2686435},
	{SG18737, &UA1986525},
	{SG18747, &UA2671928},
	{SG18748, &UA2965749},
	{SG18759, &UA2707618},
	{SG18766, &UA731605},
	{SG18767, &UA737447},
	{SG18768, &UA1992628},
	{SG18769, &UA2029217},
	{SG18781, &UA662903},
	{SG18782, &UA2027741},
	{SG18783, &UA2743471},
	{SG18784, &UA2032905},
	{SG18817, &UA781197},
	{SG18818, &UA2023774},
	{SG18819, &UA2033926},
	{SG18820, &UA2223387},
	{SG18821, &UA2643585},
	{SG18822, &UA2348337},
	{SG18823, &UA2420618},
	{SG18824, &UA2694207},
	{SG18825, &UA2839221},
	{SG18832, &UA892394},
	{SG18833, &UA2809385},
	{SG18834, &UA1979520},
	{SG18835, &UA2428491},
	{SG18848, &UA862960},
	{SG18853, &UA2431076},
	{SG18856, &UA2205327},
	{SG18883, &UA2753915},
	{SG18884, &UA2799417},
	{SG18885, &UA2366357},
	{SG18886, &UA2582143},
	{SG18887, &UA2009136},
	{SG18888, &UA1963439},
	{SG18889, &UA2026919},
	{SG18890, &UA2395828},
	{SG18891, &UA3070057},
	{SG18892, &UA2257158},
	{SG18893, &UA2835558},
	{SG18896, &UA2028131},
	{SG18900, &UA2512854},
	{SG18901, &UA3020457},
	{SG18904, &UA3058627},
	{SG18905, &UA3080229},
	{SG18907, &UA2424535},
	{SG18910, &UA2237116},
	{SG18911, &UA2420556},
	{SG18917, &UA2235101},
	{SG18918, &UA2257028},
	{SG18920, &UA2355830},
	{SG18921, &UA2790380},
	{SG18922, &UA2034336},
	{SG18923, &UA2496275},
	{SG18924, &UA2760108},
	{SG18925, &UA2484289},
	{SG18926, &UA2752562},
	{SG18927, &UA827306},
	{SG18928, &UA1958921},
	{SG18929, &UA1962293},
	{SG18930, &UA2032304},
	{SG18931, &UA2809341},
	{SG18942, &UA2716920},
	{SG18943, &UA2870157},
	{SG18944, &UA1997525},
	{SG18952, &UA899643},
	{SG18955, &UA853052},
	{SG18956, &UA2025019},
	{SG18957, &UA2441841},
	{SG18958, &UA2817131},
	{SG18966, &UA2653052},
	{SG18969, &UA1953075},
	{SG18971, &UA2347519},
	{SG18972, &UA2982074},
	{SG18996, &UA2222431},
	{SG19001, &UA2257137},
	{SG19003, &UA2715891},
	{SG19004, &UA2783837},
	{SG19013, &UA2236059},
	{SG19020, &UA2432663},
	{SG19030, &UA2510300},
	{SG19034, &UA2659919},
	{SG19037, &UA3020459},
	{SG19048, &UA2191310},
	{SG19049, &UA2530447},
	{SG19050, &UA2537989},
	{SG19051, &UA3024432},
	{SG19068, &UA744856},
	{SG19069, &UA908989},
	{SG19070, &UA1985222},
	{SG19071, &UA2998638},
	{SG19072, &UA1963451},
	{SG19073, &UA2383567},
	{SG19074, &UA2503816},
	{SG19075, &UA2817191},
	{SG19076, &UA2817346},
	{SG19089, &UA2020072},
	{SG19095, &UA3023004},
	{SG19096, &UA2350291},
	{SG19097, &UA2593888},
	{SG19109, &UA2712454},
	{SG19111, &UA2257167},
	{SG19112, &UA2652849},
	{SG19113, &UA2934948},
	{SG19135, &UA2510775},
	{SG19136, &UA3021055},
	{SG19137, &UA2354663},
	{SG19138, &UA583867},
	{SG19139, &UA1940393},
	{SG19140, &UA2024361},
	{SG19141, &UA2032571},
	{SG19146, &UA1990538},
	{SG19147, &UA2205352},
	{SG19148, &UA2230873},
	{SG19149, &UA2403212},
	{SG19156, &UA3023847},
	{SG19157, &UA3074155},
	{SG19158, &UA2598159},
	{SG19161, &UA2423327},
	{SG19167, &UA2431073},
	{SG19168, &UA2428320},
	{SG19169, &UA2663626},
	{SG19174, &UA2581108},
	{SG19175, &UA2599516},
	{SG19177, &UA2231518},
	{SG19181, &UA2366497},
	{SG19190, &UA2435166},
	{SG19191, &UA2460782},
	{SG19192, &UA2033335},
	{SG19212, &UA1960048},
	{SG19213, &UA1970692},
	{SG19214, &UA2458153},
	{SG19215, &UA2570784},
	{SG19216, &UA903166},
	{SG19217, &UA2962914},
	{SG19218, &UA3013233},
	{SG19219, &UA3013892},
	{SG19230, &UA848699},
	{SG19231, &UA2496227},
	{SG19232, &UA2512987},
	{SG19233, &UA3021198},
	{SG19236, &UA753949},
	{SG19237, &UA2026712},
	{SG19238, &UA2033784},
	{SG19239, &UA2352904},
	{SG19241, &UA2520575},
	{SG19243, &UA2431074},
	{SG19288, &UA2564317},
	{SG19289, &UA2961809},
	{SG19297, &UA1977999},
	{SG19298, &UA2024725},
	{SG19299, &UA2031694},
	{SG19300, &UA2758423},
	{SG19301, &UA3002609},
	{SG19304, &UA2769419},
	{SG19308, &UA2257109},
	{SG19309, &UA2717001},
	{SG19310, &UA2772021},
	{SG19326, &UA2600339},
	{SG19331, &UA2193302},
	{SG19332, &UA2728062},
	{SG19339, &UA2462049},
	{SG19340, &UA2469148},
	{SG19342, &UA2423129},
	{SG19344, &UA2951440},
	{SG19350, &UA2457374},
	{SG19363, &UA2740174},
	{SG19364, &UA843060},
	{SG19365, &UA2685763},
	{SG19366, &UA2431069},
	{SG19368, &UA2355159},
	{SG19369, &UA2505837},
	{SG19370, &UA680129},
	{SG19371, &UA1940880},
	{SG19372, &UA2034486},
	{SG19373, &UA2375758},
	{SG19374, &UA3042582},
	{SG19375, &UA3070259},
	{SG19376, &UA2027252},
	{SG19377, &UA2413941},
	{SG19378, &UA2789410},
	{SG19379, &UA625759},
	{SG19385, &UA1959910},
	{SG19387, &UA2404092},
	{SG19391, &UA2401382},
	{SG19392, &UA2579794},
	{SG19394, &UA2697655},
	{SG19396, &UA2704717},
	{SG19401, &UA743887},
	{SG19402, &UA2027610},
	{SG19403, &UA2031564},
	{SG19404, &UA2032082},
	{SG19405, &UA2412270},
	{SG19406, &UA2743656},
	{SG19414, &UA2466974},
	{SG19417, &UA369788},
	{SG19418, &UA2034536},
	{SG19419, &UA3117129},
	{SG19420, &UA1944961},
	{SG19421, &UA1981343},
	{SG19422, &UA2025083},
	{SG19423, &UA2686943},
	{SG19425, &UA2187906},
	{SG19426, &UA2483292},
	{SG19427, &UA2719271},
	{SG19430, &UA2234819},
	{SG19437, &UA2010271},
	{SG19440, &UA2029420},
	{SG19442, &UA2461335},
	{SG19443, &UA3023451},
	{SG19444, &UA2715883},
	{SG19445, &UA2703969},
	{SG19448, &UA2431450},
	{SG19453, &UA833063},
	{SG19454, &UA2026718},
	{SG19455, &UA2033833},
	{SG19456, &UA3042240},
	{SG19457, &UA2895333},
	{SG19458, &UA2507979},
	{SG19459, &UA2857134},
	{SG19460, &UA3087003},
	{SG19462, &UA1968691},
	{SG19463, &UA2029125},
	{SG19464, &UA2031526},
	{SG19465, &UA2820855},
	{SG19476, &UA2375882},
	{SG19477, &UA2522469},
	{SG19478, &UA2981529},
	{SG19479, &UA2984285},
	{SG19492, &UA1977916},
	{SG19493, &UA2810890},
	{SG19506, &UA1986552},
	{SG19507, &UA764620},
	{SG19508, &UA2782415},
	{SG19514, &UA1994053},
	{SG19521, &UA624485},
	{SG19522, &UA2034092},
	{SG19523, &UA1994025},
	{SG19524, &UA2413076},
	{SG19525, &UA2413489},
	{SG19527, &UA2449909},
	{SG19528, &UA2475623},
	{SG19529, &UA2758775},
	{SG19530, &UA2834583},
	{SG19534, &UA697599},
	{SG19535, &UA2024101},
	{SG19536, &UA2031607},
	{SG19540, &UA2655686},
	{SG19543, &UA2655745},
	{SG19544, &UA3044339},
	{SG19550, &UA3039278},
	{SG19575, &UA2400190},
	{SG19577, &UA680938},
	{SG19578, &UA2980363},
	{SG19584, &UA2353907},
	{SG19590, &UA2234999},
	{SG19591, &UA2504665},
	{SG19592, &UA2557568},
	{SG19596, &UA1990192},
	{SG19597, &UA3119288},
	{SG19598, &UA2666392},
	{SG19599, &UA2711877},
	{SG19603, &UA2737822},
	{SG19606, &UA2596876},
	{SG19607, &UA2845885},
	{SG19614, &UA2534722},
	{SG19620, &UA673895},
	{SG19623, &UA2580356},
	{SG19624, &UA2866155},
	{SG19625, &UA2997360},
	{SG19626, &UA3010247},
	{SG19637, &UA2354088},
	{SG19638, &UA2561772},
	{SG19639, &UA2598033},
	{SG19640, &UA2029998},
	{SG19642, &UA1959503},
	{SG19643, &UA3061678},
	{SG19645, &UA2355161},
	{SG19646, &UA2753837},
	{SG19660, &UA2199025},
	{SG19663, &UA2027153},
	{SG19665, &UA2030653},
	{SG19668, &UA2505177},
	{SG19669, &UA2730093},
	{SG19672, &UA2237122},
	{SG19673, &UA2480224},
	{SG19688, &UA1907180},
	{SG19692, &UA2770059},
	{SG19701, &UA2423631},
	{SG19705, &UA2842763},
	{SG19706, &UA2844190},
	{SG19707, &UA2508846},
	{SG19708, &UA3020547},
	{SG19715, &UA2197193},
	{SG19716, &UA2817892},
	{SG19717, &UA2707923},
	{SG19718, &UA1964986},
	{SG19719, &UA2234651},
	{SG19720, &UA842477},
	{SG19721, &UA2597929},
	{SG19722, &UA2431206},
	{SG19727, &UA2771843},
	{SG19736, &UA1952440},
	{SG19737, &UA2933521},
	{SG19738, &UA2933530},
	{SG19743, &UA657011},
	{SG19744, &UA2026470},
	{SG19745, &UA2375973},
	{SG19768, &UA2530068},
	{SG19774, &UA890878},
	{SG19777, &UA2879903},
	{SG19778, &UA2233385},
	{SG19779, &UA2987548},
	{SG19780, &UA2382060},
	{SG19781, &UA2382824},
	{SG19785, &UA2032372},
	{SG19790, &UA1986032},
	{SG19791, &UA2032723},
	{SG19792, &UA2185335},
	{SG19793, &UA2763443},
	{SG19794, &UA2748270},
	{SG19796, &UA2026220},
	{SG19798, &UA2028503},
	{SG19799, &UA2257073},
	{SG19800, &UA2574457},
	{SG19801, &UA2589493},
	{SG19802, &UA2816370},
	{SG19803, &UA2818781},
	{SG19804, &UA2822134},
	{SG19812, &UA2456113},
	{SG19820, &UA2879654},
	{SG19821, &UA2958266},
	{SG19822, &UA3001022},
	{SG19827, &UA1952166},
	{SG19828, &UA2332546},
	{SG19830, &UA2376034},
	{SG19849, &UA2028544},
	{SG19850, &UA2417073},
	{SG19851, &UA2817295},
	{SG19853, &UA2505062},
	{SG19854, &UA2796550},
	{SG19859, &UA2033670},
	{SG19872, &UA1988423},
	{SG19873, &UA2565729},
	{SG19874, &UA2736808},
	{SG19879, &UA2754020},
	{SG19881, &UA2431078},
	{SG19885, &UA2384863},
	{SG19887, &UA2718551},
	{SG19889, &UA2709183},
	{SG19891, &UA2722446},
	{SG19892, &UA2428003},
	{SG19893, &UA2503173},
	{SG19896, &UA689328},
	{SG19897, &UA2021762},
	{SG19898, &UA2033886},
	{SG19899, &UA1973675},
	{SG19901, &UA2412867},
	{SG19903, &UA3086655},
	{SG19912, &UA1992304},
	{SG19916, &UA2023941},
	{SG19918, &UA2033998},
	{SG19925, &UA2738095},
	{SG19933, &UA2461329},
	{SG19937, &UA2508473},
	{SG19939, &UA3125050},
	{SG19959, &UA2380322},
	{SG19961, &UA2339952},
	{SG19962, &UA2374314},
	{SG19963, &UA2574489},
	{SG19965, &UA1949478},
	{SG19968, &UA2557236},
	{SG19975, &UA1985117},
	{SG19976, &UA2347549},
	{SG19977, &UA2593616},
	{SG19978, &UA2375384},
	{SG19979, &UA2589741},
	{SG19982, &UA2559812},
	{SG19983, &UA2033972},
	{SG19984, &UA2190405},
	{SG19985, &UA2233270},
	{SG19988, &UA909556},
	{SG19990, &UA2667135},
	{SG19991, &UA2667910},
	{SG19992, &UA2669396},
	{SG19993, &UA2538024},
	{SG19994, &UA3022572},
	{SG20023, &UA2395837},
	{SG20024, &UA2546221},
	{SG20025, &UA2482559},
	{SG20026, &UA2431917},
	{SG20031, &UA2189576},
	{SG20045, &UA1938141},
	{SG20046, &UA2412115},
	{SG20055, &UA2028149},
	{SG20056, &UA2816308},
	{SG20057, &UA2784444},
	{SG20058, &UA2427014},
	{SG20059, &UA2526634},
	{SG20064, &UA2896345},
	{SG20069, &UA789266},
	{SG20070, &UA2432257},
	{SG20077, &UA3024868},
	{SG20084, &UA2771648},
	{SG20085, &UA1995951},
	{SG20089, &UA368822},
	{SG20090, &UA370774},
	{SG20091, &UA787166},
	{SG20092, &UA805703},
	{SG20093, &UA1939174},
	{SG20094, &UA1946331},
	{SG20095, &UA2026317},
	{SG20096, &UA2031860},
	{SG20105, &UA2026930},
	{SG20106, &UA2032844},
	{SG20107, &UA2520475},
	{SG20108, &UA2743760},
	{SG20109, &UA2796575},
	{SG20111, &UA2028670},
	{SG20116, &UA2817864},
	{SG20117, &UA2029104},
	{SG20118, &UA2686497},
	{SG20119, &UA3073975},
	{SG20129, &UA1970008},
	{SG20138, &UA2257097},
	{SG20139, &UA2899090},
	{SG20140, &UA3029131},
	{SG20144, &UA2666320},
	{SG20149, &UA2349190},
	{SG20152, &UA2262264},
	{SG20159, &UA2383117},
	{SG20160, &UA2352715},
	{SG20162, &UA2667720},
	{SG20163, &UA2901215},
	{SG20165, &UA2354922},
	{SG20172, &UA2605725},
	{SG20173, &UA2713547},
	{SG20177, &UA2482217},
	{SG20178, &UA2715760},
	{SG20192, &UA2486287},
	{SG20193, &UA2831576},
	{SG20194, &UA2954230},
	{SG20195, &UA2981578},
	{SG20197, &UA2469869},
	{SG20198, &UA2602705},
	{SG20199, &UA2718569},
	{SG20200, &UA2769097},
	{SG20204, &UA2353982},
	{SG20205, &UA2894854},
	{SG20206, &UA2530413},
	{SG20209, &UA3021401},
	{SG20210, &UA2531091},
	{SG20212, &UA2030019},
	{SG20219, &UA2433073},
	{SG20231, &UA3109572},
	{SG20232, &UA3127412},
	{SG20234, &UA2762248},
	{SG20235, &UA2770881},
	{SG20236, &UA2223002},
	{SG20237, &UA2235004},
	{SG20238, &UA2719350},
	{SG20246, &UA611542},
	{SG20247, &UA2563772},
	{SG20260, &UA2422970},
	{SG20261, &UA2772195},
	{SG20275, &UA2963726},
	{SG20276, &UA369635},
	{SG20277, &UA2758702},
	{SG20278, &UA663864},
	{SG20306, &UA2257149},
	{SG20325, &UA2235144},
	{SG20326, &UA2230810},
	{SG20328, &UA2531066},
	{SG20340, &UA872381},
	{SG20341, &UA2958390},
	{SG20342, &UA1993430},
	{SG20343, &UA2031548},
	{SG20344, &UA2775760},
	{SG20345, &UA3098846},
	{SG20346, &UA2716634},
	{SG20347, &UA3094026},
	{SG20361, &UA2952961},
	{SG20364, &UA2234643},
	{SG20380, &UA2381716},
	{SG20382, &UA2384453},
	{SG20399, &UA899898},
	{SG20400, &UA1986204},
	{SG20401, &UA2531184},
	{SG20409, &UA2837517},
	{SG20410, &UA2237681},
	{SG20411, &UA2833595},
	{SG20413, &UA2428373},
	{SG20415, &UA2546134},
	{SG20417, &UA2696295},
	{SG20424, &UA2176715},
	{SG20427, &UA2897726},
	{SG20443, &UA2032607},
	{SG20444, &UA2445397},
	{SG20446, &UA2686769},
	{SG20448, &UA2664452},
	{SG20449, &UA2699373},
	{SG20450, &UA2348751},
	{SG20451, &UA2933687},
	{SG20452, &UA3042920},
	{SG20453, &UA2879672},
	{SG20456, &UA2030349},
	{SG20457, &UA2461249},
	{SG20458, &UA2560313},
	{SG20459, &UA2666996},
	{SG20464, &UA2423843},
	{SG20471, &UA2619871},
	{SG20472, &UA2999450},
	{SG20479, &UA2033367},
	{SG20480, &UA2033995},
	{SG20481, &UA2476025},
	{SG20483, &UA2809281},
	{SG20484, &UA2665961},
	{SG20485, &UA2849401},
	{SG20486, &UA2754005},
	{SG20490, &UA2190380},
	{SG20497, &UA2752636},
	{SG20501, &UA2530341},
	{SG20512, &UA3012468},
	{SG20517, &UA2531122},
	{SG20520, &UA2581758},
	{SG20521, &UA3021934},
	{SG20522, &UA3025613},
	{SG20523, &UA3094455},
	{SG20524, &UA1984025},
	{SG20526, &UA2262248},
	{SG20533, &UA2715723},
	{SG20535, &UA2768486},
	{SG20544, &UA1973340},
	{SG20545, &UA2589897},
	{SG20547, &UA2461814},
	{SG20548, &UA2482106},
	{SG20549, &UA2897744},
	{SG20552, &UA2713596},
	{SG20553, &UA2737060},
	{SG20554, &UA2865392},
	{SG20556, &UA896358},
	{SG20557, &UA2659441},
	{SG20559, &UA2789439},
	{SG20560, &UA2817023},
	{SG20567, &UA2487486},
	{SG20568, &UA2665533},
	{SG20572, &UA3059740},
	{SG20573, &UA3125479},
	{SG20575, &UA2483119},
	{SG20578, &UA2659022},
	{SG20579, &UA2952109},
	{SG20580, &UA2232828},
	{SG20597, &UA2025662},
	{SG20606, &UA1972266},
	{SG20607, &UA2660767},
	{SG20609, &UA2520522},
	{SG20610, &UA2808624},
	{SG20618, &UA2529403},
	{SG20622, &UA2027372},
	{SG20624, &UA2964515},
	{SG20627, &UA3080074},
	{SG20628, &UA2770904},
	{SG20629, &UA2774126},
	{SG20630, &UA2901191},
	{SG20635, &UA2616322},
	{SG20640, &UA2549847},
	{SG20644, &UA3023662},
	{SG20645, &UA2024590},
	{SG20646, &UA2027224},
	{SG20647, &UA2495225},
	{SG20648, &UA2808712},
	{SG20649, &UA2257054},
	{SG20663, &UA2980370},
	{SG20665, &UA2353713},
	{SG20666, &UA2395625},
	{SG20667, &UA2524231},
	{SG20669, &UA2505542},
	{SG20679, &UA2022327},
	{SG20680, &UA2025154},
	{SG20688, &UA2752008},
	{SG20691, &UA2237675},
	{SG20709, &UA2652880},
	{SG20710, &UA2746393},
	{SG20711, &UA2481563},
	{SG20717, &UA2767441},
	{SG20722, &UA2509067},
	{SG20726, &UA2579797},
	{SG20729, &UA2621220},
	{SG20733, &UA3026143},
	{SG20734, &UA2431682},
	{SG20736, &UA2529828},
	{SG20739, &UA2565242},
	{SG20740, &UA3020438},
	{SG20741, &UA2877105},
	{SG20747, &UA2034510},
	{SG20748, &UA2469881},
	{SG20749, &UA2750596},
	{SG20754, &UA1990029},
	{SG20756, &UA2686803},
	{SG20761, &UA2030902},
	{SG20763, &UA2032146},
	{SG20768, &UA2901234},
	{SG20778, &UA3000066},
	{SG20779, &UA2658399},
	{SG20782, &UA2028472},
	{SG20789, &UA2453563},
	{SG20790, &UA3021607},
	{SG20794, &UA2383309},
	{SG20801, &UA2534839},
	{SG20802, &UA3021083},
	{SG20803, &UA2629927},
	{SG20824, &UA2467472},
	{SG20841, &UA2234908},
	{SG20842, &UA2338478},
	{SG20843, &UA2338760},
	{SG20844, &UA2354034},
	{SG20845, &UA2481374},
	{SG20846, &UA2670005},
	{SG20850, &UA2462051},
	{SG20855, &UA2944276},
	{SG20861, &UA2753367},
	{SG20862, &UA3078986},
	{SG20864, &UA3108706},
	{SG20865, &UA2257082},
	{SG20866, &UA2366444},
	{SG20871, &UA2800573},
	{SG20872, &UA2714943},
	{SG20875, &UA2382751},
	{SG20877, &UA2739794},
	{SG20882, &UA903213},
	{SG20885, &UA2375117},
	{SG20887, &UA2849686},
	{SG20888, &UA2469862},
	{SG20889, &UA2696836},
	{SG20890, &UA2773242},
	{SG20893, &UA3023580},
	{SG20894, &UA3030087},
	{SG20899, &UA2375226},
	{SG20901, &UA2945597},
	{SG20908, &UA2421639},
	{SG20910, &UA2262074},
	{SG20918, &UA2503224},
	{SG20919, &UA3027684},
	{SG20924, &UA2482501},
	{SG20928, &UA2532223},
	{SG20932, &UA3023735},
	{SG20938, &UA2944810},
	{SG20939, &UA3109270},
	{SG20941, &UA2944454},
	{SG20946, &UA2027002},
	{SG20947, &UA2228099},
	{SG20954, &UA2027394},
	{SG20955, &UA2349478},
	{SG20956, &UA2384341},
	{SG20961, &UA2944560},
	{SG20967, &UA2570262},
	{SG20968, &UA2706000},
	{SG20974, &UA2732147},
	{SG20975, &UA2799208},
	{SG20976, &UA2939223},
	{SG20978, &UA1995045},
	{SG20979, &UA2343261},
	{SG20981, &UA2799045},
	{SG20982, &UA2963495},
	{SG20988, &UA2898798},
	{SG20995, &UA2530452},
	{SG20999, &UA2781796},
	{SG21001, &UA2939888},
	{SG21006, &UA2354055},
	{SG21015, &UA2817568},
	{SG21023, &UA2600081},
	{SG21024, &UA2730024},
	{SG21025, &UA2850551},
	{SG21027, &UA2751895},
	{SG21028, &UA3111002},
	{SG21030, &UA2617017},
	{SG21031, &UA2617464},
	{SG21052, &UA885142},
	{SG21053, &UA2026746},
	{SG21056, &UA2029730},
	{SG21070, &UA2795096},
	{SG21076, &UA1988263},
	{SG21077, &UA2460054},
	{SG21092, &UA2733704},
	{SG21095, &UA2753368},
	{SG21099, &UA2223377},
	{SG21103, &UA2402267},
	{SG21105, &UA2455033},
	{SG21106, &UA2697611},
	{SG21107, &UA2527992},
	{SG21114, &UA2667763},
	{SG21125, &UA2340602},
	{SG21126, &UA2383779},
	{SG21127, &UA2341584},
	{SG21128, &UA2421825},
	{SG21135, &UA2027075},
	{SG21139, &UA2481937},
	{SG21141, &UA2663701},
	{SG21143, &UA2850941},
	{SG21144, &UA3058240},
	{SG21148, &UA2940159},
	{SG21153, &UA2565861},
	{SG21155, &UA2934794},
	{SG21157, &UA2709318},
	{SG21161, &UA2779701},
	{SG21162, &UA3100825},
	{SG21165, &UA2398527},
	{SG21166, &UA2424085},
	{SG21167, &UA2551681},
	{SG21168, &UA2598050},
	{SG21171, &UA2964274},
	{SG21172, &UA2964719},
	{SG21173, &UA2966947},
	{SG21174, &UA2685925},
	{SG21176, &UA2380404},
	{SG21178, &UA2933828},
	{SG21180, &UA2565742},
	{SG21182, &UA2870695},
	{SG21184, &UA2952640},
	{SG21190, &UA3093826},
	{SG21195, &UA2027250},
	{SG21196, &UA2651361},
	{SG21199, &UA2428708},
	{SG21206, &UA3015915},
	{SG21211, &UA2029320},
	{SG21212, &UA2765587},
	{SG21217, &UA2026166},
	{SG21218, &UA2685928},
	{SG21219, &UA2817029},
	{SG21220, &UA3099320},
	{SG21223, &UA2755383},
	{SG21226, &UA2455906},
	{SG21231, &UA1958543},
	{SG21235, &UA2478423},
	{SG21236, &UA2504341},
	{SG21237, &UA2667726},
	{SG21242, &UA2799965},
	{SG21245, &UA2026807},
	{SG21246, &UA2344181},
	{SG21249, &UA2554748},
	{SG21250, &UA2559424},
	{SG21255, &UA2938029},
	{SG21256, &UA783821},
	{SG21258, &UA2034391},
	{SG21260, &UA2469864},
	{SG21261, &UA2486406},
	{SG21262, &UA2557595},
	{SG21266, &UA3060960},
	{SG21267, &UA789390},
	{SG21269, &UA2609722},
	{SG21270, &UA2610553},
	{SG21272, &UA2529910},
	{SG21274, &UA2469230},
	{SG21275, &UA2504396},
	{SG21276, &UA2522579},
	{SG21277, &UA2758483},
	{SG21284, &UA2032696},
	{SG21285, &UA2869967},
	{SG21290, &UA2033649},
	{SG21294, &UA2867686},
	{SG21297, &UA2483300},
	{SG21300, &UA2863287},
	{SG21301, &UA2737761},
	{SG21302, &UA2738003},
	{SG21303, &UA2752546},
	{SG21305, &UA2865352},
	{SG21306, &UA2868989},
	{SG21310, &UA2939379},
	{SG21311, &UA2939875},
	{SG21333, &UA2381943},
	{SG21346, &UA2024663},
	{SG21347, &UA2028879},
	{SG21348, &UA2353298},
	{SG21350, &UA2752290},
	{SG21376, &UA2553940},
	{SG21377, &UA2790582},
	{SG21378, &UA2735610},
	{SG21384, &UA2644540},
	{SG21387, &UA2736927},
	{SG21401, &UA2598656},
	{SG21402, &UA2727214},
	{SG21403, &UA2768551},
	{SG21404, &UA2867143},
	{SG21405, &UA2867475},
	{SG21406, &UA2938611},
	{SG21411, &UA2799512},
	{SG21419, &UA3062147},
	{SG21429, &UA2558454},
	{SG21433, &UA2879443},
	{SG21435, &UA2868937},
	{SG21438, &UA2033302},
	{SG21441, &UA2658828},
	{SG21442, &UA2751991},
	{SG21447, &UA2880679},
	{SG21450, &UA2817167},
	{SG21451, &UA2455512},
	{SG21452, &UA2457581},
	{SG21453, &UA2752500},
	{SG21454, &UA2768966},
	{SG21458, &UA2554560},
	{SG21459, &UA2642448},
	{SG21460, &UA2879322},
	{SG21461, &UA2879191},
	{SG21467, &UA2026621},
	{SG21468, &UA2481035},
	{SG21472, &UA2731064},
	{SG21475, &UA2892834},
	{SG21476, &UA2961057},
	{SG21480, &UA2360564},
	{SG21481, &UA2480866},
	{SG21484, &UA2697797},
	{SG21489, &UA2957138},
	{SG21490, &UA3016639},
	{SG21499, &UA2478017},
	{SG21502, &UA2559574},
	{SG21503, &UA2578285},
	{SG21504, &UA2768702},
	{SG21507, &UA2960307},
	{SG21514, &UA2743486},
	{SG21517, &UA2480210},
	{SG21521, &UA2661008},
	{SG21522, &UA2768425},
	{SG21526, &UA2878679},
	{SG21527, &UA3028113},
	{SG21533, &UA2029948},
	{SG21534, &UA2030365},
	{SG21535, &UA2480906},
	{SG21536, &UA2704528},
	{SG21537, &UA2737246},
	{SG21538, &UA2752402},
	{SG21539, &UA2867218},
	{SG21551, &UA2594117},
	{SG21558, &UA2894217},
	{SG21559, &UA2937755},
	{SG21564, &UA2339387},
	{SG21566, &UA2452949},
	{SG21574, &UA2983263},
	{SG21575, &UA3079076},
	{SG21579, &UA2457535},
	{SG21580, &UA2659677},
	{SG21581, &UA2892496},
	{SG21582, &UA2958349},
	{SG21589, &UA2034179},
	{SG21590, &UA2580666},
	{SG21591, &UA2714154},
	{SG21595, &UA2768705},
	{SG21596, &UA2893952},
	{SG21600, &UA698950},
	{SG21607, &UA2482071},
	{SG21610, &UA3034443},
	{SG21611, &UA3104177},
	{SG21613, &UA3079356},
	{SG21614, &UA1995828},
	{SG21615, &UA2384353},
	{SG21617, &UA2502252},
	{SG21620, &UA2715999},
	{SG21621, &UA2799494},
	{SG21622, &UA2893031},
	{SG21623, &UA3079353},
	{SG21625, &UA2659594},
	{SG21626, &UA3058321},
	{SG21627, &UA2662062},
	{SG21628, &UA3124951},
	{SG21630, &UA2764936},
	{SG21632, &UA2814243},
	{SG21633, &UA2852773},
	{SG21634, &UA2890794},
	{SG21635, &UA3086775},
	{SG21636, &UA2937402},
	{SG21639, &UA3113537},
	{SG21641, &UA2979900},
	{SG21642, &UA2260444},
	{SG21643, &UA2366128},
	{SG21644, &UA2620542},
	{SG21646, &UA2891368},
	{SG21648, &UA3011244},
	{SG21650, &UA3058611},
	{SG21654, &UA2526594},
	{SG21655, &UA2700950},
	{SG21656, &UA2938035},
	{SG21659, &UA3059067},
	{SG21663, &UA3108447},
	{SG21664, &UA3109069},
	{SG21665, &UA3110065},
	{SG21672, &UA2021907},
	{SG21674, &UA2373775},
	{SG21675, &UA2425023},
	{SG21676, &UA2502632},
	{SG21682, &UA2727218},
	{SG21683, &UA2937424},
	{SG21686, &UA2998648},
	{SG21687, &UA2027572},
	{SG21688, &UA2934895},
	{SG21689, &UA2025955},
	{SG21690, &UA2727162},
	{SG21692, &UA2583804},
	{SG21693, &UA2706840},
	{SG21694, &UA2727260},
	{SG21699, &UA2859225},
	{SG21700, &UA2865416},
	{SG21710, &UA2598074},
	{SG21711, &UA2730006},
	{SG21712, &UA2866051},
	{SG21713, &UA2958287},
	{SG21719, &UA3079610},
	{SG21720, &UA3079802},
	{SG21721, &UA3108736},
	{SG21722, &UA3094695},
	{SG21724, &UA3125958},
	{SG21728, &UA1988914},
	{SG21729, &UA2478027},
	{SG21730, &UA2715888},
	{SG21735, &UA2959701},
	{SG21737, &UA3058663},
	{SG21739, &UA2999641},
	{SG21740, &UA3088512},
	{SG21741, &UA2031460},
	{SG21746, &UA2527952},
	{SG21748, &UA2554942},
	{SG21752, &UA2730789},
	{SG21753, &UA2934699},
	{SG21754, &UA2764996},
	{SG21756, &UA2962182},
	{SG21758, &UA3079710},
	{SG21765, &UA2384340},
	{SG21766, &UA2426826},
	{SG21767, &UA2553253},
	{SG21768, &UA2694811},
	{SG21771, &UA3027612},
	{SG21778, &UA2029227},
	{SG21779, &UA2459976},
	{SG21780, &UA2597690},
	{SG21781, &UA2630598},
	{SG21784, &UA2734584},
	{SG21785, &UA2750654},
	{SG21787, &UA3005177},
	{SG21806, &UA2863117},
	{SG21808, &UA3029925},
	{SG21813, &UA2860737},
	{SG21814, &UA2876807},
	{SG21817, &UA2997511},
	{SG21818, &UA3058310},
	{SG21821, &UA2029307},
	{SG21823, &UA2398686},
	{SG21830, &UA2668238},
	{SG21831, &UA2696020},
	{SG21833, &UA2767408},
	{SG21834, &UA2961999},
	{SG21838, &UA3003850},
	{SG21843, &UA3049034},
	{SG21844, &UA3058689},
	{SG21845, &UA3060585},
	{SG21851, &UA2025095},
	{SG21852, &UA2028969},
	{SG21853, &UA2030376},
	{SG21854, &UA2031431},
	{SG21856, &UA2453495},
	{SG21858, &UA2520683},
	{SG21861, &UA2862317},
	{SG21862, &UA2937722},
	{SG21864, &UA2963419},
	{SG21865, &UA3088450},
	{SG21866, &UA3121637},
	{SG21870, &UA2027885},
	{SG21878, &UA2735168},
	{SG21880, &UA2748354},
	{SG21881, &UA2779419},
	{SG21884, &UA2852213},
	{SG21886, &UA2892395},
	{SG21892, &UA368823},
	{SG21893, &UA828970},
	{SG21896, &UA2462411},
	{SG21897, &UA2462414},
	{SG21901, &UA2578258},
	{SG21902, &UA2594089},
	{SG21903, &UA2659219},
	{SG21906, &UA2713107},
	{SG21907, &UA2736864},
	{SG21908, &UA2892120},
	{SG21910, &UA2937834},
	{SG21912, &UA2957511},
	{SG21925, &UA2502854},
	{SG21926, &UA2578778},
	{SG21927, &UA2580474},
	{SG21928, &UA2598684},
	{SG21936, &UA2937808},
	{SG21938, &UA2956998},
	{SG21939, &UA2960482},
	{SG21945, &UA2623852},
	{SG21947, &UA2028259},
	{SG21948, &UA2339753},
	{SG21951, &UA2339568},
	{SG21952, &UA2503836},
	{SG21953, &UA2546137},
	{SG21957, &UA2792790},
	{SG21958, &UA2957526},
	{SG21959, &UA2958101},
	{SG21961, &UA2997808},
	{SG21967, &UA1950601},
	{SG21968, &UA2027664},
	{SG21969, &UA2029010},
	{SG21970, &UA2338761},
	{SG21971, &UA2951230},
	{SG21977, &UA2553205},
	{SG21980, &UA2668938},
	{SG21983, &UA2764601},
	{SG21984, &UA2799733},
	{SG21985, &UA2826691},
	{SG21987, &UA2868035},
	{SG21989, &UA2891014},
	{SG21990, &UA3057093},
	{SG21991, &UA3114165},
	{SG21998, &UA2028339},
	{SG22000, &UA2029071},
	{SG22002, &UA2589678},
	{SG22005, &UA2777931},
	{SG22010, &UA3001760},
	{SG22013, &UA3079461},
	{SG22015, &UA3079577},
	{SG22016, &UA3114796},
	{SG22018, &UA2022123},
	{SG22021, &UA1959544},
	{SG22022, &UA1992702},
	{SG22023, &UA2031244},
	{SG22028, &UA2701575},
	{SG22034, &UA2967059},
	{SG22035, &UA2999089},
	{SG22049, &UA2425020},
	{SG22050, &UA2552778},
	{SG22053, &UA2654069},
	{SG22059, &UA2782010},
	{SG22061, &UA2958226},
	{SG22062, &UA2958313},
	{SG22063, &UA2964668},
	{SG22065, &UA3002800},
	{SG22066, &UA3007306},
	{SG22071, &UA3018822},
	{SG22078, &UA768733},
	{SG22082, &UA1970087},
	{SG22083, &UA2025102},
	{SG22084, &UA2704539},
	{SG22085, &UA2734764},
	{SG22086, &UA2849245},
	{SG22089, &UA2867619},
	{SG22092, &UA2957550},
	{SG22095, &UA3051156},
	{SG22098, &UA3108974},
	{SG22105, &UA787483},
	{SG22107, &UA2027738},
	{SG22108, &UA2029004},
	{SG22109, &UA2030091},
	{SG22110, &UA2032073},
	{SG22112, &UA2997560},
	{SG22114, &UA2448771},
	{SG22115, &UA2481263},
	{SG22119, &UA2653097},
	{SG22120, &UA2665047},
	{SG22122, &UA2707874},
	{SG22123, &UA2735064},
	{SG22124, &UA2746620},
	{SG22125, &UA2750850},
	{SG22135, &UA3010499},
	{SG22139, &UA3109367},
	{SG22148, &UA2020858},
	{SG22149, &UA2028593},
	{SG22153, &UA2033828},
	{SG22161, &UA2659252},
	{SG22165, &UA2794287},
	{SG22167, &UA2856577},
	{SG22168, &UA2863116},
	{SG22172, &UA2950537},
	{SG22176, &UA2999205},
	{SG22178, &UA3015497},
	{SG22182, &UA3034541},
	{SG22185, &UA3109046},
	{SG22196, &UA2028346},
	{SG22198, &UA2400884},
	{SG22201, &UA2436653},
	{SG22206, &UA2594741},
	{SG22208, &UA2644264},
	{SG22209, &UA2652726},
	{SG22210, &UA2659405},
	{SG22211, &UA2661004},
	{SG22212, &UA2668579},
	{SG22213, &UA2748259},
	{SG22217, &UA2868941},
	{SG22218, &UA2890544},
	{SG22220, &UA2999588},
	{SG22221, &UA3052243},
	{SG22222, &UA3055183},
	{SG22225, &UA2544569},
	{SG22226, &UA2028375},
	{SG22227, &UA2337956},
	{SG22228, &UA2451727},
	{SG22229, &UA2552908},
	{SG22230, &UA2648806},
	{SG22231, &UA2694928},
	{SG22232, &UA2701156},
	{SG22235, &UA2729780},
	{SG22237, &UA2731404},
	{SG22239, &UA2734742},
	{SG22242, &UA2766835},
	{SG22243, &UA2799808},
	{SG22245, &UA2937421},
	{SG22247, &UA2956364},
	{SG22248, &UA2963322},
	{SG22250, &UA2997831},
	{SG22255, &UA3110066},
	{SG22261, &UA2579510},
	{SG22276, &UA2867719},
	{SG22277, &UA2878350},
	{SG22279, &UA2879458},
	{SG22280, &UA2938171},
	{SG22283, &UA2957569},
	{SG22284, &UA2985669},
	{SG22286, &UA3008678},
	{SG22287, &UA3012894},
	{SG22288, &UA3016836},
	{SG22290, &UA3074399},
	{SG22299, &UA2592977},
	{SG22300, &UA2657239},
	{SG22301, &UA2701362},
	{SG22302, &UA2701372},
	{SG22303, &UA2702373},
	{SG22304, &UA2703665},
	{SG22305, &UA2780784},
	{SG22307, &UA2795596},
	{SG22312, &UA2937193},
	{SG22314, &UA2982529},
	{SG22315, &UA2986461},
	{SG22317, &UA2998425},
	{SG22318, &UA3050645},
	{SG22319, &UA3103338},
	{SG22322, &UA3109672},
	{SG22324, &UA2455799},
	{SG22325, &UA2653487},
	{SG22326, &UA2659797},
	{SG22327, &UA2694318},
	{SG22328, &UA2704243},
	{SG22330, &UA2734554},
	{SG22332, &UA2748245},
	{SG22334, &UA2779400},
	{SG22335, &UA2781312},
	{SG22336, &UA2855236},
	{SG22343, &UA2958087},
	{SG22349, &UA2997266},
	{SG22352, &UA2998495},
	{SG22353, &UA3029613},
	{SG22354, &UA3074011},
	{SG22356, &UA3074089},
	{SG22357, &UA3077621},
	{SG22359, &UA3079839},
	{SG22366, &UA2338124},
	{SG22367, &UA2421676},
	{SG22370, &UA2659814},
	{SG22371, &UA2661022},
	{SG22373, &UA2695967},
	{SG22377, &UA2751143},
	{SG22378, &UA2761276},
	{SG22379, &UA2791721},
	{SG22381, &UA3000871},
	{SG22382, &UA3013687},
	{SG22384, &UA3016988},
	{SG22386, &UA3022374},
	{SG22388, &UA3049269},
	{SG22390, &UA3074678},
	{SG22391, &UA3075157},
	{SG22393, &UA3106273},
	{SG22394, &UA3108837},
	{SG22402, &UA2028213},
	{SG22404, &UA2528045},
	{SG22405, &UA2596848},
	{SG22409, &UA2695051},
	{SG22410, &UA2695326},
	{SG22411, &UA2695562},
	{SG22413, &UA2701381},
	{SG22415, &UA2762931},
	{SG22416, &UA2780274},
	{SG22418, &UA2780687},
	{SG22420, &UA2841933},
	{SG22422, &UA2843217},
	{SG22423, &UA2850553},
	{SG22424, &UA2857610},
	{SG22425, &UA2862243},
	{SG22435, &UA3056747},
	{SG22437, &UA3088850},
	{SG22439, &UA3107728},
	{SG22440, &UA3108298},
	{SG22441, &UA3108364},
	{SG22442, &UA3108839},
	{SG22447, &UA3121312},
	{SG22454, &UA2032059},
	{SG22455, &UA2442315},
	{SG22456, &UA2461322},
	{SG22458, &UA2506364},
	{SG22459, &UA2526331},
	{SG22460, &UA2593179},
	{SG22462, &UA2634695},
	{SG22464, &UA2648760},
	{SG22465, &UA2654862},
	{SG22466, &UA2657355},
	{SG22468, &UA2705089},
	{SG22469, &UA2766760},
	{SG22470, &UA2766795},
	{SG22471, &UA2779080},
	{SG22472, &UA2781974},
	{SG22473, &UA2833433},
	{SG22475, &UA2843196},
	{SG22476, &UA2857335},
	{SG22481, &UA2890462},
	{SG22487, &UA2991667},
	{SG22490, &UA2996460},
	{SG22493, &UA2998563},
	{SG22495, &UA2999381},
	{SG22496, &UA3012207},
	{SG22499, &UA3012797},
	{SG22501, &UA3021673},
	{SG22505, &UA3074223},
	{SG22508, &UA3078386},
	{SG22510, &UA3108765},
	{SG22512, &UA3110743},
	{SG22514, &UA3113542},
	{SG22525, &UA2028334},
	{SG22529, &UA2469980},
	{SG22532, &UA2695622},
	{SG22533, &UA2699234},
	{SG22534, &UA2703401},
	{SG22535, &UA2780100},
	{SG22537, &UA2837831},
	{SG22539, &UA2848787},
	{SG22541, &UA2864759},
	{SG22544, &UA2950854},
	{SG22546, &UA2951326},
	{SG22548, &UA2956311},
	{SG22550, &UA2982810},
	{SG22551, &UA2984189},
	{SG22552, &UA2990106},
	{SG22553, &UA2997607},
	{SG22554, &UA3008708},
	{SG22555, &UA3048176},
	{SG22560, &UA3050151},
	{SG22562, &UA3050376},
	{SG22563, &UA3056529},
	{SG22566, &UA3088466},
	{SG22568, &UA3099657},
	{SG22570, &UA3102278},
	{SG22571, &UA3102882},
	{SG22573, &UA3104387},
	{SG22574, &UA3104534},
	{SG22580, &UA3125984},
	{SG22593, &UA2027138},
	{SG22598, &UA2366818},
	{SG22599, &UA2383526},
	{SG22600, &UA2481569},
	{SG22602, &UA2644097},
	{SG22604, &UA2744111},
	{SG22605, &UA2767144},
	{SG22608, &UA2835449},
	{SG22612, &UA2844061},
	{SG22615, &UA2850643},
	{SG22616, &UA2853400},
	{SG22617, &UA2855749},
	{SG22619, &UA2858746},
	{SG22621, &UA2860562},
	{SG22624, &UA2861919},
	{SG22627, &UA2862737},
	{SG22629, &UA2866944},
	{SG22630, &UA2939572},
	{SG22631, &UA2956769},
	{SG22635, &UA2981420},
	{SG22638, &UA2982061},
	{SG22639, &UA2989072},
	{SG22653, &UA2992997},
	{SG22655, &UA2997704},
	{SG22657, &UA3043587},
	{SG22658, &UA3044571},
	{SG22659, &UA3046063},
	{SG22660, &UA3048591},
	{SG22662, &UA3050294},
	{SG22663, &UA3056564},
	{SG22665, &UA3074379},
	{SG22667, &UA3080518},
	{SG22669, &UA3103965},
	{SG22670, &UA3104465},
	{SG22671, &UA3107797},
	{SG22681, &UA2027802},
	{SG22682, &UA2033370},
	{SG22684, &UA2364870},
	{SG22685, &UA2450601},
	{SG22688, &UA2644360},
	{SG22690, &UA2777930},
	{SG22692, &UA2842946},
	{SG22693, &UA2843477},
	{SG22694, &UA2848554},
	{SG22695, &UA2853296},
	{SG22697, &UA2857345},
	{SG22698, &UA2858493},
	{SG22700, &UA2951226},
	{SG22701, &UA2952167},
	{SG22702, &UA2980715},
	{SG22703, &UA2982171},
	{SG22704, &UA2986326},
	{SG22705, &UA2993313},
	{SG22709, &UA2999812},
	{SG22712, &UA3045548},
	{SG22713, &UA3046352},
	{SG22714, &UA3046403},
	{SG22716, &UA3058840},
	{SG22717, &UA3073454},
	{SG22719, &UA3087173},
	{SG22720, &UA3087191},
	{SG22721, &UA3092186},
	{SG22723, &UA3092240},
	{SG22726, &UA3099246},
	{SG22731, &UA3100737},
	{SG22732, &UA3102581},
	{SG22733, &UA3106864},
	{SG22736, &UA3117568},
	{SG22737, &UA3119656},
	{SG22738, &UA3121141},
	{SG22741, &UA3122237},
	{SG22744, &UA3125494},
	{SG22745, &UA3125718},
	{SG22747, &UA3126931},
	{SG22748, &UA3127111},
	{SG22753, &UA2520909},
	{SG22762, &UA2775902},
	{SG22764, &UA2839083},
	{SG22768, &UA2869059},
	{SG22780, &UA2032830},
	{SG22782, &UA2033497},
	{SG22783, &UA2445532},
	{SG22788, &UA2651656},
	{SG22789, &UA2827748},
	{SG22790, &UA2836240},
	{SG22802, &UA2446279},
	{SG22815, &UA2339931},
	{SG22816, &UA2340148},
	{SG22822, &UA2831174},
	{SG22824, &UA2630764},
	{SG22825, &UA2788041},
	{SG22826, &UA2817484},
	{SG22847, &UA2028517},
	{SG22848, &UA899034},
	{SG22849, &UA2028978},
	{SG22851, &UA2627355},
	{SG22858, &UA852585},
	{SG22859, &UA2026579},
	{SG22860, &UA2035262},
	{SG22861, &UA2035266},
	{SG22863, &UA2630737},
	{SG22880, &UA1976124},
	{SG22884, &UA2618097},
	{SG22886, &UA2684699},
	{SG22889, &UA459277},
	{SG22892, &UA2024530},
	{SG22893, &UA2024768},
	{SG22897, &UA3084116},
	{SG22904, &UA605418},
	{SG22909, &UA2026424},
	{SG22918, &UA2021325},
	{SG22919, &UA814612},
	{SG22921, &UA2026231},
	{SG22922, &UA2031837},
	{SG22923, &UA2032729},
	{SG22925, &UA2817724},
	{SG22930, &UA1994994},
	{SG22932, &UA2029485},
	{SG22933, &UA2032734},
	{SG22938, &UA3084043},
	{SG22940, &UA3084046},
	{SG22955, &UA2024927},
	{SG22956, &UA2031584},
	{SG22959, &UA2032948},
	{SG22963, &UA2617288},
	{SG22968, &UA2622849},
	{SG22981, &UA370112},
	{SG22982, &UA459284},
	{SG22984, &UA2029718},
	{SG23007, &UA459303},
	{SG23013, &UA2025077},
	{SG23014, &UA2026314},
	{SG23016, &UA2034221},
	{SG23023, &UA2626252},
	{SG23024, &UA2634738},
	{SG23027, &UA3084120},
	{SG23030, &UA2025170},
	{SG23032, &UA2026393},
	{SG23047, &UA2029265},
	{SG23050, &UA2635680},
	{SG23053, &UA2618458},
	{SG23054, &UA2622403},
	{SG23063, &UA3084066},
	{SG23069, &UA459275},
	{SG23070, &UA459311},
	{SG23074, &UA721116},
	{SG23079, &UA2586586},
	{SG23102, &UA3084096},
	{SG23115, &UA858300},
	{SG23119, &UA2026875},
	{SG23133, &UA2627720},
	{SG23144, &UA2633617},
	{SG23160, &UA3084082},
	{NULL}};
const Segmentdata DAndroidVersion2_3_7[] = {
	{SG12944, &UA249016},
	{SG12951, &UA653622},
	{SG12952, &UA662027},
	{SG12957, &UA1918696},
	{SG12959, &UA1934233},
	{SG12960, &UA2022154},
	{SG13176, &UA369598},
	{SG13193, &UA693547},
	{SG13196, &UA1936846},
	{SG13200, &UA2025911},
	{SG13204, &UA2413001},
	{SG13206, &UA2423881},
	{SG13209, &UA2518095},
	{SG13211, &UA2751420},
	{SG13213, &UA2765870},
	{SG13216, &UA2842895},
	{SG13218, &UA2933683},
	{SG13317, &UA2034317},
	{SG13319, &UA2748125},
	{SG13425, &UA249833},
	{SG13427, &UA354635},
	{SG13430, &UA573527},
	{SG13435, &UA2021581},
	{SG13436, &UA2021991},
	{SG13439, &UA2027704},
	{SG13445, &UA2814045},
	{SG13446, &UA2853451},
	{SG13448, &UA3069739},
	{SG13628, &UA2382876},
	{SG13630, &UA2503507},
	{SG13690, &UA421628},
	{SG13691, &UA2020707},
	{SG13692, &UA2020861},
	{SG13694, &UA2727962},
	{SG13695, &UA2996885},
	{SG13987, &UA891535},
	{SG13989, &UA898411},
	{SG13992, &UA900679},
	{SG13994, &UA902029},
	{SG13997, &UA904153},
	{SG13999, &UA924100},
	{SG14001, &UA1956580},
	{SG14003, &UA1990415},
	{SG14005, &UA1997108},
	{SG14007, &UA2003029},
	{SG14009, &UA2024572},
	{SG14011, &UA2026824},
	{SG14013, &UA2029101},
	{SG14015, &UA2030424},
	{SG14016, &UA2030437},
	{SG14019, &UA2033185},
	{SG14023, &UA2042952},
	{SG14025, &UA2166814},
	{SG14028, &UA2168316},
	{SG14031, &UA2205590},
	{SG14033, &UA2228159},
	{SG14035, &UA2237439},
	{SG14036, &UA2340510},
	{SG14038, &UA2350834},
	{SG14040, &UA2384581},
	{SG14043, &UA2420620},
	{SG14044, &UA2452223},
	{SG14046, &UA2455037},
	{SG14048, &UA2528296},
	{SG14049, &UA2688461},
	{SG14050, &UA2759938},
	{SG14051, &UA2815649},
	{SG14052, &UA2982737},
	{SG14053, &UA2988977},
	{SG14156, &UA249712},
	{SG14160, &UA660474},
	{SG14162, &UA1908066},
	{SG14168, &UA2022666},
	{SG14177, &UA3042234},
	{SG14206, &UA632768},
	{SG14211, &UA2022784},
	{SG14216, &UA3099737},
	{SG14358, &UA2641739},
	{SG14360, &UA2981819},
	{SG14434, &UA644109},
	{SG14435, &UA661873},
	{SG14439, &UA715005},
	{SG14442, &UA746416},
	{SG14445, &UA759306},
	{SG14447, &UA764636},
	{SG14448, &UA781408},
	{SG14451, &UA886289},
	{SG14458, &UA1937802},
	{SG14460, &UA1938279},
	{SG14464, &UA1945499},
	{SG14469, &UA1971073},
	{SG14471, &UA1972117},
	{SG14472, &UA1990464},
	{SG14474, &UA2020697},
	{SG14476, &UA2021909},
	{SG14481, &UA2022834},
	{SG14497, &UA2025483},
	{SG14507, &UA2029123},
	{SG14508, &UA2029209},
	{SG14509, &UA2029606},
	{SG14511, &UA2030294},
	{SG14526, &UA2033655},
	{SG14532, &UA2223536},
	{SG14533, &UA2224922},
	{SG14536, &UA2242321},
	{SG14539, &UA2309973},
	{SG14542, &UA2334581},
	{SG14545, &UA2342357},
	{SG14549, &UA2415269},
	{SG14553, &UA2443074},
	{SG14555, &UA2444647},
	{SG14556, &UA2445289},
	{SG14558, &UA2452634},
	{SG14561, &UA2475630},
	{SG14562, &UA2475699},
	{SG14563, &UA2482044},
	{SG14565, &UA2483242},
	{SG14576, &UA2505065},
	{SG14579, &UA2520292},
	{SG14581, &UA2520301},
	{SG14583, &UA2520651},
	{SG14585, &UA2545756},
	{SG14586, &UA2546405},
	{SG14587, &UA2547750},
	{SG14588, &UA2549253},
	{SG14589, &UA2588555},
	{SG14594, &UA2595408},
	{SG14600, &UA2600232},
	{SG14605, &UA2643003},
	{SG14608, &UA2644562},
	{SG14610, &UA2645667},
	{SG14612, &UA2650904},
	{SG14617, &UA2652732},
	{SG14621, &UA2655074},
	{SG14623, &UA2659087},
	{SG14625, &UA2664504},
	{SG14629, &UA2684705},
	{SG14630, &UA2686642},
	{SG14632, &UA2692351},
	{SG14636, &UA2704956},
	{SG14640, &UA2722360},
	{SG14643, &UA2723057},
	{SG14647, &UA2727180},
	{SG14650, &UA2731544},
	{SG14653, &UA2743531},
	{SG14658, &UA2764405},
	{SG14660, &UA2769645},
	{SG14662, &UA2775812},
	{SG14667, &UA2787783},
	{SG14673, &UA2792773},
	{SG14676, &UA2795928},
	{SG14677, &UA2806565},
	{SG14682, &UA2815729},
	{SG14683, &UA2815954},
	{SG14688, &UA2816554},
	{SG14692, &UA2817035},
	{SG14697, &UA2818173},
	{SG14700, &UA2823590},
	{SG14701, &UA2826245},
	{SG14704, &UA2833596},
	{SG14706, &UA2842941},
	{SG14707, &UA2843408},
	{SG14708, &UA2844902},
	{SG14710, &UA2936991},
	{SG14711, &UA2937257},
	{SG14712, &UA2950504},
	{SG14713, &UA2958593},
	{SG14714, &UA2958914},
	{SG14715, &UA2980367},
	{SG14716, &UA2980693},
	{SG14717, &UA2981099},
	{SG14718, &UA2983204},
	{SG14719, &UA2993631},
	{SG14720, &UA2996012},
	{SG14721, &UA2998267},
	{SG14725, &UA3003203},
	{SG14728, &UA3006116},
	{SG14731, &UA3041633},
	{SG14737, &UA3044590},
	{SG14744, &UA3070216},
	{SG14747, &UA3070630},
	{SG14748, &UA3070691},
	{SG14754, &UA3079363},
	{SG14760, &UA3098329},
	{SG14761, &UA3098466},
	{SG14765, &UA3100127},
	{SG14768, &UA3101054},
	{SG14775, &UA3107665},
	{SG14777, &UA3108074},
	{SG14782, &UA3117265},
	{SG14988, &UA660964},
	{SG14989, &UA2032546},
	{SG14990, &UA2643892},
	{SG15014, &UA2020919},
	{SG15015, &UA2360389},
	{SG15016, &UA2380172},
	{SG15017, &UA2401265},
	{SG15018, &UA2404725},
	{SG15019, &UA2642876},
	{SG15020, &UA2788631},
	{SG15021, &UA2819158},
	{SG15196, &UA2523718},
	{SG15197, &UA2545143},
	{SG15270, &UA248972},
	{SG15271, &UA651029},
	{SG15272, &UA2004801},
	{SG15273, &UA2582129},
	{SG15274, &UA2597720},
	{SG15275, &UA2706588},
	{SG15276, &UA2743601},
	{SG15277, &UA2759291},
	{SG15278, &UA3117506},
	{SG15292, &UA351277},
	{SG15297, &UA2034388},
	{SG15298, &UA2206024},
	{SG15312, &UA370954},
	{SG15349, &UA2445543},
	{SG15382, &UA2607884},
	{SG15383, &UA2608208},
	{SG15384, &UA2608802},
	{SG15385, &UA2610650},
	{SG15386, &UA2613055},
	{SG15387, &UA2620761},
	{SG15388, &UA2622168},
	{SG15395, &UA2625262},
	{SG15396, &UA2627136},
	{SG15397, &UA2637435},
	{SG15444, &UA440504},
	{SG15445, &UA440864},
	{SG15446, &UA603584},
	{SG15447, &UA624787},
	{SG15448, &UA639557},
	{SG15449, &UA664232},
	{SG15450, &UA725916},
	{SG15451, &UA732306},
	{SG15452, &UA849327},
	{SG15453, &UA862608},
	{SG15454, &UA895651},
	{SG15455, &UA900145},
	{SG15456, &UA900175},
	{SG15457, &UA900413},
	{SG15458, &UA900429},
	{SG15459, &UA902589},
	{SG15460, &UA1956912},
	{SG15461, &UA1977958},
	{SG15462, &UA2024516},
	{SG15463, &UA2024571},
	{SG15464, &UA2025182},
	{SG15465, &UA2028933},
	{SG15466, &UA2032691},
	{SG15468, &UA2032990},
	{SG15470, &UA2033464},
	{SG15472, &UA2811559},
	{SG15766, &UA1938272},
	{SG15767, &UA1963421},
	{SG15768, &UA2025462},
	{SG15769, &UA2317331},
	{SG15770, &UA2364779},
	{SG15771, &UA2520583},
	{SG15772, &UA2936996},
	{SG15773, &UA2957686},
	{SG15774, &UA2980478},
	{SG15779, &UA249909},
	{SG15780, &UA887657},
	{SG15781, &UA1903059},
	{SG15782, &UA2019777},
	{SG15793, &UA924164},
	{SG15794, &UA1996395},
	{SG15795, &UA2000851},
	{SG15796, &UA2030498},
	{SG15797, &UA2185596},
	{SG15798, &UA2257173},
	{SG15799, &UA2342982},
	{SG15800, &UA2384890},
	{SG15801, &UA2404213},
	{SG15802, &UA2594910},
	{SG15803, &UA2645308},
	{SG15804, &UA2982176},
	{SG15805, &UA3117991},
	{SG15806, &UA249245},
	{SG15807, &UA576353},
	{SG15808, &UA248856},
	{SG15809, &UA249838},
	{SG15810, &UA250025},
	{SG15811, &UA255813},
	{SG15812, &UA344515},
	{SG15813, &UA362718},
	{SG15814, &UA368837},
	{SG15816, &UA370757},
	{SG15818, &UA370883},
	{SG15819, &UA397201},
	{SG15823, &UA622471},
	{SG15826, &UA682605},
	{SG15829, &UA725303},
	{SG15830, &UA750944},
	{SG15831, &UA764063},
	{SG15832, &UA789634},
	{SG15833, &UA2020832},
	{SG15834, &UA2022136},
	{SG15836, &UA2030443},
	{SG15838, &UA2032909},
	{SG15839, &UA2336154},
	{SG15841, &UA2394120},
	{SG15842, &UA2505734},
	{SG15844, &UA2683026},
	{SG15845, &UA2775249},
	{SG15849, &UA3041737},
	{SG15851, &UA3087381},
	{SG15874, &UA709956},
	{SG15875, &UA873074},
	{SG15876, &UA1905408},
	{SG15877, &UA1965406},
	{SG15878, &UA2021597},
	{SG15879, &UA2022817},
	{SG15880, &UA2028087},
	{SG15881, &UA2034329},
	{SG15888, &UA2024433},
	{SG15889, &UA2025874},
	{SG15916, &UA715216},
	{SG15918, &UA765541},
	{SG15921, &UA849252},
	{SG15922, &UA2021531},
	{SG15925, &UA2868557},
	{SG15954, &UA459178},
	{SG15981, &UA2033723},
	{SG16113, &UA628298},
	{SG16115, &UA631050},
	{SG16117, &UA649823},
	{SG16119, &UA665245},
	{SG16121, &UA742103},
	{SG16123, &UA768122},
	{SG16125, &UA831195},
	{SG16126, &UA908073},
	{SG16127, &UA1958815},
	{SG16128, &UA1964904},
	{SG16129, &UA1991178},
	{SG16130, &UA1994361},
	{SG16131, &UA1994821},
	{SG16132, &UA1994989},
	{SG16133, &UA1995404},
	{SG16134, &UA2023904},
	{SG16135, &UA2024740},
	{SG16136, &UA2025254},
	{SG16137, &UA2027589},
	{SG16138, &UA2030910},
	{SG16139, &UA2031804},
	{SG16140, &UA2034184},
	{SG16141, &UA2356353},
	{SG16142, &UA2372572},
	{SG16143, &UA2395967},
	{SG16144, &UA2480738},
	{SG16145, &UA2510695},
	{SG16146, &UA2589496},
	{SG16147, &UA2644256},
	{SG16148, &UA2709016},
	{SG16149, &UA2810529},
	{SG16153, &UA3117453},
	{SG16173, &UA677472},
	{SG16175, &UA248859},
	{SG16176, &UA252013},
	{SG16177, &UA369175},
	{SG16178, &UA658722},
	{SG16179, &UA670981},
	{SG16180, &UA2025140},
	{SG16261, &UA250375},
	{SG16264, &UA626546},
	{SG16266, &UA697428},
	{SG16270, &UA1950845},
	{SG16272, &UA2022440},
	{SG16276, &UA2849702},
	{SG16321, &UA2031488},
	{SG16322, &UA2590167},
	{SG16323, &UA2642048},
	{SG16324, &UA3041454},
	{SG16338, &UA251011},
	{SG16339, &UA578485},
	{SG16340, &UA685378},
	{SG16341, &UA2024034},
	{SG16342, &UA2692780},
	{SG16352, &UA710412},
	{SG16353, &UA899381},
	{SG16354, &UA1966106},
	{SG16355, &UA1995115},
	{SG16356, &UA2007381},
	{SG16357, &UA2024984},
	{SG16359, &UA2030311},
	{SG16360, &UA2030339},
	{SG16361, &UA2030824},
	{SG16362, &UA2172778},
	{SG16363, &UA2257193},
	{SG16364, &UA2480290},
	{SG16365, &UA2519923},
	{SG16366, &UA2648208},
	{SG16413, &UA250528},
	{SG16415, &UA2022456},
	{SG16418, &UA369241},
	{SG16419, &UA902854},
	{SG16420, &UA905080},
	{SG16421, &UA1950977},
	{SG16422, &UA1959852},
	{SG16423, &UA1961391},
	{SG16424, &UA1982662},
	{SG16425, &UA1987386},
	{SG16426, &UA1991913},
	{SG16427, &UA2004688},
	{SG16428, &UA2007380},
	{SG16429, &UA2009137},
	{SG16430, &UA2016153},
	{SG16431, &UA2029623},
	{SG16432, &UA2033150},
	{SG16433, &UA2187374},
	{SG16434, &UA2198337},
	{SG16435, &UA2223389},
	{SG16436, &UA2349124},
	{SG16437, &UA2413871},
	{SG16438, &UA2686979},
	{SG16480, &UA585059},
	{SG16482, &UA622447},
	{SG16485, &UA342039},
	{SG16486, &UA662720},
	{SG16487, &UA771806},
	{SG16488, &UA849011},
	{SG16489, &UA2022504},
	{SG16490, &UA2033704},
	{SG16551, &UA754665},
	{SG16552, &UA2021471},
	{SG16557, &UA249911},
	{SG16566, &UA2022642},
	{SG16574, &UA1986541},
	{SG16578, &UA2985162},
	{SG16627, &UA1997510},
	{SG16628, &UA2242278},
	{SG16697, &UA369311},
	{SG16718, &UA248913},
	{SG16721, &UA2032727},
	{SG16747, &UA249840},
	{SG16750, &UA250070},
	{SG16752, &UA342442},
	{SG16756, &UA576456},
	{SG16760, &UA2021789},
	{SG16762, &UA2024583},
	{SG16763, &UA249270},
	{SG16764, &UA250723},
	{SG16765, &UA646517},
	{SG16766, &UA1940240},
	{SG16767, &UA1996790},
	{SG16768, &UA2411758},
	{SG16772, &UA2022206},
	{SG16799, &UA2023220},
	{SG16800, &UA2577034},
	{SG16803, &UA2811199},
	{SG16815, &UA250883},
	{SG16828, &UA2021343},
	{SG16830, &UA379587},
	{SG16831, &UA1926258},
	{SG16881, &UA889783},
	{SG16883, &UA2031038},
	{SG16884, &UA2225669},
	{SG16886, &UA2693873},
	{SG16888, &UA2696358},
	{SG16890, &UA3124833},
	{SG16943, &UA650310},
	{SG16945, &UA680725},
	{SG16947, &UA686758},
	{SG16951, &UA699136},
	{SG16953, &UA711421},
	{SG16955, &UA723080},
	{SG16957, &UA769533},
	{SG16959, &UA775194},
	{SG16960, &UA830312},
	{SG16961, &UA2022992},
	{SG16962, &UA2024727},
	{SG16963, &UA2032522},
	{SG16964, &UA2222005},
	{SG16965, &UA2475853},
	{SG16966, &UA2496724},
	{SG16967, &UA2686223},
	{SG16968, &UA2811103},
	{SG16969, &UA2006108},
	{SG16970, &UA2007370},
	{SG16971, &UA2156425},
	{SG16972, &UA2389501},
	{SG16973, &UA2429876},
	{SG16974, &UA2483797},
	{SG16975, &UA2489158},
	{SG16976, &UA2561597},
	{SG16994, &UA2999983},
	{SG16995, &UA2021335},
	{SG16996, &UA2551629},
	{SG16997, &UA768014},
	{SG16998, &UA652658},
	{SG16999, &UA1997083},
	{SG17000, &UA1997135},
	{SG17001, &UA2555375},
	{SG17002, &UA2594660},
	{SG17003, &UA2726392},
	{SG17004, &UA2779710},
	{SG17005, &UA2841350},
	{SG17006, &UA2982890},
	{SG17053, &UA2958441},
	{SG17059, &UA1996094},
	{SG17060, &UA2236031},
	{SG17061, &UA2415492},
	{SG17062, &UA2450928},
	{SG17063, &UA2453651},
	{SG17064, &UA2465974},
	{SG17065, &UA2478949},
	{SG17066, &UA2500937},
	{SG17068, &UA1938722},
	{SG17069, &UA2023384},
	{SG17077, &UA248864},
	{SG17078, &UA1933840},
	{SG17079, &UA840072},
	{SG17080, &UA877313},
	{SG17081, &UA899930},
	{SG17082, &UA1971282},
	{SG17083, &UA1981934},
	{SG17084, &UA1993874},
	{SG17085, &UA1994183},
	{SG17086, &UA2028094},
	{SG17087, &UA2029446},
	{SG17088, &UA2032261},
	{SG17089, &UA2496697},
	{SG17090, &UA809935},
	{SG17091, &UA2021363},
	{SG17092, &UA2024123},
	{SG17101, &UA250023},
	{SG17102, &UA660788},
	{SG17103, &UA647887},
	{SG17104, &UA760382},
	{SG17105, &UA784168},
	{SG17106, &UA872638},
	{SG17107, &UA907991},
	{SG17108, &UA908380},
	{SG17109, &UA2026783},
	{SG17110, &UA2685741},
	{SG17111, &UA249889},
	{SG17112, &UA2020380},
	{SG17113, &UA2023768},
	{SG17114, &UA2027821},
	{SG17115, &UA2032324},
	{SG17158, &UA250796},
	{SG17159, &UA622389},
	{SG17160, &UA685549},
	{SG17161, &UA697549},
	{SG17162, &UA700884},
	{SG17163, &UA1982306},
	{SG17164, &UA2022782},
	{SG17165, &UA2223599},
	{SG17166, &UA2503172},
	{SG17167, &UA2722493},
	{SG17209, &UA809591},
	{SG17211, &UA2257183},
	{SG17212, &UA2403155},
	{SG17213, &UA2939580},
	{SG17237, &UA250007},
	{SG17238, &UA1901353},
	{SG17288, &UA249020},
	{SG17330, &UA2030640},
	{SG17404, &UA452972},
	{SG17405, &UA763042},
	{SG17406, &UA1943340},
	{SG17407, &UA2027784},
	{SG17408, &UA2030203},
	{SG17518, &UA250032},
	{SG17519, &UA252233},
	{SG17520, &UA692125},
	{SG17521, &UA727261},
	{SG17522, &UA1946218},
	{SG17523, &UA1990293},
	{SG17524, &UA2021947},
	{SG17525, &UA2023127},
	{SG17526, &UA2023873},
	{SG17527, &UA2034043},
	{SG17528, &UA2404378},
	{SG17529, &UA2982718},
	{SG17552, &UA1941562},
	{SG17553, &UA2028936},
	{SG17570, &UA720042},
	{SG17571, &UA2027694},
	{SG17572, &UA2383416},
	{SG17573, &UA2476752},
	{SG17726, &UA613254},
	{SG17728, &UA1988933},
	{SG17729, &UA2031507},
	{SG17730, &UA2815092},
	{SG17818, &UA349267},
	{SG17820, &UA422370},
	{SG17821, &UA2030532},
	{SG17823, &UA2589728},
	{SG17858, &UA2402612},
	{SG17881, &UA628883},
	{SG17883, &UA2034533},
	{SG17897, &UA428373},
	{SG17898, &UA2816129},
	{SG17907, &UA575688},
	{SG17909, &UA714035},
	{SG17910, &UA735665},
	{SG17911, &UA816874},
	{SG17913, &UA1907383},
	{SG17915, &UA1927998},
	{SG17917, &UA1938464},
	{SG17918, &UA1994253},
	{SG17919, &UA2020449},
	{SG17920, &UA2021497},
	{SG17921, &UA2025734},
	{SG17922, &UA2027752},
	{SG17923, &UA2028773},
	{SG17924, &UA2034633},
	{SG17925, &UA2236452},
	{SG17926, &UA2412704},
	{SG17927, &UA2450336},
	{SG17928, &UA2453086},
	{SG17929, &UA2594442},
	{SG17931, &UA2729460},
	{SG17933, &UA2815881},
	{SG17934, &UA2819554},
	{SG17948, &UA250625},
	{SG17949, &UA342484},
	{SG17950, &UA354023},
	{SG17951, &UA622370},
	{SG17952, &UA630944},
	{SG17958, &UA2996534},
	{SG17964, &UA2383923},
	{SG17965, &UA2666000},
	{SG17966, &UA2983027},
	{SG17969, &UA252331},
	{SG17971, &UA380081},
	{SG17973, &UA699062},
	{SG17975, &UA736665},
	{SG17976, &UA858950},
	{SG17977, &UA2020365},
	{SG17978, &UA2021131},
	{SG17979, &UA3117100},
	{SG17980, &UA1960923},
	{SG17981, &UA1996372},
	{SG17982, &UA2025437},
	{SG17983, &UA2028553},
	{SG17984, &UA2029341},
	{SG17985, &UA2030813},
	{SG17986, &UA2031928},
	{SG17987, &UA2033289},
	{SG17988, &UA2228128},
	{SG17989, &UA2464467},
	{SG17990, &UA2484376},
	{SG17991, &UA2497646},
	{SG17992, &UA2520587},
	{SG17993, &UA2787179},
	{SG17994, &UA2820482},
	{SG17995, &UA2848337},
	{SG17996, &UA3117546},
	{SG17998, &UA2262384},
	{SG18005, &UA856560},
	{SG18006, &UA2031458},
	{SG18007, &UA2454550},
	{SG18044, &UA457528},
	{SG18045, &UA627144},
	{SG18046, &UA628251},
	{SG18047, &UA665931},
	{SG18048, &UA862995},
	{SG18049, &UA869495},
	{SG18051, &UA2374541},
	{SG18052, &UA2405771},
	{SG18053, &UA2655108},
	{SG18073, &UA660556},
	{SG18074, &UA910778},
	{SG18075, &UA1985960},
	{SG18081, &UA252084},
	{SG18082, &UA575349},
	{SG18083, &UA657822},
	{SG18084, &UA1938719},
	{SG18085, &UA1981405},
	{SG18086, &UA2021448},
	{SG18087, &UA2021894},
	{SG18111, &UA252033},
	{SG18112, &UA421787},
	{SG18113, &UA695460},
	{SG18114, &UA711296},
	{SG18115, &UA1930610},
	{SG18117, &UA1971792},
	{SG18118, &UA2024208},
	{SG18159, &UA716652},
	{SG18167, &UA252108},
	{SG18168, &UA782594},
	{SG18169, &UA872805},
	{SG18170, &UA2006258},
	{SG18175, &UA632069},
	{SG18313, &UA623027},
	{SG18314, &UA641062},
	{SG18315, &UA2024694},
	{SG18343, &UA249704},
	{SG18344, &UA573891},
	{SG18345, &UA705771},
	{SG18346, &UA2026640},
	{SG18358, &UA245164},
	{SG18364, &UA249366},
	{SG18366, &UA250053},
	{SG18367, &UA397793},
	{SG18368, &UA638333},
	{SG18379, &UA2423087},
	{SG18402, &UA249752},
	{SG18405, &UA250715},
	{SG18408, &UA663476},
	{SG18410, &UA677593},
	{SG18413, &UA2555855},
	{SG18427, &UA251924},
	{SG18428, &UA680202},
	{SG18429, &UA1963019},
	{SG18432, &UA632406},
	{SG18433, &UA658387},
	{SG18434, &UA680479},
	{SG18435, &UA2021876},
	{SG18436, &UA2658888},
	{SG18437, &UA776132},
	{SG18438, &UA2030202},
	{SG18439, &UA2647786},
	{SG18480, &UA899130},
	{SG18481, &UA900832},
	{SG18482, &UA908293},
	{SG18483, &UA2033831},
	{SG18484, &UA2337433},
	{SG18485, &UA2366550},
	{SG18486, &UA2527101},
	{SG18487, &UA2808759},
	{SG18529, &UA255859},
	{SG18530, &UA364267},
	{SG18567, &UA2019833},
	{SG18568, &UA2022393},
	{SG18594, &UA694678},
	{SG18595, &UA2843109},
	{SG18643, &UA2021900},
	{SG18644, &UA2023449},
	{SG18645, &UA2812593},
	{SG18715, &UA1961620},
	{SG18735, &UA635923},
	{SG18736, &UA2475135},
	{SG18753, &UA629182},
	{SG18754, &UA2024430},
	{SG18755, &UA2166395},
	{SG18776, &UA697071},
	{SG18854, &UA680043},
	{SG18860, &UA672450},
	{SG18861, &UA2032040},
	{SG18875, &UA575334},
	{SG18876, &UA649563},
	{SG18877, &UA669600},
	{SG18878, &UA1919923},
	{SG18879, &UA1965613},
	{SG18880, &UA2027769},
	{SG18881, &UA2813230},
	{SG18882, &UA2816658},
	{SG18897, &UA702651},
	{SG18898, &UA712411},
	{SG18908, &UA2032936},
	{SG18919, &UA2356618},
	{SG18932, &UA455972},
	{SG18933, &UA599917},
	{SG18934, &UA599975},
	{SG18935, &UA603593},
	{SG18936, &UA679145},
	{SG18937, &UA862471},
	{SG18938, &UA885516},
	{SG18939, &UA1997677},
	{SG18940, &UA2029937},
	{SG18941, &UA2811251},
	{SG18963, &UA369860},
	{SG18964, &UA627036},
	{SG18965, &UA696614},
	{SG18967, &UA686949},
	{SG18968, &UA1960684},
	{SG18995, &UA251355},
	{SG18997, &UA731461},
	{SG18998, &UA2021769},
	{SG18999, &UA2022116},
	{SG19022, &UA251939},
	{SG19023, &UA859857},
	{SG19024, &UA1906127},
	{SG19025, &UA2020387},
	{SG19027, &UA2034299},
	{SG19123, &UA368715},
	{SG19125, &UA627448},
	{SG19127, &UA1958149},
	{SG19129, &UA2029917},
	{SG19150, &UA369703},
	{SG19151, &UA623275},
	{SG19152, &UA757859},
	{SG19153, &UA2025212},
	{SG19154, &UA2033586},
	{SG19165, &UA2787796},
	{SG19178, &UA342450},
	{SG19179, &UA421732},
	{SG19180, &UA659818},
	{SG19182, &UA751613},
	{SG19183, &UA829253},
	{SG19184, &UA1973252},
	{SG19185, &UA2021700},
	{SG19220, &UA454935},
	{SG19221, &UA628934},
	{SG19222, &UA674641},
	{SG19223, &UA901453},
	{SG19224, &UA2765102},
	{SG19225, &UA2952420},
	{SG19246, &UA859190},
	{SG19249, &UA1948306},
	{SG19251, &UA1960652},
	{SG19253, &UA1978571},
	{SG19255, &UA2022125},
	{SG19256, &UA2025355},
	{SG19257, &UA2027244},
	{SG19258, &UA2027878},
	{SG19259, &UA2033986},
	{SG19260, &UA2443362},
	{SG19261, &UA2477060},
	{SG19263, &UA2549692},
	{SG19264, &UA2641122},
	{SG19266, &UA2788249},
	{SG19267, &UA2790366},
	{SG19268, &UA2809472},
	{SG19269, &UA2857448},
	{SG19270, &UA3097923},
	{SG19271, &UA3099338},
	{SG19272, &UA251950},
	{SG19273, &UA369612},
	{SG19274, &UA637919},
	{SG19275, &UA2023298},
	{SG19276, &UA2026081},
	{SG19279, &UA251052},
	{SG19280, &UA363805},
	{SG19281, &UA677621},
	{SG19282, &UA1903943},
	{SG19291, &UA728733},
	{SG19413, &UA573929},
	{SG19424, &UA2793078},
	{SG19428, &UA669133},
	{SG19446, &UA2658105},
	{SG19449, &UA368856},
	{SG19487, &UA661348},
	{SG19488, &UA833127},
	{SG19489, &UA2463886},
	{SG19498, &UA358437},
	{SG19499, &UA1950158},
	{SG19500, &UA2982227},
	{SG19513, &UA722273},
	{SG19515, &UA2021837},
	{SG19516, &UA2810077},
	{SG19517, &UA379932},
	{SG19518, &UA2023582},
	{SG19545, &UA251932},
	{SG19546, &UA2021297},
	{SG19547, &UA2032258},
	{SG19557, &UA251934},
	{SG19579, &UA867457},
	{SG19580, &UA2028600},
	{SG19581, &UA2033028},
	{SG19582, &UA2399957},
	{SG19583, &UA2983146},
	{SG19632, &UA249305},
	{SG19633, &UA574665},
	{SG19634, &UA721939},
	{SG19636, &UA2021512},
	{SG19647, &UA3042419},
	{SG19662, &UA2171381},
	{SG19664, &UA2446201},
	{SG19666, &UA2592862},
	{SG19667, &UA2759769},
	{SG19670, &UA2467766},
	{SG19671, &UA2659639},
	{SG19769, &UA591056},
	{SG19770, &UA2026259},
	{SG19773, &UA3121389},
	{SG19782, &UA2544163},
	{SG19805, &UA2816015},
	{SG19806, &UA696440},
	{SG19811, &UA629204},
	{SG19813, &UA764517},
	{SG19814, &UA1952534},
	{SG19815, &UA2024181},
	{SG19816, &UA2029880},
	{SG19817, &UA2334698},
	{SG19818, &UA2506642},
	{SG19819, &UA2775449},
	{SG19856, &UA2934825},
	{SG19861, &UA705451},
	{SG19864, &UA2023423},
	{SG19871, &UA250124},
	{SG19894, &UA369473},
	{SG19895, &UA2034337},
	{SG19904, &UA651939},
	{SG19913, &UA702504},
	{SG19914, &UA2020085},
	{SG19915, &UA2020171},
	{SG19917, &UA2022948},
	{SG19919, &UA2031785},
	{SG19920, &UA831057},
	{SG19921, &UA2024923},
	{SG19922, &UA2652160},
	{SG19923, &UA3098151},
	{SG19926, &UA1984789},
	{SG19927, &UA2024025},
	{SG19929, &UA2025103},
	{SG19930, &UA2032440},
	{SG19931, &UA2032978},
	{SG19932, &UA2574141},
	{SG19966, &UA843950},
	{SG19981, &UA774855},
	{SG19986, &UA1955242},
	{SG19987, &UA251931},
	{SG20001, &UA756101},
	{SG20017, &UA723563},
	{SG20018, &UA2023282},
	{SG20019, &UA2029868},
	{SG20020, &UA2317174},
	{SG20021, &UA2528387},
	{SG20027, &UA251913},
	{SG20029, &UA1939895},
	{SG20030, &UA2021558},
	{SG20065, &UA437138},
	{SG20066, &UA1940029},
	{SG20067, &UA2021178},
	{SG20068, &UA2519099},
	{SG20087, &UA725430},
	{SG20097, &UA706831},
	{SG20100, &UA1994004},
	{SG20101, &UA2023902},
	{SG20102, &UA2835615},
	{SG20122, &UA626371},
	{SG20124, &UA1942615},
	{SG20125, &UA2022368},
	{SG20126, &UA2495319},
	{SG20128, &UA3042098},
	{SG20167, &UA633680},
	{SG20168, &UA2474330},
	{SG20169, &UA2686528},
	{SG20170, &UA575018},
	{SG20171, &UA642881},
	{SG20174, &UA2719030},
	{SG20175, &UA2784564},
	{SG20176, &UA2956610},
	{SG20188, &UA252555},
	{SG20189, &UA369020},
	{SG20190, &UA2022153},
	{SG20191, &UA2588980},
	{SG20196, &UA754502},
	{SG20218, &UA2849749},
	{SG20229, &UA684225},
	{SG20230, &UA751806},
	{SG20249, &UA251927},
	{SG20250, &UA2025557},
	{SG20251, &UA2026020},
	{SG20271, &UA3061095},
	{SG20279, &UA625560},
	{SG20280, &UA2028727},
	{SG20290, &UA423489},
	{SG20292, &UA604844},
	{SG20294, &UA605522},
	{SG20296, &UA659512},
	{SG20297, &UA864571},
	{SG20302, &UA886131},
	{SG20303, &UA1966365},
	{SG20304, &UA2390952},
	{SG20305, &UA2727833},
	{SG20309, &UA866977},
	{SG20310, &UA2026790},
	{SG20311, &UA2028448},
	{SG20312, &UA2374254},
	{SG20314, &UA2381754},
	{SG20315, &UA2453701},
	{SG20316, &UA2742986},
	{SG20317, &UA2812353},
	{SG20362, &UA3099625},
	{SG20363, &UA3107529},
	{SG20365, &UA889924},
	{SG20394, &UA663378},
	{SG20403, &UA692020},
	{SG20455, &UA800002},
	{SG20461, &UA630408},
	{SG20462, &UA826749},
	{SG20492, &UA829368},
	{SG20494, &UA2029721},
	{SG20498, &UA765138},
	{SG20499, &UA1995922},
	{SG20500, &UA2033345},
	{SG20518, &UA1952319},
	{SG20519, &UA2816269},
	{SG20543, &UA2020768},
	{SG20550, &UA250867},
	{SG20551, &UA2025608},
	{SG20555, &UA2511713},
	{SG20558, &UA811654},
	{SG20586, &UA2958032},
	{SG20589, &UA862921},
	{SG20590, &UA1990541},
	{SG20591, &UA574692},
	{SG20592, &UA2019767},
	{SG20593, &UA2023742},
	{SG20594, &UA2027291},
	{SG20595, &UA654990},
	{SG20596, &UA2659208},
	{SG20611, &UA709496},
	{SG20612, &UA2715173},
	{SG20613, &UA2936653},
	{SG20614, &UA649922},
	{SG20615, &UA737011},
	{SG20616, &UA846352},
	{SG20617, &UA2020088},
	{SG20620, &UA1934000},
	{SG20621, &UA1974460},
	{SG20623, &UA368942},
	{SG20625, &UA735221},
	{SG20626, &UA858069},
	{SG20641, &UA845131},
	{SG20656, &UA857021},
	{SG20658, &UA2745988},
	{SG20668, &UA3017984},
	{SG20672, &UA738111},
	{SG20673, &UA1964861},
	{SG20683, &UA3026354},
	{SG20686, &UA3062252},
	{SG20687, &UA249252},
	{SG20689, &UA2021832},
	{SG20692, &UA2223198},
	{SG20693, &UA2664665},
	{SG20694, &UA2669685},
	{SG20695, &UA2752019},
	{SG20696, &UA2799910},
	{SG20700, &UA345458},
	{SG20701, &UA365000},
	{SG20713, &UA2413353},
	{SG20715, &UA2553320},
	{SG20723, &UA370694},
	{SG20725, &UA379924},
	{SG20727, &UA2878929},
	{SG20752, &UA369749},
	{SG20776, &UA812257},
	{SG20777, &UA764187},
	{SG20780, &UA682336},
	{SG20781, &UA725347},
	{SG20784, &UA806205},
	{SG20785, &UA2026225},
	{SG20795, &UA739831},
	{SG20796, &UA1902545},
	{SG20797, &UA1978628},
	{SG20798, &UA2025446},
	{SG20799, &UA2029264},
	{SG20800, &UA2029296},
	{SG20810, &UA689281},
	{SG20811, &UA2027234},
	{SG20867, &UA342440},
	{SG20869, &UA788887},
	{SG20870, &UA2021007},
	{SG20883, &UA368667},
	{SG20884, &UA661921},
	{SG20886, &UA2034624},
	{SG20891, &UA2257209},
	{SG20892, &UA2475652},
	{SG20906, &UA2235095},
	{SG20916, &UA1957708},
	{SG20917, &UA2818010},
	{SG20920, &UA2029974},
	{SG20921, &UA2033183},
	{SG20922, &UA2396353},
	{SG20944, &UA1970266},
	{SG20945, &UA2461330},
	{SG20950, &UA576546},
	{SG20951, &UA2021890},
	{SG20952, &UA2024039},
	{SG20953, &UA2031684},
	{SG20957, &UA652564},
	{SG20958, &UA693665},
	{SG20959, &UA2025138},
	{SG20960, &UA252043},
	{SG20962, &UA2023146},
	{SG20963, &UA639583},
	{SG20964, &UA836098},
	{SG20980, &UA2033749},
	{SG21029, &UA1972991},
	{SG21040, &UA576587},
	{SG21041, &UA778065},
	{SG21042, &UA1937829},
	{SG21058, &UA2944085},
	{SG21059, &UA2944845},
	{SG21071, &UA827421},
	{SG21072, &UA2030020},
	{SG21082, &UA574310},
	{SG21100, &UA1945856},
	{SG21115, &UA763495},
	{SG21116, &UA811543},
	{SG21117, &UA2025743},
	{SG21118, &UA2612780},
	{SG21119, &UA2624637},
	{SG21175, &UA880126},
	{SG21197, &UA574165},
	{SG21198, &UA2026904},
	{SG21200, &UA2644038},
	{SG21203, &UA2754278},
	{SG21213, &UA2024295},
	{SG21214, &UA2034032},
	{SG21221, &UA370651},
	{SG21222, &UA2608161},
	{SG21227, &UA861565},
	{SG21228, &UA2695826},
	{SG21233, &UA654435},
	{SG21254, &UA2034594},
	{SG21278, &UA2027021},
	{SG21279, &UA2337332},
	{SG21280, &UA2743557},
	{SG21286, &UA691250},
	{SG21329, &UA2028236},
	{SG21335, &UA2023753},
	{SG21336, &UA2025649},
	{SG21337, &UA2442095},
	{SG21343, &UA746626},
	{SG21344, &UA2816988},
	{SG21345, &UA774512},
	{SG21349, &UA730786},
	{SG21357, &UA576621},
	{SG21358, &UA575798},
	{SG21361, &UA2788699},
	{SG21371, &UA2783924},
	{SG21409, &UA2023067},
	{SG21417, &UA689835},
	{SG21418, &UA3071716},
	{SG21422, &UA2620985},
	{SG21455, &UA2023250},
	{SG21474, &UA2626154},
	{SG21478, &UA1985120},
	{SG21525, &UA251533},
	{SG21547, &UA575486},
	{SG21548, &UA1939238},
	{SG21549, &UA2021740},
	{SG21570, &UA2401341},
	{SG21608, &UA2028991},
	{SG21609, &UA860188},
	{SG21612, &UA2983151},
	{SG21660, &UA646276},
	{SG21661, &UA1967201},
	{SG21662, &UA2375235},
	{SG21678, &UA659836},
	{SG21679, &UA1953983},
	{SG21680, &UA2032942},
	{SG21681, &UA2373008},
	{SG21709, &UA865653},
	{SG21723, &UA2737296},
	{SG21742, &UA694887},
	{SG21743, &UA724023},
	{SG21744, &UA2026002},
	{SG21745, &UA2775749},
	{SG21747, &UA2531177},
	{SG21791, &UA3095057},
	{SG21792, &UA3115104},
	{SG21794, &UA379886},
	{SG21795, &UA2022021},
	{SG21799, &UA2023320},
	{SG21800, &UA2031993},
	{SG21807, &UA459167},
	{SG21809, &UA2772390},
	{SG21811, &UA3028931},
	{SG21812, &UA773494},
	{SG21857, &UA2384296},
	{SG21859, &UA2652881},
	{SG21873, &UA369212},
	{SG21874, &UA1973945},
	{SG21875, &UA2025309},
	{SG21890, &UA2799266},
	{SG21915, &UA2469243},
	{SG21916, &UA2863315},
	{SG21923, &UA2032317},
	{SG21929, &UA2684450},
	{SG21930, &UA3044578},
	{SG21931, &UA2945154},
	{SG21946, &UA370658},
	{SG21950, &UA737268},
	{SG21962, &UA2366628},
	{SG21978, &UA1967333},
	{SG21979, &UA1985478},
	{SG21982, &UA2561259},
	{SG21992, &UA2022243},
	{SG22039, &UA2028231},
	{SG22040, &UA2029785},
	{SG22041, &UA2362607},
	{SG22051, &UA2020593},
	{SG22052, &UA2383301},
	{SG22074, &UA729690},
	{SG22075, &UA2394815},
	{SG22087, &UA349011},
	{SG22088, &UA355744},
	{SG22090, &UA806527},
	{SG22091, &UA834596},
	{SG22093, &UA2257096},
	{SG22099, &UA2965432},
	{SG22100, &UA3051490},
	{SG22102, &UA3079607},
	{SG22126, &UA706772},
	{SG22138, &UA576569},
	{SG22143, &UA2364322},
	{SG22145, &UA2466512},
	{SG22156, &UA1904949},
	{SG22169, &UA2796667},
	{SG22170, &UA2996374},
	{SG22183, &UA578482},
	{SG22184, &UA651772},
	{SG22203, &UA634311},
	{SG22204, &UA1964514},
	{SG22205, &UA2023550},
	{SG22240, &UA2423847},
	{SG22263, &UA1913050},
	{SG22282, &UA2789872},
	{SG22294, &UA2413874},
	{SG22320, &UA2760032},
	{SG22321, &UA2807674},
	{SG22340, &UA2787083},
	{SG22341, &UA2700323},
	{SG22342, &UA2701018},
	{SG22351, &UA2867238},
	{SG22360, &UA687185},
	{SG22368, &UA667266},
	{SG22380, &UA2963724},
	{SG22383, &UA346126},
	{SG22385, &UA641525},
	{SG22387, &UA573511},
	{SG22396, &UA2764914},
	{SG22397, &UA2866719},
	{SG22398, &UA2867152},
	{SG22403, &UA2395760},
	{SG22428, &UA2320295},
	{SG22429, &UA2722773},
	{SG22433, &UA2958232},
	{SG22438, &UA1906411},
	{SG22444, &UA2451340},
	{SG22463, &UA1935514},
	{SG22486, &UA1936360},
	{SG22492, &UA1936534},
	{SG22494, &UA2024355},
	{SG22498, &UA2581146},
	{SG22515, &UA2020333},
	{SG22524, &UA2596066},
	{SG22538, &UA2020272},
	{SG22540, &UA723756},
	{SG22549, &UA2545571},
	{SG22643, &UA2893046},
	{SG22664, &UA2790712},
	{SG22673, &UA1987336},
	{SG22715, &UA1976234},
	{SG22746, &UA712495},
	{SG22750, &UA2652048},
	{SG22755, &UA3013616},
	{SG22770, &UA2023609},
	{SG22771, &UA2026118},
	{SG22772, &UA2743744},
	{SG22817, &UA2413415},
	{SG22829, &UA2950141},
	{SG22850, &UA2520922},
	{SG22877, &UA2839923},
	{SG22883, &UA2023183},
	{SG22887, &UA2748020},
	{SG22890, &UA3049459},
	{SG22899, &UA2668246},
	{SG22901, &UA2685009},
	{SG22926, &UA2020359},
	{SG22927, &UA2019874},
	{SG22934, &UA2466198},
	{SG22970, &UA689805},
	{SG22977, &UA2666235},
	{SG22989, &UA3109436},
	{SG22995, &UA2652878},
	{SG23000, &UA2703360},
	{SG23002, &UA2734484},
	{SG23006, &UA2790203},
	{SG23022, &UA3047331},
	{SG23038, &UA357829},
	{SG23040, &UA2024121},
	{SG23045, &UA2544295},
	{SG23057, &UA3027660},
	{SG23082, &UA2957382},
	{SG23092, &UA3110435},
	{SG23101, &UA2027264},
	{SG23107, &UA2957588},
	{SG23110, &UA3000839},
	{SG23116, &UA575400},
	{SG23118, &UA1948120},
	{SG23125, &UA2830525},
	{SG23126, &UA2878303},
	{SG23156, &UA2022981},
	{SG23157, &UA2544405},
	{SG23182, &UA3070793},
	{SG23193, &UA660391},
	{SG23206, &UA2762854},
	{SG23207, &UA2814879},
	{SG23217, &UA3049961},
	{SG23219, &UA3071044},
	{SG23224, &UA655079},
	{SG23227, &UA2026565},
	{SG23232, &UA2520690},
	{SG23239, &UA2843630},
	{SG23240, &UA2844379},
	{SG23243, &UA2851311},
	{SG23244, &UA2853269},
	{SG23246, &UA2997081},
	{SG23249, &UA3104001},
	{SG23262, &UA359104},
	{SG23263, &UA740436},
	{SG23275, &UA3045928},
	{SG23276, &UA3048251},
	{SG23282, &UA3086550},
	{SG23284, &UA3088330},
	{SG23285, &UA3105771},
	{SG23299, &UA2823045},
	{SG23313, &UA2813244},
	{SG23316, &UA421804},
	{SG23318, &UA654634},
	{SG23349, &UA2026897},
	{SG23354, &UA2631115},
	{SG23356, &UA2811195},
	{SG23357, &UA2815769},
	{SG23370, &UA1925109},
	{SG23372, &UA2525936},
	{SG23373, &UA2817841},
	{SG23381, &UA371348},
	{SG23397, &UA2022609},
	{SG23398, &UA2023207},
	{SG23400, &UA2025524},
	{SG23401, &UA2032203},
	{SG23411, &UA370997},
	{SG23414, &UA1907112},
	{SG23437, &UA358710},
	{SG23438, &UA2025171},
	{SG23439, &UA2034406},
	{SG23450, &UA458948},
	{SG23453, &UA2021428},
	{SG23454, &UA2023800},
	{SG23455, &UA2029326},
	{SG23479, &UA2026705},
	{SG23496, &UA359856},
	{SG23507, &UA2032184},
	{SG23508, &UA2032771},
	{SG23549, &UA2022137},
	{SG23550, &UA2025451},
	{SG23594, &UA370408},
	{SG23602, &UA642957},
	{SG23627, &UA2629090},
	{SG23628, &UA2629435},
	{SG23652, &UA369796},
	{SG23653, &UA369881},
	{SG23654, &UA370187},
	{SG23655, &UA458861},
	{SG23661, &UA2027953},
	{SG23670, &UA2607531},
	{SG23679, &UA2617019},
	{SG23715, &UA2626836},
	{SG23735, &UA2629331},
	{SG23787, &UA2636879},
	{NULL}};
const Segmentdata DAndroidVersion2_4[] = {
	{SG12945, &UA459214},
	{SG16298, &UA459201},
	{SG16299, &UA459199},
	{SG16730, &UA459208},
	{SG16731, &UA459217},
	{SG16732, &UA459218},
	{SG16733, &UA459227},
	{SG16734, &UA459237},
	{SG19526, &UA2395923},
	{NULL}};
const Segmentdata DAndroidVersion3[] = {
	{SG15313, &UA33283},
	{SG15314, &UA33386},
	{SG15315, &UA48529},
	{SG15316, &UA48535},
	{SG15317, &UA48543},
	{SG15318, &UA61565},
	{SG15319, &UA68979},
	{SG15320, &UA73731},
	{SG15321, &UA207950},
	{SG15322, &UA396193},
	{SG15323, &UA569299},
	{SG15324, &UA2019872},
	{SG15325, &UA2022237},
	{SG15326, &UA2215833},
	{SG15327, &UA2607598},
	{SG15328, &UA2607973},
	{SG15329, &UA2607995},
	{SG15330, &UA2608574},
	{SG15331, &UA2609429},
	{SG15332, &UA2609723},
	{SG15333, &UA2610169},
	{SG15334, &UA2610235},
	{SG15335, &UA2610907},
	{SG15336, &UA2611029},
	{SG15337, &UA2611482},
	{SG15338, &UA2613895},
	{SG15339, &UA2616207},
	{SG15340, &UA2616257},
	{SG15341, &UA2616393},
	{SG15342, &UA2617357},
	{SG15343, &UA2617748},
	{SG15344, &UA2618058},
	{SG15345, &UA2618482},
	{SG15346, &UA2618535},
	{SG15347, &UA2618797},
	{SG15348, &UA2619019},
	{SG15350, &UA2619026},
	{SG15351, &UA2619114},
	{SG15352, &UA2619428},
	{SG15353, &UA2619445},
	{SG15354, &UA2619502},
	{SG15355, &UA2619526},
	{SG15356, &UA2619563},
	{SG15357, &UA2619787},
	{SG15358, &UA2620086},
	{SG15359, &UA2620751},
	{SG15360, &UA2621213},
	{SG15361, &UA2621904},
	{SG15362, &UA2622509},
	{SG15363, &UA2622844},
	{SG15364, &UA2623104},
	{SG15365, &UA2623687},
	{SG15366, &UA2623854},
	{SG15367, &UA2624394},
	{SG15368, &UA2625213},
	{SG15369, &UA2625476},
	{SG15370, &UA2626979},
	{SG15371, &UA2627521},
	{SG15372, &UA2627834},
	{SG15373, &UA2628919},
	{SG15374, &UA2629058},
	{SG15375, &UA2630505},
	{SG15376, &UA2630781},
	{SG15404, &UA350550},
	{SG15405, &UA360835},
	{SG15406, &UA586661},
	{SG15407, &UA623046},
	{SG15408, &UA689991},
	{SG15409, &UA755617},
	{SG15410, &UA771734},
	{SG15411, &UA838526},
	{SG15412, &UA877602},
	{SG15413, &UA2024364},
	{SG15414, &UA2026106},
	{SG15415, &UA2026665},
	{SG15416, &UA2607643},
	{SG15417, &UA2607823},
	{SG15418, &UA2607953},
	{SG15419, &UA2608080},
	{SG15420, &UA2609219},
	{SG15421, &UA2609281},
	{SG15422, &UA2610879},
	{SG15423, &UA2611344},
	{SG15424, &UA2611315},
	{SG15425, &UA2611391},
	{SG15426, &UA2612178},
	{SG15427, &UA2613128},
	{SG15428, &UA2613896},
	{SG15429, &UA2615362},
	{SG15430, &UA2615831},
	{SG15431, &UA2615835},
	{SG15432, &UA2616037},
	{SG15433, &UA2616347},
	{SG15434, &UA2616387},
	{SG15435, &UA2616597},
	{SG15436, &UA2617011},
	{SG15437, &UA2617137},
	{SG15438, &UA2617341},
	{SG15439, &UA2617506},
	{SG15440, &UA2617599},
	{SG15441, &UA2617810},
	{SG15442, &UA2618039},
	{SG15443, &UA2618329},
	{SG15467, &UA2618420},
	{SG15469, &UA2618706},
	{SG15471, &UA2618743},
	{SG15473, &UA2619938},
	{SG15474, &UA2620346},
	{SG15475, &UA2620966},
	{SG15476, &UA2621519},
	{SG15477, &UA2622381},
	{SG15478, &UA2622884},
	{SG15479, &UA2624468},
	{SG15480, &UA2624818},
	{SG15481, &UA2627078},
	{SG15482, &UA2627709},
	{SG15483, &UA2628022},
	{SG15484, &UA2629088},
	{SG15485, &UA2630279},
	{SG15486, &UA2633838},
	{SG15487, &UA2634302},
	{SG15488, &UA2636056},
	{SG15489, &UA2636127},
	{SG15852, &UA249317},
	{SG15853, &UA361182},
	{SG15854, &UA622893},
	{SG15855, &UA624999},
	{SG15856, &UA634534},
	{SG15857, &UA691292},
	{SG15858, &UA825066},
	{SG15859, &UA2021731},
	{SG15860, &UA2025654},
	{SG15861, &UA2608138},
	{SG15862, &UA2611230},
	{SG15863, &UA2611343},
	{SG15864, &UA2611679},
	{SG15865, &UA2611994},
	{SG15866, &UA2613732},
	{SG15867, &UA2615339},
	{SG15868, &UA2616063},
	{SG15869, &UA2616125},
	{SG15870, &UA2616700},
	{SG15871, &UA2616955},
	{SG15872, &UA2617184},
	{SG15873, &UA2619881},
	{SG15882, &UA2622026},
	{SG15883, &UA2622588},
	{SG15884, &UA2623466},
	{SG15885, &UA2626499},
	{SG15886, &UA2628524},
	{SG15887, &UA2809036},
	{SG16041, &UA33367},
	{SG16043, &UA200904},
	{SG16045, &UA316369},
	{SG16047, &UA323666},
	{SG16049, &UA369342},
	{SG16051, &UA659431},
	{SG16053, &UA2021614},
	{SG16055, &UA2462997},
	{SG16057, &UA2607626},
	{SG16058, &UA2607778},
	{SG16060, &UA2607791},
	{SG16063, &UA2608261},
	{SG16064, &UA2608737},
	{SG16066, &UA2609607},
	{SG16069, &UA2610013},
	{SG16070, &UA2611221},
	{SG16071, &UA2615192},
	{SG16072, &UA2615360},
	{SG16073, &UA2615911},
	{SG16074, &UA2615921},
	{SG16075, &UA2616469},
	{SG16076, &UA2616806},
	{SG16078, &UA2617171},
	{SG16080, &UA2619838},
	{SG16082, &UA2620159},
	{SG16083, &UA2626131},
	{SG16084, &UA2630578},
	{SG16086, &UA2631683},
	{SG16181, &UA340629},
	{SG16182, &UA371147},
	{SG16183, &UA647175},
	{SG16185, &UA665608},
	{SG16186, &UA1945425},
	{SG16188, &UA2025030},
	{SG16190, &UA2027422},
	{SG16192, &UA2034042},
	{SG16193, &UA2216330},
	{SG16194, &UA2608819},
	{SG16195, &UA2609719},
	{SG16196, &UA2609781},
	{SG16197, &UA2609981},
	{SG16198, &UA2611888},
	{SG16199, &UA2612772},
	{SG16200, &UA2613185},
	{SG16201, &UA2613949},
	{SG16202, &UA2614869},
	{SG16203, &UA2614967},
	{SG16204, &UA2615368},
	{SG16206, &UA2615701},
	{SG16207, &UA2616569},
	{SG16208, &UA2616745},
	{SG16210, &UA2617367},
	{SG16212, &UA2617594},
	{SG16215, &UA2618044},
	{SG16217, &UA2618250},
	{SG16219, &UA2618632},
	{SG16221, &UA2619082},
	{SG16222, &UA2619150},
	{SG16225, &UA2619193},
	{SG16226, &UA2619716},
	{SG16227, &UA2621046},
	{SG16228, &UA2621532},
	{SG16229, &UA2621903},
	{SG16230, &UA2623580},
	{SG16231, &UA2624263},
	{SG16232, &UA2624553},
	{SG16234, &UA2626520},
	{SG16235, &UA2628630},
	{SG16236, &UA2629012},
	{SG16238, &UA2630040},
	{SG16240, &UA2635618},
	{SG16518, &UA32851},
	{SG16519, &UA73345},
	{SG16520, &UA253484},
	{SG16521, &UA331832},
	{SG16522, &UA369249},
	{SG16523, &UA369896},
	{SG16524, &UA647180},
	{SG16527, &UA838541},
	{SG16528, &UA1938295},
	{SG16529, &UA2020926},
	{SG16702, &UA49458},
	{SG16703, &UA567070},
	{SG16704, &UA575508},
	{SG16705, &UA643304},
	{SG16706, &UA734501},
	{SG16707, &UA1902916},
	{SG16708, &UA2311635},
	{SG16709, &UA2608116},
	{SG16710, &UA2608616},
	{SG16711, &UA2609454},
	{SG16712, &UA2609795},
	{SG16713, &UA2609975},
	{SG16714, &UA2612604},
	{SG16715, &UA2615743},
	{SG16716, &UA2615987},
	{SG16717, &UA2617780},
	{SG16719, &UA2617974},
	{SG16720, &UA2618937},
	{SG16722, &UA2619299},
	{SG16723, &UA2619706},
	{SG16724, &UA2619925},
	{SG16725, &UA2620754},
	{SG16726, &UA2624564},
	{SG16727, &UA2624602},
	{SG16728, &UA2631584},
	{SG16729, &UA2632807},
	{SG16773, &UA452998},
	{SG16774, &UA628347},
	{SG16775, &UA2024811},
	{SG16776, &UA2031626},
	{SG16777, &UA2031847},
	{SG16778, &UA2032623},
	{SG16779, &UA2034530},
	{SG16780, &UA2609910},
	{SG16781, &UA2611859},
	{SG16782, &UA2612444},
	{SG16783, &UA2617258},
	{SG16784, &UA2617417},
	{SG16785, &UA2617578},
	{SG16786, &UA2618367},
	{SG16787, &UA2618672},
	{SG16788, &UA2619622},
	{SG16789, &UA2620613},
	{SG16790, &UA2622603},
	{SG16791, &UA2622912},
	{SG16792, &UA2623964},
	{SG16793, &UA2624620},
	{SG16794, &UA2625157},
	{SG16795, &UA2629354},
	{SG16796, &UA2630493},
	{SG16797, &UA2632147},
	{SG16798, &UA2633940},
	{SG16837, &UA625169},
	{SG16838, &UA687481},
	{SG16839, &UA706477},
	{SG16840, &UA774655},
	{SG16841, &UA1953540},
	{SG16842, &UA2370865},
	{SG16843, &UA2444216},
	{SG16844, &UA2463559},
	{SG16845, &UA2608794},
	{SG16846, &UA2609177},
	{SG16847, &UA2609516},
	{SG16848, &UA2610035},
	{SG16849, &UA2613574},
	{SG16850, &UA2616553},
	{SG16851, &UA2618226},
	{SG16852, &UA2618559},
	{SG16853, &UA2619160},
	{SG16854, &UA2619232},
	{SG16855, &UA2619330},
	{SG16856, &UA2624070},
	{SG16857, &UA2624103},
	{SG16858, &UA2630043},
	{SG16859, &UA2635280},
	{SG16860, &UA2635806},
	{SG16936, &UA245285},
	{SG16937, &UA252039},
	{SG16938, &UA359315},
	{SG16939, &UA451883},
	{SG16940, &UA571271},
	{SG16941, &UA629583},
	{SG16942, &UA632024},
	{SG16944, &UA633048},
	{SG16946, &UA658385},
	{SG16948, &UA826308},
	{SG16950, &UA841687},
	{SG16954, &UA1937752},
	{SG16956, &UA2020747},
	{SG16958, &UA2020843},
	{SG17007, &UA194004},
	{SG17008, &UA218007},
	{SG17009, &UA354864},
	{SG17010, &UA369512},
	{SG17014, &UA369323},
	{SG17015, &UA578130},
	{SG17016, &UA622549},
	{SG17017, &UA624095},
	{SG17018, &UA899021},
	{SG17019, &UA1958155},
	{SG17020, &UA1961648},
	{SG17021, &UA2022436},
	{SG17033, &UA452960},
	{SG17036, &UA623810},
	{SG17037, &UA626405},
	{SG17038, &UA662134},
	{SG17039, &UA685316},
	{SG17040, &UA2463603},
	{SG17041, &UA2609387},
	{SG17042, &UA2610060},
	{SG17043, &UA2614815},
	{SG17044, &UA2616537},
	{SG17045, &UA2617910},
	{SG17046, &UA2618023},
	{SG17047, &UA2619980},
	{SG17048, &UA2623940},
	{SG17049, &UA2625641},
	{SG17051, &UA2626311},
	{SG17052, &UA2626835},
	{SG17055, &UA48675},
	{SG17056, &UA74002},
	{SG17057, &UA569266},
	{SG17058, &UA636644},
	{SG17239, &UA574449},
	{SG17240, &UA641466},
	{SG17241, &UA2022212},
	{SG17242, &UA2608125},
	{SG17243, &UA2609244},
	{SG17244, &UA2614208},
	{SG17245, &UA2616316},
	{SG17246, &UA2616826},
	{SG17247, &UA2617261},
	{SG17248, &UA2618086},
	{SG17249, &UA2619324},
	{SG17250, &UA2619734},
	{SG17251, &UA2620596},
	{SG17252, &UA2620863},
	{SG17253, &UA2620927},
	{SG17254, &UA2621984},
	{SG17255, &UA2622312},
	{SG17256, &UA2625905},
	{SG17257, &UA2628359},
	{SG17258, &UA2636687},
	{SG17259, &UA225320},
	{SG17260, &UA556637},
	{SG17261, &UA625018},
	{SG17262, &UA691499},
	{SG17263, &UA2607718},
	{SG17265, &UA2607919},
	{SG17267, &UA2610344},
	{SG17268, &UA2612272},
	{SG17269, &UA2615480},
	{SG17270, &UA2615674},
	{SG17271, &UA2621611},
	{SG17272, &UA2627522},
	{SG17273, &UA2629189},
	{SG17274, &UA335066},
	{SG17275, &UA352393},
	{SG17276, &UA369494},
	{SG17277, &UA659299},
	{SG17278, &UA681491},
	{SG17279, &UA720698},
	{SG17280, &UA2721745},
	{SG17281, &UA2807929},
	{SG17282, &UA2949244},
	{SG17283, &UA251468},
	{SG17284, &UA397411},
	{SG17285, &UA578636},
	{SG17286, &UA632669},
	{SG17287, &UA634687},
	{SG17293, &UA368763},
	{SG17294, &UA398309},
	{SG17295, &UA398482},
	{SG17296, &UA430085},
	{SG17297, &UA430090},
	{SG17298, &UA430096},
	{SG17299, &UA430103},
	{SG17300, &UA430104},
	{SG17301, &UA430147},
	{SG17302, &UA622947},
	{SG17303, &UA634128},
	{SG17304, &UA679029},
	{SG17305, &UA699854},
	{SG17306, &UA2024706},
	{SG17315, &UA254472},
	{SG17316, &UA674676},
	{SG17332, &UA73462},
	{SG17333, &UA236488},
	{SG17334, &UA246453},
	{SG17335, &UA248217},
	{SG17336, &UA330416},
	{SG17337, &UA701648},
	{SG17338, &UA2024524},
	{SG17341, &UA90787},
	{SG17343, &UA223144},
	{SG17345, &UA402615},
	{SG17347, &UA402616},
	{SG17349, &UA402618},
	{SG17351, &UA402620},
	{SG17353, &UA402624},
	{SG17355, &UA402630},
	{SG17356, &UA402649},
	{SG17358, &UA402650},
	{SG17359, &UA402651},
	{SG17360, &UA402661},
	{SG17361, &UA402673},
	{SG17362, &UA402675},
	{SG17363, &UA402686},
	{SG17364, &UA402763},
	{SG17366, &UA402780},
	{SG17367, &UA403279},
	{SG17368, &UA776491},
	{SG17425, &UA42781},
	{SG17426, &UA75016},
	{SG17427, &UA351549},
	{SG17428, &UA452503},
	{SG17429, &UA622890},
	{SG17430, &UA633688},
	{SG17438, &UA625967},
	{SG17439, &UA658933},
	{SG17440, &UA2033651},
	{SG17441, &UA2034103},
	{SG17442, &UA2222759},
	{SG17443, &UA2396378},
	{SG17444, &UA2397928},
	{SG17445, &UA2412180},
	{SG17446, &UA2504598},
	{SG17447, &UA2524162},
	{SG17448, &UA2608374},
	{SG17449, &UA2610652},
	{SG17450, &UA2611445},
	{SG17451, &UA2616139},
	{SG17452, &UA2619156},
	{SG17453, &UA2619595},
	{SG17454, &UA2622125},
	{SG17455, &UA2622174},
	{SG17456, &UA2624085},
	{SG17457, &UA2633533},
	{SG17458, &UA2636242},
	{SG17459, &UA625986},
	{SG17460, &UA752323},
	{SG17461, &UA2034325},
	{SG17475, &UA230432},
	{SG17476, &UA250318},
	{SG17477, &UA682830},
	{SG17554, &UA349573},
	{SG17555, &UA395712},
	{SG17556, &UA633278},
	{SG17557, &UA1943076},
	{SG17558, &UA2021741},
	{SG17595, &UA625222},
	{SG17596, &UA658848},
	{SG17597, &UA674184},
	{SG17598, &UA837245},
	{SG17599, &UA2337967},
	{SG17600, &UA2607944},
	{SG17601, &UA2608078},
	{SG17602, &UA2609434},
	{SG17603, &UA2620283},
	{SG17604, &UA210514},
	{SG17605, &UA243120},
	{SG17606, &UA244921},
	{SG17608, &UA250722},
	{SG17610, &UA328202},
	{SG17612, &UA334216},
	{SG17614, &UA341547},
	{SG17615, &UA342293},
	{SG17616, &UA354847},
	{SG17617, &UA402664},
	{SG17618, &UA402694},
	{SG17619, &UA402703},
	{SG17620, &UA402704},
	{SG17621, &UA402708},
	{SG17622, &UA402713},
	{SG17623, &UA402717},
	{SG17624, &UA402718},
	{SG17625, &UA402719},
	{SG17626, &UA402726},
	{SG17628, &UA402727},
	{SG17630, &UA402731},
	{SG17632, &UA402735},
	{SG17633, &UA402742},
	{SG17635, &UA402745},
	{SG17637, &UA402750},
	{SG17638, &UA402777},
	{SG17639, &UA402782},
	{SG17640, &UA402787},
	{SG17641, &UA402799},
	{SG17642, &UA402839},
	{SG17643, &UA402884},
	{SG17644, &UA402946},
	{SG17645, &UA402962},
	{SG17646, &UA402968},
	{SG17647, &UA403013},
	{SG17648, &UA403023},
	{SG17649, &UA403032},
	{SG17650, &UA403108},
	{SG17651, &UA403184},
	{SG17652, &UA403294},
	{SG17653, &UA403298},
	{SG17654, &UA2025744},
	{SG17657, &UA42786},
	{SG17658, &UA43050},
	{SG17659, &UA58676},
	{SG17660, &UA58700},
	{SG17661, &UA58715},
	{SG17662, &UA58720},
	{SG17663, &UA58728},
	{SG17664, &UA75422},
	{SG17665, &UA75812},
	{SG17666, &UA330025},
	{SG17667, &UA657589},
	{SG17689, &UA342177},
	{SG17690, &UA643080},
	{SG17691, &UA1917727},
	{SG17692, &UA2023342},
	{SG17776, &UA623473},
	{SG17778, &UA631554},
	{SG17780, &UA653524},
	{SG17781, &UA674033},
	{SG17783, &UA690084},
	{SG17785, &UA758987},
	{SG17787, &UA766534},
	{SG17789, &UA834059},
	{SG17791, &UA894822},
	{SG17792, &UA895834},
	{SG17794, &UA902148},
	{SG17795, &UA909126},
	{SG17796, &UA910022},
	{SG17798, &UA1952790},
	{SG17800, &UA1960161},
	{SG17801, &UA1964089},
	{SG17802, &UA1971647},
	{SG17803, &UA1984648},
	{SG17804, &UA1989459},
	{SG17805, &UA2031455},
	{SG17806, &UA2034111},
	{SG17807, &UA2411970},
	{SG17808, &UA2642783},
	{SG17809, &UA2652695},
	{SG17810, &UA2889187},
	{SG17811, &UA2983293},
	{SG17812, &UA2988685},
	{SG17885, &UA627632},
	{SG17886, &UA1955799},
	{SG17904, &UA49906},
	{SG17905, &UA630066},
	{SG17906, &UA2022381},
	{SG17912, &UA23662},
	{SG17914, &UA316122},
	{SG17916, &UA713661},
	{SG17959, &UA199889},
	{SG17960, &UA236792},
	{SG17961, &UA330633},
	{SG17962, &UA436816},
	{SG17963, &UA560168},
	{SG18022, &UA33381},
	{SG18023, &UA240542},
	{SG18024, &UA702657},
	{SG18025, &UA2018918},
	{SG18096, &UA69329},
	{SG18097, &UA566499},
	{SG18122, &UA20278},
	{SG18123, &UA20283},
	{SG18124, &UA20280},
	{SG18125, &UA42754},
	{SG18126, &UA42768},
	{SG18127, &UA42778},
	{SG18128, &UA58692},
	{SG18129, &UA58693},
	{SG18130, &UA58694},
	{SG18131, &UA58706},
	{SG18132, &UA58710},
	{SG18133, &UA58717},
	{SG18134, &UA58953},
	{SG18135, &UA864974},
	{SG18140, &UA227999},
	{SG18141, &UA575625},
	{SG18142, &UA582220},
	{SG18143, &UA641797},
	{SG18151, &UA402914},
	{SG18152, &UA403029},
	{SG18153, &UA403037},
	{SG18154, &UA403074},
	{SG18155, &UA403097},
	{SG18156, &UA403145},
	{SG18157, &UA403309},
	{SG18158, &UA711957},
	{SG18160, &UA767039},
	{SG18161, &UA787346},
	{SG18162, &UA1964591},
	{SG18163, &UA2026102},
	{SG18164, &UA2026633},
	{SG18165, &UA2034005},
	{SG18166, &UA2519772},
	{SG18176, &UA192255},
	{SG18177, &UA313670},
	{SG18178, &UA314388},
	{SG18179, &UA314690},
	{SG18180, &UA318670},
	{SG18181, &UA371355},
	{SG18182, &UA559093},
	{SG18183, &UA717825},
	{SG18189, &UA1885102},
	{SG18202, &UA78012},
	{SG18204, &UA686826},
	{SG18205, &UA694681},
	{SG18207, &UA2024469},
	{SG18225, &UA653236},
	{SG18226, &UA668481},
	{SG18227, &UA2022624},
	{SG18266, &UA380073},
	{SG18267, &UA2024005},
	{SG18269, &UA2608127},
	{SG18271, &UA2615017},
	{SG18273, &UA2616862},
	{SG18274, &UA2617789},
	{SG18277, &UA2618941},
	{SG18278, &UA2621744},
	{SG18284, &UA232946},
	{SG18285, &UA713608},
	{SG18286, &UA1906542},
	{SG18287, &UA90491},
	{SG18288, &UA181087},
	{SG18289, &UA183412},
	{SG18290, &UA304049},
	{SG18291, &UA402607},
	{SG18292, &UA402605},
	{SG18293, &UA402622},
	{SG18294, &UA402643},
	{SG18295, &UA402646},
	{SG18296, &UA402668},
	{SG18297, &UA402678},
	{SG18299, &UA402724},
	{SG18300, &UA402845},
	{SG18301, &UA402766},
	{SG18302, &UA402943},
	{SG18303, &UA403040},
	{SG18304, &UA403068},
	{SG18305, &UA2257219},
	{SG18318, &UA192836},
	{SG18319, &UA640489},
	{SG18320, &UA656699},
	{SG18321, &UA2031186},
	{SG18339, &UA220273},
	{SG18340, &UA640941},
	{SG18341, &UA651042},
	{SG18342, &UA756073},
	{SG18347, &UA794743},
	{SG18348, &UA1894445},
	{SG18349, &UA1906316},
	{SG18350, &UA2021764},
	{SG18351, &UA2022791},
	{SG18352, &UA2025708},
	{SG18441, &UA626422},
	{SG18442, &UA1950616},
	{SG18448, &UA352014},
	{SG18449, &UA361965},
	{SG18450, &UA575683},
	{SG18452, &UA627148},
	{SG18453, &UA632748},
	{SG18454, &UA683099},
	{SG18455, &UA753814},
	{SG18456, &UA2023235},
	{SG18476, &UA627995},
	{SG18477, &UA633182},
	{SG18478, &UA2019350},
	{SG18479, &UA2022902},
	{SG18517, &UA575254},
	{SG18518, &UA576268},
	{SG18519, &UA576508},
	{SG18520, &UA578468},
	{SG18521, &UA630542},
	{SG18525, &UA672864},
	{SG18526, &UA2024600},
	{SG18527, &UA33421},
	{SG18528, &UA861733},
	{SG18531, &UA227583},
	{SG18532, &UA567603},
	{SG18533, &UA2023361},
	{SG18534, &UA2023740},
	{SG18535, &UA2031926},
	{SG18536, &UA2519182},
	{SG18549, &UA402853},
	{SG18550, &UA402944},
	{SG18551, &UA402959},
	{SG18552, &UA403001},
	{SG18554, &UA403048},
	{SG18555, &UA403049},
	{SG18556, &UA403054},
	{SG18557, &UA403057},
	{SG18558, &UA403019},
	{SG18559, &UA403085},
	{SG18560, &UA403204},
	{SG18561, &UA403210},
	{SG18562, &UA403224},
	{SG18563, &UA2025645},
	{SG18579, &UA370233},
	{SG18580, &UA370340},
	{SG18581, &UA396912},
	{SG18582, &UA579181},
	{SG18583, &UA630821},
	{SG18584, &UA655983},
	{SG18585, &UA727955},
	{SG18586, &UA825247},
	{SG18587, &UA845408},
	{SG18588, &UA1900947},
	{SG18589, &UA1935732},
	{SG18590, &UA1990747},
	{SG18591, &UA2021603},
	{SG18596, &UA2022739},
	{SG18597, &UA2023517},
	{SG18598, &UA2026587},
	{SG18599, &UA2027048},
	{SG18600, &UA2028298},
	{SG18601, &UA2030956},
	{SG18602, &UA2031590},
	{SG18603, &UA2032837},
	{SG18604, &UA2032862},
	{SG18605, &UA2033896},
	{SG18606, &UA2413620},
	{SG18607, &UA2463887},
	{SG18608, &UA2468908},
	{SG18609, &UA2544812},
	{SG18610, &UA2556409},
	{SG18611, &UA2642538},
	{SG18612, &UA2644027},
	{SG18613, &UA2650850},
	{SG18614, &UA2746480},
	{SG18615, &UA2758481},
	{SG18616, &UA2759059},
	{SG18617, &UA2980783},
	{SG18621, &UA352114},
	{SG18623, &UA355576},
	{SG18625, &UA358718},
	{SG18626, &UA626332},
	{SG18627, &UA727100},
	{SG18628, &UA2034318},
	{SG18629, &UA2808434},
	{SG18636, &UA624826},
	{SG18637, &UA739263},
	{SG18638, &UA2030625},
	{SG18662, &UA641048},
	{SG18663, &UA2021463},
	{SG18665, &UA2610246},
	{SG18666, &UA2617704},
	{SG18668, &UA78040},
	{SG18669, &UA332254},
	{SG18670, &UA729583},
	{SG18671, &UA2021338},
	{SG18672, &UA708512},
	{SG18698, &UA646195},
	{SG18700, &UA724077},
	{SG18701, &UA2030699},
	{SG18702, &UA2495929},
	{SG18703, &UA2610961},
	{SG18704, &UA2620202},
	{SG18705, &UA2622527},
	{SG18706, &UA2630235},
	{SG18707, &UA2635251},
	{SG18716, &UA369219},
	{SG18717, &UA402929},
	{SG18718, &UA402941},
	{SG18719, &UA403064},
	{SG18720, &UA403075},
	{SG18721, &UA403083},
	{SG18723, &UA403122},
	{SG18725, &UA403141},
	{SG18727, &UA403143},
	{SG18729, &UA403168},
	{SG18730, &UA403195},
	{SG18731, &UA403229},
	{SG18732, &UA403319},
	{SG18733, &UA874709},
	{SG18734, &UA2025486},
	{SG18777, &UA251829},
	{SG18778, &UA624691},
	{SG18779, &UA912980},
	{SG18780, &UA2033072},
	{SG18801, &UA361786},
	{SG18802, &UA637873},
	{SG18803, &UA2022506},
	{SG18830, &UA630023},
	{SG18831, &UA2024917},
	{SG18836, &UA69468},
	{SG18837, &UA77905},
	{SG18838, &UA2032923},
	{SG18857, &UA246634},
	{SG18858, &UA249373},
	{SG18859, &UA712461},
	{SG18945, &UA693553},
	{SG18946, &UA719014},
	{SG18947, &UA2022790},
	{SG18948, &UA668705},
	{SG18949, &UA678111},
	{SG18950, &UA754658},
	{SG18951, &UA769802},
	{SG18953, &UA869720},
	{SG18954, &UA1905494},
	{SG18973, &UA626158},
	{SG18974, &UA707231},
	{SG18975, &UA713197},
	{SG18976, &UA805216},
	{SG18977, &UA864860},
	{SG18978, &UA900495},
	{SG18979, &UA902525},
	{SG18980, &UA1972140},
	{SG18981, &UA1972315},
	{SG18982, &UA2025520},
	{SG18983, &UA2026315},
	{SG18984, &UA2031990},
	{SG18985, &UA2032457},
	{SG18986, &UA2444440},
	{SG18987, &UA3085263},
	{SG18988, &UA213198},
	{SG18989, &UA219000},
	{SG18990, &UA331397},
	{SG18991, &UA2019243},
	{SG18992, &UA2020169},
	{SG19054, &UA364205},
	{SG19055, &UA2024350},
	{SG19106, &UA648910},
	{SG19107, &UA695656},
	{SG19108, &UA2022382},
	{SG19114, &UA316300},
	{SG19116, &UA338065},
	{SG19117, &UA344987},
	{SG19119, &UA413007},
	{SG19120, &UA574701},
	{SG19121, &UA640686},
	{SG19122, &UA769449},
	{SG19124, &UA1893700},
	{SG19126, &UA1908525},
	{SG19128, &UA1964283},
	{SG19130, &UA2018948},
	{SG19131, &UA2483380},
	{SG19132, &UA2573574},
	{SG19133, &UA2849789},
	{SG19134, &UA2981466},
	{SG19201, &UA402956},
	{SG19202, &UA403044},
	{SG19203, &UA403091},
	{SG19204, &UA403281},
	{SG19205, &UA721137},
	{SG19206, &UA797748},
	{SG19207, &UA870799},
	{SG19208, &UA2027215},
	{SG19209, &UA2032840},
	{SG19210, &UA3101684},
	{SG19247, &UA68109},
	{SG19248, &UA319725},
	{SG19250, &UA2607582},
	{SG19252, &UA2607586},
	{SG19254, &UA2607656},
	{SG19285, &UA227719},
	{SG19290, &UA362852},
	{SG19312, &UA68043},
	{SG19313, &UA558250},
	{SG19314, &UA1904594},
	{SG19315, &UA2019387},
	{SG19316, &UA452095},
	{SG19317, &UA571886},
	{SG19318, &UA672903},
	{SG19319, &UA711235},
	{SG19320, &UA765684},
	{SG19321, &UA2540556},
	{SG19322, &UA2540562},
	{SG19323, &UA2806680},
	{SG19361, &UA629817},
	{SG19362, &UA1935834},
	{SG19411, &UA190403},
	{SG19412, &UA2019564},
	{SG19501, &UA344494},
	{SG19502, &UA2030023},
	{SG19652, &UA183923},
	{SG19682, &UA624717},
	{SG19683, &UA2608197},
	{SG19684, &UA2627278},
	{SG19726, &UA221372},
	{SG19740, &UA90689},
	{SG19749, &UA698908},
	{SG19750, &UA2843047},
	{SG19756, &UA634155},
	{SG19757, &UA1932128},
	{SG19758, &UA2026449},
	{SG19759, &UA2033959},
	{SG19760, &UA2034508},
	{SG19761, &UA2400872},
	{SG19765, &UA738867},
	{SG19766, &UA792480},
	{SG19767, &UA1983479},
	{SG19784, &UA632270},
	{SG19809, &UA363889},
	{SG19810, &UA650731},
	{SG19838, &UA43041},
	{SG19839, &UA73194},
	{SG19840, &UA75816},
	{SG19841, &UA630510},
	{SG19842, &UA649382},
	{SG19875, &UA669632},
	{SG19876, &UA2608734},
	{SG19877, &UA2620911},
	{SG19946, &UA2431548},
	{SG19947, &UA2620280},
	{SG19995, &UA33219},
	{SG19996, &UA43045},
	{SG19997, &UA43046},
	{SG19998, &UA43049},
	{SG19999, &UA43051},
	{SG20002, &UA43052},
	{SG20004, &UA58950},
	{SG20006, &UA58957},
	{SG20008, &UA58956},
	{SG20010, &UA58961},
	{SG20011, &UA58969},
	{SG20012, &UA73691},
	{SG20088, &UA743698},
	{SG20103, &UA815597},
	{SG20104, &UA2031183},
	{SG20179, &UA20730},
	{SG20180, &UA20732},
	{SG20181, &UA20733},
	{SG20182, &UA20735},
	{SG20183, &UA43043},
	{SG20184, &UA43048},
	{SG20185, &UA58960},
	{SG20186, &UA58963},
	{SG20187, &UA773687},
	{SG20233, &UA241877},
	{SG20263, &UA624012},
	{SG20264, &UA2024388},
	{SG20281, &UA673408},
	{SG20282, &UA743986},
	{SG20283, &UA2020781},
	{SG20284, &UA2168122},
	{SG20285, &UA2444897},
	{SG20286, &UA2940157},
	{SG20287, &UA3047229},
	{SG20288, &UA3057468},
	{SG20289, &UA3103171},
	{SG20358, &UA47607},
	{SG20359, &UA47616},
	{SG20360, &UA395337},
	{SG20396, &UA637684},
	{SG20397, &UA2033608},
	{SG20467, &UA651336},
	{SG20505, &UA23279},
	{SG20506, &UA47609},
	{SG20507, &UA369558},
	{SG20508, &UA2607570},
	{SG20569, &UA2030274},
	{SG20570, &UA2372670},
	{SG20571, &UA2979982},
	{SG20697, &UA2637359},
	{SG20699, &UA2774059},
	{SG20804, &UA581259},
	{SG20805, &UA2026256},
	{SG20806, &UA2816548},
	{SG20808, &UA2239095},
	{SG20847, &UA368865},
	{SG20923, &UA1991599},
	{SG20925, &UA2022836},
	{SG20926, &UA2365428},
	{SG20927, &UA3118386},
	{SG20949, &UA3039299},
	{SG20965, &UA627173},
	{SG20996, &UA2435626},
	{SG20997, &UA2623903},
	{SG21142, &UA834954},
	{SG21208, &UA849657},
	{SG21209, &UA2022899},
	{SG21210, &UA2814713},
	{SG21243, &UA625363},
	{SG21296, &UA717033},
	{SG21325, &UA60518},
	{SG21326, &UA222604},
	{SG21327, &UA833218},
	{SG21328, &UA2428121},
	{SG21356, &UA728198},
	{SG21370, &UA3031199},
	{SG21416, &UA3031735},
	{SG21469, &UA2033526},
	{SG21470, &UA2448391},
	{SG21471, &UA2814325},
	{SG21473, &UA90688},
	{SG21572, &UA2666533},
	{SG21616, &UA68603},
	{SG21637, &UA2774319},
	{SG21640, &UA2030881},
	{SG21673, &UA2653545},
	{SG21685, &UA2784793},
	{SG21727, &UA2797113},
	{SG21731, &UA2538202},
	{SG21761, &UA2257176},
	{SG21762, &UA2616227},
	{SG21788, &UA47624},
	{SG21789, &UA60515},
	{SG21803, &UA2625125},
	{SG21804, &UA2626331},
	{SG21815, &UA625255},
	{SG21816, &UA848615},
	{SG21824, &UA2622938},
	{SG21825, &UA2623269},
	{SG21847, &UA2486588},
	{SG21848, &UA2621263},
	{SG21933, &UA2578200},
	{SG21954, &UA2667651},
	{SG21955, &UA2854079},
	{SG21975, &UA2026068},
	{SG21976, &UA2981305},
	{SG22017, &UA2758984},
	{SG22019, &UA2442375},
	{SG22020, &UA2609740},
	{SG22026, &UA3033547},
	{SG22173, &UA2624674},
	{SG22174, &UA2624936},
	{SG22219, &UA655413},
	{SG22260, &UA2625178},
	{SG22273, &UA761652},
	{SG22274, &UA2024021},
	{SG22275, &UA2033146},
	{SG22292, &UA664355},
	{SG22445, &UA3016289},
	{SG22502, &UA3056361},
	{SG22536, &UA3060878},
	{SG22545, &UA710702},
	{SG22547, &UA1924955},
	{SG22591, &UA2528896},
	{SG22614, &UA2624473},
	{SG22633, &UA3061015},
	{SG22683, &UA3115612},
	{SG22710, &UA2020496},
	{SG22711, &UA2608081},
	{SG22718, &UA2620169},
	{SG22724, &UA2799278},
	{SG22725, &UA2799327},
	{SG22729, &UA2865880},
	{SG22743, &UA3060715},
	{SG22777, &UA2455607},
	{SG22778, &UA2616199},
	{SG22779, &UA2621407},
	{SG22796, &UA2026711},
	{SG22798, &UA2612140},
	{SG22819, &UA2525552},
	{SG22823, &UA1963174},
	{SG22828, &UA2623048},
	{SG22838, &UA2622187},
	{SG22839, &UA2622749},
	{SG22853, &UA2809822},
	{SG22871, &UA2610903},
	{SG22872, &UA2613689},
	{SG22902, &UA2652994},
	{SG22920, &UA2651964},
	{SG22946, &UA2879379},
	{SG22952, &UA3115431},
	{SG22985, &UA304184},
	{SG22992, &UA2798980},
	{SG23008, &UA2654092},
	{SG23025, &UA203426},
	{SG23026, &UA560341},
	{SG23028, &UA2613998},
	{SG23029, &UA2617010},
	{SG23058, &UA2632872},
	{SG23059, &UA2630722},
	{SG23060, &UA2637499},
	{SG23085, &UA2626879},
	{SG23105, &UA2609270},
	{SG23106, &UA2609886},
	{SG23111, &UA3114978},
	{SG23114, &UA2021341},
	{SG23135, &UA2607701},
	{SG23136, &UA2610351},
	{SG23137, &UA2633446},
	{SG23139, &UA2622205},
	{SG23140, &UA2631765},
	{SG23158, &UA329351},
	{SG23159, &UA565124},
	{SG23172, &UA2894213},
	{SG23173, &UA2952204},
	{SG23209, &UA2684572},
	{SG23222, &UA440423},
	{SG23225, &UA855358},
	{SG23226, &UA1930326},
	{SG23228, &UA2020705},
	{SG23242, &UA3016182},
	{SG23252, &UA2615436},
	{SG23253, &UA2634452},
	{SG23254, &UA2624403},
	{SG23255, &UA2630673},
	{SG23256, &UA2628394},
	{SG23267, &UA3078190},
	{SG23283, &UA2854723},
	{SG23287, &UA3044192},
	{SG23295, &UA2033231},
	{SG23296, &UA2607781},
	{SG23297, &UA2607915},
	{SG23298, &UA2616889},
	{SG23307, &UA3078667},
	{SG23309, &UA3100554},
	{SG23323, &UA2032353},
	{SG23347, &UA3080026},
	{SG23427, &UA2608840},
	{SG23428, &UA2627966},
	{SG23430, &UA2629284},
	{SG23444, &UA2617418},
	{SG23458, &UA2610109},
	{SG23459, &UA2619439},
	{SG23460, &UA2623394},
	{SG23461, &UA2624131},
	{SG23462, &UA2615633},
	{SG23464, &UA2624780},
	{SG23473, &UA2634986},
	{SG23474, &UA2632609},
	{SG23492, &UA2615377},
	{SG23494, &UA2631332},
	{SG23495, &UA2633420},
	{SG23497, &UA2630154},
	{SG23504, &UA2614334},
	{SG23505, &UA2618705},
	{SG23506, &UA2620431},
	{SG23516, &UA2635476},
	{SG23519, &UA2631758},
	{SG23531, &UA195774},
	{SG23532, &UA1993558},
	{SG23533, &UA2027165},
	{SG23534, &UA2023016},
	{SG23542, &UA2611194},
	{SG23543, &UA2616305},
	{SG23544, &UA2634420},
	{SG23555, &UA2613200},
	{SG23556, &UA2633069},
	{SG23559, &UA2622906},
	{SG23560, &UA2622340},
	{SG23561, &UA2629597},
	{SG23618, &UA2612460},
	{SG23622, &UA2620020},
	{SG23623, &UA2625936},
	{SG23625, &UA2623920},
	{SG23634, &UA2608723},
	{SG23635, &UA2609031},
	{SG23636, &UA2615073},
	{SG23637, &UA2625852},
	{SG23638, &UA2632611},
	{SG23639, &UA2635890},
	{SG23640, &UA2633992},
	{SG23650, &UA40105},
	{SG23651, &UA2615399},
	{SG23656, &UA2616245},
	{SG23658, &UA2637843},
	{SG23681, &UA532358},
	{SG23684, &UA2018861},
	{SG23696, &UA2613680},
	{SG23698, &UA2608921},
	{SG23700, &UA2610555},
	{SG23702, &UA2623152},
	{SG23703, &UA2610966},
	{SG23705, &UA2635387},
	{SG23709, &UA2624737},
	{SG23710, &UA2635655},
	{SG23711, &UA2620389},
	{SG23717, &UA2619672},
	{SG23719, &UA2622594},
	{SG23721, &UA2635790},
	{SG23725, &UA2623217},
	{SG23727, &UA2623675},
	{SG23729, &UA2623428},
	{SG23730, &UA2624887},
	{SG23732, &UA2623943},
	{SG23734, &UA2635527},
	{SG23740, &UA2634198},
	{SG23743, &UA2635072},
	{SG23830, &UA2609020},
	{SG23832, &UA2609307},
	{SG23834, &UA2610857},
	{SG23836, &UA2615972},
	{SG23837, &UA2616279},
	{SG23839, &UA2623626},
	{SG23846, &UA2620912},
	{SG23849, &UA2623679},
	{SG23851, &UA2624515},
	{SG23858, &UA2636804},
	{SG23931, &UA90553},
	{SG23955, &UA2024138},
	{SG23967, &UA2610501},
	{SG23969, &UA2610730},
	{SG23971, &UA2617525},
	{SG23973, &UA2613741},
	{SG23975, &UA2614365},
	{SG23980, &UA2617726},
	{SG23987, &UA2620964},
	{SG24016, &UA2631522},
	{SG24018, &UA2634096},
	{SG24114, &UA403016},
	{SG24117, &UA2610284},
	{SG24118, &UA2611086},
	{SG24119, &UA2613525},
	{SG24120, &UA2617092},
	{SG24122, &UA2614716},
	{SG24123, &UA2622524},
	{SG24125, &UA2616338},
	{SG24127, &UA2616815},
	{SG24129, &UA2617277},
	{SG24179, &UA2619374},
	{SG24182, &UA2620430},
	{SG24184, &UA2628555},
	{SG24188, &UA2621587},
	{SG24190, &UA2633167},
	{SG24192, &UA2624178},
	{SG24198, &UA2626991},
	{SG24199, &UA2630077},
	{SG24202, &UA2627034},
	{SG24204, &UA2627924},
	{SG24206, &UA2627796},
	{SG24210, &UA2635124},
	{SG24213, &UA2631803},
	{SG24293, &UA90595},
	{SG24309, &UA1980264},
	{SG24328, &UA2607595},
	{SG24331, &UA2608487},
	{SG24336, &UA2610853},
	{SG24338, &UA2613603},
	{SG24343, &UA2617666},
	{SG24356, &UA2629179},
	{SG24358, &UA2622581},
	{SG24361, &UA2622732},
	{SG24362, &UA2625575},
	{SG24365, &UA2623045},
	{SG24369, &UA2624012},
	{SG24374, &UA2626467},
	{SG24376, &UA2626752},
	{SG24381, &UA2627540},
	{SG24383, &UA2628677},
	{SG24387, &UA2629707},
	{SG24389, &UA2637994},
	{SG24392, &UA2630546},
	{SG24394, &UA2631159},
	{SG24398, &UA2631840},
	{SG24401, &UA2632808},
	{SG24405, &UA2633158},
	{SG24408, &UA2633915},
	{SG24452, &UA47612},
	{SG24593, &UA2607533},
	{SG24596, &UA2607985},
	{SG24598, &UA2608996},
	{SG24600, &UA2609422},
	{SG24602, &UA2609941},
	{SG24604, &UA2610257},
	{SG24606, &UA2610788},
	{SG24608, &UA2612351},
	{SG24609, &UA2612495},
	{SG24612, &UA2613602},
	{SG24616, &UA2615904},
	{SG24626, &UA2618113},
	{SG24630, &UA2619570},
	{SG24632, &UA2619772},
	{SG24636, &UA2620308},
	{SG24645, &UA2621612},
	{SG24647, &UA2621647},
	{SG24650, &UA2622105},
	{SG24654, &UA2622956},
	{SG24656, &UA2623189},
	{SG24657, &UA2623322},
	{SG24659, &UA2623323},
	{SG24662, &UA2623419},
	{SG24668, &UA2625217},
	{SG24671, &UA2625379},
	{SG24678, &UA2629504},
	{SG24679, &UA2629619},
	{SG24683, &UA2629790},
	{SG24687, &UA2630138},
	{SG24689, &UA2631830},
	{SG24692, &UA2630241},
	{SG24694, &UA2633760},
	{SG24700, &UA2631620},
	{SG24702, &UA2632015},
	{SG24706, &UA2632312},
	{SG24709, &UA2632314},
	{SG24712, &UA2633313},
	{SG24715, &UA2633552},
	{SG24718, &UA2634225},
	{SG24721, &UA2634300},
	{SG24725, &UA2635198},
	{SG24731, &UA2637081},
	{SG25003, &UA2540606},
	{SG25016, &UA2609464},
	{SG25018, &UA2610456},
	{SG25021, &UA2611455},
	{SG25023, &UA2611752},
	{SG25027, &UA2611782},
	{SG25030, &UA2611975},
	{SG25032, &UA2612535},
	{SG25034, &UA2614057},
	{SG25035, &UA2614097},
	{SG25040, &UA2615624},
	{SG25045, &UA2615880},
	{SG25048, &UA2616189},
	{SG25050, &UA2616275},
	{SG25053, &UA2616426},
	{SG25057, &UA2616593},
	{SG25059, &UA2616614},
	{SG25061, &UA2616639},
	{SG25066, &UA2617045},
	{SG25069, &UA2617661},
	{SG25077, &UA2618740},
	{SG25082, &UA2618947},
	{SG25084, &UA2619046},
	{SG25093, &UA2620547},
	{SG25097, &UA2620725},
	{SG25104, &UA2621677},
	{SG25107, &UA2622485},
	{SG25110, &UA2622703},
	{SG25112, &UA2623016},
	{SG25115, &UA2623233},
	{SG25125, &UA2624987},
	{SG25127, &UA2625088},
	{SG25134, &UA2626756},
	{SG25136, &UA2626875},
	{SG25142, &UA2627464},
	{SG25145, &UA2627948},
	{SG25150, &UA2628781},
	{SG25156, &UA2629440},
	{SG25165, &UA2630322},
	{SG25168, &UA2630751},
	{SG25170, &UA2630800},
	{SG25172, &UA2630868},
	{SG25175, &UA2631161},
	{SG25177, &UA2631415},
	{SG25181, &UA2631727},
	{SG25184, &UA2632251},
	{SG25186, &UA2632279},
	{SG25189, &UA2632351},
	{SG25191, &UA2632363},
	{SG25194, &UA2632964},
	{SG25196, &UA2633025},
	{SG25200, &UA2633963},
	{SG25207, &UA2634349},
	{SG25208, &UA2634368},
	{SG25211, &UA2634448},
	{SG25214, &UA2634533},
	{SG25216, &UA2634609},
	{SG25219, &UA2634872},
	{SG25225, &UA2635584},
	{SG25226, &UA2635607},
	{SG25228, &UA2635735},
	{SG25231, &UA2636086},
	{SG25233, &UA2636091},
	{SG25235, &UA2636145},
	{SG25237, &UA2636317},
	{SG25268, &UA2636469},
	{SG25276, &UA2636918},
	{SG25280, &UA2637084},
	{SG25282, &UA2637105},
	{SG25290, &UA2637614},
	{SG25292, &UA2637775},
	{SG25295, &UA2637903},
	{SG25297, &UA2638021},
	{NULL}};
const Segmentdata DAndroidVersion4_0[] = {
	{SG13118, &UA2339366},
	{SG13119, &UA2762428},
	{SG13120, &UA2780875},
	{SG13121, &UA3070337},
	{SG13548, &UA2653860},
	{SG13552, &UA2693799},
	{SG13554, &UA2709449},
	{SG13555, &UA2726898},
	{SG13556, &UA2762985},
	{SG13585, &UA2777461},
	{SG13587, &UA2790199},
	{SG13588, &UA2810120},
	{SG13589, &UA2849769},
	{SG13591, &UA2936866},
	{SG13626, &UA2985124},
	{SG13627, &UA2990375},
	{SG13631, &UA2994840},
	{SG13632, &UA2995442},
	{SG13635, &UA2996008},
	{SG13661, &UA3049448},
	{SG13665, &UA3102871},
	{SG13668, &UA3107627},
	{SG13670, &UA3120850},
	{SG13672, &UA3121088},
	{SG13673, &UA3124889},
	{SG13674, &UA3128008},
	{SG13682, &UA2708211},
	{SG14631, &UA2614729},
	{SG14987, &UA2018343},
	{SG14991, &UA2578145},
	{SG15165, &UA2709163},
	{SG16174, &UA2626113},
	{SG16626, &UA616717},
	{SG16629, &UA2020083},
	{SG16904, &UA402797},
	{SG16928, &UA2336971},
	{SG18658, &UA2508710},
	{SG18855, &UA3063563},
	{SG19021, &UA1972661},
	{SG19199, &UA2556327},
	{SG19367, &UA2677040},
	{SG20161, &UA2666308},
	{SG20164, &UA2555272},
	{SG20166, &UA2855376},
	{SG20395, &UA2029027},
	{SG20402, &UA2523555},
	{SG20495, &UA2547729},
	{SG21265, &UA2424884},
	{SG21379, &UA2396168},
	{SG21482, &UA2985274},
	{SG21734, &UA2531574},
	{SG21819, &UA2594017},
	{SG22047, &UA2578305},
	{SG22064, &UA2030197},
	{SG22865, &UA2696897},
	{SG23003, &UA2759092},
	{SG23039, &UA2957069},
	{SG23073, &UA2026097},
	{SG23163, &UA2022727},
	{SG23366, &UA2957563},
	{SG23393, &UA3101664},
	{SG23467, &UA3092326},
	{SG23629, &UA812575},
	{NULL}};
const Segmentdata DAndroidVersion4_0_1[] = {
	{SG13549, &UA370663},
	{SG13550, &UA2702657},
	{SG13551, &UA2708110},
	{SG13553, &UA2892986},
	{SG13590, &UA1936199},
	{SG13629, &UA2416471},
	{SG13633, &UA2425083},
	{SG13634, &UA2425631},
	{SG13660, &UA2486203},
	{SG13662, &UA2486314},
	{SG13663, &UA2502545},
	{SG13664, &UA2502879},
	{SG13669, &UA2580843},
	{SG13671, &UA2583262},
	{SG13675, &UA2595166},
	{SG13676, &UA2596821},
	{SG13677, &UA2597553},
	{SG13678, &UA2597611},
	{SG13679, &UA2598844},
	{SG13680, &UA2600597},
	{SG13681, &UA2642147},
	{SG13683, &UA2652510},
	{SG13684, &UA2654220},
	{SG13685, &UA2658224},
	{SG13686, &UA2659080},
	{SG13687, &UA2659155},
	{SG13688, &UA2664003},
	{SG13689, &UA2664423},
	{SG13693, &UA2695880},
	{SG13696, &UA2701519},
	{SG13697, &UA2712286},
	{SG13698, &UA2729220},
	{SG13699, &UA2729543},
	{SG13700, &UA2734663},
	{SG13701, &UA2736484},
	{SG13702, &UA2764088},
	{SG13718, &UA2778785},
	{SG13719, &UA2779001},
	{SG13720, &UA2792168},
	{SG13721, &UA2793972},
	{SG13722, &UA2794497},
	{SG13723, &UA2795462},
	{SG13724, &UA2843116},
	{SG13726, &UA2844160},
	{SG13727, &UA2847476},
	{SG13728, &UA2848331},
	{SG13729, &UA2853504},
	{SG13730, &UA2855002},
	{SG13731, &UA2855216},
	{SG13732, &UA2858551},
	{SG13733, &UA2865188},
	{SG13734, &UA2865618},
	{SG13735, &UA2866486},
	{SG13739, &UA2935981},
	{SG13747, &UA2938129},
	{SG13756, &UA2951043},
	{SG13764, &UA2952176},
	{SG13767, &UA2952893},
	{SG13787, &UA2955258},
	{SG13796, &UA2956621},
	{SG13816, &UA2983638},
	{SG13831, &UA2986066},
	{SG13835, &UA2986299},
	{SG13840, &UA2987129},
	{SG13841, &UA2989060},
	{SG13842, &UA2989408},
	{SG13850, &UA2994102},
	{SG13851, &UA2996388},
	{SG13852, &UA2998138},
	{SG13853, &UA2999245},
	{SG13854, &UA2999808},
	{SG13855, &UA3000590},
	{SG13856, &UA3004482},
	{SG13857, &UA3010988},
	{SG13858, &UA3012647},
	{SG13860, &UA3014267},
	{SG13866, &UA3024960},
	{SG13873, &UA3043760},
	{SG13874, &UA3048652},
	{SG13875, &UA3048687},
	{SG13876, &UA3049536},
	{SG13889, &UA3076914},
	{SG13892, &UA3077987},
	{SG13908, &UA3086732},
	{SG13917, &UA3088867},
	{SG13935, &UA3100355},
	{SG13936, &UA3100866},
	{SG13937, &UA3103034},
	{SG13938, &UA3103172},
	{SG13939, &UA3108196},
	{SG13945, &UA3118760},
	{SG13955, &UA3121336},
	{SG13995, &UA2486359},
	{SG14012, &UA2729489},
	{SG14018, &UA2897408},
	{SG14021, &UA2936264},
	{SG14026, &UA2984251},
	{SG14030, &UA3073424},
	{SG14946, &UA2700184},
	{SG15048, &UA731233},
	{SG15049, &UA2361986},
	{SG15050, &UA2367999},
	{SG15051, &UA2729171},
	{SG15681, &UA369915},
	{SG16305, &UA363591},
	{SG16306, &UA363657},
	{SG16325, &UA2026737},
	{SG16358, &UA2625411},
	{SG16600, &UA1941599},
	{SG16921, &UA348823},
	{SG16923, &UA351190},
	{SG16924, &UA352437},
	{SG16929, &UA633291},
	{SG16930, &UA745940},
	{SG16931, &UA761323},
	{SG16932, &UA1936189},
	{SG16933, &UA1940967},
	{SG16934, &UA2020675},
	{SG16935, &UA2020999},
	{SG17067, &UA2759507},
	{SG17098, &UA402844},
	{SG17099, &UA402889},
	{SG17188, &UA370813},
	{SG17191, &UA1948356},
	{SG17377, &UA1930546},
	{SG17412, &UA2856889},
	{SG17530, &UA251746},
	{SG17936, &UA2815694},
	{SG18008, &UA2022709},
	{SG18119, &UA257517},
	{SG18120, &UA347043},
	{SG18121, &UA1903215},
	{SG18184, &UA609388},
	{SG18187, &UA887347},
	{SG18188, &UA2759724},
	{SG18440, &UA2775778},
	{SG18466, &UA690782},
	{SG18467, &UA1948174},
	{SG18468, &UA2028554},
	{SG18469, &UA2028749},
	{SG18576, &UA1987958},
	{SG18708, &UA2028523},
	{SG18756, &UA2848598},
	{SG18811, &UA2581892},
	{SG18849, &UA347546},
	{SG18909, &UA683277},
	{SG19211, &UA350741},
	{SG19429, &UA3030020},
	{SG19450, &UA2402151},
	{SG19519, &UA2997775},
	{SG19520, &UA356849},
	{SG19789, &UA2431354},
	{SG19832, &UA2444625},
	{SG19852, &UA2752721},
	{SG19878, &UA2740485},
	{SG19954, &UA2031359},
	{SG19974, &UA2667364},
	{SG20274, &UA2752710},
	{SG20332, &UA2739975},
	{SG20349, &UA349415},
	{SG20392, &UA751218},
	{SG20398, &UA359546},
	{SG20454, &UA2560012},
	{SG20496, &UA2987485},
	{SG20634, &UA2604720},
	{SG20724, &UA2751113},
	{SG20807, &UA2770926},
	{SG20863, &UA2706894},
	{SG20977, &UA2696231},
	{SG21004, &UA2943699},
	{SG21026, &UA380714},
	{SG21046, &UA3023832},
	{SG21109, &UA2242215},
	{SG21110, &UA2348526},
	{SG21244, &UA2958340},
	{SG21268, &UA2395706},
	{SG21351, &UA2770411},
	{SG21355, &UA2734933},
	{SG21383, &UA2022674},
	{SG21483, &UA3093652},
	{SG21488, &UA2532307},
	{SG21524, &UA2894396},
	{SG21550, &UA2664694},
	{SG21618, &UA2733600},
	{SG21619, &UA351440},
	{SG21624, &UA2768520},
	{SG21668, &UA2737615},
	{SG21736, &UA2557212},
	{SG21738, &UA856676},
	{SG21846, &UA2422730},
	{SG21877, &UA2729907},
	{SG21943, &UA645544},
	{SG21956, &UA2955886},
	{SG22045, &UA2701966},
	{SG22080, &UA2893022},
	{SG22127, &UA2340184},
	{SG22142, &UA2527284},
	{SG22421, &UA723322},
	{SG22575, &UA2968783},
	{SG22642, &UA2768698},
	{SG22775, &UA3042949},
	{SG22827, &UA399047},
	{SG22831, &UA2579815},
	{SG23041, &UA2595878},
	{SG23109, &UA3004366},
	{SG23112, &UA2849418},
	{SG23148, &UA350585},
	{SG23149, &UA681586},
	{SG23151, &UA812834},
	{SG23152, &UA2022397},
	{SG23180, &UA1917381},
	{SG23208, &UA2891706},
	{SG23325, &UA660499},
	{SG23327, &UA633092},
	{SG23337, &UA2848314},
	{SG23388, &UA655693},
	{SG23407, &UA719343},
	{SG23456, &UA348162},
	{SG25863, &UA716053},
	{SG25954, &UA2024500},
	{NULL}};
const Segmentdata DAndroidVersion4_0_2[] = {
	{SG13725, &UA2024400},
	{SG14201, &UA2496856},
	{SG14455, &UA2781418},
	{SG14554, &UA2879763},
	{SG14566, &UA2934394},
	{SG14661, &UA2993840},
	{SG14672, &UA2997654},
	{SG14709, &UA3057220},
	{SG17170, &UA350024},
	{SG17171, &UA352977},
	{SG17172, &UA357484},
	{SG17173, &UA370415},
	{SG17174, &UA687777},
	{SG17175, &UA1956564},
	{SG17176, &UA2034497},
	{SG17177, &UA2396054},
	{SG17178, &UA2444592},
	{SG17559, &UA2759237},
	{SG17869, &UA358871},
	{SG17870, &UA2021922},
	{SG17871, &UA2474388},
	{SG17872, &UA2761885},
	{SG18463, &UA459318},
	{SG18464, &UA616645},
	{SG18465, &UA898065},
	{SG18899, &UA834036},
	{SG19155, &UA360475},
	{SG19713, &UA2729343},
	{SG20909, &UA2584857},
	{SG21124, &UA2769377},
	{SG21215, &UA2711339},
	{SG21408, &UA3023531},
	{SG21835, &UA3094707},
	{SG23096, &UA382217},
	{NULL}};
const Segmentdata DAndroidVersion4_0_3[] = {
	{SG13736, &UA366276},
	{SG13737, &UA367771},
	{SG13738, &UA368873},
	{SG13740, &UA605331},
	{SG13741, &UA605436},
	{SG13742, &UA606701},
	{SG13743, &UA613070},
	{SG13744, &UA616633},
	{SG13745, &UA616638},
	{SG13746, &UA622415},
	{SG13748, &UA633579},
	{SG13749, &UA637003},
	{SG13750, &UA739721},
	{SG13751, &UA751862},
	{SG13752, &UA752290},
	{SG13753, &UA771131},
	{SG13754, &UA803901},
	{SG13755, &UA866403},
	{SG13757, &UA899763},
	{SG13758, &UA899899},
	{SG13759, &UA904383},
	{SG13760, &UA911102},
	{SG13761, &UA912224},
	{SG13762, &UA913295},
	{SG13763, &UA1973690},
	{SG13765, &UA2027880},
	{SG13766, &UA2028739},
	{SG13768, &UA2032359},
	{SG13769, &UA2032884},
	{SG13771, &UA2225143},
	{SG13774, &UA2257402},
	{SG13776, &UA2396471},
	{SG13777, &UA2445081},
	{SG13778, &UA2499807},
	{SG13780, &UA2518693},
	{SG13781, &UA2519699},
	{SG13784, &UA2578017},
	{SG13790, &UA2848023},
	{SG13793, &UA3086237},
	{SG13940, &UA359568},
	{SG13941, &UA363240},
	{SG13942, &UA369773},
	{SG13943, &UA600388},
	{SG13944, &UA629680},
	{SG13946, &UA640525},
	{SG13947, &UA649305},
	{SG13948, &UA649663},
	{SG13949, &UA666317},
	{SG13950, &UA691645},
	{SG13951, &UA707628},
	{SG13952, &UA710076},
	{SG13953, &UA723480},
	{SG13954, &UA748594},
	{SG13956, &UA780201},
	{SG13957, &UA783070},
	{SG13958, &UA789264},
	{SG13959, &UA791123},
	{SG13960, &UA794661},
	{SG13961, &UA809074},
	{SG13962, &UA818875},
	{SG13963, &UA864900},
	{SG13964, &UA872237},
	{SG13965, &UA895002},
	{SG13966, &UA896339},
	{SG13967, &UA898999},
	{SG13968, &UA899013},
	{SG13969, &UA899616},
	{SG13970, &UA902027},
	{SG13971, &UA902940},
	{SG13972, &UA907184},
	{SG13973, &UA908853},
	{SG13974, &UA1952249},
	{SG13975, &UA1952823},
	{SG13976, &UA1953673},
	{SG13977, &UA1953945},
	{SG13978, &UA1954809},
	{SG13979, &UA1957822},
	{SG13980, &UA1958412},
	{SG13981, &UA1960712},
	{SG13982, &UA1960750},
	{SG13983, &UA1961845},
	{SG13984, &UA1961922},
	{SG13985, &UA1962988},
	{SG13986, &UA1963061},
	{SG13988, &UA1963889},
	{SG13990, &UA1964715},
	{SG13991, &UA1966528},
	{SG13993, &UA1966551},
	{SG13996, &UA1967525},
	{SG13998, &UA1967823},
	{SG14000, &UA1967862},
	{SG14002, &UA1970043},
	{SG14004, &UA1970115},
	{SG14006, &UA1970432},
	{SG14008, &UA1975690},
	{SG14010, &UA1975938},
	{SG14014, &UA1981455},
	{SG14017, &UA1981545},
	{SG14020, &UA1984337},
	{SG14022, &UA1984510},
	{SG14024, &UA1985273},
	{SG14027, &UA1988396},
	{SG14029, &UA1988492},
	{SG14032, &UA1988592},
	{SG14034, &UA1989333},
	{SG14037, &UA1993970},
	{SG14039, &UA1994179},
	{SG14041, &UA1994435},
	{SG14042, &UA1994851},
	{SG14045, &UA1995081},
	{SG14047, &UA1998773},
	{SG14054, &UA2023387},
	{SG14055, &UA2023712},
	{SG14056, &UA2023956},
	{SG14057, &UA2024095},
	{SG14058, &UA2024303},
	{SG14059, &UA2024527},
	{SG14060, &UA2024625},
	{SG14061, &UA2024639},
	{SG14062, &UA2024739},
	{SG14063, &UA2024778},
	{SG14064, &UA2024792},
	{SG14065, &UA2024990},
	{SG14066, &UA2025021},
	{SG14067, &UA2025197},
	{SG14068, &UA2025206},
	{SG14069, &UA2025365},
	{SG14070, &UA2025505},
	{SG14071, &UA2025719},
	{SG14072, &UA2025720},
	{SG14073, &UA2025788},
	{SG14074, &UA2025815},
	{SG14075, &UA2025820},
	{SG14076, &UA2025861},
	{SG14077, &UA2025884},
	{SG14078, &UA2025926},
	{SG14079, &UA2026079},
	{SG14081, &UA2026082},
	{SG14083, &UA2026120},
	{SG14085, &UA2026180},
	{SG14086, &UA2026286},
	{SG14089, &UA2026397},
	{SG14091, &UA2026428},
	{SG14093, &UA2026495},
	{SG14095, &UA2026502},
	{SG14097, &UA2026578},
	{SG14099, &UA2026603},
	{SG14100, &UA2026628},
	{SG14102, &UA2026701},
	{SG14104, &UA2026755},
	{SG14106, &UA2026833},
	{SG14108, &UA2026937},
	{SG14110, &UA2026949},
	{SG14112, &UA2026966},
	{SG14114, &UA2027096},
	{SG14116, &UA2027132},
	{SG14118, &UA2027133},
	{SG14120, &UA2027267},
	{SG14122, &UA2027284},
	{SG14124, &UA2027398},
	{SG14125, &UA2027482},
	{SG14127, &UA2027552},
	{SG14129, &UA2027580},
	{SG14131, &UA2027583},
	{SG14133, &UA2027608},
	{SG14135, &UA2027714},
	{SG14137, &UA2027759},
	{SG14140, &UA2027882},
	{SG14142, &UA2028044},
	{SG14143, &UA2028074},
	{SG14145, &UA2028081},
	{SG14146, &UA2028112},
	{SG14148, &UA2028356},
	{SG14150, &UA2028451},
	{SG14151, &UA2028598},
	{SG14152, &UA2028676},
	{SG14153, &UA2028704},
	{SG14154, &UA2028708},
	{SG14155, &UA2028764},
	{SG14157, &UA2029031},
	{SG14158, &UA2029228},
	{SG14159, &UA2029237},
	{SG14161, &UA2029551},
	{SG14163, &UA2029924},
	{SG14164, &UA2029915},
	{SG14165, &UA2029978},
	{SG14166, &UA2030381},
	{SG14167, &UA2030638},
	{SG14169, &UA2030677},
	{SG14170, &UA2030850},
	{SG14171, &UA2031008},
	{SG14172, &UA2031144},
	{SG14173, &UA2031302},
	{SG14174, &UA2031413},
	{SG14175, &UA2031467},
	{SG14176, &UA2031592},
	{SG14178, &UA2031782},
	{SG14179, &UA2031857},
	{SG14180, &UA2031877},
	{SG14181, &UA2032087},
	{SG14182, &UA2032410},
	{SG14183, &UA2032415},
	{SG14184, &UA2032554},
	{SG14185, &UA2032787},
	{SG14186, &UA2032967},
	{SG14187, &UA2032976},
	{SG14188, &UA2033066},
	{SG14189, &UA2033076},
	{SG14190, &UA2033194},
	{SG14191, &UA2033459},
	{SG14192, &UA2033462},
	{SG14193, &UA2033565},
	{SG14194, &UA2033607},
	{SG14195, &UA2033734},
	{SG14196, &UA2033824},
	{SG14197, &UA2033845},
	{SG14198, &UA2034106},
	{SG14199, &UA2034119},
	{SG14200, &UA2034122},
	{SG14202, &UA2034152},
	{SG14203, &UA2034256},
	{SG14204, &UA2034349},
	{SG14205, &UA2167145},
	{SG14207, &UA2167588},
	{SG14208, &UA2168677},
	{SG14209, &UA2169529},
	{SG14210, &UA2176718},
	{SG14212, &UA2206063},
	{SG14213, &UA2219399},
	{SG14214, &UA2223989},
	{SG14215, &UA2224089},
	{SG14217, &UA2225207},
	{SG14218, &UA2226000},
	{SG14219, &UA2229030},
	{SG14220, &UA2229656},
	{SG14221, &UA2257216},
	{SG14222, &UA2257257},
	{SG14223, &UA2257261},
	{SG14224, &UA2257281},
	{SG14225, &UA2257292},
	{SG14226, &UA2257344},
	{SG14227, &UA2257467},
	{SG14228, &UA2257389},
	{SG14229, &UA2257477},
	{SG14230, &UA2329739},
	{SG14231, &UA2335195},
	{SG14232, &UA2336347},
	{SG14233, &UA2337116},
	{SG14234, &UA2337540},
	{SG14235, &UA2338309},
	{SG14236, &UA2338432},
	{SG14237, &UA2338492},
	{SG14238, &UA2339146},
	{SG14239, &UA2339492},
	{SG14240, &UA2340571},
	{SG14241, &UA2340982},
	{SG14242, &UA2343823},
	{SG14243, &UA2346900},
	{SG14244, &UA2346992},
	{SG14245, &UA2347152},
	{SG14246, &UA2350684},
	{SG14248, &UA2352832},
	{SG14251, &UA2410947},
	{SG14254, &UA2411032},
	{SG14256, &UA2411397},
	{SG14257, &UA2411409},
	{SG14259, &UA2411810},
	{SG14261, &UA2412291},
	{SG14263, &UA2412355},
	{SG14265, &UA2413059},
	{SG14267, &UA2413088},
	{SG14269, &UA2413319},
	{SG14271, &UA2413439},
	{SG14273, &UA2413877},
	{SG14275, &UA2413975},
	{SG14277, &UA2414908},
	{SG14279, &UA2415474},
	{SG14280, &UA2416401},
	{SG14281, &UA2416499},
	{SG14282, &UA2416599},
	{SG14283, &UA2417741},
	{SG14284, &UA2419865},
	{SG14285, &UA2419909},
	{SG14286, &UA2421007},
	{SG14287, &UA2421100},
	{SG14288, &UA2421230},
	{SG14289, &UA2422416},
	{SG14290, &UA2422618},
	{SG14291, &UA2423231},
	{SG14292, &UA2423806},
	{SG14293, &UA2423807},
	{SG14294, &UA2424276},
	{SG14295, &UA2426152},
	{SG14296, &UA2426344},
	{SG14297, &UA2427460},
	{SG14298, &UA2428419},
	{SG14299, &UA2428987},
	{SG14300, &UA2432698},
	{SG14301, &UA2441675},
	{SG14302, &UA2442674},
	{SG14303, &UA2442814},
	{SG14304, &UA2443279},
	{SG14305, &UA2445359},
	{SG14306, &UA2445412},
	{SG14307, &UA2445547},
	{SG14308, &UA2446734},
	{SG14309, &UA2447614},
	{SG14310, &UA2449027},
	{SG14311, &UA2449166},
	{SG14312, &UA2450377},
	{SG14313, &UA2450523},
	{SG14314, &UA2453117},
	{SG14315, &UA2453425},
	{SG14316, &UA2453639},
	{SG14317, &UA2453724},
	{SG14318, &UA2455457},
	{SG14319, &UA2456662},
	{SG14320, &UA2456804},
	{SG14321, &UA2457557},
	{SG14322, &UA2458772},
	{SG14323, &UA2459724},
	{SG14324, &UA2460394},
	{SG14325, &UA2461933},
	{SG14326, &UA2463296},
	{SG14327, &UA2463810},
	{SG14328, &UA2464168},
	{SG14329, &UA2464608},
	{SG14330, &UA2464618},
	{SG14331, &UA2465578},
	{SG14332, &UA2465783},
	{SG14333, &UA2466040},
	{SG14334, &UA2466484},
	{SG14335, &UA2466538},
	{SG14336, &UA2467194},
	{SG14337, &UA2467565},
	{SG14338, &UA2467952},
	{SG14339, &UA2469309},
	{SG14340, &UA2469377},
	{SG14341, &UA2474751},
	{SG14342, &UA2474800},
	{SG14343, &UA2475882},
	{SG14344, &UA2475956},
	{SG14345, &UA2476957},
	{SG14346, &UA2478273},
	{SG14347, &UA2478607},
	{SG14348, &UA2480010},
	{SG14349, &UA2480327},
	{SG14350, &UA2480797},
	{SG14351, &UA2480943},
	{SG14352, &UA2481294},
	{SG14353, &UA2481407},
	{SG14354, &UA2482804},
	{SG14355, &UA2482930},
	{SG14356, &UA2483100},
	{SG14357, &UA2483652},
	{SG14359, &UA2483898},
	{SG14361, &UA2484460},
	{SG14362, &UA2484525},
	{SG14363, &UA2486528},
	{SG14364, &UA2486569},
	{SG14365, &UA2487328},
	{SG14366, &UA2487682},
	{SG14367, &UA2494699},
	{SG14368, &UA2495778},
	{SG14369, &UA2496522},
	{SG14370, &UA2496862},
	{SG14371, &UA2497014},
	{SG14372, &UA2497161},
	{SG14373, &UA2498790},
	{SG14374, &UA2498896},
	{SG14375, &UA2498915},
	{SG14376, &UA2499655},
	{SG14377, &UA2499676},
	{SG14378, &UA2500191},
	{SG14379, &UA2502138},
	{SG14380, &UA2502458},
	{SG14381, &UA2502555},
	{SG14382, &UA2502639},
	{SG14383, &UA2502734},
	{SG14384, &UA2503188},
	{SG14385, &UA2503238},
	{SG14386, &UA2504920},
	{SG14387, &UA2505189},
	{SG14388, &UA2506904},
	{SG14389, &UA2518781},
	{SG14390, &UA2518967},
	{SG14391, &UA2518975},
	{SG14392, &UA2519948},
	{SG14393, &UA2520087},
	{SG14394, &UA2520438},
	{SG14395, &UA2520677},
	{SG14396, &UA2520731},
	{SG14397, &UA2520822},
	{SG14398, &UA2520942},
	{SG14399, &UA2524460},
	{SG14400, &UA2524645},
	{SG14401, &UA2524919},
	{SG14402, &UA2525967},
	{SG14403, &UA2526121},
	{SG14404, &UA2526486},
	{SG14405, &UA2526925},
	{SG14406, &UA2526992},
	{SG14407, &UA2527282},
	{SG14408, &UA2527873},
	{SG14409, &UA2527891},
	{SG14410, &UA2527909},
	{SG14411, &UA2528154},
	{SG14412, &UA2529484},
	{SG14413, &UA2529719},
	{SG14414, &UA2529963},
	{SG14415, &UA2530021},
	{SG14416, &UA2530600},
	{SG14417, &UA2530813},
	{SG14418, &UA2531146},
	{SG14419, &UA2543761},
	{SG14420, &UA2545162},
	{SG14421, &UA2545363},
	{SG14422, &UA2545800},
	{SG14423, &UA2546498},
	{SG14424, &UA2547436},
	{SG14425, &UA2548010},
	{SG14426, &UA2549457},
	{SG14427, &UA2549991},
	{SG14428, &UA2551985},
	{SG14429, &UA2552236},
	{SG14430, &UA2552814},
	{SG14431, &UA2552911},
	{SG14432, &UA2553062},
	{SG14433, &UA2553092},
	{SG14436, &UA2553232},
	{SG14437, &UA2553250},
	{SG14438, &UA2553283},
	{SG14440, &UA2553998},
	{SG14441, &UA2554217},
	{SG14443, &UA2555199},
	{SG14444, &UA2555237},
	{SG14446, &UA2555451},
	{SG14449, &UA2556026},
	{SG14450, &UA2556028},
	{SG14452, &UA2556719},
	{SG14453, &UA2557306},
	{SG14454, &UA2557436},
	{SG14456, &UA2558913},
	{SG14457, &UA2574294},
	{SG14459, &UA2574350},
	{SG14461, &UA2574603},
	{SG14462, &UA2575341},
	{SG14463, &UA2576565},
	{SG14465, &UA2577669},
	{SG14466, &UA2577945},
	{SG14467, &UA2578009},
	{SG14468, &UA2578116},
	{SG14470, &UA2578158},
	{SG14473, &UA2580185},
	{SG14485, &UA2580896},
	{SG14486, &UA2582780},
	{SG14487, &UA2588153},
	{SG14488, &UA2588417},
	{SG14489, &UA2589047},
	{SG14490, &UA2589167},
	{SG14491, &UA2589284},
	{SG14492, &UA2589626},
	{SG14493, &UA2590111},
	{SG14494, &UA2590482},
	{SG14495, &UA2590577},
	{SG14496, &UA2590654},
	{SG14498, &UA2591506},
	{SG14499, &UA2593403},
	{SG14500, &UA2594235},
	{SG14501, &UA2594353},
	{SG14502, &UA2594815},
	{SG14503, &UA2594887},
	{SG14504, &UA2595630},
	{SG14505, &UA2596498},
	{SG14506, &UA2597182},
	{SG14510, &UA2599160},
	{SG14512, &UA2602472},
	{SG14513, &UA2603032},
	{SG14514, &UA2605332},
	{SG14515, &UA2640847},
	{SG14516, &UA2641233},
	{SG14517, &UA2641602},
	{SG14518, &UA2642738},
	{SG14519, &UA2642947},
	{SG14520, &UA2643170},
	{SG14521, &UA2643385},
	{SG14522, &UA2644309},
	{SG14523, &UA2644491},
	{SG14524, &UA2644639},
	{SG14525, &UA2645648},
	{SG14527, &UA2645681},
	{SG14528, &UA2646777},
	{SG14529, &UA2650274},
	{SG14530, &UA2650877},
	{SG14531, &UA2651198},
	{SG14534, &UA2652995},
	{SG14535, &UA2653334},
	{SG14537, &UA2653380},
	{SG14538, &UA2653614},
	{SG14540, &UA2654024},
	{SG14541, &UA2654051},
	{SG14543, &UA2654307},
	{SG14544, &UA2654353},
	{SG14546, &UA2655267},
	{SG14547, &UA2656171},
	{SG14548, &UA2656604},
	{SG14550, &UA2656831},
	{SG14551, &UA2657350},
	{SG14552, &UA2657691},
	{SG14557, &UA2658902},
	{SG14559, &UA2659082},
	{SG14560, &UA2659690},
	{SG14564, &UA2664153},
	{SG14567, &UA2684343},
	{SG14568, &UA2684992},
	{SG14569, &UA2685487},
	{SG14570, &UA2685841},
	{SG14571, &UA2686093},
	{SG14572, &UA2686139},
	{SG14573, &UA2686246},
	{SG14574, &UA2686694},
	{SG14575, &UA2686832},
	{SG14577, &UA2687304},
	{SG14578, &UA2687796},
	{SG14580, &UA2688191},
	{SG14582, &UA2692230},
	{SG14584, &UA2692327},
	{SG14590, &UA2692858},
	{SG14591, &UA2692925},
	{SG14592, &UA2693338},
	{SG14593, &UA2693406},
	{SG14595, &UA2693723},
	{SG14596, &UA2693824},
	{SG14597, &UA2693894},
	{SG14598, &UA2693904},
	{SG14599, &UA2693905},
	{SG14601, &UA2693961},
	{SG14602, &UA2694091},
	{SG14603, &UA2694219},
	{SG14604, &UA2694592},
	{SG14606, &UA2694766},
	{SG14607, &UA2695023},
	{SG14609, &UA2695550},
	{SG14611, &UA2695983},
	{SG14613, &UA2696235},
	{SG14614, &UA2696238},
	{SG14615, &UA2696292},
	{SG14616, &UA2696661},
	{SG14618, &UA2696828},
	{SG14619, &UA2696870},
	{SG14620, &UA2696903},
	{SG14622, &UA2697275},
	{SG14624, &UA2697902},
	{SG14626, &UA2699397},
	{SG14627, &UA2699807},
	{SG14628, &UA2700794},
	{SG14633, &UA2721398},
	{SG14634, &UA2722504},
	{SG14635, &UA2722952},
	{SG14637, &UA2722962},
	{SG14638, &UA2723492},
	{SG14639, &UA2723515},
	{SG14641, &UA2723823},
	{SG14642, &UA2724133},
	{SG14644, &UA2725365},
	{SG14645, &UA2725512},
	{SG14646, &UA2726455},
	{SG14648, &UA2726552},
	{SG14649, &UA2727211},
	{SG14651, &UA2727398},
	{SG14652, &UA2728012},
	{SG14654, &UA2728483},
	{SG14655, &UA2728853},
	{SG14656, &UA2729642},
	{SG14657, &UA2729943},
	{SG14659, &UA2731683},
	{SG14663, &UA2732733},
	{SG14664, &UA2735132},
	{SG14665, &UA2742939},
	{SG14666, &UA2745942},
	{SG14668, &UA2746294},
	{SG14669, &UA2746326},
	{SG14670, &UA2746417},
	{SG14671, &UA2746578},
	{SG14674, &UA2747363},
	{SG14675, &UA2747413},
	{SG14678, &UA2747675},
	{SG14679, &UA2747995},
	{SG14680, &UA2748144},
	{SG14681, &UA2748607},
	{SG14684, &UA2758316},
	{SG14685, &UA2758386},
	{SG14686, &UA2758734},
	{SG14687, &UA2759125},
	{SG14689, &UA2759191},
	{SG14690, &UA2759332},
	{SG14691, &UA2759843},
	{SG14693, &UA2760106},
	{SG14694, &UA2761230},
	{SG14695, &UA2761607},
	{SG14696, &UA2761747},
	{SG14698, &UA2762154},
	{SG14699, &UA2762183},
	{SG14702, &UA2762902},
	{SG14703, &UA2762941},
	{SG14705, &UA2764521},
	{SG14722, &UA2767335},
	{SG14723, &UA2767857},
	{SG14724, &UA2775103},
	{SG14726, &UA2775708},
	{SG14727, &UA2776036},
	{SG14729, &UA2777332},
	{SG14730, &UA2777790},
	{SG14732, &UA2778370},
	{SG14733, &UA2778792},
	{SG14734, &UA2779129},
	{SG14735, &UA2779279},
	{SG14736, &UA2779878},
	{SG14738, &UA2786898},
	{SG14739, &UA2787305},
	{SG14740, &UA2788133},
	{SG14741, &UA2788285},
	{SG14742, &UA2789404},
	{SG14743, &UA2789999},
	{SG14745, &UA2790014},
	{SG14746, &UA2790035},
	{SG14749, &UA2791114},
	{SG14750, &UA2791312},
	{SG14751, &UA2791330},
	{SG14752, &UA2791518},
	{SG14753, &UA2791541},
	{SG14755, &UA2791584},
	{SG14756, &UA2791627},
	{SG14757, &UA2792512},
	{SG14758, &UA2792942},
	{SG14759, &UA2793953},
	{SG14762, &UA2809080},
	{SG14763, &UA2810085},
	{SG14764, &UA2810379},
	{SG14766, &UA2811220},
	{SG14767, &UA2811315},
	{SG14769, &UA2813123},
	{SG14770, &UA2814263},
	{SG14771, &UA2814902},
	{SG14772, &UA2815194},
	{SG14773, &UA2815235},
	{SG14774, &UA2815893},
	{SG14776, &UA2816366},
	{SG14778, &UA2816506},
	{SG14779, &UA2816902},
	{SG14780, &UA2817015},
	{SG14784, &UA2817198},
	{SG14785, &UA2817256},
	{SG14787, &UA2818144},
	{SG14789, &UA2818277},
	{SG14791, &UA2818454},
	{SG14793, &UA2818688},
	{SG14795, &UA2818841},
	{SG14796, &UA2819022},
	{SG14798, &UA2820120},
	{SG14804, &UA2822790},
	{SG14805, &UA2824459},
	{SG14806, &UA2824614},
	{SG14807, &UA2825043},
	{SG14808, &UA2825927},
	{SG14809, &UA2830362},
	{SG14810, &UA2831320},
	{SG14811, &UA2831516},
	{SG14812, &UA2832183},
	{SG14813, &UA2835166},
	{SG14814, &UA2835238},
	{SG14815, &UA2835662},
	{SG14816, &UA2835976},
	{SG14817, &UA2836060},
	{SG14818, &UA2836224},
	{SG14819, &UA2837667},
	{SG14820, &UA2838315},
	{SG14821, &UA2841292},
	{SG14822, &UA2843276},
	{SG14823, &UA2844613},
	{SG14824, &UA2846140},
	{SG14825, &UA2846147},
	{SG14826, &UA2847675},
	{SG14827, &UA2847978},
	{SG14828, &UA2848144},
	{SG14829, &UA2848248},
	{SG14830, &UA2848808},
	{SG14831, &UA2854467},
	{SG14832, &UA2855292},
	{SG14833, &UA2857447},
	{SG14834, &UA2857819},
	{SG14835, &UA2858904},
	{SG14836, &UA2859905},
	{SG14837, &UA2863216},
	{SG14838, &UA2863392},
	{SG14839, &UA2863517},
	{SG14840, &UA2864293},
	{SG14841, &UA2878983},
	{SG14842, &UA2889002},
	{SG14843, &UA2889123},
	{SG14844, &UA2889661},
	{SG14845, &UA2890425},
	{SG14846, &UA2891509},
	{SG14847, &UA2933749},
	{SG14848, &UA2933896},
	{SG14849, &UA2934491},
	{SG14850, &UA2934497},
	{SG14851, &UA2934870},
	{SG14852, &UA2935180},
	{SG14853, &UA2949832},
	{SG14854, &UA2950080},
	{SG14855, &UA2950270},
	{SG14856, &UA2950363},
	{SG14857, &UA2950439},
	{SG14858, &UA2950672},
	{SG14859, &UA2950835},
	{SG14860, &UA2951144},
	{SG14861, &UA2951240},
	{SG14862, &UA2951596},
	{SG14863, &UA2952499},
	{SG14864, &UA2952865},
	{SG14865, &UA2953825},
	{SG14866, &UA2954126},
	{SG14867, &UA2955338},
	{SG14868, &UA2957007},
	{SG14869, &UA2958186},
	{SG14870, &UA2980328},
	{SG14871, &UA2980882},
	{SG14872, &UA2980891},
	{SG14873, &UA2981129},
	{SG14874, &UA2981495},
	{SG14875, &UA2981621},
	{SG14876, &UA2982055},
	{SG14877, &UA2982246},
	{SG14878, &UA2982867},
	{SG14879, &UA2983140},
	{SG14880, &UA2983180},
	{SG14881, &UA2983772},
	{SG14882, &UA2985485},
	{SG14883, &UA2986296},
	{SG14884, &UA2987081},
	{SG14885, &UA2987358},
	{SG14886, &UA2987384},
	{SG14887, &UA2987410},
	{SG14888, &UA2988090},
	{SG14893, &UA2991980},
	{SG14896, &UA2995268},
	{SG14898, &UA2996034},
	{SG14901, &UA2997226},
	{SG14905, &UA2998440},
	{SG14911, &UA3012783},
	{SG14912, &UA3014784},
	{SG14915, &UA3020083},
	{SG14919, &UA3042230},
	{SG14921, &UA3042564},
	{SG14922, &UA3042610},
	{SG14923, &UA3042903},
	{SG14930, &UA3046712},
	{SG14931, &UA3047101},
	{SG14932, &UA3050127},
	{SG14933, &UA3056667},
	{SG14934, &UA3069964},
	{SG14935, &UA3070214},
	{SG14936, &UA3070493},
	{SG14937, &UA3070580},
	{SG14938, &UA3070622},
	{SG14939, &UA3070763},
	{SG14940, &UA3071458},
	{SG14941, &UA3073311},
	{SG14942, &UA3077996},
	{SG14943, &UA3078400},
	{SG14944, &UA3078899},
	{SG14945, &UA3085323},
	{SG14947, &UA3086555},
	{SG14948, &UA3086668},
	{SG14949, &UA3086677},
	{SG14950, &UA3086695},
	{SG14951, &UA3098460},
	{SG14952, &UA3098479},
	{SG14953, &UA3098512},
	{SG14954, &UA3098730},
	{SG14964, &UA3099564},
	{SG14965, &UA3099663},
	{SG14966, &UA3099848},
	{SG14967, &UA3100094},
	{SG14968, &UA3100421},
	{SG14969, &UA3101287},
	{SG14970, &UA3101648},
	{SG14971, &UA3102270},
	{SG14972, &UA3102365},
	{SG14973, &UA3109138},
	{SG14974, &UA3117731},
	{SG14975, &UA3118424},
	{SG14976, &UA3118567},
	{SG14977, &UA3118958},
	{SG14978, &UA3120784},
	{SG14979, &UA3120966},
	{SG14980, &UA3121024},
	{SG14981, &UA3124061},
	{SG14982, &UA3124602},
	{SG14983, &UA459161},
	{SG14984, &UA2352475},
	{SG14985, &UA2425632},
	{SG14986, &UA2641006},
	{SG15052, &UA622429},
	{SG15053, &UA654182},
	{SG15054, &UA680098},
	{SG15055, &UA715673},
	{SG15056, &UA854191},
	{SG15057, &UA896018},
	{SG15058, &UA896360},
	{SG15059, &UA907875},
	{SG15060, &UA909553},
	{SG15061, &UA1960093},
	{SG15062, &UA1962066},
	{SG15063, &UA1990033},
	{SG15064, &UA1991916},
	{SG15065, &UA1994113},
	{SG15066, &UA1996488},
	{SG15067, &UA2026091},
	{SG15068, &UA2026200},
	{SG15069, &UA2033069},
	{SG15072, &UA2834753},
	{SG15166, &UA438853},
	{SG15167, &UA438854},
	{SG15168, &UA455742},
	{SG15169, &UA457682},
	{SG15170, &UA608772},
	{SG15171, &UA609609},
	{SG15173, &UA616671},
	{SG15175, &UA624882},
	{SG15177, &UA629752},
	{SG15178, &UA652719},
	{SG15180, &UA670489},
	{SG15181, &UA728771},
	{SG15183, &UA731741},
	{SG15185, &UA742163},
	{SG15186, &UA770144},
	{SG15187, &UA779067},
	{SG15188, &UA788455},
	{SG15189, &UA829164},
	{SG15190, &UA843964},
	{SG15191, &UA879107},
	{SG15192, &UA893382},
	{SG15193, &UA899119},
	{SG15194, &UA900471},
	{SG15195, &UA904118},
	{SG15198, &UA904980},
	{SG15199, &UA905057},
	{SG15200, &UA907881},
	{SG15201, &UA1953259},
	{SG15202, &UA1968073},
	{SG15203, &UA2023757},
	{SG15204, &UA2026567},
	{SG15205, &UA2031252},
	{SG15206, &UA2032138},
	{SG15207, &UA2032994},
	{SG15208, &UA2033706},
	{SG15209, &UA2518752},
	{SG15210, &UA2588949},
	{SG15211, &UA2596618},
	{SG15212, &UA2597194},
	{SG15213, &UA2599684},
	{SG15214, &UA2658125},
	{SG15215, &UA2658386},
	{SG15216, &UA2658892},
	{SG15217, &UA2659078},
	{SG15218, &UA2659203},
	{SG15219, &UA2814413},
	{SG15220, &UA2848956},
	{SG15221, &UA2849882},
	{SG15222, &UA2849959},
	{SG15223, &UA458589},
	{SG15224, &UA458595},
	{SG15225, &UA458600},
	{SG15226, &UA895400},
	{SG15227, &UA2168046},
	{SG15228, &UA2608658},
	{SG15229, &UA2610977},
	{SG15230, &UA2612082},
	{SG15231, &UA2613473},
	{SG15232, &UA2613736},
	{SG15233, &UA2614597},
	{SG15234, &UA2615056},
	{SG15235, &UA2615988},
	{SG15236, &UA2616484},
	{SG15237, &UA2616877},
	{SG15238, &UA2617396},
	{SG15239, &UA2617668},
	{SG15240, &UA2617740},
	{SG15241, &UA2617781},
	{SG15242, &UA2618417},
	{SG15243, &UA2619165},
	{SG15244, &UA2619183},
	{SG15245, &UA2619910},
	{SG15246, &UA2620933},
	{SG15247, &UA2621667},
	{SG15248, &UA2621711},
	{SG15249, &UA2622101},
	{SG15250, &UA2622404},
	{SG15251, &UA2623494},
	{SG15252, &UA2624232},
	{SG15253, &UA2624273},
	{SG15254, &UA2624303},
	{SG15255, &UA2624692},
	{SG15256, &UA2625055},
	{SG15257, &UA2625629},
	{SG15258, &UA2625894},
	{SG15259, &UA2627153},
	{SG15260, &UA2628274},
	{SG15261, &UA2628450},
	{SG15262, &UA2628914},
	{SG15263, &UA2629194},
	{SG15264, &UA2630234},
	{SG15265, &UA2630666},
	{SG15266, &UA2632159},
	{SG15267, &UA2633334},
	{SG15268, &UA2635513},
	{SG15269, &UA2637565},
	{SG15521, &UA361776},
	{SG15522, &UA366046},
	{SG15523, &UA397867},
	{SG15524, &UA422336},
	{SG15525, &UA437281},
	{SG15526, &UA583601},
	{SG15527, &UA605743},
	{SG15528, &UA607331},
	{SG15529, &UA616730},
	{SG15530, &UA636202},
	{SG15531, &UA636739},
	{SG15532, &UA668157},
	{SG15533, &UA674640},
	{SG15534, &UA678639},
	{SG15535, &UA684408},
	{SG15536, &UA725372},
	{SG15537, &UA729968},
	{SG15538, &UA740625},
	{SG15539, &UA749213},
	{SG15540, &UA796534},
	{SG15541, &UA799386},
	{SG15542, &UA834487},
	{SG15543, &UA850320},
	{SG15544, &UA855144},
	{SG15545, &UA888796},
	{SG15546, &UA896145},
	{SG15547, &UA899214},
	{SG15548, &UA900267},
	{SG15549, &UA900401},
	{SG15550, &UA905564},
	{SG15551, &UA910630},
	{SG15552, &UA1948372},
	{SG15553, &UA1952777},
	{SG15554, &UA1954526},
	{SG15555, &UA1954811},
	{SG15556, &UA1957654},
	{SG15557, &UA1959339},
	{SG15558, &UA1962483},
	{SG15559, &UA1965936},
	{SG15560, &UA1966676},
	{SG15561, &UA1971377},
	{SG15562, &UA1980618},
	{SG15563, &UA1981532},
	{SG15564, &UA1989901},
	{SG15566, &UA1990084},
	{SG15567, &UA1993781},
	{SG15570, &UA1995594},
	{SG15571, &UA2024294},
	{SG15572, &UA2024482},
	{SG15573, &UA2025854},
	{SG15574, &UA2025912},
	{SG15575, &UA2027522},
	{SG15576, &UA2029075},
	{SG15578, &UA2031011},
	{SG15580, &UA2031904},
	{SG15583, &UA2033089},
	{SG15585, &UA2033311},
	{SG15587, &UA2033633},
	{SG15589, &UA2034017},
	{SG15591, &UA2034028},
	{SG15593, &UA2034138},
	{SG15595, &UA2034395},
	{SG15597, &UA2175109},
	{SG15599, &UA2176353},
	{SG15603, &UA2204931},
	{SG15605, &UA2225413},
	{SG15607, &UA2225868},
	{SG15609, &UA2227739},
	{SG15612, &UA2257258},
	{SG15613, &UA2257291},
	{SG15616, &UA2338912},
	{SG15618, &UA2351700},
	{SG15622, &UA2360203},
	{SG15623, &UA2360365},
	{SG15625, &UA2360659},
	{SG15628, &UA2365956},
	{SG15629, &UA2366192},
	{SG15630, &UA2366371},
	{SG15631, &UA2369540},
	{SG15632, &UA2371628},
	{SG15633, &UA2372397},
	{SG15634, &UA2372818},
	{SG15635, &UA2373059},
	{SG15636, &UA2373336},
	{SG15637, &UA2373346},
	{SG15638, &UA2374738},
	{SG15639, &UA2375150},
	{SG15640, &UA2375157},
	{SG15641, &UA2375409},
	{SG15642, &UA2375430},
	{SG15643, &UA2375744},
	{SG15644, &UA2375917},
	{SG15645, &UA2375954},
	{SG15646, &UA2376157},
	{SG15647, &UA2376203},
	{SG15648, &UA2376677},
	{SG15649, &UA2379517},
	{SG15650, &UA2379737},
	{SG15651, &UA2381023},
	{SG15652, &UA2381724},
	{SG15653, &UA2381970},
	{SG15654, &UA2383096},
	{SG15655, &UA2383323},
	{SG15656, &UA2383632},
	{SG15657, &UA2384030},
	{SG15658, &UA2384431},
	{SG15659, &UA2384938},
	{SG15660, &UA2385272},
	{SG15661, &UA2390965},
	{SG15662, &UA2391287},
	{SG15663, &UA2393868},
	{SG15664, &UA2394886},
	{SG15665, &UA2395519},
	{SG15666, &UA2395735},
	{SG15667, &UA2395842},
	{SG15668, &UA2396258},
	{SG15669, &UA2396340},
	{SG15670, &UA2397052},
	{SG15671, &UA2397279},
	{SG15672, &UA2397718},
	{SG15673, &UA2397857},
	{SG15674, &UA2397997},
	{SG15675, &UA2398752},
	{SG15676, &UA2399058},
	{SG15677, &UA2399774},
	{SG15678, &UA2399844},
	{SG15679, &UA2400241},
	{SG15680, &UA2400280},
	{SG15682, &UA2400641},
	{SG15683, &UA2400750},
	{SG15684, &UA2401026},
	{SG15685, &UA2401030},
	{SG15686, &UA2401426},
	{SG15687, &UA2402371},
	{SG15688, &UA2405928},
	{SG15689, &UA2411943},
	{SG15690, &UA2413252},
	{SG15691, &UA2417106},
	{SG15692, &UA2423861},
	{SG15693, &UA2444913},
	{SG15694, &UA2458182},
	{SG15695, &UA2476316},
	{SG15696, &UA2480723},
	{SG15697, &UA2502052},
	{SG15698, &UA2502703},
	{SG15699, &UA2523323},
	{SG15700, &UA2589705},
	{SG15701, &UA2590847},
	{SG15702, &UA2592250},
	{SG15703, &UA2641018},
	{SG15704, &UA2645247},
	{SG15705, &UA2652689},
	{SG15706, &UA2654957},
	{SG15707, &UA2659269},
	{SG15708, &UA2659685},
	{SG15709, &UA2663434},
	{SG15710, &UA2666649},
	{SG15711, &UA2686225},
	{SG15712, &UA2695073},
	{SG15713, &UA2699134},
	{SG15714, &UA2721076},
	{SG15715, &UA2722880},
	{SG15716, &UA2788376},
	{SG15717, &UA2810121},
	{SG15718, &UA2810503},
	{SG15719, &UA2810902},
	{SG15720, &UA2827608},
	{SG15721, &UA2833662},
	{SG15722, &UA2850776},
	{SG15723, &UA2936002},
	{SG15724, &UA2950718},
	{SG15725, &UA2957474},
	{SG15726, &UA2980659},
	{SG15727, &UA2980818},
	{SG15728, &UA2980914},
	{SG15729, &UA2981017},
	{SG15730, &UA2981208},
	{SG15731, &UA3043285},
	{SG15732, &UA3057536},
	{SG15733, &UA3070133},
	{SG15734, &UA3117445},
	{SG15752, &UA1971335},
	{SG15753, &UA3069691},
	{SG15892, &UA438819},
	{SG15893, &UA438881},
	{SG15894, &UA588577},
	{SG15895, &UA603888},
	{SG15896, &UA609222},
	{SG15897, &UA622307},
	{SG15898, &UA625159},
	{SG15899, &UA630586},
	{SG15900, &UA657844},
	{SG15901, &UA665228},
	{SG15902, &UA667428},
	{SG15903, &UA720815},
	{SG15904, &UA766397},
	{SG15905, &UA810569},
	{SG15906, &UA863937},
	{SG15907, &UA895792},
	{SG15908, &UA899052},
	{SG15909, &UA903867},
	{SG15910, &UA1955234},
	{SG15911, &UA1960868},
	{SG15912, &UA1970361},
	{SG15913, &UA1973927},
	{SG15914, &UA1988147},
	{SG15915, &UA1993205},
	{SG15917, &UA2218806},
	{SG15919, &UA2234075},
	{SG15920, &UA2235191},
	{SG15923, &UA2497306},
	{SG15924, &UA2545584},
	{SG15926, &UA2597163},
	{SG15927, &UA2597686},
	{SG15928, &UA2597920},
	{SG15929, &UA2598495},
	{SG15930, &UA2658396},
	{SG15931, &UA2658979},
	{SG15932, &UA2777227},
	{SG15933, &UA2809988},
	{SG15934, &UA2850482},
	{SG15935, &UA2850665},
	{SG15955, &UA892469},
	{SG15956, &UA1987741},
	{SG15957, &UA2257301},
	{SG15958, &UA2573575},
	{SG15959, &UA2611383},
	{SG15960, &UA2611690},
	{SG15961, &UA2614914},
	{SG15962, &UA2615285},
	{SG15963, &UA2615680},
	{SG15964, &UA2616559},
	{SG15965, &UA2617001},
	{SG15966, &UA2618543},
	{SG15967, &UA2618549},
	{SG15968, &UA2618899},
	{SG15969, &UA2619171},
	{SG15970, &UA2620332},
	{SG15971, &UA2620553},
	{SG15972, &UA2620900},
	{SG15973, &UA2622516},
	{SG15977, &UA2623684},
	{SG15978, &UA2629665},
	{SG15980, &UA2633895},
	{SG16092, &UA438851},
	{SG16093, &UA438855},
	{SG16095, &UA662880},
	{SG16096, &UA663200},
	{SG16098, &UA789288},
	{SG16099, &UA900425},
	{SG16100, &UA900459},
	{SG16101, &UA906972},
	{SG16102, &UA910003},
	{SG16103, &UA910605},
	{SG16104, &UA923888},
	{SG16105, &UA1958521},
	{SG16106, &UA1961810},
	{SG16107, &UA1968002},
	{SG16108, &UA1973101},
	{SG16109, &UA1992321},
	{SG16110, &UA1995018},
	{SG16111, &UA2027536},
	{SG16112, &UA2030456},
	{SG16114, &UA2032999},
	{SG16116, &UA2180548},
	{SG16118, &UA2181920},
	{SG16120, &UA2188972},
	{SG16122, &UA2226447},
	{SG16124, &UA2362585},
	{SG16150, &UA2402424},
	{SG16151, &UA2410257},
	{SG16152, &UA2546521},
	{SG16154, &UA2686912},
	{SG16155, &UA2848966},
	{SG16156, &UA2957390},
	{SG16157, &UA3006053},
	{SG16172, &UA458623},
	{SG16184, &UA458627},
	{SG16187, &UA899198},
	{SG16189, &UA1959477},
	{SG16191, &UA1965389},
	{SG16205, &UA2230926},
	{SG16209, &UA2546898},
	{SG16211, &UA2608286},
	{SG16213, &UA2608972},
	{SG16214, &UA2609883},
	{SG16216, &UA2610660},
	{SG16218, &UA2611053},
	{SG16220, &UA2611661},
	{SG16223, &UA2614222},
	{SG16224, &UA2614314},
	{SG16233, &UA2618075},
	{SG16237, &UA2618803},
	{SG16239, &UA2619295},
	{SG16241, &UA2620237},
	{SG16242, &UA2620637},
	{SG16243, &UA2623553},
	{SG16244, &UA2624202},
	{SG16245, &UA2624704},
	{SG16246, &UA2627065},
	{SG16247, &UA2629245},
	{SG16248, &UA2631150},
	{SG16249, &UA2632367},
	{SG16250, &UA2633943},
	{SG16251, &UA2635158},
	{SG16262, &UA369126},
	{SG16263, &UA369381},
	{SG16265, &UA371206},
	{SG16267, &UA371230},
	{SG16268, &UA453341},
	{SG16269, &UA588605},
	{SG16271, &UA589253},
	{SG16273, &UA636921},
	{SG16274, &UA646722},
	{SG16275, &UA648881},
	{SG16277, &UA680278},
	{SG16278, &UA824689},
	{SG16279, &UA833249},
	{SG16280, &UA897004},
	{SG16281, &UA1950951},
	{SG16282, &UA1955903},
	{SG16283, &UA2023652},
	{SG16285, &UA2026921},
	{SG16286, &UA2026965},
	{SG16288, &UA2030201},
	{SG16291, &UA2443752},
	{SG16295, &UA2811949},
	{SG16296, &UA2813011},
	{SG16297, &UA2816774},
	{SG16383, &UA603679},
	{SG16384, &UA675217},
	{SG16385, &UA778459},
	{SG16386, &UA2024914},
	{SG16387, &UA2027173},
	{SG16388, &UA2029316},
	{SG16530, &UA1951688},
	{SG16531, &UA1961300},
	{SG16532, &UA1974408},
	{SG16533, &UA1978913},
	{SG16536, &UA1980055},
	{SG16537, &UA1993433},
	{SG16539, &UA2006929},
	{SG16541, &UA2025774},
	{SG16553, &UA2026844},
	{SG16554, &UA2028076},
	{SG16555, &UA2028675},
	{SG16556, &UA2029845},
	{SG16558, &UA2222426},
	{SG16559, &UA2224638},
	{SG16560, &UA2341731},
	{SG16561, &UA2375958},
	{SG16562, &UA2390360},
	{SG16563, &UA2433359},
	{SG16564, &UA2451123},
	{SG16565, &UA2454181},
	{SG16567, &UA2578188},
	{SG16568, &UA2584493},
	{SG16569, &UA2586003},
	{SG16570, &UA2694039},
	{SG16571, &UA2824939},
	{SG16579, &UA624153},
	{SG16580, &UA698634},
	{SG16581, &UA1962729},
	{SG16601, &UA605594},
	{SG16602, &UA899219},
	{SG16603, &UA909312},
	{SG16604, &UA2003512},
	{SG16605, &UA2032272},
	{SG16606, &UA2033884},
	{SG16607, &UA2198533},
	{SG16608, &UA2406597},
	{SG16609, &UA2434023},
	{SG16610, &UA2845833},
	{SG16611, &UA2951464},
	{SG16612, &UA426715},
	{SG16613, &UA454294},
	{SG16614, &UA652475},
	{SG16615, &UA663944},
	{SG16616, &UA680628},
	{SG16617, &UA901643},
	{SG16618, &UA1981476},
	{SG16619, &UA2027699},
	{SG16620, &UA2031615},
	{SG16621, &UA2589308},
	{SG16633, &UA812305},
	{SG16634, &UA901752},
	{SG16635, &UA904209},
	{SG16636, &UA909699},
	{SG16637, &UA924454},
	{SG16638, &UA1953407},
	{SG16639, &UA1965616},
	{SG16640, &UA1978732},
	{SG16641, &UA1989286},
	{SG16642, &UA1991182},
	{SG16643, &UA1993485},
	{SG16644, &UA2029843},
	{SG16645, &UA2030147},
	{SG16646, &UA2033032},
	{SG16647, &UA2166875},
	{SG16648, &UA2168698},
	{SG16649, &UA2180275},
	{SG16650, &UA2181253},
	{SG16651, &UA2368151},
	{SG16652, &UA2383153},
	{SG16653, &UA2405867},
	{SG16654, &UA2407300},
	{SG16655, &UA2426324},
	{SG16656, &UA2429705},
	{SG16657, &UA2458088},
	{SG16658, &UA2468963},
	{SG16659, &UA2483759},
	{SG16660, &UA2485256},
	{SG16661, &UA2506051},
	{SG16662, &UA2528593},
	{SG16663, &UA2738204},
	{SG16664, &UA2980110},
	{SG16665, &UA3088801},
	{SG16689, &UA616634},
	{SG16690, &UA661161},
	{SG16691, &UA743685},
	{SG16692, &UA806401},
	{SG16693, &UA901293},
	{SG16694, &UA2033294},
	{SG16695, &UA2814131},
	{SG16696, &UA2839185},
	{SG16735, &UA782823},
	{SG16736, &UA856312},
	{SG16737, &UA858913},
	{SG16738, &UA873557},
	{SG16739, &UA899375},
	{SG16740, &UA911683},
	{SG16741, &UA911760},
	{SG16742, &UA912462},
	{SG16743, &UA1992209},
	{SG16744, &UA1992400},
	{SG16745, &UA2028465},
	{SG16746, &UA2029332},
	{SG16748, &UA2031545},
	{SG16749, &UA2033677},
	{SG16751, &UA2222782},
	{SG16753, &UA2235168},
	{SG16754, &UA2445086},
	{SG16755, &UA2586604},
	{SG16757, &UA2642311},
	{SG16758, &UA2934429},
	{SG16759, &UA2980362},
	{SG16761, &UA3069945},
	{SG16807, &UA369196},
	{SG16808, &UA369499},
	{SG16809, &UA369548},
	{SG16810, &UA616654},
	{SG16811, &UA616708},
	{SG16812, &UA628070},
	{SG16813, &UA637681},
	{SG16814, &UA682395},
	{SG16816, &UA728189},
	{SG16817, &UA896738},
	{SG16818, &UA896852},
	{SG16819, &UA908436},
	{SG16820, &UA1966044},
	{SG16821, &UA2023759},
	{SG16822, &UA2024161},
	{SG16823, &UA2026386},
	{SG16824, &UA2028064},
	{SG16825, &UA2034003},
	{SG16826, &UA2444736},
	{SG16827, &UA2473824},
	{SG16829, &UA2982811},
	{SG16861, &UA612892},
	{SG16862, &UA683959},
	{SG16863, &UA842188},
	{SG16864, &UA875437},
	{SG16865, &UA896220},
	{SG16866, &UA896698},
	{SG16867, &UA899665},
	{SG16868, &UA903172},
	{SG16869, &UA1983240},
	{SG16870, &UA1996417},
	{SG16871, &UA2025024},
	{SG16872, &UA2026558},
	{SG16873, &UA2029727},
	{SG16874, &UA2030787},
	{SG16875, &UA2034376},
	{SG16876, &UA2171376},
	{SG16877, &UA2227331},
	{SG16878, &UA2340264},
	{SG16879, &UA2413157},
	{SG16880, &UA2496013},
	{SG16882, &UA2519098},
	{SG16885, &UA2813898},
	{SG16887, &UA2817331},
	{SG16889, &UA2950189},
	{SG16898, &UA368853},
	{SG16899, &UA775084},
	{SG16900, &UA2608090},
	{SG16901, &UA2609551},
	{SG16902, &UA2609564},
	{SG16903, &UA2609798},
	{SG16905, &UA2615999},
	{SG16906, &UA2620990},
	{SG16907, &UA2622503},
	{SG16908, &UA2632156},
	{SG16909, &UA2632449},
	{SG16910, &UA874001},
	{SG16911, &UA1951244},
	{SG16912, &UA2227396},
	{SG16913, &UA2608368},
	{SG16914, &UA2611276},
	{SG16915, &UA2615634},
	{SG16916, &UA2615885},
	{SG16917, &UA2616562},
	{SG16918, &UA2617102},
	{SG16919, &UA2617421},
	{SG16920, &UA2617725},
	{SG16922, &UA2618975},
	{SG16925, &UA2622424},
	{SG16926, &UA2622553},
	{SG16927, &UA2623547},
	{SG16988, &UA630601},
	{SG16989, &UA648028},
	{SG16990, &UA2608327},
	{SG16991, &UA2622260},
	{SG17050, &UA369566},
	{SG17054, &UA2642855},
	{SG17117, &UA368220},
	{SG17118, &UA441155},
	{SG17119, &UA441272},
	{SG17120, &UA609382},
	{SG17121, &UA609644},
	{SG17122, &UA616639},
	{SG17123, &UA685169},
	{SG17124, &UA697417},
	{SG17125, &UA742544},
	{SG17126, &UA767686},
	{SG17127, &UA864970},
	{SG17128, &UA892926},
	{SG17129, &UA895691},
	{SG17130, &UA897773},
	{SG17131, &UA909163},
	{SG17132, &UA909562},
	{SG17133, &UA1959577},
	{SG17134, &UA1970585},
	{SG17135, &UA1986460},
	{SG17138, &UA1994101},
	{SG17139, &UA2025011},
	{SG17140, &UA2025859},
	{SG17141, &UA2031531},
	{SG17142, &UA2032246},
	{SG17143, &UA2032498},
	{SG17144, &UA2034509},
	{SG17145, &UA2323110},
	{SG17146, &UA2359364},
	{SG17147, &UA2362689},
	{SG17148, &UA2369537},
	{SG17149, &UA2404537},
	{SG17150, &UA2405730},
	{SG17151, &UA2445472},
	{SG17152, &UA2578219},
	{SG17153, &UA2659402},
	{SG17184, &UA368840},
	{SG17185, &UA1989558},
	{SG17186, &UA2024979},
	{SG17187, &UA2025683},
	{SG17189, &UA615617},
	{SG17190, &UA879228},
	{SG17192, &UA2608355},
	{SG17193, &UA2610465},
	{SG17194, &UA2611550},
	{SG17195, &UA2615692},
	{SG17196, &UA2615986},
	{SG17197, &UA2617517},
	{SG17198, &UA2617924},
	{SG17199, &UA2618967},
	{SG17200, &UA2625362},
	{SG17201, &UA2629459},
	{SG17202, &UA2636556},
	{SG17203, &UA614762},
	{SG17204, &UA657042},
	{SG17205, &UA903237},
	{SG17206, &UA911833},
	{SG17207, &UA1962240},
	{SG17208, &UA1965114},
	{SG17210, &UA2031421},
	{SG17214, &UA2762516},
	{SG17216, &UA607444},
	{SG17217, &UA754286},
	{SG17218, &UA2411430},
	{SG17219, &UA2420481},
	{SG17220, &UA2608295},
	{SG17221, &UA2610512},
	{SG17222, &UA2611775},
	{SG17223, &UA2612911},
	{SG17224, &UA2618747},
	{SG17225, &UA2620354},
	{SG17226, &UA2624266},
	{SG17227, &UA2624863},
	{SG17228, &UA2625353},
	{SG17229, &UA2627223},
	{SG17230, &UA2631023},
	{SG17231, &UA410843},
	{SG17232, &UA639536},
	{SG17233, &UA651000},
	{SG17234, &UA663296},
	{SG17235, &UA1956969},
	{SG17236, &UA2395510},
	{SG17289, &UA608876},
	{SG17290, &UA616507},
	{SG17291, &UA616521},
	{SG17292, &UA685195},
	{SG17317, &UA603392},
	{SG17318, &UA611387},
	{SG17319, &UA616711},
	{SG17320, &UA714456},
	{SG17321, &UA740640},
	{SG17322, &UA790359},
	{SG17323, &UA843898},
	{SG17324, &UA855908},
	{SG17325, &UA2028569},
	{SG17340, &UA616578},
	{SG17342, &UA359050},
	{SG17344, &UA359180},
	{SG17346, &UA359256},
	{SG17348, &UA359331},
	{SG17350, &UA359619},
	{SG17352, &UA361782},
	{SG17354, &UA368369},
	{SG17357, &UA397851},
	{SG17365, &UA638662},
	{SG17369, &UA843467},
	{SG17370, &UA2022477},
	{SG17371, &UA2383446},
	{SG17372, &UA2384677},
	{SG17373, &UA2421996},
	{SG17374, &UA2652909},
	{SG17375, &UA2950138},
	{SG17376, &UA2982908},
	{SG17378, &UA805076},
	{SG17379, &UA895311},
	{SG17380, &UA1973828},
	{SG17381, &UA1995019},
	{SG17382, &UA1995317},
	{SG17383, &UA1997193},
	{SG17384, &UA616625},
	{SG17385, &UA683353},
	{SG17386, &UA2364738},
	{SG17387, &UA2444473},
	{SG17413, &UA635016},
	{SG17414, &UA673979},
	{SG17415, &UA800603},
	{SG17416, &UA878270},
	{SG17417, &UA900157},
	{SG17418, &UA900427},
	{SG17419, &UA2023958},
	{SG17420, &UA2031848},
	{SG17421, &UA2341150},
	{SG17422, &UA2361761},
	{SG17423, &UA2445712},
	{SG17424, &UA2796426},
	{SG17431, &UA368306},
	{SG17432, &UA412260},
	{SG17434, &UA616734},
	{SG17437, &UA2023876},
	{SG17462, &UA683825},
	{SG17463, &UA703184},
	{SG17464, &UA899720},
	{SG17465, &UA1969988},
	{SG17466, &UA2034343},
	{SG17467, &UA2222941},
	{SG17468, &UA2375184},
	{SG17469, &UA2411490},
	{SG17470, &UA2456017},
	{SG17471, &UA2499918},
	{SG17472, &UA2551109},
	{SG17473, &UA2640987},
	{SG17474, &UA2817071},
	{SG17478, &UA578817},
	{SG17479, &UA647996},
	{SG17480, &UA911285},
	{SG17481, &UA1961492},
	{SG17482, &UA1961756},
	{SG17483, &UA1975833},
	{SG17484, &UA1990359},
	{SG17485, &UA2022907},
	{SG17486, &UA2028222},
	{SG17487, &UA2164644},
	{SG17488, &UA2351837},
	{SG17489, &UA2502385},
	{SG17490, &UA2546330},
	{SG17491, &UA2822351},
	{SG17492, &UA2981927},
	{SG17493, &UA3070446},
	{SG17494, &UA638727},
	{SG17495, &UA1974726},
	{SG17496, &UA1992629},
	{SG17497, &UA1992666},
	{SG17498, &UA1996739},
	{SG17499, &UA2029197},
	{SG17500, &UA2222890},
	{SG17501, &UA2315955},
	{SG17502, &UA2353270},
	{SG17503, &UA2391311},
	{SG17504, &UA2487373},
	{SG17505, &UA2490359},
	{SG17506, &UA2696206},
	{SG17507, &UA616681},
	{SG17508, &UA616748},
	{SG17531, &UA359301},
	{SG17532, &UA363778},
	{SG17533, &UA369321},
	{SG17536, &UA403007},
	{SG17538, &UA403009},
	{SG17540, &UA403022},
	{SG17541, &UA403039},
	{SG17542, &UA403059},
	{SG17543, &UA403065},
	{SG17544, &UA403062},
	{SG17545, &UA403144},
	{SG17546, &UA403190},
	{SG17547, &UA403192},
	{SG17548, &UA694980},
	{SG17549, &UA762431},
	{SG17550, &UA823786},
	{SG17551, &UA1938805},
	{SG17560, &UA609305},
	{SG17561, &UA610180},
	{SG17562, &UA611525},
	{SG17563, &UA615498},
	{SG17564, &UA900639},
	{SG17565, &UA900743},
	{SG17566, &UA900956},
	{SG17567, &UA904660},
	{SG17568, &UA1954680},
	{SG17569, &UA1992745},
	{SG17582, &UA616632},
	{SG17583, &UA685256},
	{SG17584, &UA762805},
	{SG17585, &UA2030727},
	{SG17590, &UA783217},
	{SG17591, &UA827851},
	{SG17592, &UA1950871},
	{SG17593, &UA2187998},
	{SG17594, &UA2484015},
	{SG17627, &UA2028804},
	{SG17629, &UA2257311},
	{SG17631, &UA2356515},
	{SG17634, &UA2414104},
	{SG17636, &UA2504758},
	{SG17668, &UA779826},
	{SG17669, &UA868674},
	{SG17670, &UA898546},
	{SG17671, &UA1959598},
	{SG17672, &UA370031},
	{SG17673, &UA601816},
	{SG17674, &UA627502},
	{SG17675, &UA659464},
	{SG17676, &UA811802},
	{SG17677, &UA829987},
	{SG17678, &UA863010},
	{SG17679, &UA2218534},
	{SG17680, &UA2686186},
	{SG17681, &UA2708159},
	{SG17701, &UA603896},
	{SG17702, &UA658754},
	{SG17703, &UA2027689},
	{SG17708, &UA655786},
	{SG17709, &UA685392},
	{SG17710, &UA899047},
	{SG17711, &UA899415},
	{SG17712, &UA1973551},
	{SG17713, &UA2185746},
	{SG17714, &UA2235580},
	{SG17715, &UA2257277},
	{SG17716, &UA2507648},
	{SG17717, &UA2508034},
	{SG17718, &UA2685781},
	{SG17719, &UA2688828},
	{SG17720, &UA582504},
	{SG17721, &UA616749},
	{SG17722, &UA628892},
	{SG17723, &UA738997},
	{SG17724, &UA1962781},
	{SG17725, &UA1968698},
	{SG17727, &UA3042571},
	{SG17731, &UA1954443},
	{SG17732, &UA1975589},
	{SG17733, &UA1985125},
	{SG17734, &UA1992756},
	{SG17735, &UA2025782},
	{SG17736, &UA2026146},
	{SG17737, &UA2026928},
	{SG17738, &UA2027177},
	{SG17739, &UA2027354},
	{SG17740, &UA2028092},
	{SG17741, &UA2028263},
	{SG17742, &UA2028359},
	{SG17743, &UA2028487},
	{SG17744, &UA2029289},
	{SG17745, &UA2030267},
	{SG17746, &UA2030622},
	{SG17747, &UA2032179},
	{SG17748, &UA2032450},
	{SG17749, &UA2228458},
	{SG17750, &UA2261472},
	{SG17751, &UA2314082},
	{SG17752, &UA2329618},
	{SG17753, &UA2338733},
	{SG17754, &UA2339339},
	{SG17755, &UA2345482},
	{SG17756, &UA2411005},
	{SG17757, &UA2413462},
	{SG17758, &UA2413638},
	{SG17759, &UA2422470},
	{SG17760, &UA2442885},
	{SG17761, &UA2445442},
	{SG17762, &UA2449924},
	{SG17763, &UA2455605},
	{SG17764, &UA2476553},
	{SG17765, &UA2477523},
	{SG17766, &UA2478214},
	{SG17767, &UA2479456},
	{SG17768, &UA2481672},
	{SG17769, &UA2481678},
	{SG17770, &UA2481736},
	{SG17771, &UA2482661},
	{SG17772, &UA2487247},
	{SG17773, &UA2497163},
	{SG17774, &UA2499558},
	{SG17775, &UA2505468},
	{SG17777, &UA2519891},
	{SG17779, &UA2520248},
	{SG17782, &UA2524427},
	{SG17784, &UA2524532},
	{SG17786, &UA2525131},
	{SG17788, &UA2527068},
	{SG17790, &UA2527436},
	{SG17793, &UA2527683},
	{SG17797, &UA2528103},
	{SG17799, &UA2528409},
	{SG17816, &UA2545240},
	{SG17817, &UA2545651},
	{SG17819, &UA2559305},
	{SG17822, &UA2578506},
	{SG17824, &UA2589373},
	{SG17825, &UA2593834},
	{SG17826, &UA2594016},
	{SG17827, &UA2644267},
	{SG17828, &UA2644403},
	{SG17829, &UA2660449},
	{SG17830, &UA2694428},
	{SG17831, &UA2721551},
	{SG17832, &UA2758359},
	{SG17833, &UA2762293},
	{SG17834, &UA2790727},
	{SG17835, &UA2812527},
	{SG17836, &UA2814049},
	{SG17837, &UA2815867},
	{SG17838, &UA2815925},
	{SG17839, &UA2816118},
	{SG17840, &UA2817080},
	{SG17841, &UA2818004},
	{SG17842, &UA2820068},
	{SG17843, &UA2822338},
	{SG17844, &UA2823679},
	{SG17845, &UA2825388},
	{SG17846, &UA2832263},
	{SG17847, &UA2833025},
	{SG17848, &UA2834597},
	{SG17849, &UA2837132},
	{SG17850, &UA2839229},
	{SG17851, &UA2840248},
	{SG17852, &UA2845342},
	{SG17853, &UA2854058},
	{SG17854, &UA2934038},
	{SG17855, &UA2983313},
	{SG17856, &UA2984430},
	{SG17857, &UA2985925},
	{SG17859, &UA3044391},
	{SG17860, &UA3045226},
	{SG17861, &UA3085531},
	{SG17876, &UA458633},
	{SG17877, &UA458636},
	{SG17878, &UA899864},
	{SG17879, &UA1965185},
	{SG17880, &UA2025301},
	{SG17882, &UA2327682},
	{SG17884, &UA2496085},
	{SG17937, &UA857790},
	{SG17938, &UA2024669},
	{SG17939, &UA2026780},
	{SG17940, &UA2027978},
	{SG17941, &UA2028959},
	{SG17942, &UA2191112},
	{SG17943, &UA2366205},
	{SG17944, &UA2413488},
	{SG17945, &UA2427036},
	{SG17946, &UA2646769},
	{SG17947, &UA2951169},
	{SG17967, &UA690858},
	{SG17968, &UA722734},
	{SG17970, &UA765592},
	{SG17972, &UA895184},
	{SG17974, &UA1991544},
	{SG18009, &UA360380},
	{SG18010, &UA368851},
	{SG18011, &UA686097},
	{SG18012, &UA770238},
	{SG18013, &UA802947},
	{SG18014, &UA862569},
	{SG18015, &UA898773},
	{SG18018, &UA2030617},
	{SG18019, &UA2496777},
	{SG18020, &UA2570289},
	{SG18021, &UA2746751},
	{SG18033, &UA2033438},
	{SG18034, &UA2195939},
	{SG18035, &UA2238552},
	{SG18036, &UA2362165},
	{SG18037, &UA2426848},
	{SG18038, &UA2429775},
	{SG18039, &UA2437350},
	{SG18040, &UA2451931},
	{SG18041, &UA2460240},
	{SG18042, &UA2656473},
	{SG18043, &UA2730770},
	{SG18050, &UA901773},
	{SG18054, &UA2380406},
	{SG18055, &UA403354},
	{SG18056, &UA895466},
	{SG18057, &UA1961417},
	{SG18058, &UA2028096},
	{SG18059, &UA2031743},
	{SG18060, &UA2187896},
	{SG18061, &UA2257181},
	{SG18062, &UA2384524},
	{SG18063, &UA2428302},
	{SG18064, &UA2551000},
	{SG18065, &UA2578804},
	{SG18066, &UA3070521},
	{SG18067, &UA3118964},
	{SG18070, &UA2005200},
	{SG18071, &UA2011785},
	{SG18072, &UA2689143},
	{SG18106, &UA863688},
	{SG18107, &UA1949527},
	{SG18108, &UA2029501},
	{SG18109, &UA2034093},
	{SG18110, &UA2395368},
	{SG18136, &UA423471},
	{SG18137, &UA616696},
	{SG18138, &UA2574348},
	{SG18139, &UA2686479},
	{SG18144, &UA658467},
	{SG18145, &UA2031894},
	{SG18146, &UA2033907},
	{SG18147, &UA2257677},
	{SG18148, &UA2503268},
	{SG18149, &UA2686151},
	{SG18150, &UA2788154},
	{SG18190, &UA2039836},
	{SG18191, &UA2193488},
	{SG18192, &UA2236174},
	{SG18193, &UA2523784},
	{SG18197, &UA616718},
	{SG18198, &UA1952595},
	{SG18199, &UA2652341},
	{SG18201, &UA380160},
	{SG18203, &UA854462},
	{SG18206, &UA1960497},
	{SG18208, &UA1993387},
	{SG18209, &UA1996092},
	{SG18210, &UA2023808},
	{SG18211, &UA2028641},
	{SG18212, &UA2031105},
	{SG18213, &UA2242218},
	{SG18214, &UA2338511},
	{SG18215, &UA2465292},
	{SG18216, &UA2521317},
	{SG18217, &UA2859633},
	{SG18218, &UA3057590},
	{SG18219, &UA3098578},
	{SG18224, &UA359993},
	{SG18228, &UA781221},
	{SG18229, &UA868803},
	{SG18230, &UA2021728},
	{SG18231, &UA2034409},
	{SG18232, &UA2150171},
	{SG18233, &UA2455567},
	{SG18234, &UA2502118},
	{SG18235, &UA2758817},
	{SG18241, &UA899418},
	{SG18242, &UA1994439},
	{SG18243, &UA1997244},
	{SG18244, &UA2030471},
	{SG18245, &UA2034371},
	{SG18246, &UA2190014},
	{SG18247, &UA2257286},
	{SG18248, &UA2342886},
	{SG18249, &UA2395848},
	{SG18250, &UA2445854},
	{SG18251, &UA2501190},
	{SG18252, &UA2554196},
	{SG18253, &UA2685873},
	{SG18254, &UA2763457},
	{SG18255, &UA2790725},
	{SG18256, &UA2817585},
	{SG18257, &UA2833240},
	{SG18258, &UA3042088},
	{SG18280, &UA2608427},
	{SG18281, &UA2608724},
	{SG18282, &UA2611576},
	{SG18312, &UA2609163},
	{SG18322, &UA1972512},
	{SG18323, &UA2029100},
	{SG18324, &UA2178616},
	{SG18325, &UA2236060},
	{SG18326, &UA2239018},
	{SG18327, &UA2390979},
	{SG18328, &UA2446309},
	{SG18329, &UA2469224},
	{SG18330, &UA2475884},
	{SG18331, &UA2503249},
	{SG18332, &UA2520610},
	{SG18333, &UA2558972},
	{SG18334, &UA2567749},
	{SG18335, &UA2693959},
	{SG18336, &UA2763468},
	{SG18337, &UA3071463},
	{SG18338, &UA3121364},
	{SG18353, &UA2566499},
	{SG18354, &UA2566685},
	{SG18355, &UA2567101},
	{SG18356, &UA2567120},
	{SG18357, &UA2583211},
	{SG18359, &UA609400},
	{SG18360, &UA609494},
	{SG18361, &UA901946},
	{SG18362, &UA901979},
	{SG18363, &UA904331},
	{SG18365, &UA2026412},
	{SG18369, &UA609399},
	{SG18370, &UA900615},
	{SG18371, &UA901929},
	{SG18372, &UA1954188},
	{SG18373, &UA1959558},
	{SG18374, &UA1989599},
	{SG18380, &UA609398},
	{SG18381, &UA902337},
	{SG18382, &UA903935},
	{SG18383, &UA1950409},
	{SG18384, &UA1957419},
	{SG18385, &UA1982131},
	{SG18386, &UA2023472},
	{SG18387, &UA2027896},
	{SG18388, &UA2028426},
	{SG18389, &UA2030459},
	{SG18390, &UA2034623},
	{SG18391, &UA2194369},
	{SG18392, &UA2230336},
	{SG18393, &UA2261486},
	{SG18394, &UA2342262},
	{SG18395, &UA2349213},
	{SG18396, &UA2360246},
	{SG18397, &UA2366561},
	{SG18398, &UA2366732},
	{SG18399, &UA2373753},
	{SG18400, &UA2376026},
	{SG18401, &UA2377578},
	{SG18403, &UA2380271},
	{SG18404, &UA2380577},
	{SG18406, &UA2382027},
	{SG18407, &UA2383265},
	{SG18409, &UA2383562},
	{SG18411, &UA2383589},
	{SG18412, &UA2384445},
	{SG18414, &UA2393630},
	{SG18415, &UA2399290},
	{SG18416, &UA2403589},
	{SG18417, &UA2413246},
	{SG18418, &UA2418219},
	{SG18419, &UA2451002},
	{SG18420, &UA2453266},
	{SG18421, &UA2523998},
	{SG18422, &UA2526607},
	{SG18423, &UA2553734},
	{SG18424, &UA2589859},
	{SG18425, &UA2596658},
	{SG18426, &UA2647559},
	{SG18430, &UA2761955},
	{SG18431, &UA2839729},
	{SG18444, &UA900701},
	{SG18445, &UA2261487},
	{SG18447, &UA2659416},
	{SG18457, &UA609355},
	{SG18458, &UA611160},
	{SG18459, &UA900711},
	{SG18460, &UA901723},
	{SG18470, &UA633519},
	{SG18471, &UA764195},
	{SG18473, &UA453554},
	{SG18474, &UA1960286},
	{SG18475, &UA2202798},
	{SG18490, &UA902928},
	{SG18491, &UA1992551},
	{SG18492, &UA2708115},
	{SG18493, &UA2726781},
	{SG18494, &UA715667},
	{SG18495, &UA652016},
	{SG18496, &UA893861},
	{SG18507, &UA662932},
	{SG18508, &UA854989},
	{SG18509, &UA2026071},
	{SG18511, &UA2819440},
	{SG18512, &UA428368},
	{SG18513, &UA848517},
	{SG18514, &UA1987751},
	{SG18515, &UA1994157},
	{SG18516, &UA2178596},
	{SG18540, &UA896609},
	{SG18541, &UA899191},
	{SG18542, &UA899217},
	{SG18543, &UA2033912},
	{SG18544, &UA2419751},
	{SG18547, &UA853027},
	{SG18548, &UA1989731},
	{SG18569, &UA428547},
	{SG18570, &UA874264},
	{SG18571, &UA862093},
	{SG18572, &UA2786543},
	{SG18575, &UA903044},
	{SG18577, &UA901456},
	{SG18578, &UA1972383},
	{SG18618, &UA2031595},
	{SG18631, &UA366742},
	{SG18632, &UA644844},
	{SG18633, &UA657170},
	{SG18634, &UA1945779},
	{SG18635, &UA2028218},
	{SG18646, &UA359879},
	{SG18647, &UA366552},
	{SG18648, &UA686753},
	{SG18649, &UA2024444},
	{SG18650, &UA2025308},
	{SG18651, &UA2033312},
	{SG18652, &UA609292},
	{SG18654, &UA758646},
	{SG18655, &UA2031151},
	{SG18659, &UA2666810},
	{SG18664, &UA901298},
	{SG18667, &UA1959645},
	{SG18673, &UA601064},
	{SG18674, &UA675601},
	{SG18675, &UA723712},
	{SG18676, &UA1982635},
	{SG18677, &UA2028881},
	{SG18678, &UA2044022},
	{SG18679, &UA2044282},
	{SG18680, &UA2179108},
	{SG18681, &UA2352254},
	{SG18682, &UA2355757},
	{SG18683, &UA2983735},
	{SG18684, &UA2029253},
	{SG18685, &UA2205644},
	{SG18686, &UA2608452},
	{SG18687, &UA2612554},
	{SG18689, &UA2973480},
	{SG18690, &UA2974037},
	{SG18691, &UA2974612},
	{SG18692, &UA2974804},
	{SG18738, &UA841373},
	{SG18739, &UA2024027},
	{SG18740, &UA2608746},
	{SG18741, &UA2609231},
	{SG18742, &UA2610288},
	{SG18743, &UA2611565},
	{SG18744, &UA2618061},
	{SG18745, &UA2261458},
	{SG18746, &UA2405792},
	{SG18749, &UA863576},
	{SG18750, &UA1951734},
	{SG18751, &UA2025035},
	{SG18752, &UA2034600},
	{SG18757, &UA609326},
	{SG18758, &UA609488},
	{SG18760, &UA610033},
	{SG18761, &UA901563},
	{SG18762, &UA901882},
	{SG18763, &UA906815},
	{SG18764, &UA909124},
	{SG18765, &UA2025195},
	{SG18770, &UA2027312},
	{SG18771, &UA2676024},
	{SG18772, &UA2676683},
	{SG18773, &UA2676830},
	{SG18774, &UA2677278},
	{SG18775, &UA2677284},
	{SG18785, &UA352178},
	{SG18786, &UA353071},
	{SG18787, &UA353129},
	{SG18788, &UA358781},
	{SG18789, &UA359205},
	{SG18790, &UA360284},
	{SG18791, &UA1978275},
	{SG18792, &UA2026742},
	{SG18793, &UA2949810},
	{SG18794, &UA609506},
	{SG18795, &UA902245},
	{SG18796, &UA904814},
	{SG18797, &UA3064874},
	{SG18798, &UA3066883},
	{SG18799, &UA3067903},
	{SG18800, &UA3082480},
	{SG18804, &UA609405},
	{SG18805, &UA900643},
	{SG18806, &UA1964036},
	{SG18807, &UA2025851},
	{SG18808, &UA458640},
	{SG18809, &UA1995975},
	{SG18810, &UA2820947},
	{SG18812, &UA2752958},
	{SG18813, &UA2753456},
	{SG18814, &UA2753846},
	{SG18815, &UA2753973},
	{SG18816, &UA2892855},
	{SG18826, &UA901261},
	{SG18827, &UA1969242},
	{SG18828, &UA1994823},
	{SG18829, &UA2614279},
	{SG18839, &UA863716},
	{SG18840, &UA900660},
	{SG18841, &UA1953011},
	{SG18842, &UA2027637},
	{SG18843, &UA2257192},
	{SG18844, &UA902868},
	{SG18845, &UA1964891},
	{SG18846, &UA2257293},
	{SG18847, &UA2496193},
	{SG18850, &UA609509},
	{SG18851, &UA901713},
	{SG18852, &UA3021955},
	{SG18862, &UA880844},
	{SG18863, &UA886662},
	{SG18864, &UA899488},
	{SG18865, &UA2028256},
	{SG18866, &UA2031746},
	{SG18867, &UA2043760},
	{SG18868, &UA2495808},
	{SG18869, &UA2500706},
	{SG18870, &UA2817161},
	{SG18871, &UA2566524},
	{SG18872, &UA2567239},
	{SG18873, &UA2862478},
	{SG18874, &UA2531304},
	{SG18894, &UA904701},
	{SG18895, &UA2335904},
	{SG18902, &UA2408229},
	{SG18903, &UA2846839},
	{SG18906, &UA777058},
	{SG18912, &UA776816},
	{SG18913, &UA2025521},
	{SG18914, &UA2033577},
	{SG18915, &UA2034441},
	{SG18916, &UA2787156},
	{SG18959, &UA800951},
	{SG18960, &UA2376960},
	{SG18961, &UA2608109},
	{SG18962, &UA2608755},
	{SG18970, &UA2445738},
	{SG18993, &UA2566684},
	{SG18994, &UA3045934},
	{SG19000, &UA2867500},
	{SG19002, &UA2453184},
	{SG19005, &UA2974598},
	{SG19006, &UA2975750},
	{SG19007, &UA2608278},
	{SG19008, &UA2609069},
	{SG19009, &UA2257363},
	{SG19010, &UA2401281},
	{SG19011, &UA2594340},
	{SG19012, &UA2566500},
	{SG19014, &UA690290},
	{SG19015, &UA893306},
	{SG19016, &UA2443379},
	{SG19017, &UA2375870},
	{SG19018, &UA2413374},
	{SG19019, &UA2654290},
	{SG19026, &UA887159},
	{SG19028, &UA895140},
	{SG19029, &UA902974},
	{SG19031, &UA1964920},
	{SG19032, &UA2023278},
	{SG19033, &UA2026710},
	{SG19035, &UA2375966},
	{SG19036, &UA2393512},
	{SG19038, &UA2476807},
	{SG19039, &UA2665489},
	{SG19040, &UA2700536},
	{SG19041, &UA2700745},
	{SG19042, &UA2750766},
	{SG19043, &UA2779525},
	{SG19044, &UA2986926},
	{SG19045, &UA3071641},
	{SG19046, &UA2369645},
	{SG19047, &UA2889338},
	{SG19052, &UA459128},
	{SG19053, &UA2322767},
	{SG19056, &UA2430055},
	{SG19057, &UA2436237},
	{SG19058, &UA2974378},
	{SG19059, &UA1956281},
	{SG19060, &UA2444036},
	{SG19061, &UA818352},
	{SG19062, &UA875501},
	{SG19063, &UA1960166},
	{SG19064, &UA2031640},
	{SG19065, &UA2257296},
	{SG19066, &UA2444995},
	{SG19067, &UA2721163},
	{SG19077, &UA456739},
	{SG19078, &UA579446},
	{SG19079, &UA652270},
	{SG19080, &UA780715},
	{SG19081, &UA828581},
	{SG19082, &UA2023999},
	{SG19083, &UA2386617},
	{SG19084, &UA2502665},
	{SG19085, &UA2658690},
	{SG19086, &UA2827931},
	{SG19087, &UA2566537},
	{SG19088, &UA2974019},
	{SG19090, &UA820914},
	{SG19091, &UA899133},
	{SG19092, &UA907744},
	{SG19093, &UA2575278},
	{SG19094, &UA2983345},
	{SG19098, &UA741098},
	{SG19099, &UA2034246},
	{SG19100, &UA368825},
	{SG19101, &UA1971042},
	{SG19102, &UA1996283},
	{SG19103, &UA2218679},
	{SG19104, &UA2257335},
	{SG19105, &UA2449843},
	{SG19110, &UA2735360},
	{SG19115, &UA370248},
	{SG19118, &UA812661},
	{SG19142, &UA903088},
	{SG19143, &UA2024890},
	{SG19144, &UA827413},
	{SG19145, &UA900903},
	{SG19159, &UA609514},
	{SG19160, &UA904370},
	{SG19162, &UA2813187},
	{SG19163, &UA2754329},
	{SG19164, &UA2755078},
	{SG19166, &UA2032387},
	{SG19170, &UA458608},
	{SG19171, &UA2033491},
	{SG19172, &UA2178418},
	{SG19173, &UA2224590},
	{SG19176, &UA2811480},
	{SG19186, &UA403385},
	{SG19187, &UA1967494},
	{SG19188, &UA612702},
	{SG19189, &UA901600},
	{SG19193, &UA3064653},
	{SG19194, &UA3115058},
	{SG19195, &UA616677},
	{SG19196, &UA2642927},
	{SG19197, &UA609336},
	{SG19198, &UA901581},
	{SG19200, &UA2475386},
	{SG19226, &UA1981960},
	{SG19227, &UA2261512},
	{SG19228, &UA2261595},
	{SG19229, &UA2935498},
	{SG19234, &UA902135},
	{SG19235, &UA908949},
	{SG19240, &UA609209},
	{SG19242, &UA2441826},
	{SG19244, &UA2974151},
	{SG19245, &UA3021005},
	{SG19262, &UA609721},
	{SG19265, &UA907717},
	{SG19277, &UA907574},
	{SG19278, &UA2381243},
	{SG19283, &UA2566488},
	{SG19284, &UA2990081},
	{SG19286, &UA2202664},
	{SG19287, &UA2889446},
	{SG19292, &UA816538},
	{SG19293, &UA899031},
	{SG19294, &UA1961084},
	{SG19295, &UA2024155},
	{SG19296, &UA2395522},
	{SG19302, &UA609433},
	{SG19303, &UA902709},
	{SG19305, &UA3067201},
	{SG19306, &UA3082866},
	{SG19307, &UA901283},
	{SG19311, &UA2567103},
	{SG19324, &UA2566883},
	{SG19325, &UA2583213},
	{SG19327, &UA781653},
	{SG19328, &UA903911},
	{SG19329, &UA609404},
	{SG19330, &UA900908},
	{SG19333, &UA613623},
	{SG19334, &UA900677},
	{SG19335, &UA908789},
	{SG19336, &UA2027965},
	{SG19337, &UA2033246},
	{SG19338, &UA905100},
	{SG19341, &UA2637591},
	{SG19343, &UA678119},
	{SG19345, &UA1962078},
	{SG19346, &UA1993649},
	{SG19347, &UA1995790},
	{SG19348, &UA2026652},
	{SG19349, &UA2034278},
	{SG19351, &UA2671307},
	{SG19352, &UA2676741},
	{SG19353, &UA2860874},
	{SG19354, &UA2755540},
	{SG19355, &UA609346},
	{SG19356, &UA609840},
	{SG19357, &UA901221},
	{SG19358, &UA903734},
	{SG19359, &UA908855},
	{SG19360, &UA2023764},
	{SG19380, &UA895669},
	{SG19381, &UA2445300},
	{SG19382, &UA642410},
	{SG19383, &UA902542},
	{SG19384, &UA2196267},
	{SG19386, &UA2362652},
	{SG19388, &UA2566523},
	{SG19389, &UA2566909},
	{SG19390, &UA2667707},
	{SG19393, &UA2566916},
	{SG19395, &UA2636560},
	{SG19397, &UA2034787},
	{SG19398, &UA2034946},
	{SG19399, &UA2257298},
	{SG19400, &UA2741605},
	{SG19407, &UA924267},
	{SG19408, &UA2475881},
	{SG19409, &UA1972923},
	{SG19410, &UA2335088},
	{SG19415, &UA1962750},
	{SG19416, &UA2028285},
	{SG19431, &UA866242},
	{SG19432, &UA900697},
	{SG19433, &UA599710},
	{SG19434, &UA369753},
	{SG19435, &UA2026968},
	{SG19436, &UA2032442},
	{SG19438, &UA2496250},
	{SG19439, &UA2518423},
	{SG19441, &UA2574649},
	{SG19447, &UA2974475},
	{SG19451, &UA2676278},
	{SG19452, &UA2735438},
	{SG19461, &UA2724029},
	{SG19466, &UA2233969},
	{SG19467, &UA2550745},
	{SG19468, &UA1995778},
	{SG19469, &UA2414481},
	{SG19470, &UA609700},
	{SG19471, &UA901688},
	{SG19472, &UA911772},
	{SG19473, &UA1995016},
	{SG19474, &UA2026003},
	{SG19475, &UA2521593},
	{SG19480, &UA1974530},
	{SG19481, &UA1996918},
	{SG19482, &UA2028912},
	{SG19483, &UA2178021},
	{SG19484, &UA2235573},
	{SG19485, &UA2981693},
	{SG19486, &UA3068270},
	{SG19490, &UA911654},
	{SG19491, &UA2476001},
	{SG19494, &UA2754684},
	{SG19495, &UA2867662},
	{SG19496, &UA903036},
	{SG19497, &UA2827482},
	{SG19503, &UA1980043},
	{SG19504, &UA3042530},
	{SG19505, &UA2676229},
	{SG19509, &UA2741734},
	{SG19510, &UA2873530},
	{SG19511, &UA894309},
	{SG19512, &UA903813},
	{SG19531, &UA686667},
	{SG19532, &UA2396341},
	{SG19533, &UA2817407},
	{SG19537, &UA757605},
	{SG19538, &UA2544172},
	{SG19539, &UA2973654},
	{SG19541, &UA2975791},
	{SG19542, &UA3022825},
	{SG19548, &UA845536},
	{SG19549, &UA2684247},
	{SG19551, &UA359467},
	{SG19552, &UA359470},
	{SG19553, &UA359857},
	{SG19554, &UA371234},
	{SG19555, &UA663817},
	{SG19556, &UA2022204},
	{SG19558, &UA777355},
	{SG19559, &UA1958344},
	{SG19560, &UA1959082},
	{SG19561, &UA2025086},
	{SG19562, &UA2385381},
	{SG19563, &UA2401057},
	{SG19564, &UA2466531},
	{SG19565, &UA2467354},
	{SG19566, &UA2522960},
	{SG19567, &UA2526948},
	{SG19568, &UA2554599},
	{SG19569, &UA2812379},
	{SG19570, &UA2835776},
	{SG19571, &UA2982585},
	{SG19572, &UA1996408},
	{SG19573, &UA2031567},
	{SG19574, &UA2477386},
	{SG19576, &UA2974991},
	{SG19585, &UA1993955},
	{SG19586, &UA1996222},
	{SG19587, &UA2026108},
	{SG19588, &UA2413463},
	{SG19589, &UA2559533},
	{SG19593, &UA2676018},
	{SG19594, &UA872551},
	{SG19595, &UA2473580},
	{SG19600, &UA2753781},
	{SG19601, &UA2754406},
	{SG19602, &UA2799088},
	{SG19604, &UA403386},
	{SG19605, &UA3043317},
	{SG19608, &UA609349},
	{SG19609, &UA901689},
	{SG19610, &UA901844},
	{SG19611, &UA903048},
	{SG19612, &UA904014},
	{SG19613, &UA2026128},
	{SG19615, &UA610894},
	{SG19616, &UA901039},
	{SG19617, &UA2032235},
	{SG19618, &UA2203818},
	{SG19619, &UA2191473},
	{SG19621, &UA2500172},
	{SG19622, &UA2746837},
	{SG19627, &UA1965978},
	{SG19628, &UA2027995},
	{SG19629, &UA2456142},
	{SG19630, &UA2510694},
	{SG19631, &UA2996423},
	{SG19635, &UA2975083},
	{SG19641, &UA2734808},
	{SG19644, &UA2938240},
	{SG19648, &UA2028805},
	{SG19649, &UA2340015},
	{SG19650, &UA2985329},
	{SG19651, &UA3042908},
	{SG19653, &UA924833},
	{SG19654, &UA2031045},
	{SG19655, &UA2033075},
	{SG19656, &UA2368742},
	{SG19657, &UA2603604},
	{SG19658, &UA2990807},
	{SG19659, &UA3021810},
	{SG19661, &UA2635192},
	{SG19674, &UA2025029},
	{SG19675, &UA2232880},
	{SG19676, &UA2395645},
	{SG19677, &UA2444777},
	{SG19678, &UA2502481},
	{SG19679, &UA2545934},
	{SG19680, &UA2693085},
	{SG19681, &UA2836998},
	{SG19685, &UA3065597},
	{SG19686, &UA3067486},
	{SG19687, &UA3110913},
	{SG19689, &UA2670583},
	{SG19690, &UA2676678},
	{SG19691, &UA2708557},
	{SG19693, &UA903081},
	{SG19694, &UA1954415},
	{SG19695, &UA1991584},
	{SG19696, &UA2025227},
	{SG19697, &UA2027665},
	{SG19698, &UA2030512},
	{SG19699, &UA899125},
	{SG19700, &UA911627},
	{SG19702, &UA2403115},
	{SG19703, &UA2436231},
	{SG19704, &UA2951470},
	{SG19709, &UA2974919},
	{SG19710, &UA3023140},
	{SG19711, &UA2676073},
	{SG19712, &UA2735316},
	{SG19714, &UA635181},
	{SG19723, &UA361021},
	{SG19724, &UA366432},
	{SG19725, &UA667599},
	{SG19728, &UA2356415},
	{SG19729, &UA2567574},
	{SG19730, &UA366141},
	{SG19731, &UA369180},
	{SG19732, &UA689285},
	{SG19733, &UA2023444},
	{SG19734, &UA2364269},
	{SG19735, &UA2473422},
	{SG19739, &UA2205975},
	{SG19741, &UA2753003},
	{SG19742, &UA3065370},
	{SG19746, &UA2368526},
	{SG19747, &UA3051171},
	{SG19748, &UA2754718},
	{SG19751, &UA908427},
	{SG19752, &UA2974993},
	{SG19753, &UA910919},
	{SG19754, &UA2481655},
	{SG19755, &UA3065258},
	{SG19762, &UA2029733},
	{SG19763, &UA2933699},
	{SG19764, &UA2975011},
	{SG19771, &UA2676635},
	{SG19772, &UA2676754},
	{SG19775, &UA627348},
	{SG19776, &UA901662},
	{SG19783, &UA2675975},
	{SG19786, &UA2368714},
	{SG19787, &UA2845206},
	{SG19788, &UA606026},
	{SG19795, &UA2755999},
	{SG19797, &UA2783279},
	{SG19807, &UA863909},
	{SG19808, &UA2724450},
	{SG19823, &UA2033116},
	{SG19824, &UA2167972},
	{SG19825, &UA2420519},
	{SG19826, &UA2676838},
	{SG19829, &UA2666880},
	{SG19831, &UA2974490},
	{SG19833, &UA2476361},
	{SG19834, &UA2744051},
	{SG19835, &UA2874511},
	{SG19836, &UA3068064},
	{SG19837, &UA2566515},
	{SG19843, &UA899865},
	{SG19844, &UA2033665},
	{SG19845, &UA2034490},
	{SG19846, &UA609296},
	{SG19847, &UA901280},
	{SG19848, &UA2026135},
	{SG19855, &UA2566491},
	{SG19857, &UA2566506},
	{SG19858, &UA2570041},
	{SG19860, &UA2772904},
	{SG19862, &UA2774067},
	{SG19863, &UA2784826},
	{SG19865, &UA2869274},
	{SG19866, &UA839488},
	{SG19867, &UA905104},
	{SG19868, &UA909212},
	{SG19869, &UA909403},
	{SG19870, &UA2424598},
	{SG19880, &UA901294},
	{SG19882, &UA887217},
	{SG19883, &UA2033283},
	{SG19884, &UA2589480},
	{SG19886, &UA902151},
	{SG19888, &UA2023827},
	{SG19890, &UA2566547},
	{SG19900, &UA2973274},
	{SG19902, &UA2974034},
	{SG19905, &UA2411657},
	{SG19906, &UA1957483},
	{SG19907, &UA1966438},
	{SG19908, &UA1976528},
	{SG19909, &UA2160597},
	{SG19910, &UA1981384},
	{SG19911, &UA2824331},
	{SG19924, &UA3067604},
	{SG19928, &UA3094535},
	{SG19934, &UA2974872},
	{SG19935, &UA2783376},
	{SG19936, &UA2798844},
	{SG19938, &UA2892363},
	{SG19940, &UA2892471},
	{SG19941, &UA2892526},
	{SG19942, &UA2892959},
	{SG19943, &UA2893001},
	{SG19944, &UA2893123},
	{SG19945, &UA2958318},
	{SG19948, &UA2999277},
	{SG19949, &UA2999452},
	{SG19950, &UA609678},
	{SG19951, &UA902488},
	{SG19952, &UA2566808},
	{SG19953, &UA2566627},
	{SG19955, &UA809644},
	{SG19956, &UA901271},
	{SG19957, &UA2975071},
	{SG19958, &UA906244},
	{SG19960, &UA2362613},
	{SG19964, &UA2567224},
	{SG19967, &UA2753338},
	{SG19969, &UA3065384},
	{SG19970, &UA3068321},
	{SG19971, &UA766842},
	{SG19972, &UA2412198},
	{SG19973, &UA2545086},
	{SG19980, &UA2237175},
	{SG19989, &UA2361889},
	{SG20000, &UA2341416},
	{SG20003, &UA2355482},
	{SG20005, &UA2386286},
	{SG20007, &UA2974081},
	{SG20009, &UA2974907},
	{SG20013, &UA2461240},
	{SG20014, &UA360091},
	{SG20015, &UA398477},
	{SG20016, &UA2025356},
	{SG20022, &UA900910},
	{SG20028, &UA2754408},
	{SG20032, &UA2257343},
	{SG20033, &UA2367051},
	{SG20034, &UA2527981},
	{SG20035, &UA2696340},
	{SG20036, &UA2985170},
	{SG20037, &UA2566622},
	{SG20038, &UA2567267},
	{SG20039, &UA2706876},
	{SG20040, &UA368821},
	{SG20041, &UA369251},
	{SG20042, &UA898062},
	{SG20043, &UA1974966},
	{SG20044, &UA2366841},
	{SG20047, &UA2611281},
	{SG20048, &UA907381},
	{SG20049, &UA2028025},
	{SG20050, &UA2030289},
	{SG20051, &UA2338316},
	{SG20052, &UA2339767},
	{SG20053, &UA2648340},
	{SG20054, &UA2746324},
	{SG20060, &UA2566538},
	{SG20061, &UA2566977},
	{SG20062, &UA2584083},
	{SG20063, &UA2975055},
	{SG20071, &UA3065622},
	{SG20072, &UA783756},
	{SG20073, &UA1956824},
	{SG20074, &UA2026614},
	{SG20075, &UA2027670},
	{SG20076, &UA2588372},
	{SG20078, &UA2868910},
	{SG20079, &UA924885},
	{SG20080, &UA2257248},
	{SG20081, &UA2024301},
	{SG20082, &UA2832157},
	{SG20083, &UA2755331},
	{SG20086, &UA3022596},
	{SG20098, &UA2461344},
	{SG20099, &UA2626454},
	{SG20110, &UA2361766},
	{SG20112, &UA2468251},
	{SG20113, &UA2508334},
	{SG20114, &UA2593774},
	{SG20115, &UA2600308},
	{SG20120, &UA609425},
	{SG20121, &UA901029},
	{SG20123, &UA2974877},
	{SG20127, &UA2583418},
	{SG20130, &UA2584854},
	{SG20131, &UA2602927},
	{SG20132, &UA2667635},
	{SG20133, &UA2750106},
	{SG20134, &UA2974292},
	{SG20135, &UA909256},
	{SG20136, &UA1953980},
	{SG20137, &UA2603534},
	{SG20141, &UA672313},
	{SG20142, &UA902065},
	{SG20143, &UA2199093},
	{SG20145, &UA2634037},
	{SG20146, &UA2754382},
	{SG20147, &UA2676490},
	{SG20148, &UA3126345},
	{SG20150, &UA3065382},
	{SG20151, &UA1991758},
	{SG20153, &UA2946991},
	{SG20154, &UA3016288},
	{SG20155, &UA3031210},
	{SG20156, &UA3059195},
	{SG20157, &UA2027979},
	{SG20158, &UA2454964},
	{SG20201, &UA616663},
	{SG20202, &UA890755},
	{SG20203, &UA1991867},
	{SG20207, &UA869373},
	{SG20208, &UA2788448},
	{SG20211, &UA2755137},
	{SG20213, &UA763351},
	{SG20214, &UA902212},
	{SG20215, &UA903370},
	{SG20216, &UA2337516},
	{SG20217, &UA3021253},
	{SG20220, &UA2566794},
	{SG20221, &UA2754264},
	{SG20222, &UA2567944},
	{SG20223, &UA2602925},
	{SG20224, &UA610619},
	{SG20225, &UA1960541},
	{SG20226, &UA2416911},
	{SG20227, &UA3022947},
	{SG20228, &UA692567},
	{SG20239, &UA2672745},
	{SG20240, &UA2707728},
	{SG20241, &UA2707759},
	{SG20242, &UA2712528},
	{SG20243, &UA2751669},
	{SG20244, &UA2780935},
	{SG20245, &UA2936387},
	{SG20248, &UA2753547},
	{SG20252, &UA2680916},
	{SG20253, &UA2783944},
	{SG20254, &UA2799861},
	{SG20255, &UA2868934},
	{SG20256, &UA2892791},
	{SG20257, &UA2958246},
	{SG20258, &UA2958889},
	{SG20259, &UA3088566},
	{SG20262, &UA2026270},
	{SG20265, &UA2676177},
	{SG20266, &UA455323},
	{SG20267, &UA767869},
	{SG20268, &UA896718},
	{SG20269, &UA2027268},
	{SG20270, &UA3085053},
	{SG20272, &UA2620539},
	{SG20273, &UA2202578},
	{SG20291, &UA2974646},
	{SG20293, &UA2975570},
	{SG20295, &UA3023029},
	{SG20298, &UA2551788},
	{SG20299, &UA2653695},
	{SG20300, &UA2044083},
	{SG20301, &UA2401396},
	{SG20307, &UA2669082},
	{SG20308, &UA2859806},
	{SG20313, &UA3081115},
	{SG20318, &UA2384991},
	{SG20319, &UA3067643},
	{SG20320, &UA3065595},
	{SG20321, &UA3081120},
	{SG20322, &UA3081394},
	{SG20323, &UA3083080},
	{SG20324, &UA3083332},
	{SG20327, &UA2011090},
	{SG20329, &UA2566621},
	{SG20330, &UA2566825},
	{SG20331, &UA2566920},
	{SG20333, &UA2567346},
	{SG20334, &UA2583561},
	{SG20335, &UA2566513},
	{SG20336, &UA2567520},
	{SG20337, &UA3067054},
	{SG20338, &UA3081784},
	{SG20339, &UA3094184},
	{SG20348, &UA609458},
	{SG20350, &UA907295},
	{SG20351, &UA2974114},
	{SG20352, &UA2974648},
	{SG20353, &UA2976089},
	{SG20354, &UA2754075},
	{SG20355, &UA2755533},
	{SG20356, &UA2783686},
	{SG20357, &UA2676667},
	{SG20366, &UA2030912},
	{SG20367, &UA3068194},
	{SG20368, &UA905828},
	{SG20369, &UA2754384},
	{SG20370, &UA2757188},
	{SG20371, &UA2974502},
	{SG20372, &UA766084},
	{SG20373, &UA906653},
	{SG20374, &UA912271},
	{SG20375, &UA2024546},
	{SG20376, &UA2868777},
	{SG20377, &UA2956886},
	{SG20378, &UA2996239},
	{SG20379, &UA3048612},
	{SG20381, &UA822126},
	{SG20383, &UA2034253},
	{SG20384, &UA2609800},
	{SG20385, &UA2610234},
	{SG20386, &UA2611904},
	{SG20387, &UA2612675},
	{SG20388, &UA2615895},
	{SG20389, &UA2618519},
	{SG20390, &UA2622100},
	{SG20391, &UA2622329},
	{SG20393, &UA2630289},
	{SG20404, &UA3022632},
	{SG20405, &UA879216},
	{SG20406, &UA902696},
	{SG20407, &UA2753958},
	{SG20408, &UA2754479},
	{SG20412, &UA2601192},
	{SG20414, &UA2707254},
	{SG20416, &UA2708394},
	{SG20418, &UA2732946},
	{SG20419, &UA2733057},
	{SG20420, &UA2750272},
	{SG20421, &UA2794474},
	{SG20422, &UA2862256},
	{SG20423, &UA2989982},
	{SG20425, &UA906404},
	{SG20426, &UA2981098},
	{SG20428, &UA2676921},
	{SG20429, &UA2361448},
	{SG20430, &UA2583234},
	{SG20431, &UA2844882},
	{SG20432, &UA2882591},
	{SG20433, &UA3061213},
	{SG20434, &UA2567576},
	{SG20435, &UA2974238},
	{SG20436, &UA2567222},
	{SG20437, &UA1958091},
	{SG20438, &UA2027669},
	{SG20439, &UA2257611},
	{SG20440, &UA2574685},
	{SG20441, &UA2788755},
	{SG20442, &UA2791412},
	{SG20445, &UA2783420},
	{SG20447, &UA2566635},
	{SG20460, &UA2365867},
	{SG20463, &UA909361},
	{SG20465, &UA781050},
	{SG20466, &UA1985887},
	{SG20468, &UA2027438},
	{SG20469, &UA2360227},
	{SG20470, &UA2474664},
	{SG20473, &UA2974803},
	{SG20474, &UA2975700},
	{SG20475, &UA2975796},
	{SG20476, &UA2975813},
	{SG20477, &UA2976013},
	{SG20478, &UA2879313},
	{SG20482, &UA2676101},
	{SG20487, &UA2756587},
	{SG20488, &UA2023750},
	{SG20489, &UA2609598},
	{SG20491, &UA2754647},
	{SG20493, &UA2799212},
	{SG20502, &UA2975956},
	{SG20503, &UA3023425},
	{SG20504, &UA669638},
	{SG20509, &UA2958558},
	{SG20510, &UA723281},
	{SG20511, &UA2446473},
	{SG20513, &UA2659348},
	{SG20514, &UA2985118},
	{SG20515, &UA2566810},
	{SG20516, &UA2567402},
	{SG20525, &UA2402904},
	{SG20527, &UA2974087},
	{SG20528, &UA840823},
	{SG20529, &UA2025190},
	{SG20530, &UA2033405},
	{SG20531, &UA2348183},
	{SG20532, &UA2519437},
	{SG20534, &UA2525088},
	{SG20536, &UA2526920},
	{SG20537, &UA2652179},
	{SG20538, &UA2652700},
	{SG20539, &UA2692482},
	{SG20540, &UA2763846},
	{SG20541, &UA2837611},
	{SG20542, &UA2981614},
	{SG20546, &UA3065103},
	{SG20561, &UA2223445},
	{SG20562, &UA2380618},
	{SG20563, &UA3043440},
	{SG20564, &UA2566914},
	{SG20565, &UA2567688},
	{SG20566, &UA2200751},
	{SG20574, &UA2360153},
	{SG20576, &UA2026986},
	{SG20577, &UA2353532},
	{SG20581, &UA2676080},
	{SG20582, &UA2367721},
	{SG20583, &UA2467763},
	{SG20584, &UA2192190},
	{SG20585, &UA1988077},
	{SG20587, &UA3021260},
	{SG20588, &UA3022005},
	{SG20598, &UA3107821},
	{SG20599, &UA1996483},
	{SG20600, &UA2676249},
	{SG20601, &UA2862326},
	{SG20602, &UA2712461},
	{SG20603, &UA2713239},
	{SG20604, &UA2734820},
	{SG20605, &UA2750859},
	{SG20608, &UA2864510},
	{SG20619, &UA2734937},
	{SG20631, &UA895681},
	{SG20632, &UA896034},
	{SG20633, &UA3067601},
	{SG20636, &UA1995256},
	{SG20637, &UA2029649},
	{SG20638, &UA2032564},
	{SG20639, &UA2261457},
	{SG20642, &UA2026049},
	{SG20643, &UA2364526},
	{SG20650, &UA1991034},
	{SG20651, &UA2567245},
	{SG20652, &UA2567261},
	{SG20653, &UA2585176},
	{SG20654, &UA2673944},
	{SG20655, &UA2676008},
	{SG20657, &UA2707087},
	{SG20659, &UA908786},
	{SG20660, &UA911457},
	{SG20661, &UA902541},
	{SG20662, &UA2735873},
	{SG20664, &UA2755335},
	{SG20670, &UA2677173},
	{SG20671, &UA2707351},
	{SG20674, &UA2234378},
	{SG20675, &UA2384594},
	{SG20676, &UA2753613},
	{SG20677, &UA2754578},
	{SG20678, &UA2770278},
	{SG20681, &UA895202},
	{SG20682, &UA905781},
	{SG20684, &UA1996697},
	{SG20685, &UA2257274},
	{SG20690, &UA2362430},
	{SG20698, &UA2366064},
	{SG20702, &UA2756006},
	{SG20703, &UA2975541},
	{SG20704, &UA3021829},
	{SG20705, &UA2566703},
	{SG20706, &UA2602011},
	{SG20707, &UA2755269},
	{SG20708, &UA2782613},
	{SG20712, &UA2181911},
	{SG20714, &UA2550829},
	{SG20716, &UA2937889},
	{SG20718, &UA3126219},
	{SG20719, &UA2676015},
	{SG20720, &UA2676752},
	{SG20721, &UA2751344},
	{SG20728, &UA3126958},
	{SG20730, &UA588758},
	{SG20731, &UA2029272},
	{SG20732, &UA2031776},
	{SG20735, &UA3068328},
	{SG20737, &UA3081038},
	{SG20738, &UA3081389},
	{SG20742, &UA2753208},
	{SG20743, &UA2753174},
	{SG20744, &UA2753771},
	{SG20745, &UA2755087},
	{SG20746, &UA2755265},
	{SG20750, &UA2365607},
	{SG20751, &UA2420411},
	{SG20753, &UA600190},
	{SG20755, &UA603596},
	{SG20757, &UA661597},
	{SG20758, &UA905439},
	{SG20759, &UA2199523},
	{SG20760, &UA2366387},
	{SG20762, &UA2532294},
	{SG20764, &UA2881988},
	{SG20765, &UA3080145},
	{SG20766, &UA2676435},
	{SG20767, &UA2734741},
	{SG20769, &UA909100},
	{SG20770, &UA2755356},
	{SG20771, &UA2782904},
	{SG20772, &UA3065476},
	{SG20773, &UA3081473},
	{SG20774, &UA2973490},
	{SG20775, &UA2974064},
	{SG20783, &UA2676942},
	{SG20786, &UA609907},
	{SG20787, &UA901499},
	{SG20788, &UA2444905},
	{SG20791, &UA2973895},
	{SG20792, &UA2974066},
	{SG20793, &UA3021720},
	{SG20809, &UA3126360},
	{SG20812, &UA659105},
	{SG20813, &UA826462},
	{SG20814, &UA2033918},
	{SG20815, &UA2754415},
	{SG20816, &UA1956732},
	{SG20817, &UA1995173},
	{SG20818, &UA2026831},
	{SG20819, &UA2028505},
	{SG20820, &UA2028565},
	{SG20821, &UA2028636},
	{SG20822, &UA2028782},
	{SG20823, &UA2029278},
	{SG20825, &UA2230836},
	{SG20826, &UA2458077},
	{SG20827, &UA2641110},
	{SG20828, &UA2850295},
	{SG20829, &UA900386},
	{SG20830, &UA904606},
	{SG20831, &UA1969848},
	{SG20832, &UA1968603},
	{SG20833, &UA1995593},
	{SG20834, &UA2012023},
	{SG20835, &UA2031374},
	{SG20836, &UA2239160},
	{SG20837, &UA2368093},
	{SG20838, &UA2387396},
	{SG20839, &UA2795699},
	{SG20840, &UA2899258},
	{SG20848, &UA899048},
	{SG20849, &UA2029579},
	{SG20851, &UA1992924},
	{SG20852, &UA2030918},
	{SG20853, &UA3061490},
	{SG20854, &UA3064980},
	{SG20856, &UA3080319},
	{SG20857, &UA2676334},
	{SG20858, &UA2767782},
	{SG20859, &UA2752968},
	{SG20860, &UA2950014},
	{SG20868, &UA2228695},
	{SG20873, &UA2673365},
	{SG20874, &UA2676441},
	{SG20876, &UA1977303},
	{SG20878, &UA2770349},
	{SG20879, &UA2781646},
	{SG20880, &UA710046},
	{SG20881, &UA1952765},
	{SG20895, &UA2637871},
	{SG20896, &UA2566898},
	{SG20897, &UA2567014},
	{SG20898, &UA902391},
	{SG20900, &UA903432},
	{SG20902, &UA403355},
	{SG20903, &UA2221007},
	{SG20904, &UA2339400},
	{SG20905, &UA2787454},
	{SG20907, &UA2950725},
	{SG20911, &UA2566681},
	{SG20912, &UA2583959},
	{SG20913, &UA2603903},
	{SG20914, &UA2029362},
	{SG20915, &UA2708562},
	{SG20929, &UA2976110},
	{SG20930, &UA3022738},
	{SG20931, &UA2031569},
	{SG20933, &UA2177490},
	{SG20934, &UA2222935},
	{SG20935, &UA2413227},
	{SG20936, &UA2686461},
	{SG20937, &UA2753790},
	{SG20940, &UA2676676},
	{SG20942, &UA2567129},
	{SG20943, &UA2567378},
	{SG20948, &UA2422017},
	{SG20966, &UA2432152},
	{SG20969, &UA2026713},
	{SG20970, &UA2411225},
	{SG20971, &UA2554513},
	{SG20972, &UA2726872},
	{SG20973, &UA2950726},
	{SG20983, &UA2816121},
	{SG20984, &UA609631},
	{SG20985, &UA901585},
	{SG20986, &UA1965203},
	{SG20987, &UA2674430},
	{SG20989, &UA2675268},
	{SG20990, &UA2675521},
	{SG20991, &UA2675846},
	{SG20992, &UA2676437},
	{SG20993, &UA1987267},
	{SG20994, &UA2588263},
	{SG20998, &UA1960816},
	{SG21000, &UA1960316},
	{SG21002, &UA2976093},
	{SG21003, &UA3023767},
	{SG21005, &UA1995554},
	{SG21007, &UA2609352},
	{SG21008, &UA2610133},
	{SG21009, &UA2612402},
	{SG21010, &UA2257297},
	{SG21011, &UA2676726},
	{SG21012, &UA2566701},
	{SG21013, &UA2583369},
	{SG21014, &UA2989791},
	{SG21016, &UA3021557},
	{SG21017, &UA2861683},
	{SG21018, &UA3065617},
	{SG21019, &UA3110342},
	{SG21020, &UA2752949},
	{SG21021, &UA2754170},
	{SG21022, &UA2676501},
	{SG21032, &UA2167285},
	{SG21033, &UA2567315},
	{SG21034, &UA2567513},
	{SG21035, &UA2567568},
	{SG21036, &UA789322},
	{SG21037, &UA1957189},
	{SG21038, &UA2026822},
	{SG21039, &UA2676294},
	{SG21043, &UA2673062},
	{SG21044, &UA2674240},
	{SG21045, &UA2676190},
	{SG21047, &UA1967796},
	{SG21048, &UA2235124},
	{SG21049, &UA2830505},
	{SG21050, &UA2991586},
	{SG21051, &UA3091843},
	{SG21054, &UA3067595},
	{SG21055, &UA3111241},
	{SG21057, &UA2257295},
	{SG21060, &UA2422446},
	{SG21061, &UA2844909},
	{SG21062, &UA877486},
	{SG21063, &UA2588654},
	{SG21064, &UA2242269},
	{SG21065, &UA2376070},
	{SG21066, &UA2455546},
	{SG21067, &UA2835316},
	{SG21068, &UA2754065},
	{SG21069, &UA2755527},
	{SG21073, &UA2755543},
	{SG21074, &UA2770319},
	{SG21075, &UA2877104},
	{SG21078, &UA2566646},
	{SG21079, &UA2567706},
	{SG21080, &UA2583230},
	{SG21081, &UA2601159},
	{SG21083, &UA715547},
	{SG21084, &UA906026},
	{SG21085, &UA1960314},
	{SG21086, &UA1992255},
	{SG21087, &UA2608297},
	{SG21088, &UA2676828},
	{SG21089, &UA2676979},
	{SG21090, &UA2257276},
	{SG21091, &UA2602807},
	{SG21093, &UA2030110},
	{SG21094, &UA2382589},
	{SG21096, &UA910925},
	{SG21097, &UA2180558},
	{SG21098, &UA2726429},
	{SG21101, &UA1966394},
	{SG21102, &UA2496357},
	{SG21104, &UA2981138},
	{SG21108, &UA2028103},
	{SG21111, &UA1987204},
	{SG21112, &UA2026922},
	{SG21113, &UA2374797},
	{SG21120, &UA807542},
	{SG21121, &UA899097},
	{SG21122, &UA2034195},
	{SG21123, &UA2573441},
	{SG21129, &UA2973466},
	{SG21130, &UA2974249},
	{SG21131, &UA3023624},
	{SG21132, &UA1984688},
	{SG21133, &UA2676671},
	{SG21134, &UA2676674},
	{SG21136, &UA1993033},
	{SG21137, &UA1995685},
	{SG21138, &UA2466325},
	{SG21140, &UA2257240},
	{SG21145, &UA2366753},
	{SG21146, &UA2457801},
	{SG21147, &UA3063995},
	{SG21149, &UA3081181},
	{SG21150, &UA3081340},
	{SG21151, &UA3083611},
	{SG21152, &UA2773897},
	{SG21154, &UA2261485},
	{SG21156, &UA3065338},
	{SG21158, &UA2511906},
	{SG21159, &UA2602959},
	{SG21160, &UA2661606},
	{SG21163, &UA1988679},
	{SG21164, &UA2257328},
	{SG21169, &UA2261488},
	{SG21170, &UA2466066},
	{SG21177, &UA2957370},
	{SG21179, &UA3082170},
	{SG21181, &UA911718},
	{SG21183, &UA2467892},
	{SG21185, &UA2847278},
	{SG21186, &UA3051080},
	{SG21187, &UA2753545},
	{SG21188, &UA2754483},
	{SG21189, &UA2754640},
	{SG21191, &UA2975059},
	{SG21192, &UA2975715},
	{SG21193, &UA2975861},
	{SG21194, &UA3021964},
	{SG21201, &UA765350},
	{SG21202, &UA2412242},
	{SG21204, &UA2975072},
	{SG21205, &UA2583341},
	{SG21207, &UA2235419},
	{SG21216, &UA2622942},
	{SG21224, &UA2426025},
	{SG21225, &UA2557102},
	{SG21229, &UA675726},
	{SG21230, &UA2032069},
	{SG21232, &UA2566924},
	{SG21234, &UA2555067},
	{SG21238, &UA903262},
	{SG21239, &UA2646228},
	{SG21240, &UA2030560},
	{SG21241, &UA2985698},
	{SG21247, &UA2224084},
	{SG21248, &UA2475760},
	{SG21251, &UA3080461},
	{SG21252, &UA3080934},
	{SG21253, &UA3094971},
	{SG21257, &UA3080981},
	{SG21259, &UA3082801},
	{SG21263, &UA2403358},
	{SG21264, &UA2464593},
	{SG21271, &UA1969889},
	{SG21273, &UA2666776},
	{SG21281, &UA2633756},
	{SG21282, &UA363967},
	{SG21283, &UA668468},
	{SG21287, &UA2975934},
	{SG21288, &UA2428539},
	{SG21289, &UA2838638},
	{SG21291, &UA609486},
	{SG21292, &UA900916},
	{SG21293, &UA2024743},
	{SG21295, &UA2583205},
	{SG21298, &UA2205330},
	{SG21299, &UA2257400},
	{SG21304, &UA2347835},
	{SG21307, &UA2567505},
	{SG21308, &UA2861249},
	{SG21309, &UA2567087},
	{SG21312, &UA680566},
	{SG21313, &UA2383912},
	{SG21314, &UA2398029},
	{SG21315, &UA2419722},
	{SG21316, &UA2478914},
	{SG21317, &UA2686986},
	{SG21318, &UA2707421},
	{SG21319, &UA824235},
	{SG21320, &UA829209},
	{SG21321, &UA2412965},
	{SG21322, &UA2611529},
	{SG21323, &UA2239277},
	{SG21324, &UA2673788},
	{SG21330, &UA2025290},
	{SG21331, &UA2027854},
	{SG21332, &UA2029520},
	{SG21334, &UA3068395},
	{SG21338, &UA657621},
	{SG21339, &UA2158571},
	{SG21340, &UA1980341},
	{SG21341, &UA2672309},
	{SG21342, &UA2676665},
	{SG21352, &UA2675059},
	{SG21353, &UA2676350},
	{SG21354, &UA2750907},
	{SG21359, &UA2257259},
	{SG21360, &UA2378926},
	{SG21362, &UA2382101},
	{SG21363, &UA2383432},
	{SG21364, &UA3085659},
	{SG21365, &UA2176688},
	{SG21366, &UA3080081},
	{SG21367, &UA3083594},
	{SG21368, &UA3083836},
	{SG21369, &UA902553},
	{SG21372, &UA2362291},
	{SG21373, &UA2012939},
	{SG21374, &UA2634773},
	{SG21375, &UA2604077},
	{SG21380, &UA833404},
	{SG21381, &UA2030820},
	{SG21382, &UA2619963},
	{SG21385, &UA2566886},
	{SG21386, &UA616685},
	{SG21388, &UA2206122},
	{SG21389, &UA2566936},
	{SG21390, &UA2583317},
	{SG21391, &UA2427916},
	{SG21392, &UA2483870},
	{SG21393, &UA2505731},
	{SG21394, &UA2711478},
	{SG21395, &UA2753867},
	{SG21396, &UA2769945},
	{SG21397, &UA2893125},
	{SG21398, &UA3068263},
	{SG21399, &UA2025345},
	{SG21400, &UA2482049},
	{SG21407, &UA2029736},
	{SG21410, &UA3062556},
	{SG21412, &UA2755428},
	{SG21413, &UA2756753},
	{SG21414, &UA2770057},
	{SG21415, &UA2800285},
	{SG21420, &UA2257334},
	{SG21421, &UA2746458},
	{SG21423, &UA688637},
	{SG21424, &UA2032857},
	{SG21425, &UA2840441},
	{SG21426, &UA2460287},
	{SG21427, &UA2659800},
	{SG21428, &UA3099240},
	{SG21430, &UA2670736},
	{SG21431, &UA2672542},
	{SG21432, &UA2678423},
	{SG21434, &UA2356690},
	{SG21436, &UA2714583},
	{SG21437, &UA3033285},
	{SG21439, &UA2632900},
	{SG21440, &UA2774298},
	{SG21443, &UA2221753},
	{SG21444, &UA2975577},
	{SG21445, &UA2976091},
	{SG21446, &UA3023514},
	{SG21448, &UA2752936},
	{SG21449, &UA3065585},
	{SG21456, &UA2025293},
	{SG21457, &UA2025792},
	{SG21462, &UA896821},
	{SG21463, &UA1952343},
	{SG21464, &UA3080699},
	{SG21465, &UA909644},
	{SG21466, &UA2375851},
	{SG21477, &UA2566800},
	{SG21479, &UA2458132},
	{SG21485, &UA2581549},
	{SG21486, &UA2672029},
	{SG21487, &UA2974085},
	{SG21491, &UA664026},
	{SG21492, &UA1979865},
	{SG21493, &UA1974292},
	{SG21494, &UA2442995},
	{SG21495, &UA2675249},
	{SG21496, &UA2676042},
	{SG21497, &UA2677303},
	{SG21498, &UA2706638},
	{SG21500, &UA2676036},
	{SG21501, &UA2676039},
	{SG21505, &UA2672966},
	{SG21506, &UA2567320},
	{SG21508, &UA2257271},
	{SG21509, &UA2551585},
	{SG21510, &UA1997017},
	{SG21511, &UA2031880},
	{SG21512, &UA901931},
	{SG21513, &UA2028738},
	{SG21515, &UA2026900},
	{SG21516, &UA2410329},
	{SG21518, &UA804971},
	{SG21519, &UA2734883},
	{SG21520, &UA2738335},
	{SG21523, &UA1994363},
	{SG21528, &UA2428179},
	{SG21529, &UA2621593},
	{SG21530, &UA2362305},
	{SG21531, &UA2416683},
	{SG21532, &UA2637793},
	{SG21540, &UA899044},
	{SG21541, &UA2032250},
	{SG21542, &UA362088},
	{SG21543, &UA630926},
	{SG21544, &UA863523},
	{SG21545, &UA805383},
	{SG21546, &UA2543399},
	{SG21552, &UA2773099},
	{SG21553, &UA2784798},
	{SG21554, &UA1970670},
	{SG21555, &UA2684990},
	{SG21556, &UA2028036},
	{SG21557, &UA2466706},
	{SG21560, &UA2202074},
	{SG21561, &UA2424490},
	{SG21562, &UA2538344},
	{SG21563, &UA2674423},
	{SG21565, &UA2676576},
	{SG21567, &UA2707911},
	{SG21568, &UA2947558},
	{SG21569, &UA2966604},
	{SG21571, &UA2975531},
	{SG21573, &UA1998273},
	{SG21576, &UA2566959},
	{SG21577, &UA3020944},
	{SG21578, &UA2033228},
	{SG21583, &UA2753012},
	{SG21584, &UA1961158},
	{SG21585, &UA1963990},
	{SG21586, &UA2261473},
	{SG21587, &UA2521780},
	{SG21588, &UA1966994},
	{SG21592, &UA2567121},
	{SG21593, &UA2973964},
	{SG21594, &UA3021167},
	{SG21597, &UA758765},
	{SG21598, &UA1993478},
	{SG21599, &UA2553450},
	{SG21601, &UA2865758},
	{SG21602, &UA3000722},
	{SG21603, &UA3092702},
	{SG21604, &UA2166644},
	{SG21605, &UA2230894},
	{SG21606, &UA3098585},
	{SG21629, &UA2769803},
	{SG21631, &UA3081242},
	{SG21638, &UA2705117},
	{SG21645, &UA3093496},
	{SG21647, &UA3022548},
	{SG21649, &UA1971281},
	{SG21651, &UA2349786},
	{SG21652, &UA2933277},
	{SG21653, &UA3068251},
	{SG21657, &UA2490520},
	{SG21658, &UA2508003},
	{SG21666, &UA899382},
	{SG21667, &UA1972214},
	{SG21669, &UA1964207},
	{SG21670, &UA2339274},
	{SG21671, &UA2399246},
	{SG21677, &UA2566529},
	{SG21684, &UA923865},
	{SG21691, &UA2032785},
	{SG21695, &UA764509},
	{SG21696, &UA902500},
	{SG21697, &UA1961540},
	{SG21698, &UA3067180},
	{SG21701, &UA864759},
	{SG21702, &UA3068393},
	{SG21703, &UA2975734},
	{SG21704, &UA2974068},
	{SG21705, &UA1993946},
	{SG21706, &UA2034072},
	{SG21707, &UA369902},
	{SG21708, &UA2026629},
	{SG21714, &UA1982321},
	{SG21715, &UA900497},
	{SG21716, &UA2030273},
	{SG21717, &UA2783082},
	{SG21718, &UA3083192},
	{SG21725, &UA2223646},
	{SG21726, &UA2754220},
	{SG21732, &UA3022701},
	{SG21733, &UA1993086},
	{SG21749, &UA2403740},
	{SG21750, &UA2754643},
	{SG21751, &UA2754650},
	{SG21755, &UA1966075},
	{SG21757, &UA895460},
	{SG21759, &UA584321},
	{SG21760, &UA675762},
	{SG21763, &UA814074},
	{SG21764, &UA2024233},
	{SG21769, &UA2753288},
	{SG21770, &UA902366},
	{SG21772, &UA2703537},
	{SG21773, &UA2869749},
	{SG21774, &UA3005447},
	{SG21775, &UA3087251},
	{SG21776, &UA2673183},
	{SG21777, &UA2936160},
	{SG21782, &UA2031964},
	{SG21783, &UA2566494},
	{SG21786, &UA2574721},
	{SG21790, &UA2769038},
	{SG21793, &UA2756087},
	{SG21796, &UA3081625},
	{SG21797, &UA3083606},
	{SG21798, &UA3126318},
	{SG21801, &UA3021574},
	{SG21802, &UA2026440},
	{SG21805, &UA2783535},
	{SG21810, &UA2261459},
	{SG21820, &UA2483373},
	{SG21822, &UA2194320},
	{SG21826, &UA2670728},
	{SG21827, &UA2672515},
	{SG21828, &UA3065782},
	{SG21829, &UA3083134},
	{SG21832, &UA2974686},
	{SG21836, &UA2228559},
	{SG21837, &UA2566918},
	{SG21839, &UA2632354},
	{SG21840, &UA2974720},
	{SG21841, &UA3021717},
	{SG21842, &UA3028229},
	{SG21849, &UA2796391},
	{SG21850, &UA2893202},
	{SG21855, &UA2676932},
	{SG21860, &UA2676917},
	{SG21863, &UA1982375},
	{SG21867, &UA608695},
	{SG21868, &UA2383352},
	{SG21869, &UA2619989},
	{SG21871, &UA2632838},
	{SG21872, &UA2899408},
	{SG21876, &UA2618435},
	{SG21879, &UA2582690},
	{SG21882, &UA369623},
	{SG21883, &UA760699},
	{SG21885, &UA727867},
	{SG21887, &UA2257327},
	{SG21888, &UA2566998},
	{SG21889, &UA2974321},
	{SG21891, &UA2974927},
	{SG21894, &UA2752967},
	{SG21895, &UA2753283},
	{SG21898, &UA3082912},
	{SG21899, &UA3083544},
	{SG21900, &UA3080216},
	{SG21904, &UA2676273},
	{SG21905, &UA2754501},
	{SG21909, &UA858586},
	{SG21911, &UA2623327},
	{SG21913, &UA2711298},
	{SG21914, &UA2226868},
	{SG21917, &UA1955997},
	{SG21918, &UA2180443},
	{SG21919, &UA2974871},
	{SG21920, &UA2974642},
	{SG21921, &UA3023805},
	{SG21922, &UA2479819},
	{SG21924, &UA2976098},
	{SG21932, &UA911562},
	{SG21934, &UA369309},
	{SG21935, &UA412397},
	{SG21937, &UA2033967},
	{SG21940, &UA2769770},
	{SG21941, &UA900286},
	{SG21942, &UA2743018},
	{SG21944, &UA2782597},
	{SG21949, &UA3063905},
	{SG21960, &UA905664},
	{SG21963, &UA2752947},
	{SG21964, &UA2754624},
	{SG21965, &UA2782728},
	{SG21966, &UA2867362},
	{SG21972, &UA2675075},
	{SG21973, &UA2974472},
	{SG21974, &UA3023613},
	{SG21981, &UA2030520},
	{SG21986, &UA2708912},
	{SG21988, &UA2772914},
	{SG21993, &UA2566907},
	{SG21994, &UA1978620},
	{SG21995, &UA2025704},
	{SG21996, &UA2030355},
	{SG21997, &UA2375836},
	{SG21999, &UA2520488},
	{SG22001, &UA2676233},
	{SG22003, &UA2444013},
	{SG22004, &UA2497770},
	{SG22006, &UA438360},
	{SG22007, &UA2567249},
	{SG22008, &UA2421044},
	{SG22009, &UA2654355},
	{SG22011, &UA3059374},
	{SG22012, &UA2659092},
	{SG22014, &UA2799086},
	{SG22024, &UA2567100},
	{SG22025, &UA2583966},
	{SG22027, &UA3083191},
	{SG22029, &UA3023242},
	{SG22030, &UA2428536},
	{SG22031, &UA2847536},
	{SG22032, &UA2708972},
	{SG22033, &UA2782668},
	{SG22036, &UA2567960},
	{SG22037, &UA2584157},
	{SG22038, &UA2732907},
	{SG22042, &UA2670732},
	{SG22043, &UA2257321},
	{SG22044, &UA2737774},
	{SG22046, &UA2257352},
	{SG22048, &UA2257379},
	{SG22054, &UA2485213},
	{SG22055, &UA2676937},
	{SG22056, &UA2466233},
	{SG22057, &UA782604},
	{SG22058, &UA2034480},
	{SG22060, &UA2709254},
	{SG22067, &UA2023987},
	{SG22068, &UA2027209},
	{SG22069, &UA2027499},
	{SG22070, &UA2194816},
	{SG22072, &UA859322},
	{SG22073, &UA2557371},
	{SG22076, &UA2773111},
	{SG22077, &UA2937471},
	{SG22079, &UA2376085},
	{SG22081, &UA2755084},
	{SG22094, &UA2558199},
	{SG22096, &UA3114882},
	{SG22097, &UA3115606},
	{SG22101, &UA1994204},
	{SG22103, &UA2974089},
	{SG22104, &UA3022842},
	{SG22106, &UA3081079},
	{SG22111, &UA2973664},
	{SG22113, &UA895285},
	{SG22116, &UA2755734},
	{SG22117, &UA2773464},
	{SG22118, &UA2892572},
	{SG22121, &UA909518},
	{SG22128, &UA2026655},
	{SG22129, &UA2028872},
	{SG22130, &UA2330877},
	{SG22131, &UA3071710},
	{SG22132, &UA905930},
	{SG22133, &UA2443702},
	{SG22134, &UA2178671},
	{SG22136, &UA2552264},
	{SG22137, &UA2950754},
	{SG22140, &UA2566926},
	{SG22141, &UA2862579},
	{SG22144, &UA582054},
	{SG22146, &UA622346},
	{SG22147, &UA820199},
	{SG22150, &UA1958299},
	{SG22151, &UA2028907},
	{SG22152, &UA1977734},
	{SG22154, &UA2624526},
	{SG22155, &UA2630434},
	{SG22157, &UA2033192},
	{SG22158, &UA2333396},
	{SG22159, &UA2417873},
	{SG22160, &UA2520320},
	{SG22162, &UA903061},
	{SG22163, &UA2704185},
	{SG22164, &UA2861423},
	{SG22166, &UA3031066},
	{SG22171, &UA2756040},
	{SG22175, &UA3080330},
	{SG22177, &UA2450444},
	{SG22179, &UA2755194},
	{SG22180, &UA2404187},
	{SG22181, &UA3127202},
	{SG22186, &UA2566941},
	{SG22187, &UA2028152},
	{SG22188, &UA2030762},
	{SG22189, &UA2427282},
	{SG22190, &UA2755437},
	{SG22191, &UA3079697},
	{SG22192, &UA2956954},
	{SG22193, &UA2427402},
	{SG22194, &UA2484217},
	{SG22195, &UA2762964},
	{SG22197, &UA2424239},
	{SG22199, &UA3126188},
	{SG22200, &UA2774073},
	{SG22202, &UA2893198},
	{SG22207, &UA3031266},
	{SG22214, &UA2415323},
	{SG22215, &UA3042844},
	{SG22216, &UA2754710},
	{SG22223, &UA1979914},
	{SG22224, &UA2953323},
	{SG22233, &UA2893326},
	{SG22234, &UA2754821},
	{SG22236, &UA2948027},
	{SG22238, &UA3132852},
	{SG22241, &UA2257269},
	{SG22244, &UA2667240},
	{SG22246, &UA2032864},
	{SG22249, &UA2974317},
	{SG22251, &UA616668},
	{SG22252, &UA688575},
	{SG22253, &UA1937889},
	{SG22254, &UA2503670},
	{SG22256, &UA2566769},
	{SG22257, &UA1967036},
	{SG22258, &UA1996267},
	{SG22259, &UA2367215},
	{SG22262, &UA371156},
	{SG22264, &UA3033045},
	{SG22265, &UA3033332},
	{SG22266, &UA3078972},
	{SG22267, &UA3079801},
	{SG22268, &UA2029036},
	{SG22269, &UA1981085},
	{SG22270, &UA1994203},
	{SG22271, &UA2025449},
	{SG22272, &UA2713809},
	{SG22278, &UA2674223},
	{SG22281, &UA2024831},
	{SG22285, &UA2774327},
	{SG22289, &UA2583425},
	{SG22291, &UA781083},
	{SG22293, &UA2354966},
	{SG22295, &UA2033666},
	{SG22296, &UA2442173},
	{SG22297, &UA3021817},
	{SG22298, &UA2671095},
	{SG22306, &UA2737929},
	{SG22308, &UA2753335},
	{SG22309, &UA2867583},
	{SG22310, &UA2894723},
	{SG22311, &UA2673630},
	{SG22313, &UA2402131},
	{SG22316, &UA3093655},
	{SG22323, &UA2566961},
	{SG22329, &UA2167763},
	{SG22331, &UA2667580},
	{SG22333, &UA3081380},
	{SG22337, &UA2360165},
	{SG22338, &UA3080158},
	{SG22339, &UA3111381},
	{SG22344, &UA2028323},
	{SG22345, &UA2652018},
	{SG22346, &UA2032711},
	{SG22347, &UA2235821},
	{SG22348, &UA2507920},
	{SG22350, &UA2598320},
	{SG22355, &UA2676489},
	{SG22358, &UA3022559},
	{SG22361, &UA684997},
	{SG22362, &UA2567325},
	{SG22363, &UA2583175},
	{SG22364, &UA757194},
	{SG22365, &UA1973475},
	{SG22369, &UA2754499},
	{SG22372, &UA899662},
	{SG22374, &UA2426766},
	{SG22375, &UA2507058},
	{SG22376, &UA2834887},
	{SG22389, &UA2671117},
	{SG22392, &UA2676086},
	{SG22395, &UA2502350},
	{SG22399, &UA2364814},
	{SG22400, &UA2574722},
	{SG22401, &UA2176335},
	{SG22406, &UA2029467},
	{SG22407, &UA2556262},
	{SG22408, &UA2567004},
	{SG22412, &UA2754165},
	{SG22414, &UA2868274},
	{SG22417, &UA1997334},
	{SG22419, &UA2235067},
	{SG22426, &UA2561066},
	{SG22427, &UA3114671},
	{SG22430, &UA2199123},
	{SG22431, &UA2467441},
	{SG22432, &UA2637795},
	{SG22434, &UA1992399},
	{SG22436, &UA2030173},
	{SG22443, &UA3079601},
	{SG22446, &UA3110549},
	{SG22448, &UA2583141},
	{SG22449, &UA2676031},
	{SG22450, &UA2754414},
	{SG22451, &UA3022958},
	{SG22452, &UA2676990},
	{SG22453, &UA2736611},
	{SG22457, &UA2580238},
	{SG22461, &UA2870898},
	{SG22467, &UA2223233},
	{SG22474, &UA2945953},
	{SG22477, &UA1959636},
	{SG22478, &UA2496667},
	{SG22479, &UA1984440},
	{SG22480, &UA2029532},
	{SG22482, &UA2028705},
	{SG22483, &UA2167125},
	{SG22484, &UA2719693},
	{SG22485, &UA2724908},
	{SG22488, &UA2673052},
	{SG22489, &UA2673562},
	{SG22491, &UA2708672},
	{SG22497, &UA2027927},
	{SG22500, &UA2461345},
	{SG22503, &UA608903},
	{SG22504, &UA2683977},
	{SG22506, &UA3081104},
	{SG22507, &UA3109379},
	{SG22509, &UA3095844},
	{SG22511, &UA2567702},
	{SG22513, &UA2997044},
	{SG22516, &UA1956660},
	{SG22517, &UA2652936},
	{SG22518, &UA2708204},
	{SG22519, &UA2798846},
	{SG22520, &UA3002884},
	{SG22521, &UA2783118},
	{SG22522, &UA2780038},
	{SG22523, &UA2876096},
	{SG22526, &UA2583196},
	{SG22527, &UA2667618},
	{SG22528, &UA2779952},
	{SG22530, &UA2842604},
	{SG22531, &UA2601596},
	{SG22542, &UA2031902},
	{SG22543, &UA2552527},
	{SG22556, &UA2735515},
	{SG22557, &UA3033147},
	{SG22558, &UA2974122},
	{SG22559, &UA2975032},
	{SG22561, &UA2028722},
	{SG22564, &UA796635},
	{SG22565, &UA2340405},
	{SG22567, &UA2770417},
	{SG22569, &UA2774114},
	{SG22572, &UA3082733},
	{SG22576, &UA2026673},
	{SG22577, &UA2034332},
	{SG22578, &UA2510304},
	{SG22579, &UA3081102},
	{SG22581, &UA2676420},
	{SG22582, &UA2200538},
	{SG22583, &UA2774061},
	{SG22584, &UA3033070},
	{SG22585, &UA2028315},
	{SG22586, &UA2031732},
	{SG22587, &UA2031910},
	{SG22588, &UA2032818},
	{SG22589, &UA2034560},
	{SG22590, &UA2450244},
	{SG22592, &UA2652417},
	{SG22594, &UA643456},
	{SG22595, &UA829280},
	{SG22596, &UA1969273},
	{SG22597, &UA2027523},
	{SG22601, &UA2857798},
	{SG22603, &UA2257625},
	{SG22606, &UA2344043},
	{SG22607, &UA2428498},
	{SG22609, &UA2798608},
	{SG22610, &UA2799271},
	{SG22611, &UA2958115},
	{SG22613, &UA3020921},
	{SG22618, &UA1962917},
	{SG22620, &UA2452416},
	{SG22622, &UA2549174},
	{SG22623, &UA2647751},
	{SG22625, &UA2694003},
	{SG22626, &UA2762350},
	{SG22628, &UA2892591},
	{SG22632, &UA2030087},
	{SG22634, &UA2414072},
	{SG22636, &UA2414799},
	{SG22637, &UA2644154},
	{SG22640, &UA2509838},
	{SG22641, &UA3017009},
	{SG22644, &UA2671144},
	{SG22645, &UA2773817},
	{SG22646, &UA2937397},
	{SG22647, &UA3080075},
	{SG22648, &UA690993},
	{SG22649, &UA907289},
	{SG22650, &UA2430234},
	{SG22651, &UA2865825},
	{SG22652, &UA2774140},
	{SG22654, &UA2261432},
	{SG22656, &UA3064595},
	{SG22661, &UA2670884},
	{SG22666, &UA2347047},
	{SG22668, &UA2938710},
	{SG22672, &UA2028712},
	{SG22674, &UA2558038},
	{SG22675, &UA3083271},
	{SG22676, &UA2784827},
	{SG22677, &UA369555},
	{SG22678, &UA807695},
	{SG22679, &UA2027145},
	{SG22680, &UA632771},
	{SG22686, &UA2867119},
	{SG22687, &UA2383461},
	{SG22689, &UA2861106},
	{SG22691, &UA889381},
	{SG22696, &UA2774069},
	{SG22699, &UA1971811},
	{SG22706, &UA1952084},
	{SG22707, &UA2027867},
	{SG22708, &UA2817479},
	{SG22722, &UA2034535},
	{SG22727, &UA2676339},
	{SG22728, &UA2711001},
	{SG22730, &UA2872986},
	{SG22734, &UA2461098},
	{SG22735, &UA2622823},
	{SG22739, &UA2520746},
	{SG22740, &UA2737381},
	{SG22742, &UA3021675},
	{SG22749, &UA3061933},
	{SG22751, &UA2615956},
	{SG22752, &UA2994213},
	{SG22754, &UA2232448},
	{SG22756, &UA2257337},
	{SG22757, &UA2375940},
	{SG22758, &UA2447549},
	{SG22759, &UA2826931},
	{SG22760, &UA2467428},
	{SG22761, &UA2994855},
	{SG22763, &UA3080543},
	{SG22765, &UA2495048},
	{SG22766, &UA2519625},
	{SG22767, &UA2218681},
	{SG22769, &UA2976682},
	{SG22773, &UA1968503},
	{SG22774, &UA2024100},
	{SG22776, &UA3013259},
	{SG22781, &UA2384637},
	{SG22784, &UA423768},
	{SG22785, &UA821365},
	{SG22786, &UA836990},
	{SG22787, &UA2025476},
	{SG22791, &UA2782956},
	{SG22792, &UA1954220},
	{SG22793, &UA2659180},
	{SG22794, &UA2962517},
	{SG22795, &UA3058845},
	{SG22797, &UA1952922},
	{SG22799, &UA2520156},
	{SG22800, &UA2674747},
	{SG22801, &UA2677239},
	{SG22803, &UA903512},
	{SG22804, &UA1996473},
	{SG22805, &UA1974598},
	{SG22806, &UA2029276},
	{SG22807, &UA2564251},
	{SG22808, &UA2730141},
	{SG22809, &UA2764931},
	{SG22810, &UA2869283},
	{SG22811, &UA2872875},
	{SG22812, &UA1992592},
	{SG22813, &UA2032378},
	{SG22814, &UA2257541},
	{SG22818, &UA1957581},
	{SG22820, &UA2028113},
	{SG22821, &UA2655210},
	{SG22830, &UA2771079},
	{SG22832, &UA2361500},
	{SG22833, &UA2024786},
	{SG22834, &UA2027919},
	{SG22835, &UA2028822},
	{SG22836, &UA1960054},
	{SG22837, &UA2755063},
	{SG22840, &UA2616131},
	{SG22841, &UA2617412},
	{SG22842, &UA2461328},
	{SG22843, &UA2483493},
	{SG22844, &UA2566954},
	{SG22845, &UA2678414},
	{SG22846, &UA3095253},
	{SG22852, &UA1989992},
	{SG22854, &UA2028700},
	{SG22855, &UA2360713},
	{SG22856, &UA2383676},
	{SG22857, &UA2699037},
	{SG22862, &UA2573842},
	{SG22864, &UA3021821},
	{SG22866, &UA2262148},
	{SG22867, &UA2557842},
	{SG22868, &UA2614707},
	{SG22869, &UA2219272},
	{SG22870, &UA2975576},
	{SG22873, &UA2232930},
	{SG22874, &UA2336886},
	{SG22875, &UA2772993},
	{SG22876, &UA3081690},
	{SG22878, &UA2028948},
	{SG22879, &UA3101404},
	{SG22881, &UA2708721},
	{SG22882, &UA2754164},
	{SG22885, &UA3026431},
	{SG22888, &UA3094805},
	{SG22891, &UA2957622},
	{SG22894, &UA3022691},
	{SG22895, &UA3023923},
	{SG22896, &UA2755286},
	{SG22898, &UA2676742},
	{SG22900, &UA2316811},
	{SG22903, &UA2873250},
	{SG22905, &UA2257399},
	{SG22906, &UA2789052},
	{SG22907, &UA3120061},
	{SG22908, &UA2456003},
	{SG22910, &UA2974237},
	{SG22911, &UA2443216},
	{SG22912, &UA2594753},
	{SG22913, &UA2700915},
	{SG22914, &UA2758849},
	{SG22915, &UA2808927},
	{SG22916, &UA2229194},
	{SG22917, &UA2381760},
	{SG22924, &UA2421358},
	{SG22928, &UA908779},
	{SG22929, &UA2024983},
	{SG22931, &UA2351878},
	{SG22935, &UA2676922},
	{SG22936, &UA2028762},
	{SG22937, &UA2029026},
	{SG22939, &UA2712742},
	{SG22941, &UA2714163},
	{SG22942, &UA2799019},
	{SG22943, &UA2228733},
	{SG22944, &UA2417275},
	{SG22945, &UA2567343},
	{SG22947, &UA2735362},
	{SG22948, &UA2796834},
	{SG22949, &UA2754189},
	{SG22950, &UA1952609},
	{SG22951, &UA1993473},
	{SG22953, &UA3080830},
	{SG22954, &UA3083179},
	{SG22957, &UA2352508},
	{SG22958, &UA2505397},
	{SG22960, &UA2659386},
	{SG22961, &UA2753164},
	{SG22962, &UA2773277},
	{SG22964, &UA724957},
	{SG22965, &UA1962699},
	{SG22966, &UA2031864},
	{SG22967, &UA3107784},
	{SG22969, &UA2676664},
	{SG22971, &UA458361},
	{SG22972, &UA696046},
	{SG22973, &UA2024766},
	{SG22974, &UA2421126},
	{SG22975, &UA368859},
	{SG22976, &UA2030713},
	{SG22978, &UA2382370},
	{SG22979, &UA2028137},
	{SG22980, &UA3085440},
	{SG22983, &UA2368028},
	{SG22986, &UA1994845},
	{SG22987, &UA2404662},
	{SG22988, &UA2168416},
	{SG22990, &UA2403243},
	{SG22991, &UA2583295},
	{SG22993, &UA2901223},
	{SG22994, &UA2353670},
	{SG22996, &UA2579856},
	{SG22997, &UA2601332},
	{SG22998, &UA2612784},
	{SG22999, &UA2774134},
	{SG23001, &UA3022519},
	{SG23004, &UA1966898},
	{SG23005, &UA1985308},
	{SG23009, &UA2412797},
	{SG23010, &UA2620479},
	{SG23011, &UA2414831},
	{SG23012, &UA3043982},
	{SG23015, &UA2872870},
	{SG23017, &UA866986},
	{SG23018, &UA2540553},
	{SG23019, &UA2540614},
	{SG23020, &UA2540604},
	{SG23021, &UA2363720},
	{SG23031, &UA2400753},
	{SG23033, &UA2549041},
	{SG23034, &UA2652546},
	{SG23035, &UA2833935},
	{SG23036, &UA2404069},
	{SG23037, &UA2675694},
	{SG23042, &UA2728169},
	{SG23043, &UA2756277},
	{SG23044, &UA2869691},
	{SG23046, &UA412259},
	{SG23048, &UA711718},
	{SG23049, &UA2401486},
	{SG23051, &UA2774125},
	{SG23052, &UA3074117},
	{SG23055, &UA2032593},
	{SG23056, &UA2033157},
	{SG23061, &UA2787099},
	{SG23062, &UA2980413},
	{SG23064, &UA3132937},
	{SG23065, &UA2737319},
	{SG23066, &UA2933847},
	{SG23067, &UA1993783},
	{SG23068, &UA2530943},
	{SG23071, &UA2384875},
	{SG23072, &UA2417804},
	{SG23075, &UA2898792},
	{SG23076, &UA2197739},
	{SG23077, &UA2431770},
	{SG23078, &UA2603947},
	{SG23080, &UA3011586},
	{SG23081, &UA3062374},
	{SG23083, &UA2423841},
	{SG23084, &UA2478322},
	{SG23086, &UA2782831},
	{SG23087, &UA2868306},
	{SG23088, &UA1955526},
	{SG23089, &UA1989804},
	{SG23090, &UA2443983},
	{SG23091, &UA2339090},
	{SG23093, &UA2674752},
	{SG23094, &UA2714706},
	{SG23095, &UA3094800},
	{SG23097, &UA2027370},
	{SG23098, &UA2028864},
	{SG23099, &UA2336683},
	{SG23100, &UA2028261},
	{SG23103, &UA785668},
	{SG23104, &UA899377},
	{SG23108, &UA2331353},
	{SG23113, &UA3032695},
	{SG23117, &UA2504752},
	{SG23120, &UA2636276},
	{SG23121, &UA2784837},
	{SG23122, &UA3032710},
	{SG23123, &UA2033744},
	{SG23124, &UA2257479},
	{SG23127, &UA2583253},
	{SG23128, &UA2583326},
	{SG23129, &UA2027956},
	{SG23130, &UA2558819},
	{SG23131, &UA2701070},
	{SG23132, &UA2566683},
	{SG23134, &UA3026079},
	{SG23138, &UA773751},
	{SG23141, &UA2413272},
	{SG23142, &UA2775093},
	{SG23143, &UA2674445},
	{SG23145, &UA2644786},
	{SG23146, &UA2869651},
	{SG23147, &UA2879510},
	{SG23150, &UA3021725},
	{SG23153, &UA888001},
	{SG23154, &UA2810287},
	{SG23155, &UA2664849},
	{SG23161, &UA2672862},
	{SG23162, &UA2860983},
	{SG23164, &UA2504609},
	{SG23165, &UA2583397},
	{SG23166, &UA2660576},
	{SG23167, &UA2025474},
	{SG23168, &UA2809581},
	{SG23169, &UA2179280},
	{SG23170, &UA2224286},
	{SG23171, &UA2464745},
	{SG23174, &UA2754635},
	{SG23175, &UA2027662},
	{SG23176, &UA2031811},
	{SG23177, &UA2475085},
	{SG23178, &UA2033423},
	{SG23179, &UA2414065},
	{SG23181, &UA2666986},
	{SG23183, &UA2153987},
	{SG23184, &UA2677051},
	{SG23185, &UA2944561},
	{SG23186, &UA893286},
	{SG23187, &UA2222538},
	{SG23188, &UA2557287},
	{SG23189, &UA2784897},
	{SG23190, &UA3065402},
	{SG23191, &UA872498},
	{SG23192, &UA2381863},
	{SG23194, &UA2666774},
	{SG23195, &UA2779914},
	{SG23196, &UA2860694},
	{SG23197, &UA2707991},
	{SG23198, &UA2710101},
	{SG23199, &UA2734697},
	{SG23200, &UA2859962},
	{SG23201, &UA2784821},
	{SG23202, &UA2868896},
	{SG23203, &UA3037953},
	{SG23204, &UA3082964},
	{SG23205, &UA2547032},
	{SG23210, &UA2961652},
	{SG23211, &UA3019020},
	{SG23212, &UA3024237},
	{SG23213, &UA3042308},
	{SG23214, &UA2167010},
	{SG23215, &UA2470749},
	{SG23216, &UA2673351},
	{SG23218, &UA2968175},
	{SG23220, &UA2230618},
	{SG23221, &UA2384297},
	{SG23223, &UA2416934},
	{SG23229, &UA2945980},
	{SG23230, &UA3107068},
	{SG23231, &UA2969389},
	{SG23233, &UA2030174},
	{SG23234, &UA2366601},
	{SG23235, &UA2784796},
	{SG23236, &UA2784104},
	{SG23237, &UA2997557},
	{SG23238, &UA3037955},
	{SG23241, &UA890523},
	{SG23245, &UA2350376},
	{SG23247, &UA3022060},
	{SG23248, &UA3115439},
	{SG23250, &UA370351},
	{SG23251, &UA2029554},
	{SG23257, &UA2754628},
	{SG23258, &UA2799106},
	{SG23259, &UA2849214},
	{SG23260, &UA899062},
	{SG23261, &UA1989530},
	{SG23264, &UA2673231},
	{SG23265, &UA3058999},
	{SG23266, &UA3078666},
	{SG23268, &UA2023481},
	{SG23269, &UA2798768},
	{SG23270, &UA2872522},
	{SG23271, &UA2989873},
	{SG23272, &UA762419},
	{SG23273, &UA2026073},
	{SG23274, &UA2424288},
	{SG23277, &UA851797},
	{SG23278, &UA2031868},
	{SG23279, &UA2868809},
	{SG23280, &UA2999479},
	{SG23281, &UA2027762},
	{SG23286, &UA2375399},
	{SG23288, &UA2342354},
	{SG23289, &UA2456981},
	{SG23290, &UA2417039},
	{SG23291, &UA2687601},
	{SG23292, &UA2419477},
	{SG23293, &UA2033843},
	{SG23294, &UA2347796},
	{SG23300, &UA3023551},
	{SG23301, &UA3063309},
	{SG23302, &UA2466986},
	{SG23303, &UA2462061},
	{SG23304, &UA2637663},
	{SG23305, &UA2995556},
	{SG23306, &UA2583151},
	{SG23308, &UA2755421},
	{SG23310, &UA1951400},
	{SG23311, &UA2458048},
	{SG23312, &UA2602993},
	{SG23314, &UA2025119},
	{SG23315, &UA2031981},
	{SG23317, &UA2584855},
	{SG23319, &UA2676088},
	{SG23320, &UA2774158},
	{SG23321, &UA2801259},
	{SG23322, &UA369990},
	{SG23324, &UA2842883},
	{SG23326, &UA2534677},
	{SG23328, &UA2579052},
	{SG23329, &UA2632258},
	{SG23330, &UA2968277},
	{SG23331, &UA712855},
	{SG23332, &UA2447625},
	{SG23333, &UA2337850},
	{SG23334, &UA2403926},
	{SG23335, &UA2559789},
	{SG23336, &UA3060428},
	{SG23338, &UA2028362},
	{SG23339, &UA2401914},
	{SG23340, &UA2991281},
	{SG23341, &UA2944941},
	{SG23342, &UA453146},
	{SG23343, &UA2028736},
	{SG23344, &UA664287},
	{SG23345, &UA1983290},
	{SG23346, &UA2463828},
	{SG23348, &UA2752556},
	{SG23350, &UA3132843},
	{SG23351, &UA397413},
	{SG23352, &UA2223006},
	{SG23353, &UA2227736},
	{SG23355, &UA2419967},
	{SG23358, &UA2555901},
	{SG23359, &UA2695641},
	{SG23360, &UA2583074},
	{SG23361, &UA2841209},
	{SG23362, &UA2667568},
	{SG23363, &UA2708267},
	{SG23364, &UA3080484},
	{SG23365, &UA3094307},
	{SG23367, &UA2032099},
	{SG23368, &UA2734266},
	{SG23369, &UA2025929},
	{SG23371, &UA2654501},
	{SG23374, &UA3019801},
	{SG23375, &UA3092541},
	{SG23376, &UA2030485},
	{SG23377, &UA2236806},
	{SG23378, &UA2371817},
	{SG23379, &UA2382146},
	{SG23380, &UA2384558},
	{SG23382, &UA2974650},
	{SG23383, &UA3080412},
	{SG23384, &UA903425},
	{SG23385, &UA1953913},
	{SG23386, &UA2033021},
	{SG23387, &UA2737846},
	{SG23389, &UA2527417},
	{SG23390, &UA2613558},
	{SG23391, &UA2614632},
	{SG23392, &UA2636264},
	{SG23394, &UA2894298},
	{SG23395, &UA3033345},
	{SG23396, &UA3023394},
	{SG23399, &UA3131337},
	{SG23402, &UA2401798},
	{SG23403, &UA2584099},
	{SG23404, &UA2701163},
	{SG23405, &UA2798995},
	{SG23406, &UA3020858},
	{SG23408, &UA2034049},
	{SG23409, &UA2979567},
	{SG23410, &UA2028387},
	{SG23412, &UA2505135},
	{SG23413, &UA2528945},
	{SG23415, &UA3081396},
	{SG23416, &UA3094555},
	{SG23417, &UA2235096},
	{SG23418, &UA2933516},
	{SG23419, &UA2399313},
	{SG23420, &UA2769621},
	{SG23421, &UA2799330},
	{SG23422, &UA3031492},
	{SG23423, &UA2033849},
	{SG23424, &UA2423505},
	{SG23425, &UA2577306},
	{SG23426, &UA2603920},
	{SG23429, &UA2772673},
	{SG23431, &UA2869790},
	{SG23432, &UA3059446},
	{SG23433, &UA3108413},
	{SG23434, &UA2426127},
	{SG23435, &UA2257385},
	{SG23436, &UA2817620},
	{SG23440, &UA2032288},
	{SG23441, &UA2422350},
	{SG23442, &UA2424346},
	{SG23443, &UA2962129},
	{SG23445, &UA2420167},
	{SG23446, &UA2420874},
	{SG23447, &UA3062420},
	{SG23448, &UA3095636},
	{SG23449, &UA3115503},
	{SG23451, &UA2237388},
	{SG23452, &UA2383503},
	{SG23457, &UA3095061},
	{SG23463, &UA368828},
	{SG23465, &UA2378286},
	{SG23466, &UA2615344},
	{SG23468, &UA2773816},
	{SG23469, &UA2858339},
	{SG23470, &UA2937364},
	{SG23471, &UA3021197},
	{SG23472, &UA427041},
	{SG23475, &UA2032924},
	{SG23476, &UA2583225},
	{SG23477, &UA2709098},
	{SG23478, &UA2784037},
	{SG23480, &UA2349683},
	{SG23481, &UA2421061},
	{SG23482, &UA2764333},
	{SG23483, &UA3004298},
	{SG23484, &UA2769860},
	{SG23485, &UA3021443},
	{SG23486, &UA3022711},
	{SG23487, &UA3059011},
	{SG23488, &UA1954377},
	{SG23489, &UA2257376},
	{SG23490, &UA2787276},
	{SG23491, &UA2580490},
	{SG23493, &UA2860637},
	{SG23498, &UA3022586},
	{SG23499, &UA1955524},
	{SG23500, &UA2366432},
	{SG23501, &UA2167801},
	{SG23502, &UA2484449},
	{SG23503, &UA2503107},
	{SG23509, &UA777494},
	{SG23510, &UA1990222},
	{SG23511, &UA2222963},
	{SG23512, &UA2467287},
	{SG23513, &UA2504427},
	{SG23514, &UA2713628},
	{SG23515, &UA2676936},
	{SG23517, &UA2714082},
	{SG23518, &UA2772283},
	{SG23520, &UA2782780},
	{SG23521, &UA2799103},
	{SG23522, &UA2938017},
	{SG23523, &UA3065259},
	{SG23524, &UA1991745},
	{SG23525, &UA2028251},
	{SG23526, &UA2314004},
	{SG23527, &UA2024538},
	{SG23528, &UA2257485},
	{SG23529, &UA2583778},
	{SG23530, &UA2601152},
	{SG23535, &UA2026995},
	{SG23536, &UA1990693},
	{SG23537, &UA2485193},
	{SG23538, &UA2581357},
	{SG23539, &UA2764424},
	{SG23540, &UA2706258},
	{SG23541, &UA2671091},
	{SG23545, &UA2769650},
	{SG23546, &UA3059371},
	{SG23547, &UA3022963},
	{SG23548, &UA3021693},
	{SG23551, &UA2449328},
	{SG23552, &UA2616197},
	{SG23553, &UA2671120},
	{SG23554, &UA2961251},
	{SG23557, &UA2025087},
	{SG23558, &UA2029494},
	{SG23562, &UA2629543},
	{SG23563, &UA2769358},
	{SG23564, &UA3094850},
	{SG23571, &UA1961334},
	{SG23572, &UA1981587},
	{SG23574, &UA2025644},
	{SG23576, &UA2223256},
	{SG23606, &UA2565996},
	{SG23608, &UA2580506},
	{SG23612, &UA2676643},
	{SG23616, &UA2799516},
	{SG23617, &UA2893367},
	{SG23619, &UA2870894},
	{SG23620, &UA2892911},
	{SG23621, &UA2965333},
	{SG23624, &UA3023240},
	{SG23626, &UA3094149},
	{SG23630, &UA2337198},
	{SG23631, &UA2403826},
	{SG23632, &UA2657138},
	{SG23633, &UA2799207},
	{SG23641, &UA1958641},
	{SG23642, &UA1994206},
	{SG23643, &UA2400059},
	{SG23644, &UA2667768},
	{SG23645, &UA2031919},
	{SG23646, &UA2603898},
	{SG23647, &UA2675060},
	{SG23648, &UA2726687},
	{SG23649, &UA2750817},
	{SG23674, &UA803454},
	{SG23676, &UA2643850},
	{SG23678, &UA2469870},
	{SG23683, &UA3042499},
	{SG23689, &UA3062276},
	{SG23693, &UA3132648},
	{SG23706, &UA1956203},
	{SG23708, &UA2026924},
	{SG23713, &UA2316546},
	{SG23722, &UA2545821},
	{SG23724, &UA2742995},
	{SG23728, &UA2666879},
	{SG23733, &UA2752884},
	{SG23739, &UA2784835},
	{SG23741, &UA2795889},
	{SG23744, &UA2991044},
	{SG23747, &UA2799008},
	{SG23750, &UA3003066},
	{SG23755, &UA3013972},
	{SG23771, &UA1969898},
	{SG23774, &UA2482271},
	{SG23777, &UA2413226},
	{SG23786, &UA2733216},
	{SG23793, &UA2677216},
	{SG23795, &UA2708784},
	{SG23800, &UA2782854},
	{SG23801, &UA3049857},
	{SG23814, &UA2944069},
	{SG23817, &UA3023257},
	{SG23853, &UA2362901},
	{SG23855, &UA2425861},
	{SG23860, &UA2474861},
	{SG23863, &UA2525508},
	{SG23869, &UA2672378},
	{SG23879, &UA3030820},
	{SG23884, &UA3110454},
	{SG23902, &UA840851},
	{SG23903, &UA843626},
	{SG23909, &UA2350512},
	{SG23912, &UA2378547},
	{SG23915, &UA2414569},
	{SG23923, &UA2708379},
	{SG23928, &UA2799468},
	{SG23933, &UA2869925},
	{SG23935, &UA2893082},
	{SG23947, &UA3093739},
	{SG23961, &UA1959505},
	{SG23963, &UA2029512},
	{SG23965, &UA2034550},
	{SG24011, &UA2350384},
	{SG24013, &UA2410314},
	{SG24014, &UA2820219},
	{SG24015, &UA2412599},
	{SG24017, &UA2559835},
	{SG24019, &UA2637734},
	{SG24020, &UA2667065},
	{SG24021, &UA2667311},
	{SG24022, &UA2710642},
	{SG24023, &UA2779399},
	{SG24024, &UA2784074},
	{SG24026, &UA2790537},
	{SG24028, &UA3043820},
	{SG24082, &UA411991},
	{SG24090, &UA2027745},
	{SG24092, &UA2034090},
	{SG24096, &UA2379086},
	{SG24099, &UA2418132},
	{SG24101, &UA2455622},
	{SG24112, &UA2674426},
	{SG24116, &UA2879274},
	{SG24121, &UA2344768},
	{SG24124, &UA2375502},
	{SG24126, &UA2420790},
	{SG24128, &UA2455072},
	{SG24130, &UA2496545},
	{SG24131, &UA2549712},
	{SG24134, &UA2602986},
	{SG24144, &UA2959917},
	{SG24167, &UA1963632},
	{SG24170, &UA1992641},
	{SG24171, &UA2684722},
	{SG24237, &UA2707141},
	{SG24246, &UA2783419},
	{SG24252, &UA2869182},
	{SG24257, &UA2998522},
	{SG24471, &UA3132064},
	{SG24493, &UA1972878},
	{SG24500, &UA2340135},
	{SG24504, &UA2399979},
	{SG24512, &UA2534637},
	{SG24514, &UA2535010},
	{SG24516, &UA2600981},
	{SG24521, &UA2667997},
	{SG24523, &UA2708236},
	{SG24546, &UA3079669},
	{SG24560, &UA2025062},
	{SG24566, &UA2374960},
	{SG24569, &UA2467078},
	{SG24575, &UA2597133},
	{SG24585, &UA2733964},
	{SG24589, &UA2784531},
	{SG24640, &UA2384497},
	{SG24642, &UA2413203},
	{SG24643, &UA2424304},
	{SG24665, &UA2712201},
	{SG24675, &UA2782998},
	{SG24681, &UA2799429},
	{SG24684, &UA2866828},
	{SG24690, &UA2869876},
	{SG24707, &UA3058883},
	{SG24716, &UA3112543},
	{SG24738, &UA2026682},
	{SG24740, &UA2028490},
	{SG24743, &UA2034473},
	{SG24744, &UA2722027},
	{SG24750, &UA2445407},
	{SG24753, &UA2950117},
	{SG24761, &UA2579329},
	{SG24769, &UA2624095},
	{SG24778, &UA2734633},
	{SG24779, &UA2734754},
	{SG24790, &UA2866407},
	{SG24817, &UA2032394},
	{SG24820, &UA2395748},
	{SG24876, &UA2783294},
	{SG24886, &UA2960815},
	{SG24918, &UA2027582},
	{SG24920, &UA2445034},
	{SG24927, &UA2403750},
	{SG24930, &UA2424279},
	{SG24934, &UA2479840},
	{SG24944, &UA2654672},
	{SG24953, &UA2734654},
	{SG24955, &UA2750957},
	{SG24957, &UA2734901},
	{SG24965, &UA2877860},
	{SG25005, &UA2027554},
	{SG25006, &UA2034252},
	{SG25010, &UA2223405},
	{SG25024, &UA2502723},
	{SG25026, &UA2808913},
	{SG25042, &UA2676642},
	{SG25054, &UA2752423},
	{SG25071, &UA2860367},
	{SG25095, &UA3109543},
	{SG25099, &UA3126299},
	{SG25118, &UA2223895},
	{SG25141, &UA2733907},
	{SG25212, &UA3116524},
	{SG25247, &UA2028983},
	{SG25249, &UA2030721},
	{SG25259, &UA2455920},
	{SG25269, &UA2555558},
	{SG25285, &UA2707432},
	{SG25316, &UA3080482},
	{SG25320, &UA3058841},
	{SG25323, &UA3094677},
	{SG25328, &UA3109394},
	{SG25342, &UA2388012},
	{SG25344, &UA2457630},
	{SG25347, &UA2401786},
	{SG25349, &UA2429696},
	{SG25377, &UA2848390},
	{SG25407, &UA2028143},
	{SG25409, &UA2335115},
	{SG25434, &UA2744132},
	{SG25463, &UA3078374},
	{SG25481, &UA2026685},
	{SG25487, &UA2372833},
	{SG25495, &UA2599951},
	{SG25496, &UA2867966},
	{SG25497, &UA2892887},
	{SG25502, &UA3109431},
	{SG25544, &UA2553231},
	{SG25554, &UA2696377},
	{SG25566, &UA2796341},
	{SG25580, &UA3001934},
	{SG25582, &UA3020704},
	{SG25633, &UA2027024},
	{SG25635, &UA2031555},
	{SG25645, &UA2628264},
	{SG25647, &UA2622452},
	{SG25657, &UA2735512},
	{SG25675, &UA3073527},
	{SG25680, &UA2974331},
	{SG25684, &UA3009342},
	{SG25717, &UA2029207},
	{SG25726, &UA2384746},
	{SG25729, &UA2417935},
	{SG25735, &UA2477602},
	{SG25764, &UA2961826},
	{SG25770, &UA3059182},
	{SG25787, &UA3127257},
	{SG25823, &UA2376470},
	{SG25828, &UA2450590},
	{SG25832, &UA2484828},
	{SG25835, &UA2669372},
	{SG25836, &UA2779176},
	{SG25837, &UA2863234},
	{SG25838, &UA2879363},
	{SG25839, &UA2935757},
	{SG25840, &UA2937105},
	{SG25851, &UA2028721},
	{SG25852, &UA2527779},
	{SG25853, &UA2842372},
	{SG25854, &UA2866738},
	{SG25855, &UA2938429},
	{SG25856, &UA3021585},
	{SG25857, &UA3023381},
	{SG25858, &UA3109713},
	{SG25859, &UA2023926},
	{SG25860, &UA2028171},
	{SG25861, &UA2425810},
	{SG25862, &UA2466342},
	{SG25864, &UA2597327},
	{SG25865, &UA2598727},
	{SG25866, &UA2620774},
	{SG25867, &UA2652661},
	{SG25868, &UA2707307},
	{SG25869, &UA2707417},
	{SG25870, &UA2799426},
	{SG25871, &UA2877434},
	{SG25872, &UA3004364},
	{SG25873, &UA3014950},
	{SG25874, &UA2376021},
	{SG25875, &UA2383388},
	{SG25876, &UA2829946},
	{SG25877, &UA2413830},
	{SG25878, &UA2767752},
	{SG25879, &UA2667484},
	{SG25880, &UA2767240},
	{SG25881, &UA2827413},
	{SG25882, &UA2869318},
	{SG25883, &UA2997935},
	{SG25884, &UA3009015},
	{SG25885, &UA3023210},
	{SG25886, &UA593895},
	{SG25887, &UA892900},
	{SG25888, &UA2527804},
	{SG25889, &UA2602397},
	{SG25890, &UA2658308},
	{SG25891, &UA2659249},
	{SG25892, &UA2666718},
	{SG25893, &UA2667474},
	{SG25894, &UA2711813},
	{SG25895, &UA2734837},
	{SG25896, &UA2748005},
	{SG25897, &UA2789791},
	{SG25898, &UA2784253},
	{SG25899, &UA2797230},
	{SG25900, &UA2797583},
	{SG25901, &UA2799041},
	{SG25902, &UA2877420},
	{SG25903, &UA2949808},
	{SG25904, &UA3098220},
	{SG25905, &UA3004784},
	{SG25906, &UA3110915},
	{SG25907, &UA2028682},
	{SG25908, &UA2611758},
	{SG25909, &UA2395710},
	{SG25910, &UA2643578},
	{SG25911, &UA2519663},
	{SG25912, &UA2648343},
	{SG25913, &UA2577686},
	{SG25914, &UA2708121},
	{SG25915, &UA2711121},
	{SG25916, &UA2733362},
	{SG25917, &UA2794151},
	{SG25918, &UA2794495},
	{SG25919, &UA3109461},
	{SG25920, &UA2030209},
	{SG25921, &UA2594038},
	{SG25922, &UA2648019},
	{SG25923, &UA2653526},
	{SG25924, &UA2658142},
	{SG25925, &UA2708205},
	{SG25926, &UA2800232},
	{SG25927, &UA2867394},
	{SG25928, &UA2877474},
	{SG25929, &UA2998057},
	{SG25930, &UA3126515},
	{SG25931, &UA2027991},
	{SG25932, &UA2527717},
	{SG25933, &UA2765385},
	{SG25934, &UA2864571},
	{SG25935, &UA3016888},
	{SG25936, &UA3022330},
	{SG25937, &UA837007},
	{SG25938, &UA1974384},
	{SG25939, &UA3050096},
	{SG25940, &UA2318886},
	{SG25941, &UA2335397},
	{SG25942, &UA2863121},
	{SG25944, &UA2413701},
	{SG25945, &UA2446168},
	{SG25946, &UA2466812},
	{SG25947, &UA2667691},
	{SG25948, &UA2667874},
	{SG25949, &UA2833621},
	{SG25950, &UA2734849},
	{SG25951, &UA2798853},
	{SG25952, &UA2935301},
	{SG25953, &UA2938260},
	{SG25955, &UA3132610},
	{SG25956, &UA2022735},
	{SG25957, &UA1967661},
	{SG25958, &UA2384476},
	{SG25959, &UA2599585},
	{SG25960, &UA2667505},
	{SG25961, &UA2732862},
	{SG25962, &UA2867617},
	{SG25963, &UA3095535},
	{SG25964, &UA3113838},
	{SG25965, &UA3132913},
	{SG25966, &UA2505739},
	{SG25967, &UA2505742},
	{SG25968, &UA2732870},
	{SG25969, &UA2735165},
	{SG25970, &UA2765619},
	{SG25971, &UA2797901},
	{SG25972, &UA2867536},
	{SG25973, &UA3059384},
	{SG25974, &UA3084054},
	{SG25975, &UA682833},
	{SG25976, &UA2033678},
	{SG25977, &UA2574821},
	{SG25978, &UA2809417},
	{SG25979, &UA2652308},
	{SG25980, &UA2658980},
	{SG25982, &UA2730685},
	{SG25983, &UA2814774},
	{SG25985, &UA2994713},
	{SG25988, &UA3011181},
	{SG25989, &UA3049900},
	{SG25990, &UA3057539},
	{SG25991, &UA824532},
	{SG25992, &UA2413623},
	{SG25993, &UA2455807},
	{SG25994, &UA2591905},
	{SG25995, &UA2713567},
	{SG25996, &UA2799679},
	{SG25997, &UA2867212},
	{SG26023, &UA3061429},
	{SG26026, &UA3079784},
	{SG26028, &UA3092745},
	{SG26032, &UA3132948},
	{SG26033, &UA2032557},
	{SG26034, &UA2025032},
	{SG26035, &UA2029365},
	{SG26036, &UA2410608},
	{SG26037, &UA2547810},
	{SG26038, &UA2574478},
	{SG26039, &UA2579099},
	{SG26040, &UA2701371},
	{SG26041, &UA2712145},
	{SG26042, &UA2735532},
	{SG26043, &UA2867404},
	{SG26044, &UA2869560},
	{SG26045, &UA2984463},
	{SG26046, &UA2997934},
	{SG26047, &UA2998590},
	{SG26048, &UA3003369},
	{SG26049, &UA3021061},
	{SG26063, &UA763655},
	{SG26082, &UA2594649},
	{SG26087, &UA2652639},
	{SG26090, &UA2662780},
	{SG26091, &UA2667182},
	{SG26093, &UA2667692},
	{SG26111, &UA2867604},
	{SG26113, &UA2897407},
	{SG26114, &UA2937511},
	{SG26115, &UA3020692},
	{SG26116, &UA3084094},
	{SG26117, &UA3127843},
	{SG26118, &UA3103247},
	{SG26120, &UA2027476},
	{SG26121, &UA2029966},
	{SG26122, &UA2167199},
	{SG26123, &UA2418725},
	{SG26124, &UA2553068},
	{SG26125, &UA2553176},
	{SG26126, &UA2600616},
	{SG26127, &UA2618057},
	{SG26128, &UA2652823},
	{SG26129, &UA2706973},
	{SG26130, &UA2711412},
	{SG26132, &UA2951097},
	{SG26133, &UA2953576},
	{SG26134, &UA2963077},
	{SG26137, &UA3021751},
	{SG26138, &UA3023233},
	{SG26139, &UA3050475},
	{SG26140, &UA3113039},
	{SG26141, &UA2024657},
	{SG26142, &UA2581042},
	{SG26143, &UA2594515},
	{SG26145, &UA2660441},
	{SG26146, &UA2662362},
	{SG26147, &UA2695903},
	{SG26148, &UA2733032},
	{SG26150, &UA2750954},
	{SG26162, &UA2869215},
	{SG26176, &UA2964284},
	{SG26194, &UA3108907},
	{SG26232, &UA2025588},
	{SG26245, &UA2420931},
	{SG26246, &UA2448350},
	{SG26247, &UA2453243},
	{SG26248, &UA2479747},
	{SG26250, &UA2616507},
	{SG26251, &UA2625563},
	{SG26252, &UA2629941},
	{SG26253, &UA2654052},
	{SG26254, &UA2659446},
	{SG26255, &UA2667894},
	{SG26257, &UA2734649},
	{SG26258, &UA2816741},
	{SG26259, &UA2879386},
	{SG26260, &UA2891511},
	{SG26261, &UA2892823},
	{SG26264, &UA2987487},
	{SG26265, &UA2998230},
	{SG26266, &UA3000139},
	{SG26267, &UA3006603},
	{SG26275, &UA3024356},
	{SG26303, &UA589890},
	{SG26305, &UA1961108},
	{SG26309, &UA2025036},
	{SG26315, &UA2502621},
	{SG26319, &UA2552403},
	{SG26322, &UA2660126},
	{SG26323, &UA2722852},
	{SG26324, &UA2734925},
	{SG26325, &UA2735502},
	{SG26326, &UA2750844},
	{SG26327, &UA2839220},
	{SG26328, &UA2999131},
	{SG26331, &UA3110651},
	{SG26332, &UA3126956},
	{SG26393, &UA2371895},
	{SG26394, &UA2373048},
	{SG26395, &UA2420280},
	{SG26396, &UA2496681},
	{SG26397, &UA2523547},
	{SG26398, &UA2653767},
	{SG26399, &UA2659593},
	{SG26403, &UA2728067},
	{SG26404, &UA2764801},
	{SG26405, &UA2778984},
	{SG26406, &UA2791983},
	{SG26407, &UA2869285},
	{SG26408, &UA3010282},
	{SG26409, &UA3048083},
	{SG26410, &UA3127467},
	{SG26411, &UA3132428},
	{SG26413, &UA403263},
	{SG26414, &UA403293},
	{SG26415, &UA425012},
	{SG26425, &UA2223453},
	{SG26429, &UA2415038},
	{SG26435, &UA2477502},
	{SG26445, &UA2602994},
	{SG26465, &UA2661618},
	{SG26474, &UA2723662},
	{SG26480, &UA2750882},
	{SG26484, &UA2779903},
	{SG26504, &UA2896028},
	{SG26511, &UA2937815},
	{SG26516, &UA2968581},
	{SG26519, &UA2987328},
	{SG26525, &UA2997207},
	{SG26530, &UA2999196},
	{SG26539, &UA3022828},
	{SG26544, &UA3073821},
	{SG26545, &UA3131021},
	{SG26550, &UA3110476},
	{SG26554, &UA1952818},
	{SG26555, &UA2025506},
	{SG26556, &UA2033785},
	{SG26558, &UA2350372},
	{SG26561, &UA2502967},
	{SG26562, &UA2553505},
	{SG26563, &UA2590537},
	{SG26564, &UA2595700},
	{SG26565, &UA2660641},
	{SG26566, &UA2776491},
	{SG26568, &UA2866655},
	{SG26570, &UA2868499},
	{SG26572, &UA2870264},
	{SG26573, &UA2877780},
	{SG26575, &UA2957904},
	{SG26577, &UA3020050},
	{SG26578, &UA3022322},
	{SG26579, &UA3074169},
	{SG26580, &UA3077966},
	{SG26581, &UA3109621},
	{SG26582, &UA3127521},
	{SG26583, &UA682900},
	{SG26585, &UA1975522},
	{SG26586, &UA2413198},
	{SG26587, &UA2454625},
	{SG26588, &UA2609636},
	{SG26589, &UA2637894},
	{SG26590, &UA2695748},
	{SG26591, &UA2701382},
	{SG26592, &UA2703183},
	{SG26593, &UA2780019},
	{SG26594, &UA2792419},
	{SG26595, &UA2847796},
	{SG26597, &UA2879205},
	{SG26599, &UA2938044},
	{SG26603, &UA2999039},
	{SG26604, &UA3000019},
	{SG26605, &UA3000334},
	{SG26606, &UA3078835},
	{SG26627, &UA3109486},
	{SG26630, &UA3126957},
	{SG26635, &UA3128318},
	{SG26637, &UA3132424},
	{SG26639, &UA3132905},
	{SG26670, &UA2034601},
	{SG26676, &UA2476409},
	{SG26685, &UA2595114},
	{SG26697, &UA2645671},
	{SG26715, &UA2730004},
	{SG26725, &UA2746463},
	{SG26741, &UA2794613},
	{SG26751, &UA2854021},
	{SG26755, &UA2862163},
	{SG26761, &UA2891149},
	{SG26772, &UA2893036},
	{SG26777, &UA2937887},
	{SG26780, &UA2955264},
	{SG26781, &UA3014404},
	{SG26782, &UA3021969},
	{SG26808, &UA3092500},
	{SG26818, &UA3108869},
	{SG26823, &UA3114299},
	{SG26829, &UA2222567},
	{SG26834, &UA2342996},
	{SG26848, &UA2634027},
	{SG26853, &UA2652100},
	{SG26855, &UA2652553},
	{SG26860, &UA2660440},
	{SG26862, &UA2693842},
	{SG26873, &UA2765641},
	{SG26879, &UA2790067},
	{SG26884, &UA2843760},
	{SG26887, &UA2845698},
	{SG26890, &UA2854024},
	{SG26898, &UA2861685},
	{SG26902, &UA2864264},
	{SG26905, &UA3024939},
	{SG26906, &UA3049344},
	{SG26907, &UA3073401},
	{SG26908, &UA3088449},
	{SG26909, &UA3125361},
	{SG26910, &UA3126778},
	{SG26911, &UA3132382},
	{SG26914, &UA2410738},
	{SG26928, &UA2620963},
	{SG26930, &UA2686594},
	{SG26931, &UA2727015},
	{SG26932, &UA2730023},
	{SG26933, &UA2749861},
	{SG26934, &UA2833368},
	{SG26935, &UA2851731},
	{SG26936, &UA2861250},
	{SG26937, &UA2935942},
	{SG26938, &UA2938216},
	{SG26939, &UA2958469},
	{SG26940, &UA2993481},
	{SG26941, &UA3006701},
	{SG26942, &UA3050058},
	{SG26943, &UA3050612},
	{SG26944, &UA3056132},
	{SG26945, &UA3058787},
	{SG26946, &UA3088679},
	{SG26947, &UA3103240},
	{SG26948, &UA3126489},
	{SG26949, &UA3126688},
	{SG26950, &UA3126882},
	{SG26951, &UA3127165},
	{SG26952, &UA2027578},
	{SG26953, &UA2028910},
	{SG26954, &UA2615251},
	{SG26955, &UA2622148},
	{SG26956, &UA2733329},
	{SG26957, &UA2734693},
	{SG26958, &UA2832090},
	{SG26959, &UA2847611},
	{SG26960, &UA2850822},
	{SG26961, &UA2861626},
	{SG26962, &UA2863297},
	{SG26963, &UA2956491},
	{SG26964, &UA2958774},
	{SG26965, &UA2998177},
	{SG26966, &UA2999916},
	{SG26967, &UA3047768},
	{SG26968, &UA3055747},
	{SG26969, &UA3057261},
	{SG26970, &UA3072612},
	{SG26971, &UA3079804},
	{SG26972, &UA3085653},
	{SG26973, &UA3104942},
	{SG26974, &UA3105562},
	{SG26975, &UA3108012},
	{SG26976, &UA3108162},
	{SG26977, &UA3108730},
	{SG26978, &UA3118950},
	{SG26979, &UA3126182},
	{SG26980, &UA3126608},
	{SG26981, &UA1991885},
	{SG26982, &UA2167766},
	{SG26983, &UA2495852},
	{SG26984, &UA2619331},
	{SG26985, &UA2632127},
	{SG26986, &UA2650949},
	{SG26987, &UA2657753},
	{SG26988, &UA2657950},
	{SG26989, &UA2727756},
	{SG26990, &UA2826379},
	{SG26991, &UA2849267},
	{SG26992, &UA2861670},
	{SG26993, &UA2863142},
	{SG26994, &UA2863693},
	{SG26995, &UA2937841},
	{SG26996, &UA2938113},
	{SG26997, &UA2999981},
	{SG26998, &UA3022553},
	{SG26999, &UA3048566},
	{SG27000, &UA3050170},
	{SG27001, &UA3070118},
	{SG27002, &UA3087806},
	{SG27003, &UA3088017},
	{SG27004, &UA3100215},
	{SG27009, &UA3118293},
	{SG27010, &UA3121595},
	{SG27011, &UA3126210},
	{SG27012, &UA2339664},
	{SG27013, &UA2401081},
	{SG27014, &UA2444881},
	{SG27015, &UA2500649},
	{SG27017, &UA2519349},
	{SG27018, &UA2614236},
	{SG27019, &UA2637672},
	{SG27020, &UA2825707},
	{SG27021, &UA2839278},
	{SG27022, &UA2850698},
	{SG27023, &UA2862490},
	{SG27024, &UA2863595},
	{SG27025, &UA3098749},
	{SG27026, &UA623967},
	{SG27027, &UA2620035},
	{SG27028, &UA2623005},
	{SG27029, &UA2843705},
	{SG27030, &UA2844926},
	{SG27031, &UA2033457},
	{SG27032, &UA2338126},
	{SG27033, &UA2627957},
	{SG27034, &UA2641339},
	{SG27035, &UA2811319},
	{SG27036, &UA2817372},
	{SG27037, &UA2026539},
	{SG27038, &UA2628401},
	{SG27039, &UA2629186},
	{SG27040, &UA2830838},
	{SG27066, &UA2023616},
	{SG27067, &UA2447611},
	{SG27068, &UA2494709},
	{SG27070, &UA2616265},
	{SG27071, &UA2648621},
	{SG27072, &UA2823758},
	{SG27073, &UA1946415},
	{SG27074, &UA2025479},
	{SG27079, &UA2519009},
	{SG27081, &UA2608443},
	{SG27085, &UA2620267},
	{SG27087, &UA2621148},
	{SG27089, &UA2621472},
	{SG27095, &UA2684730},
	{SG27098, &UA2816698},
	{SG27108, &UA616735},
	{SG27109, &UA832269},
	{SG27110, &UA2025123},
	{SG27111, &UA2540563},
	{SG27112, &UA2627212},
	{SG27113, &UA2029777},
	{SG27129, &UA2623644},
	{SG27130, &UA2622166},
	{SG27131, &UA2815138},
	{SG27166, &UA597067},
	{SG27167, &UA857324},
	{SG27168, &UA2638001},
	{SG27171, &UA603793},
	{SG27173, &UA855379},
	{SG27192, &UA2610272},
	{SG27197, &UA2630094},
	{SG27201, &UA2629827},
	{SG27236, &UA738363},
	{SG27237, &UA758250},
	{SG27238, &UA2023951},
	{SG27241, &UA2617006},
	{SG27243, &UA2620610},
	{SG27246, &UA2625016},
	{SG27249, &UA2627750},
	{SG27257, &UA2810725},
	{SG27291, &UA399006},
	{SG27292, &UA1957328},
	{SG27293, &UA2608752},
	{SG27294, &UA2625226},
	{SG27309, &UA797960},
	{SG27310, &UA838260},
	{SG27311, &UA2026117},
	{SG27312, &UA2025003},
	{SG27314, &UA2618512},
	{SG27316, &UA2633694},
	{SG27324, &UA2257270},
	{SG27325, &UA912329},
	{SG27326, &UA2025894},
	{SG27327, &UA2033579},
	{SG27328, &UA2034322},
	{SG27330, &UA2607526},
	{SG27362, &UA438377},
	{SG27363, &UA821690},
	{SG27364, &UA2029631},
	{SG27365, &UA2615672},
	{SG27366, &UA2616106},
	{SG27367, &UA2625510},
	{SG27368, &UA2619383},
	{SG27369, &UA2635089},
	{SG27370, &UA2619629},
	{SG27374, &UA2620405},
	{SG27375, &UA2623402},
	{SG27376, &UA2635634},
	{SG27377, &UA2637142},
	{SG27387, &UA456404},
	{SG27389, &UA1975201},
	{SG27392, &UA2611636},
	{SG27393, &UA2617235},
	{SG27394, &UA2627745},
	{SG27490, &UA616590},
	{SG27497, &UA896993},
	{SG27500, &UA1956848},
	{SG27516, &UA2029615},
	{SG27535, &UA2616203},
	{SG27538, &UA2616608},
	{SG27542, &UA2617759},
	{SG27550, &UA2619698},
	{SG27567, &UA2633215},
	{SG27591, &UA2636794},
	{SG27734, &UA368881},
	{SG27748, &UA859187},
	{SG27755, &UA1991141},
	{SG27756, &UA2030979},
	{SG27757, &UA2540622},
	{SG27758, &UA2610504},
	{SG27759, &UA2615955},
	{SG27760, &UA2616126},
	{SG27761, &UA2616341},
	{SG27762, &UA2630457},
	{SG27778, &UA2623750},
	{SG27794, &UA2633847},
	{SG27805, &UA2633575},
	{SG27812, &UA2635293},
	{SG27815, &UA2635761},
	{SG27820, &UA2637808},
	{SG27937, &UA369190},
	{SG27943, &UA370430},
	{SG27962, &UA912469},
	{SG27970, &UA2024548},
	{SG27972, &UA2026497},
	{SG27993, &UA2540573},
	{SG27997, &UA2540628},
	{SG28007, &UA2608172},
	{SG28013, &UA2612877},
	{SG28026, &UA2616793},
	{SG28031, &UA2617214},
	{SG28043, &UA2619641},
	{SG28045, &UA2619737},
	{SG28063, &UA2623068},
	{SG28078, &UA2625868},
	{SG28099, &UA2629763},
	{SG28118, &UA2632650},
	{SG28133, &UA2635117},
	{SG28160, &UA2637868},
	{SG28215, &UA3084057},
	{NULL}};
const Segmentdata DAndroidVersion4_0_5[] = {
	{SG25981, &UA1955818},
	{SG25984, &UA2222777},
	{SG25986, &UA2982598},
	{SG25987, &UA3117230},
	{SG26119, &UA2023900},
	{SG26131, &UA2257792},
	{SG26135, &UA2420049},
	{SG26136, &UA2420165},
	{SG26144, &UA2455719},
	{SG26185, &UA2483315},
	{SG26237, &UA2504901},
	{SG26244, &UA2524814},
	{SG26249, &UA2527240},
	{SG26256, &UA2552062},
	{SG26262, &UA2577515},
	{SG26263, &UA2579832},
	{SG26321, &UA2658339},
	{SG26329, &UA2728599},
	{SG26330, &UA2732170},
	{SG26336, &UA2746806},
	{SG26339, &UA2747028},
	{SG26357, &UA2751733},
	{SG26366, &UA2762433},
	{SG26372, &UA2763447},
	{SG26388, &UA2786761},
	{SG26391, &UA2791000},
	{SG26392, &UA2835024},
	{SG26400, &UA2846650},
	{SG26401, &UA2848957},
	{SG26402, &UA2878108},
	{SG26412, &UA2934738},
	{SG26455, &UA2950215},
	{SG26463, &UA2952036},
	{SG26471, &UA2952991},
	{SG26503, &UA2979296},
	{SG26514, &UA2982007},
	{SG26551, &UA2987976},
	{SG26552, &UA2992284},
	{SG26553, &UA2995619},
	{SG26557, &UA2997717},
	{SG26567, &UA3045295},
	{SG26574, &UA3049865},
	{SG26576, &UA3070199},
	{SG26584, &UA3086068},
	{SG26596, &UA2402629},
	{SG26598, &UA2420003},
	{SG26600, &UA2594666},
	{SG26601, &UA2651870},
	{SG26602, &UA3072938},
	{SG27005, &UA2449100},
	{SG27006, &UA2775787},
	{SG27007, &UA2829296},
	{SG27008, &UA2849559},
	{SG27069, &UA2815495},
	{SG27372, &UA767208},
	{SG27373, &UA1982568},
	{SG27378, &UA2362654},
	{SG27380, &UA2376193},
	{SG27381, &UA2377039},
	{SG27382, &UA2385107},
	{SG27383, &UA2400922},
	{SG27384, &UA2420681},
	{SG27385, &UA2557638},
	{SG27386, &UA2648682},
	{SG27388, &UA2775082},
	{SG27390, &UA3042235},
	{SG31382, &UA2502516},
	{SG31396, &UA2527796},
	{SG31625, &UA2503817},
	{SG31626, &UA2760452},
	{SG31628, &UA2837112},
	{SG31888, &UA2031757},
	{SG31889, &UA2645518},
	{SG32053, &UA2385500},
	{SG32054, &UA2421622},
	{SG32055, &UA2424073},
	{SG32056, &UA2455675},
	{SG32057, &UA2532955},
	{SG32342, &UA2466080},
	{SG33569, &UA1993171},
	{SG33570, &UA2026663},
	{SG33571, &UA2027658},
	{SG33572, &UA2029654},
	{SG33575, &UA2030429},
	{SG33577, &UA2032275},
	{SG33580, &UA2410718},
	{SG33585, &UA2653321},
	{SG33590, &UA2818047},
	{SG33592, &UA2878130},
	{SG33594, &UA3042851},
	{SG33595, &UA3043072},
	{SG33596, &UA3071462},
	{SG33706, &UA2545119},
	{SG34578, &UA2413503},
	{SG34668, &UA3047028},
	{SG34802, &UA2957582},
	{SG35470, &UA2379200},
	{SG36261, &UA2261532},
	{SG36423, &UA2666892},
	{SG36758, &UA2735529},
	{SG36766, &UA2937436},
	{SG36768, &UA2997465},
	{SG36889, &UA2424895},
	{SG37173, &UA2616090},
	{SG38209, &UA1996400},
	{SG38210, &UA2817707},
	{SG38358, &UA2626082},
	{SG39523, &UA2026281},
	{SG39524, &UA2950749},
	{SG39607, &UA2456566},
	{SG40675, &UA2465097},
	{SG40842, &UA2457337},
	{SG40863, &UA2029185},
	{SG41499, &UA2418663},
	{SG42816, &UA1973414},
	{SG43430, &UA2868301},
	{SG43751, &UA895043},
	{SG43869, &UA2030796},
	{SG44952, &UA2778460},
	{SG46404, &UA2396010},
	{SG47626, &UA2507404},
	{SG48569, &UA2733522},
	{SG49036, &UA3056629},
	{SG49392, &UA2707083},
	{SG49481, &UA2998806},
	{SG50140, &UA2957923},
	{SG50339, &UA2859767},
	{SG51503, &UA2607522},
	{SG51504, &UA2607527},
	{SG51796, &UA2607536},
	{NULL}};
const Segmentdata DAndroidVersion4_1[] = {
	{SG25943, &UA2653934},
	{SG26084, &UA2708514},
	{SG27016, &UA2642158},
	{SG27239, &UA2417725},
	{SG27240, &UA2419375},
	{SG27252, &UA2455188},
	{SG27308, &UA2891099},
	{SG27313, &UA2937754},
	{SG27315, &UA2938711},
	{SG27322, &UA2988749},
	{SG27323, &UA2989139},
	{SG27329, &UA2997684},
	{SG27331, &UA2999243},
	{SG27332, &UA3000433},
	{SG27360, &UA3008683},
	{SG27361, &UA3009900},
	{SG27371, &UA3020618},
	{SG27379, &UA3089385},
	{SG27391, &UA3121163},
	{SG29219, &UA911381},
	{SG29220, &UA2366440},
	{SG29341, &UA909175},
	{SG29342, &UA909928},
	{SG29343, &UA910024},
	{SG29344, &UA1950813},
	{SG29345, &UA2759871},
	{SG29548, &UA2653309},
	{SG29710, &UA2633889},
	{SG29873, &UA2863840},
	{SG30126, &UA351412},
	{SG30127, &UA1995168},
	{SG30937, &UA909110},
	{SG30938, &UA1955698},
	{SG30940, &UA2794944},
	{SG31059, &UA402898},
	{SG31060, &UA402907},
	{SG31061, &UA402908},
	{SG31062, &UA402915},
	{SG31063, &UA402973},
	{SG31064, &UA402978},
	{SG31065, &UA403045},
	{SG31066, &UA2393248},
	{SG31067, &UA2658378},
	{SG31233, &UA910759},
	{SG31234, &UA2240862},
	{SG31235, &UA2768363},
	{SG31978, &UA1950209},
	{SG32209, &UA616364},
	{SG32210, &UA1961796},
	{SG32214, &UA2028347},
	{SG32216, &UA2241494},
	{SG32219, &UA2241598},
	{SG32220, &UA2241964},
	{SG32640, &UA2229755},
	{SG32643, &UA2569931},
	{SG32717, &UA910752},
	{SG32845, &UA910750},
	{SG32975, &UA2425521},
	{SG32987, &UA3054995},
	{SG33128, &UA2824681},
	{SG33705, &UA1985907},
	{SG35239, &UA2677294},
	{SG35268, &UA2707231},
	{SG35276, &UA3021559},
	{SG35425, &UA2769766},
	{SG35514, &UA3081781},
	{SG35784, &UA2026346},
	{SG36281, &UA2602924},
	{SG36431, &UA2507008},
	{SG36432, &UA2764809},
	{SG36976, &UA3111880},
	{SG37213, &UA2031433},
	{SG37224, &UA2437258},
	{SG37238, &UA3021814},
	{SG37271, &UA2769541},
	{SG37910, &UA2560825},
	{SG37933, &UA2707594},
	{SG38982, &UA2580419},
	{SG39596, &UA2727949},
	{SG40014, &UA2600431},
	{SG40389, &UA2863389},
	{SG40426, &UA2582862},
	{SG40822, &UA2580119},
	{SG41086, &UA2616769},
	{SG42374, &UA2707620},
	{SG42737, &UA2795749},
	{SG43090, &UA2527305},
	{SG44137, &UA2767960},
	{SG45136, &UA2994542},
	{SG45137, &UA3099772},
	{SG45267, &UA2729700},
	{SG46749, &UA2455699},
	{SG47296, &UA3006957},
	{SG47610, &UA2455945},
	{SG49112, &UA2694869},
	{SG49338, &UA3012462},
	{SG49939, &UA2693187},
	{SG51191, &UA3043616},
	{SG52687, &UA2241984},
	{SG52791, &UA2242176},
	{NULL}};
const Segmentdata DAndroidVersion4_0_4[] = {
	{SG23565, &UA441506},
	{SG23566, &UA615303},
	{SG23567, &UA622302},
	{SG23568, &UA622424},
	{SG23569, &UA636265},
	{SG23570, &UA662954},
	{SG23573, &UA894080},
	{SG23575, &UA901481},
	{SG23577, &UA1971535},
	{SG23578, &UA1975884},
	{SG23579, &UA1986308},
	{SG23580, &UA1996822},
	{SG23581, &UA2025783},
	{SG23582, &UA2026176},
	{SG23583, &UA2026332},
	{SG23584, &UA2026767},
	{SG23585, &UA2028692},
	{SG23586, &UA2030191},
	{SG23587, &UA2031399},
	{SG23588, &UA2033358},
	{SG23589, &UA2041756},
	{SG23590, &UA2165441},
	{SG23591, &UA2203556},
	{SG23592, &UA2203975},
	{SG23593, &UA2232800},
	{SG23595, &UA2257532},
	{SG23596, &UA2355027},
	{SG23597, &UA2380373},
	{SG23598, &UA2396941},
	{SG23599, &UA2421988},
	{SG23600, &UA2453136},
	{SG23601, &UA2502796},
	{SG23603, &UA2526722},
	{SG23604, &UA2583164},
	{SG23605, &UA2583615},
	{SG23607, &UA2588165},
	{SG23609, &UA2727546},
	{SG23610, &UA2790549},
	{SG23611, &UA2810859},
	{SG23613, &UA2833470},
	{SG23614, &UA2833968},
	{SG23615, &UA2950756},
	{SG23657, &UA704951},
	{SG23659, &UA766820},
	{SG23660, &UA769312},
	{SG23662, &UA896195},
	{SG23663, &UA900396},
	{SG23664, &UA906680},
	{SG23665, &UA1953076},
	{SG23666, &UA1953637},
	{SG23667, &UA1962145},
	{SG23668, &UA1968437},
	{SG23669, &UA1976733},
	{SG23671, &UA1978295},
	{SG23672, &UA1989442},
	{SG23673, &UA1991990},
	{SG23675, &UA1997219},
	{SG23677, &UA2025104},
	{SG23680, &UA2026422},
	{SG23682, &UA2027043},
	{SG23685, &UA2027717},
	{SG23686, &UA2027793},
	{SG23687, &UA2028368},
	{SG23688, &UA2028447},
	{SG23690, &UA2028518},
	{SG23691, &UA2028605},
	{SG23692, &UA2028673},
	{SG23694, &UA2029068},
	{SG23695, &UA2029337},
	{SG23697, &UA2029455},
	{SG23699, &UA2029565},
	{SG23701, &UA2029735},
	{SG23704, &UA2030036},
	{SG23707, &UA2032017},
	{SG23712, &UA2032484},
	{SG23714, &UA2032731},
	{SG23716, &UA2032850},
	{SG23718, &UA2032921},
	{SG23720, &UA2032985},
	{SG23723, &UA2033131},
	{SG23726, &UA2033520},
	{SG23731, &UA2033969},
	{SG23736, &UA2034574},
	{SG23737, &UA2161431},
	{SG23738, &UA2165850},
	{SG23742, &UA2185331},
	{SG23745, &UA2198899},
	{SG23746, &UA2202532},
	{SG23748, &UA2203513},
	{SG23749, &UA2204045},
	{SG23751, &UA2224166},
	{SG23752, &UA2226830},
	{SG23753, &UA2228204},
	{SG23754, &UA2230210},
	{SG23756, &UA2230529},
	{SG23757, &UA2232409},
	{SG23758, &UA2234752},
	{SG23759, &UA2234807},
	{SG23760, &UA2238955},
	{SG23761, &UA2239000},
	{SG23762, &UA2257413},
	{SG23763, &UA2257500},
	{SG23764, &UA2257544},
	{SG23765, &UA2257599},
	{SG23766, &UA2257709},
	{SG23767, &UA2257722},
	{SG23768, &UA2257745},
	{SG23769, &UA2257921},
	{SG23770, &UA2332579},
	{SG23772, &UA2337502},
	{SG23773, &UA2337668},
	{SG23775, &UA2342981},
	{SG23776, &UA2343042},
	{SG23778, &UA2343291},
	{SG23779, &UA2343381},
	{SG23780, &UA2343583},
	{SG23781, &UA2343732},
	{SG23782, &UA2344332},
	{SG23783, &UA2346317},
	{SG23784, &UA2349727},
	{SG23785, &UA2350010},
	{SG23788, &UA2351672},
	{SG23789, &UA2352006},
	{SG23790, &UA2353362},
	{SG23791, &UA2353603},
	{SG23792, &UA2359408},
	{SG23794, &UA2413156},
	{SG23796, &UA2413559},
	{SG23797, &UA2414164},
	{SG23798, &UA2415382},
	{SG23799, &UA2416024},
	{SG23802, &UA2416487},
	{SG23803, &UA2416753},
	{SG23804, &UA2417426},
	{SG23805, &UA2417838},
	{SG23806, &UA2418025},
	{SG23807, &UA2418188},
	{SG23808, &UA2419209},
	{SG23809, &UA2420011},
	{SG23810, &UA2420188},
	{SG23811, &UA2420294},
	{SG23812, &UA2420621},
	{SG23813, &UA2420633},
	{SG23815, &UA2420636},
	{SG23816, &UA2420875},
	{SG23818, &UA2421033},
	{SG23819, &UA2421111},
	{SG23820, &UA2421243},
	{SG23821, &UA2421248},
	{SG23822, &UA2421380},
	{SG23823, &UA2421442},
	{SG23824, &UA2421530},
	{SG23825, &UA2421552},
	{SG23826, &UA2421904},
	{SG23827, &UA2421941},
	{SG23828, &UA2422032},
	{SG23829, &UA2422214},
	{SG23831, &UA2422250},
	{SG23833, &UA2422262},
	{SG23835, &UA2422663},
	{SG23838, &UA2423252},
	{SG23840, &UA2423360},
	{SG23841, &UA2423411},
	{SG23842, &UA2423524},
	{SG23843, &UA2423561},
	{SG23844, &UA2423576},
	{SG23845, &UA2423895},
	{SG23847, &UA2424111},
	{SG23848, &UA2424285},
	{SG23850, &UA2424485},
	{SG23852, &UA2424675},
	{SG23854, &UA2424796},
	{SG23856, &UA2424848},
	{SG23857, &UA2425147},
	{SG23859, &UA2425571},
	{SG23861, &UA2425608},
	{SG23862, &UA2425716},
	{SG23864, &UA2425781},
	{SG23865, &UA2425892},
	{SG23866, &UA2425955},
	{SG23867, &UA2426036},
	{SG23868, &UA2426233},
	{SG23870, &UA2426410},
	{SG23871, &UA2426857},
	{SG23872, &UA2427061},
	{SG23873, &UA2428345},
	{SG23874, &UA2428471},
	{SG23875, &UA2428581},
	{SG23876, &UA2432244},
	{SG23877, &UA2433591},
	{SG23878, &UA2442905},
	{SG23880, &UA2443057},
	{SG23881, &UA2444624},
	{SG23882, &UA2447142},
	{SG23883, &UA2448890},
	{SG23885, &UA2450282},
	{SG23886, &UA2450875},
	{SG23887, &UA2454373},
	{SG23888, &UA2454393},
	{SG23889, &UA2454575},
	{SG23890, &UA2454778},
	{SG23891, &UA2454853},
	{SG23892, &UA2455197},
	{SG23893, &UA2455366},
	{SG23894, &UA2455386},
	{SG23895, &UA2455396},
	{SG23896, &UA2455388},
	{SG23897, &UA2455476},
	{SG23898, &UA2455488},
	{SG23899, &UA2455511},
	{SG23900, &UA2455528},
	{SG23901, &UA2455580},
	{SG23904, &UA2455740},
	{SG23905, &UA2455842},
	{SG23906, &UA2456563},
	{SG23907, &UA2456798},
	{SG23908, &UA2456885},
	{SG23910, &UA2456971},
	{SG23911, &UA2457041},
	{SG23913, &UA2457051},
	{SG23914, &UA2457186},
	{SG23916, &UA2457387},
	{SG23917, &UA2457424},
	{SG23918, &UA2457462},
	{SG23919, &UA2457474},
	{SG23920, &UA2457543},
	{SG23921, &UA2457865},
	{SG23922, &UA2457877},
	{SG23924, &UA2458073},
	{SG23925, &UA2458360},
	{SG23926, &UA2460775},
	{SG23927, &UA2464616},
	{SG23929, &UA2464869},
	{SG23930, &UA2464954},
	{SG23932, &UA2465267},
	{SG23934, &UA2465661},
	{SG23936, &UA2466172},
	{SG23937, &UA2466207},
	{SG23938, &UA2466217},
	{SG23939, &UA2466705},
	{SG23940, &UA2466841},
	{SG23941, &UA2467281},
	{SG23942, &UA2467284},
	{SG23943, &UA2467351},
	{SG23944, &UA2467402},
	{SG23945, &UA2467443},
	{SG23946, &UA2467463},
	{SG23948, &UA2467662},
	{SG23949, &UA2467902},
	{SG23950, &UA2468504},
	{SG23951, &UA2468634},
	{SG23952, &UA2468639},
	{SG23953, &UA2469160},
	{SG23954, &UA2469355},
	{SG23956, &UA2473660},
	{SG23957, &UA2476372},
	{SG23958, &UA2477412},
	{SG23959, &UA2477918},
	{SG23960, &UA2478278},
	{SG23962, &UA2479946},
	{SG23964, &UA2480296},
	{SG23966, &UA2480317},
	{SG23968, &UA2480649},
	{SG23970, &UA2480776},
	{SG23972, &UA2480896},
	{SG23974, &UA2480977},
	{SG23976, &UA2481052},
	{SG23977, &UA2481288},
	{SG23978, &UA2481593},
	{SG23979, &UA2481729},
	{SG23981, &UA2481782},
	{SG23982, &UA2481944},
	{SG23983, &UA2482051},
	{SG23984, &UA2482364},
	{SG23985, &UA2482609},
	{SG23986, &UA2482704},
	{SG23988, &UA2482763},
	{SG23989, &UA2482857},
	{SG23990, &UA2482951},
	{SG23991, &UA2482993},
	{SG23992, &UA2483139},
	{SG23993, &UA2483211},
	{SG23994, &UA2483468},
	{SG23995, &UA2483483},
	{SG23996, &UA2483889},
	{SG23997, &UA2484098},
	{SG23998, &UA2485179},
	{SG23999, &UA2485746},
	{SG24000, &UA2485822},
	{SG24001, &UA2486196},
	{SG24002, &UA2486386},
	{SG24003, &UA2486499},
	{SG24004, &UA2486604},
	{SG24005, &UA2486637},
	{SG24006, &UA2486810},
	{SG24007, &UA2487867},
	{SG24008, &UA2498149},
	{SG24009, &UA2498459},
	{SG24010, &UA2498746},
	{SG24012, &UA2500408},
	{SG24025, &UA2500553},
	{SG24027, &UA2500808},
	{SG24029, &UA2501280},
	{SG24030, &UA2501949},
	{SG24031, &UA2502448},
	{SG24032, &UA2502524},
	{SG24033, &UA2502570},
	{SG24034, &UA2502726},
	{SG24035, &UA2502894},
	{SG24036, &UA2503425},
	{SG24037, &UA2503544},
	{SG24038, &UA2503761},
	{SG24039, &UA2503815},
	{SG24040, &UA2503924},
	{SG24041, &UA2503980},
	{SG24042, &UA2504150},
	{SG24043, &UA2504316},
	{SG24044, &UA2504507},
	{SG24045, &UA2504918},
	{SG24046, &UA2504957},
	{SG24047, &UA2505367},
	{SG24048, &UA2505698},
	{SG24049, &UA2505770},
	{SG24050, &UA2506581},
	{SG24051, &UA2506952},
	{SG24052, &UA2506938},
	{SG24053, &UA2507082},
	{SG24054, &UA2507311},
	{SG24055, &UA2507335},
	{SG24056, &UA2519896},
	{SG24057, &UA2520406},
	{SG24058, &UA2520783},
	{SG24059, &UA2520858},
	{SG24060, &UA2522118},
	{SG24061, &UA2522282},
	{SG24062, &UA2525069},
	{SG24063, &UA2525509},
	{SG24064, &UA2525725},
	{SG24065, &UA2526001},
	{SG24066, &UA2526075},
	{SG24067, &UA2526355},
	{SG24068, &UA2526388},
	{SG24069, &UA2526599},
	{SG24070, &UA2526648},
	{SG24071, &UA2526656},
	{SG24072, &UA2526710},
	{SG24073, &UA2526943},
	{SG24074, &UA2527246},
	{SG24075, &UA2527394},
	{SG24076, &UA2527507},
	{SG24077, &UA2527527},
	{SG24078, &UA2527878},
	{SG24079, &UA2527946},
	{SG24080, &UA2528064},
	{SG24081, &UA2528115},
	{SG24083, &UA2528254},
	{SG24084, &UA2528301},
	{SG24085, &UA2528315},
	{SG24086, &UA2528456},
	{SG24087, &UA2528844},
	{SG24088, &UA2528887},
	{SG24089, &UA2529346},
	{SG24091, &UA2529365},
	{SG24093, &UA2529433},
	{SG24094, &UA2529624},
	{SG24095, &UA2529693},
	{SG24097, &UA2529947},
	{SG24098, &UA2529966},
	{SG24100, &UA2529979},
	{SG24102, &UA2530136},
	{SG24103, &UA2530218},
	{SG24104, &UA2530249},
	{SG24105, &UA2530323},
	{SG24106, &UA2530804},
	{SG24107, &UA2533080},
	{SG24108, &UA2533090},
	{SG24109, &UA2534426},
	{SG24110, &UA2534596},
	{SG24111, &UA2536108},
	{SG24113, &UA2543213},
	{SG24115, &UA2546485},
	{SG24132, &UA2548120},
	{SG24133, &UA2548633},
	{SG24135, &UA2550331},
	{SG24136, &UA2550574},
	{SG24137, &UA2550955},
	{SG24138, &UA2551384},
	{SG24139, &UA2551754},
	{SG24140, &UA2552629},
	{SG24141, &UA2552834},
	{SG24142, &UA2553140},
	{SG24143, &UA2553229},
	{SG24145, &UA2553284},
	{SG24146, &UA2553386},
	{SG24147, &UA2553673},
	{SG24148, &UA2554158},
	{SG24149, &UA2554170},
	{SG24150, &UA2554417},
	{SG24151, &UA2554436},
	{SG24152, &UA2554479},
	{SG24153, &UA2554593},
	{SG24154, &UA2554911},
	{SG24155, &UA2555009},
	{SG24156, &UA2555071},
	{SG24157, &UA2555187},
	{SG24158, &UA2555269},
	{SG24159, &UA2555379},
	{SG24160, &UA2555392},
	{SG24161, &UA2555433},
	{SG24162, &UA2555579},
	{SG24163, &UA2555670},
	{SG24164, &UA2555685},
	{SG24165, &UA2555816},
	{SG24166, &UA2555853},
	{SG24168, &UA2555906},
	{SG24169, &UA2555938},
	{SG24172, &UA2556175},
	{SG24173, &UA2556237},
	{SG24174, &UA2556384},
	{SG24175, &UA2556440},
	{SG24176, &UA2556508},
	{SG24177, &UA2556666},
	{SG24178, &UA2556711},
	{SG24180, &UA2556875},
	{SG24181, &UA2556881},
	{SG24183, &UA2557324},
	{SG24185, &UA2557452},
	{SG24186, &UA2558120},
	{SG24187, &UA2558160},
	{SG24189, &UA2558426},
	{SG24191, &UA2561039},
	{SG24193, &UA2561764},
	{SG24194, &UA2564155},
	{SG24195, &UA2577219},
	{SG24196, &UA2577226},
	{SG24197, &UA2577521},
	{SG24200, &UA2577592},
	{SG24201, &UA2577723},
	{SG24203, &UA2577871},
	{SG24205, &UA2578139},
	{SG24207, &UA2578243},
	{SG24208, &UA2578434},
	{SG24209, &UA2578488},
	{SG24211, &UA2578503},
	{SG24212, &UA2578704},
	{SG24214, &UA2578764},
	{SG24215, &UA2578838},
	{SG24216, &UA2578994},
	{SG24217, &UA2579030},
	{SG24218, &UA2579323},
	{SG24219, &UA2579493},
	{SG24220, &UA2579569},
	{SG24221, &UA2579610},
	{SG24222, &UA2579718},
	{SG24223, &UA2579747},
	{SG24224, &UA2579756},
	{SG24225, &UA2579889},
	{SG24226, &UA2579929},
	{SG24227, &UA2580155},
	{SG24228, &UA2580196},
	{SG24229, &UA2580221},
	{SG24230, &UA2580296},
	{SG24231, &UA2581071},
	{SG24232, &UA2581895},
	{SG24233, &UA2589756},
	{SG24234, &UA2590108},
	{SG24235, &UA2592311},
	{SG24236, &UA2592765},
	{SG24238, &UA2593592},
	{SG24239, &UA2593871},
	{SG24240, &UA2594075},
	{SG24241, &UA2594197},
	{SG24242, &UA2594256},
	{SG24243, &UA2594267},
	{SG24244, &UA2594315},
	{SG24245, &UA2594320},
	{SG24247, &UA2594335},
	{SG24248, &UA2594328},
	{SG24249, &UA2594543},
	{SG24250, &UA2594807},
	{SG24251, &UA2594859},
	{SG24253, &UA2594871},
	{SG24254, &UA2595015},
	{SG24255, &UA2595106},
	{SG24256, &UA2595270},
	{SG24258, &UA2595366},
	{SG24259, &UA2595470},
	{SG24260, &UA2595637},
	{SG24261, &UA2595741},
	{SG24262, &UA2596089},
	{SG24263, &UA2596370},
	{SG24264, &UA2596481},
	{SG24265, &UA2596660},
	{SG24266, &UA2596695},
	{SG24267, &UA2596697},
	{SG24268, &UA2596769},
	{SG24269, &UA2596869},
	{SG24270, &UA2596877},
	{SG24271, &UA2596912},
	{SG24272, &UA2596942},
	{SG24273, &UA2597230},
	{SG24274, &UA2597271},
	{SG24275, &UA2597488},
	{SG24276, &UA2597498},
	{SG24277, &UA2597622},
	{SG24278, &UA2597861},
	{SG24279, &UA2597921},
	{SG24280, &UA2598349},
	{SG24281, &UA2598515},
	{SG24282, &UA2599911},
	{SG24283, &UA2600401},
	{SG24284, &UA2600629},
	{SG24285, &UA2600788},
	{SG24286, &UA2601491},
	{SG24287, &UA2601546},
	{SG24288, &UA2605534},
	{SG24289, &UA2642313},
	{SG24290, &UA2643486},
	{SG24291, &UA2644099},
	{SG24292, &UA2646765},
	{SG24294, &UA2648405},
	{SG24295, &UA2649663},
	{SG24296, &UA2649679},
	{SG24297, &UA2650987},
	{SG24298, &UA2652288},
	{SG24299, &UA2652297},
	{SG24300, &UA2652440},
	{SG24301, &UA2652475},
	{SG24302, &UA2652538},
	{SG24303, &UA2652547},
	{SG24304, &UA2652634},
	{SG24305, &UA2652986},
	{SG24306, &UA2653246},
	{SG24307, &UA2653249},
	{SG24308, &UA2653471},
	{SG24310, &UA2653518},
	{SG24311, &UA2653530},
	{SG24312, &UA2653583},
	{SG24313, &UA2653958},
	{SG24314, &UA2654029},
	{SG24315, &UA2654110},
	{SG24316, &UA2654241},
	{SG24317, &UA2654260},
	{SG24318, &UA2654274},
	{SG24319, &UA2654690},
	{SG24320, &UA2654839},
	{SG24321, &UA2654846},
	{SG24322, &UA2654875},
	{SG24323, &UA2654948},
	{SG24324, &UA2654928},
	{SG24325, &UA2655003},
	{SG24326, &UA2655082},
	{SG24327, &UA2655094},
	{SG24329, &UA2655268},
	{SG24330, &UA2655492},
	{SG24332, &UA2655494},
	{SG24333, &UA2655720},
	{SG24334, &UA2655724},
	{SG24335, &UA2655867},
	{SG24337, &UA2656244},
	{SG24339, &UA2656432},
	{SG24340, &UA2656699},
	{SG24341, &UA2657082},
	{SG24342, &UA2657336},
	{SG24344, &UA2657366},
	{SG24345, &UA2657482},
	{SG24346, &UA2657492},
	{SG24347, &UA2657602},
	{SG24348, &UA2657683},
	{SG24349, &UA2657747},
	{SG24350, &UA2657763},
	{SG24351, &UA2657779},
	{SG24352, &UA2657892},
	{SG24353, &UA2657960},
	{SG24354, &UA2657961},
	{SG24355, &UA2658044},
	{SG24357, &UA2658062},
	{SG24359, &UA2658557},
	{SG24360, &UA2658616},
	{SG24363, &UA2659188},
	{SG24364, &UA2659256},
	{SG24366, &UA2659596},
	{SG24367, &UA2659638},
	{SG24368, &UA2660049},
	{SG24370, &UA2660453},
	{SG24371, &UA2660595},
	{SG24372, &UA2660906},
	{SG24373, &UA2660940},
	{SG24375, &UA2661282},
	{SG24377, &UA2662066},
	{SG24378, &UA2662386},
	{SG24379, &UA2662621},
	{SG24380, &UA2662641},
	{SG24382, &UA2662948},
	{SG24384, &UA2663109},
	{SG24385, &UA2664513},
	{SG24386, &UA2664592},
	{SG24388, &UA2665095},
	{SG24390, &UA2665267},
	{SG24391, &UA2665277},
	{SG24393, &UA2665281},
	{SG24395, &UA2665404},
	{SG24396, &UA2665664},
	{SG24397, &UA2665777},
	{SG24399, &UA2665797},
	{SG24400, &UA2665839},
	{SG24402, &UA2665993},
	{SG24403, &UA2666292},
	{SG24404, &UA2666349},
	{SG24406, &UA2666385},
	{SG24407, &UA2666552},
	{SG24409, &UA2666814},
	{SG24410, &UA2667516},
	{SG24411, &UA2667731},
	{SG24412, &UA2686780},
	{SG24413, &UA2687034},
	{SG24414, &UA2687058},
	{SG24415, &UA2687325},
	{SG24416, &UA2687843},
	{SG24417, &UA2688504},
	{SG24418, &UA2689087},
	{SG24419, &UA2689216},
	{SG24420, &UA2689834},
	{SG24421, &UA2691557},
	{SG24422, &UA2692064},
	{SG24423, &UA2692701},
	{SG24424, &UA2693041},
	{SG24425, &UA2693110},
	{SG24426, &UA2693183},
	{SG24427, &UA2693298},
	{SG24428, &UA2693335},
	{SG24429, &UA2693669},
	{SG24430, &UA2693960},
	{SG24431, &UA2694309},
	{SG24432, &UA2694344},
	{SG24433, &UA2694352},
	{SG24434, &UA2694703},
	{SG24435, &UA2694866},
	{SG24436, &UA2695003},
	{SG24437, &UA2695078},
	{SG24438, &UA2695086},
	{SG24439, &UA2695455},
	{SG24440, &UA2695457},
	{SG24441, &UA2695480},
	{SG24442, &UA2695506},
	{SG24443, &UA2695563},
	{SG24444, &UA2695694},
	{SG24445, &UA2695713},
	{SG24446, &UA2695765},
	{SG24447, &UA2695794},
	{SG24448, &UA2696118},
	{SG24449, &UA2696123},
	{SG24450, &UA2696372},
	{SG24451, &UA2696394},
	{SG24453, &UA2696927},
	{SG24454, &UA2696938},
	{SG24455, &UA2697000},
	{SG24456, &UA2697096},
	{SG24457, &UA2697106},
	{SG24458, &UA2697262},
	{SG24459, &UA2697428},
	{SG24460, &UA2697489},
	{SG24461, &UA2697746},
	{SG24462, &UA2698053},
	{SG24463, &UA2698063},
	{SG24464, &UA2698163},
	{SG24465, &UA2698164},
	{SG24466, &UA2698189},
	{SG24467, &UA2698272},
	{SG24468, &UA2698290},
	{SG24469, &UA2698347},
	{SG24470, &UA2698388},
	{SG24472, &UA2698857},
	{SG24473, &UA2699183},
	{SG24474, &UA2699369},
	{SG24475, &UA2699748},
	{SG24476, &UA2699976},
	{SG24477, &UA2700039},
	{SG24478, &UA2700172},
	{SG24479, &UA2700183},
	{SG24480, &UA2700387},
	{SG24481, &UA2700445},
	{SG24482, &UA2700523},
	{SG24483, &UA2700661},
	{SG24484, &UA2700886},
	{SG24485, &UA2700944},
	{SG24486, &UA2701112},
	{SG24487, &UA2701122},
	{SG24488, &UA2701214},
	{SG24489, &UA2701249},
	{SG24490, &UA2701252},
	{SG24491, &UA2701254},
	{SG24492, &UA2701678},
	{SG24494, &UA2701745},
	{SG24495, &UA2701967},
	{SG24496, &UA2702089},
	{SG24497, &UA2702576},
	{SG24498, &UA2702651},
	{SG24499, &UA2702674},
	{SG24501, &UA2703300},
	{SG24502, &UA2703546},
	{SG24503, &UA2703637},
	{SG24505, &UA2703725},
	{SG24506, &UA2703774},
	{SG24507, &UA2703968},
	{SG24508, &UA2704435},
	{SG24509, &UA2704442},
	{SG24510, &UA2704593},
	{SG24511, &UA2704745},
	{SG24513, &UA2705387},
	{SG24515, &UA2705925},
	{SG24517, &UA2706096},
	{SG24518, &UA2706326},
	{SG24519, &UA2707613},
	{SG24520, &UA2708221},
	{SG24522, &UA2711776},
	{SG24524, &UA2723960},
	{SG24525, &UA2724849},
	{SG24526, &UA2725180},
	{SG24527, &UA2725350},
	{SG24528, &UA2726045},
	{SG24529, &UA2726235},
	{SG24530, &UA2726433},
	{SG24531, &UA2726542},
	{SG24532, &UA2726675},
	{SG24533, &UA2726895},
	{SG24534, &UA2726923},
	{SG24535, &UA2726970},
	{SG24536, &UA2727081},
	{SG24537, &UA2727296},
	{SG24538, &UA2727342},
	{SG24539, &UA2727373},
	{SG24540, &UA2727673},
	{SG24541, &UA2727755},
	{SG24542, &UA2727740},
	{SG24543, &UA2727909},
	{SG24544, &UA2727956},
	{SG24545, &UA2728194},
	{SG24547, &UA2728224},
	{SG24548, &UA2728265},
	{SG24549, &UA2728428},
	{SG24550, &UA2728543},
	{SG24551, &UA2728574},
	{SG24552, &UA2728925},
	{SG24553, &UA2728951},
	{SG24554, &UA2729065},
	{SG24555, &UA2729104},
	{SG24556, &UA2729271},
	{SG24557, &UA2729401},
	{SG24558, &UA2729682},
	{SG24559, &UA2729838},
	{SG24561, &UA2729870},
	{SG24562, &UA2730028},
	{SG24563, &UA2730339},
	{SG24564, &UA2730381},
	{SG24565, &UA2731205},
	{SG24567, &UA2731521},
	{SG24568, &UA2732090},
	{SG24570, &UA2732151},
	{SG24571, &UA2732247},
	{SG24572, &UA2732284},
	{SG24573, &UA2732362},
	{SG24574, &UA2732410},
	{SG24576, &UA2732688},
	{SG24577, &UA2733383},
	{SG24578, &UA2733392},
	{SG24579, &UA2733955},
	{SG24580, &UA2734188},
	{SG24581, &UA2734377},
	{SG24582, &UA2735120},
	{SG24583, &UA2735770},
	{SG24584, &UA2736055},
	{SG24586, &UA2736775},
	{SG24587, &UA2738764},
	{SG24588, &UA2744985},
	{SG24590, &UA2745709},
	{SG24591, &UA2745803},
	{SG24592, &UA2746284},
	{SG24594, &UA2746774},
	{SG24595, &UA2746784},
	{SG24597, &UA2746996},
	{SG24599, &UA2747009},
	{SG24601, &UA2747045},
	{SG24603, &UA2747105},
	{SG24605, &UA2747127},
	{SG24607, &UA2747431},
	{SG24610, &UA2747814},
	{SG24611, &UA2748002},
	{SG24613, &UA2748050},
	{SG24614, &UA2748249},
	{SG24615, &UA2748286},
	{SG24617, &UA2748292},
	{SG24618, &UA2748545},
	{SG24619, &UA2748573},
	{SG24620, &UA2748589},
	{SG24621, &UA2749134},
	{SG24622, &UA2749178},
	{SG24623, &UA2749340},
	{SG24624, &UA2749631},
	{SG24625, &UA2749678},
	{SG24627, &UA2749685},
	{SG24628, &UA2749804},
	{SG24629, &UA2749825},
	{SG24631, &UA2749879},
	{SG24633, &UA2750113},
	{SG24634, &UA2750337},
	{SG24635, &UA2750475},
	{SG24637, &UA2750889},
	{SG24638, &UA2750948},
	{SG24639, &UA2751441},
	{SG24641, &UA2751807},
	{SG24644, &UA2759689},
	{SG24646, &UA2759860},
	{SG24648, &UA2760129},
	{SG24649, &UA2760526},
	{SG24651, &UA2761643},
	{SG24652, &UA2761780},
	{SG24653, &UA2762094},
	{SG24655, &UA2762185},
	{SG24658, &UA2762232},
	{SG24660, &UA2762304},
	{SG24661, &UA2762333},
	{SG24663, &UA2762544},
	{SG24664, &UA2762559},
	{SG24666, &UA2762589},
	{SG24667, &UA2762678},
	{SG24669, &UA2762699},
	{SG24670, &UA2762701},
	{SG24672, &UA2762763},
	{SG24673, &UA2762778},
	{SG24674, &UA2762839},
	{SG24676, &UA2762849},
	{SG24677, &UA2762958},
	{SG24680, &UA2763134},
	{SG24682, &UA2763221},
	{SG24685, &UA2763660},
	{SG24686, &UA2763698},
	{SG24688, &UA2763770},
	{SG24691, &UA2763962},
	{SG24693, &UA2764046},
	{SG24695, &UA2764091},
	{SG24696, &UA2764116},
	{SG24697, &UA2764271},
	{SG24698, &UA2764329},
	{SG24699, &UA2765380},
	{SG24701, &UA2765879},
	{SG24703, &UA2766116},
	{SG24704, &UA2766311},
	{SG24705, &UA2766687},
	{SG24708, &UA2766862},
	{SG24710, &UA2766962},
	{SG24711, &UA2767070},
	{SG24713, &UA2767530},
	{SG24714, &UA2767534},
	{SG24717, &UA2767652},
	{SG24719, &UA2768149},
	{SG24720, &UA2768482},
	{SG24722, &UA2775572},
	{SG24723, &UA2775634},
	{SG24724, &UA2775690},
	{SG24726, &UA2775762},
	{SG24727, &UA2777505},
	{SG24728, &UA2777643},
	{SG24729, &UA2777645},
	{SG24730, &UA2777646},
	{SG24732, &UA2777675},
	{SG24733, &UA2777762},
	{SG24734, &UA2777764},
	{SG24735, &UA2777846},
	{SG24736, &UA2778039},
	{SG24737, &UA2778115},
	{SG24739, &UA2778173},
	{SG24741, &UA2778241},
	{SG24742, &UA2778291},
	{SG24745, &UA2778378},
	{SG24746, &UA2778489},
	{SG24747, &UA2778509},
	{SG24748, &UA2778518},
	{SG24749, &UA2778557},
	{SG24751, &UA2778666},
	{SG24752, &UA2778729},
	{SG24754, &UA2778901},
	{SG24755, &UA2778935},
	{SG24756, &UA2779126},
	{SG24757, &UA2779250},
	{SG24758, &UA2779588},
	{SG24759, &UA2779721},
	{SG24760, &UA2779953},
	{SG24762, &UA2780060},
	{SG24763, &UA2780063},
	{SG24764, &UA2780184},
	{SG24765, &UA2780234},
	{SG24766, &UA2780308},
	{SG24767, &UA2780950},
	{SG24768, &UA2781044},
	{SG24770, &UA2781106},
	{SG24771, &UA2781310},
	{SG24772, &UA2781905},
	{SG24773, &UA2782032},
	{SG24774, &UA2789371},
	{SG24775, &UA2789658},
	{SG24776, &UA2789667},
	{SG24777, &UA2789909},
	{SG24780, &UA2790002},
	{SG24781, &UA2790318},
	{SG24782, &UA2790462},
	{SG24783, &UA2790754},
	{SG24784, &UA2790821},
	{SG24785, &UA2790957},
	{SG24786, &UA2791048},
	{SG24787, &UA2791076},
	{SG24788, &UA2791085},
	{SG24789, &UA2791134},
	{SG24791, &UA2791259},
	{SG24792, &UA2791380},
	{SG24793, &UA2791445},
	{SG24794, &UA2791523},
	{SG24795, &UA2792094},
	{SG24796, &UA2792152},
	{SG24797, &UA2792556},
	{SG24798, &UA2792984},
	{SG24799, &UA2793137},
	{SG24800, &UA2793351},
	{SG24801, &UA2793404},
	{SG24802, &UA2793866},
	{SG24803, &UA2794028},
	{SG24804, &UA2794070},
	{SG24805, &UA2794264},
	{SG24806, &UA2794304},
	{SG24807, &UA2794382},
	{SG24808, &UA2794440},
	{SG24809, &UA2794554},
	{SG24810, &UA2795205},
	{SG24811, &UA2795515},
	{SG24812, &UA2795659},
	{SG24813, &UA2795761},
	{SG24814, &UA2796264},
	{SG24815, &UA2797207},
	{SG24816, &UA2808924},
	{SG24818, &UA2809941},
	{SG24819, &UA2811484},
	{SG24821, &UA2816394},
	{SG24822, &UA2817060},
	{SG24823, &UA2817064},
	{SG24824, &UA2817798},
	{SG24825, &UA2818143},
	{SG24826, &UA2819252},
	{SG24827, &UA2819567},
	{SG24828, &UA2822372},
	{SG24829, &UA2824352},
	{SG24830, &UA2824373},
	{SG24831, &UA2825830},
	{SG24832, &UA2826855},
	{SG24833, &UA2828459},
	{SG24834, &UA2828711},
	{SG24835, &UA2828737},
	{SG24836, &UA2829782},
	{SG24837, &UA2830061},
	{SG24838, &UA2830300},
	{SG24839, &UA2832954},
	{SG24840, &UA2833021},
	{SG24841, &UA2833132},
	{SG24842, &UA2833400},
	{SG24843, &UA2834461},
	{SG24844, &UA2836298},
	{SG24845, &UA2836434},
	{SG24846, &UA2836596},
	{SG24847, &UA2836841},
	{SG24848, &UA2837454},
	{SG24849, &UA2838044},
	{SG24850, &UA2838076},
	{SG24851, &UA2838891},
	{SG24852, &UA2839143},
	{SG24853, &UA2839753},
	{SG24854, &UA2839997},
	{SG24855, &UA2840144},
	{SG24856, &UA2840274},
	{SG24857, &UA2840983},
	{SG24858, &UA2841106},
	{SG24859, &UA2842049},
	{SG24860, &UA2842097},
	{SG24861, &UA2843021},
	{SG24862, &UA2843022},
	{SG24863, &UA2843126},
	{SG24864, &UA2843386},
	{SG24865, &UA2843513},
	{SG24866, &UA2843679},
	{SG24867, &UA2843782},
	{SG24868, &UA2843799},
	{SG24869, &UA2843870},
	{SG24870, &UA2843989},
	{SG24871, &UA2844267},
	{SG24872, &UA2844369},
	{SG24873, &UA2844370},
	{SG24874, &UA2844700},
	{SG24875, &UA2844736},
	{SG24877, &UA2844828},
	{SG24878, &UA2844905},
	{SG24879, &UA2845366},
	{SG24880, &UA2845497},
	{SG24881, &UA2845958},
	{SG24882, &UA2846113},
	{SG24883, &UA2846603},
	{SG24884, &UA2846913},
	{SG24885, &UA2846939},
	{SG24887, &UA2847175},
	{SG24888, &UA2847205},
	{SG24889, &UA2847303},
	{SG24890, &UA2847315},
	{SG24891, &UA2847365},
	{SG24892, &UA2847478},
	{SG24893, &UA2847515},
	{SG24894, &UA2847581},
	{SG24895, &UA2847638},
	{SG24896, &UA2847642},
	{SG24897, &UA2847643},
	{SG24898, &UA2848109},
	{SG24899, &UA2848168},
	{SG24900, &UA2848231},
	{SG24901, &UA2848246},
	{SG24902, &UA2848320},
	{SG24903, &UA2848411},
	{SG24904, &UA2848485},
	{SG24905, &UA2848871},
	{SG24906, &UA2849079},
	{SG24907, &UA2849489},
	{SG24908, &UA2849506},
	{SG24909, &UA2850051},
	{SG24910, &UA2850094},
	{SG24911, &UA2850134},
	{SG24912, &UA2850265},
	{SG24913, &UA2850422},
	{SG24914, &UA2850667},
	{SG24915, &UA2851542},
	{SG24916, &UA2853613},
	{SG24917, &UA2853838},
	{SG24919, &UA2854170},
	{SG24921, &UA2854230},
	{SG24922, &UA2854447},
	{SG24923, &UA2854578},
	{SG24924, &UA2854673},
	{SG24925, &UA2854958},
	{SG24926, &UA2855051},
	{SG24928, &UA2855616},
	{SG24929, &UA2855606},
	{SG24931, &UA2855735},
	{SG24932, &UA2857230},
	{SG24933, &UA2857553},
	{SG24935, &UA2858020},
	{SG24936, &UA2858198},
	{SG24937, &UA2858497},
	{SG24938, &UA2859193},
	{SG24939, &UA2859325},
	{SG24940, &UA2860247},
	{SG24941, &UA2861298},
	{SG24942, &UA2861475},
	{SG24943, &UA2861650},
	{SG24945, &UA2862766},
	{SG24946, &UA2862892},
	{SG24947, &UA2863135},
	{SG24948, &UA2863357},
	{SG24949, &UA2863393},
	{SG24950, &UA2864227},
	{SG24951, &UA2864652},
	{SG24952, &UA2865130},
	{SG24954, &UA2865180},
	{SG24956, &UA2865284},
	{SG24958, &UA2865407},
	{SG24959, &UA2865442},
	{SG24960, &UA2865471},
	{SG24961, &UA2865551},
	{SG24962, &UA2865562},
	{SG24963, &UA2865861},
	{SG24964, &UA2865877},
	{SG24966, &UA2865900},
	{SG24967, &UA2866679},
	{SG24968, &UA2866917},
	{SG24969, &UA2867141},
	{SG24970, &UA2867535},
	{SG24971, &UA2869389},
	{SG24972, &UA2874808},
	{SG24973, &UA2878153},
	{SG24974, &UA2878230},
	{SG24975, &UA2878383},
	{SG24976, &UA2878872},
	{SG24977, &UA2879016},
	{SG24978, &UA2879069},
	{SG24979, &UA2879083},
	{SG24980, &UA2879403},
	{SG24981, &UA2889004},
	{SG24982, &UA2889191},
	{SG24983, &UA2889217},
	{SG24984, &UA2889223},
	{SG24985, &UA2889266},
	{SG24986, &UA2889381},
	{SG24987, &UA2889395},
	{SG24988, &UA2889421},
	{SG24989, &UA2889426},
	{SG24990, &UA2889479},
	{SG24991, &UA2890078},
	{SG24992, &UA2890200},
	{SG24993, &UA2890213},
	{SG24994, &UA2890347},
	{SG24995, &UA2890764},
	{SG24996, &UA2890916},
	{SG24997, &UA2890968},
	{SG24998, &UA2890975},
	{SG24999, &UA2890991},
	{SG25000, &UA2891052},
	{SG25001, &UA2891113},
	{SG25002, &UA2891184},
	{SG25004, &UA2891219},
	{SG25007, &UA2891517},
	{SG25008, &UA2891595},
	{SG25009, &UA2891667},
	{SG25011, &UA2891732},
	{SG25012, &UA2892006},
	{SG25013, &UA2892054},
	{SG25014, &UA2892108},
	{SG25015, &UA2892272},
	{SG25017, &UA2892327},
	{SG25019, &UA2892382},
	{SG25020, &UA2892766},
	{SG25022, &UA2892814},
	{SG25025, &UA2893950},
	{SG25028, &UA2894452},
	{SG25029, &UA2896966},
	{SG25031, &UA2933367},
	{SG25033, &UA2933524},
	{SG25036, &UA2933914},
	{SG25037, &UA2933941},
	{SG25038, &UA2933944},
	{SG25039, &UA2933952},
	{SG25041, &UA2934021},
	{SG25043, &UA2934046},
	{SG25044, &UA2934083},
	{SG25046, &UA2934085},
	{SG25047, &UA2934113},
	{SG25049, &UA2934306},
	{SG25051, &UA2934344},
	{SG25052, &UA2934482},
	{SG25055, &UA2934525},
	{SG25056, &UA2934549},
	{SG25058, &UA2934707},
	{SG25060, &UA2934768},
	{SG25062, &UA2934811},
	{SG25063, &UA2934863},
	{SG25064, &UA2934956},
	{SG25065, &UA2935174},
	{SG25067, &UA2935187},
	{SG25068, &UA2935234},
	{SG25070, &UA2935266},
	{SG25072, &UA2935269},
	{SG25073, &UA2935534},
	{SG25074, &UA2935536},
	{SG25075, &UA2936001},
	{SG25076, &UA2936008},
	{SG25078, &UA2936067},
	{SG25079, &UA2936198},
	{SG25080, &UA2936435},
	{SG25081, &UA2936533},
	{SG25083, &UA2936698},
	{SG25085, &UA2937022},
	{SG25086, &UA2937060},
	{SG25087, &UA2937136},
	{SG25088, &UA2937205},
	{SG25089, &UA2937264},
	{SG25090, &UA2937300},
	{SG25091, &UA2937310},
	{SG25092, &UA2937437},
	{SG25094, &UA2937476},
	{SG25096, &UA2937614},
	{SG25098, &UA2937793},
	{SG25100, &UA2937860},
	{SG25101, &UA2938253},
	{SG25102, &UA2938283},
	{SG25103, &UA2938282},
	{SG25105, &UA2939163},
	{SG25106, &UA2950407},
	{SG25108, &UA2950750},
	{SG25109, &UA2950841},
	{SG25111, &UA2950886},
	{SG25113, &UA2951057},
	{SG25114, &UA2951208},
	{SG25116, &UA2951282},
	{SG25117, &UA2951322},
	{SG25119, &UA2951367},
	{SG25120, &UA2951414},
	{SG25121, &UA2951439},
	{SG25122, &UA2951504},
	{SG25123, &UA2951513},
	{SG25124, &UA2951645},
	{SG25126, &UA2951656},
	{SG25128, &UA2951768},
	{SG25129, &UA2951817},
	{SG25130, &UA2951916},
	{SG25131, &UA2951936},
	{SG25132, &UA2951958},
	{SG25133, &UA2952048},
	{SG25135, &UA2952128},
	{SG25137, &UA2952277},
	{SG25138, &UA2952299},
	{SG25139, &UA2952723},
	{SG25140, &UA2952729},
	{SG25143, &UA2952830},
	{SG25144, &UA2952892},
	{SG25146, &UA2952946},
	{SG25147, &UA2953042},
	{SG25148, &UA2953082},
	{SG25149, &UA2953118},
	{SG25151, &UA2953133},
	{SG25152, &UA2953211},
	{SG25153, &UA2953306},
	{SG25154, &UA2953403},
	{SG25155, &UA2953497},
	{SG25157, &UA2953664},
	{SG25158, &UA2953797},
	{SG25159, &UA2953839},
	{SG25160, &UA2953945},
	{SG25161, &UA2954309},
	{SG25162, &UA2954637},
	{SG25163, &UA2955035},
	{SG25164, &UA2955222},
	{SG25166, &UA2955354},
	{SG25167, &UA2955611},
	{SG25169, &UA2955687},
	{SG25171, &UA2955759},
	{SG25173, &UA2955851},
	{SG25174, &UA2955946},
	{SG25176, &UA2956193},
	{SG25178, &UA2956323},
	{SG25179, &UA2956327},
	{SG25180, &UA2956398},
	{SG25182, &UA2956450},
	{SG25183, &UA2956521},
	{SG25185, &UA2956557},
	{SG25187, &UA2956784},
	{SG25188, &UA2956795},
	{SG25190, &UA2956958},
	{SG25192, &UA2956991},
	{SG25193, &UA2957153},
	{SG25195, &UA2957834},
	{SG25197, &UA2957951},
	{SG25198, &UA2958041},
	{SG25199, &UA2958059},
	{SG25201, &UA2958215},
	{SG25202, &UA2958307},
	{SG25203, &UA2958454},
	{SG25204, &UA2958457},
	{SG25205, &UA2958493},
	{SG25206, &UA2958503},
	{SG25209, &UA2958915},
	{SG25210, &UA2960342},
	{SG25213, &UA2960364},
	{SG25215, &UA2961642},
	{SG25217, &UA2962081},
	{SG25218, &UA2962254},
	{SG25220, &UA2962698},
	{SG25221, &UA2963192},
	{SG25222, &UA2964072},
	{SG25223, &UA2964736},
	{SG25224, &UA2970974},
	{SG25227, &UA2981288},
	{SG25229, &UA2981839},
	{SG25230, &UA2981915},
	{SG25232, &UA2981918},
	{SG25234, &UA2982017},
	{SG25236, &UA2982089},
	{SG25238, &UA2982233},
	{SG25239, &UA2982054},
	{SG25240, &UA2982538},
	{SG25241, &UA2982614},
	{SG25242, &UA2982633},
	{SG25243, &UA2982676},
	{SG25244, &UA2982692},
	{SG25245, &UA2982696},
	{SG25246, &UA2982764},
	{SG25248, &UA2983013},
	{SG25250, &UA2983124},
	{SG25251, &UA2983155},
	{SG25252, &UA2983171},
	{SG25253, &UA2983191},
	{SG25254, &UA2983335},
	{SG25255, &UA2983378},
	{SG25256, &UA2983400},
	{SG25257, &UA2983466},
	{SG25258, &UA2983498},
	{SG25260, &UA2983524},
	{SG25261, &UA2983527},
	{SG25262, &UA2983645},
	{SG25263, &UA2983650},
	{SG25264, &UA2983653},
	{SG25265, &UA2983682},
	{SG25266, &UA2983843},
	{SG25267, &UA2984049},
	{SG25270, &UA2984159},
	{SG25271, &UA2984248},
	{SG25272, &UA2984353},
	{SG25273, &UA2984368},
	{SG25274, &UA2984499},
	{SG25275, &UA2984708},
	{SG25277, &UA2984763},
	{SG25278, &UA2984827},
	{SG25279, &UA2984836},
	{SG25281, &UA2984922},
	{SG25283, &UA2984952},
	{SG25284, &UA2985027},
	{SG25286, &UA2985042},
	{SG25287, &UA2985062},
	{SG25288, &UA2985087},
	{SG25289, &UA2985096},
	{SG25291, &UA2985328},
	{SG25293, &UA2985401},
	{SG25294, &UA2985500},
	{SG25296, &UA2985511},
	{SG25298, &UA2985531},
	{SG25299, &UA2985606},
	{SG25300, &UA2985674},
	{SG25301, &UA2985722},
	{SG25302, &UA2985752},
	{SG25303, &UA2985793},
	{SG25304, &UA2985811},
	{SG25305, &UA2985936},
	{SG25306, &UA2986060},
	{SG25307, &UA2986143},
	{SG25308, &UA2986163},
	{SG25309, &UA2986170},
	{SG25310, &UA2986200},
	{SG25311, &UA2986343},
	{SG25312, &UA2986418},
	{SG25313, &UA2986530},
	{SG25314, &UA2986564},
	{SG25315, &UA2986576},
	{SG25317, &UA2986651},
	{SG25318, &UA2986628},
	{SG25319, &UA2986698},
	{SG25321, &UA2986839},
	{SG25322, &UA2986956},
	{SG25324, &UA2986953},
	{SG25325, &UA2987011},
	{SG25326, &UA2987175},
	{SG25327, &UA2987191},
	{SG25329, &UA2987206},
	{SG25330, &UA2987242},
	{SG25331, &UA2987272},
	{SG25332, &UA2987327},
	{SG25333, &UA2987355},
	{SG25334, &UA2987389},
	{SG25335, &UA2987486},
	{SG25336, &UA2987534},
	{SG25337, &UA2987667},
	{SG25338, &UA2987712},
	{SG25339, &UA2987728},
	{SG25340, &UA2987843},
	{SG25341, &UA2988119},
	{SG25343, &UA2988222},
	{SG25345, &UA2988300},
	{SG25346, &UA2988369},
	{SG25348, &UA2988425},
	{SG25350, &UA2988513},
	{SG25351, &UA2988917},
	{SG25352, &UA2989062},
	{SG25353, &UA2989104},
	{SG25354, &UA2989229},
	{SG25355, &UA2989262},
	{SG25356, &UA2989448},
	{SG25357, &UA2989537},
	{SG25358, &UA2989628},
	{SG25359, &UA2990041},
	{SG25360, &UA2990372},
	{SG25361, &UA2990488},
	{SG25362, &UA2990499},
	{SG25363, &UA2990597},
	{SG25364, &UA2990618},
	{SG25365, &UA2990644},
	{SG25366, &UA2990771},
	{SG25367, &UA2990843},
	{SG25368, &UA2991445},
	{SG25369, &UA2991540},
	{SG25370, &UA2991670},
	{SG25371, &UA2992116},
	{SG25372, &UA2992172},
	{SG25373, &UA2992182},
	{SG25374, &UA2992360},
	{SG25375, &UA2992631},
	{SG25376, &UA2992799},
	{SG25378, &UA2993031},
	{SG25379, &UA2993389},
	{SG25380, &UA2993506},
	{SG25381, &UA2994086},
	{SG25382, &UA2994288},
	{SG25383, &UA2994380},
	{SG25384, &UA2994503},
	{SG25385, &UA2994545},
	{SG25386, &UA2994740},
	{SG25387, &UA2994874},
	{SG25388, &UA2995015},
	{SG25389, &UA2995017},
	{SG25390, &UA2995340},
	{SG25391, &UA2995641},
	{SG25392, &UA2995700},
	{SG25393, &UA2995803},
	{SG25394, &UA2995817},
	{SG25395, &UA2996369},
	{SG25396, &UA2996515},
	{SG25397, &UA2996649},
	{SG25398, &UA2996674},
	{SG25399, &UA2996817},
	{SG25400, &UA2997046},
	{SG25401, &UA2997056},
	{SG25402, &UA2997138},
	{SG25403, &UA2997147},
	{SG25404, &UA2997382},
	{SG25405, &UA2997789},
	{SG25406, &UA2998102},
	{SG25408, &UA2998165},
	{SG25410, &UA2998202},
	{SG25411, &UA2998243},
	{SG25412, &UA2998296},
	{SG25413, &UA2998548},
	{SG25414, &UA2998620},
	{SG25415, &UA2998644},
	{SG25416, &UA2998945},
	{SG25417, &UA2999047},
	{SG25418, &UA2999059},
	{SG25419, &UA2999105},
	{SG25420, &UA2999119},
	{SG25421, &UA2999330},
	{SG25422, &UA2999524},
	{SG25423, &UA2999663},
	{SG25424, &UA2999665},
	{SG25425, &UA2999674},
	{SG25426, &UA2999697},
	{SG25427, &UA2999922},
	{SG25428, &UA2999977},
	{SG25429, &UA3000010},
	{SG25430, &UA3001270},
	{SG25431, &UA3001636},
	{SG25432, &UA3001936},
	{SG25433, &UA3002797},
	{SG25435, &UA3002860},
	{SG25436, &UA3002898},
	{SG25437, &UA3003067},
	{SG25438, &UA3003520},
	{SG25439, &UA3003521},
	{SG25440, &UA3003771},
	{SG25441, &UA3003958},
	{SG25442, &UA3004131},
	{SG25443, &UA3004956},
	{SG25444, &UA3005276},
	{SG25445, &UA3006274},
	{SG25446, &UA3006415},
	{SG25447, &UA3009366},
	{SG25448, &UA3009581},
	{SG25449, &UA3009907},
	{SG25450, &UA3010243},
	{SG25451, &UA3011189},
	{SG25452, &UA3011619},
	{SG25453, &UA3012164},
	{SG25454, &UA3012973},
	{SG25455, &UA3014705},
	{SG25456, &UA3015538},
	{SG25457, &UA3016463},
	{SG25458, &UA3016745},
	{SG25459, &UA3018461},
	{SG25460, &UA3018998},
	{SG25461, &UA3020623},
	{SG25462, &UA3022627},
	{SG25464, &UA3027427},
	{SG25465, &UA3031253},
	{SG25466, &UA3038893},
	{SG25467, &UA3042227},
	{SG25468, &UA3042416},
	{SG25469, &UA3042549},
	{SG25470, &UA3042760},
	{SG25471, &UA3042771},
	{SG25472, &UA3042780},
	{SG25473, &UA3042812},
	{SG25474, &UA3042820},
	{SG25475, &UA3042911},
	{SG25476, &UA3042919},
	{SG25477, &UA3043082},
	{SG25478, &UA3043249},
	{SG25479, &UA3043290},
	{SG25480, &UA3043338},
	{SG25482, &UA3043442},
	{SG25483, &UA3043514},
	{SG25484, &UA3043550},
	{SG25485, &UA3043614},
	{SG25486, &UA3043615},
	{SG25488, &UA3043637},
	{SG25489, &UA3043680},
	{SG25490, &UA3043795},
	{SG25491, &UA3043816},
	{SG25492, &UA3043985},
	{SG25493, &UA3044038},
	{SG25494, &UA3044087},
	{SG25498, &UA3044110},
	{SG25499, &UA3044270},
	{SG25500, &UA3044378},
	{SG25501, &UA3044423},
	{SG25503, &UA3044512},
	{SG25504, &UA3044677},
	{SG25505, &UA3044655},
	{SG25506, &UA3044699},
	{SG25507, &UA3044715},
	{SG25508, &UA3044920},
	{SG25509, &UA3044980},
	{SG25510, &UA3045027},
	{SG25511, &UA3045123},
	{SG25512, &UA3045325},
	{SG25513, &UA3045360},
	{SG25514, &UA3045400},
	{SG25515, &UA3045523},
	{SG25516, &UA3045621},
	{SG25517, &UA3045730},
	{SG25518, &UA3045816},
	{SG25519, &UA3045842},
	{SG25520, &UA3046201},
	{SG25521, &UA3046389},
	{SG25522, &UA3046410},
	{SG25523, &UA3047161},
	{SG25524, &UA3047212},
	{SG25525, &UA3047188},
	{SG25526, &UA3047431},
	{SG25527, &UA3047439},
	{SG25528, &UA3047755},
	{SG25529, &UA3048207},
	{SG25530, &UA3048463},
	{SG25531, &UA3048645},
	{SG25532, &UA3048742},
	{SG25533, &UA3048779},
	{SG25534, &UA3048819},
	{SG25535, &UA3048930},
	{SG25536, &UA3048986},
	{SG25537, &UA3049261},
	{SG25538, &UA3049287},
	{SG25539, &UA3049302},
	{SG25540, &UA3049335},
	{SG25541, &UA3049882},
	{SG25542, &UA3049953},
	{SG25543, &UA3049983},
	{SG25545, &UA3050143},
	{SG25546, &UA3050201},
	{SG25547, &UA3050308},
	{SG25548, &UA3050467},
	{SG25549, &UA3050473},
	{SG25550, &UA3050555},
	{SG25551, &UA3050575},
	{SG25552, &UA3050683},
	{SG25553, &UA3051884},
	{SG25555, &UA3054445},
	{SG25556, &UA3055544},
	{SG25557, &UA3056090},
	{SG25558, &UA3056484},
	{SG25559, &UA3056851},
	{SG25560, &UA3056917},
	{SG25561, &UA3057361},
	{SG25562, &UA3057523},
	{SG25563, &UA3058268},
	{SG25564, &UA3058905},
	{SG25565, &UA3059728},
	{SG25567, &UA3069975},
	{SG25568, &UA3070282},
	{SG25569, &UA3070294},
	{SG25570, &UA3070711},
	{SG25571, &UA3070783},
	{SG25572, &UA3070971},
	{SG25573, &UA3070992},
	{SG25574, &UA3071027},
	{SG25575, &UA3071029},
	{SG25576, &UA3071078},
	{SG25577, &UA3071113},
	{SG25578, &UA3071214},
	{SG25579, &UA3071271},
	{SG25581, &UA3071356},
	{SG25583, &UA3071426},
	{SG25584, &UA3071857},
	{SG25585, &UA3071901},
	{SG25586, &UA3072371},
	{SG25587, &UA3072505},
	{SG25588, &UA3072563},
	{SG25589, &UA3073267},
	{SG25590, &UA3073598},
	{SG25591, &UA3073691},
	{SG25592, &UA3073768},
	{SG25593, &UA3073836},
	{SG25594, &UA3073857},
	{SG25595, &UA3073876},
	{SG25596, &UA3073921},
	{SG25597, &UA3074050},
	{SG25598, &UA3074173},
	{SG25599, &UA3074230},
	{SG25600, &UA3074295},
	{SG25601, &UA3074973},
	{SG25602, &UA3075043},
	{SG25603, &UA3076321},
	{SG25604, &UA3077219},
	{SG25605, &UA3078216},
	{SG25606, &UA3078262},
	{SG25607, &UA3078306},
	{SG25608, &UA3078504},
	{SG25609, &UA3078685},
	{SG25610, &UA3078797},
	{SG25611, &UA3078824},
	{SG25612, &UA3078932},
	{SG25613, &UA3079226},
	{SG25614, &UA3079399},
	{SG25615, &UA3079465},
	{SG25616, &UA3084976},
	{SG25617, &UA3085445},
	{SG25618, &UA3085501},
	{SG25619, &UA3085700},
	{SG25620, &UA3085927},
	{SG25621, &UA3086002},
	{SG25622, &UA3086116},
	{SG25623, &UA3086164},
	{SG25624, &UA3086177},
	{SG25625, &UA3086243},
	{SG25626, &UA3086300},
	{SG25627, &UA3086316},
	{SG25628, &UA3086317},
	{SG25629, &UA3086331},
	{SG25630, &UA3086486},
	{SG25631, &UA3086810},
	{SG25632, &UA3087096},
	{SG25634, &UA3087333},
	{SG25636, &UA3087431},
	{SG25637, &UA3087437},
	{SG25638, &UA3087510},
	{SG25639, &UA3087683},
	{SG25640, &UA3087808},
	{SG25641, &UA3087885},
	{SG25642, &UA3087936},
	{SG25643, &UA3088070},
	{SG25644, &UA3088396},
	{SG25646, &UA3088704},
	{SG25648, &UA3089978},
	{SG25649, &UA3091403},
	{SG25650, &UA3091642},
	{SG25651, &UA3091965},
	{SG25652, &UA3092205},
	{SG25653, &UA3093047},
	{SG25654, &UA3098510},
	{SG25655, &UA3098604},
	{SG25656, &UA3098633},
	{SG25658, &UA3098663},
	{SG25659, &UA3098675},
	{SG25660, &UA3098933},
	{SG25661, &UA3098939},
	{SG25662, &UA3098951},
	{SG25663, &UA3099005},
	{SG25664, &UA3099072},
	{SG25665, &UA3099110},
	{SG25666, &UA3099129},
	{SG25667, &UA3099234},
	{SG25668, &UA3099271},
	{SG25669, &UA3099305},
	{SG25670, &UA3099391},
	{SG25671, &UA3099427},
	{SG25672, &UA3099435},
	{SG25673, &UA3099457},
	{SG25674, &UA3099575},
	{SG25676, &UA3099652},
	{SG25677, &UA3099682},
	{SG25678, &UA3099775},
	{SG25679, &UA3099826},
	{SG25681, &UA3099860},
	{SG25682, &UA3100009},
	{SG25683, &UA3100077},
	{SG25685, &UA3100079},
	{SG25686, &UA3100089},
	{SG25687, &UA3100231},
	{SG25688, &UA3100277},
	{SG25689, &UA3100297},
	{SG25690, &UA3100302},
	{SG25691, &UA3100327},
	{SG25692, &UA3100447},
	{SG25693, &UA3100654},
	{SG25694, &UA3100661},
	{SG25695, &UA3100753},
	{SG25696, &UA3100961},
	{SG25697, &UA3101059},
	{SG25698, &UA3101095},
	{SG25699, &UA3101849},
	{SG25700, &UA3101910},
	{SG25701, &UA3102002},
	{SG25702, &UA3102726},
	{SG25703, &UA3102791},
	{SG25704, &UA3103064},
	{SG25705, &UA3103118},
	{SG25706, &UA3103148},
	{SG25707, &UA3103153},
	{SG25708, &UA3103260},
	{SG25709, &UA3103416},
	{SG25710, &UA3103514},
	{SG25711, &UA3105073},
	{SG25712, &UA3105075},
	{SG25713, &UA3105582},
	{SG25714, &UA3105841},
	{SG25715, &UA3106225},
	{SG25716, &UA3107186},
	{SG25718, &UA3107814},
	{SG25719, &UA3107947},
	{SG25720, &UA3107971},
	{SG25721, &UA3108033},
	{SG25722, &UA3108116},
	{SG25723, &UA3108167},
	{SG25724, &UA3108391},
	{SG25725, &UA3108637},
	{SG25727, &UA3109137},
	{SG25728, &UA3109189},
	{SG25730, &UA3113532},
	{SG25731, &UA3117886},
	{SG25732, &UA3117893},
	{SG25733, &UA3118037},
	{SG25734, &UA3118049},
	{SG25736, &UA3118091},
	{SG25737, &UA3118098},
	{SG25738, &UA3118334},
	{SG25739, &UA3118349},
	{SG25740, &UA3118761},
	{SG25741, &UA3118911},
	{SG25742, &UA3118949},
	{SG25743, &UA3119021},
	{SG25744, &UA3119126},
	{SG25745, &UA3119189},
	{SG25746, &UA3119339},
	{SG25747, &UA3119486},
	{SG25748, &UA3119670},
	{SG25749, &UA3120429},
	{SG25750, &UA3120526},
	{SG25751, &UA3120781},
	{SG25752, &UA3120939},
	{SG25753, &UA3120993},
	{SG25754, &UA3121021},
	{SG25755, &UA3121511},
	{SG25756, &UA3121569},
	{SG25757, &UA3121771},
	{SG25758, &UA3122630},
	{SG25759, &UA3122953},
	{SG25760, &UA3124607},
	{SG25761, &UA3124786},
	{SG25762, &UA3124950},
	{SG25763, &UA3124999},
	{SG25765, &UA3125028},
	{SG25766, &UA3125137},
	{SG25767, &UA3125140},
	{SG25768, &UA3125437},
	{SG25769, &UA3125446},
	{SG25771, &UA3125654},
	{SG25772, &UA692283},
	{SG25773, &UA707498},
	{SG25774, &UA807183},
	{SG25775, &UA827157},
	{SG25776, &UA877589},
	{SG25777, &UA895362},
	{SG25778, &UA895894},
	{SG25779, &UA899319},
	{SG25780, &UA899766},
	{SG25781, &UA899954},
	{SG25782, &UA900274},
	{SG25783, &UA900455},
	{SG25784, &UA902246},
	{SG25785, &UA905196},
	{SG25786, &UA906312},
	{SG25788, &UA907053},
	{SG25789, &UA909895},
	{SG25790, &UA909986},
	{SG25791, &UA911145},
	{SG25792, &UA912973},
	{SG25793, &UA925360},
	{SG25794, &UA1959397},
	{SG25795, &UA1965583},
	{SG25796, &UA1976759},
	{SG25797, &UA1990702},
	{SG25798, &UA2001697},
	{SG25799, &UA2003042},
	{SG25800, &UA2013274},
	{SG25801, &UA2027059},
	{SG25802, &UA2027810},
	{SG25803, &UA2028384},
	{SG25804, &UA2030390},
	{SG25805, &UA2030989},
	{SG25806, &UA2032808},
	{SG25807, &UA2042026},
	{SG25808, &UA2174393},
	{SG25809, &UA2174873},
	{SG25810, &UA2182306},
	{SG25811, &UA2201485},
	{SG25812, &UA2203401},
	{SG25813, &UA2223412},
	{SG25814, &UA2225885},
	{SG25815, &UA2257393},
	{SG25816, &UA2257450},
	{SG25817, &UA2257504},
	{SG25818, &UA2257602},
	{SG25819, &UA2257706},
	{SG25820, &UA2352675},
	{SG25821, &UA2355598},
	{SG25822, &UA2375535},
	{SG25824, &UA2395193},
	{SG25825, &UA2398263},
	{SG25826, &UA2410620},
	{SG25827, &UA2418340},
	{SG25829, &UA2507300},
	{SG25830, &UA2520796},
	{SG25831, &UA2548355},
	{SG25833, &UA2553596},
	{SG25834, &UA2577950},
	{SG25841, &UA2588876},
	{SG25842, &UA2590532},
	{SG25843, &UA2652905},
	{SG25844, &UA2653527},
	{SG25845, &UA2726311},
	{SG25846, &UA2829905},
	{SG25847, &UA2832724},
	{SG25848, &UA2841095},
	{SG25849, &UA2937285},
	{SG25850, &UA2982250},
	{SG25998, &UA622430},
	{SG25999, &UA722362},
	{SG26000, &UA898467},
	{SG26001, &UA923932},
	{SG26002, &UA1960962},
	{SG26003, &UA1994659},
	{SG26004, &UA1997039},
	{SG26005, &UA1998753},
	{SG26006, &UA2008384},
	{SG26007, &UA2027449},
	{SG26008, &UA2027872},
	{SG26009, &UA2032024},
	{SG26010, &UA2033102},
	{SG26011, &UA2040607},
	{SG26012, &UA2041194},
	{SG26013, &UA2159582},
	{SG26014, &UA2179085},
	{SG26015, &UA2191883},
	{SG26016, &UA2201664},
	{SG26017, &UA2203403},
	{SG26018, &UA2223962},
	{SG26019, &UA2242334},
	{SG26020, &UA2346639},
	{SG26021, &UA2375166},
	{SG26022, &UA2400261},
	{SG26024, &UA2408017},
	{SG26025, &UA2426059},
	{SG26027, &UA2454855},
	{SG26029, &UA2502358},
	{SG26030, &UA2526181},
	{SG26031, &UA2745159},
	{SG26149, &UA712784},
	{SG26151, &UA900572},
	{SG26153, &UA906957},
	{SG26156, &UA1992517},
	{SG26158, &UA1996249},
	{SG26159, &UA2001391},
	{SG26161, &UA2025457},
	{SG26164, &UA2028502},
	{SG26165, &UA2031971},
	{SG26167, &UA2033519},
	{SG26169, &UA2033658},
	{SG26171, &UA2042983},
	{SG26172, &UA2166708},
	{SG26174, &UA2193309},
	{SG26200, &UA2197916},
	{SG26202, &UA2204929},
	{SG26204, &UA2205851},
	{SG26205, &UA2206110},
	{SG26207, &UA2206151},
	{SG26208, &UA2235465},
	{SG26210, &UA2341976},
	{SG26211, &UA2351384},
	{SG26213, &UA2355838},
	{SG26214, &UA2356264},
	{SG26216, &UA2365347},
	{SG26218, &UA2413784},
	{SG26220, &UA2414316},
	{SG26225, &UA2658567},
	{SG26228, &UA2659136},
	{SG26230, &UA2659171},
	{SG26233, &UA2659427},
	{SG26235, &UA2659840},
	{SG26268, &UA923928},
	{SG26269, &UA1982028},
	{SG26270, &UA2026133},
	{SG26271, &UA2030093},
	{SG26272, &UA2194711},
	{SG26273, &UA2203363},
	{SG26274, &UA2236364},
	{SG26276, &UA2353920},
	{SG26277, &UA2361142},
	{SG26278, &UA2613502},
	{SG26279, &UA2614176},
	{SG26280, &UA2614865},
	{SG26281, &UA2615217},
	{SG26282, &UA2615454},
	{SG26283, &UA2615715},
	{SG26284, &UA2615838},
	{SG26285, &UA2616587},
	{SG26286, &UA2616949},
	{SG26287, &UA2617061},
	{SG26288, &UA2617069},
	{SG26289, &UA2617143},
	{SG26290, &UA2617350},
	{SG26291, &UA2617643},
	{SG26292, &UA2618071},
	{SG26293, &UA2618370},
	{SG26294, &UA2618395},
	{SG26295, &UA2619866},
	{SG26296, &UA2619912},
	{SG26297, &UA2620054},
	{SG26298, &UA2621914},
	{SG26299, &UA2621924},
	{SG26300, &UA2622123},
	{SG26301, &UA2622237},
	{SG26302, &UA2622393},
	{SG26304, &UA2622430},
	{SG26306, &UA2623525},
	{SG26307, &UA2623561},
	{SG26308, &UA2624075},
	{SG26310, &UA2627556},
	{SG26311, &UA2629027},
	{SG26312, &UA2631611},
	{SG26313, &UA2635844},
	{SG26314, &UA2637196},
	{SG26316, &UA2637564},
	{SG26317, &UA2799940},
	{SG26318, &UA2843659},
	{SG26320, &UA3031356},
	{SG26610, &UA441511},
	{SG26612, &UA454097},
	{SG26613, &UA454580},
	{SG26614, &UA457544},
	{SG26616, &UA607397},
	{SG26617, &UA612771},
	{SG26620, &UA616197},
	{SG26622, &UA622363},
	{SG26625, &UA639418},
	{SG26628, &UA851242},
	{SG26629, &UA895005},
	{SG26631, &UA899376},
	{SG26632, &UA899413},
	{SG26633, &UA909834},
	{SG26634, &UA910204},
	{SG26636, &UA911716},
	{SG26638, &UA1961975},
	{SG26640, &UA1994992},
	{SG26641, &UA1995063},
	{SG26642, &UA2025430},
	{SG26643, &UA2026398},
	{SG26644, &UA2032374},
	{SG26645, &UA2033094},
	{SG26646, &UA2033433},
	{SG26647, &UA2186207},
	{SG26648, &UA2186510},
	{SG26649, &UA2203092},
	{SG26650, &UA2203564},
	{SG26651, &UA2227170},
	{SG26652, &UA2238272},
	{SG26654, &UA2239227},
	{SG26656, &UA2242305},
	{SG26660, &UA2257623},
	{SG26662, &UA2335517},
	{SG26665, &UA2342961},
	{SG26666, &UA2350374},
	{SG26667, &UA2353155},
	{SG26668, &UA2354601},
	{SG26669, &UA2355270},
	{SG26671, &UA2356638},
	{SG26672, &UA2360444},
	{SG26673, &UA2360842},
	{SG26674, &UA2361392},
	{SG26675, &UA2363138},
	{SG26677, &UA2363892},
	{SG26678, &UA2364440},
	{SG26679, &UA2365093},
	{SG26680, &UA2365823},
	{SG26681, &UA2366047},
	{SG26682, &UA2366353},
	{SG26683, &UA2366635},
	{SG26684, &UA2366691},
	{SG26686, &UA2366842},
	{SG26687, &UA2367071},
	{SG26688, &UA2367255},
	{SG26690, &UA2367783},
	{SG26696, &UA2373535},
	{SG26701, &UA2375480},
	{SG26703, &UA2375694},
	{SG26706, &UA2375971},
	{SG26710, &UA2376691},
	{SG26712, &UA2377227},
	{SG26713, &UA2378160},
	{SG26716, &UA2378211},
	{SG26720, &UA2380262},
	{SG26723, &UA2382030},
	{SG26726, &UA2382121},
	{SG26728, &UA2382320},
	{SG26730, &UA2382581},
	{SG26732, &UA2382997},
	{SG26734, &UA2383048},
	{SG26736, &UA2383594},
	{SG26738, &UA2383697},
	{SG26740, &UA2383731},
	{SG26742, &UA2384812},
	{SG26743, &UA2384888},
	{SG26744, &UA2384987},
	{SG26745, &UA2385124},
	{SG26746, &UA2385232},
	{SG26747, &UA2385320},
	{SG26748, &UA2385514},
	{SG26749, &UA2386239},
	{SG26750, &UA2386387},
	{SG26752, &UA2386568},
	{SG26753, &UA2387152},
	{SG26754, &UA2387490},
	{SG26756, &UA2387827},
	{SG26757, &UA2388817},
	{SG26758, &UA2393367},
	{SG26759, &UA2395560},
	{SG26760, &UA2395726},
	{SG26762, &UA2396053},
	{SG26763, &UA2396458},
	{SG26764, &UA2397357},
	{SG26765, &UA2397362},
	{SG26766, &UA2398348},
	{SG26767, &UA2399510},
	{SG26768, &UA2400316},
	{SG26769, &UA2400317},
	{SG26770, &UA2400679},
	{SG26771, &UA2400720},
	{SG26773, &UA2400938},
	{SG26774, &UA2400969},
	{SG26775, &UA2401022},
	{SG26776, &UA2401016},
	{SG26778, &UA2401043},
	{SG26779, &UA2401433},
	{SG26783, &UA2402190},
	{SG26784, &UA2402262},
	{SG26785, &UA2402559},
	{SG26786, &UA2403397},
	{SG26787, &UA2403951},
	{SG26788, &UA2404050},
	{SG26789, &UA2404708},
	{SG26790, &UA2404866},
	{SG26791, &UA2404977},
	{SG26792, &UA2405227},
	{SG26793, &UA2406676},
	{SG26794, &UA2414021},
	{SG26795, &UA2416027},
	{SG26796, &UA2420403},
	{SG26797, &UA2422376},
	{SG26798, &UA2424074},
	{SG26799, &UA2426214},
	{SG26800, &UA2428363},
	{SG26801, &UA2433439},
	{SG26802, &UA2437183},
	{SG26803, &UA2437249},
	{SG26804, &UA2437589},
	{SG26805, &UA2455593},
	{SG26806, &UA2457261},
	{SG26807, &UA2457440},
	{SG26809, &UA2468387},
	{SG26810, &UA2469312},
	{SG26811, &UA2483665},
	{SG26812, &UA2484530},
	{SG26813, &UA2485860},
	{SG26814, &UA2499705},
	{SG26815, &UA2500853},
	{SG26816, &UA2501913},
	{SG26817, &UA2508170},
	{SG26819, &UA2522708},
	{SG26820, &UA2525496},
	{SG26821, &UA2527660},
	{SG26822, &UA2527713},
	{SG26824, &UA2528582},
	{SG26825, &UA2552500},
	{SG26826, &UA2554601},
	{SG26827, &UA2558597},
	{SG26828, &UA2560590},
	{SG26830, &UA2561384},
	{SG26831, &UA2563068},
	{SG26832, &UA2565458},
	{SG26833, &UA2577728},
	{SG26835, &UA2578014},
	{SG26836, &UA2578103},
	{SG26837, &UA2596597},
	{SG26838, &UA2598637},
	{SG26839, &UA2600684},
	{SG26840, &UA2652167},
	{SG26841, &UA2652271},
	{SG26842, &UA2652628},
	{SG26843, &UA2653174},
	{SG26844, &UA2653954},
	{SG26845, &UA2655505},
	{SG26846, &UA2656243},
	{SG26847, &UA2656417},
	{SG26849, &UA2659552},
	{SG26850, &UA2691294},
	{SG26851, &UA2698377},
	{SG26852, &UA2703353},
	{SG26854, &UA2706708},
	{SG26856, &UA2734429},
	{SG26857, &UA2746499},
	{SG26858, &UA2746718},
	{SG26859, &UA2764925},
	{SG26861, &UA2768690},
	{SG26863, &UA2778057},
	{SG26864, &UA2778490},
	{SG26865, &UA2790177},
	{SG26866, &UA2790773},
	{SG26867, &UA2816971},
	{SG26868, &UA2824259},
	{SG26869, &UA2829209},
	{SG26870, &UA2838134},
	{SG26871, &UA2848439},
	{SG26872, &UA2849089},
	{SG26874, &UA2857420},
	{SG26875, &UA2863742},
	{SG26876, &UA2876696},
	{SG26877, &UA2878268},
	{SG26878, &UA2889532},
	{SG26880, &UA2889572},
	{SG26881, &UA2934214},
	{SG26882, &UA2950319},
	{SG26883, &UA2950922},
	{SG26885, &UA2981615},
	{SG26886, &UA2985350},
	{SG26888, &UA2986681},
	{SG26889, &UA2987503},
	{SG26891, &UA2987745},
	{SG26892, &UA2988049},
	{SG26893, &UA3043101},
	{SG26894, &UA3043344},
	{SG26895, &UA3044338},
	{SG26896, &UA3070346},
	{SG26897, &UA3099228},
	{SG26899, &UA3099622},
	{SG26900, &UA3107503},
	{SG26901, &UA3118533},
	{SG26903, &UA3118682},
	{SG26904, &UA3119518},
	{SG26912, &UA909017},
	{SG26913, &UA924859},
	{SG26915, &UA1953961},
	{SG26916, &UA2007461},
	{SG26917, &UA2030559},
	{SG26918, &UA2030756},
	{SG26919, &UA2191499},
	{SG26920, &UA2232271},
	{SG26921, &UA2236127},
	{SG26922, &UA2338804},
	{SG26923, &UA2423869},
	{SG26924, &UA2525545},
	{SG26925, &UA2528416},
	{SG26926, &UA2745653},
	{SG26927, &UA2746123},
	{SG26929, &UA2820619},
	{SG27041, &UA1954336},
	{SG27042, &UA1996699},
	{SG27043, &UA2028664},
	{SG27044, &UA2029184},
	{SG27045, &UA2167961},
	{SG27046, &UA2182443},
	{SG27047, &UA2192579},
	{SG27048, &UA2195947},
	{SG27049, &UA2199434},
	{SG27050, &UA2199843},
	{SG27051, &UA2201861},
	{SG27052, &UA2205763},
	{SG27053, &UA2257443},
	{SG27054, &UA2356305},
	{SG27055, &UA2386137},
	{SG27056, &UA2416989},
	{SG27057, &UA2527453},
	{SG27058, &UA2597338},
	{SG27059, &UA2598150},
	{SG27060, &UA2659156},
	{SG27061, &UA2659452},
	{SG27062, &UA2828801},
	{SG27063, &UA2849897},
	{SG27064, &UA3057005},
	{SG27065, &UA3118885},
	{SG27075, &UA902475},
	{SG27076, &UA1972524},
	{SG27077, &UA2257843},
	{SG27078, &UA2615032},
	{SG27080, &UA2615096},
	{SG27082, &UA2615939},
	{SG27083, &UA2616650},
	{SG27084, &UA2616900},
	{SG27086, &UA2617948},
	{SG27088, &UA2620142},
	{SG27090, &UA2620651},
	{SG27091, &UA2621086},
	{SG27092, &UA2621836},
	{SG27093, &UA2622638},
	{SG27094, &UA2623379},
	{SG27096, &UA2623541},
	{SG27097, &UA2623573},
	{SG27099, &UA2623575},
	{SG27100, &UA2624229},
	{SG27101, &UA2625543},
	{SG27102, &UA2625895},
	{SG27103, &UA2627322},
	{SG27104, &UA2627503},
	{SG27105, &UA2628699},
	{SG27106, &UA2635589},
	{SG27107, &UA2635588},
	{SG27132, &UA802072},
	{SG27133, &UA810716},
	{SG27134, &UA890774},
	{SG27135, &UA897563},
	{SG27136, &UA898912},
	{SG27137, &UA903210},
	{SG27138, &UA908297},
	{SG27139, &UA910716},
	{SG27140, &UA911768},
	{SG27141, &UA1950991},
	{SG27142, &UA1983686},
	{SG27143, &UA1988481},
	{SG27144, &UA2027832},
	{SG27145, &UA2030225},
	{SG27146, &UA2033761},
	{SG27147, &UA2167625},
	{SG27148, &UA2193855},
	{SG27149, &UA2222767},
	{SG27150, &UA2223051},
	{SG27151, &UA2227672},
	{SG27152, &UA2257672},
	{SG27153, &UA2319327},
	{SG27154, &UA2339880},
	{SG27155, &UA2356155},
	{SG27156, &UA2364988},
	{SG27157, &UA2367460},
	{SG27158, &UA2457794},
	{SG27159, &UA2521098},
	{SG27160, &UA2822226},
	{SG27161, &UA3069981},
	{SG27162, &UA2355665},
	{SG27163, &UA2659145},
	{SG27164, &UA2706436},
	{SG27165, &UA3046670},
	{SG27169, &UA899632},
	{SG27170, &UA1964502},
	{SG27172, &UA2195658},
	{SG27174, &UA2198946},
	{SG27175, &UA2356901},
	{SG27176, &UA2413456},
	{SG27177, &UA2460652},
	{SG27178, &UA2507200},
	{SG27179, &UA2611440},
	{SG27180, &UA2612182},
	{SG27181, &UA2614667},
	{SG27182, &UA2614906},
	{SG27183, &UA2615190},
	{SG27184, &UA2615279},
	{SG27185, &UA2615328},
	{SG27186, &UA2615894},
	{SG27187, &UA2616014},
	{SG27188, &UA2616202},
	{SG27189, &UA2616294},
	{SG27190, &UA2616705},
	{SG27191, &UA2617236},
	{SG27193, &UA2617397},
	{SG27194, &UA2617458},
	{SG27195, &UA2617490},
	{SG27196, &UA2617845},
	{SG27198, &UA2617883},
	{SG27199, &UA2617929},
	{SG27200, &UA2618320},
	{SG27202, &UA2618449},
	{SG27203, &UA2618463},
	{SG27204, &UA2618637},
	{SG27205, &UA2619513},
	{SG27206, &UA2620335},
	{SG27207, &UA2621067},
	{SG27208, &UA2621211},
	{SG27209, &UA2621514},
	{SG27210, &UA2621620},
	{SG27211, &UA2621969},
	{SG27212, &UA2622096},
	{SG27213, &UA2622339},
	{SG27214, &UA2622595},
	{SG27215, &UA2622671},
	{SG27216, &UA2622910},
	{SG27217, &UA2623579},
	{SG27218, &UA2624072},
	{SG27219, &UA2624645},
	{SG27220, &UA2625126},
	{SG27221, &UA2625242},
	{SG27222, &UA2626790},
	{SG27223, &UA2627579},
	{SG27224, &UA2627820},
	{SG27225, &UA2628147},
	{SG27226, &UA2628481},
	{SG27227, &UA2629085},
	{SG27228, &UA2630265},
	{SG27229, &UA2630351},
	{SG27230, &UA2634967},
	{SG27231, &UA2635059},
	{SG27232, &UA2635632},
	{SG27233, &UA2636319},
	{SG27234, &UA3020373},
	{SG27235, &UA3056920},
	{SG27242, &UA2004600},
	{SG27244, &UA2032131},
	{SG27245, &UA2033324},
	{SG27247, &UA2035149},
	{SG27248, &UA2035157},
	{SG27250, &UA2035234},
	{SG27251, &UA2191571},
	{SG27253, &UA2193192},
	{SG27254, &UA2205960},
	{SG27255, &UA2229728},
	{SG27256, &UA2238877},
	{SG27258, &UA2342379},
	{SG27259, &UA2347305},
	{SG27260, &UA2349646},
	{SG27261, &UA2382496},
	{SG27262, &UA2388503},
	{SG27263, &UA2401677},
	{SG27264, &UA2417159},
	{SG27265, &UA2419902},
	{SG27266, &UA2424662},
	{SG27267, &UA2426004},
	{SG27268, &UA2426425},
	{SG27269, &UA2430029},
	{SG27270, &UA2459310},
	{SG27271, &UA2480436},
	{SG27272, &UA2483825},
	{SG27273, &UA2485355},
	{SG27274, &UA2500336},
	{SG27275, &UA2505274},
	{SG27276, &UA2507127},
	{SG27277, &UA2508002},
	{SG27278, &UA2554453},
	{SG27279, &UA2568417},
	{SG27280, &UA2579063},
	{SG27281, &UA2580339},
	{SG27282, &UA2583297},
	{SG27283, &UA2668326},
	{SG27284, &UA2668753},
	{SG27285, &UA2729059},
	{SG27286, &UA2730097},
	{SG27287, &UA2832793},
	{SG27288, &UA2985601},
	{SG27289, &UA2995744},
	{SG27290, &UA3070269},
	{SG27295, &UA616154},
	{SG27296, &UA775639},
	{SG27297, &UA2032100},
	{SG27298, &UA2033505},
	{SG27299, &UA2360160},
	{SG27300, &UA2403189},
	{SG27301, &UA2444443},
	{SG27302, &UA2457730},
	{SG27303, &UA2464041},
	{SG27304, &UA2484480},
	{SG27305, &UA2520067},
	{SG27306, &UA2579908},
	{SG27307, &UA2694241},
	{SG27436, &UA902255},
	{SG27438, &UA907862},
	{SG27440, &UA1980540},
	{SG27442, &UA1986024},
	{SG27444, &UA2014761},
	{SG27448, &UA2033428},
	{SG27450, &UA2224490},
	{SG27452, &UA2235918},
	{SG27454, &UA2529913},
	{SG27456, &UA2826382},
	{SG27457, &UA2837957},
	{SG27459, &UA2845608},
	{SG27462, &UA2935399},
	{SG27464, &UA3042468},
	{SG28163, &UA454823},
	{SG28164, &UA608842},
	{SG28166, &UA686981},
	{SG28168, &UA797917},
	{SG28170, &UA845445},
	{SG28173, &UA1972418},
	{SG28174, &UA1981099},
	{SG28176, &UA1984436},
	{SG28178, &UA2025031},
	{SG28180, &UA2030307},
	{SG28182, &UA2039701},
	{SG28183, &UA2231970},
	{SG28185, &UA2257858},
	{SG28187, &UA2365630},
	{SG28189, &UA2463874},
	{SG28191, &UA2466696},
	{SG28193, &UA2467307},
	{SG28195, &UA2820935},
	{SG28196, &UA2846919},
	{SG28275, &UA2229370},
	{SG28277, &UA2461272},
	{SG28289, &UA456605},
	{SG28292, &UA868116},
	{SG28294, &UA893905},
	{SG28297, &UA895984},
	{SG28300, &UA1993197},
	{SG28301, &UA1998926},
	{SG28303, &UA2025231},
	{SG28305, &UA2025467},
	{SG28308, &UA2034352},
	{SG28310, &UA2390654},
	{SG28311, &UA2552175},
	{SG28314, &UA2693336},
	{SG28382, &UA2365762},
	{SG28386, &UA2424723},
	{SG28392, &UA2527537},
	{SG28478, &UA874170},
	{SG28480, &UA2032639},
	{SG28553, &UA2778199},
	{SG28554, &UA2851377},
	{SG28678, &UA2001942},
	{SG28679, &UA2235285},
	{SG28680, &UA2257624},
	{SG28681, &UA2403649},
	{SG28682, &UA2424491},
	{SG28683, &UA2455992},
	{SG28684, &UA2505162},
	{SG28685, &UA2530863},
	{SG28696, &UA2484822},
	{SG28697, &UA2487448},
	{SG28698, &UA2555793},
	{SG28700, &UA2654233},
	{SG28701, &UA2727141},
	{SG28759, &UA804125},
	{SG28760, &UA2024676},
	{SG28761, &UA2183547},
	{SG28763, &UA2204049},
	{SG28764, &UA2224970},
	{SG28765, &UA2351433},
	{SG28766, &UA2525349},
	{SG28773, &UA2400129},
	{SG28774, &UA2693238},
	{SG28775, &UA605609},
	{SG28776, &UA1971155},
	{SG28777, &UA1997100},
	{SG28778, &UA2030210},
	{SG28779, &UA2030244},
	{SG28780, &UA2031724},
	{SG28781, &UA2230037},
	{SG28782, &UA2230699},
	{SG28783, &UA2236855},
	{SG28784, &UA2423437},
	{SG28785, &UA2613139},
	{SG28787, &UA2613884},
	{SG28788, &UA2614029},
	{SG28790, &UA2614368},
	{SG28791, &UA2614642},
	{SG28793, &UA2614913},
	{SG28794, &UA2614916},
	{SG28796, &UA2615148},
	{SG28797, &UA2615278},
	{SG28798, &UA2615518},
	{SG28799, &UA2615812},
	{SG28800, &UA2615991},
	{SG28801, &UA2616116},
	{SG28802, &UA2616171},
	{SG28803, &UA2616240},
	{SG28804, &UA2616309},
	{SG28805, &UA2616993},
	{SG28806, &UA2617154},
	{SG28807, &UA2617471},
	{SG28808, &UA2617628},
	{SG28809, &UA2617663},
	{SG28810, &UA2617817},
	{SG28811, &UA2617890},
	{SG28812, &UA2618087},
	{SG28813, &UA2618135},
	{SG28814, &UA2618601},
	{SG28815, &UA2618891},
	{SG28816, &UA2618893},
	{SG28817, &UA2618908},
	{SG28818, &UA2618956},
	{SG28819, &UA2619059},
	{SG28820, &UA2619661},
	{SG28821, &UA2619798},
	{SG28822, &UA2620047},
	{SG28823, &UA2620248},
	{SG28824, &UA2620404},
	{SG28825, &UA2620473},
	{SG28826, &UA2620474},
	{SG28827, &UA2620497},
	{SG28828, &UA2620563},
	{SG28829, &UA2621669},
	{SG28830, &UA2621809},
	{SG28831, &UA2622363},
	{SG28832, &UA2622564},
	{SG28833, &UA2622909},
	{SG28834, &UA2623000},
	{SG28835, &UA2623976},
	{SG28836, &UA2624038},
	{SG28837, &UA2624242},
	{SG28838, &UA2624251},
	{SG28839, &UA2624756},
	{SG28840, &UA2625258},
	{SG28841, &UA2625294},
	{SG28842, &UA2625462},
	{SG28843, &UA2626058},
	{SG28844, &UA2626077},
	{SG28846, &UA2626945},
	{SG28847, &UA2627126},
	{SG28848, &UA2628012},
	{SG28850, &UA2628390},
	{SG28851, &UA2628449},
	{SG28853, &UA2628688},
	{SG28854, &UA2630735},
	{SG28855, &UA2630833},
	{SG28857, &UA2632046},
	{SG28859, &UA2632746},
	{SG28860, &UA2633254},
	{SG28861, &UA2633738},
	{SG28863, &UA2635351},
	{SG28865, &UA2637182},
	{SG28866, &UA2637355},
	{SG28867, &UA2000167},
	{SG28868, &UA2189292},
	{SG28869, &UA2367028},
	{SG28870, &UA2613477},
	{SG28871, &UA2616988},
	{SG28874, &UA2618093},
	{SG28875, &UA2620328},
	{SG28876, &UA2621053},
	{SG28877, &UA2621498},
	{SG28879, &UA2622000},
	{SG28880, &UA2622335},
	{SG28881, &UA2622354},
	{SG28883, &UA2623604},
	{SG28885, &UA2623944},
	{SG28890, &UA2624954},
	{SG28892, &UA2625677},
	{SG28894, &UA2627289},
	{SG28896, &UA2629288},
	{SG28900, &UA2634489},
	{SG28902, &UA2637814},
	{SG29003, &UA454813},
	{SG29005, &UA605591},
	{SG29007, &UA1994205},
	{SG29008, &UA1995465},
	{SG29010, &UA2227890},
	{SG29012, &UA2456411},
	{SG29013, &UA2498927},
	{SG29015, &UA2557659},
	{SG29017, &UA2608834},
	{SG29019, &UA2611806},
	{SG29020, &UA2612945},
	{SG29022, &UA2613189},
	{SG29024, &UA2613317},
	{SG29025, &UA2613617},
	{SG29026, &UA2614670},
	{SG29028, &UA2614926},
	{SG29031, &UA2616137},
	{SG29032, &UA2616434},
	{SG29034, &UA2616585},
	{SG29035, &UA2616756},
	{SG29036, &UA2616927},
	{SG29037, &UA2617240},
	{SG29039, &UA2617393},
	{SG29041, &UA2617411},
	{SG29043, &UA2618980},
	{SG29045, &UA2619166},
	{SG29047, &UA2619654},
	{SG29048, &UA2619887},
	{SG29049, &UA2620788},
	{SG29051, &UA2620844},
	{SG29053, &UA2622139},
	{SG29056, &UA2622484},
	{SG29058, &UA2623377},
	{SG29059, &UA2624151},
	{SG29061, &UA2624706},
	{SG29062, &UA2624803},
	{SG29065, &UA2625532},
	{SG29069, &UA2630341},
	{SG29070, &UA2631237},
	{SG29074, &UA2631760},
	{SG29076, &UA2632878},
	{SG29078, &UA2633386},
	{SG29080, &UA2633550},
	{SG29082, &UA2634864},
	{SG29084, &UA2635594},
	{SG29111, &UA2168603},
	{SG29112, &UA2237671},
	{SG29113, &UA2242279},
	{SG29114, &UA2257518},
	{SG29115, &UA2400672},
	{SG29116, &UA2420901},
	{SG29117, &UA2455808},
	{SG29118, &UA2553003},
	{SG29119, &UA2555694},
	{SG29120, &UA2939582},
	{SG29121, &UA2983091},
	{SG29122, &UA2989303},
	{SG29123, &UA3016611},
	{SG29124, &UA3020507},
	{SG29125, &UA3048861},
	{SG29126, &UA3050435},
	{SG29127, &UA3103180},
	{SG29128, &UA3107707},
	{SG29129, &UA912496},
	{SG29130, &UA1972754},
	{SG29131, &UA1979455},
	{SG29132, &UA1987253},
	{SG29133, &UA1990601},
	{SG29134, &UA1998538},
	{SG29135, &UA2002953},
	{SG29136, &UA2006926},
	{SG29137, &UA2009035},
	{SG29138, &UA2009718},
	{SG29139, &UA2025089},
	{SG29140, &UA2026610},
	{SG29141, &UA2027329},
	{SG29142, &UA2027366},
	{SG29143, &UA2027639},
	{SG29144, &UA2028342},
	{SG29145, &UA2029112},
	{SG29146, &UA2030374},
	{SG29147, &UA2030533},
	{SG29148, &UA2032294},
	{SG29149, &UA2033695},
	{SG29150, &UA2034298},
	{SG29151, &UA2177626},
	{SG29152, &UA2177850},
	{SG29153, &UA2187783},
	{SG29154, &UA2218797},
	{SG29155, &UA2220850},
	{SG29156, &UA2257439},
	{SG29157, &UA2257471},
	{SG29158, &UA2343705},
	{SG29159, &UA2345893},
	{SG29160, &UA2349982},
	{SG29161, &UA2351667},
	{SG29162, &UA2355189},
	{SG29163, &UA2355452},
	{SG29164, &UA2365205},
	{SG29165, &UA2397594},
	{SG29166, &UA2401584},
	{SG29167, &UA2413882},
	{SG29168, &UA2414057},
	{SG29169, &UA2419258},
	{SG29170, &UA2423413},
	{SG29171, &UA2475750},
	{SG29172, &UA2477202},
	{SG29173, &UA2477934},
	{SG29174, &UA2479092},
	{SG29175, &UA2481009},
	{SG29176, &UA2502532},
	{SG29177, &UA2552984},
	{SG29178, &UA2575642},
	{SG29179, &UA2641916},
	{SG29180, &UA2651624},
	{SG29181, &UA2693848},
	{SG29182, &UA2694317},
	{SG29183, &UA2727109},
	{SG29184, &UA2744894},
	{SG29185, &UA2790500},
	{SG29186, &UA2816972},
	{SG29187, &UA2821772},
	{SG29188, &UA2827253},
	{SG29189, &UA2831255},
	{SG29190, &UA2889073},
	{SG29191, &UA2950586},
	{SG29192, &UA2980646},
	{SG29193, &UA2988754},
	{SG29194, &UA3042368},
	{SG29195, &UA3085457},
	{SG29197, &UA2191109},
	{SG29198, &UA2223515},
	{SG29200, &UA2235692},
	{SG29201, &UA2239158},
	{SG29202, &UA2352366},
	{SG29203, &UA2353919},
	{SG29204, &UA2458885},
	{SG29205, &UA2504392},
	{SG29207, &UA2506636},
	{SG29208, &UA2526502},
	{SG29209, &UA2527925},
	{SG29210, &UA2551858},
	{SG29211, &UA2656727},
	{SG29212, &UA2697373},
	{SG29213, &UA2698477},
	{SG29215, &UA2700424},
	{SG29216, &UA2708471},
	{SG29231, &UA456151},
	{SG29232, &UA715055},
	{SG29233, &UA729562},
	{SG29234, &UA1967891},
	{SG29235, &UA1987432},
	{SG29236, &UA2029400},
	{SG29237, &UA2033671},
	{SG29251, &UA903319},
	{SG29252, &UA912281},
	{SG29253, &UA1956396},
	{SG29254, &UA1995450},
	{SG29255, &UA1998767},
	{SG29256, &UA2010373},
	{SG29257, &UA2028874},
	{SG29258, &UA2385904},
	{SG29259, &UA2545906},
	{SG29260, &UA2546445},
	{SG29261, &UA2654070},
	{SG29262, &UA2727143},
	{SG29263, &UA2759077},
	{SG29264, &UA2765539},
	{SG29265, &UA2993638},
	{SG29266, &UA906492},
	{SG29267, &UA1957310},
	{SG29268, &UA2167200},
	{SG29269, &UA2609433},
	{SG29270, &UA2610787},
	{SG29271, &UA2614566},
	{SG29272, &UA2614889},
	{SG29273, &UA2615695},
	{SG29274, &UA2615809},
	{SG29275, &UA2615854},
	{SG29276, &UA2616095},
	{SG29277, &UA2616476},
	{SG29278, &UA2617209},
	{SG29279, &UA2617370},
	{SG29280, &UA2618138},
	{SG29281, &UA2618194},
	{SG29282, &UA2619537},
	{SG29283, &UA2620910},
	{SG29284, &UA2621218},
	{SG29285, &UA2621440},
	{SG29286, &UA2621849},
	{SG29287, &UA2621880},
	{SG29288, &UA2623729},
	{SG29289, &UA2624807},
	{SG29290, &UA2625525},
	{SG29291, &UA2628625},
	{SG29292, &UA2629244},
	{SG29293, &UA2630446},
	{SG29294, &UA2041860},
	{SG29295, &UA2205823},
	{SG29296, &UA2242280},
	{SG29297, &UA2471649},
	{SG29298, &UA2584962},
	{SG29314, &UA2035159},
	{SG29315, &UA2035164},
	{SG29316, &UA2035173},
	{SG29317, &UA2035185},
	{SG29318, &UA2035200},
	{SG29319, &UA2035238},
	{SG29320, &UA2035214},
	{SG29321, &UA2203371},
	{SG29322, &UA2229090},
	{SG29323, &UA2425176},
	{SG29324, &UA2426063},
	{SG29325, &UA2426432},
	{SG29326, &UA2428093},
	{SG29327, &UA2428140},
	{SG29328, &UA2432100},
	{SG29329, &UA2455684},
	{SG29330, &UA2458154},
	{SG29331, &UA2459261},
	{SG29332, &UA2484755},
	{SG29333, &UA2487948},
	{SG29334, &UA2509137},
	{SG29335, &UA2528216},
	{SG29336, &UA2580244},
	{SG29337, &UA2653937},
	{SG29338, &UA2698701},
	{SG29339, &UA2728993},
	{SG29340, &UA2781778},
	{SG29346, &UA912626},
	{SG29347, &UA1991360},
	{SG29348, &UA1991439},
	{SG29349, &UA2182255},
	{SG29350, &UA2608955},
	{SG29351, &UA2610394},
	{SG29352, &UA2610518},
	{SG29353, &UA2610692},
	{SG29354, &UA2614715},
	{SG29355, &UA2615693},
	{SG29356, &UA2615729},
	{SG29357, &UA2616092},
	{SG29358, &UA2616704},
	{SG29359, &UA2617538},
	{SG29360, &UA2618831},
	{SG29361, &UA2618877},
	{SG29362, &UA2619003},
	{SG29363, &UA2620372},
	{SG29364, &UA2621388},
	{SG29365, &UA2621725},
	{SG29366, &UA2622146},
	{SG29367, &UA2622367},
	{SG29368, &UA2622406},
	{SG29369, &UA2622418},
	{SG29370, &UA2622498},
	{SG29371, &UA2623176},
	{SG29372, &UA2623977},
	{SG29373, &UA2626481},
	{SG29374, &UA2627848},
	{SG29375, &UA2629500},
	{SG29376, &UA2629959},
	{SG29377, &UA2631007},
	{SG29378, &UA2637256},
	{SG29379, &UA1979781},
	{SG29380, &UA2031596},
	{SG29381, &UA2033733},
	{SG29382, &UA2530774},
	{SG29383, &UA2556018},
	{SG29384, &UA2987254},
	{SG29582, &UA899600},
	{SG29583, &UA1969834},
	{SG29584, &UA2029180},
	{SG29585, &UA2030753},
	{SG29586, &UA2032980},
	{SG29587, &UA2383899},
	{SG29588, &UA2498474},
	{SG29589, &UA2848553},
	{SG29590, &UA3042332},
	{SG29591, &UA608689},
	{SG29592, &UA748241},
	{SG29593, &UA790907},
	{SG29594, &UA860433},
	{SG29595, &UA895986},
	{SG29596, &UA902170},
	{SG29597, &UA903178},
	{SG29598, &UA906688},
	{SG29599, &UA907471},
	{SG29600, &UA908049},
	{SG29601, &UA910083},
	{SG29602, &UA912478},
	{SG29603, &UA924225},
	{SG29604, &UA924756},
	{SG29605, &UA1954759},
	{SG29606, &UA1955079},
	{SG29607, &UA1991377},
	{SG29608, &UA2029150},
	{SG29609, &UA2030265},
	{SG29610, &UA2031462},
	{SG29611, &UA2201969},
	{SG29612, &UA2223439},
	{SG29613, &UA2372499},
	{SG29614, &UA2412681},
	{SG29615, &UA2463773},
	{SG29616, &UA2809178},
	{SG29617, &UA2829232},
	{SG29641, &UA616548},
	{SG29642, &UA616603},
	{SG29643, &UA902484},
	{SG29644, &UA2030400},
	{SG29645, &UA2031610},
	{SG29646, &UA2699165},
	{SG29647, &UA368939},
	{SG29648, &UA369680},
	{SG29649, &UA399049},
	{SG29650, &UA399045},
	{SG29651, &UA428362},
	{SG29652, &UA440681},
	{SG29653, &UA454568},
	{SG29654, &UA603301},
	{SG29655, &UA609353},
	{SG29656, &UA610057},
	{SG29657, &UA616703},
	{SG29658, &UA616741},
	{SG29659, &UA616745},
	{SG29660, &UA622482},
	{SG29661, &UA656177},
	{SG29662, &UA711841},
	{SG29663, &UA812010},
	{SG29664, &UA812379},
	{SG29665, &UA826320},
	{SG29666, &UA863535},
	{SG29667, &UA899812},
	{SG29668, &UA900098},
	{SG29670, &UA904585},
	{SG29671, &UA924360},
	{SG29672, &UA1952381},
	{SG29673, &UA1995011},
	{SG29674, &UA2017885},
	{SG29675, &UA2024098},
	{SG29676, &UA2030671},
	{SG29678, &UA2033271},
	{SG29679, &UA2227195},
	{SG29680, &UA2395327},
	{SG29681, &UA2413804},
	{SG29705, &UA2475088},
	{SG29706, &UA2815038},
	{SG29707, &UA1995051},
	{SG29708, &UA2197315},
	{SG29709, &UA2257654},
	{SG29800, &UA2354596},
	{SG29801, &UA2362932},
	{SG29802, &UA2459601},
	{SG29803, &UA2480126},
	{SG29804, &UA2480308},
	{SG29805, &UA2696694},
	{SG29806, &UA2747220},
	{SG29807, &UA2768566},
	{SG29808, &UA3056818},
	{SG29809, &UA2035223},
	{SG29810, &UA2257505},
	{SG29811, &UA2362680},
	{SG29812, &UA2368790},
	{SG29813, &UA2385431},
	{SG29814, &UA2385978},
	{SG29815, &UA2404039},
	{SG29816, &UA2427015},
	{SG29817, &UA2427649},
	{SG29818, &UA2437268},
	{SG29819, &UA2457097},
	{SG29820, &UA2459034},
	{SG29821, &UA2461027},
	{SG29822, &UA2485046},
	{SG29823, &UA2507301},
	{SG29824, &UA2510658},
	{SG29825, &UA2531148},
	{SG29826, &UA2534867},
	{SG29827, &UA2553289},
	{SG29828, &UA2579549},
	{SG29829, &UA2670296},
	{SG29830, &UA2732012},
	{SG29831, &UA2791266},
	{SG29832, &UA2795539},
	{SG29833, &UA2844894},
	{SG29874, &UA2194218},
	{SG29875, &UA2200111},
	{SG29876, &UA2356303},
	{SG29877, &UA2383234},
	{SG29878, &UA2527617},
	{SG29879, &UA2653464},
	{SG29880, &UA2708142},
	{SG29881, &UA3007221},
	{SG29882, &UA2027879},
	{SG29883, &UA2387606},
	{SG29884, &UA2404105},
	{SG29885, &UA2416234},
	{SG29886, &UA2423736},
	{SG29887, &UA2425890},
	{SG29888, &UA2427123},
	{SG29889, &UA2457709},
	{SG29890, &UA2509698},
	{SG29891, &UA2527326},
	{SG29892, &UA2528035},
	{SG29893, &UA2529533},
	{SG29894, &UA2537178},
	{SG29895, &UA2595980},
	{SG29896, &UA2850122},
	{SG29897, &UA441485},
	{SG29898, &UA604799},
	{SG29899, &UA605724},
	{SG29900, &UA615683},
	{SG29901, &UA901035},
	{SG29902, &UA908903},
	{SG29903, &UA1986254},
	{SG29904, &UA2685377},
	{SG29905, &UA2034653},
	{SG29906, &UA2226870},
	{SG29907, &UA2257438},
	{SG29908, &UA2257565},
	{SG29909, &UA2257604},
	{SG29910, &UA2352651},
	{SG29911, &UA2362181},
	{SG29912, &UA2405105},
	{SG29913, &UA2421852},
	{SG29914, &UA2427692},
	{SG29915, &UA2428143},
	{SG29916, &UA2430826},
	{SG29917, &UA2531900},
	{SG29918, &UA2532813},
	{SG29919, &UA2557517},
	{SG29920, &UA2560593},
	{SG29921, &UA2560601},
	{SG29922, &UA2563475},
	{SG29923, &UA2570341},
	{SG29924, &UA2583136},
	{SG29925, &UA2663350},
	{SG29926, &UA2698240},
	{SG29927, &UA2733926},
	{SG29928, &UA2864531},
	{SG29929, &UA3073138},
	{SG29932, &UA2352371},
	{SG29933, &UA2367593},
	{SG29936, &UA2468258},
	{SG29939, &UA2468287},
	{SG29941, &UA2695095},
	{SG29943, &UA2790403},
	{SG29946, &UA2224201},
	{SG29947, &UA2444992},
	{SG29948, &UA2480919},
	{SG29949, &UA2525618},
	{SG29950, &UA616601},
	{SG29951, &UA900309},
	{SG29952, &UA2034209},
	{SG29953, &UA2257481},
	{SG29954, &UA2344565},
	{SG29955, &UA2454933},
	{SG29956, &UA2552594},
	{SG29957, &UA2555167},
	{SG29958, &UA2693182},
	{SG29961, &UA900441},
	{SG29962, &UA2027619},
	{SG29963, &UA2554079},
	{SG29964, &UA2655139},
	{SG29965, &UA3015874},
	{SG29967, &UA368834},
	{SG29968, &UA369276},
	{SG29969, &UA369481},
	{SG29970, &UA401270},
	{SG29971, &UA403058},
	{SG29972, &UA403082},
	{SG29973, &UA403129},
	{SG29974, &UA403165},
	{SG29975, &UA403175},
	{SG29976, &UA403292},
	{SG29977, &UA616642},
	{SG29978, &UA710583},
	{SG29979, &UA760456},
	{SG29980, &UA830013},
	{SG29981, &UA901670},
	{SG29982, &UA2023611},
	{SG29983, &UA2025777},
	{SG29984, &UA2257601},
	{SG29985, &UA2653283},
	{SG29986, &UA672020},
	{SG29987, &UA908826},
	{SG29988, &UA908908},
	{SG29989, &UA909240},
	{SG29990, &UA909429},
	{SG29991, &UA911679},
	{SG29992, &UA912480},
	{SG29993, &UA923868},
	{SG29994, &UA1972845},
	{SG29995, &UA1979285},
	{SG29996, &UA1993555},
	{SG29997, &UA1996935},
	{SG29998, &UA2029590},
	{SG29999, &UA2031032},
	{SG30000, &UA2032022},
	{SG30001, &UA2230427},
	{SG30002, &UA2232261},
	{SG30003, &UA2234000},
	{SG30004, &UA2236425},
	{SG30005, &UA2237051},
	{SG30006, &UA2257639},
	{SG30007, &UA2257666},
	{SG30008, &UA2339720},
	{SG30009, &UA2349276},
	{SG30010, &UA2386287},
	{SG30011, &UA2403695},
	{SG30012, &UA2478881},
	{SG30013, &UA2502754},
	{SG30014, &UA2553432},
	{SG30015, &UA2562377},
	{SG30016, &UA2563813},
	{SG30017, &UA2577852},
	{SG30018, &UA2602462},
	{SG30019, &UA2703144},
	{SG30020, &UA1996415},
	{SG30021, &UA2029189},
	{SG30022, &UA2481680},
	{SG30023, &UA2661822},
	{SG30024, &UA3044540},
	{SG30025, &UA3046814},
	{SG30026, &UA3092025},
	{SG30027, &UA609311},
	{SG30028, &UA900683},
	{SG30029, &UA2005034},
	{SG30030, &UA2027092},
	{SG30031, &UA2030129},
	{SG30032, &UA2261465},
	{SG30057, &UA874092},
	{SG30059, &UA879053},
	{SG30063, &UA889075},
	{SG30066, &UA896574},
	{SG30069, &UA2031427},
	{SG30071, &UA2033980},
	{SG30073, &UA3046052},
	{SG30100, &UA2028638},
	{SG30101, &UA2223008},
	{SG30102, &UA2237418},
	{SG30103, &UA2599139},
	{SG30128, &UA2178393},
	{SG30129, &UA2228732},
	{SG30130, &UA2257836},
	{SG30131, &UA2478800},
	{SG30132, &UA2500220},
	{SG30133, &UA2505482},
	{SG30134, &UA2389495},
	{SG30135, &UA2404391},
	{SG30136, &UA2423191},
	{SG30137, &UA2499324},
	{SG30138, &UA2505155},
	{SG30139, &UA2554796},
	{SG30140, &UA2558159},
	{SG30141, &UA2578906},
	{SG30142, &UA2583039},
	{SG30143, &UA2600118},
	{SG30144, &UA2657740},
	{SG30145, &UA2697202},
	{SG30146, &UA2727862},
	{SG30147, &UA2728826},
	{SG30148, &UA2766479},
	{SG30149, &UA2779943},
	{SG30150, &UA2983112},
	{SG30151, &UA3070625},
	{SG30165, &UA911121},
	{SG30166, &UA1994697},
	{SG30168, &UA1996959},
	{SG30169, &UA1997082},
	{SG30175, &UA3046787},
	{SG30178, &UA892561},
	{SG30179, &UA896727},
	{SG30180, &UA898903},
	{SG30181, &UA924201},
	{SG30182, &UA2033253},
	{SG30185, &UA616688},
	{SG30186, &UA694203},
	{SG30187, &UA2025128},
	{SG30188, &UA2031636},
	{SG30191, &UA2499333},
	{SG30200, &UA2002920},
	{SG30201, &UA2027957},
	{SG30202, &UA2030282},
	{SG30203, &UA2030994},
	{SG30204, &UA2230761},
	{SG30205, &UA2449787},
	{SG30210, &UA1995035},
	{SG30212, &UA2025343},
	{SG30215, &UA2027424},
	{SG30217, &UA2028264},
	{SG30219, &UA2028681},
	{SG30221, &UA2028800},
	{SG30222, &UA2029241},
	{SG30223, &UA2033616},
	{SG30224, &UA2222033},
	{SG30225, &UA2228403},
	{SG30226, &UA2229172},
	{SG30227, &UA2261506},
	{SG30228, &UA2261531},
	{SG30229, &UA2261582},
	{SG30230, &UA2261583},
	{SG30231, &UA2326439},
	{SG30232, &UA2338663},
	{SG30233, &UA2340117},
	{SG30234, &UA2344004},
	{SG30235, &UA2344183},
	{SG30236, &UA2344271},
	{SG30237, &UA2345539},
	{SG30238, &UA2414635},
	{SG30239, &UA2415553},
	{SG30240, &UA2416985},
	{SG30241, &UA2417710},
	{SG30242, &UA2419233},
	{SG30243, &UA2419456},
	{SG30244, &UA2421345},
	{SG30245, &UA2422355},
	{SG30246, &UA2424477},
	{SG30247, &UA2425046},
	{SG30248, &UA2426093},
	{SG30249, &UA2427501},
	{SG30250, &UA2428123},
	{SG30251, &UA2428730},
	{SG30252, &UA2445769},
	{SG30253, &UA2452438},
	{SG30254, &UA2452850},
	{SG30255, &UA2454602},
	{SG30256, &UA2455000},
	{SG30257, &UA2455310},
	{SG30258, &UA2455577},
	{SG30259, &UA2455707},
	{SG30260, &UA2456584},
	{SG30261, &UA2457698},
	{SG30262, &UA2457929},
	{SG30263, &UA2458050},
	{SG30264, &UA2458705},
	{SG30265, &UA2465176},
	{SG30266, &UA2465535},
	{SG30267, &UA2465743},
	{SG30268, &UA2466490},
	{SG30269, &UA2466979},
	{SG30270, &UA2467349},
	{SG30271, &UA2467398},
	{SG30272, &UA2467493},
	{SG30273, &UA2467835},
	{SG30274, &UA2468633},
	{SG30275, &UA2476817},
	{SG30276, &UA2477817},
	{SG30277, &UA2478307},
	{SG30278, &UA2478430},
	{SG30279, &UA2480028},
	{SG30280, &UA2480281},
	{SG30281, &UA2480636},
	{SG30282, &UA2481829},
	{SG30283, &UA2481896},
	{SG30284, &UA2482058},
	{SG30285, &UA2482309},
	{SG30286, &UA2482507},
	{SG30287, &UA2483477},
	{SG30288, &UA2484432},
	{SG30289, &UA2485667},
	{SG30290, &UA2486980},
	{SG30291, &UA2499767},
	{SG30292, &UA2500799},
	{SG30293, &UA2501508},
	{SG30294, &UA2501586},
	{SG30295, &UA2502284},
	{SG30296, &UA2502525},
	{SG30297, &UA2502572},
	{SG30298, &UA2502603},
	{SG30299, &UA2502583},
	{SG30300, &UA2502791},
	{SG30301, &UA2502904},
	{SG30302, &UA2503143},
	{SG30303, &UA2503345},
	{SG30304, &UA2503754},
	{SG30305, &UA2504100},
	{SG30307, &UA2504303},
	{SG30308, &UA2504310},
	{SG30311, &UA2504408},
	{SG30312, &UA2504506},
	{SG30313, &UA2504662},
	{SG30314, &UA2504734},
	{SG30315, &UA2504967},
	{SG30316, &UA2504987},
	{SG30317, &UA2505056},
	{SG30318, &UA2505233},
	{SG30320, &UA2505355},
	{SG30321, &UA2505676},
	{SG30323, &UA2506030},
	{SG30324, &UA2506648},
	{SG30326, &UA2520629},
	{SG30327, &UA2521951},
	{SG30329, &UA2524795},
	{SG30331, &UA2526546},
	{SG30334, &UA2527591},
	{SG30336, &UA2527644},
	{SG30338, &UA2527815},
	{SG30340, &UA2527825},
	{SG30342, &UA2527887},
	{SG30343, &UA2527907},
	{SG30344, &UA2528562},
	{SG30345, &UA2529261},
	{SG30346, &UA2529572},
	{SG30347, &UA2529754},
	{SG30348, &UA2529918},
	{SG30349, &UA2530670},
	{SG30350, &UA2535017},
	{SG30351, &UA2549500},
	{SG30352, &UA2551560},
	{SG30353, &UA2553317},
	{SG30354, &UA2554220},
	{SG30355, &UA2554410},
	{SG30356, &UA2556376},
	{SG30357, &UA2556577},
	{SG30358, &UA2557012},
	{SG30359, &UA2557526},
	{SG30360, &UA2558162},
	{SG30361, &UA2558179},
	{SG30362, &UA2558404},
	{SG30365, &UA2574695},
	{SG30367, &UA2578389},
	{SG30368, &UA2578420},
	{SG30370, &UA2580039},
	{SG30372, &UA2592709},
	{SG30374, &UA2593496},
	{SG30377, &UA2594639},
	{SG30379, &UA2595111},
	{SG30381, &UA2595547},
	{SG30383, &UA2596083},
	{SG30384, &UA2596175},
	{SG30386, &UA2596632},
	{SG30388, &UA2596729},
	{SG30390, &UA2596920},
	{SG30392, &UA2597859},
	{SG30395, &UA2647560},
	{SG30396, &UA2648411},
	{SG30398, &UA2651944},
	{SG30399, &UA2652133},
	{SG30400, &UA2655791},
	{SG30401, &UA2655882},
	{SG30402, &UA2656407},
	{SG30403, &UA2656820},
	{SG30404, &UA2658441},
	{SG30405, &UA2659387},
	{SG30406, &UA2662350},
	{SG30407, &UA2664875},
	{SG30408, &UA2668141},
	{SG30409, &UA2691569},
	{SG30410, &UA2693480},
	{SG30411, &UA2696065},
	{SG30412, &UA2696463},
	{SG30413, &UA2698096},
	{SG30414, &UA2698442},
	{SG30415, &UA2701618},
	{SG30416, &UA2705142},
	{SG30417, &UA2705633},
	{SG30418, &UA2727267},
	{SG30419, &UA2727781},
	{SG30420, &UA2728877},
	{SG30421, &UA2729131},
	{SG30422, &UA2729370},
	{SG30423, &UA2729874},
	{SG30424, &UA2729965},
	{SG30425, &UA2730157},
	{SG30426, &UA2733933},
	{SG30427, &UA2746065},
	{SG30428, &UA2748031},
	{SG30429, &UA2749517},
	{SG30430, &UA2761851},
	{SG30431, &UA2762656},
	{SG30432, &UA2762812},
	{SG30433, &UA2762923},
	{SG30434, &UA2763031},
	{SG30435, &UA2763339},
	{SG30436, &UA2765038},
	{SG30437, &UA2765152},
	{SG30438, &UA2775653},
	{SG30439, &UA2778648},
	{SG30440, &UA2779201},
	{SG30441, &UA2794702},
	{SG30442, &UA2796530},
	{SG30443, &UA2816943},
	{SG30444, &UA2822148},
	{SG30445, &UA2823857},
	{SG30446, &UA2824380},
	{SG30447, &UA2830420},
	{SG30448, &UA2830689},
	{SG30449, &UA2831056},
	{SG30451, &UA2832208},
	{SG30454, &UA2833901},
	{SG30456, &UA2834366},
	{SG30458, &UA2836376},
	{SG30461, &UA2837138},
	{SG30462, &UA2838541},
	{SG30465, &UA2839600},
	{SG30466, &UA2839673},
	{SG30469, &UA2843089},
	{SG30471, &UA2844533},
	{SG30473, &UA2844606},
	{SG30474, &UA2845271},
	{SG30477, &UA2845468},
	{SG30479, &UA2845717},
	{SG30481, &UA2848484},
	{SG30482, &UA2848688},
	{SG30484, &UA2849012},
	{SG30485, &UA2849443},
	{SG30486, &UA2855384},
	{SG30487, &UA2892329},
	{SG30488, &UA2892755},
	{SG30489, &UA2895730},
	{SG30490, &UA2936216},
	{SG30491, &UA2936853},
	{SG30492, &UA2951862},
	{SG30493, &UA2951911},
	{SG30494, &UA2956569},
	{SG30495, &UA2956837},
	{SG30496, &UA2957759},
	{SG30497, &UA2959599},
	{SG30498, &UA2982178},
	{SG30499, &UA2982848},
	{SG30500, &UA2982999},
	{SG30503, &UA2983764},
	{SG30505, &UA2984654},
	{SG30507, &UA2984699},
	{SG30536, &UA2985400},
	{SG30540, &UA2991895},
	{SG30543, &UA2998248},
	{SG30545, &UA2999303},
	{SG30546, &UA2999310},
	{SG30549, &UA2999779},
	{SG30553, &UA3006436},
	{SG30556, &UA3007773},
	{SG30559, &UA3009397},
	{SG30561, &UA3009966},
	{SG30565, &UA3016270},
	{SG30568, &UA3019379},
	{SG30571, &UA3043304},
	{SG30573, &UA3043844},
	{SG30576, &UA3044649},
	{SG30579, &UA3045036},
	{SG30582, &UA3049758},
	{SG30585, &UA3070512},
	{SG30588, &UA3071778},
	{SG30591, &UA3073824},
	{SG30593, &UA3077675},
	{SG30595, &UA3086196},
	{SG30598, &UA3088874},
	{SG30601, &UA3099952},
	{SG30603, &UA3100181},
	{SG30605, &UA3108300},
	{SG30608, &UA3118812},
	{SG30610, &UA3125341},
	{SG30612, &UA3126573},
	{SG30652, &UA2030619},
	{SG30708, &UA1974511},
	{SG30709, &UA2229039},
	{SG30710, &UA2231515},
	{SG30711, &UA2257781},
	{SG30712, &UA2727200},
	{SG30713, &UA2758343},
	{SG30722, &UA454563},
	{SG30723, &UA758729},
	{SG30724, &UA2031283},
	{SG30725, &UA2393565},
	{SG30726, &UA2413402},
	{SG30727, &UA2416754},
	{SG30728, &UA2433702},
	{SG30729, &UA2034046},
	{SG30730, &UA2355343},
	{SG30731, &UA2738338},
	{SG30732, &UA2868573},
	{SG30733, &UA2984007},
	{SG30743, &UA2034663},
	{SG30744, &UA2034664},
	{SG30745, &UA2034671},
	{SG30746, &UA2034719},
	{SG30747, &UA2034744},
	{SG30748, &UA2034756},
	{SG30749, &UA2034771},
	{SG30750, &UA2034821},
	{SG30751, &UA2034833},
	{SG30752, &UA2034855},
	{SG30753, &UA2034942},
	{SG30754, &UA2034952},
	{SG30755, &UA2034963},
	{SG30756, &UA2035034},
	{SG30757, &UA2035106},
	{SG30758, &UA2204862},
	{SG30759, &UA2239094},
	{SG30760, &UA2356210},
	{SG30761, &UA2362789},
	{SG30762, &UA2387499},
	{SG30763, &UA2388282},
	{SG30764, &UA2425595},
	{SG30765, &UA2430084},
	{SG30766, &UA2461202},
	{SG30767, &UA2652229},
	{SG30768, &UA2665464},
	{SG30794, &UA1959492},
	{SG30795, &UA1995900},
	{SG30796, &UA2193363},
	{SG30797, &UA2193478},
	{SG30798, &UA2236521},
	{SG30799, &UA2239269},
	{SG30800, &UA2365119},
	{SG30801, &UA2384782},
	{SG30802, &UA2418429},
	{SG30803, &UA2431109},
	{SG30804, &UA2452645},
	{SG30805, &UA2552569},
	{SG30806, &UA2653073},
	{SG30807, &UA2727425},
	{SG30808, &UA3117821},
	{SG30809, &UA3117935},
	{SG30810, &UA656477},
	{SG30811, &UA749044},
	{SG30812, &UA804195},
	{SG30813, &UA907965},
	{SG30814, &UA1979215},
	{SG30815, &UA1994370},
	{SG30816, &UA2031054},
	{SG30817, &UA2031238},
	{SG30818, &UA2355790},
	{SG30845, &UA885419},
	{SG30846, &UA900391},
	{SG30847, &UA901532},
	{SG30848, &UA903838},
	{SG30849, &UA911274},
	{SG30850, &UA911353},
	{SG30851, &UA911555},
	{SG30852, &UA2023573},
	{SG30853, &UA2375154},
	{SG30854, &UA2385056},
	{SG30855, &UA2394260},
	{SG30856, &UA2466540},
	{SG30857, &UA2948084},
	{SG30858, &UA3044939},
	{SG30859, &UA2342037},
	{SG30860, &UA2981916},
	{SG30898, &UA2029013},
	{SG30899, &UA2179512},
	{SG30900, &UA2368008},
	{SG30901, &UA2434303},
	{SG30902, &UA2506083},
	{SG30903, &UA2991930},
	{SG30904, &UA2027864},
	{SG30905, &UA2042734},
	{SG30906, &UA2257529},
	{SG30907, &UA2257536},
	{SG30908, &UA2496280},
	{SG30909, &UA2496772},
	{SG30910, &UA2506827},
	{SG30911, &UA2525797},
	{SG30913, &UA2704091},
	{SG30914, &UA2733418},
	{SG30915, &UA2856789},
	{SG30923, &UA907631},
	{SG30924, &UA907678},
	{SG30925, &UA909522},
	{SG30926, &UA913187},
	{SG30927, &UA1961797},
	{SG30928, &UA1995467},
	{SG30929, &UA2030751},
	{SG30930, &UA2367557},
	{SG30931, &UA2415342},
	{SG30932, &UA1993987},
	{SG30933, &UA1998766},
	{SG30934, &UA2449408},
	{SG30935, &UA2692272},
	{SG30936, &UA2833725},
	{SG30941, &UA899770},
	{SG30942, &UA1961295},
	{SG30943, &UA2029008},
	{SG30944, &UA2230312},
	{SG30945, &UA2233992},
	{SG30994, &UA910530},
	{SG30995, &UA1996617},
	{SG30996, &UA2028045},
	{SG30997, &UA2031353},
	{SG30998, &UA2204018},
	{SG30999, &UA2364714},
	{SG31000, &UA2463499},
	{SG31001, &UA2499730},
	{SG31002, &UA2401047},
	{SG31003, &UA2414444},
	{SG31004, &UA2839469},
	{SG31005, &UA2684877},
	{SG31006, &UA2729438},
	{SG31015, &UA1982211},
	{SG31017, &UA2242221},
	{SG31019, &UA2348179},
	{SG31021, &UA2694131},
	{SG31044, &UA900259},
	{SG31045, &UA2349639},
	{SG31046, &UA2375516},
	{SG31047, &UA2400676},
	{SG31048, &UA2935064},
	{SG31049, &UA2025241},
	{SG31050, &UA2166626},
	{SG31051, &UA2204799},
	{SG31052, &UA2237741},
	{SG31053, &UA2420237},
	{SG31054, &UA2437374},
	{SG31055, &UA2026619},
	{SG31056, &UA2030232},
	{SG31057, &UA2031409},
	{SG31058, &UA2033916},
	{SG31068, &UA454869},
	{SG31069, &UA622434},
	{SG31070, &UA906299},
	{SG31071, &UA2024214},
	{SG31072, &UA2034214},
	{SG31073, &UA2729676},
	{SG31074, &UA2750915},
	{SG31075, &UA2856392},
	{SG31076, &UA604839},
	{SG31077, &UA1951043},
	{SG31078, &UA1968864},
	{SG31079, &UA2024930},
	{SG31080, &UA2025071},
	{SG31081, &UA2028827},
	{SG31082, &UA2031274},
	{SG31083, &UA2363073},
	{SG31086, &UA2476424},
	{SG31088, &UA2835748},
	{SG31090, &UA909452},
	{SG31091, &UA1970028},
	{SG31092, &UA1970159},
	{SG31093, &UA1993185},
	{SG31094, &UA1997264},
	{SG31095, &UA2002769},
	{SG31096, &UA2017588},
	{SG31097, &UA2026149},
	{SG31098, &UA2026268},
	{SG31099, &UA2026826},
	{SG31100, &UA2027379},
	{SG31101, &UA2027763},
	{SG31102, &UA2189397},
	{SG31103, &UA2395658},
	{SG31104, &UA2553085},
	{SG31105, &UA2229288},
	{SG31107, &UA2235421},
	{SG31110, &UA2426007},
	{SG31112, &UA2457522},
	{SG31113, &UA2461253},
	{SG31114, &UA2469272},
	{SG31115, &UA2507967},
	{SG31116, &UA2597911},
	{SG31117, &UA2652394},
	{SG31118, &UA2686148},
	{SG31119, &UA2746976},
	{SG31140, &UA616723},
	{SG31142, &UA1992886},
	{SG31144, &UA2166767},
	{SG31145, &UA2381548},
	{SG31146, &UA2592751},
	{SG31149, &UA2617021},
	{SG31151, &UA2617065},
	{SG31152, &UA2617598},
	{SG31154, &UA2619824},
	{SG31156, &UA2622573},
	{SG31157, &UA2624338},
	{SG31159, &UA2624414},
	{SG31160, &UA2625130},
	{SG31162, &UA2626623},
	{SG31163, &UA2628236},
	{SG31165, &UA2629982},
	{SG31167, &UA2635662},
	{SG31187, &UA1989619},
	{SG31188, &UA2030069},
	{SG31189, &UA2347450},
	{SG31190, &UA908646},
	{SG31191, &UA2034571},
	{SG31192, &UA2383596},
	{SG31193, &UA2608329},
	{SG31194, &UA2612386},
	{SG31195, &UA2614578},
	{SG31196, &UA2615290},
	{SG31197, &UA2615892},
	{SG31198, &UA2616350},
	{SG31199, &UA2616382},
	{SG31200, &UA2616579},
	{SG31201, &UA2616996},
	{SG31202, &UA2617778},
	{SG31203, &UA2618188},
	{SG31204, &UA2618650},
	{SG31205, &UA2618651},
	{SG31206, &UA2619124},
	{SG31207, &UA2619151},
	{SG31208, &UA2619567},
	{SG31209, &UA2619816},
	{SG31210, &UA2621350},
	{SG31211, &UA2621573},
	{SG31212, &UA2622893},
	{SG31213, &UA2624192},
	{SG31214, &UA2624289},
	{SG31215, &UA2626246},
	{SG31216, &UA2628138},
	{SG31217, &UA2629036},
	{SG31218, &UA2635917},
	{SG31219, &UA2636069},
	{SG31220, &UA2257685},
	{SG31221, &UA2516541},
	{SG31222, &UA2536621},
	{SG31223, &UA2659422},
	{SG31224, &UA2991524},
	{SG31225, &UA1990582},
	{SG31226, &UA2234083},
	{SG31227, &UA2242255},
	{SG31228, &UA2693213},
	{SG31229, &UA2818947},
	{SG31230, &UA2201600},
	{SG31231, &UA2368189},
	{SG31232, &UA2424670},
	{SG31253, &UA2566517},
	{SG31255, &UA2566899},
	{SG31257, &UA2567117},
	{SG31259, &UA2567564},
	{SG31268, &UA2417619},
	{SG31269, &UA2504054},
	{SG31271, &UA3071614},
	{SG31273, &UA3091946},
	{SG31275, &UA604931},
	{SG31277, &UA615650},
	{SG31279, &UA895628},
	{SG31281, &UA904969},
	{SG31283, &UA906057},
	{SG31285, &UA2027500},
	{SG31286, &UA2028418},
	{SG31288, &UA2030984},
	{SG31290, &UA2031599},
	{SG31296, &UA804685},
	{SG31297, &UA901894},
	{SG31298, &UA902710},
	{SG31299, &UA1979614},
	{SG31300, &UA2001438},
	{SG31301, &UA2477838},
	{SG31302, &UA2666410},
	{SG31303, &UA2698685},
	{SG31304, &UA2700207},
	{SG31305, &UA2712703},
	{SG31306, &UA2952337},
	{SG31307, &UA2391214},
	{SG31308, &UA2391247},
	{SG31309, &UA2505151},
	{SG31310, &UA3044165},
	{SG31311, &UA609455},
	{SG31312, &UA902820},
	{SG31313, &UA1972136},
	{SG31314, &UA1996095},
	{SG31315, &UA1998768},
	{SG31316, &UA2031420},
	{SG31317, &UA2204151},
	{SG31318, &UA2205461},
	{SG31319, &UA2230363},
	{SG31320, &UA2234376},
	{SG31321, &UA2382825},
	{SG31322, &UA2451887},
	{SG31332, &UA609302},
	{SG31333, &UA634523},
	{SG31334, &UA902236},
	{SG31335, &UA908274},
	{SG31336, &UA909961},
	{SG31337, &UA1954505},
	{SG31338, &UA2227018},
	{SG31339, &UA2227158},
	{SG31340, &UA2261463},
	{SG31341, &UA2261529},
	{SG31342, &UA2379730},
	{SG31343, &UA2382339},
	{SG31344, &UA2383398},
	{SG31345, &UA2390313},
	{SG31346, &UA2399621},
	{SG31347, &UA2400751},
	{SG31348, &UA2401582},
	{SG31349, &UA2401681},
	{SG31350, &UA2404346},
	{SG31351, &UA2412772},
	{SG31352, &UA2415873},
	{SG31353, &UA2423801},
	{SG31354, &UA2425367},
	{SG31355, &UA2425311},
	{SG31356, &UA2452277},
	{SG31357, &UA2457818},
	{SG31358, &UA2469254},
	{SG31359, &UA2481828},
	{SG31360, &UA2505111},
	{SG31361, &UA2506708},
	{SG31362, &UA2526217},
	{SG31363, &UA2746640},
	{SG31364, &UA2747812},
	{SG31365, &UA2837553},
	{SG31366, &UA2197919},
	{SG31367, &UA2205922},
	{SG31368, &UA2237428},
	{SG31369, &UA2257936},
	{SG31370, &UA2355145},
	{SG31371, &UA2366514},
	{SG31372, &UA2390975},
	{SG31373, &UA2404129},
	{SG31374, &UA2422429},
	{SG31375, &UA2425817},
	{SG31376, &UA2428248},
	{SG31377, &UA2456009},
	{SG31378, &UA2483774},
	{SG31379, &UA2526286},
	{SG31380, &UA2531358},
	{SG31381, &UA2764309},
	{SG31406, &UA1960331},
	{SG31407, &UA2362786},
	{SG31408, &UA2158799},
	{SG31409, &UA2382773},
	{SG31410, &UA907798},
	{SG31411, &UA1996121},
	{SG31412, &UA2549785},
	{SG31413, &UA2658630},
	{SG31428, &UA2166077},
	{SG31429, &UA2366980},
	{SG31430, &UA2400330},
	{SG31431, &UA2417891},
	{SG31432, &UA2455691},
	{SG31433, &UA2551489},
	{SG31434, &UA2652989},
	{SG31435, &UA2833159},
	{SG31436, &UA3085827},
	{SG31437, &UA2367973},
	{SG31438, &UA2432689},
	{SG31439, &UA1992588},
	{SG31440, &UA2193494},
	{SG31441, &UA2375481},
	{SG31442, &UA2551777},
	{SG31443, &UA2744530},
	{SG31444, &UA2168188},
	{SG31445, &UA2383822},
	{SG31446, &UA2014758},
	{SG31447, &UA2203346},
	{SG31448, &UA2257412},
	{SG31449, &UA2368166},
	{SG31450, &UA2469277},
	{SG31451, &UA2507971},
	{SG31452, &UA2035281},
	{SG31453, &UA2035283},
	{SG31454, &UA2354334},
	{SG31455, &UA2416119},
	{SG31456, &UA2455558},
	{SG31457, &UA2530626},
	{SG31469, &UA2173875},
	{SG31470, &UA2420898},
	{SG31471, &UA2554316},
	{SG31472, &UA2992782},
	{SG31473, &UA2362337},
	{SG31474, &UA2596258},
	{SG31475, &UA2777277},
	{SG31476, &UA1971834},
	{SG31480, &UA2427458},
	{SG31481, &UA2506505},
	{SG31482, &UA2261507},
	{SG31483, &UA2261534},
	{SG31484, &UA2379958},
	{SG31497, &UA2257596},
	{SG31499, &UA2423630},
	{SG31501, &UA2467602},
	{SG31518, &UA2386104},
	{SG31519, &UA2430926},
	{SG31520, &UA2656277},
	{SG31521, &UA2656909},
	{SG31543, &UA603259},
	{SG31544, &UA2026277},
	{SG31545, &UA2029022},
	{SG31546, &UA2030800},
	{SG31547, &UA2031001},
	{SG31548, &UA2032335},
	{SG31549, &UA2032698},
	{SG31550, &UA2033114},
	{SG31551, &UA2403482},
	{SG31552, &UA2424511},
	{SG31553, &UA2457315},
	{SG31554, &UA2653088},
	{SG31555, &UA2696048},
	{SG31556, &UA2699779},
	{SG31557, &UA2834511},
	{SG31558, &UA2992651},
	{SG31559, &UA3086613},
	{SG31560, &UA2257638},
	{SG31561, &UA2424188},
	{SG31562, &UA2242177},
	{SG31563, &UA2403696},
	{SG31564, &UA2505158},
	{SG31565, &UA2983281},
	{SG31566, &UA3027865},
	{SG31567, &UA2546914},
	{SG31568, &UA2558773},
	{SG31569, &UA2705891},
	{SG31620, &UA924236},
	{SG31621, &UA1961508},
	{SG31622, &UA2027918},
	{SG31623, &UA2032868},
	{SG31624, &UA2463852},
	{SG31627, &UA2478642},
	{SG31629, &UA2830174},
	{SG31630, &UA3043603},
	{SG31632, &UA622454},
	{SG31633, &UA1991986},
	{SG31634, &UA1993227},
	{SG31635, &UA2017136},
	{SG31636, &UA2225536},
	{SG31637, &UA2387779},
	{SG31641, &UA2030836},
	{SG31642, &UA2360405},
	{SG31643, &UA2397980},
	{SG31644, &UA2404228},
	{SG31645, &UA2479307},
	{SG31646, &UA2500199},
	{SG31647, &UA2190347},
	{SG31648, &UA2351376},
	{SG31649, &UA2479969},
	{SG31650, &UA2257808},
	{SG31651, &UA2424426},
	{SG31652, &UA2257394},
	{SG31654, &UA2257418},
	{SG31656, &UA2338503},
	{SG31658, &UA2425911},
	{SG31659, &UA2426783},
	{SG31664, &UA904998},
	{SG31665, &UA924207},
	{SG31666, &UA2033880},
	{SG31667, &UA2257488},
	{SG31668, &UA2445838},
	{SG31669, &UA2520901},
	{SG31670, &UA2670247},
	{SG31671, &UA2727297},
	{SG31672, &UA3099167},
	{SG31673, &UA1957695},
	{SG31674, &UA2242227},
	{SG31675, &UA901909},
	{SG31676, &UA2198145},
	{SG31677, &UA2391449},
	{SG31690, &UA1954458},
	{SG31692, &UA2402552},
	{SG31694, &UA2610554},
	{SG31696, &UA1996982},
	{SG31698, &UA2029587},
	{SG31699, &UA2353723},
	{SG31701, &UA2423983},
	{SG31702, &UA2463953},
	{SG31704, &UA2501679},
	{SG31722, &UA2257686},
	{SG31723, &UA2712930},
	{SG31724, &UA2739715},
	{SG31725, &UA1964504},
	{SG31726, &UA2041775},
	{SG31727, &UA2197816},
	{SG31728, &UA2352887},
	{SG31729, &UA2712913},
	{SG31730, &UA2740175},
	{SG31731, &UA2001674},
	{SG31732, &UA2822109},
	{SG31736, &UA888195},
	{SG31750, &UA622465},
	{SG31751, &UA656587},
	{SG31752, &UA671142},
	{SG31753, &UA684302},
	{SG31754, &UA777389},
	{SG31755, &UA901802},
	{SG31756, &UA1963720},
	{SG31757, &UA2024840},
	{SG31758, &UA2027941},
	{SG31759, &UA2029305},
	{SG31760, &UA2032053},
	{SG31761, &UA2032171},
	{SG31783, &UA609338},
	{SG31784, &UA902829},
	{SG31785, &UA911286},
	{SG31786, &UA2033201},
	{SG31787, &UA2041812},
	{SG31788, &UA2206140},
	{SG31791, &UA2034052},
	{SG31792, &UA2201953},
	{SG31793, &UA2454731},
	{SG31796, &UA2425610},
	{SG31797, &UA2468516},
	{SG31798, &UA2469885},
	{SG31799, &UA2482989},
	{SG31800, &UA2505203},
	{SG31801, &UA2530327},
	{SG31802, &UA2535955},
	{SG31803, &UA2537468},
	{SG31804, &UA2565496},
	{SG31805, &UA2582284},
	{SG31806, &UA2603703},
	{SG31807, &UA2667190},
	{SG31808, &UA2698573},
	{SG31809, &UA2709148},
	{SG31810, &UA2767167},
	{SG31811, &UA2257642},
	{SG31812, &UA2555424},
	{SG31813, &UA2643098},
	{SG31814, &UA2554247},
	{SG31815, &UA2586647},
	{SG31816, &UA2586650},
	{SG31817, &UA2566745},
	{SG31818, &UA2566744},
	{SG31819, &UA2566750},
	{SG31820, &UA2605937},
	{SG31830, &UA2030490},
	{SG31831, &UA2397792},
	{SG31832, &UA2598146},
	{SG31833, &UA905296},
	{SG31834, &UA2608128},
	{SG31835, &UA2420370},
	{SG31836, &UA2559740},
	{SG31837, &UA2029805},
	{SG31838, &UA2203695},
	{SG31839, &UA2403333},
	{SG31841, &UA2423912},
	{SG31843, &UA2689396},
	{SG31844, &UA3090811},
	{SG31845, &UA898943},
	{SG31846, &UA2382542},
	{SG31852, &UA758734},
	{SG31853, &UA830752},
	{SG31854, &UA2026333},
	{SG31855, &UA2034002},
	{SG31856, &UA2044075},
	{SG31857, &UA2426763},
	{SG31858, &UA2033402},
	{SG31859, &UA2422033},
	{SG31860, &UA1978423},
	{SG31861, &UA2026906},
	{SG31862, &UA2223505},
	{SG31863, &UA2382640},
	{SG31864, &UA2613528},
	{SG31865, &UA2614460},
	{SG31866, &UA2614919},
	{SG31867, &UA2614928},
	{SG31868, &UA2615193},
	{SG31869, &UA2615909},
	{SG31870, &UA2616437},
	{SG31871, &UA2617331},
	{SG31872, &UA2618962},
	{SG31873, &UA2619681},
	{SG31874, &UA2619782},
	{SG31876, &UA2620091},
	{SG31877, &UA2620420},
	{SG31878, &UA2622089},
	{SG31879, &UA2622288},
	{SG31880, &UA2627281},
	{SG31881, &UA2628848},
	{SG31882, &UA2629558},
	{SG31883, &UA2631540},
	{SG31884, &UA2637374},
	{SG31885, &UA2974473},
	{SG31886, &UA2976025},
	{SG31887, &UA2976113},
	{SG31904, &UA605613},
	{SG31906, &UA677947},
	{SG31907, &UA2031208},
	{SG31908, &UA2032260},
	{SG31913, &UA2170824},
	{SG31914, &UA2403568},
	{SG31915, &UA2695483},
	{SG31920, &UA1968853},
	{SG31921, &UA2033489},
	{SG31922, &UA2613682},
	{SG31923, &UA2614521},
	{SG31924, &UA2614991},
	{SG31925, &UA2616578},
	{SG31926, &UA2616986},
	{SG31927, &UA2618785},
	{SG31928, &UA2619897},
	{SG31929, &UA2621986},
	{SG31930, &UA2624067},
	{SG31931, &UA2631321},
	{SG31932, &UA2635171},
	{SG31933, &UA2635173},
	{SG31934, &UA2261508},
	{SG31938, &UA2167039},
	{SG31939, &UA2353044},
	{SG31940, &UA2365775},
	{SG31942, &UA2405358},
	{SG31951, &UA2155591},
	{SG31952, &UA2203560},
	{SG31954, &UA2339859},
	{SG31955, &UA2482928},
	{SG31956, &UA2486829},
	{SG31957, &UA2596491},
	{SG31958, &UA2658501},
	{SG31959, &UA2711584},
	{SG31961, &UA2727509},
	{SG31962, &UA2781446},
	{SG31964, &UA2790966},
	{SG31966, &UA2854696},
	{SG31968, &UA2895021},
	{SG31969, &UA3085779},
	{SG31970, &UA2386046},
	{SG31971, &UA2469394},
	{SG31972, &UA2727264},
	{SG31973, &UA2675527},
	{SG31974, &UA2676020},
	{SG31975, &UA2676198},
	{SG31976, &UA2677170},
	{SG31987, &UA368970},
	{SG31988, &UA602902},
	{SG31989, &UA622382},
	{SG31990, &UA622445},
	{SG31991, &UA625154},
	{SG31992, &UA647503},
	{SG31993, &UA674154},
	{SG31994, &UA762106},
	{SG31995, &UA766873},
	{SG31996, &UA823783},
	{SG31997, &UA905716},
	{SG31998, &UA909423},
	{SG32001, &UA1985157},
	{SG32002, &UA2024237},
	{SG32003, &UA2028717},
	{SG32004, &UA2029384},
	{SG32005, &UA2524760},
	{SG32006, &UA2561238},
	{SG32007, &UA2665062},
	{SG32008, &UA609501},
	{SG32009, &UA1971318},
	{SG32010, &UA2168079},
	{SG32011, &UA2610523},
	{SG32012, &UA2362770},
	{SG32013, &UA904449},
	{SG32014, &UA907947},
	{SG32015, &UA911272},
	{SG32016, &UA2026288},
	{SG32017, &UA2043757},
	{SG32018, &UA2402543},
	{SG32019, &UA3065717},
	{SG32020, &UA3083023},
	{SG32021, &UA3083346},
	{SG32023, &UA901457},
	{SG32024, &UA905488},
	{SG32025, &UA2361685},
	{SG32026, &UA3064645},
	{SG32027, &UA2753548},
	{SG32028, &UA2754725},
	{SG32029, &UA2755073},
	{SG32030, &UA2556971},
	{SG32031, &UA2582012},
	{SG32032, &UA2710955},
	{SG32033, &UA2973945},
	{SG32034, &UA2701261},
	{SG32035, &UA2781771},
	{SG32036, &UA1966801},
	{SG32037, &UA1995664},
	{SG32038, &UA2005521},
	{SG32039, &UA2180930},
	{SG32040, &UA2648131},
	{SG32042, &UA911507},
	{SG32043, &UA2360233},
	{SG32044, &UA2386116},
	{SG32045, &UA2622707},
	{SG32046, &UA2487082},
	{SG32047, &UA2030779},
	{SG32048, &UA2261549},
	{SG32049, &UA2035260},
	{SG32050, &UA2204239},
	{SG32051, &UA2353159},
	{SG32052, &UA2554282},
	{SG32058, &UA2034185},
	{SG32059, &UA2170601},
	{SG32060, &UA2175597},
	{SG32061, &UA2180549},
	{SG32062, &UA2360557},
	{SG32063, &UA2362504},
	{SG32064, &UA2403555},
	{SG32065, &UA2457369},
	{SG32066, &UA2500033},
	{SG32067, &UA2597141},
	{SG32068, &UA2652356},
	{SG32069, &UA2031617},
	{SG32070, &UA2257724},
	{SG32071, &UA2257881},
	{SG32072, &UA2365933},
	{SG32073, &UA2379706},
	{SG32074, &UA2383483},
	{SG32075, &UA2383696},
	{SG32076, &UA2395545},
	{SG32077, &UA2403653},
	{SG32078, &UA2418726},
	{SG32079, &UA2420833},
	{SG32080, &UA2422661},
	{SG32081, &UA2528557},
	{SG32082, &UA2653391},
	{SG32083, &UA2654971},
	{SG32084, &UA2692446},
	{SG32085, &UA2714793},
	{SG32086, &UA2764006},
	{SG32087, &UA2777942},
	{SG32088, &UA2778728},
	{SG32089, &UA2934000},
	{SG32090, &UA2991174},
	{SG32091, &UA2994698},
	{SG32092, &UA1988083},
	{SG32093, &UA2226021},
	{SG32094, &UA2226690},
	{SG32101, &UA658060},
	{SG32102, &UA668833},
	{SG32103, &UA899773},
	{SG32104, &UA902154},
	{SG32105, &UA2008874},
	{SG32106, &UA2024565},
	{SG32107, &UA2034132},
	{SG32108, &UA2191511},
	{SG32109, &UA2554211},
	{SG32112, &UA2974293},
	{SG32114, &UA2974488},
	{SG32116, &UA2974724},
	{SG32119, &UA2418912},
	{SG32120, &UA2466137},
	{SG32121, &UA2787021},
	{SG32122, &UA2787195},
	{SG32123, &UA2028496},
	{SG32124, &UA2175684},
	{SG32125, &UA2396859},
	{SG32126, &UA2692336},
	{SG32127, &UA2693057},
	{SG32128, &UA2566641},
	{SG32129, &UA2567011},
	{SG32130, &UA2601592},
	{SG32131, &UA2989218},
	{SG32133, &UA2386240},
	{SG32136, &UA2502917},
	{SG32137, &UA2981195},
	{SG32146, &UA2382314},
	{SG32149, &UA2457995},
	{SG32175, &UA2566507},
	{SG32176, &UA2567254},
	{SG32177, &UA925514},
	{SG32178, &UA2233959},
	{SG32179, &UA2347451},
	{SG32180, &UA2348626},
	{SG32181, &UA2385667},
	{SG32182, &UA2433809},
	{SG32183, &UA2651492},
	{SG32184, &UA2777347},
	{SG32185, &UA2826975},
	{SG32186, &UA2983266},
	{SG32187, &UA2029663},
	{SG32188, &UA2426240},
	{SG32211, &UA887368},
	{SG32212, &UA2417553},
	{SG32215, &UA2257392},
	{SG32217, &UA2540510},
	{SG32218, &UA2540512},
	{SG32221, &UA1991482},
	{SG32222, &UA2403639},
	{SG32223, &UA2412132},
	{SG32224, &UA2528841},
	{SG32225, &UA2580004},
	{SG32226, &UA2597523},
	{SG32227, &UA2700093},
	{SG32230, &UA2029577},
	{SG32231, &UA2451678},
	{SG32232, &UA2008920},
	{SG32233, &UA2227017},
	{SG32234, &UA2762584},
	{SG32235, &UA3065148},
	{SG32236, &UA2198253},
	{SG32237, &UA2205675},
	{SG32238, &UA2467182},
	{SG32239, &UA2973963},
	{SG32243, &UA2709040},
	{SG32301, &UA2419975},
	{SG32303, &UA2456649},
	{SG32305, &UA2466957},
	{SG32307, &UA2482690},
	{SG32309, &UA2529197},
	{SG32312, &UA2554429},
	{SG32314, &UA2762841},
	{SG32316, &UA2889466},
	{SG32329, &UA2025990},
	{SG32331, &UA2027924},
	{SG32333, &UA2502258},
	{SG32335, &UA2556017},
	{SG32337, &UA2609293},
	{SG32339, &UA2616160},
	{SG32341, &UA2616242},
	{SG32343, &UA2616853},
	{SG32345, &UA2616968},
	{SG32348, &UA2618884},
	{SG32350, &UA2621971},
	{SG32351, &UA2622345},
	{SG32353, &UA2622469},
	{SG32355, &UA2625140},
	{SG32356, &UA2626325},
	{SG32358, &UA2628419},
	{SG32360, &UA2629142},
	{SG32361, &UA2631575},
	{SG32363, &UA2994537},
	{SG32389, &UA1995707},
	{SG32393, &UA2450101},
	{SG32395, &UA2526808},
	{SG32400, &UA2708987},
	{SG32425, &UA2566498},
	{SG32428, &UA2566882},
	{SG32430, &UA2567570},
	{SG32432, &UA2584084},
	{SG32434, &UA2600348},
	{SG32435, &UA2602400},
	{SG32437, &UA2602401},
	{SG32438, &UA2602996},
	{SG32440, &UA2878438},
	{SG32442, &UA1991438},
	{SG32444, &UA2205789},
	{SG32445, &UA2355890},
	{SG32453, &UA2753008},
	{SG32455, &UA2754259},
	{SG32456, &UA2754949},
	{SG32468, &UA2391601},
	{SG32469, &UA2427968},
	{SG32477, &UA891746},
	{SG32479, &UA2025360},
	{SG32481, &UA2027660},
	{SG32484, &UA2200012},
	{SG32486, &UA2502203},
	{SG32488, &UA2788890},
	{SG32489, &UA3044442},
	{SG32505, &UA2451698},
	{SG32512, &UA2974046},
	{SG32514, &UA2975062},
	{SG32516, &UA3025627},
	{SG32519, &UA2034099},
	{SG32521, &UA2397634},
	{SG32523, &UA2614655},
	{SG32525, &UA2615866},
	{SG32527, &UA2620919},
	{SG32529, &UA2621120},
	{SG32530, &UA2626034},
	{SG32532, &UA2629129},
	{SG32538, &UA2257600},
	{SG32539, &UA2480877},
	{SG32540, &UA2790281},
	{SG32541, &UA2843325},
	{SG32542, &UA2457321},
	{SG32543, &UA2652703},
	{SG32545, &UA2867509},
	{SG32547, &UA2566540},
	{SG32549, &UA2567023},
	{SG32552, &UA1994528},
	{SG32553, &UA2025010},
	{SG32554, &UA2030867},
	{SG32555, &UA2377091},
	{SG32559, &UA2029571},
	{SG32560, &UA2482335},
	{SG32572, &UA2547815},
	{SG32573, &UA2702645},
	{SG32578, &UA2362029},
	{SG32579, &UA2426540},
	{SG32580, &UA2604677},
	{SG32581, &UA3086516},
	{SG32582, &UA2425618},
	{SG32583, &UA2503614},
	{SG32584, &UA2666032},
	{SG32585, &UA2751010},
	{SG32586, &UA2767310},
	{SG32587, &UA2768650},
	{SG32588, &UA2793457},
	{SG32589, &UA2935898},
	{SG32590, &UA2938103},
	{SG32596, &UA2974016},
	{SG32597, &UA2367627},
	{SG32598, &UA2385650},
	{SG32599, &UA2467190},
	{SG32600, &UA2480510},
	{SG32601, &UA2506077},
	{SG32602, &UA2553416},
	{SG32603, &UA2654779},
	{SG32604, &UA2657910},
	{SG32605, &UA2982961},
	{SG32606, &UA1994791},
	{SG32607, &UA2030767},
	{SG32608, &UA2184219},
	{SG32609, &UA2346804},
	{SG32610, &UA2608728},
	{SG32611, &UA2614416},
	{SG32612, &UA2619346},
	{SG32613, &UA2620175},
	{SG32614, &UA908232},
	{SG32615, &UA1961419},
	{SG32616, &UA2027515},
	{SG32617, &UA2448185},
	{SG32618, &UA2465604},
	{SG32619, &UA2744360},
	{SG32620, &UA2810070},
	{SG32621, &UA857632},
	{SG32622, &UA2026160},
	{SG32623, &UA2419081},
	{SG32624, &UA2528293},
	{SG32632, &UA2974154},
	{SG32636, &UA2424764},
	{SG32644, &UA3065240},
	{SG32645, &UA3065735},
	{SG32646, &UA3066212},
	{SG32647, &UA2566814},
	{SG32648, &UA2566934},
	{SG32649, &UA2567257},
	{SG32650, &UA2583109},
	{SG32651, &UA2583372},
	{SG32652, &UA2601590},
	{SG32653, &UA2601775},
	{SG32654, &UA2603536},
	{SG32655, &UA2668281},
	{SG32656, &UA2733494},
	{SG32657, &UA2794703},
	{SG32658, &UA2007232},
	{SG32659, &UA2187340},
	{SG32660, &UA2355657},
	{SG32661, &UA2372862},
	{SG32662, &UA2419756},
	{SG32663, &UA2421630},
	{SG32664, &UA2455669},
	{SG32666, &UA2581501},
	{SG32667, &UA2704774},
	{SG32668, &UA2732172},
	{SG32669, &UA2814416},
	{SG32670, &UA2992680},
	{SG32671, &UA3088454},
	{SG32672, &UA3100924},
	{SG32673, &UA3110596},
	{SG32674, &UA2566709},
	{SG32675, &UA2026150},
	{SG32676, &UA2257900},
	{SG32677, &UA2416844},
	{SG32678, &UA2040272},
	{SG32679, &UA2333081},
	{SG32680, &UA2555385},
	{SG32681, &UA2556974},
	{SG32682, &UA2167362},
	{SG32683, &UA2231925},
	{SG32684, &UA2552353},
	{SG32685, &UA3042900},
	{SG32686, &UA2974363},
	{SG32687, &UA3021063},
	{SG32688, &UA910581},
	{SG32689, &UA2394390},
	{SG32697, &UA2028938},
	{SG32698, &UA2729512},
	{SG32699, &UA2032328},
	{SG32700, &UA2219125},
	{SG32701, &UA2257628},
	{SG32702, &UA2341579},
	{SG32703, &UA2481086},
	{SG32704, &UA2839603},
	{SG32705, &UA3099673},
	{SG32706, &UA673813},
	{SG32707, &UA2029954},
	{SG32708, &UA2257758},
	{SG32709, &UA2384678},
	{SG32710, &UA2404535},
	{SG32711, &UA2422978},
	{SG32712, &UA2426365},
	{SG32713, &UA2437486},
	{SG32714, &UA2463939},
	{SG32715, &UA2467114},
	{SG32716, &UA2504660},
	{SG32718, &UA2505616},
	{SG32719, &UA2535605},
	{SG32720, &UA2561943},
	{SG32721, &UA2688226},
	{SG32722, &UA2697383},
	{SG32723, &UA2748346},
	{SG32724, &UA2847859},
	{SG32725, &UA3099017},
	{SG32728, &UA2676347},
	{SG32729, &UA2676491},
	{SG32731, &UA2676789},
	{SG32733, &UA2677165},
	{SG32736, &UA896218},
	{SG32737, &UA2031566},
	{SG32739, &UA2549432},
	{SG32741, &UA2338833},
	{SG32742, &UA2399266},
	{SG32743, &UA2595282},
	{SG32744, &UA633525},
	{SG32745, &UA1969796},
	{SG32746, &UA2169482},
	{SG32747, &UA2522491},
	{SG32748, &UA2703900},
	{SG32749, &UA2385750},
	{SG32750, &UA2404777},
	{SG32751, &UA2556546},
	{SG32752, &UA2846024},
	{SG32753, &UA2007541},
	{SG32754, &UA2613590},
	{SG32755, &UA2179430},
	{SG32756, &UA2235817},
	{SG32757, &UA2455328},
	{SG32758, &UA2816633},
	{SG32759, &UA2368866},
	{SG32760, &UA2655497},
	{SG32761, &UA912476},
	{SG32762, &UA2199634},
	{SG32763, &UA2232268},
	{SG32764, &UA2353320},
	{SG32765, &UA2383307},
	{SG32766, &UA2425978},
	{SG32767, &UA2502206},
	{SG32768, &UA2678365},
	{SG32769, &UA2706223},
	{SG32770, &UA2764471},
	{SG32771, &UA2779882},
	{SG32772, &UA2795884},
	{SG32773, &UA2995404},
	{SG32774, &UA3047513},
	{SG32776, &UA3107603},
	{SG32788, &UA2026914},
	{SG32789, &UA2476162},
	{SG32792, &UA911255},
	{SG32793, &UA2026980},
	{SG32794, &UA2548918},
	{SG32803, &UA872675},
	{SG32805, &UA903215},
	{SG32806, &UA912054},
	{SG32807, &UA2166985},
	{SG32808, &UA2377802},
	{SG32809, &UA2951019},
	{SG32810, &UA2752974},
	{SG32811, &UA2756001},
	{SG32812, &UA2879357},
	{SG32814, &UA2026939},
	{SG32815, &UA2343491},
	{SG32816, &UA2481053},
	{SG32817, &UA2695194},
	{SG32818, &UA2566802},
	{SG32819, &UA2766704},
	{SG32820, &UA1972223},
	{SG32821, &UA2026726},
	{SG32822, &UA2242281},
	{SG32823, &UA1963623},
	{SG32824, &UA2232430},
	{SG32825, &UA2449931},
	{SG32826, &UA2689765},
	{SG32827, &UA2821020},
	{SG32828, &UA909263},
	{SG32829, &UA1982788},
	{SG32830, &UA2157099},
	{SG32831, &UA2418352},
	{SG32832, &UA2579086},
	{SG32833, &UA2763432},
	{SG32834, &UA2850270},
	{SG32835, &UA2985709},
	{SG32836, &UA2586645},
	{SG32837, &UA912473},
	{SG32838, &UA2428127},
	{SG32840, &UA3094186},
	{SG32871, &UA609359},
	{SG32873, &UA907320},
	{SG32874, &UA1995566},
	{SG32875, &UA2792431},
	{SG32878, &UA2201561},
	{SG32879, &UA2203503},
	{SG32880, &UA2204149},
	{SG32881, &UA2205666},
	{SG32882, &UA2237129},
	{SG32883, &UA2359207},
	{SG32884, &UA2383942},
	{SG32885, &UA2406900},
	{SG32886, &UA2431822},
	{SG32887, &UA2461822},
	{SG32888, &UA2470410},
	{SG32889, &UA2934271},
	{SG32890, &UA2205045},
	{SG32892, &UA2034652},
	{SG32893, &UA908887},
	{SG32894, &UA2023946},
	{SG32895, &UA2686753},
	{SG32908, &UA3064763},
	{SG32909, &UA1952879},
	{SG32911, &UA1979277},
	{SG32912, &UA1996368},
	{SG32913, &UA2178423},
	{SG32914, &UA2338817},
	{SG32915, &UA2411231},
	{SG32916, &UA2643749},
	{SG32917, &UA2737445},
	{SG32923, &UA2261502},
	{SG32924, &UA2502663},
	{SG32925, &UA3072011},
	{SG32926, &UA2030544},
	{SG32927, &UA2566700},
	{SG32928, &UA2566799},
	{SG32929, &UA2567251},
	{SG32930, &UA2567326},
	{SG32931, &UA2583174},
	{SG32932, &UA2667826},
	{SG32933, &UA2668009},
	{SG32934, &UA2668475},
	{SG32936, &UA2732977},
	{SG32940, &UA2029043},
	{SG32941, &UA2032015},
	{SG32942, &UA2034658},
	{SG32943, &UA2034836},
	{SG32944, &UA2230571},
	{SG32945, &UA2361895},
	{SG32946, &UA2480782},
	{SG32947, &UA1997048},
	{SG32948, &UA2167558},
	{SG32949, &UA2418513},
	{SG32950, &UA2449179},
	{SG32951, &UA2593586},
	{SG32957, &UA609776},
	{SG32958, &UA900945},
	{SG32960, &UA2432653},
	{SG32962, &UA3021425},
	{SG32964, &UA609378},
	{SG32965, &UA900630},
	{SG32966, &UA2357196},
	{SG32967, &UA1976970},
	{SG32968, &UA2421433},
	{SG32970, &UA2225171},
	{SG32971, &UA2226281},
	{SG32972, &UA2257897},
	{SG32973, &UA2648420},
	{SG32974, &UA2980997},
	{SG32978, &UA2033115},
	{SG32979, &UA2167537},
	{SG32980, &UA2432246},
	{SG32981, &UA2465330},
	{SG32984, &UA2862481},
	{SG32992, &UA2166806},
	{SG32994, &UA2655730},
	{SG32995, &UA2416730},
	{SG32996, &UA2485877},
	{SG32997, &UA2847632},
	{SG33003, &UA680446},
	{SG33006, &UA834210},
	{SG33008, &UA906279},
	{SG33010, &UA1955325},
	{SG33013, &UA1965476},
	{SG33014, &UA2024842},
	{SG33015, &UA2033461},
	{SG33016, &UA2788864},
	{SG33017, &UA2988629},
	{SG33018, &UA2173879},
	{SG33019, &UA2455463},
	{SG33020, &UA2172433},
	{SG33021, &UA3065140},
	{SG33022, &UA3067599},
	{SG33023, &UA901781},
	{SG33024, &UA2032117},
	{SG33025, &UA2566928},
	{SG33027, &UA2204765},
	{SG33032, &UA2756415},
	{SG33033, &UA2973485},
	{SG33034, &UA2973900},
	{SG33035, &UA3020973},
	{SG33036, &UA2200369},
	{SG33037, &UA2228727},
	{SG33038, &UA2566690},
	{SG33039, &UA2566903},
	{SG33040, &UA2719305},
	{SG33046, &UA2314075},
	{SG33047, &UA2654181},
	{SG33048, &UA2353032},
	{SG33049, &UA2503362},
	{SG33051, &UA1988452},
	{SG33052, &UA2233052},
	{SG33053, &UA2476432},
	{SG33054, &UA911879},
	{SG33055, &UA2026464},
	{SG33056, &UA2027870},
	{SG33057, &UA2595607},
	{SG33058, &UA2741124},
	{SG33060, &UA2868603},
	{SG33063, &UA1966233},
	{SG33066, &UA2033887},
	{SG33067, &UA2444833},
	{SG33077, &UA1987428},
	{SG33079, &UA2026070},
	{SG33084, &UA2234087},
	{SG33086, &UA2257431},
	{SG33088, &UA2354383},
	{SG33090, &UA2367027},
	{SG33092, &UA2367688},
	{SG33094, &UA2594428},
	{SG33096, &UA2653218},
	{SG33098, &UA2879631},
	{SG33100, &UA2901197},
	{SG33103, &UA2972004},
	{SG33105, &UA3053204},
	{SG33107, &UA896480},
	{SG33109, &UA902537},
	{SG33111, &UA2545224},
	{SG33114, &UA2235402},
	{SG33122, &UA2028817},
	{SG33123, &UA2428421},
	{SG33131, &UA2540508},
	{SG33133, &UA2201391},
	{SG33134, &UA2261516},
	{SG33136, &UA2261608},
	{SG33137, &UA2386919},
	{SG33139, &UA2566626},
	{SG33140, &UA2583248},
	{SG33141, &UA2385254},
	{SG33142, &UA2420549},
	{SG33143, &UA2421811},
	{SG33144, &UA2423695},
	{SG33145, &UA2466271},
	{SG33146, &UA2503671},
	{SG33147, &UA2523786},
	{SG33148, &UA2529569},
	{SG33149, &UA2556468},
	{SG33150, &UA2578026},
	{SG33151, &UA2652737},
	{SG33152, &UA2791012},
	{SG33153, &UA2836898},
	{SG33155, &UA2950677},
	{SG33157, &UA3118568},
	{SG33185, &UA2676237},
	{SG33186, &UA2676688},
	{SG33187, &UA2734959},
	{SG33188, &UA2347868},
	{SG33190, &UA2506509},
	{SG33191, &UA2554618},
	{SG33192, &UA2754648},
	{SG33193, &UA2754941},
	{SG33194, &UA3065083},
	{SG33196, &UA3080932},
	{SG33197, &UA2462075},
	{SG33198, &UA2566686},
	{SG33199, &UA2566753},
	{SG33200, &UA2567263},
	{SG33201, &UA2667522},
	{SG33202, &UA2974073},
	{SG33203, &UA2974797},
	{SG33204, &UA2975800},
	{SG33206, &UA2976114},
	{SG33208, &UA3021058},
	{SG33210, &UA3021149},
	{SG33212, &UA3021645},
	{SG33214, &UA3023773},
	{SG33217, &UA2257597},
	{SG33218, &UA2552966},
	{SG33219, &UA2566696},
	{SG33220, &UA2567093},
	{SG33221, &UA2567523},
	{SG33222, &UA2702236},
	{SG33223, &UA2791509},
	{SG33224, &UA2567563},
	{SG33225, &UA2862523},
	{SG33226, &UA2566525},
	{SG33227, &UA2634143},
	{SG33246, &UA2034685},
	{SG33247, &UA2034707},
	{SG33249, &UA2034726},
	{SG33250, &UA2034806},
	{SG33251, &UA2034869},
	{SG33252, &UA2034893},
	{SG33254, &UA2034894},
	{SG33255, &UA2034902},
	{SG33257, &UA2034916},
	{SG33259, &UA2034929},
	{SG33261, &UA2034694},
	{SG33263, &UA2034948},
	{SG33265, &UA2034955},
	{SG33267, &UA2034974},
	{SG33269, &UA2035022},
	{SG33271, &UA2035047},
	{SG33273, &UA2035081},
	{SG33275, &UA2035142},
	{SG33277, &UA2230704},
	{SG33279, &UA2386224},
	{SG33281, &UA2422817},
	{SG33282, &UA2426435},
	{SG33283, &UA2537453},
	{SG33284, &UA2556485},
	{SG33286, &UA2577726},
	{SG33287, &UA2578173},
	{SG33288, &UA2695888},
	{SG33289, &UA3042828},
	{SG33290, &UA3117964},
	{SG33291, &UA2235737},
	{SG33292, &UA2741403},
	{SG33294, &UA2755134},
	{SG33296, &UA2755862},
	{SG33297, &UA2174520},
	{SG33298, &UA2257752},
	{SG33299, &UA2352735},
	{SG33300, &UA2420902},
	{SG33301, &UA2457105},
	{SG33302, &UA2502762},
	{SG33303, &UA2526023},
	{SG33304, &UA2778063},
	{SG33305, &UA2848572},
	{SG33306, &UA3043041},
	{SG33307, &UA3044637},
	{SG33308, &UA2973949},
	{SG33309, &UA2976689},
	{SG33311, &UA2205380},
	{SG33313, &UA2226434},
	{SG33315, &UA2836999},
	{SG33318, &UA2426722},
	{SG33319, &UA2428794},
	{SG33320, &UA2727440},
	{SG33321, &UA2729738},
	{SG33322, &UA2986750},
	{SG33323, &UA3070412},
	{SG33335, &UA2755333},
	{SG33340, &UA2257772},
	{SG33341, &UA2338336},
	{SG33342, &UA2349729},
	{SG33343, &UA2454767},
	{SG33344, &UA2464522},
	{SG33345, &UA2464860},
	{SG33346, &UA2577775},
	{SG33347, &UA2696464},
	{SG33348, &UA2698125},
	{SG33349, &UA2846065},
	{SG33350, &UA2395175},
	{SG33351, &UA2764167},
	{SG33352, &UA2953444},
	{SG33353, &UA3022833},
	{SG33368, &UA2739938},
	{SG33369, &UA2937676},
	{SG33372, &UA2028896},
	{SG33374, &UA2500381},
	{SG33376, &UA2522605},
	{SG33378, &UA2566514},
	{SG33379, &UA2583204},
	{SG33381, &UA610060},
	{SG33382, &UA908972},
	{SG33383, &UA2017926},
	{SG33384, &UA2026185},
	{SG33385, &UA613215},
	{SG33413, &UA2973967},
	{SG33414, &UA2974447},
	{SG33415, &UA3020864},
	{SG33416, &UA3022551},
	{SG33417, &UA3023206},
	{SG33418, &UA3023800},
	{SG33419, &UA3057979},
	{SG33420, &UA3092236},
	{SG33421, &UA2423979},
	{SG33422, &UA2461751},
	{SG33436, &UA2693607},
	{SG33438, &UA2878975},
	{SG33440, &UA3012802},
	{SG33442, &UA3015104},
	{SG33444, &UA2753032},
	{SG33445, &UA2755072},
	{SG33446, &UA2755537},
	{SG33447, &UA2974362},
	{SG33448, &UA2974656},
	{SG33449, &UA3020479},
	{SG33450, &UA3020939},
	{SG33451, &UA3021166},
	{SG33452, &UA3022179},
	{SG33453, &UA3022218},
	{SG33454, &UA3022336},
	{SG33455, &UA2676325},
	{SG33456, &UA2676822},
	{SG33464, &UA2672692},
	{SG33465, &UA2672984},
	{SG33466, &UA2673963},
	{SG33467, &UA2675070},
	{SG33468, &UA2676093},
	{SG33469, &UA2710837},
	{SG33470, &UA2713166},
	{SG33471, &UA2713320},
	{SG33472, &UA2735814},
	{SG33473, &UA816712},
	{SG33474, &UA870406},
	{SG33475, &UA1984908},
	{SG33476, &UA2027293},
	{SG33477, &UA2609081},
	{SG33478, &UA2611468},
	{SG33479, &UA2611694},
	{SG33480, &UA2611753},
	{SG33481, &UA2612972},
	{SG33482, &UA2614673},
	{SG33483, &UA2618799},
	{SG33484, &UA2632731},
	{SG33485, &UA2634270},
	{SG33487, &UA3063762},
	{SG33490, &UA3065996},
	{SG33493, &UA3066540},
	{SG33495, &UA3080899},
	{SG33497, &UA3094500},
	{SG33498, &UA3094633},
	{SG33499, &UA3111023},
	{SG33505, &UA899871},
	{SG33506, &UA2026329},
	{SG33507, &UA2027895},
	{SG33508, &UA2483520},
	{SG33509, &UA2592545},
	{SG33510, &UA2696128},
	{SG33511, &UA2696284},
	{SG33512, &UA2701868},
	{SG33513, &UA2763303},
	{SG33514, &UA2988035},
	{SG33515, &UA3086512},
	{SG33518, &UA2523750},
	{SG33519, &UA2815600},
	{SG33524, &UA2029103},
	{SG33526, &UA2204030},
	{SG33527, &UA2364575},
	{SG33528, &UA2427258},
	{SG33529, &UA2450586},
	{SG33530, &UA2530074},
	{SG33531, &UA2950938},
	{SG33532, &UA3042340},
	{SG33533, &UA2753856},
	{SG33534, &UA2753914},
	{SG33535, &UA2769381},
	{SG33536, &UA2769618},
	{SG33537, &UA2773449},
	{SG33538, &UA2773610},
	{SG33539, &UA2773819},
	{SG33540, &UA2783005},
	{SG33547, &UA1995008},
	{SG33548, &UA2450958},
	{SG33549, &UA2615198},
	{SG33550, &UA2615736},
	{SG33551, &UA2620217},
	{SG33552, &UA2621650},
	{SG33553, &UA2625649},
	{SG33554, &UA2631133},
	{SG33555, &UA2632318},
	{SG33556, &UA2636213},
	{SG33557, &UA2637545},
	{SG33558, &UA3066885},
	{SG33559, &UA3081243},
	{SG33560, &UA3082494},
	{SG33561, &UA3082602},
	{SG33562, &UA3083653},
	{SG33563, &UA3126782},
	{SG33564, &UA3126794},
	{SG33568, &UA2376342},
	{SG33574, &UA2384067},
	{SG33578, &UA2753200},
	{SG33581, &UA2754161},
	{SG33583, &UA2754573},
	{SG33586, &UA2769579},
	{SG33588, &UA2782605},
	{SG33591, &UA2798923},
	{SG33593, &UA2800000},
	{SG33597, &UA2879413},
	{SG33616, &UA2028753},
	{SG33617, &UA3067658},
	{SG33618, &UA3067659},
	{SG33619, &UA3083659},
	{SG33621, &UA1996445},
	{SG33645, &UA2674418},
	{SG33646, &UA2676189},
	{SG33652, &UA2753859},
	{SG33653, &UA2754939},
	{SG33654, &UA2755061},
	{SG33655, &UA2756745},
	{SG33656, &UA2782624},
	{SG33657, &UA2800284},
	{SG33658, &UA2867776},
	{SG33659, &UA2868700},
	{SG33660, &UA2769281},
	{SG33661, &UA1960532},
	{SG33674, &UA2566522},
	{SG33675, &UA2667402},
	{SG33678, &UA2668478},
	{SG33689, &UA2457383},
	{SG33690, &UA2580216},
	{SG33691, &UA2659123},
	{SG33692, &UA2762385},
	{SG33697, &UA2030350},
	{SG33698, &UA2362997},
	{SG33699, &UA2428115},
	{SG33700, &UA2838925},
	{SG33701, &UA2933812},
	{SG33702, &UA2367477},
	{SG33703, &UA2747337},
	{SG33704, &UA2963628},
	{SG33707, &UA3023393},
	{SG33714, &UA2502002},
	{SG33715, &UA2503674},
	{SG33716, &UA2622710},
	{SG33717, &UA2631010},
	{SG33719, &UA2637890},
	{SG33724, &UA2676355},
	{SG33726, &UA2706709},
	{SG33732, &UA2174463},
	{SG33733, &UA2236467},
	{SG33734, &UA2345644},
	{SG33735, &UA2348294},
	{SG33736, &UA2502897},
	{SG33739, &UA369447},
	{SG33740, &UA454608},
	{SG33741, &UA624537},
	{SG33742, &UA723668},
	{SG33743, &UA725718},
	{SG33744, &UA858214},
	{SG33745, &UA869509},
	{SG33747, &UA903757},
	{SG33750, &UA2030300},
	{SG33755, &UA2561027},
	{SG33756, &UA2764618},
	{SG33757, &UA2337909},
	{SG33758, &UA2649385},
	{SG33759, &UA2223098},
	{SG33760, &UA2421143},
	{SG33761, &UA2468460},
	{SG33762, &UA2726744},
	{SG33763, &UA2779554},
	{SG33764, &UA2839162},
	{SG33765, &UA3006206},
	{SG33766, &UA3053279},
	{SG33767, &UA1994998},
	{SG33770, &UA2431644},
	{SG33772, &UA2518471},
	{SG33773, &UA2695270},
	{SG33775, &UA3117606},
	{SG33777, &UA1997272},
	{SG33778, &UA2257748},
	{SG33779, &UA2511245},
	{SG33780, &UA3033398},
	{SG33781, &UA3108471},
	{SG33784, &UA2554936},
	{SG33785, &UA2766644},
	{SG33786, &UA2550642},
	{SG33787, &UA2974023},
	{SG33788, &UA3006006},
	{SG33789, &UA3009887},
	{SG33790, &UA3057563},
	{SG33791, &UA3060023},
	{SG33792, &UA3126407},
	{SG33793, &UA2676243},
	{SG33803, &UA2004618},
	{SG33804, &UA2023703},
	{SG33806, &UA2355530},
	{SG33808, &UA2524968},
	{SG33810, &UA2526723},
	{SG33812, &UA2597518},
	{SG33814, &UA2650890},
	{SG33816, &UA2695812},
	{SG33818, &UA2733276},
	{SG33819, &UA2793422},
	{SG33821, &UA2849317},
	{SG33823, &UA3072499},
	{SG33825, &UA3098741},
	{SG33827, &UA3103419},
	{SG33831, &UA2425943},
	{SG33832, &UA2485496},
	{SG33834, &UA2528734},
	{SG33837, &UA2530211},
	{SG33840, &UA2555935},
	{SG33843, &UA2557789},
	{SG33846, &UA2656143},
	{SG33849, &UA2702273},
	{SG33853, &UA2733049},
	{SG33857, &UA2734582},
	{SG33860, &UA2747314},
	{SG33862, &UA2764316},
	{SG33864, &UA3044594},
	{SG33866, &UA3073000},
	{SG33870, &UA2035010},
	{SG33872, &UA2035088},
	{SG33873, &UA2404020},
	{SG33874, &UA2430311},
	{SG33875, &UA2482319},
	{SG33876, &UA2485485},
	{SG33877, &UA2845341},
	{SG33878, &UA2848511},
	{SG33884, &UA2029759},
	{SG33886, &UA2427018},
	{SG33887, &UA2529892},
	{SG33888, &UA2166866},
	{SG33889, &UA2454916},
	{SG33890, &UA3117792},
	{SG33892, &UA2769434},
	{SG33894, &UA2879252},
	{SG33902, &UA2566939},
	{SG33904, &UA2567376},
	{SG33905, &UA2583128},
	{SG33907, &UA2583968},
	{SG33914, &UA908718},
	{SG33915, &UA2336885},
	{SG33917, &UA2422061},
	{SG33918, &UA2622194},
	{SG33920, &UA2629362},
	{SG33921, &UA2630276},
	{SG33922, &UA2714043},
	{SG33924, &UA2175673},
	{SG33926, &UA2257668},
	{SG33927, &UA2933895},
	{SG33934, &UA904351},
	{SG33936, &UA907294},
	{SG33940, &UA2974030},
	{SG33941, &UA896189},
	{SG33942, &UA2034271},
	{SG33947, &UA2973958},
	{SG33949, &UA1995374},
	{SG33950, &UA2229004},
	{SG33952, &UA2380920},
	{SG33955, &UA2657151},
	{SG33956, &UA3086246},
	{SG33957, &UA3063715},
	{SG33958, &UA3067065},
	{SG33959, &UA3081050},
	{SG33960, &UA3081101},
	{SG33961, &UA3082058},
	{SG33962, &UA3083941},
	{SG33963, &UA3094746},
	{SG33964, &UA3114358},
	{SG33965, &UA2431224},
	{SG33967, &UA1991938},
	{SG33969, &UA2027633},
	{SG33970, &UA2399235},
	{SG33971, &UA2505184},
	{SG33972, &UA2663869},
	{SG33974, &UA2732548},
	{SG33976, &UA2764228},
	{SG33977, &UA2985683},
	{SG33979, &UA3100818},
	{SG33980, &UA2672310},
	{SG33981, &UA2676691},
	{SG33982, &UA2708584},
	{SG33983, &UA3065340},
	{SG33984, &UA3068338},
	{SG33985, &UA3094157},
	{SG33986, &UA2364860},
	{SG33987, &UA2975952},
	{SG34012, &UA2674215},
	{SG34014, &UA2676016},
	{SG34015, &UA2676483},
	{SG34018, &UA2676980},
	{SG34019, &UA2708385},
	{SG34020, &UA2711695},
	{SG34021, &UA2735314},
	{SG34022, &UA2736225},
	{SG34023, &UA2736610},
	{SG34024, &UA2767260},
	{SG34025, &UA2566887},
	{SG34026, &UA2583123},
	{SG34027, &UA2390429},
	{SG34028, &UA2467167},
	{SG34029, &UA3031378},
	{SG34030, &UA2423872},
	{SG34031, &UA2585753},
	{SG34032, &UA2606215},
	{SG34033, &UA2766146},
	{SG34035, &UA2461257},
	{SG34036, &UA2554839},
	{SG34037, &UA2666627},
	{SG34042, &UA2755357},
	{SG34044, &UA2636477},
	{SG34045, &UA2754726},
	{SG34046, &UA2755089},
	{SG34057, &UA2566502},
	{SG34058, &UA2566910},
	{SG34059, &UA2633704},
	{SG34060, &UA2633885},
	{SG34061, &UA2634689},
	{SG34062, &UA2634755},
	{SG34083, &UA3094789},
	{SG34085, &UA2973463},
	{SG34086, &UA2975023},
	{SG34090, &UA2677291},
	{SG34091, &UA2735871},
	{SG34092, &UA2202410},
	{SG34093, &UA2485864},
	{SG34095, &UA2778671},
	{SG34096, &UA2975018},
	{SG34097, &UA2388207},
	{SG34098, &UA2437663},
	{SG34099, &UA2571348},
	{SG34100, &UA2698693},
	{SG34101, &UA2739946},
	{SG34105, &UA2226847},
	{SG34107, &UA909025},
	{SG34108, &UA1990663},
	{SG34109, &UA1993399},
	{SG34110, &UA2390481},
	{SG34111, &UA2257817},
	{SG34112, &UA2418203},
	{SG34113, &UA2454813},
	{SG34114, &UA2464555},
	{SG34115, &UA2956771},
	{SG34116, &UA2673031},
	{SG34117, &UA2673189},
	{SG34118, &UA2673222},
	{SG34119, &UA2676505},
	{SG34120, &UA2676581},
	{SG34121, &UA2676943},
	{SG34122, &UA2676999},
	{SG34123, &UA2706677},
	{SG34124, &UA2974067},
	{SG34125, &UA2975815},
	{SG34127, &UA2712463},
	{SG34144, &UA2752946},
	{SG34145, &UA2754326},
	{SG34147, &UA2755268},
	{SG34153, &UA1972502},
	{SG34154, &UA2024312},
	{SG34156, &UA2411372},
	{SG34160, &UA2043759},
	{SG34162, &UA2206818},
	{SG34164, &UA2206817},
	{SG34166, &UA2206819},
	{SG34168, &UA2387510},
	{SG34170, &UA2423781},
	{SG34172, &UA2467138},
	{SG34174, &UA2530290},
	{SG34175, &UA2555364},
	{SG34177, &UA2597862},
	{SG34179, &UA2658019},
	{SG34181, &UA2659376},
	{SG34185, &UA2731673},
	{SG34188, &UA2737746},
	{SG34190, &UA2737871},
	{SG34192, &UA2781672},
	{SG34194, &UA2792412},
	{SG34196, &UA2794075},
	{SG34198, &UA2892023},
	{SG34200, &UA2935144},
	{SG34202, &UA2984491},
	{SG34204, &UA2996892},
	{SG34206, &UA3071263},
	{SG34208, &UA3087108},
	{SG34209, &UA3106000},
	{SG34211, &UA3121132},
	{SG34244, &UA656525},
	{SG34246, &UA780527},
	{SG34248, &UA807704},
	{SG34249, &UA854517},
	{SG34250, &UA1991667},
	{SG34251, &UA2027290},
	{SG34252, &UA2663704},
	{SG34253, &UA2740080},
	{SG34254, &UA2955874},
	{SG34255, &UA924435},
	{SG34256, &UA2448963},
	{SG34257, &UA2563861},
	{SG34258, &UA2668988},
	{SG34259, &UA2698703},
	{SG34260, &UA2780474},
	{SG34265, &UA2566707},
	{SG34266, &UA669124},
	{SG34267, &UA856438},
	{SG34268, &UA1950283},
	{SG34269, &UA2030709},
	{SG34270, &UA2033346},
	{SG34271, &UA2355637},
	{SG34272, &UA2475678},
	{SG34278, &UA2457240},
	{SG34279, &UA2467405},
	{SG34280, &UA2595154},
	{SG34282, &UA2974718},
	{SG34284, &UA2974801},
	{SG34285, &UA3023243},
	{SG34286, &UA2353074},
	{SG34287, &UA2426596},
	{SG34288, &UA3066879},
	{SG34292, &UA2753286},
	{SG34293, &UA2381819},
	{SG34294, &UA2503401},
	{SG34295, &UA2547357},
	{SG34296, &UA2726309},
	{SG34297, &UA3071047},
	{SG34298, &UA3111189},
	{SG34299, &UA3114929},
	{SG34301, &UA2566965},
	{SG34302, &UA2583785},
	{SG34303, &UA2232459},
	{SG34304, &UA1997822},
	{SG34305, &UA2366029},
	{SG34306, &UA1967249},
	{SG34307, &UA2406369},
	{SG34308, &UA2420456},
	{SG34309, &UA2430880},
	{SG34310, &UA925495},
	{SG34314, &UA2755108},
	{SG34315, &UA2463956},
	{SG34318, &UA2481911},
	{SG34319, &UA2671960},
	{SG34320, &UA2676496},
	{SG34321, &UA2676510},
	{SG34322, &UA2708388},
	{SG34324, &UA3021591},
	{SG34325, &UA3023583},
	{SG34326, &UA3057980},
	{SG34327, &UA2224478},
	{SG34340, &UA2369661},
	{SG34341, &UA2391205},
	{SG34342, &UA2428627},
	{SG34343, &UA2487844},
	{SG34344, &UA2504313},
	{SG34345, &UA2508312},
	{SG34346, &UA2585956},
	{SG34347, &UA2601943},
	{SG34348, &UA2766711},
	{SG34351, &UA2498188},
	{SG34353, &UA2986982},
	{SG34367, &UA2411331},
	{SG34369, &UA2699399},
	{SG34371, &UA2790745},
	{SG34372, &UA2848962},
	{SG34375, &UA2974992},
	{SG34389, &UA2676332},
	{SG34390, &UA2676436},
	{SG34451, &UA2201656},
	{SG34460, &UA2676504},
	{SG34462, &UA2406359},
	{SG34464, &UA2483198},
	{SG34476, &UA2030092},
	{SG34492, &UA2740288},
	{SG34493, &UA2354701},
	{SG34494, &UA2366707},
	{SG34495, &UA2466227},
	{SG34508, &UA2752935},
	{SG34509, &UA2753070},
	{SG34511, &UA3065317},
	{SG34518, &UA605488},
	{SG34520, &UA788168},
	{SG34521, &UA909169},
	{SG34523, &UA1981966},
	{SG34524, &UA2024518},
	{SG34525, &UA2167466},
	{SG34528, &UA3066222},
	{SG34537, &UA3066220},
	{SG34539, &UA3068315},
	{SG34542, &UA2380771},
	{SG34543, &UA899114},
	{SG34544, &UA2206777},
	{SG34545, &UA2206774},
	{SG34546, &UA2206793},
	{SG34547, &UA2206800},
	{SG34548, &UA2206801},
	{SG34549, &UA2206806},
	{SG34551, &UA2206811},
	{SG34552, &UA2206813},
	{SG34554, &UA2257589},
	{SG34557, &UA2257674},
	{SG34560, &UA2356322},
	{SG34561, &UA2404144},
	{SG34563, &UA2423468},
	{SG34565, &UA2456266},
	{SG34567, &UA2527325},
	{SG34568, &UA2552306},
	{SG34569, &UA2693754},
	{SG34570, &UA2701462},
	{SG34571, &UA2746927},
	{SG34572, &UA2950810},
	{SG34573, &UA2676291},
	{SG34579, &UA2167090},
	{SG34581, &UA2550496},
	{SG34583, &UA2566775},
	{SG34585, &UA2667643},
	{SG34587, &UA2566705},
	{SG34589, &UA2676037},
	{SG34591, &UA2676044},
	{SG34603, &UA2368958},
	{SG34607, &UA2360880},
	{SG34608, &UA2426710},
	{SG34610, &UA2560319},
	{SG34611, &UA2854308},
	{SG34612, &UA2889407},
	{SG34615, &UA2974476},
	{SG34621, &UA607780},
	{SG34623, &UA806775},
	{SG34625, &UA2031128},
	{SG34627, &UA2377850},
	{SG34628, &UA3068055},
	{SG34629, &UA2945130},
	{SG34630, &UA2974615},
	{SG34632, &UA2975806},
	{SG34636, &UA2566643},
	{SG34639, &UA911396},
	{SG34640, &UA2442969},
	{SG34645, &UA2261513},
	{SG34646, &UA2428229},
	{SG34647, &UA2384945},
	{SG34648, &UA2844901},
	{SG34649, &UA2602043},
	{SG34650, &UA2669365},
	{SG34651, &UA2708144},
	{SG34652, &UA2570264},
	{SG34653, &UA2773089},
	{SG34654, &UA2879296},
	{SG34655, &UA2974787},
	{SG34656, &UA2974997},
	{SG34657, &UA2424786},
	{SG34658, &UA2233605},
	{SG34659, &UA2404503},
	{SG34660, &UA2698113},
	{SG34661, &UA2778498},
	{SG34662, &UA2233695},
	{SG34663, &UA2367284},
	{SG34664, &UA2428478},
	{SG34665, &UA2753268},
	{SG34666, &UA2753451},
	{SG34667, &UA2754634},
	{SG34671, &UA3065471},
	{SG34672, &UA3066211},
	{SG34679, &UA2386730},
	{SG34680, &UA2595428},
	{SG34692, &UA910599},
	{SG34697, &UA2431004},
	{SG34698, &UA2436170},
	{SG34699, &UA2457689},
	{SG34700, &UA2484520},
	{SG34701, &UA2704862},
	{SG34702, &UA2716051},
	{SG34703, &UA2390871},
	{SG34706, &UA1996047},
	{SG34707, &UA3064869},
	{SG34710, &UA2676191},
	{SG34711, &UA2677038},
	{SG34712, &UA2706790},
	{SG34714, &UA3067912},
	{SG34716, &UA3080594},
	{SG34719, &UA2566900},
	{SG34720, &UA2866753},
	{SG34721, &UA2975085},
	{SG34732, &UA2379294},
	{SG34734, &UA2721856},
	{SG34735, &UA2828862},
	{SG34736, &UA2769694},
	{SG34738, &UA2782384},
	{SG34740, &UA2782637},
	{SG34741, &UA2782782},
	{SG34742, &UA2799034},
	{SG34744, &UA2799324},
	{SG34746, &UA2799392},
	{SG34748, &UA2799416},
	{SG34750, &UA2867775},
	{SG34752, &UA2867809},
	{SG34754, &UA2868221},
	{SG34756, &UA2868303},
	{SG34758, &UA2869084},
	{SG34760, &UA2876522},
	{SG34762, &UA2877415},
	{SG34764, &UA2877578},
	{SG34766, &UA2879311},
	{SG34768, &UA2879649},
	{SG34769, &UA2892562},
	{SG34770, &UA2892590},
	{SG34771, &UA2892804},
	{SG34772, &UA2892906},
	{SG34773, &UA2892989},
	{SG34774, &UA2893156},
	{SG34775, &UA2893375},
	{SG34776, &UA2893559},
	{SG34777, &UA2937435},
	{SG34778, &UA2937642},
	{SG34779, &UA2937695},
	{SG34780, &UA2937725},
	{SG34781, &UA2938010},
	{SG34782, &UA2938155},
	{SG34783, &UA2938241},
	{SG34784, &UA2938299},
	{SG34785, &UA2957433},
	{SG34786, &UA2957852},
	{SG34788, &UA2958631},
	{SG34790, &UA2997487},
	{SG34791, &UA2997760},
	{SG34792, &UA2998536},
	{SG34793, &UA2998588},
	{SG34794, &UA2999894},
	{SG34795, &UA3000418},
	{SG34796, &UA3004774},
	{SG34797, &UA3049623},
	{SG34798, &UA3049831},
	{SG34799, &UA3050264},
	{SG34801, &UA2231131},
	{SG34806, &UA2600333},
	{SG34808, &UA2753776},
	{SG34809, &UA2774081},
	{SG34815, &UA2566543},
	{SG34817, &UA2732975},
	{SG34820, &UA1995857},
	{SG34821, &UA1996199},
	{SG34822, &UA2010458},
	{SG34823, &UA2413764},
	{SG34824, &UA2437379},
	{SG34825, &UA2437409},
	{SG34826, &UA2602762},
	{SG34827, &UA2673595},
	{SG34828, &UA2257715},
	{SG34829, &UA2657860},
	{SG34830, &UA3067589},
	{SG34831, &UA3081175},
	{SG34841, &UA2427652},
	{SG34843, &UA2732587},
	{SG34847, &UA2666839},
	{SG34849, &UA2753458},
	{SG34850, &UA2755526},
	{SG34851, &UA2798878},
	{SG34852, &UA2938157},
	{SG34854, &UA2029589},
	{SG34856, &UA2224284},
	{SG34857, &UA2228414},
	{SG34859, &UA2350217},
	{SG34861, &UA2416747},
	{SG34862, &UA2525874},
	{SG34863, &UA2689157},
	{SG34864, &UA2725283},
	{SG34865, &UA2750032},
	{SG34869, &UA2754266},
	{SG34874, &UA2206014},
	{SG34875, &UA2455079},
	{SG34876, &UA3065524},
	{SG34877, &UA3081276},
	{SG34880, &UA1960436},
	{SG34881, &UA1994693},
	{SG34882, &UA2365340},
	{SG34887, &UA3064879},
	{SG34889, &UA3080537},
	{SG34890, &UA2974495},
	{SG34891, &UA3021667},
	{SG34893, &UA2566533},
	{SG34895, &UA2668492},
	{SG34905, &UA2383044},
	{SG34907, &UA2233788},
	{SG34926, &UA2753327},
	{SG34929, &UA2974094},
	{SG34931, &UA2975530},
	{SG34933, &UA2973902},
	{SG34947, &UA2405233},
	{SG34949, &UA2566637},
	{SG34951, &UA2601153},
	{SG34957, &UA2415874},
	{SG34959, &UA2556835},
	{SG34967, &UA2030496},
	{SG34976, &UA2764566},
	{SG34997, &UA2012160},
	{SG34999, &UA2426219},
	{SG35002, &UA2981994},
	{SG35007, &UA1993043},
	{SG35010, &UA2027488},
	{SG35012, &UA2030054},
	{SG35015, &UA2775129},
	{SG35037, &UA3022142},
	{SG35039, &UA3022344},
	{SG35042, &UA2566757},
	{SG35044, &UA2569932},
	{SG35052, &UA454822},
	{SG35053, &UA647704},
	{SG35054, &UA887243},
	{SG35055, &UA2027930},
	{SG35057, &UA2031184},
	{SG35058, &UA2033199},
	{SG35067, &UA2360613},
	{SG35070, &UA2653852},
	{SG35098, &UA2391298},
	{SG35101, &UA2846899},
	{SG35104, &UA2566824},
	{SG35109, &UA2769382},
	{SG35111, &UA2798505},
	{SG35113, &UA2868348},
	{SG35123, &UA2257528},
	{SG35124, &UA2403661},
	{SG35126, &UA2427173},
	{SG35129, &UA2566503},
	{SG35132, &UA2009065},
	{SG35133, &UA2027365},
	{SG35134, &UA2033250},
	{SG35135, &UA2362531},
	{SG35136, &UA2466635},
	{SG35137, &UA2762326},
	{SG35140, &UA1991073},
	{SG35144, &UA2261464},
	{SG35147, &UA2353361},
	{SG35150, &UA2466953},
	{SG35152, &UA2467735},
	{SG35154, &UA2529577},
	{SG35156, &UA2532494},
	{SG35158, &UA2554076},
	{SG35160, &UA2659602},
	{SG35161, &UA2748163},
	{SG35162, &UA2355226},
	{SG35163, &UA2594277},
	{SG35164, &UA2695303},
	{SG35168, &UA2584864},
	{SG35170, &UA3023852},
	{SG35172, &UA2383793},
	{SG35173, &UA2727269},
	{SG35175, &UA2539371},
	{SG35176, &UA2855746},
	{SG35178, &UA2223191},
	{SG35179, &UA2480662},
	{SG35180, &UA2507799},
	{SG35181, &UA2556016},
	{SG35182, &UA2779897},
	{SG35184, &UA2853262},
	{SG35187, &UA3046244},
	{SG35188, &UA3049063},
	{SG35191, &UA2387505},
	{SG35193, &UA2654175},
	{SG35195, &UA2234356},
	{SG35197, &UA2365654},
	{SG35200, &UA2422230},
	{SG35202, &UA2738298},
	{SG35204, &UA2872447},
	{SG35206, &UA2754798},
	{SG35208, &UA2755738},
	{SG35211, &UA2754236},
	{SG35213, &UA2774077},
	{SG35217, &UA2755136},
	{SG35220, &UA2937879},
	{SG35227, &UA3065610},
	{SG35228, &UA2566952},
	{SG35229, &UA2583249},
	{SG35230, &UA2584860},
	{SG35231, &UA2702620},
	{SG35232, &UA2400970},
	{SG35233, &UA2734128},
	{SG35234, &UA2567020},
	{SG35235, &UA2583199},
	{SG35237, &UA2953568},
	{SG35238, &UA2867014},
	{SG35246, &UA608875},
	{SG35248, &UA899816},
	{SG35249, &UA906877},
	{SG35253, &UA2975803},
	{SG35254, &UA2566708},
	{SG35255, &UA2583371},
	{SG35256, &UA2975004},
	{SG35259, &UA2355582},
	{SG35260, &UA2367494},
	{SG35261, &UA2403756},
	{SG35262, &UA2426072},
	{SG35263, &UA2427663},
	{SG35264, &UA2462073},
	{SG35265, &UA2470419},
	{SG35266, &UA2486491},
	{SG35267, &UA2506876},
	{SG35269, &UA2530673},
	{SG35270, &UA2756013},
	{SG35271, &UA2784206},
	{SG35272, &UA2557083},
	{SG35273, &UA2042129},
	{SG35274, &UA2594245},
	{SG35275, &UA2651654},
	{SG35277, &UA2431856},
	{SG35278, &UA2625823},
	{SG35279, &UA2028211},
	{SG35280, &UA2648210},
	{SG35281, &UA2983392},
	{SG35282, &UA3065592},
	{SG35287, &UA2974455},
	{SG35288, &UA2975074},
	{SG35297, &UA1991165},
	{SG35298, &UA2028858},
	{SG35299, &UA2813151},
	{SG35300, &UA2225283},
	{SG35302, &UA2567945},
	{SG35303, &UA2583075},
	{SG35304, &UA2583564},
	{SG35305, &UA2602990},
	{SG35306, &UA2605923},
	{SG35307, &UA2666881},
	{SG35308, &UA2667035},
	{SG35309, &UA2667340},
	{SG35310, &UA2667523},
	{SG35311, &UA2732896},
	{SG35312, &UA2779940},
	{SG35313, &UA2793746},
	{SG35315, &UA2953758},
	{SG35336, &UA2204344},
	{SG35338, &UA2421971},
	{SG35341, &UA2566946},
	{SG35343, &UA2567137},
	{SG35367, &UA2670901},
	{SG35369, &UA2671049},
	{SG35370, &UA2671104},
	{SG35372, &UA2677186},
	{SG35375, &UA2567110},
	{SG35378, &UA2567256},
	{SG35380, &UA1994991},
	{SG35382, &UA2029662},
	{SG35383, &UA2676022},
	{SG35384, &UA2677293},
	{SG35385, &UA2707996},
	{SG35386, &UA2257380},
	{SG35388, &UA2577933},
	{SG35390, &UA606019},
	{SG35391, &UA615654},
	{SG35392, &UA2031380},
	{SG35393, &UA2877806},
	{SG35394, &UA2676253},
	{SG35395, &UA1995866},
	{SG35397, &UA3068324},
	{SG35405, &UA2753622},
	{SG35407, &UA2753878},
	{SG35408, &UA3110839},
	{SG35409, &UA2975744},
	{SG35411, &UA2040901},
	{SG35412, &UA2513341},
	{SG35415, &UA2947380},
	{SG35418, &UA3015445},
	{SG35421, &UA1993858},
	{SG35423, &UA2645613},
	{SG35424, &UA2822275},
	{SG35427, &UA2346366},
	{SG35429, &UA2423637},
	{SG35431, &UA2535020},
	{SG35432, &UA2578100},
	{SG35433, &UA2778708},
	{SG35435, &UA2879908},
	{SG35436, &UA2424687},
	{SG35543, &UA910040},
	{SG35546, &UA2355446},
	{SG35547, &UA2511262},
	{SG35551, &UA2566894},
	{SG35552, &UA2667052},
	{SG35553, &UA2566823},
	{SG35557, &UA2975941},
	{SG35559, &UA2975995},
	{SG35560, &UA2974900},
	{SG35562, &UA2348113},
	{SG35564, &UA2526193},
	{SG35565, &UA2834978},
	{SG35568, &UA2567090},
	{SG35572, &UA2222403},
	{SG35574, &UA2385376},
	{SG35578, &UA2203978},
	{SG35580, &UA3065571},
	{SG35587, &UA2567007},
	{SG35588, &UA2583779},
	{SG35594, &UA2975742},
	{SG35595, &UA3020969},
	{SG35597, &UA3022504},
	{SG35598, &UA3023385},
	{SG35599, &UA1959177},
	{SG35600, &UA2261499},
	{SG35601, &UA2975053},
	{SG35603, &UA3024117},
	{SG35604, &UA3065596},
	{SG35605, &UA3067053},
	{SG35606, &UA2483182},
	{SG35607, &UA2974925},
	{SG35608, &UA3023225},
	{SG35610, &UA2756016},
	{SG35615, &UA2390387},
	{SG35616, &UA2436691},
	{SG35618, &UA2510944},
	{SG35620, &UA2601706},
	{SG35621, &UA2936170},
	{SG35622, &UA3063081},
	{SG35623, &UA2673258},
	{SG35625, &UA2675076},
	{SG35627, &UA2677709},
	{SG35628, &UA2706873},
	{SG35629, &UA2707137},
	{SG35630, &UA2707325},
	{SG35631, &UA2707599},
	{SG35632, &UA2708045},
	{SG35633, &UA2708381},
	{SG35634, &UA2708720},
	{SG35635, &UA2710099},
	{SG35636, &UA2713325},
	{SG35637, &UA2736925},
	{SG35638, &UA2767867},
	{SG35639, &UA2676287},
	{SG35640, &UA2707588},
	{SG35642, &UA2403676},
	{SG35644, &UA2734880},
	{SG35646, &UA2867055},
	{SG35648, &UA2427863},
	{SG35656, &UA2957601},
	{SG35661, &UA2754815},
	{SG35662, &UA2755552},
	{SG35663, &UA2769979},
	{SG35664, &UA2796248},
	{SG35665, &UA2867072},
	{SG35667, &UA2867210},
	{SG35669, &UA2867744},
	{SG35670, &UA2867846},
	{SG35671, &UA2876353},
	{SG35672, &UA2877621},
	{SG35673, &UA2879434},
	{SG35674, &UA2892885},
	{SG35684, &UA2937816},
	{SG35686, &UA2753018},
	{SG35687, &UA2755070},
	{SG35688, &UA2868439},
	{SG35690, &UA2385061},
	{SG35691, &UA2594173},
	{SG35692, &UA2695731},
	{SG35698, &UA2707919},
	{SG35701, &UA2708355},
	{SG35702, &UA2768140},
	{SG35709, &UA911758},
	{SG35712, &UA2625412},
	{SG35718, &UA2202579},
	{SG35720, &UA1998748},
	{SG35721, &UA2404169},
	{SG35722, &UA2424207},
	{SG35723, &UA2482036},
	{SG35724, &UA2505036},
	{SG35725, &UA2526856},
	{SG35728, &UA2973477},
	{SG35731, &UA2028973},
	{SG35732, &UA2653803},
	{SG35734, &UA2813694},
	{SG35735, &UA2230819},
	{SG35736, &UA2367288},
	{SG35737, &UA2427639},
	{SG35738, &UA2754239},
	{SG35739, &UA2756010},
	{SG35756, &UA3021527},
	{SG35757, &UA2402969},
	{SG35758, &UA2695640},
	{SG35761, &UA2367430},
	{SG35762, &UA2367506},
	{SG35764, &UA2600291},
	{SG35766, &UA2668064},
	{SG35767, &UA2673051},
	{SG35770, &UA2956329},
	{SG35772, &UA925515},
	{SG35773, &UA2366388},
	{SG35774, &UA2384846},
	{SG35775, &UA2400260},
	{SG35776, &UA2566768},
	{SG35777, &UA2602009},
	{SG35781, &UA610135},
	{SG35783, &UA901587},
	{SG35785, &UA2044069},
	{SG35786, &UA2044087},
	{SG35787, &UA2044092},
	{SG35788, &UA2455879},
	{SG35789, &UA2031422},
	{SG35790, &UA2033050},
	{SG35791, &UA2240473},
	{SG35792, &UA2762390},
	{SG35793, &UA2173910},
	{SG35796, &UA2567119},
	{SG35799, &UA2891001},
	{SG35803, &UA2233627},
	{SG35804, &UA2234187},
	{SG35806, &UA2235619},
	{SG35811, &UA2203281},
	{SG35814, &UA3065466},
	{SG35819, &UA2228272},
	{SG35820, &UA2838519},
	{SG35821, &UA2368947},
	{SG35822, &UA2367959},
	{SG35823, &UA2983532},
	{SG35826, &UA2583140},
	{SG35844, &UA2961218},
	{SG35845, &UA3057700},
	{SG35849, &UA2425394},
	{SG35851, &UA2511261},
	{SG35861, &UA609630},
	{SG35863, &UA902817},
	{SG35868, &UA2566919},
	{SG35870, &UA2783657},
	{SG35871, &UA2799204},
	{SG35872, &UA2868008},
	{SG35876, &UA2859661},
	{SG35878, &UA2366882},
	{SG35881, &UA2754813},
	{SG35882, &UA2561767},
	{SG35887, &UA2204925},
	{SG35888, &UA2974316},
	{SG35889, &UA2391318},
	{SG35890, &UA2973487},
	{SG35891, &UA2974889},
	{SG35892, &UA3065369},
	{SG35893, &UA3081695},
	{SG35894, &UA3094718},
	{SG35895, &UA3110308},
	{SG35896, &UA1960633},
	{SG35897, &UA2029489},
	{SG35898, &UA2376120},
	{SG35899, &UA2734630},
	{SG35900, &UA2987715},
	{SG35901, &UA2988957},
	{SG35902, &UA2994044},
	{SG35903, &UA2998186},
	{SG35904, &UA3058152},
	{SG35905, &UA3101815},
	{SG35907, &UA3121476},
	{SG35909, &UA763544},
	{SG35910, &UA2030379},
	{SG35911, &UA2033605},
	{SG35912, &UA2555623},
	{SG35913, &UA2610760},
	{SG35914, &UA2614672},
	{SG35916, &UA2616001},
	{SG35917, &UA2616303},
	{SG35918, &UA2616809},
	{SG35919, &UA2617192},
	{SG35920, &UA2617193},
	{SG35921, &UA2618241},
	{SG35922, &UA2618748},
	{SG35924, &UA2619300},
	{SG35925, &UA2621360},
	{SG35926, &UA2622092},
	{SG35927, &UA2622408},
	{SG35928, &UA2625139},
	{SG35929, &UA2628305},
	{SG35930, &UA2629402},
	{SG35931, &UA2631040},
	{SG35932, &UA2637636},
	{SG35934, &UA2733518},
	{SG35935, &UA2739354},
	{SG35937, &UA3103962},
	{SG35938, &UA657001},
	{SG35939, &UA2030999},
	{SG35943, &UA2430326},
	{SG35944, &UA2625041},
	{SG35951, &UA3068316},
	{SG35953, &UA2974036},
	{SG35954, &UA2975546},
	{SG35955, &UA612774},
	{SG35956, &UA699823},
	{SG35957, &UA802477},
	{SG35958, &UA900122},
	{SG35959, &UA905650},
	{SG35961, &UA1990651},
	{SG35965, &UA2769477},
	{SG35968, &UA2770686},
	{SG35992, &UA2601193},
	{SG35993, &UA2601594},
	{SG35994, &UA2602439},
	{SG35995, &UA2606245},
	{SG35996, &UA2667633},
	{SG35997, &UA2667634},
	{SG35998, &UA2668199},
	{SG35999, &UA2668619},
	{SG36000, &UA2702481},
	{SG36001, &UA2703288},
	{SG36002, &UA2706412},
	{SG36005, &UA2708614},
	{SG36008, &UA2710634},
	{SG36010, &UA2732442},
	{SG36013, &UA2766759},
	{SG36014, &UA2766963},
	{SG36016, &UA2794869},
	{SG36017, &UA2856353},
	{SG36019, &UA2861424},
	{SG36021, &UA2938217},
	{SG36022, &UA2954102},
	{SG36023, &UA2958431},
	{SG36024, &UA2988715},
	{SG36025, &UA2990195},
	{SG36026, &UA3050541},
	{SG36027, &UA3050630},
	{SG36028, &UA3056644},
	{SG36030, &UA3086957},
	{SG36031, &UA3086970},
	{SG36033, &UA2458281},
	{SG36034, &UA2984317},
	{SG36035, &UA2750808},
	{SG36040, &UA2974359},
	{SG36043, &UA2567092},
	{SG36044, &UA2381293},
	{SG36046, &UA2754270},
	{SG36047, &UA2754637},
	{SG36048, &UA2583345},
	{SG36050, &UA3065738},
	{SG36051, &UA3067898},
	{SG36052, &UA2426085},
	{SG36053, &UA2459478},
	{SG36056, &UA2753617},
	{SG36057, &UA2769538},
	{SG36058, &UA2769872},
	{SG36059, &UA2869189},
	{SG36088, &UA2030987},
	{SG36090, &UA2261537},
	{SG36092, &UA2566881},
	{SG36093, &UA2583108},
	{SG36095, &UA2676426},
	{SG36098, &UA2676997},
	{SG36100, &UA1993192},
	{SG36101, &UA2551121},
	{SG36103, &UA2003757},
	{SG36105, &UA2010541},
	{SG36107, &UA2437397},
	{SG36108, &UA2437556},
	{SG36109, &UA2604043},
	{SG36110, &UA2670917},
	{SG36111, &UA2404198},
	{SG36112, &UA2951275},
	{SG36113, &UA2756012},
	{SG36115, &UA1986394},
	{SG36116, &UA2447883},
	{SG36117, &UA3083201},
	{SG36118, &UA3083552},
	{SG36119, &UA2191009},
	{SG36120, &UA2228506},
	{SG36121, &UA2200011},
	{SG36127, &UA2234946},
	{SG36128, &UA2480354},
	{SG36129, &UA3070687},
	{SG36130, &UA2257499},
	{SG36131, &UA605108},
	{SG36132, &UA666773},
	{SG36133, &UA2030990},
	{SG36134, &UA2754632},
	{SG36136, &UA2567194},
	{SG36137, &UA2795754},
	{SG36147, &UA2974596},
	{SG36150, &UA2567013},
	{SG36152, &UA2425237},
	{SG36154, &UA2844452},
	{SG36157, &UA1995862},
	{SG36180, &UA2752928},
	{SG36182, &UA2783084},
	{SG36185, &UA2671843},
	{SG36186, &UA2675388},
	{SG36188, &UA3088274},
	{SG36194, &UA1995190},
	{SG36196, &UA2689745},
	{SG36201, &UA2175408},
	{SG36207, &UA2206200},
	{SG36210, &UA2389492},
	{SG36211, &UA2404485},
	{SG36213, &UA2459117},
	{SG36214, &UA2561095},
	{SG36215, &UA2672866},
	{SG36216, &UA2673138},
	{SG36218, &UA2796629},
	{SG36219, &UA2872336},
	{SG36222, &UA2404414},
	{SG36223, &UA2458800},
	{SG36224, &UA2461123},
	{SG36225, &UA2461142},
	{SG36226, &UA2469382},
	{SG36227, &UA2508030},
	{SG36228, &UA2537043},
	{SG36229, &UA2555242},
	{SG36230, &UA2756642},
	{SG36231, &UA2867895},
	{SG36233, &UA753747},
	{SG36234, &UA796477},
	{SG36235, &UA1975622},
	{SG36236, &UA2025537},
	{SG36237, &UA2366733},
	{SG36238, &UA2566528},
	{SG36239, &UA2652615},
	{SG36240, &UA2425042},
	{SG36241, &UA2567694},
	{SG36255, &UA2457666},
	{SG36256, &UA2354420},
	{SG36257, &UA2413782},
	{SG36267, &UA2566955},
	{SG36269, &UA2567379},
	{SG36273, &UA2976119},
	{SG36277, &UA2566617},
	{SG36278, &UA2583570},
	{SG36287, &UA1977460},
	{SG36288, &UA2028535},
	{SG36311, &UA2031053},
	{SG36313, &UA3065023},
	{SG36315, &UA3068070},
	{SG36316, &UA3068254},
	{SG36330, &UA911293},
	{SG36331, &UA1995356},
	{SG36352, &UA907440},
	{SG36353, &UA2261604},
	{SG36354, &UA2381179},
	{SG36355, &UA2583322},
	{SG36356, &UA2667968},
	{SG36359, &UA2527975},
	{SG36361, &UA2974617},
	{SG36366, &UA2676236},
	{SG36367, &UA2734725},
	{SG36368, &UA2676034},
	{SG36369, &UA2676340},
	{SG36370, &UA2736091},
	{SG36373, &UA2458121},
	{SG36374, &UA2205123},
	{SG36375, &UA2257545},
	{SG36376, &UA2456387},
	{SG36377, &UA2838845},
	{SG36378, &UA2678419},
	{SG36379, &UA2864848},
	{SG36380, &UA2996921},
	{SG36383, &UA2734781},
	{SG36387, &UA1997678},
	{SG36388, &UA2644322},
	{SG36389, &UA2031775},
	{SG36390, &UA2550368},
	{SG36391, &UA1995177},
	{SG36392, &UA1995794},
	{SG36394, &UA2028701},
	{SG36395, &UA2820889},
	{SG36396, &UA2876921},
	{SG36397, &UA3061355},
	{SG36398, &UA3065744},
	{SG36401, &UA3083223},
	{SG36408, &UA2676277},
	{SG36410, &UA2676915},
	{SG36412, &UA2753102},
	{SG36414, &UA2231519},
	{SG36418, &UA2257487},
	{SG36420, &UA2386542},
	{SG36426, &UA2554377},
	{SG36427, &UA2950763},
	{SG36428, &UA2753278},
	{SG36433, &UA3020701},
	{SG36434, &UA3021146},
	{SG36436, &UA3021223},
	{SG36439, &UA3021573},
	{SG36440, &UA3021737},
	{SG36441, &UA3021741},
	{SG36442, &UA3021961},
	{SG36443, &UA3022047},
	{SG36444, &UA3022133},
	{SG36445, &UA3022404},
	{SG36446, &UA3022503},
	{SG36447, &UA3022624},
	{SG36448, &UA3023192},
	{SG36449, &UA3023236},
	{SG36450, &UA3023244},
	{SG36451, &UA3023241},
	{SG36452, &UA3023294},
	{SG36453, &UA3023434},
	{SG36455, &UA3023439},
	{SG36457, &UA3023591},
	{SG36459, &UA3023593},
	{SG36461, &UA3023585},
	{SG36462, &UA3023685},
	{SG36463, &UA3023835},
	{SG36464, &UA3024814},
	{SG36465, &UA3026074},
	{SG36466, &UA3057754},
	{SG36467, &UA3058110},
	{SG36468, &UA3059118},
	{SG36469, &UA3078259},
	{SG36470, &UA3092152},
	{SG36471, &UA3092311},
	{SG36474, &UA3108002},
	{SG36480, &UA2602798},
	{SG36481, &UA2507962},
	{SG36482, &UA2561203},
	{SG36483, &UA2780853},
	{SG36484, &UA2796385},
	{SG36485, &UA2672904},
	{SG36486, &UA2706646},
	{SG36487, &UA2708232},
	{SG36488, &UA2708376},
	{SG36489, &UA2708911},
	{SG36490, &UA2712065},
	{SG36491, &UA2712267},
	{SG36492, &UA2712277},
	{SG36493, &UA2712524},
	{SG36494, &UA2712532},
	{SG36496, &UA2734862},
	{SG36497, &UA2735229},
	{SG36499, &UA2735452},
	{SG36500, &UA2735569},
	{SG36501, &UA2736275},
	{SG36503, &UA2750883},
	{SG36505, &UA2750966},
	{SG36506, &UA2751729},
	{SG36507, &UA2768511},
	{SG36509, &UA2780798},
	{SG36511, &UA2860081},
	{SG36513, &UA2860733},
	{SG36514, &UA2861225},
	{SG36516, &UA2861924},
	{SG36517, &UA2862058},
	{SG36518, &UA2862815},
	{SG36519, &UA2864160},
	{SG36520, &UA2993454},
	{SG36521, &UA3048374},
	{SG36522, &UA3057734},
	{SG36523, &UA3114560},
	{SG36524, &UA3120070},
	{SG36525, &UA3126181},
	{SG36527, &UA2356798},
	{SG36528, &UA2368094},
	{SG36530, &UA2433723},
	{SG36532, &UA2471642},
	{SG36534, &UA2538048},
	{SG36536, &UA3100541},
	{SG36541, &UA2754502},
	{SG36542, &UA2755264},
	{SG36543, &UA1995020},
	{SG36544, &UA2381690},
	{SG36545, &UA2402445},
	{SG36546, &UA2354998},
	{SG36547, &UA2391344},
	{SG36548, &UA2404012},
	{SG36549, &UA2430474},
	{SG36550, &UA2436146},
	{SG36551, &UA2490328},
	{SG36554, &UA2536062},
	{SG36556, &UA2564035},
	{SG36558, &UA2582753},
	{SG36560, &UA2654507},
	{SG36562, &UA2676346},
	{SG36591, &UA2419637},
	{SG36592, &UA3070796},
	{SG36593, &UA3081351},
	{SG36594, &UA2505406},
	{SG36595, &UA2527853},
	{SG36596, &UA2979786},
	{SG36597, &UA608220},
	{SG36598, &UA866368},
	{SG36599, &UA2200754},
	{SG36600, &UA2333683},
	{SG36602, &UA2982736},
	{SG36604, &UA1970687},
	{SG36605, &UA2029681},
	{SG36606, &UA2413519},
	{SG36607, &UA2426605},
	{SG36608, &UA2452353},
	{SG36609, &UA2481358},
	{SG36610, &UA2483170},
	{SG36611, &UA2486495},
	{SG36612, &UA2556008},
	{SG36613, &UA2653516},
	{SG36614, &UA3043871},
	{SG36615, &UA2257538},
	{SG36616, &UA2481207},
	{SG36617, &UA2842040},
	{SG36618, &UA2676012},
	{SG36623, &UA2421066},
	{SG36624, &UA2464188},
	{SG36634, &UA2537751},
	{SG36637, &UA2188213},
	{SG36641, &UA2566976},
	{SG36643, &UA2583085},
	{SG36648, &UA2753871},
	{SG36650, &UA2975936},
	{SG36672, &UA2675567},
	{SG36673, &UA2737298},
	{SG36674, &UA2390417},
	{SG36675, &UA2695567},
	{SG36676, &UA2729864},
	{SG36677, &UA2951232},
	{SG36678, &UA3062321},
	{SG36684, &UA1966003},
	{SG36685, &UA1995655},
	{SG36686, &UA2564108},
	{SG36687, &UA2676595},
	{SG36688, &UA2750989},
	{SG36716, &UA2674745},
	{SG36718, &UA2676508},
	{SG36723, &UA2385903},
	{SG36724, &UA2401524},
	{SG36725, &UA2502155},
	{SG36726, &UA2731534},
	{SG36730, &UA607744},
	{SG36731, &UA2028961},
	{SG36732, &UA2031402},
	{SG36733, &UA609590},
	{SG36734, &UA811918},
	{SG36735, &UA904423},
	{SG36738, &UA2261523},
	{SG36739, &UA2361461},
	{SG36744, &UA3068320},
	{SG36759, &UA2404252},
	{SG36760, &UA2843402},
	{SG36761, &UA2510092},
	{SG36769, &UA2754332},
	{SG36770, &UA2405058},
	{SG36771, &UA1996944},
	{SG36772, &UA2204918},
	{SG36773, &UA2234344},
	{SG36774, &UA2426006},
	{SG36775, &UA2934396},
	{SG36776, &UA3032622},
	{SG36791, &UA2459501},
	{SG36793, &UA2483475},
	{SG36795, &UA2505427},
	{SG36797, &UA2728899},
	{SG36799, &UA2878297},
	{SG36800, &UA2982898},
	{SG36802, &UA2670756},
	{SG36804, &UA2678421},
	{SG36806, &UA3073088},
	{SG36809, &UA2974792},
	{SG36813, &UA2753869},
	{SG36816, &UA2676179},
	{SG36820, &UA3067597},
	{SG36825, &UA2754473},
	{SG36850, &UA2555984},
	{SG36854, &UA2753079},
	{SG36867, &UA3065365},
	{SG36880, &UA2582932},
	{SG36890, &UA2752934},
	{SG36892, &UA2753792},
	{SG36895, &UA3067915},
	{SG36896, &UA3109366},
	{SG36898, &UA2024047},
	{SG36901, &UA2527089},
	{SG36902, &UA2656216},
	{SG36903, &UA2656281},
	{SG36905, &UA2956132},
	{SG36910, &UA2042070},
	{SG36911, &UA2192096},
	{SG36912, &UA2352377},
	{SG36913, &UA2404142},
	{SG36914, &UA3043545},
	{SG36915, &UA3039427},
	{SG36916, &UA3107987},
	{SG36920, &UA2434135},
	{SG36922, &UA2702589},
	{SG36924, &UA2752332},
	{SG36927, &UA2957220},
	{SG36928, &UA2455726},
	{SG36931, &UA2555734},
	{SG36932, &UA2716520},
	{SG36933, &UA2762789},
	{SG36946, &UA3065143},
	{SG36953, &UA2749777},
	{SG36956, &UA2781731},
	{SG36958, &UA2992418},
	{SG36960, &UA2385446},
	{SG36962, &UA2427763},
	{SG36964, &UA2655041},
	{SG36972, &UA910814},
	{SG36974, &UA2031145},
	{SG36980, &UA2391587},
	{SG36981, &UA2482766},
	{SG36982, &UA2667859},
	{SG36983, &UA2936153},
	{SG36986, &UA2400900},
	{SG36988, &UA2418599},
	{SG36990, &UA2593949},
	{SG36991, &UA2594796},
	{SG36993, &UA2745324},
	{SG36996, &UA910213},
	{SG36997, &UA1996485},
	{SG36998, &UA2200310},
	{SG36999, &UA2437283},
	{SG37000, &UA2716819},
	{SG37001, &UA2837147},
	{SG37007, &UA2893002},
	{SG37021, &UA2031051},
	{SG37023, &UA2206198},
	{SG37024, &UA2390390},
	{SG37025, &UA2601667},
	{SG37027, &UA2673233},
	{SG37028, &UA2714636},
	{SG37032, &UA2676098},
	{SG37033, &UA609329},
	{SG37034, &UA748228},
	{SG37035, &UA910417},
	{SG37040, &UA2676926},
	{SG37053, &UA2457229},
	{SG37054, &UA2427049},
	{SG37055, &UA2427455},
	{SG37056, &UA2469528},
	{SG37058, &UA2677873},
	{SG37060, &UA3072953},
	{SG37062, &UA2027739},
	{SG37064, &UA2452360},
	{SG37066, &UA2973897},
	{SG37067, &UA3023544},
	{SG37068, &UA2028437},
	{SG37069, &UA2029061},
	{SG37070, &UA2231310},
	{SG37078, &UA2777321},
	{SG37079, &UA3065431},
	{SG37080, &UA3083477},
	{SG37081, &UA3093546},
	{SG37083, &UA3093982},
	{SG37084, &UA3094155},
	{SG37085, &UA3094714},
	{SG37086, &UA3095259},
	{SG37088, &UA3109121},
	{SG37090, &UA3109304},
	{SG37091, &UA3109931},
	{SG37092, &UA3109942},
	{SG37094, &UA3111034},
	{SG37095, &UA3111145},
	{SG37096, &UA3111853},
	{SG37097, &UA3114926},
	{SG37098, &UA3126304},
	{SG37099, &UA3126329},
	{SG37100, &UA3126403},
	{SG37101, &UA3126427},
	{SG37102, &UA3126574},
	{SG37103, &UA3126654},
	{SG37104, &UA3126821},
	{SG37105, &UA3126824},
	{SG37106, &UA3126872},
	{SG37118, &UA2365794},
	{SG37119, &UA2430892},
	{SG37120, &UA2383425},
	{SG37121, &UA666409},
	{SG37122, &UA680816},
	{SG37123, &UA761219},
	{SG37124, &UA1995831},
	{SG37125, &UA2028875},
	{SG37128, &UA2672511},
	{SG37129, &UA2674444},
	{SG37130, &UA2676419},
	{SG37132, &UA2735643},
	{SG37135, &UA2035282},
	{SG37136, &UA2423225},
	{SG37137, &UA2475927},
	{SG37138, &UA2481744},
	{SG37139, &UA2483745},
	{SG37140, &UA2497213},
	{SG37143, &UA2503361},
	{SG37144, &UA2528596},
	{SG37146, &UA2528961},
	{SG37148, &UA2653241},
	{SG37150, &UA2766062},
	{SG37152, &UA2841415},
	{SG37154, &UA3043781},
	{SG37156, &UA2566504},
	{SG37159, &UA2767397},
	{SG37160, &UA3047837},
	{SG37161, &UA2754708},
	{SG37162, &UA2261609},
	{SG37164, &UA2467676},
	{SG37171, &UA2755352},
	{SG37179, &UA1967642},
	{SG37180, &UA2686691},
	{SG37181, &UA1991469},
	{SG37182, &UA2441003},
	{SG37188, &UA2362529},
	{SG37189, &UA752857},
	{SG37190, &UA1981009},
	{SG37193, &UA1982006},
	{SG37194, &UA2024154},
	{SG37196, &UA2419567},
	{SG37197, &UA2419690},
	{SG37198, &UA2445355},
	{SG37199, &UA2525683},
	{SG37200, &UA2553220},
	{SG37201, &UA2649765},
	{SG37202, &UA2692686},
	{SG37203, &UA2692918},
	{SG37204, &UA2776280},
	{SG37205, &UA2195936},
	{SG37208, &UA2468007},
	{SG37211, &UA2766732},
	{SG37217, &UA1978755},
	{SG37219, &UA2005525},
	{SG37221, &UA2171847},
	{SG37222, &UA2897735},
	{SG37226, &UA1997509},
	{SG37227, &UA2599833},
	{SG37231, &UA3067048},
	{SG37239, &UA2218315},
	{SG37240, &UA2556308},
	{SG37241, &UA3043937},
	{SG37242, &UA2428735},
	{SG37244, &UA2428303},
	{SG37245, &UA2954753},
	{SG37246, &UA3118542},
	{SG37248, &UA2028582},
	{SG37249, &UA2596585},
	{SG37250, &UA2649754},
	{SG37251, &UA2975043},
	{SG37254, &UA2673083},
	{SG37255, &UA2676289},
	{SG37256, &UA3101702},
	{SG37263, &UA2755261},
	{SG37264, &UA899521},
	{SG37265, &UA2026645},
	{SG37266, &UA2368892},
	{SG37267, &UA2973270},
	{SG37268, &UA1996216},
	{SG37272, &UA2974079},
	{SG37274, &UA2713896},
	{SG37275, &UA2715432},
	{SG37276, &UA2752188},
	{SG37277, &UA2784711},
	{SG37278, &UA2796886},
	{SG37279, &UA2798857},
	{SG37283, &UA2365556},
	{SG37284, &UA2673872},
	{SG37286, &UA2709417},
	{SG37288, &UA2232575},
	{SG37289, &UA2763598},
	{SG37290, &UA2848131},
	{SG37291, &UA2257417},
	{SG37296, &UA2203960},
	{SG37297, &UA2426815},
	{SG37298, &UA2566279},
	{SG37299, &UA2784723},
	{SG37300, &UA3014178},
	{SG37302, &UA2717573},
	{SG37304, &UA2800685},
	{SG37308, &UA2029618},
	{SG37309, &UA2205958},
	{SG37310, &UA2420328},
	{SG37311, &UA2567347},
	{SG37314, &UA2754503},
	{SG37317, &UA2567237},
	{SG37318, &UA1977979},
	{SG37319, &UA2016159},
	{SG37326, &UA2034702},
	{SG37328, &UA2034738},
	{SG37330, &UA2388445},
	{SG37331, &UA2456926},
	{SG37333, &UA2843866},
	{SG37337, &UA2385830},
	{SG37339, &UA2595122},
	{SG37356, &UA2676422},
	{SG37357, &UA2734851},
	{SG37359, &UA2203009},
	{SG37360, &UA2538613},
	{SG37361, &UA2552744},
	{SG37363, &UA3066672},
	{SG37380, &UA2714934},
	{SG37382, &UA2938012},
	{SG37387, &UA2676431},
	{SG37389, &UA2676669},
	{SG37397, &UA3068330},
	{SG37398, &UA2567003},
	{SG37402, &UA2507732},
	{SG37404, &UA2706153},
	{SG37406, &UA2854159},
	{SG37408, &UA2676239},
	{SG37410, &UA2735309},
	{SG37414, &UA2753343},
	{SG37415, &UA2872991},
	{SG37416, &UA2595591},
	{SG37417, &UA2695055},
	{SG37421, &UA2701573},
	{SG37423, &UA2762585},
	{SG37425, &UA3065579},
	{SG37427, &UA3066539},
	{SG37438, &UA2673536},
	{SG37439, &UA2676271},
	{SG37444, &UA3066207},
	{SG37449, &UA2401474},
	{SG37450, &UA2951236},
	{SG37454, &UA2368733},
	{SG37455, &UA2975050},
	{SG37456, &UA2168140},
	{SG37457, &UA801414},
	{SG37458, &UA2026187},
	{SG37459, &UA2422382},
	{SG37462, &UA2764581},
	{SG37464, &UA830441},
	{SG37465, &UA2031141},
	{SG37466, &UA1951966},
	{SG37467, &UA1993537},
	{SG37473, &UA2415967},
	{SG37475, &UA2478024},
	{SG37476, &UA3086459},
	{SG37477, &UA2752962},
	{SG37478, &UA2773717},
	{SG37479, &UA2973271},
	{SG37480, &UA2975044},
	{SG37481, &UA3022555},
	{SG37483, &UA2780796},
	{SG37484, &UA2954795},
	{SG37487, &UA2676779},
	{SG37488, &UA2685840},
	{SG37490, &UA2230843},
	{SG37491, &UA2398569},
	{SG37492, &UA2418087},
	{SG37493, &UA2504336},
	{SG37494, &UA2579420},
	{SG37497, &UA2755359},
	{SG37498, &UA3066522},
	{SG37505, &UA2567124},
	{SG37506, &UA2236375},
	{SG37507, &UA2350817},
	{SG37508, &UA2466147},
	{SG37509, &UA2467950},
	{SG37510, &UA2653082},
	{SG37511, &UA2748222},
	{SG37512, &UA2778502},
	{SG37513, &UA2844244},
	{SG37514, &UA3100626},
	{SG37515, &UA609005},
	{SG37516, &UA673560},
	{SG37517, &UA2198884},
	{SG37522, &UA2560583},
	{SG37524, &UA2879879},
	{SG37525, &UA2512002},
	{SG37527, &UA3021775},
	{SG37534, &UA2351618},
	{SG37535, &UA3070703},
	{SG37540, &UA2974923},
	{SG37542, &UA3022984},
	{SG37547, &UA2730327},
	{SG37556, &UA2233620},
	{SG37557, &UA2233833},
	{SG37558, &UA2234690},
	{SG37559, &UA2232158},
	{SG37560, &UA2658231},
	{SG37561, &UA2738011},
	{SG37562, &UA2778691},
	{SG37563, &UA2782532},
	{SG37564, &UA3012895},
	{SG37565, &UA3022318},
	{SG37566, &UA3088709},
	{SG37567, &UA3125201},
	{SG37568, &UA3125212},
	{SG37570, &UA3125394},
	{SG37574, &UA2676351},
	{SG37575, &UA2736197},
	{SG37576, &UA2663176},
	{SG37577, &UA2941023},
	{SG37591, &UA3065613},
	{SG37593, &UA3110803},
	{SG37595, &UA2754823},
	{SG37601, &UA2468433},
	{SG37602, &UA2671907},
	{SG37603, &UA2772908},
	{SG37604, &UA2566915},
	{SG37605, &UA2754186},
	{SG37606, &UA2353421},
	{SG37607, &UA2655643},
	{SG37608, &UA2659242},
	{SG37609, &UA2659261},
	{SG37611, &UA2791244},
	{SG37612, &UA2986704},
	{SG37613, &UA2383421},
	{SG37614, &UA2423836},
	{SG37615, &UA2598714},
	{SG37616, &UA2429443},
	{SG37620, &UA2454886},
	{SG37621, &UA2455293},
	{SG37623, &UA2502039},
	{SG37624, &UA2530658},
	{SG37628, &UA2407364},
	{SG37630, &UA2754175},
	{SG37635, &UA1995059},
	{SG37645, &UA2975701},
	{SG37651, &UA2672189},
	{SG37652, &UA2677302},
	{SG37656, &UA2974894},
	{SG37660, &UA2676286},
	{SG37669, &UA2353037},
	{SG37677, &UA905525},
	{SG37679, &UA2027240},
	{SG37680, &UA2028550},
	{SG37682, &UA2759785},
	{SG37684, &UA905094},
	{SG37687, &UA2611471},
	{SG37689, &UA2612983},
	{SG37691, &UA2613294},
	{SG37692, &UA2617384},
	{SG37694, &UA2566697},
	{SG37697, &UA2673460},
	{SG37699, &UA3046667},
	{SG37701, &UA2567207},
	{SG37702, &UA2676675},
	{SG37704, &UA2204374},
	{SG37707, &UA2566922},
	{SG37708, &UA2583424},
	{SG37712, &UA2010505},
	{SG37714, &UA2528552},
	{SG37717, &UA2754060},
	{SG37718, &UA2672611},
	{SG37719, &UA2676021},
	{SG37720, &UA2973656},
	{SG37721, &UA2975005},
	{SG37723, &UA2566544},
	{SG37724, &UA2567507},
	{SG37725, &UA2755119},
	{SG37726, &UA2484657},
	{SG37727, &UA2549239},
	{SG37728, &UA2934565},
	{SG37729, &UA2997370},
	{SG37731, &UA2756009},
	{SG37732, &UA2796193},
	{SG37739, &UA2675379},
	{SG37740, &UA2362014},
	{SG37741, &UA2367902},
	{SG37742, &UA2430272},
	{SG37744, &UA2257904},
	{SG37746, &UA2427386},
	{SG37747, &UA2987117},
	{SG37749, &UA3068388},
	{SG37758, &UA2566890},
	{SG37761, &UA2584087},
	{SG37762, &UA2199132},
	{SG37763, &UA2974446},
	{SG37766, &UA2413756},
	{SG37767, &UA2353321},
	{SG37768, &UA2652904},
	{SG37769, &UA3080462},
	{SG37770, &UA3081080},
	{SG37787, &UA2676434},
	{SG37796, &UA2567082},
	{SG37798, &UA2670892},
	{SG37800, &UA2676632},
	{SG37801, &UA2748190},
	{SG37804, &UA909402},
	{SG37806, &UA2864678},
	{SG37816, &UA2457220},
	{SG37817, &UA2844782},
	{SG37818, &UA2434151},
	{SG37819, &UA2510306},
	{SG37820, &UA2674139},
	{SG37821, &UA2717027},
	{SG37826, &UA609203},
	{SG37827, &UA899333},
	{SG37828, &UA1953763},
	{SG37829, &UA1962515},
	{SG37830, &UA1966422},
	{SG37831, &UA2360555},
	{SG37832, &UA2548188},
	{SG37841, &UA2355808},
	{SG37842, &UA2366117},
	{SG37843, &UA2420789},
	{SG37844, &UA2430836},
	{SG37845, &UA2577699},
	{SG37849, &UA2261548},
	{SG37850, &UA2195895},
	{SG37851, &UA2232432},
	{SG37853, &UA2754243},
	{SG37854, &UA3068416},
	{SG37855, &UA3083824},
	{SG37858, &UA2028240},
	{SG37860, &UA2229451},
	{SG37862, &UA2231086},
	{SG37864, &UA2566917},
	{SG37865, &UA2234296},
	{SG37866, &UA2385542},
	{SG37867, &UA2563777},
	{SG37868, &UA2601085},
	{SG37869, &UA2777539},
	{SG37870, &UA3128378},
	{SG37871, &UA1955691},
	{SG37873, &UA2621577},
	{SG37875, &UA909645},
	{SG37877, &UA1995411},
	{SG37879, &UA2004627},
	{SG37880, &UA2017652},
	{SG37881, &UA2200626},
	{SG37882, &UA2511046},
	{SG37884, &UA2585335},
	{SG37886, &UA2756647},
	{SG37888, &UA3121287},
	{SG37893, &UA2426097},
	{SG37894, &UA2652916},
	{SG37895, &UA2695807},
	{SG37896, &UA2745452},
	{SG37914, &UA2934601},
	{SG37917, &UA2387556},
	{SG37934, &UA2193406},
	{SG37935, &UA2428662},
	{SG37936, &UA2229006},
	{SG37938, &UA2982837},
	{SG37939, &UA2676605},
	{SG37941, &UA2709408},
	{SG37944, &UA2673371},
	{SG37946, &UA2994955},
	{SG37954, &UA2567145},
	{SG37957, &UA2753162},
	{SG37958, &UA2754267},
	{SG37959, &UA2974621},
	{SG37961, &UA2734774},
	{SG37962, &UA3102865},
	{SG37966, &UA3067646},
	{SG37968, &UA2461081},
	{SG37969, &UA2974660},
	{SG37970, &UA2975703},
	{SG37973, &UA2028009},
	{SG37974, &UA2376075},
	{SG37975, &UA2415210},
	{SG37976, &UA2420645},
	{SG37977, &UA2526776},
	{SG37978, &UA2592074},
	{SG37980, &UA2777462},
	{SG37981, &UA2983631},
	{SG37984, &UA1996421},
	{SG37985, &UA2257814},
	{SG37986, &UA2686704},
	{SG37987, &UA3066542},
	{SG37991, &UA3064139},
	{SG37992, &UA3068322},
	{SG37995, &UA2480040},
	{SG37997, &UA2230465},
	{SG37998, &UA2257939},
	{SG38002, &UA2561585},
	{SG38003, &UA2663291},
	{SG38006, &UA2676837},
	{SG38007, &UA3023032},
	{SG38008, &UA2410683},
	{SG38009, &UA3024073},
	{SG38010, &UA3028921},
	{SG38011, &UA3061602},
	{SG38012, &UA3061607},
	{SG38013, &UA3062722},
	{SG38014, &UA3064903},
	{SG38027, &UA2842885},
	{SG38031, &UA2566927},
	{SG38032, &UA2567105},
	{SG38033, &UA2600977},
	{SG38034, &UA2666761},
	{SG38035, &UA2668825},
	{SG38036, &UA2703604},
	{SG38038, &UA2795104},
	{SG38040, &UA2863420},
	{SG38042, &UA2755372},
	{SG38043, &UA1958222},
	{SG38044, &UA2423517},
	{SG38045, &UA2759084},
	{SG38048, &UA2033522},
	{SG38049, &UA2527358},
	{SG38050, &UA2560843},
	{SG38051, &UA3050406},
	{SG38054, &UA2570303},
	{SG38055, &UA2774062},
	{SG38056, &UA2659497},
	{SG38057, &UA3044607},
	{SG38058, &UA1995948},
	{SG38059, &UA1997227},
	{SG38060, &UA2566302},
	{SG38066, &UA2676689},
	{SG38067, &UA2708970},
	{SG38068, &UA3082855},
	{SG38073, &UA1964922},
	{SG38074, &UA2407297},
	{SG38075, &UA2421752},
	{SG38076, &UA2512491},
	{SG38077, &UA2598649},
	{SG38080, &UA2566765},
	{SG38082, &UA909975},
	{SG38084, &UA2025350},
	{SG38086, &UA2030100},
	{SG38088, &UA2686221},
	{SG38090, &UA2816220},
	{SG38092, &UA2205640},
	{SG38093, &UA2228905},
	{SG38094, &UA2344552},
	{SG38095, &UA2499621},
	{SG38096, &UA2880817},
	{SG38097, &UA2973317},
	{SG38098, &UA3062684},
	{SG38099, &UA3111683},
	{SG38102, &UA2668138},
	{SG38106, &UA1995970},
	{SG38107, &UA2026709},
	{SG38108, &UA3080937},
	{SG38109, &UA3083146},
	{SG38110, &UA3083202},
	{SG38111, &UA3110305},
	{SG38122, &UA1996973},
	{SG38124, &UA2000844},
	{SG38126, &UA2362303},
	{SG38127, &UA2403477},
	{SG38128, &UA2566241},
	{SG38129, &UA2706381},
	{SG38130, &UA2237489},
	{SG38131, &UA2457854},
	{SG38132, &UA2754162},
	{SG38133, &UA2426559},
	{SG38136, &UA2753408},
	{SG38137, &UA2754088},
	{SG38149, &UA2753339},
	{SG38150, &UA2755081},
	{SG38152, &UA2755520},
	{SG38154, &UA2428305},
	{SG38157, &UA2535431},
	{SG38158, &UA2794544},
	{SG38160, &UA2845662},
	{SG38161, &UA2567252},
	{SG38162, &UA3068204},
	{SG38163, &UA3094918},
	{SG38164, &UA2795005},
	{SG38165, &UA2676186},
	{SG38166, &UA2753963},
	{SG38167, &UA2368164},
	{SG38168, &UA2504473},
	{SG38169, &UA2653022},
	{SG38172, &UA2662837},
	{SG38173, &UA2773854},
	{SG38176, &UA2676692},
	{SG38179, &UA924237},
	{SG38180, &UA2355891},
	{SG38185, &UA2975985},
	{SG38188, &UA2397241},
	{SG38189, &UA2567128},
	{SG38193, &UA2974498},
	{SG38196, &UA2192507},
	{SG38203, &UA2236938},
	{SG38205, &UA2350761},
	{SG38206, &UA2789659},
	{SG38208, &UA2567012},
	{SG38215, &UA2362301},
	{SG38216, &UA2705356},
	{SG38217, &UA3047047},
	{SG38218, &UA3099818},
	{SG38221, &UA2566925},
	{SG38222, &UA2676851},
	{SG38223, &UA2781894},
	{SG38228, &UA2484309},
	{SG38229, &UA2506828},
	{SG38231, &UA2558496},
	{SG38232, &UA2658897},
	{SG38233, &UA2764629},
	{SG38235, &UA3065094},
	{SG38237, &UA2431853},
	{SG38240, &UA2626013},
	{SG38242, &UA2428175},
	{SG38243, &UA3099693},
	{SG38251, &UA2383513},
	{SG38252, &UA2432780},
	{SG38254, &UA3118551},
	{SG38256, &UA3067598},
	{SG38263, &UA2461327},
	{SG38264, &UA2486121},
	{SG38281, &UA3067188},
	{SG38282, &UA2974608},
	{SG38288, &UA2361100},
	{SG38289, &UA2996805},
	{SG38290, &UA2754337},
	{SG38293, &UA1995454},
	{SG38295, &UA2722715},
	{SG38297, &UA2975997},
	{SG38299, &UA2600582},
	{SG38300, &UA2669029},
	{SG38302, &UA2728485},
	{SG38309, &UA2404761},
	{SG38310, &UA2433956},
	{SG38311, &UA2537646},
	{SG38314, &UA2677124},
	{SG38316, &UA2773907},
	{SG38317, &UA3023579},
	{SG38319, &UA899754},
	{SG38321, &UA2186922},
	{SG38323, &UA2325429},
	{SG38324, &UA2427801},
	{SG38325, &UA907783},
	{SG38326, &UA2023533},
	{SG38327, &UA2229196},
	{SG38333, &UA2567381},
	{SG38335, &UA2605945},
	{SG38342, &UA2403121},
	{SG38343, &UA2487227},
	{SG38344, &UA2606767},
	{SG38345, &UA2754352},
	{SG38346, &UA2403468},
	{SG38348, &UA3094159},
	{SG38362, &UA2490447},
	{SG38368, &UA1993631},
	{SG38369, &UA2670757},
	{SG38370, &UA2754721},
	{SG38371, &UA2486440},
	{SG38372, &UA2666818},
	{SG38376, &UA2183559},
	{SG38385, &UA2257550},
	{SG38386, &UA2557643},
	{SG38387, &UA3046323},
	{SG38388, &UA2975550},
	{SG38389, &UA3066531},
	{SG38390, &UA2485542},
	{SG38405, &UA2257390},
	{SG38406, &UA2566891},
	{SG38408, &UA2568397},
	{SG38409, &UA2422458},
	{SG38410, &UA2778140},
	{SG38424, &UA2235623},
	{SG38425, &UA2833533},
	{SG38426, &UA2567238},
	{SG38428, &UA606528},
	{SG38429, &UA640245},
	{SG38430, &UA2032887},
	{SG38431, &UA2457934},
	{SG38433, &UA2529566},
	{SG38434, &UA2791249},
	{SG38437, &UA3068406},
	{SG38439, &UA2261541},
	{SG38441, &UA2354120},
	{SG38442, &UA2367204},
	{SG38446, &UA1990784},
	{SG38447, &UA2030988},
	{SG38450, &UA2976006},
	{SG38458, &UA2673957},
	{SG38477, &UA2567089},
	{SG38479, &UA2973273},
	{SG38481, &UA2356500},
	{SG38484, &UA1961454},
	{SG38487, &UA2421086},
	{SG38488, &UA2985108},
	{SG38489, &UA2235091},
	{SG38495, &UA2975544},
	{SG38501, &UA2754351},
	{SG38502, &UA2442402},
	{SG38504, &UA2567500},
	{SG38511, &UA2365802},
	{SG38513, &UA2466048},
	{SG38515, &UA2577979},
	{SG38518, &UA2654733},
	{SG38524, &UA791437},
	{SG38525, &UA2773255},
	{SG38526, &UA2782826},
	{SG38527, &UA2784825},
	{SG38529, &UA2258167},
	{SG38530, &UA3068333},
	{SG38531, &UA717337},
	{SG38532, &UA828925},
	{SG38533, &UA909547},
	{SG38534, &UA1968459},
	{SG38535, &UA2026143},
	{SG38538, &UA2973473},
	{SG38539, &UA3022646},
	{SG38543, &UA2567262},
	{SG38548, &UA846888},
	{SG38549, &UA2183965},
	{SG38550, &UA2399100},
	{SG38551, &UA2695891},
	{SG38552, &UA2727436},
	{SG38555, &UA2466852},
	{SG38557, &UA2697315},
	{SG38559, &UA2844270},
	{SG38563, &UA3068205},
	{SG38570, &UA1988767},
	{SG38571, &UA2025279},
	{SG38572, &UA2545726},
	{SG38576, &UA2483385},
	{SG38578, &UA3080933},
	{SG38579, &UA2975851},
	{SG38580, &UA3068409},
	{SG38583, &UA2375668},
	{SG38584, &UA2385263},
	{SG38585, &UA2385714},
	{SG38586, &UA2428462},
	{SG38587, &UA2528370},
	{SG38592, &UA2030771},
	{SG38598, &UA2261501},
	{SG38600, &UA2461163},
	{SG38603, &UA2528585},
	{SG38604, &UA2988684},
	{SG38605, &UA2566508},
	{SG38607, &UA2975748},
	{SG38610, &UA2594980},
	{SG38612, &UA3080591},
	{SG38613, &UA2257806},
	{SG38614, &UA2973652},
	{SG38615, &UA2974638},
	{SG38621, &UA609372},
	{SG38623, &UA896770},
	{SG38626, &UA910968},
	{SG38629, &UA2676500},
	{SG38636, &UA2405785},
	{SG38637, &UA2844890},
	{SG38639, &UA2974240},
	{SG38650, &UA2376933},
	{SG38656, &UA2662404},
	{SG38657, &UA3081685},
	{SG38661, &UA2467412},
	{SG38662, &UA2579337},
	{SG38663, &UA2843925},
	{SG38667, &UA2948004},
	{SG38669, &UA2965855},
	{SG38670, &UA3059057},
	{SG38674, &UA2033105},
	{SG38676, &UA2419512},
	{SG38677, &UA2387440},
	{SG38680, &UA2974899},
	{SG38682, &UA2975092},
	{SG38683, &UA3020851},
	{SG38685, &UA3021958},
	{SG38687, &UA3023099},
	{SG38695, &UA2171666},
	{SG38702, &UA2504048},
	{SG38704, &UA2753676},
	{SG38710, &UA2389088},
	{SG38712, &UA2843567},
	{SG38713, &UA3068405},
	{SG38716, &UA2673346},
	{SG38718, &UA2708262},
	{SG38725, &UA2566904},
	{SG38727, &UA2676843},
	{SG38729, &UA2754377},
	{SG38732, &UA2194699},
	{SG38741, &UA2567370},
	{SG38757, &UA2508317},
	{SG38760, &UA2508655},
	{SG38762, &UA3021582},
	{SG38764, &UA3092219},
	{SG38766, &UA2676820},
	{SG38771, &UA2228221},
	{SG38774, &UA2420747},
	{SG38775, &UA2027478},
	{SG38777, &UA2625666},
	{SG38779, &UA2223088},
	{SG38781, &UA2577148},
	{SG38790, &UA893633},
	{SG38794, &UA2198517},
	{SG38797, &UA3064985},
	{SG38801, &UA2750228},
	{SG38807, &UA2973966},
	{SG38808, &UA3021247},
	{SG38809, &UA2974090},
	{SG38811, &UA609373},
	{SG38812, &UA899822},
	{SG38813, &UA908579},
	{SG38814, &UA2029259},
	{SG38815, &UA2641318},
	{SG38816, &UA904495},
	{SG38820, &UA2594550},
	{SG38824, &UA3073435},
	{SG38837, &UA2756671},
	{SG38838, &UA2388653},
	{SG38839, &UA876537},
	{SG38840, &UA911442},
	{SG38841, &UA2528178},
	{SG38842, &UA2528357},
	{SG38844, &UA1993859},
	{SG38848, &UA2435616},
	{SG38851, &UA2676238},
	{SG38852, &UA2171189},
	{SG38853, &UA2396430},
	{SG38858, &UA2897822},
	{SG38859, &UA3056939},
	{SG38861, &UA3025151},
	{SG38863, &UA3028940},
	{SG38868, &UA2975069},
	{SG38869, &UA2601053},
	{SG38870, &UA2782265},
	{SG38871, &UA2988535},
	{SG38872, &UA2996764},
	{SG38873, &UA2975548},
	{SG38874, &UA3020716},
	{SG38890, &UA2425992},
	{SG38891, &UA2427638},
	{SG38892, &UA2427925},
	{SG38893, &UA2577961},
	{SG38896, &UA912468},
	{SG38898, &UA2405535},
	{SG38902, &UA2029709},
	{SG38903, &UA2562118},
	{SG38904, &UA2566905},
	{SG38905, &UA2567961},
	{SG38906, &UA2671821},
	{SG38907, &UA2693309},
	{SG38916, &UA2261515},
	{SG38918, &UA2421584},
	{SG38919, &UA2671093},
	{SG38920, &UA2677178},
	{SG38921, &UA2947546},
	{SG38922, &UA2030167},
	{SG38923, &UA2420679},
	{SG38925, &UA2353589},
	{SG38926, &UA2420940},
	{SG38927, &UA2950692},
	{SG38929, &UA810041},
	{SG38931, &UA908416},
	{SG38933, &UA2425504},
	{SG38934, &UA2632918},
	{SG38935, &UA2633897},
	{SG38936, &UA2773894},
	{SG38938, &UA2221154},
	{SG38939, &UA2529254},
	{SG38940, &UA2559258},
	{SG38941, &UA2699573},
	{SG38942, &UA2937223},
	{SG38943, &UA2755086},
	{SG38946, &UA2180804},
	{SG38948, &UA2362616},
	{SG38950, &UA2374154},
	{SG38953, &UA3022718},
	{SG38954, &UA924154},
	{SG38956, &UA2389052},
	{SG38958, &UA2566950},
	{SG38964, &UA2754073},
	{SG38965, &UA2771264},
	{SG38967, &UA1987168},
	{SG38968, &UA2160157},
	{SG38969, &UA2479646},
	{SG38970, &UA2430722},
	{SG38971, &UA3066541},
	{SG38972, &UA2029351},
	{SG38973, &UA2232467},
	{SG38974, &UA2426990},
	{SG38976, &UA2437631},
	{SG38981, &UA2974277},
	{SG38988, &UA2657531},
	{SG38990, &UA2420520},
	{SG38991, &UA2528261},
	{SG38992, &UA2655761},
	{SG38993, &UA2664518},
	{SG38994, &UA2846480},
	{SG38995, &UA2955740},
	{SG38998, &UA2010149},
	{SG38999, &UA2360276},
	{SG39000, &UA2760367},
	{SG39001, &UA2981478},
	{SG39003, &UA2206705},
	{SG39004, &UA2402095},
	{SG39005, &UA2422116},
	{SG39006, &UA3082808},
	{SG39007, &UA3067291},
	{SG39008, &UA2975792},
	{SG39010, &UA2974323},
	{SG39025, &UA2704458},
	{SG39026, &UA3081617},
	{SG39027, &UA3083654},
	{SG39028, &UA3132853},
	{SG39029, &UA1958147},
	{SG39030, &UA2227817},
	{SG39031, &UA2351476},
	{SG39032, &UA2353461},
	{SG39033, &UA2650108},
	{SG39038, &UA2567317},
	{SG39039, &UA2754223},
	{SG39043, &UA2566893},
	{SG39044, &UA2428575},
	{SG39045, &UA2467512},
	{SG39046, &UA2529205},
	{SG39047, &UA2565260},
	{SG39048, &UA2460826},
	{SG39049, &UA2242178},
	{SG39050, &UA2672675},
	{SG39058, &UA2566817},
	{SG39060, &UA2974463},
	{SG39062, &UA2566484},
	{SG39067, &UA2676928},
	{SG39068, &UA2863809},
	{SG39071, &UA2507827},
	{SG39074, &UA1997107},
	{SG39075, &UA2427127},
	{SG39078, &UA2419071},
	{SG39079, &UA2677283},
	{SG39080, &UA1995320},
	{SG39082, &UA2179923},
	{SG39083, &UA2546496},
	{SG39086, &UA2028527},
	{SG39088, &UA2223252},
	{SG39089, &UA2229398},
	{SG39091, &UA2230489},
	{SG39092, &UA2355430},
	{SG39093, &UA2427255},
	{SG39094, &UA2752829},
	{SG39097, &UA3068050},
	{SG39099, &UA1984434},
	{SG39100, &UA1993596},
	{SG39101, &UA1996062},
	{SG39102, &UA2007445},
	{SG39103, &UA2582990},
	{SG39105, &UA2670726},
	{SG39113, &UA2029834},
	{SG39115, &UA2400403},
	{SG39116, &UA2466160},
	{SG39118, &UA2528009},
	{SG39120, &UA2747420},
	{SG39128, &UA2485257},
	{SG39130, &UA3081026},
	{SG39133, &UA2624640},
	{SG39136, &UA2556942},
	{SG39138, &UA2666824},
	{SG39139, &UA2699515},
	{SG39142, &UA2699663},
	{SG39144, &UA2856462},
	{SG39145, &UA3015165},
	{SG39146, &UA3050307},
	{SG39147, &UA3071439},
	{SG39148, &UA3102100},
	{SG39149, &UA2602922},
	{SG39150, &UA2864975},
	{SG39151, &UA1956614},
	{SG39152, &UA3068391},
	{SG39153, &UA3094787},
	{SG39154, &UA3126216},
	{SG39160, &UA2467640},
	{SG39161, &UA2566693},
	{SG39162, &UA2634390},
	{SG39164, &UA2975710},
	{SG39165, &UA2567205},
	{SG39166, &UA2353592},
	{SG39167, &UA2659120},
	{SG39168, &UA2257664},
	{SG39169, &UA2671832},
	{SG39170, &UA2973478},
	{SG39171, &UA715654},
	{SG39172, &UA845974},
	{SG39173, &UA2026768},
	{SG39174, &UA2341839},
	{SG39175, &UA2469299},
	{SG39177, &UA2756004},
	{SG39181, &UA2564028},
	{SG39182, &UA874423},
	{SG39183, &UA908787},
	{SG39184, &UA2026242},
	{SG39189, &UA728039},
	{SG39190, &UA904297},
	{SG39192, &UA2025155},
	{SG39194, &UA2893617},
	{SG39198, &UA2480029},
	{SG39199, &UA2819244},
	{SG39200, &UA2406816},
	{SG39201, &UA2753111},
	{SG39202, &UA2257700},
	{SG39203, &UA2760725},
	{SG39204, &UA2363174},
	{SG39206, &UA2483763},
	{SG39207, &UA2676826},
	{SG39208, &UA2714742},
	{SG39211, &UA2404690},
	{SG39212, &UA2581584},
	{SG39213, &UA2754806},
	{SG39223, &UA3081280},
	{SG39227, &UA2896604},
	{SG39228, &UA2899128},
	{SG39229, &UA2901208},
	{SG39230, &UA2944039},
	{SG39231, &UA2944994},
	{SG39232, &UA2963260},
	{SG39238, &UA2672574},
	{SG39240, &UA2712777},
	{SG39243, &UA2976117},
	{SG39248, &UA2567122},
	{SG39250, &UA2889313},
	{SG39252, &UA2531751},
	{SG39255, &UA2738256},
	{SG39256, &UA2974045},
	{SG39257, &UA1967995},
	{SG39259, &UA2427584},
	{SG39262, &UA2257627},
	{SG39267, &UA2459465},
	{SG39268, &UA3068417},
	{SG39269, &UA3065572},
	{SG39276, &UA2185590},
	{SG39278, &UA2380553},
	{SG39280, &UA2485507},
	{SG39290, &UA2387530},
	{SG39303, &UA2854276},
	{SG39308, &UA2351381},
	{SG39310, &UA2829947},
	{SG39311, &UA3042982},
	{SG39312, &UA2423265},
	{SG39313, &UA2553522},
	{SG39314, &UA3099291},
	{SG39323, &UA2952737},
	{SG39325, &UA3068221},
	{SG39328, &UA2752953},
	{SG39330, &UA3066526},
	{SG39334, &UA2975080},
	{SG39336, &UA2975056},
	{SG39337, &UA2975097},
	{SG39339, &UA2583291},
	{SG39340, &UA2673245},
	{SG39343, &UA2709092},
	{SG39346, &UA2861860},
	{SG39356, &UA2033391},
	{SG39357, &UA2366976},
	{SG39358, &UA2455964},
	{SG39362, &UA2425730},
	{SG39363, &UA2654903},
	{SG39365, &UA2531143},
	{SG39366, &UA2781720},
	{SG39367, &UA2863514},
	{SG39371, &UA3064757},
	{SG39400, &UA2753957},
	{SG39401, &UA3080913},
	{SG39402, &UA3083746},
	{SG39404, &UA3095523},
	{SG39405, &UA2566642},
	{SG39407, &UA2257903},
	{SG39409, &UA2754237},
	{SG39415, &UA2508645},
	{SG39416, &UA3023558},
	{SG39418, &UA2671143},
	{SG39419, &UA2755278},
	{SG39420, &UA2958134},
	{SG39421, &UA2567371},
	{SG39422, &UA3080327},
	{SG39423, &UA3081231},
	{SG39424, &UA2437103},
	{SG39425, &UA2433382},
	{SG39426, &UA2738157},
	{SG39427, &UA2676230},
	{SG39428, &UA3080980},
	{SG39438, &UA2566767},
	{SG39439, &UA2456966},
	{SG39442, &UA2755122},
	{SG39444, &UA3065601},
	{SG39445, &UA904131},
	{SG39447, &UA2027427},
	{SG39455, &UA2467644},
	{SG39457, &UA3118638},
	{SG39465, &UA2974100},
	{SG39466, &UA3066874},
	{SG39467, &UA3110804},
	{SG39471, &UA2007356},
	{SG39479, &UA2028939},
	{SG39481, &UA2257855},
	{SG39482, &UA2676941},
	{SG39488, &UA2362159},
	{SG39490, &UA2745838},
	{SG39492, &UA2838979},
	{SG39498, &UA2755287},
	{SG39499, &UA2769575},
	{SG39501, &UA2770423},
	{SG39503, &UA2866979},
	{SG39505, &UA2893428},
	{SG39506, &UA3081088},
	{SG39509, &UA2404958},
	{SG39510, &UA2942455},
	{SG39535, &UA2596273},
	{SG39536, &UA3101788},
	{SG39537, &UA2782623},
	{SG39539, &UA2867075},
	{SG39547, &UA2366667},
	{SG39549, &UA2594957},
	{SG39550, &UA2755549},
	{SG39552, &UA2769361},
	{SG39554, &UA2756940},
	{SG39557, &UA2406595},
	{SG39559, &UA2425120},
	{SG39560, &UA2426960},
	{SG39565, &UA2033516},
	{SG39566, &UA2475506},
	{SG39569, &UA2231885},
	{SG39570, &UA2261530},
	{SG39571, &UA2455710},
	{SG39572, &UA2465778},
	{SG39574, &UA2452698},
	{SG39575, &UA2466881},
	{SG39576, &UA2664386},
	{SG39577, &UA2937666},
	{SG39578, &UA3088254},
	{SG39579, &UA2567316},
	{SG39581, &UA2676264},
	{SG39582, &UA2181710},
	{SG39584, &UA3033250},
	{SG39585, &UA3061283},
	{SG39586, &UA3062541},
	{SG39587, &UA3080415},
	{SG39588, &UA3081087},
	{SG39590, &UA3083283},
	{SG39606, &UA2506766},
	{SG39613, &UA2400996},
	{SG39622, &UA1958983},
	{SG39624, &UA2456560},
	{SG39625, &UA2503161},
	{SG39627, &UA2580157},
	{SG39630, &UA2690792},
	{SG39632, &UA2768701},
	{SG39634, &UA2791285},
	{SG39635, &UA2845890},
	{SG39636, &UA2964517},
	{SG39637, &UA3099464},
	{SG39639, &UA2976016},
	{SG39642, &UA2033038},
	{SG39646, &UA3030379},
	{SG39651, &UA2206712},
	{SG39652, &UA2206716},
	{SG39653, &UA2676447},
	{SG39657, &UA2231091},
	{SG39660, &UA2465154},
	{SG39668, &UA2257478},
	{SG39669, &UA2239210},
	{SG39670, &UA2567206},
	{SG39671, &UA2566762},
	{SG39673, &UA2703359},
	{SG39691, &UA607771},
	{SG39693, &UA754886},
	{SG39696, &UA904971},
	{SG39698, &UA3043432},
	{SG39715, &UA2753324},
	{SG39717, &UA2783603},
	{SG39720, &UA3081607},
	{SG39724, &UA2338148},
	{SG39726, &UA2567244},
	{SG39729, &UA2714347},
	{SG39731, &UA2782193},
	{SG39733, &UA1990295},
	{SG39735, &UA2257652},
	{SG39736, &UA2335974},
	{SG39739, &UA2529880},
	{SG39756, &UA2030561},
	{SG39760, &UA2400096},
	{SG39764, &UA2712198},
	{SG39765, &UA2751508},
	{SG39766, &UA2431077},
	{SG39770, &UA2432555},
	{SG39771, &UA2472076},
	{SG39772, &UA2702564},
	{SG39773, &UA2708224},
	{SG39774, &UA2754961},
	{SG39775, &UA2798943},
	{SG39777, &UA794921},
	{SG39779, &UA905719},
	{SG39782, &UA2204135},
	{SG39783, &UA2362132},
	{SG39784, &UA2367236},
	{SG39786, &UA2658208},
	{SG39788, &UA2567331},
	{SG39789, &UA2666989},
	{SG39790, &UA2794699},
	{SG39791, &UA2755255},
	{SG39792, &UA2029066},
	{SG39793, &UA2671478},
	{SG39794, &UA2674205},
	{SG39795, &UA2676507},
	{SG39796, &UA2735315},
	{SG39805, &UA2222531},
	{SG39806, &UA3070290},
	{SG39809, &UA2566942},
	{SG39811, &UA2753011},
	{SG39812, &UA2756738},
	{SG39813, &UA2872916},
	{SG39814, &UA3079819},
	{SG39815, &UA2470259},
	{SG39816, &UA2755432},
	{SG39817, &UA2769844},
	{SG39820, &UA2975741},
	{SG39821, &UA3021375},
	{SG39823, &UA2774156},
	{SG39825, &UA3050252},
	{SG39827, &UA2388550},
	{SG39830, &UA2562875},
	{SG39831, &UA2974038},
	{SG39833, &UA2974730},
	{SG39835, &UA2198885},
	{SG39836, &UA2338459},
	{SG39837, &UA2429923},
	{SG39838, &UA2430986},
	{SG39839, &UA2992202},
	{SG39840, &UA2529868},
	{SG39841, &UA2746524},
	{SG39843, &UA2778910},
	{SG39844, &UA2988037},
	{SG39850, &UA2385681},
	{SG39856, &UA2975953},
	{SG39858, &UA3022836},
	{SG39862, &UA2734846},
	{SG39863, &UA2566930},
	{SG39865, &UA2754716},
	{SG39866, &UA2365869},
	{SG39867, &UA2754350},
	{SG39870, &UA2976696},
	{SG39872, &UA3083292},
	{SG39880, &UA766699},
	{SG39890, &UA2257396},
	{SG39892, &UA2718786},
	{SG39893, &UA2738301},
	{SG39896, &UA2996940},
	{SG39901, &UA2580198},
	{SG39906, &UA2227320},
	{SG39907, &UA2237775},
	{SG39908, &UA2201432},
	{SG39909, &UA2468057},
	{SG39910, &UA2469420},
	{SG39911, &UA2485629},
	{SG39913, &UA2603741},
	{SG39915, &UA2583235},
	{SG39916, &UA2950864},
	{SG39922, &UA2226928},
	{SG39923, &UA2974727},
	{SG39925, &UA605534},
	{SG39927, &UA889084},
	{SG39929, &UA2344892},
	{SG39938, &UA2239222},
	{SG39940, &UA2754063},
	{SG39941, &UA2566487},
	{SG39942, &UA2754937},
	{SG39943, &UA3081482},
	{SG39945, &UA608795},
	{SG39947, &UA629719},
	{SG39948, &UA907922},
	{SG39949, &UA2610249},
	{SG39950, &UA2616424},
	{SG39951, &UA2523843},
	{SG39953, &UA2755332},
	{SG39957, &UA1990573},
	{SG39958, &UA2655269},
	{SG39966, &UA2028255},
	{SG39967, &UA2166961},
	{SG39968, &UA2833771},
	{SG39970, &UA2670622},
	{SG39975, &UA2487075},
	{SG39976, &UA2756017},
	{SG39977, &UA2567957},
	{SG39978, &UA2676821},
	{SG39979, &UA2257665},
	{SG39981, &UA2672197},
	{SG39984, &UA2973465},
	{SG39985, &UA1993314},
	{SG39988, &UA2461248},
	{SG39989, &UA2462048},
	{SG39990, &UA2427299},
	{SG39991, &UA2432456},
	{SG39994, &UA2975721},
	{SG39996, &UA3021879},
	{SG39998, &UA3020296},
	{SG40000, &UA3021096},
	{SG40001, &UA3022016},
	{SG40002, &UA3022260},
	{SG40004, &UA3023380},
	{SG40006, &UA1965032},
	{SG40007, &UA2261543},
	{SG40008, &UA2383353},
	{SG40009, &UA2422095},
	{SG40010, &UA2763006},
	{SG40011, &UA2755410},
	{SG40012, &UA1995436},
	{SG40013, &UA2031385},
	{SG40015, &UA2753045},
	{SG40016, &UA2942036},
	{SG40017, &UA2964999},
	{SG40018, &UA3059701},
	{SG40019, &UA2360169},
	{SG40021, &UA2382085},
	{SG40022, &UA2505881},
	{SG40023, &UA2529525},
	{SG40024, &UA2974457},
	{SG40025, &UA2974479},
	{SG40033, &UA1980350},
	{SG40034, &UA2367136},
	{SG40035, &UA2530205},
	{SG40036, &UA3118965},
	{SG40040, &UA2975048},
	{SG40046, &UA622327},
	{SG40047, &UA903248},
	{SG40049, &UA1992648},
	{SG40052, &UA2004097},
	{SG40054, &UA3030656},
	{SG40057, &UA2257744},
	{SG40060, &UA2852950},
	{SG40061, &UA2986624},
	{SG40065, &UA2034283},
	{SG40067, &UA2456956},
	{SG40069, &UA2653324},
	{SG40073, &UA2701024},
	{SG40084, &UA2530148},
	{SG40085, &UA3006640},
	{SG40086, &UA3050287},
	{SG40088, &UA3118957},
	{SG40089, &UA2380306},
	{SG40090, &UA2480443},
	{SG40091, &UA2748490},
	{SG40092, &UA2839867},
	{SG40093, &UA2987670},
	{SG40094, &UA2483298},
	{SG40096, &UA2528951},
	{SG40098, &UA2667073},
	{SG40099, &UA2676596},
	{SG40113, &UA3081240},
	{SG40116, &UA2425990},
	{SG40117, &UA2624546},
	{SG40118, &UA2628347},
	{SG40124, &UA3082864},
	{SG40126, &UA804323},
	{SG40128, &UA2025033},
	{SG40131, &UA2030247},
	{SG40202, &UA2403090},
	{SG40204, &UA2424961},
	{SG40209, &UA2676729},
	{SG40212, &UA2774335},
	{SG40214, &UA2800096},
	{SG40215, &UA2974714},
	{SG40216, &UA2024292},
	{SG40217, &UA2338759},
	{SG40218, &UA2446933},
	{SG40221, &UA2753877},
	{SG40223, &UA2877407},
	{SG40225, &UA2974723},
	{SG40226, &UA2974636},
	{SG40228, &UA3022547},
	{SG40230, &UA3132865},
	{SG40231, &UA2799702},
	{SG40236, &UA768212},
	{SG40238, &UA1991039},
	{SG40240, &UA2025235},
	{SG40242, &UA2026888},
	{SG40243, &UA910575},
	{SG40244, &UA1968549},
	{SG40246, &UA2653638},
	{SG40247, &UA2354579},
	{SG40248, &UA2597296},
	{SG40250, &UA2616096},
	{SG40251, &UA2628038},
	{SG40253, &UA2631578},
	{SG40254, &UA2484308},
	{SG40263, &UA3031263},
	{SG40264, &UA3092927},
	{SG40265, &UA778930},
	{SG40266, &UA909516},
	{SG40267, &UA1959686},
	{SG40268, &UA2025578},
	{SG40269, &UA2984573},
	{SG40270, &UA2715110},
	{SG40271, &UA2962442},
	{SG40272, &UA2030693},
	{SG40273, &UA2428482},
	{SG40274, &UA3099631},
	{SG40275, &UA2754629},
	{SG40277, &UA607624},
	{SG40278, &UA736162},
	{SG40279, &UA902278},
	{SG40287, &UA1984823},
	{SG40288, &UA2232521},
	{SG40295, &UA842084},
	{SG40297, &UA1982567},
	{SG40302, &UA2467227},
	{SG40305, &UA2617799},
	{SG40306, &UA2625735},
	{SG40308, &UA2630384},
	{SG40311, &UA2961462},
	{SG40313, &UA2962956},
	{SG40322, &UA2506180},
	{SG40324, &UA2567112},
	{SG40326, &UA2636399},
	{SG40328, &UA2772915},
	{SG40330, &UA2674204},
	{SG40332, &UA2773892},
	{SG40334, &UA2953930},
	{SG40337, &UA2974643},
	{SG40339, &UA904279},
	{SG40340, &UA2608095},
	{SG40341, &UA2455972},
	{SG40342, &UA2464659},
	{SG40343, &UA2654193},
	{SG40345, &UA2838723},
	{SG40347, &UA2485534},
	{SG40348, &UA3099714},
	{SG40351, &UA2863526},
	{SG40354, &UA2426502},
	{SG40355, &UA2432242},
	{SG40356, &UA2793312},
	{SG40357, &UA2567123},
	{SG40358, &UA2635957},
	{SG40361, &UA2737397},
	{SG40364, &UA2769728},
	{SG40368, &UA898320},
	{SG40369, &UA908636},
	{SG40370, &UA2758697},
	{SG40374, &UA2420762},
	{SG40375, &UA3085809},
	{SG40376, &UA2567266},
	{SG40377, &UA2468898},
	{SG40379, &UA2754652},
	{SG40381, &UA607309},
	{SG40383, &UA902336},
	{SG40384, &UA2474123},
	{SG40386, &UA2754816},
	{SG40392, &UA673883},
	{SG40393, &UA678949},
	{SG40394, &UA910596},
	{SG40398, &UA2487017},
	{SG40400, &UA2466637},
	{SG40401, &UA3020943},
	{SG40402, &UA3021649},
	{SG40403, &UA3080708},
	{SG40404, &UA622437},
	{SG40406, &UA903400},
	{SG40408, &UA2026289},
	{SG40413, &UA2754631},
	{SG40428, &UA2421302},
	{SG40430, &UA2583394},
	{SG40432, &UA2603541},
	{SG40434, &UA2666763},
	{SG40435, &UA2704297},
	{SG40436, &UA2989189},
	{SG40437, &UA2625986},
	{SG40438, &UA2669222},
	{SG40439, &UA2974613},
	{SG40446, &UA2866556},
	{SG40447, &UA3091781},
	{SG40448, &UA3121018},
	{SG40449, &UA3083356},
	{SG40451, &UA2670911},
	{SG40452, &UA2676250},
	{SG40453, &UA3082868},
	{SG40454, &UA3126473},
	{SG40459, &UA2355199},
	{SG40460, &UA2566913},
	{SG40462, &UA2603539},
	{SG40464, &UA2658728},
	{SG40465, &UA2675848},
	{SG40466, &UA2709164},
	{SG40468, &UA2557799},
	{SG40469, &UA2567396},
	{SG40472, &UA3021153},
	{SG40473, &UA2384213},
	{SG40474, &UA2552524},
	{SG40482, &UA893126},
	{SG40483, &UA2384293},
	{SG40484, &UA2519381},
	{SG40485, &UA2674222},
	{SG40486, &UA2755080},
	{SG40490, &UA3068211},
	{SG40496, &UA2563483},
	{SG40498, &UA2947436},
	{SG40504, &UA2239012},
	{SG40505, &UA2482202},
	{SG40508, &UA698251},
	{SG40510, &UA785108},
	{SG40512, &UA906570},
	{SG40515, &UA2716741},
	{SG40516, &UA3066205},
	{SG40522, &UA660358},
	{SG40523, &UA803340},
	{SG40524, &UA899437},
	{SG40525, &UA2545650},
	{SG40526, &UA708755},
	{SG40527, &UA888150},
	{SG40528, &UA907386},
	{SG40529, &UA2238739},
	{SG40530, &UA3117948},
	{SG40531, &UA2562610},
	{SG40532, &UA2562930},
	{SG40534, &UA2602013},
	{SG40553, &UA2753965},
	{SG40561, &UA2975042},
	{SG40562, &UA2480845},
	{SG40563, &UA2701838},
	{SG40564, &UA2833588},
	{SG40566, &UA2935863},
	{SG40570, &UA2755998},
	{SG40571, &UA3094082},
	{SG40572, &UA3114928},
	{SG40574, &UA2360915},
	{SG40575, &UA2608504},
	{SG40576, &UA2507403},
	{SG40577, &UA2676924},
	{SG40578, &UA2676940},
	{SG40581, &UA3065147},
	{SG40582, &UA3068067},
	{SG40586, &UA2567021},
	{SG40589, &UA2560809},
	{SG40590, &UA2773256},
	{SG40594, &UA2945972},
	{SG40598, &UA3081051},
	{SG40602, &UA2458143},
	{SG40603, &UA2468127},
	{SG40604, &UA2600528},
	{SG40612, &UA2422321},
	{SG40615, &UA2537321},
	{SG40618, &UA2754940},
	{SG40621, &UA2799042},
	{SG40624, &UA2567106},
	{SG40626, &UA2666765},
	{SG40627, &UA2702447},
	{SG40631, &UA2658325},
	{SG40635, &UA2877824},
	{SG40639, &UA2947353},
	{SG40640, &UA2975963},
	{SG40642, &UA2974119},
	{SG40646, &UA2567967},
	{SG40652, &UA2796299},
	{SG40655, &UA2030692},
	{SG40656, &UA2557629},
	{SG40658, &UA2602706},
	{SG40660, &UA2855042},
	{SG40662, &UA2961923},
	{SG40697, &UA3051363},
	{SG40698, &UA2676337},
	{SG40700, &UA3022967},
	{SG40701, &UA2528927},
	{SG40702, &UA2558077},
	{SG40703, &UA2579791},
	{SG40704, &UA2581030},
	{SG40706, &UA2897926},
	{SG40708, &UA2898913},
	{SG40710, &UA3093340},
	{SG40712, &UA3093405},
	{SG40715, &UA2735101},
	{SG40721, &UA2461151},
	{SG40722, &UA2555250},
	{SG40723, &UA2599248},
	{SG40725, &UA2955805},
	{SG40728, &UA2673074},
	{SG40729, &UA2676586},
	{SG40736, &UA2974879},
	{SG40737, &UA3021144},
	{SG40744, &UA2670614},
	{SG40751, &UA2315327},
	{SG40752, &UA2487461},
	{SG40753, &UA2566956},
	{SG40754, &UA2673362},
	{SG40755, &UA2709494},
	{SG40756, &UA2709744},
	{SG40757, &UA2734838},
	{SG40758, &UA2735114},
	{SG40760, &UA2437331},
	{SG40762, &UA2675256},
	{SG40763, &UA2755069},
	{SG40764, &UA2799866},
	{SG40765, &UA2867897},
	{SG40766, &UA2893098},
	{SG40767, &UA2893500},
	{SG40769, &UA2974116},
	{SG40771, &UA3081272},
	{SG40776, &UA2597573},
	{SG40777, &UA2599851},
	{SG40783, &UA2676747},
	{SG40784, &UA2735574},
	{SG40785, &UA2753872},
	{SG40786, &UA3083343},
	{SG40788, &UA1992538},
	{SG40789, &UA2028127},
	{SG40790, &UA2029621},
	{SG40792, &UA2832067},
	{SG40794, &UA1993816},
	{SG40795, &UA2032971},
	{SG40796, &UA2227312},
	{SG40798, &UA2230180},
	{SG40800, &UA2420046},
	{SG40801, &UA2415675},
	{SG40802, &UA2424595},
	{SG40803, &UA2459054},
	{SG40804, &UA2534666},
	{SG40806, &UA3094681},
	{SG40811, &UA2423754},
	{SG40812, &UA2570336},
	{SG40818, &UA2970320},
	{SG40819, &UA3060065},
	{SG40823, &UA2773103},
	{SG40824, &UA2769174},
	{SG40825, &UA2782677},
	{SG40826, &UA2782712},
	{SG40827, &UA2868578},
	{SG40830, &UA2942531},
	{SG40832, &UA2975046},
	{SG40835, &UA2261518},
	{SG40836, &UA2507994},
	{SG40837, &UA2662883},
	{SG40838, &UA2975000},
	{SG40839, &UA3022151},
	{SG40840, &UA3022823},
	{SG40841, &UA3023116},
	{SG40847, &UA2753271},
	{SG40849, &UA2783306},
	{SG40851, &UA2876517},
	{SG40854, &UA2893377},
	{SG40856, &UA2974075},
	{SG40859, &UA2385173},
	{SG40861, &UA2505654},
	{SG40862, &UA2676748},
	{SG40865, &UA2558382},
	{SG40866, &UA2667435},
	{SG40868, &UA2862479},
	{SG40869, &UA2879409},
	{SG40870, &UA3080328},
	{SG40871, &UA3094524},
	{SG40874, &UA2197803},
	{SG40875, &UA2234947},
	{SG40878, &UA2421306},
	{SG40879, &UA2582841},
	{SG40880, &UA2676601},
	{SG40881, &UA2975724},
	{SG40882, &UA3028656},
	{SG40883, &UA2580813},
	{SG40884, &UA2676089},
	{SG40885, &UA2753452},
	{SG40886, &UA3036133},
	{SG40922, &UA2676680},
	{SG40926, &UA2231974},
	{SG40929, &UA2714892},
	{SG40942, &UA2205255},
	{SG40958, &UA2465748},
	{SG40960, &UA2833291},
	{SG40963, &UA2560451},
	{SG40964, &UA2701781},
	{SG40965, &UA2560507},
	{SG40969, &UA2677158},
	{SG40972, &UA2975943},
	{SG40974, &UA3022409},
	{SG40976, &UA3024144},
	{SG40990, &UA2431080},
	{SG40992, &UA2731416},
	{SG40993, &UA3082161},
	{SG40994, &UA3093947},
	{SG40995, &UA865474},
	{SG40996, &UA2034045},
	{SG40997, &UA2383991},
	{SG41003, &UA3080080},
	{SG41010, &UA2840621},
	{SG41014, &UA2567329},
	{SG41015, &UA2567567},
	{SG41016, &UA2567399},
	{SG41017, &UA2598026},
	{SG41018, &UA2702921},
	{SG41019, &UA2854605},
	{SG41020, &UA3037311},
	{SG41025, &UA2229000},
	{SG41026, &UA2236517},
	{SG41027, &UA2531000},
	{SG41029, &UA2557176},
	{SG41032, &UA2737731},
	{SG41034, &UA3047739},
	{SG41052, &UA2230331},
	{SG41058, &UA2426480},
	{SG41060, &UA2483431},
	{SG41061, &UA2847102},
	{SG41062, &UA2559701},
	{SG41063, &UA2987392},
	{SG41064, &UA2567700},
	{SG41065, &UA2753057},
	{SG41066, &UA3080485},
	{SG41067, &UA3083286},
	{SG41068, &UA3094534},
	{SG41072, &UA2765027},
	{SG41075, &UA2236426},
	{SG41077, &UA2525333},
	{SG41078, &UA2354709},
	{SG41082, &UA3021138},
	{SG41084, &UA3062548},
	{SG41085, &UA3081678},
	{SG41091, &UA2566884},
	{SG41092, &UA2615547},
	{SG41093, &UA2433097},
	{SG41094, &UA2510294},
	{SG41095, &UA3125041},
	{SG41096, &UA2529488},
	{SG41097, &UA2702168},
	{SG41102, &UA2666829},
	{SG41104, &UA2670024},
	{SG41105, &UA2676687},
	{SG41106, &UA2709486},
	{SG41109, &UA2754850},
	{SG41111, &UA2755406},
	{SG41113, &UA3115555},
	{SG41127, &UA2973492},
	{SG41133, &UA2753861},
	{SG41134, &UA2773850},
	{SG41136, &UA613297},
	{SG41137, &UA734465},
	{SG41138, &UA2030406},
	{SG41143, &UA2361272},
	{SG41146, &UA2428314},
	{SG41149, &UA2676944},
	{SG41150, &UA2754654},
	{SG41159, &UA2566444},
	{SG41162, &UA2595205},
	{SG41164, &UA2697659},
	{SG41166, &UA2675838},
	{SG41168, &UA2715175},
	{SG41173, &UA910205},
	{SG41176, &UA2415169},
	{SG41177, &UA2456800},
	{SG41178, &UA2485703},
	{SG41179, &UA2791344},
	{SG41197, &UA912138},
	{SG41199, &UA2504327},
	{SG41201, &UA1993405},
	{SG41204, &UA2237628},
	{SG41206, &UA2257854},
	{SG41208, &UA2425454},
	{SG41210, &UA2483668},
	{SG41211, &UA2594869},
	{SG41212, &UA2768364},
	{SG41213, &UA3007919},
	{SG41214, &UA3047191},
	{SG41216, &UA3102993},
	{SG41217, &UA1996902},
	{SG41220, &UA2558984},
	{SG41222, &UA2667013},
	{SG41223, &UA2705728},
	{SG41224, &UA2958953},
	{SG41225, &UA2670613},
	{SG41227, &UA2413413},
	{SG41229, &UA2421280},
	{SG41231, &UA2423201},
	{SG41233, &UA2426196},
	{SG41235, &UA2566935},
	{SG41238, &UA456955},
	{SG41241, &UA892413},
	{SG41243, &UA908383},
	{SG41244, &UA1964141},
	{SG41245, &UA2455550},
	{SG41246, &UA2641479},
	{SG41250, &UA2453155},
	{SG41255, &UA1983137},
	{SG41256, &UA2975549},
	{SG41257, &UA3020720},
	{SG41259, &UA3037081},
	{SG41263, &UA2462055},
	{SG41264, &UA2974088},
	{SG41268, &UA2557738},
	{SG41270, &UA2792576},
	{SG41271, &UA2580237},
	{SG41273, &UA2763966},
	{SG41277, &UA2975554},
	{SG41283, &UA911669},
	{SG41285, &UA2205630},
	{SG41286, &UA2583414},
	{SG41287, &UA2600965},
	{SG41288, &UA2604499},
	{SG41289, &UA2879911},
	{SG41292, &UA2526680},
	{SG41293, &UA3070268},
	{SG41300, &UA2532389},
	{SG41301, &UA2752981},
	{SG41302, &UA2869845},
	{SG41312, &UA2421712},
	{SG41313, &UA2422653},
	{SG41314, &UA2557420},
	{SG41315, &UA2766113},
	{SG41317, &UA2558246},
	{SG41319, &UA2661459},
	{SG41322, &UA2755412},
	{SG41323, &UA2867965},
	{SG41324, &UA3081116},
	{SG41325, &UA2029984},
	{SG41326, &UA2760633},
	{SG41334, &UA2672454},
	{SG41335, &UA2754626},
	{SG41339, &UA912341},
	{SG41341, &UA1966200},
	{SG41342, &UA1978560},
	{SG41343, &UA1994432},
	{SG41344, &UA2233478},
	{SG41345, &UA2349222},
	{SG41346, &UA2416857},
	{SG41347, &UA2424749},
	{SG41348, &UA2552934},
	{SG41349, &UA2427719},
	{SG41350, &UA2671580},
	{SG41351, &UA2708401},
	{SG41353, &UA2567373},
	{SG41356, &UA3081341},
	{SG41357, &UA2676192},
	{SG41358, &UA2706668},
	{SG41370, &UA2506557},
	{SG41371, &UA3099870},
	{SG41373, &UA2659425},
	{SG41374, &UA2891278},
	{SG41375, &UA2427138},
	{SG41376, &UA2582928},
	{SG41377, &UA2675069},
	{SG41383, &UA2528629},
	{SG41384, &UA2567715},
	{SG41389, &UA2485712},
	{SG41390, &UA2503129},
	{SG41391, &UA2554895},
	{SG41393, &UA2595391},
	{SG41394, &UA2748090},
	{SG41396, &UA2534581},
	{SG41397, &UA2583700},
	{SG41398, &UA2602605},
	{SG41399, &UA2666961},
	{SG41400, &UA2974666},
	{SG41401, &UA895974},
	{SG41402, &UA908866},
	{SG41403, &UA2506962},
	{SG41404, &UA2533925},
	{SG41405, &UA2729872},
	{SG41406, &UA2797656},
	{SG41407, &UA3067296},
	{SG41409, &UA2024108},
	{SG41411, &UA2366719},
	{SG41413, &UA2010564},
	{SG41416, &UA2257524},
	{SG41418, &UA2404666},
	{SG41420, &UA2772286},
	{SG41426, &UA2466327},
	{SG41428, &UA2468867},
	{SG41430, &UA2934395},
	{SG41436, &UA2937505},
	{SG41437, &UA2718784},
	{SG41438, &UA2755263},
	{SG41439, &UA2875080},
	{SG41443, &UA2373968},
	{SG41444, &UA2402910},
	{SG41445, &UA2567693},
	{SG41446, &UA2600028},
	{SG41447, &UA2671657},
	{SG41450, &UA2567389},
	{SG41451, &UA2676482},
	{SG41457, &UA2374074},
	{SG41458, &UA2415268},
	{SG41459, &UA2484195},
	{SG41460, &UA3071146},
	{SG41461, &UA2417986},
	{SG41462, &UA2423218},
	{SG41464, &UA2484106},
	{SG41470, &UA2975745},
	{SG41477, &UA2584143},
	{SG41478, &UA2667544},
	{SG41479, &UA2600240},
	{SG41480, &UA2625428},
	{SG41482, &UA2677292},
	{SG41485, &UA3080244},
	{SG41488, &UA912025},
	{SG41489, &UA802915},
	{SG41490, &UA889762},
	{SG41491, &UA2028617},
	{SG41492, &UA2032893},
	{SG41493, &UA2222311},
	{SG41494, &UA2693256},
	{SG41495, &UA2029078},
	{SG41496, &UA2427971},
	{SG41497, &UA2567323},
	{SG41498, &UA2676584},
	{SG41500, &UA2709358},
	{SG41501, &UA2712200},
	{SG41502, &UA2994675},
	{SG41504, &UA1978391},
	{SG41505, &UA2414476},
	{SG41507, &UA2500096},
	{SG41516, &UA2383141},
	{SG41517, &UA2446007},
	{SG41518, &UA2498726},
	{SG41519, &UA2692885},
	{SG41520, &UA2693965},
	{SG41521, &UA2830285},
	{SG41522, &UA2400631},
	{SG41523, &UA2466300},
	{SG41524, &UA2782934},
	{SG41525, &UA3080656},
	{SG41526, &UA2338758},
	{SG41528, &UA2456377},
	{SG41529, &UA3088425},
	{SG41530, &UA3080012},
	{SG41531, &UA3080596},
	{SG41535, &UA609415},
	{SG41536, &UA910309},
	{SG41539, &UA1996044},
	{SG41542, &UA2676280},
	{SG41544, &UA2708112},
	{SG41545, &UA2974115},
	{SG41546, &UA2431768},
	{SG41547, &UA2507041},
	{SG41548, &UA2987183},
	{SG41549, &UA2567586},
	{SG41550, &UA2583125},
	{SG41552, &UA2603301},
	{SG41554, &UA2936875},
	{SG41556, &UA2863169},
	{SG41561, &UA2595731},
	{SG41562, &UA2747925},
	{SG41563, &UA2844096},
	{SG41565, &UA2934434},
	{SG41566, &UA2755058},
	{SG41567, &UA2974644},
	{SG41569, &UA2437634},
	{SG41572, &UA2567947},
	{SG41574, &UA2702388},
	{SG41575, &UA2567692},
	{SG41576, &UA2600908},
	{SG41586, &UA1995233},
	{SG41589, &UA2230737},
	{SG41591, &UA2233834},
	{SG41597, &UA2585171},
	{SG41599, &UA2601584},
	{SG41602, &UA2667299},
	{SG41604, &UA2668270},
	{SG41605, &UA2857052},
	{SG41609, &UA923994},
	{SG41610, &UA1994342},
	{SG41611, &UA2167324},
	{SG41612, &UA2445587},
	{SG41613, &UA2405061},
	{SG41616, &UA2976690},
	{SG41620, &UA2567264},
	{SG41621, &UA2676935},
	{SG41624, &UA2708692},
	{SG41627, &UA2673941},
	{SG41632, &UA3022686},
	{SG41645, &UA2505195},
	{SG41647, &UA911391},
	{SG41648, &UA2840807},
	{SG41649, &UA2365677},
	{SG41650, &UA2367016},
	{SG41651, &UA2687268},
	{SG41652, &UA2790346},
	{SG41655, &UA2673227},
	{SG41660, &UA2774291},
	{SG41669, &UA2712062},
	{SG41671, &UA3082870},
	{SG41673, &UA2218161},
	{SG41674, &UA2535972},
	{SG41675, &UA2767541},
	{SG41678, &UA2676438},
	{SG41679, &UA2782446},
	{SG41680, &UA2888714},
	{SG41681, &UA2888715},
	{SG41682, &UA2888716},
	{SG41684, &UA3083830},
	{SG41685, &UA3095531},
	{SG41686, &UA3111329},
	{SG41687, &UA2527078},
	{SG41688, &UA2652846},
	{SG41690, &UA2712513},
	{SG41695, &UA2783982},
	{SG41702, &UA2228371},
	{SG41704, &UA2528341},
	{SG41708, &UA2567220},
	{SG41709, &UA2567575},
	{SG41710, &UA2674448},
	{SG41711, &UA2735118},
	{SG41714, &UA905527},
	{SG41717, &UA2567590},
	{SG41722, &UA2975877},
	{SG41726, &UA606162},
	{SG41729, &UA794044},
	{SG41734, &UA2382132},
	{SG41736, &UA2459538},
	{SG41752, &UA863449},
	{SG41753, &UA2025299},
	{SG41757, &UA2230494},
	{SG41760, &UA2566624},
	{SG41762, &UA2583568},
	{SG41765, &UA2676099},
	{SG41767, &UA3081117},
	{SG41768, &UA3080246},
	{SG41771, &UA2662772},
	{SG41772, &UA3119313},
	{SG41777, &UA2901236},
	{SG41778, &UA607569},
	{SG41779, &UA889039},
	{SG41780, &UA910398},
	{SG41783, &UA2030220},
	{SG41786, &UA2424398},
	{SG41791, &UA2752384},
	{SG41799, &UA2383348},
	{SG41801, &UA2478125},
	{SG41805, &UA2714782},
	{SG41825, &UA3021467},
	{SG41827, &UA3022955},
	{SG41829, &UA3023834},
	{SG41836, &UA2228223},
	{SG41838, &UA2404823},
	{SG41840, &UA2426665},
	{SG41842, &UA2483053},
	{SG41843, &UA2538207},
	{SG41852, &UA3083189},
	{SG41854, &UA609363},
	{SG41855, &UA631627},
	{SG41858, &UA2561921},
	{SG41868, &UA2194823},
	{SG41869, &UA2505757},
	{SG41890, &UA2367200},
	{SG41891, &UA2426795},
	{SG41902, &UA2428148},
	{SG41903, &UA2400882},
	{SG41904, &UA2416215},
	{SG41905, &UA2746734},
	{SG41906, &UA2984655},
	{SG41907, &UA2481742},
	{SG41908, &UA2667183},
	{SG41910, &UA2567708},
	{SG41914, &UA3021460},
	{SG41915, &UA3022615},
	{SG41918, &UA2428430},
	{SG41920, &UA2753703},
	{SG41921, &UA3080417},
	{SG41926, &UA2476823},
	{SG41928, &UA2829283},
	{SG41930, &UA2958402},
	{SG41934, &UA2566819},
	{SG41935, &UA2567214},
	{SG41936, &UA2704833},
	{SG41943, &UA2848296},
	{SG41951, &UA2567019},
	{SG41953, &UA3046197},
	{SG41956, &UA2974931},
	{SG41957, &UA3068432},
	{SG41959, &UA3081106},
	{SG41966, &UA2528246},
	{SG41967, &UA2843482},
	{SG41968, &UA2567138},
	{SG41970, &UA2738258},
	{SG41974, &UA2974929},
	{SG41975, &UA3094737},
	{SG41976, &UA3110919},
	{SG41977, &UA3111718},
	{SG41978, &UA3113257},
	{SG41980, &UA3116527},
	{SG41981, &UA3127386},
	{SG41988, &UA2503847},
	{SG41989, &UA2621763},
	{SG41990, &UA2581107},
	{SG41991, &UA2671807},
	{SG41995, &UA2566818},
	{SG41998, &UA2674224},
	{SG42001, &UA3080709},
	{SG42006, &UA2433170},
	{SG42008, &UA2425761},
	{SG42009, &UA2566963},
	{SG42010, &UA2566770},
	{SG42013, &UA2755054},
	{SG42015, &UA2879607},
	{SG42025, &UA887934},
	{SG42026, &UA1969678},
	{SG42028, &UA2027516},
	{SG42029, &UA2774325},
	{SG42030, &UA2869562},
	{SG42031, &UA2975967},
	{SG42032, &UA3094786},
	{SG42037, &UA2422805},
	{SG42041, &UA2655794},
	{SG42042, &UA2718911},
	{SG42043, &UA2738128},
	{SG42044, &UA3083765},
	{SG42050, &UA2223461},
	{SG42051, &UA2551246},
	{SG42053, &UA2645202},
	{SG42055, &UA2557583},
	{SG42059, &UA2676846},
	{SG42060, &UA3081084},
	{SG42061, &UA2567319},
	{SG42062, &UA2584096},
	{SG42063, &UA2988838},
	{SG42064, &UA2737563},
	{SG42065, &UA2754488},
	{SG42066, &UA2947364},
	{SG42073, &UA3083048},
	{SG42074, &UA2755156},
	{SG42075, &UA2773867},
	{SG42076, &UA2799779},
	{SG42078, &UA2877917},
	{SG42080, &UA2937528},
	{SG42084, &UA3080329},
	{SG42086, &UA3083938},
	{SG42087, &UA2423668},
	{SG42088, &UA2570339},
	{SG42089, &UA2583209},
	{SG42091, &UA2754168},
	{SG42097, &UA2973657},
	{SG42102, &UA2506926},
	{SG42114, &UA2670260},
	{SG42115, &UA2508844},
	{SG42119, &UA2566932},
	{SG42121, &UA2579208},
	{SG42123, &UA2647816},
	{SG42127, &UA2784146},
	{SG42128, &UA2867924},
	{SG42133, &UA2601088},
	{SG42135, &UA3074185},
	{SG42141, &UA727046},
	{SG42143, &UA2033597},
	{SG42144, &UA2233968},
	{SG42146, &UA2933755},
	{SG42147, &UA3070198},
	{SG42149, &UA2456501},
	{SG42150, &UA2652931},
	{SG42152, &UA2843269},
	{SG42154, &UA2505907},
	{SG42155, &UA2675393},
	{SG42156, &UA2782181},
	{SG42157, &UA2992441},
	{SG42158, &UA2942472},
	{SG42159, &UA2975990},
	{SG42160, &UA3080325},
	{SG42170, &UA2558071},
	{SG42174, &UA2431767},
	{SG42175, &UA2664591},
	{SG42180, &UA2422252},
	{SG42181, &UA2460844},
	{SG42183, &UA2583396},
	{SG42184, &UA2603864},
	{SG42185, &UA2748453},
	{SG42186, &UA2779977},
	{SG42188, &UA2889731},
	{SG42191, &UA2957844},
	{SG42192, &UA3107952},
	{SG42193, &UA2797130},
	{SG42195, &UA2656235},
	{SG42196, &UA2481222},
	{SG42197, &UA2528914},
	{SG42198, &UA2505311},
	{SG42199, &UA2566527},
	{SG42200, &UA2580743},
	{SG42201, &UA2756481},
	{SG42204, &UA2975558},
	{SG42205, &UA3020866},
	{SG42206, &UA3021234},
	{SG42208, &UA3021721},
	{SG42209, &UA2990725},
	{SG42210, &UA3065521},
	{SG42211, &UA3083020},
	{SG42212, &UA3083826},
	{SG42214, &UA3127186},
	{SG42216, &UA3095181},
	{SG42217, &UA3114360},
	{SG42231, &UA2455435},
	{SG42233, &UA2676284},
	{SG42235, &UA2711808},
	{SG42237, &UA2734612},
	{SG42238, &UA2735189},
	{SG42239, &UA2768141},
	{SG42240, &UA2755114},
	{SG42241, &UA2756413},
	{SG42242, &UA2797908},
	{SG42243, &UA2877847},
	{SG42244, &UA2781195},
	{SG42245, &UA2994061},
	{SG42246, &UA3081628},
	{SG42247, &UA3109623},
	{SG42252, &UA2663153},
	{SG42256, &UA2677159},
	{SG42258, &UA2709131},
	{SG42259, &UA2713676},
	{SG42260, &UA2769952},
	{SG42263, &UA2783472},
	{SG42264, &UA2866909},
	{SG42267, &UA2706289},
	{SG42269, &UA2733278},
	{SG42270, &UA2794201},
	{SG42274, &UA2754793},
	{SG42277, &UA3083259},
	{SG42279, &UA1969842},
	{SG42280, &UA2395141},
	{SG42282, &UA2567572},
	{SG42283, &UA2859648},
	{SG42284, &UA2581318},
	{SG42286, &UA3058015},
	{SG42287, &UA2584867},
	{SG42288, &UA2667242},
	{SG42289, &UA2953585},
	{SG42291, &UA2975471},
	{SG42292, &UA3022189},
	{SG42295, &UA2218792},
	{SG42296, &UA2480774},
	{SG42298, &UA2257787},
	{SG42300, &UA2461164},
	{SG42301, &UA2557828},
	{SG42302, &UA2622668},
	{SG42303, &UA2624270},
	{SG42305, &UA2625472},
	{SG42306, &UA2627417},
	{SG42308, &UA2799104},
	{SG42312, &UA2385095},
	{SG42313, &UA2455144},
	{SG42314, &UA2531370},
	{SG42315, &UA2537244},
	{SG42316, &UA2538286},
	{SG42317, &UA3058314},
	{SG42318, &UA2708668},
	{SG42319, &UA2712828},
	{SG42320, &UA2901105},
	{SG42321, &UA3034129},
	{SG42322, &UA3108976},
	{SG42324, &UA3131645},
	{SG42329, &UA2552784},
	{SG42330, &UA2728890},
	{SG42331, &UA2836016},
	{SG42332, &UA2951736},
	{SG42334, &UA2567950},
	{SG42336, &UA2974721},
	{SG42338, &UA3044109},
	{SG42339, &UA2399085},
	{SG42340, &UA2457373},
	{SG42341, &UA2556663},
	{SG42342, &UA2749767},
	{SG42345, &UA2676371},
	{SG42348, &UA2868013},
	{SG42350, &UA2483405},
	{SG42351, &UA2534595},
	{SG42352, &UA2600114},
	{SG42353, &UA2755258},
	{SG42354, &UA2765146},
	{SG42355, &UA2496706},
	{SG42356, &UA2520352},
	{SG42357, &UA2577998},
	{SG42359, &UA2504243},
	{SG42360, &UA2555878},
	{SG42361, &UA2660720},
	{SG42363, &UA2736486},
	{SG42368, &UA2502527},
	{SG42369, &UA3099726},
	{SG42370, &UA2529745},
	{SG42372, &UA2669225},
	{SG42373, &UA2780938},
	{SG42378, &UA2581715},
	{SG42390, &UA2027847},
	{SG42392, &UA2424350},
	{SG42394, &UA2837348},
	{SG42404, &UA2528157},
	{SG42405, &UA2984795},
	{SG42406, &UA2987291},
	{SG42410, &UA2670604},
	{SG42413, &UA2709239},
	{SG42415, &UA2734624},
	{SG42417, &UA2753780},
	{SG42418, &UA2783061},
	{SG42419, &UA2937683},
	{SG42420, &UA2937991},
	{SG42421, &UA2974653},
	{SG42426, &UA2350051},
	{SG42428, &UA2455456},
	{SG42430, &UA2658159},
	{SG42432, &UA2699410},
	{SG42434, &UA2459467},
	{SG42435, &UA2537196},
	{SG42437, &UA2663267},
	{SG42439, &UA2703385},
	{SG42441, &UA2764952},
	{SG42443, &UA2985899},
	{SG42444, &UA2986882},
	{SG42447, &UA2783375},
	{SG42451, &UA2997972},
	{SG42458, &UA603588},
	{SG42459, &UA1979062},
	{SG42460, &UA2025434},
	{SG42461, &UA2704066},
	{SG42463, &UA2457748},
	{SG42464, &UA2567565},
	{SG42465, &UA2975537},
	{SG42466, &UA2420725},
	{SG42468, &UA2837908},
	{SG42469, &UA2981982},
	{SG42470, &UA2455861},
	{SG42471, &UA2540514},
	{SG42473, &UA2602611},
	{SG42475, &UA2604170},
	{SG42480, &UA2677183},
	{SG42483, &UA2677301},
	{SG42514, &UA2033128},
	{SG42516, &UA2383001},
	{SG42522, &UA2505707},
	{SG42523, &UA2667228},
	{SG42530, &UA2892550},
	{SG42536, &UA2261593},
	{SG42537, &UA2404276},
	{SG42538, &UA2422242},
	{SG42541, &UA2754242},
	{SG42542, &UA2769440},
	{SG42543, &UA2769578},
	{SG42546, &UA2866840},
	{SG42547, &UA2893704},
	{SG42557, &UA2400871},
	{SG42558, &UA1983106},
	{SG42559, &UA2033053},
	{SG42561, &UA2029792},
	{SG42569, &UA2901218},
	{SG42570, &UA3004432},
	{SG42572, &UA2892316},
	{SG42579, &UA2384298},
	{SG42580, &UA2658063},
	{SG42582, &UA2676988},
	{SG42585, &UA2754804},
	{SG42587, &UA2869211},
	{SG42590, &UA2773608},
	{SG42592, &UA2782899},
	{SG42594, &UA2879332},
	{SG42602, &UA2459894},
	{SG42633, &UA2753216},
	{SG42634, &UA2753967},
	{SG42635, &UA2879481},
	{SG42636, &UA3033807},
	{SG42640, &UA2188963},
	{SG42641, &UA2385279},
	{SG42643, &UA2426910},
	{SG42645, &UA2457681},
	{SG42647, &UA2753211},
	{SG42649, &UA2754334},
	{SG42654, &UA1990778},
	{SG42655, &UA2257444},
	{SG42656, &UA2799359},
	{SG42657, &UA2796926},
	{SG42658, &UA3022867},
	{SG42664, &UA2437615},
	{SG42665, &UA2421902},
	{SG42667, &UA2427412},
	{SG42674, &UA2384933},
	{SG42680, &UA2567579},
	{SG42685, &UA3031136},
	{SG42691, &UA905663},
	{SG42692, &UA2029240},
	{SG42695, &UA2445432},
	{SG42700, &UA1995986},
	{SG42706, &UA2753874},
	{SG42707, &UA2783411},
	{SG42708, &UA2997349},
	{SG42709, &UA2755074},
	{SG42741, &UA2023332},
	{SG42742, &UA2030414},
	{SG42743, &UA2177291},
	{SG42795, &UA2754794},
	{SG42796, &UA2974076},
	{SG42797, &UA2976701},
	{SG42798, &UA2969594},
	{SG42799, &UA2976058},
	{SG42800, &UA3022991},
	{SG42805, &UA2225023},
	{SG42806, &UA3042470},
	{SG42809, &UA2226653},
	{SG42811, &UA2258089},
	{SG42814, &UA2981364},
	{SG42817, &UA905944},
	{SG42818, &UA1996081},
	{SG42821, &UA2784885},
	{SG42826, &UA2031484},
	{SG42827, &UA2758840},
	{SG42831, &UA2870627},
	{SG42833, &UA2350299},
	{SG42834, &UA2613271},
	{SG42835, &UA2618948},
	{SG42837, &UA2619791},
	{SG42838, &UA2621252},
	{SG42839, &UA2598606},
	{SG42840, &UA2478038},
	{SG42841, &UA2776441},
	{SG42843, &UA2982138},
	{SG42847, &UA2601335},
	{SG42848, &UA2584861},
	{SG42851, &UA2712039},
	{SG42852, &UA3063442},
	{SG42865, &UA2421628},
	{SG42866, &UA2422925},
	{SG42867, &UA2657906},
	{SG42869, &UA2728503},
	{SG42870, &UA2951655},
	{SG42876, &UA2800381},
	{SG42878, &UA2738341},
	{SG42881, &UA2769389},
	{SG42886, &UA2428425},
	{SG42887, &UA2467679},
	{SG42888, &UA2671079},
	{SG42891, &UA745249},
	{SG42894, &UA908040},
	{SG42895, &UA2527741},
	{SG42899, &UA2567025},
	{SG42905, &UA2754070},
	{SG42907, &UA2771850},
	{SG42908, &UA2796682},
	{SG42911, &UA2879203},
	{SG42913, &UA2356553},
	{SG42917, &UA2458618},
	{SG42918, &UA2566773},
	{SG42919, &UA2676848},
	{SG42920, &UA2678416},
	{SG42921, &UA2699867},
	{SG42922, &UA3080942},
	{SG42923, &UA3083132},
	{SG42924, &UA2029206},
	{SG42925, &UA2452845},
	{SG42926, &UA2191498},
	{SG42927, &UA2527704},
	{SG42930, &UA2677155},
	{SG42932, &UA2862926},
	{SG42937, &UA2368320},
	{SG42942, &UA2583347},
	{SG42943, &UA2708341},
	{SG42944, &UA3031516},
	{SG42945, &UA3083557},
	{SG42950, &UA2403174},
	{SG42951, &UA2422422},
	{SG42952, &UA2975968},
	{SG42956, &UA3080507},
	{SG42960, &UA2226623},
	{SG42961, &UA2480793},
	{SG42962, &UA2530140},
	{SG42963, &UA2567580},
	{SG42965, &UA2625495},
	{SG42967, &UA2674981},
	{SG42970, &UA3102117},
	{SG42972, &UA2676852},
	{SG42974, &UA2754802},
	{SG42975, &UA2898806},
	{SG42976, &UA3010074},
	{SG42977, &UA3083051},
	{SG42978, &UA772338},
	{SG42979, &UA2026851},
	{SG42983, &UA2504385},
	{SG42984, &UA2354229},
	{SG42986, &UA2425129},
	{SG42988, &UA2566940},
	{SG42990, &UA3021224},
	{SG42992, &UA3021452},
	{SG42993, &UA3021874},
	{SG42994, &UA2379475},
	{SG42995, &UA2567248},
	{SG42998, &UA2673254},
	{SG43013, &UA2968672},
	{SG43015, &UA3000520},
	{SG43017, &UA3008874},
	{SG43019, &UA3009614},
	{SG43021, &UA3091614},
	{SG43029, &UA2403572},
	{SG43030, &UA2476389},
	{SG43031, &UA2427181},
	{SG43032, &UA2605946},
	{SG43033, &UA2578757},
	{SG43034, &UA2594396},
	{SG43035, &UA2746982},
	{SG43036, &UA2604078},
	{SG43037, &UA2674731},
	{SG43038, &UA2754342},
	{SG43039, &UA2754347},
	{SG43040, &UA2772530},
	{SG43042, &UA2974099},
	{SG43044, &UA2966840},
	{SG43045, &UA3079654},
	{SG43049, &UA2850444},
	{SG43052, &UA2673271},
	{SG43054, &UA2975024},
	{SG43056, &UA3063302},
	{SG43061, &UA748923},
	{SG43063, &UA763465},
	{SG43065, &UA1971029},
	{SG43066, &UA2034001},
	{SG43068, &UA1977096},
	{SG43069, &UA2890769},
	{SG43071, &UA2554223},
	{SG43072, &UA2746779},
	{SG43073, &UA2983060},
	{SG43074, &UA3043141},
	{SG43075, &UA2566774},
	{SG43076, &UA2676749},
	{SG43082, &UA2222391},
	{SG43083, &UA2469663},
	{SG43085, &UA2754566},
	{SG43086, &UA2868523},
	{SG43088, &UA2963641},
	{SG43091, &UA3065193},
	{SG43096, &UA2481137},
	{SG43097, &UA2355100},
	{SG43098, &UA2365107},
	{SG43099, &UA2428743},
	{SG43100, &UA2469140},
	{SG43101, &UA2560775},
	{SG43107, &UA1954143},
	{SG43114, &UA2461757},
	{SG43116, &UA2530792},
	{SG43117, &UA2665771},
	{SG43120, &UA3057656},
	{SG43122, &UA2531901},
	{SG43123, &UA2556692},
	{SG43124, &UA2952157},
	{SG43127, &UA2677177},
	{SG43131, &UA2773107},
	{SG43133, &UA2755065},
	{SG43136, &UA3083212},
	{SG43137, &UA634043},
	{SG43138, &UA911777},
	{SG43139, &UA2414571},
	{SG43140, &UA2416015},
	{SG43148, &UA2613774},
	{SG43151, &UA2738089},
	{SG43152, &UA2794229},
	{SG43156, &UA2865772},
	{SG43194, &UA2257719},
	{SG43198, &UA2461053},
	{SG43199, &UA2466561},
	{SG43202, &UA2864572},
	{SG43209, &UA2509856},
	{SG43211, &UA2530070},
	{SG43214, &UA2734529},
	{SG43218, &UA2754340},
	{SG43228, &UA2424197},
	{SG43229, &UA2502375},
	{SG43230, &UA2567498},
	{SG43231, &UA2708083},
	{SG43240, &UA2241406},
	{SG43241, &UA2241484},
	{SG43243, &UA2242101},
	{SG43246, &UA2870045},
	{SG43247, &UA2524089},
	{SG43251, &UA2556979},
	{SG43254, &UA2583569},
	{SG43256, &UA2713847},
	{SG43258, &UA2966396},
	{SG43262, &UA3023024},
	{SG43264, &UA3025617},
	{SG43266, &UA911292},
	{SG43270, &UA2407282},
	{SG43275, &UA3081271},
	{SG43280, &UA2449969},
	{SG43282, &UA2227229},
	{SG43284, &UA2403518},
	{SG43285, &UA2403705},
	{SG43288, &UA2677161},
	{SG43290, &UA1994573},
	{SG43291, &UA2229084},
	{SG43292, &UA2483133},
	{SG43293, &UA2545333},
	{SG43297, &UA2353918},
	{SG43300, &UA2616218},
	{SG43302, &UA2366095},
	{SG43305, &UA2466488},
	{SG43306, &UA2845361},
	{SG43307, &UA2583320},
	{SG43308, &UA2633169},
	{SG43309, &UA2637431},
	{SG43315, &UA2773906},
	{SG43318, &UA3093686},
	{SG43320, &UA2477636},
	{SG43322, &UA2025731},
	{SG43324, &UA2505227},
	{SG43325, &UA2583226},
	{SG43326, &UA2606485},
	{SG43329, &UA2796245},
	{SG43334, &UA3081230},
	{SG43340, &UA2596988},
	{SG43342, &UA2425282},
	{SG43345, &UA2896171},
	{SG43349, &UA857584},
	{SG43351, &UA911409},
	{SG43357, &UA2676931},
	{SG43358, &UA2710090},
	{SG43359, &UA2735405},
	{SG43361, &UA2707288},
	{SG43363, &UA2756321},
	{SG43364, &UA2885871},
	{SG43366, &UA2936930},
	{SG43368, &UA2973910},
	{SG43369, &UA3022181},
	{SG43370, &UA3022952},
	{SG43371, &UA3060873},
	{SG43372, &UA3108737},
	{SG43377, &UA2028471},
	{SG43383, &UA2466288},
	{SG43387, &UA2842496},
	{SG43391, &UA2354897},
	{SG43393, &UA2556129},
	{SG43398, &UA2425032},
	{SG43404, &UA2602244},
	{SG43407, &UA2613410},
	{SG43421, &UA2505375},
	{SG43422, &UA2426464},
	{SG43424, &UA2499092},
	{SG43464, &UA2673355},
	{SG43466, &UA2974896},
	{SG43467, &UA3083067},
	{SG43468, &UA3094533},
	{SG43471, &UA2230777},
	{SG43473, &UA2672412},
	{SG43476, &UA2671991},
	{SG43477, &UA2716535},
	{SG43483, &UA3083339},
	{SG43491, &UA2705525},
	{SG43494, &UA3083384},
	{SG43503, &UA2032716},
	{SG43505, &UA2386547},
	{SG43506, &UA2421519},
	{SG43508, &UA2431869},
	{SG43510, &UA2483149},
	{SG43512, &UA2566644},
	{SG43513, &UA2675837},
	{SG43514, &UA2774137},
	{SG43515, &UA3022700},
	{SG43516, &UA3022816},
	{SG43517, &UA3058088},
	{SG43518, &UA3032263},
	{SG43519, &UA3022177},
	{SG43520, &UA3022335},
	{SG43521, &UA3022529},
	{SG43522, &UA3023678},
	{SG43525, &UA2696940},
	{SG43526, &UA2468970},
	{SG43527, &UA2671148},
	{SG43530, &UA2676502},
	{SG43532, &UA2672920},
	{SG43533, &UA2703331},
	{SG43534, &UA2782301},
	{SG43535, &UA2782580},
	{SG43542, &UA2666597},
	{SG43544, &UA2955977},
	{SG43553, &UA3065198},
	{SG43555, &UA3081241},
	{SG43556, &UA3065538},
	{SG43557, &UA3111847},
	{SG43570, &UA2424003},
	{SG43573, &UA2753983},
	{SG43574, &UA3022977},
	{SG43575, &UA605254},
	{SG43576, &UA709641},
	{SG43578, &UA909261},
	{SG43582, &UA2603909},
	{SG43583, &UA2607546},
	{SG43585, &UA2607552},
	{SG43588, &UA2661406},
	{SG43589, &UA2713235},
	{SG43590, &UA2713873},
	{SG43591, &UA2736201},
	{SG43592, &UA2958411},
	{SG43595, &UA3028296},
	{SG43599, &UA2375202},
	{SG43600, &UA3086414},
	{SG43601, &UA2399019},
	{SG43602, &UA2416992},
	{SG43605, &UA2552404},
	{SG43607, &UA2552442},
	{SG43608, &UA2693717},
	{SG43609, &UA2567705},
	{SG43611, &UA2676428},
	{SG43615, &UA1995536},
	{SG43616, &UA3100039},
	{SG43618, &UA2665867},
	{SG43619, &UA2657578},
	{SG43620, &UA2689627},
	{SG43621, &UA2937413},
	{SG43622, &UA2994204},
	{SG43623, &UA3048548},
	{SG43625, &UA3021255},
	{SG43628, &UA3082763},
	{SG43633, &UA2384449},
	{SG43634, &UA2456226},
	{SG43635, &UA2521883},
	{SG43636, &UA2985245},
	{SG43637, &UA2482398},
	{SG43639, &UA2487074},
	{SG43642, &UA2526721},
	{SG43643, &UA2535290},
	{SG43644, &UA2662940},
	{SG43645, &UA2677282},
	{SG43646, &UA2695108},
	{SG43648, &UA3020297},
	{SG43649, &UA3021348},
	{SG43655, &UA2502753},
	{SG43656, &UA2857962},
	{SG43660, &UA2704234},
	{SG43667, &UA2782004},
	{SG43670, &UA3064793},
	{SG43671, &UA3094151},
	{SG43672, &UA3094674},
	{SG43673, &UA3112539},
	{SG43677, &UA2261581},
	{SG43679, &UA2671463},
	{SG43680, &UA2707391},
	{SG43681, &UA2701544},
	{SG43682, &UA2855263},
	{SG43683, &UA2769198},
	{SG43691, &UA2360881},
	{SG43692, &UA2616508},
	{SG43693, &UA2384831},
	{SG43696, &UA2879599},
	{SG43697, &UA2881401},
	{SG43698, &UA3081035},
	{SG43713, &UA2566758},
	{SG43714, &UA2677243},
	{SG43716, &UA2735150},
	{SG43717, &UA2862873},
	{SG43718, &UA2735024},
	{SG43722, &UA2782968},
	{SG43728, &UA3130737},
	{SG43730, &UA3131558},
	{SG43756, &UA3132035},
	{SG43760, &UA2340273},
	{SG43761, &UA2425636},
	{SG43763, &UA2486853},
	{SG43764, &UA2698806},
	{SG43768, &UA2570571},
	{SG43770, &UA2583368},
	{SG43771, &UA2668790},
	{SG43772, &UA2583218},
	{SG43773, &UA2667167},
	{SG43777, &UA2667907},
	{SG43778, &UA2704602},
	{SG43780, &UA2780328},
	{SG43782, &UA2947625},
	{SG43786, &UA2261580},
	{SG43787, &UA2457336},
	{SG43788, &UA2339534},
	{SG43789, &UA2832970},
	{SG43790, &UA3042574},
	{SG43791, &UA2765589},
	{SG43792, &UA2799277},
	{SG43802, &UA2029131},
	{SG43803, &UA2352520},
	{SG43805, &UA2739994},
	{SG43806, &UA2755083},
	{SG43811, &UA2974651},
	{SG43818, &UA2511894},
	{SG43820, &UA2559813},
	{SG43821, &UA2599528},
	{SG43823, &UA2755453},
	{SG43824, &UA2767353},
	{SG43827, &UA2975743},
	{SG43831, &UA3081480},
	{SG43833, &UA3126982},
	{SG43834, &UA3130590},
	{SG43835, &UA3094130},
	{SG43840, &UA2475068},
	{SG43841, &UA2033436},
	{SG43845, &UA2378343},
	{SG43847, &UA2414264},
	{SG43848, &UA2704178},
	{SG43849, &UA3057516},
	{SG43851, &UA3093514},
	{SG43856, &UA611633},
	{SG43859, &UA1981010},
	{SG43862, &UA2385420},
	{SG43864, &UA2425460},
	{SG43865, &UA2527033},
	{SG43866, &UA2427588},
	{SG43868, &UA2530934},
	{SG43871, &UA2601195},
	{SG43872, &UA2601591},
	{SG43873, &UA2625631},
	{SG43875, &UA2676509},
	{SG43876, &UA2735076},
	{SG43885, &UA2783712},
	{SG43889, &UA2028393},
	{SG43890, &UA2546707},
	{SG43894, &UA2427189},
	{SG43895, &UA2530226},
	{SG43898, &UA2746809},
	{SG43902, &UA2676266},
	{SG43904, &UA2676986},
	{SG43907, &UA2732257},
	{SG43910, &UA2769695},
	{SG43912, &UA2872160},
	{SG43913, &UA2958510},
	{SG43914, &UA3083481},
	{SG43925, &UA2365409},
	{SG43927, &UA2830482},
	{SG43930, &UA2483085},
	{SG43931, &UA2485813},
	{SG43937, &UA3021504},
	{SG43939, &UA3062996},
	{SG43942, &UA3083165},
	{SG43952, &UA2428510},
	{SG43953, &UA2781965},
	{SG43954, &UA2586648},
	{SG43955, &UA2712139},
	{SG43958, &UA2617187},
	{SG43964, &UA3021490},
	{SG43965, &UA3058069},
	{SG43972, &UA2845573},
	{SG43973, &UA2261602},
	{SG43975, &UA2448863},
	{SG43977, &UA2626891},
	{SG43978, &UA2428333},
	{SG43980, &UA2567236},
	{SG43981, &UA2676091},
	{SG43982, &UA2770908},
	{SG43983, &UA2966210},
	{SG43988, &UA892545},
	{SG43989, &UA1983736},
	{SG43995, &UA2459561},
	{SG43997, &UA2469507},
	{SG43998, &UA2506487},
	{SG44000, &UA2529894},
	{SG44002, &UA2670747},
	{SG44004, &UA2784768},
	{SG44011, &UA2531115},
	{SG44014, &UA3093822},
	{SG44015, &UA3109299},
	{SG44023, &UA2534134},
	{SG44025, &UA2537448},
	{SG44026, &UA2599679},
	{SG44027, &UA2658853},
	{SG44028, &UA2754333},
	{SG44030, &UA2896600},
	{SG44031, &UA3037943},
	{SG44033, &UA3027502},
	{SG44034, &UA3081081},
	{SG44035, &UA3114303},
	{SG44047, &UA2352085},
	{SG44048, &UA2426969},
	{SG44050, &UA2584865},
	{SG44051, &UA2567391},
	{SG44052, &UA2602607},
	{SG44053, &UA2855291},
	{SG44054, &UA2755425},
	{SG44055, &UA2773859},
	{SG44056, &UA2783458},
	{SG44057, &UA3031489},
	{SG44059, &UA3021815},
	{SG44066, &UA1973237},
	{SG44068, &UA2422227},
	{SG44069, &UA2424424},
	{SG44070, &UA2540017},
	{SG44071, &UA2666117},
	{SG44072, &UA2672678},
	{SG44073, &UA2676833},
	{SG44074, &UA2735014},
	{SG44076, &UA3059470},
	{SG44077, &UA3093879},
	{SG44078, &UA3126264},
	{SG44079, &UA3126733},
	{SG44080, &UA3126917},
	{SG44122, &UA2413921},
	{SG44123, &UA2421667},
	{SG44124, &UA3098864},
	{SG44126, &UA2456737},
	{SG44128, &UA2676987},
	{SG44129, &UA2800103},
	{SG44130, &UA2869417},
	{SG44138, &UA454605},
	{SG44141, &UA799415},
	{SG44143, &UA2027137},
	{SG44145, &UA2684739},
	{SG44152, &UA2558282},
	{SG44156, &UA2651488},
	{SG44157, &UA2694390},
	{SG44158, &UA2952450},
	{SG44159, &UA3070373},
	{SG44160, &UA2700209},
	{SG44161, &UA2770097},
	{SG44171, &UA1996169},
	{SG44172, &UA2582978},
	{SG44174, &UA2030540},
	{SG44180, &UA2458430},
	{SG44181, &UA2485907},
	{SG44182, &UA2674201},
	{SG44183, &UA2694846},
	{SG44184, &UA2796392},
	{SG44185, &UA2754185},
	{SG44195, &UA2261500},
	{SG44198, &UA2567372},
	{SG44200, &UA2676998},
	{SG44202, &UA3048367},
	{SG44204, &UA3083253},
	{SG44205, &UA3131308},
	{SG44208, &UA2224650},
	{SG44210, &UA2486553},
	{SG44214, &UA2386074},
	{SG44216, &UA2402939},
	{SG44219, &UA2527409},
	{SG44220, &UA2502759},
	{SG44221, &UA2655370},
	{SG44222, &UA2986538},
	{SG44223, &UA2676335},
	{SG44228, &UA1965217},
	{SG44230, &UA2353593},
	{SG44232, &UA2353770},
	{SG44233, &UA2583421},
	{SG44234, &UA2772895},
	{SG44235, &UA2765915},
	{SG44236, &UA2779755},
	{SG44237, &UA2860161},
	{SG44238, &UA3120280},
	{SG44239, &UA2772886},
	{SG44240, &UA2898568},
	{SG44243, &UA2559762},
	{SG44244, &UA2453280},
	{SG44247, &UA2604561},
	{SG44248, &UA2782905},
	{SG44252, &UA2204355},
	{SG44260, &UA2667413},
	{SG44262, &UA2667906},
	{SG44265, &UA2752964},
	{SG44266, &UA2770630},
	{SG44270, &UA2964355},
	{SG44278, &UA3094975},
	{SG44279, &UA3083935},
	{SG44282, &UA2191389},
	{SG44283, &UA2529118},
	{SG44289, &UA2691182},
	{SG44291, &UA2841491},
	{SG44292, &UA2945346},
	{SG44293, &UA2973386},
	{SG44294, &UA3124121},
	{SG44295, &UA3032826},
	{SG44304, &UA2745585},
	{SG44305, &UA2034405},
	{SG44306, &UA2029020},
	{SG44307, &UA2505126},
	{SG44308, &UA2029661},
	{SG44309, &UA2426065},
	{SG44310, &UA2430691},
	{SG44311, &UA2604122},
	{SG44314, &UA2676845},
	{SG44319, &UA2769106},
	{SG44320, &UA2773558},
	{SG44322, &UA2974368},
	{SG44328, &UA3064722},
	{SG44332, &UA2228309},
	{SG44333, &UA2501988},
	{SG44335, &UA2455131},
	{SG44336, &UA2619158},
	{SG44346, &UA2974994},
	{SG44349, &UA3022407},
	{SG44350, &UA2842670},
	{SG44352, &UA1995696},
	{SG44354, &UA2032597},
	{SG44355, &UA2342575},
	{SG44357, &UA2645250},
	{SG44359, &UA2827347},
	{SG44364, &UA3083230},
	{SG44368, &UA2422560},
	{SG44374, &UA2672274},
	{SG44376, &UA2707327},
	{SG44379, &UA2675857},
	{SG44382, &UA2784889},
	{SG44392, &UA3094373},
	{SG44424, &UA2236040},
	{SG44425, &UA2413540},
	{SG44427, &UA2415338},
	{SG44429, &UA2461247},
	{SG44430, &UA2483913},
	{SG44437, &UA2564828},
	{SG44438, &UA2565217},
	{SG44439, &UA3057827},
	{SG44440, &UA2583344},
	{SG44441, &UA2597450},
	{SG44445, &UA2667579},
	{SG44448, &UA2670745},
	{SG44450, &UA2737234},
	{SG44463, &UA1979451},
	{SG44464, &UA2340153},
	{SG44466, &UA2386105},
	{SG44467, &UA2424457},
	{SG44469, &UA2603598},
	{SG44470, &UA2663330},
	{SG44472, &UA2633937},
	{SG44475, &UA2799840},
	{SG44477, &UA2702877},
	{SG44482, &UA912204},
	{SG44483, &UA2520695},
	{SG44484, &UA2426121},
	{SG44489, &UA2566951},
	{SG44495, &UA2800660},
	{SG44496, &UA2867948},
	{SG44498, &UA2893420},
	{SG44504, &UA2341352},
	{SG44505, &UA2545845},
	{SG44506, &UA2469879},
	{SG44508, &UA2601586},
	{SG44510, &UA2784688},
	{SG44512, &UA2783065},
	{SG44514, &UA2782338},
	{SG44519, &UA3083175},
	{SG44525, &UA2385763},
	{SG44526, &UA2425105},
	{SG44527, &UA2458912},
	{SG44528, &UA2554775},
	{SG44529, &UA2854937},
	{SG44530, &UA2674208},
	{SG44531, &UA2677000},
	{SG44537, &UA2417128},
	{SG44538, &UA2382780},
	{SG44539, &UA2423037},
	{SG44540, &UA2423127},
	{SG44541, &UA2593615},
	{SG44542, &UA2776404},
	{SG44544, &UA2600839},
	{SG44546, &UA2603899},
	{SG44547, &UA2675517},
	{SG44549, &UA2796807},
	{SG44550, &UA2868690},
	{SG44551, &UA2896468},
	{SG44554, &UA3060430},
	{SG44562, &UA454865},
	{SG44565, &UA2352189},
	{SG44566, &UA2526273},
	{SG44567, &UA2364911},
	{SG44568, &UA2421749},
	{SG44569, &UA2664846},
	{SG44572, &UA2959657},
	{SG44575, &UA2895416},
	{SG44578, &UA2944226},
	{SG44579, &UA3011179},
	{SG44581, &UA3093661},
	{SG44587, &UA2484181},
	{SG44588, &UA2535585},
	{SG44589, &UA2527800},
	{SG44590, &UA2528308},
	{SG44593, &UA2567195},
	{SG44594, &UA2567969},
	{SG44595, &UA2579666},
	{SG44596, &UA2951430},
	{SG44597, &UA2747869},
	{SG44598, &UA2755408},
	{SG44603, &UA2798392},
	{SG44605, &UA2867630},
	{SG44613, &UA2548284},
	{SG44617, &UA2564068},
	{SG44620, &UA2567083},
	{SG44621, &UA2667408},
	{SG44622, &UA2673966},
	{SG44625, &UA2719120},
	{SG44626, &UA2864938},
	{SG44628, &UA2784883},
	{SG44630, &UA3059262},
	{SG44632, &UA3060400},
	{SG44637, &UA609050},
	{SG44638, &UA744780},
	{SG44639, &UA904859},
	{SG44640, &UA2027324},
	{SG44641, &UA2364927},
	{SG44642, &UA2367107},
	{SG44644, &UA2386310},
	{SG44645, &UA2389719},
	{SG44648, &UA2426724},
	{SG44657, &UA2530724},
	{SG44659, &UA2567569},
	{SG44660, &UA2671875},
	{SG44661, &UA2706768},
	{SG44663, &UA2711817},
	{SG44664, &UA2769560},
	{SG44665, &UA3023137},
	{SG44666, &UA3020161},
	{SG44667, &UA3023486},
	{SG44670, &UA2460395},
	{SG44671, &UA2234741},
	{SG44672, &UA2423870},
	{SG44673, &UA2520839},
	{SG44674, &UA2574849},
	{SG44675, &UA2692770},
	{SG44677, &UA2981943},
	{SG44681, &UA2600980},
	{SG44682, &UA2863063},
	{SG44685, &UA2673945},
	{SG44688, &UA2677185},
	{SG44690, &UA2734870},
	{SG44691, &UA2994398},
	{SG44693, &UA2783369},
	{SG44698, &UA903629},
	{SG44700, &UA2828914},
	{SG44702, &UA2231684},
	{SG44703, &UA2460936},
	{SG44705, &UA2481059},
	{SG44709, &UA2607545},
	{SG44710, &UA2675267},
	{SG44712, &UA2737067},
	{SG44713, &UA2767442},
	{SG44715, &UA2798924},
	{SG44718, &UA2780099},
	{SG44719, &UA2934943},
	{SG44721, &UA3022638},
	{SG44722, &UA3023199},
	{SG44733, &UA2026913},
	{SG44734, &UA2033680},
	{SG44736, &UA3042608},
	{SG44737, &UA2459027},
	{SG44738, &UA2503090},
	{SG44739, &UA2566929},
	{SG44740, &UA2579754},
	{SG44741, &UA2583127},
	{SG44742, &UA2583231},
	{SG44743, &UA2599948},
	{SG44744, &UA2667108},
	{SG44745, &UA2732254},
	{SG44747, &UA2771074},
	{SG44749, &UA3022178},
	{SG44752, &UA3023933},
	{SG44759, &UA2257353},
	{SG44763, &UA2389822},
	{SG44764, &UA2455280},
	{SG44765, &UA2467202},
	{SG44766, &UA2512529},
	{SG44769, &UA2602987},
	{SG44771, &UA2657444},
	{SG44774, &UA2663890},
	{SG44777, &UA2870130},
	{SG44778, &UA2961188},
	{SG44779, &UA2975564},
	{SG44782, &UA3021491},
	{SG44790, &UA2416063},
	{SG44791, &UA2426234},
	{SG44794, &UA2839861},
	{SG44797, &UA2527752},
	{SG44799, &UA2531548},
	{SG44800, &UA3108110},
	{SG44801, &UA2844094},
	{SG44802, &UA2580135},
	{SG44804, &UA2664327},
	{SG44806, &UA2784841},
	{SG44808, &UA2943642},
	{SG44815, &UA608808},
	{SG44816, &UA679875},
	{SG44819, &UA899441},
	{SG44820, &UA2445785},
	{SG44821, &UA1958036},
	{SG44822, &UA2347660},
	{SG44824, &UA2795219},
	{SG44827, &UA2365961},
	{SG44866, &UA2601027},
	{SG44867, &UA2584088},
	{SG44868, &UA2670913},
	{SG44872, &UA3020935},
	{SG44873, &UA3080493},
	{SG44878, &UA2452744},
	{SG44881, &UA2536107},
	{SG44882, &UA2558290},
	{SG44884, &UA2583126},
	{SG44885, &UA2583963},
	{SG44887, &UA2716665},
	{SG44889, &UA2737467},
	{SG44891, &UA2754172},
	{SG44898, &UA2028905},
	{SG44899, &UA2469201},
	{SG44900, &UA2479213},
	{SG44901, &UA2822056},
	{SG44903, &UA3117687},
	{SG44905, &UA2580635},
	{SG44906, &UA2676174},
	{SG44908, &UA2717868},
	{SG44911, &UA2795861},
	{SG44916, &UA3009233},
	{SG44918, &UA3022837},
	{SG44919, &UA3082429},
	{SG44921, &UA2027392},
	{SG44927, &UA2402776},
	{SG44928, &UA2430434},
	{SG44930, &UA2536786},
	{SG44933, &UA2764382},
	{SG44935, &UA2556096},
	{SG44937, &UA2879009},
	{SG44938, &UA2583294},
	{SG44940, &UA2670751},
	{SG44941, &UA2676292},
	{SG44942, &UA2677226},
	{SG44945, &UA2737409},
	{SG44946, &UA2768460},
	{SG44949, &UA2769609},
	{SG44950, &UA2782848},
	{SG44951, &UA3022697},
	{SG44954, &UA1987052},
	{SG44955, &UA2238818},
	{SG44960, &UA2384469},
	{SG44962, &UA2400978},
	{SG44964, &UA2557228},
	{SG44965, &UA2583273},
	{SG44967, &UA2580089},
	{SG44968, &UA2693748},
	{SG44969, &UA2583184},
	{SG44971, &UA2782260},
	{SG44972, &UA2850198},
	{SG44973, &UA2794946},
	{SG44974, &UA3108787},
	{SG44976, &UA3031335},
	{SG44977, &UA3021723},
	{SG44978, &UA3023792},
	{SG44979, &UA3057747},
	{SG44980, &UA3083073},
	{SG44988, &UA2026125},
	{SG44990, &UA2415696},
	{SG44993, &UA2419300},
	{SG44995, &UA2458775},
	{SG44996, &UA2557977},
	{SG44997, &UA2778693},
	{SG44998, &UA2592685},
	{SG44999, &UA2894619},
	{SG45001, &UA2607122},
	{SG45002, &UA2702623},
	{SG45003, &UA2708391},
	{SG45006, &UA2733217},
	{SG45012, &UA3081288},
	{SG45013, &UA2178006},
	{SG45017, &UA2383201},
	{SG45018, &UA2454718},
	{SG45019, &UA2505220},
	{SG45021, &UA2844041},
	{SG45022, &UA2549568},
	{SG45023, &UA2982284},
	{SG45024, &UA2583960},
	{SG45025, &UA2602398},
	{SG45028, &UA2751623},
	{SG45033, &UA2870077},
	{SG45034, &UA2999600},
	{SG45036, &UA3022595},
	{SG45038, &UA3021444},
	{SG45040, &UA3022849},
	{SG45041, &UA3023518},
	{SG45042, &UA3022219},
	{SG45058, &UA2423655},
	{SG45060, &UA2429482},
	{SG45062, &UA2703862},
	{SG45063, &UA2795772},
	{SG45064, &UA2961362},
	{SG45065, &UA2867307},
	{SG45066, &UA2867838},
	{SG45068, &UA2867762},
	{SG45071, &UA3022055},
	{SG45076, &UA3020565},
	{SG45078, &UA3081195},
	{SG45082, &UA3094527},
	{SG45084, &UA3115337},
	{SG45085, &UA2400249},
	{SG45086, &UA2656512},
	{SG45087, &UA2815709},
	{SG45090, &UA1957398},
	{SG45091, &UA1970534},
	{SG45093, &UA2397774},
	{SG45094, &UA2461336},
	{SG45097, &UA2663145},
	{SG45098, &UA2774332},
	{SG45099, &UA2866941},
	{SG45100, &UA2755997},
	{SG45103, &UA2869279},
	{SG45104, &UA2869789},
	{SG45105, &UA3022970},
	{SG45106, &UA3094498},
	{SG45107, &UA3081619},
	{SG45118, &UA2223593},
	{SG45123, &UA2506979},
	{SG45125, &UA2459482},
	{SG45128, &UA2749368},
	{SG45130, &UA2782827},
	{SG45131, &UA2782662},
	{SG45132, &UA3022814},
	{SG45141, &UA909872},
	{SG45143, &UA2030342},
	{SG45145, &UA2421326},
	{SG45149, &UA2579799},
	{SG45150, &UA2659114},
	{SG45151, &UA2696587},
	{SG45153, &UA2705985},
	{SG45159, &UA2784794},
	{SG45162, &UA2863268},
	{SG45165, &UA3021133},
	{SG45166, &UA3020802},
	{SG45167, &UA3059077},
	{SG45168, &UA3094115},
	{SG45182, &UA2033566},
	{SG45184, &UA2235556},
	{SG45186, &UA2421024},
	{SG45187, &UA2424574},
	{SG45188, &UA2485533},
	{SG45190, &UA2507564},
	{SG45196, &UA2578747},
	{SG45198, &UA2659359},
	{SG45203, &UA2674983},
	{SG45204, &UA2763986},
	{SG45205, &UA2784829},
	{SG45210, &UA3021754},
	{SG45211, &UA3025853},
	{SG45212, &UA3108977},
	{SG45215, &UA3131479},
	{SG45224, &UA2500737},
	{SG45227, &UA2028276},
	{SG45230, &UA2030336},
	{SG45234, &UA2368487},
	{SG45236, &UA2476660},
	{SG45238, &UA2381976},
	{SG45240, &UA2504954},
	{SG45243, &UA2671278},
	{SG45244, &UA2780899},
	{SG45245, &UA2673236},
	{SG45246, &UA2675676},
	{SG45247, &UA2702742},
	{SG45248, &UA2755555},
	{SG45249, &UA2764083},
	{SG45250, &UA2776725},
	{SG45251, &UA2769659},
	{SG45252, &UA2945791},
	{SG45253, &UA3007541},
	{SG45255, &UA3081041},
	{SG45291, &UA2594238},
	{SG45293, &UA2595794},
	{SG45294, &UA2567322},
	{SG45295, &UA2769492},
	{SG45296, &UA2772671},
	{SG45298, &UA2798679},
	{SG45299, &UA2999225},
	{SG45301, &UA2937706},
	{SG45302, &UA3015814},
	{SG45315, &UA2467341},
	{SG45318, &UA2579764},
	{SG45319, &UA2583188},
	{SG45320, &UA2863841},
	{SG45322, &UA2600838},
	{SG45323, &UA2666442},
	{SG45325, &UA2676583},
	{SG45326, &UA2709485},
	{SG45328, &UA2696002},
	{SG45329, &UA2734489},
	{SG45332, &UA2975583},
	{SG45334, &UA3023527},
	{SG45338, &UA3083589},
	{SG45344, &UA2174479},
	{SG45349, &UA2398729},
	{SG45350, &UA2481171},
	{SG45351, &UA2529454},
	{SG45358, &UA2984421},
	{SG45361, &UA2859868},
	{SG45362, &UA2965329},
	{SG45365, &UA3094617},
	{SG45368, &UA3081676},
	{SG45375, &UA2261528},
	{SG45377, &UA2420363},
	{SG45379, &UA2402189},
	{SG45381, &UA2452351},
	{SG45383, &UA2556390},
	{SG45385, &UA2601530},
	{SG45389, &UA2637386},
	{SG45391, &UA2668170},
	{SG45394, &UA2709591},
	{SG45397, &UA2769968},
	{SG45400, &UA2877902},
	{SG45402, &UA2752459},
	{SG45411, &UA3027505},
	{SG45426, &UA2501326},
	{SG45430, &UA2581218},
	{SG45432, &UA2603904},
	{SG45433, &UA2635848},
	{SG45434, &UA2676298},
	{SG45437, &UA2675277},
	{SG45438, &UA2798884},
	{SG45440, &UA2784842},
	{SG45444, &UA2769696},
	{SG45448, &UA2782733},
	{SG45450, &UA2867411},
	{SG45452, &UA2783697},
	{SG45453, &UA2799265},
	{SG45454, &UA2865906},
	{SG45455, &UA3022018},
	{SG45457, &UA3062534},
	{SG45467, &UA2350867},
	{SG45469, &UA2427790},
	{SG45470, &UA2555814},
	{SG45471, &UA2675221},
	{SG45472, &UA2579966},
	{SG45505, &UA2667008},
	{SG45511, &UA2713897},
	{SG45513, &UA2755275},
	{SG45514, &UA2769875},
	{SG45515, &UA2773002},
	{SG45519, &UA3022279},
	{SG45520, &UA3021322},
	{SG45521, &UA3021038},
	{SG45533, &UA902735},
	{SG45534, &UA2223877},
	{SG45536, &UA2379774},
	{SG45537, &UA2469202},
	{SG45538, &UA2459448},
	{SG45539, &UA2527787},
	{SG45540, &UA2555154},
	{SG45541, &UA2650549},
	{SG45542, &UA2663009},
	{SG45543, &UA2674439},
	{SG45544, &UA2706787},
	{SG45546, &UA2777803},
	{SG45547, &UA2784838},
	{SG45548, &UA2890863},
	{SG45550, &UA2989316},
	{SG45552, &UA2943391},
	{SG45554, &UA3021456},
	{SG45555, &UA3022270},
	{SG45559, &UA3062198},
	{SG45562, &UA3062358},
	{SG45565, &UA3096489},
	{SG45569, &UA2401542},
	{SG45570, &UA2643148},
	{SG45573, &UA2484714},
	{SG45574, &UA2436693},
	{SG45575, &UA2457622},
	{SG45576, &UA2480812},
	{SG45577, &UA2482828},
	{SG45578, &UA2583780},
	{SG45580, &UA2616510},
	{SG45611, &UA2731446},
	{SG45613, &UA2878800},
	{SG45616, &UA2857078},
	{SG45618, &UA3016901},
	{SG45619, &UA2939576},
	{SG45620, &UA2968157},
	{SG45622, &UA3021053},
	{SG45635, &UA2024630},
	{SG45637, &UA2257391},
	{SG45641, &UA2552938},
	{SG45643, &UA2746373},
	{SG45648, &UA2469519},
	{SG45651, &UA2529818},
	{SG45652, &UA2659248},
	{SG45653, &UA2583076},
	{SG45654, &UA2667477},
	{SG45655, &UA2707343},
	{SG45656, &UA2713316},
	{SG45657, &UA2769108},
	{SG45658, &UA2771082},
	{SG45659, &UA2975705},
	{SG45660, &UA3022857},
	{SG45661, &UA3023098},
	{SG45662, &UA3023397},
	{SG45664, &UA3092756},
	{SG45675, &UA2503584},
	{SG45679, &UA2415147},
	{SG45682, &UA2447490},
	{SG45684, &UA2549675},
	{SG45686, &UA3098289},
	{SG45688, &UA2583563},
	{SG45695, &UA2666405},
	{SG45696, &UA2734699},
	{SG45697, &UA2751582},
	{SG45698, &UA2992135},
	{SG45699, &UA2734750},
	{SG45704, &UA2755518},
	{SG45708, &UA2769175},
	{SG45710, &UA2892772},
	{SG45711, &UA2773291},
	{SG45712, &UA2866969},
	{SG45713, &UA2877433},
	{SG45715, &UA2877827},
	{SG45716, &UA2892456},
	{SG45718, &UA2959379},
	{SG45719, &UA3022416},
	{SG45723, &UA2745586},
	{SG45726, &UA673928},
	{SG45729, &UA2473877},
	{SG45732, &UA2469228},
	{SG45734, &UA2446890},
	{SG45735, &UA2456110},
	{SG45737, &UA2950596},
	{SG45738, &UA2603543},
	{SG45739, &UA2666983},
	{SG45741, &UA2708474},
	{SG45746, &UA2869264},
	{SG45747, &UA3012601},
	{SG45748, &UA3023126},
	{SG45750, &UA3023228},
	{SG45763, &UA3022456},
	{SG45765, &UA3062460},
	{SG45767, &UA3081464},
	{SG45771, &UA851443},
	{SG45772, &UA1958931},
	{SG45773, &UA2031187},
	{SG45777, &UA2578124},
	{SG45779, &UA2845418},
	{SG45783, &UA2469200},
	{SG45786, &UA2413775},
	{SG45788, &UA2442909},
	{SG45792, &UA2579844},
	{SG45795, &UA2597708},
	{SG45798, &UA2603533},
	{SG45802, &UA2706945},
	{SG45804, &UA2712726},
	{SG45805, &UA2738115},
	{SG45807, &UA2774000},
	{SG45809, &UA2944423},
	{SG45810, &UA2966961},
	{SG45812, &UA3032708},
	{SG45814, &UA3032071},
	{SG45815, &UA3059291},
	{SG45817, &UA3081191},
	{SG45823, &UA2384433},
	{SG45824, &UA2466851},
	{SG45825, &UA2468391},
	{SG45826, &UA2503385},
	{SG45827, &UA2560609},
	{SG45829, &UA2693936},
	{SG45831, &UA2834569},
	{SG45832, &UA3107907},
	{SG45833, &UA2711780},
	{SG45836, &UA2734938},
	{SG45837, &UA2865749},
	{SG45838, &UA2769388},
	{SG45841, &UA2783512},
	{SG45842, &UA2866931},
	{SG45845, &UA3081680},
	{SG45846, &UA3095744},
	{SG45847, &UA3131322},
	{SG45848, &UA3131598},
	{SG45856, &UA2480521},
	{SG45859, &UA2227001},
	{SG45860, &UA2366289},
	{SG45862, &UA2257456},
	{SG45864, &UA2354277},
	{SG45865, &UA2384382},
	{SG45867, &UA2425693},
	{SG45868, &UA2436503},
	{SG45872, &UA2467239},
	{SG45873, &UA2508152},
	{SG45875, &UA2557385},
	{SG45877, &UA2565353},
	{SG45878, &UA2731286},
	{SG45879, &UA2675218},
	{SG45880, &UA2653302},
	{SG45884, &UA2753062},
	{SG45887, &UA2957114},
	{SG45888, &UA3021594},
	{SG45890, &UA3020844},
	{SG45891, &UA3021954},
	{SG45892, &UA3015471},
	{SG45894, &UA3021245},
	{SG45896, &UA3023390},
	{SG45899, &UA3023035},
	{SG45900, &UA3024143},
	{SG45902, &UA3093627},
	{SG45911, &UA1993749},
	{SG45914, &UA2027114},
	{SG45916, &UA2033816},
	{SG45918, &UA2530112},
	{SG45919, &UA2455151},
	{SG45924, &UA2583367},
	{SG45926, &UA2598354},
	{SG45927, &UA2600010},
	{SG45929, &UA2603537},
	{SG45932, &UA2670735},
	{SG45935, &UA2672682},
	{SG45936, &UA2675518},
	{SG45937, &UA2797197},
	{SG45939, &UA2996909},
	{SG45942, &UA2781499},
	{SG45944, &UA2795968},
	{SG45947, &UA2800661},
	{SG45950, &UA2962001},
	{SG45962, &UA2235634},
	{SG45970, &UA2413444},
	{SG45972, &UA2650358},
	{SG45975, &UA2471725},
	{SG45977, &UA2534323},
	{SG45979, &UA2583696},
	{SG45981, &UA2784886},
	{SG45984, &UA2975983},
	{SG45989, &UA3080458},
	{SG45990, &UA2354784},
	{SG45992, &UA2415846},
	{SG45994, &UA2451609},
	{SG46001, &UA2583323},
	{SG46003, &UA2712278},
	{SG46006, &UA2799365},
	{SG46007, &UA2869507},
	{SG46008, &UA2866776},
	{SG46009, &UA2944324},
	{SG46010, &UA3020718},
	{SG46011, &UA2994270},
	{SG46012, &UA3020703},
	{SG46013, &UA3094710},
	{SG46014, &UA3127092},
	{SG46015, &UA3094303},
	{SG46016, &UA3083438},
	{SG46017, &UA3108625},
	{SG46019, &UA3114672},
	{SG46029, &UA2340767},
	{SG46035, &UA2600425},
	{SG46036, &UA2623154},
	{SG46037, &UA2860108},
	{SG46038, &UA2670912},
	{SG46039, &UA2677300},
	{SG46042, &UA2714569},
	{SG46043, &UA2730342},
	{SG46044, &UA2951276},
	{SG46045, &UA2754232},
	{SG46049, &UA2877372},
	{SG46050, &UA2799021},
	{SG46054, &UA3093626},
	{SG46067, &UA2535072},
	{SG46070, &UA2564221},
	{SG46071, &UA2603895},
	{SG46072, &UA2583571},
	{SG46073, &UA2597161},
	{SG46075, &UA2986378},
	{SG46076, &UA2667066},
	{SG46078, &UA2675392},
	{SG46079, &UA2862797},
	{SG46080, &UA2676481},
	{SG46081, &UA2698836},
	{SG46082, &UA2757208},
	{SG46083, &UA2782888},
	{SG46084, &UA2782909},
	{SG46085, &UA2877370},
	{SG46086, &UA2868624},
	{SG46087, &UA3050508},
	{SG46089, &UA3021730},
	{SG46090, &UA3022715},
	{SG46092, &UA3079605},
	{SG46094, &UA3094636},
	{SG46102, &UA1950795},
	{SG46110, &UA2454120},
	{SG46112, &UA2466084},
	{SG46115, &UA2555650},
	{SG46116, &UA2596568},
	{SG46117, &UA2601522},
	{SG46118, &UA2603861},
	{SG46122, &UA2655574},
	{SG46123, &UA2715761},
	{SG46125, &UA2771460},
	{SG46126, &UA2879239},
	{SG46130, &UA2963978},
	{SG46132, &UA3002964},
	{SG46137, &UA3023712},
	{SG46141, &UA3083386},
	{SG46142, &UA3111330},
	{SG46143, &UA3115064},
	{SG46153, &UA3085806},
	{SG46155, &UA2257501},
	{SG46156, &UA3120908},
	{SG46158, &UA2373355},
	{SG46159, &UA2726420},
	{SG46160, &UA2934293},
	{SG46162, &UA2482514},
	{SG46163, &UA2934222},
	{SG46165, &UA2503306},
	{SG46166, &UA2529920},
	{SG46172, &UA2677288},
	{SG46174, &UA2713935},
	{SG46175, &UA2750216},
	{SG46178, &UA2772760},
	{SG46179, &UA2773137},
	{SG46181, &UA2892820},
	{SG46182, &UA2894450},
	{SG46187, &UA3094114},
	{SG46189, &UA3094501},
	{SG46198, &UA1955095},
	{SG46199, &UA2034597},
	{SG46200, &UA2371944},
	{SG46201, &UA2419631},
	{SG46205, &UA2425821},
	{SG46207, &UA2482206},
	{SG46208, &UA2728191},
	{SG46210, &UA2526330},
	{SG46212, &UA2777459},
	{SG46215, &UA2585169},
	{SG46218, &UA2709237},
	{SG46220, &UA2711691},
	{SG46228, &UA2998799},
	{SG46229, &UA2876100},
	{SG46230, &UA3016837},
	{SG46231, &UA3023379},
	{SG46234, &UA3022844},
	{SG46235, &UA3023430},
	{SG46236, &UA3063885},
	{SG46242, &UA2400615},
	{SG46243, &UA2506015},
	{SG46244, &UA2527249},
	{SG46245, &UA2557681},
	{SG46246, &UA2583277},
	{SG46249, &UA2669541},
	{SG46250, &UA2673569},
	{SG46252, &UA2796695},
	{SG46253, &UA2861171},
	{SG46255, &UA3022020},
	{SG46257, &UA3063766},
	{SG46263, &UA827262},
	{SG46265, &UA2031936},
	{SG46267, &UA2029291},
	{SG46270, &UA2527512},
	{SG46273, &UA2601115},
	{SG46275, &UA2623099},
	{SG46279, &UA2676587},
	{SG46280, &UA2676639},
	{SG46285, &UA2755404},
	{SG46287, &UA2769371},
	{SG46291, &UA2783943},
	{SG46294, &UA2861360},
	{SG46295, &UA2889991},
	{SG46297, &UA3049316},
	{SG46300, &UA3065192},
	{SG46343, &UA2032883},
	{SG46344, &UA2338032},
	{SG46346, &UA2747249},
	{SG46348, &UA2470752},
	{SG46350, &UA2444432},
	{SG46356, &UA2706570},
	{SG46358, &UA2580674},
	{SG46359, &UA2852405},
	{SG46361, &UA2603906},
	{SG46362, &UA2676305},
	{SG46363, &UA2695610},
	{SG46364, &UA2892242},
	{SG46366, &UA3081336},
	{SG46376, &UA2239519},
	{SG46377, &UA2451908},
	{SG46379, &UA2460828},
	{SG46382, &UA2584847},
	{SG46384, &UA2676067},
	{SG46385, &UA2955690},
	{SG46387, &UA2881399},
	{SG46388, &UA2894722},
	{SG46390, &UA2752557},
	{SG46395, &UA2770053},
	{SG46397, &UA2774089},
	{SG46400, &UA2797448},
	{SG46401, &UA2850263},
	{SG46403, &UA2879593},
	{SG46405, &UA2964073},
	{SG46406, &UA3019988},
	{SG46407, &UA3061412},
	{SG46408, &UA3010415},
	{SG46409, &UA3077183},
	{SG46412, &UA3115121},
	{SG46419, &UA2683652},
	{SG46431, &UA2526700},
	{SG46432, &UA2591924},
	{SG46433, &UA2829814},
	{SG46434, &UA2366820},
	{SG46436, &UA2380101},
	{SG46437, &UA2416673},
	{SG46439, &UA2615938},
	{SG46440, &UA2626491},
	{SG46441, &UA2422480},
	{SG46442, &UA2425896},
	{SG46444, &UA2555500},
	{SG46445, &UA2583560},
	{SG46447, &UA2600971},
	{SG46448, &UA2601531},
	{SG46450, &UA2675276},
	{SG46452, &UA2675278},
	{SG46454, &UA2861725},
	{SG46459, &UA2708344},
	{SG46463, &UA2867002},
	{SG46467, &UA2768449},
	{SG46469, &UA2859713},
	{SG46470, &UA2799251},
	{SG46471, &UA3049138},
	{SG46472, &UA2866036},
	{SG46473, &UA3021570},
	{SG46474, &UA3023542},
	{SG46475, &UA3080395},
	{SG46476, &UA3094183},
	{SG46481, &UA3094842},
	{SG46483, &UA3130949},
	{SG46495, &UA2033248},
	{SG46496, &UA2257510},
	{SG46502, &UA2029449},
	{SG46504, &UA2352126},
	{SG46506, &UA2227989},
	{SG46508, &UA2234924},
	{SG46509, &UA2458097},
	{SG46511, &UA2585173},
	{SG46513, &UA2855529},
	{SG46516, &UA2654452},
	{SG46517, &UA2747286},
	{SG46519, &UA3058606},
	{SG46521, &UA2773294},
	{SG46522, &UA2783624},
	{SG46523, &UA2784000},
	{SG46524, &UA2797693},
	{SG46526, &UA2799833},
	{SG46527, &UA2967892},
	{SG46528, &UA2970545},
	{SG46529, &UA3021329},
	{SG46531, &UA3029361},
	{SG46533, &UA3062913},
	{SG46535, &UA3094531},
	{SG46544, &UA923913},
	{SG46545, &UA2023473},
	{SG46549, &UA2480779},
	{SG46554, &UA2565858},
	{SG46555, &UA2584097},
	{SG46556, &UA2593956},
	{SG46557, &UA2663323},
	{SG46558, &UA2675396},
	{SG46560, &UA2734681},
	{SG46562, &UA2748481},
	{SG46565, &UA2768362},
	{SG46566, &UA2780619},
	{SG46567, &UA2783722},
	{SG46568, &UA2783840},
	{SG46570, &UA2800657},
	{SG46571, &UA2944038},
	{SG46574, &UA3094931},
	{SG46577, &UA2032284},
	{SG46578, &UA2617431},
	{SG46579, &UA2383747},
	{SG46580, &UA2421588},
	{SG46581, &UA2426905},
	{SG46582, &UA2458119},
	{SG46583, &UA2467906},
	{SG46586, &UA2503175},
	{SG46589, &UA2504275},
	{SG46593, &UA2583324},
	{SG46596, &UA2594297},
	{SG46597, &UA2603304},
	{SG46600, &UA2610694},
	{SG46601, &UA2630500},
	{SG46606, &UA2768729},
	{SG46607, &UA2769585},
	{SG46609, &UA2771308},
	{SG46611, &UA2781439},
	{SG46612, &UA2783293},
	{SG46614, &UA2799519},
	{SG46616, &UA2858162},
	{SG46620, &UA3039495},
	{SG46621, &UA3020953},
	{SG46622, &UA3023407},
	{SG46623, &UA3093684},
	{SG46624, &UA3093664},
	{SG46625, &UA3109060},
	{SG46626, &UA3109185},
	{SG46627, &UA3126436},
	{SG46630, &UA411748},
	{SG46631, &UA440709},
	{SG46632, &UA441070},
	{SG46634, &UA610081},
	{SG46635, &UA900541},
	{SG46644, &UA2032170},
	{SG46645, &UA2257675},
	{SG46647, &UA2486824},
	{SG46648, &UA2595708},
	{SG46649, &UA2836064},
	{SG46651, &UA2599420},
	{SG46652, &UA2666850},
	{SG46674, &UA2675565},
	{SG46675, &UA2678425},
	{SG46676, &UA2799225},
	{SG46678, &UA2784252},
	{SG46681, &UA2794681},
	{SG46684, &UA2965387},
	{SG46690, &UA3022620},
	{SG46691, &UA3110307},
	{SG46693, &UA3093563},
	{SG46698, &UA819128},
	{SG46700, &UA2032263},
	{SG46701, &UA2742974},
	{SG46702, &UA924001},
	{SG46704, &UA1999156},
	{SG46705, &UA2352261},
	{SG46709, &UA2455732},
	{SG46711, &UA2417773},
	{SG46714, &UA2570936},
	{SG46715, &UA2467569},
	{SG46716, &UA2469209},
	{SG46717, &UA2481237},
	{SG46718, &UA2502530},
	{SG46722, &UA2601589},
	{SG46724, &UA2861135},
	{SG46725, &UA2863466},
	{SG46729, &UA2957808},
	{SG46731, &UA3022680},
	{SG46733, &UA3023986},
	{SG46734, &UA3024176},
	{SG46736, &UA3113633},
	{SG46737, &UA3094508},
	{SG46738, &UA3094369},
	{SG46739, &UA3094723},
	{SG46750, &UA2028336},
	{SG46751, &UA2030114},
	{SG46754, &UA2346579},
	{SG46756, &UA2414117},
	{SG46757, &UA2419925},
	{SG46760, &UA2458776},
	{SG46762, &UA2505538},
	{SG46764, &UA2536785},
	{SG46765, &UA2583224},
	{SG46766, &UA2604125},
	{SG46768, &UA2675326},
	{SG46769, &UA2710435},
	{SG46770, &UA2772535},
	{SG46771, &UA2868958},
	{SG46772, &UA2869043},
	{SG46773, &UA2892768},
	{SG46774, &UA3021373},
	{SG46775, &UA2963054},
	{SG46776, &UA3021654},
	{SG46777, &UA3022518},
	{SG46779, &UA2966602},
	{SG46782, &UA3021785},
	{SG46784, &UA3022866},
	{SG46787, &UA3094116},
	{SG46800, &UA2420754},
	{SG46807, &UA2579983},
	{SG46809, &UA2673965},
	{SG46810, &UA2657718},
	{SG46814, &UA2675526},
	{SG46818, &UA2707928},
	{SG46819, &UA2793622},
	{SG46822, &UA2752463},
	{SG46823, &UA2767366},
	{SG46824, &UA2769651},
	{SG46826, &UA2799508},
	{SG46831, &UA3016294},
	{SG46836, &UA3094092},
	{SG46838, &UA3093528},
	{SG46839, &UA3093681},
	{SG46841, &UA3094729},
	{SG46842, &UA3094708},
	{SG46853, &UA2419348},
	{SG46858, &UA2423109},
	{SG46861, &UA2465975},
	{SG46862, &UA2466136},
	{SG46863, &UA2506730},
	{SG46865, &UA2598163},
	{SG46866, &UA2643616},
	{SG46868, &UA2624754},
	{SG46869, &UA2662694},
	{SG46870, &UA2672458},
	{SG46871, &UA2672609},
	{SG46872, &UA2676655},
	{SG46873, &UA2738207},
	{SG46874, &UA2751873},
	{SG46875, &UA2869529},
	{SG46876, &UA2780288},
	{SG46877, &UA2782930},
	{SG46883, &UA2964893},
	{SG46884, &UA3022707},
	{SG46886, &UA3080134},
	{SG46898, &UA2425236},
	{SG46899, &UA2457577},
	{SG46900, &UA2580116},
	{SG46903, &UA2602076},
	{SG46904, &UA2603863},
	{SG46907, &UA2631103},
	{SG46910, &UA2671092},
	{SG46915, &UA2737781},
	{SG46916, &UA2752341},
	{SG46920, &UA2782867},
	{SG46922, &UA2784529},
	{SG46923, &UA2798397},
	{SG46925, &UA2855676},
	{SG46926, &UA2963513},
	{SG46931, &UA3059381},
	{SG46934, &UA3059258},
	{SG46936, &UA3094368},
	{SG46937, &UA3079407},
	{SG46938, &UA3109508},
	{SG46939, &UA3109659},
	{SG46943, &UA1961226},
	{SG46945, &UA2475527},
	{SG46946, &UA2355116},
	{SG46947, &UA2399949},
	{SG46948, &UA2426235},
	{SG46949, &UA2458155},
	{SG46951, &UA2507233},
	{SG46953, &UA2584851},
	{SG46958, &UA2664531},
	{SG46961, &UA2674960},
	{SG46964, &UA2708693},
	{SG46965, &UA2749352},
	{SG46966, &UA2783923},
	{SG46967, &UA2784894},
	{SG46968, &UA2872172},
	{SG46969, &UA2878488},
	{SG46970, &UA2892956},
	{SG46972, &UA2894333},
	{SG46975, &UA2999060},
	{SG46978, &UA3022319},
	{SG46979, &UA3022149},
	{SG46980, &UA3064147},
	{SG46981, &UA3082948},
	{SG46983, &UA3113319},
	{SG46985, &UA3126220},
	{SG46988, &UA771491},
	{SG46990, &UA924239},
	{SG46991, &UA906575},
	{SG46994, &UA2257751},
	{SG46996, &UA2376054},
	{SG46998, &UA2583271},
	{SG47000, &UA2623317},
	{SG47002, &UA2626484},
	{SG47005, &UA2707808},
	{SG47008, &UA2708862},
	{SG47009, &UA2735096},
	{SG47010, &UA2728085},
	{SG47011, &UA2879228},
	{SG47012, &UA2769652},
	{SG47013, &UA2771840},
	{SG47014, &UA2774111},
	{SG47016, &UA2782797},
	{SG47018, &UA2783700},
	{SG47019, &UA2799087},
	{SG47021, &UA3022906},
	{SG47022, &UA3023851},
	{SG47033, &UA2222518},
	{SG47034, &UA2350925},
	{SG47035, &UA2419098},
	{SG47040, &UA2579788},
	{SG47042, &UA2597326},
	{SG47043, &UA2850614},
	{SG47044, &UA2604175},
	{SG47045, &UA2668105},
	{SG47046, &UA2668864},
	{SG47047, &UA2794663},
	{SG47048, &UA2672293},
	{SG47049, &UA2676744},
	{SG47050, &UA2945206},
	{SG47051, &UA3024018},
	{SG47052, &UA3020567},
	{SG47053, &UA3025168},
	{SG47055, &UA3094602},
	{SG47056, &UA3094930},
	{SG47057, &UA3132705},
	{SG47068, &UA2376082},
	{SG47069, &UA2232520},
	{SG47070, &UA2362637},
	{SG47072, &UA2427415},
	{SG47074, &UA2458797},
	{SG47077, &UA2485805},
	{SG47078, &UA2538022},
	{SG47079, &UA2558333},
	{SG47081, &UA2597647},
	{SG47087, &UA2713951},
	{SG47089, &UA2849589},
	{SG47092, &UA3072861},
	{SG47097, &UA2892853},
	{SG47098, &UA2798819},
	{SG47100, &UA2798992},
	{SG47101, &UA2894240},
	{SG47102, &UA2875470},
	{SG47103, &UA2943492},
	{SG47106, &UA2944227},
	{SG47108, &UA2945982},
	{SG47111, &UA2965170},
	{SG47113, &UA2965607},
	{SG47114, &UA3020137},
	{SG47115, &UA3022567},
	{SG47116, &UA3023123},
	{SG47117, &UA3026605},
	{SG47119, &UA3032685},
	{SG47120, &UA3094801},
	{SG47121, &UA3114296},
	{SG47123, &UA3064146},
	{SG47124, &UA3066818},
	{SG47126, &UA3115336},
	{SG47130, &UA741956},
	{SG47131, &UA809979},
	{SG47132, &UA1981412},
	{SG47133, &UA2033836},
	{SG47135, &UA2028467},
	{SG47137, &UA2398563},
	{SG47138, &UA2602014},
	{SG47142, &UA2656913},
	{SG47146, &UA2667185},
	{SG47149, &UA2670609},
	{SG47151, &UA2672186},
	{SG47153, &UA2678415},
	{SG47156, &UA2699551},
	{SG47157, &UA2768346},
	{SG47158, &UA2782367},
	{SG47159, &UA2783702},
	{SG47162, &UA2958886},
	{SG47163, &UA3061783},
	{SG47165, &UA3059000},
	{SG47166, &UA3092648},
	{SG47167, &UA3028797},
	{SG47168, &UA3093565},
	{SG47170, &UA3125855},
	{SG47171, &UA3130739},
	{SG47172, &UA907709},
	{SG47173, &UA2686815},
	{SG47174, &UA2809387},
	{SG47176, &UA2028088},
	{SG47182, &UA2424305},
	{SG47183, &UA2468521},
	{SG47186, &UA2602399},
	{SG47187, &UA2653050},
	{SG47188, &UA2659535},
	{SG47189, &UA2667836},
	{SG47191, &UA2666963},
	{SG47196, &UA2737675},
	{SG47197, &UA2997832},
	{SG47198, &UA2876182},
	{SG47199, &UA3088599},
	{SG47200, &UA2892453},
	{SG47201, &UA2953517},
	{SG47202, &UA3020940},
	{SG47203, &UA3022545},
	{SG47213, &UA2529491},
	{SG47215, &UA2555305},
	{SG47217, &UA2600009},
	{SG47218, &UA2601244},
	{SG47220, &UA2667127},
	{SG47221, &UA2667832},
	{SG47222, &UA2672020},
	{SG47223, &UA2701511},
	{SG47226, &UA2708349},
	{SG47260, &UA2764843},
	{SG47262, &UA2782640},
	{SG47263, &UA2783183},
	{SG47264, &UA2797498},
	{SG47265, &UA2799249},
	{SG47266, &UA2799684},
	{SG47267, &UA2800233},
	{SG47269, &UA2878298},
	{SG47271, &UA2893372},
	{SG47273, &UA2867836},
	{SG47274, &UA2873964},
	{SG47275, &UA2879062},
	{SG47278, &UA2892674},
	{SG47279, &UA2937491},
	{SG47280, &UA3021580},
	{SG47281, &UA3059055},
	{SG47282, &UA3125404},
	{SG47283, &UA3063300},
	{SG47285, &UA3112702},
	{SG47294, &UA1993361},
	{SG47295, &UA2031690},
	{SG47297, &UA2760789},
	{SG47325, &UA2483264},
	{SG47327, &UA2536787},
	{SG47329, &UA2603902},
	{SG47331, &UA2707334},
	{SG47333, &UA2733368},
	{SG47334, &UA2798685},
	{SG47335, &UA2868545},
	{SG47336, &UA2783077},
	{SG47338, &UA2856440},
	{SG47340, &UA2964279},
	{SG47343, &UA3015591},
	{SG47344, &UA3062124},
	{SG47345, &UA3079658},
	{SG47346, &UA3065416},
	{SG47349, &UA3095261},
	{SG47351, &UA3093988},
	{SG47355, &UA1992595},
	{SG47357, &UA2378634},
	{SG47358, &UA2479306},
	{SG47361, &UA2780752},
	{SG47363, &UA2701446},
	{SG47364, &UA2731325},
	{SG47365, &UA2737533},
	{SG47367, &UA2773821},
	{SG47369, &UA2773920},
	{SG47370, &UA2781841},
	{SG47371, &UA2792983},
	{SG47372, &UA2795845},
	{SG47375, &UA2868542},
	{SG47381, &UA2963280},
	{SG47385, &UA3057821},
	{SG47386, &UA3114080},
	{SG47387, &UA3094069},
	{SG47388, &UA3109236},
	{SG47399, &UA2352190},
	{SG47401, &UA2448296},
	{SG47404, &UA2469030},
	{SG47406, &UA2502371},
	{SG47407, &UA2504750},
	{SG47408, &UA2507889},
	{SG47409, &UA2527377},
	{SG47410, &UA2555016},
	{SG47411, &UA2558643},
	{SG47412, &UA2605960},
	{SG47416, &UA2637834},
	{SG47417, &UA2654786},
	{SG47421, &UA2702965},
	{SG47422, &UA2706667},
	{SG47425, &UA3108322},
	{SG47427, &UA2712368},
	{SG47430, &UA2752654},
	{SG47431, &UA2765169},
	{SG47432, &UA2861812},
	{SG47434, &UA2781171},
	{SG47439, &UA2866509},
	{SG47440, &UA2958013},
	{SG47441, &UA3024015},
	{SG47444, &UA3065348},
	{SG47445, &UA3093685},
	{SG47454, &UA1961252},
	{SG47456, &UA2028684},
	{SG47463, &UA2483295},
	{SG47464, &UA2861937},
	{SG47465, &UA2576872},
	{SG47466, &UA2688063},
	{SG47469, &UA2667306},
	{SG47470, &UA2671477},
	{SG47472, &UA2712820},
	{SG47473, &UA2714231},
	{SG47474, &UA2714270},
	{SG47478, &UA2769767},
	{SG47480, &UA2782464},
	{SG47481, &UA2782527},
	{SG47483, &UA2799280},
	{SG47484, &UA2868580},
	{SG47485, &UA2892205},
	{SG47486, &UA3021946},
	{SG47489, &UA3021098},
	{SG47491, &UA3023781},
	{SG47494, &UA3029235},
	{SG47496, &UA3033069},
	{SG47505, &UA2341614},
	{SG47506, &UA2654536},
	{SG47507, &UA2366843},
	{SG47509, &UA2486244},
	{SG47512, &UA2507201},
	{SG47513, &UA2984215},
	{SG47514, &UA2553543},
	{SG47516, &UA2566158},
	{SG47517, &UA2584853},
	{SG47522, &UA2729601},
	{SG47524, &UA2749860},
	{SG47527, &UA2769616},
	{SG47533, &UA2944374},
	{SG47535, &UA2980937},
	{SG47536, &UA3011953},
	{SG47539, &UA3054952},
	{SG47540, &UA3022699},
	{SG47541, &UA3025179},
	{SG47542, &UA3064003},
	{SG47543, &UA3094506},
	{SG47544, &UA3080598},
	{SG47545, &UA1964335},
	{SG47546, &UA2167971},
	{SG47549, &UA2365491},
	{SG47550, &UA2467299},
	{SG47555, &UA2404960},
	{SG47556, &UA2423639},
	{SG47557, &UA2424643},
	{SG47559, &UA2478422},
	{SG47560, &UA2481526},
	{SG47561, &UA2555000},
	{SG47562, &UA2571193},
	{SG47563, &UA2580186},
	{SG47564, &UA2600036},
	{SG47565, &UA2601605},
	{SG47566, &UA2602005},
	{SG47567, &UA2656324},
	{SG47571, &UA2703795},
	{SG47574, &UA2708042},
	{SG47575, &UA2712949},
	{SG47580, &UA2783078},
	{SG47582, &UA2800087},
	{SG47584, &UA2865231},
	{SG47585, &UA2867593},
	{SG47586, &UA2868544},
	{SG47587, &UA2877779},
	{SG47588, &UA2996988},
	{SG47589, &UA2900611},
	{SG47590, &UA2966605},
	{SG47592, &UA3012941},
	{SG47596, &UA3094433},
	{SG47597, &UA3110845},
	{SG47598, &UA3109408},
	{SG47599, &UA3094511},
	{SG47601, &UA3114637},
	{SG47603, &UA3132841},
	{SG47605, &UA3132935},
	{SG47607, &UA3132947},
	{SG47623, &UA1986672},
	{SG47625, &UA2410395},
	{SG47627, &UA2445941},
	{SG47628, &UA2455736},
	{SG47629, &UA2466333},
	{SG47631, &UA2480534},
	{SG47633, &UA2501612},
	{SG47636, &UA2527543},
	{SG47639, &UA2671956},
	{SG47641, &UA2675532},
	{SG47642, &UA2676877},
	{SG47645, &UA2731743},
	{SG47646, &UA2733011},
	{SG47647, &UA2864046},
	{SG47652, &UA2765371},
	{SG47654, &UA2773853},
	{SG47655, &UA2773993},
	{SG47657, &UA2797452},
	{SG47658, &UA2800094},
	{SG47660, &UA2800665},
	{SG47661, &UA2848375},
	{SG47664, &UA2865733},
	{SG47665, &UA2877813},
	{SG47667, &UA2869424},
	{SG47668, &UA2892929},
	{SG47670, &UA2941582},
	{SG47671, &UA2966562},
	{SG47673, &UA3022959},
	{SG47674, &UA3023248},
	{SG47675, &UA3026615},
	{SG47676, &UA3061690},
	{SG47682, &UA2504290},
	{SG47683, &UA2526317},
	{SG47684, &UA2656791},
	{SG47685, &UA2661839},
	{SG47686, &UA2663403},
	{SG47687, &UA2854208},
	{SG47688, &UA2667432},
	{SG47689, &UA2707575},
	{SG47690, &UA2709093},
	{SG47692, &UA2750036},
	{SG47694, &UA2782376},
	{SG47696, &UA2783092},
	{SG47697, &UA2866022},
	{SG47698, &UA2867393},
	{SG47699, &UA2879341},
	{SG47701, &UA2965913},
	{SG47702, &UA3019198},
	{SG47705, &UA2966346},
	{SG47708, &UA3012166},
	{SG47709, &UA2971287},
	{SG47710, &UA3008813},
	{SG47711, &UA3022502},
	{SG47712, &UA3022265},
	{SG47716, &UA3079836},
	{SG47721, &UA3094309},
	{SG47724, &UA3126340},
	{SG47726, &UA3115426},
	{SG47762, &UA2505190},
	{SG47763, &UA2566033},
	{SG47767, &UA2733409},
	{SG47769, &UA2734776},
	{SG47771, &UA2755285},
	{SG47773, &UA2769350},
	{SG47775, &UA2770273},
	{SG47777, &UA2773924},
	{SG47779, &UA2782343},
	{SG47780, &UA2783449},
	{SG47782, &UA2783753},
	{SG47785, &UA2864908},
	{SG47786, &UA2958286},
	{SG47790, &UA3020419},
	{SG47800, &UA3081125},
	{SG47802, &UA3093688},
	{SG47804, &UA3095481},
	{SG47807, &UA3111528},
	{SG47810, &UA3131406},
	{SG47813, &UA765194},
	{SG47815, &UA905334},
	{SG47817, &UA2479993},
	{SG47818, &UA2423121},
	{SG47823, &UA2349408},
	{SG47825, &UA2383494},
	{SG47827, &UA2483130},
	{SG47828, &UA2506462},
	{SG47831, &UA2583269},
	{SG47832, &UA2602017},
	{SG47834, &UA2621947},
	{SG47835, &UA2653619},
	{SG47836, &UA2703704},
	{SG47837, &UA2707870},
	{SG47839, &UA2714380},
	{SG47840, &UA2724096},
	{SG47841, &UA2762675},
	{SG47843, &UA2735633},
	{SG47844, &UA2860948},
	{SG47845, &UA2863811},
	{SG47846, &UA2763785},
	{SG47847, &UA2781725},
	{SG47848, &UA2782040},
	{SG47849, &UA2800230},
	{SG47850, &UA2879702},
	{SG47855, &UA2945150},
	{SG47858, &UA2961371},
	{SG47860, &UA2961998},
	{SG47862, &UA3023456},
	{SG47863, &UA3020572},
	{SG47865, &UA3023878},
	{SG47868, &UA3093238},
	{SG47872, &UA3093550},
	{SG47875, &UA3112423},
	{SG47881, &UA2342899},
	{SG47885, &UA2398992},
	{SG47888, &UA2423613},
	{SG47889, &UA2457802},
	{SG47890, &UA2506925},
	{SG47895, &UA2600973},
	{SG47896, &UA2604079},
	{SG47899, &UA2631582},
	{SG47900, &UA2636712},
	{SG47901, &UA3101820},
	{SG47903, &UA2672979},
	{SG47904, &UA2708969},
	{SG47905, &UA2709757},
	{SG47906, &UA2863006},
	{SG47907, &UA2733170},
	{SG47908, &UA2733701},
	{SG47909, &UA2751289},
	{SG47910, &UA2865933},
	{SG47912, &UA2799254},
	{SG47916, &UA2858174},
	{SG47918, &UA2877476},
	{SG47924, &UA3022606},
	{SG47928, &UA3023413},
	{SG47930, &UA3079622},
	{SG47932, &UA3026387},
	{SG47934, &UA3111517},
	{SG47935, &UA3083718},
	{SG47936, &UA3109411},
	{SG47945, &UA889246},
	{SG47946, &UA1996180},
	{SG47948, &UA2027996},
	{SG47954, &UA2403510},
	{SG47955, &UA2825431},
	{SG47958, &UA2416330},
	{SG47959, &UA2427157},
	{SG47962, &UA2466237},
	{SG47965, &UA2487233},
	{SG47968, &UA2528558},
	{SG47970, &UA2559870},
	{SG47971, &UA2577900},
	{SG47973, &UA2836749},
	{SG47975, &UA2581461},
	{SG47976, &UA2595353},
	{SG47978, &UA2670899},
	{SG47979, &UA2677222},
	{SG47981, &UA2696104},
	{SG47982, &UA2828094},
	{SG47983, &UA2700165},
	{SG47985, &UA2734626},
	{SG47987, &UA2773162},
	{SG47988, &UA2792930},
	{SG47990, &UA2797001},
	{SG47991, &UA2860815},
	{SG47992, &UA2879310},
	{SG47995, &UA2945156},
	{SG47996, &UA3059702},
	{SG47997, &UA3021366},
	{SG48011, &UA2025122},
	{SG48012, &UA2027149},
	{SG48013, &UA2413315},
	{SG48016, &UA2376008},
	{SG48019, &UA2415565},
	{SG48021, &UA2455789},
	{SG48022, &UA2466282},
	{SG48023, &UA2485812},
	{SG48026, &UA2505583},
	{SG48028, &UA2554489},
	{SG48033, &UA2666987},
	{SG48036, &UA2668589},
	{SG48038, &UA2712205},
	{SG48039, &UA2738775},
	{SG48040, &UA2749428},
	{SG48041, &UA2752126},
	{SG48042, &UA2764971},
	{SG48043, &UA2769503},
	{SG48044, &UA2772866},
	{SG48045, &UA2777418},
	{SG48046, &UA2779189},
	{SG48048, &UA2783307},
	{SG48049, &UA2784103},
	{SG48050, &UA2867488},
	{SG48054, &UA2800574},
	{SG48056, &UA3100610},
	{SG48059, &UA2879466},
	{SG48062, &UA2944361},
	{SG48066, &UA3023415},
	{SG48067, &UA3023665},
	{SG48068, &UA3094608},
	{SG48070, &UA3112540},
	{SG48071, &UA3111301},
	{SG48075, &UA2257667},
	{SG48076, &UA2376117},
	{SG48077, &UA2425132},
	{SG48078, &UA2455603},
	{SG48079, &UA3099610},
	{SG48080, &UA2502360},
	{SG48081, &UA2528513},
	{SG48082, &UA2558340},
	{SG48083, &UA2564219},
	{SG48085, &UA2579883},
	{SG48089, &UA2600967},
	{SG48091, &UA2605522},
	{SG48092, &UA2633951},
	{SG48093, &UA2783287},
	{SG48094, &UA2667827},
	{SG48096, &UA2667912},
	{SG48097, &UA2670731},
	{SG48098, &UA2676995},
	{SG48100, &UA2736801},
	{SG48104, &UA2779565},
	{SG48106, &UA2779802},
	{SG48107, &UA2782425},
	{SG48108, &UA2796797},
	{SG48109, &UA2956688},
	{SG48111, &UA2982855},
	{SG48112, &UA3007228},
	{SG48128, &UA3079260},
	{SG48129, &UA3059167},
	{SG48131, &UA3112421},
	{SG48132, &UA3079109},
	{SG48138, &UA3112103},
	{SG48146, &UA2454092},
	{SG48148, &UA2467921},
	{SG48149, &UA2472080},
	{SG48151, &UA2477552},
	{SG48152, &UA2483002},
	{SG48153, &UA2497035},
	{SG48155, &UA2506765},
	{SG48159, &UA2603263},
	{SG48160, &UA2603529},
	{SG48161, &UA2661083},
	{SG48162, &UA2667174},
	{SG48163, &UA2988370},
	{SG48164, &UA2668983},
	{SG48165, &UA2670603},
	{SG48166, &UA2708788},
	{SG48167, &UA2732727},
	{SG48168, &UA2751517},
	{SG48169, &UA2762189},
	{SG48170, &UA3098263},
	{SG48172, &UA2768397},
	{SG48174, &UA2769733},
	{SG48176, &UA2772026},
	{SG48178, &UA2794168},
	{SG48182, &UA2877582},
	{SG48187, &UA2962721},
	{SG48189, &UA3002903},
	{SG48190, &UA3010716},
	{SG48192, &UA3023689},
	{SG48193, &UA3023757},
	{SG48196, &UA3028166},
	{SG48198, &UA3060347},
	{SG48199, &UA3078165},
	{SG48200, &UA3079061},
	{SG48201, &UA3079634},
	{SG48202, &UA3093263},
	{SG48210, &UA2480432},
	{SG48212, &UA2483869},
	{SG48213, &UA2527598},
	{SG48216, &UA2565743},
	{SG48218, &UA2600358},
	{SG48224, &UA2667525},
	{SG48225, &UA2671032},
	{SG48227, &UA2673058},
	{SG48230, &UA2701363},
	{SG48231, &UA2850573},
	{SG48232, &UA2707867},
	{SG48233, &UA2749858},
	{SG48235, &UA2782577},
	{SG48237, &UA2783227},
	{SG48239, &UA2833369},
	{SG48240, &UA2895019},
	{SG48243, &UA2797581},
	{SG48246, &UA2799121},
	{SG48247, &UA2799869},
	{SG48248, &UA2867244},
	{SG48249, &UA2869819},
	{SG48253, &UA3015217},
	{SG48255, &UA3021510},
	{SG48262, &UA3109199},
	{SG48267, &UA3114794},
	{SG48269, &UA3130732},
	{SG48277, &UA1954918},
	{SG48284, &UA2529917},
	{SG48286, &UA2530080},
	{SG48287, &UA2554830},
	{SG48288, &UA2565463},
	{SG48289, &UA2603538},
	{SG48290, &UA2630527},
	{SG48291, &UA2633122},
	{SG48292, &UA2993519},
	{SG48293, &UA2663763},
	{SG48295, &UA2663991},
	{SG48296, &UA2694240},
	{SG48297, &UA2987365},
	{SG48298, &UA2706772},
	{SG48300, &UA2734478},
	{SG48302, &UA2745218},
	{SG48303, &UA2778614},
	{SG48306, &UA2782810},
	{SG48308, &UA2798823},
	{SG48310, &UA2798898},
	{SG48312, &UA2800683},
	{SG48315, &UA2937975},
	{SG48317, &UA2952920},
	{SG48318, &UA2958491},
	{SG48320, &UA2958496},
	{SG48323, &UA3017481},
	{SG48325, &UA3079576},
	{SG48328, &UA3022296},
	{SG48330, &UA3023306},
	{SG48331, &UA3078523},
	{SG48332, &UA3109847},
	{SG48334, &UA3126566},
	{SG48336, &UA3130736},
	{SG48338, &UA3132847},
	{SG48354, &UA1961150},
	{SG48356, &UA2399265},
	{SG48357, &UA2430094},
	{SG48359, &UA2466059},
	{SG48360, &UA2480778},
	{SG48361, &UA2503142},
	{SG48362, &UA2506961},
	{SG48364, &UA2530305},
	{SG48365, &UA2531144},
	{SG48367, &UA2547809},
	{SG48371, &UA2595058},
	{SG48374, &UA2597606},
	{SG48375, &UA2601609},
	{SG48376, &UA2602037},
	{SG48379, &UA2613889},
	{SG48381, &UA2617793},
	{SG48382, &UA2619060},
	{SG48384, &UA2621212},
	{SG48386, &UA2622539},
	{SG48388, &UA2708907},
	{SG48389, &UA2709357},
	{SG48390, &UA2729739},
	{SG48391, &UA2751949},
	{SG48392, &UA2894214},
	{SG48393, &UA2769535},
	{SG48394, &UA2769885},
	{SG48398, &UA2861200},
	{SG48399, &UA2863371},
	{SG48403, &UA2934427},
	{SG48405, &UA2938583},
	{SG48406, &UA3000528},
	{SG48407, &UA2997552},
	{SG48410, &UA3061404},
	{SG48411, &UA3017604},
	{SG48412, &UA3021099},
	{SG48415, &UA3061037},
	{SG48418, &UA3111212},
	{SG48431, &UA2033642},
	{SG48436, &UA2257738},
	{SG48438, &UA2341862},
	{SG48440, &UA2420196},
	{SG48441, &UA2466121},
	{SG48446, &UA2553989},
	{SG48447, &UA2601162},
	{SG48448, &UA2603766},
	{SG48449, &UA2655270},
	{SG48450, &UA2667631},
	{SG48452, &UA2676925},
	{SG48455, &UA2702538},
	{SG48456, &UA2707328},
	{SG48457, &UA2710096},
	{SG48458, &UA2710422},
	{SG48460, &UA2713129},
	{SG48461, &UA2713246},
	{SG48462, &UA2733054},
	{SG48464, &UA2866342},
	{SG48466, &UA2765701},
	{SG48470, &UA2768292},
	{SG48473, &UA2799369},
	{SG48474, &UA2799517},
	{SG48479, &UA2894490},
	{SG48484, &UA2938230},
	{SG48487, &UA2961996},
	{SG48489, &UA3000872},
	{SG48490, &UA3022023},
	{SG48491, &UA3029474},
	{SG48492, &UA3093354},
	{SG48501, &UA2339954},
	{SG48503, &UA2260192},
	{SG48504, &UA2456434},
	{SG48506, &UA2503508},
	{SG48507, &UA2554383},
	{SG48508, &UA2556014},
	{SG48509, &UA2558623},
	{SG48510, &UA2579072},
	{SG48511, &UA2985998},
	{SG48513, &UA2632274},
	{SG48515, &UA2637013},
	{SG48517, &UA2623770},
	{SG48519, &UA2656533},
	{SG48520, &UA2863239},
	{SG48521, &UA2668103},
	{SG48522, &UA2668407},
	{SG48524, &UA2671987},
	{SG48525, &UA2673374},
	{SG48526, &UA2714904},
	{SG48527, &UA2782671},
	{SG48528, &UA2784092},
	{SG48530, &UA2797657},
	{SG48531, &UA2798821},
	{SG48533, &UA2853089},
	{SG48534, &UA2867074},
	{SG48535, &UA2867615},
	{SG48537, &UA2879548},
	{SG48540, &UA2952660},
	{SG48547, &UA2963609},
	{SG48549, &UA2996069},
	{SG48550, &UA2996917},
	{SG48552, &UA3013577},
	{SG48554, &UA3078645},
	{SG48555, &UA3022187},
	{SG48558, &UA3079725},
	{SG48564, &UA3093517},
	{SG48570, &UA2334385},
	{SG48571, &UA2404149},
	{SG48573, &UA2482383},
	{SG48574, &UA2486117},
	{SG48575, &UA2504800},
	{SG48576, &UA2530060},
	{SG48578, &UA2574514},
	{SG48580, &UA2766553},
	{SG48583, &UA2661763},
	{SG48585, &UA2712731},
	{SG48587, &UA2769813},
	{SG48620, &UA2798402},
	{SG48622, &UA2798561},
	{SG48626, &UA2800150},
	{SG48629, &UA2864822},
	{SG48630, &UA2869122},
	{SG48631, &UA2957610},
	{SG48632, &UA2897413},
	{SG48634, &UA2945161},
	{SG48635, &UA2957116},
	{SG48636, &UA2959700},
	{SG48637, &UA2962220},
	{SG48641, &UA3003751},
	{SG48645, &UA3012967},
	{SG48646, &UA3015677},
	{SG48647, &UA3078669},
	{SG48648, &UA3021246},
	{SG48651, &UA3021576},
	{SG48653, &UA3023677},
	{SG48654, &UA3057819},
	{SG48657, &UA3093232},
	{SG48658, &UA3079246},
	{SG48659, &UA3111717},
	{SG48660, &UA3131595},
	{SG48668, &UA2372559},
	{SG48671, &UA2422015},
	{SG48672, &UA2446491},
	{SG48675, &UA2455208},
	{SG48677, &UA2479892},
	{SG48679, &UA2487223},
	{SG48680, &UA2520558},
	{SG48681, &UA2529327},
	{SG48683, &UA2555993},
	{SG48684, &UA2556843},
	{SG48685, &UA2583314},
	{SG48686, &UA2594095},
	{SG48687, &UA2596134},
	{SG48689, &UA2600261},
	{SG48691, &UA2666890},
	{SG48693, &UA2666997},
	{SG48695, &UA2667200},
	{SG48698, &UA2707464},
	{SG48699, &UA2708233},
	{SG48700, &UA2709087},
	{SG48701, &UA2709489},
	{SG48704, &UA2791681},
	{SG48705, &UA2794156},
	{SG48706, &UA2797391},
	{SG48707, &UA2798887},
	{SG48710, &UA2957682},
	{SG48711, &UA2866547},
	{SG48713, &UA2866822},
	{SG48716, &UA2877171},
	{SG48718, &UA2933865},
	{SG48719, &UA3119586},
	{SG48721, &UA2998096},
	{SG48723, &UA2968178},
	{SG48725, &UA3015872},
	{SG48730, &UA3131867},
	{SG48742, &UA1989048},
	{SG48744, &UA2027415},
	{SG48751, &UA2340427},
	{SG48754, &UA2456367},
	{SG48755, &UA2789081},
	{SG48757, &UA2529127},
	{SG48758, &UA2555417},
	{SG48760, &UA2652769},
	{SG48761, &UA2659953},
	{SG48763, &UA2665402},
	{SG48766, &UA2667407},
	{SG48769, &UA2701373},
	{SG48773, &UA2710640},
	{SG48777, &UA2734755},
	{SG48780, &UA2783102},
	{SG48782, &UA2799963},
	{SG48784, &UA2825350},
	{SG48785, &UA2850268},
	{SG48786, &UA2877108},
	{SG48787, &UA2984253},
	{SG48788, &UA3024022},
	{SG48789, &UA3028232},
	{SG48793, &UA3064438},
	{SG48796, &UA3094588},
	{SG48799, &UA3109369},
	{SG48800, &UA3127211},
	{SG48802, &UA3128942},
	{SG48804, &UA3132945},
	{SG48811, &UA2554272},
	{SG48812, &UA2543988},
	{SG48813, &UA2383192},
	{SG48814, &UA2413220},
	{SG48815, &UA2452566},
	{SG48817, &UA2484092},
	{SG48819, &UA2504808},
	{SG48822, &UA2529915},
	{SG48823, &UA2551952},
	{SG48828, &UA2598588},
	{SG48829, &UA2599237},
	{SG48830, &UA2618184},
	{SG48831, &UA2663802},
	{SG48832, &UA2706793},
	{SG48833, &UA2707326},
	{SG48835, &UA2709084},
	{SG48838, &UA2766961},
	{SG48840, &UA2863510},
	{SG48843, &UA2781882},
	{SG48847, &UA2784107},
	{SG48851, &UA2799807},
	{SG48856, &UA2869277},
	{SG48857, &UA2876321},
	{SG48858, &UA2895959},
	{SG48861, &UA2962042},
	{SG48866, &UA3022968},
	{SG48869, &UA3108786},
	{SG48871, &UA3092985},
	{SG48873, &UA3109026},
	{SG48875, &UA3115050},
	{SG48876, &UA3115607},
	{SG48882, &UA606851},
	{SG48888, &UA2454242},
	{SG48891, &UA2485005},
	{SG48895, &UA2505861},
	{SG48898, &UA2536650},
	{SG48900, &UA2536889},
	{SG48901, &UA2554285},
	{SG48902, &UA2583788},
	{SG48903, &UA2602006},
	{SG48905, &UA2603901},
	{SG48912, &UA2666985},
	{SG48914, &UA2676836},
	{SG48917, &UA2677693},
	{SG48918, &UA2697561},
	{SG48919, &UA2698928},
	{SG48920, &UA2707385},
	{SG48921, &UA2713330},
	{SG48925, &UA2992311},
	{SG48926, &UA2767894},
	{SG48927, &UA2799708},
	{SG48928, &UA2861050},
	{SG48929, &UA2994006},
	{SG48930, &UA2867208},
	{SG48931, &UA2868512},
	{SG48933, &UA2937375},
	{SG48934, &UA3000615},
	{SG48936, &UA2958187},
	{SG48938, &UA2960671},
	{SG48941, &UA3000140},
	{SG48942, &UA3000726},
	{SG48944, &UA3013145},
	{SG48945, &UA3022014},
	{SG48953, &UA2749439},
	{SG48955, &UA2892636},
	{SG48957, &UA1977876},
	{SG48959, &UA2957722},
	{SG48961, &UA1991241},
	{SG48963, &UA2030027},
	{SG48967, &UA2353319},
	{SG48969, &UA2418081},
	{SG48970, &UA2423407},
	{SG48972, &UA2504826},
	{SG48973, &UA2528527},
	{SG48974, &UA2657558},
	{SG48976, &UA2555510},
	{SG48977, &UA2583319},
	{SG48979, &UA2591883},
	{SG48980, &UA2596173},
	{SG48982, &UA2661676},
	{SG48985, &UA2668990},
	{SG48989, &UA2701468},
	{SG48990, &UA2708426},
	{SG48995, &UA2799032},
	{SG48997, &UA2834786},
	{SG48998, &UA3043178},
	{SG49000, &UA2868642},
	{SG49001, &UA2869047},
	{SG49002, &UA2870147},
	{SG49003, &UA2897844},
	{SG49005, &UA2951028},
	{SG49010, &UA2997822},
	{SG49012, &UA2999938},
	{SG49014, &UA3014743},
	{SG49015, &UA3023245},
	{SG49016, &UA3023516},
	{SG49018, &UA3049484},
	{SG49020, &UA3092752},
	{SG49030, &UA2691304},
	{SG49031, &UA2222924},
	{SG49033, &UA2617480},
	{SG49034, &UA2473741},
	{SG49040, &UA2667106},
	{SG49041, &UA2668496},
	{SG49042, &UA2670780},
	{SG49043, &UA2705936},
	{SG49044, &UA2707373},
	{SG49046, &UA2710731},
	{SG49047, &UA2713571},
	{SG49048, &UA2713948},
	{SG49049, &UA2733007},
	{SG49050, &UA2733493},
	{SG49054, &UA2767044},
	{SG49055, &UA2783100},
	{SG49057, &UA2796200},
	{SG49059, &UA2845270},
	{SG49060, &UA2867276},
	{SG49088, &UA2869094},
	{SG49089, &UA2877367},
	{SG49092, &UA2963721},
	{SG49095, &UA3016083},
	{SG49096, &UA3018757},
	{SG49097, &UA3020570},
	{SG49098, &UA3023620},
	{SG49099, &UA3080396},
	{SG49101, &UA3111445},
	{SG49105, &UA3130731},
	{SG49125, &UA2260279},
	{SG49126, &UA2404900},
	{SG49127, &UA2414846},
	{SG49128, &UA2478236},
	{SG49129, &UA2481941},
	{SG49131, &UA2555031},
	{SG49136, &UA2579179},
	{SG49138, &UA2602989},
	{SG49139, &UA2837808},
	{SG49140, &UA2656313},
	{SG49141, &UA2835416},
	{SG49143, &UA2656723},
	{SG49144, &UA2661418},
	{SG49146, &UA2667553},
	{SG49148, &UA2668276},
	{SG49150, &UA2675512},
	{SG49152, &UA2696536},
	{SG49154, &UA2711002},
	{SG49156, &UA2734297},
	{SG49159, &UA2734806},
	{SG49197, &UA2868836},
	{SG49199, &UA2869871},
	{SG49202, &UA2892548},
	{SG49204, &UA2892822},
	{SG49207, &UA2957239},
	{SG49208, &UA2963727},
	{SG49209, &UA2999297},
	{SG49210, &UA3010201},
	{SG49212, &UA3011298},
	{SG49214, &UA3021571},
	{SG49216, &UA3021947},
	{SG49217, &UA3022209},
	{SG49223, &UA3049783},
	{SG49224, &UA3088668},
	{SG49225, &UA3050172},
	{SG49233, &UA3111566},
	{SG49244, &UA2034109},
	{SG49245, &UA2451535},
	{SG49246, &UA2460289},
	{SG49248, &UA2469392},
	{SG49250, &UA2503916},
	{SG49252, &UA2505400},
	{SG49254, &UA2520602},
	{SG49256, &UA2558510},
	{SG49257, &UA2659251},
	{SG49258, &UA2668830},
	{SG49259, &UA2675501},
	{SG49260, &UA2694288},
	{SG49261, &UA2698965},
	{SG49262, &UA2708274},
	{SG49263, &UA2709410},
	{SG49264, &UA2710089},
	{SG49266, &UA2717042},
	{SG49268, &UA2762993},
	{SG49269, &UA2768779},
	{SG49270, &UA2782845},
	{SG49273, &UA2796428},
	{SG49274, &UA2798982},
	{SG49276, &UA2847890},
	{SG49279, &UA2865675},
	{SG49281, &UA2867028},
	{SG49282, &UA2867229},
	{SG49284, &UA2867780},
	{SG49285, &UA2868166},
	{SG49287, &UA2879235},
	{SG49289, &UA2891797},
	{SG49292, &UA2997852},
	{SG49293, &UA3021371},
	{SG49294, &UA3057860},
	{SG49296, &UA3022184},
	{SG49298, &UA3023921},
	{SG49299, &UA3059104},
	{SG49300, &UA3086192},
	{SG49302, &UA3093828},
	{SG49303, &UA3103166},
	{SG49304, &UA3121782},
	{SG49305, &UA3116532},
	{SG49306, &UA3131321},
	{SG49321, &UA2025034},
	{SG49322, &UA2722335},
	{SG49326, &UA2026500},
	{SG49331, &UA2503536},
	{SG49332, &UA2529819},
	{SG49333, &UA2558288},
	{SG49334, &UA2596947},
	{SG49335, &UA2602988},
	{SG49337, &UA2633782},
	{SG49339, &UA2658609},
	{SG49343, &UA2666717},
	{SG49345, &UA2667244},
	{SG49346, &UA2676879},
	{SG49348, &UA2686879},
	{SG49350, &UA2707000},
	{SG49352, &UA2708421},
	{SG49354, &UA2708664},
	{SG49355, &UA2709760},
	{SG49356, &UA2713253},
	{SG49358, &UA2726589},
	{SG49361, &UA2729047},
	{SG49364, &UA2765475},
	{SG49365, &UA2769435},
	{SG49366, &UA2769798},
	{SG49367, &UA2782842},
	{SG49370, &UA2848369},
	{SG49372, &UA2869280},
	{SG49373, &UA2879459},
	{SG49374, &UA2892658},
	{SG49377, &UA2956366},
	{SG49383, &UA3007083},
	{SG49384, &UA3015404},
	{SG49387, &UA3023829},
	{SG49390, &UA3109433},
	{SG49391, &UA3111852},
	{SG49408, &UA2026554},
	{SG49409, &UA2813086},
	{SG49414, &UA2506792},
	{SG49418, &UA2525580},
	{SG49419, &UA2530183},
	{SG49423, &UA2549211},
	{SG49426, &UA2555295},
	{SG49428, &UA2643349},
	{SG49429, &UA2659157},
	{SG49431, &UA2659529},
	{SG49432, &UA2660100},
	{SG49434, &UA2667433},
	{SG49435, &UA2668269},
	{SG49436, &UA2668852},
	{SG49437, &UA2708264},
	{SG49438, &UA2711687},
	{SG49439, &UA2714049},
	{SG49440, &UA2733893},
	{SG49441, &UA2734565},
	{SG49442, &UA2734745},
	{SG49443, &UA2734839},
	{SG49444, &UA2735516},
	{SG49446, &UA2768531},
	{SG49448, &UA2769493},
	{SG49449, &UA2783721},
	{SG49450, &UA2796842},
	{SG49452, &UA2843855},
	{SG49454, &UA3099392},
	{SG49456, &UA2868929},
	{SG49459, &UA2869927},
	{SG49461, &UA2877769},
	{SG49466, &UA2935749},
	{SG49468, &UA2957899},
	{SG49471, &UA2963977},
	{SG49476, &UA3004195},
	{SG49478, &UA3016630},
	{SG49482, &UA3017571},
	{SG49486, &UA3023628},
	{SG49487, &UA3023658},
	{SG49488, &UA3023924},
	{SG49501, &UA1991352},
	{SG49505, &UA2414437},
	{SG49507, &UA2426898},
	{SG49510, &UA2466419},
	{SG49511, &UA2494526},
	{SG49512, &UA2505083},
	{SG49514, &UA2526178},
	{SG49515, &UA2548265},
	{SG49516, &UA2686379},
	{SG49518, &UA2578060},
	{SG49519, &UA2579886},
	{SG49520, &UA2580328},
	{SG49521, &UA2583168},
	{SG49522, &UA2598309},
	{SG49524, &UA2601200},
	{SG49525, &UA2665122},
	{SG49527, &UA2668260},
	{SG49530, &UA2696853},
	{SG49531, &UA2707865},
	{SG49532, &UA2713804},
	{SG49533, &UA2713877},
	{SG49536, &UA2714302},
	{SG49539, &UA2729136},
	{SG49540, &UA2732075},
	{SG49545, &UA2735923},
	{SG49547, &UA2767673},
	{SG49550, &UA2773408},
	{SG49552, &UA2792170},
	{SG49554, &UA2799525},
	{SG49557, &UA2865604},
	{SG49559, &UA2875559},
	{SG49560, &UA2877354},
	{SG49561, &UA2877422},
	{SG49563, &UA2934527},
	{SG49567, &UA2960477},
	{SG49568, &UA2970566},
	{SG49569, &UA2990370},
	{SG49570, &UA2999335},
	{SG49571, &UA3010986},
	{SG49572, &UA3014228},
	{SG49574, &UA3022213},
	{SG49576, &UA3023119},
	{SG49582, &UA3056099},
	{SG49586, &UA3059380},
	{SG49590, &UA3127297},
	{SG49593, &UA3113636},
	{SG49603, &UA2026273},
	{SG49610, &UA2349453},
	{SG49613, &UA2454030},
	{SG49614, &UA2455122},
	{SG49616, &UA2475801},
	{SG49617, &UA2479282},
	{SG49621, &UA2507344},
	{SG49622, &UA2524523},
	{SG49623, &UA2527692},
	{SG49624, &UA2527839},
	{SG49625, &UA2528831},
	{SG49628, &UA2646292},
	{SG49631, &UA2667186},
	{SG49632, &UA2667243},
	{SG49633, &UA2667485},
	{SG49634, &UA2668278},
	{SG49635, &UA2668629},
	{SG49639, &UA2708663},
	{SG49641, &UA2710094},
	{SG49642, &UA2712456},
	{SG49644, &UA2728063},
	{SG49648, &UA2735003},
	{SG49649, &UA2735222},
	{SG49651, &UA2746613},
	{SG49655, &UA2844728},
	{SG49656, &UA2792265},
	{SG49658, &UA2797082},
	{SG49661, &UA2797699},
	{SG49662, &UA2849084},
	{SG49663, &UA2867284},
	{SG49664, &UA2869217},
	{SG49665, &UA2870042},
	{SG49666, &UA2877922},
	{SG49667, &UA2879479},
	{SG49668, &UA2879559},
	{SG49669, &UA2938126},
	{SG49671, &UA2957742},
	{SG49672, &UA2958396},
	{SG49674, &UA2969482},
	{SG49675, &UA2996610},
	{SG49678, &UA3015830},
	{SG49679, &UA3021232},
	{SG49681, &UA3021644},
	{SG49682, &UA3022372},
	{SG49684, &UA3059126},
	{SG49685, &UA3060351},
	{SG49686, &UA3079485},
	{SG49689, &UA3132637},
	{SG49699, &UA1984854},
	{SG49702, &UA2417839},
	{SG49703, &UA2425768},
	{SG49704, &UA2426922},
	{SG49705, &UA2506747},
	{SG49711, &UA2526878},
	{SG49712, &UA2578988},
	{SG49714, &UA2599952},
	{SG49716, &UA2600346},
	{SG49718, &UA2623748},
	{SG49719, &UA2652655},
	{SG49720, &UA2660756},
	{SG49721, &UA2664427},
	{SG49722, &UA2676918},
	{SG49724, &UA2701448},
	{SG49725, &UA2706616},
	{SG49729, &UA2712725},
	{SG49730, &UA2732967},
	{SG49732, &UA2735186},
	{SG49733, &UA2749917},
	{SG49740, &UA2845857},
	{SG49742, &UA2860586},
	{SG49747, &UA2869415},
	{SG49749, &UA2877355},
	{SG49753, &UA2892462},
	{SG49754, &UA2892876},
	{SG49755, &UA2937477},
	{SG49756, &UA2962225},
	{SG49757, &UA2986887},
	{SG49758, &UA3006108},
	{SG49759, &UA3022092},
	{SG49760, &UA3022382},
	{SG49761, &UA3022710},
	{SG49762, &UA3022694},
	{SG49763, &UA3022975},
	{SG49765, &UA3023615},
	{SG49766, &UA3024096},
	{SG49767, &UA3055168},
	{SG49771, &UA3108425},
	{SG49773, &UA3109131},
	{SG49774, &UA3109169},
	{SG49781, &UA2352196},
	{SG49784, &UA2261509},
	{SG49786, &UA2384338},
	{SG49789, &UA2504607},
	{SG49791, &UA2506553},
	{SG49794, &UA2527432},
	{SG49796, &UA2528175},
	{SG49798, &UA2528580},
	{SG49799, &UA2528852},
	{SG49803, &UA2559576},
	{SG49805, &UA2560316},
	{SG49808, &UA2598073},
	{SG49810, &UA2599246},
	{SG49811, &UA2603540},
	{SG49812, &UA2631466},
	{SG49814, &UA2667107},
	{SG49815, &UA2698846},
	{SG49816, &UA2708047},
	{SG49817, &UA2713517},
	{SG49818, &UA2713566},
	{SG49820, &UA2735145},
	{SG49822, &UA2747183},
	{SG49824, &UA2762953},
	{SG49826, &UA2779991},
	{SG49827, &UA2797196},
	{SG49828, &UA2798855},
	{SG49830, &UA2839149},
	{SG49833, &UA2867550},
	{SG49835, &UA2867851},
	{SG49838, &UA2869926},
	{SG49839, &UA2877773},
	{SG49841, &UA2892643},
	{SG49842, &UA2892916},
	{SG49844, &UA2893332},
	{SG49845, &UA2896740},
	{SG49846, &UA2953440},
	{SG49847, &UA2987580},
	{SG49848, &UA2997016},
	{SG49849, &UA2997343},
	{SG49851, &UA2999767},
	{SG49853, &UA3000776},
	{SG49854, &UA3007443},
	{SG49855, &UA3009916},
	{SG49856, &UA3010927},
	{SG49857, &UA3013621},
	{SG49859, &UA3021950},
	{SG49889, &UA3022364},
	{SG49890, &UA3023631},
	{SG49892, &UA3024024},
	{SG49894, &UA3110544},
	{SG49911, &UA3110929},
	{SG49912, &UA3114558},
	{SG49914, &UA3115173},
	{SG49915, &UA3115178},
	{SG49929, &UA2034543},
	{SG49933, &UA1958482},
	{SG49934, &UA2033451},
	{SG49935, &UA2553039},
	{SG49936, &UA2553396},
	{SG49937, &UA2667062},
	{SG49938, &UA2667114},
	{SG49940, &UA2667305},
	{SG49941, &UA2667828},
	{SG49942, &UA2668045},
	{SG49943, &UA2674848},
	{SG49945, &UA2696766},
	{SG49947, &UA2707433},
	{SG49949, &UA2728955},
	{SG49950, &UA2736944},
	{SG49951, &UA2751554},
	{SG49953, &UA2766731},
	{SG49955, &UA2768328},
	{SG49956, &UA2778789},
	{SG49957, &UA2798975},
	{SG49958, &UA2799323},
	{SG49959, &UA2799334},
	{SG49961, &UA2799806},
	{SG49963, &UA2855248},
	{SG49964, &UA2856010},
	{SG49965, &UA2857987},
	{SG49967, &UA2866910},
	{SG49968, &UA2867312},
	{SG49970, &UA2879207},
	{SG49971, &UA2890457},
	{SG49972, &UA2890962},
	{SG49974, &UA2892436},
	{SG49976, &UA2892773},
	{SG49979, &UA2892990},
	{SG49980, &UA2893071},
	{SG49982, &UA2958332},
	{SG49987, &UA2999525},
	{SG49989, &UA3003769},
	{SG49991, &UA3020843},
	{SG49995, &UA3022997},
	{SG49997, &UA3058072},
	{SG50004, &UA2365111},
	{SG50005, &UA2382313},
	{SG50006, &UA2383474},
	{SG50008, &UA2453627},
	{SG50009, &UA2467254},
	{SG50010, &UA2478789},
	{SG50013, &UA2483458},
	{SG50015, &UA2505016},
	{SG50019, &UA2528373},
	{SG50021, &UA2546848},
	{SG50023, &UA2578639},
	{SG50024, &UA2597978},
	{SG50027, &UA2617189},
	{SG50031, &UA2636873},
	{SG50035, &UA2659218},
	{SG50037, &UA2660148},
	{SG50039, &UA2666683},
	{SG50041, &UA2667032},
	{SG50047, &UA2703910},
	{SG50050, &UA2707135},
	{SG50053, &UA2709762},
	{SG50056, &UA2734728},
	{SG50057, &UA2735426},
	{SG50059, &UA2735622},
	{SG50061, &UA2750082},
	{SG50065, &UA2783007},
	{SG50066, &UA2792356},
	{SG50067, &UA2799522},
	{SG50068, &UA2800046},
	{SG50069, &UA2847737},
	{SG50070, &UA2850661},
	{SG50073, &UA2860466},
	{SG50074, &UA2865046},
	{SG50075, &UA2865274},
	{SG50079, &UA2877239},
	{SG50080, &UA2879338},
	{SG50081, &UA2879555},
	{SG50082, &UA2879676},
	{SG50083, &UA2892377},
	{SG50085, &UA2892899},
	{SG50086, &UA2937825},
	{SG50090, &UA2963422},
	{SG50093, &UA2998445},
	{SG50094, &UA2998468},
	{SG50097, &UA2999791},
	{SG50099, &UA3000352},
	{SG50101, &UA3015136},
	{SG50102, &UA3015772},
	{SG50103, &UA3020518},
	{SG50104, &UA3021650},
	{SG50105, &UA3021658},
	{SG50107, &UA3021925},
	{SG50109, &UA3022099},
	{SG50110, &UA3022737},
	{SG50118, &UA3058847},
	{SG50121, &UA3074316},
	{SG50128, &UA3110039},
	{SG50134, &UA3129102},
	{SG50136, &UA3131342},
	{SG50143, &UA2366159},
	{SG50146, &UA2455655},
	{SG50148, &UA2458425},
	{SG50152, &UA2505280},
	{SG50154, &UA2505399},
	{SG50157, &UA2528030},
	{SG50158, &UA2530692},
	{SG50161, &UA2596153},
	{SG50164, &UA2658413},
	{SG50166, &UA2667596},
	{SG50167, &UA2707086},
	{SG50168, &UA2707809},
	{SG50170, &UA2707916},
	{SG50171, &UA2708146},
	{SG50173, &UA2727270},
	{SG50174, &UA2733074},
	{SG50175, &UA2734860},
	{SG50176, &UA2735225},
	{SG50177, &UA2735355},
	{SG50178, &UA2736259},
	{SG50179, &UA2746068},
	{SG50181, &UA2747261},
	{SG50182, &UA2750650},
	{SG50185, &UA2750959},
	{SG50186, &UA2751009},
	{SG50193, &UA2780841},
	{SG50197, &UA2851110},
	{SG50199, &UA2855189},
	{SG50200, &UA2856364},
	{SG50201, &UA2863461},
	{SG50203, &UA2867381},
	{SG50206, &UA2892523},
	{SG50209, &UA2892751},
	{SG50210, &UA2958370},
	{SG50212, &UA2961366},
	{SG50213, &UA2990038},
	{SG50214, &UA2990768},
	{SG50217, &UA3021145},
	{SG50218, &UA3022733},
	{SG50219, &UA3023406},
	{SG50221, &UA3057896},
	{SG50224, &UA3078354},
	{SG50230, &UA3110879},
	{SG50231, &UA3111095},
	{SG50233, &UA3116536},
	{SG50235, &UA3126266},
	{SG50236, &UA3126297},
	{SG50246, &UA2023830},
	{SG50247, &UA2027167},
	{SG50249, &UA3088773},
	{SG50254, &UA2385698},
	{SG50261, &UA2579523},
	{SG50264, &UA2601552},
	{SG50265, &UA2617158},
	{SG50266, &UA2623764},
	{SG50268, &UA2637735},
	{SG50270, &UA2657581},
	{SG50273, &UA2662618},
	{SG50276, &UA2664859},
	{SG50278, &UA2666721},
	{SG50280, &UA2667128},
	{SG50281, &UA2667189},
	{SG50282, &UA2667551},
	{SG50283, &UA2700148},
	{SG50285, &UA2702446},
	{SG50287, &UA2702708},
	{SG50288, &UA2703798},
	{SG50291, &UA2711138},
	{SG50293, &UA2728269},
	{SG50296, &UA2734546},
	{SG50298, &UA2735428},
	{SG50299, &UA2747492},
	{SG50300, &UA2749989},
	{SG50302, &UA2763880},
	{SG50304, &UA2764383},
	{SG50305, &UA2777709},
	{SG50307, &UA2790751},
	{SG50308, &UA2844881},
	{SG50309, &UA2859779},
	{SG50310, &UA2860031},
	{SG50311, &UA2867441},
	{SG50312, &UA2867463},
	{SG50313, &UA2867957},
	{SG50314, &UA2868010},
	{SG50315, &UA2868774},
	{SG50317, &UA2868819},
	{SG50319, &UA2877040},
	{SG50320, &UA2879163},
	{SG50321, &UA2879222},
	{SG50324, &UA2892854},
	{SG50325, &UA2892960},
	{SG50326, &UA2892961},
	{SG50327, &UA2893170},
	{SG50328, &UA2893672},
	{SG50330, &UA2935627},
	{SG50333, &UA2937894},
	{SG50335, &UA2952790},
	{SG50338, &UA2958544},
	{SG50341, &UA2966792},
	{SG50344, &UA2983557},
	{SG50347, &UA2989698},
	{SG50350, &UA2991155},
	{SG50353, &UA2997430},
	{SG50355, &UA2998610},
	{SG50360, &UA3009896},
	{SG50361, &UA3010847},
	{SG50362, &UA3012704},
	{SG50364, &UA3014016},
	{SG50366, &UA3020932},
	{SG50368, &UA3022611},
	{SG50369, &UA3050949},
	{SG50370, &UA3052715},
	{SG50372, &UA3078108},
	{SG50373, &UA3079805},
	{SG50374, &UA3079843},
	{SG50376, &UA3090981},
	{SG50379, &UA3109933},
	{SG50381, &UA3111923},
	{SG50382, &UA3114492},
	{SG50384, &UA3114556},
	{SG50385, &UA3131438},
	{SG50386, &UA3132919},
	{SG50404, &UA2028953},
	{SG50409, &UA2338731},
	{SG50412, &UA2529240},
	{SG50415, &UA2556136},
	{SG50416, &UA2561348},
	{SG50417, &UA2603530},
	{SG50418, &UA2613417},
	{SG50419, &UA2619769},
	{SG50420, &UA2655431},
	{SG50422, &UA2658962},
	{SG50424, &UA2659253},
	{SG50426, &UA2660599},
	{SG50429, &UA2660897},
	{SG50432, &UA2667347},
	{SG50434, &UA2667723},
	{SG50435, &UA2668104},
	{SG50436, &UA2695662},
	{SG50437, &UA2695706},
	{SG50442, &UA2708219},
	{SG50444, &UA2713572},
	{SG50445, &UA2714866},
	{SG50446, &UA2725953},
	{SG50448, &UA2734547},
	{SG50451, &UA2735558},
	{SG50455, &UA2735713},
	{SG50456, &UA2735878},
	{SG50459, &UA2746354},
	{SG50460, &UA2750051},
	{SG50461, &UA2751446},
	{SG50462, &UA2763904},
	{SG50463, &UA2781292},
	{SG50464, &UA2793096},
	{SG50465, &UA2797494},
	{SG50466, &UA2842601},
	{SG50467, &UA2842708},
	{SG50468, &UA2848972},
	{SG50469, &UA2855430},
	{SG50470, &UA2855470},
	{SG50471, &UA2857740},
	{SG50473, &UA2864906},
	{SG50474, &UA2867280},
	{SG50476, &UA2868630},
	{SG50477, &UA2879508},
	{SG50479, &UA2879565},
	{SG50481, &UA2893354},
	{SG50483, &UA2897412},
	{SG50484, &UA2937344},
	{SG50485, &UA2937350},
	{SG50487, &UA2956471},
	{SG50491, &UA2996941},
	{SG50492, &UA2997080},
	{SG50494, &UA2998942},
	{SG50495, &UA3004226},
	{SG50496, &UA3007106},
	{SG50498, &UA3008383},
	{SG50499, &UA3015491},
	{SG50501, &UA3021140},
	{SG50503, &UA3021152},
	{SG50504, &UA3021638},
	{SG50505, &UA3022087},
	{SG50507, &UA3023432},
	{SG50508, &UA3024350},
	{SG50511, &UA3058090},
	{SG50515, &UA3093128},
	{SG50519, &UA3111101},
	{SG50523, &UA3111518},
	{SG50525, &UA3114234},
	{SG50526, &UA3114559},
	{SG50528, &UA3114792},
	{SG50530, &UA3120661},
	{SG50531, &UA3127397},
	{SG50534, &UA2029610},
	{SG50536, &UA3045173},
	{SG50540, &UA2349709},
	{SG50545, &UA2422656},
	{SG50548, &UA2602004},
	{SG50553, &UA2659122},
	{SG50555, &UA2659554},
	{SG50557, &UA2659591},
	{SG50559, &UA2702091},
	{SG50560, &UA2702820},
	{SG50563, &UA2734826},
	{SG50564, &UA2734842},
	{SG50565, &UA2734915},
	{SG50566, &UA2735333},
	{SG50567, &UA2735542},
	{SG50568, &UA2735911},
	{SG50570, &UA2746585},
	{SG50574, &UA2778989},
	{SG50576, &UA2781014},
	{SG50577, &UA2792061},
	{SG50578, &UA2817508},
	{SG50580, &UA2834696},
	{SG50582, &UA2845701},
	{SG50586, &UA2860726},
	{SG50587, &UA2860852},
	{SG50588, &UA2861580},
	{SG50589, &UA2864791},
	{SG50590, &UA2867954},
	{SG50591, &UA2869693},
	{SG50592, &UA2879188},
	{SG50593, &UA2890172},
	{SG50594, &UA2892403},
	{SG50596, &UA2892794},
	{SG50598, &UA2952202},
	{SG50599, &UA2958450},
	{SG50601, &UA2995240},
	{SG50603, &UA2996899},
	{SG50606, &UA3011514},
	{SG50607, &UA3012646},
	{SG50608, &UA3014371},
	{SG50610, &UA3017874},
	{SG50611, &UA3021148},
	{SG50613, &UA3023694},
	{SG50614, &UA3023775},
	{SG50616, &UA3024121},
	{SG50617, &UA3024414},
	{SG50619, &UA3027946},
	{SG50620, &UA3030085},
	{SG50622, &UA3053713},
	{SG50625, &UA3057813},
	{SG50702, &UA3079255},
	{SG50703, &UA3079379},
	{SG50706, &UA3112460},
	{SG50707, &UA3112542},
	{SG50708, &UA3113225},
	{SG50710, &UA3126526},
	{SG50722, &UA403283},
	{SG50724, &UA863698},
	{SG50726, &UA903338},
	{SG50728, &UA1977264},
	{SG50729, &UA2449290},
	{SG50730, &UA2475456},
	{SG50732, &UA2593846},
	{SG50735, &UA2601777},
	{SG50737, &UA2605924},
	{SG50741, &UA2611161},
	{SG50743, &UA2611400},
	{SG50746, &UA2612960},
	{SG50747, &UA2615816},
	{SG50753, &UA2667611},
	{SG50754, &UA2668275},
	{SG50757, &UA2668722},
	{SG50760, &UA2708343},
	{SG50761, &UA2711689},
	{SG50763, &UA2712638},
	{SG50765, &UA2713944},
	{SG50768, &UA2714485},
	{SG50772, &UA2734653},
	{SG50774, &UA2734739},
	{SG50776, &UA2735907},
	{SG50778, &UA2748463},
	{SG50780, &UA2749951},
	{SG50782, &UA2750806},
	{SG50783, &UA2751015},
	{SG50785, &UA2767722},
	{SG50786, &UA2768054},
	{SG50788, &UA2781699},
	{SG50789, &UA2790352},
	{SG50791, &UA2850716},
	{SG50793, &UA2864405},
	{SG50794, &UA2867548},
	{SG50795, &UA2877299},
	{SG50797, &UA2877817},
	{SG50798, &UA2879177},
	{SG50804, &UA2992048},
	{SG50805, &UA2997484},
	{SG50806, &UA2998553},
	{SG50807, &UA2998713},
	{SG50809, &UA2999175},
	{SG50812, &UA3016839},
	{SG50813, &UA3017198},
	{SG50814, &UA3020627},
	{SG50816, &UA3021040},
	{SG50818, &UA3021233},
	{SG50819, &UA3021583},
	{SG50822, &UA3021832},
	{SG50823, &UA3022485},
	{SG50825, &UA3022486},
	{SG50827, &UA3022563},
	{SG50828, &UA3023373},
	{SG50830, &UA3047144},
	{SG50831, &UA3049467},
	{SG50834, &UA3079321},
	{SG50836, &UA3092690},
	{SG50839, &UA3109095},
	{SG50841, &UA3111849},
	{SG50843, &UA3112706},
	{SG50845, &UA3126206},
	{SG50846, &UA3127138},
	{SG50847, &UA3127166},
	{SG50848, &UA3132624},
	{SG50870, &UA2240912},
	{SG50873, &UA2445290},
	{SG50875, &UA2502649},
	{SG50879, &UA2553291},
	{SG50881, &UA2594908},
	{SG50883, &UA2601606},
	{SG50885, &UA2629226},
	{SG50886, &UA2646922},
	{SG50887, &UA2653981},
	{SG50890, &UA2698580},
	{SG50891, &UA2703557},
	{SG50892, &UA2703695},
	{SG50893, &UA2706949},
	{SG50895, &UA2707140},
	{SG50897, &UA2715319},
	{SG50899, &UA2716467},
	{SG50900, &UA2730558},
	{SG50902, &UA2733297},
	{SG50904, &UA2735015},
	{SG50905, &UA2735148},
	{SG50906, &UA2735662},
	{SG50909, &UA2750835},
	{SG50910, &UA2765402},
	{SG50911, &UA2766849},
	{SG50912, &UA2767007},
	{SG50913, &UA2767209},
	{SG50914, &UA2767920},
	{SG50915, &UA2779913},
	{SG50916, &UA2780969},
	{SG50917, &UA2781736},
	{SG50918, &UA2790733},
	{SG50919, &UA2792013},
	{SG50920, &UA2837092},
	{SG50921, &UA2838679},
	{SG50922, &UA2845203},
	{SG50923, &UA2846576},
	{SG50924, &UA2848983},
	{SG50926, &UA2861616},
	{SG50932, &UA2879383},
	{SG50934, &UA2879444},
	{SG50936, &UA2879521},
	{SG50937, &UA2892678},
	{SG50939, &UA2892835},
	{SG50940, &UA2892946},
	{SG50942, &UA2893639},
	{SG50943, &UA2895250},
	{SG50945, &UA2896738},
	{SG50947, &UA2897695},
	{SG50948, &UA2937632},
	{SG50950, &UA2955581},
	{SG50957, &UA2985280},
	{SG50958, &UA2985385},
	{SG50959, &UA2988859},
	{SG50962, &UA2997795},
	{SG50963, &UA2997880},
	{SG50964, &UA2997889},
	{SG50967, &UA3006028},
	{SG50968, &UA3010666},
	{SG50969, &UA3012044},
	{SG50972, &UA3021430},
	{SG50973, &UA3022033},
	{SG50974, &UA3022573},
	{SG50979, &UA3048046},
	{SG50981, &UA3050107},
	{SG50983, &UA3054889},
	{SG50984, &UA3056562},
	{SG50985, &UA3060345},
	{SG50998, &UA3109314},
	{SG51001, &UA3110067},
	{SG51003, &UA3111303},
	{SG51006, &UA3115174},
	{SG51008, &UA3126309},
	{SG51010, &UA3126359},
	{SG51012, &UA3126558},
	{SG51013, &UA3127392},
	{SG51020, &UA900297},
	{SG51021, &UA1980201},
	{SG51022, &UA1990545},
	{SG51025, &UA2029738},
	{SG51030, &UA2351990},
	{SG51033, &UA2352153},
	{SG51045, &UA2529431},
	{SG51048, &UA2555289},
	{SG51049, &UA2601196},
	{SG51050, &UA2613898},
	{SG51051, &UA2624183},
	{SG51052, &UA2633302},
	{SG51053, &UA2621902},
	{SG51054, &UA2655736},
	{SG51055, &UA2693424},
	{SG51056, &UA2694105},
	{SG51057, &UA2709155},
	{SG51059, &UA2732922},
	{SG51060, &UA2734643},
	{SG51061, &UA2734783},
	{SG51062, &UA2734852},
	{SG51063, &UA2735639},
	{SG51066, &UA2736102},
	{SG51067, &UA2748244},
	{SG51069, &UA2767378},
	{SG51070, &UA2769429},
	{SG51073, &UA2794701},
	{SG51074, &UA2832498},
	{SG51076, &UA2833015},
	{SG51077, &UA2838136},
	{SG51079, &UA2848350},
	{SG51081, &UA2850640},
	{SG51084, &UA2860622},
	{SG51086, &UA2879212},
	{SG51088, &UA2892884},
	{SG51090, &UA2934372},
	{SG51094, &UA2937727},
	{SG51097, &UA2982435},
	{SG51098, &UA2983023},
	{SG51099, &UA2988198},
	{SG51100, &UA2990052},
	{SG51104, &UA2995358},
	{SG51105, &UA2995771},
	{SG51106, &UA2996599},
	{SG51111, &UA2997406},
	{SG51113, &UA2998698},
	{SG51115, &UA2999403},
	{SG51117, &UA2999730},
	{SG51119, &UA3001252},
	{SG51121, &UA3004876},
	{SG51122, &UA3009737},
	{SG51123, &UA3010246},
	{SG51124, &UA3020563},
	{SG51125, &UA3020801},
	{SG51126, &UA3022467},
	{SG51127, &UA3044565},
	{SG51128, &UA3045293},
	{SG51129, &UA3049192},
	{SG51130, &UA3049546},
	{SG51131, &UA3050210},
	{SG51132, &UA3073875},
	{SG51133, &UA3074241},
	{SG51135, &UA3088667},
	{SG51136, &UA3093020},
	{SG51137, &UA3103151},
	{SG51138, &UA3107903},
	{SG51141, &UA3109388},
	{SG51142, &UA3109968},
	{SG51144, &UA3114551},
	{SG51145, &UA3115057},
	{SG51150, &UA3115270},
	{SG51153, &UA3126399},
	{SG51155, &UA3126428},
	{SG51157, &UA3126493},
	{SG51158, &UA3126522},
	{SG51159, &UA3127010},
	{SG51163, &UA3131928},
	{SG51164, &UA3132623},
	{SG51176, &UA2034402},
	{SG51178, &UA2352201},
	{SG51179, &UA3121392},
	{SG51182, &UA2366758},
	{SG51184, &UA2484715},
	{SG51185, &UA2485207},
	{SG51190, &UA2580360},
	{SG51197, &UA2637239},
	{SG51199, &UA2893465},
	{SG51201, &UA2645904},
	{SG51203, &UA2656201},
	{SG51204, &UA2663989},
	{SG51206, &UA2702369},
	{SG51207, &UA2703372},
	{SG51208, &UA2732842},
	{SG51210, &UA2733121},
	{SG51211, &UA2735294},
	{SG51212, &UA2735404},
	{SG51214, &UA2748282},
	{SG51215, &UA2751038},
	{SG51218, &UA2763081},
	{SG51220, &UA2767077},
	{SG51223, &UA2767774},
	{SG51224, &UA2768381},
	{SG51225, &UA2793862},
	{SG51227, &UA2794317},
	{SG51234, &UA2849076},
	{SG51235, &UA2850332},
	{SG51238, &UA2853020},
	{SG51240, &UA2853850},
	{SG51242, &UA2856974},
	{SG51244, &UA2863225},
	{SG51245, &UA2863714},
	{SG51246, &UA2864777},
	{SG51247, &UA2865606},
	{SG51248, &UA2867502},
	{SG51250, &UA2868972},
	{SG51251, &UA2878195},
	{SG51252, &UA2879498},
	{SG51253, &UA2892438},
	{SG51254, &UA2892559},
	{SG51255, &UA2892589},
	{SG51256, &UA2892729},
	{SG51258, &UA2893180},
	{SG51260, &UA2894219},
	{SG51261, &UA2937352},
	{SG51263, &UA2937850},
	{SG51264, &UA2937869},
	{SG51265, &UA2937923},
	{SG51266, &UA2937973},
	{SG51267, &UA2937979},
	{SG51268, &UA2940355},
	{SG51272, &UA2950344},
	{SG51273, &UA2953850},
	{SG51275, &UA2958247},
	{SG51276, &UA2960946},
	{SG51278, &UA2981619},
	{SG51280, &UA2983892},
	{SG51283, &UA2988823},
	{SG51284, &UA2988847},
	{SG51285, &UA2990201},
	{SG51286, &UA2995410},
	{SG51287, &UA2998862},
	{SG51289, &UA2999923},
	{SG51292, &UA3001028},
	{SG51293, &UA3001158},
	{SG51295, &UA3002083},
	{SG51296, &UA3005480},
	{SG51297, &UA3012414},
	{SG51298, &UA3015519},
	{SG51299, &UA3018084},
	{SG51300, &UA3021434},
	{SG51301, &UA3021743},
	{SG51302, &UA3021917},
	{SG51303, &UA3045590},
	{SG51304, &UA3045742},
	{SG51306, &UA3055357},
	{SG51308, &UA3078590},
	{SG51312, &UA3090189},
	{SG51314, &UA3092835},
	{SG51316, &UA3093216},
	{SG51317, &UA3094724},
	{SG51319, &UA3106327},
	{SG51320, &UA3106409},
	{SG51321, &UA3106565},
	{SG51325, &UA3107832},
	{SG51327, &UA3108353},
	{SG51328, &UA3108363},
	{SG51330, &UA3109245},
	{SG51332, &UA3109311},
	{SG51335, &UA3127169},
	{SG51337, &UA3127236},
	{SG51339, &UA3127256},
	{SG51343, &UA3130747},
	{SG51344, &UA3131320},
	{SG51369, &UA2378210},
	{SG51374, &UA2557024},
	{SG51381, &UA2635954},
	{SG51382, &UA2637601},
	{SG51386, &UA2647079},
	{SG51387, &UA2653228},
	{SG51389, &UA2655500},
	{SG51390, &UA2702580},
	{SG51392, &UA2735010},
	{SG51393, &UA2736088},
	{SG51394, &UA2736865},
	{SG51395, &UA2748803},
	{SG51396, &UA2751004},
	{SG51397, &UA2751150},
	{SG51399, &UA2764326},
	{SG51401, &UA2780848},
	{SG51403, &UA2793663},
	{SG51409, &UA2841436},
	{SG51415, &UA2847099},
	{SG51421, &UA2856896},
	{SG51425, &UA2860559},
	{SG51427, &UA2861904},
	{SG51429, &UA2862672},
	{SG51431, &UA2862893},
	{SG51434, &UA2863149},
	{SG51436, &UA2865751},
	{SG51438, &UA2866105},
	{SG51440, &UA2867988},
	{SG51441, &UA2869414},
	{SG51442, &UA2877243},
	{SG51446, &UA2892743},
	{SG51447, &UA2892953},
	{SG51448, &UA2937550},
	{SG51450, &UA2937715},
	{SG51451, &UA2937865},
	{SG51453, &UA2940151},
	{SG51454, &UA2952265},
	{SG51457, &UA2958214},
	{SG51458, &UA2958451},
	{SG51460, &UA2963383},
	{SG51463, &UA2983779},
	{SG51465, &UA2989349},
	{SG51466, &UA2989924},
	{SG51468, &UA2990276},
	{SG51471, &UA2993838},
	{SG51474, &UA2996900},
	{SG51475, &UA2997359},
	{SG51476, &UA2997485},
	{SG51479, &UA2998444},
	{SG51481, &UA2998694},
	{SG51485, &UA2998793},
	{SG51488, &UA2999497},
	{SG51493, &UA3003604},
	{SG51497, &UA3006356},
	{SG51500, &UA3014236},
	{SG51501, &UA3021367},
	{SG51502, &UA3022384},
	{SG51505, &UA3046565},
	{SG51507, &UA3049870},
	{SG51508, &UA3050408},
	{SG51510, &UA3056446},
	{SG51511, &UA3056748},
	{SG51514, &UA3056911},
	{SG51516, &UA3056961},
	{SG51518, &UA3057145},
	{SG51522, &UA3078413},
	{SG51525, &UA3088206},
	{SG51527, &UA3088474},
	{SG51528, &UA3088795},
	{SG51529, &UA3089219},
	{SG51530, &UA3092589},
	{SG51533, &UA3093385},
	{SG51537, &UA3107663},
	{SG51538, &UA3107800},
	{SG51539, &UA3108056},
	{SG51546, &UA3111022},
	{SG51547, &UA3112265},
	{SG51551, &UA3121710},
	{SG51553, &UA3126467},
	{SG51554, &UA3126511},
	{SG51556, &UA3127532},
	{SG51559, &UA3130952},
	{SG51560, &UA3131741},
	{SG51585, &UA2024440},
	{SG51589, &UA2383067},
	{SG51592, &UA2460294},
	{SG51594, &UA2472078},
	{SG51598, &UA2611557},
	{SG51599, &UA2621968},
	{SG51602, &UA2619657},
	{SG51603, &UA2692963},
	{SG51605, &UA2733382},
	{SG51606, &UA2735234},
	{SG51607, &UA2762806},
	{SG51608, &UA2795739},
	{SG51614, &UA2839973},
	{SG51619, &UA2846930},
	{SG51620, &UA2852509},
	{SG51622, &UA2860853},
	{SG51624, &UA2861665},
	{SG51625, &UA2861796},
	{SG51629, &UA2865327},
	{SG51630, &UA2868921},
	{SG51631, &UA2892416},
	{SG51632, &UA2892824},
	{SG51633, &UA2937527},
	{SG51635, &UA2938708},
	{SG51637, &UA2953429},
	{SG51638, &UA2957681},
	{SG51639, &UA2958583},
	{SG51642, &UA2984907},
	{SG51644, &UA2988901},
	{SG51645, &UA2989064},
	{SG51647, &UA2990077},
	{SG51649, &UA2992189},
	{SG51650, &UA2993833},
	{SG51652, &UA2996598},
	{SG51654, &UA2997350},
	{SG51655, &UA2997506},
	{SG51656, &UA2997514},
	{SG51662, &UA3013263},
	{SG51664, &UA3013826},
	{SG51666, &UA3022048},
	{SG51668, &UA3022391},
	{SG51669, &UA3022580},
	{SG51671, &UA3022598},
	{SG51672, &UA3028149},
	{SG51675, &UA3048278},
	{SG51678, &UA3050157},
	{SG51681, &UA3050380},
	{SG51682, &UA3056303},
	{SG51686, &UA3058071},
	{SG51688, &UA3058077},
	{SG51689, &UA3058096},
	{SG51690, &UA3058575},
	{SG51691, &UA3059110},
	{SG51692, &UA3070839},
	{SG51693, &UA3071800},
	{SG51696, &UA3073908},
	{SG51697, &UA3077241},
	{SG51699, &UA3078110},
	{SG51700, &UA3086573},
	{SG51701, &UA3088129},
	{SG51706, &UA3101622},
	{SG51713, &UA3103678},
	{SG51714, &UA3103749},
	{SG51716, &UA3107875},
	{SG51719, &UA3108249},
	{SG51720, &UA3109134},
	{SG51721, &UA3109290},
	{SG51723, &UA3115035},
	{SG51725, &UA3118292},
	{SG51727, &UA3119648},
	{SG51729, &UA3124158},
	{SG51732, &UA3125629},
	{SG51735, &UA3126132},
	{SG51742, &UA3126819},
	{SG51743, &UA3127162},
	{SG51746, &UA3127846},
	{SG51748, &UA3130696},
	{SG51762, &UA2028380},
	{SG51768, &UA2338020},
	{SG51771, &UA2483080},
	{SG51773, &UA2527280},
	{SG51775, &UA2626561},
	{SG51776, &UA2622473},
	{SG51782, &UA2702752},
	{SG51784, &UA2707103},
	{SG51785, &UA2734652},
	{SG51786, &UA2735224},
	{SG51787, &UA2736735},
	{SG51790, &UA2750858},
	{SG51793, &UA2791965},
	{SG51794, &UA2794091},
	{SG51801, &UA2834044},
	{SG51802, &UA2836737},
	{SG51808, &UA2846243},
	{SG51811, &UA2847724},
	{SG51813, &UA2850613},
	{SG51818, &UA2854654},
	{SG51820, &UA2855836},
	{SG51822, &UA2859970},
	{SG51825, &UA2860770},
	{SG51826, &UA2862264},
	{SG51827, &UA2862715},
	{SG51828, &UA2862790},
	{SG51829, &UA2863031},
	{SG51830, &UA2869377},
	{SG51831, &UA2869981},
	{SG51832, &UA2879531},
	{SG51834, &UA2894215},
	{SG51836, &UA2895652},
	{SG51839, &UA2937077},
	{SG51840, &UA2938073},
	{SG51841, &UA2938218},
	{SG51845, &UA2981602},
	{SG51847, &UA2986627},
	{SG51849, &UA2987658},
	{SG51850, &UA2988326},
	{SG51851, &UA2988331},
	{SG51854, &UA2989600},
	{SG51858, &UA2994832},
	{SG51859, &UA2996795},
	{SG51860, &UA2997753},
	{SG51862, &UA2999338},
	{SG51864, &UA2999668},
	{SG51867, &UA3015429},
	{SG51869, &UA3015906},
	{SG51872, &UA3022011},
	{SG51873, &UA3022641},
	{SG51879, &UA3048452},
	{SG51881, &UA3050226},
	{SG51882, &UA3056549},
	{SG51887, &UA3058073},
	{SG51890, &UA3073340},
	{SG51891, &UA3073572},
	{SG51892, &UA3073966},
	{SG51893, &UA3075376},
	{SG51895, &UA3076568},
	{SG51896, &UA3077415},
	{SG51897, &UA3078027},
	{SG51898, &UA3079392},
	{SG51899, &UA3079668},
	{SG51900, &UA3088195},
	{SG51901, &UA3088619},
	{SG51905, &UA3099484},
	{SG51906, &UA3100758},
	{SG51910, &UA3102660},
	{SG51915, &UA3106376},
	{SG51917, &UA3107007},
	{SG51918, &UA3108260},
	{SG51920, &UA3109182},
	{SG51922, &UA3110731},
	{SG51924, &UA3118515},
	{SG51927, &UA3118593},
	{SG51932, &UA3121035},
	{SG51935, &UA3123307},
	{SG51936, &UA3124292},
	{SG51938, &UA3125514},
	{SG51945, &UA3126218},
	{SG51946, &UA3126464},
	{SG51947, &UA3126562},
	{SG51949, &UA3126856},
	{SG51950, &UA3126869},
	{SG51952, &UA3126953},
	{SG51953, &UA3127352},
	{SG51954, &UA3127445},
	{SG51955, &UA3127471},
	{SG51956, &UA3127478},
	{SG51957, &UA3127546},
	{SG51958, &UA3127749},
	{SG51960, &UA3128319},
	{SG51962, &UA3130308},
	{SG51963, &UA3130697},
	{SG51985, &UA2028340},
	{SG51993, &UA2241402},
	{SG51995, &UA2241404},
	{SG51996, &UA2242094},
	{SG51998, &UA2242095},
	{SG52005, &UA2429759},
	{SG52007, &UA2549908},
	{SG52010, &UA2630257},
	{SG52011, &UA2628399},
	{SG52012, &UA2634010},
	{SG52014, &UA2694597},
	{SG52018, &UA2730728},
	{SG52020, &UA2749977},
	{SG52021, &UA2751931},
	{SG52023, &UA2760524},
	{SG52025, &UA2794273},
	{SG52026, &UA2810272},
	{SG52030, &UA2835027},
	{SG52034, &UA2845629},
	{SG52039, &UA2855290},
	{SG52041, &UA2858243},
	{SG52043, &UA2860774},
	{SG52045, &UA2861063},
	{SG52047, &UA2861327},
	{SG52050, &UA2862819},
	{SG52051, &UA2863215},
	{SG52054, &UA2863873},
	{SG52059, &UA2866849},
	{SG52060, &UA2867861},
	{SG52064, &UA2950284},
	{SG52066, &UA2950700},
	{SG52068, &UA2957158},
	{SG52070, &UA2957576},
	{SG52073, &UA2958578},
	{SG52079, &UA2989033},
	{SG52081, &UA2990701},
	{SG52083, &UA2992993},
	{SG52085, &UA2995564},
	{SG52086, &UA2996052},
	{SG52088, &UA2997473},
	{SG52092, &UA3004486},
	{SG52093, &UA3018232},
	{SG52094, &UA3022004},
	{SG52097, &UA3044262},
	{SG52099, &UA3044566},
	{SG52101, &UA3045054},
	{SG52102, &UA3048229},
	{SG52103, &UA3049781},
	{SG52104, &UA3050095},
	{SG52106, &UA3056714},
	{SG52109, &UA3071341},
	{SG52110, &UA3073156},
	{SG52111, &UA3079424},
	{SG52112, &UA3085492},
	{SG52115, &UA3086499},
	{SG52116, &UA3087802},
	{SG52117, &UA3088322},
	{SG52120, &UA3092546},
	{SG52121, &UA3092942},
	{SG52122, &UA3098888},
	{SG52123, &UA3099045},
	{SG52124, &UA3100002},
	{SG52126, &UA3100659},
	{SG52128, &UA3100729},
	{SG52130, &UA3100800},
	{SG52131, &UA3102092},
	{SG52134, &UA3103046},
	{SG52136, &UA3105900},
	{SG52137, &UA3106295},
	{SG52141, &UA3107812},
	{SG52142, &UA3107834},
	{SG52147, &UA3108220},
	{SG52148, &UA3118835},
	{SG52149, &UA3119238},
	{SG52150, &UA3120972},
	{SG52151, &UA3121507},
	{SG52155, &UA3121781},
	{SG52157, &UA3122099},
	{SG52159, &UA3125602},
	{SG52162, &UA3126096},
	{SG52165, &UA3126395},
	{SG52167, &UA3126575},
	{SG52169, &UA3126615},
	{SG52172, &UA3126618},
	{SG52175, &UA3126719},
	{SG52176, &UA3126759},
	{SG52178, &UA3127097},
	{SG52180, &UA3127292},
	{SG52182, &UA3127438},
	{SG52185, &UA3129906},
	{SG52186, &UA3132422},
	{SG52195, &UA2026213},
	{SG52203, &UA2449414},
	{SG52204, &UA2455091},
	{SG52207, &UA2607542},
	{SG52209, &UA2607551},
	{SG52211, &UA2622353},
	{SG52217, &UA2827950},
	{SG52221, &UA2837746},
	{SG52224, &UA2842907},
	{SG52227, &UA2847858},
	{SG52229, &UA2847975},
	{SG52232, &UA2850920},
	{SG52234, &UA2857087},
	{SG52235, &UA2861087},
	{SG52237, &UA2862255},
	{SG52239, &UA2863957},
	{SG52241, &UA2864723},
	{SG52257, &UA2030137},
	{SG52268, &UA2617072},
	{SG52273, &UA2620923},
	{SG52275, &UA2627403},
	{SG52282, &UA2817363},
	{SG52283, &UA2829652},
	{SG52288, &UA2847795},
	{SG52290, &UA2857722},
	{SG52291, &UA2858896},
	{SG52293, &UA2860032},
	{SG52294, &UA2861379},
	{SG52296, &UA2861738},
	{SG52297, &UA2861978},
	{SG52298, &UA2862520},
	{SG52299, &UA2863677},
	{SG52315, &UA2024762},
	{SG52319, &UA2609572},
	{SG52321, &UA2616959},
	{SG52324, &UA2623513},
	{SG52329, &UA2623194},
	{SG52330, &UA2625861},
	{SG52331, &UA2627962},
	{SG52333, &UA2722477},
	{SG52334, &UA2786847},
	{SG52336, &UA2819707},
	{SG52338, &UA2833514},
	{SG52356, &UA1955661},
	{SG52357, &UA2029151},
	{SG52358, &UA2339290},
	{SG52360, &UA2622070},
	{SG52362, &UA2622389},
	{SG52363, &UA2622514},
	{SG52365, &UA2623209},
	{SG52366, &UA2636300},
	{SG52368, &UA2617295},
	{SG52370, &UA2636309},
	{SG52371, &UA2632687},
	{SG52397, &UA2028403},
	{SG52402, &UA2626938},
	{SG52403, &UA2616172},
	{SG52404, &UA2630959},
	{SG52405, &UA2636606},
	{SG52406, &UA2623053},
	{SG52407, &UA2628913},
	{SG52408, &UA2685280},
	{SG52409, &UA2721924},
	{SG52410, &UA2811676},
	{SG52434, &UA785996},
	{SG52436, &UA791311},
	{SG52451, &UA2028173},
	{SG52453, &UA2546734},
	{SG52455, &UA2617252},
	{SG52456, &UA2624719},
	{SG52459, &UA2616215},
	{SG52461, &UA2625390},
	{SG52495, &UA2625118},
	{SG52496, &UA2624064},
	{SG52497, &UA2626563},
	{SG52498, &UA2630525},
	{SG52499, &UA2815228},
	{SG52511, &UA2610664},
	{SG52512, &UA2623162},
	{SG52514, &UA2624188},
	{SG52516, &UA2634713},
	{SG52544, &UA1960351},
	{SG52545, &UA2032839},
	{SG52549, &UA2619086},
	{SG52550, &UA2616451},
	{SG52552, &UA2617573},
	{SG52554, &UA2622931},
	{SG52556, &UA2624274},
	{SG52557, &UA2620222},
	{SG52559, &UA2620668},
	{SG52560, &UA2637888},
	{SG52564, &UA2631652},
	{SG52590, &UA2034339},
	{SG52591, &UA2545440},
	{SG52594, &UA2617717},
	{SG52595, &UA2623106},
	{SG52598, &UA2635333},
	{SG52607, &UA2635897},
	{SG52608, &UA2818169},
	{SG52643, &UA2612728},
	{SG52645, &UA2619257},
	{SG52647, &UA2620165},
	{SG52651, &UA2622218},
	{SG52653, &UA2634105},
	{SG52679, &UA608905},
	{SG52685, &UA2616360},
	{SG52686, &UA2621355},
	{SG52689, &UA2619650},
	{SG52690, &UA2622122},
	{SG52691, &UA2622818},
	{SG52692, &UA2633285},
	{SG52693, &UA2637780},
	{SG52694, &UA2635528},
	{SG52715, &UA724322},
	{SG52719, &UA2623814},
	{SG52720, &UA2616398},
	{SG52723, &UA2625854},
	{SG52724, &UA2624359},
	{SG52725, &UA2626512},
	{SG52726, &UA2624672},
	{SG52728, &UA2635205},
	{SG52729, &UA2635427},
	{SG52746, &UA1984073},
	{SG52748, &UA2027289},
	{SG52749, &UA2032396},
	{SG52755, &UA2586644},
	{SG52763, &UA2619401},
	{SG52766, &UA2625511},
	{SG52769, &UA2620618},
	{SG52775, &UA2625052},
	{SG52778, &UA2634592},
	{SG52817, &UA2026460},
	{SG52820, &UA1951161},
	{SG52833, &UA2619817},
	{SG52834, &UA2621113},
	{SG52838, &UA2628355},
	{SG52839, &UA2635062},
	{SG52863, &UA605709},
	{SG52864, &UA454810},
	{SG52866, &UA2622500},
	{SG52874, &UA1959783},
	{SG52879, &UA2026647},
	{SG52881, &UA2028909},
	{SG52882, &UA2621588},
	{SG52883, &UA2619109},
	{SG52884, &UA2627856},
	{SG52886, &UA2617971},
	{SG52889, &UA2636380},
	{SG52890, &UA2619127},
	{SG52891, &UA2622169},
	{SG52892, &UA2622643},
	{SG52893, &UA2622067},
	{SG52897, &UA2630373},
	{SG52898, &UA2628552},
	{SG52900, &UA2637842},
	{SG52927, &UA616586},
	{SG52928, &UA2027390},
	{SG52935, &UA2026886},
	{SG52936, &UA2033465},
	{SG52942, &UA2462422},
	{SG52943, &UA2462426},
	{SG52946, &UA2586643},
	{SG52952, &UA2617535},
	{SG52955, &UA2619087},
	{SG52956, &UA2617442},
	{SG52957, &UA2617562},
	{SG52959, &UA2619797},
	{SG52961, &UA2621026},
	{SG52962, &UA2623100},
	{SG52964, &UA2628468},
	{SG52965, &UA2631778},
	{SG52966, &UA2632978},
	{SG52967, &UA2633164},
	{SG52969, &UA2633156},
	{SG52971, &UA2636957},
	{SG52972, &UA2634235},
	{SG52973, &UA2636802},
	{SG52975, &UA2634931},
	{SG52977, &UA3084112},
	{SG53009, &UA616518},
	{SG53010, &UA616582},
	{SG53011, &UA616596},
	{SG53022, &UA2607528},
	{SG53025, &UA2614026},
	{SG53027, &UA2616488},
	{SG53028, &UA2624252},
	{SG53029, &UA2617410},
	{SG53031, &UA2620713},
	{SG53033, &UA2618673},
	{SG53037, &UA2619877},
	{SG53039, &UA2620721},
	{SG53041, &UA2621191},
	{SG53043, &UA2626115},
	{SG53046, &UA2622427},
	{SG53048, &UA2622666},
	{SG53050, &UA2623955},
	{SG53055, &UA2624297},
	{SG53056, &UA2625763},
	{SG53057, &UA2635730},
	{SG53060, &UA2627441},
	{SG53062, &UA2629390},
	{SG53064, &UA2629412},
	{SG53065, &UA2631230},
	{SG53067, &UA2630723},
	{SG53068, &UA2631201},
	{SG53071, &UA2633861},
	{SG53073, &UA2634385},
	{SG53074, &UA2637405},
	{SG53079, &UA3084056},
	{SG53080, &UA3084099},
	{SG53107, &UA1989146},
	{SG53112, &UA2608203},
	{SG53114, &UA2611005},
	{SG53115, &UA2613956},
	{SG53116, &UA2615265},
	{SG53117, &UA2615523},
	{SG53118, &UA2616401},
	{SG53119, &UA2616712},
	{SG53120, &UA2617124},
	{SG53121, &UA2617392},
	{SG53122, &UA2628152},
	{SG53123, &UA2618197},
	{SG53124, &UA2618264},
	{SG53125, &UA2619491},
	{SG53127, &UA2620730},
	{SG53128, &UA2624286},
	{SG53129, &UA2621428},
	{SG53131, &UA2621432},
	{SG53133, &UA2622807},
	{SG53134, &UA2622855},
	{SG53135, &UA2622856},
	{SG53137, &UA2623333},
	{SG53138, &UA2623642},
	{SG53140, &UA2624922},
	{SG53141, &UA2625284},
	{SG53143, &UA2628157},
	{SG53145, &UA2628462},
	{SG53148, &UA2629678},
	{SG53150, &UA2629756},
	{SG53153, &UA2630469},
	{SG53160, &UA2634285},
	{SG53162, &UA2637366},
	{SG53163, &UA2636373},
	{SG53165, &UA2636919},
	{SG53166, &UA2637020},
	{SG53167, &UA2637730},
	{SG53168, &UA3084047},
	{SG53169, &UA3084067},
	{SG53170, &UA3084117},
	{SG53216, &UA616561},
	{SG53219, &UA1962419},
	{SG53222, &UA2025321},
	{SG53226, &UA2031672},
	{SG53227, &UA2032383},
	{SG53233, &UA2586649},
	{SG53234, &UA2607534},
	{SG53235, &UA2607553},
	{SG53238, &UA2611936},
	{SG53241, &UA2615039},
	{SG53243, &UA2615613},
	{SG53244, &UA2615717},
	{SG53246, &UA2615748},
	{SG53247, &UA2616201},
	{SG53249, &UA2616314},
	{SG53252, &UA2616475},
	{SG53254, &UA2616645},
	{SG53255, &UA2616750},
	{SG53257, &UA2616863},
	{SG53259, &UA2617387},
	{SG53260, &UA2618275},
	{SG53262, &UA2618546},
	{SG53264, &UA2618810},
	{SG53266, &UA2619010},
	{SG53268, &UA2619393},
	{SG53269, &UA2619477},
	{SG53272, &UA2619744},
	{SG53274, &UA2619946},
	{SG53277, &UA2620045},
	{SG53278, &UA2620176},
	{SG53280, &UA2621420},
	{SG53281, &UA2621917},
	{SG53283, &UA2622558},
	{SG53284, &UA2622695},
	{SG53287, &UA2623156},
	{SG53290, &UA2623342},
	{SG53291, &UA2624241},
	{SG53294, &UA2624837},
	{SG53295, &UA2625110},
	{SG53296, &UA2625441},
	{SG53297, &UA2625878},
	{SG53298, &UA2626754},
	{SG53299, &UA2626934},
	{SG53301, &UA2627029},
	{SG53303, &UA2627098},
	{SG53306, &UA2627624},
	{SG53307, &UA2628903},
	{SG53308, &UA2628950},
	{SG53309, &UA2628954},
	{SG53310, &UA2628990},
	{SG53311, &UA2629047},
	{SG53312, &UA2629216},
	{SG53313, &UA2629227},
	{SG53314, &UA2629749},
	{SG53315, &UA2629821},
	{SG53316, &UA2629843},
	{SG53317, &UA2630708},
	{SG53318, &UA2631672},
	{SG53319, &UA2632131},
	{SG53320, &UA2632317},
	{SG53321, &UA2632387},
	{SG53324, &UA2632811},
	{SG53327, &UA2633205},
	{SG53328, &UA2633718},
	{SG53330, &UA2633936},
	{SG53331, &UA2633966},
	{SG53333, &UA2634043},
	{SG53334, &UA2634321},
	{SG53335, &UA2634334},
	{SG53338, &UA2634505},
	{SG53342, &UA2635239},
	{SG53345, &UA2635303},
	{SG53349, &UA2636093},
	{SG53350, &UA2636570},
	{SG53352, &UA2636719},
	{SG53354, &UA2637003},
	{SG53357, &UA2637179},
	{SG53358, &UA2637320},
	{SG53359, &UA2637451},
	{SG53360, &UA2637592},
	{SG53364, &UA2637879},
	{SG53366, &UA2676267},
	{SG53368, &UA2748168},
	{SG53369, &UA2778995},
	{SG53372, &UA2848410},
	{SG53384, &UA3084100},
	{SG53385, &UA3084073},
	{SG53386, &UA3084102},
	{NULL}};
const Segmentdata DAndroidVersion4_1_1[] = {
	{SG26050, &UA1995260},
	{SG26051, &UA1996084},
	{SG26052, &UA1996253},
	{SG26053, &UA1996481},
	{SG26054, &UA1996993},
	{SG26055, &UA2030840},
	{SG26056, &UA2031761},
	{SG26057, &UA2033256},
	{SG26058, &UA2042571},
	{SG26059, &UA2204699},
	{SG26060, &UA2204705},
	{SG26061, &UA2204884},
	{SG26062, &UA2232290},
	{SG26064, &UA2232999},
	{SG26065, &UA2257965},
	{SG26066, &UA2257983},
	{SG26067, &UA2338576},
	{SG26068, &UA2348617},
	{SG26069, &UA2355404},
	{SG26070, &UA2360797},
	{SG26071, &UA2365726},
	{SG26072, &UA2381899},
	{SG26073, &UA2385434},
	{SG26074, &UA2423516},
	{SG26075, &UA2424128},
	{SG26076, &UA2425250},
	{SG26077, &UA2436122},
	{SG26078, &UA2455654},
	{SG26079, &UA2457196},
	{SG26080, &UA2458617},
	{SG26081, &UA2466647},
	{SG26083, &UA2480211},
	{SG26085, &UA2483108},
	{SG26086, &UA2485312},
	{SG26088, &UA2485458},
	{SG26089, &UA2487385},
	{SG26092, &UA2523829},
	{SG26094, &UA2527722},
	{SG26095, &UA2528561},
	{SG26096, &UA2549116},
	{SG26097, &UA2597961},
	{SG26098, &UA2600660},
	{SG26099, &UA2696810},
	{SG26100, &UA2699499},
	{SG26101, &UA2701313},
	{SG26102, &UA2733867},
	{SG26103, &UA2747185},
	{SG26104, &UA2748216},
	{SG26105, &UA2763891},
	{SG26106, &UA2778725},
	{SG26107, &UA2780988},
	{SG26108, &UA2781757},
	{SG26109, &UA2839057},
	{SG26110, &UA2936613},
	{SG26112, &UA2988299},
	{SG26333, &UA925687},
	{SG26334, &UA925688},
	{SG26335, &UA2202472},
	{SG26337, &UA2203429},
	{SG26338, &UA2205398},
	{SG26340, &UA2206013},
	{SG26341, &UA2206160},
	{SG26342, &UA2229547},
	{SG26343, &UA2229783},
	{SG26344, &UA2230514},
	{SG26345, &UA2230691},
	{SG26346, &UA2230935},
	{SG26347, &UA2233268},
	{SG26348, &UA2237133},
	{SG26349, &UA2257645},
	{SG26350, &UA2257984},
	{SG26351, &UA2354065},
	{SG26352, &UA2355428},
	{SG26353, &UA2355924},
	{SG26354, &UA2356216},
	{SG26355, &UA2356768},
	{SG26356, &UA2360748},
	{SG26358, &UA2367666},
	{SG26359, &UA2383714},
	{SG26360, &UA2384381},
	{SG26361, &UA2399522},
	{SG26362, &UA2403618},
	{SG26363, &UA2403665},
	{SG26364, &UA2424494},
	{SG26365, &UA2426223},
	{SG26367, &UA2451726},
	{SG26368, &UA2453304},
	{SG26369, &UA2456390},
	{SG26370, &UA2459780},
	{SG26371, &UA2467206},
	{SG26373, &UA2483582},
	{SG26374, &UA2483899},
	{SG26375, &UA2506601},
	{SG26376, &UA2527938},
	{SG26377, &UA2554655},
	{SG26378, &UA2555003},
	{SG26379, &UA2555221},
	{SG26380, &UA2558874},
	{SG26381, &UA2592464},
	{SG26382, &UA2594183},
	{SG26383, &UA2653581},
	{SG26384, &UA2730218},
	{SG26385, &UA2768752},
	{SG26386, &UA2889253},
	{SG26387, &UA2936676},
	{SG26389, &UA3107471},
	{SG26390, &UA3117904},
	{SG26416, &UA1996265},
	{SG26417, &UA2006116},
	{SG26418, &UA2028466},
	{SG26419, &UA2028793},
	{SG26420, &UA2028819},
	{SG26421, &UA2033523},
	{SG26422, &UA2183224},
	{SG26423, &UA2189033},
	{SG26424, &UA2190422},
	{SG26426, &UA2191124},
	{SG26427, &UA2194831},
	{SG26428, &UA2203523},
	{SG26430, &UA2204067},
	{SG26431, &UA2204706},
	{SG26432, &UA2204868},
	{SG26433, &UA2228901},
	{SG26434, &UA2235994},
	{SG26436, &UA2237120},
	{SG26437, &UA2257712},
	{SG26438, &UA2257959},
	{SG26439, &UA2257960},
	{SG26440, &UA2257963},
	{SG26441, &UA2353924},
	{SG26442, &UA2356113},
	{SG26443, &UA2360755},
	{SG26444, &UA2399514},
	{SG26446, &UA2400685},
	{SG26447, &UA2405031},
	{SG26448, &UA2405630},
	{SG26449, &UA2419234},
	{SG26450, &UA2422836},
	{SG26451, &UA2426601},
	{SG26452, &UA2430025},
	{SG26453, &UA2433810},
	{SG26454, &UA2454948},
	{SG26456, &UA2467689},
	{SG26457, &UA2555465},
	{SG26458, &UA2594331},
	{SG26460, &UA2594783},
	{SG26493, &UA2596715},
	{SG26498, &UA2696428},
	{SG26501, &UA2745982},
	{SG26506, &UA2777524},
	{SG26508, &UA2777734},
	{SG26510, &UA2789902},
	{SG26521, &UA3070979},
	{SG27114, &UA925215},
	{SG27115, &UA1996825},
	{SG27116, &UA2009558},
	{SG27117, &UA2029671},
	{SG27118, &UA2198812},
	{SG27119, &UA2226650},
	{SG27120, &UA2383073},
	{SG27121, &UA2401679},
	{SG27122, &UA2449694},
	{SG27123, &UA2456276},
	{SG27124, &UA2651864},
	{SG27125, &UA2830244},
	{SG27126, &UA2851251},
	{SG27127, &UA2860510},
	{SG27128, &UA3070120},
	{SG27395, &UA924293},
	{SG27396, &UA2029098},
	{SG27397, &UA2029585},
	{SG27398, &UA2199569},
	{SG27399, &UA2201660},
	{SG27400, &UA2342347},
	{SG27401, &UA2352428},
	{SG27402, &UA2417701},
	{SG27403, &UA2419846},
	{SG27404, &UA2420744},
	{SG27405, &UA2421777},
	{SG27406, &UA2422058},
	{SG27407, &UA2422263},
	{SG27408, &UA2422267},
	{SG27409, &UA2422414},
	{SG27410, &UA2422428},
	{SG27411, &UA2422938},
	{SG27412, &UA2423959},
	{SG27413, &UA2425349},
	{SG27414, &UA2426413},
	{SG27415, &UA2426460},
	{SG27416, &UA2427424},
	{SG27417, &UA2427764},
	{SG27418, &UA2428017},
	{SG27419, &UA2430621},
	{SG27420, &UA2437281},
	{SG27421, &UA2455115},
	{SG27422, &UA2455467},
	{SG27423, &UA2455561},
	{SG27424, &UA2457938},
	{SG27425, &UA2458135},
	{SG27426, &UA2460272},
	{SG27427, &UA2461012},
	{SG27428, &UA2461166},
	{SG27429, &UA2461303},
	{SG27430, &UA2466106},
	{SG27431, &UA2466730},
	{SG27432, &UA2467277},
	{SG27433, &UA2467945},
	{SG27434, &UA2468427},
	{SG27435, &UA2468769},
	{SG27437, &UA2480798},
	{SG27439, &UA2481692},
	{SG27441, &UA2481750},
	{SG27443, &UA2482875},
	{SG27445, &UA2483091},
	{SG27446, &UA2483332},
	{SG27447, &UA2483854},
	{SG27449, &UA2485117},
	{SG27451, &UA2486139},
	{SG27453, &UA2487013},
	{SG27455, &UA2504148},
	{SG27458, &UA2505298},
	{SG27460, &UA2505371},
	{SG27461, &UA2505402},
	{SG27463, &UA2505891},
	{SG27465, &UA2506398},
	{SG27466, &UA2506986},
	{SG27467, &UA2507054},
	{SG27468, &UA2507273},
	{SG27469, &UA2507673},
	{SG27470, &UA2507968},
	{SG27471, &UA2510711},
	{SG27472, &UA2523854},
	{SG27473, &UA2525672},
	{SG27474, &UA2527247},
	{SG27475, &UA2527470},
	{SG27476, &UA2527933},
	{SG27477, &UA2527982},
	{SG27478, &UA2528314},
	{SG27479, &UA2528365},
	{SG27480, &UA2528809},
	{SG27481, &UA2528908},
	{SG27482, &UA2528938},
	{SG27483, &UA2529315},
	{SG27484, &UA2529553},
	{SG27485, &UA2529891},
	{SG27486, &UA2530127},
	{SG27487, &UA2551789},
	{SG27488, &UA2552907},
	{SG27489, &UA2554154},
	{SG27491, &UA2554325},
	{SG27492, &UA2554557},
	{SG27493, &UA2555188},
	{SG27494, &UA2555533},
	{SG27495, &UA2555562},
	{SG27496, &UA2555580},
	{SG27498, &UA2555589},
	{SG27499, &UA2555856},
	{SG27501, &UA2555857},
	{SG27502, &UA2556082},
	{SG27503, &UA2556154},
	{SG27504, &UA2556238},
	{SG27505, &UA2556486},
	{SG27506, &UA2556498},
	{SG27507, &UA2556727},
	{SG27508, &UA2557237},
	{SG27509, &UA2557254},
	{SG27510, &UA2557598},
	{SG27511, &UA2557611},
	{SG27512, &UA2557925},
	{SG27513, &UA2558034},
	{SG27514, &UA2558351},
	{SG27515, &UA2559739},
	{SG27517, &UA2560826},
	{SG27518, &UA2562371},
	{SG27519, &UA2565241},
	{SG27520, &UA2577765},
	{SG27521, &UA2578027},
	{SG27522, &UA2578329},
	{SG27523, &UA2578670},
	{SG27524, &UA2578717},
	{SG27525, &UA2578724},
	{SG27526, &UA2578895},
	{SG27527, &UA2579003},
	{SG27528, &UA2579825},
	{SG27529, &UA2580374},
	{SG27530, &UA2580493},
	{SG27531, &UA2581842},
	{SG27532, &UA2582535},
	{SG27533, &UA2582761},
	{SG27534, &UA2582766},
	{SG27536, &UA2593605},
	{SG27537, &UA2595275},
	{SG27539, &UA2595325},
	{SG27540, &UA2595581},
	{SG27541, &UA2595881},
	{SG27543, &UA2596334},
	{SG27544, &UA2597165},
	{SG27545, &UA2597335},
	{SG27546, &UA2597526},
	{SG27547, &UA2597541},
	{SG27548, &UA2597640},
	{SG27549, &UA2598502},
	{SG27551, &UA2598992},
	{SG27552, &UA2599642},
	{SG27553, &UA2599754},
	{SG27554, &UA2600167},
	{SG27555, &UA2600400},
	{SG27556, &UA2600419},
	{SG27557, &UA2600493},
	{SG27558, &UA2603524},
	{SG27559, &UA2603632},
	{SG27560, &UA2604384},
	{SG27561, &UA2604556},
	{SG27562, &UA2604945},
	{SG27563, &UA2651295},
	{SG27564, &UA2653421},
	{SG27565, &UA2653904},
	{SG27566, &UA2654597},
	{SG27568, &UA2654603},
	{SG27569, &UA2655397},
	{SG27570, &UA2655934},
	{SG27571, &UA2655937},
	{SG27572, &UA2656344},
	{SG27573, &UA2656713},
	{SG27574, &UA2657627},
	{SG27575, &UA2657754},
	{SG27576, &UA2657825},
	{SG27577, &UA2658121},
	{SG27578, &UA2658144},
	{SG27579, &UA2658380},
	{SG27580, &UA2659109},
	{SG27581, &UA2659239},
	{SG27582, &UA2659539},
	{SG27583, &UA2659584},
	{SG27584, &UA2659637},
	{SG27585, &UA2660102},
	{SG27586, &UA2660159},
	{SG27587, &UA2660727},
	{SG27588, &UA2661491},
	{SG27589, &UA2661625},
	{SG27590, &UA2661792},
	{SG27592, &UA2662311},
	{SG27593, &UA2662450},
	{SG27594, &UA2662839},
	{SG27595, &UA2663137},
	{SG27596, &UA2663319},
	{SG27597, &UA2663756},
	{SG27598, &UA2665247},
	{SG27599, &UA2665368},
	{SG27600, &UA2665555},
	{SG27601, &UA2666070},
	{SG27602, &UA2666140},
	{SG27603, &UA2666215},
	{SG27604, &UA2666245},
	{SG27605, &UA2666446},
	{SG27606, &UA2666734},
	{SG27607, &UA2668325},
	{SG27608, &UA2668473},
	{SG27609, &UA2668479},
	{SG27610, &UA2668563},
	{SG27611, &UA2668752},
	{SG27612, &UA2670945},
	{SG27613, &UA2693043},
	{SG27614, &UA2693165},
	{SG27615, &UA2695749},
	{SG27616, &UA2696064},
	{SG27617, &UA2696498},
	{SG27618, &UA2696882},
	{SG27619, &UA2697139},
	{SG27620, &UA2697292},
	{SG27621, &UA2697468},
	{SG27622, &UA2697682},
	{SG27623, &UA2698476},
	{SG27624, &UA2698785},
	{SG27625, &UA2698824},
	{SG27626, &UA2698972},
	{SG27627, &UA2699448},
	{SG27628, &UA2699517},
	{SG27629, &UA2699693},
	{SG27630, &UA2699710},
	{SG27631, &UA2699719},
	{SG27632, &UA2699733},
	{SG27633, &UA2700255},
	{SG27634, &UA2700522},
	{SG27635, &UA2700599},
	{SG27636, &UA2701150},
	{SG27637, &UA2701265},
	{SG27638, &UA2701404},
	{SG27639, &UA2701684},
	{SG27640, &UA2702056},
	{SG27641, &UA2702445},
	{SG27642, &UA2702715},
	{SG27643, &UA2702858},
	{SG27644, &UA2703278},
	{SG27645, &UA2703640},
	{SG27646, &UA2703749},
	{SG27647, &UA2703995},
	{SG27648, &UA2704572},
	{SG27649, &UA2705141},
	{SG27650, &UA2705169},
	{SG27651, &UA2705434},
	{SG27652, &UA2706401},
	{SG27653, &UA2706473},
	{SG27654, &UA2706642},
	{SG27655, &UA2706843},
	{SG27656, &UA2706892},
	{SG27657, &UA2707072},
	{SG27658, &UA2707245},
	{SG27659, &UA2707750},
	{SG27660, &UA2707782},
	{SG27661, &UA2708057},
	{SG27662, &UA2712937},
	{SG27663, &UA2712953},
	{SG27664, &UA2713712},
	{SG27665, &UA2715279},
	{SG27666, &UA2717574},
	{SG27667, &UA2718466},
	{SG27668, &UA2726805},
	{SG27669, &UA2727299},
	{SG27670, &UA2727493},
	{SG27671, &UA2728056},
	{SG27672, &UA2729128},
	{SG27673, &UA2729160},
	{SG27674, &UA2729244},
	{SG27675, &UA2729563},
	{SG27676, &UA2729600},
	{SG27677, &UA2729687},
	{SG27678, &UA2730592},
	{SG27679, &UA2730874},
	{SG27680, &UA2731261},
	{SG27681, &UA2732231},
	{SG27682, &UA2732401},
	{SG27683, &UA2732636},
	{SG27684, &UA2732648},
	{SG27685, &UA2732720},
	{SG27686, &UA2732766},
	{SG27687, &UA2732802},
	{SG27688, &UA2732910},
	{SG27689, &UA2733601},
	{SG27690, &UA2733890},
	{SG27691, &UA2733947},
	{SG27692, &UA2733992},
	{SG27693, &UA2734510},
	{SG27694, &UA2737698},
	{SG27695, &UA2737973},
	{SG27696, &UA2738286},
	{SG27697, &UA2738299},
	{SG27698, &UA2738351},
	{SG27699, &UA2746928},
	{SG27700, &UA2747132},
	{SG27701, &UA2747151},
	{SG27702, &UA2747836},
	{SG27703, &UA2748100},
	{SG27704, &UA2748114},
	{SG27705, &UA2748255},
	{SG27706, &UA2748276},
	{SG27707, &UA2748302},
	{SG27708, &UA2748628},
	{SG27709, &UA2749218},
	{SG27710, &UA2749225},
	{SG27711, &UA2749429},
	{SG27712, &UA2749580},
	{SG27713, &UA2749554},
	{SG27714, &UA2750009},
	{SG27715, &UA2750029},
	{SG27716, &UA2750043},
	{SG27717, &UA2750304},
	{SG27718, &UA2750392},
	{SG27719, &UA2750904},
	{SG27720, &UA2751110},
	{SG27721, &UA2751224},
	{SG27722, &UA2751226},
	{SG27723, &UA2751325},
	{SG27724, &UA2751564},
	{SG27725, &UA2751908},
	{SG27726, &UA2752367},
	{SG27727, &UA2752498},
	{SG27728, &UA2752579},
	{SG27729, &UA2753188},
	{SG27730, &UA2762570},
	{SG27731, &UA2762574},
	{SG27732, &UA2762921},
	{SG27733, &UA2763154},
	{SG27735, &UA2763212},
	{SG27736, &UA2763256},
	{SG27737, &UA2763609},
	{SG27738, &UA2763623},
	{SG27739, &UA2763789},
	{SG27740, &UA2764246},
	{SG27741, &UA2764352},
	{SG27742, &UA2764704},
	{SG27743, &UA2764817},
	{SG27744, &UA2764933},
	{SG27745, &UA2765045},
	{SG27746, &UA2765051},
	{SG27747, &UA2765167},
	{SG27749, &UA2765654},
	{SG27750, &UA2766375},
	{SG27751, &UA2766488},
	{SG27752, &UA2766779},
	{SG27753, &UA2767300},
	{SG27754, &UA2767306},
	{SG27763, &UA2767626},
	{SG27764, &UA2767813},
	{SG27765, &UA2768361},
	{SG27766, &UA2768493},
	{SG27767, &UA2768514},
	{SG27768, &UA2768727},
	{SG27769, &UA2768995},
	{SG27770, &UA2770261},
	{SG27771, &UA2772142},
	{SG27772, &UA2777604},
	{SG27773, &UA2777732},
	{SG27774, &UA2778590},
	{SG27775, &UA2779386},
	{SG27776, &UA2779906},
	{SG27777, &UA2780039},
	{SG27779, &UA2780062},
	{SG27780, &UA2780197},
	{SG27781, &UA2780256},
	{SG27782, &UA2780581},
	{SG27783, &UA2780906},
	{SG27784, &UA2781287},
	{SG27785, &UA2781662},
	{SG27786, &UA2781785},
	{SG27787, &UA2781878},
	{SG27788, &UA2782259},
	{SG27789, &UA2790415},
	{SG27790, &UA2790987},
	{SG27791, &UA2791008},
	{SG27792, &UA2791087},
	{SG27793, &UA2791333},
	{SG27795, &UA2791447},
	{SG27796, &UA2792214},
	{SG27797, &UA2792255},
	{SG27798, &UA2793749},
	{SG27799, &UA2793805},
	{SG27800, &UA2794145},
	{SG27801, &UA2794157},
	{SG27802, &UA2794283},
	{SG27803, &UA2794490},
	{SG27804, &UA2794543},
	{SG27806, &UA2794873},
	{SG27807, &UA2795125},
	{SG27808, &UA2795284},
	{SG27809, &UA2795320},
	{SG27810, &UA2795404},
	{SG27811, &UA2799199},
	{SG27813, &UA2799355},
	{SG27814, &UA2835187},
	{SG27816, &UA2836843},
	{SG27817, &UA2837417},
	{SG27818, &UA2840426},
	{SG27819, &UA2843726},
	{SG27821, &UA2843854},
	{SG27822, &UA2844148},
	{SG27823, &UA2844197},
	{SG27824, &UA2844374},
	{SG27825, &UA2844764},
	{SG27826, &UA2844983},
	{SG27827, &UA2847202},
	{SG27828, &UA2847593},
	{SG27829, &UA2847595},
	{SG27830, &UA2847680},
	{SG27831, &UA2847852},
	{SG27832, &UA2848461},
	{SG27833, &UA2848903},
	{SG27834, &UA2848986},
	{SG27835, &UA2849235},
	{SG27836, &UA2849456},
	{SG27837, &UA2849887},
	{SG27838, &UA2849958},
	{SG27839, &UA2850020},
	{SG27840, &UA2850104},
	{SG27841, &UA2850872},
	{SG27842, &UA2851132},
	{SG27843, &UA2851178},
	{SG27844, &UA2852038},
	{SG27845, &UA2852088},
	{SG27846, &UA2852225},
	{SG27847, &UA2852827},
	{SG27848, &UA2855693},
	{SG27849, &UA2855743},
	{SG27850, &UA2855944},
	{SG27851, &UA2856359},
	{SG27852, &UA2857344},
	{SG27853, &UA2858151},
	{SG27854, &UA2858324},
	{SG27855, &UA2858799},
	{SG27856, &UA2858930},
	{SG27857, &UA2858943},
	{SG27858, &UA2859333},
	{SG27859, &UA2859403},
	{SG27860, &UA2859453},
	{SG27861, &UA2859522},
	{SG27862, &UA2859635},
	{SG27863, &UA2861800},
	{SG27864, &UA2861961},
	{SG27865, &UA2862632},
	{SG27866, &UA2862824},
	{SG27867, &UA2863596},
	{SG27868, &UA2864009},
	{SG27869, &UA2864248},
	{SG27870, &UA2864434},
	{SG27871, &UA2864643},
	{SG27872, &UA2864700},
	{SG27873, &UA2865334},
	{SG27874, &UA2865372},
	{SG27875, &UA2865519},
	{SG27876, &UA2866002},
	{SG27877, &UA2866146},
	{SG27878, &UA2866301},
	{SG27879, &UA2866364},
	{SG27880, &UA2866474},
	{SG27881, &UA2866545},
	{SG27882, &UA2866808},
	{SG27883, &UA2867011},
	{SG27884, &UA2867012},
	{SG27885, &UA2867474},
	{SG27886, &UA2868019},
	{SG27887, &UA2868147},
	{SG27888, &UA2868273},
	{SG27889, &UA2869007},
	{SG27890, &UA2869290},
	{SG27891, &UA2878334},
	{SG27892, &UA2878530},
	{SG27893, &UA2878634},
	{SG27894, &UA2878655},
	{SG27895, &UA2878653},
	{SG27896, &UA2878906},
	{SG27897, &UA2878918},
	{SG27898, &UA2878995},
	{SG27899, &UA2879279},
	{SG27900, &UA2879317},
	{SG27901, &UA2889560},
	{SG27902, &UA2890199},
	{SG27903, &UA2890297},
	{SG27904, &UA2890372},
	{SG27905, &UA2890395},
	{SG27906, &UA2890839},
	{SG27907, &UA2890919},
	{SG27908, &UA2891107},
	{SG27909, &UA2891341},
	{SG27910, &UA2891396},
	{SG27911, &UA2891683},
	{SG27912, &UA2891752},
	{SG27913, &UA2891805},
	{SG27914, &UA2891843},
	{SG27915, &UA2892033},
	{SG27916, &UA2892040},
	{SG27917, &UA2892093},
	{SG27918, &UA2892161},
	{SG27919, &UA2892265},
	{SG27920, &UA2892331},
	{SG27921, &UA2892551},
	{SG27922, &UA2892656},
	{SG27923, &UA2892737},
	{SG27924, &UA2892942},
	{SG27925, &UA2893052},
	{SG27926, &UA2893110},
	{SG27927, &UA2893467},
	{SG27928, &UA2893882},
	{SG27929, &UA2897611},
	{SG27930, &UA2934373},
	{SG27931, &UA2934521},
	{SG27932, &UA2934625},
	{SG27933, &UA2934626},
	{SG27934, &UA2934613},
	{SG27935, &UA2934787},
	{SG27936, &UA2934991},
	{SG27938, &UA2935139},
	{SG27939, &UA2935221},
	{SG27940, &UA2935332},
	{SG27941, &UA2935526},
	{SG27942, &UA2935794},
	{SG27944, &UA2935796},
	{SG27945, &UA2936121},
	{SG27946, &UA2936177},
	{SG27947, &UA2936331},
	{SG27948, &UA2936416},
	{SG27949, &UA2936482},
	{SG27950, &UA2936678},
	{SG27951, &UA2936729},
	{SG27952, &UA2936744},
	{SG27953, &UA2937152},
	{SG27954, &UA2937245},
	{SG27955, &UA2937306},
	{SG27956, &UA2937433},
	{SG27957, &UA2937466},
	{SG27958, &UA2937645},
	{SG27959, &UA2937792},
	{SG27960, &UA2937866},
	{SG27961, &UA2938142},
	{SG27963, &UA2938160},
	{SG27964, &UA2938163},
	{SG27965, &UA2939136},
	{SG27966, &UA2939720},
	{SG27967, &UA2940488},
	{SG27968, &UA2940604},
	{SG27969, &UA2941532},
	{SG27971, &UA2942543},
	{SG27973, &UA2950983},
	{SG27974, &UA2951103},
	{SG27975, &UA2951401},
	{SG27976, &UA2951493},
	{SG27977, &UA2951594},
	{SG27978, &UA2951822},
	{SG27979, &UA2951815},
	{SG27980, &UA2951893},
	{SG27981, &UA2951965},
	{SG27982, &UA2951976},
	{SG27983, &UA2952101},
	{SG27984, &UA2952162},
	{SG27985, &UA2952324},
	{SG27986, &UA2952347},
	{SG27987, &UA2952473},
	{SG27988, &UA2952590},
	{SG27989, &UA2952722},
	{SG27990, &UA2952745},
	{SG27991, &UA2952807},
	{SG27992, &UA2952889},
	{SG27994, &UA2952941},
	{SG27995, &UA2953115},
	{SG27996, &UA2953121},
	{SG27998, &UA2953298},
	{SG27999, &UA2953412},
	{SG28000, &UA2953446},
	{SG28001, &UA2953519},
	{SG28002, &UA2953622},
	{SG28003, &UA2953689},
	{SG28004, &UA2953884},
	{SG28005, &UA2954049},
	{SG28006, &UA2954414},
	{SG28008, &UA2954806},
	{SG28009, &UA2954904},
	{SG28010, &UA2955056},
	{SG28011, &UA2955205},
	{SG28012, &UA2955575},
	{SG28014, &UA2955596},
	{SG28015, &UA2955787},
	{SG28016, &UA2955964},
	{SG28017, &UA2955985},
	{SG28018, &UA2956074},
	{SG28019, &UA2956245},
	{SG28020, &UA2956312},
	{SG28021, &UA2956700},
	{SG28022, &UA2956766},
	{SG28023, &UA2956864},
	{SG28024, &UA2956989},
	{SG28025, &UA2957074},
	{SG28027, &UA2957143},
	{SG28028, &UA2957206},
	{SG28029, &UA2957257},
	{SG28030, &UA2957313},
	{SG28032, &UA2957321},
	{SG28033, &UA2957495},
	{SG28034, &UA2957637},
	{SG28035, &UA2957672},
	{SG28036, &UA2957692},
	{SG28037, &UA2957939},
	{SG28038, &UA2958037},
	{SG28039, &UA2958044},
	{SG28040, &UA2958268},
	{SG28041, &UA2958395},
	{SG28042, &UA2958537},
	{SG28044, &UA2958594},
	{SG28046, &UA2958610},
	{SG28047, &UA2958628},
	{SG28048, &UA2959375},
	{SG28049, &UA2959619},
	{SG28050, &UA2959620},
	{SG28051, &UA2959755},
	{SG28052, &UA2960708},
	{SG28053, &UA2961487},
	{SG28054, &UA2962443},
	{SG28055, &UA2963655},
	{SG28056, &UA2964103},
	{SG28057, &UA2966006},
	{SG28058, &UA2972409},
	{SG28059, &UA2982053},
	{SG28060, &UA2982839},
	{SG28061, &UA2982874},
	{SG28062, &UA2983156},
	{SG28064, &UA2983206},
	{SG28065, &UA2983385},
	{SG28066, &UA2983556},
	{SG28067, &UA2983806},
	{SG28068, &UA2983814},
	{SG28069, &UA2983905},
	{SG28070, &UA2984219},
	{SG28071, &UA2984331},
	{SG28072, &UA2984720},
	{SG28073, &UA2984770},
	{SG28074, &UA2984758},
	{SG28075, &UA2984814},
	{SG28076, &UA2984825},
	{SG28077, &UA2984999},
	{SG28079, &UA2985161},
	{SG28080, &UA2985344},
	{SG28081, &UA2985445},
	{SG28082, &UA2985493},
	{SG28083, &UA2985587},
	{SG28084, &UA2985820},
	{SG28085, &UA2985863},
	{SG28086, &UA2985977},
	{SG28087, &UA2986070},
	{SG28088, &UA2986496},
	{SG28089, &UA2986580},
	{SG28090, &UA2986583},
	{SG28091, &UA2986658},
	{SG28092, &UA2986682},
	{SG28093, &UA2986753},
	{SG28094, &UA2986794},
	{SG28095, &UA2986992},
	{SG28096, &UA2987020},
	{SG28097, &UA2987023},
	{SG28098, &UA2987185},
	{SG28100, &UA2987307},
	{SG28101, &UA2987319},
	{SG28102, &UA2987324},
	{SG28103, &UA2987347},
	{SG28104, &UA2987424},
	{SG28105, &UA2988328},
	{SG28106, &UA2988581},
	{SG28107, &UA2988819},
	{SG28108, &UA2989137},
	{SG28109, &UA2989364},
	{SG28110, &UA2989367},
	{SG28111, &UA2989629},
	{SG28112, &UA2989870},
	{SG28113, &UA2990545},
	{SG28114, &UA2990736},
	{SG28115, &UA2990902},
	{SG28116, &UA2991832},
	{SG28117, &UA2992216},
	{SG28119, &UA2992220},
	{SG28120, &UA2992257},
	{SG28121, &UA2992362},
	{SG28122, &UA2992542},
	{SG28123, &UA2992660},
	{SG28124, &UA2992805},
	{SG28125, &UA2992823},
	{SG28126, &UA2993051},
	{SG28127, &UA2993096},
	{SG28128, &UA2993286},
	{SG28129, &UA2993815},
	{SG28130, &UA2993870},
	{SG28131, &UA2994096},
	{SG28132, &UA2994218},
	{SG28134, &UA2994251},
	{SG28135, &UA2994257},
	{SG28136, &UA2994329},
	{SG28137, &UA2994440},
	{SG28138, &UA2994566},
	{SG28139, &UA2994623},
	{SG28140, &UA2994750},
	{SG28141, &UA2994774},
	{SG28142, &UA2994883},
	{SG28143, &UA2994903},
	{SG28144, &UA2994987},
	{SG28145, &UA2995010},
	{SG28146, &UA2995359},
	{SG28147, &UA2995543},
	{SG28148, &UA2995768},
	{SG28149, &UA2995881},
	{SG28150, &UA2996010},
	{SG28151, &UA2996212},
	{SG28152, &UA2996222},
	{SG28153, &UA2996277},
	{SG28154, &UA2996339},
	{SG28155, &UA2996343},
	{SG28156, &UA2996383},
	{SG28157, &UA2996452},
	{SG28158, &UA2996721},
	{SG28159, &UA2996725},
	{SG28161, &UA2996746},
	{SG28162, &UA2996847},
	{SG28165, &UA2996944},
	{SG28167, &UA2997099},
	{SG28169, &UA2997169},
	{SG28171, &UA2997182},
	{SG28172, &UA2997215},
	{SG28175, &UA2997259},
	{SG28177, &UA2997339},
	{SG28179, &UA2997392},
	{SG28181, &UA2997431},
	{SG28184, &UA2997434},
	{SG28186, &UA2997480},
	{SG28188, &UA2997581},
	{SG28190, &UA2997591},
	{SG28192, &UA2997627},
	{SG28194, &UA2997691},
	{SG28197, &UA2997900},
	{SG28198, &UA2997919},
	{SG28199, &UA2998026},
	{SG28200, &UA2998114},
	{SG28201, &UA2998123},
	{SG28202, &UA2998155},
	{SG28203, &UA2998158},
	{SG28204, &UA2998168},
	{SG28205, &UA2998284},
	{SG28206, &UA2998308},
	{SG28207, &UA2998408},
	{SG28208, &UA2998550},
	{SG28209, &UA2998562},
	{SG28210, &UA2998577},
	{SG28211, &UA2998651},
	{SG28212, &UA2998702},
	{SG28213, &UA2998820},
	{SG28214, &UA2998830},
	{SG28216, &UA2998986},
	{SG28217, &UA2999014},
	{SG28218, &UA2999049},
	{SG28219, &UA2999055},
	{SG28220, &UA2999096},
	{SG28221, &UA2999408},
	{SG28222, &UA2999469},
	{SG28223, &UA2999553},
	{SG28224, &UA2999764},
	{SG28225, &UA3000521},
	{SG28226, &UA3000577},
	{SG28227, &UA3000807},
	{SG28228, &UA3001707},
	{SG28229, &UA3001710},
	{SG28230, &UA3002428},
	{SG28231, &UA3002578},
	{SG28232, &UA3002648},
	{SG28233, &UA3002660},
	{SG28234, &UA3002765},
	{SG28235, &UA3002889},
	{SG28236, &UA3002983},
	{SG28237, &UA3003143},
	{SG28238, &UA3003266},
	{SG28239, &UA3004081},
	{SG28240, &UA3004782},
	{SG28241, &UA3005093},
	{SG28243, &UA3005192},
	{SG28244, &UA3006371},
	{SG28245, &UA3006529},
	{SG28246, &UA3006832},
	{SG28247, &UA3006897},
	{SG28248, &UA3007094},
	{SG28249, &UA3007216},
	{SG28250, &UA3007311},
	{SG28253, &UA3008012},
	{SG28254, &UA3008509},
	{SG28255, &UA3008554},
	{SG28256, &UA3009001},
	{SG28257, &UA3010165},
	{SG28258, &UA3010763},
	{SG28259, &UA3011117},
	{SG28260, &UA3011752},
	{SG28261, &UA3011920},
	{SG28263, &UA3012615},
	{SG28264, &UA3012659},
	{SG28266, &UA3012703},
	{SG28267, &UA3012753},
	{SG28268, &UA3012959},
	{SG28269, &UA3013082},
	{SG28270, &UA3013132},
	{SG28271, &UA3013241},
	{SG28272, &UA3013246},
	{SG28273, &UA3013265},
	{SG28274, &UA3013798},
	{SG28276, &UA3014639},
	{SG28279, &UA3014731},
	{SG28280, &UA3015101},
	{SG28281, &UA3016014},
	{SG28282, &UA3016314},
	{SG28283, &UA3016614},
	{SG28284, &UA3016943},
	{SG28285, &UA3017155},
	{SG28286, &UA3017523},
	{SG28287, &UA3017846},
	{SG28288, &UA3018297},
	{SG28290, &UA3018905},
	{SG28291, &UA3019065},
	{SG28293, &UA3019085},
	{SG28295, &UA3019116},
	{SG28298, &UA3020243},
	{SG28302, &UA3021050},
	{SG28304, &UA3021265},
	{SG28306, &UA3021274},
	{SG28309, &UA3021526},
	{SG28312, &UA3022883},
	{SG28313, &UA3023608},
	{SG28315, &UA3024044},
	{SG28316, &UA3024130},
	{SG28317, &UA3024175},
	{SG28318, &UA3025564},
	{SG28319, &UA3026630},
	{SG28320, &UA3027504},
	{SG28321, &UA3027512},
	{SG28322, &UA3032057},
	{SG28323, &UA3033831},
	{SG28324, &UA3033949},
	{SG28325, &UA3034860},
	{SG28326, &UA3043124},
	{SG28327, &UA3044025},
	{SG28328, &UA3044130},
	{SG28329, &UA3044389},
	{SG28330, &UA3044400},
	{SG28331, &UA3044510},
	{SG28332, &UA3044557},
	{SG28333, &UA3044641},
	{SG28334, &UA3044660},
	{SG28335, &UA3044686},
	{SG28336, &UA3044723},
	{SG28337, &UA3044780},
	{SG28338, &UA3044838},
	{SG28339, &UA3044841},
	{SG28340, &UA3045010},
	{SG28341, &UA3045026},
	{SG28342, &UA3045056},
	{SG28343, &UA3045404},
	{SG28344, &UA3045445},
	{SG28345, &UA3045478},
	{SG28347, &UA3045607},
	{SG28348, &UA3045665},
	{SG28349, &UA3045814},
	{SG28350, &UA3045884},
	{SG28351, &UA3047205},
	{SG28352, &UA3047274},
	{SG28353, &UA3047528},
	{SG28354, &UA3047585},
	{SG28355, &UA3048124},
	{SG28356, &UA3048153},
	{SG28357, &UA3048244},
	{SG28358, &UA3048419},
	{SG28359, &UA3048605},
	{SG28360, &UA3048617},
	{SG28361, &UA3048724},
	{SG28363, &UA3048728},
	{SG28364, &UA3048791},
	{SG28366, &UA3048834},
	{SG28368, &UA3048854},
	{SG28369, &UA3048885},
	{SG28370, &UA3048909},
	{SG28372, &UA3048911},
	{SG28373, &UA3049085},
	{SG28374, &UA3049104},
	{SG28375, &UA3049164},
	{SG28376, &UA3049213},
	{SG28378, &UA3049369},
	{SG28379, &UA3049656},
	{SG28381, &UA3049697},
	{SG28383, &UA3049749},
	{SG28384, &UA3049802},
	{SG28385, &UA3049827},
	{SG28388, &UA3049931},
	{SG28390, &UA3050108},
	{SG28391, &UA3050249},
	{SG28393, &UA3050251},
	{SG28394, &UA3050309},
	{SG28395, &UA3050823},
	{SG28397, &UA3050864},
	{SG28399, &UA3051165},
	{SG28401, &UA3051399},
	{SG28403, &UA3051979},
	{SG28404, &UA3052369},
	{SG28406, &UA3052380},
	{SG28408, &UA3053691},
	{SG28410, &UA3055382},
	{SG28411, &UA3056304},
	{SG28413, &UA3056558},
	{SG28415, &UA3056561},
	{SG28416, &UA3056631},
	{SG28418, &UA3056855},
	{SG28420, &UA3057017},
	{SG28421, &UA3057070},
	{SG28422, &UA3057256},
	{SG28423, &UA3057580},
	{SG28424, &UA3057650},
	{SG28425, &UA3057718},
	{SG28426, &UA3057786},
	{SG28427, &UA3058014},
	{SG28428, &UA3058099},
	{SG28429, &UA3058426},
	{SG28430, &UA3058601},
	{SG28432, &UA3058755},
	{SG28433, &UA3058854},
	{SG28435, &UA3058921},
	{SG28436, &UA3059027},
	{SG28437, &UA3059299},
	{SG28438, &UA3059626},
	{SG28439, &UA3059709},
	{SG28440, &UA3060702},
	{SG28441, &UA3063783},
	{SG28442, &UA3064178},
	{SG28444, &UA3070252},
	{SG28446, &UA3070511},
	{SG28447, &UA3071391},
	{SG28448, &UA3071412},
	{SG28449, &UA3071667},
	{SG28450, &UA3071713},
	{SG28452, &UA3071777},
	{SG28453, &UA3071848},
	{SG28454, &UA3071940},
	{SG28456, &UA3072041},
	{SG28458, &UA3072205},
	{SG28460, &UA3072234},
	{SG28461, &UA3072452},
	{SG28462, &UA3072560},
	{SG28464, &UA3073172},
	{SG28466, &UA3073569},
	{SG28467, &UA3073830},
	{SG28469, &UA3073928},
	{SG28470, &UA3074120},
	{SG28472, &UA3074250},
	{SG28474, &UA3074283},
	{SG28475, &UA3074393},
	{SG28476, &UA3074552},
	{SG28477, &UA3074723},
	{SG28479, &UA3077856},
	{SG28481, &UA3078148},
	{SG28482, &UA3078207},
	{SG28483, &UA3078215},
	{SG28484, &UA3078365},
	{SG28485, &UA3078562},
	{SG28486, &UA3078925},
	{SG28487, &UA3079195},
	{SG28488, &UA3079651},
	{SG28489, &UA3079824},
	{SG28490, &UA3080007},
	{SG28491, &UA3080919},
	{SG28492, &UA3081580},
	{SG28493, &UA3081634},
	{SG28494, &UA3086113},
	{SG28495, &UA3086474},
	{SG28496, &UA3086521},
	{SG28497, &UA3086759},
	{SG28499, &UA3086962},
	{SG28500, &UA3087016},
	{SG28501, &UA3087224},
	{SG28503, &UA3087236},
	{SG28505, &UA3087306},
	{SG28507, &UA3087698},
	{SG28509, &UA3087853},
	{SG28510, &UA3087958},
	{SG28511, &UA3088149},
	{SG28513, &UA3088261},
	{SG28514, &UA3088328},
	{SG28515, &UA3088669},
	{SG28516, &UA3088788},
	{SG28517, &UA3088923},
	{SG28518, &UA3090062},
	{SG28519, &UA3091814},
	{SG28520, &UA3091850},
	{SG28521, &UA3091976},
	{SG28522, &UA3091999},
	{SG28523, &UA3092217},
	{SG28524, &UA3092235},
	{SG28525, &UA3092320},
	{SG28526, &UA3092339},
	{SG28527, &UA3092363},
	{SG28528, &UA3092443},
	{SG28529, &UA3092457},
	{SG28530, &UA3092559},
	{SG28531, &UA3092592},
	{SG28532, &UA3092915},
	{SG28533, &UA3093375},
	{SG28534, &UA3096206},
	{SG28535, &UA3099328},
	{SG28536, &UA3099852},
	{SG28537, &UA3099878},
	{SG28538, &UA3100206},
	{SG28539, &UA3100283},
	{SG28540, &UA3100634},
	{SG28541, &UA3101078},
	{SG28542, &UA3101328},
	{SG28543, &UA3101365},
	{SG28544, &UA3101421},
	{SG28545, &UA3101975},
	{SG28546, &UA3102087},
	{SG28547, &UA3102617},
	{SG28548, &UA3102788},
	{SG28549, &UA3102860},
	{SG28550, &UA3102926},
	{SG28551, &UA3102985},
	{SG28552, &UA3103124},
	{SG28555, &UA3103140},
	{SG28556, &UA3103184},
	{SG28557, &UA3103208},
	{SG28558, &UA3103334},
	{SG28559, &UA3103349},
	{SG28560, &UA3103609},
	{SG28566, &UA3103725},
	{SG28568, &UA3103941},
	{SG28571, &UA3104293},
	{SG28573, &UA3105643},
	{SG28574, &UA3106902},
	{SG28575, &UA3106980},
	{SG28577, &UA3107411},
	{SG28578, &UA3107460},
	{SG28580, &UA3107520},
	{SG28583, &UA3107806},
	{SG28584, &UA3107891},
	{SG28585, &UA3107946},
	{SG28586, &UA3107944},
	{SG28588, &UA3107964},
	{SG28590, &UA3108043},
	{SG28592, &UA3108238},
	{SG28594, &UA3108421},
	{SG28596, &UA3109178},
	{SG28597, &UA3109195},
	{SG28599, &UA3109449},
	{SG28601, &UA3110393},
	{SG28602, &UA3111065},
	{SG28604, &UA3114388},
	{SG28605, &UA3114581},
	{SG28606, &UA3118459},
	{SG28608, &UA3118877},
	{SG28609, &UA3119235},
	{SG28611, &UA3119310},
	{SG28612, &UA3119439},
	{SG28613, &UA3119644},
	{SG28614, &UA3119655},
	{SG28615, &UA3120138},
	{SG28617, &UA3120159},
	{SG28619, &UA3120384},
	{SG28620, &UA3120386},
	{SG28621, &UA3120480},
	{SG28622, &UA3120537},
	{SG28624, &UA3120687},
	{SG28625, &UA3120783},
	{SG28626, &UA3120865},
	{SG28627, &UA3120978},
	{SG28629, &UA3121027},
	{SG28631, &UA3121029},
	{SG28633, &UA3121038},
	{SG28634, &UA3121076},
	{SG28636, &UA3121134},
	{SG28638, &UA3121187},
	{SG28640, &UA3121493},
	{SG28641, &UA3121724},
	{SG28643, &UA3121816},
	{SG28644, &UA3122156},
	{SG28645, &UA3122230},
	{SG28646, &UA3122362},
	{SG28647, &UA3122612},
	{SG28648, &UA3122629},
	{SG28649, &UA3122676},
	{SG28650, &UA3122898},
	{SG28651, &UA3124514},
	{SG28652, &UA3124522},
	{SG28653, &UA3124653},
	{SG28654, &UA3124677},
	{SG28655, &UA3124807},
	{SG28656, &UA3124841},
	{SG28657, &UA3124882},
	{SG28659, &UA3124920},
	{SG28660, &UA3124961},
	{SG28662, &UA3125099},
	{SG28664, &UA3125177},
	{SG28665, &UA3125183},
	{SG28666, &UA3125241},
	{SG28669, &UA3125317},
	{SG28670, &UA3125357},
	{SG28671, &UA3125371},
	{SG28672, &UA3125593},
	{SG28673, &UA3126538},
	{SG28674, &UA3126731},
	{SG28675, &UA3126934},
	{SG28676, &UA3127073},
	{SG28677, &UA3127234},
	{SG28884, &UA1996293},
	{SG28887, &UA2027795},
	{SG28889, &UA2182288},
	{SG28891, &UA2231932},
	{SG28893, &UA2238852},
	{SG28895, &UA2258019},
	{SG28898, &UA2339118},
	{SG28901, &UA2344312},
	{SG28903, &UA2384857},
	{SG28905, &UA2416135},
	{SG28906, &UA2426155},
	{SG28907, &UA2430757},
	{SG28908, &UA2437181},
	{SG28910, &UA2457086},
	{SG28912, &UA2459652},
	{SG28914, &UA2460210},
	{SG28916, &UA2461987},
	{SG28917, &UA2462122},
	{SG28918, &UA2467429},
	{SG28920, &UA2468160},
	{SG28922, &UA2487450},
	{SG28924, &UA2506967},
	{SG28926, &UA2507592},
	{SG28927, &UA2507775},
	{SG28929, &UA2528153},
	{SG28931, &UA2530022},
	{SG28933, &UA2530660},
	{SG28934, &UA2559114},
	{SG28936, &UA2597011},
	{SG28939, &UA2655303},
	{SG28941, &UA2655464},
	{SG28942, &UA2657401},
	{SG28943, &UA2692667},
	{SG28945, &UA2695347},
	{SG28947, &UA2716090},
	{SG28949, &UA2763991},
	{SG28951, &UA2847462},
	{SG28952, &UA2850239},
	{SG28954, &UA2878254},
	{SG28957, &UA2938111},
	{SG28960, &UA2986784},
	{SG29221, &UA2348567},
	{SG29222, &UA2399898},
	{SG29223, &UA2401575},
	{SG29224, &UA2404682},
	{SG29225, &UA2424681},
	{SG29226, &UA2458928},
	{SG29227, &UA2578868},
	{SG29228, &UA2596905},
	{SG29229, &UA2659489},
	{SG29230, &UA2659548},
	{SG29238, &UA2240792},
	{SG29239, &UA2406873},
	{SG29240, &UA2459479},
	{SG29241, &UA2459527},
	{SG29242, &UA2461034},
	{SG29243, &UA2504871},
	{SG29244, &UA2505880},
	{SG29245, &UA2512818},
	{SG29246, &UA2707796},
	{SG29247, &UA2938234},
	{SG29248, &UA2996707},
	{SG29249, &UA2998807},
	{SG29250, &UA3103362},
	{SG29427, &UA2029463},
	{SG29428, &UA2229157},
	{SG29429, &UA2229551},
	{SG29430, &UA2230021},
	{SG29432, &UA2361305},
	{SG29434, &UA2362262},
	{SG29435, &UA2362528},
	{SG29436, &UA2366654},
	{SG29437, &UA2367166},
	{SG29438, &UA2368130},
	{SG29439, &UA2380215},
	{SG29440, &UA2381673},
	{SG29441, &UA2382941},
	{SG29442, &UA2383519},
	{SG29443, &UA2385158},
	{SG29444, &UA2385416},
	{SG29445, &UA2385995},
	{SG29446, &UA2391430},
	{SG29447, &UA2399494},
	{SG29448, &UA2401678},
	{SG29449, &UA2402539},
	{SG29450, &UA2403281},
	{SG29451, &UA2404384},
	{SG29452, &UA2418952},
	{SG29453, &UA2419770},
	{SG29454, &UA2423760},
	{SG29455, &UA2425184},
	{SG29456, &UA2426954},
	{SG29457, &UA2429877},
	{SG29458, &UA2432781},
	{SG29459, &UA2434386},
	{SG29460, &UA2451198},
	{SG29461, &UA2459108},
	{SG29462, &UA2460424},
	{SG29463, &UA2466826},
	{SG29464, &UA2467256},
	{SG29465, &UA2468147},
	{SG29466, &UA2469223},
	{SG29467, &UA2481871},
	{SG29468, &UA2484369},
	{SG29469, &UA2485374},
	{SG29470, &UA2485549},
	{SG29471, &UA2486238},
	{SG29472, &UA2487323},
	{SG29473, &UA2488403},
	{SG29474, &UA2504455},
	{SG29475, &UA2507381},
	{SG29476, &UA2509594},
	{SG29477, &UA2510710},
	{SG29478, &UA2516571},
	{SG29479, &UA2518502},
	{SG29480, &UA2527335},
	{SG29481, &UA2528232},
	{SG29482, &UA2528611},
	{SG29483, &UA2530251},
	{SG29484, &UA2531403},
	{SG29485, &UA2532277},
	{SG29486, &UA2534662},
	{SG29487, &UA2556758},
	{SG29488, &UA2557456},
	{SG29489, &UA2558069},
	{SG29490, &UA2558305},
	{SG29491, &UA2558477},
	{SG29492, &UA2561098},
	{SG29493, &UA2561731},
	{SG29494, &UA2563067},
	{SG29495, &UA2582860},
	{SG29496, &UA2596926},
	{SG29497, &UA2597872},
	{SG29498, &UA2597880},
	{SG29499, &UA2598415},
	{SG29500, &UA2599187},
	{SG29501, &UA2654304},
	{SG29502, &UA2655399},
	{SG29503, &UA2657282},
	{SG29504, &UA2658575},
	{SG29505, &UA2665715},
	{SG29506, &UA2666264},
	{SG29507, &UA2667173},
	{SG29508, &UA2667858},
	{SG29509, &UA2673826},
	{SG29510, &UA2697518},
	{SG29511, &UA2699514},
	{SG29512, &UA2707019},
	{SG29513, &UA2707205},
	{SG29514, &UA2708415},
	{SG29515, &UA2713594},
	{SG29516, &UA2728702},
	{SG29517, &UA2729077},
	{SG29518, &UA2730066},
	{SG29519, &UA2730854},
	{SG29520, &UA2731289},
	{SG29521, &UA2733222},
	{SG29522, &UA2734196},
	{SG29523, &UA2747447},
	{SG29524, &UA2747718},
	{SG29525, &UA2762938},
	{SG29526, &UA2779392},
	{SG29527, &UA2782002},
	{SG29528, &UA2791446},
	{SG29529, &UA2794679},
	{SG29530, &UA2797137},
	{SG29531, &UA2798955},
	{SG29532, &UA2799056},
	{SG29533, &UA2843931},
	{SG29534, &UA2854699},
	{SG29535, &UA2859362},
	{SG29536, &UA2862332},
	{SG29537, &UA2867518},
	{SG29538, &UA2869041},
	{SG29539, &UA2890715},
	{SG29540, &UA2934440},
	{SG29541, &UA2934900},
	{SG29542, &UA2952466},
	{SG29543, &UA2956127},
	{SG29544, &UA2965356},
	{SG29545, &UA2984326},
	{SG29546, &UA2987467},
	{SG29547, &UA2992957},
	{SG29549, &UA3000687},
	{SG29550, &UA3011163},
	{SG29551, &UA3023334},
	{SG29552, &UA3027018},
	{SG29553, &UA3043981},
	{SG29554, &UA3044395},
	{SG29555, &UA3046471},
	{SG29556, &UA3050298},
	{SG29557, &UA3072906},
	{SG29558, &UA3078552},
	{SG29559, &UA3086327},
	{SG29560, &UA3088792},
	{SG29561, &UA3090540},
	{SG29562, &UA3100693},
	{SG29563, &UA3102330},
	{SG29564, &UA3120818},
	{SG29565, &UA925689},
	{SG29566, &UA2228514},
	{SG29567, &UA2228657},
	{SG29568, &UA2229272},
	{SG29569, &UA2235432},
	{SG29570, &UA2235488},
	{SG29571, &UA2237769},
	{SG29572, &UA2261540},
	{SG29573, &UA2261633},
	{SG29574, &UA2261666},
	{SG29575, &UA2348449},
	{SG29576, &UA2355080},
	{SG29577, &UA2391289},
	{SG29578, &UA2400854},
	{SG29579, &UA2423661},
	{SG29580, &UA2505922},
	{SG29581, &UA2579675},
	{SG29618, &UA2400853},
	{SG29619, &UA2428594},
	{SG29620, &UA2506390},
	{SG29621, &UA2614891},
	{SG29622, &UA2615092},
	{SG29623, &UA2622301},
	{SG29624, &UA2622655},
	{SG29625, &UA2623227},
	{SG29626, &UA2623681},
	{SG29627, &UA2624741},
	{SG29628, &UA2625710},
	{SG29629, &UA2625937},
	{SG29630, &UA2629600},
	{SG29631, &UA2633920},
	{SG29632, &UA2634598},
	{SG29633, &UA2635767},
	{SG29634, &UA2635853},
	{SG29635, &UA2636198},
	{SG29636, &UA2636741},
	{SG29637, &UA2637053},
	{SG29638, &UA2637168},
	{SG29639, &UA2637170},
	{SG29640, &UA2637585},
	{SG29700, &UA2029858},
	{SG29701, &UA2032474},
	{SG29702, &UA2257991},
	{SG29703, &UA2258008},
	{SG29704, &UA2385422},
	{SG29716, &UA911827},
	{SG29718, &UA912404},
	{SG29720, &UA913190},
	{SG29722, &UA923784},
	{SG29724, &UA923809},
	{SG29726, &UA925182},
	{SG29728, &UA1950439},
	{SG29730, &UA1975914},
	{SG29732, &UA1980067},
	{SG29734, &UA1997253},
	{SG29736, &UA1999573},
	{SG29737, &UA2000589},
	{SG29739, &UA2028650},
	{SG29741, &UA2028677},
	{SG29743, &UA2033333},
	{SG29744, &UA2042683},
	{SG29745, &UA2204700},
	{SG29746, &UA2367187},
	{SG29747, &UA2405455},
	{SG29748, &UA2420671},
	{SG29749, &UA2445538},
	{SG29750, &UA2824517},
	{SG29751, &UA2983291},
	{SG29752, &UA2257969},
	{SG29753, &UA2258053},
	{SG29754, &UA2421673},
	{SG29755, &UA2426427},
	{SG29756, &UA2428998},
	{SG29757, &UA2429924},
	{SG29758, &UA2460083},
	{SG29759, &UA2461155},
	{SG29760, &UA2467901},
	{SG29761, &UA2468333},
	{SG29762, &UA2491522},
	{SG29763, &UA2509090},
	{SG29764, &UA2511292},
	{SG29765, &UA2531831},
	{SG29766, &UA2567408},
	{SG29767, &UA2658122},
	{SG29768, &UA2660433},
	{SG29769, &UA2661243},
	{SG29770, &UA2675587},
	{SG29771, &UA2699756},
	{SG29772, &UA2700354},
	{SG29773, &UA2707475},
	{SG29774, &UA2708167},
	{SG29775, &UA2714684},
	{SG29776, &UA2716915},
	{SG29777, &UA2719281},
	{SG29778, &UA2719294},
	{SG29779, &UA2778166},
	{SG29780, &UA2840065},
	{SG29781, &UA2849204},
	{SG29782, &UA2851597},
	{SG29783, &UA2852388},
	{SG29784, &UA2857639},
	{SG29785, &UA2893043},
	{SG29786, &UA2985380},
	{SG29787, &UA3044235},
	{SG29860, &UA924777},
	{SG29862, &UA1999352},
	{SG29864, &UA2012765},
	{SG29865, &UA2225717},
	{SG29866, &UA2367295},
	{SG29867, &UA2386283},
	{SG29868, &UA2386313},
	{SG29869, &UA2430899},
	{SG29870, &UA2456144},
	{SG29871, &UA2482583},
	{SG29872, &UA3118717},
	{SG29930, &UA2416777},
	{SG29931, &UA2427439},
	{SG30033, &UA2228224},
	{SG30034, &UA2391339},
	{SG30035, &UA2419879},
	{SG30036, &UA2421871},
	{SG30037, &UA2508087},
	{SG30038, &UA2615385},
	{SG30040, &UA2615572},
	{SG30042, &UA2616077},
	{SG30044, &UA2616762},
	{SG30045, &UA2621802},
	{SG30047, &UA2622149},
	{SG30048, &UA2623109},
	{SG30050, &UA2623971},
	{SG30051, &UA2625357},
	{SG30053, &UA2626195},
	{SG30055, &UA2626833},
	{SG30058, &UA2626857},
	{SG30061, &UA2627084},
	{SG30064, &UA2628386},
	{SG30068, &UA2628520},
	{SG30070, &UA2629093},
	{SG30072, &UA2630106},
	{SG30074, &UA2631067},
	{SG30075, &UA2631699},
	{SG30076, &UA2631902},
	{SG30077, &UA2632302},
	{SG30078, &UA2633239},
	{SG30079, &UA2633719},
	{SG30080, &UA2634994},
	{SG30081, &UA2635847},
	{SG30082, &UA2638018},
	{SG30083, &UA2973268},
	{SG30084, &UA2973464},
	{SG30085, &UA2973908},
	{SG30086, &UA2974319},
	{SG30087, &UA2974645},
	{SG30088, &UA2974793},
	{SG30089, &UA2974880},
	{SG30092, &UA2975031},
	{SG30095, &UA3021671},
	{SG30096, &UA3022801},
	{SG30097, &UA3023114},
	{SG30098, &UA3077963},
	{SG30099, &UA3107838},
	{SG30104, &UA2006229},
	{SG30105, &UA2006679},
	{SG30106, &UA2177336},
	{SG30107, &UA2186176},
	{SG30108, &UA2196742},
	{SG30109, &UA2204667},
	{SG30110, &UA2205857},
	{SG30111, &UA2353656},
	{SG30112, &UA2356504},
	{SG30113, &UA2357104},
	{SG30114, &UA2366802},
	{SG30115, &UA3065086},
	{SG30116, &UA3066532},
	{SG30117, &UA3066878},
	{SG30118, &UA3067893},
	{SG30119, &UA3081030},
	{SG30120, &UA3081185},
	{SG30121, &UA3081278},
	{SG30122, &UA3083293},
	{SG30123, &UA3094436},
	{SG30124, &UA3111553},
	{SG30125, &UA3114555},
	{SG30152, &UA2566453},
	{SG30153, &UA2566680},
	{SG30154, &UA2566710},
	{SG30155, &UA2566797},
	{SG30156, &UA2566822},
	{SG30157, &UA2566931},
	{SG30158, &UA2566962},
	{SG30159, &UA2567095},
	{SG30160, &UA2567098},
	{SG30161, &UA2567133},
	{SG30162, &UA2567268},
	{SG30163, &UA2567327},
	{SG30164, &UA2567495},
	{SG30167, &UA2600345},
	{SG30170, &UA2667177},
	{SG30171, &UA2667401},
	{SG30172, &UA2667472},
	{SG30173, &UA2667837},
	{SG30174, &UA2732962},
	{SG30176, &UA2749910},
	{SG30177, &UA2988586},
	{SG30183, &UA2566531},
	{SG30184, &UA2566532},
	{SG30189, &UA2566795},
	{SG30190, &UA2566807},
	{SG30192, &UA2566809},
	{SG30193, &UA2566888},
	{SG30194, &UA2567193},
	{SG30195, &UA2583278},
	{SG30196, &UA2583801},
	{SG30197, &UA2583962},
	{SG30198, &UA2584095},
	{SG30199, &UA2732916},
	{SG30363, &UA2380718},
	{SG30364, &UA2480083},
	{SG30366, &UA2487853},
	{SG30369, &UA2615389},
	{SG30371, &UA2616134},
	{SG30373, &UA2616868},
	{SG30375, &UA2617156},
	{SG30376, &UA2623067},
	{SG30378, &UA2625843},
	{SG30380, &UA2625916},
	{SG30382, &UA2626671},
	{SG30385, &UA2630104},
	{SG30387, &UA2632112},
	{SG30389, &UA2632831},
	{SG30391, &UA2633244},
	{SG30393, &UA2634421},
	{SG30394, &UA2635578},
	{SG30397, &UA2636826},
	{SG30501, &UA2401947},
	{SG30502, &UA2420752},
	{SG30504, &UA2459883},
	{SG30506, &UA2597291},
	{SG30508, &UA2615587},
	{SG30509, &UA2618555},
	{SG30510, &UA2621988},
	{SG30511, &UA2622946},
	{SG30512, &UA2623591},
	{SG30513, &UA2624049},
	{SG30514, &UA2624726},
	{SG30515, &UA2625490},
	{SG30516, &UA2626187},
	{SG30517, &UA2627717},
	{SG30518, &UA2627812},
	{SG30519, &UA2628175},
	{SG30520, &UA2628385},
	{SG30522, &UA2628587},
	{SG30525, &UA2629786},
	{SG30527, &UA2630533},
	{SG30528, &UA2631468},
	{SG30529, &UA2633652},
	{SG30535, &UA2636855},
	{SG30538, &UA2636863},
	{SG30541, &UA2637668},
	{SG30544, &UA2638003},
	{SG30547, &UA2223521},
	{SG30551, &UA2257717},
	{SG30554, &UA2366528},
	{SG30557, &UA2455167},
	{SG30560, &UA2470131},
	{SG30562, &UA2546755},
	{SG30564, &UA2612307},
	{SG30567, &UA2614509},
	{SG30570, &UA2615681},
	{SG30575, &UA2625385},
	{SG30577, &UA2625836},
	{SG30580, &UA2625991},
	{SG30584, &UA2627026},
	{SG30587, &UA2627083},
	{SG30590, &UA2627239},
	{SG30592, &UA2627350},
	{SG30594, &UA2627457},
	{SG30596, &UA2628156},
	{SG30599, &UA2628427},
	{SG30602, &UA2628605},
	{SG30604, &UA2628701},
	{SG30606, &UA2629673},
	{SG30607, &UA2629718},
	{SG30609, &UA2630111},
	{SG30611, &UA2631397},
	{SG30613, &UA2631501},
	{SG30614, &UA2633272},
	{SG30615, &UA2634177},
	{SG30616, &UA2635191},
	{SG30617, &UA2635212},
	{SG30618, &UA2635327},
	{SG30619, &UA2636217},
	{SG30620, &UA2362445},
	{SG30621, &UA2384080},
	{SG30622, &UA2454027},
	{SG30623, &UA2469497},
	{SG30624, &UA2553165},
	{SG30625, &UA2613144},
	{SG30626, &UA2615847},
	{SG30627, &UA2616182},
	{SG30628, &UA2621262},
	{SG30629, &UA2623627},
	{SG30630, &UA2625020},
	{SG30631, &UA2625378},
	{SG30632, &UA2626046},
	{SG30633, &UA2626446},
	{SG30634, &UA2626612},
	{SG30635, &UA2627147},
	{SG30636, &UA2627549},
	{SG30637, &UA2627621},
	{SG30638, &UA2627950},
	{SG30639, &UA2628310},
	{SG30640, &UA2629064},
	{SG30641, &UA2631042},
	{SG30642, &UA2631510},
	{SG30643, &UA2631559},
	{SG30644, &UA2632136},
	{SG30645, &UA2632242},
	{SG30646, &UA2632895},
	{SG30647, &UA2633476},
	{SG30648, &UA2634993},
	{SG30649, &UA2635640},
	{SG30650, &UA2636359},
	{SG30651, &UA2637471},
	{SG30668, &UA2973667},
	{SG30670, &UA2974281},
	{SG30674, &UA2975036},
	{SG30676, &UA2975533},
	{SG30679, &UA3022191},
	{SG30680, &UA3022272},
	{SG30682, &UA3023268},
	{SG30683, &UA3023548},
	{SG30694, &UA2566510},
	{SG30695, &UA2566526},
	{SG30696, &UA2566536},
	{SG30697, &UA2566640},
	{SG30698, &UA2567008},
	{SG30699, &UA2567115},
	{SG30700, &UA2567204},
	{SG30701, &UA2567216},
	{SG30702, &UA2567221},
	{SG30703, &UA2567259},
	{SG30704, &UA2567333},
	{SG30705, &UA2584098},
	{SG30706, &UA2585167},
	{SG30707, &UA2667184},
	{SG30714, &UA3065334},
	{SG30715, &UA3066896},
	{SG30716, &UA3067472},
	{SG30717, &UA3081049},
	{SG30718, &UA3093669},
	{SG30719, &UA3094976},
	{SG30720, &UA3113521},
	{SG30721, &UA2240795},
	{SG30781, &UA1996126},
	{SG30782, &UA2010500},
	{SG30783, &UA2205669},
	{SG30784, &UA2352787},
	{SG30785, &UA2384728},
	{SG30787, &UA2464749},
	{SG30788, &UA2477329},
	{SG30789, &UA2762454},
	{SG30791, &UA2852703},
	{SG30793, &UA3044150},
	{SG30836, &UA925476},
	{SG30837, &UA1982477},
	{SG30838, &UA1995937},
	{SG30839, &UA1996254},
	{SG30840, &UA2003523},
	{SG30841, &UA2028602},
	{SG30842, &UA2028639},
	{SG30843, &UA2204866},
	{SG30844, &UA2973894},
	{SG30861, &UA2974161},
	{SG30862, &UA2974387},
	{SG30863, &UA2974715},
	{SG30864, &UA2976014},
	{SG30865, &UA3022993},
	{SG30866, &UA3023436},
	{SG30867, &UA3107824},
	{SG30912, &UA3064880},
	{SG30916, &UA3065441},
	{SG30917, &UA3065465},
	{SG30918, &UA3066530},
	{SG30919, &UA3066880},
	{SG30920, &UA3081344},
	{SG30921, &UA3082222},
	{SG30922, &UA3126840},
	{SG30946, &UA2752948},
	{SG30947, &UA2753445},
	{SG30948, &UA2754244},
	{SG30949, &UA2754335},
	{SG30950, &UA2754575},
	{SG30951, &UA2755059},
	{SG30952, &UA2755353},
	{SG30953, &UA2756008},
	{SG30954, &UA2773843},
	{SG30955, &UA2800513},
	{SG30956, &UA2877176},
	{SG30958, &UA2877301},
	{SG30962, &UA2937747},
	{SG30963, &UA3073982},
	{SG30976, &UA2043996},
	{SG30977, &UA2043999},
	{SG30978, &UA2257673},
	{SG30979, &UA2362378},
	{SG30980, &UA2363034},
	{SG30981, &UA2368090},
	{SG30982, &UA2401470},
	{SG30983, &UA2482020},
	{SG30984, &UA2488651},
	{SG30985, &UA2594401},
	{SG30986, &UA2596344},
	{SG30987, &UA2671198},
	{SG30988, &UA2691109},
	{SG30989, &UA2713473},
	{SG30990, &UA2763608},
	{SG30991, &UA2933669},
	{SG30992, &UA2985035},
	{SG30993, &UA3044355},
	{SG31106, &UA2195394},
	{SG31108, &UA2382015},
	{SG31109, &UA2508189},
	{SG31111, &UA2655383},
	{SG31141, &UA2670605},
	{SG31143, &UA2671179},
	{SG31147, &UA2675691},
	{SG31148, &UA2676040},
	{SG31150, &UA2676184},
	{SG31153, &UA2676283},
	{SG31155, &UA2676424},
	{SG31158, &UA2676672},
	{SG31161, &UA2676731},
	{SG31164, &UA2677160},
	{SG31166, &UA2677287},
	{SG31168, &UA2708038},
	{SG31169, &UA2709363},
	{SG31170, &UA2734900},
	{SG31171, &UA2735057},
	{SG31172, &UA2735337},
	{SG31173, &UA2736251},
	{SG31175, &UA2864569},
	{SG31236, &UA2400968},
	{SG31237, &UA2657534},
	{SG31323, &UA2754410},
	{SG31324, &UA2755125},
	{SG31325, &UA2755127},
	{SG31326, &UA2755358},
	{SG31327, &UA2769746},
	{SG31328, &UA2769784},
	{SG31329, &UA2783527},
	{SG31330, &UA2866731},
	{SG31331, &UA2875469},
	{SG31383, &UA911957},
	{SG31384, &UA1996708},
	{SG31385, &UA2237076},
	{SG31386, &UA2405860},
	{SG31387, &UA3106722},
	{SG31388, &UA2237434},
	{SG31389, &UA2670730},
	{SG31390, &UA2673067},
	{SG31391, &UA2674743},
	{SG31392, &UA2675066},
	{SG31393, &UA2675840},
	{SG31394, &UA2676023},
	{SG31395, &UA2676090},
	{SG31397, &UA2676102},
	{SG31398, &UA2676677},
	{SG31399, &UA2676839},
	{SG31400, &UA2708119},
	{SG31401, &UA2713808},
	{SG31414, &UA2741598},
	{SG31415, &UA2741700},
	{SG31416, &UA2752817},
	{SG31417, &UA2753279},
	{SG31418, &UA2754079},
	{SG31419, &UA2754853},
	{SG31420, &UA2769039},
	{SG31421, &UA2769543},
	{SG31422, &UA2769691},
	{SG31423, &UA2029082},
	{SG31424, &UA2033054},
	{SG31425, &UA2201487},
	{SG31426, &UA2422459},
	{SG31427, &UA3085599},
	{SG31458, &UA2360625},
	{SG31459, &UA2423987},
	{SG31522, &UA2671033},
	{SG31523, &UA2671856},
	{SG31524, &UA2673537},
	{SG31525, &UA2673565},
	{SG31526, &UA2676029},
	{SG31527, &UA2676197},
	{SG31528, &UA2676240},
	{SG31529, &UA2676258},
	{SG31530, &UA2676484},
	{SG31531, &UA2676594},
	{SG31532, &UA2676778},
	{SG31533, &UA2677304},
	{SG31534, &UA2714575},
	{SG31535, &UA3087667},
	{SG31536, &UA911868},
	{SG31537, &UA924834},
	{SG31538, &UA1997242},
	{SG31539, &UA2006201},
	{SG31540, &UA2025440},
	{SG31541, &UA2225972},
	{SG31542, &UA2549926},
	{SG31631, &UA2261667},
	{SG31638, &UA2948181},
	{SG31639, &UA3021949},
	{SG31640, &UA2462432},
	{SG31678, &UA913421},
	{SG31679, &UA2014819},
	{SG31680, &UA2224898},
	{SG31681, &UA2240917},
	{SG31682, &UA2241915},
	{SG31683, &UA2242109},
	{SG31733, &UA2974735},
	{SG31777, &UA1996354},
	{SG31778, &UA2198944},
	{SG31781, &UA2205049},
	{SG31782, &UA2416217},
	{SG31789, &UA2676038},
	{SG31790, &UA2676578},
	{SG31840, &UA3081233},
	{SG31842, &UA3113184},
	{SG31847, &UA2676823},
	{SG31848, &UA2678420},
	{SG31849, &UA2707088},
	{SG32022, &UA3068412},
	{SG32097, &UA2168169},
	{SG32098, &UA2366065},
	{SG32099, &UA2405883},
	{SG32100, &UA2985264},
	{SG32110, &UA2359489},
	{SG32111, &UA2362482},
	{SG32113, &UA2421587},
	{SG32115, &UA2426166},
	{SG32117, &UA2746892},
	{SG32118, &UA2844831},
	{SG32145, &UA2386448},
	{SG32148, &UA2404343},
	{SG32151, &UA2406899},
	{SG32153, &UA2467535},
	{SG32156, &UA2697312},
	{SG32157, &UA2844020},
	{SG32158, &UA2717629},
	{SG32159, &UA2719020},
	{SG32160, &UA2729326},
	{SG32161, &UA2755228},
	{SG32162, &UA2770150},
	{SG32163, &UA2770164},
	{SG32164, &UA2784996},
	{SG32165, &UA2797462},
	{SG32166, &UA2799220},
	{SG32167, &UA2867396},
	{SG32168, &UA2869999},
	{SG32169, &UA2893877},
	{SG32170, &UA2970549},
	{SG32171, &UA2973547},
	{SG32172, &UA3012751},
	{SG32173, &UA3050029},
	{SG32174, &UA3050729},
	{SG32189, &UA3106821},
	{SG32190, &UA3132769},
	{SG32281, &UA2566541},
	{SG32282, &UA2567086},
	{SG32286, &UA2602016},
	{SG32377, &UA2028860},
	{SG32378, &UA2029523},
	{SG32380, &UA2201681},
	{SG32383, &UA2426728},
	{SG32385, &UA2429216},
	{SG32387, &UA2528070},
	{SG32390, &UA2652545},
	{SG32392, &UA2657311},
	{SG32397, &UA2994630},
	{SG32421, &UA2014443},
	{SG32422, &UA2029758},
	{SG32423, &UA2196131},
	{SG32426, &UA2201661},
	{SG32429, &UA2364021},
	{SG32433, &UA2447282},
	{SG32471, &UA2566623},
	{SG32474, &UA2566694},
	{SG32690, &UA2674173},
	{SG32691, &UA2707885},
	{SG32692, &UA2712166},
	{SG32693, &UA2719433},
	{SG32694, &UA3017534},
	{SG32695, &UA3019367},
	{SG32726, &UA2753609},
	{SG32727, &UA2774085},
	{SG32730, &UA2406512},
	{SG32732, &UA2422714},
	{SG32734, &UA2503307},
	{SG32735, &UA2507330},
	{SG32738, &UA2791170},
	{SG32740, &UA3044107},
	{SG32775, &UA2205676},
	{SG32777, &UA2205843},
	{SG32778, &UA2504042},
	{SG32779, &UA2556108},
	{SG32780, &UA2577867},
	{SG32782, &UA1996756},
	{SG32783, &UA2010072},
	{SG32784, &UA2030213},
	{SG32785, &UA2040604},
	{SG32786, &UA2202078},
	{SG32787, &UA2225215},
	{SG32813, &UA2527273},
	{SG32841, &UA1996093},
	{SG32842, &UA2191012},
	{SG32843, &UA2261533},
	{SG32844, &UA2261563},
	{SG32846, &UA2351962},
	{SG32848, &UA2455563},
	{SG32849, &UA2530259},
	{SG32850, &UA2361020},
	{SG32851, &UA2427401},
	{SG32852, &UA3048906},
	{SG32906, &UA2755135},
	{SG32907, &UA2232042},
	{SG32935, &UA2415906},
	{SG32937, &UA2425399},
	{SG32938, &UA2483622},
	{SG32939, &UA2506870},
	{SG32955, &UA1995357},
	{SG32956, &UA2013244},
	{SG32959, &UA2354545},
	{SG32961, &UA2791932},
	{SG32976, &UA2240794},
	{SG32977, &UA2483362},
	{SG33050, &UA2197983},
	{SG33059, &UA2422651},
	{SG33061, &UA2555690},
	{SG33062, &UA912948},
	{SG33064, &UA924882},
	{SG33065, &UA925481},
	{SG33068, &UA1996856},
	{SG33069, &UA2006928},
	{SG33070, &UA2029133},
	{SG33071, &UA2192583},
	{SG33072, &UA2201774},
	{SG33073, &UA2205089},
	{SG33074, &UA2238804},
	{SG33075, &UA2241047},
	{SG33078, &UA2241067},
	{SG33080, &UA2241103},
	{SG33081, &UA2241191},
	{SG33082, &UA2241221},
	{SG33083, &UA2241230},
	{SG33085, &UA2241241},
	{SG33087, &UA2241273},
	{SG33089, &UA2241063},
	{SG33091, &UA2241287},
	{SG33093, &UA2241289},
	{SG33095, &UA2241295},
	{SG33097, &UA2241311},
	{SG33099, &UA2241348},
	{SG33101, &UA2241352},
	{SG33102, &UA2241353},
	{SG33104, &UA2241360},
	{SG33106, &UA2241367},
	{SG33108, &UA2241389},
	{SG33110, &UA2241416},
	{SG33112, &UA2241434},
	{SG33113, &UA2241444},
	{SG33115, &UA2241482},
	{SG33116, &UA2241483},
	{SG33117, &UA2241489},
	{SG33118, &UA2241490},
	{SG33119, &UA2241540},
	{SG33120, &UA2241613},
	{SG33121, &UA2241706},
	{SG33124, &UA2241774},
	{SG33125, &UA2241929},
	{SG33126, &UA2242070},
	{SG33127, &UA2851521},
	{SG33129, &UA3073472},
	{SG33154, &UA2486402},
	{SG33156, &UA2532562},
	{SG33159, &UA2558010},
	{SG33161, &UA2659072},
	{SG33162, &UA2705961},
	{SG33164, &UA2707944},
	{SG33165, &UA2729611},
	{SG33167, &UA2738152},
	{SG33169, &UA2748467},
	{SG33170, &UA2765789},
	{SG33172, &UA2773648},
	{SG33174, &UA2796873},
	{SG33176, &UA2864026},
	{SG33177, &UA2866735},
	{SG33179, &UA2998269},
	{SG33180, &UA3005375},
	{SG33238, &UA2503000},
	{SG33295, &UA3022511},
	{SG33310, &UA3112024},
	{SG33312, &UA925371},
	{SG33314, &UA2032492},
	{SG33316, &UA2466118},
	{SG33386, &UA2462468},
	{SG33387, &UA2462469},
	{SG33388, &UA2462470},
	{SG33389, &UA2601133},
	{SG33390, &UA2672555},
	{SG33391, &UA2734000},
	{SG33392, &UA2734961},
	{SG33393, &UA2752405},
	{SG33394, &UA2939890},
	{SG33395, &UA2994509},
	{SG33396, &UA2994979},
	{SG33397, &UA3013033},
	{SG33398, &UA3015283},
	{SG33400, &UA3021332},
	{SG33402, &UA3024512},
	{SG33404, &UA3047296},
	{SG33406, &UA3083447},
	{SG33408, &UA3093648},
	{SG33489, &UA2566616},
	{SG33492, &UA2566749},
	{SG33496, &UA2583373},
	{SG33504, &UA3070497},
	{SG33516, &UA2346129},
	{SG33517, &UA2366804},
	{SG33541, &UA1996991},
	{SG33542, &UA2015559},
	{SG33543, &UA2033039},
	{SG33544, &UA2192560},
	{SG33545, &UA2825230},
	{SG33546, &UA2829792},
	{SG33565, &UA2566485},
	{SG33566, &UA2566760},
	{SG33567, &UA2566803},
	{SG33600, &UA1996203},
	{SG33601, &UA2222980},
	{SG33602, &UA2229200},
	{SG33603, &UA2420776},
	{SG33604, &UA2425779},
	{SG33605, &UA2706043},
	{SG33607, &UA2729222},
	{SG33609, &UA2763994},
	{SG33610, &UA2764987},
	{SG33612, &UA2817285},
	{SG33613, &UA2851794},
	{SG33614, &UA2863825},
	{SG33615, &UA3077116},
	{SG33651, &UA2653378},
	{SG33668, &UA2005268},
	{SG33669, &UA2031108},
	{SG33670, &UA2199446},
	{SG33677, &UA3064894},
	{SG33681, &UA3065474},
	{SG33683, &UA2413850},
	{SG33684, &UA2421524},
	{SG33685, &UA2694118},
	{SG33686, &UA2566971},
	{SG33687, &UA2571157},
	{SG33688, &UA2604718},
	{SG33693, &UA2973951},
	{SG33694, &UA2976011},
	{SG33695, &UA3065590},
	{SG33696, &UA2405702},
	{SG33710, &UA2223255},
	{SG33712, &UA2402025},
	{SG33768, &UA1996344},
	{SG33769, &UA2015072},
	{SG33771, &UA2204395},
	{SG33774, &UA2366706},
	{SG33776, &UA2722814},
	{SG33782, &UA2703934},
	{SG33783, &UA2748433},
	{SG33798, &UA2030158},
	{SG33800, &UA2034452},
	{SG33802, &UA2384121},
	{SG33805, &UA2385565},
	{SG33807, &UA2398767},
	{SG33809, &UA2401476},
	{SG33811, &UA2467335},
	{SG33813, &UA2482045},
	{SG33815, &UA2483513},
	{SG33817, &UA2484482},
	{SG33820, &UA2485914},
	{SG33822, &UA2504521},
	{SG33824, &UA2505252},
	{SG33826, &UA2505562},
	{SG33828, &UA2506261},
	{SG33829, &UA2506548},
	{SG33830, &UA2506604},
	{SG33833, &UA2528446},
	{SG33835, &UA2555950},
	{SG33838, &UA2557240},
	{SG33841, &UA2558287},
	{SG33844, &UA2598346},
	{SG33847, &UA2599699},
	{SG33850, &UA2659426},
	{SG33852, &UA2661468},
	{SG33855, &UA2696924},
	{SG33859, &UA2728980},
	{SG33906, &UA2426677},
	{SG33908, &UA2427461},
	{SG33909, &UA2467947},
	{SG33951, &UA2974303},
	{SG33953, &UA2975955},
	{SG33973, &UA2558536},
	{SG33975, &UA2665359},
	{SG33978, &UA2783540},
	{SG33997, &UA1996264},
	{SG33998, &UA1996834},
	{SG34000, &UA1998006},
	{SG34003, &UA1999862},
	{SG34005, &UA2004292},
	{SG34007, &UA2006967},
	{SG34009, &UA2197402},
	{SG34010, &UA2198086},
	{SG34011, &UA2198991},
	{SG34034, &UA2974320},
	{SG34094, &UA2578021},
	{SG34102, &UA2008441},
	{SG34103, &UA2029080},
	{SG34104, &UA2178343},
	{SG34106, &UA2201790},
	{SG34126, &UA2974289},
	{SG34128, &UA2974489},
	{SG34131, &UA3021647},
	{SG34134, &UA1955942},
	{SG34135, &UA2030442},
	{SG34136, &UA2229703},
	{SG34137, &UA2350042},
	{SG34138, &UA2415685},
	{SG34139, &UA2432476},
	{SG34143, &UA2480624},
	{SG34146, &UA2484534},
	{SG34148, &UA2487009},
	{SG34149, &UA2502756},
	{SG34150, &UA2503865},
	{SG34151, &UA2530303},
	{SG34152, &UA2558062},
	{SG34155, &UA2559413},
	{SG34157, &UA2578749},
	{SG34158, &UA2579353},
	{SG34159, &UA2580549},
	{SG34161, &UA2580954},
	{SG34163, &UA2595046},
	{SG34165, &UA2604087},
	{SG34167, &UA2654740},
	{SG34169, &UA2658691},
	{SG34171, &UA2668780},
	{SG34173, &UA2674299},
	{SG34176, &UA2694432},
	{SG34178, &UA2697480},
	{SG34180, &UA2701649},
	{SG34183, &UA2702092},
	{SG34186, &UA2703678},
	{SG34189, &UA2718671},
	{SG34191, &UA2732917},
	{SG34193, &UA2746328},
	{SG34195, &UA2749173},
	{SG34197, &UA2761994},
	{SG34199, &UA2762933},
	{SG34201, &UA2769056},
	{SG34203, &UA2781324},
	{SG34205, &UA2796590},
	{SG34207, &UA2830837},
	{SG34210, &UA2844467},
	{SG34212, &UA2849539},
	{SG34213, &UA2850363},
	{SG34214, &UA2853858},
	{SG34215, &UA2859927},
	{SG34216, &UA2936836},
	{SG34217, &UA2951853},
	{SG34218, &UA2959572},
	{SG34219, &UA2985293},
	{SG34220, &UA2995339},
	{SG34221, &UA2995416},
	{SG34222, &UA2999295},
	{SG34223, &UA2999685},
	{SG34224, &UA2999711},
	{SG34225, &UA3001757},
	{SG34226, &UA3002944},
	{SG34227, &UA3005248},
	{SG34228, &UA3005456},
	{SG34229, &UA3010325},
	{SG34230, &UA3019790},
	{SG34231, &UA3050672},
	{SG34232, &UA3050880},
	{SG34233, &UA3057512},
	{SG34234, &UA3078540},
	{SG34235, &UA3092798},
	{SG34236, &UA3093532},
	{SG34238, &UA3100640},
	{SG34240, &UA3102503},
	{SG34241, &UA3102811},
	{SG34242, &UA3107716},
	{SG34243, &UA3121630},
	{SG34245, &UA2462382},
	{SG34247, &UA2462407},
	{SG34281, &UA3065095},
	{SG34283, &UA3094770},
	{SG34300, &UA2975065},
	{SG34311, &UA3065473},
	{SG34312, &UA3066537},
	{SG34313, &UA3111016},
	{SG34323, &UA2860590},
	{SG34329, &UA1996138},
	{SG34331, &UA2261562},
	{SG34333, &UA2458175},
	{SG34349, &UA925439},
	{SG34350, &UA2030097},
	{SG34378, &UA2553810},
	{SG34379, &UA2658293},
	{SG34404, &UA2225892},
	{SG34428, &UA925291},
	{SG34430, &UA1998799},
	{SG34433, &UA2197252},
	{SG34436, &UA2241310},
	{SG34446, &UA2030306},
	{SG34447, &UA2030537},
	{SG34448, &UA2176569},
	{SG34449, &UA2190115},
	{SG34450, &UA2257994},
	{SG34452, &UA2352202},
	{SG34455, &UA2200541},
	{SG34456, &UA2466250},
	{SG34457, &UA2481322},
	{SG34458, &UA2424994},
	{SG34459, &UA2505401},
	{SG34473, &UA2580283},
	{SG34474, &UA2606182},
	{SG34475, &UA2659765},
	{SG34477, &UA3068053},
	{SG34485, &UA2566493},
	{SG34488, &UA2567111},
	{SG34490, &UA3074391},
	{SG34496, &UA2423589},
	{SG34497, &UA2428261},
	{SG34498, &UA2430759},
	{SG34499, &UA2469366},
	{SG34500, &UA2469501},
	{SG34501, &UA2581153},
	{SG34502, &UA2679515},
	{SG34503, &UA2714034},
	{SG34504, &UA2739583},
	{SG34505, &UA2951632},
	{SG34506, &UA2700065},
	{SG34507, &UA2796043},
	{SG34510, &UA2672517},
	{SG34530, &UA2774048},
	{SG34533, &UA2774071},
	{SG34535, &UA2868676},
	{SG34536, &UA2938009},
	{SG34550, &UA2413772},
	{SG34553, &UA2583922},
	{SG34555, &UA2704729},
	{SG34556, &UA2727268},
	{SG34559, &UA2749162},
	{SG34562, &UA2997705},
	{SG34564, &UA3045239},
	{SG34566, &UA3101582},
	{SG34574, &UA2741647},
	{SG34575, &UA2755546},
	{SG34595, &UA2362479},
	{SG34606, &UA2965602},
	{SG34609, &UA3032893},
	{SG34634, &UA2422999},
	{SG34635, &UA2460271},
	{SG34637, &UA2506425},
	{SG34638, &UA2667882},
	{SG34669, &UA2566761},
	{SG34670, &UA2566970},
	{SG34676, &UA2558737},
	{SG34677, &UA2668468},
	{SG34678, &UA2675088},
	{SG34681, &UA2719259},
	{SG34682, &UA2749101},
	{SG34683, &UA2855433},
	{SG34684, &UA2858606},
	{SG34685, &UA2892132},
	{SG34686, &UA2997784},
	{SG34687, &UA3014398},
	{SG34688, &UA3048693},
	{SG34689, &UA3100527},
	{SG34704, &UA2676261},
	{SG34705, &UA2676740},
	{SG34713, &UA2670891},
	{SG34715, &UA2676010},
	{SG34717, &UA2676423},
	{SG34800, &UA3064770},
	{SG34804, &UA2382139},
	{SG34816, &UA2715362},
	{SG34818, &UA2756011},
	{SG34819, &UA2232348},
	{SG34832, &UA2741601},
	{SG34833, &UA2754418},
	{SG34835, &UA2871783},
	{SG34840, &UA2044000},
	{SG34842, &UA2382935},
	{SG34844, &UA2236373},
	{SG34845, &UA2950571},
	{SG34846, &UA2652501},
	{SG34848, &UA2462396},
	{SG34853, &UA2566501},
	{SG34855, &UA2566763},
	{SG34858, &UA2667059},
	{SG34860, &UA2706286},
	{SG34873, &UA3131404},
	{SG34886, &UA2566625},
	{SG34888, &UA2567335},
	{SG34921, &UA924913},
	{SG34923, &UA1994861},
	{SG34925, &UA1996303},
	{SG34941, &UA2753059},
	{SG34944, &UA2754491},
	{SG34984, &UA1996275},
	{SG34986, &UA1997992},
	{SG34988, &UA1999152},
	{SG34989, &UA2002709},
	{SG34992, &UA2183088},
	{SG34994, &UA2194057},
	{SG34996, &UA2198668},
	{SG35000, &UA2422984},
	{SG35031, &UA3080915},
	{SG35077, &UA1997828},
	{SG35130, &UA2973965},
	{SG35131, &UA3131710},
	{SG35138, &UA2423351},
	{SG35165, &UA3111144},
	{SG35169, &UA2975559},
	{SG35171, &UA2401770},
	{SG35174, &UA2974305},
	{SG35177, &UA3023530},
	{SG35196, &UA2404324},
	{SG35198, &UA2429474},
	{SG35199, &UA2654416},
	{SG35201, &UA2719157},
	{SG35203, &UA2733510},
	{SG35205, &UA2735710},
	{SG35207, &UA2751428},
	{SG35209, &UA2781789},
	{SG35210, &UA2845665},
	{SG35212, &UA2864990},
	{SG35214, &UA2866785},
	{SG35215, &UA3059542},
	{SG35216, &UA2428512},
	{SG35218, &UA2483660},
	{SG35219, &UA2484902},
	{SG35221, &UA2578832},
	{SG35222, &UA2701413},
	{SG35223, &UA2889569},
	{SG35226, &UA2755531},
	{SG35289, &UA2223385},
	{SG35292, &UA2421408},
	{SG35294, &UA2832900},
	{SG35301, &UA2566483},
	{SG35318, &UA2974166},
	{SG35321, &UA2566639},
	{SG35350, &UA2507956},
	{SG35351, &UA2530310},
	{SG35352, &UA2557623},
	{SG35353, &UA2558297},
	{SG35355, &UA2558571},
	{SG35357, &UA2563907},
	{SG35359, &UA2659494},
	{SG35361, &UA2668484},
	{SG35363, &UA2670060},
	{SG35364, &UA2700838},
	{SG35365, &UA2704481},
	{SG35366, &UA2736804},
	{SG35368, &UA2766356},
	{SG35371, &UA2861815},
	{SG35373, &UA2870925},
	{SG35374, &UA2878482},
	{SG35377, &UA2942487},
	{SG35379, &UA3108144},
	{SG35381, &UA3119769},
	{SG35401, &UA2974044},
	{SG35402, &UA2418217},
	{SG35403, &UA2437355},
	{SG35404, &UA2974273},
	{SG35410, &UA2455509},
	{SG35413, &UA2840226},
	{SG35414, &UA2673093},
	{SG35417, &UA2676259},
	{SG35426, &UA2566445},
	{SG35428, &UA2566521},
	{SG35430, &UA2567388},
	{SG35440, &UA3016897},
	{SG35443, &UA3031691},
	{SG35448, &UA2606169},
	{SG35451, &UA2676235},
	{SG35453, &UA2676488},
	{SG35454, &UA2676645},
	{SG35455, &UA2712626},
	{SG35456, &UA2768177},
	{SG35459, &UA2236522},
	{SG35462, &UA2261605},
	{SG35468, &UA2385215},
	{SG35469, &UA2622781},
	{SG35474, &UA925195},
	{SG35475, &UA1996154},
	{SG35476, &UA2029740},
	{SG35477, &UA2029800},
	{SG35478, &UA2031348},
	{SG35479, &UA2233475},
	{SG35480, &UA2236868},
	{SG35481, &UA2827451},
	{SG35482, &UA2933787},
	{SG35483, &UA3099394},
	{SG35533, &UA2462474},
	{SG35537, &UA2429480},
	{SG35550, &UA2569929},
	{SG35554, &UA2586642},
	{SG35555, &UA1998256},
	{SG35556, &UA1998887},
	{SG35558, &UA2203274},
	{SG35567, &UA2770692},
	{SG35571, &UA3028626},
	{SG35581, &UA3065237},
	{SG35602, &UA3081478},
	{SG35609, &UA2973957},
	{SG35611, &UA2974268},
	{SG35612, &UA2974601},
	{SG35613, &UA3022696},
	{SG35614, &UA2566618},
	{SG35641, &UA2613446},
	{SG35643, &UA2614943},
	{SG35645, &UA2615126},
	{SG35647, &UA2615772},
	{SG35650, &UA2615843},
	{SG35652, &UA2616442},
	{SG35654, &UA2616480},
	{SG35657, &UA2619858},
	{SG35659, &UA2619860},
	{SG35681, &UA2676445},
	{SG35682, &UA2676927},
	{SG35689, &UA2570594},
	{SG35693, &UA2774042},
	{SG35694, &UA2783292},
	{SG35696, &UA2947005},
	{SG35699, &UA2965159},
	{SG35703, &UA3060683},
	{SG35711, &UA2981815},
	{SG35726, &UA2467840},
	{SG35727, &UA2167203},
	{SG35729, &UA2665502},
	{SG35747, &UA2872129},
	{SG35752, &UA2010892},
	{SG35753, &UA2198815},
	{SG35754, &UA2241301},
	{SG35755, &UA2241558},
	{SG35769, &UA2586628},
	{SG35779, &UA2566495},
	{SG35780, &UA2567028},
	{SG35782, &UA2567516},
	{SG35827, &UA2423174},
	{SG35828, &UA2425845},
	{SG35829, &UA2435430},
	{SG35831, &UA2698451},
	{SG35835, &UA2170778},
	{SG35836, &UA2416954},
	{SG35838, &UA2761963},
	{SG35841, &UA2763686},
	{SG35842, &UA3048198},
	{SG35857, &UA3130723},
	{SG35860, &UA3131300},
	{SG35862, &UA3132837},
	{SG35866, &UA3065088},
	{SG35867, &UA3080832},
	{SG35869, &UA3081382},
	{SG35874, &UA2566711},
	{SG35875, &UA2567318},
	{SG35877, &UA2382138},
	{SG35879, &UA2482536},
	{SG35880, &UA2651721},
	{SG35883, &UA2679138},
	{SG35884, &UA2709605},
	{SG35885, &UA2566798},
	{SG35886, &UA2583214},
	{SG35906, &UA2566698},
	{SG35908, &UA2863701},
	{SG35915, &UA2566546},
	{SG35923, &UA2566441},
	{SG35936, &UA2974807},
	{SG35963, &UA2360372},
	{SG35967, &UA2456702},
	{SG35970, &UA2482986},
	{SG35972, &UA2504840},
	{SG35973, &UA2580213},
	{SG35976, &UA2711702},
	{SG35978, &UA2726201},
	{SG35984, &UA3042652},
	{SG35988, &UA3120605},
	{SG35990, &UA3065157},
	{SG36004, &UA2469540},
	{SG36007, &UA2856411},
	{SG36036, &UA2197817},
	{SG36037, &UA2510412},
	{SG36038, &UA2625025},
	{SG36039, &UA2628506},
	{SG36045, &UA3022812},
	{SG36054, &UA2234950},
	{SG36055, &UA2708688},
	{SG36060, &UA2975058},
	{SG36065, &UA2741697},
	{SG36068, &UA2755337},
	{SG36076, &UA3065144},
	{SG36079, &UA2755082},
	{SG36084, &UA2782531},
	{SG36104, &UA2974812},
	{SG36106, &UA3022651},
	{SG36114, &UA2364882},
	{SG36123, &UA2754588},
	{SG36135, &UA3080137},
	{SG36138, &UA3081386},
	{SG36139, &UA3093616},
	{SG36140, &UA3093824},
	{SG36141, &UA3094442},
	{SG36142, &UA3094725},
	{SG36143, &UA3095258},
	{SG36144, &UA3108805},
	{SG36145, &UA3109249},
	{SG36146, &UA3109376},
	{SG36148, &UA3109480},
	{SG36149, &UA3109626},
	{SG36151, &UA3110394},
	{SG36153, &UA3110863},
	{SG36155, &UA3111211},
	{SG36156, &UA3111410},
	{SG36158, &UA3112544},
	{SG36159, &UA3112705},
	{SG36160, &UA3113030},
	{SG36161, &UA3113806},
	{SG36163, &UA3114670},
	{SG36167, &UA3126235},
	{SG36169, &UA3126346},
	{SG36171, &UA3126394},
	{SG36173, &UA3126506},
	{SG36175, &UA3126507},
	{SG36177, &UA3126525},
	{SG36178, &UA3126602},
	{SG36179, &UA3126605},
	{SG36181, &UA3126744},
	{SG36183, &UA3126813},
	{SG36184, &UA3126916},
	{SG36187, &UA3127023},
	{SG36199, &UA2223029},
	{SG36200, &UA2626815},
	{SG36217, &UA2772750},
	{SG36220, &UA2773883},
	{SG36221, &UA2799834},
	{SG36232, &UA2583965},
	{SG36253, &UA2671102},
	{SG36254, &UA2676577},
	{SG36258, &UA2675322},
	{SG36259, &UA2676345},
	{SG36260, &UA2709746},
	{SG36262, &UA2974492},
	{SG36263, &UA3022835},
	{SG36264, &UA3023917},
	{SG36270, &UA2471719},
	{SG36271, &UA2489173},
	{SG36272, &UA2764509},
	{SG36274, &UA2792307},
	{SG36275, &UA2851307},
	{SG36276, &UA2988361},
	{SG36279, &UA2975736},
	{SG36283, &UA2566539},
	{SG36285, &UA2603299},
	{SG36289, &UA3035292},
	{SG36291, &UA3108618},
	{SG36295, &UA2427590},
	{SG36296, &UA2728860},
	{SG36298, &UA2851944},
	{SG36306, &UA2754711},
	{SG36307, &UA2531197},
	{SG36308, &UA2670336},
	{SG36309, &UA3045402},
	{SG36310, &UA2973467},
	{SG36312, &UA3021200},
	{SG36317, &UA2187560},
	{SG36319, &UA2455277},
	{SG36320, &UA2611364},
	{SG36321, &UA2614705},
	{SG36324, &UA2615528},
	{SG36325, &UA2615989},
	{SG36327, &UA2616948},
	{SG36329, &UA2630977},
	{SG36337, &UA2878428},
	{SG36338, &UA3013336},
	{SG36340, &UA2755110},
	{SG36342, &UA2771272},
	{SG36351, &UA2754714},
	{SG36360, &UA2669948},
	{SG36381, &UA3083190},
	{SG36384, &UA2597910},
	{SG36385, &UA2599372},
	{SG36386, &UA2659815},
	{SG36393, &UA2434730},
	{SG36400, &UA2617090},
	{SG36403, &UA2618257},
	{SG36404, &UA2622375},
	{SG36407, &UA2628354},
	{SG36417, &UA2536405},
	{SG36419, &UA2582707},
	{SG36421, &UA2659389},
	{SG36422, &UA2778698},
	{SG36425, &UA2863483},
	{SG36429, &UA2233821},
	{SG36430, &UA2424628},
	{SG36438, &UA2741714},
	{SG36454, &UA2353220},
	{SG36456, &UA2367389},
	{SG36458, &UA2505931},
	{SG36460, &UA2674441},
	{SG36472, &UA2567448},
	{SG36502, &UA2755256},
	{SG36504, &UA2755343},
	{SG36508, &UA1996996},
	{SG36510, &UA2014303},
	{SG36512, &UA2232264},
	{SG36515, &UA3065149},
	{SG36526, &UA1997067},
	{SG36529, &UA2016297},
	{SG36531, &UA2197348},
	{SG36538, &UA2199095},
	{SG36539, &UA2205806},
	{SG36552, &UA924227},
	{SG36553, &UA2033239},
	{SG36555, &UA2044086},
	{SG36557, &UA2044091},
	{SG36559, &UA2044093},
	{SG36561, &UA2044101},
	{SG36563, &UA2044104},
	{SG36564, &UA2044109},
	{SG36565, &UA2044113},
	{SG36566, &UA2044177},
	{SG36567, &UA2044185},
	{SG36568, &UA2044213},
	{SG36569, &UA2044217},
	{SG36571, &UA2044229},
	{SG36573, &UA2044290},
	{SG36590, &UA3064991},
	{SG36653, &UA2422327},
	{SG36654, &UA2454885},
	{SG36655, &UA2458021},
	{SG36656, &UA2554966},
	{SG36657, &UA2754269},
	{SG36662, &UA3068334},
	{SG36663, &UA3081107},
	{SG36667, &UA1997332},
	{SG36668, &UA2014131},
	{SG36669, &UA2201512},
	{SG36670, &UA2205660},
	{SG36671, &UA2660062},
	{SG36683, &UA2676103},
	{SG36703, &UA1997231},
	{SG36704, &UA2257940},
	{SG36705, &UA2692189},
	{SG36706, &UA3064872},
	{SG36708, &UA1997078},
	{SG36709, &UA2044145},
	{SG36710, &UA2044197},
	{SG36712, &UA2710487},
	{SG36715, &UA2741710},
	{SG36721, &UA2675842},
	{SG36722, &UA2676333},
	{SG36727, &UA2230322},
	{SG36728, &UA2525007},
	{SG36729, &UA2763571},
	{SG36736, &UA2560716},
	{SG36737, &UA2749407},
	{SG36740, &UA2672770},
	{SG36742, &UA2751518},
	{SG36743, &UA2768978},
	{SG36746, &UA2937440},
	{SG36747, &UA2956418},
	{SG36748, &UA2972388},
	{SG36749, &UA3015367},
	{SG36750, &UA3019513},
	{SG36751, &UA3057072},
	{SG36752, &UA3057496},
	{SG36753, &UA2566755},
	{SG36754, &UA2634379},
	{SG36777, &UA2462489},
	{SG36778, &UA2737625},
	{SG36779, &UA2795987},
	{SG36780, &UA2797670},
	{SG36781, &UA2866116},
	{SG36784, &UA2935725},
	{SG36807, &UA2741731},
	{SG36844, &UA2257967},
	{SG36848, &UA2258056},
	{SG36883, &UA2675578},
	{SG36885, &UA2676182},
	{SG36887, &UA2671834},
	{SG36888, &UA2676194},
	{SG36891, &UA2676841},
	{SG36893, &UA3064983},
	{SG36894, &UA3067596},
	{SG36900, &UA2676027},
	{SG36904, &UA2403638},
	{SG36906, &UA2579014},
	{SG36907, &UA2975025},
	{SG36925, &UA2676933},
	{SG36941, &UA2676272},
	{SG36943, &UA2677167},
	{SG37002, &UA2041421},
	{SG37004, &UA2497791},
	{SG37077, &UA2583961},
	{SG37107, &UA2502264},
	{SG37141, &UA2938051},
	{SG37165, &UA2009386},
	{SG37167, &UA2353007},
	{SG37168, &UA2476367},
	{SG37169, &UA2835538},
	{SG37170, &UA2892389},
	{SG37186, &UA2754922},
	{SG37187, &UA2769530},
	{SG37216, &UA2753699},
	{SG37220, &UA2769794},
	{SG37223, &UA2782626},
	{SG37225, &UA2783171},
	{SG37228, &UA2783354},
	{SG37229, &UA2783835},
	{SG37230, &UA2784533},
	{SG37232, &UA2800086},
	{SG37233, &UA2800676},
	{SG37234, &UA2867408},
	{SG37235, &UA2879482},
	{SG37236, &UA2893293},
	{SG37237, &UA2938521},
	{SG37243, &UA2532103},
	{SG37253, &UA2676338},
	{SG37257, &UA3065457},
	{SG37258, &UA1996989},
	{SG37259, &UA2012507},
	{SG37260, &UA2205041},
	{SG37261, &UA2032120},
	{SG37262, &UA2033263},
	{SG37269, &UA2405462},
	{SG37270, &UA2654951},
	{SG37273, &UA2676017},
	{SG37292, &UA2367501},
	{SG37293, &UA2511351},
	{SG37294, &UA2844015},
	{SG37295, &UA3006758},
	{SG37307, &UA2862518},
	{SG37340, &UA3065248},
	{SG37342, &UA3067920},
	{SG37343, &UA3068062},
	{SG37344, &UA3081110},
	{SG37345, &UA3126472},
	{SG37346, &UA2425831},
	{SG37347, &UA2780623},
	{SG37348, &UA2973959},
	{SG37349, &UA2974470},
	{SG37351, &UA2568103},
	{SG37352, &UA2858515},
	{SG37354, &UA3108451},
	{SG37355, &UA3125964},
	{SG37366, &UA3067061},
	{SG37367, &UA3067289},
	{SG37368, &UA2008897},
	{SG37370, &UA2403454},
	{SG37371, &UA2535410},
	{SG37372, &UA2752576},
	{SG37373, &UA2855389},
	{SG37374, &UA3086811},
	{SG37399, &UA2665589},
	{SG37440, &UA2367747},
	{SG37441, &UA2603507},
	{SG37442, &UA2655498},
	{SG37446, &UA2773345},
	{SG37451, &UA2586574},
	{SG37452, &UA2766110},
	{SG37489, &UA2033044},
	{SG37495, &UA2513535},
	{SG37496, &UA2514085},
	{SG37499, &UA2974162},
	{SG37500, &UA2974448},
	{SG37501, &UA3021161},
	{SG37502, &UA2737627},
	{SG37503, &UA2867971},
	{SG37504, &UA2566892},
	{SG37518, &UA2974379},
	{SG37520, &UA2974918},
	{SG37526, &UA2583340},
	{SG37528, &UA2584416},
	{SG37529, &UA2602012},
	{SG37530, &UA2603478},
	{SG37531, &UA2603940},
	{SG37532, &UA2667430},
	{SG37533, &UA2667834},
	{SG37536, &UA2702482},
	{SG37537, &UA2702969},
	{SG37538, &UA2733055},
	{SG37539, &UA2766309},
	{SG37541, &UA2865276},
	{SG37543, &UA2891038},
	{SG37544, &UA2989053},
	{SG37545, &UA2989660},
	{SG37546, &UA2990234},
	{SG37569, &UA2537054},
	{SG37571, &UA2855436},
	{SG37572, &UA3020079},
	{SG37573, &UA3107931},
	{SG37578, &UA2567247},
	{SG37579, &UA2403401},
	{SG37581, &UA2406019},
	{SG37584, &UA2753764},
	{SG37588, &UA2997494},
	{SG37596, &UA2011374},
	{SG37617, &UA2677083},
	{SG37618, &UA2706896},
	{SG37619, &UA2708558},
	{SG37622, &UA2735924},
	{SG37625, &UA2780776},
	{SG37626, &UA2780942},
	{SG37627, &UA2860928},
	{SG37629, &UA2863874},
	{SG37631, &UA2958810},
	{SG37632, &UA3003822},
	{SG37633, &UA3006564},
	{SG37634, &UA3007537},
	{SG37636, &UA3015399},
	{SG37637, &UA3018363},
	{SG37638, &UA3021932},
	{SG37639, &UA3028126},
	{SG37640, &UA3056929},
	{SG37641, &UA3059487},
	{SG37642, &UA3092197},
	{SG37643, &UA3092469},
	{SG37644, &UA3108501},
	{SG37646, &UA3125151},
	{SG37649, &UA2752950},
	{SG37650, &UA2938584},
	{SG37654, &UA2008881},
	{SG37655, &UA2200321},
	{SG37657, &UA2030427},
	{SG37658, &UA2041597},
	{SG37659, &UA2352437},
	{SG37661, &UA3020790},
	{SG37662, &UA3020796},
	{SG37663, &UA3021067},
	{SG37664, &UA3021100},
	{SG37665, &UA3021493},
	{SG37666, &UA3021549},
	{SG37667, &UA3021782},
	{SG37668, &UA3021819},
	{SG37670, &UA3022188},
	{SG37671, &UA3022362},
	{SG37672, &UA3022342},
	{SG37673, &UA3022413},
	{SG37674, &UA3022549},
	{SG37675, &UA3022552},
	{SG37676, &UA3022630},
	{SG37678, &UA3022637},
	{SG37681, &UA3022703},
	{SG37683, &UA3022855},
	{SG37685, &UA3022905},
	{SG37686, &UA3023125},
	{SG37688, &UA3023691},
	{SG37690, &UA3023771},
	{SG37693, &UA3023922},
	{SG37695, &UA3024145},
	{SG37735, &UA3078499},
	{SG37736, &UA3107723},
	{SG37737, &UA3107742},
	{SG37738, &UA3127818},
	{SG37743, &UA2200872},
	{SG37751, &UA1996416},
	{SG37752, &UA2002681},
	{SG37753, &UA2201778},
	{SG37754, &UA2597536},
	{SG37755, &UA2657774},
	{SG37771, &UA2698324},
	{SG37802, &UA2228686},
	{SG37803, &UA2650419},
	{SG37805, &UA2788417},
	{SG37807, &UA2676175},
	{SG37808, &UA2754585},
	{SG37822, &UA2974035},
	{SG37833, &UA2384847},
	{SG37834, &UA2654598},
	{SG37838, &UA2008064},
	{SG37839, &UA2199795},
	{SG37846, &UA2016141},
	{SG37847, &UA2029567},
	{SG37848, &UA2200621},
	{SG37876, &UA2705175},
	{SG37878, &UA2878881},
	{SG37898, &UA2674210},
	{SG37900, &UA2676737},
	{SG37901, &UA2677005},
	{SG37902, &UA2706647},
	{SG37905, &UA2707486},
	{SG37907, &UA2713946},
	{SG37909, &UA2734747},
	{SG37911, &UA2734896},
	{SG37913, &UA2735785},
	{SG37915, &UA2736087},
	{SG37918, &UA2751767},
	{SG37919, &UA2860997},
	{SG37920, &UA2865643},
	{SG37921, &UA2993358},
	{SG37922, &UA2459257},
	{SG37923, &UA2484919},
	{SG37924, &UA2598096},
	{SG37926, &UA2719601},
	{SG37927, &UA2011710},
	{SG37928, &UA2028628},
	{SG37929, &UA2203953},
	{SG37930, &UA2468892},
	{SG37931, &UA2778692},
	{SG37951, &UA913368},
	{SG37953, &UA2044111},
	{SG37955, &UA3031128},
	{SG37967, &UA2585479},
	{SG37988, &UA2425848},
	{SG37989, &UA2556618},
	{SG37990, &UA2796171},
	{SG37996, &UA2404173},
	{SG38005, &UA3033508},
	{SG38020, &UA2753056},
	{SG38021, &UA2753147},
	{SG38022, &UA2754066},
	{SG38023, &UA2671108},
	{SG38024, &UA2676270},
	{SG38025, &UA2676331},
	{SG38026, &UA2566451},
	{SG38028, &UA2566966},
	{SG38029, &UA2974018},
	{SG38037, &UA2673349},
	{SG38039, &UA2676591},
	{SG38041, &UA3112644},
	{SG38046, &UA2367233},
	{SG38047, &UA2727984},
	{SG38061, &UA2437307},
	{SG38062, &UA2597311},
	{SG38063, &UA2986177},
	{SG38064, &UA2430623},
	{SG38065, &UA2697239},
	{SG38234, &UA2676056},
	{SG38236, &UA2667358},
	{SG38238, &UA2234845},
	{SG38239, &UA2552695},
	{SG38249, &UA1995940},
	{SG38250, &UA2030832},
	{SG38257, &UA2179402},
	{SG38258, &UA2198248},
	{SG38262, &UA2586578},
	{SG38266, &UA2957859},
	{SG38268, &UA3092384},
	{SG38269, &UA3065461},
	{SG38279, &UA3067908},
	{SG38305, &UA2973892},
	{SG38318, &UA2567587},
	{SG38320, &UA2752965},
	{SG38322, &UA2755517},
	{SG38353, &UA2536073},
	{SG38355, &UA2853831},
	{SG38361, &UA2425163},
	{SG38363, &UA2656695},
	{SG38365, &UA2406289},
	{SG38366, &UA2509145},
	{SG38367, &UA2983844},
	{SG38397, &UA2752973},
	{SG38399, &UA2754586},
	{SG38412, &UA2700426},
	{SG38417, &UA2365490},
	{SG38419, &UA2466863},
	{SG38432, &UA1996811},
	{SG38465, &UA2846758},
	{SG38468, &UA2942530},
	{SG38470, &UA2353415},
	{SG38471, &UA2465745},
	{SG38473, &UA2528698},
	{SG38474, &UA2778574},
	{SG38475, &UA3044154},
	{SG38476, &UA3070256},
	{SG38478, &UA2976107},
	{SG38480, &UA2011111},
	{SG38485, &UA2008453},
	{SG38486, &UA2199799},
	{SG38496, &UA2487940},
	{SG38497, &UA2738247},
	{SG38498, &UA2988438},
	{SG38499, &UA2753883},
	{SG38506, &UA2462379},
	{SG38528, &UA2755118},
	{SG38553, &UA2773712},
	{SG38564, &UA2028609},
	{SG38565, &UA2029811},
	{SG38567, &UA2029960},
	{SG38608, &UA1996039},
	{SG38609, &UA2014323},
	{SG38611, &UA2258137},
	{SG38630, &UA2973887},
	{SG38631, &UA2974374},
	{SG38632, &UA2974460},
	{SG38633, &UA2975698},
	{SG38634, &UA3023113},
	{SG38635, &UA2032475},
	{SG38638, &UA2234846},
	{SG38653, &UA2204783},
	{SG38655, &UA3065091},
	{SG38693, &UA2719154},
	{SG38694, &UA2935902},
	{SG38696, &UA2029762},
	{SG38697, &UA2032485},
	{SG38699, &UA2033035},
	{SG38701, &UA2235600},
	{SG38705, &UA2945033},
	{SG38715, &UA2753277},
	{SG38717, &UA2754579},
	{SG38719, &UA2782538},
	{SG38720, &UA2892941},
	{SG38731, &UA3065087},
	{SG38733, &UA2769619},
	{SG38735, &UA2784002},
	{SG38737, &UA2798861},
	{SG38738, &UA2798897},
	{SG38740, &UA2799092},
	{SG38743, &UA2866911},
	{SG38745, &UA2867440},
	{SG38746, &UA2869118},
	{SG38748, &UA2872917},
	{SG38750, &UA2877856},
	{SG38752, &UA2877862},
	{SG38755, &UA2893066},
	{SG38759, &UA2893439},
	{SG38761, &UA2893562},
	{SG38763, &UA2937726},
	{SG38765, &UA2937790},
	{SG38767, &UA2938130},
	{SG38768, &UA2999353},
	{SG38769, &UA2999790},
	{SG38770, &UA3058281},
	{SG38772, &UA2177019},
	{SG38773, &UA2445962},
	{SG38825, &UA2553486},
	{SG38826, &UA2950818},
	{SG38827, &UA3086335},
	{SG38835, &UA2878934},
	{SG38845, &UA2563812},
	{SG38847, &UA2901100},
	{SG38849, &UA2961965},
	{SG38850, &UA3001656},
	{SG38860, &UA2973663},
	{SG38862, &UA3020569},
	{SG38879, &UA2596252},
	{SG38882, &UA2660155},
	{SG38955, &UA2702691},
	{SG38978, &UA1996043},
	{SG38980, &UA2042167},
	{SG38983, &UA2201508},
	{SG38985, &UA2963928},
	{SG38986, &UA2974031},
	{SG39002, &UA2676681},
	{SG39019, &UA2507714},
	{SG39020, &UA2764610},
	{SG39036, &UA2456876},
	{SG39037, &UA2762059},
	{SG39052, &UA2429194},
	{SG39053, &UA2659432},
	{SG39055, &UA2696214},
	{SG39056, &UA2699445},
	{SG39069, &UA3037793},
	{SG39070, &UA3083261},
	{SG39155, &UA2737046},
	{SG39156, &UA2737579},
	{SG39157, &UA2752549},
	{SG39158, &UA2752814},
	{SG39159, &UA2964012},
	{SG39180, &UA2436564},
	{SG39185, &UA2701909},
	{SG39186, &UA2729477},
	{SG39187, &UA2676172},
	{SG39188, &UA2676353},
	{SG39191, &UA2603868},
	{SG39193, &UA2994062},
	{SG39195, &UA1997248},
	{SG39196, &UA2028612},
	{SG39197, &UA2552697},
	{SG39205, &UA2676985},
	{SG39209, &UA2974156},
	{SG39214, &UA924829},
	{SG39215, &UA1996423},
	{SG39216, &UA1999203},
	{SG39217, &UA2611030},
	{SG39218, &UA2612835},
	{SG39219, &UA2614136},
	{SG39220, &UA2614254},
	{SG39221, &UA2616694},
	{SG39222, &UA2616776},
	{SG39224, &UA2616778},
	{SG39225, &UA2616837},
	{SG39226, &UA2622409},
	{SG39241, &UA2676582},
	{SG39242, &UA2767225},
	{SG39244, &UA2369690},
	{SG39253, &UA2676193},
	{SG39254, &UA2676755},
	{SG39271, &UA2342611},
	{SG39272, &UA2695764},
	{SG39279, &UA2229913},
	{SG39281, &UA2502861},
	{SG39282, &UA2985832},
	{SG39301, &UA2567016},
	{SG39302, &UA2583270},
	{SG39315, &UA2879666},
	{SG39321, &UA2670752},
	{SG39322, &UA2676252},
	{SG39332, &UA2755348},
	{SG39333, &UA2867679},
	{SG39335, &UA3021781},
	{SG39338, &UA2975034},
	{SG39342, &UA2182343},
	{SG39345, &UA2469214},
	{SG39348, &UA2506807},
	{SG39350, &UA2656575},
	{SG39352, &UA2839712},
	{SG39353, &UA2889584},
	{SG39354, &UA2938225},
	{SG39359, &UA2506418},
	{SG39360, &UA2558217},
	{SG39361, &UA2559257},
	{SG39364, &UA2566687},
	{SG39368, &UA3109472},
	{SG39382, &UA2944159},
	{SG39388, &UA2030811},
	{SG39389, &UA2676185},
	{SG39390, &UA2714701},
	{SG39391, &UA2879895},
	{SG39392, &UA3014252},
	{SG39396, &UA3080707},
	{SG39397, &UA3094747},
	{SG39403, &UA3031340},
	{SG39406, &UA2462488},
	{SG39408, &UA1995471},
	{SG39410, &UA2030079},
	{SG39411, &UA2032021},
	{SG39412, &UA2236063},
	{SG39413, &UA2237160},
	{SG39417, &UA3030840},
	{SG39433, &UA2754950},
	{SG39436, &UA2937351},
	{SG39446, &UA2667736},
	{SG39448, &UA2856376},
	{SG39450, &UA2566747},
	{SG39451, &UA2567116},
	{SG39452, &UA2570661},
	{SG39453, &UA2773261},
	{SG39454, &UA2675405},
	{SG39456, &UA3031710},
	{SG39459, &UA3033505},
	{SG39460, &UA3035727},
	{SG39461, &UA3037092},
	{SG39462, &UA3059458},
	{SG39474, &UA2973909},
	{SG39476, &UA1997032},
	{SG39477, &UA2012754},
	{SG39480, &UA2200854},
	{SG39483, &UA2489575},
	{SG39485, &UA2753403},
	{SG39500, &UA2974635},
	{SG39502, &UA3021095},
	{SG39504, &UA3125037},
	{SG39512, &UA3094277},
	{SG39513, &UA2712770},
	{SG39514, &UA2994631},
	{SG39527, &UA2534933},
	{SG39528, &UA2666979},
	{SG39529, &UA2734115},
	{SG39530, &UA2879604},
	{SG39531, &UA3030380},
	{SG39532, &UA3108042},
	{SG39533, &UA2566974},
	{SG39534, &UA2668437},
	{SG39544, &UA2532720},
	{SG39545, &UA2604139},
	{SG39546, &UA2858719},
	{SG39548, &UA2773725},
	{SG39556, &UA2513768},
	{SG39558, &UA2672737},
	{SG39561, &UA2712901},
	{SG39562, &UA2508651},
	{SG39563, &UA2857022},
	{SG39564, &UA3064981},
	{SG39593, &UA2538000},
	{SG39594, &UA2558490},
	{SG39597, &UA2730731},
	{SG39608, &UA2871477},
	{SG39609, &UA2939523},
	{SG39612, &UA2601408},
	{SG39614, &UA2766567},
	{SG39626, &UA2029057},
	{SG39629, &UA2236212},
	{SG39633, &UA2353938},
	{SG39644, &UA2564446},
	{SG39647, &UA2996760},
	{SG39649, &UA3086835},
	{SG39650, &UA3088684},
	{SG39658, &UA2678919},
	{SG39659, &UA2679778},
	{SG39662, &UA3065077},
	{SG39663, &UA3094071},
	{SG39694, &UA2974291},
	{SG39701, &UA3030594},
	{SG39710, &UA2707549},
	{SG39711, &UA2995269},
	{SG39742, &UA2425798},
	{SG39745, &UA2483094},
	{SG39748, &UA2483309},
	{SG39778, &UA2354278},
	{SG39780, &UA3043446},
	{SG39781, &UA2224148},
	{SG39785, &UA2529307},
	{SG39787, &UA2669763},
	{SG39797, &UA2420557},
	{SG39798, &UA2460967},
	{SG39799, &UA2738720},
	{SG39800, &UA2739993},
	{SG39801, &UA2755003},
	{SG39802, &UA2782013},
	{SG39803, &UA2937407},
	{SG39818, &UA2560408},
	{SG39819, &UA2935134},
	{SG39832, &UA2469369},
	{SG39834, &UA2486555},
	{SG39842, &UA2842549},
	{SG39855, &UA2461173},
	{SG39857, &UA2765379},
	{SG39864, &UA2753470},
	{SG39868, &UA2562101},
	{SG39869, &UA2863764},
	{SG39881, &UA2353529},
	{SG39886, &UA2709884},
	{SG39888, &UA2892005},
	{SG39912, &UA1997947},
	{SG39914, &UA2001056},
	{SG39917, &UA2239152},
	{SG39918, &UA2485940},
	{SG39919, &UA2670293},
	{SG39920, &UA2750623},
	{SG39921, &UA2532958},
	{SG39931, &UA2468808},
	{SG39932, &UA2762885},
	{SG39934, &UA3065459},
	{SG39935, &UA3094093},
	{SG39936, &UA2679684},
	{SG39937, &UA2708923},
	{SG39939, &UA2699112},
	{SG39946, &UA3080986},
	{SG39969, &UA2566542},
	{SG40003, &UA2505226},
	{SG40005, &UA2599126},
	{SG40031, &UA2975013},
	{SG40032, &UA3022367},
	{SG40037, &UA2535307},
	{SG40038, &UA2779621},
	{SG40039, &UA2994665},
	{SG40048, &UA2885339},
	{SG40051, &UA3126185},
	{SG40064, &UA2181300},
	{SG40066, &UA2258022},
	{SG40074, &UA2560997},
	{SG40076, &UA2660448},
	{SG40080, &UA2850842},
	{SG40087, &UA2755133},
	{SG40095, &UA2405960},
	{SG40097, &UA2422546},
	{SG40103, &UA2202576},
	{SG40110, &UA2676756},
	{SG40111, &UA2677007},
	{SG40127, &UA2485545},
	{SG40129, &UA2703135},
	{SG40136, &UA2232040},
	{SG40137, &UA3043313},
	{SG40138, &UA2490755},
	{SG40139, &UA2850683},
	{SG40140, &UA2974733},
	{SG40141, &UA3021435},
	{SG40149, &UA2005128},
	{SG40150, &UA2012037},
	{SG40151, &UA2201260},
	{SG40152, &UA2564441},
	{SG40153, &UA3068192},
	{SG40154, &UA2669594},
	{SG40155, &UA3065600},
	{SG40156, &UA2734828},
	{SG40157, &UA3088172},
	{SG40159, &UA3067175},
	{SG40160, &UA3067656},
	{SG40176, &UA2659337},
	{SG40177, &UA2844272},
	{SG40178, &UA2896307},
	{SG40179, &UA2985984},
	{SG40184, &UA2783699},
	{SG40186, &UA2015814},
	{SG40187, &UA2029015},
	{SG40188, &UA2031364},
	{SG40191, &UA2437368},
	{SG40192, &UA2512588},
	{SG40193, &UA2935084},
	{SG40198, &UA2784791},
	{SG40199, &UA2753069},
	{SG40201, &UA3084114},
	{SG40205, &UA3115558},
	{SG40210, &UA2729306},
	{SG40211, &UA3087210},
	{SG40213, &UA3100047},
	{SG40219, &UA1994699},
	{SG40220, &UA1997040},
	{SG40222, &UA2012718},
	{SG40224, &UA2028041},
	{SG40227, &UA2584205},
	{SG40229, &UA2707192},
	{SG40232, &UA2707498},
	{SG40233, &UA3102497},
	{SG40234, &UA2028752},
	{SG40235, &UA2029964},
	{SG40237, &UA2199628},
	{SG40239, &UA2206133},
	{SG40245, &UA2531150},
	{SG40249, &UA2754492},
	{SG40252, &UA2956647},
	{SG40257, &UA1998122},
	{SG40276, &UA3080506},
	{SG40280, &UA2469260},
	{SG40289, &UA2754274},
	{SG40290, &UA2877786},
	{SG40299, &UA2565827},
	{SG40300, &UA2935219},
	{SG40314, &UA2666131},
	{SG40315, &UA2937098},
	{SG40344, &UA2755128},
	{SG40346, &UA2773915},
	{SG40349, &UA2974307},
	{SG40350, &UA2974870},
	{SG40359, &UA2753274},
	{SG40360, &UA2957863},
	{SG40367, &UA2433381},
	{SG40372, &UA2426160},
	{SG40373, &UA2747301},
	{SG40380, &UA3031262},
	{SG40382, &UA3031336},
	{SG40385, &UA2425310},
	{SG40387, &UA2753440},
	{SG40390, &UA2567140},
	{SG40391, &UA2858562},
	{SG40416, &UA2458579},
	{SG40418, &UA2458783},
	{SG40419, &UA2459447},
	{SG40422, &UA2537992},
	{SG40423, &UA2670065},
	{SG40424, &UA2766362},
	{SG40425, &UA2847605},
	{SG40427, &UA2850069},
	{SG40429, &UA2961132},
	{SG40431, &UA3092586},
	{SG40433, &UA3125232},
	{SG40440, &UA2885443},
	{SG40443, &UA2896173},
	{SG40475, &UA2897689},
	{SG40476, &UA2943976},
	{SG40477, &UA2961248},
	{SG40487, &UA2004939},
	{SG40488, &UA2570750},
	{SG40489, &UA2992248},
	{SG40491, &UA2869023},
	{SG40492, &UA3107575},
	{SG40511, &UA3061620},
	{SG40517, &UA3093738},
	{SG40518, &UA2798399},
	{SG40519, &UA2872219},
	{SG40520, &UA2879050},
	{SG40521, &UA2799372},
	{SG40533, &UA3109345},
	{SG40543, &UA2897275},
	{SG40545, &UA2965504},
	{SG40555, &UA2421663},
	{SG40557, &UA2841166},
	{SG40559, &UA2985101},
	{SG40580, &UA3068199},
	{SG40592, &UA2800203},
	{SG40593, &UA3121459},
	{SG40613, &UA2383422},
	{SG40614, &UA2400562},
	{SG40616, &UA2457860},
	{SG40617, &UA2466446},
	{SG40620, &UA2466697},
	{SG40625, &UA2203972},
	{SG40628, &UA2234836},
	{SG40632, &UA2561201},
	{SG40633, &UA2580812},
	{SG40634, &UA2792153},
	{SG40637, &UA2566615},
	{SG40649, &UA2483321},
	{SG40651, &UA2845864},
	{SG40654, &UA3043980},
	{SG40664, &UA2032712},
	{SG40667, &UA2339497},
	{SG40692, &UA3067185},
	{SG40695, &UA2427359},
	{SG40696, &UA2433296},
	{SG40699, &UA2753881},
	{SG40724, &UA2749919},
	{SG40726, &UA3001037},
	{SG40727, &UA3072905},
	{SG40730, &UA2506112},
	{SG40731, &UA2746450},
	{SG40732, &UA2799841},
	{SG40734, &UA2934721},
	{SG40738, &UA2754493},
	{SG40739, &UA2755409},
	{SG40741, &UA2755419},
	{SG40742, &UA2939423},
	{SG40743, &UA2997767},
	{SG40828, &UA2261539},
	{SG40858, &UA2433078},
	{SG40898, &UA2676449},
	{SG40910, &UA2530266},
	{SG40911, &UA3071406},
	{SG40913, &UA2420871},
	{SG40915, &UA2480298},
	{SG40916, &UA2615976},
	{SG40933, &UA2433177},
	{SG40951, &UA2531009},
	{SG40953, &UA2952406},
	{SG40966, &UA2554445},
	{SG40968, &UA2890779},
	{SG40983, &UA2529335},
	{SG40984, &UA2599821},
	{SG41004, &UA2532582},
	{SG41005, &UA2779100},
	{SG41006, &UA2987311},
	{SG41023, &UA2557670},
	{SG41031, &UA2668261},
	{SG41039, &UA2567382},
	{SG41040, &UA1999916},
	{SG41042, &UA2008837},
	{SG41044, &UA2201492},
	{SG41048, &UA2738448},
	{SG41051, &UA3103754},
	{SG41055, &UA2483202},
	{SG41056, &UA2595968},
	{SG41057, &UA2728584},
	{SG41059, &UA2890075},
	{SG41069, &UA2563719},
	{SG41080, &UA2774040},
	{SG41081, &UA2859510},
	{SG41083, &UA2869113},
	{SG41088, &UA2387271},
	{SG41089, &UA2485623},
	{SG41090, &UA2426391},
	{SG41099, &UA2770762},
	{SG41100, &UA3049470},
	{SG41110, &UA2876916},
	{SG41112, &UA3106865},
	{SG41115, &UA2754580},
	{SG41139, &UA2585274},
	{SG41140, &UA2668997},
	{SG41147, &UA2365880},
	{SG41148, &UA2383377},
	{SG41152, &UA3068266},
	{SG41153, &UA2203958},
	{SG41171, &UA2486797},
	{SG41174, &UA2939579},
	{SG41180, &UA2467980},
	{SG41182, &UA2781741},
	{SG41183, &UA2878949},
	{SG41184, &UA2956690},
	{SG41185, &UA2995531},
	{SG41186, &UA3048935},
	{SG41188, &UA3104452},
	{SG41215, &UA3081111},
	{SG41218, &UA2633549},
	{SG41219, &UA2635414},
	{SG41221, &UA2783706},
	{SG41232, &UA2760290},
	{SG41249, &UA2676993},
	{SG41252, &UA2466473},
	{SG41253, &UA2469878},
	{SG41254, &UA2701415},
	{SG41260, &UA2699061},
	{SG41261, &UA2952916},
	{SG41262, &UA2999325},
	{SG41265, &UA2580312},
	{SG41266, &UA2855165},
	{SG41267, &UA2778923},
	{SG41269, &UA2865619},
	{SG41272, &UA2562397},
	{SG41274, &UA3071926},
	{SG41276, &UA2577964},
	{SG41279, &UA2565404},
	{SG41282, &UA2707348},
	{SG41290, &UA2779675},
	{SG41291, &UA2988112},
	{SG41294, &UA2991373},
	{SG41296, &UA3024936},
	{SG41298, &UA3058341},
	{SG41299, &UA3076926},
	{SG41320, &UA2794887},
	{SG41327, &UA2515532},
	{SG41331, &UA3023699},
	{SG41333, &UA2974622},
	{SG41337, &UA2387364},
	{SG41338, &UA2419216},
	{SG41340, &UA2451570},
	{SG41359, &UA2633571},
	{SG41360, &UA2773097},
	{SG41366, &UA2556249},
	{SG41368, &UA2737464},
	{SG41369, &UA3079381},
	{SG41372, &UA2200378},
	{SG41423, &UA2508035},
	{SG41424, &UA2511263},
	{SG41435, &UA2422587},
	{SG41442, &UA2738446},
	{SG41448, &UA2042668},
	{SG41449, &UA2236210},
	{SG41452, &UA2974599},
	{SG41453, &UA2611869},
	{SG41454, &UA2612754},
	{SG41455, &UA2612806},
	{SG41456, &UA2615533},
	{SG41467, &UA2974092},
	{SG41472, &UA2205031},
	{SG41473, &UA2615471},
	{SG41474, &UA2615648},
	{SG41475, &UA2616144},
	{SG41476, &UA2598059},
	{SG41484, &UA3065000},
	{SG41486, &UA2507423},
	{SG41487, &UA2663075},
	{SG41503, &UA2511997},
	{SG41508, &UA2669598},
	{SG41510, &UA2748507},
	{SG41512, &UA2937080},
	{SG41532, &UA2030240},
	{SG41533, &UA2032717},
	{SG41534, &UA2346881},
	{SG41537, &UA2382920},
	{SG41540, &UA2753204},
	{SG41541, &UA2937628},
	{SG41543, &UA2232871},
	{SG41553, &UA2795974},
	{SG41555, &UA2999660},
	{SG41557, &UA3050347},
	{SG41564, &UA2670729},
	{SG41581, &UA3094199},
	{SG41584, &UA2460050},
	{SG41585, &UA2655924},
	{SG41587, &UA3099981},
	{SG41594, &UA1996830},
	{SG41596, &UA2383541},
	{SG41600, &UA2692682},
	{SG41608, &UA3022472},
	{SG41614, &UA1996023},
	{SG41615, &UA2029007},
	{SG41617, &UA2033049},
	{SG41618, &UA2199026},
	{SG41619, &UA2524885},
	{SG41668, &UA3131530},
	{SG41672, &UA2032025},
	{SG41677, &UA2774287},
	{SG41696, &UA2487257},
	{SG41701, &UA2604437},
	{SG41718, &UA2702057},
	{SG41719, &UA2985150},
	{SG41725, &UA2459264},
	{SG41728, &UA2699032},
	{SG41733, &UA2984869},
	{SG41735, &UA2461748},
	{SG41737, &UA2570638},
	{SG41738, &UA2987971},
	{SG41739, &UA2871099},
	{SG41741, &UA2999601},
	{SG41742, &UA2603917},
	{SG41745, &UA2707101},
	{SG41746, &UA2866275},
	{SG41748, &UA2935327},
	{SG41749, &UA2990102},
	{SG41750, &UA2994120},
	{SG41751, &UA2998762},
	{SG41754, &UA3108616},
	{SG41766, &UA2734861},
	{SG41769, &UA2559971},
	{SG41770, &UA2765716},
	{SG41785, &UA2507934},
	{SG41787, &UA2869235},
	{SG41788, &UA2560806},
	{SG41789, &UA2800309},
	{SG41790, &UA2855902},
	{SG41798, &UA2753850},
	{SG41802, &UA3131095},
	{SG41803, &UA3131080},
	{SG41804, &UA3132923},
	{SG41813, &UA2203280},
	{SG41814, &UA2241300},
	{SG41816, &UA2241304},
	{SG41819, &UA2426564},
	{SG41820, &UA2963733},
	{SG41821, &UA2753006},
	{SG41822, &UA2879497},
	{SG41823, &UA2754346},
	{SG41824, &UA2755129},
	{SG41833, &UA2006091},
	{SG41857, &UA2865050},
	{SG41859, &UA3028337},
	{SG41861, &UA3065583},
	{SG41866, &UA2536303},
	{SG41867, &UA2952643},
	{SG41874, &UA2738364},
	{SG41875, &UA2676254},
	{SG41876, &UA2676757},
	{SG41879, &UA2734449},
	{SG41896, &UA2582804},
	{SG41919, &UA2738010},
	{SG41922, &UA3067640},
	{SG41938, &UA2234839},
	{SG41945, &UA2422336},
	{SG41948, &UA2557666},
	{SG41950, &UA2986539},
	{SG41955, &UA2681009},
	{SG41962, &UA2580307},
	{SG41965, &UA2709060},
	{SG41969, &UA2557786},
	{SG41994, &UA2663174},
	{SG41996, &UA2856719},
	{SG41997, &UA3047355},
	{SG42011, &UA2677175},
	{SG42067, &UA2596441},
	{SG42069, &UA2764370},
	{SG42071, &UA2851953},
	{SG42077, &UA2029367},
	{SG42079, &UA2204724},
	{SG42103, &UA2676187},
	{SG42104, &UA2755684},
	{SG42105, &UA2974388},
	{SG42164, &UA2431071},
	{SG42166, &UA2486116},
	{SG42169, &UA1996984},
	{SG42171, &UA2599834},
	{SG42172, &UA3068057},
	{SG42187, &UA2582299},
	{SG42189, &UA2600340},
	{SG42190, &UA2989161},
	{SG42194, &UA2429763},
	{SG42202, &UA2878853},
	{SG42203, &UA3124599},
	{SG42207, &UA2230659},
	{SG42213, &UA2566754},
	{SG42215, &UA2602285},
	{SG42218, &UA2032478},
	{SG42220, &UA2648679},
	{SG42222, &UA2535133},
	{SG42248, &UA3045431},
	{SG42250, &UA3046568},
	{SG42251, &UA2868516},
	{SG42261, &UA1997508},
	{SG42262, &UA2193407},
	{SG42265, &UA2362411},
	{SG42268, &UA2509031},
	{SG42323, &UA2436190},
	{SG42325, &UA2460935},
	{SG42327, &UA2740519},
	{SG42328, &UA2869249},
	{SG42333, &UA2033104},
	{SG42335, &UA2228410},
	{SG42362, &UA2510707},
	{SG42364, &UA2714037},
	{SG42366, &UA2716790},
	{SG42371, &UA2603721},
	{SG42376, &UA3067058},
	{SG42379, &UA2620703},
	{SG42380, &UA2620704},
	{SG42381, &UA2637508},
	{SG42388, &UA2469079},
	{SG42399, &UA2479944},
	{SG42400, &UA2504554},
	{SG42401, &UA2659345},
	{SG42402, &UA2728178},
	{SG42403, &UA2995696},
	{SG42408, &UA2732133},
	{SG42411, &UA2795275},
	{SG42433, &UA2458865},
	{SG42446, &UA2612524},
	{SG42448, &UA2614019},
	{SG42452, &UA2615200},
	{SG42453, &UA2615970},
	{SG42455, &UA2626175},
	{SG42456, &UA2630485},
	{SG42457, &UA2676026},
	{SG42462, &UA3030823},
	{SG42477, &UA2029763},
	{SG42479, &UA2611173},
	{SG42481, &UA2611416},
	{SG42482, &UA2612539},
	{SG42484, &UA2616233},
	{SG42485, &UA2618667},
	{SG42486, &UA2341863},
	{SG42487, &UA2614252},
	{SG42488, &UA2423064},
	{SG42489, &UA2754338},
	{SG42497, &UA2868718},
	{SG42498, &UA2957388},
	{SG42499, &UA3074339},
	{SG42500, &UA2511509},
	{SG42503, &UA2749569},
	{SG42504, &UA2736628},
	{SG42505, &UA2957387},
	{SG42510, &UA2183971},
	{SG42511, &UA2258054},
	{SG42512, &UA2576925},
	{SG42513, &UA2558939},
	{SG42515, &UA2376944},
	{SG42519, &UA2612227},
	{SG42520, &UA2615961},
	{SG42521, &UA2632863},
	{SG42528, &UA2974914},
	{SG42562, &UA2676593},
	{SG42565, &UA2767449},
	{SG42567, &UA2992251},
	{SG42578, &UA2567260},
	{SG42596, &UA2665527},
	{SG42597, &UA3018934},
	{SG42609, &UA2029165},
	{SG42614, &UA2671147},
	{SG42617, &UA2767753},
	{SG42626, &UA2170990},
	{SG42627, &UA2421173},
	{SG42628, &UA2892338},
	{SG42631, &UA3068418},
	{SG42632, &UA3083207},
	{SG42637, &UA2754246},
	{SG42638, &UA2781341},
	{SG42639, &UA2937598},
	{SG42646, &UA2610953},
	{SG42648, &UA2615487},
	{SG42650, &UA2615535},
	{SG42651, &UA2617768},
	{SG42652, &UA2625697},
	{SG42653, &UA2634250},
	{SG42659, &UA2424261},
	{SG42660, &UA2429417},
	{SG42661, &UA2505694},
	{SG42662, &UA2984341},
	{SG42663, &UA2557673},
	{SG42682, &UA2600903},
	{SG42684, &UA2230863},
	{SG42686, &UA2676263},
	{SG42694, &UA2424416},
	{SG42696, &UA2752231},
	{SG42698, &UA3030525},
	{SG42701, &UA2793354},
	{SG42703, &UA2997617},
	{SG42722, &UA2029181},
	{SG42723, &UA2031905},
	{SG42724, &UA2239099},
	{SG42725, &UA2773123},
	{SG42726, &UA2879570},
	{SG42727, &UA3051554},
	{SG42728, &UA3021090},
	{SG42729, &UA3067902},
	{SG42734, &UA2869473},
	{SG42751, &UA2965043},
	{SG42766, &UA2974093},
	{SG42771, &UA2676982},
	{SG42773, &UA2707241},
	{SG42776, &UA2736490},
	{SG42777, &UA2995080},
	{SG42778, &UA2028622},
	{SG42779, &UA2257993},
	{SG42780, &UA2655068},
	{SG42782, &UA2731637},
	{SG42783, &UA2760130},
	{SG42784, &UA2509303},
	{SG42819, &UA2733531},
	{SG42824, &UA2676290},
	{SG42825, &UA2993634},
	{SG42828, &UA2878601},
	{SG42829, &UA2952766},
	{SG42832, &UA3074271},
	{SG42836, &UA3039502},
	{SG42842, &UA2671146},
	{SG42844, &UA2671479},
	{SG42845, &UA2709487},
	{SG42846, &UA2992784},
	{SG42849, &UA2892538},
	{SG42850, &UA3013529},
	{SG42853, &UA2028660},
	{SG42855, &UA2031408},
	{SG42859, &UA2612371},
	{SG42861, &UA2614756},
	{SG42864, &UA2672775},
	{SG42875, &UA2768098},
	{SG42892, &UA2601909},
	{SG42893, &UA2729640},
	{SG42896, &UA2974901},
	{SG42897, &UA2974593},
	{SG42909, &UA3067585},
	{SG42928, &UA2437423},
	{SG42939, &UA2206703},
	{SG42941, &UA2558114},
	{SG42946, &UA2486390},
	{SG42947, &UA2749497},
	{SG42948, &UA2714613},
	{SG42949, &UA3120228},
	{SG42953, &UA2433511},
	{SG42954, &UA2469173},
	{SG42955, &UA2616831},
	{SG42958, &UA2997297},
	{SG42959, &UA3073798},
	{SG42973, &UA2580270},
	{SG42981, &UA2258021},
	{SG42985, &UA2752113},
	{SG42987, &UA3049231},
	{SG42999, &UA2603409},
	{SG43001, &UA2735004},
	{SG43004, &UA2797425},
	{SG43007, &UA2634810},
	{SG43053, &UA2556064},
	{SG43055, &UA2862977},
	{SG43057, &UA2667116},
	{SG43058, &UA2030843},
	{SG43064, &UA2712803},
	{SG43067, &UA2800088},
	{SG43080, &UA2531411},
	{SG43081, &UA2734385},
	{SG43092, &UA2028873},
	{SG43093, &UA2029127},
	{SG43094, &UA2187904},
	{SG43095, &UA2342372},
	{SG43102, &UA3081780},
	{SG43103, &UA3082589},
	{SG43104, &UA3083756},
	{SG43105, &UA2528367},
	{SG43106, &UA2732204},
	{SG43108, &UA2795767},
	{SG43109, &UA2849850},
	{SG43110, &UA2892319},
	{SG43111, &UA3101206},
	{SG43130, &UA2768462},
	{SG43132, &UA2797443},
	{SG43134, &UA2974153},
	{SG43153, &UA2415815},
	{SG43155, &UA2481913},
	{SG43157, &UA2554373},
	{SG43158, &UA2532230},
	{SG43159, &UA2675419},
	{SG43162, &UA3109167},
	{SG43169, &UA2707096},
	{SG43170, &UA2792212},
	{SG43171, &UA2976697},
	{SG43179, &UA2012820},
	{SG43180, &UA2174592},
	{SG43182, &UA2258074},
	{SG43183, &UA2760107},
	{SG43184, &UA2974080},
	{SG43186, &UA2675855},
	{SG43187, &UA2991886},
	{SG43195, &UA2420615},
	{SG43201, &UA2462487},
	{SG43205, &UA2768452},
	{SG43220, &UA2200086},
	{SG43223, &UA2423375},
	{SG43225, &UA2483084},
	{SG43227, &UA2536355},
	{SG43239, &UA2033097},
	{SG43242, &UA2469088},
	{SG43244, &UA3044622},
	{SG43245, &UA3105409},
	{SG43250, &UA2675033},
	{SG43253, &UA3120568},
	{SG43255, &UA2772751},
	{SG43257, &UA2974310},
	{SG43260, &UA3080548},
	{SG43263, &UA3080903},
	{SG43265, &UA3094687},
	{SG43271, &UA2664512},
	{SG43278, &UA2258057},
	{SG43286, &UA2580570},
	{SG43296, &UA2427265},
	{SG43298, &UA2600710},
	{SG43299, &UA2853666},
	{SG43301, &UA2975948},
	{SG43310, &UA2712382},
	{SG43312, &UA2892364},
	{SG43314, &UA2782603},
	{SG43316, &UA2869723},
	{SG43317, &UA3049654},
	{SG43319, &UA2014857},
	{SG43321, &UA2562064},
	{SG43323, &UA2706198},
	{SG43327, &UA3083599},
	{SG43328, &UA3093819},
	{SG43330, &UA3094234},
	{SG43331, &UA3094447},
	{SG43332, &UA3095062},
	{SG43333, &UA3114925},
	{SG43335, &UA2864885},
	{SG43336, &UA3020855},
	{SG43337, &UA3021779},
	{SG43338, &UA3022995},
	{SG43339, &UA2466156},
	{SG43341, &UA2658968},
	{SG43355, &UA2600588},
	{SG43356, &UA2755330},
	{SG43373, &UA2567454},
	{SG43376, &UA2867211},
	{SG43380, &UA2426630},
	{SG43384, &UA2713846},
	{SG43388, &UA2843463},
	{SG43395, &UA2973898},
	{SG43396, &UA3068401},
	{SG43419, &UA2768898},
	{SG43420, &UA3082744},
	{SG43425, &UA2506386},
	{SG43431, &UA1995642},
	{SG43432, &UA1996866},
	{SG43433, &UA2194419},
	{SG43435, &UA2258065},
	{SG43437, &UA2385123},
	{SG43438, &UA2425008},
	{SG43439, &UA2667616},
	{SG43443, &UA2560821},
	{SG43446, &UA2938600},
	{SG43453, &UA2454471},
	{SG43455, &UA2616571},
	{SG43457, &UA2753398},
	{SG43462, &UA2973915},
	{SG43463, &UA3028035},
	{SG43469, &UA2633548},
	{SG43470, &UA2773275},
	{SG43472, &UA2876519},
	{SG43524, &UA2763692},
	{SG43528, &UA2032472},
	{SG43531, &UA2704709},
	{SG43538, &UA2703509},
	{SG43548, &UA2029562},
	{SG43550, &UA2186465},
	{SG43552, &UA2366817},
	{SG43554, &UA2428176},
	{SG43559, &UA2583172},
	{SG43561, &UA2585175},
	{SG43565, &UA2718810},
	{SG43581, &UA2749277},
	{SG43598, &UA2659296},
	{SG43612, &UA3131331},
	{SG43614, &UA2658856},
	{SG43657, &UA2893388},
	{SG43658, &UA3083256},
	{SG43662, &UA2598985},
	{SG43665, &UA2770336},
	{SG43678, &UA3130995},
	{SG43684, &UA2672586},
	{SG43689, &UA2559761},
	{SG43694, &UA3031504},
	{SG43703, &UA2800093},
	{SG43705, &UA2751322},
	{SG43738, &UA3068326},
	{SG43743, &UA2675357},
	{SG43746, &UA2961306},
	{SG43762, &UA2763203},
	{SG43766, &UA2756902},
	{SG43774, &UA2232258},
	{SG43775, &UA2355660},
	{SG43793, &UA2976691},
	{SG43794, &UA3081377},
	{SG43822, &UA3023764},
	{SG43828, &UA3031593},
	{SG43842, &UA2432464},
	{SG43843, &UA2840809},
	{SG43844, &UA2957782},
	{SG43850, &UA2185322},
	{SG43853, &UA2732881},
	{SG43874, &UA2457714},
	{SG43879, &UA2581240},
	{SG43881, &UA2582095},
	{SG43888, &UA2200093},
	{SG43892, &UA2400232},
	{SG43896, &UA2875375},
	{SG43908, &UA2597126},
	{SG43909, &UA2865156},
	{SG43911, &UA3014501},
	{SG43918, &UA2559656},
	{SG43919, &UA2986578},
	{SG43921, &UA2663539},
	{SG43922, &UA2778885},
	{SG43926, &UA3083199},
	{SG43933, &UA3039329},
	{SG43941, &UA2507750},
	{SG43943, &UA2567113},
	{SG43944, &UA2783284},
	{SG43947, &UA3066219},
	{SG43957, &UA2460779},
	{SG43959, &UA2795425},
	{SG43960, &UA2875471},
	{SG43961, &UA2958023},
	{SG43962, &UA2958486},
	{SG43963, &UA2958494},
	{SG43979, &UA3032402},
	{SG43984, &UA2772769},
	{SG43986, &UA3082758},
	{SG43987, &UA2028631},
	{SG43992, &UA2425070},
	{SG43994, &UA2459209},
	{SG44005, &UA2714811},
	{SG44008, &UA2230045},
	{SG44012, &UA2734222},
	{SG44013, &UA2894361},
	{SG44018, &UA2564830},
	{SG44029, &UA2961886},
	{SG44040, &UA2486687},
	{SG44044, &UA2659723},
	{SG44049, &UA3049859},
	{SG44062, &UA2562915},
	{SG44063, &UA2600027},
	{SG44075, &UA3024109},
	{SG44084, &UA2667042},
	{SG44087, &UA3102670},
	{SG44093, &UA2967514},
	{SG44094, &UA3124901},
	{SG44102, &UA2663712},
	{SG44103, &UA2973470},
	{SG44105, &UA3108796},
	{SG44109, &UA2506793},
	{SG44112, &UA3080318},
	{SG44116, &UA2014550},
	{SG44117, &UA2030149},
	{SG44118, &UA2975797},
	{SG44119, &UA3027858},
	{SG44132, &UA2773535},
	{SG44133, &UA2797104},
	{SG44134, &UA2798772},
	{SG44135, &UA2879358},
	{SG44153, &UA2596617},
	{SG44154, &UA2794119},
	{SG44155, &UA2753607},
	{SG44163, &UA3080178},
	{SG44170, &UA2559608},
	{SG44173, &UA2707384},
	{SG44177, &UA2796459},
	{SG44179, &UA2768660},
	{SG44186, &UA2715154},
	{SG44187, &UA3082980},
	{SG44188, &UA3080838},
	{SG44199, &UA2567243},
	{SG44201, &UA2570466},
	{SG44203, &UA2712061},
	{SG44224, &UA2703492},
	{SG44226, &UA2963220},
	{SG44229, &UA2462059},
	{SG44273, &UA2582531},
	{SG44274, &UA2769858},
	{SG44275, &UA2799129},
	{SG44277, &UA2964928},
	{SG44280, &UA3063055},
	{SG44321, &UA2344606},
	{SG44323, &UA2435434},
	{SG44325, &UA2484451},
	{SG44326, &UA2985850},
	{SG44330, &UA2664790},
	{SG44331, &UA2782970},
	{SG44361, &UA2461332},
	{SG44363, &UA2536811},
	{SG44366, &UA3080257},
	{SG44378, &UA2228133},
	{SG44381, &UA2722743},
	{SG44384, &UA2586550},
	{SG44388, &UA2955667},
	{SG44390, &UA2956321},
	{SG44391, &UA2992053},
	{SG44393, &UA2992079},
	{SG44394, &UA3022223},
	{SG44395, &UA3022373},
	{SG44396, &UA3023395},
	{SG44402, &UA2230309},
	{SG44403, &UA2585268},
	{SG44404, &UA2601345},
	{SG44405, &UA2861393},
	{SG44408, &UA3048718},
	{SG44410, &UA3093243},
	{SG44411, &UA2362979},
	{SG44412, &UA2432540},
	{SG44414, &UA2753980},
	{SG44416, &UA2485538},
	{SG44426, &UA2536888},
	{SG44428, &UA2659254},
	{SG44431, &UA2562548},
	{SG44432, &UA3118842},
	{SG44435, &UA2973888},
	{SG44436, &UA2974072},
	{SG44459, &UA2963051},
	{SG44471, &UA2505577},
	{SG44473, &UA2791448},
	{SG44478, &UA2707714},
	{SG44480, &UA2782276},
	{SG44481, &UA3080254},
	{SG44486, &UA2704272},
	{SG44487, &UA2853379},
	{SG44490, &UA3083369},
	{SG44492, &UA3082938},
	{SG44494, &UA3094440},
	{SG44500, &UA2559758},
	{SG44501, &UA2958144},
	{SG44503, &UA2658190},
	{SG44507, &UA2673244},
	{SG44509, &UA3081113},
	{SG44534, &UA2430122},
	{SG44560, &UA2974245},
	{SG44564, &UA2567687},
	{SG44591, &UA2507525},
	{SG44592, &UA3014789},
	{SG44599, &UA2879326},
	{SG44631, &UA2879571},
	{SG44633, &UA2033485},
	{SG44634, &UA2657624},
	{SG44635, &UA3099995},
	{SG44636, &UA3080380},
	{SG44653, &UA2988398},
	{SG44668, &UA2601744},
	{SG44669, &UA2975562},
	{SG44679, &UA2550399},
	{SG44711, &UA2353983},
	{SG44714, &UA2533470},
	{SG44716, &UA2560168},
	{SG44717, &UA2700281},
	{SG44735, &UA2667969},
	{SG44748, &UA2773956},
	{SG44751, &UA2782632},
	{SG44755, &UA2796238},
	{SG44767, &UA2783355},
	{SG44781, &UA2170992},
	{SG44783, &UA2612260},
	{SG44784, &UA2612781},
	{SG44785, &UA2615756},
	{SG44792, &UA2350576},
	{SG44803, &UA2976009},
	{SG44810, &UA2974095},
	{SG44823, &UA2424083},
	{SG44825, &UA2727553},
	{SG44829, &UA2485710},
	{SG44833, &UA2705750},
	{SG44850, &UA2736652},
	{SG44852, &UA2891591},
	{SG44853, &UA3087931},
	{SG44856, &UA2898419},
	{SG44857, &UA3020052},
	{SG44858, &UA3017807},
	{SG44860, &UA3031011},
	{SG44861, &UA3059106},
	{SG44862, &UA3060463},
	{SG44863, &UA3125291},
	{SG44902, &UA2732913},
	{SG44904, &UA2754803},
	{SG44907, &UA2872630},
	{SG44920, &UA3132873},
	{SG44939, &UA2029816},
	{SG44943, &UA2755536},
	{SG44947, &UA3083144},
	{SG44948, &UA3083769},
	{SG44959, &UA2730249},
	{SG44966, &UA2746505},
	{SG44970, &UA2561455},
	{SG44986, &UA2462486},
	{SG44989, &UA2769701},
	{SG44991, &UA2975561},
	{SG45000, &UA2449364},
	{SG45004, &UA2738037},
	{SG45005, &UA3120845},
	{SG45009, &UA3027546},
	{SG45026, &UA2879678},
	{SG45027, &UA3032693},
	{SG45039, &UA2468435},
	{SG45045, &UA2755522},
	{SG45046, &UA2772496},
	{SG45047, &UA2892925},
	{SG45049, &UA3081330},
	{SG45067, &UA2557330},
	{SG45070, &UA2658940},
	{SG45072, &UA2792337},
	{SG45074, &UA3044521},
	{SG45075, &UA2676590},
	{SG45083, &UA3081343},
	{SG45088, &UA2170829},
	{SG45089, &UA2365059},
	{SG45092, &UA2774166},
	{SG45095, &UA2031763},
	{SG45096, &UA2937189},
	{SG45108, &UA2457992},
	{SG45109, &UA2655277},
	{SG45110, &UA2672447},
	{SG45113, &UA2938246},
	{SG45114, &UA2848797},
	{SG45116, &UA3021577},
	{SG45117, &UA3078001},
	{SG45133, &UA2595639},
	{SG45135, &UA2637644},
	{SG45138, &UA2735223},
	{SG45140, &UA3081060},
	{SG45152, &UA2955663},
	{SG45154, &UA2997665},
	{SG45155, &UA2635219},
	{SG45156, &UA2636393},
	{SG45157, &UA2866238},
	{SG45158, &UA2566751},
	{SG45169, &UA2613458},
	{SG45170, &UA2613937},
	{SG45172, &UA2615915},
	{SG45175, &UA2620256},
	{SG45179, &UA2621104},
	{SG45181, &UA2468394},
	{SG45183, &UA2558284},
	{SG45185, &UA2633925},
	{SG45189, &UA2773889},
	{SG45191, &UA2878683},
	{SG45192, &UA3102813},
	{SG45193, &UA2975572},
	{SG45221, &UA2600639},
	{SG45223, &UA3071624},
	{SG45229, &UA2738759},
	{SG45233, &UA2715204},
	{SG45235, &UA2734632},
	{SG45237, &UA2783056},
	{SG45239, &UA2800623},
	{SG45241, &UA2879014},
	{SG45254, &UA2595285},
	{SG45256, &UA2763544},
	{SG45257, &UA2676831},
	{SG45258, &UA2754623},
	{SG45262, &UA2876323},
	{SG45266, &UA3026816},
	{SG45272, &UA2870792},
	{SG45274, &UA2974652},
	{SG45276, &UA2996811},
	{SG45285, &UA2711452},
	{SG45286, &UA2747316},
	{SG45288, &UA2964693},
	{SG45290, &UA3027713},
	{SG45297, &UA2705237},
	{SG45300, &UA2755047},
	{SG45303, &UA2950533},
	{SG45304, &UA2565297},
	{SG45310, &UA2768951},
	{SG45311, &UA2895570},
	{SG45321, &UA2599142},
	{SG45324, &UA2796384},
	{SG45327, &UA2799371},
	{SG45339, &UA2437587},
	{SG45342, &UA2567707},
	{SG45345, &UA2706592},
	{SG45347, &UA2707402},
	{SG45348, &UA2994124},
	{SG45352, &UA2869729},
	{SG45354, &UA2999594},
	{SG45364, &UA2879134},
	{SG45366, &UA3051480},
	{SG45378, &UA2796773},
	{SG45380, &UA3096560},
	{SG45393, &UA2672822},
	{SG45396, &UA2859168},
	{SG45398, &UA3048449},
	{SG45401, &UA2716675},
	{SG45404, &UA2766800},
	{SG45407, &UA2768007},
	{SG45409, &UA2998609},
	{SG45420, &UA2583420},
	{SG45422, &UA2974999},
	{SG45425, &UA3131653},
	{SG45435, &UA2231217},
	{SG45439, &UA2673167},
	{SG45441, &UA2732866},
	{SG45442, &UA2865794},
	{SG45443, &UA2879663},
	{SG45451, &UA3083934},
	{SG45460, &UA2485052},
	{SG45463, &UA2751002},
	{SG45465, &UA2769684},
	{SG45473, &UA2532273},
	{SG45479, &UA2796044},
	{SG45481, &UA2529936},
	{SG45483, &UA2644638},
	{SG45484, &UA2860351},
	{SG45489, &UA2599840},
	{SG45490, &UA3073520},
	{SG45494, &UA2225967},
	{SG45499, &UA2459816},
	{SG45501, &UA2634070},
	{SG45508, &UA2751404},
	{SG45510, &UA2998169},
	{SG45524, &UA2422838},
	{SG45530, &UA2603415},
	{SG45532, &UA2701247},
	{SG45590, &UA2666422},
	{SG45593, &UA2773844},
	{SG45594, &UA2997390},
	{SG45595, &UA3005494},
	{SG45596, &UA3049514},
	{SG45601, &UA2705297},
	{SG45603, &UA2733090},
	{SG45604, &UA2955284},
	{SG45610, &UA3027870},
	{SG45612, &UA3080190},
	{SG45625, &UA2738177},
	{SG45628, &UA3088618},
	{SG45629, &UA2972145},
	{SG45644, &UA2030522},
	{SG45646, &UA2383496},
	{SG45665, &UA2943128},
	{SG45669, &UA3037205},
	{SG45674, &UA2584163},
	{SG45676, &UA2766785},
	{SG45677, &UA2701425},
	{SG45678, &UA2713953},
	{SG45685, &UA2975988},
	{SG45687, &UA2975799},
	{SG45703, &UA2896879},
	{SG45714, &UA2172108},
	{SG45717, &UA2179637},
	{SG45722, &UA2707045},
	{SG45728, &UA2866235},
	{SG45730, &UA2945355},
	{SG45733, &UA3022490},
	{SG45736, &UA3094349},
	{SG45740, &UA2000888},
	{SG45742, &UA2007184},
	{SG45744, &UA2199439},
	{SG45759, &UA2232593},
	{SG45760, &UA2665467},
	{SG45761, &UA2795081},
	{SG45762, &UA2873715},
	{SG45764, &UA3012002},
	{SG45766, &UA2937463},
	{SG45768, &UA3037217},
	{SG45782, &UA2732449},
	{SG45784, &UA2896330},
	{SG45787, &UA3039512},
	{SG45800, &UA2423567},
	{SG45803, &UA2602982},
	{SG45818, &UA2581952},
	{SG45828, &UA2634111},
	{SG45830, &UA2704214},
	{SG45834, &UA2877899},
	{SG45835, &UA2892374},
	{SG45839, &UA3059266},
	{SG45849, &UA2672045},
	{SG45851, &UA2603259},
	{SG45863, &UA3033277},
	{SG45876, &UA2428150},
	{SG45881, &UA2630299},
	{SG45882, &UA2633832},
	{SG45889, &UA2872217},
	{SG45905, &UA2348890},
	{SG45908, &UA2581860},
	{SG45917, &UA2879380},
	{SG45938, &UA2717592},
	{SG45940, &UA2862262},
	{SG45941, &UA3076893},
	{SG45943, &UA2866112},
	{SG45945, &UA2997971},
	{SG45946, &UA3088447},
	{SG45948, &UA3107745},
	{SG45952, &UA2975702},
	{SG45957, &UA2423435},
	{SG45959, &UA2754474},
	{SG45960, &UA2867487},
	{SG45961, &UA2943790},
	{SG45964, &UA3077604},
	{SG45966, &UA2976683},
	{SG45967, &UA2974709},
	{SG45968, &UA3092209},
	{SG45969, &UA3031509},
	{SG45971, &UA2974106},
	{SG45973, &UA3064791},
	{SG45974, &UA3080044},
	{SG45976, &UA3080539},
	{SG45983, &UA2765307},
	{SG45987, &UA2800238},
	{SG46020, &UA2692857},
	{SG46021, &UA2483510},
	{SG46022, &UA2632666},
	{SG46023, &UA2706166},
	{SG46026, &UA2960413},
	{SG46030, &UA3021496},
	{SG46046, &UA2612051},
	{SG46047, &UA2615317},
	{SG46048, &UA2615844},
	{SG46051, &UA2755839},
	{SG46052, &UA2892641},
	{SG46053, &UA2772683},
	{SG46059, &UA2383767},
	{SG46060, &UA2422232},
	{SG46062, &UA2982625},
	{SG46064, &UA2991245},
	{SG46066, &UA2583083},
	{SG46068, &UA2708504},
	{SG46069, &UA2567199},
	{SG46074, &UA2738073},
	{SG46077, &UA3053101},
	{SG46091, &UA2664142},
	{SG46093, &UA2768258},
	{SG46095, &UA2975785},
	{SG46096, &UA3021735},
	{SG46097, &UA3083288},
	{SG46119, &UA2419913},
	{SG46124, &UA2768163},
	{SG46127, &UA2891524},
	{SG46128, &UA3023688},
	{SG46129, &UA3062119},
	{SG46134, &UA3080831},
	{SG46138, &UA3131179},
	{SG46144, &UA2659307},
	{SG46145, &UA2675380},
	{SG46146, &UA2754275},
	{SG46170, &UA2783938},
	{SG46173, &UA3023359},
	{SG46183, &UA2354816},
	{SG46184, &UA2459602},
	{SG46185, &UA2483750},
	{SG46186, &UA2623220},
	{SG46188, &UA2637797},
	{SG46190, &UA2535833},
	{SG46191, &UA2603254},
	{SG46192, &UA2669433},
	{SG46197, &UA3022516},
	{SG46204, &UA2419431},
	{SG46206, &UA2551114},
	{SG46214, &UA2668033},
	{SG46216, &UA2713650},
	{SG46222, &UA2869809},
	{SG46224, &UA3021726},
	{SG46237, &UA2677004},
	{SG46238, &UA2713679},
	{SG46239, &UA2749686},
	{SG46240, &UA2769902},
	{SG46241, &UA3078573},
	{SG46254, &UA2676690},
	{SG46256, &UA2945607},
	{SG46271, &UA2558904},
	{SG46272, &UA2632059},
	{SG46277, &UA2704296},
	{SG46283, &UA2699594},
	{SG46284, &UA2795944},
	{SG46289, &UA3131309},
	{SG46302, &UA2427196},
	{SG46303, &UA2561200},
	{SG46304, &UA2659212},
	{SG46305, &UA2952489},
	{SG46306, &UA2707350},
	{SG46308, &UA2708268},
	{SG46310, &UA2718532},
	{SG46312, &UA2867883},
	{SG46314, &UA3058952},
	{SG46315, &UA2733595},
	{SG46316, &UA2751059},
	{SG46323, &UA2380116},
	{SG46324, &UA2484940},
	{SG46325, &UA2937201},
	{SG46326, &UA3021590},
	{SG46328, &UA3080372},
	{SG46339, &UA2578260},
	{SG46340, &UA2668070},
	{SG46347, &UA2452596},
	{SG46360, &UA2879711},
	{SG46365, &UA2637114},
	{SG46368, &UA2664986},
	{SG46369, &UA3022841},
	{SG46372, &UA2781777},
	{SG46375, &UA2002003},
	{SG46381, &UA2535756},
	{SG46392, &UA2997190},
	{SG46398, &UA3094276},
	{SG46411, &UA2422806},
	{SG46413, &UA2594959},
	{SG46417, &UA2469229},
	{SG46423, &UA2600840},
	{SG46429, &UA2709099},
	{SG46435, &UA3016598},
	{SG46446, &UA2384754},
	{SG46449, &UA2562551},
	{SG46451, &UA2863455},
	{SG46453, &UA2955297},
	{SG46456, &UA2671089},
	{SG46464, &UA2754490},
	{SG46485, &UA2989706},
	{SG46486, &UA2467841},
	{SG46487, &UA2560151},
	{SG46491, &UA2676600},
	{SG46494, &UA2893039},
	{SG46498, &UA3081109},
	{SG46510, &UA2029941},
	{SG46512, &UA2610671},
	{SG46514, &UA2613476},
	{SG46515, &UA2616529},
	{SG46518, &UA2695388},
	{SG46520, &UA2708964},
	{SG46525, &UA2953470},
	{SG46534, &UA2560660},
	{SG46537, &UA2600449},
	{SG46539, &UA2668357},
	{SG46540, &UA2670607},
	{SG46541, &UA2700421},
	{SG46542, &UA2752894},
	{SG46543, &UA2772863},
	{SG46546, &UA2879920},
	{SG46547, &UA2892230},
	{SG46548, &UA3022273},
	{SG46564, &UA2485243},
	{SG46608, &UA2753215},
	{SG46646, &UA3081471},
	{SG46650, &UA2422979},
	{SG46657, &UA2567218},
	{SG46660, &UA2597341},
	{SG46662, &UA3099773},
	{SG46667, &UA2713118},
	{SG46669, &UA2754191},
	{SG46670, &UA2872747},
	{SG46671, &UA2892418},
	{SG46679, &UA2665813},
	{SG46683, &UA2774283},
	{SG46685, &UA2937807},
	{SG46686, &UA2755553},
	{SG46695, &UA2704123},
	{SG46697, &UA2848684},
	{SG46719, &UA2505783},
	{SG46721, &UA2690808},
	{SG46727, &UA2704574},
	{SG46790, &UA2712246},
	{SG46791, &UA2732144},
	{SG46795, &UA2782445},
	{SG46805, &UA3126641},
	{SG46821, &UA2349023},
	{SG46828, &UA2536111},
	{SG46829, &UA2729337},
	{SG46830, &UA2735787},
	{SG46832, &UA2769887},
	{SG46833, &UA2867868},
	{SG46834, &UA2892574},
	{SG46840, &UA2975977},
	{SG46849, &UA2558214},
	{SG46856, &UA2795732},
	{SG46859, &UA2799351},
	{SG46860, &UA2800680},
	{SG46867, &UA3096345},
	{SG46908, &UA2339452},
	{SG46909, &UA2612264},
	{SG46913, &UA2612416},
	{SG46914, &UA2618518},
	{SG46917, &UA2469350},
	{SG46921, &UA2599913},
	{SG46924, &UA2708460},
	{SG46927, &UA2768603},
	{SG46928, &UA2770509},
	{SG46935, &UA3094549},
	{SG46940, &UA2226948},
	{SG46944, &UA2481419},
	{SG46950, &UA2672913},
	{SG46962, &UA2974314},
	{SG46973, &UA2229771},
	{SG46976, &UA2352438},
	{SG46984, &UA2774160},
	{SG46986, &UA2896731},
	{SG47036, &UA2529713},
	{SG47037, &UA2583038},
	{SG47038, &UA2649896},
	{SG47058, &UA2599314},
	{SG47060, &UA2704360},
	{SG47061, &UA2705080},
	{SG47076, &UA2449964},
	{SG47082, &UA2602441},
	{SG47085, &UA2867202},
	{SG47088, &UA2773124},
	{SG47099, &UA3083940},
	{SG47109, &UA1997252},
	{SG47112, &UA2012244},
	{SG47118, &UA2530937},
	{SG47122, &UA2558942},
	{SG47169, &UA2596301},
	{SG47184, &UA2580137},
	{SG47193, &UA2781442},
	{SG47204, &UA2466235},
	{SG47205, &UA2583191},
	{SG47207, &UA2665123},
	{SG47211, &UA2890061},
	{SG47224, &UA2754722},
	{SG47225, &UA2879437},
	{SG47238, &UA2636611},
	{SG47239, &UA2673373},
	{SG47243, &UA2799247},
	{SG47247, &UA2966723},
	{SG47251, &UA2584174},
	{SG47255, &UA2841631},
	{SG47257, &UA2974118},
	{SG47261, &UA3021774},
	{SG47277, &UA3081237},
	{SG47288, &UA2769163},
	{SG47292, &UA3083079},
	{SG47293, &UA3080748},
	{SG47303, &UA2503144},
	{SG47304, &UA2506855},
	{SG47305, &UA2529287},
	{SG47306, &UA2579780},
	{SG47307, &UA2580285},
	{SG47309, &UA2704493},
	{SG47310, &UA2846371},
	{SG47317, &UA2974710},
	{SG47318, &UA2999485},
	{SG47319, &UA3081681},
	{SG47328, &UA2350286},
	{SG47330, &UA2615084},
	{SG47368, &UA2753700},
	{SG47390, &UA2580963},
	{SG47395, &UA2893102},
	{SG47396, &UA3030828},
	{SG47397, &UA2974265},
	{SG47420, &UA2654428},
	{SG47426, &UA2748745},
	{SG47428, &UA3121519},
	{SG47429, &UA2771202},
	{SG47436, &UA3083284},
	{SG47449, &UA2583313},
	{SG47450, &UA2732695},
	{SG47451, &UA2733268},
	{SG47452, &UA2770228},
	{SG47467, &UA2467768},
	{SG47471, &UA2958196},
	{SG47476, &UA2975676},
	{SG47482, &UA3132936},
	{SG47492, &UA2227773},
	{SG47497, &UA2567510},
	{SG47500, &UA2670734},
	{SG47511, &UA2482885},
	{SG47515, &UA2601608},
	{SG47520, &UA2795796},
	{SG47521, &UA2878742},
	{SG47528, &UA3022415},
	{SG47534, &UA3096349},
	{SG47547, &UA2767433},
	{SG47548, &UA2892613},
	{SG47551, &UA3017823},
	{SG47553, &UA3059605},
	{SG47568, &UA2466243},
	{SG47572, &UA2613429},
	{SG47573, &UA2615651},
	{SG47581, &UA2708676},
	{SG47602, &UA2532627},
	{SG47608, &UA2673242},
	{SG47609, &UA2674227},
	{SG47611, &UA2736271},
	{SG47612, &UA2754353},
	{SG47615, &UA2958414},
	{SG47616, &UA2952116},
	{SG47621, &UA3083486},
	{SG47622, &UA3094308},
	{SG47649, &UA2480177},
	{SG47666, &UA2863872},
	{SG47669, &UA2974443},
	{SG47672, &UA3081048},
	{SG47681, &UA2666787},
	{SG47700, &UA2457556},
	{SG47706, &UA2600231},
	{SG47713, &UA2769534},
	{SG47718, &UA2944088},
	{SG47722, &UA3059448},
	{SG47725, &UA3023743},
	{SG47736, &UA2600045},
	{SG47737, &UA2660905},
	{SG47745, &UA2530329},
	{SG47746, &UA2623783},
	{SG47748, &UA2709057},
	{SG47754, &UA2773724},
	{SG47776, &UA2613625},
	{SG47778, &UA2615932},
	{SG47784, &UA2580827},
	{SG47788, &UA2755262},
	{SG47792, &UA2793465},
	{SG47794, &UA2779723},
	{SG47820, &UA2750618},
	{SG47822, &UA2751595},
	{SG47826, &UA2773810},
	{SG47829, &UA2898424},
	{SG47830, &UA3125665},
	{SG47833, &UA3063693},
	{SG47853, &UA2702783},
	{SG47856, &UA2953836},
	{SG47859, &UA2755175},
	{SG47861, &UA2770952},
	{SG47864, &UA3023779},
	{SG47870, &UA3046305},
	{SG47873, &UA3080938},
	{SG47894, &UA2772485},
	{SG47911, &UA2257788},
	{SG47913, &UA2351241},
	{SG47919, &UA2467677},
	{SG47925, &UA2579992},
	{SG47927, &UA2738131},
	{SG47933, &UA2939555},
	{SG47956, &UA2751935},
	{SG47957, &UA2773991},
	{SG47993, &UA2767314},
	{SG47994, &UA2772675},
	{SG47998, &UA3023027},
	{SG48010, &UA2585174},
	{SG48014, &UA2674221},
	{SG48015, &UA2708390},
	{SG48017, &UA2767203},
	{SG48047, &UA2467491},
	{SG48052, &UA2559790},
	{SG48053, &UA2561905},
	{SG48060, &UA2625512},
	{SG48061, &UA2635338},
	{SG48063, &UA2629518},
	{SG48065, &UA2631111},
	{SG48069, &UA2983777},
	{SG48087, &UA2597487},
	{SG48090, &UA2633775},
	{SG48095, &UA2768824},
	{SG48099, &UA2879492},
	{SG48101, &UA2869654},
	{SG48102, &UA2956616},
	{SG48103, &UA3023543},
	{SG48121, &UA2405782},
	{SG48123, &UA2530970},
	{SG48126, &UA2596564},
	{SG48127, &UA2748323},
	{SG48135, &UA2896029},
	{SG48145, &UA2819487},
	{SG48147, &UA2673235},
	{SG48150, &UA2735853},
	{SG48154, &UA2781687},
	{SG48156, &UA2867799},
	{SG48157, &UA2892744},
	{SG48158, &UA3059976},
	{SG48177, &UA2564370},
	{SG48181, &UA2602191},
	{SG48183, &UA2732108},
	{SG48186, &UA2770935},
	{SG48188, &UA2799791},
	{SG48191, &UA3020719},
	{SG48194, &UA3094788},
	{SG48205, &UA2454896},
	{SG48206, &UA2426538},
	{SG48207, &UA2761534},
	{SG48208, &UA2470405},
	{SG48211, &UA2505882},
	{SG48223, &UA2867477},
	{SG48228, &UA2992748},
	{SG48229, &UA3020854},
	{SG48250, &UA2662789},
	{SG48256, &UA2769439},
	{SG48264, &UA2892635},
	{SG48265, &UA2943812},
	{SG48271, &UA3021803},
	{SG48275, &UA3096308},
	{SG48280, &UA2528351},
	{SG48281, &UA2666534},
	{SG48294, &UA2995970},
	{SG48299, &UA3094548},
	{SG48319, &UA2652232},
	{SG48321, &UA2226695},
	{SG48322, &UA2346952},
	{SG48327, &UA2468154},
	{SG48329, &UA2482184},
	{SG48333, &UA2635822},
	{SG48335, &UA2604732},
	{SG48344, &UA2796205},
	{SG48348, &UA2869688},
	{SG48351, &UA3022640},
	{SG48434, &UA3001066},
	{SG48435, &UA3092529},
	{SG48437, &UA3022329},
	{SG48451, &UA2466187},
	{SG48453, &UA2466191},
	{SG48454, &UA2839148},
	{SG48467, &UA2671485},
	{SG48471, &UA2733721},
	{SG48472, &UA2750767},
	{SG48478, &UA2783522},
	{SG48485, &UA2998185},
	{SG48486, &UA3022597},
	{SG48488, &UA3060877},
	{SG48499, &UA2768930},
	{SG48500, &UA2894216},
	{SG48502, &UA3017040},
	{SG48505, &UA3130733},
	{SG48532, &UA2506367},
	{SG48541, &UA2783789},
	{SG48543, &UA2893024},
	{SG48548, &UA2855850},
	{SG48551, &UA3012774},
	{SG48553, &UA3079600},
	{SG48557, &UA3062754},
	{SG48559, &UA3028096},
	{SG48560, &UA3083647},
	{SG48568, &UA2342987},
	{SG48572, &UA2483846},
	{SG48577, &UA2657744},
	{SG48584, &UA2773129},
	{SG48586, &UA2942488},
	{SG48588, &UA3021870},
	{SG48590, &UA3081194},
	{SG48591, &UA3094013},
	{SG48593, &UA3131303},
	{SG48594, &UA3132707},
	{SG48603, &UA2668588},
	{SG48604, &UA2765814},
	{SG48607, &UA2868089},
	{SG48623, &UA2556176},
	{SG48633, &UA2705201},
	{SG48638, &UA2784792},
	{SG48640, &UA2849022},
	{SG48642, &UA2849446},
	{SG48644, &UA2869018},
	{SG48649, &UA3019931},
	{SG48656, &UA3080940},
	{SG48661, &UA2455403},
	{SG48662, &UA2457933},
	{SG48663, &UA2505074},
	{SG48669, &UA2856029},
	{SG48673, &UA2879137},
	{SG48674, &UA2940515},
	{SG48676, &UA3023536},
	{SG48678, &UA3022589},
	{SG48682, &UA3094266},
	{SG48746, &UA2486901},
	{SG48748, &UA2555970},
	{SG48770, &UA3109368},
	{SG48771, &UA3065264},
	{SG48778, &UA3113321},
	{SG48790, &UA2555290},
	{SG48797, &UA2696614},
	{SG48798, &UA3091180},
	{SG48806, &UA2893047},
	{SG48807, &UA2901201},
	{SG48808, &UA2937819},
	{SG48809, &UA3024188},
	{SG48810, &UA3094845},
	{SG48824, &UA2000874},
	{SG48825, &UA2007208},
	{SG48827, &UA2745168},
	{SG48834, &UA2594501},
	{SG48848, &UA2870149},
	{SG48855, &UA2999587},
	{SG48859, &UA3030755},
	{SG48862, &UA3112266},
	{SG48883, &UA2600344},
	{SG48885, &UA2673181},
	{SG48886, &UA2707724},
	{SG48887, &UA2767400},
	{SG48890, &UA2769037},
	{SG48892, &UA2795400},
	{SG48894, &UA2868130},
	{SG48896, &UA2939347},
	{SG48899, &UA2944275},
	{SG48906, &UA2998660},
	{SG48907, &UA3102892},
	{SG48915, &UA3083489},
	{SG48937, &UA2692107},
	{SG48947, &UA2773857},
	{SG48968, &UA2029200},
	{SG48975, &UA2701217},
	{SG48978, &UA2709492},
	{SG48981, &UA2773625},
	{SG48983, &UA2954554},
	{SG48984, &UA3087505},
	{SG48987, &UA2963980},
	{SG48988, &UA3124329},
	{SG48992, &UA3022045},
	{SG48994, &UA3094686},
	{SG49013, &UA2713878},
	{SG49017, &UA2784250},
	{SG49019, &UA2879748},
	{SG49021, &UA2892547},
	{SG49025, &UA3031582},
	{SG49026, &UA3060744},
	{SG49027, &UA3107110},
	{SG49035, &UA2384124},
	{SG49037, &UA2453998},
	{SG49039, &UA2580710},
	{SG49045, &UA2780832},
	{SG49058, &UA2784628},
	{SG49061, &UA2861686},
	{SG49062, &UA2868375},
	{SG49063, &UA2998424},
	{SG49064, &UA2956898},
	{SG49066, &UA3079581},
	{SG49073, &UA2776794},
	{SG49076, &UA2558124},
	{SG49079, &UA2604076},
	{SG49081, &UA2769873},
	{SG49082, &UA2784769},
	{SG49083, &UA2792277},
	{SG49084, &UA2795050},
	{SG49090, &UA3083186},
	{SG49091, &UA3094616},
	{SG49107, &UA2481849},
	{SG49109, &UA2623613},
	{SG49113, &UA2782583},
	{SG49114, &UA2795964},
	{SG49116, &UA2937212},
	{SG49119, &UA3021662},
	{SG49122, &UA3023212},
	{SG49123, &UA3093966},
	{SG49134, &UA2482856},
	{SG49137, &UA2482882},
	{SG49142, &UA2633102},
	{SG49155, &UA2768860},
	{SG49173, &UA2706997},
	{SG49174, &UA2712199},
	{SG49176, &UA2999522},
	{SG49178, &UA2769546},
	{SG49182, &UA2858118},
	{SG49183, &UA2864821},
	{SG49186, &UA2867214},
	{SG49188, &UA2867603},
	{SG49190, &UA2869186},
	{SG49191, &UA2891335},
	{SG49192, &UA2962583},
	{SG49194, &UA3021589},
	{SG49198, &UA3000311},
	{SG49200, &UA3022868},
	{SG49201, &UA3044914},
	{SG49205, &UA3113436},
	{SG49206, &UA3115436},
	{SG49230, &UA2708508},
	{SG49231, &UA2708616},
	{SG49232, &UA2779987},
	{SG49234, &UA2867304},
	{SG49235, &UA2796596},
	{SG49238, &UA2963725},
	{SG49240, &UA3022258},
	{SG49241, &UA3021384},
	{SG49242, &UA3130953},
	{SG49247, &UA2381689},
	{SG49265, &UA2867973},
	{SG49267, &UA2873091},
	{SG49271, &UA2961755},
	{SG49275, &UA3021826},
	{SG49277, &UA3093653},
	{SG49280, &UA3112417},
	{SG49286, &UA3132605},
	{SG49308, &UA2830518},
	{SG49315, &UA2654519},
	{SG49317, &UA2773406},
	{SG49318, &UA2774117},
	{SG49320, &UA2865210},
	{SG49323, &UA2890864},
	{SG49325, &UA3023803},
	{SG49329, &UA3093330},
	{SG49347, &UA2527639},
	{SG49357, &UA2668482},
	{SG49360, &UA2705981},
	{SG49362, &UA2706951},
	{SG49363, &UA2713949},
	{SG49368, &UA2790402},
	{SG49369, &UA3028629},
	{SG49379, &UA3110805},
	{SG49381, &UA3092251},
	{SG49396, &UA2535415},
	{SG49399, &UA2580747},
	{SG49401, &UA2668012},
	{SG49403, &UA2676373},
	{SG49404, &UA2676847},
	{SG49406, &UA2714565},
	{SG49410, &UA2892457},
	{SG49411, &UA3049577},
	{SG49422, &UA3021330},
	{SG49424, &UA3021368},
	{SG49445, &UA2258077},
	{SG49447, &UA2500094},
	{SG49451, &UA2469285},
	{SG49453, &UA2485050},
	{SG49455, &UA2554226},
	{SG49458, &UA2611754},
	{SG49460, &UA2612819},
	{SG49467, &UA2675510},
	{SG49472, &UA2712888},
	{SG49475, &UA2734263},
	{SG49477, &UA2751481},
	{SG49489, &UA2955867},
	{SG49492, &UA2999614},
	{SG49493, &UA3121162},
	{SG49494, &UA3054828},
	{SG49495, &UA3105989},
	{SG49517, &UA2600633},
	{SG49523, &UA2702219},
	{SG49535, &UA3099598},
	{SG49538, &UA2862096},
	{SG49541, &UA2869207},
	{SG49543, &UA2879527},
	{SG49546, &UA2893018},
	{SG49555, &UA3079874},
	{SG49562, &UA3095525},
	{SG49564, &UA3113637},
	{SG49565, &UA3131611},
	{SG49566, &UA3132965},
	{SG49601, &UA2666766},
	{SG49602, &UA2667307},
	{SG49605, &UA2713163},
	{SG49606, &UA2713575},
	{SG49608, &UA2749279},
	{SG49609, &UA2783586},
	{SG49612, &UA2796875},
	{SG49615, &UA2869728},
	{SG49618, &UA2879568},
	{SG49620, &UA2897415},
	{SG49626, &UA3020624},
	{SG49629, &UA3062712},
	{SG49630, &UA3093645},
	{SG49673, &UA2595768},
	{SG49680, &UA2750084},
	{SG49683, &UA2783691},
	{SG49688, &UA2994833},
	{SG49693, &UA3029330},
	{SG49695, &UA3094139},
	{SG49709, &UA2709136},
	{SG49726, &UA3019453},
	{SG49728, &UA3022612},
	{SG49779, &UA2666965},
	{SG49780, &UA2667188},
	{SG49783, &UA2699289},
	{SG49790, &UA2799071},
	{SG49793, &UA2847301},
	{SG49800, &UA2878302},
	{SG49802, &UA2892381},
	{SG49813, &UA3021042},
	{SG49836, &UA2381188},
	{SG49840, &UA2465907},
	{SG49843, &UA2559788},
	{SG49850, &UA2713870},
	{SG49852, &UA2747575},
	{SG49858, &UA2798629},
	{SG49864, &UA2957753},
	{SG49866, &UA2965396},
	{SG49877, &UA2501593},
	{SG49878, &UA2507743},
	{SG49881, &UA2580163},
	{SG49884, &UA2706568},
	{SG49888, &UA2863965},
	{SG49891, &UA2865082},
	{SG49893, &UA2937493},
	{SG49895, &UA2961465},
	{SG49896, &UA3079719},
	{SG49899, &UA3083928},
	{SG49913, &UA2751413},
	{SG49916, &UA2767702},
	{SG49918, &UA2784247},
	{SG49921, &UA2866775},
	{SG49922, &UA2867400},
	{SG49923, &UA2937218},
	{SG49924, &UA2957886},
	{SG49927, &UA2998024},
	{SG49930, &UA3013059},
	{SG49932, &UA3048456},
	{SG49960, &UA2580412},
	{SG49962, &UA2600322},
	{SG49966, &UA2673560},
	{SG49969, &UA2709705},
	{SG49973, &UA2768373},
	{SG49975, &UA2773910},
	{SG49983, &UA2869420},
	{SG49985, &UA2893396},
	{SG49990, &UA2990928},
	{SG49992, &UA3046441},
	{SG49994, &UA3000719},
	{SG50007, &UA2991748},
	{SG50011, &UA2671044},
	{SG50016, &UA2706872},
	{SG50018, &UA2729510},
	{SG50022, &UA2784625},
	{SG50025, &UA2800432},
	{SG50028, &UA2845425},
	{SG50029, &UA2850054},
	{SG50030, &UA3086467},
	{SG50034, &UA2866117},
	{SG50044, &UA2943917},
	{SG50048, &UA2953581},
	{SG50052, &UA2958206},
	{SG50060, &UA3021653},
	{SG50062, &UA3049059},
	{SG50064, &UA3078394},
	{SG50088, &UA2228049},
	{SG50096, &UA2463936},
	{SG50098, &UA2817176},
	{SG50100, &UA2503391},
	{SG50106, &UA2667164},
	{SG50108, &UA2701881},
	{SG50111, &UA2766605},
	{SG50114, &UA2783163},
	{SG50119, &UA2798605},
	{SG50123, &UA2869894},
	{SG50126, &UA2877720},
	{SG50131, &UA2961746},
	{SG50138, &UA3057558},
	{SG50139, &UA3132421},
	{SG50156, &UA2226069},
	{SG50163, &UA2454333},
	{SG50211, &UA2529673},
	{SG50220, &UA2798931},
	{SG50222, &UA2799036},
	{SG50226, &UA2857305},
	{SG50227, &UA2954572},
	{SG50229, &UA2897698},
	{SG50234, &UA2956839},
	{SG50238, &UA2998145},
	{SG50241, &UA3110302},
	{SG50242, &UA3094231},
	{SG50259, &UA2455723},
	{SG50262, &UA2604498},
	{SG50267, &UA2645687},
	{SG50269, &UA2671028},
	{SG50271, &UA2708374},
	{SG50272, &UA2709493},
	{SG50274, &UA2732944},
	{SG50279, &UA2750724},
	{SG50284, &UA2878828},
	{SG50290, &UA2994965},
	{SG50329, &UA2241249},
	{SG50334, &UA2452337},
	{SG50343, &UA2602038},
	{SG50348, &UA2662598},
	{SG50359, &UA2796243},
	{SG50365, &UA2957611},
	{SG50371, &UA3021791},
	{SG50375, &UA3078485},
	{SG50377, &UA3090239},
	{SG50378, &UA3110570},
	{SG50380, &UA3132925},
	{SG50400, &UA2600190},
	{SG50405, &UA2635329},
	{SG50407, &UA2652357},
	{SG50408, &UA2659470},
	{SG50410, &UA2733424},
	{SG50411, &UA2741563},
	{SG50413, &UA2748494},
	{SG50414, &UA2783450},
	{SG50421, &UA2958403},
	{SG50423, &UA2962874},
	{SG50431, &UA3023267},
	{SG50438, &UA3074219},
	{SG50439, &UA3125763},
	{SG50440, &UA3102149},
	{SG50441, &UA3120362},
	{SG50480, &UA2603993},
	{SG50486, &UA2696145},
	{SG50489, &UA2711922},
	{SG50490, &UA2732681},
	{SG50497, &UA2793827},
	{SG50500, &UA2800286},
	{SG50506, &UA2868371},
	{SG50512, &UA2957631},
	{SG50517, &UA3006673},
	{SG50518, &UA3006952},
	{SG50521, &UA3023273},
	{SG50538, &UA2506639},
	{SG50595, &UA2713514},
	{SG50597, &UA2714307},
	{SG50600, &UA2791545},
	{SG50602, &UA2799133},
	{SG50604, &UA2800089},
	{SG50624, &UA3087564},
	{SG50629, &UA3110889},
	{SG50651, &UA2666875},
	{SG50657, &UA2735021},
	{SG50659, &UA2748393},
	{SG50660, &UA2749049},
	{SG50662, &UA2793515},
	{SG50664, &UA2867728},
	{SG50667, &UA2879563},
	{SG50675, &UA2998160},
	{SG50676, &UA3024111},
	{SG50679, &UA3051829},
	{SG50683, &UA3086436},
	{SG50684, &UA3099932},
	{SG50687, &UA3118518},
	{SG50689, &UA3109629},
	{SG50752, &UA2352559},
	{SG50769, &UA2580558},
	{SG50771, &UA2604127},
	{SG50784, &UA2730746},
	{SG50787, &UA2751703},
	{SG50790, &UA2799367},
	{SG50792, &UA2857159},
	{SG50796, &UA2868695},
	{SG50799, &UA2879449},
	{SG50800, &UA2893951},
	{SG50802, &UA2946016},
	{SG50811, &UA3110453},
	{SG50859, &UA2696619},
	{SG50861, &UA2707393},
	{SG50863, &UA2710831},
	{SG50866, &UA2730837},
	{SG50871, &UA2877168},
	{SG50872, &UA2879309},
	{SG50874, &UA2938275},
	{SG50876, &UA2990765},
	{SG50877, &UA2998058},
	{SG50878, &UA2999355},
	{SG50882, &UA3021242},
	{SG50927, &UA2596104},
	{SG50935, &UA2667121},
	{SG50941, &UA2711809},
	{SG50944, &UA2729095},
	{SG50951, &UA2848083},
	{SG50961, &UA2953983},
	{SG50965, &UA2989490},
	{SG50970, &UA3022599},
	{SG50975, &UA3113527},
	{SG51005, &UA2528501},
	{SG51007, &UA2563811},
	{SG51014, &UA2734628},
	{SG51015, &UA2798628},
	{SG51019, &UA2997940},
	{SG51023, &UA3019024},
	{SG51024, &UA3019029},
	{SG51027, &UA3059033},
	{SG51064, &UA2482515},
	{SG51072, &UA2633905},
	{SG51078, &UA2709419},
	{SG51082, &UA2734958},
	{SG51087, &UA2840691},
	{SG51089, &UA2852268},
	{SG51092, &UA2868421},
	{SG51095, &UA2878288},
	{SG51096, &UA2959928},
	{SG51102, &UA3022140},
	{SG51112, &UA3059863},
	{SG51120, &UA3125490},
	{SG51139, &UA2529618},
	{SG51140, &UA2594843},
	{SG51143, &UA2666716},
	{SG51148, &UA2693495},
	{SG51154, &UA2708763},
	{SG51156, &UA2709112},
	{SG51160, &UA2751246},
	{SG51161, &UA2798973},
	{SG51165, &UA2869638},
	{SG51166, &UA2875476},
	{SG51167, &UA2879412},
	{SG51172, &UA3023517},
	{SG51175, &UA3048459},
	{SG51177, &UA3094920},
	{SG51180, &UA3111154},
	{SG51181, &UA3115314},
	{SG51205, &UA2481455},
	{SG51209, &UA2604119},
	{SG51213, &UA2700964},
	{SG51216, &UA2713940},
	{SG51221, &UA2734244},
	{SG51226, &UA2795237},
	{SG51230, &UA2849727},
	{SG51231, &UA2862302},
	{SG51233, &UA2867066},
	{SG51239, &UA2879166},
	{SG51243, &UA2897411},
	{SG51257, &UA3013309},
	{SG51259, &UA3021033},
	{SG51262, &UA3021790},
	{SG51269, &UA3023718},
	{SG51277, &UA3110732},
	{SG51279, &UA3112887},
	{SG51307, &UA2700827},
	{SG51309, &UA2707191},
	{SG51311, &UA2728566},
	{SG51322, &UA2850181},
	{SG51326, &UA2856092},
	{SG51331, &UA2879245},
	{SG51333, &UA2954359},
	{SG51338, &UA2999257},
	{SG51341, &UA3004882},
	{SG51346, &UA3021465},
	{SG51348, &UA3022852},
	{SG51349, &UA3023100},
	{SG51379, &UA2481693},
	{SG51383, &UA2526987},
	{SG51388, &UA2601243},
	{SG51391, &UA2667588},
	{SG51398, &UA2745421},
	{SG51400, &UA2751951},
	{SG51402, &UA2780431},
	{SG51406, &UA2847290},
	{SG51413, &UA2867279},
	{SG51416, &UA2869776},
	{SG51419, &UA2879103},
	{SG51422, &UA2879618},
	{SG51428, &UA2938060},
	{SG51430, &UA2940356},
	{SG51444, &UA3008997},
	{SG51449, &UA3021544},
	{SG51452, &UA3022408},
	{SG51455, &UA3049573},
	{SG51459, &UA3077490},
	{SG51467, &UA3110877},
	{SG51472, &UA3126257},
	{SG51552, &UA2555174},
	{SG51555, &UA2665409},
	{SG51564, &UA2762381},
	{SG51566, &UA2793647},
	{SG51567, &UA2877665},
	{SG51569, &UA2879541},
	{SG51570, &UA2892647},
	{SG51572, &UA2896881},
	{SG51575, &UA2955009},
	{SG51577, &UA2958545},
	{SG51580, &UA2985670},
	{SG51583, &UA2992535},
	{SG51587, &UA2999361},
	{SG51591, &UA3054206},
	{SG51634, &UA3111482},
	{SG51636, &UA3113439},
	{SG51658, &UA2031006},
	{SG51659, &UA2031360},
	{SG51674, &UA2637860},
	{SG51680, &UA2653703},
	{SG51694, &UA2733218},
	{SG51698, &UA2764245},
	{SG51702, &UA2865570},
	{SG51703, &UA2866853},
	{SG51712, &UA2893333},
	{SG51722, &UA2990237},
	{SG51730, &UA2998632},
	{SG51739, &UA3023592},
	{SG51747, &UA3093236},
	{SG51749, &UA3116518},
	{SG51788, &UA2029159},
	{SG51791, &UA2644413},
	{SG51795, &UA2461079},
	{SG51799, &UA2531173},
	{SG51807, &UA2627418},
	{SG51809, &UA2630467},
	{SG51816, &UA2666946},
	{SG51823, &UA2708722},
	{SG51843, &UA2876350},
	{SG51848, &UA2937896},
	{SG51853, &UA2957378},
	{SG51856, &UA2987069},
	{SG51863, &UA3016186},
	{SG51868, &UA3024273},
	{SG51876, &UA3057746},
	{SG51889, &UA3131750},
	{SG51921, &UA2631379},
	{SG51928, &UA2688570},
	{SG51939, &UA2737008},
	{SG51959, &UA2893173},
	{SG51961, &UA2937513},
	{SG51964, &UA2988052},
	{SG51965, &UA3003279},
	{SG51966, &UA3015531},
	{SG51967, &UA3021192},
	{SG51968, &UA3023758},
	{SG51969, &UA3049371},
	{SG51970, &UA3058550},
	{SG51977, &UA3087858},
	{SG52008, &UA2013742},
	{SG52009, &UA2029921},
	{SG52027, &UA2657244},
	{SG52040, &UA2762670},
	{SG52044, &UA2791622},
	{SG52057, &UA2878419},
	{SG52062, &UA2938135},
	{SG52072, &UA2995430},
	{SG52076, &UA2999316},
	{SG52080, &UA3021081},
	{SG52087, &UA3045630},
	{SG52089, &UA3056815},
	{SG52091, &UA3060069},
	{SG52100, &UA3109967},
	{SG52127, &UA2536529},
	{SG52138, &UA2712211},
	{SG52143, &UA2736501},
	{SG52145, &UA2739406},
	{SG52152, &UA2861676},
	{SG52154, &UA2864439},
	{SG52156, &UA2864896},
	{SG52160, &UA2987065},
	{SG52161, &UA2987844},
	{SG52163, &UA2995337},
	{SG52189, &UA3079343},
	{SG52191, &UA3091964},
	{SG52194, &UA3109336},
	{SG52196, &UA3126567},
	{SG52199, &UA3132885},
	{SG52200, &UA3132918},
	{SG52219, &UA2241281},
	{SG52246, &UA2734872},
	{SG52247, &UA2735860},
	{SG52250, &UA2748068},
	{SG52251, &UA2750004},
	{SG52254, &UA2861522},
	{SG52255, &UA2863703},
	{SG52258, &UA2879448},
	{SG52260, &UA2892999},
	{SG52262, &UA2935149},
	{SG52264, &UA2937485},
	{SG52266, &UA2943064},
	{SG52267, &UA2952580},
	{SG52271, &UA2958363},
	{SG52278, &UA2989034},
	{SG52287, &UA3001959},
	{SG52289, &UA3010418},
	{SG52295, &UA3018369},
	{SG52300, &UA3023857},
	{SG52301, &UA3046930},
	{SG52302, &UA3050117},
	{SG52303, &UA3061401},
	{SG52304, &UA3079553},
	{SG52305, &UA3087878},
	{SG52306, &UA3093542},
	{SG52307, &UA3101472},
	{SG52313, &UA3124919},
	{SG52354, &UA3127255},
	{SG52355, &UA3127426},
	{SG52376, &UA2030493},
	{SG52377, &UA2030905},
	{SG52381, &UA2487588},
	{SG52384, &UA2562291},
	{SG52389, &UA2636125},
	{SG52398, &UA2707786},
	{SG52401, &UA2738363},
	{SG52411, &UA2869431},
	{SG52412, &UA2879620},
	{SG52414, &UA2937615},
	{SG52415, &UA2937886},
	{SG52417, &UA2938139},
	{SG52418, &UA2938189},
	{SG52423, &UA2987426},
	{SG52428, &UA2989454},
	{SG52433, &UA3000022},
	{SG52435, &UA3000083},
	{SG52438, &UA3004879},
	{SG52440, &UA3005893},
	{SG52443, &UA3008745},
	{SG52446, &UA3013464},
	{SG52460, &UA3055263},
	{SG52462, &UA3074251},
	{SG52463, &UA3074348},
	{SG52464, &UA3078391},
	{SG52465, &UA3078474},
	{SG52468, &UA3102257},
	{SG52470, &UA3107798},
	{SG52476, &UA3110912},
	{SG52477, &UA3126225},
	{SG52478, &UA3126368},
	{SG52502, &UA2362503},
	{SG52508, &UA2658012},
	{SG52509, &UA2733200},
	{SG52513, &UA2831351},
	{SG52517, &UA2856845},
	{SG52518, &UA2860334},
	{SG52523, &UA2937872},
	{SG52525, &UA2951946},
	{SG52529, &UA2983634},
	{SG52532, &UA2984192},
	{SG52533, &UA2986939},
	{SG52539, &UA2998035},
	{SG52542, &UA3006678},
	{SG52546, &UA3058148},
	{SG52548, &UA3073956},
	{SG52555, &UA3092987},
	{SG52558, &UA3102948},
	{SG52561, &UA3103578},
	{SG52565, &UA3108931},
	{SG52569, &UA3113516},
	{SG52580, &UA3128429},
	{SG52582, &UA3131450},
	{SG52610, &UA2383436},
	{SG52613, &UA2521725},
	{SG52634, &UA2859821},
	{SG52636, &UA2892522},
	{SG52637, &UA2897416},
	{SG52638, &UA2938068},
	{SG52639, &UA2938228},
	{SG52640, &UA2958199},
	{SG52641, &UA2958590},
	{SG52649, &UA2988135},
	{SG52652, &UA2988829},
	{SG52654, &UA2998521},
	{SG52655, &UA2999023},
	{SG52656, &UA2999186},
	{SG52657, &UA3001366},
	{SG52658, &UA3010027},
	{SG52659, &UA3010326},
	{SG52661, &UA3020860},
	{SG52663, &UA3044829},
	{SG52668, &UA3056511},
	{SG52675, &UA3071540},
	{SG52676, &UA3072648},
	{SG52680, &UA3087478},
	{SG52697, &UA3127275},
	{SG52701, &UA3130955},
	{SG52702, &UA3131707},
	{SG52721, &UA2603205},
	{SG52727, &UA2635400},
	{SG52732, &UA2794203},
	{SG52738, &UA2860021},
	{SG52744, &UA2957860},
	{SG52745, &UA2982563},
	{SG52747, &UA2993894},
	{SG52752, &UA3000797},
	{SG52757, &UA3024118},
	{SG52759, &UA3046118},
	{SG52762, &UA3055445},
	{SG52768, &UA3072337},
	{SG52779, &UA3076315},
	{SG52781, &UA3087546},
	{SG52782, &UA3087899},
	{SG52783, &UA3088571},
	{SG52786, &UA3100254},
	{SG52789, &UA3101666},
	{SG52790, &UA3102912},
	{SG52798, &UA3118908},
	{SG52800, &UA3119030},
	{SG52801, &UA3120388},
	{SG52802, &UA3120911},
	{SG52811, &UA3126900},
	{SG52842, &UA3130694},
	{SG52869, &UA2479104},
	{SG52871, &UA2557408},
	{SG52877, &UA2665198},
	{SG52878, &UA2667552},
	{SG52887, &UA2852061},
	{SG52896, &UA2864047},
	{SG52899, &UA2952317},
	{SG52901, &UA2958345},
	{SG52904, &UA2986422},
	{SG52906, &UA2997469},
	{SG52907, &UA2998796},
	{SG52908, &UA3000835},
	{SG52909, &UA3046643},
	{SG52910, &UA3047586},
	{SG52913, &UA3058292},
	{SG52915, &UA3074152},
	{SG52920, &UA3092349},
	{SG52924, &UA3099831},
	{SG52925, &UA3099982},
	{SG52930, &UA3102193},
	{SG52931, &UA3102381},
	{SG52933, &UA3105908},
	{SG52939, &UA3108532},
	{SG52948, &UA3119523},
	{SG52954, &UA3121728},
	{SG52958, &UA3125227},
	{SG52960, &UA3125330},
	{SG52963, &UA3125653},
	{SG52968, &UA3127192},
	{SG53008, &UA2010490},
	{SG53012, &UA2230807},
	{SG53016, &UA2623124},
	{SG53019, &UA2623641},
	{SG53032, &UA2845245},
	{SG53042, &UA2861170},
	{SG53045, &UA2862854},
	{SG53076, &UA2347110},
	{SG53081, &UA2635057},
	{SG53113, &UA2419571},
	{SG53177, &UA2199028},
	{SG53179, &UA2203515},
	{SG53181, &UA2417802},
	{SG53186, &UA2630995},
	{SG53198, &UA2241306},
	{SG53202, &UA2628647},
	{SG53220, &UA2031976},
	{SG53223, &UA2032708},
	{SG53232, &UA2633015},
	{SG53276, &UA2033254},
	{SG53279, &UA2418803},
	{SG53285, &UA2627163},
	{SG53286, &UA2629059},
	{SG53288, &UA2632927},
	{SG53289, &UA2633308},
	{SG53337, &UA2623011},
	{SG53340, &UA2627202},
	{SG53341, &UA2628151},
	{SG53344, &UA2633441},
	{SG53394, &UA2616740},
	{SG53395, &UA2629303},
	{SG53396, &UA2625792},
	{SG53397, &UA2626715},
	{SG53398, &UA2628621},
	{SG53429, &UA2030563},
	{SG53431, &UA2033389},
	{SG53436, &UA2632091},
	{SG53437, &UA2621796},
	{SG53439, &UA2630006},
	{SG53443, &UA2621734},
	{SG53446, &UA2625319},
	{SG53492, &UA2476145},
	{SG53494, &UA2618326},
	{SG53495, &UA2633330},
	{SG53496, &UA2624056},
	{SG53497, &UA2633124},
	{SG53501, &UA2633923},
	{SG53502, &UA2624768},
	{SG53503, &UA2634479},
	{SG53504, &UA2635266},
	{SG53505, &UA2636937},
	{SG53528, &UA2032860},
	{SG53536, &UA2625660},
	{SG53537, &UA2629832},
	{SG53539, &UA2628725},
	{SG53540, &UA2630033},
	{SG53541, &UA2635493},
	{SG53542, &UA2636641},
	{SG53562, &UA2241157},
	{SG53568, &UA2624102},
	{SG53569, &UA2624360},
	{SG53570, &UA2625759},
	{SG53571, &UA2631105},
	{SG53572, &UA2626004},
	{SG53573, &UA2626352},
	{SG53574, &UA2627023},
	{SG53578, &UA2628044},
	{SG53579, &UA2628899},
	{SG53580, &UA2636184},
	{SG53582, &UA2632204},
	{SG53583, &UA2634806},
	{SG53603, &UA2242126},
	{SG53618, &UA2627431},
	{SG53619, &UA2627723},
	{SG53621, &UA2629522},
	{SG53624, &UA2632278},
	{SG53643, &UA2637849},
	{SG53683, &UA2636896},
	{SG53689, &UA2620819},
	{SG53692, &UA2630149},
	{SG53693, &UA2630405},
	{SG53694, &UA2630406},
	{SG53695, &UA2631222},
	{SG53696, &UA2633348},
	{SG53697, &UA2634214},
	{SG53701, &UA2635249},
	{SG53702, &UA2636906},
	{SG53704, &UA2637977},
	{SG53718, &UA3084051},
	{SG53720, &UA3084072},
	{SG53862, &UA2032491},
	{SG53865, &UA2044193},
	{SG53867, &UA2044224},
	{SG53869, &UA2241317},
	{SG53870, &UA2241335},
	{SG53888, &UA2616576},
	{SG53907, &UA2624069},
	{SG53909, &UA2624081},
	{SG53911, &UA2624826},
	{SG53914, &UA2627930},
	{SG53916, &UA2628127},
	{SG53923, &UA2630167},
	{SG53925, &UA2630596},
	{SG53926, &UA2630642},
	{SG53933, &UA2631422},
	{SG53939, &UA2631767},
	{SG53953, &UA2632693},
	{SG53966, &UA2634107},
	{SG53980, &UA2634916},
	{SG53983, &UA2634952},
	{SG53993, &UA2635596},
	{SG53996, &UA2635615},
	{SG54026, &UA2636344},
	{SG54034, &UA2636543},
	{SG54039, &UA2636746},
	{SG54046, &UA2637004},
	{SG54049, &UA2637023},
	{SG54056, &UA2637447},
	{NULL}};
const Segmentdata DAndroidVersion4_2[] = {
	{SG29788, &UA2359229},
	{SG29789, &UA2362759},
	{SG29790, &UA2386304},
	{SG29791, &UA2389111},
	{SG30769, &UA2359326},
	{SG30770, &UA2708723},
	{SG31242, &UA2241438},
	{SG31243, &UA2241439},
	{SG31244, &UA2241668},
	{SG31245, &UA2241699},
	{SG31246, &UA2241731},
	{SG31247, &UA2241738},
	{SG31248, &UA2242011},
	{SG31249, &UA2359438},
	{SG31250, &UA2362748},
	{SG31251, &UA2404860},
	{SG31916, &UA2746930},
	{SG31917, &UA2863746},
	{SG31918, &UA2995836},
	{SG31919, &UA3046446},
	{SG31937, &UA2362197},
	{SG32041, &UA2566439},
	{SG32213, &UA2359567},
	{SG32251, &UA2567104},
	{SG32327, &UA2973474},
	{SG32493, &UA2974071},
	{SG32500, &UA2362952},
	{SG32503, &UA2567225},
	{SG32625, &UA2240800},
	{SG32626, &UA2240801},
	{SG32627, &UA2240802},
	{SG32628, &UA2240803},
	{SG32629, &UA2240804},
	{SG32630, &UA2240805},
	{SG32631, &UA2240806},
	{SG32633, &UA2240817},
	{SG32634, &UA2362351},
	{SG32635, &UA2387832},
	{SG32665, &UA3020968},
	{SG32891, &UA2754380},
	{SG32919, &UA2753068},
	{SG32963, &UA2773548},
	{SG33189, &UA2677296},
	{SG33239, &UA2672602},
	{SG33293, &UA2865708},
	{SG33317, &UA3061042},
	{SG33339, &UA2676241},
	{SG33930, &UA3065599},
	{SG34355, &UA3081674},
	{SG34424, &UA2360583},
	{SG34427, &UA2429693},
	{SG34432, &UA2527900},
	{SG34437, &UA2982207},
	{SG34673, &UA2599949},
	{SG34910, &UA2407285},
	{SG34913, &UA2600563},
	{SG34914, &UA2727809},
	{SG34918, &UA2792268},
	{SG35185, &UA3023136},
	{SG35342, &UA2782798},
	{SG35531, &UA2621572},
	{SG35532, &UA2623136},
	{SG35742, &UA2676486},
	{SG35843, &UA2695815},
	{SG36029, &UA2431361},
	{SG36032, &UA2847757},
	{SG37764, &UA3082765},
	{SG38125, &UA2391606},
	{SG38945, &UA2388268},
	{SG38947, &UA2426756},
	{SG38949, &UA2482795},
	{SG38957, &UA2734711},
	{SG39016, &UA2367192},
	{SG39163, &UA3023419},
	{SG39379, &UA2423749},
	{SG39380, &UA2988009},
	{SG39387, &UA3099973},
	{SG40362, &UA2958030},
	{SG40420, &UA2708383},
	{SG40450, &UA2621899},
	{SG40645, &UA2433636},
	{SG40807, &UA2422724},
	{SG41247, &UA2362522},
	{SG41551, &UA2637993},
	{SG41743, &UA2799206},
	{SG41923, &UA2600042},
	{SG42230, &UA3022517},
	{SG42266, &UA3131415},
	{SG42290, &UA2709091},
	{SG42375, &UA2507038},
	{SG42377, &UA2507710},
	{SG42740, &UA3099842},
	{SG44046, &UA3077909},
	{SG45275, &UA2868414},
	{SG46258, &UA2387215},
	{SG46259, &UA2404450},
	{SG46740, &UA2404141},
	{SG46887, &UA2759800},
	{SG46888, &UA2729091},
	{SG48029, &UA2635879},
	{SG48523, &UA2505784},
	{SG49093, &UA2485249},
	{SG49823, &UA2619085},
	{SG50165, &UA2860095},
	{SG51903, &UA2030479},
	{SG51980, &UA2604123},
	{SG51987, &UA2734645},
	{SG54213, &UA3084052},
	{NULL}};
const Segmentdata DAndroidVersion4_2_1[] = {
	{SG30206, &UA2369730},
	{SG30207, &UA2388147},
	{SG30208, &UA2388416},
	{SG30209, &UA2406159},
	{SG30211, &UA2407202},
	{SG30213, &UA2505251},
	{SG30214, &UA2511780},
	{SG30216, &UA2596091},
	{SG30218, &UA2654855},
	{SG30220, &UA2792465},
	{SG31084, &UA2368596},
	{SG31085, &UA2467553},
	{SG31087, &UA2711116},
	{SG31089, &UA2846580},
	{SG31653, &UA2387681},
	{SG31655, &UA2390878},
	{SG31657, &UA2426880},
	{SG31660, &UA2467059},
	{SG31661, &UA2467426},
	{SG31662, &UA2513180},
	{SG31663, &UA3079853},
	{SG31707, &UA2892385},
	{SG31714, &UA3126246},
	{SG31821, &UA2567832},
	{SG31822, &UA2567842},
	{SG31823, &UA2714685},
	{SG31824, &UA2717085},
	{SG31825, &UA2738350},
	{SG31826, &UA2781268},
	{SG31827, &UA2850491},
	{SG31828, &UA2989171},
	{SG31829, &UA3049619},
	{SG31875, &UA2540494},
	{SG32240, &UA2505936},
	{SG32241, &UA2511267},
	{SG32242, &UA2701585},
	{SG32244, &UA2751663},
	{SG32245, &UA2752239},
	{SG32246, &UA2768683},
	{SG32247, &UA2781864},
	{SG32248, &UA2793896},
	{SG32249, &UA2794511},
	{SG32250, &UA2799314},
	{SG32252, &UA2799356},
	{SG32253, &UA2859890},
	{SG32254, &UA2860038},
	{SG32255, &UA2864133},
	{SG32256, &UA2865537},
	{SG32257, &UA2865626},
	{SG32258, &UA2866891},
	{SG32259, &UA2868461},
	{SG32260, &UA2869289},
	{SG32261, &UA2878942},
	{SG32262, &UA2892397},
	{SG32263, &UA2893109},
	{SG32264, &UA2936358},
	{SG32265, &UA2936699},
	{SG32266, &UA2937069},
	{SG32267, &UA2937564},
	{SG32268, &UA2937568},
	{SG32269, &UA2937783},
	{SG32270, &UA2937838},
	{SG32271, &UA2955905},
	{SG32272, &UA2955952},
	{SG32273, &UA2956290},
	{SG32274, &UA2956736},
	{SG32275, &UA2957128},
	{SG32276, &UA2957183},
	{SG32277, &UA2957449},
	{SG32278, &UA2957694},
	{SG32279, &UA2958161},
	{SG32280, &UA2958188},
	{SG32283, &UA2958603},
	{SG32284, &UA2958983},
	{SG32285, &UA2961526},
	{SG32287, &UA2962954},
	{SG32288, &UA2964650},
	{SG32289, &UA2986895},
	{SG32290, &UA2989164},
	{SG32291, &UA2990059},
	{SG32292, &UA2993808},
	{SG32293, &UA2994038},
	{SG32294, &UA2994551},
	{SG32295, &UA2995355},
	{SG32296, &UA2996018},
	{SG32297, &UA2996235},
	{SG32298, &UA2996579},
	{SG32299, &UA2996654},
	{SG32300, &UA2996781},
	{SG32302, &UA2997060},
	{SG32304, &UA2997328},
	{SG32306, &UA2997468},
	{SG32308, &UA2997598},
	{SG32310, &UA2997662},
	{SG32311, &UA2997828},
	{SG32313, &UA2997843},
	{SG32315, &UA2998050},
	{SG32317, &UA2998175},
	{SG32318, &UA2998371},
	{SG32319, &UA2998384},
	{SG32320, &UA2998740},
	{SG32321, &UA2998976},
	{SG32322, &UA2999028},
	{SG32323, &UA2999304},
	{SG32324, &UA2999358},
	{SG32325, &UA2999365},
	{SG32326, &UA2999554},
	{SG32328, &UA3000836},
	{SG32330, &UA3002372},
	{SG32332, &UA3002947},
	{SG32334, &UA3002970},
	{SG32336, &UA3003749},
	{SG32338, &UA3004772},
	{SG32340, &UA3005843},
	{SG32344, &UA3007202},
	{SG32346, &UA3007715},
	{SG32347, &UA3009095},
	{SG32349, &UA3010505},
	{SG32352, &UA3011591},
	{SG32354, &UA3011889},
	{SG32357, &UA3012480},
	{SG32359, &UA3014957},
	{SG32362, &UA3016171},
	{SG32364, &UA3018694},
	{SG32365, &UA3021240},
	{SG32366, &UA3021249},
	{SG32367, &UA3027012},
	{SG32368, &UA3027496},
	{SG32369, &UA3034255},
	{SG32370, &UA3034561},
	{SG32371, &UA3047692},
	{SG32372, &UA3047895},
	{SG32373, &UA3048522},
	{SG32374, &UA3048683},
	{SG32375, &UA3048957},
	{SG32376, &UA3049298},
	{SG32379, &UA3050105},
	{SG32381, &UA3050186},
	{SG32382, &UA3050233},
	{SG32384, &UA3050245},
	{SG32386, &UA3050302},
	{SG32388, &UA3050381},
	{SG32391, &UA3050453},
	{SG32394, &UA3050456},
	{SG32396, &UA3050512},
	{SG32398, &UA3050932},
	{SG32399, &UA3051571},
	{SG32401, &UA3054505},
	{SG32402, &UA3054762},
	{SG32403, &UA3054899},
	{SG32404, &UA3056992},
	{SG32405, &UA3057765},
	{SG32406, &UA3057824},
	{SG32407, &UA3057849},
	{SG32408, &UA3057869},
	{SG32409, &UA3058005},
	{SG32410, &UA3058201},
	{SG32411, &UA3058396},
	{SG32412, &UA3058745},
	{SG32413, &UA3058800},
	{SG32414, &UA3058981},
	{SG32415, &UA3059204},
	{SG32416, &UA3059508},
	{SG32417, &UA3072823},
	{SG32418, &UA3073411},
	{SG32419, &UA3073759},
	{SG32420, &UA3073991},
	{SG32424, &UA3074262},
	{SG32427, &UA3077626},
	{SG32431, &UA3078047},
	{SG32436, &UA3078423},
	{SG32439, &UA3078876},
	{SG32441, &UA3079047},
	{SG32443, &UA3079637},
	{SG32446, &UA3079781},
	{SG32447, &UA3079864},
	{SG32448, &UA3080251},
	{SG32449, &UA3080397},
	{SG32450, &UA3086515},
	{SG32451, &UA3087314},
	{SG32452, &UA3088053},
	{SG32454, &UA3088958},
	{SG32457, &UA3089055},
	{SG32458, &UA3091681},
	{SG32459, &UA3091896},
	{SG32460, &UA3092021},
	{SG32461, &UA3092148},
	{SG32462, &UA3092179},
	{SG32463, &UA3092228},
	{SG32464, &UA3092357},
	{SG32465, &UA3092642},
	{SG32466, &UA3092780},
	{SG32467, &UA3093060},
	{SG32470, &UA3101763},
	{SG32472, &UA3102315},
	{SG32473, &UA3102355},
	{SG32475, &UA3102809},
	{SG32476, &UA3103015},
	{SG32478, &UA3103384},
	{SG32480, &UA3103394},
	{SG32482, &UA3103638},
	{SG32483, &UA3104879},
	{SG32485, &UA3105020},
	{SG32487, &UA3106102},
	{SG32490, &UA3106881},
	{SG32491, &UA3107091},
	{SG32492, &UA3107383},
	{SG32494, &UA3108075},
	{SG32495, &UA3108131},
	{SG32496, &UA3108158},
	{SG32497, &UA3108163},
	{SG32498, &UA3108175},
	{SG32499, &UA3108250},
	{SG32501, &UA3108801},
	{SG32502, &UA3109166},
	{SG32504, &UA3120239},
	{SG32506, &UA3120250},
	{SG32507, &UA3120624},
	{SG32508, &UA3120918},
	{SG32509, &UA3121201},
	{SG32510, &UA3121387},
	{SG32511, &UA3121421},
	{SG32513, &UA3124260},
	{SG32515, &UA3125079},
	{SG32517, &UA3125157},
	{SG32518, &UA3125501},
	{SG32520, &UA3125585},
	{SG32522, &UA3125716},
	{SG32524, &UA3125734},
	{SG32526, &UA3125744},
	{SG32528, &UA3125778},
	{SG32531, &UA3125894},
	{SG32533, &UA3126103},
	{SG32534, &UA3127095},
	{SG32535, &UA2389104},
	{SG32536, &UA2483444},
	{SG32537, &UA2709329},
	{SG32696, &UA2566706},
	{SG32795, &UA2429717},
	{SG32796, &UA2431753},
	{SG32797, &UA2433908},
	{SG32798, &UA2512378},
	{SG32800, &UA2513264},
	{SG32801, &UA2516387},
	{SG32876, &UA2368289},
	{SG32877, &UA2423288},
	{SG32952, &UA2457650},
	{SG32953, &UA2566545},
	{SG32954, &UA2567242},
	{SG32969, &UA2974788},
	{SG33026, &UA2975007},
	{SG33028, &UA2391597},
	{SG33029, &UA2708114},
	{SG33030, &UA2566820},
	{SG33031, &UA2602446},
	{SG33130, &UA2429428},
	{SG33132, &UA2460342},
	{SG33135, &UA2514519},
	{SG33138, &UA2731043},
	{SG33158, &UA2391181},
	{SG33160, &UA2405366},
	{SG33163, &UA2511518},
	{SG33166, &UA2429918},
	{SG33168, &UA2459861},
	{SG33171, &UA2460053},
	{SG33173, &UA2467770},
	{SG33175, &UA2515531},
	{SG33178, &UA2537696},
	{SG33181, &UA2655536},
	{SG33182, &UA2776853},
	{SG33183, &UA2845183},
	{SG33184, &UA2985625},
	{SG33228, &UA2974283},
	{SG33274, &UA2540495},
	{SG33280, &UA2782110},
	{SG33285, &UA2996595},
	{SG33611, &UA2754719},
	{SG33679, &UA2754712},
	{SG33746, &UA2741595},
	{SG33881, &UA2676247},
	{SG33883, &UA2676842},
	{SG33943, &UA2437262},
	{SG33944, &UA2437282},
	{SG33945, &UA2468948},
	{SG33946, &UA2512077},
	{SG33966, &UA2513567},
	{SG33968, &UA2559171},
	{SG33995, &UA2676450},
	{SG33996, &UA2708422},
	{SG34043, &UA3064768},
	{SG34084, &UA2676013},
	{SG34087, &UA2657943},
	{SG34088, &UA2675421},
	{SG34089, &UA3049211},
	{SG34237, &UA2433749},
	{SG34239, &UA2514120},
	{SG34328, &UA2429872},
	{SG34330, &UA2437312},
	{SG34332, &UA2505109},
	{SG34334, &UA2510785},
	{SG34444, &UA2429873},
	{SG34445, &UA2515261},
	{SG34517, &UA3094526},
	{SG34519, &UA2427434},
	{SG34522, &UA2765165},
	{SG34696, &UA3065329},
	{SG34708, &UA2975981},
	{SG34709, &UA3022969},
	{SG34878, &UA2677127},
	{SG34879, &UA2714279},
	{SG35148, &UA3081112},
	{SG35224, &UA2621685},
	{SG35225, &UA2636054},
	{SG35236, &UA3080414},
	{SG35240, &UA2390938},
	{SG35241, &UA2405787},
	{SG35242, &UA2507722},
	{SG35243, &UA2536167},
	{SG35244, &UA2431371},
	{SG35247, &UA2510121},
	{SG35314, &UA2755535},
	{SG35316, &UA2783590},
	{SG35317, &UA2800378},
	{SG35331, &UA2436626},
	{SG35333, &UA2486260},
	{SG35335, &UA2560280},
	{SG35337, &UA2712284},
	{SG35339, &UA2738165},
	{SG35340, &UA2865483},
	{SG35396, &UA3021643},
	{SG35437, &UA3114789},
	{SG35450, &UA2567345},
	{SG35501, &UA2974014},
	{SG35502, &UA2974477},
	{SG35503, &UA2975942},
	{SG35505, &UA3022791},
	{SG35548, &UA3021501},
	{SG35549, &UA3022819},
	{SG35561, &UA2660594},
	{SG35563, &UA2698373},
	{SG35566, &UA2709550},
	{SG35569, &UA2734029},
	{SG35570, &UA2734530},
	{SG35573, &UA2735048},
	{SG35575, &UA2735352},
	{SG35576, &UA2736772},
	{SG35577, &UA2768043},
	{SG35579, &UA2796215},
	{SG35582, &UA2878960},
	{SG35583, &UA2961357},
	{SG35584, &UA2989037},
	{SG35585, &UA2994701},
	{SG35586, &UA3013922},
	{SG35589, &UA3057514},
	{SG35590, &UA3072801},
	{SG35591, &UA3088073},
	{SG35592, &UA3093164},
	{SG35593, &UA3093198},
	{SG35596, &UA3101129},
	{SG35832, &UA2431035},
	{SG35853, &UA2467473},
	{SG35854, &UA2486058},
	{SG35855, &UA2506883},
	{SG35856, &UA2531270},
	{SG35858, &UA2535641},
	{SG35859, &UA2537567},
	{SG35864, &UA2845549},
	{SG35865, &UA2985219},
	{SG35873, &UA3022787},
	{SG35952, &UA3022946},
	{SG36015, &UA2670616},
	{SG36018, &UA2677172},
	{SG36020, &UA2713396},
	{SG36097, &UA2773997},
	{SG36265, &UA2769372},
	{SG36266, &UA2773962},
	{SG36268, &UA2870034},
	{SG36280, &UA2566497},
	{SG36282, &UA2709212},
	{SG36284, &UA2750963},
	{SG36286, &UA2781969},
	{SG36302, &UA2566806},
	{SG36304, &UA2567114},
	{SG36305, &UA2583417},
	{SG36322, &UA2435852},
	{SG36323, &UA2512624},
	{SG36326, &UA2703819},
	{SG36328, &UA2728869},
	{SG36346, &UA2622921},
	{SG36348, &UA2623225},
	{SG36350, &UA2632535},
	{SG36435, &UA3083757},
	{SG36437, &UA3112420},
	{SG36579, &UA2741712},
	{SG36580, &UA2754330},
	{SG36581, &UA2754378},
	{SG36582, &UA2892728},
	{SG36619, &UA2676686},
	{SG36679, &UA2408066},
	{SG36680, &UA2457644},
	{SG36681, &UA2563857},
	{SG36682, &UA2863592},
	{SG36717, &UA2670624},
	{SG36719, &UA2677180},
	{SG36720, &UA2708867},
	{SG36755, &UA2676274},
	{SG36756, &UA2712921},
	{SG36757, &UA2718006},
	{SG36789, &UA3062708},
	{SG36792, &UA3080904},
	{SG36794, &UA3080928},
	{SG36796, &UA3081099},
	{SG36798, &UA3081479},
	{SG36801, &UA3082229},
	{SG36803, &UA3083148},
	{SG36805, &UA3083498},
	{SG36808, &UA3093629},
	{SG36811, &UA3094271},
	{SG36818, &UA3094851},
	{SG36821, &UA3108622},
	{SG36823, &UA3109371},
	{SG36826, &UA3109749},
	{SG36827, &UA3110303},
	{SG36828, &UA3110340},
	{SG36829, &UA3110844},
	{SG36830, &UA3110842},
	{SG36831, &UA3111014},
	{SG36832, &UA3111244},
	{SG36833, &UA3111481},
	{SG36834, &UA3111525},
	{SG36835, &UA3111648},
	{SG36836, &UA3112164},
	{SG36837, &UA3112885},
	{SG36838, &UA3113024},
	{SG36842, &UA3113322},
	{SG36846, &UA3113518},
	{SG36849, &UA3113638},
	{SG36852, &UA3114075},
	{SG36856, &UA3114304},
	{SG36858, &UA3114356},
	{SG36860, &UA3114795},
	{SG36862, &UA3115067},
	{SG36864, &UA3126227},
	{SG36866, &UA3126234},
	{SG36869, &UA3126269},
	{SG36871, &UA3126523},
	{SG36872, &UA3126564},
	{SG36873, &UA3126878},
	{SG36874, &UA3126984},
	{SG36875, &UA3127012},
	{SG36876, &UA3127133},
	{SG36877, &UA3127266},
	{SG36878, &UA3127398},
	{SG36879, &UA3127655},
	{SG37111, &UA2460971},
	{SG37112, &UA2490685},
	{SG37113, &UA2511237},
	{SG37247, &UA2565910},
	{SG37252, &UA2437324},
	{SG37280, &UA2424225},
	{SG37281, &UA2426184},
	{SG37282, &UA2426187},
	{SG37285, &UA2435391},
	{SG37287, &UA2507601},
	{SG37316, &UA3064662},
	{SG37320, &UA2428459},
	{SG37321, &UA2428742},
	{SG37323, &UA2430601},
	{SG37325, &UA2460688},
	{SG37358, &UA2974266},
	{SG37369, &UA2973469},
	{SG37482, &UA2469372},
	{SG37485, &UA2469656},
	{SG37486, &UA2985482},
	{SG37519, &UA2567208},
	{SG37521, &UA2570777},
	{SG37523, &UA2604497},
	{SG37647, &UA2508299},
	{SG37648, &UA2509577},
	{SG37705, &UA2437516},
	{SG37706, &UA2558566},
	{SG37709, &UA2633496},
	{SG37750, &UA3065085},
	{SG37925, &UA3083437},
	{SG37940, &UA2754383},
	{SG37942, &UA2893316},
	{SG37947, &UA2470549},
	{SG37948, &UA2532353},
	{SG37979, &UA2866886},
	{SG38144, &UA2435907},
	{SG38145, &UA2509694},
	{SG38146, &UA2492472},
	{SG38147, &UA2531274},
	{SG38148, &UA2580510},
	{SG38151, &UA2778786},
	{SG38153, &UA2866622},
	{SG38155, &UA2878928},
	{SG38156, &UA2889424},
	{SG38159, &UA2994033},
	{SG38170, &UA2567401},
	{SG38171, &UA2667701},
	{SG38190, &UA2460872},
	{SG38191, &UA2536139},
	{SG38192, &UA2712302},
	{SG38248, &UA2516583},
	{SG38260, &UA2676245},
	{SG38261, &UA2734800},
	{SG38280, &UA2741721},
	{SG38298, &UA2974136},
	{SG38303, &UA3065379},
	{SG38330, &UA2507793},
	{SG38331, &UA2600373},
	{SG38332, &UA2662986},
	{SG38334, &UA2878731},
	{SG38336, &UA2607521},
	{SG38374, &UA3006273},
	{SG38375, &UA3014341},
	{SG38411, &UA2437329},
	{SG38413, &UA2532986},
	{SG38414, &UA2624507},
	{SG38415, &UA2630065},
	{SG38435, &UA2433032},
	{SG38436, &UA2662468},
	{SG38457, &UA2437464},
	{SG38460, &UA2516380},
	{SG38509, &UA2533235},
	{SG38651, &UA2486894},
	{SG38654, &UA2515528},
	{SG38658, &UA2435484},
	{SG38659, &UA2514388},
	{SG38660, &UA2599426},
	{SG38672, &UA2750507},
	{SG38675, &UA2869930},
	{SG38799, &UA3083762},
	{SG38806, &UA2879300},
	{SG38822, &UA2437676},
	{SG38823, &UA2535261},
	{SG38831, &UA2469294},
	{SG38832, &UA2507090},
	{SG38833, &UA2536176},
	{SG38854, &UA2423297},
	{SG38855, &UA2426173},
	{SG38865, &UA2515423},
	{SG38886, &UA2676199},
	{SG38917, &UA2531753},
	{SG38959, &UA2487319},
	{SG38960, &UA2556866},
	{SG38961, &UA2675428},
	{SG39009, &UA2484305},
	{SG39011, &UA2851934},
	{SG39012, &UA2432411},
	{SG39014, &UA2514408},
	{SG39034, &UA2433645},
	{SG39035, &UA2533293},
	{SG39129, &UA2434234},
	{SG39131, &UA2534826},
	{SG39141, &UA2429001},
	{SG39258, &UA2624912},
	{SG39260, &UA2629191},
	{SG39261, &UA2752025},
	{SG39327, &UA3064990},
	{SG39511, &UA3021486},
	{SG39525, &UA2892043},
	{SG39526, &UA2959235},
	{SG39672, &UA3020299},
	{SG39674, &UA3021533},
	{SG39675, &UA3021640},
	{SG39676, &UA3021733},
	{SG39678, &UA3021828},
	{SG39680, &UA3022001},
	{SG39681, &UA3022106},
	{SG39682, &UA3022317},
	{SG39683, &UA3022387},
	{SG39684, &UA3022395},
	{SG39685, &UA3022613},
	{SG39686, &UA3022635},
	{SG39687, &UA3022799},
	{SG39688, &UA3023149},
	{SG39689, &UA3023155},
	{SG39690, &UA3023196},
	{SG39692, &UA3023229},
	{SG39695, &UA3023277},
	{SG39697, &UA3023828},
	{SG39699, &UA3023833},
	{SG39700, &UA3023877},
	{SG39702, &UA3024091},
	{SG39703, &UA3024107},
	{SG39705, &UA3027003},
	{SG39707, &UA3030874},
	{SG39709, &UA3058191},
	{SG39712, &UA3107922},
	{SG39714, &UA3124991},
	{SG39738, &UA2460906},
	{SG39741, &UA2629040},
	{SG39744, &UA2424480},
	{SG39747, &UA2459552},
	{SG39749, &UA2559183},
	{SG39824, &UA2622897},
	{SG39826, &UA2735638},
	{SG39829, &UA2754535},
	{SG39952, &UA2406007},
	{SG39954, &UA2536879},
	{SG39955, &UA2555629},
	{SG40020, &UA3132876},
	{SG40027, &UA2461032},
	{SG40029, &UA2562887},
	{SG40075, &UA2708158},
	{SG40077, &UA2737879},
	{SG40079, &UA2866732},
	{SG40104, &UA3021031},
	{SG40105, &UA3021379},
	{SG40106, &UA3021538},
	{SG40107, &UA3022267},
	{SG40108, &UA3023919},
	{SG40109, &UA3023929},
	{SG40119, &UA2676658},
	{SG40142, &UA2428630},
	{SG40143, &UA2504912},
	{SG40144, &UA2531164},
	{SG40145, &UA2600289},
	{SG40203, &UA2868898},
	{SG40207, &UA2965424},
	{SG40208, &UA3016421},
	{SG40281, &UA2531011},
	{SG40282, &UA2750706},
	{SG40283, &UA2953947},
	{SG40284, &UA3019134},
	{SG40285, &UA3057999},
	{SG40286, &UA3090463},
	{SG40366, &UA3101208},
	{SG40395, &UA3068210},
	{SG40414, &UA2469865},
	{SG40499, &UA2768443},
	{SG40500, &UA2998182},
	{SG40502, &UA3107879},
	{SG40506, &UA2796158},
	{SG40573, &UA2936471},
	{SG40585, &UA2772922},
	{SG40599, &UA2460776},
	{SG40608, &UA2486395},
	{SG40609, &UA2629266},
	{SG40670, &UA2671581},
	{SG40676, &UA2621217},
	{SG40677, &UA2622851},
	{SG40678, &UA2636020},
	{SG40773, &UA2752116},
	{SG40787, &UA2974020},
	{SG40808, &UA2737637},
	{SG40809, &UA2999211},
	{SG40810, &UA3006052},
	{SG40852, &UA2752413},
	{SG40855, &UA2996667},
	{SG40889, &UA2529304},
	{SG40890, &UA2622220},
	{SG40892, &UA2624052},
	{SG40893, &UA2630795},
	{SG41008, &UA2489180},
	{SG41009, &UA2513076},
	{SG41175, &UA2430562},
	{SG41191, &UA2468133},
	{SG41194, &UA2661702},
	{SG41251, &UA2487665},
	{SG41408, &UA2680138},
	{SG41463, &UA2783626},
	{SG41465, &UA2800210},
	{SG41466, &UA2800384},
	{SG41468, &UA2893004},
	{SG41469, &UA2937624},
	{SG41471, &UA2998010},
	{SG41506, &UA2870352},
	{SG41509, &UA3050025},
	{SG41538, &UA3065133},
	{SG41558, &UA2436427},
	{SG41676, &UA2530743},
	{SG41693, &UA2945877},
	{SG41730, &UA2798864},
	{SG41732, &UA3056867},
	{SG41740, &UA2514463},
	{SG41792, &UA2973675},
	{SG41828, &UA2781616},
	{SG41830, &UA2957794},
	{SG41847, &UA2434395},
	{SG41849, &UA2507931},
	{SG41856, &UA2583566},
	{SG41865, &UA2605540},
	{SG41986, &UA2469431},
	{SG41987, &UA2563977},
	{SG42005, &UA3022114},
	{SG42018, &UA2422196},
	{SG42021, &UA2583364},
	{SG42024, &UA2433192},
	{SG42049, &UA2624184},
	{SG42054, &UA2406659},
	{SG42100, &UA2431956},
	{SG42101, &UA2599071},
	{SG42134, &UA2530749},
	{SG42173, &UA2973648},
	{SG42182, &UA2407122},
	{SG42285, &UA3059974},
	{SG42337, &UA2770684},
	{SG42347, &UA2655780},
	{SG42436, &UA2865440},
	{SG42438, &UA2996190},
	{SG42440, &UA3022832},
	{SG42442, &UA3023352},
	{SG42445, &UA3101871},
	{SG42449, &UA3111694},
	{SG42450, &UA3121544},
	{SG42454, &UA3125169},
	{SG42490, &UA2388549},
	{SG42524, &UA2490045},
	{SG42526, &UA2704271},
	{SG42539, &UA2865282},
	{SG42540, &UA2997583},
	{SG42544, &UA2897961},
	{SG42545, &UA3106763},
	{SG42550, &UA2896968},
	{SG42552, &UA3103404},
	{SG42556, &UA2422702},
	{SG42606, &UA2852868},
	{SG42618, &UA3023776},
	{SG42629, &UA2437537},
	{SG42630, &UA2531193},
	{SG42642, &UA2428346},
	{SG42644, &UA2469828},
	{SG42666, &UA2425130},
	{SG42668, &UA2436127},
	{SG42676, &UA2468855},
	{SG42679, &UA2748699},
	{SG42693, &UA2557538},
	{SG42711, &UA2584094},
	{SG42716, &UA2869463},
	{SG42717, &UA2943251},
	{SG42719, &UA2955352},
	{SG42720, &UA2958618},
	{SG42721, &UA3077864},
	{SG42749, &UA2631829},
	{SG42756, &UA3021079},
	{SG42781, &UA2877941},
	{SG42792, &UA2485698},
	{SG42802, &UA2709132},
	{SG42813, &UA2467363},
	{SG42815, &UA2699442},
	{SG42820, &UA3110735},
	{SG42860, &UA2987939},
	{SG42863, &UA2633692},
	{SG42980, &UA2505315},
	{SG42982, &UA2536763},
	{SG42989, &UA3010013},
	{SG42991, &UA3091748},
	{SG43010, &UA2958463},
	{SG43011, &UA3106714},
	{SG43046, &UA2944002},
	{SG43047, &UA2770099},
	{SG43048, &UA2869481},
	{SG43084, &UA3110552},
	{SG43188, &UA2707903},
	{SG43192, &UA2782882},
	{SG43193, &UA2797409},
	{SG43196, &UA2879280},
	{SG43197, &UA2892417},
	{SG43200, &UA2892606},
	{SG43203, &UA2892783},
	{SG43204, &UA2893014},
	{SG43206, &UA2894241},
	{SG43207, &UA2937703},
	{SG43210, &UA2937809},
	{SG43212, &UA2937981},
	{SG43215, &UA2957235},
	{SG43217, &UA2997000},
	{SG43222, &UA2958297},
	{SG43224, &UA2877843},
	{SG43249, &UA2782706},
	{SG43273, &UA2468883},
	{SG43274, &UA2659614},
	{SG43287, &UA2567394},
	{SG43311, &UA2768580},
	{SG43313, &UA3073231},
	{SG43350, &UA3065576},
	{SG43365, &UA2507803},
	{SG43367, &UA2515313},
	{SG43378, &UA2670742},
	{SG43423, &UA2633353},
	{SG43426, &UA2507275},
	{SG43429, &UA2713800},
	{SG43459, &UA2892928},
	{SG43461, &UA3058009},
	{SG43482, &UA3023840},
	{SG43489, &UA2469385},
	{SG43492, &UA2531255},
	{SG43536, &UA2961748},
	{SG43537, &UA3078235},
	{SG43596, &UA2512894},
	{SG43597, &UA2537216},
	{SG43610, &UA3132010},
	{SG43613, &UA2891614},
	{SG43631, &UA2735514},
	{SG43659, &UA2600923},
	{SG43661, &UA2861939},
	{SG43668, &UA2959658},
	{SG43669, &UA3124822},
	{SG43674, &UA2709236},
	{SG43685, &UA2422340},
	{SG43686, &UA2599293},
	{SG43695, &UA3068389},
	{SG43702, &UA2507025},
	{SG43707, &UA2735977},
	{SG43709, &UA3088334},
	{SG43711, &UA3021056},
	{SG43781, &UA2755271},
	{SG43816, &UA2486638},
	{SG43817, &UA2565452},
	{SG43819, &UA2782847},
	{SG43860, &UA2934832},
	{SG43861, &UA2876805},
	{SG43897, &UA2557451},
	{SG43899, &UA2729952},
	{SG43920, &UA2861772},
	{SG43928, &UA2752895},
	{SG43929, &UA3073822},
	{SG43968, &UA2783690},
	{SG43974, &UA2430811},
	{SG43976, &UA2487841},
	{SG43999, &UA2675858},
	{SG44003, &UA2433697},
	{SG44006, &UA2407668},
	{SG44007, &UA2983800},
	{SG44009, &UA2435210},
	{SG44010, &UA2484554},
	{SG44032, &UA2658617},
	{SG44058, &UA2426317},
	{SG44064, &UA2633355},
	{SG44081, &UA2624711},
	{SG44104, &UA2580204},
	{SG44115, &UA2707239},
	{SG44206, &UA2667034},
	{SG44300, &UA2782562},
	{SG44301, &UA3002024},
	{SG44324, &UA2713329},
	{SG44334, &UA2975732},
	{SG44353, &UA2956379},
	{SG44367, &UA2459150},
	{SG44369, &UA3023719},
	{SG44385, &UA3056065},
	{SG44386, &UA3076227},
	{SG44387, &UA3078638},
	{SG44389, &UA3106985},
	{SG44413, &UA2598827},
	{SG44419, &UA2508083},
	{SG44453, &UA2987134},
	{SG44454, &UA3107978},
	{SG44462, &UA2664131},
	{SG44600, &UA2795751},
	{SG44601, &UA3050576},
	{SG44615, &UA2458356},
	{SG44618, &UA2850638},
	{SG44707, &UA2938078},
	{SG44708, &UA3077457},
	{SG44754, &UA2508193},
	{SG44757, &UA2583416},
	{SG44772, &UA2423316},
	{SG44787, &UA2601158},
	{SG44788, &UA3083071},
	{SG44789, &UA3083294},
	{SG44811, &UA2713875},
	{SG44812, &UA2715167},
	{SG44813, &UA2718872},
	{SG44834, &UA2879719},
	{SG44836, &UA3103116},
	{SG44864, &UA3126442},
	{SG44875, &UA2799227},
	{SG44883, &UA2556253},
	{SG44910, &UA2867403},
	{SG44913, &UA2957793},
	{SG44914, &UA3100811},
	{SG44975, &UA2512904},
	{SG44992, &UA3060741},
	{SG45016, &UA2434306},
	{SG45048, &UA3082936},
	{SG45053, &UA2964834},
	{SG45164, &UA2667356},
	{SG45174, &UA2938302},
	{SG45177, &UA3106077},
	{SG45202, &UA2751025},
	{SG45213, &UA3083289},
	{SG45214, &UA3113800},
	{SG45242, &UA2998908},
	{SG45268, &UA2697065},
	{SG45269, &UA2869606},
	{SG45280, &UA2974658},
	{SG45287, &UA2798686},
	{SG45307, &UA2507163},
	{SG45382, &UA3082162},
	{SG45436, &UA2879655},
	{SG45446, &UA2437201},
	{SG45476, &UA2735571},
	{SG45477, &UA2859658},
	{SG45478, &UA2996748},
	{SG45516, &UA3067176},
	{SG45545, &UA2963425},
	{SG45614, &UA2504597},
	{SG45615, &UA2635895},
	{SG45617, &UA3080015},
	{SG45630, &UA3021289},
	{SG45671, &UA2506983},
	{SG45691, &UA2750797},
	{SG45775, &UA2942331},
	{SG45806, &UA2423739},
	{SG45843, &UA2974760},
	{SG45844, &UA3078467},
	{SG45954, &UA2622311},
	{SG45955, &UA2625381},
	{SG45956, &UA2628121},
	{SG46002, &UA2879218},
	{SG46025, &UA3019950},
	{SG46027, &UA3071311},
	{SG46040, &UA2769845},
	{SG46056, &UA3083149},
	{SG46061, &UA2533426},
	{SG46063, &UA2850492},
	{SG46065, &UA2487270},
	{SG46088, &UA3132764},
	{SG46106, &UA3081239},
	{SG46108, &UA3081056},
	{SG46120, &UA2976015},
	{SG46121, &UA3023426},
	{SG46209, &UA2487411},
	{SG46260, &UA2974731},
	{SG46290, &UA2768704},
	{SG46292, &UA2937084},
	{SG46293, &UA3049375},
	{SG46301, &UA2667492},
	{SG46333, &UA2961524},
	{SG46335, &UA3078454},
	{SG46341, &UA3080907},
	{SG46530, &UA2428067},
	{SG46561, &UA2658070},
	{SG46592, &UA2754584},
	{SG46595, &UA2779925},
	{SG46618, &UA2735170},
	{SG46633, &UA3023145},
	{SG46677, &UA3063837},
	{SG46699, &UA2735641},
	{SG46720, &UA2871489},
	{SG46723, &UA3083479},
	{SG46758, &UA3018275},
	{SG46759, &UA3078847},
	{SG46789, &UA2943409},
	{SG46803, &UA3132954},
	{SG46816, &UA2596929},
	{SG46911, &UA3081693},
	{SG46932, &UA3083603},
	{SG46963, &UA2428506},
	{SG46989, &UA2976021},
	{SG47025, &UA2633924},
	{SG47039, &UA2843585},
	{SG47041, &UA2750376},
	{SG47067, &UA3083127},
	{SG47083, &UA2869197},
	{SG47107, &UA2769027},
	{SG47110, &UA2867748},
	{SG47125, &UA2714372},
	{SG47139, &UA2957359},
	{SG47141, &UA3074267},
	{SG47161, &UA2975536},
	{SG47185, &UA3083445},
	{SG47230, &UA2974107},
	{SG47234, &UA3130586},
	{SG47253, &UA2486392},
	{SG47256, &UA2625628},
	{SG47259, &UA2637174},
	{SG47270, &UA2461384},
	{SG47291, &UA2974892},
	{SG47308, &UA2860469},
	{SG47322, &UA2715306},
	{SG47332, &UA2429887},
	{SG47337, &UA2782080},
	{SG47353, &UA2973893},
	{SG47366, &UA3022097},
	{SG47378, &UA2516620},
	{SG47382, &UA2892602},
	{SG47392, &UA2752818},
	{SG47393, &UA2781942},
	{SG47453, &UA2458426},
	{SG47487, &UA2468618},
	{SG47490, &UA2752482},
	{SG47558, &UA3017139},
	{SG47569, &UA2621910},
	{SG47570, &UA2623868},
	{SG47579, &UA3034474},
	{SG47591, &UA2704460},
	{SG47593, &UA2974964},
	{SG47595, &UA2968246},
	{SG47624, &UA3080078},
	{SG47635, &UA2459470},
	{SG47638, &UA3083600},
	{SG47663, &UA2958938},
	{SG47677, &UA3080455},
	{SG47730, &UA2993879},
	{SG47742, &UA2529354},
	{SG47764, &UA2965692},
	{SG47787, &UA3021536},
	{SG47814, &UA2937284},
	{SG47816, &UA2991122},
	{SG47838, &UA2530966},
	{SG47842, &UA2865069},
	{SG47884, &UA2667437},
	{SG47915, &UA2734717},
	{SG47920, &UA2870066},
	{SG47922, &UA3049082},
	{SG47937, &UA2488239},
	{SG47989, &UA3081392},
	{SG47999, &UA2770877},
	{SG48024, &UA2468728},
	{SG48027, &UA2507600},
	{SG48031, &UA2892668},
	{SG48035, &UA3027947},
	{SG48037, &UA3083184},
	{SG48058, &UA2866639},
	{SG48074, &UA2970227},
	{SG48137, &UA2975540},
	{SG48144, &UA3021250},
	{SG48179, &UA2999676},
	{SG48180, &UA3092295},
	{SG48195, &UA2556939},
	{SG48197, &UA2596535},
	{SG48203, &UA3021163},
	{SG48270, &UA2769438},
	{SG48274, &UA3021663},
	{SG48278, &UA2528955},
	{SG48307, &UA2780880},
	{SG48340, &UA2676873},
	{SG48346, &UA2866627},
	{SG48347, &UA2960038},
	{SG48349, &UA3021248},
	{SG48355, &UA2666884},
	{SG48401, &UA2997663},
	{SG48402, &UA3048480},
	{SG48433, &UA2863087},
	{SG48561, &UA2459258},
	{SG48562, &UA2622192},
	{SG48563, &UA2623528},
	{SG48565, &UA2624705},
	{SG48579, &UA2958974},
	{SG48581, &UA3000659},
	{SG48602, &UA3106497},
	{SG48617, &UA2770414},
	{SG48624, &UA3093217},
	{SG48639, &UA2851213},
	{SG48643, &UA2668824},
	{SG48655, &UA2958074},
	{SG48715, &UA3083225},
	{SG48731, &UA3022207},
	{SG48732, &UA3080250},
	{SG48753, &UA2773570},
	{SG48826, &UA3110203},
	{SG48837, &UA2622472},
	{SG48839, &UA2625440},
	{SG48841, &UA2627450},
	{SG48844, &UA2635245},
	{SG48850, &UA2751522},
	{SG48853, &UA3077525},
	{SG48860, &UA2958172},
	{SG48971, &UA3063980},
	{SG49004, &UA3109870},
	{SG49007, &UA3127146},
	{SG49029, &UA2772408},
	{SG49032, &UA3094345},
	{SG49053, &UA3021880},
	{SG49074, &UA2458731},
	{SG49075, &UA2534868},
	{SG49077, &UA2555291},
	{SG49130, &UA3001201},
	{SG49132, &UA3122127},
	{SG49160, &UA2704756},
	{SG49161, &UA2634223},
	{SG49203, &UA2961184},
	{SG49236, &UA2879029},
	{SG49237, &UA2892345},
	{SG49239, &UA3095254},
	{SG49243, &UA2468615},
	{SG49249, &UA3021136},
	{SG49251, &UA3063696},
	{SG49253, &UA3094461},
	{SG49255, &UA3094504},
	{SG49291, &UA3002424},
	{SG49297, &UA3095116},
	{SG49336, &UA2966040},
	{SG49340, &UA3063769},
	{SG49341, &UA3094453},
	{SG49371, &UA2660424},
	{SG49378, &UA2799404},
	{SG49394, &UA2749870},
	{SG49398, &UA3002952},
	{SG49433, &UA3021824},
	{SG49469, &UA2997516},
	{SG49473, &UA3023208},
	{SG49497, &UA2633356},
	{SG49498, &UA2601156},
	{SG49544, &UA2879729},
	{SG49548, &UA2965153},
	{SG49578, &UA2963920},
	{SG49581, &UA2966495},
	{SG49604, &UA2627643},
	{SG49607, &UA2938188},
	{SG49653, &UA3092369},
	{SG49660, &UA2860260},
	{SG49687, &UA2782434},
	{SG49698, &UA2486815},
	{SG49707, &UA2796995},
	{SG49748, &UA2460944},
	{SG49750, &UA2486347},
	{SG49764, &UA3130937},
	{SG49821, &UA2483126},
	{SG49837, &UA2997872},
	{SG49860, &UA2781390},
	{SG49865, &UA2868802},
	{SG49867, &UA3022830},
	{SG49869, &UA3097276},
	{SG49873, &UA2596711},
	{SG49875, &UA2764068},
	{SG49879, &UA3058103},
	{SG49897, &UA2487336},
	{SG49900, &UA2793872},
	{SG49901, &UA2860818},
	{SG49902, &UA2874504},
	{SG49903, &UA2878808},
	{SG49905, &UA2954735},
	{SG49917, &UA3020865},
	{SG49920, &UA3094275},
	{SG49948, &UA3094974},
	{SG50141, &UA2624317},
	{SG50142, &UA2633614},
	{SG50188, &UA2799405},
	{SG50237, &UA3022262},
	{SG50275, &UA2996977},
	{SG50277, &UA3011279},
	{SG50322, &UA3031208},
	{SG50323, &UA3023485},
	{SG50363, &UA2937377},
	{SG50367, &UA2998733},
	{SG50544, &UA3095063},
	{SG50561, &UA2506739},
	{SG50562, &UA2850998},
	{SG50583, &UA2996758},
	{SG50584, &UA3060209},
	{SG50618, &UA2892373},
	{SG50623, &UA3022385},
	{SG50626, &UA3023142},
	{SG50627, &UA3023837},
	{SG50628, &UA3059309},
	{SG50645, &UA2893053},
	{SG50647, &UA3020086},
	{SG50648, &UA3107636},
	{SG50650, &UA3023200},
	{SG50656, &UA3094503},
	{SG50674, &UA2530276},
	{SG50686, &UA3022506},
	{SG50688, &UA3025640},
	{SG50711, &UA2484776},
	{SG50739, &UA2730620},
	{SG50742, &UA3071321},
	{SG50766, &UA3094027},
	{SG50803, &UA2783093},
	{SG50833, &UA2484685},
	{SG50842, &UA2799997},
	{SG50851, &UA3093656},
	{SG50853, &UA3078273},
	{SG50889, &UA2713872},
	{SG50898, &UA2799792},
	{SG50907, &UA2892669},
	{SG50908, &UA2958330},
	{SG50966, &UA3109492},
	{SG50987, &UA2586660},
	{SG50989, &UA3017908},
	{SG50991, &UA2634798},
	{SG51028, &UA3108269},
	{SG51041, &UA3108479},
	{SG51085, &UA3094188},
	{SG51103, &UA2486038},
	{SG51107, &UA2623213},
	{SG51108, &UA2624171},
	{SG51110, &UA2631475},
	{SG51149, &UA2864663},
	{SG51162, &UA2996580},
	{SG51174, &UA2510135},
	{SG51294, &UA3022158},
	{SG51361, &UA2869725},
	{SG51366, &UA3022492},
	{SG51370, &UA3108952},
	{SG51408, &UA2867215},
	{SG51420, &UA3018244},
	{SG51426, &UA3083243},
	{SG51506, &UA2506974},
	{SG51535, &UA3022281},
	{SG51562, &UA2624355},
	{SG51563, &UA2628776},
	{SG51565, &UA2704927},
	{SG51571, &UA2958018},
	{SG51574, &UA2964024},
	{SG51576, &UA3000045},
	{SG51582, &UA3050075},
	{SG51604, &UA2469421},
	{SG51626, &UA3015527},
	{SG51628, &UA3021823},
	{SG51677, &UA2867852},
	{SG51709, &UA2483782},
	{SG51731, &UA2800409},
	{SG51737, &UA3016309},
	{SG51750, &UA3109213},
	{SG51774, &UA2737952},
	{SG51779, &UA2879693},
	{SG51780, &UA2935936},
	{SG51781, &UA2937055},
	{SG52090, &UA3110902},
	{SG52125, &UA3009461},
	{SG52132, &UA3023679},
	{SG52164, &UA2879478},
	{SG52171, &UA2960707},
	{SG52173, &UA2996425},
	{SG52174, &UA2997316},
	{SG52197, &UA2485466},
	{SG52198, &UA2507373},
	{SG52201, &UA2596961},
	{SG52202, &UA2625714},
	{SG52214, &UA2938047},
	{SG52222, &UA3023848},
	{SG52269, &UA2879658},
	{SG52272, &UA2880399},
	{SG52274, &UA2893038},
	{SG52279, &UA2965911},
	{SG52280, &UA2993688},
	{SG52352, &UA2797636},
	{SG52353, &UA2800656},
	{SG52359, &UA3019990},
	{SG52361, &UA3109094},
	{SG52369, &UA3092645},
	{SG52422, &UA2867047},
	{SG52424, &UA2940914},
	{SG52466, &UA2627551},
	{SG52467, &UA2634672},
	{SG52469, &UA2631176},
	{SG52479, &UA2879511},
	{SG52547, &UA3010953},
	{SG52605, &UA2625651},
	{SG52612, &UA2783696},
	{SG52615, &UA2876103},
	{SG52616, &UA2877368},
	{SG52617, &UA2941576},
	{SG52619, &UA2958260},
	{SG52623, &UA3017135},
	{SG52627, &UA3060434},
	{SG52660, &UA2879417},
	{SG52664, &UA2958375},
	{SG52667, &UA3003293},
	{SG52671, &UA3004137},
	{SG52674, &UA3022145},
	{SG52678, &UA3112104},
	{SG52706, &UA2995026},
	{SG52711, &UA3093373},
	{SG52713, &UA3103909},
	{SG52714, &UA3121645},
	{SG52722, &UA2504675},
	{SG52733, &UA2879441},
	{SG52734, &UA2936775},
	{SG52735, &UA2963508},
	{SG52736, &UA2997470},
	{SG52737, &UA3050142},
	{SG52780, &UA2626936},
	{SG52785, &UA2798403},
	{SG52795, &UA3059562},
	{SG52796, &UA3061600},
	{SG52799, &UA3103061},
	{SG52844, &UA3022489},
	{SG52872, &UA2738024},
	{SG52875, &UA2868107},
	{SG52876, &UA2868378},
	{SG52880, &UA2892632},
	{SG52885, &UA3113797},
	{SG52911, &UA2937517},
	{SG52914, &UA3009395},
	{SG52917, &UA3022831},
	{SG52947, &UA2484037},
	{SG52949, &UA2505949},
	{SG52953, &UA2622181},
	{SG52970, &UA3001195},
	{SG52974, &UA3011664},
	{SG52976, &UA3019202},
	{SG52978, &UA3109358},
	{SG53013, &UA2879173},
	{SG53018, &UA3049354},
	{SG53024, &UA3095750},
	{SG53072, &UA2799809},
	{SG53082, &UA3013890},
	{SG53084, &UA3021592},
	{SG53086, &UA3050106},
	{SG53089, &UA3108738},
	{SG53109, &UA2998275},
	{SG53110, &UA3021560},
	{SG53111, &UA3109305},
	{SG53139, &UA2506795},
	{SG53171, &UA2751866},
	{SG53178, &UA2958499},
	{SG53184, &UA3023763},
	{SG53188, &UA3075850},
	{SG53194, &UA3109932},
	{SG53204, &UA2738021},
	{SG53282, &UA3013306},
	{SG53292, &UA3081217},
	{SG53293, &UA3103073},
	{SG53323, &UA2631782},
	{SG53363, &UA2956555},
	{SG53371, &UA2997377},
	{SG53374, &UA2998089},
	{SG53382, &UA3057027},
	{SG53383, &UA3109246},
	{SG53392, &UA2634131},
	{SG53393, &UA2657374},
	{SG53414, &UA3021875},
	{SG53417, &UA3022277},
	{SG53418, &UA3077079},
	{SG53420, &UA3107980},
	{SG53421, &UA3127214},
	{SG53438, &UA2506794},
	{SG53462, &UA2628732},
	{SG53463, &UA2633141},
	{SG53471, &UA2853665},
	{SG53491, &UA3022007},
	{SG53512, &UA2735231},
	{SG53519, &UA2994000},
	{SG53523, &UA3000423},
	{SG53533, &UA3078555},
	{SG53535, &UA3109098},
	{SG53545, &UA2530484},
	{SG53566, &UA3021555},
	{SG53581, &UA3108382},
	{SG53584, &UA3131393},
	{SG53599, &UA2781400},
	{SG53622, &UA2939330},
	{SG53628, &UA3006111},
	{SG53633, &UA3056655},
	{SG53636, &UA3092230},
	{SG53638, &UA3106963},
	{SG53641, &UA3109129},
	{SG53642, &UA3110878},
	{SG53646, &UA3127065},
	{SG53648, &UA3131386},
	{SG53709, &UA3024100},
	{SG53711, &UA3048866},
	{SG53712, &UA3049649},
	{SG53721, &UA3058141},
	{SG53731, &UA3126296},
	{SG53732, &UA3126723},
	{SG53733, &UA3127272},
	{SG53737, &UA3131580},
	{SG53894, &UA3001781},
	{SG53915, &UA3027968},
	{SG53917, &UA3045513},
	{SG53920, &UA3049716},
	{SG53921, &UA3052517},
	{SG54018, &UA3093100},
	{SG54025, &UA3109929},
	{SG54036, &UA3125119},
	{SG54053, &UA3127140},
	{SG54108, &UA2623592},
	{SG54110, &UA2633344},
	{SG54124, &UA2701051},
	{SG54130, &UA2728455},
	{SG54158, &UA2850546},
	{SG54167, &UA2997593},
	{SG54204, &UA3073266},
	{SG54205, &UA3077868},
	{SG54206, &UA3103097},
	{SG54207, &UA3107055},
	{SG54212, &UA3127239},
	{SG54237, &UA2533414},
	{SG54238, &UA2558241},
	{SG54243, &UA2626020},
	{SG54244, &UA2630732},
	{SG54265, &UA2939596},
	{SG54266, &UA2963221},
	{SG54267, &UA3004416},
	{SG54283, &UA3108118},
	{SG54284, &UA3108956},
	{SG54292, &UA3121512},
	{SG54298, &UA3126217},
	{SG54301, &UA3126795},
	{SG54327, &UA2844194},
	{SG54359, &UA3100033},
	{SG54361, &UA3101915},
	{SG54363, &UA3102266},
	{SG54365, &UA3103253},
	{SG54366, &UA3103429},
	{SG54381, &UA3125255},
	{SG54383, &UA3126377},
	{SG54384, &UA3127070},
	{SG54394, &UA2634960},
	{SG54405, &UA2865452},
	{SG54543, &UA2623609},
	{SG54544, &UA2629607},
	{SG54615, &UA2623829},
	{SG54826, &UA2622080},
	{SG54827, &UA2630408},
	{SG54874, &UA2623464},
	{SG54880, &UA2626122},
	{SG54881, &UA2628534},
	{SG54885, &UA2626447},
	{SG54886, &UA2629611},
	{SG54914, &UA2635254},
	{SG54925, &UA2624933},
	{SG54936, &UA2607524},
	{SG54939, &UA2637594},
	{SG54954, &UA2629073},
	{SG54955, &UA2630813},
	{SG54957, &UA2630745},
	{SG54995, &UA2623951},
	{SG54997, &UA2635045},
	{SG55001, &UA2631425},
	{SG55002, &UA2633047},
	{SG55023, &UA2622748},
	{SG55024, &UA2627365},
	{SG55025, &UA2629031},
	{SG55036, &UA2633217},
	{SG55038, &UA2637251},
	{SG55039, &UA2637869},
	{SG55138, &UA2624089},
	{SG55139, &UA2627344},
	{SG55140, &UA2627702},
	{SG55141, &UA2629628},
	{SG55142, &UA2629753},
	{SG55143, &UA2629812},
	{SG55144, &UA2629869},
	{SG55145, &UA2631511},
	{SG55146, &UA2631614},
	{SG55147, &UA2632396},
	{SG55148, &UA2632455},
	{SG55149, &UA2632541},
	{SG55152, &UA2635516},
	{SG55153, &UA2636076},
	{SG55154, &UA2636998},
	{SG55156, &UA2637444},
	{SG55157, &UA2637641},
	{SG55164, &UA3068939},
	{SG55165, &UA3084062},
	{SG55166, &UA3084088},
	{NULL}};
const Segmentdata DAndroidVersion4_1_2[] = {
	{SG26152, &UA2232316},
	{SG26154, &UA2235161},
	{SG26155, &UA2242346},
	{SG26157, &UA2258002},
	{SG26160, &UA2355950},
	{SG26163, &UA2402363},
	{SG26166, &UA2437333},
	{SG26168, &UA2457985},
	{SG26170, &UA2460873},
	{SG26173, &UA2469474},
	{SG26175, &UA2469859},
	{SG26177, &UA2483957},
	{SG26178, &UA2489485},
	{SG26179, &UA2502484},
	{SG26180, &UA2504304},
	{SG26181, &UA2505257},
	{SG26182, &UA2507376},
	{SG26183, &UA2514680},
	{SG26184, &UA2539083},
	{SG26186, &UA2555977},
	{SG26187, &UA2559571},
	{SG26188, &UA2560908},
	{SG26189, &UA2603762},
	{SG26190, &UA2660078},
	{SG26191, &UA2665370},
	{SG26192, &UA2666694},
	{SG26193, &UA2675413},
	{SG26195, &UA2678026},
	{SG26196, &UA2695707},
	{SG26197, &UA2701304},
	{SG26198, &UA2729057},
	{SG26199, &UA2729981},
	{SG26201, &UA2740417},
	{SG26203, &UA2741468},
	{SG26206, &UA2754697},
	{SG26209, &UA2764002},
	{SG26212, &UA2797474},
	{SG26215, &UA2843521},
	{SG26217, &UA2858037},
	{SG26219, &UA2864903},
	{SG26221, &UA2878193},
	{SG26222, &UA2891834},
	{SG26223, &UA2895512},
	{SG26224, &UA2899386},
	{SG26226, &UA2937155},
	{SG26227, &UA2971484},
	{SG26229, &UA2972552},
	{SG26231, &UA2975754},
	{SG26234, &UA3028300},
	{SG26236, &UA3059456},
	{SG26238, &UA3070809},
	{SG26239, &UA3087045},
	{SG26240, &UA3101422},
	{SG26241, &UA3124750},
	{SG26242, &UA3125759},
	{SG26243, &UA3125896},
	{SG26459, &UA2367109},
	{SG26461, &UA2404053},
	{SG26462, &UA2404775},
	{SG26464, &UA2423862},
	{SG26466, &UA2424137},
	{SG26467, &UA2424969},
	{SG26468, &UA2426188},
	{SG26469, &UA2427432},
	{SG26470, &UA2428447},
	{SG26472, &UA2428785},
	{SG26473, &UA2431875},
	{SG26475, &UA2432847},
	{SG26476, &UA2434738},
	{SG26477, &UA2437682},
	{SG26478, &UA2458187},
	{SG26479, &UA2467997},
	{SG26481, &UA2468317},
	{SG26482, &UA2482892},
	{SG26483, &UA2482976},
	{SG26485, &UA2505642},
	{SG26486, &UA2508571},
	{SG26487, &UA2514719},
	{SG26488, &UA2529040},
	{SG26489, &UA2529866},
	{SG26490, &UA2531481},
	{SG26491, &UA2534587},
	{SG26492, &UA2539851},
	{SG26494, &UA2556235},
	{SG26495, &UA2556812},
	{SG26496, &UA2558037},
	{SG26497, &UA2559411},
	{SG26499, &UA2561531},
	{SG26500, &UA2567427},
	{SG26502, &UA2571477},
	{SG26505, &UA2579547},
	{SG26507, &UA2583221},
	{SG26509, &UA2583548},
	{SG26512, &UA2597415},
	{SG26513, &UA2597419},
	{SG26515, &UA2600494},
	{SG26517, &UA2606251},
	{SG26518, &UA2656375},
	{SG26520, &UA2657633},
	{SG26522, &UA2658076},
	{SG26523, &UA2668095},
	{SG26524, &UA2673890},
	{SG26526, &UA2677515},
	{SG26527, &UA2695025},
	{SG26528, &UA2702530},
	{SG26529, &UA2705672},
	{SG26531, &UA2707076},
	{SG26532, &UA2738850},
	{SG26533, &UA2747554},
	{SG26534, &UA2748620},
	{SG26535, &UA2753853},
	{SG26536, &UA2846791},
	{SG26537, &UA2862898},
	{SG26538, &UA2869440},
	{SG26540, &UA2935907},
	{SG26541, &UA2974766},
	{SG26542, &UA2976055},
	{SG26543, &UA2990723},
	{SG26546, &UA2992579},
	{SG26547, &UA3025456},
	{SG26548, &UA3031639},
	{SG26549, &UA3034676},
	{SG26559, &UA3047272},
	{SG26560, &UA3059576},
	{SG26569, &UA3062477},
	{SG26571, &UA3072784},
	{SG26607, &UA2235644},
	{SG26608, &UA2237963},
	{SG26609, &UA2257952},
	{SG26611, &UA2384553},
	{SG26615, &UA2421993},
	{SG26618, &UA2430961},
	{SG26619, &UA2431759},
	{SG26621, &UA2432614},
	{SG26623, &UA2455604},
	{SG26624, &UA2461394},
	{SG26626, &UA2462258},
	{SG26653, &UA2469333},
	{SG26655, &UA2485503},
	{SG26657, &UA2502864},
	{SG26658, &UA2510342},
	{SG26659, &UA2511373},
	{SG26661, &UA2528730},
	{SG26663, &UA2531414},
	{SG26664, &UA2554801},
	{SG26689, &UA2559653},
	{SG26691, &UA2563856},
	{SG26692, &UA2564869},
	{SG26693, &UA2569592},
	{SG26694, &UA2570297},
	{SG26695, &UA2596464},
	{SG26698, &UA2659396},
	{SG26699, &UA2660544},
	{SG26700, &UA2669788},
	{SG26702, &UA2674014},
	{SG26704, &UA2677984},
	{SG26705, &UA2680757},
	{SG26707, &UA2716455},
	{SG26708, &UA2735029},
	{SG26709, &UA2737495},
	{SG26711, &UA2752493},
	{SG26714, &UA2878285},
	{SG26717, &UA2879644},
	{SG26718, &UA2885325},
	{SG26719, &UA2894138},
	{SG26721, &UA2898642},
	{SG26722, &UA2899137},
	{SG26724, &UA2948573},
	{SG26727, &UA2985301},
	{SG26729, &UA2985424},
	{SG26731, &UA2989644},
	{SG26733, &UA3033179},
	{SG26735, &UA3067515},
	{SG26737, &UA3104593},
	{SG26739, &UA3120512},
	{SG27317, &UA2234668},
	{SG27318, &UA2242263},
	{SG27319, &UA2257851},
	{SG27320, &UA2258125},
	{SG27321, &UA2502437},
	{SG27333, &UA2503232},
	{SG27334, &UA2504845},
	{SG27335, &UA2527161},
	{SG27336, &UA2527558},
	{SG27337, &UA2553178},
	{SG27338, &UA2566850},
	{SG27339, &UA2596941},
	{SG27340, &UA2598165},
	{SG27341, &UA2656308},
	{SG27342, &UA2658546},
	{SG27343, &UA2661090},
	{SG27344, &UA2666535},
	{SG27345, &UA2695879},
	{SG27346, &UA2701303},
	{SG27347, &UA2708574},
	{SG27348, &UA2711362},
	{SG27349, &UA2762782},
	{SG27350, &UA2769469},
	{SG27351, &UA2783253},
	{SG27352, &UA2861569},
	{SG27353, &UA2934407},
	{SG27354, &UA2935446},
	{SG27355, &UA2937970},
	{SG27356, &UA2946718},
	{SG27357, &UA2952880},
	{SG27358, &UA2988387},
	{SG27359, &UA3107484},
	{SG28242, &UA2422372},
	{SG28251, &UA2457302},
	{SG28252, &UA2457740},
	{SG28262, &UA2503519},
	{SG28265, &UA2504538},
	{SG28278, &UA2530280},
	{SG28296, &UA2578101},
	{SG28299, &UA2578496},
	{SG28307, &UA2580692},
	{SG28346, &UA2671069},
	{SG28362, &UA2703272},
	{SG28365, &UA2703726},
	{SG28367, &UA2704029},
	{SG28371, &UA2705988},
	{SG28377, &UA2709498},
	{SG28380, &UA2713188},
	{SG28387, &UA2728007},
	{SG28389, &UA2729051},
	{SG28396, &UA2731095},
	{SG28398, &UA2731291},
	{SG28400, &UA2732125},
	{SG28402, &UA2732316},
	{SG28405, &UA2732438},
	{SG28407, &UA2732588},
	{SG28409, &UA2732748},
	{SG28412, &UA2732815},
	{SG28414, &UA2733819},
	{SG28417, &UA2734397},
	{SG28419, &UA2737617},
	{SG28431, &UA2751501},
	{SG28434, &UA2752028},
	{SG28443, &UA2766503},
	{SG28445, &UA2766782},
	{SG28451, &UA2768754},
	{SG28455, &UA2778823},
	{SG28457, &UA2779615},
	{SG28459, &UA2779786},
	{SG28463, &UA2780817},
	{SG28465, &UA2780840},
	{SG28468, &UA2780991},
	{SG28471, &UA2784113},
	{SG28473, &UA2790854},
	{SG28498, &UA2793915},
	{SG28502, &UA2794510},
	{SG28504, &UA2794658},
	{SG28506, &UA2794865},
	{SG28508, &UA2795262},
	{SG28512, &UA2840086},
	{SG28561, &UA2859609},
	{SG28562, &UA2862220},
	{SG28563, &UA2864961},
	{SG28564, &UA2867314},
	{SG28565, &UA2867929},
	{SG28567, &UA2869637},
	{SG28569, &UA2869659},
	{SG28570, &UA2870784},
	{SG28572, &UA2878439},
	{SG28576, &UA2879071},
	{SG28579, &UA2879502},
	{SG28581, &UA2879524},
	{SG28582, &UA2889471},
	{SG28587, &UA2890971},
	{SG28589, &UA2891167},
	{SG28591, &UA2891499},
	{SG28593, &UA2891974},
	{SG28595, &UA2892446},
	{SG28598, &UA2892796},
	{SG28600, &UA2892848},
	{SG28603, &UA2893386},
	{SG28607, &UA2934622},
	{SG28610, &UA2934875},
	{SG28616, &UA2936127},
	{SG28618, &UA2936283},
	{SG28623, &UA2936957},
	{SG28628, &UA2937653},
	{SG28630, &UA2937820},
	{SG28632, &UA2937831},
	{SG28635, &UA2937948},
	{SG28637, &UA2938235},
	{SG28639, &UA2938526},
	{SG28642, &UA2942546},
	{SG28658, &UA2955143},
	{SG28661, &UA2955241},
	{SG28663, &UA2955949},
	{SG28667, &UA2956376},
	{SG28668, &UA2956611},
	{SG28686, &UA2956730},
	{SG28687, &UA2957070},
	{SG28688, &UA2957592},
	{SG28689, &UA2958036},
	{SG28690, &UA2958185},
	{SG28691, &UA2958352},
	{SG28692, &UA2958356},
	{SG28693, &UA2958975},
	{SG28694, &UA2959346},
	{SG28695, &UA2968974},
	{SG28699, &UA2985645},
	{SG28702, &UA2985886},
	{SG28703, &UA2986500},
	{SG28704, &UA2987155},
	{SG28705, &UA2988839},
	{SG28706, &UA2989011},
	{SG28707, &UA2989108},
	{SG28708, &UA2990000},
	{SG28709, &UA2990359},
	{SG28710, &UA2990621},
	{SG28711, &UA2991694},
	{SG28712, &UA2992209},
	{SG28713, &UA2992449},
	{SG28714, &UA2992751},
	{SG28715, &UA2993874},
	{SG28716, &UA2994007},
	{SG28717, &UA2994328},
	{SG28718, &UA2994663},
	{SG28719, &UA2994899},
	{SG28720, &UA2996325},
	{SG28721, &UA2996553},
	{SG28722, &UA2996755},
	{SG28723, &UA2996893},
	{SG28724, &UA2996919},
	{SG28725, &UA2997245},
	{SG28726, &UA2997303},
	{SG28727, &UA2997448},
	{SG28728, &UA2997928},
	{SG28729, &UA2998183},
	{SG28730, &UA2998283},
	{SG28731, &UA2998307},
	{SG28732, &UA2998622},
	{SG28733, &UA2998994},
	{SG28734, &UA2999248},
	{SG28735, &UA2999341},
	{SG28736, &UA2999386},
	{SG28737, &UA2999471},
	{SG28738, &UA2999486},
	{SG28739, &UA2999586},
	{SG28740, &UA2999687},
	{SG28741, &UA3000163},
	{SG28742, &UA3000242},
	{SG28743, &UA3000431},
	{SG28744, &UA3001277},
	{SG28745, &UA3001405},
	{SG28746, &UA3001858},
	{SG28747, &UA3002046},
	{SG28748, &UA3003535},
	{SG28749, &UA3004391},
	{SG28750, &UA3004789},
	{SG28751, &UA3006561},
	{SG28752, &UA3007545},
	{SG28753, &UA3007697},
	{SG28754, &UA3008276},
	{SG28755, &UA3008376},
	{SG28756, &UA3009885},
	{SG28757, &UA3010452},
	{SG28758, &UA3011242},
	{SG28762, &UA3012752},
	{SG28767, &UA3013268},
	{SG28768, &UA3013604},
	{SG28769, &UA3014146},
	{SG28770, &UA3014373},
	{SG28771, &UA3014395},
	{SG28772, &UA3016241},
	{SG28786, &UA3016396},
	{SG28789, &UA3016946},
	{SG28792, &UA3018074},
	{SG28795, &UA3018115},
	{SG28845, &UA3018434},
	{SG28849, &UA3020494},
	{SG28852, &UA3020750},
	{SG28856, &UA3021331},
	{SG28858, &UA3022608},
	{SG28862, &UA3029368},
	{SG28864, &UA3031034},
	{SG28872, &UA3045936},
	{SG28873, &UA3046088},
	{SG28878, &UA3048622},
	{SG28882, &UA3049001},
	{SG28886, &UA3049107},
	{SG28888, &UA3049165},
	{SG28897, &UA3049850},
	{SG28899, &UA3049970},
	{SG28904, &UA3050652},
	{SG28909, &UA3054479},
	{SG28911, &UA3054706},
	{SG28913, &UA3055149},
	{SG28915, &UA3055670},
	{SG28919, &UA3056739},
	{SG28921, &UA3056948},
	{SG28923, &UA3057052},
	{SG28925, &UA3057071},
	{SG28928, &UA3057320},
	{SG28930, &UA3057511},
	{SG28932, &UA3057560},
	{SG28935, &UA3057818},
	{SG28937, &UA3057837},
	{SG28938, &UA3057907},
	{SG28940, &UA3058233},
	{SG28944, &UA3058926},
	{SG28946, &UA3058949},
	{SG28948, &UA3059038},
	{SG28950, &UA3059494},
	{SG28953, &UA3059744},
	{SG28955, &UA3059978},
	{SG28956, &UA3060076},
	{SG28958, &UA3060639},
	{SG28959, &UA3062316},
	{SG28961, &UA3066122},
	{SG28962, &UA3071331},
	{SG28963, &UA3071984},
	{SG28964, &UA3072816},
	{SG28965, &UA3072868},
	{SG28966, &UA3073905},
	{SG28967, &UA3074147},
	{SG28968, &UA3074153},
	{SG28969, &UA3074257},
	{SG28970, &UA3074280},
	{SG28971, &UA3074270},
	{SG28972, &UA3075192},
	{SG28973, &UA3076737},
	{SG28974, &UA3077722},
	{SG28975, &UA3077827},
	{SG28976, &UA3077882},
	{SG28977, &UA3078376},
	{SG28978, &UA3078511},
	{SG28979, &UA3078843},
	{SG28980, &UA3079105},
	{SG28981, &UA3079339},
	{SG28982, &UA3080024},
	{SG28983, &UA3087103},
	{SG28984, &UA3087615},
	{SG28985, &UA3087750},
	{SG28986, &UA3087922},
	{SG28987, &UA3088616},
	{SG28988, &UA3088764},
	{SG28989, &UA3088926},
	{SG28990, &UA3090380},
	{SG28991, &UA3091595},
	{SG28992, &UA3091798},
	{SG28993, &UA3091865},
	{SG28994, &UA3091990},
	{SG28995, &UA3092036},
	{SG28996, &UA3092208},
	{SG28997, &UA3092274},
	{SG28998, &UA3092525},
	{SG28999, &UA3092662},
	{SG29000, &UA3092852},
	{SG29001, &UA3092907},
	{SG29002, &UA3099983},
	{SG29004, &UA3100618},
	{SG29006, &UA3100812},
	{SG29009, &UA3101448},
	{SG29011, &UA3101846},
	{SG29014, &UA3102155},
	{SG29016, &UA3102183},
	{SG29018, &UA3102340},
	{SG29021, &UA3102970},
	{SG29023, &UA3102975},
	{SG29027, &UA3103135},
	{SG29029, &UA3103137},
	{SG29030, &UA3103196},
	{SG29033, &UA3103245},
	{SG29038, &UA3104127},
	{SG29040, &UA3104476},
	{SG29042, &UA3105808},
	{SG29044, &UA3106094},
	{SG29046, &UA3106492},
	{SG29050, &UA3107474},
	{SG29052, &UA3107650},
	{SG29054, &UA3107661},
	{SG29055, &UA3107698},
	{SG29057, &UA3107790},
	{SG29060, &UA3107845},
	{SG29063, &UA3107950},
	{SG29064, &UA3107954},
	{SG29066, &UA3108145},
	{SG29067, &UA3108176},
	{SG29068, &UA3108284},
	{SG29071, &UA3108274},
	{SG29072, &UA3108344},
	{SG29073, &UA3108386},
	{SG29075, &UA3108639},
	{SG29077, &UA3108657},
	{SG29079, &UA3108662},
	{SG29081, &UA3108848},
	{SG29083, &UA3108862},
	{SG29085, &UA3111689},
	{SG29086, &UA3118729},
	{SG29087, &UA3119035},
	{SG29088, &UA3119184},
	{SG29089, &UA3120180},
	{SG29090, &UA3121086},
	{SG29091, &UA3121150},
	{SG29092, &UA3121554},
	{SG29093, &UA3121579},
	{SG29094, &UA3121649},
	{SG29095, &UA3121807},
	{SG29096, &UA3122051},
	{SG29097, &UA3124618},
	{SG29098, &UA3124777},
	{SG29099, &UA3124835},
	{SG29100, &UA3124880},
	{SG29101, &UA3125017},
	{SG29102, &UA3125090},
	{SG29103, &UA3125149},
	{SG29104, &UA3125259},
	{SG29105, &UA3125331},
	{SG29106, &UA3125420},
	{SG29107, &UA3125457},
	{SG29108, &UA3126355},
	{SG29109, &UA3127008},
	{SG29110, &UA3127396},
	{SG29196, &UA2257985},
	{SG29199, &UA2355117},
	{SG29206, &UA2482675},
	{SG29214, &UA2653921},
	{SG29217, &UA2879420},
	{SG29218, &UA2942259},
	{SG29299, &UA2422603},
	{SG29300, &UA2456979},
	{SG29301, &UA2491399},
	{SG29302, &UA2530228},
	{SG29303, &UA2532555},
	{SG29304, &UA2538105},
	{SG29305, &UA2598147},
	{SG29306, &UA2604413},
	{SG29307, &UA2605401},
	{SG29308, &UA2661450},
	{SG29309, &UA2857682},
	{SG29310, &UA2972860},
	{SG29311, &UA2983236},
	{SG29312, &UA2986841},
	{SG29313, &UA3019200},
	{SG29385, &UA2487324},
	{SG29386, &UA2506984},
	{SG29387, &UA2571599},
	{SG29388, &UA2706101},
	{SG29389, &UA2709502},
	{SG29390, &UA2887168},
	{SG29391, &UA2963152},
	{SG29392, &UA3023910},
	{SG29393, &UA3047682},
	{SG29394, &UA3087841},
	{SG29395, &UA2426056},
	{SG29396, &UA2458991},
	{SG29397, &UA2533303},
	{SG29398, &UA2559658},
	{SG29399, &UA2600020},
	{SG29400, &UA2605190},
	{SG29401, &UA2605853},
	{SG29402, &UA2659795},
	{SG29403, &UA2662110},
	{SG29404, &UA2662640},
	{SG29405, &UA2665857},
	{SG29406, &UA2666179},
	{SG29407, &UA2666876},
	{SG29408, &UA2669209},
	{SG29409, &UA2709557},
	{SG29410, &UA2710496},
	{SG29411, &UA2711654},
	{SG29412, &UA2712399},
	{SG29413, &UA2719031},
	{SG29414, &UA2751469},
	{SG29415, &UA2773434},
	{SG29416, &UA2773995},
	{SG29417, &UA2780854},
	{SG29418, &UA2796879},
	{SG29419, &UA2877341},
	{SG29420, &UA2879633},
	{SG29421, &UA2879760},
	{SG29422, &UA2890640},
	{SG29423, &UA2901194},
	{SG29424, &UA2956604},
	{SG29425, &UA2961455},
	{SG29426, &UA3001514},
	{SG29431, &UA3021352},
	{SG29433, &UA3048526},
	{SG29669, &UA2367394},
	{SG29677, &UA2422398},
	{SG29682, &UA2481520},
	{SG29683, &UA2507992},
	{SG29684, &UA2582669},
	{SG29685, &UA2699404},
	{SG29686, &UA2704589},
	{SG29687, &UA2734799},
	{SG29688, &UA2734931},
	{SG29689, &UA2765424},
	{SG29690, &UA2779617},
	{SG29691, &UA2779783},
	{SG29692, &UA2845913},
	{SG29693, &UA2848353},
	{SG29694, &UA2878631},
	{SG29695, &UA2937360},
	{SG29696, &UA2996313},
	{SG29697, &UA3012148},
	{SG29698, &UA3099184},
	{SG29699, &UA3108101},
	{SG29711, &UA2367199},
	{SG29712, &UA2403851},
	{SG29713, &UA2408178},
	{SG29714, &UA2423572},
	{SG29715, &UA2425151},
	{SG29717, &UA2426359},
	{SG29719, &UA2426451},
	{SG29721, &UA2427435},
	{SG29723, &UA2427936},
	{SG29725, &UA2431894},
	{SG29727, &UA2434770},
	{SG29729, &UA2484235},
	{SG29731, &UA2506709},
	{SG29733, &UA2535257},
	{SG29735, &UA2562066},
	{SG29738, &UA2603552},
	{SG29740, &UA2848202},
	{SG29742, &UA2983903},
	{SG29792, &UA2466575},
	{SG29793, &UA2578598},
	{SG29794, &UA2617685},
	{SG29795, &UA2620150},
	{SG29796, &UA2635391},
	{SG29797, &UA2636023},
	{SG29798, &UA2636312},
	{SG29799, &UA2636833},
	{SG29834, &UA2257949},
	{SG29835, &UA2362535},
	{SG29836, &UA2435740},
	{SG29837, &UA2459363},
	{SG29838, &UA2505611},
	{SG29839, &UA2528514},
	{SG29840, &UA2530196},
	{SG29841, &UA2531049},
	{SG29842, &UA2577895},
	{SG29843, &UA2595618},
	{SG29844, &UA2664085},
	{SG29845, &UA2666236},
	{SG29846, &UA2670056},
	{SG29847, &UA2680845},
	{SG29848, &UA2702936},
	{SG29849, &UA2762991},
	{SG29850, &UA2767839},
	{SG29851, &UA2885307},
	{SG29852, &UA2958148},
	{SG29853, &UA2974412},
	{SG29854, &UA2985199},
	{SG29855, &UA3015178},
	{SG29856, &UA3044654},
	{SG29857, &UA3049727},
	{SG29858, &UA3062694},
	{SG29859, &UA3091998},
	{SG29861, &UA3096070},
	{SG29863, &UA3100000},
	{SG29934, &UA2206125},
	{SG29935, &UA2231623},
	{SG29937, &UA2232476},
	{SG29938, &UA2237121},
	{SG29940, &UA2257887},
	{SG29942, &UA2354931},
	{SG29944, &UA2387297},
	{SG29945, &UA2466310},
	{SG29959, &UA2356091},
	{SG29960, &UA2480335},
	{SG29966, &UA3009913},
	{SG30039, &UA2231791},
	{SG30041, &UA2238137},
	{SG30043, &UA2238242},
	{SG30046, &UA2402015},
	{SG30049, &UA2455764},
	{SG30052, &UA2483429},
	{SG30054, &UA2748317},
	{SG30056, &UA2833140},
	{SG30060, &UA2854972},
	{SG30062, &UA2943986},
	{SG30065, &UA2947860},
	{SG30067, &UA2987078},
	{SG30090, &UA2484831},
	{SG30091, &UA2507451},
	{SG30093, &UA2855725},
	{SG30094, &UA2986063},
	{SG30306, &UA2483329},
	{SG30309, &UA2527428},
	{SG30310, &UA2578085},
	{SG30319, &UA2628474},
	{SG30322, &UA2631148},
	{SG30325, &UA2631544},
	{SG30328, &UA2633795},
	{SG30330, &UA2634974},
	{SG30332, &UA2636706},
	{SG30333, &UA2637154},
	{SG30335, &UA2637446},
	{SG30337, &UA2637740},
	{SG30339, &UA2879099},
	{SG30341, &UA3020700},
	{SG30450, &UA2973665},
	{SG30452, &UA2973899},
	{SG30453, &UA2973939},
	{SG30455, &UA2973955},
	{SG30457, &UA2974022},
	{SG30459, &UA2974164},
	{SG30460, &UA2974306},
	{SG30463, &UA2974482},
	{SG30464, &UA2974728},
	{SG30467, &UA2974873},
	{SG30468, &UA2974910},
	{SG30470, &UA2974922},
	{SG30472, &UA2974928},
	{SG30475, &UA2975084},
	{SG30476, &UA2975575},
	{SG30478, &UA3021093},
	{SG30480, &UA3021566},
	{SG30483, &UA3024222},
	{SG30521, &UA2235175},
	{SG30523, &UA2237768},
	{SG30524, &UA2237933},
	{SG30526, &UA2238057},
	{SG30530, &UA2427234},
	{SG30531, &UA2430970},
	{SG30532, &UA2432035},
	{SG30533, &UA2436416},
	{SG30534, &UA2529640},
	{SG30537, &UA2658023},
	{SG30539, &UA2676853},
	{SG30542, &UA2878150},
	{SG30548, &UA3064663},
	{SG30550, &UA3064884},
	{SG30552, &UA3064893},
	{SG30555, &UA3064989},
	{SG30558, &UA3065079},
	{SG30563, &UA3065252},
	{SG30566, &UA3065381},
	{SG30569, &UA3065582},
	{SG30572, &UA3065612},
	{SG30574, &UA3065743},
	{SG30578, &UA3067197},
	{SG30581, &UA3068061},
	{SG30583, &UA3068214},
	{SG30586, &UA3080077},
	{SG30589, &UA3080127},
	{SG30597, &UA3126465},
	{SG30600, &UA3126963},
	{SG30653, &UA2566490},
	{SG30654, &UA2566518},
	{SG30655, &UA2566628},
	{SG30656, &UA2566695},
	{SG30657, &UA2566759},
	{SG30658, &UA2567094},
	{SG30659, &UA2567091},
	{SG30660, &UA2567127},
	{SG30661, &UA2567201},
	{SG30662, &UA2567253},
	{SG30663, &UA2567258},
	{SG30664, &UA2567334},
	{SG30665, &UA2567344},
	{SG30666, &UA2567384},
	{SG30667, &UA2567509},
	{SG30669, &UA2567699},
	{SG30671, &UA2583792},
	{SG30672, &UA2584164},
	{SG30673, &UA2584866},
	{SG30675, &UA2602497},
	{SG30677, &UA2603894},
	{SG30678, &UA2666999},
	{SG30681, &UA2668774},
	{SG30684, &UA2566442},
	{SG30685, &UA2566511},
	{SG30686, &UA2566704},
	{SG30687, &UA2566908},
	{SG30688, &UA2567107},
	{SG30689, &UA2567385},
	{SG30690, &UA2583274},
	{SG30691, &UA2733412},
	{SG30692, &UA2878435},
	{SG30693, &UA2967614},
	{SG30734, &UA2595596},
	{SG30735, &UA2597780},
	{SG30736, &UA2616622},
	{SG30737, &UA2627779},
	{SG30738, &UA2636372},
	{SG30739, &UA2770358},
	{SG30740, &UA2879070},
	{SG30741, &UA2936071},
	{SG30742, &UA3027014},
	{SG30771, &UA2594697},
	{SG30772, &UA2617029},
	{SG30773, &UA2617707},
	{SG30774, &UA2634856},
	{SG30775, &UA2635496},
	{SG30776, &UA2635543},
	{SG30777, &UA2635681},
	{SG30778, &UA2367390},
	{SG30779, &UA2622766},
	{SG30780, &UA2623952},
	{SG30786, &UA2421892},
	{SG30790, &UA2618346},
	{SG30792, &UA2618756},
	{SG30819, &UA2973948},
	{SG30820, &UA2973968},
	{SG30821, &UA2974039},
	{SG30822, &UA2974288},
	{SG30823, &UA2974370},
	{SG30824, &UA2974800},
	{SG30825, &UA2974903},
	{SG30826, &UA3021354},
	{SG30827, &UA3022510},
	{SG30828, &UA2566450},
	{SG30829, &UA2566636},
	{SG30830, &UA2566634},
	{SG30831, &UA2566772},
	{SG30832, &UA2567196},
	{SG30833, &UA2567213},
	{SG30834, &UA2567571},
	{SG30835, &UA2567698},
	{SG30868, &UA2571470},
	{SG30869, &UA2583105},
	{SG30870, &UA2583207},
	{SG30871, &UA2584102},
	{SG30872, &UA2602039},
	{SG30873, &UA2603479},
	{SG30874, &UA2667084},
	{SG30875, &UA2673780},
	{SG30876, &UA2675918},
	{SG30877, &UA2878422},
	{SG30878, &UA2899413},
	{SG30879, &UA3064655},
	{SG30880, &UA3064660},
	{SG30881, &UA3065156},
	{SG30882, &UA3065380},
	{SG30883, &UA3065733},
	{SG30884, &UA3065974},
	{SG30885, &UA3066224},
	{SG30886, &UA3067293},
	{SG30887, &UA3110906},
	{SG30888, &UA2467037},
	{SG30889, &UA2490691},
	{SG30890, &UA2507427},
	{SG30891, &UA2563850},
	{SG30892, &UA2675041},
	{SG30893, &UA2737899},
	{SG30894, &UA2801161},
	{SG30895, &UA2855264},
	{SG30896, &UA2975789},
	{SG30897, &UA2976020},
	{SG30939, &UA2695238},
	{SG30957, &UA2234212},
	{SG30959, &UA2238145},
	{SG30960, &UA2360552},
	{SG30961, &UA2384881},
	{SG30964, &UA2973661},
	{SG30965, &UA2973953},
	{SG30966, &UA2974247},
	{SG30967, &UA2974264},
	{SG30968, &UA2974269},
	{SG30969, &UA2974285},
	{SG30970, &UA2974324},
	{SG30971, &UA2974377},
	{SG30972, &UA2974445},
	{SG30973, &UA2975526},
	{SG30974, &UA2975929},
	{SG30975, &UA2976001},
	{SG31007, &UA3064775},
	{SG31008, &UA3064881},
	{SG31009, &UA3064982},
	{SG31010, &UA3065137},
	{SG31011, &UA3065368},
	{SG31012, &UA3065460},
	{SG31013, &UA3065479},
	{SG31014, &UA3065588},
	{SG31016, &UA3067178},
	{SG31018, &UA3067192},
	{SG31020, &UA3067907},
	{SG31022, &UA3081031},
	{SG31023, &UA2603688},
	{SG31024, &UA2668500},
	{SG31025, &UA2679310},
	{SG31026, &UA2679669},
	{SG31027, &UA2697265},
	{SG31028, &UA2712881},
	{SG31029, &UA2716877},
	{SG31030, &UA2717252},
	{SG31031, &UA2739600},
	{SG31032, &UA2752441},
	{SG31033, &UA2755303},
	{SG31034, &UA2797208},
	{SG31035, &UA2860725},
	{SG31036, &UA2937253},
	{SG31037, &UA2937955},
	{SG31038, &UA2994487},
	{SG31039, &UA3003524},
	{SG31040, &UA3080017},
	{SG31041, &UA3081669},
	{SG31042, &UA3091983},
	{SG31043, &UA3125250},
	{SG31120, &UA2740348},
	{SG31121, &UA2741596},
	{SG31122, &UA2741696},
	{SG31123, &UA2741701},
	{SG31124, &UA2741726},
	{SG31125, &UA2741733},
	{SG31126, &UA2752945},
	{SG31127, &UA2753849},
	{SG31128, &UA2753873},
	{SG31129, &UA2754720},
	{SG31130, &UA2754936},
	{SG31131, &UA2754951},
	{SG31132, &UA2755085},
	{SG31133, &UA2755338},
	{SG31134, &UA2755551},
	{SG31135, &UA2773713},
	{SG31136, &UA2869309},
	{SG31137, &UA2879320},
	{SG31138, &UA2892619},
	{SG31139, &UA2893499},
	{SG31174, &UA2737801},
	{SG31176, &UA2774320},
	{SG31177, &UA2774322},
	{SG31178, &UA2783705},
	{SG31179, &UA2784809},
	{SG31180, &UA2869865},
	{SG31181, &UA2877251},
	{SG31182, &UA2956796},
	{SG31183, &UA3000021},
	{SG31184, &UA3061767},
	{SG31185, &UA3064440},
	{SG31186, &UA3107187},
	{SG31238, &UA2430864},
	{SG31239, &UA2508302},
	{SG31240, &UA2696881},
	{SG31241, &UA2879046},
	{SG31252, &UA2670615},
	{SG31254, &UA2670737},
	{SG31256, &UA2671635},
	{SG31258, &UA2672198},
	{SG31260, &UA2673947},
	{SG31261, &UA2674420},
	{SG31262, &UA2676032},
	{SG31263, &UA2676075},
	{SG31264, &UA2676077},
	{SG31265, &UA2676100},
	{SG31266, &UA2676200},
	{SG31267, &UA2676242},
	{SG31270, &UA2676425},
	{SG31272, &UA2676682},
	{SG31274, &UA2676684},
	{SG31276, &UA2676780},
	{SG31278, &UA2676829},
	{SG31280, &UA2677157},
	{SG31282, &UA2677162},
	{SG31284, &UA2677295},
	{SG31287, &UA2709101},
	{SG31289, &UA2710429},
	{SG31291, &UA2713247},
	{SG31292, &UA2734786},
	{SG31293, &UA2736435},
	{SG31294, &UA3047652},
	{SG31295, &UA3047676},
	{SG31402, &UA2469026},
	{SG31403, &UA2482472},
	{SG31404, &UA2560017},
	{SG31405, &UA2850166},
	{SG31460, &UA2739856},
	{SG31461, &UA2741707},
	{SG31462, &UA2741708},
	{SG31463, &UA2741718},
	{SG31464, &UA2755355},
	{SG31465, &UA2757162},
	{SG31466, &UA2799343},
	{SG31467, &UA2868576},
	{SG31468, &UA2948290},
	{SG31477, &UA2197579},
	{SG31478, &UA2233629},
	{SG31479, &UA2652627},
	{SG31485, &UA2456415},
	{SG31486, &UA2529544},
	{SG31487, &UA2559218},
	{SG31488, &UA2600023},
	{SG31489, &UA2708238},
	{SG31490, &UA2708372},
	{SG31491, &UA2896088},
	{SG31492, &UA2952513},
	{SG31493, &UA2257931},
	{SG31494, &UA2430979},
	{SG31495, &UA2486585},
	{SG31496, &UA2531769},
	{SG31498, &UA2570879},
	{SG31500, &UA2601059},
	{SG31502, &UA2658313},
	{SG31503, &UA2659335},
	{SG31504, &UA2667917},
	{SG31505, &UA2733206},
	{SG31506, &UA2992131},
	{SG31507, &UA3020092},
	{SG31508, &UA2676014},
	{SG31509, &UA2676244},
	{SG31510, &UA2676269},
	{SG31511, &UA2676285},
	{SG31512, &UA2676427},
	{SG31513, &UA2676673},
	{SG31514, &UA2677223},
	{SG31515, &UA2735620},
	{SG31516, &UA2970917},
	{SG31517, &UA2991510},
	{SG31570, &UA2740592},
	{SG31571, &UA2741702},
	{SG31572, &UA2741730},
	{SG31573, &UA2753005},
	{SG31574, &UA2753270},
	{SG31575, &UA2753280},
	{SG31576, &UA2753331},
	{SG31577, &UA2754641},
	{SG31578, &UA2754691},
	{SG31579, &UA2755048},
	{SG31580, &UA2755388},
	{SG31581, &UA2770041},
	{SG31582, &UA2879265},
	{SG31583, &UA2386402},
	{SG31603, &UA2510802},
	{SG31604, &UA2510920},
	{SG31605, &UA2583003},
	{SG31606, &UA2675824},
	{SG31607, &UA2678944},
	{SG31608, &UA2732387},
	{SG31609, &UA2800687},
	{SG31610, &UA2863501},
	{SG31611, &UA2865632},
	{SG31612, &UA2872323},
	{SG31613, &UA2957695},
	{SG31614, &UA2974967},
	{SG31615, &UA3023951},
	{SG31616, &UA3107116},
	{SG31617, &UA2483997},
	{SG31618, &UA2485006},
	{SG31619, &UA2535459},
	{SG31684, &UA2670753},
	{SG31685, &UA2670889},
	{SG31686, &UA2673085},
	{SG31687, &UA2673353},
	{SG31688, &UA2675042},
	{SG31689, &UA2675528},
	{SG31691, &UA2675844},
	{SG31693, &UA2676084},
	{SG31695, &UA2676095},
	{SG31697, &UA2676170},
	{SG31700, &UA2676573},
	{SG31703, &UA2676730},
	{SG31705, &UA2676734},
	{SG31706, &UA2676736},
	{SG31708, &UA2677229},
	{SG31709, &UA2677976},
	{SG31710, &UA2735487},
	{SG31711, &UA2861445},
	{SG31712, &UA2879049},
	{SG31713, &UA3013243},
	{SG31715, &UA2237050},
	{SG31716, &UA2237793},
	{SG31717, &UA2526635},
	{SG31718, &UA2652660},
	{SG31719, &UA2694044},
	{SG31720, &UA2994021},
	{SG31721, &UA3024136},
	{SG31734, &UA2508767},
	{SG31735, &UA2670034},
	{SG31737, &UA2974146},
	{SG31738, &UA2975944},
	{SG31739, &UA2462418},
	{SG31740, &UA2462425},
	{SG31741, &UA2462427},
	{SG31742, &UA2462430},
	{SG31743, &UA2734182},
	{SG31744, &UA2737565},
	{SG31745, &UA2766792},
	{SG31746, &UA2768739},
	{SG31747, &UA2992543},
	{SG31748, &UA3010485},
	{SG31749, &UA3103183},
	{SG31762, &UA2567109},
	{SG31763, &UA2583268},
	{SG31764, &UA2667957},
	{SG31765, &UA2712993},
	{SG31766, &UA2795453},
	{SG31767, &UA2796303},
	{SG31768, &UA2796304},
	{SG31769, &UA2869295},
	{SG31770, &UA2874503},
	{SG31771, &UA2875372},
	{SG31772, &UA2900421},
	{SG31773, &UA2953756},
	{SG31774, &UA3009608},
	{SG31775, &UA3078948},
	{SG31776, &UA3082214},
	{SG31779, &UA3111951},
	{SG31780, &UA3125486},
	{SG31794, &UA2197587},
	{SG31795, &UA2242113},
	{SG31850, &UA2974270},
	{SG31851, &UA2974665},
	{SG31890, &UA2257930},
	{SG31891, &UA2466471},
	{SG31892, &UA2580123},
	{SG31893, &UA2672411},
	{SG31894, &UA2703975},
	{SG31895, &UA2767543},
	{SG31896, &UA2985461},
	{SG31897, &UA3004157},
	{SG31898, &UA2713770},
	{SG31899, &UA2754471},
	{SG31900, &UA2781911},
	{SG31901, &UA2875565},
	{SG31902, &UA2877418},
	{SG31903, &UA2896091},
	{SG31905, &UA3020270},
	{SG31909, &UA2672459},
	{SG31910, &UA2676849},
	{SG31911, &UA2937452},
	{SG31912, &UA2946180},
	{SG31935, &UA3064783},
	{SG31936, &UA3066529},
	{SG31941, &UA2535785},
	{SG31943, &UA2563396},
	{SG31944, &UA2604382},
	{SG31945, &UA2669377},
	{SG31946, &UA2673421},
	{SG31947, &UA2741648},
	{SG31948, &UA2752781},
	{SG31949, &UA2799159},
	{SG31950, &UA2871388},
	{SG31953, &UA3125893},
	{SG31960, &UA2676282},
	{SG31963, &UA2676930},
	{SG31965, &UA2677001},
	{SG31967, &UA2677068},
	{SG31977, &UA2937702},
	{SG32095, &UA3064866},
	{SG32096, &UA3067664},
	{SG32132, &UA2585336},
	{SG32134, &UA2605942},
	{SG32135, &UA2765598},
	{SG32138, &UA2536542},
	{SG32139, &UA2670761},
	{SG32140, &UA2731141},
	{SG32141, &UA2767706},
	{SG32142, &UA2861664},
	{SG32143, &UA2937873},
	{SG32144, &UA2942547},
	{SG32147, &UA2955856},
	{SG32150, &UA2968324},
	{SG32152, &UA2969839},
	{SG32154, &UA3057706},
	{SG32155, &UA3124785},
	{SG32191, &UA2425156},
	{SG32192, &UA2509469},
	{SG32193, &UA2582138},
	{SG32194, &UA2585725},
	{SG32195, &UA2658409},
	{SG32196, &UA2659785},
	{SG32197, &UA2703635},
	{SG32198, &UA2732984},
	{SG32199, &UA2738273},
	{SG32200, &UA2750558},
	{SG32201, &UA2750975},
	{SG32202, &UA2778957},
	{SG32203, &UA2795228},
	{SG32204, &UA2892451},
	{SG32205, &UA2938082},
	{SG32206, &UA2946173},
	{SG32207, &UA2966044},
	{SG32208, &UA2971766},
	{SG32228, &UA2661615},
	{SG32229, &UA2709873},
	{SG32544, &UA2566812},
	{SG32546, &UA2583066},
	{SG32548, &UA2583194},
	{SG32550, &UA2583969},
	{SG32551, &UA2584850},
	{SG32556, &UA2355742},
	{SG32557, &UA2502710},
	{SG32558, &UA2696895},
	{SG32561, &UA2238444},
	{SG32562, &UA2355433},
	{SG32563, &UA2425081},
	{SG32564, &UA2427834},
	{SG32565, &UA2487453},
	{SG32566, &UA2652932},
	{SG32567, &UA2700877},
	{SG32568, &UA2716120},
	{SG32569, &UA2763668},
	{SG32570, &UA2767889},
	{SG32571, &UA2983895},
	{SG32574, &UA2567000},
	{SG32575, &UA2602015},
	{SG32576, &UA2670206},
	{SG32577, &UA2670218},
	{SG32591, &UA2581159},
	{SG32592, &UA2607234},
	{SG32593, &UA2702562},
	{SG32594, &UA2751448},
	{SG32595, &UA3046250},
	{SG32637, &UA2974024},
	{SG32638, &UA2974287},
	{SG32639, &UA2974614},
	{SG32641, &UA2975079},
	{SG32642, &UA3022220},
	{SG32790, &UA2741600},
	{SG32791, &UA2753888},
	{SG32799, &UA2430927},
	{SG32802, &UA2846474},
	{SG32804, &UA3048966},
	{SG32839, &UA2237048},
	{SG32847, &UA2582906},
	{SG32853, &UA2565147},
	{SG32854, &UA2597404},
	{SG32855, &UA2666904},
	{SG32856, &UA2677987},
	{SG32857, &UA2679252},
	{SG32858, &UA2712412},
	{SG32859, &UA2737042},
	{SG32860, &UA2737439},
	{SG32861, &UA2752819},
	{SG32862, &UA2778830},
	{SG32863, &UA2867782},
	{SG32864, &UA2879630},
	{SG32865, &UA2960666},
	{SG32866, &UA3049121},
	{SG32867, &UA2561251},
	{SG32868, &UA2659429},
	{SG32869, &UA2665460},
	{SG32870, &UA2667561},
	{SG32872, &UA3000388},
	{SG32896, &UA2462482},
	{SG32897, &UA2462483},
	{SG32898, &UA2561769},
	{SG32899, &UA2892714},
	{SG32900, &UA2899414},
	{SG32901, &UA2965001},
	{SG32902, &UA3006481},
	{SG32903, &UA2531341},
	{SG32904, &UA2704189},
	{SG32905, &UA2995472},
	{SG32910, &UA2261654},
	{SG32918, &UA2605309},
	{SG32920, &UA2751060},
	{SG32921, &UA2891343},
	{SG32922, &UA2945537},
	{SG32982, &UA2741711},
	{SG32983, &UA2753567},
	{SG32985, &UA2355403},
	{SG32986, &UA2555576},
	{SG32988, &UA2670773},
	{SG32989, &UA2679626},
	{SG32990, &UA2717663},
	{SG32991, &UA2795704},
	{SG32993, &UA2866689},
	{SG32998, &UA2509011},
	{SG32999, &UA2582361},
	{SG33000, &UA2604425},
	{SG33001, &UA2673699},
	{SG33002, &UA2675958},
	{SG33004, &UA2706589},
	{SG33005, &UA2770288},
	{SG33007, &UA2853365},
	{SG33009, &UA2944945},
	{SG33011, &UA2998314},
	{SG33012, &UA3019545},
	{SG33041, &UA2231011},
	{SG33042, &UA2237831},
	{SG33043, &UA2482799},
	{SG33044, &UA2727866},
	{SG33045, &UA2840850},
	{SG33076, &UA2467603},
	{SG33195, &UA2672761},
	{SG33205, &UA2424637},
	{SG33207, &UA2482924},
	{SG33209, &UA2556358},
	{SG33211, &UA2712224},
	{SG33213, &UA2868526},
	{SG33215, &UA2936766},
	{SG33216, &UA2988515},
	{SG33229, &UA2236723},
	{SG33230, &UA2560426},
	{SG33231, &UA2668941},
	{SG33232, &UA2557844},
	{SG33233, &UA2569520},
	{SG33234, &UA2988601},
	{SG33235, &UA3126739},
	{SG33236, &UA2513335},
	{SG33237, &UA2598750},
	{SG33240, &UA2201623},
	{SG33241, &UA2203710},
	{SG33242, &UA2241370},
	{SG33243, &UA2241371},
	{SG33244, &UA2241430},
	{SG33245, &UA2241436},
	{SG33248, &UA2241466},
	{SG33253, &UA2241560},
	{SG33256, &UA2241729},
	{SG33258, &UA2241740},
	{SG33260, &UA2241748},
	{SG33262, &UA2241750},
	{SG33264, &UA2241789},
	{SG33266, &UA2241820},
	{SG33268, &UA2241893},
	{SG33270, &UA2241922},
	{SG33272, &UA2241937},
	{SG33276, &UA2241996},
	{SG33278, &UA2242072},
	{SG33324, &UA2597332},
	{SG33325, &UA2669324},
	{SG33326, &UA2794164},
	{SG33327, &UA2886363},
	{SG33328, &UA3016544},
	{SG33329, &UA2513852},
	{SG33330, &UA2561610},
	{SG33331, &UA2580844},
	{SG33332, &UA2581631},
	{SG33333, &UA2679319},
	{SG33334, &UA2713322},
	{SG33336, &UA2849024},
	{SG33337, &UA2895050},
	{SG33338, &UA3022748},
	{SG33354, &UA2602576},
	{SG33355, &UA2668207},
	{SG33356, &UA2669044},
	{SG33357, &UA2700286},
	{SG33358, &UA2715165},
	{SG33359, &UA2729731},
	{SG33360, &UA2729768},
	{SG33361, &UA2838385},
	{SG33362, &UA2850592},
	{SG33363, &UA2853963},
	{SG33364, &UA3119454},
	{SG33365, &UA2676768},
	{SG33366, &UA2678955},
	{SG33367, &UA2701576},
	{SG33370, &UA2872562},
	{SG33371, &UA2974042},
	{SG33373, &UA2974441},
	{SG33375, &UA2974602},
	{SG33377, &UA2975060},
	{SG33380, &UA3058134},
	{SG33399, &UA2709570},
	{SG33401, &UA2867731},
	{SG33403, &UA2879919},
	{SG33405, &UA2880206},
	{SG33407, &UA2884703},
	{SG33409, &UA2964172},
	{SG33410, &UA3013494},
	{SG33411, &UA3017221},
	{SG33412, &UA3075319},
	{SG33423, &UA3064758},
	{SG33424, &UA3064876},
	{SG33425, &UA3065737},
	{SG33426, &UA3067645},
	{SG33427, &UA2232490},
	{SG33428, &UA2237761},
	{SG33429, &UA2384744},
	{SG33430, &UA2506123},
	{SG33431, &UA2508515},
	{SG33432, &UA2595086},
	{SG33433, &UA2558416},
	{SG33434, &UA2702351},
	{SG33435, &UA2709361},
	{SG33437, &UA2766954},
	{SG33439, &UA2779801},
	{SG33441, &UA2940411},
	{SG33443, &UA2997246},
	{SG33486, &UA2741720},
	{SG33488, &UA2752943},
	{SG33491, &UA2753227},
	{SG33494, &UA2755120},
	{SG33500, &UA3065247},
	{SG33501, &UA3065257},
	{SG33502, &UA3065367},
	{SG33503, &UA3065723},
	{SG33520, &UA3035295},
	{SG33521, &UA3036275},
	{SG33522, &UA3064765},
	{SG33523, &UA3079099},
	{SG33525, &UA3080655},
	{SG33598, &UA2557762},
	{SG33599, &UA2606348},
	{SG33606, &UA2566906},
	{SG33608, &UA2567338},
	{SG33620, &UA2973938},
	{SG33622, &UA2975016},
	{SG33623, &UA2975022},
	{SG33624, &UA2975951},
	{SG33625, &UA2481611},
	{SG33626, &UA2554876},
	{SG33627, &UA2935245},
	{SG33647, &UA2402127},
	{SG33648, &UA2432778},
	{SG33649, &UA2535551},
	{SG33650, &UA2659341},
	{SG33662, &UA2231716},
	{SG33663, &UA2385308},
	{SG33664, &UA3043465},
	{SG33665, &UA2566805},
	{SG33666, &UA2567202},
	{SG33667, &UA2666889},
	{SG33671, &UA2730124},
	{SG33672, &UA2755097},
	{SG33673, &UA2997288},
	{SG33676, &UA2566923},
	{SG33680, &UA2567581},
	{SG33682, &UA2865275},
	{SG33708, &UA2257953},
	{SG33709, &UA2422772},
	{SG33711, &UA2480157},
	{SG33713, &UA2526804},
	{SG33718, &UA2557763},
	{SG33720, &UA2570057},
	{SG33721, &UA2581996},
	{SG33722, &UA2584172},
	{SG33723, &UA2596315},
	{SG33725, &UA2702254},
	{SG33727, &UA2766462},
	{SG33728, &UA2793135},
	{SG33729, &UA2861020},
	{SG33730, &UA2871487},
	{SG33731, &UA2993171},
	{SG33737, &UA2434058},
	{SG33738, &UA2459546},
	{SG33748, &UA2505733},
	{SG33749, &UA2599444},
	{SG33751, &UA2667488},
	{SG33752, &UA2713493},
	{SG33753, &UA2734579},
	{SG33754, &UA2736608},
	{SG33794, &UA2752519},
	{SG33795, &UA2780617},
	{SG33796, &UA2784411},
	{SG33797, &UA2866182},
	{SG33799, &UA2954545},
	{SG33801, &UA3035738},
	{SG33836, &UA2361652},
	{SG33839, &UA2747169},
	{SG33842, &UA2422697},
	{SG33845, &UA2423715},
	{SG33848, &UA2424092},
	{SG33851, &UA2487344},
	{SG33854, &UA2527445},
	{SG33856, &UA2555357},
	{SG33858, &UA2698627},
	{SG33861, &UA2707488},
	{SG33863, &UA2734368},
	{SG33865, &UA2764398},
	{SG33867, &UA2769730},
	{SG33868, &UA2846977},
	{SG33869, &UA3002959},
	{SG33871, &UA3020348},
	{SG33879, &UA3064867},
	{SG33880, &UA3064975},
	{SG33882, &UA3080753},
	{SG33885, &UA2238338},
	{SG33891, &UA2567712},
	{SG33893, &UA2706129},
	{SG33895, &UA2566452},
	{SG33896, &UA2566957},
	{SG33897, &UA2567241},
	{SG33898, &UA2567250},
	{SG33899, &UA2567494},
	{SG33900, &UA2667967},
	{SG33901, &UA2706736},
	{SG33903, &UA2779961},
	{SG33910, &UA2401912},
	{SG33911, &UA2490597},
	{SG33912, &UA2974616},
	{SG33913, &UA3022712},
	{SG33916, &UA3064767},
	{SG33919, &UA2428085},
	{SG33923, &UA2672405},
	{SG33925, &UA2710087},
	{SG33928, &UA2764969},
	{SG33929, &UA2767239},
	{SG33931, &UA2783714},
	{SG33932, &UA3004786},
	{SG33933, &UA3016528},
	{SG33935, &UA3022029},
	{SG33937, &UA3050516},
	{SG33938, &UA3058181},
	{SG33939, &UA3111747},
	{SG33948, &UA2755126},
	{SG33988, &UA2231060},
	{SG33989, &UA2367253},
	{SG33990, &UA2402324},
	{SG33991, &UA2527458},
	{SG33992, &UA2746702},
	{SG33993, &UA2792383},
	{SG33994, &UA2862062},
	{SG33999, &UA2728879},
	{SG34001, &UA2864809},
	{SG34002, &UA2878289},
	{SG34004, &UA2878466},
	{SG34006, &UA3013271},
	{SG34008, &UA3091436},
	{SG34013, &UA3065333},
	{SG34016, &UA2734913},
	{SG34017, &UA2740109},
	{SG34038, &UA2671067},
	{SG34039, &UA2673224},
	{SG34040, &UA2781988},
	{SG34041, &UA3022487},
	{SG34049, &UA2358576},
	{SG34050, &UA2366978},
	{SG34051, &UA2385191},
	{SG34052, &UA2404428},
	{SG34053, &UA2404548},
	{SG34054, &UA2483281},
	{SG34055, &UA2485852},
	{SG34056, &UA2490522},
	{SG34063, &UA2530172},
	{SG34064, &UA2536314},
	{SG34065, &UA2603774},
	{SG34066, &UA2790297},
	{SG34067, &UA2997332},
	{SG34068, &UA3119444},
	{SG34069, &UA2974028},
	{SG34070, &UA3021134},
	{SG34071, &UA2502857},
	{SG34072, &UA2504584},
	{SG34073, &UA2531607},
	{SG34074, &UA2653876},
	{SG34075, &UA2663337},
	{SG34076, &UA2784039},
	{SG34077, &UA2858041},
	{SG34078, &UA2951873},
	{SG34079, &UA2988356},
	{SG34080, &UA2988740},
	{SG34081, &UA3030155},
	{SG34082, &UA2974041},
	{SG34129, &UA2556275},
	{SG34130, &UA2566880},
	{SG34132, &UA2667308},
	{SG34133, &UA2676592},
	{SG34140, &UA2607499},
	{SG34141, &UA2607503},
	{SG34142, &UA2607506},
	{SG34182, &UA2973890},
	{SG34184, &UA2974105},
	{SG34187, &UA2974158},
	{SG34261, &UA2741736},
	{SG34262, &UA2753684},
	{SG34263, &UA2756399},
	{SG34264, &UA2756411},
	{SG34273, &UA2235150},
	{SG34274, &UA2238411},
	{SG34275, &UA2258129},
	{SG34276, &UA2504587},
	{SG34277, &UA2985365},
	{SG34289, &UA2974150},
	{SG34290, &UA2570872},
	{SG34291, &UA2667427},
	{SG34316, &UA2560014},
	{SG34317, &UA2668162},
	{SG34335, &UA2235172},
	{SG34336, &UA2258126},
	{SG34337, &UA2402067},
	{SG34338, &UA2455566},
	{SG34339, &UA2794446},
	{SG34352, &UA2597379},
	{SG34354, &UA2669867},
	{SG34356, &UA2657817},
	{SG34357, &UA2717662},
	{SG34358, &UA2768285},
	{SG34359, &UA2796648},
	{SG34360, &UA3019696},
	{SG34361, &UA3023814},
	{SG34362, &UA3076919},
	{SG34363, &UA2975730},
	{SG34364, &UA3064769},
	{SG34365, &UA3067050},
	{SG34366, &UA2421510},
	{SG34368, &UA2455431},
	{SG34370, &UA2467201},
	{SG34373, &UA2480916},
	{SG34374, &UA2483186},
	{SG34376, &UA2503266},
	{SG34377, &UA2694414},
	{SG34380, &UA2765558},
	{SG34381, &UA2837532},
	{SG34382, &UA2844409},
	{SG34383, &UA2844573},
	{SG34384, &UA2848902},
	{SG34385, &UA2849192},
	{SG34386, &UA2864107},
	{SG34387, &UA2889555},
	{SG34388, &UA2942573},
	{SG34391, &UA2956613},
	{SG34392, &UA2957555},
	{SG34393, &UA2962149},
	{SG34394, &UA2988873},
	{SG34395, &UA2992933},
	{SG34396, &UA2996347},
	{SG34397, &UA3010778},
	{SG34398, &UA3015987},
	{SG34399, &UA3019690},
	{SG34400, &UA3043561},
	{SG34401, &UA3047035},
	{SG34402, &UA3049593},
	{SG34403, &UA3056701},
	{SG34405, &UA3057359},
	{SG34406, &UA3058175},
	{SG34407, &UA3073877},
	{SG34408, &UA3088670},
	{SG34409, &UA3103199},
	{SG34410, &UA3107053},
	{SG34411, &UA3121276},
	{SG34412, &UA2462397},
	{SG34413, &UA2462403},
	{SG34414, &UA2515524},
	{SG34415, &UA2564149},
	{SG34416, &UA2566150},
	{SG34417, &UA2567061},
	{SG34418, &UA2601268},
	{SG34419, &UA2602471},
	{SG34420, &UA2670099},
	{SG34421, &UA2672673},
	{SG34422, &UA2674089},
	{SG34423, &UA2704823},
	{SG34425, &UA2716440},
	{SG34426, &UA2716459},
	{SG34429, &UA2717066},
	{SG34431, &UA2767886},
	{SG34434, &UA2870440},
	{SG34435, &UA2870742},
	{SG34438, &UA2675765},
	{SG34439, &UA2715412},
	{SG34440, &UA2755636},
	{SG34441, &UA2901150},
	{SG34442, &UA2947994},
	{SG34443, &UA3065749},
	{SG34453, &UA3094848},
	{SG34454, &UA3111555},
	{SG34461, &UA2766043},
	{SG34463, &UA2768066},
	{SG34465, &UA2780094},
	{SG34466, &UA2783990},
	{SG34467, &UA2784783},
	{SG34468, &UA2962446},
	{SG34469, &UA3062270},
	{SG34470, &UA3063528},
	{SG34471, &UA3080770},
	{SG34472, &UA3095122},
	{SG34478, &UA2533512},
	{SG34479, &UA2560027},
	{SG34480, &UA2562853},
	{SG34481, &UA2564269},
	{SG34482, &UA2600856},
	{SG34483, &UA2676467},
	{SG34484, &UA2705756},
	{SG34486, &UA2741387},
	{SG34487, &UA2753829},
	{SG34489, &UA2862434},
	{SG34491, &UA2945962},
	{SG34512, &UA2955347},
	{SG34513, &UA2988476},
	{SG34514, &UA2997429},
	{SG34515, &UA2999015},
	{SG34516, &UA3100569},
	{SG34526, &UA2974017},
	{SG34527, &UA3023028},
	{SG34529, &UA2668374},
	{SG34531, &UA2732547},
	{SG34532, &UA2892280},
	{SG34534, &UA3020404},
	{SG34538, &UA3064777},
	{SG34540, &UA3094712},
	{SG34541, &UA3083758},
	{SG34558, &UA2230990},
	{SG34576, &UA2554608},
	{SG34577, &UA2710612},
	{SG34580, &UA2770010},
	{SG34582, &UA2800235},
	{SG34584, &UA2855753},
	{SG34586, &UA2867745},
	{SG34588, &UA2937261},
	{SG34590, &UA2953238},
	{SG34592, &UA2962400},
	{SG34593, &UA2993614},
	{SG34594, &UA3000713},
	{SG34596, &UA3056641},
	{SG34597, &UA3059493},
	{SG34598, &UA3060498},
	{SG34599, &UA3074268},
	{SG34600, &UA3108892},
	{SG34601, &UA3121610},
	{SG34602, &UA3127117},
	{SG34604, &UA2234371},
	{SG34605, &UA2503193},
	{SG34613, &UA2675580},
	{SG34614, &UA2707895},
	{SG34616, &UA2712896},
	{SG34617, &UA2937406},
	{SG34618, &UA2937996},
	{SG34619, &UA2717019},
	{SG34620, &UA2718179},
	{SG34622, &UA2718935},
	{SG34624, &UA2964017},
	{SG34626, &UA2999009},
	{SG34631, &UA2585301},
	{SG34633, &UA2859606},
	{SG34641, &UA2670582},
	{SG34642, &UA2675948},
	{SG34643, &UA2676181},
	{SG34644, &UA2676342},
	{SG34674, &UA2432349},
	{SG34675, &UA2231805},
	{SG34690, &UA2235163},
	{SG34691, &UA2257988},
	{SG34693, &UA2366474},
	{SG34694, &UA2768959},
	{SG34695, &UA2778106},
	{SG34718, &UA2356275},
	{SG34722, &UA3064994},
	{SG34723, &UA3114297},
	{SG34724, &UA2796312},
	{SG34725, &UA2796543},
	{SG34726, &UA2869301},
	{SG34727, &UA2894927},
	{SG34728, &UA2958625},
	{SG34729, &UA3063156},
	{SG34730, &UA3079787},
	{SG34731, &UA2676105},
	{SG34733, &UA2676171},
	{SG34737, &UA2566512},
	{SG34739, &UA2567018},
	{SG34743, &UA2748335},
	{SG34745, &UA2800123},
	{SG34747, &UA2800351},
	{SG34749, &UA2865516},
	{SG34751, &UA2996449},
	{SG34753, &UA2998293},
	{SG34755, &UA3009613},
	{SG34757, &UA3019976},
	{SG34759, &UA3028251},
	{SG34761, &UA3054862},
	{SG34763, &UA3073806},
	{SG34765, &UA3091959},
	{SG34767, &UA3121160},
	{SG34787, &UA2937833},
	{SG34789, &UA2999259},
	{SG34803, &UA2670758},
	{SG34805, &UA2670905},
	{SG34807, &UA2670902},
	{SG34810, &UA2676081},
	{SG34811, &UA2676575},
	{SG34812, &UA2713237},
	{SG34813, &UA2713854},
	{SG34814, &UA2736316},
	{SG34834, &UA2570463},
	{SG34836, &UA2772767},
	{SG34837, &UA2773082},
	{SG34838, &UA2773881},
	{SG34839, &UA2774055},
	{SG34866, &UA2426292},
	{SG34867, &UA2529018},
	{SG34868, &UA2956070},
	{SG34871, &UA3047198},
	{SG34872, &UA3099658},
	{SG34883, &UA2754642},
	{SG34884, &UA2773350},
	{SG34885, &UA2354618},
	{SG34896, &UA2945958},
	{SG34897, &UA2946986},
	{SG34899, &UA3020512},
	{SG34901, &UA3030579},
	{SG34902, &UA3031103},
	{SG34935, &UA2706005},
	{SG34937, &UA2737506},
	{SG34939, &UA2996625},
	{SG34978, &UA2555614},
	{SG34980, &UA2668781},
	{SG35008, &UA2467043},
	{SG35011, &UA2469128},
	{SG35014, &UA2471118},
	{SG35016, &UA2697837},
	{SG35018, &UA2729756},
	{SG35020, &UA2753907},
	{SG35022, &UA2791480},
	{SG35023, &UA2997543},
	{SG35025, &UA3080488},
	{SG35068, &UA2974267},
	{SG35078, &UA2658958},
	{SG35080, &UA2675609},
	{SG35082, &UA2716585},
	{SG35099, &UA2566519},
	{SG35103, &UA2566953},
	{SG35106, &UA2583374},
	{SG35139, &UA2559897},
	{SG35141, &UA2566657},
	{SG35142, &UA2667558},
	{SG35143, &UA2765168},
	{SG35145, &UA2989781},
	{SG35146, &UA2671119},
	{SG35149, &UA2672983},
	{SG35151, &UA2675630},
	{SG35153, &UA2676354},
	{SG35155, &UA2713530},
	{SG35157, &UA2716846},
	{SG35159, &UA2717077},
	{SG35166, &UA2676493},
	{SG35167, &UA2707730},
	{SG35183, &UA2741723},
	{SG35186, &UA2798991},
	{SG35189, &UA2483141},
	{SG35190, &UA2530320},
	{SG35192, &UA2675272},
	{SG35194, &UA2677289},
	{SG35245, &UA3064977},
	{SG35250, &UA2261564},
	{SG35251, &UA2566449},
	{SG35252, &UA2567501},
	{SG35257, &UA2741705},
	{SG35258, &UA2892597},
	{SG35283, &UA2261635},
	{SG35284, &UA2668899},
	{SG35285, &UA2716499},
	{SG35286, &UA2755279},
	{SG35290, &UA2423761},
	{SG35291, &UA2656471},
	{SG35293, &UA2426725},
	{SG35295, &UA2555112},
	{SG35296, &UA2751333},
	{SG35320, &UA2604380},
	{SG35322, &UA2716914},
	{SG35323, &UA2945377},
	{SG35324, &UA2947987},
	{SG35325, &UA2567560},
	{SG35326, &UA2559703},
	{SG35327, &UA2732936},
	{SG35328, &UA2768595},
	{SG35329, &UA2939578},
	{SG35330, &UA3088464},
	{SG35332, &UA3106594},
	{SG35334, &UA2974487},
	{SG35344, &UA3130738},
	{SG35345, &UA3131174},
	{SG35346, &UA3131333},
	{SG35347, &UA3131355},
	{SG35348, &UA3132613},
	{SG35349, &UA3132860},
	{SG35354, &UA2669999},
	{SG35356, &UA2675219},
	{SG35358, &UA2869764},
	{SG35360, &UA2871781},
	{SG35362, &UA2877600},
	{SG35376, &UA2583212},
	{SG35387, &UA2233789},
	{SG35389, &UA2367346},
	{SG35398, &UA2741740},
	{SG35399, &UA2753348},
	{SG35400, &UA2753621},
	{SG35406, &UA2467372},
	{SG35416, &UA2236716},
	{SG35419, &UA2258199},
	{SG35420, &UA2384457},
	{SG35422, &UA2485135},
	{SG35434, &UA3065376},
	{SG35457, &UA2596693},
	{SG35458, &UA2667704},
	{SG35460, &UA2750947},
	{SG35461, &UA2998167},
	{SG35463, &UA3062122},
	{SG35464, &UA2355371},
	{SG35465, &UA2368736},
	{SG35466, &UA2487383},
	{SG35467, &UA2657569},
	{SG35471, &UA2942037},
	{SG35472, &UA2964021},
	{SG35473, &UA2972553},
	{SG35484, &UA3022139},
	{SG35485, &UA2586653},
	{SG35486, &UA2768933},
	{SG35487, &UA3015289},
	{SG35488, &UA3049856},
	{SG35489, &UA3108551},
	{SG35490, &UA3130871},
	{SG35491, &UA3131348},
	{SG35492, &UA3131470},
	{SG35493, &UA3131643},
	{SG35494, &UA3131758},
	{SG35495, &UA3132007},
	{SG35496, &UA3132013},
	{SG35497, &UA3132708},
	{SG35498, &UA3132872},
	{SG35499, &UA3132949},
	{SG35500, &UA3064878},
	{SG35504, &UA2384725},
	{SG35506, &UA3064992},
	{SG35507, &UA3080212},
	{SG35508, &UA3080825},
	{SG35509, &UA3081228},
	{SG35510, &UA3081614},
	{SG35511, &UA3083178},
	{SG35512, &UA3094214},
	{SG35513, &UA3109651},
	{SG35515, &UA2974144},
	{SG35516, &UA2231055},
	{SG35517, &UA2737900},
	{SG35518, &UA2974149},
	{SG35519, &UA2562723},
	{SG35520, &UA3065130},
	{SG35521, &UA2974361},
	{SG35522, &UA3065480},
	{SG35523, &UA2973904},
	{SG35524, &UA2974380},
	{SG35525, &UA2974734},
	{SG35526, &UA3021383},
	{SG35527, &UA3022051},
	{SG35528, &UA3022647},
	{SG35529, &UA3023084},
	{SG35530, &UA3030027},
	{SG35538, &UA2487372},
	{SG35539, &UA2554835},
	{SG35540, &UA2668258},
	{SG35541, &UA2754413},
	{SG35542, &UA2790748},
	{SG35544, &UA2792007},
	{SG35545, &UA2870608},
	{SG35617, &UA2857283},
	{SG35619, &UA2886371},
	{SG35624, &UA3064974},
	{SG35626, &UA3065150},
	{SG35649, &UA2753088},
	{SG35651, &UA2753282},
	{SG35653, &UA2753328},
	{SG35655, &UA2896309},
	{SG35658, &UA2697577},
	{SG35660, &UA2943991},
	{SG35666, &UA3064771},
	{SG35668, &UA3065154},
	{SG35675, &UA2238892},
	{SG35676, &UA2402824},
	{SG35677, &UA2563039},
	{SG35678, &UA2666606},
	{SG35679, &UA3028235},
	{SG35680, &UA2633638},
	{SG35683, &UA2973947},
	{SG35685, &UA2633689},
	{SG35695, &UA2974449},
	{SG35697, &UA2974478},
	{SG35700, &UA2974600},
	{SG35706, &UA2796922},
	{SG35707, &UA2866793},
	{SG35708, &UA2878710},
	{SG35710, &UA2879385},
	{SG35713, &UA2957620},
	{SG35714, &UA3001462},
	{SG35715, &UA3006120},
	{SG35716, &UA3020450},
	{SG35717, &UA3050131},
	{SG35719, &UA3064648},
	{SG35730, &UA3024575},
	{SG35733, &UA2974083},
	{SG35740, &UA2672372},
	{SG35743, &UA2974796},
	{SG35759, &UA2560471},
	{SG35760, &UA2703367},
	{SG35763, &UA2670897},
	{SG35765, &UA2676176},
	{SG35768, &UA2974155},
	{SG35771, &UA2974454},
	{SG35778, &UA2860876},
	{SG35794, &UA2947367},
	{SG35795, &UA3030578},
	{SG35797, &UA3031224},
	{SG35798, &UA3031484},
	{SG35800, &UA3032696},
	{SG35801, &UA3032888},
	{SG35802, &UA3059598},
	{SG35805, &UA2676978},
	{SG35807, &UA2238526},
	{SG35808, &UA2261568},
	{SG35809, &UA2468301},
	{SG35810, &UA2578069},
	{SG35812, &UA2596117},
	{SG35813, &UA2596563},
	{SG35815, &UA2604117},
	{SG35816, &UA2727506},
	{SG35817, &UA2778464},
	{SG35818, &UA2790647},
	{SG35824, &UA3065081},
	{SG35825, &UA2261636},
	{SG35830, &UA2232119},
	{SG35833, &UA2242374},
	{SG35834, &UA2455971},
	{SG35837, &UA2566505},
	{SG35839, &UA2566816},
	{SG35840, &UA2668975},
	{SG35933, &UA2516494},
	{SG35949, &UA2566756},
	{SG35960, &UA2481509},
	{SG35981, &UA2238456},
	{SG35983, &UA2425314},
	{SG35986, &UA2503795},
	{SG36003, &UA3027718},
	{SG36006, &UA3027890},
	{SG36009, &UA3029047},
	{SG36011, &UA3030126},
	{SG36012, &UA3036004},
	{SG36049, &UA3081352},
	{SG36061, &UA3065386},
	{SG36062, &UA3065377},
	{SG36063, &UA3065611},
	{SG36064, &UA3067075},
	{SG36066, &UA2974280},
	{SG36067, &UA2974383},
	{SG36072, &UA2974895},
	{SG36074, &UA2708203},
	{SG36077, &UA2566492},
	{SG36089, &UA2616789},
	{SG36091, &UA2618404},
	{SG36094, &UA2620411},
	{SG36096, &UA2630231},
	{SG36099, &UA3121017},
	{SG36102, &UA2586658},
	{SG36122, &UA2660593},
	{SG36124, &UA2670733},
	{SG36125, &UA2671215},
	{SG36126, &UA2676173},
	{SG36162, &UA2570155},
	{SG36164, &UA2772747},
	{SG36165, &UA2773253},
	{SG36166, &UA2773279},
	{SG36168, &UA2773909},
	{SG36170, &UA2774049},
	{SG36172, &UA2783704},
	{SG36174, &UA2796776},
	{SG36176, &UA2937752},
	{SG36189, &UA2947549},
	{SG36190, &UA2966401},
	{SG36191, &UA3014138},
	{SG36192, &UA3031250},
	{SG36193, &UA3031519},
	{SG36195, &UA3032882},
	{SG36197, &UA3038894},
	{SG36198, &UA3092766},
	{SG36202, &UA2532480},
	{SG36203, &UA2560202},
	{SG36204, &UA2563536},
	{SG36205, &UA2694321},
	{SG36206, &UA2711363},
	{SG36208, &UA2731974},
	{SG36209, &UA2779656},
	{SG36212, &UA3062361},
	{SG36242, &UA2755530},
	{SG36243, &UA2749786},
	{SG36244, &UA2893003},
	{SG36245, &UA3000532},
	{SG36246, &UA2741542},
	{SG36247, &UA2753009},
	{SG36248, &UA2421463},
	{SG36249, &UA2403100},
	{SG36250, &UA2564882},
	{SG36251, &UA2668720},
	{SG36252, &UA3065477},
	{SG36290, &UA2462479},
	{SG36292, &UA2974483},
	{SG36293, &UA2974282},
	{SG36294, &UA3022195},
	{SG36297, &UA2238236},
	{SG36299, &UA2456476},
	{SG36300, &UA2586621},
	{SG36301, &UA2670741},
	{SG36303, &UA2673213},
	{SG36314, &UA2708400},
	{SG36318, &UA2740105},
	{SG36332, &UA2698241},
	{SG36333, &UA2863567},
	{SG36334, &UA2865333},
	{SG36335, &UA3073662},
	{SG36336, &UA3108506},
	{SG36339, &UA2781960},
	{SG36341, &UA2869482},
	{SG36343, &UA2869598},
	{SG36344, &UA2869888},
	{SG36345, &UA2893385},
	{SG36347, &UA3066829},
	{SG36349, &UA3079473},
	{SG36357, &UA2677231},
	{SG36358, &UA2713167},
	{SG36362, &UA2973962},
	{SG36363, &UA2974033},
	{SG36364, &UA2748103},
	{SG36365, &UA2937438},
	{SG36371, &UA2844838},
	{SG36372, &UA2961622},
	{SG36382, &UA3065102},
	{SG36399, &UA3130745},
	{SG36402, &UA3131081},
	{SG36405, &UA3131353},
	{SG36406, &UA3131469},
	{SG36409, &UA3131663},
	{SG36411, &UA3131682},
	{SG36413, &UA3131718},
	{SG36415, &UA2974043},
	{SG36416, &UA3126672},
	{SG36424, &UA2599973},
	{SG36473, &UA2974137},
	{SG36475, &UA2484555},
	{SG36476, &UA2583708},
	{SG36477, &UA2469371},
	{SG36478, &UA2660070},
	{SG36479, &UA2892839},
	{SG36533, &UA2681004},
	{SG36535, &UA2959591},
	{SG36537, &UA3124490},
	{SG36540, &UA2584090},
	{SG36570, &UA2772410},
	{SG36572, &UA2539251},
	{SG36574, &UA2606520},
	{SG36575, &UA2974140},
	{SG36576, &UA2710257},
	{SG36577, &UA2937287},
	{SG36578, &UA2962779},
	{SG36583, &UA2667202},
	{SG36584, &UA2566796},
	{SG36585, &UA2566968},
	{SG36586, &UA2755560},
	{SG36587, &UA3027199},
	{SG36588, &UA3027747},
	{SG36589, &UA3022204},
	{SG36620, &UA2555215},
	{SG36621, &UA2727987},
	{SG36622, &UA2778459},
	{SG36625, &UA2697919},
	{SG36626, &UA2703115},
	{SG36627, &UA2713482},
	{SG36628, &UA2714622},
	{SG36629, &UA2729529},
	{SG36630, &UA2781850},
	{SG36631, &UA2861000},
	{SG36632, &UA2986807},
	{SG36633, &UA2992684},
	{SG36635, &UA2994016},
	{SG36636, &UA2997320},
	{SG36638, &UA2998315},
	{SG36639, &UA2999268},
	{SG36640, &UA3049049},
	{SG36642, &UA3049583},
	{SG36644, &UA3062379},
	{SG36645, &UA3074340},
	{SG36646, &UA3078247},
	{SG36647, &UA3102784},
	{SG36649, &UA3109365},
	{SG36651, &UA3114663},
	{SG36652, &UA2755539},
	{SG36658, &UA2355053},
	{SG36659, &UA2622735},
	{SG36660, &UA2627315},
	{SG36661, &UA2632155},
	{SG36664, &UA2974913},
	{SG36698, &UA2974996},
	{SG36699, &UA2975038},
	{SG36700, &UA2975086},
	{SG36701, &UA2763894},
	{SG36702, &UA2891875},
	{SG36707, &UA2892771},
	{SG36711, &UA2741741},
	{SG36713, &UA3063679},
	{SG36714, &UA2754263},
	{SG36782, &UA2741735},
	{SG36783, &UA2769349},
	{SG36785, &UA2769351},
	{SG36786, &UA2769385},
	{SG36787, &UA2783454},
	{SG36788, &UA2798394},
	{SG36790, &UA2892470},
	{SG36810, &UA2679143},
	{SG36812, &UA2713271},
	{SG36814, &UA2716744},
	{SG36815, &UA2795529},
	{SG36817, &UA2862708},
	{SG36819, &UA2865458},
	{SG36822, &UA3016212},
	{SG36824, &UA3049725},
	{SG36839, &UA2701808},
	{SG36841, &UA2767968},
	{SG36845, &UA2956007},
	{SG36881, &UA2958996},
	{SG36882, &UA2998093},
	{SG36884, &UA3001588},
	{SG36886, &UA3107020},
	{SG36897, &UA2974272},
	{SG36899, &UA2754345},
	{SG36908, &UA2957087},
	{SG36909, &UA3021202},
	{SG36917, &UA2664479},
	{SG36918, &UA3080252},
	{SG36919, &UA3080303},
	{SG36921, &UA3081224},
	{SG36923, &UA3081783},
	{SG36926, &UA3083270},
	{SG36929, &UA3094095},
	{SG36930, &UA3094185},
	{SG36934, &UA3094798},
	{SG36935, &UA3094980},
	{SG36936, &UA3108394},
	{SG36937, &UA3109375},
	{SG36938, &UA3109928},
	{SG36939, &UA3110788},
	{SG36940, &UA3110933},
	{SG36942, &UA3113520},
	{SG36944, &UA3114300},
	{SG36945, &UA3114302},
	{SG36947, &UA3114357},
	{SG36948, &UA3114563},
	{SG36949, &UA3114853},
	{SG36950, &UA3114927},
	{SG36951, &UA3126222},
	{SG36952, &UA3126261},
	{SG36954, &UA3126471},
	{SG36959, &UA3126521},
	{SG36961, &UA3126622},
	{SG36963, &UA3126785},
	{SG36965, &UA3126786},
	{SG36966, &UA3126979},
	{SG36967, &UA3126980},
	{SG36968, &UA3127018},
	{SG36969, &UA3127317},
	{SG36970, &UA3127353},
	{SG36971, &UA3127399},
	{SG36973, &UA3127904},
	{SG36975, &UA3128442},
	{SG36977, &UA2261656},
	{SG36978, &UA2712356},
	{SG36979, &UA3074001},
	{SG36984, &UA2773114},
	{SG36985, &UA2773276},
	{SG36987, &UA2773888},
	{SG36989, &UA2784888},
	{SG36992, &UA2887662},
	{SG36994, &UA2897702},
	{SG36995, &UA3121466},
	{SG37006, &UA2566746},
	{SG37009, &UA2567125},
	{SG37010, &UA2584862},
	{SG37011, &UA2600972},
	{SG37012, &UA2603859},
	{SG37013, &UA2667172},
	{SG37014, &UA2937054},
	{SG37015, &UA2511376},
	{SG37016, &UA2670560},
	{SG37017, &UA2676771},
	{SG37019, &UA2942870},
	{SG37020, &UA2955263},
	{SG37022, &UA2755109},
	{SG37026, &UA3064661},
	{SG37029, &UA2754241},
	{SG37030, &UA2755060},
	{SG37031, &UA2755113},
	{SG37036, &UA2713562},
	{SG37037, &UA2865720},
	{SG37038, &UA2867266},
	{SG37039, &UA2872497},
	{SG37041, &UA2943397},
	{SG37042, &UA2963442},
	{SG37043, &UA2996911},
	{SG37044, &UA2996955},
	{SG37045, &UA2997151},
	{SG37046, &UA2998421},
	{SG37047, &UA2998513},
	{SG37048, &UA3010733},
	{SG37049, &UA3048800},
	{SG37050, &UA3088770},
	{SG37051, &UA3091501},
	{SG37052, &UA2672528},
	{SG37057, &UA2536944},
	{SG37059, &UA2701815},
	{SG37061, &UA3102232},
	{SG37063, &UA2676479},
	{SG37065, &UA2676641},
	{SG37071, &UA2527533},
	{SG37072, &UA3021236},
	{SG37073, &UA3065249},
	{SG37075, &UA3094926},
	{SG37087, &UA2974275},
	{SG37089, &UA2975073},
	{SG37093, &UA2784861},
	{SG37108, &UA2658655},
	{SG37109, &UA2781151},
	{SG37110, &UA3102281},
	{SG37131, &UA2974603},
	{SG37163, &UA2755519},
	{SG37166, &UA2973488},
	{SG37172, &UA3065335},
	{SG37174, &UA2258000},
	{SG37175, &UA2616462},
	{SG37176, &UA2617160},
	{SG37177, &UA2619906},
	{SG37178, &UA2630583},
	{SG37183, &UA2958773},
	{SG37184, &UA3018849},
	{SG37185, &UA3020643},
	{SG37191, &UA2754582},
	{SG37192, &UA2769451},
	{SG37195, &UA2783532},
	{SG37312, &UA3065132},
	{SG37313, &UA2585293},
	{SG37315, &UA2606843},
	{SG37322, &UA2673743},
	{SG37324, &UA2675469},
	{SG37327, &UA2866145},
	{SG37329, &UA2617682},
	{SG37332, &UA2622838},
	{SG37334, &UA2675311},
	{SG37335, &UA2783184},
	{SG37336, &UA3121168},
	{SG37338, &UA2676629},
	{SG37341, &UA2678565},
	{SG37350, &UA2462420},
	{SG37353, &UA2599561},
	{SG37362, &UA2676374},
	{SG37364, &UA2676492},
	{SG37365, &UA2755115},
	{SG37375, &UA2676994},
	{SG37376, &UA2678424},
	{SG37377, &UA2860573},
	{SG37378, &UA2716996},
	{SG37379, &UA2961253},
	{SG37381, &UA2997750},
	{SG37383, &UA3012787},
	{SG37384, &UA3022245},
	{SG37385, &UA3092521},
	{SG37386, &UA2670898},
	{SG37388, &UA2676433},
	{SG37390, &UA2738346},
	{SG37391, &UA2769751},
	{SG37392, &UA2895813},
	{SG37393, &UA2965446},
	{SG37394, &UA3015653},
	{SG37395, &UA3019728},
	{SG37396, &UA3124254},
	{SG37419, &UA2754153},
	{SG37429, &UA2237959},
	{SG37431, &UA3093979},
	{SG37443, &UA2233838},
	{SG37445, &UA2232116},
	{SG37447, &UA2238345},
	{SG37448, &UA2738647},
	{SG37453, &UA2974605},
	{SG37460, &UA3065591},
	{SG37461, &UA3068195},
	{SG37463, &UA3093579},
	{SG37468, &UA2044227},
	{SG37469, &UA2402358},
	{SG37474, &UA2974619},
	{SG37548, &UA2672391},
	{SG37549, &UA2860012},
	{SG37550, &UA2868559},
	{SG37551, &UA3002883},
	{SG37552, &UA2583362},
	{SG37553, &UA2703286},
	{SG37554, &UA2671474},
	{SG37555, &UA2853212},
	{SG37580, &UA3065385},
	{SG37582, &UA2755123},
	{SG37583, &UA2595672},
	{SG37585, &UA2695654},
	{SG37586, &UA2705034},
	{SG37587, &UA2707389},
	{SG37589, &UA2779407},
	{SG37590, &UA2935993},
	{SG37592, &UA2997798},
	{SG37594, &UA3121281},
	{SG37597, &UA2747531},
	{SG37598, &UA2795584},
	{SG37599, &UA2957703},
	{SG37600, &UA2773525},
	{SG37610, &UA3066873},
	{SG37710, &UA2234669},
	{SG37711, &UA2258039},
	{SG37713, &UA2844958},
	{SG37715, &UA2554152},
	{SG37716, &UA2700472},
	{SG37730, &UA2676094},
	{SG37733, &UA3017280},
	{SG37734, &UA3031218},
	{SG37745, &UA3032700},
	{SG37748, &UA3033325},
	{SG37756, &UA2579787},
	{SG37757, &UA2867401},
	{SG37765, &UA2433997},
	{SG37772, &UA2753046},
	{SG37773, &UA2754331},
	{SG37774, &UA2488408},
	{SG37775, &UA2557994},
	{SG37776, &UA2234194},
	{SG37777, &UA2569595},
	{SG37778, &UA2570858},
	{SG37779, &UA2583958},
	{SG37780, &UA2596788},
	{SG37781, &UA2672748},
	{SG37782, &UA2678020},
	{SG37783, &UA2713621},
	{SG37784, &UA2793317},
	{SG37785, &UA2877703},
	{SG37786, &UA2870624},
	{SG37788, &UA2672371},
	{SG37789, &UA2674209},
	{SG37790, &UA2676442},
	{SG37792, &UA2676633},
	{SG37794, &UA2706842},
	{SG37795, &UA2707912},
	{SG37797, &UA2719405},
	{SG37799, &UA2559240},
	{SG37809, &UA2876904},
	{SG37810, &UA2899033},
	{SG37811, &UA2566630},
	{SG37812, &UA2567130},
	{SG37813, &UA2753272},
	{SG37814, &UA2754709},
	{SG37815, &UA2755387},
	{SG37823, &UA2566691},
	{SG37824, &UA2671098},
	{SG37825, &UA2676035},
	{SG37835, &UA3065728},
	{SG37836, &UA3065984},
	{SG37837, &UA3083238},
	{SG37840, &UA2796461},
	{SG37852, &UA2755347},
	{SG37856, &UA2570009},
	{SG37857, &UA2772905},
	{SG37859, &UA2774284},
	{SG37861, &UA2782830},
	{SG37863, &UA2937875},
	{SG37872, &UA2257929},
	{SG37874, &UA2487259},
	{SG37883, &UA2565553},
	{SG37885, &UA2583263},
	{SG37887, &UA2704221},
	{SG37889, &UA2710304},
	{SG37890, &UA2752303},
	{SG37891, &UA2861804},
	{SG37892, &UA3017444},
	{SG37897, &UA2974810},
	{SG37899, &UA2672680},
	{SG37916, &UA3065577},
	{SG37932, &UA2734928},
	{SG37937, &UA2974382},
	{SG37943, &UA2754405},
	{SG37945, &UA2755599},
	{SG37949, &UA3065242},
	{SG37950, &UA3081629},
	{SG37952, &UA2676246},
	{SG37956, &UA2670893},
	{SG37960, &UA2754163},
	{SG37963, &UA2974906},
	{SG37964, &UA2429903},
	{SG37965, &UA2657203},
	{SG37993, &UA2673250},
	{SG37994, &UA2676603},
	{SG37999, &UA2234205},
	{SG38000, &UA2728720},
	{SG38001, &UA2879037},
	{SG38004, &UA2755351},
	{SG38015, &UA2710208},
	{SG38016, &UA2767166},
	{SG38017, &UA2861322},
	{SG38018, &UA2868515},
	{SG38019, &UA2877622},
	{SG38030, &UA2770106},
	{SG38083, &UA2704750},
	{SG38085, &UA2768382},
	{SG38087, &UA2863318},
	{SG38089, &UA2867137},
	{SG38091, &UA2566689},
	{SG38113, &UA2257853},
	{SG38115, &UA2427716},
	{SG38118, &UA2662320},
	{SG38121, &UA2973662},
	{SG38123, &UA2974139},
	{SG38134, &UA2607496},
	{SG38135, &UA3065341},
	{SG38138, &UA2741633},
	{SG38139, &UA2753385},
	{SG38140, &UA2772862},
	{SG38141, &UA2998470},
	{SG38142, &UA3057290},
	{SG38143, &UA2974286},
	{SG38181, &UA2257950},
	{SG38182, &UA2595712},
	{SG38183, &UA2996409},
	{SG38184, &UA3000016},
	{SG38186, &UA3078429},
	{SG38187, &UA3065467},
	{SG38194, &UA2754942},
	{SG38195, &UA2671110},
	{SG38197, &UA2676178},
	{SG38198, &UA2676827},
	{SG38200, &UA3087624},
	{SG38211, &UA2873618},
	{SG38212, &UA2483328},
	{SG38213, &UA2557321},
	{SG38214, &UA3073512},
	{SG38219, &UA2754420},
	{SG38220, &UA2773344},
	{SG38225, &UA2877839},
	{SG38226, &UA2999671},
	{SG38227, &UA3001125},
	{SG38230, &UA2784167},
	{SG38241, &UA2676085},
	{SG38247, &UA2233262},
	{SG38253, &UA2231074},
	{SG38255, &UA2261634},
	{SG38259, &UA2974147},
	{SG38265, &UA2555074},
	{SG38267, &UA2729693},
	{SG38272, &UA2675684},
	{SG38275, &UA2566509},
	{SG38277, &UA2567696},
	{SG38291, &UA3066533},
	{SG38292, &UA2938653},
	{SG38294, &UA2968736},
	{SG38301, &UA3065325},
	{SG38304, &UA3066893},
	{SG38307, &UA2673946},
	{SG38308, &UA2676348},
	{SG38337, &UA2676604},
	{SG38338, &UA2676850},
	{SG38339, &UA2709240},
	{SG38340, &UA2974794},
	{SG38341, &UA2974905},
	{SG38347, &UA2681028},
	{SG38349, &UA2961813},
	{SG38350, &UA3015945},
	{SG38351, &UA3017043},
	{SG38352, &UA3057775},
	{SG38354, &UA3058236},
	{SG38356, &UA3059797},
	{SG38357, &UA3125444},
	{SG38359, &UA2755354},
	{SG38360, &UA2974325},
	{SG38364, &UA2798753},
	{SG38373, &UA2741557},
	{SG38377, &UA3066889},
	{SG38378, &UA3080836},
	{SG38379, &UA2261535},
	{SG38380, &UA2712819},
	{SG38381, &UA2861513},
	{SG38382, &UA2985038},
	{SG38383, &UA3007610},
	{SG38384, &UA3065478},
	{SG38391, &UA2731018},
	{SG38392, &UA2765523},
	{SG38393, &UA2871708},
	{SG38394, &UA2956986},
	{SG38395, &UA2988657},
	{SG38396, &UA3005039},
	{SG38398, &UA3049752},
	{SG38400, &UA3059872},
	{SG38401, &UA3075992},
	{SG38402, &UA3102199},
	{SG38403, &UA3120374},
	{SG38404, &UA3124969},
	{SG38407, &UA2755311},
	{SG38427, &UA2753870},
	{SG38445, &UA2754271},
	{SG38448, &UA2505393},
	{SG38449, &UA2796801},
	{SG38451, &UA3083632},
	{SG38462, &UA2755542},
	{SG38482, &UA3066872},
	{SG38483, &UA2257888},
	{SG38490, &UA2604689},
	{SG38491, &UA2672239},
	{SG38492, &UA2738830},
	{SG38493, &UA3101461},
	{SG38494, &UA3102254},
	{SG38500, &UA3024025},
	{SG38503, &UA2973666},
	{SG38505, &UA2974799},
	{SG38507, &UA2662936},
	{SG38508, &UA2799414},
	{SG38510, &UA2969234},
	{SG38512, &UA3018056},
	{SG38514, &UA3024716},
	{SG38516, &UA3101414},
	{SG38517, &UA3102973},
	{SG38519, &UA2769390},
	{SG38520, &UA2796541},
	{SG38521, &UA2869561},
	{SG38522, &UA2583167},
	{SG38523, &UA2670035},
	{SG38536, &UA2567009},
	{SG38537, &UA2583697},
	{SG38540, &UA2944326},
	{SG38541, &UA2964603},
	{SG38542, &UA2971340},
	{SG38544, &UA3006572},
	{SG38545, &UA3020746},
	{SG38546, &UA3058324},
	{SG38547, &UA2783270},
	{SG38554, &UA2748484},
	{SG38556, &UA2784800},
	{SG38558, &UA2879141},
	{SG38560, &UA2995096},
	{SG38561, &UA2997299},
	{SG38562, &UA3103229},
	{SG38566, &UA2974145},
	{SG38568, &UA2976139},
	{SG38569, &UA2567690},
	{SG38573, &UA2566813},
	{SG38574, &UA2583293},
	{SG38575, &UA2603302},
	{SG38577, &UA2988292},
	{SG38581, &UA2770085},
	{SG38582, &UA2772140},
	{SG38588, &UA2674217},
	{SG38589, &UA2674219},
	{SG38590, &UA2714073},
	{SG38591, &UA2734422},
	{SG38593, &UA2795823},
	{SG38594, &UA2796766},
	{SG38595, &UA2879021},
	{SG38596, &UA2991242},
	{SG38597, &UA3029069},
	{SG38599, &UA2754851},
	{SG38601, &UA2529757},
	{SG38602, &UA2935313},
	{SG38606, &UA2566530},
	{SG38616, &UA2769481},
	{SG38617, &UA2770709},
	{SG38618, &UA2879523},
	{SG38619, &UA2997388},
	{SG38620, &UA2974386},
	{SG38622, &UA2235647},
	{SG38624, &UA2765493},
	{SG38625, &UA2894488},
	{SG38627, &UA3064993},
	{SG38628, &UA3083476},
	{SG38640, &UA2956254},
	{SG38641, &UA3027893},
	{SG38642, &UA2735243},
	{SG38643, &UA2780705},
	{SG38644, &UA2897087},
	{SG38645, &UA2937497},
	{SG38646, &UA2998389},
	{SG38647, &UA3019674},
	{SG38648, &UA3021280},
	{SG38649, &UA3107562},
	{SG38652, &UA3108085},
	{SG38664, &UA2707756},
	{SG38665, &UA2713173},
	{SG38666, &UA2734894},
	{SG38668, &UA2736022},
	{SG38671, &UA2964347},
	{SG38673, &UA3016820},
	{SG38678, &UA3072887},
	{SG38679, &UA3072916},
	{SG38681, &UA3078334},
	{SG38684, &UA3107993},
	{SG38686, &UA3108084},
	{SG38688, &UA3109272},
	{SG38689, &UA3121581},
	{SG38690, &UA3124954},
	{SG38691, &UA3126064},
	{SG38692, &UA3127524},
	{SG38698, &UA2891018},
	{SG38700, &UA3018389},
	{SG38703, &UA2258198},
	{SG38706, &UA2754225},
	{SG38707, &UA2973903},
	{SG38708, &UA2975039},
	{SG38709, &UA2237864},
	{SG38711, &UA3071046},
	{SG38714, &UA3064987},
	{SG38721, &UA3020800},
	{SG38722, &UA3021270},
	{SG38723, &UA3021492},
	{SG38724, &UA3021495},
	{SG38726, &UA3021665},
	{SG38728, &UA3021809},
	{SG38730, &UA3022000},
	{SG38734, &UA3022626},
	{SG38736, &UA3022681},
	{SG38739, &UA3023410},
	{SG38742, &UA3023445},
	{SG38744, &UA3023915},
	{SG38747, &UA3024040},
	{SG38749, &UA3027010},
	{SG38751, &UA3057717},
	{SG38753, &UA3058060},
	{SG38754, &UA3058119},
	{SG38756, &UA3059558},
	{SG38758, &UA3077930},
	{SG38796, &UA3078733},
	{SG38798, &UA3092399},
	{SG38800, &UA3126199},
	{SG38802, &UA2665521},
	{SG38803, &UA2737345},
	{SG38804, &UA2999084},
	{SG38805, &UA3062782},
	{SG38828, &UA3065569},
	{SG38829, &UA2973482},
	{SG38830, &UA2975718},
	{SG38834, &UA2797164},
	{SG38836, &UA2838839},
	{SG38843, &UA2367096},
	{SG38846, &UA3087253},
	{SG38856, &UA2676329},
	{SG38857, &UA2676739},
	{SG38866, &UA2567524},
	{SG38867, &UA2601242},
	{SG38875, &UA2672473},
	{SG38876, &UA2674245},
	{SG38878, &UA2676679},
	{SG38880, &UA2715722},
	{SG38883, &UA2734560},
	{SG38884, &UA2996451},
	{SG38887, &UA2676588},
	{SG38888, &UA2752959},
	{SG38889, &UA2754801},
	{SG38894, &UA2973269},
	{SG38895, &UA3067198},
	{SG38897, &UA3067186},
	{SG38899, &UA3082891},
	{SG38900, &UA3083205},
	{SG38901, &UA3109307},
	{SG38912, &UA3065250},
	{SG38914, &UA3080313},
	{SG38924, &UA2237682},
	{SG38937, &UA2361546},
	{SG38951, &UA3065390},
	{SG38952, &UA3083194},
	{SG38962, &UA2537568},
	{SG38963, &UA2702101},
	{SG38975, &UA2676082},
	{SG38977, &UA2676668},
	{SG38979, &UA2706975},
	{SG38984, &UA2736643},
	{SG38987, &UA2878316},
	{SG38989, &UA2998325},
	{SG38996, &UA2354899},
	{SG38997, &UA2554738},
	{SG39013, &UA2586640},
	{SG39015, &UA2718750},
	{SG39017, &UA2044115},
	{SG39018, &UA2756578},
	{SG39021, &UA3032941},
	{SG39022, &UA3039175},
	{SG39023, &UA3061649},
	{SG39040, &UA2754577},
	{SG39041, &UA2975099},
	{SG39042, &UA2975932},
	{SG39051, &UA2566496},
	{SG39054, &UA2583227},
	{SG39057, &UA2403647},
	{SG39059, &UA2753128},
	{SG39061, &UA3039402},
	{SG39063, &UA3058466},
	{SG39064, &UA3065074},
	{SG39065, &UA3065548},
	{SG39066, &UA3108747},
	{SG39072, &UA2875247},
	{SG39073, &UA2975082},
	{SG39076, &UA2734721},
	{SG39077, &UA3065090},
	{SG39081, &UA2676276},
	{SG39084, &UA3096282},
	{SG39085, &UA3096296},
	{SG39087, &UA3096365},
	{SG39090, &UA3129224},
	{SG39095, &UA2891258},
	{SG39096, &UA2965580},
	{SG39098, &UA3017527},
	{SG39104, &UA2998305},
	{SG39106, &UA3026054},
	{SG39107, &UA2508179},
	{SG39108, &UA2712759},
	{SG39109, &UA2713469},
	{SG39110, &UA2857098},
	{SG39111, &UA2870015},
	{SG39112, &UA3023226},
	{SG39117, &UA2672612},
	{SG39119, &UA2674847},
	{SG39121, &UA2957259},
	{SG39122, &UA3022954},
	{SG39123, &UA2406141},
	{SG39124, &UA2483087},
	{SG39176, &UA2671103},
	{SG39178, &UA2676279},
	{SG39179, &UA2718640},
	{SG39270, &UA2676487},
	{SG39273, &UA2567006},
	{SG39274, &UA2602806},
	{SG39275, &UA2669917},
	{SG39277, &UA2672337},
	{SG39283, &UA2586622},
	{SG39284, &UA2864624},
	{SG39285, &UA2892414},
	{SG39286, &UA2895202},
	{SG39287, &UA3058097},
	{SG39288, &UA3059532},
	{SG39289, &UA3080343},
	{SG39291, &UA2232485},
	{SG39292, &UA2237778},
	{SG39293, &UA2867397},
	{SG39294, &UA2955943},
	{SG39295, &UA2970587},
	{SG39296, &UA2996569},
	{SG39297, &UA3000155},
	{SG39298, &UA3016593},
	{SG39299, &UA3023893},
	{SG39300, &UA3061199},
	{SG39304, &UA2238302},
	{SG39305, &UA2261570},
	{SG39306, &UA2367026},
	{SG39307, &UA2713389},
	{SG39309, &UA2754687},
	{SG39316, &UA2738227},
	{SG39317, &UA2768540},
	{SG39318, &UA2879361},
	{SG39319, &UA2954957},
	{SG39320, &UA2995975},
	{SG39324, &UA2754531},
	{SG39326, &UA3065727},
	{SG39355, &UA2974623},
	{SG39369, &UA2566699},
	{SG39370, &UA2666967},
	{SG39372, &UA2755557},
	{SG39373, &UA2797654},
	{SG39374, &UA3019372},
	{SG39375, &UA3090052},
	{SG39376, &UA3130801},
	{SG39377, &UA3132012},
	{SG39378, &UA3067068},
	{SG39385, &UA2880209},
	{SG39386, &UA2944354},
	{SG39393, &UA2974501},
	{SG39394, &UA2975037},
	{SG39395, &UA3022484},
	{SG39429, &UA2470072},
	{SG39430, &UA2998975},
	{SG39431, &UA2537328},
	{SG39432, &UA2600129},
	{SG39434, &UA2603708},
	{SG39435, &UA2670088},
	{SG39437, &UA2503071},
	{SG39440, &UA2974816},
	{SG39441, &UA2231063},
	{SG39443, &UA2554915},
	{SG39458, &UA2355268},
	{SG39463, &UA2719032},
	{SG39464, &UA2738195},
	{SG39468, &UA2676328},
	{SG39469, &UA2708698},
	{SG39470, &UA3094336},
	{SG39472, &UA2749634},
	{SG39473, &UA2955280},
	{SG39475, &UA2753165},
	{SG39478, &UA2974915},
	{SG39484, &UA2973670},
	{SG39486, &UA2975717},
	{SG39487, &UA2975726},
	{SG39489, &UA2728291},
	{SG39491, &UA2801275},
	{SG39493, &UA2801278},
	{SG39494, &UA2606408},
	{SG39495, &UA3065605},
	{SG39496, &UA2712083},
	{SG39497, &UA2956209},
	{SG39507, &UA2753450},
	{SG39508, &UA2892579},
	{SG39515, &UA2853925},
	{SG39516, &UA3058186},
	{SG39519, &UA2354623},
	{SG39521, &UA2527556},
	{SG39522, &UA2833770},
	{SG39573, &UA3064646},
	{SG39580, &UA3068065},
	{SG39583, &UA2974117},
	{SG39589, &UA2237715},
	{SG39591, &UA2626464},
	{SG39592, &UA2627442},
	{SG39615, &UA2770770},
	{SG39616, &UA2771507},
	{SG39617, &UA2782336},
	{SG39618, &UA2868314},
	{SG39619, &UA2937950},
	{SG39620, &UA2957403},
	{SG39621, &UA2975021},
	{SG39623, &UA3132857},
	{SG39628, &UA2580663},
	{SG39631, &UA2868445},
	{SG39638, &UA3026648},
	{SG39640, &UA3059631},
	{SG39641, &UA2234197},
	{SG39648, &UA3065986},
	{SG39654, &UA2238294},
	{SG39655, &UA2877602},
	{SG39661, &UA2973461},
	{SG39664, &UA2973276},
	{SG39665, &UA2947255},
	{SG39666, &UA2971383},
	{SG39667, &UA3065243},
	{SG39822, &UA2362866},
	{SG39828, &UA2582683},
	{SG39845, &UA2974791},
	{SG39846, &UA2797695},
	{SG39847, &UA2798449},
	{SG39848, &UA2799390},
	{SG39849, &UA2867281},
	{SG39851, &UA2877880},
	{SG39852, &UA2879202},
	{SG39853, &UA2892770},
	{SG39854, &UA2893353},
	{SG39859, &UA2937822},
	{SG39860, &UA2937895},
	{SG39861, &UA2960182},
	{SG39887, &UA3019947},
	{SG39889, &UA3093458},
	{SG39891, &UA3103391},
	{SG39894, &UA2567300},
	{SG39895, &UA2890330},
	{SG39897, &UA2559222},
	{SG39898, &UA2582763},
	{SG39899, &UA2671472},
	{SG39900, &UA2754152},
	{SG39902, &UA2792958},
	{SG39903, &UA2973723},
	{SG39904, &UA2597397},
	{SG39905, &UA2258045},
	{SG39924, &UA2709404},
	{SG39926, &UA2778997},
	{SG39928, &UA2867191},
	{SG39930, &UA2975015},
	{SG39933, &UA2975811},
	{SG39944, &UA2753687},
	{SG39956, &UA2898727},
	{SG39959, &UA2943898},
	{SG39960, &UA2963319},
	{SG39961, &UA2964598},
	{SG39962, &UA2566811},
	{SG39963, &UA2567002},
	{SG39964, &UA2567400},
	{SG39965, &UA2859567},
	{SG39971, &UA2673774},
	{SG39972, &UA2676257},
	{SG39973, &UA3047486},
	{SG39974, &UA3023869},
	{SG39980, &UA3066215},
	{SG39982, &UA2581564},
	{SG39983, &UA2765766},
	{SG39986, &UA2672089},
	{SG39987, &UA2677176},
	{SG39992, &UA2708745},
	{SG39993, &UA2750137},
	{SG39995, &UA2964929},
	{SG39997, &UA3007885},
	{SG39999, &UA3092481},
	{SG40044, &UA3065614},
	{SG40062, &UA2556460},
	{SG40063, &UA2952691},
	{SG40068, &UA2892116},
	{SG40070, &UA2998971},
	{SG40071, &UA2892152},
	{SG40072, &UA2945362},
	{SG40078, &UA2732680},
	{SG40081, &UA2752240},
	{SG40082, &UA2879750},
	{SG40083, &UA2991509},
	{SG40100, &UA2586652},
	{SG40101, &UA3013248},
	{SG40102, &UA3065255},
	{SG40114, &UA2963318},
	{SG40115, &UA3021736},
	{SG40130, &UA2662701},
	{SG40132, &UA2714214},
	{SG40133, &UA2935206},
	{SG40134, &UA2936570},
	{SG40135, &UA2993317},
	{SG40146, &UA2586657},
	{SG40147, &UA2586659},
	{SG40162, &UA2362249},
	{SG40163, &UA2384738},
	{SG40165, &UA2557474},
	{SG40167, &UA2567219},
	{SG40169, &UA2668786},
	{SG40170, &UA3027730},
	{SG40171, &UA3027733},
	{SG40172, &UA3028228},
	{SG40173, &UA3028271},
	{SG40174, &UA3029028},
	{SG40175, &UA3029632},
	{SG40180, &UA2672897},
	{SG40182, &UA2676825},
	{SG40185, &UA2974817},
	{SG40189, &UA2752680},
	{SG40194, &UA2555861},
	{SG40195, &UA2778551},
	{SG40196, &UA3131466},
	{SG40197, &UA3132955},
	{SG40291, &UA2560572},
	{SG40292, &UA2709225},
	{SG40293, &UA2752368},
	{SG40294, &UA2960186},
	{SG40296, &UA3059411},
	{SG40298, &UA3107096},
	{SG40301, &UA3111705},
	{SG40303, &UA2673554},
	{SG40304, &UA2676336},
	{SG40307, &UA2708404},
	{SG40309, &UA2752160},
	{SG40310, &UA2707405},
	{SG40312, &UA2866521},
	{SG40316, &UA2540566},
	{SG40317, &UA2540581},
	{SG40318, &UA2753961},
	{SG40352, &UA2674961},
	{SG40353, &UA2714088},
	{SG40363, &UA2974032},
	{SG40365, &UA2355478},
	{SG40378, &UA2799175},
	{SG40388, &UA2974452},
	{SG40405, &UA2730437},
	{SG40407, &UA2773594},
	{SG40409, &UA2796239},
	{SG40410, &UA2385276},
	{SG40411, &UA2768900},
	{SG40412, &UA2958595},
	{SG40415, &UA2666004},
	{SG40417, &UA3067060},
	{SG40421, &UA2658692},
	{SG40457, &UA2676765},
	{SG40461, &UA2430152},
	{SG40463, &UA2580630},
	{SG40467, &UA3074101},
	{SG40470, &UA2864599},
	{SG40471, &UA3065584},
	{SG40478, &UA2233256},
	{SG40479, &UA2360504},
	{SG40480, &UA2455521},
	{SG40493, &UA2974729},
	{SG40494, &UA3068399},
	{SG40495, &UA3083664},
	{SG40497, &UA3110904},
	{SG40501, &UA2676506},
	{SG40503, &UA2860575},
	{SG40507, &UA3065321},
	{SG40509, &UA2975946},
	{SG40513, &UA3064873},
	{SG40514, &UA3066519},
	{SG40535, &UA2973674},
	{SG40536, &UA2973912},
	{SG40537, &UA2974930},
	{SG40538, &UA2975805},
	{SG40539, &UA2361484},
	{SG40540, &UA2455836},
	{SG40541, &UA2579371},
	{SG40542, &UA2618353},
	{SG40544, &UA2619209},
	{SG40546, &UA2619465},
	{SG40547, &UA2621653},
	{SG40548, &UA2622103},
	{SG40549, &UA2627789},
	{SG40550, &UA2637487},
	{SG40551, &UA2675895},
	{SG40552, &UA2795594},
	{SG40554, &UA2679284},
	{SG40556, &UA2712121},
	{SG40558, &UA2862387},
	{SG40560, &UA2753556},
	{SG40565, &UA2677154},
	{SG40567, &UA2714384},
	{SG40568, &UA3065253},
	{SG40569, &UA3094443},
	{SG40579, &UA2784815},
	{SG40587, &UA2975945},
	{SG40588, &UA3020717},
	{SG40591, &UA3081034},
	{SG40595, &UA2706353},
	{SG40596, &UA2750629},
	{SG40597, &UA2869453},
	{SG40600, &UA2655700},
	{SG40601, &UA2779855},
	{SG40605, &UA2974284},
	{SG40606, &UA3021237},
	{SG40607, &UA3034636},
	{SG40610, &UA2754258},
	{SG40611, &UA2897962},
	{SG40619, &UA2753199},
	{SG40622, &UA2754249},
	{SG40623, &UA3067587},
	{SG40636, &UA2356632},
	{SG40638, &UA2558222},
	{SG40641, &UA3068252},
	{SG40643, &UA3094932},
	{SG40644, &UA2567215},
	{SG40647, &UA2583781},
	{SG40648, &UA2600975},
	{SG40650, &UA2707678},
	{SG40653, &UA2738316},
	{SG40657, &UA3068218},
	{SG40659, &UA3081186},
	{SG40661, &UA3081687},
	{SG40663, &UA3083297},
	{SG40666, &UA2572106},
	{SG40669, &UA2606185},
	{SG40671, &UA2672157},
	{SG40673, &UA2970155},
	{SG40674, &UA3128716},
	{SG40679, &UA2940150},
	{SG40680, &UA2963421},
	{SG40681, &UA3056272},
	{SG40682, &UA2699804},
	{SG40683, &UA2703074},
	{SG40684, &UA2780021},
	{SG40686, &UA2936824},
	{SG40687, &UA2957017},
	{SG40689, &UA3102251},
	{SG40691, &UA3120530},
	{SG40705, &UA2756701},
	{SG40707, &UA2877927},
	{SG40709, &UA2892379},
	{SG40711, &UA2892892},
	{SG40713, &UA2943380},
	{SG40714, &UA3000344},
	{SG40716, &UA2974357},
	{SG40717, &UA2975972},
	{SG40718, &UA2715180},
	{SG40719, &UA2738191},
	{SG40720, &UA2975547},
	{SG40740, &UA2753976},
	{SG40745, &UA3096278},
	{SG40746, &UA3096401},
	{SG40747, &UA3110451},
	{SG40748, &UA3114507},
	{SG40749, &UA3115339},
	{SG40750, &UA3115428},
	{SG40759, &UA2753546},
	{SG40761, &UA2754477},
	{SG40768, &UA2894626},
	{SG40770, &UA2962568},
	{SG40772, &UA2966348},
	{SG40774, &UA2972111},
	{SG40775, &UA3005045},
	{SG40778, &UA2355162},
	{SG40779, &UA2428524},
	{SG40780, &UA2571475},
	{SG40781, &UA2607160},
	{SG40782, &UA3065343},
	{SG40791, &UA2973677},
	{SG40793, &UA3068046},
	{SG40797, &UA3065988},
	{SG40799, &UA3094233},
	{SG40813, &UA2257977},
	{SG40814, &UA2420590},
	{SG40815, &UA2457159},
	{SG40816, &UA2974274},
	{SG40817, &UA2976122},
	{SG40820, &UA3030599},
	{SG40821, &UA3031214},
	{SG40829, &UA2711054},
	{SG40831, &UA2718809},
	{SG40833, &UA2539859},
	{SG40834, &UA2656200},
	{SG40843, &UA2753276},
	{SG40844, &UA2753690},
	{SG40845, &UA2754938},
	{SG40846, &UA2754797},
	{SG40848, &UA2567118},
	{SG40850, &UA2584849},
	{SG40853, &UA2667165},
	{SG40857, &UA2796586},
	{SG40860, &UA3066881},
	{SG40864, &UA2975077},
	{SG40867, &UA2658234},
	{SG40872, &UA3021162},
	{SG40873, &UA3059577},
	{SG40876, &UA2676344},
	{SG40877, &UA2734763},
	{SG40887, &UA2740103},
	{SG40888, &UA2796412},
	{SG40891, &UA2232006},
	{SG40894, &UA2669589},
	{SG40895, &UA2713397},
	{SG40896, &UA2863504},
	{SG40897, &UA2975532},
	{SG40899, &UA2508326},
	{SG40900, &UA2566586},
	{SG40901, &UA2783874},
	{SG40902, &UA2897348},
	{SG40903, &UA3077430},
	{SG40904, &UA2540496},
	{SG40905, &UA2540504},
	{SG40906, &UA2795625},
	{SG40907, &UA2959788},
	{SG40908, &UA3020566},
	{SG40909, &UA3022054},
	{SG40912, &UA3064774},
	{SG40914, &UA3081105},
	{SG40917, &UA2557605},
	{SG40918, &UA2764198},
	{SG40919, &UA3102386},
	{SG40920, &UA2740665},
	{SG40921, &UA2752903},
	{SG40923, &UA2900959},
	{SG40924, &UA2944114},
	{SG40925, &UA2963499},
	{SG40927, &UA2466591},
	{SG40928, &UA2784164},
	{SG40946, &UA2878840},
	{SG40948, &UA2944508},
	{SG40950, &UA2989607},
	{SG40956, &UA2567203},
	{SG40957, &UA2606462},
	{SG40959, &UA2868528},
	{SG40961, &UA2876966},
	{SG40962, &UA2879580},
	{SG40982, &UA2754370},
	{SG40985, &UA2670938},
	{SG40987, &UA2707789},
	{SG40989, &UA2734573},
	{SG40998, &UA2893204},
	{SG40999, &UA2898413},
	{SG41000, &UA2936945},
	{SG41001, &UA2938161},
	{SG41002, &UA2943405},
	{SG41007, &UA2511749},
	{SG41011, &UA2603585},
	{SG41012, &UA2953335},
	{SG41013, &UA3028936},
	{SG41028, &UA2892849},
	{SG41030, &UA2944518},
	{SG41033, &UA2974716},
	{SG41035, &UA3022838},
	{SG41036, &UA2975959},
	{SG41037, &UA3022180},
	{SG41038, &UA2558097},
	{SG41050, &UA2258001},
	{SG41053, &UA2354213},
	{SG41054, &UA2557686},
	{SG41098, &UA2671112},
	{SG41101, &UA2676872},
	{SG41103, &UA3035591},
	{SG41114, &UA2734788},
	{SG41119, &UA2567126},
	{SG41120, &UA2585168},
	{SG41121, &UA2601375},
	{SG41122, &UA3132008},
	{SG41123, &UA2675320},
	{SG41124, &UA2714649},
	{SG41125, &UA2753266},
	{SG41126, &UA2782639},
	{SG41128, &UA2599214},
	{SG41129, &UA2713181},
	{SG41130, &UA2991258},
	{SG41131, &UA3065731},
	{SG41132, &UA3081864},
	{SG41144, &UA3068203},
	{SG41151, &UA2753543},
	{SG41154, &UA3067047},
	{SG41155, &UA2794871},
	{SG41156, &UA3049651},
	{SG41157, &UA3088556},
	{SG41158, &UA3066877},
	{SG41160, &UA3032673},
	{SG41181, &UA2975543},
	{SG41190, &UA2508107},
	{SG41192, &UA2508698},
	{SG41193, &UA2581735},
	{SG41195, &UA2893930},
	{SG41196, &UA3031037},
	{SG41198, &UA3065609},
	{SG41200, &UA3068051},
	{SG41202, &UA3067921},
	{SG41203, &UA3080660},
	{SG41205, &UA3080823},
	{SG41207, &UA3081779},
	{SG41209, &UA3083362},
	{SG41237, &UA3131443},
	{SG41239, &UA3132580},
	{SG41240, &UA3132938},
	{SG41242, &UA3067183},
	{SG41248, &UA2974998},
	{SG41258, &UA2736264},
	{SG41275, &UA2938259},
	{SG41278, &UA2748580},
	{SG41280, &UA2942035},
	{SG41281, &UA3049227},
	{SG41284, &UA3054944},
	{SG41295, &UA2567336},
	{SG41297, &UA2602707},
	{SG41303, &UA2670288},
	{SG41304, &UA2734254},
	{SG41305, &UA2877937},
	{SG41306, &UA2670849},
	{SG41307, &UA2855879},
	{SG41308, &UA3080409},
	{SG41309, &UA3081334},
	{SG41310, &UA3083150},
	{SG41311, &UA3083452},
	{SG41316, &UA2367267},
	{SG41318, &UA2596556},
	{SG41321, &UA2753858},
	{SG41328, &UA3061060},
	{SG41329, &UA3093719},
	{SG41330, &UA3067605},
	{SG41332, &UA2558821},
	{SG41336, &UA2753050},
	{SG41361, &UA2974113},
	{SG41362, &UA2937336},
	{SG41363, &UA3015265},
	{SG41364, &UA2862857},
	{SG41365, &UA3078514},
	{SG41367, &UA3067067},
	{SG41378, &UA2233618},
	{SG41379, &UA2670289},
	{SG41380, &UA2944177},
	{SG41381, &UA2954106},
	{SG41382, &UA3063789},
	{SG41385, &UA2663181},
	{SG41386, &UA3002734},
	{SG41387, &UA3017797},
	{SG41388, &UA3047063},
	{SG41410, &UA2784806},
	{SG41412, &UA2800707},
	{SG41414, &UA2801050},
	{SG41415, &UA2885446},
	{SG41417, &UA2897688},
	{SG41419, &UA2943782},
	{SG41421, &UA3112495},
	{SG41422, &UA3128021},
	{SG41425, &UA2604564},
	{SG41427, &UA2858953},
	{SG41429, &UA2855218},
	{SG41431, &UA3103414},
	{SG41440, &UA2753004},
	{SG41441, &UA2800476},
	{SG41568, &UA2975067},
	{SG41570, &UA2355097},
	{SG41571, &UA2790219},
	{SG41573, &UA3118256},
	{SG41579, &UA2974712},
	{SG41580, &UA2983750},
	{SG41582, &UA3004643},
	{SG41583, &UA3071741},
	{SG41598, &UA2783165},
	{SG41601, &UA2938062},
	{SG41603, &UA2944646},
	{SG41606, &UA2505696},
	{SG41607, &UA2748377},
	{SG41623, &UA2603800},
	{SG41626, &UA2604177},
	{SG41629, &UA2716917},
	{SG41631, &UA2755007},
	{SG41634, &UA2886139},
	{SG41636, &UA3002467},
	{SG41637, &UA3077641},
	{SG41639, &UA3095024},
	{SG41642, &UA2676989},
	{SG41644, &UA2708109},
	{SG41646, &UA2799155},
	{SG41653, &UA2387399},
	{SG41654, &UA2565378},
	{SG41656, &UA2713017},
	{SG41657, &UA2997090},
	{SG41658, &UA2975528},
	{SG41659, &UA2976023},
	{SG41661, &UA3021507},
	{SG41662, &UA3023133},
	{SG41663, &UA2671442},
	{SG41664, &UA2672861},
	{SG41665, &UA2705495},
	{SG41666, &UA2708680},
	{SG41667, &UA2714147},
	{SG41670, &UA3073448},
	{SG41683, &UA2507078},
	{SG41689, &UA2934752},
	{SG41691, &UA3065251},
	{SG41692, &UA3066534},
	{SG41697, &UA2890972},
	{SG41699, &UA2668211},
	{SG41700, &UA2954597},
	{SG41703, &UA3067069},
	{SG41705, &UA3082806},
	{SG41706, &UA3083193},
	{SG41707, &UA3083448},
	{SG41712, &UA2238225},
	{SG41713, &UA2670619},
	{SG41715, &UA2713669},
	{SG41716, &UA3068044},
	{SG41720, &UA2867138},
	{SG41721, &UA3009609},
	{SG41723, &UA3080910},
	{SG41724, &UA3083081},
	{SG41727, &UA3083247},
	{SG41731, &UA3094614},
	{SG41744, &UA2755416},
	{SG41793, &UA2735604},
	{SG41794, &UA2999717},
	{SG41795, &UA2737858},
	{SG41796, &UA2773969},
	{SG41797, &UA3067609},
	{SG41800, &UA2772903},
	{SG41806, &UA2236727},
	{SG41807, &UA2627149},
	{SG41811, &UA2362228},
	{SG41812, &UA2632920},
	{SG41815, &UA2596490},
	{SG41817, &UA3086214},
	{SG41818, &UA2769912},
	{SG41834, &UA2772768},
	{SG41835, &UA2799524},
	{SG41837, &UA2753183},
	{SG41839, &UA2957784},
	{SG41841, &UA2999520},
	{SG41844, &UA3096273},
	{SG41845, &UA3096281},
	{SG41846, &UA3110350},
	{SG41848, &UA3111438},
	{SG41850, &UA3115505},
	{SG41851, &UA3115507},
	{SG41860, &UA2755640},
	{SG41862, &UA2769709},
	{SG41863, &UA2769974},
	{SG41864, &UA2783931},
	{SG41870, &UA2231859},
	{SG41871, &UA2553518},
	{SG41872, &UA2566933},
	{SG41873, &UA2753612},
	{SG41877, &UA2258197},
	{SG41878, &UA2422478},
	{SG41880, &UA2770679},
	{SG41881, &UA2899419},
	{SG41886, &UA3031326},
	{SG41888, &UA2237979},
	{SG41889, &UA2426986},
	{SG41892, &UA2659206},
	{SG41893, &UA2706662},
	{SG41894, &UA2795492},
	{SG41895, &UA2868984},
	{SG41897, &UA2958589},
	{SG41898, &UA3003496},
	{SG41899, &UA3058587},
	{SG41909, &UA3068407},
	{SG41911, &UA2676443},
	{SG41912, &UA2708419},
	{SG41913, &UA2711141},
	{SG41916, &UA2671189},
	{SG41917, &UA2948547},
	{SG41924, &UA3065231},
	{SG41937, &UA2754480},
	{SG41939, &UA3027868},
	{SG41940, &UA3027967},
	{SG41942, &UA3107930},
	{SG41952, &UA2994733},
	{SG41954, &UA3030753},
	{SG41963, &UA2974110},
	{SG41964, &UA2774290},
	{SG41979, &UA2893239},
	{SG41982, &UA3021397},
	{SG41983, &UA3025435},
	{SG41992, &UA2772776},
	{SG42003, &UA2516343},
	{SG42007, &UA3066871},
	{SG42012, &UA2865908},
	{SG42014, &UA2891662},
	{SG42016, &UA2936796},
	{SG42017, &UA2956271},
	{SG42019, &UA2957603},
	{SG42020, &UA2998841},
	{SG42022, &UA3057867},
	{SG42023, &UA3072682},
	{SG42027, &UA2566804},
	{SG42033, &UA2384491},
	{SG42034, &UA2756015},
	{SG42035, &UA2944279},
	{SG42045, &UA2973906},
	{SG42046, &UA3023519},
	{SG42047, &UA2558742},
	{SG42052, &UA2975525},
	{SG42057, &UA2666807},
	{SG42068, &UA2737957},
	{SG42070, &UA2974163},
	{SG42072, &UA3065604},
	{SG42081, &UA2673221},
	{SG42082, &UA2676753},
	{SG42083, &UA2713405},
	{SG42085, &UA2734923},
	{SG42090, &UA2719733},
	{SG42092, &UA2719735},
	{SG42093, &UA2769428},
	{SG42094, &UA2868521},
	{SG42095, &UA2879440},
	{SG42096, &UA3127373},
	{SG42098, &UA2234204},
	{SG42099, &UA2974278},
	{SG42106, &UA2772898},
	{SG42107, &UA2739592},
	{SG42108, &UA2957152},
	{SG42110, &UA2974440},
	{SG42112, &UA3083206},
	{SG42113, &UA3083379},
	{SG42116, &UA2754349},
	{SG42118, &UA2974868},
	{SG42120, &UA2975027},
	{SG42122, &UA3093548},
	{SG42124, &UA3109251},
	{SG42125, &UA3111521},
	{SG42126, &UA3066891},
	{SG42129, &UA3094458},
	{SG42130, &UA3112966},
	{SG42131, &UA3114912},
	{SG42132, &UA3126611},
	{SG42136, &UA2567210},
	{SG42137, &UA2668945},
	{SG42138, &UA2784774},
	{SG42139, &UA2958007},
	{SG42140, &UA2973651},
	{SG42142, &UA2754478},
	{SG42145, &UA2945880},
	{SG42148, &UA2562526},
	{SG42151, &UA2999771},
	{SG42153, &UA3109325},
	{SG42161, &UA2681029},
	{SG42162, &UA3077646},
	{SG42163, &UA3079721},
	{SG42165, &UA3107051},
	{SG42167, &UA3067295},
	{SG42168, &UA2975747},
	{SG42176, &UA2946984},
	{SG42177, &UA2965603},
	{SG42178, &UA2969642},
	{SG42179, &UA3083152},
	{SG42219, &UA2676598},
	{SG42221, &UA2709067},
	{SG42223, &UA2709070},
	{SG42224, &UA2711694},
	{SG42225, &UA2860572},
	{SG42226, &UA2738734},
	{SG42227, &UA3074115},
	{SG42228, &UA3039198},
	{SG42229, &UA3068206},
	{SG42232, &UA2233603},
	{SG42234, &UA2234191},
	{SG42236, &UA2403929},
	{SG42253, &UA2658196},
	{SG42254, &UA2704484},
	{SG42255, &UA2878919},
	{SG42257, &UA2753891},
	{SG42271, &UA3067375},
	{SG42272, &UA3081626},
	{SG42273, &UA3083082},
	{SG42275, &UA3083257},
	{SG42276, &UA3110341},
	{SG42278, &UA3066223},
	{SG42293, &UA2709496},
	{SG42294, &UA2756003},
	{SG42297, &UA2738754},
	{SG42299, &UA3049544},
	{SG42309, &UA3096284},
	{SG42310, &UA3115508},
	{SG42311, &UA3130695},
	{SG42343, &UA2772759},
	{SG42344, &UA2863830},
	{SG42349, &UA2404297},
	{SG42358, &UA3068335},
	{SG42365, &UA2261668},
	{SG42367, &UA2507781},
	{SG42391, &UA2974366},
	{SG42393, &UA3021386},
	{SG42395, &UA3065151},
	{SG42396, &UA3094532},
	{SG42397, &UA3094799},
	{SG42398, &UA3112879},
	{SG42407, &UA2605434},
	{SG42409, &UA2991322},
	{SG42412, &UA2740102},
	{SG42414, &UA2755000},
	{SG42416, &UA2772412},
	{SG42422, &UA2529616},
	{SG42423, &UA2581827},
	{SG42424, &UA2582829},
	{SG42425, &UA3000307},
	{SG42427, &UA2566440},
	{SG42429, &UA2780662},
	{SG42431, &UA2999448},
	{SG42467, &UA2366625},
	{SG42472, &UA2670268},
	{SG42474, &UA2714472},
	{SG42476, &UA2781119},
	{SG42478, &UA2956315},
	{SG42491, &UA2482366},
	{SG42492, &UA2595997},
	{SG42493, &UA3043751},
	{SG42494, &UA2679350},
	{SG42495, &UA2936764},
	{SG42496, &UA2753766},
	{SG42501, &UA2876121},
	{SG42502, &UA2900834},
	{SG42506, &UA2773113},
	{SG42507, &UA2731079},
	{SG42508, &UA2996333},
	{SG42509, &UA3081078},
	{SG42517, &UA2765953},
	{SG42518, &UA2958532},
	{SG42525, &UA2753000},
	{SG42527, &UA2973896},
	{SG42529, &UA2975950},
	{SG42531, &UA2975954},
	{SG42532, &UA3020852},
	{SG42533, &UA2974026},
	{SG42534, &UA2675024},
	{SG42535, &UA3131341},
	{SG42548, &UA2973649},
	{SG42549, &UA2974891},
	{SG42551, &UA3020520},
	{SG42553, &UA3021670},
	{SG42554, &UA3023039},
	{SG42555, &UA3107857},
	{SG42560, &UA2485044},
	{SG42563, &UA2485937},
	{SG42564, &UA2489105},
	{SG42566, &UA2670314},
	{SG42568, &UA2966977},
	{SG42571, &UA2366958},
	{SG42573, &UA2423099},
	{SG42576, &UA2895334},
	{SG42577, &UA2960971},
	{SG42581, &UA2558881},
	{SG42598, &UA2774063},
	{SG42599, &UA2784082},
	{SG42600, &UA2869559},
	{SG42601, &UA2973462},
	{SG42603, &UA3020846},
	{SG42604, &UA3033832},
	{SG42605, &UA3092380},
	{SG42613, &UA2563113},
	{SG42622, &UA2752775},
	{SG42623, &UA2797142},
	{SG42624, &UA2938048},
	{SG42625, &UA2966146},
	{SG42669, &UA2753203},
	{SG42671, &UA2771049},
	{SG42672, &UA2879065},
	{SG42673, &UA2865823},
	{SG42675, &UA2964350},
	{SG42677, &UA3015119},
	{SG42678, &UA3019809},
	{SG42681, &UA3108572},
	{SG42683, &UA3124520},
	{SG42687, &UA2585261},
	{SG42688, &UA2668980},
	{SG42689, &UA2990623},
	{SG42690, &UA2676188},
	{SG42697, &UA2781347},
	{SG42699, &UA3056826},
	{SG42702, &UA2975523},
	{SG42704, &UA2238603},
	{SG42705, &UA2716347},
	{SG42718, &UA2738367},
	{SG42730, &UA3080589},
	{SG42731, &UA3081029},
	{SG42733, &UA3081177},
	{SG42735, &UA2536723},
	{SG42736, &UA3022846},
	{SG42738, &UA2634256},
	{SG42739, &UA2773090},
	{SG42744, &UA2701586},
	{SG42745, &UA2752961},
	{SG42748, &UA2385108},
	{SG42754, &UA2506354},
	{SG42757, &UA2678411},
	{SG42759, &UA2712646},
	{SG42760, &UA2863566},
	{SG42762, &UA2974295},
	{SG42764, &UA2975028},
	{SG42765, &UA2973790},
	{SG42787, &UA2876112},
	{SG42789, &UA2555502},
	{SG42790, &UA2765046},
	{SG42793, &UA2893908},
	{SG42794, &UA3020881},
	{SG42801, &UA2975020},
	{SG42803, &UA2729897},
	{SG42804, &UA2754251},
	{SG42807, &UA2530097},
	{SG42810, &UA2852639},
	{SG42812, &UA3049041},
	{SG42822, &UA2677286},
	{SG42823, &UA2863508},
	{SG42830, &UA3068278},
	{SG42854, &UA2712951},
	{SG42856, &UA2737686},
	{SG42857, &UA3102818},
	{SG42862, &UA3034113},
	{SG42868, &UA2422728},
	{SG42871, &UA2502729},
	{SG42872, &UA2528777},
	{SG42873, &UA2554964},
	{SG42874, &UA2555850},
	{SG42877, &UA2982888},
	{SG42879, &UA2987801},
	{SG42880, &UA2989758},
	{SG42882, &UA2767142},
	{SG42883, &UA3088468},
	{SG42889, &UA2633368},
	{SG42890, &UA2773254},
	{SG42898, &UA2540590},
	{SG42900, &UA2868452},
	{SG42901, &UA2892721},
	{SG42902, &UA2997419},
	{SG42903, &UA3057571},
	{SG42904, &UA3059264},
	{SG42906, &UA2973646},
	{SG42910, &UA2795661},
	{SG42912, &UA3033156},
	{SG42914, &UA2355238},
	{SG42915, &UA3031486},
	{SG42916, &UA3108495},
	{SG42929, &UA3130945},
	{SG42931, &UA3132770},
	{SG42933, &UA2671393},
	{SG42934, &UA2707567},
	{SG42935, &UA2870782},
	{SG42936, &UA3073785},
	{SG42938, &UA3074141},
	{SG42940, &UA2970462},
	{SG42957, &UA3064998},
	{SG42964, &UA2898293},
	{SG42966, &UA2899273},
	{SG42968, &UA3016633},
	{SG42969, &UA3096285},
	{SG42971, &UA3111492},
	{SG43002, &UA2729228},
	{SG43003, &UA2954328},
	{SG43005, &UA3067306},
	{SG43006, &UA2422166},
	{SG43008, &UA2666335},
	{SG43009, &UA2751449},
	{SG43012, &UA2793330},
	{SG43014, &UA2858413},
	{SG43016, &UA2708690},
	{SG43018, &UA2865630},
	{SG43041, &UA2235152},
	{SG43043, &UA2363430},
	{SG43059, &UA2385298},
	{SG43060, &UA2635676},
	{SG43062, &UA3068048},
	{SG43070, &UA3068200},
	{SG43112, &UA2232798},
	{SG43115, &UA2615803},
	{SG43118, &UA2616975},
	{SG43119, &UA2618028},
	{SG43121, &UA2629942},
	{SG43125, &UA3131005},
	{SG43135, &UA3081612},
	{SG43143, &UA2670611},
	{SG43145, &UA2706792},
	{SG43147, &UA2734753},
	{SG43149, &UA2670617},
	{SG43150, &UA2736508},
	{SG43154, &UA2428354},
	{SG43165, &UA2510508},
	{SG43166, &UA3022880},
	{SG43167, &UA3046482},
	{SG43168, &UA3067904},
	{SG43172, &UA3068196},
	{SG43177, &UA2739604},
	{SG43178, &UA2986537},
	{SG43181, &UA2947376},
	{SG43208, &UA2233811},
	{SG43213, &UA2384941},
	{SG43216, &UA2531700},
	{SG43219, &UA2605512},
	{SG43221, &UA2797329},
	{SG43226, &UA2659640},
	{SG43232, &UA2676916},
	{SG43233, &UA2707758},
	{SG43234, &UA2708425},
	{SG43235, &UA2711140},
	{SG43236, &UA2736696},
	{SG43237, &UA2882673},
	{SG43238, &UA3050773},
	{SG43248, &UA2426892},
	{SG43252, &UA2676758},
	{SG43259, &UA2719041},
	{SG43261, &UA2738020},
	{SG43267, &UA2736582},
	{SG43268, &UA3000416},
	{SG43269, &UA3074553},
	{SG43272, &UA2755525},
	{SG43276, &UA3068045},
	{SG43277, &UA3094969},
	{SG43279, &UA3126283},
	{SG43281, &UA3066523},
	{SG43283, &UA3065337},
	{SG43289, &UA2436053},
	{SG43346, &UA2258128},
	{SG43348, &UA2727223},
	{SG43352, &UA2667589},
	{SG43353, &UA2953738},
	{SG43354, &UA2991637},
	{SG43360, &UA2754644},
	{SG43362, &UA2783616},
	{SG43374, &UA2657625},
	{SG43375, &UA3119396},
	{SG43381, &UA2567377},
	{SG43385, &UA2602332},
	{SG43389, &UA2795269},
	{SG43402, &UA2558293},
	{SG43408, &UA3130734},
	{SG43415, &UA2237767},
	{SG43434, &UA2584770},
	{SG43436, &UA2861199},
	{SG43440, &UA2753446},
	{SG43441, &UA2797325},
	{SG43442, &UA2670908},
	{SG43444, &UA2736635},
	{SG43445, &UA2698004},
	{SG43447, &UA2784887},
	{SG43448, &UA2873145},
	{SG43449, &UA3000424},
	{SG43450, &UA2975542},
	{SG43451, &UA3067588},
	{SG43452, &UA2705789},
	{SG43454, &UA2855405},
	{SG43456, &UA3029901},
	{SG43458, &UA3077269},
	{SG43460, &UA3121102},
	{SG43465, &UA3132862},
	{SG43475, &UA2974493},
	{SG43478, &UA2426339},
	{SG43479, &UA2428423},
	{SG43480, &UA2458127},
	{SG43481, &UA2485358},
	{SG43484, &UA2675250},
	{SG43485, &UA2677275},
	{SG43486, &UA2974302},
	{SG43490, &UA2672047},
	{SG43493, &UA2770079},
	{SG43495, &UA2864845},
	{SG43496, &UA2870118},
	{SG43497, &UA3102156},
	{SG43498, &UA2974663},
	{SG43500, &UA2675763},
	{SG43502, &UA2676097},
	{SG43504, &UA2956883},
	{SG43523, &UA2234662},
	{SG43529, &UA2261629},
	{SG43546, &UA2654028},
	{SG43547, &UA3120037},
	{SG43549, &UA2879742},
	{SG43551, &UA3018185},
	{SG43564, &UA2754354},
	{SG43566, &UA2901173},
	{SG43567, &UA2944933},
	{SG43568, &UA3080382},
	{SG43569, &UA3080700},
	{SG43571, &UA3082977},
	{SG43572, &UA3110571},
	{SG43577, &UA2769219},
	{SG43584, &UA2774330},
	{SG43586, &UA3028610},
	{SG43587, &UA3065979},
	{SG43603, &UA2880687},
	{SG43604, &UA2942357},
	{SG43606, &UA2968894},
	{SG43624, &UA2671072},
	{SG43626, &UA2952252},
	{SG43627, &UA3101386},
	{SG43629, &UA2769386},
	{SG43630, &UA2676976},
	{SG43632, &UA2709034},
	{SG43640, &UA2509245},
	{SG43647, &UA2734419},
	{SG43650, &UA2595542},
	{SG43651, &UA2680958},
	{SG43652, &UA2680961},
	{SG43653, &UA2680981},
	{SG43654, &UA2976000},
	{SG43663, &UA3080463},
	{SG43664, &UA3081047},
	{SG43666, &UA3094558},
	{SG43675, &UA2567941},
	{SG43676, &UA2605734},
	{SG43687, &UA2581193},
	{SG43688, &UA2708464},
	{SG43690, &UA2974916},
	{SG43699, &UA2878741},
	{SG43700, &UA2996208},
	{SG43701, &UA3121622},
	{SG43704, &UA2527963},
	{SG43706, &UA2727878},
	{SG43708, &UA2763422},
	{SG43710, &UA3014847},
	{SG43712, &UA2749090},
	{SG43715, &UA2753172},
	{SG43719, &UA2355082},
	{SG43720, &UA2384743},
	{SG43721, &UA2482622},
	{SG43724, &UA2974299},
	{SG43725, &UA3021272},
	{SG43726, &UA3021578},
	{SG43727, &UA3023754},
	{SG43729, &UA3025174},
	{SG43731, &UA2705609},
	{SG43732, &UA2861697},
	{SG43733, &UA2994584},
	{SG43734, &UA3101885},
	{SG43735, &UA2716825},
	{SG43736, &UA3050023},
	{SG43737, &UA2975740},
	{SG43739, &UA2583947},
	{SG43740, &UA2603685},
	{SG43741, &UA2584167},
	{SG43742, &UA2862736},
	{SG43744, &UA2999683},
	{SG43745, &UA2696597},
	{SG43747, &UA2698227},
	{SG43748, &UA2990742},
	{SG43749, &UA2992559},
	{SG43750, &UA3047728},
	{SG43752, &UA2676585},
	{SG43753, &UA2862286},
	{SG43754, &UA2879801},
	{SG43755, &UA3067304},
	{SG43757, &UA2959782},
	{SG43758, &UA3052007},
	{SG43759, &UA3055370},
	{SG43769, &UA2461078},
	{SG43776, &UA2261551},
	{SG43779, &UA2796023},
	{SG43784, &UA2676429},
	{SG43795, &UA3081348},
	{SG43796, &UA3081378},
	{SG43797, &UA2736733},
	{SG43798, &UA2970829},
	{SG43801, &UA2635778},
	{SG43807, &UA2530300},
	{SG43808, &UA2579681},
	{SG43809, &UA2763065},
	{SG43810, &UA2847985},
	{SG43813, &UA2566801},
	{SG43814, &UA2601075},
	{SG43815, &UA2667470},
	{SG43825, &UA2384962},
	{SG43826, &UA2974464},
	{SG43829, &UA3067194},
	{SG43832, &UA2358441},
	{SG43836, &UA2974469},
	{SG43837, &UA2975316},
	{SG43838, &UA3021355},
	{SG43839, &UA3024235},
	{SG43846, &UA2486988},
	{SG43852, &UA2998232},
	{SG43854, &UA2878937},
	{SG43855, &UA2899096},
	{SG43857, &UA2800666},
	{SG43858, &UA3043390},
	{SG43863, &UA2257976},
	{SG43867, &UA2672605},
	{SG43870, &UA3047855},
	{SG43877, &UA2753053},
	{SG43878, &UA2879664},
	{SG43880, &UA2753791},
	{SG43886, &UA2456460},
	{SG43887, &UA2773908},
	{SG43891, &UA3068209},
	{SG43893, &UA3065378},
	{SG43900, &UA2751983},
	{SG43901, &UA3015763},
	{SG43903, &UA2772926},
	{SG43905, &UA3064889},
	{SG43906, &UA3065730},
	{SG43915, &UA2556542},
	{SG43916, &UA2386700},
	{SG43917, &UA2481426},
	{SG43923, &UA3067195},
	{SG43924, &UA3094741},
	{SG43932, &UA3132024},
	{SG43934, &UA2799015},
	{SG43935, &UA2901595},
	{SG43936, &UA2961810},
	{SG43938, &UA3067661},
	{SG43940, &UA3093687},
	{SG43945, &UA2734175},
	{SG43946, &UA2734239},
	{SG43948, &UA2767315},
	{SG43949, &UA2990076},
	{SG43950, &UA3019688},
	{SG43951, &UA3049216},
	{SG43969, &UA2974883},
	{SG43970, &UA3021429},
	{SG43971, &UA3068063},
	{SG43985, &UA3031632},
	{SG43990, &UA2623754},
	{SG43991, &UA2628351},
	{SG43993, &UA2755009},
	{SG43996, &UA3080544},
	{SG44001, &UA3011241},
	{SG44016, &UA2567474},
	{SG44017, &UA2669468},
	{SG44019, &UA2404051},
	{SG44020, &UA2660967},
	{SG44021, &UA2765018},
	{SG44022, &UA2856704},
	{SG44024, &UA2753892},
	{SG44036, &UA2671195},
	{SG44082, &UA2504139},
	{SG44083, &UA2616916},
	{SG44085, &UA2618818},
	{SG44086, &UA2620069},
	{SG44088, &UA2622482},
	{SG44089, &UA2625397},
	{SG44090, &UA2627156},
	{SG44091, &UA2628429},
	{SG44092, &UA2261565},
	{SG44095, &UA2616966},
	{SG44096, &UA2739405},
	{SG44098, &UA3088415},
	{SG44110, &UA2631675},
	{SG44111, &UA3009493},
	{SG44113, &UA2937106},
	{SG44114, &UA2996305},
	{SG44120, &UA2869897},
	{SG44121, &UA3074174},
	{SG44125, &UA2363434},
	{SG44127, &UA2384546},
	{SG44131, &UA2455506},
	{SG44136, &UA2974595},
	{SG44139, &UA2703810},
	{SG44140, &UA2704294},
	{SG44142, &UA2989787},
	{SG44144, &UA3046647},
	{SG44146, &UA3032518},
	{SG44147, &UA3130722},
	{SG44149, &UA3130944},
	{SG44150, &UA3131302},
	{SG44151, &UA3131323},
	{SG44162, &UA2974239},
	{SG44164, &UA2673366},
	{SG44165, &UA2714156},
	{SG44175, &UA2669514},
	{SG44176, &UA2753786},
	{SG44178, &UA2937705},
	{SG44189, &UA2676727},
	{SG44190, &UA2714424},
	{SG44191, &UA2754224},
	{SG44192, &UA2880227},
	{SG44193, &UA2942088},
	{SG44194, &UA3057095},
	{SG44196, &UA3058748},
	{SG44197, &UA3077711},
	{SG44207, &UA2385302},
	{SG44209, &UA2675695},
	{SG44211, &UA2708348},
	{SG44212, &UA2879327},
	{SG44213, &UA2969162},
	{SG44215, &UA3026361},
	{SG44217, &UA3056431},
	{SG44218, &UA3058267},
	{SG44225, &UA2974814},
	{SG44227, &UA2362361},
	{SG44241, &UA2749186},
	{SG44242, &UA2965516},
	{SG44245, &UA2956396},
	{SG44246, &UA3109250},
	{SG44249, &UA2457375},
	{SG44250, &UA2466270},
	{SG44251, &UA2616731},
	{SG44253, &UA2618722},
	{SG44254, &UA2619531},
	{SG44255, &UA2620934},
	{SG44256, &UA2621513},
	{SG44257, &UA2623141},
	{SG44258, &UA2630827},
	{SG44259, &UA2633682},
	{SG44261, &UA2635294},
	{SG44263, &UA2667309},
	{SG44264, &UA2794919},
	{SG44267, &UA2710912},
	{SG44268, &UA2768396},
	{SG44269, &UA3030657},
	{SG44271, &UA2772526},
	{SG44272, &UA2753127},
	{SG44276, &UA3030372},
	{SG44281, &UA2753885},
	{SG44284, &UA2715161},
	{SG44285, &UA2715757},
	{SG44286, &UA3080013},
	{SG44287, &UA3080211},
	{SG44288, &UA3082815},
	{SG44290, &UA3109122},
	{SG44296, &UA2974719},
	{SG44297, &UA3021967},
	{SG44298, &UA3022610},
	{SG44299, &UA3023916},
	{SG44302, &UA2422203},
	{SG44303, &UA2675515},
	{SG44313, &UA2580811},
	{SG44316, &UA2670689},
	{SG44317, &UA2734078},
	{SG44318, &UA3029382},
	{SG44327, &UA2753284},
	{SG44329, &UA2764113},
	{SG44337, &UA2354263},
	{SG44338, &UA2555832},
	{SG44339, &UA2594641},
	{SG44340, &UA2882667},
	{SG44341, &UA3001344},
	{SG44342, &UA3001508},
	{SG44343, &UA3011853},
	{SG44344, &UA2974592},
	{SG44345, &UA3021755},
	{SG44347, &UA3077801},
	{SG44348, &UA3108294},
	{SG44351, &UA2756014},
	{SG44356, &UA2895902},
	{SG44358, &UA2938175},
	{SG44360, &UA2999256},
	{SG44362, &UA3131314},
	{SG44365, &UA2756535},
	{SG44370, &UA2738717},
	{SG44371, &UA3102676},
	{SG44372, &UA2752937},
	{SG44373, &UA2774122},
	{SG44375, &UA2797412},
	{SG44377, &UA2938069},
	{SG44383, &UA2707470},
	{SG44397, &UA2423092},
	{SG44398, &UA2654728},
	{SG44399, &UA2794916},
	{SG44400, &UA2952944},
	{SG44401, &UA2986700},
	{SG44418, &UA2753857},
	{SG44420, &UA2772791},
	{SG44421, &UA2782629},
	{SG44422, &UA2877358},
	{SG44423, &UA3088419},
	{SG44433, &UA2602078},
	{SG44434, &UA3015439},
	{SG44442, &UA3020934},
	{SG44444, &UA2897670},
	{SG44447, &UA2944930},
	{SG44449, &UA3056575},
	{SG44451, &UA3115119},
	{SG44452, &UA3131385},
	{SG44455, &UA3035655},
	{SG44456, &UA3059109},
	{SG44457, &UA3125286},
	{SG44458, &UA3082920},
	{SG44460, &UA2900429},
	{SG44465, &UA2354266},
	{SG44468, &UA2795937},
	{SG44474, &UA2678417},
	{SG44476, &UA2751566},
	{SG44479, &UA3031247},
	{SG44485, &UA2604838},
	{SG44488, &UA3067652},
	{SG44491, &UA2566944},
	{SG44493, &UA2864971},
	{SG44497, &UA3034008},
	{SG44499, &UA2567477},
	{SG44502, &UA3030826},
	{SG44517, &UA2566967},
	{SG44518, &UA2601197},
	{SG44520, &UA2752655},
	{SG44521, &UA2974304},
	{SG44522, &UA3021927},
	{SG44523, &UA3021996},
	{SG44524, &UA3081023},
	{SG44532, &UA2603405},
	{SG44533, &UA2769662},
	{SG44535, &UA3067911},
	{SG44536, &UA3039431},
	{SG44543, &UA3094575},
	{SG44545, &UA3109668},
	{SG44548, &UA2869478},
	{SG44552, &UA2366673},
	{SG44553, &UA2402720},
	{SG44555, &UA2624534},
	{SG44556, &UA2633487},
	{SG44557, &UA2675932},
	{SG44558, &UA2735400},
	{SG44559, &UA2869199},
	{SG44563, &UA2717659},
	{SG44570, &UA3081189},
	{SG44580, &UA2713471},
	{SG44582, &UA2943972},
	{SG44583, &UA2233247},
	{SG44584, &UA2404146},
	{SG44602, &UA2567585},
	{SG44604, &UA2938301},
	{SG44606, &UA2601881},
	{SG44607, &UA2975738},
	{SG44608, &UA3021668},
	{SG44609, &UA3023220},
	{SG44610, &UA3022938},
	{SG44611, &UA3023597},
	{SG44619, &UA2602486},
	{SG44623, &UA2708936},
	{SG44624, &UA3073920},
	{SG44627, &UA2974654},
	{SG44629, &UA2583086},
	{SG44643, &UA2355267},
	{SG44655, &UA2976100},
	{SG44656, &UA3021323},
	{SG44658, &UA3022378},
	{SG44676, &UA2355372},
	{SG44678, &UA2502605},
	{SG44680, &UA2594327},
	{SG44684, &UA3033073},
	{SG44687, &UA3060066},
	{SG44692, &UA3081232},
	{SG44696, &UA2601515},
	{SG44697, &UA3057618},
	{SG44699, &UA2713701},
	{SG44701, &UA2965534},
	{SG44726, &UA2594729},
	{SG44727, &UA2864767},
	{SG44730, &UA3022990},
	{SG44731, &UA2976017},
	{SG44732, &UA3021511},
	{SG44746, &UA2579057},
	{SG44750, &UA3068339},
	{SG44753, &UA3080936},
	{SG44756, &UA3065468},
	{SG44758, &UA2503890},
	{SG44760, &UA2555159},
	{SG44761, &UA2578746},
	{SG44762, &UA2778168},
	{SG44770, &UA2530501},
	{SG44773, &UA2662593},
	{SG44775, &UA3057884},
	{SG44786, &UA3033649},
	{SG44793, &UA2974657},
	{SG44795, &UA2974815},
	{SG44796, &UA3020300},
	{SG44798, &UA3131305},
	{SG44805, &UA2503335},
	{SG44807, &UA2485363},
	{SG44809, &UA2238298},
	{SG44814, &UA2676330},
	{SG44817, &UA3032168},
	{SG44818, &UA3093274},
	{SG44830, &UA2719737},
	{SG44832, &UA3006372},
	{SG44840, &UA2261679},
	{SG44849, &UA2566448},
	{SG44851, &UA2601772},
	{SG44855, &UA2420916},
	{SG44859, &UA3083198},
	{SG44865, &UA3030816},
	{SG44869, &UA3074027},
	{SG44874, &UA2234193},
	{SG44876, &UA2567386},
	{SG44879, &UA2675064},
	{SG44880, &UA2751335},
	{SG44886, &UA2510700},
	{SG44888, &UA2667170},
	{SG44890, &UA2991241},
	{SG44925, &UA2754412},
	{SG44931, &UA3031580},
	{SG44932, &UA3033321},
	{SG44934, &UA3059196},
	{SG44936, &UA3022148},
	{SG44953, &UA2754272},
	{SG44956, &UA2567341},
	{SG44957, &UA2583275},
	{SG44958, &UA2583565},
	{SG44961, &UA2666886},
	{SG44963, &UA2737390},
	{SG44981, &UA2706725},
	{SG44982, &UA2732046},
	{SG44983, &UA2891199},
	{SG44984, &UA2997537},
	{SG44985, &UA2751194},
	{SG44987, &UA2861659},
	{SG44994, &UA2768765},
	{SG45007, &UA2566771},
	{SG45008, &UA3072610},
	{SG45010, &UA2757299},
	{SG45011, &UA3081688},
	{SG45014, &UA2367286},
	{SG45015, &UA2402351},
	{SG45020, &UA2562207},
	{SG45029, &UA2738646},
	{SG45030, &UA2975012},
	{SG45031, &UA3020269},
	{SG45032, &UA3020923},
	{SG45043, &UA2718807},
	{SG45044, &UA2768440},
	{SG45051, &UA2480626},
	{SG45052, &UA2632464},
	{SG45056, &UA3034629},
	{SG45057, &UA3107888},
	{SG45059, &UA3108548},
	{SG45061, &UA3132946},
	{SG45069, &UA2783713},
	{SG45073, &UA3001603},
	{SG45101, &UA2659661},
	{SG45102, &UA3119422},
	{SG45111, &UA2769028},
	{SG45112, &UA2774120},
	{SG45120, &UA2782238},
	{SG45122, &UA2937405},
	{SG45124, &UA3081349},
	{SG45126, &UA3082008},
	{SG45127, &UA3083433},
	{SG45129, &UA2238416},
	{SG45134, &UA2756561},
	{SG45142, &UA2354708},
	{SG45144, &UA2796542},
	{SG45146, &UA2967143},
	{SG45147, &UA2967550},
	{SG45148, &UA2967881},
	{SG45160, &UA2670627},
	{SG45161, &UA2737279},
	{SG45163, &UA2676485},
	{SG45171, &UA2670755},
	{SG45173, &UA2673540},
	{SG45176, &UA2674207},
	{SG45178, &UA2735030},
	{SG45180, &UA2735210},
	{SG45194, &UA2734567},
	{SG45195, &UA2865704},
	{SG45197, &UA3072981},
	{SG45199, &UA2773900},
	{SG45200, &UA2901186},
	{SG45201, &UA3007890},
	{SG45206, &UA2675105},
	{SG45207, &UA2727635},
	{SG45208, &UA2800400},
	{SG45209, &UA3089408},
	{SG45216, &UA2428469},
	{SG45217, &UA2700461},
	{SG45218, &UA2983096},
	{SG45219, &UA2719710},
	{SG45220, &UA2719712},
	{SG45222, &UA2719715},
	{SG45225, &UA2719717},
	{SG45226, &UA2719725},
	{SG45228, &UA2719720},
	{SG45231, &UA2939169},
	{SG45232, &UA2999910},
	{SG45259, &UA3083597},
	{SG45260, &UA3083930},
	{SG45261, &UA3126237},
	{SG45263, &UA3130726},
	{SG45264, &UA3130728},
	{SG45265, &UA3132863},
	{SG45270, &UA2769379},
	{SG45271, &UA2974995},
	{SG45273, &UA3068396},
	{SG45281, &UA2773660},
	{SG45284, &UA2784102},
	{SG45292, &UA2772758},
	{SG45305, &UA2584101},
	{SG45306, &UA2601374},
	{SG45308, &UA2667284},
	{SG45309, &UA2857763},
	{SG45312, &UA2755066},
	{SG45313, &UA2975524},
	{SG45314, &UA2566682},
	{SG45316, &UA2792478},
	{SG45317, &UA2994696},
	{SG45330, &UA2528796},
	{SG45331, &UA2607498},
	{SG45333, &UA2657664},
	{SG45335, &UA2861129},
	{SG45336, &UA2772772},
	{SG45337, &UA2783083},
	{SG45340, &UA2975049},
	{SG45341, &UA2974458},
	{SG45346, &UA2384818},
	{SG45353, &UA2728844},
	{SG45355, &UA2791289},
	{SG45356, &UA2854081},
	{SG45357, &UA2985956},
	{SG45359, &UA2995912},
	{SG45360, &UA3071243},
	{SG45363, &UA2755270},
	{SG45367, &UA2800374},
	{SG45369, &UA3002441},
	{SG45372, &UA2974101},
	{SG45373, &UA3022957},
	{SG45374, &UA3081089},
	{SG45384, &UA2957510},
	{SG45399, &UA2708967},
	{SG45405, &UA2736336},
	{SG45408, &UA3073916},
	{SG45410, &UA2772761},
	{SG45412, &UA2892166},
	{SG45413, &UA2957660},
	{SG45415, &UA2957991},
	{SG45417, &UA2997213},
	{SG45418, &UA3090907},
	{SG45419, &UA2974365},
	{SG45421, &UA2709753},
	{SG45423, &UA3062631},
	{SG45424, &UA3115036},
	{SG45427, &UA3022601},
	{SG45428, &UA3030108},
	{SG45445, &UA2526465},
	{SG45447, &UA2753462},
	{SG45449, &UA2782812},
	{SG45456, &UA2261569},
	{SG45461, &UA2670606},
	{SG45462, &UA2955549},
	{SG45464, &UA3081689},
	{SG45466, &UA3094235},
	{SG45468, &UA3115060},
	{SG45474, &UA2774165},
	{SG45475, &UA3039410},
	{SG45480, &UA2677171},
	{SG45482, &UA2772677},
	{SG45485, &UA2258032},
	{SG45486, &UA2362532},
	{SG45487, &UA2482580},
	{SG45488, &UA2753782},
	{SG45496, &UA2703597},
	{SG45498, &UA2995727},
	{SG45517, &UA2713775},
	{SG45518, &UA2960834},
	{SG45526, &UA2566631},
	{SG45531, &UA2975552},
	{SG45535, &UA2565565},
	{SG45549, &UA2774151},
	{SG45556, &UA2566896},
	{SG45557, &UA2702622},
	{SG45558, &UA2676934},
	{SG45560, &UA2707462},
	{SG45561, &UA2713936},
	{SG45563, &UA2974369},
	{SG45564, &UA2974885},
	{SG45566, &UA3020946},
	{SG45567, &UA3021776},
	{SG45568, &UA3131713},
	{SG45572, &UA2261653},
	{SG45579, &UA2713954},
	{SG45581, &UA2737302},
	{SG45582, &UA2764274},
	{SG45583, &UA3083147},
	{SG45584, &UA3093615},
	{SG45585, &UA3114923},
	{SG45586, &UA2530017},
	{SG45587, &UA2503063},
	{SG45588, &UA2701945},
	{SG45589, &UA2865018},
	{SG45591, &UA2717883},
	{SG45592, &UA2869297},
	{SG45597, &UA2772928},
	{SG45605, &UA2666917},
	{SG45607, &UA2733356},
	{SG45621, &UA2799959},
	{SG45623, &UA3103230},
	{SG45624, &UA2895069},
	{SG45626, &UA3080126},
	{SG45636, &UA2455694},
	{SG45638, &UA2753778},
	{SG45639, &UA2877584},
	{SG45642, &UA2792759},
	{SG45645, &UA2974659},
	{SG45647, &UA3021235},
	{SG45649, &UA3096279},
	{SG45650, &UA3114748},
	{SG45663, &UA2895087},
	{SG45666, &UA2556455},
	{SG45670, &UA2973486},
	{SG45672, &UA3021740},
	{SG45673, &UA2974726},
	{SG45680, &UA2585256},
	{SG45681, &UA2860392},
	{SG45683, &UA2676938},
	{SG45689, &UA3020564},
	{SG45690, &UA3021661},
	{SG45692, &UA3022512},
	{SG45700, &UA2865633},
	{SG45701, &UA2866107},
	{SG45702, &UA2866228},
	{SG45705, &UA2956858},
	{SG45706, &UA3049420},
	{SG45707, &UA3091773},
	{SG45709, &UA3125365},
	{SG45720, &UA2354900},
	{SG45721, &UA2566353},
	{SG45724, &UA2738268},
	{SG45725, &UA2770049},
	{SG45727, &UA2798827},
	{SG45731, &UA3061493},
	{SG45743, &UA2704871},
	{SG45745, &UA3093713},
	{SG45749, &UA2354453},
	{SG45751, &UA2567521},
	{SG45752, &UA2583229},
	{SG45753, &UA2671262},
	{SG45754, &UA3029905},
	{SG45755, &UA3065391},
	{SG45756, &UA3079940},
	{SG45757, &UA3082738},
	{SG45758, &UA3094802},
	{SG45774, &UA2782974},
	{SG45789, &UA2998568},
	{SG45790, &UA2799321},
	{SG45791, &UA3074344},
	{SG45793, &UA3022058},
	{SG45808, &UA2770237},
	{SG45811, &UA3031260},
	{SG45813, &UA3080826},
	{SG45819, &UA2672078},
	{SG45820, &UA3107078},
	{SG45822, &UA2782435},
	{SG45853, &UA2567255},
	{SG45854, &UA2958466},
	{SG45855, &UA2749142},
	{SG45857, &UA2891810},
	{SG45858, &UA2895196},
	{SG45861, &UA2975704},
	{SG45883, &UA3080590},
	{SG45885, &UA3080650},
	{SG45886, &UA3094769},
	{SG45893, &UA2655759},
	{SG45895, &UA2951620},
	{SG45897, &UA2698997},
	{SG45898, &UA2989962},
	{SG45901, &UA2974467},
	{SG45910, &UA2976108},
	{SG45913, &UA3023392},
	{SG45923, &UA2424610},
	{SG45925, &UA2529696},
	{SG45928, &UA2773159},
	{SG45930, &UA2898734},
	{SG45934, &UA3032361},
	{SG45951, &UA2469424},
	{SG45958, &UA2737847},
	{SG45978, &UA2554494},
	{SG45980, &UA2850321},
	{SG45982, &UA2973489},
	{SG45985, &UA3083249},
	{SG45986, &UA3083372},
	{SG45988, &UA3096286},
	{SG46018, &UA2714736},
	{SG46024, &UA2355405},
	{SG46028, &UA2560135},
	{SG46031, &UA2858603},
	{SG46034, &UA3064899},
	{SG46041, &UA2676352},
	{SG46103, &UA2707868},
	{SG46104, &UA2796194},
	{SG46105, &UA2960198},
	{SG46107, &UA3057009},
	{SG46109, &UA3121699},
	{SG46111, &UA3125407},
	{SG46113, &UA3080124},
	{SG46114, &UA3081347},
	{SG46131, &UA2672152},
	{SG46136, &UA2774286},
	{SG46139, &UA2799255},
	{SG46140, &UA2894726},
	{SG46149, &UA2752980},
	{SG46150, &UA2756069},
	{SG46151, &UA2769432},
	{SG46152, &UA2784111},
	{SG46154, &UA3057675},
	{SG46157, &UA3132851},
	{SG46164, &UA2386238},
	{SG46171, &UA2711693},
	{SG46176, &UA2975713},
	{SG46177, &UA2975555},
	{SG46180, &UA3065483},
	{SG46193, &UA2466279},
	{SG46194, &UA2933920},
	{SG46195, &UA3032711},
	{SG46196, &UA2698877},
	{SG46217, &UA2694812},
	{SG46219, &UA2769655},
	{SG46221, &UA2793522},
	{SG46247, &UA2583964},
	{SG46248, &UA2859037},
	{SG46262, &UA2752349},
	{SG46264, &UA3101883},
	{SG46266, &UA2774294},
	{SG46319, &UA2258030},
	{SG46320, &UA2672425},
	{SG46321, &UA2994767},
	{SG46327, &UA2583081},
	{SG46345, &UA2456028},
	{SG46349, &UA2736374},
	{SG46352, &UA3020924},
	{SG46354, &UA3021326},
	{SG46355, &UA3022323},
	{SG46357, &UA3081388},
	{SG46367, &UA2430252},
	{SG46370, &UA2737232},
	{SG46371, &UA2740006},
	{SG46373, &UA2755413},
	{SG46374, &UA2899153},
	{SG46378, &UA2457534},
	{SG46380, &UA2601702},
	{SG46383, &UA2851872},
	{SG46386, &UA2754645},
	{SG46389, &UA2878647},
	{SG46391, &UA2998115},
	{SG46393, &UA3006956},
	{SG46394, &UA3074131},
	{SG46396, &UA3079019},
	{SG46399, &UA3088613},
	{SG46402, &UA3132021},
	{SG46410, &UA2966871},
	{SG46415, &UA2480885},
	{SG46416, &UA2702740},
	{SG46421, &UA2784097},
	{SG46424, &UA2869296},
	{SG46425, &UA2894037},
	{SG46427, &UA3020694},
	{SG46457, &UA2669265},
	{SG46461, &UA2670401},
	{SG46462, &UA2702527},
	{SG46465, &UA2737216},
	{SG46466, &UA3088802},
	{SG46468, &UA2780200},
	{SG46477, &UA2402583},
	{SG46480, &UA2624006},
	{SG46484, &UA2941352},
	{SG46488, &UA2755277},
	{SG46489, &UA2870018},
	{SG46490, &UA2934606},
	{SG46493, &UA3081337},
	{SG46497, &UA2556500},
	{SG46499, &UA2674755},
	{SG46501, &UA2676349},
	{SG46505, &UA2709297},
	{SG46507, &UA2974450},
	{SG46532, &UA2942639},
	{SG46536, &UA3081062},
	{SG46538, &UA3130584},
	{SG46559, &UA2754328},
	{SG46563, &UA3029629},
	{SG46572, &UA2427438},
	{SG46573, &UA2461992},
	{SG46584, &UA2700164},
	{SG46587, &UA2791357},
	{SG46590, &UA2850115},
	{SG46591, &UA2710645},
	{SG46594, &UA2753039},
	{SG46598, &UA2769149},
	{SG46599, &UA2772781},
	{SG46602, &UA2796538},
	{SG46604, &UA2958668},
	{SG46605, &UA3083370},
	{SG46628, &UA3020628},
	{SG46629, &UA3020870},
	{SG46637, &UA2555758},
	{SG46639, &UA2679664},
	{SG46640, &UA2711303},
	{SG46641, &UA2995525},
	{SG46642, &UA2892863},
	{SG46643, &UA3021135},
	{SG46653, &UA2899411},
	{SG46654, &UA2957905},
	{SG46656, &UA2958455},
	{SG46659, &UA3001946},
	{SG46661, &UA3088190},
	{SG46663, &UA2973650},
	{SG46664, &UA3026339},
	{SG46665, &UA3108656},
	{SG46666, &UA2766157},
	{SG46668, &UA2504580},
	{SG46673, &UA2794951},
	{SG46706, &UA2422801},
	{SG46707, &UA2469481},
	{SG46708, &UA2486819},
	{SG46710, &UA2505851},
	{SG46712, &UA2708669},
	{SG46713, &UA3102374},
	{SG46741, &UA2974322},
	{SG46742, &UA3081082},
	{SG46744, &UA3081613},
	{SG46746, &UA3126536},
	{SG46748, &UA3094337},
	{SG46753, &UA2401797},
	{SG46761, &UA2942548},
	{SG46763, &UA2976053},
	{SG46767, &UA3036653},
	{SG46781, &UA3062604},
	{SG46783, &UA3080901},
	{SG46785, &UA3083535},
	{SG46788, &UA2402193},
	{SG46793, &UA3121358},
	{SG46794, &UA2580956},
	{SG46796, &UA2852001},
	{SG46797, &UA2940893},
	{SG46798, &UA2666678},
	{SG46799, &UA2707285},
	{SG46801, &UA2863331},
	{SG46802, &UA2734675},
	{SG46804, &UA2735049},
	{SG46806, &UA2737539},
	{SG46808, &UA2769359},
	{SG46811, &UA2967671},
	{SG46812, &UA2974500},
	{SG46815, &UA3082968},
	{SG46817, &UA3081196},
	{SG46820, &UA3094559},
	{SG46835, &UA2540497},
	{SG46837, &UA2540503},
	{SG46857, &UA2537089},
	{SG46864, &UA2784754},
	{SG46878, &UA2421920},
	{SG46879, &UA2422885},
	{SG46880, &UA2503026},
	{SG46881, &UA2975935},
	{SG46882, &UA3021241},
	{SG46889, &UA2261547},
	{SG46890, &UA2769537},
	{SG46891, &UA2773571},
	{SG46892, &UA2893005},
	{SG46893, &UA2772623},
	{SG46894, &UA2877593},
	{SG46895, &UA2938007},
	{SG46896, &UA2898180},
	{SG46897, &UA3080828},
	{SG46901, &UA2656658},
	{SG46902, &UA2953422},
	{SG46905, &UA2530271},
	{SG46906, &UA2957791},
	{SG46987, &UA2583393},
	{SG46992, &UA2755493},
	{SG46993, &UA2769812},
	{SG46995, &UA3019668},
	{SG47001, &UA2944708},
	{SG47003, &UA2769504},
	{SG47004, &UA2782664},
	{SG47006, &UA3015918},
	{SG47007, &UA3078228},
	{SG47020, &UA2676418},
	{SG47027, &UA2480535},
	{SG47028, &UA2700273},
	{SG47029, &UA2766105},
	{SG47030, &UA2773954},
	{SG47031, &UA2791587},
	{SG47059, &UA3022564},
	{SG47062, &UA3029912},
	{SG47064, &UA3054711},
	{SG47071, &UA2384778},
	{SG47073, &UA2670045},
	{SG47075, &UA2455884},
	{SG47080, &UA2567390},
	{SG47084, &UA2784795},
	{SG47086, &UA2869250},
	{SG47090, &UA3021749},
	{SG47091, &UA3022050},
	{SG47093, &UA3022543},
	{SG47094, &UA2997748},
	{SG47095, &UA3058817},
	{SG47096, &UA3081692},
	{SG47127, &UA2596250},
	{SG47128, &UA2749995},
	{SG47140, &UA2583339},
	{SG47144, &UA2659314},
	{SG47147, &UA2748326},
	{SG47150, &UA2792321},
	{SG47154, &UA2673938},
	{SG47160, &UA3059926},
	{SG47236, &UA2899131},
	{SG47240, &UA2666622},
	{SG47242, &UA2732573},
	{SG47244, &UA2901231},
	{SG47246, &UA2944471},
	{SG47249, &UA3061352},
	{SG47258, &UA2754233},
	{SG47268, &UA2354536},
	{SG47272, &UA2662590},
	{SG47276, &UA2774164},
	{SG47284, &UA2505296},
	{SG47286, &UA2556886},
	{SG47287, &UA2986396},
	{SG47289, &UA2664027},
	{SG47290, &UA2799105},
	{SG47298, &UA2583198},
	{SG47299, &UA2600982},
	{SG47300, &UA2667181},
	{SG47301, &UA2773524},
	{SG47302, &UA2879887},
	{SG47311, &UA2997616},
	{SG47312, &UA2879690},
	{SG47313, &UA3008156},
	{SG47314, &UA2973905},
	{SG47315, &UA3018266},
	{SG47316, &UA3058054},
	{SG47326, &UA2504940},
	{SG47339, &UA2768776},
	{SG47341, &UA2773352},
	{SG47350, &UA2708220},
	{SG47354, &UA2424747},
	{SG47356, &UA2713168},
	{SG47359, &UA3020725},
	{SG47360, &UA3050594},
	{SG47362, &UA3080755},
	{SG47373, &UA2672125},
	{SG47374, &UA2677704},
	{SG47376, &UA2707578},
	{SG47379, &UA2995276},
	{SG47380, &UA3039414},
	{SG47389, &UA2752049},
	{SG47391, &UA2769574},
	{SG47394, &UA2974078},
	{SG47398, &UA2854949},
	{SG47400, &UA2975075},
	{SG47402, &UA3031511},
	{SG47403, &UA3059445},
	{SG47405, &UA3081470},
	{SG47413, &UA2585170},
	{SG47414, &UA2600030},
	{SG47415, &UA2600038},
	{SG47418, &UA2738308},
	{SG47419, &UA2867977},
	{SG47423, &UA2849083},
	{SG47424, &UA2891964},
	{SG47433, &UA2488236},
	{SG47437, &UA2634481},
	{SG47438, &UA2636708},
	{SG47442, &UA3033056},
	{SG47443, &UA2974070},
	{SG47446, &UA3080371},
	{SG47447, &UA3080865},
	{SG47448, &UA3132631},
	{SG47457, &UA2671107},
	{SG47458, &UA2782631},
	{SG47459, &UA2783356},
	{SG47460, &UA2796965},
	{SG47461, &UA2866653},
	{SG47462, &UA3018085},
	{SG47475, &UA2975804},
	{SG47477, &UA3092424},
	{SG47488, &UA2754734},
	{SG47493, &UA3022998},
	{SG47501, &UA2975538},
	{SG47502, &UA2974309},
	{SG47503, &UA3022554},
	{SG47504, &UA3023438},
	{SG47508, &UA3029878},
	{SG47518, &UA2527223},
	{SG47519, &UA2696983},
	{SG47526, &UA2784879},
	{SG47529, &UA2964025},
	{SG47531, &UA3077858},
	{SG47532, &UA2975671},
	{SG47583, &UA2583691},
	{SG47600, &UA2583169},
	{SG47604, &UA2942652},
	{SG47606, &UA3121881},
	{SG47614, &UA2675279},
	{SG47617, &UA2753389},
	{SG47618, &UA2774056},
	{SG47619, &UA2866854},
	{SG47620, &UA2937855},
	{SG47630, &UA2773185},
	{SG47634, &UA3022561},
	{SG47637, &UA3083561},
	{SG47640, &UA2423594},
	{SG47643, &UA2580795},
	{SG47648, &UA2657663},
	{SG47650, &UA3099853},
	{SG47656, &UA2769558},
	{SG47659, &UA2945151},
	{SG47662, &UA2966355},
	{SG47691, &UA2973479},
	{SG47693, &UA3054788},
	{SG47695, &UA3096249},
	{SG47703, &UA2424515},
	{SG47704, &UA2506818},
	{SG47707, &UA2763117},
	{SG47714, &UA2774144},
	{SG47715, &UA2797441},
	{SG47717, &UA2892917},
	{SG47719, &UA3021168},
	{SG47720, &UA3031249},
	{SG47723, &UA3094358},
	{SG47727, &UA2403265},
	{SG47728, &UA2427923},
	{SG47729, &UA2527439},
	{SG47731, &UA3078587},
	{SG47732, &UA3039313},
	{SG47733, &UA3081675},
	{SG47734, &UA3132714},
	{SG47735, &UA3081398},
	{SG47738, &UA2480633},
	{SG47739, &UA2855905},
	{SG47740, &UA3020417},
	{SG47741, &UA2975999},
	{SG47743, &UA3080170},
	{SG47747, &UA2666866},
	{SG47749, &UA2958808},
	{SG47750, &UA2676058},
	{SG47752, &UA2708044},
	{SG47753, &UA2713692},
	{SG47755, &UA2753051},
	{SG47756, &UA2773966},
	{SG47757, &UA2773664},
	{SG47758, &UA2797582},
	{SG47759, &UA2877298},
	{SG47772, &UA3093865},
	{SG47774, &UA3115335},
	{SG47793, &UA2673615},
	{SG47795, &UA2935170},
	{SG47797, &UA2936994},
	{SG47798, &UA3056617},
	{SG47799, &UA3077121},
	{SG47801, &UA2997989},
	{SG47803, &UA3049139},
	{SG47806, &UA3088374},
	{SG47808, &UA3092355},
	{SG47809, &UA3081097},
	{SG47811, &UA3083181},
	{SG47866, &UA2567502},
	{SG47867, &UA3047236},
	{SG47874, &UA2667017},
	{SG47876, &UA3021651},
	{SG47877, &UA2753036},
	{SG47878, &UA2968094},
	{SG47879, &UA2975556},
	{SG47880, &UA3033081},
	{SG47886, &UA2527483},
	{SG47887, &UA2779553},
	{SG47891, &UA2603303},
	{SG47892, &UA2667481},
	{SG47893, &UA2861468},
	{SG47897, &UA2631286},
	{SG47898, &UA2635801},
	{SG47902, &UA2799114},
	{SG47914, &UA2889463},
	{SG47917, &UA2457769},
	{SG47921, &UA2799081},
	{SG47923, &UA3019230},
	{SG47926, &UA3092114},
	{SG47929, &UA2975580},
	{SG47931, &UA3022276},
	{SG47938, &UA2738718},
	{SG47939, &UA2767298},
	{SG47940, &UA2866686},
	{SG47941, &UA2957642},
	{SG47942, &UA3124561},
	{SG47949, &UA3021500},
	{SG47951, &UA3081333},
	{SG47952, &UA3130725},
	{SG47953, &UA3131086},
	{SG47960, &UA2799246},
	{SG47961, &UA3050819},
	{SG47964, &UA3074328},
	{SG47966, &UA2972712},
	{SG47972, &UA3031562},
	{SG47974, &UA3109150},
	{SG47977, &UA3096277},
	{SG47984, &UA2784707},
	{SG47986, &UA3096347},
	{SG48003, &UA2714244},
	{SG48004, &UA2754348},
	{SG48005, &UA2941733},
	{SG48006, &UA2956262},
	{SG48007, &UA3016355},
	{SG48008, &UA3033173},
	{SG48009, &UA3020862},
	{SG48018, &UA2699923},
	{SG48020, &UA2385241},
	{SG48025, &UA2676832},
	{SG48030, &UA2774092},
	{SG48032, &UA2945145},
	{SG48034, &UA3080253},
	{SG48051, &UA2617375},
	{SG48055, &UA2765076},
	{SG48057, &UA2778929},
	{SG48064, &UA3083373},
	{SG48110, &UA2727291},
	{SG48115, &UA2941777},
	{SG48116, &UA2953441},
	{SG48117, &UA2993673},
	{SG48118, &UA3074358},
	{SG48119, &UA3023526},
	{SG48120, &UA3021231},
	{SG48124, &UA3130939},
	{SG48130, &UA2734431},
	{SG48136, &UA2975807},
	{SG48139, &UA2354824},
	{SG48209, &UA2527499},
	{SG48214, &UA2766630},
	{SG48215, &UA3073897},
	{SG48217, &UA3014802},
	{SG48219, &UA2770948},
	{SG48220, &UA2774001},
	{SG48221, &UA2799401},
	{SG48222, &UA2782019},
	{SG48234, &UA2767760},
	{SG48236, &UA3109118},
	{SG48238, &UA2703290},
	{SG48241, &UA3064344},
	{SG48244, &UA3108069},
	{SG48251, &UA2697181},
	{SG48254, &UA2622875},
	{SG48257, &UA2637334},
	{SG48258, &UA2638032},
	{SG48261, &UA2993770},
	{SG48266, &UA2753219},
	{SG48268, &UA2892971},
	{SG48272, &UA2754638},
	{SG48273, &UA3028285},
	{SG48276, &UA2261657},
	{SG48279, &UA2455635},
	{SG48282, &UA2851336},
	{SG48283, &UA3049767},
	{SG48285, &UA3121644},
	{SG48301, &UA2676743},
	{SG48304, &UA2753007},
	{SG48305, &UA2938031},
	{SG48309, &UA3023118},
	{SG48311, &UA2973673},
	{SG48313, &UA2999210},
	{SG48316, &UA3108552},
	{SG48337, &UA2422413},
	{SG48339, &UA2528303},
	{SG48341, &UA2554331},
	{SG48343, &UA2668784},
	{SG48345, &UA2895545},
	{SG48352, &UA2974102},
	{SG48353, &UA2976698},
	{SG48363, &UA2567005},
	{SG48366, &UA2633888},
	{SG48368, &UA2634797},
	{SG48369, &UA2583695},
	{SG48370, &UA2877601},
	{SG48372, &UA2673163},
	{SG48373, &UA3103307},
	{SG48380, &UA2782249},
	{SG48383, &UA2868373},
	{SG48387, &UA3094730},
	{SG48404, &UA2784755},
	{SG48413, &UA2366525},
	{SG48414, &UA2456049},
	{SG48420, &UA2774336},
	{SG48421, &UA2800679},
	{SG48422, &UA2994470},
	{SG48423, &UA3015877},
	{SG48424, &UA3022972},
	{SG48425, &UA2976007},
	{SG48426, &UA3036276},
	{SG48427, &UA3080131},
	{SG48428, &UA3130730},
	{SG48439, &UA2773870},
	{SG48442, &UA3021553},
	{SG48443, &UA3078152},
	{SG48444, &UA3081273},
	{SG48445, &UA3083754},
	{SG48459, &UA2596507},
	{SG48463, &UA2665110},
	{SG48465, &UA2666273},
	{SG48468, &UA2778423},
	{SG48469, &UA3044923},
	{SG48476, &UA2941933},
	{SG48480, &UA3023105},
	{SG48481, &UA3050240},
	{SG48482, &UA3053934},
	{SG48483, &UA3091193},
	{SG48589, &UA3081383},
	{SG48592, &UA2579524},
	{SG48595, &UA2711926},
	{SG48596, &UA2672461},
	{SG48597, &UA2715056},
	{SG48598, &UA2799368},
	{SG48599, &UA2937545},
	{SG48600, &UA3080495},
	{SG48601, &UA3132859},
	{SG48605, &UA2663162},
	{SG48606, &UA2675776},
	{SG48608, &UA2796342},
	{SG48609, &UA3013126},
	{SG48610, &UA2773482},
	{SG48611, &UA2971844},
	{SG48612, &UA2893495},
	{SG48613, &UA2945904},
	{SG48614, &UA2993080},
	{SG48615, &UA3080459},
	{SG48616, &UA3111850},
	{SG48621, &UA2556780},
	{SG48664, &UA2632723},
	{SG48666, &UA2603905},
	{SG48667, &UA2753252},
	{SG48670, &UA2957484},
	{SG48688, &UA2634739},
	{SG48690, &UA3092364},
	{SG48692, &UA2670602},
	{SG48697, &UA2755346},
	{SG48702, &UA2944990},
	{SG48717, &UA2457445},
	{SG48720, &UA2567503},
	{SG48722, &UA2737633},
	{SG48724, &UA2975798},
	{SG48726, &UA3080452},
	{SG48727, &UA3130729},
	{SG48734, &UA2616859},
	{SG48735, &UA2617344},
	{SG48736, &UA2618070},
	{SG48737, &UA2675894},
	{SG48738, &UA2713175},
	{SG48739, &UA2734939},
	{SG48740, &UA2798714},
	{SG48741, &UA2753695},
	{SG48743, &UA2756851},
	{SG48745, &UA2769355},
	{SG48747, &UA2958999},
	{SG48749, &UA2960299},
	{SG48750, &UA3074891},
	{SG48752, &UA3039311},
	{SG48756, &UA2232804},
	{SG48759, &UA2752792},
	{SG48767, &UA2739608},
	{SG48768, &UA2956670},
	{SG48772, &UA2782401},
	{SG48775, &UA2801271},
	{SG48776, &UA2866734},
	{SG48779, &UA2967547},
	{SG48781, &UA3021230},
	{SG48783, &UA3083363},
	{SG48791, &UA2653007},
	{SG48792, &UA2554018},
	{SG48794, &UA2555825},
	{SG48795, &UA2934504},
	{SG48801, &UA2769773},
	{SG48803, &UA2773709},
	{SG48805, &UA3033066},
	{SG48816, &UA2735160},
	{SG48818, &UA2865893},
	{SG48820, &UA2998828},
	{SG48821, &UA2769383},
	{SG48836, &UA2733140},
	{SG48842, &UA2799248},
	{SG48845, &UA2867814},
	{SG48846, &UA3088711},
	{SG48849, &UA2901220},
	{SG48852, &UA2958298},
	{SG48854, &UA3131411},
	{SG48865, &UA2605533},
	{SG48867, &UA2862116},
	{SG48868, &UA3017986},
	{SG48870, &UA2782828},
	{SG48872, &UA3032801},
	{SG48874, &UA3020792},
	{SG48877, &UA3081054},
	{SG48878, &UA3083662},
	{SG48889, &UA2455921},
	{SG48893, &UA2838755},
	{SG48897, &UA3017105},
	{SG48904, &UA2737702},
	{SG48908, &UA2783453},
	{SG48909, &UA2938244},
	{SG48910, &UA3020272},
	{SG48911, &UA3080659},
	{SG48913, &UA3109717},
	{SG48916, &UA3127827},
	{SG48932, &UA2537620},
	{SG48935, &UA2676326},
	{SG48939, &UA2773359},
	{SG48940, &UA2784086},
	{SG48943, &UA2864629},
	{SG48946, &UA2975994},
	{SG48948, &UA3109396},
	{SG48949, &UA3111580},
	{SG48950, &UA3126912},
	{SG48951, &UA2384590},
	{SG48952, &UA3073240},
	{SG48956, &UA2672071},
	{SG48958, &UA2718761},
	{SG48960, &UA2868571},
	{SG48962, &UA2888373},
	{SG48964, &UA2868822},
	{SG48965, &UA2892654},
	{SG48966, &UA2956824},
	{SG48986, &UA3049008},
	{SG48991, &UA3111218},
	{SG48993, &UA3111707},
	{SG48999, &UA2673582},
	{SG49006, &UA2953129},
	{SG49008, &UA3022604},
	{SG49009, &UA2974444},
	{SG49011, &UA2967121},
	{SG49022, &UA2617138},
	{SG49023, &UA2617266},
	{SG49024, &UA2617349},
	{SG49065, &UA3061610},
	{SG49070, &UA3081045},
	{SG49072, &UA2457282},
	{SG49078, &UA2583258},
	{SG49080, &UA2670727},
	{SG49085, &UA2961812},
	{SG49086, &UA2974451},
	{SG49087, &UA3030365},
	{SG49094, &UA2355141},
	{SG49100, &UA2711078},
	{SG49103, &UA2879573},
	{SG49104, &UA3022520},
	{SG49106, &UA3023532},
	{SG49108, &UA3081277},
	{SG49110, &UA3062368},
	{SG49111, &UA3083763},
	{SG49115, &UA2567395},
	{SG49117, &UA2653074},
	{SG49118, &UA2755415},
	{SG49120, &UA2868747},
	{SG49121, &UA3108278},
	{SG49158, &UA3080979},
	{SG49163, &UA2796053},
	{SG49164, &UA2865731},
	{SG49166, &UA3080842},
	{SG49167, &UA3081032},
	{SG49168, &UA3111515},
	{SG49172, &UA2355028},
	{SG49175, &UA2539944},
	{SG49177, &UA2600332},
	{SG49179, &UA2666971},
	{SG49180, &UA2710483},
	{SG49181, &UA2749851},
	{SG49184, &UA2937915},
	{SG49185, &UA2773606},
	{SG49187, &UA2794535},
	{SG49189, &UA3046349},
	{SG49193, &UA3036637},
	{SG49195, &UA3021833},
	{SG49196, &UA3030387},
	{SG49211, &UA2659272},
	{SG49213, &UA2957594},
	{SG49215, &UA2754173},
	{SG49218, &UA2773874},
	{SG49219, &UA2879171},
	{SG49220, &UA2999702},
	{SG49221, &UA2866057},
	{SG49222, &UA3020949},
	{SG49227, &UA3094759},
	{SG49278, &UA2601729},
	{SG49283, &UA2676385},
	{SG49288, &UA2784753},
	{SG49290, &UA2974717},
	{SG49295, &UA3130587},
	{SG49307, &UA2583318},
	{SG49310, &UA2767799},
	{SG49312, &UA2784084},
	{SG49313, &UA2796800},
	{SG49314, &UA2963278},
	{SG49316, &UA3076623},
	{SG49319, &UA2467050},
	{SG49327, &UA3032182},
	{SG49328, &UA3082983},
	{SG49342, &UA2672192},
	{SG49344, &UA2713170},
	{SG49349, &UA2755411},
	{SG49351, &UA2756334},
	{SG49353, &UA2773866},
	{SG49359, &UA3081085},
	{SG49375, &UA2666376},
	{SG49376, &UA2669215},
	{SG49380, &UA2800098},
	{SG49382, &UA2892829},
	{SG49386, &UA3032061},
	{SG49388, &UA2999013},
	{SG49389, &UA3108045},
	{SG49397, &UA2633685},
	{SG49400, &UA2708060},
	{SG49402, &UA3022796},
	{SG49405, &UA3023223},
	{SG49407, &UA3081083},
	{SG49412, &UA2708743},
	{SG49415, &UA2753319},
	{SG49417, &UA2783723},
	{SG49420, &UA2792208},
	{SG49425, &UA2954182},
	{SG49427, &UA3023513},
	{SG49430, &UA3032480},
	{SG49462, &UA2675053},
	{SG49464, &UA2763506},
	{SG49465, &UA3046561},
	{SG49470, &UA2956135},
	{SG49474, &UA3037930},
	{SG49479, &UA3081618},
	{SG49483, &UA3081289},
	{SG49484, &UA3094834},
	{SG49485, &UA3083335},
	{SG49496, &UA2355686},
	{SG49500, &UA2583176},
	{SG49513, &UA3094771},
	{SG49526, &UA2354821},
	{SG49528, &UA2355146},
	{SG49529, &UA2621266},
	{SG49534, &UA2598604},
	{SG49537, &UA2674449},
	{SG49542, &UA2681021},
	{SG49549, &UA2783701},
	{SG49551, &UA2799591},
	{SG49553, &UA2975711},
	{SG49556, &UA3064712},
	{SG49558, &UA3081225},
	{SG49575, &UA2676983},
	{SG49577, &UA2719585},
	{SG49579, &UA2719627},
	{SG49580, &UA2719628},
	{SG49583, &UA2719649},
	{SG49584, &UA2719667},
	{SG49585, &UA2719675},
	{SG49587, &UA2719680},
	{SG49588, &UA2719689},
	{SG49589, &UA2719704},
	{SG49591, &UA2961719},
	{SG49592, &UA2783071},
	{SG49594, &UA2869200},
	{SG49595, &UA3050969},
	{SG49596, &UA2874773},
	{SG49611, &UA2975978},
	{SG49619, &UA3093678},
	{SG49627, &UA2429426},
	{SG49636, &UA2666773},
	{SG49637, &UA2668690},
	{SG49638, &UA2702625},
	{SG49640, &UA2753345},
	{SG49643, &UA2769729},
	{SG49645, &UA2782972},
	{SG49646, &UA2935264},
	{SG49647, &UA3103398},
	{SG49650, &UA2955765},
	{SG49652, &UA3058325},
	{SG49654, &UA3080368},
	{SG49670, &UA2654016},
	{SG49676, &UA2963496},
	{SG49677, &UA3132016},
	{SG49690, &UA3000024},
	{SG49691, &UA2704526},
	{SG49692, &UA2767504},
	{SG49694, &UA2957139},
	{SG49696, &UA3061220},
	{SG49697, &UA3078696},
	{SG49700, &UA2367043},
	{SG49701, &UA2486903},
	{SG49706, &UA2707084},
	{SG49708, &UA2708563},
	{SG49710, &UA2750942},
	{SG49713, &UA2869842},
	{SG49715, &UA3022410},
	{SG49717, &UA2975987},
	{SG49727, &UA2556799},
	{SG49731, &UA2597644},
	{SG49734, &UA2767644},
	{SG49736, &UA2901699},
	{SG49738, &UA2769110},
	{SG49739, &UA2799622},
	{SG49741, &UA2870443},
	{SG49743, &UA2944988},
	{SG49745, &UA2962572},
	{SG49746, &UA2944365},
	{SG49751, &UA2994870},
	{SG49752, &UA3049243},
	{SG49775, &UA3131433},
	{SG49776, &UA3132029},
	{SG49778, &UA2503298},
	{SG49782, &UA2583187},
	{SG49785, &UA2674742},
	{SG49787, &UA2735948},
	{SG49788, &UA2766907},
	{SG49792, &UA2757297},
	{SG49795, &UA2757298},
	{SG49797, &UA2773666},
	{SG49806, &UA2954925},
	{SG49807, &UA2994346},
	{SG49809, &UA3022792},
	{SG49819, &UA2528697},
	{SG49825, &UA2709389},
	{SG49829, &UA2769698},
	{SG49831, &UA2870073},
	{SG49832, &UA2998921},
	{SG49834, &UA2987663},
	{SG49861, &UA2676880},
	{SG49862, &UA2700540},
	{SG49863, &UA2703062},
	{SG49868, &UA2781172},
	{SG49870, &UA2940670},
	{SG49871, &UA3094438},
	{SG49872, &UA3094803},
	{SG49874, &UA2578367},
	{SG49876, &UA2653966},
	{SG49880, &UA2867380},
	{SG49882, &UA3093618},
	{SG49883, &UA3093914},
	{SG49885, &UA3078911},
	{SG49886, &UA3108503},
	{SG49887, &UA3108849},
	{SG49928, &UA2798075},
	{SG49978, &UA2658564},
	{SG49984, &UA2734905},
	{SG49986, &UA3087739},
	{SG49988, &UA2750297},
	{SG49993, &UA2874509},
	{SG49996, &UA2968507},
	{SG49998, &UA3026020},
	{SG50000, &UA3093394},
	{SG50002, &UA2633804},
	{SG50003, &UA2893393},
	{SG50012, &UA2404033},
	{SG50017, &UA2503013},
	{SG50020, &UA2622768},
	{SG50026, &UA2603758},
	{SG50032, &UA2728066},
	{SG50033, &UA2747013},
	{SG50036, &UA2782967},
	{SG50038, &UA2784085},
	{SG50040, &UA2855242},
	{SG50042, &UA2865564},
	{SG50045, &UA2898653},
	{SG50049, &UA3021084},
	{SG50054, &UA3023687},
	{SG50055, &UA3023928},
	{SG50058, &UA3131398},
	{SG50071, &UA2455474},
	{SG50072, &UA2456165},
	{SG50076, &UA2792661},
	{SG50077, &UA2675710},
	{SG50078, &UA2676061},
	{SG50084, &UA2860744},
	{SG50087, &UA2941538},
	{SG50089, &UA3083357},
	{SG50091, &UA3065540},
	{SG50092, &UA3109344},
	{SG50095, &UA3131085},
	{SG50112, &UA2555350},
	{SG50115, &UA2695325},
	{SG50120, &UA2583325},
	{SG50122, &UA2734686},
	{SG50124, &UA2734824},
	{SG50125, &UA2735628},
	{SG50127, &UA2791491},
	{SG50129, &UA2784622},
	{SG50130, &UA2793246},
	{SG50132, &UA2962911},
	{SG50133, &UA3006210},
	{SG50135, &UA3031331},
	{SG50137, &UA3081376},
	{SG50144, &UA2732268},
	{SG50145, &UA2752202},
	{SG50147, &UA2769320},
	{SG50149, &UA2773820},
	{SG50150, &UA2773958},
	{SG50151, &UA2799784},
	{SG50153, &UA2885445},
	{SG50155, &UA2892883},
	{SG50159, &UA2962539},
	{SG50160, &UA3021970},
	{SG50162, &UA3083374},
	{SG50169, &UA2428625},
	{SG50172, &UA2355163},
	{SG50180, &UA2602610},
	{SG50184, &UA2769376},
	{SG50187, &UA2773562},
	{SG50190, &UA2799275},
	{SG50191, &UA2800377},
	{SG50192, &UA2862677},
	{SG50194, &UA2956261},
	{SG50195, &UA2945204},
	{SG50196, &UA2958192},
	{SG50198, &UA2966412},
	{SG50202, &UA3020523},
	{SG50204, &UA3080018},
	{SG50205, &UA3093788},
	{SG50207, &UA3109457},
	{SG50208, &UA3110284},
	{SG50216, &UA2458290},
	{SG50223, &UA2900612},
	{SG50225, &UA2938210},
	{SG50228, &UA3023932},
	{SG50232, &UA3083355},
	{SG50243, &UA2676365},
	{SG50244, &UA2703370},
	{SG50245, &UA2732966},
	{SG50248, &UA2755283},
	{SG50250, &UA2794761},
	{SG50251, &UA2800684},
	{SG50252, &UA2870741},
	{SG50253, &UA2879430},
	{SG50255, &UA2937553},
	{SG50256, &UA2870896},
	{SG50257, &UA3030669},
	{SG50258, &UA2879198},
	{SG50260, &UA3023023},
	{SG50263, &UA3131304},
	{SG50286, &UA2714152},
	{SG50289, &UA2768780},
	{SG50292, &UA2796666},
	{SG50294, &UA2798559},
	{SG50295, &UA2892759},
	{SG50297, &UA2855407},
	{SG50301, &UA3022216},
	{SG50303, &UA3022403},
	{SG50306, &UA3095649},
	{SG50331, &UA2583321},
	{SG50332, &UA2732533},
	{SG50336, &UA2637044},
	{SG50337, &UA2995677},
	{SG50340, &UA2671844},
	{SG50342, &UA2677002},
	{SG50345, &UA2770731},
	{SG50346, &UA2770728},
	{SG50349, &UA2944321},
	{SG50351, &UA3006233},
	{SG50352, &UA2935903},
	{SG50354, &UA2956546},
	{SG50356, &UA3023733},
	{SG50357, &UA3023109},
	{SG50358, &UA3059979},
	{SG50383, &UA2604717},
	{SG50387, &UA2674841},
	{SG50391, &UA2800153},
	{SG50394, &UA2943783},
	{SG50396, &UA3033271},
	{SG50397, &UA3078929},
	{SG50398, &UA3094809},
	{SG50399, &UA3080076},
	{SG50401, &UA3083220},
	{SG50402, &UA3115120},
	{SG50403, &UA3115338},
	{SG50425, &UA2658191},
	{SG50427, &UA2673055},
	{SG50430, &UA2716573},
	{SG50433, &UA2798119},
	{SG50443, &UA2958221},
	{SG50447, &UA3081103},
	{SG50449, &UA3093495},
	{SG50450, &UA3113029},
	{SG50453, &UA3126390},
	{SG50454, &UA3094701},
	{SG50457, &UA3131313},
	{SG50472, &UA2506598},
	{SG50475, &UA2656091},
	{SG50478, &UA2707963},
	{SG50482, &UA2753365},
	{SG50488, &UA3022505},
	{SG50513, &UA2706999},
	{SG50514, &UA2709134},
	{SG50516, &UA2772870},
	{SG50520, &UA3023194},
	{SG50524, &UA3024139},
	{SG50527, &UA3061929},
	{SG50529, &UA3083666},
	{SG50532, &UA2384650},
	{SG50533, &UA2846754},
	{SG50535, &UA2466443},
	{SG50537, &UA2708883},
	{SG50542, &UA2698845},
	{SG50543, &UA2752692},
	{SG50546, &UA2769941},
	{SG50547, &UA2944035},
	{SG50549, &UA2892335},
	{SG50550, &UA3121578},
	{SG50551, &UA3079391},
	{SG50569, &UA2487481},
	{SG50573, &UA2704001},
	{SG50575, &UA2715051},
	{SG50579, &UA2794400},
	{SG50581, &UA2879035},
	{SG50585, &UA2962072},
	{SG50605, &UA2504844},
	{SG50609, &UA2677280},
	{SG50612, &UA2709752},
	{SG50615, &UA2782647},
	{SG50621, &UA3022725},
	{SG50632, &UA2384662},
	{SG50635, &UA2458446},
	{SG50636, &UA2427063},
	{SG50639, &UA2421443},
	{SG50640, &UA2600334},
	{SG50641, &UA2601554},
	{SG50642, &UA2670903},
	{SG50643, &UA2713172},
	{SG50644, &UA2869317},
	{SG50646, &UA2867955},
	{SG50649, &UA3060767},
	{SG50652, &UA3021132},
	{SG50653, &UA3022482},
	{SG50654, &UA3020653},
	{SG50658, &UA3094808},
	{SG50661, &UA3094675},
	{SG50668, &UA2366991},
	{SG50671, &UA2455533},
	{SG50672, &UA2556393},
	{SG50673, &UA2484018},
	{SG50680, &UA2867285},
	{SG50681, &UA2868422},
	{SG50682, &UA2892894},
	{SG50685, &UA2964176},
	{SG50690, &UA3022498},
	{SG50691, &UA3060466},
	{SG50692, &UA3132934},
	{SG50705, &UA2428760},
	{SG50712, &UA2673958},
	{SG50713, &UA2671027},
	{SG50716, &UA2675922},
	{SG50717, &UA2796673},
	{SG50719, &UA3028631},
	{SG50720, &UA3033273},
	{SG50721, &UA3078975},
	{SG50723, &UA3093787},
	{SG50736, &UA2567027},
	{SG50748, &UA2712727},
	{SG50749, &UA2737910},
	{SG50750, &UA2769430},
	{SG50751, &UA2783295},
	{SG50755, &UA2958217},
	{SG50758, &UA3122743},
	{SG50762, &UA3016629},
	{SG50764, &UA3021336},
	{SG50767, &UA3022642},
	{SG50770, &UA3027905},
	{SG50773, &UA3038052},
	{SG50775, &UA3081053},
	{SG50777, &UA3111649},
	{SG50779, &UA3130948},
	{SG50808, &UA2653301},
	{SG50810, &UA2779724},
	{SG50815, &UA2770985},
	{SG50820, &UA2782911},
	{SG50824, &UA2891111},
	{SG50826, &UA2893131},
	{SG50829, &UA2964357},
	{SG50832, &UA3022200},
	{SG50835, &UA3093349},
	{SG50844, &UA2355670},
	{SG50849, &UA2668820},
	{SG50850, &UA2718566},
	{SG50856, &UA2769030},
	{SG50858, &UA2879484},
	{SG50860, &UA2944553},
	{SG50862, &UA3021595},
	{SG50864, &UA3021377},
	{SG50865, &UA2968808},
	{SG50867, &UA3059864},
	{SG50869, &UA3094797},
	{SG50880, &UA2606282},
	{SG50884, &UA2672175},
	{SG50888, &UA2708560},
	{SG50894, &UA2935115},
	{SG50896, &UA2953761},
	{SG50901, &UA3059199},
	{SG50903, &UA3027008},
	{SG50925, &UA2778568},
	{SG50929, &UA2602396},
	{SG50938, &UA2714392},
	{SG50946, &UA2799022},
	{SG50949, &UA2879085},
	{SG50953, &UA2994959},
	{SG50955, &UA3003768},
	{SG50956, &UA3050569},
	{SG50960, &UA3093932},
	{SG50978, &UA2423839},
	{SG50986, &UA2672023},
	{SG50988, &UA2892609},
	{SG50990, &UA2782576},
	{SG50992, &UA2856901},
	{SG50993, &UA2863071},
	{SG50994, &UA2869995},
	{SG50995, &UA2944582},
	{SG50996, &UA2962037},
	{SG50997, &UA3015352},
	{SG51000, &UA3018677},
	{SG51002, &UA3029324},
	{SG51004, &UA3083365},
	{SG51009, &UA3109547},
	{SG51011, &UA3112880},
	{SG51016, &UA2603266},
	{SG51017, &UA2675853},
	{SG51018, &UA2770020},
	{SG51026, &UA2963972},
	{SG51031, &UA3057852},
	{SG51034, &UA3079778},
	{SG51036, &UA3093986},
	{SG51039, &UA3094094},
	{SG51040, &UA3094530},
	{SG51043, &UA3095050},
	{SG51046, &UA3131733},
	{SG51065, &UA2600940},
	{SG51068, &UA2708382},
	{SG51071, &UA2781526},
	{SG51075, &UA3022227},
	{SG51080, &UA3026944},
	{SG51083, &UA3080984},
	{SG51101, &UA2526849},
	{SG51147, &UA2869919},
	{SG51151, &UA2848955},
	{SG51152, &UA2893073},
	{SG51187, &UA3022558},
	{SG51188, &UA3021324},
	{SG51193, &UA3062355},
	{SG51194, &UA3027671},
	{SG51195, &UA3048830},
	{SG51196, &UA3060642},
	{SG51198, &UA3094672},
	{SG51200, &UA3093771},
	{SG51202, &UA3131420},
	{SG51217, &UA2673252},
	{SG51219, &UA2832606},
	{SG51222, &UA3085777},
	{SG51228, &UA2767288},
	{SG51229, &UA3024386},
	{SG51232, &UA2798842},
	{SG51236, &UA2892891},
	{SG51241, &UA3022371},
	{SG51249, &UA3109624},
	{SG51270, &UA2484996},
	{SG51271, &UA2507969},
	{SG51274, &UA2666962},
	{SG51281, &UA2864377},
	{SG51282, &UA2962910},
	{SG51288, &UA3094778},
	{SG51290, &UA3115118},
	{SG51291, &UA3130942},
	{SG51305, &UA2594808},
	{SG51313, &UA2894487},
	{SG51315, &UA2938523},
	{SG51318, &UA3005751},
	{SG51323, &UA3023813},
	{SG51324, &UA3025616},
	{SG51329, &UA3109628},
	{SG51342, &UA3043413},
	{SG51345, &UA2842227},
	{SG51347, &UA2456797},
	{SG51350, &UA2792148},
	{SG51351, &UA2738060},
	{SG51352, &UA2604124},
	{SG51353, &UA2635582},
	{SG51354, &UA2666735},
	{SG51355, &UA2704515},
	{SG51356, &UA2709048},
	{SG51357, &UA2769586},
	{SG51358, &UA2772876},
	{SG51359, &UA2774116},
	{SG51360, &UA2778404},
	{SG51364, &UA2798604},
	{SG51367, &UA2892419},
	{SG51368, &UA3050912},
	{SG51371, &UA3060464},
	{SG51372, &UA3019614},
	{SG51373, &UA3020977},
	{SG51375, &UA3023401},
	{SG51376, &UA3060552},
	{SG51380, &UA3094273},
	{SG51404, &UA2797641},
	{SG51405, &UA2937456},
	{SG51407, &UA2799224},
	{SG51410, &UA2892218},
	{SG51411, &UA2984863},
	{SG51412, &UA2956620},
	{SG51414, &UA3102708},
	{SG51417, &UA3022117},
	{SG51418, &UA3021432},
	{SG51423, &UA3012199},
	{SG51424, &UA3022618},
	{SG51432, &UA3079791},
	{SG51433, &UA3125774},
	{SG51435, &UA3110306},
	{SG51437, &UA3093677},
	{SG51456, &UA2658054},
	{SG51461, &UA2708783},
	{SG51464, &UA2769032},
	{SG51469, &UA3014803},
	{SG51470, &UA2800509},
	{SG51473, &UA2868298},
	{SG51477, &UA2957471},
	{SG51478, &UA2999380},
	{SG51480, &UA2957657},
	{SG51482, &UA2998496},
	{SG51484, &UA2958864},
	{SG51487, &UA2958979},
	{SG51489, &UA3009735},
	{SG51490, &UA3077917},
	{SG51492, &UA3058909},
	{SG51524, &UA2601076},
	{SG51526, &UA2634359},
	{SG51531, &UA2708673},
	{SG51532, &UA2710203},
	{SG51534, &UA2718963},
	{SG51536, &UA2749884},
	{SG51540, &UA2779809},
	{SG51541, &UA2780972},
	{SG51542, &UA2782552},
	{SG51544, &UA2868625},
	{SG51545, &UA2937910},
	{SG51548, &UA2997580},
	{SG51549, &UA3014618},
	{SG51550, &UA3080670},
	{SG51568, &UA2673260},
	{SG51573, &UA2752812},
	{SG51578, &UA2782495},
	{SG51581, &UA2872188},
	{SG51588, &UA3023102},
	{SG51590, &UA3078674},
	{SG51593, &UA3111158},
	{SG51596, &UA3131296},
	{SG51610, &UA2867342},
	{SG51611, &UA2868481},
	{SG51613, &UA2954448},
	{SG51615, &UA2957462},
	{SG51616, &UA2958311},
	{SG51617, &UA2995503},
	{SG51621, &UA3059102},
	{SG51623, &UA3033034},
	{SG51627, &UA3027682},
	{SG51641, &UA2747600},
	{SG51643, &UA2555322},
	{SG51646, &UA2603481},
	{SG51648, &UA2604169},
	{SG51653, &UA2713520},
	{SG51657, &UA2751602},
	{SG51663, &UA2865351},
	{SG51665, &UA2879149},
	{SG51667, &UA2892222},
	{SG51670, &UA2944705},
	{SG51673, &UA2966039},
	{SG51676, &UA3024473},
	{SG51679, &UA3056592},
	{SG51683, &UA3094611},
	{SG51684, &UA3109424},
	{SG51685, &UA3113422},
	{SG51687, &UA3130589},
	{SG51705, &UA2384893},
	{SG51708, &UA2423483},
	{SG51715, &UA2604128},
	{SG51724, &UA2783190},
	{SG51726, &UA2797015},
	{SG51728, &UA2799424},
	{SG51734, &UA2963974},
	{SG51736, &UA3023202},
	{SG51738, &UA2998572},
	{SG51741, &UA3023017},
	{SG51744, &UA3112122},
	{SG51745, &UA3127491},
	{SG51753, &UA2361542},
	{SG51754, &UA2667581},
	{SG51755, &UA2706413},
	{SG51756, &UA2710088},
	{SG51757, &UA2763377},
	{SG51758, &UA2779972},
	{SG51759, &UA2891781},
	{SG51760, &UA2941026},
	{SG51761, &UA2956876},
	{SG51763, &UA2966473},
	{SG51764, &UA3048221},
	{SG51766, &UA3012747},
	{SG51767, &UA3017442},
	{SG51769, &UA3020930},
	{SG51770, &UA3110187},
	{SG51772, &UA3131753},
	{SG51789, &UA2860577},
	{SG51792, &UA2716153},
	{SG51797, &UA2957172},
	{SG51804, &UA2800206},
	{SG51810, &UA2892730},
	{SG51814, &UA3022501},
	{SG51815, &UA2987532},
	{SG51817, &UA3004990},
	{SG51819, &UA3104589},
	{SG51821, &UA3021390},
	{SG51824, &UA3114666},
	{SG51852, &UA2616334},
	{SG51855, &UA2621098},
	{SG51857, &UA2708427},
	{SG51861, &UA2709615},
	{SG51865, &UA2735559},
	{SG51866, &UA2752373},
	{SG51870, &UA2773929},
	{SG51874, &UA2783188},
	{SG51877, &UA2796072},
	{SG51883, &UA2999619},
	{SG51884, &UA3073683},
	{SG51885, &UA3005241},
	{SG51886, &UA3010024},
	{SG51888, &UA3083585},
	{SG51894, &UA3131329},
	{SG51908, &UA2485608},
	{SG51916, &UA2607338},
	{SG51919, &UA2712649},
	{SG51923, &UA2752811},
	{SG51926, &UA2778190},
	{SG51929, &UA2783693},
	{SG51933, &UA2868092},
	{SG51934, &UA2869709},
	{SG51937, &UA2937469},
	{SG51940, &UA2991800},
	{SG51941, &UA3011677},
	{SG51948, &UA3059596},
	{SG51951, &UA3093911},
	{SG51971, &UA2702864},
	{SG51973, &UA2997710},
	{SG51975, &UA2713580},
	{SG51982, &UA2869206},
	{SG51983, &UA2879710},
	{SG51984, &UA2898857},
	{SG51986, &UA2936371},
	{SG51988, &UA2971888},
	{SG51989, &UA3003608},
	{SG51991, &UA3059597},
	{SG51994, &UA3015678},
	{SG51997, &UA3020493},
	{SG51999, &UA3021977},
	{SG52000, &UA3049355},
	{SG52001, &UA3056402},
	{SG52002, &UA3078649},
	{SG52003, &UA3125568},
	{SG52004, &UA3114924},
	{SG52006, &UA3130936},
	{SG52019, &UA2402873},
	{SG52022, &UA2457408},
	{SG52024, &UA2484809},
	{SG52028, &UA2602606},
	{SG52029, &UA2654298},
	{SG52031, &UA2657250},
	{SG52032, &UA2791962},
	{SG52033, &UA2671971},
	{SG52035, &UA2681020},
	{SG52036, &UA2712069},
	{SG52037, &UA2713947},
	{SG52038, &UA2715885},
	{SG52042, &UA2762600},
	{SG52046, &UA2779951},
	{SG52048, &UA2985597},
	{SG52049, &UA2783880},
	{SG52052, &UA2796994},
	{SG52053, &UA2798551},
	{SG52055, &UA2799256},
	{SG52056, &UA2799360},
	{SG52058, &UA2799839},
	{SG52061, &UA2891062},
	{SG52063, &UA2938921},
	{SG52065, &UA2963615},
	{SG52067, &UA3046740},
	{SG52069, &UA2999695},
	{SG52071, &UA3048065},
	{SG52074, &UA3021387},
	{SG52075, &UA3057868},
	{SG52077, &UA3111382},
	{SG52078, &UA3126443},
	{SG52082, &UA3115786},
	{SG52084, &UA3132662},
	{SG52098, &UA2466171},
	{SG52105, &UA2677062},
	{SG52107, &UA2706841},
	{SG52108, &UA2707798},
	{SG52113, &UA2798775},
	{SG52118, &UA2991739},
	{SG52119, &UA3016910},
	{SG52135, &UA3057879},
	{SG52139, &UA3079131},
	{SG52140, &UA3079708},
	{SG52144, &UA3131413},
	{SG52146, &UA3132966},
	{SG52166, &UA2705808},
	{SG52177, &UA2799004},
	{SG52179, &UA2799281},
	{SG52181, &UA2867109},
	{SG52183, &UA2894835},
	{SG52184, &UA2954598},
	{SG52187, &UA2938117},
	{SG52188, &UA2956993},
	{SG52190, &UA3060990},
	{SG52192, &UA3059713},
	{SG52193, &UA3061013},
	{SG52205, &UA2603545},
	{SG52206, &UA2660606},
	{SG52208, &UA3108329},
	{SG52210, &UA2661219},
	{SG52212, &UA2715160},
	{SG52213, &UA2791469},
	{SG52215, &UA2768721},
	{SG52216, &UA2868305},
	{SG52218, &UA2799126},
	{SG52220, &UA2800107},
	{SG52223, &UA2867413},
	{SG52225, &UA2893844},
	{SG52226, &UA2879880},
	{SG52228, &UA2952651},
	{SG52230, &UA2958224},
	{SG52231, &UA2963264},
	{SG52233, &UA3013465},
	{SG52236, &UA3022961},
	{SG52238, &UA3081014},
	{SG52240, &UA3078561},
	{SG52242, &UA3094922},
	{SG52256, &UA2850500},
	{SG52259, &UA2556858},
	{SG52276, &UA2797446},
	{SG52277, &UA2800151},
	{SG52281, &UA2868895},
	{SG52284, &UA2945799},
	{SG52286, &UA3014022},
	{SG52311, &UA2455534},
	{SG52320, &UA2747963},
	{SG52322, &UA2748129},
	{SG52326, &UA2994593},
	{SG52327, &UA2751879},
	{SG52328, &UA2769171},
	{SG52332, &UA2870317},
	{SG52335, &UA2959383},
	{SG52337, &UA2999862},
	{SG52339, &UA3006197},
	{SG52340, &UA3125652},
	{SG52341, &UA3056145},
	{SG52342, &UA3058589},
	{SG52344, &UA3059198},
	{SG52348, &UA3115330},
	{SG52349, &UA3131732},
	{SG52364, &UA2528592},
	{SG52367, &UA2531215},
	{SG52372, &UA2661223},
	{SG52373, &UA2668617},
	{SG52374, &UA2859350},
	{SG52375, &UA2701056},
	{SG52378, &UA2767763},
	{SG52379, &UA2778703},
	{SG52380, &UA2877173},
	{SG52382, &UA3058548},
	{SG52383, &UA2967278},
	{SG52385, &UA2998069},
	{SG52386, &UA3021089},
	{SG52387, &UA3022006},
	{SG52388, &UA3022698},
	{SG52390, &UA3108945},
	{SG52392, &UA3060260},
	{SG52393, &UA3079679},
	{SG52395, &UA3094581},
	{SG52399, &UA3126351},
	{SG52400, &UA3109496},
	{SG52413, &UA2601583},
	{SG52416, &UA2676059},
	{SG52419, &UA3044579},
	{SG52420, &UA2783006},
	{SG52425, &UA2868409},
	{SG52426, &UA2869069},
	{SG52427, &UA3007716},
	{SG52429, &UA2877849},
	{SG52430, &UA2879344},
	{SG52431, &UA2892682},
	{SG52432, &UA2936548},
	{SG52439, &UA3021575},
	{SG52442, &UA3022306},
	{SG52445, &UA3022786},
	{SG52448, &UA3029591},
	{SG52450, &UA3029627},
	{SG52452, &UA3050423},
	{SG52454, &UA3115437},
	{SG52472, &UA2456707},
	{SG52473, &UA2843199},
	{SG52480, &UA2712424},
	{SG52481, &UA2713398},
	{SG52482, &UA2736956},
	{SG52484, &UA2936774},
	{SG52485, &UA2937912},
	{SG52486, &UA2957198},
	{SG52491, &UA3012715},
	{SG52492, &UA3012963},
	{SG52493, &UA3017100},
	{SG52494, &UA3020431},
	{SG52500, &UA3057973},
	{SG52501, &UA3061197},
	{SG52504, &UA3109941},
	{SG52519, &UA2556704},
	{SG52520, &UA2667487},
	{SG52521, &UA2677297},
	{SG52522, &UA2734411},
	{SG52524, &UA2794494},
	{SG52526, &UA2798921},
	{SG52527, &UA2869036},
	{SG52528, &UA2869246},
	{SG52530, &UA2870044},
	{SG52531, &UA2876180},
	{SG52534, &UA2898277},
	{SG52537, &UA3049622},
	{SG52538, &UA2998655},
	{SG52540, &UA3016107},
	{SG52541, &UA3092820},
	{SG52543, &UA3132663},
	{SG52551, &UA2456880},
	{SG52553, &UA2484900},
	{SG52563, &UA2790668},
	{SG52568, &UA2734647},
	{SG52570, &UA2764354},
	{SG52571, &UA2795880},
	{SG52572, &UA2855213},
	{SG52573, &UA2951979},
	{SG52576, &UA2859218},
	{SG52577, &UA2984894},
	{SG52578, &UA3006054},
	{SG52579, &UA3022144},
	{SG52581, &UA3023907},
	{SG52583, &UA3110522},
	{SG52584, &UA3066899},
	{SG52585, &UA3073345},
	{SG52586, &UA3080393},
	{SG52587, &UA3088819},
	{SG52588, &UA3103395},
	{SG52589, &UA3110572},
	{SG52601, &UA2241862},
	{SG52604, &UA2402208},
	{SG52609, &UA2562759},
	{SG52611, &UA2600686},
	{SG52614, &UA2714574},
	{SG52618, &UA2797638},
	{SG52620, &UA2798450},
	{SG52621, &UA3001166},
	{SG52624, &UA2937609},
	{SG52625, &UA2958295},
	{SG52628, &UA2993512},
	{SG52629, &UA3007668},
	{SG52630, &UA3020723},
	{SG52631, &UA3083240},
	{SG52632, &UA3077321},
	{SG52633, &UA3121345},
	{SG52665, &UA2459377},
	{SG52669, &UA2507559},
	{SG52695, &UA2600478},
	{SG52696, &UA2833721},
	{SG52698, &UA2706022},
	{SG52699, &UA2708512},
	{SG52700, &UA2937484},
	{SG52703, &UA2799230},
	{SG52704, &UA2864649},
	{SG52707, &UA2955579},
	{SG52708, &UA2959673},
	{SG52709, &UA3047558},
	{SG52710, &UA3049883},
	{SG52712, &UA3086469},
	{SG52730, &UA3108566},
	{SG52731, &UA3130744},
	{SG52741, &UA2680972},
	{SG52743, &UA2706666},
	{SG52750, &UA2866851},
	{SG52751, &UA2869497},
	{SG52753, &UA2893671},
	{SG52754, &UA2936924},
	{SG52756, &UA2944650},
	{SG52758, &UA2958369},
	{SG52760, &UA2963516},
	{SG52761, &UA2993462},
	{SG52764, &UA3022381},
	{SG52767, &UA3050323},
	{SG52770, &UA3079405},
	{SG52773, &UA3111227},
	{SG52774, &UA3127227},
	{SG52776, &UA3127342},
	{SG52777, &UA3131416},
	{SG52787, &UA2384836},
	{SG52792, &UA2527323},
	{SG52793, &UA2554670},
	{SG52797, &UA2712462},
	{SG52803, &UA2797635},
	{SG52805, &UA2865730},
	{SG52806, &UA2865854},
	{SG52807, &UA2870897},
	{SG52808, &UA2879330},
	{SG52809, &UA2952964},
	{SG52810, &UA2955960},
	{SG52812, &UA2958078},
	{SG52813, &UA2958229},
	{SG52814, &UA2961819},
	{SG52816, &UA2999534},
	{SG52819, &UA3021369},
	{SG52821, &UA3022211},
	{SG52822, &UA3022339},
	{SG52823, &UA3024023},
	{SG52824, &UA3057308},
	{SG52825, &UA3059708},
	{SG52826, &UA3074252},
	{SG52827, &UA3088232},
	{SG52828, &UA3094794},
	{SG52829, &UA3102484},
	{SG52830, &UA3103963},
	{SG52832, &UA3131431},
	{SG52843, &UA2601241},
	{SG52845, &UA2669549},
	{SG52846, &UA2729938},
	{SG52847, &UA2734491},
	{SG52849, &UA2848919},
	{SG52851, &UA2755104},
	{SG52853, &UA2782253},
	{SG52854, &UA2873523},
	{SG52856, &UA2936682},
	{SG52857, &UA3022956},
	{SG52858, &UA3024021},
	{SG52860, &UA3057598},
	{SG52861, &UA3059376},
	{SG52862, &UA3073854},
	{SG52865, &UA3093426},
	{SG52867, &UA3094158},
	{SG52868, &UA3094268},
	{SG52870, &UA3110976},
	{SG52873, &UA3132854},
	{SG52888, &UA2982417},
	{SG52921, &UA2703255},
	{SG52923, &UA2708958},
	{SG52926, &UA2764355},
	{SG52929, &UA2797637},
	{SG52932, &UA2869204},
	{SG52934, &UA2877853},
	{SG52938, &UA3022840},
	{SG52941, &UA3058087},
	{SG52945, &UA3078543},
	{SG52979, &UA2708261},
	{SG52980, &UA2708719},
	{SG52981, &UA2726950},
	{SG52982, &UA2733034},
	{SG52983, &UA2734634},
	{SG52984, &UA2764417},
	{SG52985, &UA2794363},
	{SG52986, &UA2863099},
	{SG52987, &UA2879308},
	{SG52989, &UA2957320},
	{SG52990, &UA2987238},
	{SG52992, &UA2993639},
	{SG52993, &UA2998023},
	{SG52995, &UA3004473},
	{SG52996, &UA3004781},
	{SG52999, &UA3021035},
	{SG53002, &UA3072857},
	{SG53005, &UA3108433},
	{SG53006, &UA3110396},
	{SG53007, &UA3115433},
	{SG53017, &UA2666914},
	{SG53020, &UA2667047},
	{SG53021, &UA2667719},
	{SG53023, &UA2706287},
	{SG53026, &UA2753180},
	{SG53030, &UA2799051},
	{SG53034, &UA2868897},
	{SG53035, &UA2869261},
	{SG53036, &UA2870161},
	{SG53038, &UA2877829},
	{SG53040, &UA2879400},
	{SG53047, &UA2958448},
	{SG53049, &UA2993626},
	{SG53051, &UA2994865},
	{SG53052, &UA3001648},
	{SG53053, &UA3010718},
	{SG53054, &UA3015665},
	{SG53058, &UA3059295},
	{SG53059, &UA3092038},
	{SG53061, &UA3093368},
	{SG53063, &UA3109771},
	{SG53066, &UA3132618},
	{SG53083, &UA2714076},
	{SG53085, &UA2733009},
	{SG53087, &UA2738219},
	{SG53088, &UA2779029},
	{SG53090, &UA2879364},
	{SG53091, &UA2893953},
	{SG53092, &UA2934182},
	{SG53093, &UA2943401},
	{SG53094, &UA2958091},
	{SG53095, &UA3012415},
	{SG53098, &UA3023899},
	{SG53099, &UA3049957},
	{SG53100, &UA3053584},
	{SG53101, &UA3055856},
	{SG53102, &UA3059105},
	{SG53126, &UA2634787},
	{SG53130, &UA2680965},
	{SG53132, &UA2707138},
	{SG53136, &UA2767833},
	{SG53144, &UA2935046},
	{SG53146, &UA2942291},
	{SG53152, &UA3014135},
	{SG53154, &UA3021652},
	{SG53158, &UA3048782},
	{SG53159, &UA3057075},
	{SG53161, &UA3110905},
	{SG53164, &UA3131812},
	{SG53173, &UA2527215},
	{SG53175, &UA2667441},
	{SG53176, &UA2669405},
	{SG53182, &UA2714832},
	{SG53183, &UA2861844},
	{SG53185, &UA2877787},
	{SG53187, &UA2879295},
	{SG53189, &UA2955218},
	{SG53190, &UA2957169},
	{SG53191, &UA2957427},
	{SG53192, &UA2960002},
	{SG53193, &UA2964722},
	{SG53195, &UA2998259},
	{SG53196, &UA2999543},
	{SG53205, &UA3014488},
	{SG53206, &UA3015554},
	{SG53207, &UA3020630},
	{SG53208, &UA3022021},
	{SG53210, &UA3022156},
	{SG53211, &UA3023001},
	{SG53212, &UA3064013},
	{SG53214, &UA3115608},
	{SG53215, &UA3130742},
	{SG53225, &UA2530337},
	{SG53228, &UA2597944},
	{SG53229, &UA2657968},
	{SG53236, &UA2864734},
	{SG53237, &UA2868038},
	{SG53239, &UA2872362},
	{SG53240, &UA2877811},
	{SG53242, &UA2879294},
	{SG53245, &UA2893011},
	{SG53248, &UA2957727},
	{SG53251, &UA2958636},
	{SG53253, &UA2962728},
	{SG53256, &UA2998153},
	{SG53261, &UA3019897},
	{SG53263, &UA3021805},
	{SG53265, &UA3023041},
	{SG53267, &UA3078260},
	{SG53270, &UA3115611},
	{SG53271, &UA3109303},
	{SG53273, &UA3109554},
	{SG53275, &UA3109647},
	{SG53325, &UA2528849},
	{SG53326, &UA2583007},
	{SG53332, &UA2657167},
	{SG53336, &UA2668616},
	{SG53339, &UA2700737},
	{SG53343, &UA2730570},
	{SG53346, &UA2734924},
	{SG53347, &UA2735058},
	{SG53348, &UA2763475},
	{SG53351, &UA2860663},
	{SG53353, &UA2877859},
	{SG53355, &UA2879395},
	{SG53356, &UA2879442},
	{SG53361, &UA2957773},
	{SG53362, &UA2962569},
	{SG53365, &UA2992902},
	{SG53367, &UA3004367},
	{SG53370, &UA3021773},
	{SG53373, &UA3022331},
	{SG53375, &UA3049910},
	{SG53376, &UA3059203},
	{SG53377, &UA3078588},
	{SG53378, &UA3089112},
	{SG53379, &UA3115006},
	{SG53380, &UA3115049},
	{SG53381, &UA3131391},
	{SG53387, &UA2455583},
	{SG53389, &UA2734451},
	{SG53390, &UA2734558},
	{SG53400, &UA2754390},
	{SG53402, &UA2766764},
	{SG53403, &UA2777848},
	{SG53404, &UA2782022},
	{SG53406, &UA2794016},
	{SG53408, &UA2860734},
	{SG53409, &UA2861938},
	{SG53410, &UA2868420},
	{SG53411, &UA2879316},
	{SG53412, &UA2879387},
	{SG53413, &UA2889470},
	{SG53415, &UA2897409},
	{SG53416, &UA2899257},
	{SG53428, &UA2937863},
	{SG53430, &UA2956804},
	{SG53432, &UA2961416},
	{SG53433, &UA2986800},
	{SG53434, &UA2988796},
	{SG53435, &UA3020922},
	{SG53440, &UA3058083},
	{SG53442, &UA3132962},
	{SG53444, &UA3078531},
	{SG53445, &UA3109842},
	{SG53447, &UA3111093},
	{SG53451, &UA2480660},
	{SG53453, &UA2558096},
	{SG53455, &UA2704006},
	{SG53456, &UA2737824},
	{SG53457, &UA2739954},
	{SG53458, &UA2751764},
	{SG53459, &UA2752067},
	{SG53460, &UA2767779},
	{SG53465, &UA2868351},
	{SG53466, &UA2870309},
	{SG53467, &UA2879329},
	{SG53468, &UA2892376},
	{SG53469, &UA2892615},
	{SG53470, &UA3000575},
	{SG53472, &UA3002232},
	{SG53473, &UA3004083},
	{SG53474, &UA3005449},
	{SG53475, &UA3021423},
	{SG53476, &UA3022590},
	{SG53477, &UA3022727},
	{SG53479, &UA3079120},
	{SG53480, &UA3092891},
	{SG53482, &UA3109302},
	{SG53487, &UA3127184},
	{SG53488, &UA3130989},
	{SG53489, &UA3131316},
	{SG53490, &UA3131318},
	{SG53508, &UA2422296},
	{SG53509, &UA2667241},
	{SG53510, &UA2667302},
	{SG53511, &UA2668328},
	{SG53513, &UA2706895},
	{SG53514, &UA2712066},
	{SG53515, &UA2734636},
	{SG53516, &UA2867146},
	{SG53517, &UA2892757},
	{SG53518, &UA2892987},
	{SG53520, &UA2934490},
	{SG53521, &UA2938361},
	{SG53522, &UA2951248},
	{SG53524, &UA2955337},
	{SG53525, &UA2957050},
	{SG53526, &UA2957079},
	{SG53527, &UA2960630},
	{SG53529, &UA2988153},
	{SG53530, &UA2989278},
	{SG53531, &UA2995926},
	{SG53532, &UA2997673},
	{SG53550, &UA3012645},
	{SG53552, &UA3013550},
	{SG53554, &UA3017872},
	{SG53555, &UA3022802},
	{SG53557, &UA3030267},
	{SG53559, &UA3073898},
	{SG53560, &UA3108617},
	{SG53561, &UA3110903},
	{SG53563, &UA3130961},
	{SG53590, &UA2726974},
	{SG53593, &UA2484218},
	{SG53594, &UA2637538},
	{SG53595, &UA2652585},
	{SG53596, &UA2702830},
	{SG53597, &UA2713519},
	{SG53598, &UA2729286},
	{SG53602, &UA2767583},
	{SG53604, &UA2879567},
	{SG53605, &UA2893013},
	{SG53608, &UA2997830},
	{SG53610, &UA2998166},
	{SG53611, &UA2999356},
	{SG53612, &UA3020794},
	{SG53614, &UA3023462},
	{SG53615, &UA3029309},
	{SG53616, &UA3079579},
	{SG53617, &UA3090144},
	{SG53620, &UA3103369},
	{SG53623, &UA3132959},
	{SG53631, &UA2384850},
	{SG53634, &UA3108561},
	{SG53637, &UA2708423},
	{SG53639, &UA2734823},
	{SG53640, &UA2735281},
	{SG53644, &UA2794538},
	{SG53645, &UA2841012},
	{SG53647, &UA2860025},
	{SG53649, &UA2879360},
	{SG53650, &UA2892371},
	{SG53651, &UA2892596},
	{SG53652, &UA2893177},
	{SG53653, &UA2988386},
	{SG53654, &UA2998751},
	{SG53655, &UA2999083},
	{SG53656, &UA2999878},
	{SG53657, &UA3004475},
	{SG53658, &UA3011862},
	{SG53659, &UA3018862},
	{SG53660, &UA3024027},
	{SG53661, &UA3028180},
	{SG53662, &UA3048473},
	{SG53663, &UA3049313},
	{SG53664, &UA3058182},
	{SG53665, &UA3058674},
	{SG53666, &UA3078708},
	{SG53667, &UA3086010},
	{SG53668, &UA3092344},
	{SG53669, &UA3093277},
	{SG53670, &UA3108513},
	{SG53671, &UA3128013},
	{SG53672, &UA3128092},
	{SG53673, &UA3132906},
	{SG53674, &UA3132922},
	{SG53675, &UA3132933},
	{SG53681, &UA2466762},
	{SG53682, &UA2554549},
	{SG53686, &UA2747056},
	{SG53688, &UA2752353},
	{SG53707, &UA2791920},
	{SG53708, &UA2845737},
	{SG53710, &UA2863131},
	{SG53713, &UA2892571},
	{SG53715, &UA2892795},
	{SG53717, &UA2892875},
	{SG53719, &UA2897843},
	{SG53722, &UA2943865},
	{SG53723, &UA2953484},
	{SG53724, &UA2965242},
	{SG53725, &UA2998281},
	{SG53726, &UA3012657},
	{SG53727, &UA3020632},
	{SG53729, &UA3023988},
	{SG53734, &UA3078939},
	{SG53736, &UA3079582},
	{SG53738, &UA3088171},
	{SG53739, &UA3092278},
	{SG53740, &UA3092892},
	{SG53741, &UA3108554},
	{SG53743, &UA3109938},
	{SG53747, &UA3111185},
	{SG53754, &UA3127213},
	{SG53758, &UA3131618},
	{SG53760, &UA3131806},
	{SG53763, &UA3132886},
	{SG53852, &UA2709345},
	{SG53854, &UA2733273},
	{SG53855, &UA2734730},
	{SG53856, &UA2735379},
	{SG53857, &UA2741788},
	{SG53858, &UA2847121},
	{SG53859, &UA2849209},
	{SG53860, &UA2851517},
	{SG53861, &UA2851982},
	{SG53866, &UA2937371},
	{SG53868, &UA2953043},
	{SG53872, &UA2982900},
	{SG53873, &UA2997033},
	{SG53874, &UA2999044},
	{SG53875, &UA2999073},
	{SG53877, &UA3000157},
	{SG53878, &UA3009308},
	{SG53882, &UA3022152},
	{SG53883, &UA3023376},
	{SG53884, &UA3023521},
	{SG53885, &UA3027002},
	{SG53887, &UA3029857},
	{SG53889, &UA3048891},
	{SG53890, &UA3056962},
	{SG53891, &UA3062363},
	{SG53892, &UA3074043},
	{SG53893, &UA3078347},
	{SG53895, &UA3080949},
	{SG53896, &UA3127402},
	{SG53897, &UA3091993},
	{SG53898, &UA3092829},
	{SG53899, &UA3104699},
	{SG53900, &UA3108323},
	{SG53901, &UA3108889},
	{SG53902, &UA3110968},
	{SG53903, &UA3111181},
	{SG53904, &UA3113025},
	{SG53905, &UA3127194},
	{SG53906, &UA3131397},
	{SG53908, &UA3132638},
	{SG53910, &UA3132756},
	{SG53924, &UA2241768},
	{SG53962, &UA2695394},
	{SG53971, &UA2710433},
	{SG53974, &UA2729678},
	{SG53977, &UA2732925},
	{SG53985, &UA2735903},
	{SG53991, &UA2750826},
	{SG53999, &UA2767479},
	{SG54008, &UA2793067},
	{SG54017, &UA2838088},
	{SG54020, &UA2866582},
	{SG54021, &UA2868628},
	{SG54024, &UA2950874},
	{SG54027, &UA2957813},
	{SG54030, &UA2959377},
	{SG54032, &UA2959862},
	{SG54047, &UA2993916},
	{SG54057, &UA2998127},
	{SG54058, &UA3011959},
	{SG54059, &UA3014824},
	{SG54060, &UA3018993},
	{SG54064, &UA3056740},
	{SG54067, &UA3061016},
	{SG54068, &UA3076550},
	{SG54069, &UA3077941},
	{SG54070, &UA3078841},
	{SG54071, &UA3078885},
	{SG54072, &UA3078945},
	{SG54073, &UA3092688},
	{SG54074, &UA3092786},
	{SG54075, &UA3093306},
	{SG54076, &UA3105016},
	{SG54097, &UA3121438},
	{SG54105, &UA3126524},
	{SG54112, &UA3127357},
	{SG54116, &UA3127444},
	{SG54119, &UA3128113},
	{SG54122, &UA3128145},
	{SG54128, &UA3131394},
	{SG54134, &UA3132978},
	{SG54208, &UA2634716},
	{SG54209, &UA2707554},
	{SG54210, &UA2733294},
	{SG54211, &UA2741787},
	{SG54214, &UA2799383},
	{SG54215, &UA2841058},
	{SG54217, &UA2893045},
	{SG54218, &UA2937465},
	{SG54219, &UA2937697},
	{SG54221, &UA2938190},
	{SG54222, &UA2958902},
	{SG54223, &UA2984332},
	{SG54224, &UA2986838},
	{SG54225, &UA2991869},
	{SG54226, &UA2993466},
	{SG54227, &UA2994775},
	{SG54228, &UA2996513},
	{SG54229, &UA2997862},
	{SG54230, &UA2997996},
	{SG54231, &UA2998446},
	{SG54232, &UA3000480},
	{SG54233, &UA3007224},
	{SG54234, &UA3013602},
	{SG54240, &UA3049751},
	{SG54241, &UA3051967},
	{SG54242, &UA3053677},
	{SG54245, &UA3056870},
	{SG54249, &UA3092253},
	{SG54250, &UA3092531},
	{SG54251, &UA3092773},
	{SG54252, &UA3093303},
	{SG54254, &UA3108894},
	{SG54255, &UA3110932},
	{SG54256, &UA3125136},
	{SG54257, &UA3125334},
	{SG54258, &UA3126003},
	{SG54259, &UA3127131},
	{SG54260, &UA3127492},
	{SG54261, &UA3127632},
	{SG54262, &UA3130563},
	{SG54263, &UA3132431},
	{SG54264, &UA3132964},
	{SG54268, &UA2578478},
	{SG54269, &UA2578823},
	{SG54270, &UA2579654},
	{SG54271, &UA2698654},
	{SG54272, &UA2719662},
	{SG54273, &UA2719678},
	{SG54275, &UA2734995},
	{SG54276, &UA2735175},
	{SG54277, &UA2795593},
	{SG54278, &UA2840754},
	{SG54279, &UA2847271},
	{SG54280, &UA2857866},
	{SG54281, &UA2861173},
	{SG54282, &UA2861671},
	{SG54285, &UA2869983},
	{SG54287, &UA2893064},
	{SG54289, &UA2894720},
	{SG54290, &UA2897089},
	{SG54291, &UA2936789},
	{SG54293, &UA2937710},
	{SG54294, &UA2937750},
	{SG54295, &UA2937801},
	{SG54296, &UA2950981},
	{SG54297, &UA2957737},
	{SG54299, &UA2958342},
	{SG54300, &UA2960358},
	{SG54302, &UA2993193},
	{SG54303, &UA2993415},
	{SG54304, &UA2998785},
	{SG54305, &UA2999405},
	{SG54306, &UA2999645},
	{SG54307, &UA3006106},
	{SG54308, &UA3007714},
	{SG54309, &UA3010801},
	{SG54310, &UA3011234},
	{SG54312, &UA3022280},
	{SG54313, &UA3047744},
	{SG54320, &UA3058698},
	{SG54321, &UA3060876},
	{SG54322, &UA3070586},
	{SG54323, &UA3074245},
	{SG54324, &UA3074356},
	{SG54325, &UA3074407},
	{SG54326, &UA3085749},
	{SG54328, &UA3086659},
	{SG54329, &UA3091804},
	{SG54330, &UA3091871},
	{SG54331, &UA3092453},
	{SG54332, &UA3092518},
	{SG54333, &UA3108164},
	{SG54334, &UA3108308},
	{SG54335, &UA3108806},
	{SG54336, &UA3109253},
	{SG54337, &UA3110546},
	{SG54338, &UA3113525},
	{SG54339, &UA3126136},
	{SG54340, &UA3127112},
	{SG54341, &UA3127341},
	{SG54342, &UA3127391},
	{SG54343, &UA3127425},
	{SG54344, &UA3130588},
	{SG54345, &UA3131401},
	{SG54347, &UA3131583},
	{SG54353, &UA2577878},
	{SG54354, &UA2667971},
	{SG54358, &UA2735373},
	{SG54360, &UA2780585},
	{SG54362, &UA2795924},
	{SG54368, &UA2840292},
	{SG54371, &UA2844587},
	{SG54372, &UA2861066},
	{SG54373, &UA2861806},
	{SG54374, &UA2892259},
	{SG54375, &UA2892869},
	{SG54376, &UA2893387},
	{SG54377, &UA2937396},
	{SG54378, &UA2952783},
	{SG54379, &UA2954226},
	{SG54380, &UA2958713},
	{SG54382, &UA2985975},
	{SG54385, &UA3043031},
	{SG54388, &UA3060547},
	{SG54395, &UA3075646},
	{SG54396, &UA3077536},
	{SG54397, &UA3086378},
	{SG54398, &UA3086465},
	{SG54399, &UA3092164},
	{SG54400, &UA3100912},
	{SG54402, &UA3101707},
	{SG54403, &UA3103536},
	{SG54406, &UA3105201},
	{SG54407, &UA3106022},
	{SG54408, &UA3107079},
	{SG54409, &UA3107920},
	{SG54410, &UA3119090},
	{SG54411, &UA3125288},
	{SG54412, &UA3125492},
	{SG54416, &UA3126354},
	{SG54418, &UA3127063},
	{SG54419, &UA3127085},
	{SG54420, &UA3127159},
	{SG54421, &UA3127249},
	{SG54422, &UA3127318},
	{SG54423, &UA3130954},
	{SG54424, &UA3132856},
	{SG54509, &UA2633634},
	{SG54510, &UA2635010},
	{SG54512, &UA2702315},
	{SG54513, &UA2734757},
	{SG54514, &UA2734895},
	{SG54515, &UA2765811},
	{SG54516, &UA2780876},
	{SG54518, &UA2780931},
	{SG54520, &UA2860545},
	{SG54521, &UA2864200},
	{SG54522, &UA2864705},
	{SG54523, &UA2933936},
	{SG54524, &UA2936754},
	{SG54525, &UA2937921},
	{SG54526, &UA2958073},
	{SG54527, &UA2958235},
	{SG54528, &UA2989848},
	{SG54529, &UA2996826},
	{SG54530, &UA2999741},
	{SG54531, &UA3001604},
	{SG54533, &UA3023047},
	{SG54537, &UA3071051},
	{SG54539, &UA3073227},
	{SG54541, &UA3073997},
	{SG54545, &UA3088393},
	{SG54547, &UA3091570},
	{SG54548, &UA3100690},
	{SG54550, &UA3103620},
	{SG54551, &UA3107917},
	{SG54552, &UA3108347},
	{SG54553, &UA3110977},
	{SG54554, &UA3118774},
	{SG54555, &UA3123240},
	{SG54556, &UA3124998},
	{SG54557, &UA3125655},
	{SG54561, &UA3126438},
	{SG54571, &UA3127011},
	{SG54573, &UA3127130},
	{SG54575, &UA3127322},
	{SG54581, &UA3127691},
	{SG54588, &UA3132977},
	{SG54614, &UA2840076},
	{SG54616, &UA2843982},
	{SG54618, &UA2858791},
	{SG54619, &UA2858797},
	{SG54620, &UA2860845},
	{SG54621, &UA2862137},
	{SG54623, &UA2863137},
	{SG54626, &UA2865514},
	{SG54627, &UA2866278},
	{SG54631, &UA2892749},
	{SG54632, &UA2937575},
	{SG54633, &UA2937652},
	{SG54635, &UA2957906},
	{SG54638, &UA2988477},
	{SG54639, &UA2993540},
	{SG54642, &UA2997463},
	{SG54643, &UA2998013},
	{SG54646, &UA3002715},
	{SG54647, &UA3011786},
	{SG54648, &UA3013579},
	{SG54690, &UA3043510},
	{SG54694, &UA3057809},
	{SG54696, &UA3058929},
	{SG54700, &UA3059113},
	{SG54719, &UA3087962},
	{SG54725, &UA3091784},
	{SG54732, &UA3092577},
	{SG54738, &UA3100578},
	{SG54739, &UA3100934},
	{SG54740, &UA3103103},
	{SG54741, &UA3105270},
	{SG54742, &UA3106179},
	{SG54743, &UA3106831},
	{SG54755, &UA3107507},
	{SG54756, &UA3107779},
	{SG54757, &UA3108008},
	{SG54758, &UA3108246},
	{SG54759, &UA3108915},
	{SG54760, &UA3111270},
	{SG54761, &UA3118041},
	{SG54764, &UA3121140},
	{SG54765, &UA3121384},
	{SG54766, &UA3124571},
	{SG54767, &UA3124886},
	{SG54768, &UA3124927},
	{SG54770, &UA3125594},
	{SG54771, &UA3125614},
	{SG54798, &UA3126648},
	{SG54804, &UA3127057},
	{SG54809, &UA3127141},
	{SG54815, &UA3127273},
	{SG54822, &UA3132909},
	{SG54825, &UA2839045},
	{SG54828, &UA2860578},
	{SG54831, &UA2862045},
	{SG54832, &UA2863093},
	{SG54873, &UA2586638},
	{SG54875, &UA2629071},
	{SG54876, &UA2832599},
	{SG54877, &UA2866082},
	{SG54893, &UA2680986},
	{SG54897, &UA2680998},
	{SG54907, &UA2241810},
	{SG54916, &UA2586629},
	{SG54918, &UA2586656},
	{SG54919, &UA2719703},
	{SG54923, &UA2637155},
	{SG54924, &UA2634138},
	{SG54932, &UA2620192},
	{SG54933, &UA2631692},
	{SG54934, &UA2633809},
	{SG54935, &UA2680960},
	{SG54967, &UA2621590},
	{SG54969, &UA2629901},
	{SG54994, &UA2680962},
	{SG54996, &UA2680991},
	{SG55010, &UA2607538},
	{SG55013, &UA2637912},
	{SG55014, &UA2620544},
	{SG55015, &UA2624366},
	{SG55016, &UA2624955},
	{SG55017, &UA2632105},
	{SG55033, &UA2681000},
	{SG55034, &UA2681014},
	{SG55035, &UA2801274},
	{SG55047, &UA2241896},
	{SG55053, &UA2634266},
	{SG55055, &UA2627064},
	{SG55057, &UA2635665},
	{SG55059, &UA2801270},
	{SG55115, &UA2586583},
	{SG55116, &UA2586655},
	{SG55118, &UA2623285},
	{SG55119, &UA2623381},
	{SG55120, &UA2629001},
	{SG55121, &UA2623891},
	{SG55123, &UA2636307},
	{SG55124, &UA2635802},
	{SG55125, &UA2632874},
	{SG55128, &UA2681005},
	{SG55129, &UA2719716},
	{SG55130, &UA2741789},
	{SG55133, &UA3068918},
	{SG55136, &UA3084065},
	{SG55155, &UA2607540},
	{SG55159, &UA2616283},
	{SG55162, &UA2621510},
	{SG55168, &UA2630606},
	{SG55173, &UA2633227},
	{SG55190, &UA2719734},
	{SG55212, &UA3068929},
	{SG55214, &UA3084076},
	{SG55215, &UA3116553},
	{SG55291, &UA2242115},
	{SG55293, &UA2242157},
	{SG55294, &UA2530760},
	{SG55296, &UA2540485},
	{SG55297, &UA2540498},
	{SG55300, &UA2607539},
	{SG55302, &UA2616087},
	{SG55306, &UA2619397},
	{SG55308, &UA2620992},
	{SG55312, &UA2623035},
	{SG55314, &UA2624450},
	{SG55316, &UA2625468},
	{SG55317, &UA2628505},
	{SG55318, &UA2630021},
	{SG55319, &UA2631029},
	{SG55320, &UA2633080},
	{SG55324, &UA2634275},
	{SG55326, &UA2635471},
	{SG55328, &UA2636478},
	{SG55329, &UA2637729},
	{SG55330, &UA2680950},
	{SG55331, &UA2681016},
	{SG55332, &UA2719711},
	{SG55333, &UA2719719},
	{SG55334, &UA2784671},
	{SG55335, &UA2845324},
	{SG55336, &UA3068916},
	{SG55337, &UA3084049},
	{SG55338, &UA3084089},
	{SG55339, &UA3084104},
	{SG55340, &UA3097333},
	{SG55341, &UA3130693},
	{NULL}};
const Segmentdata DAndroidVersion4_2_2[] = {
	{SG31584, &UA2667916},
	{SG31585, &UA2670094},
	{SG31586, &UA2709043},
	{SG31587, &UA2710359},
	{SG31588, &UA2715756},
	{SG31589, &UA2716550},
	{SG31590, &UA2717847},
	{SG31591, &UA2737408},
	{SG31592, &UA2740648},
	{SG31593, &UA2885332},
	{SG31594, &UA2898307},
	{SG31595, &UA2998140},
	{SG31596, &UA3035056},
	{SG31597, &UA3036397},
	{SG31598, &UA3067929},
	{SG31599, &UA3081146},
	{SG31600, &UA3096455},
	{SG31601, &UA3114768},
	{SG31602, &UA3114769},
	{SG31979, &UA2470271},
	{SG31980, &UA2492651},
	{SG31981, &UA2512909},
	{SG31982, &UA2514558},
	{SG31983, &UA2535466},
	{SG31984, &UA2563823},
	{SG31985, &UA2663007},
	{SG31986, &UA2705768},
	{SG31999, &UA2862021},
	{SG32000, &UA2966990},
	{SG32781, &UA2512911},
	{SG33457, &UA2515541},
	{SG33458, &UA2516750},
	{SG33459, &UA2563304},
	{SG33460, &UA2584343},
	{SG33461, &UA2704049},
	{SG33462, &UA2768678},
	{SG33463, &UA3087050},
	{SG33573, &UA2766203},
	{SG33576, &UA2784912},
	{SG33579, &UA2799388},
	{SG33582, &UA2870618},
	{SG33584, &UA2879421},
	{SG33587, &UA2895720},
	{SG33589, &UA2937554},
	{SG33628, &UA2957874},
	{SG33629, &UA2957931},
	{SG33630, &UA2958372},
	{SG33631, &UA2972874},
	{SG33632, &UA2974943},
	{SG33633, &UA2999622},
	{SG33634, &UA3001961},
	{SG33635, &UA3013435},
	{SG33636, &UA3022090},
	{SG33637, &UA3022153},
	{SG33638, &UA3022228},
	{SG33639, &UA3034169},
	{SG33640, &UA3036909},
	{SG33641, &UA3050187},
	{SG33642, &UA3090035},
	{SG33643, &UA3094136},
	{SG33644, &UA3124892},
	{SG33954, &UA2856790},
	{SG34047, &UA2669054},
	{SG34048, &UA2766307},
	{SG34870, &UA2701389},
	{SG34892, &UA2750577},
	{SG34894, &UA2751290},
	{SG34898, &UA2780182},
	{SG34900, &UA2791557},
	{SG34903, &UA2858349},
	{SG34904, &UA2859250},
	{SG34906, &UA2868030},
	{SG34908, &UA2878976},
	{SG34909, &UA2937158},
	{SG34911, &UA2938596},
	{SG34912, &UA2944711},
	{SG34915, &UA2958190},
	{SG34916, &UA2958482},
	{SG34917, &UA2960972},
	{SG34919, &UA2962938},
	{SG34920, &UA2969464},
	{SG34922, &UA2989416},
	{SG34924, &UA2993693},
	{SG34927, &UA2996978},
	{SG34928, &UA2998729},
	{SG34930, &UA2999051},
	{SG34932, &UA2999159},
	{SG34934, &UA3000152},
	{SG34936, &UA3000795},
	{SG34938, &UA3003186},
	{SG34940, &UA3011301},
	{SG34942, &UA3012155},
	{SG34943, &UA3013023},
	{SG34945, &UA3015181},
	{SG34946, &UA3015781},
	{SG34948, &UA3015862},
	{SG34950, &UA3016258},
	{SG34952, &UA3017720},
	{SG34953, &UA3020941},
	{SG34954, &UA3022500},
	{SG34955, &UA3022999},
	{SG34956, &UA3024945},
	{SG34958, &UA3030149},
	{SG34960, &UA3031118},
	{SG34961, &UA3046391},
	{SG34962, &UA3049338},
	{SG34963, &UA3053315},
	{SG34964, &UA3056882},
	{SG34965, &UA3057174},
	{SG34966, &UA3057235},
	{SG34968, &UA3057291},
	{SG34969, &UA3057701},
	{SG34970, &UA3057769},
	{SG34971, &UA3058109},
	{SG34972, &UA3058502},
	{SG34973, &UA3058742},
	{SG34974, &UA3058862},
	{SG34975, &UA3059015},
	{SG34977, &UA3059454},
	{SG34979, &UA3059516},
	{SG34981, &UA3060798},
	{SG34982, &UA3061759},
	{SG34983, &UA3061982},
	{SG34985, &UA3074105},
	{SG34987, &UA3074183},
	{SG34990, &UA3074242},
	{SG34991, &UA3074307},
	{SG34993, &UA3075767},
	{SG34995, &UA3077765},
	{SG34998, &UA3078200},
	{SG35001, &UA3078385},
	{SG35003, &UA3078405},
	{SG35004, &UA3078517},
	{SG35005, &UA3078823},
	{SG35006, &UA3078826},
	{SG35009, &UA3078941},
	{SG35013, &UA3079117},
	{SG35017, &UA3079151},
	{SG35019, &UA3079306},
	{SG35021, &UA3079507},
	{SG35024, &UA3079779},
	{SG35026, &UA3079863},
	{SG35027, &UA3083239},
	{SG35028, &UA3083722},
	{SG35029, &UA3088183},
	{SG35030, &UA3089047},
	{SG35032, &UA3090816},
	{SG35033, &UA3091764},
	{SG35034, &UA3091856},
	{SG35035, &UA3092002},
	{SG35036, &UA3092182},
	{SG35038, &UA3092189},
	{SG35040, &UA3092265},
	{SG35041, &UA3092389},
	{SG35043, &UA3092400},
	{SG35045, &UA3092925},
	{SG35046, &UA3093462},
	{SG35047, &UA3093498},
	{SG35048, &UA3093608},
	{SG35049, &UA3094129},
	{SG35050, &UA3095943},
	{SG35051, &UA3102781},
	{SG35056, &UA3105227},
	{SG35059, &UA3106168},
	{SG35060, &UA3106308},
	{SG35061, &UA3107560},
	{SG35062, &UA3107755},
	{SG35063, &UA3107850},
	{SG35064, &UA3107837},
	{SG35065, &UA3108107},
	{SG35066, &UA3108244},
	{SG35069, &UA3108484},
	{SG35071, &UA3108526},
	{SG35072, &UA3108534},
	{SG35073, &UA3108539},
	{SG35074, &UA3108709},
	{SG35075, &UA3108913},
	{SG35076, &UA3108935},
	{SG35079, &UA3108997},
	{SG35081, &UA3109063},
	{SG35083, &UA3109264},
	{SG35084, &UA3110001},
	{SG35085, &UA3110217},
	{SG35086, &UA3110909},
	{SG35087, &UA3111686},
	{SG35088, &UA3112568},
	{SG35089, &UA3120716},
	{SG35090, &UA3121609},
	{SG35091, &UA3123979},
	{SG35092, &UA3124814},
	{SG35093, &UA3124916},
	{SG35094, &UA3125100},
	{SG35095, &UA3125273},
	{SG35096, &UA3125320},
	{SG35097, &UA3125333},
	{SG35100, &UA3125384},
	{SG35102, &UA3125441},
	{SG35105, &UA3125478},
	{SG35107, &UA3125485},
	{SG35108, &UA3125489},
	{SG35110, &UA3125541},
	{SG35112, &UA3125552},
	{SG35114, &UA3125650},
	{SG35115, &UA3125667},
	{SG35116, &UA3125877},
	{SG35117, &UA3125926},
	{SG35118, &UA3126051},
	{SG35119, &UA3126233},
	{SG35120, &UA3126294},
	{SG35121, &UA3126432},
	{SG35122, &UA3127064},
	{SG35125, &UA3129255},
	{SG35127, &UA2515525},
	{SG35128, &UA2533186},
	{SG35319, &UA2566438},
	{SG35438, &UA2534903},
	{SG35439, &UA2537048},
	{SG35441, &UA2537055},
	{SG35442, &UA2679675},
	{SG35444, &UA2705715},
	{SG35445, &UA2867466},
	{SG35446, &UA2897967},
	{SG35447, &UA2948412},
	{SG35449, &UA2965087},
	{SG35452, &UA2968143},
	{SG35534, &UA2515540},
	{SG35535, &UA2986551},
	{SG35536, &UA3008329},
	{SG35704, &UA2566120},
	{SG35705, &UA2567017},
	{SG35741, &UA2973946},
	{SG35744, &UA2706598},
	{SG35745, &UA2712285},
	{SG35746, &UA2783059},
	{SG35748, &UA2795200},
	{SG35749, &UA2938469},
	{SG35750, &UA3076974},
	{SG35751, &UA3089052},
	{SG35846, &UA2888666},
	{SG35847, &UA2975581},
	{SG35848, &UA2515537},
	{SG35850, &UA2566443},
	{SG35852, &UA2566645},
	{SG35940, &UA2537632},
	{SG35941, &UA2567435},
	{SG35942, &UA2678068},
	{SG35945, &UA2898414},
	{SG35946, &UA2998289},
	{SG35947, &UA3011184},
	{SG35948, &UA3016685},
	{SG35950, &UA3121181},
	{SG35962, &UA2509948},
	{SG35964, &UA2513989},
	{SG35966, &UA2516462},
	{SG35969, &UA2662841},
	{SG35971, &UA2707543},
	{SG35974, &UA2537618},
	{SG35975, &UA2537630},
	{SG35977, &UA2539853},
	{SG35979, &UA2561461},
	{SG35980, &UA2714182},
	{SG35982, &UA2717580},
	{SG35985, &UA2752428},
	{SG35987, &UA2768825},
	{SG35989, &UA2781519},
	{SG35991, &UA2892897},
	{SG36041, &UA2974021},
	{SG36042, &UA3021561},
	{SG36069, &UA2711437},
	{SG36070, &UA2712496},
	{SG36071, &UA2769748},
	{SG36073, &UA2771208},
	{SG36075, &UA2892476},
	{SG36078, &UA2937230},
	{SG36080, &UA2960692},
	{SG36081, &UA2998046},
	{SG36082, &UA2999258},
	{SG36083, &UA3008718},
	{SG36085, &UA3013803},
	{SG36086, &UA3055169},
	{SG36087, &UA3092454},
	{SG36495, &UA2741699},
	{SG36498, &UA2873886},
	{SG36601, &UA2715376},
	{SG36603, &UA2741715},
	{SG36665, &UA2741703},
	{SG36666, &UA2769782},
	{SG36689, &UA2886409},
	{SG36690, &UA2895702},
	{SG36691, &UA3019761},
	{SG36692, &UA3026330},
	{SG36693, &UA3026489},
	{SG36694, &UA3061103},
	{SG36695, &UA3125248},
	{SG36696, &UA3126553},
	{SG36697, &UA3129231},
	{SG36741, &UA2676104},
	{SG36745, &UA2676750},
	{SG36762, &UA2669826},
	{SG36763, &UA2750373},
	{SG36764, &UA2869700},
	{SG36765, &UA2879525},
	{SG36767, &UA3050001},
	{SG36840, &UA2536598},
	{SG36843, &UA2536612},
	{SG36847, &UA2565784},
	{SG36851, &UA2600691},
	{SG36853, &UA2666490},
	{SG36855, &UA2708062},
	{SG36857, &UA2716496},
	{SG36859, &UA2733573},
	{SG36861, &UA2766673},
	{SG36863, &UA2947399},
	{SG36865, &UA3017134},
	{SG36868, &UA3017512},
	{SG36870, &UA3056428},
	{SG36955, &UA2676011},
	{SG36957, &UA2676293},
	{SG37003, &UA3064656},
	{SG37005, &UA3068411},
	{SG37008, &UA3126374},
	{SG37018, &UA3065464},
	{SG37074, &UA2675976},
	{SG37076, &UA2707174},
	{SG37082, &UA3048812},
	{SG37114, &UA2669602},
	{SG37115, &UA2874774},
	{SG37116, &UA2935762},
	{SG37117, &UA3074496},
	{SG37126, &UA2670980},
	{SG37127, &UA2966271},
	{SG37133, &UA2536595},
	{SG37134, &UA2971386},
	{SG37142, &UA2879013},
	{SG37145, &UA2945572},
	{SG37147, &UA3022539},
	{SG37149, &UA3057761},
	{SG37151, &UA3058691},
	{SG37153, &UA3059232},
	{SG37155, &UA3062034},
	{SG37157, &UA3074239},
	{SG37158, &UA3114897},
	{SG37206, &UA2537078},
	{SG37207, &UA2562925},
	{SG37209, &UA2665744},
	{SG37210, &UA2717074},
	{SG37212, &UA2768785},
	{SG37214, &UA2879217},
	{SG37215, &UA2956219},
	{SG37218, &UA3121474},
	{SG37301, &UA2534068},
	{SG37303, &UA2717932},
	{SG37305, &UA2973941},
	{SG37306, &UA2974795},
	{SG37400, &UA2939186},
	{SG37401, &UA2969665},
	{SG37403, &UA3014933},
	{SG37405, &UA3064642},
	{SG37407, &UA3065616},
	{SG37409, &UA3066225},
	{SG37411, &UA3080593},
	{SG37412, &UA3083375},
	{SG37413, &UA3109622},
	{SG37418, &UA2560457},
	{SG37420, &UA2560799},
	{SG37422, &UA2704246},
	{SG37424, &UA2731504},
	{SG37426, &UA2781897},
	{SG37428, &UA2956547},
	{SG37430, &UA3031048},
	{SG37432, &UA3060397},
	{SG37433, &UA3063153},
	{SG37434, &UA3064033},
	{SG37435, &UA3067497},
	{SG37436, &UA3114753},
	{SG37437, &UA3115740},
	{SG37470, &UA2706648},
	{SG37471, &UA2712918},
	{SG37472, &UA2738237},
	{SG37653, &UA3064999},
	{SG37696, &UA2973885},
	{SG37698, &UA2974152},
	{SG37700, &UA2974271},
	{SG37703, &UA3020931},
	{SG37722, &UA2712353},
	{SG37759, &UA2974040},
	{SG37760, &UA3026076},
	{SG37791, &UA3064984},
	{SG37793, &UA3080310},
	{SG37903, &UA2704812},
	{SG37904, &UA2736164},
	{SG37906, &UA2937684},
	{SG37908, &UA3101042},
	{SG37912, &UA2974027},
	{SG37971, &UA3064997},
	{SG37972, &UA3083366},
	{SG37982, &UA3065570},
	{SG37983, &UA3113445},
	{SG38052, &UA3055564},
	{SG38053, &UA3064038},
	{SG38069, &UA3063052},
	{SG38070, &UA3064868},
	{SG38071, &UA3065993},
	{SG38072, &UA3066001},
	{SG38078, &UA2515589},
	{SG38079, &UA2532086},
	{SG38081, &UA2769457},
	{SG38100, &UA2534070},
	{SG38101, &UA2948414},
	{SG38103, &UA3011784},
	{SG38104, &UA2973471},
	{SG38105, &UA2974142},
	{SG38112, &UA2703919},
	{SG38114, &UA2714463},
	{SG38116, &UA2783621},
	{SG38117, &UA2895249},
	{SG38119, &UA2956545},
	{SG38120, &UA3002796},
	{SG38174, &UA2703793},
	{SG38175, &UA2715000},
	{SG38177, &UA2706403},
	{SG38178, &UA2715710},
	{SG38199, &UA2741725},
	{SG38201, &UA2754404},
	{SG38202, &UA2755131},
	{SG38204, &UA2772262},
	{SG38207, &UA2893097},
	{SG38224, &UA2535478},
	{SG38244, &UA2603648},
	{SG38245, &UA2709215},
	{SG38246, &UA2711481},
	{SG38270, &UA2973272},
	{SG38271, &UA2973943},
	{SG38273, &UA2975030},
	{SG38274, &UA3023534},
	{SG38276, &UA3023854},
	{SG38278, &UA3024210},
	{SG38283, &UA2710842},
	{SG38284, &UA2753354},
	{SG38285, &UA2937312},
	{SG38286, &UA3006510},
	{SG38287, &UA3092213},
	{SG38296, &UA3064640},
	{SG38306, &UA2566454},
	{SG38312, &UA2937734},
	{SG38313, &UA2966315},
	{SG38315, &UA3020688},
	{SG38328, &UA3022201},
	{SG38329, &UA3023386},
	{SG38416, &UA2974065},
	{SG38418, &UA2974297},
	{SG38420, &UA2974811},
	{SG38421, &UA2974890},
	{SG38422, &UA2975029},
	{SG38423, &UA3022805},
	{SG38438, &UA3064871},
	{SG38440, &UA3064659},
	{SG38443, &UA2795390},
	{SG38444, &UA2879625},
	{SG38452, &UA2704292},
	{SG38453, &UA2716928},
	{SG38454, &UA2864646},
	{SG38455, &UA2942510},
	{SG38456, &UA2995714},
	{SG38459, &UA2999058},
	{SG38461, &UA3002568},
	{SG38463, &UA3046589},
	{SG38464, &UA3049485},
	{SG38466, &UA3049760},
	{SG38467, &UA3050203},
	{SG38469, &UA3057444},
	{SG38472, &UA3078752},
	{SG38776, &UA2536313},
	{SG38778, &UA2565200},
	{SG38780, &UA2568077},
	{SG38782, &UA2603724},
	{SG38783, &UA2662325},
	{SG38784, &UA2714114},
	{SG38785, &UA2716463},
	{SG38786, &UA2737985},
	{SG38787, &UA2768339},
	{SG38788, &UA2780126},
	{SG38789, &UA2860590},
	{SG38791, &UA2863161},
	{SG38792, &UA2956237},
	{SG38793, &UA2956249},
	{SG38795, &UA2888674},
	{SG38810, &UA2974143},
	{SG38817, &UA2703895},
	{SG38818, &UA2712908},
	{SG38819, &UA2957044},
	{SG38821, &UA3000120},
	{SG38864, &UA2974138},
	{SG38877, &UA2974141},
	{SG38881, &UA3020787},
	{SG38885, &UA3057614},
	{SG38908, &UA2670610},
	{SG38909, &UA2672304},
	{SG38910, &UA2676195},
	{SG38911, &UA2676234},
	{SG38913, &UA2676728},
	{SG38915, &UA2709128},
	{SG38928, &UA2713502},
	{SG38930, &UA2719300},
	{SG38932, &UA3048955},
	{SG38944, &UA2705398},
	{SG38966, &UA2719301},
	{SG39024, &UA2741709},
	{SG39114, &UA2795566},
	{SG39125, &UA2741717},
	{SG39126, &UA2774083},
	{SG39127, &UA2879340},
	{SG39132, &UA2754409},
	{SG39134, &UA2754421},
	{SG39135, &UA2754574},
	{SG39137, &UA2755521},
	{SG39140, &UA2773188},
	{SG39143, &UA2877904},
	{SG39210, &UA2583342},
	{SG39233, &UA2566633},
	{SG39234, &UA2566692},
	{SG39235, &UA2566943},
	{SG39236, &UA2583790},
	{SG39237, &UA2600029},
	{SG39239, &UA2667061},
	{SG39245, &UA2538619},
	{SG39246, &UA2582232},
	{SG39247, &UA2708065},
	{SG39249, &UA2781628},
	{SG39251, &UA2958061},
	{SG39263, &UA2631686},
	{SG39264, &UA2636905},
	{SG39265, &UA2794879},
	{SG39266, &UA2997176},
	{SG39329, &UA2708682},
	{SG39331, &UA2756885},
	{SG39341, &UA3065134},
	{SG39344, &UA3065383},
	{SG39347, &UA3065721},
	{SG39349, &UA3068332},
	{SG39351, &UA3083385},
	{SG39381, &UA2607518},
	{SG39383, &UA3113453},
	{SG39384, &UA3125983},
	{SG39398, &UA2669850},
	{SG39399, &UA2714033},
	{SG39414, &UA2769280},
	{SG39449, &UA2676076},
	{SG39517, &UA3065482},
	{SG39518, &UA3067899},
	{SG39520, &UA3083278},
	{SG39538, &UA2741739},
	{SG39540, &UA2784119},
	{SG39541, &UA2974157},
	{SG39542, &UA2974893},
	{SG39543, &UA2976031},
	{SG39551, &UA2566343},
	{SG39553, &UA2737437},
	{SG39555, &UA2737692},
	{SG39567, &UA3064647},
	{SG39568, &UA3083592},
	{SG39595, &UA2675401},
	{SG39598, &UA2675513},
	{SG39599, &UA2675850},
	{SG39600, &UA2676275},
	{SG39601, &UA2676448},
	{SG39602, &UA2676981},
	{SG39603, &UA2676996},
	{SG39604, &UA2719438},
	{SG39605, &UA2861222},
	{SG39610, &UA2882693},
	{SG39611, &UA3026687},
	{SG39643, &UA2673372},
	{SG39645, &UA2719297},
	{SG39656, &UA2676840},
	{SG39677, &UA2713045},
	{SG39679, &UA2715809},
	{SG39704, &UA3080082},
	{SG39706, &UA3080540},
	{SG39708, &UA3081122},
	{SG39713, &UA3083655},
	{SG39716, &UA3093680},
	{SG39718, &UA3093662},
	{SG39719, &UA3094519},
	{SG39721, &UA3094684},
	{SG39722, &UA3094727},
	{SG39723, &UA3094833},
	{SG39725, &UA3094849},
	{SG39727, &UA3094872},
	{SG39728, &UA3109616},
	{SG39730, &UA3110733},
	{SG39732, &UA3111520},
	{SG39734, &UA3111529},
	{SG39737, &UA3113179},
	{SG39740, &UA3113799},
	{SG39743, &UA3113805},
	{SG39746, &UA3114082},
	{SG39750, &UA3114922},
	{SG39751, &UA3115044},
	{SG39752, &UA3126459},
	{SG39753, &UA3126482},
	{SG39754, &UA3126560},
	{SG39755, &UA3126669},
	{SG39757, &UA3126735},
	{SG39758, &UA3126987},
	{SG39759, &UA3127115},
	{SG39761, &UA3127129},
	{SG39762, &UA3127270},
	{SG39763, &UA3127448},
	{SG39767, &UA2537952},
	{SG39768, &UA2731158},
	{SG39769, &UA2855316},
	{SG39776, &UA3064778},
	{SG39804, &UA2753705},
	{SG39807, &UA2708849},
	{SG39808, &UA2937019},
	{SG39810, &UA3121618},
	{SG39871, &UA2536297},
	{SG39873, &UA2565026},
	{SG39874, &UA2565476},
	{SG39875, &UA2711244},
	{SG39876, &UA2718799},
	{SG39877, &UA2733019},
	{SG39878, &UA2866111},
	{SG39879, &UA2897172},
	{SG39882, &UA2942805},
	{SG39883, &UA2942959},
	{SG39884, &UA2960685},
	{SG39885, &UA2748976},
	{SG40026, &UA3065100},
	{SG40028, &UA3081037},
	{SG40030, &UA3094609},
	{SG40041, &UA2532484},
	{SG40042, &UA2534059},
	{SG40043, &UA2537056},
	{SG40045, &UA3011585},
	{SG40050, &UA2532479},
	{SG40053, &UA2536149},
	{SG40055, &UA2583094},
	{SG40056, &UA2975802},
	{SG40058, &UA3020842},
	{SG40059, &UA3021965},
	{SG40112, &UA3081620},
	{SG40120, &UA2531998},
	{SG40121, &UA2532471},
	{SG40122, &UA2536320},
	{SG40123, &UA2536614},
	{SG40125, &UA2561458},
	{SG40148, &UA3022009},
	{SG40158, &UA3065155},
	{SG40161, &UA2897269},
	{SG40164, &UA2974384},
	{SG40166, &UA2974453},
	{SG40168, &UA3023529},
	{SG40181, &UA2774408},
	{SG40183, &UA3026933},
	{SG40190, &UA2974360},
	{SG40200, &UA2713956},
	{SG40206, &UA2717898},
	{SG40241, &UA3065131},
	{SG40255, &UA2532160},
	{SG40256, &UA2532472},
	{SG40258, &UA2601891},
	{SG40259, &UA2856602},
	{SG40260, &UA2974252},
	{SG40261, &UA3021537},
	{SG40262, &UA3092448},
	{SG40319, &UA2566960},
	{SG40320, &UA2567015},
	{SG40321, &UA2567134},
	{SG40323, &UA2567948},
	{SG40325, &UA2567951},
	{SG40327, &UA2583143},
	{SG40329, &UA2583200},
	{SG40331, &UA2666719},
	{SG40333, &UA2667180},
	{SG40335, &UA2974160},
	{SG40336, &UA2709745},
	{SG40338, &UA2714022},
	{SG40371, &UA2754946},
	{SG40396, &UA2536616},
	{SG40397, &UA2567192},
	{SG40399, &UA2708919},
	{SG40441, &UA2793281},
	{SG40442, &UA2990260},
	{SG40444, &UA3059378},
	{SG40445, &UA3062644},
	{SG40455, &UA2565479},
	{SG40456, &UA2635238},
	{SG40458, &UA2970951},
	{SG40481, &UA2756414},
	{SG40583, &UA3014534},
	{SG40584, &UA3057626},
	{SG40629, &UA2709260},
	{SG40630, &UA2714045},
	{SG40665, &UA3065481},
	{SG40668, &UA3083329},
	{SG40672, &UA3094215},
	{SG40685, &UA2741719},
	{SG40688, &UA2769987},
	{SG40690, &UA2873520},
	{SG40693, &UA2562414},
	{SG40694, &UA2737793},
	{SG40733, &UA2754587},
	{SG40735, &UA2892996},
	{SG40805, &UA2794720},
	{SG40930, &UA2534909},
	{SG40931, &UA2699375},
	{SG40932, &UA2707661},
	{SG40934, &UA2537629},
	{SG40935, &UA2599060},
	{SG40936, &UA2602473},
	{SG40937, &UA2701120},
	{SG40938, &UA2734232},
	{SG40939, &UA2765812},
	{SG40940, &UA2768260},
	{SG40941, &UA2862319},
	{SG40943, &UA2865804},
	{SG40944, &UA2892042},
	{SG40945, &UA2994664},
	{SG40947, &UA2996139},
	{SG40949, &UA2998652},
	{SG40952, &UA2998860},
	{SG40954, &UA3000723},
	{SG40955, &UA3124425},
	{SG40967, &UA2973653},
	{SG40970, &UA2973911},
	{SG40971, &UA2974874},
	{SG40973, &UA2976097},
	{SG40975, &UA2976695},
	{SG40977, &UA3021994},
	{SG40978, &UA3022398},
	{SG40979, &UA3022886},
	{SG40980, &UA3023111},
	{SG40981, &UA3024353},
	{SG40986, &UA2566486},
	{SG40988, &UA2566901},
	{SG40991, &UA2584092},
	{SG41021, &UA2562765},
	{SG41022, &UA2704218},
	{SG41024, &UA2752158},
	{SG41041, &UA2710019},
	{SG41043, &UA2714667},
	{SG41045, &UA2796120},
	{SG41046, &UA2865796},
	{SG41047, &UA2947874},
	{SG41049, &UA3091775},
	{SG41070, &UA2801261},
	{SG41071, &UA2801265},
	{SG41073, &UA3017457},
	{SG41074, &UA3106339},
	{SG41076, &UA3108230},
	{SG41079, &UA2957788},
	{SG41087, &UA3064978},
	{SG41107, &UA2537065},
	{SG41108, &UA2969579},
	{SG41116, &UA2671105},
	{SG41117, &UA2672110},
	{SG41118, &UA2676251},
	{SG41135, &UA2773347},
	{SG41141, &UA2703930},
	{SG41142, &UA2713720},
	{SG41145, &UA2956721},
	{SG41161, &UA2705240},
	{SG41163, &UA2709257},
	{SG41165, &UA2794310},
	{SG41167, &UA2864250},
	{SG41169, &UA2937659},
	{SG41170, &UA3020185},
	{SG41172, &UA3127377},
	{SG41187, &UA2867131},
	{SG41189, &UA2880695},
	{SG41226, &UA2666109},
	{SG41228, &UA2668169},
	{SG41230, &UA2782341},
	{SG41234, &UA2937191},
	{SG41236, &UA2863171},
	{SG41352, &UA2535476},
	{SG41354, &UA2632244},
	{SG41355, &UA2968947},
	{SG41392, &UA2567437},
	{SG41395, &UA2968199},
	{SG41432, &UA2534905},
	{SG41433, &UA2635453},
	{SG41434, &UA2712391},
	{SG41481, &UA2535477},
	{SG41483, &UA2969944},
	{SG41511, &UA2944320},
	{SG41513, &UA3028135},
	{SG41514, &UA3028234},
	{SG41515, &UA3079729},
	{SG41527, &UA2734423},
	{SG41559, &UA2875171},
	{SG41560, &UA2885356},
	{SG41577, &UA2537623},
	{SG41578, &UA2714666},
	{SG41588, &UA2565037},
	{SG41590, &UA2664399},
	{SG41592, &UA2713994},
	{SG41593, &UA2765849},
	{SG41595, &UA2892142},
	{SG41622, &UA2753267},
	{SG41625, &UA2754265},
	{SG41628, &UA2754581},
	{SG41630, &UA2754627},
	{SG41633, &UA2754933},
	{SG41635, &UA2769283},
	{SG41638, &UA2868104},
	{SG41640, &UA2870780},
	{SG41641, &UA2877346},
	{SG41643, &UA2892536},
	{SG41694, &UA3132002},
	{SG41698, &UA3065002},
	{SG41747, &UA3066520},
	{SG41755, &UA2773317},
	{SG41756, &UA2773557},
	{SG41758, &UA2784804},
	{SG41759, &UA3003077},
	{SG41761, &UA2996367},
	{SG41763, &UA3049055},
	{SG41764, &UA3059984},
	{SG41773, &UA2565478},
	{SG41774, &UA2666135},
	{SG41775, &UA2998608},
	{SG41776, &UA3009803},
	{SG41781, &UA2567445},
	{SG41782, &UA2737446},
	{SG41784, &UA2768643},
	{SG41808, &UA2535458},
	{SG41809, &UA2537062},
	{SG41810, &UA2708531},
	{SG41826, &UA3065364},
	{SG41831, &UA2534894},
	{SG41832, &UA2710947},
	{SG41853, &UA2737457},
	{SG41882, &UA2536319},
	{SG41883, &UA2566228},
	{SG41884, &UA2663265},
	{SG41885, &UA2784909},
	{SG41887, &UA2936777},
	{SG41900, &UA2997918},
	{SG41901, &UA3007705},
	{SG41925, &UA2719299},
	{SG41927, &UA2741260},
	{SG41929, &UA2783710},
	{SG41931, &UA2883128},
	{SG41932, &UA2969514},
	{SG41933, &UA3083584},
	{SG41941, &UA2538606},
	{SG41944, &UA2565383},
	{SG41946, &UA2737836},
	{SG41947, &UA2971371},
	{SG41949, &UA3076279},
	{SG41958, &UA2583786},
	{SG41960, &UA2753468},
	{SG41961, &UA2798630},
	{SG41971, &UA2896425},
	{SG41972, &UA3057301},
	{SG41973, &UA3078353},
	{SG41984, &UA2869811},
	{SG41985, &UA2885079},
	{SG41993, &UA2753275},
	{SG41999, &UA2767454},
	{SG42000, &UA3012644},
	{SG42002, &UA2536601},
	{SG42004, &UA2719450},
	{SG42036, &UA2892349},
	{SG42038, &UA2937156},
	{SG42039, &UA3108760},
	{SG42040, &UA3121799},
	{SG42048, &UA2892568},
	{SG42056, &UA2564106},
	{SG42058, &UA2970423},
	{SG42109, &UA2537069},
	{SG42111, &UA2753093},
	{SG42117, &UA2676923},
	{SG42249, &UA2974358},
	{SG42281, &UA3067070},
	{SG42304, &UA2565796},
	{SG42307, &UA2968699},
	{SG42326, &UA2973960},
	{SG42346, &UA3066230},
	{SG42382, &UA3061598},
	{SG42383, &UA3065999},
	{SG42384, &UA3080020},
	{SG42385, &UA3080713},
	{SG42386, &UA3081393},
	{SG42387, &UA3094832},
	{SG42389, &UA3127069},
	{SG42574, &UA2975709},
	{SG42575, &UA3022499},
	{SG42583, &UA2670620},
	{SG42584, &UA2676281},
	{SG42586, &UA2676386},
	{SG42588, &UA2676654},
	{SG42589, &UA2676670},
	{SG42591, &UA2708545},
	{SG42593, &UA2711816},
	{SG42595, &UA2737827},
	{SG42607, &UA2750982},
	{SG42608, &UA2944861},
	{SG42610, &UA2998876},
	{SG42611, &UA3007127},
	{SG42612, &UA3020236},
	{SG42615, &UA3058843},
	{SG42616, &UA3108016},
	{SG42619, &UA3030939},
	{SG42620, &UA3059265},
	{SG42621, &UA3065092},
	{SG42670, &UA2937090},
	{SG42710, &UA2943983},
	{SG42712, &UA2945966},
	{SG42713, &UA2973284},
	{SG42714, &UA3033718},
	{SG42715, &UA3126337},
	{SG42732, &UA2975006},
	{SG42746, &UA3021062},
	{SG42747, &UA3021424},
	{SG42750, &UA3021783},
	{SG42752, &UA3022644},
	{SG42753, &UA3022704},
	{SG42755, &UA3022720},
	{SG42758, &UA3023176},
	{SG42761, &UA3023625},
	{SG42763, &UA3023636},
	{SG42767, &UA3024211},
	{SG42768, &UA3057724},
	{SG42769, &UA3058076},
	{SG42770, &UA3058081},
	{SG42772, &UA3078267},
	{SG42774, &UA3092204},
	{SG42775, &UA3092391},
	{SG42785, &UA2567553},
	{SG42786, &UA2713885},
	{SG42788, &UA2537628},
	{SG42791, &UA2764664},
	{SG42808, &UA3066524},
	{SG42858, &UA3131669},
	{SG42884, &UA2996276},
	{SG42885, &UA3068414},
	{SG42996, &UA2560450},
	{SG42997, &UA2704701},
	{SG43000, &UA3073447},
	{SG43020, &UA2566236},
	{SG43022, &UA2601078},
	{SG43023, &UA2732857},
	{SG43024, &UA2765611},
	{SG43025, &UA2962070},
	{SG43026, &UA2988627},
	{SG43027, &UA2990497},
	{SG43028, &UA2706874},
	{SG43050, &UA3130797},
	{SG43051, &UA3132877},
	{SG43077, &UA2707586},
	{SG43078, &UA2739795},
	{SG43079, &UA3131463},
	{SG43087, &UA2567433},
	{SG43089, &UA2708328},
	{SG43113, &UA2937428},
	{SG43126, &UA2793780},
	{SG43128, &UA2957513},
	{SG43129, &UA3015683},
	{SG43141, &UA2879130},
	{SG43142, &UA2897916},
	{SG43144, &UA3092337},
	{SG43146, &UA3108927},
	{SG43160, &UA2974123},
	{SG43161, &UA2974250},
	{SG43163, &UA3021068},
	{SG43164, &UA3021080},
	{SG43173, &UA2676439},
	{SG43174, &UA2676824},
	{SG43175, &UA2860985},
	{SG43176, &UA3047472},
	{SG43185, &UA2975939},
	{SG43189, &UA2567191},
	{SG43190, &UA2708993},
	{SG43191, &UA2865594},
	{SG43294, &UA2569881},
	{SG43295, &UA2772756},
	{SG43303, &UA3018593},
	{SG43304, &UA3108254},
	{SG43343, &UA3067298},
	{SG43344, &UA3080977},
	{SG43347, &UA3031098},
	{SG43379, &UA2997405},
	{SG43382, &UA2997546},
	{SG43386, &UA3010764},
	{SG43390, &UA3015324},
	{SG43392, &UA3018972},
	{SG43394, &UA3048514},
	{SG43397, &UA3049380},
	{SG43399, &UA3050618},
	{SG43400, &UA3057838},
	{SG43401, &UA3058242},
	{SG43403, &UA3073546},
	{SG43405, &UA3073927},
	{SG43406, &UA3079746},
	{SG43409, &UA3091492},
	{SG43410, &UA3101468},
	{SG43411, &UA3103360},
	{SG43412, &UA3109033},
	{SG43413, &UA3110338},
	{SG43414, &UA3123123},
	{SG43416, &UA3067066},
	{SG43417, &UA3065339},
	{SG43418, &UA3111088},
	{SG43427, &UA2563960},
	{SG43428, &UA2752694},
	{SG43474, &UA2751091},
	{SG43487, &UA2755111},
	{SG43488, &UA2755523},
	{SG43499, &UA3029352},
	{SG43501, &UA3029362},
	{SG43507, &UA2769313},
	{SG43509, &UA2866452},
	{SG43511, &UA2896176},
	{SG43539, &UA2937232},
	{SG43540, &UA2957936},
	{SG43541, &UA3017808},
	{SG43543, &UA3058485},
	{SG43545, &UA3058567},
	{SG43558, &UA2570121},
	{SG43560, &UA2772910},
	{SG43562, &UA2708780},
	{SG43563, &UA2708847},
	{SG43579, &UA2675095},
	{SG43580, &UA2975712},
	{SG43593, &UA2603204},
	{SG43594, &UA3015920},
	{SG43617, &UA3065138},
	{SG43638, &UA2631649},
	{SG43641, &UA3096917},
	{SG43723, &UA2797502},
	{SG43765, &UA2780285},
	{SG43767, &UA3036772},
	{SG43783, &UA2960060},
	{SG43785, &UA3065581},
	{SG43799, &UA2570034},
	{SG43800, &UA2772906},
	{SG43804, &UA3067179},
	{SG43812, &UA3131351},
	{SG43830, &UA2877970},
	{SG43882, &UA2633414},
	{SG43883, &UA2633520},
	{SG43884, &UA2740083},
	{SG43956, &UA2885369},
	{SG43966, &UA2536301},
	{SG43967, &UA2868462},
	{SG44037, &UA3065109},
	{SG44038, &UA3067832},
	{SG44039, &UA3079481},
	{SG44041, &UA3080634},
	{SG44042, &UA3080636},
	{SG44043, &UA3080806},
	{SG44045, &UA3094001},
	{SG44060, &UA2973481},
	{SG44061, &UA2974597},
	{SG44065, &UA3067190},
	{SG44067, &UA3065981},
	{SG44097, &UA2562199},
	{SG44099, &UA2968479},
	{SG44100, &UA3021254},
	{SG44101, &UA3021502},
	{SG44106, &UA2562369},
	{SG44107, &UA2668939},
	{SG44108, &UA2715471},
	{SG44148, &UA2567442},
	{SG44166, &UA2938351},
	{SG44167, &UA2965726},
	{SG44168, &UA2977072},
	{SG44169, &UA3022851},
	{SG44231, &UA3066528},
	{SG44312, &UA2566779},
	{SG44315, &UA2768889},
	{SG44380, &UA2974371},
	{SG44406, &UA2667493},
	{SG44407, &UA2961887},
	{SG44409, &UA3106200},
	{SG44415, &UA2756754},
	{SG44417, &UA2974818},
	{SG44443, &UA2753273},
	{SG44446, &UA2755124},
	{SG44461, &UA3066517},
	{SG44511, &UA2753774},
	{SG44513, &UA2755538},
	{SG44515, &UA2800585},
	{SG44516, &UA2879122},
	{SG44561, &UA2705799},
	{SG44571, &UA3067200},
	{SG44573, &UA2568071},
	{SG44574, &UA2603860},
	{SG44576, &UA2717073},
	{SG44577, &UA2936508},
	{SG44585, &UA3080218},
	{SG44586, &UA3093665},
	{SG44612, &UA2534061},
	{SG44614, &UA2892151},
	{SG44616, &UA3036766},
	{SG44646, &UA2567828},
	{SG44647, &UA2674165},
	{SG44649, &UA2710514},
	{SG44650, &UA2710836},
	{SG44651, &UA2738713},
	{SG44652, &UA2752810},
	{SG44654, &UA2800258},
	{SG44662, &UA3065484},
	{SG44683, &UA2898940},
	{SG44686, &UA2937761},
	{SG44689, &UA2947553},
	{SG44694, &UA3068052},
	{SG44695, &UA3082985},
	{SG44704, &UA2974921},
	{SG44706, &UA3022795},
	{SG44720, &UA3057722},
	{SG44723, &UA2569597},
	{SG44724, &UA2716801},
	{SG44725, &UA2737919},
	{SG44728, &UA2938736},
	{SG44729, &UA3021718},
	{SG44768, &UA3023728},
	{SG44776, &UA3131340},
	{SG44780, &UA2969147},
	{SG44826, &UA2532170},
	{SG44828, &UA2566318},
	{SG44831, &UA2607208},
	{SG44835, &UA2751836},
	{SG44837, &UA2794851},
	{SG44838, &UA2566912},
	{SG44839, &UA2567342},
	{SG44841, &UA2567085},
	{SG44842, &UA2567562},
	{SG44843, &UA2583193},
	{SG44844, &UA2602041},
	{SG44845, &UA2732929},
	{SG44846, &UA2737342},
	{SG44847, &UA2752707},
	{SG44848, &UA3033734},
	{SG44854, &UA3068253},
	{SG44870, &UA2562920},
	{SG44871, &UA2708769},
	{SG44877, &UA3028262},
	{SG44892, &UA2974581},
	{SG44893, &UA2975810},
	{SG44894, &UA2568300},
	{SG44895, &UA2738026},
	{SG44896, &UA2957103},
	{SG44897, &UA2534063},
	{SG44909, &UA2673617},
	{SG44912, &UA2709756},
	{SG44915, &UA2870142},
	{SG44917, &UA2893342},
	{SG44922, &UA2633247},
	{SG44923, &UA2636058},
	{SG44924, &UA2937242},
	{SG44926, &UA2995854},
	{SG44929, &UA2536597},
	{SG44944, &UA3132628},
	{SG45035, &UA2560764},
	{SG45037, &UA2967299},
	{SG45050, &UA2772909},
	{SG45054, &UA2581987},
	{SG45055, &UA2738732},
	{SG45077, &UA2975001},
	{SG45079, &UA2570362},
	{SG45080, &UA2672759},
	{SG45081, &UA2713605},
	{SG45115, &UA2535468},
	{SG45119, &UA2877969},
	{SG45121, &UA3036919},
	{SG45139, &UA2941528},
	{SG45277, &UA2672704},
	{SG45278, &UA2677169},
	{SG45279, &UA2862143},
	{SG45282, &UA2782817},
	{SG45283, &UA2798631},
	{SG45289, &UA2945978},
	{SG45343, &UA3067187},
	{SG45370, &UA2752696},
	{SG45371, &UA2768199},
	{SG45376, &UA3039240},
	{SG45386, &UA3001123},
	{SG45387, &UA3058345},
	{SG45388, &UA3107847},
	{SG45390, &UA2974649},
	{SG45392, &UA2975949},
	{SG45395, &UA3023387},
	{SG45403, &UA2892511},
	{SG45406, &UA2894258},
	{SG45414, &UA2754260},
	{SG45416, &UA2772678},
	{SG45429, &UA2563744},
	{SG45431, &UA2971611},
	{SG45458, &UA2975930},
	{SG45459, &UA3023135},
	{SG45491, &UA2712203},
	{SG45492, &UA2734716},
	{SG45493, &UA2859929},
	{SG45495, &UA3006418},
	{SG45497, &UA3006955},
	{SG45500, &UA3092638},
	{SG45502, &UA3107215},
	{SG45503, &UA3121559},
	{SG45504, &UA3121562},
	{SG45506, &UA3124837},
	{SG45507, &UA3125564},
	{SG45509, &UA3126008},
	{SG45512, &UA3127662},
	{SG45522, &UA2701238},
	{SG45523, &UA2973889},
	{SG45525, &UA2753015},
	{SG45527, &UA2783541},
	{SG45528, &UA2774409},
	{SG45529, &UA3016284},
	{SG45551, &UA2514297},
	{SG45553, &UA2751169},
	{SG45571, &UA3131339},
	{SG45598, &UA2537071},
	{SG45599, &UA2663117},
	{SG45600, &UA2738260},
	{SG45602, &UA2767653},
	{SG45606, &UA2566999},
	{SG45608, &UA2567709},
	{SG45609, &UA2733454},
	{SG45627, &UA2534058},
	{SG45631, &UA2602521},
	{SG45632, &UA2961528},
	{SG45633, &UA2988928},
	{SG45634, &UA3078576},
	{SG45640, &UA2975535},
	{SG45667, &UA2561457},
	{SG45668, &UA3018745},
	{SG45693, &UA2533175},
	{SG45694, &UA2535480},
	{SG45769, &UA2536321},
	{SG45770, &UA2857824},
	{SG45776, &UA2581232},
	{SG45778, &UA2892269},
	{SG45780, &UA2974639},
	{SG45781, &UA3066227},
	{SG45785, &UA3067918},
	{SG45794, &UA2562372},
	{SG45796, &UA2936700},
	{SG45797, &UA2944803},
	{SG45799, &UA3078815},
	{SG45801, &UA3107752},
	{SG45816, &UA2566748},
	{SG45821, &UA3126429},
	{SG45840, &UA2974496},
	{SG45850, &UA2977090},
	{SG45852, &UA3064005},
	{SG45866, &UA3064023},
	{SG45869, &UA3063037},
	{SG45870, &UA3131079},
	{SG45871, &UA3131350},
	{SG45874, &UA2975070},
	{SG45903, &UA2754715},
	{SG45904, &UA2782261},
	{SG45906, &UA3058059},
	{SG45907, &UA3108802},
	{SG45909, &UA3068331},
	{SG45912, &UA3082722},
	{SG45915, &UA3113640},
	{SG45920, &UA2566785},
	{SG45921, &UA2793483},
	{SG45922, &UA2866216},
	{SG45931, &UA2675336},
	{SG45933, &UA2676255},
	{SG45949, &UA2568292},
	{SG45953, &UA2737443},
	{SG45963, &UA3068047},
	{SG45965, &UA3080976},
	{SG45991, &UA2566839},
	{SG45993, &UA2567834},
	{SG45995, &UA2674167},
	{SG45996, &UA2712700},
	{SG45997, &UA2739708},
	{SG45998, &UA2739716},
	{SG45999, &UA2567001},
	{SG46000, &UA2633365},
	{SG46004, &UA2865819},
	{SG46005, &UA2938529},
	{SG46032, &UA3063297},
	{SG46033, &UA3068318},
	{SG46055, &UA2562911},
	{SG46057, &UA2892584},
	{SG46058, &UA2956987},
	{SG46098, &UA2797775},
	{SG46099, &UA2895072},
	{SG46100, &UA2961929},
	{SG46101, &UA3023826},
	{SG46133, &UA2768633},
	{SG46135, &UA3049052},
	{SG46147, &UA2866923},
	{SG46148, &UA2962223},
	{SG46161, &UA2781854},
	{SG46167, &UA2755344},
	{SG46168, &UA2782670},
	{SG46169, &UA3068279},
	{SG46202, &UA2974318},
	{SG46203, &UA3057822},
	{SG46211, &UA2676019},
	{SG46213, &UA2676776},
	{SG46223, &UA2961570},
	{SG46225, &UA3092663},
	{SG46226, &UA3125631},
	{SG46227, &UA3132768},
	{SG46232, &UA2936724},
	{SG46233, &UA3036941},
	{SG46251, &UA2975958},
	{SG46261, &UA3065373},
	{SG46268, &UA2566702},
	{SG46269, &UA2583215},
	{SG46274, &UA2797400},
	{SG46276, &UA2867090},
	{SG46278, &UA2870779},
	{SG46281, &UA2892575},
	{SG46282, &UA2892642},
	{SG46286, &UA3049852},
	{SG46288, &UA3124907},
	{SG46296, &UA2756889},
	{SG46298, &UA2754943},
	{SG46299, &UA2879144},
	{SG46307, &UA2740416},
	{SG46309, &UA2894888},
	{SG46311, &UA3018633},
	{SG46313, &UA3027673},
	{SG46317, &UA2866256},
	{SG46318, &UA2998998},
	{SG46322, &UA2754583},
	{SG46329, &UA3031481},
	{SG46330, &UA2996340},
	{SG46331, &UA3015322},
	{SG46332, &UA2603663},
	{SG46334, &UA2665826},
	{SG46336, &UA2700859},
	{SG46337, &UA2935295},
	{SG46338, &UA2936939},
	{SG46342, &UA2566752},
	{SG46351, &UA2974711},
	{SG46353, &UA2975008},
	{SG46414, &UA2880096},
	{SG46418, &UA2561413},
	{SG46420, &UA2671056},
	{SG46422, &UA2714539},
	{SG46426, &UA3026042},
	{SG46428, &UA2753269},
	{SG46430, &UA2783240},
	{SG46438, &UA3082731},
	{SG46443, &UA2676446},
	{SG46455, &UA2900574},
	{SG46458, &UA2943765},
	{SG46460, &UA3058749},
	{SG46478, &UA2566958},
	{SG46479, &UA2634451},
	{SG46482, &UA3082716},
	{SG46492, &UA2665563},
	{SG46500, &UA2676819},
	{SG46503, &UA2714414},
	{SG46550, &UA2753549},
	{SG46551, &UA2975003},
	{SG46552, &UA3023037},
	{SG46553, &UA2974111},
	{SG46569, &UA2946992},
	{SG46575, &UA2975051},
	{SG46576, &UA2975081},
	{SG46585, &UA2664408},
	{SG46588, &UA2751779},
	{SG46603, &UA2756233},
	{SG46610, &UA2974876},
	{SG46613, &UA2963718},
	{SG46615, &UA3019362},
	{SG46617, &UA3025164},
	{SG46619, &UA3104614},
	{SG46636, &UA2975865},
	{SG46638, &UA3021871},
	{SG46655, &UA2535474},
	{SG46658, &UA2536593},
	{SG46672, &UA3131717},
	{SG46680, &UA2737769},
	{SG46682, &UA2858884},
	{SG46687, &UA3033940},
	{SG46688, &UA3058958},
	{SG46689, &UA3059012},
	{SG46692, &UA2754372},
	{SG46694, &UA2672457},
	{SG46696, &UA2676919},
	{SG46703, &UA2937954},
	{SG46726, &UA2566949},
	{SG46728, &UA2601582},
	{SG46730, &UA3067592},
	{SG46732, &UA2673616},
	{SG46735, &UA2676694},
	{SG46743, &UA2705239},
	{SG46745, &UA2752329},
	{SG46747, &UA2868904},
	{SG46752, &UA2564372},
	{SG46755, &UA2738168},
	{SG46778, &UA2583045},
	{SG46780, &UA3021327},
	{SG46786, &UA2976012},
	{SG46792, &UA2754796},
	{SG46813, &UA3096288},
	{SG46825, &UA2956008},
	{SG46827, &UA2998121},
	{SG46843, &UA2891463},
	{SG46844, &UA2995676},
	{SG46845, &UA2734861},
	{SG46846, &UA3067488},
	{SG46847, &UA3083490},
	{SG46848, &UA3083254},
	{SG46850, &UA2567217},
	{SG46851, &UA2584846},
	{SG46852, &UA2794982},
	{SG46854, &UA2867139},
	{SG46855, &UA2958338},
	{SG46885, &UA2976094},
	{SG46912, &UA3067063},
	{SG46918, &UA2564440},
	{SG46919, &UA2708320},
	{SG46929, &UA2782982},
	{SG46930, &UA2865374},
	{SG46933, &UA2754948},
	{SG46941, &UA2974881},
	{SG46942, &UA3024033},
	{SG46952, &UA2773199},
	{SG46954, &UA2799872},
	{SG46955, &UA2885388},
	{SG46956, &UA3027874},
	{SG46957, &UA3108279},
	{SG46959, &UA3031638},
	{SG46960, &UA3093196},
	{SG46971, &UA2566791},
	{SG46974, &UA2738444},
	{SG46977, &UA2754723},
	{SG46982, &UA2772695},
	{SG46997, &UA2714540},
	{SG46999, &UA2768943},
	{SG47015, &UA3080132},
	{SG47017, &UA3111716},
	{SG47023, &UA2676984},
	{SG47024, &UA2735036},
	{SG47026, &UA2751961},
	{SG47032, &UA2798996},
	{SG47054, &UA2755336},
	{SG47063, &UA2535479},
	{SG47065, &UA2535788},
	{SG47066, &UA2662475},
	{SG47104, &UA3019622},
	{SG47105, &UA3125846},
	{SG47129, &UA2670744},
	{SG47134, &UA2535461},
	{SG47136, &UA2975040},
	{SG47143, &UA2561914},
	{SG47145, &UA2868453},
	{SG47148, &UA2995589},
	{SG47152, &UA2533190},
	{SG47155, &UA2858864},
	{SG47164, &UA2974462},
	{SG47175, &UA2564664},
	{SG47177, &UA3016938},
	{SG47178, &UA2752169},
	{SG47179, &UA2781928},
	{SG47180, &UA2868313},
	{SG47181, &UA2957461},
	{SG47190, &UA3011518},
	{SG47192, &UA3019551},
	{SG47194, &UA3020501},
	{SG47195, &UA3050783},
	{SG47206, &UA2536318},
	{SG47208, &UA2670626},
	{SG47209, &UA3065976},
	{SG47210, &UA2566619},
	{SG47212, &UA2754058},
	{SG47214, &UA2861551},
	{SG47216, &UA2865477},
	{SG47219, &UA3027653},
	{SG47227, &UA3001672},
	{SG47228, &UA3065803},
	{SG47229, &UA3125287},
	{SG47231, &UA3093522},
	{SG47232, &UA2939395},
	{SG47233, &UA3009499},
	{SG47235, &UA2892202},
	{SG47237, &UA3025163},
	{SG47241, &UA2974902},
	{SG47245, &UA2664268},
	{SG47248, &UA2999662},
	{SG47250, &UA3008510},
	{SG47252, &UA2673343},
	{SG47254, &UA3032818},
	{SG47320, &UA2676324},
	{SG47321, &UA2861026},
	{SG47323, &UA2958170},
	{SG47324, &UA3019805},
	{SG47342, &UA3131472},
	{SG47347, &UA2566620},
	{SG47348, &UA2606247},
	{SG47352, &UA3028268},
	{SG47377, &UA2755702},
	{SG47383, &UA2774400},
	{SG47384, &UA3018944},
	{SG47435, &UA3020037},
	{SG47455, &UA2888299},
	{SG47468, &UA2676693},
	{SG47479, &UA3021722},
	{SG47495, &UA2901213},
	{SG47498, &UA2567337},
	{SG47499, &UA2667831},
	{SG47510, &UA2997690},
	{SG47523, &UA2603888},
	{SG47525, &UA3056400},
	{SG47530, &UA2975566},
	{SG47537, &UA3081281},
	{SG47538, &UA3081284},
	{SG47552, &UA2974924},
	{SG47554, &UA3057984},
	{SG47576, &UA2599228},
	{SG47577, &UA2768445},
	{SG47578, &UA2971203},
	{SG47594, &UA2567671},
	{SG47613, &UA2754944},
	{SG47632, &UA3021485},
	{SG47644, &UA3082742},
	{SG47651, &UA2754273},
	{SG47653, &UA2899123},
	{SG47678, &UA2599405},
	{SG47679, &UA2936735},
	{SG47680, &UA3012460},
	{SG47744, &UA3029659},
	{SG47751, &UA2974074},
	{SG47760, &UA2892098},
	{SG47761, &UA3008268},
	{SG47765, &UA2665759},
	{SG47766, &UA2734499},
	{SG47768, &UA2935348},
	{SG47770, &UA3049151},
	{SG47781, &UA2794418},
	{SG47783, &UA2937535},
	{SG47789, &UA2901216},
	{SG47791, &UA3017877},
	{SG47796, &UA2784918},
	{SG47805, &UA3081183},
	{SG47812, &UA3080857},
	{SG47819, &UA2997853},
	{SG47821, &UA3073720},
	{SG47824, &UA3078281},
	{SG47851, &UA3018101},
	{SG47852, &UA3027300},
	{SG47854, &UA3061785},
	{SG47857, &UA3082727},
	{SG47869, &UA2676028},
	{SG47871, &UA2709085},
	{SG47882, &UA2969231},
	{SG47883, &UA3020950},
	{SG47943, &UA2666699},
	{SG47944, &UA2769400},
	{SG47947, &UA2567198},
	{SG47950, &UA2862525},
	{SG47963, &UA2561582},
	{SG47967, &UA3031068},
	{SG47969, &UA3029498},
	{SG47980, &UA2567142},
	{SG48000, &UA2707843},
	{SG48001, &UA2711758},
	{SG48002, &UA2714469},
	{SG48072, &UA2970574},
	{SG48073, &UA3092362},
	{SG48084, &UA2738244},
	{SG48086, &UA2997741},
	{SG48088, &UA2964282},
	{SG48105, &UA3081387},
	{SG48113, &UA2961577},
	{SG48114, &UA3092289},
	{SG48122, &UA2602723},
	{SG48125, &UA2774065},
	{SG48133, &UA2664428},
	{SG48134, &UA2958278},
	{SG48140, &UA3006416},
	{SG48141, &UA3011545},
	{SG48142, &UA3080367},
	{SG48143, &UA3095060},
	{SG48171, &UA2944656},
	{SG48173, &UA2969670},
	{SG48175, &UA3015292},
	{SG48184, &UA2534064},
	{SG48185, &UA2565564},
	{SG48204, &UA2947697},
	{SG48226, &UA2537052},
	{SG48242, &UA3031047},
	{SG48245, &UA2975697},
	{SG48252, &UA3068397},
	{SG48259, &UA2562020},
	{SG48260, &UA3011205},
	{SG48263, &UA3049769},
	{SG48314, &UA2567139},
	{SG48324, &UA2677279},
	{SG48326, &UA2735570},
	{SG48342, &UA3000715},
	{SG48350, &UA2708920},
	{SG48358, &UA2679927},
	{SG48377, &UA3060875},
	{SG48378, &UA3125348},
	{SG48385, &UA3131467},
	{SG48395, &UA2892630},
	{SG48396, &UA3059790},
	{SG48397, &UA3105061},
	{SG48400, &UA2975696},
	{SG48408, &UA2600095},
	{SG48409, &UA2754724},
	{SG48416, &UA2582480},
	{SG48417, &UA2670020},
	{SG48419, &UA2752851},
	{SG48429, &UA2975539},
	{SG48430, &UA3082981},
	{SG48432, &UA3029075},
	{SG48475, &UA2599082},
	{SG48477, &UA2768963},
	{SG48493, &UA2664984},
	{SG48494, &UA2702311},
	{SG48495, &UA2749441},
	{SG48496, &UA2956955},
	{SG48497, &UA3126810},
	{SG48498, &UA3080911},
	{SG48512, &UA2667807},
	{SG48514, &UA2751910},
	{SG48516, &UA2710723},
	{SG48518, &UA2714678},
	{SG48529, &UA3082970},
	{SG48536, &UA2714089},
	{SG48538, &UA2855352},
	{SG48539, &UA2960416},
	{SG48542, &UA3082766},
	{SG48544, &UA3067621},
	{SG48545, &UA3093747},
	{SG48546, &UA3114931},
	{SG48556, &UA3065719},
	{SG48566, &UA2567135},
	{SG48567, &UA2667657},
	{SG48582, &UA3037936},
	{SG48618, &UA3081121},
	{SG48619, &UA3083496},
	{SG48625, &UA2976059},
	{SG48627, &UA3110816},
	{SG48628, &UA3081058},
	{SG48650, &UA2774080},
	{SG48652, &UA2876894},
	{SG48665, &UA2974086},
	{SG48694, &UA3022192},
	{SG48696, &UA3024511},
	{SG48703, &UA2677298},
	{SG48708, &UA2599295},
	{SG48709, &UA2952649},
	{SG48712, &UA2567332},
	{SG48714, &UA2637943},
	{SG48728, &UA2674740},
	{SG48729, &UA2999624},
	{SG48733, &UA3039276},
	{SG48762, &UA2749135},
	{SG48764, &UA2781588},
	{SG48765, &UA2895546},
	{SG48774, &UA3080989},
	{SG48863, &UA3019163},
	{SG48864, &UA3020335},
	{SG48879, &UA2773884},
	{SG48880, &UA2755339},
	{SG48881, &UA2772684},
	{SG48884, &UA3132943},
	{SG48922, &UA2868692},
	{SG48923, &UA3008860},
	{SG48924, &UA3022338},
	{SG48954, &UA2560810},
	{SG48996, &UA3083495},
	{SG49028, &UA2964049},
	{SG49038, &UA2674203},
	{SG49051, &UA2796774},
	{SG49052, &UA3074968},
	{SG49056, &UA3080710},
	{SG49067, &UA2633279},
	{SG49068, &UA2635313},
	{SG49069, &UA3088031},
	{SG49071, &UA2801268},
	{SG49102, &UA2973669},
	{SG49124, &UA3013224},
	{SG49133, &UA2900016},
	{SG49135, &UA2774087},
	{SG49145, &UA2536592},
	{SG49147, &UA2854322},
	{SG49149, &UA2536299},
	{SG49151, &UA2602548},
	{SG49153, &UA2782461},
	{SG49157, &UA2879537},
	{SG49162, &UA2879798},
	{SG49165, &UA2708966},
	{SG49169, &UA2895317},
	{SG49170, &UA3016838},
	{SG49171, &UA2958258},
	{SG49226, &UA3036661},
	{SG49228, &UA3130720},
	{SG49229, &UA3132958},
	{SG49272, &UA2563805},
	{SG49301, &UA2965785},
	{SG49309, &UA2975991},
	{SG49311, &UA3130938},
	{SG49324, &UA2567496},
	{SG49330, &UA2566535},
	{SG49385, &UA2885918},
	{SG49393, &UA2582688},
	{SG49395, &UA3080586},
	{SG49413, &UA2582714},
	{SG49416, &UA3018583},
	{SG49421, &UA2892171},
	{SG49457, &UA2877892},
	{SG49463, &UA3083644},
	{SG49480, &UA2676087},
	{SG49490, &UA2750638},
	{SG49491, &UA3012470},
	{SG49499, &UA3023378},
	{SG49502, &UA2566964},
	{SG49503, &UA2719587},
	{SG49504, &UA2719597},
	{SG49506, &UA2719708},
	{SG49508, &UA3030021},
	{SG49509, &UA3107804},
	{SG49573, &UA2704585},
	{SG49597, &UA2676231},
	{SG49598, &UA2753851},
	{SG49599, &UA2867198},
	{SG49600, &UA2773116},
	{SG49657, &UA2672790},
	{SG49659, &UA3011041},
	{SG49723, &UA3131324},
	{SG49735, &UA2706334},
	{SG49737, &UA3088476},
	{SG49744, &UA3096289},
	{SG49768, &UA2773015},
	{SG49769, &UA3061189},
	{SG49770, &UA3080019},
	{SG49772, &UA3131736},
	{SG49777, &UA3023377},
	{SG49801, &UA2800681},
	{SG49804, &UA3032063},
	{SG49898, &UA3081100},
	{SG49904, &UA2599505},
	{SG49906, &UA2974662},
	{SG49907, &UA2977087},
	{SG49908, &UA2977097},
	{SG49909, &UA3092929},
	{SG49910, &UA3108264},
	{SG49919, &UA2773710},
	{SG49925, &UA2666360},
	{SG49926, &UA2768743},
	{SG49931, &UA2896859},
	{SG49944, &UA2974108},
	{SG49946, &UA3080311},
	{SG49952, &UA2585177},
	{SG49954, &UA2636988},
	{SG49977, &UA2664697},
	{SG49981, &UA3088954},
	{SG49999, &UA3020414},
	{SG50001, &UA2773811},
	{SG50014, &UA2892409},
	{SG50043, &UA2782763},
	{SG50046, &UA2885656},
	{SG50051, &UA3115108},
	{SG50063, &UA2672260},
	{SG50113, &UA2773182},
	{SG50116, &UA2797407},
	{SG50117, &UA2877044},
	{SG50183, &UA2799687},
	{SG50189, &UA2975719},
	{SG50215, &UA3022806},
	{SG50239, &UA2799408},
	{SG50240, &UA3013909},
	{SG50316, &UA2767829},
	{SG50318, &UA3050948},
	{SG50388, &UA2565034},
	{SG50389, &UA2669694},
	{SG50390, &UA2781815},
	{SG50392, &UA2633816},
	{SG50393, &UA2957590},
	{SG50395, &UA2974604},
	{SG50406, &UA2563402},
	{SG50428, &UA2960484},
	{SG50452, &UA2756566},
	{SG50458, &UA2974713},
	{SG50493, &UA3131392},
	{SG50502, &UA2582971},
	{SG50509, &UA3080638},
	{SG50510, &UA3093504},
	{SG50522, &UA3083269},
	{SG50539, &UA2781800},
	{SG50541, &UA2859808},
	{SG50552, &UA2534053},
	{SG50554, &UA2674738},
	{SG50556, &UA2749232},
	{SG50558, &UA2782582},
	{SG50571, &UA2773898},
	{SG50572, &UA3025625},
	{SG50630, &UA2770064},
	{SG50631, &UA2582475},
	{SG50633, &UA2704735},
	{SG50634, &UA2870153},
	{SG50637, &UA2893301},
	{SG50638, &UA3014619},
	{SG50655, &UA3095731},
	{SG50663, &UA2599229},
	{SG50665, &UA3019386},
	{SG50666, &UA2738171},
	{SG50669, &UA3022370},
	{SG50670, &UA3024092},
	{SG50677, &UA2754411},
	{SG50678, &UA2774328},
	{SG50693, &UA2769437},
	{SG50694, &UA2937840},
	{SG50695, &UA2877951},
	{SG50696, &UA2877953},
	{SG50697, &UA2877960},
	{SG50698, &UA2877958},
	{SG50699, &UA3109007},
	{SG50700, &UA3125438},
	{SG50701, &UA3125560},
	{SG50704, &UA3057493},
	{SG50709, &UA2937123},
	{SG50714, &UA2994763},
	{SG50715, &UA2996116},
	{SG50718, &UA2999714},
	{SG50725, &UA2669408},
	{SG50727, &UA3093101},
	{SG50731, &UA2784840},
	{SG50733, &UA2873371},
	{SG50734, &UA3104016},
	{SG50738, &UA2996854},
	{SG50740, &UA3020291},
	{SG50744, &UA3083832},
	{SG50745, &UA3080662},
	{SG50756, &UA2772300},
	{SG50759, &UA2799131},
	{SG50781, &UA2599850},
	{SG50801, &UA3063895},
	{SG50817, &UA3022882},
	{SG50821, &UA3131716},
	{SG50837, &UA3021567},
	{SG50838, &UA3022264},
	{SG50840, &UA3132870},
	{SG50852, &UA2711476},
	{SG50854, &UA2866477},
	{SG50855, &UA2937167},
	{SG50857, &UA2996546},
	{SG50868, &UA2676579},
	{SG50928, &UA2892336},
	{SG50930, &UA3092343},
	{SG50931, &UA3061489},
	{SG50933, &UA3081264},
	{SG50952, &UA2667403},
	{SG50954, &UA3021642},
	{SG50971, &UA2663818},
	{SG50976, &UA2719634},
	{SG50977, &UA3022275},
	{SG50980, &UA2943214},
	{SG50982, &UA2867013},
	{SG50999, &UA2677692},
	{SG51029, &UA2718979},
	{SG51032, &UA2752153},
	{SG51035, &UA2860443},
	{SG51037, &UA2784790},
	{SG51038, &UA2944408},
	{SG51042, &UA3062717},
	{SG51044, &UA3081573},
	{SG51047, &UA3094493},
	{SG51058, &UA3022800},
	{SG51091, &UA2599654},
	{SG51093, &UA2866728},
	{SG51109, &UA2667030},
	{SG51114, &UA2769168},
	{SG51116, &UA3020706},
	{SG51118, &UA3081465},
	{SG51134, &UA2773270},
	{SG51146, &UA2583216},
	{SG51168, &UA2708507},
	{SG51169, &UA2719414},
	{SG51170, &UA2867310},
	{SG51171, &UA2999318},
	{SG51173, &UA3083156},
	{SG51183, &UA2631217},
	{SG51186, &UA2583106},
	{SG51189, &UA2662954},
	{SG51192, &UA2796162},
	{SG51237, &UA2974805},
	{SG51310, &UA2769953},
	{SG51334, &UA2784716},
	{SG51336, &UA3024168},
	{SG51340, &UA3032069},
	{SG51362, &UA3019844},
	{SG51363, &UA3126740},
	{SG51365, &UA3083364},
	{SG51377, &UA2607327},
	{SG51378, &UA2856606},
	{SG51384, &UA2942911},
	{SG51385, &UA3022939},
	{SG51439, &UA3010609},
	{SG51443, &UA3021193},
	{SG51445, &UA3115506},
	{SG51462, &UA3083493},
	{SG51483, &UA2712288},
	{SG51486, &UA2752688},
	{SG51491, &UA2944804},
	{SG51494, &UA3012742},
	{SG51495, &UA3057160},
	{SG51496, &UA3057337},
	{SG51498, &UA3037939},
	{SG51499, &UA3092196},
	{SG51509, &UA3102308},
	{SG51512, &UA2765487},
	{SG51513, &UA3008682},
	{SG51515, &UA2602800},
	{SG51517, &UA2892267},
	{SG51519, &UA2716969},
	{SG51520, &UA2771509},
	{SG51521, &UA2938022},
	{SG51523, &UA2974725},
	{SG51543, &UA3060752},
	{SG51557, &UA2754230},
	{SG51558, &UA2879488},
	{SG51561, &UA3081616},
	{SG51579, &UA3023139},
	{SG51584, &UA3029908},
	{SG51586, &UA3094290},
	{SG51595, &UA2561596},
	{SG51597, &UA2997235},
	{SG51600, &UA2861237},
	{SG51601, &UA2937228},
	{SG51609, &UA2671097},
	{SG51612, &UA3020972},
	{SG51618, &UA3132878},
	{SG51640, &UA2769772},
	{SG51651, &UA2583784},
	{SG51660, &UA2793716},
	{SG51661, &UA2957106},
	{SG51695, &UA3035724},
	{SG51704, &UA2704448},
	{SG51707, &UA2601661},
	{SG51710, &UA2793430},
	{SG51711, &UA2937224},
	{SG51717, &UA2898919},
	{SG51718, &UA2972129},
	{SG51733, &UA2733720},
	{SG51740, &UA3130935},
	{SG51751, &UA3022788},
	{SG51752, &UA3062656},
	{SG51765, &UA3093679},
	{SG51777, &UA2749971},
	{SG51778, &UA2863997},
	{SG51783, &UA3080286},
	{SG51798, &UA2632331},
	{SG51800, &UA2632633},
	{SG51803, &UA2711477},
	{SG51805, &UA3088856},
	{SG51806, &UA3107709},
	{SG51812, &UA2893596},
	{SG51833, &UA2755342},
	{SG51835, &UA2797026},
	{SG51837, &UA2877357},
	{SG51838, &UA2879362},
	{SG51842, &UA3109085},
	{SG51844, &UA3093663},
	{SG51846, &UA3109452},
	{SG51871, &UA3079383},
	{SG51875, &UA2960664},
	{SG51878, &UA3059531},
	{SG51880, &UA3080995},
	{SG51902, &UA2673952},
	{SG51904, &UA2754250},
	{SG51907, &UA3028263},
	{SG51909, &UA3082696},
	{SG51911, &UA3083063},
	{SG51912, &UA3079961},
	{SG51913, &UA3080434},
	{SG51914, &UA3093172},
	{SG51925, &UA2563995},
	{SG51930, &UA2667592},
	{SG51931, &UA2670750},
	{SG51942, &UA2972109},
	{SG51943, &UA3023522},
	{SG51944, &UA3092370},
	{SG51972, &UA2734598},
	{SG51974, &UA2937211},
	{SG51976, &UA2988799},
	{SG51978, &UA3020413},
	{SG51979, &UA3021960},
	{SG51981, &UA3023108},
	{SG51990, &UA2564236},
	{SG51992, &UA2937707},
	{SG52013, &UA2878849},
	{SG52015, &UA2892726},
	{SG52016, &UA3077162},
	{SG52017, &UA2997075},
	{SG52095, &UA2937366},
	{SG52096, &UA3023232},
	{SG52114, &UA3083605},
	{SG52129, &UA2583298},
	{SG52133, &UA3021268},
	{SG52153, &UA2738080},
	{SG52158, &UA3111276},
	{SG52168, &UA2869167},
	{SG52170, &UA3048175},
	{SG52243, &UA2938254},
	{SG52244, &UA2769932},
	{SG52245, &UA2773118},
	{SG52248, &UA2977095},
	{SG52249, &UA2977098},
	{SG52252, &UA3020797},
	{SG52253, &UA3022136},
	{SG52261, &UA2676309},
	{SG52263, &UA2702653},
	{SG52265, &UA2996831},
	{SG52270, &UA3083598},
	{SG52285, &UA2779359},
	{SG52292, &UA3062165},
	{SG52308, &UA2665741},
	{SG52309, &UA3057268},
	{SG52310, &UA2675836},
	{SG52312, &UA2772861},
	{SG52314, &UA2975057},
	{SG52316, &UA3023431},
	{SG52317, &UA3080010},
	{SG52318, &UA3109653},
	{SG52323, &UA3112883},
	{SG52325, &UA3112884},
	{SG52343, &UA2674440},
	{SG52345, &UA2715801},
	{SG52346, &UA2749838},
	{SG52347, &UA2865393},
	{SG52350, &UA2771243},
	{SG52351, &UA2997765},
	{SG52391, &UA2768114},
	{SG52394, &UA2997628},
	{SG52396, &UA3020421},
	{SG52421, &UA3023128},
	{SG52437, &UA2663826},
	{SG52441, &UA2537616},
	{SG52444, &UA2601952},
	{SG52447, &UA2864568},
	{SG52449, &UA2987565},
	{SG52457, &UA2956692},
	{SG52458, &UA3022203},
	{SG52471, &UA2754074},
	{SG52474, &UA3094556},
	{SG52475, &UA3094450},
	{SG52483, &UA2702032},
	{SG52487, &UA3022557},
	{SG52488, &UA3011506},
	{SG52489, &UA3045341},
	{SG52490, &UA3121407},
	{SG52503, &UA2583173},
	{SG52505, &UA2600984},
	{SG52506, &UA3038896},
	{SG52507, &UA3083381},
	{SG52510, &UA2706950},
	{SG52515, &UA2971555},
	{SG52535, &UA3019753},
	{SG52536, &UA3028231},
	{SG52562, &UA2663719},
	{SG52566, &UA2709868},
	{SG52567, &UA3121631},
	{SG52574, &UA3021552},
	{SG52575, &UA3021915},
	{SG52592, &UA2769901},
	{SG52593, &UA2872752},
	{SG52596, &UA2994233},
	{SG52597, &UA3087947},
	{SG52599, &UA3021195},
	{SG52600, &UA3022893},
	{SG52602, &UA3023850},
	{SG52603, &UA3080309},
	{SG52606, &UA3093910},
	{SG52622, &UA2799011},
	{SG52626, &UA3024178},
	{SG52635, &UA2583370},
	{SG52642, &UA2996464},
	{SG52644, &UA3106984},
	{SG52646, &UA3025944},
	{SG52648, &UA3115610},
	{SG52650, &UA3131720},
	{SG52662, &UA2600250},
	{SG52666, &UA2752648},
	{SG52670, &UA2856955},
	{SG52672, &UA2972105},
	{SG52673, &UA2769494},
	{SG52677, &UA3093971},
	{SG52681, &UA2602042},
	{SG52682, &UA2719727},
	{SG52683, &UA3126857},
	{SG52684, &UA2796075},
	{SG52688, &UA3083650},
	{SG52705, &UA2874809},
	{SG52716, &UA2567949},
	{SG52717, &UA2783707},
	{SG52718, &UA2975567},
	{SG52739, &UA2584100},
	{SG52740, &UA2773527},
	{SG52742, &UA3021320},
	{SG52765, &UA2939140},
	{SG52771, &UA3111304},
	{SG52772, &UA3132774},
	{SG52784, &UA2770250},
	{SG52788, &UA2944410},
	{SG52794, &UA3110604},
	{SG52804, &UA2630939},
	{SG52815, &UA3054514},
	{SG52818, &UA3115557},
	{SG52831, &UA2855480},
	{SG52835, &UA2733038},
	{SG52836, &UA2860769},
	{SG52837, &UA3061203},
	{SG52840, &UA2967268},
	{SG52841, &UA3081227},
	{SG52848, &UA2768645},
	{SG52850, &UA2878999},
	{SG52852, &UA2782628},
	{SG52855, &UA3082754},
	{SG52859, &UA2600580},
	{SG52894, &UA2706678},
	{SG52895, &UA2954135},
	{SG52902, &UA2879187},
	{SG52903, &UA3023033},
	{SG52905, &UA3049631},
	{SG52912, &UA3101625},
	{SG52916, &UA2782352},
	{SG52918, &UA3003691},
	{SG52919, &UA3122864},
	{SG52922, &UA3081285},
	{SG52937, &UA2676656},
	{SG52940, &UA2752628},
	{SG52944, &UA2796348},
	{SG52950, &UA3024150},
	{SG52951, &UA3049176},
	{SG52988, &UA2988274},
	{SG52991, &UA2733219},
	{SG52994, &UA2752369},
	{SG52997, &UA2799009},
	{SG52998, &UA2896090},
	{SG53000, &UA3016608},
	{SG53001, &UA3061302},
	{SG53003, &UA3094768},
	{SG53004, &UA3083371},
	{SG53014, &UA2603023},
	{SG53015, &UA3064592},
	{SG53044, &UA2944645},
	{SG53069, &UA2665913},
	{SG53070, &UA3073672},
	{SG53075, &UA3021380},
	{SG53077, &UA3083586},
	{SG53078, &UA3094496},
	{SG53096, &UA3017551},
	{SG53097, &UA3029486},
	{SG53103, &UA2632478},
	{SG53104, &UA2710426},
	{SG53105, &UA2737721},
	{SG53106, &UA2935781},
	{SG53108, &UA3130946},
	{SG53142, &UA2583346},
	{SG53147, &UA2675077},
	{SG53149, &UA2709750},
	{SG53151, &UA2713723},
	{SG53155, &UA2867459},
	{SG53156, &UA2892758},
	{SG53157, &UA3021060},
	{SG53172, &UA2707938},
	{SG53174, &UA3003465},
	{SG53180, &UA3023375},
	{SG53197, &UA3015131},
	{SG53199, &UA3013005},
	{SG53200, &UA3021431},
	{SG53201, &UA3060417},
	{SG53203, &UA3130583},
	{SG53209, &UA2799331},
	{SG53213, &UA3033255},
	{SG53217, &UA2674739},
	{SG53218, &UA2734252},
	{SG53221, &UA2800236},
	{SG53224, &UA2938430},
	{SG53230, &UA3096890},
	{SG53231, &UA3129306},
	{SG53250, &UA2709598},
	{SG53258, &UA2956719},
	{SG53300, &UA2998201},
	{SG53302, &UA3011976},
	{SG53304, &UA3049828},
	{SG53305, &UA3063290},
	{SG53322, &UA2799575},
	{SG53329, &UA3033549},
	{SG53388, &UA3021656},
	{SG53391, &UA3079313},
	{SG53399, &UA2977081},
	{SG53401, &UA2977083},
	{SG53405, &UA3079519},
	{SG53407, &UA3081033},
	{SG53419, &UA2631219},
	{SG53422, &UA2783985},
	{SG53423, &UA3058122},
	{SG53424, &UA2900571},
	{SG53425, &UA2958678},
	{SG53426, &UA3025167},
	{SG53427, &UA3065060},
	{SG53441, &UA2854741},
	{SG53448, &UA2987154},
	{SG53449, &UA3056710},
	{SG53450, &UA3030592},
	{SG53452, &UA2672314},
	{SG53454, &UA2869609},
	{SG53461, &UA3078320},
	{SG53464, &UA3080435},
	{SG53478, &UA3023539},
	{SG53481, &UA3060580},
	{SG53483, &UA3109243},
	{SG53484, &UA3111523},
	{SG53485, &UA3113441},
	{SG53486, &UA3110123},
	{SG53493, &UA2699208},
	{SG53498, &UA2769886},
	{SG53499, &UA2771240},
	{SG53500, &UA2774401},
	{SG53506, &UA2944117},
	{SG53507, &UA3065361},
	{SG53534, &UA2635230},
	{SG53538, &UA2784766},
	{SG53543, &UA3007877},
	{SG53544, &UA2583121},
	{SG53546, &UA2705505},
	{SG53547, &UA2892780},
	{SG53548, &UA2892943},
	{SG53549, &UA3017072},
	{SG53551, &UA3064150},
	{SG53553, &UA3058769},
	{SG53556, &UA3108049},
	{SG53558, &UA3125823},
	{SG53564, &UA2738443},
	{SG53565, &UA2738248},
	{SG53567, &UA2779696},
	{SG53575, &UA3095529},
	{SG53576, &UA3079227},
	{SG53577, &UA3116525},
	{SG53585, &UA2634649},
	{SG53586, &UA2637652},
	{SG53587, &UA2738081},
	{SG53588, &UA2780912},
	{SG53589, &UA3087433},
	{SG53591, &UA2956917},
	{SG53592, &UA2962197},
	{SG53600, &UA3058734},
	{SG53601, &UA3094366},
	{SG53606, &UA2706839},
	{SG53607, &UA2779902},
	{SG53609, &UA2892312},
	{SG53613, &UA3022155},
	{SG53625, &UA2604375},
	{SG53626, &UA2716107},
	{SG53627, &UA2752226},
	{SG53629, &UA2766198},
	{SG53630, &UA2782846},
	{SG53632, &UA3013824},
	{SG53635, &UA3131679},
	{SG53676, &UA2875251},
	{SG53677, &UA2891761},
	{SG53678, &UA2939553},
	{SG53679, &UA3026694},
	{SG53680, &UA3056705},
	{SG53684, &UA2799023},
	{SG53685, &UA2962513},
	{SG53687, &UA3023416},
	{SG53690, &UA3093959},
	{SG53691, &UA3132942},
	{SG53698, &UA2666708},
	{SG53699, &UA2702811},
	{SG53700, &UA3007922},
	{SG53703, &UA2784001},
	{SG53705, &UA2867130},
	{SG53706, &UA2869413},
	{SG53714, &UA3095743},
	{SG53716, &UA3111892},
	{SG53728, &UA2793480},
	{SG53730, &UA2583185},
	{SG53735, &UA2671113},
	{SG53742, &UA2782815},
	{SG53772, &UA3095522},
	{SG53779, &UA3094135},
	{SG53784, &UA3115114},
	{SG53824, &UA2583967},
	{SG53831, &UA2631467},
	{SG53836, &UA2668866},
	{SG53844, &UA2735013},
	{SG53846, &UA2797299},
	{SG53847, &UA2798902},
	{SG53848, &UA3011511},
	{SG53849, &UA3023021},
	{SG53850, &UA3023237},
	{SG53851, &UA3059107},
	{SG53853, &UA3115504},
	{SG53863, &UA2602995},
	{SG53864, &UA2667995},
	{SG53871, &UA2667033},
	{SG53876, &UA2867231},
	{SG53879, &UA2936773},
	{SG53880, &UA3072117},
	{SG53881, &UA2960652},
	{SG53886, &UA3023471},
	{SG53912, &UA3062100},
	{SG53913, &UA2795886},
	{SG53918, &UA2937900},
	{SG53919, &UA2868103},
	{SG53922, &UA3056154},
	{SG54004, &UA2764468},
	{SG54007, &UA3055986},
	{SG54019, &UA2937765},
	{SG54022, &UA2977102},
	{SG54023, &UA3019811},
	{SG54061, &UA3057533},
	{SG54062, &UA3121006},
	{SG54063, &UA3114930},
	{SG54065, &UA3113026},
	{SG54066, &UA3114552},
	{SG54142, &UA2737448},
	{SG54150, &UA2782255},
	{SG54159, &UA2956341},
	{SG54160, &UA3015827},
	{SG54166, &UA2635376},
	{SG54169, &UA2707579},
	{SG54177, &UA2676602},
	{SG54183, &UA2708968},
	{SG54186, &UA2734658},
	{SG54216, &UA3022119},
	{SG54220, &UA3106532},
	{SG54235, &UA2583567},
	{SG54236, &UA2584856},
	{SG54239, &UA2768227},
	{SG54246, &UA2937519},
	{SG54247, &UA2991844},
	{SG54248, &UA2996787},
	{SG54253, &UA3131801},
	{SG54274, &UA2602440},
	{SG54286, &UA3061505},
	{SG54288, &UA3132908},
	{SG54311, &UA2631749},
	{SG54314, &UA2877660},
	{SG54315, &UA2878805},
	{SG54316, &UA2953683},
	{SG54317, &UA3023382},
	{SG54318, &UA3027308},
	{SG54319, &UA3115501},
	{SG54346, &UA3021837},
	{SG54348, &UA3111554},
	{SG54349, &UA3109962},
	{SG54350, &UA3114668},
	{SG54351, &UA3132861},
	{SG54352, &UA3132951},
	{SG54355, &UA2870020},
	{SG54356, &UA2892607},
	{SG54357, &UA2892860},
	{SG54364, &UA3024227},
	{SG54367, &UA3059098},
	{SG54369, &UA3059710},
	{SG54370, &UA3061262},
	{SG54386, &UA2714192},
	{SG54387, &UA2801264},
	{SG54389, &UA3020625},
	{SG54390, &UA3058356},
	{SG54391, &UA3051043},
	{SG54392, &UA3090596},
	{SG54393, &UA3130956},
	{SG54401, &UA2890995},
	{SG54404, &UA2705226},
	{SG54413, &UA3017729},
	{SG54414, &UA3023418},
	{SG54415, &UA3049018},
	{SG54417, &UA3123343},
	{SG54442, &UA2604121},
	{SG54453, &UA2708615},
	{SG54463, &UA2782578},
	{SG54476, &UA2962345},
	{SG54483, &UA3003437},
	{SG54488, &UA3016606},
	{SG54491, &UA3020293},
	{SG54499, &UA3058922},
	{SG54502, &UA3061552},
	{SG54505, &UA3127015},
	{SG54506, &UA3131759},
	{SG54507, &UA2712730},
	{SG54508, &UA2797333},
	{SG54511, &UA2877879},
	{SG54517, &UA3095255},
	{SG54519, &UA3131317},
	{SG54532, &UA2735544},
	{SG54534, &UA2798393},
	{SG54535, &UA2854149},
	{SG54536, &UA2868983},
	{SG54538, &UA2879197},
	{SG54540, &UA2892366},
	{SG54542, &UA2958153},
	{SG54546, &UA3007970},
	{SG54549, &UA3109426},
	{SG54559, &UA2600207},
	{SG54562, &UA2602330},
	{SG54595, &UA2798626},
	{SG54601, &UA3049635},
	{SG54608, &UA2877867},
	{SG54609, &UA2994811},
	{SG54610, &UA3113903},
	{SG54611, &UA3132600},
	{SG54612, &UA3132941},
	{SG54613, &UA2667031},
	{SG54617, &UA2709135},
	{SG54622, &UA2867499},
	{SG54624, &UA2869311},
	{SG54625, &UA2876093},
	{SG54628, &UA2879343},
	{SG54629, &UA2879634},
	{SG54630, &UA2890205},
	{SG54634, &UA2957410},
	{SG54636, &UA3021227},
	{SG54637, &UA3023535},
	{SG54640, &UA3057282},
	{SG54641, &UA3059637},
	{SG54644, &UA3079803},
	{SG54645, &UA3096444},
	{SG54685, &UA2634074},
	{SG54686, &UA2662734},
	{SG54691, &UA2861865},
	{SG54692, &UA3056526},
	{SG54693, &UA2865151},
	{SG54706, &UA3021160},
	{SG54711, &UA3022217},
	{SG54724, &UA3110257},
	{SG54728, &UA3110502},
	{SG54762, &UA2601597},
	{SG54763, &UA2605735},
	{SG54769, &UA2779616},
	{SG54772, &UA2878833},
	{SG54778, &UA2944651},
	{SG54786, &UA2998791},
	{SG54796, &UA3022571},
	{SG54800, &UA3039383},
	{SG54813, &UA3122753},
	{SG54817, &UA3131310},
	{SG54819, &UA3131816},
	{SG54823, &UA2666759},
	{SG54824, &UA2782811},
	{SG54829, &UA3023400},
	{SG54830, &UA3023621},
	{SG54833, &UA3108823},
	{SG54878, &UA2868168},
	{SG54879, &UA2869506},
	{SG54882, &UA3109658},
	{SG54883, &UA3110849},
	{SG54884, &UA2631256},
	{SG54887, &UA2944318},
	{SG54888, &UA2958079},
	{SG54889, &UA2970167},
	{SG54890, &UA3015043},
	{SG54891, &UA3023926},
	{SG54892, &UA3103411},
	{SG54894, &UA3111516},
	{SG54895, &UA3111556},
	{SG54896, &UA2855553},
	{SG54898, &UA2877965},
	{SG54899, &UA2956596},
	{SG54900, &UA2964918},
	{SG54901, &UA2995504},
	{SG54902, &UA3023842},
	{SG54903, &UA3078735},
	{SG54904, &UA3110119},
	{SG54905, &UA3113801},
	{SG54906, &UA3131442},
	{SG54908, &UA2866928},
	{SG54909, &UA2867849},
	{SG54910, &UA2869796},
	{SG54911, &UA2879189},
	{SG54912, &UA2879248},
	{SG54913, &UA3057482},
	{SG54915, &UA3059293},
	{SG54917, &UA3079488},
	{SG54920, &UA2708910},
	{SG54921, &UA2734887},
	{SG54922, &UA2966181},
	{SG54926, &UA3049618},
	{SG54927, &UA3061786},
	{SG54928, &UA3078048},
	{SG54929, &UA3079882},
	{SG54930, &UA2713176},
	{SG54931, &UA2735573},
	{SG54937, &UA2938905},
	{SG54938, &UA3009428},
	{SG54940, &UA3020517},
	{SG54941, &UA3021225},
	{SG54942, &UA3021787},
	{SG54943, &UA3022839},
	{SG54944, &UA3023216},
	{SG54945, &UA3051285},
	{SG54946, &UA3056938},
	{SG54947, &UA3079800},
	{SG54948, &UA3092066},
	{SG54949, &UA3110314},
	{SG54950, &UA3115063},
	{SG54951, &UA2667624},
	{SG54952, &UA2892236},
	{SG54953, &UA2938229},
	{SG54956, &UA3013379},
	{SG54958, &UA3018189},
	{SG54959, &UA3023110},
	{SG54960, &UA3057731},
	{SG54961, &UA3059720},
	{SG54962, &UA3079165},
	{SG54963, &UA3110428},
	{SG54964, &UA3114915},
	{SG54965, &UA2994642},
	{SG54966, &UA2667280},
	{SG54968, &UA2749864},
	{SG54970, &UA2866371},
	{SG54971, &UA2879034},
	{SG54972, &UA2937115},
	{SG54973, &UA2943895},
	{SG54974, &UA2959732},
	{SG54975, &UA3001901},
	{SG54976, &UA3016278},
	{SG54977, &UA3048896},
	{SG54978, &UA3080234},
	{SG54979, &UA3093192},
	{SG54980, &UA2703518},
	{SG54981, &UA2879396},
	{SG54982, &UA2953525},
	{SG54983, &UA2990109},
	{SG54984, &UA2999333},
	{SG54985, &UA3021895},
	{SG54986, &UA3022332},
	{SG54987, &UA3030756},
	{SG54988, &UA3048619},
	{SG54989, &UA3078644},
	{SG54990, &UA3131038},
	{SG54991, &UA3091879},
	{SG54992, &UA3092896},
	{SG54993, &UA3132846},
	{SG54998, &UA2635093},
	{SG54999, &UA2635667},
	{SG55000, &UA2713810},
	{SG55003, &UA2751553},
	{SG55004, &UA2896426},
	{SG55005, &UA2995455},
	{SG55006, &UA2996683},
	{SG55007, &UA2998668},
	{SG55008, &UA3000327},
	{SG55009, &UA3016231},
	{SG55011, &UA3022225},
	{SG55012, &UA3023701},
	{SG55018, &UA3091837},
	{SG55019, &UA3110293},
	{SG55020, &UA3110986},
	{SG55021, &UA3111911},
	{SG55022, &UA2750125},
	{SG55026, &UA3016573},
	{SG55027, &UA3021700},
	{SG55028, &UA3021971},
	{SG55029, &UA3057026},
	{SG55030, &UA3057526},
	{SG55031, &UA3057719},
	{SG55032, &UA3101140},
	{SG55037, &UA2749876},
	{SG55040, &UA2861720},
	{SG55041, &UA2865683},
	{SG55042, &UA2869639},
	{SG55043, &UA2892546},
	{SG55044, &UA2892569},
	{SG55045, &UA2937500},
	{SG55046, &UA2944958},
	{SG55048, &UA2989041},
	{SG55049, &UA2997648},
	{SG55050, &UA3012737},
	{SG55051, &UA3056620},
	{SG55052, &UA3058116},
	{SG55054, &UA3078773},
	{SG55056, &UA3108623},
	{SG55058, &UA3111015},
	{SG55060, &UA3126469},
	{SG55061, &UA3127468},
	{SG55113, &UA2939202},
	{SG55114, &UA2944270},
	{SG55117, &UA2996932},
	{SG55122, &UA3021972},
	{SG55126, &UA3058544},
	{SG55127, &UA3059006},
	{SG55131, &UA3102279},
	{SG55132, &UA3108037},
	{SG55134, &UA3110710},
	{SG55135, &UA3111688},
	{SG55137, &UA3132858},
	{SG55150, &UA2707100},
	{SG55151, &UA2750798},
	{SG55158, &UA2893669},
	{SG55160, &UA2954432},
	{SG55161, &UA2971559},
	{SG55163, &UA2997238},
	{SG55167, &UA3021350},
	{SG55169, &UA3021539},
	{SG55170, &UA3023528},
	{SG55171, &UA3024920},
	{SG55172, &UA3045378},
	{SG55177, &UA3076600},
	{SG55188, &UA3092784},
	{SG55198, &UA3107801},
	{SG55211, &UA3124923},
	{SG55213, &UA3126601},
	{SG55234, &UA2767437},
	{SG55236, &UA2767620},
	{SG55248, &UA2858372},
	{SG55256, &UA2863092},
	{SG55290, &UA2892638},
	{SG55292, &UA2937131},
	{SG55295, &UA2953386},
	{SG55298, &UA2957396},
	{SG55299, &UA2988667},
	{SG55301, &UA2999063},
	{SG55303, &UA3019126},
	{SG55304, &UA3021374},
	{SG55305, &UA3024112},
	{SG55307, &UA3057808},
	{SG55309, &UA3071749},
	{SG55310, &UA3078706},
	{SG55311, &UA3079558},
	{SG55313, &UA3088547},
	{SG55315, &UA3106274},
	{SG55321, &UA3109076},
	{SG55322, &UA3109350},
	{SG55323, &UA3112100},
	{SG55325, &UA3125144},
	{SG55327, &UA3126481},
	{SG55342, &UA2637857},
	{SG55343, &UA2869259},
	{SG55344, &UA2961640},
	{SG55345, &UA2977111},
	{SG55346, &UA2997048},
	{SG55347, &UA3018484},
	{SG55348, &UA3019226},
	{SG55349, &UA3077788},
	{SG55350, &UA3078019},
	{SG55351, &UA3091967},
	{SG55352, &UA3092166},
	{SG55353, &UA3092614},
	{SG55354, &UA3092719},
	{SG55355, &UA3104074},
	{SG55356, &UA3109156},
	{SG55357, &UA3125555},
	{SG55358, &UA3125897},
	{SG55359, &UA3125920},
	{SG55360, &UA3126268},
	{SG55361, &UA3126392},
	{SG55362, &UA3128428},
	{SG55388, &UA2631949},
	{SG55391, &UA2632150},
	{SG55403, &UA2751362},
	{SG55427, &UA2937993},
	{SG55435, &UA3009532},
	{SG55436, &UA3049668},
	{SG55437, &UA3063305},
	{SG55438, &UA3077965},
	{SG55439, &UA3087625},
	{SG55440, &UA3089111},
	{SG55441, &UA3093045},
	{SG55442, &UA3106124},
	{SG55476, &UA3126400},
	{SG55499, &UA3131384},
	{SG55648, &UA2735153},
	{SG55704, &UA2859941},
	{SG55715, &UA2896967},
	{SG55716, &UA2937941},
	{SG55730, &UA3001049},
	{SG55736, &UA3018852},
	{SG55748, &UA3056377},
	{SG55749, &UA3058983},
	{SG55751, &UA3073331},
	{SG55768, &UA3101874},
	{SG55780, &UA3103165},
	{SG55792, &UA3106579},
	{SG55800, &UA3107334},
	{SG55814, &UA3108221},
	{SG55842, &UA3120829},
	{SG55852, &UA3121523},
	{SG55873, &UA3125029},
	{SG55877, &UA3125052},
	{SG55904, &UA3126135},
	{SG55908, &UA3126263},
	{SG55911, &UA3126339},
	{SG55918, &UA3126710},
	{SG55923, &UA3126885},
	{SG55932, &UA3127163},
	{SG55939, &UA3127615},
	{SG55943, &UA3131402},
	{SG55988, &UA2865029},
	{SG56078, &UA2855769},
	{SG56088, &UA2864138},
	{SG56092, &UA2864937},
	{SG56095, &UA2865754},
	{SG56099, &UA2866395},
	{SG56263, &UA2631421},
	{SG56425, &UA2977110},
	{SG56427, &UA2977115},
	{SG56680, &UA2632820},
	{SG56681, &UA2633063},
	{SG56682, &UA2632653},
	{SG56683, &UA2977119},
	{SG56684, &UA2634827},
	{SG56685, &UA2634290},
	{SG56693, &UA2607535},
	{SG56694, &UA2635823},
	{SG56695, &UA2774416},
	{SG56696, &UA2977093},
	{SG56697, &UA2977122},
	{SG56698, &UA2977123},
	{SG56699, &UA2634615},
	{SG56700, &UA2632408},
	{SG56701, &UA2633269},
	{SG56702, &UA2633713},
	{SG56703, &UA2977118},
	{SG56704, &UA2977080},
	{SG56705, &UA2977104},
	{SG56706, &UA2977106},
	{SG56707, &UA3084064},
	{SG56713, &UA2632654},
	{SG56714, &UA2635787},
	{SG56715, &UA2635080},
	{SG56716, &UA2774402},
	{SG56717, &UA3068981},
	{SG56718, &UA2637978},
	{SG56719, &UA2634969},
	{SG56720, &UA2634651},
	{SG56721, &UA2637805},
	{SG56722, &UA2719661},
	{SG56723, &UA3068937},
	{SG56724, &UA2632385},
	{SG56725, &UA2632454},
	{SG56726, &UA2632676},
	{SG56727, &UA2634249},
	{SG56728, &UA2634888},
	{SG56731, &UA2635574},
	{SG56741, &UA2636376},
	{SG56744, &UA2636428},
	{SG56747, &UA2636503},
	{SG56754, &UA2636925},
	{SG56761, &UA2637300},
	{SG56765, &UA2719588},
	{SG56766, &UA2719702},
	{SG56767, &UA2877950},
	{SG56768, &UA2977073},
	{SG56769, &UA2977079},
	{SG56770, &UA2977086},
	{SG56771, &UA2977092},
	{SG56772, &UA2977107},
	{SG56773, &UA3024026},
	{SG56792, &UA3068928},
	{SG56795, &UA3068932},
	{SG56803, &UA3084074},
	{SG56810, &UA3084109},
	{NULL}};
const Device *DAppleCoreMedia[] = {
	&UA94715,
	&UA109634,
	&UA150766,
	&UA160226,
	&UA168294,
	&UA176876,
	&UA177248,
	&UA181544,
	&UA189427,
	&UA196029,
	&UA197798,
	&UA235394,
	&UA391451,
	&UA448132,
	&UA812363,
	&UA251253,
	&UA398524,
	&UA398585,
	&UA630479,
	&UA634493,
	&UA640986,
	&UA668209,
	&UA1989389,
	&UA2161420,
	&UA2164629,
	&UA2195613,
	&UA2205972,
	&UA2237419,
	&UA2421069,
	&UA2455381,
	&UA1970112,
	&UA2186295,
	&UA2200542,
	&UA2231047,
	&UA2404344,
	&UA2458314,
	&UA2466673,
	&UA2502202,
	&UA2557998,
	&UA2584146,
	&UA2799901,
	&UA3001780,
	&UA3002282,
	&UA3088849,
	&UA3096134,
	&UA2352124,
	&UA2457936,
	&UA2668432,
	&UA2933312,
	&UA3008739,
	&UA2525551,
	&UA3098809,
	&UA86795,
	&UA88545,
	NULL};
const Device *DAudioVox[] = {
	&UA80130,
	&UA80131,
	&UA80128,
	NULL};
const Segmentdata DApple[] = {
	{SG53744, &UA2359519},
	{SG53745, &UA2391468},
	{SG53746, &UA2437254},
	{SG53748, &UA2482346},
	{SG53749, &UA2486731},
	{SG53750, &UA2487675},
	{SG53751, &UA2487691},
	{SG53752, &UA2492474},
	{SG53753, &UA2509670},
	{SG53755, &UA2509855},
	{SG53756, &UA2509953},
	{SG53757, &UA2511228},
	{SG53759, &UA2511932},
	{SG53761, &UA2512576},
	{SG53762, &UA2513077},
	{SG53764, &UA2515658},
	{SG53765, &UA2516521},
	{SG53766, &UA2531290},
	{SG53767, &UA2531377},
	{SG53768, &UA2532340},
	{SG53769, &UA2532507},
	{SG53770, &UA2532639},
	{SG53771, &UA2532873},
	{SG53773, &UA2533028},
	{SG53774, &UA2533403},
	{SG53775, &UA2535185},
	{SG53776, &UA2536609},
	{SG53777, &UA2537705},
	{SG53778, &UA2538125},
	{SG53780, &UA2560604},
	{SG53781, &UA2561130},
	{SG53782, &UA2561419},
	{SG53783, &UA2562684},
	{SG53785, &UA2563717},
	{SG53786, &UA2563853},
	{SG53787, &UA2563878},
	{SG53788, &UA2564403},
	{SG53789, &UA2564407},
	{SG53790, &UA2567626},
	{SG53791, &UA2570099},
	{SG53792, &UA2570332},
	{SG53793, &UA2583078},
	{SG53794, &UA2584024},
	{SG53795, &UA2585369},
	{SG53796, &UA2601719},
	{SG53797, &UA2602045},
	{SG53798, &UA2603775},
	{SG53799, &UA2606071},
	{SG53800, &UA2606839},
	{SG53801, &UA2664714},
	{SG53802, &UA2668734},
	{SG53803, &UA2669356},
	{SG53804, &UA2669734},
	{SG53805, &UA2670073},
	{SG53806, &UA2670659},
	{SG53807, &UA2672716},
	{SG53808, &UA2675062},
	{SG53809, &UA2675389},
	{SG53810, &UA2709288},
	{SG53811, &UA2713971},
	{SG53812, &UA2715915},
	{SG53813, &UA2715933},
	{SG53814, &UA2738109},
	{SG53815, &UA2752750},
	{SG53816, &UA2752853},
	{SG53817, &UA2769454},
	{SG53818, &UA2779663},
	{SG53819, &UA2780748},
	{SG53820, &UA2799238},
	{SG53821, &UA2850848},
	{SG53822, &UA2852846},
	{SG53823, &UA2854150},
	{SG53825, &UA2869589},
	{SG53826, &UA2872520},
	{SG53827, &UA2877394},
	{SG53828, &UA2880727},
	{SG53829, &UA2881312},
	{SG53830, &UA2881572},
	{SG53832, &UA2896694},
	{SG53833, &UA2898498},
	{SG53834, &UA2937999},
	{SG53835, &UA2958302},
	{SG53837, &UA2992709},
	{SG53838, &UA2992863},
	{SG53839, &UA3004160},
	{SG53840, &UA3006641},
	{SG53841, &UA3014309},
	{SG53842, &UA3023511},
	{SG53843, &UA3024472},
	{SG53845, &UA3073621},
	{SG53927, &UA895501},
	{SG53928, &UA908825},
	{SG53929, &UA913409},
	{SG53930, &UA1993140},
	{SG53931, &UA2012818},
	{SG53932, &UA2040632},
	{SG53934, &UA2043451},
	{SG53935, &UA2200096},
	{SG53936, &UA2202637},
	{SG53937, &UA2204693},
	{SG53938, &UA2205805},
	{SG53940, &UA2231040},
	{SG53941, &UA2231050},
	{SG53942, &UA2231149},
	{SG53943, &UA2232693},
	{SG53944, &UA2233623},
	{SG53945, &UA2234300},
	{SG53946, &UA2236014},
	{SG53947, &UA2260994},
	{SG53948, &UA2261035},
	{SG53949, &UA2261066},
	{SG53950, &UA2261077},
	{SG53951, &UA2261155},
	{SG53952, &UA2261343},
	{SG53954, &UA2261351},
	{SG53955, &UA2261359},
	{SG53956, &UA2356810},
	{SG53957, &UA2356816},
	{SG53958, &UA2357212},
	{SG53959, &UA2363033},
	{SG53960, &UA2368553},
	{SG53961, &UA2386324},
	{SG53963, &UA2386403},
	{SG53964, &UA2390486},
	{SG53965, &UA2404527},
	{SG53967, &UA2404782},
	{SG53968, &UA2426599},
	{SG53969, &UA2428242},
	{SG53970, &UA2428494},
	{SG53972, &UA2429477},
	{SG53973, &UA2429904},
	{SG53975, &UA2430746},
	{SG53976, &UA2431096},
	{SG53978, &UA2436623},
	{SG53979, &UA2437553},
	{SG53981, &UA2437623},
	{SG53982, &UA2437648},
	{SG53984, &UA2455484},
	{SG53986, &UA2456156},
	{SG53987, &UA2460420},
	{SG53988, &UA2460613},
	{SG53989, &UA2461223},
	{SG53990, &UA2469395},
	{SG53992, &UA2469518},
	{SG53994, &UA2469886},
	{SG53995, &UA2469890},
	{SG53997, &UA2470556},
	{SG53998, &UA2470555},
	{SG54000, &UA2486630},
	{SG54001, &UA2487059},
	{SG54002, &UA2487100},
	{SG54003, &UA2487230},
	{SG54005, &UA2553685},
	{SG54006, &UA2578484},
	{SG54009, &UA2597496},
	{SG54010, &UA2656725},
	{SG54011, &UA2657834},
	{SG54012, &UA2762592},
	{SG54013, &UA2840517},
	{SG54014, &UA2847819},
	{SG54015, &UA2848878},
	{SG54016, &UA2849415},
	{SG54028, &UA2362699},
	{SG54029, &UA2408165},
	{SG54031, &UA2437189},
	{SG54033, &UA2482242},
	{SG54035, &UA2490822},
	{SG54037, &UA2491609},
	{SG54038, &UA2492696},
	{SG54040, &UA2508014},
	{SG54041, &UA2509291},
	{SG54042, &UA2510462},
	{SG54043, &UA2510726},
	{SG54044, &UA2511915},
	{SG54045, &UA2513022},
	{SG54048, &UA2513220},
	{SG54050, &UA2513847},
	{SG54051, &UA2516586},
	{SG54052, &UA2530910},
	{SG54054, &UA2532514},
	{SG54055, &UA2532859},
	{SG54077, &UA2533724},
	{SG54078, &UA2533728},
	{SG54079, &UA2535163},
	{SG54080, &UA2535618},
	{SG54081, &UA2537126},
	{SG54082, &UA2538620},
	{SG54083, &UA2539701},
	{SG54084, &UA2539841},
	{SG54085, &UA2558714},
	{SG54086, &UA2560632},
	{SG54087, &UA2564961},
	{SG54088, &UA2570507},
	{SG54089, &UA2570637},
	{SG54090, &UA2570903},
	{SG54091, &UA2570993},
	{SG54092, &UA2571071},
	{SG54093, &UA2582179},
	{SG54094, &UA2582467},
	{SG54095, &UA2582734},
	{SG54096, &UA2583010},
	{SG54098, &UA2583335},
	{SG54099, &UA2584751},
	{SG54100, &UA2585862},
	{SG54101, &UA2600959},
	{SG54102, &UA2601670},
	{SG54103, &UA2606450},
	{SG54104, &UA2606826},
	{SG54106, &UA2606982},
	{SG54107, &UA2607276},
	{SG54109, &UA2607431},
	{SG54111, &UA2607443},
	{SG54113, &UA2699214},
	{SG54114, &UA2700724},
	{SG54115, &UA2703224},
	{SG54117, &UA2733258},
	{SG54118, &UA2738923},
	{SG54120, &UA2778623},
	{SG54121, &UA2779768},
	{SG54123, &UA2779965},
	{SG54125, &UA2780228},
	{SG54126, &UA2850680},
	{SG54127, &UA2851036},
	{SG54129, &UA2851065},
	{SG54131, &UA2880284},
	{SG54132, &UA2882035},
	{SG54133, &UA2893977},
	{SG54135, &UA2898529},
	{SG54136, &UA2898545},
	{SG54137, &UA2935924},
	{SG54138, &UA2939388},
	{SG54139, &UA2942323},
	{SG54140, &UA2954853},
	{SG54141, &UA2959463},
	{SG54143, &UA2966733},
	{SG54144, &UA2973504},
	{SG54145, &UA2992611},
	{SG54146, &UA2999000},
	{SG54147, &UA3014052},
	{SG54148, &UA3020820},
	{SG54149, &UA3053065},
	{SG54151, &UA3072377},
	{SG54152, &UA3074303},
	{SG54153, &UA3083507},
	{SG54154, &UA3086697},
	{SG54155, &UA3087170},
	{SG54156, &UA3090113},
	{SG54157, &UA3102332},
	{SG54161, &UA347418},
	{SG54162, &UA422684},
	{SG54163, &UA441497},
	{SG54164, &UA599671},
	{SG54165, &UA615691},
	{SG54168, &UA616340},
	{SG54170, &UA616502},
	{SG54171, &UA623272},
	{SG54172, &UA661684},
	{SG54173, &UA808970},
	{SG54174, &UA852574},
	{SG54175, &UA855729},
	{SG54176, &UA868873},
	{SG54178, &UA874317},
	{SG54179, &UA896217},
	{SG54180, &UA899392},
	{SG54181, &UA899718},
	{SG54182, &UA1969673},
	{SG54184, &UA1994832},
	{SG54185, &UA2159175},
	{SG54187, &UA2199601},
	{SG54188, &UA2234680},
	{SG54189, &UA2234996},
	{SG54190, &UA2353318},
	{SG54191, &UA2363300},
	{SG54192, &UA2394002},
	{SG54193, &UA2402191},
	{SG54194, &UA2420781},
	{SG54195, &UA2422518},
	{SG54196, &UA2476431},
	{SG54197, &UA2487228},
	{SG54198, &UA2659178},
	{SG54199, &UA2729344},
	{SG54200, &UA2816697},
	{SG54201, &UA2817625},
	{SG54202, &UA2840584},
	{SG54203, &UA2970008},
	{SG54425, &UA908806},
	{SG54426, &UA908818},
	{SG54427, &UA913377},
	{SG54428, &UA1993995},
	{SG54429, &UA2012819},
	{SG54430, &UA2041264},
	{SG54431, &UA2042494},
	{SG54432, &UA2200632},
	{SG54433, &UA2204690},
	{SG54434, &UA2205569},
	{SG54435, &UA2232488},
	{SG54436, &UA2235160},
	{SG54437, &UA2260576},
	{SG54438, &UA2260579},
	{SG54439, &UA2260586},
	{SG54440, &UA2260589},
	{SG54441, &UA2260648},
	{SG54443, &UA2260647},
	{SG54444, &UA2260703},
	{SG54445, &UA2350788},
	{SG54446, &UA2356773},
	{SG54447, &UA2357092},
	{SG54448, &UA2361095},
	{SG54449, &UA2368835},
	{SG54450, &UA2383755},
	{SG54451, &UA2386551},
	{SG54452, &UA2386600},
	{SG54454, &UA2403801},
	{SG54455, &UA2403838},
	{SG54456, &UA2404168},
	{SG54457, &UA2406775},
	{SG54458, &UA2408191},
	{SG54459, &UA2421372},
	{SG54460, &UA2421640},
	{SG54461, &UA2421769},
	{SG54462, &UA2422471},
	{SG54464, &UA2424875},
	{SG54465, &UA2427302},
	{SG54466, &UA2427896},
	{SG54467, &UA2428850},
	{SG54468, &UA2428926},
	{SG54469, &UA2429490},
	{SG54470, &UA2429983},
	{SG54471, &UA2430515},
	{SG54472, &UA2437571},
	{SG54473, &UA2437597},
	{SG54474, &UA2437601},
	{SG54475, &UA2456680},
	{SG54477, &UA2469003},
	{SG54478, &UA2469339},
	{SG54479, &UA2470552},
	{SG54480, &UA2485434},
	{SG54481, &UA2487399},
	{SG54482, &UA2506746},
	{SG54484, &UA2507299},
	{SG54485, &UA2529372},
	{SG54486, &UA2556646},
	{SG54487, &UA2580547},
	{SG54489, &UA2595698},
	{SG54490, &UA2669891},
	{SG54492, &UA2694808},
	{SG54493, &UA2700712},
	{SG54494, &UA2727554},
	{SG54495, &UA2763734},
	{SG54496, &UA2764160},
	{SG54497, &UA2826817},
	{SG54498, &UA2840731},
	{SG54500, &UA2841655},
	{SG54501, &UA2861578},
	{SG54503, &UA2878159},
	{SG54504, &UA3086234},
	{SG54558, &UA90617},
	{SG54560, &UA177678},
	{SG54563, &UA177920},
	{SG54564, &UA179157},
	{SG54565, &UA179278},
	{SG54566, &UA180461},
	{SG54567, &UA190816},
	{SG54568, &UA190819},
	{SG54569, &UA194365},
	{SG54570, &UA206989},
	{SG54572, &UA223911},
	{SG54574, &UA232669},
	{SG54576, &UA236477},
	{SG54577, &UA339633},
	{SG54578, &UA376506},
	{SG54579, &UA446113},
	{SG54580, &UA549890},
	{SG54582, &UA569688},
	{SG54583, &UA635265},
	{SG54584, &UA640534},
	{SG54585, &UA648202},
	{SG54586, &UA694435},
	{SG54587, &UA721638},
	{SG54589, &UA805817},
	{SG54590, &UA842447},
	{SG54591, &UA850993},
	{SG54592, &UA860483},
	{SG54593, &UA1969498},
	{SG54594, &UA1972053},
	{SG54596, &UA2130198},
	{SG54597, &UA2142225},
	{SG54598, &UA2297415},
	{SG54599, &UA2659316},
	{SG54600, &UA2659420},
	{SG54602, &UA2699295},
	{SG54603, &UA2709415},
	{SG54604, &UA2735142},
	{SG54605, &UA2850426},
	{SG54606, &UA3043722},
	{SG54607, &UA3078896},
	{SG54649, &UA2798619},
	{SG54650, &UA2799132},
	{SG54651, &UA2874157},
	{SG54652, &UA2876899},
	{SG54653, &UA2885394},
	{SG54654, &UA2892756},
	{SG54655, &UA2944863},
	{SG54656, &UA2947243},
	{SG54657, &UA2948001},
	{SG54658, &UA3012710},
	{SG54659, &UA3023162},
	{SG54660, &UA3024925},
	{SG54661, &UA3031707},
	{SG54662, &UA3033016},
	{SG54663, &UA3033684},
	{SG54664, &UA3033714},
	{SG54665, &UA3034362},
	{SG54666, &UA3059097},
	{SG54667, &UA3061127},
	{SG54668, &UA3065203},
	{SG54669, &UA3078895},
	{SG54670, &UA3093231},
	{SG54671, &UA3094864},
	{SG54672, &UA3109306},
	{SG54673, &UA3109374},
	{SG54674, &UA3111530},
	{SG54675, &UA3111576},
	{SG54676, &UA3112216},
	{SG54677, &UA3114309},
	{SG54678, &UA3115084},
	{SG54679, &UA3115124},
	{SG54680, &UA3115705},
	{SG54681, &UA3125828},
	{SG54682, &UA3129539},
	{SG54683, &UA3130252},
	{SG54684, &UA3130363},
	{SG54687, &UA367957},
	{SG54688, &UA441525},
	{SG54689, &UA455471},
	{SG54695, &UA615696},
	{SG54697, &UA616338},
	{SG54698, &UA616339},
	{SG54699, &UA616496},
	{SG54701, &UA623096},
	{SG54702, &UA623184},
	{SG54703, &UA662578},
	{SG54704, &UA696600},
	{SG54705, &UA703801},
	{SG54707, &UA812672},
	{SG54708, &UA834507},
	{SG54709, &UA842736},
	{SG54710, &UA896659},
	{SG54712, &UA2006124},
	{SG54713, &UA2041882},
	{SG54714, &UA2155436},
	{SG54715, &UA2165626},
	{SG54716, &UA2189061},
	{SG54717, &UA2202608},
	{SG54718, &UA2205551},
	{SG54720, &UA2260521},
	{SG54721, &UA2316026},
	{SG54722, &UA2364751},
	{SG54723, &UA2366948},
	{SG54726, &UA2386215},
	{SG54727, &UA2404673},
	{SG54729, &UA2425805},
	{SG54730, &UA2429268},
	{SG54731, &UA2463149},
	{SG54733, &UA2506368},
	{SG54734, &UA2660838},
	{SG54735, &UA2735372},
	{SG54736, &UA2813185},
	{SG54737, &UA2824819},
	{SG54744, &UA2359360},
	{SG54745, &UA2368060},
	{SG54746, &UA2391465},
	{SG54747, &UA2437231},
	{SG54748, &UA2437293},
	{SG54749, &UA2460332},
	{SG54750, &UA2467983},
	{SG54751, &UA2469654},
	{SG54752, &UA2470630},
	{SG54753, &UA2471692},
	{SG54754, &UA2484384},
	{SG54773, &UA2486269},
	{SG54774, &UA2488397},
	{SG54775, &UA2508106},
	{SG54776, &UA2508280},
	{SG54777, &UA2510862},
	{SG54779, &UA2515239},
	{SG54780, &UA2515454},
	{SG54781, &UA2516358},
	{SG54782, &UA2516519},
	{SG54783, &UA2529621},
	{SG54784, &UA2531508},
	{SG54785, &UA2532929},
	{SG54787, &UA2533139},
	{SG54788, &UA2533742},
	{SG54789, &UA2534907},
	{SG54790, &UA2535114},
	{SG54791, &UA2536315},
	{SG54792, &UA2537066},
	{SG54793, &UA2538607},
	{SG54794, &UA2565895},
	{SG54795, &UA2583702},
	{SG54797, &UA2585384},
	{SG54799, &UA2596544},
	{SG54801, &UA2603831},
	{SG54802, &UA2606145},
	{SG54803, &UA2607196},
	{SG54805, &UA2705904},
	{SG54806, &UA2710027},
	{SG54807, &UA2713008},
	{SG54808, &UA2739597},
	{SG54810, &UA2752989},
	{SG54811, &UA2845971},
	{SG54812, &UA2855789},
	{SG54814, &UA2858258},
	{SG54816, &UA2884017},
	{SG54818, &UA2895193},
	{SG54820, &UA2958131},
	{SG54821, &UA2991711},
	{SG54834, &UA5625},
	{SG54835, &UA5637},
	{SG54836, &UA5689},
	{SG54837, &UA5730},
	{SG54838, &UA37159},
	{SG54839, &UA46546},
	{SG54840, &UA46548},
	{SG54841, &UA46551},
	{SG54842, &UA46550},
	{SG54843, &UA46561},
	{SG54844, &UA175803},
	{SG54845, &UA185626},
	{SG54846, &UA192004},
	{SG54847, &UA202540},
	{SG54848, &UA316743},
	{SG54849, &UA318720},
	{SG54850, &UA555815},
	{SG54851, &UA692547},
	{SG54852, &UA795188},
	{SG54853, &UA870565},
	{SG54854, &UA871896},
	{SG54855, &UA1894766},
	{SG54856, &UA1985355},
	{SG54857, &UA2141173},
	{SG54858, &UA2214030},
	{SG54859, &UA2294565},
	{SG54860, &UA2472702},
	{SG54861, &UA2505243},
	{SG54862, &UA2658354},
	{SG54863, &UA2658460},
	{SG54864, &UA2659275},
	{SG54865, &UA2660239},
	{SG54866, &UA2663477},
	{SG54867, &UA2711580},
	{SG54868, &UA2720656},
	{SG54869, &UA2800705},
	{SG54870, &UA3041022},
	{SG54871, &UA3041423},
	{SG54872, &UA3071022},
	{SG55062, &UA5629},
	{SG55063, &UA5652},
	{SG55064, &UA5654},
	{SG55065, &UA5670},
	{SG55066, &UA5688},
	{SG55067, &UA5703},
	{SG55068, &UA5719},
	{SG55069, &UA5720},
	{SG55070, &UA5724},
	{SG55071, &UA5702},
	{SG55072, &UA5740},
	{SG55073, &UA30063},
	{SG55074, &UA37154},
	{SG55075, &UA37161},
	{SG55076, &UA46553},
	{SG55077, &UA46557},
	{SG55078, &UA74148},
	{SG55079, &UA90486},
	{SG55080, &UA90718},
	{SG55081, &UA90720},
	{SG55082, &UA176081},
	{SG55083, &UA181290},
	{SG55084, &UA184912},
	{SG55085, &UA188253},
	{SG55086, &UA197706},
	{SG55087, &UA226620},
	{SG55088, &UA234873},
	{SG55089, &UA360711},
	{SG55090, &UA363556},
	{SG55091, &UA394205},
	{SG55092, &UA420020},
	{SG55093, &UA455658},
	{SG55094, &UA455812},
	{SG55095, &UA456054},
	{SG55096, &UA541766},
	{SG55097, &UA547683},
	{SG55098, &UA553553},
	{SG55099, &UA624467},
	{SG55100, &UA631268},
	{SG55101, &UA663539},
	{SG55102, &UA771061},
	{SG55103, &UA2297081},
	{SG55104, &UA2411018},
	{SG55105, &UA2419471},
	{SG55106, &UA2643045},
	{SG55107, &UA2681955},
	{SG55108, &UA2778645},
	{SG55109, &UA2835253},
	{SG55110, &UA2934148},
	{SG55111, &UA2977929},
	{SG55112, &UA3117433},
	{SG55174, &UA913446},
	{SG55175, &UA2012811},
	{SG55176, &UA2042349},
	{SG55178, &UA2178044},
	{SG55179, &UA2202433},
	{SG55180, &UA2203031},
	{SG55181, &UA2260597},
	{SG55182, &UA2260616},
	{SG55183, &UA2260650},
	{SG55184, &UA2260669},
	{SG55185, &UA2347220},
	{SG55186, &UA2356959},
	{SG55187, &UA2360621},
	{SG55189, &UA2362282},
	{SG55191, &UA2363433},
	{SG55192, &UA2426291},
	{SG55193, &UA2435908},
	{SG55194, &UA2437561},
	{SG55195, &UA2466741},
	{SG55196, &UA2693704},
	{SG55197, &UA2787399},
	{SG55199, &UA2794403},
	{SG55200, &UA2834999},
	{SG55201, &UA2839510},
	{SG55202, &UA2849675},
	{SG55203, &UA2987914},
	{SG55204, &UA2987967},
	{SG55205, &UA3042952},
	{SG55206, &UA3045350},
	{SG55207, &UA3072565},
	{SG55208, &UA3079914},
	{SG55209, &UA3098839},
	{SG55210, &UA3108770},
	{SG55216, &UA37121},
	{SG55217, &UA37141},
	{SG55218, &UA37174},
	{SG55219, &UA46471},
	{SG55220, &UA70722},
	{SG55221, &UA90980},
	{SG55222, &UA90981},
	{SG55223, &UA91000},
	{SG55224, &UA358604},
	{SG55225, &UA360481},
	{SG55226, &UA575006},
	{SG55227, &UA575234},
	{SG55228, &UA616501},
	{SG55229, &UA637320},
	{SG55230, &UA652612},
	{SG55231, &UA691829},
	{SG55232, &UA692955},
	{SG55233, &UA725819},
	{SG55235, &UA792479},
	{SG55237, &UA2215059},
	{SG55238, &UA2221249},
	{SG55239, &UA2304190},
	{SG55240, &UA2395525},
	{SG55241, &UA2418787},
	{SG55242, &UA2505723},
	{SG55243, &UA2520759},
	{SG55244, &UA2597943},
	{SG55245, &UA2659318},
	{SG55246, &UA2839104},
	{SG55247, &UA2012036},
	{SG55249, &UA2012809},
	{SG55250, &UA2034642},
	{SG55251, &UA2034643},
	{SG55252, &UA2040848},
	{SG55253, &UA2201601},
	{SG55254, &UA2201792},
	{SG55255, &UA2201942},
	{SG55257, &UA2237046},
	{SG55258, &UA2237417},
	{SG55259, &UA2261058},
	{SG55260, &UA2261167},
	{SG55261, &UA2341377},
	{SG55262, &UA2341471},
	{SG55263, &UA2351831},
	{SG55264, &UA2356728},
	{SG55265, &UA2356937},
	{SG55266, &UA2357077},
	{SG55267, &UA2386639},
	{SG55268, &UA2437557},
	{SG55269, &UA2456492},
	{SG55270, &UA2457213},
	{SG55271, &UA2460879},
	{SG55272, &UA2505647},
	{SG55273, &UA2507161},
	{SG55274, &UA2525660},
	{SG55275, &UA2528532},
	{SG55276, &UA2538627},
	{SG55277, &UA2595572},
	{SG55278, &UA2653134},
	{SG55279, &UA2746852},
	{SG55280, &UA2779429},
	{SG55281, &UA2779735},
	{SG55282, &UA2840908},
	{SG55283, &UA2859540},
	{SG55284, &UA2874248},
	{SG55285, &UA2934848},
	{SG55286, &UA2952407},
	{SG55287, &UA2952633},
	{SG55288, &UA3042913},
	{SG55289, &UA3088004},
	{SG55363, &UA90485},
	{SG55364, &UA90570},
	{SG55365, &UA167931},
	{SG55366, &UA180440},
	{SG55367, &UA180456},
	{SG55368, &UA180505},
	{SG55369, &UA180773},
	{SG55370, &UA181257},
	{SG55371, &UA182631},
	{SG55372, &UA182850},
	{SG55373, &UA190951},
	{SG55374, &UA191838},
	{SG55375, &UA191839},
	{SG55376, &UA191883},
	{SG55377, &UA208630},
	{SG55378, &UA222941},
	{SG55379, &UA222996},
	{SG55380, &UA231665},
	{SG55381, &UA231992},
	{SG55382, &UA236777},
	{SG55383, &UA251288},
	{SG55384, &UA251687},
	{SG55385, &UA257556},
	{SG55386, &UA305721},
	{SG55387, &UA306595},
	{SG55389, &UA310583},
	{SG55390, &UA311456},
	{SG55392, &UA312496},
	{SG55393, &UA314737},
	{SG55394, &UA317171},
	{SG55395, &UA317644},
	{SG55396, &UA329882},
	{SG55397, &UA455565},
	{SG55398, &UA603324},
	{SG55399, &UA603502},
	{SG55400, &UA603565},
	{SG55401, &UA607894},
	{SG55402, &UA616497},
	{SG55404, &UA616498},
	{SG55405, &UA616499},
	{SG55406, &UA624832},
	{SG55407, &UA629644},
	{SG55408, &UA642476},
	{SG55409, &UA648090},
	{SG55410, &UA706596},
	{SG55411, &UA783942},
	{SG55412, &UA789378},
	{SG55413, &UA843014},
	{SG55414, &UA1876161},
	{SG55415, &UA1922277},
	{SG55416, &UA1990234},
	{SG55417, &UA2141367},
	{SG55418, &UA2161600},
	{SG55419, &UA2203758},
	{SG55420, &UA2229706},
	{SG55421, &UA2260731},
	{SG55422, &UA2279677},
	{SG55423, &UA2302976},
	{SG55424, &UA2436519},
	{SG55425, &UA2459350},
	{SG55426, &UA2511438},
	{SG55428, &UA2514531},
	{SG55429, &UA2531206},
	{SG55430, &UA2582694},
	{SG55431, &UA2594266},
	{SG55432, &UA2652983},
	{SG55433, &UA2710649},
	{SG55434, &UA2735123},
	{SG55526, &UA30021},
	{SG55527, &UA30054},
	{SG55528, &UA30071},
	{SG55529, &UA46547},
	{SG55530, &UA46614},
	{SG55531, &UA149907},
	{SG55532, &UA169423},
	{SG55533, &UA170449},
	{SG55534, &UA174861},
	{SG55535, &UA180151},
	{SG55536, &UA188854},
	{SG55537, &UA188909},
	{SG55538, &UA222936},
	{SG55539, &UA231610},
	{SG55540, &UA603984},
	{SG55541, &UA630722},
	{SG55542, &UA655628},
	{SG55543, &UA808878},
	{SG55544, &UA1858267},
	{SG55545, &UA2157671},
	{SG55637, &UA2303318},
	{SG55639, &UA2321067},
	{SG55642, &UA2639581},
	{SG55645, &UA2659751},
	{SG55647, &UA2663878},
	{SG55650, &UA2698213},
	{SG55652, &UA2702567},
	{SG55717, &UA2359517},
	{SG55718, &UA2368110},
	{SG55719, &UA2391462},
	{SG55720, &UA2437235},
	{SG55721, &UA2470433},
	{SG55722, &UA2486558},
	{SG55723, &UA2506009},
	{SG55724, &UA2508284},
	{SG55725, &UA2515651},
	{SG55726, &UA2534896},
	{SG55727, &UA2535196},
	{SG55728, &UA2535472},
	{SG55729, &UA2538935},
	{SG55731, &UA2560427},
	{SG55732, &UA2561415},
	{SG55733, &UA2561574},
	{SG55734, &UA2582269},
	{SG55735, &UA2583703},
	{SG55737, &UA2604533},
	{SG55738, &UA2671983},
	{SG55739, &UA2706732},
	{SG55740, &UA2749271},
	{SG55741, &UA2790770},
	{SG55742, &UA2853632},
	{SG55743, &UA2879079},
	{SG55744, &UA2992077},
	{SG55745, &UA3035530},
	{SG55746, &UA3047919},
	{SG55747, &UA3059548},
	{SG55750, &UA2431286},
	{SG56321, &UA2488883},
	{SG56323, &UA2489292},
	{SG56326, &UA2490032},
	{SG56329, &UA2490736},
	{SG56332, &UA2492712},
	{SG56335, &UA2509710},
	{SG56338, &UA2513850},
	{SG56340, &UA2515660},
	{SG56342, &UA2532509},
	{SG56345, &UA2533053},
	{SG56348, &UA2539898},
	{SG56350, &UA2561836},
	{SG56352, &UA2565361},
	{SG56356, &UA2565380},
	{SG56358, &UA2583024},
	{SG56360, &UA2585300},
	{SG56362, &UA2585643},
	{SG56364, &UA2606989},
	{SG56366, &UA2607368},
	{SG56368, &UA2607382},
	{SG56369, &UA2703838},
	{SG56372, &UA2737341},
	{SG56374, &UA2765175},
	{SG56376, &UA2846475},
	{SG56378, &UA2851294},
	{SG56380, &UA2936101},
	{SG56383, &UA2942935},
	{SG56385, &UA2946848},
	{SG56387, &UA2952825},
	{SG56389, &UA2961513},
	{SG56391, &UA3046537},
	{SG56395, &UA3058305},
	{SG56686, &UA2434465},
	{SG56687, &UA2434473},
	{SG56688, &UA2434668},
	{SG56689, &UA2434669},
	{SG56690, &UA2434691},
	{SG56691, &UA2435094},
	{SG56692, &UA2435380},
	{SG56708, &UA2434467},
	{SG56709, &UA2434476},
	{SG56710, &UA2434480},
	{SG56711, &UA2434485},
	{SG56712, &UA2434577},
	{SG56729, &UA2885398},
	{SG56730, &UA2886774},
	{SG56732, &UA2947242},
	{SG56733, &UA2948092},
	{SG56734, &UA2965151},
	{SG56735, &UA3023745},
	{SG56736, &UA3023768},
	{SG56737, &UA3028155},
	{SG56738, &UA3033020},
	{SG56739, &UA3034134},
	{SG56740, &UA3034766},
	{SG56742, &UA3038094},
	{SG56743, &UA3059073},
	{SG56745, &UA3080191},
	{SG56746, &UA3096922},
	{SG56748, &UA3109692},
	{SG56749, &UA3111776},
	{SG56750, &UA3111837},
	{SG56751, &UA3113622},
	{SG56752, &UA3114594},
	{SG56753, &UA3114948},
	{SG56755, &UA3115083},
	{SG56756, &UA3115388},
	{SG56757, &UA3115458},
	{SG56758, &UA3115791},
	{SG56759, &UA3127084},
	{SG56760, &UA3128099},
	{SG56762, &UA3129324},
	{SG56763, &UA3130235},
	{SG56764, &UA3130253},
	{SG56774, &UA366542},
	{SG56775, &UA398172},
	{SG56776, &UA421881},
	{SG56777, &UA422323},
	{SG56778, &UA422380},
	{SG56779, &UA454799},
	{SG56780, &UA585740},
	{SG56781, &UA587468},
	{SG56782, &UA587511},
	{SG56783, &UA589698},
	{SG56784, &UA590180},
	{SG56785, &UA616123},
	{SG56786, &UA636688},
	{SG56787, &UA777976},
	{SG56788, &UA793768},
	{SG56789, &UA888147},
	{SG56790, &UA895623},
	{SG56791, &UA899035},
	{SG56793, &UA908452},
	{SG56794, &UA910038},
	{SG56796, &UA1976479},
	{SG56797, &UA1986550},
	{SG56798, &UA1992144},
	{SG56799, &UA2154562},
	{SG56800, &UA2156230},
	{SG56801, &UA2222203},
	{SG56802, &UA2323292},
	{SG56804, &UA2352535},
	{SG56805, &UA2382300},
	{SG56806, &UA2442872},
	{SG56807, &UA2554634},
	{SG56808, &UA3043061},
	{SG56809, &UA3047024},
	{SG56811, &UA366619},
	{SG56812, &UA422343},
	{SG56813, &UA428114},
	{SG56814, &UA587481},
	{SG56815, &UA588689},
	{SG56816, &UA590248},
	{SG56817, &UA599983},
	{SG56818, &UA609260},
	{SG56819, &UA672158},
	{SG56820, &UA804160},
	{SG56821, &UA899810},
	{SG56822, &UA907928},
	{SG56823, &UA909003},
	{SG56824, &UA1949593},
	{SG56825, &UA2161539},
	{SG56826, &UA2260489},
	{SG56827, &UA2260522},
	{SG56828, &UA2260573},
	{SG56829, &UA2331983},
	{SG56830, &UA2354934},
	{SG56831, &UA2361916},
	{SG56832, &UA2475023},
	{SG56833, &UA2504798},
	{SG56834, &UA2577715},
	{SG56835, &UA2692476},
	{SG56836, &UA2817920},
	{SG56837, &UA2979502},
	{SG56838, &UA3072501},
	{SG56839, &UA54620},
	{SG56840, &UA63028},
	{SG56841, &UA89789},
	{SG56842, &UA90113},
	{SG56843, &UA124466},
	{SG56844, &UA126344},
	{SG56845, &UA279616},
	{SG56846, &UA291888},
	{SG56847, &UA623410},
	{SG56848, &UA675428},
	{SG56849, &UA692881},
	{SG56850, &UA742583},
	{SG56851, &UA747262},
	{SG56852, &UA2139544},
	{SG56853, &UA2222148},
	{SG56854, &UA2260100},
	{SG56855, &UA2413352},
	{SG56856, &UA2512896},
	{SG56857, &UA2662517},
	{SG56858, &UA2804633},
	{SG56859, &UA2807438},
	{SG56860, &UA2977867},
	{SG56861, &UA37122},
	{SG56862, &UA37142},
	{SG56863, &UA37173},
	{SG56864, &UA37178},
	{SG56865, &UA70719},
	{SG56866, &UA91013},
	{SG56867, &UA90997},
	{SG56868, &UA231283},
	{SG56869, &UA355444},
	{SG56870, &UA452068},
	{SG56871, &UA572872},
	{SG56872, &UA622532},
	{SG56873, &UA653796},
	{SG56874, &UA653873},
	{SG56875, &UA662582},
	{SG56876, &UA725368},
	{SG56877, &UA772298},
	{SG56878, &UA815095},
	{SG56879, &UA1960125},
	{SG56880, &UA1968906},
	{SG56881, &UA2153389},
	{SG56882, &UA2305618},
	{SG56883, &UA2379956},
	{SG56884, &UA2421851},
	{SG56885, &UA2440443},
	{SG56886, &UA2811125},
	{SG56887, &UA2864109},
	{SG56888, &UA5634},
	{SG56889, &UA5662},
	{SG56890, &UA5718},
	{SG56891, &UA46556},
	{SG56892, &UA181413},
	{SG56893, &UA232715},
	{SG56894, &UA567890},
	{SG56895, &UA1891347},
	{SG56896, &UA2518024},
	{SG56897, &UA2597356},
	{SG56898, &UA2600104},
	{SG56899, &UA2600525},
	{SG56900, &UA2600548},
	{SG56901, &UA2640125},
	{SG56902, &UA2658438},
	{SG56903, &UA2659146},
	{SG56904, &UA2949900},
	{SG56905, &UA3070048},
	{SG56906, &UA29888},
	{SG56907, &UA29982},
	{SG56908, &UA29997},
	{SG56909, &UA30003},
	{SG56910, &UA30005},
	{SG56911, &UA30029},
	{SG56912, &UA90033},
	{SG56913, &UA90716},
	{SG56914, &UA144293},
	{SG56915, &UA157476},
	{SG56916, &UA158477},
	{SG56917, &UA170702},
	{SG56918, &UA171157},
	{SG56919, &UA171455},
	{SG56920, &UA171760},
	{SG56921, &UA173187},
	{SG56922, &UA180446},
	{SG56923, &UA181455},
	{SG56924, &UA190871},
	{SG56925, &UA190943},
	{SG56926, &UA197165},
	{SG56927, &UA211401},
	{SG56928, &UA222987},
	{SG56929, &UA223912},
	{SG56930, &UA231618},
	{SG56931, &UA232025},
	{SG56932, &UA300736},
	{SG56933, &UA363568},
	{SG56934, &UA455731},
	{SG56935, &UA503464},
	{SG56936, &UA605750},
	{SG56937, &UA624742},
	{SG56938, &UA625402},
	{SG56939, &UA658340},
	{SG56940, &UA687339},
	{SG56941, &UA699271},
	{SG56942, &UA785438},
	{SG56943, &UA1981459},
	{SG56944, &UA1983957},
	{SG56945, &UA2207731},
	{SG56946, &UA2222354},
	{SG56947, &UA2418662},
	{SG56948, &UA2462838},
	{SG56949, &UA2762044},
	{SG56950, &UA2387755},
	{SG56951, &UA2408171},
	{SG56952, &UA2435229},
	{SG56953, &UA2488893},
	{SG56954, &UA2489112},
	{SG56955, &UA2492638},
	{SG56956, &UA2492636},
	{SG56957, &UA2516523},
	{SG56958, &UA2532607},
	{SG56959, &UA2585701},
	{SG56960, &UA2712031},
	{SG57060, &UA2717086},
	{SG57061, &UA2717095},
	{SG57062, &UA2717564},
	{SG57063, &UA2717570},
	{SG57064, &UA2717576},
	{SG57065, &UA2717568},
	{SG57066, &UA2717871},
	{SG57067, &UA2718652},
	{SG57068, &UA3026507},
	{SG57069, &UA3026508},
	{SG57070, &UA3026513},
	{SG57071, &UA3026578},
	{SG57072, &UA3026590},
	{SG57073, &UA3026972},
	{SG57074, &UA3027045},
	{SG57075, &UA3028984},
	{SG57076, &UA3092408},
	{SG57077, &UA2205341},
	{SG57078, &UA2260587},
	{SG57079, &UA2260666},
	{SG57080, &UA2260670},
	{SG57081, &UA2362347},
	{SG57082, &UA2437691},
	{SG57083, &UA2460932},
	{SG57084, &UA3025634},
	{SG57085, &UA3027670},
	{SG57086, &UA3027724},
	{SG57087, &UA3030456},
	{SG57088, &UA3030768},
	{SG57089, &UA3031522},
	{SG57090, &UA3031947},
	{SG57091, &UA3032043},
	{SG57092, &UA3032347},
	{SG57093, &UA3033301},
	{SG57094, &UA3033394},
	{SG57095, &UA3034094},
	{SG57096, &UA3035600},
	{SG57097, &UA3036016},
	{SG57098, &UA3036157},
	{SG57099, &UA3037104},
	{SG57100, &UA3037136},
	{SG57101, &UA3037942},
	{SG57102, &UA3038196},
	{SG57103, &UA3039484},
	{SG57104, &UA3062577},
	{SG57105, &UA3066113},
	{SG57106, &UA3066825},
	{SG57107, &UA3079750},
	{SG57108, &UA3080184},
	{SG57109, &UA3082217},
	{SG57110, &UA3083827},
	{SG57111, &UA3092809},
	{SG57112, &UA3094865},
	{SG57113, &UA3108865},
	{SG57114, &UA3109625},
	{SG57115, &UA3110239},
	{SG57116, &UA3113755},
	{SG57117, &UA3114148},
	{SG57118, &UA3114193},
	{SG57119, &UA3115138},
	{SG57120, &UA3115741},
	{SG57121, &UA3115759},
	{SG57122, &UA3126891},
	{SG57123, &UA3127049},
	{SG57124, &UA3128010},
	{SG57125, &UA3128273},
	{SG57126, &UA3128780},
	{SG57127, &UA3128981},
	{SG57128, &UA3129083},
	{SG57129, &UA3130811},
	{SG57130, &UA3130982},
	{SG57131, &UA2436067},
	{SG57132, &UA2492643},
	{SG57133, &UA2532510},
	{SG57134, &UA2532511},
	{SG57135, &UA2585475},
	{SG57136, &UA90799},
	{SG57137, &UA90801},
	{SG57138, &UA90803},
	{SG57139, &UA90806},
	{SG57140, &UA90844},
	{SG57141, &UA90871},
	{SG57142, &UA257274},
	{SG57143, &UA358439},
	{SG57144, &UA622533},
	{SG57145, &UA624555},
	{SG57146, &UA637556},
	{SG57147, &UA676909},
	{SG57148, &UA708363},
	{SG57149, &UA1892083},
	{SG57150, &UA1956435},
	{SG57151, &UA2153358},
	{SG57152, &UA2319193},
	{SG57153, &UA2354061},
	{SG57154, &UA2355738},
	{SG57155, &UA2386616},
	{SG57156, &UA2664565},
	{SG57157, &UA2205336},
	{SG57158, &UA2205345},
	{SG57159, &UA2261168},
	{SG57160, &UA2261169},
	{SG57161, &UA2437635},
	{SG57162, &UA616429},
	{SG57163, &UA1999586},
	{SG57164, &UA2043647},
	{SG57165, &UA2166928},
	{SG57166, &UA2201959},
	{SG57167, &UA2202159},
	{SG57168, &UA2232219},
	{SG57169, &UA2238381},
	{SG57170, &UA2261375},
	{SG57171, &UA2261389},
	{SG57172, &UA2261391},
	{SG57173, &UA2357216},
	{SG57174, &UA2403960},
	{SG57175, &UA2404202},
	{SG57176, &UA2425080},
	{SG57177, &UA2428899},
	{SG57178, &UA2434132},
	{SG57179, &UA2481544},
	{SG57180, &UA2484561},
	{SG57182, &UA2484678},
	{SG57183, &UA2578413},
	{SG57184, &UA3070206},
	{SG57185, &UA90290},
	{SG57186, &UA91110},
	{SG57187, &UA147443},
	{SG57188, &UA153184},
	{SG57189, &UA305118},
	{SG57190, &UA688801},
	{SG57191, &UA719898},
	{SG57192, &UA880508},
	{SG57193, &UA2143717},
	{SG57194, &UA2207243},
	{SG57195, &UA2642827},
	{SG57196, &UA2642831},
	{SG57197, &UA2711755},
	{SG57373, &UA427334},
	{SG57374, &UA441534},
	{SG57375, &UA615680},
	{SG57376, &UA615681},
	{SG57377, &UA615687},
	{SG57378, &UA616215},
	{SG57379, &UA625788},
	{SG57380, &UA691048},
	{SG57381, &UA1986631},
	{SG57382, &UA2041678},
	{SG57383, &UA2178193},
	{SG57384, &UA2202492},
	{SG57385, &UA2218255},
	{SG57386, &UA2325040},
	{SG57387, &UA2419408},
	{SG57388, &UA2544037},
	{SG57389, &UA3100884},
	{SG57391, &UA29953},
	{SG57392, &UA29971},
	{SG57393, &UA29975},
	{SG57394, &UA64003},
	{SG57395, &UA131611},
	{SG57396, &UA171339},
	{SG57397, &UA300808},
	{SG57398, &UA303167},
	{SG57399, &UA372610},
	{SG57400, &UA433956},
	{SG57401, &UA482639},
	{SG57402, &UA485364},
	{SG57403, &UA677659},
	{SG57404, &UA789651},
	{SG57405, &UA1851711},
	{SG57406, &UA1861354},
	{SG57407, &UA1865063},
	{SG57408, &UA1983369},
	{SG57409, &UA1995365},
	{SG57410, &UA2234759},
	{SG57411, &UA2662010},
	{SG57412, &UA2664353},
	{SG57413, &UA2568259},
	{SG57414, &UA2568318},
	{SG57415, &UA2568325},
	{SG57416, &UA2568329},
	{SG57417, &UA2568359},
	{SG57418, &UA2568365},
	{SG57419, &UA2585372},
	{SG57420, &UA2605882},
	{SG57421, &UA2709284},
	{SG57422, &UA2437337},
	{SG57423, &UA2456836},
	{SG57424, &UA2487506},
	{SG57425, &UA2490677},
	{SG57426, &UA2532516},
	{SG57427, &UA2583799},
	{SG57428, &UA2891464},
	{SG57432, &UA2717087},
	{SG57433, &UA2717088},
	{SG57434, &UA2717567},
	{SG57435, &UA2719068},
	{SG57436, &UA2205354},
	{SG57437, &UA2205487},
	{SG57438, &UA2260495},
	{SG57439, &UA2502116},
	{SG57440, &UA29917},
	{SG57441, &UA29976},
	{SG57442, &UA90029},
	{SG57443, &UA170944},
	{SG57444, &UA1973741},
	{SG57445, &UA2127761},
	{SG57446, &UA2128933},
	{SG57447, &UA2268112},
	{SG57448, &UA2282174},
	{SG57449, &UA2659362},
	{SG57450, &UA5644},
	{SG57451, &UA5708},
	{SG57452, &UA5721},
	{SG57453, &UA90489},
	{SG57454, &UA90726},
	{SG57455, &UA169279},
	{SG57456, &UA180441},
	{SG57457, &UA180794},
	{SG57458, &UA182051},
	{SG57459, &UA191904},
	{SG57460, &UA223927},
	{SG57461, &UA223926},
	{SG57462, &UA231624},
	{SG57463, &UA232029},
	{SG57464, &UA232610},
	{SG57465, &UA455871},
	{SG57466, &UA456291},
	{SG57467, &UA457075},
	{SG57468, &UA625089},
	{SG57469, &UA626086},
	{SG57470, &UA628349},
	{SG57471, &UA636912},
	{SG57472, &UA749873},
	{SG57473, &UA750916},
	{SG57474, &UA1952147},
	{SG57475, &UA1984432},
	{SG57476, &UA2202423},
	{SG57477, &UA2203920},
	{SG57478, &UA2204245},
	{SG57479, &UA2205609},
	{SG57480, &UA2229990},
	{SG57481, &UA2336874},
	{SG57482, &UA2393500},
	{SG57483, &UA2432745},
	{SG57484, &UA2708156},
	{SG57485, &UA2709089},
	{SG57486, &UA2711123},
	{SG57487, &UA2726625},
	{SG57488, &UA3026496},
	{SG57489, &UA3026845},
	{SG57490, &UA3026859},
	{SG57491, &UA3027404},
	{SG57492, &UA90800},
	{SG57493, &UA90802},
	{SG57494, &UA90804},
	{SG57495, &UA90807},
	{SG57496, &UA90866},
	{SG57497, &UA90864},
	{SG57498, &UA90872},
	{SG57499, &UA90874},
	{SG57500, &UA200885},
	{SG57501, &UA255937},
	{SG57502, &UA256417},
	{SG57503, &UA257838},
	{SG57504, &UA359887},
	{SG57505, &UA361065},
	{SG57506, &UA616495},
	{SG57507, &UA671880},
	{SG57508, &UA740191},
	{SG57509, &UA890789},
	{SG57510, &UA1916276},
	{SG57511, &UA1947089},
	{SG57512, &UA2202538},
	{SG57513, &UA2205102},
	{SG57514, &UA2403613},
	{SG57515, &UA2422832},
	{SG57516, &UA2444161},
	{SG57517, &UA29928},
	{SG57518, &UA29938},
	{SG57519, &UA62189},
	{SG57520, &UA88729},
	{SG57521, &UA89125},
	{SG57522, &UA305254},
	{SG57523, &UA372027},
	{SG57524, &UA513137},
	{SG57525, &UA766163},
	{SG57526, &UA845189},
	{SG57527, &UA1853694},
	{SG57528, &UA2265865},
	{SG57529, &UA2662877},
	{SG57530, &UA2701312},
	{SG57531, &UA2769463},
	{SG57532, &UA2769668},
	{SG57533, &UA2770263},
	{SG57534, &UA2771206},
	{SG57535, &UA2772837},
	{SG57536, &UA2772845},
	{SG57537, &UA2773048},
	{SG57538, &UA2773063},
	{SG57539, &UA2937721},
	{SG57540, &UA2568302},
	{SG57541, &UA2568306},
	{SG57542, &UA2569937},
	{SG57543, &UA2585916},
	{SG57544, &UA3032051},
	{SG57545, &UA3037887},
	{SG57546, &UA3059874},
	{SG57547, &UA3060921},
	{SG57548, &UA3062565},
	{SG57549, &UA3093799},
	{SG57550, &UA3094919},
	{SG57551, &UA3109324},
	{SG57552, &UA3109887},
	{SG57553, &UA3113150},
	{SG57554, &UA3114763},
	{SG57555, &UA3129516},
	{SG57556, &UA3130090},
	{SG57557, &UA3130246},
	{SG57558, &UA2431221},
	{SG57559, &UA924836},
	{SG57560, &UA2012817},
	{SG57561, &UA2042766},
	{SG57562, &UA2200241},
	{SG57563, &UA2261403},
	{SG57564, &UA2425009},
	{SG57565, &UA2552136},
	{SG57566, &UA2594137},
	{SG57567, &UA2596476},
	{SG57568, &UA2597881},
	{SG57569, &UA2598078},
	{SG57570, &UA2205408},
	{SG57571, &UA2205474},
	{SG57572, &UA2205554},
	{SG57573, &UA2205717},
	{SG57574, &UA2382694},
	{SG57575, &UA2746533},
	{SG57576, &UA30030},
	{SG57577, &UA516452},
	{SG57578, &UA2600152},
	{SG57579, &UA2434558},
	{SG57580, &UA2435248},
	{SG57581, &UA2469133},
	{SG57582, &UA2436093},
	{SG57583, &UA2772326},
	{SG57584, &UA2796726},
	{SG57585, &UA2796934},
	{SG57586, &UA2797352},
	{SG57587, &UA2799012},
	{SG57588, &UA2799172},
	{SG57589, &UA2800686},
	{SG57590, &UA2869298},
	{SG57591, &UA2870027},
	{SG57592, &UA2870899},
	{SG57593, &UA2872528},
	{SG57594, &UA2874345},
	{SG57595, &UA2875691},
	{SG57596, &UA2883660},
	{SG57597, &UA2886762},
	{SG57598, &UA2887918},
	{SG57599, &UA2888524},
	{SG57600, &UA2897121},
	{SG57601, &UA2898329},
	{SG57602, &UA2899277},
	{SG57603, &UA2945794},
	{SG57604, &UA2947000},
	{SG57605, &UA2947240},
	{SG57606, &UA2947238},
	{SG57607, &UA2963955},
	{SG57608, &UA2972863},
	{SG57609, &UA2974029},
	{SG57610, &UA2974950},
	{SG57611, &UA2975964},
	{SG57612, &UA3022935},
	{SG57613, &UA3023647},
	{SG57614, &UA3024241},
	{SG57615, &UA3024283},
	{SG57616, &UA3026314},
	{SG57617, &UA3074675},
	{SG57618, &UA3110075},
	{SG57619, &UA3115618},
	{SG57620, &UA3128305},
	{SG57623, &UA29929},
	{SG57624, &UA89009},
	{SG57625, &UA89810},
	{SG57626, &UA93500},
	{SG57627, &UA94072},
	{SG57628, &UA147391},
	{SG57629, &UA2596211},
	{SG57630, &UA2596538},
	{SG57631, &UA2658890},
	{SG57632, &UA2711888},
	{SG57633, &UA2367465},
	{SG57634, &UA2405715},
	{SG57635, &UA2437385},
	{SG57636, &UA2487363},
	{SG57637, &UA2487694},
	{SG57638, &UA2489078},
	{SG57639, &UA2492473},
	{SG57640, &UA2516525},
	{SG57641, &UA2532515},
	{SG57642, &UA2585303},
	{SG57643, &UA2666225},
	{SG57644, &UA2709272},
	{SG57649, &UA422388},
	{SG57650, &UA623152},
	{SG57651, &UA896333},
	{SG57652, &UA2394751},
	{SG57653, &UA2525751},
	{SG57654, &UA2598075},
	{SG57655, &UA2685364},
	{SG57657, &UA2770268},
	{SG57658, &UA2772835},
	{SG57659, &UA2772841},
	{SG57660, &UA2772842},
	{SG57661, &UA2434557},
	{SG57662, &UA2435112},
	{SG57663, &UA2436052},
	{SG57664, &UA2436268},
	{SG57665, &UA2557405},
	{SG57666, &UA2367472},
	{SG57667, &UA2488894},
	{SG57668, &UA2532751},
	{SG57669, &UA2565783},
	{SG57670, &UA2585299},
	{SG57671, &UA2717090},
	{SG57672, &UA2717092},
	{SG57673, &UA2717572},
	{SG57674, &UA2717579},
	{SG57675, &UA2717583},
	{SG57676, &UA2708567},
	{SG57677, &UA2717093},
	{SG57678, &UA2717091},
	{SG57679, &UA2717569},
	{SG57680, &UA2717575},
	{SG57681, &UA2717876},
	{SG57682, &UA2717865},
	{SG57683, &UA3095658},
	{SG57684, &UA3095930},
	{SG57685, &UA3109641},
	{SG57686, &UA3115190},
	{SG57687, &UA3115355},
	{SG57688, &UA3115405},
	{SG57689, &UA2356973},
	{SG57690, &UA2362594},
	{SG57691, &UA2489523},
	{SG57692, &UA2489667},
	{SG57693, &UA2489950},
	{SG57694, &UA2490030},
	{SG57695, &UA2512869},
	{SG57696, &UA2532871},
	{SG57697, &UA2585566},
	{SG57698, &UA2710754},
	{SG57699, &UA2568312},
	{SG57700, &UA2568313},
	{SG57701, &UA2568321},
	{SG57702, &UA2568376},
	{SG57703, &UA2568866},
	{SG57704, &UA2569104},
	{SG57705, &UA2569109},
	{SG57706, &UA30012},
	{SG57707, &UA90488},
	{SG57708, &UA157622},
	{SG57709, &UA158830},
	{SG57710, &UA421307},
	{SG57711, &UA628325},
	{SG57712, &UA747502},
	{SG57713, &UA2419409},
	{SG57714, &UA2712573},
	{SG57715, &UA2808889},
	{SG57716, &UA2568316},
	{SG57717, &UA2568358},
	{SG57718, &UA2568361},
	{SG57719, &UA2568371},
	{SG57720, &UA2568372},
	{SG57722, &UA3026582},
	{SG57723, &UA3026721},
	{SG57724, &UA3027081},
	{SG57725, &UA3027137},
	{SG57726, &UA3027332},
	{SG57727, &UA3028827},
	{SG57728, &UA3030163},
	{SG57729, &UA3068965},
	{SG57730, &UA3096147},
	{SG57731, &UA3096615},
	{SG57732, &UA3109921},
	{SG57733, &UA3115351},
	{SG57734, &UA29886},
	{SG57735, &UA29896},
	{SG57736, &UA417205},
	{SG57737, &UA2437890},
	{SG57738, &UA2658954},
	{SG57739, &UA2659433},
	{SG57740, &UA924442},
	{SG57741, &UA2041377},
	{SG57742, &UA2202710},
	{SG57743, &UA2261146},
	{SG57744, &UA2261148},
	{SG57745, &UA2437560},
	{SG57746, &UA29936},
	{SG57747, &UA88631},
	{SG57748, &UA2406366},
	{SG57749, &UA2659528},
	{SG57774, &UA3022353},
	{SG57775, &UA3026586},
	{SG57776, &UA3026843},
	{SG57777, &UA3026850},
	{SG57778, &UA3026977},
	{SG57779, &UA2508113},
	{SG57780, &UA2509122},
	{SG57781, &UA2510525},
	{SG57782, &UA2511761},
	{SG57783, &UA2515079},
	{SG57784, &UA2532763},
	{SG57785, &UA2533054},
	{SG57786, &UA2533075},
	{SG57787, &UA2537401},
	{SG57788, &UA2538610},
	{SG57789, &UA2560906},
	{SG57790, &UA2583065},
	{SG57791, &UA2585555},
	{SG57792, &UA2606184},
	{SG57793, &UA2749305},
	{SG57794, &UA2854422},
	{SG57795, &UA2887164},
	{SG57909, &UA2972134},
	{SG57910, &UA2972147},
	{SG57911, &UA2974951},
	{SG57912, &UA3026723},
	{SG57913, &UA3033030},
	{SG57914, &UA3034364},
	{SG57915, &UA3083921},
	{SG57916, &UA3115363},
	{SG57917, &UA3115474},
	{SG57918, &UA3115528},
	{SG57919, &UA90886},
	{SG57920, &UA333557},
	{SG57921, &UA622534},
	{SG57922, &UA784178},
	{SG57923, &UA1892772},
	{SG57924, &UA2597517},
	{SG57925, &UA2658424},
	{SG57926, &UA2659282},
	{SG57943, &UA90889},
	{SG57944, &UA90894},
	{SG57945, &UA90888},
	{SG57946, &UA234031},
	{SG57947, &UA361455},
	{SG57948, &UA456692},
	{SG57949, &UA623197},
	{SG57950, &UA793759},
	{SG57951, &UA1948935},
	{SG57952, &UA2202395},
	{SG57953, &UA2203268},
	{SG57954, &UA2261365},
	{SG57955, &UA29887},
	{SG57956, &UA29919},
	{SG57957, &UA29920},
	{SG57958, &UA29978},
	{SG57959, &UA301127},
	{SG57960, &UA455736},
	{SG57961, &UA696287},
	{SG57962, &UA2553533},
	{SG57963, &UA3082795},
	{SG57964, &UA3082797},
	{SG57965, &UA3082798},
	{SG57966, &UA3082988},
	{SG57967, &UA3082998},
	{SG57968, &UA3083003},
	{SG57969, &UA3083012},
	{SG57970, &UA3083422},
	{SG57971, &UA3096169},
	{SG57972, &UA3096438},
	{SG57973, &UA3096506},
	{SG57974, &UA3096515},
	{SG57975, &UA3096547},
	{SG57976, &UA3096551},
	{SG57977, &UA3096558},
	{SG57978, &UA3096565},
	{SG57980, &UA135427},
	{SG57981, &UA155627},
	{SG57982, &UA156915},
	{SG57983, &UA289940},
	{SG57984, &UA293066},
	{SG57985, &UA455810},
	{SG57986, &UA603572},
	{SG57987, &UA658888},
	{SG57988, &UA715247},
	{SG57989, &UA730532},
	{SG57990, &UA803675},
	{SG57991, &UA1903211},
	{SG57992, &UA2317888},
	{SG57993, &UA2198235},
	{SG57994, &UA2205500},
	{SG57995, &UA2205615},
	{SG57996, &UA2205691},
	{SG57997, &UA2205814},
	{SG57998, &UA2234291},
	{SG57999, &UA2235348},
	{SG58000, &UA2239218},
	{SG58001, &UA2366865},
	{SG58002, &UA2773044},
	{SG58003, &UA2773047},
	{SG58004, &UA2773059},
	{SG58005, &UA2773068},
	{SG58006, &UA2773076},
	{SG58007, &UA2772848},
	{SG58008, &UA2773045},
	{SG58009, &UA2773079},
	{SG58010, &UA2773060},
	{SG58011, &UA2773488},
	{SG58012, &UA2773502},
	{SG58013, &UA2783678},
	{SG58014, &UA29948},
	{SG58015, &UA29950},
	{SG58016, &UA29952},
	{SG58017, &UA29970},
	{SG58018, &UA29972},
	{SG58019, &UA29957},
	{SG58020, &UA89803},
	{SG58021, &UA143505},
	{SG58022, &UA489826},
	{SG58023, &UA605710},
	{SG58024, &UA635228},
	{SG58025, &UA639238},
	{SG58026, &UA708612},
	{SG58027, &UA29926},
	{SG58028, &UA29927},
	{SG58029, &UA29934},
	{SG58030, &UA29937},
	{SG58031, &UA29941},
	{SG58032, &UA29944},
	{SG58033, &UA29981},
	{SG58034, &UA89170},
	{SG58035, &UA285019},
	{SG58036, &UA505035},
	{SG58037, &UA545180},
	{SG58038, &UA915840},
	{SG58039, &UA2890744},
	{SG58044, &UA2435621},
	{SG58045, &UA2435644},
	{SG58046, &UA2436073},
	{SG58047, &UA2436277},
	{SG58048, &UA2436452},
	{SG58049, &UA2437314},
	{SG58050, &UA2461209},
	{SG58051, &UA2468538},
	{SG58052, &UA2485749},
	{SG58053, &UA2434553},
	{SG58054, &UA2434938},
	{SG58055, &UA2437098},
	{SG58056, &UA2598109},
	{SG58057, &UA2205607},
	{SG58058, &UA2205626},
	{SG58059, &UA2205791},
	{SG58060, &UA2488272},
	{SG58061, &UA2532517},
	{SG58062, &UA2533253},
	{SG58063, &UA2585370},
	{SG58064, &UA907321},
	{SG58065, &UA908454},
	{SG58066, &UA1960665},
	{SG58068, &UA3094398},
	{SG58069, &UA3096505},
	{SG58070, &UA3096511},
	{SG58071, &UA3096528},
	{SG58072, &UA2193295},
	{SG58073, &UA2205517},
	{SG58074, &UA2717565},
	{SG58075, &UA2717566},
	{SG58076, &UA2717585},
	{SG58077, &UA2717864},
	{SG58078, &UA2717096},
	{SG58079, &UA2717863},
	{SG58080, &UA2769957},
	{SG58081, &UA2205347},
	{SG58082, &UA2238046},
	{SG58083, &UA2261382},
	{SG58084, &UA2597838},
	{SG58085, &UA2600524},
	{SG58089, &UA89546},
	{SG58090, &UA89819},
	{SG58091, &UA96252},
	{SG58092, &UA114545},
	{SG58093, &UA116545},
	{SG58094, &UA126502},
	{SG58095, &UA126905},
	{SG58096, &UA135240},
	{SG58097, &UA275642},
	{SG58098, &UA388863},
	{SG58099, &UA484853},
	{SG58100, &UA522601},
	{SG58101, &UA3082793},
	{SG58102, &UA3082987},
	{SG58103, &UA3082992},
	{SG58104, &UA3083403},
	{SG58105, &UA2971588},
	{SG58106, &UA2972128},
	{SG58107, &UA3026581},
	{SG58108, &UA3033031},
	{SG58109, &UA3034358},
	{SG58110, &UA3083242},
	{SG58111, &UA3115385},
	{SG58112, &UA3026653},
	{SG58113, &UA3027473},
	{SG58114, &UA3092435},
	{SG58115, &UA98458},
	{SG58116, &UA2133285},
	{SG58117, &UA2709258},
	{SG58118, &UA3026561},
	{SG58119, &UA3026970},
	{SG58120, &UA3028777},
	{SG58121, &UA3030236},
	{SG58122, &UA32208},
	{SG58123, &UA84799},
	{SG58124, &UA84801},
	{SG58125, &UA84800},
	{SG58126, &UA84802},
	{SG58127, &UA84803},
	{SG58128, &UA84804},
	{SG58129, &UA98239},
	{SG58130, &UA127203},
	{SG58131, &UA167067},
	{SG58132, &UA169573},
	{SG58133, &UA261599},
	{SG58134, &UA289897},
	{SG58135, &UA294588},
	{SG58136, &UA522804},
	{SG58137, &UA643393},
	{SG58138, &UA770081},
	{SG58139, &UA2265285},
	{SG58140, &UA2373422},
	{SG58141, &UA2439697},
	{SG58142, &UA2720792},
	{SG58143, &UA2717934},
	{SG58144, &UA2752825},
	{SG58145, &UA2042398},
	{SG58146, &UA2260679},
	{SG58147, &UA2356710},
	{SG58148, &UA2356730},
	{SG58149, &UA2369748},
	{SG58150, &UA2427593},
	{SG58151, &UA2437619},
	{SG58152, &UA2717935},
	{SG58153, &UA2753576},
	{SG58154, &UA2708579},
	{SG58155, &UA2717577},
	{SG58156, &UA2717582},
	{SG58157, &UA2717586},
	{SG58158, &UA2996396},
	{SG58159, &UA427345},
	{SG58160, &UA427356},
	{SG58161, &UA615511},
	{SG58162, &UA808977},
	{SG58163, &UA898906},
	{SG58164, &UA2165316},
	{SG58165, &UA3033024},
	{SG58166, &UA3034369},
	{SG58167, &UA3083302},
	{SG58168, &UA3115365},
	{SG58169, &UA29954},
	{SG58170, &UA106201},
	{SG58171, &UA2658516},
	{SG58172, &UA2659486},
	{SG58173, &UA2489374},
	{SG58174, &UA2489524},
	{SG58175, &UA2532640},
	{SG58176, &UA2585608},
	{SG58177, &UA2797348},
	{SG58178, &UA2867768},
	{SG58179, &UA2875862},
	{SG58180, &UA2885402},
	{SG58181, &UA2900021},
	{SG58182, &UA2947246},
	{SG58183, &UA2948104},
	{SG58184, &UA3025932},
	{SG58185, &UA3026320},
	{SG58186, &UA3033029},
	{SG58187, &UA3033379},
	{SG58188, &UA3034137},
	{SG58189, &UA3034375},
	{SG58190, &UA3083310},
	{SG58191, &UA3096870},
	{SG58192, &UA3108088},
	{SG58193, &UA3115087},
	{SG58194, &UA3115354},
	{SG58195, &UA3115450},
	{SG58196, &UA2568301},
	{SG58197, &UA2568366},
	{SG58198, &UA2568367},
	{SG58199, &UA2585385},
	{SG58200, &UA2568308},
	{SG58201, &UA2568524},
	{SG58202, &UA2940548},
	{SG58203, &UA2947248},
	{SG58204, &UA2947251},
	{SG58205, &UA3026317},
	{SG58206, &UA3033019},
	{SG58207, &UA3034361},
	{SG58208, &UA3079813},
	{SG58209, &UA3111574},
	{SG58210, &UA3112851},
	{SG58211, &UA3115352},
	{SG58212, &UA88549},
	{SG58213, &UA88610},
	{SG58214, &UA125879},
	{SG58215, &UA165226},
	{SG58216, &UA2658249},
	{SG58217, &UA2659375},
	{SG58238, &UA2434812},
	{SG58256, &UA2436260},
	{SG58257, &UA2436471},
	{SG58260, &UA2205427},
	{SG58261, &UA2205812},
	{SG58262, &UA2597214},
	{SG58263, &UA2850060},
	{SG58266, &UA5631},
	{SG58267, &UA5651},
	{SG58268, &UA5684},
	{SG58269, &UA5729},
	{SG58270, &UA37167},
	{SG58271, &UA46568},
	{SG58272, &UA2300909},
	{SG58273, &UA2492060},
	{SG58274, &UA2512868},
	{SG58275, &UA2532926},
	{SG58276, &UA2582603},
	{SG58277, &UA2738246},
	{SG58278, &UA2202058},
	{SG58279, &UA2205254},
	{SG58280, &UA2205543},
	{SG58281, &UA2233841},
	{SG58282, &UA2357093},
	{SG58283, &UA2357528},
	{SG58284, &UA2404734},
	{SG58285, &UA2462079},
	{SG58286, &UA2470048},
	{SG58287, &UA2483742},
	{SG58288, &UA2538611},
	{SG58289, &UA2539850},
	{SG58290, &UA2539852},
	{SG58291, &UA2657081},
	{SG58292, &UA2763529},
	{SG58293, &UA2933677},
	{SG58294, &UA3042633},
	{SG58295, &UA3085513},
	{SG58296, &UA3099169},
	{SG58297, &UA3099885},
	{SG58298, &UA2883658},
	{SG58299, &UA2947245},
	{SG58300, &UA2970831},
	{SG58301, &UA3023477},
	{SG58302, &UA3096317},
	{SG58303, &UA3108013},
	{SG58304, &UA3109234},
	{SG58305, &UA3115392},
	{SG58306, &UA3130646},
	{SG58307, &UA2492647},
	{SG58308, &UA2509951},
	{SG58309, &UA2510222},
	{SG58310, &UA2511952},
	{SG58311, &UA2532865},
	{SG58312, &UA2536100},
	{SG58313, &UA2601989},
	{SG58314, &UA3026891},
	{SG58315, &UA88722},
	{SG58316, &UA92935},
	{SG58317, &UA93503},
	{SG58318, &UA93948},
	{SG58319, &UA93974},
	{SG58320, &UA96009},
	{SG58321, &UA104796},
	{SG58322, &UA121158},
	{SG58323, &UA124941},
	{SG58324, &UA157927},
	{SG58325, &UA414893},
	{SG58326, &UA417678},
	{SG58327, &UA3026971},
	{SG58328, &UA3027076},
	{SG58329, &UA3029282},
	{SG58330, &UA3029509},
	{SG58331, &UA2568320},
	{SG58332, &UA2568373},
	{SG58333, &UA2569094},
	{SG58334, &UA2569382},
	{SG58335, &UA2436477},
	{SG58336, &UA2436539},
	{SG58337, &UA2435093},
	{SG58338, &UA2437463},
	{SG58339, &UA2800692},
	{SG58340, &UA2885397},
	{SG58341, &UA2947254},
	{SG58342, &UA3028145},
	{SG58343, &UA3034285},
	{SG58344, &UA3038870},
	{SG58345, &UA3094480},
	{SG58346, &UA3115623},
	{SG58347, &UA907083},
	{SG58348, &UA907235},
	{SG58349, &UA910223},
	{SG58350, &UA910448},
	{SG58351, &UA1964833},
	{SG58352, &UA2197777},
	{SG58353, &UA2205557},
	{SG58354, &UA2205610},
	{SG58355, &UA2205875},
	{SG58356, &UA2205880},
	{SG58357, &UA2235500},
	{SG58358, &UA2659371},
	{SG58359, &UA2772844},
	{SG58360, &UA2772847},
	{SG58361, &UA2773053},
	{SG58362, &UA2773491},
	{SG58364, &UA180538},
	{SG58365, &UA181987},
	{SG58366, &UA182911},
	{SG58367, &UA191862},
	{SG58368, &UA191933},
	{SG58369, &UA222940},
	{SG58370, &UA233963},
	{SG58371, &UA309638},
	{SG58372, &UA312737},
	{SG58373, &UA2772432},
	{SG58374, &UA2772850},
	{SG58375, &UA2938822},
	{SG58376, &UA29897},
	{SG58377, &UA90031},
	{SG58378, &UA133917},
	{SG58379, &UA143323},
	{SG58380, &UA624415},
	{SG58381, &UA655746},
	{SG58382, &UA727455},
	{SG58383, &UA1846804},
	{SG58384, &UA2162088},
	{SG58385, &UA2674428},
	{SG58387, &UA2435376},
	{SG58388, &UA2436264},
	{SG58389, &UA2436439},
	{SG58390, &UA2437209},
	{SG58391, &UA2437360},
	{SG58392, &UA2461211},
	{SG58393, &UA2858042},
	{SG58395, &UA2436535},
	{SG58396, &UA2436638},
	{SG58403, &UA2676702},
	{SG58404, &UA910469},
	{SG58405, &UA2198542},
	{SG58406, &UA2260773},
	{SG58407, &UA2436647},
	{SG58408, &UA2466997},
	{SG58409, &UA2529395},
	{SG58415, &UA2773066},
	{SG58416, &UA2773496},
	{SG58417, &UA2783672},
	{SG58418, &UA2784156},
	{SG58419, &UA2489857},
	{SG58420, &UA2489951},
	{SG58421, &UA2532610},
	{SG58422, &UA2585559},
	{SG58426, &UA3039873},
	{SG58427, &UA3039874},
	{SG58428, &UA3082796},
	{SG58429, &UA3082799},
	{SG58430, &UA3083432},
	{SG58431, &UA3110675},
	{SG58432, &UA3115633},
	{SG58433, &UA3116186},
	{SG58435, &UA2507883},
	{SG58436, &UA2565714},
	{SG58437, &UA2585694},
	{SG58438, &UA2607315},
	{SG58439, &UA2738291},
	{SG58440, &UA2861044},
	{SG58441, &UA351991},
	{SG58442, &UA2431737},
	{SG58448, &UA2434569},
	{SG58449, &UA2435629},
	{SG58450, &UA2659113},
	{SG58451, &UA29943},
	{SG58452, &UA63029},
	{SG58453, &UA140907},
	{SG58454, &UA2489089},
	{SG58455, &UA2511934},
	{SG58456, &UA2532769},
	{SG58457, &UA2585466},
	{SG58458, &UA2710743},
	{SG58473, &UA3039888},
	{SG58474, &UA3082794},
	{SG58475, &UA3082986},
	{SG58476, &UA3095934},
	{SG58477, &UA3115378},
	{SG58478, &UA2485469},
	{SG58479, &UA2503987},
	{SG58480, &UA2514722},
	{SG58481, &UA2535910},
	{SG58482, &UA2562022},
	{SG58483, &UA2564672},
	{SG58484, &UA2570610},
	{SG58485, &UA2581665},
	{SG58486, &UA2606819},
	{SG58487, &UA2607445},
	{SG58488, &UA2670334},
	{SG58489, &UA2675606},
	{SG58490, &UA2700772},
	{SG58491, &UA2701549},
	{SG58492, &UA2712928},
	{SG58493, &UA2738281},
	{SG58494, &UA2750477},
	{SG58495, &UA2752520},
	{SG58496, &UA2767140},
	{SG58497, &UA2937173},
	{SG58498, &UA2962871},
	{SG58499, &UA2997308},
	{SG58500, &UA2012813},
	{SG58501, &UA2040925},
	{SG58502, &UA2203816},
	{SG58503, &UA2261395},
	{SG58504, &UA2386107},
	{SG58505, &UA2486298},
	{SG58506, &UA84805},
	{SG58507, &UA86999},
	{SG58508, &UA88317},
	{SG58509, &UA106792},
	{SG58510, &UA111531},
	{SG58511, &UA274525},
	{SG58512, &UA2849645},
	{SG58513, &UA2677408},
	{SG58514, &UA2679631},
	{SG58515, &UA2712911},
	{SG58516, &UA2713771},
	{SG58517, &UA2715414},
	{SG58518, &UA2715742},
	{SG58519, &UA2736298},
	{SG58520, &UA2795768},
	{SG58521, &UA2995738},
	{SG58522, &UA2041690},
	{SG58523, &UA2041905},
	{SG58524, &UA2437114},
	{SG58525, &UA2437200},
	{SG58526, &UA2515310},
	{SG58528, &UA2486964},
	{SG58529, &UA2532513},
	{SG58530, &UA2585373},
	{SG58531, &UA2755034},
	{SG58532, &UA2489758},
	{SG58533, &UA2535182},
	{SG58534, &UA2607026},
	{SG58535, &UA2765815},
	{SG58536, &UA70721},
	{SG58537, &UA91001},
	{SG58538, &UA622992},
	{SG58539, &UA2371956},
	{SG58543, &UA2436487},
	{SG58544, &UA2468955},
	{SG58545, &UA2469933},
	{SG58547, &UA2858099},
	{SG58548, &UA2669291},
	{SG58549, &UA2509668},
	{SG58550, &UA2532856},
	{SG58551, &UA2604567},
	{SG58552, &UA30027},
	{SG58553, &UA30028},
	{SG58554, &UA171626},
	{SG58555, &UA182769},
	{SG58556, &UA194217},
	{SG58557, &UA236446},
	{SG58558, &UA304778},
	{SG58559, &UA329878},
	{SG58560, &UA387724},
	{SG58561, &UA2436624},
	{SG58562, &UA2436730},
	{SG58563, &UA2437364},
	{SG58564, &UA2490274},
	{SG58565, &UA2532952},
	{SG58566, &UA2558899},
	{SG58567, &UA2562710},
	{SG58568, &UA2585532},
	{SG58569, &UA2956831},
	{SG58570, &UA2567844},
	{SG58571, &UA2570002},
	{SG58572, &UA2567836},
	{SG58573, &UA2570654},
	{SG58575, &UA2432247},
	{SG58576, &UA2947949},
	{SG58577, &UA3016941},
	{SG58578, &UA3026313},
	{SG58579, &UA3033026},
	{SG58580, &UA3035718},
	{SG58581, &UA3081133},
	{SG58582, &UA3115353},
	{SG58585, &UA635915},
	{SG58586, &UA674398},
	{SG58587, &UA2495023},
	{SG58588, &UA2669310},
	{SG58589, &UA2703531},
	{SG58599, &UA2709202},
	{SG58601, &UA2717587},
	{SG58603, &UA2718800},
	{SG58611, &UA454504},
	{SG58612, &UA454681},
	{SG58613, &UA456750},
	{SG58614, &UA615707},
	{SG58615, &UA624154},
	{SG58616, &UA626276},
	{SG58617, &UA640809},
	{SG58618, &UA663888},
	{SG58619, &UA807030},
	{SG58620, &UA2043387},
	{SG58621, &UA2197906},
	{SG58622, &UA2229705},
	{SG58623, &UA2352084},
	{SG58624, &UA2936920},
	{SG58627, &UA616453},
	{SG58628, &UA2356448},
	{SG58629, &UA2356778},
	{SG58630, &UA2431322},
	{SG58631, &UA3096541},
	{SG58632, &UA3096657},
	{SG58633, &UA3096683},
	{SG58634, &UA3096724},
	{SG58635, &UA3109513},
	{SG58636, &UA3109542},
	{SG58637, &UA3115494},
	{SG58638, &UA2043046},
	{SG58639, &UA2242184},
	{SG58640, &UA2437602},
	{SG58641, &UA2195621},
	{SG58645, &UA90840},
	{SG58646, &UA359969},
	{SG58647, &UA456324},
	{SG58648, &UA589744},
	{SG58649, &UA635976},
	{SG58650, &UA640615},
	{SG58651, &UA662085},
	{SG58652, &UA704528},
	{SG58653, &UA800333},
	{SG58654, &UA915857},
	{SG58655, &UA2232464},
	{SG58656, &UA2481400},
	{SG58657, &UA2505289},
	{SG58658, &UA2748440},
	{SG58659, &UA2984208},
	{SG58660, &UA2508158},
	{SG58661, &UA2510133},
	{SG58662, &UA2511506},
	{SG58663, &UA2531241},
	{SG58664, &UA2533313},
	{SG58665, &UA2535164},
	{SG58666, &UA2606994},
	{SG58667, &UA2768706},
	{SG58668, &UA2390427},
	{SG58670, &UA2204895},
	{SG58671, &UA2242197},
	{SG58672, &UA2426931},
	{SG58673, &UA910099},
	{SG58674, &UA910459},
	{SG58675, &UA2600768},
	{SG58676, &UA2658398},
	{SG58677, &UA2043316},
	{SG58678, &UA2233259},
	{SG58679, &UA2261385},
	{SG58683, &UA3082995},
	{SG58684, &UA3082996},
	{SG58685, &UA3083010},
	{SG58686, &UA3083002},
	{SG58687, &UA3083406},
	{SG58688, &UA3083410},
	{SG58689, &UA3095975},
	{SG58694, &UA909835},
	{SG58695, &UA910378},
	{SG58696, &UA910807},
	{SG58697, &UA910821},
	{SG58698, &UA911069},
	{SG58699, &UA911190},
	{SG58700, &UA911230},
	{SG58701, &UA911474},
	{SG58702, &UA2260715},
	{SG58703, &UA2511475},
	{SG58704, &UA2514228},
	{SG58705, &UA2534357},
	{SG58706, &UA2537316},
	{SG58707, &UA2564184},
	{SG58708, &UA2571306},
	{SG58709, &UA2582863},
	{SG58710, &UA2607468},
	{SG58711, &UA2669797},
	{SG58712, &UA2713012},
	{SG58713, &UA2764807},
	{SG58714, &UA2898645},
	{SG58715, &UA3082989},
	{SG58716, &UA3096071},
	{SG58719, &UA2508163},
	{SG58720, &UA2510762},
	{SG58721, &UA2538240},
	{SG58722, &UA2561168},
	{SG58723, &UA2569609},
	{SG58724, &UA2582070},
	{SG58725, &UA2597580},
	{SG58726, &UA2607309},
	{SG58727, &UA2660235},
	{SG58728, &UA2671537},
	{SG58729, &UA2714036},
	{SG58730, &UA2738249},
	{SG58731, &UA2739507},
	{SG58732, &UA2854290},
	{SG58733, &UA2995766},
	{SG58734, &UA3072475},
	{SG58735, &UA3073086},
	{SG58738, &UA2967889},
	{SG58739, &UA3024395},
	{SG58740, &UA3094873},
	{SG58741, &UA3109483},
	{SG58742, &UA3111613},
	{SG58743, &UA3115460},
	{SG58744, &UA3115673},
	{SG58745, &UA3127822},
	{SG58746, &UA3128974},
	{SG58747, &UA3129911},
	{SG58748, &UA2436485},
	{SG58749, &UA2437116},
	{SG58750, &UA2885400},
	{SG58751, &UA2947258},
	{SG58752, &UA3080761},
	{SG58753, &UA3096207},
	{SG58754, &UA3115675},
	{SG58755, &UA87001},
	{SG58756, &UA87085},
	{SG58757, &UA372316},
	{SG58758, &UA466000},
	{SG58759, &UA734733},
	{SG58760, &UA2149676},
	{SG58761, &UA2712421},
	{SG58762, &UA2982653},
	{SG58767, &UA2717590},
	{SG58768, &UA2717890},
	{SG58769, &UA2718544},
	{SG58772, &UA3096536},
	{SG58773, &UA3096543},
	{SG58774, &UA3096548},
	{SG58775, &UA3096576},
	{SG58776, &UA3096833},
	{SG58777, &UA3026783},
	{SG58778, &UA3028971},
	{SG58779, &UA3030405},
	{SG58780, &UA2195623},
	{SG58781, &UA2489756},
	{SG58782, &UA2490137},
	{SG58783, &UA2511146},
	{SG58784, &UA2512589},
	{SG58785, &UA2512860},
	{SG58786, &UA2533036},
	{SG58787, &UA2566191},
	{SG58788, &UA2585576},
	{SG58789, &UA2710578},
	{SG58790, &UA2713962},
	{SG58791, &UA2990732},
	{SG58792, &UA2997062},
	{SG58793, &UA3044714},
	{SG58794, &UA181389},
	{SG58795, &UA182276},
	{SG58796, &UA191956},
	{SG58797, &UA223082},
	{SG58798, &UA236015},
	{SG58799, &UA311647},
	{SG58800, &UA2160547},
	{SG58801, &UA2934356},
	{SG58802, &UA3096866},
	{SG58803, &UA3109471},
	{SG58804, &UA3115614},
	{SG58805, &UA2710785},
	{SG58806, &UA2717593},
	{SG58808, &UA3083004},
	{SG58809, &UA3083005},
	{SG58810, &UA3083008},
	{SG58811, &UA3083013},
	{SG58812, &UA3083411},
	{SG58813, &UA29942},
	{SG58814, &UA32026},
	{SG58815, &UA87380},
	{SG58816, &UA120984},
	{SG58817, &UA2597960},
	{SG58818, &UA2735695},
	{SG58819, &UA910458},
	{SG58820, &UA910613},
	{SG58821, &UA2239381},
	{SG58822, &UA2362780},
	{SG58823, &UA2405711},
	{SG58824, &UA2482996},
	{SG58825, &UA2488063},
	{SG58826, &UA2568844},
	{SG58827, &UA2569821},
	{SG58828, &UA2570189},
	{SG58829, &UA2796198},
	{SG58830, &UA2796242},
	{SG58831, &UA2867272},
	{SG58832, &UA3024247},
	{SG58873, &UA909772},
	{SG58874, &UA909781},
	{SG58875, &UA909971},
	{SG58876, &UA2741603},
	{SG58877, &UA2782879},
	{SG58878, &UA2799507},
	{SG58880, &UA2568842},
	{SG58881, &UA2569374},
	{SG58882, &UA2583549},
	{SG58885, &UA2800688},
	{SG58886, &UA2885396},
	{SG58887, &UA2947247},
	{SG58888, &UA3026311},
	{SG58889, &UA3096509},
	{SG58890, &UA3109536},
	{SG58891, &UA3109894},
	{SG58892, &UA3110177},
	{SG58893, &UA2168626},
	{SG58894, &UA2197113},
	{SG58895, &UA2201514},
	{SG58896, &UA2202088},
	{SG58897, &UA2203944},
	{SG58898, &UA2237622},
	{SG58899, &UA2349550},
	{SG58900, &UA2356949},
	{SG58901, &UA2381963},
	{SG58902, &UA2386837},
	{SG58903, &UA2433995},
	{SG58904, &UA2481129},
	{SG58905, &UA2486413},
	{SG58906, &UA2843072},
	{SG58907, &UA2996365},
	{SG58908, &UA2739980},
	{SG58909, &UA2799912},
	{SG58912, &UA427333},
	{SG58913, &UA615678},
	{SG58914, &UA1954721},
	{SG58915, &UA2708253},
	{SG58916, &UA2717094},
	{SG58917, &UA616377},
	{SG58918, &UA2162964},
	{SG58919, &UA2200463},
	{SG58920, &UA2236236},
	{SG58922, &UA2261337},
	{SG58923, &UA2356797},
	{SG58924, &UA2359199},
	{SG58925, &UA2403726},
	{SG58926, &UA2434681},
	{SG58927, &UA2437106},
	{SG58928, &UA2460900},
	{SG58929, &UA2486700},
	{SG58930, &UA2487317},
	{SG58931, &UA2557214},
	{SG58933, &UA427426},
	{SG58934, &UA615688},
	{SG58935, &UA617858},
	{SG58936, &UA619149},
	{SG58937, &UA2432170},
	{SG58938, &UA3026768},
	{SG58939, &UA3030177},
	{SG58942, &UA427408},
	{SG58943, &UA615715},
	{SG58944, &UA97119},
	{SG58945, &UA2658305},
	{SG58946, &UA2773065},
	{SG58947, &UA2783580},
	{SG58948, &UA2783674},
	{SG58952, &UA616376},
	{SG58953, &UA908543},
	{SG58954, &UA2197908},
	{SG58955, &UA2203411},
	{SG58956, &UA2261180},
	{SG58957, &UA2261184},
	{SG58958, &UA2352194},
	{SG58959, &UA2356975},
	{SG58960, &UA2367595},
	{SG58961, &UA2415300},
	{SG58962, &UA2425921},
	{SG58963, &UA2437636},
	{SG58964, &UA2457493},
	{SG58965, &UA2658896},
	{SG58966, &UA2727609},
	{SG58967, &UA2850782},
	{SG58968, &UA2983538},
	{SG58969, &UA2568377},
	{SG58970, &UA2569994},
	{SG58971, &UA2198464},
	{SG58972, &UA2205489},
	{SG58973, &UA2234085},
	{SG58974, &UA2236041},
	{SG58975, &UA2236286},
	{SG58976, &UA2236428},
	{SG58977, &UA46622},
	{SG58978, &UA330810},
	{SG58979, &UA377041},
	{SG58980, &UA436619},
	{SG58981, &UA524665},
	{SG58982, &UA571313},
	{SG58983, &UA639577},
	{SG58984, &UA783719},
	{SG58985, &UA815289},
	{SG58986, &UA835551},
	{SG58987, &UA849097},
	{SG58988, &UA2139407},
	{SG58989, &UA2142421},
	{SG58990, &UA2369894},
	{SG58991, &UA2374503},
	{SG58992, &UA2437859},
	{SG58993, &UA2437982},
	{SG58994, &UA2439236},
	{SG58995, &UA2445444},
	{SG58996, &UA2472843},
	{SG58997, &UA2472974},
	{SG58998, &UA2477998},
	{SG58999, &UA2520458},
	{SG59000, &UA2544560},
	{SG59001, &UA2639470},
	{SG59002, &UA2987156},
	{SG59003, &UA2491913},
	{SG59004, &UA2509844},
	{SG59005, &UA2532766},
	{SG59006, &UA2607412},
	{SG59010, &UA89805},
	{SG59011, &UA108095},
	{SG59012, &UA124988},
	{SG59013, &UA125866},
	{SG59014, &UA763813},
	{SG59015, &UA783337},
	{SG59016, &UA2261405},
	{SG59019, &UA2205624},
	{SG59020, &UA2234492},
	{SG59021, &UA2659443},
	{SG59022, &UA2773487},
	{SG59023, &UA2783340},
	{SG59024, &UA2783574},
	{SG59025, &UA2436610},
	{SG59026, &UA2437193},
	{SG59027, &UA3029153},
	{SG59028, &UA3029363},
	{SG59029, &UA3029374},
	{SG59030, &UA2741604},
	{SG59031, &UA2755138},
	{SG59032, &UA2769105},
	{SG59033, &UA3027630},
	{SG59034, &UA3028695},
	{SG59035, &UA3064105},
	{SG59036, &UA183709},
	{SG59037, &UA183838},
	{SG59038, &UA193875},
	{SG59039, &UA225453},
	{SG59040, &UA233388},
	{SG59041, &UA313518},
	{SG59042, &UA3082991},
	{SG59043, &UA3083419},
	{SG59044, &UA3095885},
	{SG59045, &UA3096178},
	{SG59046, &UA2043685},
	{SG59047, &UA2260603},
	{SG59048, &UA2657562},
	{SG59049, &UA2696997},
	{SG59050, &UA2841749},
	{SG59051, &UA2568382},
	{SG59052, &UA2584268},
	{SG59053, &UA109908},
	{SG59054, &UA804631},
	{SG59056, &UA427331},
	{SG59057, &UA615686},
	{SG59058, &UA2436483},
	{SG59059, &UA2437362},
	{SG59060, &UA2468868},
	{SG59061, &UA2469123},
	{SG59062, &UA2469441},
	{SG59063, &UA2556743},
	{SG59064, &UA2677117},
	{SG59065, &UA191330},
	{SG59066, &UA658946},
	{SG59067, &UA2205678},
	{SG59068, &UA3082990},
	{SG59069, &UA3083951},
	{SG59070, &UA3095884},
	{SG59071, &UA3096039},
	{SG59072, &UA3096160},
	{SG59073, &UA3115636},
	{SG59075, &UA2773493},
	{SG59076, &UA2784632},
	{SG59077, &UA2515657},
	{SG59078, &UA2563361},
	{SG59079, &UA2604681},
	{SG59080, &UA2607372},
	{SG59087, &UA2538616},
	{SG59088, &UA2562157},
	{SG59089, &UA2843427},
	{SG59090, &UA2952969},
	{SG59091, &UA2784727},
	{SG59092, &UA2784744},
	{SG59093, &UA2797500},
	{SG59094, &UA2797639},
	{SG59095, &UA2868843},
	{SG59096, &UA2876108},
	{SG59097, &UA2958220},
	{SG59098, &UA2667535},
	{SG59099, &UA2676529},
	{SG59100, &UA2703912},
	{SG59101, &UA2708700},
	{SG59102, &UA2710039},
	{SG59103, &UA2710361},
	{SG59104, &UA2713477},
	{SG59105, &UA2733665},
	{SG59106, &UA2737629},
	{SG59107, &UA2737794},
	{SG59108, &UA2752489},
	{SG59109, &UA2752596},
	{SG59110, &UA2768854},
	{SG59111, &UA2995925},
	{SG59112, &UA607431},
	{SG59113, &UA625432},
	{SG59114, &UA667364},
	{SG59115, &UA671337},
	{SG59116, &UA686477},
	{SG59117, &UA799137},
	{SG59118, &UA1986356},
	{SG59119, &UA2163202},
	{SG59120, &UA2260854},
	{SG59121, &UA2328639},
	{SG59122, &UA2573879},
	{SG59125, &UA29930},
	{SG59126, &UA29932},
	{SG59127, &UA29935},
	{SG59128, &UA29939},
	{SG59129, &UA2511127},
	{SG59130, &UA2564173},
	{SG59131, &UA2607003},
	{SG59132, &UA2510023},
	{SG59133, &UA2511758},
	{SG59134, &UA2532296},
	{SG59135, &UA2534978},
	{SG59136, &UA2537376},
	{SG59137, &UA2562477},
	{SG59138, &UA2606925},
	{SG59139, &UA2739709},
	{SG59141, &UA2773501},
	{SG59142, &UA2800588},
	{SG59144, &UA2510757},
	{SG59145, &UA2510766},
	{SG59146, &UA2510773},
	{SG59147, &UA2533039},
	{SG59148, &UA2536834},
	{SG59149, &UA2606936},
	{SG59150, &UA2796249},
	{SG59151, &UA909830},
	{SG59152, &UA910372},
	{SG59153, &UA910359},
	{SG59154, &UA910436},
	{SG59155, &UA910892},
	{SG59156, &UA912304},
	{SG59208, &UA2382901},
	{SG59209, &UA2385605},
	{SG59210, &UA2526105},
	{SG59212, &UA3095795},
	{SG59213, &UA3130390},
	{SG59214, &UA2490748},
	{SG59215, &UA686572},
	{SG59216, &UA700848},
	{SG59217, &UA777629},
	{SG59218, &UA156845},
	{SG59229, &UA2237042},
	{SG59230, &UA2362604},
	{SG59231, &UA2362794},
	{SG59232, &UA2510316},
	{SG59233, &UA2533035},
	{SG59234, &UA2564673},
	{SG59235, &UA2585834},
	{SG59236, &UA3069029},
	{SG59237, &UA3080765},
	{SG59238, &UA3080772},
	{SG59239, &UA3084014},
	{SG59241, &UA2873103},
	{SG59242, &UA3096889},
	{SG59243, &UA3131033},
	{SG59244, &UA5643},
	{SG59245, &UA5667},
	{SG59246, &UA5674},
	{SG59247, &UA5714},
	{SG59248, &UA37144},
	{SG59249, &UA46560},
	{SG59250, &UA2784136},
	{SG59251, &UA2796269},
	{SG59252, &UA2800097},
	{SG59253, &UA2964453},
	{SG59274, &UA30056},
	{SG59275, &UA2705489},
	{SG59276, &UA2462720},
	{SG59277, &UA2508539},
	{SG59278, &UA2534989},
	{SG59279, &UA2606890},
	{SG59291, &UA2738957},
	{SG59292, &UA2796380},
	{SG59293, &UA2408206},
	{SG59294, &UA2426552},
	{SG59295, &UA2458382},
	{SG59296, &UA2467413},
	{SG59297, &UA2531112},
	{SG59298, &UA2661860},
	{SG59299, &UA2974391},
	{SG59300, &UA3030229},
	{SG59301, &UA3033399},
	{SG59302, &UA3036125},
	{SG59303, &UA3094949},
	{SG59304, &UA3128252},
	{SG59305, &UA2568340},
	{SG59306, &UA2601524},
	{SG59312, &UA615721},
	{SG59313, &UA658580},
	{SG59314, &UA1996894},
	{SG59316, &UA2039613},
	{SG59317, &UA2197902},
	{SG59319, &UA910674},
	{SG59320, &UA1950949},
	{SG59321, &UA2404500},
	{SG59322, &UA2566095},
	{SG59323, &UA2567683},
	{SG59324, &UA2640635},
	{SG59325, &UA2758389},
	{SG59326, &UA2792715},
	{SG59327, &UA2857914},
	{SG59328, &UA2890963},
	{SG59329, &UA3071751},
	{SG59330, &UA90808},
	{SG59331, &UA90875},
	{SG59332, &UA2490908},
	{SG59333, &UA2533141},
	{SG59334, &UA2607011},
	{SG59335, &UA2436056},
	{SG59336, &UA2469442},
	{SG59337, &UA2356859},
	{SG59338, &UA2424710},
	{SG59339, &UA2461380},
	{SG59340, &UA2528236},
	{SG59348, &UA2482971},
	{SG59349, &UA2533343},
	{SG59350, &UA2607392},
	{SG59351, &UA2755562},
	{SG59352, &UA2770456},
	{SG59353, &UA2773301},
	{SG59354, &UA2773925},
	{SG59355, &UA2773937},
	{SG59356, &UA2796775},
	{SG59358, &UA2877297},
	{SG59360, &UA2884019},
	{SG59362, &UA2885448},
	{SG59363, &UA2900484},
	{SG59364, &UA2945012},
	{SG59365, &UA2964306},
	{SG59366, &UA3037439},
	{SG59367, &UA3037443},
	{SG59368, &UA3093027},
	{SG59369, &UA2715174},
	{SG59370, &UA2715882},
	{SG59371, &UA2716588},
	{SG59372, &UA2751375},
	{SG59373, &UA2955750},
	{SG59381, &UA2567097},
	{SG59382, &UA2583197},
	{SG59384, &UA2583413},
	{SG59385, &UA2601585},
	{SG59386, &UA2672776},
	{SG59387, &UA2675589},
	{SG59388, &UA2489986},
	{SG59389, &UA2532749},
	{SG59390, &UA2585697},
	{SG59391, &UA329681},
	{SG59392, &UA632028},
	{SG59393, &UA696685},
	{SG59394, &UA700758},
	{SG59395, &UA1949761},
	{SG59396, &UA2129991},
	{SG59397, &UA2337263},
	{SG59398, &UA2492628},
	{SG59399, &UA2573296},
	{SG59400, &UA2745484},
	{SG59401, &UA3069037},
	{SG59402, &UA3111824},
	{SG59403, &UA3116450},
	{SG59404, &UA3116589},
	{SG59408, &UA87387},
	{SG59410, &UA165485},
	{SG59443, &UA156749},
	{SG59444, &UA488837},
	{SG59445, &UA1850482},
	{SG59446, &UA29945},
	{SG59447, &UA29961},
	{SG59448, &UA29965},
	{SG59449, &UA2801307},
	{SG59450, &UA2978637},
	{SG59451, &UA2408262},
	{SG59475, &UA2430818},
	{SG59476, &UA2459844},
	{SG59477, &UA2722870},
	{SG59478, &UA2202544},
	{SG59479, &UA2356966},
	{SG59480, &UA2597846},
	{SG59481, &UA910643},
	{SG59482, &UA911124},
	{SG59483, &UA2597520},
	{SG59484, &UA2598032},
	{SG59485, &UA3026681},
	{SG59486, &UA3030801},
	{SG59487, &UA3033330},
	{SG59488, &UA3079557},
	{SG59489, &UA2535743},
	{SG59490, &UA2581062},
	{SG59491, &UA2665130},
	{SG59492, &UA2367023},
	{SG59493, &UA2387493},
	{SG59494, &UA2484174},
	{SG59495, &UA2532927},
	{SG59496, &UA2661077},
	{SG59497, &UA2663448},
	{SG59498, &UA2986801},
	{SG59501, &UA910885},
	{SG59502, &UA911348},
	{SG59503, &UA911591},
	{SG59504, &UA911560},
	{SG59505, &UA911893},
	{SG59506, &UA912129},
	{SG59507, &UA2600201},
	{SG59508, &UA2659355},
	{SG59509, &UA90929},
	{SG59510, &UA91002},
	{SG59512, &UA603319},
	{SG59513, &UA603374},
	{SG59514, &UA654129},
	{SG59515, &UA2154853},
	{SG59516, &UA2197917},
	{SG59517, &UA2441614},
	{SG59518, &UA2844137},
	{SG59519, &UA90890},
	{SG59520, &UA90893},
	{SG59521, &UA30031},
	{SG59522, &UA30057},
	{SG59523, &UA30058},
	{SG59524, &UA728082},
	{SG59525, &UA1975771},
	{SG59526, &UA2367208},
	{SG59529, &UA3013092},
	{SG59530, &UA3032773},
	{SG59531, &UA3032786},
	{SG59532, &UA3056932},
	{SG59533, &UA3057255},
	{SG59534, &UA3078176},
	{SG59535, &UA3096915},
	{SG59536, &UA3129484},
	{SG59537, &UA2513673},
	{SG59538, &UA2703883},
	{SG59539, &UA2203658},
	{SG59540, &UA2348888},
	{SG59541, &UA2357274},
	{SG59542, &UA2386010},
	{SG59543, &UA2461013},
	{SG59544, &UA2205674},
	{SG59545, &UA2362711},
	{SG59546, &UA2460905},
	{SG59547, &UA3096045},
	{SG59548, &UA3130005},
	{SG59549, &UA2678366},
	{SG59550, &UA2767998},
	{SG59551, &UA2864383},
	{SG59552, &UA2995338},
	{SG59553, &UA90810},
	{SG59554, &UA622912},
	{SG59555, &UA2970826},
	{SG59556, &UA3030101},
	{SG59557, &UA3033711},
	{SG59558, &UA3036128},
	{SG59559, &UA3094661},
	{SG59560, &UA3115389},
	{SG59561, &UA2403530},
	{SG59562, &UA2427254},
	{SG59563, &UA2490562},
	{SG59564, &UA2533479},
	{SG59565, &UA2537154},
	{SG59566, &UA2602465},
	{SG59567, &UA2710767},
	{SG59568, &UA2204072},
	{SG59569, &UA2221634},
	{SG59570, &UA2356749},
	{SG59571, &UA2357345},
	{SG59572, &UA2460119},
	{SG59573, &UA2657719},
	{SG59574, &UA2334757},
	{SG59575, &UA2404211},
	{SG59576, &UA2437295},
	{SG59577, &UA2437369},
	{SG59578, &UA2460565},
	{SG59579, &UA2462224},
	{SG59580, &UA2470332},
	{SG59581, &UA2484247},
	{SG59582, &UA2487195},
	{SG59583, &UA2507599},
	{SG59584, &UA2710303},
	{SG59585, &UA2713608},
	{SG59586, &UA2719401},
	{SG59587, &UA3048196},
	{SG59588, &UA455366},
	{SG59589, &UA599759},
	{SG59590, &UA599780},
	{SG59591, &UA615773},
	{SG59592, &UA616089},
	{SG59593, &UA646475},
	{SG59594, &UA657919},
	{SG59595, &UA2234251},
	{SG59596, &UA2575987},
	{SG59597, &UA2775694},
	{SG59598, &UA2889109},
	{SG59599, &UA3093022},
	{SG59600, &UA3115294},
	{SG59601, &UA3115454},
	{SG59602, &UA29998},
	{SG59603, &UA178849},
	{SG59604, &UA191861},
	{SG59605, &UA234073},
	{SG59606, &UA418347},
	{SG59607, &UA707098},
	{SG59608, &UA711408},
	{SG59609, &UA2569947},
	{SG59610, &UA2583189},
	{SG59611, &UA2602022},
	{SG59612, &UA2602652},
	{SG59613, &UA2604671},
	{SG59615, &UA2506282},
	{SG59616, &UA2510317},
	{SG59617, &UA2561417},
	{SG59618, &UA2564212},
	{SG59619, &UA2607397},
	{SG59620, &UA2665809},
	{SG59622, &UA3069030},
	{SG59623, &UA3083308},
	{SG59624, &UA3084012},
	{SG59625, &UA3116566},
	{SG59626, &UA3116572},
	{SG59627, &UA3116597},
	{SG59628, &UA88334},
	{SG59629, &UA94075},
	{SG59630, &UA100876},
	{SG59631, &UA107145},
	{SG59632, &UA261859},
	{SG59641, &UA2756734},
	{SG59642, &UA2770857},
	{SG59643, &UA2869727},
	{SG59644, &UA2884018},
	{SG59645, &UA2899375},
	{SG59646, &UA3013135},
	{SG59647, &UA3016165},
	{SG59648, &UA3037436},
	{SG59649, &UA632713},
	{SG59650, &UA2403430},
	{SG59652, &UA2582291},
	{SG59653, &UA2596751},
	{SG59654, &UA2671388},
	{SG59655, &UA2796548},
	{SG59656, &UA3048824},
	{SG59657, &UA2762071},
	{SG59658, &UA3085903},
	{SG59659, &UA2516415},
	{SG59660, &UA2536218},
	{SG59661, &UA3095591},
	{SG59662, &UA3129993},
	{SG59663, &UA2204066},
	{SG59664, &UA2232789},
	{SG59665, &UA2260445},
	{SG59666, &UA2356918},
	{SG59667, &UA2357080},
	{SG59668, &UA2424583},
	{SG59688, &UA2708916},
	{SG59689, &UA2794220},
	{SG59690, &UA2995750},
	{SG59691, &UA3094025},
	{SG59692, &UA3115390},
	{SG59693, &UA427423},
	{SG59694, &UA656846},
	{SG59695, &UA3083006},
	{SG59696, &UA3096335},
	{SG59697, &UA3096905},
	{SG59698, &UA2437221},
	{SG59699, &UA2529881},
	{SG59700, &UA304321},
	{SG59701, &UA310683},
	{SG59702, &UA666785},
	{SG59703, &UA827805},
	{SG59704, &UA2372904},
	{SG59705, &UA2480502},
	{SG59706, &UA3096830},
	{SG59707, &UA3130349},
	{SG59708, &UA3094869},
	{SG59709, &UA3128976},
	{SG59710, &UA3111813},
	{SG59711, &UA29931},
	{SG59712, &UA29947},
	{SG59713, &UA88110},
	{SG59714, &UA174236},
	{SG59715, &UA263789},
	{SG59716, &UA2437453},
	{SG59717, &UA2484536},
	{SG59718, &UA2484443},
	{SG59719, &UA2533034},
	{SG59720, &UA2566607},
	{SG59721, &UA2606954},
	{SG59722, &UA2709295},
	{SG59723, &UA2205663},
	{SG59724, &UA2362719},
	{SG59725, &UA2430862},
	{SG59726, &UA3106794},
	{SG59728, &UA2563351},
	{SG59729, &UA2854661},
	{SG59730, &UA3027350},
	{SG59731, &UA2871485},
	{SG59732, &UA2969274},
	{SG59733, &UA3094870},
	{SG59734, &UA3115393},
	{SG59735, &UA3128082},
	{SG59736, &UA2718564},
	{SG59737, &UA2737805},
	{SG59739, &UA3095886},
	{SG59740, &UA3113536},
	{SG59744, &UA30022},
	{SG59745, &UA30059},
	{SG59746, &UA295282},
	{SG59768, &UA2509667},
	{SG59769, &UA2532864},
	{SG59770, &UA2585647},
	{SG59771, &UA2368606},
	{SG59772, &UA2389490},
	{SG59773, &UA2407369},
	{SG59774, &UA2424223},
	{SG59775, &UA2425159},
	{SG59776, &UA2482867},
	{SG59777, &UA3083418},
	{SG59778, &UA3126967},
	{SG59779, &UA2203391},
	{SG59780, &UA2222936},
	{SG59781, &UA2261063},
	{SG59782, &UA2352128},
	{SG59783, &UA2403886},
	{SG59784, &UA2454991},
	{SG59785, &UA2466897},
	{SG59786, &UA2841647},
	{SG59787, &UA2866688},
	{SG59788, &UA158360},
	{SG59789, &UA519776},
	{SG59790, &UA2847224},
	{SG59791, &UA2008904},
	{SG59793, &UA3030033},
	{SG59794, &UA3092641},
	{SG59795, &UA153048},
	{SG59796, &UA159248},
	{SG59797, &UA896063},
	{SG59798, &UA2510754},
	{SG59799, &UA2531841},
	{SG59800, &UA2535207},
	{SG59801, &UA2558339},
	{SG59802, &UA2850965},
	{SG59803, &UA166620},
	{SG59804, &UA303005},
	{SG59805, &UA418727},
	{SG59806, &UA444478},
	{SG59807, &UA539224},
	{SG59808, &UA635678},
	{SG59809, &UA742159},
	{SG59810, &UA2131276},
	{SG59811, &UA2392116},
	{SG59812, &UA2392883},
	{SG59813, &UA2437943},
	{SG59814, &UA2438983},
	{SG59815, &UA2742013},
	{SG59816, &UA2966375},
	{SG59817, &UA3094417},
	{SG59818, &UA3131068},
	{SG59819, &UA3028146},
	{SG59820, &UA3034750},
	{SG59821, &UA3110720},
	{SG59822, &UA3129053},
	{SG59823, &UA3095881},
	{SG59824, &UA3130334},
	{SG59828, &UA96093},
	{SG59829, &UA2733951},
	{SG59830, &UA851775},
	{SG59831, &UA3039830},
	{SG59832, &UA3083404},
	{SG59833, &UA3095962},
	{SG59834, &UA3130634},
	{SG59835, &UA2200553},
	{SG59836, &UA2242205},
	{SG59846, &UA152386},
	{SG59847, &UA152872},
	{SG59848, &UA292536},
	{SG59856, &UA2716334},
	{SG59874, &UA3096148},
	{SG59875, &UA3110077},
	{SG59876, &UA3115630},
	{SG59877, &UA3125854},
	{SG59878, &UA3128124},
	{SG59881, &UA2202601},
	{SG59882, &UA2232801},
	{SG59883, &UA2260237},
	{SG59884, &UA2366587},
	{SG59885, &UA2761981},
	{SG59886, &UA3117941},
	{SG59889, &UA2796945},
	{SG59890, &UA2885401},
	{SG59891, &UA2947932},
	{SG59892, &UA3111960},
	{SG59893, &UA2569829},
	{SG59894, &UA2667566},
	{SG59896, &UA2512819},
	{SG59897, &UA2557637},
	{SG59898, &UA2600183},
	{SG59899, &UA2659865},
	{SG59900, &UA2702364},
	{SG59901, &UA2703092},
	{SG59902, &UA2713864},
	{SG59903, &UA3056284},
	{SG59904, &UA2602366},
	{SG59905, &UA2670514},
	{SG59906, &UA2679625},
	{SG59907, &UA2711506},
	{SG59908, &UA2716100},
	{SG59909, &UA2716103},
	{SG59910, &UA2862979},
	{SG59911, &UA907677},
	{SG59912, &UA2670209},
	{SG59913, &UA2772486},
	{SG59914, &UA3121506},
	{SG59916, &UA2514707},
	{SG59917, &UA2564878},
	{SG59918, &UA2670934},
	{SG59919, &UA2432276},
	{SG59920, &UA2436421},
	{SG59921, &UA2783145},
	{SG59922, &UA2879602},
	{SG59923, &UA2797439},
	{SG59924, &UA2864918},
	{SG59925, &UA2892922},
	{SG59926, &UA3005196},
	{SG59927, &UA3006959},
	{SG59928, &UA3050476},
	{SG59929, &UA3060546},
	{SG59930, &UA3097269},
	{SG59931, &UA3129820},
	{SG59932, &UA2368509},
	{SG59933, &UA184008},
	{SG59934, &UA191881},
	{SG59935, &UA237779},
	{SG59936, &UA420567},
	{SG59937, &UA435134},
	{SG59938, &UA691564},
	{SG59939, &UA444303},
	{SG59940, &UA2720155},
	{SG59942, &UA2944703},
	{SG59943, &UA2945091},
	{SG59944, &UA3017370},
	{SG59945, &UA3017578},
	{SG59946, &UA3057037},
	{SG59947, &UA3057118},
	{SG59948, &UA3057369},
	{SG59950, &UA46466},
	{SG59951, &UA256754},
	{SG59952, &UA626856},
	{SG59953, &UA631324},
	{SG59954, &UA661704},
	{SG59955, &UA751927},
	{SG59956, &UA801101},
	{SG59957, &UA2308591},
	{SG59958, &UA2527191},
	{SG59959, &UA2786302},
	{SG59960, &UA3043421},
	{SG59961, &UA2425245},
	{SG59962, &UA2430914},
	{SG59969, &UA1995881},
	{SG59970, &UA2007376},
	{SG59971, &UA2782043},
	{SG59972, &UA3054017},
	{SG59973, &UA2710623},
	{SG59974, &UA2715426},
	{SG59975, &UA361934},
	{SG59976, &UA2463382},
	{SG59977, &UA2476068},
	{SG59978, &UA2782599},
	{SG59980, &UA2944539},
	{SG59981, &UA2987891},
	{SG59982, &UA2988672},
	{SG59983, &UA3059835},
	{SG59984, &UA3086506},
	{SG59985, &UA3123945},
	{SG59986, &UA95592},
	{SG59987, &UA114717},
	{SG59988, &UA135124},
	{SG59989, &UA3098754},
	{SG59990, &UA2260203},
	{SG59991, &UA2368900},
	{SG59992, &UA2426522},
	{SG59993, &UA2852011},
	{SG59994, &UA2563213},
	{SG60000, &UA3028396},
	{SG60001, &UA3078799},
	{SG60004, &UA2781660},
	{SG60005, &UA2958986},
	{SG60006, &UA2687271},
	{SG60007, &UA2979165},
	{SG60008, &UA2426484},
	{SG60009, &UA2458594},
	{SG60010, &UA2469475},
	{SG60011, &UA2848826},
	{SG60012, &UA2708380},
	{SG60013, &UA3060137},
	{SG60014, &UA3109550},
	{SG60015, &UA3129326},
	{SG60030, &UA3095648},
	{SG60031, &UA3129901},
	{SG60033, &UA2202181},
	{SG60034, &UA2237425},
	{SG60035, &UA2237717},
	{SG60036, &UA2357070},
	{SG60037, &UA2357098},
	{SG60038, &UA2386725},
	{SG60039, &UA2406822},
	{SG60040, &UA2406823},
	{SG60041, &UA2505651},
	{SG60042, &UA2470289},
	{SG60043, &UA2486504},
	{SG60044, &UA2868093},
	{SG60045, &UA2876348},
	{SG60046, &UA2879250},
	{SG60047, &UA2960631},
	{SG60048, &UA3006563},
	{SG60050, &UA2355927},
	{SG60051, &UA2357081},
	{SG60052, &UA2428552},
	{SG60053, &UA2733795},
	{SG60054, &UA2737799},
	{SG60055, &UA2752419},
	{SG60056, &UA2752685},
	{SG60057, &UA2768724},
	{SG60058, &UA2859235},
	{SG60059, &UA2861465},
	{SG60060, &UA2666887},
	{SG60061, &UA2667057},
	{SG60062, &UA2667669},
	{SG60063, &UA2534670},
	{SG60064, &UA2566081},
	{SG60065, &UA2672550},
	{SG60066, &UA2780896},
	{SG60067, &UA2862373},
	{SG60068, &UA2868156},
	{SG60069, &UA2510607},
	{SG60070, &UA2511048},
	{SG60071, &UA2533372},
	{SG60072, &UA2536831},
	{SG60073, &UA2581940},
	{SG60074, &UA2606832},
	{SG60075, &UA2568288},
	{SG60076, &UA2599484},
	{SG60077, &UA2857742},
	{SG60078, &UA3071937},
	{SG60079, &UA2533213},
	{SG60080, &UA2700578},
	{SG60081, &UA2602913},
	{SG60082, &UA2679634},
	{SG60083, &UA2716114},
	{SG60084, &UA2860960},
	{SG60085, &UA2865851},
	{SG60086, &UA924067},
	{SG60087, &UA1957635},
	{SG60088, &UA1978056},
	{SG60089, &UA2686632},
	{SG60105, &UA2562077},
	{SG60107, &UA330419},
	{SG60108, &UA540710},
	{SG60109, &UA729372},
	{SG60110, &UA1897586},
	{SG60111, &UA2260684},
	{SG60112, &UA2288235},
	{SG60113, &UA2442077},
	{SG60114, &UA2495000},
	{SG60115, &UA2495142},
	{SG60117, &UA3116585},
	{SG60118, &UA3116586},
	{SG60119, &UA3116594},
	{SG60120, &UA3129530},
	{SG60121, &UA2668244},
	{SG60122, &UA2737643},
	{SG60123, &UA2780215},
	{SG60124, &UA3073370},
	{SG60128, &UA2168025},
	{SG60129, &UA2260140},
	{SG60146, &UA2456489},
	{SG60150, &UA2436508},
	{SG60151, &UA2437638},
	{SG60152, &UA1950125},
	{SG60153, &UA1956782},
	{SG60173, &UA2319604},
	{SG60174, &UA2441992},
	{SG60175, &UA2639920},
	{SG60176, &UA2685921},
	{SG60177, &UA2799043},
	{SG60178, &UA2892879},
	{SG60179, &UA3050168},
	{SG60180, &UA3074193},
	{SG60181, &UA3021613},
	{SG60182, &UA3058874},
	{SG60183, &UA3059080},
	{SG60184, &UA3079245},
	{SG60185, &UA3079797},
	{SG60186, &UA3080999},
	{SG60187, &UA454598},
	{SG60188, &UA741806},
	{SG60189, &UA798003},
	{SG60190, &UA2222483},
	{SG60191, &UA160948},
	{SG60192, &UA2567858},
	{SG60193, &UA2703024},
	{SG60194, &UA2879591},
	{SG60195, &UA156750},
	{SG60196, &UA293357},
	{SG60197, &UA400084},
	{SG60198, &UA2784787},
	{SG60199, &UA2202772},
	{SG60200, &UA2237426},
	{SG60201, &UA2358008},
	{SG60202, &UA2387355},
	{SG60203, &UA2406826},
	{SG60204, &UA2408251},
	{SG60206, &UA2426942},
	{SG60207, &UA2428862},
	{SG60208, &UA2429357},
	{SG60209, &UA2468912},
	{SG60210, &UA2737389},
	{SG60211, &UA2752690},
	{SG60212, &UA2768642},
	{SG60213, &UA2868656},
	{SG60214, &UA2956740},
	{SG60216, &UA2242377},
	{SG60217, &UA2356683},
	{SG60218, &UA2203825},
	{SG60219, &UA2357208},
	{SG60222, &UA1960730},
	{SG60223, &UA1993336},
	{SG60224, &UA2166640},
	{SG60225, &UA2393227},
	{SG60226, &UA2589172},
	{SG60230, &UA3110591},
	{SG60231, &UA3114497},
	{SG60232, &UA3128079},
	{SG60233, &UA3082198},
	{SG60234, &UA3080763},
	{SG60235, &UA3080767},
	{SG60236, &UA3080771},
	{SG60237, &UA3115748},
	{SG60238, &UA3116567},
	{SG60239, &UA3116569},
	{SG60240, &UA3116573},
	{SG60241, &UA29907},
	{SG60242, &UA2666226},
	{SG60243, &UA2667604},
	{SG60244, &UA2708227},
	{SG60245, &UA2858783},
	{SG60246, &UA2990674},
	{SG60247, &UA3114908},
	{SG60248, &UA3129208},
	{SG60249, &UA3083946},
	{SG60250, &UA3096036},
	{SG60254, &UA2367889},
	{SG60255, &UA2424655},
	{SG60256, &UA2503822},
	{SG60257, &UA2564952},
	{SG60258, &UA2597663},
	{SG60259, &UA2607051},
	{SG60260, &UA2704356},
	{SG60261, &UA2731676},
	{SG60262, &UA2664288},
	{SG60263, &UA2736897},
	{SG60264, &UA2997423},
	{SG60265, &UA2430495},
	{SG60266, &UA2507875},
	{SG60267, &UA2697229},
	{SG60288, &UA658603},
	{SG60289, &UA735652},
	{SG60290, &UA2490339},
	{SG60291, &UA2533341},
	{SG60292, &UA2606872},
	{SG60293, &UA3111376},
	{SG60294, &UA3130655},
	{SG60295, &UA857889},
	{SG60296, &UA1953239},
	{SG60297, &UA1957071},
	{SG60298, &UA1958567},
	{SG60299, &UA3006005},
	{SG60300, &UA3108193},
	{SG60301, &UA330716},
	{SG60307, &UA2797005},
	{SG60308, &UA2436475},
	{SG60309, &UA2260710},
	{SG60329, &UA2597882},
	{SG60330, &UA2751130},
	{SG60331, &UA2708548},
	{SG60332, &UA2937182},
	{SG60333, &UA2431368},
	{SG60334, &UA2844596},
	{SG60342, &UA93296},
	{SG60344, &UA93943},
	{SG60350, &UA586620},
	{SG60351, &UA2751495},
	{SG60352, &UA2603682},
	{SG60353, &UA2709252},
	{SG60359, &UA2566036},
	{SG60360, &UA3100525},
	{SG60361, &UA3114407},
	{SG60362, &UA3130925},
	{SG60365, &UA2964574},
	{SG60366, &UA2966314},
	{SG60367, &UA2664181},
	{SG60368, &UA2530980},
	{SG60369, &UA2535976},
	{SG60382, &UA3019387},
	{SG60383, &UA3078725},
	{SG60384, &UA3078955},
	{SG60402, &UA2261363},
	{SG60403, &UA2367406},
	{SG60404, &UA2664137},
	{SG60405, &UA2698861},
	{SG60406, &UA2703122},
	{SG60408, &UA29959},
	{SG60409, &UA106334},
	{SG60410, &UA116173},
	{SG60411, &UA2202778},
	{SG60412, &UA2237721},
	{SG60413, &UA2357867},
	{SG60414, &UA2844275},
	{SG60416, &UA2470291},
	{SG60417, &UA2470501},
	{SG60418, &UA2227395},
	{SG60419, &UA2347435},
	{SG60420, &UA2354386},
	{SG60421, &UA2384718},
	{SG60424, &UA2422531},
	{SG60426, &UA2881213},
	{SG60427, &UA2937719},
	{SG60428, &UA1995265},
	{SG60429, &UA2007375},
	{SG60430, &UA2327759},
	{SG60431, &UA564122},
	{SG60432, &UA689218},
	{SG60433, &UA2143107},
	{SG60434, &UA2370097},
	{SG60435, &UA2438805},
	{SG60436, &UA2786259},
	{SG60438, &UA2869898},
	{SG60439, &UA2897085},
	{SG60440, &UA2945466},
	{SG60441, &UA3013776},
	{SG60442, &UA1992203},
	{SG60443, &UA2156879},
	{SG60444, &UA2002389},
	{SG60445, &UA2185510},
	{SG60446, &UA2346590},
	{SG60447, &UA2659081},
	{SG60448, &UA90895},
	{SG60449, &UA256643},
	{SG60452, &UA87653},
	{SG60453, &UA96805},
	{SG60454, &UA715202},
	{SG60455, &UA722894},
	{SG60456, &UA2949935},
	{SG60457, &UA3098731},
	{SG60458, &UA2429445},
	{SG60459, &UA2435656},
	{SG60462, &UA3013353},
	{SG60463, &UA3107456},
	{SG60464, &UA2000094},
	{SG60465, &UA2200395},
	{SG60470, &UA2712358},
	{SG60471, &UA2734125},
	{SG60472, &UA2752491},
	{SG60473, &UA2781852},
	{SG60474, &UA1962436},
	{SG60475, &UA1978131},
	{SG60477, &UA2783304},
	{SG60478, &UA2667399},
	{SG60479, &UA2702724},
	{SG60480, &UA2863724},
	{SG60483, &UA3057388},
	{SG60484, &UA3077719},
	{SG60485, &UA3077900},
	{SG60486, &UA3078516},
	{SG60495, &UA615376},
	{SG60496, &UA629971},
	{SG60497, &UA685667},
	{SG60498, &UA2355740},
	{SG60499, &UA2790200},
	{SG60500, &UA2426819},
	{SG60501, &UA2467812},
	{SG60502, &UA2486804},
	{SG60503, &UA2848968},
	{SG60504, &UA664342},
	{SG60505, &UA823387},
	{SG60506, &UA2354788},
	{SG60507, &UA3065672},
	{SG60508, &UA2863760},
	{SG60509, &UA2361090},
	{SG60510, &UA2368450},
	{SG60511, &UA3088906},
	{SG60512, &UA3103598},
	{SG60513, &UA3107591},
	{SG60514, &UA2202192},
	{SG60515, &UA2237427},
	{SG60516, &UA2367351},
	{SG60517, &UA3106292},
	{SG60518, &UA3111312},
	{SG60519, &UA3112157},
	{SG60520, &UA3129992},
	{SG60521, &UA2700752},
	{SG60522, &UA2705041},
	{SG60523, &UA2795230},
	{SG60524, &UA2179585},
	{SG60525, &UA2230437},
	{SG60526, &UA2870625},
	{SG60527, &UA2869891},
	{SG60528, &UA2879253},
	{SG60529, &UA3030927},
	{SG60530, &UA3108535},
	{SG60531, &UA911663},
	{SG60532, &UA2721495},
	{SG60534, &UA2235412},
	{SG60535, &UA2367790},
	{SG60536, &UA2734225},
	{SG60537, &UA2779782},
	{SG60538, &UA2988533},
	{SG60539, &UA3113418},
	{SG60540, &UA3128355},
	{SG60541, &UA1992576},
	{SG60542, &UA648612},
	{SG60543, &UA915570},
	{SG60544, &UA2714618},
	{SG60547, &UA2356038},
	{SG60548, &UA2362871},
	{SG60551, &UA3108649},
	{SG60553, &UA3128584},
	{SG60555, &UA359655},
	{SG60556, &UA587057},
	{SG60557, &UA730080},
	{SG60558, &UA2150577},
	{SG60559, &UA2434317},
	{SG60560, &UA2435899},
	{SG60561, &UA2509404},
	{SG60562, &UA2534853},
	{SG60563, &UA2860785},
	{SG60566, &UA3068882},
	{SG60567, &UA3068884},
	{SG60568, &UA3068885},
	{SG60569, &UA3068886},
	{SG60570, &UA3068887},
	{SG60571, &UA3068888},
	{SG60572, &UA3068890},
	{SG60573, &UA3068889},
	{SG60574, &UA2503627},
	{SG60575, &UA2654325},
	{SG60576, &UA670941},
	{SG60577, &UA2309356},
	{SG60578, &UA2967831},
	{SG60579, &UA3019705},
	{SG60581, &UA2426331},
	{SG60582, &UA2845768},
	{SG60583, &UA2748138},
	{SG60586, &UA921601},
	{SG60587, &UA2797454},
	{SG60588, &UA2429149},
	{SG60589, &UA2583228},
	{SG60590, &UA613171},
	{SG60591, &UA680550},
	{SG60592, &UA905745},
	{SG60593, &UA1995526},
	{SG60594, &UA2163901},
	{SG60595, &UA2332124},
	{SG60596, &UA2167030},
	{SG60599, &UA2751619},
	{SG60600, &UA2769704},
	{SG60606, &UA2869726},
	{SG60607, &UA2964566},
	{SG60608, &UA3092946},
	{SG60610, &UA415512},
	{SG60611, &UA495341},
	{SG60616, &UA605141},
	{SG60617, &UA669262},
	{SG60618, &UA1957300},
	{SG60619, &UA2242237},
	{SG60620, &UA2395045},
	{SG60624, &UA2261329},
	{SG60630, &UA2748787},
	{SG60631, &UA2854283},
	{SG60632, &UA3110658},
	{SG60633, &UA3130059},
	{SG60636, &UA2537250},
	{SG60637, &UA2601095},
	{SG60638, &UA2766380},
	{SG60641, &UA278847},
	{SG60642, &UA2433562},
	{SG60648, &UA87821},
	{SG60649, &UA87856},
	{SG60650, &UA500370},
	{SG60651, &UA692260},
	{SG60652, &UA2719331},
	{SG60653, &UA87002},
	{SG60654, &UA2265673},
	{SG60658, &UA3018367},
	{SG60659, &UA3020486},
	{SG60660, &UA3091925},
	{SG60661, &UA2598744},
	{SG60675, &UA3109072},
	{SG60676, &UA3130774},
	{SG60677, &UA2155889},
	{SG60678, &UA2413383},
	{SG60679, &UA2809153},
	{SG60680, &UA455992},
	{SG60681, &UA2869299},
	{SG60682, &UA2966912},
	{SG60683, &UA3079675},
	{SG60684, &UA848599},
	{SG60685, &UA856671},
	{SG60686, &UA2168219},
	{SG60687, &UA2416432},
	{SG60688, &UA2260809},
	{SG60689, &UA3065677},
	{SG60696, &UA2854306},
	{SG60697, &UA2891192},
	{SG60698, &UA2999854},
	{SG60700, &UA1915813},
	{SG60701, &UA2439305},
	{SG60705, &UA3058902},
	{SG60706, &UA3079696},
	{SG60707, &UA601844},
	{SG60708, &UA757129},
	{SG60709, &UA2701015},
	{SG60710, &UA2781937},
	{SG60711, &UA2423098},
	{SG60712, &UA2503371},
	{SG60713, &UA2844049},
	{SG60714, &UA2706156},
	{SG60715, &UA2430404},
	{SG60722, &UA2214858},
	{SG60724, &UA2676297},
	{SG60730, &UA454788},
	{SG60731, &UA454787},
	{SG60732, &UA883142},
	{SG60733, &UA2668416},
	{SG60734, &UA2793555},
	{SG60737, &UA3068898},
	{SG60738, &UA3068900},
	{SG60739, &UA3068901},
	{SG60740, &UA3068902},
	{SG60741, &UA3068904},
	{SG60742, &UA3068903},
	{SG60743, &UA2007708},
	{SG60744, &UA2837214},
	{SG60745, &UA2423705},
	{SG60751, &UA3015516},
	{SG60752, &UA3057616},
	{SG60753, &UA3059014},
	{SG60754, &UA3125661},
	{SG60755, &UA3126811},
	{SG60756, &UA3127031},
	{SG60757, &UA652945},
	{SG60758, &UA2537480},
	{SG60759, &UA2991129},
	{SG60760, &UA3071541},
	{SG60761, &UA3100046},
	{SG60762, &UA3113423},
	{SG60763, &UA3116560},
	{SG60764, &UA440662},
	{SG60765, &UA2351508},
	{SG60766, &UA2466438},
	{SG60772, &UA3059096},
	{SG60773, &UA3059592},
	{SG60774, &UA2200545},
	{SG60775, &UA2840546},
	{SG60776, &UA2664837},
	{SG60781, &UA2670828},
	{SG60782, &UA2716712},
	{SG60787, &UA603693},
	{SG60788, &UA604159},
	{SG60789, &UA720262},
	{SG60790, &UA2433539},
	{SG60791, &UA868065},
	{SG60792, &UA868895},
	{SG60793, &UA2167235},
	{SG60794, &UA2746739},
	{SG60800, &UA3059742},
	{SG60802, &UA3078767},
	{SG60805, &UA762323},
	{SG60806, &UA1994753},
	{SG60807, &UA630664},
	{SG60808, &UA661766},
	{SG60814, &UA2385621},
	{SG60815, &UA2738004},
	{SG60816, &UA2751371},
	{SG60818, &UA3110015},
	{SG60819, &UA3131024},
	{SG60830, &UA2662014},
	{SG60831, &UA3091807},
	{SG60832, &UA3111494},
	{SG60837, &UA2580118},
	{SG60838, &UA2563479},
	{SG60839, &UA2531302},
	{SG60840, &UA833617},
	{SG60841, &UA1985731},
	{SG60842, &UA1987560},
	{SG60843, &UA2384584},
	{SG60844, &UA2692985},
	{SG60854, &UA714014},
	{SG60855, &UA1983938},
	{SG60857, &UA2169790},
	{SG60859, &UA2340568},
	{SG60862, &UA2938598},
	{SG60863, &UA691419},
	{SG60864, &UA745253},
	{SG60865, &UA2333370},
	{SG60866, &UA2998020},
	{SG60867, &UA428489},
	{SG60868, &UA1986794},
	{SG60869, &UA535449},
	{SG60870, &UA766996},
	{SG60871, &UA806076},
	{SG60873, &UA863146},
	{SG60874, &UA2208370},
	{SG60875, &UA588183},
	{SG60876, &UA684039},
	{SG60879, &UA2528822},
	{SG60880, &UA3095995},
	{SG60881, &UA363703},
	{SG60882, &UA1940031},
	{SG60888, &UA3016696},
	{SG60889, &UA3110398},
	{SG60892, &UA156767},
	{SG60893, &UA163570},
	{SG60902, &UA3015107},
	{SG60903, &UA657982},
	{SG60904, &UA882055},
	{SG60905, &UA1997064},
	{SG60906, &UA1975856},
	{SG60908, &UA344396},
	{SG60909, &UA649150},
	{SG60910, &UA2313371},
	{SG60912, &UA2594856},
	{SG60913, &UA2790276},
	{SG60914, &UA2749411},
	{SG60915, &UA2780952},
	{SG60916, &UA2937507},
	{SG60917, &UA2849656},
	{SG60918, &UA3073214},
	{SG60923, &UA2426780},
	{SG60924, &UA2836442},
	{SG60925, &UA2642517},
	{SG60926, &UA2840941},
	{SG60927, &UA2992478},
	{SG60928, &UA3047202},
	{SG60929, &UA363790},
	{SG60930, &UA723161},
	{SG60932, &UA2865171},
	{SG60945, &UA211961},
	{SG60946, &UA317574},
	{SG60947, &UA613137},
	{SG60948, &UA687248},
	{SG60949, &UA817600},
	{SG60950, &UA675066},
	{SG60951, &UA2949213},
	{SG60956, &UA793199},
	{SG60957, &UA1967113},
	{SG60958, &UA1962575},
	{SG60959, &UA2580293},
	{SG60962, &UA2861741},
	{SG60969, &UA2478326},
	{SG60970, &UA912205},
	{SG60971, &UA2157950},
	{SG60972, &UA2158991},
	{SG60973, &UA2314084},
	{SG60974, &UA87074},
	{SG60975, &UA606328},
	{SG60976, &UA838053},
	{SG60981, &UA2869012},
	{SG60982, &UA2003250},
	{SG60983, &UA2481568},
	{SG60986, &UA912226},
	{SG60987, &UA1957344},
	{SG60994, &UA2400675},
	{SG60995, &UA2475772},
	{SG61000, &UA613190},
	{SG61001, &UA2328301},
	{SG61003, &UA3116583},
	{SG61004, &UA3116588},
	{SG61005, &UA3116591},
	{SG61006, &UA3116592},
	{SG61007, &UA3116595},
	{SG61014, &UA906857},
	{SG61015, &UA1965228},
	{SG61016, &UA2403348},
	{SG61018, &UA2455442},
	{SG61023, &UA3087300},
	{SG61026, &UA3128233},
	{SG61027, &UA2422264},
	{SG61030, &UA2722463},
	{SG61031, &UA2795067},
	{SG61034, &UA2951184},
	{SG61037, &UA3127833},
	{SG61038, &UA2557497},
	{SG61039, &UA2581803},
	{SG61041, &UA2726964},
	{SG61042, &UA2862443},
	{SG61046, &UA3128159},
	{SG61047, &UA102799},
	{SG61048, &UA612348},
	{SG61049, &UA615155},
	{SG61050, &UA656512},
	{SG61051, &UA2527291},
	{SG61054, &UA3121510},
	{SG61056, &UA2372847},
	{SG61057, &UA2427033},
	{SG61058, &UA2864042},
	{SG61059, &UA2983200},
	{SG61060, &UA2991621},
	{SG61061, &UA2993305},
	{SG61063, &UA3092436},
	{SG61064, &UA3109161},
	{SG61067, &UA578662},
	{SG61068, &UA754216},
	{SG61073, &UA2840604},
	{SG61074, &UA2860358},
	{SG61075, &UA3087428},
	{SG61076, &UA3093142},
	{SG61077, &UA3099767},
	{SG61080, &UA3128129},
	{SG61081, &UA717204},
	{SG61082, &UA2209125},
	{SG61083, &UA849990},
	{SG61084, &UA1992217},
	{SG61085, &UA2411454},
	{SG61086, &UA2721259},
	{SG61087, &UA2868608},
	{SG61089, &UA3049417},
	{SG61090, &UA3086523},
	{SG61095, &UA3130826},
	{SG61101, &UA2835084},
	{SG61102, &UA2844149},
	{SG61104, &UA2443566},
	{SG61107, &UA2865530},
	{SG61108, &UA359734},
	{SG61109, &UA814285},
	{SG61110, &UA2325836},
	{SG61111, &UA2167872},
	{SG61112, &UA2333782},
	{SG61113, &UA2821008},
	{SG61114, &UA654463},
	{SG61115, &UA805581},
	{SG61116, &UA2411645},
	{SG61117, &UA3085451},
	{SG61120, &UA2549234},
	{SG61121, &UA2830147},
	{SG61124, &UA2835496},
	{SG61125, &UA3068893},
	{SG61126, &UA2156414},
	{SG61127, &UA2348891},
	{SG61128, &UA2350358},
	{SG61133, &UA615543},
	{SG61134, &UA626061},
	{SG61135, &UA617361},
	{SG61136, &UA1949474},
	{SG61137, &UA2260798},
	{SG61140, &UA2270227},
	{SG61141, &UA616847},
	{SG61142, &UA422368},
	{SG61143, &UA589923},
	{SG61144, &UA599960},
	{SG61145, &UA617723},
	{SG61146, &UA617000},
	{SG61147, &UA2201843},
	{SG61152, &UA351273},
	{SG61153, &UA2304770},
	{SG61154, &UA2309263},
	{SG61155, &UA2320457},
	{SG61156, &UA350481},
	{SG61157, &UA360853},
	{SG61158, &UA616954},
	{SG61159, &UA618253},
	{SG61160, &UA1845573},
	{SG61161, &UA1935012},
	{SG61162, &UA2168055},
	{SG61163, &UA2261191},
	{SG61164, &UA2758419},
	{SG61165, &UA3068875},
	{SG61166, &UA3068876},
	{SG61167, &UA3068878},
	{SG61168, &UA3068879},
	{SG61171, &UA617005},
	{SG61172, &UA617273},
	{SG61173, &UA618120},
	{SG61174, &UA618278},
	{SG61175, &UA833380},
	{SG61177, &UA2200479},
	{SG61178, &UA2333719},
	{SG61181, &UA456157},
	{SG61182, &UA517901},
	{SG61183, &UA849701},
	{SG61184, &UA870547},
	{SG61186, &UA2149435},
	{SG61187, &UA2221857},
	{SG61189, &UA3116561},
	{SG61191, &UA87076},
	{SG61192, &UA619061},
	{SG61193, &UA2220114},
	{SG61194, &UA708937},
	{SG61195, &UA765910},
	{SG61196, &UA1944914},
	{SG61198, &UA3068877},
	{SG61199, &UA3068880},
	{SG61200, &UA3068881},
	{SG61201, &UA456232},
	{SG61202, &UA622003},
	{SG61203, &UA617039},
	{SG61204, &UA617606},
	{SG61205, &UA1957385},
	{SG61208, &UA2162262},
	{SG61210, &UA3068892},
	{SG61211, &UA3068895},
	{SG61212, &UA3068896},
	{SG61213, &UA3068897},
	{SG61214, &UA3068905},
	{SG61218, &UA440999},
	{SG61220, &UA618246},
	{SG61221, &UA618268},
	{SG61222, &UA1987900},
	{SG61223, &UA2334789},
	{SG61228, &UA590231},
	{SG61229, &UA618245},
	{SG61230, &UA618381},
	{SG61231, &UA619803},
	{SG61232, &UA622329},
	{SG61233, &UA1959578},
	{SG61237, &UA3116558},
	{SG61238, &UA3116559},
	{SG61243, &UA607209},
	{SG61244, &UA618788},
	{SG61245, &UA619499},
	{SG61246, &UA621669},
	{SG61247, &UA621973},
	{SG61248, &UA622177},
	{SG61249, &UA622023},
	{SG61250, &UA622403},
	{SG61251, &UA622413},
	{SG61252, &UA622355},
	{SG61253, &UA622481},
	{SG61254, &UA622449},
	{SG61255, &UA899664},
	{SG61256, &UA1986102},
	{SG61260, &UA3068891},
	{SG61261, &UA3068894},
	{SG61262, &UA3068899},
	{SG61263, &UA3068977},
	{SG61264, &UA3068978},
	{SG61265, &UA3068979},
	{SG61266, &UA3068980},
	{SG61267, &UA622126},
	{NULL}};
const Device *DBenQ[] = {
	&UA81333,
	&UA89123,
	&UA1837097,
	&UA22430,
	&UA68585,
	&UA3540,
	&UA68562,
	&UA50138,
	&UA50158,
	&UA68565,
	&UA68580,
	&UA33454,
	&UA55026,
	&UA50474,
	&UA27810,
	&UA27811,
	&UA33344,
	&UA15868,
	&UA68032,
	&UA84946,
	NULL};
const Segmentdata DBolt[] = {
	{SG57247, &UA252700},
	{SG57248, &UA668399},
	{SG57721, &UA2409113},
	{SG59009, &UA2536822},
	{NULL}};
const Segmentdata DBot[] = {
	{SG55443, &UA86117},
	{SG55444, &UA92227},
	{SG55445, &UA92684},
	{SG55446, &UA92696},
	{SG55447, &UA92703},
	{SG55448, &UA92771},
	{SG55449, &UA92922},
	{SG55450, &UA93018},
	{SG55451, &UA93041},
	{SG55452, &UA93281},
	{SG55453, &UA93331},
	{SG55454, &UA93630},
	{SG55455, &UA93937},
	{SG55456, &UA93959},
	{SG55457, &UA93920},
	{SG55458, &UA94010},
	{SG55459, &UA94407},
	{SG55460, &UA94697},
	{SG55461, &UA94694},
	{SG55462, &UA95232},
	{SG55463, &UA95382},
	{SG55464, &UA95898},
	{SG55465, &UA97810},
	{SG55466, &UA97822},
	{SG55467, &UA98196},
	{SG55468, &UA98197},
	{SG55469, &UA98214},
	{SG55470, &UA98329},
	{SG55471, &UA98473},
	{SG55472, &UA98698},
	{SG55473, &UA98809},
	{SG55474, &UA98991},
	{SG55475, &UA99006},
	{SG55477, &UA99130},
	{SG55478, &UA99137},
	{SG55479, &UA99276},
	{SG55480, &UA99610},
	{SG55481, &UA99789},
	{SG55482, &UA100643},
	{SG55483, &UA100709},
	{SG55484, &UA101266},
	{SG55485, &UA101986},
	{SG55486, &UA102824},
	{SG55487, &UA102876},
	{SG55488, &UA102981},
	{SG55489, &UA103116},
	{SG55490, &UA103322},
	{SG55491, &UA104246},
	{SG55492, &UA104662},
	{SG55493, &UA104738},
	{SG55494, &UA105242},
	{SG55495, &UA105346},
	{SG55496, &UA105715},
	{SG55497, &UA106427},
	{SG55498, &UA107482},
	{SG55500, &UA107611},
	{SG55501, &UA107898},
	{SG55502, &UA108091},
	{SG55503, &UA108126},
	{SG55504, &UA112224},
	{SG55505, &UA112637},
	{SG55506, &UA113147},
	{SG55507, &UA114535},
	{SG55508, &UA115001},
	{SG55509, &UA116076},
	{SG55510, &UA116158},
	{SG55511, &UA116738},
	{SG55512, &UA120993},
	{SG55513, &UA121000},
	{SG55514, &UA121179},
	{SG55515, &UA121287},
	{SG55516, &UA121628},
	{SG55517, &UA121908},
	{SG55518, &UA121924},
	{SG55519, &UA122489},
	{SG55520, &UA123761},
	{SG55521, &UA123833},
	{SG55522, &UA124455},
	{SG55523, &UA125680},
	{SG55524, &UA126440},
	{SG55525, &UA126658},
	{SG55546, &UA126814},
	{SG55547, &UA127221},
	{SG55548, &UA129118},
	{SG55549, &UA130129},
	{SG55550, &UA130423},
	{SG55551, &UA130976},
	{SG55552, &UA131005},
	{SG55553, &UA131793},
	{SG55554, &UA131816},
	{SG55555, &UA131935},
	{SG55556, &UA132001},
	{SG55557, &UA132271},
	{SG55558, &UA133171},
	{SG55559, &UA134084},
	{SG55560, &UA134137},
	{SG55561, &UA135086},
	{SG55562, &UA135424},
	{SG55563, &UA135917},
	{SG55564, &UA138229},
	{SG55565, &UA139140},
	{SG55566, &UA139586},
	{SG55567, &UA140940},
	{SG55568, &UA144477},
	{SG55569, &UA146478},
	{SG55570, &UA147692},
	{SG55571, &UA151554},
	{SG55572, &UA151901},
	{SG55573, &UA152025},
	{SG55574, &UA153187},
	{SG55575, &UA153680},
	{SG55576, &UA154010},
	{SG55577, &UA154518},
	{SG55578, &UA154959},
	{SG55579, &UA155557},
	{SG55580, &UA156993},
	{SG55581, &UA160837},
	{SG55582, &UA162400},
	{SG55583, &UA162787},
	{SG55584, &UA163219},
	{SG55585, &UA164159},
	{SG55586, &UA164244},
	{SG55587, &UA164363},
	{SG55588, &UA165364},
	{SG55589, &UA167093},
	{SG55590, &UA167353},
	{SG55591, &UA167726},
	{SG55592, &UA168354},
	{SG55593, &UA168406},
	{SG55594, &UA169932},
	{SG55595, &UA170637},
	{SG55596, &UA170643},
	{SG55597, &UA170764},
	{SG55598, &UA171843},
	{SG55599, &UA172055},
	{SG55600, &UA173812},
	{SG55601, &UA173868},
	{SG55602, &UA174933},
	{SG55603, &UA175065},
	{SG55604, &UA176043},
	{SG55605, &UA180078},
	{SG55606, &UA180238},
	{SG55607, &UA180351},
	{SG55608, &UA185011},
	{SG55609, &UA187503},
	{SG55610, &UA188947},
	{SG55611, &UA189261},
	{SG55612, &UA189845},
	{SG55613, &UA189854},
	{SG55614, &UA190718},
	{SG55615, &UA191802},
	{SG55616, &UA196999},
	{SG55617, &UA199707},
	{SG55618, &UA200702},
	{SG55619, &UA204265},
	{SG55620, &UA204280},
	{SG55621, &UA204423},
	{SG55622, &UA204569},
	{SG55623, &UA205242},
	{SG55624, &UA205473},
	{SG55625, &UA206349},
	{SG55626, &UA206992},
	{SG55627, &UA208049},
	{SG55628, &UA208228},
	{SG55629, &UA212615},
	{SG55630, &UA212817},
	{SG55631, &UA215856},
	{SG55632, &UA219041},
	{SG55633, &UA224167},
	{SG55634, &UA227398},
	{SG55635, &UA228024},
	{SG55636, &UA228355},
	{SG55638, &UA230032},
	{SG55640, &UA231217},
	{SG55641, &UA232862},
	{SG55643, &UA234064},
	{SG55644, &UA239849},
	{SG55646, &UA240971},
	{SG55649, &UA241736},
	{SG55651, &UA241763},
	{SG55653, &UA242569},
	{SG55654, &UA243888},
	{SG55655, &UA248952},
	{SG55656, &UA249443},
	{SG55657, &UA250303},
	{SG55658, &UA250774},
	{SG55659, &UA250842},
	{SG55660, &UA252058},
	{SG55661, &UA252558},
	{SG55662, &UA252817},
	{SG55663, &UA253398},
	{SG55664, &UA255846},
	{SG55665, &UA257307},
	{SG55666, &UA257583},
	{SG55667, &UA258026},
	{SG55668, &UA258031},
	{SG55669, &UA259488},
	{SG55670, &UA259503},
	{SG55671, &UA263838},
	{SG55672, &UA266413},
	{SG55673, &UA269471},
	{SG55674, &UA270353},
	{SG55675, &UA274750},
	{SG55676, &UA274967},
	{SG55677, &UA274973},
	{SG55678, &UA275321},
	{SG55679, &UA277680},
	{SG55680, &UA281734},
	{SG55681, &UA294794},
	{SG55682, &UA300739},
	{SG55683, &UA303349},
	{SG55684, &UA303835},
	{SG55685, &UA304097},
	{SG55686, &UA304770},
	{SG55687, &UA305971},
	{SG55688, &UA306405},
	{SG55689, &UA311680},
	{SG55690, &UA317805},
	{SG55691, &UA318302},
	{SG55692, &UA320062},
	{SG55693, &UA326282},
	{SG55694, &UA329806},
	{SG55695, &UA335592},
	{SG55696, &UA336003},
	{SG55697, &UA344541},
	{SG55698, &UA345461},
	{SG55699, &UA346131},
	{SG55700, &UA346316},
	{SG55701, &UA346834},
	{SG55702, &UA347086},
	{SG55703, &UA347112},
	{SG55705, &UA347686},
	{SG55706, &UA347748},
	{SG55707, &UA349477},
	{SG55708, &UA349594},
	{SG55709, &UA350598},
	{SG55710, &UA350724},
	{SG55711, &UA351684},
	{SG55712, &UA352092},
	{SG55713, &UA352364},
	{SG55714, &UA352795},
	{SG55752, &UA359463},
	{SG55753, &UA359701},
	{SG55754, &UA363681},
	{SG55755, &UA365863},
	{SG55756, &UA366662},
	{SG55757, &UA367531},
	{SG55758, &UA372187},
	{SG55759, &UA372602},
	{SG55760, &UA374290},
	{SG55761, &UA375119},
	{SG55762, &UA375932},
	{SG55763, &UA377097},
	{SG55764, &UA377603},
	{SG55765, &UA378123},
	{SG55766, &UA378247},
	{SG55767, &UA382220},
	{SG55769, &UA382239},
	{SG55770, &UA382208},
	{SG55771, &UA382273},
	{SG55772, &UA382342},
	{SG55773, &UA382372},
	{SG55774, &UA382664},
	{SG55775, &UA383037},
	{SG55776, &UA383783},
	{SG55777, &UA383945},
	{SG55778, &UA387790},
	{SG55779, &UA389154},
	{SG55781, &UA389919},
	{SG55782, &UA389931},
	{SG55783, &UA391889},
	{SG55784, &UA396555},
	{SG55785, &UA396971},
	{SG55786, &UA400305},
	{SG55787, &UA401277},
	{SG55788, &UA401362},
	{SG55789, &UA402007},
	{SG55790, &UA402527},
	{SG55791, &UA407893},
	{SG55793, &UA408642},
	{SG55794, &UA409379},
	{SG55795, &UA412994},
	{SG55796, &UA413500},
	{SG55797, &UA413577},
	{SG55798, &UA413736},
	{SG55799, &UA416026},
	{SG55801, &UA416844},
	{SG55802, &UA417294},
	{SG55803, &UA417410},
	{SG55804, &UA419140},
	{SG55805, &UA419588},
	{SG55806, &UA421151},
	{SG55807, &UA422689},
	{SG55808, &UA423742},
	{SG55809, &UA427869},
	{SG55810, &UA428371},
	{SG55811, &UA429980},
	{SG55812, &UA430451},
	{SG55813, &UA432724},
	{SG55815, &UA432975},
	{SG55816, &UA433000},
	{SG55817, &UA433694},
	{SG55818, &UA433766},
	{SG55819, &UA436848},
	{SG55820, &UA436966},
	{SG55821, &UA437315},
	{SG55822, &UA437390},
	{SG55823, &UA437508},
	{SG55824, &UA438782},
	{SG55825, &UA438950},
	{SG55826, &UA439601},
	{SG55827, &UA441023},
	{SG55828, &UA442099},
	{SG55829, &UA443538},
	{SG55830, &UA443904},
	{SG55831, &UA444360},
	{SG55832, &UA444910},
	{SG55833, &UA448171},
	{SG55834, &UA449754},
	{SG55835, &UA450154},
	{SG55836, &UA452245},
	{SG55837, &UA452666},
	{SG55838, &UA452703},
	{SG55839, &UA452785},
	{SG55840, &UA454070},
	{SG55841, &UA454981},
	{SG55843, &UA455963},
	{SG55844, &UA456558},
	{SG55845, &UA456630},
	{SG55846, &UA457546},
	{SG55847, &UA459457},
	{SG55848, &UA459756},
	{SG55849, &UA459917},
	{SG55850, &UA460000},
	{SG55851, &UA460354},
	{SG55853, &UA460360},
	{SG55854, &UA460371},
	{SG55855, &UA460413},
	{SG55856, &UA460784},
	{SG55857, &UA461159},
	{SG55858, &UA462368},
	{SG55859, &UA468862},
	{SG55860, &UA473408},
	{SG55861, &UA475285},
	{SG55862, &UA475788},
	{SG55863, &UA482876},
	{SG55864, &UA485145},
	{SG55865, &UA493410},
	{SG55866, &UA493490},
	{SG55867, &UA502573},
	{SG55868, &UA503451},
	{SG55869, &UA503505},
	{SG55870, &UA503677},
	{SG55871, &UA505417},
	{SG55872, &UA509985},
	{SG55874, &UA512003},
	{SG55875, &UA512559},
	{SG55876, &UA515291},
	{SG55878, &UA516490},
	{SG55879, &UA516713},
	{SG55880, &UA516821},
	{SG55881, &UA516868},
	{SG55882, &UA517804},
	{SG55883, &UA517820},
	{SG55884, &UA519144},
	{SG55885, &UA522103},
	{SG55886, &UA522216},
	{SG55887, &UA522783},
	{SG55888, &UA522942},
	{SG55889, &UA523812},
	{SG55890, &UA524018},
	{SG55891, &UA524281},
	{SG55892, &UA524322},
	{SG55893, &UA525395},
	{SG55894, &UA527596},
	{SG55895, &UA530024},
	{SG55896, &UA530916},
	{SG55897, &UA530924},
	{SG55898, &UA532631},
	{SG55899, &UA533622},
	{SG55900, &UA534717},
	{SG55901, &UA536761},
	{SG55902, &UA540010},
	{SG55903, &UA540633},
	{SG55905, &UA540721},
	{SG55906, &UA545417},
	{SG55907, &UA545597},
	{SG55909, &UA548426},
	{SG55910, &UA552149},
	{SG55912, &UA552834},
	{SG55913, &UA553278},
	{SG55914, &UA558932},
	{SG55915, &UA558976},
	{SG55916, &UA563278},
	{SG55917, &UA564485},
	{SG55919, &UA568075},
	{SG55920, &UA568874},
	{SG55921, &UA571603},
	{SG55922, &UA573184},
	{SG55924, &UA573562},
	{SG55925, &UA573572},
	{SG55926, &UA574691},
	{SG55927, &UA577067},
	{SG55928, &UA577457},
	{SG55929, &UA577656},
	{SG55930, &UA584133},
	{SG55931, &UA584774},
	{SG55933, &UA588392},
	{SG55934, &UA588690},
	{SG55935, &UA591061},
	{SG55936, &UA596902},
	{SG55937, &UA599562},
	{SG55938, &UA600509},
	{SG55940, &UA605184},
	{SG55941, &UA605686},
	{SG55942, &UA608049},
	{SG55944, &UA609001},
	{SG55945, &UA609603},
	{SG55946, &UA609705},
	{SG55947, &UA610144},
	{SG55948, &UA613019},
	{SG55949, &UA614852},
	{SG55950, &UA622957},
	{SG55951, &UA622967},
	{SG55952, &UA622990},
	{SG55953, &UA622993},
	{SG55954, &UA623371},
	{SG55955, &UA624172},
	{SG55956, &UA625260},
	{SG55957, &UA625696},
	{SG55958, &UA625844},
	{SG55959, &UA627505},
	{SG55960, &UA627965},
	{SG55961, &UA628540},
	{SG55962, &UA629020},
	{SG55963, &UA629100},
	{SG55964, &UA629869},
	{SG55965, &UA632047},
	{SG55966, &UA633552},
	{SG55967, &UA634149},
	{SG55968, &UA635294},
	{SG55969, &UA636270},
	{SG55970, &UA637193},
	{SG55971, &UA637641},
	{SG55972, &UA640695},
	{SG55973, &UA642786},
	{SG55974, &UA643332},
	{SG55975, &UA645060},
	{SG55976, &UA645148},
	{SG55977, &UA645171},
	{SG55978, &UA648818},
	{SG55979, &UA649482},
	{SG55980, &UA651349},
	{SG55981, &UA652215},
	{SG55982, &UA652337},
	{SG55983, &UA659298},
	{SG55984, &UA663373},
	{SG55985, &UA663836},
	{SG55986, &UA664895},
	{SG55987, &UA664916},
	{SG55989, &UA664964},
	{SG55990, &UA666050},
	{SG55991, &UA666089},
	{SG55992, &UA667067},
	{SG55993, &UA667229},
	{SG55994, &UA667844},
	{SG55995, &UA668092},
	{SG55996, &UA671048},
	{SG55997, &UA672381},
	{SG55998, &UA677204},
	{SG55999, &UA679558},
	{SG56000, &UA688428},
	{SG56001, &UA692615},
	{SG56002, &UA692876},
	{SG56003, &UA692959},
	{SG56004, &UA696372},
	{SG56005, &UA700367},
	{SG56006, &UA704556},
	{SG56007, &UA704867},
	{SG56008, &UA708212},
	{SG56009, &UA712451},
	{SG56010, &UA721603},
	{SG56011, &UA722503},
	{SG56012, &UA726156},
	{SG56013, &UA738267},
	{SG56014, &UA744262},
	{SG56015, &UA746314},
	{SG56016, &UA748219},
	{SG56017, &UA750157},
	{SG56018, &UA752436},
	{SG56019, &UA764678},
	{SG56020, &UA766043},
	{SG56021, &UA768937},
	{SG56022, &UA773033},
	{SG56023, &UA777266},
	{SG56024, &UA778022},
	{SG56025, &UA788274},
	{SG56026, &UA789460},
	{SG56027, &UA789850},
	{SG56028, &UA801246},
	{SG56029, &UA801594},
	{SG56030, &UA820572},
	{SG56031, &UA831471},
	{SG56032, &UA836682},
	{SG56033, &UA842566},
	{SG56034, &UA847808},
	{SG56035, &UA858183},
	{SG56036, &UA862224},
	{SG56037, &UA863625},
	{SG56038, &UA864457},
	{SG56039, &UA867421},
	{SG56040, &UA868961},
	{SG56041, &UA871532},
	{SG56042, &UA871831},
	{SG56043, &UA874725},
	{SG56044, &UA884931},
	{SG56045, &UA890199},
	{SG56046, &UA891938},
	{SG56047, &UA896061},
	{SG56048, &UA896091},
	{SG56049, &UA896150},
	{SG56050, &UA897526},
	{SG56051, &UA897839},
	{SG56052, &UA900399},
	{SG56053, &UA900653},
	{SG56054, &UA900911},
	{SG56055, &UA900919},
	{SG56056, &UA901023},
	{SG56057, &UA901559},
	{SG56058, &UA901853},
	{SG56059, &UA903098},
	{SG56060, &UA908708},
	{SG56061, &UA908776},
	{SG56062, &UA909131},
	{SG56063, &UA910190},
	{SG56064, &UA911698},
	{SG56065, &UA912027},
	{SG56066, &UA912232},
	{SG56067, &UA912613},
	{SG56068, &UA913482},
	{SG56069, &UA914535},
	{SG56070, &UA914783},
	{SG56071, &UA914895},
	{SG56072, &UA923786},
	{SG56073, &UA923819},
	{SG56074, &UA923852},
	{SG56075, &UA924099},
	{SG56076, &UA924438},
	{SG56077, &UA924473},
	{SG56079, &UA924824},
	{SG56080, &UA924964},
	{SG56081, &UA925089},
	{SG56082, &UA925205},
	{SG56083, &UA925369},
	{SG56084, &UA1836241},
	{SG56085, &UA1839103},
	{SG56086, &UA1839305},
	{SG56087, &UA1840266},
	{SG56089, &UA1842254},
	{SG56090, &UA1858522},
	{SG56091, &UA1866140},
	{SG56093, &UA1866429},
	{SG56094, &UA1867016},
	{SG56096, &UA1871063},
	{SG56097, &UA1875120},
	{SG56098, &UA1875286},
	{SG56100, &UA1879191},
	{SG56101, &UA1879853},
	{SG56102, &UA1881269},
	{SG56103, &UA1882206},
	{SG56104, &UA1884043},
	{SG56105, &UA1886081},
	{SG56106, &UA1887341},
	{SG56107, &UA1893602},
	{SG56108, &UA1893763},
	{SG56109, &UA1894259},
	{SG56110, &UA1894671},
	{SG56111, &UA1894741},
	{SG56112, &UA1894852},
	{SG56113, &UA1894869},
	{SG56114, &UA1894938},
	{SG56115, &UA1895165},
	{SG56116, &UA1895355},
	{SG56117, &UA1895481},
	{SG56118, &UA1895542},
	{SG56119, &UA1895615},
	{SG56120, &UA1895726},
	{SG56121, &UA1895990},
	{SG56122, &UA1896169},
	{SG56123, &UA1896208},
	{SG56124, &UA1896238},
	{SG56125, &UA1896568},
	{SG56126, &UA1896647},
	{SG56127, &UA1896697},
	{SG56128, &UA1896915},
	{SG56129, &UA1896942},
	{SG56130, &UA1897881},
	{SG56131, &UA1898270},
	{SG56132, &UA1898620},
	{SG56133, &UA1899378},
	{SG56134, &UA1900489},
	{SG56135, &UA1900803},
	{SG56136, &UA1902727},
	{SG56137, &UA1903101},
	{SG56138, &UA1904045},
	{SG56139, &UA1904955},
	{SG56140, &UA1907659},
	{SG56141, &UA1915906},
	{SG56142, &UA1923970},
	{SG56143, &UA1925243},
	{SG56144, &UA1935965},
	{SG56145, &UA1937016},
	{SG56146, &UA1966719},
	{SG56147, &UA1966839},
	{SG56148, &UA1966932},
	{SG56149, &UA1979728},
	{SG56150, &UA1996879},
	{SG56151, &UA2003652},
	{SG56152, &UA2003902},
	{SG56153, &UA2004631},
	{SG56154, &UA2017134},
	{SG56155, &UA2017940},
	{SG56156, &UA2017972},
	{SG56157, &UA2039654},
	{SG56158, &UA2040389},
	{SG56159, &UA2040852},
	{SG56160, &UA2042610},
	{SG56161, &UA2131047},
	{SG56162, &UA2132711},
	{SG56163, &UA2132813},
	{SG56164, &UA2132991},
	{SG56165, &UA2135411},
	{SG56166, &UA2137178},
	{SG56167, &UA2137567},
	{SG56168, &UA2142690},
	{SG56169, &UA2143301},
	{SG56170, &UA2145421},
	{SG56171, &UA2145497},
	{SG56172, &UA2148809},
	{SG56173, &UA2149023},
	{SG56174, &UA2149338},
	{SG56175, &UA2150355},
	{SG56176, &UA2152813},
	{SG56177, &UA2152925},
	{SG56178, &UA2153686},
	{SG56179, &UA2156234},
	{SG56180, &UA2157259},
	{SG56181, &UA2160539},
	{SG56182, &UA2162419},
	{SG56183, &UA2163738},
	{SG56184, &UA2164770},
	{SG56185, &UA2167103},
	{SG56186, &UA2167300},
	{SG56187, &UA2167946},
	{SG56188, &UA2168044},
	{SG56189, &UA2168266},
	{SG56190, &UA2187551},
	{SG56191, &UA2193466},
	{SG56192, &UA2195630},
	{SG56193, &UA2197566},
	{SG56194, &UA2199511},
	{SG56195, &UA2200145},
	{SG56196, &UA2200527},
	{SG56197, &UA2202320},
	{SG56198, &UA2202904},
	{SG56199, &UA2203606},
	{SG56200, &UA2204074},
	{SG56201, &UA2204385},
	{SG56202, &UA2204762},
	{SG56203, &UA2206155},
	{SG56204, &UA2207226},
	{SG56205, &UA2209336},
	{SG56206, &UA2214948},
	{SG56207, &UA2216388},
	{SG56208, &UA2219501},
	{SG56209, &UA2219507},
	{SG56210, &UA2221115},
	{SG56211, &UA2223158},
	{SG56212, &UA2223365},
	{SG56213, &UA2224706},
	{SG56214, &UA2228613},
	{SG56215, &UA2228668},
	{SG56216, &UA2230326},
	{SG56217, &UA2233608},
	{SG56218, &UA2235749},
	{SG56219, &UA2236468},
	{SG56220, &UA2236573},
	{SG56221, &UA2236735},
	{SG56222, &UA2236853},
	{SG56223, &UA2238897},
	{SG56224, &UA2239295},
	{SG56225, &UA2239493},
	{SG56226, &UA2257201},
	{SG56227, &UA2257347},
	{SG56228, &UA2257579},
	{SG56229, &UA2260458},
	{SG56230, &UA2260462},
	{SG56231, &UA2260463},
	{SG56232, &UA2261757},
	{SG56233, &UA2267004},
	{SG56234, &UA2273590},
	{SG56235, &UA2275280},
	{SG56236, &UA2277968},
	{SG56237, &UA2279501},
	{SG56238, &UA2281570},
	{SG56239, &UA2285676},
	{SG56240, &UA2291031},
	{SG56241, &UA2291727},
	{SG56242, &UA2298594},
	{SG56243, &UA2300383},
	{SG56244, &UA2300685},
	{SG56245, &UA2301032},
	{SG56246, &UA2303959},
	{SG56247, &UA2308500},
	{SG56248, &UA2339435},
	{SG56249, &UA2341432},
	{SG56250, &UA2350581},
	{SG56251, &UA2355410},
	{SG56252, &UA2357152},
	{SG56253, &UA2357839},
	{SG56254, &UA2358293},
	{SG56255, &UA2359990},
	{SG56256, &UA2361721},
	{SG56257, &UA2362756},
	{SG56258, &UA2364426},
	{SG56259, &UA2368270},
	{SG56260, &UA2368852},
	{SG56261, &UA2375624},
	{SG56262, &UA2377136},
	{SG56264, &UA2384739},
	{SG56265, &UA2385002},
	{SG56266, &UA2385872},
	{SG56267, &UA2386991},
	{SG56268, &UA2387441},
	{SG56269, &UA2387785},
	{SG56270, &UA2389102},
	{SG56271, &UA2391460},
	{SG56272, &UA2391625},
	{SG56273, &UA2391721},
	{SG56274, &UA2392355},
	{SG56275, &UA2393640},
	{SG56276, &UA2394368},
	{SG56277, &UA2394396},
	{SG56278, &UA2397011},
	{SG56279, &UA2403839},
	{SG56280, &UA2403940},
	{SG56281, &UA2404474},
	{SG56282, &UA2405479},
	{SG56283, &UA2405509},
	{SG56284, &UA2407293},
	{SG56285, &UA2408505},
	{SG56286, &UA2419673},
	{SG56287, &UA2422620},
	{SG56288, &UA2424608},
	{SG56289, &UA2425067},
	{SG56290, &UA2425152},
	{SG56291, &UA2426326},
	{SG56292, &UA2427274},
	{SG56293, &UA2428102},
	{SG56294, &UA2428399},
	{SG56295, &UA2429977},
	{SG56296, &UA2430878},
	{SG56297, &UA2431858},
	{SG56298, &UA2431878},
	{SG56299, &UA2432505},
	{SG56300, &UA2433052},
	{SG56301, &UA2434621},
	{SG56302, &UA2436585},
	{SG56303, &UA2437149},
	{SG56304, &UA2437487},
	{SG56305, &UA2437705},
	{SG56306, &UA2437795},
	{SG56307, &UA2438681},
	{SG56308, &UA2438980},
	{SG56309, &UA2439516},
	{SG56310, &UA2440503},
	{SG56311, &UA2440613},
	{SG56312, &UA2443436},
	{SG56313, &UA2447745},
	{SG56314, &UA2455400},
	{SG56315, &UA2455662},
	{SG56316, &UA2455883},
	{SG56317, &UA2458334},
	{SG56318, &UA2460845},
	{SG56319, &UA2461304},
	{SG56320, &UA2461847},
	{SG56322, &UA2461892},
	{SG56324, &UA2462350},
	{SG56325, &UA2467860},
	{SG56327, &UA2470762},
	{SG56328, &UA2471623},
	{SG56330, &UA2471646},
	{SG56331, &UA2477450},
	{SG56333, &UA2477927},
	{SG56334, &UA2483461},
	{SG56336, &UA2487040},
	{SG56337, &UA2487645},
	{SG56339, &UA2487923},
	{SG56341, &UA2488342},
	{SG56343, &UA2488371},
	{SG56344, &UA2488878},
	{SG56346, &UA2489703},
	{SG56347, &UA2490291},
	{SG56349, &UA2490868},
	{SG56351, &UA2490863},
	{SG56353, &UA2491628},
	{SG56354, &UA2492687},
	{SG56355, &UA2492850},
	{SG56357, &UA2503009},
	{SG56359, &UA2504966},
	{SG56361, &UA2505873},
	{SG56363, &UA2507107},
	{SG56365, &UA2507196},
	{SG56367, &UA2508127},
	{SG56370, &UA2514116},
	{SG56371, &UA2515807},
	{SG56373, &UA2516448},
	{SG56375, &UA2516699},
	{SG56377, &UA2519335},
	{SG56379, &UA2528693},
	{SG56381, &UA2530690},
	{SG56382, &UA2531942},
	{SG56384, &UA2531933},
	{SG56386, &UA2533003},
	{SG56388, &UA2533193},
	{SG56390, &UA2533337},
	{SG56392, &UA2533363},
	{SG56393, &UA2533621},
	{SG56394, &UA2536193},
	{SG56396, &UA2536199},
	{SG56397, &UA2536294},
	{SG56398, &UA2537639},
	{SG56399, &UA2537692},
	{SG56400, &UA2539653},
	{SG56401, &UA2540874},
	{SG56402, &UA2546201},
	{SG56403, &UA2546298},
	{SG56404, &UA2547228},
	{SG56405, &UA2553010},
	{SG56406, &UA2553767},
	{SG56407, &UA2553970},
	{SG56408, &UA2557290},
	{SG56409, &UA2564096},
	{SG56410, &UA2565902},
	{SG56411, &UA2566320},
	{SG56412, &UA2566838},
	{SG56413, &UA2567489},
	{SG56414, &UA2570705},
	{SG56415, &UA2571562},
	{SG56416, &UA2572621},
	{SG56417, &UA2573149},
	{SG56418, &UA2573443},
	{SG56419, &UA2574506},
	{SG56420, &UA2576953},
	{SG56421, &UA2577494},
	{SG56422, &UA2578800},
	{SG56423, &UA2580782},
	{SG56424, &UA2581149},
	{SG56426, &UA2581379},
	{SG56428, &UA2584150},
	{SG56429, &UA2584180},
	{SG56430, &UA2584208},
	{SG56431, &UA2585625},
	{SG56432, &UA2585757},
	{SG56433, &UA2585762},
	{SG56434, &UA2587541},
	{SG56435, &UA2596440},
	{SG56436, &UA2597297},
	{SG56437, &UA2601127},
	{SG56438, &UA2601991},
	{SG56439, &UA2602947},
	{SG56440, &UA2603123},
	{SG56441, &UA2604410},
	{SG56442, &UA2604456},
	{SG56443, &UA2605396},
	{SG56444, &UA2640291},
	{SG56445, &UA2641334},
	{SG56446, &UA2641394},
	{SG56447, &UA2644303},
	{SG56448, &UA2647245},
	{SG56449, &UA2657768},
	{SG56450, &UA2659279},
	{SG56451, &UA2659278},
	{SG56452, &UA2659412},
	{SG56453, &UA2667043},
	{SG56454, &UA2672653},
	{SG56455, &UA2673220},
	{SG56456, &UA2674169},
	{SG56457, &UA2675398},
	{SG56458, &UA2675991},
	{SG56459, &UA2677128},
	{SG56460, &UA2678798},
	{SG56461, &UA2678810},
	{SG56462, &UA2679091},
	{SG56463, &UA2680948},
	{SG56464, &UA2681555},
	{SG56465, &UA2681635},
	{SG56466, &UA2681872},
	{SG56467, &UA2682716},
	{SG56468, &UA2683687},
	{SG56469, &UA2684275},
	{SG56470, &UA2684823},
	{SG56471, &UA2685121},
	{SG56472, &UA2689241},
	{SG56473, &UA2693070},
	{SG56474, &UA2693460},
	{SG56475, &UA2694864},
	{SG56476, &UA2696114},
	{SG56477, &UA2697257},
	{SG56478, &UA2699152},
	{SG56479, &UA2699320},
	{SG56480, &UA2700200},
	{SG56481, &UA2702189},
	{SG56482, &UA2702862},
	{SG56483, &UA2705051},
	{SG56484, &UA2708590},
	{SG56485, &UA2709264},
	{SG56486, &UA2709567},
	{SG56487, &UA2713021},
	{SG56488, &UA2714304},
	{SG56489, &UA2715203},
	{SG56490, &UA2715650},
	{SG56491, &UA2716793},
	{SG56492, &UA2717872},
	{SG56493, &UA2718549},
	{SG56494, &UA2719457},
	{SG56495, &UA2720232},
	{SG56496, &UA2721061},
	{SG56497, &UA2726942},
	{SG56498, &UA2727173},
	{SG56499, &UA2729696},
	{SG56500, &UA2729726},
	{SG56501, &UA2729727},
	{SG56502, &UA2729728},
	{SG56503, &UA2729745},
	{SG56504, &UA2729746},
	{SG56505, &UA2729800},
	{SG56506, &UA2729802},
	{SG56507, &UA2732781},
	{SG56508, &UA2733146},
	{SG56509, &UA2738088},
	{SG56510, &UA2738267},
	{SG56511, &UA2738344},
	{SG56512, &UA2739321},
	{SG56513, &UA2739802},
	{SG56514, &UA2739922},
	{SG56515, &UA2740220},
	{SG56516, &UA2740310},
	{SG56517, &UA2741386},
	{SG56518, &UA2741693},
	{SG56519, &UA2741884},
	{SG56520, &UA2741918},
	{SG56521, &UA2742331},
	{SG56522, &UA2742961},
	{SG56523, &UA2748078},
	{SG56524, &UA2748140},
	{SG56525, &UA2748141},
	{SG56526, &UA2748177},
	{SG56527, &UA2751161},
	{SG56528, &UA2753030},
	{SG56529, &UA2753176},
	{SG56530, &UA2753185},
	{SG56531, &UA2753421},
	{SG56532, &UA2754599},
	{SG56533, &UA2754771},
	{SG56534, &UA2755132},
	{SG56535, &UA2762831},
	{SG56536, &UA2763953},
	{SG56537, &UA2764432},
	{SG56538, &UA2764431},
	{SG56539, &UA2765107},
	{SG56540, &UA2767161},
	{SG56541, &UA2767859},
	{SG56542, &UA2768208},
	{SG56543, &UA2769470},
	{SG56544, &UA2769850},
	{SG56545, &UA2770451},
	{SG56546, &UA2771065},
	{SG56547, &UA2771285},
	{SG56548, &UA2772417},
	{SG56549, &UA2772941},
	{SG56550, &UA2772952},
	{SG56551, &UA2774159},
	{SG56552, &UA2774272},
	{SG56553, &UA2775106},
	{SG56554, &UA2777308},
	{SG56555, &UA2778845},
	{SG56556, &UA2778999},
	{SG56557, &UA2779027},
	{SG56558, &UA2779132},
	{SG56559, &UA2780198},
	{SG56560, &UA2784143},
	{SG56561, &UA2784691},
	{SG56562, &UA2784935},
	{SG56563, &UA2784977},
	{SG56564, &UA2785698},
	{SG56565, &UA2786613},
	{SG56566, &UA2786647},
	{SG56567, &UA2789935},
	{SG56568, &UA2791466},
	{SG56569, &UA2792373},
	{SG56570, &UA2793702},
	{SG56571, &UA2793976},
	{SG56572, &UA2796709},
	{SG56573, &UA2797239},
	{SG56574, &UA2797245},
	{SG56575, &UA2799602},
	{SG56576, &UA2802040},
	{SG56577, &UA2805238},
	{SG56578, &UA2805382},
	{SG56579, &UA2807329},
	{SG56580, &UA2808471},
	{SG56581, &UA2846208},
	{SG56582, &UA2849753},
	{SG56583, &UA2849788},
	{SG56584, &UA2850170},
	{SG56585, &UA2850171},
	{SG56586, &UA2851594},
	{SG56587, &UA2852668},
	{SG56588, &UA2854266},
	{SG56589, &UA2858659},
	{SG56590, &UA2859850},
	{SG56591, &UA2861433},
	{SG56592, &UA2863637},
	{SG56593, &UA2864472},
	{SG56594, &UA2870781},
	{SG56595, &UA2874505},
	{SG56596, &UA2877409},
	{SG56597, &UA2877440},
	{SG56598, &UA2877483},
	{SG56599, &UA2878377},
	{SG56600, &UA2879904},
	{SG56601, &UA2881622},
	{SG56602, &UA2885337},
	{SG56603, &UA2885454},
	{SG56604, &UA2885465},
	{SG56605, &UA2888801},
	{SG56606, &UA2889583},
	{SG56607, &UA2890828},
	{SG56608, &UA2899262},
	{SG56609, &UA2899373},
	{SG56610, &UA2933019},
	{SG56611, &UA2936469},
	{SG56612, &UA2952143},
	{SG56613, &UA2962401},
	{SG56614, &UA2977568},
	{SG56615, &UA2979276},
	{SG56616, &UA2984376},
	{SG56617, &UA2985935},
	{SG56618, &UA2986039},
	{SG56619, &UA2986544},
	{SG56620, &UA2987642},
	{SG56621, &UA2988821},
	{SG56622, &UA2993179},
	{SG56623, &UA3002443},
	{SG56624, &UA3018638},
	{SG56625, &UA3021427},
	{SG56626, &UA3025643},
	{SG56627, &UA3031152},
	{SG56628, &UA3032999},
	{SG56629, &UA3033995},
	{SG56630, &UA3038853},
	{SG56631, &UA3039361},
	{SG56632, &UA3042731},
	{SG56633, &UA3044633},
	{SG56634, &UA3045486},
	{SG56635, &UA3045767},
	{SG56636, &UA3045768},
	{SG56637, &UA3045799},
	{SG56638, &UA3047697},
	{SG56639, &UA3057448},
	{SG56640, &UA3057679},
	{SG56641, &UA3057941},
	{SG56642, &UA3057947},
	{SG56643, &UA3057959},
	{SG56644, &UA3067695},
	{SG56645, &UA3069325},
	{SG56646, &UA3071676},
	{SG56647, &UA3072837},
	{SG56648, &UA3073930},
	{SG56649, &UA3078082},
	{SG56650, &UA3078092},
	{SG56651, &UA3078100},
	{SG56652, &UA3078102},
	{SG56653, &UA3078115},
	{SG56654, &UA3078127},
	{SG56655, &UA3078128},
	{SG56656, &UA3078161},
	{SG56657, &UA3078202},
	{SG56658, &UA3079179},
	{SG56659, &UA3079223},
	{SG56660, &UA3079411},
	{SG56661, &UA3084592},
	{SG56662, &UA3084935},
	{SG56663, &UA3086519},
	{SG56664, &UA3101370},
	{SG56665, &UA3109297},
	{SG56666, &UA3109298},
	{SG56667, &UA3109407},
	{SG56668, &UA3109519},
	{SG56669, &UA3109520},
	{SG56670, &UA3114773},
	{SG56671, &UA3115387},
	{SG56672, &UA3116504},
	{SG56673, &UA3118745},
	{SG56674, &UA3120921},
	{SG56675, &UA3126843},
	{SG56676, &UA3126844},
	{SG56677, &UA3126847},
	{SG56678, &UA3126951},
	{SG56679, &UA3128287},
	{SG56961, &UA101992},
	{SG56962, &UA106714},
	{SG56963, &UA144189},
	{SG56964, &UA236802},
	{SG56965, &UA253232},
	{SG56966, &UA342117},
	{SG56967, &UA349531},
	{SG56968, &UA351739},
	{SG56969, &UA380966},
	{SG56970, &UA381499},
	{SG56971, &UA401466},
	{SG56972, &UA422377},
	{SG56973, &UA440995},
	{SG56974, &UA446434},
	{SG56975, &UA454245},
	{SG56976, &UA457872},
	{SG56977, &UA460558},
	{SG56978, &UA474244},
	{SG56979, &UA523187},
	{SG56980, &UA534691},
	{SG56981, &UA576226},
	{SG56982, &UA593951},
	{SG56983, &UA603787},
	{SG56984, &UA604773},
	{SG56985, &UA679155},
	{SG56986, &UA820486},
	{SG56987, &UA878156},
	{SG56988, &UA895648},
	{SG56989, &UA896050},
	{SG56990, &UA899918},
	{SG56991, &UA901656},
	{SG56992, &UA916313},
	{SG56993, &UA1991448},
	{SG56994, &UA1998279},
	{SG56995, &UA2000675},
	{SG56996, &UA2006157},
	{SG56997, &UA2043052},
	{SG56998, &UA2171249},
	{SG56999, &UA2201496},
	{SG57000, &UA2205697},
	{SG57001, &UA2232230},
	{SG57002, &UA2233450},
	{SG57003, &UA2234364},
	{SG57004, &UA2237132},
	{SG57005, &UA2238905},
	{SG57006, &UA2242206},
	{SG57007, &UA2242475},
	{SG57008, &UA2341985},
	{SG57009, &UA2355807},
	{SG57010, &UA2367260},
	{SG57011, &UA2402069},
	{SG57012, &UA2408446},
	{SG57013, &UA2421814},
	{SG57014, &UA2426117},
	{SG57015, &UA2430616},
	{SG57016, &UA2436184},
	{SG57017, &UA2437579},
	{SG57018, &UA2460570},
	{SG57019, &UA2479105},
	{SG57020, &UA2486175},
	{SG57021, &UA2487944},
	{SG57022, &UA2488346},
	{SG57023, &UA2493586},
	{SG57024, &UA2494416},
	{SG57025, &UA2504867},
	{SG57026, &UA2514006},
	{SG57027, &UA2514353},
	{SG57028, &UA2529662},
	{SG57029, &UA2534987},
	{SG57030, &UA2554919},
	{SG57031, &UA2559755},
	{SG57032, &UA2561626},
	{SG57033, &UA2565581},
	{SG57034, &UA2571471},
	{SG57035, &UA2571516},
	{SG57036, &UA2577764},
	{SG57037, &UA2586538},
	{SG57038, &UA2589738},
	{SG57039, &UA2591870},
	{SG57040, &UA2601652},
	{SG57041, &UA2656501},
	{SG57042, &UA2670826},
	{SG57043, &UA2679942},
	{SG57044, &UA2681154},
	{SG57045, &UA2686356},
	{SG57046, &UA2713421},
	{SG57047, &UA2713980},
	{SG57048, &UA2715224},
	{SG57049, &UA2721475},
	{SG57050, &UA2764041},
	{SG57051, &UA2786794},
	{SG57052, &UA2808745},
	{SG57053, &UA2866594},
	{SG57054, &UA2895371},
	{SG57055, &UA2896467},
	{SG57056, &UA2897828},
	{SG57057, &UA2943992},
	{SG57058, &UA2973985},
	{SG57059, &UA3057727},
	{SG57181, &UA2490338},
	{SG57198, &UA389494},
	{SG57199, &UA418834},
	{SG57200, &UA2898639},
	{SG57201, &UA2944037},
	{SG57202, &UA2946465},
	{SG57203, &UA2947336},
	{SG57204, &UA2948279},
	{SG57205, &UA2949192},
	{SG57206, &UA2950710},
	{SG57207, &UA2955180},
	{SG57208, &UA2957088},
	{SG57209, &UA2957292},
	{SG57210, &UA2957422},
	{SG57211, &UA2957606},
	{SG57212, &UA2968016},
	{SG57213, &UA2971954},
	{SG57214, &UA2972444},
	{SG57215, &UA2973637},
	{SG57216, &UA2974205},
	{SG57217, &UA2974855},
	{SG57218, &UA2976151},
	{SG57219, &UA2978568},
	{SG57220, &UA2979535},
	{SG57221, &UA2979774},
	{SG57222, &UA2982858},
	{SG57223, &UA2988101},
	{SG57224, &UA2989170},
	{SG57225, &UA2990811},
	{SG57226, &UA2990870},
	{SG57227, &UA2994418},
	{SG57228, &UA2999362},
	{SG57229, &UA2999792},
	{SG57230, &UA3000058},
	{SG57231, &UA3003075},
	{SG57232, &UA3004301},
	{SG57233, &UA3018310},
	{SG57234, &UA3018314},
	{SG57235, &UA3018315},
	{SG57236, &UA3018431},
	{SG57237, &UA3018885},
	{SG57238, &UA3019098},
	{SG57239, &UA3020458},
	{SG57240, &UA3020663},
	{SG57241, &UA3023725},
	{SG57242, &UA3024159},
	{SG57243, &UA3024233},
	{SG57244, &UA3027916},
	{SG57245, &UA3028864},
	{SG57246, &UA3028865},
	{SG57249, &UA3029963},
	{SG57250, &UA3030203},
	{SG57251, &UA3032691},
	{SG57252, &UA3034061},
	{SG57253, &UA3034219},
	{SG57254, &UA3034672},
	{SG57255, &UA3036243},
	{SG57256, &UA3036300},
	{SG57257, &UA3039301},
	{SG57258, &UA3041946},
	{SG57259, &UA3043784},
	{SG57260, &UA3046598},
	{SG57261, &UA3047437},
	{SG57262, &UA3048320},
	{SG57263, &UA3051449},
	{SG57264, &UA3052551},
	{SG57265, &UA3056936},
	{SG57266, &UA3057544},
	{SG57267, &UA3057678},
	{SG57268, &UA3057677},
	{SG57269, &UA3057926},
	{SG57270, &UA3057939},
	{SG57271, &UA3057946},
	{SG57272, &UA3057950},
	{SG57273, &UA3057954},
	{SG57274, &UA3057958},
	{SG57275, &UA3057960},
	{SG57276, &UA3057962},
	{SG57277, &UA3061687},
	{SG57278, &UA3062711},
	{SG57279, &UA3067506},
	{SG57280, &UA3069940},
	{SG57281, &UA3070387},
	{SG57282, &UA3070662},
	{SG57283, &UA3074499},
	{SG57284, &UA3077696},
	{SG57285, &UA3077892},
	{SG57286, &UA3078093},
	{SG57287, &UA3078098},
	{SG57288, &UA3078101},
	{SG57289, &UA3078114},
	{SG57290, &UA3078124},
	{SG57291, &UA3078125},
	{SG57292, &UA3078162},
	{SG57293, &UA3080775},
	{SG57294, &UA3084001},
	{SG57295, &UA3084033},
	{SG57296, &UA3084036},
	{SG57297, &UA3084707},
	{SG57298, &UA3092622},
	{SG57299, &UA3093700},
	{SG57300, &UA3095033},
	{SG57301, &UA3097580},
	{SG57302, &UA3097957},
	{SG57303, &UA3099696},
	{SG57304, &UA3109293},
	{SG57305, &UA3109525},
	{SG57306, &UA3112831},
	{SG57307, &UA3114123},
	{SG57308, &UA3116658},
	{SG57309, &UA3116884},
	{SG57310, &UA3118473},
	{SG57311, &UA3118826},
	{SG57312, &UA3129745},
	{SG57313, &UA3130577},
	{SG57314, &UA92479},
	{SG57315, &UA93027},
	{SG57316, &UA94271},
	{SG57317, &UA97960},
	{SG57318, &UA98129},
	{SG57319, &UA98993},
	{SG57320, &UA100103},
	{SG57321, &UA100442},
	{SG57322, &UA103665},
	{SG57323, &UA145003},
	{SG57324, &UA154904},
	{SG57325, &UA157679},
	{SG57326, &UA172279},
	{SG57327, &UA188498},
	{SG57328, &UA191167},
	{SG57329, &UA227173},
	{SG57330, &UA250287},
	{SG57331, &UA283944},
	{SG57332, &UA304178},
	{SG57333, &UA335355},
	{SG57334, &UA364282},
	{SG57335, &UA368607},
	{SG57336, &UA389157},
	{SG57337, &UA418796},
	{SG57338, &UA430197},
	{SG57339, &UA436156},
	{SG57340, &UA459768},
	{SG57341, &UA503394},
	{SG57342, &UA506295},
	{SG57343, &UA509582},
	{SG57344, &UA547384},
	{SG57345, &UA560316},
	{SG57346, &UA565248},
	{SG57347, &UA578585},
	{SG57348, &UA587170},
	{SG57349, &UA635457},
	{SG57350, &UA649075},
	{SG57351, &UA673340},
	{SG57352, &UA701618},
	{SG57353, &UA759852},
	{SG57354, &UA773745},
	{SG57355, &UA840014},
	{SG57356, &UA1888268},
	{SG57357, &UA1902101},
	{SG57358, &UA1996471},
	{SG57359, &UA2040890},
	{SG57360, &UA2142249},
	{SG57361, &UA2221254},
	{SG57362, &UA2268513},
	{SG57363, &UA2391766},
	{SG57364, &UA2600242},
	{SG57365, &UA2638825},
	{SG57366, &UA2677380},
	{SG57367, &UA2696751},
	{SG57368, &UA2713624},
	{SG57369, &UA2718847},
	{SG57370, &UA2785785},
	{SG57371, &UA2787177},
	{SG57372, &UA2888521},
	{SG57750, &UA102823},
	{SG57751, &UA127374},
	{SG57752, &UA197012},
	{SG57753, &UA233395},
	{SG57754, &UA255977},
	{SG57755, &UA270817},
	{SG57756, &UA300188},
	{SG57757, &UA306058},
	{SG57758, &UA355481},
	{SG57759, &UA373453},
	{SG57760, &UA416010},
	{SG57761, &UA416740},
	{SG57762, &UA452935},
	{SG57763, &UA912670},
	{SG57764, &UA914773},
	{SG57765, &UA2229032},
	{SG57766, &UA2368136},
	{SG57767, &UA2404163},
	{SG57768, &UA2408276},
	{SG57769, &UA2431873},
	{SG57770, &UA2433552},
	{SG57771, &UA2461225},
	{SG57772, &UA2461226},
	{SG57796, &UA232840},
	{SG57797, &UA255195},
	{SG57798, &UA632126},
	{SG57799, &UA909506},
	{SG57800, &UA2193293},
	{SG57801, &UA2567484},
	{SG57802, &UA2569817},
	{SG57803, &UA2947237},
	{SG57806, &UA136551},
	{SG57807, &UA145981},
	{SG57808, &UA224042},
	{SG57809, &UA430093},
	{SG57810, &UA724207},
	{SG57811, &UA1927502},
	{SG57812, &UA1934168},
	{SG57813, &UA2159157},
	{SG57814, &UA2258664},
	{SG57815, &UA2443031},
	{SG57816, &UA2487768},
	{SG57817, &UA2537523},
	{SG57818, &UA2763518},
	{SG57819, &UA2810601},
	{SG57820, &UA92387},
	{SG57821, &UA103869},
	{SG57822, &UA248551},
	{SG57823, &UA309972},
	{SG57824, &UA349333},
	{SG57825, &UA351557},
	{SG57826, &UA375721},
	{SG57827, &UA453333},
	{SG57828, &UA512009},
	{SG57829, &UA554588},
	{SG57830, &UA588090},
	{SG57831, &UA625473},
	{SG57832, &UA691328},
	{SG57833, &UA693585},
	{SG57834, &UA700693},
	{SG57835, &UA701548},
	{SG57836, &UA897262},
	{SG57837, &UA920784},
	{SG57838, &UA1915289},
	{SG57839, &UA2164214},
	{SG57840, &UA2167908},
	{SG57841, &UA2215976},
	{SG57842, &UA2262003},
	{SG57843, &UA2336015},
	{SG57844, &UA2354831},
	{SG57845, &UA2367078},
	{SG57846, &UA2367082},
	{SG57847, &UA2403699},
	{SG57848, &UA2427832},
	{SG57849, &UA2460571},
	{SG57850, &UA2460579},
	{SG57851, &UA2461050},
	{SG57852, &UA2461351},
	{SG57853, &UA2471621},
	{SG57854, &UA2503313},
	{SG57855, &UA2554253},
	{SG57856, &UA2558265},
	{SG57857, &UA2594766},
	{SG57858, &UA2596956},
	{SG57859, &UA2599551},
	{SG57860, &UA2668982},
	{SG57861, &UA2673827},
	{SG57862, &UA2708921},
	{SG57863, &UA2708932},
	{SG57864, &UA2732743},
	{SG57865, &UA2738818},
	{SG57866, &UA2768615},
	{SG57867, &UA2769835},
	{SG57868, &UA2770765},
	{SG57869, &UA2796387},
	{SG57870, &UA2858795},
	{SG57871, &UA2872331},
	{SG57872, &UA2872554},
	{SG57873, &UA2877432},
	{SG57874, &UA2878780},
	{SG57875, &UA2894928},
	{SG57876, &UA2895417},
	{SG57877, &UA2898411},
	{SG57878, &UA2901360},
	{SG57879, &UA2933112},
	{SG57880, &UA2938336},
	{SG57881, &UA2945732},
	{SG57882, &UA2958517},
	{SG57883, &UA2971341},
	{SG57884, &UA2974904},
	{SG57885, &UA2974898},
	{SG57886, &UA3011893},
	{SG57887, &UA3013002},
	{SG57888, &UA3021356},
	{SG57889, &UA3023311},
	{SG57890, &UA3024556},
	{SG57891, &UA3025960},
	{SG57892, &UA3033454},
	{SG57893, &UA3047613},
	{SG57894, &UA3057784},
	{SG57895, &UA3057917},
	{SG57896, &UA3057936},
	{SG57897, &UA3058598},
	{SG57898, &UA3063889},
	{SG57899, &UA3064214},
	{SG57900, &UA3066649},
	{SG57901, &UA3083911},
	{SG57902, &UA3092124},
	{SG57903, &UA3108060},
	{SG57904, &UA3108844},
	{SG57905, &UA3111764},
	{SG57906, &UA3114960},
	{SG57907, &UA3120202},
	{SG57908, &UA3131723},
	{SG57928, &UA2944422},
	{SG57929, &UA2967122},
	{SG57930, &UA2973810},
	{SG57931, &UA2973980},
	{SG57932, &UA2985796},
	{SG57933, &UA3010435},
	{SG57934, &UA3018312},
	{SG57935, &UA3032714},
	{SG57936, &UA3033391},
	{SG57937, &UA3036045},
	{SG57938, &UA3067625},
	{SG57939, &UA3072684},
	{SG57940, &UA3078179},
	{SG57941, &UA3107363},
	{SG57942, &UA3119028},
	{SG58042, &UA197530},
	{SG58043, &UA1996470},
	{SG58218, &UA86114},
	{SG58219, &UA93982},
	{SG58220, &UA95423},
	{SG58221, &UA98145},
	{SG58222, &UA102180},
	{SG58223, &UA105113},
	{SG58224, &UA121213},
	{SG58225, &UA121820},
	{SG58226, &UA125672},
	{SG58227, &UA166235},
	{SG58228, &UA169682},
	{SG58229, &UA194613},
	{SG58230, &UA201731},
	{SG58231, &UA299873},
	{SG58232, &UA347547},
	{SG58233, &UA601625},
	{SG58234, &UA627003},
	{SG58235, &UA771679},
	{SG58236, &UA1898729},
	{SG58237, &UA2197102},
	{SG58239, &UA92919},
	{SG58240, &UA3092125},
	{SG58241, &UA3100019},
	{SG58242, &UA93036},
	{SG58243, &UA98151},
	{SG58244, &UA191038},
	{SG58245, &UA382339},
	{SG58246, &UA444555},
	{SG58247, &UA459926},
	{SG58248, &UA501861},
	{SG58249, &UA517646},
	{SG58250, &UA543528},
	{SG58251, &UA902791},
	{SG58252, &UA1893600},
	{SG58253, &UA1895382},
	{SG58254, &UA1896911},
	{SG58255, &UA2126829},
	{SG58258, &UA89023},
	{SG58259, &UA2981631},
	{SG58401, &UA2437380},
	{SG58402, &UA2468369},
	{SG58423, &UA366336},
	{SG58425, &UA2581796},
	{SG58434, &UA2844523},
	{SG58459, &UA272193},
	{SG58460, &UA331849},
	{SG58461, &UA388382},
	{SG58462, &UA407886},
	{SG58463, &UA436661},
	{SG58464, &UA615293},
	{SG58465, &UA1896760},
	{SG58466, &UA1897085},
	{SG58467, &UA1897384},
	{SG58468, &UA1940826},
	{SG58469, &UA2236558},
	{SG58470, &UA2309586},
	{SG58471, &UA2471067},
	{SG58472, &UA2710368},
	{SG58527, &UA460465},
	{SG58540, &UA30727},
	{SG58541, &UA1880304},
	{SG58542, &UA2362166},
	{SG58574, &UA401560},
	{SG58583, &UA895510},
	{SG58584, &UA902644},
	{SG58590, &UA93882},
	{SG58591, &UA125189},
	{SG58592, &UA161457},
	{SG58593, &UA167157},
	{SG58594, &UA191860},
	{SG58595, &UA193567},
	{SG58596, &UA276107},
	{SG58597, &UA657596},
	{SG58598, &UA736014},
	{SG58600, &UA783455},
	{SG58602, &UA2271051},
	{SG58604, &UA2437543},
	{SG58605, &UA366680},
	{SG58606, &UA451570},
	{SG58607, &UA493950},
	{SG58608, &UA531913},
	{SG58609, &UA564794},
	{SG58610, &UA2248555},
	{SG58625, &UA100371},
	{SG58626, &UA416021},
	{SG58642, &UA603608},
	{SG58643, &UA925386},
	{SG58644, &UA2356191},
	{SG58680, &UA2195951},
	{SG58681, &UA93227},
	{SG58682, &UA2741568},
	{SG58690, &UA92402},
	{SG58691, &UA1898149},
	{SG58692, &UA1898659},
	{SG58717, &UA211548},
	{SG58718, &UA150227},
	{SG58736, &UA2569396},
	{SG58737, &UA2882037},
	{SG58763, &UA395437},
	{SG58764, &UA2204335},
	{SG58765, &UA2539846},
	{SG58766, &UA2751917},
	{SG58770, &UA2213020},
	{SG58771, &UA2738806},
	{SG58807, &UA2462363},
	{SG58834, &UA102729},
	{SG58835, &UA93023},
	{SG58836, &UA191968},
	{SG58837, &UA347152},
	{SG58838, &UA348575},
	{SG58839, &UA359652},
	{SG58840, &UA569098},
	{SG58841, &UA600329},
	{SG58842, &UA602996},
	{SG58843, &UA685155},
	{SG58844, &UA703759},
	{SG58845, &UA741860},
	{SG58846, &UA762446},
	{SG58847, &UA893474},
	{SG58848, &UA1905403},
	{SG58849, &UA1933772},
	{SG58850, &UA2228460},
	{SG58851, &UA2230388},
	{SG58852, &UA2320916},
	{SG58853, &UA2361844},
	{SG58854, &UA2422886},
	{SG58855, &UA2598709},
	{SG58856, &UA2683958},
	{SG58857, &UA2692688},
	{SG58858, &UA2794350},
	{SG58859, &UA2795066},
	{SG58860, &UA2952316},
	{SG58861, &UA3015805},
	{SG58862, &UA3019841},
	{SG58863, &UA3023322},
	{SG58864, &UA3026682},
	{SG58865, &UA3030545},
	{SG58866, &UA3041825},
	{SG58867, &UA3056750},
	{SG58868, &UA3057923},
	{SG58869, &UA3057948},
	{SG58870, &UA3077736},
	{SG58871, &UA3092497},
	{SG58872, &UA3120237},
	{SG58879, &UA2948411},
	{SG58910, &UA101035},
	{SG58911, &UA108786},
	{SG58921, &UA567814},
	{SG58932, &UA2671058},
	{SG58940, &UA199701},
	{SG58941, &UA841235},
	{SG58951, &UA2005305},
	{SG59007, &UA366626},
	{SG59074, &UA646325},
	{SG59081, &UA256050},
	{SG59082, &UA430252},
	{SG59083, &UA2292234},
	{SG59085, &UA148820},
	{SG59086, &UA798186},
	{SG59123, &UA2606562},
	{SG59124, &UA2717083},
	{SG59140, &UA901326},
	{SG59143, &UA576163},
	{SG59157, &UA94794},
	{SG59158, &UA98531},
	{SG59159, &UA101820},
	{SG59160, &UA104844},
	{SG59161, &UA112166},
	{SG59162, &UA120401},
	{SG59163, &UA143399},
	{SG59164, &UA143402},
	{SG59165, &UA144562},
	{SG59166, &UA144821},
	{SG59167, &UA151893},
	{SG59168, &UA233517},
	{SG59169, &UA277560},
	{SG59170, &UA283647},
	{SG59171, &UA298703},
	{SG59172, &UA374974},
	{SG59173, &UA382525},
	{SG59174, &UA384830},
	{SG59175, &UA501317},
	{SG59176, &UA518295},
	{SG59177, &UA541125},
	{SG59178, &UA551542},
	{SG59179, &UA691852},
	{SG59180, &UA699441},
	{SG59181, &UA712381},
	{SG59182, &UA730931},
	{SG59183, &UA745472},
	{SG59184, &UA780453},
	{SG59185, &UA873993},
	{SG59186, &UA893765},
	{SG59187, &UA1841053},
	{SG59188, &UA1852665},
	{SG59189, &UA1894643},
	{SG59190, &UA1896381},
	{SG59191, &UA1980539},
	{SG59192, &UA2127235},
	{SG59193, &UA2127807},
	{SG59194, &UA2217515},
	{SG59195, &UA2266754},
	{SG59196, &UA2267584},
	{SG59197, &UA2279425},
	{SG59198, &UA2313953},
	{SG59199, &UA2333380},
	{SG59200, &UA2487775},
	{SG59201, &UA2566202},
	{SG59202, &UA2569438},
	{SG59203, &UA2673414},
	{SG59204, &UA2752757},
	{SG59205, &UA2776502},
	{SG59206, &UA3057920},
	{SG59211, &UA2356518},
	{SG59219, &UA372489},
	{SG59220, &UA613368},
	{SG59221, &UA915812},
	{SG59222, &UA2682289},
	{SG59223, &UA2783642},
	{SG59224, &UA3057938},
	{SG59225, &UA3069364},
	{SG59226, &UA3078222},
	{SG59227, &UA3091929},
	{SG59280, &UA2986568},
	{SG59281, &UA3000041},
	{SG59282, &UA3018428},
	{SG59283, &UA3026060},
	{SG59284, &UA3029738},
	{SG59285, &UA3029737},
	{SG59286, &UA3047673},
	{SG59287, &UA3097407},
	{SG59288, &UA3097420},
	{SG59289, &UA3102623},
	{SG59290, &UA3130824},
	{SG59308, &UA2003034},
	{SG59309, &UA2261979},
	{SG59310, &UA2262059},
	{SG59311, &UA2357567},
	{SG59318, &UA236548},
	{SG59341, &UA92805},
	{SG59342, &UA372587},
	{SG59343, &UA376055},
	{SG59344, &UA538783},
	{SG59345, &UA626941},
	{SG59346, &UA751299},
	{SG59347, &UA1874090},
	{SG59374, &UA135408},
	{SG59375, &UA492025},
	{SG59376, &UA685753},
	{SG59377, &UA804804},
	{SG59378, &UA2265533},
	{SG59379, &UA2273330},
	{SG59380, &UA3018313},
	{SG59383, &UA3057928},
	{SG59406, &UA1894402},
	{SG59407, &UA2784969},
	{SG59499, &UA138665},
	{SG59500, &UA2337815},
	{SG59527, &UA3031353},
	{SG59528, &UA3114368},
	{SG59633, &UA668822},
	{SG59634, &UA740868},
	{SG59635, &UA2383089},
	{SG59636, &UA2601662},
	{SG59637, &UA2863025},
	{SG59638, &UA2994151},
	{SG59738, &UA3026168},
	{SG59742, &UA95065},
	{SG59743, &UA297822},
	{SG59825, &UA165924},
	{SG59826, &UA531005},
	{SG59837, &UA398953},
	{SG59838, &UA1996695},
	{SG59839, &UA2570230},
	{SG59840, &UA3008066},
	{SG59841, &UA3058492},
	{SG59842, &UA707814},
	{SG59849, &UA122964},
	{SG59850, &UA160098},
	{SG59851, &UA529939},
	{SG59852, &UA651397},
	{SG59853, &UA721683},
	{SG59854, &UA3078094},
	{SG59855, &UA3078118},
	{SG59879, &UA556063},
	{SG59880, &UA688521},
	{SG59887, &UA141167},
	{SG59888, &UA435557},
	{SG59895, &UA183420},
	{SG59941, &UA624748},
	{SG59995, &UA2466242},
	{SG59996, &UA2671085},
	{SG59997, &UA2718739},
	{SG59998, &UA2793756},
	{SG59999, &UA2800301},
	{SG60032, &UA102183},
	{SG60049, &UA2606685},
	{SG60116, &UA141958},
	{SG60130, &UA3035691},
	{SG60131, &UA3038805},
	{SG60147, &UA2734830},
	{SG60148, &UA2767668},
	{SG60149, &UA2767961},
	{SG60160, &UA2879030},
	{SG60162, &UA2942335},
	{SG60165, &UA2943724},
	{SG60220, &UA137805},
	{SG60221, &UA548187},
	{SG60227, &UA3018140},
	{SG60228, &UA3032620},
	{SG60229, &UA3057652},
	{SG60251, &UA2530055},
	{SG60252, &UA2535616},
	{SG60253, &UA2582758},
	{SG60268, &UA142536},
	{SG60269, &UA394690},
	{SG60270, &UA517788},
	{SG60271, &UA1894933},
	{SG60272, &UA1895352},
	{SG60273, &UA1899197},
	{SG60274, &UA1905606},
	{SG60275, &UA1946886},
	{SG60276, &UA3042165},
	{SG60285, &UA2666923},
	{SG60286, &UA2668894},
	{SG60287, &UA2675778},
	{SG60302, &UA2226709},
	{SG60303, &UA2258928},
	{SG60304, &UA2259176},
	{SG60305, &UA2401943},
	{SG60322, &UA2367047},
	{SG60323, &UA2405739},
	{SG60324, &UA2423587},
	{SG60325, &UA2426753},
	{SG60326, &UA2458994},
	{SG60327, &UA2484547},
	{SG60328, &UA2487483},
	{SG60335, &UA2933855},
	{SG60336, &UA2978619},
	{SG60337, &UA3000831},
	{SG60338, &UA3010761},
	{SG60339, &UA3019452},
	{SG60340, &UA3020575},
	{SG60341, &UA3020640},
	{SG60343, &UA3020920},
	{SG60345, &UA3057943},
	{SG60346, &UA3074327},
	{SG60347, &UA3078609},
	{SG60348, &UA3092178},
	{SG60349, &UA3105087},
	{SG60354, &UA2040955},
	{SG60355, &UA2043221},
	{SG60356, &UA2192194},
	{SG60357, &UA2205384},
	{SG60358, &UA2337879},
	{SG60393, &UA2754458},
	{SG60394, &UA2784939},
	{SG60395, &UA2799799},
	{SG60396, &UA2876200},
	{SG60397, &UA2899734},
	{SG60398, &UA2970270},
	{SG60399, &UA3020307},
	{SG60400, &UA3026029},
	{SG60401, &UA3028236},
	{SG60407, &UA126239},
	{SG60415, &UA92870},
	{SG60422, &UA898229},
	{SG60467, &UA479533},
	{SG60468, &UA3018311},
	{SG60469, &UA3020641},
	{SG60476, &UA2671078},
	{SG60482, &UA443993},
	{SG60487, &UA2508048},
	{SG60533, &UA138910},
	{SG60545, &UA2752311},
	{SG60546, &UA2866429},
	{SG60549, &UA3060453},
	{SG60550, &UA3063575},
	{SG60552, &UA3080248},
	{SG60554, &UA3115409},
	{SG60564, &UA253824},
	{SG60584, &UA2536469},
	{SG60585, &UA2597817},
	{SG60597, &UA2671726},
	{SG60598, &UA2677112},
	{SG60601, &UA2893171},
	{SG60602, &UA2998011},
	{SG60609, &UA2359844},
	{SG60612, &UA113211},
	{SG60613, &UA2401624},
	{SG60614, &UA2407102},
	{SG60615, &UA2555593},
	{SG60621, &UA3020616},
	{SG60622, &UA3077119},
	{SG60627, &UA665905},
	{SG60628, &UA674013},
	{SG60629, &UA849425},
	{SG60634, &UA2467600},
	{SG60635, &UA2508147},
	{SG60639, &UA258454},
	{SG60640, &UA2334628},
	{SG60643, &UA2669175},
	{SG60644, &UA2707085},
	{SG60646, &UA2230766},
	{SG60647, &UA2354898},
	{SG60655, &UA2361780},
	{SG60656, &UA2405267},
	{SG60657, &UA2434787},
	{SG60690, &UA2367797},
	{SG60691, &UA2455041},
	{SG60692, &UA2458889},
	{SG60693, &UA2603527},
	{SG60694, &UA2737664},
	{SG60695, &UA2853015},
	{SG60699, &UA2362476},
	{SG60702, &UA2039730},
	{SG60703, &UA2040599},
	{SG60704, &UA2200234},
	{SG60723, &UA1906516},
	{SG60725, &UA690026},
	{SG60726, &UA704784},
	{SG60727, &UA712015},
	{SG60728, &UA1904646},
	{SG60729, &UA1930593},
	{SG60746, &UA845326},
	{SG60747, &UA904790},
	{SG60748, &UA907779},
	{SG60749, &UA908162},
	{SG60750, &UA1985367},
	{SG60767, &UA2539588},
	{SG60768, &UA2567361},
	{SG60769, &UA2584902},
	{SG60770, &UA141117},
	{SG60771, &UA1895698},
	{SG60777, &UA916007},
	{SG60778, &UA1896992},
	{SG60779, &UA1898179},
	{SG60780, &UA1898742},
	{SG60795, &UA2471703},
	{SG60796, &UA2487436},
	{SG60797, &UA2512206},
	{SG60798, &UA140531},
	{SG60799, &UA485600},
	{SG60801, &UA678767},
	{SG60803, &UA1853907},
	{SG60804, &UA3057918},
	{SG60820, &UA1856155},
	{SG60821, &UA1909033},
	{SG60827, &UA475726},
	{SG60828, &UA659777},
	{SG60829, &UA349313},
	{SG60833, &UA590206},
	{SG60834, &UA591141},
	{SG60835, &UA591193},
	{SG60836, &UA1953333},
	{SG60856, &UA590346},
	{SG60858, &UA1939222},
	{SG60860, &UA1960740},
	{SG60861, &UA1972021},
	{SG60872, &UA93065},
	{SG60877, &UA2258911},
	{SG60878, &UA2366783},
	{SG60883, &UA924491},
	{SG60884, &UA1995189},
	{SG60885, &UA1995654},
	{SG60886, &UA2562489},
	{SG60887, &UA2850556},
	{SG60890, &UA656733},
	{SG60891, &UA660631},
	{SG60894, &UA813278},
	{SG60895, &UA899478},
	{SG60896, &UA905704},
	{SG60897, &UA2320487},
	{SG60898, &UA2328040},
	{SG60899, &UA684034},
	{SG60900, &UA689811},
	{SG60901, &UA1920526},
	{SG60907, &UA3108403},
	{SG60919, &UA882139},
	{SG60920, &UA915826},
	{SG60931, &UA2948595},
	{SG60934, &UA2201264},
	{SG60935, &UA2231817},
	{SG60936, &UA2354830},
	{SG60942, &UA407883},
	{SG60943, &UA2752693},
	{SG60944, &UA3007542},
	{SG60952, &UA2668205},
	{SG60953, &UA3018432},
	{SG60954, &UA3057949},
	{SG60955, &UA3057955},
	{SG60961, &UA2937419},
	{SG60963, &UA476301},
	{SG60964, &UA684199},
	{SG60965, &UA1870230},
	{SG60966, &UA3057930},
	{SG60967, &UA3057951},
	{SG60968, &UA3078126},
	{SG60977, &UA773902},
	{SG60978, &UA872143},
	{SG60979, &UA1871457},
	{SG60980, &UA2484466},
	{SG60984, &UA1995409},
	{SG60985, &UA1995638},
	{SG60988, &UA3036682},
	{SG60989, &UA647608},
	{SG60990, &UA802957},
	{SG60991, &UA833654},
	{SG60992, &UA1985827},
	{SG60996, &UA2718524},
	{SG60997, &UA351451},
	{SG60998, &UA2040267},
	{SG60999, &UA498996},
	{SG61008, &UA256035},
	{SG61009, &UA1837162},
	{SG61010, &UA1935000},
	{SG61012, &UA720806},
	{SG61013, &UA1940183},
	{SG61024, &UA924843},
	{SG61025, &UA1995184},
	{SG61028, &UA785745},
	{SG61029, &UA839044},
	{SG61032, &UA2962032},
	{SG61033, &UA3058735},
	{SG61035, &UA651200},
	{SG61036, &UA780707},
	{SG61040, &UA713868},
	{SG61043, &UA553050},
	{SG61044, &UA2193465},
	{SG61045, &UA2231976},
	{SG61052, &UA2769775},
	{SG61053, &UA2956851},
	{SG61055, &UA177483},
	{SG61062, &UA1904460},
	{SG61065, &UA2952390},
	{SG61066, &UA3092271},
	{SG61069, &UA496764},
	{SG61070, &UA547974},
	{SG61071, &UA1854359},
	{SG61072, &UA2958284},
	{SG61078, &UA317164},
	{SG61088, &UA3028063},
	{SG61092, &UA2486512},
	{SG61093, &UA2711181},
	{SG61094, &UA2778888},
	{SG61100, &UA2423706},
	{SG61105, &UA1918222},
	{SG61106, &UA2392543},
	{SG61118, &UA760175},
	{SG61119, &UA2262224},
	{SG61122, &UA786382},
	{SG61123, &UA2328511},
	{SG61138, &UA2658805},
	{SG61139, &UA3057929},
	{SG61150, &UA2733129},
	{SG61151, &UA3057919},
	{SG61170, &UA2893810},
	{SG61176, &UA3011666},
	{SG61180, &UA2736658},
	{SG61188, &UA648603},
	{SG61190, &UA3018309},
	{SG61197, &UA2660647},
	{SG61206, &UA753485},
	{SG61207, &UA2271750},
	{SG61215, &UA829519},
	{SG61216, &UA2599604},
	{SG61217, &UA2735567},
	{SG61219, &UA2767811},
	{SG61224, &UA3078204},
	{SG61225, &UA2732826},
	{SG61226, &UA3078203},
	{SG61227, &UA3091809},
	{SG61236, &UA3092319},
	{SG61239, &UA2766520},
	{SG61241, &UA2998042},
	{SG61242, &UA3057925},
	{SG61258, &UA2878807},
	{SG61259, &UA3030062},
	{SG61271, &UA2801673},
	{SG61273, &UA661938},
	{SG61274, &UA907549},
	{SG61275, &UA1897965},
	{SG61276, &UA2666247},
	{SG61277, &UA2869037},
	{SG61279, &UA3000207},
	{SG61282, &UA2825660},
	{SG61283, &UA3126034},
	{SG61284, &UA2338745},
	{SG61285, &UA2846245},
	{SG61286, &UA2892763},
	{SG61287, &UA3107331},
	{SG61288, &UA3107348},
	{SG61289, &UA3107424},
	{SG61290, &UA3107548},
	{SG61291, &UA3125966},
	{SG61292, &UA2210506},
	{SG61293, &UA2844998},
	{SG61294, &UA2852238},
	{SG61295, &UA3127567},
	{SG61297, &UA2847791},
	{SG61298, &UA259541},
	{SG61299, &UA2148133},
	{SG61300, &UA899181},
	{SG61301, &UA2339927},
	{SG61302, &UA2349418},
	{SG61304, &UA158298},
	{SG61308, &UA621845},
	{SG61311, &UA126430},
	{SG61314, &UA664754},
	{SG61316, &UA2977320},
	{SG61318, &UA2192110},
	{SG61321, &UA2977296},
	{SG61322, &UA364283},
	{NULL}};
const Device *DBird[] = {
	&UA89713,
	&UA129762,
	&UA89808,
	&UA87620,
	&UA3541,
	&UA88523,
	NULL};
const Segmentdata DBlackBerry1[] = {
	{SG57390, &UA3115142},
	{SG57429, &UA46283},
	{SG57430, &UA62952},
	{SG57431, &UA2641673},
	{SG57621, &UA53295},
	{SG57622, &UA2362409},
	{SG57645, &UA52167},
	{SG57646, &UA2320656},
	{SG57647, &UA2370869},
	{SG57648, &UA56486},
	{SG57656, &UA63272},
	{SG57773, &UA2431171},
	{SG57804, &UA11770},
	{SG57805, &UA56051},
	{SG57927, &UA429222},
	{SG57979, &UA53786},
	{SG58040, &UA2402710},
	{SG58041, &UA2229993},
	{SG58067, &UA896320},
	{SG58086, &UA64575},
	{SG58087, &UA48650},
	{SG58088, &UA429224},
	{SG58264, &UA2573332},
	{SG58265, &UA64788},
	{SG58363, &UA64094},
	{SG58386, &UA50156},
	{SG58394, &UA23736},
	{SG58397, &UA3021046},
	{SG58398, &UA3021402},
	{SG58399, &UA3024367},
	{SG58400, &UA3057715},
	{SG58410, &UA2867847},
	{SG58411, &UA2879660},
	{SG58412, &UA2957373},
	{SG58413, &UA3049562},
	{SG58414, &UA2425873},
	{SG58424, &UA68406},
	{SG58443, &UA2781659},
	{SG58444, &UA3005448},
	{SG58445, &UA3078571},
	{SG58446, &UA3092291},
	{SG58447, &UA853292},
	{SG58546, &UA64674},
	{SG58669, &UA2431219},
	{SG58693, &UA64151},
	{SG58833, &UA63776},
	{SG58883, &UA6665},
	{SG58884, &UA43980},
	{SG58949, &UA8126},
	{SG58950, &UA63673},
	{SG59008, &UA63015},
	{SG59017, &UA62208},
	{SG59018, &UA64684},
	{SG59055, &UA429130},
	{SG59084, &UA2798175},
	{SG59207, &UA3690},
	{SG59228, &UA429139},
	{SG59240, &UA64890},
	{SG59254, &UA2262470},
	{SG59255, &UA2262543},
	{SG59256, &UA2355293},
	{SG59257, &UA2385483},
	{SG59258, &UA2403041},
	{SG59259, &UA2423818},
	{SG59260, &UA2425078},
	{SG59261, &UA2425505},
	{SG59262, &UA2467150},
	{SG59263, &UA2504008},
	{SG59264, &UA2504655},
	{SG59265, &UA2528134},
	{SG59266, &UA2694311},
	{SG59267, &UA2696470},
	{SG59268, &UA2778102},
	{SG59269, &UA2842552},
	{SG59270, &UA2983053},
	{SG59271, &UA2985880},
	{SG59272, &UA3071290},
	{SG59273, &UA26812},
	{SG59307, &UA54481},
	{SG59315, &UA2431990},
	{SG59357, &UA64286},
	{SG59359, &UA44166},
	{SG59361, &UA62068},
	{SG59405, &UA69469},
	{SG59409, &UA2385477},
	{SG59411, &UA2403251},
	{SG59412, &UA2422464},
	{SG59413, &UA2423057},
	{SG59414, &UA2426631},
	{SG59415, &UA2429660},
	{SG59416, &UA2456016},
	{SG59417, &UA2458019},
	{SG59418, &UA2467175},
	{SG59419, &UA2480652},
	{SG59420, &UA2481293},
	{SG59421, &UA2482965},
	{SG59422, &UA2505127},
	{SG59423, &UA2505179},
	{SG59424, &UA2528427},
	{SG59425, &UA2553670},
	{SG59426, &UA2554447},
	{SG59427, &UA2555372},
	{SG59428, &UA2594418},
	{SG59429, &UA2595286},
	{SG59430, &UA2653388},
	{SG59431, &UA2655522},
	{SG59432, &UA2694633},
	{SG59433, &UA2695577},
	{SG59434, &UA2695760},
	{SG59435, &UA2696259},
	{SG59436, &UA2696313},
	{SG59437, &UA2746840},
	{SG59438, &UA2791565},
	{SG59439, &UA2841557},
	{SG59440, &UA2934947},
	{SG59441, &UA2951129},
	{SG59442, &UA3119290},
	{SG59452, &UA2385140},
	{SG59453, &UA2385268},
	{SG59454, &UA2385544},
	{SG59455, &UA2423052},
	{SG59456, &UA2425635},
	{SG59457, &UA2426531},
	{SG59458, &UA2502747},
	{SG59459, &UA2504864},
	{SG59460, &UA2505295},
	{SG59461, &UA2528529},
	{SG59462, &UA2555371},
	{SG59463, &UA2578437},
	{SG59464, &UA2653940},
	{SG59465, &UA2659303},
	{SG59466, &UA2695020},
	{SG59467, &UA2727479},
	{SG59468, &UA2728173},
	{SG59469, &UA2843627},
	{SG59470, &UA2950950},
	{SG59471, &UA2955360},
	{SG59472, &UA3044047},
	{SG59473, &UA3118212},
	{SG59474, &UA3118348},
	{SG59511, &UA2431237},
	{SG59614, &UA16091},
	{SG59621, &UA62767},
	{SG59639, &UA3115852},
	{SG59640, &UA63115},
	{SG59651, &UA63147},
	{SG59669, &UA2385730},
	{SG59670, &UA2428657},
	{SG59671, &UA2466544},
	{SG59672, &UA2467019},
	{SG59673, &UA2482220},
	{SG59674, &UA2504296},
	{SG59675, &UA2553449},
	{SG59676, &UA2555150},
	{SG59677, &UA2556335},
	{SG59678, &UA2594672},
	{SG59679, &UA2597892},
	{SG59680, &UA2653952},
	{SG59681, &UA2654111},
	{SG59682, &UA2654637},
	{SG59683, &UA2696234},
	{SG59684, &UA2762940},
	{SG59685, &UA2985759},
	{SG59686, &UA3086016},
	{SG59687, &UA63355},
	{SG59727, &UA55019},
	{SG59741, &UA63049},
	{SG59747, &UA2385110},
	{SG59748, &UA2421112},
	{SG59749, &UA2502634},
	{SG59750, &UA2503072},
	{SG59751, &UA2505140},
	{SG59752, &UA2553879},
	{SG59753, &UA2594452},
	{SG59754, &UA2594764},
	{SG59755, &UA2653870},
	{SG59756, &UA2654867},
	{SG59757, &UA2695057},
	{SG59758, &UA2728125},
	{SG59759, &UA2762715},
	{SG59760, &UA2985164},
	{SG59761, &UA3045000},
	{SG59762, &UA2431929},
	{SG59763, &UA24194},
	{SG59764, &UA36508},
	{SG59765, &UA45073},
	{SG59766, &UA75288},
	{SG59767, &UA64388},
	{SG59792, &UA64142},
	{SG59827, &UA574996},
	{SG59843, &UA8315},
	{SG59844, &UA8320},
	{SG59845, &UA34546},
	{SG59857, &UA2421861},
	{SG59858, &UA2423463},
	{SG59859, &UA2480803},
	{SG59860, &UA2555202},
	{SG59861, &UA2594730},
	{SG59862, &UA2654093},
	{SG59863, &UA2697024},
	{SG59864, &UA2764087},
	{SG59865, &UA2795724},
	{SG59866, &UA2840452},
	{SG59867, &UA2848818},
	{SG59868, &UA2849670},
	{SG59869, &UA2849814},
	{SG59870, &UA2951076},
	{SG59871, &UA2951294},
	{SG59872, &UA2951972},
	{SG59873, &UA3047034},
	{SG59915, &UA11709},
	{SG59949, &UA1852},
	{SG59963, &UA2361653},
	{SG59964, &UA2466511},
	{SG59965, &UA2698519},
	{SG59966, &UA2728508},
	{SG59967, &UA3049099},
	{SG59968, &UA3120516},
	{SG59979, &UA8837},
	{SG60002, &UA51100},
	{SG60003, &UA16727},
	{SG60016, &UA34592},
	{SG60017, &UA63749},
	{SG60018, &UA2423787},
	{SG60019, &UA2428209},
	{SG60020, &UA2504367},
	{SG60021, &UA2553993},
	{SG60022, &UA2554465},
	{SG60023, &UA2554614},
	{SG60024, &UA2594768},
	{SG60025, &UA2594792},
	{SG60026, &UA2596911},
	{SG60027, &UA2654806},
	{SG60028, &UA2842237},
	{SG60029, &UA2951718},
	{SG60090, &UA2423025},
	{SG60091, &UA2528170},
	{SG60092, &UA2555798},
	{SG60093, &UA2597065},
	{SG60094, &UA2652837},
	{SG60095, &UA2653117},
	{SG60096, &UA2695350},
	{SG60097, &UA2790594},
	{SG60098, &UA2847234},
	{SG60099, &UA2849488},
	{SG60100, &UA2890536},
	{SG60101, &UA2934177},
	{SG60102, &UA2984047},
	{SG60103, &UA3043453},
	{SG60104, &UA3099670},
	{SG60106, &UA12596},
	{SG60125, &UA22881},
	{SG60126, &UA22885},
	{SG60127, &UA36225},
	{SG60132, &UA2425416},
	{SG60133, &UA2466392},
	{SG60134, &UA2482541},
	{SG60135, &UA2503081},
	{SG60136, &UA2578785},
	{SG60137, &UA2594904},
	{SG60138, &UA2595395},
	{SG60139, &UA2694929},
	{SG60140, &UA2695414},
	{SG60141, &UA2763172},
	{SG60142, &UA2844825},
	{SG60143, &UA2848025},
	{SG60144, &UA2983591},
	{SG60145, &UA3099541},
	{SG60154, &UA2504841},
	{SG60155, &UA2527731},
	{SG60156, &UA2595468},
	{SG60157, &UA2653515},
	{SG60158, &UA2695836},
	{SG60159, &UA2696004},
	{SG60161, &UA2746797},
	{SG60163, &UA2762752},
	{SG60164, &UA2778660},
	{SG60166, &UA2889194},
	{SG60167, &UA2952020},
	{SG60168, &UA2952111},
	{SG60169, &UA2985906},
	{SG60170, &UA3044575},
	{SG60171, &UA3071353},
	{SG60172, &UA3087539},
	{SG60205, &UA2430861},
	{SG60215, &UA50207},
	{SG60277, &UA1629},
	{SG60278, &UA72242},
	{SG60279, &UA2425384},
	{SG60280, &UA2456743},
	{SG60281, &UA2481206},
	{SG60282, &UA2578356},
	{SG60283, &UA2694726},
	{SG60284, &UA3049217},
	{SG60306, &UA5008},
	{SG60310, &UA2455609},
	{SG60311, &UA2455865},
	{SG60312, &UA2481422},
	{SG60313, &UA2553063},
	{SG60314, &UA2595125},
	{SG60315, &UA2653686},
	{SG60316, &UA2695100},
	{SG60317, &UA2763062},
	{SG60318, &UA2777860},
	{SG60319, &UA2778364},
	{SG60320, &UA2791155},
	{SG60321, &UA3118289},
	{SG60363, &UA28001},
	{SG60364, &UA36825},
	{SG60370, &UA2480775},
	{SG60371, &UA2595087},
	{SG60372, &UA2655371},
	{SG60373, &UA2728137},
	{SG60374, &UA2728220},
	{SG60375, &UA2848714},
	{SG60376, &UA2951260},
	{SG60377, &UA2951284},
	{SG60378, &UA2984011},
	{SG60379, &UA2986371},
	{SG60380, &UA3071465},
	{SG60381, &UA3099337},
	{SG60385, &UA2456362},
	{SG60386, &UA2526603},
	{SG60387, &UA2653479},
	{SG60388, &UA2762436},
	{SG60389, &UA2889477},
	{SG60390, &UA2951877},
	{SG60391, &UA2984916},
	{SG60392, &UA3044452},
	{SG60423, &UA30322},
	{SG60425, &UA31229},
	{SG60437, &UA62678},
	{SG60450, &UA63770},
	{SG60451, &UA23934},
	{SG60460, &UA6674},
	{SG60461, &UA17143},
	{SG60466, &UA64414},
	{SG60481, &UA429053},
	{SG60488, &UA2427623},
	{SG60489, &UA2595356},
	{SG60490, &UA2653350},
	{SG60491, &UA2746573},
	{SG60492, &UA2843961},
	{SG60493, &UA2951086},
	{SG60494, &UA3071948},
	{SG60565, &UA86816},
	{SG60580, &UA25755},
	{SG60603, &UA2555436},
	{SG60604, &UA2654408},
	{SG60605, &UA2695881},
	{SG60623, &UA2777707},
	{SG60625, &UA2982883},
	{SG60626, &UA3070391},
	{SG60645, &UA30962},
	{SG60662, &UA2839787},
	{SG60663, &UA2845855},
	{SG60664, &UA2951125},
	{SG60665, &UA2952173},
	{SG60666, &UA2955022},
	{SG60667, &UA2982274},
	{SG60668, &UA2983542},
	{SG60669, &UA3043036},
	{SG60670, &UA3043170},
	{SG60671, &UA3070685},
	{SG60672, &UA3086437},
	{SG60673, &UA3099462},
	{SG60674, &UA3118742},
	{SG60716, &UA2654500},
	{SG60717, &UA2655302},
	{SG60718, &UA2655844},
	{SG60719, &UA2842565},
	{SG60720, &UA2951447},
	{SG60721, &UA2951520},
	{SG60735, &UA715644},
	{SG60736, &UA1906311},
	{SG60783, &UA2553799},
	{SG60784, &UA2654113},
	{SG60785, &UA2934036},
	{SG60786, &UA2983758},
	{SG60809, &UA2456932},
	{SG60810, &UA2503182},
	{SG60811, &UA2578231},
	{SG60812, &UA32037},
	{SG60813, &UA2384996},
	{SG60817, &UA737965},
	{SG60822, &UA2746722},
	{SG60823, &UA2747467},
	{SG60824, &UA2846682},
	{SG60825, &UA2983640},
	{SG60826, &UA3124553},
	{SG60845, &UA2654757},
	{SG60846, &UA2696081},
	{SG60847, &UA2727876},
	{SG60848, &UA2777669},
	{SG60849, &UA3118632},
	{SG60850, &UA2654893},
	{SG60851, &UA2655422},
	{SG60852, &UA2655846},
	{SG60853, &UA2791104},
	{SG60911, &UA884927},
	{SG60921, &UA32173},
	{SG60922, &UA64380},
	{SG60933, &UA2753424},
	{SG60937, &UA2652882},
	{SG60938, &UA2951280},
	{SG60939, &UA2983636},
	{SG60940, &UA3043192},
	{SG60941, &UA3100562},
	{SG60960, &UA7198},
	{SG60993, &UA14877},
	{SG61002, &UA67736},
	{SG61011, &UA2329314},
	{SG61017, &UA8217},
	{SG61019, &UA2727540},
	{SG61020, &UA2835503},
	{SG61021, &UA2982704},
	{SG61022, &UA2984135},
	{SG61079, &UA2683},
	{SG61091, &UA63273},
	{SG61096, &UA2578935},
	{SG61097, &UA2983075},
	{SG61098, &UA48847},
	{SG61099, &UA76081},
	{SG61103, &UA1901613},
	{SG61129, &UA2695808},
	{SG61130, &UA2777598},
	{SG61131, &UA2982677},
	{SG61132, &UA30484},
	{SG61148, &UA2694177},
	{SG61149, &UA2983710},
	{SG61169, &UA1854},
	{SG61179, &UA24349},
	{SG61185, &UA10415},
	{SG61209, &UA51936},
	{SG61234, &UA14947},
	{SG61235, &UA22735},
	{SG61240, &UA2588314},
	{SG61257, &UA64659},
	{SG61270, &UA2728032},
	{SG61272, &UA32733},
	{SG61281, &UA3070405},
	{SG61296, &UA2807530},
	{SG61303, &UA2518176},
	{SG61305, &UA11965},
	{SG61306, &UA28815},
	{SG61307, &UA2376379},
	{SG61309, &UA2758330},
	{SG61310, &UA45061},
	{SG61312, &UA33081},
	{SG61313, &UA2216570},
	{SG61317, &UA27579},
	{SG61319, &UA80230},
	{SG61320, &UA2979060},
	{SG61323, &UA3098820},
	{SG61324, &UA35335},
	{SG61326, &UA3125134},
	{SG61327, &UA55025},
	{SG61328, &UA80250},
	{SG61329, &UA2742410},
	{SG61330, &UA3085952},
	{SG61331, &UA62342},
	{SG61332, &UA80229},
	{SG61333, &UA80246},
	{SG61334, &UA33594},
	{SG61335, &UA32907},
	{SG61336, &UA1627},
	{SG61337, &UA4833},
	{SG61338, &UA2902},
	{SG61339, &UA25761},
	{SG61343, &UA31245},
	{SG61344, &UA36160},
	{SG61346, &UA80279},
	{SG61347, &UA45040},
	{SG61348, &UA80248},
	{SG61349, &UA12384},
	{SG61350, &UA65152},
	{SG61351, &UA27257},
	{SG61352, &UA602632},
	{SG61353, &UA49718},
	{SG61354, &UA1925092},
	{SG61358, &UA76576},
	{SG61359, &UA80193},
	{SG61370, &UA429367},
	{SG61371, &UA429430},
	{SG61457, &UA18184},
	{SG61479, &UA47639},
	{SG61480, &UA67908},
	{SG61481, &UA4766},
	{SG61482, &UA24353},
	{SG61483, &UA30089},
	{SG61484, &UA33884},
	{SG61485, &UA38760},
	{SG61486, &UA48655},
	{SG61521, &UA80273},
	{SG61522, &UA80259},
	{SG61531, &UA87096},
	{NULL}};
const Device *DCatchAllEditDistance[] = {
	&UA669685,
	&UA677076,
	&UA686355,
	&UA806151,
	&UA95141,
	&UA2842991,
	&UA3040624,
	&UA505068,
	&UA2243753,
	&UA3072358,
	&UA641477,
	&UA684851,
	&UA712430,
	&UA754691,
	&UA786144,
	&UA890201,
	&UA1992546,
	&UA2158162,
	&UA2219681,
	&UA2314273,
	&UA2330521,
	&UA2395538,
	&UA2554870,
	&UA2594601,
	&UA2817604,
	&UA2843913,
	&UA10048,
	&UA13466,
	&UA13470,
	&UA101497,
	&UA101498,
	&UA133053,
	&UA162828,
	&UA460575,
	&UA603614,
	&UA603869,
	&UA615858,
	&UA747648,
	&UA2005711,
	&UA2512796,
	&UA2512906,
	&UA2640193,
	&UA2709560,
	&UA2719551,
	&UA2935634,
	&UA3000313,
	&UA2395660,
	&UA2587678,
	&UA2654393,
	&UA1980195,
	&UA1964434,
	&UA2467257,
	&UA2468771,
	&UA2528654,
	&UA2553918,
	&UA2577853,
	&UA2658337,
	&UA2659085,
	&UA2697602,
	&UA2698359,
	&UA2704200,
	&UA2708708,
	&UA2731091,
	&UA2763128,
	&UA2764232,
	&UA2878266,
	&UA3042868,
	&UA2979947,
	&UA2467700,
	&UA2528630,
	&UA2554386,
	&UA2598650,
	&UA2603555,
	&UA2653375,
	&UA2698356,
	&UA2731685,
	&UA2778795,
	&UA2790974,
	&UA2793799,
	&UA2849278,
	&UA2983184,
	&UA2985314,
	&UA3044046,
	&UA608925,
	&UA630683,
	&UA911770,
	&UA911779,
	&UA2355749,
	&UA2356056,
	&UA2492145,
	&UA2508911,
	&UA2585684,
	&UA2600849,
	&UA2714997,
	&UA2740525,
	&UA2772950,
	&UA2780947,
	&UA3067729,
	&UA3050327,
	&UA616646,
	&UA349718,
	&UA361858,
	&UA23660,
	&UA92406,
	&UA124434,
	&UA148246,
	&UA151900,
	&UA151907,
	&UA152288,
	&UA152328,
	&UA171130,
	&UA232013,
	&UA300341,
	&UA348036,
	&UA901236,
	&UA2014088,
	&UA2570292,
	&UA130986,
	&UA151362,
	&UA152591,
	&UA180149,
	&UA628410,
	&UA912854,
	&UA2200481,
	&UA2362773,
	&UA2362799,
	&UA2513939,
	&UA2568161,
	&UA2569989,
	&UA2719021,
	&UA2796733,
	&UA2972570,
	&UA193972,
	&UA200580,
	&UA315124,
	&UA316578,
	&UA448621,
	&UA2980918,
	&UA2880140,
	&UA2896172,
	&UA3019231,
	&UA2421009,
	&UA2597260,
	&UA2653318,
	&UA2659313,
	&UA2841200,
	&UA2865525,
	&UA2985753,
	&UA2997251,
	&UA3088233,
	&UA123571,
	&UA207256,
	&UA324167,
	&UA381338,
	&UA401250,
	&UA831449,
	&UA911775,
	&UA1965226,
	&UA2233801,
	&UA2242320,
	&UA2428525,
	&UA2487455,
	&UA2490549,
	&UA2716095,
	&UA2784293,
	&UA2137030,
	&UA2284951,
	&UA448207,
	&UA567633,
	&UA1865685,
	&UA2206872,
	&UA2437763,
	&UA2948629,
	&UA1935253,
	&UA2991945,
	&UA2650306,
	&UA2776150,
	&UA2828525,
	&UA2981343,
	&UA577771,
	&UA1999452,
	&UA454025,
	&UA625506,
	&UA896684,
	&UA2681873,
	&UA3091855,
	&UA30062,
	&UA302634,
	&UA1871274,
	&UA2400547,
	&UA2784775,
	&UA2879588,
	&UA2886438,
	&UA3028273,
	&UA2619370,
	&UA2622920,
	&UA2340055,
	&UA2486646,
	&UA2659250,
	&UA2729733,
	&UA2729834,
	&UA2764415,
	&UA2845003,
	&UA2850650,
	&UA2890887,
	&UA2892800,
	&UA2993008,
	&UA2997701,
	&UA2998441,
	&UA2999323,
	&UA3000589,
	&UA359561,
	&UA616731,
	&UA710537,
	&UA2257220,
	&UA2398612,
	&UA2411269,
	&UA2463608,
	&UA2463814,
	&UA2494675,
	&UA3048707,
	&UA2205827,
	&UA2578681,
	&UA2754180,
	&UA2773083,
	&UA2773673,
	&UA2774171,
	&UA2781404,
	&UA2870017,
	&UA2871601,
	&UA2873621,
	&UA3030106,
	&UA3034870,
	&UA3036000,
	&UA3036006,
	&UA3078544,
	&UA2334719,
	&UA2356277,
	&UA2483890,
	&UA2505070,
	&UA2524273,
	&UA2526430,
	&UA2577632,
	&UA2580294,
	&UA2596172,
	&UA2599083,
	&UA2645702,
	&UA2660960,
	&UA2663210,
	&UA2666604,
	&UA2737577,
	&UA902895,
	&UA2222173,
	&UA608744,
	&UA2355804,
	&UA2359488,
	&UA2408160,
	&UA2414160,
	&UA2429900,
	&UA2461827,
	&UA2511521,
	&UA2515521,
	&UA2516720,
	&UA2657814,
	&UA2679672,
	&UA2732342,
	&UA2737635,
	&UA2784785,
	&UA2614124,
	&UA2616025,
	&UA2619600,
	&UA2635162,
	&UA2428172,
	&UA2462203,
	&UA2490556,
	&UA2492141,
	&UA2605189,
	&UA2712992,
	&UA2740612,
	&UA2874394,
	&UA2888687,
	&UA2899268,
	&UA2948415,
	&UA3025453,
	&UA3034093,
	&UA3036994,
	&UA3115586,
	&UA754129,
	&UA303256,
	&UA760530,
	&UA92819,
	&UA110283,
	&UA124653,
	&UA130255,
	&UA131033,
	&UA154340,
	&UA160645,
	&UA195402,
	&UA210384,
	&UA265384,
	&UA417799,
	&UA563926,
	&UA1898404,
	&UA2368740,
	&UA2682180,
	&UA2616217,
	&UA2619447,
	&UA2619725,
	&UA2626324,
	&UA2634310,
	&UA2035153,
	&UA2695942,
	&UA1955782,
	&UA2235176,
	&UA2375788,
	&UA2383538,
	&UA2482898,
	&UA2578827,
	&UA2601526,
	&UA2658455,
	&UA2697023,
	&UA2728187,
	&UA2779532,
	&UA2848880,
	&UA2850144,
	&UA2986782,
	&UA2999382,
	&UA456195,
	&UA626960,
	&UA633122,
	&UA2202480,
	&UA2371905,
	&UA2401617,
	&UA2455508,
	&UA2652743,
	&UA2773259,
	&UA2783677,
	&UA2871615,
	&UA2893570,
	&UA2941953,
	&UA3030107,
	&UA3030148,
	&UA2504557,
	&UA183426,
	&UA189717,
	&UA409428,
	&UA572695,
	&UA629959,
	&UA682534,
	&UA781589,
	&UA2024459,
	&UA2025330,
	&UA2200212,
	&UA2663376,
	&UA2697320,
	&UA2698267,
	&UA2813365,
	&UA2966883,
	&UA2625222,
	&UA2629990,
	&UA2633087,
	&UA456754,
	&UA2160789,
	&UA2161973,
	&UA2455585,
	&UA2473924,
	&UA2964930,
	&UA3058568,
	&UA3121632,
	&UA1980351,
	&UA92229,
	&UA94197,
	&UA98177,
	&UA98606,
	&UA99071,
	&UA102232,
	&UA104753,
	&UA123060,
	&UA137559,
	&UA145107,
	&UA147276,
	&UA147474,
	&UA149062,
	&UA149450,
	&UA164170,
	&UA2984891,
	&UA149622,
	&UA421311,
	&UA543950,
	&UA2364276,
	&UA2588241,
	&UA2681812,
	&UA2757852,
	&UA2803994,
	&UA2859249,
	&UA3087963,
	&UA3100721,
	&UA516766,
	&UA2288860,
	&UA162117,
	&UA170980,
	&UA216227,
	&UA262838,
	&UA297378,
	&UA351176,
	&UA366314,
	&UA705584,
	&UA705705,
	&UA743315,
	&UA767229,
	&UA805664,
	&UA2473236,
	&UA2742259,
	&UA2774170,
	&UA2427844,
	&UA1876782,
	&UA475249,
	&UA2141957,
	&UA2607800,
	&UA2609810,
	&UA2621562,
	&UA2623348,
	&UA2624757,
	&UA2629010,
	&UA2631471,
	&UA2633382,
	&UA156132,
	&UA2502322,
	&UA2639346,
	&UA2639671,
	&UA2639816,
	&UA2727596,
	&UA2615666,
	&UA2619020,
	&UA2633410,
	&UA92806,
	&UA94199,
	&UA106430,
	&UA688372,
	&UA2163398,
	&UA2281212,
	&UA2516853,
	&UA3058047,
	&UA190612,
	&UA2937963,
	&UA2965378,
	&UA2972141,
	&UA2983585,
	&UA3012175,
	&UA3015093,
	&UA3033935,
	&UA3064045,
	&UA3077851,
	&UA3084034,
	&UA3096271,
	&UA3107402,
	&UA3111761,
	&UA3114275,
	&UA3129560,
	&UA2616971,
	&UA2629087,
	&UA2635781,
	&UA2612814,
	&UA2618542,
	&UA2620134,
	&UA2620866,
	&UA2622496,
	&UA2624057,
	&UA2626288,
	&UA2627241,
	&UA2793527,
	&UA89270,
	&UA104629,
	&UA109864,
	&UA127660,
	&UA141959,
	&UA157442,
	&UA282179,
	&UA292521,
	&UA450617,
	&UA473691,
	&UA567502,
	&UA890148,
	&UA2042981,
	&UA2371741,
	&UA2408647,
	&UA2566664,
	&UA2567497,
	&UA117957,
	&UA396961,
	&UA575206,
	&UA701583,
	&UA716962,
	&UA726261,
	&UA745695,
	&UA1938183,
	&UA1939472,
	&UA2149891,
	&UA2810919,
	&UA3042115,
	&UA2636700,
	&UA2617654,
	&UA2622453,
	&UA2624143,
	&UA2632777,
	&UA2611349,
	&UA2614483,
	&UA2614969,
	&UA2615807,
	&UA2625515,
	&UA2629782,
	&UA2635521,
	&UA2637996,
	&UA2625906,
	&UA2626545,
	&UA2627512,
	&UA2632924,
	&UA2634846,
	&UA2635466,
	&UA2636028,
	&UA2625987,
	&UA2632772,
	&UA2633223,
	&UA2635128,
	&UA2615919,
	&UA2622202,
	&UA2624193,
	&UA2624627,
	&UA2626224,
	&UA2626405,
	&UA2626434,
	&UA2629524,
	&UA2631736,
	&UA2635861,
	&UA2636977,
	&UA2948474,
	&UA2975737,
	&UA3023020,
	&UA3061615,
	&UA3083155,
	&UA2979704,
	&UA2208164,
	&UA23041,
	&UA93049,
	&UA94198,
	&UA106338,
	&UA115618,
	&UA118078,
	&UA625752,
	&UA2490859,
	&UA2490871,
	&UA2533320,
	&UA2989666,
	&UA2613165,
	&UA2629419,
	&UA2957347,
	&UA2959640,
	&UA2778121,
	&UA2846207,
	&UA2601354,
	&UA2618887,
	&UA899615,
	&UA2734983,
	&UA2841404,
	&UA2846227,
	&UA2615589,
	&UA2621923,
	&UA2632460,
	&UA396888,
	&UA1937948,
	&UA2216017,
	&UA2619325,
	&UA2622653,
	&UA451379,
	&UA2639347,
	&UA2612702,
	&UA2633267,
	&UA2634217,
	&UA2755435,
	&UA2892789,
	&UA2678413,
	&UA2781605,
	&UA169935,
	&UA99890,
	&UA538851,
	&UA603334,
	&UA754831,
	&UA884543,
	&UA901728,
	&UA902620,
	&UA1896268,
	&UA1897671,
	&UA1897837,
	&UA2391466,
	&UA2995810,
	&UA3089205,
	&UA2866869,
	&UA2869119,
	&UA2958228,
	&UA3003273,
	&UA3006953,
	&UA3019214,
	&UA3023509,
	&UA3054191,
	&UA3060112,
	&UA3073576,
	&UA3074124,
	&UA3078206,
	&UA3092490,
	&UA3108105,
	&UA3125701,
	&UA2139346,
	&UA2149956,
	&UA2258522,
	&UA2986316,
	&UA93141,
	&UA97816,
	&UA438315,
	&UA532550,
	&UA624558,
	&UA629843,
	&UA633735,
	&UA634261,
	&UA642563,
	&UA1992450,
	&UA2039624,
	&UA2168320,
	&UA2172767,
	&UA2203643,
	&UA2774168,
	&UA2891145,
	&UA2998692,
	&UA2561580,
	&UA2562923,
	&UA2599741,
	&UA175777,
	&UA719560,
	&UA2166581,
	&UA2978831,
	&UA2979309,
	&UA33430,
	&UA145708,
	&UA146667,
	&UA163054,
	&UA315249,
	&UA383357,
	&UA1931011,
	&UA2020630,
	&UA2801486,
	&UA97279,
	&UA166332,
	&UA481533,
	&UA1842988,
	&UA3073352,
	&UA3124484,
	&UA448372,
	&UA98999,
	&UA141962,
	&UA1874528,
	&UA221335,
	&UA2860016,
	&UA2957751,
	&UA2690750,
	&UA2713369,
	&UA100327,
	&UA127037,
	&UA140389,
	&UA226575,
	&UA393681,
	&UA408216,
	&UA608631,
	&UA612870,
	&UA622668,
	&UA645408,
	&UA672224,
	&UA705198,
	&UA891582,
	&UA900542,
	&UA901327,
	&UA134008,
	&UA658006,
	&UA2443261,
	&UA2217845,
	&UA3023635,
	&UA3125236,
	&UA2889488,
	&UA91534,
	&UA94064,
	&UA94162,
	&UA94817,
	&UA99357,
	&UA121513,
	&UA173917,
	&UA175171,
	&UA181104,
	&UA186628,
	&UA269559,
	&UA92396,
	&UA92877,
	&UA93939,
	&UA176633,
	&UA238826,
	&UA290681,
	&UA2238090,
	&UA2492143,
	&UA2510937,
	&UA2679933,
	&UA2740232,
	&UA2753247,
	&UA2771290,
	&UA2870912,
	&UA2888689,
	&UA253783,
	&UA2893498,
	&UA2893581,
	&UA2895906,
	&UA2898295,
	&UA2958458,
	&UA2999150,
	&UA2999309,
	&UA3000929,
	&UA3004939,
	&UA3006048,
	&UA3006644,
	&UA3015060,
	&UA3019687,
	&UA3050620,
	&UA3074326,
	&UA79886,
	&UA79883,
	&UA2982881,
	&UA427339,
	&UA427524,
	&UA441484,
	&UA903622,
	&UA923815,
	&UA923826,
	&UA924815,
	&UA1996221,
	&UA2041314,
	&UA2155834,
	&UA2751996,
	&UA2752389,
	&UA79890,
	&UA382213,
	&UA1977501,
	&UA232723,
	&UA235398,
	&UA609249,
	&UA613278,
	&UA613284,
	&UA613349,
	&UA613350,
	&UA613595,
	&UA2601137,
	&UA2602051,
	&UA2602312,
	&UA2603280,
	&UA2603842,
	&UA2603844,
	&UA2604153,
	&UA91269,
	&UA2584170,
	&UA2604483,
	&UA2604530,
	&UA2964867,
	&UA3058441,
	&UA412975,
	&UA878074,
	&UA900507,
	&UA2043122,
	&UA2262882,
	&UA2462197,
	&UA2585460,
	&UA2601875,
	&UA2678753,
	&UA2888619,
	&UA2974394,
	&UA2992864,
	&UA3027132,
	&UA3030581,
	&UA3035775,
	&UA93317,
	&UA140405,
	&UA681494,
	&UA696704,
	&UA808273,
	&UA1837163,
	&UA2392318,
	&UA2001048,
	&UA2204099,
	&UA2408266,
	&UA2492134,
	&UA2771287,
	&UA2870910,
	&UA2888682,
	&UA2974878,
	&UA2975509,
	&UA3025461,
	&UA3032607,
	&UA3037712,
	&UA3037753,
	&UA3068049,
	&UA3080592,
	&UA1988273,
	&UA116011,
	&UA372617,
	&UA2696183,
	&UA2809781,
	&UA3073250,
	&UA2636818,
	&UA381219,
	&UA398811,
	&UA92399,
	&UA105711,
	&UA261698,
	&UA2405835,
	&UA2508091,
	&UA2680224,
	&UA2936012,
	&UA2241457,
	&UA2241461,
	&UA2241468,
	&UA2241465,
	&UA2242080,
	&UA2356692,
	&UA2386408,
	&UA2426926,
	&UA2428162,
	&UA2437267,
	&UA2566835,
	&UA2567665,
	&UA2570277,
	&UA2570286,
	&UA2582958,
	&UA2601983,
	&UA2603560,
	&UA2603679,
	&UA2604379,
	&UA2666593,
	&UA175917,
	&UA2282367,
	&UA2933194,
	&UA2934093,
	&UA2937739,
	&UA2950923,
	&UA2953771,
	&UA2977415,
	&UA2977675,
	&UA2979607,
	&UA3008505,
	&UA3040714,
	&UA3058935,
	&UA3084855,
	&UA3108219,
	&UA2999863,
	&UA2672788,
	&UA2735125,
	&UA2799198,
	&UA3057799,
	&UA3092512,
	&UA2237666,
	&UA2428052,
	&UA2570874,
	&UA2603592,
	&UA385278,
	&UA2199975,
	&UA2356521,
	&UA2356708,
	&UA2356717,
	&UA2356971,
	&UA2437577,
	&UA2532698,
	&UA2532752,
	&UA2605947,
	&UA2606858,
	&UA2606958,
	&UA2607374,
	&UA2710762,
	&UA2712549,
	&UA895509,
	&UA2010426,
	&UA2197911,
	&UA2757160,
	&UA2535169,
	&UA2735038,
	&UA2749427,
	&UA2957801,
	&UA3009965,
	&UA2469414,
	&UA2665490,
	&UA2997572,
	&UA144083,
	&UA2702159,
	&UA614780,
	&UA821828,
	&UA876431,
	&UA905456,
	&UA913433,
	&UA2001872,
	&UA2030778,
	&UA2355057,
	&UA2450213,
	&UA2516372,
	&UA2557182,
	&UA488822,
	&UA2758084,
	&UA2225654,
	&UA2608760,
	&UA2623800,
	&UA2628717,
	&UA2341334,
	&UA111528,
	&UA131567,
	&UA157171,
	&UA164715,
	&UA296602,
	&UA416807,
	&UA716727,
	&UA1872609,
	&UA2456978,
	&UA2774851,
	&UA2981460,
	&UA2018720,
	&UA1996146,
	&UA2028335,
	&UA2256931,
	&UA3043578,
	&UA503356,
	&UA516789,
	&UA826638,
	&UA2139235,
	&UA2720213,
	&UA2675605,
	&UA2799413,
	&UA3011519,
	&UA3047704,
	&UA3125343,
	&UA2516623,
	&UA79914,
	&UA79899,
	&UA901543,
	&UA909770,
	&UA1995689,
	&UA2204453,
	&UA2236866,
	&UA2982084,
	&UA2760614,
	&UA43314,
	&UA61870,
	&UA88719,
	&UA130235,
	&UA164716,
	&UA2145998,
	&UA2393042,
	&UA5020,
	&UA542539,
	&UA2481384,
	&UA2534213,
	&UA2767099,
	&UA2879405,
	&UA2997916,
	&UA2466304,
	&UA2483453,
	&UA2596217,
	&UA2596296,
	&UA2601689,
	&UA2604549,
	&UA2659133,
	&UA2660864,
	&UA2667235,
	&UA2708896,
	&UA2735357,
	&UA2736571,
	&UA2737995,
	&UA2768465,
	&UA2768501,
	&UA2387887,
	&UA2531340,
	&UA2657720,
	&UA2679840,
	&UA2870016,
	&UA2873841,
	&UA2882475,
	&UA2938571,
	&UA2972380,
	&UA3031020,
	&UA3036007,
	&UA3036752,
	&UA3058876,
	&UA3078882,
	&UA3094704,
	&UA3125539,
	&UA2998337,
	&UA2784929,
	&UA2800313,
	&UA2875641,
	&UA2876989,
	&UA2947396,
	&UA2963964,
	&UA2973593,
	&UA2974689,
	&UA3024035,
	&UA3027643,
	&UA3031941,
	&UA3061130,
	&UA3062642,
	&UA3066917,
	&UA2354720,
	&UA2424482,
	&UA2529374,
	&UA2570873,
	&UA2733300,
	&UA79895,
	&UA2508166,
	&UA2712736,
	&UA2793247,
	&UA344831,
	&UA609259,
	&UA902157,
	&UA925084,
	&UA2040383,
	&UA2042025,
	&UA2030996,
	&UA2384490,
	&UA2457656,
	&UA2516373,
	&UA79904,
	&UA2507348,
	&UA3062537,
	&UA3063130,
	&UA3094437,
	&UA3109930,
	&UA3113444,
	&UA174963,
	&UA2360624,
	&UA2487941,
	&UA2018061,
	&UA924757,
	&UA2028816,
	&UA2228367,
	&UA2368647,
	&UA3124349,
	&UA2018490,
	&UA2386951,
	&UA2390392,
	&UA2531399,
	&UA2661915,
	&UA2749436,
	&UA2750784,
	&UA2762820,
	&UA2790705,
	&UA2956701,
	&UA2989799,
	&UA2996054,
	&UA3072766,
	&UA3088820,
	&UA3101014,
	&UA3120235,
	&UA2521016,
	&UA2558262,
	&UA3103206,
	&UA124007,
	&UA2567900,
	&UA2716362,
	&UA79892,
	&UA2705618,
	&UA2963970,
	&UA2971254,
	&UA3020695,
	&UA3023937,
	&UA2740018,
	&UA3018856,
	&UA79888,
	&UA2799253,
	&UA909526,
	&UA1996242,
	&UA2204452,
	&UA2229555,
	&UA2236865,
	&UA2256941,
	&UA92768,
	&UA105708,
	&UA109744,
	&UA123527,
	&UA148324,
	&UA169390,
	&UA173893,
	&UA302383,
	&UA2530134,
	&UA2579836,
	&UA2580310,
	&UA2659410,
	&UA2702400,
	&UA2702917,
	&UA2707127,
	&UA2749204,
	&UA2848775,
	&UA2984243,
	&UA454072,
	&UA608487,
	&UA612124,
	&UA612446,
	&UA725493,
	&UA768117,
	&UA894549,
	&UA1951165,
	&UA1953694,
	&UA1966066,
	&UA1978431,
	&UA1994222,
	&UA2153260,
	&UA2153883,
	&UA2222006,
	&UA907507,
	&UA924831,
	&UA2027722,
	&UA2041282,
	&UA2042318,
	&UA2205385,
	&UA2762217,
	&UA2715721,
	&UA2737584,
	&UA2738813,
	&UA2781587,
	&UA2795272,
	&UA2864922,
	&UA2867729,
	&UA2878936,
	&UA2898051,
	&UA2974683,
	&UA3018349,
	&UA3021600,
	&UA3025541,
	&UA3078239,
	&UA3092279,
	&UA79881,
	&UA3045504,
	&UA623390,
	&UA627706,
	&UA629198,
	&UA630100,
	&UA642780,
	&UA647798,
	&UA672119,
	&UA675479,
	&UA686583,
	&UA692751,
	&UA699290,
	&UA712624,
	&UA738722,
	&UA742608,
	&UA748690,
	&UA3014089,
	&UA497322,
	&UA2018540,
	&UA94491,
	&UA96312,
	&UA100532,
	&UA101344,
	&UA103781,
	&UA103971,
	&UA123375,
	&UA138054,
	&UA261005,
	&UA265183,
	&UA302021,
	&UA529290,
	&UA901715,
	&UA1966440,
	&UA2669347,
	&UA2886399,
	&UA2603050,
	&UA85159,
	&UA2258449,
	&UA140257,
	&UA792809,
	&UA2197574,
	&UA2202800,
	&UA2260125,
	&UA2368560,
	&UA2368585,
	&UA2407243,
	&UA2435299,
	&UA2489179,
	&UA2511357,
	&UA2511495,
	&UA2562408,
	&UA2895430,
	&UA3066822,
	&UA45104,
	&UA141575,
	&UA853465,
	&UA899637,
	&UA2712085,
	&UA2712564,
	&UA2718968,
	&UA874480,
	&UA2367077,
	&UA2385888,
	&UA2386201,
	&UA2403957,
	&UA2428404,
	&UA2668838,
	&UA2668849,
	&UA2669617,
	&UA2669719,
	&UA2670652,
	&UA2675223,
	&UA2703625,
	&UA2704050,
	&UA2767123,
	&UA2755143,
	&UA3017351,
	&UA3108419,
	&UA2219738,
	&UA2708599,
	&UA430163,
	&UA93326,
	&UA130267,
	&UA1966877,
	&UA2603336,
	&UA2605457,
	&UA2605664,
	&UA2679612,
	&UA2679616,
	&UA2739315,
	&UA2752893,
	&UA2753806,
	&UA2796933,
	&UA2888564,
	&UA2896914,
	&UA2944535,
	&UA3038809,
	&UA3067726,
	&UA3096098,
	&UA2235767,
	&UA646591,
	&UA2151299,
	&UA2215914,
	&UA2310468,
	&UA2317712,
	&UA2505608,
	&UA2556991,
	&UA2557060,
	&UA2573900,
	&UA2573950,
	&UA2574262,
	&UA2589222,
	&UA2651805,
	&UA2658420,
	&UA2698481,
	&UA2605791,
	&UA2605795,
	&UA2679611,
	&UA2679618,
	&UA2710761,
	&UA2739311,
	&UA2739317,
	&UA2796730,
	&UA2796732,
	&UA2888582,
	&UA2888614,
	&UA3038195,
	&UA3038785,
	&UA3067719,
	&UA3067724,
	&UA2312444,
	&UA2535636,
	&UA10050,
	&UA10054,
	&UA481385,
	&UA2408202,
	&UA2408197,
	&UA2425855,
	&UA2585769,
	&UA2385762,
	&UA2427592,
	&UA2529636,
	&UA2568080,
	&UA2602520,
	&UA2603569,
	&UA2666080,
	&UA2666737,
	&UA2666972,
	&UA2700357,
	&UA2704955,
	&UA2705751,
	&UA2708165,
	&UA2780181,
	&UA2795159,
	&UA1977437,
	&UA2397312,
	&UA2432108,
	&UA2466950,
	&UA2546050,
	&UA2574565,
	&UA2574843,
	&UA2652955,
	&UA2693232,
	&UA2792120,
	&UA2841482,
	&UA2842737,
	&UA2878255,
	&UA3071021,
	&UA12707,
	&UA2626256,
	&UA421906,
	&UA622988,
	&UA674006,
	&UA689535,
	&UA694982,
	&UA705486,
	&UA720069,
	&UA726294,
	&UA753899,
	&UA758332,
	&UA768247,
	&UA772416,
	&UA772918,
	&UA775605,
	&UA1914992,
	&UA259495,
	&UA2717763,
	&UA3057922,
	&UA2868194,
	&UA3003184,
	&UA94729,
	&UA2978841,
	&UA79877,
	&UA33066,
	&UA79875,
	&UA90026,
	&UA2031846,
	&UA2230753,
	&UA2327908,
	&UA2517923,
	&UA2528625,
	&UA2660841,
	&UA2664405,
	&UA2666123,
	&UA2709396,
	&UA2727114,
	&UA2733153,
	&UA2736552,
	&UA2795290,
	&UA2864740,
	&UA101287,
	&UA131712,
	&UA132926,
	&UA2471636,
	&UA132729,
	&UA2719026,
	&UA2039867,
	&UA774272,
	&UA782200,
	&UA9952,
	&UA1994162,
	&UA2043053,
	&UA900116,
	&UA2469446,
	&UA2472038,
	&UA2472060,
	&UA2897410,
	&UA2944550,
	&UA3011512,
	&UA2201976,
	&UA2205977,
	&UA2260608,
	&UA2260649,
	&UA2356776,
	&UA2386970,
	&UA2404402,
	&UA2425568,
	&UA2428531,
	&UA2433189,
	&UA2578456,
	&UA2203851,
	&UA2760427,
	&UA119160,
	&UA387391,
	&UA401300,
	&UA414420,
	&UA446140,
	&UA496648,
	&UA505462,
	&UA585954,
	&UA631871,
	&UA696569,
	&UA890355,
	&UA433690,
	&UA515200,
	&UA536789,
	&UA2225003,
	&UA2261999,
	&UA2367272,
	&UA2401280,
	&UA2476120,
	&UA2692673,
	&UA2709248,
	&UA2734575,
	&UA2766911,
	&UA2863871,
	&UA2891778,
	&UA2996870,
	&UA2201568,
	&UA2202147,
	&UA2203197,
	&UA2260974,
	&UA2260977,
	&UA2261073,
	&UA2261147,
	&UA2356715,
	&UA2437568,
	&UA26623,
	&UA2877592,
	&UA2569279,
	&UA3039893,
	&UA2491831,
	&UA2487134,
	&UA2668066,
	&UA64334,
	&UA3064100,
	&UA3064095,
	&UA612009,
	&UA833531,
	&UA856980,
	&UA887577,
	&UA1956516,
	&UA1964182,
	&UA1992670,
	&UA2161831,
	&UA2163146,
	&UA2221710,
	&UA2334191,
	&UA2339418,
	&UA2412237,
	&UA2641991,
	&UA2685887,
	&UA790156,
	&UA1964820,
	&UA2242228,
	&UA2268710,
	&UA2302898,
	&UA2319267,
	&UA2337567,
	&UA2392430,
	&UA2410605,
	&UA2437204,
	&UA2441263,
	&UA2472951,
	&UA2588298,
	&UA2596666,
	&UA2659349,
	&UA908829,
	&UA924950,
	&UA1982150,
	&UA1995264,
	&UA2404641,
	&UA2468014,
	&UA2507382,
	&UA2507521,
	&UA2654676,
	&UA2696871,
	&UA2696911,
	&UA2728994,
	&UA2763084,
	&UA2764526,
	&UA2764955,
	&UA2984239,
	&UA2985968,
	&UA3043707,
	&UA609976,
	&UA575228,
	&UA253775,
	&UA910159,
	&UA2031619,
	&UA2679629,
	&UA3016940,
	&UA2860029,
	&UA79897,
	&UA846766,
	&UA2483664,
	&UA2487582,
	&UA2533321,
	&UA2582776,
	&UA2599987,
	&UA2740587,
	&UA2437646,
	&UA2750934,
	&UA2567476,
	&UA3125054,
	&UA2571398,
	&UA2671207,
	&UA2719291,
	&UA2733894,
	&UA2754150,
	&UA2754268,
	&UA2859151,
	&UA2869364,
	&UA2872326,
	&UA2966721,
	&UA2967891,
	&UA3028151,
	&UA3034374,
	&UA3037093,
	&UA3080856,
	&UA79901,
	&UA257244,
	&UA3070613,
	&UA64421,
	&UA20202,
	&UA49277,
	&UA228246,
	&UA786783,
	&UA899336,
	&UA903205,
	&UA905860,
	&UA2793857,
	&UA12705,
	&UA36223,
	&UA40781,
	&UA925804,
	&UA2182244,
	&UA2672575,
	&UA2859169,
	&UA2895186,
	&UA3077849,
	&UA3108097,
	&UA2025678,
	&UA2223404,
	&UA2380551,
	&UA2382225,
	&UA2426528,
	&UA2520509,
	&UA2528631,
	&UA2654061,
	&UA2695469,
	&UA2701654,
	&UA2838100,
	&UA2998837,
	&UA446789,
	&UA472523,
	&UA499457,
	&UA534141,
	&UA648764,
	&UA673097,
	&UA691574,
	&UA899412,
	&UA1863952,
	&UA2128101,
	&UA2138093,
	&UA2758960,
	&UA33425,
	&UA2738002,
	&UA2768516,
	&UA2795270,
	&UA3018901,
	&UA3020707,
	&UA3125475,
	&UA2388457,
	&UA107004,
	&UA123547,
	&UA377693,
	&UA2742841,
	&UA2945954,
	&UA3004363,
	&UA896613,
	&UA138488,
	&UA164937,
	&UA7232,
	&UA512075,
	&UA858529,
	&UA897124,
	&UA1899116,
	&UA1939696,
	&UA2126149,
	&UA2128177,
	&UA2130659,
	&UA912056,
	&UA2767714,
	&UA368764,
	&UA369662,
	&UA2026365,
	&UA2033309,
	&UA2192452,
	&UA2217629,
	&UA2242185,
	&UA2374723,
	&UA2474456,
	&UA2520686,
	&UA2545597,
	&UA2700156,
	&UA2808938,
	&UA3041788,
	&UA3085563,
	&UA895243,
	&UA895259,
	&UA901046,
	&UA902201,
	&UA907462,
	&UA909958,
	&UA910647,
	&UA924791,
	&UA924826,
	&UA1997808,
	&UA2000847,
	&UA2202083,
	&UA2202917,
	&UA2260562,
	&UA2367655,
	&UA243038,
	&UA2560818,
	&UA140669,
	&UA442714,
	&UA530861,
	&UA549045,
	&UA643848,
	&UA1858931,
	&UA2737127,
	&UA23106,
	&UA3502,
	&UA7542,
	&UA13456,
	&UA59266,
	&UA87891,
	&UA2879209,
	&UA788745,
	&UA2306474,
	&UA2807880,
	&UA2571694,
	&UA2677092,
	&UA2702722,
	&UA2796345,
	&UA2599170,
	&UA2664231,
	&UA2699053,
	&UA2700565,
	&UA2700710,
	&UA2712487,
	&UA2729924,
	&UA2764272,
	&UA2793724,
	&UA2853594,
	&UA2860281,
	&UA2866368,
	&UA2954232,
	&UA2990806,
	&UA2996732,
	&UA87059,
	&UA87851,
	&UA2405316,
	&UA10158,
	&UA2483331,
	&UA2484471,
	&UA2504668,
	&UA2530086,
	&UA2535653,
	&UA2577940,
	&UA2699941,
	&UA2707547,
	&UA2767519,
	&UA2849567,
	&UA2864672,
	&UA2935351,
	&UA2951712,
	&UA2984967,
	&UA2986194,
	&UA2943781,
	&UA1945117,
	&UA2374786,
	&UA2526946,
	&UA2552258,
	&UA2656927,
	&UA2810980,
	&UA2936807,
	&UA3103348,
	&UA2463271,
	&UA62695,
	&UA2528487,
	&UA2560823,
	&UA2791753,
	&UA2845607,
	&UA3070092,
	&UA3099623,
	&UA54094,
	&UA62958,
	&UA85992,
	&UA86983,
	&UA115804,
	&UA537098,
	&UA905771,
	&UA124204,
	&UA2404245,
	&UA3084416,
	&UA3084417,
	&UA3084415,
	&UA3111789,
	&UA3114993,
	&UA3115133,
	&UA54051,
	&UA314000,
	&UA375423,
	&UA673974,
	&UA740791,
	&UA823050,
	&UA2787245,
	&UA2878387,
	&UA2989244,
	&UA2666492,
	&UA2708103,
	&UA2765635,
	&UA2879185,
	&UA2937338,
	&UA2937593,
	&UA2951973,
	&UA2965682,
	&UA2997923,
	&UA2999140,
	&UA2999464,
	&UA3002480,
	&UA3004365,
	&UA3089444,
	&UA3120433,
	&UA2489080,
	&UA2532651,
	&UA2533477,
	&UA2585464,
	&UA3132633,
	&UA624713,
	&UA632799,
	&UA636832,
	&UA651656,
	&UA666808,
	&UA683895,
	&UA695766,
	&UA708802,
	&UA724685,
	&UA854854,
	&UA1915881,
	&UA1935063,
	&UA2148136,
	&UA2148571,
	&UA2215656,
	&UA401598,
	&UA24266,
	&UA28090,
	&UA851284,
	&UA893053,
	&UA924846,
	&UA1956050,
	&UA2028885,
	&UA3131660,
	&UA89076,
	&UA2892091,
	&UA2988335,
	&UA2997751,
	&UA2489281,
	&UA2489628,
	&UA2511756,
	&UA2512872,
	&UA2515670,
	&UA2533381,
	&UA2585568,
	&UA2892201,
	&UA22091,
	&UA86970,
	&UA371964,
	&UA372979,
	&UA418221,
	&UA430165,
	&UA459853,
	&UA527152,
	&UA542282,
	&UA623063,
	&UA745817,
	&UA1882023,
	&UA2133070,
	&UA2672765,
	&UA2709548,
	&UA2981261,
	&UA2994598,
	&UA699154,
	&UA2021966,
	&UA2022378,
	&UA2022457,
	&UA3019449,
	&UA43660,
	&UA156280,
	&UA1881917,
	&UA266930,
	&UA289214,
	&UA514198,
	&UA752687,
	&UA794014,
	&UA874777,
	&UA1878448,
	&UA2210651,
	&UA2242566,
	&UA2280554,
	&UA2492832,
	&UA2641447,
	&UA2681361,
	&UA2978767,
	&UA3040723,
	&UA3100063,
	&UA2262004,
	&UA2404100,
	&UA2460721,
	&UA2570299,
	&UA2601663,
	&UA2667187,
	&UA2667642,
	&UA2697056,
	&UA2700311,
	&UA2732837,
	&UA2749707,
	&UA2361891,
	&UA2716168,
	&UA2782726,
	&UA2756427,
	&UA2948284,
	&UA164248,
	&UA164249,
	&UA167186,
	&UA178030,
	&UA245344,
	&UA270143,
	&UA465111,
	&UA516599,
	&UA575482,
	&UA628899,
	&UA2010485,
	&UA2464123,
	&UA2472202,
	&UA2806301,
	&UA3083621,
	&UA715817,
	&UA1886993,
	&UA2409430,
	&UA631781,
	&UA766844,
	&UA2385547,
	&UA2430912,
	&UA2459481,
	&UA2528290,
	&UA2671728,
	&UA2712482,
	&UA2737243,
	&UA2759764,
	&UA2799832,
	&UA2937656,
	&UA3015543,
	&UA3018526,
	&UA3081572,
	&UA2509854,
	&UA2406465,
	&UA3125092,
	&UA22305,
	&UA94343,
	&UA371531,
	&UA165349,
	&UA176624,
	&UA189125,
	&UA244752,
	&UA379899,
	&UA516462,
	&UA623029,
	&UA633693,
	&UA2010223,
	&UA2802060,
	&UA3083299,
	&UA447268,
	&UA487152,
	&UA622901,
	&UA640954,
	&UA775523,
	&UA1880836,
	&UA1889876,
	&UA1900621,
	&UA400768,
	&UA432395,
	&UA447246,
	&UA447799,
	&UA450697,
	&UA459852,
	&UA459988,
	&UA493935,
	&UA539929,
	&UA573124,
	&UA594113,
	&UA644659,
	&UA708899,
	&UA872175,
	&UA1895322,
	&UA2355338,
	&UA2779062,
	&UA760202,
	&UA775139,
	&UA1965235,
	&UA1988351,
	&UA2175687,
	&UA2197304,
	&UA2354988,
	&UA2403447,
	&UA2406279,
	&UA2489704,
	&UA2483430,
	&UA2595526,
	&UA2599056,
	&UA2658118,
	&UA2983189,
	&UA191496,
	&UA204290,
	&UA299011,
	&UA321588,
	&UA322128,
	&UA322150,
	&UA446526,
	&UA865160,
	&UA880776,
	&UA1886756,
	&UA2211204,
	&UA2439353,
	&UA587047,
	&UA639979,
	&UA643631,
	&UA645787,
	&UA674723,
	&UA700305,
	&UA717124,
	&UA1941244,
	&UA2302900,
	&UA2594339,
	&UA64895,
	&UA2665806,
	&UA566808,
	&UA829883,
	&UA79885,
	&UA2569883,
	&UA1995936,
	&UA2043291,
	&UA253785,
	&UA496539,
	&UA527199,
	&UA622898,
	&UA643202,
	&UA771999,
	&UA804435,
	&UA867053,
	&UA1909348,
	&UA2264855,
	&UA2283574,
	&UA66558,
	&UA62526,
	&UA2568826,
	&UA2794219,
	&UA2875895,
	&UA88391,
	&UA89219,
	&UA90178,
	&UA99644,
	&UA628532,
	&UA2203853,
	&UA2219320,
	&UA2232612,
	&UA2242471,
	&UA2242483,
	&UA2327237,
	&UA2362466,
	&UA2443169,
	&UA2556138,
	&UA2588809,
	&UA2644016,
	&UA2658362,
	&UA2699890,
	&UA895249,
	&UA902477,
	&UA909719,
	&UA924945,
	&UA1969876,
	&UA1997214,
	&UA2170602,
	&UA2175409,
	&UA2202228,
	&UA2236398,
	&UA2260853,
	&UA2403993,
	&UA2404619,
	&UA2454710,
	&UA2841145,
	&UA241982,
	&UA502171,
	&UA760267,
	&UA803334,
	&UA1839838,
	&UA1857454,
	&UA1877074,
	&UA2749956,
	&UA2793743,
	&UA2878487,
	&UA2988334,
	&UA2403644,
	&UA2437327,
	&UA2506656,
	&UA2570283,
	&UA2595357,
	&UA2603680,
	&UA2705077,
	&UA2750019,
	&UA341074,
	&UA573585,
	&UA574179,
	&UA574951,
	&UA580032,
	&UA681042,
	&UA684336,
	&UA694751,
	&UA694893,
	&UA701834,
	&UA727406,
	&UA2302150,
	&UA2371068,
	&UA2758782,
	&UA2810820,
	&UA354574,
	&UA379106,
	&UA573975,
	&UA2020648,
	&UA2030331,
	&UA2819325,
	&UA2527176,
	&UA2578002,
	&UA2762528,
	&UA2818875,
	&UA2825768,
	&UA2726603,
	&UA2018166,
	&UA2030866,
	&UA2043689,
	&UA2458852,
	&UA2560190,
	&UA2601194,
	&UA2605889,
	&UA2677409,
	&UA2740100,
	&UA2869368,
	&UA2897112,
	&UA3033499,
	&UA3066120,
	&UA3126245,
	&UA3126787,
	&UA3130828,
	&UA2850044,
	&UA253782,
	&UA395958,
	&UA773524,
	&UA800109,
	&UA1896156,
	&UA2318306,
	&UA2720407,
	&UA569752,
	&UA1894672,
	&UA1997037,
	&UA2224479,
	&UA2408163,
	&UA2483800,
	&UA2483363,
	&UA2528783,
	&UA2530894,
	&UA2565354,
	&UA2600651,
	&UA2600654,
	&UA2664464,
	&UA2698282,
	&UA2705368,
	&UA2706729,
	&UA2728277,
	&UA2730434,
	&UA2734326,
	&UA2781951,
	&UA2858027,
	&UA2483428,
	&UA2507051,
	&UA2706804,
	&UA2796074,
	&UA2857081,
	&UA2858766,
	&UA3044324,
	&UA3088214,
	&UA3118322,
	&UA3108023,
	&UA537081,
	&UA49129,
	&UA1987471,
	&UA2739550,
	&UA461422,
	&UA500977,
	&UA555034,
	&UA834555,
	&UA1963824,
	&UA2578789,
	&UA2969707,
	&UA923961,
	&UA908809,
	&UA87812,
	&UA89078,
	&UA493404,
	&UA563658,
	&UA682795,
	&UA880217,
	&UA891802,
	&UA895844,
	&UA2374913,
	&UA152290,
	&UA535877,
	&UA628614,
	&UA632155,
	&UA638370,
	&UA1862920,
	&UA1886879,
	&UA1889540,
	&UA1893173,
	&UA1894363,
	&UA1905822,
	&UA18471,
	&UA51822,
	&UA442009,
	&UA449740,
	&UA480462,
	&UA481048,
	&UA539678,
	&UA541109,
	&UA544989,
	&UA656475,
	&UA749480,
	&UA791290,
	&UA875089,
	&UA2210891,
	&UA2260217,
	&UA2801549,
	&UA2803164,
	&UA901996,
	&UA30479,
	&UA33198,
	&UA54552,
	&UA169521,
	&UA197811,
	&UA436559,
	&UA413302,
	&UA459851,
	&UA480697,
	&UA502842,
	&UA637456,
	&UA656815,
	&UA1836511,
	&UA2262179,
	&UA2677156,
	&UA2681777,
	&UA2719800,
	&UA2742033,
	&UA2751145,
	&UA2802595,
	&UA2361744,
	&UA2387428,
	&UA478585,
	&UA511261,
	&UA704188,
	&UA708418,
	&UA730751,
	&UA843117,
	&UA1836528,
	&UA1840552,
	&UA1885843,
	&UA2140246,
	&UA2472323,
	&UA102228,
	&UA2659223,
	&UA2792924,
	&UA2935435,
	&UA2953494,
	&UA2953815,
	&UA2986375,
	&UA2989606,
	&UA2996846,
	&UA2997195,
	&UA3045439,
	&UA152331,
	&UA43633,
	&UA121775,
	&UA62797,
	&UA87016,
	&UA444130,
	&UA489939,
	&UA518271,
	&UA539623,
	&UA673795,
	&UA683452,
	&UA685544,
	&UA706459,
	&UA732650,
	&UA1869756,
	&UA1880288,
	&UA1895759,
	&UA2145025,
	&UA2812622,
	&UA2889074,
	&UA2195633,
	&UA2367343,
	&UA2421411,
	&UA2425540,
	&UA2485394,
	&UA30992,
	&UA605572,
	&UA606294,
	&UA606963,
	&UA607572,
	&UA671673,
	&UA816778,
	&UA817558,
	&UA1966536,
	&UA1976757,
	&UA2160326,
	&UA2165664,
	&UA2332603,
	&UA53446,
	&UA578601,
	&UA671137,
	&UA715085,
	&UA741481,
	&UA1904892,
	&UA1907480,
	&UA1924618,
	&UA2402795,
	&UA616617,
	&UA616623,
	&UA2034645,
	&UA589274,
	&UA744849,
	&UA821316,
	&UA898025,
	&UA2767571,
	&UA2780260,
	&UA2854352,
	&UA2869536,
	&UA2940152,
	&UA3027655,
	&UA3057167,
	&UA3102691,
	&UA3109945,
	&UA2483585,
	&UA2579928,
	&UA2597010,
	&UA2656459,
	&UA2845555,
	&UA3118634,
	&UA18523,
	&UA61402,
	&UA2164821,
	&UA151902,
	&UA1964598,
	&UA1988074,
	&UA1991975,
	&UA1996075,
	&UA2155949,
	&UA2157415,
	&UA2167006,
	&UA2167589,
	&UA2254820,
	&UA2375389,
	&UA2375916,
	&UA3021830,
	&UA3108090,
	&UA3125466,
	&UA1965518,
	&UA2293418,
	&UA2541451,
	&UA2757575,
	&UA2759982,
	&UA10056,
	&UA925694,
	&UA31532,
	&UA192613,
	&UA194596,
	&UA194944,
	&UA320353,
	&UA321605,
	&UA391771,
	&UA809453,
	&UA2140545,
	&UA2288524,
	&UA51109,
	&UA527956,
	&UA2730314,
	&UA692983,
	&UA2031777,
	&UA2752730,
	&UA2866225,
	&UA506322,
	&UA1934343,
	&UA2125810,
	&UA2156059,
	&UA2503481,
	&UA2734531,
	&UA2830731,
	&UA2850524,
	&UA2861785,
	&UA3092450,
	&UA2772414,
	&UA2946174,
	&UA2215478,
	&UA2517728,
	&UA692705,
	&UA2269638,
	&UA2759007,
	&UA201718,
	&UA324176,
	&UA394231,
	&UA394880,
	&UA539099,
	&UA550937,
	&UA552377,
	&UA650775,
	&UA747228,
	&UA827151,
	&UA1882015,
	&UA2949277,
	&UA2949464,
	&UA2558008,
	&UA2667864,
	&UA2749822,
	&UA2781884,
	&UA2867419,
	&UA2995506,
	&UA3054667,
	&UA380051,
	&UA397476,
	&UA453495,
	&UA633080,
	&UA655014,
	&UA659717,
	&UA686531,
	&UA690192,
	&UA695387,
	&UA699830,
	&UA709577,
	&UA721620,
	&UA743971,
	&UA787081,
	&UA848830,
	&UA2352935,
	&UA2835685,
	&UA1976851,
	&UA2204795,
	&UA31045,
	&UA2656255,
	&UA2134654,
	&UA2242308,
	&UA2978881,
	&UA3004046,
	&UA313944,
	&UA504632,
	&UA542017,
	&UA717585,
	&UA1886376,
	&UA188706,
	&UA2233257,
	&UA2236183,
	&UA2360773,
	&UA2367760,
	&UA33329,
	&UA2335406,
	&UA2596438,
	&UA771622,
	&UA2632539,
	&UA2670800,
	&UA2674302,
	&UA2714936,
	&UA2769184,
	&UA2769779,
	&UA2862221,
	&UA2879733,
	&UA2961649,
	&UA3013466,
	&UA3109174,
	&UA33294,
	&UA2659672,
	&UA163338,
	&UA10053,
	&UA27941,
	&UA2694602,
	&UA2694663,
	&UA2827925,
	&UA2839656,
	&UA2530969,
	&UA2556695,
	&UA2708898,
	&UA3045379,
	&UA8941,
	&UA2491940,
	&UA1995067,
	&UA475456,
	&UA869071,
	&UA879938,
	&UA1884829,
	&UA2204787,
	&UA2204927,
	&UA2366582,
	&UA1971651,
	&UA1988001,
	&UA2163257,
	&UA2362475,
	&UA2482191,
	&UA2536664,
	&UA2653265,
	&UA2746464,
	&UA2679639,
	&UA2537074,
	&UA448068,
	&UA603874,
	&UA818535,
	&UA2711516,
	&UA2770675,
	&UA2507603,
	&UA2558807,
	&UA86495,
	&UA22997,
	&UA84080,
	&UA2413679,
	&UA2838458,
	&UA2951599,
	&UA830118,
	&UA925710,
	&UA925759,
	&UA925789,
	&UA925842,
	&UA2567738,
	&UA720336,
	&UA2486323,
	&UA2601969,
	&UA2669126,
	&UA2780680,
	&UA2859488,
	&UA2876967,
	&UA2675597,
	&UA54595,
	&UA2593272,
	&UA2684561,
	&UA2727934,
	&UA2842982,
	&UA2666113,
	&UA2731642,
	&UA2986938,
	&UA608443,
	&UA869513,
	&UA1956369,
	&UA2162623,
	&UA2335367,
	&UA2445125,
	&UA2698146,
	&UA2752651,
	&UA2791824,
	&UA2229240,
	&UA2353554,
	&UA2421511,
	&UA2743658,
	&UA5040,
	&UA9918,
	&UA4975,
	&UA20837,
	&UA30856,
	&UA64025,
	&UA2716906,
	&UA2799069,
	&UA2944329,
	&UA3018787,
	&UA10060,
	&UA925717,
	&UA516901,
	&UA623045,
	&UA687328,
	&UA547903,
	&UA30654,
	&UA2265963,
	&UA2697298,
	&UA2418058,
	&UA2520561,
	&UA2868985,
	&UA2405714,
	&UA87022,
	&UA90786,
	&UA3023209,
	&UA3126604,
	&UA1953912,
	&UA1968166,
	&UA1969801,
	&UA1995977,
	&UA2006117,
	&UA2007385,
	&UA2157158,
	&UA2260714,
	&UA2260732,
	&UA2334396,
	&UA2337817,
	&UA2426591,
	&UA2456559,
	&UA2653291,
	&UA2695245,
	&UA2434292,
	&UA2559777,
	&UA2706106,
	&UA382153,
	&UA2222007,
	&UA456664,
	&UA608447,
	&UA639569,
	&UA1976566,
	&UA2656175,
	&UA2658046,
	&UA2730717,
	&UA2779591,
	&UA3047782,
	&UA3073529,
	&UA327837,
	&UA331065,
	&UA421178,
	&UA697708,
	&UA726527,
	&UA759050,
	&UA832267,
	&UA860275,
	&UA1896775,
	&UA1900713,
	&UA2142148,
	&UA2293249,
	&UA1986425,
	&UA2335132,
	&UA2482461,
	&UA2543965,
	&UA2593644,
	&UA2954445,
	&UA2999647,
	&UA2567479,
	&UA88788,
	&UA265067,
	&UA708585,
	&UA2783404,
	&UA2879929,
	&UA237090,
	&UA2733604,
	&UA2750941,
	&UA2772259,
	&UA2876982,
	&UA2970830,
	&UA3029026,
	&UA3094310,
	&UA53999,
	&UA2232091,
	&UA2203691,
	&UA2236462,
	&UA2387204,
	&UA2406905,
	&UA2486944,
	&UA430173,
	&UA438973,
	&UA442463,
	&UA459848,
	&UA699068,
	&UA726093,
	&UA2206838,
	&UA2766423,
	&UA2933036,
	&UA401783,
	&UA2591505,
	&UA2704753,
	&UA2714728,
	&UA2716669,
	&UA2771790,
	&UA2843078,
	&UA3031367,
	&UA3083044,
	&UA829293,
	&UA2486680,
	&UA562545,
	&UA1890915,
	&UA1903153,
	&UA695644,
	&UA2511663,
	&UA2672040,
	&UA2203000,
	&UA2203526,
	&UA2229358,
	&UA2361729,
	&UA2366715,
	&UA2385001,
	&UA2419384,
	&UA2421316,
	&UA2456442,
	&UA2653077,
	&UA2839003,
	&UA140556,
	&UA900369,
	&UA900787,
	&UA901344,
	&UA2027230,
	&UA2033026,
	&UA2222501,
	&UA2223155,
	&UA2818227,
	&UA2560136,
	&UA2660063,
	&UA2764760,
	&UA3071647,
	&UA24722,
	&UA2531651,
	&UA2531701,
	&UA2560294,
	&UA430254,
	&UA2467223,
	&UA2579289,
	&UA2654833,
	&UA3086054,
	&UA2952758,
	&UA3045158,
	&UA2375696,
	&UA2670109,
	&UA2737255,
	&UA2771220,
	&UA2896465,
	&UA3018608,
	&UA3028239,
	&UA3093302,
	&UA2736683,
	&UA20331,
	&UA2319534,
	&UA2388286,
	&UA2386187,
	&UA2389825,
	&UA2426507,
	&UA50260,
	&UA2376100,
	&UA2580156,
	&UA2692855,
	&UA2705206,
	&UA2729163,
	&UA2950877,
	&UA2951300,
	&UA2957980,
	&UA3086151,
	&UA3092468,
	&UA676247,
	&UA895379,
	&UA903354,
	&UA910864,
	&UA1996357,
	&UA1999890,
	&UA2202971,
	&UA2362791,
	&UA2554002,
	&UA2666346,
	&UA2667099,
	&UA2705807,
	&UA2782048,
	&UA2874772,
	&UA3020622,
	&UA87551,
	&UA396276,
	&UA437173,
	&UA2013889,
	&UA2131586,
	&UA2280779,
	&UA3083638,
	&UA2446282,
	&UA2590467,
	&UA2821493,
	&UA507453,
	&UA2587103,
	&UA2815481,
	&UA3084570,
	&UA3117355,
	&UA2700981,
	&UA2733486,
	&UA2855362,
	&UA2955530,
	&UA588622,
	&UA597128,
	&UA924296,
	&UA683149,
	&UA1937209,
	&UA2148825,
	&UA2495973,
	&UA2584779,
	&UA67816,
	&UA2713005,
	&UA2783573,
	&UA447473,
	&UA576439,
	&UA671831,
	&UA651734,
	&UA2242333,
	&UA533788,
	&UA545515,
	&UA557870,
	&UA1872528,
	&UA2242428,
	&UA2275977,
	&UA2290274,
	&UA2391681,
	&UA2472314,
	&UA2472311,
	&UA697214,
	&UA784899,
	&UA2813437,
	&UA2670466,
	&UA2736514,
	&UA2799483,
	&UA2863696,
	&UA2937988,
	&UA2964080,
	&UA3010687,
	&UA3055485,
	&UA5861,
	&UA30502,
	&UA2398575,
	&UA2698786,
	&UA89725,
	&UA89939,
	&UA447727,
	&UA523541,
	&UA529275,
	&UA545394,
	&UA2132987,
	&UA2278946,
	&UA439165,
	&UA895248,
	&UA895425,
	&UA898736,
	&UA902918,
	&UA1953462,
	&UA1953656,
	&UA2224109,
	&UA2325519,
	&UA2372028,
	&UA1886593,
	&UA23418,
	&UA308427,
	&UA894092,
	&UA2142180,
	&UA2707748,
	&UA2775337,
	&UA304867,
	&UA2853446,
	&UA2997105,
	&UA3092951,
	&UA895542,
	&UA904060,
	&UA911634,
	&UA1995589,
	&UA2002937,
	&UA2172908,
	&UA2173877,
	&UA2204766,
	&UA2260775,
	&UA2362542,
	&UA2653652,
	&UA859587,
	&UA2842550,
	&UA2952149,
	&UA2390,
	&UA603070,
	&UA87425,
	&UA87099,
	&UA1959973,
	&UA1993351,
	&UA2163638,
	&UA2375844,
	&UA2445618,
	&UA2445739,
	&UA2649330,
	&UA2600161,
	&UA2847803,
	&UA2861884,
	&UA318109,
	&UA1893571,
	&UA2727481,
	&UA2747074,
	&UA2778219,
	&UA2790333,
	&UA2950969,
	&UA2984334,
	&UA54787,
	&UA546923,
	&UA3092211,
	&UA3092254,
	&UA3107813,
	&UA517825,
	&UA552424,
	&UA817939,
	&UA566170,
	&UA2541861,
	&UA88189,
	&UA88402,
	&UA88831,
	&UA88990,
	&UA89747,
	&UA89750,
	&UA643208,
	&UA1990765,
	&UA2159349,
	&UA2239607,
	&UA2644641,
	&UA718309,
	&UA18834,
	&UA21512,
	&UA32928,
	&UA69563,
	&UA230574,
	&UA297244,
	&UA812553,
	&UA515301,
	&UA520237,
	&UA474769,
	&UA694188,
	&UA527317,
	&UA26631,
	&UA20430,
	&UA1956745,
	&UA2579408,
	&UA2242408,
	&UA704781,
	&UA2678418,
	&UA573829,
	&UA736051,
	&UA1904820,
	&UA1951226,
	&UA2465234,
	&UA2641967,
	&UA2811374,
	&UA3095514,
	&UA2698682,
	&UA1887011,
	&UA253992,
	&UA254008,
	&UA624793,
	&UA1937797,
	&UA2786294,
	&UA2164708,
	&UA662850,
	&UA756003,
	&UA2265772,
	&UA43583,
	&UA2474831,
	&UA18506,
	&UA1956191,
	&UA1958520,
	&UA2334892,
	&UA2369723,
	&UA2695750,
	&UA2411139,
	&UA3078341,
	&UA24069,
	&UA2595260,
	&UA2827573,
	&UA14824,
	&UA24613,
	&UA64869,
	&UA1895651,
	&UA1885770,
	&UA2651165,
	&UA2986533,
	&UA3075204,
	&UA2027319,
	&UA2794112,
	&UA2996105,
	&UA2657724,
	&UA2750118,
	&UA2864420,
	&UA2936480,
	&UA2384523,
	&UA2437216,
	&UA2750902,
	&UA2860424,
	&UA3086916,
	&UA2666650,
	&UA387788,
	&UA703983,
	&UA880948,
	&UA445552,
	&UA2795590,
	&UA2867511,
	&UA2891694,
	&UA3011410,
	&UA3018006,
	&UA2438138,
	&UA2955950,
	&UA2956663,
	&UA3000570,
	&UA2499515,
	&UA2836144,
	&UA2849283,
	&UA2951200,
	&UA2796230,
	&UA748903,
	&UA15620,
	&UA30348,
	&UA688656,
	&UA899590,
	&UA900165,
	&UA900512,
	&UA1964760,
	&UA1988105,
	&UA2563482,
	&UA2781827,
	&UA560376,
	&UA566064,
	&UA2257032,
	&UA2749965,
	&UA2680969,
	&UA2680970,
	&UA2680984,
	&UA2470559,
	&UA2490198,
	&UA2510864,
	&UA2653113,
	&UA2489963,
	&UA925704,
	&UA925760,
	&UA925806,
	&UA925835,
	&UA925872,
	&UA2034646,
	&UA2818622,
	&UA2472027,
	&UA628226,
	&UA2518151,
	&UA3022508,
	&UA785153,
	&UA1991937,
	&UA2189034,
	&UA2226190,
	&UA2346868,
	&UA2377660,
	&UA2828507,
	&UA19435,
	&UA58122,
	&UA2463213,
	&UA622911,
	&UA882346,
	&UA893538,
	&UA899383,
	&UA1982655,
	&UA2335159,
	&UA2372707,
	&UA2743536,
	&UA2758997,
	&UA2357985,
	&UA2405900,
	&UA2433621,
	&UA9015,
	&UA87427,
	&UA160174,
	&UA2041165,
	&UA2043153,
	&UA2205470,
	&UA659462,
	&UA712317,
	&UA792107,
	&UA2383319,
	&UA2820976,
	&UA2752527,
	&UA2961927,
	&UA3057594,
	&UA3002733,
	&UA62964,
	&UA2407225,
	&UA20697,
	&UA2427685,
	&UA2817367,
	&UA64610,
	&UA707564,
	&UA710533,
	&UA722811,
	&UA819221,
	&UA1992759,
	&UA2221963,
	&UA3100340,
	&UA2866726,
	&UA2958991,
	&UA2489702,
	&UA709438,
	&UA882833,
	&UA1991662,
	&UA2454972,
	&UA2154800,
	&UA2317126,
	&UA2361277,
	&UA2481398,
	&UA2202689,
	&UA2367706,
	&UA2214945,
	&UA2807335,
	&UA2694157,
	&UA2696168,
	&UA3099645,
	&UA2709168,
	&UA263379,
	&UA1941400,
	&UA1996759,
	&UA2000062,
	&UA662989,
	&UA1949834,
	&UA55244,
	&UA574468,
	&UA2242384,
	&UA208379,
	&UA2138993,
	&UA720797,
	&UA753417,
	&UA2848659,
	&UA305473,
	&UA2673,
	&UA2676,
	&UA434053,
	&UA492684,
	&UA496803,
	&UA925695,
	&UA17225,
	&UA2039813,
	&UA2042436,
	&UA2354809,
	&UA2549146,
	&UA451125,
	&UA2224085,
	&UA2528111,
	&UA2799263,
	&UA2955030,
	&UA2962029,
	&UA66858,
	&UA911154,
	&UA2003851,
	&UA2235459,
	&UA2403522,
	&UA48943,
	&UA2028813,
	&UA2538810,
	&UA2570106,
	&UA2584291,
	&UA18654,
	&UA20855,
	&UA2242367,
	&UA2445104,
	&UA2495301,
	&UA2696398,
	&UA520860,
	&UA756450,
	&UA3069491,
	&UA2682123,
	&UA1914010,
	&UA2032093,
	&UA2398008,
	&UA2672052,
	&UA2674906,
	&UA3013603,
	&UA62354,
	&UA350960,
	&UA648717,
	&UA814584,
	&UA1938531,
	&UA2149248,
	&UA355875,
	&UA700837,
	&UA710559,
	&UA2216518,
	&UA8882,
	&UA86998,
	&UA2783879,
	&UA2872155,
	&UA2886597,
	&UA2364306,
	&UA2408918,
	&UA629120,
	&UA693893,
	&UA708673,
	&UA724411,
	&UA3032715,
	&UA572451,
	&UA1999901,
	&UA2865940,
	&UA54794,
	&UA455943,
	&UA12950,
	&UA87058,
	&UA2197420,
	&UA2238010,
	&UA2374805,
	&UA253779,
	&UA1945895,
	&UA2687905,
	&UA2953315,
	&UA207178,
	&UA2209872,
	&UA572132,
	&UA3065665,
	&UA253784,
	&UA2578293,
	&UA28978,
	&UA730788,
	&UA728369,
	&UA2370480,
	&UA2494079,
	&UA2160617,
	&UA2863217,
	&UA2674348,
	&UA52436,
	&UA64643,
	&UA771694,
	&UA2528071,
	&UA2776557,
	&UA2680973,
	&UA2680966,
	&UA2681019,
	&UA3020974,
	&UA50254,
	&UA2413286,
	&UA2809332,
	&UA2530316,
	&UA2778554,
	&UA2863942,
	&UA2987119,
	&UA3046234,
	&UA23362,
	&UA359500,
	&UA1865301,
	&UA2264142,
	&UA2199851,
	&UA2503583,
	&UA2823869,
	&UA2781862,
	&UA647514,
	&UA731430,
	&UA1937514,
	&UA2736354,
	&UA910801,
	&UA2441632,
	&UA2733425,
	&UA2671,
	&UA44368,
	&UA33299,
	&UA2134256,
	&UA2291915,
	&UA84096,
	&UA32157,
	&UA50445,
	&UA574281,
	&UA2487457,
	&UA3021259,
	&UA26460,
	&UA2541342,
	&UA2979881,
	&UA2694480,
	&UA82,
	&UA22180,
	&UA573477,
	&UA1918325,
	&UA2695462,
	&UA23982,
	&UA49168,
	&UA253778,
	&UA710646,
	&UA713088,
	&UA26368,
	&UA493615,
	&UA771438,
	&UA1849046,
	&UA2267900,
	&UA567683,
	&UA572933,
	&UA572153,
	&UA774643,
	&UA605565,
	&UA660470,
	&UA1989526,
	&UA2259689,
	&UA2315724,
	&UA2331327,
	&UA2589229,
	&UA920475,
	&UA1945306,
	&UA3058851,
	&UA3109281,
	&UA40328,
	&UA40330,
	&UA55529,
	&UA57036,
	&UA77204,
	&UA2127872,
	&UA2727977,
	&UA2733632,
	&UA2733906,
	&UA2865701,
	&UA2997895,
	&UA2999097,
	&UA21245,
	&UA3058501,
	&UA3126048,
	&UA3060187,
	&UA3126539,
	&UA21500,
	&UA42701,
	&UA242910,
	&UA783083,
	&UA2587466,
	&UA769311,
	&UA2444668,
	&UA192101,
	&UA197369,
	&UA198151,
	&UA325720,
	&UA419893,
	&UA2292847,
	&UA502773,
	&UA767756,
	&UA2814323,
	&UA2654303,
	&UA2857247,
	&UA2857794,
	&UA3099163,
	&UA253923,
	&UA50473,
	&UA2503892,
	&UA2578075,
	&UA21395,
	&UA26373,
	&UA425396,
	&UA649449,
	&UA2150235,
	&UA2299815,
	&UA2549095,
	&UA3058255,
	&UA54839,
	&UA633942,
	&UA693796,
	&UA912477,
	&UA2408904,
	&UA2850049,
	&UA2908,
	&UA2273181,
	&UA2160902,
	&UA2463766,
	&UA672769,
	&UA697191,
	&UA2149989,
	&UA427427,
	&UA601648,
	&UA603579,
	&UA615685,
	&UA708530,
	&UA773416,
	&UA1903240,
	&UA2208964,
	&UA2949857,
	&UA32872,
	&UA43584,
	&UA84582,
	&UA347782,
	&UA351533,
	&UA352603,
	&UA366220,
	&UA643873,
	&UA366481,
	&UA621916,
	&UA569313,
	&UA2213539,
	&UA2554588,
	&UA12326,
	&UA44693,
	&UA302908,
	&UA533311,
	&UA544718,
	&UA906136,
	&UA908649,
	&UA1863604,
	&UA2801878,
	&UA1895513,
	&UA2023365,
	&UA2866876,
	&UA2890778,
	&UA64346,
	&UA925815,
	&UA925824,
	&UA2853827,
	&UA2653557,
	&UA2790432,
	&UA44659,
	&UA842475,
	&UA52823,
	&UA2802888,
	&UA599801,
	&UA817081,
	&UA889545,
	&UA1985745,
	&UA760961,
	&UA2652941,
	&UA2695289,
	&UA3058188,
	&UA87559,
	&UA24518,
	&UA66648,
	&UA89712,
	&UA452165,
	&UA572308,
	&UA731755,
	&UA1937637,
	&UA2504633,
	&UA2681784,
	&UA3069575,
	&UA243535,
	&UA2301346,
	&UA515867,
	&UA537717,
	&UA634203,
	&UA728552,
	&UA745121,
	&UA806730,
	&UA1903588,
	&UA2554802,
	&UA3077953,
	&UA67715,
	&UA755733,
	&UA1888888,
	&UA2301370,
	&UA2788739,
	&UA3040775,
	&UA3092347,
	&UA23656,
	&UA27521,
	&UA49461,
	&UA63681,
	&UA87415,
	&UA63729,
	&UA85783,
	&UA589882,
	&UA1942213,
	&UA2321430,
	&UA2409371,
	&UA2520466,
	&UA2762869,
	&UA2835458,
	&UA2977059,
	&UA2977068,
	&UA3118720,
	&UA23926,
	&UA23927,
	&UA23992,
	&UA524327,
	&UA2131312,
	&UA621893,
	&UA622259,
	&UA622287,
	&UA622326,
	&UA1933936,
	&UA2242470,
	&UA2394248,
	&UA2552919,
	&UA2589255,
	&UA2685751,
	&UA2695682,
	&UA2759158,
	&UA2859317,
	&UA441003,
	&UA652743,
	&UA855760,
	&UA452483,
	&UA462864,
	&UA527173,
	&UA772840,
	&UA925082,
	&UA2028810,
	&UA1905306,
	&UA2242271,
	&UA2934232,
	&UA3126317,
	&UA89723,
	&UA450612,
	&UA2346795,
	&UA2814202,
	&UA2932906,
	&UA3057231,
	&UA3070919,
	&UA24000,
	&UA61821,
	&UA63006,
	&UA464282,
	&UA464340,
	&UA466657,
	&UA579372,
	&UA1938228,
	&UA605785,
	&UA615420,
	&UA622019,
	&UA771436,
	&UA868943,
	&UA2216196,
	&UA2216328,
	&UA2223512,
	&UA2339541,
	&UA2277259,
	&UA2312707,
	&UA2786524,
	&UA2808807,
	&UA3057372,
	&UA3121077,
	&UA2981,
	&UA71766,
	&UA925891,
	&UA57111,
	&UA49372,
	&UA63000,
	&UA250952,
	&UA792020,
	&UA2029481,
	&UA2148242,
	&UA2148378,
	&UA2527792,
	&UA2574727,
	&UA2774685,
	&UA2795314,
	&UA2983614,
	&UA3121023,
	&UA23895,
	&UA444225,
	&UA529194,
	&UA715295,
	&UA1961577,
	&UA2301662,
	&UA2370560,
	&UA2540991,
	&UA2685053,
	&UA2847648,
	&UA2853327,
	&UA351731,
	&UA355516,
	&UA453145,
	&UA582681,
	&UA1947819,
	&UA2310422,
	&UA615800,
	&UA767147,
	&UA1937731,
	&UA2287046,
	&UA2302538,
	&UA22361,
	&UA23246,
	&UA925752,
	&UA2439428,
	&UA87103,
	&UA654878,
	&UA1938139,
	&UA692374,
	&UA1901255,
	&UA86578,
	&UA21799,
	&UA22940,
	&UA22952,
	&UA57265,
	&UA55851,
	&UA68854,
	&UA253933,
	&UA427437,
	&UA427453,
	&UA601623,
	&UA615695,
	&UA622719,
	&UA623015,
	&UA806581,
	&UA825896,
	&UA575491,
	&UA621894,
	&UA621942,
	&UA622033,
	&UA622239,
	&UA747518,
	&UA2167438,
	&UA2032341,
	&UA2518656,
	&UA21190,
	&UA24402,
	&UA79879,
	&UA253939,
	&UA539989,
	&UA583795,
	&UA756100,
	&UA1989910,
	&UA599781,
	&UA2154308,
	&UA1998153,
	&UA2194422,
	&UA2352045,
	&UA2824613,
	&UA87672,
	&UA465701,
	&UA2147417,
	&UA10258,
	&UA40009,
	&UA16532,
	&UA21797,
	&UA535173,
	&UA702111,
	&UA2815750,
	&UA43768,
	&UA46771,
	&UA68808,
	&UA85786,
	&UA421651,
	&UA601605,
	&UA18170,
	&UA61966,
	&UA71205,
	&UA212873,
	&UA216166,
	&UA331858,
	&UA636092,
	&UA650988,
	&UA412450,
	&UA601684,
	&UA604243,
	&UA739794,
	&UA427436,
	&UA564726,
	&UA567453,
	&UA572688,
	&UA574756,
	&UA695307,
	&UA771223,
	&UA2316312,
	&UA2607487,
	&UA2977061,
	&UA26171,
	&UA33093,
	&UA68346,
	&UA69901,
	&UA88944,
	&UA347738,
	&UA366197,
	&UA584696,
	&UA419363,
	&UA436853,
	&UA1917567,
	&UA902950,
	&UA1846386,
	&UA2310966,
	&UA8897,
	&UA13659,
	&UA23354,
	&UA24068,
	&UA28434,
	&UA39915,
	&UA45103,
	&UA46697,
	&UA88214,
	&UA371268,
	&UA1890269,
	&UA1938834,
	&UA2242087,
	&UA2242096,
	&UA2242100,
	&UA3084418,
	&UA3084419,
	&UA3084420,
	&UA3084421,
	&UA3084422,
	&UA21425,
	&UA32742,
	&UA33129,
	&UA253981,
	&UA427276,
	&UA621865,
	&UA621952,
	&UA544529,
	&UA925761,
	&UA2242088,
	&UA2242103,
	&UA2242097,
	&UA11266,
	&UA21283,
	&UA22900,
	&UA61635,
	&UA68195,
	&UA69044,
	&UA253786,
	&UA621909,
	&UA2033322,
	&UA643925,
	&UA2031571,
	&UA21749,
	&UA33218,
	&UA59896,
	&UA62539,
	&UA85784,
	&UA85785,
	&UA253935,
	&UA458452,
	&UA605074,
	&UA605481,
	&UA650734,
	&UA622191,
	&UA800004,
	&UA925705,
	&UA2977065,
	&UA21522,
	&UA87849,
	&UA50044,
	&UA87461,
	&UA56507,
	&UA69157,
	&UA370516,
	&UA569754,
	&UA876086,
	&UA637453,
	&UA2241453,
	&UA2241460,
	&UA2317415,
	&UA2977056,
	&UA54618,
	&UA21738,
	&UA22420,
	&UA31977,
	&UA44373,
	&UA65022,
	&UA69246,
	&UA87034,
	&UA87898,
	&UA253774,
	&UA253823,
	&UA253964,
	&UA566088,
	&UA2007459,
	&UA2242389,
	&UA2607479,
	&UA2628807,
	&UA2631328,
	&UA2681006,
	&UA2681015,
	&UA20322,
	&UA32752,
	&UA68374,
	&UA69061,
	&UA70539,
	&UA87382,
	&UA88581,
	&UA332115,
	&UA653883,
	&UA1904776,
	&UA2241429,
	&UA2308515,
	&UA2617198,
	&UA2622001,
	&UA2623963,
	&UA2624434,
	&UA3097337,
	&UA14361,
	&UA20987,
	&UA26884,
	&UA29539,
	&UA30898,
	&UA38638,
	&UA55055,
	&UA56658,
	&UA57754,
	&UA60844,
	&UA67691,
	&UA67890,
	&UA67915,
	&UA69656,
	&UA76262,
	&UA82680,
	&UA85200,
	&UA86376,
	&UA86508,
	&UA86650,
	&UA135944,
	&UA253949,
	&UA257876,
	&UA366205,
	&UA370255,
	&UA606902,
	&UA876256,
	&UA1934628,
	&UA1977898,
	&UA2314384,
	&UA2586661,
	&UA2607481,
	&UA2607486,
	&UA2611320,
	&UA2615149,
	&UA2616467,
	&UA2618881,
	&UA2619660,
	&UA2619892,
	&UA2620378,
	&UA2620601,
	&UA2620873,
	&UA2621563,
	&UA2621817,
	&UA2622289,
	&UA2622383,
	&UA2622604,
	&UA2622914,
	&UA2623471,
	&UA2624304,
	&UA2626344,
	&UA2626601,
	&UA2626668,
	&UA2627137,
	&UA2627145,
	&UA2629566,
	&UA2629778,
	&UA2633579,
	&UA2635127,
	&UA2635912,
	&UA2636208,
	&UA2977062,
	&UA2977070,
	&UA79916,
	&UA79906,
	&UA253777,
	&UA253776,
	&UA253780,
	&UA253781,
	&UA253787,
	&UA253788,
	&UA253789,
	&UA622516,
	&UA622526,
	NULL};
const Segmentdata DBlackBerry2[] = {
	{SG61268, &UA60291},
	{SG61269, &UA2232879},
	{SG61278, &UA40379},
	{SG61280, &UA75075},
	{SG61315, &UA63216},
	{SG61325, &UA429157},
	{SG61340, &UA625199},
	{SG61341, &UA2184856},
	{SG61342, &UA885420},
	{SG61345, &UA49560},
	{SG61355, &UA230174},
	{SG61726, &UA2362787},
	{SG61805, &UA2196727},
	{SG61828, &UA349878},
	{SG61862, &UA627034},
	{SG61954, &UA380967},
	{SG62108, &UA48821},
	{SG62109, &UA623533},
	{SG62422, &UA382090},
	{SG62689, &UA687945},
	{SG62794, &UA2361062},
	{SG62853, &UA350391},
	{SG63233, &UA924173},
	{SG63242, &UA349364},
	{SG63336, &UA1996307},
	{SG63432, &UA49952},
	{SG63747, &UA49839},
	{SG63925, &UA72779},
	{SG63967, &UA429114},
	{SG64182, &UA2368546},
	{SG64383, &UA58528},
	{SG64496, &UA867936},
	{SG64533, &UA2670838},
	{SG64701, &UA79310},
	{SG65285, &UA429278},
	{SG65393, &UA2460051},
	{SG67074, &UA429374},
	{SG67273, &UA3098142},
	{SG67328, &UA49797},
	{SG67396, &UA46728},
	{SG67459, &UA60942},
	{SG67463, &UA428993},
	{NULL}};
const Segmentdata DChrome0_9[] = {
	{SG61829, &UA2495397},
	{SG61830, &UA2496800},
	{SG62169, &UA2159030},
	{SG62323, &UA182835},
	{SG62324, &UA301306},
	{SG62325, &UA525282},
	{SG62327, &UA2288553},
	{SG62329, &UA2423767},
	{SG62331, &UA2480467},
	{SG62333, &UA2580020},
	{SG62335, &UA2673185},
	{SG62337, &UA2678042},
	{SG62338, &UA2708354},
	{SG62341, &UA2708666},
	{SG62343, &UA2709337},
	{SG62345, &UA2709526},
	{SG62346, &UA2735051},
	{SG62413, &UA91069},
	{SG62414, &UA133678},
	{SG62415, &UA137809},
	{SG62416, &UA252691},
	{SG62417, &UA634410},
	{SG62418, &UA675202},
	{SG62419, &UA824059},
	{SG62420, &UA2316217},
	{SG62421, &UA2362173},
	{SG62707, &UA12930},
	{SG62708, &UA113190},
	{SG62709, &UA130501},
	{SG62710, &UA2259889},
	{SG62711, &UA2332393},
	{SG62712, &UA2786641},
	{SG62713, &UA2801409},
	{SG62826, &UA159877},
	{SG62827, &UA294746},
	{SG62828, &UA836985},
	{SG62829, &UA2375315},
	{SG62830, &UA2384955},
	{SG62831, &UA171512},
	{SG62832, &UA172087},
	{SG62833, &UA491557},
	{SG62834, &UA2276782},
	{SG62835, &UA2323161},
	{SG62836, &UA2385099},
	{SG62837, &UA2673264},
	{SG62838, &UA2673341},
	{SG62839, &UA2678279},
	{SG62840, &UA2680941},
	{SG62841, &UA2680942},
	{SG62842, &UA2708865},
	{SG62843, &UA2709771},
	{SG62844, &UA2710999},
	{SG62845, &UA2735095},
	{SG62846, &UA2736283},
	{SG62847, &UA2806189},
	{SG62848, &UA2860251},
	{SG62849, &UA2865518},
	{SG62850, &UA2865585},
	{SG62851, &UA2865666},
	{SG62852, &UA3119784},
	{SG62913, &UA117949},
	{SG62915, &UA2125535},
	{SG62917, &UA2392920},
	{SG62919, &UA2457870},
	{SG62921, &UA2503314},
	{SG62922, &UA2812848},
	{SG62934, &UA105255},
	{SG62935, &UA135692},
	{SG62936, &UA461980},
	{SG62937, &UA628176},
	{SG62938, &UA784888},
	{SG62939, &UA2740248},
	{SG62940, &UA95127},
	{SG62941, &UA124171},
	{SG62942, &UA258117},
	{SG62943, &UA712733},
	{SG62944, &UA896817},
	{SG62945, &UA2323179},
	{SG62946, &UA118071},
	{SG62947, &UA266563},
	{SG62948, &UA715726},
	{SG62949, &UA829313},
	{SG62950, &UA2455889},
	{SG62951, &UA2785747},
	{SG62952, &UA149271},
	{SG62953, &UA193635},
	{SG62954, &UA196247},
	{SG62955, &UA291932},
	{SG62956, &UA546467},
	{SG62957, &UA627461},
	{SG62958, &UA666630},
	{SG62959, &UA709645},
	{SG62960, &UA849046},
	{SG62961, &UA2289711},
	{SG62962, &UA2421187},
	{SG62970, &UA136461},
	{SG62971, &UA136829},
	{SG62972, &UA300911},
	{SG62973, &UA395655},
	{SG62975, &UA885822},
	{SG62976, &UA2314555},
	{SG62977, &UA2594555},
	{SG63082, &UA153895},
	{SG63083, &UA168649},
	{SG63084, &UA294760},
	{SG63085, &UA2272590},
	{SG63086, &UA2360478},
	{SG63087, &UA2366729},
	{SG63088, &UA2402816},
	{SG63089, &UA2413666},
	{SG63090, &UA2483159},
	{SG63091, &UA2651853},
	{SG63092, &UA2680943},
	{SG63093, &UA2708265},
	{SG63094, &UA2708345},
	{SG63095, &UA2709332},
	{SG63096, &UA2709447},
	{SG63097, &UA2710985},
	{SG63098, &UA2735202},
	{SG63099, &UA2735971},
	{SG63100, &UA2859969},
	{SG63101, &UA2860382},
	{SG63122, &UA149564},
	{SG63123, &UA205628},
	{SG63124, &UA283365},
	{SG63125, &UA291993},
	{SG63126, &UA432405},
	{SG63127, &UA646519},
	{SG63128, &UA1886168},
	{SG63129, &UA2287533},
	{SG63130, &UA2503458},
	{SG63131, &UA2674212},
	{SG63132, &UA2675828},
	{SG63133, &UA2678290},
	{SG63134, &UA2708511},
	{SG63135, &UA2709455},
	{SG63136, &UA2711118},
	{SG63137, &UA2735133},
	{SG63138, &UA2735698},
	{SG63140, &UA2811158},
	{SG63518, &UA97809},
	{SG63519, &UA111090},
	{SG63520, &UA122529},
	{SG63521, &UA259516},
	{SG63522, &UA382471},
	{SG63523, &UA416447},
	{SG63524, &UA857298},
	{SG63525, &UA1867460},
	{SG63526, &UA1886119},
	{SG63527, &UA2131467},
	{SG63528, &UA2139314},
	{SG63529, &UA2212028},
	{SG63530, &UA2278325},
	{SG63531, &UA2316793},
	{SG63532, &UA2332897},
	{SG63533, &UA2337372},
	{SG63534, &UA2441229},
	{SG63535, &UA2775357},
	{SG63536, &UA2815321},
	{SG63555, &UA99082},
	{SG63556, &UA259524},
	{SG63557, &UA1874668},
	{SG63558, &UA2293966},
	{SG63559, &UA2741856},
	{SG63766, &UA92445},
	{SG63768, &UA430162},
	{SG63771, &UA539928},
	{SG63773, &UA667467},
	{SG63775, &UA1963867},
	{SG63777, &UA2176721},
	{SG63779, &UA2705801},
	{SG63780, &UA93088},
	{SG63781, &UA765406},
	{SG63782, &UA766260},
	{SG63783, &UA1884554},
	{SG63784, &UA2333332},
	{SG63982, &UA298751},
	{SG63983, &UA451522},
	{SG64016, &UA92878},
	{SG64017, &UA2181594},
	{SG64018, &UA2183528},
	{SG64019, &UA2350520},
	{SG64112, &UA186134},
	{SG64113, &UA393013},
	{SG64138, &UA103329},
	{SG64140, &UA124918},
	{SG64142, &UA602209},
	{SG64143, &UA93305},
	{SG64173, &UA95405},
	{SG64183, &UA96718},
	{SG64184, &UA167372},
	{SG64185, &UA204513},
	{SG64186, &UA272945},
	{SG64187, &UA285779},
	{SG64188, &UA413444},
	{SG64189, &UA428761},
	{SG64190, &UA525941},
	{SG64191, &UA893330},
	{SG64192, &UA2720732},
	{SG64407, &UA163141},
	{SG64408, &UA163149},
	{SG64409, &UA164661},
	{SG64410, &UA176885},
	{SG64411, &UA187068},
	{SG64412, &UA199360},
	{SG64413, &UA199862},
	{SG64414, &UA443951},
	{SG64415, &UA684159},
	{SG64416, &UA763156},
	{SG64417, &UA882303},
	{SG64418, &UA2673263},
	{SG64419, &UA2673773},
	{SG64420, &UA2674436},
	{SG64421, &UA2678789},
	{SG64422, &UA2680938},
	{SG64423, &UA2735271},
	{SG64424, &UA2860198},
	{SG64487, &UA99000},
	{SG64488, &UA193880},
	{SG64489, &UA249848},
	{SG64490, &UA589330},
	{SG64491, &UA1840978},
	{SG64492, &UA1883927},
	{SG64493, &UA2334506},
	{SG64494, &UA2644046},
	{SG64495, &UA2775662},
	{SG64497, &UA162364},
	{SG64498, &UA434497},
	{SG64499, &UA507293},
	{SG64539, &UA130331},
	{SG64540, &UA136561},
	{SG64541, &UA137972},
	{SG64542, &UA140716},
	{SG64543, &UA166625},
	{SG64544, &UA2324713},
	{SG64695, &UA10338},
	{SG64696, &UA173281},
	{SG64697, &UA315969},
	{SG64698, &UA494419},
	{SG64699, &UA753120},
	{SG64700, &UA2144315},
	{SG64702, &UA3087990},
	{SG64703, &UA173140},
	{SG64704, &UA3080567},
	{SG64705, &UA149275},
	{SG64706, &UA195592},
	{SG64707, &UA378476},
	{SG64719, &UA159755},
	{SG64721, &UA168658},
	{SG64723, &UA185182},
	{SG64725, &UA258239},
	{SG64727, &UA437226},
	{SG64729, &UA509543},
	{SG64730, &UA767765},
	{SG64731, &UA880228},
	{SG64732, &UA2305840},
	{SG64733, &UA3117602},
	{SG64763, &UA137332},
	{SG64764, &UA254585},
	{SG64765, &UA562508},
	{SG64766, &UA720780},
	{SG64767, &UA882912},
	{SG64768, &UA172582},
	{SG64769, &UA172639},
	{SG64770, &UA172912},
	{SG64771, &UA199732},
	{SG64772, &UA295360},
	{SG64773, &UA303966},
	{SG64774, &UA387148},
	{SG64775, &UA672271},
	{SG64776, &UA820491},
	{SG64780, &UA159879},
	{SG64782, &UA779153},
	{SG64785, &UA110713},
	{SG64814, &UA496204},
	{SG64815, &UA150367},
	{SG64816, &UA150796},
	{SG64817, &UA843077},
	{SG64818, &UA1864092},
	{SG64819, &UA2127618},
	{SG64820, &UA2131433},
	{SG64821, &UA2139375},
	{SG64887, &UA112808},
	{SG64888, &UA121454},
	{SG64889, &UA136076},
	{SG64890, &UA146646},
	{SG64891, &UA268815},
	{SG64892, &UA470320},
	{SG64893, &UA2128569},
	{SG64894, &UA2161232},
	{SG64895, &UA2394955},
	{SG64900, &UA110168},
	{SG64901, &UA260685},
	{SG64902, &UA290567},
	{SG64903, &UA491468},
	{SG64904, &UA727947},
	{SG64905, &UA826721},
	{SG64906, &UA862433},
	{SG64907, &UA2258343},
	{SG64908, &UA2642440},
	{SG64909, &UA172314},
	{SG64910, &UA495177},
	{SG64911, &UA735651},
	{SG64912, &UA818854},
	{SG64942, &UA108224},
	{SG64944, &UA120981},
	{SG64946, &UA142362},
	{SG64948, &UA710163},
	{SG64949, &UA736433},
	{SG64950, &UA2139524},
	{SG64951, &UA2164258},
	{SG64952, &UA2212684},
	{SG64953, &UA2375122},
	{SG64978, &UA2299912},
	{SG64992, &UA103761},
	{SG64993, &UA140114},
	{SG64994, &UA899171},
	{SG64995, &UA2522106},
	{SG64996, &UA2810767},
	{SG64997, &UA136467},
	{SG64998, &UA136459},
	{SG64999, &UA167083},
	{SG65000, &UA184296},
	{SG65001, &UA202899},
	{SG65002, &UA277604},
	{SG65003, &UA478120},
	{SG65004, &UA643300},
	{SG65005, &UA689958},
	{SG65006, &UA1892703},
	{SG65007, &UA2332262},
	{SG65008, &UA2301223},
	{SG65009, &UA2301382},
	{SG65042, &UA100058},
	{SG65043, &UA719507},
	{SG65044, &UA829427},
	{SG65045, &UA2131415},
	{SG65046, &UA2540801},
	{SG65047, &UA106500},
	{SG65048, &UA539051},
	{SG65049, &UA539710},
	{SG65050, &UA661660},
	{SG65051, &UA760547},
	{SG65052, &UA2144088},
	{SG65053, &UA2164465},
	{SG65054, &UA2805160},
	{SG65077, &UA2151839},
	{SG65109, &UA1949978},
	{SG65110, &UA1990356},
	{SG65123, &UA1879517},
	{SG65127, &UA1903521},
	{SG65185, &UA149278},
	{SG65186, &UA167875},
	{SG65187, &UA177293},
	{SG65188, &UA200707},
	{SG65189, &UA291275},
	{SG65190, &UA291693},
	{SG65191, &UA291945},
	{SG65192, &UA373128},
	{SG65193, &UA490052},
	{SG65194, &UA161900},
	{SG65195, &UA168222},
	{SG65196, &UA300918},
	{SG65197, &UA302549},
	{SG65198, &UA303235},
	{SG65199, &UA778781},
	{SG65200, &UA872186},
	{SG65201, &UA1854969},
	{SG65202, &UA2258236},
	{SG65203, &UA139770},
	{SG65204, &UA372434},
	{SG65315, &UA131975},
	{SG65317, &UA599493},
	{SG65319, &UA639951},
	{SG65324, &UA2411371},
	{SG65349, &UA149375},
	{SG65350, &UA408129},
	{SG65351, &UA408934},
	{SG65363, &UA1889335},
	{SG65364, &UA1910699},
	{SG65365, &UA2139886},
	{SG65366, &UA2259824},
	{SG65399, &UA173283},
	{SG65400, &UA301347},
	{SG65401, &UA305317},
	{SG65402, &UA307253},
	{SG65403, &UA528162},
	{SG65404, &UA667475},
	{SG65405, &UA787548},
	{SG65406, &UA1863809},
	{SG65466, &UA152514},
	{SG65467, &UA166066},
	{SG65468, &UA176010},
	{SG65469, &UA224424},
	{SG65470, &UA260050},
	{SG65471, &UA265363},
	{SG65472, &UA459868},
	{SG65473, &UA831208},
	{SG65474, &UA859557},
	{SG65506, &UA2371724},
	{SG65556, &UA171875},
	{SG65593, &UA263971},
	{SG65600, &UA383932},
	{SG65601, &UA471417},
	{SG65602, &UA652975},
	{SG65603, &UA705354},
	{SG65604, &UA1839810},
	{SG65605, &UA1840524},
	{SG65625, &UA150011},
	{SG65626, &UA292084},
	{SG65627, &UA385562},
	{SG65628, &UA431686},
	{SG65629, &UA480464},
	{SG65630, &UA483966},
	{SG65631, &UA484279},
	{SG65632, &UA2375732},
	{SG65672, &UA160497},
	{SG65680, &UA172168},
	{SG65681, &UA408329},
	{SG65682, &UA717241},
	{SG65704, &UA154129},
	{SG65721, &UA174016},
	{SG65725, &UA846491},
	{SG65752, &UA171378},
	{SG65753, &UA1866600},
	{SG65754, &UA72080},
	{SG65755, &UA72185},
	{SG65756, &UA72216},
	{SG65758, &UA72423},
	{SG65760, &UA72585},
	{SG65762, &UA72668},
	{SG65763, &UA72774},
	{SG65765, &UA72896},
	{SG65766, &UA73061},
	{SG65768, &UA73064},
	{SG65770, &UA73088},
	{SG65772, &UA73577},
	{SG65774, &UA73656},
	{SG65776, &UA74201},
	{SG65778, &UA74334},
	{SG65780, &UA74841},
	{SG65783, &UA77607},
	{SG65784, &UA196743},
	{SG65786, &UA236444},
	{SG65788, &UA247891},
	{SG65790, &UA250307},
	{SG65792, &UA316457},
	{SG65794, &UA318568},
	{SG65796, &UA329621},
	{SG65798, &UA331541},
	{SG65800, &UA331897},
	{SG65802, &UA395718},
	{SG65804, &UA550174},
	{SG65806, &UA563342},
	{SG65809, &UA574336},
	{SG65811, &UA574834},
	{SG65813, &UA629983},
	{SG65815, &UA737777},
	{SG65817, &UA766054},
	{SG65818, &UA821211},
	{SG65819, &UA829144},
	{SG65820, &UA858549},
	{SG65821, &UA875867},
	{SG65822, &UA1887485},
	{SG65823, &UA1905464},
	{SG65824, &UA2141908},
	{SG65829, &UA95751},
	{SG65877, &UA788987},
	{SG65879, &UA171380},
	{SG65904, &UA2870576},
	{SG65905, &UA92418},
	{SG65906, &UA2364026},
	{SG65915, &UA408511},
	{SG65916, &UA775638},
	{SG65917, &UA2144818},
	{SG65918, &UA2308582},
	{SG65919, &UA2542028},
	{SG65921, &UA162934},
	{SG65922, &UA498711},
	{SG65923, &UA148342},
	{SG65924, &UA495816},
	{SG65934, &UA2364170},
	{SG65954, &UA174673},
	{SG66043, &UA509520},
	{SG66044, &UA898097},
	{SG66045, &UA2166264},
	{SG66053, &UA232937},
	{SG66054, &UA344207},
	{SG66055, &UA810226},
	{SG66056, &UA883640},
	{SG66057, &UA899117},
	{SG66058, &UA2382544},
	{SG66132, &UA150891},
	{SG66133, &UA151074},
	{SG66160, &UA160798},
	{SG66162, &UA168310},
	{SG66178, &UA119183},
	{SG66180, &UA2267385},
	{SG66416, &UA795200},
	{SG66417, &UA395937},
	{SG66424, &UA117403},
	{SG66544, &UA93455},
	{SG66545, &UA165765},
	{SG66626, &UA138530},
	{SG66627, &UA810057},
	{SG66630, &UA236705},
	{SG66631, &UA344659},
	{SG66632, &UA719727},
	{SG66633, &UA808477},
	{SG66634, &UA878697},
	{SG66635, &UA2348654},
	{SG66639, &UA233536},
	{SG66640, &UA299938},
	{SG66641, &UA312510},
	{SG66642, &UA914496},
	{SG66653, &UA160343},
	{SG66763, &UA152413},
	{SG66769, &UA2266553},
	{SG66773, &UA185274},
	{SG66774, &UA436040},
	{SG66777, &UA299078},
	{SG66780, &UA396930},
	{SG66781, &UA667384},
	{SG66782, &UA895981},
	{SG66783, &UA2349843},
	{SG66784, &UA2819902},
	{SG66787, &UA2130985},
	{SG66802, &UA638657},
	{SG66825, &UA285904},
	{SG66830, &UA741061},
	{SG66853, &UA900333},
	{SG66854, &UA914800},
	{SG66855, &UA862554},
	{SG66860, &UA344871},
	{SG66861, &UA450745},
	{SG66862, &UA835135},
	{SG66863, &UA878325},
	{SG66864, &UA2420881},
	{SG66877, &UA510134},
	{SG66878, &UA863512},
	{SG66884, &UA175462},
	{SG66890, &UA76707},
	{SG66891, &UA78521},
	{SG66892, &UA328984},
	{SG66893, &UA397348},
	{SG66894, &UA702499},
	{SG66895, &UA721815},
	{SG66896, &UA737071},
	{SG66897, &UA1909351},
	{SG66898, &UA2296400},
	{SG66905, &UA2590151},
	{SG66916, &UA2475375},
	{SG66917, &UA2545329},
	{SG66930, &UA200521},
	{SG66931, &UA647824},
	{SG66932, &UA1868443},
	{SG66933, &UA165962},
	{SG66934, &UA268367},
	{SG66935, &UA664749},
	{SG66936, &UA744515},
	{SG66937, &UA860079},
	{SG66938, &UA896233},
	{SG66939, &UA2267597},
	{SG66942, &UA290135},
	{SG66943, &UA294083},
	{SG66944, &UA669247},
	{SG66960, &UA174032},
	{SG66965, &UA790574},
	{SG66972, &UA154110},
	{SG66977, &UA676601},
	{SG66978, &UA882476},
	{SG66987, &UA2465949},
	{SG66988, &UA2657996},
	{SG66989, &UA2844145},
	{SG66993, &UA163558},
	{SG67013, &UA155263},
	{SG67018, &UA219956},
	{SG67019, &UA914813},
	{SG67027, &UA264800},
	{SG67051, &UA76184},
	{SG67052, &UA78161},
	{SG67053, &UA629672},
	{SG67054, &UA697153},
	{SG67055, &UA2145357},
	{SG67056, &UA2149979},
	{SG67067, &UA891434},
	{SG67070, &UA2208429},
	{SG67077, &UA222785},
	{SG67078, &UA1966757},
	{SG67079, &UA2640933},
	{SG67080, &UA644712},
	{SG67083, &UA670278},
	{SG67084, &UA750009},
	{SG67085, &UA2268610},
	{SG67091, &UA875379},
	{SG67130, &UA286819},
	{SG67131, &UA1852018},
	{SG67181, &UA753509},
	{SG67192, &UA670142},
	{SG67193, &UA680891},
	{SG67196, &UA824466},
	{SG67197, &UA680782},
	{SG67203, &UA1962384},
	{SG67204, &UA562538},
	{SG67205, &UA652566},
	{SG67206, &UA796664},
	{SG67207, &UA1961439},
	{SG67208, &UA2651914},
	{SG67210, &UA133252},
	{SG67211, &UA438971},
	{SG67228, &UA2211478},
	{SG67229, &UA2392387},
	{SG67235, &UA307863},
	{SG67236, &UA550018},
	{SG67237, &UA300762},
	{SG67238, &UA505821},
	{SG67277, &UA524519},
	{SG67282, &UA882358},
	{SG67283, &UA896232},
	{SG67284, &UA2258231},
	{SG67296, &UA275297},
	{SG67301, &UA823509},
	{SG67302, &UA325496},
	{SG67303, &UA835973},
	{SG67304, &UA914901},
	{SG67323, &UA715634},
	{SG67326, &UA153076},
	{SG67357, &UA859160},
	{SG67358, &UA2127695},
	{SG67378, &UA522040},
	{SG67384, &UA153616},
	{SG67390, &UA2408782},
	{SG67424, &UA384963},
	{SG67428, &UA160412},
	{SG67429, &UA750250},
	{SG67430, &UA312493},
	{SG67431, &UA463907},
	{SG67432, &UA2263726},
	{SG67443, &UA1869161},
	{SG67454, &UA304113},
	{SG67460, &UA2275964},
	{SG67464, &UA463435},
	{SG67465, &UA798771},
	{SG67474, &UA2685397},
	{SG67486, &UA849161},
	{SG67487, &UA2260397},
	{SG67490, &UA412753},
	{SG67491, &UA858457},
	{SG67492, &UA2977980},
	{SG67493, &UA869732},
	{SG67494, &UA360234},
	{SG67495, &UA361735},
	{SG67497, &UA399569},
	{SG67520, &UA852046},
	{SG67535, &UA2516922},
	{SG67537, &UA2983680},
	{SG67541, &UA2789578},
	{SG67551, &UA1852853},
	{SG67574, &UA676608},
	{SG67587, &UA512719},
	{SG67647, &UA806175},
	{SG67669, &UA874164},
	{SG67685, &UA372277},
	{SG67691, &UA1842711},
	{SG67707, &UA2136816},
	{SG67710, &UA2258284},
	{SG67712, &UA2317538},
	{SG67715, &UA157160},
	{SG67716, &UA2260242},
	{SG67717, &UA179050},
	{SG67723, &UA472860},
	{SG67725, &UA1864886},
	{SG67734, &UA2277807},
	{SG67750, &UA522038},
	{SG67751, &UA1863607},
	{SG67752, &UA2318079},
	{SG67763, &UA2274054},
	{SG67771, &UA498230},
	{SG67774, &UA2260325},
	{SG67779, &UA576443},
	{SG67780, &UA2166369},
	{SG67781, &UA2338006},
	{SG67793, &UA440270},
	{NULL}};
const Device *DCatchAllMobile[] = {
	&UA7772,
	&UA20517,
	&UA23438,
	&UA23999,
	&UA49175,
	&UA53462,
	&UA86166,
	&UA102504,
	&UA271952,
	&UA300081,
	&UA2956298,
	&UA2990318,
	&UA3126477,
	&UA571141,
	&UA649669,
	&UA1896884,
	&UA1964023,
	&UA2653895,
	&UA2846609,
	&UA3086198,
	&UA2622464,
	&UA3109289,
	&UA3093854,
	&UA85208,
	&UA87307,
	&UA87736,
	&UA136021,
	&UA144750,
	&UA623034,
	&UA891929,
	&UA2383642,
	&UA2415865,
	&UA2419109,
	&UA2598136,
	&UA2601044,
	&UA2658771,
	&UA2767105,
	&UA2769683,
	&UA3109002,
	&UA3093304,
	&UA3062174,
	&UA3111068,
	&UA3109141,
	&UA573612,
	&UA3093855,
	&UA2387153,
	&UA2387243,
	&UA2389110,
	&UA2390859,
	&UA2404098,
	&UA2404638,
	&UA2405631,
	&UA2406789,
	&UA2424538,
	&UA2427230,
	&UA2427651,
	&UA2428322,
	&UA2429422,
	&UA2430316,
	&UA2431963,
	&UA2460204,
	&UA2467480,
	&UA2481687,
	&UA2555396,
	&UA2578121,
	&UA2778083,
	&UA2507921,
	&UA2507945,
	&UA2533114,
	&UA2559156,
	&UA2560095,
	&UA2560139,
	&UA2580086,
	&UA2581032,
	&UA2581066,
	&UA2659485,
	&UA2659713,
	&UA2661324,
	&UA2662290,
	&UA2662737,
	&UA2699741,
	&UA2732325,
	&UA2779083,
	&UA2852276,
	&UA2853313,
	&UA3071496,
	&UA3118955,
	&UA88633,
	&UA2429600,
	&UA2461759,
	&UA2461811,
	&UA2469292,
	&UA2484170,
	&UA2486751,
	&UA2487018,
	&UA2487068,
	&UA2556007,
	&UA2557594,
	&UA2557779,
	&UA2596772,
	&UA2657254,
	&UA2847745,
	&UA2778175,
	&UA30261,
	&UA2738516,
	&UA179493,
	&UA420139,
	&UA440192,
	&UA506391,
	&UA578606,
	&UA639132,
	&UA900824,
	&UA1979525,
	&UA2134210,
	&UA2149082,
	&UA2158381,
	&UA2260447,
	&UA2303942,
	&UA2310667,
	&UA2370185,
	&UA2455606,
	&UA2738761,
	&UA31365,
	&UA2696729,
	&UA2727491,
	&UA2983132,
	&UA3086852,
	&UA32687,
	&UA2265975,
	&UA89115,
	&UA259502,
	&UA262862,
	&UA2712934,
	&UA2714644,
	&UA2738006,
	&UA2751464,
	&UA2751638,
	&UA2768423,
	&UA2768502,
	&UA2773431,
	&UA2868224,
	&UA2869314,
	&UA2870360,
	&UA2872181,
	&UA2878690,
	&UA2879247,
	&UA3049219,
	&UA3087752,
	&UA48874,
	&UA90035,
	&UA692346,
	&UA711611,
	&UA2362340,
	&UA2405801,
	&UA2457642,
	&UA2483060,
	&UA2696056,
	&UA3085982,
	&UA3023747,
	&UA3025936,
	&UA3030019,
	&UA3032616,
	&UA3034284,
	&UA3058391,
	&UA3059020,
	&UA3080028,
	&UA3092392,
	&UA3092797,
	&UA3092975,
	&UA3093160,
	&UA3108448,
	&UA3108705,
	&UA2750263,
	&UA2886764,
	&UA2900451,
	&UA2901171,
	&UA2938279,
	&UA2941530,
	&UA2962038,
	&UA3005191,
	&UA3019706,
	&UA2019515,
	&UA2459016,
	&UA2708703,
	&UA2762617,
	&UA3100185,
	&UA2508192,
	&UA2701738,
	&UA2730160,
	&UA2748972,
	&UA2987725,
	&UA2230512,
	&UA2879367,
	&UA77543,
	&UA395841,
	&UA2383478,
	&UA2944179,
	&UA2958531,
	&UA2972710,
	&UA2972718,
	&UA2974676,
	&UA3004428,
	&UA3009458,
	&UA3012722,
	&UA3056521,
	&UA3057974,
	&UA160544,
	&UA633273,
	&UA2424591,
	&UA2958453,
	&UA2488393,
	&UA2580164,
	&UA2580961,
	&UA2872453,
	&UA2660113,
	&UA2734631,
	&UA2958063,
	&UA2729896,
	&UA2878269,
	&UA3015259,
	&UA3015692,
	&UA3074308,
	&UA3110597,
	&UA2580219,
	&UA2779192,
	&UA2851912,
	&UA2857891,
	&UA2934788,
	&UA2952355,
	&UA2953322,
	&UA2986266,
	&UA2988814,
	&UA3045656,
	&UA3045877,
	&UA3071817,
	&UA3086547,
	&UA3087266,
	&UA3100158,
	&UA3100354,
	&UA3119014,
	&UA3119819,
	&UA2530757,
	&UA2558576,
	&UA379110,
	&UA2242582,
	&UA2456679,
	&UA230042,
	&UA2168485,
	&UA646338,
	&UA1963099,
	&UA54861,
	&UA2423521,
	&UA2733020,
	&UA2733225,
	&UA3072119,
	&UA2242617,
	&UA2571061,
	&UA2704793,
	&UA2898812,
	&UA2400976,
	&UA2450380,
	&UA90179,
	&UA2356506,
	&UA2390406,
	&UA2456596,
	&UA2457783,
	&UA2504792,
	&UA2424087,
	&UA2437330,
	&UA2747482,
	&UA2424495,
	&UA2457225,
	&UA2482878,
	&UA2843032,
	&UA2428286,
	&UA2506441,
	&UA2697057,
	&UA32807,
	&UA2712607,
	&UA2797168,
	&UA33439,
	&UA2553107,
	&UA2695379,
	&UA1966847,
	&UA157449,
	&UA3027683,
	&UA3058299,
	&UA90085,
	&UA1901540,
	&UA460018,
	&UA639228,
	&UA645601,
	&UA2264520,
	&UA2461309,
	&UA2529103,
	&UA2467025,
	&UA2656205,
	&UA2696003,
	&UA2728374,
	&UA2844397,
	&UA2878173,
	&UA2951503,
	&UA2715779,
	&UA2769328,
	&UA2784900,
	&UA2878723,
	&UA2722662,
	&UA2879899,
	&UA3009962,
	&UA2427139,
	&UA2695671,
	&UA2579570,
	&UA2730970,
	&UA2749667,
	&UA90734,
	&UA2157120,
	&UA2432721,
	&UA2527668,
	&UA2482078,
	&UA89929,
	&UA401062,
	&UA2159079,
	&UA2558285,
	&UA2559993,
	&UA2554813,
	&UA185544,
	&UA2422635,
	&UA2506411,
	&UA2790607,
	&UA2579734,
	&UA2779966,
	&UA2558389,
	&UA90563,
	&UA561836,
	&UA32497,
	&UA1943916,
	&UA2457964,
	&UA2528252,
	&UA2654003,
	&UA2428271,
	&UA2653806,
	&UA2867174,
	&UA2892902,
	&UA3072896,
	&UA2339451,
	&UA2458101,
	&UA2727466,
	&UA167223,
	&UA2531902,
	&UA2659517,
	&UA2730075,
	&UA2856735,
	&UA2986182,
	&UA2565842,
	&UA1942092,
	&UA2466915,
	&UA2727274,
	&UA90735,
	&UA575381,
	&UA90731,
	&UA2809916,
	&UA2659457,
	&UA2987620,
	&UA90736,
	&UA2457598,
	&UA3059329,
	&UA3061012,
	&UA2485718,
	&UA2596577,
	&UA2580230,
	&UA2707540,
	&UA2851080,
	&UA2422360,
	&UA2533211,
	&UA2580066,
	&UA2764280,
	&UA2961134,
	&UA2422037,
	&UA2841556,
	&UA2582034,
	&UA2463946,
	&UA2563595,
	&UA2778894,
	&UA2410298,
	&UA2469211,
	&UA2469291,
	&UA2786239,
	&UA3041357,
	&UA3084837,
	&UA3032716,
	&UA3031636,
	&UA49409,
	&UA2661885,
	&UA2973696,
	&UA2560093,
	&UA2767899,
	&UA2894939,
	&UA2424168,
	&UA2430095,
	&UA2528203,
	&UA3043444,
	&UA2582425,
	&UA2792087,
	&UA2457954,
	&UA2840927,
	&UA3059066,
	&UA2662451,
	&UA2727063,
	&UA2951790,
	&UA3058870,
	&UA2747115,
	&UA2958624,
	&UA2458423,
	&UA2953554,
	&UA2706480,
	&UA2428709,
	&UA2456485,
	&UA2846037,
	&UA2558093,
	&UA2560467,
	&UA2855084,
	&UA2559654,
	&UA2747508,
	&UA2404486,
	&UA2426853,
	&UA2482714,
	&UA2594507,
	&UA3118387,
	&UA2430822,
	&UA2558376,
	&UA2952266,
	&UA2659511,
	&UA2962151,
	&UA2963190,
	&UA2556119,
	&UA2368925,
	&UA2428325,
	&UA2430944,
	&UA2504998,
	&UA2597081,
	&UA2695096,
	&UA3118277,
	&UA2665652,
	&UA2962974,
	&UA538717,
	&UA2485092,
	&UA2485541,
	&UA2362258,
	&UA2391547,
	&UA2423654,
	&UA2527361,
	&UA2504746,
	&UA2890088,
	&UA2484775,
	&UA2660121,
	&UA2748349,
	&UA176621,
	&UA547308,
	&UA2409761,
	&UA2529402,
	&UA61153,
	&UA2560767,
	&UA2779020,
	&UA2846042,
	&UA3079499,
	&UA1940539,
	&UA2469876,
	&UA2487021,
	&UA2586730,
	&UA2467222,
	&UA2699862,
	&UA3028280,
	&UA3033037,
	&UA2779233,
	&UA3078620,
	&UA2303668,
	&UA2701200,
	&UA3100070,
	&UA2701355,
	&UA2944236,
	&UA3013891,
	&UA3092044,
	&UA3002118,
	&UA2423144,
	&UA2430370,
	&UA2749153,
	&UA2767766,
	&UA3058782,
	&UA111877,
	&UA2482329,
	&UA2529610,
	&UA3010714,
	&UA2555261,
	&UA3079310,
	&UA2426715,
	&UA2654140,
	&UA2951117,
	&UA2504953,
	&UA2557414,
	&UA2659611,
	&UA2855967,
	&UA2850477,
	&UA3030139,
	&UA3078772,
	&UA2907,
	&UA158682,
	&UA445006,
	&UA2483047,
	&UA2484385,
	&UA2555813,
	&UA2505392,
	&UA2596996,
	&UA371177,
	&UA2019745,
	&UA2022026,
	&UA2298369,
	&UA2528227,
	&UA2529687,
	&UA2542245,
	&UA2655458,
	&UA2696143,
	&UA2742217,
	&UA2865968,
	&UA3060697,
	&UA3078621,
	&UA3079180,
	&UA143775,
	&UA2728894,
	&UA2764950,
	&UA2960945,
	&UA3014077,
	&UA3009727,
	&UA3014409,
	&UA61789,
	&UA64401,
	&UA2528696,
	&UA2598610,
	&UA2986368,
	&UA2795065,
	&UA2507024,
	&UA2728163,
	&UA3003905,
	&UA3023461,
	&UA3107884,
	&UA15288,
	&UA63423,
	&UA2660874,
	&UA3108492,
	&UA2485378,
	&UA2505462,
	&UA2528636,
	&UA2560315,
	&UA2597542,
	&UA2738025,
	&UA2868388,
	&UA2763853,
	&UA2844695,
	&UA2963899,
	&UA3015611,
	&UA3059059,
	&UA3092974,
	&UA50146,
	&UA50298,
	&UA67756,
	&UA59143,
	&UA2586829,
	&UA103027,
	&UA2456973,
	&UA2457785,
	&UA2483192,
	&UA2555587,
	&UA2698942,
	&UA2764554,
	&UA2781272,
	&UA2962913,
	&UA3005205,
	&UA3016335,
	&UA62731,
	&UA79564,
	&UA2469513,
	&UA2696069,
	&UA2846130,
	&UA2864645,
	&UA2995418,
	&UA3031425,
	&UA3125677,
	&UA2462807,
	&UA2655855,
	&UA2697363,
	&UA2747887,
	&UA2794753,
	&UA2958904,
	&UA2935123,
	&UA2683492,
	&UA2996112,
	&UA3078505,
	&UA3084983,
	&UA3123916,
	&UA3125587,
	&UA518116,
	&UA2663545,
	&UA2665452,
	&UA2751907,
	&UA2868627,
	&UA2842246,
	&UA2844862,
	&UA3056544,
	&UA2596329,
	&UA2797323,
	&UA2865844,
	&UA2843284,
	&UA2863218,
	&UA2864882,
	&UA1933815,
	&UA2659978,
	&UA2942989,
	&UA54025,
	&UA2459559,
	&UA2778961,
	&UA2952861,
	&UA49937,
	&UA50136,
	&UA54297,
	&UA2580417,
	&UA2848934,
	&UA64539,
	&UA2730051,
	&UA3044702,
	&UA2737378,
	&UA2869209,
	&UA3080021,
	&UA44045,
	&UA89818,
	&UA2461308,
	&UA2556322,
	&UA2598672,
	&UA87633,
	&UA2599144,
	&UA2698686,
	&UA2706976,
	&UA3092788,
	&UA2665255,
	&UA2865767,
	&UA2937848,
	&UA87416,
	&UA2960131,
	&UA3093131,
	&UA26701,
	&UA53459,
	&UA2422737,
	&UA2843617,
	&UA2555273,
	&UA2599569,
	&UA3052325,
	&UA33373,
	&UA44591,
	&UA54174,
	&UA54334,
	&UA58815,
	&UA60856,
	&UA88650,
	&UA2657281,
	&UA2660347,
	&UA2852410,
	&UA2852553,
	&UA2986854,
	&UA3013063,
	&UA3013990,
	&UA3014833,
	&UA3058783,
	&UA3108663,
	&UA18827,
	&UA22239,
	&UA2428950,
	&UA2467096,
	&UA2482999,
	&UA2659538,
	&UA2698661,
	&UA2763521,
	&UA2764260,
	&UA2844846,
	&UA2951396,
	&UA2767739,
	&UA2958357,
	&UA2963144,
	&UA3008553,
	&UA3013553,
	&UA3059745,
	&UA3108441,
	&UA62274,
	&UA62999,
	&UA2482968,
	&UA2484383,
	&UA2579922,
	&UA2654883,
	&UA2665207,
	&UA2702519,
	&UA2747058,
	&UA2752307,
	&UA2797769,
	&UA2861949,
	&UA2792285,
	&UA2847306,
	&UA3044090,
	&UA3021020,
	&UA3030147,
	&UA3074500,
	&UA3126275,
	&UA4562,
	&UA20740,
	&UA61169,
	&UA2432873,
	&UA2505294,
	&UA2555487,
	&UA2748423,
	&UA2847785,
	&UA2891332,
	&UA2987835,
	&UA3100501,
	&UA2953142,
	&UA3003156,
	&UA3058566,
	&UA2513,
	&UA61747,
	&UA8844,
	&UA13658,
	&UA21667,
	&UA30267,
	&UA53493,
	&UA87418,
	&UA88451,
	&UA88902,
	&UA2428401,
	&UA2458061,
	&UA2749913,
	&UA2867543,
	&UA32506,
	&UA44019,
	&UA51094,
	&UA61648,
	&UA2458664,
	&UA2598004,
	&UA2702291,
	&UA2702865,
	&UA2763086,
	&UA2763201,
	&UA2851226,
	&UA2951413,
	&UA3006104,
	&UA3010974,
	&UA3058961,
	&UA3126734,
	&UA17371,
	&UA18746,
	&UA21010,
	&UA22642,
	&UA53475,
	&UA2458453,
	&UA2579234,
	&UA2697831,
	&UA2849094,
	&UA2952372,
	&UA2986345,
	&UA3092782,
	&UA3108412,
	&UA3125459,
	NULL};
const Segmentdata DChrome10_19[] = {
	{SG61389, &UA368202},
	{SG61390, &UA441158},
	{SG61391, &UA454773},
	{SG61392, &UA458294},
	{SG61393, &UA605640},
	{SG61394, &UA606640},
	{SG61395, &UA607524},
	{SG61396, &UA624761},
	{SG61397, &UA692168},
	{SG61398, &UA844263},
	{SG61399, &UA895330},
	{SG61400, &UA903739},
	{SG61401, &UA909419},
	{SG61402, &UA911039},
	{SG61403, &UA911529},
	{SG61404, &UA1989409},
	{SG61405, &UA1997105},
	{SG61406, &UA2005209},
	{SG61407, &UA2007980},
	{SG61408, &UA2043660},
	{SG61409, &UA2043627},
	{SG61410, &UA2167892},
	{SG61411, &UA2193435},
	{SG61412, &UA2194222},
	{SG61413, &UA2197495},
	{SG61414, &UA2203275},
	{SG61415, &UA2206102},
	{SG61416, &UA2206184},
	{SG61417, &UA2218672},
	{SG61418, &UA2220282},
	{SG61419, &UA2229354},
	{SG61420, &UA2235766},
	{SG61421, &UA2235856},
	{SG61422, &UA2238275},
	{SG61423, &UA2260303},
	{SG61424, &UA2260369},
	{SG61425, &UA2260370},
	{SG61426, &UA2260422},
	{SG61427, &UA2340481},
	{SG61428, &UA2355605},
	{SG61429, &UA2358588},
	{SG61430, &UA2359533},
	{SG61431, &UA2363165},
	{SG61432, &UA2369027},
	{SG61433, &UA2388105},
	{SG61434, &UA2388223},
	{SG61435, &UA2390783},
	{SG61436, &UA2391464},
	{SG61437, &UA2391463},
	{SG61438, &UA2407514},
	{SG61439, &UA2408420},
	{SG61440, &UA2424228},
	{SG61441, &UA2430534},
	{SG61442, &UA2431386},
	{SG61443, &UA2432955},
	{SG61444, &UA2432998},
	{SG61445, &UA2437338},
	{SG61446, &UA2455797},
	{SG61447, &UA2461976},
	{SG61448, &UA2470518},
	{SG61449, &UA2472041},
	{SG61450, &UA2490823},
	{SG61451, &UA2508001},
	{SG61452, &UA2535470},
	{SG61453, &UA2545041},
	{SG61454, &UA2566581},
	{SG61455, &UA2568263},
	{SG61456, &UA2644116},
	{SG61458, &UA2813906},
	{SG61459, &UA2947806},
	{SG61460, &UA2980326},
	{SG61461, &UA2995438},
	{SG61462, &UA353880},
	{SG61463, &UA401278},
	{SG61464, &UA401829},
	{SG61465, &UA427022},
	{SG61466, &UA591195},
	{SG61467, &UA599939},
	{SG61468, &UA2166764},
	{SG61469, &UA2167524},
	{SG61470, &UA2259147},
	{SG61471, &UA2259148},
	{SG61472, &UA2355036},
	{SG61473, &UA2364648},
	{SG61474, &UA2367871},
	{SG61475, &UA2404559},
	{SG61476, &UA2597218},
	{SG61477, &UA2598915},
	{SG61478, &UA2600320},
	{SG61745, &UA897296},
	{SG61746, &UA897319},
	{SG61747, &UA900904},
	{SG61748, &UA901022},
	{SG61749, &UA902276},
	{SG61750, &UA903232},
	{SG61751, &UA1952132},
	{SG61752, &UA2355339},
	{SG61753, &UA2355558},
	{SG61754, &UA2363069},
	{SG61755, &UA2363083},
	{SG61756, &UA2428779},
	{SG61757, &UA2453103},
	{SG61758, &UA2504924},
	{SG61759, &UA2640751},
	{SG61760, &UA2695166},
	{SG61791, &UA247170},
	{SG61792, &UA249762},
	{SG61793, &UA343072},
	{SG61794, &UA343332},
	{SG61795, &UA675304},
	{SG61796, &UA699364},
	{SG61797, &UA744854},
	{SG61798, &UA2152442},
	{SG61799, &UA2233988},
	{SG61800, &UA2259197},
	{SG61801, &UA2305499},
	{SG61802, &UA2364699},
	{SG61803, &UA2980507},
	{SG61804, &UA2980698},
	{SG61806, &UA207812},
	{SG61807, &UA207856},
	{SG61808, &UA216184},
	{SG61809, &UA320688},
	{SG61810, &UA578378},
	{SG61811, &UA602414},
	{SG61812, &UA1893121},
	{SG61813, &UA1992915},
	{SG61814, &UA2355291},
	{SG61815, &UA2375479},
	{SG61816, &UA2409013},
	{SG61817, &UA2708510},
	{SG61818, &UA2709374},
	{SG61819, &UA2709520},
	{SG61820, &UA2710210},
	{SG61821, &UA2711329},
	{SG61822, &UA2803848},
	{SG61823, &UA2860757},
	{SG61824, &UA2878101},
	{SG61825, &UA2933590},
	{SG61826, &UA2979916},
	{SG61827, &UA2980045},
	{SG61902, &UA91043},
	{SG61903, &UA91052},
	{SG61904, &UA252197},
	{SG61905, &UA252342},
	{SG61906, &UA255824},
	{SG61907, &UA255965},
	{SG61908, &UA919567},
	{SG61909, &UA2362117},
	{SG61910, &UA2466367},
	{SG62027, &UA347151},
	{SG62028, &UA351793},
	{SG62029, &UA353541},
	{SG62030, &UA366606},
	{SG62031, &UA368129},
	{SG62032, &UA368130},
	{SG62033, &UA380155},
	{SG62034, &UA2259351},
	{SG62035, &UA2362170},
	{SG62036, &UA2495397},
	{SG62037, &UA2496800},
	{SG62038, &UA2596284},
	{SG62039, &UA2809748},
	{SG62040, &UA2979839},
	{SG62041, &UA3101614},
	{SG62054, &UA381323},
	{SG62056, &UA401369},
	{SG62058, &UA401392},
	{SG62059, &UA401925},
	{SG62060, &UA601391},
	{SG62061, &UA1951866},
	{SG62062, &UA2161855},
	{SG62063, &UA2166577},
	{SG62064, &UA2258965},
	{SG62065, &UA2425902},
	{SG62066, &UA2590700},
	{SG62067, &UA2596237},
	{SG62068, &UA2602558},
	{SG62069, &UA2817533},
	{SG62072, &UA221015},
	{SG62074, &UA247980},
	{SG62077, &UA333314},
	{SG62079, &UA336442},
	{SG62081, &UA337799},
	{SG62083, &UA451407},
	{SG62084, &UA669035},
	{SG62086, &UA2455907},
	{SG62088, &UA2639240},
	{SG62090, &UA2644326},
	{SG62092, &UA3083309},
	{SG62326, &UA217860},
	{SG62328, &UA248984},
	{SG62330, &UA249773},
	{SG62332, &UA615627},
	{SG62334, &UA630284},
	{SG62336, &UA860563},
	{SG62339, &UA2234481},
	{SG62340, &UA2298999},
	{SG62342, &UA2818200},
	{SG62344, &UA2980424},
	{SG62347, &UA3041828},
	{SG62348, &UA181055},
	{SG62349, &UA521054},
	{SG62350, &UA756869},
	{SG62351, &UA822507},
	{SG62352, &UA1870337},
	{SG62353, &UA2151221},
	{SG62354, &UA2159887},
	{SG62355, &UA2480491},
	{SG62356, &UA2674459},
	{SG62357, &UA2674483},
	{SG62358, &UA2933586},
	{SG62359, &UA2958512},
	{SG62360, &UA2973290},
	{SG62362, &UA368200},
	{SG62364, &UA453892},
	{SG62366, &UA587348},
	{SG62368, &UA605856},
	{SG62370, &UA606088},
	{SG62372, &UA614977},
	{SG62374, &UA625454},
	{SG62376, &UA627121},
	{SG62378, &UA636143},
	{SG62379, &UA666977},
	{SG62380, &UA904573},
	{SG62381, &UA907270},
	{SG62382, &UA1965280},
	{SG62383, &UA1979300},
	{SG62384, &UA1998571},
	{SG62385, &UA2015114},
	{SG62386, &UA2043695},
	{SG62387, &UA2167448},
	{SG62388, &UA2206187},
	{SG62389, &UA2235770},
	{SG62390, &UA2259102},
	{SG62391, &UA2355667},
	{SG62392, &UA2371856},
	{SG62393, &UA2374532},
	{SG62394, &UA2407662},
	{SG62395, &UA2411737},
	{SG62396, &UA2415828},
	{SG62397, &UA2424224},
	{SG62398, &UA2430857},
	{SG62400, &UA2433144},
	{SG62403, &UA2472065},
	{SG62405, &UA2535471},
	{SG62407, &UA2567537},
	{SG62409, &UA2939577},
	{SG62423, &UA174912},
	{SG62424, &UA181109},
	{SG62425, &UA195382},
	{SG62426, &UA205402},
	{SG62427, &UA319554},
	{SG62428, &UA673659},
	{SG62429, &UA771860},
	{SG62430, &UA1863875},
	{SG62431, &UA2146238},
	{SG62432, &UA2213679},
	{SG62433, &UA2456018},
	{SG62434, &UA2527248},
	{SG62435, &UA2596285},
	{SG62436, &UA2205896},
	{SG62437, &UA2205908},
	{SG62438, &UA2205911},
	{SG62439, &UA2206027},
	{SG62440, &UA2206039},
	{SG62441, &UA2206054},
	{SG62442, &UA2206118},
	{SG62443, &UA2236461},
	{SG62444, &UA2236463},
	{SG62445, &UA2237040},
	{SG62446, &UA2237043},
	{SG62447, &UA2237045},
	{SG62448, &UA2237699},
	{SG62449, &UA2238742},
	{SG62450, &UA2238748},
	{SG62451, &UA2238806},
	{SG62452, &UA2238963},
	{SG62453, &UA2260082},
	{SG62454, &UA2260316},
	{SG62455, &UA2353119},
	{SG62456, &UA2357742},
	{SG62457, &UA2357966},
	{SG62458, &UA2358168},
	{SG62459, &UA2358297},
	{SG62460, &UA2358315},
	{SG62461, &UA2361201},
	{SG62462, &UA2368854},
	{SG62463, &UA2369029},
	{SG62464, &UA2387135},
	{SG62465, &UA2406023},
	{SG62466, &UA2431938},
	{SG62467, &UA2432016},
	{SG62468, &UA2432144},
	{SG62469, &UA2432251},
	{SG62470, &UA2432502},
	{SG62471, &UA2432503},
	{SG62472, &UA2432504},
	{SG62473, &UA2432506},
	{SG62474, &UA2432508},
	{SG62475, &UA2432509},
	{SG62476, &UA2432510},
	{SG62477, &UA2432511},
	{SG62478, &UA2432512},
	{SG62479, &UA2432507},
	{SG62480, &UA2432814},
	{SG62481, &UA2432928},
	{SG62482, &UA2433199},
	{SG62483, &UA2433305},
	{SG62484, &UA2433307},
	{SG62485, &UA2433310},
	{SG62486, &UA2433596},
	{SG62487, &UA2433598},
	{SG62488, &UA2433597},
	{SG62489, &UA2433595},
	{SG62490, &UA2433803},
	{SG62491, &UA2433804},
	{SG62492, &UA2435844},
	{SG62493, &UA2460996},
	{SG62494, &UA2461180},
	{SG62495, &UA2461181},
	{SG62496, &UA2461182},
	{SG62497, &UA2461186},
	{SG62498, &UA2461275},
	{SG62499, &UA2461927},
	{SG62500, &UA2461928},
	{SG62501, &UA2461929},
	{SG62502, &UA2461931},
	{SG62503, &UA2461932},
	{SG62504, &UA2461934},
	{SG62505, &UA2461935},
	{SG62506, &UA2461936},
	{SG62507, &UA2461939},
	{SG62508, &UA2461940},
	{SG62509, &UA2461941},
	{SG62510, &UA2461942},
	{SG62511, &UA2461943},
	{SG62512, &UA2461944},
	{SG62513, &UA2469256},
	{SG62514, &UA2469257},
	{SG62515, &UA2469258},
	{SG62516, &UA2469259},
	{SG62517, &UA2469367},
	{SG62518, &UA2469370},
	{SG62519, &UA2469373},
	{SG62520, &UA2469889},
	{SG62521, &UA2470073},
	{SG62522, &UA2470074},
	{SG62523, &UA2470083},
	{SG62524, &UA2485674},
	{SG62525, &UA2485802},
	{SG62526, &UA2486941},
	{SG62527, &UA2513313},
	{SG62528, &UA2513324},
	{SG62529, &UA2513857},
	{SG62530, &UA2514333},
	{SG62531, &UA2514338},
	{SG62532, &UA2516374},
	{SG62533, &UA2537699},
	{SG62534, &UA2538907},
	{SG62535, &UA2538911},
	{SG62536, &UA2538955},
	{SG62537, &UA2538982},
	{SG62538, &UA2538984},
	{SG62539, &UA2539527},
	{SG62540, &UA2539659},
	{SG62541, &UA2561304},
	{SG62542, &UA2561850},
	{SG62543, &UA2566065},
	{SG62544, &UA2566066},
	{SG62545, &UA2566064},
	{SG62546, &UA2566068},
	{SG62547, &UA2566277},
	{SG62548, &UA2566276},
	{SG62549, &UA2566283},
	{SG62550, &UA2566295},
	{SG62551, &UA2566563},
	{SG62552, &UA2566567},
	{SG62553, &UA2566574},
	{SG62554, &UA2566577},
	{SG62555, &UA2566579},
	{SG62556, &UA2567036},
	{SG62557, &UA2567040},
	{SG62558, &UA2567041},
	{SG62559, &UA2567042},
	{SG62560, &UA2567043},
	{SG62561, &UA2567045},
	{SG62562, &UA2567047},
	{SG62563, &UA2567354},
	{SG62564, &UA2567359},
	{SG62565, &UA2567595},
	{SG62566, &UA2567599},
	{SG62567, &UA2567745},
	{SG62568, &UA2567756},
	{SG62569, &UA2568035},
	{SG62570, &UA2568034},
	{SG62571, &UA2568176},
	{SG62572, &UA2568185},
	{SG62573, &UA2568193},
	{SG62574, &UA2568203},
	{SG62575, &UA2568394},
	{SG62576, &UA2568396},
	{SG62577, &UA2568410},
	{SG62578, &UA2568412},
	{SG62579, &UA2568765},
	{SG62580, &UA2569200},
	{SG62581, &UA2569442},
	{SG62582, &UA2569560},
	{SG62583, &UA2569896},
	{SG62584, &UA2570201},
	{SG62585, &UA2570212},
	{SG62586, &UA2583047},
	{SG62587, &UA2583051},
	{SG62588, &UA2583082},
	{SG62589, &UA2583138},
	{SG62590, &UA2583152},
	{SG62591, &UA2583303},
	{SG62592, &UA2583379},
	{SG62593, &UA2585478},
	{SG62594, &UA2585703},
	{SG62595, &UA2600472},
	{SG62596, &UA2601013},
	{SG62597, &UA2601473},
	{SG62598, &UA2601523},
	{SG62599, &UA2602648},
	{SG62600, &UA2603149},
	{SG62601, &UA2603589},
	{SG62602, &UA2603654},
	{SG62603, &UA2603777},
	{SG62604, &UA2604211},
	{SG62605, &UA2604834},
	{SG62606, &UA2666706},
	{SG62607, &UA2666707},
	{SG62608, &UA2666772},
	{SG62609, &UA2666836},
	{SG62610, &UA2666898},
	{SG62611, &UA2666981},
	{SG62612, &UA2667179},
	{SG62613, &UA2667276},
	{SG62614, &UA2667327},
	{SG62615, &UA2667670},
	{SG62616, &UA2668035},
	{SG62617, &UA2668067},
	{SG62618, &UA2668075},
	{SG62619, &UA2668508},
	{SG62620, &UA2668977},
	{SG62621, &UA2674277},
	{SG62622, &UA2676530},
	{SG62623, &UA2677368},
	{SG62624, &UA2702723},
	{SG62625, &UA2703257},
	{SG62626, &UA2704530},
	{SG62627, &UA2705113},
	{SG62628, &UA2705325},
	{SG62629, &UA2705596},
	{SG62630, &UA2705873},
	{SG62631, &UA2705903},
	{SG62632, &UA2707102},
	{SG62633, &UA2707435},
	{SG62634, &UA2715197},
	{SG62635, &UA2715354},
	{SG62636, &UA2717840},
	{SG62637, &UA2719362},
	{SG62638, &UA2732736},
	{SG62639, &UA2732827},
	{SG62640, &UA2732834},
	{SG62641, &UA2732868},
	{SG62642, &UA2740215},
	{SG62643, &UA2749770},
	{SG62644, &UA2753225},
	{SG62645, &UA2765982},
	{SG62646, &UA2770333},
	{SG62647, &UA2793324},
	{SG62648, &UA2855415},
	{SG62649, &UA2855619},
	{SG62650, &UA2857031},
	{SG62651, &UA2857084},
	{SG62652, &UA2857331},
	{SG62653, &UA2858837},
	{SG62654, &UA2859778},
	{SG62655, &UA2863641},
	{SG62656, &UA2863755},
	{SG62657, &UA2864045},
	{SG62658, &UA2864232},
	{SG62659, &UA2864519},
	{SG62660, &UA2865175},
	{SG62661, &UA2878946},
	{SG62662, &UA2878981},
	{SG62663, &UA2891935},
	{SG62664, &UA2891957},
	{SG62665, &UA2956226},
	{SG62666, &UA2995280},
	{SG62667, &UA3001727},
	{SG62668, &UA3006140},
	{SG62669, &UA3020014},
	{SG62670, &UA3106260},
	{SG62671, &UA3106988},
	{SG62672, &UA897299},
	{SG62673, &UA897344},
	{SG62674, &UA898193},
	{SG62675, &UA900906},
	{SG62676, &UA901082},
	{SG62677, &UA905648},
	{SG62678, &UA2259071},
	{SG62679, &UA2363058},
	{SG62680, &UA2387120},
	{SG62681, &UA2400994},
	{SG62682, &UA2423625},
	{SG62683, &UA2424043},
	{SG62684, &UA199106},
	{SG62685, &UA204611},
	{SG62686, &UA210469},
	{SG62687, &UA219919},
	{SG62688, &UA663952},
	{SG62690, &UA776353},
	{SG62691, &UA1887323},
	{SG62692, &UA1993352},
	{SG62693, &UA2292628},
	{SG62694, &UA2340403},
	{SG62695, &UA2355681},
	{SG62696, &UA2374372},
	{SG62697, &UA2597904},
	{SG62698, &UA2673267},
	{SG62699, &UA2709381},
	{SG62700, &UA2709512},
	{SG62701, &UA2709514},
	{SG62702, &UA2712119},
	{SG62703, &UA2751232},
	{SG62704, &UA2758853},
	{SG62705, &UA2759015},
	{SG62706, &UA2980695},
	{SG62714, &UA347781},
	{SG62715, &UA351710},
	{SG62717, &UA351784},
	{SG62719, &UA355648},
	{SG62721, &UA368194},
	{SG62722, &UA398266},
	{SG62723, &UA622888},
	{SG62724, &UA690160},
	{SG62725, &UA755439},
	{SG62726, &UA789687},
	{SG62727, &UA2159030},
	{SG62728, &UA2258980},
	{SG62729, &UA2413288},
	{SG62730, &UA2543053},
	{SG62731, &UA2597901},
	{SG62732, &UA3100681},
	{SG62733, &UA3118120},
	{SG62734, &UA18168},
	{SG62735, &UA180127},
	{SG62736, &UA193039},
	{SG62737, &UA193157},
	{SG62738, &UA195670},
	{SG62739, &UA195977},
	{SG62740, &UA518696},
	{SG62741, &UA560633},
	{SG62742, &UA653223},
	{SG62744, &UA1865225},
	{SG62746, &UA2135187},
	{SG62748, &UA2258593},
	{SG62750, &UA2258627},
	{SG62752, &UA2335201},
	{SG62754, &UA2362364},
	{SG62756, &UA2371338},
	{SG62758, &UA2580394},
	{SG62760, &UA2743910},
	{SG62762, &UA2980382},
	{SG62786, &UA91033},
	{SG62787, &UA252368},
	{SG62788, &UA255968},
	{SG62789, &UA587062},
	{SG62790, &UA626319},
	{SG62791, &UA629036},
	{SG62792, &UA1925220},
	{SG62793, &UA2148216},
	{SG62795, &UA2185632},
	{SG62796, &UA2355358},
	{SG62797, &UA2807633},
	{SG62798, &UA2950197},
	{SG62799, &UA180105},
	{SG62801, &UA180276},
	{SG62803, &UA311997},
	{SG62805, &UA623246},
	{SG62808, &UA2280597},
	{SG62810, &UA2385201},
	{SG62812, &UA2673357},
	{SG62814, &UA2675008},
	{SG62818, &UA2675942},
	{SG62819, &UA2708347},
	{SG62820, &UA2709330},
	{SG62821, &UA2710979},
	{SG62822, &UA2716830},
	{SG62823, &UA2735056},
	{SG62824, &UA2768027},
	{SG62825, &UA2840680},
	{SG62854, &UA252116},
	{SG62855, &UA253620},
	{SG62856, &UA255778},
	{SG62857, &UA577209},
	{SG62858, &UA1895711},
	{SG62859, &UA1905568},
	{SG62860, &UA2141913},
	{SG62963, &UA23661},
	{SG62964, &UA180921},
	{SG62965, &UA186744},
	{SG62966, &UA434389},
	{SG62967, &UA544266},
	{SG62968, &UA624210},
	{SG62969, &UA664893},
	{SG62974, &UA757718},
	{SG62978, &UA778911},
	{SG62979, &UA805231},
	{SG62980, &UA858708},
	{SG62981, &UA891540},
	{SG62982, &UA895145},
	{SG62983, &UA895554},
	{SG62984, &UA900920},
	{SG62985, &UA1960306},
	{SG62986, &UA1961101},
	{SG62987, &UA1997111},
	{SG62988, &UA1997834},
	{SG62989, &UA2163474},
	{SG62990, &UA2168627},
	{SG62991, &UA2168628},
	{SG62992, &UA2179635},
	{SG62993, &UA2190911},
	{SG62994, &UA2234459},
	{SG62995, &UA2258151},
	{SG62996, &UA2259958},
	{SG62997, &UA2260215},
	{SG62998, &UA2275929},
	{SG62999, &UA2339616},
	{SG63000, &UA2346416},
	{SG63001, &UA2356158},
	{SG63002, &UA2360335},
	{SG63003, &UA2413758},
	{SG63004, &UA2415083},
	{SG63005, &UA2434869},
	{SG63006, &UA2457673},
	{SG63007, &UA2477379},
	{SG63008, &UA2485445},
	{SG63009, &UA2486784},
	{SG63010, &UA2495493},
	{SG63011, &UA2505436},
	{SG63012, &UA2531211},
	{SG63013, &UA2544161},
	{SG63014, &UA2545985},
	{SG63015, &UA2552636},
	{SG63016, &UA2552936},
	{SG63017, &UA2556891},
	{SG63018, &UA2557563},
	{SG63019, &UA2582828},
	{SG63020, &UA2598015},
	{SG63021, &UA2598177},
	{SG63022, &UA2654104},
	{SG63023, &UA2674525},
	{SG63024, &UA2674667},
	{SG63025, &UA2674857},
	{SG63026, &UA2674865},
	{SG63027, &UA2675211},
	{SG63028, &UA2683611},
	{SG63029, &UA2699294},
	{SG63030, &UA2714923},
	{SG63031, &UA2717843},
	{SG63032, &UA2719083},
	{SG63033, &UA2748429},
	{SG63034, &UA2758684},
	{SG63035, &UA2763975},
	{SG63036, &UA2809213},
	{SG63037, &UA2813018},
	{SG63038, &UA2815273},
	{SG63039, &UA2843190},
	{SG63040, &UA2875314},
	{SG63041, &UA2897643},
	{SG63042, &UA2934489},
	{SG63043, &UA2956945},
	{SG63044, &UA2966987},
	{SG63045, &UA2985574},
	{SG63046, &UA2998709},
	{SG63047, &UA2999483},
	{SG63048, &UA3058138},
	{SG63049, &UA3071793},
	{SG63050, &UA3073645},
	{SG63051, &UA3079020},
	{SG63052, &UA3091621},
	{SG63053, &UA3100396},
	{SG63054, &UA3102887},
	{SG63055, &UA221371},
	{SG63056, &UA247030},
	{SG63057, &UA247991},
	{SG63058, &UA333717},
	{SG63059, &UA636101},
	{SG63060, &UA2259089},
	{SG63061, &UA2259262},
	{SG63062, &UA2295269},
	{SG63063, &UA2297218},
	{SG63064, &UA2355516},
	{SG63065, &UA2817795},
	{SG63201, &UA353872},
	{SG63202, &UA401267},
	{SG63203, &UA602871},
	{SG63204, &UA656196},
	{SG63205, &UA683121},
	{SG63206, &UA897923},
	{SG63207, &UA899834},
	{SG63208, &UA1972346},
	{SG63209, &UA1989965},
	{SG63210, &UA2000165},
	{SG63211, &UA2039629},
	{SG63212, &UA2204043},
	{SG63214, &UA2236464},
	{SG63216, &UA2260064},
	{SG63218, &UA2335268},
	{SG63219, &UA2335822},
	{SG63220, &UA2374236},
	{SG63221, &UA2496676},
	{SG63222, &UA2534850},
	{SG63223, &UA2599534},
	{SG63224, &UA2674477},
	{SG63225, &UA2674613},
	{SG63226, &UA2674792},
	{SG63227, &UA2674874},
	{SG63228, &UA2676617},
	{SG63230, &UA2715577},
	{SG63232, &UA2718861},
	{SG63235, &UA2752635},
	{SG63237, &UA3117746},
	{SG63337, &UA79950},
	{SG63338, &UA221016},
	{SG63339, &UA247033},
	{SG63340, &UA727292},
	{SG63341, &UA2375845},
	{SG63342, &UA2567658},
	{SG63343, &UA2672975},
	{SG63344, &UA2678793},
	{SG63345, &UA2708164},
	{SG63346, &UA2709427},
	{SG63347, &UA2710717},
	{SG63348, &UA2711746},
	{SG63349, &UA2750956},
	{SG63350, &UA2787084},
	{SG63351, &UA2860022},
	{SG63388, &UA456136},
	{SG63389, &UA610013},
	{SG63390, &UA626808},
	{SG63391, &UA899229},
	{SG63392, &UA900667},
	{SG63393, &UA900841},
	{SG63394, &UA901772},
	{SG63395, &UA902115},
	{SG63396, &UA924490},
	{SG63397, &UA1976846},
	{SG63398, &UA2333086},
	{SG63399, &UA2333410},
	{SG63400, &UA2377590},
	{SG63401, &UA2674475},
	{SG63402, &UA2674514},
	{SG63403, &UA2674794},
	{SG63404, &UA2674994},
	{SG63405, &UA2675708},
	{SG63406, &UA2714590},
	{SG63407, &UA2717981},
	{SG63408, &UA3032779},
	{SG63409, &UA3043679},
	{SG63410, &UA3070916},
	{SG63411, &UA3070931},
	{SG63412, &UA180523},
	{SG63413, &UA181420},
	{SG63414, &UA198133},
	{SG63415, &UA305316},
	{SG63416, &UA309224},
	{SG63417, &UA311563},
	{SG63418, &UA390757},
	{SG63419, &UA416388},
	{SG63420, &UA859921},
	{SG63421, &UA2303443},
	{SG63422, &UA2318350},
	{SG63423, &UA2330815},
	{SG63424, &UA2433593},
	{SG63425, &UA2976830},
	{SG63426, &UA3022434},
	{SG63427, &UA3022570},
	{SG63428, &UA3022729},
	{SG63429, &UA3022749},
	{SG63430, &UA3023083},
	{SG63431, &UA3023178},
	{SG63433, &UA3023185},
	{SG63434, &UA3024119},
	{SG63435, &UA3031165},
	{SG63436, &UA3032928},
	{SG63437, &UA3032930},
	{SG63438, &UA3032931},
	{SG63439, &UA3032939},
	{SG63440, &UA3032943},
	{SG63441, &UA3032953},
	{SG63442, &UA3033489},
	{SG63443, &UA3033496},
	{SG63444, &UA3033497},
	{SG63445, &UA3033507},
	{SG63446, &UA3033491},
	{SG63447, &UA3033986},
	{SG63448, &UA3033991},
	{SG63449, &UA3033999},
	{SG63450, &UA3034112},
	{SG63451, &UA3034365},
	{SG63452, &UA3034679},
	{SG63453, &UA3035715},
	{SG63454, &UA3035725},
	{SG63455, &UA3035730},
	{SG63456, &UA3036109},
	{SG63457, &UA3036132},
	{SG63458, &UA3036122},
	{SG63459, &UA3036413},
	{SG63460, &UA3036421},
	{SG63461, &UA3036508},
	{SG63462, &UA3036510},
	{SG63463, &UA3036519},
	{SG63464, &UA3036526},
	{SG63465, &UA3036525},
	{SG63466, &UA3036827},
	{SG63467, &UA3037083},
	{SG63468, &UA3037085},
	{SG63469, &UA3037100},
	{SG63470, &UA3037099},
	{SG63471, &UA3037103},
	{SG63472, &UA3037106},
	{SG63473, &UA3038046},
	{SG63474, &UA3059922},
	{SG63475, &UA3059931},
	{SG63476, &UA3060554},
	{SG63477, &UA3060555},
	{SG63478, &UA3060839},
	{SG63479, &UA3061297},
	{SG63480, &UA3061298},
	{SG63481, &UA3061301},
	{SG63482, &UA3061430},
	{SG63483, &UA3062280},
	{SG63484, &UA3078750},
	{SG63485, &UA3079178},
	{SG63486, &UA3079502},
	{SG63487, &UA3079623},
	{SG63488, &UA3079662},
	{SG63489, &UA3079915},
	{SG63560, &UA783740},
	{SG63561, &UA787751},
	{SG63562, &UA2195943},
	{SG63563, &UA2205752},
	{SG63564, &UA2217804},
	{SG63565, &UA2235731},
	{SG63566, &UA2259140},
	{SG63567, &UA2950703},
	{SG63568, &UA3118103},
	{SG63569, &UA199107},
	{SG63570, &UA207671},
	{SG63571, &UA208889},
	{SG63572, &UA211060},
	{SG63573, &UA216813},
	{SG63574, &UA219929},
	{SG63575, &UA323074},
	{SG63576, &UA327004},
	{SG63577, &UA648475},
	{SG63578, &UA786195},
	{SG63579, &UA2319351},
	{SG63580, &UA2674816},
	{SG63581, &UA2674863},
	{SG63582, &UA2718186},
	{SG63583, &UA352836},
	{SG63584, &UA352839},
	{SG63585, &UA366169},
	{SG63586, &UA366199},
	{SG63587, &UA588252},
	{SG63588, &UA2360186},
	{SG63589, &UA2375182},
	{SG63603, &UA412432},
	{SG63604, &UA604616},
	{SG63605, &UA605809},
	{SG63606, &UA607012},
	{SG63607, &UA613225},
	{SG63608, &UA2217506},
	{SG63609, &UA2220484},
	{SG63610, &UA2366680},
	{SG63611, &UA2674694},
	{SG63612, &UA2674803},
	{SG63613, &UA2674815},
	{SG63614, &UA2674871},
	{SG63615, &UA2674886},
	{SG63616, &UA2714753},
	{SG63617, &UA2717994},
	{SG63618, &UA2956932},
	{SG63619, &UA2989010},
	{SG63620, &UA401374},
	{SG63621, &UA401556},
	{SG63622, &UA591033},
	{SG63623, &UA2010435},
	{SG63624, &UA2259260},
	{SG63625, &UA2425059},
	{SG63626, &UA2545367},
	{SG63627, &UA2778003},
	{SG63628, &UA2790798},
	{SG63629, &UA412585},
	{SG63630, &UA427946},
	{SG63631, &UA457869},
	{SG63632, &UA614928},
	{SG63633, &UA614992},
	{SG63634, &UA623181},
	{SG63635, &UA849232},
	{SG63636, &UA2202054},
	{SG63637, &UA2477229},
	{SG63638, &UA399121},
	{SG63639, &UA820591},
	{SG63640, &UA897618},
	{SG63641, &UA900842},
	{SG63642, &UA901793},
	{SG63643, &UA907953},
	{SG63644, &UA2153121},
	{SG63645, &UA2360764},
	{SG63646, &UA2391579},
	{SG63647, &UA2422303},
	{SG63676, &UA79941},
	{SG63677, &UA198098},
	{SG63678, &UA219868},
	{SG63679, &UA327433},
	{SG63680, &UA2168230},
	{SG63681, &UA2259158},
	{SG63682, &UA2373167},
	{SG63683, &UA2374281},
	{SG63684, &UA2409601},
	{SG63685, &UA2411205},
	{SG63686, &UA2805957},
	{SG63687, &UA2821719},
	{SG63688, &UA3072438},
	{SG63748, &UA181064},
	{SG63749, &UA183468},
	{SG63750, &UA184489},
	{SG63751, &UA300109},
	{SG63752, &UA581750},
	{SG63753, &UA759205},
	{SG63754, &UA2323913},
	{SG63755, &UA2674649},
	{SG63756, &UA2674700},
	{SG63757, &UA2674715},
	{SG63758, &UA2675018},
	{SG63759, &UA2675721},
	{SG63760, &UA2718409},
	{SG63761, &UA2718770},
	{SG63792, &UA412897},
	{SG63793, &UA412941},
	{SG63794, &UA412963},
	{SG63795, &UA428601},
	{SG63796, &UA613342},
	{SG63797, &UA634223},
	{SG63798, &UA825498},
	{SG63799, &UA899135},
	{SG63800, &UA904085},
	{SG63801, &UA1979307},
	{SG63802, &UA1980022},
	{SG63803, &UA2200563},
	{SG63804, &UA2761137},
	{SG63805, &UA2878148},
	{SG63841, &UA205656},
	{SG63842, &UA218555},
	{SG63843, &UA249559},
	{SG63844, &UA560071},
	{SG63845, &UA623812},
	{SG63846, &UA1886748},
	{SG63847, &UA2674467},
	{SG63848, &UA2674616},
	{SG63849, &UA2674691},
	{SG63850, &UA2675210},
	{SG63851, &UA2675714},
	{SG63852, &UA2714601},
	{SG63853, &UA2751946},
	{SG63854, &UA2983761},
	{SG63855, &UA190416},
	{SG63856, &UA195664},
	{SG63857, &UA195668},
	{SG63858, &UA195672},
	{SG63859, &UA309393},
	{SG63860, &UA315805},
	{SG63861, &UA445813},
	{SG63862, &UA449216},
	{SG63863, &UA632920},
	{SG63864, &UA638423},
	{SG63865, &UA873899},
	{SG63866, &UA1863832},
	{SG63867, &UA2258700},
	{SG63868, &UA2293941},
	{SG63869, &UA2393245},
	{SG63870, &UA2674621},
	{SG63871, &UA2674677},
	{SG63873, &UA2714757},
	{SG63900, &UA205669},
	{SG63901, &UA249210},
	{SG63902, &UA249765},
	{SG63903, &UA250085},
	{SG63904, &UA250096},
	{SG63905, &UA250126},
	{SG63906, &UA250166},
	{SG63907, &UA250161},
	{SG63908, &UA251311},
	{SG63909, &UA341243},
	{SG63910, &UA398321},
	{SG63911, &UA2258016},
	{SG63912, &UA2337415},
	{SG63913, &UA2374270},
	{SG63914, &UA2674708},
	{SG63915, &UA2674858},
	{SG63916, &UA2715169},
	{SG63917, &UA222587},
	{SG63918, &UA237619},
	{SG63919, &UA246367},
	{SG63920, &UA379362},
	{SG63921, &UA421570},
	{SG63922, &UA539327},
	{SG63923, &UA2674462},
	{SG63924, &UA2674683},
	{SG63926, &UA2674877},
	{SG63927, &UA2675187},
	{SG63928, &UA2675194},
	{SG63929, &UA2714932},
	{SG63930, &UA2735611},
	{SG63931, &UA640487},
	{SG63932, &UA672904},
	{SG63933, &UA816636},
	{SG63934, &UA2009042},
	{SG63935, &UA2168277},
	{SG63936, &UA2193489},
	{SG63937, &UA2205234},
	{SG63938, &UA2206004},
	{SG63939, &UA2234011},
	{SG63940, &UA347708},
	{SG63941, &UA366217},
	{SG63942, &UA366218},
	{SG63943, &UA366326},
	{SG63944, &UA399007},
	{SG63945, &UA663800},
	{SG63946, &UA899758},
	{SG63947, &UA2168214},
	{SG63948, &UA2357846},
	{SG63949, &UA2844082},
	{SG63965, &UA900928},
	{SG63966, &UA901572},
	{SG63968, &UA2169146},
	{SG63969, &UA2360573},
	{SG63970, &UA2363182},
	{SG63971, &UA79932},
	{SG63972, &UA174924},
	{SG63973, &UA181119},
	{SG63974, &UA207039},
	{SG63975, &UA311465},
	{SG63976, &UA315052},
	{SG63977, &UA2333127},
	{SG63978, &UA2338098},
	{SG63979, &UA2933592},
	{SG63980, &UA2980383},
	{SG63981, &UA3042316},
	{SG63984, &UA609167},
	{SG63985, &UA614905},
	{SG63986, &UA614911},
	{SG63987, &UA614915},
	{SG63988, &UA614926},
	{SG63989, &UA614924},
	{SG63990, &UA614952},
	{SG63991, &UA614991},
	{SG63992, &UA615007},
	{SG63993, &UA830712},
	{SG63994, &UA2596294},
	{SG63995, &UA2674641},
	{SG63996, &UA2674851},
	{SG63997, &UA2714920},
	{SG63998, &UA3031566},
	{SG64057, &UA368173},
	{SG64059, &UA368213},
	{SG64061, &UA412431},
	{SG64063, &UA457772},
	{SG64065, &UA605617},
	{SG64067, &UA631047},
	{SG64070, &UA1964876},
	{SG64072, &UA2005704},
	{SG64074, &UA2043425},
	{SG64077, &UA2206621},
	{SG64079, &UA2233630},
	{SG64082, &UA2236175},
	{SG64084, &UA2360235},
	{SG64086, &UA2405708},
	{SG64087, &UA2431095},
	{SG64088, &UA2433581},
	{SG64089, &UA2472059},
	{SG64090, &UA2510339},
	{SG64091, &UA2528717},
	{SG64092, &UA2666722},
	{SG64093, &UA2967076},
	{SG64094, &UA3028154},
	{SG64114, &UA897314},
	{SG64115, &UA897521},
	{SG64116, &UA900839},
	{SG64117, &UA2363159},
	{SG64118, &UA2391107},
	{SG64119, &UA401283},
	{SG64120, &UA401290},
	{SG64121, &UA401320},
	{SG64122, &UA401390},
	{SG64123, &UA401423},
	{SG64124, &UA412895},
	{SG64125, &UA412968},
	{SG64126, &UA428564},
	{SG64127, &UA602319},
	{SG64128, &UA891105},
	{SG64129, &UA2223860},
	{SG64130, &UA2374246},
	{SG64131, &UA2473535},
	{SG64132, &UA2597176},
	{SG64133, &UA2600202},
	{SG64134, &UA2674611},
	{SG64135, &UA2674807},
	{SG64136, &UA2716430},
	{SG64144, &UA219051},
	{SG64145, &UA219625},
	{SG64146, &UA221034},
	{SG64147, &UA247982},
	{SG64148, &UA247985},
	{SG64149, &UA331939},
	{SG64150, &UA333715},
	{SG64151, &UA334625},
	{SG64152, &UA409433},
	{SG64153, &UA715235},
	{SG64154, &UA2258255},
	{SG64155, &UA2674790},
	{SG64156, &UA2674808},
	{SG64157, &UA2718690},
	{SG64158, &UA237082},
	{SG64159, &UA237917},
	{SG64160, &UA247041},
	{SG64161, &UA381940},
	{SG64162, &UA734789},
	{SG64163, &UA2200131},
	{SG64164, &UA79923},
	{SG64165, &UA174063},
	{SG64166, &UA640847},
	{SG64167, &UA2674806},
	{SG64168, &UA2675005},
	{SG64169, &UA634615},
	{SG64170, &UA738231},
	{SG64171, &UA896537},
	{SG64172, &UA898701},
	{SG64211, &UA79968},
	{SG64212, &UA91034},
	{SG64213, &UA91044},
	{SG64214, &UA252543},
	{SG64215, &UA254953},
	{SG64216, &UA688582},
	{SG64217, &UA2673769},
	{SG64218, &UA2708160},
	{SG64219, &UA2708861},
	{SG64220, &UA2709384},
	{SG64221, &UA2735319},
	{SG64222, &UA2768130},
	{SG64223, &UA2818063},
	{SG64224, &UA2860027},
	{SG64355, &UA901002},
	{SG64356, &UA901043},
	{SG64357, &UA901098},
	{SG64358, &UA901229},
	{SG64359, &UA901762},
	{SG64360, &UA901817},
	{SG64361, &UA901943},
	{SG64362, &UA902198},
	{SG64363, &UA902321},
	{SG64364, &UA902479},
	{SG64365, &UA2367487},
	{SG64366, &UA2597307},
	{SG64367, &UA2674460},
	{SG64368, &UA2674998},
	{SG64369, &UA2719145},
	{SG64387, &UA412969},
	{SG64388, &UA2191264},
	{SG64389, &UA2424037},
	{SG64390, &UA2554586},
	{SG64391, &UA2654403},
	{SG64392, &UA2766454},
	{SG64425, &UA91045},
	{SG64426, &UA252004},
	{SG64427, &UA379236},
	{SG64428, &UA628844},
	{SG64429, &UA669395},
	{SG64430, &UA2202361},
	{SG64454, &UA79959},
	{SG64456, &UA249763},
	{SG64458, &UA250072},
	{SG64460, &UA250269},
	{SG64467, &UA91032},
	{SG64468, &UA91111},
	{SG64469, &UA91112},
	{SG64470, &UA251867},
	{SG64471, &UA251971},
	{SG64472, &UA255821},
	{SG64473, &UA345507},
	{SG64474, &UA345509},
	{SG64475, &UA629108},
	{SG64476, &UA2410142},
	{SG64477, &UA2674696},
	{SG64478, &UA2675007},
	{SG64479, &UA2715740},
	{SG64500, &UA2797002},
	{SG64501, &UA3022577},
	{SG64502, &UA3023296},
	{SG64503, &UA3024328},
	{SG64504, &UA3030976},
	{SG64505, &UA3031629},
	{SG64506, &UA3031718},
	{SG64507, &UA3032927},
	{SG64508, &UA3032933},
	{SG64509, &UA3032944},
	{SG64510, &UA3032949},
	{SG64511, &UA3033504},
	{SG64512, &UA3033976},
	{SG64513, &UA3033977},
	{SG64514, &UA3033993},
	{SG64515, &UA3034127},
	{SG64516, &UA3034760},
	{SG64517, &UA3035716},
	{SG64518, &UA3035729},
	{SG64519, &UA3036112},
	{SG64520, &UA3036126},
	{SG64521, &UA3036131},
	{SG64522, &UA3059780},
	{SG64523, &UA3059925},
	{SG64524, &UA3059929},
	{SG64525, &UA3060844},
	{SG64526, &UA3061105},
	{SG64527, &UA3062781},
	{SG64528, &UA3079427},
	{SG64529, &UA3079659},
	{SG64530, &UA3079703},
	{SG64531, &UA3079833},
	{SG64532, &UA3080230},
	{SG64548, &UA347338},
	{SG64550, &UA347785},
	{SG64552, &UA347786},
	{SG64553, &UA351801},
	{SG64554, &UA351901},
	{SG64556, &UA366180},
	{SG64557, &UA366278},
	{SG64558, &UA368139},
	{SG64559, &UA368163},
	{SG64560, &UA710852},
	{SG64562, &UA2332597},
	{SG64563, &UA2339103},
	{SG64564, &UA2339853},
	{SG64565, &UA2674517},
	{SG64566, &UA2674610},
	{SG64567, &UA2718950},
	{SG64568, &UA924973},
	{SG64569, &UA925433},
	{SG64570, &UA2422609},
	{SG64571, &UA2423160},
	{SG64572, &UA611384},
	{SG64573, &UA2006325},
	{SG64574, &UA2200646},
	{SG64575, &UA2234199},
	{SG64576, &UA2360766},
	{SG64577, &UA2584142},
	{SG64578, &UA2796573},
	{SG64579, &UA2800466},
	{SG64580, &UA3094416},
	{SG64581, &UA3131148},
	{SG64582, &UA214630},
	{SG64583, &UA250157},
	{SG64584, &UA250761},
	{SG64585, &UA640778},
	{SG64586, &UA838983},
	{SG64587, &UA1980136},
	{SG64588, &UA2301089},
	{SG64589, &UA2333737},
	{SG64590, &UA2674670},
	{SG64591, &UA2674697},
	{SG64592, &UA2674707},
	{SG64593, &UA2674883},
	{SG64594, &UA2676613},
	{SG64595, &UA2715754},
	{SG64596, &UA2716836},
	{SG64597, &UA257425},
	{SG64598, &UA366503},
	{SG64599, &UA381228},
	{SG64600, &UA401375},
	{SG64601, &UA412898},
	{SG64602, &UA412973},
	{SG64603, &UA427658},
	{SG64604, &UA428576},
	{SG64605, &UA458303},
	{SG64606, &UA583467},
	{SG64607, &UA586914},
	{SG64608, &UA590572},
	{SG64637, &UA428277},
	{SG64638, &UA440790},
	{SG64639, &UA453713},
	{SG64640, &UA457629},
	{SG64641, &UA588774},
	{SG64642, &UA603604},
	{SG64643, &UA605657},
	{SG64644, &UA605773},
	{SG64645, &UA606766},
	{SG64646, &UA614133},
	{SG64647, &UA614921},
	{SG64648, &UA1994639},
	{SG64649, &UA457905},
	{SG64650, &UA607458},
	{SG64651, &UA615366},
	{SG64652, &UA688272},
	{SG64653, &UA881010},
	{SG64654, &UA897382},
	{SG64655, &UA897636},
	{SG64656, &UA901109},
	{SG64657, &UA901950},
	{SG64658, &UA904671},
	{SG64659, &UA347712},
	{SG64660, &UA359703},
	{SG64661, &UA366231},
	{SG64662, &UA366317},
	{SG64663, &UA687688},
	{SG64664, &UA912393},
	{SG64665, &UA2315638},
	{SG64666, &UA2315732},
	{SG64667, &UA2331547},
	{SG64668, &UA2421227},
	{SG64669, &UA2674510},
	{SG64670, &UA2674623},
	{SG64671, &UA2674795},
	{SG64672, &UA2675189},
	{SG64673, &UA2675207},
	{SG64674, &UA2714941},
	{SG64675, &UA2718189},
	{SG64686, &UA181069},
	{SG64687, &UA634790},
	{SG64688, &UA249611},
	{SG64689, &UA249794},
	{SG64690, &UA322347},
	{SG64691, &UA628042},
	{SG64692, &UA756856},
	{SG64693, &UA2203305},
	{SG64708, &UA603351},
	{SG64709, &UA610740},
	{SG64710, &UA623189},
	{SG64711, &UA749703},
	{SG64712, &UA897380},
	{SG64713, &UA897515},
	{SG64714, &UA901054},
	{SG64715, &UA901206},
	{SG64734, &UA401284},
	{SG64735, &UA401319},
	{SG64736, &UA401625},
	{SG64737, &UA412942},
	{SG64738, &UA426966},
	{SG64739, &UA427048},
	{SG64740, &UA666073},
	{SG64741, &UA865264},
	{SG64742, &UA2674856},
	{SG64743, &UA2736731},
	{SG64754, &UA366259},
	{SG64755, &UA366327},
	{SG64756, &UA900244},
	{SG64757, &UA900759},
	{SG64758, &UA2597175},
	{SG64777, &UA623495},
	{SG64778, &UA625694},
	{SG64779, &UA785480},
	{SG64781, &UA1959884},
	{SG64783, &UA1960722},
	{SG64784, &UA2836231},
	{SG64786, &UA220906},
	{SG64787, &UA224953},
	{SG64788, &UA351703},
	{SG64789, &UA366234},
	{SG64790, &UA368189},
	{SG64791, &UA583779},
	{SG64792, &UA1936775},
	{SG64793, &UA205718},
	{SG64794, &UA652294},
	{SG64795, &UA665234},
	{SG64796, &UA1887520},
	{SG64797, &UA2137638},
	{SG64798, &UA2139075},
	{SG64799, &UA2258982},
	{SG64822, &UA623140},
	{SG64823, &UA624969},
	{SG64824, &UA1968656},
	{SG64825, &UA2421839},
	{SG64826, &UA436211},
	{SG64827, &UA724802},
	{SG64828, &UA784362},
	{SG64829, &UA2589351},
	{SG64830, &UA2933591},
	{SG64831, &UA614980},
	{SG64832, &UA2002720},
	{SG64833, &UA2201425},
	{SG64834, &UA2381793},
	{SG64835, &UA2580104},
	{SG64836, &UA2695207},
	{SG64837, &UA2750182},
	{SG64838, &UA2797422},
	{SG64839, &UA3107582},
	{SG64847, &UA187413},
	{SG64848, &UA639395},
	{SG64849, &UA1876285},
	{SG64850, &UA252011},
	{SG64851, &UA252559},
	{SG64852, &UA252834},
	{SG64853, &UA255777},
	{SG64854, &UA2674508},
	{SG64855, &UA2674511},
	{SG64856, &UA2674524},
	{SG64857, &UA2674651},
	{SG64858, &UA2674864},
	{SG64859, &UA2714741},
	{SG64860, &UA2717979},
	{SG64861, &UA897367},
	{SG64862, &UA899976},
	{SG64863, &UA900115},
	{SG64864, &UA900161},
	{SG64865, &UA900278},
	{SG64866, &UA901555},
	{SG64867, &UA901648},
	{SG64868, &UA902329},
	{SG64869, &UA2674869},
	{SG64870, &UA2718867},
	{SG64871, &UA378899},
	{SG64872, &UA789669},
	{SG64873, &UA440746},
	{SG64874, &UA440826},
	{SG64875, &UA2260039},
	{SG64876, &UA2353563},
	{SG64877, &UA2375861},
	{SG64878, &UA2408168},
	{SG64879, &UA2430877},
	{SG64880, &UA2433270},
	{SG64881, &UA2470754},
	{SG64882, &UA2557262},
	{SG64883, &UA2600484},
	{SG64884, &UA2875379},
	{SG64885, &UA2945806},
	{SG64886, &UA3024380},
	{SG64922, &UA255559},
	{SG64923, &UA568834},
	{SG64924, &UA441216},
	{SG64925, &UA457608},
	{SG64926, &UA457786},
	{SG64927, &UA603889},
	{SG64928, &UA614960},
	{SG64929, &UA615021},
	{SG64930, &UA712220},
	{SG64931, &UA2222119},
	{SG64932, &UA2674674},
	{SG64933, &UA2717910},
	{SG64934, &UA2039853},
	{SG64935, &UA2164342},
	{SG64936, &UA2455849},
	{SG64937, &UA2526715},
	{SG64938, &UA903803},
	{SG64939, &UA904054},
	{SG64940, &UA352905},
	{SG64941, &UA366210},
	{SG64943, &UA366255},
	{SG64945, &UA368158},
	{SG64947, &UA368161},
	{SG64954, &UA638802},
	{SG64955, &UA330814},
	{SG64956, &UA335605},
	{SG64957, &UA336007},
	{SG64958, &UA337787},
	{SG64959, &UA738792},
	{SG64960, &UA2258818},
	{SG64961, &UA2258881},
	{SG64962, &UA2639169},
	{SG64963, &UA2774613},
	{SG64964, &UA1995015},
	{SG64965, &UA2153100},
	{SG64966, &UA2232677},
	{SG64967, &UA2359980},
	{SG64968, &UA2506325},
	{SG64969, &UA2558022},
	{SG64970, &UA2600698},
	{SG64971, &UA2708324},
	{SG64972, &UA2732522},
	{SG64973, &UA2752430},
	{SG64974, &UA3087384},
	{SG64975, &UA2039614},
	{SG64976, &UA2152645},
	{SG64977, &UA2367535},
	{SG64982, &UA682778},
	{SG64984, &UA2554849},
	{SG64988, &UA615568},
	{SG64989, &UA690855},
	{SG64990, &UA1954849},
	{SG64991, &UA2840654},
	{SG65013, &UA763325},
	{SG65014, &UA906684},
	{SG65015, &UA250073},
	{SG65016, &UA250091},
	{SG65017, &UA250172},
	{SG65018, &UA250275},
	{SG65019, &UA250295},
	{SG65020, &UA250441},
	{SG65021, &UA343727},
	{SG65022, &UA2258220},
	{SG65023, &UA2372038},
	{SG65024, &UA2674472},
	{SG65025, &UA2751811},
	{SG65026, &UA221057},
	{SG65027, &UA222605},
	{SG65028, &UA338236},
	{SG65029, &UA653338},
	{SG65030, &UA717279},
	{SG65031, &UA778501},
	{SG65032, &UA1897047},
	{SG65033, &UA2332663},
	{SG65034, &UA2674867},
	{SG65035, &UA2735836},
	{SG65036, &UA200583},
	{SG65037, &UA205313},
	{SG65038, &UA218643},
	{SG65039, &UA329217},
	{SG65040, &UA2290964},
	{SG65041, &UA2372241},
	{SG65055, &UA604635},
	{SG65056, &UA605027},
	{SG65057, &UA2315354},
	{SG65058, &UA2594510},
	{SG65069, &UA441461},
	{SG65070, &UA2519915},
	{SG65071, &UA222049},
	{SG65072, &UA232116},
	{SG65073, &UA2319595},
	{SG65074, &UA343871},
	{SG65075, &UA401196},
	{SG65076, &UA3087990},
	{SG65079, &UA224673},
	{SG65080, &UA250319},
	{SG65081, &UA1903498},
	{SG65082, &UA2374023},
	{SG65083, &UA401986},
	{SG65084, &UA2005691},
	{SG65085, &UA2199850},
	{SG65086, &UA2600228},
	{SG65087, &UA2797185},
	{SG65088, &UA3096185},
	{SG65089, &UA91053},
	{SG65090, &UA256008},
	{SG65091, &UA575365},
	{SG65092, &UA628220},
	{SG65093, &UA629764},
	{SG65094, &UA2146815},
	{SG65095, &UA2146967},
	{SG65096, &UA2462822},
	{SG65097, &UA2674528},
	{SG65098, &UA2718648},
	{SG65099, &UA347737},
	{SG65100, &UA351723},
	{SG65101, &UA352591},
	{SG65102, &UA353623},
	{SG65103, &UA366367},
	{SG65104, &UA380867},
	{SG65105, &UA587490},
	{SG65106, &UA693790},
	{SG65107, &UA2674814},
	{SG65108, &UA2715586},
	{SG65113, &UA216786},
	{SG65114, &UA218591},
	{SG65115, &UA219227},
	{SG65116, &UA326982},
	{SG65117, &UA327035},
	{SG65118, &UA641720},
	{SG65119, &UA668581},
	{SG65120, &UA715299},
	{SG65121, &UA2674801},
	{SG65122, &UA2707706},
	{SG65128, &UA359164},
	{SG65129, &UA398520},
	{SG65130, &UA401346},
	{SG65131, &UA401405},
	{SG65132, &UA412760},
	{SG65133, &UA412948},
	{SG65134, &UA413008},
	{SG65135, &UA427909},
	{SG65136, &UA427965},
	{SG65137, &UA428600},
	{SG65138, &UA428620},
	{SG65139, &UA458302},
	{SG65140, &UA603156},
	{SG65141, &UA923754},
	{SG65142, &UA192626},
	{SG65143, &UA195666},
	{SG65144, &UA316292},
	{SG65145, &UA316507},
	{SG65146, &UA536156},
	{SG65147, &UA539088},
	{SG65148, &UA678590},
	{SG65149, &UA685093},
	{SG65150, &UA903994},
	{SG65151, &UA2375762},
	{SG65152, &UA2674606},
	{SG65153, &UA2718646},
	{SG65154, &UA252152},
	{SG65155, &UA252190},
	{SG65156, &UA632992},
	{SG65157, &UA646377},
	{SG65158, &UA728667},
	{SG65159, &UA874345},
	{SG65160, &UA2294449},
	{SG65161, &UA2295372},
	{SG65162, &UA1876974},
	{SG65163, &UA1879568},
	{SG65164, &UA2301255},
	{SG65165, &UA2370816},
	{SG65166, &UA925405},
	{SG65167, &UA2017342},
	{SG65168, &UA2202826},
	{SG65169, &UA2556569},
	{SG65170, &UA2781348},
	{SG65171, &UA3091820},
	{SG65183, &UA663652},
	{SG65184, &UA2714748},
	{SG65216, &UA672510},
	{SG65217, &UA1986865},
	{SG65218, &UA2042872},
	{SG65219, &UA2202814},
	{SG65220, &UA2586038},
	{SG65221, &UA2800027},
	{SG65222, &UA3094880},
	{SG65224, &UA925389},
	{SG65225, &UA1996345},
	{SG65226, &UA2338946},
	{SG65227, &UA2553322},
	{SG65228, &UA603638},
	{SG65229, &UA603807},
	{SG65230, &UA3118007},
	{SG65252, &UA924966},
	{SG65254, &UA925054},
	{SG65256, &UA925399},
	{SG65258, &UA1996322},
	{SG65260, &UA1996355},
	{SG65262, &UA2176670},
	{SG65274, &UA366235},
	{SG65275, &UA834653},
	{SG65276, &UA900233},
	{SG65277, &UA1993666},
	{SG65278, &UA2156960},
	{SG65279, &UA412912},
	{SG65284, &UA412926},
	{SG65286, &UA440778},
	{SG65287, &UA440786},
	{SG65288, &UA457616},
	{SG65289, &UA457730},
	{SG65290, &UA603580},
	{SG65291, &UA603605},
	{SG65292, &UA605648},
	{SG65293, &UA605777},
	{SG65294, &UA614916},
	{SG65295, &UA614974},
	{SG65296, &UA205696},
	{SG65297, &UA210283},
	{SG65298, &UA229456},
	{SG65299, &UA629418},
	{SG65300, &UA682852},
	{SG65301, &UA2202447},
	{SG65302, &UA701628},
	{SG65303, &UA2002724},
	{SG65304, &UA2202489},
	{SG65305, &UA2602578},
	{SG65306, &UA2797560},
	{SG65307, &UA3114347},
	{SG65309, &UA401754},
	{SG65310, &UA409919},
	{SG65312, &UA924970},
	{SG65313, &UA924979},
	{SG65316, &UA628759},
	{SG65318, &UA634841},
	{SG65320, &UA660043},
	{SG65321, &UA769035},
	{SG65322, &UA780689},
	{SG65323, &UA806976},
	{SG65325, &UA878256},
	{SG65326, &UA2259061},
	{SG65327, &UA925401},
	{SG65328, &UA1996051},
	{SG65329, &UA1996260},
	{SG65330, &UA1996721},
	{SG65331, &UA1996835},
	{SG65332, &UA2167230},
	{SG65333, &UA2168773},
	{SG65334, &UA2226370},
	{SG65335, &UA2258143},
	{SG65336, &UA2597000},
	{SG65341, &UA181106},
	{SG65342, &UA311588},
	{SG65343, &UA543602},
	{SG65344, &UA674482},
	{SG65345, &UA767623},
	{SG65346, &UA770983},
	{SG65347, &UA780822},
	{SG65348, &UA2281076},
	{SG65355, &UA251541},
	{SG65356, &UA334967},
	{SG65357, &UA417638},
	{SG65358, &UA656641},
	{SG65359, &UA742779},
	{SG65360, &UA755800},
	{SG65361, &UA2201495},
	{SG65362, &UA2543702},
	{SG65367, &UA603636},
	{SG65368, &UA603712},
	{SG65369, &UA1882219},
	{SG65370, &UA2135355},
	{SG65371, &UA2259318},
	{SG65372, &UA2282658},
	{SG65373, &UA2284051},
	{SG65374, &UA2284348},
	{SG65375, &UA2286186},
	{SG65376, &UA1956764},
	{SG65389, &UA448772},
	{SG65390, &UA1882169},
	{SG65391, &UA250285},
	{SG65392, &UA250438},
	{SG65394, &UA881073},
	{SG65395, &UA250612},
	{SG65396, &UA345661},
	{SG65407, &UA925411},
	{SG65408, &UA925423},
	{SG65409, &UA428922},
	{SG65410, &UA841760},
	{SG65413, &UA574314},
	{SG65414, &UA2192559},
	{SG65415, &UA2192566},
	{SG65416, &UA2200377},
	{SG65417, &UA2235302},
	{SG65418, &UA2337241},
	{SG65419, &UA2349410},
	{SG65420, &UA2350035},
	{SG65421, &UA2366407},
	{SG65422, &UA2380765},
	{SG65423, &UA2382508},
	{SG65424, &UA2382748},
	{SG65425, &UA2383400},
	{SG65426, &UA2400870},
	{SG65427, &UA2420116},
	{SG65428, &UA2420325},
	{SG65429, &UA2421678},
	{SG65430, &UA2454632},
	{SG65431, &UA2455166},
	{SG65432, &UA2465690},
	{SG65433, &UA2480605},
	{SG65434, &UA2502228},
	{SG65435, &UA2502461},
	{SG65436, &UA2523851},
	{SG65437, &UA2524539},
	{SG65438, &UA2527368},
	{SG65439, &UA2554056},
	{SG65440, &UA2554339},
	{SG65441, &UA2593714},
	{SG65442, &UA2692406},
	{SG65443, &UA2693718},
	{SG65444, &UA2746590},
	{SG65445, &UA2777071},
	{SG65446, &UA2840109},
	{SG65447, &UA2844263},
	{SG65448, &UA3024911},
	{SG65449, &UA3025302},
	{SG65450, &UA3027062},
	{SG65451, &UA3027066},
	{SG65452, &UA3028089},
	{SG65453, &UA3029802},
	{SG65454, &UA3030443},
	{SG65455, &UA3030445},
	{SG65456, &UA3058562},
	{SG65457, &UA3058830},
	{SG65458, &UA3078907},
	{SG65459, &UA3078983},
	{SG65460, &UA3079063},
	{SG65461, &UA3092472},
	{SG65462, &UA3092527},
	{SG65463, &UA3107932},
	{SG65464, &UA3125202},
	{SG65465, &UA3125245},
	{SG65475, &UA252394},
	{SG65476, &UA255438},
	{SG65477, &UA2221446},
	{SG65478, &UA815465},
	{SG65479, &UA2473528},
	{SG65482, &UA898864},
	{SG65483, &UA899161},
	{SG65487, &UA441432},
	{SG65488, &UA1991685},
	{SG65509, &UA190970},
	{SG65510, &UA207092},
	{SG65511, &UA393651},
	{SG65512, &UA560377},
	{SG65513, &UA786140},
	{SG65514, &UA2137946},
	{SG65515, &UA2371349},
	{SG65519, &UA196236},
	{SG65521, &UA200324},
	{SG65522, &UA212848},
	{SG65523, &UA241358},
	{SG65524, &UA322014},
	{SG65525, &UA333348},
	{SG65526, &UA334937},
	{SG65527, &UA336892},
	{SG65528, &UA546783},
	{SG65529, &UA631757},
	{SG65530, &UA634401},
	{SG65531, &UA643176},
	{SG65532, &UA644722},
	{SG65533, &UA897458},
	{SG65534, &UA900303},
	{SG65535, &UA908709},
	{SG65536, &UA1887420},
	{SG65537, &UA1888699},
	{SG65538, &UA1994436},
	{SG65539, &UA2042126},
	{SG65540, &UA2201612},
	{SG65541, &UA2600218},
	{SG65542, &UA2873720},
	{SG65543, &UA3126673},
	{SG65557, &UA250094},
	{SG65558, &UA340058},
	{SG65559, &UA569879},
	{SG65560, &UA643216},
	{SG65561, &UA838649},
	{SG65562, &UA1895635},
	{SG65563, &UA2258833},
	{SG65564, &UA2803731},
	{SG65565, &UA218657},
	{SG65566, &UA323070},
	{SG65567, &UA334166},
	{SG65568, &UA435976},
	{SG65569, &UA451499},
	{SG65570, &UA651799},
	{SG65571, &UA752231},
	{SG65572, &UA912988},
	{SG65594, &UA901703},
	{SG65595, &UA901784},
	{SG65596, &UA197716},
	{SG65597, &UA390232},
	{SG65598, &UA2131054},
	{SG65599, &UA2275765},
	{SG65606, &UA899641},
	{SG65607, &UA899655},
	{SG65608, &UA216840},
	{SG65609, &UA218257},
	{SG65623, &UA362289},
	{SG65624, &UA362650},
	{SG65633, &UA1906254},
	{SG65634, &UA2002800},
	{SG65635, &UA2202284},
	{SG65636, &UA2599135},
	{SG65637, &UA2869900},
	{SG65638, &UA3113742},
	{SG65639, &UA925418},
	{SG65640, &UA1996404},
	{SG65641, &UA1996839},
	{SG65642, &UA1996843},
	{SG65650, &UA346833},
	{SG65651, &UA346892},
	{SG65652, &UA380989},
	{SG65653, &UA399090},
	{SG65654, &UA401410},
	{SG65655, &UA402486},
	{SG65656, &UA412713},
	{SG65657, &UA427689},
	{SG65658, &UA457620},
	{SG65659, &UA582365},
	{SG65660, &UA601311},
	{SG65661, &UA602270},
	{SG65662, &UA608464},
	{SG65663, &UA614155},
	{SG65664, &UA660091},
	{SG65665, &UA663122},
	{SG65666, &UA664544},
	{SG65667, &UA690551},
	{SG65668, &UA747322},
	{SG65669, &UA754407},
	{SG65670, &UA1943219},
	{SG65671, &UA2160073},
	{SG65676, &UA250380},
	{SG65677, &UA557993},
	{SG65703, &UA2769848},
	{SG65706, &UA401953},
	{SG65707, &UA833255},
	{SG65708, &UA252482},
	{SG65709, &UA252487},
	{SG65710, &UA428926},
	{SG65711, &UA795102},
	{SG65713, &UA243761},
	{SG65714, &UA243980},
	{SG65719, &UA738273},
	{SG65720, &UA760715},
	{SG65723, &UA252404},
	{SG65724, &UA252417},
	{SG65726, &UA2173376},
	{SG65729, &UA1973769},
	{SG65730, &UA2040402},
	{SG65731, &UA2204144},
	{SG65732, &UA2602528},
	{SG65733, &UA2797750},
	{SG65734, &UA3111042},
	{SG65735, &UA1986860},
	{SG65736, &UA2424578},
	{SG65737, &UA2564588},
	{SG65738, &UA2652641},
	{SG65743, &UA310280},
	{SG65744, &UA557600},
	{SG65745, &UA795521},
	{SG65749, &UA604001},
	{SG65750, &UA2158807},
	{SG65751, &UA2335211},
	{SG65830, &UA662546},
	{SG65832, &UA897393},
	{SG65833, &UA897522},
	{SG65834, &UA901924},
	{SG65846, &UA205702},
	{SG65847, &UA207224},
	{SG65849, &UA902685},
	{SG65850, &UA906588},
	{SG65851, &UA428900},
	{SG65852, &UA428923},
	{SG65853, &UA245176},
	{SG65854, &UA245196},
	{SG65855, &UA900117},
	{SG65856, &UA901423},
	{SG65857, &UA2440400},
	{SG65858, &UA2758198},
	{SG65871, &UA237668},
	{SG65872, &UA252420},
	{SG65873, &UA196548},
	{SG65874, &UA2293090},
	{SG65878, &UA615798},
	{SG65881, &UA603517},
	{SG65882, &UA682422},
	{SG65883, &UA828007},
	{SG65884, &UA835928},
	{SG65885, &UA864402},
	{SG65886, &UA897228},
	{SG65887, &UA899398},
	{SG65888, &UA1953784},
	{SG65889, &UA1956783},
	{SG65890, &UA1983788},
	{SG65891, &UA2152530},
	{SG65892, &UA2157091},
	{SG65893, &UA2334847},
	{SG65894, &UA2388521},
	{SG65895, &UA2404780},
	{SG65896, &UA2405116},
	{SG65897, &UA2423675},
	{SG65898, &UA2428136},
	{SG65899, &UA2467242},
	{SG65900, &UA2483297},
	{SG65901, &UA2554930},
	{SG65902, &UA2984359},
	{SG65903, &UA3085046},
	{SG65913, &UA925397},
	{SG65914, &UA2759427},
	{SG65920, &UA2933760},
	{SG65931, &UA899335},
	{SG65932, &UA1952214},
	{SG65933, &UA1957810},
	{SG65938, &UA255925},
	{SG65939, &UA345479},
	{SG65940, &UA347499},
	{SG65941, &UA353647},
	{SG65942, &UA366333},
	{SG65943, &UA368206},
	{SG65944, &UA919950},
	{SG65945, &UA2757915},
	{SG65946, &UA347403},
	{SG65947, &UA347460},
	{SG65948, &UA346815},
	{SG65949, &UA2216248},
	{SG65950, &UA2259221},
	{SG65951, &UA184241},
	{SG65952, &UA2160621},
	{SG65953, &UA2411131},
	{SG65955, &UA239633},
	{SG65956, &UA339436},
	{SG65957, &UA192937},
	{SG65958, &UA193884},
	{SG66046, &UA2711814},
	{SG66059, &UA2356481},
	{SG66060, &UA366271},
	{SG66061, &UA648783},
	{SG66062, &UA901433},
	{SG66063, &UA902434},
	{SG66067, &UA630598},
	{SG66068, &UA634929},
	{SG66069, &UA1999471},
	{SG66074, &UA362492},
	{SG66075, &UA398249},
	{SG66076, &UA923968},
	{SG66077, &UA1964671},
	{SG66078, &UA1986233},
	{SG66079, &UA1995968},
	{SG66080, &UA2156420},
	{SG66081, &UA2227552},
	{SG66082, &UA2235178},
	{SG66083, &UA2339798},
	{SG66084, &UA2374132},
	{SG66085, &UA2410394},
	{SG66086, &UA2722175},
	{SG66087, &UA2889282},
	{SG66090, &UA681535},
	{SG66091, &UA3042313},
	{SG66134, &UA2534835},
	{SG66431, &UA435358},
	{SG66433, &UA2137080},
	{SG66481, &UA627626},
	{SG66482, &UA2297134},
	{SG66483, &UA645674},
	{SG66484, &UA2334997},
	{SG66485, &UA2374295},
	{SG66487, &UA675620},
	{SG66488, &UA830482},
	{SG66489, &UA682544},
	{SG66490, &UA899345},
	{SG66491, &UA1993080},
	{SG66492, &UA2155006},
	{SG66493, &UA2164775},
	{SG66494, &UA2168157},
	{SG66495, &UA2222984},
	{SG66497, &UA2336897},
	{SG66499, &UA2589517},
	{SG66500, &UA2722642},
	{SG66501, &UA2815256},
	{SG66502, &UA2819510},
	{SG66503, &UA2830914},
	{SG66504, &UA3043161},
	{SG66505, &UA207154},
	{SG66506, &UA316380},
	{SG66507, &UA2338489},
	{SG66508, &UA244464},
	{SG66509, &UA573784},
	{SG66512, &UA615806},
	{SG66515, &UA252484},
	{SG66516, &UA1905986},
	{SG66517, &UA2231508},
	{SG66518, &UA2260059},
	{SG66519, &UA2386611},
	{SG66520, &UA2386896},
	{SG66523, &UA2006081},
	{SG66524, &UA2015101},
	{SG66525, &UA2451140},
	{SG66526, &UA1996516},
	{SG66527, &UA1997034},
	{SG66533, &UA243743},
	{SG66534, &UA813909},
	{SG66535, &UA251360},
	{SG66536, &UA409596},
	{SG66537, &UA668461},
	{SG66538, &UA817773},
	{SG66539, &UA2258782},
	{SG66540, &UA614149},
	{SG66541, &UA2528551},
	{SG66542, &UA2529845},
	{SG66543, &UA2737762},
	{SG66546, &UA402006},
	{SG66547, &UA428865},
	{SG66548, &UA704870},
	{SG66549, &UA819507},
	{SG66550, &UA2333383},
	{SG66551, &UA2444652},
	{SG66557, &UA898208},
	{SG66558, &UA2505499},
	{SG66559, &UA2555306},
	{SG66560, &UA2737757},
	{SG66574, &UA2772605},
	{SG66575, &UA1994663},
	{SG66576, &UA2179812},
	{SG66577, &UA2202648},
	{SG66578, &UA2585258},
	{SG66579, &UA2795532},
	{SG66580, &UA3093127},
	{SG66586, &UA627023},
	{SG66587, &UA668812},
	{SG66598, &UA184138},
	{SG66612, &UA367436},
	{SG66614, &UA367681},
	{SG66636, &UA205694},
	{SG66637, &UA211920},
	{SG66638, &UA553439},
	{SG66645, &UA604397},
	{SG66646, &UA607692},
	{SG66650, &UA796278},
	{SG66651, &UA2167693},
	{SG66652, &UA2748920},
	{SG66656, &UA2206706},
	{SG66661, &UA428799},
	{SG66669, &UA656180},
	{SG66670, &UA675301},
	{SG66671, &UA734388},
	{SG66672, &UA761517},
	{SG66673, &UA768062},
	{SG66674, &UA777707},
	{SG66675, &UA923941},
	{SG66676, &UA1994540},
	{SG66677, &UA2160064},
	{SG66678, &UA2221549},
	{SG66679, &UA2327151},
	{SG66680, &UA2545876},
	{SG66681, &UA381335},
	{SG66682, &UA401292},
	{SG66684, &UA428588},
	{SG66686, &UA428595},
	{SG66687, &UA458308},
	{SG66691, &UA190755},
	{SG66692, &UA317823},
	{SG66693, &UA2274567},
	{SG66694, &UA2275899},
	{SG66695, &UA2276638},
	{SG66696, &UA2486883},
	{SG66697, &UA2009051},
	{SG66698, &UA899439},
	{SG66699, &UA899533},
	{SG66705, &UA362692},
	{SG66706, &UA397474},
	{SG66707, &UA2769436},
	{SG66708, &UA193492},
	{SG66709, &UA241028},
	{SG66710, &UA717030},
	{SG66711, &UA380904},
	{SG66712, &UA648100},
	{SG66713, &UA651833},
	{SG66714, &UA791647},
	{SG66715, &UA829549},
	{SG66716, &UA848968},
	{SG66717, &UA896722},
	{SG66718, &UA1954831},
	{SG66719, &UA1962000},
	{SG66722, &UA255858},
	{SG66723, &UA350361},
	{SG66724, &UA353558},
	{SG66725, &UA366257},
	{SG66726, &UA367228},
	{SG66727, &UA368599},
	{SG66728, &UA380823},
	{SG66729, &UA381224},
	{SG66730, &UA411839},
	{SG66731, &UA427343},
	{SG66732, &UA428344},
	{SG66733, &UA428353},
	{SG66734, &UA428909},
	{SG66735, &UA590250},
	{SG66736, &UA823285},
	{SG66737, &UA1926662},
	{SG66738, &UA1935330},
	{SG66739, &UA2305429},
	{SG66740, &UA2441057},
	{SG66741, &UA240443},
	{SG66742, &UA247406},
	{SG66743, &UA249803},
	{SG66744, &UA250771},
	{SG66745, &UA329804},
	{SG66746, &UA334849},
	{SG66747, &UA336562},
	{SG66748, &UA340248},
	{SG66749, &UA622678},
	{SG66750, &UA660340},
	{SG66751, &UA689584},
	{SG66752, &UA760044},
	{SG66753, &UA878609},
	{SG66754, &UA880086},
	{SG66755, &UA885607},
	{SG66756, &UA891497},
	{SG66757, &UA1892840},
	{SG66758, &UA2139358},
	{SG66759, &UA2140276},
	{SG66760, &UA2140802},
	{SG66761, &UA2144894},
	{SG66762, &UA2259959},
	{SG66764, &UA2290864},
	{SG66765, &UA2291142},
	{SG66766, &UA2292124},
	{SG66767, &UA2293991},
	{SG66768, &UA2296601},
	{SG66770, &UA2316284},
	{SG66771, &UA2807604},
	{SG66772, &UA3042303},
	{SG66778, &UA183294},
	{SG66779, &UA312434},
	{SG66788, &UA347404},
	{SG66800, &UA2016145},
	{SG66803, &UA190772},
	{SG66804, &UA316640},
	{SG66807, &UA899410},
	{SG66808, &UA1989734},
	{SG66809, &UA2761905},
	{SG66810, &UA1965986},
	{SG66811, &UA2202871},
	{SG66812, &UA2518987},
	{SG66813, &UA2589754},
	{SG66815, &UA242862},
	{SG66816, &UA248414},
	{SG66817, &UA255980},
	{SG66818, &UA316966},
	{SG66819, &UA362624},
	{SG66820, &UA647167},
	{SG66821, &UA794514},
	{SG66822, &UA2259107},
	{SG66823, &UA2395204},
	{SG66824, &UA2783670},
	{SG66831, &UA901868},
	{SG66832, &UA2843045},
	{SG66833, &UA458609},
	{SG66836, &UA252414},
	{SG66837, &UA252490},
	{SG66838, &UA347332},
	{SG66839, &UA347732},
	{SG66842, &UA250208},
	{SG66843, &UA250249},
	{SG66844, &UA366440},
	{SG66856, &UA622902},
	{SG66857, &UA638504},
	{SG66858, &UA227022},
	{SG66859, &UA227465},
	{SG66865, &UA250075},
	{SG66866, &UA341219},
	{SG66867, &UA342987},
	{SG66868, &UA573789},
	{SG66869, &UA645780},
	{SG66870, &UA872947},
	{SG66871, &UA2294898},
	{SG66872, &UA227577},
	{SG66873, &UA237438},
	{SG66874, &UA614936},
	{SG66883, &UA190700},
	{SG66885, &UA196304},
	{SG66886, &UA205693},
	{SG66899, &UA186889},
	{SG66906, &UA347147},
	{SG66907, &UA682719},
	{SG66908, &UA603199},
	{SG66909, &UA605908},
	{SG66910, &UA615465},
	{SG66911, &UA818770},
	{SG66912, &UA897920},
	{SG66913, &UA899654},
	{SG66914, &UA901756},
	{SG66915, &UA903910},
	{SG66923, &UA607854},
	{SG66924, &UA608851},
	{SG66926, &UA2565788},
	{SG66929, &UA367643},
	{SG66940, &UA226935},
	{SG66941, &UA2370621},
	{SG66947, &UA249262},
	{SG66950, &UA904601},
	{SG66951, &UA623392},
	{SG66952, &UA191445},
	{SG66958, &UA244926},
	{SG66959, &UA342113},
	{SG66966, &UA637208},
	{SG66967, &UA694464},
	{SG66968, &UA336257},
	{SG66969, &UA377583},
	{SG66973, &UA236474},
	{SG66974, &UA236671},
	{SG66975, &UA401274},
	{SG66976, &UA590210},
	{SG66979, &UA606261},
	{SG66980, &UA606560},
	{SG66981, &UA251354},
	{SG66982, &UA251358},
	{SG66983, &UA241414},
	{SG66984, &UA241779},
	{SG66990, &UA428930},
	{SG66991, &UA428931},
	{SG66994, &UA2879558},
	{SG66995, &UA453201},
	{SG66996, &UA586881},
	{SG66997, &UA704787},
	{SG66998, &UA2230576},
	{SG67004, &UA902690},
	{SG67005, &UA1970488},
	{SG67006, &UA2167456},
	{SG67007, &UA631656},
	{SG67010, &UA193427},
	{SG67011, &UA320285},
	{SG67012, &UA347614},
	{SG67014, &UA352177},
	{SG67015, &UA904004},
	{SG67016, &UA2156226},
	{SG67017, &UA2457494},
	{SG67022, &UA612653},
	{SG67023, &UA664568},
	{SG67024, &UA2231988},
	{SG67028, &UA427490},
	{SG67029, &UA632729},
	{SG67030, &UA1986603},
	{SG67031, &UA2204372},
	{SG67041, &UA886416},
	{SG67042, &UA886419},
	{SG67043, &UA380133},
	{SG67044, &UA2394742},
	{SG67057, &UA190761},
	{SG67058, &UA2138097},
	{SG67059, &UA612060},
	{SG67060, &UA924024},
	{SG67061, &UA1999491},
	{SG67062, &UA2195935},
	{SG67063, &UA2669343},
	{SG67064, &UA3003071},
	{SG67065, &UA1993347},
	{SG67066, &UA2652980},
	{SG67068, &UA427394},
	{SG67069, &UA250142},
	{SG67071, &UA237625},
	{SG67072, &UA900168},
	{SG67073, &UA901397},
	{SG67075, &UA422723},
	{SG67076, &UA422786},
	{SG67088, &UA348688},
	{SG67089, &UA349877},
	{SG67092, &UA631204},
	{SG67094, &UA3036129},
	{SG67095, &UA3061099},
	{SG67096, &UA252236},
	{SG67097, &UA314669},
	{SG67098, &UA332364},
	{SG67099, &UA337086},
	{SG67100, &UA635397},
	{SG67101, &UA670837},
	{SG67102, &UA673993},
	{SG67103, &UA901954},
	{SG67104, &UA236458},
	{SG67105, &UA606464},
	{SG67106, &UA899663},
	{SG67107, &UA901400},
	{SG67108, &UA234707},
	{SG67109, &UA234728},
	{SG67110, &UA219787},
	{SG67111, &UA222292},
	{SG67112, &UA368205},
	{SG67113, &UA368204},
	{SG67114, &UA250315},
	{SG67115, &UA250322},
	{SG67117, &UA244647},
	{SG67118, &UA252830},
	{SG67119, &UA348757},
	{SG67120, &UA589615},
	{SG67121, &UA630337},
	{SG67122, &UA640077},
	{SG67123, &UA915820},
	{SG67124, &UA1915655},
	{SG67125, &UA2134383},
	{SG67126, &UA336028},
	{SG67127, &UA366430},
	{SG67132, &UA312878},
	{SG67133, &UA347333},
	{SG67179, &UA605283},
	{SG67180, &UA772276},
	{SG67182, &UA615278},
	{SG67183, &UA615348},
	{SG67184, &UA626154},
	{SG67185, &UA242088},
	{SG67194, &UA428932},
	{SG67195, &UA2330173},
	{SG67198, &UA2161012},
	{SG67199, &UA441072},
	{SG67200, &UA441254},
	{SG67209, &UA181827},
	{SG67213, &UA1993469},
	{SG67214, &UA2178073},
	{SG67215, &UA2236939},
	{SG67216, &UA2708215},
	{SG67217, &UA2939104},
	{SG67218, &UA316554},
	{SG67219, &UA1956973},
	{SG67220, &UA2335600},
	{SG67221, &UA243895},
	{SG67222, &UA244638},
	{SG67223, &UA631736},
	{SG67224, &UA677646},
	{SG67225, &UA886752},
	{SG67226, &UA251365},
	{SG67227, &UA190363},
	{SG67230, &UA250513},
	{SG67231, &UA409844},
	{SG67232, &UA234844},
	{SG67233, &UA615223},
	{SG67234, &UA899739},
	{SG67241, &UA2780111},
	{SG67242, &UA2863582},
	{SG67243, &UA2937958},
	{SG67244, &UA182192},
	{SG67245, &UA700574},
	{SG67246, &UA821457},
	{SG67247, &UA2321240},
	{SG67248, &UA1958838},
	{SG67249, &UA2235576},
	{SG67250, &UA2750250},
	{SG67251, &UA3001525},
	{SG67252, &UA2445106},
	{SG67253, &UA2972241},
	{SG67254, &UA2561868},
	{SG67255, &UA3072664},
	{SG67256, &UA695525},
	{SG67257, &UA820579},
	{SG67258, &UA366523},
	{SG67259, &UA368133},
	{SG67260, &UA635465},
	{SG67261, &UA397645},
	{SG67262, &UA397801},
	{SG67264, &UA441281},
	{SG67265, &UA706722},
	{SG67266, &UA779143},
	{SG67271, &UA250314},
	{SG67272, &UA250474},
	{SG67274, &UA368215},
	{SG67275, &UA903524},
	{SG67276, &UA2750760},
	{SG67278, &UA255549},
	{SG67279, &UA367651},
	{SG67280, &UA368357},
	{SG67281, &UA401361},
	{SG67287, &UA2540502},
	{SG67291, &UA250344},
	{SG67292, &UA2766317},
	{SG67293, &UA199775},
	{SG67294, &UA316877},
	{SG67295, &UA2737703},
	{SG67297, &UA2659225},
	{SG67298, &UA2367115},
	{SG67306, &UA245100},
	{SG67307, &UA250642},
	{SG67308, &UA252415},
	{SG67309, &UA252984},
	{SG67310, &UA255759},
	{SG67311, &UA629175},
	{SG67312, &UA640287},
	{SG67313, &UA1901644},
	{SG67316, &UA2149249},
	{SG67317, &UA2494130},
	{SG67318, &UA2332553},
	{SG67319, &UA2350719},
	{SG67320, &UA2794506},
	{SG67321, &UA3014954},
	{SG67327, &UA327811},
	{SG67329, &UA252423},
	{SG67330, &UA252483},
	{SG67332, &UA348430},
	{SG67333, &UA367631},
	{SG67334, &UA2312804},
	{SG67335, &UA367732},
	{SG67339, &UA440745},
	{SG67340, &UA440954},
	{SG67341, &UA457675},
	{SG67342, &UA457719},
	{SG67343, &UA606043},
	{SG67344, &UA614920},
	{SG67345, &UA614986},
	{SG67346, &UA2568187},
	{SG67347, &UA2669306},
	{SG67348, &UA2706370},
	{SG67351, &UA349061},
	{SG67352, &UA422338},
	{SG67353, &UA851239},
	{SG67354, &UA2401042},
	{SG67355, &UA347242},
	{SG67356, &UA351620},
	{SG67359, &UA684677},
	{SG67360, &UA606861},
	{SG67361, &UA250299},
	{SG67362, &UA255981},
	{SG67363, &UA342023},
	{SG67364, &UA398670},
	{SG67365, &UA638307},
	{SG67366, &UA872445},
	{SG67367, &UA895539},
	{SG67368, &UA915876},
	{SG67369, &UA2365520},
	{SG67370, &UA2749844},
	{SG67374, &UA251042},
	{SG67375, &UA709619},
	{SG67376, &UA745781},
	{SG67377, &UA2350878},
	{SG67380, &UA2360979},
	{SG67382, &UA567390},
	{SG67383, &UA771145},
	{SG67385, &UA890790},
	{SG67386, &UA2142093},
	{SG67387, &UA454786},
	{SG67388, &UA591408},
	{SG67425, &UA874606},
	{SG67426, &UA237873},
	{SG67427, &UA353643},
	{SG67433, &UA401257},
	{SG67434, &UA590233},
	{SG67437, &UA192630},
	{SG67438, &UA316970},
	{SG67439, &UA428866},
	{SG67445, &UA251465},
	{SG67446, &UA718253},
	{SG67447, &UA839772},
	{SG67448, &UA2383477},
	{SG67449, &UA252704},
	{SG67450, &UA670602},
	{SG67451, &UA915865},
	{SG67452, &UA1897500},
	{SG67455, &UA226449},
	{SG67456, &UA234520},
	{SG67461, &UA2727789},
	{SG67462, &UA2983671},
	{SG67466, &UA2454318},
	{SG67467, &UA2764137},
	{SG67468, &UA3001765},
	{SG67469, &UA2526774},
	{SG67470, &UA896004},
	{SG67471, &UA2360009},
	{SG67472, &UA2413150},
	{SG67473, &UA2684837},
	{SG67475, &UA243947},
	{SG67476, &UA1978904},
	{SG67480, &UA663048},
	{SG67481, &UA709701},
	{SG67482, &UA2933373},
	{SG67485, &UA689566},
	{SG67496, &UA584980},
	{SG67498, &UA2662072},
	{SG67499, &UA3124293},
	{SG67500, &UA729580},
	{SG67501, &UA606801},
	{SG67502, &UA607195},
	{SG67503, &UA854570},
	{SG67504, &UA2401445},
	{SG67505, &UA3009343},
	{SG67506, &UA255053},
	{SG67509, &UA364090},
	{SG67512, &UA428934},
	{SG67513, &UA822981},
	{SG67514, &UA2998298},
	{SG67517, &UA331789},
	{SG67518, &UA329709},
	{SG67519, &UA451753},
	{SG67521, &UA912731},
	{SG67522, &UA2162745},
	{SG67523, &UA2682089},
	{SG67524, &UA2981106},
	{SG67525, &UA251362},
	{SG67526, &UA421726},
	{SG67527, &UA363581},
	{SG67528, &UA2494005},
	{SG67529, &UA2639867},
	{SG67530, &UA2578412},
	{SG67531, &UA2301658},
	{SG67532, &UA2786064},
	{SG67538, &UA812886},
	{SG67539, &UA422890},
	{SG67540, &UA437545},
	{SG67542, &UA2554039},
	{SG67546, &UA674109},
	{SG67547, &UA792052},
	{SG67548, &UA615525},
	{SG67549, &UA615526},
	{SG67550, &UA2162157},
	{SG67553, &UA2454401},
	{SG67556, &UA447507},
	{SG67559, &UA2469187},
	{SG67560, &UA2469471},
	{SG67570, &UA1889275},
	{SG67571, &UA899725},
	{SG67572, &UA899746},
	{SG67573, &UA3059932},
	{SG67576, &UA901976},
	{SG67577, &UA902229},
	{SG67579, &UA2553372},
	{SG67580, &UA3050673},
	{SG67581, &UA250456},
	{SG67582, &UA1904165},
	{SG67583, &UA350743},
	{SG67588, &UA252702},
	{SG67589, &UA641449},
	{SG67590, &UA1943872},
	{SG67591, &UA2147338},
	{SG67592, &UA2258328},
	{SG67593, &UA2982288},
	{SG67594, &UA2526575},
	{SG67596, &UA2987751},
	{SG67597, &UA2995762},
	{SG67641, &UA338914},
	{SG67642, &UA725046},
	{SG67643, &UA368150},
	{SG67644, &UA368178},
	{SG67645, &UA441258},
	{SG67646, &UA441471},
	{SG67651, &UA325867},
	{SG67652, &UA2642182},
	{SG67654, &UA2721629},
	{SG67661, &UA2999282},
	{SG67666, &UA887599},
	{SG67667, &UA889818},
	{SG67668, &UA2439376},
	{SG67670, &UA242051},
	{SG67671, &UA435327},
	{SG67672, &UA1877966},
	{SG67673, &UA2825349},
	{SG67676, &UA336338},
	{SG67677, &UA337563},
	{SG67678, &UA710127},
	{SG67679, &UA2556526},
	{SG67687, &UA560323},
	{SG67688, &UA2259968},
	{SG67689, &UA2548058},
	{SG67692, &UA3069627},
	{SG67693, &UA3117201},
	{SG67703, &UA250135},
	{SG67704, &UA1902277},
	{SG67705, &UA334454},
	{SG67706, &UA566224},
	{SG67711, &UA2258512},
	{SG67713, &UA904151},
	{SG67714, &UA906828},
	{SG67718, &UA2696179},
	{SG67726, &UA1890061},
	{SG67727, &UA2139417},
	{SG67728, &UA2288685},
	{SG67729, &UA2290844},
	{SG67730, &UA2420756},
	{SG67731, &UA2550839},
	{SG67732, &UA2638745},
	{SG67735, &UA236504},
	{SG67736, &UA238539},
	{SG67737, &UA339364},
	{SG67738, &UA341494},
	{SG67739, &UA675130},
	{SG67740, &UA1860261},
	{SG67741, &UA675212},
	{SG67745, &UA2861284},
	{SG67746, &UA203956},
	{SG67748, &UA315846},
	{SG67753, &UA3004467},
	{SG67758, &UA380763},
	{SG67770, &UA1957453},
	{SG67772, &UA2519881},
	{SG67775, &UA2997571},
	{SG67776, &UA337631},
	{SG67777, &UA1897364},
	{SG67782, &UA343262},
	{SG67783, &UA343291},
	{SG67784, &UA637378},
	{SG67785, &UA316581},
	{SG67790, &UA2763886},
	{SG67794, &UA3041525},
	{SG67796, &UA341244},
	{SG67797, &UA347259},
	{SG67798, &UA2260205},
	{SG67799, &UA2285781},
	{SG67824, &UA341393},
	{SG67826, &UA3096433},
	{SG67829, &UA2726973},
	{SG67835, &UA3073349},
	{SG67850, &UA380910},
	{SG67851, &UA1950830},
	{SG67854, &UA891191},
	{SG67855, &UA893329},
	{SG67857, &UA368179},
	{SG67858, &UA454045},
	{SG67859, &UA454215},
	{SG67860, &UA869564},
	{SG67862, &UA1983687},
	{SG67863, &UA2152364},
	{SG67864, &UA2161886},
	{SG67865, &UA2710152},
	{SG67870, &UA856317},
	{SG67871, &UA2260026},
	{SG67875, &UA3044777},
	{SG67876, &UA3048545},
	{SG67877, &UA607405},
	{SG67878, &UA608703},
	{SG67879, &UA2259984},
	{SG67880, &UA2296434},
	{SG67881, &UA2652935},
	{SG67882, &UA2734202},
	{SG67889, &UA625252},
	{SG67890, &UA2280237},
	{SG67891, &UA2286508},
	{SG67894, &UA3041163},
	{SG67896, &UA646674},
	{SG67897, &UA2454084},
	{SG67903, &UA2258707},
	{SG67906, &UA3127324},
	{SG67907, &UA615622},
	{SG67908, &UA2165289},
	{SG67909, &UA1986682},
	{SG67910, &UA2331180},
	{SG67911, &UA2260068},
	{SG67912, &UA2260001},
	{SG67913, &UA2338620},
	{SG67914, &UA2442852},
	{SG67915, &UA2463484},
	{SG67916, &UA2832481},
	{SG67919, &UA515266},
	{SG67920, &UA1868088},
	{SG67922, &UA241662},
	{SG67923, &UA804648},
	{SG67924, &UA831826},
	{SG67925, &UA899829},
	{SG67926, &UA899838},
	{SG67927, &UA1995840},
	{SG67928, &UA597628},
	{SG67929, &UA1972710},
	{SG67930, &UA616106},
	{SG67931, &UA813304},
	{SG67932, &UA1899617},
	{SG67933, &UA2141048},
	{SG67934, &UA251626},
	{SG67941, &UA2141462},
	{SG67942, &UA2301740},
	{SG67943, &UA2299443},
	{SG67945, &UA2330127},
	{SG67946, &UA251531},
	{SG67947, &UA350610},
	{SG67948, &UA622735},
	{SG67949, &UA740578},
	{SG67950, &UA2159662},
	{SG67952, &UA2314841},
	{SG67953, &UA245046},
	{SG67954, &UA342185},
	{SG67955, &UA398519},
	{SG67956, &UA923739},
	{SG67957, &UA864898},
	{SG67958, &UA2318193},
	{SG67959, &UA361777},
	{SG67960, &UA380771},
	{SG67968, &UA1946229},
	{SG67969, &UA251556},
	{SG67973, &UA616425},
	{SG67978, &UA79925},
	{SG67979, &UA79934},
	{SG67980, &UA79943},
	{SG67981, &UA79952},
	{SG67982, &UA79961},
	{SG67983, &UA79970},
	{NULL}};
const Segmentdata DChrome20_29[] = {
	{SG61356, &UA2539646},
	{SG61357, &UA2566665},
	{SG61360, &UA2567735},
	{SG61361, &UA2571020},
	{SG61362, &UA2580143},
	{SG61363, &UA2585525},
	{SG61364, &UA2602786},
	{SG61365, &UA2605299},
	{SG61366, &UA2672640},
	{SG61367, &UA2673310},
	{SG61368, &UA2675656},
	{SG61369, &UA2678533},
	{SG61372, &UA2998257},
	{SG61373, &UA2675845},
	{SG61374, &UA2678970},
	{SG61375, &UA2679180},
	{SG61376, &UA2715222},
	{SG61377, &UA2717948},
	{SG61378, &UA2719034},
	{SG61379, &UA2740625},
	{SG61380, &UA2753239},
	{SG61381, &UA2756850},
	{SG61382, &UA2756948},
	{SG61383, &UA2774226},
	{SG61384, &UA2783144},
	{SG61385, &UA2799887},
	{SG61386, &UA2873739},
	{SG61387, &UA2875857},
	{SG61388, &UA2880145},
	{SG61487, &UA2754610},
	{SG61488, &UA2754613},
	{SG61489, &UA2774360},
	{SG61490, &UA2774361},
	{SG61491, &UA2873606},
	{SG61492, &UA2873637},
	{SG61493, &UA2874343},
	{SG61494, &UA2874813},
	{SG61495, &UA2875505},
	{SG61496, &UA2876734},
	{SG61497, &UA2877025},
	{SG61498, &UA2877026},
	{SG61499, &UA2901775},
	{SG61500, &UA2940886},
	{SG61501, &UA3056126},
	{SG61502, &UA3092239},
	{SG61503, &UA3093716},
	{SG61504, &UA3094314},
	{SG61505, &UA2754081},
	{SG61506, &UA2773262},
	{SG61507, &UA2784740},
	{SG61508, &UA2797726},
	{SG61509, &UA2877275},
	{SG61510, &UA2888690},
	{SG61511, &UA2897664},
	{SG61512, &UA2898331},
	{SG61513, &UA2900938},
	{SG61514, &UA2945258},
	{SG61515, &UA2948433},
	{SG61516, &UA2964766},
	{SG61517, &UA2970602},
	{SG61518, &UA2975920},
	{SG61519, &UA3024746},
	{SG61520, &UA3026271},
	{SG61523, &UA3029372},
	{SG61524, &UA2193071},
	{SG61525, &UA2201510},
	{SG61526, &UA2232852},
	{SG61527, &UA2236557},
	{SG61528, &UA2238021},
	{SG61529, &UA2258846},
	{SG61530, &UA2357636},
	{SG61532, &UA2357906},
	{SG61533, &UA2367345},
	{SG61534, &UA2387965},
	{SG61535, &UA2388470},
	{SG61536, &UA2389904},
	{SG61537, &UA2390255},
	{SG61538, &UA2408308},
	{SG61539, &UA2433760},
	{SG61540, &UA2433879},
	{SG61541, &UA2458252},
	{SG61542, &UA2864706},
	{SG61543, &UA2973133},
	{SG61544, &UA3024738},
	{SG61545, &UA3024740},
	{SG61546, &UA3026215},
	{SG61547, &UA3026282},
	{SG61548, &UA3026557},
	{SG61549, &UA3027645},
	{SG61550, &UA3028067},
	{SG61551, &UA3028080},
	{SG61552, &UA3028312},
	{SG61553, &UA3028709},
	{SG61554, &UA3029778},
	{SG61555, &UA3030403},
	{SG61556, &UA3033341},
	{SG61557, &UA3062244},
	{SG61558, &UA3126698},
	{SG61559, &UA2567774},
	{SG61560, &UA2567781},
	{SG61561, &UA2567790},
	{SG61562, &UA2567792},
	{SG61563, &UA2567974},
	{SG61564, &UA2567985},
	{SG61565, &UA2568530},
	{SG61566, &UA2569395},
	{SG61567, &UA2569742},
	{SG61568, &UA2569752},
	{SG61569, &UA2584211},
	{SG61570, &UA2606473},
	{SG61571, &UA2674496},
	{SG61572, &UA2700858},
	{SG61573, &UA2708549},
	{SG61574, &UA3020400},
	{SG61575, &UA3024014},
	{SG61576, &UA3025937},
	{SG61577, &UA3058318},
	{SG61578, &UA3093214},
	{SG61579, &UA2191137},
	{SG61580, &UA2201785},
	{SG61581, &UA2387457},
	{SG61582, &UA2387455},
	{SG61583, &UA2406237},
	{SG61584, &UA2406378},
	{SG61585, &UA2408288},
	{SG61586, &UA2408369},
	{SG61587, &UA2408395},
	{SG61588, &UA2408447},
	{SG61589, &UA2427450},
	{SG61590, &UA2428543},
	{SG61591, &UA2429892},
	{SG61592, &UA2432590},
	{SG61593, &UA2433533},
	{SG61594, &UA2436326},
	{SG61595, &UA2437186},
	{SG61596, &UA2437185},
	{SG61597, &UA2457181},
	{SG61598, &UA2468450},
	{SG61599, &UA2540611},
	{SG61600, &UA2540630},
	{SG61601, &UA2747438},
	{SG61602, &UA2854685},
	{SG61603, &UA2352718},
	{SG61604, &UA2358156},
	{SG61605, &UA2363163},
	{SG61606, &UA2390892},
	{SG61607, &UA2390925},
	{SG61608, &UA2408108},
	{SG61609, &UA2408180},
	{SG61610, &UA2430546},
	{SG61611, &UA2435761},
	{SG61612, &UA2461916},
	{SG61613, &UA2462199},
	{SG61614, &UA2470872},
	{SG61615, &UA2488069},
	{SG61616, &UA2488941},
	{SG61617, &UA2492228},
	{SG61618, &UA2509046},
	{SG61619, &UA2512010},
	{SG61620, &UA2700573},
	{SG61621, &UA2850376},
	{SG61622, &UA2259377},
	{SG61623, &UA2471951},
	{SG61624, &UA2471956},
	{SG61625, &UA2487696},
	{SG61626, &UA2488093},
	{SG61627, &UA2492110},
	{SG61628, &UA2492150},
	{SG61629, &UA2492279},
	{SG61630, &UA2492341},
	{SG61631, &UA2492375},
	{SG61632, &UA2492462},
	{SG61633, &UA2508391},
	{SG61634, &UA2535014},
	{SG61635, &UA2540554},
	{SG61636, &UA2540608},
	{SG61637, &UA2599747},
	{SG61638, &UA2666664},
	{SG61639, &UA2713280},
	{SG61640, &UA2737939},
	{SG61641, &UA3064093},
	{SG61642, &UA3067541},
	{SG61643, &UA3067630},
	{SG61644, &UA3067634},
	{SG61645, &UA3067981},
	{SG61646, &UA3068128},
	{SG61647, &UA3068229},
	{SG61648, &UA3068244},
	{SG61649, &UA3082195},
	{SG61650, &UA3082208},
	{SG61651, &UA3094274},
	{SG61652, &UA3095637},
	{SG61653, &UA3114448},
	{SG61654, &UA2673796},
	{SG61655, &UA2676125},
	{SG61656, &UA2676569},
	{SG61657, &UA2678354},
	{SG61658, &UA2678359},
	{SG61659, &UA2678386},
	{SG61660, &UA2678391},
	{SG61661, &UA2678541},
	{SG61662, &UA2678621},
	{SG61663, &UA2679926},
	{SG61664, &UA2679929},
	{SG61665, &UA2715878},
	{SG61666, &UA2752512},
	{SG61667, &UA2996522},
	{SG61668, &UA3005275},
	{SG61669, &UA3044643},
	{SG61670, &UA3047829},
	{SG61671, &UA3088225},
	{SG61672, &UA2368157},
	{SG61673, &UA2461172},
	{SG61674, &UA2470718},
	{SG61675, &UA2471082},
	{SG61676, &UA2490957},
	{SG61677, &UA2492403},
	{SG61678, &UA2509584},
	{SG61679, &UA2512829},
	{SG61680, &UA2538362},
	{SG61681, &UA2538377},
	{SG61682, &UA2567794},
	{SG61683, &UA2568515},
	{SG61684, &UA2568678},
	{SG61685, &UA2569461},
	{SG61686, &UA2570045},
	{SG61687, &UA2571175},
	{SG61688, &UA2584382},
	{SG61689, &UA2857316},
	{SG61690, &UA2203723},
	{SG61691, &UA2203726},
	{SG61692, &UA2203876},
	{SG61693, &UA2203983},
	{SG61694, &UA2204447},
	{SG61695, &UA2231187},
	{SG61696, &UA2234775},
	{SG61697, &UA2234776},
	{SG61698, &UA2238473},
	{SG61699, &UA2354732},
	{SG61700, &UA2368280},
	{SG61701, &UA2369702},
	{SG61702, &UA2516475},
	{SG61703, &UA2516759},
	{SG61704, &UA2535790},
	{SG61705, &UA2538176},
	{SG61706, &UA2564640},
	{SG61707, &UA2570075},
	{SG61708, &UA2718559},
	{SG61709, &UA2718793},
	{SG61710, &UA2972550},
	{SG61711, &UA2973989},
	{SG61712, &UA3022659},
	{SG61713, &UA3023275},
	{SG61714, &UA3026699},
	{SG61715, &UA3027954},
	{SG61716, &UA3031455},
	{SG61717, &UA3033164},
	{SG61718, &UA3034436},
	{SG61719, &UA3036664},
	{SG61720, &UA3062176},
	{SG61721, &UA3063754},
	{SG61722, &UA3067802},
	{SG61723, &UA3067811},
	{SG61724, &UA3096638},
	{SG61725, &UA3096646},
	{SG61727, &UA2753618},
	{SG61728, &UA2770625},
	{SG61729, &UA2773265},
	{SG61730, &UA2774046},
	{SG61731, &UA2784873},
	{SG61732, &UA2796701},
	{SG61733, &UA2797617},
	{SG61734, &UA2875297},
	{SG61735, &UA2877555},
	{SG61736, &UA2877655},
	{SG61737, &UA2885426},
	{SG61738, &UA2898056},
	{SG61739, &UA2899357},
	{SG61740, &UA2899358},
	{SG61741, &UA2948143},
	{SG61742, &UA2948500},
	{SG61743, &UA3025914},
	{SG61744, &UA3026217},
	{SG61761, &UA2675270},
	{SG61762, &UA2675365},
	{SG61763, &UA2678739},
	{SG61764, &UA2680856},
	{SG61765, &UA2707659},
	{SG61766, &UA2714475},
	{SG61767, &UA2715648},
	{SG61768, &UA2718664},
	{SG61769, &UA2719038},
	{SG61770, &UA2738369},
	{SG61771, &UA2740088},
	{SG61772, &UA2741586},
	{SG61773, &UA2741687},
	{SG61774, &UA2756972},
	{SG61775, &UA2757025},
	{SG61776, &UA2774237},
	{SG61777, &UA2774255},
	{SG61778, &UA2860949},
	{SG61779, &UA2873854},
	{SG61780, &UA2874062},
	{SG61781, &UA2487839},
	{SG61782, &UA2567171},
	{SG61783, &UA2568010},
	{SG61784, &UA2583605},
	{SG61785, &UA2600174},
	{SG61786, &UA2601363},
	{SG61787, &UA2602790},
	{SG61788, &UA2678406},
	{SG61789, &UA2678827},
	{SG61790, &UA2857677},
	{SG61831, &UA2015623},
	{SG61832, &UA2015624},
	{SG61833, &UA2040558},
	{SG61834, &UA2041086},
	{SG61835, &UA2041372},
	{SG61836, &UA2041730},
	{SG61837, &UA2191014},
	{SG61838, &UA2230758},
	{SG61839, &UA2233997},
	{SG61840, &UA2349259},
	{SG61841, &UA2551442},
	{SG61842, &UA2603216},
	{SG61843, &UA2843800},
	{SG61844, &UA3101945},
	{SG61845, &UA2756420},
	{SG61846, &UA2774359},
	{SG61847, &UA2800611},
	{SG61848, &UA2873610},
	{SG61849, &UA2873781},
	{SG61850, &UA2873936},
	{SG61851, &UA2874218},
	{SG61852, &UA2876292},
	{SG61853, &UA2876777},
	{SG61854, &UA2877028},
	{SG61855, &UA2893050},
	{SG61856, &UA2938463},
	{SG61857, &UA2958465},
	{SG61858, &UA2968905},
	{SG61859, &UA3002706},
	{SG61860, &UA3052621},
	{SG61861, &UA3080873},
	{SG61863, &UA3062784},
	{SG61864, &UA3062789},
	{SG61865, &UA3062793},
	{SG61866, &UA3062799},
	{SG61867, &UA3063505},
	{SG61868, &UA3065632},
	{SG61869, &UA3065809},
	{SG61870, &UA3093712},
	{SG61871, &UA3096518},
	{SG61872, &UA3096691},
	{SG61873, &UA3114033},
	{SG61874, &UA3114686},
	{SG61875, &UA2193450},
	{SG61876, &UA2202769},
	{SG61877, &UA2231007},
	{SG61878, &UA2232850},
	{SG61879, &UA2238126},
	{SG61880, &UA2259167},
	{SG61881, &UA2259319},
	{SG61882, &UA2357887},
	{SG61883, &UA2358372},
	{SG61884, &UA2388054},
	{SG61885, &UA2388087},
	{SG61886, &UA2389948},
	{SG61887, &UA2390007},
	{SG61888, &UA2408370},
	{SG61889, &UA2408371},
	{SG61890, &UA2756419},
	{SG61891, &UA2770716},
	{SG61892, &UA2783421},
	{SG61893, &UA2783440},
	{SG61894, &UA2873607},
	{SG61895, &UA2873677},
	{SG61896, &UA2874411},
	{SG61897, &UA2875806},
	{SG61898, &UA2877030},
	{SG61899, &UA2877032},
	{SG61900, &UA2877311},
	{SG61901, &UA2882358},
	{SG61911, &UA3039541},
	{SG61912, &UA3064404},
	{SG61913, &UA3065732},
	{SG61914, &UA3067629},
	{SG61915, &UA3067740},
	{SG61916, &UA3067952},
	{SG61917, &UA3082197},
	{SG61918, &UA3082314},
	{SG61919, &UA3093893},
	{SG61920, &UA3094647},
	{SG61921, &UA2752738},
	{SG61922, &UA3024590},
	{SG61923, &UA3024742},
	{SG61924, &UA3024749},
	{SG61925, &UA3024880},
	{SG61926, &UA3028346},
	{SG61927, &UA3028930},
	{SG61928, &UA3029604},
	{SG61929, &UA3029651},
	{SG61930, &UA3029787},
	{SG61931, &UA2740608},
	{SG61932, &UA2755513},
	{SG61933, &UA2756582},
	{SG61934, &UA2769172},
	{SG61935, &UA2871609},
	{SG61936, &UA2873601},
	{SG61937, &UA2873702},
	{SG61938, &UA2873706},
	{SG61939, &UA2873924},
	{SG61940, &UA2876739},
	{SG61941, &UA2877508},
	{SG61942, &UA2885960},
	{SG61943, &UA2967542},
	{SG61944, &UA3090145},
	{SG61945, &UA3103775},
	{SG61946, &UA3020112},
	{SG61947, &UA3024625},
	{SG61948, &UA3024741},
	{SG61949, &UA3024962},
	{SG61950, &UA3025368},
	{SG61951, &UA3025704},
	{SG61952, &UA3027259},
	{SG61953, &UA3028482},
	{SG61955, &UA3028772},
	{SG61956, &UA3030024},
	{SG61957, &UA3079068},
	{SG61958, &UA2387454},
	{SG61959, &UA2408291},
	{SG61960, &UA2408346},
	{SG61961, &UA2408364},
	{SG61962, &UA2408375},
	{SG61963, &UA2427005},
	{SG61964, &UA2428541},
	{SG61965, &UA2430980},
	{SG61966, &UA2435082},
	{SG61967, &UA2436332},
	{SG61968, &UA2437177},
	{SG61969, &UA2461205},
	{SG61970, &UA2486606},
	{SG61971, &UA2507430},
	{SG61972, &UA2532923},
	{SG61973, &UA2540594},
	{SG61974, &UA2557207},
	{SG61975, &UA2847844},
	{SG61976, &UA2849740},
	{SG61977, &UA2859864},
	{SG61978, &UA1994658},
	{SG61979, &UA1997673},
	{SG61980, &UA2014095},
	{SG61981, &UA2040420},
	{SG61982, &UA2042910},
	{SG61983, &UA2193785},
	{SG61984, &UA2196369},
	{SG61985, &UA2199787},
	{SG61986, &UA2200996},
	{SG61987, &UA2201630},
	{SG61988, &UA2203785},
	{SG61989, &UA2236195},
	{SG61990, &UA2237984},
	{SG61991, &UA2238027},
	{SG61992, &UA2350574},
	{SG61993, &UA2526926},
	{SG61994, &UA2695874},
	{SG61995, &UA2239304},
	{SG61996, &UA2353741},
	{SG61997, &UA2359339},
	{SG61998, &UA2361127},
	{SG61999, &UA2363187},
	{SG62000, &UA2368869},
	{SG62001, &UA2390991},
	{SG62002, &UA2408110},
	{SG62003, &UA2435718},
	{SG62004, &UA2436054},
	{SG62005, &UA2436737},
	{SG62006, &UA2460307},
	{SG62007, &UA2461957},
	{SG62008, &UA2462194},
	{SG62009, &UA2488064},
	{SG62010, &UA2488202},
	{SG62011, &UA2492329},
	{SG62012, &UA2492376},
	{SG62013, &UA2567775},
	{SG62014, &UA2567796},
	{SG62015, &UA2567865},
	{SG62016, &UA2567874},
	{SG62017, &UA2567881},
	{SG62018, &UA2568378},
	{SG62019, &UA2569776},
	{SG62020, &UA2584646},
	{SG62021, &UA2605850},
	{SG62022, &UA2891466},
	{SG62023, &UA3060213},
	{SG62024, &UA3108741},
	{SG62025, &UA3120871},
	{SG62026, &UA3125125},
	{SG62042, &UA2471955},
	{SG62043, &UA2488553},
	{SG62044, &UA2492149},
	{SG62045, &UA2492187},
	{SG62046, &UA2492272},
	{SG62047, &UA2492351},
	{SG62048, &UA2492372},
	{SG62049, &UA2510341},
	{SG62050, &UA2510497},
	{SG62051, &UA2540599},
	{SG62052, &UA2559569},
	{SG62053, &UA2767450},
	{SG62055, &UA2850600},
	{SG62057, &UA2852983},
	{SG62070, &UA3064751},
	{SG62071, &UA3064766},
	{SG62073, &UA3065300},
	{SG62075, &UA3067631},
	{SG62076, &UA3067806},
	{SG62078, &UA3068074},
	{SG62080, &UA3068230},
	{SG62082, &UA3082193},
	{SG62085, &UA2367618},
	{SG62087, &UA2461095},
	{SG62089, &UA2470714},
	{SG62091, &UA2470741},
	{SG62093, &UA2490170},
	{SG62094, &UA2491062},
	{SG62095, &UA2509316},
	{SG62096, &UA2510939},
	{SG62097, &UA2514379},
	{SG62098, &UA2532531},
	{SG62099, &UA2532543},
	{SG62100, &UA2538580},
	{SG62101, &UA2538924},
	{SG62102, &UA2567879},
	{SG62103, &UA2567997},
	{SG62104, &UA2568961},
	{SG62105, &UA2569530},
	{SG62106, &UA2571052},
	{SG62107, &UA2571271},
	{SG62110, &UA2567779},
	{SG62111, &UA2567786},
	{SG62112, &UA2567795},
	{SG62113, &UA2567800},
	{SG62114, &UA2568257},
	{SG62115, &UA2569381},
	{SG62116, &UA2569765},
	{SG62117, &UA2569911},
	{SG62118, &UA2584651},
	{SG62119, &UA3027980},
	{SG62120, &UA3072840},
	{SG62121, &UA2878888},
	{SG62122, &UA2898317},
	{SG62123, &UA2898330},
	{SG62124, &UA2968285},
	{SG62125, &UA3024737},
	{SG62126, &UA3024752},
	{SG62127, &UA3024957},
	{SG62128, &UA3024988},
	{SG62129, &UA3028770},
	{SG62130, &UA3030408},
	{SG62131, &UA2567785},
	{SG62132, &UA2567787},
	{SG62133, &UA2567808},
	{SG62134, &UA2568041},
	{SG62135, &UA2569634},
	{SG62136, &UA2584173},
	{SG62137, &UA2584220},
	{SG62138, &UA2604389},
	{SG62139, &UA2734502},
	{SG62140, &UA3071932},
	{SG62141, &UA2357451},
	{SG62142, &UA2357471},
	{SG62143, &UA2358755},
	{SG62144, &UA2362059},
	{SG62145, &UA2389798},
	{SG62146, &UA2435778},
	{SG62147, &UA2437176},
	{SG62148, &UA2437184},
	{SG62149, &UA2437228},
	{SG62150, &UA2437581},
	{SG62151, &UA2508045},
	{SG62152, &UA2203725},
	{SG62153, &UA2203797},
	{SG62154, &UA2203805},
	{SG62155, &UA2203982},
	{SG62156, &UA2231639},
	{SG62157, &UA2233051},
	{SG62158, &UA2234779},
	{SG62159, &UA2238054},
	{SG62160, &UA2259288},
	{SG62161, &UA2355031},
	{SG62162, &UA2390838},
	{SG62163, &UA2534052},
	{SG62164, &UA2535778},
	{SG62165, &UA2540610},
	{SG62166, &UA2570712},
	{SG62167, &UA2578140},
	{SG62168, &UA2711639},
	{SG62170, &UA2675556},
	{SG62171, &UA2676973},
	{SG62172, &UA2678356},
	{SG62173, &UA2678393},
	{SG62174, &UA2678676},
	{SG62175, &UA2679011},
	{SG62176, &UA2679137},
	{SG62177, &UA2679930},
	{SG62178, &UA2712675},
	{SG62179, &UA2719148},
	{SG62180, &UA2736901},
	{SG62181, &UA2752467},
	{SG62182, &UA2865760},
	{SG62183, &UA2674785},
	{SG62184, &UA2678583},
	{SG62185, &UA2678584},
	{SG62186, &UA2678597},
	{SG62187, &UA2678736},
	{SG62188, &UA2678740},
	{SG62189, &UA2679582},
	{SG62190, &UA2679984},
	{SG62191, &UA2707368},
	{SG62192, &UA2936959},
	{SG62193, &UA3075457},
	{SG62194, &UA2356450},
	{SG62195, &UA2357462},
	{SG62196, &UA2437159},
	{SG62197, &UA2437170},
	{SG62198, &UA2437410},
	{SG62199, &UA2437497},
	{SG62200, &UA2579974},
	{SG62201, &UA2780948},
	{SG62202, &UA3090481},
	{SG62203, &UA911034},
	{SG62204, &UA912380},
	{SG62205, &UA912381},
	{SG62206, &UA912536},
	{SG62207, &UA912535},
	{SG62208, &UA913181},
	{SG62209, &UA924167},
	{SG62210, &UA2355282},
	{SG62211, &UA2355527},
	{SG62212, &UA2389616},
	{SG62213, &UA2414586},
	{SG62214, &UA2746897},
	{SG62222, &UA2567788},
	{SG62223, &UA2567819},
	{SG62224, &UA2567818},
	{SG62225, &UA2567937},
	{SG62226, &UA2567972},
	{SG62227, &UA2568216},
	{SG62228, &UA2584054},
	{SG62229, &UA2604099},
	{SG62230, &UA2770105},
	{SG62231, &UA2783478},
	{SG62232, &UA2783628},
	{SG62233, &UA2796498},
	{SG62234, &UA2873648},
	{SG62235, &UA2873652},
	{SG62236, &UA2874649},
	{SG62237, &UA2876962},
	{SG62238, &UA2877031},
	{SG62239, &UA2877023},
	{SG62240, &UA2877382},
	{SG62241, &UA2887703},
	{SG62242, &UA2470409},
	{SG62243, &UA2471961},
	{SG62244, &UA2489208},
	{SG62245, &UA2490841},
	{SG62246, &UA2492153},
	{SG62247, &UA2492186},
	{SG62248, &UA2492503},
	{SG62249, &UA2492509},
	{SG62250, &UA2509887},
	{SG62251, &UA2512538},
	{SG62252, &UA2537886},
	{SG62253, &UA2956188},
	{SG62254, &UA2673811},
	{SG62255, &UA2676471},
	{SG62256, &UA2677412},
	{SG62257, &UA2678392},
	{SG62258, &UA2678407},
	{SG62259, &UA2678503},
	{SG62260, &UA2678687},
	{SG62261, &UA2679928},
	{SG62262, &UA2679941},
	{SG62263, &UA2717636},
	{SG62264, &UA2870156},
	{SG62265, &UA2969040},
	{SG62266, &UA2971083},
	{SG62267, &UA3024754},
	{SG62268, &UA3025065},
	{SG62269, &UA3026537},
	{SG62270, &UA3027176},
	{SG62271, &UA3028913},
	{SG62272, &UA3029231},
	{SG62273, &UA3030006},
	{SG62274, &UA2461905},
	{SG62275, &UA2492189},
	{SG62276, &UA2492219},
	{SG62277, &UA2492326},
	{SG62278, &UA2508791},
	{SG62279, &UA2512847},
	{SG62280, &UA2700845},
	{SG62281, &UA2781999},
	{SG62282, &UA3079355},
	{SG62283, &UA2357458},
	{SG62284, &UA2357466},
	{SG62285, &UA2387483},
	{SG62286, &UA2387582},
	{SG62287, &UA2408432},
	{SG62288, &UA2427676},
	{SG62289, &UA2427812},
	{SG62290, &UA2429898},
	{SG62291, &UA2429917},
	{SG62292, &UA2432922},
	{SG62293, &UA2435494},
	{SG62294, &UA2436402},
	{SG62295, &UA2437232},
	{SG62296, &UA2437323},
	{SG62297, &UA2467070},
	{SG62298, &UA2957846},
	{SG62299, &UA2873609},
	{SG62300, &UA2873693},
	{SG62301, &UA2873707},
	{SG62302, &UA2873758},
	{SG62303, &UA2873982},
	{SG62304, &UA2875349},
	{SG62305, &UA2876065},
	{SG62306, &UA2882382},
	{SG62307, &UA2886195},
	{SG62308, &UA2357460},
	{SG62309, &UA2387461},
	{SG62310, &UA2387615},
	{SG62311, &UA2408293},
	{SG62312, &UA2428537},
	{SG62313, &UA2429505},
	{SG62314, &UA2430531},
	{SG62315, &UA2433561},
	{SG62316, &UA2434334},
	{SG62317, &UA2435962},
	{SG62318, &UA2436380},
	{SG62319, &UA2437242},
	{SG62320, &UA2437270},
	{SG62321, &UA2460472},
	{SG62322, &UA2556754},
	{SG62361, &UA3067633},
	{SG62363, &UA3067756},
	{SG62365, &UA3068154},
	{SG62367, &UA3082364},
	{SG62369, &UA3093913},
	{SG62371, &UA3094585},
	{SG62373, &UA3111001},
	{SG62375, &UA3116375},
	{SG62377, &UA3127774},
	{SG62399, &UA3062783},
	{SG62401, &UA3062787},
	{SG62402, &UA3062791},
	{SG62404, &UA3062792},
	{SG62406, &UA3062798},
	{SG62408, &UA3065630},
	{SG62410, &UA3065635},
	{SG62411, &UA3081265},
	{SG62412, &UA3126582},
	{SG62716, &UA2357296},
	{SG62718, &UA2437165},
	{SG62720, &UA2437171},
	{SG62743, &UA2437175},
	{SG62745, &UA2437182},
	{SG62747, &UA2437240},
	{SG62749, &UA2437241},
	{SG62751, &UA2437246},
	{SG62753, &UA2437263},
	{SG62755, &UA2437332},
	{SG62757, &UA2437477},
	{SG62759, &UA2597685},
	{SG62761, &UA2674646},
	{SG62763, &UA2674684},
	{SG62764, &UA2714921},
	{SG62765, &UA2492148},
	{SG62766, &UA2492182},
	{SG62767, &UA2492220},
	{SG62768, &UA2492198},
	{SG62769, &UA2492401},
	{SG62770, &UA2492513},
	{SG62771, &UA2492625},
	{SG62772, &UA2511169},
	{SG62773, &UA2531609},
	{SG62774, &UA2658725},
	{SG62775, &UA2567777},
	{SG62776, &UA2567780},
	{SG62777, &UA2567791},
	{SG62778, &UA2567806},
	{SG62779, &UA2567938},
	{SG62780, &UA2569767},
	{SG62781, &UA2570199},
	{SG62782, &UA2584247},
	{SG62783, &UA2584752},
	{SG62784, &UA3086933},
	{SG62785, &UA3107951},
	{SG62800, &UA2461903},
	{SG62802, &UA2470417},
	{SG62804, &UA2471958},
	{SG62806, &UA2492179},
	{SG62807, &UA2492501},
	{SG62809, &UA2492507},
	{SG62811, &UA2492609},
	{SG62813, &UA2509076},
	{SG62815, &UA2510500},
	{SG62816, &UA2535881},
	{SG62817, &UA2729815},
	{SG62861, &UA2015619},
	{SG62862, &UA2016038},
	{SG62863, &UA2016043},
	{SG62864, &UA2042168},
	{SG62865, &UA2042294},
	{SG62866, &UA2235271},
	{SG62867, &UA2540558},
	{SG62868, &UA3094815},
	{SG62869, &UA2675447},
	{SG62870, &UA2678576},
	{SG62871, &UA2678581},
	{SG62872, &UA2678925},
	{SG62873, &UA2678926},
	{SG62874, &UA2678929},
	{SG62875, &UA2679722},
	{SG62876, &UA2679833},
	{SG62877, &UA2708301},
	{SG62878, &UA2567801},
	{SG62879, &UA2567811},
	{SG62880, &UA2567812},
	{SG62881, &UA2567789},
	{SG62882, &UA2567875},
	{SG62883, &UA2567882},
	{SG62884, &UA2567914},
	{SG62885, &UA2567936},
	{SG62886, &UA2568064},
	{SG62887, &UA2568248},
	{SG62888, &UA2583823},
	{SG62889, &UA2674629},
	{SG62890, &UA2674861},
	{SG62891, &UA2714758},
	{SG62892, &UA2193628},
	{SG62893, &UA2195896},
	{SG62894, &UA2200313},
	{SG62895, &UA2201482},
	{SG62896, &UA2201649},
	{SG62897, &UA2201813},
	{SG62898, &UA2203776},
	{SG62899, &UA2203801},
	{SG62900, &UA2205536},
	{SG62901, &UA2236199},
	{SG62902, &UA2259114},
	{SG62903, &UA2349273},
	{SG62904, &UA2350579},
	{SG62905, &UA2837782},
	{SG62906, &UA2873598},
	{SG62907, &UA2873777},
	{SG62908, &UA2873779},
	{SG62909, &UA2873918},
	{SG62910, &UA2874011},
	{SG62911, &UA2874142},
	{SG62912, &UA2874325},
	{SG62914, &UA2874392},
	{SG62916, &UA2874393},
	{SG62918, &UA2875073},
	{SG62920, &UA2877330},
	{SG62923, &UA3062785},
	{SG62924, &UA3062786},
	{SG62925, &UA3062794},
	{SG62926, &UA3062796},
	{SG62927, &UA3062801},
	{SG62928, &UA3062802},
	{SG62929, &UA3062804},
	{SG62930, &UA3065628},
	{SG62931, &UA3065631},
	{SG62932, &UA3065634},
	{SG62933, &UA3080101},
	{SG63066, &UA2470640},
	{SG63067, &UA2488024},
	{SG63068, &UA2510992},
	{SG63069, &UA2515588},
	{SG63070, &UA2530328},
	{SG63071, &UA2602219},
	{SG63072, &UA2674608},
	{SG63073, &UA2674632},
	{SG63074, &UA2674652},
	{SG63075, &UA2674876},
	{SG63076, &UA2675208},
	{SG63077, &UA2715171},
	{SG63078, &UA2716576},
	{SG63079, &UA2482854},
	{SG63080, &UA2492151},
	{SG63081, &UA2492193},
	{SG63102, &UA2492214},
	{SG63103, &UA2492218},
	{SG63104, &UA2492311},
	{SG63105, &UA2492337},
	{SG63106, &UA2492362},
	{SG63107, &UA2492393},
	{SG63108, &UA2492559},
	{SG63109, &UA2510200},
	{SG63110, &UA2511641},
	{SG63111, &UA2674464},
	{SG63112, &UA2674659},
	{SG63113, &UA2715423},
	{SG63114, &UA3024559},
	{SG63115, &UA3024762},
	{SG63116, &UA3024965},
	{SG63117, &UA3025084},
	{SG63118, &UA3025367},
	{SG63119, &UA3027150},
	{SG63120, &UA3027410},
	{SG63121, &UA3029689},
	{SG63139, &UA2675137},
	{SG63141, &UA2675749},
	{SG63142, &UA2678094},
	{SG63143, &UA2678363},
	{SG63144, &UA2678370},
	{SG63145, &UA2678397},
	{SG63146, &UA2678526},
	{SG63147, &UA2679097},
	{SG63148, &UA2679937},
	{SG63149, &UA2679938},
	{SG63150, &UA2898310},
	{SG63151, &UA2898311},
	{SG63152, &UA2898326},
	{SG63153, &UA2966445},
	{SG63154, &UA3011791},
	{SG63155, &UA3015162},
	{SG63156, &UA2674473},
	{SG63157, &UA2674628},
	{SG63158, &UA2674925},
	{SG63159, &UA2677328},
	{SG63160, &UA2678586},
	{SG63161, &UA2678592},
	{SG63162, &UA2678593},
	{SG63163, &UA2678730},
	{SG63164, &UA2678733},
	{SG63165, &UA2678744},
	{SG63166, &UA2679176},
	{SG63167, &UA2679340},
	{SG63168, &UA2679527},
	{SG63169, &UA2718745},
	{SG63170, &UA2203743},
	{SG63171, &UA2203777},
	{SG63172, &UA2233625},
	{SG63173, &UA2235704},
	{SG63174, &UA2238620},
	{SG63175, &UA2239325},
	{SG63176, &UA2239371},
	{SG63177, &UA2260199},
	{SG63178, &UA2352127},
	{SG63179, &UA2355584},
	{SG63180, &UA2403502},
	{SG63181, &UA2530116},
	{SG63182, &UA2596664},
	{SG63183, &UA2674626},
	{SG63184, &UA2674644},
	{SG63185, &UA2674654},
	{SG63186, &UA2675185},
	{SG63187, &UA2675191},
	{SG63188, &UA2716514},
	{SG63189, &UA2716820},
	{SG63190, &UA2983065},
	{SG63191, &UA2983899},
	{SG63192, &UA2994543},
	{SG63193, &UA3062788},
	{SG63194, &UA3062790},
	{SG63195, &UA3062797},
	{SG63196, &UA3093646},
	{SG63197, &UA3094726},
	{SG63198, &UA3115325},
	{SG63199, &UA3127323},
	{SG63200, &UA3127561},
	{SG63213, &UA2966255},
	{SG63215, &UA3024560},
	{SG63217, &UA3024942},
	{SG63229, &UA3025070},
	{SG63231, &UA3025543},
	{SG63234, &UA3025724},
	{SG63236, &UA3025915},
	{SG63238, &UA3026186},
	{SG63239, &UA3026652},
	{SG63240, &UA3027268},
	{SG63241, &UA3027403},
	{SG63243, &UA912384},
	{SG63244, &UA912398},
	{SG63245, &UA912410},
	{SG63246, &UA912664},
	{SG63247, &UA913240},
	{SG63248, &UA923864},
	{SG63249, &UA2235720},
	{SG63250, &UA3099264},
	{SG63251, &UA2538246},
	{SG63252, &UA2567793},
	{SG63253, &UA2584662},
	{SG63254, &UA2597995},
	{SG63255, &UA2674500},
	{SG63256, &UA2674533},
	{SG63257, &UA2674615},
	{SG63258, &UA2674645},
	{SG63259, &UA2675000},
	{SG63260, &UA2707495},
	{SG63261, &UA2714965},
	{SG63262, &UA2716828},
	{SG63263, &UA2750747},
	{SG63264, &UA2766154},
	{SG63265, &UA2792593},
	{SG63266, &UA2958243},
	{SG63267, &UA2999634},
	{SG63268, &UA3049422},
	{SG63269, &UA3088579},
	{SG63270, &UA3108925},
	{SG63271, &UA2357689},
	{SG63272, &UA2360473},
	{SG63273, &UA2408297},
	{SG63274, &UA2433229},
	{SG63275, &UA2458711},
	{SG63276, &UA2459143},
	{SG63277, &UA2674653},
	{SG63278, &UA2674705},
	{SG63279, &UA2674822},
	{SG63280, &UA2675199},
	{SG63281, &UA2675712},
	{SG63282, &UA2707506},
	{SG63283, &UA2707710},
	{SG63284, &UA2844939},
	{SG63285, &UA2952272},
	{SG63286, &UA3009899},
	{SG63287, &UA2203729},
	{SG63288, &UA2203733},
	{SG63289, &UA2203782},
	{SG63290, &UA2203784},
	{SG63291, &UA2203861},
	{SG63292, &UA2204087},
	{SG63293, &UA2234465},
	{SG63294, &UA2238409},
	{SG63295, &UA2238520},
	{SG63296, &UA2258348},
	{SG63297, &UA2258376},
	{SG63298, &UA2354653},
	{SG63299, &UA2436909},
	{SG63300, &UA2536309},
	{SG63301, &UA2567857},
	{SG63302, &UA2597513},
	{SG63303, &UA2749240},
	{SG63304, &UA2857825},
	{SG63305, &UA3046142},
	{SG63306, &UA2203724},
	{SG63307, &UA2203731},
	{SG63308, &UA2204096},
	{SG63309, &UA2231147},
	{SG63310, &UA2233981},
	{SG63311, &UA2234791},
	{SG63312, &UA2238591},
	{SG63313, &UA2457137},
	{SG63314, &UA2535454},
	{SG63315, &UA2535456},
	{SG63316, &UA2583062},
	{SG63317, &UA2752488},
	{SG63318, &UA2854556},
	{SG63319, &UA910390},
	{SG63320, &UA913163},
	{SG63321, &UA924811},
	{SG63322, &UA925309},
	{SG63323, &UA1996987},
	{SG63324, &UA1997134},
	{SG63325, &UA2001065},
	{SG63326, &UA2003026},
	{SG63327, &UA2005478},
	{SG63328, &UA2005610},
	{SG63329, &UA2010166},
	{SG63330, &UA2010185},
	{SG63331, &UA2010996},
	{SG63332, &UA2016031},
	{SG63333, &UA2041324},
	{SG63334, &UA2194155},
	{SG63335, &UA2384518},
	{SG63359, &UA2674471},
	{SG63360, &UA2674501},
	{SG63361, &UA2674798},
	{SG63362, &UA2674811},
	{SG63363, &UA2675705},
	{SG63364, &UA2675736},
	{SG63365, &UA2678403},
	{SG63366, &UA2678521},
	{SG63367, &UA2707701},
	{SG63368, &UA2708293},
	{SG63369, &UA2713777},
	{SG63370, &UA2716386},
	{SG63371, &UA2795054},
	{SG63372, &UA3047932},
	{SG63373, &UA2203728},
	{SG63374, &UA2203732},
	{SG63375, &UA2203793},
	{SG63376, &UA2231057},
	{SG63377, &UA2232666},
	{SG63378, &UA2235257},
	{SG63379, &UA2402656},
	{SG63380, &UA2553570},
	{SG63381, &UA2597519},
	{SG63382, &UA2598527},
	{SG63383, &UA2708257},
	{SG63384, &UA2731187},
	{SG63385, &UA2734405},
	{SG63386, &UA2781083},
	{SG63387, &UA3091616},
	{SG63490, &UA2407483},
	{SG63491, &UA2407485},
	{SG63492, &UA2460072},
	{SG63493, &UA2460397},
	{SG63494, &UA2713488},
	{SG63495, &UA2714031},
	{SG63496, &UA2754024},
	{SG63497, &UA2770383},
	{SG63498, &UA2956399},
	{SG63499, &UA2770999},
	{SG63500, &UA2774181},
	{SG63501, &UA2774192},
	{SG63502, &UA2784162},
	{SG63503, &UA2784248},
	{SG63504, &UA2784690},
	{SG63505, &UA2898624},
	{SG63506, &UA2015112},
	{SG63507, &UA2015617},
	{SG63508, &UA2015621},
	{SG63509, &UA2041484},
	{SG63510, &UA2200548},
	{SG63511, &UA2343924},
	{SG63512, &UA2549005},
	{SG63513, &UA2600427},
	{SG63514, &UA2600762},
	{SG63515, &UA2603748},
	{SG63516, &UA2781230},
	{SG63517, &UA3091584},
	{SG63537, &UA2193923},
	{SG63538, &UA2203727},
	{SG63539, &UA2203736},
	{SG63540, &UA2203740},
	{SG63541, &UA2203791},
	{SG63542, &UA2203884},
	{SG63543, &UA2203894},
	{SG63544, &UA2203895},
	{SG63545, &UA2204010},
	{SG63546, &UA2231336},
	{SG63547, &UA2235714},
	{SG63548, &UA2258267},
	{SG63549, &UA2597219},
	{SG63550, &UA2674482},
	{SG63551, &UA2674793},
	{SG63552, &UA2714928},
	{SG63553, &UA2857581},
	{SG63554, &UA2989126},
	{SG63590, &UA2203735},
	{SG63591, &UA2203779},
	{SG63592, &UA2231135},
	{SG63593, &UA2234471},
	{SG63594, &UA2259293},
	{SG63595, &UA2534065},
	{SG63596, &UA2534066},
	{SG63597, &UA2534901},
	{SG63598, &UA2535464},
	{SG63599, &UA2568097},
	{SG63600, &UA2603780},
	{SG63601, &UA2718789},
	{SG63602, &UA2737693},
	{SG63653, &UA2357770},
	{SG63654, &UA2358090},
	{SG63655, &UA2388035},
	{SG63656, &UA2388326},
	{SG63657, &UA2389899},
	{SG63658, &UA2390013},
	{SG63659, &UA2408330},
	{SG63660, &UA2431961},
	{SG63661, &UA2000889},
	{SG63662, &UA2015636},
	{SG63663, &UA2041221},
	{SG63664, &UA2197565},
	{SG63665, &UA2200776},
	{SG63666, &UA2204330},
	{SG63667, &UA2354642},
	{SG63668, &UA2466346},
	{SG63669, &UA2674630},
	{SG63670, &UA2674640},
	{SG63671, &UA2674809},
	{SG63672, &UA2674873},
	{SG63673, &UA2675009},
	{SG63674, &UA2718192},
	{SG63675, &UA2719087},
	{SG63689, &UA2677726},
	{SG63690, &UA2678854},
	{SG63691, &UA2709269},
	{SG63692, &UA2709277},
	{SG63693, &UA2710751},
	{SG63694, &UA2714042},
	{SG63695, &UA2714346},
	{SG63696, &UA2715357},
	{SG63697, &UA2716720},
	{SG63698, &UA2716724},
	{SG63699, &UA2716903},
	{SG63700, &UA2719366},
	{SG63701, &UA2719386},
	{SG63702, &UA2719478},
	{SG63703, &UA2740187},
	{SG63704, &UA2740301},
	{SG63705, &UA2740352},
	{SG63706, &UA2740506},
	{SG63707, &UA2754187},
	{SG63708, &UA2754343},
	{SG63709, &UA2754489},
	{SG63710, &UA2784144},
	{SG63711, &UA2784294},
	{SG63712, &UA2784296},
	{SG63713, &UA2784302},
	{SG63714, &UA2870198},
	{SG63715, &UA2870199},
	{SG63716, &UA2870201},
	{SG63717, &UA2875692},
	{SG63718, &UA2875696},
	{SG63719, &UA2875698},
	{SG63720, &UA2888596},
	{SG63721, &UA2888601},
	{SG63722, &UA2888629},
	{SG63723, &UA2888656},
	{SG63724, &UA2888657},
	{SG63725, &UA2888658},
	{SG63726, &UA2888659},
	{SG63727, &UA2947719},
	{SG63728, &UA2947991},
	{SG63729, &UA2947995},
	{SG63730, &UA2947999},
	{SG63731, &UA2955656},
	{SG63732, &UA3024944},
	{SG63733, &UA3024966},
	{SG63734, &UA3025186},
	{SG63735, &UA3036654},
	{SG63736, &UA3037935},
	{SG63737, &UA3038880},
	{SG63738, &UA3092484},
	{SG63739, &UA2015618},
	{SG63740, &UA2015629},
	{SG63741, &UA2015637},
	{SG63742, &UA2016034},
	{SG63743, &UA2017888},
	{SG63744, &UA2041734},
	{SG63745, &UA2042650},
	{SG63746, &UA2196404},
	{SG63762, &UA2770342},
	{SG63763, &UA2773272},
	{SG63764, &UA2773281},
	{SG63765, &UA2870901},
	{SG63767, &UA2875715},
	{SG63769, &UA2876111},
	{SG63770, &UA2876597},
	{SG63772, &UA2885335},
	{SG63774, &UA2885347},
	{SG63776, &UA2887917},
	{SG63778, &UA2888358},
	{SG63806, &UA2888661},
	{SG63807, &UA2946045},
	{SG63808, &UA2948036},
	{SG63809, &UA2948317},
	{SG63810, &UA2972563},
	{SG63811, &UA2972562},
	{SG63812, &UA2972564},
	{SG63813, &UA2973785},
	{SG63814, &UA2974738},
	{SG63815, &UA2974741},
	{SG63816, &UA2975893},
	{SG63817, &UA2975903},
	{SG63818, &UA2975909},
	{SG63819, &UA3028224},
	{SG63820, &UA3028233},
	{SG63821, &UA3030752},
	{SG63822, &UA3033003},
	{SG63823, &UA3033180},
	{SG63824, &UA3033438},
	{SG63825, &UA3036829},
	{SG63826, &UA3036834},
	{SG63827, &UA3036957},
	{SG63828, &UA3037090},
	{SG63829, &UA3037091},
	{SG63830, &UA3037102},
	{SG63831, &UA3063569},
	{SG63832, &UA3063570},
	{SG63833, &UA3063578},
	{SG63834, &UA3095672},
	{SG63835, &UA3095673},
	{SG63836, &UA3095674},
	{SG63837, &UA2602785},
	{SG63838, &UA2605385},
	{SG63839, &UA2677095},
	{SG63840, &UA2679372},
	{SG63872, &UA2877410},
	{SG63874, &UA2884699},
	{SG63875, &UA2946165},
	{SG63876, &UA3011523},
	{SG63877, &UA3015626},
	{SG63878, &UA3018599},
	{SG63879, &UA3025465},
	{SG63880, &UA3027316},
	{SG63881, &UA3027550},
	{SG63882, &UA3028018},
	{SG63883, &UA3028071},
	{SG63884, &UA3028486},
	{SG63885, &UA3029223},
	{SG63886, &UA3029640},
	{SG63887, &UA3029914},
	{SG63888, &UA3030061},
	{SG63889, &UA3034257},
	{SG63890, &UA2675175},
	{SG63891, &UA2740509},
	{SG63892, &UA2741425},
	{SG63893, &UA2753670},
	{SG63894, &UA2755302},
	{SG63895, &UA2757172},
	{SG63896, &UA2774275},
	{SG63897, &UA2784394},
	{SG63898, &UA2874464},
	{SG63899, &UA2874539},
	{SG63950, &UA2015620},
	{SG63951, &UA2015638},
	{SG63952, &UA2016035},
	{SG63953, &UA2016040},
	{SG63954, &UA2016042},
	{SG63955, &UA2016046},
	{SG63956, &UA2016047},
	{SG63957, &UA2016060},
	{SG63958, &UA2042270},
	{SG63959, &UA2199228},
	{SG63960, &UA2199297},
	{SG63961, &UA2597352},
	{SG63962, &UA2674656},
	{SG63963, &UA2674675},
	{SG63964, &UA2719075},
	{SG63999, &UA2769907},
	{SG64000, &UA2877378},
	{SG64001, &UA2888698},
	{SG64002, &UA2898716},
	{SG64003, &UA2945461},
	{SG64004, &UA2947878},
	{SG64005, &UA2947887},
	{SG64006, &UA3026086},
	{SG64007, &UA3026389},
	{SG64008, &UA2389741},
	{SG64009, &UA2459890},
	{SG64010, &UA2462323},
	{SG64011, &UA2469541},
	{SG64012, &UA2487604},
	{SG64013, &UA2488144},
	{SG64014, &UA2489333},
	{SG64015, &UA2492460},
	{SG64020, &UA2040911},
	{SG64021, &UA2042103},
	{SG64022, &UA2195618},
	{SG64023, &UA2197564},
	{SG64024, &UA2197568},
	{SG64025, &UA2198088},
	{SG64026, &UA2201777},
	{SG64027, &UA2202167},
	{SG64028, &UA2202389},
	{SG64029, &UA2202674},
	{SG64030, &UA2206181},
	{SG64031, &UA2232209},
	{SG64032, &UA2232228},
	{SG64033, &UA2232848},
	{SG64034, &UA2232862},
	{SG64035, &UA2235705},
	{SG64036, &UA2237251},
	{SG64037, &UA2237252},
	{SG64038, &UA2259953},
	{SG64039, &UA2259957},
	{SG64040, &UA2259960},
	{SG64041, &UA2352653},
	{SG64042, &UA2356025},
	{SG64043, &UA2357473},
	{SG64044, &UA2357759},
	{SG64045, &UA2359379},
	{SG64046, &UA2359541},
	{SG64047, &UA2359542},
	{SG64048, &UA2362484},
	{SG64049, &UA2362765},
	{SG64050, &UA2368393},
	{SG64051, &UA2385917},
	{SG64052, &UA2388495},
	{SG64053, &UA2390678},
	{SG64054, &UA2390681},
	{SG64055, &UA2403262},
	{SG64056, &UA2404939},
	{SG64058, &UA2407504},
	{SG64060, &UA2408231},
	{SG64062, &UA2408232},
	{SG64064, &UA2427303},
	{SG64066, &UA2427561},
	{SG64068, &UA2437226},
	{SG64069, &UA2437273},
	{SG64071, &UA2437288},
	{SG64073, &UA2459285},
	{SG64075, &UA2462374},
	{SG64076, &UA2470137},
	{SG64078, &UA2470139},
	{SG64080, &UA2530753},
	{SG64081, &UA2558943},
	{SG64083, &UA2731110},
	{SG64085, &UA2842301},
	{SG64095, &UA2200602},
	{SG64096, &UA2200615},
	{SG64097, &UA2426463},
	{SG64098, &UA2466330},
	{SG64099, &UA2008999},
	{SG64100, &UA2205475},
	{SG64101, &UA2205606},
	{SG64102, &UA2205611},
	{SG64103, &UA2205794},
	{SG64104, &UA2205803},
	{SG64105, &UA2205891},
	{SG64106, &UA2231571},
	{SG64107, &UA2258179},
	{SG64108, &UA2417012},
	{SG64109, &UA2647444},
	{SG64110, &UA2674624},
	{SG64111, &UA2716431},
	{SG64174, &UA2015625},
	{SG64175, &UA2015628},
	{SG64176, &UA2040569},
	{SG64177, &UA2203080},
	{SG64178, &UA2204425},
	{SG64179, &UA2385086},
	{SG64180, &UA2725799},
	{SG64181, &UA2889119},
	{SG64193, &UA2538337},
	{SG64194, &UA2538822},
	{SG64195, &UA2565726},
	{SG64196, &UA2568163},
	{SG64197, &UA2568518},
	{SG64198, &UA2569088},
	{SG64199, &UA2569473},
	{SG64200, &UA2571048},
	{SG64201, &UA3023901},
	{SG64202, &UA3031493},
	{SG64203, &UA3033597},
	{SG64204, &UA3036980},
	{SG64205, &UA3062349},
	{SG64206, &UA3064084},
	{SG64207, &UA3065052},
	{SG64208, &UA3065849},
	{SG64209, &UA3096676},
	{SG64210, &UA3110355},
	{SG64225, &UA2369712},
	{SG64226, &UA2369717},
	{SG64227, &UA2387542},
	{SG64228, &UA2388222},
	{SG64229, &UA2405679},
	{SG64230, &UA2406786},
	{SG64231, &UA2407512},
	{SG64232, &UA2407516},
	{SG64233, &UA2429847},
	{SG64234, &UA2430180},
	{SG64235, &UA2430941},
	{SG64236, &UA2434671},
	{SG64237, &UA2435233},
	{SG64238, &UA2435896},
	{SG64239, &UA2437451},
	{SG64240, &UA2461909},
	{SG64241, &UA2461912},
	{SG64242, &UA2461978},
	{SG64243, &UA2470855},
	{SG64244, &UA2471620},
	{SG64245, &UA2471635},
	{SG64246, &UA2487417},
	{SG64247, &UA2487420},
	{SG64248, &UA2487511},
	{SG64249, &UA2489170},
	{SG64250, &UA2490860},
	{SG64251, &UA2508979},
	{SG64252, &UA2509103},
	{SG64253, &UA2513071},
	{SG64254, &UA2513075},
	{SG64255, &UA2515894},
	{SG64256, &UA2516060},
	{SG64257, &UA2516762},
	{SG64258, &UA2516763},
	{SG64259, &UA2532810},
	{SG64260, &UA2533069},
	{SG64261, &UA2568078},
	{SG64262, &UA2568095},
	{SG64263, &UA2568100},
	{SG64264, &UA2569186},
	{SG64265, &UA2572244},
	{SG64266, &UA2572245},
	{SG64267, &UA2732840},
	{SG64268, &UA2489787},
	{SG64269, &UA2489788},
	{SG64270, &UA2489795},
	{SG64271, &UA2510399},
	{SG64272, &UA2510407},
	{SG64273, &UA2512453},
	{SG64274, &UA2513389},
	{SG64275, &UA2516469},
	{SG64276, &UA2536141},
	{SG64277, &UA2537297},
	{SG64278, &UA2537299},
	{SG64279, &UA2537306},
	{SG64280, &UA2539128},
	{SG64281, &UA2539455},
	{SG64282, &UA2539617},
	{SG64283, &UA2570203},
	{SG64284, &UA2570207},
	{SG64285, &UA2570224},
	{SG64286, &UA2570278},
	{SG64287, &UA2571013},
	{SG64288, &UA2571021},
	{SG64289, &UA2571014},
	{SG64290, &UA2585777},
	{SG64291, &UA2585787},
	{SG64292, &UA2585790},
	{SG64293, &UA2602797},
	{SG64294, &UA2605201},
	{SG64295, &UA2605244},
	{SG64296, &UA2673520},
	{SG64297, &UA2673832},
	{SG64298, &UA2675967},
	{SG64299, &UA2675970},
	{SG64300, &UA2675959},
	{SG64301, &UA2680304},
	{SG64302, &UA2738235},
	{SG64303, &UA2738827},
	{SG64304, &UA2781865},
	{SG64305, &UA2782310},
	{SG64306, &UA2861491},
	{SG64307, &UA2865746},
	{SG64308, &UA2866095},
	{SG64309, &UA2952050},
	{SG64310, &UA2568007},
	{SG64311, &UA2568021},
	{SG64312, &UA2568191},
	{SG64313, &UA2569177},
	{SG64314, &UA2569175},
	{SG64315, &UA2569185},
	{SG64316, &UA2586566},
	{SG64317, &UA2601743},
	{SG64318, &UA2604366},
	{SG64320, &UA2604419},
	{SG64322, &UA2604527},
	{SG64324, &UA2606189},
	{SG64326, &UA2606444},
	{SG64328, &UA2673956},
	{SG64330, &UA2674427},
	{SG64331, &UA2675500},
	{SG64332, &UA2678942},
	{SG64333, &UA2679179},
	{SG64334, &UA2679459},
	{SG64335, &UA2679462},
	{SG64336, &UA2679464},
	{SG64337, &UA2679531},
	{SG64338, &UA2710745},
	{SG64339, &UA2710750},
	{SG64340, &UA2712036},
	{SG64341, &UA2714979},
	{SG64342, &UA2714981},
	{SG64343, &UA2714970},
	{SG64344, &UA2719387},
	{SG64345, &UA2719484},
	{SG64346, &UA2719486},
	{SG64347, &UA2741411},
	{SG64348, &UA2741412},
	{SG64349, &UA2741413},
	{SG64350, &UA2855816},
	{SG64351, &UA2869864},
	{SG64352, &UA2873810},
	{SG64353, &UA2873926},
	{SG64354, &UA2874095},
	{SG64370, &UA925325},
	{SG64371, &UA925328},
	{SG64372, &UA1995372},
	{SG64373, &UA1997541},
	{SG64374, &UA1998448},
	{SG64375, &UA2000911},
	{SG64376, &UA2003527},
	{SG64377, &UA2003718},
	{SG64378, &UA2005283},
	{SG64379, &UA2005399},
	{SG64380, &UA2007127},
	{SG64381, &UA2016032},
	{SG64382, &UA2016051},
	{SG64393, &UA911573},
	{SG64394, &UA912386},
	{SG64395, &UA912402},
	{SG64396, &UA912412},
	{SG64397, &UA912423},
	{SG64398, &UA912510},
	{SG64399, &UA923999},
	{SG64400, &UA1987545},
	{SG64401, &UA2010801},
	{SG64402, &UA2597963},
	{SG64403, &UA2946083},
	{SG64404, &UA2947935},
	{SG64405, &UA3015160},
	{SG64406, &UA3015891},
	{SG64431, &UA2200611},
	{SG64432, &UA2402836},
	{SG64433, &UA2424317},
	{SG64434, &UA2424827},
	{SG64435, &UA2426421},
	{SG64436, &UA2675171},
	{SG64437, &UA2994686},
	{SG64438, &UA910376},
	{SG64439, &UA912442},
	{SG64440, &UA924096},
	{SG64441, &UA2163689},
	{SG64442, &UA2260004},
	{SG64443, &UA2674502},
	{SG64444, &UA2674527},
	{SG64445, &UA2674870},
	{SG64446, &UA2674884},
	{SG64447, &UA2675012},
	{SG64448, &UA2717995},
	{SG64449, &UA2718531},
	{SG64450, &UA2738779},
	{SG64451, &UA2832767},
	{SG64452, &UA3043671},
	{SG64480, &UA2972407},
	{SG64481, &UA2973597},
	{SG64482, &UA3021704},
	{SG64483, &UA3025618},
	{SG64484, &UA3058258},
	{SG64485, &UA3058263},
	{SG64486, &UA3078236},
	{SG64534, &UA2016039},
	{SG64535, &UA2016030},
	{SG64536, &UA2186224},
	{SG64537, &UA2553892},
	{SG64538, &UA2842242},
	{SG64545, &UA2193631},
	{SG64546, &UA2194064},
	{SG64547, &UA2201648},
	{SG64549, &UA2201753},
	{SG64551, &UA2203737},
	{SG64555, &UA2203995},
	{SG64561, &UA2258889},
	{SG64609, &UA912014},
	{SG64610, &UA925347},
	{SG64611, &UA2008926},
	{SG64612, &UA2017931},
	{SG64613, &UA2199101},
	{SG64614, &UA2199445},
	{SG64615, &UA2359260},
	{SG64616, &UA2359274},
	{SG64617, &UA2431874},
	{SG64618, &UA2431879},
	{SG64619, &UA2533975},
	{SG64620, &UA2536983},
	{SG64621, &UA2605222},
	{SG64622, &UA2605246},
	{SG64623, &UA2713969},
	{SG64624, &UA2718668},
	{SG64625, &UA2877632},
	{SG64626, &UA2877690},
	{SG64627, &UA2946739},
	{SG64628, &UA2947233},
	{SG64629, &UA3083972},
	{SG64630, &UA3111763},
	{SG64631, &UA2408255},
	{SG64632, &UA2533375},
	{SG64633, &UA2714027},
	{SG64634, &UA2717129},
	{SG64635, &UA2869980},
	{SG64636, &UA3118454},
	{SG64676, &UA2408254},
	{SG64677, &UA2434550},
	{SG64678, &UA2458914},
	{SG64679, &UA2582676},
	{SG64680, &UA2604430},
	{SG64681, &UA2712024},
	{SG64682, &UA2752712},
	{SG64683, &UA2753369},
	{SG64684, &UA2754393},
	{SG64685, &UA2768922},
	{SG64716, &UA2569938},
	{SG64717, &UA2571463},
	{SG64718, &UA2667590},
	{SG64720, &UA2668142},
	{SG64722, &UA2672763},
	{SG64724, &UA2704609},
	{SG64726, &UA2707783},
	{SG64728, &UA2993952},
	{SG64744, &UA2885383},
	{SG64745, &UA2885392},
	{SG64746, &UA3027539},
	{SG64747, &UA3027765},
	{SG64748, &UA3028083},
	{SG64749, &UA3029032},
	{SG64750, &UA3029084},
	{SG64751, &UA3029911},
	{SG64752, &UA3030634},
	{SG64753, &UA3059048},
	{SG64759, &UA911700},
	{SG64760, &UA912388},
	{SG64761, &UA912390},
	{SG64762, &UA912650},
	{SG64800, &UA910066},
	{SG64801, &UA910429},
	{SG64802, &UA912385},
	{SG64803, &UA912392},
	{SG64804, &UA912405},
	{SG64805, &UA912543},
	{SG64806, &UA912655},
	{SG64807, &UA912765},
	{SG64808, &UA913280},
	{SG64809, &UA1968912},
	{SG64810, &UA2580298},
	{SG64811, &UA2674679},
	{SG64812, &UA2674818},
	{SG64813, &UA2714925},
	{SG64840, &UA912427},
	{SG64841, &UA2017696},
	{SG64842, &UA2039636},
	{SG64843, &UA2558119},
	{SG64844, &UA2597902},
	{SG64845, &UA2781345},
	{SG64846, &UA3097026},
	{SG64896, &UA696981},
	{SG64897, &UA912420},
	{SG64898, &UA1951419},
	{SG64899, &UA2675769},
	{SG64913, &UA894930},
	{SG64914, &UA898218},
	{SG64915, &UA900739},
	{SG64916, &UA908315},
	{SG64917, &UA910385},
	{SG64918, &UA911099},
	{SG64919, &UA912521},
	{SG64920, &UA912816},
	{SG64921, &UA1983016},
	{SG64979, &UA924306},
	{SG64980, &UA924753},
	{SG64981, &UA2260096},
	{SG64983, &UA2348606},
	{SG64985, &UA2355495},
	{SG64986, &UA2400651},
	{SG64987, &UA2830304},
	{SG65010, &UA2947601},
	{SG65011, &UA2948535},
	{SG65012, &UA3017685},
	{SG65059, &UA2884696},
	{SG65060, &UA3027536},
	{SG65061, &UA3028062},
	{SG65062, &UA3028481},
	{SG65063, &UA3028933},
	{SG65064, &UA3029036},
	{SG65065, &UA3029735},
	{SG65066, &UA3030041},
	{SG65067, &UA3030056},
	{SG65068, &UA3030550},
	{SG65111, &UA2947936},
	{SG65112, &UA3016441},
	{SG65124, &UA3017738},
	{SG65125, &UA3018937},
	{SG65126, &UA3020360},
	{SG65172, &UA910095},
	{SG65173, &UA910232},
	{SG65174, &UA910549},
	{SG65175, &UA910815},
	{SG65176, &UA910999},
	{SG65177, &UA911270},
	{SG65178, &UA911561},
	{SG65179, &UA911728},
	{SG65180, &UA1990708},
	{SG65181, &UA2674474},
	{SG65182, &UA2752109},
	{SG65205, &UA2202182},
	{SG65206, &UA2232349},
	{SG65207, &UA2844293},
	{SG65208, &UA2595820},
	{SG65209, &UA2984016},
	{SG65210, &UA2407288},
	{SG65211, &UA2430138},
	{SG65212, &UA2717810},
	{SG65213, &UA2738264},
	{SG65214, &UA2238849},
	{SG65215, &UA2259002},
	{SG65223, &UA3062795},
	{SG65231, &UA903954},
	{SG65232, &UA910776},
	{SG65233, &UA912899},
	{SG65234, &UA924878},
	{SG65235, &UA924881},
	{SG65236, &UA925090},
	{SG65237, &UA1997319},
	{SG65238, &UA1997549},
	{SG65239, &UA1998276},
	{SG65240, &UA2007339},
	{SG65241, &UA2007353},
	{SG65242, &UA2010374},
	{SG65243, &UA2010480},
	{SG65244, &UA2010797},
	{SG65245, &UA2041666},
	{SG65246, &UA2042018},
	{SG65247, &UA2042908},
	{SG65248, &UA2043323},
	{SG65249, &UA2201944},
	{SG65250, &UA2232322},
	{SG65251, &UA2232329},
	{SG65253, &UA2233786},
	{SG65255, &UA2236181},
	{SG65257, &UA2236182},
	{SG65259, &UA2236185},
	{SG65261, &UA2386915},
	{SG65263, &UA925354},
	{SG65264, &UA2008929},
	{SG65265, &UA2198999},
	{SG65266, &UA2359480},
	{SG65267, &UA2431877},
	{SG65268, &UA2531328},
	{SG65269, &UA2605179},
	{SG65270, &UA2717852},
	{SG65271, &UA2877646},
	{SG65272, &UA2946106},
	{SG65273, &UA3111767},
	{SG65280, &UA2237744},
	{SG65281, &UA2238894},
	{SG65282, &UA2402626},
	{SG65283, &UA2426400},
	{SG65308, &UA686796},
	{SG65311, &UA2043504},
	{SG65314, &UA2402813},
	{SG65337, &UA2508021},
	{SG65338, &UA2510275},
	{SG65339, &UA2731007},
	{SG65340, &UA2851562},
	{SG65352, &UA2205249},
	{SG65353, &UA2525056},
	{SG65354, &UA2834302},
	{SG65377, &UA2674855},
	{SG65378, &UA2714964},
	{SG65379, &UA2715036},
	{SG65380, &UA2715031},
	{SG65381, &UA2716152},
	{SG65382, &UA2884698},
	{SG65383, &UA2943977},
	{SG65384, &UA3027527},
	{SG65385, &UA3028002},
	{SG65386, &UA3029602},
	{SG65387, &UA3029913},
	{SG65388, &UA3031401},
	{SG65397, &UA2360578},
	{SG65398, &UA2360820},
	{SG65411, &UA2011682},
	{SG65412, &UA2011854},
	{SG65480, &UA2197575},
	{SG65481, &UA2197577},
	{SG65484, &UA2470943},
	{SG65485, &UA2470944},
	{SG65486, &UA2986363},
	{SG65489, &UA2428454},
	{SG65490, &UA2583284},
	{SG65491, &UA2739971},
	{SG65492, &UA2883823},
	{SG65493, &UA2963084},
	{SG65494, &UA3029635},
	{SG65495, &UA3095231},
	{SG65496, &UA2945943},
	{SG65497, &UA2969148},
	{SG65498, &UA3028763},
	{SG65499, &UA895155},
	{SG65500, &UA908805},
	{SG65501, &UA909537},
	{SG65502, &UA909874},
	{SG65503, &UA909946},
	{SG65504, &UA912600},
	{SG65505, &UA912627},
	{SG65507, &UA2680157},
	{SG65508, &UA2680488},
	{SG65516, &UA3025083},
	{SG65517, &UA2753923},
	{SG65518, &UA2784543},
	{SG65520, &UA2875520},
	{SG65544, &UA2487638},
	{SG65545, &UA2487639},
	{SG65546, &UA2774331},
	{SG65547, &UA2885387},
	{SG65548, &UA2969602},
	{SG65549, &UA3026377},
	{SG65550, &UA3027884},
	{SG65551, &UA3028474},
	{SG65552, &UA3030064},
	{SG65553, &UA3108062},
	{SG65554, &UA912738},
	{SG65555, &UA913263},
	{SG65573, &UA2225028},
	{SG65574, &UA2225165},
	{SG65575, &UA2362129},
	{SG65576, &UA2367782},
	{SG65577, &UA2433551},
	{SG65578, &UA2462074},
	{SG65579, &UA2479294},
	{SG65580, &UA2530842},
	{SG65581, &UA2606859},
	{SG65582, &UA2670483},
	{SG65583, &UA2712550},
	{SG65584, &UA2718659},
	{SG65585, &UA2877522},
	{SG65586, &UA2887636},
	{SG65587, &UA2945815},
	{SG65588, &UA2971266},
	{SG65589, &UA3114758},
	{SG65590, &UA3115275},
	{SG65591, &UA2566061},
	{SG65592, &UA2566293},
	{SG65610, &UA911754},
	{SG65611, &UA2040414},
	{SG65612, &UA2235745},
	{SG65613, &UA2359228},
	{SG65614, &UA2433553},
	{SG65615, &UA2564493},
	{SG65616, &UA2671070},
	{SG65617, &UA2710021},
	{SG65618, &UA2877751},
	{SG65619, &UA2968054},
	{SG65620, &UA3097278},
	{SG65621, &UA2874146},
	{SG65622, &UA3026778},
	{SG65644, &UA686164},
	{SG65645, &UA2670823},
	{SG65646, &UA2678089},
	{SG65647, &UA2716700},
	{SG65648, &UA2948373},
	{SG65649, &UA2969415},
	{SG65673, &UA2408362},
	{SG65674, &UA2570156},
	{SG65675, &UA2772763},
	{SG65678, &UA2434173},
	{SG65679, &UA3026825},
	{SG65683, &UA2568848},
	{SG65684, &UA3031101},
	{SG65685, &UA641789},
	{SG65686, &UA668374},
	{SG65687, &UA895252},
	{SG65688, &UA898477},
	{SG65689, &UA900407},
	{SG65690, &UA902002},
	{SG65691, &UA903438},
	{SG65692, &UA904678},
	{SG65693, &UA905433},
	{SG65694, &UA909493},
	{SG65695, &UA910788},
	{SG65696, &UA912504},
	{SG65697, &UA913269},
	{SG65698, &UA925315},
	{SG65699, &UA925510},
	{SG65700, &UA2222817},
	{SG65701, &UA2873711},
	{SG65702, &UA3051327},
	{SG65705, &UA2678888},
	{SG65712, &UA2492385},
	{SG65715, &UA2539961},
	{SG65716, &UA2570552},
	{SG65717, &UA2571084},
	{SG65718, &UA2857136},
	{SG65722, &UA2874067},
	{SG65727, &UA2258472},
	{SG65728, &UA3025919},
	{SG65739, &UA2470970},
	{SG65740, &UA2492004},
	{SG65741, &UA2512050},
	{SG65742, &UA2408423},
	{SG65746, &UA2360780},
	{SG65747, &UA2458075},
	{SG65748, &UA2695419},
	{SG65757, &UA2361795},
	{SG65759, &UA2387049},
	{SG65761, &UA2407032},
	{SG65764, &UA2435567},
	{SG65767, &UA2458336},
	{SG65769, &UA2435543},
	{SG65771, &UA2437659},
	{SG65773, &UA2661166},
	{SG65775, &UA2766496},
	{SG65777, &UA2779268},
	{SG65779, &UA2890464},
	{SG65781, &UA2891956},
	{SG65782, &UA2935888},
	{SG65785, &UA2954687},
	{SG65787, &UA2955854},
	{SG65789, &UA3030098},
	{SG65791, &UA3031793},
	{SG65793, &UA3032059},
	{SG65795, &UA3034862},
	{SG65797, &UA3036344},
	{SG65799, &UA3058915},
	{SG65801, &UA3058957},
	{SG65803, &UA3059897},
	{SG65805, &UA3060070},
	{SG65807, &UA3079517},
	{SG65808, &UA3107996},
	{SG65810, &UA3108100},
	{SG65812, &UA3125238},
	{SG65814, &UA3125256},
	{SG65816, &UA3125461},
	{SG65825, &UA2774072},
	{SG65826, &UA2774136},
	{SG65827, &UA2877450},
	{SG65828, &UA2877512},
	{SG65831, &UA2204002},
	{SG65835, &UA2204098},
	{SG65836, &UA2507447},
	{SG65837, &UA2529239},
	{SG65838, &UA2737760},
	{SG65839, &UA2568328},
	{SG65840, &UA2012426},
	{SG65841, &UA2170811},
	{SG65842, &UA2195034},
	{SG65843, &UA2197591},
	{SG65844, &UA2366415},
	{SG65845, &UA2385036},
	{SG65848, &UA2673799},
	{SG65859, &UA2556703},
	{SG65860, &UA2567067},
	{SG65861, &UA2768934},
	{SG65862, &UA3027729},
	{SG65863, &UA925308},
	{SG65864, &UA1997182},
	{SG65865, &UA2001933},
	{SG65866, &UA2005456},
	{SG65867, &UA2006443},
	{SG65868, &UA2011091},
	{SG65869, &UA2040804},
	{SG65870, &UA2042073},
	{SG65875, &UA2361209},
	{SG65876, &UA2369706},
	{SG65880, &UA2487605},
	{SG65907, &UA2430925},
	{SG65908, &UA2437346},
	{SG65909, &UA2040800},
	{SG65910, &UA2529671},
	{SG65911, &UA2555747},
	{SG65912, &UA2737759},
	{SG65925, &UA2487641},
	{SG65926, &UA2598101},
	{SG65927, &UA2660052},
	{SG65928, &UA912773},
	{SG65929, &UA2160826},
	{SG65930, &UA2223182},
	{SG65935, &UA2674010},
	{SG65936, &UA2862208},
	{SG65937, &UA2936657},
	{SG66016, &UA2566573},
	{SG66019, &UA2704289},
	{SG66021, &UA2935354},
	{SG66032, &UA3035431},
	{SG66033, &UA3068054},
	{SG66034, &UA3084200},
	{SG66035, &UA3084230},
	{SG66036, &UA3084232},
	{SG66037, &UA3084244},
	{SG66038, &UA3084269},
	{SG66039, &UA3084270},
	{SG66040, &UA3084271},
	{SG66041, &UA3084274},
	{SG66042, &UA3084280},
	{SG66047, &UA3029407},
	{SG66048, &UA2042275},
	{SG66049, &UA2876038},
	{SG66050, &UA2783291},
	{SG66051, &UA3027547},
	{SG66052, &UA3058777},
	{SG66064, &UA2885322},
	{SG66065, &UA3027769},
	{SG66066, &UA3058740},
	{SG66070, &UA2487643},
	{SG66071, &UA2487936},
	{SG66072, &UA2676526},
	{SG66073, &UA2679537},
	{SG66088, &UA2368018},
	{SG66089, &UA2391584},
	{SG66092, &UA3039906},
	{SG66093, &UA3039907},
	{SG66094, &UA3039908},
	{SG66095, &UA3039909},
	{SG66096, &UA3039910},
	{SG66097, &UA3039912},
	{SG66098, &UA3039913},
	{SG66099, &UA3039914},
	{SG66100, &UA3039916},
	{SG66101, &UA3039917},
	{SG66102, &UA3039919},
	{SG66103, &UA3039924},
	{SG66104, &UA3039925},
	{SG66105, &UA3039926},
	{SG66106, &UA3039927},
	{SG66107, &UA3039928},
	{SG66108, &UA3039929},
	{SG66109, &UA3039930},
	{SG66110, &UA3039931},
	{SG66111, &UA3039933},
	{SG66112, &UA3039934},
	{SG66113, &UA3039935},
	{SG66114, &UA3039936},
	{SG66115, &UA3039938},
	{SG66116, &UA3039939},
	{SG66117, &UA3039941},
	{SG66118, &UA3039943},
	{SG66119, &UA3039944},
	{SG66120, &UA3039945},
	{SG66121, &UA3039946},
	{SG66122, &UA3039947},
	{SG66123, &UA3039948},
	{SG66124, &UA3039949},
	{SG66125, &UA3039950},
	{SG66126, &UA3039951},
	{SG66127, &UA3039954},
	{SG66128, &UA3039955},
	{SG66129, &UA3039956},
	{SG66130, &UA3039957},
	{SG66131, &UA3039958},
	{SG66135, &UA3039959},
	{SG66136, &UA3039960},
	{SG66137, &UA3039961},
	{SG66138, &UA3039963},
	{SG66139, &UA3039964},
	{SG66140, &UA3039965},
	{SG66141, &UA3039967},
	{SG66142, &UA3039969},
	{SG66143, &UA3039970},
	{SG66144, &UA3039972},
	{SG66145, &UA3039973},
	{SG66146, &UA3039971},
	{SG66147, &UA3039974},
	{SG66148, &UA3039975},
	{SG66149, &UA3039976},
	{SG66150, &UA3039977},
	{SG66151, &UA3039978},
	{SG66152, &UA3039979},
	{SG66153, &UA3039980},
	{SG66154, &UA3039981},
	{SG66155, &UA3039982},
	{SG66156, &UA3039983},
	{SG66157, &UA3039984},
	{SG66158, &UA3039985},
	{SG66159, &UA3039986},
	{SG66161, &UA3039993},
	{SG66163, &UA3040002},
	{SG66164, &UA3040003},
	{SG66165, &UA3040005},
	{SG66166, &UA3040007},
	{SG66167, &UA3040008},
	{SG66168, &UA3040009},
	{SG66169, &UA3040006},
	{SG66170, &UA3040010},
	{SG66171, &UA3040011},
	{SG66172, &UA3040012},
	{SG66173, &UA3040013},
	{SG66174, &UA3040014},
	{SG66175, &UA3040015},
	{SG66176, &UA3040016},
	{SG66177, &UA3040017},
	{SG66179, &UA3040018},
	{SG66181, &UA3040019},
	{SG66182, &UA3040020},
	{SG66183, &UA3040021},
	{SG66184, &UA3040022},
	{SG66185, &UA3040023},
	{SG66186, &UA3040024},
	{SG66187, &UA3040025},
	{SG66188, &UA3040026},
	{SG66189, &UA3040027},
	{SG66190, &UA3040028},
	{SG66191, &UA3040029},
	{SG66192, &UA3040030},
	{SG66193, &UA3040031},
	{SG66194, &UA3040032},
	{SG66195, &UA3040033},
	{SG66196, &UA3040035},
	{SG66197, &UA3039932},
	{SG66198, &UA3040036},
	{SG66199, &UA3040037},
	{SG66200, &UA3040038},
	{SG66201, &UA3040039},
	{SG66202, &UA3040040},
	{SG66203, &UA3040041},
	{SG66204, &UA3040042},
	{SG66205, &UA3040043},
	{SG66206, &UA3040044},
	{SG66207, &UA3040045},
	{SG66208, &UA3040046},
	{SG66209, &UA3040047},
	{SG66210, &UA3040048},
	{SG66211, &UA3040049},
	{SG66212, &UA3040050},
	{SG66213, &UA3040051},
	{SG66214, &UA3040052},
	{SG66215, &UA3040053},
	{SG66216, &UA3040054},
	{SG66217, &UA3040055},
	{SG66218, &UA3040056},
	{SG66219, &UA3040057},
	{SG66220, &UA3040058},
	{SG66221, &UA3040059},
	{SG66222, &UA3040060},
	{SG66223, &UA3040062},
	{SG66224, &UA3040063},
	{SG66225, &UA3040065},
	{SG66226, &UA3040066},
	{SG66227, &UA3040067},
	{SG66228, &UA3040034},
	{SG66229, &UA3040071},
	{SG66230, &UA3040073},
	{SG66231, &UA3040074},
	{SG66232, &UA3040075},
	{SG66233, &UA3040076},
	{SG66234, &UA3040077},
	{SG66235, &UA3040079},
	{SG66236, &UA3040080},
	{SG66237, &UA3040081},
	{SG66238, &UA3040070},
	{SG66239, &UA3040083},
	{SG66240, &UA3040084},
	{SG66241, &UA3040086},
	{SG66242, &UA3040085},
	{SG66243, &UA3040088},
	{SG66244, &UA3040087},
	{SG66245, &UA3040089},
	{SG66246, &UA3040090},
	{SG66247, &UA3040092},
	{SG66248, &UA3040093},
	{SG66249, &UA3040094},
	{SG66250, &UA3040095},
	{SG66251, &UA3040096},
	{SG66252, &UA3040068},
	{SG66253, &UA3040110},
	{SG66254, &UA3040111},
	{SG66255, &UA3040112},
	{SG66256, &UA3040113},
	{SG66257, &UA3040114},
	{SG66258, &UA3040115},
	{SG66259, &UA3040116},
	{SG66260, &UA3040117},
	{SG66261, &UA3040118},
	{SG66262, &UA3040120},
	{SG66263, &UA3040119},
	{SG66264, &UA3040122},
	{SG66265, &UA3040123},
	{SG66266, &UA3040124},
	{SG66267, &UA3040126},
	{SG66268, &UA3040129},
	{SG66269, &UA3040132},
	{SG66270, &UA3040133},
	{SG66271, &UA3040134},
	{SG66272, &UA3040135},
	{SG66273, &UA3040136},
	{SG66274, &UA3040137},
	{SG66275, &UA3040138},
	{SG66276, &UA3040139},
	{SG66277, &UA3040140},
	{SG66278, &UA3040141},
	{SG66279, &UA3040142},
	{SG66280, &UA3040144},
	{SG66281, &UA3040145},
	{SG66282, &UA3040106},
	{SG66283, &UA3040147},
	{SG66284, &UA3040148},
	{SG66285, &UA3040149},
	{SG66286, &UA3040150},
	{SG66287, &UA3040151},
	{SG66288, &UA3040152},
	{SG66289, &UA3040153},
	{SG66290, &UA3040156},
	{SG66291, &UA3040160},
	{SG66292, &UA3040161},
	{SG66293, &UA3040143},
	{SG66294, &UA3040181},
	{SG66295, &UA3040183},
	{SG66296, &UA3040185},
	{SG66297, &UA3040186},
	{SG66298, &UA3040188},
	{SG66299, &UA3040189},
	{SG66300, &UA3040190},
	{SG66301, &UA3040191},
	{SG66302, &UA3040192},
	{SG66303, &UA3040193},
	{SG66304, &UA3040194},
	{SG66305, &UA3040196},
	{SG66306, &UA3040197},
	{SG66307, &UA3040198},
	{SG66308, &UA3040195},
	{SG66309, &UA3040199},
	{SG66310, &UA3040200},
	{SG66311, &UA3040201},
	{SG66312, &UA3040202},
	{SG66313, &UA3040203},
	{SG66314, &UA3040204},
	{SG66315, &UA3040206},
	{SG66316, &UA3040174},
	{SG66317, &UA3040207},
	{SG66318, &UA3040208},
	{SG66319, &UA3040209},
	{SG66320, &UA3040210},
	{SG66321, &UA3040211},
	{SG66322, &UA3040212},
	{SG66323, &UA3040213},
	{SG66324, &UA3040214},
	{SG66325, &UA3040215},
	{SG66326, &UA3040216},
	{SG66327, &UA3040217},
	{SG66328, &UA3040218},
	{SG66329, &UA3040219},
	{SG66330, &UA3040220},
	{SG66331, &UA3040221},
	{SG66332, &UA3040222},
	{SG66333, &UA3040223},
	{SG66334, &UA3040225},
	{SG66335, &UA3040226},
	{SG66336, &UA3040227},
	{SG66337, &UA3040228},
	{SG66338, &UA3040229},
	{SG66339, &UA3040231},
	{SG66340, &UA3040230},
	{SG66341, &UA3040232},
	{SG66342, &UA3040233},
	{SG66343, &UA3040234},
	{SG66344, &UA3040205},
	{SG66345, &UA3040235},
	{SG66346, &UA3040236},
	{SG66347, &UA3040238},
	{SG66348, &UA3040240},
	{SG66349, &UA3040241},
	{SG66350, &UA3040243},
	{SG66351, &UA3040242},
	{SG66352, &UA3040244},
	{SG66353, &UA3040245},
	{SG66354, &UA3040249},
	{SG66355, &UA3040250},
	{SG66356, &UA3040251},
	{SG66357, &UA3040253},
	{SG66358, &UA3040252},
	{SG66359, &UA3040254},
	{SG66360, &UA3040255},
	{SG66361, &UA3040256},
	{SG66362, &UA3040257},
	{SG66363, &UA3040258},
	{SG66364, &UA3040239},
	{SG66365, &UA3040260},
	{SG66366, &UA3040261},
	{SG66367, &UA3040262},
	{SG66368, &UA3040263},
	{SG66369, &UA3040264},
	{SG66370, &UA3040265},
	{SG66371, &UA3040266},
	{SG66372, &UA3040267},
	{SG66373, &UA3040268},
	{SG66374, &UA3040269},
	{SG66375, &UA3040270},
	{SG66376, &UA3040259},
	{SG66377, &UA3040272},
	{SG66378, &UA3040273},
	{SG66379, &UA3040275},
	{SG66380, &UA3040274},
	{SG66381, &UA3040276},
	{SG66382, &UA3040277},
	{SG66383, &UA3040278},
	{SG66384, &UA3040279},
	{SG66385, &UA3040280},
	{SG66386, &UA3040281},
	{SG66387, &UA3040282},
	{SG66388, &UA3040284},
	{SG66389, &UA3040285},
	{SG66390, &UA3040286},
	{SG66391, &UA3040287},
	{SG66392, &UA3040288},
	{SG66393, &UA3040289},
	{SG66394, &UA3040290},
	{SG66395, &UA3040292},
	{SG66396, &UA3040293},
	{SG66397, &UA3040294},
	{SG66398, &UA3040295},
	{SG66399, &UA3040296},
	{SG66400, &UA3040297},
	{SG66401, &UA3040299},
	{SG66402, &UA3040300},
	{SG66403, &UA3040301},
	{SG66404, &UA3040302},
	{SG66405, &UA3040303},
	{SG66406, &UA3040271},
	{SG66407, &UA3040308},
	{SG66408, &UA3040309},
	{SG66409, &UA3040310},
	{SG66410, &UA3040311},
	{SG66411, &UA3040312},
	{SG66412, &UA3040313},
	{SG66413, &UA3040314},
	{SG66414, &UA3040315},
	{SG66415, &UA3040316},
	{SG66418, &UA3040317},
	{SG66419, &UA3040318},
	{SG66420, &UA3040319},
	{SG66421, &UA3040320},
	{SG66422, &UA3040321},
	{SG66423, &UA3040322},
	{SG66425, &UA3040323},
	{SG66426, &UA3040324},
	{SG66427, &UA3040325},
	{SG66428, &UA3040326},
	{SG66429, &UA3040328},
	{SG66430, &UA3040329},
	{SG66432, &UA3040330},
	{SG66434, &UA3040331},
	{SG66435, &UA3040332},
	{SG66436, &UA3040334},
	{SG66437, &UA3040333},
	{SG66438, &UA3040335},
	{SG66439, &UA3040336},
	{SG66440, &UA3040337},
	{SG66441, &UA3040338},
	{SG66442, &UA3040339},
	{SG66443, &UA3040305},
	{SG66444, &UA3040342},
	{SG66445, &UA3040343},
	{SG66446, &UA3040344},
	{SG66447, &UA3040345},
	{SG66448, &UA3040346},
	{SG66449, &UA3040347},
	{SG66450, &UA3040348},
	{SG66451, &UA3040349},
	{SG66452, &UA3040350},
	{SG66453, &UA3040351},
	{SG66454, &UA3040352},
	{SG66455, &UA3040355},
	{SG66456, &UA3040356},
	{SG66457, &UA3040357},
	{SG66458, &UA3040359},
	{SG66459, &UA3040360},
	{SG66460, &UA3040361},
	{SG66461, &UA3040362},
	{SG66462, &UA3040363},
	{SG66463, &UA3040365},
	{SG66464, &UA3040366},
	{SG66465, &UA3040368},
	{SG66466, &UA3040369},
	{SG66467, &UA3040370},
	{SG66468, &UA3040340},
	{SG66469, &UA3040374},
	{SG66470, &UA3040375},
	{SG66471, &UA3040376},
	{SG66472, &UA3040377},
	{SG66473, &UA3040378},
	{SG66474, &UA3040379},
	{SG66475, &UA3040380},
	{SG66476, &UA3040381},
	{SG66477, &UA3040383},
	{SG66478, &UA3040385},
	{SG66479, &UA3040388},
	{SG66480, &UA3040371},
	{SG66486, &UA2432755},
	{SG66496, &UA2233055},
	{SG66498, &UA2233056},
	{SG66510, &UA2674595},
	{SG66511, &UA2260473},
	{SG66513, &UA2566281},
	{SG66514, &UA2569450},
	{SG66521, &UA2200193},
	{SG66522, &UA2236471},
	{SG66528, &UA2569098},
	{SG66529, &UA2405710},
	{SG66530, &UA2580010},
	{SG66531, &UA2799108},
	{SG66532, &UA3115273},
	{SG66552, &UA2531760},
	{SG66553, &UA2585273},
	{SG66554, &UA2765551},
	{SG66555, &UA2780915},
	{SG66556, &UA3114968},
	{SG66561, &UA2002586},
	{SG66562, &UA2014789},
	{SG66563, &UA2671191},
	{SG66564, &UA2672588},
	{SG66565, &UA2040947},
	{SG66566, &UA2201585},
	{SG66567, &UA2454775},
	{SG66568, &UA2601731},
	{SG66569, &UA2797053},
	{SG66570, &UA3115547},
	{SG66581, &UA2674164},
	{SG66582, &UA2674177},
	{SG66583, &UA2674665},
	{SG66584, &UA2715048},
	{SG66585, &UA2736681},
	{SG66588, &UA912800},
	{SG66589, &UA2528550},
	{SG66590, &UA2528880},
	{SG66591, &UA2737758},
	{SG66592, &UA2674521},
	{SG66593, &UA2715044},
	{SG66594, &UA2716002},
	{SG66595, &UA2492413},
	{SG66596, &UA2042759},
	{SG66597, &UA2840195},
	{SG66599, &UA2197905},
	{SG66600, &UA2203949},
	{SG66601, &UA2361556},
	{SG66602, &UA2595492},
	{SG66603, &UA913169},
	{SG66604, &UA1992533},
	{SG66605, &UA924959},
	{SG66606, &UA925081},
	{SG66607, &UA1996213},
	{SG66608, &UA1997125},
	{SG66609, &UA2001935},
	{SG66610, &UA2003607},
	{SG66611, &UA2003723},
	{SG66613, &UA2005587},
	{SG66615, &UA2010802},
	{SG66616, &UA2014090},
	{SG66617, &UA2014085},
	{SG66618, &UA2016058},
	{SG66619, &UA2040842},
	{SG66620, &UA2040843},
	{SG66621, &UA2041406},
	{SG66622, &UA2042076},
	{SG66623, &UA2043391},
	{SG66624, &UA2043591},
	{SG66625, &UA2810428},
	{SG66628, &UA2359564},
	{SG66629, &UA2362339},
	{SG66643, &UA2387948},
	{SG66644, &UA2388478},
	{SG66647, &UA907781},
	{SG66648, &UA2674480},
	{SG66649, &UA2715170},
	{SG66654, &UA2675748},
	{SG66655, &UA2676000},
	{SG66657, &UA2675159},
	{SG66658, &UA2781690},
	{SG66659, &UA2866400},
	{SG66660, &UA3096216},
	{SG66662, &UA2204111},
	{SG66663, &UA2487653},
	{SG66664, &UA2487939},
	{SG66665, &UA2567348},
	{SG66666, &UA2582988},
	{SG66667, &UA2674149},
	{SG66668, &UA2719144},
	{SG66683, &UA2040854},
	{SG66685, &UA2042680},
	{SG66688, &UA2007126},
	{SG66689, &UA2040311},
	{SG66690, &UA3085813},
	{SG66700, &UA1999116},
	{SG66701, &UA2016858},
	{SG66702, &UA2197912},
	{SG66703, &UA2234844},
	{SG66704, &UA2455147},
	{SG66720, &UA2585036},
	{SG66721, &UA2667506},
	{SG66775, &UA2606127},
	{SG66776, &UA2606148},
	{SG66785, &UA2237413},
	{SG66786, &UA2237720},
	{SG66789, &UA2509342},
	{SG66790, &UA2513109},
	{SG66791, &UA2195620},
	{SG66792, &UA2200198},
	{SG66793, &UA2774070},
	{SG66794, &UA2877511},
	{SG66795, &UA2259342},
	{SG66796, &UA2259962},
	{SG66797, &UA2259998},
	{SG66798, &UA2260002},
	{SG66799, &UA2260003},
	{SG66801, &UA2260049},
	{SG66805, &UA923837},
	{SG66806, &UA1992230},
	{SG66814, &UA2232359},
	{SG66826, &UA913458},
	{SG66827, &UA913460},
	{SG66828, &UA2042123},
	{SG66829, &UA2202962},
	{SG66834, &UA2537298},
	{SG66835, &UA2537303},
	{SG66840, &UA2674519},
	{SG66841, &UA2719151},
	{SG66845, &UA2001271},
	{SG66846, &UA2001288},
	{SG66847, &UA2710123},
	{SG66848, &UA2774163},
	{SG66849, &UA2877572},
	{SG66850, &UA2900033},
	{SG66851, &UA2879694},
	{SG66852, &UA2886225},
	{SG66875, &UA2424489},
	{SG66876, &UA2595540},
	{SG66879, &UA910820},
	{SG66880, &UA2436518},
	{SG66881, &UA2009119},
	{SG66882, &UA2009313},
	{SG66887, &UA2356390},
	{SG66888, &UA2602711},
	{SG66889, &UA2879626},
	{SG66900, &UA911651},
	{SG66901, &UA911655},
	{SG66902, &UA2231579},
	{SG66903, &UA2259969},
	{SG66904, &UA2424406},
	{SG66918, &UA2710309},
	{SG66919, &UA2717248},
	{SG66920, &UA2362332},
	{SG66921, &UA2362348},
	{SG66922, &UA2579440},
	{SG66925, &UA2041821},
	{SG66927, &UA2715880},
	{SG66928, &UA2715993},
	{SG66945, &UA2202165},
	{SG66946, &UA2202172},
	{SG66948, &UA2238570},
	{SG66949, &UA2259034},
	{SG66953, &UA2205738},
	{SG66954, &UA2679617},
	{SG66955, &UA2719396},
	{SG66956, &UA913435},
	{SG66961, &UA2738205},
	{SG66962, &UA2774139},
	{SG66963, &UA2877567},
	{SG66964, &UA3019354},
	{SG66970, &UA2513235},
	{SG66971, &UA2532348},
	{SG66985, &UA1997065},
	{SG66986, &UA2547845},
	{SG66992, &UA2535267},
	{SG66999, &UA1996172},
	{SG67000, &UA2204769},
	{SG67001, &UA2651006},
	{SG67002, &UA1950985},
	{SG67003, &UA1963938},
	{SG67008, &UA2361358},
	{SG67009, &UA2368524},
	{SG67020, &UA2356282},
	{SG67021, &UA2424054},
	{SG67025, &UA2437393},
	{SG67026, &UA2505953},
	{SG67032, &UA1996888},
	{SG67033, &UA2001939},
	{SG67034, &UA2200547},
	{SG67035, &UA2673132},
	{SG67036, &UA3012748},
	{SG67037, &UA3019117},
	{SG67038, &UA3020748},
	{SG67039, &UA2001279},
	{SG67040, &UA2724412},
	{SG67045, &UA2677381},
	{SG67046, &UA908320},
	{SG67047, &UA910587},
	{SG67048, &UA910713},
	{SG67049, &UA912587},
	{SG67050, &UA913001},
	{SG67081, &UA1995925},
	{SG67082, &UA1996394},
	{SG67086, &UA2009128},
	{SG67087, &UA2546530},
	{SG67090, &UA912031},
	{SG67093, &UA2715421},
	{SG67116, &UA2675553},
	{SG67128, &UA2390922},
	{SG67129, &UA2428431},
	{SG67134, &UA3039895},
	{SG67135, &UA3039897},
	{SG67136, &UA3039898},
	{SG67137, &UA3039899},
	{SG67138, &UA3039901},
	{SG67139, &UA3039902},
	{SG67140, &UA3039903},
	{SG67141, &UA3039911},
	{SG67142, &UA3039942},
	{SG67143, &UA3040061},
	{SG67144, &UA3040064},
	{SG67145, &UA3040072},
	{SG67146, &UA3040078},
	{SG67147, &UA3040082},
	{SG67148, &UA3040091},
	{SG67149, &UA3040097},
	{SG67150, &UA3040098},
	{SG67151, &UA3040100},
	{SG67152, &UA3040101},
	{SG67153, &UA3040102},
	{SG67154, &UA3040103},
	{SG67155, &UA3040109},
	{SG67156, &UA3040125},
	{SG67157, &UA3040187},
	{SG67158, &UA3040224},
	{SG67159, &UA3040237},
	{SG67160, &UA3040246},
	{SG67161, &UA3040247},
	{SG67162, &UA3040248},
	{SG67163, &UA3040283},
	{SG67164, &UA3040291},
	{SG67165, &UA3040304},
	{SG67166, &UA3040327},
	{SG67167, &UA3040341},
	{SG67168, &UA3040353},
	{SG67169, &UA3040358},
	{SG67170, &UA3040364},
	{SG67171, &UA3040367},
	{SG67172, &UA3040372},
	{SG67173, &UA3040373},
	{SG67174, &UA3040384},
	{SG67175, &UA3040386},
	{SG67176, &UA3040389},
	{SG67177, &UA2701570},
	{SG67178, &UA2749715},
	{SG67186, &UA2733246},
	{SG67187, &UA2437422},
	{SG67188, &UA2528995},
	{SG67201, &UA2714942},
	{SG67202, &UA2751515},
	{SG67239, &UA2042674},
	{SG67240, &UA2205860},
	{SG67263, &UA2537305},
	{SG67267, &UA2734593},
	{SG67268, &UA2751274},
	{SG67269, &UA2676562},
	{SG67270, &UA2863150},
	{SG67285, &UA2537307},
	{SG67286, &UA2602938},
	{SG67288, &UA2951579},
	{SG67289, &UA2896726},
	{SG67290, &UA3028625},
	{SG67299, &UA2600012},
	{SG67300, &UA2852647},
	{SG67305, &UA912943},
	{SG67322, &UA2229727},
	{SG67324, &UA924292},
	{SG67325, &UA1994381},
	{SG67336, &UA2954271},
	{SG67337, &UA924775},
	{SG67338, &UA925474},
	{SG67349, &UA2007010},
	{SG67350, &UA2008088},
	{SG67371, &UA2422888},
	{SG67379, &UA2424290},
	{SG67381, &UA3117951},
	{SG67389, &UA2676818},
	{SG67391, &UA3040397},
	{SG67392, &UA3040398},
	{SG67393, &UA3040399},
	{SG67394, &UA3040400},
	{SG67395, &UA3040402},
	{SG67397, &UA3040403},
	{SG67398, &UA3040404},
	{SG67399, &UA3040405},
	{SG67400, &UA3040406},
	{SG67401, &UA3040407},
	{SG67402, &UA3040408},
	{SG67403, &UA3040409},
	{SG67404, &UA3040410},
	{SG67405, &UA3040411},
	{SG67406, &UA3040412},
	{SG67407, &UA3040413},
	{SG67408, &UA3040414},
	{SG67409, &UA3040415},
	{SG67410, &UA3040416},
	{SG67411, &UA3040418},
	{SG67412, &UA3040417},
	{SG67413, &UA3040419},
	{SG67414, &UA3040420},
	{SG67415, &UA3040421},
	{SG67416, &UA3040422},
	{SG67417, &UA3040423},
	{SG67418, &UA3040424},
	{SG67419, &UA3040425},
	{SG67420, &UA3040426},
	{SG67421, &UA3040427},
	{SG67422, &UA3040428},
	{SG67423, &UA3040401},
	{SG67435, &UA2202190},
	{SG67436, &UA2231058},
	{SG67440, &UA2567073},
	{SG67441, &UA2858119},
	{SG67442, &UA2368283},
	{SG67453, &UA3018417},
	{SG67457, &UA2168009},
	{SG67458, &UA2644563},
	{SG67477, &UA2862026},
	{SG67478, &UA2862213},
	{SG67479, &UA2958210},
	{SG67483, &UA2009121},
	{SG67484, &UA2009138},
	{SG67488, &UA2001280},
	{SG67489, &UA2001864},
	{SG67507, &UA3058971},
	{SG67510, &UA2737479},
	{SG67511, &UA2938534},
	{SG67515, &UA911764},
	{SG67516, &UA912026},
	{SG67534, &UA2234070},
	{SG67544, &UA2202194},
	{SG67545, &UA2258843},
	{SG67552, &UA2350343},
	{SG67575, &UA2401918},
	{SG67578, &UA2176714},
	{SG67598, &UA2355174},
	{SG67599, &UA2832660},
	{SG67601, &UA3039987},
	{SG67602, &UA3039988},
	{SG67603, &UA3039989},
	{SG67604, &UA3039990},
	{SG67605, &UA3039991},
	{SG67606, &UA3039992},
	{SG67607, &UA3039994},
	{SG67608, &UA3039995},
	{SG67609, &UA3039996},
	{SG67610, &UA3039997},
	{SG67611, &UA3039999},
	{SG67612, &UA3039998},
	{SG67613, &UA3040000},
	{SG67614, &UA3040001},
	{SG67615, &UA3040004},
	{SG67616, &UA3040154},
	{SG67617, &UA3040155},
	{SG67618, &UA3040157},
	{SG67619, &UA3040158},
	{SG67620, &UA3040159},
	{SG67621, &UA3040162},
	{SG67622, &UA3040163},
	{SG67623, &UA3040164},
	{SG67624, &UA3040165},
	{SG67625, &UA3040166},
	{SG67626, &UA3040167},
	{SG67627, &UA3040168},
	{SG67628, &UA3040169},
	{SG67629, &UA3040170},
	{SG67630, &UA3040171},
	{SG67631, &UA3040173},
	{SG67632, &UA3040172},
	{SG67633, &UA3040175},
	{SG67634, &UA3040176},
	{SG67635, &UA3040177},
	{SG67636, &UA3040178},
	{SG67637, &UA3040179},
	{SG67638, &UA3040180},
	{SG67639, &UA3040182},
	{SG67640, &UA3040184},
	{SG67674, &UA2353363},
	{SG67675, &UA2455705},
	{SG67709, &UA2008108},
	{SG67724, &UA2950691},
	{SG67733, &UA2950753},
	{SG67742, &UA2467965},
	{SG67743, &UA2499957},
	{SG67747, &UA2001291},
	{SG67749, &UA2402780},
	{SG67765, &UA911746},
	{SG67766, &UA912053},
	{SG67767, &UA924478},
	{SG67768, &UA924892},
	{SG67769, &UA2361024},
	{SG67791, &UA2229549},
	{SG67823, &UA2384834},
	{SG67852, &UA2010563},
	{SG67853, &UA2010815},
	{SG67898, &UA2937626},
	{SG67899, &UA3073684},
	{SG67900, &UA3085420},
	{SG67951, &UA2399488},
	{SG67961, &UA3040392},
	{SG67962, &UA3040393},
	{SG67963, &UA3040394},
	{SG67964, &UA3040395},
	{SG67965, &UA3040396},
	{SG67966, &UA3069039},
	{SG67967, &UA3069038},
	{SG67970, &UA1956970},
	{SG67971, &UA1997446},
	{SG67974, &UA3040307},
	{SG67975, &UA3069040},
	{SG67976, &UA3069041},
	{NULL}};
const Segmentdata DChrome30_39[] = {
	{SG62215, &UA3115228},
	{SG62216, &UA3115654},
	{SG62217, &UA3115655},
	{SG62218, &UA3128351},
	{SG62219, &UA3129791},
	{SG62220, &UA3130084},
	{SG62221, &UA3130914},
	{SG63352, &UA3111107},
	{SG63353, &UA3115650},
	{SG63354, &UA3115674},
	{SG63355, &UA3129769},
	{SG63356, &UA3129957},
	{SG63357, &UA3130850},
	{SG63358, &UA3131176},
	{SG63648, &UA3114129},
	{SG63649, &UA3115648},
	{SG63650, &UA3115676},
	{SG63651, &UA3129240},
	{SG63652, &UA3129996},
	{SG63785, &UA3092272},
	{SG63786, &UA3115646},
	{SG63787, &UA3129164},
	{SG63788, &UA3129180},
	{SG63789, &UA3130150},
	{SG63790, &UA3131113},
	{SG63791, &UA3132747},
	{SG64137, &UA3111121},
	{SG64139, &UA3115678},
	{SG64141, &UA3132827},
	{SG64319, &UA3115649},
	{SG64321, &UA3128317},
	{SG64323, &UA3128881},
	{SG64325, &UA3129331},
	{SG64327, &UA3130085},
	{SG64329, &UA3131356},
	{SG64384, &UA3111109},
	{SG64385, &UA3111108},
	{SG64386, &UA3132678},
	{SG64453, &UA3115659},
	{SG64455, &UA3128891},
	{SG64457, &UA3129477},
	{SG64459, &UA3129478},
	{SG64461, &UA3129597},
	{SG64462, &UA3130082},
	{SG64463, &UA3130469},
	{SG64464, &UA3130656},
	{SG64465, &UA3130857},
	{SG64466, &UA3130859},
	{SG64694, &UA3129179},
	{SG65643, &UA3111115},
	{SG66571, &UA2386693},
	{SG66572, &UA3131440},
	{SG66573, &UA3131885},
	{SG67977, &UA3040574},
	{NULL}};
const Segmentdata DDoCoMo[] = {
	{SG65078, &UA2436529},
	{SG65959, &UA86536},
	{SG65960, &UA87608},
	{SG65961, &UA87611},
	{SG65962, &UA87613},
	{SG65963, &UA90390},
	{SG65964, &UA90391},
	{SG65965, &UA90388},
	{SG65966, &UA90387},
	{SG65967, &UA373905},
	{SG65968, &UA374233},
	{SG65969, &UA400337},
	{SG65970, &UA415471},
	{SG65971, &UA432213},
	{SG65972, &UA445955},
	{SG65973, &UA481773},
	{SG65974, &UA484290},
	{SG65975, &UA488784},
	{SG65976, &UA494904},
	{SG65977, &UA503583},
	{SG65978, &UA509977},
	{SG65979, &UA511789},
	{SG65980, &UA512879},
	{SG65981, &UA515927},
	{SG65982, &UA519359},
	{SG65983, &UA520852},
	{SG65984, &UA522113},
	{SG65985, &UA523808},
	{SG65986, &UA554224},
	{SG65987, &UA559326},
	{SG65988, &UA560973},
	{SG65989, &UA561202},
	{SG65990, &UA561474},
	{SG65991, &UA563633},
	{SG65992, &UA566006},
	{SG65993, &UA569238},
	{SG65994, &UA572106},
	{SG65995, &UA635276},
	{SG65996, &UA640569},
	{SG65997, &UA650218},
	{SG65998, &UA671550},
	{SG65999, &UA672949},
	{SG66000, &UA731751},
	{SG66001, &UA736784},
	{SG66002, &UA756240},
	{SG66003, &UA757053},
	{SG66004, &UA762493},
	{SG66005, &UA767630},
	{SG66006, &UA1852317},
	{SG66007, &UA1894829},
	{SG66008, &UA1895017},
	{SG66009, &UA1939258},
	{SG66010, &UA1942069},
	{SG66011, &UA2138221},
	{SG66012, &UA2156211},
	{SG66013, &UA2207793},
	{SG66014, &UA2211611},
	{SG66015, &UA2216597},
	{SG66017, &UA2216717},
	{SG66018, &UA2242247},
	{SG66020, &UA2242301},
	{SG66022, &UA2265098},
	{SG66023, &UA2268190},
	{SG66024, &UA2269522},
	{SG66025, &UA2274781},
	{SG66026, &UA2292935},
	{SG66027, &UA2494656},
	{SG66028, &UA2519542},
	{SG66029, &UA2527501},
	{SG66030, &UA2694810},
	{SG66031, &UA2786169},
	{SG66957, &UA51226},
	{SG67189, &UA56054},
	{SG67190, &UA90285},
	{SG67191, &UA284800},
	{SG67212, &UA416155},
	{SG67314, &UA90282},
	{SG67315, &UA132017},
	{SG67331, &UA90105},
	{SG67372, &UA52056},
	{SG67373, &UA90280},
	{SG67444, &UA89893},
	{SG67508, &UA89892},
	{SG67533, &UA52788},
	{SG67536, &UA301594},
	{SG67543, &UA518329},
	{SG67554, &UA89891},
	{SG67555, &UA89915},
	{SG67584, &UA66982},
	{SG67585, &UA160422},
	{SG67586, &UA561944},
	{SG67595, &UA649683},
	{SG67600, &UA56148},
	{SG67648, &UA148785},
	{SG67649, &UA56777},
	{SG67650, &UA89917},
	{SG67653, &UA56770},
	{SG67662, &UA56792},
	{SG67663, &UA89916},
	{SG67664, &UA90554},
	{SG67665, &UA2270851},
	{SG67680, &UA300073},
	{SG67681, &UA135933},
	{SG67682, &UA89942},
	{SG67683, &UA159308},
	{SG67684, &UA496940},
	{SG67686, &UA294975},
	{SG67690, &UA166250},
	{SG67694, &UA67110},
	{SG67695, &UA158236},
	{SG67696, &UA50231},
	{SG67708, &UA89943},
	{SG67719, &UA90276},
	{SG67720, &UA90524},
	{SG67721, &UA140095},
	{SG67722, &UA56144},
	{SG67744, &UA158114},
	{SG67754, &UA299701},
	{SG67755, &UA67292},
	{SG67756, &UA90283},
	{SG67757, &UA90286},
	{SG67759, &UA162055},
	{SG67760, &UA56162},
	{SG67761, &UA89914},
	{SG67762, &UA90284},
	{SG67764, &UA2655972},
	{SG67773, &UA56156},
	{SG67778, &UA56354},
	{SG67786, &UA168098},
	{SG67787, &UA501037},
	{SG67788, &UA89890},
	{SG67789, &UA90009},
	{SG67795, &UA56134},
	{SG67802, &UA50459},
	{SG67803, &UA52782},
	{SG67804, &UA56160},
	{SG67805, &UA67126},
	{SG67825, &UA90036},
	{SG67827, &UA67009},
	{SG67828, &UA2292575},
	{SG67848, &UA55992},
	{SG67849, &UA56125},
	{SG67856, &UA2699614},
	{SG67861, &UA2408793},
	{SG67868, &UA1870085},
	{SG67869, &UA50218},
	{SG67874, &UA644821},
	{SG67883, &UA52057},
	{SG67886, &UA67268},
	{SG67887, &UA90279},
	{SG67888, &UA284885},
	{SG67901, &UA52058},
	{SG67902, &UA90275},
	{SG67904, &UA67125},
	{SG67905, &UA90006},
	{SG67935, &UA2977353},
	{SG67936, &UA56153},
	{SG67937, &UA90010},
	{SG67938, &UA90012},
	{SG67944, &UA2242226},
	{SG67972, &UA90627},
	{SG67989, &UA85522},
	{SG67997, &UA90038},
	{SG68009, &UA87038},
	{SG68018, &UA56142},
	{SG68019, &UA56019},
	{SG68038, &UA88489},
	{SG68083, &UA85499},
	{NULL}};
const Segmentdata DFirefoxDesktop[] = {
	{SG67557, &UA7085},
	{SG67558, &UA626415},
	{SG67561, &UA638069},
	{SG67562, &UA673868},
	{SG67563, &UA875434},
	{SG67564, &UA916167},
	{SG67565, &UA1870720},
	{SG67566, &UA2137031},
	{SG67567, &UA2260104},
	{SG67568, &UA2286450},
	{SG67569, &UA2670882},
	{SG67655, &UA444820},
	{SG67656, &UA455832},
	{SG67657, &UA507148},
	{SG67658, &UA599494},
	{SG67659, &UA634509},
	{SG67660, &UA2375903},
	{SG67697, &UA25416},
	{SG67698, &UA499753},
	{SG67699, &UA551246},
	{SG67700, &UA575177},
	{SG67701, &UA2319158},
	{SG67702, &UA2370121},
	{SG67792, &UA907437},
	{SG67800, &UA2040491},
	{SG67801, &UA2401256},
	{SG67806, &UA6654},
	{SG67807, &UA446952},
	{SG67808, &UA454483},
	{SG67809, &UA464023},
	{SG67810, &UA484253},
	{SG67811, &UA514044},
	{SG67812, &UA549107},
	{SG67813, &UA696093},
	{SG67814, &UA704319},
	{SG67815, &UA900457},
	{SG67816, &UA2263139},
	{SG67817, &UA2270273},
	{SG67818, &UA2511219},
	{SG67819, &UA2545228},
	{SG67820, &UA2758031},
	{SG67821, &UA2805936},
	{SG67822, &UA2806166},
	{SG67830, &UA28065},
	{SG67831, &UA478147},
	{SG67832, &UA551914},
	{SG67833, &UA589737},
	{SG67834, &UA590007},
	{SG67836, &UA603068},
	{SG67837, &UA656827},
	{SG67838, &UA713396},
	{SG67839, &UA743732},
	{SG67840, &UA780774},
	{SG67841, &UA791968},
	{SG67842, &UA809941},
	{SG67843, &UA823615},
	{SG67844, &UA890908},
	{SG67845, &UA895980},
	{SG67846, &UA2207376},
	{SG67847, &UA2331957},
	{SG67866, &UA2670037},
	{SG67867, &UA2672347},
	{SG67872, &UA2368276},
	{SG67873, &UA2405100},
	{SG67884, &UA2349502},
	{SG67885, &UA2360882},
	{SG67892, &UA2513360},
	{SG67893, &UA2537477},
	{SG67895, &UA777889},
	{SG67917, &UA2944979},
	{SG67918, &UA2965736},
	{SG67921, &UA2186931},
	{SG67939, &UA2432114},
	{SG67940, &UA2461061},
	{SG67984, &UA913451},
	{SG67985, &UA913457},
	{SG67986, &UA2390796},
	{SG67987, &UA2467029},
	{SG67988, &UA779555},
	{SG67990, &UA2673577},
	{SG67991, &UA625021},
	{SG67992, &UA445991},
	{SG67993, &UA1842603},
	{SG67994, &UA2368607},
	{SG67995, &UA2740409},
	{SG67996, &UA2850849},
	{SG67998, &UA464929},
	{SG67999, &UA479434},
	{SG68000, &UA599677},
	{SG68001, &UA739610},
	{SG68002, &UA780071},
	{SG68003, &UA864219},
	{SG68004, &UA875505},
	{SG68005, &UA896464},
	{SG68006, &UA899281},
	{SG68007, &UA2166594},
	{SG68008, &UA2206894},
	{SG68010, &UA503707},
	{SG68011, &UA590134},
	{SG68012, &UA676266},
	{SG68013, &UA680822},
	{SG68014, &UA690122},
	{SG68015, &UA2203270},
	{SG68016, &UA2803812},
	{SG68017, &UA641495},
	{SG68020, &UA575592},
	{SG68021, &UA778904},
	{SG68022, &UA825791},
	{SG68023, &UA2013048},
	{SG68024, &UA2136305},
	{SG68025, &UA2157021},
	{SG68026, &UA2514391},
	{SG68027, &UA444637},
	{SG68028, &UA449503},
	{SG68029, &UA459863},
	{SG68030, &UA491329},
	{SG68031, &UA502561},
	{SG68032, &UA646022},
	{SG68033, &UA721940},
	{SG68034, &UA776435},
	{SG68035, &UA844328},
	{SG68036, &UA877165},
	{SG68037, &UA2742146},
	{SG68039, &UA2368885},
	{SG68041, &UA803633},
	{SG68042, &UA852068},
	{SG68043, &UA2266357},
	{SG68044, &UA2587159},
	{SG68045, &UA609371},
	{SG68046, &UA627059},
	{SG68047, &UA894271},
	{SG68048, &UA1961684},
	{SG68049, &UA2144116},
	{SG68050, &UA2206158},
	{SG68051, &UA2259009},
	{SG68052, &UA2259142},
	{SG68053, &UA2386525},
	{SG68054, &UA2469082},
	{SG68055, &UA2527216},
	{SG68056, &UA2995703},
	{SG68057, &UA480274},
	{SG68058, &UA553594},
	{SG68059, &UA700724},
	{SG68060, &UA764288},
	{SG68061, &UA779754},
	{SG68062, &UA794576},
	{SG68063, &UA872256},
	{SG68064, &UA2221274},
	{SG68065, &UA2235373},
	{SG68066, &UA2259883},
	{SG68067, &UA2438506},
	{SG68068, &UA2753401},
	{SG68069, &UA3097770},
	{SG68070, &UA900588},
	{SG68071, &UA925274},
	{SG68072, &UA925365},
	{SG68073, &UA1996626},
	{SG68074, &UA2686374},
	{SG68075, &UA614144},
	{SG68076, &UA900943},
	{SG68077, &UA2043596},
	{SG68078, &UA2366165},
	{SG68079, &UA2383459},
	{SG68080, &UA3023302},
	{SG68081, &UA2351873},
	{SG68082, &UA2428193},
	{SG68084, &UA924887},
	{SG68085, &UA1995103},
	{SG68086, &UA1995360},
	{SG68087, &UA2391314},
	{SG68088, &UA2391317},
	{SG68089, &UA2538617},
	{SG68090, &UA757269},
	{SG68091, &UA811048},
	{SG68092, &UA855429},
	{SG68093, &UA862298},
	{SG68094, &UA872910},
	{SG68095, &UA2202888},
	{SG68096, &UA2471326},
	{SG68097, &UA2750104},
	{SG68098, &UA2783296},
	{SG68099, &UA589781},
	{SG68100, &UA591073},
	{SG68101, &UA2323120},
	{SG68102, &UA2461500},
	{SG68103, &UA2673033},
	{SG68104, &UA624777},
	{SG68105, &UA2008915},
	{SG68106, &UA2469408},
	{SG68107, &UA2530169},
	{SG68108, &UA2570290},
	{SG68110, &UA1995245},
	{SG68111, &UA2260102},
	{SG68112, &UA2361722},
	{SG68113, &UA2680762},
	{SG68114, &UA2751192},
	{SG68115, &UA2756668},
	{SG68116, &UA2756691},
	{SG68117, &UA2861105},
	{SG68118, &UA2993391},
	{SG68119, &UA2043019},
	{SG68120, &UA2043354},
	{SG68121, &UA2043622},
	{SG68122, &UA2200364},
	{SG68123, &UA2206072},
	{SG68124, &UA2350043},
	{SG68125, &UA2384247},
	{SG68126, &UA614775},
	{SG68127, &UA615133},
	{SG68128, &UA631236},
	{SG68129, &UA2163024},
	{SG68130, &UA2713298},
	{SG68131, &UA2784778},
	{SG68132, &UA2784853},
	{SG68133, &UA2999475},
	{SG68134, &UA3049232},
	{SG68135, &UA453557},
	{SG68136, &UA696574},
	{SG68137, &UA710768},
	{SG68138, &UA894284},
	{SG68139, &UA2139943},
	{SG68140, &UA2235240},
	{SG68141, &UA2260110},
	{SG68142, &UA2505830},
	{SG68143, &UA454380},
	{SG68144, &UA73026},
	{SG68145, &UA452659},
	{SG68146, &UA681100},
	{SG68147, &UA1902480},
	{SG68148, &UA1990643},
	{SG68149, &UA2569803},
	{SG68150, &UA2574149},
	{SG68151, &UA2748154},
	{SG68152, &UA649117},
	{SG68153, &UA2387789},
	{SG68154, &UA2258853},
	{SG68155, &UA1995071},
	{SG68156, &UA2260188},
	{SG68157, &UA2367594},
	{SG68158, &UA4389},
	{SG68159, &UA552132},
	{SG68160, &UA702743},
	{SG68161, &UA729544},
	{SG68162, &UA783644},
	{SG68163, &UA813095},
	{SG68164, &UA2409395},
	{SG68165, &UA2807393},
	{SG68166, &UA2740995},
	{SG68167, &UA2947716},
	{SG68168, &UA3112320},
	{SG68169, &UA2888648},
	{SG68170, &UA3093221},
	{SG68171, &UA3111795},
	{SG68172, &UA693386},
	{SG68173, &UA817402},
	{SG68174, &UA454542},
	{SG68175, &UA512019},
	{SG68176, &UA638291},
	{SG68177, &UA691182},
	{SG68178, &UA725625},
	{SG68179, &UA736531},
	{SG68180, &UA842757},
	{SG68181, &UA871562},
	{SG68182, &UA1893049},
	{SG68183, &UA2219458},
	{SG68184, &UA2473717},
	{SG68185, &UA2502473},
	{SG68186, &UA2956194},
	{SG68187, &UA2407659},
	{SG68188, &UA2675376},
	{SG68189, &UA2766473},
	{SG68190, &UA615668},
	{SG68191, &UA640231},
	{SG68192, &UA2356388},
	{SG68193, &UA2436517},
	{SG68195, &UA924839},
	{SG68196, &UA924840},
	{SG68197, &UA2390798},
	{SG68198, &UA2488290},
	{SG68199, &UA2849479},
	{SG68200, &UA2015616},
	{SG68201, &UA2042135},
	{SG68202, &UA2234789},
	{SG68203, &UA2356557},
	{SG68204, &UA3057443},
	{SG68205, &UA456586},
	{SG68206, &UA614772},
	{SG68207, &UA686124},
	{SG68208, &UA1949163},
	{SG68209, &UA2888356},
	{SG68210, &UA2897123},
	{SG68211, &UA854017},
	{SG68213, &UA900525},
	{SG68214, &UA902623},
	{SG68215, &UA905899},
	{SG68216, &UA1959794},
	{SG68217, &UA2314425},
	{SG68218, &UA2786705},
	{SG68219, &UA900885},
	{SG68220, &UA901506},
	{SG68221, &UA924860},
	{SG68222, &UA1996343},
	{SG68223, &UA2413030},
	{SG68224, &UA2503007},
	{SG68225, &UA2042162},
	{SG68226, &UA2043619},
	{SG68227, &UA2189649},
	{SG68228, &UA3057442},
	{SG68229, &UA2043356},
	{SG68230, &UA2202075},
	{SG68231, &UA588707},
	{SG68232, &UA902622},
	{SG68233, &UA903472},
	{SG68234, &UA903802},
	{SG68235, &UA603067},
	{SG68236, &UA783223},
	{SG68237, &UA894653},
	{SG68238, &UA1888395},
	{SG68239, &UA2233999},
	{SG68240, &UA2978569},
	{SG68255, &UA2143426},
	{SG68256, &UA445847},
	{SG68257, &UA2009050},
	{SG68258, &UA2641801},
	{SG68259, &UA2891565},
	{SG68260, &UA2390622},
	{SG68261, &UA640621},
	{SG68262, &UA726116},
	{SG68263, &UA1962880},
	{SG68264, &UA1967348},
	{SG68265, &UA2700456},
	{SG68266, &UA454709},
	{SG68267, &UA2675386},
	{SG68268, &UA2676388},
	{SG68269, &UA2784862},
	{SG68270, &UA3047271},
	{SG68271, &UA2353732},
	{SG68272, &UA2354171},
	{SG68273, &UA2471040},
	{SG68274, &UA2563788},
	{SG68275, &UA2784893},
	{SG68276, &UA2799518},
	{SG68277, &UA902608},
	{SG68278, &UA902626},
	{SG68279, &UA908167},
	{SG68280, &UA614796},
	{SG68281, &UA2742727},
	{SG68282, &UA2531127},
	{SG68283, &UA613269},
	{SG68284, &UA904774},
	{SG68285, &UA2354687},
	{SG68286, &UA2684683},
	{SG68287, &UA487972},
	{SG68288, &UA523223},
	{SG68289, &UA677206},
	{SG68290, &UA733211},
	{SG68291, &UA781454},
	{SG68292, &UA891798},
	{SG68293, &UA1953513},
	{SG68294, &UA2128905},
	{SG68295, &UA2223369},
	{SG68296, &UA2848541},
	{SG68297, &UA836389},
	{SG68298, &UA2043051},
	{SG68299, &UA2043085},
	{SG68300, &UA2451322},
	{SG68301, &UA2537633},
	{SG68302, &UA2595484},
	{SG68303, &UA2008913},
	{SG68304, &UA2186799},
	{SG68305, &UA2390623},
	{SG68306, &UA2532961},
	{SG68307, &UA1935221},
	{SG68308, &UA1989077},
	{SG68309, &UA2676384},
	{SG68310, &UA2598879},
	{SG68311, &UA3016191},
	{SG68312, &UA3077329},
	{SG68313, &UA722313},
	{SG68314, &UA2043686},
	{SG68315, &UA2461923},
	{SG68316, &UA840529},
	{SG68317, &UA2001937},
	{SG68318, &UA2258896},
	{SG68319, &UA3048820},
	{SG68320, &UA477939},
	{SG68321, &UA851995},
	{SG68322, &UA924483},
	{SG68323, &UA924890},
	{SG68324, &UA2042947},
	{SG68325, &UA2194223},
	{SG68326, &UA2470285},
	{SG68327, &UA2538236},
	{SG68328, &UA2539804},
	{SG68329, &UA2703950},
	{SG68330, &UA2755180},
	{SG68331, &UA2772569},
	{SG68332, &UA2386026},
	{SG68333, &UA910766},
	{SG68334, &UA2740941},
	{SG68335, &UA2740959},
	{SG68336, &UA2947722},
	{SG68337, &UA3057191},
	{SG68338, &UA3089415},
	{SG68339, &UA2390865},
	{SG68340, &UA662193},
	{SG68341, &UA2306921},
	{SG68342, &UA2043016},
	{SG68343, &UA2228157},
	{SG68344, &UA558192},
	{SG68345, &UA558838},
	{SG68346, &UA475195},
	{SG68347, &UA540018},
	{SG68348, &UA541243},
	{SG68349, &UA672267},
	{SG68350, &UA692430},
	{SG68351, &UA825302},
	{SG68352, &UA896097},
	{SG68353, &UA915485},
	{SG68354, &UA1893648},
	{SG68355, &UA2291685},
	{SG68356, &UA2387369},
	{SG68357, &UA808347},
	{SG68358, &UA2132867},
	{SG68359, &UA326845},
	{SG68360, &UA793369},
	{SG68361, &UA2390065},
	{SG68362, &UA2490671},
	{SG68363, &UA2367932},
	{SG68364, &UA2487933},
	{SG68365, &UA2756610},
	{SG68366, &UA2770013},
	{SG68367, &UA743784},
	{SG68368, &UA689548},
	{SG68369, &UA633886},
	{SG68370, &UA698125},
	{SG68371, &UA2703763},
	{SG68372, &UA2719050},
	{SG68373, &UA2470101},
	{SG68374, &UA2567473},
	{SG68375, &UA2784928},
	{SG68376, &UA2944520},
	{SG68377, &UA1999712},
	{SG68378, &UA3086229},
	{SG68379, &UA3015100},
	{SG68380, &UA2388411},
	{SG68381, &UA2471006},
	{SG68382, &UA924948},
	{SG68383, &UA3096409},
	{SG68384, &UA2538136},
	{SG68385, &UA2387453},
	{SG68386, &UA2739897},
	{SG68387, &UA3066943},
	{SG68388, &UA3095669},
	{SG68389, &UA3096350},
	{SG68390, &UA2739863},
	{SG68391, &UA2740031},
	{SG68392, &UA3096295},
	{SG68393, &UA3096617},
	{SG68394, &UA2534697},
	{SG68395, &UA2535165},
	{SG68396, &UA2946392},
	{SG68397, &UA2566433},
	{SG68398, &UA902621},
	{SG68399, &UA2388398},
	{SG68400, &UA2043012},
	{SG68401, &UA2693180},
	{SG68402, &UA2043017},
	{SG68403, &UA821082},
	{SG68404, &UA896020},
	{SG68405, &UA2388421},
	{SG68406, &UA2387479},
	{SG68407, &UA2486137},
	{SG68408, &UA1999169},
	{SG68409, &UA2487359},
	{SG68410, &UA902639},
	{SG68411, &UA905311},
	{SG68412, &UA622965},
	{SG68413, &UA627791},
	{SG68414, &UA1990766},
	{SG68415, &UA2148218},
	{SG68416, &UA2314464},
	{SG68417, &UA589371},
	{SG68418, &UA606110},
	{SG68419, &UA2167890},
	{SG68420, &UA2219433},
	{SG68421, &UA2466776},
	{SG68422, &UA2717894},
	{SG68423, &UA630042},
	{SG68424, &UA845269},
	{SG68425, &UA2331607},
	{SG68426, &UA2185219},
	{SG68427, &UA2260216},
	{SG68428, &UA637010},
	{SG68429, &UA651778},
	{SG68430, &UA878847},
	{SG68431, &UA894385},
	{SG68432, &UA2234785},
	{SG68433, &UA2424520},
	{SG68434, &UA2403878},
	{SG68435, &UA2738853},
	{SG68436, &UA2740050},
	{SG68437, &UA3096292},
	{SG68438, &UA3114402},
	{SG68439, &UA2004621},
	{SG68440, &UA913462},
	{SG68441, &UA1996320},
	{SG68442, &UA826482},
	{SG68443, &UA836189},
	{SG68444, &UA2260035},
	{SG68445, &UA2462064},
	{SG68446, &UA2779672},
	{SG68447, &UA2427013},
	{SG68448, &UA590294},
	{SG68449, &UA613227},
	{SG68450, &UA656346},
	{SG68451, &UA2383390},
	{SG68452, &UA23039},
	{SG68453, &UA442985},
	{SG68454, &UA650332},
	{SG68455, &UA680809},
	{SG68456, &UA3039307},
	{SG68457, &UA3039308},
	{SG68458, &UA3096348},
	{SG68459, &UA2236371},
	{SG68460, &UA2487975},
	{SG68461, &UA2774280},
	{SG68462, &UA901557},
	{SG68463, &UA1971182},
	{SG68464, &UA2222191},
	{SG68465, &UA511515},
	{SG68466, &UA552365},
	{SG68467, &UA32853},
	{SG68468, &UA454150},
	{SG68469, &UA504528},
	{SG68470, &UA636922},
	{SG68471, &UA656261},
	{SG68472, &UA660195},
	{SG68473, &UA876002},
	{SG68474, &UA915269},
	{SG68475, &UA1989401},
	{SG68476, &UA2413349},
	{SG68477, &UA2413832},
	{SG68478, &UA913432},
	{SG68479, &UA1996940},
	{SG68480, &UA772158},
	{SG68481, &UA902368},
	{SG68482, &UA902776},
	{SG68483, &UA577780},
	{SG68484, &UA582134},
	{SG68485, &UA731105},
	{SG68486, &UA795454},
	{SG68487, &UA600539},
	{SG68488, &UA810750},
	{SG68489, &UA623083},
	{SG68490, &UA860000},
	{SG68491, &UA601782},
	{SG68492, &UA2364787},
	{SG68493, &UA456572},
	{SG68494, &UA627125},
	{SG68495, &UA726405},
	{SG68496, &UA894648},
	{SG68497, &UA2234466},
	{SG68498, &UA2474053},
	{SG68499, &UA2806102},
	{SG68500, &UA603713},
	{SG68501, &UA736236},
	{SG68502, &UA622958},
	{SG68503, &UA600501},
	{SG68504, &UA770501},
	{SG68505, &UA902635},
	{SG68506, &UA575478},
	{SG68507, &UA584909},
	{SG68508, &UA2643081},
	{SG68509, &UA2719288},
	{SG68510, &UA913454},
	{SG68511, &UA2406372},
	{SG68512, &UA2408214},
	{SG68513, &UA924883},
	{SG68514, &UA2169425},
	{SG68515, &UA633041},
	{SG68516, &UA635342},
	{SG68517, &UA601674},
	{SG68518, &UA784741},
	{SG68519, &UA683350},
	{SG68520, &UA726708},
	{SG68521, &UA2350954},
	{SG68522, &UA2541430},
	{SG68523, &UA2406916},
	{SG68524, &UA2490676},
	{SG68525, &UA854887},
	{SG68526, &UA2041190},
	{SG68527, &UA632575},
	{SG68528, &UA894280},
	{SG68529, &UA1890991},
	{SG68530, &UA2258860},
	{SG68531, &UA2356349},
	{SG68532, &UA796098},
	{SG68533, &UA881555},
	{SG68534, &UA2263935},
	{SG68535, &UA904079},
	{SG68536, &UA2567714},
	{SG68537, &UA2567943},
	{SG68538, &UA456203},
	{SG68539, &UA630268},
	{SG68540, &UA1983045},
	{SG68541, &UA455257},
	{SG68542, &UA3085917},
	{SG68543, &UA589374},
	{SG68544, &UA2233711},
	{SG68545, &UA2394167},
	{SG68546, &UA2767575},
	{SG68547, &UA630734},
	{SG68548, &UA717434},
	{SG68549, &UA443111},
	{SG68550, &UA2639906},
	{SG68551, &UA456174},
	{SG68552, &UA600530},
	{SG68553, &UA622956},
	{SG68554, &UA1841829},
	{SG68555, &UA2266723},
	{SG68556, &UA30678},
	{SG68557, &UA625087},
	{SG68558, &UA632804},
	{SG68559, &UA1870416},
	{SG68560, &UA1885377},
	{SG68561, &UA2768298},
	{SG68562, &UA3116865},
	{SG68563, &UA2674928},
	{SG68564, &UA2860995},
	{SG68565, &UA637775},
	{SG68566, &UA627393},
	{SG68567, &UA2680908},
	{SG68568, &UA2737585},
	{SG68569, &UA2756987},
	{SG68570, &UA2757121},
	{SG68571, &UA662024},
	{SG68572, &UA2947739},
	{SG68573, &UA2947738},
	{SG68574, &UA2470791},
	{SG68575, &UA2489010},
	{SG68576, &UA546303},
	{SG68577, &UA757163},
	{SG68578, &UA869339},
	{SG68579, &UA890733},
	{SG68580, &UA2484240},
	{SG68581, &UA924469},
	{SG68582, &UA925473},
	{SG68583, &UA2168068},
	{SG68584, &UA2540134},
	{SG68585, &UA2565873},
	{SG68586, &UA2515433},
	{SG68587, &UA901128},
	{SG68588, &UA1995938},
	{SG68589, &UA1996050},
	{SG68590, &UA2604350},
	{SG68591, &UA2894863},
	{SG68592, &UA716499},
	{SG68593, &UA2715147},
	{SG68594, &UA2739408},
	{SG68595, &UA2784867},
	{SG68596, &UA32854},
	{SG68597, &UA582136},
	{SG68598, &UA582647},
	{SG68599, &UA590205},
	{SG68600, &UA660855},
	{SG68601, &UA845804},
	{SG68602, &UA1992492},
	{SG68603, &UA2569625},
	{SG68604, &UA582145},
	{SG68605, &UA645661},
	{SG68606, &UA654092},
	{SG68607, &UA923027},
	{SG68608, &UA1934175},
	{SG68609, &UA2259017},
	{SG68610, &UA2441032},
	{SG68611, &UA2561910},
	{SG68612, &UA2672633},
	{SG68613, &UA2675063},
	{SG68614, &UA600319},
	{SG68615, &UA2144684},
	{SG68616, &UA622546},
	{SG68617, &UA2487698},
	{SG68618, &UA2487699},
	{SG68619, &UA2606862},
	{SG68620, &UA2771506},
	{SG68621, &UA2946389},
	{SG68622, &UA2200320},
	{SG68623, &UA2232699},
	{SG68624, &UA2406893},
	{SG68625, &UA2533074},
	{SG68626, &UA2571479},
	{SG68627, &UA2718666},
	{SG68628, &UA2877696},
	{SG68629, &UA2946094},
	{SG68630, &UA3032462},
	{SG68631, &UA858467},
	{SG68632, &UA790322},
	{SG68633, &UA2201622},
	{SG68634, &UA2201627},
	{SG68635, &UA871015},
	{SG68636, &UA2202903},
	{SG68637, &UA2202905},
	{SG68638, &UA622544},
	{SG68639, &UA2885328},
	{SG68640, &UA2885470},
	{SG68641, &UA786591},
	{SG68642, &UA777177},
	{SG68643, &UA891489},
	{SG68644, &UA913406},
	{SG68645, &UA1997062},
	{SG68646, &UA2964203},
	{SG68647, &UA2388119},
	{SG68648, &UA2259177},
	{SG68649, &UA2714169},
	{SG68650, &UA2963923},
	{SG68651, &UA623228},
	{SG68652, &UA492615},
	{SG68653, &UA2674544},
	{SG68654, &UA2714278},
	{SG68655, &UA2531767},
	{SG68656, &UA924324},
	{SG68657, &UA753426},
	{SG68658, &UA2673637},
	{SG68659, &UA2680475},
	{SG68662, &UA626229},
	{SG68663, &UA743793},
	{SG68664, &UA796480},
	{SG68665, &UA625234},
	{SG68666, &UA659712},
	{SG68667, &UA1955873},
	{SG68668, &UA800072},
	{SG68669, &UA2288339},
	{SG68670, &UA460993},
	{SG68671, &UA450850},
	{SG68672, &UA598020},
	{SG68673, &UA855178},
	{SG68674, &UA872322},
	{SG68675, &UA2350161},
	{SG68676, &UA2359831},
	{SG68677, &UA2370768},
	{SG68678, &UA2898053},
	{SG68679, &UA2944299},
	{SG68680, &UA3093285},
	{SG68681, &UA2673870},
	{SG68682, &UA2674972},
	{SG68683, &UA2784973},
	{SG68684, &UA2870268},
	{SG68685, &UA2757830},
	{SG68686, &UA2510984},
	{SG68687, &UA2515446},
	{SG68688, &UA527229},
	{SG68689, &UA642005},
	{SG68690, &UA889673},
	{SG68691, &UA852194},
	{SG68692, &UA2948496},
	{SG68693, &UA455244},
	{SG68694, &UA2238809},
	{SG68695, &UA54028},
	{SG68696, &UA552805},
	{SG68697, &UA668278},
	{SG68698, &UA1904934},
	{SG68699, &UA2296664},
	{SG68700, &UA819730},
	{SG68701, &UA2502233},
	{SG68702, &UA2569626},
	{SG68703, &UA2583734},
	{SG68704, &UA2741625},
	{SG68705, &UA2753386},
	{SG68706, &UA2962399},
	{SG68707, &UA2221227},
	{SG68708, &UA670234},
	{SG68709, &UA2668180},
	{SG68710, &UA852564},
	{SG68711, &UA2468047},
	{SG68712, &UA638861},
	{SG68713, &UA678139},
	{SG68714, &UA703989},
	{SG68715, &UA2877019},
	{SG68716, &UA3018995},
	{SG68717, &UA764775},
	{SG68718, &UA2391494},
	{SG68719, &UA2407250},
	{SG68720, &UA2638060},
	{SG68721, &UA662931},
	{SG68722, &UA511654},
	{SG68723, &UA756386},
	{SG68724, &UA1845063},
	{SG68725, &UA2795263},
	{SG68726, &UA461796},
	{SG68727, &UA2129047},
	{SG68728, &UA2129049},
	{SG68729, &UA2442123},
	{SG68730, &UA2897914},
	{SG68731, &UA2971587},
	{SG68732, &UA1993831},
	{SG68733, &UA2202900},
	{SG68734, &UA2752252},
	{SG68735, &UA800181},
	{SG68736, &UA2042044},
	{SG68756, &UA2405632},
	{SG68782, &UA90608},
	{SG68784, &UA807255},
	{SG68785, &UA2349587},
	{SG68786, &UA576479},
	{SG68789, &UA2944783},
	{SG68790, &UA2886518},
	{SG68791, &UA3092796},
	{SG68792, &UA3111800},
	{SG68793, &UA640109},
	{SG68794, &UA683011},
	{SG68795, &UA706497},
	{SG68796, &UA600178},
	{SG68797, &UA624881},
	{SG68798, &UA604102},
	{SG68799, &UA1953934},
	{SG68800, &UA2675797},
	{SG68801, &UA2707886},
	{SG68802, &UA646851},
	{SG68803, &UA881488},
	{SG68804, &UA1872664},
	{SG68805, &UA2460333},
	{SG68806, &UA812756},
	{SG68807, &UA2408236},
	{SG68808, &UA2430013},
	{SG68810, &UA634546},
	{SG68811, &UA2347275},
	{SG68812, &UA702738},
	{SG68813, &UA2810393},
	{SG68814, &UA2567964},
	{SG68815, &UA2604176},
	{SG68816, &UA2680895},
	{SG68817, &UA2754146},
	{SG68818, &UA2899126},
	{SG68819, &UA2944602},
	{SG68820, &UA865966},
	{SG68821, &UA2200616},
	{SG68822, &UA2235162},
	{SG68823, &UA2406895},
	{SG68824, &UA2535325},
	{SG68825, &UA2567490},
	{SG68826, &UA2709461},
	{SG68827, &UA2886656},
	{SG68828, &UA2968685},
	{SG68829, &UA3035526},
	{SG68830, &UA2876602},
	{SG68831, &UA2944882},
	{SG68832, &UA2708922},
	{SG68833, &UA2041418},
	{SG68834, &UA902187},
	{SG68835, &UA2816399},
	{SG68836, &UA633595},
	{SG68837, &UA2567971},
	{SG68838, &UA2567962},
	{SG68839, &UA2678441},
	{SG68840, &UA690125},
	{SG68841, &UA774879},
	{SG68842, &UA906465},
	{SG68843, &UA2041419},
	{SG68844, &UA2230327},
	{SG68845, &UA2782792},
	{SG68846, &UA2288737},
	{SG68847, &UA2462368},
	{SG68848, &UA2489793},
	{SG68849, &UA479357},
	{SG68850, &UA2502914},
	{SG68851, &UA2506948},
	{SG68852, &UA2962219},
	{SG68853, &UA2675811},
	{SG68854, &UA2201316},
	{SG68855, &UA2201704},
	{SG68888, &UA2737194},
	{SG68889, &UA496746},
	{SG68890, &UA2125897},
	{SG68891, &UA460854},
	{SG68892, &UA2515962},
	{SG68893, &UA3111798},
	{SG68894, &UA580624},
	{SG68895, &UA629620},
	{SG68896, &UA1997007},
	{SG68897, &UA2667705},
	{SG68898, &UA2487695},
	{SG68899, &UA2531698},
	{SG68900, &UA2516127},
	{SG68901, &UA2738180},
	{SG68902, &UA2201628},
	{SG68903, &UA2358434},
	{SG68904, &UA2456128},
	{SG68905, &UA2469946},
	{SG68906, &UA668874},
	{SG68907, &UA2587546},
	{SG68908, &UA603306},
	{SG68909, &UA749077},
	{SG68910, &UA1941348},
	{SG68911, &UA476615},
	{SG68914, &UA2531003},
	{SG68915, &UA2567403},
	{SG68916, &UA838518},
	{SG68917, &UA2406809},
	{SG68918, &UA2569971},
	{SG68919, &UA2158392},
	{SG68920, &UA2675150},
	{SG68921, &UA2389838},
	{SG68922, &UA2673705},
	{SG68923, &UA2680230},
	{SG68924, &UA590270},
	{SG68925, &UA2384289},
	{SG68926, &UA2406376},
	{SG68927, &UA2406649},
	{SG68928, &UA924942},
	{SG68929, &UA1999886},
	{SG68930, &UA2234360},
	{SG68931, &UA645424},
	{SG68932, &UA2737224},
	{SG68933, &UA3031565},
	{SG68934, &UA3031564},
	{SG68935, &UA635710},
	{SG68936, &UA645747},
	{SG68937, &UA2589043},
	{SG68938, &UA2040899},
	{SG68939, &UA2406448},
	{SG68940, &UA2234220},
	{SG68941, &UA574251},
	{SG68942, &UA648177},
	{SG68943, &UA2152966},
	{SG68944, &UA2233977},
	{SG68945, &UA536222},
	{SG68946, &UA2239315},
	{SG68947, &UA901504},
	{SG68948, &UA2888608},
	{SG68949, &UA563169},
	{SG68950, &UA741739},
	{SG68951, &UA2042094},
	{SG68952, &UA2850670},
	{SG68953, &UA2487700},
	{SG68954, &UA2488271},
	{SG68955, &UA622921},
	{SG68956, &UA655461},
	{SG68957, &UA655632},
	{SG68958, &UA2258913},
	{SG68959, &UA2405266},
	{SG68960, &UA2199567},
	{SG68961, &UA2435185},
	{SG68962, &UA545389},
	{SG68963, &UA739994},
	{SG68964, &UA830534},
	{SG68965, &UA904417},
	{SG68966, &UA904860},
	{SG68967, &UA2406402},
	{SG68968, &UA2406631},
	{SG68969, &UA624173},
	{SG68970, &UA2680602},
	{SG68971, &UA2718624},
	{SG68972, &UA2891922},
	{SG68973, &UA624169},
	{SG68974, &UA1938929},
	{SG68975, &UA2509859},
	{SG68976, &UA924467},
	{SG68977, &UA683473},
	{SG68978, &UA717570},
	{SG68979, &UA780849},
	{SG68980, &UA2203355},
	{SG68981, &UA2887915},
	{SG68982, &UA2897727},
	{SG68983, &UA2976849},
	{SG68984, &UA3023253},
	{SG68985, &UA2423566},
	{SG68987, &UA2389885},
	{SG68988, &UA28068},
	{SG68990, &UA2406463},
	{SG68991, &UA798907},
	{SG68992, &UA2197031},
	{SG68993, &UA3031573},
	{SG68994, &UA2461946},
	{SG68995, &UA2893808},
	{SG68996, &UA2365234},
	{SG68997, &UA2429709},
	{SG68998, &UA526347},
	{SG68999, &UA599482},
	{SG69000, &UA1868789},
	{SG69001, &UA783344},
	{SG69002, &UA2507824},
	{SG69003, &UA823634},
	{SG69004, &UA2801180},
	{SG69005, &UA2009033},
	{SG69006, &UA2017656},
	{SG69007, &UA3028293},
	{SG69008, &UA3108345},
	{SG69010, &UA925480},
	{SG69011, &UA907435},
	{SG69012, &UA2567965},
	{SG69013, &UA2584859},
	{SG69014, &UA2566590},
	{SG69015, &UA627493},
	{SG69016, &UA1884374},
	{SG69017, &UA3016433},
	{SG69018, &UA909465},
	{SG69019, &UA738539},
	{SG69020, &UA756933},
	{SG69021, &UA2209315},
	{SG69022, &UA924847},
	{SG69023, &UA676854},
	{SG69024, &UA905949},
	{SG69025, &UA2603165},
	{SG69026, &UA2351770},
	{SG69027, &UA2230868},
	{SG69028, &UA2258899},
	{SG69029, &UA2429519},
	{SG69030, &UA2601538},
	{SG69031, &UA2738241},
	{SG69032, &UA2854992},
	{SG69033, &UA2899030},
	{SG69034, &UA2967920},
	{SG69035, &UA3062563},
	{SG69036, &UA184771},
	{SG69037, &UA666662},
	{SG69038, &UA635573},
	{SG69039, &UA2712207},
	{SG69040, &UA2569630},
	{SG69041, &UA908443},
	{SG69042, &UA2712104},
	{SG69043, &UA468058},
	{SG69044, &UA719775},
	{SG69045, &UA924782},
	{SG69046, &UA1995366},
	{SG69047, &UA2769750},
	{SG69048, &UA461983},
	{SG69049, &UA862160},
	{SG69050, &UA614148},
	{SG69051, &UA442278},
	{SG69052, &UA531076},
	{SG69053, &UA558278},
	{SG69054, &UA889981},
	{SG69055, &UA2893805},
	{SG69056, &UA614853},
	{SG69057, &UA614868},
	{SG69058, &UA2895004},
	{SG69059, &UA2515844},
	{SG69060, &UA2499839},
	{SG69061, &UA538566},
	{SG69062, &UA813967},
	{SG69063, &UA903742},
	{SG69065, &UA2675439},
	{SG69066, &UA691277},
	{SG69067, &UA867247},
	{SG69068, &UA624891},
	{SG69069, &UA2487701},
	{SG69070, &UA2508298},
	{SG69071, &UA2800146},
	{SG69072, &UA630179},
	{SG69073, &UA2194217},
	{SG69074, &UA2201625},
	{SG69075, &UA2235410},
	{SG69076, &UA903684},
	{SG69077, &UA907377},
	{SG69078, &UA704073},
	{SG69079, &UA1870312},
	{SG69080, &UA588596},
	{SG69081, &UA634958},
	{SG69082, &UA463382},
	{SG69083, &UA453500},
	{SG69084, &UA453502},
	{SG69085, &UA2674880},
	{SG69086, &UA443306},
	{SG69087, &UA463233},
	{SG69088, &UA2815728},
	{SG69089, &UA2042095},
	{SG69090, &UA773312},
	{SG69091, &UA2131030},
	{SG69092, &UA468800},
	{SG69093, &UA2462369},
	{SG69094, &UA2489238},
	{SG69095, &UA2386760},
	{SG69096, &UA2426749},
	{SG69097, &UA2562198},
	{SG69098, &UA2603522},
	{SG69099, &UA2738252},
	{SG69100, &UA2886357},
	{SG69101, &UA2970834},
	{SG69102, &UA3062273},
	{SG69103, &UA647462},
	{SG69104, &UA470735},
	{SG69105, &UA915770},
	{SG69106, &UA2736362},
	{SG69107, &UA601381},
	{SG69108, &UA2367095},
	{SG69109, &UA627848},
	{SG69110, &UA613311},
	{SG69111, &UA833660},
	{SG69112, &UA2516389},
	{SG69113, &UA2678442},
	{SG69114, &UA2569825},
	{SG69116, &UA2042736},
	{SG69117, &UA477698},
	{SG69118, &UA2672809},
	{SG69119, &UA2201736},
	{SG69120, &UA2235193},
	{SG69121, &UA481270},
	{SG69122, &UA518264},
	{SG69123, &UA2674001},
	{SG69124, &UA2680593},
	{SG69125, &UA2258206},
	{SG69126, &UA2679925},
	{SG69127, &UA2962518},
	{SG69128, &UA924479},
	{SG69129, &UA1995092},
	{SG69130, &UA2167019},
	{SG69131, &UA826202},
	{SG69132, &UA775908},
	{SG69133, &UA3012171},
	{SG69134, &UA740022},
	{SG69135, &UA2407112},
	{SG69136, &UA1854663},
	{SG69137, &UA2672354},
	{SG69138, &UA901970},
	{SG69139, &UA827967},
	{SG69140, &UA834848},
	{SG69141, &UA907904},
	{SG69142, &UA2364515},
	{SG69143, &UA2567173},
	{SG69144, &UA2236472},
	{SG69145, &UA2259287},
	{SG69146, &UA2879232},
	{SG69147, &UA647842},
	{SG69148, &UA756614},
	{SG69149, &UA2675467},
	{SG69150, &UA2228102},
	{SG69151, &UA384612},
	{SG69152, &UA2722135},
	{SG69153, &UA633530},
	{SG69154, &UA2228885},
	{SG69155, &UA1995110},
	{SG69156, &UA2797404},
	{SG69157, &UA470782},
	{SG69158, &UA783519},
	{SG69159, &UA2144294},
	{SG69160, &UA2146009},
	{SG69161, &UA2683693},
	{SG69162, &UA908183},
	{SG69164, &UA2569820},
	{SG69165, &UA2406885},
	{SG69166, &UA2389836},
	{SG69167, &UA2567404},
	{SG69168, &UA2741239},
	{SG69169, &UA2238597},
	{SG69170, &UA2532042},
	{SG69192, &UA766023},
	{SG69193, &UA904843},
	{SG69194, &UA909205},
	{SG69195, &UA2533387},
	{SG69196, &UA3095429},
	{SG69197, &UA888068},
	{SG69198, &UA896633},
	{SG69199, &UA2469840},
	{SG69200, &UA2489797},
	{SG69201, &UA2468038},
	{SG69202, &UA833995},
	{SG69203, &UA881538},
	{SG69204, &UA2463433},
	{SG69205, &UA2042735},
	{SG69206, &UA3096092},
	{SG69207, &UA510928},
	{SG69208, &UA1860598},
	{SG69209, &UA2258091},
	{SG69210, &UA2461229},
	{SG69211, &UA3044616},
	{SG69212, &UA2470515},
	{SG69213, &UA700757},
	{SG69214, &UA1872478},
	{SG69215, &UA2166047},
	{SG69216, &UA2329604},
	{SG69217, &UA2774537},
	{SG69218, &UA2439113},
	{SG69219, &UA2642056},
	{SG69220, &UA2239316},
	{SG69221, &UA2879324},
	{SG69222, &UA2510401},
	{SG69223, &UA758889},
	{SG69225, &UA2463057},
	{SG69226, &UA2295284},
	{SG69227, &UA2739198},
	{SG69228, &UA2667839},
	{SG69229, &UA822368},
	{SG69230, &UA2389812},
	{SG69245, &UA2017655},
	{SG69246, &UA2188973},
	{SG69247, &UA638537},
	{SG69248, &UA2893763},
	{SG69249, &UA687192},
	{SG69250, &UA3091470},
	{SG69251, &UA566609},
	{SG69252, &UA3015362},
	{SG69253, &UA2510402},
	{SG69254, &UA2571511},
	{SG69255, &UA2307235},
	{SG69256, &UA2195944},
	{SG69257, &UA712999},
	{SG69258, &UA2041278},
	{SG69259, &UA2408056},
	{SG69260, &UA2896401},
	{SG69262, &UA705221},
	{SG69263, &UA2394834},
	{SG69264, &UA699491},
	{SG69265, &UA491803},
	{SG69266, &UA587051},
	{SG69267, &UA2665748},
	{SG69268, &UA2675562},
	{SG69269, &UA2536550},
	{SG69270, &UA758435},
	{SG69271, &UA2259226},
	{SG69272, &UA2371613},
	{SG69273, &UA2961823},
	{SG69274, &UA446019},
	{SG69275, &UA463305},
	{SG69276, &UA859595},
	{SG69277, &UA2392685},
	{SG69278, &UA633600},
	{SG69279, &UA1904682},
	{SG69280, &UA695280},
	{SG69281, &UA861740},
	{SG69301, &UA2537213},
	{SG69303, &UA687063},
	{SG69304, &UA683954},
	{SG69305, &UA2389843},
	{SG69306, &UA546849},
	{SG69307, &UA765007},
	{SG69308, &UA610105},
	{SG69309, &UA727238},
	{SG69310, &UA2471322},
	{SG69311, &UA2490856},
	{SG69312, &UA2678440},
	{SG69313, &UA591957},
	{SG69314, &UA656163},
	{SG69315, &UA673875},
	{SG69316, &UA854720},
	{SG69317, &UA2270942},
	{SG69318, &UA2289948},
	{SG69319, &UA2472148},
	{SG69320, &UA2516983},
	{SG69321, &UA2846010},
	{SG69322, &UA626195},
	{SG69323, &UA2949323},
	{SG69324, &UA514323},
	{SG69325, &UA2567664},
	{SG69326, &UA2258191},
	{SG69327, &UA858677},
	{SG69328, &UA2507823},
	{SG69329, &UA1995099},
	{SG69330, &UA614893},
	{SG69331, &UA688143},
	{SG69333, &UA449082},
	{SG69334, &UA1951985},
	{SG69335, &UA852416},
	{SG69336, &UA869101},
	{SG69337, &UA2211626},
	{SG69338, &UA556521},
	{SG69339, &UA785191},
	{SG69340, &UA687934},
	{SG69341, &UA630051},
	{SG69342, &UA2483539},
	{SG69343, &UA466007},
	{SG69344, &UA647343},
	{SG69345, &UA587287},
	{SG69346, &UA2846008},
	{SG69347, &UA2144953},
	{SG69348, &UA924766},
	{SG69349, &UA2201713},
	{SG69350, &UA2202230},
	{SG69398, &UA1995271},
	{SG69399, &UA2003831},
	{SG69400, &UA1886793},
	{SG69401, &UA783312},
	{SG69402, &UA850932},
	{SG69403, &UA799095},
	{SG69404, &UA2259277},
	{SG69405, &UA792930},
	{SG69406, &UA647619},
	{SG69407, &UA615302},
	{SG69408, &UA2258147},
	{SG69409, &UA478851},
	{SG69410, &UA773540},
	{SG69411, &UA2726658},
	{SG69412, &UA896417},
	{SG69413, &UA907246},
	{SG69414, &UA2040603},
	{SG69415, &UA645257},
	{SG69416, &UA656350},
	{SG69417, &UA884015},
	{SG69418, &UA924776},
	{SG69419, &UA1841584},
	{SG69420, &UA2894416},
	{SG69421, &UA1996948},
	{SG69422, &UA645692},
	{SG69423, &UA697567},
	{SG69424, &UA902021},
	{SG69425, &UA1887308},
	{SG69426, &UA2593541},
	{SG69427, &UA493693},
	{SG69428, &UA2390281},
	{SG69429, &UA908463},
	{SG69430, &UA625625},
	{SG69431, &UA824918},
	{SG69432, &UA1860023},
	{SG69433, &UA2127188},
	{SG69434, &UA797800},
	{SG69435, &UA2530071},
	{SG69436, &UA2560820},
	{SG69437, &UA2896470},
	{SG69438, &UA760782},
	{SG69439, &UA2040382},
	{SG69440, &UA2239390},
	{SG69441, &UA709090},
	{SG69442, &UA807445},
	{SG69443, &UA2720561},
	{SG69444, &UA613592},
	{SG69445, &UA2390634},
	{SG69446, &UA1997072},
	{SG69447, &UA903768},
	{SG69448, &UA867097},
	{SG69451, &UA899776},
	{SG69465, &UA699852},
	{SG69466, &UA906090},
	{SG69467, &UA830700},
	{SG69468, &UA2203509},
	{SG69469, &UA2259235},
	{SG69470, &UA717518},
	{SG69471, &UA2145577},
	{SG69472, &UA663768},
	{SG69473, &UA877203},
	{SG69474, &UA2210575},
	{SG69475, &UA558118},
	{SG69476, &UA888642},
	{SG69477, &UA2751479},
	{SG69478, &UA687204},
	{SG69479, &UA797626},
	{SG69480, &UA637470},
	{SG69481, &UA2573075},
	{SG69482, &UA2238870},
	{SG69483, &UA655462},
	{SG69484, &UA444791},
	{SG69485, &UA632392},
	{SG69486, &UA793089},
	{SG69487, &UA591822},
	{SG69488, &UA2312743},
	{SG69489, &UA734235},
	{SG69490, &UA869807},
	{SG69491, &UA899550},
	{SG69492, &UA2040384},
	{SG69495, &UA769953},
	{SG69496, &UA1863346},
	{SG69497, &UA812173},
	{SG69498, &UA906510},
	{SG69499, &UA909540},
	{SG69500, &UA682679},
	{SG69501, &UA499098},
	{SG69502, &UA880738},
	{SG69503, &UA2007715},
	{SG69504, &UA517318},
	{SG69505, &UA659062},
	{SG69506, &UA738976},
	{SG69507, &UA585617},
	{SG69508, &UA659159},
	{SG69509, &UA2167533},
	{SG69510, &UA789303},
	{SG69526, &UA448002},
	{SG69527, &UA454482},
	{SG69528, &UA785020},
	{SG69529, &UA467172},
	{SG69548, &UA2485702},
	{SG69549, &UA2988244},
	{SG69550, &UA457083},
	{SG69551, &UA658579},
	{SG69552, &UA2409250},
	{SG69553, &UA2169939},
	{SG69554, &UA514752},
	{SG69555, &UA2211688},
	{SG69556, &UA611091},
	{SG69557, &UA852428},
	{SG69558, &UA2162223},
	{SG69559, &UA1904040},
	{SG69560, &UA737456},
	{SG69561, &UA532652},
	{SG69562, &UA2392367},
	{SG69563, &UA2517490},
	{SG69564, &UA530436},
	{SG69565, &UA666787},
	{SG69566, &UA635919},
	{SG69567, &UA856384},
	{SG69568, &UA2712113},
	{SG69570, &UA2017665},
	{SG69571, &UA2180555},
	{SG69572, &UA753424},
	{SG69573, &UA2167290},
	{SG69574, &UA549383},
	{SG69575, &UA717986},
	{SG69576, &UA2202734},
	{SG69577, &UA2203317},
	{SG69588, &UA770425},
	{SG69589, &UA2202990},
	{SG69625, &UA587624},
	{SG69626, &UA763031},
	{SG69627, &UA857085},
	{SG69628, &UA2007362},
	{SG69629, &UA2944040},
	{SG69630, &UA3026059},
	{SG69631, &UA633764},
	{SG69632, &UA861743},
	{SG69633, &UA2781141},
	{SG69634, &UA550625},
	{SG69635, &UA589998},
	{SG69636, &UA2259853},
	{SG69666, &UA452749},
	{SG69667, &UA2230700},
	{SG69668, &UA1996177},
	{SG69669, &UA1996363},
	{SG69670, &UA2714571},
	{SG69671, &UA1984935},
	{SG69672, &UA2418525},
	{SG69673, &UA742522},
	{SG69674, &UA822180},
	{SG69675, &UA875174},
	{SG69677, &UA924803},
	{SG69678, &UA2259467},
	{SG69679, &UA478027},
	{SG69680, &UA565326},
	{SG69682, &UA613280},
	{SG69683, &UA1849588},
	{SG69714, &UA2144966},
	{SG69715, &UA640087},
	{SG69716, &UA822680},
	{SG69717, &UA2711815},
	{SG69718, &UA2505829},
	{SG69719, &UA2708638},
	{SG69720, &UA694777},
	{SG69721, &UA835775},
	{SG69722, &UA2505762},
	{SG69734, &UA497166},
	{SG69735, &UA716197},
	{SG69736, &UA512145},
	{SG69737, &UA647275},
	{SG69747, &UA899167},
	{SG69748, &UA711121},
	{SG69749, &UA2235603},
	{SG69750, &UA2201116},
	{SG69751, &UA687474},
	{SG69752, &UA867450},
	{SG69771, &UA501249},
	{SG69772, &UA589369},
	{SG69773, &UA724234},
	{SG69774, &UA1943153},
	{SG69775, &UA2409373},
	{SG69792, &UA2270435},
	{SG69793, &UA2642243},
	{SG69794, &UA2707392},
	{SG69814, &UA2009038},
	{SG69815, &UA3014445},
	{SG69816, &UA2496284},
	{SG69817, &UA632262},
	{SG69818, &UA901788},
	{SG69819, &UA924951},
	{SG69822, &UA1856047},
	{SG69823, &UA2360554},
	{SG69824, &UA2963488},
	{SG69825, &UA2200539},
	{SG69826, &UA2211904},
	{SG69827, &UA2260239},
	{SG69828, &UA2042760},
	{SG69829, &UA2186810},
	{SG69830, &UA453125},
	{SG69831, &UA2852971},
	{SG69837, &UA515285},
	{SG69838, &UA729632},
	{SG69839, &UA2334640},
	{SG69840, &UA2886516},
	{SG69841, &UA458366},
	{SG69854, &UA515001},
	{SG69855, &UA696057},
	{SG69856, &UA829562},
	{SG69857, &UA2752084},
	{SG69858, &UA2408507},
	{SG69859, &UA752490},
	{SG69860, &UA898111},
	{SG69861, &UA531985},
	{SG69862, &UA818953},
	{SG69863, &UA571298},
	{SG69864, &UA875666},
	{SG69868, &UA2707310},
	{SG69869, &UA462572},
	{SG69870, &UA601459},
	{SG69872, &UA613764},
	{SG69873, &UA903877},
	{SG69881, &UA645663},
	{SG69882, &UA657746},
	{SG69883, &UA882538},
	{SG69884, &UA2587395},
	{SG69885, &UA635415},
	{SG69886, &UA696676},
	{SG69887, &UA2467394},
	{SG69888, &UA1938028},
	{SG69889, &UA2309122},
	{SG69890, &UA1992543},
	{SG69893, &UA590039},
	{SG69894, &UA2682477},
	{SG69895, &UA724394},
	{SG69896, &UA924814},
	{SG69897, &UA908522},
	{SG69898, &UA1959874},
	{SG69899, &UA2146917},
	{SG69900, &UA1842802},
	{SG69901, &UA2442541},
	{SG69902, &UA1886672},
	{SG69903, &UA2473292},
	{SG69904, &UA574491},
	{SG69905, &UA2215030},
	{SG69907, &UA846714},
	{SG69908, &UA478898},
	{SG69909, &UA645827},
	{SG69910, &UA658781},
	{SG69911, &UA870177},
	{SG69912, &UA2308371},
	{SG69913, &UA2372678},
	{SG69923, &UA2878375},
	{SG69937, &UA460281},
	{SG69941, &UA589939},
	{SG69942, &UA723653},
	{SG69944, &UA919129},
	{SG69945, &UA2545105},
	{SG69946, &UA475007},
	{SG69947, &UA1885244},
	{SG69955, &UA800593},
	{SG69956, &UA924288},
	{SG69957, &UA484037},
	{SG69958, &UA845795},
	{SG69959, &UA736620},
	{SG69960, &UA1865592},
	{SG69971, &UA771980},
	{SG69972, &UA853608},
	{SG69973, &UA855671},
	{SG69974, &UA2316062},
	{SG69976, &UA600310},
	{SG69977, &UA647450},
	{SG69978, &UA1970673},
	{SG69982, &UA551737},
	{SG69983, &UA2977617},
	{SG69984, &UA648730},
	{SG69994, &UA501908},
	{SG69995, &UA845347},
	{SG69996, &UA2332999},
	{SG69997, &UA2714664},
	{SG70004, &UA1890184},
	{SG70014, &UA545868},
	{SG70015, &UA589197},
	{SG70016, &UA703630},
	{SG70017, &UA760051},
	{SG70018, &UA2207320},
	{SG70019, &UA2209245},
	{SG70020, &UA2682200},
	{SG70021, &UA2977461},
	{SG70022, &UA455685},
	{SG70023, &UA671691},
	{SG70024, &UA827702},
	{SG70025, &UA515176},
	{SG70026, &UA1897536},
	{SG70027, &UA902531},
	{SG70028, &UA2333873},
	{SG70033, &UA772451},
	{SG70034, &UA854305},
	{SG70036, &UA639018},
	{SG70037, &UA1995358},
	{SG70059, &UA557645},
	{SG70060, &UA915886},
	{SG70061, &UA1898459},
	{SG70062, &UA2433661},
	{SG70063, &UA452328},
	{SG70064, &UA677973},
	{SG70065, &UA537567},
	{SG70068, &UA2258891},
	{SG70072, &UA588762},
	{SG70075, &UA1948664},
	{SG70080, &UA830187},
	{SG70087, &UA1944291},
	{SG70090, &UA1945760},
	{SG70091, &UA2315962},
	{SG70093, &UA554607},
	{SG70094, &UA660837},
	{SG70100, &UA2237706},
	{SG70103, &UA2981423},
	{SG70105, &UA542900},
	{SG70106, &UA1899054},
	{SG70107, &UA2203001},
	{SG70108, &UA2517431},
	{SG70109, &UA2686292},
	{SG70110, &UA2322417},
	{SG70111, &UA883047},
	{SG70112, &UA2040762},
	{SG70124, &UA2978772},
	{SG70125, &UA3015550},
	{SG70137, &UA2352764},
	{SG70138, &UA816363},
	{SG70139, &UA1990309},
	{SG70140, &UA2641442},
	{SG70141, &UA2192581},
	{SG70142, &UA2807963},
	{SG70143, &UA2285654},
	{SG70144, &UA2640212},
	{SG70151, &UA2983665},
	{SG70152, &UA808991},
	{SG70153, &UA866543},
	{SG70156, &UA2956151},
	{SG70157, &UA3125512},
	{SG70163, &UA2982574},
	{SG70180, &UA2889490},
	{SG70181, &UA575086},
	{SG70182, &UA623795},
	{SG70196, &UA689283},
	{SG70197, &UA2312399},
	{SG70198, &UA1899478},
	{SG70199, &UA2797304},
	{SG70200, &UA2133314},
	{SG70201, &UA2279220},
	{SG70202, &UA2683181},
	{SG70203, &UA2475243},
	{SG70205, &UA1996872},
	{SG70216, &UA2819818},
	{SG70228, &UA785872},
	{SG70229, &UA2529270},
	{SG70230, &UA2643619},
	{SG70245, &UA3050494},
	{SG70249, &UA455678},
	{SG70250, &UA1982994},
	{SG70259, &UA2786672},
	{SG70267, &UA483310},
	{SG70275, &UA515635},
	{SG70276, &UA811417},
	{SG70281, &UA3084942},
	{SG70283, &UA2282841},
	{SG70284, &UA556284},
	{SG70285, &UA665480},
	{SG70286, &UA813647},
	{SG70287, &UA846518},
	{SG70298, &UA2995019},
	{SG70300, &UA455356},
	{SG70301, &UA572324},
	{SG70302, &UA2373943},
	{SG70306, &UA2767451},
	{SG70309, &UA2982075},
	{SG70312, &UA470349},
	{SG70313, &UA702406},
	{SG70323, &UA2878926},
	{SG70324, &UA2948867},
	{SG70325, &UA3046816},
	{SG70326, &UA523190},
	{SG70327, &UA583751},
	{SG70328, &UA2213116},
	{SG70329, &UA2720571},
	{SG70334, &UA591312},
	{SG70335, &UA776766},
	{SG70336, &UA794899},
	{SG70337, &UA1900286},
	{SG70338, &UA1900385},
	{SG70339, &UA2260054},
	{SG70340, &UA541038},
	{SG70341, &UA574078},
	{SG70342, &UA681886},
	{SG70343, &UA784845},
	{SG70344, &UA834404},
	{SG70345, &UA1841424},
	{SG70346, &UA1948872},
	{SG70347, &UA2329661},
	{SG70349, &UA2697442},
	{SG70350, &UA2705853},
	{SG70358, &UA2843155},
	{SG70363, &UA2977299},
	{SG70364, &UA3041952},
	{SG70365, &UA2327902},
	{SG70366, &UA2638422},
	{SG70369, &UA2999231},
	{SG70375, &UA719649},
	{SG70376, &UA1838472},
	{SG70377, &UA2192134},
	{SG70384, &UA669521},
	{SG70385, &UA849167},
	{SG70386, &UA2266390},
	{SG70387, &UA2392516},
	{SG70388, &UA2494379},
	{SG70389, &UA2494984},
	{SG70406, &UA2956809},
	{SG70416, &UA685800},
	{SG70417, &UA1869472},
	{SG70418, &UA1899892},
	{SG70420, &UA2373772},
	{SG70424, &UA2986297},
	{SG70450, &UA789343},
	{SG70451, &UA1842804},
	{SG70452, &UA1940144},
	{SG70453, &UA2293308},
	{SG70454, &UA2593981},
	{SG70460, &UA3119141},
	{SG70466, &UA501555},
	{SG70467, &UA518633},
	{SG70468, &UA706100},
	{SG70469, &UA1904058},
	{SG70470, &UA2147186},
	{SG70471, &UA2209141},
	{SG70474, &UA2438191},
	{SG70475, &UA2473145},
	{SG70476, &UA2574213},
	{SG70477, &UA2639909},
	{SG70483, &UA2983070},
	{SG70487, &UA3117439},
	{SG70488, &UA3118378},
	{SG70493, &UA499407},
	{SG70499, &UA2265955},
	{SG70500, &UA2285705},
	{SG70506, &UA862100},
	{SG70507, &UA905800},
	{SG70508, &UA2267247},
	{SG70509, &UA2282217},
	{SG70510, &UA2298259},
	{SG70511, &UA2300400},
	{SG70512, &UA2324057},
	{SG70513, &UA2802726},
	{SG70514, &UA446903},
	{SG70515, &UA473383},
	{SG70525, &UA483595},
	{SG70528, &UA466749},
	{SG70530, &UA1986879},
	{SG70536, &UA477577},
	{SG70537, &UA498619},
	{SG70538, &UA3039894},
	{SG70585, &UA3097334},
	{SG70586, &UA3097336},
	{SG70592, &UA2616863},
	{SG70602, &UA3097330},
	{SG70603, &UA3097331},
	{NULL}};
const Segmentdata DFirefoxFennec[] = {
	{SG68660, &UA613244},
	{SG68661, &UA900545},
	{SG68787, &UA613243},
	{SG68788, &UA900544},
	{SG69115, &UA901736},
	{SG69163, &UA2043655},
	{SG69224, &UA2362250},
	{SG69261, &UA2018724},
	{SG69332, &UA614808},
	{SG69494, &UA1839374},
	{SG69569, &UA601654},
	{SG69676, &UA402261},
	{SG69684, &UA614791},
	{SG69853, &UA614840},
	{SG69865, &UA575483},
	{SG69866, &UA597159},
	{SG69867, &UA2640183},
	{SG69871, &UA456574},
	{SG69891, &UA836021},
	{SG69892, &UA3069486},
	{SG69939, &UA903784},
	{SG69940, &UA907978},
	{SG69943, &UA2222727},
	{SG69975, &UA454805},
	{SG70005, &UA847630},
	{SG70032, &UA2000186},
	{SG70092, &UA3013902},
	{SG70175, &UA654984},
	{SG70270, &UA2472426},
	{SG70411, &UA816303},
	{SG70455, &UA2719756},
	{SG70505, &UA2018767},
	{SG70529, &UA2589652},
	{SG70551, &UA535867},
	{SG70620, &UA2021682},
	{SG70651, &UA2020440},
	{SG70694, &UA2242179},
	{NULL}};
const Device *DFly[] = {
	&UA90621,
	&UA2941951,
	&UA2346037,
	&UA2421440,
	&UA430112,
	&UA2474417,
	&UA2224,
	&UA18725,
	&UA70007,
	&UA54968,
	&UA23495,
	&UA47649,
	&UA60601,
	&UA49629,
	&UA89005,
	&UA11176,
	&UA43631,
	&UA21499,
	&UA15965,
	&UA50016,
	&UA53467,
	&UA54029,
	&UA63644,
	&UA21750,
	&UA44926,
	&UA54070,
	&UA86340,
	NULL};
const Segmentdata DHTC[] = {
	{SG68040, &UA3080874},
	{SG68109, &UA3110186},
	{SG68194, &UA3093562},
	{SG68212, &UA3080258},
	{SG68241, &UA2339384},
	{SG68242, &UA2339913},
	{SG68243, &UA2383135},
	{SG68244, &UA2596242},
	{SG68245, &UA2597098},
	{SG68246, &UA2598179},
	{SG68247, &UA2686686},
	{SG68248, &UA2728046},
	{SG68249, &UA2818101},
	{SG68250, &UA2823208},
	{SG68251, &UA2934180},
	{SG68252, &UA2982031},
	{SG68253, &UA2984714},
	{SG68254, &UA3064168},
	{SG68737, &UA2043785},
	{SG68738, &UA2043787},
	{SG68739, &UA2197873},
	{SG68740, &UA2423680},
	{SG68741, &UA2427212},
	{SG68742, &UA2467250},
	{SG68743, &UA2507679},
	{SG68744, &UA2507684},
	{SG68745, &UA2508185},
	{SG68746, &UA2509941},
	{SG68747, &UA2528128},
	{SG68748, &UA2528266},
	{SG68749, &UA2530977},
	{SG68750, &UA2555508},
	{SG68751, &UA2557848},
	{SG68752, &UA2559050},
	{SG68753, &UA2559595},
	{SG68754, &UA2560740},
	{SG68755, &UA2563045},
	{SG68757, &UA2563410},
	{SG68758, &UA2582912},
	{SG68759, &UA2597037},
	{SG68760, &UA2599587},
	{SG68761, &UA2656770},
	{SG68762, &UA2658028},
	{SG68763, &UA2662078},
	{SG68764, &UA2663097},
	{SG68765, &UA2673479},
	{SG68766, &UA2675646},
	{SG68767, &UA2676460},
	{SG68768, &UA2697698},
	{SG68769, &UA2713476},
	{SG68770, &UA2717005},
	{SG68771, &UA2727243},
	{SG68772, &UA2728521},
	{SG68773, &UA2767554},
	{SG68774, &UA2781507},
	{SG68775, &UA2844472},
	{SG68776, &UA2854845},
	{SG68777, &UA3036027},
	{SG68778, &UA3061542},
	{SG68779, &UA3066788},
	{SG68780, &UA3082588},
	{SG68781, &UA3093329},
	{SG68783, &UA3126262},
	{SG68809, &UA370407},
	{SG68856, &UA2405234},
	{SG68857, &UA2421257},
	{SG68858, &UA2429527},
	{SG68859, &UA2432021},
	{SG68860, &UA2433520},
	{SG68861, &UA2486660},
	{SG68862, &UA2506089},
	{SG68863, &UA2506158},
	{SG68864, &UA2507162},
	{SG68865, &UA2529874},
	{SG68866, &UA2530161},
	{SG68867, &UA2556076},
	{SG68868, &UA2604449},
	{SG68869, &UA2659820},
	{SG68870, &UA2669878},
	{SG68871, &UA2670103},
	{SG68872, &UA2671014},
	{SG68873, &UA2704404},
	{SG68874, &UA2705396},
	{SG68875, &UA2705758},
	{SG68876, &UA2729253},
	{SG68877, &UA2738031},
	{SG68878, &UA2752155},
	{SG68879, &UA2866612},
	{SG68880, &UA2987993},
	{SG68881, &UA3030059},
	{SG68882, &UA3030327},
	{SG68883, &UA3036017},
	{SG68884, &UA3058590},
	{SG68885, &UA3058694},
	{SG68886, &UA3058762},
	{SG68887, &UA3079565},
	{SG68912, &UA2456167},
	{SG68913, &UA2691199},
	{SG68986, &UA3058369},
	{SG68989, &UA2029620},
	{SG69009, &UA2225294},
	{SG69064, &UA2937629},
	{SG69171, &UA32},
	{SG69172, &UA4998},
	{SG69173, &UA5850},
	{SG69174, &UA5855},
	{SG69175, &UA8581},
	{SG69176, &UA10051},
	{SG69177, &UA10055},
	{SG69178, &UA30459},
	{SG69179, &UA30464},
	{SG69180, &UA30467},
	{SG69181, &UA30469},
	{SG69182, &UA30471},
	{SG69183, &UA30472},
	{SG69184, &UA30454},
	{SG69185, &UA31302},
	{SG69186, &UA38014},
	{SG69187, &UA46760},
	{SG69188, &UA438869},
	{SG69189, &UA887029},
	{SG69190, &UA925697},
	{SG69191, &UA925890},
	{SG69231, &UA869},
	{SG69232, &UA4999},
	{SG69233, &UA16210},
	{SG69234, &UA16211},
	{SG69235, &UA16218},
	{SG69236, &UA16216},
	{SG69237, &UA16219},
	{SG69238, &UA16220},
	{SG69239, &UA16222},
	{SG69240, &UA16224},
	{SG69241, &UA62029},
	{SG69242, &UA62996},
	{SG69243, &UA63299},
	{SG69244, &UA63847},
	{SG69282, &UA2679},
	{SG69283, &UA13469},
	{SG69284, &UA13544},
	{SG69285, &UA21962},
	{SG69286, &UA21963},
	{SG69287, &UA21967},
	{SG69288, &UA23808},
	{SG69289, &UA24127},
	{SG69290, &UA29232},
	{SG69291, &UA32511},
	{SG69292, &UA34864},
	{SG69293, &UA45057},
	{SG69294, &UA45058},
	{SG69295, &UA48468},
	{SG69296, &UA51201},
	{SG69297, &UA77587},
	{SG69298, &UA43654},
	{SG69299, &UA2958292},
	{SG69300, &UA2982234},
	{SG69302, &UA3125619},
	{SG69351, &UA19018},
	{SG69352, &UA31298},
	{SG69353, &UA31299},
	{SG69354, &UA35409},
	{SG69355, &UA54763},
	{SG69356, &UA89141},
	{SG69357, &UA100984},
	{SG69358, &UA116898},
	{SG69359, &UA128468},
	{SG69360, &UA135029},
	{SG69361, &UA171711},
	{SG69362, &UA171963},
	{SG69363, &UA376983},
	{SG69364, &UA433048},
	{SG69365, &UA443071},
	{SG69366, &UA720494},
	{SG69367, &UA809289},
	{SG69368, &UA1861002},
	{SG69369, &UA30},
	{SG69370, &UA73},
	{SG69371, &UA31},
	{SG69372, &UA3935},
	{SG69373, &UA5001},
	{SG69374, &UA5848},
	{SG69375, &UA5854},
	{SG69376, &UA5856},
	{SG69377, &UA7203},
	{SG69378, &UA10049},
	{SG69379, &UA16208},
	{SG69380, &UA16217},
	{SG69381, &UA30452},
	{SG69382, &UA30453},
	{SG69383, &UA30457},
	{SG69384, &UA30463},
	{SG69385, &UA30466},
	{SG69386, &UA30474},
	{SG69387, &UA30476},
	{SG69388, &UA31300},
	{SG69389, &UA31301},
	{SG69390, &UA35406},
	{SG69391, &UA35413},
	{SG69392, &UA38636},
	{SG69393, &UA46761},
	{SG69394, &UA46762},
	{SG69395, &UA51410},
	{SG69396, &UA57109},
	{SG69397, &UA61917},
	{SG69449, &UA5798},
	{SG69450, &UA2131981},
	{SG69452, &UA5074},
	{SG69453, &UA5080},
	{SG69454, &UA7548},
	{SG69455, &UA7549},
	{SG69456, &UA7551},
	{SG69457, &UA28149},
	{SG69458, &UA38204},
	{SG69459, &UA38205},
	{SG69460, &UA51839},
	{SG69461, &UA73997},
	{SG69462, &UA74835},
	{SG69463, &UA79581},
	{SG69464, &UA89737},
	{SG69493, &UA2807643},
	{SG69511, &UA2466},
	{SG69512, &UA2469},
	{SG69513, &UA2467},
	{SG69514, &UA21177},
	{SG69515, &UA21178},
	{SG69516, &UA21180},
	{SG69517, &UA21181},
	{SG69518, &UA21182},
	{SG69519, &UA21184},
	{SG69520, &UA21185},
	{SG69521, &UA21895},
	{SG69522, &UA22172},
	{SG69523, &UA35930},
	{SG69524, &UA54119},
	{SG69525, &UA89029},
	{SG69530, &UA4211},
	{SG69531, &UA7063},
	{SG69532, &UA13467},
	{SG69533, &UA13535},
	{SG69534, &UA24125},
	{SG69535, &UA24126},
	{SG69536, &UA32512},
	{SG69537, &UA48469},
	{SG69538, &UA61470},
	{SG69539, &UA66688},
	{SG69540, &UA114333},
	{SG69541, &UA270961},
	{SG69542, &UA273317},
	{SG69543, &UA295261},
	{SG69544, &UA385884},
	{SG69545, &UA863563},
	{SG69546, &UA1839935},
	{SG69547, &UA2126872},
	{SG69578, &UA2339},
	{SG69579, &UA19013},
	{SG69580, &UA19011},
	{SG69581, &UA19015},
	{SG69582, &UA32510},
	{SG69583, &UA44263},
	{SG69584, &UA61970},
	{SG69585, &UA388375},
	{SG69586, &UA483605},
	{SG69587, &UA488660},
	{SG69590, &UA19008},
	{SG69591, &UA24232},
	{SG69592, &UA24755},
	{SG69593, &UA32503},
	{SG69594, &UA32504},
	{SG69595, &UA44226},
	{SG69596, &UA44262},
	{SG69597, &UA56611},
	{SG69637, &UA28073},
	{SG69638, &UA57884},
	{SG69639, &UA57907},
	{SG69640, &UA438810},
	{SG69641, &UA2146885},
	{SG69642, &UA2165390},
	{SG69643, &UA2308764},
	{SG69644, &UA2398546},
	{SG69645, &UA2399239},
	{SG69646, &UA2444558},
	{SG69647, &UA2468036},
	{SG69648, &UA2685291},
	{SG69649, &UA2693737},
	{SG69650, &UA2696176},
	{SG69651, &UA2746670},
	{SG69652, &UA2791009},
	{SG69653, &UA2809243},
	{SG69654, &UA2819066},
	{SG69655, &UA2843447},
	{SG69656, &UA2956134},
	{SG69657, &UA2956190},
	{SG69658, &UA2985305},
	{SG69659, &UA3046311},
	{SG69660, &UA3048373},
	{SG69661, &UA3086283},
	{SG69662, &UA3087656},
	{SG69663, &UA3117031},
	{SG69664, &UA3118646},
	{SG69665, &UA3125838},
	{SG69681, &UA1891686},
	{SG69685, &UA13455},
	{SG69686, &UA29206},
	{SG69687, &UA46715},
	{SG69688, &UA66650},
	{SG69689, &UA87008},
	{SG69690, &UA87403},
	{SG69691, &UA88470},
	{SG69692, &UA88471},
	{SG69693, &UA88469},
	{SG69694, &UA92235},
	{SG69695, &UA553766},
	{SG69696, &UA681651},
	{SG69697, &UA723090},
	{SG69698, &UA729456},
	{SG69699, &UA790857},
	{SG69700, &UA1851289},
	{SG69701, &UA2125572},
	{SG69702, &UA2977342},
	{SG69703, &UA688211},
	{SG69704, &UA690874},
	{SG69705, &UA707886},
	{SG69706, &UA925706},
	{SG69707, &UA925734},
	{SG69708, &UA925735},
	{SG69709, &UA925786},
	{SG69710, &UA925833},
	{SG69711, &UA925790},
	{SG69712, &UA925919},
	{SG69713, &UA2360253},
	{SG69723, &UA13537},
	{SG69724, &UA13538},
	{SG69725, &UA13539},
	{SG69726, &UA13540},
	{SG69727, &UA13542},
	{SG69728, &UA29233},
	{SG69729, &UA34863},
	{SG69730, &UA49295},
	{SG69731, &UA54798},
	{SG69732, &UA62486},
	{SG69733, &UA66689},
	{SG69738, &UA4246},
	{SG69739, &UA12325},
	{SG69740, &UA23737},
	{SG69741, &UA24270},
	{SG69742, &UA24267},
	{SG69743, &UA28981},
	{SG69744, &UA28982},
	{SG69745, &UA66428},
	{SG69746, &UA87842},
	{SG69753, &UA3662},
	{SG69754, &UA3666},
	{SG69755, &UA5833},
	{SG69756, &UA7300},
	{SG69757, &UA7301},
	{SG69758, &UA13449},
	{SG69759, &UA13463},
	{SG69760, &UA29204},
	{SG69761, &UA29208},
	{SG69762, &UA29205},
	{SG69763, &UA34834},
	{SG69764, &UA34835},
	{SG69765, &UA51451},
	{SG69766, &UA63921},
	{SG69767, &UA66649},
	{SG69768, &UA66651},
	{SG69769, &UA74814},
	{SG69770, &UA78229},
	{SG69776, &UA4815},
	{SG69777, &UA22622},
	{SG69778, &UA27943},
	{SG69779, &UA27949},
	{SG69780, &UA27954},
	{SG69781, &UA88483},
	{SG69782, &UA89272},
	{SG69783, &UA266939},
	{SG69784, &UA372496},
	{SG69785, &UA550822},
	{SG69786, &UA637878},
	{SG69787, &UA762649},
	{SG69788, &UA821115},
	{SG69789, &UA854336},
	{SG69790, &UA2134767},
	{SG69791, &UA2242352},
	{SG69795, &UA9432},
	{SG69796, &UA21179},
	{SG69797, &UA39946},
	{SG69798, &UA438804},
	{SG69799, &UA438862},
	{SG69800, &UA925755},
	{SG69801, &UA571261},
	{SG69802, &UA752627},
	{SG69803, &UA1903679},
	{SG69804, &UA2144175},
	{SG69805, &UA2353534},
	{SG69806, &UA2402575},
	{SG69807, &UA2422423},
	{SG69808, &UA2423851},
	{SG69809, &UA2442898},
	{SG69810, &UA2455838},
	{SG69811, &UA2721938},
	{SG69812, &UA2833197},
	{SG69813, &UA2843601},
	{SG69820, &UA16212},
	{SG69821, &UA62066},
	{SG69832, &UA2408809},
	{SG69833, &UA10057},
	{SG69834, &UA13541},
	{SG69835, &UA45056},
	{SG69836, &UA500581},
	{SG69842, &UA4242},
	{SG69843, &UA12324},
	{SG69844, &UA23735},
	{SG69845, &UA28976},
	{SG69846, &UA28984},
	{SG69847, &UA45090},
	{SG69848, &UA45089},
	{SG69849, &UA49358},
	{SG69850, &UA66424},
	{SG69851, &UA66425},
	{SG69852, &UA74386},
	{SG69874, &UA1900410},
	{SG69875, &UA2148499},
	{SG69876, &UA2318182},
	{SG69877, &UA2366402},
	{SG69878, &UA2423475},
	{SG69879, &UA2480159},
	{SG69880, &UA2840504},
	{SG69906, &UA28437},
	{SG69914, &UA5075},
	{SG69915, &UA28148},
	{SG69916, &UA28151},
	{SG69917, &UA90153},
	{SG69918, &UA146093},
	{SG69919, &UA178549},
	{SG69920, &UA494705},
	{SG69921, &UA511281},
	{SG69922, &UA2262600},
	{SG69924, &UA4810},
	{SG69925, &UA4812},
	{SG69926, &UA23974},
	{SG69927, &UA27945},
	{SG69928, &UA27947},
	{SG69929, &UA27951},
	{SG69930, &UA27952},
	{SG69931, &UA27953},
	{SG69932, &UA27955},
	{SG69933, &UA36627},
	{SG69934, &UA36806},
	{SG69935, &UA36807},
	{SG69936, &UA45610},
	{SG69938, &UA51260},
	{SG69948, &UA9431},
	{SG69949, &UA20638},
	{SG69950, &UA20849},
	{SG69951, &UA32947},
	{SG69952, &UA53433},
	{SG69953, &UA54118},
	{SG69954, &UA64859},
	{SG69961, &UA13244},
	{SG69962, &UA5015},
	{SG69963, &UA11508},
	{SG69964, &UA19949},
	{SG69965, &UA40311},
	{SG69966, &UA55528},
	{SG69967, &UA86653},
	{SG69968, &UA86988},
	{SG69969, &UA87373},
	{SG69970, &UA87845},
	{SG69979, &UA2366864},
	{SG69980, &UA2384781},
	{SG69981, &UA2386826},
	{SG69985, &UA28150},
	{SG69986, &UA69727},
	{SG69987, &UA438841},
	{SG69988, &UA438858},
	{SG69989, &UA438859},
	{SG69990, &UA527638},
	{SG69991, &UA776786},
	{SG69992, &UA925722},
	{SG69993, &UA69380},
	{SG69998, &UA23814},
	{SG69999, &UA34318},
	{SG70000, &UA34317},
	{SG70001, &UA54350},
	{SG70002, &UA55953},
	{SG70003, &UA61038},
	{SG70006, &UA1901127},
	{SG70007, &UA2299578},
	{SG70008, &UA2384261},
	{SG70009, &UA2401215},
	{SG70010, &UA2427217},
	{SG70011, &UA2457176},
	{SG70012, &UA2721358},
	{SG70013, &UA2778094},
	{SG70029, &UA22439},
	{SG70030, &UA23995},
	{SG70031, &UA42184},
	{SG70035, &UA1963425},
	{SG70038, &UA2413780},
	{SG70039, &UA420999},
	{SG70040, &UA698591},
	{SG70041, &UA748092},
	{SG70042, &UA925702},
	{SG70043, &UA925740},
	{SG70044, &UA925780},
	{SG70045, &UA925900},
	{SG70046, &UA925920},
	{SG70047, &UA7319},
	{SG70048, &UA10613},
	{SG70049, &UA24587},
	{SG70050, &UA32617},
	{SG70051, &UA32620},
	{SG70052, &UA32621},
	{SG70053, &UA32739},
	{SG70054, &UA32769},
	{SG70055, &UA37923},
	{SG70056, &UA51347},
	{SG70057, &UA76194},
	{SG70066, &UA5150},
	{SG70067, &UA5151},
	{SG70069, &UA5164},
	{SG70070, &UA28088},
	{SG70071, &UA39625},
	{SG70073, &UA42585},
	{SG70074, &UA45724},
	{SG70076, &UA45715},
	{SG70077, &UA45760},
	{SG70078, &UA52962},
	{SG70079, &UA53089},
	{SG70081, &UA53136},
	{SG70082, &UA69731},
	{SG70083, &UA438807},
	{SG70084, &UA438812},
	{SG70085, &UA438832},
	{SG70086, &UA678281},
	{SG70088, &UA2312272},
	{SG70089, &UA2979536},
	{SG70095, &UA19424},
	{SG70096, &UA419188},
	{SG70097, &UA477861},
	{SG70098, &UA1839018},
	{SG70099, &UA1867183},
	{SG70101, &UA2509051},
	{SG70102, &UA2698977},
	{SG70104, &UA14226},
	{SG70113, &UA396219},
	{SG70114, &UA660615},
	{SG70115, &UA2147138},
	{SG70116, &UA2158577},
	{SG70117, &UA2296958},
	{SG70118, &UA2297559},
	{SG70119, &UA2502614},
	{SG70120, &UA2503373},
	{SG70121, &UA2578072},
	{SG70122, &UA2746557},
	{SG70123, &UA2838474},
	{SG70126, &UA397063},
	{SG70127, &UA568008},
	{SG70128, &UA570115},
	{SG70129, &UA627268},
	{SG70130, &UA1928449},
	{SG70131, &UA1982463},
	{SG70132, &UA2155795},
	{SG70133, &UA2466292},
	{SG70134, &UA2483040},
	{SG70135, &UA2543897},
	{SG70136, &UA2978058},
	{SG70145, &UA12334},
	{SG70146, &UA33048},
	{SG70147, &UA63936},
	{SG70148, &UA88178},
	{SG70149, &UA789096},
	{SG70150, &UA2242351},
	{SG70154, &UA8579},
	{SG70155, &UA10062},
	{SG70158, &UA5797},
	{SG70159, &UA20129},
	{SG70160, &UA30264},
	{SG70161, &UA37193},
	{SG70162, &UA66433},
	{SG70164, &UA3946},
	{SG70165, &UA5206},
	{SG70166, &UA28163},
	{SG70167, &UA32738},
	{SG70168, &UA45783},
	{SG70169, &UA262569},
	{SG70170, &UA536885},
	{SG70171, &UA710961},
	{SG70172, &UA1857486},
	{SG70173, &UA64056},
	{SG70174, &UA89018},
	{SG70176, &UA2482558},
	{SG70177, &UA27950},
	{SG70178, &UA90342},
	{SG70179, &UA925908},
	{SG70183, &UA2672},
	{SG70184, &UA2677},
	{SG70185, &UA21957},
	{SG70186, &UA22386},
	{SG70187, &UA61757},
	{SG70188, &UA24737},
	{SG70189, &UA44518},
	{SG70190, &UA60651},
	{SG70191, &UA87424},
	{SG70192, &UA87846},
	{SG70193, &UA69186},
	{SG70194, &UA724386},
	{SG70195, &UA2293938},
	{SG70204, &UA2560468},
	{SG70206, &UA59650},
	{SG70207, &UA61547},
	{SG70208, &UA69973},
	{SG70209, &UA71537},
	{SG70210, &UA72723},
	{SG70211, &UA438860},
	{SG70212, &UA568642},
	{SG70213, &UA1969927},
	{SG70214, &UA2222871},
	{SG70215, &UA2743250},
	{SG70217, &UA2212284},
	{SG70218, &UA237670},
	{SG70219, &UA452097},
	{SG70220, &UA572565},
	{SG70221, &UA628417},
	{SG70222, &UA642490},
	{SG70223, &UA758166},
	{SG70224, &UA1994150},
	{SG70225, &UA2316987},
	{SG70226, &UA2456358},
	{SG70227, &UA2505032},
	{SG70231, &UA777507},
	{SG70232, &UA925711},
	{SG70233, &UA925741},
	{SG70234, &UA925767},
	{SG70235, &UA925809},
	{SG70236, &UA925820},
	{SG70237, &UA925916},
	{SG70238, &UA2034648},
	{SG70239, &UA2419389},
	{SG70240, &UA11506},
	{SG70241, &UA24376},
	{SG70242, &UA26645},
	{SG70243, &UA40310},
	{SG70244, &UA40312},
	{SG70246, &UA573779},
	{SG70247, &UA609097},
	{SG70248, &UA925831},
	{SG70251, &UA4813},
	{SG70252, &UA12444},
	{SG70253, &UA24504},
	{SG70254, &UA38807},
	{SG70255, &UA38808},
	{SG70256, &UA75495},
	{SG70257, &UA2508337},
	{SG70258, &UA10059},
	{SG70260, &UA2421922},
	{SG70261, &UA2423239},
	{SG70262, &UA2427653},
	{SG70263, &UA47346},
	{SG70264, &UA66655},
	{SG70265, &UA27963},
	{SG70266, &UA63192},
	{SG70268, &UA69390},
	{SG70269, &UA562506},
	{SG70271, &UA2217831},
	{SG70272, &UA11572},
	{SG70273, &UA26706},
	{SG70274, &UA26707},
	{SG70277, &UA33289},
	{SG70278, &UA48913},
	{SG70279, &UA623957},
	{SG70280, &UA2597883},
	{SG70282, &UA13252},
	{SG70288, &UA786323},
	{SG70289, &UA925701},
	{SG70290, &UA925775},
	{SG70291, &UA925784},
	{SG70292, &UA925865},
	{SG70293, &UA74600},
	{SG70294, &UA547945},
	{SG70295, &UA749140},
	{SG70296, &UA2319229},
	{SG70297, &UA2791208},
	{SG70299, &UA2905},
	{SG70303, &UA30335},
	{SG70304, &UA30336},
	{SG70305, &UA30337},
	{SG70307, &UA2713445},
	{SG70308, &UA2800211},
	{SG70310, &UA68743},
	{SG70311, &UA770008},
	{SG70314, &UA573933},
	{SG70315, &UA737237},
	{SG70316, &UA2466432},
	{SG70317, &UA2503468},
	{SG70318, &UA2527653},
	{SG70319, &UA2980650},
	{SG70320, &UA2402034},
	{SG70321, &UA2456160},
	{SG70322, &UA2456730},
	{SG70330, &UA69267},
	{SG70331, &UA11259},
	{SG70332, &UA53492},
	{SG70333, &UA62998},
	{SG70348, &UA69180},
	{SG70351, &UA43422},
	{SG70352, &UA43493},
	{SG70353, &UA59443},
	{SG70354, &UA71943},
	{SG70355, &UA79690},
	{SG70356, &UA438808},
	{SG70357, &UA438878},
	{SG70359, &UA2150692},
	{SG70360, &UA2573542},
	{SG70361, &UA2807263},
	{SG70362, &UA628787},
	{SG70367, &UA2580599},
	{SG70368, &UA2764632},
	{SG70370, &UA19892},
	{SG70371, &UA89692},
	{SG70372, &UA3013161},
	{SG70373, &UA3020515},
	{SG70374, &UA228343},
	{SG70378, &UA2457107},
	{SG70379, &UA2482061},
	{SG70380, &UA2574612},
	{SG70381, &UA69109},
	{SG70382, &UA2804354},
	{SG70383, &UA2595222},
	{SG70390, &UA1923184},
	{SG70391, &UA2456083},
	{SG70392, &UA2504694},
	{SG70393, &UA2641344},
	{SG70394, &UA2653445},
	{SG70395, &UA2839977},
	{SG70396, &UA2660090},
	{SG70397, &UA2850850},
	{SG70398, &UA18507},
	{SG70399, &UA18511},
	{SG70400, &UA2383116},
	{SG70401, &UA2420939},
	{SG70402, &UA2653481},
	{SG70403, &UA2694693},
	{SG70404, &UA58},
	{SG70405, &UA24332},
	{SG70407, &UA64908},
	{SG70408, &UA4878},
	{SG70409, &UA26915},
	{SG70410, &UA29867},
	{SG70412, &UA41792},
	{SG70413, &UA57669},
	{SG70414, &UA69776},
	{SG70415, &UA70762},
	{SG70419, &UA2805498},
	{SG70421, &UA2402159},
	{SG70422, &UA2502766},
	{SG70423, &UA2841726},
	{SG70425, &UA18171},
	{SG70426, &UA2674},
	{SG70427, &UA2675},
	{SG70428, &UA21955},
	{SG70429, &UA21956},
	{SG70430, &UA21961},
	{SG70431, &UA36090},
	{SG70432, &UA36088},
	{SG70433, &UA44517},
	{SG70434, &UA86918},
	{SG70435, &UA5146},
	{SG70436, &UA28055},
	{SG70437, &UA28058},
	{SG70438, &UA28059},
	{SG70439, &UA28060},
	{SG70440, &UA36844},
	{SG70441, &UA45693},
	{SG70442, &UA45700},
	{SG70443, &UA45706},
	{SG70444, &UA45707},
	{SG70445, &UA53388},
	{SG70446, &UA70802},
	{SG70447, &UA71421},
	{SG70448, &UA438875},
	{SG70449, &UA438873},
	{SG70456, &UA2603372},
	{SG70457, &UA68519},
	{SG70458, &UA557809},
	{SG70459, &UA2758543},
	{SG70461, &UA68352},
	{SG70462, &UA652395},
	{SG70463, &UA2395317},
	{SG70464, &UA2519790},
	{SG70465, &UA2863144},
	{SG70472, &UA2963222},
	{SG70473, &UA3027013},
	{SG70478, &UA2660445},
	{SG70479, &UA2737098},
	{SG70480, &UA2783609},
	{SG70481, &UA27964},
	{SG70482, &UA296638},
	{SG70484, &UA2455957},
	{SG70485, &UA2465985},
	{SG70486, &UA2528398},
	{SG70489, &UA69104},
	{SG70490, &UA2752068},
	{SG70491, &UA2868433},
	{SG70494, &UA3018479},
	{SG70495, &UA3020691},
	{SG70496, &UA2909},
	{SG70497, &UA86900},
	{SG70498, &UA88347},
	{SG70501, &UA18510},
	{SG70502, &UA20551},
	{SG70503, &UA39876},
	{SG70504, &UA87450},
	{SG70516, &UA11505},
	{SG70517, &UA88159},
	{SG70518, &UA44068},
	{SG70519, &UA2021030},
	{SG70520, &UA2165428},
	{SG70521, &UA2579593},
	{SG70522, &UA2692167},
	{SG70523, &UA2748734},
	{SG70524, &UA2765139},
	{SG70526, &UA264136},
	{SG70527, &UA265029},
	{SG70531, &UA89089},
	{SG70532, &UA561980},
	{SG70533, &UA564280},
	{SG70534, &UA45758},
	{SG70535, &UA421075},
	{SG70539, &UA377749},
	{SG70540, &UA2032591},
	{SG70541, &UA736976},
	{SG70542, &UA1938366},
	{SG70543, &UA2737531},
	{SG70544, &UA2869276},
	{SG70545, &UA69198},
	{SG70546, &UA550717},
	{SG70547, &UA2218360},
	{SG70548, &UA2439570},
	{SG70549, &UA1888882},
	{SG70550, &UA62295},
	{SG70553, &UA3027979},
	{SG70554, &UA3063989},
	{SG70555, &UA3061259},
	{SG70556, &UA4253},
	{SG70557, &UA925877},
	{SG70558, &UA2467578},
	{SG70559, &UA2484120},
	{SG70560, &UA2781755},
	{SG70561, &UA2879507},
	{SG70562, &UA12448},
	{SG70563, &UA5143},
	{SG70564, &UA5145},
	{SG70565, &UA5147},
	{SG70566, &UA5148},
	{SG70567, &UA36845},
	{SG70568, &UA36848},
	{SG70569, &UA45695},
	{SG70570, &UA45696},
	{SG70571, &UA45697},
	{SG70572, &UA45699},
	{SG70573, &UA45702},
	{SG70574, &UA45704},
	{SG70575, &UA77927},
	{SG70576, &UA438828},
	{SG70578, &UA698031},
	{SG70579, &UA9537},
	{SG70580, &UA22093},
	{SG70581, &UA86894},
	{SG70582, &UA3030342},
	{SG70583, &UA3083246},
	{SG70584, &UA1883825},
	{SG70587, &UA2659643},
	{SG70588, &UA3026523},
	{SG70589, &UA3093377},
	{SG70590, &UA420804},
	{SG70593, &UA2303355},
	{SG70594, &UA2442362},
	{SG70595, &UA2496596},
	{SG70596, &UA2214683},
	{SG70598, &UA697932},
	{SG70604, &UA66557},
	{SG70605, &UA2962186},
	{SG70607, &UA4821},
	{SG70608, &UA54643},
	{SG70611, &UA3024815},
	{SG70612, &UA3093590},
	{SG70613, &UA2434},
	{SG70614, &UA567222},
	{SG70619, &UA747295},
	{SG70621, &UA2701266},
	{SG70623, &UA2962578},
	{SG70624, &UA5003},
	{SG70625, &UA16165},
	{SG70626, &UA38015},
	{SG70627, &UA2963266},
	{SG70628, &UA3026161},
	{SG70629, &UA3091806},
	{SG70630, &UA3092684},
	{SG70631, &UA18088},
	{SG70632, &UA49341},
	{SG70633, &UA687498},
	{SG70634, &UA2293821},
	{SG70635, &UA2301441},
	{SG70636, &UA2938140},
	{SG70637, &UA3018462},
	{SG70638, &UA2481},
	{SG70639, &UA69145},
	{SG70640, &UA2481183},
	{SG70641, &UA2951083},
	{SG70642, &UA2706533},
	{SG70643, &UA26136},
	{SG70644, &UA26137},
	{SG70645, &UA26138},
	{SG70646, &UA90024},
	{SG70648, &UA543451},
	{SG70649, &UA21272},
	{SG70650, &UA22010},
	{SG70652, &UA2659519},
	{SG70653, &UA3501},
	{SG70654, &UA37968},
	{SG70655, &UA19016},
	{SG70656, &UA21920},
	{SG70657, &UA531510},
	{SG70658, &UA1897385},
	{SG70659, &UA2758278},
	{SG70660, &UA2795979},
	{SG70661, &UA2958075},
	{SG70663, &UA8735},
	{SG70664, &UA566468},
	{SG70665, &UA3060992},
	{SG70669, &UA2302000},
	{SG70670, &UA2777555},
	{SG70671, &UA2844155},
	{SG70673, &UA2729986},
	{SG70674, &UA2962576},
	{SG70675, &UA3078449},
	{SG70676, &UA3108395},
	{SG70678, &UA8034},
	{SG70679, &UA450350},
	{SG70681, &UA2730190},
	{SG70683, &UA4877},
	{SG70684, &UA14225},
	{SG70685, &UA44181},
	{SG70686, &UA44182},
	{SG70687, &UA2938016},
	{SG70688, &UA3107626},
	{SG70689, &UA11385},
	{SG70690, &UA20141},
	{SG70691, &UA23963},
	{SG70692, &UA20998},
	{SG70693, &UA21000},
	{SG70695, &UA64906},
	{SG70697, &UA5311},
	{SG70698, &UA10058},
	{SG70699, &UA14223},
	{SG70700, &UA56592},
	{SG70701, &UA20996},
	{SG70702, &UA21041},
	{SG70703, &UA39912},
	{SG70704, &UA87356},
	{SG70705, &UA88747},
	{SG70706, &UA2889834},
	{SG70707, &UA44994},
	{SG70708, &UA925720},
	{SG70709, &UA925737},
	{SG70712, &UA2659686},
	{SG70714, &UA44507},
	{SG70715, &UA723224},
	{SG70717, &UA2607490},
	{SG70718, &UA2607495},
	{SG70719, &UA2852398},
	{SG70720, &UA3014216},
	{SG70721, &UA3109477},
	{SG70722, &UA10052},
	{SG70723, &UA925732},
	{SG70724, &UA925769},
	{SG70725, &UA925917},
	{SG70726, &UA2834993},
	{SG70727, &UA925821},
	{SG70728, &UA925840},
	{SG70729, &UA925911},
	{SG70730, &UA925914},
	{SG70731, &UA2290286},
	{SG70732, &UA2778960},
	{SG70734, &UA726},
	{SG70735, &UA16016},
	{SG70736, &UA31883},
	{SG70737, &UA87299},
	{SG70738, &UA22338},
	{SG70739, &UA23432},
	{SG70741, &UA1898281},
	{SG70742, &UA2774737},
	{SG70743, &UA2765490},
	{SG70744, &UA2482},
	{SG70745, &UA566093},
	{SG70746, &UA3093392},
	{SG70747, &UA5163},
	{SG70748, &UA28082},
	{SG70749, &UA38837},
	{SG70750, &UA38843},
	{SG70751, &UA76876},
	{SG70752, &UA448584},
	{SG70753, &UA1992344},
	{SG70754, &UA2147756},
	{SG70755, &UA3018390},
	{SG70757, &UA2321196},
	{SG70758, &UA2371768},
	{SG70761, &UA53837},
	{SG70762, &UA72994},
	{SG70763, &UA2695279},
	{SG70764, &UA3094182},
	{SG70765, &UA22469},
	{SG70766, &UA59894},
	{SG70767, &UA2517077},
	{SG70769, &UA4258},
	{SG70770, &UA24335},
	{SG70771, &UA19009},
	{SG70772, &UA438840},
	{SG70773, &UA1899288},
	{SG70774, &UA2607474},
	{SG70775, &UA2852077},
	{SG70776, &UA52824},
	{SG70777, &UA2306746},
	{SG70778, &UA2589241},
	{SG70779, &UA2607485},
	{SG70780, &UA2854375},
	{SG70781, &UA2863600},
	{SG70783, &UA3059087},
	{SG70784, &UA19012},
	{SG70785, &UA19017},
	{SG70786, &UA34451},
	{SG70787, &UA21244},
	{SG70788, &UA35935},
	{SG70789, &UA21919},
	{SG70790, &UA36228},
	{SG70791, &UA64628},
	{SG70793, &UA2775510},
	{SG70794, &UA2851831},
	{SG70795, &UA2933122},
	{SG70797, &UA24516},
	{SG70798, &UA35933},
	{SG70799, &UA71524},
	{SG70800, &UA26639},
	{SG70801, &UA572408},
	{SG70802, &UA1896904},
	{SG70803, &UA2775561},
	{SG70804, &UA2793114},
	{SG70805, &UA2958076},
	{SG70806, &UA3058327},
	{SG70807, &UA3058715},
	{SG70808, &UA3092316},
	{SG70809, &UA3128416},
	{SG70811, &UA32857},
	{SG70812, &UA2685953},
	{SG70813, &UA2743488},
	{SG70814, &UA3126783},
	{SG70815, &UA2906},
	{SG70816, &UA12323},
	{SG70817, &UA12328},
	{SG70818, &UA2296372},
	{SG70820, &UA2638776},
	{SG70821, &UA2788051},
	{SG70822, &UA2856583},
	{SG70823, &UA3017245},
	{SG70824, &UA3126307},
	{SG70826, &UA33493},
	{SG70827, &UA86564},
	{SG70828, &UA68102},
	{SG70829, &UA624425},
	{SG70830, &UA707902},
	{SG70831, &UA2720821},
	{SG70832, &UA2803511},
	{SG70833, &UA3077989},
	{SG70834, &UA7269},
	{SG70835, &UA87374},
	{SG70836, &UA53593},
	{SG70837, &UA438824},
	{SG70838, &UA421047},
	{SG70840, &UA2607484},
	{SG70841, &UA2607489},
	{SG70842, &UA2986167},
	{SG70843, &UA3101703},
	{SG70844, &UA19010},
	{SG70845, &UA69514},
	{SG70846, &UA438823},
	{SG70848, &UA67897},
	{SG70849, &UA68169},
	{SG70850, &UA68367},
	{SG70851, &UA492203},
	{SG70852, &UA1902558},
	{SG70853, &UA3107647},
	{SG70854, &UA16014},
	{SG70855, &UA26236},
	{SG70856, &UA87036},
	{SG70857, &UA2607476},
	{SG70859, &UA3928},
	{SG70860, &UA8736},
	{SG70861, &UA529641},
	{SG70862, &UA2277659},
	{SG70864, &UA3068990},
	{SG70865, &UA3068995},
	{SG70866, &UA13464},
	{SG70867, &UA13543},
	{SG70868, &UA60536},
	{SG70869, &UA67005},
	{SG70870, &UA4579},
	{SG70871, &UA21954},
	{SG70872, &UA21959},
	{SG70873, &UA14228},
	{SG70874, &UA44661},
	{SG70875, &UA50514},
	{SG70876, &UA438837},
	{SG70877, &UA80358},
	{SG70878, &UA2376961},
	{SG70879, &UA67611},
	{SG70880, &UA495818},
	{SG70881, &UA1869394},
	{SG70882, &UA2144390},
	{SG70883, &UA2480},
	{SG70884, &UA24336},
	{SG70885, &UA26644},
	{SG70886, &UA553120},
	{SG70887, &UA653217},
	{SG70888, &UA925731},
	{SG70891, &UA22612},
	{SG70892, &UA67251},
	{SG70893, &UA68605},
	{SG70894, &UA88746},
	{SG70895, &UA2439},
	{SG70896, &UA35924},
	{SG70897, &UA3927},
	{SG70898, &UA5149},
	{SG70899, &UA45689},
	{SG70900, &UA45708},
	{SG70901, &UA18509},
	{SG70902, &UA49890},
	{SG70903, &UA43018},
	{SG70904, &UA58934},
	{SG70907, &UA87907},
	{SG70908, &UA558054},
	{SG70909, &UA1888901},
	{SG70910, &UA2759296},
	{SG70911, &UA28161},
	{SG70912, &UA53794},
	{SG70913, &UA75533},
	{SG70914, &UA67377},
	{SG70917, &UA12336},
	{SG70918, &UA21513},
	{SG70919, &UA88388},
	{SG70920, &UA60319},
	{SG70921, &UA532731},
	{SG70922, &UA2281114},
	{SG70923, &UA3068984},
	{SG70924, &UA91},
	{SG70925, &UA4255},
	{SG70926, &UA28162},
	{SG70927, &UA24337},
	{SG70928, &UA39911},
	{SG70929, &UA61018},
	{SG70930, &UA88873},
	{SG70932, &UA90},
	{SG70933, &UA7267},
	{SG70934, &UA51430},
	{SG70935, &UA16017},
	{SG70936, &UA45106},
	{SG70937, &UA54283},
	{SG70938, &UA58932},
	{SG70939, &UA69069},
	{SG70942, &UA2817753},
	{SG70943, &UA3068985},
	{SG70944, &UA3068986},
	{SG70945, &UA3068997},
	{SG70946, &UA3069000},
	{SG70947, &UA36856},
	{SG70948, &UA36857},
	{SG70949, &UA54390},
	{SG70950, &UA67597},
	{SG70951, &UA70800},
	{SG70952, &UA77150},
	{SG70953, &UA86538},
	{SG70954, &UA577783},
	{SG70955, &UA2032497},
	{SG70956, &UA2032901},
	{SG70957, &UA2146225},
	{SG70959, &UA870},
	{SG70960, &UA32018},
	{SG70961, &UA2436},
	{SG70962, &UA8033},
	{SG70963, &UA8038},
	{SG70964, &UA10063},
	{SG70965, &UA13250},
	{SG70966, &UA24334},
	{SG70967, &UA44657},
	{SG70968, &UA56518},
	{SG70969, &UA88422},
	{SG70970, &UA559543},
	{SG70971, &UA10096},
	{SG70974, &UA26509},
	{SG70975, &UA28980},
	{SG70976, &UA87370},
	{SG70977, &UA43345},
	{SG70978, &UA69872},
	{SG70979, &UA2025617},
	{SG70980, &UA2607491},
	{SG70981, &UA12443},
	{SG70982, &UA16223},
	{SG70983, &UA20997},
	{SG70984, &UA21438},
	{SG70985, &UA36092},
	{SG70986, &UA43534},
	{SG70987, &UA53713},
	{SG70988, &UA87375},
	{SG70989, &UA87839},
	{SG70990, &UA546520},
	{SG70991, &UA2607482},
	{SG70992, &UA727},
	{SG70993, &UA2657},
	{SG70994, &UA8035},
	{SG70995, &UA16303},
	{SG70996, &UA21452},
	{SG70997, &UA44658},
	{SG70999, &UA49454},
	{SG71001, &UA87566},
	{SG71002, &UA53772},
	{SG71003, &UA61379},
	{SG71004, &UA66652},
	{SG71005, &UA86485},
	{SG71006, &UA438805},
	{SG71007, &UA438838},
	{SG71008, &UA2540551},
	{SG71009, &UA2540564},
	{SG71010, &UA2607475},
	{SG71012, &UA4811},
	{SG71013, &UA12706},
	{SG71014, &UA20986},
	{SG71015, &UA24338},
	{SG71016, &UA30334},
	{SG71017, &UA39910},
	{SG71018, &UA43533},
	{SG71019, &UA46700},
	{SG71020, &UA68793},
	{SG71021, &UA72375},
	{SG71023, &UA86512},
	{SG71024, &UA86542},
	{SG71025, &UA87365},
	{SG71026, &UA87405},
	{SG71027, &UA87549},
	{SG71028, &UA88446},
	{SG71029, &UA90572},
	{SG71030, &UA2540550},
	{SG71032, &UA2540582},
	{NULL}};
const Device *DHuawei[] = {
	&UA2998902,
	&UA3089003,
	&UA2468345,
	&UA2555122,
	&UA2556032,
	&UA2556077,
	&UA2580303,
	&UA2667080,
	&UA2763470,
	&UA3016386,
	&UA2019977,
	&UA3085101,
	&UA53517,
	&UA55522,
	&UA3093449,
	&UA2829921,
	&UA3103200,
	&UA2527618,
	&UA75720,
	&UA2728299,
	&UA2437697,
	&UA2437696,
	&UA2555410,
	&UA2876975,
	&UA2986540,
	&UA2157621,
	&UA2165053,
	&UA2219462,
	&UA2331690,
	&UA2379106,
	&UA2777760,
	&UA2789630,
	&UA2815567,
	&UA3042236,
	&UA53974,
	&UA2193490,
	&UA2934537,
	&UA2356770,
	&UA2323151,
	&UA1960570,
	&UA1983989,
	&UA2161860,
	&UA2371964,
	&UA2377516,
	&UA2414103,
	&UA2463447,
	&UA2950176,
	&UA1942319,
	&UA2392347,
	&UA2410277,
	&UA2410441,
	&UA61787,
	&UA2596228,
	&UA3057122,
	&UA30973,
	&UA54483,
	&UA2732262,
	&UA49220,
	&UA2791437,
	&UA24758,
	&UA58672,
	&UA70012,
	&UA63009,
	&UA24732,
	&UA46961,
	&UA75837,
	&UA2850493,
	&UA2988034,
	&UA22252,
	&UA22253,
	&UA43224,
	&UA2594738,
	&UA2729101,
	&UA3043536,
	&UA3048577,
	&UA3046626,
	&UA3100443,
	&UA2727729,
	&UA2745901,
	&UA2848143,
	&UA3070137,
	&UA61504,
	&UA20911,
	&UA24720,
	&UA2339186,
	&UA2643086,
	&UA49365,
	&UA2934097,
	&UA3086266,
	&UA2859550,
	&UA2994031,
	&UA2162929,
	&UA2328100,
	&UA2020955,
	&UA2218326,
	&UA2701253,
	&UA3049109,
	&UA26418,
	&UA49488,
	&UA61928,
	&UA2745241,
	&UA2843988,
	&UA2762822,
	&UA2820429,
	&UA2987874,
	&UA2984933,
	&UA2988175,
	&UA3100172,
	&UA3109648,
	&UA2812142,
	&UA2842391,
	&UA22211,
	&UA61127,
	&UA23665,
	&UA40188,
	&UA61019,
	&UA22487,
	&UA33079,
	&UA53603,
	&UA23089,
	&UA48138,
	&UA60971,
	NULL};
const Device *DKyocera[] = {
	&UA14235,
	&UA44202,
	&UA44203,
	&UA56598,
	&UA627157,
	&UA2559655,
	&UA14538,
	&UA14537,
	&UA33089,
	&UA137329,
	&UA207141,
	&UA229389,
	&UA2542585,
	&UA20252,
	&UA20427,
	&UA31570,
	&UA31571,
	&UA45019,
	&UA54559,
	&UA750724,
	&UA26641,
	&UA86913,
	&UA24290,
	&UA49419,
	&UA63312,
	&UA156947,
	&UA629240,
	&UA12119,
	&UA30867,
	&UA44032,
	&UA64763,
	&UA66299,
	&UA102201,
	&UA272320,
	&UA286123,
	&UA30803,
	&UA30806,
	&UA53633,
	&UA63294,
	&UA87774,
	&UA31038,
	&UA31041,
	&UA54398,
	&UA63824,
	&UA30947,
	&UA50626,
	&UA26790,
	&UA54479,
	&UA88985,
	&UA89080,
	&UA1938785,
	&UA32655,
	&UA51358,
	&UA20121,
	&UA20415,
	&UA21495,
	&UA30983,
	&UA58809,
	&UA44033,
	&UA56500,
	&UA84520,
	&UA9496,
	&UA84518,
	&UA43781,
	&UA84514,
	NULL};
const Segmentdata DKDDI[] = {
	{SG69598, &UA379322},
	{SG69599, &UA415622},
	{SG69600, &UA470846},
	{SG69601, &UA487744},
	{SG69602, &UA492114},
	{SG69603, &UA525993},
	{SG69604, &UA529900},
	{SG69605, &UA551804},
	{SG69606, &UA560612},
	{SG69607, &UA563608},
	{SG69608, &UA563822},
	{SG69609, &UA569236},
	{SG69610, &UA624103},
	{SG69611, &UA662023},
	{SG69612, &UA669785},
	{SG69613, &UA696824},
	{SG69614, &UA698176},
	{SG69615, &UA758527},
	{SG69616, &UA786011},
	{SG69617, &UA1891716},
	{SG69618, &UA1933103},
	{SG69619, &UA1981294},
	{SG69620, &UA2267356},
	{SG69621, &UA2270461},
	{SG69622, &UA2339879},
	{SG69623, &UA2697863},
	{SG69624, &UA2786166},
	{SG70058, &UA2437144},
	{SG70492, &UA2167352},
	{SG70552, &UA2374210},
	{SG70577, &UA2526655},
	{SG70591, &UA197621},
	{SG70597, &UA182934},
	{SG70599, &UA2527105},
	{SG70600, &UA202406},
	{SG70601, &UA2400593},
	{SG70606, &UA135885},
	{SG70609, &UA2223358},
	{SG70610, &UA2218200},
	{SG70615, &UA68693},
	{SG70616, &UA2359993},
	{SG70617, &UA2463259},
	{SG70618, &UA2412644},
	{SG70622, &UA136402},
	{SG70647, &UA202347},
	{SG70662, &UA203949},
	{SG70666, &UA2445908},
	{SG70667, &UA90106},
	{SG70668, &UA172763},
	{SG70672, &UA300685},
	{SG70677, &UA393431},
	{SG70680, &UA391066},
	{SG70682, &UA553312},
	{SG70696, &UA234861},
	{SG70710, &UA69078},
	{SG70711, &UA153200},
	{SG70713, &UA89658},
	{SG70716, &UA90399},
	{SG70733, &UA2332407},
	{SG70740, &UA149673},
	{SG70756, &UA288932},
	{SG70759, &UA56317},
	{SG70760, &UA396152},
	{SG70768, &UA318166},
	{SG70782, &UA88272},
	{SG70792, &UA90398},
	{SG70796, &UA657185},
	{SG70810, &UA87134},
	{SG70819, &UA87069},
	{SG70825, &UA318151},
	{SG70847, &UA67210},
	{SG70858, &UA286127},
	{SG70863, &UA563798},
	{SG70889, &UA562861},
	{SG70890, &UA87945},
	{SG70905, &UA87151},
	{SG70906, &UA90525},
	{SG70915, &UA87222},
	{SG70916, &UA385441},
	{SG70931, &UA394377},
	{SG70940, &UA87283},
	{SG70941, &UA89647},
	{SG70958, &UA87124},
	{SG70972, &UA477690},
	{SG70973, &UA132643},
	{SG70998, &UA87165},
	{SG71000, &UA87964},
	{SG71011, &UA87189},
	{SG71022, &UA87238},
	{SG71031, &UA87127},
	{SG71033, &UA134868},
	{SG71034, &UA432671},
	{SG71035, &UA2242265},
	{SG71036, &UA88277},
	{SG71037, &UA67175},
	{SG71038, &UA87240},
	{SG71039, &UA87929},
	{SG71040, &UA89650},
	{SG71041, &UA87272},
	{SG71042, &UA88624},
	{SG71043, &UA89762},
	{SG71044, &UA502884},
	{SG71045, &UA2803830},
	{SG71046, &UA87181},
	{SG71047, &UA88276},
	{SG71048, &UA87158},
	{SG71049, &UA87163},
	{SG71050, &UA88591},
	{SG71051, &UA89366},
	{SG71052, &UA87213},
	{SG71053, &UA87206},
	{SG71054, &UA87224},
	{SG71055, &UA87612},
	{SG71056, &UA89760},
	{SG71057, &UA87191},
	{SG71058, &UA90007},
	{SG71059, &UA491487},
	{SG71073, &UA87230},
	{SG71074, &UA87237},
	{SG71075, &UA89651},
	{SG71076, &UA90263},
	{SG71077, &UA507002},
	{SG71078, &UA89360},
	{SG71079, &UA88596},
	{SG71080, &UA89356},
	{SG71081, &UA61299},
	{SG71082, &UA87141},
	{SG71083, &UA84765},
	{SG71084, &UA87161},
	{SG71085, &UA87952},
	{SG71086, &UA87982},
	{NULL}};
const Device *DLCT[] = {
	&UA87605,
	&UA90109,
	&UA844306,
	&UA62313,
	NULL};
const Segmentdata DLenovo[] = {
	{SG70839, &UA2995408},
	{SG73125, &UA70654},
	{SG73349, &UA2149501},
	{SG73353, &UA88612},
	{SG73501, &UA2028207},
	{SG73584, &UA2999409},
	{SG73715, &UA820769},
	{SG73771, &UA769278},
	{SG73772, &UA2027418},
	{SG73883, &UA88519},
	{SG73951, &UA2709080},
	{SG74004, &UA2029164},
	{SG74042, &UA2382231},
	{SG74500, &UA2553772},
	{SG74671, &UA2362290},
	{SG74743, &UA622937},
	{SG74744, &UA641901},
	{SG74745, &UA1899398},
	{SG74746, &UA2805197},
	{SG75156, &UA89743},
	{SG75157, &UA122030},
	{SG75159, &UA2263547},
	{SG75161, &UA2265801},
	{SG75310, &UA122311},
	{SG75333, &UA2726760},
	{SG75347, &UA13655},
	{SG75348, &UA16373},
	{SG75349, &UA34476},
	{SG75608, &UA44361},
	{SG75609, &UA54225},
	{SG75654, &UA63483},
	{SG75665, &UA17050},
	{SG75666, &UA17049},
	{SG75701, &UA63915},
	{SG75712, &UA21820},
	{SG75714, &UA45226},
	{SG75715, &UA62281},
	{SG75717, &UA60592},
	{SG75732, &UA49922},
	{SG75733, &UA88826},
	{SG75734, &UA54785},
	{SG75760, &UA54101},
	{SG75775, &UA21174},
	{SG75777, &UA59476},
	{SG75797, &UA67714},
	{SG75811, &UA30858},
	{SG75812, &UA32993},
	{SG75819, &UA50258},
	{NULL}};
const Segmentdata DLG[] = {
	{SG73409, &UA22618},
	{SG73529, &UA63310},
	{SG73530, &UA63930},
	{SG73782, &UA63128},
	{SG73783, &UA63763},
	{SG73784, &UA64064},
	{SG74005, &UA27267},
	{SG74006, &UA59192},
	{SG74023, &UA62722},
	{SG74024, &UA89098},
	{SG74230, &UA44273},
	{SG74233, &UA64077},
	{SG74351, &UA23984},
	{SG74430, &UA31525},
	{SG74680, &UA20255},
	{SG74819, &UA24355},
	{SG74879, &UA20466},
	{SG74880, &UA36746},
	{SG74889, &UA64676},
	{SG74892, &UA63720},
	{SG75011, &UA32758},
	{SG75012, &UA62635},
	{SG75015, &UA18808},
	{SG75017, &UA24527},
	{SG75018, &UA62711},
	{SG75125, &UA60870},
	{SG75166, &UA64743},
	{SG75265, &UA49059},
	{SG75272, &UA63649},
	{SG75350, &UA64336},
	{SG75384, &UA13067},
	{SG75465, &UA24531},
	{SG75511, &UA33377},
	{SG75521, &UA26800},
	{SG75546, &UA87305},
	{SG75571, &UA24676},
	{SG75576, &UA96140},
	{SG75603, &UA21729},
	{SG75604, &UA24716},
	{SG75605, &UA34399},
	{SG75610, &UA24377},
	{SG75643, &UA22440},
	{SG75645, &UA62221},
	{SG75689, &UA22602},
	{SG75697, &UA62943},
	{SG75736, &UA24595},
	{SG75751, &UA53907},
	{SG75754, &UA63131},
	{SG75818, &UA87333},
	{SG75852, &UA60008},
	{SG75939, &UA24601},
	{SG75965, &UA4823},
	{SG75972, &UA61124},
	{SG75980, &UA22431},
	{SG75981, &UA60587},
	{SG75988, &UA49378},
	{SG76003, &UA22173},
	{SG76004, &UA44042},
	{SG76009, &UA45336},
	{SG76020, &UA51853},
	{SG76038, &UA22625},
	{SG76043, &UA33371},
	{SG76053, &UA22840},
	{SG76069, &UA23003},
	{SG76070, &UA23262},
	{SG76073, &UA5796},
	{SG76074, &UA23247},
	{SG76075, &UA24664},
	{SG76076, &UA49287},
	{SG76077, &UA62853},
	{SG76159, &UA20442},
	{NULL}};
const Segmentdata DLG_[] = {
	{SG73561, &UA359637},
	{SG73915, &UA794873},
	{SG74025, &UA30843},
	{SG74026, &UA64445},
	{SG74027, &UA64495},
	{SG74182, &UA64221},
	{SG74294, &UA17069},
	{SG74295, &UA64725},
	{SG74296, &UA64871},
	{SG74362, &UA62120},
	{SG74363, &UA63534},
	{SG74377, &UA2784},
	{SG74378, &UA4530},
	{SG74379, &UA44585},
	{SG74380, &UA54612},
	{SG74381, &UA64625},
	{SG74444, &UA56046},
	{SG74445, &UA872373},
	{SG74540, &UA29447},
	{SG74611, &UA12628},
	{SG74630, &UA25575},
	{SG74631, &UA26450},
	{SG74632, &UA63685},
	{SG74656, &UA64420},
	{SG74657, &UA64454},
	{SG74666, &UA9922},
	{SG74667, &UA50425},
	{SG74668, &UA62869},
	{SG74670, &UA63144},
	{SG74675, &UA64418},
	{SG74678, &UA63162},
	{SG74679, &UA64653},
	{SG74712, &UA56043},
	{SG74748, &UA18342},
	{SG74770, &UA13258},
	{SG74771, &UA64872},
	{SG74772, &UA2645},
	{SG74774, &UA21837},
	{SG74775, &UA54455},
	{SG74777, &UA63856},
	{SG74781, &UA32602},
	{SG74782, &UA64718},
	{SG74783, &UA2842520},
	{SG74784, &UA2843216},
	{SG74785, &UA2952273},
	{SG74786, &UA9408},
	{SG74787, &UA49208},
	{SG74788, &UA60734},
	{SG74789, &UA63101},
	{SG74811, &UA20510},
	{SG74812, &UA20511},
	{SG74813, &UA20512},
	{SG74814, &UA21018},
	{SG74815, &UA21526},
	{SG74816, &UA29445},
	{SG74817, &UA54829},
	{SG74818, &UA62410},
	{SG74829, &UA20126},
	{SG74830, &UA55066},
	{SG74833, &UA75788},
	{SG74860, &UA6429},
	{SG74861, &UA44339},
	{SG74862, &UA20038},
	{SG74863, &UA64599},
	{SG74864, &UA64818},
	{SG74896, &UA33403},
	{SG74897, &UA53280},
	{SG74898, &UA63316},
	{SG74912, &UA64042},
	{SG74913, &UA64860},
	{SG74919, &UA24477},
	{SG74920, &UA63821},
	{SG74926, &UA62430},
	{SG74941, &UA24451},
	{SG74942, &UA51627},
	{SG74943, &UA55146},
	{SG74944, &UA60585},
	{SG74989, &UA2464},
	{SG74990, &UA63270},
	{SG74991, &UA63467},
	{SG74992, &UA64341},
	{SG75002, &UA44197},
	{SG75019, &UA32141},
	{SG75044, &UA54371},
	{SG75045, &UA62626},
	{SG75049, &UA753447},
	{SG75079, &UA26788},
	{SG75080, &UA63233},
	{SG75081, &UA2406470},
	{SG75089, &UA8860},
	{SG75090, &UA27585},
	{SG75091, &UA35105},
	{SG75106, &UA26713},
	{SG75107, &UA26714},
	{SG75108, &UA63976},
	{SG75114, &UA24620},
	{SG75115, &UA44217},
	{SG75118, &UA63011},
	{SG75131, &UA64173},
	{SG75132, &UA67129},
	{SG75167, &UA63687},
	{SG75184, &UA64773},
	{SG75201, &UA63877},
	{SG75202, &UA2509945},
	{SG75203, &UA63652},
	{SG75207, &UA63098},
	{SG75212, &UA24519},
	{SG75213, &UA48970},
	{SG75219, &UA54392},
	{SG75222, &UA61672},
	{SG75223, &UA2205883},
	{SG75233, &UA31165},
	{SG75235, &UA7575},
	{SG75236, &UA8587},
	{SG75237, &UA8899},
	{SG75238, &UA24603},
	{SG75239, &UA87433},
	{SG75245, &UA45015},
	{SG75266, &UA63074},
	{SG75267, &UA61788},
	{SG75268, &UA24392},
	{SG75269, &UA24501},
	{SG75270, &UA644402},
	{SG75277, &UA61619},
	{SG75280, &UA44119},
	{SG75295, &UA62867},
	{SG75303, &UA50373},
	{SG75304, &UA54413},
	{SG75323, &UA13545},
	{SG75324, &UA13546},
	{SG75325, &UA13547},
	{SG75326, &UA13548},
	{SG75327, &UA13549},
	{SG75328, &UA63592},
	{SG75329, &UA66690},
	{SG75330, &UA66691},
	{SG75331, &UA66692},
	{SG75332, &UA66693},
	{SG75344, &UA63059},
	{SG75351, &UA15956},
	{SG75352, &UA15959},
	{SG75356, &UA2496848},
	{SG75360, &UA4024},
	{SG75361, &UA22358},
	{SG75362, &UA2427780},
	{SG75363, &UA2559260},
	{SG75367, &UA64601},
	{SG75368, &UA21136},
	{SG75369, &UA23890},
	{SG75370, &UA62729},
	{SG75410, &UA43570},
	{SG75411, &UA58500},
	{SG75412, &UA62909},
	{SG75413, &UA58447},
	{SG75414, &UA64222},
	{SG75420, &UA20455},
	{SG75423, &UA2599870},
	{SG75424, &UA64399},
	{SG75438, &UA3032601},
	{SG75439, &UA7657},
	{SG75440, &UA62242},
	{SG75444, &UA4440},
	{SG75445, &UA4442},
	{SG75446, &UA54989},
	{SG75447, &UA1953601},
	{SG75462, &UA26464},
	{SG75463, &UA53777},
	{SG75464, &UA62536},
	{SG75466, &UA2562206},
	{SG75468, &UA8510},
	{SG75469, &UA2383415},
	{SG75477, &UA60810},
	{SG75481, &UA61701},
	{SG75482, &UA63405},
	{SG75483, &UA69993},
	{SG75484, &UA30165},
	{SG75485, &UA105},
	{SG75486, &UA27379},
	{SG75497, &UA24559},
	{SG75499, &UA2898006},
	{SG75503, &UA2461122},
	{SG75506, &UA63315},
	{SG75508, &UA17913},
	{SG75509, &UA21857},
	{SG75510, &UA24666},
	{SG75512, &UA62565},
	{SG75513, &UA1966969},
	{SG75516, &UA64144},
	{SG75519, &UA63291},
	{SG75520, &UA29856},
	{SG75541, &UA25782},
	{SG75542, &UA54164},
	{SG75547, &UA2423649},
	{SG75558, &UA23504},
	{SG75559, &UA61778},
	{SG75565, &UA53596},
	{SG75573, &UA24667},
	{SG75574, &UA2738176},
	{SG75575, &UA1952085},
	{SG75577, &UA2242380},
	{SG75578, &UA2505384},
	{SG75582, &UA2599236},
	{SG75586, &UA32710},
	{SG75602, &UA58660},
	{SG75611, &UA3026819},
	{SG75619, &UA87720},
	{SG75625, &UA27378},
	{SG75632, &UA837925},
	{SG75635, &UA2394106},
	{SG75639, &UA54134},
	{SG75640, &UA64177},
	{SG75641, &UA2413151},
	{SG75642, &UA2878125},
	{SG75653, &UA2941836},
	{SG75663, &UA2354336},
	{SG75673, &UA24583},
	{SG75674, &UA51989},
	{SG75690, &UA64629},
	{SG75694, &UA20132},
	{SG75695, &UA64006},
	{SG75696, &UA55050},
	{SG75704, &UA33396},
	{SG75705, &UA62694},
	{SG75713, &UA63967},
	{SG75719, &UA29381},
	{SG75720, &UA26743},
	{SG75735, &UA2483180},
	{SG75739, &UA13663},
	{SG75743, &UA26789},
	{SG75745, &UA54196},
	{SG75747, &UA33139},
	{SG75749, &UA33422},
	{SG75753, &UA59884},
	{SG75755, &UA63769},
	{SG75756, &UA2217743},
	{SG75758, &UA2422035},
	{SG75761, &UA21746},
	{SG75762, &UA49939},
	{SG75763, &UA2415809},
	{SG75771, &UA23966},
	{SG75772, &UA30865},
	{SG75773, &UA58641},
	{SG75776, &UA2506444},
	{SG75778, &UA2596324},
	{SG75779, &UA22697},
	{SG75796, &UA30349},
	{SG75798, &UA63121},
	{SG75804, &UA2752013},
	{SG75805, &UA22427},
	{SG75806, &UA23385},
	{SG75808, &UA62296},
	{SG75809, &UA76417},
	{SG75810, &UA2505909},
	{SG75828, &UA63105},
	{SG75829, &UA63767},
	{SG75832, &UA87435},
	{SG75833, &UA23340},
	{SG75847, &UA2411428},
	{SG75848, &UA3058584},
	{SG75853, &UA24673},
	{SG75854, &UA32819},
	{SG75857, &UA43915},
	{SG75858, &UA53647},
	{SG75859, &UA2445267},
	{SG75860, &UA2464936},
	{SG75881, &UA2795676},
	{SG75883, &UA2518670},
	{SG75887, &UA63249},
	{SG75888, &UA64461},
	{SG75907, &UA23973},
	{SG75908, &UA55219},
	{SG75918, &UA64303},
	{SG75919, &UA2792800},
	{SG75921, &UA22873},
	{SG75922, &UA23500},
	{SG75926, &UA32462},
	{SG75956, &UA2944152},
	{SG75958, &UA22238},
	{SG75985, &UA3060323},
	{SG75986, &UA24557},
	{SG75987, &UA26967},
	{SG75992, &UA2968787},
	{SG75998, &UA2242304},
	{SG76006, &UA62233},
	{SG76015, &UA23657},
	{SG76037, &UA44213},
	{SG76048, &UA2858913},
	{SG76049, &UA2465},
	{SG76050, &UA21664},
	{SG76051, &UA24680},
	{SG76064, &UA79291},
	{SG76079, &UA3012782},
	{SG76091, &UA62204},
	{SG76093, &UA2444515},
	{SG76094, &UA2592259},
	{SG76095, &UA2693666},
	{SG76096, &UA21202},
	{SG76097, &UA21308},
	{SG76099, &UA22425},
	{SG76100, &UA42029},
	{SG76101, &UA87324},
	{SG76102, &UA26150},
	{SG76103, &UA25504},
	{SG76104, &UA55144},
	{SG76105, &UA55217},
	{SG76111, &UA2482970},
	{SG76112, &UA3011600},
	{SG76122, &UA49217},
	{SG76124, &UA2768728},
	{SG76125, &UA96},
	{SG76126, &UA21307},
	{SG76160, &UA3080025},
	{SG76163, &UA12998},
	{SG76164, &UA22419},
	{SG76165, &UA78459},
	{SG76166, &UA33378},
	{SG76175, &UA2779046},
	{SG76176, &UA44227},
	{SG76177, &UA48894},
	{SG76178, &UA53676},
	{SG76179, &UA63185},
	{SG76181, &UA2701470},
	{SG76182, &UA61868},
	{SG76183, &UA23258},
	{SG76185, &UA62513},
	{SG76186, &UA62775},
	{SG76187, &UA70224},
	{SG76195, &UA2850494},
	{SG76197, &UA9292},
	{SG76198, &UA26757},
	{SG76199, &UA62195},
	{SG76207, &UA2213710},
	{SG76208, &UA2657521},
	{SG76211, &UA20120},
	{SG76212, &UA2579073},
	{SG76220, &UA20027},
	{SG76221, &UA21447},
	{SG76222, &UA44318},
	{SG76224, &UA2507296},
	{SG76229, &UA3100921},
	{SG76230, &UA22448},
	{SG76232, &UA63129},
	{SG76235, &UA2877997},
	{SG76237, &UA4432},
	{SG76254, &UA2682555},
	{SG76265, &UA21213},
	{SG76266, &UA22177},
	{SG76271, &UA2786219},
	{SG76280, &UA22349},
	{SG76281, &UA61121},
	{SG76282, &UA75125},
	{SG76283, &UA61837},
	{SG76284, &UA2729159},
	{SG76285, &UA2868137},
	{SG76289, &UA699503},
	{SG76303, &UA5053},
	{SG76304, &UA64452},
	{SG76305, &UA61241},
	{SG76310, &UA2778194},
	{SG76314, &UA2987125},
	{SG76327, &UA3125515},
	{SG76328, &UA54632},
	{SG76329, &UA53818},
	{SG76332, &UA573364},
	{SG76334, &UA2300892},
	{SG76351, &UA5038},
	{SG76352, &UA23518},
	{SG76353, &UA27965},
	{SG76354, &UA62256},
	{SG76358, &UA2145611},
	{SG76359, &UA2307153},
	{SG76363, &UA2824958},
	{SG76365, &UA3044717},
	{SG76366, &UA3107208},
	{SG76368, &UA3108055},
	{SG76377, &UA2392196},
	{SG76378, &UA2440039},
	{SG76379, &UA2817477},
	{SG76382, &UA8988},
	{SG76383, &UA61764},
	{SG76384, &UA9165},
	{SG76385, &UA19957},
	{SG76386, &UA24654},
	{SG76387, &UA20148},
	{SG76388, &UA20149},
	{SG76389, &UA24655},
	{SG76390, &UA21817},
	{SG76391, &UA51013},
	{SG76396, &UA27966},
	{SG76397, &UA54430},
	{SG76408, &UA22917},
	{SG76409, &UA23363},
	{SG76411, &UA61375},
	{SG76412, &UA64736},
	{SG76415, &UA2370324},
	{SG76417, &UA18812},
	{SG76418, &UA21378},
	{SG76419, &UA22865},
	{SG76420, &UA23086},
	{SG76421, &UA24093},
	{SG76422, &UA62289},
	{SG76423, &UA63347},
	{SG76426, &UA31507},
	{SG76427, &UA44200},
	{SG76428, &UA53345},
	{SG76433, &UA21535},
	{SG76434, &UA24369},
	{SG76435, &UA67435},
	{SG76436, &UA2129533},
	{SG76437, &UA33458},
	{SG76438, &UA61162},
	{SG76439, &UA61254},
	{SG76442, &UA24644},
	{SG76443, &UA32983},
	{SG76444, &UA49310},
	{SG76445, &UA61280},
	{SG76446, &UA71488},
	{SG76448, &UA21260},
	{SG76449, &UA23442},
	{SG76450, &UA26792},
	{SG76451, &UA48137},
	{SG76452, &UA55135},
	{SG76453, &UA61404},
	{SG76454, &UA63188},
	{SG76455, &UA64040},
	{SG76456, &UA89770},
	{SG76460, &UA27968},
	{SG76461, &UA32923},
	{SG76463, &UA22169},
	{SG76464, &UA22413},
	{SG76465, &UA22601},
	{SG76466, &UA24630},
	{SG76467, &UA33018},
	{SG76468, &UA33117},
	{SG76473, &UA61567},
	{SG76477, &UA23563},
	{SG76478, &UA21736},
	{SG76481, &UA62753},
	{SG76482, &UA55001},
	{SG76483, &UA64451},
	{SG76484, &UA86668},
	{SG76489, &UA21446},
	{SG76490, &UA23958},
	{SG76491, &UA26147},
	{SG76492, &UA32792},
	{SG76493, &UA49346},
	{SG76494, &UA87641},
	{SG76497, &UA61104},
	{SG76498, &UA22364},
	{SG76499, &UA22375},
	{SG76500, &UA23163},
	{SG76501, &UA31160},
	{SG76502, &UA32876},
	{SG76503, &UA33353},
	{SG76504, &UA60939},
	{SG76505, &UA64843},
	{SG76506, &UA89070},
	{SG76508, &UA4737},
	{SG76515, &UA77043},
	{SG76518, &UA21282},
	{SG76519, &UA21442},
	{SG76522, &UA21752},
	{SG76525, &UA33391},
	{SG76526, &UA61339},
	{SG76527, &UA73524},
	{SG76528, &UA83929},
	{SG76529, &UA87833},
	{SG76532, &UA22276},
	{SG76533, &UA32970},
	{SG76534, &UA33334},
	{SG76535, &UA53468},
	{SG76536, &UA54456},
	{SG76537, &UA69560},
	{SG76538, &UA69590},
	{SG76540, &UA83826},
	{SG76541, &UA83872},
	{SG76542, &UA84070},
	{SG76543, &UA87556},
	{SG76544, &UA88603},
	{SG76557, &UA24581},
	{SG76558, &UA32938},
	{SG76559, &UA32977},
	{SG76560, &UA33136},
	{SG76561, &UA33194},
	{SG76564, &UA55092},
	{SG76565, &UA61073},
	{SG76567, &UA83938},
	{SG76568, &UA84052},
	{SG76569, &UA84134},
	{SG76570, &UA88689},
	{SG76574, &UA22078},
	{SG76575, &UA22245},
	{SG76576, &UA22923},
	{SG76577, &UA32895},
	{SG76580, &UA33064},
	{SG76581, &UA33397},
	{SG76582, &UA45387},
	{SG76583, &UA45493},
	{SG76584, &UA49329},
	{SG76585, &UA49510},
	{SG76586, &UA49982},
	{SG76588, &UA53951},
	{SG76589, &UA56808},
	{SG76591, &UA60537},
	{SG76592, &UA60555},
	{SG76593, &UA61142},
	{SG76594, &UA61447},
	{SG76595, &UA67608},
	{SG76596, &UA68846},
	{SG76602, &UA83818},
	{SG76603, &UA83831},
	{SG76604, &UA83925},
	{SG76605, &UA83927},
	{SG76606, &UA84180},
	{SG76607, &UA87428},
	{NULL}};
const Segmentdata DMIDP_CLDC[] = {
	{SG71060, &UA20437},
	{SG71061, &UA23052},
	{SG71062, &UA23151},
	{SG71063, &UA54445},
	{SG71064, &UA56928},
	{SG71065, &UA63813},
	{SG71066, &UA75512},
	{SG71067, &UA87795},
	{SG71068, &UA88485},
	{SG71069, &UA88672},
	{SG71070, &UA140891},
	{SG71071, &UA173611},
	{SG71072, &UA262000},
	{SG72640, &UA2642782},
	{SG72641, &UA2995805},
	{SG72642, &UA3042769},
	{SG72665, &UA2167110},
	{SG73526, &UA2413660},
	{SG73527, &UA2851106},
	{SG73528, &UA3071831},
	{SG73599, &UA23035},
	{SG73600, &UA23119},
	{SG73601, &UA76471},
	{SG73602, &UA90162},
	{SG73603, &UA432838},
	{SG73604, &UA2484602},
	{SG73605, &UA2505952},
	{SG73606, &UA2506064},
	{SG73607, &UA2528565},
	{SG73608, &UA2556630},
	{SG73609, &UA2557754},
	{SG73610, &UA2582287},
	{SG73611, &UA2597348},
	{SG73612, &UA2597527},
	{SG73613, &UA2656092},
	{SG73614, &UA2656956},
	{SG73615, &UA2657483},
	{SG73616, &UA2659041},
	{SG73617, &UA2659435},
	{SG73618, &UA2660458},
	{SG73619, &UA2662340},
	{SG73620, &UA2663753},
	{SG73621, &UA2699330},
	{SG73622, &UA2703187},
	{SG73623, &UA2705503},
	{SG73624, &UA2707872},
	{SG73625, &UA2709413},
	{SG73626, &UA2712457},
	{SG73627, &UA2726773},
	{SG73628, &UA2728778},
	{SG73629, &UA2729913},
	{SG73630, &UA2729970},
	{SG73631, &UA2732184},
	{SG73632, &UA2734318},
	{SG73633, &UA2734458},
	{SG73634, &UA2735303},
	{SG73635, &UA2737000},
	{SG73636, &UA2737112},
	{SG73637, &UA2778390},
	{SG73638, &UA2780118},
	{SG73639, &UA2792288},
	{SG73640, &UA2793860},
	{SG73641, &UA2794336},
	{SG73642, &UA2844836},
	{SG73643, &UA2853072},
	{SG73644, &UA2854830},
	{SG73645, &UA2855697},
	{SG73646, &UA2858838},
	{SG73647, &UA2865671},
	{SG73648, &UA2868659},
	{SG73649, &UA2892485},
	{SG73650, &UA2934609},
	{SG73651, &UA2935897},
	{SG73652, &UA2935967},
	{SG73653, &UA2956139},
	{SG73654, &UA2957275},
	{SG73655, &UA2986458},
	{SG73656, &UA2987226},
	{SG73657, &UA2988030},
	{SG73658, &UA2989071},
	{SG73659, &UA2994417},
	{SG73660, &UA2994974},
	{SG73661, &UA2995140},
	{SG73662, &UA2995219},
	{SG73663, &UA2996763},
	{SG73664, &UA2998047},
	{SG73665, &UA2998333},
	{SG73666, &UA2999289},
	{SG73667, &UA3000796},
	{SG73668, &UA3002886},
	{SG73669, &UA3005895},
	{SG73670, &UA3028329},
	{SG73671, &UA3045804},
	{SG73672, &UA3048390},
	{SG73673, &UA3049424},
	{SG73674, &UA3049746},
	{SG73675, &UA3056781},
	{SG73676, &UA3057812},
	{SG73677, &UA3057870},
	{SG73678, &UA3074080},
	{SG73679, &UA3079851},
	{SG73680, &UA3088348},
	{SG73681, &UA3089007},
	{SG73725, &UA641641},
	{SG73726, &UA699794},
	{SG73727, &UA3022357},
	{SG73831, &UA53965},
	{SG73833, &UA56394},
	{SG74029, &UA277588},
	{SG74432, &UA63905},
	{SG74633, &UA5795},
	{SG74634, &UA43550},
	{SG74635, &UA53605},
	{SG74636, &UA53606},
	{SG74637, &UA54175},
	{SG74638, &UA56228},
	{SG74639, &UA56923},
	{SG74640, &UA63050},
	{SG74641, &UA64882},
	{SG74652, &UA2511},
	{SG74653, &UA35487},
	{SG74654, &UA63842},
	{SG74655, &UA89775},
	{SG74658, &UA22918},
	{SG74665, &UA53806},
	{SG74669, &UA2995903},
	{SG74820, &UA87703},
	{SG74821, &UA87893},
	{SG74822, &UA88661},
	{SG74823, &UA88876},
	{SG74824, &UA88934},
	{SG74825, &UA88877},
	{SG74826, &UA89139},
	{SG74827, &UA90778},
	{SG74828, &UA554708},
	{SG74831, &UA2273446},
	{SG74832, &UA2299877},
	{SG74890, &UA634092},
	{SG74891, &UA2598972},
	{SG74921, &UA87863},
	{SG74922, &UA14212},
	{SG74923, &UA14213},
	{SG74924, &UA44136},
	{SG74925, &UA44137},
	{SG74927, &UA7824},
	{SG74928, &UA9901},
	{SG74929, &UA9902},
	{SG74930, &UA9903},
	{SG74931, &UA9904},
	{SG74932, &UA9905},
	{SG74975, &UA49625},
	{SG74976, &UA77430},
	{SG74977, &UA90729},
	{SG74978, &UA90730},
	{SG74979, &UA435614},
	{SG74980, &UA511222},
	{SG74981, &UA511490},
	{SG74982, &UA517183},
	{SG74983, &UA522248},
	{SG74984, &UA1891361},
	{SG74985, &UA1896309},
	{SG75038, &UA15882},
	{SG75039, &UA15883},
	{SG75040, &UA63663},
	{SG75041, &UA63662},
	{SG75042, &UA88896},
	{SG75104, &UA20225},
	{SG75105, &UA54556},
	{SG75185, &UA5445},
	{SG75186, &UA29446},
	{SG75187, &UA37023},
	{SG75188, &UA62882},
	{SG75189, &UA63264},
	{SG75190, &UA2403319},
	{SG75191, &UA1953538},
	{SG75192, &UA2686610},
	{SG75204, &UA2981450},
	{SG75205, &UA3070292},
	{SG75214, &UA2520076},
	{SG75220, &UA24743},
	{SG75221, &UA59881},
	{SG75232, &UA55103},
	{SG75246, &UA26419},
	{SG75247, &UA26420},
	{SG75249, &UA55322},
	{SG75251, &UA61452},
	{SG75252, &UA62449},
	{SG75253, &UA63276},
	{SG75271, &UA2403089},
	{SG75281, &UA53595},
	{SG75282, &UA54362},
	{SG75285, &UA64174},
	{SG75305, &UA2556770},
	{SG75306, &UA2959642},
	{SG75357, &UA20424},
	{SG75358, &UA58813},
	{SG75359, &UA2303348},
	{SG75364, &UA632367},
	{SG75365, &UA2843936},
	{SG75371, &UA9479},
	{SG75372, &UA9480},
	{SG75373, &UA90740},
	{SG75374, &UA90741},
	{SG75375, &UA690170},
	{SG75377, &UA23606},
	{SG75378, &UA23607},
	{SG75379, &UA23608},
	{SG75380, &UA33382},
	{SG75381, &UA62348},
	{SG75382, &UA62346},
	{SG75383, &UA64457},
	{SG75419, &UA642439},
	{SG75421, &UA2168372},
	{SG75432, &UA53787},
	{SG75433, &UA63392},
	{SG75443, &UA2781830},
	{SG75478, &UA2558595},
	{SG75480, &UA2704617},
	{SG75498, &UA2164101},
	{SG75515, &UA61056},
	{SG75522, &UA2257},
	{SG75523, &UA18814},
	{SG75524, &UA18816},
	{SG75525, &UA18817},
	{SG75526, &UA18819},
	{SG75527, &UA18823},
	{SG75528, &UA18824},
	{SG75529, &UA18825},
	{SG75530, &UA20643},
	{SG75531, &UA20644},
	{SG75532, &UA35808},
	{SG75533, &UA35807},
	{SG75534, &UA39243},
	{SG75535, &UA39248},
	{SG75536, &UA54916},
	{SG75537, &UA62478},
	{SG75538, &UA62580},
	{SG75539, &UA62891},
	{SG75540, &UA89097},
	{SG75549, &UA62237},
	{SG75566, &UA54576},
	{SG75570, &UA2360237},
	{SG75572, &UA2482522},
	{SG75595, &UA88671},
	{SG75596, &UA53634},
	{SG75597, &UA56586},
	{SG75598, &UA56587},
	{SG75599, &UA62283},
	{SG75600, &UA62339},
	{SG75601, &UA62340},
	{SG75624, &UA2879642},
	{SG75636, &UA2967599},
	{SG75637, &UA3092965},
	{SG75638, &UA3126519},
	{SG75648, &UA54026},
	{SG75672, &UA9016},
	{SG75702, &UA22304},
	{SG75703, &UA53607},
	{SG75710, &UA21743},
	{SG75711, &UA53896},
	{SG75716, &UA2559290},
	{SG75718, &UA2421961},
	{SG75737, &UA2162959},
	{SG75738, &UA292616},
	{SG75744, &UA2863796},
	{SG75757, &UA24725},
	{SG75759, &UA58617},
	{SG75768, &UA2893363},
	{SG75769, &UA2937595},
	{SG75770, &UA3050067},
	{SG75774, &UA182169},
	{SG75780, &UA4762},
	{SG75781, &UA26632},
	{SG75782, &UA26633},
	{SG75783, &UA27523},
	{SG75784, &UA27527},
	{SG75785, &UA27526},
	{SG75786, &UA36764},
	{SG75787, &UA45505},
	{SG75788, &UA54380},
	{SG75789, &UA63258},
	{SG75790, &UA63891},
	{SG75795, &UA2578513},
	{SG75807, &UA62568},
	{SG75821, &UA3014736},
	{SG75822, &UA3078144},
	{SG75834, &UA2217822},
	{SG75855, &UA2447746},
	{SG75862, &UA2552315},
	{SG75893, &UA155394},
	{SG75900, &UA2242336},
	{SG75909, &UA2167601},
	{SG75920, &UA176439},
	{SG75925, &UA2420921},
	{SG75927, &UA54896},
	{SG75928, &UA160959},
	{SG75947, &UA19886},
	{SG75948, &UA41566},
	{SG75949, &UA76256},
	{SG75950, &UA133878},
	{SG75953, &UA2525390},
	{SG75978, &UA2223068},
	{SG75979, &UA2396427},
	{SG75989, &UA2555055},
	{SG75993, &UA18676},
	{SG75995, &UA32935},
	{SG75997, &UA48021},
	{SG76008, &UA2861656},
	{SG76013, &UA2220905},
	{SG76019, &UA2596098},
	{SG76034, &UA62382},
	{SG76054, &UA13054},
	{SG76055, &UA13056},
	{SG76056, &UA43854},
	{SG76057, &UA49100},
	{SG76058, &UA49101},
	{SG76059, &UA54807},
	{SG76060, &UA56884},
	{SG76061, &UA62253},
	{SG76062, &UA63086},
	{SG76063, &UA63940},
	{SG76065, &UA2700047},
	{SG76071, &UA789626},
	{SG76072, &UA2475646},
	{SG76087, &UA2442289},
	{SG76113, &UA487869},
	{SG76114, &UA510590},
	{SG76115, &UA724361},
	{SG76116, &UA2134989},
	{SG76117, &UA2140939},
	{SG76123, &UA2983008},
	{SG76184, &UA2596016},
	{SG76189, &UA43784},
	{SG76192, &UA2590525},
	{SG76196, &UA2410671},
	{SG76218, &UA24536},
	{SG76219, &UA24646},
	{SG76231, &UA66645},
	{SG76252, &UA2548424},
	{SG76253, &UA2763741},
	{SG76269, &UA2496743},
	{SG76270, &UA4021},
	{SG76272, &UA49091},
	{SG76273, &UA49539},
	{SG76292, &UA61510},
	{SG76302, &UA48991},
	{SG76360, &UA64707},
	{SG76370, &UA3109125},
	{SG76373, &UA2641766},
	{SG76380, &UA20421},
	{SG76381, &UA24757},
	{SG76424, &UA2982720},
	{SG76425, &UA2985016},
	{SG76475, &UA11381},
	{SG76476, &UA11380},
	{SG76488, &UA2986923},
	{SG76545, &UA20171},
	{SG76546, &UA78053},
	{SG76566, &UA58810},
	{SG76573, &UA3099042},
	{SG76587, &UA23985},
	{SG76590, &UA86844},
	{SG76611, &UA32156},
	{SG76614, &UA22418},
	{SG76615, &UA86287},
	{SG76619, &UA2463},
	{SG76620, &UA21176},
	{SG76621, &UA20389},
	{SG76622, &UA33204},
	{SG76623, &UA49895},
	{SG76629, &UA563773},
	{SG76632, &UA10095},
	{SG76633, &UA25572},
	{SG76634, &UA21373},
	{SG76635, &UA21374},
	{SG76636, &UA62162},
	{SG76637, &UA28373},
	{SG76638, &UA42881},
	{SG76639, &UA54994},
	{SG76644, &UA20423},
	{SG76647, &UA20422},
	{SG76648, &UA23653},
	{SG76652, &UA50995},
	{SG76654, &UA90176},
	{SG76661, &UA63787},
	{SG76663, &UA2041},
	{SG76664, &UA50971},
	{SG76665, &UA50970},
	{SG76666, &UA56910},
	{SG76667, &UA3239},
	{SG76668, &UA54868},
	{SG76669, &UA89156},
	{SG76670, &UA89688},
	{SG76671, &UA953},
	{SG76672, &UA44996},
	{SG76673, &UA369727},
	{SG76675, &UA22302},
	{SG76676, &UA24147},
	{SG76677, &UA45000},
	{SG76678, &UA56665},
	{SG76679, &UA62192},
	{SG76680, &UA60553},
	{SG76683, &UA33110},
	{SG76684, &UA63882},
	{SG76688, &UA43764},
	{SG76692, &UA9965},
	{SG76693, &UA17447},
	{SG76694, &UA35729},
	{SG76695, &UA49138},
	{SG76696, &UA49276},
	{SG76697, &UA55031},
	{SG76698, &UA62416},
	{SG76702, &UA23809},
	{SG76703, &UA23883},
	{SG76705, &UA29751},
	{SG76706, &UA31099},
	{SG76707, &UA49522},
	{SG76708, &UA53726},
	{SG76709, &UA87776},
	{SG76719, &UA26965},
	{SG76720, &UA32954},
	{SG76721, &UA53680},
	{SG76722, &UA73630},
	{SG76725, &UA258004},
	{SG76729, &UA27996},
	{NULL}};
const Segmentdata DLGE_[] = {
	{SG74589, &UA31286},
	{SG74808, &UA72767},
	{SG74911, &UA30903},
	{SG75013, &UA49215},
	{SG75014, &UA49216},
	{SG75035, &UA2385864},
	{SG75036, &UA2528738},
	{SG75043, &UA49802},
	{SG75168, &UA40011},
	{SG75169, &UA14392},
	{SG75170, &UA14393},
	{SG75171, &UA45020},
	{SG75206, &UA24753},
	{SG75229, &UA30837},
	{SG75230, &UA30838},
	{SG75231, &UA61248},
	{SG75243, &UA33176},
	{SG75261, &UA43562},
	{SG75262, &UA45238},
	{SG75366, &UA26462},
	{SG75376, &UA754214},
	{SG75425, &UA70840},
	{SG75441, &UA54468},
	{SG75514, &UA31043},
	{SG75548, &UA89764},
	{SG75607, &UA64502},
	{SG75620, &UA44286},
	{SG75664, &UA55075},
	{SG75675, &UA2978552},
	{SG75721, &UA44002},
	{SG75816, &UA21726},
	{SG75817, &UA31037},
	{SG75820, &UA30985},
	{SG75823, &UA49951},
	{SG75851, &UA55238},
	{SG75911, &UA49053},
	{SG75912, &UA84058},
	{SG75913, &UA84060},
	{SG75914, &UA86658},
	{SG75915, &UA86688},
	{SG75916, &UA87619},
	{SG75917, &UA3049779},
	{SG76052, &UA53905},
	{SG76066, &UA55122},
	{SG76078, &UA63222},
	{SG76098, &UA54506},
	{SG76161, &UA55270},
	{SG76162, &UA61842},
	{SG76174, &UA86972},
	{SG76288, &UA32091},
	{SG76291, &UA26365},
	{SG76301, &UA24485},
	{SG76313, &UA53861},
	{SG76459, &UA5033},
	{SG76462, &UA24308},
	{SG76531, &UA23879},
	{SG76539, &UA62083},
	{SG76578, &UA21630},
	{SG76612, &UA23975},
	{SG76618, &UA33128},
	{SG76626, &UA61243},
	{SG76655, &UA24569},
	{SG76699, &UA33054},
	{SG76726, &UA60811},
	{SG76730, &UA33297},
	{SG76733, &UA63435},
	{SG76750, &UA61037},
	{SG76756, &UA23584},
	{SG76757, &UA62403},
	{SG76758, &UA62655},
	{SG76760, &UA87777},
	{SG76763, &UA21636},
	{SG76764, &UA22206},
	{SG76765, &UA48879},
	{SG76768, &UA24483},
	{SG76769, &UA24505},
	{SG76778, &UA33150},
	{SG76779, &UA86740},
	{SG76780, &UA62490},
	{SG76781, &UA69512},
	{SG76789, &UA62554},
	{SG76803, &UA24546},
	{SG76804, &UA26451},
	{SG76806, &UA64156},
	{SG76807, &UA87831},
	{SG76811, &UA49136},
	{SG76812, &UA62553},
	{SG76813, &UA21891},
	{SG76814, &UA22687},
	{SG76816, &UA24573},
	{SG76836, &UA49999},
	{SG76837, &UA87113},
	{SG76838, &UA87432},
	{SG76839, &UA87822},
	{NULL}};
const Segmentdata DMOT_[] = {
	{SG75016, &UA61515},
	{SG75092, &UA379422},
	{SG75093, &UA656904},
	{SG75094, &UA1905554},
	{SG75095, &UA2401371},
	{SG75234, &UA681490},
	{SG75296, &UA307},
	{SG75297, &UA305},
	{SG75298, &UA311},
	{SG75299, &UA312},
	{SG75300, &UA14431},
	{SG75301, &UA14446},
	{SG75302, &UA35106},
	{SG75334, &UA44284},
	{SG75335, &UA67349},
	{SG75336, &UA577},
	{SG75337, &UA623},
	{SG75338, &UA10343},
	{SG75339, &UA10347},
	{SG75340, &UA15765},
	{SG75341, &UA15815},
	{SG75342, &UA15827},
	{SG75343, &UA54184},
	{SG75345, &UA442768},
	{SG75346, &UA783107},
	{SG75385, &UA2111},
	{SG75386, &UA13958},
	{SG75387, &UA13962},
	{SG75388, &UA13974},
	{SG75389, &UA13978},
	{SG75390, &UA13979},
	{SG75391, &UA13980},
	{SG75392, &UA23397},
	{SG75393, &UA29469},
	{SG75394, &UA29472},
	{SG75395, &UA29475},
	{SG75396, &UA51680},
	{SG75397, &UA66842},
	{SG75398, &UA66841},
	{SG75399, &UA66853},
	{SG75400, &UA637511},
	{SG75401, &UA674644},
	{SG75409, &UA336574},
	{SG75467, &UA32543},
	{SG75491, &UA2879},
	{SG75492, &UA22810},
	{SG75493, &UA50996},
	{SG75494, &UA55871},
	{SG75495, &UA62317},
	{SG75496, &UA86295},
	{SG75507, &UA2165270},
	{SG75550, &UA44179},
	{SG75551, &UA55267},
	{SG75552, &UA61977},
	{SG75553, &UA64030},
	{SG75554, &UA67827},
	{SG75555, &UA87328},
	{SG75556, &UA1845902},
	{SG75557, &UA2264193},
	{SG75591, &UA2422},
	{SG75592, &UA2426},
	{SG75593, &UA61482},
	{SG75594, &UA82967},
	{SG75606, &UA2242665},
	{SG75613, &UA31803},
	{SG75614, &UA61851},
	{SG75616, &UA62405},
	{SG75655, &UA22409},
	{SG75667, &UA507789},
	{SG75668, &UA1994146},
	{SG75683, &UA15238},
	{SG75684, &UA29859},
	{SG75685, &UA31518},
	{SG75686, &UA31519},
	{SG75687, &UA37476},
	{SG75688, &UA50054},
	{SG75722, &UA403},
	{SG75723, &UA2102},
	{SG75724, &UA7794},
	{SG75725, &UA7845},
	{SG75726, &UA15423},
	{SG75727, &UA30848},
	{SG75728, &UA61650},
	{SG75729, &UA61958},
	{SG75746, &UA49805},
	{SG75748, &UA395167},
	{SG75750, &UA2650643},
	{SG75752, &UA3117782},
	{SG75767, &UA90083},
	{SG75830, &UA20153},
	{SG75831, &UA21641},
	{SG75835, &UA391},
	{SG75836, &UA15394},
	{SG75837, &UA15401},
	{SG75838, &UA15404},
	{SG75856, &UA2168910},
	{SG75866, &UA62467},
	{SG75870, &UA13019},
	{SG75871, &UA18051},
	{SG75872, &UA20343},
	{SG75873, &UA50250},
	{SG75874, &UA56292},
	{SG75875, &UA63259},
	{SG75876, &UA67322},
	{SG75877, &UA67388},
	{SG75878, &UA135403},
	{SG75879, &UA140782},
	{SG75880, &UA667005},
	{SG75891, &UA24706},
	{SG75892, &UA67299},
	{SG75894, &UA67748},
	{SG75895, &UA459991},
	{SG75896, &UA7768},
	{SG75897, &UA11232},
	{SG75902, &UA20098},
	{SG75904, &UA541245},
	{SG75937, &UA17281},
	{SG75938, &UA62890},
	{SG75951, &UA8107},
	{SG75952, &UA63991},
	{SG75959, &UA34481},
	{SG75960, &UA68167},
	{SG75976, &UA38559},
	{SG75977, &UA38560},
	{SG76005, &UA697007},
	{SG76007, &UA2256},
	{SG76021, &UA30863},
	{SG76022, &UA32459},
	{SG76023, &UA32460},
	{SG76024, &UA53792},
	{SG76025, &UA60580},
	{SG76026, &UA60985},
	{SG76027, &UA64231},
	{SG76028, &UA89995},
	{SG76029, &UA2320694},
	{SG76030, &UA2933482},
	{SG76041, &UA20096},
	{SG76042, &UA2279086},
	{SG76046, &UA20726},
	{SG76047, &UA26963},
	{SG76088, &UA63903},
	{SG76092, &UA24143},
	{SG76106, &UA17037},
	{SG76107, &UA17040},
	{SG76108, &UA2210578},
	{SG76130, &UA53916},
	{SG76151, &UA49514},
	{SG76167, &UA18864},
	{SG76168, &UA18871},
	{SG76170, &UA18876},
	{SG76171, &UA39304},
	{SG76172, &UA1879783},
	{SG76180, &UA24387},
	{SG76200, &UA20143},
	{SG76201, &UA20151},
	{SG76202, &UA20152},
	{SG76203, &UA22624},
	{SG76204, &UA30972},
	{SG76205, &UA32856},
	{SG76206, &UA50633},
	{SG76213, &UA64041},
	{SG76238, &UA5565},
	{SG76239, &UA5566},
	{SG76240, &UA5567},
	{SG76241, &UA5573},
	{SG76242, &UA29729},
	{SG76243, &UA29727},
	{SG76244, &UA29731},
	{SG76245, &UA29733},
	{SG76246, &UA29738},
	{SG76247, &UA29742},
	{SG76248, &UA46369},
	{SG76249, &UA54268},
	{SG76250, &UA58081},
	{SG76251, &UA74338},
	{SG76256, &UA64067},
	{SG76274, &UA706093},
	{SG76275, &UA2274027},
	{SG76276, &UA2306883},
	{SG76286, &UA94},
	{SG76287, &UA32158},
	{SG76293, &UA24404},
	{SG76295, &UA23435},
	{SG76296, &UA24007},
	{SG76297, &UA87640},
	{SG76300, &UA94200},
	{SG76309, &UA1903373},
	{SG76311, &UA2499315},
	{SG76315, &UA2431},
	{SG76316, &UA489823},
	{SG76317, &UA2891},
	{SG76318, &UA2893},
	{SG76319, &UA2894},
	{SG76320, &UA22874},
	{SG76321, &UA22876},
	{SG76322, &UA22877},
	{SG76323, &UA22878},
	{SG76324, &UA22880},
	{SG76325, &UA36222},
	{SG76326, &UA44650},
	{SG76333, &UA48936},
	{SG76364, &UA54448},
	{SG76374, &UA62940},
	{SG76375, &UA36762},
	{SG76376, &UA49103},
	{SG76413, &UA23514},
	{SG76414, &UA60975},
	{SG76416, &UA145041},
	{SG76440, &UA18066},
	{SG76441, &UA67379},
	{SG76447, &UA13052},
	{SG76458, &UA20032},
	{SG76470, &UA28485},
	{SG76479, &UA62894},
	{SG76480, &UA5092},
	{SG76509, &UA13678},
	{SG76510, &UA13681},
	{SG76511, &UA13679},
	{SG76512, &UA66723},
	{SG76513, &UA66724},
	{SG76514, &UA66730},
	{SG76516, &UA70133},
	{SG76517, &UA443224},
	{SG76520, &UA50299},
	{SG76521, &UA87751},
	{SG76530, &UA61758},
	{SG76547, &UA401},
	{SG76548, &UA62470},
	{SG76555, &UA62673},
	{SG76556, &UA54310},
	{SG76563, &UA67901},
	{SG76572, &UA64029},
	{SG76579, &UA16693},
	{SG76597, &UA21934},
	{SG76598, &UA21935},
	{SG76599, &UA21936},
	{SG76600, &UA21937},
	{SG76601, &UA63187},
	{SG76609, &UA2895},
	{SG76610, &UA22875},
	{SG76627, &UA5829},
	{SG76628, &UA30365},
	{SG76640, &UA54406},
	{SG76641, &UA54407},
	{SG76642, &UA55326},
	{SG76643, &UA61841},
	{SG76650, &UA23900},
	{SG76656, &UA20301},
	{SG76662, &UA50132},
	{SG76681, &UA16768},
	{SG76682, &UA51107},
	{SG76685, &UA51093},
	{SG76700, &UA20876},
	{SG76701, &UA87729},
	{SG76713, &UA31496},
	{SG76727, &UA62571},
	{SG76739, &UA11598},
	{SG76740, &UA15611},
	{SG76742, &UA31632},
	{SG76743, &UA54117},
	{SG76744, &UA61673},
	{SG76745, &UA64178},
	{SG76746, &UA25427},
	{SG76747, &UA67343},
	{SG76751, &UA26463},
	{SG76752, &UA88008},
	{SG76753, &UA56316},
	{SG76754, &UA55000},
	{SG76761, &UA61980},
	{SG76762, &UA533653},
	{SG76771, &UA24560},
	{SG76772, &UA50103},
	{SG76773, &UA50121},
	{SG76774, &UA50133},
	{SG76775, &UA50185},
	{SG76776, &UA50188},
	{SG76777, &UA82718},
	{SG76790, &UA26143},
	{SG76791, &UA26146},
	{SG76792, &UA41273},
	{SG76793, &UA51290},
	{SG76797, &UA13961},
	{SG76798, &UA13975},
	{SG76799, &UA13977},
	{SG76800, &UA13981},
	{SG76801, &UA29423},
	{SG76802, &UA29471},
	{SG76805, &UA20115},
	{SG76815, &UA54659},
	{SG76817, &UA58490},
	{SG76818, &UA15607},
	{SG76819, &UA82945},
	{SG76820, &UA2263998},
	{SG76821, &UA9296},
	{SG76822, &UA54635},
	{SG76823, &UA21527},
	{SG76824, &UA24275},
	{SG76825, &UA33069},
	{SG76827, &UA21633},
	{SG76828, &UA62912},
	{SG76829, &UA63426},
	{SG76830, &UA43778},
	{SG76831, &UA2808029},
	{SG76832, &UA53685},
	{SG76833, &UA82915},
	{SG76834, &UA19964},
	{SG76840, &UA61426},
	{SG76841, &UA66989},
	{SG76842, &UA67105},
	{SG76843, &UA82445},
	{SG76844, &UA400},
	{SG76845, &UA3642},
	{SG76846, &UA48028},
	{SG76847, &UA66731},
	{SG76848, &UA2265515},
	{SG76849, &UA24747},
	{SG76850, &UA68564},
	{SG76852, &UA21896},
	{SG76853, &UA23739},
	{SG76855, &UA88201},
	{SG76859, &UA54598},
	{SG76860, &UA24096},
	{SG76861, &UA63389},
	{SG76862, &UA33466},
	{SG76863, &UA5992},
	{SG76864, &UA15390},
	{SG76865, &UA86819},
	{SG76866, &UA21721},
	{SG76867, &UA56588},
	{SG76868, &UA87302},
	{SG76869, &UA63876},
	{SG76870, &UA82964},
	{SG76873, &UA33102},
	{SG76874, &UA63934},
	{SG76876, &UA21112},
	{SG76877, &UA21113},
	{SG76878, &UA23170},
	{SG76879, &UA32894},
	{SG76880, &UA60993},
	{SG76881, &UA16931},
	{SG76882, &UA20666},
	{SG76883, &UA44649},
	{SG76884, &UA59336},
	{SG76885, &UA62461},
	{SG76886, &UA67537},
	{SG76887, &UA67549},
	{SG76888, &UA68561},
	{SG76889, &UA63402},
	{SG76890, &UA72922},
	{SG76891, &UA43853},
	{SG76892, &UA64184},
	{SG76893, &UA14421},
	{SG76894, &UA56109},
	{SG76895, &UA60986},
	{SG76897, &UA14552},
	{SG76898, &UA18815},
	{SG76899, &UA22219},
	{SG76900, &UA22685},
	{SG76901, &UA52777},
	{SG76902, &UA60970},
	{SG76903, &UA61078},
	{SG76904, &UA68588},
	{SG76905, &UA2892},
	{SG76906, &UA7792},
	{SG76907, &UA13674},
	{SG76908, &UA20234},
	{SG76909, &UA27715},
	{SG76910, &UA27880},
	{SG76911, &UA35278},
	{SG76912, &UA37318},
	{SG76913, &UA41542},
	{SG76914, &UA61076},
	{SG76916, &UA82626},
	{SG76917, &UA87309},
	{NULL}};
const Device *DNec[] = {
	&UA87905,
	&UA52826,
	&UA62139,
	&UA84237,
	&UA32754,
	&UA84258,
	&UA84255,
	NULL};
const Segmentdata DMSIEDesktop[] = {
	{SG71087, &UA4906},
	{SG71088, &UA92420},
	{SG71089, &UA92690},
	{SG71090, &UA92847},
	{SG71091, &UA93274},
	{SG71092, &UA95104},
	{SG71093, &UA97807},
	{SG71094, &UA98603},
	{SG71095, &UA103155},
	{SG71096, &UA107322},
	{SG71097, &UA115943},
	{SG71098, &UA116741},
	{SG71099, &UA117540},
	{SG71100, &UA117804},
	{SG71101, &UA119428},
	{SG71102, &UA120233},
	{SG71103, &UA123349},
	{SG71104, &UA127556},
	{SG71105, &UA132734},
	{SG71106, &UA133651},
	{SG71107, &UA143535},
	{SG71108, &UA144773},
	{SG71109, &UA145237},
	{SG71110, &UA146840},
	{SG71111, &UA156763},
	{SG71112, &UA171022},
	{SG71113, &UA178738},
	{SG71114, &UA179035},
	{SG71115, &UA273722},
	{SG71116, &UA274801},
	{SG71117, &UA274832},
	{SG71118, &UA277301},
	{SG71119, &UA293754},
	{SG71120, &UA313891},
	{SG71121, &UA366362},
	{SG71122, &UA373633},
	{SG71123, &UA374223},
	{SG71124, &UA389765},
	{SG71125, &UA397246},
	{SG71126, &UA443398},
	{SG71127, &UA483468},
	{SG71128, &UA495468},
	{SG71129, &UA506803},
	{SG71130, &UA513422},
	{SG71131, &UA513961},
	{SG71132, &UA520209},
	{SG71133, &UA520299},
	{SG71134, &UA541229},
	{SG71135, &UA570709},
	{SG71136, &UA580386},
	{SG71137, &UA625342},
	{SG71138, &UA633015},
	{SG71139, &UA659281},
	{SG71140, &UA661463},
	{SG71141, &UA677248},
	{SG71142, &UA679270},
	{SG71143, &UA686117},
	{SG71144, &UA688512},
	{SG71145, &UA708232},
	{SG71146, &UA731932},
	{SG71147, &UA735607},
	{SG71148, &UA750947},
	{SG71149, &UA751159},
	{SG71150, &UA814067},
	{SG71151, &UA845981},
	{SG71152, &UA900450},
	{SG71153, &UA1836615},
	{SG71154, &UA1838593},
	{SG71155, &UA1858388},
	{SG71156, &UA1878949},
	{SG71157, &UA1881636},
	{SG71158, &UA1896182},
	{SG71159, &UA1902306},
	{SG71160, &UA1919824},
	{SG71161, &UA1954287},
	{SG71162, &UA1966347},
	{SG71163, &UA2039609},
	{SG71164, &UA2127968},
	{SG71165, &UA2137138},
	{SG71166, &UA2149081},
	{SG71167, &UA2155092},
	{SG71168, &UA2155609},
	{SG71169, &UA2160015},
	{SG71170, &UA2166252},
	{SG71171, &UA2166285},
	{SG71172, &UA2184804},
	{SG71173, &UA2203704},
	{SG71174, &UA2203708},
	{SG71175, &UA2208127},
	{SG71176, &UA2216613},
	{SG71177, &UA2219555},
	{SG71178, &UA2271546},
	{SG71179, &UA2273249},
	{SG71180, &UA2285771},
	{SG71181, &UA2312863},
	{SG71182, &UA2332358},
	{SG71183, &UA2336772},
	{SG71184, &UA2337432},
	{SG71185, &UA2339963},
	{SG71186, &UA2349577},
	{SG71187, &UA2392203},
	{SG71188, &UA2394241},
	{SG71189, &UA2400799},
	{SG71190, &UA2412943},
	{SG71191, &UA2423679},
	{SG71192, &UA2429500},
	{SG71193, &UA2431047},
	{SG71194, &UA2433554},
	{SG71195, &UA2439105},
	{SG71196, &UA2451812},
	{SG71197, &UA2486924},
	{SG71198, &UA2495472},
	{SG71199, &UA2510422},
	{SG71200, &UA2511222},
	{SG71201, &UA2517723},
	{SG71202, &UA2521007},
	{SG71203, &UA2545747},
	{SG71204, &UA2574078},
	{SG71205, &UA2585994},
	{SG71206, &UA2590173},
	{SG71207, &UA2605645},
	{SG71208, &UA2607155},
	{SG71209, &UA2650755},
	{SG71210, &UA2687648},
	{SG71211, &UA2691336},
	{SG71212, &UA2725834},
	{SG71213, &UA2726514},
	{SG71214, &UA2742811},
	{SG71215, &UA2743900},
	{SG71216, &UA2747795},
	{SG71217, &UA2789361},
	{SG71218, &UA2807615},
	{SG71219, &UA2807807},
	{SG71220, &UA2811856},
	{SG71221, &UA2844167},
	{SG71222, &UA2933643},
	{SG71223, &UA2933758},
	{SG71224, &UA2949948},
	{SG71225, &UA2956723},
	{SG71226, &UA2979008},
	{SG71227, &UA3045864},
	{SG71228, &UA3069094},
	{SG71229, &UA3108754},
	{SG71230, &UA10374},
	{SG71231, &UA25414},
	{SG71232, &UA92180},
	{SG71233, &UA95262},
	{SG71234, &UA98147},
	{SG71235, &UA99449},
	{SG71236, &UA99995},
	{SG71237, &UA101313},
	{SG71238, &UA102209},
	{SG71239, &UA102858},
	{SG71240, &UA107328},
	{SG71241, &UA109124},
	{SG71242, &UA112802},
	{SG71243, &UA119365},
	{SG71244, &UA122110},
	{SG71245, &UA122694},
	{SG71246, &UA125318},
	{SG71247, &UA138860},
	{SG71248, &UA140523},
	{SG71249, &UA140582},
	{SG71250, &UA142735},
	{SG71251, &UA143139},
	{SG71252, &UA146120},
	{SG71253, &UA146665},
	{SG71254, &UA148896},
	{SG71255, &UA150456},
	{SG71256, &UA151315},
	{SG71257, &UA151399},
	{SG71258, &UA151975},
	{SG71259, &UA153566},
	{SG71260, &UA160901},
	{SG71261, &UA163389},
	{SG71262, &UA168799},
	{SG71263, &UA176650},
	{SG71264, &UA179266},
	{SG71265, &UA183398},
	{SG71266, &UA271076},
	{SG71267, &UA282787},
	{SG71268, &UA286859},
	{SG71269, &UA302232},
	{SG71270, &UA305256},
	{SG71271, &UA339913},
	{SG71272, &UA373592},
	{SG71273, &UA375051},
	{SG71274, &UA377752},
	{SG71275, &UA378688},
	{SG71276, &UA491199},
	{SG71277, &UA492877},
	{SG71278, &UA514309},
	{SG71279, &UA517247},
	{SG71280, &UA518769},
	{SG71281, &UA541230},
	{SG71282, &UA541231},
	{SG71283, &UA544673},
	{SG71284, &UA559141},
	{SG71285, &UA568242},
	{SG71286, &UA570850},
	{SG71287, &UA589282},
	{SG71288, &UA682853},
	{SG71289, &UA694634},
	{SG71290, &UA707289},
	{SG71291, &UA716436},
	{SG71292, &UA717921},
	{SG71293, &UA719209},
	{SG71294, &UA720141},
	{SG71295, &UA734891},
	{SG71296, &UA738833},
	{SG71297, &UA740020},
	{SG71298, &UA751972},
	{SG71299, &UA754903},
	{SG71300, &UA757943},
	{SG71301, &UA761239},
	{SG71302, &UA762255},
	{SG71303, &UA786231},
	{SG71304, &UA816342},
	{SG71305, &UA819532},
	{SG71306, &UA845513},
	{SG71307, &UA857357},
	{SG71308, &UA879981},
	{SG71309, &UA896058},
	{SG71310, &UA1873068},
	{SG71311, &UA1886576},
	{SG71312, &UA1891040},
	{SG71313, &UA1935419},
	{SG71314, &UA1966079},
	{SG71315, &UA1972514},
	{SG71316, &UA1986353},
	{SG71317, &UA1991124},
	{SG71318, &UA2138907},
	{SG71319, &UA2145826},
	{SG71320, &UA2150709},
	{SG71321, &UA2154239},
	{SG71322, &UA2156385},
	{SG71323, &UA2168954},
	{SG71324, &UA2221265},
	{SG71325, &UA2221314},
	{SG71326, &UA2254284},
	{SG71327, &UA2256254},
	{SG71328, &UA2258748},
	{SG71329, &UA2267607},
	{SG71330, &UA2272483},
	{SG71331, &UA2272848},
	{SG71332, &UA2286215},
	{SG71333, &UA2298089},
	{SG71334, &UA2319006},
	{SG71335, &UA2319039},
	{SG71336, &UA2329329},
	{SG71337, &UA2361161},
	{SG71338, &UA2393206},
	{SG71339, &UA2397255},
	{SG71340, &UA2397579},
	{SG71341, &UA2409216},
	{SG71342, &UA2412933},
	{SG71343, &UA2413162},
	{SG71344, &UA2430280},
	{SG71345, &UA2431907},
	{SG71346, &UA2438283},
	{SG71347, &UA2478059},
	{SG71348, &UA2493090},
	{SG71349, &UA2493485},
	{SG71350, &UA2494509},
	{SG71351, &UA2510873},
	{SG71352, &UA2536396},
	{SG71353, &UA2569839},
	{SG71354, &UA2574681},
	{SG71355, &UA2576249},
	{SG71356, &UA2587398},
	{SG71357, &UA2588188},
	{SG71358, &UA2660022},
	{SG71359, &UA2668027},
	{SG71360, &UA2668315},
	{SG71361, &UA2681304},
	{SG71362, &UA2686060},
	{SG71363, &UA2695234},
	{SG71364, &UA2726469},
	{SG71365, &UA2742950},
	{SG71366, &UA2774783},
	{SG71367, &UA2775864},
	{SG71368, &UA2776079},
	{SG71369, &UA2777041},
	{SG71370, &UA2779092},
	{SG71371, &UA2780750},
	{SG71372, &UA2808560},
	{SG71373, &UA2810515},
	{SG71374, &UA2812874},
	{SG71375, &UA2814427},
	{SG71376, &UA2851256},
	{SG71377, &UA2857543},
	{SG71378, &UA2861257},
	{SG71379, &UA2948576},
	{SG71380, &UA2949515},
	{SG71381, &UA2949963},
	{SG71382, &UA2950126},
	{SG71383, &UA2953209},
	{SG71384, &UA2980060},
	{SG71385, &UA2980376},
	{SG71386, &UA2982015},
	{SG71387, &UA2982454},
	{SG71388, &UA3010245},
	{SG71389, &UA3024343},
	{SG71390, &UA3041537},
	{SG71391, &UA3117790},
	{SG71392, &UA3120808},
	{SG71393, &UA93983},
	{SG71394, &UA95167},
	{SG71395, &UA97815},
	{SG71396, &UA98343},
	{SG71397, &UA98488},
	{SG71398, &UA98989},
	{SG71399, &UA99617},
	{SG71400, &UA100368},
	{SG71401, &UA101044},
	{SG71402, &UA101168},
	{SG71403, &UA103708},
	{SG71404, &UA105219},
	{SG71405, &UA109614},
	{SG71406, &UA113083},
	{SG71407, &UA124206},
	{SG71408, &UA125466},
	{SG71409, &UA127296},
	{SG71410, &UA128492},
	{SG71411, &UA131546},
	{SG71412, &UA133535},
	{SG71413, &UA135199},
	{SG71414, &UA136084},
	{SG71415, &UA139007},
	{SG71416, &UA149706},
	{SG71417, &UA152117},
	{SG71418, &UA155286},
	{SG71419, &UA157922},
	{SG71420, &UA165206},
	{SG71421, &UA167648},
	{SG71422, &UA170367},
	{SG71423, &UA171346},
	{SG71424, &UA183396},
	{SG71425, &UA240430},
	{SG71426, &UA258193},
	{SG71427, &UA264937},
	{SG71428, &UA296450},
	{SG71429, &UA300493},
	{SG71430, &UA301384},
	{SG71431, &UA376752},
	{SG71432, &UA393750},
	{SG71433, &UA399560},
	{SG71434, &UA417745},
	{SG71435, &UA439152},
	{SG71436, &UA442217},
	{SG71437, &UA443070},
	{SG71438, &UA449697},
	{SG71439, &UA467889},
	{SG71440, &UA469698},
	{SG71441, &UA473496},
	{SG71442, &UA473568},
	{SG71443, &UA482516},
	{SG71444, &UA506875},
	{SG71445, &UA508074},
	{SG71446, &UA515199},
	{SG71447, &UA529861},
	{SG71448, &UA541232},
	{SG71449, &UA558562},
	{SG71450, &UA586293},
	{SG71451, &UA641029},
	{SG71452, &UA643815},
	{SG71453, &UA651966},
	{SG71454, &UA685467},
	{SG71455, &UA689305},
	{SG71456, &UA690908},
	{SG71457, &UA704738},
	{SG71458, &UA715802},
	{SG71459, &UA731306},
	{SG71460, &UA739596},
	{SG71461, &UA742520},
	{SG71462, &UA767650},
	{SG71463, &UA771639},
	{SG71464, &UA783580},
	{SG71465, &UA798168},
	{SG71466, &UA806170},
	{SG71467, &UA808246},
	{SG71468, &UA816128},
	{SG71469, &UA826162},
	{SG71470, &UA889881},
	{SG71471, &UA902191},
	{SG71472, &UA1837211},
	{SG71473, &UA1841811},
	{SG71474, &UA1856613},
	{SG71475, &UA1887100},
	{SG71476, &UA1890791},
	{SG71477, &UA1949297},
	{SG71478, &UA1952675},
	{SG71479, &UA2127683},
	{SG71480, &UA2155776},
	{SG71481, &UA2166719},
	{SG71482, &UA2207749},
	{SG71483, &UA2211988},
	{SG71484, &UA2244751},
	{SG71485, &UA2314616},
	{SG71486, &UA2318144},
	{SG71487, &UA2356962},
	{SG71488, &UA2371911},
	{SG71489, &UA2392600},
	{SG71490, &UA2412126},
	{SG71491, &UA2419011},
	{SG71492, &UA2433303},
	{SG71493, &UA2462628},
	{SG71494, &UA2502469},
	{SG71495, &UA2514708},
	{SG71496, &UA2517511},
	{SG71497, &UA2540711},
	{SG71498, &UA2597388},
	{SG71499, &UA2667944},
	{SG71500, &UA2706105},
	{SG71501, &UA2720311},
	{SG71502, &UA2754805},
	{SG71503, &UA2801475},
	{SG71504, &UA2804382},
	{SG71505, &UA2817913},
	{SG71506, &UA2832344},
	{SG71507, &UA2845570},
	{SG71508, &UA2883682},
	{SG71509, &UA2899266},
	{SG71510, &UA2932990},
	{SG71511, &UA2973607},
	{SG71512, &UA2982600},
	{SG71513, &UA3041785},
	{SG71514, &UA3075125},
	{SG71515, &UA3116182},
	{SG71516, &UA129075},
	{SG71517, &UA135093},
	{SG71518, &UA136936},
	{SG71519, &UA137264},
	{SG71520, &UA137804},
	{SG71521, &UA138807},
	{SG71522, &UA141321},
	{SG71523, &UA142528},
	{SG71524, &UA145276},
	{SG71525, &UA157126},
	{SG71526, &UA160570},
	{SG71527, &UA161525},
	{SG71528, &UA163106},
	{SG71529, &UA164518},
	{SG71530, &UA166734},
	{SG71531, &UA170037},
	{SG71532, &UA170398},
	{SG71533, &UA174309},
	{SG71534, &UA177649},
	{SG71535, &UA189337},
	{SG71536, &UA204448},
	{SG71537, &UA212865},
	{SG71538, &UA216690},
	{SG71539, &UA221306},
	{SG71540, &UA275362},
	{SG71541, &UA284501},
	{SG71542, &UA308155},
	{SG71543, &UA379388},
	{SG71544, &UA395165},
	{SG71545, &UA420362},
	{SG71546, &UA433969},
	{SG71547, &UA445010},
	{SG71548, &UA452172},
	{SG71549, &UA483746},
	{SG71550, &UA490991},
	{SG71551, &UA515396},
	{SG71552, &UA515753},
	{SG71553, &UA517125},
	{SG71554, &UA518841},
	{SG71555, &UA524023},
	{SG71556, &UA526642},
	{SG71557, &UA530584},
	{SG71558, &UA542917},
	{SG71559, &UA568029},
	{SG71560, &UA604996},
	{SG71561, &UA684609},
	{SG71562, &UA688331},
	{SG71563, &UA695019},
	{SG71564, &UA702664},
	{SG71565, &UA736054},
	{SG71566, &UA791393},
	{SG71567, &UA793651},
	{SG71568, &UA803508},
	{SG71569, &UA828259},
	{SG71570, &UA831805},
	{SG71571, &UA852809},
	{SG71572, &UA855053},
	{SG71573, &UA855420},
	{SG71574, &UA872197},
	{SG71575, &UA894308},
	{SG71576, &UA1845547},
	{SG71577, &UA1853520},
	{SG71578, &UA1894156},
	{SG71579, &UA1924416},
	{SG71580, &UA1934284},
	{SG71581, &UA1942246},
	{SG71582, &UA2006200},
	{SG71583, &UA2144969},
	{SG71584, &UA2153659},
	{SG71585, &UA2156196},
	{SG71586, &UA2156482},
	{SG71587, &UA2160335},
	{SG71588, &UA2176415},
	{SG71589, &UA2209114},
	{SG71590, &UA2220089},
	{SG71591, &UA2230879},
	{SG71592, &UA2245574},
	{SG71593, &UA2268807},
	{SG71594, &UA2296989},
	{SG71595, &UA2321455},
	{SG71596, &UA2336289},
	{SG71597, &UA2355197},
	{SG71598, &UA2393486},
	{SG71599, &UA2420607},
	{SG71600, &UA2438329},
	{SG71601, &UA2495700},
	{SG71602, &UA2536400},
	{SG71603, &UA2542226},
	{SG71604, &UA2553105},
	{SG71605, &UA2553571},
	{SG71606, &UA2572766},
	{SG71607, &UA2580740},
	{SG71608, &UA2639384},
	{SG71609, &UA2640429},
	{SG71610, &UA2690832},
	{SG71611, &UA2722124},
	{SG71612, &UA2808204},
	{SG71613, &UA2934120},
	{SG71614, &UA92827},
	{SG71615, &UA95081},
	{SG71616, &UA98143},
	{SG71617, &UA98337},
	{SG71618, &UA106496},
	{SG71619, &UA112709},
	{SG71620, &UA115269},
	{SG71621, &UA123821},
	{SG71622, &UA126602},
	{SG71623, &UA129453},
	{SG71624, &UA136704},
	{SG71625, &UA140108},
	{SG71626, &UA145509},
	{SG71627, &UA146611},
	{SG71628, &UA151873},
	{SG71629, &UA158258},
	{SG71630, &UA165052},
	{SG71631, &UA210409},
	{SG71632, &UA239851},
	{SG71633, &UA251307},
	{SG71634, &UA263147},
	{SG71635, &UA278226},
	{SG71636, &UA299301},
	{SG71637, &UA334474},
	{SG71638, &UA358133},
	{SG71639, &UA397056},
	{SG71640, &UA414981},
	{SG71641, &UA422355},
	{SG71642, &UA493457},
	{SG71643, &UA515827},
	{SG71644, &UA533894},
	{SG71645, &UA568887},
	{SG71646, &UA569828},
	{SG71647, &UA605303},
	{SG71648, &UA623394},
	{SG71649, &UA628180},
	{SG71650, &UA632842},
	{SG71651, &UA651095},
	{SG71652, &UA714161},
	{SG71653, &UA726454},
	{SG71654, &UA777573},
	{SG71655, &UA832411},
	{SG71656, &UA864659},
	{SG71657, &UA898518},
	{SG71658, &UA903652},
	{SG71659, &UA1955163},
	{SG71660, &UA2000272},
	{SG71661, &UA2003621},
	{SG71662, &UA2004541},
	{SG71663, &UA2165599},
	{SG71664, &UA2181639},
	{SG71665, &UA2183504},
	{SG71666, &UA2194241},
	{SG71667, &UA2196507},
	{SG71668, &UA2197325},
	{SG71669, &UA2198534},
	{SG71670, &UA2200269},
	{SG71671, &UA2204237},
	{SG71672, &UA2223859},
	{SG71673, &UA2254190},
	{SG71674, &UA2273044},
	{SG71675, &UA2283996},
	{SG71676, &UA2311923},
	{SG71677, &UA2370085},
	{SG71678, &UA2400092},
	{SG71679, &UA2430071},
	{SG71680, &UA2436217},
	{SG71681, &UA2439179},
	{SG71682, &UA2440366},
	{SG71683, &UA2454118},
	{SG71684, &UA2462556},
	{SG71685, &UA2486831},
	{SG71686, &UA2501917},
	{SG71687, &UA2516652},
	{SG71688, &UA2569824},
	{SG71689, &UA2676892},
	{SG71690, &UA2689487},
	{SG71691, &UA2693809},
	{SG71692, &UA2696574},
	{SG71693, &UA2697423},
	{SG71694, &UA2698193},
	{SG71695, &UA2728698},
	{SG71696, &UA2741539},
	{SG71697, &UA2760844},
	{SG71698, &UA2817776},
	{SG71699, &UA2870752},
	{SG71700, &UA2941526},
	{SG71701, &UA2948342},
	{SG71702, &UA2982163},
	{SG71703, &UA2984255},
	{SG71704, &UA3031465},
	{SG71705, &UA3073498},
	{SG71706, &UA3084957},
	{SG71707, &UA3095399},
	{SG71708, &UA3098616},
	{SG71709, &UA3124585},
	{SG71710, &UA51271},
	{SG71711, &UA93505},
	{SG71712, &UA94047},
	{SG71713, &UA95385},
	{SG71714, &UA97973},
	{SG71715, &UA98483},
	{SG71716, &UA98562},
	{SG71717, &UA103659},
	{SG71718, &UA108252},
	{SG71719, &UA116536},
	{SG71720, &UA122629},
	{SG71721, &UA130630},
	{SG71722, &UA130968},
	{SG71723, &UA131031},
	{SG71724, &UA138617},
	{SG71725, &UA142690},
	{SG71726, &UA160674},
	{SG71727, &UA164421},
	{SG71728, &UA166062},
	{SG71729, &UA167889},
	{SG71730, &UA172141},
	{SG71731, &UA178083},
	{SG71732, &UA185731},
	{SG71733, &UA186301},
	{SG71734, &UA188272},
	{SG71735, &UA230117},
	{SG71736, &UA282848},
	{SG71737, &UA332680},
	{SG71738, &UA371480},
	{SG71739, &UA394304},
	{SG71740, &UA409339},
	{SG71741, &UA443216},
	{SG71742, &UA509692},
	{SG71743, &UA514162},
	{SG71744, &UA543691},
	{SG71745, &UA551420},
	{SG71746, &UA703641},
	{SG71747, &UA708734},
	{SG71748, &UA748912},
	{SG71749, &UA765016},
	{SG71750, &UA778222},
	{SG71751, &UA803868},
	{SG71752, &UA863717},
	{SG71753, &UA881363},
	{SG71754, &UA891817},
	{SG71755, &UA1850588},
	{SG71756, &UA1877576},
	{SG71757, &UA1895354},
	{SG71758, &UA1896858},
	{SG71759, &UA1954746},
	{SG71760, &UA1959324},
	{SG71761, &UA1983667},
	{SG71762, &UA1999434},
	{SG71763, &UA2006022},
	{SG71764, &UA2146182},
	{SG71765, &UA2170009},
	{SG71766, &UA2195298},
	{SG71767, &UA2203761},
	{SG71768, &UA2212466},
	{SG71769, &UA2216772},
	{SG71770, &UA2236447},
	{SG71771, &UA2245146},
	{SG71772, &UA2246387},
	{SG71773, &UA2263320},
	{SG71774, &UA2354101},
	{SG71775, &UA2412119},
	{SG71776, &UA2414514},
	{SG71777, &UA2436568},
	{SG71778, &UA2463399},
	{SG71779, &UA2467821},
	{SG71780, &UA2468589},
	{SG71781, &UA2472752},
	{SG71782, &UA2483469},
	{SG71783, &UA2495694},
	{SG71784, &UA2502466},
	{SG71785, &UA2522709},
	{SG71786, &UA2639789},
	{SG71787, &UA2808255},
	{SG71788, &UA2816335},
	{SG71789, &UA2851852},
	{SG71790, &UA99994},
	{SG71791, &UA119808},
	{SG71792, &UA135357},
	{SG71793, &UA136697},
	{SG71794, &UA137797},
	{SG71795, &UA140808},
	{SG71796, &UA164273},
	{SG71797, &UA164302},
	{SG71798, &UA164648},
	{SG71799, &UA165354},
	{SG71800, &UA166344},
	{SG71801, &UA173206},
	{SG71802, &UA175536},
	{SG71803, &UA179544},
	{SG71804, &UA183894},
	{SG71805, &UA191734},
	{SG71806, &UA310320},
	{SG71807, &UA310831},
	{SG71808, &UA421600},
	{SG71809, &UA434884},
	{SG71810, &UA436651},
	{SG71811, &UA505399},
	{SG71812, &UA541172},
	{SG71813, &UA544003},
	{SG71814, &UA544942},
	{SG71815, &UA559428},
	{SG71816, &UA654509},
	{SG71817, &UA658171},
	{SG71818, &UA667757},
	{SG71819, &UA671704},
	{SG71820, &UA683940},
	{SG71821, &UA685566},
	{SG71822, &UA686205},
	{SG71823, &UA693689},
	{SG71824, &UA695259},
	{SG71825, &UA736381},
	{SG71826, &UA748070},
	{SG71827, &UA761094},
	{SG71828, &UA762161},
	{SG71829, &UA809093},
	{SG71830, &UA810737},
	{SG71831, &UA816716},
	{SG71832, &UA892386},
	{SG71833, &UA1860543},
	{SG71834, &UA1890788},
	{SG71835, &UA1903741},
	{SG71836, &UA1908397},
	{SG71837, &UA1940123},
	{SG71838, &UA1969946},
	{SG71839, &UA2143094},
	{SG71840, &UA2144318},
	{SG71841, &UA2146912},
	{SG71842, &UA2155707},
	{SG71843, &UA2161649},
	{SG71844, &UA2210304},
	{SG71845, &UA2210868},
	{SG71846, &UA2212552},
	{SG71847, &UA2259357},
	{SG71848, &UA2302083},
	{SG71849, &UA2304484},
	{SG71850, &UA2309322},
	{SG71851, &UA2314595},
	{SG71852, &UA2367341},
	{SG71853, &UA2368708},
	{SG71854, &UA2409404},
	{SG71855, &UA2429011},
	{SG71856, &UA2432209},
	{SG71857, &UA2442926},
	{SG71858, &UA2444783},
	{SG71859, &UA2445207},
	{SG71860, &UA2458219},
	{SG71861, &UA2490144},
	{SG71862, &UA2490277},
	{SG71863, &UA2507826},
	{SG71864, &UA2507947},
	{SG71865, &UA2537668},
	{SG71866, &UA2554584},
	{SG71867, &UA2554787},
	{SG71868, &UA2567630},
	{SG71869, &UA2582620},
	{SG71870, &UA2585281},
	{SG71871, &UA2585919},
	{SG71872, &UA2594586},
	{SG71873, &UA2639298},
	{SG71874, &UA2644010},
	{SG71875, &UA2649564},
	{SG71876, &UA2654041},
	{SG71877, &UA2666688},
	{SG71878, &UA2669381},
	{SG71879, &UA2669576},
	{SG71880, &UA2675743},
	{SG71881, &UA2677104},
	{SG71882, &UA2679382},
	{SG71883, &UA2684596},
	{SG71884, &UA2695382},
	{SG71885, &UA2728266},
	{SG71886, &UA2740058},
	{SG71887, &UA2740436},
	{SG71888, &UA2752919},
	{SG71889, &UA2772204},
	{SG71890, &UA2773973},
	{SG71891, &UA2773976},
	{SG71892, &UA2794805},
	{SG71893, &UA2796957},
	{SG71894, &UA2806056},
	{SG71895, &UA2806310},
	{SG71896, &UA2839020},
	{SG71897, &UA2870799},
	{SG71898, &UA2871483},
	{SG71899, &UA2872308},
	{SG71900, &UA2879907},
	{SG71901, &UA2889460},
	{SG71902, &UA2946412},
	{SG71903, &UA2947368},
	{SG71904, &UA2967411},
	{SG71905, &UA2979504},
	{SG71906, &UA2980907},
	{SG71907, &UA2981755},
	{SG71908, &UA2988625},
	{SG71909, &UA2994395},
	{SG71910, &UA2995647},
	{SG71911, &UA3022473},
	{SG71912, &UA3024049},
	{SG71913, &UA3033969},
	{SG71914, &UA3034117},
	{SG71915, &UA3041750},
	{SG71916, &UA3043280},
	{SG71917, &UA3050898},
	{SG71918, &UA3061048},
	{SG71919, &UA3062635},
	{SG71920, &UA3069535},
	{SG71921, &UA3083323},
	{SG71922, &UA3083563},
	{SG71923, &UA3098588},
	{SG71924, &UA3110477},
	{SG71925, &UA3125156},
	{SG71926, &UA3128213},
	{SG71927, &UA8113},
	{SG71928, &UA54333},
	{SG71929, &UA55087},
	{SG71930, &UA91801},
	{SG71931, &UA92833},
	{SG71932, &UA93076},
	{SG71933, &UA93087},
	{SG71934, &UA93292},
	{SG71935, &UA95099},
	{SG71936, &UA97395},
	{SG71937, &UA98132},
	{SG71938, &UA98602},
	{SG71939, &UA99125},
	{SG71940, &UA99210},
	{SG71941, &UA99230},
	{SG71942, &UA99822},
	{SG71943, &UA100822},
	{SG71944, &UA101107},
	{SG71945, &UA103999},
	{SG71946, &UA104938},
	{SG71947, &UA115577},
	{SG71948, &UA115936},
	{SG71949, &UA123993},
	{SG71950, &UA129738},
	{SG71951, &UA130541},
	{SG71952, &UA130577},
	{SG71953, &UA130973},
	{SG71954, &UA132838},
	{SG71955, &UA134652},
	{SG71956, &UA142411},
	{SG71957, &UA145173},
	{SG71958, &UA151600},
	{SG71959, &UA153610},
	{SG71960, &UA156314},
	{SG71961, &UA159738},
	{SG71962, &UA170484},
	{SG71963, &UA176559},
	{SG71964, &UA236546},
	{SG71965, &UA259789},
	{SG71966, &UA260496},
	{SG71967, &UA261354},
	{SG71968, &UA268800},
	{SG71969, &UA272428},
	{SG71970, &UA299436},
	{SG71971, &UA373159},
	{SG71972, &UA421962},
	{SG71973, &UA470901},
	{SG71974, &UA480356},
	{SG71975, &UA490095},
	{SG71976, &UA506125},
	{SG71977, &UA553364},
	{SG71978, &UA559025},
	{SG71979, &UA584844},
	{SG71980, &UA590314},
	{SG71981, &UA639754},
	{SG71982, &UA649156},
	{SG71983, &UA656859},
	{SG71984, &UA665169},
	{SG71985, &UA667965},
	{SG71986, &UA668552},
	{SG71987, &UA680888},
	{SG71988, &UA688948},
	{SG71989, &UA693326},
	{SG71990, &UA717723},
	{SG71991, &UA718354},
	{SG71992, &UA720819},
	{SG71993, &UA737098},
	{SG71994, &UA788572},
	{SG71995, &UA828742},
	{SG71996, &UA869945},
	{SG71997, &UA909865},
	{SG71998, &UA1866875},
	{SG71999, &UA1884923},
	{SG72000, &UA1969691},
	{SG72001, &UA2195915},
	{SG72002, &UA2207787},
	{SG72003, &UA2218077},
	{SG72004, &UA2266954},
	{SG72005, &UA2275660},
	{SG72006, &UA2276221},
	{SG72007, &UA2276781},
	{SG72008, &UA2281161},
	{SG72009, &UA2290985},
	{SG72010, &UA2312861},
	{SG72011, &UA2314138},
	{SG72012, &UA2335519},
	{SG72013, &UA2371414},
	{SG72014, &UA2411837},
	{SG72015, &UA2412395},
	{SG72016, &UA2437775},
	{SG72017, &UA2437944},
	{SG72018, &UA2438188},
	{SG72019, &UA2443572},
	{SG72020, &UA2502467},
	{SG72021, &UA2504904},
	{SG72022, &UA2505433},
	{SG72023, &UA2519594},
	{SG72024, &UA2527252},
	{SG72025, &UA2529761},
	{SG72026, &UA2544580},
	{SG72027, &UA2558144},
	{SG72028, &UA2589542},
	{SG72029, &UA2639827},
	{SG72030, &UA2643612},
	{SG72031, &UA2656626},
	{SG72032, &UA2658986},
	{SG72033, &UA2692055},
	{SG72034, &UA2726187},
	{SG72035, &UA2761370},
	{SG72036, &UA2791714},
	{SG72037, &UA2802324},
	{SG72038, &UA2806615},
	{SG72039, &UA2827719},
	{SG72040, &UA2933224},
	{SG72041, &UA2933997},
	{SG72042, &UA2934422},
	{SG72043, &UA2995859},
	{SG72044, &UA3042102},
	{SG72045, &UA178034},
	{SG72046, &UA179870},
	{SG72047, &UA288788},
	{SG72048, &UA652806},
	{SG72049, &UA663218},
	{SG72050, &UA663344},
	{SG72051, &UA2362718},
	{SG72052, &UA2392444},
	{SG72053, &UA2408907},
	{SG72054, &UA2466239},
	{SG72055, &UA2503176},
	{SG72056, &UA2527597},
	{SG72057, &UA2554814},
	{SG72058, &UA2561421},
	{SG72059, &UA2564411},
	{SG72060, &UA2594573},
	{SG72061, &UA2606770},
	{SG72062, &UA2607082},
	{SG72063, &UA2651848},
	{SG72064, &UA2657100},
	{SG72065, &UA2658642},
	{SG72066, &UA2668288},
	{SG72067, &UA2694777},
	{SG72068, &UA2695435},
	{SG72069, &UA2697734},
	{SG72070, &UA2706555},
	{SG72071, &UA2728098},
	{SG72072, &UA2732677},
	{SG72073, &UA2734133},
	{SG72074, &UA2734348},
	{SG72075, &UA2764101},
	{SG72076, &UA2780578},
	{SG72077, &UA2780638},
	{SG72078, &UA2796531},
	{SG72079, &UA2837654},
	{SG72080, &UA2845604},
	{SG72081, &UA2856291},
	{SG72082, &UA2982735},
	{SG72083, &UA2993033},
	{SG72084, &UA3046958},
	{SG72085, &UA3108115},
	{SG72086, &UA3120483},
	{SG72087, &UA211794},
	{SG72088, &UA217601},
	{SG72089, &UA217804},
	{SG72090, &UA221614},
	{SG72091, &UA223238},
	{SG72092, &UA226949},
	{SG72093, &UA231451},
	{SG72094, &UA233665},
	{SG72095, &UA238866},
	{SG72096, &UA395071},
	{SG72097, &UA419952},
	{SG72098, &UA436864},
	{SG72099, &UA554236},
	{SG72100, &UA559157},
	{SG72101, &UA560619},
	{SG72102, &UA561647},
	{SG72103, &UA573403},
	{SG72104, &UA576122},
	{SG72105, &UA576136},
	{SG72106, &UA603723},
	{SG72107, &UA603733},
	{SG72108, &UA603809},
	{SG72109, &UA603918},
	{SG72110, &UA603934},
	{SG72111, &UA604093},
	{SG72112, &UA604300},
	{SG72113, &UA604647},
	{SG72114, &UA605509},
	{SG72115, &UA607072},
	{SG72116, &UA629361},
	{SG72117, &UA632046},
	{SG72118, &UA638664},
	{SG72119, &UA647089},
	{SG72120, &UA656289},
	{SG72121, &UA668376},
	{SG72122, &UA685015},
	{SG72123, &UA693887},
	{SG72124, &UA707458},
	{SG72125, &UA707976},
	{SG72126, &UA708628},
	{SG72127, &UA708675},
	{SG72128, &UA714371},
	{SG72129, &UA727457},
	{SG72130, &UA747747},
	{SG72131, &UA748997},
	{SG72132, &UA760978},
	{SG72133, &UA783936},
	{SG72134, &UA793338},
	{SG72135, &UA832627},
	{SG72136, &UA844816},
	{SG72137, &UA897371},
	{SG72138, &UA1910647},
	{SG72139, &UA1941706},
	{SG72140, &UA1949819},
	{SG72141, &UA1959416},
	{SG72142, &UA1965486},
	{SG72143, &UA1967986},
	{SG72144, &UA1983157},
	{SG72145, &UA1988751},
	{SG72146, &UA1995179},
	{SG72147, &UA2142485},
	{SG72148, &UA2149119},
	{SG72149, &UA2156654},
	{SG72150, &UA2158959},
	{SG72151, &UA2190943},
	{SG72152, &UA2221036},
	{SG72153, &UA2221914},
	{SG72154, &UA2299234},
	{SG72155, &UA2303763},
	{SG72156, &UA2305563},
	{SG72157, &UA2317230},
	{SG72158, &UA2360447},
	{SG72159, &UA2361411},
	{SG72160, &UA2362067},
	{SG72161, &UA2362469},
	{SG72162, &UA2362646},
	{SG72163, &UA2362846},
	{SG72164, &UA2362887},
	{SG72165, &UA2362924},
	{SG72166, &UA2363143},
	{SG72167, &UA2367668},
	{SG72168, &UA2367867},
	{SG72169, &UA2368351},
	{SG72170, &UA2368352},
	{SG72171, &UA2393749},
	{SG72172, &UA2397065},
	{SG72173, &UA2397124},
	{SG72174, &UA2422474},
	{SG72175, &UA2424672},
	{SG72176, &UA2425756},
	{SG72177, &UA2426089},
	{SG72178, &UA2427559},
	{SG72179, &UA2441855},
	{SG72180, &UA2445299},
	{SG72181, &UA2447792},
	{SG72182, &UA2450934},
	{SG72183, &UA2457138},
	{SG72184, &UA2481840},
	{SG72185, &UA2482342},
	{SG72186, &UA2482529},
	{SG72187, &UA2493652},
	{SG72188, &UA2493804},
	{SG72189, &UA2494983},
	{SG72190, &UA2497260},
	{SG72191, &UA2503853},
	{SG72192, &UA2507959},
	{SG72193, &UA2509124},
	{SG72194, &UA2510308},
	{SG72195, &UA2510788},
	{SG72196, &UA2516156},
	{SG72197, &UA2516440},
	{SG72198, &UA2516546},
	{SG72199, &UA2516603},
	{SG72200, &UA2516715},
	{SG72201, &UA2517435},
	{SG72202, &UA2531376},
	{SG72203, &UA2531385},
	{SG72204, &UA2538004},
	{SG72205, &UA2538520},
	{SG72206, &UA2542547},
	{SG72207, &UA2544322},
	{SG72208, &UA2552237},
	{SG72209, &UA2556582},
	{SG72210, &UA2559747},
	{SG72211, &UA2562223},
	{SG72212, &UA2562420},
	{SG72213, &UA2581226},
	{SG72214, &UA2590513},
	{SG72215, &UA2599453},
	{SG72216, &UA2602700},
	{SG72217, &UA2640709},
	{SG72218, &UA2643908},
	{SG72219, &UA2647167},
	{SG72220, &UA2651585},
	{SG72221, &UA2655487},
	{SG72222, &UA2664063},
	{SG72223, &UA2665421},
	{SG72224, &UA2666940},
	{SG72225, &UA2682895},
	{SG72226, &UA2683150},
	{SG72227, &UA2684422},
	{SG72228, &UA2705556},
	{SG72229, &UA2706014},
	{SG72230, &UA2708725},
	{SG72231, &UA2714818},
	{SG72232, &UA2720478},
	{SG72233, &UA2732760},
	{SG72234, &UA2732792},
	{SG72235, &UA2732882},
	{SG72236, &UA2768668},
	{SG72237, &UA2793283},
	{SG72238, &UA2793656},
	{SG72239, &UA2794904},
	{SG72240, &UA2795101},
	{SG72241, &UA2795118},
	{SG72242, &UA2795537},
	{SG72243, &UA2814837},
	{SG72244, &UA2857436},
	{SG72245, &UA2857622},
	{SG72246, &UA2858032},
	{SG72247, &UA2866231},
	{SG72248, &UA2889289},
	{SG72249, &UA2892704},
	{SG72250, &UA2937262},
	{SG72251, &UA2953225},
	{SG72252, &UA2953909},
	{SG72253, &UA2956979},
	{SG72254, &UA2957416},
	{SG72255, &UA2974420},
	{SG72256, &UA2974864},
	{SG72257, &UA2976144},
	{SG72258, &UA2979125},
	{SG72259, &UA2997740},
	{SG72260, &UA2999493},
	{SG72261, &UA3007227},
	{SG72262, &UA3020240},
	{SG72263, &UA3022354},
	{SG72264, &UA3022432},
	{SG72265, &UA3022622},
	{SG72266, &UA3022661},
	{SG72267, &UA3022856},
	{SG72268, &UA3023184},
	{SG72269, &UA3023449},
	{SG72270, &UA3023450},
	{SG72271, &UA3024115},
	{SG72272, &UA3024352},
	{SG72273, &UA3025432},
	{SG72274, &UA3025948},
	{SG72275, &UA3042365},
	{SG72276, &UA3046823},
	{SG72277, &UA3047994},
	{SG72278, &UA3049154},
	{SG72279, &UA3057865},
	{SG72280, &UA3058082},
	{SG72281, &UA3071906},
	{SG72282, &UA3078462},
	{SG72283, &UA3080608},
	{SG72284, &UA3082188},
	{SG72285, &UA3083103},
	{SG72286, &UA3083576},
	{SG72287, &UA3083581},
	{SG72288, &UA3083627},
	{SG72289, &UA3083657},
	{SG72290, &UA3083730},
	{SG72291, &UA3083822},
	{SG72292, &UA3083914},
	{SG72293, &UA3094583},
	{SG72294, &UA3095115},
	{SG72295, &UA3095432},
	{SG72296, &UA3095600},
	{SG72297, &UA3095727},
	{SG72298, &UA3096032},
	{SG72299, &UA3111018},
	{SG72300, &UA3111142},
	{SG72301, &UA3111160},
	{SG72302, &UA3112210},
	{SG72303, &UA3125264},
	{SG72304, &UA3127408},
	{SG72305, &UA3128296},
	{SG72306, &UA91989},
	{SG72307, &UA134990},
	{SG72308, &UA140540},
	{SG72309, &UA141186},
	{SG72310, &UA159920},
	{SG72311, &UA173753},
	{SG72312, &UA177242},
	{SG72313, &UA179285},
	{SG72314, &UA189286},
	{SG72315, &UA312663},
	{SG72316, &UA343333},
	{SG72317, &UA417940},
	{SG72318, &UA638610},
	{SG72319, &UA695919},
	{SG72320, &UA700901},
	{SG72321, &UA706676},
	{SG72322, &UA770092},
	{SG72323, &UA782742},
	{SG72324, &UA820483},
	{SG72325, &UA1868596},
	{SG72326, &UA1913860},
	{SG72327, &UA2041260},
	{SG72328, &UA2133797},
	{SG72329, &UA2135704},
	{SG72330, &UA2136635},
	{SG72331, &UA2137433},
	{SG72332, &UA2164681},
	{SG72333, &UA2165556},
	{SG72334, &UA2205772},
	{SG72335, &UA2211658},
	{SG72336, &UA2296153},
	{SG72337, &UA2367342},
	{SG72338, &UA2412087},
	{SG72339, &UA2430587},
	{SG72340, &UA2440344},
	{SG72341, &UA2511615},
	{SG72342, &UA2545683},
	{SG72343, &UA2651102},
	{SG72344, &UA2656400},
	{SG72345, &UA2670859},
	{SG72346, &UA2726731},
	{SG72347, &UA2787014},
	{SG72348, &UA2807505},
	{SG72349, &UA2823271},
	{SG72350, &UA195057},
	{SG72351, &UA195124},
	{SG72352, &UA195137},
	{SG72353, &UA661343},
	{SG72354, &UA2384887},
	{SG72355, &UA2418899},
	{SG72356, &UA2423443},
	{SG72357, &UA2463493},
	{SG72358, &UA2526610},
	{SG72359, &UA2533504},
	{SG72360, &UA2558466},
	{SG72361, &UA2606699},
	{SG72362, &UA2653213},
	{SG72363, &UA2694879},
	{SG72364, &UA2695158},
	{SG72365, &UA2695180},
	{SG72366, &UA2777911},
	{SG72367, &UA2793198},
	{SG72368, &UA2844249},
	{SG72369, &UA2996448},
	{SG72370, &UA3008512},
	{SG72371, &UA3050254},
	{SG72372, &UA231453},
	{SG72373, &UA340032},
	{SG72374, &UA599087},
	{SG72375, &UA628021},
	{SG72376, &UA635721},
	{SG72377, &UA723260},
	{SG72378, &UA794372},
	{SG72379, &UA860248},
	{SG72380, &UA892948},
	{SG72381, &UA1901458},
	{SG72382, &UA1902954},
	{SG72383, &UA1958462},
	{SG72384, &UA1968328},
	{SG72385, &UA1980580},
	{SG72386, &UA2161476},
	{SG72387, &UA2224971},
	{SG72388, &UA2245661},
	{SG72389, &UA2325423},
	{SG72390, &UA2359896},
	{SG72391, &UA2360212},
	{SG72392, &UA2362280},
	{SG72393, &UA2375080},
	{SG72394, &UA2387026},
	{SG72395, &UA2414005},
	{SG72396, &UA2424198},
	{SG72397, &UA2456274},
	{SG72398, &UA2495435},
	{SG72399, &UA2503397},
	{SG72400, &UA2642454},
	{SG72401, &UA2746477},
	{SG72402, &UA2792638},
	{SG72403, &UA2806998},
	{SG72404, &UA2807499},
	{SG72405, &UA2980511},
	{SG72406, &UA3011923},
	{SG72407, &UA2673309},
	{SG72408, &UA2673382},
	{SG72409, &UA2673817},
	{SG72410, &UA2674125},
	{SG72411, &UA2675667},
	{SG72412, &UA2676710},
	{SG72413, &UA2676975},
	{SG72414, &UA2677022},
	{SG72415, &UA2678686},
	{SG72416, &UA2678814},
	{SG72417, &UA2679187},
	{SG72418, &UA2679360},
	{SG72419, &UA2715748},
	{SG72420, &UA2716063},
	{SG72421, &UA2719162},
	{SG72422, &UA2735499},
	{SG72423, &UA2736138},
	{SG72424, &UA2736373},
	{SG72425, &UA2752695},
	{SG72426, &UA2862780},
	{SG72427, &UA137022},
	{SG72428, &UA137832},
	{SG72429, &UA138500},
	{SG72430, &UA138775},
	{SG72431, &UA165661},
	{SG72432, &UA167720},
	{SG72433, &UA171030},
	{SG72434, &UA175433},
	{SG72435, &UA182262},
	{SG72436, &UA445219},
	{SG72437, &UA471290},
	{SG72438, &UA472345},
	{SG72439, &UA523379},
	{SG72440, &UA782467},
	{SG72441, &UA1884669},
	{SG72442, &UA1885402},
	{SG72443, &UA2164022},
	{SG72444, &UA2213431},
	{SG72445, &UA2410410},
	{SG72446, &UA2519826},
	{SG72447, &UA2768564},
	{SG72448, &UA3048775},
	{SG72449, &UA2756977},
	{SG72450, &UA2784519},
	{SG72451, &UA2873734},
	{SG72452, &UA2873709},
	{SG72453, &UA2874802},
	{SG72454, &UA2875068},
	{SG72455, &UA2875650},
	{SG72456, &UA2876753},
	{SG72457, &UA2880054},
	{SG72458, &UA2882588},
	{SG72459, &UA2883808},
	{SG72460, &UA2884374},
	{SG72461, &UA2885238},
	{SG72462, &UA2897151},
	{SG72463, &UA3077215},
	{SG72464, &UA2966893},
	{SG72465, &UA3024756},
	{SG72466, &UA3025371},
	{SG72467, &UA3025911},
	{SG72468, &UA3026595},
	{SG72469, &UA3026718},
	{SG72470, &UA3026960},
	{SG72471, &UA3027085},
	{SG72472, &UA3027087},
	{SG72473, &UA3027273},
	{SG72474, &UA3028394},
	{SG72475, &UA3028408},
	{SG72476, &UA3028701},
	{SG72477, &UA3029866},
	{SG72478, &UA3125510},
	{SG72479, &UA634509},
	{SG72480, &UA2408323},
	{SG72481, &UA2408332},
	{SG72482, &UA2408366},
	{SG72483, &UA2427868},
	{SG72484, &UA2432300},
	{SG72485, &UA2432371},
	{SG72486, &UA2432697},
	{SG72487, &UA2432712},
	{SG72488, &UA2433217},
	{SG72489, &UA2433747},
	{SG72490, &UA2433767},
	{SG72491, &UA2434797},
	{SG72492, &UA2436318},
	{SG72493, &UA2458633},
	{SG72494, &UA2458765},
	{SG72495, &UA2459253},
	{SG72496, &UA2483304},
	{SG72497, &UA2487905},
	{SG72498, &UA2487893},
	{SG72499, &UA2492204},
	{SG72500, &UA2492212},
	{SG72501, &UA2492512},
	{SG72502, &UA2508849},
	{SG72503, &UA2509288},
	{SG72504, &UA2509328},
	{SG72505, &UA2509472},
	{SG72506, &UA2509978},
	{SG72507, &UA2510313},
	{SG72508, &UA2510540},
	{SG72509, &UA2532022},
	{SG72510, &UA2699012},
	{SG72511, &UA2844352},
	{SG72512, &UA2850713},
	{SG72513, &UA2986312},
	{SG72514, &UA3120917},
	{SG72515, &UA796281},
	{SG72516, &UA862024},
	{SG72517, &UA878818},
	{SG72518, &UA2795932},
	{SG72519, &UA2993141},
	{SG72520, &UA2567798},
	{SG72521, &UA2567880},
	{SG72522, &UA2568219},
	{SG72523, &UA2568520},
	{SG72524, &UA2568843},
	{SG72525, &UA2583626},
	{SG72526, &UA2583900},
	{SG72527, &UA2584014},
	{SG72528, &UA2584079},
	{SG72529, &UA2584224},
	{SG72530, &UA2605503},
	{SG72531, &UA2605844},
	{SG72532, &UA2752120},
	{SG72533, &UA3063958},
	{SG72534, &UA3067636},
	{SG72535, &UA3067752},
	{SG72536, &UA3067761},
	{SG72537, &UA3067803},
	{SG72538, &UA3067814},
	{SG72539, &UA3067928},
	{SG72540, &UA3067950},
	{SG72541, &UA3067961},
	{SG72542, &UA3067965},
	{SG72543, &UA3067991},
	{SG72544, &UA3068032},
	{SG72545, &UA3068123},
	{SG72546, &UA3079925},
	{SG72547, &UA3080284},
	{SG72548, &UA3082340},
	{SG72549, &UA3093869},
	{SG72550, &UA3112008},
	{SG72551, &UA3115041},
	{SG72552, &UA3127777},
	{SG72553, &UA3127794},
	{SG72554, &UA2945889},
	{SG72555, &UA3024748},
	{SG72556, &UA3024750},
	{SG72557, &UA3024755},
	{SG72558, &UA3024949},
	{SG72559, &UA3025064},
	{SG72560, &UA3025079},
	{SG72561, &UA3025193},
	{SG72562, &UA3025533},
	{SG72563, &UA3025705},
	{SG72564, &UA3025707},
	{SG72565, &UA3025717},
	{SG72566, &UA3026166},
	{SG72567, &UA3026462},
	{SG72568, &UA3027911},
	{SG72569, &UA3028030},
	{SG72570, &UA3028411},
	{SG72571, &UA3028466},
	{SG72572, &UA3028682},
	{SG72573, &UA3029760},
	{SG72574, &UA3029877},
	{SG72575, &UA3029972},
	{SG72576, &UA3029999},
	{SG72577, &UA3030409},
	{SG72578, &UA3061056},
	{SG72579, &UA551246},
	{SG72580, &UA2756910},
	{SG72581, &UA2757023},
	{SG72582, &UA2770957},
	{SG72583, &UA2873651},
	{SG72584, &UA2873649},
	{SG72585, &UA2873684},
	{SG72586, &UA2873921},
	{SG72587, &UA2874158},
	{SG72588, &UA2874160},
	{SG72589, &UA2874594},
	{SG72590, &UA2874657},
	{SG72591, &UA2875740},
	{SG72592, &UA2876227},
	{SG72593, &UA2876996},
	{SG72594, &UA2884932},
	{SG72595, &UA2885557},
	{SG72596, &UA2940459},
	{SG72597, &UA2941790},
	{SG72598, &UA2958222},
	{SG72599, &UA2670520},
	{SG72600, &UA2673319},
	{SG72601, &UA2674022},
	{SG72602, &UA2674110},
	{SG72603, &UA2674404},
	{SG72604, &UA2675987},
	{SG72605, &UA2678399},
	{SG72606, &UA2678405},
	{SG72607, &UA2678700},
	{SG72608, &UA2678878},
	{SG72609, &UA2678882},
	{SG72610, &UA2679015},
	{SG72611, &UA2715340},
	{SG72612, &UA2715532},
	{SG72613, &UA2715832},
	{SG72614, &UA2716399},
	{SG72615, &UA2716778},
	{SG72616, &UA2716934},
	{SG72617, &UA2717464},
	{SG72618, &UA2735686},
	{SG72619, &UA2736195},
	{SG72620, &UA2860571},
	{SG72621, &UA2861560},
	{SG72622, &UA2936242},
	{SG72623, &UA2993575},
	{SG72624, &UA3004542},
	{SG72625, &UA3111316},
	{SG72626, &UA94673},
	{SG72627, &UA96501},
	{SG72628, &UA98524},
	{SG72629, &UA123447},
	{SG72630, &UA132090},
	{SG72631, &UA145535},
	{SG72632, &UA153517},
	{SG72633, &UA385057},
	{SG72634, &UA459769},
	{SG72635, &UA641328},
	{SG72636, &UA703314},
	{SG72637, &UA2410540},
	{SG72638, &UA2518181},
	{SG72639, &UA2743130},
	{SG72643, &UA2361978},
	{SG72644, &UA2389934},
	{SG72645, &UA2408300},
	{SG72646, &UA2408304},
	{SG72647, &UA2408355},
	{SG72648, &UA2408361},
	{SG72649, &UA2408387},
	{SG72650, &UA2408408},
	{SG72651, &UA2408412},
	{SG72652, &UA2427659},
	{SG72653, &UA2432060},
	{SG72654, &UA2432832},
	{SG72655, &UA2432996},
	{SG72656, &UA2433762},
	{SG72657, &UA2433892},
	{SG72658, &UA2434198},
	{SG72659, &UA2458417},
	{SG72660, &UA2458450},
	{SG72661, &UA2458708},
	{SG72662, &UA2654947},
	{SG72663, &UA3043850},
	{SG72664, &UA3103969},
	{SG72666, &UA2203778},
	{SG72667, &UA2203789},
	{SG72668, &UA2203794},
	{SG72669, &UA2232103},
	{SG72670, &UA2238112},
	{SG72671, &UA2238120},
	{SG72672, &UA2238209},
	{SG72673, &UA2238218},
	{SG72674, &UA2238231},
	{SG72675, &UA2238309},
	{SG72676, &UA2249455},
	{SG72677, &UA2253591},
	{SG72678, &UA2354037},
	{SG72679, &UA2466144},
	{SG72680, &UA2578455},
	{SG72681, &UA2705949},
	{SG72682, &UA2487801},
	{SG72683, &UA2488807},
	{SG72684, &UA2492180},
	{SG72685, &UA2492190},
	{SG72686, &UA2492232},
	{SG72687, &UA2492234},
	{SG72688, &UA2492236},
	{SG72689, &UA2492300},
	{SG72690, &UA2492303},
	{SG72691, &UA2492464},
	{SG72692, &UA2510273},
	{SG72693, &UA2510290},
	{SG72694, &UA2513196},
	{SG72695, &UA2557813},
	{SG72696, &UA2659515},
	{SG72697, &UA2748914},
	{SG72698, &UA2779590},
	{SG72699, &UA2952720},
	{SG72700, &UA2956853},
	{SG72701, &UA3088137},
	{SG72702, &UA2538398},
	{SG72703, &UA2538587},
	{SG72704, &UA2567810},
	{SG72705, &UA2567820},
	{SG72706, &UA2567859},
	{SG72707, &UA2567876},
	{SG72708, &UA2568054},
	{SG72709, &UA2568143},
	{SG72710, &UA2568258},
	{SG72711, &UA2568856},
	{SG72712, &UA2583954},
	{SG72713, &UA2584377},
	{SG72714, &UA2584663},
	{SG72715, &UA2605732},
	{SG72716, &UA2711308},
	{SG72717, &UA2855586},
	{SG72718, &UA2989760},
	{SG72719, &UA3019521},
	{SG72720, &UA184782},
	{SG72721, &UA189422},
	{SG72722, &UA311898},
	{SG72723, &UA393313},
	{SG72724, &UA632494},
	{SG72725, &UA717331},
	{SG72726, &UA747610},
	{SG72727, &UA1886606},
	{SG72728, &UA1944882},
	{SG72729, &UA1954988},
	{SG72730, &UA1957036},
	{SG72731, &UA1961346},
	{SG72732, &UA1996106},
	{SG72733, &UA2151490},
	{SG72734, &UA2159638},
	{SG72735, &UA2195395},
	{SG72736, &UA2215519},
	{SG72737, &UA2216892},
	{SG72738, &UA2336553},
	{SG72739, &UA2417588},
	{SG72740, &UA2418330},
	{SG72741, &UA2425790},
	{SG72742, &UA2490689},
	{SG72743, &UA2577046},
	{SG72744, &UA2692191},
	{SG72745, &UA2775881},
	{SG72746, &UA2889176},
	{SG72747, &UA2980203},
	{SG72748, &UA2980957},
	{SG72749, &UA2981534},
	{SG72750, &UA207952},
	{SG72751, &UA394775},
	{SG72752, &UA452085},
	{SG72753, &UA542484},
	{SG72754, &UA561236},
	{SG72755, &UA566411},
	{SG72756, &UA604988},
	{SG72757, &UA605178},
	{SG72758, &UA700181},
	{SG72759, &UA710712},
	{SG72760, &UA713578},
	{SG72761, &UA719990},
	{SG72762, &UA1904123},
	{SG72763, &UA1981536},
	{SG72764, &UA2170860},
	{SG72765, &UA2220881},
	{SG72766, &UA2301579},
	{SG72767, &UA2362874},
	{SG72768, &UA2390147},
	{SG72769, &UA2456927},
	{SG72770, &UA2482782},
	{SG72771, &UA2516556},
	{SG72772, &UA2524647},
	{SG72773, &UA2537918},
	{SG72774, &UA2583465},
	{SG72775, &UA2700415},
	{SG72776, &UA2786643},
	{SG72777, &UA2982212},
	{SG72778, &UA2987409},
	{SG72779, &UA3023082},
	{SG72780, &UA3023962},
	{SG72781, &UA3058363},
	{SG72782, &UA3083814},
	{SG72783, &UA3095547},
	{SG72784, &UA3100285},
	{SG72785, &UA120053},
	{SG72786, &UA194787},
	{SG72787, &UA375501},
	{SG72788, &UA383549},
	{SG72789, &UA418341},
	{SG72790, &UA2294812},
	{SG72791, &UA2791883},
	{SG72792, &UA2850035},
	{SG72793, &UA2223444},
	{SG72794, &UA2335587},
	{SG72795, &UA2340147},
	{SG72796, &UA2368178},
	{SG72797, &UA2371994},
	{SG72798, &UA2386723},
	{SG72799, &UA2387000},
	{SG72800, &UA2403727},
	{SG72801, &UA2423450},
	{SG72802, &UA2445615},
	{SG72803, &UA2520391},
	{SG72804, &UA2642529},
	{SG72805, &UA2813592},
	{SG72806, &UA2203738},
	{SG72807, &UA2203742},
	{SG72808, &UA2203730},
	{SG72809, &UA2203865},
	{SG72810, &UA2203896},
	{SG72811, &UA2203981},
	{SG72812, &UA2203988},
	{SG72813, &UA2231614},
	{SG72814, &UA2237801},
	{SG72815, &UA2237847},
	{SG72816, &UA2238251},
	{SG72817, &UA2238470},
	{SG72818, &UA2246685},
	{SG72819, &UA2259653},
	{SG72820, &UA2354395},
	{SG72821, &UA2354521},
	{SG72822, &UA2354628},
	{SG72823, &UA2355114},
	{SG72824, &UA2653915},
	{SG72825, &UA2762807},
	{SG72826, &UA246719},
	{SG72827, &UA248188},
	{SG72828, &UA330420},
	{SG72829, &UA437187},
	{SG72830, &UA1988497},
	{SG72831, &UA2158414},
	{SG72832, &UA2232420},
	{SG72833, &UA2238372},
	{SG72834, &UA2362965},
	{SG72835, &UA2374652},
	{SG72836, &UA2402882},
	{SG72837, &UA2429191},
	{SG72838, &UA2429562},
	{SG72839, &UA2455989},
	{SG72840, &UA2503062},
	{SG72841, &UA2516444},
	{SG72842, &UA2528658},
	{SG72843, &UA2568506},
	{SG72844, &UA2676911},
	{SG72845, &UA2695884},
	{SG72846, &UA2721499},
	{SG72847, &UA2741521},
	{SG72848, &UA2818256},
	{SG72849, &UA2849242},
	{SG72850, &UA2871624},
	{SG72851, &UA2943864},
	{SG72852, &UA2947522},
	{SG72853, &UA3034757},
	{SG72854, &UA3048586},
	{SG72855, &UA3083879},
	{SG72856, &UA1968751},
	{SG72857, &UA1976172},
	{SG72858, &UA1994850},
	{SG72859, &UA1995072},
	{SG72860, &UA2160260},
	{SG72861, &UA2167311},
	{SG72862, &UA2167895},
	{SG72863, &UA2168119},
	{SG72864, &UA2183028},
	{SG72865, &UA2223269},
	{SG72866, &UA2245691},
	{SG72867, &UA2245754},
	{SG72868, &UA2245784},
	{SG72869, &UA2245786},
	{SG72870, &UA2338109},
	{SG72871, &UA2342208},
	{SG72872, &UA2345329},
	{SG72873, &UA2346180},
	{SG72874, &UA2363218},
	{SG72875, &UA2366447},
	{SG72876, &UA2367414},
	{SG72877, &UA2367613},
	{SG72878, &UA2375967},
	{SG72879, &UA2381071},
	{SG72880, &UA2383202},
	{SG72881, &UA2384830},
	{SG72882, &UA2384860},
	{SG72883, &UA2400835},
	{SG72884, &UA2421718},
	{SG72885, &UA2422282},
	{SG72886, &UA2456270},
	{SG72887, &UA2468611},
	{SG72888, &UA2481391},
	{SG72889, &UA2481644},
	{SG72890, &UA2481927},
	{SG72891, &UA2483246},
	{SG72892, &UA2485931},
	{SG72893, &UA2502995},
	{SG72894, &UA2507623},
	{SG72895, &UA2507631},
	{SG72896, &UA2527670},
	{SG72897, &UA2530815},
	{SG72898, &UA2553566},
	{SG72899, &UA2553951},
	{SG72900, &UA2554267},
	{SG72901, &UA2554879},
	{SG72902, &UA2556373},
	{SG72903, &UA2557457},
	{SG72904, &UA2559141},
	{SG72905, &UA2560056},
	{SG72906, &UA2568526},
	{SG72907, &UA2569103},
	{SG72908, &UA2569620},
	{SG72909, &UA2569992},
	{SG72910, &UA2579641},
	{SG72911, &UA2580327},
	{SG72912, &UA2580375},
	{SG72913, &UA2583555},
	{SG72914, &UA2584257},
	{SG72915, &UA2584314},
	{SG72916, &UA2585059},
	{SG72917, &UA2593833},
	{SG72918, &UA2595363},
	{SG72919, &UA2595508},
	{SG72920, &UA2599988},
	{SG72921, &UA2602023},
	{SG72922, &UA2605557},
	{SG72923, &UA2606460},
	{SG72924, &UA2653326},
	{SG72925, &UA2659266},
	{SG72926, &UA2666980},
	{SG72927, &UA2667079},
	{SG72928, &UA2667869},
	{SG72929, &UA2668115},
	{SG72930, &UA2668538},
	{SG72931, &UA2668694},
	{SG72932, &UA2668831},
	{SG72933, &UA2668851},
	{SG72934, &UA2669546},
	{SG72935, &UA2703000},
	{SG72936, &UA2704120},
	{SG72937, &UA2707218},
	{SG72938, &UA2711307},
	{SG72939, &UA2727154},
	{SG72940, &UA2727201},
	{SG72941, &UA2727499},
	{SG72942, &UA2729717},
	{SG72943, &UA2732968},
	{SG72944, &UA2733220},
	{SG72945, &UA2733221},
	{SG72946, &UA2733252},
	{SG72947, &UA2733470},
	{SG72948, &UA2733702},
	{SG72949, &UA2733905},
	{SG72950, &UA2736042},
	{SG72951, &UA2740042},
	{SG72952, &UA2740048},
	{SG72953, &UA2741529},
	{SG72954, &UA2741617},
	{SG72955, &UA2750460},
	{SG72956, &UA2753019},
	{SG72957, &UA2753257},
	{SG72958, &UA2753372},
	{SG72959, &UA2756548},
	{SG72960, &UA2770865},
	{SG72961, &UA2771320},
	{SG72962, &UA2782455},
	{SG72963, &UA2782448},
	{SG72964, &UA2782547},
	{SG72965, &UA2782789},
	{SG72966, &UA2791455},
	{SG72967, &UA2795231},
	{SG72968, &UA2796463},
	{SG72969, &UA2796490},
	{SG72970, &UA2797018},
	{SG72971, &UA2800145},
	{SG72972, &UA2817891},
	{SG72973, &UA2824355},
	{SG72974, &UA2840601},
	{SG72975, &UA2843831},
	{SG72976, &UA2844726},
	{SG72977, &UA2851399},
	{SG72978, &UA2852174},
	{SG72979, &UA2856659},
	{SG72980, &UA2858467},
	{SG72981, &UA2866833},
	{SG72982, &UA2867099},
	{SG72983, &UA2868440},
	{SG72984, &UA2868582},
	{SG72985, &UA2868672},
	{SG72986, &UA2868744},
	{SG72987, &UA2868912},
	{SG72988, &UA2870186},
	{SG72989, &UA2892432},
	{SG72990, &UA2947152},
	{SG72991, &UA2947176},
	{SG72992, &UA2947355},
	{SG72993, &UA2948288},
	{SG72994, &UA2948297},
	{SG72995, &UA2948320},
	{SG72996, &UA2948324},
	{SG72997, &UA2966215},
	{SG72998, &UA2966910},
	{SG72999, &UA2967029},
	{SG73000, &UA2967563},
	{SG73001, &UA2968796},
	{SG73002, &UA2969646},
	{SG73003, &UA2970563},
	{SG73004, &UA2970920},
	{SG73005, &UA2982318},
	{SG73006, &UA3014239},
	{SG73007, &UA3015213},
	{SG73008, &UA3015750},
	{SG73009, &UA3016710},
	{SG73010, &UA3017691},
	{SG73011, &UA3017714},
	{SG73012, &UA3018109},
	{SG73013, &UA3018298},
	{SG73014, &UA3018440},
	{SG73015, &UA3018925},
	{SG73016, &UA3020093},
	{SG73017, &UA3020882},
	{SG73018, &UA3021186},
	{SG73019, &UA3043376},
	{SG73020, &UA3057380},
	{SG73021, &UA3057622},
	{SG73022, &UA3057801},
	{SG73023, &UA3077878},
	{SG73024, &UA3078174},
	{SG73025, &UA3083967},
	{SG73026, &UA3087835},
	{SG73027, &UA3088067},
	{SG73028, &UA3088397},
	{SG73029, &UA3094720},
	{SG73030, &UA3094874},
	{SG73031, &UA3096126},
	{SG73032, &UA3096146},
	{SG73033, &UA3107576},
	{SG73034, &UA3107672},
	{SG73035, &UA3109443},
	{SG73036, &UA3109774},
	{SG73037, &UA3112218},
	{SG73038, &UA3115468},
	{SG73039, &UA3124809},
	{SG73040, &UA3126643},
	{SG73041, &UA3126775},
	{SG73042, &UA3127712},
	{SG73043, &UA3128581},
	{SG73044, &UA3130285},
	{SG73045, &UA97723},
	{SG73046, &UA100803},
	{SG73047, &UA120854},
	{SG73048, &UA139276},
	{SG73049, &UA205701},
	{SG73050, &UA236529},
	{SG73051, &UA288183},
	{SG73052, &UA293710},
	{SG73053, &UA383635},
	{SG73054, &UA416679},
	{SG73055, &UA444960},
	{SG73056, &UA520529},
	{SG73057, &UA698255},
	{SG73058, &UA702681},
	{SG73059, &UA723474},
	{SG73060, &UA765288},
	{SG73061, &UA772342},
	{SG73062, &UA804720},
	{SG73063, &UA1863295},
	{SG73064, &UA1941325},
	{SG73065, &UA1942731},
	{SG73066, &UA1945353},
	{SG73067, &UA1946477},
	{SG73068, &UA1948245},
	{SG73069, &UA1989339},
	{SG73070, &UA1992686},
	{SG73071, &UA1994657},
	{SG73072, &UA2141168},
	{SG73073, &UA2151308},
	{SG73074, &UA2154750},
	{SG73075, &UA2160336},
	{SG73076, &UA2162665},
	{SG73077, &UA2163669},
	{SG73078, &UA2165670},
	{SG73079, &UA2167461},
	{SG73080, &UA2217074},
	{SG73081, &UA2321142},
	{SG73082, &UA2326416},
	{SG73083, &UA2330030},
	{SG73084, &UA2335464},
	{SG73085, &UA2371003},
	{SG73086, &UA2408503},
	{SG73087, &UA2410022},
	{SG73088, &UA2591048},
	{SG73089, &UA2682171},
	{SG73090, &UA2686478},
	{SG73091, &UA2686972},
	{SG73092, &UA2741804},
	{SG73093, &UA2741888},
	{SG73094, &UA2743537},
	{SG73095, &UA2743655},
	{SG73096, &UA2745100},
	{SG73097, &UA2746685},
	{SG73098, &UA2759270},
	{SG73099, &UA2787235},
	{SG73100, &UA2796625},
	{SG73101, &UA2812062},
	{SG73102, &UA2813965},
	{SG73103, &UA2815035},
	{SG73104, &UA2817002},
	{SG73105, &UA2828636},
	{SG73106, &UA2829270},
	{SG73107, &UA2829788},
	{SG73108, &UA2832934},
	{SG73109, &UA2848088},
	{SG73110, &UA2888891},
	{SG73111, &UA2889048},
	{SG73112, &UA2933383},
	{SG73113, &UA2933523},
	{SG73114, &UA2950187},
	{SG73115, &UA2950219},
	{SG73116, &UA2950432},
	{SG73117, &UA2950485},
	{SG73118, &UA2950685},
	{SG73119, &UA2979664},
	{SG73120, &UA2981084},
	{SG73121, &UA2981636},
	{SG73122, &UA2985479},
	{SG73123, &UA3069668},
	{SG73124, &UA3098448},
	{SG73126, &UA1994645},
	{SG73127, &UA1995075},
	{SG73128, &UA2338911},
	{SG73129, &UA2339317},
	{SG73130, &UA2356148},
	{SG73131, &UA2367297},
	{SG73132, &UA2367250},
	{SG73133, &UA2382056},
	{SG73134, &UA2384858},
	{SG73135, &UA2386388},
	{SG73136, &UA2428619},
	{SG73137, &UA2491521},
	{SG73138, &UA2531299},
	{SG73139, &UA2533365},
	{SG73140, &UA2548084},
	{SG73141, &UA2553711},
	{SG73142, &UA2553876},
	{SG73143, &UA2557965},
	{SG73144, &UA2562041},
	{SG73145, &UA2583616},
	{SG73146, &UA2583919},
	{SG73147, &UA2583943},
	{SG73148, &UA2584022},
	{SG73149, &UA2584281},
	{SG73150, &UA2589588},
	{SG73151, &UA2605629},
	{SG73152, &UA2659166},
	{SG73153, &UA2666990},
	{SG73154, &UA2667765},
	{SG73155, &UA2695098},
	{SG73156, &UA2695371},
	{SG73157, &UA2695758},
	{SG73158, &UA2695980},
	{SG73159, &UA2705433},
	{SG73160, &UA2705714},
	{SG73161, &UA2706959},
	{SG73162, &UA2708892},
	{SG73163, &UA2712871},
	{SG73164, &UA2713336},
	{SG73165, &UA2727829},
	{SG73166, &UA2732999},
	{SG73167, &UA2739172},
	{SG73168, &UA2740008},
	{SG73169, &UA2740460},
	{SG73170, &UA2741535},
	{SG73171, &UA2741612},
	{SG73172, &UA2750239},
	{SG73173, &UA2753193},
	{SG73174, &UA2754676},
	{SG73175, &UA2769996},
	{SG73176, &UA2770379},
	{SG73177, &UA2796802},
	{SG73178, &UA2797158},
	{SG73179, &UA2816292},
	{SG73180, &UA2842632},
	{SG73181, &UA2855643},
	{SG73182, &UA2868487},
	{SG73183, &UA2879090},
	{SG73184, &UA2879195},
	{SG73185, &UA2879251},
	{SG73186, &UA2937602},
	{SG73187, &UA2938122},
	{SG73188, &UA2946040},
	{SG73189, &UA2947171},
	{SG73190, &UA2947525},
	{SG73191, &UA2947853},
	{SG73192, &UA2948031},
	{SG73193, &UA2957394},
	{SG73194, &UA2958021},
	{SG73195, &UA2966223},
	{SG73196, &UA2966398},
	{SG73197, &UA2967152},
	{SG73198, &UA2967793},
	{SG73199, &UA2969736},
	{SG73200, &UA2982882},
	{SG73201, &UA2985336},
	{SG73202, &UA2997910},
	{SG73203, &UA2998700},
	{SG73204, &UA3014386},
	{SG73205, &UA3014928},
	{SG73206, &UA3015042},
	{SG73207, &UA3016041},
	{SG73208, &UA3017748},
	{SG73209, &UA3019868},
	{SG73210, &UA3020596},
	{SG73211, &UA3057453},
	{SG73212, &UA3057483},
	{SG73213, &UA3057643},
	{SG73214, &UA3077866},
	{SG73215, &UA3078340},
	{SG73216, &UA3078420},
	{SG73217, &UA3083964},
	{SG73218, &UA3092082},
	{SG73219, &UA3096187},
	{SG73220, &UA3096214},
	{SG73221, &UA3096247},
	{SG73222, &UA3107229},
	{SG73223, &UA3107455},
	{SG73224, &UA3107955},
	{SG73225, &UA3109444},
	{SG73226, &UA3110705},
	{SG73227, &UA3114862},
	{SG73228, &UA3121257},
	{SG73229, &UA3125033},
	{SG73230, &UA3126685},
	{SG73231, &UA3127542},
	{SG73232, &UA3127706},
	{SG73233, &UA53601},
	{SG73234, &UA99983},
	{SG73235, &UA118643},
	{SG73236, &UA127479},
	{SG73237, &UA142069},
	{SG73238, &UA261763},
	{SG73239, &UA1997290},
	{SG73240, &UA2001277},
	{SG73241, &UA2007819},
	{SG73242, &UA2041065},
	{SG73243, &UA2041080},
	{SG73244, &UA2041746},
	{SG73245, &UA2041989},
	{SG73246, &UA2343127},
	{SG73247, &UA2344387},
	{SG73248, &UA2367759},
	{SG73249, &UA2380600},
	{SG73250, &UA2416770},
	{SG73251, &UA2591335},
	{SG73252, &UA2824109},
	{SG73253, &UA604539},
	{SG73254, &UA622649},
	{SG73255, &UA816049},
	{SG73256, &UA895783},
	{SG73257, &UA900163},
	{SG73258, &UA1966799},
	{SG73259, &UA1975611},
	{SG73260, &UA2160083},
	{SG73261, &UA2164421},
	{SG73262, &UA2167025},
	{SG73263, &UA2222696},
	{SG73264, &UA2223541},
	{SG73265, &UA2245660},
	{SG73266, &UA2245725},
	{SG73267, &UA2337538},
	{SG73268, &UA2339236},
	{SG73269, &UA2359843},
	{SG73270, &UA2361616},
	{SG73271, &UA2362813},
	{SG73272, &UA2362824},
	{SG73273, &UA2368305},
	{SG73274, &UA2368416},
	{SG73275, &UA2387978},
	{SG73276, &UA2390209},
	{SG73277, &UA2392149},
	{SG73278, &UA2404134},
	{SG73279, &UA2404776},
	{SG73280, &UA2425620},
	{SG73281, &UA2428138},
	{SG73282, &UA2445490},
	{SG73283, &UA2466520},
	{SG73284, &UA2482029},
	{SG73285, &UA2482175},
	{SG73286, &UA2482453},
	{SG73287, &UA2482991},
	{SG73288, &UA2514000},
	{SG73289, &UA2516160},
	{SG73290, &UA2516161},
	{SG73291, &UA2530094},
	{SG73292, &UA2531386},
	{SG73293, &UA2532180},
	{SG73294, &UA2537813},
	{SG73295, &UA2560336},
	{SG73296, &UA2560721},
	{SG73297, &UA2562718},
	{SG73298, &UA2563798},
	{SG73299, &UA2566876},
	{SG73300, &UA2581228},
	{SG73301, &UA2582450},
	{SG73302, &UA2599630},
	{SG73303, &UA2654324},
	{SG73304, &UA2664345},
	{SG73305, &UA2695804},
	{SG73306, &UA2700688},
	{SG73307, &UA2765060},
	{SG73308, &UA2765660},
	{SG73309, &UA2793222},
	{SG73310, &UA2811650},
	{SG73311, &UA2811800},
	{SG73312, &UA2837890},
	{SG73313, &UA2853592},
	{SG73314, &UA2859236},
	{SG73315, &UA3021738},
	{SG73316, &UA3023182},
	{SG73317, &UA3023411},
	{SG73318, &UA3023465},
	{SG73319, &UA3023560},
	{SG73320, &UA3025165},
	{SG73321, &UA3025428},
	{SG73322, &UA3078088},
	{SG73323, &UA3078468},
	{SG73324, &UA3081715},
	{SG73325, &UA3082919},
	{SG73326, &UA3083568},
	{SG73327, &UA3083683},
	{SG73328, &UA3083739},
	{SG73329, &UA3094738},
	{SG73330, &UA3095829},
	{SG73331, &UA3110972},
	{SG73332, &UA3110979},
	{SG73333, &UA3111673},
	{SG73334, &UA3126796},
	{SG73335, &UA3127368},
	{SG73336, &UA252360},
	{SG73337, &UA255083},
	{SG73338, &UA453173},
	{SG73339, &UA454034},
	{SG73340, &UA660262},
	{SG73341, &UA703143},
	{SG73342, &UA789719},
	{SG73343, &UA862958},
	{SG73344, &UA1930079},
	{SG73345, &UA2231563},
	{SG73346, &UA2335744},
	{SG73347, &UA2786843},
	{SG73348, &UA2840703},
	{SG73350, &UA32708},
	{SG73351, &UA159202},
	{SG73352, &UA362802},
	{SG73354, &UA243035},
	{SG73355, &UA2408222},
	{SG73356, &UA2408331},
	{SG73357, &UA2408336},
	{SG73358, &UA2430854},
	{SG73359, &UA2432633},
	{SG73360, &UA2432708},
	{SG73361, &UA2434537},
	{SG73362, &UA2436390},
	{SG73363, &UA2460545},
	{SG73364, &UA160310},
	{SG73365, &UA172970},
	{SG73366, &UA196096},
	{SG73367, &UA389424},
	{SG73368, &UA440401},
	{SG73369, &UA685677},
	{SG73370, &UA2330134},
	{SG73371, &UA2440193},
	{SG73372, &UA2509636},
	{SG73373, &UA2699907},
	{SG73374, &UA2726845},
	{SG73375, &UA2791751},
	{SG73376, &UA2792165},
	{SG73377, &UA2792792},
	{SG73378, &UA2849286},
	{SG73379, &UA2674037},
	{SG73380, &UA2674114},
	{SG73381, &UA2674546},
	{SG73382, &UA2675371},
	{SG73383, &UA2678527},
	{SG73384, &UA2678528},
	{SG73385, &UA2678737},
	{SG73386, &UA2679194},
	{SG73387, &UA2717243},
	{SG73388, &UA2717246},
	{SG73389, &UA2719122},
	{SG73390, &UA2736824},
	{SG73391, &UA712055},
	{SG73392, &UA857353},
	{SG73393, &UA2190085},
	{SG73394, &UA2859404},
	{SG73395, &UA3099627},
	{SG73396, &UA97892},
	{SG73397, &UA134093},
	{SG73398, &UA520385},
	{SG73399, &UA630752},
	{SG73400, &UA2302286},
	{SG73401, &UA2306690},
	{SG73402, &UA2510263},
	{SG73403, &UA2581090},
	{SG73404, &UA2658810},
	{SG73405, &UA2658869},
	{SG73406, &UA2659583},
	{SG73407, &UA2659774},
	{SG73408, &UA2792870},
	{SG73410, &UA2753532},
	{SG73411, &UA2783724},
	{SG73412, &UA2784509},
	{SG73413, &UA2784643},
	{SG73414, &UA2784654},
	{SG73415, &UA2869733},
	{SG73416, &UA2873766},
	{SG73417, &UA2873987},
	{SG73418, &UA2880450},
	{SG73419, &UA2942148},
	{SG73420, &UA30679},
	{SG73421, &UA95620},
	{SG73422, &UA100486},
	{SG73423, &UA122427},
	{SG73424, &UA135688},
	{SG73425, &UA259545},
	{SG73426, &UA260539},
	{SG73427, &UA279086},
	{SG73428, &UA285185},
	{SG73429, &UA285834},
	{SG73430, &UA289031},
	{SG73431, &UA416557},
	{SG73432, &UA442653},
	{SG73433, &UA450803},
	{SG73434, &UA484953},
	{SG73435, &UA496902},
	{SG73436, &UA551102},
	{SG73437, &UA624244},
	{SG73438, &UA659856},
	{SG73439, &UA675101},
	{SG73440, &UA716510},
	{SG73441, &UA730133},
	{SG73442, &UA804182},
	{SG73443, &UA1887577},
	{SG73444, &UA1937851},
	{SG73445, &UA2266700},
	{SG73446, &UA2272833},
	{SG73447, &UA2306926},
	{SG73448, &UA2801397},
	{SG73449, &UA2948940},
	{SG73450, &UA3025188},
	{SG73451, &UA3025322},
	{SG73452, &UA3026098},
	{SG73453, &UA3027211},
	{SG73454, &UA3028415},
	{SG73455, &UA3029155},
	{SG73456, &UA3029269},
	{SG73457, &UA3029517},
	{SG73458, &UA3029886},
	{SG73459, &UA3058614},
	{SG73460, &UA1953383},
	{SG73461, &UA1953780},
	{SG73462, &UA1958812},
	{SG73463, &UA1992509},
	{SG73464, &UA2220660},
	{SG73465, &UA2248159},
	{SG73466, &UA2334989},
	{SG73467, &UA2658939},
	{SG73468, &UA2850904},
	{SG73469, &UA2488089},
	{SG73470, &UA2490848},
	{SG73471, &UA2492184},
	{SG73472, &UA2492404},
	{SG73473, &UA2492555},
	{SG73474, &UA2492547},
	{SG73475, &UA2510086},
	{SG73476, &UA2511093},
	{SG73477, &UA2511493},
	{SG73478, &UA2537826},
	{SG73479, &UA1953337},
	{SG73480, &UA1965080},
	{SG73481, &UA1987526},
	{SG73482, &UA1990843},
	{SG73483, &UA2256648},
	{SG73484, &UA2413425},
	{SG73485, &UA2596336},
	{SG73486, &UA2791704},
	{SG73487, &UA22117},
	{SG73488, &UA690912},
	{SG73489, &UA2505180},
	{SG73490, &UA93316},
	{SG73491, &UA104475},
	{SG73492, &UA121204},
	{SG73493, &UA130421},
	{SG73494, &UA382278},
	{SG73495, &UA496319},
	{SG73496, &UA538597},
	{SG73497, &UA877597},
	{SG73498, &UA2658840},
	{SG73499, &UA2851957},
	{SG73500, &UA3041939},
	{SG73502, &UA2567925},
	{SG73503, &UA2567991},
	{SG73504, &UA2568124},
	{SG73505, &UA2568160},
	{SG73506, &UA2568369},
	{SG73507, &UA2584520},
	{SG73508, &UA2584870},
	{SG73509, &UA2584943},
	{SG73510, &UA2605847},
	{SG73511, &UA2860457},
	{SG73512, &UA3036469},
	{SG73513, &UA3064841},
	{SG73514, &UA3067260},
	{SG73515, &UA3067272},
	{SG73516, &UA3067850},
	{SG73517, &UA3067884},
	{SG73518, &UA3068226},
	{SG73519, &UA3080738},
	{SG73520, &UA94674},
	{SG73521, &UA99212},
	{SG73522, &UA134562},
	{SG73523, &UA2128942},
	{SG73524, &UA2258012},
	{SG73525, &UA2504903},
	{SG73531, &UA2008820},
	{SG73532, &UA2009818},
	{SG73533, &UA2010180},
	{SG73534, &UA2015626},
	{SG73535, &UA2015631},
	{SG73536, &UA2189938},
	{SG73537, &UA2201688},
	{SG73538, &UA2343990},
	{SG73539, &UA2345592},
	{SG73540, &UA2403119},
	{SG73541, &UA2470507},
	{SG73542, &UA2500878},
	{SG73543, &UA2525449},
	{SG73544, &UA2649121},
	{SG73545, &UA2657255},
	{SG73546, &UA2731473},
	{SG73547, &UA2845981},
	{SG73548, &UA2952516},
	{SG73549, &UA3086371},
	{SG73550, &UA114773},
	{SG73551, &UA157551},
	{SG73552, &UA399866},
	{SG73553, &UA487097},
	{SG73554, &UA1994379},
	{SG73555, &UA2165629},
	{SG73556, &UA2441722},
	{SG73557, &UA2687007},
	{SG73558, &UA2722730},
	{SG73559, &UA2787265},
	{SG73560, &UA3042405},
	{SG73562, &UA428558},
	{SG73563, &UA625067},
	{SG73564, &UA649398},
	{SG73565, &UA2158249},
	{SG73566, &UA2259331},
	{SG73567, &UA2775657},
	{SG73568, &UA2824353},
	{SG73569, &UA2828026},
	{SG73570, &UA2836484},
	{SG73571, &UA2982161},
	{SG73572, &UA3117926},
	{SG73573, &UA94027},
	{SG73574, &UA112254},
	{SG73575, &UA153192},
	{SG73576, &UA271074},
	{SG73577, &UA470982},
	{SG73578, &UA772306},
	{SG73579, &UA2136802},
	{SG73580, &UA2137941},
	{SG73581, &UA2144200},
	{SG73582, &UA2267601},
	{SG73583, &UA2524051},
	{SG73585, &UA92716},
	{SG73586, &UA149322},
	{SG73587, &UA162534},
	{SG73588, &UA196402},
	{SG73589, &UA389321},
	{SG73590, &UA543800},
	{SG73591, &UA652531},
	{SG73592, &UA715450},
	{SG73593, &UA1961958},
	{SG73594, &UA2359701},
	{SG73595, &UA2721566},
	{SG73596, &UA2790163},
	{SG73597, &UA3028033},
	{SG73598, &UA3069347},
	{SG73682, &UA460748},
	{SG73683, &UA656327},
	{SG73684, &UA768713},
	{SG73685, &UA775588},
	{SG73686, &UA803519},
	{SG73687, &UA837218},
	{SG73688, &UA1961008},
	{SG73689, &UA1971245},
	{SG73690, &UA1988975},
	{SG73691, &UA2222381},
	{SG73692, &UA2372582},
	{SG73693, &UA2394961},
	{SG73694, &UA2685149},
	{SG73695, &UA2947822},
	{SG73696, &UA101099},
	{SG73697, &UA270101},
	{SG73698, &UA786273},
	{SG73699, &UA2164083},
	{SG73700, &UA2166540},
	{SG73701, &UA2166649},
	{SG73702, &UA2315079},
	{SG73703, &UA2330667},
	{SG73704, &UA2374704},
	{SG73705, &UA2546383},
	{SG73706, &UA2722526},
	{SG73707, &UA2725129},
	{SG73708, &UA2727933},
	{SG73709, &UA2745291},
	{SG73710, &UA2758789},
	{SG73711, &UA2775858},
	{SG73712, &UA2787282},
	{SG73713, &UA2950505},
	{SG73714, &UA3041863},
	{SG73716, &UA2203804},
	{SG73717, &UA2203859},
	{SG73718, &UA2203915},
	{SG73719, &UA2203990},
	{SG73720, &UA2231775},
	{SG73721, &UA2232320},
	{SG73722, &UA2238339},
	{SG73723, &UA2238614},
	{SG73724, &UA2245123},
	{SG73728, &UA92699},
	{SG73729, &UA97981},
	{SG73730, &UA459860},
	{SG73731, &UA766989},
	{SG73732, &UA1900285},
	{SG73733, &UA1901864},
	{SG73734, &UA2261907},
	{SG73735, &UA2262058},
	{SG73736, &UA2375045},
	{SG73737, &UA2395492},
	{SG73738, &UA2447588},
	{SG73739, &UA2475975},
	{SG73740, &UA2527145},
	{SG73741, &UA2547309},
	{SG73742, &UA2553846},
	{SG73743, &UA2694836},
	{SG73744, &UA2696060},
	{SG73745, &UA2727703},
	{SG73746, &UA2777379},
	{SG73747, &UA2842373},
	{SG73748, &UA2936759},
	{SG73749, &UA3117353},
	{SG73750, &UA2408280},
	{SG73751, &UA2432320},
	{SG73752, &UA2434026},
	{SG73753, &UA2436046},
	{SG73754, &UA2486330},
	{SG73755, &UA2583912},
	{SG73756, &UA2583997},
	{SG73757, &UA2602715},
	{SG73758, &UA2668225},
	{SG73759, &UA2794512},
	{SG73760, &UA138441},
	{SG73761, &UA151983},
	{SG73762, &UA164085},
	{SG73763, &UA439167},
	{SG73764, &UA536746},
	{SG73765, &UA555306},
	{SG73766, &UA1881759},
	{SG73767, &UA1947394},
	{SG73768, &UA2140485},
	{SG73769, &UA94086},
	{SG73770, &UA2273839},
	{SG73773, &UA79838},
	{SG73774, &UA130753},
	{SG73775, &UA151381},
	{SG73776, &UA413360},
	{SG73777, &UA546515},
	{SG73778, &UA698557},
	{SG73779, &UA2362709},
	{SG73780, &UA2362714},
	{SG73781, &UA2426030},
	{SG73785, &UA93507},
	{SG73786, &UA113414},
	{SG73787, &UA687601},
	{SG73788, &UA795955},
	{SG73789, &UA2509458},
	{SG73790, &UA2658875},
	{SG73791, &UA2660686},
	{SG73792, &UA2704563},
	{SG73793, &UA2791762},
	{SG73794, &UA2792811},
	{SG73795, &UA2792899},
	{SG73796, &UA899808},
	{SG73797, &UA1957679},
	{SG73798, &UA2155306},
	{SG73799, &UA2419525},
	{SG73800, &UA2655275},
	{SG73801, &UA170397},
	{SG73802, &UA185636},
	{SG73803, &UA395430},
	{SG73804, &UA495300},
	{SG73805, &UA718322},
	{SG73806, &UA850442},
	{SG73807, &UA2720465},
	{SG73808, &UA92928},
	{SG73809, &UA111824},
	{SG73810, &UA143996},
	{SG73811, &UA492419},
	{SG73812, &UA630906},
	{SG73813, &UA2127266},
	{SG73814, &UA2509650},
	{SG73815, &UA2660071},
	{SG73816, &UA2660760},
	{SG73817, &UA2791908},
	{SG73818, &UA97901},
	{SG73819, &UA118329},
	{SG73820, &UA125120},
	{SG73821, &UA503676},
	{SG73822, &UA653657},
	{SG73823, &UA2741750},
	{SG73824, &UA2715816},
	{SG73825, &UA2718723},
	{SG73826, &UA2796941},
	{SG73827, &UA2796946},
	{SG73828, &UA2865757},
	{SG73829, &UA2875571},
	{SG73830, &UA2959174},
	{SG73832, &UA2959401},
	{SG73834, &UA3010120},
	{SG73835, &UA237388},
	{SG73836, &UA346280},
	{SG73837, &UA603859},
	{SG73838, &UA604456},
	{SG73839, &UA604497},
	{SG73840, &UA654039},
	{SG73841, &UA708034},
	{SG73842, &UA1969750},
	{SG73843, &UA2154360},
	{SG73844, &UA2247098},
	{SG73845, &UA2315731},
	{SG73846, &UA2318076},
	{SG73847, &UA2363059},
	{SG73848, &UA2368788},
	{SG73849, &UA2368871},
	{SG73850, &UA2482523},
	{SG73851, &UA2516673},
	{SG73852, &UA2531505},
	{SG73853, &UA2565389},
	{SG73854, &UA2664237},
	{SG73855, &UA2665602},
	{SG73856, &UA2704461},
	{SG73857, &UA2720540},
	{SG73858, &UA2793694},
	{SG73859, &UA2852797},
	{SG73860, &UA2855560},
	{SG73861, &UA2856930},
	{SG73862, &UA2933690},
	{SG73863, &UA3023772},
	{SG73864, &UA3059085},
	{SG73865, &UA3083819},
	{SG73866, &UA3095875},
	{SG73867, &UA3110480},
	{SG73868, &UA3114649},
	{SG73869, &UA93084},
	{SG73870, &UA128926},
	{SG73871, &UA226776},
	{SG73872, &UA385148},
	{SG73873, &UA808211},
	{SG73874, &UA1957909},
	{SG73875, &UA2153148},
	{SG73876, &UA2157374},
	{SG73877, &UA2166919},
	{SG73878, &UA2338763},
	{SG73879, &UA2774058},
	{SG73880, &UA2824095},
	{SG73881, &UA3020629},
	{SG73882, &UA3027927},
	{SG73884, &UA92927},
	{SG73885, &UA116727},
	{SG73886, &UA702350},
	{SG73887, &UA2418373},
	{SG73888, &UA2509432},
	{SG73889, &UA2509457},
	{SG73890, &UA2597826},
	{SG73891, &UA2661228},
	{SG73892, &UA2729562},
	{SG73893, &UA2791770},
	{SG73894, &UA2791862},
	{SG73895, &UA2792525},
	{SG73896, &UA2792566},
	{SG73897, &UA2792600},
	{SG73898, &UA2793090},
	{SG73899, &UA2849927},
	{SG73900, &UA2850043},
	{SG73901, &UA2850881},
	{SG73902, &UA2977606},
	{SG73903, &UA401611},
	{SG73904, &UA409671},
	{SG73905, &UA428808},
	{SG73906, &UA458321},
	{SG73907, &UA590438},
	{SG73908, &UA591166},
	{SG73909, &UA597006},
	{SG73910, &UA692365},
	{SG73911, &UA865996},
	{SG73912, &UA899053},
	{SG73913, &UA2332935},
	{SG73914, &UA2375884},
	{SG73916, &UA454055},
	{SG73917, &UA605885},
	{SG73918, &UA606282},
	{SG73919, &UA606524},
	{SG73920, &UA609325},
	{SG73921, &UA643693},
	{SG73922, &UA2332444},
	{SG73923, &UA2520290},
	{SG73924, &UA2524323},
	{SG73925, &UA2543653},
	{SG73926, &UA2731214},
	{SG73927, &UA2747802},
	{SG73928, &UA897534},
	{SG73929, &UA897561},
	{SG73930, &UA897623},
	{SG73931, &UA897865},
	{SG73932, &UA897997},
	{SG73933, &UA898100},
	{SG73934, &UA899452},
	{SG73935, &UA1985551},
	{SG73936, &UA2495155},
	{SG73937, &UA2574188},
	{SG73938, &UA2726886},
	{SG73939, &UA911482},
	{SG73940, &UA911509},
	{SG73941, &UA912677},
	{SG73942, &UA924026},
	{SG73943, &UA1964545},
	{SG73944, &UA2154983},
	{SG73945, &UA2159008},
	{SG73946, &UA2220230},
	{SG73947, &UA2545805},
	{SG73948, &UA2573494},
	{SG73949, &UA2786931},
	{SG73950, &UA2994946},
	{SG73952, &UA208036},
	{SG73953, &UA242073},
	{SG73954, &UA375734},
	{SG73955, &UA421054},
	{SG73956, &UA704011},
	{SG73957, &UA708387},
	{SG73958, &UA98156},
	{SG73959, &UA386929},
	{SG73960, &UA2148688},
	{SG73961, &UA2335492},
	{SG73962, &UA2527346},
	{SG73963, &UA2597452},
	{SG73964, &UA2693738},
	{SG73965, &UA2773972},
	{SG73966, &UA3099295},
	{SG73967, &UA503941},
	{SG73968, &UA664959},
	{SG73969, &UA836933},
	{SG73970, &UA1951376},
	{SG73971, &UA1951391},
	{SG73972, &UA1954985},
	{SG73973, &UA1960062},
	{SG73974, &UA1964937},
	{SG73975, &UA1984913},
	{SG73976, &UA2222587},
	{SG73977, &UA2243036},
	{SG73978, &UA2247787},
	{SG73979, &UA2257995},
	{SG73980, &UA2334857},
	{SG73981, &UA2374426},
	{SG73982, &UA2374972},
	{SG73983, &UA2376046},
	{SG73984, &UA2394547},
	{SG73985, &UA2395336},
	{SG73986, &UA2413247},
	{SG73987, &UA2413304},
	{SG73988, &UA2413682},
	{SG73989, &UA2413732},
	{SG73990, &UA2413806},
	{SG73991, &UA3098975},
	{SG73992, &UA2677406},
	{SG73993, &UA2679635},
	{SG73994, &UA2901013},
	{SG73995, &UA2946027},
	{SG73996, &UA2946073},
	{SG73997, &UA2964036},
	{SG73998, &UA2965981},
	{SG73999, &UA2966835},
	{SG74000, &UA3013779},
	{SG74001, &UA3014708},
	{SG74002, &UA3016749},
	{SG74003, &UA3019515},
	{SG74007, &UA344202},
	{SG74008, &UA576322},
	{SG74009, &UA628868},
	{SG74010, &UA664852},
	{SG74011, &UA690314},
	{SG74012, &UA782314},
	{SG74013, &UA1907941},
	{SG74014, &UA2148894},
	{SG74015, &UA2161825},
	{SG74016, &UA2162630},
	{SG74017, &UA2330649},
	{SG74018, &UA2423108},
	{SG74019, &UA2517719},
	{SG74020, &UA2528649},
	{SG74021, &UA2842451},
	{SG74022, &UA3070440},
	{SG74028, &UA897851},
	{SG74030, &UA897882},
	{SG74031, &UA898098},
	{SG74032, &UA898446},
	{SG74033, &UA900838},
	{SG74034, &UA900958},
	{SG74035, &UA901540},
	{SG74036, &UA1968232},
	{SG74037, &UA1977139},
	{SG74038, &UA2166076},
	{SG74039, &UA2334366},
	{SG74040, &UA2643341},
	{SG74041, &UA2766987},
	{SG74043, &UA127438},
	{SG74044, &UA271683},
	{SG74045, &UA444808},
	{SG74046, &UA791832},
	{SG74047, &UA822914},
	{SG74048, &UA2509376},
	{SG74049, &UA2852000},
	{SG74050, &UA105771},
	{SG74051, &UA262743},
	{SG74052, &UA264302},
	{SG74053, &UA414060},
	{SG74054, &UA505457},
	{SG74055, &UA515053},
	{SG74056, &UA2210124},
	{SG74057, &UA93343},
	{SG74058, &UA93907},
	{SG74059, &UA106785},
	{SG74060, &UA117422},
	{SG74061, &UA138324},
	{SG74062, &UA141352},
	{SG74063, &UA154565},
	{SG74064, &UA176058},
	{SG74065, &UA261044},
	{SG74066, &UA270570},
	{SG74067, &UA382398},
	{SG74068, &UA392954},
	{SG74069, &UA414692},
	{SG74070, &UA445789},
	{SG74071, &UA461330},
	{SG74072, &UA511543},
	{SG74073, &UA538124},
	{SG74074, &UA552008},
	{SG74075, &UA559163},
	{SG74076, &UA692877},
	{SG74077, &UA713835},
	{SG74078, &UA728159},
	{SG74079, &UA758502},
	{SG74080, &UA779848},
	{SG74081, &UA855298},
	{SG74082, &UA2126627},
	{SG74083, &UA2209431},
	{SG74084, &UA2265344},
	{SG74085, &UA2303944},
	{SG74086, &UA2989210},
	{SG74087, &UA3087580},
	{SG74088, &UA353591},
	{SG74089, &UA357841},
	{SG74090, &UA366463},
	{SG74091, &UA397518},
	{SG74092, &UA397882},
	{SG74093, &UA582399},
	{SG74094, &UA585266},
	{SG74095, &UA843498},
	{SG74096, &UA199127},
	{SG74097, &UA320418},
	{SG74098, &UA326144},
	{SG74099, &UA426977},
	{SG74100, &UA737906},
	{SG74101, &UA758812},
	{SG74102, &UA2211416},
	{SG74103, &UA2681934},
	{SG74104, &UA3045677},
	{SG74105, &UA97133},
	{SG74106, &UA818704},
	{SG74107, &UA829223},
	{SG74108, &UA1989170},
	{SG74109, &UA2158045},
	{SG74110, &UA2165659},
	{SG74111, &UA2166210},
	{SG74112, &UA2222589},
	{SG74113, &UA2574352},
	{SG74114, &UA2574638},
	{SG74115, &UA2641358},
	{SG74116, &UA2685467},
	{SG74117, &UA2692844},
	{SG74118, &UA2722509},
	{SG74119, &UA2812271},
	{SG74120, &UA2950101},
	{SG74121, &UA2951033},
	{SG74122, &UA1948425},
	{SG74123, &UA1955073},
	{SG74124, &UA1993167},
	{SG74125, &UA2155329},
	{SG74126, &UA2162410},
	{SG74127, &UA2167790},
	{SG74128, &UA2440464},
	{SG74129, &UA2475116},
	{SG74130, &UA2822427},
	{SG74131, &UA2834566},
	{SG74132, &UA2950302},
	{SG74133, &UA138777},
	{SG74134, &UA417059},
	{SG74135, &UA469062},
	{SG74136, &UA528583},
	{SG74137, &UA680327},
	{SG74138, &UA696409},
	{SG74139, &UA706143},
	{SG74140, &UA817071},
	{SG74141, &UA1858185},
	{SG74142, &UA1869959},
	{SG74143, &UA1941860},
	{SG74144, &UA1947050},
	{SG74145, &UA1960064},
	{SG74146, &UA1993799},
	{SG74147, &UA2150421},
	{SG74148, &UA2152631},
	{SG74149, &UA2154343},
	{SG74150, &UA2157535},
	{SG74151, &UA2166470},
	{SG74152, &UA2218998},
	{SG74153, &UA2256763},
	{SG74154, &UA2309998},
	{SG74155, &UA2443606},
	{SG74156, &UA2494270},
	{SG74157, &UA2520353},
	{SG74158, &UA2553953},
	{SG74159, &UA2555060},
	{SG74160, &UA2594817},
	{SG74161, &UA2686966},
	{SG74162, &UA2723497},
	{SG74163, &UA2743314},
	{SG74164, &UA2746889},
	{SG74165, &UA2786909},
	{SG74166, &UA2787743},
	{SG74167, &UA2790196},
	{SG74168, &UA2826232},
	{SG74169, &UA2830140},
	{SG74170, &UA2837723},
	{SG74171, &UA2840895},
	{SG74172, &UA2933476},
	{SG74173, &UA2933801},
	{SG74174, &UA2949704},
	{SG74175, &UA2950234},
	{SG74176, &UA2950428},
	{SG74177, &UA2980869},
	{SG74178, &UA2981511},
	{SG74179, &UA3116635},
	{SG74180, &UA3118082},
	{SG74181, &UA3118393},
	{SG74183, &UA94190},
	{SG74184, &UA2659005},
	{SG74185, &UA2792797},
	{SG74186, &UA152045},
	{SG74187, &UA491952},
	{SG74188, &UA2221075},
	{SG74189, &UA2573316},
	{SG74190, &UA2660630},
	{SG74191, &UA2808460},
	{SG74192, &UA3098524},
	{SG74193, &UA93048},
	{SG74194, &UA169003},
	{SG74195, &UA50066},
	{SG74196, &UA104540},
	{SG74197, &UA114581},
	{SG74198, &UA1886572},
	{SG74199, &UA2143265},
	{SG74200, &UA2296564},
	{SG74201, &UA2471720},
	{SG74202, &UA2791829},
	{SG74203, &UA124098},
	{SG74204, &UA135396},
	{SG74205, &UA167268},
	{SG74206, &UA275538},
	{SG74207, &UA813443},
	{SG74208, &UA2805313},
	{SG74209, &UA2937977},
	{SG74210, &UA2984875},
	{SG74211, &UA573742},
	{SG74212, &UA1984664},
	{SG74213, &UA1993764},
	{SG74214, &UA2154196},
	{SG74215, &UA2158791},
	{SG74216, &UA2761776},
	{SG74217, &UA2775876},
	{SG74218, &UA53631},
	{SG74219, &UA99075},
	{SG74220, &UA123356},
	{SG74221, &UA2509626},
	{SG74222, &UA2580052},
	{SG74223, &UA2660247},
	{SG74224, &UA2791715},
	{SG74225, &UA2949960},
	{SG74226, &UA146527},
	{SG74227, &UA190023},
	{SG74228, &UA241774},
	{SG74229, &UA242077},
	{SG74231, &UA315910},
	{SG74232, &UA317337},
	{SG74234, &UA349581},
	{SG74235, &UA408297},
	{SG74236, &UA497510},
	{SG74237, &UA538553},
	{SG74238, &UA563774},
	{SG74239, &UA571741},
	{SG74240, &UA613787},
	{SG74241, &UA613899},
	{SG74242, &UA629870},
	{SG74243, &UA662747},
	{SG74244, &UA704473},
	{SG74245, &UA736116},
	{SG74246, &UA742906},
	{SG74247, &UA851886},
	{SG74248, &UA924875},
	{SG74249, &UA925180},
	{SG74250, &UA1842394},
	{SG74251, &UA1854685},
	{SG74252, &UA1996956},
	{SG74253, &UA2008977},
	{SG74254, &UA2041329},
	{SG74255, &UA2144524},
	{SG74256, &UA2205104},
	{SG74257, &UA2205106},
	{SG74258, &UA2226054},
	{SG74259, &UA2232824},
	{SG74260, &UA2337271},
	{SG74261, &UA2338659},
	{SG74262, &UA2370565},
	{SG74263, &UA2384369},
	{SG74264, &UA2435935},
	{SG74265, &UA2435944},
	{SG74266, &UA2436150},
	{SG74267, &UA2492761},
	{SG74268, &UA2528646},
	{SG74269, &UA2528665},
	{SG74270, &UA2579429},
	{SG74271, &UA2596257},
	{SG74272, &UA2675878},
	{SG74273, &UA2676201},
	{SG74274, &UA2676955},
	{SG74275, &UA2679970},
	{SG74276, &UA2795311},
	{SG74277, &UA2799192},
	{SG74278, &UA2799287},
	{SG74279, &UA2799307},
	{SG74280, &UA2835479},
	{SG74281, &UA2842357},
	{SG74282, &UA2869438},
	{SG74283, &UA2952309},
	{SG74284, &UA2954654},
	{SG74285, &UA2957025},
	{SG74286, &UA2992458},
	{SG74287, &UA3023806},
	{SG74288, &UA3023817},
	{SG74289, &UA3023820},
	{SG74290, &UA3028161},
	{SG74291, &UA3071163},
	{SG74292, &UA3099472},
	{SG74293, &UA3101870},
	{SG74297, &UA50116},
	{SG74298, &UA521288},
	{SG74299, &UA2849889},
	{SG74300, &UA378709},
	{SG74301, &UA516663},
	{SG74302, &UA551401},
	{SG74303, &UA1841031},
	{SG74304, &UA2244155},
	{SG74305, &UA99372},
	{SG74306, &UA104805},
	{SG74307, &UA117999},
	{SG74308, &UA179288},
	{SG74309, &UA257537},
	{SG74310, &UA655599},
	{SG74311, &UA691868},
	{SG74312, &UA698261},
	{SG74313, &UA728367},
	{SG74314, &UA808734},
	{SG74315, &UA2216416},
	{SG74316, &UA2509883},
	{SG74317, &UA2580074},
	{SG74318, &UA2658860},
	{SG74319, &UA2658927},
	{SG74320, &UA2791673},
	{SG74321, &UA2791769},
	{SG74322, &UA2791828},
	{SG74323, &UA353166},
	{SG74324, &UA357618},
	{SG74325, &UA366264},
	{SG74326, &UA368185},
	{SG74327, &UA397857},
	{SG74328, &UA437320},
	{SG74329, &UA586729},
	{SG74330, &UA587476},
	{SG74331, &UA754044},
	{SG74332, &UA828763},
	{SG74333, &UA850704},
	{SG74334, &UA2440904},
	{SG74335, &UA2474308},
	{SG74336, &UA2599808},
	{SG74337, &UA2747595},
	{SG74338, &UA2845336},
	{SG74339, &UA2989784},
	{SG74340, &UA239128},
	{SG74341, &UA255761},
	{SG74342, &UA329715},
	{SG74343, &UA436980},
	{SG74344, &UA437044},
	{SG74345, &UA572391},
	{SG74346, &UA574553},
	{SG74347, &UA882955},
	{SG74348, &UA1897604},
	{SG74349, &UA2308915},
	{SG74350, &UA2353875},
	{SG74352, &UA1983664},
	{SG74353, &UA1990877},
	{SG74354, &UA2167259},
	{SG74355, &UA2167653},
	{SG74356, &UA2813815},
	{SG74357, &UA2817000},
	{SG74358, &UA720487},
	{SG74359, &UA789296},
	{SG74360, &UA1987345},
	{SG74361, &UA593573},
	{SG74364, &UA629727},
	{SG74365, &UA658799},
	{SG74366, &UA873271},
	{SG74367, &UA2163880},
	{SG74368, &UA2727140},
	{SG74369, &UA2743591},
	{SG74370, &UA2831172},
	{SG74371, &UA2950016},
	{SG74372, &UA198114},
	{SG74373, &UA218583},
	{SG74374, &UA327763},
	{SG74375, &UA637160},
	{SG74376, &UA2291564},
	{SG74382, &UA2433594},
	{SG74383, &UA733236},
	{SG74384, &UA1951570},
	{SG74385, &UA1952655},
	{SG74386, &UA1962979},
	{SG74387, &UA1969676},
	{SG74388, &UA2242681},
	{SG74389, &UA2253412},
	{SG74390, &UA2364689},
	{SG74391, &UA2395302},
	{SG74392, &UA2412839},
	{SG74393, &UA2497039},
	{SG74394, &UA2547025},
	{SG74395, &UA2574776},
	{SG74396, &UA2815805},
	{SG74397, &UA2817858},
	{SG74398, &UA2933710},
	{SG74399, &UA2934971},
	{SG74400, &UA2161430},
	{SG74401, &UA2205979},
	{SG74402, &UA2206137},
	{SG74403, &UA2229595},
	{SG74404, &UA2259669},
	{SG74405, &UA2362031},
	{SG74406, &UA2382064},
	{SG74407, &UA2410454},
	{SG74408, &UA2500276},
	{SG74409, &UA2790209},
	{SG74410, &UA3118084},
	{SG74411, &UA2509637},
	{SG74412, &UA2531840},
	{SG74413, &UA2580045},
	{SG74414, &UA2658838},
	{SG74415, &UA2659569},
	{SG74416, &UA2700271},
	{SG74417, &UA2730388},
	{SG74418, &UA2791844},
	{SG74419, &UA2791884},
	{SG74420, &UA2792885},
	{SG74421, &UA2556773},
	{SG74422, &UA2660029},
	{SG74423, &UA2714668},
	{SG74424, &UA2205540},
	{SG74425, &UA2205971},
	{SG74426, &UA2651643},
	{SG74427, &UA2981949},
	{SG74428, &UA3117807},
	{SG74429, &UA3118010},
	{SG74431, &UA924170},
	{SG74433, &UA100053},
	{SG74434, &UA109432},
	{SG74435, &UA123599},
	{SG74436, &UA260794},
	{SG74437, &UA261829},
	{SG74438, &UA855825},
	{SG74439, &UA1859856},
	{SG74440, &UA2362551},
	{SG74441, &UA2746962},
	{SG74442, &UA2774428},
	{SG74443, &UA2985907},
	{SG74446, &UA316940},
	{SG74447, &UA502123},
	{SG74448, &UA518251},
	{SG74449, &UA887123},
	{SG74450, &UA1872958},
	{SG74451, &UA1990450},
	{SG74452, &UA1992234},
	{SG74453, &UA2165875},
	{SG74454, &UA2168020},
	{SG74455, &UA2221925},
	{SG74456, &UA2321640},
	{SG74457, &UA2408467},
	{SG74458, &UA2408617},
	{SG74459, &UA2447551},
	{SG74460, &UA2473512},
	{SG74461, &UA2574798},
	{SG74462, &UA2686047},
	{SG74463, &UA2686102},
	{SG74464, &UA2742945},
	{SG74465, &UA2747406},
	{SG74466, &UA2775718},
	{SG74467, &UA2787362},
	{SG74468, &UA2788687},
	{SG74469, &UA2816924},
	{SG74470, &UA2836444},
	{SG74471, &UA2839416},
	{SG74472, &UA2889111},
	{SG74473, &UA2949954},
	{SG74474, &UA2950151},
	{SG74475, &UA2980988},
	{SG74476, &UA3047153},
	{SG74477, &UA401373},
	{SG74478, &UA401378},
	{SG74479, &UA428728},
	{SG74480, &UA589517},
	{SG74481, &UA590496},
	{SG74482, &UA900862},
	{SG74483, &UA2155364},
	{SG74484, &UA2314914},
	{SG74485, &UA2333221},
	{SG74486, &UA2495428},
	{SG74487, &UA2809293},
	{SG74488, &UA2861762},
	{SG74489, &UA3047813},
	{SG74490, &UA91108},
	{SG74491, &UA252119},
	{SG74492, &UA252304},
	{SG74493, &UA257424},
	{SG74494, &UA686989},
	{SG74495, &UA923372},
	{SG74496, &UA2245920},
	{SG74497, &UA2247364},
	{SG74498, &UA2473325},
	{SG74499, &UA2721035},
	{SG74501, &UA98853},
	{SG74502, &UA2157614},
	{SG74503, &UA2163984},
	{SG74504, &UA2165286},
	{SG74505, &UA2166006},
	{SG74506, &UA2166431},
	{SG74507, &UA2167192},
	{SG74508, &UA2408621},
	{SG74509, &UA2686649},
	{SG74510, &UA2725624},
	{SG74511, &UA2743566},
	{SG74512, &UA2775837},
	{SG74513, &UA2775980},
	{SG74514, &UA2776834},
	{SG74515, &UA2788265},
	{SG74516, &UA2824876},
	{SG74517, &UA2827577},
	{SG74518, &UA2838272},
	{SG74519, &UA2980264},
	{SG74520, &UA2981965},
	{SG74521, &UA2982793},
	{SG74522, &UA2983251},
	{SG74523, &UA2901022},
	{SG74524, &UA2945688},
	{SG74525, &UA2963715},
	{SG74526, &UA2965112},
	{SG74527, &UA164136},
	{SG74528, &UA672157},
	{SG74529, &UA739568},
	{SG74530, &UA882815},
	{SG74531, &UA896868},
	{SG74532, &UA2639383},
	{SG74533, &UA2773443},
	{SG74534, &UA1961918},
	{SG74535, &UA1964614},
	{SG74536, &UA2339272},
	{SG74537, &UA2364563},
	{SG74538, &UA2395451},
	{SG74539, &UA2413031},
	{SG74541, &UA180492},
	{SG74542, &UA308538},
	{SG74543, &UA632199},
	{SG74544, &UA1874200},
	{SG74545, &UA2540821},
	{SG74546, &UA3040805},
	{SG74547, &UA157921},
	{SG74548, &UA170416},
	{SG74549, &UA512047},
	{SG74550, &UA623206},
	{SG74551, &UA916106},
	{SG74552, &UA1874656},
	{SG74553, &UA2274328},
	{SG74554, &UA2286930},
	{SG74555, &UA2502468},
	{SG74556, &UA2545003},
	{SG74557, &UA2658785},
	{SG74558, &UA2745007},
	{SG74559, &UA1953585},
	{SG74560, &UA1954696},
	{SG74561, &UA2328620},
	{SG74562, &UA2375259},
	{SG74563, &UA2395641},
	{SG74564, &UA2444856},
	{SG74565, &UA453711},
	{SG74566, &UA454056},
	{SG74567, &UA605628},
	{SG74568, &UA605659},
	{SG74569, &UA606288},
	{SG74570, &UA608852},
	{SG74571, &UA694267},
	{SG74572, &UA830982},
	{SG74573, &UA2331969},
	{SG74574, &UA2364719},
	{SG74575, &UA3071320},
	{SG74576, &UA3071512},
	{SG74577, &UA117688},
	{SG74578, &UA338720},
	{SG74579, &UA792821},
	{SG74580, &UA1960173},
	{SG74581, &UA1988417},
	{SG74582, &UA1994149},
	{SG74583, &UA2338007},
	{SG74584, &UA2702751},
	{SG74585, &UA2849791},
	{SG74586, &UA50319},
	{SG74587, &UA2791891},
	{SG74588, &UA3034287},
	{SG74590, &UA213186},
	{SG74591, &UA236558},
	{SG74592, &UA335329},
	{SG74593, &UA2218204},
	{SG74594, &UA2297839},
	{SG74595, &UA2473419},
	{SG74596, &UA2641951},
	{SG74597, &UA2986413},
	{SG74598, &UA152739},
	{SG74599, &UA523678},
	{SG74600, &UA529916},
	{SG74601, &UA630865},
	{SG74602, &UA747022},
	{SG74603, &UA785540},
	{SG74604, &UA833968},
	{SG74605, &UA2217151},
	{SG74606, &UA2277281},
	{SG74607, &UA2642480},
	{SG74608, &UA2727514},
	{SG74609, &UA2816891},
	{SG74610, &UA2834991},
	{SG74612, &UA3025909},
	{SG74613, &UA3026210},
	{SG74614, &UA3027129},
	{SG74615, &UA3030287},
	{SG74616, &UA911089},
	{SG74617, &UA912391},
	{SG74618, &UA912519},
	{SG74619, &UA912785},
	{SG74620, &UA912888},
	{SG74621, &UA924053},
	{SG74622, &UA1950597},
	{SG74623, &UA2217796},
	{SG74624, &UA2338765},
	{SG74625, &UA2360231},
	{SG74626, &UA2494610},
	{SG74627, &UA2814366},
	{SG74628, &UA2889273},
	{SG74629, &UA2992072},
	{SG74642, &UA2873997},
	{SG74643, &UA2876697},
	{SG74644, &UA2883815},
	{SG74645, &UA192316},
	{SG74646, &UA314727},
	{SG74647, &UA535292},
	{SG74648, &UA2245677},
	{SG74649, &UA2363986},
	{SG74650, &UA2424724},
	{SG74651, &UA2595102},
	{SG74659, &UA190999},
	{SG74660, &UA316148},
	{SG74661, &UA557767},
	{SG74662, &UA732465},
	{SG74663, &UA2472948},
	{SG74664, &UA2517416},
	{SG74672, &UA79845},
	{SG74673, &UA835204},
	{SG74674, &UA2509628},
	{SG74676, &UA2581666},
	{SG74677, &UA2792230},
	{SG74681, &UA2509384},
	{SG74682, &UA2658841},
	{SG74683, &UA2791723},
	{SG74684, &UA2791871},
	{SG74685, &UA2849346},
	{SG74686, &UA144168},
	{SG74687, &UA571811},
	{SG74688, &UA1850524},
	{SG74689, &UA2509382},
	{SG74690, &UA2658767},
	{SG74691, &UA2658945},
	{SG74692, &UA2775285},
	{SG74693, &UA2791688},
	{SG74694, &UA2791873},
	{SG74695, &UA2509627},
	{SG74696, &UA2580053},
	{SG74697, &UA2581069},
	{SG74698, &UA2658783},
	{SG74699, &UA2792686},
	{SG74700, &UA2510259},
	{SG74701, &UA2700364},
	{SG74702, &UA2791864},
	{SG74703, &UA2792331},
	{SG74704, &UA2792545},
	{SG74705, &UA2509252},
	{SG74706, &UA2580078},
	{SG74707, &UA2700277},
	{SG74708, &UA2791906},
	{SG74709, &UA2851095},
	{SG74710, &UA710893},
	{SG74711, &UA2792796},
	{SG74713, &UA457796},
	{SG74714, &UA643679},
	{SG74715, &UA818681},
	{SG74716, &UA1967134},
	{SG74717, &UA2162851},
	{SG74718, &UA2163174},
	{SG74719, &UA2326562},
	{SG74720, &UA2330397},
	{SG74721, &UA2332762},
	{SG74722, &UA2395861},
	{SG74723, &UA2413186},
	{SG74724, &UA2554320},
	{SG74725, &UA2653165},
	{SG74726, &UA2695957},
	{SG74727, &UA2745903},
	{SG74728, &UA2822447},
	{SG74729, &UA2833229},
	{SG74730, &UA2982481},
	{SG74731, &UA2982667},
	{SG74732, &UA3098865},
	{SG74733, &UA2216397},
	{SG74734, &UA2509392},
	{SG74735, &UA2661532},
	{SG74736, &UA2792612},
	{SG74737, &UA2792645},
	{SG74738, &UA2793093},
	{SG74739, &UA2675122},
	{SG74740, &UA2679562},
	{SG74741, &UA2709882},
	{SG74742, &UA2716187},
	{SG74747, &UA2509455},
	{SG74749, &UA2580054},
	{SG74750, &UA2581303},
	{SG74751, &UA2792786},
	{SG74752, &UA2792803},
	{SG74753, &UA106308},
	{SG74754, &UA121036},
	{SG74755, &UA253772},
	{SG74756, &UA378837},
	{SG74757, &UA560277},
	{SG74758, &UA698122},
	{SG74759, &UA702321},
	{SG74760, &UA2817189},
	{SG74761, &UA97920},
	{SG74762, &UA100884},
	{SG74763, &UA372473},
	{SG74764, &UA465439},
	{SG74765, &UA729888},
	{SG74766, &UA744493},
	{SG74767, &UA1939345},
	{SG74768, &UA2729587},
	{SG74769, &UA2792822},
	{SG74773, &UA2509397},
	{SG74776, &UA2580059},
	{SG74778, &UA2700259},
	{SG74779, &UA2729546},
	{SG74780, &UA2792548},
	{SG74790, &UA1994810},
	{SG74791, &UA2159021},
	{SG74792, &UA2159613},
	{SG74793, &UA2162411},
	{SG74794, &UA2165786},
	{SG74795, &UA2166606},
	{SG74796, &UA2723828},
	{SG74797, &UA2744621},
	{SG74798, &UA2787140},
	{SG74799, &UA2787407},
	{SG74800, &UA2789764},
	{SG74801, &UA2790270},
	{SG74802, &UA2815014},
	{SG74803, &UA2821424},
	{SG74804, &UA2842508},
	{SG74805, &UA2935249},
	{SG74806, &UA2949932},
	{SG74807, &UA2980151},
	{SG74809, &UA2980204},
	{SG74810, &UA3085419},
	{SG74834, &UA249978},
	{SG74835, &UA250167},
	{SG74836, &UA250171},
	{SG74837, &UA437201},
	{SG74838, &UA574595},
	{SG74839, &UA1904775},
	{SG74840, &UA2541230},
	{SG74841, &UA92638},
	{SG74842, &UA94703},
	{SG74843, &UA141171},
	{SG74844, &UA141275},
	{SG74845, &UA2509357},
	{SG74846, &UA2580072},
	{SG74847, &UA2658759},
	{SG74848, &UA2792546},
	{SG74849, &UA145084},
	{SG74850, &UA237471},
	{SG74851, &UA291802},
	{SG74852, &UA435690},
	{SG74853, &UA575484},
	{SG74854, &UA702450},
	{SG74855, &UA1858720},
	{SG74856, &UA2128209},
	{SG74857, &UA2129550},
	{SG74858, &UA2210403},
	{SG74859, &UA2671315},
	{SG74865, &UA1994536},
	{SG74866, &UA2162567},
	{SG74867, &UA2164658},
	{SG74868, &UA2166802},
	{SG74869, &UA2775608},
	{SG74870, &UA2775632},
	{SG74871, &UA2787146},
	{SG74872, &UA2788548},
	{SG74873, &UA2950153},
	{SG74874, &UA2950252},
	{SG74875, &UA2950258},
	{SG74876, &UA2950486},
	{SG74877, &UA118847},
	{SG74878, &UA122828},
	{SG74881, &UA197090},
	{SG74882, &UA401024},
	{SG74883, &UA570746},
	{SG74884, &UA726477},
	{SG74885, &UA2272038},
	{SG74886, &UA2479556},
	{SG74887, &UA2668462},
	{SG74888, &UA2989387},
	{SG74893, &UA2191004},
	{SG74894, &UA2566429},
	{SG74895, &UA2749950},
	{SG74899, &UA484638},
	{SG74900, &UA639074},
	{SG74901, &UA711800},
	{SG74902, &UA833930},
	{SG74903, &UA1952156},
	{SG74904, &UA2221235},
	{SG74905, &UA2393372},
	{SG74906, &UA2409456},
	{SG74907, &UA2567989},
	{SG74908, &UA2568840},
	{SG74909, &UA2600299},
	{SG74910, &UA2606270},
	{SG74914, &UA142710},
	{SG74915, &UA2509424},
	{SG74916, &UA2509913},
	{SG74917, &UA2658926},
	{SG74918, &UA2851073},
	{SG74933, &UA2481186},
	{SG74934, &UA2502450},
	{SG74935, &UA2509366},
	{SG74936, &UA2658792},
	{SG74937, &UA2660231},
	{SG74938, &UA2791648},
	{SG74939, &UA2791794},
	{SG74940, &UA2849345},
	{SG74945, &UA2489240},
	{SG74946, &UA2492451},
	{SG74947, &UA2508468},
	{SG74948, &UA2512323},
	{SG74949, &UA2797246},
	{SG74950, &UA2870119},
	{SG74951, &UA3088928},
	{SG74952, &UA313010},
	{SG74953, &UA514107},
	{SG74954, &UA539939},
	{SG74955, &UA561758},
	{SG74956, &UA628646},
	{SG74957, &UA658182},
	{SG74958, &UA680473},
	{SG74959, &UA723692},
	{SG74960, &UA1900545},
	{SG74961, &UA1996857},
	{SG74962, &UA2190117},
	{SG74963, &UA2205108},
	{SG74964, &UA2264635},
	{SG74965, &UA2297255},
	{SG74966, &UA2409081},
	{SG74967, &UA2409168},
	{SG74968, &UA2436017},
	{SG74969, &UA2495991},
	{SG74970, &UA2541907},
	{SG74971, &UA2640495},
	{SG74972, &UA2716047},
	{SG74973, &UA2939377},
	{SG74974, &UA3024133},
	{SG74986, &UA105524},
	{SG74987, &UA260735},
	{SG74988, &UA2791855},
	{SG74993, &UA372249},
	{SG74994, &UA399180},
	{SG74995, &UA545112},
	{SG74996, &UA649194},
	{SG74997, &UA2439942},
	{SG74998, &UA2408386},
	{SG74999, &UA2432752},
	{SG75000, &UA2433795},
	{SG75001, &UA2437253},
	{SG75003, &UA2709228},
	{SG75004, &UA2710687},
	{SG75005, &UA2753384},
	{SG75006, &UA2753392},
	{SG75007, &UA2784076},
	{SG75008, &UA2784890},
	{SG75009, &UA2877563},
	{SG75010, &UA2880358},
	{SG75020, &UA2015627},
	{SG75021, &UA2015635},
	{SG75022, &UA2041363},
	{SG75023, &UA2042005},
	{SG75024, &UA2178145},
	{SG75025, &UA2200772},
	{SG75026, &UA2204922},
	{SG75027, &UA2651022},
	{SG75028, &UA2689832},
	{SG75029, &UA2583606},
	{SG75030, &UA2602262},
	{SG75031, &UA2669041},
	{SG75032, &UA696320},
	{SG75033, &UA2218631},
	{SG75034, &UA2333893},
	{SG75037, &UA2792848},
	{SG75046, &UA2737636},
	{SG75047, &UA2948462},
	{SG75048, &UA2956717},
	{SG75050, &UA2577846},
	{SG75051, &UA2646904},
	{SG75052, &UA2653366},
	{SG75053, &UA2694375},
	{SG75054, &UA2950848},
	{SG75055, &UA2740576},
	{SG75056, &UA2752839},
	{SG75057, &UA2769424},
	{SG75058, &UA2769663},
	{SG75059, &UA2796788},
	{SG75060, &UA2937381},
	{SG75061, &UA2232991},
	{SG75062, &UA2356479},
	{SG75063, &UA2356480},
	{SG75064, &UA2362695},
	{SG75065, &UA2387598},
	{SG75066, &UA2406933},
	{SG75067, &UA2407326},
	{SG75068, &UA2429729},
	{SG75069, &UA2459983},
	{SG75070, &UA2839009},
	{SG75071, &UA2167799},
	{SG75072, &UA897343},
	{SG75073, &UA897388},
	{SG75074, &UA897759},
	{SG75075, &UA898780},
	{SG75076, &UA903930},
	{SG75077, &UA2163713},
	{SG75078, &UA2497211},
	{SG75082, &UA1873892},
	{SG75083, &UA1994535},
	{SG75084, &UA2165922},
	{SG75085, &UA2788373},
	{SG75086, &UA2789196},
	{SG75087, &UA2950123},
	{SG75088, &UA2950233},
	{SG75096, &UA92389},
	{SG75097, &UA432799},
	{SG75098, &UA462400},
	{SG75099, &UA633431},
	{SG75100, &UA857803},
	{SG75101, &UA2208049},
	{SG75102, &UA2222497},
	{SG75103, &UA2595684},
	{SG75109, &UA2221213},
	{SG75110, &UA2360140},
	{SG75111, &UA2425870},
	{SG75112, &UA3041847},
	{SG75113, &UA3118266},
	{SG75116, &UA625726},
	{SG75117, &UA634050},
	{SG75119, &UA2161092},
	{SG75120, &UA2167310},
	{SG75121, &UA2775597},
	{SG75122, &UA2787370},
	{SG75123, &UA2950214},
	{SG75124, &UA2950353},
	{SG75126, &UA2203472},
	{SG75127, &UA79835},
	{SG75128, &UA464937},
	{SG75129, &UA2509912},
	{SG75130, &UA2792676},
	{SG75133, &UA244027},
	{SG75134, &UA313842},
	{SG75135, &UA415130},
	{SG75136, &UA484429},
	{SG75137, &UA486114},
	{SG75138, &UA614335},
	{SG75139, &UA635716},
	{SG75140, &UA685081},
	{SG75141, &UA720927},
	{SG75142, &UA735127},
	{SG75143, &UA835577},
	{SG75144, &UA858778},
	{SG75145, &UA924880},
	{SG75146, &UA2008975},
	{SG75147, &UA2144710},
	{SG75148, &UA2168189},
	{SG75149, &UA2205250},
	{SG75150, &UA2234295},
	{SG75151, &UA2344760},
	{SG75152, &UA2382501},
	{SG75153, &UA2429052},
	{SG75154, &UA2435943},
	{SG75155, &UA2516940},
	{SG75158, &UA2676711},
	{SG75160, &UA2780941},
	{SG75162, &UA2799309},
	{SG75163, &UA2837959},
	{SG75164, &UA2959784},
	{SG75165, &UA3028278},
	{SG75172, &UA1956262},
	{SG75173, &UA2154763},
	{SG75174, &UA2687063},
	{SG75175, &UA2722636},
	{SG75176, &UA2822269},
	{SG75177, &UA2950004},
	{SG75178, &UA2155400},
	{SG75179, &UA2163576},
	{SG75180, &UA2823203},
	{SG75181, &UA2839262},
	{SG75182, &UA2841811},
	{SG75183, &UA2950089},
	{SG75193, &UA2797713},
	{SG75194, &UA2877266},
	{SG75195, &UA1953779},
	{SG75196, &UA2167918},
	{SG75197, &UA2777167},
	{SG75198, &UA2789073},
	{SG75199, &UA2933661},
	{SG75200, &UA2981452},
	{SG75208, &UA2970466},
	{SG75209, &UA2971324},
	{SG75210, &UA3015871},
	{SG75211, &UA3021440},
	{SG75215, &UA189140},
	{SG75216, &UA253770},
	{SG75217, &UA2509427},
	{SG75218, &UA2791861},
	{SG75224, &UA1869751},
	{SG75225, &UA1874455},
	{SG75226, &UA2373578},
	{SG75227, &UA2443297},
	{SG75228, &UA2815374},
	{SG75240, &UA126330},
	{SG75241, &UA127737},
	{SG75242, &UA481364},
	{SG75244, &UA2851227},
	{SG75248, &UA2427281},
	{SG75250, &UA2427883},
	{SG75254, &UA2432390},
	{SG75255, &UA2460262},
	{SG75256, &UA2460913},
	{SG75257, &UA2486354},
	{SG75258, &UA2878816},
	{SG75259, &UA2891427},
	{SG75260, &UA2944111},
	{SG75263, &UA2999142},
	{SG75264, &UA3105840},
	{SG75273, &UA253773},
	{SG75274, &UA2509625},
	{SG75275, &UA2654753},
	{SG75276, &UA2791812},
	{SG75278, &UA2509896},
	{SG75279, &UA2659634},
	{SG75283, &UA347412},
	{SG75284, &UA898435},
	{SG75286, &UA2509400},
	{SG75287, &UA2658786},
	{SG75288, &UA2216460},
	{SG75289, &UA2337837},
	{SG75290, &UA2376003},
	{SG75291, &UA2412046},
	{SG75292, &UA2415909},
	{SG75293, &UA2442616},
	{SG75294, &UA2777705},
	{SG75307, &UA2346395},
	{SG75308, &UA2427350},
	{SG75309, &UA2496947},
	{SG75311, &UA2509358},
	{SG75312, &UA2852704},
	{SG75313, &UA636852},
	{SG75314, &UA775809},
	{SG75315, &UA1970062},
	{SG75316, &UA2394874},
	{SG75317, &UA2442510},
	{SG75318, &UA2982724},
	{SG75319, &UA3118053},
	{SG75320, &UA114105},
	{SG75321, &UA2658813},
	{SG75322, &UA2792648},
	{SG75353, &UA645926},
	{SG75354, &UA729831},
	{SG75355, &UA2147565},
	{SG75402, &UA118705},
	{SG75403, &UA2580050},
	{SG75404, &UA2658796},
	{SG75405, &UA2659000},
	{SG75406, &UA2792340},
	{SG75407, &UA2792840},
	{SG75408, &UA2850987},
	{SG75415, &UA349304},
	{SG75416, &UA571505},
	{SG75417, &UA898326},
	{SG75418, &UA2588297},
	{SG75422, &UA2791752},
	{SG75426, &UA2670764},
	{SG75427, &UA2670770},
	{SG75428, &UA2675885},
	{SG75429, &UA2675985},
	{SG75430, &UA2862948},
	{SG75431, &UA253769},
	{SG75434, &UA2885350},
	{SG75435, &UA2899409},
	{SG75436, &UA2944090},
	{SG75437, &UA2991262},
	{SG75442, &UA94678},
	{SG75448, &UA1886770},
	{SG75449, &UA2144495},
	{SG75450, &UA2297116},
	{SG75451, &UA2408592},
	{SG75452, &UA2409115},
	{SG75453, &UA2473172},
	{SG75454, &UA2586722},
	{SG75455, &UA824853},
	{SG75456, &UA2165462},
	{SG75457, &UA2217603},
	{SG75458, &UA2443480},
	{SG75459, &UA2495866},
	{SG75460, &UA2851601},
	{SG75461, &UA3042899},
	{SG75470, &UA401296},
	{SG75471, &UA437942},
	{SG75472, &UA590590},
	{SG75473, &UA603075},
	{SG75474, &UA2221748},
	{SG75475, &UA2334911},
	{SG75476, &UA3069695},
	{SG75479, &UA2876818},
	{SG75487, &UA142711},
	{SG75488, &UA433201},
	{SG75489, &UA1842639},
	{SG75490, &UA2658946},
	{SG75500, &UA2556800},
	{SG75501, &UA2935881},
	{SG75502, &UA2944086},
	{SG75504, &UA3077038},
	{SG75505, &UA3105994},
	{SG75517, &UA2342885},
	{SG75518, &UA2425714},
	{SG75543, &UA2232094},
	{SG75544, &UA2238188},
	{SG75545, &UA2354631},
	{SG75560, &UA414753},
	{SG75561, &UA3070002},
	{SG75562, &UA2581003},
	{SG75563, &UA2792618},
	{SG75564, &UA2852075},
	{SG75567, &UA311925},
	{SG75568, &UA635808},
	{SG75569, &UA734900},
	{SG75579, &UA2660313},
	{SG75580, &UA2792682},
	{SG75581, &UA2849892},
	{SG75583, &UA2566596},
	{SG75584, &UA2567064},
	{SG75585, &UA2603869},
	{SG75587, &UA2581282},
	{SG75588, &UA2661193},
	{SG75589, &UA2791832},
	{SG75590, &UA2849820},
	{SG75612, &UA2496641},
	{SG75615, &UA2744219},
	{SG75617, &UA2674592},
	{SG75618, &UA2735899},
	{SG75621, &UA2658925},
	{SG75622, &UA2700544},
	{SG75623, &UA2792882},
	{SG75626, &UA2459162},
	{SG75627, &UA155229},
	{SG75628, &UA177426},
	{SG75629, &UA197861},
	{SG75630, &UA276907},
	{SG75631, &UA300268},
	{SG75633, &UA469086},
	{SG75634, &UA523814},
	{SG75644, &UA2658809},
	{SG75646, &UA2791808},
	{SG75647, &UA2851532},
	{SG75649, &UA1885181},
	{SG75650, &UA2242381},
	{SG75651, &UA2502125},
	{SG75652, &UA2720814},
	{SG75656, &UA250078},
	{SG75657, &UA343288},
	{SG75658, &UA437004},
	{SG75659, &UA573438},
	{SG75660, &UA854195},
	{SG75661, &UA1901829},
	{SG75662, &UA2300216},
	{SG75669, &UA2658995},
	{SG75670, &UA2700402},
	{SG75671, &UA2851659},
	{SG75676, &UA1958432},
	{SG75677, &UA1976514},
	{SG75678, &UA1994220},
	{SG75679, &UA2166986},
	{SG75680, &UA2217091},
	{SG75681, &UA2758225},
	{SG75682, &UA3069795},
	{SG75691, &UA122861},
	{SG75692, &UA123541},
	{SG75693, &UA2580073},
	{SG75698, &UA796355},
	{SG75699, &UA888923},
	{SG75700, &UA1953100},
	{SG75706, &UA2427897},
	{SG75707, &UA2468245},
	{SG75708, &UA2468938},
	{SG75709, &UA2748156},
	{SG75730, &UA2935855},
	{SG75731, &UA2957454},
	{SG75740, &UA2700433},
	{SG75741, &UA2792636},
	{SG75742, &UA2850082},
	{SG75764, &UA2791836},
	{SG75765, &UA2792602},
	{SG75766, &UA2849866},
	{SG75791, &UA2190856},
	{SG75792, &UA2202171},
	{SG75793, &UA2202656},
	{SG75794, &UA2354721},
	{SG75799, &UA720724},
	{SG75800, &UA1931551},
	{SG75801, &UA2148551},
	{SG75802, &UA2151964},
	{SG75803, &UA2812594},
	{SG75813, &UA2660310},
	{SG75814, &UA2660328},
	{SG75815, &UA2849947},
	{SG75824, &UA2700405},
	{SG75825, &UA2792541},
	{SG75826, &UA2792679},
	{SG75827, &UA2851533},
	{SG75839, &UA465688},
	{SG75840, &UA720684},
	{SG75841, &UA770502},
	{SG75842, &UA773598},
	{SG75843, &UA873827},
	{SG75844, &UA2320949},
	{SG75845, &UA2730136},
	{SG75846, &UA2888915},
	{SG75849, &UA2791791},
	{SG75850, &UA2850762},
	{SG75861, &UA465143},
	{SG75863, &UA495327},
	{SG75864, &UA2376048},
	{SG75865, &UA2792798},
	{SG75867, &UA271065},
	{SG75868, &UA529283},
	{SG75869, &UA2793138},
	{SG75882, &UA98491},
	{SG75884, &UA2470662},
	{SG75885, &UA2658347},
	{SG75886, &UA2700186},
	{SG75889, &UA135297},
	{SG75890, &UA1842368},
	{SG75898, &UA120599},
	{SG75899, &UA2529194},
	{SG75901, &UA2583783},
	{SG75903, &UA2594721},
	{SG75905, &UA2648459},
	{SG75906, &UA2792527},
	{SG75910, &UA559227},
	{SG75923, &UA2849983},
	{SG75924, &UA2849984},
	{SG75929, &UA912444},
	{SG75930, &UA912581},
	{SG75931, &UA912595},
	{SG75932, &UA924057},
	{SG75933, &UA924082},
	{SG75934, &UA1954969},
	{SG75935, &UA2338226},
	{SG75936, &UA2425411},
	{SG75940, &UA2366386},
	{SG75941, &UA2384958},
	{SG75942, &UA2436311},
	{SG75943, &UA2529022},
	{SG75944, &UA2715777},
	{SG75945, &UA2896668},
	{SG75946, &UA3027961},
	{SG75954, &UA1864686},
	{SG75955, &UA3119161},
	{SG75957, &UA791178},
	{SG75961, &UA2555327},
	{SG75962, &UA2727144},
	{SG75963, &UA2746642},
	{SG75964, &UA2982547},
	{SG75966, &UA656229},
	{SG75967, &UA847437},
	{SG75968, &UA1954295},
	{SG75969, &UA1961128},
	{SG75970, &UA2257875},
	{SG75971, &UA2337587},
	{SG75973, &UA2163176},
	{SG75974, &UA2260412},
	{SG75975, &UA2831860},
	{SG75982, &UA2462002},
	{SG75983, &UA2658914},
	{SG75984, &UA2850080},
	{SG75990, &UA2971172},
	{SG75991, &UA3016905},
	{SG75994, &UA570409},
	{SG75996, &UA2805497},
	{SG75999, &UA1853487},
	{SG76000, &UA512337},
	{SG76001, &UA671444},
	{SG76002, &UA2700467},
	{SG76010, &UA2753420},
	{SG76011, &UA2937216},
	{SG76012, &UA2998623},
	{SG76014, &UA2965384},
	{SG76016, &UA2791677},
	{SG76017, &UA2850905},
	{SG76018, &UA2570160},
	{SG76031, &UA2690426},
	{SG76032, &UA2722005},
	{SG76033, &UA2722430},
	{SG76035, &UA2847038},
	{SG76036, &UA3118081},
	{SG76039, &UA2792990},
	{SG76040, &UA2851165},
	{SG76044, &UA2660427},
	{SG76045, &UA2791732},
	{SG76067, &UA812982},
	{SG76068, &UA2250176},
	{SG76080, &UA1892078},
	{SG76081, &UA2660496},
	{SG76082, &UA2791728},
	{SG76083, &UA98115},
	{SG76084, &UA2580094},
	{SG76085, &UA2792903},
	{SG76086, &UA2799510},
	{SG76089, &UA2791978},
	{SG76090, &UA2792763},
	{SG76109, &UA623717},
	{SG76110, &UA898697},
	{SG76118, &UA2505998},
	{SG76119, &UA2370805},
	{SG76120, &UA2658835},
	{SG76121, &UA3117730},
	{SG76127, &UA2412881},
	{SG76128, &UA2472333},
	{SG76129, &UA2496953},
	{SG76131, &UA257474},
	{SG76132, &UA356019},
	{SG76133, &UA361800},
	{SG76134, &UA362702},
	{SG76135, &UA368282},
	{SG76136, &UA441024},
	{SG76137, &UA451128},
	{SG76138, &UA586147},
	{SG76139, &UA588537},
	{SG76140, &UA588618},
	{SG76141, &UA590382},
	{SG76142, &UA591548},
	{SG76143, &UA603964},
	{SG76144, &UA604205},
	{SG76145, &UA609234},
	{SG76146, &UA609252},
	{SG76147, &UA650123},
	{SG76148, &UA787155},
	{SG76149, &UA813522},
	{SG76150, &UA820487},
	{SG76152, &UA823612},
	{SG76153, &UA834468},
	{SG76154, &UA850276},
	{SG76155, &UA1958447},
	{SG76156, &UA2305464},
	{SG76157, &UA2339848},
	{SG76158, &UA2508814},
	{SG76169, &UA888336},
	{SG76173, &UA2435429},
	{SG76188, &UA2660567},
	{SG76190, &UA2599117},
	{SG76191, &UA2730135},
	{SG76193, &UA2659699},
	{SG76194, &UA2791865},
	{SG76209, &UA2849949},
	{SG76210, &UA2658999},
	{SG76214, &UA2696508},
	{SG76215, &UA2951053},
	{SG76216, &UA2982824},
	{SG76217, &UA3118223},
	{SG76223, &UA2660691},
	{SG76225, &UA426918},
	{SG76226, &UA2519428},
	{SG76227, &UA678485},
	{SG76228, &UA2658819},
	{SG76233, &UA624610},
	{SG76234, &UA2581312},
	{SG76236, &UA2849802},
	{SG76255, &UA2852255},
	{SG76257, &UA104417},
	{SG76258, &UA2462649},
	{SG76259, &UA2891566},
	{SG76260, &UA351777},
	{SG76261, &UA357715},
	{SG76262, &UA367699},
	{SG76263, &UA380861},
	{SG76264, &UA398445},
	{SG76267, &UA1935717},
	{SG76268, &UA2245671},
	{SG76277, &UA2658758},
	{SG76278, &UA2658762},
	{SG76279, &UA2791642},
	{SG76290, &UA2661490},
	{SG76294, &UA2699590},
	{SG76298, &UA2597449},
	{SG76299, &UA2851187},
	{SG76306, &UA2661338},
	{SG76307, &UA2699609},
	{SG76308, &UA2857010},
	{SG76312, &UA2851510},
	{SG76330, &UA2961339},
	{SG76331, &UA3056607},
	{SG76335, &UA2851838},
	{SG76336, &UA381287},
	{SG76337, &UA584028},
	{SG76338, &UA1942590},
	{SG76339, &UA1944627},
	{SG76340, &UA2310062},
	{SG76341, &UA2310541},
	{SG76342, &UA367756},
	{SG76343, &UA367860},
	{SG76344, &UA585232},
	{SG76345, &UA453843},
	{SG76346, &UA457766},
	{SG76347, &UA605641},
	{SG76348, &UA605645},
	{SG76349, &UA606146},
	{SG76350, &UA606898},
	{SG76355, &UA2005194},
	{SG76356, &UA2008105},
	{SG76357, &UA2791696},
	{SG76361, &UA2894131},
	{SG76362, &UA3005552},
	{SG76367, &UA2693655},
	{SG76369, &UA2409874},
	{SG76371, &UA3048365},
	{SG76372, &UA3090829},
	{SG76392, &UA333316},
	{SG76393, &UA745172},
	{SG76394, &UA2498324},
	{SG76395, &UA2849805},
	{SG76398, &UA2408985},
	{SG76399, &UA2444367},
	{SG76400, &UA2545145},
	{SG76401, &UA2979682},
	{SG76402, &UA2792603},
	{SG76403, &UA600636},
	{SG76404, &UA813391},
	{SG76405, &UA823291},
	{SG76406, &UA2333045},
	{SG76407, &UA2130904},
	{SG76410, &UA2792764},
	{SG76429, &UA2850083},
	{SG76430, &UA2153730},
	{SG76431, &UA2154687},
	{SG76432, &UA2945221},
	{SG76457, &UA2849843},
	{SG76469, &UA2409221},
	{SG76471, &UA2438967},
	{SG76472, &UA2445139},
	{SG76474, &UA2851881},
	{SG76485, &UA473140},
	{SG76486, &UA2978628},
	{SG76487, &UA3041591},
	{SG76495, &UA2138986},
	{SG76496, &UA2813298},
	{SG76507, &UA2851692},
	{SG76523, &UA447320},
	{SG76524, &UA2541804},
	{SG76549, &UA252264},
	{SG76550, &UA254764},
	{SG76551, &UA257391},
	{SG76552, &UA350460},
	{SG76553, &UA624158},
	{SG76554, &UA643025},
	{SG76562, &UA2245626},
	{SG76571, &UA2658836},
	{SG76608, &UA279267},
	{SG76613, &UA2849858},
	{SG76616, &UA352794},
	{SG76617, &UA361769},
	{SG76624, &UA910464},
	{SG76625, &UA1995587},
	{SG76630, &UA361836},
	{SG76631, &UA2309181},
	{SG76645, &UA2332630},
	{SG76646, &UA2495279},
	{SG76649, &UA2580031},
	{SG76651, &UA3008999},
	{SG76653, &UA2998332},
	{SG76657, &UA2791818},
	{SG76658, &UA387738},
	{SG76659, &UA2412677},
	{SG76660, &UA2472467},
	{SG76674, &UA2144214},
	{SG76686, &UA351384},
	{SG76687, &UA352870},
	{SG76689, &UA420117},
	{SG76690, &UA2433055},
	{SG76691, &UA2846234},
	{SG76704, &UA2438946},
	{SG76710, &UA497000},
	{SG76711, &UA2979893},
	{SG76712, &UA2245613},
	{SG76714, &UA282102},
	{SG76715, &UA1867245},
	{SG76716, &UA1898335},
	{SG76717, &UA1900378},
	{SG76718, &UA2258216},
	{SG76723, &UA2805229},
	{SG76724, &UA3099022},
	{SG76728, &UA2527751},
	{SG76731, &UA97610},
	{SG76732, &UA2462915},
	{SG76734, &UA3043270},
	{SG76735, &UA3071922},
	{SG76736, &UA516890},
	{SG76737, &UA524230},
	{SG76738, &UA2472825},
	{SG76741, &UA2792709},
	{SG76748, &UA327466},
	{SG76749, &UA668641},
	{SG76755, &UA2517675},
	{SG76759, &UA2959338},
	{SG76766, &UA2439260},
	{SG76767, &UA2520447},
	{SG76770, &UA2438568},
	{SG76782, &UA2993965},
	{SG76783, &UA249770},
	{SG76784, &UA342948},
	{SG76785, &UA409516},
	{SG76786, &UA1960174},
	{SG76787, &UA2165971},
	{SG76788, &UA1905144},
	{SG76794, &UA3056143},
	{SG76795, &UA350797},
	{SG76796, &UA351421},
	{SG76808, &UA2437778},
	{SG76809, &UA2762558},
	{SG76810, &UA2888791},
	{SG76826, &UA2981095},
	{SG76835, &UA2484143},
	{SG76851, &UA3108078},
	{SG76854, &UA104011},
	{SG76856, &UA849027},
	{SG76857, &UA2041345},
	{SG76858, &UA2196266},
	{SG76871, &UA254019},
	{SG76872, &UA2303063},
	{SG76875, &UA2160366},
	{SG76896, &UA3119066},
	{SG76915, &UA251730},
	{SG76918, &UA603966},
	{SG76919, &UA616157},
	{SG76920, &UA741375},
	{SG76921, &UA607307},
	{SG76922, &UA1865732},
	{SG76923, &UA2144791},
	{SG76924, &UA377081},
	{SG76925, &UA2266683},
	{SG76926, &UA514541},
	{SG76927, &UA2324018},
	{SG76928, &UA911661},
	{SG76929, &UA1988786},
	{SG76930, &UA898057},
	{SG76931, &UA899300},
	{NULL}};
const Device *DNexian[] = {
	&UA526030,
	&UA616233,
	&UA616236,
	&UA616232,
	&UA616270,
	&UA616271,
	&UA616294,
	&UA616307,
	&UA616328,
	&UA354154,
	&UA616245,
	&UA616241,
	&UA616249,
	&UA616248,
	&UA616257,
	&UA616261,
	&UA616266,
	&UA616285,
	&UA616295,
	&UA616320,
	&UA616322,
	&UA616330,
	&UA616244,
	&UA616242,
	&UA69533,
	&UA616250,
	&UA616272,
	&UA616273,
	&UA616319,
	&UA616323,
	&UA24092,
	&UA616287,
	&UA33285,
	&UA48792,
	&UA49622,
	&UA49708,
	&UA616314,
	&UA49318,
	&UA22207,
	&UA616252,
	&UA616291,
	&UA63501,
	&UA89845,
	&UA22026,
	&UA89739,
	&UA616281,
	&UA24636,
	&UA89695,
	&UA63588,
	&UA23421,
	&UA616237,
	&UA32789,
	&UA49870,
	&UA24535,
	&UA24733,
	&UA49660,
	&UA24623,
	&UA24474,
	&UA88780,
	&UA63516,
	&UA89700,
	&UA49972,
	&UA18521,
	&UA89799,
	&UA616325,
	&UA33362,
	&UA33395,
	&UA50038,
	&UA49679,
	&UA33392,
	&UA24616,
	&UA22994,
	NULL};
const Segmentdata DNokia[] = {
	{SG76932, &UA67247},
	{SG76933, &UA812115},
	{SG76934, &UA2448700},
	{SG76935, &UA2752544},
	{SG76936, &UA18738},
	{SG76937, &UA67029},
	{SG76938, &UA68963},
	{SG76939, &UA68991},
	{SG76940, &UA24231},
	{SG76941, &UA26961},
	{SG76942, &UA31053},
	{SG76943, &UA2984997},
	{SG76953, &UA68478},
	{SG76954, &UA103350},
	{SG76955, &UA2035305},
	{SG76956, &UA2362456},
	{SG76957, &UA2367170},
	{SG76958, &UA2401811},
	{SG76959, &UA2469505},
	{SG76960, &UA2481528},
	{SG76961, &UA3061789},
	{SG76975, &UA68166},
	{SG76976, &UA69521},
	{SG77175, &UA166238},
	{SG77282, &UA51229},
	{SG77283, &UA69526},
	{SG77284, &UA20572},
	{SG77285, &UA33426},
	{SG77286, &UA57001},
	{SG77287, &UA2951466},
	{SG77457, &UA2035304},
	{SG77458, &UA2382013},
	{SG77459, &UA2387511},
	{SG77460, &UA2506074},
	{SG77462, &UA2506381},
	{SG77464, &UA3064039},
	{SG77576, &UA8493},
	{SG77577, &UA52267},
	{SG77578, &UA64355},
	{SG77579, &UA397350},
	{SG77682, &UA68394},
	{SG77683, &UA69525},
	{SG77686, &UA2493771},
	{SG77746, &UA67220},
	{SG77749, &UA490413},
	{SG77753, &UA624958},
	{SG77819, &UA24594},
	{SG77820, &UA44114},
	{SG77821, &UA622986},
	{SG78019, &UA33314},
	{SG78020, &UA2371208},
	{SG78021, &UA3099520},
	{SG78309, &UA2458158},
	{SG78310, &UA2516062},
	{SG78311, &UA2530254},
	{SG78312, &UA2531600},
	{SG78313, &UA2716809},
	{SG78318, &UA81129},
	{SG78319, &UA120962},
	{SG78324, &UA69532},
	{SG78325, &UA2202609},
	{SG78326, &UA2232263},
	{SG78327, &UA377133},
	{SG78362, &UA16590},
	{SG78363, &UA24533},
	{SG78365, &UA41167},
	{SG78367, &UA61525},
	{SG78369, &UA2983154},
	{SG78451, &UA598726},
	{SG78459, &UA69530},
	{SG78462, &UA2384820},
	{SG78463, &UA2387677},
	{SG78464, &UA2405480},
	{SG78465, &UA2422965},
	{SG78466, &UA2429980},
	{SG78478, &UA640056},
	{SG78479, &UA20526},
	{SG78480, &UA45128},
	{SG78481, &UA64378},
	{SG78482, &UA34957},
	{SG78483, &UA54008},
	{SG78495, &UA7134},
	{SG78496, &UA33448},
	{SG78497, &UA2984890},
	{SG78532, &UA72},
	{SG78533, &UA130},
	{SG78542, &UA25381},
	{SG78543, &UA25378},
	{SG78544, &UA2719571},
	{SG78545, &UA2800205},
	{SG78546, &UA2938110},
	{SG78547, &UA3003037},
	{SG78548, &UA3023569},
	{SG78549, &UA3066654},
	{SG78550, &UA2043995},
	{SG78670, &UA2664628},
	{SG78671, &UA2675610},
	{SG78672, &UA2714805},
	{SG78673, &UA2715741},
	{SG78674, &UA2738337},
	{SG78675, &UA2892921},
	{SG78676, &UA3095937},
	{SG78677, &UA3102662},
	{SG78678, &UA33287},
	{SG78679, &UA1985847},
	{SG78680, &UA2695913},
	{SG78738, &UA5758},
	{SG78740, &UA64844},
	{SG78753, &UA623375},
	{SG78755, &UA2694700},
	{SG78757, &UA3043737},
	{SG78865, &UA16687},
	{SG78866, &UA63899},
	{SG78867, &UA63925},
	{SG78868, &UA69638},
	{SG78955, &UA2167547},
	{SG78958, &UA2700101},
	{SG78959, &UA1221},
	{SG78960, &UA1224},
	{SG78961, &UA1289},
	{SG78962, &UA80490},
	{SG78963, &UA2533008},
	{SG78964, &UA2167718},
	{SG78969, &UA14269},
	{SG78970, &UA63496},
	{SG78971, &UA665409},
	{SG78972, &UA2554669},
	{SG78973, &UA2844403},
	{SG79100, &UA2507982},
	{SG79113, &UA395759},
	{SG79114, &UA397366},
	{SG79130, &UA9897},
	{SG79132, &UA15513},
	{SG79134, &UA15562},
	{SG79195, &UA611},
	{SG79196, &UA606},
	{SG79197, &UA2661},
	{SG79198, &UA64825},
	{SG79199, &UA2488275},
	{SG79200, &UA2505451},
	{SG79201, &UA2597049},
	{SG79468, &UA2402711},
	{SG79470, &UA2404814},
	{SG79472, &UA2430801},
	{SG79474, &UA2458576},
	{SG79549, &UA13793},
	{SG79550, &UA13817},
	{SG79551, &UA32166},
	{SG79552, &UA63970},
	{SG79555, &UA667902},
	{SG79556, &UA2405010},
	{SG79557, &UA2405337},
	{SG79558, &UA2422710},
	{SG79559, &UA2427193},
	{SG79568, &UA2405030},
	{SG79571, &UA2423298},
	{SG79573, &UA2423901},
	{SG79575, &UA2425728},
	{SG79577, &UA2483523},
	{SG79626, &UA628898},
	{SG79627, &UA2691197},
	{SG79628, &UA2185252},
	{SG79629, &UA2193304},
	{SG79630, &UA2421696},
	{SG79695, &UA631042},
	{SG79697, &UA2167052},
	{SG79699, &UA2642259},
	{SG79724, &UA2349624},
	{SG79727, &UA2387384},
	{SG79736, &UA69856},
	{SG79737, &UA74467},
	{SG79738, &UA77552},
	{SG79739, &UA2227330},
	{SG79745, &UA2512227},
	{SG79747, &UA2204914},
	{SG79787, &UA2422571},
	{SG79789, &UA2422745},
	{SG79791, &UA2426537},
	{SG79793, &UA2461145},
	{SG79809, &UA899397},
	{SG79811, &UA2759161},
	{SG79834, &UA69520},
	{SG79883, &UA3869},
	{SG79884, &UA3887},
	{SG79885, &UA25462},
	{SG79886, &UA2230724},
	{SG79889, &UA2387659},
	{SG79890, &UA2389103},
	{SG79891, &UA2433276},
	{SG79892, &UA2455840},
	{SG79893, &UA2510070},
	{SG79894, &UA67054},
	{SG79896, &UA2507980},
	{SG79899, &UA2168359},
	{SG79900, &UA2402581},
	{SG79901, &UA2406344},
	{SG79902, &UA2425884},
	{SG79903, &UA2505417},
	{SG79904, &UA2227792},
	{SG79905, &UA2261306},
	{SG79913, &UA2511080},
	{SG79917, &UA67082},
	{SG79939, &UA6570},
	{SG79940, &UA6579},
	{SG79941, &UA25770},
	{SG79942, &UA54713},
	{SG79943, &UA54828},
	{SG79952, &UA2488270},
	{SG79953, &UA2507006},
	{SG79954, &UA2510521},
	{SG79955, &UA2697695},
	{SG79957, &UA2402442},
	{SG79958, &UA2405923},
	{SG79959, &UA2428095},
	{SG79966, &UA2782337},
	{SG79967, &UA2405202},
	{SG79968, &UA649161},
	{SG79969, &UA674939},
	{SG79970, &UA2160378},
	{SG79973, &UA2508249},
	{SG79974, &UA2794840},
	{SG79975, &UA2406142},
	{SG79976, &UA2391134},
	{SG79993, &UA2507636},
	{SG80026, &UA437256},
	{SG80028, &UA440219},
	{SG80030, &UA2808264},
	{SG80160, &UA1904},
	{SG80161, &UA1907},
	{SG80162, &UA13872},
	{SG80163, &UA13875},
	{SG80164, &UA20355},
	{SG80165, &UA25213},
	{SG80166, &UA29318},
	{SG80207, &UA2488874},
	{SG80208, &UA2528719},
	{SG80209, &UA2656535},
	{SG80210, &UA2729969},
	{SG80211, &UA2843999},
	{SG80212, &UA2489387},
	{SG80213, &UA2529396},
	{SG80214, &UA2598491},
	{SG80215, &UA2952485},
	{SG80216, &UA2986068},
	{SG80217, &UA2668539},
	{SG80218, &UA2716115},
	{SG80219, &UA2781780},
	{SG80220, &UA1949},
	{SG80221, &UA1939},
	{SG80222, &UA17254},
	{SG80294, &UA2425604},
	{SG80297, &UA2938137},
	{SG80299, &UA2950689},
	{SG80302, &UA2959526},
	{SG80305, &UA3012551},
	{SG80308, &UA3050520},
	{SG80310, &UA3057551},
	{SG80312, &UA3057639},
	{SG80314, &UA3093031},
	{SG80316, &UA3106675},
	{SG80318, &UA3108343},
	{SG80320, &UA3114859},
	{SG80323, &UA3114969},
	{SG80335, &UA2488274},
	{SG80336, &UA2558306},
	{SG80337, &UA2791533},
	{SG80341, &UA2354901},
	{SG80362, &UA2422198},
	{SG80364, &UA2423044},
	{SG80366, &UA2427232},
	{SG80405, &UA2353297},
	{SG80406, &UA2387382},
	{SG80407, &UA2362444},
	{SG80408, &UA2390866},
	{SG80409, &UA50300},
	{SG80410, &UA2405802},
	{SG80411, &UA2422874},
	{SG80412, &UA2458023},
	{SG80418, &UA2261189},
	{SG80419, &UA1959843},
	{SG80420, &UA1986689},
	{SG80421, &UA2652431},
	{SG80422, &UA2043550},
	{SG80423, &UA3044898},
	{SG80424, &UA2387669},
	{SG80425, &UA2422660},
	{SG80426, &UA2508300},
	{SG80427, &UA1994971},
	{SG80428, &UA2744455},
	{SG80429, &UA2952038},
	{SG80430, &UA2511333},
	{SG80431, &UA2749051},
	{SG80432, &UA2387654},
	{SG80433, &UA2390368},
	{SG80434, &UA2420678},
	{SG80435, &UA2429526},
	{SG80436, &UA2654381},
	{SG80437, &UA3070888},
	{SG80438, &UA1989},
	{SG80439, &UA2012},
	{SG80440, &UA36251},
	{SG80441, &UA62548},
	{SG80450, &UA689286},
	{SG80451, &UA2261406},
	{SG80452, &UA14076},
	{SG80453, &UA14091},
	{SG80460, &UA2205189},
	{SG80461, &UA2721954},
	{SG80462, &UA6804},
	{SG80463, &UA27503},
	{SG80464, &UA43725},
	{SG80465, &UA63395},
	{SG80466, &UA30792},
	{SG80467, &UA2488285},
	{SG80468, &UA2505732},
	{SG80469, &UA2700715},
	{SG80470, &UA2749754},
	{SG80471, &UA2851053},
	{SG80472, &UA2422823},
	{SG80473, &UA2423647},
	{SG80474, &UA2484236},
	{SG80475, &UA6827},
	{SG80476, &UA6829},
	{SG80477, &UA30976},
	{SG80478, &UA30979},
	{SG80479, &UA2557835},
	{SG80480, &UA2597645},
	{SG80481, &UA2716726},
	{SG80482, &UA2717873},
	{SG80483, &UA22710},
	{SG80484, &UA22715},
	{SG80492, &UA2405763},
	{SG80493, &UA2427242},
	{SG80494, &UA2458213},
	{SG80495, &UA2458384},
	{SG80502, &UA15714},
	{SG80504, &UA2508871},
	{SG80569, &UA2389093},
	{SG80571, &UA2982975},
	{SG80578, &UA2948287},
	{SG80579, &UA3018592},
	{SG80580, &UA2261290},
	{SG80600, &UA2235201},
	{SG80602, &UA2342053},
	{SG80604, &UA2727748},
	{SG80606, &UA2761221},
	{SG80609, &UA2205873},
	{SG80611, &UA2258413},
	{SG80613, &UA2385839},
	{SG80615, &UA2403856},
	{SG80643, &UA2074},
	{SG80645, &UA21250},
	{SG80648, &UA3098628},
	{SG80667, &UA2175379},
	{SG80672, &UA2190362},
	{SG80686, &UA1335},
	{SG80687, &UA529482},
	{SG80690, &UA3030891},
	{SG80706, &UA18727},
	{SG80708, &UA64584},
	{SG80711, &UA2558280},
	{SG80712, &UA2716540},
	{SG80713, &UA2751924},
	{SG80714, &UA2953041},
	{SG80715, &UA3102333},
	{SG80716, &UA2893206},
	{SG80717, &UA2390398},
	{SG80718, &UA1972610},
	{SG80720, &UA2509665},
	{SG80721, &UA2600330},
	{SG80722, &UA2696731},
	{SG80723, &UA2748666},
	{SG80726, &UA6945},
	{SG80727, &UA6965},
	{SG80795, &UA2236136},
	{SG80796, &UA2479942},
	{SG80797, &UA2503287},
	{SG80798, &UA2191295},
	{SG80799, &UA2470186},
	{SG80801, &UA2509843},
	{SG80807, &UA2390862},
	{SG80808, &UA2407068},
	{SG80809, &UA2423146},
	{SG80811, &UA2425800},
	{SG80817, &UA2199242},
	{SG80818, &UA2226061},
	{SG80819, &UA2401581},
	{SG80822, &UA2489117},
	{SG80823, &UA2422105},
	{SG80824, &UA2427354},
	{SG80825, &UA2457677},
	{SG80826, &UA2459563},
	{SG80827, &UA2505452},
	{SG80828, &UA2983483},
	{SG80829, &UA2387661},
	{SG80849, &UA2194864},
	{SG80860, &UA2352470},
	{SG80917, &UA353},
	{SG80919, &UA13405},
	{SG80921, &UA15336},
	{SG80922, &UA62906},
	{SG80924, &UA2531070},
	{SG80928, &UA2197870},
	{SG80994, &UA1974502},
	{SG80995, &UA2367029},
	{SG80996, &UA2424819},
	{SG80998, &UA2508082},
	{SG80999, &UA2404825},
	{SG81001, &UA2404967},
	{SG81003, &UA2433441},
	{SG81004, &UA2508659},
	{SG81005, &UA2597387},
	{SG81069, &UA2900020},
	{SG81075, &UA3031402},
	{SG81080, &UA2406515},
	{SG81082, &UA2423532},
	{SG81092, &UA2260694},
	{SG81093, &UA54463},
	{SG81094, &UA80863},
	{SG81100, &UA2168264},
	{SG81101, &UA13998},
	{SG81102, &UA14531},
	{SG81103, &UA2196574},
	{SG81104, &UA67952},
	{SG81105, &UA63974},
	{SG81106, &UA3021581},
	{SG81107, &UA14178},
	{SG81108, &UA67902},
	{SG81109, &UA609643},
	{SG81111, &UA2223451},
	{SG81120, &UA2226355},
	{SG81122, &UA2237326},
	{SG81130, &UA3026310},
	{SG81153, &UA2672161},
	{SG81155, &UA2700796},
	{SG81157, &UA2779843},
	{SG81159, &UA2953317},
	{SG81161, &UA3103431},
	{SG81170, &UA68548},
	{SG81172, &UA2424021},
	{SG81173, &UA2483288},
	{SG81174, &UA2556601},
	{SG81175, &UA7931},
	{SG81176, &UA7943},
	{SG81177, &UA2384313},
	{SG81179, &UA2867821},
	{SG81180, &UA2968797},
	{SG81181, &UA2970459},
	{SG81182, &UA2994403},
	{SG81184, &UA50440},
	{SG81194, &UA2158534},
	{SG81196, &UA2590032},
	{SG81202, &UA2510217},
	{SG81204, &UA2596443},
	{SG81206, &UA2763888},
	{SG81224, &UA63788},
	{SG81235, &UA2535968},
	{SG81236, &UA2716555},
	{SG81237, &UA2781080},
	{SG81300, &UA2896178},
	{SG81312, &UA2513215},
	{SG81314, &UA2657769},
	{SG81316, &UA2748974},
	{SG81318, &UA2791987},
	{SG81320, &UA2954406},
	{SG81332, &UA899798},
	{SG81333, &UA2746170},
	{SG81386, &UA2404601},
	{SG81389, &UA2457924},
	{SG81393, &UA687567},
	{SG81394, &UA1946716},
	{SG81395, &UA2164215},
	{SG81396, &UA2319956},
	{SG81397, &UA16384},
	{SG81398, &UA53776},
	{SG81400, &UA61703},
	{SG81417, &UA2884321},
	{SG81478, &UA2224755},
	{SG81509, &UA1373},
	{SG81510, &UA1381},
	{SG81511, &UA23386},
	{SG81512, &UA30359},
	{SG81513, &UA55143},
	{SG81514, &UA2389090},
	{SG81515, &UA16009},
	{SG81546, &UA2353552},
	{SG81560, &UA2768663},
	{SG81563, &UA2946182},
	{SG81565, &UA3019676},
	{SG81571, &UA7858},
	{SG81575, &UA2227776},
	{SG81579, &UA2204912},
	{SG81581, &UA2895187},
	{SG81582, &UA3028633},
	{SG81583, &UA3108223},
	{SG81584, &UA3112833},
	{SG81586, &UA64885},
	{SG81588, &UA2272247},
	{SG81606, &UA20990},
	{SG81608, &UA20991},
	{SG81610, &UA53773},
	{SG81631, &UA56909},
	{SG81632, &UA64233},
	{SG81633, &UA2665599},
	{SG81634, &UA2716548},
	{SG81635, &UA2866898},
	{SG81636, &UA2995801},
	{SG81637, &UA2204851},
	{SG81638, &UA2897841},
	{SG81639, &UA3055550},
	{SG81644, &UA2405878},
	{SG81645, &UA674012},
	{SG81646, &UA754141},
	{SG81647, &UA2543021},
	{SG81648, &UA3031209},
	{SG81652, &UA2511455},
	{SG81653, &UA2558133},
	{SG81654, &UA2571566},
	{SG81657, &UA2423989},
	{SG81658, &UA2428446},
	{SG81659, &UA2727167},
	{SG81660, &UA2752639},
	{SG81661, &UA2947849},
	{SG81662, &UA2535079},
	{SG81664, &UA2934694},
	{SG81665, &UA629848},
	{SG81666, &UA683186},
	{SG81667, &UA2360093},
	{SG81699, &UA2230829},
	{SG81701, &UA2239221},
	{SG81822, &UA1993837},
	{SG81828, &UA2510416},
	{SG81830, &UA2656451},
	{SG81892, &UA2879924},
	{SG81910, &UA2348865},
	{SG81925, &UA1993220},
	{SG81928, &UA2947153},
	{SG81933, &UA2558992},
	{SG81936, &UA2716447},
	{SG81938, &UA2752323},
	{SG81940, &UA2794315},
	{SG81972, &UA2353070},
	{SG81979, &UA2423829},
	{SG81981, &UA2790572},
	{SG81995, &UA666201},
	{SG81996, &UA2147126},
	{SG82002, &UA2508572},
	{SG82003, &UA2537636},
	{SG82028, &UA1993938},
	{SG82049, &UA2712565},
	{SG82106, &UA2716733},
	{SG82107, &UA2732938},
	{SG82108, &UA2230023},
	{SG82111, &UA2405199},
	{SG82175, &UA5295},
	{SG82176, &UA21932},
	{SG82177, &UA2365571},
	{SG82182, &UA64350},
	{SG82184, &UA2598639},
	{SG82185, &UA2717070},
	{SG82186, &UA2868607},
	{SG82187, &UA2993754},
	{SG82188, &UA13721},
	{SG82189, &UA54626},
	{SG82190, &UA2508777},
	{SG82194, &UA3021137},
	{SG82195, &UA1958000},
	{SG82196, &UA3764},
	{SG82197, &UA3783},
	{SG82198, &UA69361},
	{SG82199, &UA248910},
	{SG82200, &UA62848},
	{SG82201, &UA413687},
	{SG82202, &UA688426},
	{SG82204, &UA2707034},
	{SG82205, &UA2894141},
	{SG82206, &UA2228142},
	{SG82210, &UA2258506},
	{SG82211, &UA2261211},
	{SG82212, &UA2654666},
	{SG82213, &UA2531903},
	{SG82215, &UA80510},
	{SG82217, &UA691070},
	{SG82218, &UA893821},
	{SG82219, &UA2373893},
	{SG82220, &UA2381350},
	{SG82224, &UA2717581},
	{SG82225, &UA670792},
	{SG82226, &UA678197},
	{SG82227, &UA1943462},
	{SG82228, &UA2339014},
	{SG82229, &UA2424626},
	{SG82230, &UA2458250},
	{SG82234, &UA2193787},
	{SG82235, &UA2228988},
	{SG82236, &UA2552846},
	{SG82241, &UA2367628},
	{SG82243, &UA17399},
	{SG82244, &UA23918},
	{SG82245, &UA24070},
	{SG82253, &UA2193199},
	{SG82254, &UA2404745},
	{SG82255, &UA2736989},
	{SG82256, &UA2985406},
	{SG82257, &UA2707835},
	{SG82258, &UA2229505},
	{SG82259, &UA3022897},
	{SG82267, &UA2558659},
	{SG82268, &UA2205788},
	{SG82269, &UA642232},
	{SG82270, &UA734631},
	{SG82278, &UA924178},
	{SG82280, &UA1987739},
	{SG82283, &UA2594333},
	{SG82289, &UA3023686},
	{SG82292, &UA3060348},
	{SG82315, &UA2563065},
	{SG82316, &UA2509671},
	{SG82317, &UA15889},
	{SG82318, &UA61740},
	{SG82378, &UA2261778},
	{SG82399, &UA1959317},
	{SG82401, &UA1995581},
	{SG82403, &UA2788568},
	{SG82406, &UA2462436},
	{SG82407, &UA2706157},
	{SG82408, &UA2955659},
	{SG82409, &UA3103686},
	{SG82413, &UA68826},
	{SG82414, &UA437272},
	{SG82415, &UA1901009},
	{SG82424, &UA55982},
	{SG82425, &UA2429934},
	{SG82426, &UA3021806},
	{SG82427, &UA2483285},
	{SG82428, &UA2513024},
	{SG82432, &UA2198365},
	{SG82433, &UA399148},
	{SG82434, &UA720040},
	{SG82435, &UA925862},
	{SG82436, &UA2413765},
	{SG82437, &UA2496927},
	{SG82438, &UA2864297},
	{SG82439, &UA2967805},
	{SG82440, &UA3019481},
	{SG82441, &UA2234656},
	{SG82444, &UA1993998},
	{SG82445, &UA1994280},
	{SG82446, &UA3070348},
	{SG82455, &UA2947170},
	{SG82456, &UA3107668},
	{SG82464, &UA2423743},
	{SG82465, &UA2483851},
	{SG82466, &UA2656716},
	{SG82467, &UA2258390},
	{SG82468, &UA2342120},
	{SG82469, &UA2578966},
	{SG82470, &UA15477},
	{SG82471, &UA62978},
	{SG82480, &UA2511133},
	{SG82481, &UA2258479},
	{SG82482, &UA2380307},
	{SG82484, &UA2403969},
	{SG82485, &UA1956046},
	{SG82486, &UA2840783},
	{SG82487, &UA379335},
	{SG82488, &UA437292},
	{SG82491, &UA923876},
	{SG82492, &UA2645535},
	{SG82493, &UA2654449},
	{SG82494, &UA2811991},
	{SG82495, &UA2227005},
	{SG82498, &UA655560},
	{SG82499, &UA685818},
	{SG82501, &UA2445294},
	{SG82506, &UA90790},
	{SG82507, &UA371358},
	{SG82508, &UA399154},
	{SG82509, &UA925766},
	{SG82510, &UA925923},
	{SG82511, &UA2652419},
	{SG82512, &UA2952669},
	{SG82513, &UA2508779},
	{SG82514, &UA2702011},
	{SG82515, &UA2716273},
	{SG82516, &UA2988790},
	{SG82517, &UA2716278},
	{SG82562, &UA1982621},
	{SG82563, &UA2893550},
	{SG82564, &UA3020334},
	{SG82572, &UA2711365},
	{SG82576, &UA741970},
	{SG82577, &UA2589576},
	{SG82590, &UA61897},
	{SG82596, &UA2967701},
	{SG82599, &UA2362653},
	{SG82600, &UA62860},
	{SG82601, &UA69513},
	{SG82602, &UA81107},
	{SG82604, &UA2900008},
	{SG82605, &UA2563106},
	{SG82606, &UA2859989},
	{SG82607, &UA2428301},
	{SG82608, &UA3031422},
	{SG82609, &UA3059122},
	{SG82617, &UA2967277},
	{SG82619, &UA1981937},
	{SG82621, &UA1993880},
	{SG82649, &UA569723},
	{SG82650, &UA663106},
	{SG82651, &UA724070},
	{SG82652, &UA2716457},
	{SG82655, &UA2258564},
	{SG82656, &UA2383776},
	{SG82657, &UA2260477},
	{SG82662, &UA2229034},
	{SG82663, &UA2970051},
	{SG82664, &UA925921},
	{SG82665, &UA2034651},
	{SG82666, &UA2597885},
	{SG82667, &UA2607480},
	{SG82668, &UA2889196},
	{SG82669, &UA2988466},
	{SG82670, &UA2229007},
	{SG82671, &UA2556724},
	{SG82672, &UA2558017},
	{SG82673, &UA2235199},
	{SG82674, &UA2366274},
	{SG82675, &UA2505154},
	{SG82676, &UA2578087},
	{SG82679, &UA661962},
	{SG82680, &UA683493},
	{SG82681, &UA2372218},
	{SG82684, &UA2503896},
	{SG82690, &UA2230513},
	{SG82691, &UA2405862},
	{SG82692, &UA2532978},
	{SG82698, &UA3023266},
	{SG82699, &UA2429935},
	{SG82700, &UA2878886},
	{SG82701, &UA2967922},
	{SG82702, &UA3078579},
	{SG82752, &UA437262},
	{SG82760, &UA2424056},
	{SG82762, &UA2508019},
	{SG82776, &UA682491},
	{SG82777, &UA700632},
	{SG82778, &UA2460884},
	{SG82786, &UA3033032},
	{SG82787, &UA2456011},
	{SG82788, &UA2599837},
	{SG82801, &UA2559536},
	{SG82802, &UA2779086},
	{SG82803, &UA2580759},
	{SG82804, &UA900114},
	{SG82805, &UA1952763},
	{SG82806, &UA1955642},
	{SG82808, &UA2901230},
	{SG82809, &UA3016038},
	{SG82811, &UA749688},
	{SG82812, &UA2580572},
	{SG82813, &UA2599877},
	{SG82814, &UA2895183},
	{SG82815, &UA87814},
	{SG82816, &UA2558145},
	{SG82819, &UA2661686},
	{SG82820, &UA2699129},
	{SG82821, &UA2708740},
	{SG82822, &UA2709224},
	{SG82830, &UA2480887},
	{SG82833, &UA2457755},
	{SG82838, &UA2423782},
	{SG82839, &UA2426989},
	{SG82840, &UA2708288},
	{SG82841, &UA2422351},
	{SG82842, &UA2483291},
	{SG82843, &UA2655543},
	{SG82844, &UA2389086},
	{SG82845, &UA3025323},
	{SG82846, &UA3059315},
	{SG82848, &UA14326},
	{SG82849, &UA14329},
	{SG82850, &UA2422152},
	{SG82851, &UA2505168},
	{SG82852, &UA2842777},
	{SG82853, &UA2706893},
	{SG82854, &UA2893403},
	{SG82855, &UA2360738},
	{SG82856, &UA2901222},
	{SG82857, &UA18069},
	{SG82858, &UA41534},
	{SG82859, &UA64692},
	{SG82860, &UA3061520},
	{SG82861, &UA2706351},
	{SG82863, &UA625301},
	{SG82869, &UA64802},
	{SG82870, &UA15645},
	{SG82872, &UA2422034},
	{SG82873, &UA2432575},
	{SG82874, &UA2484031},
	{SG82875, &UA2840233},
	{SG82876, &UA54089},
	{SG82877, &UA2217706},
	{SG82883, &UA2716554},
	{SG82884, &UA2955320},
	{SG82885, &UA2404689},
	{SG82886, &UA2969552},
	{SG82887, &UA2422960},
	{SG82888, &UA2504834},
	{SG82892, &UA13777},
	{SG82893, &UA2223245},
	{SG82894, &UA2343028},
	{SG82895, &UA2430976},
	{SG82896, &UA2467293},
	{SG82897, &UA2579465},
	{SG82898, &UA399173},
	{SG82899, &UA688691},
	{SG82900, &UA925748},
	{SG82901, &UA925899},
	{SG82903, &UA2258490},
	{SG82904, &UA2508763},
	{SG82905, &UA2258505},
	{SG82906, &UA2344261},
	{SG82915, &UA13941},
	{SG82917, &UA13944},
	{SG82924, &UA2330114},
	{SG82925, &UA2261237},
	{SG82926, &UA2462458},
	{SG82927, &UA2462463},
	{SG82929, &UA2405446},
	{SG82932, &UA2718550},
	{SG82933, &UA2954926},
	{SG82934, &UA2698960},
	{SG82935, &UA2732960},
	{SG82936, &UA2749813},
	{SG82937, &UA2768748},
	{SG82938, &UA2865465},
	{SG82939, &UA2660618},
	{SG82940, &UA2460848},
	{SG82941, &UA2350094},
	{SG82942, &UA780341},
	{SG82943, &UA788177},
	{SG82944, &UA2643024},
	{SG82945, &UA2558620},
	{SG82948, &UA2421659},
	{SG82950, &UA2362412},
	{SG82951, &UA2735908},
	{SG82953, &UA14404},
	{SG82954, &UA637589},
	{SG82955, &UA2346954},
	{SG82956, &UA2422628},
	{SG82960, &UA2429593},
	{SG82961, &UA2423562},
	{SG82962, &UA2423867},
	{SG82963, &UA2458190},
	{SG82969, &UA2966614},
	{SG82973, &UA788493},
	{SG82974, &UA2313223},
	{SG82982, &UA2533274},
	{SG82983, &UA50216},
	{SG82984, &UA20940},
	{SG82986, &UA2422570},
	{SG82987, &UA9900},
	{SG82988, &UA61674},
	{SG82989, &UA80998},
	{SG82990, &UA2943929},
	{SG82991, &UA3057221},
	{SG82992, &UA2235275},
	{SG82993, &UA2341971},
	{SG82994, &UA2403874},
	{SG82995, &UA2960414},
	{SG82996, &UA3015804},
	{SG82997, &UA3059761},
	{SG82998, &UA3061663},
	{SG82999, &UA2894573},
	{SG83000, &UA2959789},
	{SG83004, &UA3030880},
	{SG83012, &UA2716117},
	{SG83017, &UA2226705},
	{SG83018, &UA69394},
	{SG83019, &UA248937},
	{SG83020, &UA2423323},
	{SG83021, &UA2468338},
	{SG83022, &UA2708986},
	{SG83024, &UA633228},
	{SG83025, &UA1933776},
	{SG83026, &UA2721365},
	{SG83027, &UA2968517},
	{SG83028, &UA13749},
	{SG83029, &UA20535},
	{SG83030, &UA2258478},
	{SG83032, &UA38},
	{SG83033, &UA44},
	{SG83034, &UA33473},
	{SG83035, &UA2363881},
	{SG83040, &UA2943998},
	{SG83047, &UA2429587},
	{SG83055, &UA1953086},
	{SG83056, &UA2221991},
	{SG83058, &UA2445590},
	{SG83060, &UA2844450},
	{SG83070, &UA2537081},
	{SG83071, &UA2555854},
	{SG83072, &UA679584},
	{SG83073, &UA785734},
	{SG83074, &UA1951945},
	{SG83075, &UA1955356},
	{SG83076, &UA2981508},
	{SG83077, &UA625100},
	{SG83078, &UA2943640},
	{SG83079, &UA2424438},
	{SG83084, &UA672808},
	{SG83085, &UA771908},
	{SG83086, &UA705194},
	{SG83087, &UA2967842},
	{SG83088, &UA2970469},
	{SG83089, &UA3030888},
	{SG83093, &UA680965},
	{SG83094, &UA2589348},
	{SG83097, &UA2383874},
	{SG83098, &UA2714343},
	{SG83099, &UA9430},
	{SG83100, &UA15869},
	{SG83101, &UA2422811},
	{SG83102, &UA2422871},
	{SG83103, &UA2504803},
	{SG83104, &UA2696108},
	{SG83106, &UA923952},
	{SG83107, &UA2716454},
	{SG83108, &UA2558428},
	{SG83109, &UA1982872},
	{SG83110, &UA2157267},
	{SG83111, &UA2399588},
	{SG83112, &UA691895},
	{SG83113, &UA1901294},
	{SG83114, &UA2968629},
	{SG83115, &UA2385699},
	{SG83116, &UA80512},
	{SG83125, &UA2531699},
	{SG83126, &UA10241},
	{SG83127, &UA20350},
	{SG83128, &UA52437},
	{SG83129, &UA62684},
	{SG83143, &UA2943539},
	{SG83152, &UA925830},
	{SG83154, &UA2034647},
	{SG83156, &UA14049},
	{SG83157, &UA63309},
	{SG83158, &UA9104},
	{SG83159, &UA21124},
	{SG83160, &UA69777},
	{SG83164, &UA2430089},
	{SG83165, &UA2578775},
	{SG83167, &UA2718546},
	{SG83168, &UA2961579},
	{SG83169, &UA3059919},
	{SG83170, &UA3126383},
	{SG83172, &UA3035649},
	{SG83174, &UA2236208},
	{SG83175, &UA2502968},
	{SG83176, &UA2818702},
	{SG83177, &UA1984746},
	{SG83178, &UA2809009},
	{SG83179, &UA2829938},
	{SG83180, &UA3031416},
	{SG83182, &UA2385870},
	{SG83185, &UA29382},
	{SG83198, &UA44305},
	{SG83207, &UA2969449},
	{SG83208, &UA2969937},
	{SG83209, &UA3016064},
	{SG83210, &UA3057415},
	{SG83211, &UA3057623},
	{SG83212, &UA3078611},
	{SG83213, &UA3107171},
	{SG83224, &UA2737776},
	{SG83225, &UA2970866},
	{SG83226, &UA2258455},
	{SG83228, &UA2510725},
	{SG83229, &UA2698626},
	{SG83230, &UA693671},
	{SG83233, &UA2560008},
	{SG83234, &UA2780471},
	{SG83235, &UA2422365},
	{SG83236, &UA2560370},
	{SG83237, &UA2738175},
	{SG83238, &UA576552},
	{SG83239, &UA679259},
	{SG83240, &UA2195930},
	{SG83243, &UA69326},
	{SG83244, &UA69399},
	{SG83245, &UA249005},
	{SG83246, &UA1955497},
	{SG83247, &UA1964485},
	{SG83248, &UA2166996},
	{SG83249, &UA22937},
	{SG83250, &UA2260677},
	{SG83251, &UA2717067},
	{SG83255, &UA1994973},
	{SG83258, &UA2505836},
	{SG83259, &UA2607483},
	{SG83260, &UA2607494},
	{SG83261, &UA2780267},
	{SG83262, &UA854790},
	{SG83263, &UA886137},
	{SG83264, &UA2410881},
	{SG83265, &UA2813647},
	{SG83267, &UA2967025},
	{SG83268, &UA2460882},
	{SG83269, &UA2848342},
	{SG83270, &UA658466},
	{SG83271, &UA704320},
	{SG83272, &UA2383641},
	{SG83273, &UA2843085},
	{SG83274, &UA69377},
	{SG83275, &UA249004},
	{SG83276, &UA778009},
	{SG83277, &UA691830},
	{SG83278, &UA773193},
	{SG83279, &UA2367629},
	{SG83280, &UA2901179},
	{SG83282, &UA2343375},
	{SG83283, &UA2652409},
	{SG83284, &UA2968004},
	{SG83285, &UA2427025},
	{SG83317, &UA892619},
	{SG83318, &UA896282},
	{SG83319, &UA2420877},
	{SG83320, &UA2652829},
	{SG83321, &UA1994715},
	{SG83322, &UA660666},
	{SG83328, &UA2423289},
	{SG83331, &UA2962406},
	{SG83332, &UA2403729},
	{SG83333, &UA2537960},
	{SG83335, &UA706684},
	{SG83336, &UA893441},
	{SG83337, &UA2503109},
	{SG83338, &UA2360406},
	{SG83339, &UA2456774},
	{SG83340, &UA10202},
	{SG83341, &UA14467},
	{SG83343, &UA2424451},
	{SG83344, &UA2467642},
	{SG83345, &UA2505491},
	{SG83346, &UA2556872},
	{SG83347, &UA2983801},
	{SG83348, &UA2461174},
	{SG83349, &UA2559753},
	{SG83350, &UA895067},
	{SG83351, &UA2162331},
	{SG83352, &UA2167400},
	{SG83353, &UA2965573},
	{SG83354, &UA3060048},
	{SG83355, &UA886715},
	{SG83356, &UA895959},
	{SG83357, &UA2425054},
	{SG83358, &UA2422257},
	{SG83359, &UA2457917},
	{SG83360, &UA2727699},
	{SG83361, &UA2846402},
	{SG83362, &UA1992587},
	{SG83363, &UA2162858},
	{SG83364, &UA2983491},
	{SG83373, &UA2387021},
	{SG83374, &UA2425192},
	{SG83375, &UA14144},
	{SG83377, &UA2424886},
	{SG83381, &UA2508707},
	{SG83382, &UA2429931},
	{SG83386, &UA2944302},
	{SG83387, &UA249288},
	{SG83388, &UA805700},
	{SG83400, &UA2167969},
	{SG83402, &UA421993},
	{SG83405, &UA2717888},
	{SG83406, &UA14106},
	{SG83407, &UA67206},
	{SG83418, &UA3023822},
	{SG83426, &UA2385988},
	{SG83427, &UA2422732},
	{SG83429, &UA2717081},
	{SG83433, &UA2600696},
	{SG83434, &UA2901107},
	{SG83438, &UA2230614},
	{SG83440, &UA3016345},
	{SG83441, &UA3061838},
	{SG83442, &UA2333135},
	{SG83443, &UA2508657},
	{SG83444, &UA68278},
	{SG83445, &UA118861},
	{SG83446, &UA2424436},
	{SG83447, &UA2701489},
	{SG83448, &UA2716725},
	{SG83449, &UA249832},
	{SG83450, &UA342760},
	{SG83462, &UA2738190},
	{SG83463, &UA703694},
	{SG83465, &UA2467258},
	{SG83466, &UA2533205},
	{SG83467, &UA2538345},
	{SG83468, &UA572279},
	{SG83470, &UA2158210},
	{SG83471, &UA2422695},
	{SG83472, &UA2555519},
	{SG83473, &UA2843436},
	{SG83474, &UA625974},
	{SG83475, &UA2345534},
	{SG83484, &UA17269},
	{SG83485, &UA39056},
	{SG83486, &UA2716269},
	{SG83487, &UA3100600},
	{SG83490, &UA14259},
	{SG83492, &UA2530486},
	{SG83493, &UA2607492},
	{SG83494, &UA3100856},
	{SG83495, &UA2226809},
	{SG83496, &UA2239274},
	{SG83497, &UA3034050},
	{SG83498, &UA3125480},
	{SG83500, &UA696639},
	{SG83507, &UA2701228},
	{SG83508, &UA2718555},
	{SG83510, &UA2428397},
	{SG83512, &UA2383196},
	{SG83516, &UA63580},
	{SG83518, &UA330648},
	{SG83520, &UA2751871},
	{SG83521, &UA2683811},
	{SG83522, &UA2506205},
	{SG83523, &UA2961656},
	{SG83524, &UA1960728},
	{SG83525, &UA1994039},
	{SG83526, &UA2828825},
	{SG83527, &UA2714609},
	{SG83528, &UA2969895},
	{SG83529, &UA2533406},
	{SG83530, &UA2342857},
	{SG83531, &UA31288},
	{SG83533, &UA1972674},
	{SG83534, &UA1992283},
	{SG83535, &UA2941633},
	{SG83540, &UA53564},
	{SG83541, &UA67735},
	{SG83544, &UA3033946},
	{SG83545, &UA2260662},
	{SG83546, &UA624778},
	{SG83547, &UA626688},
	{SG83548, &UA2502119},
	{SG83550, &UA62866},
	{SG83551, &UA2376611},
	{SG83555, &UA2660410},
	{SG83556, &UA2854840},
	{SG83557, &UA3030879},
	{SG83558, &UA2405447},
	{SG83559, &UA2559732},
	{SG83560, &UA2581754},
	{SG83561, &UA2705872},
	{SG83562, &UA646066},
	{SG83563, &UA696305},
	{SG83564, &UA2362374},
	{SG83565, &UA2842820},
	{SG83566, &UA2423568},
	{SG83567, &UA2555714},
	{SG83568, &UA2959702},
	{SG83569, &UA3013216},
	{SG83570, &UA2460852},
	{SG83571, &UA2445997},
	{SG83572, &UA2478871},
	{SG83573, &UA2702020},
	{SG83574, &UA2716728},
	{SG83575, &UA2935267},
	{SG83579, &UA2732851},
	{SG83580, &UA2737545},
	{SG83581, &UA725619},
	{SG83585, &UA2422354},
	{SG83586, &UA2462437},
	{SG83587, &UA3088498},
	{SG83588, &UA3032606},
	{SG83591, &UA2429596},
	{SG83592, &UA2967582},
	{SG83593, &UA2973691},
	{SG83594, &UA3057126},
	{SG83595, &UA183413},
	{SG83596, &UA2383640},
	{SG83597, &UA26126},
	{SG83598, &UA2564204},
	{SG83599, &UA249243},
	{SG83600, &UA440425},
	{SG83601, &UA2423529},
	{SG83602, &UA628938},
	{SG83603, &UA2412434},
	{SG83604, &UA3035654},
	{SG83605, &UA2966966},
	{SG83606, &UA2963662},
	{SG83607, &UA15655},
	{SG83608, &UA2560369},
	{SG83609, &UA2961368},
	{SG83610, &UA3056835},
	{SG83611, &UA2972275},
	{SG83612, &UA2385233},
	{SG83614, &UA2406756},
	{SG83615, &UA2698186},
	{SG83616, &UA2984188},
	{SG83618, &UA779192},
	{SG83619, &UA1947509},
	{SG83620, &UA2505356},
	{SG83621, &UA2459057},
	{SG83624, &UA2429591},
	{SG83625, &UA68553},
	{SG83628, &UA2560181},
	{SG83629, &UA2718792},
	{SG83630, &UA656745},
	{SG83631, &UA2167501},
	{SG83632, &UA2651272},
	{SG83635, &UA2459568},
	{SG83636, &UA2558465},
	{SG83642, &UA2963943},
	{SG83643, &UA3067000},
	{SG83644, &UA3018014},
	{SG83645, &UA3059789},
	{SG83646, &UA2967610},
	{SG83651, &UA2560092},
	{SG83652, &UA2765730},
	{SG83653, &UA2967282},
	{SG83654, &UA397137},
	{SG83655, &UA2810588},
	{SG83659, &UA11299},
	{SG83660, &UA62143},
	{SG83661, &UA63314},
	{SG83663, &UA2365772},
	{SG83664, &UA3062103},
	{SG83665, &UA2168821},
	{SG83666, &UA2440987},
	{SG83667, &UA2552023},
	{SG83668, &UA436422},
	{SG83672, &UA2451079},
	{SG83673, &UA2533202},
	{SG83674, &UA2597111},
	{SG83685, &UA2710448},
	{SG83686, &UA5792},
	{SG83687, &UA30253},
	{SG83688, &UA2383344},
	{SG83714, &UA3029927},
	{SG83715, &UA3062277},
	{SG83716, &UA2237165},
	{SG83717, &UA2835443},
	{SG83719, &UA630459},
	{SG83720, &UA2581274},
	{SG83721, &UA2661155},
	{SG83722, &UA663158},
	{SG83723, &UA2433085},
	{SG83724, &UA2383571},
	{SG83725, &UA2962538},
	{SG83728, &UA2513849},
	{SG83730, &UA8337},
	{SG83731, &UA9539},
	{SG83732, &UA9890},
	{SG83733, &UA64710},
	{SG83734, &UA68330},
	{SG83735, &UA69524},
	{SG83736, &UA69406},
	{SG83737, &UA379627},
	{SG83748, &UA2428292},
	{SG83751, &UA2971183},
	{SG83752, &UA1121},
	{SG83753, &UA17043},
	{SG83754, &UA2425762},
	{SG83755, &UA69273},
	{SG83756, &UA249661},
	{SG83757, &UA645893},
	{SG83758, &UA2419675},
	{SG83759, &UA2423730},
	{SG83760, &UA2943145},
	{SG83763, &UA3033036},
	{SG83764, &UA2382580},
	{SG83765, &UA2738018},
	{SG83767, &UA2165409},
	{SG83769, &UA676547},
	{SG83771, &UA68944},
	{SG83772, &UA790494},
	{SG83776, &UA2422582},
	{SG83777, &UA2563073},
	{SG83778, &UA2660856},
	{SG83779, &UA2458114},
	{SG83780, &UA2387596},
	{SG83781, &UA2968609},
	{SG83782, &UA2598749},
	{SG83783, &UA2855908},
	{SG83788, &UA396178},
	{SG83789, &UA2531126},
	{SG83790, &UA3102541},
	{SG83792, &UA659136},
	{SG83795, &UA2427057},
	{SG83798, &UA3096463},
	{SG83799, &UA3111046},
	{SG83800, &UA66960},
	{SG83801, &UA627442},
	{SG83802, &UA643270},
	{SG83804, &UA2717072},
	{SG83805, &UA2652624},
	{SG83806, &UA64526},
	{SG83807, &UA654825},
	{SG83808, &UA2383904},
	{SG83809, &UA2499588},
	{SG83810, &UA2373811},
	{SG83811, &UA2558272},
	{SG83812, &UA2581248},
	{SG83822, &UA2362382},
	{SG83823, &UA2426393},
	{SG83824, &UA2422729},
	{SG83825, &UA2504128},
	{SG83826, &UA2777835},
	{SG83827, &UA2536791},
	{SG83837, &UA2162646},
	{SG83838, &UA2530770},
	{SG83839, &UA2858323},
	{SG83840, &UA660076},
	{SG83841, &UA2478046},
	{SG83842, &UA2531826},
	{SG83843, &UA2580676},
	{SG83844, &UA3045763},
	{SG83845, &UA2702532},
	{SG83846, &UA2708080},
	{SG83847, &UA3050006},
	{SG83848, &UA3059885},
	{SG83858, &UA2261304},
	{SG83860, &UA2386012},
	{SG83862, &UA2670141},
	{SG83863, &UA2702194},
	{SG83864, &UA2708732},
	{SG83865, &UA2427143},
	{SG83866, &UA2969150},
	{SG83867, &UA3059128},
	{SG83868, &UA653118},
	{SG83869, &UA675239},
	{SG83870, &UA896236},
	{SG83881, &UA2457504},
	{SG83882, &UA2383911},
	{SG83883, &UA250064},
	{SG83884, &UA421159},
	{SG83885, &UA2799332},
	{SG83887, &UA3030991},
	{SG83888, &UA64781},
	{SG83889, &UA572659},
	{SG83892, &UA2961138},
	{SG83893, &UA1989458},
	{SG83894, &UA2594719},
	{SG83896, &UA2426874},
	{SG83897, &UA2970558},
	{SG83898, &UA2971188},
	{SG83899, &UA2352195},
	{SG83900, &UA2536347},
	{SG83901, &UA2218050},
	{SG83902, &UA2361462},
	{SG83909, &UA249228},
	{SG83910, &UA2564301},
	{SG83912, &UA2970813},
	{SG83914, &UA2424382},
	{SG83918, &UA16305},
	{SG83919, &UA2578594},
	{SG83920, &UA2768110},
	{SG83921, &UA2852564},
	{SG83922, &UA2878948},
	{SG83923, &UA3121121},
	{SG83928, &UA712770},
	{SG83929, &UA3085097},
	{SG83931, &UA148838},
	{SG83932, &UA8858},
	{SG83933, &UA9116},
	{SG83934, &UA2482953},
	{SG83935, &UA2747294},
	{SG83936, &UA18805},
	{SG83937, &UA52798},
	{SG83938, &UA68239},
	{SG83939, &UA2717080},
	{SG83940, &UA3057945},
	{SG83941, &UA61738},
	{SG83942, &UA3057498},
	{SG83943, &UA3060838},
	{SG83945, &UA2225146},
	{SG83946, &UA2382196},
	{SG83947, &UA2353883},
	{SG83948, &UA2486952},
	{SG83967, &UA1994086},
	{SG83971, &UA767856},
	{SG83972, &UA2401307},
	{SG83973, &UA2386086},
	{SG83974, &UA676311},
	{SG83975, &UA2375852},
	{SG83980, &UA3033336},
	{SG83981, &UA1993547},
	{SG83982, &UA2154812},
	{SG83983, &UA699282},
	{SG83984, &UA2392782},
	{SG83985, &UA1960516},
	{SG83986, &UA2339580},
	{SG83987, &UA2159925},
	{SG83988, &UA62124},
	{SG83989, &UA63635},
	{SG83990, &UA2469511},
	{SG83991, &UA2510221},
	{SG83992, &UA3032081},
	{SG84006, &UA2607493},
	{SG84007, &UA2656103},
	{SG84008, &UA2663317},
	{SG84009, &UA72940},
	{SG84012, &UA2456332},
	{SG84013, &UA2368084},
	{SG84014, &UA2973999},
	{SG84015, &UA3034501},
	{SG84016, &UA25964},
	{SG84017, &UA421559},
	{SG84018, &UA573913},
	{SG84020, &UA2422839},
	{SG84021, &UA1964821},
	{SG84022, &UA2529286},
	{SG84023, &UA2714987},
	{SG84024, &UA2961824},
	{SG84028, &UA2796763},
	{SG84029, &UA21944},
	{SG84030, &UA21947},
	{SG84031, &UA680007},
	{SG84032, &UA772776},
	{SG84033, &UA1946774},
	{SG84034, &UA2654335},
	{SG84035, &UA2700046},
	{SG84036, &UA2737809},
	{SG84037, &UA2968867},
	{SG84039, &UA2968085},
	{SG84040, &UA55877},
	{SG84043, &UA791320},
	{SG84044, &UA895104},
	{SG84060, &UA891727},
	{SG84066, &UA3063148},
	{SG84067, &UA2368079},
	{SG84069, &UA2673884},
	{SG84070, &UA2738017},
	{SG84071, &UA2751542},
	{SG84072, &UA2860259},
	{SG84073, &UA67017},
	{SG84074, &UA68515},
	{SG84075, &UA69527},
	{SG84076, &UA650203},
	{SG84077, &UA600634},
	{SG84079, &UA2481683},
	{SG84080, &UA2510611},
	{SG84083, &UA765200},
	{SG84084, &UA2957911},
	{SG84085, &UA2971045},
	{SG84086, &UA2963057},
	{SG84087, &UA397216},
	{SG84088, &UA895055},
	{SG84089, &UA2154576},
	{SG84090, &UA2545807},
	{SG84091, &UA1970835},
	{SG84092, &UA2223474},
	{SG84093, &UA2382714},
	{SG84094, &UA735593},
	{SG84095, &UA2351380},
	{SG84096, &UA2559162},
	{SG84097, &UA2794240},
	{SG84098, &UA2752458},
	{SG84105, &UA2383579},
	{SG84106, &UA2739207},
	{SG84107, &UA2997819},
	{SG84108, &UA68411},
	{SG84109, &UA1892492},
	{SG84110, &UA2589449},
	{SG84111, &UA2337087},
	{SG84112, &UA2261419},
	{SG84113, &UA2310446},
	{SG84114, &UA644075},
	{SG84115, &UA803093},
	{SG84117, &UA2707035},
	{SG84121, &UA2403648},
	{SG84122, &UA2431506},
	{SG84123, &UA24621},
	{SG84124, &UA2225585},
	{SG84125, &UA2424817},
	{SG84126, &UA62187},
	{SG84128, &UA2385875},
	{SG84129, &UA2560166},
	{SG84134, &UA2401263},
	{SG84138, &UA2510614},
	{SG84141, &UA1991342},
	{SG84142, &UA2372061},
	{SG84143, &UA2401266},
	{SG84144, &UA2581756},
	{SG84146, &UA15377},
	{SG84147, &UA695415},
	{SG84148, &UA2737755},
	{SG84149, &UA2779421},
	{SG84150, &UA2770713},
	{SG84151, &UA2784990},
	{SG84152, &UA647198},
	{SG84153, &UA2345289},
	{SG84154, &UA2738469},
	{SG84155, &UA16376},
	{SG84159, &UA2581366},
	{SG84160, &UA3061094},
	{SG84161, &UA1943965},
	{SG84163, &UA2464717},
	{SG84164, &UA68828},
	{SG84165, &UA450620},
	{SG84166, &UA628409},
	{SG84167, &UA852966},
	{SG84168, &UA2151750},
	{SG84169, &UA2484761},
	{SG84172, &UA2737925},
	{SG84173, &UA3017982},
	{SG84177, &UA895660},
	{SG84178, &UA2223048},
	{SG84180, &UA3061661},
	{SG84184, &UA2974228},
	{SG84189, &UA671422},
	{SG84190, &UA2737922},
	{SG84191, &UA18859},
	{SG84192, &UA657468},
	{SG84193, &UA695138},
	{SG84194, &UA2373472},
	{SG84195, &UA2560782},
	{SG84199, &UA3060451},
	{SG84202, &UA63690},
	{SG84204, &UA249241},
	{SG84205, &UA572703},
	{SG84206, &UA573861},
	{SG84207, &UA805807},
	{SG84208, &UA1948665},
	{SG84209, &UA2469138},
	{SG84210, &UA1995941},
	{SG84211, &UA2810061},
	{SG84212, &UA2417663},
	{SG84219, &UA2554984},
	{SG84220, &UA2319345},
	{SG84221, &UA2551955},
	{SG84222, &UA64849},
	{SG84223, &UA572715},
	{SG84224, &UA678997},
	{SG84225, &UA690919},
	{SG84226, &UA2260760},
	{SG84227, &UA2383132},
	{SG84228, &UA2545390},
	{SG84229, &UA2659394},
	{SG84230, &UA2661670},
	{SG84235, &UA2964605},
	{SG84236, &UA44694},
	{SG84237, &UA2290654},
	{SG84242, &UA658400},
	{SG84243, &UA2167051},
	{SG84244, &UA69296},
	{SG84245, &UA804547},
	{SG84246, &UA2893501},
	{SG84247, &UA2958319},
	{SG84248, &UA3017988},
	{SG84249, &UA2554459},
	{SG84250, &UA3079400},
	{SG84252, &UA2431262},
	{SG84254, &UA2752242},
	{SG84256, &UA14001},
	{SG84257, &UA16239},
	{SG84258, &UA16240},
	{SG84259, &UA34453},
	{SG84260, &UA69174},
	{SG84261, &UA2342615},
	{SG84266, &UA14116},
	{SG84267, &UA14119},
	{SG84268, &UA1995660},
	{SG84271, &UA2564380},
	{SG84272, &UA2527260},
	{SG84273, &UA2781512},
	{SG84274, &UA894960},
	{SG84275, &UA2221328},
	{SG84276, &UA2375452},
	{SG84277, &UA895075},
	{SG84278, &UA2400783},
	{SG84279, &UA2383573},
	{SG84280, &UA2599687},
	{SG84281, &UA3017314},
	{SG84282, &UA3079808},
	{SG84283, &UA3080627},
	{SG84284, &UA2970994},
	{SG84286, &UA1967073},
	{SG84287, &UA3030922},
	{SG84288, &UA397480},
	{SG84289, &UA1905970},
	{SG84295, &UA2156695},
	{SG84296, &UA2382514},
	{SG84297, &UA2483525},
	{SG84299, &UA2420705},
	{SG84300, &UA2558430},
	{SG84301, &UA2718798},
	{SG84302, &UA2969133},
	{SG84303, &UA2384218},
	{SG84304, &UA2431003},
	{SG84305, &UA2469720},
	{SG84306, &UA3033565},
	{SG84307, &UA3057806},
	{SG84308, &UA170861},
	{SG84309, &UA2467690},
	{SG84311, &UA69308},
	{SG84312, &UA342758},
	{SG84313, &UA1955668},
	{SG84314, &UA1952087},
	{SG84315, &UA1991855},
	{SG84317, &UA2260187},
	{SG84319, &UA2485091},
	{SG84320, &UA2558861},
	{SG84321, &UA2694635},
	{SG84322, &UA2674344},
	{SG84323, &UA2962185},
	{SG84324, &UA577982},
	{SG84325, &UA572047},
	{SG84326, &UA772494},
	{SG84327, &UA2262092},
	{SG84328, &UA2357115},
	{SG84329, &UA3018882},
	{SG84330, &UA573003},
	{SG84331, &UA634458},
	{SG84333, &UA2457649},
	{SG84334, &UA2424732},
	{SG84338, &UA2880226},
	{SG84340, &UA3111736},
	{SG84341, &UA626359},
	{SG84344, &UA795206},
	{SG84345, &UA2311371},
	{SG84348, &UA2738147},
	{SG84349, &UA2962030},
	{SG84355, &UA707219},
	{SG84356, &UA2162531},
	{SG84357, &UA2581304},
	{SG84358, &UA2747189},
	{SG84359, &UA44312},
	{SG84362, &UA2385855},
	{SG84363, &UA2382458},
	{SG84364, &UA2581183},
	{SG84365, &UA2970756},
	{SG84366, &UA147472},
	{SG84367, &UA2428365},
	{SG84369, &UA248938},
	{SG84370, &UA2350029},
	{SG84371, &UA2505662},
	{SG84372, &UA3039306},
	{SG84373, &UA2352322},
	{SG84374, &UA2563130},
	{SG84375, &UA2659028},
	{SG84376, &UA68758},
	{SG84377, &UA569423},
	{SG84379, &UA2504861},
	{SG84380, &UA2562722},
	{SG84381, &UA64440},
	{SG84382, &UA572001},
	{SG84384, &UA2351352},
	{SG84385, &UA2664286},
	{SG84386, &UA2728583},
	{SG84387, &UA2507155},
	{SG84389, &UA2675414},
	{SG84390, &UA2737535},
	{SG84391, &UA2744838},
	{SG84392, &UA3079319},
	{SG84393, &UA69581},
	{SG84394, &UA2233391},
	{SG84395, &UA2357297},
	{SG84397, &UA3058637},
	{SG84398, &UA3059747},
	{SG84399, &UA2166599},
	{SG84400, &UA2660920},
	{SG84401, &UA2937198},
	{SG84402, &UA2730225},
	{SG84403, &UA2738150},
	{SG84404, &UA2738236},
	{SG84405, &UA3016239},
	{SG84406, &UA68658},
	{SG84407, &UA69517},
	{SG84408, &UA87727},
	{SG84409, &UA2386774},
	{SG84410, &UA656344},
	{SG84414, &UA2579874},
	{SG84415, &UA67967},
	{SG84416, &UA581182},
	{SG84418, &UA2385246},
	{SG84419, &UA2869690},
	{SG84421, &UA2430119},
	{SG84422, &UA2697094},
	{SG84423, &UA2537991},
	{SG84424, &UA2770267},
	{SG84425, &UA2603887},
	{SG84426, &UA2506916},
	{SG84427, &UA2555578},
	{SG84428, &UA3079918},
	{SG84430, &UA2384366},
	{SG84431, &UA2451712},
	{SG84433, &UA3059983},
	{SG84434, &UA679436},
	{SG84435, &UA2216293},
	{SG84436, &UA1980333},
	{SG84437, &UA2404746},
	{SG84438, &UA2465485},
	{SG84439, &UA2733047},
	{SG84440, &UA3059298},
	{SG84441, &UA703020},
	{SG84443, &UA21774},
	{SG84444, &UA2219196},
	{SG84445, &UA2403549},
	{SG84446, &UA2485888},
	{SG84447, &UA3060255},
	{SG84449, &UA7564},
	{SG84450, &UA7566},
	{SG84451, &UA1900446},
	{SG84452, &UA2236869},
	{SG84455, &UA2674320},
	{SG84457, &UA640773},
	{SG84458, &UA676487},
	{SG84459, &UA2338853},
	{SG84460, &UA1987306},
	{SG84461, &UA2393508},
	{SG84462, &UA2504970},
	{SG84463, &UA2565154},
	{SG84466, &UA68613},
	{SG84467, &UA765659},
	{SG84468, &UA2328390},
	{SG84480, &UA69314},
	{SG84482, &UA343638},
	{SG84486, &UA2528563},
	{SG84487, &UA2554297},
	{SG84488, &UA3009918},
	{SG84489, &UA2158013},
	{SG84490, &UA2776012},
	{SG84491, &UA895892},
	{SG84492, &UA1971969},
	{SG84495, &UA62368},
	{SG84496, &UA2158773},
	{SG84497, &UA2372971},
	{SG84498, &UA2644501},
	{SG84499, &UA2659572},
	{SG84501, &UA2752372},
	{SG84502, &UA3013691},
	{SG84503, &UA8890},
	{SG84504, &UA2507029},
	{SG84506, &UA3058896},
	{SG84509, &UA2261186},
	{SG84510, &UA2387829},
	{SG84512, &UA2942925},
	{SG84513, &UA3111604},
	{SG84514, &UA3015428},
	{SG84515, &UA3019373},
	{SG84516, &UA3126896},
	{SG84518, &UA2313989},
	{SG84519, &UA2520544},
	{SG84520, &UA2695365},
	{SG84522, &UA3014974},
	{SG84523, &UA896409},
	{SG84524, &UA1994000},
	{SG84525, &UA2261424},
	{SG84527, &UA2375609},
	{SG84528, &UA1943694},
	{SG84529, &UA2374185},
	{SG84532, &UA2559399},
	{SG84533, &UA2763602},
	{SG84534, &UA2662143},
	{SG84535, &UA2698954},
	{SG84538, &UA2528709},
	{SG84539, &UA2878197},
	{SG84540, &UA3043929},
	{SG84541, &UA3070881},
	{SG84542, &UA2752325},
	{SG84543, &UA2781577},
	{SG84544, &UA2855894},
	{SG84545, &UA638773},
	{SG84546, &UA713553},
	{SG84547, &UA2352037},
	{SG84548, &UA2457711},
	{SG84549, &UA2958474},
	{SG84552, &UA379757},
	{SG84553, &UA397226},
	{SG84556, &UA2164744},
	{SG84572, &UA2960443},
	{SG84576, &UA30331},
	{SG84577, &UA64841},
	{SG84578, &UA634688},
	{SG84579, &UA2383967},
	{SG84580, &UA2527425},
	{SG84583, &UA2781733},
	{SG84584, &UA3020184},
	{SG84585, &UA70686},
	{SG84586, &UA2518426},
	{SG84587, &UA1933528},
	{SG84588, &UA2168263},
	{SG84589, &UA2382172},
	{SG84590, &UA2506731},
	{SG84591, &UA3058066},
	{SG84592, &UA2318436},
	{SG84593, &UA2386147},
	{SG84596, &UA2458580},
	{SG84598, &UA2467170},
	{SG84599, &UA3118178},
	{SG84600, &UA2738181},
	{SG84601, &UA3110677},
	{SG84603, &UA2382513},
	{SG84605, &UA2675426},
	{SG84607, &UA3018427},
	{SG84608, &UA3034497},
	{SG84609, &UA2351386},
	{SG84610, &UA2127727},
	{SG84612, &UA2000863},
	{SG84613, &UA2768944},
	{SG84615, &UA3128088},
	{SG84617, &UA2643868},
	{SG84618, &UA2362990},
	{SG84619, &UA2379483},
	{SG84620, &UA2481605},
	{SG84621, &UA579071},
	{SG84622, &UA2366602},
	{SG84623, &UA2694504},
	{SG84625, &UA2738223},
	{SG84626, &UA2752436},
	{SG84630, &UA2385057},
	{SG84631, &UA2503582},
	{SG84632, &UA2705816},
	{SG84633, &UA2961139},
	{SG84634, &UA2963741},
	{SG84635, &UA2969841},
	{SG84636, &UA3015944},
	{SG84637, &UA3021677},
	{SG84638, &UA3059093},
	{SG84639, &UA56807},
	{SG84640, &UA69179},
	{SG84641, &UA69206},
	{SG84642, &UA692276},
	{SG84643, &UA2261294},
	{SG84644, &UA2351371},
	{SG84645, &UA2380035},
	{SG84646, &UA2941510},
	{SG84648, &UA3076830},
	{SG84649, &UA3081203},
	{SG84650, &UA572242},
	{SG84651, &UA2171632},
	{SG84652, &UA2385299},
	{SG84653, &UA2556613},
	{SG84654, &UA2597535},
	{SG84659, &UA2752841},
	{SG84660, &UA3081200},
	{SG84661, &UA64615},
	{SG84662, &UA895056},
	{SG84670, &UA2752016},
	{SG84675, &UA2797447},
	{SG84683, &UA197462},
	{SG84684, &UA2212659},
	{SG84685, &UA249918},
	{SG84686, &UA893331},
	{SG84687, &UA1940649},
	{SG84688, &UA2504977},
	{SG84690, &UA2768912},
	{SG84691, &UA3014849},
	{SG84692, &UA898586},
	{SG84693, &UA2154039},
	{SG84694, &UA2469499},
	{SG84695, &UA2579511},
	{SG84696, &UA2504941},
	{SG84698, &UA62622},
	{SG84700, &UA88432},
	{SG84701, &UA2433176},
	{SG84702, &UA2699599},
	{SG84703, &UA2738035},
	{SG84704, &UA2752592},
	{SG84706, &UA3059197},
	{SG84708, &UA3081642},
	{SG84709, &UA68755},
	{SG84710, &UA68745},
	{SG84711, &UA656154},
	{SG84712, &UA69473},
	{SG84713, &UA379664},
	{SG84714, &UA667527},
	{SG84716, &UA2172434},
	{SG84720, &UA2762737},
	{SG84722, &UA69357},
	{SG84723, &UA2594271},
	{SG84724, &UA2467211},
	{SG84725, &UA2879182},
	{SG84727, &UA67879},
	{SG84729, &UA571680},
	{SG84731, &UA2344029},
	{SG84732, &UA2652468},
	{SG84733, &UA2751922},
	{SG84734, &UA2752197},
	{SG84735, &UA3015666},
	{SG84736, &UA3078665},
	{SG84737, &UA21854},
	{SG84738, &UA681626},
	{SG84740, &UA3080391},
	{SG84741, &UA1936345},
	{SG84742, &UA897744},
	{SG84743, &UA2372896},
	{SG84744, &UA2169791},
	{SG84745, &UA2260449},
	{SG84746, &UA2420625},
	{SG84747, &UA2528708},
	{SG84748, &UA2752074},
	{SG84749, &UA572448},
	{SG84750, &UA2420677},
	{SG84751, &UA2483915},
	{SG84752, &UA2554122},
	{SG84754, &UA624642},
	{SG84758, &UA677542},
	{SG84759, &UA1981087},
	{SG84760, &UA2373375},
	{SG84764, &UA2752551},
	{SG84769, &UA67933},
	{SG84770, &UA67964},
	{SG84771, &UA68353},
	{SG84772, &UA202534},
	{SG84773, &UA321194},
	{SG84774, &UA565855},
	{SG84775, &UA2804701},
	{SG84777, &UA688549},
	{SG84778, &UA2979868},
	{SG84779, &UA2934063},
	{SG84780, &UA2227392},
	{SG84781, &UA2458655},
	{SG84782, &UA2950797},
	{SG84784, &UA3026917},
	{SG84786, &UA53626},
	{SG84787, &UA2338604},
	{SG84788, &UA2484563},
	{SG84789, &UA2676580},
	{SG84790, &UA2737857},
	{SG84791, &UA2752591},
	{SG84792, &UA2892826},
	{SG84793, &UA2768855},
	{SG84795, &UA3021052},
	{SG84797, &UA56771},
	{SG84798, &UA452419},
	{SG84799, &UA816150},
	{SG84800, &UA2375585},
	{SG84801, &UA2375667},
	{SG84802, &UA2456088},
	{SG84803, &UA2703574},
	{SG84804, &UA2767102},
	{SG84806, &UA2745651},
	{SG84808, &UA3015351},
	{SG84809, &UA3077785},
	{SG84810, &UA3080299},
	{SG84811, &UA24691},
	{SG84813, &UA2752882},
	{SG84814, &UA2866596},
	{SG84815, &UA2936956},
	{SG84816, &UA3027383},
	{SG84820, &UA663155},
	{SG84821, &UA895882},
	{SG84822, &UA1968604},
	{SG84823, &UA2198290},
	{SG84824, &UA2351434},
	{SG84828, &UA2695173},
	{SG84830, &UA2752285},
	{SG84831, &UA2763987},
	{SG84832, &UA2992740},
	{SG84838, &UA61931},
	{SG84842, &UA67946},
	{SG84843, &UA2462004},
	{SG84844, &UA2644693},
	{SG84845, &UA3058595},
	{SG84846, &UA3062126},
	{SG84847, &UA69117},
	{SG84848, &UA701817},
	{SG84850, &UA3043654},
	{SG84851, &UA2486086},
	{SG84852, &UA2833399},
	{SG84853, &UA2768111},
	{SG84855, &UA2940287},
	{SG84856, &UA3034290},
	{SG84858, &UA3093323},
	{SG84859, &UA3109147},
	{SG84860, &UA3129702},
	{SG84861, &UA757403},
	{SG84862, &UA572088},
	{SG84863, &UA649101},
	{SG84864, &UA2156636},
	{SG84865, &UA2339502},
	{SG84866, &UA2383454},
	{SG84868, &UA3015230},
	{SG84870, &UA3126492},
	{SG84871, &UA16011},
	{SG84872, &UA63153},
	{SG84873, &UA56802},
	{SG84874, &UA103294},
	{SG84876, &UA397183},
	{SG84877, &UA2339808},
	{SG84878, &UA2654619},
	{SG84879, &UA2951468},
	{SG84883, &UA3079796},
	{SG84889, &UA754942},
	{SG84890, &UA2768717},
	{SG84891, &UA2935392},
	{SG84892, &UA2935511},
	{SG84893, &UA2986821},
	{SG84896, &UA3058895},
	{SG84897, &UA56151},
	{SG84898, &UA69287},
	{SG84899, &UA2694482},
	{SG84900, &UA3059172},
	{SG84901, &UA3108665},
	{SG84902, &UA719039},
	{SG84903, &UA50379},
	{SG84904, &UA68614},
	{SG84906, &UA774445},
	{SG84910, &UA2150354},
	{SG84911, &UA2216822},
	{SG84913, &UA2526146},
	{SG84914, &UA2663943},
	{SG84920, &UA2795431},
	{SG84933, &UA2351311},
	{SG84937, &UA2451477},
	{SG84945, &UA574366},
	{SG84946, &UA1886143},
	{SG84949, &UA819202},
	{SG84950, &UA1991386},
	{SG84951, &UA2165180},
	{SG84952, &UA2794742},
	{SG84953, &UA2857818},
	{SG84954, &UA3020166},
	{SG84955, &UA3090616},
	{SG84956, &UA68695},
	{SG84957, &UA773419},
	{SG84958, &UA379836},
	{SG84959, &UA2144355},
	{SG84960, &UA2424767},
	{SG84961, &UA2487067},
	{SG84962, &UA3060685},
	{SG84964, &UA16698},
	{SG84972, &UA2353097},
	{SG84974, &UA2399137},
	{SG84975, &UA2504765},
	{SG84979, &UA2379107},
	{SG84980, &UA896649},
	{SG84981, &UA2497059},
	{SG84982, &UA1945024},
	{SG84983, &UA2162485},
	{SG84984, &UA2412447},
	{SG84985, &UA2418812},
	{SG84986, &UA2459123},
	{SG84987, &UA2485093},
	{SG84988, &UA3011540},
	{SG84989, &UA3064175},
	{SG84990, &UA2521},
	{SG84991, &UA21407},
	{SG84992, &UA68373},
	{SG84994, &UA764246},
	{SG84995, &UA1936859},
	{SG84996, &UA2170773},
	{SG84999, &UA2665073},
	{SG85001, &UA2768661},
	{SG85002, &UA2937720},
	{SG85003, &UA3016426},
	{SG85005, &UA3081668},
	{SG85006, &UA22081},
	{SG85007, &UA50489},
	{SG85008, &UA67877},
	{SG85009, &UA2979258},
	{SG85011, &UA437188},
	{SG85012, &UA571118},
	{SG85013, &UA899074},
	{SG85014, &UA2759046},
	{SG85016, &UA2659098},
	{SG85017, &UA2659764},
	{SG85018, &UA2752739},
	{SG85019, &UA2781802},
	{SG85020, &UA2863398},
	{SG85021, &UA2868258},
	{SG85022, &UA2957008},
	{SG85023, &UA64259},
	{SG85024, &UA161637},
	{SG85025, &UA842712},
	{SG85027, &UA2167777},
	{SG85028, &UA2402354},
	{SG85029, &UA2777576},
	{SG85030, &UA2768596},
	{SG85035, &UA3127320},
	{SG85037, &UA69430},
	{SG85038, &UA2212874},
	{SG85039, &UA311142},
	{SG85040, &UA729069},
	{SG85042, &UA2487224},
	{SG85045, &UA8645},
	{SG85046, &UA1969562},
	{SG85047, &UA2387194},
	{SG85055, &UA2958100},
	{SG85056, &UA3108355},
	{SG85059, &UA61991},
	{SG85060, &UA50232},
	{SG85061, &UA67209},
	{SG85062, &UA68137},
	{SG85063, &UA684986},
	{SG85064, &UA717973},
	{SG85065, &UA2686641},
	{SG85066, &UA2579514},
	{SG85067, &UA2550949},
	{SG85068, &UA2660251},
	{SG85069, &UA3062197},
	{SG85070, &UA68171},
	{SG85071, &UA68629},
	{SG85072, &UA379663},
	{SG85073, &UA675227},
	{SG85074, &UA713352},
	{SG85075, &UA573495},
	{SG85076, &UA721813},
	{SG85077, &UA2761021},
	{SG85078, &UA2383435},
	{SG85079, &UA2410788},
	{SG85087, &UA2762535},
	{SG85100, &UA3115105},
	{SG85104, &UA68992},
	{SG85105, &UA556308},
	{SG85106, &UA571844},
	{SG85108, &UA2360225},
	{SG85109, &UA2577625},
	{SG85110, &UA2579541},
	{SG85111, &UA690271},
	{SG85112, &UA2314488},
	{SG85113, &UA2372016},
	{SG85114, &UA2485210},
	{SG85115, &UA2553915},
	{SG85117, &UA2766796},
	{SG85118, &UA2768674},
	{SG85119, &UA2781739},
	{SG85120, &UA2790388},
	{SG85121, &UA2892675},
	{SG85124, &UA31257},
	{SG85125, &UA54788},
	{SG85126, &UA673309},
	{SG85127, &UA2332855},
	{SG85128, &UA2372921},
	{SG85129, &UA2174872},
	{SG85130, &UA2502035},
	{SG85131, &UA2816139},
	{SG85132, &UA2487621},
	{SG85133, &UA2889619},
	{SG85134, &UA3093623},
	{SG85135, &UA61476},
	{SG85136, &UA30506},
	{SG85137, &UA62151},
	{SG85138, &UA87679},
	{SG85139, &UA756747},
	{SG85140, &UA2329228},
	{SG85142, &UA2696874},
	{SG85143, &UA2579301},
	{SG85145, &UA2781398},
	{SG85146, &UA3019366},
	{SG85147, &UA2795004},
	{SG85148, &UA2840702},
	{SG85149, &UA2841619},
	{SG85150, &UA3018910},
	{SG85151, &UA3079382},
	{SG85152, &UA3093440},
	{SG85153, &UA68643},
	{SG85154, &UA62142},
	{SG85155, &UA68069},
	{SG85156, &UA68023},
	{SG85161, &UA2169798},
	{SG85162, &UA2933629},
	{SG85163, &UA2818527},
	{SG85168, &UA2935156},
	{SG85169, &UA3010241},
	{SG85170, &UA3015062},
	{SG85171, &UA3016126},
	{SG85174, &UA9298},
	{SG85175, &UA787658},
	{SG85176, &UA2157976},
	{SG85177, &UA2385197},
	{SG85178, &UA2425398},
	{SG85179, &UA2431369},
	{SG85180, &UA2659558},
	{SG85181, &UA2870025},
	{SG85184, &UA3068993},
	{SG85185, &UA3092841},
	{SG85188, &UA543879},
	{SG85189, &UA68821},
	{SG85190, &UA818923},
	{SG85191, &UA1954718},
	{SG85192, &UA2166684},
	{SG85193, &UA2220720},
	{SG85194, &UA2441089},
	{SG85195, &UA2365559},
	{SG85196, &UA2422036},
	{SG85197, &UA2462439},
	{SG85198, &UA2528633},
	{SG85199, &UA2700575},
	{SG85208, &UA3058334},
	{SG85209, &UA3107719},
	{SG85210, &UA69336},
	{SG85211, &UA2300411},
	{SG85212, &UA2168351},
	{SG85213, &UA2416228},
	{SG85214, &UA2696373},
	{SG85215, &UA2732575},
	{SG85216, &UA2795470},
	{SG85217, &UA2796252},
	{SG85218, &UA3108057},
	{SG85223, &UA2144883},
	{SG85224, &UA629832},
	{SG85226, &UA2365356},
	{SG85227, &UA2262247},
	{SG85228, &UA2384472},
	{SG85229, &UA2386185},
	{SG85233, &UA2560665},
	{SG85234, &UA2600137},
	{SG85235, &UA3043328},
	{SG85237, &UA3080139},
	{SG85238, &UA3128483},
	{SG85239, &UA15988},
	{SG85240, &UA2307005},
	{SG85241, &UA893619},
	{SG85242, &UA1952487},
	{SG85243, &UA2813904},
	{SG85244, &UA2560059},
	{SG85245, &UA2781792},
	{SG85246, &UA3043176},
	{SG85247, &UA3060072},
	{SG85248, &UA3092687},
	{SG85249, &UA18797},
	{SG85250, &UA68320},
	{SG85251, &UA2135207},
	{SG85252, &UA68427},
	{SG85253, &UA2978092},
	{SG85254, &UA460843},
	{SG85255, &UA575421},
	{SG85256, &UA2311937},
	{SG85257, &UA696551},
	{SG85258, &UA704794},
	{SG85259, &UA905513},
	{SG85260, &UA2695237},
	{SG85261, &UA2404701},
	{SG85262, &UA2595107},
	{SG85263, &UA2794356},
	{SG85264, &UA3015876},
	{SG85265, &UA3059046},
	{SG85266, &UA3059559},
	{SG85267, &UA3093408},
	{SG85268, &UA31787},
	{SG85269, &UA69432},
	{SG85270, &UA648411},
	{SG85271, &UA201058},
	{SG85272, &UA700337},
	{SG85273, &UA1958952},
	{SG85274, &UA2445585},
	{SG85276, &UA3041707},
	{SG85277, &UA2168137},
	{SG85278, &UA2403923},
	{SG85281, &UA3044444},
	{SG85282, &UA2990645},
	{SG85283, &UA2781881},
	{SG85284, &UA2826768},
	{SG85286, &UA3018300},
	{SG85287, &UA3060352},
	{SG85292, &UA3092640},
	{SG85295, &UA3093265},
	{SG85297, &UA26342},
	{SG85298, &UA31981},
	{SG85299, &UA69244},
	{SG85300, &UA687493},
	{SG85301, &UA555806},
	{SG85302, &UA2145658},
	{SG85303, &UA2303432},
	{SG85304, &UA2368006},
	{SG85305, &UA2371888},
	{SG85306, &UA2384362},
	{SG85307, &UA2421965},
	{SG85312, &UA3129561},
	{SG85313, &UA18664},
	{SG85314, &UA69478},
	{SG85317, &UA452657},
	{SG85318, &UA710417},
	{SG85319, &UA1936695},
	{SG85320, &UA1940903},
	{SG85321, &UA2811015},
	{SG85322, &UA2232875},
	{SG85323, &UA2343809},
	{SG85324, &UA2681537},
	{SG85325, &UA2460505},
	{SG85326, &UA2507081},
	{SG85335, &UA3016694},
	{SG85336, &UA3031939},
	{SG85338, &UA3128663},
	{SG85339, &UA3078668},
	{SG85340, &UA64688},
	{SG85341, &UA896390},
	{SG85342, &UA1945933},
	{SG85343, &UA2157923},
	{SG85344, &UA2236451},
	{SG85345, &UA2337732},
	{SG85346, &UA2594948},
	{SG85347, &UA2869176},
	{SG85348, &UA2956889},
	{SG85349, &UA2960809},
	{SG85350, &UA3126117},
	{SG85351, &UA3019454},
	{SG85352, &UA3020942},
	{SG85353, &UA3078660},
	{SG85354, &UA3108341},
	{SG85355, &UA68882},
	{SG85356, &UA559899},
	{SG85357, &UA711147},
	{SG85358, &UA2319378},
	{SG85359, &UA2519543},
	{SG85360, &UA2558372},
	{SG85361, &UA2892196},
	{SG85363, &UA2987024},
	{SG85365, &UA67972},
	{SG85366, &UA176163},
	{SG85367, &UA390617},
	{SG85368, &UA333831},
	{SG85369, &UA727839},
	{SG85373, &UA1958898},
	{SG85376, &UA2148840},
	{SG85377, &UA2168076},
	{SG85378, &UA2546097},
	{SG85379, &UA2338903},
	{SG85380, &UA2575145},
	{SG85381, &UA2654185},
	{SG85382, &UA2847731},
	{SG85383, &UA23720},
	{SG85384, &UA50403},
	{SG85385, &UA67294},
	{SG85386, &UA2328991},
	{SG85387, &UA68817},
	{SG85388, &UA2804744},
	{SG85389, &UA69313},
	{SG85390, &UA628002},
	{SG85391, &UA1939840},
	{SG85392, &UA1972327},
	{SG85393, &UA2221921},
	{SG85394, &UA2478319},
	{SG85395, &UA2683527},
	{SG85405, &UA3015472},
	{SG85406, &UA3016390},
	{SG85408, &UA3093095},
	{SG85409, &UA3129701},
	{SG85410, &UA69448},
	{SG85411, &UA399142},
	{SG85413, &UA764233},
	{SG85415, &UA2377107},
	{SG85418, &UA2696700},
	{SG85419, &UA2706769},
	{SG85420, &UA3015427},
	{SG85421, &UA3019873},
	{SG85422, &UA3059018},
	{SG85423, &UA64351},
	{SG85424, &UA729557},
	{SG85425, &UA2155600},
	{SG85432, &UA3093486},
	{SG85433, &UA16643},
	{SG85434, &UA67042},
	{SG85435, &UA68104},
	{SG85436, &UA2720221},
	{SG85437, &UA2145039},
	{SG85438, &UA2223263},
	{SG85439, &UA2409775},
	{SG85440, &UA2578323},
	{SG85441, &UA2703572},
	{SG85442, &UA2705418},
	{SG85443, &UA3048697},
	{SG85444, &UA3017036},
	{SG85445, &UA3022052},
	{SG85446, &UA3027969},
	{SG85447, &UA3059019},
	{SG85449, &UA50404},
	{SG85450, &UA56785},
	{SG85451, &UA1900989},
	{SG85452, &UA837702},
	{SG85453, &UA2347235},
	{SG85454, &UA2384326},
	{SG85456, &UA2404483},
	{SG85457, &UA2552909},
	{SG85461, &UA2697917},
	{SG85462, &UA2838532},
	{SG85463, &UA2867298},
	{SG85467, &UA2892059},
	{SG85472, &UA3019973},
	{SG85475, &UA566648},
	{SG85476, &UA80609},
	{SG85477, &UA88382},
	{SG85480, &UA2706352},
	{SG85489, &UA3128417},
	{SG85492, &UA757076},
	{SG85493, &UA1995076},
	{SG85494, &UA2261417},
	{SG85495, &UA2592474},
	{SG85498, &UA2705738},
	{SG85499, &UA2844074},
	{SG85500, &UA2956563},
	{SG85503, &UA3109171},
	{SG85504, &UA68051},
	{SG85505, &UA571964},
	{SG85506, &UA1939863},
	{SG85507, &UA2309244},
	{SG85509, &UA2329323},
	{SG85510, &UA2482840},
	{SG85511, &UA2749145},
	{SG85512, &UA2892459},
	{SG85516, &UA3011648},
	{SG85517, &UA3126291},
	{SG85518, &UA3127388},
	{SG85523, &UA9406},
	{SG85524, &UA225279},
	{SG85525, &UA1960152},
	{SG85526, &UA1967529},
	{SG85527, &UA1989641},
	{SG85528, &UA2950963},
	{SG85529, &UA2644202},
	{SG85530, &UA2302877},
	{SG85531, &UA2401205},
	{SG85533, &UA2557893},
	{SG85534, &UA2768764},
	{SG85536, &UA2847447},
	{SG85537, &UA2937021},
	{SG85538, &UA2987657},
	{SG85539, &UA3002275},
	{SG85540, &UA50449},
	{SG85541, &UA1902524},
	{SG85542, &UA2403547},
	{SG85546, &UA2819441},
	{SG85550, &UA2956658},
	{SG85559, &UA3003145},
	{SG85560, &UA3075981},
	{SG85565, &UA56787},
	{SG85566, &UA63626},
	{SG85567, &UA2292735},
	{SG85568, &UA2400458},
	{SG85571, &UA2660008},
	{SG85572, &UA2698538},
	{SG85573, &UA2730050},
	{SG85574, &UA2841223},
	{SG85576, &UA3108722},
	{SG85577, &UA2133618},
	{SG85578, &UA537262},
	{SG85579, &UA50411},
	{SG85580, &UA399149},
	{SG85582, &UA574108},
	{SG85586, &UA714512},
	{SG85587, &UA899639},
	{SG85588, &UA895329},
	{SG85589, &UA2333302},
	{SG85591, &UA2413297},
	{SG85595, &UA2462438},
	{SG85596, &UA2506729},
	{SG85597, &UA2693487},
	{SG85598, &UA2814719},
	{SG85599, &UA2748509},
	{SG85600, &UA2790536},
	{SG85601, &UA2844447},
	{SG85602, &UA3105707},
	{SG85603, &UA3092993},
	{SG85604, &UA3108851},
	{SG85605, &UA3125723},
	{SG85606, &UA18644},
	{SG85607, &UA69344},
	{SG85608, &UA69484},
	{SG85609, &UA1904674},
	{SG85610, &UA898942},
	{SG85611, &UA1976273},
	{SG85612, &UA2337496},
	{SG85614, &UA2456689},
	{SG85616, &UA2818856},
	{SG85617, &UA2851221},
	{SG85618, &UA3079532},
	{SG85619, &UA3108317},
	{SG85620, &UA3108933},
	{SG85621, &UA3109177},
	{SG85622, &UA3112491},
	{SG85623, &UA348627},
	{SG85625, &UA1902072},
	{SG85626, &UA1903233},
	{SG85627, &UA2169526},
	{SG85628, &UA2285492},
	{SG85629, &UA2317384},
	{SG85630, &UA2331986},
	{SG85631, &UA2375660},
	{SG85632, &UA2392457},
	{SG85634, &UA2692516},
	{SG85635, &UA2785718},
	{SG85637, &UA2850416},
	{SG85638, &UA2892095},
	{SG85639, &UA2983581},
	{SG85641, &UA3014148},
	{SG85642, &UA3113022},
	{SG85643, &UA3078581},
	{SG85644, &UA3108943},
	{SG85645, &UA3126203},
	{SG85646, &UA3127183},
	{SG85647, &UA3130448},
	{SG85648, &UA61308},
	{SG85649, &UA67298},
	{SG85650, &UA452275},
	{SG85651, &UA541027},
	{SG85652, &UA2757772},
	{SG85653, &UA1933978},
	{SG85655, &UA2292693},
	{SG85656, &UA2300222},
	{SG85657, &UA2466417},
	{SG85658, &UA2502591},
	{SG85659, &UA2817178},
	{SG85660, &UA2686111},
	{SG85661, &UA2722104},
	{SG85666, &UA2957358},
	{SG85667, &UA2985999},
	{SG85668, &UA2993905},
	{SG85669, &UA2996108},
	{SG85673, &UA3057205},
	{SG85674, &UA3093140},
	{SG85675, &UA3108431},
	{SG85676, &UA3109157},
	{SG85677, &UA552739},
	{SG85679, &UA69489},
	{SG85680, &UA67643},
	{SG85681, &UA67991},
	{SG85682, &UA69144},
	{SG85683, &UA574299},
	{SG85684, &UA2302858},
	{SG85685, &UA656054},
	{SG85686, &UA1992511},
	{SG85687, &UA2383347},
	{SG85688, &UA2392689},
	{SG85690, &UA2543577},
	{SG85691, &UA2555705},
	{SG85694, &UA2767042},
	{SG85695, &UA2822627},
	{SG85696, &UA2833536},
	{SG85697, &UA2843680},
	{SG85698, &UA2942961},
	{SG85699, &UA3009437},
	{SG85700, &UA2951330},
	{SG85701, &UA2952585},
	{SG85702, &UA2957213},
	{SG85703, &UA2978942},
	{SG85704, &UA2982719},
	{SG85705, &UA2996262},
	{SG85706, &UA2996750},
	{SG85707, &UA3014485},
	{SG85708, &UA3018012},
	{SG85709, &UA3020690},
	{SG85710, &UA3057074},
	{SG85711, &UA3108218},
	{SG85712, &UA3111690},
	{SG85713, &UA3124310},
	{SG85714, &UA50113},
	{SG85718, &UA68230},
	{SG85719, &UA2136877},
	{SG85720, &UA68713},
	{SG85721, &UA68760},
	{SG85722, &UA333930},
	{SG85723, &UA574562},
	{SG85724, &UA2213028},
	{SG85725, &UA2232552},
	{SG85726, &UA2423420},
	{SG85727, &UA2818686},
	{SG85728, &UA2841727},
	{SG85729, &UA2844987},
	{SG85730, &UA2854127},
	{SG85732, &UA2995669},
	{SG85734, &UA3048268},
	{SG85735, &UA3056843},
	{SG85736, &UA3071400},
	{SG85737, &UA3085902},
	{SG85738, &UA3108360},
	{SG85740, &UA3125776},
	{SG85741, &UA3125868},
	{SG85743, &UA3130814},
	{SG85747, &UA1904173},
	{SG85748, &UA2150631},
	{SG85749, &UA2318118},
	{SG85750, &UA2336026},
	{SG85751, &UA2339386},
	{SG85752, &UA2409156},
	{SG85753, &UA2443285},
	{SG85754, &UA2742816},
	{SG85755, &UA2494973},
	{SG85756, &UA2597333},
	{SG85759, &UA2745473},
	{SG85761, &UA2788212},
	{SG85763, &UA2790924},
	{SG85764, &UA2835670},
	{SG85765, &UA2840232},
	{SG85766, &UA2844063},
	{SG85767, &UA2850440},
	{SG85768, &UA2856532},
	{SG85771, &UA2978998},
	{SG85772, &UA2982738},
	{SG85775, &UA2997258},
	{SG85777, &UA3043538},
	{SG85779, &UA3077290},
	{SG85780, &UA3101571},
	{SG85781, &UA3126342},
	{SG85782, &UA452205},
	{SG85783, &UA69428},
	{SG85784, &UA80740},
	{SG85785, &UA2299365},
	{SG85786, &UA2216709},
	{SG85787, &UA2289589},
	{SG85788, &UA2447550},
	{SG85789, &UA2483566},
	{SG85790, &UA2493459},
	{SG85791, &UA2502713},
	{SG85794, &UA2547712},
	{SG85797, &UA2655646},
	{SG85798, &UA2729215},
	{SG85799, &UA2826054},
	{SG85800, &UA2829072},
	{SG85801, &UA2830529},
	{SG85802, &UA2837204},
	{SG85803, &UA2839496},
	{SG85804, &UA2843299},
	{SG85806, &UA2848682},
	{SG85807, &UA2935259},
	{SG85808, &UA2953917},
	{SG85809, &UA2980692},
	{SG85810, &UA2982107},
	{SG85813, &UA3071025},
	{SG85814, &UA3071410},
	{SG85815, &UA3091859},
	{SG85816, &UA3099159},
	{SG85817, &UA3100602},
	{SG85818, &UA3106818},
	{SG85820, &UA3108180},
	{SG85822, &UA3120327},
	{SG85823, &UA3121020},
	{SG85824, &UA3125802},
	{SG85826, &UA3129564},
	{SG85832, &UA61694},
	{SG85833, &UA67071},
	{SG85835, &UA560169},
	{SG85836, &UA1901576},
	{SG85837, &UA1904897},
	{SG85838, &UA2160353},
	{SG85839, &UA2261209},
	{SG85840, &UA2286097},
	{SG85841, &UA2307024},
	{SG85842, &UA2392293},
	{SG85843, &UA2308893},
	{SG85844, &UA2329038},
	{SG85846, &UA2373783},
	{SG85850, &UA2438354},
	{SG85851, &UA2444546},
	{SG85852, &UA2496695},
	{SG85853, &UA2573758},
	{SG85854, &UA2640598},
	{SG85856, &UA2745782},
	{SG85857, &UA2801530},
	{SG85859, &UA2828446},
	{SG85860, &UA2835563},
	{SG85861, &UA2845992},
	{SG85862, &UA2866410},
	{SG85863, &UA2981293},
	{SG85865, &UA2982260},
	{SG85866, &UA2982515},
	{SG85868, &UA2982658},
	{SG85870, &UA2983794},
	{SG85871, &UA2996201},
	{SG85872, &UA3043668},
	{SG85873, &UA3048533},
	{SG85874, &UA3085290},
	{SG85876, &UA3099735},
	{SG85881, &UA3108064},
	{SG85882, &UA3119079},
	{SG85883, &UA3124872},
	{SG85884, &UA3125576},
	{SG85888, &UA3129607},
	{SG85889, &UA67934},
	{SG85890, &UA68177},
	{SG85891, &UA68768},
	{SG85892, &UA564371},
	{SG85893, &UA571897},
	{SG85896, &UA2300786},
	{SG85897, &UA2372576},
	{SG85898, &UA2383420},
	{SG85899, &UA2426667},
	{SG85900, &UA2456325},
	{SG85901, &UA2475794},
	{SG85902, &UA2555435},
	{SG85907, &UA2699849},
	{SG85908, &UA2804070},
	{SG85909, &UA2808034},
	{SG85911, &UA2819609},
	{SG85914, &UA2848944},
	{SG85918, &UA3040693},
	{SG85919, &UA3124394},
	{SG85920, &UA53901},
	{SG85921, &UA56793},
	{SG85922, &UA62930},
	{SG85923, &UA1851591},
	{SG85927, &UA2261310},
	{SG85928, &UA2306662},
	{SG85929, &UA2316352},
	{SG85930, &UA2328468},
	{SG85931, &UA2520854},
	{SG85932, &UA2643144},
	{SG85936, &UA2802276},
	{SG85937, &UA2809905},
	{SG85938, &UA2823554},
	{SG85939, &UA2833846},
	{SG85940, &UA2838296},
	{SG85941, &UA2981384},
	{SG85943, &UA64363},
	{SG85944, &UA51635},
	{SG85945, &UA43757},
	{SG85946, &UA51255},
	{SG85947, &UA66965},
	{SG85948, &UA67943},
	{SG85949, &UA68223},
	{SG85950, &UA68514},
	{SG85951, &UA68852},
	{SG85952, &UA292128},
	{SG85953, &UA687125},
	{SG85954, &UA813691},
	{SG85956, &UA2313364},
	{SG85957, &UA2372034},
	{SG85958, &UA2442743},
	{SG85959, &UA2559920},
	{SG85960, &UA2685192},
	{SG85962, &UA54186},
	{SG85963, &UA67166},
	{SG85964, &UA68068},
	{SG85965, &UA566179},
	{SG85966, &UA573842},
	{SG85967, &UA80649},
	{SG85969, &UA2220782},
	{SG85970, &UA2330350},
	{SG85972, &UA2759797},
	{SG85973, &UA2818024},
	{SG85975, &UA3015459},
	{SG85976, &UA67642},
	{SG85977, &UA64959},
	{SG85978, &UA67811},
	{SG85979, &UA68769},
	{SG85980, &UA69551},
	{SG85981, &UA349027},
	{SG85982, &UA652368},
	{SG85984, &UA1995764},
	{SG85985, &UA2319495},
	{SG85992, &UA2337528},
	{SG85993, &UA2588304},
	{SG85994, &UA2413040},
	{SG85995, &UA2543003},
	{SG85997, &UA2889021},
	{SG85998, &UA3092617},
	{SG85999, &UA3079629},
	{SG86000, &UA69391},
	{SG86001, &UA69398},
	{SG86002, &UA50237},
	{SG86004, &UA67305},
	{SG86005, &UA538836},
	{SG86006, &UA68438},
	{SG86007, &UA68928},
	{SG86008, &UA69477},
	{SG86009, &UA86870},
	{SG86010, &UA88375},
	{SG86014, &UA559750},
	{SG86016, &UA574055},
	{SG86024, &UA766479},
	{SG86027, &UA1942921},
	{SG86028, &UA2375437},
	{SG86029, &UA2591078},
	{SG86031, &UA3032064},
	{SG86032, &UA49061},
	{SG86034, &UA662271},
	{SG86035, &UA563610},
	{SG86036, &UA1953657},
	{SG86037, &UA754260},
	{SG86038, &UA767295},
	{SG86043, &UA2177998},
	{SG86046, &UA2327491},
	{SG86047, &UA2337200},
	{SG86048, &UA2375073},
	{SG86049, &UA1117},
	{SG86050, &UA505088},
	{SG86052, &UA30982},
	{SG86053, &UA32858},
	{SG86055, &UA69454},
	{SG86056, &UA62619},
	{SG86057, &UA88428},
	{SG86058, &UA466596},
	{SG86059, &UA561140},
	{SG86060, &UA1899903},
	{SG86061, &UA691129},
	{SG86062, &UA1848935},
	{SG86063, &UA2207654},
	{SG86065, &UA2158230},
	{SG86067, &UA2660432},
	{SG86068, &UA2765906},
	{SG86069, &UA3032619},
	{SG86070, &UA63990},
	{SG86071, &UA24710},
	{SG86072, &UA27304},
	{SG86073, &UA69948},
	{SG86074, &UA50307},
	{SG86075, &UA67079},
	{SG86076, &UA67794},
	{SG86077, &UA68003},
	{SG86078, &UA69449},
	{SG86079, &UA548053},
	{SG86080, &UA567615},
	{SG86082, &UA894635},
	{SG86083, &UA2177638},
	{SG86084, &UA2372947},
	{SG86087, &UA2374153},
	{SG86090, &UA2988381},
	{SG86091, &UA3068992},
	{SG86092, &UA3108958},
	{SG86093, &UA3109152},
	{SG86094, &UA88},
	{SG86095, &UA10269},
	{SG86096, &UA17370},
	{SG86097, &UA36176},
	{SG86098, &UA50195},
	{SG86099, &UA52814},
	{SG86100, &UA56775},
	{SG86101, &UA67085},
	{SG86102, &UA69125},
	{SG86103, &UA2298028},
	{SG86104, &UA88381},
	{SG86105, &UA379593},
	{SG86106, &UA528488},
	{SG86108, &UA556958},
	{SG86109, &UA673385},
	{SG86110, &UA730675},
	{SG86111, &UA1922429},
	{SG86117, &UA2152718},
	{SG86121, &UA2594084},
	{SG86124, &UA2945152},
	{SG86125, &UA64349},
	{SG86126, &UA50415},
	{SG86127, &UA1844412},
	{SG86128, &UA67884},
	{SG86129, &UA69172},
	{SG86130, &UA568720},
	{SG86131, &UA721114},
	{SG86132, &UA1938026},
	{SG86133, &UA2168117},
	{SG86134, &UA2219200},
	{SG86135, &UA2336134},
	{SG86136, &UA2342787},
	{SG86137, &UA2360658},
	{SG86139, &UA2747895},
	{SG86140, &UA2794949},
	{SG86141, &UA15508},
	{SG86142, &UA69364},
	{SG86143, &UA44593},
	{SG86144, &UA68518},
	{SG86146, &UA61368},
	{SG86147, &UA2278428},
	{SG86148, &UA67883},
	{SG86149, &UA68989},
	{SG86150, &UA2287472},
	{SG86151, &UA1917522},
	{SG86152, &UA573430},
	{SG86153, &UA805358},
	{SG86154, &UA1954541},
	{SG86155, &UA1961789},
	{SG86156, &UA2589513},
	{SG86157, &UA2729966},
	{SG86158, &UA3059862},
	{SG86159, &UA3078973},
	{SG86160, &UA15580},
	{SG86161, &UA33544},
	{SG86162, &UA54644},
	{SG86163, &UA2125786},
	{SG86164, &UA60903},
	{SG86165, &UA62566},
	{SG86166, &UA67363},
	{SG86167, &UA68697},
	{SG86168, &UA68819},
	{SG86169, &UA68987},
	{SG86170, &UA69112},
	{SG86171, &UA69143},
	{SG86172, &UA69393},
	{SG86173, &UA81098},
	{SG86174, &UA88514},
	{SG86175, &UA436723},
	{SG86180, &UA1928814},
	{SG86186, &UA1995178},
	{SG86187, &UA1995182},
	{SG86188, &UA1995829},
	{SG86191, &UA2441670},
	{SG86192, &UA2528031},
	{SG86196, &UA2847833},
	{SG86197, &UA3008387},
	{SG86198, &UA3119225},
	{SG86199, &UA663},
	{SG86200, &UA22728},
	{SG86201, &UA50224},
	{SG86202, &UA67823},
	{SG86203, &UA68516},
	{SG86204, &UA2278824},
	{SG86205, &UA68716},
	{SG86206, &UA557523},
	{SG86207, &UA69074},
	{SG86208, &UA460022},
	{SG86209, &UA531573},
	{SG86210, &UA663138},
	{SG86211, &UA838556},
	{SG86212, &UA2260383},
	{SG86213, &UA900315},
	{SG86215, &UA902891},
	{SG86216, &UA1908382},
	{SG86217, &UA1979724},
	{SG86218, &UA2316365},
	{SG86219, &UA2317432},
	{SG86220, &UA2319289},
	{SG86225, &UA2762984},
	{SG86226, &UA3068988},
	{SG86228, &UA2796},
	{SG86229, &UA22722},
	{SG86230, &UA9931},
	{SG86231, &UA63477},
	{SG86232, &UA49093},
	{SG86233, &UA50189},
	{SG86234, &UA50470},
	{SG86235, &UA67404},
	{SG86236, &UA51282},
	{SG86237, &UA56006},
	{SG86238, &UA56778},
	{SG86240, &UA67023},
	{SG86241, &UA69099},
	{SG86242, &UA69438},
	{SG86243, &UA69232},
	{SG86244, &UA69402},
	{SG86245, &UA465412},
	{SG86246, &UA575046},
	{SG86247, &UA692099},
	{SG86248, &UA2148345},
	{SG86249, &UA758068},
	{SG86251, &UA1929146},
	{SG86252, &UA1995188},
	{SG86253, &UA2157119},
	{SG86254, &UA2163808},
	{SG86255, &UA2313675},
	{SG86256, &UA2333092},
	{SG86257, &UA2344906},
	{SG86258, &UA2483508},
	{SG86259, &UA2506776},
	{SG86262, &UA2809214},
	{SG86263, &UA28444},
	{SG86264, &UA41877},
	{SG86265, &UA49562},
	{SG86266, &UA67062},
	{SG86267, &UA67836},
	{SG86268, &UA67793},
	{SG86269, &UA67829},
	{SG86270, &UA68715},
	{SG86271, &UA68844},
	{SG86272, &UA69467},
	{SG86273, &UA86876},
	{SG86274, &UA444188},
	{SG86275, &UA525271},
	{SG86276, &UA534313},
	{SG86278, &UA1884166},
	{SG86279, &UA1932864},
	{SG86280, &UA1970328},
	{SG86288, &UA2944204},
	{SG86289, &UA128},
	{SG86290, &UA65310},
	{SG86291, &UA22662},
	{SG86293, &UA56004},
	{SG86294, &UA56765},
	{SG86295, &UA67285},
	{SG86296, &UA67790},
	{SG86297, &UA67561},
	{SG86298, &UA69026},
	{SG86299, &UA68731},
	{SG86300, &UA68779},
	{SG86301, &UA69204},
	{SG86302, &UA69770},
	{SG86308, &UA88433},
	{SG86310, &UA466614},
	{SG86311, &UA515226},
	{SG86312, &UA560291},
	{SG86313, &UA574712},
	{SG86314, &UA788461},
	{SG86315, &UA907421},
	{SG86316, &UA1894548},
	{SG86317, &UA1927479},
	{SG86318, &UA1992027},
	{SG86319, &UA2217656},
	{SG86320, &UA2275947},
	{SG86321, &UA2319406},
	{SG86322, &UA2462444},
	{SG86323, &UA2833723},
	{SG86325, &UA2964905},
	{SG86326, &UA3058966},
	{SG86328, &UA13799},
	{SG86329, &UA15529},
	{SG86331, &UA20654},
	{SG86333, &UA31074},
	{SG86334, &UA34045},
	{SG86336, &UA49607},
	{SG86337, &UA55980},
	{SG86338, &UA56026},
	{SG86339, &UA56339},
	{SG86340, &UA61481},
	{SG86341, &UA65323},
	{SG86342, &UA66786},
	{SG86343, &UA66787},
	{SG86344, &UA67043},
	{SG86345, &UA67545},
	{SG86346, &UA68537},
	{SG86347, &UA68572},
	{SG86348, &UA68941},
	{SG86349, &UA68994},
	{SG86350, &UA69641},
	{SG86351, &UA69650},
	{SG86352, &UA69651},
	{SG86353, &UA81099},
	{SG86356, &UA521193},
	{SG86357, &UA551773},
	{SG86358, &UA587785},
	{SG86360, &UA707408},
	{SG86361, &UA753979},
	{SG86362, &UA1908344},
	{SG86363, &UA1952832},
	{SG86371, &UA2162550},
	{SG86372, &UA2458357},
	{SG86374, &UA2595954},
	{SG86380, &UA2658210},
	{SG86384, &UA3080332},
	{SG86385, &UA3108442},
	{SG86386, &UA3125258},
	{SG86387, &UA215},
	{SG86388, &UA480},
	{SG86389, &UA1284},
	{SG86390, &UA4360},
	{SG86391, &UA6575},
	{SG86392, &UA12179},
	{SG86393, &UA12181},
	{SG86394, &UA13795},
	{SG86395, &UA16250},
	{SG86396, &UA16900},
	{SG86397, &UA16911},
	{SG86398, &UA20630},
	{SG86400, &UA21764},
	{SG86402, &UA33504},
	{SG86405, &UA41150},
	{SG86407, &UA44599},
	{SG86410, &UA50496},
	{SG86411, &UA56015},
	{SG86413, &UA56872},
	{SG86415, &UA61128},
	{SG86416, &UA63072},
	{SG86417, &UA65024},
	{SG86431, &UA67096},
	{SG86432, &UA67277},
	{SG86433, &UA67330},
	{SG86434, &UA67376},
	{SG86435, &UA67448},
	{SG86436, &UA67527},
	{SG86437, &UA67690},
	{SG86438, &UA67973},
	{SG86439, &UA68048},
	{SG86440, &UA68086},
	{SG86441, &UA68722},
	{SG86442, &UA69225},
	{SG86443, &UA69262},
	{SG86444, &UA69597},
	{SG86445, &UA69848},
	{SG86446, &UA80954},
	{SG86447, &UA81091},
	{SG86448, &UA474991},
	{SG86449, &UA494202},
	{SG86450, &UA500415},
	{SG86451, &UA549949},
	{SG86452, &UA564878},
	{SG86453, &UA903631},
	{SG86455, &UA925907},
	{SG86456, &UA1927254},
	{SG86457, &UA2300295},
	{SG86458, &UA2306206},
	{SG86459, &UA2462443},
	{SG86460, &UA2505228},
	{SG86461, &UA2580476},
	{SG86462, &UA2597086},
	{SG86479, &UA2662307},
	{SG86480, &UA2779737},
	{SG86481, &UA3013001},
	{SG86482, &UA3044425},
	{SG86483, &UA3059009},
	{SG86484, &UA3079207},
	{SG86485, &UA3079496},
	{SG86486, &UA3125313},
	{SG86487, &UA3128420},
	{SG86488, &UA138},
	{SG86489, &UA33502},
	{NULL}};
const Segmentdata DNokiaSeries40[] = {
	{SG78477, &UA2489020},
	{SG78530, &UA2489338},
	{SG78681, &UA2489319},
	{SG78807, &UA2203611},
	{SG78809, &UA2388146},
	{SG78853, &UA2489705},
	{SG78956, &UA2488670},
	{SG78957, &UA2490096},
	{SG78967, &UA605514},
	{SG78968, &UA2184010},
	{SG79101, &UA2489038},
	{SG79102, &UA3045784},
	{SG79318, &UA2488849},
	{SG79323, &UA2489609},
	{SG79325, &UA2490638},
	{SG79553, &UA2202803},
	{SG79554, &UA2368286},
	{SG79625, &UA2003032},
	{SG79728, &UA683828},
	{SG79729, &UA902418},
	{SG79832, &UA895670},
	{SG79833, &UA2234647},
	{SG79887, &UA2489604},
	{SG79888, &UA2560691},
	{SG79897, &UA626025},
	{SG79898, &UA2230726},
	{SG79936, &UA2489019},
	{SG79944, &UA2203244},
	{SG79945, &UA2405797},
	{SG79949, &UA612419},
	{SG79950, &UA2355219},
	{SG79960, &UA2203747},
	{SG79971, &UA2203746},
	{SG79972, &UA2387205},
	{SG80036, &UA3026368},
	{SG80057, &UA2488520},
	{SG80058, &UA1971607},
	{SG80059, &UA1991705},
	{SG80060, &UA2402511},
	{SG80061, &UA2482528},
	{SG80084, &UA2202484},
	{SG80086, &UA2387099},
	{SG80338, &UA1995672},
	{SG80413, &UA2489342},
	{SG80449, &UA899201},
	{SG80485, &UA2492500},
	{SG80486, &UA2531404},
	{SG80487, &UA2202612},
	{SG80488, &UA2230170},
	{SG80489, &UA2407078},
	{SG80490, &UA3025463},
	{SG80491, &UA3025460},
	{SG80496, &UA3030631},
	{SG80498, &UA2433358},
	{SG80499, &UA2462449},
	{SG80500, &UA2715589},
	{SG80581, &UA2791413},
	{SG80582, &UA2894707},
	{SG80583, &UA3103181},
	{SG80588, &UA3030770},
	{SG80676, &UA2202766},
	{SG80678, &UA2405400},
	{SG80688, &UA2489337},
	{SG80794, &UA3027877},
	{SG80800, &UA912242},
	{SG80802, &UA2743961},
	{SG80803, &UA3043597},
	{SG80804, &UA2462459},
	{SG80805, &UA2560884},
	{SG80806, &UA3088441},
	{SG80812, &UA792183},
	{SG80813, &UA2467164},
	{SG80814, &UA2489836},
	{SG80863, &UA3025450},
	{SG80925, &UA2203630},
	{SG80926, &UA2387894},
	{SG80927, &UA3044221},
	{SG80931, &UA1996107},
	{SG80933, &UA2203817},
	{SG80934, &UA2258381},
	{SG80935, &UA2490553},
	{SG80936, &UA2507839},
	{SG80940, &UA898782},
	{SG80941, &UA905574},
	{SG80942, &UA2338445},
	{SG80943, &UA2443469},
	{SG81091, &UA628271},
	{SG81171, &UA2490775},
	{SG81578, &UA2385738},
	{SG81640, &UA625162},
	{SG81641, &UA800367},
	{SG81642, &UA2353240},
	{SG81643, &UA2553532},
	{SG81649, &UA3026058},
	{SG81650, &UA2508419},
	{SG81651, &UA2764705},
	{SG81655, &UA2203590},
	{SG81656, &UA2457732},
	{SG81668, &UA2258398},
	{SG81669, &UA3043852},
	{SG81687, &UA3026055},
	{SG81688, &UA3028253},
	{SG81689, &UA3028260},
	{SG81690, &UA3029726},
	{SG81691, &UA3030555},
	{SG81692, &UA3078873},
	{SG81731, &UA2204367},
	{SG81733, &UA2382662},
	{SG81735, &UA2555149},
	{SG81759, &UA2510528},
	{SG81922, &UA2879219},
	{SG81923, &UA3015567},
	{SG81932, &UA2606980},
	{SG81935, &UA2668873},
	{SG82020, &UA3025464},
	{SG82104, &UA780734},
	{SG82105, &UA907255},
	{SG82109, &UA598788},
	{SG82112, &UA3028613},
	{SG82113, &UA3030747},
	{SG82137, &UA3024806},
	{SG82139, &UA3028269},
	{SG82203, &UA2886777},
	{SG82221, &UA2882692},
	{SG82231, &UA625707},
	{SG82232, &UA2203663},
	{SG82233, &UA2390868},
	{SG82251, &UA2882688},
	{SG82252, &UA2203633},
	{SG82260, &UA2204691},
	{SG82261, &UA2236091},
	{SG82262, &UA2513078},
	{SG82263, &UA2706696},
	{SG82265, &UA75382},
	{SG82266, &UA2162506},
	{SG82271, &UA2202618},
	{SG82272, &UA2457760},
	{SG82319, &UA2886766},
	{SG82410, &UA639064},
	{SG82411, &UA2204241},
	{SG82412, &UA2426610},
	{SG82417, &UA2882687},
	{SG82429, &UA3024808},
	{SG82430, &UA2508328},
	{SG82431, &UA2536782},
	{SG82475, &UA3083949},
	{SG82483, &UA3096131},
	{SG82496, &UA2235374},
	{SG82497, &UA2457899},
	{SG82518, &UA2203824},
	{SG82519, &UA2421566},
	{SG82520, &UA2457619},
	{SG82561, &UA37364},
	{SG82578, &UA2507724},
	{SG82579, &UA2560376},
	{SG82580, &UA3083956},
	{SG82603, &UA69795},
	{SG82623, &UA906293},
	{SG82625, &UA2403948},
	{SG82653, &UA902269},
	{SG82654, &UA2353944},
	{SG82659, &UA626400},
	{SG82685, &UA3096082},
	{SG82689, &UA3083429},
	{SG82693, &UA903892},
	{SG82694, &UA1975208},
	{SG82695, &UA1991057},
	{SG82703, &UA3029643},
	{SG82704, &UA3058925},
	{SG82773, &UA658971},
	{SG82774, &UA1959293},
	{SG82775, &UA2422926},
	{SG82798, &UA2510527},
	{SG82799, &UA2227038},
	{SG82800, &UA2383523},
	{SG82862, &UA3030625},
	{SG82865, &UA2769239},
	{SG82866, &UA2879725},
	{SG82867, &UA3108647},
	{SG82868, &UA2171630},
	{SG82878, &UA2944509},
	{SG82879, &UA3011454},
	{SG82880, &UA3015658},
	{SG82881, &UA3021169},
	{SG82882, &UA3079593},
	{SG82889, &UA2167425},
	{SG82890, &UA2376528},
	{SG82928, &UA723925},
	{SG82949, &UA3026675},
	{SG82952, &UA2156475},
	{SG82957, &UA57010},
	{SG82959, &UA2511337},
	{SG82964, &UA1964541},
	{SG82968, &UA2882683},
	{SG82985, &UA2507608},
	{SG83016, &UA628260},
	{SG83023, &UA2598341},
	{SG83036, &UA903658},
	{SG83037, &UA2534728},
	{SG83038, &UA2535580},
	{SG83039, &UA2386778},
	{SG83080, &UA1973235},
	{SG83081, &UA1987939},
	{SG83082, &UA2777832},
	{SG83083, &UA2949819},
	{SG83090, &UA69828},
	{SG83091, &UA766322},
	{SG83105, &UA662305},
	{SG83155, &UA3028188},
	{SG83173, &UA3096231},
	{SG83186, &UA2885302},
	{SG83188, &UA2963492},
	{SG83197, &UA2900946},
	{SG83199, &UA637227},
	{SG83206, &UA3083425},
	{SG83217, &UA3096245},
	{SG83218, &UA2671268},
	{SG83219, &UA3027198},
	{SG83241, &UA1986830},
	{SG83242, &UA2445643},
	{SG83252, &UA2601291},
	{SG83256, &UA673358},
	{SG83257, &UA2154765},
	{SG83266, &UA3026046},
	{SG83281, &UA3026373},
	{SG83370, &UA2423068},
	{SG83371, &UA3096035},
	{SG83372, &UA3111173},
	{SG83376, &UA2886498},
	{SG83378, &UA2204383},
	{SG83379, &UA2229238},
	{SG83383, &UA2886782},
	{SG83384, &UA2882676},
	{SG83385, &UA3083421},
	{SG83397, &UA3027206},
	{SG83398, &UA3108467},
	{SG83399, &UA2581081},
	{SG83461, &UA2885319},
	{SG83469, &UA2204877},
	{SG83491, &UA2235849},
	{SG83514, &UA2882684},
	{SG83515, &UA640830},
	{SG83517, &UA3096251},
	{SG83519, &UA3110700},
	{SG83542, &UA3096048},
	{SG83613, &UA2513305},
	{SG83622, &UA3026689},
	{SG83623, &UA3030762},
	{SG83634, &UA631786},
	{SG83637, &UA2882675},
	{SG83640, &UA704095},
	{SG83641, &UA2885304},
	{SG83647, &UA2886769},
	{SG83669, &UA33432},
	{SG83670, &UA3028191},
	{SG83671, &UA3030015},
	{SG83683, &UA2562545},
	{SG83684, &UA2889632},
	{SG83718, &UA904430},
	{SG83727, &UA2229020},
	{SG83766, &UA2886768},
	{SG83770, &UA576173},
	{SG83773, &UA2882690},
	{SG83774, &UA2966936},
	{SG83794, &UA1992470},
	{SG83796, &UA2335747},
	{SG83797, &UA2728489},
	{SG83890, &UA692024},
	{SG83891, &UA2398472},
	{SG83904, &UA3113249},
	{SG83913, &UA3027727},
	{SG83960, &UA3030553},
	{SG83961, &UA3030632},
	{SG84010, &UA578632},
	{SG84011, &UA1946923},
	{SG84019, &UA3029634},
	{SG84025, &UA732813},
	{SG84026, &UA2221401},
	{SG84041, &UA575621},
	{SG84042, &UA623393},
	{SG84055, &UA3096306},
	{SG84058, &UA908132},
	{SG84068, &UA3027894},
	{SG84078, &UA2900843},
	{SG84081, &UA2559131},
	{SG84082, &UA2155909},
	{SG84116, &UA2885303},
	{SG84119, &UA3097273},
	{SG84120, &UA3026039},
	{SG84127, &UA578672},
	{SG84174, &UA2355599},
	{SG84175, &UA2934241},
	{SG84181, &UA2734265},
	{SG84183, &UA3096248},
	{SG84251, &UA2897685},
	{SG84255, &UA2599612},
	{SG84262, &UA452500},
	{SG84269, &UA2563451},
	{SG84270, &UA3100203},
	{SG84285, &UA2504832},
	{SG84294, &UA2734313},
	{SG84332, &UA3096011},
	{SG84368, &UA2348528},
	{SG84378, &UA3115569},
	{SG84383, &UA649547},
	{SG84388, &UA2659779},
	{SG84411, &UA2900833},
	{SG84420, &UA789450},
	{SG84442, &UA3111098},
	{SG84456, &UA2258433},
	{SG84464, &UA578668},
	{SG84465, &UA3114967},
	{SG84494, &UA3109481},
	{SG84517, &UA578663},
	{SG84521, &UA2895065},
	{SG84531, &UA41301},
	{SG84536, &UA643163},
	{SG84537, &UA659687},
	{SG84573, &UA1957971},
	{SG84574, &UA2304162},
	{SG84575, &UA3027715},
	{SG84581, &UA720485},
	{SG84582, &UA809662},
	{SG84604, &UA2705507},
	{SG84611, &UA2159822},
	{SG84614, &UA3096088},
	{SG84616, &UA49928},
	{SG84624, &UA2222569},
	{SG84628, &UA722969},
	{SG84629, &UA1962349},
	{SG84657, &UA2403152},
	{SG84658, &UA2424510},
	{SG84665, &UA1894368},
	{SG84699, &UA3027750},
	{SG84717, &UA3096255},
	{SG84718, &UA4274},
	{SG84719, &UA70471},
	{SG84726, &UA2229018},
	{SG84730, &UA3029027},
	{SG84739, &UA623166},
	{SG84763, &UA2901098},
	{SG84776, &UA690583},
	{SG84794, &UA584014},
	{SG84796, &UA578650},
	{SG84805, &UA648891},
	{SG84819, &UA578630},
	{SG84839, &UA2258404},
	{SG84841, &UA2692960},
	{SG84849, &UA578670},
	{SG84854, &UA624424},
	{SG84875, &UA452462},
	{SG84905, &UA746714},
	{SG84907, &UA1959672},
	{SG84932, &UA2361764},
	{SG84963, &UA561401},
	{SG84993, &UA578644},
	{SG85010, &UA583605},
	{SG85036, &UA704344},
	{SG85048, &UA2751257},
	{SG85058, &UA669677},
	{SG85085, &UA653353},
	{SG85098, &UA3110430},
	{SG85099, &UA3114907},
	{SG85116, &UA2944955},
	{SG85187, &UA3109351},
	{SG85296, &UA3113524},
	{SG85309, &UA2383317},
	{SG85327, &UA3109398},
	{SG85328, &UA651338},
	{SG85329, &UA583915},
	{SG85330, &UA578646},
	{SG85331, &UA709247},
	{SG85403, &UA2382925},
	{SG85404, &UA2413371},
	{SG85414, &UA41280},
	{SG85416, &UA747525},
	{SG85417, &UA2335788},
	{SG85473, &UA2750756},
	{SG85474, &UA3058872},
	{SG85483, &UA3115776},
	{SG85486, &UA2706103},
	{SG85487, &UA2765142},
	{SG85497, &UA2964215},
	{SG85508, &UA2719212},
	{SG85522, &UA777357},
	{SG85532, &UA771026},
	{SG85569, &UA3008244},
	{SG85570, &UA3018574},
	{SG85624, &UA2577634},
	{SG85733, &UA2943927},
	{SG85739, &UA2854481},
	{SG85744, &UA574376},
	{SG85746, &UA1901729},
	{SG85757, &UA2937081},
	{SG85758, &UA3110679},
	{SG85776, &UA3003627},
	{SG85792, &UA3126989},
	{SG85793, &UA3128637},
	{SG85821, &UA1902969},
	{SG85827, &UA2758863},
	{SG85855, &UA3116521},
	{SG85858, &UA651893},
	{SG85875, &UA644428},
	{SG85886, &UA2299868},
	{SG85887, &UA2413744},
	{SG85912, &UA2554160},
	{SG85968, &UA2302241},
	{SG85971, &UA2524655},
	{SG85983, &UA3058968},
	{SG86030, &UA1941311},
	{SG86081, &UA3127139},
	{SG86088, &UA583722},
	{SG86221, &UA643937},
	{SG86223, &UA1996310},
	{SG86239, &UA3004832},
	{SG86277, &UA796789},
	{SG86292, &UA785577},
	{SG86324, &UA1988305},
	{SG86355, &UA3092460},
	{SG86364, &UA658312},
	{SG86382, &UA3001048},
	{SG86472, &UA2988125},
	{SG86478, &UA1938418},
	{SG86501, &UA2817058},
	{SG86512, &UA633500},
	{SG86513, &UA1937104},
	{SG86528, &UA562418},
	{SG86530, &UA2300999},
	{SG86531, &UA2304614},
	{SG86540, &UA2952328},
	{SG86563, &UA3124023},
	{SG86567, &UA3125220},
	{SG86568, &UA2318959},
	{SG86674, &UA51339},
	{SG86682, &UA765},
	{SG86683, &UA33510},
	{SG86684, &UA34955},
	{SG86685, &UA35205},
	{SG86689, &UA90560},
	{SG86709, &UA24870},
	{NULL}};
const Segmentdata DNokiaSeries60[] = {
	{SG78314, &UA22396},
	{SG78315, &UA56457},
	{SG78316, &UA56456},
	{SG78317, &UA64571},
	{SG78320, &UA70155},
	{SG78321, &UA76951},
	{SG78322, &UA925472},
	{SG78323, &UA2272478},
	{SG78452, &UA30369},
	{SG78453, &UA33036},
	{SG78454, &UA33369},
	{SG78455, &UA34523},
	{SG78456, &UA35377},
	{SG78457, &UA62692},
	{SG78458, &UA64092},
	{SG78501, &UA21930},
	{SG78502, &UA43668},
	{SG78503, &UA44509},
	{SG78504, &UA64680},
	{SG78505, &UA67764},
	{SG78506, &UA891840},
	{SG78534, &UA4227},
	{SG78535, &UA24510},
	{SG78536, &UA33984},
	{SG78537, &UA33989},
	{SG78538, &UA61667},
	{SG78539, &UA61988},
	{SG78581, &UA8955},
	{SG78582, &UA32486},
	{SG78583, &UA35000},
	{SG78584, &UA62756},
	{SG78585, &UA64696},
	{SG78586, &UA64765},
	{SG78587, &UA251814},
	{SG78588, &UA722387},
	{SG78589, &UA3043681},
	{SG78854, &UA5789},
	{SG78855, &UA8987},
	{SG78856, &UA14864},
	{SG78857, &UA37718},
	{SG78858, &UA43512},
	{SG78859, &UA63284},
	{SG78860, &UA64620},
	{SG78862, &UA64758},
	{SG78863, &UA346872},
	{SG78864, &UA651011},
	{SG78974, &UA7509},
	{SG78975, &UA24683},
	{SG78976, &UA33463},
	{SG78977, &UA89926},
	{SG78978, &UA24584},
	{SG78979, &UA48257},
	{SG78980, &UA50045},
	{SG78981, &UA622916},
	{SG78982, &UA712832},
	{SG78983, &UA41099},
	{SG78984, &UA41101},
	{SG78985, &UA64170},
	{SG78986, &UA623709},
	{SG78987, &UA33320},
	{SG78988, &UA36815},
	{SG78989, &UA41228},
	{SG78990, &UA51106},
	{SG78991, &UA64548},
	{SG78992, &UA72649},
	{SG78993, &UA648333},
	{SG79115, &UA12635},
	{SG79116, &UA26717},
	{SG79117, &UA33470},
	{SG79119, &UA46099},
	{SG79120, &UA49141},
	{SG79121, &UA51382},
	{SG79122, &UA64228},
	{SG79123, &UA64229},
	{SG79124, &UA64826},
	{SG79730, &UA63659},
	{SG79731, &UA64747},
	{SG79732, &UA626646},
	{SG79733, &UA734284},
	{SG79734, &UA453591},
	{SG79735, &UA2017988},
	{SG79907, &UA64675},
	{SG79908, &UA346964},
	{SG79909, &UA3495},
	{SG79910, &UA63348},
	{SG79911, &UA64433},
	{SG79912, &UA64842},
	{SG79914, &UA67070},
	{SG79915, &UA69480},
	{SG79916, &UA73289},
	{SG79961, &UA15015},
	{SG79962, &UA21712},
	{SG79963, &UA64694},
	{SG79964, &UA575992},
	{SG79965, &UA737105},
	{SG80143, &UA17227},
	{SG80145, &UA24768},
	{SG80148, &UA44207},
	{SG80150, &UA615640},
	{SG80152, &UA2360258},
	{SG80442, &UA21782},
	{SG80443, &UA21784},
	{SG80444, &UA31123},
	{SG80445, &UA32562},
	{SG80446, &UA64226},
	{SG80447, &UA64460},
	{SG80448, &UA2892295},
	{SG80455, &UA61670},
	{SG80456, &UA64708},
	{SG80457, &UA251973},
	{SG80584, &UA13234},
	{SG80585, &UA24719},
	{SG80586, &UA49566},
	{SG80587, &UA50060},
	{SG80681, &UA77759},
	{SG80683, &UA605148},
	{SG80684, &UA661843},
	{SG80685, &UA2205914},
	{SG80724, &UA2002858},
	{SG80725, &UA2348310},
	{SG80749, &UA62832},
	{SG80751, &UA64669},
	{SG80815, &UA347746},
	{SG80820, &UA24703},
	{SG80821, &UA33300},
	{SG80938, &UA644780},
	{SG80939, &UA2006128},
	{SG80997, &UA2017929},
	{SG81095, &UA33372},
	{SG81096, &UA74962},
	{SG81097, &UA604108},
	{SG81098, &UA1947831},
	{SG81099, &UA2205904},
	{SG81372, &UA64381},
	{SG81373, &UA64776},
	{SG81572, &UA604064},
	{SG81573, &UA631306},
	{SG81574, &UA2205928},
	{SG81670, &UA33469},
	{SG81671, &UA56224},
	{SG81672, &UA63849},
	{SG81673, &UA64771},
	{SG81674, &UA675345},
	{SG81693, &UA635996},
	{SG81694, &UA2017658},
	{SG82061, &UA2549},
	{SG82063, &UA19218},
	{SG82065, &UA28070},
	{SG82067, &UA56177},
	{SG82069, &UA64398},
	{SG82110, &UA2042225},
	{SG82178, &UA360699},
	{SG82179, &UA584052},
	{SG82180, &UA16913},
	{SG82181, &UA33472},
	{SG82183, &UA41313},
	{SG82191, &UA906109},
	{SG82192, &UA2205903},
	{SG82193, &UA2653335},
	{SG82207, &UA64807},
	{SG82237, &UA1149},
	{SG82238, &UA6835},
	{SG82239, &UA755108},
	{SG82240, &UA1840407},
	{SG82246, &UA32149},
	{SG82247, &UA35916},
	{SG82248, &UA63358},
	{SG82249, &UA64764},
	{SG82349, &UA33361},
	{SG82489, &UA628757},
	{SG82490, &UA2042344},
	{SG82500, &UA21383},
	{SG82502, &UA63500},
	{SG82503, &UA63800},
	{SG82504, &UA64875},
	{SG82505, &UA508920},
	{SG82565, &UA23081},
	{SG82566, &UA40984},
	{SG82567, &UA60292},
	{SG82568, &UA62992},
	{SG82569, &UA64817},
	{SG82570, &UA354943},
	{SG82571, &UA24287},
	{SG82573, &UA30649},
	{SG82574, &UA64665},
	{SG82575, &UA67345},
	{SG82589, &UA98},
	{SG82591, &UA20489},
	{SG82592, &UA64192},
	{SG82593, &UA12329},
	{SG82594, &UA53885},
	{SG82595, &UA64734},
	{SG82645, &UA3795},
	{SG82646, &UA19900},
	{SG82647, &UA64592},
	{SG82648, &UA502882},
	{SG82658, &UA64624},
	{SG82660, &UA74984},
	{SG82677, &UA43706},
	{SG82678, &UA64873},
	{SG82682, &UA633757},
	{SG82683, &UA2325980},
	{SG82687, &UA63442},
	{SG82688, &UA671246},
	{SG82705, &UA9881},
	{SG82706, &UA54171},
	{SG82707, &UA64652},
	{SG82930, &UA6384},
	{SG82931, &UA64066},
	{SG82966, &UA53456},
	{SG82967, &UA64772},
	{SG82970, &UA10040},
	{SG82971, &UA16397},
	{SG82972, &UA17260},
	{SG82975, &UA35694},
	{SG82976, &UA54444},
	{SG82977, &UA62920},
	{SG82978, &UA62921},
	{SG82979, &UA67849},
	{SG82980, &UA77295},
	{SG82981, &UA88754},
	{SG83095, &UA363325},
	{SG83096, &UA1934919},
	{SG83184, &UA63706},
	{SG83214, &UA1814},
	{SG83215, &UA53477},
	{SG83216, &UA69523},
	{SG83286, &UA14231},
	{SG83287, &UA39773},
	{SG83288, &UA64611},
	{SG83289, &UA88449},
	{SG83291, &UA24591},
	{SG83293, &UA44088},
	{SG83380, &UA64785},
	{SG83389, &UA56662},
	{SG83390, &UA63745},
	{SG83403, &UA63481},
	{SG83404, &UA64407},
	{SG83431, &UA64376},
	{SG83432, &UA2126678},
	{SG83464, &UA64321},
	{SG83488, &UA53968},
	{SG83489, &UA64549},
	{SG83576, &UA53283},
	{SG83577, &UA63542},
	{SG83578, &UA87023},
	{SG83638, &UA22437},
	{SG83639, &UA64309},
	{SG83761, &UA44168},
	{SG83762, &UA64458},
	{SG83803, &UA64679},
	{SG83903, &UA659812},
	{SG83906, &UA32474},
	{SG83907, &UA51191},
	{SG83908, &UA64371},
	{SG83996, &UA17267},
	{SG83998, &UA64459},
	{SG84000, &UA64806},
	{SG84002, &UA67293},
	{SG84005, &UA637376},
	{SG84056, &UA18529},
	{SG84057, &UA64893},
	{SG84130, &UA18441},
	{SG84131, &UA55599},
	{SG84132, &UA64129},
	{SG84200, &UA17219},
	{SG84201, &UA41201},
	{SG84203, &UA64795},
	{SG84239, &UA44858},
	{SG84240, &UA63960},
	{SG84241, &UA1946260},
	{SG84253, &UA632624},
	{SG84290, &UA6863},
	{SG84291, &UA27554},
	{SG84292, &UA34129},
	{SG84293, &UA64522},
	{SG84429, &UA54022},
	{SG84453, &UA68392},
	{SG84454, &UA195852},
	{SG84485, &UA627167},
	{SG84493, &UA519127},
	{SG84555, &UA63961},
	{SG84557, &UA64554},
	{SG84666, &UA8626},
	{SG84667, &UA63682},
	{SG84668, &UA64005},
	{SG84697, &UA2580152},
	{SG84707, &UA1994062},
	{SG84715, &UA695708},
	{SG84818, &UA60872},
	{SG84857, &UA624531},
	{SG84894, &UA50843},
	{SG84895, &UA63482},
	{SG84934, &UA2095},
	{SG84935, &UA67143},
	{SG84944, &UA22274},
	{SG84997, &UA54959},
	{SG84998, &UA62666},
	{SG85004, &UA1961472},
	{SG85054, &UA2160794},
	{SG85101, &UA55848},
	{SG85102, &UA63660},
	{SG85103, &UA64497},
	{SG85141, &UA68795},
	{SG85275, &UA1973274},
	{SG85310, &UA69132},
	{SG85311, &UA231470},
	{SG85396, &UA63730},
	{SG85397, &UA561993},
	{SG85398, &UA2541600},
	{SG85431, &UA7066},
	{SG85455, &UA2160932},
	{SG85478, &UA24488},
	{SG85479, &UA79609},
	{SG85481, &UA22631},
	{SG85482, &UA62680},
	{SG85484, &UA2154668},
	{SG85496, &UA63757},
	{SG85564, &UA2155982},
	{SG85613, &UA2220370},
	{SG85633, &UA867839},
	{SG85636, &UA62761},
	{SG85678, &UA9264},
	{SG85692, &UA56780},
	{SG85693, &UA292563},
	{SG85742, &UA62965},
	{SG85778, &UA2154097},
	{SG85805, &UA68893},
	{SG85811, &UA67979},
	{SG85812, &UA308984},
	{SG85819, &UA69129},
	{SG85830, &UA2310455},
	{SG85877, &UA1872481},
	{SG85878, &UA2441042},
	{SG85879, &UA2442558},
	{SG85880, &UA2589020},
	{SG85885, &UA2161304},
	{SG85894, &UA570117},
	{SG85910, &UA19290},
	{SG85913, &UA2374780},
	{SG85915, &UA2543951},
	{SG85916, &UA3117394},
	{SG85924, &UA32191},
	{SG85925, &UA56172},
	{SG85926, &UA713949},
	{SG85942, &UA727131},
	{SG85986, &UA2335364},
	{SG85996, &UA68899},
	{SG86011, &UA7144},
	{SG86012, &UA55875},
	{SG86025, &UA2159134},
	{SG86026, &UA2242375},
	{SG86051, &UA2219984},
	{SG86089, &UA2360218},
	{SG86195, &UA2153570},
	{SG86250, &UA2339277},
	{SG86287, &UA2588712},
	{SG86303, &UA2171354},
	{SG86304, &UA2371842},
	{SG86327, &UA64364},
	{SG86332, &UA1948051},
	{SG86335, &UA1975832},
	{SG86373, &UA2157402},
	{SG86379, &UA50427},
	{SG86381, &UA632286},
	{SG86383, &UA2375070},
	{SG86403, &UA64235},
	{SG86404, &UA67135},
	{SG86414, &UA2659976},
	{SG86421, &UA2372385},
	{SG86463, &UA50154},
	{SG86464, &UA2150483},
	{SG86465, &UA2160981},
	{SG86473, &UA2421272},
	{SG86490, &UA556606},
	{SG86498, &UA3043378},
	{SG86500, &UA63446},
	{SG86502, &UA68015},
	{SG86503, &UA25569},
	{SG86504, &UA68696},
	{SG86506, &UA682565},
	{SG86508, &UA2495344},
	{SG86511, &UA2395568},
	{SG86514, &UA2721523},
	{SG86520, &UA2425684},
	{SG86522, &UA2287599},
	{SG86523, &UA68917},
	{SG86524, &UA2223359},
	{SG86525, &UA2335790},
	{SG86526, &UA2373213},
	{SG86527, &UA2441253},
	{SG86529, &UA50417},
	{SG86538, &UA2720899},
	{SG86539, &UA379188},
	{SG86544, &UA2444067},
	{SG86545, &UA2544175},
	{SG86546, &UA2587386},
	{SG86550, &UA67008},
	{SG86551, &UA436930},
	{SG86552, &UA2261295},
	{SG86553, &UA2483326},
	{SG86556, &UA2865886},
	{SG86562, &UA62262},
	{SG86564, &UA2440892},
	{SG86565, &UA2441390},
	{SG86569, &UA51964},
	{SG86570, &UA596019},
	{SG86573, &UA2543792},
	{SG86576, &UA2642177},
	{SG86578, &UA2699096},
	{SG86595, &UA664586},
	{SG86596, &UA2374636},
	{SG86599, &UA568686},
	{SG86600, &UA2545466},
	{SG86601, &UA2640790},
	{SG86605, &UA3042577},
	{SG86606, &UA43717},
	{SG86607, &UA56773},
	{SG86608, &UA379621},
	{SG86609, &UA570503},
	{SG86610, &UA2372087},
	{SG86611, &UA2448901},
	{SG86612, &UA2475547},
	{SG86614, &UA2588109},
	{SG86615, &UA2748967},
	{SG86618, &UA2340400},
	{SG86619, &UA2411370},
	{SG86624, &UA3057965},
	{SG86628, &UA69177},
	{SG86637, &UA738083},
	{SG86642, &UA67084},
	{SG86643, &UA68113},
	{SG86645, &UA1973324},
	{SG86646, &UA2217169},
	{SG86647, &UA2440219},
	{SG86659, &UA823393},
	{SG86660, &UA2260278},
	{SG86661, &UA2445750},
	{SG86662, &UA2642019},
	{SG86663, &UA2845165},
	{SG86668, &UA66997},
	{SG86669, &UA50013},
	{SG86670, &UA64505},
	{SG86671, &UA64739},
	{SG86672, &UA68895},
	{SG86673, &UA396678},
	{SG86677, &UA2644406},
	{SG86678, &UA2758317},
	{SG86679, &UA2829949},
	{SG86686, &UA898165},
	{SG86687, &UA2775803},
	{SG86688, &UA2809420},
	{SG86694, &UA2815929},
	{SG86695, &UA2949505},
	{SG86696, &UA50426},
	{SG86697, &UA760149},
	{SG86698, &UA845127},
	{SG86704, &UA50148},
	{SG86705, &UA56124},
	{SG86706, &UA570059},
	{SG86707, &UA570165},
	{SG86708, &UA2260181},
	{SG86714, &UA60387},
	{SG86716, &UA67203},
	{SG86717, &UA69153},
	{SG86718, &UA59154},
	{SG86719, &UA67095},
	{SG86720, &UA437970},
	{SG86721, &UA2331290},
	{SG86723, &UA69243},
	{SG86727, &UA48434},
	{SG86728, &UA50446},
	{SG86729, &UA57919},
	{SG86730, &UA67886},
	{SG86731, &UA2544330},
	{SG86733, &UA2807970},
	{SG86736, &UA50387},
	{SG86737, &UA571479},
	{SG86738, &UA2214860},
	{SG86743, &UA68384},
	{SG86748, &UA2311197},
	{SG86749, &UA50498},
	{SG86750, &UA68184},
	{SG86751, &UA569111},
	{SG86752, &UA570033},
	{SG86753, &UA1898864},
	{SG86754, &UA1902690},
	{SG86756, &UA2808871},
	{SG86757, &UA9889},
	{SG86758, &UA61887},
	{SG86761, &UA67776},
	{SG86762, &UA694224},
	{SG86763, &UA2323454},
	{SG86766, &UA672083},
	{SG86769, &UA2721108},
	{SG86770, &UA46690},
	{SG86772, &UA67431},
	{SG86776, &UA459239},
	{SG86777, &UA519936},
	{SG86781, &UA69348},
	{SG86782, &UA69330},
	{SG86783, &UA69535},
	{SG86784, &UA570932},
	{SG86786, &UA19720},
	{SG86788, &UA67240},
	{SG86789, &UA68107},
	{SG86790, &UA69227},
	{SG86791, &UA1972463},
	{SG86797, &UA50483},
	{SG86798, &UA51634},
	{SG86801, &UA67636},
	{SG86810, &UA1904378},
	{SG86814, &UA67323},
	{SG86815, &UA67517},
	{SG86816, &UA68083},
	{SG86817, &UA80946},
	{SG86818, &UA90383},
	{SG86820, &UA2213790},
	{SG86826, &UA27474},
	{SG86829, &UA50145},
	{SG86830, &UA55984},
	{SG86831, &UA67780},
	{SG86832, &UA80567},
	{NULL}};
const Segmentdata DOperaGeneric[] = {
	{SG76980, &UA1898556},
	{SG77162, &UA2187414},
	{SG77288, &UA2671457},
	{SG77289, &UA2368116},
	{SG77573, &UA2358305},
	{SG77574, &UA2514058},
	{SG77580, &UA871689},
	{SG78022, &UA2944959},
	{SG78023, &UA2193408},
	{SG78186, &UA2427328},
	{SG78467, &UA93333},
	{SG78468, &UA98994},
	{SG78469, &UA99143},
	{SG78470, &UA99199},
	{SG78471, &UA99897},
	{SG78472, &UA130902},
	{SG78473, &UA599786},
	{SG78474, &UA828650},
	{SG78475, &UA915853},
	{SG78476, &UA2502474},
	{SG78498, &UA93298},
	{SG78499, &UA2391637},
	{SG78500, &UA2502470},
	{SG78507, &UA2161749},
	{SG78531, &UA2670783},
	{SG78540, &UA536123},
	{SG78541, &UA717859},
	{SG78551, &UA92397},
	{SG78552, &UA92999},
	{SG78553, &UA137732},
	{SG78554, &UA304963},
	{SG78555, &UA599362},
	{SG78556, &UA755894},
	{SG78557, &UA814329},
	{SG78559, &UA820517},
	{SG78561, &UA887989},
	{SG78567, &UA2208230},
	{SG78570, &UA2271411},
	{SG78571, &UA2295396},
	{SG78572, &UA2472172},
	{SG78573, &UA2483034},
	{SG78574, &UA2502465},
	{SG78575, &UA2505316},
	{SG78576, &UA2681101},
	{SG78577, &UA2681625},
	{SG78578, &UA2683464},
	{SG78579, &UA2686740},
	{SG78580, &UA2932911},
	{SG78590, &UA815237},
	{SG78682, &UA2514061},
	{SG78861, &UA2368396},
	{SG78965, &UA807017},
	{SG78966, &UA2127434},
	{SG79018, &UA2355037},
	{SG79118, &UA2428341},
	{SG79512, &UA94067},
	{SG79521, &UA415733},
	{SG79524, &UA660651},
	{SG79526, &UA768119},
	{SG79532, &UA2126142},
	{SG79534, &UA2266394},
	{SG79835, &UA603125},
	{SG79895, &UA422245},
	{SG79906, &UA2229325},
	{SG79918, &UA388811},
	{SG79919, &UA726446},
	{SG79951, &UA2710646},
	{SG80062, &UA2508202},
	{SG80414, &UA38527},
	{SG80415, &UA414453},
	{SG80416, &UA799673},
	{SG80417, &UA893794},
	{SG80454, &UA1903839},
	{SG80497, &UA398352},
	{SG80689, &UA247704},
	{SG80719, &UA755506},
	{SG80728, &UA2425647},
	{SG80810, &UA2537183},
	{SG80816, &UA654929},
	{SG80880, &UA720359},
	{SG80923, &UA704558},
	{SG80929, &UA2735031},
	{SG80930, &UA2599844},
	{SG80932, &UA3019135},
	{SG80937, &UA808761},
	{SG81000, &UA855006},
	{SG81002, &UA1840954},
	{SG81110, &UA351518},
	{SG81183, &UA502316},
	{SG81192, &UA324754},
	{SG81437, &UA2291749},
	{SG81580, &UA3078418},
	{SG81675, &UA93324},
	{SG81676, &UA133694},
	{SG81677, &UA260170},
	{SG81678, &UA272325},
	{SG81679, &UA434587},
	{SG81680, &UA599643},
	{SG81681, &UA736632},
	{SG81682, &UA813162},
	{SG81683, &UA857547},
	{SG81684, &UA1899382},
	{SG81685, &UA2298885},
	{SG81686, &UA2502471},
	{SG81927, &UA2467649},
	{SG82045, &UA2511043},
	{SG82158, &UA1391},
	{SG82160, &UA408995},
	{SG82162, &UA631938},
	{SG82164, &UA756827},
	{SG82165, &UA843860},
	{SG82166, &UA860671},
	{SG82167, &UA875438},
	{SG82168, &UA1969245},
	{SG82169, &UA2242299},
	{SG82170, &UA2540707},
	{SG82171, &UA2681103},
	{SG82172, &UA2681820},
	{SG82173, &UA2682301},
	{SG82174, &UA2682382},
	{SG82208, &UA2387056},
	{SG82209, &UA2485552},
	{SG82216, &UA2457566},
	{SG82222, &UA712993},
	{SG82223, &UA2502472},
	{SG82242, &UA2404022},
	{SG82250, &UA2819669},
	{SG82264, &UA2721054},
	{SG82381, &UA344511},
	{SG82416, &UA94043},
	{SG82418, &UA492839},
	{SG82419, &UA630389},
	{SG82420, &UA888487},
	{SG82421, &UA2275259},
	{SG82422, &UA2504961},
	{SG82423, &UA2682627},
	{SG82472, &UA98998},
	{SG82473, &UA714898},
	{SG82474, &UA2295092},
	{SG82476, &UA93068},
	{SG82477, &UA414499},
	{SG82478, &UA599409},
	{SG82479, &UA857773},
	{SG82557, &UA2541913},
	{SG82597, &UA94062},
	{SG82598, &UA412870},
	{SG82696, &UA442961},
	{SG82697, &UA1985626},
	{SG82807, &UA2263380},
	{SG82864, &UA645229},
	{SG82871, &UA1854381},
	{SG82891, &UA835014},
	{SG82902, &UA797367},
	{SG83015, &UA2751259},
	{SG83092, &UA444715},
	{SG83124, &UA3012616},
	{SG83171, &UA442134},
	{SG83181, &UA2662802},
	{SG83183, &UA454947},
	{SG83187, &UA97236},
	{SG83189, &UA120897},
	{SG83190, &UA130292},
	{SG83191, &UA412813},
	{SG83192, &UA633551},
	{SG83193, &UA785598},
	{SG83194, &UA838758},
	{SG83195, &UA2472833},
	{SG83196, &UA2792816},
	{SG83253, &UA415679},
	{SG83254, &UA625715},
	{SG83329, &UA30382},
	{SG83330, &UA1863208},
	{SG83342, &UA2746275},
	{SG83369, &UA2468517},
	{SG83428, &UA98141},
	{SG83430, &UA2161804},
	{SG83499, &UA1934529},
	{SG83511, &UA374145},
	{SG83513, &UA2422233},
	{SG83658, &UA1890197},
	{SG83729, &UA1852986},
	{SG83775, &UA201700},
	{SG83793, &UA2553342},
	{SG83818, &UA30745},
	{SG83819, &UA107814},
	{SG83820, &UA757890},
	{SG83821, &UA2504710},
	{SG83895, &UA2164031},
	{SG83915, &UA118723},
	{SG83916, &UA271508},
	{SG83917, &UA600145},
	{SG83944, &UA1841859},
	{SG83968, &UA99374},
	{SG83969, &UA600499},
	{SG83970, &UA731828},
	{SG84162, &UA535344},
	{SG84335, &UA85824},
	{SG84336, &UA600491},
	{SG84337, &UA898223},
	{SG84432, &UA2573045},
	{SG84560, &UA685654},
	{SG84561, &UA807691},
	{SG84562, &UA821315},
	{SG84563, &UA822872},
	{SG84564, &UA871463},
	{SG84565, &UA625912},
	{SG84566, &UA838844},
	{SG84567, &UA889371},
	{SG84568, &UA2161153},
	{SG84569, &UA2282856},
	{SG84570, &UA2331354},
	{SG84571, &UA2834695},
	{SG84655, &UA31410},
	{SG84656, &UA64689},
	{SG84671, &UA408615},
	{SG84672, &UA830678},
	{SG84673, &UA838403},
	{SG84674, &UA863708},
	{SG84676, &UA1894414},
	{SG84677, &UA2638726},
	{SG84678, &UA2681824},
	{SG84679, &UA2682206},
	{SG84680, &UA2683146},
	{SG84681, &UA2684526},
	{SG84682, &UA2690942},
	{SG84753, &UA2297347},
	{SG84765, &UA166744},
	{SG84766, &UA599738},
	{SG84817, &UA64319},
	{SG84867, &UA681349},
	{SG84930, &UA85160},
	{SG84931, &UA139275},
	{SG84976, &UA3043513},
	{SG85015, &UA2271611},
	{SG85026, &UA797932},
	{SG85049, &UA261156},
	{SG85200, &UA640841},
	{SG85201, &UA718030},
	{SG85202, &UA815400},
	{SG85203, &UA824389},
	{SG85204, &UA834467},
	{SG85205, &UA2684179},
	{SG85219, &UA290906},
	{SG85220, &UA915172},
	{SG85221, &UA1967563},
	{SG85222, &UA2329830},
	{SG85430, &UA27940},
	{SG85448, &UA2528991},
	{SG85464, &UA2271650},
	{SG85465, &UA2681669},
	{SG85466, &UA3069236},
	{SG85490, &UA126705},
	{SG85491, &UA2207844},
	{SG85581, &UA2002927},
	{SG85615, &UA171170},
	{SG85689, &UA2128824},
	{SG85716, &UA96715},
	{SG85717, &UA1872658},
	{SG85731, &UA15759},
	{SG85770, &UA627065},
	{SG85828, &UA35449},
	{SG85829, &UA62146},
	{SG85974, &UA13471},
	{SG86033, &UA1984159},
	{SG86107, &UA2129054},
	{SG86116, &UA416259},
	{SG86118, &UA432814},
	{SG86119, &UA684867},
	{SG86120, &UA881756},
	{SG86176, &UA675986},
	{SG86177, &UA713801},
	{SG86178, &UA2437801},
	{SG86179, &UA2437824},
	{SG86189, &UA683240},
	{SG86190, &UA848417},
	{SG86377, &UA833528},
	{SG86378, &UA2684209},
	{SG86399, &UA636384},
	{SG86409, &UA2000845},
	{SG86491, &UA526915},
	{SG86492, &UA838462},
	{SG86493, &UA1906577},
	{SG86494, &UA2264140},
	{SG86547, &UA625580},
	{SG86555, &UA781151},
	{SG86589, &UA823558},
	{SG86590, &UA826326},
	{SG86690, &UA854514},
	{SG86691, &UA2211511},
	{SG86739, &UA532513},
	{SG86808, &UA5091},
	{SG86819, &UA2803290},
	{SG86821, &UA760010},
	{SG86824, &UA2774773},
	{SG86847, &UA845756},
	{SG86865, &UA22552},
	{SG86872, &UA820065},
	{SG86877, &UA531439},
	{SG86908, &UA811660},
	{SG86911, &UA887569},
	{SG86912, &UA1903027},
	{SG86921, &UA85854},
	{SG86922, &UA843124},
	{SG86932, &UA493229},
	{SG86944, &UA2161211},
	{SG86953, &UA485061},
	{SG86955, &UA862674},
	{SG86961, &UA87445},
	{SG86962, &UA1866350},
	{SG86968, &UA14375},
	{SG86970, &UA15758},
	{SG86971, &UA49173},
	{SG86974, &UA373869},
	{SG86985, &UA56852},
	{SG86989, &UA67104},
	{SG86992, &UA858044},
	{SG87000, &UA32488},
	{SG87012, &UA86813},
	{NULL}};
const Segmentdata DOperaMini[] = {
	{SG76944, &UA241885},
	{SG76945, &UA416743},
	{SG76946, &UA657980},
	{SG76947, &UA818689},
	{SG76948, &UA2155154},
	{SG76949, &UA2404945},
	{SG76950, &UA2432688},
	{SG76951, &UA2797829},
	{SG76952, &UA3095071},
	{SG76962, &UA230943},
	{SG76963, &UA280359},
	{SG76964, &UA473978},
	{SG76965, &UA877558},
	{SG76966, &UA2797802},
	{SG76967, &UA589212},
	{SG76968, &UA2709616},
	{SG76969, &UA2799303},
	{SG76970, &UA3035262},
	{SG76971, &UA2431098},
	{SG76972, &UA2797797},
	{SG76973, &UA3054967},
	{SG76974, &UA3115137},
	{SG76977, &UA2430886},
	{SG76978, &UA2799283},
	{SG76979, &UA2875561},
	{SG77422, &UA439415},
	{SG77424, &UA893719},
	{SG77426, &UA2043650},
	{SG77428, &UA2389249},
	{SG77430, &UA2431215},
	{SG77431, &UA2431325},
	{SG78024, &UA311915},
	{SG78025, &UA813693},
	{SG78026, &UA2389965},
	{SG78027, &UA2431122},
	{SG78028, &UA2431229},
	{SG78460, &UA358778},
	{SG78461, &UA2431226},
	{SG78484, &UA157868},
	{SG78485, &UA2431193},
	{SG78486, &UA2587622},
	{SG78487, &UA3022668},
	{SG78488, &UA916027},
	{SG78489, &UA3033836},
	{SG78490, &UA3069079},
	{SG78491, &UA347349},
	{SG78492, &UA2389188},
	{SG78493, &UA2427614},
	{SG78494, &UA2431138},
	{SG79125, &UA806936},
	{SG79126, &UA2431212},
	{SG79725, &UA3109239},
	{SG79726, &UA3115160},
	{SG79946, &UA382023},
	{SG79947, &UA2389887},
	{SG79948, &UA2431125},
	{SG80458, &UA252520},
	{SG80459, &UA2431344},
	{SG80680, &UA1890034},
	{SG80682, &UA2431169},
	{SG80729, &UA3035165},
	{SG81178, &UA3114782},
	{SG81576, &UA2405151},
	{SG81577, &UA2709945},
	{SG81663, &UA3115868},
	{SG81921, &UA2797840},
	{SG82214, &UA2945949},
	{SG82405, &UA90223},
	{SG82442, &UA901537},
	{SG82443, &UA2431331},
	{SG83200, &UA4564},
	{SG83201, &UA112294},
	{SG83202, &UA243021},
	{SG83203, &UA623095},
	{SG83204, &UA774156},
	{SG83205, &UA2431806},
	{SG83220, &UA2481235},
	{SG83221, &UA2714975},
	{SG83222, &UA2874179},
	{SG83223, &UA3095423},
	{SG83297, &UA2431537},
	{SG83299, &UA2945976},
	{SG83334, &UA2431211},
	{SG83395, &UA94063},
	{SG83396, &UA2389386},
	{SG83543, &UA2945952},
	{SG83589, &UA1861347},
	{SG83590, &UA2431239},
	{SG83662, &UA2799285},
	{SG83857, &UA2431808},
	{SG83911, &UA2432161},
	{SG84045, &UA618731},
	{SG84140, &UA2946994},
	{SG84176, &UA100382},
	{SG84179, &UA2431188},
	{SG84213, &UA2945969},
	{SG84412, &UA2431284},
	{SG84550, &UA2947723},
	{SG84554, &UA2798572},
	{SG84558, &UA843695},
	{SG84559, &UA890462},
	{SG84627, &UA2435237},
	{SG84647, &UA127471},
	{SG84705, &UA2947542},
	{SG84869, &UA2966033},
	{SG85364, &UA2947710},
	{SG85370, &UA2431983},
	{SG85372, &UA2874172},
	{SG85458, &UA702941},
	{SG85459, &UA2473131},
	{SG85460, &UA2494451},
	{SG85485, &UA652795},
	{SG85513, &UA3017608},
	{SG86064, &UA3094694},
	{SG86286, &UA2782291},
	{SG86430, &UA2431557},
	{SG86475, &UA2796125},
	{SG86507, &UA3094958},
	{SG86566, &UA832570},
	{SG86577, &UA3013091},
	{SG86594, &UA3095037},
	{SG86604, &UA3094637},
	{SG86621, &UA398802},
	{SG86622, &UA2211453},
	{SG86639, &UA2411215},
	{SG86640, &UA2432523},
	{SG86641, &UA2291380},
	{SG86644, &UA2352834},
	{SG86653, &UA1954620},
	{SG86656, &UA3110928},
	{SG86676, &UA656305},
	{SG86692, &UA3113938},
	{SG86722, &UA3110880},
	{SG86732, &UA3109568},
	{SG86792, &UA2778013},
	{SG86806, &UA428050},
	{SG86807, &UA646283},
	{SG86823, &UA2436062},
	{SG86827, &UA3112958},
	{SG86834, &UA3126879},
	{SG86842, &UA2372839},
	{SG86853, &UA3110061},
	{SG86874, &UA3110071},
	{SG86875, &UA3113388},
	{SG86884, &UA3111921},
	{SG86885, &UA3114721},
	{SG86886, &UA2436597},
	{SG86893, &UA2475792},
	{SG86896, &UA3113264},
	{SG86898, &UA3041104},
	{SG86905, &UA3127479},
	{SG86935, &UA3115272},
	{SG86939, &UA3126909},
	{SG86941, &UA2366752},
	{SG86952, &UA614025},
	{SG86964, &UA897335},
	{SG86965, &UA2159984},
	{SG86999, &UA90224},
	{SG87013, &UA3084427},
	{NULL}};
const Segmentdata DOperaDesktop[] = {
	{SG76981, &UA235494},
	{SG76982, &UA627041},
	{SG76983, &UA899556},
	{SG76984, &UA899724},
	{SG76985, &UA901401},
	{SG76986, &UA902725},
	{SG76987, &UA902833},
	{SG76988, &UA903075},
	{SG76989, &UA903096},
	{SG76990, &UA903266},
	{SG76991, &UA903388},
	{SG76992, &UA903403},
	{SG76993, &UA903371},
	{SG76994, &UA903543},
	{SG76995, &UA903556},
	{SG76996, &UA903560},
	{SG76997, &UA903708},
	{SG76998, &UA904074},
	{SG76999, &UA904428},
	{SG77000, &UA904465},
	{SG77001, &UA904651},
	{SG77002, &UA904820},
	{SG77003, &UA904866},
	{SG77004, &UA904884},
	{SG77005, &UA905054},
	{SG77006, &UA905112},
	{SG77007, &UA905175},
	{SG77008, &UA905286},
	{SG77009, &UA905343},
	{SG77010, &UA905507},
	{SG77011, &UA905723},
	{SG77012, &UA905757},
	{SG77013, &UA906118},
	{SG77014, &UA906554},
	{SG77015, &UA906831},
	{SG77016, &UA906850},
	{SG77017, &UA907022},
	{SG77018, &UA907090},
	{SG77019, &UA907451},
	{SG77020, &UA907556},
	{SG77021, &UA907804},
	{SG77022, &UA907994},
	{SG77023, &UA908619},
	{SG77024, &UA909479},
	{SG77025, &UA909691},
	{SG77026, &UA910732},
	{SG77027, &UA925364},
	{SG77028, &UA1955857},
	{SG77029, &UA1956222},
	{SG77030, &UA1968479},
	{SG77031, &UA1984572},
	{SG77032, &UA1988451},
	{SG77033, &UA1989374},
	{SG77034, &UA1994736},
	{SG77035, &UA1997123},
	{SG77036, &UA1997128},
	{SG77037, &UA1997804},
	{SG77038, &UA1997837},
	{SG77039, &UA1998537},
	{SG77040, &UA1999023},
	{SG77041, &UA1999042},
	{SG77042, &UA1999478},
	{SG77043, &UA1999508},
	{SG77044, &UA1999594},
	{SG77045, &UA1999889},
	{SG77046, &UA1999908},
	{SG77047, &UA1999913},
	{SG77048, &UA1999915},
	{SG77049, &UA2000823},
	{SG77050, &UA2001575},
	{SG77051, &UA2001678},
	{SG77052, &UA2001966},
	{SG77053, &UA2002743},
	{SG77054, &UA2002942},
	{SG77055, &UA2002987},
	{SG77056, &UA2003524},
	{SG77057, &UA2003818},
	{SG77058, &UA2003821},
	{SG77059, &UA2003844},
	{SG77060, &UA2003846},
	{SG77061, &UA2003896},
	{SG77062, &UA2004279},
	{SG77063, &UA2004633},
	{SG77064, &UA2004637},
	{SG77065, &UA2004638},
	{SG77066, &UA2004880},
	{SG77067, &UA2004892},
	{SG77068, &UA2006093},
	{SG77069, &UA2006112},
	{SG77070, &UA2007373},
	{SG77071, &UA2007382},
	{SG77072, &UA2008901},
	{SG77073, &UA2012029},
	{SG77074, &UA2016143},
	{SG77075, &UA2041308},
	{SG77076, &UA2154735},
	{SG77077, &UA2156377},
	{SG77078, &UA2161627},
	{SG77079, &UA2168257},
	{SG77080, &UA2168911},
	{SG77081, &UA2169139},
	{SG77082, &UA2169578},
	{SG77083, &UA2171352},
	{SG77084, &UA2171902},
	{SG77085, &UA2173872},
	{SG77086, &UA2175407},
	{SG77087, &UA2185163},
	{SG77088, &UA2200353},
	{SG77089, &UA2204905},
	{SG77090, &UA2205751},
	{SG77091, &UA2262767},
	{SG77092, &UA2262771},
	{SG77093, &UA2262774},
	{SG77094, &UA2262864},
	{SG77095, &UA2318250},
	{SG77096, &UA2318429},
	{SG77097, &UA2329551},
	{SG77098, &UA2333528},
	{SG77099, &UA2339531},
	{SG77100, &UA2340032},
	{SG77101, &UA2340043},
	{SG77102, &UA2340909},
	{SG77103, &UA2363041},
	{SG77104, &UA2369562},
	{SG77105, &UA2375962},
	{SG77106, &UA2384952},
	{SG77107, &UA2385918},
	{SG77108, &UA2386535},
	{SG77109, &UA2395959},
	{SG77110, &UA2403423},
	{SG77111, &UA2403650},
	{SG77112, &UA2403777},
	{SG77113, &UA2404634},
	{SG77114, &UA2404837},
	{SG77115, &UA2413959},
	{SG77116, &UA2425689},
	{SG77117, &UA2446608},
	{SG77118, &UA2455925},
	{SG77119, &UA2460264},
	{SG77120, &UA2469078},
	{SG77121, &UA2502915},
	{SG77122, &UA2504781},
	{SG77123, &UA2526612},
	{SG77124, &UA2527163},
	{SG77125, &UA2556759},
	{SG77126, &UA2578130},
	{SG77127, &UA2578875},
	{SG77128, &UA2583789},
	{SG77129, &UA2588227},
	{SG77130, &UA2600709},
	{SG77131, &UA2665939},
	{SG77132, &UA2697927},
	{SG77133, &UA2701970},
	{SG77134, &UA2705196},
	{SG77135, &UA2705782},
	{SG77136, &UA2722875},
	{SG77137, &UA2734200},
	{SG77138, &UA2749462},
	{SG77139, &UA2750781},
	{SG77140, &UA2750985},
	{SG77141, &UA2760848},
	{SG77142, &UA2775779},
	{SG77143, &UA2792139},
	{SG77144, &UA2794698},
	{SG77145, &UA2809299},
	{SG77146, &UA2819518},
	{SG77147, &UA2819900},
	{SG77148, &UA2823848},
	{SG77149, &UA2847952},
	{SG77150, &UA2850371},
	{SG77151, &UA2853966},
	{SG77152, &UA2855717},
	{SG77153, &UA2859412},
	{SG77154, &UA2859944},
	{SG77155, &UA2862610},
	{SG77156, &UA2867972},
	{SG77157, &UA2868095},
	{SG77158, &UA2988256},
	{SG77159, &UA2992161},
	{SG77160, &UA3045755},
	{SG77161, &UA3099537},
	{SG77163, &UA2013535},
	{SG77164, &UA2040426},
	{SG77165, &UA2040582},
	{SG77166, &UA2040590},
	{SG77167, &UA2040819},
	{SG77168, &UA2041075},
	{SG77169, &UA2041251},
	{SG77170, &UA2041258},
	{SG77171, &UA2041334},
	{SG77172, &UA2041344},
	{SG77173, &UA2041465},
	{SG77174, &UA2041705},
	{SG77176, &UA2041748},
	{SG77177, &UA2041782},
	{SG77178, &UA2041923},
	{SG77179, &UA2042166},
	{SG77180, &UA2042355},
	{SG77181, &UA2042360},
	{SG77182, &UA2042670},
	{SG77183, &UA2042704},
	{SG77184, &UA2042783},
	{SG77185, &UA2185198},
	{SG77186, &UA2185399},
	{SG77187, &UA2186266},
	{SG77188, &UA2186603},
	{SG77189, &UA2186888},
	{SG77190, &UA2187675},
	{SG77191, &UA2188614},
	{SG77192, &UA2188623},
	{SG77193, &UA2189029},
	{SG77194, &UA2189920},
	{SG77195, &UA2190370},
	{SG77196, &UA2190381},
	{SG77197, &UA2190386},
	{SG77198, &UA2192610},
	{SG77199, &UA2193042},
	{SG77200, &UA2193417},
	{SG77201, &UA2195263},
	{SG77202, &UA2195515},
	{SG77203, &UA2195779},
	{SG77204, &UA2198367},
	{SG77205, &UA2198375},
	{SG77206, &UA2198392},
	{SG77207, &UA2198394},
	{SG77208, &UA2199283},
	{SG77209, &UA2199394},
	{SG77210, &UA2199571},
	{SG77211, &UA2200576},
	{SG77212, &UA2201375},
	{SG77213, &UA2202513},
	{SG77214, &UA2203333},
	{SG77215, &UA2205192},
	{SG77216, &UA2205395},
	{SG77217, &UA2205402},
	{SG77218, &UA2226614},
	{SG77219, &UA2227947},
	{SG77220, &UA2227948},
	{SG77221, &UA2229282},
	{SG77222, &UA2229472},
	{SG77223, &UA2230804},
	{SG77224, &UA2230812},
	{SG77225, &UA2230869},
	{SG77226, &UA2230895},
	{SG77227, &UA2232671},
	{SG77228, &UA2233710},
	{SG77229, &UA2234661},
	{SG77230, &UA2262763},
	{SG77231, &UA2262764},
	{SG77232, &UA2262765},
	{SG77233, &UA2262769},
	{SG77234, &UA2262785},
	{SG77235, &UA2262831},
	{SG77236, &UA2262836},
	{SG77237, &UA2262841},
	{SG77238, &UA2262816},
	{SG77239, &UA2346200},
	{SG77240, &UA2369693},
	{SG77241, &UA2405874},
	{SG77242, &UA2428273},
	{SG77243, &UA2429555},
	{SG77244, &UA2431107},
	{SG77245, &UA2435741},
	{SG77246, &UA2450068},
	{SG77247, &UA2451384},
	{SG77248, &UA2455493},
	{SG77249, &UA2484545},
	{SG77250, &UA2490836},
	{SG77251, &UA2551805},
	{SG77252, &UA2560877},
	{SG77253, &UA2561846},
	{SG77254, &UA2570218},
	{SG77255, &UA2580227},
	{SG77256, &UA2599607},
	{SG77257, &UA2600607},
	{SG77258, &UA2603528},
	{SG77259, &UA2663947},
	{SG77260, &UA2669067},
	{SG77261, &UA2704451},
	{SG77262, &UA2704524},
	{SG77263, &UA2708184},
	{SG77264, &UA2711170},
	{SG77265, &UA2749200},
	{SG77266, &UA2760595},
	{SG77267, &UA2765940},
	{SG77268, &UA2770104},
	{SG77269, &UA2823770},
	{SG77270, &UA2853109},
	{SG77271, &UA2855458},
	{SG77272, &UA2856841},
	{SG77273, &UA2891958},
	{SG77274, &UA2981491},
	{SG77275, &UA2998252},
	{SG77276, &UA3001038},
	{SG77277, &UA3001420},
	{SG77278, &UA3009869},
	{SG77279, &UA3092206},
	{SG77280, &UA3092744},
	{SG77281, &UA3124858},
	{SG77290, &UA2585297},
	{SG77291, &UA2585381},
	{SG77292, &UA2668755},
	{SG77293, &UA2668947},
	{SG77294, &UA2670776},
	{SG77295, &UA2670782},
	{SG77296, &UA2671025},
	{SG77297, &UA2671670},
	{SG77298, &UA2671828},
	{SG77299, &UA2671839},
	{SG77300, &UA2672004},
	{SG77301, &UA2672092},
	{SG77302, &UA2672094},
	{SG77303, &UA2672127},
	{SG77304, &UA2672204},
	{SG77305, &UA2672208},
	{SG77306, &UA2672318},
	{SG77307, &UA2672326},
	{SG77308, &UA2672344},
	{SG77309, &UA2672543},
	{SG77310, &UA2672620},
	{SG77311, &UA2672646},
	{SG77312, &UA2672712},
	{SG77313, &UA2672813},
	{SG77314, &UA2672812},
	{SG77315, &UA2672971},
	{SG77316, &UA2672980},
	{SG77317, &UA2673080},
	{SG77318, &UA2673162},
	{SG77319, &UA2673177},
	{SG77320, &UA2673248},
	{SG77321, &UA2673253},
	{SG77322, &UA2673363},
	{SG77323, &UA2673369},
	{SG77324, &UA2673547},
	{SG77325, &UA2673551},
	{SG77326, &UA2673823},
	{SG77327, &UA2674000},
	{SG77328, &UA2674180},
	{SG77329, &UA2674198},
	{SG77330, &UA2674199},
	{SG77331, &UA2674424},
	{SG77332, &UA2674435},
	{SG77333, &UA2674447},
	{SG77334, &UA2674437},
	{SG77335, &UA2674723},
	{SG77336, &UA2674741},
	{SG77337, &UA2674831},
	{SG77338, &UA2675051},
	{SG77339, &UA2675067},
	{SG77340, &UA2675079},
	{SG77341, &UA2675252},
	{SG77342, &UA2675266},
	{SG77343, &UA2675375},
	{SG77344, &UA2675385},
	{SG77345, &UA2675700},
	{SG77346, &UA2676068},
	{SG77347, &UA2676377},
	{SG77348, &UA2676637},
	{SG77349, &UA2676647},
	{SG77350, &UA2676660},
	{SG77351, &UA2676785},
	{SG77352, &UA2676788},
	{SG77353, &UA2676882},
	{SG77354, &UA2677696},
	{SG77355, &UA2677698},
	{SG77356, &UA2677701},
	{SG77357, &UA2677711},
	{SG77358, &UA2680314},
	{SG77359, &UA2706645},
	{SG77360, &UA2707312},
	{SG77361, &UA2707472},
	{SG77362, &UA2707583},
	{SG77363, &UA2707866},
	{SG77364, &UA2707914},
	{SG77365, &UA2707935},
	{SG77366, &UA2708716},
	{SG77367, &UA2708810},
	{SG77368, &UA2709425},
	{SG77369, &UA2710432},
	{SG77370, &UA2711217},
	{SG77371, &UA2711495},
	{SG77372, &UA2712287},
	{SG77373, &UA2712880},
	{SG77374, &UA2713516},
	{SG77375, &UA2714167},
	{SG77376, &UA2714219},
	{SG77377, &UA2714229},
	{SG77378, &UA2714572},
	{SG77379, &UA2714578},
	{SG77380, &UA2714577},
	{SG77381, &UA2714808},
	{SG77382, &UA2714809},
	{SG77383, &UA2716518},
	{SG77384, &UA2717105},
	{SG77385, &UA2734822},
	{SG77386, &UA2734957},
	{SG77387, &UA2735119},
	{SG77388, &UA2735184},
	{SG77389, &UA2735318},
	{SG77390, &UA2735427},
	{SG77391, &UA2741510},
	{SG77392, &UA2751573},
	{SG77393, &UA2751832},
	{SG77394, &UA2752095},
	{SG77395, &UA2753666},
	{SG77396, &UA2754145},
	{SG77397, &UA2754544},
	{SG77398, &UA2754595},
	{SG77399, &UA2767248},
	{SG77400, &UA2767897},
	{SG77401, &UA2768558},
	{SG77402, &UA2773430},
	{SG77403, &UA2780922},
	{SG77404, &UA2781166},
	{SG77405, &UA2784746},
	{SG77406, &UA2794897},
	{SG77407, &UA2795206},
	{SG77408, &UA2797237},
	{SG77409, &UA2799503},
	{SG77410, &UA2861479},
	{SG77411, &UA2863194},
	{SG77412, &UA2864727},
	{SG77413, &UA2865040},
	{SG77414, &UA2873423},
	{SG77415, &UA2874783},
	{SG77416, &UA2875562},
	{SG77417, &UA2876120},
	{SG77418, &UA2883651},
	{SG77419, &UA2887746},
	{SG77420, &UA2893060},
	{SG77421, &UA2893869},
	{SG77423, &UA2894799},
	{SG77425, &UA2894829},
	{SG77427, &UA2946869},
	{SG77429, &UA2955601},
	{SG77432, &UA2955758},
	{SG77433, &UA2963847},
	{SG77434, &UA2965394},
	{SG77435, &UA2966502},
	{SG77436, &UA2971089},
	{SG77437, &UA2972381},
	{SG77438, &UA2991437},
	{SG77439, &UA3015697},
	{SG77440, &UA3016136},
	{SG77441, &UA3017781},
	{SG77442, &UA3023417},
	{SG77443, &UA3023759},
	{SG77444, &UA3024189},
	{SG77445, &UA3036657},
	{SG77446, &UA3039387},
	{SG77447, &UA3058517},
	{SG77448, &UA3059056},
	{SG77449, &UA3077748},
	{SG77450, &UA3113256},
	{SG77451, &UA3125458},
	{SG77452, &UA3128610},
	{SG77453, &UA2357970},
	{SG77454, &UA2357976},
	{SG77455, &UA2367745},
	{SG77456, &UA2367752},
	{SG77461, &UA2367768},
	{SG77463, &UA2367779},
	{SG77465, &UA2367920},
	{SG77466, &UA2367919},
	{SG77467, &UA2367934},
	{SG77468, &UA2368135},
	{SG77469, &UA2368258},
	{SG77470, &UA2368259},
	{SG77471, &UA2368261},
	{SG77472, &UA2368269},
	{SG77473, &UA2368272},
	{SG77474, &UA2368578},
	{SG77475, &UA2368580},
	{SG77476, &UA2368594},
	{SG77477, &UA2368595},
	{SG77478, &UA2369740},
	{SG77479, &UA2387216},
	{SG77480, &UA2387347},
	{SG77481, &UA2387535},
	{SG77482, &UA2388148},
	{SG77483, &UA2388150},
	{SG77484, &UA2388152},
	{SG77485, &UA2388154},
	{SG77486, &UA2388156},
	{SG77487, &UA2388158},
	{SG77488, &UA2388160},
	{SG77489, &UA2388161},
	{SG77490, &UA2388296},
	{SG77491, &UA2388300},
	{SG77492, &UA2388301},
	{SG77493, &UA2388308},
	{SG77494, &UA2388506},
	{SG77495, &UA2388508},
	{SG77496, &UA2389356},
	{SG77497, &UA2389478},
	{SG77498, &UA2389485},
	{SG77499, &UA2389777},
	{SG77500, &UA2390388},
	{SG77501, &UA2390857},
	{SG77502, &UA2404434},
	{SG77503, &UA2404524},
	{SG77504, &UA2404722},
	{SG77505, &UA2405092},
	{SG77506, &UA2405101},
	{SG77507, &UA2405441},
	{SG77508, &UA2405624},
	{SG77509, &UA2405626},
	{SG77510, &UA2405680},
	{SG77511, &UA2405799},
	{SG77512, &UA2405886},
	{SG77513, &UA2422733},
	{SG77514, &UA2422964},
	{SG77515, &UA2423866},
	{SG77516, &UA2424110},
	{SG77517, &UA2426508},
	{SG77518, &UA2426513},
	{SG77519, &UA2426612},
	{SG77520, &UA2426719},
	{SG77521, &UA2426741},
	{SG77522, &UA2428027},
	{SG77523, &UA2428108},
	{SG77524, &UA2428201},
	{SG77525, &UA2429412},
	{SG77526, &UA2429418},
	{SG77527, &UA2430650},
	{SG77528, &UA2431038},
	{SG77529, &UA2437657},
	{SG77530, &UA2457308},
	{SG77531, &UA2457377},
	{SG77532, &UA2457467},
	{SG77533, &UA2462054},
	{SG77534, &UA2483013},
	{SG77535, &UA2536568},
	{SG77536, &UA2554578},
	{SG77537, &UA2554846},
	{SG77538, &UA2554878},
	{SG77539, &UA2559464},
	{SG77540, &UA2563053},
	{SG77541, &UA2565803},
	{SG77542, &UA2565975},
	{SG77543, &UA2570194},
	{SG77544, &UA2596318},
	{SG77545, &UA2596854},
	{SG77546, &UA2599869},
	{SG77547, &UA2654616},
	{SG77548, &UA2668515},
	{SG77549, &UA2670762},
	{SG77550, &UA2672286},
	{SG77551, &UA2673509},
	{SG77552, &UA2708002},
	{SG77553, &UA2717469},
	{SG77554, &UA2727992},
	{SG77555, &UA2730159},
	{SG77556, &UA2769479},
	{SG77557, &UA2843058},
	{SG77558, &UA2843215},
	{SG77559, &UA2843349},
	{SG77560, &UA2843794},
	{SG77561, &UA2859959},
	{SG77562, &UA2864458},
	{SG77563, &UA2865023},
	{SG77564, &UA3005096},
	{SG77565, &UA3006467},
	{SG77566, &UA3009229},
	{SG77567, &UA3021615},
	{SG77568, &UA3059568},
	{SG77569, &UA3078487},
	{SG77570, &UA3086724},
	{SG77571, &UA3103203},
	{SG77572, &UA3103420},
	{SG77575, &UA621838},
	{SG77581, &UA2198530},
	{SG77582, &UA2203706},
	{SG77583, &UA2352707},
	{SG77584, &UA2353243},
	{SG77585, &UA2355299},
	{SG77586, &UA2355624},
	{SG77587, &UA2355880},
	{SG77588, &UA2356625},
	{SG77589, &UA2357319},
	{SG77590, &UA2357352},
	{SG77591, &UA2357357},
	{SG77592, &UA2357446},
	{SG77593, &UA2357453},
	{SG77594, &UA2357562},
	{SG77595, &UA2357680},
	{SG77596, &UA2357907},
	{SG77597, &UA2358004},
	{SG77598, &UA2358013},
	{SG77599, &UA2358071},
	{SG77600, &UA2358183},
	{SG77601, &UA2358406},
	{SG77602, &UA2358413},
	{SG77603, &UA2358841},
	{SG77604, &UA2359082},
	{SG77605, &UA2359104},
	{SG77606, &UA2359520},
	{SG77607, &UA2359524},
	{SG77608, &UA2359526},
	{SG77609, &UA2360774},
	{SG77610, &UA2361333},
	{SG77611, &UA2361321},
	{SG77612, &UA2361363},
	{SG77613, &UA2361404},
	{SG77614, &UA2361444},
	{SG77615, &UA2361495},
	{SG77616, &UA2361636},
	{SG77617, &UA2361638},
	{SG77618, &UA2361826},
	{SG77619, &UA2362185},
	{SG77620, &UA2367492},
	{SG77621, &UA2367552},
	{SG77622, &UA2368335},
	{SG77623, &UA2368440},
	{SG77624, &UA2368494},
	{SG77625, &UA2368784},
	{SG77626, &UA2386708},
	{SG77627, &UA2387025},
	{SG77628, &UA2387356},
	{SG77629, &UA2387362},
	{SG77630, &UA2387377},
	{SG77631, &UA2403524},
	{SG77632, &UA2403753},
	{SG77633, &UA2403842},
	{SG77634, &UA2405104},
	{SG77635, &UA2422862},
	{SG77636, &UA2422899},
	{SG77637, &UA2423261},
	{SG77638, &UA2425971},
	{SG77639, &UA2456841},
	{SG77640, &UA2456868},
	{SG77641, &UA2456993},
	{SG77642, &UA2457565},
	{SG77643, &UA2459657},
	{SG77644, &UA2470161},
	{SG77645, &UA2507057},
	{SG77646, &UA2556504},
	{SG77647, &UA2578556},
	{SG77648, &UA2583072},
	{SG77649, &UA2653425},
	{SG77650, &UA2659347},
	{SG77651, &UA2666523},
	{SG77652, &UA2707018},
	{SG77653, &UA2767001},
	{SG77654, &UA2790414},
	{SG77655, &UA2796493},
	{SG77656, &UA2840992},
	{SG77657, &UA2841622},
	{SG77658, &UA2841713},
	{SG77659, &UA2841785},
	{SG77660, &UA2842030},
	{SG77661, &UA2855413},
	{SG77662, &UA2864533},
	{SG77663, &UA2891447},
	{SG77664, &UA2982869},
	{SG77665, &UA3043279},
	{SG77666, &UA3045144},
	{SG77667, &UA3048236},
	{SG77668, &UA3073068},
	{SG77669, &UA2509572},
	{SG77670, &UA2509590},
	{SG77671, &UA2509600},
	{SG77672, &UA2510164},
	{SG77673, &UA2510172},
	{SG77674, &UA2510171},
	{SG77675, &UA2510174},
	{SG77676, &UA2510178},
	{SG77677, &UA2510786},
	{SG77678, &UA2510789},
	{SG77679, &UA2510792},
	{SG77680, &UA2510809},
	{SG77681, &UA2511315},
	{SG77684, &UA2511316},
	{SG77685, &UA2511318},
	{SG77687, &UA2511323},
	{SG77688, &UA2511324},
	{SG77689, &UA2511328},
	{SG77690, &UA2512436},
	{SG77691, &UA2512777},
	{SG77692, &UA2512788},
	{SG77693, &UA2512791},
	{SG77694, &UA2512793},
	{SG77695, &UA2512795},
	{SG77696, &UA2513067},
	{SG77697, &UA2513301},
	{SG77698, &UA2513307},
	{SG77699, &UA2513348},
	{SG77700, &UA2513356},
	{SG77701, &UA2513358},
	{SG77702, &UA2513882},
	{SG77703, &UA2514040},
	{SG77704, &UA2514383},
	{SG77705, &UA2514385},
	{SG77706, &UA2514407},
	{SG77707, &UA2514556},
	{SG77708, &UA2515374},
	{SG77709, &UA2515527},
	{SG77710, &UA2515783},
	{SG77711, &UA2515812},
	{SG77712, &UA2516053},
	{SG77713, &UA2516142},
	{SG77714, &UA2516227},
	{SG77715, &UA2516350},
	{SG77716, &UA2516392},
	{SG77717, &UA2516441},
	{SG77718, &UA2530809},
	{SG77719, &UA2531279},
	{SG77720, &UA2532482},
	{SG77721, &UA2533233},
	{SG77722, &UA2533240},
	{SG77723, &UA2535448},
	{SG77724, &UA2536408},
	{SG77725, &UA2536646},
	{SG77726, &UA2536866},
	{SG77727, &UA2536960},
	{SG77728, &UA2537097},
	{SG77729, &UA2537099},
	{SG77730, &UA2537171},
	{SG77731, &UA2537290},
	{SG77732, &UA2537575},
	{SG77733, &UA2537652},
	{SG77734, &UA2537731},
	{SG77735, &UA2537770},
	{SG77736, &UA2537774},
	{SG77737, &UA2537901},
	{SG77738, &UA2539484},
	{SG77739, &UA2539581},
	{SG77740, &UA2558599},
	{SG77741, &UA2559077},
	{SG77742, &UA2559079},
	{SG77743, &UA2559442},
	{SG77744, &UA2560658},
	{SG77745, &UA2561141},
	{SG77747, &UA2561496},
	{SG77748, &UA2562513},
	{SG77750, &UA2563112},
	{SG77751, &UA2563420},
	{SG77752, &UA2563954},
	{SG77754, &UA2564084},
	{SG77755, &UA2564150},
	{SG77756, &UA2564154},
	{SG77757, &UA2564711},
	{SG77758, &UA2568011},
	{SG77759, &UA2568194},
	{SG77760, &UA2571451},
	{SG77761, &UA2571561},
	{SG77762, &UA2572165},
	{SG77763, &UA2572189},
	{SG77764, &UA2580750},
	{SG77765, &UA2581342},
	{SG77766, &UA2581429},
	{SG77767, &UA2582146},
	{SG77768, &UA2585907},
	{SG77769, &UA2585982},
	{SG77770, &UA2598863},
	{SG77771, &UA2599138},
	{SG77772, &UA2599373},
	{SG77773, &UA2599672},
	{SG77774, &UA2599856},
	{SG77775, &UA2600409},
	{SG77776, &UA2602196},
	{SG77777, &UA2602454},
	{SG77778, &UA2606403},
	{SG77779, &UA2661589},
	{SG77780, &UA2661816},
	{SG77781, &UA2662887},
	{SG77782, &UA2674370},
	{SG77783, &UA2699198},
	{SG77784, &UA2701230},
	{SG77785, &UA2703098},
	{SG77786, &UA2703618},
	{SG77787, &UA2716078},
	{SG77788, &UA2718876},
	{SG77789, &UA2750007},
	{SG77790, &UA2754594},
	{SG77791, &UA2769053},
	{SG77792, &UA2792948},
	{SG77793, &UA2858094},
	{SG77794, &UA2863029},
	{SG77795, &UA2867703},
	{SG77796, &UA2868308},
	{SG77797, &UA2868650},
	{SG77798, &UA2877257},
	{SG77799, &UA2895381},
	{SG77800, &UA2898123},
	{SG77801, &UA2944303},
	{SG77802, &UA2964939},
	{SG77803, &UA3003036},
	{SG77804, &UA3003248},
	{SG77805, &UA3015659},
	{SG77806, &UA3017229},
	{SG77807, &UA3020235},
	{SG77808, &UA3020499},
	{SG77809, &UA3023087},
	{SG77810, &UA3033266},
	{SG77811, &UA3058155},
	{SG77812, &UA3058247},
	{SG77813, &UA3058276},
	{SG77814, &UA3105466},
	{SG77815, &UA3107924},
	{SG77816, &UA3125355},
	{SG77817, &UA3125471},
	{SG77818, &UA3128121},
	{SG77822, &UA777007},
	{SG77823, &UA808600},
	{SG77824, &UA811427},
	{SG77825, &UA848557},
	{SG77826, &UA899043},
	{SG77827, &UA899467},
	{SG77828, &UA899660},
	{SG77829, &UA899839},
	{SG77830, &UA903006},
	{SG77831, &UA903142},
	{SG77832, &UA903156},
	{SG77833, &UA903339},
	{SG77834, &UA903356},
	{SG77835, &UA903387},
	{SG77836, &UA903392},
	{SG77837, &UA903414},
	{SG77838, &UA903531},
	{SG77839, &UA903548},
	{SG77840, &UA903566},
	{SG77841, &UA903572},
	{SG77842, &UA903595},
	{SG77843, &UA903641},
	{SG77844, &UA904048},
	{SG77845, &UA904065},
	{SG77846, &UA904068},
	{SG77847, &UA904117},
	{SG77848, &UA904265},
	{SG77849, &UA904533},
	{SG77850, &UA904638},
	{SG77851, &UA904832},
	{SG77852, &UA905065},
	{SG77853, &UA905071},
	{SG77854, &UA905458},
	{SG77855, &UA905466},
	{SG77856, &UA905474},
	{SG77857, &UA905543},
	{SG77858, &UA905569},
	{SG77859, &UA905755},
	{SG77860, &UA905975},
	{SG77861, &UA906275},
	{SG77862, &UA906360},
	{SG77863, &UA906423},
	{SG77864, &UA906582},
	{SG77865, &UA906662},
	{SG77866, &UA906698},
	{SG77867, &UA906704},
	{SG77868, &UA906734},
	{SG77869, &UA906960},
	{SG77870, &UA907095},
	{SG77871, &UA907118},
	{SG77872, &UA907147},
	{SG77873, &UA907269},
	{SG77874, &UA908263},
	{SG77875, &UA909605},
	{SG77876, &UA911579},
	{SG77877, &UA924850},
	{SG77878, &UA1949738},
	{SG77879, &UA1954131},
	{SG77880, &UA1957564},
	{SG77881, &UA1959399},
	{SG77882, &UA1965208},
	{SG77883, &UA1965881},
	{SG77884, &UA1978745},
	{SG77885, &UA1988598},
	{SG77886, &UA1993208},
	{SG77887, &UA1994953},
	{SG77888, &UA1997115},
	{SG77889, &UA1997120},
	{SG77890, &UA1997194},
	{SG77891, &UA1997201},
	{SG77892, &UA1997807},
	{SG77893, &UA1997813},
	{SG77894, &UA1997819},
	{SG77895, &UA1997824},
	{SG77896, &UA1997826},
	{SG77897, &UA1997832},
	{SG77898, &UA1998518},
	{SG77899, &UA1998519},
	{SG77900, &UA1998527},
	{SG77901, &UA1998542},
	{SG77902, &UA1998543},
	{SG77903, &UA1999025},
	{SG77904, &UA1999029},
	{SG77905, &UA1999145},
	{SG77906, &UA1999464},
	{SG77907, &UA1999465},
	{SG77908, &UA1999486},
	{SG77909, &UA1999892},
	{SG77910, &UA2000871},
	{SG77911, &UA2000872},
	{SG77912, &UA2000873},
	{SG77913, &UA2001745},
	{SG77914, &UA2001953},
	{SG77915, &UA2001993},
	{SG77916, &UA2002418},
	{SG77917, &UA2002622},
	{SG77918, &UA2002932},
	{SG77919, &UA2002934},
	{SG77920, &UA2003271},
	{SG77921, &UA2003820},
	{SG77922, &UA2003838},
	{SG77923, &UA2003841},
	{SG77924, &UA2004274},
	{SG77925, &UA2004630},
	{SG77926, &UA2004640},
	{SG77927, &UA2004660},
	{SG77928, &UA2004672},
	{SG77929, &UA2004674},
	{SG77930, &UA2004690},
	{SG77931, &UA2007358},
	{SG77932, &UA2007367},
	{SG77933, &UA2008879},
	{SG77934, &UA2009019},
	{SG77935, &UA2012018},
	{SG77936, &UA2012040},
	{SG77937, &UA2012688},
	{SG77938, &UA2015305},
	{SG77939, &UA2042858},
	{SG77940, &UA2155273},
	{SG77941, &UA2156418},
	{SG77942, &UA2168205},
	{SG77943, &UA2168544},
	{SG77944, &UA2168700},
	{SG77945, &UA2169075},
	{SG77946, &UA2169479},
	{SG77947, &UA2169484},
	{SG77948, &UA2170769},
	{SG77949, &UA2171183},
	{SG77950, &UA2171251},
	{SG77951, &UA2189904},
	{SG77952, &UA2206001},
	{SG77953, &UA2206148},
	{SG77954, &UA2219133},
	{SG77955, &UA2225537},
	{SG77956, &UA2226102},
	{SG77957, &UA2226618},
	{SG77958, &UA2262390},
	{SG77959, &UA2262486},
	{SG77960, &UA2328700},
	{SG77961, &UA2332704},
	{SG77962, &UA2332809},
	{SG77963, &UA2339697},
	{SG77964, &UA2340343},
	{SG77965, &UA2355557},
	{SG77966, &UA2362868},
	{SG77967, &UA2362984},
	{SG77968, &UA2385919},
	{SG77969, &UA2386112},
	{SG77970, &UA2404223},
	{SG77971, &UA2405243},
	{SG77972, &UA2423746},
	{SG77973, &UA2445998},
	{SG77974, &UA2446043},
	{SG77975, &UA2455589},
	{SG77976, &UA2456101},
	{SG77977, &UA2460517},
	{SG77978, &UA2468013},
	{SG77979, &UA2480804},
	{SG77980, &UA2482495},
	{SG77981, &UA2482760},
	{SG77982, &UA2503190},
	{SG77983, &UA2503473},
	{SG77984, &UA2504387},
	{SG77985, &UA2529150},
	{SG77986, &UA2553530},
	{SG77987, &UA2583165},
	{SG77988, &UA2603531},
	{SG77989, &UA2644362},
	{SG77990, &UA2657799},
	{SG77991, &UA2684634},
	{SG77992, &UA2687088},
	{SG77993, &UA2695081},
	{SG77994, &UA2697525},
	{SG77995, &UA2705909},
	{SG77996, &UA2710517},
	{SG77997, &UA2727097},
	{SG77998, &UA2728992},
	{SG77999, &UA2765161},
	{SG78000, &UA2767825},
	{SG78001, &UA2767993},
	{SG78002, &UA2780386},
	{SG78003, &UA2810819},
	{SG78004, &UA2818226},
	{SG78005, &UA2824751},
	{SG78006, &UA2846265},
	{SG78007, &UA2846499},
	{SG78008, &UA2852621},
	{SG78009, &UA2854680},
	{SG78010, &UA2859628},
	{SG78011, &UA2867420},
	{SG78012, &UA2868694},
	{SG78013, &UA2933243},
	{SG78014, &UA2935324},
	{SG78015, &UA2988842},
	{SG78016, &UA2990457},
	{SG78017, &UA3045209},
	{SG78018, &UA3086442},
	{SG78029, &UA2900225},
	{SG78030, &UA2900254},
	{SG78031, &UA2901023},
	{SG78032, &UA2901035},
	{SG78033, &UA2901129},
	{SG78034, &UA2901131},
	{SG78035, &UA2901141},
	{SG78036, &UA2901142},
	{SG78037, &UA2944658},
	{SG78038, &UA2944660},
	{SG78039, &UA2944816},
	{SG78040, &UA2944818},
	{SG78041, &UA2944893},
	{SG78042, &UA2944970},
	{SG78043, &UA2944983},
	{SG78044, &UA2945084},
	{SG78045, &UA2945089},
	{SG78046, &UA2945193},
	{SG78047, &UA2945211},
	{SG78048, &UA2945433},
	{SG78049, &UA2945441},
	{SG78050, &UA2945443},
	{SG78051, &UA2945587},
	{SG78052, &UA2945614},
	{SG78053, &UA2945619},
	{SG78054, &UA2945628},
	{SG78055, &UA2945629},
	{SG78056, &UA2945630},
	{SG78057, &UA2945633},
	{SG78058, &UA2945672},
	{SG78059, &UA2945729},
	{SG78060, &UA2945730},
	{SG78061, &UA2945734},
	{SG78062, &UA2945830},
	{SG78063, &UA2945841},
	{SG78064, &UA2945909},
	{SG78065, &UA2946061},
	{SG78066, &UA2946072},
	{SG78067, &UA2946077},
	{SG78068, &UA2946067},
	{SG78069, &UA2946456},
	{SG78070, &UA2946479},
	{SG78071, &UA2946480},
	{SG78072, &UA2947039},
	{SG78073, &UA2947239},
	{SG78074, &UA2947383},
	{SG78075, &UA2947385},
	{SG78076, &UA2947389},
	{SG78077, &UA2948041},
	{SG78078, &UA2948453},
	{SG78079, &UA2948529},
	{SG78080, &UA2948536},
	{SG78081, &UA2948540},
	{SG78082, &UA2963589},
	{SG78083, &UA2963792},
	{SG78084, &UA2964935},
	{SG78085, &UA2964936},
	{SG78086, &UA2964938},
	{SG78087, &UA2964951},
	{SG78088, &UA2964952},
	{SG78089, &UA2965303},
	{SG78090, &UA2965305},
	{SG78091, &UA2965315},
	{SG78092, &UA2965321},
	{SG78093, &UA2965371},
	{SG78094, &UA2965377},
	{SG78095, &UA2965421},
	{SG78096, &UA2965551},
	{SG78097, &UA2965738},
	{SG78098, &UA2965742},
	{SG78099, &UA2965923},
	{SG78100, &UA2966292},
	{SG78101, &UA2966320},
	{SG78102, &UA2966448},
	{SG78103, &UA2966463},
	{SG78104, &UA2966472},
	{SG78105, &UA2966978},
	{SG78106, &UA2967063},
	{SG78107, &UA2967505},
	{SG78108, &UA2967699},
	{SG78109, &UA2967826},
	{SG78110, &UA2968188},
	{SG78111, &UA2968226},
	{SG78112, &UA2968232},
	{SG78113, &UA2968236},
	{SG78114, &UA2968800},
	{SG78115, &UA2969405},
	{SG78116, &UA2969694},
	{SG78117, &UA2971237},
	{SG78118, &UA2971342},
	{SG78119, &UA2974567},
	{SG78120, &UA2974838},
	{SG78121, &UA3011035},
	{SG78122, &UA3011038},
	{SG78123, &UA3012297},
	{SG78124, &UA3012405},
	{SG78125, &UA3012807},
	{SG78126, &UA3012808},
	{SG78127, &UA3013165},
	{SG78128, &UA3013174},
	{SG78129, &UA3013311},
	{SG78130, &UA3013428},
	{SG78131, &UA3013782},
	{SG78132, &UA3013825},
	{SG78133, &UA3013944},
	{SG78134, &UA3014304},
	{SG78135, &UA3014484},
	{SG78136, &UA3014706},
	{SG78137, &UA3014893},
	{SG78138, &UA3014896},
	{SG78139, &UA3015167},
	{SG78140, &UA3015321},
	{SG78141, &UA3015325},
	{SG78142, &UA3015455},
	{SG78143, &UA3015456},
	{SG78144, &UA3016129},
	{SG78145, &UA3016453},
	{SG78146, &UA3016471},
	{SG78147, &UA3016599},
	{SG78148, &UA3017065},
	{SG78149, &UA3017789},
	{SG78150, &UA3018247},
	{SG78151, &UA3018480},
	{SG78152, &UA3018872},
	{SG78153, &UA3019110},
	{SG78154, &UA3019132},
	{SG78155, &UA3019671},
	{SG78156, &UA3022743},
	{SG78157, &UA3024060},
	{SG78158, &UA3026868},
	{SG78159, &UA3030013},
	{SG78160, &UA3033329},
	{SG78161, &UA3034644},
	{SG78162, &UA3036342},
	{SG78163, &UA3039310},
	{SG78164, &UA3057014},
	{SG78165, &UA3057033},
	{SG78166, &UA3057060},
	{SG78167, &UA3057066},
	{SG78168, &UA3058855},
	{SG78169, &UA3077434},
	{SG78170, &UA3077673},
	{SG78171, &UA3077872},
	{SG78172, &UA3078077},
	{SG78173, &UA3078460},
	{SG78174, &UA3082230},
	{SG78175, &UA3091858},
	{SG78176, &UA3095812},
	{SG78177, &UA3097280},
	{SG78178, &UA3107369},
	{SG78179, &UA3107477},
	{SG78180, &UA3115500},
	{SG78181, &UA3124601},
	{SG78182, &UA3124863},
	{SG78183, &UA2015524},
	{SG78184, &UA2040351},
	{SG78185, &UA2040418},
	{SG78187, &UA2040969},
	{SG78188, &UA2040978},
	{SG78189, &UA2041073},
	{SG78190, &UA2041091},
	{SG78191, &UA2041207},
	{SG78192, &UA2041234},
	{SG78193, &UA2041456},
	{SG78194, &UA2041492},
	{SG78195, &UA2041796},
	{SG78196, &UA2041917},
	{SG78197, &UA2041925},
	{SG78198, &UA2041926},
	{SG78199, &UA2041930},
	{SG78200, &UA2042154},
	{SG78201, &UA2042273},
	{SG78202, &UA2042631},
	{SG78203, &UA2042708},
	{SG78204, &UA2042776},
	{SG78205, &UA2181543},
	{SG78206, &UA2184015},
	{SG78207, &UA2185062},
	{SG78208, &UA2185979},
	{SG78209, &UA2185980},
	{SG78210, &UA2187960},
	{SG78211, &UA2188622},
	{SG78212, &UA2189018},
	{SG78213, &UA2189571},
	{SG78214, &UA2189943},
	{SG78215, &UA2190366},
	{SG78216, &UA2190795},
	{SG78217, &UA2191113},
	{SG78218, &UA2191303},
	{SG78219, &UA2193040},
	{SG78220, &UA2193197},
	{SG78221, &UA2193401},
	{SG78222, &UA2194501},
	{SG78223, &UA2194591},
	{SG78224, &UA2194707},
	{SG78225, &UA2194869},
	{SG78226, &UA2195269},
	{SG78227, &UA2195654},
	{SG78228, &UA2197320},
	{SG78229, &UA2198382},
	{SG78230, &UA2199119},
	{SG78231, &UA2199275},
	{SG78232, &UA2199291},
	{SG78233, &UA2199384},
	{SG78234, &UA2200598},
	{SG78235, &UA2200760},
	{SG78236, &UA2200761},
	{SG78237, &UA2202512},
	{SG78238, &UA2202527},
	{SG78239, &UA2202839},
	{SG78240, &UA2205529},
	{SG78241, &UA2225393},
	{SG78242, &UA2225595},
	{SG78243, &UA2225594},
	{SG78244, &UA2226819},
	{SG78245, &UA2226922},
	{SG78246, &UA2228095},
	{SG78247, &UA2228333},
	{SG78248, &UA2228837},
	{SG78249, &UA2228903},
	{SG78250, &UA2228900},
	{SG78251, &UA2229002},
	{SG78252, &UA2230213},
	{SG78253, &UA2230482},
	{SG78254, &UA2230779},
	{SG78255, &UA2230837},
	{SG78256, &UA2233979},
	{SG78257, &UA2235618},
	{SG78258, &UA2262414},
	{SG78259, &UA2262415},
	{SG78260, &UA2262435},
	{SG78261, &UA2262436},
	{SG78262, &UA2262451},
	{SG78263, &UA2262479},
	{SG78264, &UA2262480},
	{SG78265, &UA2262505},
	{SG78266, &UA2262516},
	{SG78267, &UA2345160},
	{SG78268, &UA2369755},
	{SG78269, &UA2398389},
	{SG78270, &UA2404060},
	{SG78271, &UA2404525},
	{SG78272, &UA2424616},
	{SG78273, &UA2427568},
	{SG78274, &UA2436541},
	{SG78275, &UA2449712},
	{SG78276, &UA2449920},
	{SG78277, &UA2464777},
	{SG78278, &UA2471188},
	{SG78279, &UA2483559},
	{SG78280, &UA2536033},
	{SG78281, &UA2558944},
	{SG78282, &UA2559393},
	{SG78283, &UA2560814},
	{SG78284, &UA2561301},
	{SG78285, &UA2571283},
	{SG78286, &UA2581871},
	{SG78287, &UA2591774},
	{SG78288, &UA2603743},
	{SG78289, &UA2665182},
	{SG78290, &UA2666231},
	{SG78291, &UA2668835},
	{SG78292, &UA2693458},
	{SG78293, &UA2707546},
	{SG78294, &UA2709249},
	{SG78295, &UA2712609},
	{SG78296, &UA2748818},
	{SG78297, &UA2774097},
	{SG78298, &UA2777089},
	{SG78299, &UA2822715},
	{SG78300, &UA2825659},
	{SG78301, &UA2834798},
	{SG78302, &UA2956300},
	{SG78303, &UA2994535},
	{SG78304, &UA3034055},
	{SG78305, &UA3061093},
	{SG78306, &UA3078056},
	{SG78307, &UA3119113},
	{SG78308, &UA3125064},
	{SG78328, &UA2387788},
	{SG78329, &UA2388354},
	{SG78330, &UA2426258},
	{SG78331, &UA2426929},
	{SG78332, &UA2426933},
	{SG78333, &UA2427089},
	{SG78334, &UA2427101},
	{SG78335, &UA2427110},
	{SG78336, &UA2427264},
	{SG78337, &UA2427597},
	{SG78338, &UA2427612},
	{SG78339, &UA2427901},
	{SG78340, &UA2427902},
	{SG78341, &UA2428167},
	{SG78342, &UA2429197},
	{SG78343, &UA2430540},
	{SG78344, &UA2430872},
	{SG78345, &UA2432048},
	{SG78346, &UA2432402},
	{SG78347, &UA2432655},
	{SG78348, &UA2434000},
	{SG78349, &UA2434088},
	{SG78350, &UA2436129},
	{SG78351, &UA2436429},
	{SG78352, &UA2437387},
	{SG78353, &UA2437424},
	{SG78354, &UA2437439},
	{SG78355, &UA2437443},
	{SG78356, &UA2437444},
	{SG78357, &UA2437484},
	{SG78358, &UA2437495},
	{SG78359, &UA2437513},
	{SG78360, &UA2437520},
	{SG78361, &UA2437522},
	{SG78364, &UA2437525},
	{SG78366, &UA2437526},
	{SG78368, &UA2437527},
	{SG78370, &UA2437529},
	{SG78371, &UA2437533},
	{SG78372, &UA2437567},
	{SG78373, &UA2437569},
	{SG78374, &UA2437573},
	{SG78375, &UA2437600},
	{SG78376, &UA2437605},
	{SG78377, &UA2437621},
	{SG78378, &UA2437622},
	{SG78379, &UA2437607},
	{SG78380, &UA2437647},
	{SG78381, &UA2437660},
	{SG78382, &UA2459469},
	{SG78383, &UA2459846},
	{SG78384, &UA2460288},
	{SG78385, &UA2460416},
	{SG78386, &UA2460428},
	{SG78387, &UA2460566},
	{SG78388, &UA2460588},
	{SG78389, &UA2460592},
	{SG78390, &UA2460614},
	{SG78391, &UA2460816},
	{SG78392, &UA2460965},
	{SG78393, &UA2461069},
	{SG78394, &UA2461387},
	{SG78395, &UA2461798},
	{SG78396, &UA2467935},
	{SG78397, &UA2468497},
	{SG78398, &UA2468853},
	{SG78399, &UA2468888},
	{SG78400, &UA2468969},
	{SG78401, &UA2469400},
	{SG78402, &UA2469665},
	{SG78403, &UA2484145},
	{SG78404, &UA2484176},
	{SG78405, &UA2484895},
	{SG78406, &UA2485258},
	{SG78407, &UA2485259},
	{SG78408, &UA2485322},
	{SG78409, &UA2485851},
	{SG78410, &UA2486103},
	{SG78411, &UA2486781},
	{SG78412, &UA2490658},
	{SG78413, &UA2507169},
	{SG78414, &UA2508982},
	{SG78415, &UA2529500},
	{SG78416, &UA2534895},
	{SG78417, &UA2537921},
	{SG78418, &UA2538123},
	{SG78419, &UA2539612},
	{SG78420, &UA2556211},
	{SG78421, &UA2558491},
	{SG78422, &UA2560081},
	{SG78423, &UA2561714},
	{SG78424, &UA2565019},
	{SG78425, &UA2569150},
	{SG78426, &UA2600448},
	{SG78427, &UA2604210},
	{SG78428, &UA2606116},
	{SG78429, &UA2660537},
	{SG78430, &UA2665393},
	{SG78431, &UA2669035},
	{SG78432, &UA2670010},
	{SG78433, &UA2699622},
	{SG78434, &UA2716817},
	{SG78435, &UA2747526},
	{SG78436, &UA2752630},
	{SG78437, &UA2756725},
	{SG78438, &UA2763310},
	{SG78439, &UA2763824},
	{SG78440, &UA2767585},
	{SG78441, &UA2846908},
	{SG78442, &UA2847074},
	{SG78443, &UA2961130},
	{SG78444, &UA2969641},
	{SG78445, &UA2996590},
	{SG78446, &UA3076654},
	{SG78447, &UA3078648},
	{SG78448, &UA3088696},
	{SG78449, &UA3092130},
	{SG78450, &UA3107406},
	{SG78508, &UA93491},
	{SG78509, &UA98415},
	{SG78510, &UA120887},
	{SG78511, &UA153193},
	{SG78512, &UA192045},
	{SG78513, &UA301332},
	{SG78514, &UA312642},
	{SG78515, &UA321516},
	{SG78516, &UA462391},
	{SG78517, &UA473115},
	{SG78518, &UA500283},
	{SG78519, &UA812686},
	{SG78520, &UA848297},
	{SG78521, &UA1909434},
	{SG78522, &UA2140616},
	{SG78523, &UA2147832},
	{SG78524, &UA2150131},
	{SG78525, &UA2262265},
	{SG78526, &UA2276922},
	{SG78527, &UA2305447},
	{SG78528, &UA2409780},
	{SG78529, &UA2801596},
	{SG78558, &UA92871},
	{SG78560, &UA93315},
	{SG78562, &UA101675},
	{SG78563, &UA125223},
	{SG78564, &UA129522},
	{SG78565, &UA173601},
	{SG78566, &UA748325},
	{SG78568, &UA787290},
	{SG78569, &UA1898638},
	{SG78591, &UA615842},
	{SG78592, &UA624560},
	{SG78593, &UA624652},
	{SG78594, &UA628606},
	{SG78595, &UA630890},
	{SG78596, &UA632044},
	{SG78597, &UA634018},
	{SG78598, &UA647524},
	{SG78599, &UA648999},
	{SG78600, &UA656923},
	{SG78601, &UA657559},
	{SG78602, &UA659293},
	{SG78603, &UA665096},
	{SG78604, &UA669198},
	{SG78605, &UA670056},
	{SG78606, &UA671603},
	{SG78607, &UA674960},
	{SG78608, &UA679654},
	{SG78609, &UA682020},
	{SG78610, &UA686703},
	{SG78611, &UA686915},
	{SG78612, &UA688559},
	{SG78613, &UA694012},
	{SG78614, &UA708332},
	{SG78615, &UA728862},
	{SG78616, &UA755427},
	{SG78617, &UA767843},
	{SG78618, &UA770772},
	{SG78619, &UA771310},
	{SG78620, &UA771464},
	{SG78621, &UA772608},
	{SG78622, &UA774528},
	{SG78623, &UA777399},
	{SG78624, &UA783067},
	{SG78625, &UA784799},
	{SG78626, &UA799899},
	{SG78627, &UA807159},
	{SG78628, &UA814748},
	{SG78629, &UA818989},
	{SG78630, &UA828652},
	{SG78631, &UA829873},
	{SG78632, &UA834309},
	{SG78633, &UA835418},
	{SG78634, &UA875748},
	{SG78635, &UA880635},
	{SG78636, &UA886673},
	{SG78637, &UA900169},
	{SG78638, &UA900272},
	{SG78639, &UA900313},
	{SG78640, &UA900508},
	{SG78641, &UA900590},
	{SG78642, &UA904827},
	{SG78643, &UA1952858},
	{SG78644, &UA1983718},
	{SG78645, &UA1995650},
	{SG78646, &UA2159898},
	{SG78647, &UA2160091},
	{SG78648, &UA2181398},
	{SG78649, &UA2189514},
	{SG78650, &UA2218437},
	{SG78651, &UA2220628},
	{SG78652, &UA2230718},
	{SG78653, &UA2335008},
	{SG78654, &UA2385239},
	{SG78655, &UA2386550},
	{SG78656, &UA2422384},
	{SG78657, &UA2465960},
	{SG78658, &UA2486813},
	{SG78659, &UA2527289},
	{SG78660, &UA2578207},
	{SG78661, &UA2654504},
	{SG78662, &UA2666949},
	{SG78663, &UA2684924},
	{SG78664, &UA2707268},
	{SG78665, &UA2727122},
	{SG78666, &UA2839343},
	{SG78667, &UA2867975},
	{SG78668, &UA2984109},
	{SG78669, &UA3072579},
	{SG78683, &UA2585392},
	{SG78684, &UA2669626},
	{SG78685, &UA2670007},
	{SG78686, &UA2670053},
	{SG78687, &UA2670079},
	{SG78688, &UA2670775},
	{SG78689, &UA2671749},
	{SG78690, &UA2671812},
	{SG78691, &UA2671851},
	{SG78692, &UA2671885},
	{SG78693, &UA2672090},
	{SG78694, &UA2672176},
	{SG78695, &UA2672298},
	{SG78696, &UA2672345},
	{SG78697, &UA2672449},
	{SG78698, &UA2672466},
	{SG78699, &UA2672523},
	{SG78700, &UA2672524},
	{SG78701, &UA2672671},
	{SG78702, &UA2672703},
	{SG78703, &UA2672808},
	{SG78704, &UA2672814},
	{SG78705, &UA2672911},
	{SG78706, &UA2672968},
	{SG78707, &UA2672985},
	{SG78708, &UA2672999},
	{SG78709, &UA2673032},
	{SG78710, &UA2673079},
	{SG78711, &UA2673094},
	{SG78712, &UA2673225},
	{SG78713, &UA2673266},
	{SG78714, &UA2673539},
	{SG78715, &UA2673544},
	{SG78716, &UA2673632},
	{SG78717, &UA2673879},
	{SG78718, &UA2673967},
	{SG78719, &UA2674052},
	{SG78720, &UA2674183},
	{SG78721, &UA2674200},
	{SG78722, &UA2674213},
	{SG78723, &UA2674225},
	{SG78724, &UA2674722},
	{SG78725, &UA2674729},
	{SG78726, &UA2674952},
	{SG78727, &UA2674957},
	{SG78728, &UA2674964},
	{SG78729, &UA2674968},
	{SG78730, &UA2675316},
	{SG78731, &UA2675522},
	{SG78732, &UA2675566},
	{SG78733, &UA2675591},
	{SG78734, &UA2675677},
	{SG78735, &UA2675679},
	{SG78736, &UA2675775},
	{SG78737, &UA2675833},
	{SG78739, &UA2675911},
	{SG78741, &UA2675955},
	{SG78742, &UA2676369},
	{SG78743, &UA2676383},
	{SG78744, &UA2676387},
	{SG78745, &UA2676648},
	{SG78746, &UA2676662},
	{SG78747, &UA2676761},
	{SG78748, &UA2676859},
	{SG78749, &UA2676864},
	{SG78750, &UA2676867},
	{SG78751, &UA2677047},
	{SG78752, &UA2677066},
	{SG78754, &UA2677227},
	{SG78756, &UA2677232},
	{SG78758, &UA2677238},
	{SG78759, &UA2677703},
	{SG78760, &UA2679811},
	{SG78761, &UA2706350},
	{SG78762, &UA2708229},
	{SG78763, &UA2708714},
	{SG78764, &UA2708959},
	{SG78765, &UA2709021},
	{SG78766, &UA2709036},
	{SG78767, &UA2709152},
	{SG78768, &UA2709412},
	{SG78769, &UA2710431},
	{SG78770, &UA2711688},
	{SG78771, &UA2712531},
	{SG78772, &UA2713241},
	{SG78773, &UA2713871},
	{SG78774, &UA2713910},
	{SG78775, &UA2714071},
	{SG78776, &UA2714159},
	{SG78777, &UA2714224},
	{SG78778, &UA2714419},
	{SG78779, &UA2714580},
	{SG78780, &UA2714804},
	{SG78781, &UA2714810},
	{SG78782, &UA2715337},
	{SG78783, &UA2716072},
	{SG78784, &UA2717720},
	{SG78785, &UA2719097},
	{SG78786, &UA2734413},
	{SG78787, &UA2734807},
	{SG78788, &UA2734813},
	{SG78789, &UA2735002},
	{SG78790, &UA2735591},
	{SG78791, &UA2735694},
	{SG78792, &UA2735718},
	{SG78793, &UA2735910},
	{SG78794, &UA2736093},
	{SG78795, &UA2736218},
	{SG78796, &UA2740478},
	{SG78797, &UA2750857},
	{SG78798, &UA2751498},
	{SG78799, &UA2751726},
	{SG78800, &UA2754304},
	{SG78801, &UA2755250},
	{SG78802, &UA2756718},
	{SG78803, &UA2767253},
	{SG78804, &UA2767380},
	{SG78805, &UA2767474},
	{SG78806, &UA2768359},
	{SG78808, &UA2768672},
	{SG78810, &UA2772193},
	{SG78811, &UA2780842},
	{SG78812, &UA2780924},
	{SG78813, &UA2782961},
	{SG78814, &UA2794122},
	{SG78815, &UA2795233},
	{SG78816, &UA2795478},
	{SG78817, &UA2796811},
	{SG78818, &UA2860986},
	{SG78819, &UA2863242},
	{SG78820, &UA2870116},
	{SG78821, &UA2871958},
	{SG78822, &UA2876504},
	{SG78823, &UA2886066},
	{SG78824, &UA2887170},
	{SG78825, &UA2895071},
	{SG78826, &UA2896122},
	{SG78827, &UA2897402},
	{SG78828, &UA2936091},
	{SG78829, &UA2936428},
	{SG78830, &UA2955876},
	{SG78831, &UA2966140},
	{SG78832, &UA2966498},
	{SG78833, &UA2970268},
	{SG78834, &UA2971505},
	{SG78835, &UA2971584},
	{SG78836, &UA2992720},
	{SG78837, &UA2992881},
	{SG78838, &UA2993164},
	{SG78839, &UA3005898},
	{SG78840, &UA3013095},
	{SG78841, &UA3014686},
	{SG78842, &UA3023262},
	{SG78843, &UA3023770},
	{SG78844, &UA3028323},
	{SG78845, &UA3030075},
	{SG78846, &UA3034688},
	{SG78847, &UA3057101},
	{SG78848, &UA3060133},
	{SG78849, &UA3101781},
	{SG78850, &UA3107199},
	{SG78851, &UA3125254},
	{SG78852, &UA3126903},
	{SG78869, &UA164272},
	{SG78870, &UA165629},
	{SG78871, &UA172406},
	{SG78872, &UA172632},
	{SG78873, &UA186235},
	{SG78874, &UA296858},
	{SG78875, &UA298902},
	{SG78876, &UA299354},
	{SG78877, &UA300751},
	{SG78878, &UA301324},
	{SG78879, &UA301600},
	{SG78880, &UA303280},
	{SG78881, &UA328459},
	{SG78882, &UA379003},
	{SG78883, &UA408797},
	{SG78884, &UA439282},
	{SG78885, &UA446387},
	{SG78886, &UA452891},
	{SG78887, &UA498773},
	{SG78888, &UA523507},
	{SG78889, &UA524447},
	{SG78890, &UA532254},
	{SG78891, &UA557680},
	{SG78892, &UA558133},
	{SG78893, &UA566197},
	{SG78894, &UA576422},
	{SG78895, &UA583914},
	{SG78896, &UA607795},
	{SG78897, &UA688739},
	{SG78898, &UA705715},
	{SG78899, &UA705721},
	{SG78900, &UA722275},
	{SG78901, &UA733376},
	{SG78902, &UA754706},
	{SG78903, &UA759503},
	{SG78904, &UA789094},
	{SG78905, &UA798250},
	{SG78906, &UA845078},
	{SG78907, &UA892001},
	{SG78908, &UA895178},
	{SG78909, &UA906614},
	{SG78910, &UA1878621},
	{SG78911, &UA1882670},
	{SG78912, &UA1904324},
	{SG78913, &UA1955722},
	{SG78914, &UA1966275},
	{SG78915, &UA1974341},
	{SG78916, &UA1980938},
	{SG78917, &UA1990353},
	{SG78918, &UA2146185},
	{SG78919, &UA2158672},
	{SG78920, &UA2159612},
	{SG78921, &UA2220805},
	{SG78922, &UA2221017},
	{SG78923, &UA2221264},
	{SG78924, &UA2222231},
	{SG78925, &UA2228787},
	{SG78926, &UA2232277},
	{SG78927, &UA2262847},
	{SG78928, &UA2271059},
	{SG78929, &UA2285640},
	{SG78930, &UA2319200},
	{SG78931, &UA2337921},
	{SG78932, &UA2347938},
	{SG78933, &UA2395983},
	{SG78934, &UA2397878},
	{SG78935, &UA2400051},
	{SG78936, &UA2426276},
	{SG78937, &UA2444142},
	{SG78938, &UA2455742},
	{SG78939, &UA2456869},
	{SG78940, &UA2457108},
	{SG78941, &UA2518724},
	{SG78942, &UA2529425},
	{SG78943, &UA2556777},
	{SG78944, &UA2654212},
	{SG78945, &UA2694391},
	{SG78946, &UA2695145},
	{SG78947, &UA2707954},
	{SG78948, &UA2727611},
	{SG78949, &UA2732089},
	{SG78950, &UA2750008},
	{SG78951, &UA2829558},
	{SG78952, &UA3048063},
	{SG78953, &UA3086263},
	{SG78954, &UA3103677},
	{SG78994, &UA92395},
	{SG78995, &UA92752},
	{SG78996, &UA99208},
	{SG78997, &UA153869},
	{SG78998, &UA178436},
	{SG78999, &UA240570},
	{SG79000, &UA259531},
	{SG79001, &UA335604},
	{SG79002, &UA339040},
	{SG79003, &UA531362},
	{SG79004, &UA916620},
	{SG79005, &UA1879157},
	{SG79006, &UA1884194},
	{SG79007, &UA1888513},
	{SG79008, &UA1891037},
	{SG79009, &UA1902905},
	{SG79010, &UA2132169},
	{SG79011, &UA2215057},
	{SG79012, &UA2216367},
	{SG79013, &UA2275719},
	{SG79014, &UA2408689},
	{SG79015, &UA2685292},
	{SG79016, &UA2806003},
	{SG79017, &UA2977889},
	{SG79019, &UA615829},
	{SG79020, &UA622998},
	{SG79021, &UA623239},
	{SG79022, &UA623342},
	{SG79023, &UA626448},
	{SG79024, &UA628168},
	{SG79025, &UA629010},
	{SG79026, &UA630283},
	{SG79027, &UA630633},
	{SG79028, &UA632258},
	{SG79029, &UA635379},
	{SG79030, &UA642959},
	{SG79031, &UA646342},
	{SG79032, &UA647938},
	{SG79033, &UA654142},
	{SG79034, &UA663104},
	{SG79035, &UA663124},
	{SG79036, &UA668266},
	{SG79037, &UA671473},
	{SG79038, &UA673838},
	{SG79039, &UA675091},
	{SG79040, &UA676517},
	{SG79041, &UA678541},
	{SG79042, &UA685431},
	{SG79043, &UA687461},
	{SG79044, &UA688695},
	{SG79045, &UA691565},
	{SG79046, &UA699583},
	{SG79047, &UA703487},
	{SG79048, &UA709809},
	{SG79049, &UA716558},
	{SG79050, &UA740413},
	{SG79051, &UA749178},
	{SG79052, &UA755384},
	{SG79053, &UA765217},
	{SG79054, &UA767715},
	{SG79055, &UA768493},
	{SG79056, &UA768792},
	{SG79057, &UA770591},
	{SG79058, &UA773601},
	{SG79059, &UA775960},
	{SG79060, &UA783882},
	{SG79061, &UA787942},
	{SG79062, &UA789899},
	{SG79063, &UA790436},
	{SG79064, &UA791294},
	{SG79065, &UA796420},
	{SG79066, &UA804209},
	{SG79067, &UA815471},
	{SG79068, &UA817816},
	{SG79069, &UA820175},
	{SG79070, &UA830462},
	{SG79071, &UA841732},
	{SG79072, &UA849578},
	{SG79073, &UA853839},
	{SG79074, &UA860777},
	{SG79075, &UA866160},
	{SG79076, &UA866343},
	{SG79077, &UA899889},
	{SG79078, &UA900566},
	{SG79079, &UA901316},
	{SG79080, &UA901483},
	{SG79081, &UA904829},
	{SG79082, &UA906288},
	{SG79083, &UA911181},
	{SG79084, &UA1974216},
	{SG79085, &UA1995355},
	{SG79086, &UA2157249},
	{SG79087, &UA2161244},
	{SG79088, &UA2162153},
	{SG79089, &UA2183879},
	{SG79090, &UA2184211},
	{SG79091, &UA2229596},
	{SG79092, &UA2325643},
	{SG79093, &UA2355292},
	{SG79094, &UA2377704},
	{SG79095, &UA2386168},
	{SG79096, &UA2427771},
	{SG79097, &UA2456593},
	{SG79098, &UA2485471},
	{SG79099, &UA2503052},
	{SG79103, &UA2503610},
	{SG79104, &UA2667524},
	{SG79105, &UA2693957},
	{SG79106, &UA2700819},
	{SG79107, &UA2704124},
	{SG79108, &UA2728120},
	{SG79109, &UA2751636},
	{SG79110, &UA2857892},
	{SG79111, &UA2893178},
	{SG79112, &UA3118843},
	{SG79127, &UA2509571},
	{SG79128, &UA2509581},
	{SG79129, &UA2509582},
	{SG79131, &UA2509598},
	{SG79133, &UA2510161},
	{SG79135, &UA2510167},
	{SG79136, &UA2510187},
	{SG79137, &UA2510162},
	{SG79138, &UA2510781},
	{SG79139, &UA2510787},
	{SG79140, &UA2510793},
	{SG79141, &UA2510794},
	{SG79142, &UA2510814},
	{SG79143, &UA2511304},
	{SG79144, &UA2511317},
	{SG79145, &UA2512444},
	{SG79146, &UA2512454},
	{SG79147, &UA2512779},
	{SG79148, &UA2512786},
	{SG79149, &UA2512798},
	{SG79150, &UA2512805},
	{SG79151, &UA2512807},
	{SG79152, &UA2512810},
	{SG79153, &UA2512812},
	{SG79154, &UA2512813},
	{SG79155, &UA2512892},
	{SG79156, &UA2513058},
	{SG79157, &UA2513064},
	{SG79158, &UA2513066},
	{SG79159, &UA2513302},
	{SG79160, &UA2513362},
	{SG79161, &UA2513862},
	{SG79162, &UA2514039},
	{SG79163, &UA2514047},
	{SG79164, &UA2514048},
	{SG79165, &UA2514054},
	{SG79166, &UA2514357},
	{SG79167, &UA2514410},
	{SG79168, &UA2514411},
	{SG79169, &UA2514550},
	{SG79170, &UA2515198},
	{SG79171, &UA2516056},
	{SG79172, &UA2516386},
	{SG79173, &UA2516588},
	{SG79174, &UA2530745},
	{SG79175, &UA2530810},
	{SG79176, &UA2530828},
	{SG79177, &UA2531051},
	{SG79178, &UA2532320},
	{SG79179, &UA2534830},
	{SG79180, &UA2535229},
	{SG79181, &UA2535288},
	{SG79182, &UA2536099},
	{SG79183, &UA2536156},
	{SG79184, &UA2536647},
	{SG79185, &UA2536764},
	{SG79186, &UA2536859},
	{SG79187, &UA2536860},
	{SG79188, &UA2536968},
	{SG79189, &UA2537279},
	{SG79190, &UA2537280},
	{SG79191, &UA2537283},
	{SG79192, &UA2537577},
	{SG79193, &UA2537656},
	{SG79194, &UA2537693},
	{SG79202, &UA2537730},
	{SG79203, &UA2537761},
	{SG79204, &UA2537773},
	{SG79205, &UA2537788},
	{SG79206, &UA2537974},
	{SG79207, &UA2539407},
	{SG79208, &UA2539412},
	{SG79209, &UA2558314},
	{SG79210, &UA2558360},
	{SG79211, &UA2558409},
	{SG79212, &UA2558531},
	{SG79213, &UA2558535},
	{SG79214, &UA2558747},
	{SG79215, &UA2559075},
	{SG79216, &UA2559129},
	{SG79217, &UA2559602},
	{SG79218, &UA2559757},
	{SG79219, &UA2560774},
	{SG79220, &UA2560783},
	{SG79221, &UA2562822},
	{SG79222, &UA2563845},
	{SG79223, &UA2568184},
	{SG79224, &UA2568389},
	{SG79225, &UA2570694},
	{SG79226, &UA2580653},
	{SG79227, &UA2580654},
	{SG79228, &UA2581196},
	{SG79229, &UA2581373},
	{SG79230, &UA2581763},
	{SG79231, &UA2581936},
	{SG79232, &UA2582200},
	{SG79233, &UA2584490},
	{SG79234, &UA2585909},
	{SG79235, &UA2585985},
	{SG79236, &UA2597056},
	{SG79237, &UA2598260},
	{SG79238, &UA2599340},
	{SG79239, &UA2599811},
	{SG79240, &UA2600581},
	{SG79241, &UA2601476},
	{SG79242, &UA2603055},
	{SG79243, &UA2603075},
	{SG79244, &UA2603426},
	{SG79245, &UA2603437},
	{SG79246, &UA2606402},
	{SG79247, &UA2661038},
	{SG79248, &UA2661141},
	{SG79249, &UA2662299},
	{SG79250, &UA2662759},
	{SG79251, &UA2672750},
	{SG79252, &UA2674266},
	{SG79253, &UA2702201},
	{SG79254, &UA2712617},
	{SG79255, &UA2730145},
	{SG79256, &UA2737640},
	{SG79257, &UA2748641},
	{SG79258, &UA2749268},
	{SG79259, &UA2756735},
	{SG79260, &UA2772521},
	{SG79261, &UA2779389},
	{SG79262, &UA2792663},
	{SG79263, &UA2852683},
	{SG79264, &UA2855489},
	{SG79265, &UA2859057},
	{SG79266, &UA2867243},
	{SG79267, &UA2886752},
	{SG79268, &UA2934830},
	{SG79269, &UA2944453},
	{SG79270, &UA2958046},
	{SG79271, &UA2958599},
	{SG79272, &UA3017517},
	{SG79273, &UA3021524},
	{SG79274, &UA3024110},
	{SG79275, &UA3050189},
	{SG79276, &UA3056539},
	{SG79277, &UA3062127},
	{SG79278, &UA3077527},
	{SG79279, &UA3100182},
	{SG79280, &UA3106332},
	{SG79281, &UA3108065},
	{SG79282, &UA3108104},
	{SG79283, &UA3108292},
	{SG79284, &UA3126738},
	{SG79285, &UA111898},
	{SG79286, &UA126209},
	{SG79287, &UA133153},
	{SG79288, &UA133377},
	{SG79289, &UA144160},
	{SG79290, &UA145176},
	{SG79291, &UA148905},
	{SG79292, &UA152198},
	{SG79293, &UA156014},
	{SG79294, &UA157721},
	{SG79295, &UA160456},
	{SG79296, &UA168699},
	{SG79297, &UA267665},
	{SG79298, &UA277463},
	{SG79299, &UA277961},
	{SG79300, &UA279422},
	{SG79301, &UA282951},
	{SG79302, &UA292634},
	{SG79303, &UA292973},
	{SG79304, &UA297923},
	{SG79305, &UA300769},
	{SG79306, &UA391599},
	{SG79307, &UA400800},
	{SG79308, &UA414434},
	{SG79309, &UA415315},
	{SG79310, &UA416079},
	{SG79311, &UA442314},
	{SG79312, &UA444100},
	{SG79313, &UA446292},
	{SG79314, &UA463690},
	{SG79315, &UA470439},
	{SG79316, &UA494090},
	{SG79317, &UA513245},
	{SG79319, &UA522482},
	{SG79320, &UA529163},
	{SG79321, &UA529377},
	{SG79322, &UA556300},
	{SG79324, &UA565461},
	{SG79326, &UA693788},
	{SG79327, &UA750702},
	{SG79328, &UA763353},
	{SG79329, &UA764998},
	{SG79330, &UA860100},
	{SG79331, &UA862112},
	{SG79332, &UA865124},
	{SG79333, &UA1842582},
	{SG79334, &UA1854565},
	{SG79335, &UA1871801},
	{SG79336, &UA1873291},
	{SG79337, &UA1890814},
	{SG79338, &UA1899573},
	{SG79339, &UA1941838},
	{SG79340, &UA1945090},
	{SG79341, &UA2146805},
	{SG79342, &UA2148780},
	{SG79343, &UA2151767},
	{SG79344, &UA2208773},
	{SG79345, &UA2209439},
	{SG79346, &UA2219063},
	{SG79347, &UA2297071},
	{SG79348, &UA2334228},
	{SG79349, &UA2410890},
	{SG79350, &UA2419721},
	{SG79351, &UA2455850},
	{SG79352, &UA2518562},
	{SG79353, &UA2523806},
	{SG79354, &UA2684605},
	{SG79355, &UA2688239},
	{SG79356, &UA2982786},
	{SG79357, &UA3086953},
	{SG79358, &UA2359222},
	{SG79359, &UA2367471},
	{SG79360, &UA2367743},
	{SG79361, &UA2367874},
	{SG79362, &UA2367914},
	{SG79363, &UA2367916},
	{SG79364, &UA2367917},
	{SG79365, &UA2367924},
	{SG79366, &UA2367940},
	{SG79367, &UA2368288},
	{SG79368, &UA2368576},
	{SG79369, &UA2368582},
	{SG79370, &UA2368584},
	{SG79371, &UA2368586},
	{SG79372, &UA2368603},
	{SG79373, &UA2368767},
	{SG79374, &UA2369691},
	{SG79375, &UA2369733},
	{SG79376, &UA2369736},
	{SG79377, &UA2387100},
	{SG79378, &UA2387208},
	{SG79379, &UA2387340},
	{SG79380, &UA2387342},
	{SG79381, &UA2387344},
	{SG79382, &UA2387528},
	{SG79383, &UA2387533},
	{SG79384, &UA2387880},
	{SG79385, &UA2387888},
	{SG79386, &UA2388151},
	{SG79387, &UA2388293},
	{SG79388, &UA2388306},
	{SG79389, &UA2388311},
	{SG79390, &UA2388499},
	{SG79391, &UA2388500},
	{SG79392, &UA2388502},
	{SG79393, &UA2388516},
	{SG79394, &UA2389464},
	{SG79395, &UA2389473},
	{SG79396, &UA2390381},
	{SG79397, &UA2390389},
	{SG79398, &UA2390393},
	{SG79399, &UA2390397},
	{SG79400, &UA2390402},
	{SG79401, &UA2390864},
	{SG79402, &UA2391483},
	{SG79403, &UA2404523},
	{SG79404, &UA2404588},
	{SG79405, &UA2404591},
	{SG79406, &UA2405091},
	{SG79407, &UA2405094},
	{SG79408, &UA2405138},
	{SG79409, &UA2405394},
	{SG79410, &UA2405516},
	{SG79411, &UA2405621},
	{SG79412, &UA2405622},
	{SG79413, &UA2405627},
	{SG79414, &UA2405629},
	{SG79415, &UA2405678},
	{SG79416, &UA2405682},
	{SG79417, &UA2405794},
	{SG79418, &UA2405838},
	{SG79419, &UA2405839},
	{SG79420, &UA2405872},
	{SG79421, &UA2405885},
	{SG79422, &UA2406622},
	{SG79423, &UA2422130},
	{SG79424, &UA2425286},
	{SG79425, &UA2425415},
	{SG79426, &UA2426735},
	{SG79427, &UA2427777},
	{SG79428, &UA2427783},
	{SG79429, &UA2427784},
	{SG79430, &UA2428024},
	{SG79431, &UA2428318},
	{SG79432, &UA2429963},
	{SG79433, &UA2430048},
	{SG79434, &UA2431868},
	{SG79435, &UA2437652},
	{SG79436, &UA2457074},
	{SG79437, &UA2457166},
	{SG79438, &UA2457313},
	{SG79439, &UA2466975},
	{SG79440, &UA2467010},
	{SG79441, &UA2469311},
	{SG79442, &UA2482353},
	{SG79443, &UA2482745},
	{SG79444, &UA2482915},
	{SG79445, &UA2505804},
	{SG79446, &UA2534942},
	{SG79447, &UA2561579},
	{SG79448, &UA2566356},
	{SG79449, &UA2571607},
	{SG79450, &UA2581545},
	{SG79451, &UA2595057},
	{SG79452, &UA2595179},
	{SG79453, &UA2602001},
	{SG79454, &UA2603744},
	{SG79455, &UA2653762},
	{SG79456, &UA2654388},
	{SG79457, &UA2655284},
	{SG79458, &UA2655376},
	{SG79459, &UA2659507},
	{SG79460, &UA2669089},
	{SG79461, &UA2706006},
	{SG79462, &UA2707828},
	{SG79463, &UA2708821},
	{SG79464, &UA2734383},
	{SG79465, &UA2768719},
	{SG79466, &UA2770096},
	{SG79467, &UA2790524},
	{SG79469, &UA2843114},
	{SG79471, &UA2845478},
	{SG79473, &UA2851459},
	{SG79475, &UA2952711},
	{SG79476, &UA2953617},
	{SG79477, &UA2986286},
	{SG79478, &UA3003614},
	{SG79479, &UA3020451},
	{SG79480, &UA3087940},
	{SG79481, &UA3092603},
	{SG79482, &UA3103389},
	{SG79483, &UA3103811},
	{SG79484, &UA106509},
	{SG79485, &UA119491},
	{SG79486, &UA134759},
	{SG79487, &UA143468},
	{SG79488, &UA152229},
	{SG79489, &UA153396},
	{SG79490, &UA155476},
	{SG79491, &UA162740},
	{SG79492, &UA167475},
	{SG79493, &UA295363},
	{SG79494, &UA298906},
	{SG79495, &UA302863},
	{SG79496, &UA306301},
	{SG79497, &UA433701},
	{SG79498, &UA446537},
	{SG79499, &UA475552},
	{SG79500, &UA490611},
	{SG79501, &UA496135},
	{SG79502, &UA504753},
	{SG79503, &UA508262},
	{SG79504, &UA529192},
	{SG79505, &UA532780},
	{SG79506, &UA562274},
	{SG79507, &UA627104},
	{SG79508, &UA653855},
	{SG79509, &UA655528},
	{SG79510, &UA774934},
	{SG79511, &UA809030},
	{SG79513, &UA816174},
	{SG79514, &UA1880344},
	{SG79515, &UA1888026},
	{SG79516, &UA1918069},
	{SG79517, &UA1946578},
	{SG79518, &UA1952751},
	{SG79519, &UA1958378},
	{SG79520, &UA2132844},
	{SG79522, &UA2143703},
	{SG79523, &UA2156085},
	{SG79525, &UA2206988},
	{SG79527, &UA2219207},
	{SG79528, &UA2275081},
	{SG79529, &UA2288377},
	{SG79530, &UA2292672},
	{SG79531, &UA2302171},
	{SG79533, &UA2303632},
	{SG79535, &UA2323418},
	{SG79536, &UA2337895},
	{SG79537, &UA2340918},
	{SG79538, &UA2555039},
	{SG79539, &UA2592852},
	{SG79540, &UA2594303},
	{SG79541, &UA2642135},
	{SG79542, &UA2725094},
	{SG79543, &UA2777830},
	{SG79544, &UA2809070},
	{SG79545, &UA2811917},
	{SG79546, &UA2831394},
	{SG79547, &UA2835362},
	{SG79548, &UA2837981},
	{SG79560, &UA92165},
	{SG79561, &UA92826},
	{SG79562, &UA94016},
	{SG79563, &UA94622},
	{SG79564, &UA110302},
	{SG79565, &UA124957},
	{SG79566, &UA134094},
	{SG79567, &UA270866},
	{SG79569, &UA305230},
	{SG79570, &UA386685},
	{SG79572, &UA416080},
	{SG79574, &UA442580},
	{SG79576, &UA446487},
	{SG79578, &UA446495},
	{SG79579, &UA468966},
	{SG79580, &UA504118},
	{SG79581, &UA509017},
	{SG79582, &UA512183},
	{SG79583, &UA517902},
	{SG79584, &UA522055},
	{SG79585, &UA524190},
	{SG79586, &UA529880},
	{SG79587, &UA534949},
	{SG79588, &UA563269},
	{SG79589, &UA753177},
	{SG79590, &UA767957},
	{SG79591, &UA814933},
	{SG79592, &UA816906},
	{SG79593, &UA1861409},
	{SG79594, &UA1864597},
	{SG79595, &UA1895378},
	{SG79596, &UA1906774},
	{SG79597, &UA1994267},
	{SG79598, &UA2131644},
	{SG79599, &UA2151935},
	{SG79600, &UA2157284},
	{SG79601, &UA2216559},
	{SG79602, &UA2262349},
	{SG79603, &UA2262348},
	{SG79604, &UA2274703},
	{SG79605, &UA2279125},
	{SG79606, &UA2284240},
	{SG79607, &UA2301478},
	{SG79608, &UA2305376},
	{SG79609, &UA2373754},
	{SG79610, &UA2379880},
	{SG79611, &UA2520704},
	{SG79612, &UA2548643},
	{SG79613, &UA2642107},
	{SG79614, &UA2652672},
	{SG79615, &UA2655050},
	{SG79616, &UA2694849},
	{SG79617, &UA2712142},
	{SG79618, &UA2722842},
	{SG79619, &UA2727337},
	{SG79620, &UA2747940},
	{SG79621, &UA2761647},
	{SG79622, &UA2763744},
	{SG79623, &UA2775709},
	{SG79624, &UA2786992},
	{SG79631, &UA2202662},
	{SG79632, &UA2353562},
	{SG79633, &UA2354139},
	{SG79634, &UA2354459},
	{SG79635, &UA2355359},
	{SG79636, &UA2355501},
	{SG79637, &UA2356404},
	{SG79638, &UA2356627},
	{SG79639, &UA2357289},
	{SG79640, &UA2357290},
	{SG79641, &UA2357306},
	{SG79642, &UA2357524},
	{SG79643, &UA2357538},
	{SG79644, &UA2357697},
	{SG79645, &UA2357915},
	{SG79646, &UA2357997},
	{SG79647, &UA2358000},
	{SG79648, &UA2358022},
	{SG79649, &UA2358207},
	{SG79650, &UA2358362},
	{SG79651, &UA2358382},
	{SG79652, &UA2358589},
	{SG79653, &UA2358831},
	{SG79654, &UA2359087},
	{SG79655, &UA2359089},
	{SG79656, &UA2359091},
	{SG79657, &UA2359096},
	{SG79658, &UA2359106},
	{SG79659, &UA2359529},
	{SG79660, &UA2360796},
	{SG79661, &UA2360825},
	{SG79662, &UA2360847},
	{SG79663, &UA2361026},
	{SG79664, &UA2361109},
	{SG79665, &UA2361129},
	{SG79666, &UA2361299},
	{SG79667, &UA2361322},
	{SG79668, &UA2361345},
	{SG79669, &UA2361376},
	{SG79670, &UA2361490},
	{SG79671, &UA2361558},
	{SG79672, &UA2361644},
	{SG79673, &UA2361752},
	{SG79674, &UA2361973},
	{SG79675, &UA2362093},
	{SG79676, &UA2367443},
	{SG79677, &UA2367576},
	{SG79678, &UA2368004},
	{SG79679, &UA2368218},
	{SG79680, &UA2368354},
	{SG79681, &UA2368476},
	{SG79682, &UA2368485},
	{SG79683, &UA2368530},
	{SG79684, &UA2369696},
	{SG79685, &UA2386648},
	{SG79686, &UA2386821},
	{SG79687, &UA2386857},
	{SG79688, &UA2386980},
	{SG79689, &UA2387039},
	{SG79690, &UA2403763},
	{SG79691, &UA2403827},
	{SG79692, &UA2404094},
	{SG79693, &UA2404242},
	{SG79694, &UA2424076},
	{SG79696, &UA2426609},
	{SG79698, &UA2437304},
	{SG79700, &UA2456403},
	{SG79701, &UA2456474},
	{SG79702, &UA2468438},
	{SG79703, &UA2484220},
	{SG79704, &UA2554240},
	{SG79705, &UA2596548},
	{SG79706, &UA2601198},
	{SG79707, &UA2654318},
	{SG79708, &UA2665091},
	{SG79709, &UA2695083},
	{SG79710, &UA2704657},
	{SG79711, &UA2708283},
	{SG79712, &UA2727399},
	{SG79713, &UA2749444},
	{SG79714, &UA2790372},
	{SG79715, &UA2791992},
	{SG79716, &UA2800516},
	{SG79717, &UA2840853},
	{SG79718, &UA2841407},
	{SG79719, &UA2841473},
	{SG79720, &UA2852613},
	{SG79721, &UA2934610},
	{SG79722, &UA2991627},
	{SG79723, &UA2991907},
	{SG79740, &UA2387800},
	{SG79741, &UA2424674},
	{SG79742, &UA2425840},
	{SG79743, &UA2426925},
	{SG79744, &UA2426940},
	{SG79746, &UA2426941},
	{SG79748, &UA2426938},
	{SG79749, &UA2427077},
	{SG79750, &UA2427082},
	{SG79751, &UA2427083},
	{SG79752, &UA2427088},
	{SG79753, &UA2427094},
	{SG79754, &UA2427104},
	{SG79755, &UA2427084},
	{SG79756, &UA2427470},
	{SG79757, &UA2427609},
	{SG79758, &UA2428164},
	{SG79759, &UA2428178},
	{SG79760, &UA2428517},
	{SG79761, &UA2430713},
	{SG79762, &UA2431062},
	{SG79763, &UA2431101},
	{SG79764, &UA2432811},
	{SG79765, &UA2433717},
	{SG79766, &UA2433784},
	{SG79767, &UA2436443},
	{SG79768, &UA2436470},
	{SG79769, &UA2437340},
	{SG79770, &UA2437348},
	{SG79771, &UA2437403},
	{SG79772, &UA2437420},
	{SG79773, &UA2437398},
	{SG79774, &UA2437436},
	{SG79775, &UA2437438},
	{SG79776, &UA2437442},
	{SG79777, &UA2437489},
	{SG79778, &UA2437492},
	{SG79779, &UA2437506},
	{SG79780, &UA2437507},
	{SG79781, &UA2437514},
	{SG79782, &UA2437519},
	{SG79783, &UA2437528},
	{SG79784, &UA2437558},
	{SG79785, &UA2437563},
	{SG79786, &UA2437603},
	{SG79788, &UA2437654},
	{SG79790, &UA2437664},
	{SG79792, &UA2437669},
	{SG79794, &UA2437670},
	{SG79795, &UA2437674},
	{SG79796, &UA2437684},
	{SG79797, &UA2437689},
	{SG79798, &UA2437666},
	{SG79799, &UA2459847},
	{SG79800, &UA2460590},
	{SG79801, &UA2460593},
	{SG79802, &UA2460601},
	{SG79803, &UA2460720},
	{SG79804, &UA2460857},
	{SG79805, &UA2460892},
	{SG79806, &UA2460899},
	{SG79807, &UA2460966},
	{SG79808, &UA2461025},
	{SG79810, &UA2461045},
	{SG79812, &UA2467356},
	{SG79813, &UA2467510},
	{SG79814, &UA2468366},
	{SG79815, &UA2468515},
	{SG79816, &UA2468995},
	{SG79817, &UA2469175},
	{SG79818, &UA2469179},
	{SG79819, &UA2469181},
	{SG79820, &UA2469932},
	{SG79821, &UA2483698},
	{SG79822, &UA2484139},
	{SG79823, &UA2484213},
	{SG79824, &UA2484223},
	{SG79825, &UA2485099},
	{SG79826, &UA2485550},
	{SG79827, &UA2486168},
	{SG79828, &UA2486814},
	{SG79829, &UA2486875},
	{SG79830, &UA2487214},
	{SG79831, &UA2490628},
	{SG79836, &UA2506177},
	{SG79837, &UA2506426},
	{SG79838, &UA2506677},
	{SG79839, &UA2506694},
	{SG79840, &UA2507461},
	{SG79841, &UA2511652},
	{SG79842, &UA2513000},
	{SG79843, &UA2529387},
	{SG79844, &UA2529424},
	{SG79845, &UA2534069},
	{SG79846, &UA2534834},
	{SG79847, &UA2537481},
	{SG79848, &UA2538006},
	{SG79849, &UA2538528},
	{SG79850, &UA2539513},
	{SG79851, &UA2557586},
	{SG79852, &UA2558517},
	{SG79853, &UA2565456},
	{SG79854, &UA2565762},
	{SG79855, &UA2569360},
	{SG79856, &UA2579147},
	{SG79857, &UA2583071},
	{SG79858, &UA2595913},
	{SG79859, &UA2596830},
	{SG79860, &UA2603949},
	{SG79861, &UA2669107},
	{SG79862, &UA2699768},
	{SG79863, &UA2707628},
	{SG79864, &UA2709253},
	{SG79865, &UA2728813},
	{SG79866, &UA2733749},
	{SG79867, &UA2747783},
	{SG79868, &UA2754603},
	{SG79869, &UA2764235},
	{SG79870, &UA2791482},
	{SG79871, &UA2846524},
	{SG79872, &UA2846560},
	{SG79873, &UA2847119},
	{SG79874, &UA2937007},
	{SG79875, &UA2955873},
	{SG79876, &UA2963700},
	{SG79877, &UA2984464},
	{SG79878, &UA3020241},
	{SG79879, &UA3058044},
	{SG79880, &UA3079132},
	{SG79881, &UA3102954},
	{SG79882, &UA3124875},
	{SG79920, &UA96734},
	{SG79921, &UA97845},
	{SG79922, &UA100011},
	{SG79923, &UA101072},
	{SG79924, &UA103409},
	{SG79925, &UA103576},
	{SG79926, &UA145867},
	{SG79927, &UA146340},
	{SG79928, &UA188724},
	{SG79929, &UA200780},
	{SG79930, &UA234757},
	{SG79931, &UA274033},
	{SG79932, &UA536306},
	{SG79933, &UA826642},
	{SG79934, &UA1899283},
	{SG79935, &UA2131929},
	{SG79937, &UA2275311},
	{SG79938, &UA2310552},
	{SG79956, &UA2433593},
	{SG79977, &UA440917},
	{SG79978, &UA453569},
	{SG79979, &UA453573},
	{SG79980, &UA453705},
	{SG79981, &UA453736},
	{SG79982, &UA453883},
	{SG79983, &UA453900},
	{SG79984, &UA453910},
	{SG79985, &UA454328},
	{SG79986, &UA454821},
	{SG79987, &UA454883},
	{SG79988, &UA603120},
	{SG79989, &UA603126},
	{SG79990, &UA603219},
	{SG79991, &UA603331},
	{SG79992, &UA603370},
	{SG79994, &UA603528},
	{SG79995, &UA603967},
	{SG79996, &UA604229},
	{SG79997, &UA607124},
	{SG79998, &UA623588},
	{SG79999, &UA628810},
	{SG80000, &UA630229},
	{SG80001, &UA630840},
	{SG80002, &UA630896},
	{SG80003, &UA632525},
	{SG80004, &UA634763},
	{SG80005, &UA635603},
	{SG80006, &UA649991},
	{SG80007, &UA656703},
	{SG80008, &UA657140},
	{SG80009, &UA661778},
	{SG80010, &UA663717},
	{SG80011, &UA671924},
	{SG80012, &UA676314},
	{SG80013, &UA690785},
	{SG80014, &UA696447},
	{SG80015, &UA731738},
	{SG80016, &UA739866},
	{SG80017, &UA748401},
	{SG80018, &UA786691},
	{SG80019, &UA787868},
	{SG80020, &UA789257},
	{SG80021, &UA819794},
	{SG80022, &UA856051},
	{SG80023, &UA896005},
	{SG80024, &UA901768},
	{SG80025, &UA1954167},
	{SG80027, &UA1985295},
	{SG80029, &UA1987398},
	{SG80031, &UA1987719},
	{SG80032, &UA1989075},
	{SG80033, &UA1990266},
	{SG80034, &UA1990365},
	{SG80035, &UA2159077},
	{SG80037, &UA2164650},
	{SG80038, &UA2168077},
	{SG80039, &UA2177167},
	{SG80040, &UA2178684},
	{SG80041, &UA2333578},
	{SG80042, &UA2333788},
	{SG80043, &UA2369164},
	{SG80044, &UA2381773},
	{SG80045, &UA2402620},
	{SG80046, &UA2424045},
	{SG80047, &UA2456357},
	{SG80048, &UA2485354},
	{SG80049, &UA2554012},
	{SG80050, &UA2579990},
	{SG80051, &UA2588961},
	{SG80052, &UA2653681},
	{SG80053, &UA2667124},
	{SG80054, &UA2745931},
	{SG80055, &UA2860917},
	{SG80056, &UA2958141},
	{SG80063, &UA363918},
	{SG80064, &UA380071},
	{SG80065, &UA380835},
	{SG80066, &UA380986},
	{SG80067, &UA381096},
	{SG80068, &UA381097},
	{SG80069, &UA382148},
	{SG80070, &UA398590},
	{SG80071, &UA398610},
	{SG80072, &UA402564},
	{SG80073, &UA402570},
	{SG80074, &UA402567},
	{SG80075, &UA422198},
	{SG80076, &UA422238},
	{SG80077, &UA423758},
	{SG80078, &UA437311},
	{SG80079, &UA458458},
	{SG80080, &UA587638},
	{SG80081, &UA587859},
	{SG80082, &UA587924},
	{SG80083, &UA588031},
	{SG80085, &UA588099},
	{SG80087, &UA603755},
	{SG80088, &UA608797},
	{SG80089, &UA625253},
	{SG80090, &UA636113},
	{SG80091, &UA642178},
	{SG80092, &UA645615},
	{SG80093, &UA647744},
	{SG80094, &UA651832},
	{SG80095, &UA653628},
	{SG80096, &UA656165},
	{SG80097, &UA661976},
	{SG80098, &UA682907},
	{SG80099, &UA688327},
	{SG80100, &UA691929},
	{SG80101, &UA708928},
	{SG80102, &UA715463},
	{SG80103, &UA715970},
	{SG80104, &UA716130},
	{SG80105, &UA718386},
	{SG80106, &UA726974},
	{SG80107, &UA762276},
	{SG80108, &UA764506},
	{SG80109, &UA813480},
	{SG80110, &UA813482},
	{SG80111, &UA895876},
	{SG80112, &UA902351},
	{SG80113, &UA912351},
	{SG80114, &UA1949194},
	{SG80115, &UA1958244},
	{SG80116, &UA1969906},
	{SG80117, &UA1983972},
	{SG80118, &UA1989791},
	{SG80119, &UA1995241},
	{SG80120, &UA2164042},
	{SG80121, &UA2177294},
	{SG80122, &UA2186095},
	{SG80123, &UA2206003},
	{SG80124, &UA2262795},
	{SG80125, &UA2324946},
	{SG80126, &UA2367446},
	{SG80127, &UA2385145},
	{SG80128, &UA2386033},
	{SG80129, &UA2405109},
	{SG80130, &UA2410034},
	{SG80131, &UA2455893},
	{SG80132, &UA2460429},
	{SG80133, &UA2501501},
	{SG80134, &UA2596937},
	{SG80135, &UA2600100},
	{SG80136, &UA2666528},
	{SG80137, &UA2673296},
	{SG80138, &UA2686081},
	{SG80139, &UA2695400},
	{SG80140, &UA2712905},
	{SG80141, &UA2727165},
	{SG80142, &UA2736848},
	{SG80144, &UA2747166},
	{SG80146, &UA2748093},
	{SG80147, &UA2748495},
	{SG80149, &UA2749367},
	{SG80151, &UA2784445},
	{SG80153, &UA2848404},
	{SG80154, &UA2857845},
	{SG80155, &UA2864153},
	{SG80156, &UA2934804},
	{SG80157, &UA3080023},
	{SG80158, &UA3104214},
	{SG80159, &UA3122184},
	{SG80167, &UA2018034},
	{SG80168, &UA2041196},
	{SG80169, &UA2184133},
	{SG80170, &UA2186893},
	{SG80171, &UA2192618},
	{SG80172, &UA2193414},
	{SG80173, &UA2196265},
	{SG80174, &UA2196702},
	{SG80175, &UA2197879},
	{SG80176, &UA2197877},
	{SG80177, &UA2199278},
	{SG80178, &UA2200587},
	{SG80179, &UA2201559},
	{SG80180, &UA2201680},
	{SG80181, &UA2205984},
	{SG80182, &UA2226835},
	{SG80183, &UA2230471},
	{SG80184, &UA2262560},
	{SG80185, &UA2262588},
	{SG80186, &UA2262619},
	{SG80187, &UA2342394},
	{SG80188, &UA2349884},
	{SG80189, &UA2355364},
	{SG80190, &UA2362640},
	{SG80191, &UA2369295},
	{SG80192, &UA2415274},
	{SG80193, &UA2456248},
	{SG80194, &UA2526481},
	{SG80195, &UA2555138},
	{SG80196, &UA2592915},
	{SG80197, &UA2693481},
	{SG80198, &UA2729238},
	{SG80199, &UA2744841},
	{SG80200, &UA2749957},
	{SG80201, &UA2820410},
	{SG80202, &UA2822074},
	{SG80203, &UA2824684},
	{SG80204, &UA2831257},
	{SG80205, &UA2981279},
	{SG80206, &UA3070526},
	{SG80223, &UA161957},
	{SG80224, &UA164985},
	{SG80225, &UA165324},
	{SG80226, &UA166167},
	{SG80227, &UA166285},
	{SG80228, &UA169550},
	{SG80229, &UA170385},
	{SG80230, &UA171296},
	{SG80231, &UA171428},
	{SG80232, &UA232513},
	{SG80233, &UA296671},
	{SG80234, &UA298371},
	{SG80235, &UA301217},
	{SG80236, &UA301220},
	{SG80237, &UA302059},
	{SG80238, &UA302156},
	{SG80239, &UA303877},
	{SG80240, &UA304452},
	{SG80241, &UA304497},
	{SG80242, &UA329510},
	{SG80243, &UA338075},
	{SG80244, &UA374109},
	{SG80245, &UA380650},
	{SG80246, &UA394012},
	{SG80247, &UA396886},
	{SG80248, &UA396924},
	{SG80249, &UA409265},
	{SG80250, &UA432778},
	{SG80251, &UA435467},
	{SG80252, &UA444603},
	{SG80253, &UA444667},
	{SG80254, &UA515289},
	{SG80255, &UA518459},
	{SG80256, &UA519153},
	{SG80257, &UA532786},
	{SG80258, &UA536601},
	{SG80259, &UA541662},
	{SG80260, &UA546650},
	{SG80261, &UA558359},
	{SG80262, &UA607483},
	{SG80263, &UA609375},
	{SG80264, &UA632595},
	{SG80265, &UA649935},
	{SG80266, &UA694817},
	{SG80267, &UA741888},
	{SG80268, &UA750800},
	{SG80269, &UA759109},
	{SG80270, &UA794018},
	{SG80271, &UA794491},
	{SG80272, &UA820048},
	{SG80273, &UA837635},
	{SG80274, &UA868981},
	{SG80275, &UA869078},
	{SG80276, &UA896818},
	{SG80277, &UA1857147},
	{SG80278, &UA1857398},
	{SG80279, &UA1864393},
	{SG80280, &UA1905394},
	{SG80281, &UA1956806},
	{SG80282, &UA1960182},
	{SG80283, &UA1962134},
	{SG80284, &UA1963895},
	{SG80285, &UA1989050},
	{SG80286, &UA2146816},
	{SG80287, &UA2161418},
	{SG80288, &UA2163152},
	{SG80289, &UA2167540},
	{SG80290, &UA2167915},
	{SG80291, &UA2184855},
	{SG80292, &UA2215499},
	{SG80293, &UA2219740},
	{SG80295, &UA2220259},
	{SG80296, &UA2230449},
	{SG80298, &UA2262421},
	{SG80300, &UA2262507},
	{SG80301, &UA2273095},
	{SG80303, &UA2332514},
	{SG80304, &UA2333296},
	{SG80306, &UA2338120},
	{SG80307, &UA2403029},
	{SG80309, &UA2417190},
	{SG80311, &UA2441574},
	{SG80313, &UA2443534},
	{SG80315, &UA2454119},
	{SG80317, &UA2466938},
	{SG80319, &UA2480276},
	{SG80321, &UA2518297},
	{SG80322, &UA2528168},
	{SG80324, &UA2554007},
	{SG80325, &UA2557657},
	{SG80326, &UA2574097},
	{SG80327, &UA2653272},
	{SG80328, &UA2702827},
	{SG80329, &UA2732779},
	{SG80330, &UA2809881},
	{SG80331, &UA2844011},
	{SG80332, &UA2846735},
	{SG80333, &UA2993734},
	{SG80334, &UA3121679},
	{SG80339, &UA2585284},
	{SG80340, &UA2607172},
	{SG80342, &UA2672085},
	{SG80343, &UA2672311},
	{SG80344, &UA2672616},
	{SG80345, &UA2672976},
	{SG80346, &UA2674750},
	{SG80347, &UA2674973},
	{SG80348, &UA2675259},
	{SG80349, &UA2675908},
	{SG80350, &UA2676636},
	{SG80351, &UA2676767},
	{SG80352, &UA2677035},
	{SG80353, &UA2677329},
	{SG80354, &UA2707731},
	{SG80355, &UA2707732},
	{SG80356, &UA2708797},
	{SG80357, &UA2708864},
	{SG80358, &UA2709031},
	{SG80359, &UA2710638},
	{SG80360, &UA2713512},
	{SG80361, &UA2714300},
	{SG80363, &UA2714680},
	{SG80365, &UA2717008},
	{SG80367, &UA2734784},
	{SG80368, &UA2735299},
	{SG80369, &UA2735472},
	{SG80370, &UA2735524},
	{SG80371, &UA2736400},
	{SG80372, &UA2736593},
	{SG80373, &UA2751103},
	{SG80374, &UA2751202},
	{SG80375, &UA2751678},
	{SG80376, &UA2752631},
	{SG80377, &UA2753359},
	{SG80378, &UA2769307},
	{SG80379, &UA2771849},
	{SG80380, &UA2794830},
	{SG80381, &UA2798859},
	{SG80382, &UA2799415},
	{SG80383, &UA2860942},
	{SG80384, &UA2861051},
	{SG80385, &UA2861172},
	{SG80386, &UA2863165},
	{SG80387, &UA2869590},
	{SG80388, &UA2875387},
	{SG80389, &UA2878792},
	{SG80390, &UA2880208},
	{SG80391, &UA2883398},
	{SG80392, &UA2891730},
	{SG80393, &UA2893721},
	{SG80394, &UA2895369},
	{SG80395, &UA2895522},
	{SG80396, &UA2898788},
	{SG80397, &UA2901077},
	{SG80398, &UA2957572},
	{SG80399, &UA2959167},
	{SG80400, &UA3018452},
	{SG80401, &UA3020885},
	{SG80402, &UA3057466},
	{SG80403, &UA3059489},
	{SG80404, &UA3072933},
	{SG80501, &UA2472026},
	{SG80503, &UA2490178},
	{SG80505, &UA2491896},
	{SG80506, &UA2491899},
	{SG80507, &UA2491904},
	{SG80508, &UA2491908},
	{SG80509, &UA2491897},
	{SG80510, &UA2492123},
	{SG80511, &UA2492129},
	{SG80512, &UA2492131},
	{SG80513, &UA2492132},
	{SG80514, &UA2492133},
	{SG80515, &UA2492139},
	{SG80516, &UA2492178},
	{SG80517, &UA2492196},
	{SG80518, &UA2492197},
	{SG80519, &UA2492360},
	{SG80520, &UA2492445},
	{SG80521, &UA2508043},
	{SG80522, &UA2508097},
	{SG80523, &UA2508250},
	{SG80524, &UA2508769},
	{SG80525, &UA2508853},
	{SG80526, &UA2508873},
	{SG80527, &UA2508919},
	{SG80528, &UA2509806},
	{SG80529, &UA2510449},
	{SG80530, &UA2510518},
	{SG80531, &UA2510934},
	{SG80532, &UA2511029},
	{SG80533, &UA2511408},
	{SG80534, &UA2511414},
	{SG80535, &UA2511996},
	{SG80536, &UA2512012},
	{SG80537, &UA2512839},
	{SG80538, &UA2512982},
	{SG80539, &UA2515320},
	{SG80540, &UA2530451},
	{SG80541, &UA2531908},
	{SG80542, &UA2536574},
	{SG80543, &UA2537582},
	{SG80544, &UA2537729},
	{SG80545, &UA2538107},
	{SG80546, &UA2559261},
	{SG80547, &UA2563795},
	{SG80548, &UA2564606},
	{SG80549, &UA2566035},
	{SG80550, &UA2569760},
	{SG80551, &UA2580527},
	{SG80552, &UA2581942},
	{SG80553, &UA2599752},
	{SG80554, &UA2660147},
	{SG80555, &UA2662037},
	{SG80556, &UA2665805},
	{SG80557, &UA2667436},
	{SG80558, &UA2699105},
	{SG80559, &UA2699840},
	{SG80560, &UA2701799},
	{SG80561, &UA2702192},
	{SG80562, &UA2702671},
	{SG80563, &UA2702729},
	{SG80564, &UA2703149},
	{SG80565, &UA2703473},
	{SG80566, &UA2705535},
	{SG80567, &UA2730548},
	{SG80568, &UA2767553},
	{SG80570, &UA2780785},
	{SG80572, &UA2783525},
	{SG80573, &UA2850502},
	{SG80574, &UA2851108},
	{SG80575, &UA2851550},
	{SG80576, &UA2861786},
	{SG80577, &UA2952304},
	{SG80589, &UA191298},
	{SG80590, &UA191429},
	{SG80591, &UA191938},
	{SG80592, &UA196407},
	{SG80593, &UA197113},
	{SG80594, &UA202415},
	{SG80595, &UA310272},
	{SG80596, &UA310394},
	{SG80597, &UA310632},
	{SG80598, &UA311111},
	{SG80599, &UA311130},
	{SG80601, &UA311491},
	{SG80603, &UA311603},
	{SG80605, &UA312870},
	{SG80607, &UA315942},
	{SG80608, &UA316265},
	{SG80610, &UA316389},
	{SG80612, &UA319059},
	{SG80614, &UA319098},
	{SG80616, &UA319419},
	{SG80617, &UA319504},
	{SG80618, &UA320126},
	{SG80619, &UA379702},
	{SG80620, &UA382094},
	{SG80621, &UA419567},
	{SG80622, &UA542763},
	{SG80623, &UA543359},
	{SG80624, &UA552621},
	{SG80625, &UA562521},
	{SG80626, &UA569091},
	{SG80627, &UA582589},
	{SG80628, &UA652590},
	{SG80629, &UA685004},
	{SG80630, &UA694807},
	{SG80631, &UA712626},
	{SG80632, &UA725017},
	{SG80633, &UA730701},
	{SG80634, &UA759349},
	{SG80635, &UA781764},
	{SG80636, &UA791628},
	{SG80637, &UA869396},
	{SG80638, &UA899968},
	{SG80639, &UA1974821},
	{SG80640, &UA1977825},
	{SG80641, &UA1994936},
	{SG80642, &UA2137573},
	{SG80644, &UA2154909},
	{SG80646, &UA2262773},
	{SG80647, &UA2285249},
	{SG80649, &UA2314345},
	{SG80650, &UA2316428},
	{SG80651, &UA2328808},
	{SG80652, &UA2346335},
	{SG80653, &UA2348847},
	{SG80654, &UA2369581},
	{SG80655, &UA2375778},
	{SG80656, &UA2381887},
	{SG80657, &UA2454263},
	{SG80658, &UA2454737},
	{SG80659, &UA2456511},
	{SG80660, &UA2463115},
	{SG80661, &UA2469166},
	{SG80662, &UA2494640},
	{SG80663, &UA2505215},
	{SG80664, &UA2553338},
	{SG80665, &UA2553612},
	{SG80666, &UA2554164},
	{SG80668, &UA2554569},
	{SG80669, &UA2573913},
	{SG80670, &UA2685808},
	{SG80671, &UA2733232},
	{SG80673, &UA2759217},
	{SG80674, &UA2780012},
	{SG80675, &UA2813001},
	{SG80677, &UA2949649},
	{SG80679, &UA3044342},
	{SG80691, &UA35443},
	{SG80692, &UA92924},
	{SG80693, &UA95069},
	{SG80694, &UA101047},
	{SG80695, &UA119714},
	{SG80696, &UA131416},
	{SG80697, &UA238580},
	{SG80698, &UA442459},
	{SG80699, &UA445647},
	{SG80700, &UA476142},
	{SG80701, &UA627323},
	{SG80702, &UA2212265},
	{SG80703, &UA2801332},
	{SG80704, &UA1995944},
	{SG80705, &UA1996402},
	{SG80707, &UA2391203},
	{SG80709, &UA2714735},
	{SG80710, &UA3052405},
	{SG80730, &UA251635},
	{SG80731, &UA251642},
	{SG80732, &UA251650},
	{SG80733, &UA251639},
	{SG80734, &UA251751},
	{SG80735, &UA251790},
	{SG80736, &UA251801},
	{SG80737, &UA251805},
	{SG80738, &UA251834},
	{SG80739, &UA251836},
	{SG80740, &UA251842},
	{SG80741, &UA251860},
	{SG80742, &UA251874},
	{SG80743, &UA251880},
	{SG80744, &UA251890},
	{SG80745, &UA251897},
	{SG80746, &UA252000},
	{SG80747, &UA344956},
	{SG80748, &UA345838},
	{SG80750, &UA421920},
	{SG80752, &UA437105},
	{SG80753, &UA451330},
	{SG80754, &UA574808},
	{SG80755, &UA591703},
	{SG80756, &UA603066},
	{SG80757, &UA649869},
	{SG80758, &UA703514},
	{SG80759, &UA731077},
	{SG80760, &UA743411},
	{SG80761, &UA771905},
	{SG80762, &UA783652},
	{SG80763, &UA849386},
	{SG80764, &UA882744},
	{SG80765, &UA894446},
	{SG80766, &UA1902975},
	{SG80767, &UA1903046},
	{SG80768, &UA1954905},
	{SG80769, &UA1960692},
	{SG80770, &UA1977161},
	{SG80771, &UA1981527},
	{SG80772, &UA2155405},
	{SG80773, &UA2168081},
	{SG80774, &UA2190655},
	{SG80775, &UA2226660},
	{SG80776, &UA2299943},
	{SG80777, &UA2308824},
	{SG80778, &UA2315900},
	{SG80779, &UA2332961},
	{SG80780, &UA2352409},
	{SG80781, &UA2467249},
	{SG80782, &UA2481015},
	{SG80783, &UA2481332},
	{SG80784, &UA2486461},
	{SG80785, &UA2496386},
	{SG80786, &UA2654224},
	{SG80787, &UA2667283},
	{SG80788, &UA2703486},
	{SG80789, &UA2786863},
	{SG80790, &UA2826294},
	{SG80791, &UA2867044},
	{SG80792, &UA2936485},
	{SG80793, &UA912274},
	{SG80830, &UA363926},
	{SG80831, &UA368124},
	{SG80832, &UA380709},
	{SG80833, &UA380764},
	{SG80834, &UA380834},
	{SG80835, &UA380837},
	{SG80836, &UA381037},
	{SG80837, &UA381095},
	{SG80838, &UA381142},
	{SG80839, &UA382089},
	{SG80840, &UA382150},
	{SG80841, &UA402571},
	{SG80842, &UA440927},
	{SG80843, &UA587680},
	{SG80844, &UA587918},
	{SG80845, &UA588209},
	{SG80846, &UA588567},
	{SG80847, &UA588657},
	{SG80848, &UA589377},
	{SG80850, &UA590023},
	{SG80851, &UA600098},
	{SG80852, &UA607428},
	{SG80853, &UA626876},
	{SG80854, &UA628776},
	{SG80855, &UA630441},
	{SG80856, &UA661644},
	{SG80857, &UA680603},
	{SG80858, &UA681239},
	{SG80859, &UA683846},
	{SG80861, &UA684861},
	{SG80862, &UA688060},
	{SG80864, &UA688481},
	{SG80865, &UA693161},
	{SG80866, &UA696400},
	{SG80867, &UA697446},
	{SG80868, &UA704422},
	{SG80869, &UA705313},
	{SG80870, &UA709799},
	{SG80871, &UA715444},
	{SG80872, &UA727556},
	{SG80873, &UA727591},
	{SG80874, &UA731455},
	{SG80875, &UA736420},
	{SG80876, &UA739376},
	{SG80877, &UA746366},
	{SG80878, &UA770773},
	{SG80879, &UA790501},
	{SG80881, &UA805712},
	{SG80882, &UA853413},
	{SG80883, &UA862095},
	{SG80884, &UA869558},
	{SG80885, &UA901849},
	{SG80886, &UA1950889},
	{SG80887, &UA1951441},
	{SG80888, &UA1952176},
	{SG80889, &UA1953425},
	{SG80890, &UA1956013},
	{SG80891, &UA1957819},
	{SG80892, &UA1964774},
	{SG80893, &UA1965515},
	{SG80894, &UA1975958},
	{SG80895, &UA1995263},
	{SG80896, &UA2153238},
	{SG80897, &UA2174743},
	{SG80898, &UA2186173},
	{SG80899, &UA2235281},
	{SG80900, &UA2323268},
	{SG80901, &UA2333678},
	{SG80902, &UA2386689},
	{SG80903, &UA2403924},
	{SG80904, &UA2414030},
	{SG80905, &UA2422612},
	{SG80906, &UA2441526},
	{SG80907, &UA2456103},
	{SG80908, &UA2468065},
	{SG80909, &UA2600812},
	{SG80910, &UA2717787},
	{SG80911, &UA2777674},
	{SG80912, &UA2842921},
	{SG80913, &UA2844982},
	{SG80914, &UA2854866},
	{SG80915, &UA2857317},
	{SG80916, &UA2861195},
	{SG80918, &UA2868350},
	{SG80920, &UA2991077},
	{SG80944, &UA238806},
	{SG80945, &UA242512},
	{SG80946, &UA242846},
	{SG80947, &UA242851},
	{SG80948, &UA242952},
	{SG80949, &UA243252},
	{SG80950, &UA243741},
	{SG80951, &UA243748},
	{SG80952, &UA243911},
	{SG80953, &UA243963},
	{SG80954, &UA244128},
	{SG80955, &UA244155},
	{SG80956, &UA248359},
	{SG80957, &UA250051},
	{SG80958, &UA336403},
	{SG80959, &UA336554},
	{SG80960, &UA336578},
	{SG80961, &UA336647},
	{SG80962, &UA337596},
	{SG80963, &UA338060},
	{SG80964, &UA338172},
	{SG80965, &UA338192},
	{SG80966, &UA342120},
	{SG80967, &UA344211},
	{SG80968, &UA344512},
	{SG80969, &UA567833},
	{SG80970, &UA608713},
	{SG80971, &UA665744},
	{SG80972, &UA852351},
	{SG80973, &UA868563},
	{SG80974, &UA1962889},
	{SG80975, &UA1964173},
	{SG80976, &UA2166968},
	{SG80977, &UA2229357},
	{SG80978, &UA2262786},
	{SG80979, &UA2262806},
	{SG80980, &UA2325292},
	{SG80981, &UA2332895},
	{SG80982, &UA2353349},
	{SG80983, &UA2393355},
	{SG80984, &UA2411540},
	{SG80985, &UA2482157},
	{SG80986, &UA2529722},
	{SG80987, &UA2652925},
	{SG80988, &UA2694955},
	{SG80989, &UA2709197},
	{SG80990, &UA2727228},
	{SG80991, &UA2839103},
	{SG80992, &UA2869820},
	{SG80993, &UA2994176},
	{SG81006, &UA453581},
	{SG81007, &UA453693},
	{SG81008, &UA453698},
	{SG81009, &UA453720},
	{SG81010, &UA453740},
	{SG81011, &UA453752},
	{SG81012, &UA453819},
	{SG81013, &UA453916},
	{SG81014, &UA453920},
	{SG81015, &UA453849},
	{SG81016, &UA453961},
	{SG81017, &UA453965},
	{SG81018, &UA454011},
	{SG81019, &UA453921},
	{SG81020, &UA454718},
	{SG81021, &UA454934},
	{SG81022, &UA458076},
	{SG81023, &UA603078},
	{SG81024, &UA603148},
	{SG81025, &UA603340},
	{SG81026, &UA604841},
	{SG81027, &UA605866},
	{SG81028, &UA614872},
	{SG81029, &UA623013},
	{SG81030, &UA623598},
	{SG81031, &UA623653},
	{SG81032, &UA624234},
	{SG81033, &UA625505},
	{SG81034, &UA625762},
	{SG81035, &UA626411},
	{SG81036, &UA626987},
	{SG81037, &UA630008},
	{SG81038, &UA631745},
	{SG81039, &UA631999},
	{SG81040, &UA633229},
	{SG81041, &UA638499},
	{SG81042, &UA642781},
	{SG81043, &UA654317},
	{SG81044, &UA658346},
	{SG81045, &UA670978},
	{SG81046, &UA672798},
	{SG81047, &UA674876},
	{SG81048, &UA675914},
	{SG81049, &UA679870},
	{SG81050, &UA688869},
	{SG81051, &UA699437},
	{SG81052, &UA766714},
	{SG81053, &UA775158},
	{SG81054, &UA778507},
	{SG81055, &UA780385},
	{SG81056, &UA822277},
	{SG81057, &UA888052},
	{SG81058, &UA901588},
	{SG81059, &UA906591},
	{SG81060, &UA1959515},
	{SG81061, &UA1970520},
	{SG81062, &UA1975571},
	{SG81063, &UA1984835},
	{SG81064, &UA1990748},
	{SG81065, &UA1991149},
	{SG81066, &UA1996329},
	{SG81067, &UA2184217},
	{SG81068, &UA2262504},
	{SG81070, &UA2262519},
	{SG81071, &UA2315337},
	{SG81072, &UA2332908},
	{SG81073, &UA2402810},
	{SG81074, &UA2412154},
	{SG81076, &UA2443878},
	{SG81077, &UA2456323},
	{SG81078, &UA2459142},
	{SG81079, &UA2482059},
	{SG81081, &UA2506048},
	{SG81083, &UA2594330},
	{SG81084, &UA2654316},
	{SG81085, &UA2706127},
	{SG81086, &UA2957711},
	{SG81087, &UA3043069},
	{SG81088, &UA3043512},
	{SG81089, &UA197459},
	{SG81090, &UA398789},
	{SG81112, &UA2509686},
	{SG81113, &UA2510173},
	{SG81114, &UA2512032},
	{SG81115, &UA2512803},
	{SG81116, &UA2512804},
	{SG81117, &UA2514393},
	{SG81118, &UA2531413},
	{SG81119, &UA2532325},
	{SG81121, &UA2535671},
	{SG81123, &UA2536759},
	{SG81124, &UA2537473},
	{SG81125, &UA2537641},
	{SG81126, &UA2537759},
	{SG81127, &UA2537771},
	{SG81128, &UA2561250},
	{SG81129, &UA2561928},
	{SG81131, &UA2563486},
	{SG81132, &UA2564613},
	{SG81133, &UA2564705},
	{SG81134, &UA2571427},
	{SG81135, &UA2572186},
	{SG81136, &UA2580591},
	{SG81137, &UA2580596},
	{SG81138, &UA2580614},
	{SG81139, &UA2581909},
	{SG81140, &UA2582632},
	{SG81141, &UA2597851},
	{SG81142, &UA2599300},
	{SG81143, &UA2599730},
	{SG81144, &UA2599967},
	{SG81145, &UA2600481},
	{SG81146, &UA2602402},
	{SG81147, &UA2603108},
	{SG81148, &UA2603706},
	{SG81149, &UA2661102},
	{SG81150, &UA2661330},
	{SG81151, &UA2661410},
	{SG81152, &UA2663098},
	{SG81154, &UA2663992},
	{SG81156, &UA2664975},
	{SG81158, &UA2672288},
	{SG81160, &UA2700874},
	{SG81162, &UA2708201},
	{SG81163, &UA2730115},
	{SG81164, &UA2730405},
	{SG81165, &UA2730936},
	{SG81166, &UA2749009},
	{SG81167, &UA2858582},
	{SG81168, &UA2995024},
	{SG81169, &UA2998681},
	{SG81185, &UA238581},
	{SG81186, &UA242476},
	{SG81187, &UA242828},
	{SG81188, &UA242847},
	{SG81189, &UA242848},
	{SG81190, &UA242998},
	{SG81191, &UA243133},
	{SG81193, &UA243698},
	{SG81195, &UA243702},
	{SG81197, &UA243917},
	{SG81198, &UA243925},
	{SG81199, &UA243936},
	{SG81200, &UA243968},
	{SG81201, &UA243993},
	{SG81203, &UA244627},
	{SG81205, &UA244630},
	{SG81207, &UA248777},
	{SG81208, &UA250297},
	{SG81209, &UA336648},
	{SG81210, &UA336671},
	{SG81211, &UA336817},
	{SG81212, &UA337236},
	{SG81213, &UA337235},
	{SG81214, &UA337588},
	{SG81215, &UA342229},
	{SG81216, &UA379733},
	{SG81217, &UA421374},
	{SG81218, &UA580517},
	{SG81219, &UA608986},
	{SG81220, &UA665995},
	{SG81221, &UA760242},
	{SG81222, &UA769408},
	{SG81223, &UA1898143},
	{SG81225, &UA1898952},
	{SG81226, &UA1977848},
	{SG81227, &UA2154941},
	{SG81228, &UA2156398},
	{SG81229, &UA2298205},
	{SG81230, &UA2307787},
	{SG81231, &UA2312978},
	{SG81232, &UA2375244},
	{SG81233, &UA2380853},
	{SG81234, &UA2381063},
	{SG81238, &UA2466354},
	{SG81239, &UA2551254},
	{SG81240, &UA2573470},
	{SG81241, &UA2656570},
	{SG81242, &UA2758563},
	{SG81243, &UA2790398},
	{SG81244, &UA2843093},
	{SG81245, &UA2858081},
	{SG81246, &UA347237},
	{SG81247, &UA347492},
	{SG81248, &UA347573},
	{SG81249, &UA347706},
	{SG81250, &UA347733},
	{SG81251, &UA347761},
	{SG81252, &UA347769},
	{SG81253, &UA351436},
	{SG81254, &UA351489},
	{SG81255, &UA351536},
	{SG81256, &UA351680},
	{SG81257, &UA351712},
	{SG81258, &UA351715},
	{SG81259, &UA351717},
	{SG81260, &UA351726},
	{SG81261, &UA351750},
	{SG81262, &UA351767},
	{SG81263, &UA351785},
	{SG81264, &UA352779},
	{SG81265, &UA353503},
	{SG81266, &UA355815},
	{SG81267, &UA356545},
	{SG81268, &UA358816},
	{SG81269, &UA363983},
	{SG81270, &UA454796},
	{SG81271, &UA605814},
	{SG81272, &UA622547},
	{SG81273, &UA628989},
	{SG81274, &UA630830},
	{SG81275, &UA641621},
	{SG81276, &UA642788},
	{SG81277, &UA644935},
	{SG81278, &UA655377},
	{SG81279, &UA667566},
	{SG81280, &UA671465},
	{SG81281, &UA681738},
	{SG81282, &UA715980},
	{SG81283, &UA742697},
	{SG81284, &UA896820},
	{SG81285, &UA901902},
	{SG81286, &UA1934622},
	{SG81287, &UA1934995},
	{SG81288, &UA1935534},
	{SG81289, &UA1951523},
	{SG81290, &UA1952670},
	{SG81291, &UA1956320},
	{SG81292, &UA1971426},
	{SG81293, &UA1978336},
	{SG81294, &UA1990229},
	{SG81295, &UA2167059},
	{SG81296, &UA2178345},
	{SG81297, &UA2184561},
	{SG81298, &UA2232463},
	{SG81299, &UA2306804},
	{SG81301, &UA2318260},
	{SG81302, &UA2318401},
	{SG81303, &UA2334907},
	{SG81304, &UA2366890},
	{SG81305, &UA2385687},
	{SG81306, &UA2386178},
	{SG81307, &UA2403370},
	{SG81308, &UA2422147},
	{SG81309, &UA2468149},
	{SG81310, &UA2494877},
	{SG81311, &UA2589544},
	{SG81313, &UA2594532},
	{SG81315, &UA2595128},
	{SG81317, &UA2600041},
	{SG81319, &UA2653210},
	{SG81321, &UA2705128},
	{SG81322, &UA2722992},
	{SG81323, &UA2737186},
	{SG81324, &UA2747538},
	{SG81325, &UA2764478},
	{SG81326, &UA2846310},
	{SG81327, &UA2860390},
	{SG81328, &UA2867497},
	{SG81329, &UA2953943},
	{SG81330, &UA3071063},
	{SG81331, &UA3101168},
	{SG81334, &UA2670194},
	{SG81335, &UA2671998},
	{SG81336, &UA2672069},
	{SG81337, &UA2672596},
	{SG81338, &UA2673897},
	{SG81339, &UA2674171},
	{SG81340, &UA2674751},
	{SG81341, &UA2674836},
	{SG81342, &UA2707306},
	{SG81343, &UA2708036},
	{SG81344, &UA2709110},
	{SG81345, &UA2709162},
	{SG81346, &UA2709488},
	{SG81347, &UA2710430},
	{SG81348, &UA2710832},
	{SG81349, &UA2711011},
	{SG81350, &UA2711496},
	{SG81351, &UA2711810},
	{SG81352, &UA2711924},
	{SG81353, &UA2711932},
	{SG81354, &UA2712376},
	{SG81355, &UA2712940},
	{SG81356, &UA2713666},
	{SG81357, &UA2713667},
	{SG81358, &UA2716664},
	{SG81359, &UA2734705},
	{SG81360, &UA2734793},
	{SG81361, &UA2734960},
	{SG81362, &UA2735548},
	{SG81363, &UA2735621},
	{SG81364, &UA2737433},
	{SG81365, &UA2738146},
	{SG81366, &UA2750803},
	{SG81367, &UA2750894},
	{SG81368, &UA2750962},
	{SG81369, &UA2751167},
	{SG81370, &UA2767280},
	{SG81371, &UA2767921},
	{SG81374, &UA2774129},
	{SG81375, &UA2780994},
	{SG81376, &UA2793941},
	{SG81377, &UA2795187},
	{SG81378, &UA2795291},
	{SG81379, &UA2795789},
	{SG81380, &UA2860421},
	{SG81381, &UA2861859},
	{SG81382, &UA2864032},
	{SG81383, &UA2864463},
	{SG81384, &UA2891710},
	{SG81385, &UA2936287},
	{SG81387, &UA2955704},
	{SG81388, &UA2993475},
	{SG81390, &UA2995176},
	{SG81391, &UA3014091},
	{SG81392, &UA3072859},
	{SG81399, &UA2510812},
	{SG81401, &UA2516401},
	{SG81402, &UA2516517},
	{SG81403, &UA2531656},
	{SG81404, &UA2531761},
	{SG81405, &UA2533241},
	{SG81406, &UA2535475},
	{SG81407, &UA2537095},
	{SG81408, &UA2537275},
	{SG81409, &UA2558480},
	{SG81410, &UA2561080},
	{SG81411, &UA2561577},
	{SG81412, &UA2562518},
	{SG81413, &UA2563855},
	{SG81414, &UA2569920},
	{SG81415, &UA2570877},
	{SG81416, &UA2571058},
	{SG81418, &UA2571455},
	{SG81419, &UA2572155},
	{SG81420, &UA2580561},
	{SG81421, &UA2581512},
	{SG81422, &UA2581742},
	{SG81423, &UA2601604},
	{SG81424, &UA2603273},
	{SG81425, &UA2662209},
	{SG81426, &UA2662416},
	{SG81427, &UA2663693},
	{SG81428, &UA2668756},
	{SG81429, &UA2670009},
	{SG81430, &UA2699724},
	{SG81431, &UA2700643},
	{SG81432, &UA2702323},
	{SG81433, &UA2705929},
	{SG81434, &UA2716656},
	{SG81435, &UA2730562},
	{SG81436, &UA2733505},
	{SG81438, &UA2792345},
	{SG81439, &UA2800514},
	{SG81440, &UA2853299},
	{SG81441, &UA2854020},
	{SG81442, &UA2856777},
	{SG81443, &UA3048845},
	{SG81444, &UA3125442},
	{SG81445, &UA2890986},
	{SG81446, &UA2900488},
	{SG81447, &UA2944825},
	{SG81448, &UA2946472},
	{SG81449, &UA2946725},
	{SG81450, &UA2946726},
	{SG81451, &UA2948048},
	{SG81452, &UA2964039},
	{SG81453, &UA2964204},
	{SG81454, &UA2964303},
	{SG81455, &UA2964528},
	{SG81456, &UA2964945},
	{SG81457, &UA2965358},
	{SG81458, &UA2965361},
	{SG81459, &UA2966136},
	{SG81460, &UA2966144},
	{SG81461, &UA2966316},
	{SG81462, &UA2968242},
	{SG81463, &UA3010771},
	{SG81464, &UA3011453},
	{SG81465, &UA3012810},
	{SG81466, &UA3012979},
	{SG81467, &UA3013031},
	{SG81468, &UA3013170},
	{SG81469, &UA3013200},
	{SG81470, &UA3013551},
	{SG81471, &UA3014154},
	{SG81472, &UA3014184},
	{SG81473, &UA3014703},
	{SG81474, &UA3016601},
	{SG81475, &UA3016742},
	{SG81476, &UA3017253},
	{SG81477, &UA3017575},
	{SG81479, &UA3017794},
	{SG81480, &UA3017812},
	{SG81481, &UA3018904},
	{SG81482, &UA3020244},
	{SG81483, &UA3030200},
	{SG81484, &UA3033269},
	{SG81485, &UA3056648},
	{SG81486, &UA3056966},
	{SG81487, &UA3056997},
	{SG81488, &UA3057018},
	{SG81489, &UA3057180},
	{SG81490, &UA3057307},
	{SG81491, &UA3057387},
	{SG81492, &UA3057561},
	{SG81493, &UA3058126},
	{SG81494, &UA3077873},
	{SG81495, &UA3078412},
	{SG81496, &UA3078493},
	{SG81497, &UA3078818},
	{SG81498, &UA3091819},
	{SG81499, &UA3091853},
	{SG81500, &UA3091866},
	{SG81501, &UA3092088},
	{SG81502, &UA3106981},
	{SG81503, &UA3107085},
	{SG81504, &UA3107108},
	{SG81505, &UA3107105},
	{SG81506, &UA3107701},
	{SG81507, &UA3127282},
	{SG81508, &UA3127401},
	{SG81516, &UA100478},
	{SG81517, &UA101174},
	{SG81518, &UA161764},
	{SG81519, &UA166983},
	{SG81520, &UA169152},
	{SG81521, &UA287039},
	{SG81522, &UA292720},
	{SG81523, &UA435399},
	{SG81524, &UA436623},
	{SG81525, &UA490898},
	{SG81526, &UA528525},
	{SG81527, &UA551305},
	{SG81528, &UA571007},
	{SG81529, &UA576593},
	{SG81530, &UA591018},
	{SG81531, &UA609888},
	{SG81532, &UA655399},
	{SG81533, &UA705164},
	{SG81534, &UA829965},
	{SG81535, &UA896211},
	{SG81536, &UA1868261},
	{SG81537, &UA1887872},
	{SG81538, &UA1953412},
	{SG81539, &UA1955237},
	{SG81540, &UA1964729},
	{SG81541, &UA1973514},
	{SG81542, &UA1984710},
	{SG81543, &UA1990595},
	{SG81544, &UA2145647},
	{SG81545, &UA2155752},
	{SG81547, &UA2166959},
	{SG81548, &UA2208746},
	{SG81549, &UA2217444},
	{SG81550, &UA2223702},
	{SG81551, &UA2227745},
	{SG81552, &UA2262424},
	{SG81553, &UA2278768},
	{SG81554, &UA2281558},
	{SG81555, &UA2304167},
	{SG81556, &UA2306521},
	{SG81557, &UA2307665},
	{SG81558, &UA2330878},
	{SG81559, &UA2402726},
	{SG81561, &UA2422779},
	{SG81562, &UA2423620},
	{SG81564, &UA2486958},
	{SG81566, &UA2575775},
	{SG81567, &UA2594693},
	{SG81568, &UA2706222},
	{SG81569, &UA2993886},
	{SG81570, &UA3074094},
	{SG81585, &UA92740},
	{SG81587, &UA92822},
	{SG81589, &UA94700},
	{SG81590, &UA98891},
	{SG81591, &UA172121},
	{SG81592, &UA183854},
	{SG81593, &UA421862},
	{SG81594, &UA432549},
	{SG81595, &UA448410},
	{SG81596, &UA459845},
	{SG81597, &UA480594},
	{SG81598, &UA552753},
	{SG81599, &UA565790},
	{SG81600, &UA591520},
	{SG81601, &UA669852},
	{SG81602, &UA684381},
	{SG81603, &UA753268},
	{SG81604, &UA813457},
	{SG81605, &UA1868165},
	{SG81607, &UA1870011},
	{SG81609, &UA1974171},
	{SG81611, &UA1978790},
	{SG81612, &UA2131372},
	{SG81613, &UA2137749},
	{SG81614, &UA2219213},
	{SG81615, &UA2223650},
	{SG81616, &UA2262495},
	{SG81617, &UA2341564},
	{SG81618, &UA2369480},
	{SG81619, &UA2369548},
	{SG81620, &UA2372922},
	{SG81621, &UA2400049},
	{SG81622, &UA2480800},
	{SG81623, &UA2506652},
	{SG81624, &UA2527698},
	{SG81625, &UA2547714},
	{SG81626, &UA2653347},
	{SG81627, &UA2703026},
	{SG81628, &UA2762449},
	{SG81629, &UA2802152},
	{SG81630, &UA2815490},
	{SG81695, &UA2358595},
	{SG81696, &UA2362130},
	{SG81697, &UA2406140},
	{SG81698, &UA2424629},
	{SG81700, &UA2426866},
	{SG81702, &UA2427414},
	{SG81703, &UA2429778},
	{SG81704, &UA2457518},
	{SG81705, &UA2482436},
	{SG81706, &UA2506695},
	{SG81707, &UA2793214},
	{SG81708, &UA2993887},
	{SG81709, &UA3074172},
	{SG81710, &UA347239},
	{SG81711, &UA347283},
	{SG81712, &UA347241},
	{SG81713, &UA347335},
	{SG81714, &UA347491},
	{SG81715, &UA347528},
	{SG81716, &UA347773},
	{SG81717, &UA349338},
	{SG81718, &UA351485},
	{SG81719, &UA351532},
	{SG81720, &UA351571},
	{SG81721, &UA351640},
	{SG81722, &UA351673},
	{SG81723, &UA351707},
	{SG81724, &UA351734},
	{SG81725, &UA351782},
	{SG81726, &UA351745},
	{SG81727, &UA351787},
	{SG81728, &UA352097},
	{SG81729, &UA352110},
	{SG81730, &UA352602},
	{SG81732, &UA352713},
	{SG81734, &UA353241},
	{SG81736, &UA355424},
	{SG81737, &UA364119},
	{SG81738, &UA437851},
	{SG81739, &UA576103},
	{SG81740, &UA588642},
	{SG81741, &UA599689},
	{SG81742, &UA600150},
	{SG81743, &UA607798},
	{SG81744, &UA640237},
	{SG81745, &UA642054},
	{SG81746, &UA648935},
	{SG81747, &UA656910},
	{SG81748, &UA685705},
	{SG81749, &UA688973},
	{SG81750, &UA698299},
	{SG81751, &UA704084},
	{SG81752, &UA708154},
	{SG81753, &UA708903},
	{SG81754, &UA711276},
	{SG81755, &UA726054},
	{SG81756, &UA730600},
	{SG81757, &UA790903},
	{SG81758, &UA826276},
	{SG81760, &UA895650},
	{SG81761, &UA903355},
	{SG81762, &UA2156530},
	{SG81763, &UA2159362},
	{SG81764, &UA2164830},
	{SG81765, &UA2166987},
	{SG81766, &UA2191497},
	{SG81767, &UA2227154},
	{SG81768, &UA2229759},
	{SG81769, &UA2308940},
	{SG81770, &UA2317443},
	{SG81771, &UA2319066},
	{SG81772, &UA2322463},
	{SG81773, &UA2325826},
	{SG81774, &UA2385166},
	{SG81775, &UA2424035},
	{SG81776, &UA2428623},
	{SG81777, &UA2460925},
	{SG81778, &UA2480733},
	{SG81779, &UA2480954},
	{SG81780, &UA2645475},
	{SG81781, &UA2704212},
	{SG81782, &UA2734867},
	{SG81783, &UA2748511},
	{SG81784, &UA2751005},
	{SG81785, &UA2766048},
	{SG81786, &UA2855506},
	{SG81787, &UA2868090},
	{SG81788, &UA686772},
	{SG81789, &UA856006},
	{SG81790, &UA904234},
	{SG81791, &UA904595},
	{SG81792, &UA905182},
	{SG81793, &UA905764},
	{SG81794, &UA910949},
	{SG81795, &UA1954736},
	{SG81796, &UA1959677},
	{SG81797, &UA1964347},
	{SG81798, &UA1975977},
	{SG81799, &UA1986560},
	{SG81800, &UA1991883},
	{SG81801, &UA1995930},
	{SG81802, &UA1999022},
	{SG81803, &UA1999484},
	{SG81804, &UA2002926},
	{SG81805, &UA2003694},
	{SG81806, &UA2004663},
	{SG81807, &UA2007357},
	{SG81808, &UA2164067},
	{SG81809, &UA2168329},
	{SG81810, &UA2168791},
	{SG81811, &UA2169803},
	{SG81812, &UA2177124},
	{SG81813, &UA2224091},
	{SG81814, &UA2262589},
	{SG81815, &UA2369095},
	{SG81816, &UA2548227},
	{SG81817, &UA2641627},
	{SG81818, &UA2776394},
	{SG81819, &UA2809298},
	{SG81820, &UA2809323},
	{SG81821, &UA2811380},
	{SG81823, &UA158693},
	{SG81824, &UA172354},
	{SG81825, &UA304520},
	{SG81826, &UA305127},
	{SG81827, &UA516191},
	{SG81829, &UA2129077},
	{SG81831, &UA204767},
	{SG81832, &UA205949},
	{SG81833, &UA210042},
	{SG81834, &UA210058},
	{SG81835, &UA210194},
	{SG81836, &UA211695},
	{SG81837, &UA216376},
	{SG81838, &UA219314},
	{SG81839, &UA219706},
	{SG81840, &UA227990},
	{SG81841, &UA230410},
	{SG81842, &UA241986},
	{SG81843, &UA328761},
	{SG81844, &UA329905},
	{SG81845, &UA332548},
	{SG81846, &UA334193},
	{SG81847, &UA335313},
	{SG81848, &UA394600},
	{SG81849, &UA395011},
	{SG81850, &UA452064},
	{SG81851, &UA556922},
	{SG81852, &UA585597},
	{SG81853, &UA639981},
	{SG81854, &UA647240},
	{SG81855, &UA647916},
	{SG81856, &UA648699},
	{SG81857, &UA667531},
	{SG81858, &UA698788},
	{SG81859, &UA704872},
	{SG81860, &UA730947},
	{SG81861, &UA734136},
	{SG81862, &UA774973},
	{SG81863, &UA776820},
	{SG81864, &UA782598},
	{SG81865, &UA805591},
	{SG81866, &UA880698},
	{SG81867, &UA897927},
	{SG81868, &UA1890840},
	{SG81869, &UA1937422},
	{SG81870, &UA2145583},
	{SG81871, &UA2154536},
	{SG81872, &UA2156421},
	{SG81873, &UA2160655},
	{SG81874, &UA2167736},
	{SG81875, &UA2262402},
	{SG81876, &UA2262741},
	{SG81877, &UA2351756},
	{SG81878, &UA2352638},
	{SG81879, &UA2421319},
	{SG81880, &UA2457169},
	{SG81881, &UA2481305},
	{SG81882, &UA2485038},
	{SG81883, &UA2654153},
	{SG81884, &UA2667246},
	{SG81885, &UA2796343},
	{SG81886, &UA2834648},
	{SG81887, &UA92506},
	{SG81888, &UA96838},
	{SG81889, &UA106300},
	{SG81890, &UA134846},
	{SG81891, &UA139715},
	{SG81893, &UA299310},
	{SG81894, &UA400121},
	{SG81895, &UA414744},
	{SG81896, &UA416160},
	{SG81897, &UA434391},
	{SG81898, &UA459498},
	{SG81899, &UA460292},
	{SG81900, &UA477326},
	{SG81901, &UA485824},
	{SG81902, &UA535037},
	{SG81903, &UA538787},
	{SG81904, &UA553968},
	{SG81905, &UA1863372},
	{SG81906, &UA1866166},
	{SG81907, &UA1888679},
	{SG81908, &UA2150664},
	{SG81909, &UA2215411},
	{SG81911, &UA2262411},
	{SG81912, &UA2277099},
	{SG81913, &UA2281561},
	{SG81914, &UA2297296},
	{SG81915, &UA2359954},
	{SG81916, &UA2481020},
	{SG81917, &UA2518472},
	{SG81918, &UA2649513},
	{SG81919, &UA2685481},
	{SG81920, &UA2824260},
	{SG81924, &UA200253},
	{SG81926, &UA660734},
	{SG81929, &UA197223},
	{SG81930, &UA208522},
	{SG81931, &UA210854},
	{SG81934, &UA216896},
	{SG81937, &UA217164},
	{SG81939, &UA218262},
	{SG81941, &UA218452},
	{SG81942, &UA223227},
	{SG81943, &UA230301},
	{SG81944, &UA239522},
	{SG81945, &UA323494},
	{SG81946, &UA324246},
	{SG81947, &UA324896},
	{SG81948, &UA325613},
	{SG81949, &UA328370},
	{SG81950, &UA334707},
	{SG81951, &UA337978},
	{SG81952, &UA454559},
	{SG81953, &UA554829},
	{SG81954, &UA567201},
	{SG81955, &UA574429},
	{SG81956, &UA607657},
	{SG81957, &UA702095},
	{SG81958, &UA703776},
	{SG81959, &UA718745},
	{SG81960, &UA833811},
	{SG81961, &UA881972},
	{SG81962, &UA898453},
	{SG81963, &UA1888918},
	{SG81964, &UA1889326},
	{SG81965, &UA1982453},
	{SG81966, &UA2140079},
	{SG81967, &UA2147638},
	{SG81968, &UA2161111},
	{SG81969, &UA2163432},
	{SG81970, &UA2166052},
	{SG81971, &UA2167619},
	{SG81973, &UA2226244},
	{SG81974, &UA2227747},
	{SG81975, &UA2237519},
	{SG81976, &UA2262380},
	{SG81977, &UA2290420},
	{SG81978, &UA2320639},
	{SG81980, &UA2325403},
	{SG81982, &UA2395040},
	{SG81983, &UA2422534},
	{SG81984, &UA2444721},
	{SG81985, &UA2457275},
	{SG81986, &UA2466449},
	{SG81987, &UA2486703},
	{SG81988, &UA2503784},
	{SG81989, &UA2667554},
	{SG81990, &UA2690902},
	{SG81991, &UA2705470},
	{SG81992, &UA2869049},
	{SG81993, &UA2936438},
	{SG81994, &UA3043211},
	{SG81997, &UA177679},
	{SG81998, &UA178958},
	{SG81999, &UA178965},
	{SG82000, &UA179428},
	{SG82001, &UA190157},
	{SG82004, &UA191025},
	{SG82005, &UA191120},
	{SG82006, &UA197239},
	{SG82007, &UA197744},
	{SG82008, &UA198980},
	{SG82009, &UA199704},
	{SG82010, &UA202360},
	{SG82011, &UA202537},
	{SG82012, &UA203459},
	{SG82013, &UA203468},
	{SG82014, &UA310663},
	{SG82015, &UA310819},
	{SG82016, &UA310829},
	{SG82017, &UA311303},
	{SG82018, &UA312739},
	{SG82019, &UA314009},
	{SG82021, &UA314986},
	{SG82022, &UA315262},
	{SG82023, &UA315545},
	{SG82024, &UA315780},
	{SG82025, &UA315941},
	{SG82026, &UA319172},
	{SG82027, &UA319174},
	{SG82029, &UA319648},
	{SG82030, &UA319768},
	{SG82031, &UA320318},
	{SG82032, &UA322640},
	{SG82033, &UA328791},
	{SG82034, &UA359433},
	{SG82035, &UA374856},
	{SG82036, &UA376735},
	{SG82037, &UA401100},
	{SG82038, &UA419425},
	{SG82039, &UA449581},
	{SG82040, &UA537728},
	{SG82041, &UA599784},
	{SG82042, &UA608805},
	{SG82043, &UA609094},
	{SG82044, &UA669072},
	{SG82046, &UA694385},
	{SG82047, &UA704778},
	{SG82048, &UA713053},
	{SG82050, &UA758798},
	{SG82051, &UA769628},
	{SG82052, &UA772583},
	{SG82053, &UA792949},
	{SG82054, &UA810811},
	{SG82055, &UA896899},
	{SG82056, &UA1874380},
	{SG82057, &UA1875171},
	{SG82058, &UA1880510},
	{SG82059, &UA1888219},
	{SG82060, &UA1951843},
	{SG82062, &UA1963317},
	{SG82064, &UA1980232},
	{SG82066, &UA2153470},
	{SG82068, &UA2159171},
	{SG82070, &UA2167022},
	{SG82071, &UA2231890},
	{SG82072, &UA2262423},
	{SG82073, &UA2262506},
	{SG82074, &UA2283510},
	{SG82075, &UA2285896},
	{SG82076, &UA2302585},
	{SG82077, &UA2302967},
	{SG82078, &UA2317499},
	{SG82079, &UA2330810},
	{SG82080, &UA2339147},
	{SG82081, &UA2342183},
	{SG82082, &UA2344764},
	{SG82083, &UA2369253},
	{SG82084, &UA2411624},
	{SG82085, &UA2421263},
	{SG82086, &UA2463245},
	{SG82087, &UA2490372},
	{SG82088, &UA2503244},
	{SG82089, &UA2529049},
	{SG82090, &UA2594727},
	{SG82091, &UA2595013},
	{SG82092, &UA2667671},
	{SG82093, &UA2697214},
	{SG82094, &UA2727393},
	{SG82095, &UA2728011},
	{SG82096, &UA2746846},
	{SG82097, &UA2758985},
	{SG82098, &UA2760882},
	{SG82099, &UA2839621},
	{SG82100, &UA2855776},
	{SG82101, &UA2983311},
	{SG82102, &UA3045674},
	{SG82103, &UA3085768},
	{SG82114, &UA2891911},
	{SG82115, &UA2900683},
	{SG82116, &UA2965372},
	{SG82117, &UA2965419},
	{SG82118, &UA2966622},
	{SG82119, &UA2966628},
	{SG82120, &UA2967508},
	{SG82121, &UA2967511},
	{SG82122, &UA3013556},
	{SG82123, &UA3014486},
	{SG82124, &UA3016190},
	{SG82125, &UA3016871},
	{SG82126, &UA3017037},
	{SG82127, &UA3017165},
	{SG82128, &UA3017434},
	{SG82129, &UA3017561},
	{SG82130, &UA3018080},
	{SG82131, &UA3018103},
	{SG82132, &UA3018368},
	{SG82133, &UA3018605},
	{SG82134, &UA3018769},
	{SG82135, &UA3056942},
	{SG82136, &UA3056991},
	{SG82138, &UA3056998},
	{SG82140, &UA3057137},
	{SG82141, &UA3057153},
	{SG82142, &UA3057168},
	{SG82143, &UA3057225},
	{SG82144, &UA3057319},
	{SG82145, &UA3058286},
	{SG82146, &UA3060349},
	{SG82147, &UA3077720},
	{SG82148, &UA3077759},
	{SG82149, &UA3078160},
	{SG82150, &UA3078375},
	{SG82151, &UA3078433},
	{SG82152, &UA3091732},
	{SG82153, &UA3091772},
	{SG82154, &UA3091931},
	{SG82155, &UA3106777},
	{SG82156, &UA3106961},
	{SG82157, &UA3107077},
	{SG82159, &UA3107157},
	{SG82161, &UA3124521},
	{SG82163, &UA3124664},
	{SG82273, &UA2404852},
	{SG82274, &UA2427090},
	{SG82275, &UA2427892},
	{SG82276, &UA2429901},
	{SG82277, &UA2430884},
	{SG82279, &UA2431988},
	{SG82281, &UA2434684},
	{SG82282, &UA2437153},
	{SG82284, &UA2437490},
	{SG82285, &UA2459570},
	{SG82286, &UA2460781},
	{SG82287, &UA2460963},
	{SG82288, &UA2460974},
	{SG82290, &UA2461111},
	{SG82291, &UA2461217},
	{SG82293, &UA2468364},
	{SG82294, &UA2468637},
	{SG82295, &UA2468792},
	{SG82296, &UA2469074},
	{SG82297, &UA2469184},
	{SG82298, &UA2484781},
	{SG82299, &UA2484846},
	{SG82300, &UA2486279},
	{SG82301, &UA2486398},
	{SG82302, &UA2506488},
	{SG82303, &UA2506813},
	{SG82304, &UA2507150},
	{SG82305, &UA2507288},
	{SG82306, &UA2656854},
	{SG82307, &UA2697651},
	{SG82308, &UA2729033},
	{SG82309, &UA2778536},
	{SG82310, &UA2846215},
	{SG82311, &UA2846773},
	{SG82312, &UA2846898},
	{SG82313, &UA2847664},
	{SG82314, &UA2866630},
	{SG82320, &UA2472035},
	{SG82321, &UA2487652},
	{SG82322, &UA2487856},
	{SG82323, &UA2488266},
	{SG82324, &UA2488277},
	{SG82325, &UA2490067},
	{SG82326, &UA2491889},
	{SG82327, &UA2491909},
	{SG82328, &UA2491922},
	{SG82329, &UA2492124},
	{SG82330, &UA2492126},
	{SG82331, &UA2492130},
	{SG82332, &UA2492366},
	{SG82333, &UA2492632},
	{SG82334, &UA2508162},
	{SG82335, &UA2508293},
	{SG82336, &UA2508583},
	{SG82337, &UA2508591},
	{SG82338, &UA2508708},
	{SG82339, &UA2508709},
	{SG82340, &UA2508914},
	{SG82341, &UA2509497},
	{SG82342, &UA2510094},
	{SG82343, &UA2510288},
	{SG82344, &UA2510463},
	{SG82345, &UA2510655},
	{SG82346, &UA2510770},
	{SG82347, &UA2510931},
	{SG82348, &UA2511028},
	{SG82350, &UA2511259},
	{SG82351, &UA2511334},
	{SG82352, &UA2511404},
	{SG82353, &UA2511466},
	{SG82354, &UA2511632},
	{SG82355, &UA2511736},
	{SG82356, &UA2511817},
	{SG82357, &UA2511835},
	{SG82358, &UA2512088},
	{SG82359, &UA2537021},
	{SG82360, &UA2537044},
	{SG82361, &UA2537270},
	{SG82362, &UA2537938},
	{SG82363, &UA2560528},
	{SG82364, &UA2561198},
	{SG82365, &UA2563072},
	{SG82366, &UA2563200},
	{SG82367, &UA2580511},
	{SG82368, &UA2580571},
	{SG82369, &UA2583059},
	{SG82370, &UA2597106},
	{SG82371, &UA2598675},
	{SG82372, &UA2599686},
	{SG82373, &UA2599998},
	{SG82374, &UA2659536},
	{SG82375, &UA2659565},
	{SG82376, &UA2659716},
	{SG82377, &UA2660056},
	{SG82379, &UA2660436},
	{SG82380, &UA2699647},
	{SG82382, &UA2700929},
	{SG82383, &UA2701991},
	{SG82384, &UA2702852},
	{SG82385, &UA2706754},
	{SG82386, &UA2748613},
	{SG82387, &UA2751266},
	{SG82388, &UA2800572},
	{SG82389, &UA2850521},
	{SG82390, &UA2850604},
	{SG82391, &UA2850768},
	{SG82392, &UA2851214},
	{SG82393, &UA2851727},
	{SG82394, &UA2855585},
	{SG82395, &UA2855593},
	{SG82396, &UA2858177},
	{SG82397, &UA2859145},
	{SG82398, &UA2990056},
	{SG82400, &UA3047287},
	{SG82402, &UA3071504},
	{SG82404, &UA3101515},
	{SG82447, &UA336662},
	{SG82448, &UA526346},
	{SG82449, &UA659482},
	{SG82450, &UA2307880},
	{SG82451, &UA2505005},
	{SG82452, &UA2506947},
	{SG82453, &UA2682370},
	{SG82454, &UA2844940},
	{SG82457, &UA2008893},
	{SG82458, &UA2262857},
	{SG82459, &UA2348413},
	{SG82460, &UA2350248},
	{SG82461, &UA2653229},
	{SG82462, &UA2763358},
	{SG82463, &UA3099244},
	{SG82521, &UA2199530},
	{SG82522, &UA2355630},
	{SG82523, &UA2387786},
	{SG82524, &UA2405336},
	{SG82525, &UA2427297},
	{SG82526, &UA2427900},
	{SG82527, &UA2437488},
	{SG82528, &UA2437540},
	{SG82529, &UA2461250},
	{SG82530, &UA2461320},
	{SG82531, &UA2468909},
	{SG82532, &UA2469206},
	{SG82533, &UA2485182},
	{SG82534, &UA2486311},
	{SG82535, &UA2528821},
	{SG82536, &UA2557907},
	{SG82537, &UA2579786},
	{SG82538, &UA2659547},
	{SG82539, &UA2667110},
	{SG82540, &UA2697589},
	{SG82541, &UA2704163},
	{SG82542, &UA2852500},
	{SG82543, &UA2998472},
	{SG82544, &UA2203300},
	{SG82545, &UA2353740},
	{SG82546, &UA2357461},
	{SG82547, &UA2357693},
	{SG82548, &UA2357737},
	{SG82549, &UA2358548},
	{SG82550, &UA2360986},
	{SG82551, &UA2362602},
	{SG82552, &UA2386395},
	{SG82553, &UA2424602},
	{SG82554, &UA2426306},
	{SG82555, &UA2456617},
	{SG82556, &UA2456801},
	{SG82558, &UA2841097},
	{SG82559, &UA2841454},
	{SG82560, &UA2982871},
	{SG82581, &UA91490},
	{SG82582, &UA98014},
	{SG82583, &UA107926},
	{SG82584, &UA131194},
	{SG82585, &UA138956},
	{SG82586, &UA148561},
	{SG82587, &UA172220},
	{SG82588, &UA512895},
	{SG82610, &UA2361587},
	{SG82611, &UA2367473},
	{SG82612, &UA2387884},
	{SG82613, &UA2388519},
	{SG82614, &UA2389465},
	{SG82615, &UA2390401},
	{SG82616, &UA2391457},
	{SG82618, &UA2404599},
	{SG82620, &UA2405403},
	{SG82622, &UA2405633},
	{SG82624, &UA2405796},
	{SG82626, &UA2422222},
	{SG82627, &UA2426608},
	{SG82628, &UA2426859},
	{SG82629, &UA2427463},
	{SG82630, &UA2428327},
	{SG82631, &UA2457164},
	{SG82632, &UA2457371},
	{SG82633, &UA2458063},
	{SG82634, &UA2504949},
	{SG82635, &UA2528049},
	{SG82636, &UA2529211},
	{SG82637, &UA2656018},
	{SG82638, &UA2790471},
	{SG82639, &UA2795176},
	{SG82640, &UA2843033},
	{SG82641, &UA2843317},
	{SG82642, &UA2843366},
	{SG82643, &UA2843480},
	{SG82644, &UA3048579},
	{SG82661, &UA2040997},
	{SG82686, &UA251763},
	{SG82708, &UA251567},
	{SG82709, &UA251600},
	{SG82710, &UA251614},
	{SG82711, &UA251638},
	{SG82712, &UA251646},
	{SG82713, &UA251652},
	{SG82714, &UA251667},
	{SG82715, &UA251670},
	{SG82716, &UA251680},
	{SG82717, &UA251672},
	{SG82718, &UA251767},
	{SG82719, &UA251791},
	{SG82720, &UA251793},
	{SG82721, &UA251828},
	{SG82722, &UA251843},
	{SG82723, &UA251854},
	{SG82724, &UA251876},
	{SG82725, &UA251896},
	{SG82726, &UA251983},
	{SG82727, &UA251984},
	{SG82728, &UA251985},
	{SG82729, &UA251995},
	{SG82730, &UA251996},
	{SG82731, &UA252018},
	{SG82732, &UA344863},
	{SG82733, &UA344829},
	{SG82734, &UA345262},
	{SG82735, &UA345834},
	{SG82736, &UA347479},
	{SG82737, &UA351728},
	{SG82738, &UA590215},
	{SG82739, &UA654843},
	{SG82740, &UA699673},
	{SG82741, &UA737612},
	{SG82742, &UA789435},
	{SG82743, &UA790891},
	{SG82744, &UA802322},
	{SG82745, &UA807454},
	{SG82746, &UA825976},
	{SG82747, &UA838451},
	{SG82748, &UA895064},
	{SG82749, &UA1955060},
	{SG82750, &UA1972469},
	{SG82751, &UA1988869},
	{SG82753, &UA2154722},
	{SG82754, &UA2215155},
	{SG82755, &UA2220073},
	{SG82756, &UA2226844},
	{SG82757, &UA2303686},
	{SG82758, &UA2351694},
	{SG82759, &UA2364465},
	{SG82761, &UA2403569},
	{SG82763, &UA2474887},
	{SG82764, &UA2496666},
	{SG82765, &UA2503424},
	{SG82766, &UA2524062},
	{SG82767, &UA2527747},
	{SG82768, &UA2528924},
	{SG82769, &UA2667486},
	{SG82770, &UA2762467},
	{SG82771, &UA2814138},
	{SG82772, &UA2957699},
	{SG82779, &UA2237704},
	{SG82780, &UA2238941},
	{SG82781, &UA2361082},
	{SG82782, &UA2424522},
	{SG82783, &UA2456515},
	{SG82784, &UA2594895},
	{SG82785, &UA2841334},
	{SG82789, &UA94676},
	{SG82790, &UA101660},
	{SG82791, &UA101896},
	{SG82792, &UA126674},
	{SG82793, &UA549232},
	{SG82794, &UA841647},
	{SG82795, &UA1872763},
	{SG82796, &UA306386},
	{SG82797, &UA2844845},
	{SG82810, &UA797245},
	{SG82817, &UA600324},
	{SG82818, &UA916114},
	{SG82823, &UA114783},
	{SG82824, &UA128218},
	{SG82825, &UA138337},
	{SG82826, &UA147978},
	{SG82827, &UA161663},
	{SG82828, &UA385975},
	{SG82829, &UA1863992},
	{SG82831, &UA2262570},
	{SG82832, &UA2265817},
	{SG82834, &UA261742},
	{SG82835, &UA656617},
	{SG82836, &UA1941923},
	{SG82837, &UA2803840},
	{SG82847, &UA251584},
	{SG82907, &UA92707},
	{SG82908, &UA101524},
	{SG82909, &UA106744},
	{SG82910, &UA130575},
	{SG82911, &UA141474},
	{SG82912, &UA446456},
	{SG82913, &UA508477},
	{SG82914, &UA1866976},
	{SG82916, &UA2222178},
	{SG82918, &UA2262794},
	{SG82919, &UA2278916},
	{SG82920, &UA2313710},
	{SG82921, &UA2803589},
	{SG82922, &UA2815705},
	{SG82923, &UA2506898},
	{SG82946, &UA327165},
	{SG82947, &UA663602},
	{SG82958, &UA94654},
	{SG82965, &UA549330},
	{SG83001, &UA616049},
	{SG83002, &UA630376},
	{SG83003, &UA637430},
	{SG83005, &UA681344},
	{SG83006, &UA775105},
	{SG83007, &UA802291},
	{SG83008, &UA806640},
	{SG83009, &UA862615},
	{SG83010, &UA891844},
	{SG83011, &UA2158834},
	{SG83013, &UA2320281},
	{SG83014, &UA2393499},
	{SG83031, &UA2669490},
	{SG83041, &UA94393},
	{SG83042, &UA99987},
	{SG83043, &UA101994},
	{SG83044, &UA123598},
	{SG83045, &UA271134},
	{SG83046, &UA503251},
	{SG83048, &UA94076},
	{SG83049, &UA94664},
	{SG83050, &UA102024},
	{SG83051, &UA509724},
	{SG83052, &UA562039},
	{SG83053, &UA648839},
	{SG83054, &UA736626},
	{SG83057, &UA770855},
	{SG83059, &UA1875173},
	{SG83061, &UA2150617},
	{SG83062, &UA2281416},
	{SG83063, &UA2442227},
	{SG83064, &UA2450735},
	{SG83065, &UA2490364},
	{SG83066, &UA2640922},
	{SG83067, &UA2684789},
	{SG83068, &UA2746201},
	{SG83069, &UA2833617},
	{SG83117, &UA152242},
	{SG83118, &UA707252},
	{SG83119, &UA760367},
	{SG83120, &UA902552},
	{SG83121, &UA2008445},
	{SG83122, &UA2262433},
	{SG83123, &UA2327132},
	{SG83130, &UA137414},
	{SG83131, &UA306219},
	{SG83132, &UA374785},
	{SG83133, &UA443698},
	{SG83134, &UA574284},
	{SG83135, &UA801309},
	{SG83136, &UA852154},
	{SG83137, &UA1894417},
	{SG83138, &UA1957062},
	{SG83139, &UA2154591},
	{SG83140, &UA2262768},
	{SG83141, &UA2262783},
	{SG83142, &UA2285643},
	{SG83144, &UA2324186},
	{SG83145, &UA2366351},
	{SG83146, &UA2383645},
	{SG83147, &UA2398722},
	{SG83148, &UA2442566},
	{SG83149, &UA2497253},
	{SG83150, &UA2696507},
	{SG83151, &UA2697341},
	{SG83153, &UA3043324},
	{SG83161, &UA187689},
	{SG83162, &UA300157},
	{SG83163, &UA306180},
	{SG83166, &UA103307},
	{SG83227, &UA2363192},
	{SG83231, &UA2606933},
	{SG83232, &UA2781338},
	{SG83290, &UA453944},
	{SG83292, &UA453968},
	{SG83294, &UA603008},
	{SG83295, &UA603099},
	{SG83296, &UA603153},
	{SG83298, &UA603366},
	{SG83300, &UA607651},
	{SG83301, &UA623730},
	{SG83302, &UA686712},
	{SG83303, &UA755415},
	{SG83304, &UA822645},
	{SG83305, &UA2165157},
	{SG83306, &UA2369487},
	{SG83307, &UA173997},
	{SG83308, &UA198791},
	{SG83309, &UA311742},
	{SG83310, &UA382086},
	{SG83311, &UA393202},
	{SG83312, &UA702564},
	{SG83313, &UA812294},
	{SG83314, &UA880141},
	{SG83315, &UA887004},
	{SG83316, &UA2369275},
	{SG83323, &UA138279},
	{SG83324, &UA171521},
	{SG83325, &UA204674},
	{SG83326, &UA236517},
	{SG83327, &UA261423},
	{SG83365, &UA2887167},
	{SG83366, &UA3077758},
	{SG83367, &UA2004702},
	{SG83368, &UA2479742},
	{SG83391, &UA94690},
	{SG83392, &UA109136},
	{SG83393, &UA175123},
	{SG83394, &UA199773},
	{SG83401, &UA2510968},
	{SG83408, &UA422246},
	{SG83409, &UA422252},
	{SG83410, &UA588512},
	{SG83411, &UA588702},
	{SG83412, &UA590220},
	{SG83413, &UA594692},
	{SG83414, &UA622575},
	{SG83415, &UA680130},
	{SG83416, &UA688254},
	{SG83417, &UA703639},
	{SG83419, &UA721226},
	{SG83420, &UA753663},
	{SG83421, &UA1952576},
	{SG83422, &UA1993721},
	{SG83423, &UA2312880},
	{SG83424, &UA2339281},
	{SG83425, &UA2339816},
	{SG83435, &UA824262},
	{SG83436, &UA857456},
	{SG83437, &UA2638753},
	{SG83439, &UA2512432},
	{SG83451, &UA2490690},
	{SG83452, &UA2491504},
	{SG83453, &UA2511633},
	{SG83454, &UA2511832},
	{SG83455, &UA2512363},
	{SG83456, &UA2597674},
	{SG83457, &UA2699893},
	{SG83458, &UA2731078},
	{SG83459, &UA2850915},
	{SG83460, &UA2851468},
	{SG83476, &UA108404},
	{SG83477, &UA169572},
	{SG83478, &UA196353},
	{SG83479, &UA197083},
	{SG83480, &UA197231},
	{SG83481, &UA209847},
	{SG83482, &UA236702},
	{SG83483, &UA241680},
	{SG83501, &UA2490169},
	{SG83502, &UA2490347},
	{SG83503, &UA2510929},
	{SG83504, &UA2660244},
	{SG83505, &UA2765306},
	{SG83506, &UA3086449},
	{SG83509, &UA2669019},
	{SG83532, &UA2201118},
	{SG83536, &UA93011},
	{SG83537, &UA103008},
	{SG83538, &UA121091},
	{SG83539, &UA1885411},
	{SG83549, &UA791697},
	{SG83552, &UA93514},
	{SG83553, &UA459455},
	{SG83554, &UA2438013},
	{SG83582, &UA2425866},
	{SG83583, &UA2558018},
	{SG83584, &UA2847192},
	{SG83617, &UA912253},
	{SG83626, &UA433382},
	{SG83627, &UA767503},
	{SG83633, &UA2670392},
	{SG83648, &UA2512792},
	{SG83649, &UA897052},
	{SG83650, &UA2002983},
	{SG83676, &UA147833},
	{SG83677, &UA265490},
	{SG83678, &UA278612},
	{SG83679, &UA536339},
	{SG83680, &UA554181},
	{SG83681, &UA640945},
	{SG83682, &UA2262143},
	{SG83689, &UA351691},
	{SG83690, &UA352193},
	{SG83691, &UA352903},
	{SG83692, &UA353298},
	{SG83693, &UA355985},
	{SG83694, &UA398306},
	{SG83695, &UA628903},
	{SG83696, &UA653811},
	{SG83697, &UA692738},
	{SG83698, &UA701201},
	{SG83699, &UA712664},
	{SG83700, &UA1935524},
	{SG83701, &UA1938641},
	{SG83702, &UA1945868},
	{SG83703, &UA2306549},
	{SG83704, &UA2307172},
	{SG83705, &UA2307441},
	{SG83706, &UA2307654},
	{SG83707, &UA25401},
	{SG83708, &UA25400},
	{SG83709, &UA129081},
	{SG83710, &UA2361523},
	{SG83711, &UA2766658},
	{SG83712, &UA3039779},
	{SG83713, &UA3106947},
	{SG83726, &UA2017653},
	{SG83738, &UA251720},
	{SG83739, &UA251886},
	{SG83740, &UA251987},
	{SG83741, &UA344850},
	{SG83742, &UA575546},
	{SG83743, &UA686590},
	{SG83744, &UA869525},
	{SG83745, &UA1903036},
	{SG83746, &UA2301602},
	{SG83747, &UA2302246},
	{SG83749, &UA2304294},
	{SG83750, &UA2369169},
	{SG83768, &UA2511671},
	{SG83784, &UA127846},
	{SG83785, &UA135552},
	{SG83786, &UA149245},
	{SG83787, &UA643144},
	{SG83791, &UA2368399},
	{SG83813, &UA92804},
	{SG83814, &UA96094},
	{SG83815, &UA135711},
	{SG83816, &UA264618},
	{SG83817, &UA271978},
	{SG83828, &UA180249},
	{SG83829, &UA198345},
	{SG83830, &UA223159},
	{SG83831, &UA269416},
	{SG83832, &UA289628},
	{SG83833, &UA476449},
	{SG83834, &UA544918},
	{SG83835, &UA546794},
	{SG83836, &UA624867},
	{SG83849, &UA202101},
	{SG83850, &UA332950},
	{SG83851, &UA333509},
	{SG83852, &UA556453},
	{SG83853, &UA666471},
	{SG83854, &UA806991},
	{SG83855, &UA814760},
	{SG83856, &UA1888245},
	{SG83859, &UA2041368},
	{SG83861, &UA2361872},
	{SG83871, &UA899646},
	{SG83872, &UA2201252},
	{SG83873, &UA126653},
	{SG83874, &UA168765},
	{SG83875, &UA174926},
	{SG83876, &UA301599},
	{SG83877, &UA332983},
	{SG83878, &UA683795},
	{SG83879, &UA2269746},
	{SG83880, &UA2275922},
	{SG83886, &UA2434644},
	{SG83905, &UA165608},
	{SG83924, &UA613223},
	{SG83925, &UA613252},
	{SG83926, &UA2034129},
	{SG83927, &UA93710},
	{SG83930, &UA2369108},
	{SG83949, &UA241708},
	{SG83950, &UA243823},
	{SG83951, &UA244158},
	{SG83952, &UA244738},
	{SG83953, &UA336700},
	{SG83954, &UA337969},
	{SG83955, &UA569649},
	{SG83956, &UA570767},
	{SG83957, &UA804392},
	{SG83958, &UA2297351},
	{SG83959, &UA2669264},
	{SG83966, &UA2425707},
	{SG83993, &UA99633},
	{SG83994, &UA99981},
	{SG83995, &UA112187},
	{SG83997, &UA113030},
	{SG83999, &UA166696},
	{SG84001, &UA168268},
	{SG84003, &UA265957},
	{SG84004, &UA431186},
	{SG84027, &UA2425558},
	{SG84038, &UA2512431},
	{SG84046, &UA159828},
	{SG84047, &UA692778},
	{SG84048, &UA723331},
	{SG84049, &UA782310},
	{SG84050, &UA1884308},
	{SG84051, &UA1893295},
	{SG84052, &UA2472894},
	{SG84053, &UA2496554},
	{SG84054, &UA2367224},
	{SG84059, &UA633403},
	{SG84061, &UA99491},
	{SG84062, &UA104189},
	{SG84063, &UA111487},
	{SG84064, &UA242006},
	{SG84065, &UA2208747},
	{SG84099, &UA94717},
	{SG84100, &UA259510},
	{SG84101, &UA499311},
	{SG84102, &UA2682286},
	{SG84103, &UA154926},
	{SG84104, &UA2152379},
	{SG84118, &UA152323},
	{SG84133, &UA630627},
	{SG84135, &UA371590},
	{SG84136, &UA810061},
	{SG84137, &UA2147449},
	{SG84139, &UA2262158},
	{SG84145, &UA2041239},
	{SG84156, &UA261004},
	{SG84157, &UA375295},
	{SG84158, &UA693102},
	{SG84170, &UA729866},
	{SG84171, &UA2351627},
	{SG84182, &UA624196},
	{SG84185, &UA141393},
	{SG84186, &UA152156},
	{SG84187, &UA268428},
	{SG84188, &UA272607},
	{SG84196, &UA395455},
	{SG84197, &UA624292},
	{SG84198, &UA801988},
	{SG84214, &UA2513350},
	{SG84215, &UA102264},
	{SG84216, &UA112395},
	{SG84217, &UA2209918},
	{SG84218, &UA95827},
	{SG84231, &UA2235420},
	{SG84232, &UA103515},
	{SG84233, &UA128596},
	{SG84234, &UA132616},
	{SG84238, &UA2430894},
	{SG84263, &UA114816},
	{SG84264, &UA152059},
	{SG84265, &UA174676},
	{SG84310, &UA8883},
	{SG84316, &UA2491512},
	{SG84318, &UA2672657},
	{SG84339, &UA912272},
	{SG84342, &UA903234},
	{SG84343, &UA1996994},
	{SG84346, &UA2353295},
	{SG84347, &UA655063},
	{SG84350, &UA105835},
	{SG84351, &UA167798},
	{SG84352, &UA171548},
	{SG84353, &UA241696},
	{SG84354, &UA2138214},
	{SG84360, &UA238593},
	{SG84361, &UA839065},
	{SG84396, &UA2204701},
	{SG84413, &UA2422981},
	{SG84448, &UA2200862},
	{SG84469, &UA595218},
	{SG84470, &UA595668},
	{SG84471, &UA599545},
	{SG84472, &UA599681},
	{SG84473, &UA896805},
	{SG84474, &UA2025399},
	{SG84475, &UA2032936},
	{SG84476, &UA2203476},
	{SG84477, &UA2236475},
	{SG84478, &UA2237115},
	{SG84479, &UA2237117},
	{SG84481, &UA2237762},
	{SG84483, &UA2354480},
	{SG84484, &UA2356618},
	{SG84500, &UA632625},
	{SG84505, &UA629616},
	{SG84507, &UA850065},
	{SG84508, &UA2003681},
	{SG84511, &UA2368398},
	{SG84526, &UA152322},
	{SG84530, &UA699473},
	{SG84551, &UA2041517},
	{SG84594, &UA771666},
	{SG84595, &UA1881334},
	{SG84597, &UA2369499},
	{SG84602, &UA2367698},
	{SG84606, &UA616034},
	{SG84669, &UA2486435},
	{SG84689, &UA183879},
	{SG84721, &UA2369184},
	{SG84728, &UA895608},
	{SG84755, &UA152309},
	{SG84756, &UA284300},
	{SG84757, &UA543815},
	{SG84761, &UA2197968},
	{SG84762, &UA2488334},
	{SG84767, &UA300701},
	{SG84768, &UA2369270},
	{SG84783, &UA2040628},
	{SG84785, &UA643167},
	{SG84807, &UA587647},
	{SG84812, &UA2490683},
	{SG84825, &UA148134},
	{SG84826, &UA427449},
	{SG84827, &UA603231},
	{SG84829, &UA820463},
	{SG84833, &UA226840},
	{SG84834, &UA820379},
	{SG84835, &UA2283901},
	{SG84836, &UA2506853},
	{SG84837, &UA2742036},
	{SG84840, &UA616385},
	{SG84880, &UA63255},
	{SG84881, &UA177760},
	{SG84882, &UA203431},
	{SG84884, &UA2202584},
	{SG84885, &UA2202588},
	{SG84886, &UA2422898},
	{SG84887, &UA2484017},
	{SG84888, &UA2202156},
	{SG84908, &UA1870671},
	{SG84909, &UA2148377},
	{SG84915, &UA143981},
	{SG84916, &UA569013},
	{SG84917, &UA716356},
	{SG84918, &UA718160},
	{SG84919, &UA2168602},
	{SG84921, &UA2262250},
	{SG84922, &UA2364595},
	{SG84923, &UA2366549},
	{SG84924, &UA2473028},
	{SG84925, &UA2720496},
	{SG84926, &UA2743136},
	{SG84927, &UA2810878},
	{SG84928, &UA2992399},
	{SG84929, &UA71975},
	{SG84936, &UA27535},
	{SG84938, &UA165280},
	{SG84939, &UA303947},
	{SG84940, &UA454197},
	{SG84941, &UA848937},
	{SG84942, &UA1969792},
	{SG84943, &UA2028943},
	{SG84965, &UA818335},
	{SG84966, &UA858898},
	{SG84967, &UA859177},
	{SG84968, &UA893751},
	{SG84969, &UA2462586},
	{SG84970, &UA2484239},
	{SG84971, &UA2681132},
	{SG84973, &UA2801657},
	{SG84977, &UA925189},
	{SG84978, &UA2746598},
	{SG85031, &UA1939721},
	{SG85032, &UA2147026},
	{SG85033, &UA2409097},
	{SG85034, &UA2412561},
	{SG85041, &UA633306},
	{SG85043, &UA3118245},
	{SG85044, &UA2353974},
	{SG85050, &UA174800},
	{SG85051, &UA273913},
	{SG85052, &UA522144},
	{SG85053, &UA2210009},
	{SG85057, &UA102229},
	{SG85080, &UA316945},
	{SG85081, &UA391797},
	{SG85082, &UA433911},
	{SG85083, &UA469581},
	{SG85084, &UA719134},
	{SG85086, &UA599867},
	{SG85088, &UA863157},
	{SG85089, &UA875277},
	{SG85090, &UA2507771},
	{SG85091, &UA2507993},
	{SG85092, &UA2202909},
	{SG85093, &UA2238153},
	{SG85094, &UA2658607},
	{SG85095, &UA2845195},
	{SG85096, &UA2987288},
	{SG85097, &UA685917},
	{SG85107, &UA2354074},
	{SG85144, &UA2203722},
	{SG85157, &UA131128},
	{SG85158, &UA633391},
	{SG85159, &UA2176807},
	{SG85160, &UA2236205},
	{SG85164, &UA138425},
	{SG85165, &UA417903},
	{SG85166, &UA797713},
	{SG85167, &UA797802},
	{SG85172, &UA237874},
	{SG85173, &UA877560},
	{SG85182, &UA1947368},
	{SG85183, &UA2262155},
	{SG85186, &UA2201772},
	{SG85206, &UA830045},
	{SG85207, &UA1997217},
	{SG85225, &UA2042682},
	{SG85230, &UA2674733},
	{SG85231, &UA2709491},
	{SG85232, &UA765252},
	{SG85236, &UA691673},
	{SG85279, &UA632862},
	{SG85280, &UA202058},
	{SG85285, &UA2491917},
	{SG85288, &UA416902},
	{SG85289, &UA556589},
	{SG85290, &UA778375},
	{SG85291, &UA893419},
	{SG85293, &UA1874591},
	{SG85294, &UA2141925},
	{SG85308, &UA90775},
	{SG85333, &UA306729},
	{SG85334, &UA447095},
	{SG85337, &UA679896},
	{SG85362, &UA251797},
	{SG85371, &UA241692},
	{SG85374, &UA390550},
	{SG85375, &UA2438852},
	{SG85399, &UA433661},
	{SG85400, &UA2854622},
	{SG85401, &UA190796},
	{SG85402, &UA233409},
	{SG85407, &UA246372},
	{SG85426, &UA263546},
	{SG85427, &UA2369194},
	{SG85428, &UA904926},
	{SG85429, &UA2008900},
	{SG85468, &UA426939},
	{SG85469, &UA601390},
	{SG85470, &UA2495603},
	{SG85471, &UA2505948},
	{SG85488, &UA629018},
	{SG85501, &UA2512783},
	{SG85502, &UA2581073},
	{SG85514, &UA166281},
	{SG85515, &UA302374},
	{SG85519, &UA2003679},
	{SG85520, &UA2026275},
	{SG85521, &UA2476412},
	{SG85543, &UA2436244},
	{SG85544, &UA2557858},
	{SG85545, &UA337647},
	{SG85547, &UA243039},
	{SG85548, &UA302841},
	{SG85549, &UA522453},
	{SG85551, &UA696104},
	{SG85552, &UA1873836},
	{SG85553, &UA1900198},
	{SG85554, &UA2133012},
	{SG85555, &UA2148620},
	{SG85556, &UA2281471},
	{SG85557, &UA2289507},
	{SG85558, &UA2462901},
	{SG85561, &UA251103},
	{SG85562, &UA416728},
	{SG85563, &UA1888028},
	{SG85575, &UA251682},
	{SG85583, &UA347731},
	{SG85585, &UA2011110},
	{SG85590, &UA2410939},
	{SG85592, &UA2494145},
	{SG85593, &UA2518730},
	{SG85594, &UA2774901},
	{SG85640, &UA2735476},
	{SG85654, &UA598332},
	{SG85662, &UA627504},
	{SG85663, &UA656574},
	{SG85664, &UA742717},
	{SG85665, &UA1837319},
	{SG85670, &UA287878},
	{SG85671, &UA414144},
	{SG85672, &UA470510},
	{SG85715, &UA417952},
	{SG85760, &UA624573},
	{SG85762, &UA766829},
	{SG85769, &UA243203},
	{SG85773, &UA2369015},
	{SG85774, &UA2655512},
	{SG85795, &UA179449},
	{SG85796, &UA202268},
	{SG85825, &UA251976},
	{SG85831, &UA2382866},
	{SG85834, &UA141088},
	{SG85845, &UA79860},
	{SG85847, &UA172071},
	{SG85848, &UA525415},
	{SG85849, &UA710676},
	{SG85864, &UA590208},
	{SG85867, &UA745391},
	{SG85869, &UA2020855},
	{SG85903, &UA399256},
	{SG85904, &UA414809},
	{SG85905, &UA2278073},
	{SG85906, &UA3040917},
	{SG85917, &UA528581},
	{SG85933, &UA2362840},
	{SG85934, &UA2389403},
	{SG85935, &UA2958027},
	{SG85955, &UA215952},
	{SG85961, &UA653380},
	{SG85987, &UA2505414},
	{SG85988, &UA2696808},
	{SG85989, &UA876639},
	{SG85990, &UA2138465},
	{SG85991, &UA2210941},
	{SG86003, &UA815652},
	{SG86013, &UA2202585},
	{SG86015, &UA2531154},
	{SG86017, &UA2603412},
	{SG86018, &UA2701925},
	{SG86019, &UA2370888},
	{SG86020, &UA2474443},
	{SG86021, &UA2525744},
	{SG86022, &UA2685319},
	{SG86023, &UA2723625},
	{SG86039, &UA706293},
	{SG86040, &UA917217},
	{SG86041, &UA1922249},
	{SG86042, &UA2022111},
	{SG86044, &UA2025204},
	{SG86045, &UA2033972},
	{SG86054, &UA113625},
	{SG86066, &UA217718},
	{SG86085, &UA198855},
	{SG86086, &UA2209421},
	{SG86112, &UA536594},
	{SG86113, &UA656517},
	{SG86114, &UA758412},
	{SG86115, &UA3040877},
	{SG86122, &UA17235},
	{SG86123, &UA301252},
	{SG86138, &UA266555},
	{SG86145, &UA2383528},
	{SG86181, &UA2384059},
	{SG86182, &UA260232},
	{SG86183, &UA382264},
	{SG86184, &UA2262432},
	{SG86185, &UA2263534},
	{SG86193, &UA232224},
	{SG86194, &UA325028},
	{SG86222, &UA667816},
	{SG86224, &UA105086},
	{SG86260, &UA2360391},
	{SG86261, &UA2230304},
	{SG86281, &UA259593},
	{SG86282, &UA548988},
	{SG86283, &UA705208},
	{SG86284, &UA802839},
	{SG86285, &UA2131149},
	{SG86305, &UA648056},
	{SG86306, &UA706871},
	{SG86307, &UA804186},
	{SG86309, &UA1955908},
	{SG86330, &UA2358637},
	{SG86354, &UA2768118},
	{SG86359, &UA2369225},
	{SG86365, &UA413427},
	{SG86366, &UA534709},
	{SG86367, &UA798831},
	{SG86368, &UA1879832},
	{SG86369, &UA2132965},
	{SG86370, &UA2278942},
	{SG86375, &UA808005},
	{SG86376, &UA857381},
	{SG86401, &UA130989},
	{SG86406, &UA352692},
	{SG86408, &UA711095},
	{SG86412, &UA2216405},
	{SG86418, &UA172380},
	{SG86419, &UA731342},
	{SG86420, &UA2200132},
	{SG86422, &UA845815},
	{SG86423, &UA412741},
	{SG86424, &UA412744},
	{SG86425, &UA794048},
	{SG86426, &UA825201},
	{SG86427, &UA2145455},
	{SG86428, &UA2759222},
	{SG86429, &UA2933185},
	{SG86454, &UA2849110},
	{SG86466, &UA668639},
	{SG86467, &UA448363},
	{SG86468, &UA2675831},
	{SG86469, &UA609510},
	{SG86470, &UA2217912},
	{SG86471, &UA2551577},
	{SG86476, &UA396977},
	{SG86477, &UA2537576},
	{SG86495, &UA2424777},
	{SG86496, &UA2748191},
	{SG86497, &UA2564712},
	{SG86499, &UA797058},
	{SG86505, &UA131652},
	{SG86510, &UA2460259},
	{SG86515, &UA2203468},
	{SG86516, &UA2237126},
	{SG86517, &UA2237667},
	{SG86518, &UA2487851},
	{SG86521, &UA2405008},
	{SG86537, &UA2456001},
	{SG86541, &UA2146764},
	{SG86542, &UA2212429},
	{SG86543, &UA2336062},
	{SG86548, &UA2428103},
	{SG86549, &UA616392},
	{SG86554, &UA2492440},
	{SG86557, &UA689505},
	{SG86558, &UA1894522},
	{SG86559, &UA2586797},
	{SG86560, &UA2026474},
	{SG86561, &UA2444918},
	{SG86571, &UA460431},
	{SG86572, &UA647638},
	{SG86574, &UA2579951},
	{SG86575, &UA2580205},
	{SG86579, &UA2482625},
	{SG86580, &UA2657828},
	{SG86581, &UA2981907},
	{SG86582, &UA2981970},
	{SG86583, &UA3039765},
	{SG86584, &UA3039783},
	{SG86585, &UA3070612},
	{SG86586, &UA3088062},
	{SG86587, &UA3098821},
	{SG86588, &UA546346},
	{SG86591, &UA865854},
	{SG86592, &UA2019119},
	{SG86593, &UA2019367},
	{SG86597, &UA2139370},
	{SG86598, &UA2409582},
	{SG86613, &UA2349206},
	{SG86616, &UA467215},
	{SG86617, &UA1862895},
	{SG86620, &UA562693},
	{SG86623, &UA2385705},
	{SG86625, &UA1995699},
	{SG86626, &UA2544928},
	{SG86627, &UA2651536},
	{SG86629, &UA639611},
	{SG86630, &UA894984},
	{SG86631, &UA433739},
	{SG86632, &UA434609},
	{SG86633, &UA520562},
	{SG86634, &UA616398},
	{SG86635, &UA2422177},
	{SG86636, &UA2878788},
	{SG86638, &UA2421578},
	{SG86648, &UA599813},
	{SG86649, &UA2775703},
	{SG86650, &UA2889104},
	{SG86651, &UA691096},
	{SG86652, &UA2409414},
	{SG86654, &UA252034},
	{SG86655, &UA2603187},
	{SG86657, &UA2374660},
	{SG86658, &UA3039550},
	{SG86665, &UA459836},
	{SG86666, &UA461557},
	{SG86667, &UA713391},
	{SG86675, &UA616388},
	{SG86693, &UA2735336},
	{SG86699, &UA413793},
	{SG86700, &UA435478},
	{SG86701, &UA567104},
	{SG86702, &UA2021014},
	{SG86703, &UA643046},
	{SG86710, &UA842519},
	{SG86711, &UA447440},
	{SG86712, &UA1839776},
	{SG86715, &UA2767242},
	{SG86725, &UA840345},
	{SG86726, &UA2391961},
	{SG86734, &UA1895662},
	{SG86735, &UA2401594},
	{SG86740, &UA649707},
	{SG86741, &UA2018614},
	{SG86742, &UA2374278},
	{SG86744, &UA1940215},
	{SG86745, &UA2216662},
	{SG86746, &UA2720897},
	{SG86747, &UA2742987},
	{SG86759, &UA2484179},
	{SG86760, &UA2594743},
	{SG86764, &UA1894815},
	{SG86765, &UA2517538},
	{SG86767, &UA2148434},
	{SG86768, &UA2411656},
	{SG86771, &UA858492},
	{SG86773, &UA2487314},
	{SG86774, &UA2643756},
	{SG86775, &UA3117839},
	{SG86778, &UA2361804},
	{SG86779, &UA2598783},
	{SG86780, &UA831495},
	{SG86785, &UA151772},
	{SG86787, &UA2161440},
	{SG86793, &UA759372},
	{SG86794, &UA2024207},
	{SG86795, &UA2029334},
	{SG86796, &UA2203461},
	{SG86799, &UA821186},
	{SG86800, &UA573511},
	{SG86802, &UA634198},
	{SG86803, &UA2020075},
	{SG86804, &UA743199},
	{SG86805, &UA746166},
	{SG86809, &UA2594312},
	{SG86811, &UA2467244},
	{SG86812, &UA2546345},
	{SG86813, &UA186754},
	{SG86825, &UA2329760},
	{SG86828, &UA2580947},
	{SG86833, &UA451301},
	{SG86835, &UA868333},
	{SG86836, &UA2307224},
	{SG86837, &UA2658402},
	{SG86838, &UA3044817},
	{SG86839, &UA433726},
	{SG86840, &UA2848157},
	{SG86841, &UA2282588},
	{SG86843, &UA526411},
	{SG86844, &UA3069278},
	{SG86845, &UA1925700},
	{SG86846, &UA2493381},
	{SG86848, &UA436233},
	{SG86849, &UA770632},
	{SG86850, &UA1899933},
	{SG86851, &UA2212800},
	{SG86852, &UA2455074},
	{SG86855, &UA1900944},
	{SG86856, &UA2843694},
	{SG86857, &UA1883631},
	{SG86858, &UA2577790},
	{SG86859, &UA2655897},
	{SG86860, &UA2786063},
	{SG86861, &UA419677},
	{SG86863, &UA2553163},
	{SG86864, &UA2596459},
	{SG86866, &UA847524},
	{SG86867, &UA2409201},
	{SG86868, &UA2652707},
	{SG86870, &UA826489},
	{SG86871, &UA1878523},
	{SG86873, &UA2502494},
	{SG86878, &UA2408580},
	{SG86879, &UA878105},
	{SG86880, &UA2408676},
	{SG86881, &UA2553572},
	{SG86882, &UA2554296},
	{SG86883, &UA2652942},
	{SG86887, &UA427277},
	{SG86888, &UA758459},
	{SG86889, &UA1935216},
	{SG86890, &UA2022333},
	{SG86891, &UA2145934},
	{SG86892, &UA2596515},
	{SG86894, &UA573876},
	{SG86895, &UA2502543},
	{SG86897, &UA2683030},
	{SG86899, &UA822525},
	{SG86900, &UA2751401},
	{SG86901, &UA79855},
	{SG86902, &UA2323187},
	{SG86904, &UA2601694},
	{SG86910, &UA2746942},
	{SG86913, &UA616403},
	{SG86914, &UA747429},
	{SG86915, &UA1902491},
	{SG86916, &UA2262843},
	{SG86917, &UA2393710},
	{SG86918, &UA2727085},
	{SG86919, &UA2789697},
	{SG86920, &UA2985881},
	{SG86923, &UA2682698},
	{SG86924, &UA3069841},
	{SG86925, &UA822452},
	{SG86926, &UA2472697},
	{SG86927, &UA2524652},
	{SG86928, &UA2695168},
	{SG86930, &UA534039},
	{SG86931, &UA2135014},
	{SG86933, &UA2726980},
	{SG86934, &UA2839354},
	{SG86936, &UA3039549},
	{SG86937, &UA3039553},
	{SG86938, &UA3070232},
	{SG86940, &UA438693},
	{SG86943, &UA2978661},
	{SG86946, &UA2019247},
	{SG86947, &UA2019955},
	{SG86948, &UA2409930},
	{SG86949, &UA2651779},
	{SG86954, &UA2220268},
	{SG86956, &UA2392621},
	{SG86957, &UA2587235},
	{SG86958, &UA2741915},
	{SG86959, &UA2804743},
	{SG86960, &UA2815520},
	{SG86963, &UA3070838},
	{SG86966, &UA2132393},
	{SG86967, &UA2540907},
	{SG86969, &UA2933032},
	{SG86973, &UA2306918},
	{SG86975, &UA2807637},
	{SG86976, &UA2295909},
	{SG86977, &UA845102},
	{SG86979, &UA866157},
	{SG86980, &UA2213632},
	{SG86981, &UA2639222},
	{SG86982, &UA2813870},
	{SG86983, &UA2208211},
	{SG86984, &UA2371083},
	{SG86986, &UA2307896},
	{SG86987, &UA616397},
	{SG86988, &UA616409},
	{SG86993, &UA2019548},
	{SG86994, &UA2021256},
	{SG86995, &UA3039778},
	{SG86996, &UA616383},
	{SG86997, &UA2029658},
	{SG86998, &UA2807355},
	{SG87001, &UA2807691},
	{SG87003, &UA2299663},
	{SG87004, &UA2977064},
	{SG87005, &UA1852166},
	{SG87006, &UA1883445},
	{SG87007, &UA2374197},
	{SG87008, &UA2977067},
	{SG87009, &UA589293},
	{SG87014, &UA469826},
	{SG87015, &UA558841},
	{SG87016, &UA2339108},
	{SG87018, &UA924431},
	{SG87019, &UA2313198},
	{SG87020, &UA2977060},
	{SG87025, &UA2977057},
	{SG87026, &UA2977063},
	{SG87027, &UA2977066},
	{SG87029, &UA535958},
	{SG87030, &UA616423},
	{SG87032, &UA2308699},
	{SG87033, &UA2977058},
	{SG87034, &UA3039769},
	{SG87035, &UA39762},
	{SG87037, &UA616389},
	{SG87038, &UA616391},
	{SG87039, &UA616406},
	{SG87040, &UA1881493},
	{SG87045, &UA3039764},
	{SG87046, &UA3039771},
	{SG87047, &UA3039770},
	{SG87048, &UA3039785},
	{SG87049, &UA79857},
	{SG87050, &UA79861},
	{NULL}};
const Segmentdata DOperaMobi[] = {
	{SG83656, &UA129787},
	{SG83657, &UA2547338},
	{SG83675, &UA23036},
	{SG83962, &UA854393},
	{SG83963, &UA2173798},
	{SG83964, &UA2383418},
	{SG83965, &UA2455980},
	{SG83976, &UA2505358},
	{SG83977, &UA2527395},
	{SG83978, &UA2547824},
	{SG83979, &UA159097},
	{SG84298, &UA686199},
	{SG84417, &UA2314379},
	{SG84663, &UA2235000},
	{SG84664, &UA2367409},
	{SG84912, &UA576656},
	{SG84947, &UA88703},
	{SG84948, &UA103909},
	{SG85000, &UA464796},
	{SG85122, &UA624125},
	{SG85123, &UA1934712},
	{SG85315, &UA2531156},
	{SG85316, &UA2557429},
	{SG85332, &UA616189},
	{SG85412, &UA2139969},
	{SG85535, &UA2237745},
	{SG85584, &UA90798},
	{SG85745, &UA652795},
	{SG85895, &UA1997816},
	{SG86214, &UA1842558},
	{SG86227, &UA2384234},
	{SG86474, &UA1889608},
	{SG86509, &UA2558740},
	{SG86519, &UA2425299},
	{SG86532, &UA2148882},
	{SG86533, &UA2370944},
	{SG86534, &UA2409834},
	{SG86535, &UA2517894},
	{SG86536, &UA3041672},
	{SG86602, &UA440418},
	{SG86603, &UA556027},
	{SG86664, &UA1988536},
	{SG86680, &UA346287},
	{SG86681, &UA261606},
	{SG86713, &UA117838},
	{SG86724, &UA2307907},
	{SG86755, &UA179450},
	{SG86822, &UA550619},
	{SG86854, &UA1986422},
	{SG86862, &UA865883},
	{SG86869, &UA207086},
	{SG86876, &UA2410499},
	{SG86903, &UA837001},
	{SG86906, &UA2527398},
	{SG86907, &UA2554623},
	{SG86909, &UA1883919},
	{SG86929, &UA545313},
	{SG86942, &UA512055},
	{SG86945, &UA2890},
	{SG86950, &UA2210612},
	{SG86951, &UA2303256},
	{SG86972, &UA1868150},
	{SG86978, &UA729998},
	{SG86990, &UA623400},
	{SG86991, &UA881234},
	{SG87002, &UA1908646},
	{SG87010, &UA792112},
	{SG87011, &UA1868593},
	{SG87017, &UA2139762},
	{SG87021, &UA207014},
	{SG87022, &UA2158779},
	{SG87023, &UA533441},
	{SG87024, &UA744649},
	{SG87028, &UA1890834},
	{SG87031, &UA520979},
	{SG87036, &UA90311},
	{SG87041, &UA15619},
	{SG87042, &UA1858432},
	{SG87043, &UA257586},
	{SG87044, &UA322672},
	{SG87051, &UA294826},
	{SG87052, &UA2297939},
	{NULL}};
const Device *DPalm[] = {
	&UA3649,
	&UA30906,
	&UA28393,
	&UA28394,
	&UA28395,
	&UA5050,
	&UA20723,
	&UA24371,
	&UA54952,
	&UA20089,
	&UA2389,
	&UA20090,
	&UA20091,
	&UA2472765,
	&UA2805396,
	&UA2978263,
	&UA20088,
	&UA71388,
	&UA39838,
	NULL};
const Device *DPanasonic[] = {
	&UA22632,
	&UA60789,
	&UA18472,
	&UA84453,
	&UA84485,
	&UA60719,
	&UA84451,
	&UA84490,
	&UA21903,
	&UA22619,
	&UA21370,
	NULL};
const Device *DPantech[] = {
	&UA50248,
	&UA56117,
	&UA67321,
	&UA68863,
	&UA2146935,
	&UA2977759,
	&UA50370,
	&UA67333,
	&UA627166,
	&UA2219901,
	&UA50273,
	&UA50284,
	&UA2977423,
	&UA69307,
	&UA2148041,
	&UA903567,
	&UA49222,
	&UA679177,
	&UA50314,
	&UA508886,
	&UA50277,
	&UA547731,
	&UA23346,
	&UA38521,
	&UA60524,
	&UA32200,
	&UA814793,
	&UA50290,
	&UA445569,
	&UA38520,
	&UA497379,
	&UA30368,
	&UA31168,
	&UA26741,
	&UA446126,
	&UA64404,
	&UA26456,
	&UA26459,
	&UA24130,
	&UA30902,
	&UA460439,
	&UA866011,
	&UA9553,
	&UA49137,
	&UA67615,
	&UA445639,
	&UA2040791,
	&UA678452,
	&UA60733,
	&UA1885286,
	&UA21661,
	&UA30829,
	&UA54023,
	&UA59313,
	&UA21663,
	&UA22297,
	&UA85249,
	&UA23804,
	&UA525989,
	&UA61061,
	&UA85342,
	&UA43380,
	&UA88983,
	&UA85340,
	&UA89090,
	&UA85300,
	&UA85294,
	&UA61229,
	&UA61364,
	&UA85269,
	&UA85272,
	&UA85322,
	&UA85339,
	NULL};
const Device *DPhilips[] = {
	&UA2743266,
	&UA2336994,
	&UA20156,
	&UA22600,
	&UA24609,
	&UA84385,
	NULL};
const Device *DPortalmmm[] = {
	&UA85717,
	&UA22334,
	&UA85581,
	&UA85670,
	&UA85578,
	&UA85663,
	&UA85724,
	&UA85606,
	&UA85701,
	NULL};
const Device *DQtek[] = {
	&UA88170,
	NULL};
const Segmentdata DSafari[] = {
	{SG87432, &UA195639},
	{SG87434, &UA205000},
	{SG87442, &UA378522},
	{SG87444, &UA409238},
	{SG87446, &UA409489},
	{SG87448, &UA419815},
	{SG87450, &UA436297},
	{SG87451, &UA451495},
	{SG87463, &UA708743},
	{SG87464, &UA771171},
	{SG87465, &UA792347},
	{SG87466, &UA809660},
	{SG87467, &UA843772},
	{SG87468, &UA1895072},
	{SG87469, &UA2141648},
	{SG87470, &UA2301197},
	{SG87471, &UA2310835},
	{SG87472, &UA2408920},
	{SG87473, &UA2473135},
	{SG87489, &UA89802},
	{SG87973, &UA756504},
	{SG88017, &UA178246},
	{SG88021, &UA238811},
	{SG88023, &UA239195},
	{SG88024, &UA250685},
	{SG88027, &UA301338},
	{SG88030, &UA327586},
	{SG88033, &UA352126},
	{SG88034, &UA359505},
	{SG88035, &UA363613},
	{SG88194, &UA746383},
	{SG88195, &UA749686},
	{SG88196, &UA789452},
	{SG88197, &UA855227},
	{SG88198, &UA1903828},
	{SG88199, &UA1941790},
	{SG88200, &UA2258079},
	{SG88201, &UA2474078},
	{SG88365, &UA2167820},
	{SG88368, &UA2217803},
	{SG88381, &UA2416620},
	{SG88384, &UA2988138},
	{SG89130, &UA771021},
	{SG89131, &UA2149112},
	{SG89140, &UA575024},
	{SG89141, &UA726575},
	{SG89142, &UA735238},
	{SG89143, &UA767992},
	{SG89144, &UA892031},
	{SG89145, &UA1906343},
	{SG89146, &UA1922094},
	{SG89147, &UA1968567},
	{SG89148, &UA2145722},
	{SG89149, &UA2221838},
	{SG89150, &UA3069652},
	{SG89153, &UA2983721},
	{SG89163, &UA796091},
	{SG89164, &UA1961505},
	{SG89165, &UA1974250},
	{SG89166, &UA1994075},
	{SG89167, &UA2160749},
	{SG89168, &UA2473962},
	{SG89169, &UA2482732},
	{SG89170, &UA2555660},
	{SG89171, &UA2695284},
	{SG89172, &UA2751054},
	{SG89173, &UA2997092},
	{SG89174, &UA3099686},
	{SG89175, &UA2163060},
	{SG89176, &UA2321933},
	{SG89177, &UA2413817},
	{SG89181, &UA2546384},
	{SG89183, &UA2981910},
	{SG89185, &UA3042917},
	{SG89202, &UA396507},
	{SG89203, &UA797230},
	{SG89204, &UA2594472},
	{SG89205, &UA2726530},
	{SG89206, &UA2759448},
	{SG89207, &UA2843871},
	{SG89253, &UA2393208},
	{SG89254, &UA2855989},
	{SG89255, &UA2889150},
	{SG89256, &UA2980372},
	{SG89257, &UA2980804},
	{SG89258, &UA1990268},
	{SG89259, &UA2364782},
	{SG89260, &UA2394733},
	{SG89261, &UA2722281},
	{SG89262, &UA2758819},
	{SG89279, &UA661732},
	{SG89401, &UA755463},
	{SG89404, &UA1898694},
	{SG89409, &UA2221822},
	{SG89411, &UA2310439},
	{SG89413, &UA2411442},
	{SG89417, &UA2445774},
	{SG89491, &UA2570439},
	{SG89492, &UA2570876},
	{SG89493, &UA2584178},
	{SG89494, &UA2585562},
	{SG89495, &UA2604055},
	{SG89496, &UA2604102},
	{SG89497, &UA2604147},
	{SG89498, &UA2670013},
	{SG89499, &UA2671187},
	{SG89500, &UA2672617},
	{SG89501, &UA2674136},
	{SG89502, &UA2674316},
	{SG89503, &UA2674382},
	{SG89504, &UA2675774},
	{SG89505, &UA2677612},
	{SG89506, &UA2677756},
	{SG89507, &UA2677968},
	{SG89508, &UA2709286},
	{SG89509, &UA2712710},
	{SG89510, &UA2714588},
	{SG89511, &UA2715196},
	{SG89512, &UA2716485},
	{SG89513, &UA2716716},
	{SG89514, &UA2718728},
	{SG89515, &UA2718780},
	{SG89516, &UA2719290},
	{SG89517, &UA2737358},
	{SG89518, &UA2739195},
	{SG89519, &UA2739607},
	{SG89520, &UA2739848},
	{SG89521, &UA2741622},
	{SG89522, &UA2741722},
	{SG89523, &UA2753358},
	{SG89524, &UA2754166},
	{SG89525, &UA2755390},
	{SG89526, &UA2756707},
	{SG89527, &UA2769827},
	{SG89528, &UA2769910},
	{SG89529, &UA2770448},
	{SG89530, &UA2784158},
	{SG89531, &UA2784822},
	{SG89532, &UA2784949},
	{SG89533, &UA2799421},
	{SG89534, &UA2800690},
	{SG89535, &UA2861613},
	{SG89536, &UA2875856},
	{SG89537, &UA2876437},
	{SG89538, &UA2877612},
	{SG89539, &UA2886367},
	{SG89540, &UA2894110},
	{SG89541, &UA2897660},
	{SG89542, &UA2899420},
	{SG89543, &UA2943480},
	{SG89544, &UA2944162},
	{SG89545, &UA2966888},
	{SG89546, &UA2968446},
	{SG89547, &UA2971913},
	{SG89548, &UA2974001},
	{SG89549, &UA2974758},
	{SG89550, &UA2998618},
	{SG89551, &UA2999770},
	{SG89552, &UA3012706},
	{SG89553, &UA3031256},
	{SG89554, &UA3039089},
	{SG89555, &UA3088645},
	{SG89556, &UA3107705},
	{SG89557, &UA3121351},
	{SG89558, &UA2009439},
	{SG89559, &UA2173916},
	{SG89560, &UA2235232},
	{SG89561, &UA2256932},
	{SG89562, &UA2256961},
	{SG89563, &UA2354062},
	{SG89564, &UA2357847},
	{SG89565, &UA2359510},
	{SG89566, &UA2368256},
	{SG89567, &UA2368536},
	{SG89568, &UA2387673},
	{SG89569, &UA2387796},
	{SG89570, &UA2404502},
	{SG89571, &UA2406156},
	{SG89572, &UA2406953},
	{SG89573, &UA2408244},
	{SG89574, &UA2426348},
	{SG89575, &UA2429926},
	{SG89576, &UA2431750},
	{SG89577, &UA2431818},
	{SG89578, &UA2432846},
	{SG89579, &UA2433985},
	{SG89580, &UA2461862},
	{SG89581, &UA2461993},
	{SG89582, &UA2462131},
	{SG89583, &UA2470227},
	{SG89584, &UA2470801},
	{SG89585, &UA2470953},
	{SG89586, &UA2471706},
	{SG89587, &UA2487981},
	{SG89588, &UA2488283},
	{SG89589, &UA2490609},
	{SG89590, &UA2492599},
	{SG89591, &UA2492616},
	{SG89592, &UA2508510},
	{SG89593, &UA2509154},
	{SG89594, &UA2512188},
	{SG89595, &UA2513068},
	{SG89596, &UA2515428},
	{SG89597, &UA2531886},
	{SG89598, &UA2535174},
	{SG89599, &UA2536154},
	{SG89600, &UA2539847},
	{SG89601, &UA2564732},
	{SG89602, &UA2565236},
	{SG89603, &UA2571382},
	{SG89604, &UA2580818},
	{SG89605, &UA2583058},
	{SG89606, &UA2583116},
	{SG89607, &UA2604451},
	{SG89608, &UA2657963},
	{SG89609, &UA2667234},
	{SG89610, &UA2667886},
	{SG89611, &UA2692131},
	{SG89612, &UA2706450},
	{SG89613, &UA2837108},
	{SG89614, &UA2853237},
	{SG89615, &UA3044529},
	{SG89616, &UA3118980},
	{SG89617, &UA255683},
	{SG89618, &UA453159},
	{SG89619, &UA572693},
	{SG89620, &UA626753},
	{SG89621, &UA2833886},
	{SG89630, &UA500801},
	{SG89631, &UA719967},
	{SG89632, &UA2540638},
	{SG89633, &UA2540640},
	{SG89634, &UA2540639},
	{SG89635, &UA2540641},
	{SG89636, &UA2540642},
	{SG89637, &UA2540643},
	{SG89638, &UA2540644},
	{SG89639, &UA2540645},
	{SG89640, &UA2540647},
	{SG89641, &UA2540646},
	{SG89642, &UA2540648},
	{SG89643, &UA2540649},
	{SG89644, &UA2540650},
	{SG89645, &UA2540651},
	{SG89646, &UA2540652},
	{SG89647, &UA2540653},
	{SG89648, &UA2540654},
	{SG89649, &UA2540655},
	{SG89650, &UA2540656},
	{SG89651, &UA2540657},
	{SG89652, &UA2540658},
	{SG89653, &UA2540659},
	{SG89654, &UA2540660},
	{SG89655, &UA2540661},
	{SG89656, &UA2540662},
	{SG89657, &UA2540664},
	{SG89658, &UA2540666},
	{SG89659, &UA2540667},
	{SG89660, &UA2540668},
	{SG89661, &UA2540669},
	{SG89662, &UA2540670},
	{SG89663, &UA2540671},
	{SG89664, &UA2540672},
	{SG89665, &UA2540673},
	{SG89666, &UA2540674},
	{SG89667, &UA2540675},
	{SG89668, &UA2540676},
	{SG89669, &UA2540677},
	{SG89670, &UA2540678},
	{SG89671, &UA2540679},
	{SG89672, &UA2540680},
	{SG89673, &UA2540681},
	{SG89674, &UA2540682},
	{SG89675, &UA2540663},
	{SG89676, &UA2567689},
	{SG89677, &UA2585424},
	{SG89678, &UA2601990},
	{SG89679, &UA2606642},
	{SG89680, &UA2674997},
	{SG89681, &UA2675420},
	{SG89682, &UA2676135},
	{SG89683, &UA2680697},
	{SG89684, &UA2710033},
	{SG89685, &UA2713392},
	{SG89686, &UA2715239},
	{SG89687, &UA2716489},
	{SG89688, &UA2716710},
	{SG89689, &UA2718787},
	{SG89690, &UA2719278},
	{SG89691, &UA2738158},
	{SG89692, &UA2739855},
	{SG89693, &UA2740960},
	{SG89694, &UA2754672},
	{SG89695, &UA2754824},
	{SG89696, &UA2769718},
	{SG89697, &UA2781468},
	{SG89698, &UA2783703},
	{SG89699, &UA2784040},
	{SG89700, &UA2784161},
	{SG89701, &UA2796902},
	{SG89702, &UA2797562},
	{SG89703, &UA2800591},
	{SG89704, &UA2800694},
	{SG89705, &UA2863948},
	{SG89706, &UA2865199},
	{SG89707, &UA2869765},
	{SG89708, &UA2870164},
	{SG89709, &UA2872527},
	{SG89710, &UA2873632},
	{SG89711, &UA2876352},
	{SG89712, &UA2885078},
	{SG89713, &UA2894630},
	{SG89714, &UA2899272},
	{SG89715, &UA2943973},
	{SG89716, &UA2946877},
	{SG89717, &UA2957129},
	{SG89718, &UA2957316},
	{SG89719, &UA2960361},
	{SG89720, &UA2966191},
	{SG89721, &UA2971097},
	{SG89722, &UA2971146},
	{SG89723, &UA2972574},
	{SG89724, &UA2976080},
	{SG89725, &UA2997801},
	{SG89726, &UA3022366},
	{SG89727, &UA3033717},
	{SG89728, &UA3037398},
	{SG89844, &UA193980},
	{SG89846, &UA202914},
	{SG89847, &UA208562},
	{SG89849, &UA209153},
	{SG89851, &UA212798},
	{SG89853, &UA212845},
	{SG89855, &UA212849},
	{SG89857, &UA214206},
	{SG89858, &UA218939},
	{SG89859, &UA220456},
	{SG89860, &UA224528},
	{SG89861, &UA227921},
	{SG89862, &UA250437},
	{SG89863, &UA251537},
	{SG89864, &UA251540},
	{SG89865, &UA251550},
	{SG89866, &UA251551},
	{SG89867, &UA251568},
	{SG89868, &UA251570},
	{SG89869, &UA251573},
	{SG89870, &UA251712},
	{SG89871, &UA251715},
	{SG89872, &UA251881},
	{SG89873, &UA332348},
	{SG89874, &UA333320},
	{SG89875, &UA333666},
	{SG89876, &UA334105},
	{SG89877, &UA334405},
	{SG89878, &UA366793},
	{SG89879, &UA366809},
	{SG89880, &UA367111},
	{SG89881, &UA367889},
	{SG89882, &UA368266},
	{SG89883, &UA368281},
	{SG89884, &UA380415},
	{SG89885, &UA396900},
	{SG89886, &UA396934},
	{SG89887, &UA622538},
	{SG89888, &UA629021},
	{SG89889, &UA629613},
	{SG89890, &UA630484},
	{SG89891, &UA636164},
	{SG89892, &UA647300},
	{SG89893, &UA679908},
	{SG89894, &UA681064},
	{SG89895, &UA698256},
	{SG89896, &UA725134},
	{SG89897, &UA727050},
	{SG89898, &UA756818},
	{SG89899, &UA764189},
	{SG89900, &UA768906},
	{SG89901, &UA769118},
	{SG89902, &UA780868},
	{SG89903, &UA788602},
	{SG89904, &UA2217049},
	{SG89905, &UA2260481},
	{SG89946, &UA2601063},
	{SG89947, &UA2602311},
	{SG89948, &UA444934},
	{SG89949, &UA756643},
	{SG89996, &UA739919},
	{SG89997, &UA2469473},
	{SG89998, &UA2567833},
	{SG90047, &UA1996810},
	{SG90048, &UA2221377},
	{SG90052, &UA89072},
	{SG90053, &UA89706},
	{SG90054, &UA100825},
	{SG90055, &UA125552},
	{SG90056, &UA130269},
	{SG90057, &UA141816},
	{SG90058, &UA145271},
	{SG90059, &UA202354},
	{SG90060, &UA207089},
	{SG90061, &UA262529},
	{SG90062, &UA281714},
	{SG90063, &UA284887},
	{SG90064, &UA286740},
	{SG90065, &UA506986},
	{SG90066, &UA748310},
	{SG90067, &UA811062},
	{SG90068, &UA2285966},
	{SG90069, &UA2299571},
	{SG90077, &UA2769909},
	{SG90078, &UA2799179},
	{SG90079, &UA2799398},
	{SG90080, &UA2870129},
	{SG90081, &UA2873993},
	{SG90082, &UA2876607},
	{SG90083, &UA2879108},
	{SG90084, &UA2879715},
	{SG90085, &UA2885275},
	{SG90086, &UA2887023},
	{SG90087, &UA2888654},
	{SG90088, &UA2945533},
	{SG90089, &UA2948014},
	{SG90090, &UA2948413},
	{SG90091, &UA2972881},
	{SG90092, &UA2973806},
	{SG90093, &UA2976829},
	{SG90094, &UA3024923},
	{SG90095, &UA3025635},
	{SG90096, &UA3028257},
	{SG90097, &UA3029783},
	{SG90098, &UA3030867},
	{SG90099, &UA3036012},
	{SG90100, &UA3036054},
	{SG90101, &UA3036327},
	{SG90102, &UA3036422},
	{SG90103, &UA3037095},
	{SG90104, &UA3039379},
	{SG90105, &UA3050322},
	{SG90106, &UA3061049},
	{SG90107, &UA3066106},
	{SG90108, &UA3067696},
	{SG90109, &UA3068144},
	{SG90110, &UA3068184},
	{SG90111, &UA3080790},
	{SG90112, &UA3080879},
	{SG90113, &UA3082206},
	{SG90114, &UA3084015},
	{SG90115, &UA3092465},
	{SG90116, &UA3096093},
	{SG90117, &UA3096166},
	{SG90118, &UA3096908},
	{SG90119, &UA3109173},
	{SG90120, &UA3110073},
	{SG90121, &UA3113578},
	{SG90122, &UA3127913},
	{SG90123, &UA3129719},
	{SG90124, &UA3131726},
	{SG90125, &UA3131811},
	{SG90181, &UA2706475},
	{SG90182, &UA2717122},
	{SG90185, &UA247672},
	{SG90186, &UA382056},
	{SG90187, &UA623017},
	{SG90188, &UA2205261},
	{SG90189, &UA2258602},
	{SG90204, &UA88475},
	{SG90205, &UA88713},
	{SG90206, &UA88965},
	{SG90207, &UA89113},
	{SG90208, &UA89271},
	{SG90209, &UA89868},
	{SG90210, &UA90016},
	{SG90211, &UA114228},
	{SG90212, &UA120926},
	{SG90213, &UA122238},
	{SG90214, &UA125954},
	{SG90215, &UA136040},
	{SG90216, &UA140286},
	{SG90217, &UA265558},
	{SG90218, &UA269578},
	{SG90219, &UA269614},
	{SG90220, &UA305755},
	{SG90221, &UA459530},
	{SG90222, &UA459979},
	{SG90223, &UA521006},
	{SG90224, &UA525553},
	{SG90225, &UA548799},
	{SG90226, &UA1851424},
	{SG90227, &UA1895309},
	{SG90228, &UA2258631},
	{SG90229, &UA2264294},
	{SG90230, &UA2270532},
	{SG90231, &UA2339364},
	{SG90232, &UA2801515},
	{SG90287, &UA158605},
	{SG90288, &UA686335},
	{SG90289, &UA894816},
	{SG90290, &UA2814090},
	{SG90291, &UA23040},
	{SG90292, &UA87657},
	{SG90293, &UA87716},
	{SG90294, &UA134735},
	{SG90299, &UA2364808},
	{SG90300, &UA2387608},
	{SG90342, &UA81115},
	{SG90344, &UA238802},
	{SG90356, &UA59163},
	{SG90357, &UA250354},
	{SG90358, &UA2155903},
	{SG90359, &UA2192467},
	{SG90360, &UA2302605},
	{SG90361, &UA2309095},
	{SG90362, &UA90205},
	{SG90363, &UA142938},
	{SG90364, &UA258100},
	{SG90365, &UA258110},
	{SG90366, &UA258139},
	{SG90367, &UA258141},
	{SG90368, &UA258162},
	{SG90369, &UA258166},
	{SG90370, &UA258171},
	{SG90371, &UA258222},
	{SG90372, &UA258242},
	{SG90373, &UA258247},
	{SG90374, &UA258249},
	{SG90375, &UA258260},
	{SG90376, &UA258348},
	{SG90377, &UA258240},
	{SG90378, &UA258360},
	{SG90379, &UA258437},
	{SG90380, &UA258470},
	{SG90381, &UA258495},
	{SG90382, &UA258582},
	{SG90383, &UA258807},
	{SG90384, &UA258879},
	{SG90385, &UA309071},
	{SG90386, &UA505996},
	{SG90387, &UA551976},
	{SG90388, &UA671826},
	{SG90389, &UA730215},
	{SG90390, &UA835346},
	{SG90391, &UA1845492},
	{SG90392, &UA2128822},
	{SG90393, &UA2142587},
	{SG90394, &UA2142615},
	{SG90395, &UA2153960},
	{SG90396, &UA2284599},
	{SG90397, &UA2374946},
	{SG90398, &UA2375681},
	{SG90399, &UA2582728},
	{SG90400, &UA2802117},
	{SG90409, &UA90414},
	{SG90410, &UA181743},
	{SG90411, &UA213157},
	{SG90413, &UA2870011},
	{SG90414, &UA2895613},
	{SG90415, &UA2898722},
	{SG90416, &UA2943722},
	{SG90417, &UA2973130},
	{SG90418, &UA2973807},
	{SG90419, &UA2997652},
	{SG90420, &UA3005151},
	{SG90421, &UA3024338},
	{SG90422, &UA3024383},
	{SG90423, &UA3026337},
	{SG90424, &UA3032329},
	{SG90425, &UA3036318},
	{SG90426, &UA3037101},
	{SG90427, &UA3037270},
	{SG90428, &UA3039440},
	{SG90429, &UA3058772},
	{SG90430, &UA3068025},
	{SG90431, &UA3068185},
	{SG90432, &UA3082216},
	{SG90433, &UA3083838},
	{SG90434, &UA3094868},
	{SG90435, &UA3096168},
	{SG90436, &UA3096699},
	{SG90437, &UA3110819},
	{SG90438, &UA3114345},
	{SG90439, &UA3129008},
	{SG90459, &UA2708245},
	{SG90460, &UA176864},
	{SG90468, &UA2356770},
	{SG90486, &UA456218},
	{SG90487, &UA602157},
	{SG90508, &UA88559},
	{SG90509, &UA1841244},
	{SG90606, &UA8479},
	{SG90607, &UA8478},
	{SG90608, &UA9822},
	{SG90609, &UA9823},
	{SG90610, &UA9824},
	{SG90611, &UA64705},
	{SG90643, &UA2715021},
	{SG90644, &UA2715028},
	{SG90645, &UA2715038},
	{SG90674, &UA378515},
	{SG90675, &UA408936},
	{SG90676, &UA459726},
	{SG90677, &UA469912},
	{SG90678, &UA659143},
	{SG90679, &UA717794},
	{SG90680, &UA720758},
	{SG90681, &UA742493},
	{SG90682, &UA770332},
	{SG90683, &UA793683},
	{SG90684, &UA812146},
	{SG90685, &UA1839978},
	{SG90686, &UA1866480},
	{SG90687, &UA1899190},
	{SG90688, &UA2439433},
	{SG90732, &UA1688},
	{SG90733, &UA1689},
	{SG90734, &UA2235},
	{SG90735, &UA17277},
	{SG90736, &UA17279},
	{SG90737, &UA17278},
	{SG90738, &UA17280},
	{SG90739, &UA18750},
	{SG90740, &UA18751},
	{SG90741, &UA18752},
	{SG90742, &UA18753},
	{SG90743, &UA18754},
	{SG90744, &UA18755},
	{SG90745, &UA18756},
	{SG90746, &UA18757},
	{SG90747, &UA18759},
	{SG90748, &UA18760},
	{SG90749, &UA18761},
	{SG90750, &UA18762},
	{SG90751, &UA18763},
	{SG90752, &UA18764},
	{SG90753, &UA18758},
	{SG90754, &UA37421},
	{SG90755, &UA39195},
	{SG90756, &UA44848},
	{SG90757, &UA50735},
	{SG90758, &UA54990},
	{SG90759, &UA62397},
	{SG90760, &UA62815},
	{SG90761, &UA62881},
	{SG90762, &UA653872},
	{SG90765, &UA2715019},
	{SG90766, &UA2716481},
	{SG90767, &UA650},
	{SG90768, &UA651},
	{SG90769, &UA693},
	{SG90770, &UA709},
	{SG90771, &UA711},
	{SG90772, &UA712},
	{SG90773, &UA713},
	{SG90774, &UA714},
	{SG90775, &UA15969},
	{SG90776, &UA15970},
	{SG90777, &UA15971},
	{SG90778, &UA15972},
	{SG90779, &UA15973},
	{SG90780, &UA15975},
	{SG90781, &UA15976},
	{SG90782, &UA15978},
	{SG90783, &UA15979},
	{SG90784, &UA15980},
	{SG90785, &UA15981},
	{SG90786, &UA15974},
	{SG90787, &UA18433},
	{SG90788, &UA18434},
	{SG90789, &UA31847},
	{SG90790, &UA31848},
	{SG90791, &UA31849},
	{SG90792, &UA31850},
	{SG90793, &UA31860},
	{SG90794, &UA31861},
	{SG90795, &UA35301},
	{SG90796, &UA53848},
	{SG90797, &UA53849},
	{SG90798, &UA53982},
	{SG90799, &UA54190},
	{SG90800, &UA54586},
	{SG90801, &UA62171},
	{SG90802, &UA62246},
	{SG90803, &UA62443},
	{SG90804, &UA62447},
	{SG90805, &UA63646},
	{SG90806, &UA63896},
	{SG90807, &UA86682},
	{SG90808, &UA86977},
	{SG90823, &UA420175},
	{SG90824, &UA557545},
	{SG90825, &UA574679},
	{SG90826, &UA625731},
	{SG90827, &UA709214},
	{SG90828, &UA2804572},
	{SG90829, &UA201065},
	{SG90831, &UA2734492},
	{SG90832, &UA64093},
	{SG90833, &UA81124},
	{SG90864, &UA2580256},
	{SG90865, &UA2666896},
	{SG90866, &UA2935926},
	{SG90867, &UA3057521},
	{SG90868, &UA3086691},
	{SG90880, &UA2207233},
	{SG90892, &UA230099},
	{SG90893, &UA316899},
	{SG90894, &UA334475},
	{SG90895, &UA402730},
	{SG90896, &UA402752},
	{SG90897, &UA402783},
	{SG90898, &UA402842},
	{SG90899, &UA402881},
	{SG90900, &UA402864},
	{SG90901, &UA403084},
	{SG90902, &UA403095},
	{SG90903, &UA403285},
	{SG90905, &UA158639},
	{SG90906, &UA1858439},
	{SG90907, &UA2804803},
	{SG90916, &UA2236852},
	{SG90923, &UA5293},
	{SG90924, &UA6458},
	{SG90925, &UA22328},
	{SG90926, &UA28385},
	{SG90927, &UA28387},
	{SG90928, &UA28388},
	{SG90929, &UA28386},
	{SG90930, &UA36947},
	{SG90931, &UA45915},
	{SG90932, &UA46509},
	{SG90933, &UA46508},
	{SG90934, &UA51028},
	{SG90935, &UA59815},
	{SG90936, &UA61825},
	{SG90937, &UA61824},
	{SG90938, &UA62583},
	{SG90939, &UA62868},
	{SG90940, &UA64823},
	{SG90941, &UA69556},
	{SG90942, &UA88550},
	{SG90953, &UA1882291},
	{SG90954, &UA2155013},
	{SG90955, &UA2423290},
	{SG90956, &UA3016245},
	{SG90965, &UA4209},
	{SG90966, &UA4210},
	{SG90967, &UA6134},
	{SG90968, &UA20721},
	{SG90969, &UA24122},
	{SG90970, &UA24123},
	{SG90971, &UA24320},
	{SG90972, &UA31125},
	{SG90973, &UA31127},
	{SG90974, &UA31126},
	{SG90975, &UA31128},
	{SG90976, &UA31130},
	{SG90977, &UA31131},
	{SG90978, &UA31132},
	{SG90979, &UA36500},
	{SG90980, &UA37375},
	{SG90981, &UA37376},
	{SG90982, &UA37377},
	{SG90983, &UA45055},
	{SG90984, &UA50701},
	{SG90985, &UA50703},
	{SG90986, &UA50702},
	{SG90987, &UA50704},
	{SG90988, &UA55034},
	{SG90989, &UA62241},
	{SG90990, &UA64139},
	{SG90991, &UA64580},
	{SG91011, &UA2456385},
	{SG91023, &UA5998},
	{SG91024, &UA24538},
	{SG91025, &UA30812},
	{SG91026, &UA30813},
	{SG91027, &UA30814},
	{SG91028, &UA30815},
	{SG91029, &UA36478},
	{SG91030, &UA62917},
	{SG91054, &UA2977460},
	{SG91078, &UA692208},
	{SG91079, &UA2333345},
	{SG91080, &UA2334722},
	{SG91081, &UA2728110},
	{SG91082, &UA3020918},
	{SG91086, &UA90612},
	{SG91087, &UA290073},
	{SG91101, &UA2160947},
	{SG91117, &UA599485},
	{SG91121, &UA64175},
	{SG91122, &UA72571},
	{SG91129, &UA7538},
	{SG91130, &UA64491},
	{SG91132, &UA715233},
	{SG91133, &UA6266},
	{SG91134, &UA43772},
	{SG91135, &UA43773},
	{SG91136, &UA56312},
	{SG91137, &UA56314},
	{SG91138, &UA62914},
	{SG91139, &UA718799},
	{SG91185, &UA2934898},
	{SG91186, &UA896989},
	{SG91192, &UA31171},
	{SG91193, &UA31172},
	{SG91204, &UA570492},
	{SG91205, &UA749792},
	{SG91209, &UA3042252},
	{SG91212, &UA2455672},
	{SG91225, &UA2540665},
	{SG91252, &UA2775721},
	{SG91266, &UA49435},
	{SG91267, &UA347091},
	{SG91268, &UA347092},
	{SG91282, &UA3039594},
	{SG91283, &UA3039595},
	{SG91287, &UA50986},
	{SG91288, &UA56919},
	{SG91290, &UA3039646},
	{SG91291, &UA3039652},
	{SG91295, &UA3039586},
	{SG91296, &UA3039589},
	{SG91297, &UA3039625},
	{SG91298, &UA3039635},
	{SG91299, &UA33173},
	{SG91300, &UA86683},
	{SG91304, &UA3039603},
	{SG91305, &UA20645},
	{SG91306, &UA90593},
	{SG91307, &UA3039584},
	{SG91308, &UA3039602},
	{SG91309, &UA3039605},
	{SG91310, &UA3039633},
	{SG91311, &UA3039582},
	{SG91312, &UA3039562},
	{SG91313, &UA3039569},
	{SG91314, &UA3039565},
	{SG91315, &UA3039568},
	{SG91316, &UA3039572},
	{SG91317, &UA3039647},
	{SG91318, &UA3039592},
	{SG91319, &UA3039596},
	{SG91320, &UA3039600},
	{SG91321, &UA3039623},
	{SG91322, &UA3039638},
	{SG91327, &UA3039570},
	{SG91328, &UA3039574},
	{SG91329, &UA3039578},
	{SG91330, &UA3039604},
	{SG91331, &UA3039606},
	{SG91332, &UA3039607},
	{SG91333, &UA3039612},
	{SG91334, &UA3039619},
	{SG91335, &UA3039620},
	{SG91336, &UA3039622},
	{SG91337, &UA3039630},
	{SG91338, &UA3039632},
	{SG91339, &UA3039644},
	{SG91340, &UA3039645},
	{SG91341, &UA3039657},
	{SG91344, &UA24216},
	{SG91348, &UA86737},
	{SG91349, &UA616614},
	{SG91350, &UA3039583},
	{SG91352, &UA3039556},
	{SG91353, &UA3039557},
	{SG91354, &UA3039558},
	{SG91355, &UA3039559},
	{SG91356, &UA3039560},
	{SG91357, &UA3039561},
	{SG91358, &UA3039563},
	{SG91359, &UA3039564},
	{SG91360, &UA3039567},
	{SG91361, &UA3039571},
	{SG91362, &UA3039579},
	{SG91363, &UA3039580},
	{SG91364, &UA3039588},
	{SG91365, &UA3039591},
	{SG91366, &UA3039597},
	{SG91367, &UA3039598},
	{SG91368, &UA3039608},
	{SG91369, &UA3039626},
	{SG91370, &UA3039634},
	{SG91371, &UA3039639},
	{SG91372, &UA3039648},
	{SG91373, &UA3039656},
	{SG91374, &UA3039659},
	{SG91375, &UA3039663},
	{SG91376, &UA3039670},
	{SG91377, &UA616607},
	{SG91378, &UA616609},
	{SG91379, &UA616613},
	{SG91380, &UA616616},
	{SG91381, &UA616615},
	{SG91382, &UA616610},
	{NULL}};
const Segmentdata DSafariDesktop[] = {
	{SG87053, &UA2743900},
	{SG87054, &UA2817913},
	{SG87055, &UA3098588},
	{SG87056, &UA38684},
	{SG87057, &UA69784},
	{SG87058, &UA69912},
	{SG87059, &UA101491},
	{SG87060, &UA101965},
	{SG87061, &UA102141},
	{SG87062, &UA102177},
	{SG87063, &UA102785},
	{SG87064, &UA103045},
	{SG87065, &UA103105},
	{SG87066, &UA103165},
	{SG87067, &UA103275},
	{SG87068, &UA103566},
	{SG87069, &UA103802},
	{SG87070, &UA105560},
	{SG87071, &UA105652},
	{SG87072, &UA105763},
	{SG87073, &UA106911},
	{SG87074, &UA106912},
	{SG87075, &UA107215},
	{SG87076, &UA107265},
	{SG87077, &UA107411},
	{SG87078, &UA108135},
	{SG87079, &UA110006},
	{SG87080, &UA110819},
	{SG87081, &UA112167},
	{SG87082, &UA113925},
	{SG87083, &UA118881},
	{SG87084, &UA118985},
	{SG87085, &UA119346},
	{SG87086, &UA119407},
	{SG87087, &UA119551},
	{SG87088, &UA119646},
	{SG87089, &UA119744},
	{SG87090, &UA119842},
	{SG87091, &UA120146},
	{SG87092, &UA120232},
	{SG87093, &UA120337},
	{SG87094, &UA121526},
	{SG87095, &UA122992},
	{SG87096, &UA123004},
	{SG87097, &UA123028},
	{SG87098, &UA124225},
	{SG87099, &UA124717},
	{SG87100, &UA125656},
	{SG87101, &UA126068},
	{SG87102, &UA126308},
	{SG87103, &UA131841},
	{SG87104, &UA132369},
	{SG87105, &UA132370},
	{SG87106, &UA132410},
	{SG87107, &UA132487},
	{SG87108, &UA132531},
	{SG87109, &UA132506},
	{SG87110, &UA132566},
	{SG87111, &UA132750},
	{SG87112, &UA132775},
	{SG87113, &UA132744},
	{SG87114, &UA132797},
	{SG87115, &UA132809},
	{SG87116, &UA132859},
	{SG87117, &UA132864},
	{SG87118, &UA132943},
	{SG87119, &UA133023},
	{SG87120, &UA133304},
	{SG87121, &UA133348},
	{SG87122, &UA133593},
	{SG87123, &UA133827},
	{SG87124, &UA133841},
	{SG87125, &UA133869},
	{SG87126, &UA134031},
	{SG87127, &UA134416},
	{SG87128, &UA134542},
	{SG87129, &UA134661},
	{SG87130, &UA135201},
	{SG87131, &UA135253},
	{SG87132, &UA135374},
	{SG87133, &UA136033},
	{SG87134, &UA136041},
	{SG87135, &UA136333},
	{SG87136, &UA136346},
	{SG87137, &UA137350},
	{SG87138, &UA137721},
	{SG87139, &UA138353},
	{SG87140, &UA140741},
	{SG87141, &UA141437},
	{SG87142, &UA141494},
	{SG87143, &UA143409},
	{SG87144, &UA144089},
	{SG87145, &UA148083},
	{SG87146, &UA148494},
	{SG87147, &UA150039},
	{SG87148, &UA150129},
	{SG87149, &UA150702},
	{SG87150, &UA150705},
	{SG87151, &UA151033},
	{SG87152, &UA151016},
	{SG87153, &UA151347},
	{SG87154, &UA151387},
	{SG87155, &UA151552},
	{SG87156, &UA151658},
	{SG87157, &UA152297},
	{SG87158, &UA152353},
	{SG87159, &UA152769},
	{SG87160, &UA153544},
	{SG87161, &UA153547},
	{SG87162, &UA153585},
	{SG87163, &UA153590},
	{SG87164, &UA153637},
	{SG87165, &UA154365},
	{SG87166, &UA154467},
	{SG87167, &UA154483},
	{SG87168, &UA154507},
	{SG87169, &UA154903},
	{SG87170, &UA155113},
	{SG87171, &UA155477},
	{SG87172, &UA155492},
	{SG87173, &UA155534},
	{SG87174, &UA155567},
	{SG87175, &UA156100},
	{SG87176, &UA156241},
	{SG87177, &UA156251},
	{SG87178, &UA156268},
	{SG87179, &UA156276},
	{SG87180, &UA156867},
	{SG87181, &UA157011},
	{SG87182, &UA157181},
	{SG87183, &UA157401},
	{SG87184, &UA157559},
	{SG87185, &UA157670},
	{SG87186, &UA157714},
	{SG87187, &UA157744},
	{SG87188, &UA157851},
	{SG87189, &UA157919},
	{SG87190, &UA158091},
	{SG87191, &UA158675},
	{SG87192, &UA158687},
	{SG87193, &UA159063},
	{SG87194, &UA159186},
	{SG87195, &UA159380},
	{SG87196, &UA159627},
	{SG87197, &UA159735},
	{SG87198, &UA160049},
	{SG87199, &UA160348},
	{SG87200, &UA160383},
	{SG87201, &UA160682},
	{SG87202, &UA161877},
	{SG87203, &UA163309},
	{SG87204, &UA163649},
	{SG87205, &UA165587},
	{SG87206, &UA166401},
	{SG87207, &UA166403},
	{SG87208, &UA166449},
	{SG87209, &UA166466},
	{SG87210, &UA166802},
	{SG87211, &UA166810},
	{SG87212, &UA166838},
	{SG87213, &UA166880},
	{SG87214, &UA166937},
	{SG87215, &UA167082},
	{SG87216, &UA167095},
	{SG87217, &UA167099},
	{SG87218, &UA167112},
	{SG87219, &UA167386},
	{SG87220, &UA167427},
	{SG87221, &UA167432},
	{SG87222, &UA167457},
	{SG87223, &UA167461},
	{SG87224, &UA167466},
	{SG87225, &UA167495},
	{SG87226, &UA167498},
	{SG87227, &UA167554},
	{SG87228, &UA167568},
	{SG87229, &UA167604},
	{SG87230, &UA168274},
	{SG87231, &UA168291},
	{SG87232, &UA168293},
	{SG87233, &UA168312},
	{SG87234, &UA168454},
	{SG87235, &UA168464},
	{SG87236, &UA168613},
	{SG87237, &UA168623},
	{SG87238, &UA168686},
	{SG87239, &UA168744},
	{SG87240, &UA168875},
	{SG87241, &UA169121},
	{SG87242, &UA169296},
	{SG87243, &UA170182},
	{SG87244, &UA170687},
	{SG87245, &UA171470},
	{SG87246, &UA172694},
	{SG87247, &UA173813},
	{SG87248, &UA173994},
	{SG87249, &UA174836},
	{SG87250, &UA174993},
	{SG87251, &UA175306},
	{SG87252, &UA175452},
	{SG87253, &UA175603},
	{SG87254, &UA176008},
	{SG87255, &UA176009},
	{SG87256, &UA176012},
	{SG87257, &UA176007},
	{SG87258, &UA176466},
	{SG87259, &UA176601},
	{SG87260, &UA176847},
	{SG87261, &UA176866},
	{SG87262, &UA176872},
	{SG87263, &UA176998},
	{SG87264, &UA177082},
	{SG87265, &UA177202},
	{SG87266, &UA177194},
	{SG87267, &UA177274},
	{SG87268, &UA177375},
	{SG87269, &UA177377},
	{SG87270, &UA177376},
	{SG87271, &UA177882},
	{SG87272, &UA177941},
	{SG87273, &UA177960},
	{SG87274, &UA177972},
	{SG87275, &UA177994},
	{SG87276, &UA178022},
	{SG87277, &UA178178},
	{SG87278, &UA178196},
	{SG87279, &UA178774},
	{SG87280, &UA178777},
	{SG87281, &UA179152},
	{SG87282, &UA179161},
	{SG87283, &UA180170},
	{SG87284, &UA180172},
	{SG87285, &UA180412},
	{SG87286, &UA180413},
	{SG87287, &UA180416},
	{SG87288, &UA180429},
	{SG87289, &UA180432},
	{SG87290, &UA180445},
	{SG87291, &UA180458},
	{SG87292, &UA180468},
	{SG87293, &UA180477},
	{SG87294, &UA180479},
	{SG87295, &UA180484},
	{SG87296, &UA180512},
	{SG87297, &UA180513},
	{SG87298, &UA180520},
	{SG87299, &UA180531},
	{SG87300, &UA180621},
	{SG87301, &UA181282},
	{SG87302, &UA181296},
	{SG87303, &UA181320},
	{SG87304, &UA181386},
	{SG87305, &UA181418},
	{SG87306, &UA181419},
	{SG87307, &UA181424},
	{SG87308, &UA181431},
	{SG87309, &UA181523},
	{SG87310, &UA181518},
	{SG87311, &UA181782},
	{SG87312, &UA181803},
	{SG87313, &UA181822},
	{SG87314, &UA181915},
	{SG87315, &UA182085},
	{SG87316, &UA182228},
	{SG87317, &UA182298},
	{SG87318, &UA182342},
	{SG87319, &UA182454},
	{SG87320, &UA182616},
	{SG87321, &UA182765},
	{SG87322, &UA182893},
	{SG87323, &UA183019},
	{SG87324, &UA183190},
	{SG87325, &UA183191},
	{SG87326, &UA183196},
	{SG87327, &UA183197},
	{SG87328, &UA183212},
	{SG87329, &UA183305},
	{SG87330, &UA183371},
	{SG87331, &UA183474},
	{SG87332, &UA183624},
	{SG87333, &UA183639},
	{SG87334, &UA183641},
	{SG87335, &UA183660},
	{SG87336, &UA183689},
	{SG87337, &UA183711},
	{SG87338, &UA183735},
	{SG87339, &UA183851},
	{SG87340, &UA183859},
	{SG87341, &UA183928},
	{SG87342, &UA184097},
	{SG87343, &UA184101},
	{SG87344, &UA184119},
	{SG87345, &UA184312},
	{SG87346, &UA184314},
	{SG87347, &UA184653},
	{SG87348, &UA184680},
	{SG87349, &UA184957},
	{SG87350, &UA185253},
	{SG87351, &UA185387},
	{SG87352, &UA185440},
	{SG87353, &UA186188},
	{SG87354, &UA186547},
	{SG87355, &UA187930},
	{SG87356, &UA188249},
	{SG87357, &UA188477},
	{SG87358, &UA188567},
	{SG87359, &UA188911},
	{SG87360, &UA189804},
	{SG87361, &UA190517},
	{SG87362, &UA190607},
	{SG87363, &UA193534},
	{SG87364, &UA193663},
	{SG87365, &UA193736},
	{SG87366, &UA195651},
	{SG87367, &UA195925},
	{SG87368, &UA197355},
	{SG87369, &UA197590},
	{SG87370, &UA197563},
	{SG87371, &UA197788},
	{SG87372, &UA197793},
	{SG87373, &UA197800},
	{SG87374, &UA197801},
	{SG87375, &UA197831},
	{SG87376, &UA197875},
	{SG87377, &UA197957},
	{SG87378, &UA198005},
	{SG87379, &UA198229},
	{SG87380, &UA198231},
	{SG87381, &UA198239},
	{SG87382, &UA198288},
	{SG87383, &UA198386},
	{SG87384, &UA198466},
	{SG87385, &UA198431},
	{SG87386, &UA198526},
	{SG87387, &UA198553},
	{SG87388, &UA198575},
	{SG87389, &UA198587},
	{SG87390, &UA198589},
	{SG87391, &UA198608},
	{SG87392, &UA198633},
	{SG87393, &UA198697},
	{SG87394, &UA198736},
	{SG87395, &UA198751},
	{SG87396, &UA198759},
	{SG87397, &UA200133},
	{SG87398, &UA200873},
	{SG87399, &UA200966},
	{SG87400, &UA201205},
	{SG87401, &UA201539},
	{SG87402, &UA201657},
	{SG87403, &UA202358},
	{SG87404, &UA202481},
	{SG87405, &UA202590},
	{SG87406, &UA204531},
	{SG87407, &UA205368},
	{SG87408, &UA205433},
	{SG87409, &UA205607},
	{SG87410, &UA206212},
	{SG87411, &UA207475},
	{SG87412, &UA207842},
	{SG87413, &UA208970},
	{SG87414, &UA208990},
	{SG87415, &UA209770},
	{SG87416, &UA218308},
	{SG87417, &UA221085},
	{SG87418, &UA225874},
	{SG87419, &UA242793},
	{SG87420, &UA261337},
	{SG87421, &UA261743},
	{SG87422, &UA262301},
	{SG87423, &UA263498},
	{SG87424, &UA263927},
	{SG87425, &UA265429},
	{SG87426, &UA266146},
	{SG87427, &UA267094},
	{SG87428, &UA267551},
	{SG87429, &UA271684},
	{SG87430, &UA272120},
	{SG87431, &UA273017},
	{SG87433, &UA274054},
	{SG87435, &UA274152},
	{SG87436, &UA274649},
	{SG87437, &UA274944},
	{SG87438, &UA276277},
	{SG87439, &UA276688},
	{SG87440, &UA276882},
	{SG87441, &UA277132},
	{SG87443, &UA277512},
	{SG87445, &UA277671},
	{SG87447, &UA277973},
	{SG87449, &UA278234},
	{SG87452, &UA278481},
	{SG87453, &UA279264},
	{SG87454, &UA279509},
	{SG87455, &UA279901},
	{SG87456, &UA282328},
	{SG87457, &UA282671},
	{SG87458, &UA283409},
	{SG87459, &UA285044},
	{SG87460, &UA288770},
	{SG87461, &UA288766},
	{SG87462, &UA289166},
	{SG87474, &UA289607},
	{SG87475, &UA290676},
	{SG87476, &UA291577},
	{SG87477, &UA292133},
	{SG87478, &UA292231},
	{SG87479, &UA292448},
	{SG87480, &UA292548},
	{SG87481, &UA292596},
	{SG87482, &UA292651},
	{SG87483, &UA292923},
	{SG87484, &UA293064},
	{SG87485, &UA293298},
	{SG87486, &UA293416},
	{SG87487, &UA293548},
	{SG87488, &UA294288},
	{SG87490, &UA296981},
	{SG87491, &UA299130},
	{SG87492, &UA299134},
	{SG87493, &UA299926},
	{SG87494, &UA299948},
	{SG87495, &UA300002},
	{SG87496, &UA300390},
	{SG87497, &UA300403},
	{SG87498, &UA300629},
	{SG87499, &UA301522},
	{SG87500, &UA301776},
	{SG87501, &UA301817},
	{SG87502, &UA302226},
	{SG87503, &UA302270},
	{SG87504, &UA302411},
	{SG87505, &UA303491},
	{SG87506, &UA303513},
	{SG87507, &UA303626},
	{SG87508, &UA306249},
	{SG87509, &UA307257},
	{SG87510, &UA307342},
	{SG87511, &UA307423},
	{SG87512, &UA307590},
	{SG87513, &UA307636},
	{SG87514, &UA307668},
	{SG87515, &UA307916},
	{SG87516, &UA308318},
	{SG87517, &UA308965},
	{SG87518, &UA309658},
	{SG87519, &UA310133},
	{SG87520, &UA310318},
	{SG87521, &UA310535},
	{SG87522, &UA310943},
	{SG87523, &UA311280},
	{SG87524, &UA311547},
	{SG87525, &UA311756},
	{SG87526, &UA311768},
	{SG87527, &UA311781},
	{SG87528, &UA311996},
	{SG87529, &UA312322},
	{SG87530, &UA312351},
	{SG87531, &UA312393},
	{SG87532, &UA312521},
	{SG87533, &UA312578},
	{SG87534, &UA312659},
	{SG87535, &UA312688},
	{SG87536, &UA312890},
	{SG87537, &UA313337},
	{SG87538, &UA313545},
	{SG87539, &UA313957},
	{SG87540, &UA314124},
	{SG87541, &UA314426},
	{SG87542, &UA314465},
	{SG87543, &UA316618},
	{SG87544, &UA316826},
	{SG87545, &UA317521},
	{SG87546, &UA317615},
	{SG87547, &UA320985},
	{SG87548, &UA322570},
	{SG87549, &UA325086},
	{SG87550, &UA332956},
	{SG87551, &UA334637},
	{SG87552, &UA337019},
	{SG87553, &UA337477},
	{SG87554, &UA342551},
	{SG87555, &UA372224},
	{SG87556, &UA372374},
	{SG87557, &UA373317},
	{SG87558, &UA375372},
	{SG87559, &UA375568},
	{SG87560, &UA376305},
	{SG87561, &UA376586},
	{SG87562, &UA379883},
	{SG87563, &UA382527},
	{SG87564, &UA383469},
	{SG87565, &UA384246},
	{SG87566, &UA384431},
	{SG87567, &UA386250},
	{SG87568, &UA386363},
	{SG87569, &UA386552},
	{SG87570, &UA388272},
	{SG87571, &UA388500},
	{SG87572, &UA390615},
	{SG87573, &UA391627},
	{SG87574, &UA393175},
	{SG87575, &UA393323},
	{SG87576, &UA393447},
	{SG87577, &UA395602},
	{SG87578, &UA396118},
	{SG87579, &UA396146},
	{SG87580, &UA396160},
	{SG87581, &UA399446},
	{SG87582, &UA400825},
	{SG87583, &UA400831},
	{SG87584, &UA401035},
	{SG87585, &UA408513},
	{SG87586, &UA412693},
	{SG87587, &UA413509},
	{SG87588, &UA413541},
	{SG87589, &UA413864},
	{SG87590, &UA413892},
	{SG87591, &UA414080},
	{SG87592, &UA419721},
	{SG87593, &UA420176},
	{SG87594, &UA420370},
	{SG87595, &UA421223},
	{SG87596, &UA430113},
	{SG87597, &UA432863},
	{SG87598, &UA433967},
	{SG87599, &UA434013},
	{SG87600, &UA435097},
	{SG87601, &UA435120},
	{SG87602, &UA435561},
	{SG87603, &UA435584},
	{SG87604, &UA439237},
	{SG87605, &UA440119},
	{SG87606, &UA443691},
	{SG87607, &UA444393},
	{SG87608, &UA445141},
	{SG87609, &UA445315},
	{SG87610, &UA447407},
	{SG87611, &UA449698},
	{SG87612, &UA451167},
	{SG87613, &UA463946},
	{SG87614, &UA465606},
	{SG87615, &UA467188},
	{SG87616, &UA469119},
	{SG87617, &UA470241},
	{SG87618, &UA470357},
	{SG87619, &UA472006},
	{SG87620, &UA472645},
	{SG87621, &UA473334},
	{SG87622, &UA476560},
	{SG87623, &UA477899},
	{SG87624, &UA478161},
	{SG87625, &UA480685},
	{SG87626, &UA482629},
	{SG87627, &UA486633},
	{SG87628, &UA488117},
	{SG87629, &UA488184},
	{SG87630, &UA488189},
	{SG87631, &UA488839},
	{SG87632, &UA490395},
	{SG87633, &UA490665},
	{SG87634, &UA491108},
	{SG87635, &UA493109},
	{SG87636, &UA493531},
	{SG87637, &UA501835},
	{SG87638, &UA504434},
	{SG87639, &UA508388},
	{SG87640, &UA508795},
	{SG87641, &UA509126},
	{SG87642, &UA509348},
	{SG87643, &UA510823},
	{SG87644, &UA512040},
	{SG87645, &UA516019},
	{SG87646, &UA516498},
	{SG87647, &UA523992},
	{SG87648, &UA525015},
	{SG87649, &UA526972},
	{SG87650, &UA529529},
	{SG87651, &UA529709},
	{SG87652, &UA530580},
	{SG87653, &UA533448},
	{SG87654, &UA533734},
	{SG87655, &UA536720},
	{SG87656, &UA544560},
	{SG87657, &UA545130},
	{SG87658, &UA546464},
	{SG87659, &UA547039},
	{SG87660, &UA547431},
	{SG87661, &UA548081},
	{SG87662, &UA548506},
	{SG87663, &UA552387},
	{SG87664, &UA552841},
	{SG87665, &UA554639},
	{SG87666, &UA557326},
	{SG87667, &UA557534},
	{SG87668, &UA557929},
	{SG87669, &UA557962},
	{SG87670, &UA558001},
	{SG87671, &UA558023},
	{SG87672, &UA558333},
	{SG87673, &UA558684},
	{SG87674, &UA558710},
	{SG87675, &UA558995},
	{SG87676, &UA559719},
	{SG87677, &UA560138},
	{SG87678, &UA561333},
	{SG87679, &UA561397},
	{SG87680, &UA565968},
	{SG87681, &UA571191},
	{SG87682, &UA582619},
	{SG87683, &UA623367},
	{SG87684, &UA623779},
	{SG87685, &UA625037},
	{SG87686, &UA629502},
	{SG87687, &UA630472},
	{SG87688, &UA630831},
	{SG87689, &UA633148},
	{SG87690, &UA633444},
	{SG87691, &UA634021},
	{SG87692, &UA634847},
	{SG87693, &UA635596},
	{SG87694, &UA639727},
	{SG87695, &UA639736},
	{SG87696, &UA640296},
	{SG87697, &UA641473},
	{SG87698, &UA642884},
	{SG87699, &UA643836},
	{SG87700, &UA644991},
	{SG87701, &UA647749},
	{SG87702, &UA653559},
	{SG87703, &UA654901},
	{SG87704, &UA656407},
	{SG87705, &UA656882},
	{SG87706, &UA656997},
	{SG87707, &UA657056},
	{SG87708, &UA657266},
	{SG87709, &UA660880},
	{SG87710, &UA662371},
	{SG87711, &UA664357},
	{SG87712, &UA667123},
	{SG87713, &UA667756},
	{SG87714, &UA669668},
	{SG87715, &UA669807},
	{SG87716, &UA670640},
	{SG87717, &UA671193},
	{SG87718, &UA674267},
	{SG87719, &UA675289},
	{SG87720, &UA677394},
	{SG87721, &UA678613},
	{SG87722, &UA680699},
	{SG87723, &UA682671},
	{SG87724, &UA683563},
	{SG87725, &UA684081},
	{SG87726, &UA685595},
	{SG87727, &UA687237},
	{SG87728, &UA688527},
	{SG87729, &UA690811},
	{SG87730, &UA691068},
	{SG87731, &UA691987},
	{SG87732, &UA694947},
	{SG87733, &UA699283},
	{SG87734, &UA700123},
	{SG87735, &UA700366},
	{SG87736, &UA700896},
	{SG87737, &UA701535},
	{SG87738, &UA702199},
	{SG87739, &UA703907},
	{SG87740, &UA704903},
	{SG87741, &UA706890},
	{SG87742, &UA710134},
	{SG87743, &UA710789},
	{SG87744, &UA711539},
	{SG87745, &UA714374},
	{SG87746, &UA716354},
	{SG87747, &UA717234},
	{SG87748, &UA717333},
	{SG87749, &UA720154},
	{SG87750, &UA721607},
	{SG87751, &UA723057},
	{SG87752, &UA723761},
	{SG87753, &UA726623},
	{SG87754, &UA729001},
	{SG87755, &UA729301},
	{SG87756, &UA729809},
	{SG87757, &UA730099},
	{SG87758, &UA731014},
	{SG87759, &UA731739},
	{SG87760, &UA732909},
	{SG87761, &UA735419},
	{SG87762, &UA742131},
	{SG87763, &UA745193},
	{SG87764, &UA750018},
	{SG87765, &UA753592},
	{SG87766, &UA754248},
	{SG87767, &UA757136},
	{SG87768, &UA760277},
	{SG87769, &UA761135},
	{SG87770, &UA762773},
	{SG87771, &UA763164},
	{SG87772, &UA763498},
	{SG87773, &UA765649},
	{SG87774, &UA765653},
	{SG87775, &UA765753},
	{SG87776, &UA766042},
	{SG87777, &UA766455},
	{SG87778, &UA767294},
	{SG87779, &UA769006},
	{SG87780, &UA773514},
	{SG87781, &UA775518},
	{SG87782, &UA775680},
	{SG87783, &UA776279},
	{SG87784, &UA779052},
	{SG87785, &UA788048},
	{SG87786, &UA790039},
	{SG87787, &UA792343},
	{SG87788, &UA792440},
	{SG87789, &UA796754},
	{SG87790, &UA796986},
	{SG87791, &UA797255},
	{SG87792, &UA798062},
	{SG87793, &UA811301},
	{SG87794, &UA814739},
	{SG87795, &UA814930},
	{SG87796, &UA816601},
	{SG87797, &UA819306},
	{SG87798, &UA826715},
	{SG87799, &UA830612},
	{SG87800, &UA832936},
	{SG87801, &UA837792},
	{SG87802, &UA842292},
	{SG87803, &UA863840},
	{SG87804, &UA872323},
	{SG87805, &UA874868},
	{SG87806, &UA882245},
	{SG87807, &UA883024},
	{SG87808, &UA894130},
	{SG87809, &UA896819},
	{SG87810, &UA896824},
	{SG87811, &UA899291},
	{SG87812, &UA903066},
	{SG87813, &UA903727},
	{SG87814, &UA903905},
	{SG87815, &UA904207},
	{SG87816, &UA904747},
	{SG87817, &UA905958},
	{SG87818, &UA911743},
	{SG87819, &UA911749},
	{SG87820, &UA911815},
	{SG87821, &UA912322},
	{SG87822, &UA924175},
	{SG87823, &UA1836733},
	{SG87824, &UA1837291},
	{SG87825, &UA1837770},
	{SG87826, &UA1838387},
	{SG87827, &UA1839541},
	{SG87828, &UA1841358},
	{SG87829, &UA1852566},
	{SG87830, &UA1852702},
	{SG87831, &UA1857620},
	{SG87832, &UA1858683},
	{SG87833, &UA1859541},
	{SG87834, &UA1860957},
	{SG87835, &UA1861816},
	{SG87836, &UA1862383},
	{SG87837, &UA1862391},
	{SG87838, &UA1863835},
	{SG87839, &UA1866019},
	{SG87840, &UA1866470},
	{SG87841, &UA1866579},
	{SG87842, &UA1866595},
	{SG87843, &UA1867663},
	{SG87844, &UA1867870},
	{SG87845, &UA1868114},
	{SG87846, &UA1868654},
	{SG87847, &UA1869515},
	{SG87848, &UA1871992},
	{SG87849, &UA1873732},
	{SG87850, &UA1874109},
	{SG87851, &UA1875288},
	{SG87852, &UA1876918},
	{SG87853, &UA1881574},
	{SG87854, &UA1885937},
	{SG87855, &UA1888460},
	{SG87856, &UA1889957},
	{SG87857, &UA1892934},
	{SG87858, &UA1893287},
	{SG87859, &UA1893811},
	{SG87860, &UA1897772},
	{SG87861, &UA1904963},
	{SG87862, &UA1918716},
	{SG87863, &UA1923527},
	{SG87864, &UA1924502},
	{SG87865, &UA1957139},
	{SG87866, &UA1960300},
	{SG87867, &UA1961197},
	{SG87868, &UA1961423},
	{SG87869, &UA1964835},
	{SG87870, &UA1964958},
	{SG87871, &UA1965152},
	{SG87872, &UA1966727},
	{SG87873, &UA1967098},
	{SG87874, &UA1971992},
	{SG87875, &UA1986675},
	{SG87876, &UA1986971},
	{SG87877, &UA1994731},
	{SG87878, &UA2039807},
	{SG87879, &UA2125848},
	{SG87880, &UA2125967},
	{SG87881, &UA2125984},
	{SG87882, &UA2126033},
	{SG87883, &UA2126571},
	{SG87884, &UA2126724},
	{SG87885, &UA2127161},
	{SG87886, &UA2127175},
	{SG87887, &UA2130579},
	{SG87888, &UA2130848},
	{SG87889, &UA2136754},
	{SG87890, &UA2139614},
	{SG87891, &UA2140927},
	{SG87892, &UA2157511},
	{SG87893, &UA2157959},
	{SG87894, &UA2158590},
	{SG87895, &UA2159347},
	{SG87896, &UA2159890},
	{SG87897, &UA2162087},
	{SG87898, &UA2163092},
	{SG87899, &UA2163349},
	{SG87900, &UA2165023},
	{SG87901, &UA2165040},
	{SG87902, &UA2166766},
	{SG87903, &UA2166854},
	{SG87904, &UA2167157},
	{SG87905, &UA2167415},
	{SG87906, &UA2167925},
	{SG87907, &UA2168075},
	{SG87908, &UA2169366},
	{SG87909, &UA2200828},
	{SG87910, &UA2206134},
	{SG87911, &UA2207151},
	{SG87912, &UA2207690},
	{SG87913, &UA2207696},
	{SG87914, &UA2208115},
	{SG87915, &UA2209866},
	{SG87916, &UA2209992},
	{SG87917, &UA2212681},
	{SG87918, &UA2212694},
	{SG87919, &UA2214118},
	{SG87920, &UA2218670},
	{SG87921, &UA2223276},
	{SG87922, &UA2229842},
	{SG87923, &UA2232946},
	{SG87924, &UA2258669},
	{SG87925, &UA2258735},
	{SG87926, &UA2258781},
	{SG87927, &UA2265869},
	{SG87928, &UA2268918},
	{SG87929, &UA2269968},
	{SG87930, &UA2274465},
	{SG87931, &UA2276864},
	{SG87932, &UA2279981},
	{SG87933, &UA2282968},
	{SG87934, &UA2283500},
	{SG87935, &UA2283949},
	{SG87936, &UA2284013},
	{SG87937, &UA2284686},
	{SG87938, &UA2290222},
	{SG87939, &UA2291154},
	{SG87940, &UA2291816},
	{SG87941, &UA2293609},
	{SG87942, &UA2298827},
	{SG87943, &UA2300068},
	{SG87944, &UA2301290},
	{SG87945, &UA2302673},
	{SG87946, &UA2309921},
	{SG87947, &UA2328491},
	{SG87948, &UA2329509},
	{SG87949, &UA2336913},
	{SG87950, &UA2347218},
	{SG87951, &UA2350031},
	{SG87952, &UA2362161},
	{SG87953, &UA2381714},
	{SG87954, &UA2382148},
	{SG87955, &UA2382434},
	{SG87956, &UA2382480},
	{SG87957, &UA2382578},
	{SG87958, &UA2382602},
	{SG87959, &UA2393658},
	{SG87960, &UA2410856},
	{SG87961, &UA2415614},
	{SG87962, &UA2418066},
	{SG87963, &UA2418485},
	{SG87964, &UA2418706},
	{SG87965, &UA2419314},
	{SG87966, &UA2419789},
	{SG87967, &UA2441994},
	{SG87968, &UA2442335},
	{SG87969, &UA2453961},
	{SG87970, &UA2454616},
	{SG87971, &UA2465649},
	{SG87972, &UA2465864},
	{SG87974, &UA2472145},
	{SG87975, &UA2475867},
	{SG87976, &UA2500429},
	{SG87977, &UA2518366},
	{SG87978, &UA2526002},
	{SG87979, &UA2548933},
	{SG87980, &UA2573977},
	{SG87981, &UA2576334},
	{SG87982, &UA2577478},
	{SG87983, &UA2577616},
	{SG87984, &UA2589712},
	{SG87985, &UA2590584},
	{SG87986, &UA2594214},
	{SG87987, &UA2638826},
	{SG87988, &UA2644269},
	{SG87989, &UA2652209},
	{SG87990, &UA2652572},
	{SG87991, &UA2673257},
	{SG87992, &UA2673262},
	{SG87993, &UA2674206},
	{SG87994, &UA2674431},
	{SG87995, &UA2675258},
	{SG87996, &UA2678791},
	{SG87997, &UA2682121},
	{SG87998, &UA2708270},
	{SG87999, &UA2726799},
	{SG88000, &UA2726803},
	{SG88001, &UA2743367},
	{SG88002, &UA2775018},
	{SG88003, &UA2777603},
	{SG88004, &UA2786666},
	{SG88005, &UA2790049},
	{SG88006, &UA2801560},
	{SG88007, &UA2803722},
	{SG88008, &UA2804066},
	{SG88009, &UA2805611},
	{SG88010, &UA2830265},
	{SG88011, &UA2830325},
	{SG88012, &UA2838160},
	{SG88013, &UA2838251},
	{SG88014, &UA2851651},
	{SG88015, &UA2951131},
	{SG88016, &UA2977697},
	{SG88018, &UA2981616},
	{SG88019, &UA2981876},
	{SG88020, &UA3042857},
	{SG88022, &UA3069266},
	{SG88025, &UA3078436},
	{SG88026, &UA3108941},
	{SG88028, &UA3125335},
	{SG88029, &UA3128033},
	{SG88031, &UA3128199},
	{SG88032, &UA3130054},
	{SG88036, &UA895380},
	{SG88037, &UA899124},
	{SG88038, &UA900174},
	{SG88039, &UA901971},
	{SG88040, &UA902780},
	{SG88041, &UA908222},
	{SG88042, &UA909619},
	{SG88043, &UA911417},
	{SG88044, &UA924900},
	{SG88045, &UA2258463},
	{SG88046, &UA2258480},
	{SG88047, &UA2258785},
	{SG88048, &UA2422508},
	{SG88049, &UA2480905},
	{SG88050, &UA2490844},
	{SG88051, &UA2546092},
	{SG88052, &UA2567292},
	{SG88053, &UA2596475},
	{SG88054, &UA2597529},
	{SG88055, &UA2597639},
	{SG88056, &UA2679273},
	{SG88057, &UA2679622},
	{SG88058, &UA2693345},
	{SG88059, &UA2717671},
	{SG88060, &UA2784476},
	{SG88061, &UA2937246},
	{SG88062, &UA3003349},
	{SG88063, &UA3080312},
	{SG88064, &UA198301},
	{SG88065, &UA222529},
	{SG88066, &UA228716},
	{SG88067, &UA250824},
	{SG88068, &UA329047},
	{SG88069, &UA346841},
	{SG88070, &UA351558},
	{SG88071, &UA427004},
	{SG88072, &UA428876},
	{SG88073, &UA436522},
	{SG88074, &UA440928},
	{SG88075, &UA587682},
	{SG88076, &UA615808},
	{SG88077, &UA622545},
	{SG88078, &UA630452},
	{SG88079, &UA630742},
	{SG88080, &UA686725},
	{SG88081, &UA852424},
	{SG88082, &UA862530},
	{SG88083, &UA886733},
	{SG88084, &UA1892312},
	{SG88085, &UA1969123},
	{SG88086, &UA1989407},
	{SG88087, &UA2320120},
	{SG88088, &UA2329168},
	{SG88089, &UA2412111},
	{SG88090, &UA2474939},
	{SG88091, &UA2570878},
	{SG88092, &UA2584773},
	{SG88093, &UA2677414},
	{SG88094, &UA2679266},
	{SG88095, &UA2727355},
	{SG88096, &UA2795195},
	{SG88097, &UA2832913},
	{SG88098, &UA3073041},
	{SG88099, &UA93469},
	{SG88100, &UA138768},
	{SG88101, &UA164155},
	{SG88102, &UA167009},
	{SG88103, &UA167214},
	{SG88104, &UA206154},
	{SG88105, &UA299204},
	{SG88106, &UA331310},
	{SG88107, &UA350539},
	{SG88108, &UA628960},
	{SG88109, &UA633079},
	{SG88110, &UA2398169},
	{SG88111, &UA2518231},
	{SG88112, &UA2696359},
	{SG88113, &UA2740575},
	{SG88114, &UA2770587},
	{SG88115, &UA2874771},
	{SG88116, &UA92160},
	{SG88117, &UA96766},
	{SG88118, &UA98086},
	{SG88119, &UA100821},
	{SG88120, &UA104154},
	{SG88121, &UA105700},
	{SG88122, &UA114239},
	{SG88123, &UA115423},
	{SG88124, &UA119647},
	{SG88125, &UA125109},
	{SG88126, &UA142716},
	{SG88127, &UA143529},
	{SG88128, &UA152418},
	{SG88129, &UA157926},
	{SG88130, &UA164692},
	{SG88131, &UA271463},
	{SG88132, &UA274734},
	{SG88133, &UA275086},
	{SG88134, &UA275699},
	{SG88135, &UA300381},
	{SG88136, &UA478713},
	{SG88137, &UA487605},
	{SG88138, &UA509178},
	{SG88139, &UA570560},
	{SG88140, &UA665464},
	{SG88141, &UA670625},
	{SG88142, &UA822984},
	{SG88143, &UA869435},
	{SG88144, &UA878183},
	{SG88145, &UA917824},
	{SG88146, &UA2205247},
	{SG88147, &UA2258542},
	{SG88148, &UA2258559},
	{SG88149, &UA2263829},
	{SG88150, &UA2383578},
	{SG88151, &UA2494887},
	{SG88152, &UA2577631},
	{SG88153, &UA2833365},
	{SG88154, &UA178471},
	{SG88155, &UA193033},
	{SG88156, &UA195096},
	{SG88157, &UA196539},
	{SG88158, &UA202832},
	{SG88159, &UA212954},
	{SG88160, &UA222639},
	{SG88161, &UA249423},
	{SG88162, &UA250907},
	{SG88163, &UA252684},
	{SG88164, &UA310579},
	{SG88165, &UA311882},
	{SG88166, &UA337004},
	{SG88167, &UA347155},
	{SG88168, &UA360420},
	{SG88169, &UA380830},
	{SG88170, &UA428874},
	{SG88171, &UA440726},
	{SG88172, &UA623001},
	{SG88173, &UA623172},
	{SG88174, &UA626575},
	{SG88175, &UA626628},
	{SG88176, &UA666928},
	{SG88177, &UA669107},
	{SG88178, &UA678500},
	{SG88179, &UA699917},
	{SG88180, &UA877381},
	{SG88181, &UA901370},
	{SG88182, &UA1962238},
	{SG88183, &UA1995596},
	{SG88184, &UA2193781},
	{SG88185, &UA2273313},
	{SG88186, &UA2287465},
	{SG88187, &UA2574734},
	{SG88188, &UA2597767},
	{SG88189, &UA2644351},
	{SG88190, &UA2707146},
	{SG88191, &UA2795638},
	{SG88192, &UA3098496},
	{SG88193, &UA2531146},
	{SG88202, &UA102032},
	{SG88203, &UA102260},
	{SG88204, &UA102436},
	{SG88205, &UA105221},
	{SG88206, &UA105425},
	{SG88207, &UA105898},
	{SG88208, &UA106122},
	{SG88209, &UA106409},
	{SG88210, &UA106679},
	{SG88211, &UA106762},
	{SG88212, &UA107213},
	{SG88213, &UA109871},
	{SG88214, &UA110614},
	{SG88215, &UA110693},
	{SG88216, &UA114720},
	{SG88217, &UA119262},
	{SG88218, &UA119337},
	{SG88219, &UA119734},
	{SG88220, &UA119814},
	{SG88221, &UA120508},
	{SG88222, &UA121743},
	{SG88223, &UA122727},
	{SG88224, &UA122793},
	{SG88225, &UA123648},
	{SG88226, &UA124695},
	{SG88227, &UA125857},
	{SG88228, &UA126293},
	{SG88229, &UA129816},
	{SG88230, &UA130220},
	{SG88231, &UA131633},
	{SG88232, &UA132230},
	{SG88233, &UA132245},
	{SG88234, &UA132377},
	{SG88235, &UA132509},
	{SG88236, &UA132724},
	{SG88237, &UA132884},
	{SG88238, &UA132928},
	{SG88239, &UA133198},
	{SG88240, &UA133347},
	{SG88241, &UA133775},
	{SG88242, &UA133885},
	{SG88243, &UA134362},
	{SG88244, &UA134376},
	{SG88245, &UA135063},
	{SG88246, &UA135239},
	{SG88247, &UA135487},
	{SG88248, &UA135640},
	{SG88249, &UA136134},
	{SG88250, &UA136181},
	{SG88251, &UA136821},
	{SG88252, &UA137220},
	{SG88253, &UA138803},
	{SG88254, &UA139627},
	{SG88255, &UA139642},
	{SG88256, &UA146460},
	{SG88257, &UA153553},
	{SG88258, &UA153570},
	{SG88259, &UA153654},
	{SG88260, &UA153662},
	{SG88261, &UA154515},
	{SG88262, &UA154975},
	{SG88263, &UA155130},
	{SG88264, &UA155184},
	{SG88265, &UA155186},
	{SG88266, &UA155501},
	{SG88267, &UA156094},
	{SG88268, &UA156269},
	{SG88269, &UA156970},
	{SG88270, &UA157006},
	{SG88271, &UA157035},
	{SG88272, &UA157146},
	{SG88273, &UA157189},
	{SG88274, &UA157209},
	{SG88275, &UA157583},
	{SG88276, &UA157730},
	{SG88277, &UA157940},
	{SG88278, &UA158265},
	{SG88279, &UA158384},
	{SG88280, &UA158464},
	{SG88281, &UA158500},
	{SG88282, &UA158591},
	{SG88283, &UA158876},
	{SG88284, &UA159036},
	{SG88285, &UA159511},
	{SG88286, &UA159649},
	{SG88287, &UA160656},
	{SG88288, &UA160934},
	{SG88289, &UA161035},
	{SG88290, &UA173392},
	{SG88291, &UA179500},
	{SG88292, &UA180436},
	{SG88293, &UA180443},
	{SG88294, &UA180525},
	{SG88295, &UA180544},
	{SG88296, &UA180554},
	{SG88297, &UA180599},
	{SG88298, &UA180624},
	{SG88299, &UA180633},
	{SG88300, &UA181375},
	{SG88301, &UA181390},
	{SG88302, &UA181468},
	{SG88303, &UA181514},
	{SG88304, &UA181516},
	{SG88305, &UA181526},
	{SG88306, &UA181561},
	{SG88307, &UA181568},
	{SG88308, &UA181779},
	{SG88309, &UA182308},
	{SG88310, &UA182669},
	{SG88311, &UA182678},
	{SG88312, &UA183201},
	{SG88313, &UA183203},
	{SG88314, &UA183219},
	{SG88315, &UA183300},
	{SG88316, &UA183364},
	{SG88317, &UA183454},
	{SG88318, &UA183658},
	{SG88319, &UA183687},
	{SG88320, &UA183695},
	{SG88321, &UA183698},
	{SG88322, &UA183715},
	{SG88323, &UA183869},
	{SG88324, &UA183872},
	{SG88325, &UA183936},
	{SG88326, &UA184096},
	{SG88327, &UA184162},
	{SG88328, &UA184214},
	{SG88329, &UA184386},
	{SG88330, &UA184393},
	{SG88331, &UA184419},
	{SG88332, &UA184721},
	{SG88333, &UA184846},
	{SG88334, &UA185409},
	{SG88335, &UA185586},
	{SG88336, &UA185600},
	{SG88337, &UA186384},
	{SG88338, &UA186386},
	{SG88339, &UA186417},
	{SG88340, &UA186655},
	{SG88341, &UA187099},
	{SG88342, &UA187139},
	{SG88343, &UA187216},
	{SG88344, &UA187333},
	{SG88345, &UA187583},
	{SG88346, &UA188057},
	{SG88347, &UA188403},
	{SG88348, &UA189205},
	{SG88349, &UA189232},
	{SG88350, &UA190034},
	{SG88351, &UA190921},
	{SG88352, &UA199565},
	{SG88353, &UA228046},
	{SG88354, &UA228178},
	{SG88355, &UA228338},
	{SG88356, &UA228941},
	{SG88357, &UA230014},
	{SG88358, &UA230164},
	{SG88359, &UA230362},
	{SG88360, &UA231050},
	{SG88361, &UA263005},
	{SG88362, &UA267029},
	{SG88363, &UA267500},
	{SG88364, &UA268256},
	{SG88366, &UA269299},
	{SG88367, &UA271764},
	{SG88369, &UA271785},
	{SG88370, &UA272342},
	{SG88371, &UA272650},
	{SG88372, &UA274466},
	{SG88373, &UA276594},
	{SG88374, &UA276972},
	{SG88375, &UA277235},
	{SG88376, &UA277393},
	{SG88377, &UA278090},
	{SG88378, &UA278377},
	{SG88379, &UA279051},
	{SG88380, &UA279869},
	{SG88382, &UA281258},
	{SG88383, &UA286722},
	{SG88385, &UA292091},
	{SG88386, &UA292802},
	{SG88387, &UA293126},
	{SG88388, &UA293778},
	{SG88389, &UA293818},
	{SG88390, &UA294262},
	{SG88391, &UA295180},
	{SG88392, &UA295247},
	{SG88393, &UA295373},
	{SG88394, &UA296540},
	{SG88395, &UA297513},
	{SG88396, &UA300234},
	{SG88397, &UA305723},
	{SG88398, &UA306231},
	{SG88399, &UA307340},
	{SG88400, &UA307535},
	{SG88401, &UA308359},
	{SG88402, &UA308355},
	{SG88403, &UA308401},
	{SG88404, &UA309297},
	{SG88405, &UA309404},
	{SG88406, &UA309563},
	{SG88407, &UA310973},
	{SG88408, &UA311275},
	{SG88409, &UA311687},
	{SG88410, &UA312263},
	{SG88411, &UA313996},
	{SG88412, &UA314115},
	{SG88413, &UA314250},
	{SG88414, &UA314739},
	{SG88415, &UA314811},
	{SG88416, &UA314792},
	{SG88417, &UA314837},
	{SG88418, &UA315186},
	{SG88419, &UA315854},
	{SG88420, &UA317784},
	{SG88421, &UA319189},
	{SG88422, &UA372356},
	{SG88423, &UA372756},
	{SG88424, &UA382865},
	{SG88425, &UA383187},
	{SG88426, &UA383239},
	{SG88427, &UA383624},
	{SG88428, &UA384354},
	{SG88429, &UA384536},
	{SG88430, &UA386261},
	{SG88431, &UA387089},
	{SG88432, &UA399355},
	{SG88433, &UA414250},
	{SG88434, &UA430463},
	{SG88435, &UA430823},
	{SG88436, &UA431018},
	{SG88437, &UA435489},
	{SG88438, &UA448624},
	{SG88439, &UA468408},
	{SG88440, &UA469818},
	{SG88441, &UA470494},
	{SG88442, &UA472754},
	{SG88443, &UA473021},
	{SG88444, &UA473484},
	{SG88445, &UA474423},
	{SG88446, &UA475117},
	{SG88447, &UA475703},
	{SG88448, &UA476496},
	{SG88449, &UA481327},
	{SG88450, &UA492962},
	{SG88451, &UA497766},
	{SG88452, &UA500972},
	{SG88453, &UA502025},
	{SG88454, &UA519450},
	{SG88455, &UA524215},
	{SG88456, &UA529273},
	{SG88457, &UA559078},
	{SG88458, &UA566427},
	{SG88459, &UA662389},
	{SG88460, &UA681849},
	{SG88461, &UA701158},
	{SG88462, &UA704779},
	{SG88463, &UA706638},
	{SG88464, &UA734031},
	{SG88465, &UA734044},
	{SG88466, &UA736487},
	{SG88467, &UA737574},
	{SG88468, &UA767945},
	{SG88469, &UA768687},
	{SG88470, &UA777487},
	{SG88471, &UA789376},
	{SG88472, &UA805037},
	{SG88473, &UA823373},
	{SG88474, &UA837961},
	{SG88475, &UA838341},
	{SG88476, &UA863776},
	{SG88477, &UA867609},
	{SG88478, &UA889564},
	{SG88479, &UA892706},
	{SG88480, &UA895634},
	{SG88481, &UA899395},
	{SG88482, &UA913522},
	{SG88483, &UA1836388},
	{SG88484, &UA1840548},
	{SG88485, &UA1845267},
	{SG88486, &UA1851370},
	{SG88487, &UA1860363},
	{SG88488, &UA1860415},
	{SG88489, &UA1861755},
	{SG88490, &UA1865071},
	{SG88491, &UA2131205},
	{SG88492, &UA2131667},
	{SG88493, &UA2134591},
	{SG88494, &UA2207118},
	{SG88495, &UA2207757},
	{SG88496, &UA2214872},
	{SG88497, &UA2258250},
	{SG88498, &UA2263027},
	{SG88499, &UA2266671},
	{SG88500, &UA2268391},
	{SG88501, &UA2269950},
	{SG88502, &UA2276412},
	{SG88503, &UA2284764},
	{SG88504, &UA2286994},
	{SG88505, &UA2474920},
	{SG88506, &UA2644228},
	{SG88507, &UA900931},
	{SG88508, &UA905280},
	{SG88509, &UA924456},
	{SG88510, &UA925435},
	{SG88511, &UA1997241},
	{SG88512, &UA2258401},
	{SG88513, &UA2258402},
	{SG88514, &UA2387029},
	{SG88515, &UA2425268},
	{SG88516, &UA2567817},
	{SG88517, &UA2584765},
	{SG88518, &UA2679317},
	{SG88519, &UA2680707},
	{SG88520, &UA2767904},
	{SG88521, &UA2783265},
	{SG88522, &UA30673},
	{SG88523, &UA30740},
	{SG88524, &UA91315},
	{SG88525, &UA92398},
	{SG88526, &UA92404},
	{SG88527, &UA92748},
	{SG88528, &UA92936},
	{SG88529, &UA93219},
	{SG88530, &UA93330},
	{SG88531, &UA93470},
	{SG88532, &UA93522},
	{SG88533, &UA93536},
	{SG88534, &UA93642},
	{SG88535, &UA93970},
	{SG88536, &UA94153},
	{SG88537, &UA94211},
	{SG88538, &UA94596},
	{SG88539, &UA94735},
	{SG88540, &UA95248},
	{SG88541, &UA95461},
	{SG88542, &UA95505},
	{SG88543, &UA95514},
	{SG88544, &UA95743},
	{SG88545, &UA97397},
	{SG88546, &UA97470},
	{SG88547, &UA97481},
	{SG88548, &UA97670},
	{SG88549, &UA98097},
	{SG88550, &UA98098},
	{SG88551, &UA98087},
	{SG88552, &UA98432},
	{SG88553, &UA100270},
	{SG88554, &UA100831},
	{SG88555, &UA101959},
	{SG88556, &UA103453},
	{SG88557, &UA103542},
	{SG88558, &UA103784},
	{SG88559, &UA103818},
	{SG88560, &UA103979},
	{SG88561, &UA108981},
	{SG88562, &UA113217},
	{SG88563, &UA113291},
	{SG88564, &UA114181},
	{SG88565, &UA117205},
	{SG88566, &UA117341},
	{SG88567, &UA120020},
	{SG88568, &UA122475},
	{SG88569, &UA123828},
	{SG88570, &UA125163},
	{SG88571, &UA125893},
	{SG88572, &UA126495},
	{SG88573, &UA126518},
	{SG88574, &UA127088},
	{SG88575, &UA127122},
	{SG88576, &UA127124},
	{SG88577, &UA128154},
	{SG88578, &UA128542},
	{SG88579, &UA129092},
	{SG88580, &UA129613},
	{SG88581, &UA130645},
	{SG88582, &UA132081},
	{SG88583, &UA133016},
	{SG88584, &UA133084},
	{SG88585, &UA133609},
	{SG88586, &UA133902},
	{SG88587, &UA133926},
	{SG88588, &UA134027},
	{SG88589, &UA134673},
	{SG88590, &UA135055},
	{SG88591, &UA135214},
	{SG88592, &UA136876},
	{SG88593, &UA136926},
	{SG88594, &UA136944},
	{SG88595, &UA137768},
	{SG88596, &UA138065},
	{SG88597, &UA138949},
	{SG88598, &UA139257},
	{SG88599, &UA140759},
	{SG88600, &UA141212},
	{SG88601, &UA143565},
	{SG88602, &UA145648},
	{SG88603, &UA147540},
	{SG88604, &UA148877},
	{SG88605, &UA151364},
	{SG88606, &UA158954},
	{SG88607, &UA163100},
	{SG88608, &UA175491},
	{SG88609, &UA190172},
	{SG88610, &UA228973},
	{SG88611, &UA229410},
	{SG88612, &UA229509},
	{SG88613, &UA230420},
	{SG88614, &UA231661},
	{SG88615, &UA234853},
	{SG88616, &UA260024},
	{SG88617, &UA260317},
	{SG88618, &UA260633},
	{SG88619, &UA261108},
	{SG88620, &UA261580},
	{SG88621, &UA261583},
	{SG88622, &UA261871},
	{SG88623, &UA261886},
	{SG88624, &UA262295},
	{SG88625, &UA263318},
	{SG88626, &UA269860},
	{SG88627, &UA269977},
	{SG88628, &UA271028},
	{SG88629, &UA276642},
	{SG88630, &UA277535},
	{SG88631, &UA277532},
	{SG88632, &UA278163},
	{SG88633, &UA278789},
	{SG88634, &UA279303},
	{SG88635, &UA280364},
	{SG88636, &UA282090},
	{SG88637, &UA283846},
	{SG88638, &UA294068},
	{SG88639, &UA296361},
	{SG88640, &UA300262},
	{SG88641, &UA303811},
	{SG88642, &UA305934},
	{SG88643, &UA306376},
	{SG88644, &UA312553},
	{SG88645, &UA330779},
	{SG88646, &UA371402},
	{SG88647, &UA371577},
	{SG88648, &UA372178},
	{SG88649, &UA372234},
	{SG88650, &UA372548},
	{SG88651, &UA372550},
	{SG88652, &UA372643},
	{SG88653, &UA372725},
	{SG88654, &UA382374},
	{SG88655, &UA382609},
	{SG88656, &UA382729},
	{SG88657, &UA383667},
	{SG88658, &UA384131},
	{SG88659, &UA388347},
	{SG88660, &UA399196},
	{SG88661, &UA399455},
	{SG88662, &UA399935},
	{SG88663, &UA407964},
	{SG88664, &UA413473},
	{SG88665, &UA414611},
	{SG88666, &UA415550},
	{SG88667, &UA415755},
	{SG88668, &UA417398},
	{SG88669, &UA430241},
	{SG88670, &UA430456},
	{SG88671, &UA431347},
	{SG88672, &UA431990},
	{SG88673, &UA438943},
	{SG88674, &UA443774},
	{SG88675, &UA444718},
	{SG88676, &UA448949},
	{SG88677, &UA459740},
	{SG88678, &UA460373},
	{SG88679, &UA467101},
	{SG88680, &UA468509},
	{SG88681, &UA471841},
	{SG88682, &UA476599},
	{SG88683, &UA480796},
	{SG88684, &UA482124},
	{SG88685, &UA486288},
	{SG88686, &UA490167},
	{SG88687, &UA495887},
	{SG88688, &UA498059},
	{SG88689, &UA498128},
	{SG88690, &UA500767},
	{SG88691, &UA514068},
	{SG88692, &UA518742},
	{SG88693, &UA520246},
	{SG88694, &UA521936},
	{SG88695, &UA522093},
	{SG88696, &UA522321},
	{SG88697, &UA526831},
	{SG88698, &UA527154},
	{SG88699, &UA528741},
	{SG88700, &UA624677},
	{SG88701, &UA625673},
	{SG88702, &UA637106},
	{SG88703, &UA640741},
	{SG88704, &UA653527},
	{SG88705, &UA656533},
	{SG88706, &UA670997},
	{SG88707, &UA693376},
	{SG88708, &UA697181},
	{SG88709, &UA697220},
	{SG88710, &UA707860},
	{SG88711, &UA709783},
	{SG88712, &UA710059},
	{SG88713, &UA721050},
	{SG88714, &UA725524},
	{SG88715, &UA730862},
	{SG88716, &UA734186},
	{SG88717, &UA748105},
	{SG88718, &UA751356},
	{SG88719, &UA755814},
	{SG88720, &UA757600},
	{SG88721, &UA761291},
	{SG88722, &UA765932},
	{SG88723, &UA766234},
	{SG88724, &UA770833},
	{SG88725, &UA773794},
	{SG88726, &UA782773},
	{SG88727, &UA785844},
	{SG88728, &UA787730},
	{SG88729, &UA804074},
	{SG88730, &UA806799},
	{SG88731, &UA810826},
	{SG88732, &UA815875},
	{SG88733, &UA838072},
	{SG88734, &UA852422},
	{SG88735, &UA869921},
	{SG88736, &UA870881},
	{SG88737, &UA870986},
	{SG88738, &UA882782},
	{SG88739, &UA1837278},
	{SG88740, &UA1837470},
	{SG88741, &UA1852953},
	{SG88742, &UA1855050},
	{SG88743, &UA1856921},
	{SG88744, &UA1861099},
	{SG88745, &UA1863416},
	{SG88746, &UA1863993},
	{SG88747, &UA1865946},
	{SG88748, &UA1867855},
	{SG88749, &UA1888600},
	{SG88750, &UA1930313},
	{SG88751, &UA2128074},
	{SG88752, &UA2128573},
	{SG88753, &UA2132170},
	{SG88754, &UA2133718},
	{SG88755, &UA2136244},
	{SG88756, &UA2163014},
	{SG88757, &UA2163959},
	{SG88758, &UA2165476},
	{SG88759, &UA2166204},
	{SG88760, &UA2208020},
	{SG88761, &UA2209649},
	{SG88762, &UA2216348},
	{SG88763, &UA2218101},
	{SG88764, &UA2258256},
	{SG88765, &UA2258323},
	{SG88766, &UA2258353},
	{SG88767, &UA2258518},
	{SG88768, &UA2263100},
	{SG88769, &UA2265094},
	{SG88770, &UA2265630},
	{SG88771, &UA2266121},
	{SG88772, &UA2266632},
	{SG88773, &UA2267273},
	{SG88774, &UA2271366},
	{SG88775, &UA2272094},
	{SG88776, &UA2280897},
	{SG88777, &UA2294894},
	{SG88778, &UA2307367},
	{SG88779, &UA2330677},
	{SG88780, &UA2333516},
	{SG88781, &UA2359735},
	{SG88782, &UA2542008},
	{SG88783, &UA2542015},
	{SG88784, &UA2758933},
	{SG88785, &UA3069104},
	{SG88786, &UA91421},
	{SG88787, &UA91823},
	{SG88788, &UA91978},
	{SG88789, &UA92177},
	{SG88790, &UA92400},
	{SG88791, &UA92760},
	{SG88792, &UA92875},
	{SG88793, &UA93094},
	{SG88794, &UA93240},
	{SG88795, &UA93271},
	{SG88796, &UA93512},
	{SG88797, &UA93520},
	{SG88798, &UA93918},
	{SG88799, &UA94169},
	{SG88800, &UA94223},
	{SG88801, &UA94698},
	{SG88802, &UA94721},
	{SG88803, &UA95055},
	{SG88804, &UA95182},
	{SG88805, &UA95624},
	{SG88806, &UA96470},
	{SG88807, &UA96598},
	{SG88808, &UA96623},
	{SG88809, &UA96634},
	{SG88810, &UA96856},
	{SG88811, &UA97492},
	{SG88812, &UA98026},
	{SG88813, &UA98101},
	{SG88814, &UA98133},
	{SG88815, &UA98988},
	{SG88816, &UA99015},
	{SG88817, &UA99299},
	{SG88818, &UA99349},
	{SG88819, &UA100026},
	{SG88820, &UA100414},
	{SG88821, &UA100609},
	{SG88822, &UA101315},
	{SG88823, &UA101585},
	{SG88824, &UA101631},
	{SG88825, &UA101769},
	{SG88826, &UA102733},
	{SG88827, &UA104244},
	{SG88828, &UA104234},
	{SG88829, &UA104301},
	{SG88830, &UA106523},
	{SG88831, &UA106592},
	{SG88832, &UA106786},
	{SG88833, &UA110001},
	{SG88834, &UA111275},
	{SG88835, &UA114184},
	{SG88836, &UA114255},
	{SG88837, &UA114419},
	{SG88838, &UA116480},
	{SG88839, &UA117397},
	{SG88840, &UA118222},
	{SG88841, &UA118263},
	{SG88842, &UA119173},
	{SG88843, &UA120375},
	{SG88844, &UA120649},
	{SG88845, &UA122856},
	{SG88846, &UA123936},
	{SG88847, &UA125096},
	{SG88848, &UA125276},
	{SG88849, &UA126893},
	{SG88850, &UA127757},
	{SG88851, &UA127760},
	{SG88852, &UA129378},
	{SG88853, &UA129733},
	{SG88854, &UA130067},
	{SG88855, &UA130199},
	{SG88856, &UA130383},
	{SG88857, &UA130818},
	{SG88858, &UA131126},
	{SG88859, &UA131811},
	{SG88860, &UA131986},
	{SG88861, &UA132896},
	{SG88862, &UA133054},
	{SG88863, &UA133526},
	{SG88864, &UA133822},
	{SG88865, &UA133953},
	{SG88866, &UA134126},
	{SG88867, &UA134447},
	{SG88868, &UA134629},
	{SG88869, &UA137273},
	{SG88870, &UA138110},
	{SG88871, &UA139688},
	{SG88872, &UA145753},
	{SG88873, &UA151882},
	{SG88874, &UA152627},
	{SG88875, &UA152658},
	{SG88876, &UA153284},
	{SG88877, &UA158659},
	{SG88878, &UA166900},
	{SG88879, &UA167291},
	{SG88880, &UA168064},
	{SG88881, &UA177188},
	{SG88882, &UA178210},
	{SG88883, &UA179291},
	{SG88884, &UA197888},
	{SG88885, &UA199545},
	{SG88886, &UA201288},
	{SG88887, &UA259534},
	{SG88888, &UA261431},
	{SG88889, &UA261875},
	{SG88890, &UA262227},
	{SG88891, &UA262499},
	{SG88892, &UA262744},
	{SG88893, &UA262978},
	{SG88894, &UA263297},
	{SG88895, &UA263561},
	{SG88896, &UA263901},
	{SG88897, &UA264380},
	{SG88898, &UA266595},
	{SG88899, &UA270300},
	{SG88900, &UA271707},
	{SG88901, &UA271872},
	{SG88902, &UA274512},
	{SG88903, &UA274952},
	{SG88904, &UA275510},
	{SG88905, &UA277052},
	{SG88906, &UA278095},
	{SG88907, &UA278130},
	{SG88908, &UA278289},
	{SG88909, &UA278566},
	{SG88910, &UA278963},
	{SG88911, &UA279368},
	{SG88912, &UA281346},
	{SG88913, &UA281519},
	{SG88914, &UA284721},
	{SG88915, &UA289251},
	{SG88916, &UA289408},
	{SG88917, &UA291636},
	{SG88918, &UA292327},
	{SG88919, &UA294887},
	{SG88920, &UA295141},
	{SG88921, &UA295826},
	{SG88922, &UA299160},
	{SG88923, &UA301693},
	{SG88924, &UA303472},
	{SG88925, &UA311891},
	{SG88926, &UA372395},
	{SG88927, &UA372965},
	{SG88928, &UA382652},
	{SG88929, &UA383701},
	{SG88930, &UA385310},
	{SG88931, &UA390143},
	{SG88932, &UA395031},
	{SG88933, &UA408197},
	{SG88934, &UA409314},
	{SG88935, &UA412881},
	{SG88936, &UA413430},
	{SG88937, &UA415918},
	{SG88938, &UA416995},
	{SG88939, &UA417274},
	{SG88940, &UA420083},
	{SG88941, &UA441565},
	{SG88942, &UA442071},
	{SG88943, &UA442757},
	{SG88944, &UA446653},
	{SG88945, &UA446944},
	{SG88946, &UA451194},
	{SG88947, &UA461859},
	{SG88948, &UA465197},
	{SG88949, &UA465435},
	{SG88950, &UA467005},
	{SG88951, &UA472401},
	{SG88952, &UA475179},
	{SG88953, &UA477900},
	{SG88954, &UA478075},
	{SG88955, &UA489056},
	{SG88956, &UA492348},
	{SG88957, &UA499565},
	{SG88958, &UA501939},
	{SG88959, &UA503379},
	{SG88960, &UA506394},
	{SG88961, &UA511049},
	{SG88962, &UA512299},
	{SG88963, &UA518154},
	{SG88964, &UA531056},
	{SG88965, &UA532536},
	{SG88966, &UA533302},
	{SG88967, &UA533759},
	{SG88968, &UA534087},
	{SG88969, &UA534934},
	{SG88970, &UA542003},
	{SG88971, &UA556809},
	{SG88972, &UA558234},
	{SG88973, &UA558339},
	{SG88974, &UA559915},
	{SG88975, &UA562837},
	{SG88976, &UA565485},
	{SG88977, &UA599836},
	{SG88978, &UA635843},
	{SG88979, &UA639345},
	{SG88980, &UA644968},
	{SG88981, &UA645211},
	{SG88982, &UA647400},
	{SG88983, &UA654486},
	{SG88984, &UA658080},
	{SG88985, &UA659701},
	{SG88986, &UA662539},
	{SG88987, &UA669579},
	{SG88988, &UA671199},
	{SG88989, &UA676509},
	{SG88990, &UA681920},
	{SG88991, &UA683331},
	{SG88992, &UA684347},
	{SG88993, &UA693053},
	{SG88994, &UA698281},
	{SG88995, &UA701397},
	{SG88996, &UA707958},
	{SG88997, &UA709581},
	{SG88998, &UA709837},
	{SG88999, &UA711916},
	{SG89000, &UA715819},
	{SG89001, &UA717956},
	{SG89002, &UA720888},
	{SG89003, &UA722429},
	{SG89004, &UA727047},
	{SG89005, &UA729996},
	{SG89006, &UA732612},
	{SG89007, &UA734050},
	{SG89008, &UA734387},
	{SG89009, &UA747894},
	{SG89010, &UA753900},
	{SG89011, &UA757306},
	{SG89012, &UA759406},
	{SG89013, &UA760606},
	{SG89014, &UA762172},
	{SG89015, &UA764464},
	{SG89016, &UA770994},
	{SG89017, &UA775575},
	{SG89018, &UA780243},
	{SG89019, &UA780510},
	{SG89020, &UA799670},
	{SG89021, &UA804528},
	{SG89022, &UA806177},
	{SG89023, &UA809616},
	{SG89024, &UA845940},
	{SG89025, &UA847541},
	{SG89026, &UA862144},
	{SG89027, &UA877876},
	{SG89028, &UA1839225},
	{SG89029, &UA1839560},
	{SG89030, &UA1851099},
	{SG89031, &UA1851881},
	{SG89032, &UA1860599},
	{SG89033, &UA1861949},
	{SG89034, &UA1864617},
	{SG89035, &UA1865840},
	{SG89036, &UA1872863},
	{SG89037, &UA1887217},
	{SG89038, &UA1898682},
	{SG89039, &UA2125908},
	{SG89040, &UA2130782},
	{SG89041, &UA2131159},
	{SG89042, &UA2132122},
	{SG89043, &UA2136114},
	{SG89044, &UA2142033},
	{SG89045, &UA2143594},
	{SG89046, &UA2207945},
	{SG89047, &UA2215597},
	{SG89048, &UA2258251},
	{SG89049, &UA2258345},
	{SG89050, &UA2258773},
	{SG89051, &UA2264924},
	{SG89052, &UA2265905},
	{SG89053, &UA2267238},
	{SG89054, &UA2271705},
	{SG89055, &UA2274926},
	{SG89056, &UA2275283},
	{SG89057, &UA2280200},
	{SG89058, &UA2281783},
	{SG89059, &UA2282753},
	{SG89060, &UA2284080},
	{SG89061, &UA2285641},
	{SG89062, &UA2289252},
	{SG89063, &UA2293260},
	{SG89064, &UA2294739},
	{SG89065, &UA2296510},
	{SG89066, &UA2297871},
	{SG89067, &UA2315785},
	{SG89068, &UA2316894},
	{SG89069, &UA2317642},
	{SG89070, &UA2359749},
	{SG89071, &UA2359757},
	{SG89072, &UA2369789},
	{SG89073, &UA2370552},
	{SG89074, &UA2438482},
	{SG89075, &UA2493895},
	{SG89076, &UA2597346},
	{SG89077, &UA2597350},
	{SG89078, &UA2597438},
	{SG89079, &UA2597687},
	{SG89080, &UA2597903},
	{SG89081, &UA2599240},
	{SG89082, &UA2720156},
	{SG89083, &UA2720324},
	{SG89084, &UA2742189},
	{SG89085, &UA2759103},
	{SG89086, &UA2801328},
	{SG89087, &UA2802781},
	{SG89088, &UA2806116},
	{SG89089, &UA2981287},
	{SG89090, &UA102389},
	{SG89091, &UA119737},
	{SG89092, &UA151542},
	{SG89093, &UA179501},
	{SG89094, &UA180951},
	{SG89095, &UA184095},
	{SG89096, &UA2163512},
	{SG89097, &UA2204462},
	{SG89098, &UA2235279},
	{SG89099, &UA2386845},
	{SG89100, &UA2387069},
	{SG89101, &UA2389172},
	{SG89102, &UA2404040},
	{SG89103, &UA2404660},
	{SG89104, &UA2423988},
	{SG89105, &UA2425606},
	{SG89106, &UA2426737},
	{SG89107, &UA2427648},
	{SG89108, &UA2428668},
	{SG89109, &UA2467374},
	{SG89110, &UA227992},
	{SG89111, &UA228376},
	{SG89112, &UA250865},
	{SG89113, &UA251008},
	{SG89114, &UA428869},
	{SG89115, &UA428872},
	{SG89116, &UA441021},
	{SG89117, &UA441022},
	{SG89118, &UA615809},
	{SG89119, &UA615815},
	{SG89120, &UA622530},
	{SG89121, &UA622542},
	{SG89122, &UA733034},
	{SG89123, &UA775838},
	{SG89124, &UA795038},
	{SG89125, &UA806464},
	{SG89126, &UA1990921},
	{SG89127, &UA2167510},
	{SG89128, &UA2586541},
	{SG89129, &UA2586546},
	{SG89132, &UA102535},
	{SG89133, &UA120919},
	{SG89134, &UA132795},
	{SG89135, &UA153594},
	{SG89136, &UA161747},
	{SG89137, &UA2270068},
	{SG89138, &UA2298220},
	{SG89139, &UA2684326},
	{SG89151, &UA2596334},
	{SG89152, &UA2709498},
	{SG89154, &UA2258450},
	{SG89155, &UA2425283},
	{SG89156, &UA2716668},
	{SG89157, &UA2735135},
	{SG89158, &UA2834482},
	{SG89159, &UA2840101},
	{SG89160, &UA2867961},
	{SG89161, &UA2898250},
	{SG89162, &UA2943253},
	{SG89178, &UA102566},
	{SG89179, &UA121493},
	{SG89180, &UA133494},
	{SG89182, &UA174917},
	{SG89184, &UA180169},
	{SG89186, &UA180949},
	{SG89187, &UA2258945},
	{SG89188, &UA2352382},
	{SG89189, &UA2386824},
	{SG89190, &UA2404642},
	{SG89191, &UA2406788},
	{SG89192, &UA2421981},
	{SG89193, &UA2423080},
	{SG89194, &UA2423257},
	{SG89195, &UA2425354},
	{SG89196, &UA2428661},
	{SG89197, &UA2457429},
	{SG89198, &UA2457879},
	{SG89199, &UA396884},
	{SG89200, &UA628885},
	{SG89201, &UA2729801},
	{SG89208, &UA101848},
	{SG89209, &UA132847},
	{SG89210, &UA134075},
	{SG89211, &UA153623},
	{SG89212, &UA181409},
	{SG89213, &UA183317},
	{SG89214, &UA184104},
	{SG89215, &UA252700},
	{SG89216, &UA279209},
	{SG89217, &UA306936},
	{SG89218, &UA647453},
	{SG89219, &UA668399},
	{SG89220, &UA693283},
	{SG89221, &UA694206},
	{SG89222, &UA727619},
	{SG89223, &UA841709},
	{SG89224, &UA1912960},
	{SG89225, &UA1941362},
	{SG89226, &UA2164705},
	{SG89227, &UA2164994},
	{SG89228, &UA2227006},
	{SG89229, &UA2351813},
	{SG89230, &UA2386757},
	{SG89231, &UA2394450},
	{SG89232, &UA2404532},
	{SG89233, &UA2422334},
	{SG89234, &UA2422329},
	{SG89235, &UA2423462},
	{SG89236, &UA2426637},
	{SG89237, &UA2445250},
	{SG89238, &UA3033983},
	{SG89239, &UA227236},
	{SG89240, &UA228047},
	{SG89241, &UA250884},
	{SG89242, &UA250914},
	{SG89243, &UA346839},
	{SG89244, &UA428868},
	{SG89245, &UA428875},
	{SG89246, &UA440731},
	{SG89247, &UA440766},
	{SG89248, &UA615814},
	{SG89249, &UA615816},
	{SG89250, &UA622552},
	{SG89251, &UA2586539},
	{SG89252, &UA2586544},
	{SG89263, &UA344605},
	{SG89264, &UA798296},
	{SG89265, &UA2230732},
	{SG89266, &UA2349842},
	{SG89267, &UA2401306},
	{SG89268, &UA2480161},
	{SG89269, &UA2595018},
	{SG89270, &UA2857242},
	{SG89271, &UA2865257},
	{SG89272, &UA3014843},
	{SG89273, &UA3070632},
	{SG89274, &UA174118},
	{SG89275, &UA379781},
	{SG89276, &UA591401},
	{SG89277, &UA1894574},
	{SG89278, &UA2333385},
	{SG89280, &UA2502696},
	{SG89281, &UA2603276},
	{SG89282, &UA394251},
	{SG89283, &UA420409},
	{SG89284, &UA553091},
	{SG89285, &UA873473},
	{SG89286, &UA879901},
	{SG89287, &UA1879419},
	{SG89288, &UA1887466},
	{SG89289, &UA1890997},
	{SG89290, &UA1988842},
	{SG89291, &UA2137930},
	{SG89292, &UA2162002},
	{SG89293, &UA2258677},
	{SG89294, &UA2292043},
	{SG89295, &UA2402360},
	{SG89296, &UA2438320},
	{SG89297, &UA2451709},
	{SG89298, &UA2456311},
	{SG89299, &UA2493953},
	{SG89300, &UA2553614},
	{SG89301, &UA2820587},
	{SG89302, &UA2859961},
	{SG89303, &UA3018199},
	{SG89304, &UA224547},
	{SG89305, &UA227260},
	{SG89306, &UA228719},
	{SG89307, &UA232652},
	{SG89308, &UA250935},
	{SG89309, &UA251022},
	{SG89310, &UA251075},
	{SG89311, &UA344505},
	{SG89312, &UA347421},
	{SG89313, &UA350562},
	{SG89314, &UA428873},
	{SG89315, &UA428884},
	{SG89316, &UA440932},
	{SG89317, &UA615810},
	{SG89318, &UA622555},
	{SG89319, &UA626982},
	{SG89320, &UA641419},
	{SG89321, &UA647853},
	{SG89322, &UA688410},
	{SG89323, &UA691358},
	{SG89324, &UA714921},
	{SG89325, &UA800802},
	{SG89326, &UA805496},
	{SG89327, &UA857337},
	{SG89328, &UA2158936},
	{SG89329, &UA2586542},
	{SG89330, &UA2586545},
	{SG89331, &UA1181},
	{SG89332, &UA1184},
	{SG89333, &UA4074},
	{SG89334, &UA4081},
	{SG89335, &UA12124},
	{SG89336, &UA12125},
	{SG89337, &UA12136},
	{SG89338, &UA12139},
	{SG89339, &UA17087},
	{SG89340, &UA17093},
	{SG89341, &UA17097},
	{SG89342, &UA17120},
	{SG89343, &UA23541},
	{SG89344, &UA23552},
	{SG89345, &UA23559},
	{SG89346, &UA23561},
	{SG89347, &UA23601},
	{SG89348, &UA36354},
	{SG89349, &UA61805},
	{SG89350, &UA66312},
	{SG89351, &UA66420},
	{SG89352, &UA75809},
	{SG89353, &UA95312},
	{SG89354, &UA97667},
	{SG89355, &UA98490},
	{SG89356, &UA98990},
	{SG89357, &UA99692},
	{SG89358, &UA99787},
	{SG89359, &UA100066},
	{SG89360, &UA110853},
	{SG89361, &UA115803},
	{SG89362, &UA116555},
	{SG89363, &UA117764},
	{SG89364, &UA118027},
	{SG89365, &UA119450},
	{SG89366, &UA122249},
	{SG89367, &UA122297},
	{SG89368, &UA122681},
	{SG89369, &UA125585},
	{SG89370, &UA131212},
	{SG89371, &UA132955},
	{SG89372, &UA133236},
	{SG89373, &UA133336},
	{SG89374, &UA133865},
	{SG89375, &UA135873},
	{SG89376, &UA137076},
	{SG89377, &UA137374},
	{SG89378, &UA138046},
	{SG89379, &UA138608},
	{SG89380, &UA141686},
	{SG89381, &UA143415},
	{SG89382, &UA147234},
	{SG89383, &UA149968},
	{SG89384, &UA157230},
	{SG89385, &UA165428},
	{SG89386, &UA166175},
	{SG89387, &UA183930},
	{SG89388, &UA228965},
	{SG89389, &UA230416},
	{SG89390, &UA260736},
	{SG89391, &UA268036},
	{SG89392, &UA271998},
	{SG89393, &UA274786},
	{SG89394, &UA276124},
	{SG89395, &UA283438},
	{SG89396, &UA297836},
	{SG89397, &UA303065},
	{SG89398, &UA329260},
	{SG89399, &UA371711},
	{SG89400, &UA373268},
	{SG89402, &UA384717},
	{SG89403, &UA388908},
	{SG89405, &UA389183},
	{SG89406, &UA390827},
	{SG89407, &UA414561},
	{SG89408, &UA418332},
	{SG89410, &UA432599},
	{SG89412, &UA439785},
	{SG89414, &UA447100},
	{SG89415, &UA462249},
	{SG89416, &UA474782},
	{SG89418, &UA475445},
	{SG89419, &UA481642},
	{SG89420, &UA482086},
	{SG89421, &UA483302},
	{SG89422, &UA483773},
	{SG89423, &UA486311},
	{SG89424, &UA491948},
	{SG89425, &UA497840},
	{SG89426, &UA506094},
	{SG89427, &UA511292},
	{SG89428, &UA523786},
	{SG89429, &UA543428},
	{SG89430, &UA577709},
	{SG89431, &UA582586},
	{SG89432, &UA624904},
	{SG89433, &UA710933},
	{SG89434, &UA711680},
	{SG89435, &UA713158},
	{SG89436, &UA714758},
	{SG89437, &UA726642},
	{SG89438, &UA731287},
	{SG89439, &UA736130},
	{SG89440, &UA750723},
	{SG89441, &UA756801},
	{SG89442, &UA768591},
	{SG89443, &UA782372},
	{SG89444, &UA784876},
	{SG89445, &UA791022},
	{SG89446, &UA809651},
	{SG89447, &UA836012},
	{SG89448, &UA866191},
	{SG89449, &UA868820},
	{SG89450, &UA892985},
	{SG89451, &UA893369},
	{SG89452, &UA899694},
	{SG89453, &UA1836250},
	{SG89454, &UA1836797},
	{SG89455, &UA1840508},
	{SG89456, &UA1841845},
	{SG89457, &UA1855793},
	{SG89458, &UA1865962},
	{SG89459, &UA1873624},
	{SG89460, &UA1891636},
	{SG89461, &UA2126769},
	{SG89462, &UA2128842},
	{SG89463, &UA2131781},
	{SG89464, &UA2135098},
	{SG89465, &UA2212191},
	{SG89466, &UA2264256},
	{SG89467, &UA2267858},
	{SG89468, &UA2274793},
	{SG89469, &UA2274915},
	{SG89470, &UA2290710},
	{SG89471, &UA2300329},
	{SG89472, &UA2437721},
	{SG89473, &UA2438254},
	{SG89474, &UA2439501},
	{SG89475, &UA2774607},
	{SG89476, &UA2801900},
	{SG89477, &UA2802437},
	{SG89478, &UA2948841},
	{SG89479, &UA3069173},
	{SG89480, &UA3069248},
	{SG89481, &UA94041},
	{SG89482, &UA98159},
	{SG89483, &UA105267},
	{SG89484, &UA154073},
	{SG89485, &UA176153},
	{SG89486, &UA279063},
	{SG89487, &UA408574},
	{SG89488, &UA1864606},
	{SG89489, &UA2271683},
	{SG89490, &UA2335529},
	{SG89622, &UA174921},
	{SG89623, &UA190766},
	{SG89624, &UA209291},
	{SG89625, &UA494751},
	{SG89626, &UA694949},
	{SG89627, &UA2127749},
	{SG89628, &UA73026},
	{SG89629, &UA2748154},
	{SG89729, &UA92392},
	{SG89730, &UA92821},
	{SG89731, &UA93032},
	{SG89732, &UA98985},
	{SG89733, &UA130672},
	{SG89734, &UA130899},
	{SG89735, &UA273497},
	{SG89736, &UA488917},
	{SG89737, &UA534759},
	{SG89738, &UA152480},
	{SG89739, &UA623356},
	{SG89740, &UA715741},
	{SG89741, &UA902913},
	{SG89742, &UA2209453},
	{SG89743, &UA2308308},
	{SG89744, &UA2656852},
	{SG89745, &UA2762263},
	{SG89746, &UA92881},
	{SG89747, &UA93275},
	{SG89748, &UA94275},
	{SG89749, &UA94403},
	{SG89750, &UA94837},
	{SG89751, &UA95948},
	{SG89752, &UA97812},
	{SG89753, &UA98956},
	{SG89754, &UA100810},
	{SG89755, &UA106757},
	{SG89756, &UA110937},
	{SG89757, &UA116641},
	{SG89758, &UA117988},
	{SG89759, &UA121407},
	{SG89760, &UA121726},
	{SG89761, &UA129486},
	{SG89762, &UA135473},
	{SG89763, &UA136837},
	{SG89764, &UA136845},
	{SG89765, &UA142117},
	{SG89766, &UA156086},
	{SG89767, &UA157723},
	{SG89768, &UA172046},
	{SG89769, &UA195618},
	{SG89770, &UA220048},
	{SG89771, &UA228298},
	{SG89772, &UA229412},
	{SG89773, &UA260425},
	{SG89774, &UA261914},
	{SG89775, &UA264759},
	{SG89776, &UA273271},
	{SG89777, &UA275430},
	{SG89778, &UA277130},
	{SG89779, &UA277493},
	{SG89780, &UA278993},
	{SG89781, &UA282806},
	{SG89782, &UA294895},
	{SG89783, &UA299519},
	{SG89784, &UA399793},
	{SG89785, &UA430979},
	{SG89786, &UA431449},
	{SG89787, &UA431515},
	{SG89788, &UA431948},
	{SG89789, &UA442064},
	{SG89790, &UA442137},
	{SG89791, &UA444304},
	{SG89792, &UA469065},
	{SG89793, &UA482646},
	{SG89794, &UA490572},
	{SG89795, &UA494802},
	{SG89796, &UA496329},
	{SG89797, &UA523078},
	{SG89798, &UA538322},
	{SG89799, &UA599068},
	{SG89800, &UA626924},
	{SG89801, &UA683246},
	{SG89802, &UA720620},
	{SG89803, &UA724057},
	{SG89804, &UA729878},
	{SG89805, &UA731210},
	{SG89806, &UA743825},
	{SG89807, &UA761672},
	{SG89808, &UA764902},
	{SG89809, &UA777144},
	{SG89810, &UA786537},
	{SG89811, &UA790278},
	{SG89812, &UA820667},
	{SG89813, &UA822158},
	{SG89814, &UA824301},
	{SG89815, &UA846050},
	{SG89816, &UA856991},
	{SG89817, &UA867462},
	{SG89818, &UA880976},
	{SG89819, &UA1840692},
	{SG89820, &UA1841198},
	{SG89821, &UA1850576},
	{SG89822, &UA1856811},
	{SG89823, &UA2125730},
	{SG89824, &UA2126736},
	{SG89825, &UA2128215},
	{SG89826, &UA2134937},
	{SG89827, &UA2137904},
	{SG89828, &UA2275063},
	{SG89829, &UA2282123},
	{SG89830, &UA2284593},
	{SG89831, &UA2720845},
	{SG89832, &UA174898},
	{SG89833, &UA192678},
	{SG89834, &UA285232},
	{SG89835, &UA623678},
	{SG89836, &UA671668},
	{SG89837, &UA719639},
	{SG89838, &UA741835},
	{SG89839, &UA92866},
	{SG89840, &UA98832},
	{SG89841, &UA99182},
	{SG89842, &UA103014},
	{SG89843, &UA128177},
	{SG89845, &UA131284},
	{SG89848, &UA414607},
	{SG89850, &UA2258993},
	{SG89852, &UA2424559},
	{SG89854, &UA2427406},
	{SG89856, &UA2467397},
	{SG89906, &UA93069},
	{SG89907, &UA99070},
	{SG89908, &UA99982},
	{SG89909, &UA100101},
	{SG89910, &UA125166},
	{SG89911, &UA132212},
	{SG89912, &UA149797},
	{SG89913, &UA165161},
	{SG89914, &UA382474},
	{SG89915, &UA659326},
	{SG89916, &UA671516},
	{SG89917, &UA730043},
	{SG89918, &UA791698},
	{SG89919, &UA872517},
	{SG89920, &UA2207371},
	{SG89921, &UA2409113},
	{SG89922, &UA3042319},
	{SG89923, &UA2773075},
	{SG89924, &UA2799020},
	{SG89925, &UA2869954},
	{SG89926, &UA2884688},
	{SG89927, &UA2898147},
	{SG89928, &UA2948546},
	{SG89929, &UA2968948},
	{SG89930, &UA2975790},
	{SG89931, &UA3024219},
	{SG89932, &UA3024279},
	{SG89933, &UA3031448},
	{SG89934, &UA3033287},
	{SG89935, &UA3033622},
	{SG89936, &UA3034251},
	{SG89937, &UA3036984},
	{SG89938, &UA3056871},
	{SG89939, &UA3062139},
	{SG89940, &UA3063749},
	{SG89941, &UA3064189},
	{SG89942, &UA3065072},
	{SG89943, &UA3066111},
	{SG89944, &UA3080926},
	{SG89945, &UA3083312},
	{SG89950, &UA569707},
	{SG89951, &UA120214},
	{SG89952, &UA261776},
	{SG89953, &UA273604},
	{SG89954, &UA576666},
	{SG89955, &UA3125583},
	{SG89956, &UA289354},
	{SG89957, &UA374524},
	{SG89958, &UA388386},
	{SG89959, &UA416465},
	{SG89960, &UA472115},
	{SG89961, &UA897050},
	{SG89962, &UA1849399},
	{SG89963, &UA1895366},
	{SG89964, &UA1896378},
	{SG89965, &UA1896840},
	{SG89966, &UA1902995},
	{SG89967, &UA2125993},
	{SG89968, &UA2140450},
	{SG89969, &UA2208861},
	{SG89970, &UA2258644},
	{SG89971, &UA2299387},
	{SG89972, &UA2339792},
	{SG89973, &UA2554234},
	{SG89974, &UA2757532},
	{SG89975, &UA2806593},
	{SG89976, &UA813356},
	{SG89977, &UA910766},
	{SG89978, &UA251312},
	{SG89979, &UA251376},
	{SG89980, &UA350831},
	{SG89981, &UA351428},
	{SG89982, &UA409450},
	{SG89983, &UA630000},
	{SG89984, &UA630343},
	{SG89985, &UA680181},
	{SG89986, &UA685963},
	{SG89987, &UA751785},
	{SG89988, &UA778923},
	{SG89989, &UA803329},
	{SG89990, &UA2586540},
	{SG89991, &UA2586543},
	{SG89992, &UA98533},
	{SG89993, &UA101400},
	{SG89994, &UA127183},
	{SG89995, &UA431148},
	{SG89999, &UA92212},
	{SG90000, &UA104964},
	{SG90001, &UA106373},
	{SG90002, &UA125200},
	{SG90003, &UA127071},
	{SG90004, &UA344870},
	{SG90005, &UA408500},
	{SG90006, &UA693511},
	{SG90007, &UA1875853},
	{SG90008, &UA2424088},
	{SG90009, &UA2458246},
	{SG90010, &UA2466757},
	{SG90011, &UA2205968},
	{SG90012, &UA2238808},
	{SG90013, &UA2258462},
	{SG90014, &UA2349203},
	{SG90015, &UA2357078},
	{SG90016, &UA2388425},
	{SG90017, &UA2416944},
	{SG90018, &UA2425162},
	{SG90019, &UA2429536},
	{SG90020, &UA2437498},
	{SG90021, &UA2561601},
	{SG90022, &UA2796394},
	{SG90023, &UA2797280},
	{SG90024, &UA2855392},
	{SG90025, &UA2886770},
	{SG90026, &UA2947262},
	{SG90027, &UA2974580},
	{SG90028, &UA3033899},
	{SG90029, &UA3036990},
	{SG90030, &UA3067788},
	{SG90031, &UA3094884},
	{SG90032, &UA2200464},
	{SG90033, &UA2201536},
	{SG90034, &UA2203302},
	{SG90035, &UA2203575},
	{SG90036, &UA2203770},
	{SG90037, &UA2205095},
	{SG90038, &UA2285918},
	{SG90039, &UA2301762},
	{SG90040, &UA2316908},
	{SG90041, &UA2419577},
	{SG90042, &UA2642933},
	{SG90043, &UA3042797},
	{SG90044, &UA348168},
	{SG90045, &UA348863},
	{SG90046, &UA2741632},
	{SG90049, &UA330340},
	{SG90050, &UA2462661},
	{SG90051, &UA2567661},
	{SG90070, &UA168768},
	{SG90071, &UA175399},
	{SG90072, &UA193118},
	{SG90073, &UA99997},
	{SG90074, &UA100500},
	{SG90075, &UA130896},
	{SG90076, &UA2652975},
	{SG90126, &UA325564},
	{SG90127, &UA430005},
	{SG90128, &UA430108},
	{SG90129, &UA2020583},
	{SG90130, &UA287937},
	{SG90131, &UA419909},
	{SG90132, &UA2199041},
	{SG90133, &UA2232157},
	{SG90134, &UA2258979},
	{SG90135, &UA2355829},
	{SG90136, &UA2362774},
	{SG90137, &UA2386154},
	{SG90138, &UA2468961},
	{SG90139, &UA2485074},
	{SG90140, &UA2532575},
	{SG90141, &UA2536738},
	{SG90142, &UA2653030},
	{SG90143, &UA2672957},
	{SG90144, &UA2691466},
	{SG90145, &UA2709229},
	{SG90146, &UA2726666},
	{SG90147, &UA2731977},
	{SG90148, &UA2738724},
	{SG90149, &UA2781846},
	{SG90150, &UA2784910},
	{SG90151, &UA2844076},
	{SG90152, &UA3023748},
	{SG90153, &UA3091438},
	{SG90154, &UA3094465},
	{SG90155, &UA2433594},
	{SG90156, &UA834237},
	{SG90157, &UA1969040},
	{SG90158, &UA2200474},
	{SG90159, &UA2203666},
	{SG90160, &UA2203926},
	{SG90161, &UA2416448},
	{SG90162, &UA105724},
	{SG90163, &UA132796},
	{SG90164, &UA139470},
	{SG90165, &UA142512},
	{SG90166, &UA264952},
	{SG90167, &UA783790},
	{SG90168, &UA807953},
	{SG90169, &UA842705},
	{SG90170, &UA165655},
	{SG90171, &UA305057},
	{SG90172, &UA375007},
	{SG90173, &UA389629},
	{SG90174, &UA500089},
	{SG90175, &UA518529},
	{SG90176, &UA756617},
	{SG90177, &UA757122},
	{SG90178, &UA873114},
	{SG90179, &UA2137958},
	{SG90180, &UA2975529},
	{SG90183, &UA614724},
	{SG90184, &UA2580100},
	{SG90190, &UA2383934},
	{SG90191, &UA347129},
	{SG90192, &UA249681},
	{SG90193, &UA252560},
	{SG90194, &UA737926},
	{SG90195, &UA765626},
	{SG90196, &UA575308},
	{SG90197, &UA699599},
	{SG90198, &UA785417},
	{SG90199, &UA850802},
	{SG90200, &UA1960188},
	{SG90201, &UA2439140},
	{SG90202, &UA2700288},
	{SG90203, &UA166326},
	{SG90233, &UA93287},
	{SG90234, &UA93322},
	{SG90235, &UA104972},
	{SG90236, &UA122454},
	{SG90237, &UA278856},
	{SG90238, &UA898613},
	{SG90239, &UA899220},
	{SG90240, &UA1861669},
	{SG90241, &UA1876197},
	{SG90242, &UA1953429},
	{SG90243, &UA2302447},
	{SG90244, &UA608512},
	{SG90245, &UA349591},
	{SG90246, &UA401695},
	{SG90247, &UA440551},
	{SG90248, &UA454825},
	{SG90249, &UA603814},
	{SG90250, &UA614889},
	{SG90251, &UA801591},
	{SG90252, &UA814876},
	{SG90253, &UA842701},
	{SG90254, &UA2596286},
	{SG90255, &UA2599867},
	{SG90256, &UA2973153},
	{SG90257, &UA251959},
	{SG90258, &UA292320},
	{SG90259, &UA345433},
	{SG90260, &UA433564},
	{SG90261, &UA468052},
	{SG90262, &UA1932632},
	{SG90263, &UA2205324},
	{SG90264, &UA2208483},
	{SG90265, &UA2355673},
	{SG90266, &UA2368538},
	{SG90267, &UA2370155},
	{SG90268, &UA2396672},
	{SG90269, &UA2423673},
	{SG90270, &UA2505960},
	{SG90271, &UA2532390},
	{SG90272, &UA2543619},
	{SG90273, &UA2588150},
	{SG90274, &UA2599958},
	{SG90275, &UA2716715},
	{SG90276, &UA2752922},
	{SG90277, &UA2796778},
	{SG90278, &UA2804993},
	{SG90279, &UA3042860},
	{SG90280, &UA92452},
	{SG90281, &UA93289},
	{SG90282, &UA104777},
	{SG90283, &UA106247},
	{SG90284, &UA136816},
	{SG90285, &UA414739},
	{SG90286, &UA2126957},
	{SG90295, &UA6529},
	{SG90296, &UA9700},
	{SG90297, &UA42301},
	{SG90298, &UA741207},
	{SG90301, &UA2740292},
	{SG90302, &UA2740956},
	{SG90303, &UA2754139},
	{SG90304, &UA2755112},
	{SG90305, &UA2756552},
	{SG90306, &UA2769992},
	{SG90307, &UA2781952},
	{SG90308, &UA2782366},
	{SG90309, &UA2782536},
	{SG90310, &UA2798754},
	{SG90311, &UA2798875},
	{SG90312, &UA2869336},
	{SG90313, &UA2879154},
	{SG90314, &UA2892570},
	{SG90315, &UA2937636},
	{SG90316, &UA2937908},
	{SG90317, &UA2957281},
	{SG90318, &UA2996883},
	{SG90319, &UA2998231},
	{SG90320, &UA2998915},
	{SG90321, &UA2999549},
	{SG90322, &UA2999613},
	{SG90323, &UA3001254},
	{SG90324, &UA3049464},
	{SG90325, &UA3049946},
	{SG90326, &UA3073682},
	{SG90327, &UA3073782},
	{SG90328, &UA3073816},
	{SG90329, &UA3074149},
	{SG90330, &UA3088761},
	{SG90331, &UA3102987},
	{SG90332, &UA3103098},
	{SG90333, &UA3103150},
	{SG90334, &UA3121672},
	{SG90335, &UA141098},
	{SG90336, &UA236689},
	{SG90337, &UA873806},
	{SG90338, &UA218999},
	{SG90339, &UA227919},
	{SG90340, &UA575492},
	{SG90341, &UA2640332},
	{SG90343, &UA2461947},
	{SG90345, &UA229830},
	{SG90346, &UA333407},
	{SG90347, &UA720172},
	{SG90348, &UA2674327},
	{SG90349, &UA2194197},
	{SG90350, &UA2260165},
	{SG90351, &UA243904},
	{SG90352, &UA708369},
	{SG90353, &UA2568081},
	{SG90354, &UA251965},
	{SG90355, &UA419232},
	{SG90401, &UA2965085},
	{SG90402, &UA2966821},
	{SG90403, &UA30720},
	{SG90404, &UA167590},
	{SG90405, &UA328925},
	{SG90406, &UA486268},
	{SG90407, &UA650564},
	{SG90408, &UA2166872},
	{SG90412, &UA251925},
	{SG90440, &UA167736},
	{SG90441, &UA1992961},
	{SG90442, &UA1996618},
	{SG90443, &UA2040830},
	{SG90444, &UA2204649},
	{SG90445, &UA2229327},
	{SG90446, &UA2236274},
	{SG90447, &UA2340560},
	{SG90448, &UA2382299},
	{SG90449, &UA2555179},
	{SG90450, &UA2695557},
	{SG90451, &UA2705968},
	{SG90452, &UA2763460},
	{SG90453, &UA2780787},
	{SG90454, &UA2835519},
	{SG90455, &UA2939422},
	{SG90456, &UA3002732},
	{SG90457, &UA3008436},
	{SG90458, &UA3092860},
	{SG90461, &UA399553},
	{SG90462, &UA2540815},
	{SG90463, &UA99700},
	{SG90464, &UA101718},
	{SG90465, &UA123729},
	{SG90466, &UA2216562},
	{SG90467, &UA161973},
	{SG90469, &UA178198},
	{SG90470, &UA196400},
	{SG90471, &UA200827},
	{SG90472, &UA211834},
	{SG90473, &UA306834},
	{SG90474, &UA326372},
	{SG90475, &UA359649},
	{SG90476, &UA408016},
	{SG90477, &UA491138},
	{SG90478, &UA684273},
	{SG90479, &UA697468},
	{SG90480, &UA864455},
	{SG90481, &UA876667},
	{SG90482, &UA1859295},
	{SG90483, &UA1997050},
	{SG90484, &UA2296054},
	{SG90485, &UA2981113},
	{SG90488, &UA160231},
	{SG90489, &UA225389},
	{SG90490, &UA361072},
	{SG90491, &UA470814},
	{SG90492, &UA1996904},
	{SG90493, &UA2981061},
	{SG90494, &UA3018290},
	{SG90495, &UA105060},
	{SG90496, &UA263490},
	{SG90497, &UA263793},
	{SG90498, &UA112082},
	{SG90499, &UA145026},
	{SG90500, &UA259535},
	{SG90501, &UA387194},
	{SG90502, &UA724853},
	{SG90503, &UA2304293},
	{SG90504, &UA2758212},
	{SG90505, &UA608441},
	{SG90506, &UA2192285},
	{SG90507, &UA122457},
	{SG90510, &UA2897662},
	{SG90511, &UA92009},
	{SG90512, &UA129398},
	{SG90513, &UA262322},
	{SG90514, &UA274328},
	{SG90515, &UA373866},
	{SG90516, &UA386386},
	{SG90517, &UA387108},
	{SG90518, &UA500255},
	{SG90519, &UA704633},
	{SG90520, &UA742747},
	{SG90521, &UA840570},
	{SG90522, &UA1861038},
	{SG90523, &UA1863001},
	{SG90524, &UA1920653},
	{SG90525, &UA2159239},
	{SG90526, &UA2159742},
	{SG90527, &UA2258281},
	{SG90528, &UA2269484},
	{SG90529, &UA2274823},
	{SG90530, &UA2669934},
	{SG90531, &UA2955890},
	{SG90532, &UA2960782},
	{SG90533, &UA850168},
	{SG90534, &UA2459924},
	{SG90535, &UA2683835},
	{SG90536, &UA2424319},
	{SG90537, &UA2528334},
	{SG90538, &UA2564196},
	{SG90539, &UA2654927},
	{SG90540, &UA2665022},
	{SG90541, &UA2869890},
	{SG90542, &UA2988711},
	{SG90543, &UA3007712},
	{SG90544, &UA3024253},
	{SG90545, &UA3044104},
	{SG90546, &UA3057364},
	{SG90547, &UA3078732},
	{SG90548, &UA2973132},
	{SG90549, &UA99044},
	{SG90550, &UA275094},
	{SG90551, &UA431085},
	{SG90552, &UA771077},
	{SG90553, &UA809665},
	{SG90554, &UA816885},
	{SG90555, &UA837163},
	{SG90556, &UA897217},
	{SG90557, &UA143709},
	{SG90558, &UA347089},
	{SG90559, &UA555977},
	{SG90560, &UA704570},
	{SG90561, &UA2383129},
	{SG90562, &UA2385193},
	{SG90563, &UA2394163},
	{SG90564, &UA2409329},
	{SG90565, &UA2456630},
	{SG90566, &UA2595673},
	{SG90567, &UA2768736},
	{SG90568, &UA2793738},
	{SG90569, &UA2868770},
	{SG90570, &UA233175},
	{SG90571, &UA552522},
	{SG90572, &UA2324192},
	{SG90573, &UA2569607},
	{SG90574, &UA3111727},
	{SG90575, &UA241938},
	{SG90576, &UA323730},
	{SG90577, &UA359122},
	{SG90578, &UA583824},
	{SG90579, &UA659359},
	{SG90580, &UA681861},
	{SG90581, &UA740239},
	{SG90582, &UA1984842},
	{SG90583, &UA1996217},
	{SG90584, &UA2158877},
	{SG90585, &UA2199417},
	{SG90586, &UA2201856},
	{SG90587, &UA2352571},
	{SG90588, &UA2838144},
	{SG90589, &UA2838625},
	{SG90590, &UA2490686},
	{SG90591, &UA2509818},
	{SG90592, &UA2515422},
	{SG90593, &UA2563613},
	{SG90594, &UA2569928},
	{SG90595, &UA2583150},
	{SG90596, &UA2606674},
	{SG90597, &UA2670914},
	{SG90598, &UA2671897},
	{SG90599, &UA2675244},
	{SG90600, &UA2713781},
	{SG90601, &UA2713967},
	{SG90602, &UA2715810},
	{SG90603, &UA2740167},
	{SG90604, &UA2755382},
	{SG90605, &UA2756697},
	{SG90612, &UA2230179},
	{SG90613, &UA2235386},
	{SG90614, &UA2502218},
	{SG90615, &UA2554231},
	{SG90616, &UA2652638},
	{SG90617, &UA2668696},
	{SG90618, &UA2872141},
	{SG90619, &UA2997351},
	{SG90620, &UA3008717},
	{SG90621, &UA650854},
	{SG90622, &UA660583},
	{SG90623, &UA734798},
	{SG90624, &UA746273},
	{SG90625, &UA758561},
	{SG90626, &UA794714},
	{SG90627, &UA1958131},
	{SG90628, &UA2203573},
	{SG90629, &UA2233544},
	{SG90630, &UA2585272},
	{SG90631, &UA2781675},
	{SG90632, &UA2866706},
	{SG90633, &UA2566766},
	{SG90634, &UA2571288},
	{SG90635, &UA28091},
	{SG90636, &UA252026},
	{SG90637, &UA173269},
	{SG90638, &UA814105},
	{SG90639, &UA2207080},
	{SG90640, &UA2802069},
	{SG90641, &UA2389073},
	{SG90642, &UA2570956},
	{SG90646, &UA703097},
	{SG90647, &UA2570280},
	{SG90648, &UA2412202},
	{SG90649, &UA2752783},
	{SG90650, &UA2754059},
	{SG90651, &UA2672534},
	{SG90652, &UA2674730},
	{SG90653, &UA697454},
	{SG90654, &UA2411087},
	{SG90655, &UA2495914},
	{SG90656, &UA2815754},
	{SG90657, &UA2407103},
	{SG90658, &UA346966},
	{SG90659, &UA663016},
	{SG90660, &UA724388},
	{SG90661, &UA92820},
	{SG90662, &UA2207517},
	{SG90663, &UA2267892},
	{SG90664, &UA2410175},
	{SG90665, &UA451670},
	{SG90666, &UA2944805},
	{SG90667, &UA2961997},
	{SG90668, &UA415748},
	{SG90669, &UA725289},
	{SG90670, &UA2139772},
	{SG90671, &UA2295069},
	{SG90672, &UA2973996},
	{SG90673, &UA383220},
	{SG90689, &UA2039621},
	{SG90690, &UA2589933},
	{SG90691, &UA203968},
	{SG90692, &UA2769319},
	{SG90693, &UA2782525},
	{SG90694, &UA108962},
	{SG90695, &UA2972871},
	{SG90696, &UA640729},
	{SG90697, &UA658793},
	{SG90698, &UA2258175},
	{SG90699, &UA2332578},
	{SG90700, &UA2643747},
	{SG90701, &UA2889795},
	{SG90702, &UA3019972},
	{SG90703, &UA2582851},
	{SG90704, &UA179238},
	{SG90705, &UA451782},
	{SG90706, &UA289127},
	{SG90707, &UA2772427},
	{SG90708, &UA2885326},
	{SG90709, &UA3105818},
	{SG90710, &UA745005},
	{SG90711, &UA14729},
	{SG90712, &UA825961},
	{SG90713, &UA878171},
	{SG90714, &UA2268114},
	{SG90715, &UA2299623},
	{SG90716, &UA2409006},
	{SG90717, &UA2723615},
	{SG90718, &UA2791235},
	{SG90719, &UA2983310},
	{SG90720, &UA2985204},
	{SG90721, &UA2879683},
	{SG90722, &UA2972546},
	{SG90723, &UA3001414},
	{SG90724, &UA3028198},
	{SG90725, &UA3030447},
	{SG90726, &UA3039372},
	{SG90727, &UA3080547},
	{SG90728, &UA3095662},
	{SG90729, &UA367426},
	{SG90730, &UA848198},
	{SG90731, &UA2974393},
	{SG90763, &UA567790},
	{SG90764, &UA2507788},
	{SG90809, &UA2329604},
	{SG90810, &UA195442},
	{SG90811, &UA242542},
	{SG90812, &UA568284},
	{SG90813, &UA690676},
	{SG90814, &UA97465},
	{SG90815, &UA272913},
	{SG90816, &UA486980},
	{SG90817, &UA3099299},
	{SG90818, &UA367451},
	{SG90819, &UA611826},
	{SG90820, &UA6535},
	{SG90821, &UA3115088},
	{SG90822, &UA2194866},
	{SG90830, &UA2769557},
	{SG90834, &UA2235996},
	{SG90835, &UA451556},
	{SG90836, &UA1964997},
	{SG90837, &UA807170},
	{SG90838, &UA871974},
	{SG90839, &UA2757743},
	{SG90840, &UA2258466},
	{SG90841, &UA2537202},
	{SG90842, &UA2560314},
	{SG90843, &UA640310},
	{SG90844, &UA877408},
	{SG90845, &UA2334726},
	{SG90846, &UA2373026},
	{SG90847, &UA223071},
	{SG90848, &UA191213},
	{SG90849, &UA194233},
	{SG90850, &UA2182785},
	{SG90851, &UA2202378},
	{SG90852, &UA2583995},
	{SG90853, &UA2869114},
	{SG90854, &UA676258},
	{SG90855, &UA756876},
	{SG90856, &UA2167032},
	{SG90857, &UA2341092},
	{SG90858, &UA703023},
	{SG90859, &UA2294473},
	{SG90860, &UA2309370},
	{SG90861, &UA571324},
	{SG90862, &UA214126},
	{SG90863, &UA2352512},
	{SG90869, &UA98033},
	{SG90870, &UA2364093},
	{SG90871, &UA226990},
	{SG90872, &UA1902753},
	{SG90873, &UA191934},
	{SG90874, &UA285276},
	{SG90875, &UA2200640},
	{SG90876, &UA2558452},
	{SG90877, &UA396897},
	{SG90878, &UA136042},
	{SG90879, &UA2507907},
	{SG90881, &UA2210090},
	{SG90882, &UA2213031},
	{SG90883, &UA2741936},
	{SG90884, &UA2423541},
	{SG90885, &UA2475287},
	{SG90886, &UA2517929},
	{SG90887, &UA2518212},
	{SG90888, &UA3094191},
	{SG90889, &UA2898939},
	{SG90890, &UA210686},
	{SG90891, &UA344364},
	{SG90904, &UA761839},
	{SG90908, &UA2945998},
	{SG90909, &UA91852},
	{SG90910, &UA97673},
	{SG90911, &UA2945610},
	{SG90912, &UA394945},
	{SG90913, &UA781662},
	{SG90914, &UA810634},
	{SG90915, &UA1907804},
	{SG90919, &UA2567159},
	{SG90920, &UA2601176},
	{SG90921, &UA2679267},
	{SG90922, &UA227121},
	{SG90943, &UA481549},
	{SG90944, &UA558184},
	{SG90945, &UA701560},
	{SG90946, &UA875886},
	{SG90947, &UA2999085},
	{SG90948, &UA88447},
	{SG90949, &UA2739965},
	{SG90950, &UA2796893},
	{SG90952, &UA777774},
	{SG90957, &UA760123},
	{SG90958, &UA2351985},
	{SG90959, &UA2362446},
	{SG90960, &UA3050744},
	{SG90963, &UA136947},
	{SG90992, &UA2437608},
	{SG90993, &UA2511344},
	{SG90994, &UA2512581},
	{SG90996, &UA2356421},
	{SG90997, &UA539145},
	{SG90998, &UA2977941},
	{SG90999, &UA452166},
	{SG91000, &UA350988},
	{SG91001, &UA841229},
	{SG91002, &UA881462},
	{SG91003, &UA1892292},
	{SG91004, &UA1906200},
	{SG91005, &UA1991037},
	{SG91006, &UA2278159},
	{SG91007, &UA2757529},
	{SG91008, &UA2859553},
	{SG91009, &UA2937056},
	{SG91010, &UA2290647},
	{SG91012, &UA2659755},
	{SG91013, &UA2855754},
	{SG91014, &UA3079735},
	{SG91015, &UA2233713},
	{SG91016, &UA133275},
	{SG91017, &UA430462},
	{SG91018, &UA719632},
	{SG91019, &UA2783869},
	{SG91020, &UA2336448},
	{SG91021, &UA2232557},
	{SG91022, &UA1908498},
	{SG91031, &UA471298},
	{SG91032, &UA2294355},
	{SG91033, &UA525594},
	{SG91034, &UA2868225},
	{SG91035, &UA2901068},
	{SG91036, &UA338716},
	{SG91037, &UA409486},
	{SG91038, &UA571043},
	{SG91039, &UA812860},
	{SG91040, &UA1896531},
	{SG91041, &UA1904138},
	{SG91043, &UA414627},
	{SG91044, &UA2411839},
	{SG91045, &UA165748},
	{SG91046, &UA50308},
	{SG91047, &UA2977682},
	{SG91048, &UA1890739},
	{SG91049, &UA281666},
	{SG91050, &UA304003},
	{SG91051, &UA354515},
	{SG91052, &UA2221968},
	{SG91053, &UA2394535},
	{SG91055, &UA101805},
	{SG91056, &UA2258394},
	{SG91057, &UA2668942},
	{SG91058, &UA3001023},
	{SG91059, &UA701423},
	{SG91060, &UA1874817},
	{SG91061, &UA2300074},
	{SG91062, &UA2681110},
	{SG91063, &UA3010659},
	{SG91064, &UA2461246},
	{SG91065, &UA2596844},
	{SG91066, &UA121718},
	{SG91067, &UA753131},
	{SG91068, &UA2166658},
	{SG91069, &UA2258380},
	{SG91070, &UA2453891},
	{SG91071, &UA2511628},
	{SG91072, &UA796347},
	{SG91073, &UA2270820},
	{SG91074, &UA2426718},
	{SG91075, &UA2258709},
	{SG91076, &UA883427},
	{SG91077, &UA2901206},
	{SG91083, &UA2369312},
	{SG91084, &UA2425984},
	{SG91085, &UA2454406},
	{SG91088, &UA541221},
	{SG91090, &UA2405952},
	{SG91093, &UA1882289},
	{SG91094, &UA378124},
	{SG91095, &UA378841},
	{SG91096, &UA506488},
	{SG91097, &UA2164037},
	{SG91098, &UA1948732},
	{SG91099, &UA2405769},
	{SG91100, &UA2936740},
	{SG91102, &UA1995966},
	{SG91103, &UA3017196},
	{SG91104, &UA800912},
	{SG91105, &UA2706947},
	{SG91107, &UA659110},
	{SG91108, &UA2233791},
	{SG91110, &UA290402},
	{SG91111, &UA2409182},
	{SG91112, &UA1990916},
	{SG91113, &UA1874556},
	{SG91114, &UA2332125},
	{SG91115, &UA2372606},
	{SG91116, &UA2413120},
	{SG91118, &UA2800527},
	{SG91119, &UA2157992},
	{SG91120, &UA2433212},
	{SG91123, &UA203023},
	{SG91124, &UA2467310},
	{SG91125, &UA589133},
	{SG91126, &UA746112},
	{SG91127, &UA2382887},
	{SG91128, &UA2458827},
	{SG91131, &UA2495681},
	{SG91140, &UA2866426},
	{SG91141, &UA3054261},
	{SG91142, &UA718190},
	{SG91143, &UA2823686},
	{SG91144, &UA2506289},
	{SG91145, &UA2580744},
	{SG91146, &UA2735007},
	{SG91147, &UA2397974},
	{SG91148, &UA2476143},
	{SG91149, &UA2425732},
	{SG91150, &UA2866097},
	{SG91152, &UA2400208},
	{SG91153, &UA2794367},
	{SG91154, &UA2567830},
	{SG91155, &UA2669665},
	{SG91156, &UA3115755},
	{SG91157, &UA292189},
	{SG91158, &UA3013827},
	{SG91159, &UA392663},
	{SG91160, &UA837632},
	{SG91162, &UA2219680},
	{SG91164, &UA799414},
	{SG91165, &UA279591},
	{SG91166, &UA2795993},
	{SG91170, &UA1892765},
	{SG91172, &UA2958043},
	{SG91173, &UA3050716},
	{SG91174, &UA776458},
	{SG91175, &UA2999523},
	{SG91176, &UA547678},
	{SG91177, &UA710855},
	{SG91178, &UA1962418},
	{SG91179, &UA726542},
	{SG91180, &UA2166339},
	{SG91181, &UA2481790},
	{SG91182, &UA2207578},
	{SG91183, &UA2269948},
	{SG91184, &UA2274311},
	{SG91187, &UA3015912},
	{SG91188, &UA2362408},
	{SG91189, &UA280309},
	{SG91190, &UA1932158},
	{SG91191, &UA324105},
	{SG91194, &UA3044439},
	{SG91195, &UA1988426},
	{SG91196, &UA2691411},
	{SG91197, &UA2871703},
	{SG91198, &UA1949227},
	{SG91202, &UA1955429},
	{SG91213, &UA2409342},
	{SG91214, &UA2510377},
	{SG91215, &UA2505061},
	{SG91223, &UA2692922},
	{SG91226, &UA730181},
	{SG91227, &UA2142032},
	{SG91228, &UA2258658},
	{SG91229, &UA2258668},
	{SG91230, &UA2364687},
	{SG91233, &UA2671912},
	{SG91237, &UA763670},
	{SG91239, &UA2643828},
	{SG91243, &UA3117936},
	{SG91248, &UA2500133},
	{SG91249, &UA2994679},
	{SG91250, &UA1855036},
	{SG91251, &UA2320735},
	{SG91253, &UA3074353},
	{SG91254, &UA2418152},
	{SG91255, &UA2722676},
	{SG91256, &UA2218964},
	{SG91257, &UA2810325},
	{SG91258, &UA400965},
	{SG91259, &UA1855001},
	{SG91260, &UA1855003},
	{SG91262, &UA2321308},
	{SG91263, &UA2531190},
	{SG91265, &UA2485346},
	{SG91269, &UA2315620},
	{SG91270, &UA2838073},
	{SG91271, &UA2985127},
	{SG91272, &UA3069994},
	{SG91275, &UA2834723},
	{SG91281, &UA2552198},
	{SG91286, &UA2482846},
	{SG91289, &UA745627},
	{SG91292, &UA2572752},
	{SG91293, &UA2818181},
	{SG91294, &UA2841776},
	{SG91302, &UA2518844},
	{SG91303, &UA2542450},
	{SG91323, &UA475253},
	{SG91324, &UA2519751},
	{SG91325, &UA6311},
	{SG91326, &UA590271},
	{SG91342, &UA1878606},
	{SG91343, &UA819508},
	{SG91345, &UA531202},
	{SG91346, &UA2617006},
	{SG91347, &UA2619650},
	{SG91383, &UA733121},
	{SG91384, &UA896182},
	{SG91385, &UA2410536},
	{SG91386, &UA2619629},
	{SG91387, &UA2622169},
	{SG91388, &UA16422},
	{SG91389, &UA842221},
	{SG91390, &UA1998539},
	{SG91393, &UA1896857},
	{SG91394, &UA1990599},
	{SG91395, &UA2171185},
	{SG91398, &UA590385},
	{SG91399, &UA2003826},
	{SG91400, &UA2316605},
	{SG91401, &UA2616863},
	{SG91402, &UA2622695},
	{SG91405, &UA3040391},
	{SG91406, &UA80021},
	{NULL}};
const Device *DSagem[] = {
	&UA1603,
	&UA1602,
	&UA17203,
	&UA24663,
	&UA30703,
	&UA55038,
	&UA55039,
	&UA62276,
	&UA63511,
	&UA64704,
	&UA63361,
	&UA64135,
	&UA459888,
	&UA32625,
	&UA32626,
	&UA32624,
	&UA51351,
	&UA51352,
	&UA54289,
	&UA63712,
	&UA13057,
	&UA13058,
	&UA13059,
	&UA40794,
	&UA43856,
	&UA87027,
	&UA754441,
	&UA10251,
	&UA30284,
	&UA83191,
	&UA63858,
	&UA54592,
	&UA24012,
	&UA26722,
	&UA738145,
	&UA32740,
	&UA54804,
	&UA21422,
	&UA32802,
	&UA51701,
	&UA63980,
	&UA64036,
	&UA86680,
	&UA64395,
	&UA44178,
	&UA63230,
	&UA24668,
	&UA22951,
	&UA50047,
	&UA30262,
	&UA49825,
	&UA60853,
	&UA65638,
	&UA83133,
	&UA83139,
	&UA83273,
	&UA83205,
	&UA88899,
	&UA88963,
	NULL};
const Segmentdata DSamsung[] = {
	{SG90917, &UA3063453},
	{SG90918, &UA3109140},
	{SG90951, &UA3067241},
	{SG90961, &UA3062173},
	{SG90962, &UA3065844},
	{SG90964, &UA2527122},
	{SG90995, &UA2654327},
	{SG91042, &UA3094044},
	{SG91089, &UA2618788},
	{SG91091, &UA2623743},
	{SG91092, &UA2636979},
	{SG91106, &UA3088134},
	{SG91109, &UA3109278},
	{SG91151, &UA3126007},
	{SG91161, &UA2622917},
	{SG91163, &UA3066990},
	{SG91167, &UA2620810},
	{SG91168, &UA2621651},
	{SG91169, &UA2623245},
	{SG91171, &UA2765994},
	{SG91199, &UA2612747},
	{SG91200, &UA2619076},
	{SG91201, &UA2627788},
	{SG91203, &UA3109333},
	{SG91206, &UA2619069},
	{SG91207, &UA2619289},
	{SG91208, &UA2620266},
	{SG91210, &UA2627627},
	{SG91211, &UA2636627},
	{SG91216, &UA2618868},
	{SG91217, &UA2622786},
	{SG91218, &UA2626625},
	{SG91219, &UA2628400},
	{SG91220, &UA2618299},
	{SG91221, &UA2622993},
	{SG91222, &UA2624005},
	{SG91224, &UA2626772},
	{SG91231, &UA2621901},
	{SG91232, &UA2623527},
	{SG91234, &UA2633857},
	{SG91235, &UA2624882},
	{SG91236, &UA2631557},
	{SG91238, &UA2626745},
	{SG91240, &UA2630417},
	{SG91241, &UA2633431},
	{SG91242, &UA2633586},
	{SG91244, &UA2621092},
	{SG91245, &UA2626123},
	{SG91246, &UA2631759},
	{SG91247, &UA2636467},
	{SG91261, &UA369220},
	{SG91264, &UA2629663},
	{SG91273, &UA3108961},
	{SG91274, &UA2631348},
	{SG91276, &UA3109448},
	{SG91277, &UA2629417},
	{SG91278, &UA2636335},
	{SG91279, &UA2620478},
	{SG91280, &UA2635242},
	{SG91284, &UA2618424},
	{SG91285, &UA2620397},
	{SG91301, &UA3067252},
	{SG91351, &UA3093726},
	{SG91391, &UA2623790},
	{SG91392, &UA2624386},
	{SG91396, &UA2619319},
	{SG91397, &UA2625716},
	{SG91403, &UA2632272},
	{SG91404, &UA3116442},
	{SG91407, &UA3125925},
	{SG91408, &UA2632305},
	{SG91409, &UA3094024},
	{SG91410, &UA2632153},
	{SG91411, &UA2621336},
	{SG91412, &UA2945951},
	{SG91413, &UA3058400},
	{SG91414, &UA3126277},
	{SG91415, &UA2945959},
	{SG91416, &UA637275},
	{SG91417, &UA3468},
	{SG91418, &UA20518},
	{SG91419, &UA20576},
	{SG91420, &UA53368},
	{SG91421, &UA63807},
	{SG91422, &UA925212},
	{SG91423, &UA3093635},
	{SG91424, &UA2945963},
	{SG91425, &UA3109288},
	{SG91426, &UA2258140},
	{SG91427, &UA2349262},
	{SG91428, &UA10216},
	{SG91429, &UA56057},
	{SG91433, &UA7027},
	{SG91434, &UA53671},
	{SG91435, &UA54963},
	{SG91436, &UA925194},
	{SG91437, &UA62414},
	{SG91438, &UA2204810},
	{SG91441, &UA1362},
	{SG91442, &UA5300},
	{SG91443, &UA53973},
	{SG91444, &UA64673},
	{SG91445, &UA81575},
	{SG91446, &UA88774},
	{SG91447, &UA88785},
	{SG91448, &UA94082},
	{SG91449, &UA442783},
	{SG91450, &UA679775},
	{SG91451, &UA2263267},
	{SG91457, &UA3033825},
	{SG91458, &UA8985},
	{SG91459, &UA32047},
	{SG91460, &UA33293},
	{SG91461, &UA75602},
	{SG91463, &UA4747},
	{SG91464, &UA27441},
	{SG91465, &UA27454},
	{SG91466, &UA64026},
	{SG91467, &UA64437},
	{SG91469, &UA68907},
	{SG91470, &UA2945956},
	{SG91473, &UA2366131},
	{SG91474, &UA894997},
	{SG91479, &UA60915},
	{SG91480, &UA63845},
	{SG91481, &UA56113},
	{SG91482, &UA31582},
	{SG91488, &UA63852},
	{SG91489, &UA4875},
	{SG91490, &UA49108},
	{SG91491, &UA61613},
	{SG91492, &UA61919},
	{SG91493, &UA64639},
	{SG91494, &UA103328},
	{SG91495, &UA131532},
	{SG91496, &UA167158},
	{SG91497, &UA275795},
	{SG91498, &UA436491},
	{SG91499, &UA494971},
	{SG91500, &UA652247},
	{SG91501, &UA1845120},
	{SG91502, &UA2275937},
	{SG91505, &UA3079479},
	{SG91506, &UA2957767},
	{SG91507, &UA3047337},
	{SG91508, &UA9714},
	{SG91509, &UA43563},
	{SG91511, &UA63151},
	{SG91512, &UA5465},
	{SG91513, &UA8075},
	{SG91514, &UA24167},
	{SG91515, &UA29503},
	{SG91516, &UA62379},
	{SG91517, &UA63287},
	{SG91518, &UA31247},
	{SG91519, &UA8842},
	{SG91520, &UA50385},
	{SG91525, &UA895878},
	{SG91527, &UA94196},
	{SG91528, &UA22607},
	{SG91532, &UA4155},
	{SG91533, &UA7916},
	{SG91534, &UA63854},
	{SG91535, &UA79284},
	{SG91544, &UA20663},
	{SG91545, &UA43626},
	{SG91546, &UA54895},
	{SG91547, &UA49703},
	{SG91548, &UA61568},
	{SG91549, &UA63996},
	{SG91553, &UA2946978},
	{SG91554, &UA16202},
	{SG91556, &UA4149},
	{SG91557, &UA4150},
	{SG91558, &UA4159},
	{SG91559, &UA7876},
	{SG91560, &UA7878},
	{SG91561, &UA7880},
	{SG91562, &UA7877},
	{SG91563, &UA7918},
	{SG91564, &UA7924},
	{SG91565, &UA7925},
	{SG91566, &UA7927},
	{SG91567, &UA7928},
	{SG91568, &UA7929},
	{SG91569, &UA27719},
	{SG91570, &UA27720},
	{SG91571, &UA27724},
	{SG91572, &UA34241},
	{SG91573, &UA34244},
	{SG91574, &UA34245},
	{SG91575, &UA34247},
	{SG91576, &UA63995},
	{SG91577, &UA65979},
	{SG91578, &UA65982},
	{SG91579, &UA75133},
	{SG91580, &UA87098},
	{SG91581, &UA27613},
	{SG91582, &UA43711},
	{SG91583, &UA103683},
	{SG91584, &UA371651},
	{SG91585, &UA2263927},
	{SG91586, &UA50386},
	{SG91587, &UA2174370},
	{SG91588, &UA62966},
	{SG91589, &UA64049},
	{SG91591, &UA56027},
	{SG91602, &UA50196},
	{SG91603, &UA55264},
	{SG91604, &UA63780},
	{SG91605, &UA1996836},
	{SG91606, &UA13448},
	{SG91607, &UA31250},
	{SG91608, &UA3048900},
	{SG91613, &UA31162},
	{SG91614, &UA63077},
	{SG91615, &UA64189},
	{SG91616, &UA2764684},
	{SG91620, &UA12154},
	{SG91621, &UA2878308},
	{SG91626, &UA33261},
	{SG91627, &UA49407},
	{SG91628, &UA49436},
	{SG91629, &UA774954},
	{SG91630, &UA2144148},
	{SG91631, &UA9964},
	{SG91633, &UA9437},
	{SG91634, &UA16316},
	{SG91636, &UA46888},
	{SG91637, &UA53789},
	{SG91638, &UA57921},
	{SG91646, &UA64441},
	{SG91647, &UA22686},
	{SG91648, &UA63697},
	{SG91649, &UA54421},
	{SG91650, &UA113579},
	{SG91651, &UA141434},
	{SG91652, &UA459924},
	{SG91653, &UA64085},
	{SG91654, &UA6367},
	{SG91655, &UA32024},
	{SG91656, &UA64792},
	{SG91657, &UA20833},
	{SG91658, &UA32952},
	{SG91660, &UA33019},
	{SG91661, &UA41019},
	{SG91663, &UA2362227},
	{SG91664, &UA26212},
	{SG91665, &UA24470},
	{SG91666, &UA899580},
	{SG91667, &UA1985160},
	{SG91669, &UA3651},
	{SG91675, &UA220047},
	{SG91677, &UA61643},
	{SG91678, &UA64648},
	{SG91679, &UA49089},
	{SG91680, &UA63176},
	{SG91682, &UA6279},
	{SG91683, &UA64850},
	{SG91686, &UA9170},
	{SG91687, &UA24389},
	{SG91688, &UA88757},
	{SG91690, &UA16315},
	{SG91691, &UA64090},
	{SG91695, &UA63252},
	{SG91696, &UA62613},
	{SG91698, &UA2737334},
	{SG91699, &UA20351},
	{SG91700, &UA3022763},
	{SG91705, &UA63772},
	{SG91709, &UA7116},
	{SG91710, &UA16584},
	{SG91712, &UA66196},
	{SG91714, &UA12155},
	{SG91718, &UA63471},
	{SG91724, &UA30114},
	{SG91725, &UA45004},
	{SG91726, &UA55155},
	{SG91727, &UA725453},
	{SG91728, &UA2754541},
	{SG91730, &UA3088303},
	{SG91740, &UA26349},
	{SG91742, &UA2229609},
	{SG91743, &UA62452},
	{SG91744, &UA2363119},
	{SG91748, &UA54833},
	{SG91750, &UA906117},
	{SG91751, &UA1988300},
	{SG91752, &UA62030},
	{SG91753, &UA63944},
	{SG91754, &UA89047},
	{SG91755, &UA281454},
	{SG91756, &UA477154},
	{SG91757, &UA499971},
	{SG91758, &UA26297},
	{SG91759, &UA61022},
	{SG91760, &UA31979},
	{SG91761, &UA62741},
	{SG91762, &UA63605},
	{SG91763, &UA30490},
	{SG91764, &UA32905},
	{SG91765, &UA16585},
	{SG91766, &UA26956},
	{SG91767, &UA3080297},
	{SG91768, &UA55132},
	{SG91769, &UA2223526},
	{SG91770, &UA9102},
	{SG91771, &UA64713},
	{SG91772, &UA3092516},
	{SG91774, &UA2362124},
	{SG91775, &UA63609},
	{SG91776, &UA83},
	{SG91777, &UA32025},
	{SG91778, &UA96285},
	{SG91779, &UA64191},
	{SG91780, &UA2204940},
	{SG91782, &UA63339},
	{SG91783, &UA2165168},
	{SG91784, &UA2463658},
	{SG91785, &UA2700210},
	{SG91786, &UA2714653},
	{SG91787, &UA2184494},
	{SG91788, &UA15011},
	{SG91789, &UA63242},
	{SG91790, &UA889908},
	{SG91791, &UA905382},
	{SG91794, &UA2152821},
	{SG91795, &UA2403425},
	{SG91796, &UA9389},
	{SG91797, &UA15744},
	{SG91798, &UA22835},
	{SG91800, &UA23725},
	{SG91801, &UA600515},
	{SG91802, &UA609245},
	{SG91803, &UA2813987},
	{SG91807, &UA44923},
	{SG91808, &UA2162005},
	{SG91809, &UA7367},
	{SG91810, &UA63699},
	{SG91811, &UA2220632},
	{SG91814, &UA43574},
	{SG91815, &UA262616},
	{SG91816, &UA63655},
	{SG91818, &UA31102},
	{SG91819, &UA50280},
	{SG91820, &UA2528029},
	{SG91822, &UA2667741},
	{SG91823, &UA1972879},
	{SG91824, &UA1989951},
	{SG91826, &UA2234420},
	{SG91827, &UA43685},
	{SG91828, &UA2855731},
	{SG91829, &UA31598},
	{SG91830, &UA63748},
	{SG91831, &UA130861},
	{SG91832, &UA4372},
	{SG91833, &UA25404},
	{SG91834, &UA26656},
	{SG91835, &UA32864},
	{SG91836, &UA33162},
	{SG91837, &UA61123},
	{SG91838, &UA64243},
	{SG91839, &UA2353791},
	{SG91840, &UA21290},
	{SG91841, &UA30357},
	{SG91842, &UA48986},
	{SG91843, &UA898917},
	{SG91844, &UA1954917},
	{SG91845, &UA44164},
	{SG91846, &UA600594},
	{SG91847, &UA616087},
	{SG91848, &UA2810643},
	{SG91849, &UA6006},
	{SG91850, &UA105415},
	{SG91851, &UA63193},
	{SG91852, &UA67327},
	{SG91853, &UA11327},
	{SG91854, &UA54631},
	{SG91855, &UA17907},
	{SG91856, &UA27465},
	{SG91857, &UA45243},
	{SG91858, &UA7718},
	{SG91859, &UA63653},
	{SG91861, &UA2351551},
	{SG91862, &UA898962},
	{SG91863, &UA2373068},
	{SG91864, &UA2226837},
	{SG91865, &UA1959354},
	{SG91866, &UA24500},
	{SG91871, &UA6156},
	{SG91872, &UA53614},
	{SG91873, &UA63887},
	{SG91874, &UA63888},
	{SG91875, &UA628367},
	{SG91876, &UA814503},
	{SG91877, &UA734836},
	{SG91878, &UA925788},
	{SG91879, &UA2693684},
	{SG91880, &UA2347155},
	{SG91881, &UA2669342},
	{SG91882, &UA24479},
	{SG91883, &UA44219},
	{SG91884, &UA64410},
	{SG91885, &UA63727},
	{SG91890, &UA81668},
	{SG91898, &UA9815},
	{SG91900, &UA61983},
	{SG91901, &UA2188641},
	{SG91902, &UA2199382},
	{SG91904, &UA2310165},
	{SG91905, &UA656144},
	{SG91906, &UA685991},
	{SG91907, &UA61644},
	{SG91908, &UA121632},
	{SG91911, &UA63470},
	{SG91912, &UA51234},
	{SG91913, &UA86478},
	{SG91914, &UA46719},
	{SG91915, &UA55777},
	{SG91917, &UA87394},
	{SG91920, &UA5316},
	{SG91921, &UA24453},
	{SG91922, &UA45937},
	{SG91923, &UA64033},
	{SG91924, &UA64057},
	{SG91925, &UA1842205},
	{SG91928, &UA61726},
	{SG91929, &UA63894},
	{SG91930, &UA20179},
	{SG91931, &UA63913},
	{SG91932, &UA64829},
	{SG91933, &UA21429},
	{SG91934, &UA30755},
	{SG91935, &UA648869},
	{SG91936, &UA700819},
	{SG91940, &UA2391577},
	{SG91941, &UA2487942},
	{SG91942, &UA2490825},
	{SG91947, &UA55016},
	{SG91948, &UA601156},
	{SG91949, &UA636882},
	{SG91950, &UA371356},
	{SG91951, &UA437093},
	{SG91952, &UA8984},
	{SG91966, &UA69096},
	{SG91974, &UA5293},
	{SG91975, &UA6458},
	{SG91976, &UA22328},
	{SG91977, &UA28385},
	{SG91978, &UA28387},
	{SG91979, &UA28388},
	{SG91980, &UA28386},
	{SG91981, &UA36947},
	{SG91982, &UA45915},
	{SG91983, &UA46509},
	{SG91984, &UA46508},
	{SG91985, &UA51028},
	{SG91986, &UA59815},
	{SG91987, &UA61825},
	{SG91988, &UA61824},
	{SG91989, &UA62583},
	{SG91990, &UA62868},
	{SG91991, &UA64823},
	{SG91992, &UA69556},
	{SG91993, &UA88550},
	{SG91994, &UA13183},
	{SG91995, &UA13181},
	{SG91996, &UA22445},
	{SG91997, &UA43887},
	{SG91998, &UA64876},
	{SG92003, &UA24373},
	{SG92004, &UA61144},
	{SG92005, &UA55202},
	{SG92008, &UA87079},
	{SG92009, &UA88894},
	{SG92010, &UA98987},
	{SG92011, &UA261802},
	{SG92012, &UA639541},
	{SG92013, &UA860642},
	{SG92014, &UA2127844},
	{SG92016, &UA16632},
	{SG92017, &UA20308},
	{SG92018, &UA32737},
	{SG92019, &UA64663},
	{SG92020, &UA624644},
	{SG92021, &UA877566},
	{SG92022, &UA904917},
	{SG92023, &UA23253},
	{SG92024, &UA62059},
	{SG92025, &UA38538},
	{SG92026, &UA600500},
	{SG92027, &UA605011},
	{SG92028, &UA2161078},
	{SG92034, &UA3538},
	{SG92035, &UA24461},
	{SG92036, &UA25120},
	{SG92037, &UA26056},
	{SG92038, &UA61965},
	{SG92039, &UA64068},
	{SG92040, &UA64069},
	{SG92041, &UA65558},
	{SG92042, &UA18787},
	{SG92043, &UA63307},
	{SG92044, &UA62271},
	{SG92045, &UA53971},
	{SG92046, &UA9988},
	{SG92047, &UA21427},
	{SG92048, &UA3116405},
	{SG92049, &UA55429},
	{SG92050, &UA2390395},
	{SG92053, &UA9549},
	{SG92054, &UA28107},
	{SG92055, &UA51835},
	{SG92056, &UA54182},
	{SG92057, &UA60956},
	{SG92058, &UA61077},
	{SG92059, &UA61646},
	{SG92060, &UA64520},
	{SG92061, &UA87391},
	{SG92062, &UA1967559},
	{SG92063, &UA20491},
	{SG92066, &UA37377},
	{SG92068, &UA2384452},
	{SG92069, &UA2350016},
	{SG92070, &UA61690},
	{SG92073, &UA1685},
	{SG92074, &UA17273},
	{SG92075, &UA21428},
	{SG92076, &UA63318},
	{SG92077, &UA2842176},
	{SG92078, &UA62533},
	{SG92079, &UA2261425},
	{SG92080, &UA2384380},
	{SG92084, &UA55224},
	{SG92085, &UA62127},
	{SG92086, &UA626878},
	{SG92087, &UA68853},
	{SG92088, &UA2659450},
	{SG92089, &UA1965732},
	{SG92090, &UA2758305},
	{SG92092, &UA63753},
	{SG92093, &UA1963994},
	{SG92094, &UA1994128},
	{SG92100, &UA84},
	{SG92101, &UA6007},
	{SG92102, &UA6368},
	{SG92103, &UA32198},
	{SG92104, &UA30854},
	{SG92105, &UA24634},
	{SG92106, &UA5185},
	{SG92107, &UA14360},
	{SG92108, &UA22616},
	{SG92109, &UA54290},
	{SG92110, &UA656605},
	{SG92111, &UA1963507},
	{SG92113, &UA26227},
	{SG92114, &UA7366},
	{SG92115, &UA50356},
	{SG92118, &UA653076},
	{SG92119, &UA3092234},
	{SG92120, &UA3092249},
	{SG92122, &UA64609},
	{SG92123, &UA2384123},
	{SG92127, &UA4369},
	{SG92128, &UA48684},
	{SG92129, &UA55536},
	{SG92130, &UA63703},
	{SG92131, &UA32097},
	{SG92132, &UA62381},
	{SG92133, &UA12797},
	{SG92134, &UA2391366},
	{SG92136, &UA26657},
	{SG92137, &UA63686},
	{SG92138, &UA87867},
	{SG92139, &UA18547},
	{SG92140, &UA63430},
	{SG92142, &UA62431},
	{SG92144, &UA2489174},
	{SG92145, &UA63485},
	{SG92146, &UA644048},
	{SG92147, &UA663979},
	{SG92148, &UA62042},
	{SG92149, &UA2487843},
	{SG92151, &UA4032},
	{SG92152, &UA18639},
	{SG92155, &UA2261439},
	{SG92156, &UA3020805},
	{SG92157, &UA62335},
	{SG92158, &UA22870},
	{SG92159, &UA43680},
	{SG92160, &UA2391409},
	{SG92161, &UA113633},
	{SG92170, &UA119280},
	{SG92171, &UA64654},
	{SG92172, &UA24543},
	{SG92173, &UA54226},
	{SG92174, &UA61723},
	{SG92175, &UA63827},
	{SG92176, &UA9337},
	{SG92177, &UA925852},
	{SG92178, &UA2523327},
	{SG92179, &UA17912},
	{SG92180, &UA24405},
	{SG92181, &UA1953974},
	{SG92182, &UA2954916},
	{SG92183, &UA3059888},
	{SG92184, &UA2729183},
	{SG92185, &UA2840583},
	{SG92186, &UA2952628},
	{SG92187, &UA3079520},
	{SG92189, &UA2483564},
	{SG92190, &UA2166884},
	{SG92191, &UA2428291},
	{SG92192, &UA30657},
	{SG92193, &UA2261426},
	{SG92194, &UA2383906},
	{SG92195, &UA5319},
	{SG92196, &UA61743},
	{SG92197, &UA88884},
	{SG92198, &UA778332},
	{SG92199, &UA24073},
	{SG92200, &UA44853},
	{SG92204, &UA25402},
	{SG92205, &UA2390410},
	{SG92206, &UA2702517},
	{SG92207, &UA2481746},
	{SG92208, &UA16002},
	{SG92209, &UA43675},
	{SG92210, &UA20903},
	{SG92215, &UA40017},
	{SG92216, &UA61116},
	{SG92217, &UA832918},
	{SG92218, &UA371363},
	{SG92219, &UA755437},
	{SG92220, &UA768350},
	{SG92221, &UA925857},
	{SG92222, &UA645269},
	{SG92223, &UA925904},
	{SG92224, &UA2428298},
	{SG92225, &UA2809549},
	{SG92226, &UA2428288},
	{SG92227, &UA3035763},
	{SG92228, &UA17357},
	{SG92229, &UA31867},
	{SG92230, &UA63235},
	{SG92231, &UA20540},
	{SG92232, &UA61577},
	{SG92233, &UA21905},
	{SG92234, &UA61657},
	{SG92235, &UA22254},
	{SG92236, &UA2458223},
	{SG92237, &UA24262},
	{SG92238, &UA2487850},
	{SG92239, &UA2729905},
	{SG92240, &UA2799054},
	{SG92241, &UA3025942},
	{SG92242, &UA22363},
	{SG92243, &UA49413},
	{SG92245, &UA3021393},
	{SG92246, &UA769314},
	{SG92247, &UA3021506},
	{SG92249, &UA2195655},
	{SG92250, &UA55127},
	{SG92251, &UA2404280},
	{SG92254, &UA572222},
	{SG92256, &UA2893394},
	{SG92257, &UA9242},
	{SG92258, &UA55283},
	{SG92259, &UA2879582},
	{SG92260, &UA22436},
	{SG92261, &UA1988212},
	{SG92262, &UA2811137},
	{SG92263, &UA22181},
	{SG92264, &UA665968},
	{SG92265, &UA3116402},
	{SG92268, &UA24541},
	{SG92269, &UA51845},
	{SG92270, &UA62458},
	{SG92271, &UA761850},
	{SG92285, &UA2400836},
	{SG92286, &UA16179},
	{SG92287, &UA21171},
	{SG92288, &UA24579},
	{SG92289, &UA3023890},
	{SG92290, &UA63906},
	{SG92291, &UA63939},
	{SG92296, &UA2773727},
	{SG92297, &UA29379},
	{SG92298, &UA62859},
	{SG92299, &UA63155},
	{SG92300, &UA69649},
	{SG92301, &UA86957},
	{SG92302, &UA88827},
	{SG92303, &UA87674},
	{SG92304, &UA9883},
	{SG92305, &UA2429605},
	{SG92306, &UA2507880},
	{SG92307, &UA3021464},
	{SG92308, &UA24626},
	{SG92309, &UA33267},
	{SG92310, &UA53311},
	{SG92311, &UA25393},
	{SG92312, &UA1950354},
	{SG92313, &UA2373438},
	{SG92314, &UA2714869},
	{SG92315, &UA2893304},
	{SG92316, &UA2499179},
	{SG92317, &UA2428285},
	{SG92322, &UA24762},
	{SG92323, &UA578416},
	{SG92324, &UA2166850},
	{SG92325, &UA43545},
	{SG92326, &UA925800},
	{SG92328, &UA925803},
	{SG92329, &UA2412416},
	{SG92330, &UA2792068},
	{SG92331, &UA9992},
	{SG92332, &UA32116},
	{SG92333, &UA49058},
	{SG92335, &UA2487852},
	{SG92336, &UA2597890},
	{SG92337, &UA2522438},
	{SG92338, &UA2428296},
	{SG92339, &UA2869837},
	{SG92340, &UA2869812},
	{SG92341, &UA24599},
	{SG92342, &UA64496},
	{SG92343, &UA50334},
	{SG92344, &UA64059},
	{SG92345, &UA2404692},
	{SG92346, &UA2654379},
	{SG92347, &UA2402952},
	{SG92348, &UA2405881},
	{SG92349, &UA2595421},
	{SG92353, &UA54548},
	{SG92355, &UA64091},
	{SG92356, &UA24304},
	{SG92357, &UA62735},
	{SG92358, &UA20197},
	{SG92359, &UA22617},
	{SG92360, &UA20247},
	{SG92361, &UA61754},
	{SG92362, &UA64284},
	{SG92363, &UA679493},
	{SG92364, &UA2469929},
	{SG92366, &UA24529},
	{SG92367, &UA27655},
	{SG92368, &UA62184},
	{SG92369, &UA88425},
	{SG92370, &UA16019},
	{SG92371, &UA20034},
	{SG92372, &UA20838},
	{SG92373, &UA67225},
	{SG92374, &UA2275164},
	{SG92375, &UA3109176},
	{SG92379, &UA2401558},
	{SG92380, &UA20742},
	{SG92381, &UA2870545},
	{SG92383, &UA2781848},
	{SG92386, &UA2879900},
	{SG92387, &UA2965036},
	{SG92388, &UA16692},
	{SG92389, &UA1955734},
	{SG92390, &UA1966814},
	{SG92391, &UA2799044},
	{SG92392, &UA7333},
	{SG92393, &UA18512},
	{SG92394, &UA55125},
	{SG92395, &UA26801},
	{SG92396, &UA68356},
	{SG92399, &UA62028},
	{SG92400, &UA25991},
	{SG92401, &UA55045},
	{SG92403, &UA2661554},
	{SG92404, &UA27586},
	{SG92405, &UA791987},
	{SG92406, &UA3022212},
	{SG92407, &UA2351},
	{SG92408, &UA63719},
	{SG92409, &UA23079},
	{SG92412, &UA2386603},
	{SG92413, &UA2784993},
	{SG92414, &UA17954},
	{SG92415, &UA23256},
	{SG92416, &UA23894},
	{SG92417, &UA88759},
	{SG92418, &UA2325078},
	{SG92419, &UA2362732},
	{SG92420, &UA2428297},
	{SG92421, &UA3019792},
	{SG92422, &UA51475},
	{SG92423, &UA62665},
	{SG92451, &UA3059257},
	{SG92454, &UA24672},
	{SG92455, &UA30750},
	{SG92456, &UA53444},
	{SG92457, &UA696695},
	{SG92458, &UA64335},
	{SG92459, &UA54985},
	{SG92460, &UA63386},
	{SG92461, &UA690963},
	{SG92462, &UA779762},
	{SG92463, &UA2996616},
	{SG92467, &UA21872},
	{SG92468, &UA24760},
	{SG92469, &UA62609},
	{SG92470, &UA64119},
	{SG92471, &UA64578},
	{SG92472, &UA2693138},
	{SG92473, &UA3059053},
	{SG92474, &UA64411},
	{SG92475, &UA30667},
	{SG92476, &UA2450919},
	{SG92477, &UA2558713},
	{SG92513, &UA1866096},
	{SG92518, &UA18860},
	{SG92520, &UA20324},
	{SG92521, &UA20579},
	{SG92523, &UA25394},
	{SG92526, &UA525568},
	{SG92527, &UA2511621},
	{SG92528, &UA2851896},
	{SG92530, &UA54790},
	{SG92531, &UA2519092},
	{SG92532, &UA2593510},
	{SG92533, &UA2768781},
	{SG92534, &UA2781681},
	{SG92537, &UA224},
	{SG92538, &UA20698},
	{SG92539, &UA2655687},
	{SG92541, &UA695903},
	{SG92544, &UA2879691},
	{SG92545, &UA3009676},
	{SG92546, &UA3027506},
	{SG92547, &UA3059123},
	{SG92548, &UA62057},
	{SG92549, &UA746655},
	{SG92550, &UA2338588},
	{SG92551, &UA2781799},
	{SG92552, &UA3110600},
	{SG92553, &UA16362},
	{SG92554, &UA2700995},
	{SG92555, &UA64595},
	{SG92556, &UA2383391},
	{SG92557, &UA3024195},
	{SG92558, &UA22359},
	{SG92559, &UA62807},
	{SG92560, &UA24441},
	{SG92561, &UA24442},
	{SG92562, &UA26131},
	{SG92563, &UA64573},
	{SG92565, &UA514469},
	{SG92566, &UA762783},
	{SG92567, &UA2161851},
	{SG92569, &UA19882},
	{SG92570, &UA21303},
	{SG92571, &UA21304},
	{SG92572, &UA23865},
	{SG92573, &UA2559363},
	{SG92579, &UA13025},
	{SG92580, &UA14530},
	{SG92581, &UA23749},
	{SG92582, &UA30270},
	{SG92583, &UA56044},
	{SG92584, &UA62974},
	{SG92585, &UA572394},
	{SG92586, &UA2549729},
	{SG92587, &UA1959109},
	{SG92588, &UA2423221},
	{SG92589, &UA2456417},
	{SG92592, &UA31367},
	{SG92594, &UA3106804},
	{SG92595, &UA19956},
	{SG92596, &UA20662},
	{SG92597, &UA54923},
	{SG92598, &UA53931},
	{SG92599, &UA82189},
	{SG92600, &UA67224},
	{SG92601, &UA86533},
	{SG92603, &UA2312550},
	{SG92604, &UA2262909},
	{SG92606, &UA2707810},
	{SG92607, &UA2870183},
	{SG92608, &UA2866469},
	{SG92609, &UA21753},
	{SG92610, &UA62988},
	{SG92611, &UA4996},
	{SG92612, &UA21747},
	{SG92613, &UA29863},
	{SG92614, &UA43617},
	{SG92615, &UA61555},
	{SG92616, &UA2695877},
	{SG92617, &UA10323},
	{SG92618, &UA38682},
	{SG92619, &UA52453},
	{SG92620, &UA21502},
	{SG92621, &UA39041},
	{SG92622, &UA2166157},
	{SG92623, &UA2727640},
	{SG92624, &UA43831},
	{SG92625, &UA697077},
	{SG92626, &UA2159792},
	{SG92627, &UA2574006},
	{SG92628, &UA3058824},
	{SG92629, &UA22224},
	{SG92630, &UA22248},
	{SG92631, &UA24489},
	{SG92633, &UA87393},
	{SG92634, &UA87476},
	{SG92635, &UA23998},
	{SG92636, &UA26453},
	{SG92637, &UA55142},
	{SG92638, &UA452242},
	{SG92639, &UA1938979},
	{SG92640, &UA2145297},
	{SG92641, &UA2159854},
	{SG92643, &UA2816740},
	{SG92644, &UA55166},
	{SG92645, &UA22351},
	{SG92646, &UA23776},
	{SG92649, &UA2527058},
	{SG92650, &UA2528221},
	{SG92651, &UA2854269},
	{SG92652, &UA3021672},
	{SG92653, &UA3028310},
	{SG92654, &UA900},
	{SG92655, &UA23013},
	{SG92656, &UA713861},
	{SG92657, &UA4876},
	{SG92658, &UA925810},
	{SG92659, &UA925913},
	{SG92660, &UA2520703},
	{SG92661, &UA1989864},
	{SG92662, &UA2332283},
	{SG92664, &UA2847688},
	{SG92665, &UA2511338},
	{SG92666, &UA62309},
	{SG92667, &UA64179},
	{SG92668, &UA2262659},
	{SG92669, &UA3060579},
	{SG92670, &UA8991},
	{SG92671, &UA20348},
	{SG92672, &UA22696},
	{SG92673, &UA2404749},
	{SG92674, &UA2879463},
	{SG92675, &UA2898560},
	{SG92676, &UA2960189},
	{SG92679, &UA12794},
	{SG92680, &UA20263},
	{SG92681, &UA21291},
	{SG92683, &UA30226},
	{SG92684, &UA55197},
	{SG92685, &UA62327},
	{SG92686, &UA68573},
	{SG92687, &UA2963261},
	{SG92688, &UA3062288},
	{SG92689, &UA67436},
	{SG92690, &UA724518},
	{SG92691, &UA776267},
	{SG92692, &UA2370577},
	{SG92693, &UA2663564},
	{SG92696, &UA57755},
	{SG92697, &UA64118},
	{SG92698, &UA1952999},
	{SG92699, &UA2879470},
	{SG92700, &UA3016337},
	{SG92701, &UA7288},
	{SG92702, &UA22357},
	{SG92703, &UA31504},
	{SG92705, &UA557183},
	{SG92706, &UA2558691},
	{SG92707, &UA3009725},
	{SG92708, &UA2470},
	{SG92709, &UA21440},
	{SG92710, &UA63234},
	{SG92711, &UA14953},
	{SG92712, &UA21716},
	{SG92713, &UA24656},
	{SG92714, &UA73186},
	{SG92715, &UA61334},
	{SG92716, &UA88761},
	{SG92717, &UA571801},
	{SG92718, &UA837731},
	{SG92719, &UA1899604},
	{SG92720, &UA1900272},
	{SG92721, &UA2467218},
	{SG92724, &UA22447},
	{SG92725, &UA24734},
	{SG92726, &UA26839},
	{SG92727, &UA81535},
	{SG92729, &UA2694562},
	{SG92731, &UA15211},
	{SG92732, &UA44564},
	{SG92733, &UA16226},
	{SG92734, &UA16225},
	{SG92736, &UA22609},
	{SG92737, &UA24164},
	{SG92738, &UA24374},
	{SG92739, &UA54367},
	{SG92740, &UA61846},
	{SG92741, &UA62170},
	{SG92742, &UA62552},
	{SG92745, &UA2336597},
	{SG92746, &UA2805273},
	{SG92747, &UA2994964},
	{SG92748, &UA3057672},
	{SG92750, &UA5427},
	{SG92751, &UA22355},
	{SG92752, &UA23177},
	{SG92753, &UA60573},
	{SG92754, &UA56170},
	{SG92755, &UA61882},
	{SG92756, &UA2405129},
	{SG92757, &UA2404817},
	{SG92758, &UA2554851},
	{SG92760, &UA3078631},
	{SG92761, &UA22614},
	{SG92762, &UA23671},
	{SG92763, &UA28392},
	{SG92764, &UA62562},
	{SG92765, &UA2443146},
	{SG92766, &UA2527254},
	{SG92767, &UA2588514},
	{SG92768, &UA2731288},
	{SG92769, &UA2827240},
	{SG92770, &UA2891046},
	{SG92771, &UA3099964},
	{SG92772, &UA2994994},
	{SG92773, &UA3108519},
	{SG92774, &UA20405},
	{SG92775, &UA20913},
	{SG92776, &UA55864},
	{SG92777, &UA2287879},
	{SG92780, &UA2777721},
	{SG92781, &UA2983980},
	{SG92782, &UA2987232},
	{SG92785, &UA3107979},
	{SG92786, &UA3108389},
	{SG92787, &UA8570},
	{SG92789, &UA22407},
	{SG92790, &UA53624},
	{SG92791, &UA53945},
	{SG92792, &UA63881},
	{SG92793, &UA729112},
	{SG92794, &UA2409786},
	{SG92796, &UA2461171},
	{SG92797, &UA2474032},
	{SG92799, &UA2574021},
	{SG92804, &UA2982666},
	{SG92808, &UA15439},
	{SG92809, &UA15762},
	{SG92810, &UA1757},
	{SG92811, &UA22027},
	{SG92812, &UA23091},
	{SG92813, &UA22423},
	{SG92814, &UA60654},
	{SG92815, &UA23420},
	{SG92816, &UA24524},
	{SG92817, &UA60030},
	{SG92818, &UA61161},
	{SG92819, &UA61773},
	{SG92823, &UA1933790},
	{SG92825, &UA2139580},
	{SG92826, &UA2659601},
	{SG92828, &UA2843820},
	{SG92829, &UA2996608},
	{SG92830, &UA4249},
	{SG92831, &UA20593},
	{SG92832, &UA23864},
	{SG92833, &UA62779},
	{SG92834, &UA87047},
	{SG92835, &UA87474},
	{SG92836, &UA2157204},
	{SG92837, &UA2557640},
	{SG92838, &UA2694159},
	{SG92839, &UA2854402},
	{SG92840, &UA2939028},
	{SG92842, &UA3016993},
	{SG92843, &UA3108499},
	{SG92844, &UA3118182},
	{SG92845, &UA21164},
	{SG92846, &UA20270},
	{SG92847, &UA22377},
	{SG92848, &UA22393},
	{SG92849, &UA24742},
	{SG92860, &UA61152},
	{SG92864, &UA62784},
	{SG92867, &UA68093},
	{SG92872, &UA490566},
	{SG92880, &UA2684462},
	{SG92881, &UA2210746},
	{SG92882, &UA2410093},
	{SG92883, &UA2428106},
	{SG92884, &UA2495572},
	{SG92885, &UA2545446},
	{SG92886, &UA2727567},
	{SG92887, &UA2958169},
	{SG92888, &UA2933908},
	{SG92890, &UA2983246},
	{SG92891, &UA2995038},
	{SG92892, &UA16375},
	{SG92893, &UA49519},
	{SG92894, &UA20133},
	{SG92895, &UA22243},
	{SG92896, &UA23254},
	{SG92897, &UA28382},
	{SG92898, &UA50311},
	{SG92899, &UA54466},
	{SG92900, &UA60865},
	{SG92901, &UA63327},
	{SG92902, &UA81703},
	{SG92903, &UA566611},
	{SG92904, &UA571620},
	{SG92905, &UA2165110},
	{SG92906, &UA2338112},
	{SG92907, &UA2410789},
	{SG92908, &UA2413172},
	{SG92909, &UA2474152},
	{SG92910, &UA2807994},
	{SG92911, &UA2840406},
	{SG92913, &UA2978594},
	{SG92914, &UA3118892},
	{SG92915, &UA3126167},
	{SG92917, &UA26626},
	{SG92918, &UA30271},
	{SG92919, &UA30901},
	{SG92920, &UA54063},
	{SG92921, &UA67149},
	{SG92922, &UA68559},
	{SG92925, &UA572964},
	{SG92926, &UA723879},
	{SG92927, &UA838851},
	{SG92928, &UA2141320},
	{SG92929, &UA2162014},
	{SG92930, &UA2268715},
	{SG92931, &UA2273407},
	{SG92932, &UA2300805},
	{SG92933, &UA2542773},
	{SG92935, &UA2572499},
	{SG92936, &UA2596322},
	{SG92937, &UA2599421},
	{SG92938, &UA2607477},
	{SG92939, &UA2774804},
	{SG92940, &UA2801321},
	{SG92941, &UA2878200},
	{SG92942, &UA2989154},
	{SG92943, &UA3031362},
	{SG92944, &UA3041581},
	{SG92945, &UA3117086},
	{SG92948, &UA1071},
	{SG92949, &UA10223},
	{SG92950, &UA16691},
	{SG92951, &UA17233},
	{SG92952, &UA20521},
	{SG92953, &UA22402},
	{SG92954, &UA49340},
	{SG92955, &UA54426},
	{SG92956, &UA62229},
	{SG92957, &UA63161},
	{SG92958, &UA414119},
	{SG92959, &UA573435},
	{SG92960, &UA573801},
	{SG92961, &UA1905340},
	{SG92962, &UA2796050},
	{SG92963, &UA2806658},
	{SG92964, &UA2815882},
	{SG92965, &UA2837069},
	{SG92966, &UA2838048},
	{SG92968, &UA2973383},
	{SG92969, &UA5087},
	{SG92970, &UA20026},
	{SG92971, &UA21426},
	{SG92972, &UA22598},
	{SG92973, &UA54303},
	{SG92974, &UA54988},
	{SG92975, &UA64613},
	{SG92976, &UA64804},
	{SG92977, &UA562764},
	{SG92978, &UA713337},
	{SG92979, &UA2303085},
	{SG92980, &UA1872518},
	{SG92981, &UA2300974},
	{SG92982, &UA2423935},
	{SG92983, &UA2461749},
	{SG92984, &UA2804747},
	{SG92987, &UA17276},
	{SG92988, &UA21912},
	{SG92991, &UA61890},
	{SG92992, &UA25406},
	{SG92993, &UA43644},
	{SG92994, &UA49943},
	{SG92995, &UA54068},
	{SG92997, &UA2486686},
	{SG92998, &UA2506107},
	{SG92999, &UA10079},
	{SG93000, &UA21367},
	{SG93001, &UA22365},
	{SG93002, &UA22451},
	{SG93003, &UA22462},
	{SG93004, &UA32535},
	{SG93006, &UA54214},
	{SG93007, &UA2963485},
	{SG93008, &UA61705},
	{SG93009, &UA9017},
	{SG93010, &UA9019},
	{SG93011, &UA21727},
	{SG93012, &UA26132},
	{SG93013, &UA30149},
	{SG93014, &UA44072},
	{SG93015, &UA44196},
	{SG93016, &UA56653},
	{SG93017, &UA67271},
	{SG93026, &UA2167638},
	{SG93027, &UA2748427},
	{SG93029, &UA4547},
	{SG93030, &UA21656},
	{SG93031, &UA88760},
	{SG93032, &UA22257},
	{SG93033, &UA82116},
	{SG93034, &UA24607},
	{SG93035, &UA32837},
	{SG93036, &UA56166},
	{SG93037, &UA58630},
	{SG93038, &UA194391},
	{SG93040, &UA10341},
	{SG93041, &UA15915},
	{SG93042, &UA48946},
	{SG93043, &UA21354},
	{SG93044, &UA87030},
	{SG93045, &UA87451},
	{SG93046, &UA22204},
	{SG93047, &UA22595},
	{SG93048, &UA33276},
	{SG93049, &UA49545},
	{SG93050, &UA49428},
	{SG93051, &UA81517},
	{SG93052, &UA63718},
	{SG93053, &UA86938},
	{SG93054, &UA318049},
	{SG93056, &UA2171625},
	{SG93057, &UA3031114},
	{SG93058, &UA51415},
	{SG93059, &UA10003},
	{SG93060, &UA10004},
	{SG93061, &UA10005},
	{SG93062, &UA86898},
	{SG93063, &UA87886},
	{SG93064, &UA20898},
	{SG93065, &UA21873},
	{SG93066, &UA36233},
	{SG93067, &UA63142},
	{SG93068, &UA27970},
	{SG93069, &UA44918},
	{SG93070, &UA50169},
	{SG93071, &UA53675},
	{SG93072, &UA54046},
	{SG93073, &UA547480},
	{SG93074, &UA2284074},
	{SG93075, &UA2469165},
	{SG93076, &UA2535974},
	{SG93078, &UA21798},
	{SG93079, &UA22083},
	{SG93080, &UA22456},
	{SG93081, &UA22916},
	{SG93082, &UA24729},
	{SG93083, &UA31169},
	{SG93084, &UA53934},
	{SG93085, &UA55059},
	{SG93086, &UA59203},
	{SG93087, &UA62844},
	{SG93088, &UA81736},
	{SG93089, &UA784964},
	{SG93090, &UA87722},
	{SG93091, &UA88031},
	{SG93092, &UA533832},
	{SG93093, &UA2597485},
	{SG93094, &UA2598111},
	{SG93095, &UA2607478},
	{SG93096, &UA2944606},
	{SG93097, &UA16360},
	{SG93098, &UA41391},
	{SG93099, &UA22249},
	{SG93100, &UA22932},
	{SG93101, &UA34331},
	{SG93102, &UA63330},
	{SG93103, &UA50036},
	{SG93106, &UA664837},
	{SG93107, &UA507184},
	{SG93108, &UA531242},
	{SG93109, &UA535156},
	{SG93110, &UA575007},
	{SG93111, &UA1866606},
	{SG93114, &UA2422699},
	{SG93115, &UA2747450},
	{SG93116, &UA2429421},
	{SG93117, &UA2963053},
	{SG93118, &UA3025161},
	{SG93119, &UA3031107},
	{SG93120, &UA6442},
	{SG93121, &UA21654},
	{SG93122, &UA22193},
	{SG93123, &UA22924},
	{SG93124, &UA22931},
	{SG93125, &UA28108},
	{SG93126, &UA49498},
	{SG93127, &UA50105},
	{SG93128, &UA51477},
	{SG93129, &UA61020},
	{SG93130, &UA61173},
	{SG93131, &UA81558},
	{SG93132, &UA81713},
	{SG93133, &UA81555},
	{SG93134, &UA86663},
	{SG93135, &UA87686},
	{SG93136, &UA87473},
	{SG93137, &UA2431311},
	{SG93138, &UA2506960},
	{SG93140, &UA2595938},
	{SG93141, &UA2986215},
	{SG93142, &UA2737567},
	{SG93143, &UA2764447},
	{SG93144, &UA2961885},
	{SG93145, &UA10102},
	{SG93146, &UA16935},
	{SG93147, &UA22992},
	{SG93148, &UA23180},
	{SG93149, &UA24324},
	{SG93150, &UA33020},
	{SG93151, &UA27978},
	{SG93152, &UA28027},
	{SG93153, &UA28381},
	{SG93154, &UA30784},
	{SG93155, &UA45206},
	{SG93156, &UA49023},
	{SG93157, &UA52035},
	{SG93158, &UA54213},
	{SG93159, &UA60539},
	{SG93160, &UA68037},
	{SG93161, &UA81590},
	{SG93162, &UA87471},
	{SG93163, &UA90261},
	{SG93164, &UA670412},
	{SG93165, &UA2607488},
	{SG93166, &UA2960807},
	{SG93167, &UA11252},
	{SG93168, &UA21031},
	{SG93169, &UA57941},
	{SG93170, &UA21514},
	{SG93171, &UA21515},
	{SG93172, &UA21949},
	{SG93173, &UA72550},
	{SG93174, &UA22610},
	{SG93175, &UA27997},
	{SG93176, &UA32743},
	{SG93177, &UA32965},
	{SG93178, &UA33009},
	{SG93179, &UA33042},
	{SG93180, &UA560263},
	{SG93181, &UA50197},
	{SG93182, &UA54781},
	{SG93183, &UA62347},
	{SG93184, &UA68958},
	{SG93185, &UA86943},
	{SG93187, &UA460139},
	{SG93190, &UA2778539},
	{SG93191, &UA2752550},
	{SG93192, &UA3025621},
	{SG93193, &UA3031426},
	{SG93194, &UA3492},
	{SG93195, &UA4398},
	{SG93196, &UA7637},
	{SG93197, &UA9086},
	{SG93198, &UA9149},
	{SG93199, &UA10116},
	{SG93200, &UA15159},
	{SG93201, &UA21534},
	{SG93202, &UA22627},
	{SG93203, &UA22683},
	{SG93204, &UA23174},
	{SG93205, &UA24220},
	{SG93206, &UA24713},
	{SG93209, &UA67772},
	{SG93210, &UA60747},
	{SG93211, &UA68567},
	{SG93212, &UA81581},
	{SG93213, &UA87503},
	{SG93218, &UA2506202},
	{SG93219, &UA2961544},
	{SG93221, &UA3059324},
	{SG93223, &UA21029},
	{SG93224, &UA21284},
	{SG93225, &UA61569},
	{SG93226, &UA21408},
	{SG93227, &UA55027},
	{SG93228, &UA23564},
	{SG93229, &UA23971},
	{SG93230, &UA68656},
	{SG93231, &UA33056},
	{SG93232, &UA50212},
	{SG93233, &UA50455},
	{SG93234, &UA51961},
	{SG93235, &UA61226},
	{SG93236, &UA61373},
	{SG93237, &UA63398},
	{SG93238, &UA64892},
	{SG93239, &UA66986},
	{SG93240, &UA67550},
	{SG93241, &UA68654},
	{SG93242, &UA87449},
	{SG93243, &UA87644},
	{SG93244, &UA243778},
	{SG93245, &UA2599339},
	{SG93246, &UA2730504},
	{SG93247, &UA2735187},
	{SG93248, &UA2779552},
	{SG93249, &UA2960975},
	{SG93250, &UA4153},
	{SG93252, &UA35888},
	{SG93253, &UA21137},
	{SG93254, &UA22084},
	{SG93255, &UA22352},
	{SG93256, &UA24649},
	{SG93257, &UA26879},
	{SG93258, &UA32868},
	{SG93259, &UA51001},
	{SG93260, &UA34322},
	{SG93261, &UA36966},
	{SG93262, &UA49314},
	{SG93263, &UA68283},
	{SG93264, &UA49434},
	{SG93265, &UA49893},
	{SG93266, &UA53824},
	{SG93267, &UA54596},
	{SG93268, &UA61092},
	{SG93269, &UA61253},
	{SG93270, &UA61556},
	{SG93271, &UA68326},
	{SG93272, &UA86472},
	{SG93273, &UA90351},
	{SG93274, &UA2530075},
	{SG93275, &UA2662080},
	{SG93276, &UA2847896},
	{SG93277, &UA2853553},
	{SG93278, &UA2868123},
	{SG93279, &UA3048429},
	{SG93280, &UA3058760},
	{SG93281, &UA3118320},
	{SG93282, &UA906},
	{SG93283, &UA2456},
	{SG93284, &UA4546},
	{SG93285, &UA54166},
	{SG93286, &UA18731},
	{SG93287, &UA35779},
	{SG93288, &UA21292},
	{SG93289, &UA23100},
	{SG93290, &UA23195},
	{SG93291, &UA30804},
	{SG93292, &UA32781},
	{SG93293, &UA35435},
	{SG93294, &UA37290},
	{SG93295, &UA44999},
	{SG93296, &UA49228},
	{SG93297, &UA54411},
	{SG93298, &UA55981},
	{SG93299, &UA60602},
	{SG93300, &UA60957},
	{SG93301, &UA61070},
	{SG93302, &UA61252},
	{SG93303, &UA61361},
	{SG93304, &UA61766},
	{SG93305, &UA68570},
	{SG93306, &UA81548},
	{SG93307, &UA82120},
	{SG93308, &UA86942},
	{SG93309, &UA447261},
	{SG93312, &UA2457721},
	{SG93313, &UA2483440},
	{SG93314, &UA2578849},
	{SG93315, &UA2751414},
	{SG93316, &UA2762786},
	{SG93317, &UA2776407},
	{SG93318, &UA2951730},
	{SG93319, &UA3011782},
	{SG93320, &UA3017063},
	{SG93321, &UA2667},
	{SG93322, &UA3640},
	{SG93323, &UA8903},
	{SG93324, &UA16680},
	{SG93326, &UA16854},
	{SG93327, &UA19193},
	{SG93328, &UA20258},
	{SG93329, &UA20589},
	{SG93330, &UA21717},
	{SG93332, &UA22516},
	{SG93333, &UA23326},
	{SG93334, &UA32999},
	{SG93335, &UA41314},
	{SG93336, &UA44073},
	{SG93337, &UA49236},
	{SG93338, &UA49616},
	{SG93339, &UA50186},
	{SG93340, &UA50270},
	{SG93341, &UA54161},
	{SG93342, &UA56199},
	{SG93343, &UA60528},
	{SG93344, &UA60663},
	{SG93345, &UA61305},
	{SG93346, &UA61520},
	{SG93347, &UA68638},
	{SG93348, &UA68673},
	{SG93349, &UA81615},
	{SG93350, &UA74244},
	{SG93351, &UA81630},
	{SG93352, &UA81841},
	{SG93353, &UA81875},
	{SG93354, &UA87300},
	{SG93355, &UA87700},
	{SG93357, &UA88261},
	{SG93358, &UA88946},
	{SG93359, &UA89071},
	{SG93360, &UA90354},
	{SG93362, &UA2428304},
	{SG93363, &UA2658162},
	{SG93364, &UA2751571},
	{SG93365, &UA2792088},
	{SG93366, &UA2844598},
	{SG93367, &UA2962976},
	{SG93368, &UA3015150},
	{SG93369, &UA3035661},
	{SG93370, &UA3058534},
	{SG93371, &UA7729},
	{SG93372, &UA8545},
	{SG93373, &UA9286},
	{SG93374, &UA11234},
	{SG93375, &UA13066},
	{SG93376, &UA16325},
	{SG93377, &UA16366},
	{SG93378, &UA17232},
	{SG93379, &UA18514},
	{SG93380, &UA19349},
	{SG93381, &UA19950},
	{SG93382, &UA20499},
	{SG93383, &UA21950},
	{SG93384, &UA22387},
	{SG93385, &UA23353},
	{SG93386, &UA25979},
	{SG93387, &UA32239},
	{SG93388, &UA32801},
	{SG93389, &UA33010},
	{SG93390, &UA33114},
	{SG93391, &UA33445},
	{SG93395, &UA42721},
	{SG93396, &UA44074},
	{SG93397, &UA44340},
	{SG93398, &UA49125},
	{SG93399, &UA49188},
	{SG93400, &UA49213},
	{SG93401, &UA49361},
	{SG93402, &UA49359},
	{SG93403, &UA49447},
	{SG93404, &UA49804},
	{SG93405, &UA49894},
	{SG93406, &UA50171},
	{SG93407, &UA50313},
	{SG93408, &UA51235},
	{SG93409, &UA51514},
	{SG93410, &UA52811},
	{SG93411, &UA54107},
	{SG93412, &UA54199},
	{SG93413, &UA54410},
	{SG93414, &UA56093},
	{SG93415, &UA58935},
	{SG93416, &UA60744},
	{SG93417, &UA61170},
	{SG93418, &UA61194},
	{SG93419, &UA61352},
	{SG93420, &UA61355},
	{SG93421, &UA67554},
	{SG93422, &UA68035},
	{SG93423, &UA68578},
	{SG93424, &UA68637},
	{SG93425, &UA68905},
	{SG93426, &UA71376},
	{SG93427, &UA81499},
	{SG93428, &UA81546},
	{SG93429, &UA81675},
	{SG93430, &UA81723},
	{SG93431, &UA81747},
	{SG93432, &UA81905},
	{SG93433, &UA81918},
	{SG93434, &UA82118},
	{SG93435, &UA81520},
	{SG93436, &UA86228},
	{SG93437, &UA86230},
	{SG93438, &UA86493},
	{SG93439, &UA86669},
	{SG93440, &UA87472},
	{SG93441, &UA87865},
	{SG93442, &UA88067},
	{SG93443, &UA88758},
	{SG93444, &UA158256},
	{SG93446, &UA390172},
	{SG93447, &UA572788},
	{SG93448, &UA692765},
	{SG93449, &UA2506200},
	{SG93457, &UA3034282},
	{NULL}};
const Segmentdata DSamsungGT[] = {
	{SG91430, &UA5093},
	{SG91431, &UA53805},
	{SG91432, &UA105148},
	{SG91439, &UA51086},
	{SG91440, &UA68456},
	{SG91452, &UA398155},
	{SG91453, &UA753653},
	{SG91454, &UA864432},
	{SG91455, &UA911551},
	{SG91456, &UA1957320},
	{SG91462, &UA2797949},
	{SG91468, &UA582080},
	{SG91471, &UA2431546},
	{SG91472, &UA2431189},
	{SG91475, &UA43623},
	{SG91476, &UA53457},
	{SG91477, &UA55065},
	{SG91478, &UA2798055},
	{SG91483, &UA23352},
	{SG91484, &UA24647},
	{SG91485, &UA55097},
	{SG91486, &UA61337},
	{SG91487, &UA64236},
	{SG91503, &UA3016},
	{SG91504, &UA60281},
	{SG91510, &UA345259},
	{SG91521, &UA3034961},
	{SG91522, &UA422242},
	{SG91523, &UA26477},
	{SG91524, &UA56654},
	{SG91526, &UA16509},
	{SG91529, &UA24014},
	{SG91530, &UA435672},
	{SG91531, &UA625424},
	{SG91536, &UA603592},
	{SG91537, &UA33074},
	{SG91538, &UA62723},
	{SG91539, &UA459249},
	{SG91540, &UA2654039},
	{SG91541, &UA2798012},
	{SG91542, &UA79079},
	{SG91543, &UA2160147},
	{SG91550, &UA10101},
	{SG91551, &UA51617},
	{SG91552, &UA62617},
	{SG91555, &UA354214},
	{SG91590, &UA26494},
	{SG91592, &UA64770},
	{SG91593, &UA90509},
	{SG91594, &UA163362},
	{SG91595, &UA164326},
	{SG91596, &UA170044},
	{SG91597, &UA173834},
	{SG91598, &UA408474},
	{SG91599, &UA517892},
	{SG91600, &UA662116},
	{SG91601, &UA2639392},
	{SG91609, &UA625810},
	{SG91610, &UA2157076},
	{SG91611, &UA635689},
	{SG91612, &UA2262907},
	{SG91617, &UA630607},
	{SG91618, &UA3034190},
	{SG91619, &UA3115928},
	{SG91622, &UA3034990},
	{SG91623, &UA31144},
	{SG91624, &UA62063},
	{SG91625, &UA53266},
	{SG91632, &UA46821},
	{SG91635, &UA3115162},
	{SG91639, &UA37611},
	{SG91640, &UA62193},
	{SG91641, &UA151245},
	{SG91642, &UA163359},
	{SG91643, &UA302196},
	{SG91644, &UA315372},
	{SG91645, &UA31210},
	{SG91659, &UA54931},
	{SG91662, &UA179778},
	{SG91668, &UA76627},
	{SG91670, &UA636609},
	{SG91671, &UA657443},
	{SG91672, &UA1953305},
	{SG91673, &UA2339113},
	{SG91674, &UA54314},
	{SG91676, &UA2040720},
	{SG91681, &UA9036},
	{SG91684, &UA675752},
	{SG91685, &UA686917},
	{SG91689, &UA55198},
	{SG91692, &UA2192580},
	{SG91693, &UA62485},
	{SG91694, &UA30135},
	{SG91697, &UA54727},
	{SG91701, &UA8558},
	{SG91702, &UA24669},
	{SG91703, &UA53846},
	{SG91704, &UA63112},
	{SG91706, &UA22268},
	{SG91707, &UA64463},
	{SG91708, &UA79130},
	{SG91711, &UA50048},
	{SG91713, &UA62892},
	{SG91715, &UA54793},
	{SG91716, &UA3035268},
	{SG91717, &UA14386},
	{SG91719, &UA459196},
	{SG91720, &UA32992},
	{SG91721, &UA698567},
	{SG91722, &UA777675},
	{SG91723, &UA23577},
	{SG91729, &UA9222},
	{SG91731, &UA19952},
	{SG91732, &UA27293},
	{SG91733, &UA49355},
	{SG91734, &UA26703},
	{SG91735, &UA51371},
	{SG91736, &UA60816},
	{SG91737, &UA62476},
	{SG91738, &UA24726},
	{SG91739, &UA62730},
	{SG91741, &UA62897},
	{SG91745, &UA421354},
	{SG91746, &UA641180},
	{SG91747, &UA1986130},
	{SG91749, &UA636267},
	{SG91773, &UA2798810},
	{SG91781, &UA33259},
	{SG91792, &UA23997},
	{SG91793, &UA24709},
	{SG91799, &UA62792},
	{SG91804, &UA62366},
	{SG91805, &UA62630},
	{SG91806, &UA51105},
	{SG91812, &UA24684},
	{SG91813, &UA561470},
	{SG91817, &UA44236},
	{SG91821, &UA62573},
	{SG91825, &UA3039218},
	{SG91860, &UA61911},
	{SG91867, &UA9078},
	{SG91868, &UA2683068},
	{SG91869, &UA658539},
	{SG91870, &UA2791997},
	{SG91886, &UA30987},
	{SG91887, &UA55178},
	{SG91888, &UA61639},
	{SG91889, &UA64577},
	{SG91891, &UA907},
	{SG91892, &UA16320},
	{SG91893, &UA16322},
	{SG91894, &UA16364},
	{SG91895, &UA24063},
	{SG91896, &UA30228},
	{SG91897, &UA34473},
	{SG91899, &UA26479},
	{SG91903, &UA27074},
	{SG91909, &UA43745},
	{SG91910, &UA62311},
	{SG91916, &UA2540542},
	{SG91918, &UA49342},
	{SG91919, &UA672928},
	{SG91926, &UA62022},
	{SG91927, &UA680524},
	{SG91937, &UA23983},
	{SG91938, &UA2144826},
	{SG91939, &UA2154913},
	{SG91943, &UA9491},
	{SG91944, &UA33434},
	{SG91945, &UA565505},
	{SG91946, &UA3026118},
	{SG91953, &UA897},
	{SG91954, &UA898},
	{SG91955, &UA905},
	{SG91956, &UA16326},
	{SG91957, &UA16363},
	{SG91958, &UA34475},
	{SG91959, &UA34467},
	{SG91960, &UA35447},
	{SG91961, &UA61801},
	{SG91962, &UA76868},
	{SG91963, &UA30912},
	{SG91964, &UA26848},
	{SG91965, &UA34404},
	{SG91967, &UA16600},
	{SG91968, &UA38482},
	{SG91969, &UA49892},
	{SG91970, &UA61174},
	{SG91971, &UA62044},
	{SG91972, &UA2413953},
	{SG91973, &UA48381},
	{SG91999, &UA33302},
	{SG92000, &UA54136},
	{SG92001, &UA63408},
	{SG92002, &UA630029},
	{SG92006, &UA61479},
	{SG92007, &UA2472995},
	{SG92015, &UA62392},
	{SG92029, &UA67276},
	{SG92030, &UA67441},
	{SG92031, &UA68297},
	{SG92032, &UA54037},
	{SG92033, &UA61978},
	{SG92051, &UA49238},
	{SG92052, &UA1953352},
	{SG92064, &UA675148},
	{SG92065, &UA64487},
	{SG92067, &UA53454},
	{SG92071, &UA27278},
	{SG92072, &UA29861},
	{SG92081, &UA24475},
	{SG92082, &UA38547},
	{SG92083, &UA1923855},
	{SG92091, &UA2540532},
	{SG92095, &UA2347},
	{SG92096, &UA5096},
	{SG92097, &UA9291},
	{SG92098, &UA39778},
	{SG92099, &UA63169},
	{SG92112, &UA2540516},
	{SG92116, &UA799481},
	{SG92117, &UA2956862},
	{SG92121, &UA24391},
	{SG92124, &UA743701},
	{SG92125, &UA3098129},
	{SG92126, &UA9035},
	{SG92135, &UA45232},
	{SG92141, &UA60603},
	{SG92143, &UA34405},
	{SG92150, &UA2540528},
	{SG92153, &UA16314},
	{SG92154, &UA63038},
	{SG92169, &UA379318},
	{SG92188, &UA2361017},
	{SG92201, &UA37421},
	{SG92202, &UA33331},
	{SG92203, &UA1906301},
	{SG92211, &UA2466650},
	{SG92212, &UA9985},
	{SG92213, &UA43795},
	{SG92214, &UA63759},
	{SG92244, &UA63897},
	{SG92248, &UA2421478},
	{SG92252, &UA672690},
	{SG92253, &UA3098828},
	{SG92255, &UA63938},
	{SG92266, &UA54017},
	{SG92267, &UA54234},
	{SG92272, &UA2396064},
	{SG92273, &UA2496795},
	{SG92274, &UA20244},
	{SG92279, &UA61855},
	{SG92280, &UA2578740},
	{SG92281, &UA5616},
	{SG92282, &UA64655},
	{SG92283, &UA19941},
	{SG92284, &UA23826},
	{SG92292, &UA45014},
	{SG92293, &UA63060},
	{SG92294, &UA64799},
	{SG92295, &UA1952325},
	{SG92318, &UA2370593},
	{SG92319, &UA3116165},
	{SG92320, &UA8551},
	{SG92321, &UA24247},
	{SG92327, &UA1989073},
	{SG92334, &UA54784},
	{SG92350, &UA8728},
	{SG92351, &UA10411},
	{SG92352, &UA515144},
	{SG92354, &UA23976},
	{SG92365, &UA43641},
	{SG92376, &UA24744},
	{SG92377, &UA702361},
	{SG92378, &UA2431590},
	{SG92382, &UA2577596},
	{SG92384, &UA22855},
	{SG92385, &UA23413},
	{SG92397, &UA44314},
	{SG92398, &UA62391},
	{SG92402, &UA64160},
	{SG92411, &UA2401868},
	{SG92464, &UA22326},
	{SG92465, &UA46460},
	{SG92466, &UA63033},
	{SG92478, &UA48989},
	{SG92529, &UA24382},
	{SG92535, &UA20875},
	{SG92536, &UA58671},
	{SG92540, &UA49448},
	{SG92542, &UA2696484},
	{SG92543, &UA20486},
	{SG92564, &UA2712979},
	{SG92568, &UA26916},
	{SG92574, &UA10007},
	{SG92575, &UA55082},
	{SG92576, &UA760500},
	{SG92577, &UA2126386},
	{SG92578, &UA2270245},
	{SG92590, &UA62408},
	{SG92591, &UA2262899},
	{SG92593, &UA158702},
	{SG92602, &UA9410},
	{SG92605, &UA33292},
	{SG92632, &UA33431},
	{SG92642, &UA57053},
	{SG92647, &UA2456215},
	{SG92648, &UA2730306},
	{SG92663, &UA2449462},
	{SG92677, &UA6159},
	{SG92678, &UA50369},
	{SG92682, &UA2216567},
	{SG92694, &UA4543},
	{SG92695, &UA48912},
	{SG92704, &UA2424191},
	{SG92722, &UA8726},
	{SG92723, &UA10410},
	{SG92728, &UA2852970},
	{SG92730, &UA2413654},
	{SG92735, &UA24498},
	{SG92743, &UA63576},
	{SG92744, &UA656624},
	{SG92749, &UA651339},
	{SG92759, &UA3035896},
	{SG92778, &UA24244},
	{SG92779, &UA64248},
	{SG92783, &UA67100},
	{SG92784, &UA3086241},
	{SG92788, &UA2984374},
	{SG92795, &UA3115838},
	{SG92798, &UA69405},
	{SG92800, &UA54375},
	{SG92801, &UA32991},
	{SG92802, &UA2793248},
	{SG92803, &UA2482485},
	{SG92805, &UA23427},
	{SG92806, &UA49989},
	{SG92807, &UA1942560},
	{SG92820, &UA16321},
	{SG92821, &UA61802},
	{SG92822, &UA802510},
	{SG92824, &UA69222},
	{SG92827, &UA2779612},
	{SG92841, &UA2529398},
	{SG92889, &UA1993177},
	{SG92912, &UA691285},
	{SG92916, &UA2695961},
	{SG92923, &UA54587},
	{SG92924, &UA55137},
	{SG92934, &UA63007},
	{SG92946, &UA21419},
	{SG92947, &UA68345},
	{SG92967, &UA3049620},
	{SG92985, &UA2508291},
	{SG92986, &UA49533},
	{SG92989, &UA2757868},
	{SG92990, &UA2803957},
	{SG92996, &UA1865803},
	{SG93005, &UA2165190},
	{SG93028, &UA23329},
	{SG93039, &UA30905},
	{SG93055, &UA2456312},
	{SG93077, &UA2702667},
	{SG93104, &UA24006},
	{SG93105, &UA78564},
	{SG93112, &UA31178},
	{SG93113, &UA61133},
	{SG93139, &UA2463035},
	{SG93186, &UA2553941},
	{SG93188, &UA32042},
	{SG93189, &UA62375},
	{SG93207, &UA20696},
	{SG93208, &UA2302999},
	{SG93214, &UA32765},
	{SG93215, &UA2518434},
	{SG93216, &UA2456685},
	{SG93217, &UA2657342},
	{SG93220, &UA50355},
	{SG93222, &UA2553718},
	{SG93251, &UA3124610},
	{SG93310, &UA64146},
	{SG93311, &UA64839},
	{SG93325, &UA26506},
	{SG93331, &UA649769},
	{SG93356, &UA8521},
	{SG93361, &UA2554106},
	{SG93392, &UA2386175},
	{SG93393, &UA2528346},
	{SG93394, &UA2548537},
	{SG93445, &UA69085},
	{SG93450, &UA23965},
	{SG93451, &UA54246},
	{SG93452, &UA2435983},
	{SG93453, &UA2653960},
	{SG93454, &UA62037},
	{SG93455, &UA2212232},
	{SG93456, &UA2810497},
	{SG93458, &UA16932},
	{SG93459, &UA2410832},
	{SG93460, &UA3011555},
	{SG93461, &UA2698420},
	{SG93462, &UA2527261},
	{SG93463, &UA5097},
	{SG93464, &UA2466557},
	{SG93465, &UA43634},
	{SG93466, &UA52779},
	{SG93467, &UA2934933},
	{SG93468, &UA24606},
	{SG93469, &UA3050766},
	{SG93470, &UA55064},
	{SG93471, &UA1942804},
	{SG93472, &UA20267},
	{SG93473, &UA90355},
	{SG93474, &UA5098},
	{SG93475, &UA63379},
	{SG93476, &UA2372195},
	{SG93477, &UA2393986},
	{SG93478, &UA2866807},
	{SG93479, &UA2876248},
	{SG93480, &UA564436},
	{SG93481, &UA1864970},
	{SG93482, &UA2474977},
	{SG93485, &UA62322},
	{SG93486, &UA2574114},
	{SG93487, &UA27292},
	{SG93488, &UA53840},
	{SG93489, &UA61488},
	{SG93496, &UA2822112},
	{SG93497, &UA2527053},
	{SG93498, &UA67540},
	{SG93499, &UA2577718},
	{SG93500, &UA26421},
	{SG93501, &UA90367},
	{SG93502, &UA2688452},
	{SG93503, &UA2180502},
	{SG93504, &UA2720142},
	{SG93505, &UA2763190},
	{SG93506, &UA2980509},
	{SG93507, &UA23080},
	{SG93508, &UA41073},
	{SG93509, &UA37579},
	{SG93510, &UA69645},
	{SG93511, &UA2849201},
	{SG93512, &UA51200},
	{SG93513, &UA63171},
	{SG93514, &UA2352203},
	{SG93515, &UA2978834},
	{SG93516, &UA3073364},
	{SG93517, &UA21270},
	{SG93518, &UA50312},
	{SG93519, &UA54589},
	{SG93520, &UA63223},
	{SG93521, &UA2803258},
	{SG93522, &UA42189},
	{SG93523, &UA55186},
	{SG93524, &UA2804270},
	{SG93525, &UA2841367},
	{SG93526, &UA53681},
	{SG93527, &UA60978},
	{SG93528, &UA3049770},
	{SG93529, &UA68012},
	{SG93530, &UA2816693},
	{SG93531, &UA24466},
	{SG93532, &UA38492},
	{SG93533, &UA44077},
	{SG93534, &UA2747960},
	{SG93535, &UA2841846},
	{SG93536, &UA2982891},
	{SG93537, &UA3041101},
	{SG93538, &UA3084805},
	{SG93539, &UA3097905},
	{SG93540, &UA2723616},
	{SG93541, &UA3118092},
	{SG93544, &UA2438904},
	{SG93545, &UA2831759},
	{SG93546, &UA10238},
	{SG93547, &UA20402},
	{SG93548, &UA62247},
	{SG93549, &UA14334},
	{SG93550, &UA33090},
	{SG93551, &UA50734},
	{SG93552, &UA68628},
	{SG93553, &UA2242326},
	{SG93554, &UA43793},
	{SG93555, &UA6013},
	{SG93556, &UA9875},
	{SG93557, &UA16365},
	{SG93558, &UA925774},
	{SG93559, &UA78793},
	{SG93560, &UA27975},
	{SG93561, &UA33168},
	{SG93562, &UA2817149},
	{SG93563, &UA2823764},
	{SG93564, &UA2304047},
	{SG93565, &UA60812},
	{SG93566, &UA67244},
	{SG93567, &UA2820685},
	{SG93568, &UA21418},
	{SG93569, &UA27291},
	{SG93570, &UA51128},
	{SG93571, &UA62981},
	{SG93572, &UA67314},
	{SG93573, &UA21143},
	{SG93574, &UA51976},
	{SG93575, &UA62618},
	{SG93576, &UA459255},
	{SG93577, &UA20419},
	{SG93578, &UA554157},
	{SG93579, &UA2812733},
	{SG93580, &UA16319},
	{SG93581, &UA17216},
	{SG93582, &UA24775},
	{SG93583, &UA459253},
	{SG93584, &UA54405},
	{SG93585, &UA3115056},
	{SG93586, &UA33255},
	{SG93587, &UA49049},
	{SG93588, &UA49446},
	{SG93589, &UA49948},
	{SG93590, &UA64429},
	{SG93591, &UA459254},
	{SG93592, &UA3110595},
	{SG93593, &UA77744},
	{SG93594, &UA37115},
	{SG93595, &UA40792},
	{SG93596, &UA52030},
	{SG93597, &UA53623},
	{SG93598, &UA54137},
	{SG93599, &UA56063},
	{SG93600, &UA3111687},
	{SG93601, &UA53883},
	{SG93602, &UA56065},
	{SG93603, &UA78048},
	{SG93604, &UA459250},
	{SG93605, &UA3114674},
	{SG93606, &UA27021},
	{SG93607, &UA29864},
	{SG93608, &UA49373},
	{SG93609, &UA2817434},
	{SG93610, &UA9159},
	{SG93611, &UA9486},
	{SG93612, &UA36480},
	{SG93613, &UA41179},
	{SG93614, &UA56105},
	{SG93615, &UA60542},
	{SG93616, &UA61126},
	{SG93617, &UA63909},
	{SG93618, &UA2540543},
	{SG93619, &UA24311},
	{SG93620, &UA44927},
	{SG93621, &UA46506},
	{SG93622, &UA49218},
	{SG93623, &UA54138},
	{SG93624, &UA459197},
	{SG93625, &UA2322620},
	{SG93626, &UA2540523},
	{SG93627, &UA2540524},
	{SG93628, &UA2540535},
	{SG93629, &UA3126328},
	{SG93630, &UA22227},
	{SG93631, &UA31190},
	{SG93632, &UA32902},
	{SG93633, &UA35438},
	{SG93634, &UA54391},
	{SG93635, &UA67798},
	{SG93636, &UA459248},
	{SG93637, &UA2540518},
	{SG93638, &UA2540534},
	{SG93639, &UA3111478},
	{SG93640, &UA3126729},
	{NULL}};
const Segmentdata DSamsungSeries60[] = {
	{SG92162, &UA8479},
	{SG92163, &UA8478},
	{SG92164, &UA9820},
	{SG92165, &UA9822},
	{SG92166, &UA9823},
	{SG92167, &UA9824},
	{SG92168, &UA64705},
	{SG92424, &UA1688},
	{SG92425, &UA1689},
	{SG92426, &UA2235},
	{SG92427, &UA17277},
	{SG92428, &UA17279},
	{SG92429, &UA17278},
	{SG92430, &UA17280},
	{SG92431, &UA18750},
	{SG92432, &UA18751},
	{SG92433, &UA18752},
	{SG92434, &UA18753},
	{SG92435, &UA18754},
	{SG92436, &UA18755},
	{SG92437, &UA18756},
	{SG92438, &UA18757},
	{SG92439, &UA18759},
	{SG92440, &UA18760},
	{SG92441, &UA18761},
	{SG92442, &UA18762},
	{SG92443, &UA18763},
	{SG92444, &UA18764},
	{SG92445, &UA18758},
	{SG92446, &UA39195},
	{SG92447, &UA44848},
	{SG92448, &UA50735},
	{SG92449, &UA54990},
	{SG92450, &UA62397},
	{SG92452, &UA62815},
	{SG92453, &UA62881},
	{SG92479, &UA650},
	{SG92480, &UA651},
	{SG92481, &UA693},
	{SG92482, &UA709},
	{SG92483, &UA711},
	{SG92484, &UA712},
	{SG92485, &UA713},
	{SG92486, &UA714},
	{SG92487, &UA15969},
	{SG92488, &UA15970},
	{SG92489, &UA15971},
	{SG92490, &UA15972},
	{SG92491, &UA15973},
	{SG92492, &UA15975},
	{SG92493, &UA15976},
	{SG92494, &UA15978},
	{SG92495, &UA15979},
	{SG92496, &UA15980},
	{SG92497, &UA15981},
	{SG92498, &UA15974},
	{SG92499, &UA18433},
	{SG92500, &UA18434},
	{SG92501, &UA31847},
	{SG92502, &UA31848},
	{SG92503, &UA31849},
	{SG92504, &UA31850},
	{SG92505, &UA31860},
	{SG92506, &UA31861},
	{SG92507, &UA35301},
	{SG92508, &UA53848},
	{SG92509, &UA53849},
	{SG92510, &UA53982},
	{SG92511, &UA54190},
	{SG92512, &UA54586},
	{SG92514, &UA62171},
	{SG92515, &UA62246},
	{SG92516, &UA62443},
	{SG92517, &UA62447},
	{SG92519, &UA63646},
	{SG92522, &UA63896},
	{SG92524, &UA86682},
	{SG92525, &UA86977},
	{SG92850, &UA4209},
	{SG92851, &UA4210},
	{SG92852, &UA6134},
	{SG92853, &UA20721},
	{SG92854, &UA24122},
	{SG92855, &UA24123},
	{SG92856, &UA24320},
	{SG92857, &UA31125},
	{SG92858, &UA31127},
	{SG92859, &UA31126},
	{SG92861, &UA31128},
	{SG92862, &UA31130},
	{SG92863, &UA31131},
	{SG92865, &UA31132},
	{SG92866, &UA36500},
	{SG92868, &UA37375},
	{SG92869, &UA37376},
	{SG92870, &UA45055},
	{SG92871, &UA50701},
	{SG92873, &UA50703},
	{SG92874, &UA50702},
	{SG92875, &UA50704},
	{SG92876, &UA55034},
	{SG92877, &UA62241},
	{SG92878, &UA64139},
	{SG92879, &UA64580},
	{SG93018, &UA5998},
	{SG93019, &UA24538},
	{SG93020, &UA30812},
	{SG93021, &UA30813},
	{SG93022, &UA30814},
	{SG93023, &UA30815},
	{SG93024, &UA36478},
	{SG93025, &UA62917},
	{SG93483, &UA64175},
	{SG93484, &UA72571},
	{SG93490, &UA6266},
	{SG93491, &UA43772},
	{SG93492, &UA43773},
	{SG93493, &UA56312},
	{SG93494, &UA56314},
	{SG93495, &UA62914},
	{SG93542, &UA31171},
	{SG93543, &UA31172},
	{SG93661, &UA49435},
	{SG93693, &UA50986},
	{SG93694, &UA56919},
	{SG93695, &UA33173},
	{SG93696, &UA86683},
	{NULL}};
const Device *DSharp[] = {
	&UA90086,
	&UA2427684,
	&UA55024,
	&UA2595381,
	&UA19966,
	&UA63494,
	&UA84692,
	&UA54534,
	&UA44842,
	&UA64779,
	&UA53679,
	&UA54873,
	&UA54192,
	&UA53351,
	&UA20326,
	&UA84650,
	&UA21626,
	&UA54219,
	&UA61839,
	&UA24354,
	NULL};
const Device *DSanyo[] = {
	&UA43679,
	&UA61074,
	&UA33065,
	&UA54557,
	&UA55211,
	&UA2215291,
	&UA30326,
	&UA54633,
	&UA30924,
	&UA55051,
	&UA30167,
	&UA2413802,
	&UA23887,
	&UA62557,
	&UA61168,
	&UA86335,
	&UA24736,
	&UA64755,
	&UA60629,
	&UA21913,
	&UA50991,
	&UA61299,
	&UA49891,
	NULL};
const Device *DSiemens[] = {
	&UA21345,
	&UA23886,
	&UA26699,
	&UA26761,
	&UA43648,
	&UA53466,
	&UA60871,
	&UA61477,
	&UA81339,
	&UA81433,
	&UA87571,
	&UA23087,
	&UA33141,
	&UA61920,
	&UA64277,
	&UA2232,
	&UA18744,
	&UA21379,
	&UA15265,
	&UA26742,
	&UA44268,
	&UA54229,
	&UA54394,
	&UA54154,
	&UA64083,
	&UA61780,
	&UA63413,
	&UA87650,
	&UA21632,
	&UA23492,
	&UA61633,
	&UA22508,
	&UA49815,
	&UA61119,
	&UA62333,
	&UA63541,
	&UA22471,
	&UA53882,
	&UA53975,
	&UA50975,
	&UA63394,
	&UA81425,
	NULL};
const Segmentdata DSkyfire[] = {
	{SG92275, &UA6529},
	{SG92276, &UA9700},
	{SG92277, &UA42301},
	{SG92278, &UA741207},
	{NULL}};
const Segmentdata DSoftBank[] = {
	{SG92410, &UA90177},
	{SG93815, &UA90509},
	{SG93817, &UA163362},
	{SG93818, &UA164326},
	{SG93819, &UA170044},
	{SG93820, &UA173834},
	{SG93821, &UA408474},
	{SG93822, &UA517892},
	{SG93823, &UA2639392},
	{SG93848, &UA81575},
	{SG93849, &UA88774},
	{SG93850, &UA88785},
	{SG93851, &UA94082},
	{SG93852, &UA442783},
	{SG93853, &UA2263267},
	{SG93856, &UA151245},
	{SG93857, &UA163359},
	{SG93858, &UA302196},
	{SG93859, &UA315372},
	{SG93993, &UA103328},
	{SG93994, &UA131532},
	{SG93995, &UA167158},
	{SG93996, &UA275795},
	{SG93997, &UA436491},
	{SG93998, &UA494971},
	{SG93999, &UA652247},
	{SG94000, &UA1845120},
	{SG94001, &UA2275937},
	{SG94003, &UA88841},
	{SG94052, &UA86263},
	{SG94053, &UA88092},
	{SG94054, &UA88233},
	{SG94055, &UA88629},
	{SG94056, &UA101596},
	{SG94057, &UA123887},
	{SG94058, &UA124872},
	{SG94059, &UA287581},
	{SG94060, &UA89141},
	{SG94061, &UA100984},
	{SG94062, &UA116898},
	{SG94063, &UA128468},
	{SG94064, &UA135029},
	{SG94065, &UA171711},
	{SG94066, &UA171963},
	{SG94067, &UA376983},
	{SG94068, &UA433048},
	{SG94069, &UA443071},
	{SG94070, &UA720494},
	{SG94071, &UA809289},
	{SG94072, &UA1861002},
	{SG94100, &UA103683},
	{SG94101, &UA371651},
	{SG94102, &UA2263927},
	{SG94158, &UA88463},
	{SG94160, &UA93301},
	{SG94161, &UA125192},
	{SG94162, &UA131540},
	{SG94166, &UA2807643},
	{SG94182, &UA89029},
	{SG94187, &UA114333},
	{SG94188, &UA273317},
	{SG94189, &UA295261},
	{SG94190, &UA1839935},
	{SG94191, &UA89099},
	{SG94192, &UA466813},
	{SG94193, &UA113579},
	{SG94194, &UA141434},
	{SG94195, &UA459924},
	{SG94223, &UA87403},
	{SG94224, &UA2977342},
	{SG94237, &UA89047},
	{SG94238, &UA281454},
	{SG94239, &UA477154},
	{SG94240, &UA499971},
	{SG94241, &UA88483},
	{SG94242, &UA637878},
	{SG94243, &UA2134767},
	{SG94272, &UA262616},
	{SG94282, &UA130861},
	{SG94300, &UA90153},
	{SG94301, &UA146093},
	{SG94302, &UA178549},
	{SG94303, &UA494705},
	{SG94304, &UA511281},
	{SG94305, &UA2262600},
	{SG94311, &UA515144},
	{SG94315, &UA702361},
	{SG94317, &UA1842205},
	{SG94325, &UA87079},
	{SG94326, &UA88894},
	{SG94327, &UA98987},
	{SG94328, &UA261802},
	{SG94329, &UA639541},
	{SG94330, &UA860642},
	{SG94331, &UA2127844},
	{SG94334, &UA710961},
	{SG94340, &UA760500},
	{SG94341, &UA2126386},
	{SG94342, &UA2270245},
	{SG94384, &UA88884},
	{SG94385, &UA778332},
	{SG94419, &UA89862},
	{SG94420, &UA2262914},
	{SG94425, &UA89100},
	{SG94427, &UA554526},
	{SG94428, &UA838726},
	{SG94449, &UA349527},
	{SG94457, &UA58638},
	{SG94484, &UA273120},
	{SG94489, &UA2262130},
	{SG94504, &UA87314},
	{SG94510, &UA87047},
	{SG94523, &UA53354},
	{SG94529, &UA80358},
	{SG94530, &UA88019},
	{SG94531, &UA547278},
	{SG94532, &UA88014},
	{SG94535, &UA53935},
	{SG94542, &UA86938},
	{SG94543, &UA318049},
	{SG94544, &UA87664},
	{SG94545, &UA81736},
	{SG94546, &UA784964},
	{SG94555, &UA1866606},
	{SG94556, &UA87632},
	{SG94557, &UA88017},
	{SG94558, &UA88013},
	{SG94560, &UA86943},
	{SG94561, &UA460139},
	{SG94567, &UA22327},
	{SG94569, &UA87690},
	{SG94570, &UA87769},
	{SG94571, &UA61040},
	{SG94573, &UA87765},
	{SG94574, &UA87876},
	{SG94575, &UA87877},
	{SG94577, &UA42700},
	{SG94579, &UA87637},
	{SG94580, &UA87887},
	{SG94581, &UA88015},
	{SG94582, &UA20139},
	{SG94583, &UA28380},
	{SG94584, &UA28438},
	{SG94585, &UA33138},
	{SG94586, &UA38353},
	{SG94587, &UA48888},
	{SG94588, &UA49294},
	{SG94590, &UA86942},
	{SG94591, &UA87747},
	{SG94596, &UA48947},
	{SG94597, &UA51979},
	{SG94598, &UA61585},
	{SG94599, &UA86485},
	{SG94600, &UA87709},
	{SG94601, &UA87888},
	{SG94602, &UA88012},
	{SG94603, &UA88018},
	{SG94604, &UA89071},
	{SG94606, &UA21142},
	{SG94607, &UA22332},
	{SG94610, &UA86493},
	{SG94611, &UA87405},
	{SG94612, &UA87766},
	{NULL}};
const Segmentdata DSonyEricsson[] = {
	{SG93641, &UA891535},
	{SG93642, &UA900679},
	{SG93643, &UA902029},
	{SG93644, &UA904153},
	{SG93645, &UA924100},
	{SG93646, &UA1956580},
	{SG93647, &UA1990415},
	{SG93648, &UA1997108},
	{SG93649, &UA2042952},
	{SG93650, &UA2166814},
	{SG93651, &UA2168316},
	{SG93652, &UA2205590},
	{SG93653, &UA2228159},
	{SG93654, &UA2237439},
	{SG93655, &UA2350834},
	{SG93656, &UA2384581},
	{SG93657, &UA2759938},
	{SG93658, &UA2815649},
	{SG93659, &UA2982737},
	{SG93660, &UA2988977},
	{SG93662, &UA32810},
	{SG93663, &UA48638},
	{SG93664, &UA48745},
	{SG93665, &UA49661},
	{SG93666, &UA49667},
	{SG93667, &UA49674},
	{SG93668, &UA72573},
	{SG93669, &UA74196},
	{SG93670, &UA75802},
	{SG93671, &UA78067},
	{SG93672, &UA78608},
	{SG93673, &UA369507},
	{SG93674, &UA623025},
	{SG93675, &UA623464},
	{SG93676, &UA625057},
	{SG93677, &UA625344},
	{SG93678, &UA625414},
	{SG93679, &UA628507},
	{SG93680, &UA629618},
	{SG93681, &UA630679},
	{SG93682, &UA674074},
	{SG93683, &UA695615},
	{SG93684, &UA722845},
	{SG93685, &UA725288},
	{SG93686, &UA1962458},
	{SG93687, &UA1964322},
	{SG93688, &UA2585365},
	{SG93689, &UA2641739},
	{SG93690, &UA2030399},
	{SG93691, &UA2437197},
	{SG93692, &UA2656751},
	{SG93697, &UA18215},
	{SG93698, &UA18221},
	{SG93699, &UA18281},
	{SG93700, &UA18306},
	{SG93701, &UA18310},
	{SG93702, &UA18319},
	{SG93703, &UA18324},
	{SG93704, &UA18330},
	{SG93705, &UA24592},
	{SG93706, &UA32920},
	{SG93707, &UA41664},
	{SG93708, &UA41754},
	{SG93709, &UA47095},
	{SG93710, &UA47106},
	{SG93711, &UA47110},
	{SG93712, &UA47144},
	{SG93713, &UA47175},
	{SG93714, &UA47188},
	{SG93715, &UA47190},
	{SG93716, &UA47148},
	{SG93717, &UA48191},
	{SG93718, &UA49642},
	{SG93719, &UA49633},
	{SG93720, &UA49645},
	{SG93721, &UA49648},
	{SG93722, &UA57366},
	{SG93723, &UA57369},
	{SG93724, &UA57399},
	{SG93725, &UA57498},
	{SG93726, &UA57500},
	{SG93727, &UA57507},
	{SG93728, &UA57512},
	{SG93729, &UA57535},
	{SG93730, &UA57544},
	{SG93731, &UA59575},
	{SG93732, &UA60132},
	{SG93733, &UA60134},
	{SG93734, &UA60161},
	{SG93735, &UA60194},
	{SG93736, &UA61197},
	{SG93737, &UA70095},
	{SG93738, &UA70349},
	{SG93739, &UA70371},
	{SG93740, &UA70715},
	{SG93741, &UA71665},
	{SG93742, &UA74367},
	{SG93743, &UA74912},
	{SG93744, &UA75483},
	{SG93745, &UA76260},
	{SG93746, &UA76348},
	{SG93747, &UA79386},
	{SG93748, &UA90574},
	{SG93749, &UA368893},
	{SG93750, &UA368927},
	{SG93751, &UA369198},
	{SG93752, &UA371303},
	{SG93753, &UA457081},
	{SG93754, &UA624805},
	{SG93755, &UA637574},
	{SG93756, &UA2028837},
	{SG93757, &UA2643892},
	{SG93758, &UA2979075},
	{SG93759, &UA2988778},
	{SG93760, &UA32897},
	{SG93761, &UA47520},
	{SG93762, &UA47549},
	{SG93763, &UA47550},
	{SG93764, &UA47530},
	{SG93765, &UA47582},
	{SG93766, &UA48740},
	{SG93767, &UA49769},
	{SG93768, &UA49884},
	{SG93769, &UA60441},
	{SG93770, &UA70755},
	{SG93771, &UA76713},
	{SG93772, &UA76749},
	{SG93773, &UA77202},
	{SG93774, &UA79448},
	{SG93775, &UA351166},
	{SG93776, &UA424862},
	{SG93777, &UA454711},
	{SG93778, &UA623841},
	{SG93779, &UA624994},
	{SG93780, &UA628727},
	{SG93781, &UA634124},
	{SG93782, &UA637949},
	{SG93783, &UA639759},
	{SG93784, &UA659141},
	{SG93785, &UA696484},
	{SG93786, &UA1932163},
	{SG93787, &UA2032655},
	{SG93788, &UA2357303},
	{SG93789, &UA2641274},
	{SG93790, &UA2733320},
	{SG93791, &UA616665},
	{SG93792, &UA2030066},
	{SG93793, &UA2030981},
	{SG93794, &UA2375501},
	{SG93795, &UA20217},
	{SG93796, &UA46965},
	{SG93797, &UA46976},
	{SG93798, &UA46992},
	{SG93799, &UA47008},
	{SG93800, &UA47045},
	{SG93801, &UA60094},
	{SG93802, &UA70243},
	{SG93803, &UA70818},
	{SG93804, &UA458431},
	{SG93805, &UA576614},
	{SG93806, &UA623666},
	{SG93807, &UA624246},
	{SG93808, &UA625216},
	{SG93809, &UA629244},
	{SG93810, &UA633660},
	{SG93811, &UA653817},
	{SG93812, &UA662783},
	{SG93813, &UA665443},
	{SG93814, &UA703381},
	{SG93816, &UA708427},
	{SG93824, &UA2021161},
	{SG93825, &UA16776},
	{SG93826, &UA18662},
	{SG93827, &UA19745},
	{SG93828, &UA19789},
	{SG93829, &UA43940},
	{SG93830, &UA73004},
	{SG93831, &UA72886},
	{SG93832, &UA73056},
	{SG93833, &UA78285},
	{SG93834, &UA254012},
	{SG93835, &UA354611},
	{SG93836, &UA369401},
	{SG93837, &UA453126},
	{SG93838, &UA625803},
	{SG93839, &UA628101},
	{SG93840, &UA633473},
	{SG93841, &UA637141},
	{SG93842, &UA641442},
	{SG93843, &UA674056},
	{SG93844, &UA734234},
	{SG93845, &UA761450},
	{SG93846, &UA1979878},
	{SG93847, &UA2888893},
	{SG93854, &UA2008883},
	{SG93855, &UA2422631},
	{SG93860, &UA440864},
	{SG93861, &UA603584},
	{SG93862, &UA624787},
	{SG93863, &UA664232},
	{SG93864, &UA732306},
	{SG93865, &UA862608},
	{SG93866, &UA900145},
	{SG93867, &UA900413},
	{SG93868, &UA902589},
	{SG93869, &UA2024516},
	{SG93870, &UA2025182},
	{SG93871, &UA17984},
	{SG93872, &UA18017},
	{SG93873, &UA18023},
	{SG93874, &UA18035},
	{SG93875, &UA18047},
	{SG93876, &UA24565},
	{SG93877, &UA32995},
	{SG93878, &UA41464},
	{SG93879, &UA41443},
	{SG93880, &UA41483},
	{SG93881, &UA47198},
	{SG93882, &UA47248},
	{SG93883, &UA49686},
	{SG93884, &UA49688},
	{SG93885, &UA49694},
	{SG93886, &UA49696},
	{SG93887, &UA53618},
	{SG93888, &UA57163},
	{SG93889, &UA57170},
	{SG93890, &UA57181},
	{SG93891, &UA57188},
	{SG93892, &UA57190},
	{SG93893, &UA57196},
	{SG93894, &UA57198},
	{SG93895, &UA57217},
	{SG93896, &UA57224},
	{SG93897, &UA57239},
	{SG93898, &UA60209},
	{SG93899, &UA60225},
	{SG93900, &UA60228},
	{SG93901, &UA60240},
	{SG93902, &UA60249},
	{SG93903, &UA60273},
	{SG93904, &UA60272},
	{SG93905, &UA70330},
	{SG93906, &UA70421},
	{SG93907, &UA72834},
	{SG93908, &UA90548},
	{SG93909, &UA252017},
	{SG93910, &UA398611},
	{SG93911, &UA625300},
	{SG93912, &UA626496},
	{SG93913, &UA632236},
	{SG93914, &UA662124},
	{SG93915, &UA673468},
	{SG93916, &UA1902211},
	{SG93917, &UA2193409},
	{SG93918, &UA2203499},
	{SG93919, &UA2223690},
	{SG93920, &UA20746},
	{SG93921, &UA20765},
	{SG93922, &UA23503},
	{SG93923, &UA24561},
	{SG93924, &UA43084},
	{SG93925, &UA43092},
	{SG93926, &UA43119},
	{SG93927, &UA48221},
	{SG93928, &UA48223},
	{SG93929, &UA48229},
	{SG93930, &UA48234},
	{SG93931, &UA49806},
	{SG93932, &UA49864},
	{SG93933, &UA49869},
	{SG93934, &UA49868},
	{SG93935, &UA58993},
	{SG93936, &UA59029},
	{SG93937, &UA59056},
	{SG93938, &UA59049},
	{SG93939, &UA59091},
	{SG93940, &UA61201},
	{SG93941, &UA61207},
	{SG93942, &UA70587},
	{SG93943, &UA70998},
	{SG93944, &UA74481},
	{SG93945, &UA253985},
	{SG93946, &UA368781},
	{SG93947, &UA428408},
	{SG93948, &UA688825},
	{SG93949, &UA751518},
	{SG93950, &UA1963421},
	{SG93951, &UA2025462},
	{SG93952, &UA2317331},
	{SG93953, &UA2957686},
	{SG93954, &UA48116},
	{SG93955, &UA49551},
	{SG93956, &UA49558},
	{SG93957, &UA78038},
	{SG93958, &UA350411},
	{SG93959, &UA370849},
	{SG93960, &UA630394},
	{SG93961, &UA631750},
	{SG93962, &UA633731},
	{SG93963, &UA662982},
	{SG93964, &UA670306},
	{SG93965, &UA686175},
	{SG93966, &UA714738},
	{SG93967, &UA742125},
	{SG93968, &UA779582},
	{SG93969, &UA1992957},
	{SG93970, &UA924164},
	{SG93971, &UA2000851},
	{SG93972, &UA2257173},
	{SG93973, &UA2342982},
	{SG93974, &UA2404213},
	{SG93975, &UA2982176},
	{SG93976, &UA22231},
	{SG93977, &UA45508},
	{SG93978, &UA45513},
	{SG93979, &UA45587},
	{SG93980, &UA45662},
	{SG93981, &UA45677},
	{SG93982, &UA45730},
	{SG93983, &UA50027},
	{SG93984, &UA59592},
	{SG93985, &UA59607},
	{SG93986, &UA59595},
	{SG93987, &UA77922},
	{SG93988, &UA873074},
	{SG93989, &UA2021380},
	{SG93990, &UA2022607},
	{SG93991, &UA2029985},
	{SG93992, &UA2642764},
	{SG94002, &UA907616},
	{SG94004, &UA2008428},
	{SG94005, &UA2183235},
	{SG94006, &UA2230387},
	{SG94007, &UA47401},
	{SG94008, &UA49513},
	{SG94009, &UA49761},
	{SG94010, &UA49764},
	{SG94011, &UA60342},
	{SG94012, &UA60345},
	{SG94013, &UA60349},
	{SG94014, &UA60353},
	{SG94015, &UA60378},
	{SG94016, &UA60383},
	{SG94017, &UA71856},
	{SG94018, &UA75398},
	{SG94019, &UA630101},
	{SG94020, &UA630539},
	{SG94021, &UA649392},
	{SG94022, &UA680126},
	{SG94023, &UA852551},
	{SG94024, &UA1948411},
	{SG94025, &UA1964460},
	{SG94026, &UA2507725},
	{SG94027, &UA2512815},
	{SG94028, &UA2558575},
	{SG94029, &UA2807809},
	{SG94030, &UA3117500},
	{SG94031, &UA616606},
	{SG94032, &UA2007706},
	{SG94033, &UA2031060},
	{SG94034, &UA2354661},
	{SG94035, &UA2404526},
	{SG94036, &UA2429485},
	{SG94037, &UA2430905},
	{SG94038, &UA710412},
	{SG94039, &UA1966106},
	{SG94040, &UA1995115},
	{SG94041, &UA2007381},
	{SG94042, &UA2172778},
	{SG94043, &UA2480290},
	{SG94044, &UA19441},
	{SG94045, &UA63628},
	{SG94046, &UA2427262},
	{SG94047, &UA2428603},
	{SG94048, &UA2528054},
	{SG94049, &UA2191570},
	{SG94050, &UA2257486},
	{SG94051, &UA2257566},
	{SG94073, &UA369241},
	{SG94074, &UA902854},
	{SG94075, &UA905080},
	{SG94076, &UA1982662},
	{SG94077, &UA1987386},
	{SG94078, &UA1991913},
	{SG94079, &UA2007380},
	{SG94080, &UA2009137},
	{SG94081, &UA2016153},
	{SG94082, &UA2033150},
	{SG94083, &UA2187374},
	{SG94084, &UA2198337},
	{SG94085, &UA9670},
	{SG94086, &UA19154},
	{SG94087, &UA2178440},
	{SG94088, &UA47331},
	{SG94089, &UA68466},
	{SG94090, &UA223130},
	{SG94091, &UA253905},
	{SG94092, &UA625027},
	{SG94093, &UA632014},
	{SG94094, &UA1889213},
	{SG94095, &UA2024216},
	{SG94096, &UA2026057},
	{SG94097, &UA2027869},
	{SG94098, &UA2032577},
	{SG94099, &UA2442199},
	{SG94103, &UA1958746},
	{SG94104, &UA2203290},
	{SG94105, &UA2232424},
	{SG94106, &UA19626},
	{SG94107, &UA19638},
	{SG94108, &UA62540},
	{SG94109, &UA2427737},
	{SG94110, &UA32838},
	{SG94111, &UA47727},
	{SG94112, &UA47726},
	{SG94113, &UA47744},
	{SG94114, &UA47723},
	{SG94115, &UA49576},
	{SG94116, &UA49577},
	{SG94117, &UA49580},
	{SG94118, &UA49584},
	{SG94119, &UA60621},
	{SG94120, &UA70695},
	{SG94121, &UA73132},
	{SG94122, &UA74523},
	{SG94123, &UA254004},
	{SG94124, &UA624956},
	{SG94125, &UA636279},
	{SG94126, &UA658219},
	{SG94127, &UA709218},
	{SG94128, &UA1945022},
	{SG94129, &UA1966662},
	{SG94130, &UA2019667},
	{SG94131, &UA2512119},
	{SG94132, &UA2662882},
	{SG94133, &UA18535},
	{SG94134, &UA33237},
	{SG94135, &UA41909},
	{SG94136, &UA41919},
	{SG94137, &UA57775},
	{SG94138, &UA57791},
	{SG94139, &UA73469},
	{SG94140, &UA73619},
	{SG94141, &UA370236},
	{SG94142, &UA622953},
	{SG94143, &UA625377},
	{SG94144, &UA637929},
	{SG94145, &UA662432},
	{SG94146, &UA728796},
	{SG94147, &UA1990468},
	{SG94148, &UA2196253},
	{SG94149, &UA2205588},
	{SG94150, &UA2000086},
	{SG94151, &UA2353638},
	{SG94152, &UA18477},
	{SG94153, &UA18481},
	{SG94154, &UA18482},
	{SG94155, &UA48353},
	{SG94156, &UA57710},
	{SG94157, &UA57720},
	{SG94159, &UA72434},
	{SG94163, &UA632672},
	{SG94164, &UA680353},
	{SG94165, &UA2025878},
	{SG94167, &UA20381},
	{SG94168, &UA46935},
	{SG94169, &UA46946},
	{SG94170, &UA49747},
	{SG94171, &UA60059},
	{SG94172, &UA60068},
	{SG94173, &UA73564},
	{SG94174, &UA632830},
	{SG94175, &UA704714},
	{SG94176, &UA706033},
	{SG94177, &UA1939312},
	{SG94178, &UA2024036},
	{SG94179, &UA2175184},
	{SG94180, &UA2257613},
	{SG94181, &UA2551536},
	{SG94183, &UA877313},
	{SG94184, &UA899930},
	{SG94185, &UA1993874},
	{SG94186, &UA1994183},
	{SG94196, &UA2035269},
	{SG94197, &UA2425527},
	{SG94198, &UA33226},
	{SG94199, &UA43538},
	{SG94200, &UA59526},
	{SG94201, &UA74703},
	{SG94202, &UA626178},
	{SG94203, &UA33402},
	{SG94204, &UA54767},
	{SG94205, &UA119857},
	{SG94206, &UA627378},
	{SG94207, &UA659892},
	{SG94208, &UA701239},
	{SG94209, &UA893480},
	{SG94210, &UA895587},
	{SG94211, &UA2029073},
	{SG94212, &UA2010613},
	{SG94213, &UA2029321},
	{SG94214, &UA30516},
	{SG94215, &UA30527},
	{SG94216, &UA62023},
	{SG94217, &UA67605},
	{SG94218, &UA48447},
	{SG94219, &UA49979},
	{SG94220, &UA356332},
	{SG94221, &UA758694},
	{SG94222, &UA804215},
	{SG94225, &UA15081},
	{SG94226, &UA15085},
	{SG94227, &UA15128},
	{SG94228, &UA49082},
	{SG94229, &UA253878},
	{SG94230, &UA31680},
	{SG94231, &UA54849},
	{SG94232, &UA49274},
	{SG94233, &UA8994},
	{SG94234, &UA63721},
	{SG94235, &UA63756},
	{SG94236, &UA64616},
	{SG94244, &UA2030534},
	{SG94245, &UA2166821},
	{SG94246, &UA2257425},
	{SG94247, &UA47701},
	{SG94248, &UA70732},
	{SG94249, &UA15657},
	{SG94250, &UA15656},
	{SG94251, &UA63857},
	{SG94252, &UA20818},
	{SG94253, &UA20821},
	{SG94254, &UA43145},
	{SG94255, &UA43152},
	{SG94256, &UA59104},
	{SG94257, &UA90763},
	{SG94258, &UA253241},
	{SG94259, &UA453704},
	{SG94260, &UA2004712},
	{SG94261, &UA2206195},
	{SG94262, &UA43526},
	{SG94263, &UA43529},
	{SG94264, &UA59507},
	{SG94265, &UA576019},
	{SG94266, &UA626440},
	{SG94267, &UA2018018},
	{SG94268, &UA2019459},
	{SG94269, &UA49000},
	{SG94270, &UA63062},
	{SG94271, &UA64742},
	{SG94273, &UA20759},
	{SG94274, &UA59009},
	{SG94275, &UA59063},
	{SG94276, &UA73055},
	{SG94277, &UA77396},
	{SG94278, &UA628619},
	{SG94279, &UA2386780},
	{SG94280, &UA2559356},
	{SG94281, &UA2996534},
	{SG94283, &UA46823},
	{SG94284, &UA46834},
	{SG94285, &UA59932},
	{SG94286, &UA59934},
	{SG94287, &UA70593},
	{SG94288, &UA253559},
	{SG94289, &UA2256919},
	{SG94290, &UA2571969},
	{SG94291, &UA20209},
	{SG94292, &UA43365},
	{SG94293, &UA43370},
	{SG94294, &UA59287},
	{SG94295, &UA60641},
	{SG94296, &UA78736},
	{SG94297, &UA347376},
	{SG94298, &UA683479},
	{SG94299, &UA2373860},
	{SG94306, &UA6900},
	{SG94307, &UA627144},
	{SG94308, &UA628251},
	{SG94309, &UA862995},
	{SG94310, &UA2374541},
	{SG94312, &UA60287},
	{SG94313, &UA63507},
	{SG94314, &UA64550},
	{SG94316, &UA44248},
	{SG94318, &UA2511236},
	{SG94319, &UA7787},
	{SG94320, &UA21951},
	{SG94321, &UA55164},
	{SG94322, &UA61982},
	{SG94323, &UA2389856},
	{SG94324, &UA64274},
	{SG94332, &UA58328},
	{SG94333, &UA63963},
	{SG94335, &UA9405},
	{SG94336, &UA64503},
	{SG94337, &UA50976},
	{SG94338, &UA53978},
	{SG94339, &UA64671},
	{SG94343, &UA70866},
	{SG94344, &UA396509},
	{SG94345, &UA626531},
	{SG94346, &UA2194426},
	{SG94347, &UA64208},
	{SG94348, &UA59308},
	{SG94349, &UA63992},
	{SG94350, &UA7029},
	{SG94351, &UA53990},
	{SG94352, &UA62401},
	{SG94353, &UA24254},
	{SG94354, &UA62919},
	{SG94355, &UA2261943},
	{SG94356, &UA1958937},
	{SG94357, &UA662592},
	{SG94358, &UA64367},
	{SG94359, &UA64798},
	{SG94360, &UA64863},
	{SG94361, &UA596},
	{SG94362, &UA63283},
	{SG94363, &UA6794},
	{SG94364, &UA455972},
	{SG94365, &UA599917},
	{SG94366, &UA599975},
	{SG94367, &UA862471},
	{SG94368, &UA1997677},
	{SG94369, &UA28465},
	{SG94370, &UA62182},
	{SG94371, &UA33465},
	{SG94372, &UA1568},
	{SG94373, &UA22097},
	{SG94374, &UA2643},
	{SG94375, &UA18831},
	{SG94376, &UA61879},
	{SG94377, &UA38750},
	{SG94378, &UA49189},
	{SG94379, &UA53858},
	{SG94380, &UA53859},
	{SG94381, &UA63526},
	{SG94382, &UA899803},
	{SG94383, &UA64508},
	{SG94386, &UA3571},
	{SG94387, &UA2093},
	{SG94388, &UA24773},
	{SG94389, &UA33248},
	{SG94390, &UA13511},
	{SG94391, &UA54642},
	{SG94392, &UA54816},
	{SG94393, &UA64143},
	{SG94394, &UA54662},
	{SG94395, &UA54663},
	{SG94396, &UA63702},
	{SG94397, &UA2315524},
	{SG94398, &UA60918},
	{SG94399, &UA64797},
	{SG94400, &UA253829},
	{SG94401, &UA1564},
	{SG94402, &UA27981},
	{SG94403, &UA64347},
	{SG94404, &UA55866},
	{SG94405, &UA62725},
	{SG94406, &UA13898},
	{SG94407, &UA25391},
	{SG94408, &UA62589},
	{SG94409, &UA26340},
	{SG94410, &UA64009},
	{SG94411, &UA13433},
	{SG94412, &UA54976},
	{SG94413, &UA64280},
	{SG94414, &UA22006},
	{SG94415, &UA64745},
	{SG94416, &UA64619},
	{SG94417, &UA63073},
	{SG94418, &UA64717},
	{SG94421, &UA16486},
	{SG94422, &UA48988},
	{SG94423, &UA64501},
	{SG94424, &UA55286},
	{SG94426, &UA64004},
	{SG94429, &UA64015},
	{SG94430, &UA63219},
	{SG94431, &UA31593},
	{SG94432, &UA63018},
	{SG94433, &UA1872},
	{SG94434, &UA2404600},
	{SG94435, &UA2531402},
	{SG94436, &UA64691},
	{SG94437, &UA63493},
	{SG94438, &UA62132},
	{SG94439, &UA63722},
	{SG94440, &UA63404},
	{SG94441, &UA2390935},
	{SG94442, &UA63208},
	{SG94443, &UA51284},
	{SG94444, &UA24735},
	{SG94445, &UA64698},
	{SG94446, &UA64324},
	{SG94447, &UA54648},
	{SG94448, &UA64113},
	{SG94450, &UA18861},
	{SG94451, &UA18862},
	{SG94452, &UA62240},
	{SG94453, &UA62585},
	{SG94454, &UA64515},
	{SG94455, &UA21033},
	{SG94456, &UA61645},
	{SG94458, &UA55256},
	{SG94459, &UA423489},
	{SG94460, &UA604844},
	{SG94461, &UA864571},
	{SG94462, &UA2390952},
	{SG94463, &UA27298},
	{SG94464, &UA62525},
	{SG94465, &UA22395},
	{SG94466, &UA64838},
	{SG94467, &UA54625},
	{SG94468, &UA2424668},
	{SG94469, &UA2457244},
	{SG94470, &UA2482566},
	{SG94471, &UA64317},
	{SG94472, &UA53832},
	{SG94473, &UA64362},
	{SG94474, &UA64076},
	{SG94475, &UA10256},
	{SG94476, &UA63868},
	{SG94477, &UA2883},
	{SG94478, &UA2884},
	{SG94479, &UA22220},
	{SG94480, &UA40010},
	{SG94481, &UA18551},
	{SG94482, &UA53635},
	{SG94483, &UA55768},
	{SG94485, &UA17397},
	{SG94486, &UA43830},
	{SG94487, &UA61993},
	{SG94488, &UA63651},
	{SG94490, &UA63017},
	{SG94491, &UA64521},
	{SG94492, &UA7209},
	{SG94493, &UA2563202},
	{SG94494, &UA64886},
	{SG94495, &UA60951},
	{SG94496, &UA54709},
	{SG94497, &UA30927},
	{SG94498, &UA61664},
	{SG94499, &UA64506},
	{SG94500, &UA61745},
	{SG94501, &UA63880},
	{SG94502, &UA2581080},
	{SG94503, &UA2850351},
	{SG94505, &UA63692},
	{SG94506, &UA3031364},
	{SG94507, &UA62147},
	{SG94508, &UA2422725},
	{SG94509, &UA2560872},
	{SG94511, &UA14020},
	{SG94512, &UA62560},
	{SG94513, &UA64251},
	{SG94514, &UA77034},
	{SG94515, &UA2423708},
	{SG94516, &UA2963506},
	{SG94517, &UA22455},
	{SG94518, &UA64542},
	{SG94519, &UA7867},
	{SG94520, &UA21755},
	{SG94521, &UA27457},
	{SG94522, &UA62493},
	{SG94524, &UA19429},
	{SG94525, &UA2562721},
	{SG94526, &UA2596554},
	{SG94527, &UA2430723},
	{SG94528, &UA24298},
	{SG94533, &UA2236231},
	{SG94534, &UA2737442},
	{SG94536, &UA27264},
	{SG94537, &UA36743},
	{SG94538, &UA54684},
	{SG94539, &UA421844},
	{SG94540, &UA603757},
	{SG94541, &UA660917},
	{SG94547, &UA2422379},
	{SG94548, &UA29010},
	{SG94549, &UA50162},
	{SG94550, &UA665438},
	{SG94551, &UA669126},
	{SG94552, &UA2387324},
	{SG94553, &UA62794},
	{SG94554, &UA61700},
	{SG94559, &UA2385723},
	{SG94562, &UA46929},
	{SG94563, &UA60591},
	{SG94564, &UA64474},
	{SG94565, &UA2469308},
	{SG94566, &UA3019683},
	{SG94568, &UA2400862},
	{SG94576, &UA9021},
	{SG94578, &UA2577484},
	{SG94589, &UA2481564},
	{SG94592, &UA2340724},
	{SG94593, &UA63804},
	{SG94594, &UA2366325},
	{SG94595, &UA2698844},
	{SG94605, &UA2659648},
	{SG94608, &UA62362},
	{SG94609, &UA13599},
	{SG94613, &UA11269},
	{SG94614, &UA253958},
	{SG94615, &UA23001},
	{SG94616, &UA63764},
	{SG94617, &UA88762},
	{SG94618, &UA2752593},
	{SG94619, &UA656466},
	{SG94620, &UA64576},
	{SG94621, &UA2706560},
	{SG94622, &UA54821},
	{SG94623, &UA21413},
	{SG94624, &UA21451},
	{SG94625, &UA26128},
	{SG94626, &UA572336},
	{SG94627, &UA3030381},
	{SG94628, &UA30981},
	{SG94629, &UA564431},
	{SG94630, &UA2660876},
	{SG94631, &UA2383510},
	{SG94632, &UA2943894},
	{SG94633, &UA2423769},
	{SG94634, &UA59892},
	{SG94635, &UA54091},
	{SG94636, &UA3033261},
	{SG94637, &UA2443965},
	{SG94638, &UA1993151},
	{SG94639, &UA2223490},
	{SG94640, &UA2752486},
	{SG94641, &UA2330488},
	{SG94642, &UA29880},
	{SG94643, &UA19556},
	{SG94644, &UA22429},
	{SG94645, &UA28493},
	{SG94646, &UA253978},
	{SG94647, &UA2729914},
	{SG94648, &UA17126},
	{SG94649, &UA2696153},
	{SG94650, &UA2457931},
	{SG94651, &UA52028},
	{SG94652, &UA572734},
	{SG94653, &UA696384},
	{SG94654, &UA1993734},
	{SG94655, &UA2336931},
	{SG94656, &UA63066},
	{SG94657, &UA2598092},
	{SG94658, &UA677290},
	{SG94659, &UA20192},
	{SG94660, &UA2504777},
	{SG94661, &UA2457774},
	{SG94662, &UA2731902},
	{SG94663, &UA2506703},
	{SG94664, &UA2556157},
	{SG94665, &UA2459603},
	{SG94666, &UA2730110},
	{SG94667, &UA54230},
	{SG94668, &UA1892165},
	{SG94669, &UA2418483},
	{SG94670, &UA3060879},
	{SG94671, &UA62933},
	{SG94672, &UA64499},
	{SG94673, &UA2768687},
	{SG94674, &UA22941},
	{SG94675, &UA60295},
	{SG94676, &UA60899},
	{SG94677, &UA62581},
	{SG94678, &UA62582},
	{SG94679, &UA63415},
	{SG94680, &UA3069053},
	{SG94681, &UA2970816},
	{SG94682, &UA2651132},
	{SG94683, &UA17130},
	{SG94684, &UA3060584},
	{SG94685, &UA398389},
	{SG94686, &UA2579900},
	{SG94687, &UA2656760},
	{SG94688, &UA11586},
	{SG94689, &UA62529},
	{SG94690, &UA2846985},
	{SG94691, &UA2762209},
	{SG94692, &UA2950678},
	{SG94693, &UA3079275},
	{SG94694, &UA50462},
	{SG94695, &UA775874},
	{SG94696, &UA2705688},
	{SG94697, &UA50443},
	{SG94698, &UA2663898},
	{SG94699, &UA3018195},
	{SG94700, &UA25505},
	{SG94701, &UA173015},
	{SG94702, &UA2164130},
	{SG94703, &UA26363},
	{SG94704, &UA43622},
	{SG94705, &UA23726},
	{SG94706, &UA3011277},
	{SG94707, &UA3124115},
	{SG94708, &UA2660337},
	{SG94709, &UA62093},
	{SG94710, &UA2458758},
	{SG94711, &UA22360},
	{SG94712, &UA572022},
	{SG94713, &UA2522522},
	{SG94714, &UA2730384},
	{SG94715, &UA2768407},
	{SG94716, &UA61687},
	{SG94717, &UA62506},
	{SG94718, &UA63587},
	{SG94719, &UA50147},
	{SG94720, &UA51230},
	{SG94721, &UA83706},
	{SG94722, &UA3009062},
	{SG94723, &UA2351442},
	{SG94724, &UA3079487},
	{SG94725, &UA2483028},
	{SG94726, &UA3004848},
	{SG94727, &UA50331},
	{SG94728, &UA2217448},
	{SG94729, &UA806399},
	{SG94730, &UA2697278},
	{SG94731, &UA572647},
	{SG94732, &UA88557},
	{SG94733, &UA59111},
	{SG94734, &UA56066},
	{SG94735, &UA253812},
	{SG94736, &UA2853829},
	{SG94737, &UA83578},
	{SG94738, &UA2165943},
	{SG94739, &UA2654911},
	{SG94740, &UA2599274},
	{SG94741, &UA56781},
	{SG94742, &UA898977},
	{SG94743, &UA2262225},
	{SG94744, &UA253927},
	{SG94745, &UA573333},
	{SG94746, &UA893422},
	{SG94747, &UA1901132},
	{SG94748, &UA16158},
	{SG94749, &UA22434},
	{SG94750, &UA21908},
	{SG94751, &UA1935800},
	{SG94752, &UA727394},
	{SG94753, &UA2851780},
	{SG94754, &UA2993013},
	{SG94755, &UA3093482},
	{SG94756, &UA64202},
	{SG94757, &UA60847},
	{SG94758, &UA1932650},
	{SG94759, &UA3020081},
	{SG94790, &UA1940036},
	{SG94791, &UA2411310},
	{SG94792, &UA2825589},
	{SG94793, &UA2981594},
	{SG94794, &UA9403},
	{SG94795, &UA61605},
	{SG94796, &UA60787},
	{SG94797, &UA50153},
	{SG94798, &UA23171},
	{SG94799, &UA23663},
	{SG94800, &UA23009},
	{SG94801, &UA87468},
	{SG94802, &UA1935626},
	{SG94803, &UA2023642},
	{SG94804, &UA2589835},
	{SG94805, &UA54531},
	{SG94806, &UA87643},
	{SG94807, &UA2336768},
	{SG94808, &UA66998},
	{SG94809, &UA2593358},
	{SG94810, &UA3093604},
	{SG94811, &UA23108},
	{SG94812, &UA44018},
	{SG94813, &UA775690},
	{SG94814, &UA50382},
	{SG94815, &UA61803},
	{SG94816, &UA83421},
	{SG94817, &UA24528},
	{SG94818, &UA2589482},
	{SG94819, &UA2762764},
	{SG94820, &UA2845120},
	{SG94821, &UA20718},
	{SG94822, &UA62927},
	{SG94823, &UA2166546},
	{SG94824, &UA2589303},
	{SG94825, &UA61867},
	{SG94826, &UA51988},
	{SG94827, &UA2857222},
	{SG94828, &UA37999},
	{SG94829, &UA253810},
	{SG94830, &UA11645},
	{SG94831, &UA26863},
	{SG94832, &UA2308944},
	{SG94833, &UA2841812},
	{SG94834, &UA3057435},
	{SG94835, &UA20200},
	{SG94836, &UA54858},
	{SG94837, &UA22087},
	{SG94838, &UA50142},
	{SG94839, &UA2811762},
	{SG94840, &UA50366},
	{SG94841, &UA21028},
	{SG94842, &UA32978},
	{SG94843, &UA2595396},
	{SG94844, &UA22350},
	{SG94845, &UA23002},
	{SG94846, &UA55179},
	{SG94847, &UA23520},
	{SG94848, &UA14358},
	{SG94849, &UA55018},
	{SG94850, &UA54206},
	{SG94851, &UA83583},
	{SG94852, &UA2952344},
	{SG94853, &UA24774},
	{SG94854, &UA54782},
	{SG94855, &UA62491},
	{SG94856, &UA50243},
	{SG94857, &UA648350},
	{SG94858, &UA689119},
	{SG94859, &UA744097},
	{SG94860, &UA2657020},
	{SG94861, &UA50269},
	{SG94862, &UA253863},
	{SG94863, &UA2150973},
	{SG94864, &UA2700140},
	{SG94865, &UA2793149},
	{SG94866, &UA2982002},
	{SG94867, &UA63476},
	{SG94868, &UA79361},
	{SG94869, &UA33115},
	{SG94870, &UA40015},
	{SG94871, &UA55962},
	{SG94872, &UA64608},
	{SG94873, &UA541561},
	{SG94874, &UA1892232},
	{SG94875, &UA2338629},
	{SG94876, &UA2762988},
	{SG94877, &UA62803},
	{SG94878, &UA56009},
	{SG94879, &UA88669},
	{SG94880, &UA709685},
	{SG94881, &UA2455458},
	{SG94882, &UA2861438},
	{SG94883, &UA3010977},
	{SG94884, &UA3070832},
	{SG94885, &UA3108435},
	{SG94886, &UA83403},
	{SG94887, &UA52787},
	{SG94888, &UA695396},
	{SG94889, &UA2656252},
	{SG94890, &UA2979553},
	{SG94891, &UA3016474},
	{SG94892, &UA3056761},
	{SG94893, &UA3099095},
	{SG94894, &UA2370358},
	{SG94895, &UA2579772},
	{SG94896, &UA2729242},
	{SG94897, &UA2778108},
	{SG94898, &UA2951839},
	{SG94899, &UA2997295},
	{SG94900, &UA3017160},
	{SG94901, &UA3044862},
	{SG94902, &UA3070672},
	{SG94903, &UA33635},
	{SG94904, &UA50184},
	{SG94905, &UA13424},
	{SG94906, &UA253903},
	{SG94907, &UA574026},
	{SG94908, &UA579046},
	{SG94909, &UA2338482},
	{SG94910, &UA2370389},
	{SG94911, &UA2393517},
	{SG94912, &UA2722018},
	{SG94913, &UA2723259},
	{SG94914, &UA2805192},
	{SG94915, &UA2832924},
	{SG94916, &UA2847974},
	{SG94917, &UA3118285},
	{SG94918, &UA3125284},
	{SG94919, &UA2358},
	{SG94920, &UA1941701},
	{SG94921, &UA2158214},
	{SG94922, &UA2291165},
	{SG94923, &UA2814549},
	{SG94924, &UA61840},
	{SG94925, &UA706824},
	{SG94926, &UA2300741},
	{SG94927, &UA2309318},
	{SG94928, &UA2844174},
	{SG94929, &UA2844415},
	{SG94930, &UA55136},
	{SG94931, &UA12321},
	{SG94932, &UA45925},
	{SG94933, &UA50174},
	{SG94934, &UA2824630},
	{SG94935, &UA33343},
	{SG94936, &UA64022},
	{SG94937, &UA49992},
	{SG94938, &UA50339},
	{SG94939, &UA51243},
	{SG94940, &UA2279},
	{SG94941, &UA4363},
	{SG94942, &UA5025},
	{SG94943, &UA55284},
	{SG94944, &UA32785},
	{SG94945, &UA83588},
	{SG94946, &UA13900},
	{SG94947, &UA21042},
	{SG94948, &UA69547},
	{SG94949, &UA63089},
	{SG94950, &UA2831680},
	{SG94951, &UA2445684},
	{SG94952, &UA30440},
	{SG94953, &UA61979},
	{SG94954, &UA253951},
	{SG94955, &UA3936},
	{SG94956, &UA21505},
	{SG94957, &UA54031},
	{SG94958, &UA63754},
	{SG94959, &UA40695},
	{SG94960, &UA51312},
	{SG94961, &UA253926},
	{SG94962, &UA554579},
	{SG94963, &UA563569},
	{SG94964, &UA32465},
	{SG94965, &UA55879},
	{SG94966, &UA22310},
	{SG94967, &UA22613},
	{SG94968, &UA63345},
	{SG94969, &UA49824},
	{SG94970, &UA83525},
	{SG94971, &UA57801},
	{SG94972, &UA67889},
	{SG94973, &UA15418},
	{SG94974, &UA26690},
	{SG94975, &UA55586},
	{SG94976, &UA54281},
	{SG94977, &UA56326},
	{SG94978, &UA56112},
	{SG94979, &UA50412},
	{SG94980, &UA51501},
	{SG94981, &UA54568},
	{SG94982, &UA64774},
	{SG94983, &UA2221868},
	{SG94984, &UA2810028},
	{SG94985, &UA22388},
	{SG94986, &UA24381},
	{SG94987, &UA88829},
	{SG94988, &UA86922},
	{SG94989, &UA19342},
	{SG94990, &UA21879},
	{SG94991, &UA35302},
	{SG94992, &UA46103},
	{SG94993, &UA67921},
	{SG94994, &UA50238},
	{SG94995, &UA61392},
	{SG94996, &UA253874},
	{SG94997, &UA253908},
	{SG94998, &UA253957},
	{SG94999, &UA21660},
	{SG95000, &UA54216},
	{SG95001, &UA54209},
	{SG95002, &UA54943},
	{SG95003, &UA83438},
	{SG95004, &UA62686},
	{SG95005, &UA85835},
	{SG95006, &UA253994},
	{SG95007, &UA14534},
	{SG95008, &UA32009},
	{SG95009, &UA34480},
	{SG95010, &UA53553},
	{SG95011, &UA67990},
	{SG95012, &UA83472},
	{SG95013, &UA83585},
	{SG95014, &UA83668},
	{SG95015, &UA83733},
	{SG95016, &UA16754},
	{SG95017, &UA20333},
	{SG95018, &UA22426},
	{SG95019, &UA26044},
	{SG95020, &UA37696},
	{SG95021, &UA48895},
	{SG95022, &UA51254},
	{SG95023, &UA56331},
	{SG95024, &UA67215},
	{SG95025, &UA72529},
	{SG95026, &UA74997},
	{SG95027, &UA83397},
	{SG95028, &UA83410},
	{SG95029, &UA83476},
	{SG95030, &UA83539},
	{SG95031, &UA253828},
	{SG95032, &UA253899},
	{SG95033, &UA253909},
	{SG95034, &UA1942039},
	{SG95035, &UA9990},
	{SG95036, &UA20250},
	{SG95037, &UA23016},
	{SG95038, &UA23172},
	{SG95039, &UA23274},
	{SG95040, &UA29237},
	{SG95041, &UA32719},
	{SG95042, &UA34232},
	{SG95043, &UA36750},
	{SG95044, &UA36977},
	{SG95045, &UA45332},
	{SG95046, &UA49950},
	{SG95047, &UA50009},
	{SG95048, &UA50482},
	{SG95049, &UA50798},
	{SG95050, &UA51588},
	{SG95051, &UA52385},
	{SG95052, &UA55601},
	{SG95053, &UA55693},
	{SG95054, &UA58033},
	{SG95055, &UA60743},
	{SG95056, &UA69155},
	{SG95057, &UA69506},
	{SG95058, &UA83698},
	{SG95059, &UA85838},
	{SG95060, &UA85843},
	{SG95061, &UA86842},
	{SG95062, &UA88782},
	{SG95063, &UA253890},
	{SG95064, &UA253941},
	{SG95065, &UA253987},
	{SG95066, &UA51489},
	{NULL}};
const Device *DSpice[] = {
	&UA2154844,
	&UA902986,
	&UA2758357,
	&UA547990,
	&UA627240,
	&UA632129,
	&UA687721,
	&UA1940506,
	&UA1952906,
	&UA2151854,
	&UA2161677,
	&UA2166399,
	&UA2366001,
	&UA2396424,
	&UA2416918,
	&UA2496411,
	&UA3058027,
	&UA3094133,
	&UA2156004,
	&UA2158435,
	&UA2556558,
	&UA2993762,
	&UA877238,
	&UA2445922,
	&UA2814652,
	&UA49315,
	&UA61409,
	&UA79176,
	&UA336275,
	&UA2350670,
	&UA565937,
	&UA33305,
	&UA2864731,
	&UA2422412,
	&UA2864336,
	&UA2530090,
	&UA2375915,
	&UA8614,
	&UA10159,
	&UA38657,
	&UA44298,
	&UA53918,
	&UA53919,
	&UA53920,
	&UA53921,
	&UA53922,
	&UA53923,
	&UA53924,
	&UA53925,
	&UA53926,
	&UA53927,
	&UA64563,
	&UA2503340,
	&UA23733,
	&UA88639,
	&UA23336,
	&UA2733523,
	&UA2733270,
	&UA24686,
	&UA2751050,
	&UA448274,
	&UA2957040,
	&UA47632,
	&UA33318,
	&UA2853764,
	&UA17947,
	&UA53879,
	&UA2956279,
	&UA3108633,
	&UA24558,
	&UA63805,
	&UA2891446,
	&UA3107727,
	&UA475115,
	&UA2806480,
	&UA3071758,
	&UA87078,
	&UA2128589,
	&UA32915,
	&UA2440439,
	&UA87779,
	&UA23330,
	&UA55206,
	&UA87895,
	&UA88695,
	&UA88898,
	&UA87734,
	&UA88625,
	&UA89002,
	NULL};
const Segmentdata DTianyu[] = {
	{SG94572, &UA62357},
	{SG94760, &UA399181},
	{SG94761, &UA430906},
	{SG94762, &UA443976},
	{SG94763, &UA459948},
	{SG94764, &UA459950},
	{SG94765, &UA461225},
	{SG94766, &UA492887},
	{SG94767, &UA561700},
	{SG94768, &UA629386},
	{SG94769, &UA629606},
	{SG94770, &UA631739},
	{SG94771, &UA633081},
	{SG94772, &UA635832},
	{SG94773, &UA640522},
	{SG94774, &UA641484},
	{SG94775, &UA642771},
	{SG94776, &UA644363},
	{SG94777, &UA648399},
	{SG94778, &UA651406},
	{SG94779, &UA654819},
	{SG94780, &UA1859990},
	{SG94781, &UA2125528},
	{SG94782, &UA2263820},
	{SG94783, &UA2264019},
	{SG94784, &UA2264151},
	{SG94785, &UA2265264},
	{SG94786, &UA2267030},
	{SG94787, &UA2267646},
	{SG94788, &UA2439250},
	{SG94789, &UA2493062},
	{SG95067, &UA91545},
	{SG95068, &UA101935},
	{SG95069, &UA101100},
	{SG95070, &UA124205},
	{SG95071, &UA108124},
	{SG95072, &UA134571},
	{SG95073, &UA105667},
	{SG95074, &UA121230},
	{SG95078, &UA137992},
	{SG95079, &UA101652},
	{SG95080, &UA126341},
	{SG95081, &UA126379},
	{SG95082, &UA100180},
	{NULL}};
const Device *DVodafone[] = {
	&UA90177,
	&UA90509,
	&UA163362,
	&UA164326,
	&UA170044,
	&UA173834,
	&UA408474,
	&UA517892,
	&UA2639392,
	&UA81575,
	&UA88774,
	&UA88785,
	&UA94082,
	&UA442783,
	&UA2263267,
	&UA151245,
	&UA163359,
	&UA302196,
	&UA315372,
	&UA103328,
	&UA131532,
	&UA167158,
	&UA275795,
	&UA436491,
	&UA494971,
	&UA652247,
	&UA1845120,
	&UA2275937,
	&UA88841,
	&UA86263,
	&UA88092,
	&UA88233,
	&UA88629,
	&UA101596,
	&UA123887,
	&UA124872,
	&UA287581,
	&UA89141,
	&UA100984,
	&UA116898,
	&UA128468,
	&UA135029,
	&UA171711,
	&UA171963,
	&UA376983,
	&UA433048,
	&UA443071,
	&UA720494,
	&UA809289,
	&UA1861002,
	&UA103683,
	&UA371651,
	&UA2263927,
	&UA88463,
	&UA93301,
	&UA125192,
	&UA131540,
	&UA2807643,
	&UA89029,
	&UA114333,
	&UA273317,
	&UA295261,
	&UA1839935,
	&UA89099,
	&UA466813,
	&UA113579,
	&UA141434,
	&UA459924,
	&UA87403,
	&UA2977342,
	&UA89047,
	&UA281454,
	&UA477154,
	&UA499971,
	&UA88483,
	&UA637878,
	&UA2134767,
	&UA262616,
	&UA130861,
	&UA90153,
	&UA146093,
	&UA178549,
	&UA494705,
	&UA511281,
	&UA2262600,
	&UA515144,
	&UA702361,
	&UA1842205,
	&UA87079,
	&UA88894,
	&UA98987,
	&UA261802,
	&UA639541,
	&UA860642,
	&UA2127844,
	&UA710961,
	&UA760500,
	&UA2126386,
	&UA2270245,
	&UA88884,
	&UA778332,
	&UA89862,
	&UA2262914,
	&UA89100,
	&UA554526,
	&UA838726,
	&UA273120,
	&UA87314,
	&UA87047,
	&UA80358,
	&UA86938,
	&UA318049,
	&UA10003,
	&UA10004,
	&UA10005,
	&UA86898,
	&UA87886,
	&UA81736,
	&UA784964,
	&UA1866606,
	&UA86943,
	&UA460139,
	&UA86942,
	&UA21211,
	&UA86895,
	&UA86485,
	&UA89071,
	&UA86493,
	&UA87405,
	NULL};
const Device *DToshiba[] = {
	&UA2977347,
	&UA8585,
	&UA10075,
	&UA16278,
	&UA45233,
	&UA89116,
	&UA10073,
	&UA10076,
	&UA10074,
	&UA3039672,
	&UA3039677,
	&UA3039691,
	&UA3039692,
	&UA3039694,
	&UA3039704,
	&UA3039719,
	&UA3039730,
	&UA3039753,
	&UA3039761,
	&UA925841,
	&UA5202,
	&UA28145,
	&UA36894,
	&UA45777,
	&UA45778,
	&UA59267,
	&UA3039675,
	&UA3039676,
	&UA3039681,
	&UA3039688,
	&UA3039690,
	&UA3039705,
	&UA3039706,
	&UA3039679,
	&UA3039733,
	&UA3039735,
	&UA3039740,
	&UA3039759,
	&UA3039760,
	&UA3039682,
	&UA3039701,
	&UA3039702,
	&UA3039711,
	&UA3039716,
	&UA3039717,
	&UA3039720,
	&UA3039724,
	&UA3039727,
	&UA3039732,
	&UA3039742,
	&UA3039710,
	&UA3039755,
	&UA3039693,
	&UA3039695,
	&UA3039696,
	&UA3039697,
	&UA3039707,
	&UA3039708,
	&UA3039709,
	&UA3039714,
	&UA3039715,
	&UA3039721,
	&UA3039741,
	&UA3039744,
	&UA3039748,
	&UA3039678,
	&UA3039683,
	&UA3039699,
	&UA3039700,
	&UA3039712,
	&UA3039734,
	&UA3039736,
	&UA3039750,
	&UA3039752,
	&UA3039754,
	&UA3039756,
	&UA3039757,
	&UA3039674,
	&UA3039684,
	&UA3039722,
	&UA3039731,
	&UA3039737,
	&UA3039738,
	&UA3039746,
	&UA3039758,
	&UA3039703,
	&UA3039723,
	&UA3039725,
	&UA3039743,
	&UA3039747,
	&UA3039749,
	&UA3039713,
	&UA3039728,
	&UA3039729,
	&UA3039739,
	&UA3039745,
	&UA3039762,
	&UA925863,
	&UA3039680,
	&UA3039686,
	&UA3039698,
	&UA3039718,
	&UA3039751,
	&UA3039685,
	&UA3039689,
	&UA3039726,
	&UA59268,
	&UA3039687,
	NULL};
const Device *DWebOS[] = {
	&UA238811,
	&UA239195,
	&UA327586,
	&UA352126,
	&UA359505,
	&UA363613,
	&UA89072,
	&UA89706,
	&UA100825,
	&UA125552,
	&UA130269,
	&UA141816,
	&UA145271,
	&UA202354,
	&UA207089,
	&UA262529,
	&UA281714,
	&UA284887,
	&UA286740,
	&UA506986,
	&UA748310,
	&UA811062,
	&UA2285966,
	&UA2299571,
	&UA88475,
	&UA88713,
	&UA88965,
	&UA89113,
	&UA89271,
	&UA89868,
	&UA90016,
	&UA114228,
	&UA120926,
	&UA122238,
	&UA125954,
	&UA136040,
	&UA140286,
	&UA265558,
	&UA269578,
	&UA269614,
	&UA305755,
	&UA459530,
	&UA459979,
	&UA521006,
	&UA525553,
	&UA548799,
	&UA1851424,
	&UA1895309,
	&UA2258631,
	&UA2264294,
	&UA2270532,
	&UA2339364,
	&UA2801515,
	&UA378515,
	&UA408936,
	&UA459726,
	&UA469912,
	&UA659143,
	&UA717794,
	&UA720758,
	&UA742493,
	&UA770332,
	&UA793683,
	&UA812146,
	&UA1839978,
	&UA1866480,
	&UA1899190,
	&UA2439433,
	NULL};
const Device *DW3CValidator[] = {
	&UA116217,
	&UA140199,
	&UA171632,
	&UA428867,
	&UA2585292,
	NULL};
const Segmentdata DWindowsCE[] = {
	{SG95075, &UA22736},
	{SG95076, &UA80309},
	{SG95077, &UA132948},
	{SG95083, &UA4251},
	{SG95084, &UA8640},
	{SG95085, &UA24318},
	{SG95086, &UA39916},
	{SG95087, &UA41765},
	{SG95088, &UA54075},
	{SG95089, &UA54093},
	{SG95090, &UA57568},
	{SG95091, &UA59517},
	{SG95092, &UA80314},
	{SG95093, &UA80319},
	{SG95094, &UA80325},
	{SG95095, &UA80317},
	{SG95096, &UA80370},
	{SG95097, &UA80371},
	{SG95098, &UA80397},
	{SG95099, &UA80401},
	{SG95100, &UA80404},
	{SG95101, &UA80405},
	{SG95102, &UA80407},
	{SG95103, &UA85184},
	{SG95104, &UA86352},
	{SG95105, &UA86568},
	{SG95106, &UA87050},
	{SG95107, &UA87362},
	{SG95108, &UA87363},
	{SG95109, &UA87902},
	{SG95110, &UA91758},
	{SG95111, &UA101837},
	{SG95112, &UA118170},
	{SG95113, &UA3561},
	{SG95114, &UA5802},
	{SG95115, &UA13452},
	{SG95116, &UA13468},
	{SG95117, &UA21132},
	{SG95118, &UA21194},
	{SG95119, &UA38514},
	{SG95120, &UA42969},
	{SG95121, &UA46511},
	{SG95122, &UA69555},
	{SG95123, &UA93302},
	{SG95124, &UA94208},
	{SG95125, &UA99607},
	{SG95126, &UA101697},
	{SG95127, &UA134444},
	{SG95128, &UA273788},
	{SG95129, &UA277849},
	{SG95130, &UA23496},
	{SG95131, &UA27301},
	{SG95132, &UA62172},
	{SG95133, &UA5030},
	{SG95134, &UA7065},
	{SG95135, &UA21129},
	{SG95136, &UA25397},
	{SG95137, &UA512519},
	{SG95138, &UA36508},
	{SG95139, &UA45073},
	{SG95140, &UA75288},
	{SG95141, &UA2435},
	{SG95142, &UA3948},
	{SG95143, &UA3950},
	{SG95144, &UA3949},
	{SG95145, &UA7320},
	{SG95146, &UA54003},
	{SG95147, &UA73156},
	{SG95148, &UA86326},
	{SG95149, &UA87477},
	{SG95150, &UA87912},
	{SG95151, &UA161513},
	{SG95152, &UA8315},
	{SG95153, &UA34546},
	{SG95154, &UA13454},
	{SG95155, &UA30383},
	{SG95156, &UA34837},
	{SG95157, &UA40635},
	{SG95158, &UA40704},
	{SG95159, &UA53198},
	{SG95160, &UA60992},
	{SG95161, &UA62524},
	{SG95162, &UA87844},
	{SG95163, &UA88515},
	{SG95164, &UA89821},
	{SG95165, &UA111701},
	{SG95166, &UA114359},
	{SG95167, &UA132143},
	{SG95168, &UA4150},
	{SG95169, &UA7876},
	{SG95170, &UA7878},
	{SG95171, &UA7880},
	{SG95172, &UA7877},
	{SG95173, &UA7918},
	{SG95174, &UA7924},
	{SG95175, &UA7925},
	{SG95176, &UA7927},
	{SG95177, &UA7928},
	{SG95178, &UA7929},
	{SG95179, &UA27719},
	{SG95180, &UA27720},
	{SG95181, &UA27724},
	{SG95182, &UA34241},
	{SG95183, &UA34244},
	{SG95184, &UA34245},
	{SG95185, &UA34247},
	{SG95186, &UA65979},
	{SG95187, &UA65982},
	{SG95188, &UA75133},
	{SG95189, &UA87098},
	{SG95190, &UA20426},
	{SG95191, &UA21377},
	{SG95192, &UA50778},
	{SG95193, &UA69544},
	{SG95194, &UA87102},
	{SG95195, &UA87407},
	{SG95196, &UA87628},
	{SG95197, &UA88420},
	{SG95198, &UA265670},
	{SG95199, &UA508746},
	{SG95200, &UA3020534},
	{SG95201, &UA34592},
	{SG95202, &UA28158},
	{SG95206, &UA22881},
	{SG95207, &UA36225},
	{SG95209, &UA897},
	{SG95210, &UA898},
	{SG95211, &UA16326},
	{SG95212, &UA34475},
	{SG95213, &UA34467},
	{SG95214, &UA76868},
	{SG95215, &UA8041},
	{SG95216, &UA8043},
	{SG95217, &UA8042},
	{SG95218, &UA18552},
	{SG95219, &UA57957},
	{SG95220, &UA59516},
	{SG95221, &UA63904},
	{SG95222, &UA64007},
	{SG95223, &UA80328},
	{SG95224, &UA80360},
	{SG95225, &UA87396},
	{SG95226, &UA87900},
	{SG95227, &UA481600},
	{SG95228, &UA490759},
	{SG95229, &UA587372},
	{SG95230, &UA709290},
	{SG95231, &UA772414},
	{SG95232, &UA878478},
	{SG95233, &UA886027},
	{SG95234, &UA897127},
	{SG95235, &UA2211721},
	{SG95236, &UA2297889},
	{SG95237, &UA86927},
	{SG95238, &UA121524},
	{SG95239, &UA275592},
	{SG95240, &UA30653},
	{SG95241, &UA54469},
	{SG95242, &UA13427},
	{SG95243, &UA13426},
	{SG95244, &UA13431},
	{SG95245, &UA13690},
	{SG95246, &UA34829},
	{SG95247, &UA27375},
	{SG95248, &UA6367},
	{SG95249, &UA32024},
	{SG95250, &UA72242},
	{SG95251, &UA14230},
	{SG95252, &UA20506},
	{SG95253, &UA49500},
	{SG95254, &UA56680},
	{SG95255, &UA57743},
	{SG95256, &UA87049},
	{SG95257, &UA167619},
	{SG95258, &UA16315},
	{SG95289, &UA36825},
	{SG95290, &UA571196},
	{SG95291, &UA2295281},
	{SG95292, &UA2370330},
	{SG95293, &UA2127579},
	{SG95294, &UA3649},
	{SG95295, &UA5049},
	{SG95296, &UA7541},
	{SG95297, &UA18470},
	{SG95298, &UA30906},
	{SG95308, &UA86926},
	{SG95309, &UA87458},
	{SG95310, &UA165002},
	{SG95311, &UA18524},
	{SG95312, &UA869473},
	{SG95313, &UA2989891},
	{SG95314, &UA3073017},
	{SG95315, &UA539515},
	{SG95316, &UA2462657},
	{SG95317, &UA4372},
	{SG95318, &UA2478},
	{SG95319, &UA24789},
	{SG95320, &UA35932},
	{SG95321, &UA39917},
	{SG95322, &UA62876},
	{SG95323, &UA64907},
	{SG95324, &UA71993},
	{SG95325, &UA76405},
	{SG95326, &UA86528},
	{SG95327, &UA86986},
	{SG95328, &UA87367},
	{SG95329, &UA6006},
	{SG95330, &UA105415},
	{SG95332, &UA15717},
	{SG95333, &UA15720},
	{SG95334, &UA15724},
	{SG95335, &UA15722},
	{SG95336, &UA18556},
	{SG95337, &UA18557},
	{SG95338, &UA121632},
	{SG95339, &UA22901},
	{SG95340, &UA415682},
	{SG95341, &UA863713},
	{SG95342, &UA872483},
	{SG95343, &UA882050},
	{SG95344, &UA1857303},
	{SG95345, &UA2516797},
	{SG95346, &UA2785760},
	{SG95347, &UA3097377},
	{SG95348, &UA223920},
	{SG95349, &UA21396},
	{SG95350, &UA20519},
	{SG95351, &UA49247},
	{SG95352, &UA86859},
	{SG95353, &UA87101},
	{SG95391, &UA16960},
	{SG95392, &UA26646},
	{SG95393, &UA27300},
	{SG95394, &UA33636},
	{SG95395, &UA69633},
	{SG95396, &UA73736},
	{SG95397, &UA86987},
	{SG95398, &UA87685},
	{SG95399, &UA87748},
	{SG95400, &UA284828},
	{SG95401, &UA8585},
	{SG95402, &UA10075},
	{SG95403, &UA16278},
	{SG95404, &UA45233},
	{SG95405, &UA89116},
	{SG95477, &UA4254},
	{SG95478, &UA9813},
	{SG95479, &UA21651},
	{SG95480, &UA21824},
	{SG95481, &UA22313},
	{SG95482, &UA38661},
	{SG95483, &UA57958},
	{SG95484, &UA87372},
	{SG95485, &UA487801},
	{SG95486, &UA54182},
	{SG95487, &UA61077},
	{SG95488, &UA61646},
	{SG95489, &UA23779},
	{SG95490, &UA51434},
	{SG95491, &UA75837},
	{SG95492, &UA2216520},
	{SG95493, &UA2410586},
	{SG95494, &UA2545392},
	{SG95495, &UA2775486},
	{SG95496, &UA3070894},
	{SG95497, &UA3084963},
	{SG95499, &UA54414},
	{SG95501, &UA269517},
	{SG95502, &UA444202},
	{SG95503, &UA468454},
	{SG95504, &UA838466},
	{SG95505, &UA2242404},
	{SG95506, &UA84},
	{SG95507, &UA6007},
	{SG95508, &UA6368},
	{SG95509, &UA4257},
	{SG95510, &UA22620},
	{SG95511, &UA86903},
	{SG95512, &UA2268296},
	{SG95513, &UA87384},
	{SG95514, &UA14360},
	{SG95515, &UA54290},
	{SG95519, &UA433374},
	{SG95520, &UA502559},
	{SG95521, &UA675318},
	{SG95545, &UA4369},
	{SG95569, &UA50997},
	{SG95596, &UA87592},
	{SG95597, &UA119280},
	{SG95598, &UA5340},
	{SG95599, &UA36973},
	{SG95600, &UA45955},
	{SG95662, &UA20274},
	{SG95663, &UA58674},
	{SG95671, &UA174919},
	{SG95675, &UA15621},
	{SG95676, &UA27939},
	{SG95677, &UA27942},
	{SG95678, &UA2337428},
	{SG95679, &UA2952532},
	{SG95682, &UA2242591},
	{SG95683, &UA2338306},
	{SG95686, &UA11250},
	{SG95687, &UA49305},
	{SG95689, &UA21034},
	{SG95690, &UA21035},
	{SG95691, &UA29380},
	{SG95692, &UA33031},
	{SG95693, &UA46101},
	{SG95694, &UA46100},
	{SG95695, &UA59158},
	{SG95696, &UA87674},
	{SG95700, &UA23658},
	{SG95701, &UA2588346},
	{SG95705, &UA8037},
	{SG95706, &UA147398},
	{SG95707, &UA24571},
	{SG95708, &UA26647},
	{SG95709, &UA61659},
	{SG95710, &UA88425},
	{SG95712, &UA43549},
	{SG95713, &UA9276},
	{SG95714, &UA88376},
	{SG95715, &UA21027},
	{SG95716, &UA292576},
	{SG95717, &UA18555},
	{SG95718, &UA10239},
	{SG95719, &UA69564},
	{SG95720, &UA13241},
	{SG95721, &UA21827},
	{SG95723, &UA5202},
	{SG95724, &UA28145},
	{SG95725, &UA36894},
	{SG95726, &UA45777},
	{SG95727, &UA45778},
	{SG95728, &UA22024},
	{SG95731, &UA13240},
	{SG95732, &UA18553},
	{SG95733, &UA65999},
	{SG95734, &UA43632},
	{SG95735, &UA888761},
	{SG95736, &UA26640},
	{SG95737, &UA86533},
	{SG95738, &UA491605},
	{SG95739, &UA10323},
	{SG95740, &UA38682},
	{SG95741, &UA52453},
	{SG95742, &UA11643},
	{SG95770, &UA26453},
	{SG95771, &UA176465},
	{SG95772, &UA13038},
	{SG95773, &UA40782},
	{SG95774, &UA40783},
	{SG95775, &UA43801},
	{SG95776, &UA56364},
	{SG95777, &UA4559},
	{SG95778, &UA22397},
	{SG95779, &UA2277390},
	{SG95780, &UA8898},
	{SG95781, &UA51852},
	{SG95782, &UA32821},
	{SG95783, &UA80410},
	{SG95784, &UA80412},
	{SG95785, &UA80413},
	{SG95786, &UA87441},
	{SG95787, &UA4549},
	{SG95788, &UA762727},
	{SG95789, &UA1934298},
	{SG95790, &UA2308654},
	{SG95791, &UA2309748},
	{SG95792, &UA837731},
	{SG95793, &UA1899604},
	{SG95794, &UA11267},
	{SG95795, &UA20689},
	{SG95796, &UA25399},
	{SG95797, &UA449635},
	{SG95798, &UA2779431},
	{SG95799, &UA66000},
	{SG95803, &UA444055},
	{SG95804, &UA44369},
	{SG95805, &UA59515},
	{SG95806, &UA87361},
	{SG95812, &UA21249},
	{SG95813, &UA21248},
	{SG95814, &UA552},
	{SG95815, &UA58491},
	{SG95821, &UA21288},
	{SG95822, &UA708646},
	{SG95823, &UA42613},
	{SG95824, &UA48933},
	{SG95825, &UA56746},
	{SG95826, &UA61120},
	{SG95827, &UA22237},
	{SG95828, &UA32929},
	{SG95829, &UA61282},
	{SG95830, &UA61283},
	{SG95831, &UA14220},
	{SG95832, &UA44374},
	{SG95833, &UA61763},
	{SG95834, &UA20687},
	{SG95835, &UA33108},
	{SG95836, &UA88726},
	{SG95837, &UA5341},
	{SG95838, &UA7635},
	{SG95839, &UA15370},
	{SG95840, &UA19708},
	{SG95841, &UA59267},
	{SG95842, &UA503755},
	{SG95843, &UA21734},
	{SG95844, &UA78600},
	{SG95845, &UA24343},
	{SG95846, &UA565680},
	{SG95849, &UA10341},
	{SG95850, &UA21354},
	{SG95851, &UA87030},
	{SG95852, &UA87451},
	{SG95853, &UA27299},
	{SG95854, &UA32658},
	{SG95855, &UA88733},
	{SG95856, &UA20898},
	{SG95857, &UA60526},
	{SG95860, &UA5142},
	{SG95861, &UA11126},
	{SG95862, &UA39913},
	{SG95863, &UA88987},
	{SG95864, &UA20413},
	{SG95865, &UA43767},
	{SG95866, &UA53508},
	{SG95867, &UA53947},
	{SG95868, &UA87345},
	{SG95869, &UA87346},
	{SG95870, &UA69691},
	{SG95871, &UA87348},
	{SG95872, &UA87368},
	{SG95873, &UA5048},
	{SG95879, &UA21735},
	{SG95880, &UA28435},
	{SG95881, &UA59284},
	{SG95882, &UA61332},
	{SG95886, &UA32762},
	{SG95887, &UA51840},
	{SG95890, &UA7636},
	{SG95891, &UA21733},
	{SG95892, &UA69640},
	{SG95893, &UA87021},
	{SG95894, &UA87444},
	{SG95895, &UA474066},
	{SG95896, &UA40919},
	{SG95897, &UA58673},
	{SG95898, &UA59543},
	{SG95899, &UA61285},
	{SG95900, &UA72413},
	{SG95901, &UA87449},
	{SG95902, &UA28483},
	{SG95903, &UA36974},
	{SG95904, &UA54484},
	{SG95905, &UA59320},
	{SG95906, &UA86990},
	{SG95907, &UA87359},
	{SG95908, &UA5951},
	{SG95909, &UA20275},
	{SG95910, &UA28484},
	{SG95911, &UA30103},
	{SG95912, &UA44372},
	{SG95913, &UA53350},
	{SG95914, &UA60940},
	{SG95915, &UA86978},
	{SG95916, &UA8981},
	{SG95917, &UA14224},
	{SG95918, &UA21276},
	{SG95919, &UA22945},
	{SG95920, &UA32984},
	{SG95921, &UA35150},
	{SG95922, &UA53946},
	{SG95923, &UA56595},
	{SG95924, &UA59268},
	{SG95925, &UA87028},
	{SG95926, &UA88052},
	{SG95929, &UA378},
	{SG95932, &UA15369},
	{SG95933, &UA15371},
	{SG95934, &UA16192},
	{SG95935, &UA16325},
	{SG95936, &UA22629},
	{SG95937, &UA26452},
	{SG95938, &UA27973},
	{SG95939, &UA28408},
	{SG95940, &UA28482},
	{SG95941, &UA31454},
	{SG95942, &UA35936},
	{SG95943, &UA41890},
	{SG95944, &UA43386},
	{SG95945, &UA51894},
	{SG95946, &UA53289},
	{SG95947, &UA60855},
	{SG95948, &UA87347},
	{SG95949, &UA87557},
	{NULL}};
const Device *DWindowsPhone[] = {
	&UA90792,
	&UA90791,
	&UA399133,
	&UA399151,
	&UA399152,
	&UA399163,
	&UA661750,
	&UA681407,
	&UA761163,
	&UA925778,
	&UA925802,
	&UA925854,
	&UA2812109,
	&UA2981938,
	&UA3085032,
	&UA90788,
	&UA90795,
	&UA371366,
	&UA399158,
	&UA399162,
	&UA399175,
	&UA678546,
	&UA761318,
	&UA925855,
	&UA2545942,
	&UA2979432,
	&UA2982202,
	&UA243526,
	&UA247409,
	&UA250779,
	&UA364185,
	&UA625489,
	&UA636518,
	&UA639840,
	&UA667019,
	&UA697810,
	&UA718072,
	&UA925827,
	&UA925866,
	&UA925879,
	&UA925889,
	&UA2418370,
	&UA403345,
	&UA896822,
	&UA898349,
	&UA925750,
	&UA925795,
	&UA925797,
	&UA925799,
	&UA925829,
	&UA925885,
	&UA925924,
	&UA2463809,
	&UA2730651,
	&UA399168,
	&UA896472,
	&UA925777,
	&UA925793,
	&UA2528737,
	&UA2790074,
	&UA197739,
	&UA210654,
	&UA223423,
	&UA640235,
	&UA805702,
	&UA925699,
	&UA925744,
	&UA925816,
	&UA925864,
	&UA2857970,
	&UA68256,
	&UA318681,
	&UA925798,
	&UA925845,
	&UA2765130,
	&UA249906,
	&UA356975,
	&UA421815,
	&UA552133,
	&UA641968,
	&UA732745,
	&UA764392,
	&UA772485,
	&UA774953,
	&UA925733,
	&UA925826,
	&UA925886,
	&UA925905,
	&UA925906,
	&UA2768512,
	&UA207329,
	&UA217497,
	&UA224857,
	&UA228764,
	&UA658511,
	&UA925696,
	&UA925703,
	&UA925918,
	&UA2035302,
	&UA2043790,
	&UA2401664,
	&UA2404374,
	&UA2428559,
	&UA2652832,
	&UA2698357,
	&UA3080867,
	&UA573204,
	&UA2353216,
	&UA2366446,
	&UA2503814,
	&UA2582438,
	&UA2599797,
	&UA2717115,
	&UA2861276,
	&UA2862429,
	&UA3048740,
	&UA3062755,
	&UA3071625,
	&UA3100306,
	&UA3119038,
	&UA214586,
	&UA216202,
	&UA246244,
	&UA221412,
	&UA223352,
	&UA225270,
	&UA371364,
	&UA925708,
	&UA925871,
	&UA203945,
	&UA66962,
	&UA67916,
	&UA90381,
	&UA848283,
	&UA925715,
	&UA2765129,
	&UA68916,
	&UA68975,
	&UA925878,
	&UA2721989,
	&UA458641,
	&UA782831,
	&UA222009,
	&UA234957,
	&UA630711,
	&UA798417,
	&UA925707,
	&UA925853,
	&UA876531,
	&UA215310,
	&UA925782,
	&UA2362366,
	&UA2423711,
	&UA2467071,
	&UA2656203,
	&UA2984884,
	&UA3067001,
	&UA403341,
	&UA641392,
	&UA757580,
	&UA56098,
	&UA56340,
	&UA184133,
	&UA847029,
	&UA925729,
	&UA2857969,
	&UA2240798,
	&UA2206719,
	&UA2482601,
	&UA2582575,
	&UA196549,
	&UA2413148,
	&UA371357,
	&UA391983,
	&UA756611,
	&UA878608,
	&UA925745,
	&UA2260438,
	&UA2384416,
	&UA2645784,
	&UA2651657,
	&UA2693900,
	&UA2727950,
	&UA2746297,
	&UA3000417,
	&UA3021839,
	&UA3100983,
	&UA197608,
	&UA628979,
	&UA815418,
	&UA925787,
	&UA925896,
	&UA2667263,
	&UA2707834,
	&UA2751762,
	&UA2751784,
	&UA2956871,
	&UA3088357,
	&UA3109769,
	&UA175720,
	&UA238201,
	&UA248552,
	&UA809148,
	&UA2428391,
	&UA2506235,
	&UA2507102,
	&UA2530106,
	&UA2557590,
	&UA2558370,
	&UA2597152,
	&UA2670643,
	&UA2734535,
	&UA3058940,
	&UA3072534,
	&UA3079664,
	&UA55988,
	&UA56803,
	&UA67650,
	&UA232893,
	&UA925922,
	&UA2366703,
	&UA2489642,
	&UA2558007,
	&UA2664712,
	&UA2995052,
	&UA56073,
	&UA66948,
	&UA66952,
	&UA66953,
	&UA925756,
	&UA925875,
	&UA2367530,
	&UA68842,
	&UA56150,
	&UA56175,
	&UA925805,
	&UA925825,
	&UA925848,
	&UA925791,
	&UA925869,
	&UA2415238,
	&UA2469314,
	&UA56012,
	&UA56099,
	&UA90317,
	&UA175587,
	&UA2741794,
	&UA2454874,
	&UA2455523,
	&UA2035303,
	&UA2427237,
	&UA2503839,
	&UA2597433,
	&UA2707941,
	&UA2848906,
	&UA56092,
	&UA56769,
	&UA67242,
	&UA90514,
	&UA67205,
	&UA2741792,
	&UA56058,
	&UA925785,
	&UA925847,
	&UA2670995,
	&UA2781352,
	&UA2936373,
	&UA2870146,
	&UA2900423,
	&UA3019222,
	&UA3080876,
	&UA396306,
	&UA2363132,
	&UA2382122,
	&UA2666082,
	&UA2698422,
	&UA2794043,
	&UA3126000,
	&UA67751,
	&UA430161,
	&UA199603,
	&UA310522,
	&UA2361275,
	&UA2527743,
	&UA2528549,
	&UA3108795,
	&UA3119498,
	&UA2526982,
	&UA67740,
	&UA198208,
	&UA675919,
	&UA687796,
	&UA2043761,
	&UA56056,
	&UA399135,
	&UA399139,
	&UA399140,
	&UA399167,
	&UA399134,
	&UA399172,
	&UA399164,
	&UA2672847,
	&UA2737052,
	&UA2865027,
	&UA3087884,
	&UA399150,
	&UA399174,
	&UA925808,
	&UA925812,
	&UA654777,
	&UA2428032,
	&UA2696601,
	&UA2729797,
	&UA2748689,
	&UA2751070,
	&UA2764763,
	&UA2985146,
	&UA659575,
	&UA153304,
	&UA2602612,
	&UA3050340,
	&UA2240799,
	&UA2506027,
	&UA2597471,
	&UA2792642,
	&UA69507,
	&UA925700,
	&UA925730,
	&UA925768,
	&UA925807,
	&UA2750492,
	&UA925742,
	&UA399157,
	&UA2366414,
	&UA90302,
	&UA850779,
	&UA2765094,
	&UA331243,
	&UA2467859,
	&UA2479968,
	&UA68096,
	&UA648049,
	&UA2496331,
	&UA399147,
	&UA399153,
	&UA399166,
	&UA399169,
	&UA399170,
	&UA925851,
	&UA925850,
	&UA925882,
	&UA2879288,
	&UA2941025,
	&UA3093382,
	&UA3107056,
	&UA475806,
	&UA925870,
	&UA2998614,
	&UA3022821,
	&UA399143,
	&UA399159,
	&UA925728,
	&UA925736,
	&UA925837,
	&UA925860,
	&UA2685276,
	&UA69015,
	&UA2385571,
	&UA2710956,
	&UA2994966,
	&UA2731646,
	&UA371359,
	&UA371361,
	&UA688401,
	&UA765354,
	&UA694932,
	&UA925858,
	&UA853848,
	&UA2857968,
	&UA701131,
	&UA848280,
	&UA2858000,
	&UA847403,
	&UA2857966,
	&UA848524,
	&UA2857965,
	&UA2741795,
	&UA848463,
	&UA2579909,
	&UA630477,
	&UA714740,
	&UA925743,
	&UA925773,
	&UA925776,
	&UA925818,
	&UA925849,
	&UA661582,
	&UA731701,
	&UA2554344,
	&UA241249,
	&UA759380,
	&UA2893044,
	&UA925822,
	&UA2707955,
	&UA626901,
	&UA633584,
	&UA925723,
	&UA731573,
	&UA925811,
	&UA925765,
	&UA925903,
	&UA2499837,
	&UA458642,
	&UA2952033,
	&UA925813,
	&UA2698804,
	&UA90303,
	&UA664334,
	&UA925718,
	&UA925758,
	&UA925909,
	&UA734254,
	&UA925898,
	&UA2413790,
	&UA2695661,
	&UA666254,
	&UA925817,
	&UA2550185,
	&UA68470,
	&UA925714,
	&UA561166,
	&UA925738,
	&UA2728105,
	&UA925781,
	&UA925796,
	&UA52784,
	&UA56336,
	&UA925846,
	&UA925876,
	&UA925897,
	&UA925712,
	&UA925839,
	&UA925895,
	&UA563749,
	&UA925725,
	&UA925873,
	&UA2863813,
	&UA557833,
	&UA925739,
	&UA925874,
	&UA2445725,
	&UA925883,
	&UA2521436,
	&UA56072,
	&UA925912,
	&UA3100607,
	&UA925844,
	&UA925881,
	&UA641362,
	&UA925771,
	&UA925861,
	&UA925887,
	&UA925716,
	&UA925724,
	&UA439776,
	&UA925719,
	&UA925749,
	&UA925832,
	&UA925843,
	&UA925764,
	&UA925783,
	&UA569999,
	&UA925757,
	&UA925836,
	&UA925762,
	&UA925819,
	&UA925859,
	&UA399165,
	&UA925754,
	&UA925763,
	&UA925856,
	&UA925893,
	&UA925713,
	&UA925770,
	&UA2034650,
	&UA2043763,
	NULL};
const Segmentdata DWordPress[] = {
	{SG95406, &UA258765},
	{SG95407, &UA258862},
	{SG95408, &UA258874},
	{SG95409, &UA258903},
	{SG95410, &UA258921},
	{SG95411, &UA259090},
	{SG95412, &UA430021},
	{SG95413, &UA454838},
	{SG95414, &UA512041},
	{SG95415, &UA532232},
	{SG95416, &UA564899},
	{SG95417, &UA581063},
	{SG95418, &UA582803},
	{SG95419, &UA621852},
	{SG95420, &UA666252},
	{SG95421, &UA700764},
	{SG95422, &UA854447},
	{SG95423, &UA1885562},
	{SG95424, &UA2194203},
	{SG95425, &UA2205034},
	{SG95426, &UA2231033},
	{SG95427, &UA2231035},
	{SG95428, &UA2237039},
	{SG95429, &UA2238117},
	{SG95430, &UA2238449},
	{SG95431, &UA2238561},
	{SG95432, &UA2262908},
	{SG95433, &UA2262910},
	{SG95434, &UA2350080},
	{SG95435, &UA2350479},
	{SG95436, &UA2356281},
	{SG95437, &UA2422110},
	{SG95438, &UA2532806},
	{SG95439, &UA2562523},
	{SG95440, &UA2603820},
	{SG95441, &UA2659448},
	{SG95442, &UA2659508},
	{SG95443, &UA2659628},
	{SG95444, &UA2659702},
	{SG95445, &UA2659704},
	{SG95446, &UA2659706},
	{SG95447, &UA2659707},
	{SG95448, &UA2659708},
	{SG95449, &UA2659709},
	{SG95450, &UA2659710},
	{SG95451, &UA2659871},
	{SG95452, &UA2660373},
	{SG95453, &UA2665186},
	{SG95454, &UA2700071},
	{SG95455, &UA2700072},
	{SG95456, &UA2700074},
	{SG95457, &UA2700075},
	{SG95458, &UA2700076},
	{SG95459, &UA2700120},
	{SG95460, &UA2700121},
	{SG95461, &UA2700122},
	{SG95462, &UA2700123},
	{SG95463, &UA2700124},
	{SG95464, &UA2700125},
	{SG95465, &UA2700126},
	{SG95466, &UA2700127},
	{SG95467, &UA2700128},
	{SG95468, &UA2700129},
	{SG95469, &UA2701311},
	{SG95470, &UA2701310},
	{SG95471, &UA2705146},
	{SG95472, &UA2733122},
	{SG95473, &UA2844797},
	{SG95474, &UA2845493},
	{SG95475, &UA2854681},
	{SG95476, &UA2879687},
	{SG95563, &UA2997796},
	{SG95564, &UA3017855},
	{SG95565, &UA3020168},
	{SG95566, &UA3020916},
	{SG95567, &UA3029777},
	{SG95568, &UA3088392},
	{NULL}};
const Device *DZTE[] = {
	&UA844859,
	&UA2497015,
	&UA2550670,
	&UA2686499,
	&UA2687374,
	&UA2781255,
	&UA2950110,
	&UA3048094,
	&UA2747640,
	&UA2764982,
	&UA2790844,
	&UA2990140,
	&UA3107238,
	&UA2009012,
	&UA2433590,
	&UA2955050,
	&UA3050621,
	&UA67169,
	&UA2502831,
	&UA49275,
	&UA49705,
	&UA49907,
	&UA54425,
	&UA2161717,
	&UA2392300,
	&UA2641240,
	&UA2806429,
	&UA61507,
	&UA2217678,
	&UA2301622,
	&UA2577559,
	&UA2640782,
	&UA49462,
	&UA70974,
	&UA651115,
	&UA2022715,
	&UA49463,
	&UA2398256,
	&UA2758731,
	&UA104481,
	&UA2262086,
	&UA2419335,
	&UA2790285,
	&UA8856,
	&UA21877,
	&UA24721,
	&UA63581,
	&UA49904,
	&UA2481112,
	&UA379719,
	&UA49531,
	&UA2363931,
	&UA2954416,
	&UA2520647,
	&UA2726720,
	&UA2978469,
	&UA3084714,
	&UA3109037,
	&UA2418477,
	&UA2764325,
	&UA2181677,
	&UA13665,
	&UA2223882,
	&UA2657062,
	&UA4616,
	&UA4617,
	&UA7374,
	&UA7375,
	&UA7377,
	&UA21646,
	&UA63870,
	&UA64803,
	&UA396301,
	&UA61227,
	&UA30668,
	&UA30669,
	&UA2582913,
	&UA21627,
	&UA24622,
	&UA32930,
	&UA49402,
	&UA64562,
	&UA30351,
	&UA2033781,
	&UA22170,
	&UA64422,
	&UA44003,
	&UA64194,
	&UA20411,
	&UA20414,
	&UA23566,
	&UA33127,
	&UA51079,
	&UA58808,
	&UA60725,
	&UA1951705,
	&UA19958,
	&UA43846,
	&UA2659827,
	&UA14855,
	&UA21122,
	&UA2156324,
	&UA2805883,
	&UA22446,
	&UA24635,
	&UA28486,
	&UA45956,
	&UA64543,
	&UA64744,
	&UA2505419,
	&UA61151,
	&UA2816848,
	&UA44297,
	&UA32909,
	&UA1931558,
	&UA22863,
	&UA777052,
	&UA1973758,
	&UA2851335,
	&UA54286,
	&UA20523,
	&UA58875,
	&UA44052,
	&UA51628,
	&UA48891,
	&UA54436,
	&UA88881,
	&UA44285,
	&UA2526862,
	&UA1995474,
	&UA2721019,
	&UA2758994,
	&UA3044504,
	&UA10237,
	&UA64250,
	&UA44303,
	&UA43759,
	&UA2878930,
	&UA64434,
	&UA24568,
	&UA54301,
	&UA62970,
	&UA22082,
	&UA21653,
	&UA41847,
	&UA45227,
	&UA45228,
	&UA60917,
	&UA60924,
	&UA61934,
	&UA44084,
	&UA60572,
	&UA43712,
	&UA63204,
	&UA7378,
	&UA48984,
	&UA20829,
	&UA63043,
	&UA8942,
	&UA8943,
	&UA21269,
	&UA62877,
	&UA2698628,
	&UA43836,
	&UA61235,
	&UA64268,
	&UA5174,
	&UA60564,
	&UA30948,
	&UA44924,
	&UA53779,
	&UA54446,
	&UA87459,
	&UA53835,
	&UA53464,
	&UA63108,
	&UA63321,
	&UA22591,
	&UA49375,
	&UA60925,
	&UA23499,
	&UA32760,
	&UA50050,
	&UA43779,
	&UA43852,
	&UA60568,
	&UA61466,
	&UA43708,
	&UA20408,
	&UA48892,
	&UA32953,
	&UA20441,
	&UA21511,
	&UA61512,
	&UA20386,
	&UA30748,
	&UA87465,
	&UA61356,
	&UA23006,
	&UA54167,
	&UA28095,
	&UA50005,
	NULL};
const Segmentdata DUnknownWebkit[] = {
	{SG95203, &UA603047},
	{SG95204, &UA2218222},
	{SG95205, &UA2557574},
	{SG95208, &UA2733269},
	{SG95259, &UA195975},
	{SG95260, &UA196090},
	{SG95261, &UA319262},
	{SG95262, &UA628413},
	{SG95263, &UA701808},
	{SG95264, &UA2537624},
	{SG95265, &UA171163},
	{SG95266, &UA205629},
	{SG95267, &UA230936},
	{SG95268, &UA232087},
	{SG95269, &UA249543},
	{SG95270, &UA274289},
	{SG95271, &UA298795},
	{SG95272, &UA374018},
	{SG95273, &UA391473},
	{SG95274, &UA505278},
	{SG95275, &UA543888},
	{SG95276, &UA595788},
	{SG95277, &UA635754},
	{SG95278, &UA669054},
	{SG95279, &UA869988},
	{SG95280, &UA902578},
	{SG95281, &UA1892886},
	{SG95282, &UA2004710},
	{SG95283, &UA2215225},
	{SG95284, &UA2219978},
	{SG95285, &UA2279760},
	{SG95286, &UA2357257},
	{SG95287, &UA2534136},
	{SG95288, &UA2678836},
	{SG95354, &UA1996319},
	{SG95355, &UA2041168},
	{SG95356, &UA2164065},
	{SG95357, &UA2195638},
	{SG95358, &UA2199899},
	{SG95359, &UA2201592},
	{SG95360, &UA2203340},
	{SG95361, &UA2203599},
	{SG95362, &UA2205830},
	{SG95363, &UA2206141},
	{SG95364, &UA2232623},
	{SG95365, &UA2232714},
	{SG95366, &UA2237527},
	{SG95367, &UA2350057},
	{SG95368, &UA2360619},
	{SG95369, &UA2381147},
	{SG95370, &UA2383484},
	{SG95371, &UA2399594},
	{SG95372, &UA2415907},
	{SG95373, &UA2421886},
	{SG95374, &UA2454509},
	{SG95375, &UA2465596},
	{SG95376, &UA2524819},
	{SG95377, &UA2551002},
	{SG95378, &UA2577487},
	{SG95379, &UA2593971},
	{SG95380, &UA2652046},
	{SG95381, &UA2691526},
	{SG95382, &UA2691747},
	{SG95383, &UA2715456},
	{SG95384, &UA2726535},
	{SG95385, &UA2844719},
	{SG95386, &UA2894297},
	{SG95387, &UA2981700},
	{SG95388, &UA2999391},
	{SG95389, &UA3005224},
	{SG95390, &UA3006190},
	{SG95498, &UA2387489},
	{SG95516, &UA473746},
	{SG95517, &UA687485},
	{SG95518, &UA895729},
	{SG95522, &UA305956},
	{SG95523, &UA670197},
	{SG95524, &UA896162},
	{SG95525, &UA896273},
	{SG95526, &UA896907},
	{SG95527, &UA897364},
	{SG95528, &UA911705},
	{SG95529, &UA1960202},
	{SG95530, &UA1996703},
	{SG95531, &UA2039630},
	{SG95532, &UA2200121},
	{SG95533, &UA2201590},
	{SG95534, &UA2201596},
	{SG95535, &UA2232262},
	{SG95536, &UA2237163},
	{SG95537, &UA2237603},
	{SG95538, &UA2314456},
	{SG95539, &UA2332594},
	{SG95540, &UA2349781},
	{SG95541, &UA2350213},
	{SG95542, &UA2352522},
	{SG95543, &UA2353955},
	{SG95544, &UA2366174},
	{SG95546, &UA2454658},
	{SG95547, &UA2454975},
	{SG95548, &UA2466030},
	{SG95549, &UA2481503},
	{SG95550, &UA2500385},
	{SG95551, &UA2525675},
	{SG95552, &UA2551112},
	{SG95553, &UA2553208},
	{SG95554, &UA2577740},
	{SG95555, &UA2578308},
	{SG95556, &UA2695018},
	{SG95557, &UA2726763},
	{SG95558, &UA2746437},
	{SG95559, &UA2835010},
	{SG95560, &UA2981886},
	{SG95561, &UA2981960},
	{SG95562, &UA3007022},
	{SG95570, &UA187680},
	{SG95571, &UA195396},
	{SG95572, &UA222600},
	{SG95573, &UA245248},
	{SG95574, &UA264605},
	{SG95575, &UA289999},
	{SG95576, &UA382956},
	{SG95577, &UA431020},
	{SG95578, &UA484213},
	{SG95579, &UA545987},
	{SG95580, &UA576210},
	{SG95581, &UA629900},
	{SG95582, &UA761563},
	{SG95583, &UA776779},
	{SG95584, &UA820634},
	{SG95585, &UA834328},
	{SG95586, &UA854134},
	{SG95587, &UA1851002},
	{SG95588, &UA1884772},
	{SG95589, &UA1996990},
	{SG95590, &UA2268406},
	{SG95591, &UA2355155},
	{SG95592, &UA2506262},
	{SG95593, &UA2588144},
	{SG95594, &UA2668110},
	{SG95595, &UA2956978},
	{SG95601, &UA95955},
	{SG95602, &UA96565},
	{SG95603, &UA96798},
	{SG95604, &UA99207},
	{SG95605, &UA133506},
	{SG95606, &UA154473},
	{SG95607, &UA166808},
	{SG95608, &UA181410},
	{SG95609, &UA191138},
	{SG95610, &UA195228},
	{SG95611, &UA351743},
	{SG95612, &UA391553},
	{SG95613, &UA409326},
	{SG95614, &UA568006},
	{SG95615, &UA575512},
	{SG95616, &UA678963},
	{SG95617, &UA685079},
	{SG95618, &UA743351},
	{SG95619, &UA767332},
	{SG95620, &UA773725},
	{SG95621, &UA788004},
	{SG95622, &UA793126},
	{SG95623, &UA812178},
	{SG95624, &UA896321},
	{SG95625, &UA898945},
	{SG95626, &UA901824},
	{SG95627, &UA911691},
	{SG95628, &UA1876287},
	{SG95629, &UA1950500},
	{SG95630, &UA1955861},
	{SG95631, &UA1956606},
	{SG95632, &UA1960335},
	{SG95633, &UA1988811},
	{SG95634, &UA1995540},
	{SG95635, &UA1998521},
	{SG95636, &UA2010015},
	{SG95637, &UA2149815},
	{SG95638, &UA2156469},
	{SG95639, &UA2166434},
	{SG95640, &UA2167614},
	{SG95641, &UA2179027},
	{SG95642, &UA2202995},
	{SG95643, &UA2217815},
	{SG95644, &UA2222225},
	{SG95645, &UA2234955},
	{SG95646, &UA2291891},
	{SG95647, &UA2308341},
	{SG95648, &UA2335930},
	{SG95649, &UA2359997},
	{SG95650, &UA2410727},
	{SG95651, &UA2412832},
	{SG95652, &UA2413432},
	{SG95653, &UA2473549},
	{SG95654, &UA2500801},
	{SG95655, &UA2520316},
	{SG95656, &UA2546174},
	{SG95657, &UA2569179},
	{SG95658, &UA2588493},
	{SG95659, &UA2726383},
	{SG95660, &UA2814801},
	{SG95661, &UA3087872},
	{SG95664, &UA366310},
	{SG95665, &UA729254},
	{SG95666, &UA840846},
	{SG95667, &UA881316},
	{SG95668, &UA1979270},
	{SG95669, &UA2152326},
	{SG95670, &UA2519863},
	{SG95711, &UA2752544},
	{SG95743, &UA93620},
	{SG95744, &UA97585},
	{SG95745, &UA99226},
	{SG95746, &UA121656},
	{SG95747, &UA140006},
	{SG95748, &UA178778},
	{SG95749, &UA228806},
	{SG95750, &UA233428},
	{SG95751, &UA449270},
	{SG95752, &UA565211},
	{SG95753, &UA697515},
	{SG95754, &UA792398},
	{SG95755, &UA849139},
	{SG95756, &UA1893165},
	{SG95757, &UA1903427},
	{SG95758, &UA1904327},
	{SG95759, &UA2145229},
	{SG95760, &UA2258287},
	{SG95761, &UA2293352},
	{SG95762, &UA2293937},
	{SG95763, &UA2296147},
	{SG95764, &UA2316948},
	{SG95765, &UA2370376},
	{SG95766, &UA2440011},
	{SG95767, &UA2639806},
	{SG95768, &UA2694571},
	{SG95769, &UA2720380},
	{SG95847, &UA657557},
	{SG95848, &UA770758},
	{SG95874, &UA511272},
	{SG95875, &UA2313649},
	{SG95876, &UA2774728},
	{SG95877, &UA2815698},
	{SG95878, &UA2978138},
	{SG95883, &UA2799073},
	{SG95884, &UA3022682},
	{SG95885, &UA3112842},
	{SG95968, &UA234683},
	{SG95971, &UA2348727},
	{SG96009, &UA211501},
	{SG96038, &UA2862011},
	{SG96039, &UA2336915},
	{SG96040, &UA2502207},
	{SG96041, &UA2574368},
	{SG96455, &UA172567},
	{SG96456, &UA196399},
	{SG96457, &UA672151},
	{SG96458, &UA821604},
	{SG96459, &UA859537},
	{SG96460, &UA2315227},
	{SG96478, &UA2714927},
	{SG96479, &UA2784041},
	{SG96480, &UA2901189},
	{SG96481, &UA2969587},
	{SG96812, &UA1894147},
	{SG96813, &UA830148},
	{SG96814, &UA2145160},
	{SG96852, &UA2141598},
	{SG96889, &UA2893823},
	{SG96890, &UA3062313},
	{SG96912, &UA393642},
	{SG96930, &UA2684434},
	{SG96954, &UA2294468},
	{SG96955, &UA2495372},
	{SG97215, &UA2868429},
	{SG97217, &UA2518463},
	{SG97276, &UA2768661},
	{SG97351, &UA2998184},
	{SG97360, &UA720451},
	{SG97448, &UA2892459},
	{SG97557, &UA2306272},
	{NULL}};
const Segmentdata DUnknownGecko[] = {
	{SG95299, &UA823007},
	{SG95300, &UA3115252},
	{SG95301, &UA3024948},
	{SG95302, &UA471570},
	{SG95303, &UA2426108},
	{SG95304, &UA2439856},
	{SG95305, &UA2894308},
	{SG95306, &UA677879},
	{SG95307, &UA794774},
	{SG95331, &UA654788},
	{SG95500, &UA2168906},
	{SG95672, &UA163670},
	{SG95673, &UA2263423},
	{SG95674, &UA2583012},
	{SG95680, &UA628061},
	{SG95681, &UA3041727},
	{SG95684, &UA551781},
	{SG95685, &UA758886},
	{SG95688, &UA792158},
	{SG95697, &UA913469},
	{SG95698, &UA2311003},
	{SG95699, &UA2681753},
	{SG95702, &UA32842},
	{SG95703, &UA1996413},
	{SG95704, &UA2202911},
	{SG95722, &UA623136},
	{SG95729, &UA2604059},
	{SG95730, &UA2670982},
	{SG95800, &UA2670402},
	{SG95801, &UA2670419},
	{SG95802, &UA2310777},
	{SG95807, &UA729752},
	{SG95808, &UA459873},
	{SG95809, &UA878926},
	{SG95810, &UA904540},
	{SG95811, &UA1840080},
	{SG95817, &UA2349591},
	{SG95818, &UA2795765},
	{SG95819, &UA2869166},
	{SG95820, &UA3092345},
	{SG95858, &UA2368156},
	{SG95859, &UA2387296},
	{SG95888, &UA2232421},
	{SG95889, &UA2986178},
	{SG95927, &UA573071},
	{SG95928, &UA2493126},
	{SG95950, &UA2235639},
	{SG95951, &UA125931},
	{SG95952, &UA636800},
	{SG95953, &UA678184},
	{SG95954, &UA2167926},
	{SG95955, &UA2555492},
	{SG95956, &UA2719119},
	{SG95959, &UA2458614},
	{SG95966, &UA2559969},
	{SG95970, &UA2459553},
	{SG95973, &UA515290},
	{SG95974, &UA2386417},
	{SG95976, &UA2674187},
	{SG95979, &UA2516400},
	{SG95980, &UA2708939},
	{SG95981, &UA2751180},
	{SG95982, &UA3057981},
	{SG95983, &UA2601104},
	{SG95986, &UA2962877},
	{SG95988, &UA2426055},
	{SG95991, &UA827325},
	{SG95992, &UA2412164},
	{SG95993, &UA2638233},
	{SG95994, &UA477917},
	{SG95996, &UA2438883},
	{SG95997, &UA2439139},
	{SG95999, &UA2506473},
	{SG96000, &UA2551810},
	{SG96011, &UA903236},
	{SG96012, &UA2189171},
	{SG96013, &UA2862308},
	{SG96014, &UA2956378},
	{SG96016, &UA2964016},
	{SG96017, &UA3017910},
	{SG96019, &UA2962876},
	{SG96020, &UA2341802},
	{SG96024, &UA2681088},
	{SG96025, &UA653539},
	{SG96026, &UA2200233},
	{SG96029, &UA2937144},
	{SG96030, &UA2937640},
	{SG96032, &UA2985502},
	{SG96034, &UA562102},
	{SG96035, &UA895407},
	{SG96036, &UA2212654},
	{SG96037, &UA2986202},
	{SG96047, &UA2260133},
	{SG96418, &UA2214232},
	{SG96419, &UA2682440},
	{SG96420, &UA2720439},
	{SG96421, &UA2459291},
	{SG96422, &UA1950786},
	{SG96426, &UA2260270},
	{SG96427, &UA745032},
	{SG96431, &UA725826},
	{SG96432, &UA1842952},
	{SG96433, &UA1995416},
	{SG96434, &UA485121},
	{SG96435, &UA2134464},
	{SG96436, &UA2147788},
	{SG96437, &UA2259994},
	{SG96438, &UA2352347},
	{SG96446, &UA2670840},
	{SG96448, &UA3107852},
	{SG96450, &UA2717896},
	{SG96451, &UA2572340},
	{SG96452, &UA2682735},
	{SG96464, &UA1995198},
	{SG96465, &UA676574},
	{SG96466, &UA2715817},
	{SG96471, &UA908701},
	{SG96473, &UA2936590},
	{SG96476, &UA906443},
	{SG96484, &UA86927},
	{SG96489, &UA180091},
	{SG96490, &UA234765},
	{SG96491, &UA740297},
	{SG96771, &UA2259999},
	{SG96772, &UA2290035},
	{SG96773, &UA665316},
	{SG96782, &UA2409739},
	{SG96783, &UA121182},
	{SG96784, &UA1857580},
	{SG96785, &UA2259692},
	{SG96786, &UA2125741},
	{SG96787, &UA2131041},
	{SG96788, &UA479742},
	{SG96789, &UA1884306},
	{SG96790, &UA2125663},
	{SG96791, &UA2785935},
	{SG96796, &UA490897},
	{SG96797, &UA2126914},
	{SG96798, &UA2601355},
	{SG96809, &UA2274072},
	{SG96810, &UA2639199},
	{SG96816, &UA2492478},
	{SG96817, &UA473269},
	{SG96818, &UA2464287},
	{SG96819, &UA2557241},
	{SG96820, &UA2701758},
	{SG96821, &UA2714343},
	{SG96825, &UA484446},
	{SG96826, &UA513156},
	{SG96827, &UA795469},
	{SG96830, &UA2557471},
	{SG96831, &UA2558117},
	{SG96835, &UA442492},
	{SG96836, &UA2714755},
	{SG96838, &UA480488},
	{SG96839, &UA2126453},
	{SG96840, &UA2587302},
	{SG96843, &UA2937701},
	{SG96844, &UA2495599},
	{SG96847, &UA639194},
	{SG96848, &UA741947},
	{SG96849, &UA2494628},
	{SG96855, &UA625897},
	{SG96856, &UA2484463},
	{SG96858, &UA502022},
	{SG96859, &UA2476388},
	{SG96860, &UA481855},
	{SG96861, &UA2439921},
	{SG96863, &UA463482},
	{SG96866, &UA2587171},
	{SG96868, &UA1894744},
	{SG96869, &UA2429697},
	{SG96876, &UA2814340},
	{SG96879, &UA2938821},
	{SG96880, &UA2958872},
	{SG96883, &UA2495470},
	{SG96884, &UA2516880},
	{SG96885, &UA2141323},
	{SG96886, &UA2718730},
	{SG96894, &UA2430214},
	{SG96895, &UA2486037},
	{SG96903, &UA2715818},
	{SG96906, &UA551291},
	{SG96907, &UA899349},
	{SG96908, &UA1988714},
	{SG96910, &UA2138059},
	{SG96911, &UA2813579},
	{SG96915, &UA652952},
	{SG96916, &UA1953308},
	{SG96917, &UA2148608},
	{SG96918, &UA539875},
	{SG96920, &UA2413459},
	{SG96924, &UA788578},
	{SG96925, &UA2639290},
	{SG96931, &UA355191},
	{SG96932, &UA359426},
	{SG96934, &UA2812145},
	{SG96935, &UA895758},
	{SG96936, &UA462661},
	{SG96937, &UA476312},
	{SG96938, &UA832312},
	{SG96939, &UA496014},
	{SG96943, &UA462486},
	{SG96944, &UA480282},
	{SG96957, &UA2800147},
	{SG96958, &UA3018055},
	{SG96959, &UA686438},
	{SG96968, &UA2409043},
	{SG96970, &UA795935},
	{SG96972, &UA714527},
	{SG96973, &UA835902},
	{SG96978, &UA2517121},
	{SG96981, &UA701248},
	{SG96982, &UA2639575},
	{SG97042, &UA2671261},
	{SG97044, &UA1893328},
	{SG97072, &UA628907},
	{SG97073, &UA2670275},
	{SG97157, &UA2669709},
	{SG97161, &UA2267491},
	{SG97165, &UA760473},
	{SG97172, &UA2819764},
	{SG97173, &UA2703328},
	{SG97174, &UA1867553},
	{SG97176, &UA2462698},
	{SG97204, &UA464601},
	{SG97205, &UA505425},
	{SG97206, &UA697111},
	{SG97218, &UA1858509},
	{SG97219, &UA830986},
	{SG97220, &UA2408691},
	{SG97227, &UA2752691},
	{SG97228, &UA2669959},
	{SG97262, &UA811277},
	{SG97263, &UA2472131},
	{SG97266, &UA2438680},
	{SG97267, &UA2587420},
	{SG97270, &UA2493634},
	{SG97273, &UA2588383},
	{SG97275, &UA2263599},
	{SG97277, &UA2213676},
	{SG97278, &UA2439527},
	{SG97279, &UA601730},
	{SG97280, &UA2409558},
	{SG97282, &UA2409463},
	{SG97284, &UA2588346},
	{SG97287, &UA513243},
	{SG97289, &UA2682977},
	{SG97290, &UA470871},
	{SG97291, &UA1891571},
	{SG97292, &UA2439323},
	{SG97295, &UA756482},
	{SG97296, &UA695675},
	{SG97297, &UA2638861},
	{SG97301, &UA1870716},
	{SG97302, &UA2638688},
	{SG97366, &UA2878447},
	{SG97368, &UA2453723},
	{SG97380, &UA567636},
	{SG97381, &UA2207092},
	{SG97384, &UA2936707},
	{SG97385, &UA1861711},
	{SG97391, &UA2496116},
	{SG97392, &UA2681943},
	{SG97395, &UA753910},
	{SG97399, &UA2695105},
	{SG97400, &UA2541005},
	{SG97406, &UA514725},
	{SG97409, &UA2866380},
	{SG97421, &UA690537},
	{SG97422, &UA760363},
	{SG97423, &UA2217417},
	{SG97425, &UA3068993},
	{SG97429, &UA2260402},
	{SG97430, &UA2806947},
	{SG97431, &UA2866201},
	{SG97443, &UA815082},
	{SG97445, &UA710263},
	{SG97446, &UA2475047},
	{SG97447, &UA2657294},
	{SG97449, &UA854095},
	{SG97453, &UA513489},
	{SG97454, &UA2408970},
	{SG97461, &UA792039},
	{SG97472, &UA2128443},
	{SG97474, &UA2474033},
	{SG97476, &UA2541114},
	{SG97484, &UA2774573},
	{SG97494, &UA732839},
	{SG97496, &UA2391908},
	{SG97505, &UA538849},
	{SG97507, &UA797449},
	{SG97508, &UA2291796},
	{SG97509, &UA2463872},
	{SG97510, &UA2640001},
	{SG97512, &UA2285408},
	{SG97514, &UA572815},
	{SG97516, &UA3097714},
	{SG97518, &UA1888220},
	{SG97519, &UA2641059},
	{SG97523, &UA730873},
	{SG97524, &UA2130684},
	{SG97527, &UA2131682},
	{SG97528, &UA2720210},
	{SG97531, &UA2639586},
	{SG97532, &UA2686711},
	{SG97535, &UA2437735},
	{SG97540, &UA2259965},
	{SG97541, &UA2260032},
	{SG97545, &UA3041226},
	{SG97549, &UA558651},
	{SG97554, &UA2209161},
	{SG97556, &UA2995680},
	{SG97562, &UA509900},
	{SG97563, &UA552983},
	{SG97564, &UA2206877},
	{SG97565, &UA2304028},
	{SG97594, &UA574821},
	{SG97595, &UA754453},
	{SG97597, &UA2125927},
	{SG97598, &UA2258643},
	{SG97607, &UA451993},
	{SG97608, &UA837030},
	{SG97609, &UA1864446},
	{SG97617, &UA2801687},
	{SG97619, &UA2977466},
	{SG97655, &UA2293450},
	{SG97657, &UA2810723},
	{SG97659, &UA3068990},
	{SG97660, &UA3068995},
	{SG97672, &UA484741},
	{SG97673, &UA512811},
	{SG97677, &UA443887},
	{SG97678, &UA474863},
	{SG97679, &UA581607},
	{SG97680, &UA3068992},
	{SG97681, &UA442787},
	{SG97683, &UA3068984},
	{SG97693, &UA3068985},
	{SG97694, &UA3068986},
	{SG97695, &UA3068997},
	{SG97696, &UA3069000},
	{SG97697, &UA3068988},
	{NULL}};
const Segmentdata DThunderbird[] = {
	{SG95816, &UA3032600},
	{SG95957, &UA2423778},
	{SG95958, &UA3032610},
	{SG95977, &UA3027749},
	{SG95978, &UA3033012},
	{SG96010, &UA2366512},
	{SG96015, &UA2259146},
	{SG96021, &UA3033374},
	{SG96028, &UA3033021},
	{SG96219, &UA3032615},
	{SG96428, &UA2040601},
	{SG96441, &UA442723},
	{SG96442, &UA472710},
	{SG96443, &UA2259038},
	{SG96444, &UA2403310},
	{SG96445, &UA633535},
	{SG96453, &UA634074},
	{SG96477, &UA2258895},
	{SG96486, &UA870455},
	{SG96518, &UA614896},
	{SG96778, &UA823115},
	{SG96780, &UA2260244},
	{SG96794, &UA2041281},
	{SG96795, &UA2205762},
	{SG96799, &UA460853},
	{SG96800, &UA904227},
	{SG96808, &UA904336},
	{SG96829, &UA2042098},
	{SG96832, &UA631521},
	{SG96833, &UA614898},
	{SG96834, &UA785046},
	{SG96837, &UA2258211},
	{SG96841, &UA2204825},
	{SG96850, &UA2160979},
	{SG96854, &UA2258253},
	{SG96857, &UA563622},
	{SG96865, &UA2259215},
	{SG96867, &UA2258136},
	{SG96870, &UA740278},
	{SG96871, &UA2258168},
	{SG96877, &UA820838},
	{SG96887, &UA909301},
	{SG96888, &UA1996192},
	{SG96891, &UA458536},
	{SG96892, &UA836351},
	{SG96893, &UA2406468},
	{SG96896, &UA777282},
	{SG96899, &UA615028},
	{SG96901, &UA782346},
	{SG96905, &UA903662},
	{SG96914, &UA600196},
	{SG96926, &UA2041840},
	{SG96928, &UA505642},
	{SG96929, &UA2758691},
	{SG96933, &UA2597728},
	{SG96940, &UA471470},
	{SG96941, &UA822347},
	{SG96942, &UA924482},
	{SG96945, &UA2945328},
	{SG96947, &UA2040904},
	{SG96948, &UA2042024},
	{SG96950, &UA626648},
	{SG96952, &UA2391565},
	{SG96956, &UA1895876},
	{SG96961, &UA2679714},
	{SG96969, &UA615168},
	{SG96974, &UA2487849},
	{SG96983, &UA904021},
	{SG97043, &UA2757256},
	{SG97074, &UA2570221},
	{SG97156, &UA590181},
	{SG97159, &UA454617},
	{SG97160, &UA627931},
	{SG97163, &UA785731},
	{SG97209, &UA2146417},
	{SG97210, &UA924439},
	{SG97261, &UA614941},
	{SG97268, &UA597184},
	{SG97269, &UA1990919},
	{SG97274, &UA601171},
	{SG97283, &UA2041164},
	{SG97286, &UA2411526},
	{SG97293, &UA789401},
	{SG97298, &UA924069},
	{SG97299, &UA457104},
	{SG97352, &UA2202076},
	{SG97353, &UA585459},
	{SG97364, &UA1996328},
	{SG97372, &UA640216},
	{SG97379, &UA1996634},
	{SG97382, &UA903846},
	{SG97383, &UA671640},
	{SG97386, &UA682355},
	{SG97387, &UA852460},
	{SG97388, &UA925202},
	{SG97389, &UA2166306},
	{SG97397, &UA625921},
	{SG97398, &UA903461},
	{SG97401, &UA2405551},
	{SG97402, &UA924785},
	{SG97403, &UA634279},
	{SG97404, &UA1963406},
	{SG97405, &UA2771276},
	{SG97408, &UA1908120},
	{SG97419, &UA725210},
	{SG97420, &UA726486},
	{SG97424, &UA2677385},
	{SG97439, &UA924787},
	{SG97441, &UA685191},
	{SG97442, &UA2487361},
	{SG97450, &UA654857},
	{SG97451, &UA454456},
	{SG97455, &UA906428},
	{SG97456, &UA601334},
	{SG97457, &UA2571158},
	{SG97459, &UA2812219},
	{SG97468, &UA599341},
	{SG97470, &UA461944},
	{SG97471, &UA878587},
	{SG97475, &UA2770275},
	{SG97483, &UA826564},
	{SG97485, &UA505606},
	{SG97493, &UA583647},
	{SG97495, &UA583929},
	{SG97511, &UA807396},
	{SG97513, &UA2165934},
	{SG97515, &UA2686671},
	{SG97520, &UA910382},
	{SG97521, &UA2487845},
	{SG97522, &UA667447},
	{SG97530, &UA576094},
	{SG97533, &UA2602915},
	{SG97534, &UA2716738},
	{SG97538, &UA2422153},
	{SG97539, &UA2785829},
	{SG97546, &UA848086},
	{SG97547, &UA705970},
	{SG97548, &UA1891109},
	{SG97551, &UA1949352},
	{SG97552, &UA585407},
	{SG97553, &UA2313274},
	{SG97559, &UA726599},
	{SG97561, &UA2147518},
	{SG97567, &UA645509},
	{SG97569, &UA2530562},
	{SG97571, &UA2716658},
	{SG97572, &UA2595798},
	{SG97573, &UA2799971},
	{SG97596, &UA646703},
	{SG97599, &UA507443},
	{SG97600, &UA1859549},
	{SG97601, &UA2868106},
	{SG97616, &UA2422701},
	{SG97618, &UA2589659},
	{SG97620, &UA836019},
	{SG97621, &UA2604833},
	{SG97653, &UA2546391},
	{SG97654, &UA2717786},
	{SG97656, &UA2721910},
	{SG97661, &UA586252},
	{SG97663, &UA2557951},
	{SG97669, &UA880356},
	{SG97670, &UA2483203},
	{SG97674, &UA830791},
	{SG97682, &UA2842819},
	{SG97684, &UA2977432},
	{SG97686, &UA574312},
	{SG97687, &UA726285},
	{SG97688, &UA896546},
	{SG97689, &UA2129412},
	{SG97690, &UA2741954},
	{SG97691, &UA2842126},
	{SG97698, &UA576112},
	{SG97699, &UA1888463},
	{SG97701, &UA2893419},
	{SG97706, &UA1904524},
	{SG97707, &UA2259088},
	{SG97721, &UA2292114},
	{SG97722, &UA2294933},
	{SG97723, &UA2297815},
	{SG97727, &UA573337},
	{SG97728, &UA2294133},
	{SG97729, &UA2855490},
	{SG97730, &UA2861940},
	{SG97731, &UA557613},
	{NULL}};
const Segmentdata DSilk[] = {
	{SG95930, &UA459195},
	{SG95931, &UA628749},
	{SG95960, &UA459194},
	{SG95961, &UA615114},
	{SG95962, &UA623418},
	{SG95963, &UA626608},
	{SG95964, &UA1948885},
	{SG95965, &UA2408357},
	{SG95967, &UA3020745},
	{SG95969, &UA2879467},
	{SG95972, &UA2599845},
	{SG95975, &UA2714132},
	{SG95984, &UA622884},
	{SG95985, &UA2408424},
	{SG95987, &UA3020905},
	{SG95989, &UA2880203},
	{SG95990, &UA2403383},
	{SG95995, &UA2601557},
	{SG95998, &UA2713497},
	{SG96001, &UA2489910},
	{SG96002, &UA346965},
	{SG96003, &UA615349},
	{SG96004, &UA641214},
	{SG96005, &UA658271},
	{SG96006, &UA2408431},
	{SG96007, &UA2537061},
	{SG96008, &UA3033642},
	{SG96018, &UA2879172},
	{SG96022, &UA2404656},
	{SG96023, &UA2492645},
	{SG96027, &UA2537072},
	{SG96031, &UA2600653},
	{SG96033, &UA2704394},
	{SG96334, &UA3037410},
	{SG96423, &UA2603086},
	{SG96424, &UA2879890},
	{SG96425, &UA2231984},
	{SG96429, &UA2702558},
	{SG96430, &UA2468382},
	{SG96439, &UA2605289},
	{SG96440, &UA2355320},
	{SG96447, &UA2408185},
	{SG96449, &UA2605819},
	{SG96454, &UA2887909},
	{SG96461, &UA2540006},
	{SG96462, &UA2489674},
	{SG96463, &UA3037416},
	{SG96467, &UA2407100},
	{SG96468, &UA2492641},
	{SG96469, &UA2879636},
	{SG96470, &UA2531585},
	{SG96472, &UA3033960},
	{SG96474, &UA2716979},
	{SG96475, &UA2406340},
	{SG96482, &UA2599456},
	{SG96483, &UA2707894},
	{SG96485, &UA2488067},
	{SG96487, &UA2236722},
	{SG96488, &UA2540015},
	{SG96774, &UA2362758},
	{SG96775, &UA2489373},
	{SG96776, &UA3030990},
	{SG96777, &UA2404369},
	{SG96779, &UA2540022},
	{SG96781, &UA2878923},
	{SG96792, &UA2887912},
	{SG96793, &UA3034336},
	{SG96801, &UA3034176},
	{SG96802, &UA2042443},
	{SG96803, &UA2599847},
	{SG96804, &UA2719477},
	{SG96805, &UA2468523},
	{SG96806, &UA2886394},
	{SG96807, &UA2705389},
	{SG96811, &UA2602529},
	{SG96815, &UA2406956},
	{SG96822, &UA2561896},
	{SG96823, &UA2489425},
	{SG96824, &UA2605816},
	{SG96828, &UA2235606},
	{SG96842, &UA2407232},
	{SG96845, &UA2732598},
	{SG96846, &UA2361992},
	{SG96851, &UA2711125},
	{SG96853, &UA2040846},
	{SG96862, &UA2230950},
	{SG96864, &UA2600969},
	{SG96872, &UA2488331},
	{SG96873, &UA2362750},
	{SG96874, &UA2885351},
	{SG96875, &UA3033886},
	{SG96878, &UA2471632},
	{SG96881, &UA2230003},
	{SG96882, &UA2540029},
	{SG96897, &UA2039625},
	{SG96898, &UA2407228},
	{SG96900, &UA2406380},
	{SG96902, &UA2490738},
	{SG96904, &UA2565334},
	{SG96909, &UA2486886},
	{SG96913, &UA2362832},
	{SG96921, &UA2604216},
	{SG96922, &UA2489665},
	{SG96923, &UA2565913},
	{SG96927, &UA2363137},
	{SG96946, &UA2404220},
	{SG96949, &UA3039096},
	{SG96951, &UA2879892},
	{SG96953, &UA2369655},
	{SG96960, &UA2486881},
	{SG96971, &UA2565416},
	{SG96977, &UA2170361},
	{SG96979, &UA2737256},
	{SG96980, &UA2040403},
	{SG97045, &UA2258081},
	{SG97158, &UA2489424},
	{SG97162, &UA2363051},
	{SG97164, &UA2430898},
	{SG97175, &UA2602364},
	{SG97177, &UA2565936},
	{SG97207, &UA2170484},
	{SG97208, &UA2188120},
	{SG97212, &UA2468233},
	{SG97213, &UA2257313},
	{SG97214, &UA2470458},
	{SG97216, &UA2362058},
	{SG97241, &UA3023351},
	{SG97260, &UA2897354},
	{SG97264, &UA2895421},
	{SG97265, &UA714066},
	{SG97271, &UA3036368},
	{SG97272, &UA2739352},
	{SG97285, &UA2605958},
	{SG97288, &UA2734395},
	{SG97294, &UA2206075},
	{SG97300, &UA2257676},
	{SG97361, &UA2429137},
	{SG97362, &UA2362797},
	{SG97363, &UA2469937},
	{SG97365, &UA2177733},
	{SG97367, &UA2604687},
	{SG97369, &UA3037010},
	{SG97370, &UA2368037},
	{SG97390, &UA2175683},
	{SG97393, &UA2888662},
	{SG97394, &UA2490932},
	{SG97396, &UA2383615},
	{SG97407, &UA2732670},
	{SG97426, &UA2569590},
	{SG97438, &UA3032951},
	{SG97440, &UA2367609},
	{SG97444, &UA2487375},
	{SG97452, &UA2898724},
	{SG97460, &UA2487855},
	{SG97469, &UA2433589},
	{SG97491, &UA2739852},
	{SG97492, &UA2424327},
	{SG97499, &UA2172432},
	{SG97517, &UA2200276},
	{SG97525, &UA2567666},
	{SG97526, &UA2739356},
	{SG97529, &UA2515242},
	{SG97558, &UA2566660},
	{SG97560, &UA2471953},
	{SG97566, &UA2356098},
	{SG97568, &UA2423850},
	{SG97570, &UA2653698},
	{SG97622, &UA2487580},
	{SG97662, &UA2406748},
	{SG97671, &UA2391436},
	{SG97675, &UA2340839},
	{SG97692, &UA2708938},
	{SG97724, &UA2380863},
	{SG97732, &UA3079005},
	{SG97744, &UA2487294},
	{SG97768, &UA2866252},
	{SG97770, &UA2413884},
	{SG97773, &UA3055940},
	{NULL}};
const Segmentdata DCFNetwork_Darwin[] = {
	{SG96042, &UA2199864},
	{SG96043, &UA2200143},
	{SG96044, &UA2200279},
	{SG96045, &UA2202547},
	{SG96046, &UA2202602},
	{SG96048, &UA2203153},
	{SG96049, &UA2203308},
	{SG96050, &UA2203534},
	{SG96051, &UA2203767},
	{SG96052, &UA2205983},
	{SG96053, &UA2206082},
	{SG96054, &UA2233041},
	{SG96055, &UA2233401},
	{SG96056, &UA2237624},
	{SG96057, &UA2238363},
	{SG96058, &UA2242288},
	{SG96059, &UA2242353},
	{SG96060, &UA2261756},
	{SG96061, &UA2348726},
	{SG96062, &UA2350212},
	{SG96063, &UA2351699},
	{SG96064, &UA2351906},
	{SG96065, &UA2363175},
	{SG96066, &UA2367311},
	{SG96067, &UA2368656},
	{SG96068, &UA2381120},
	{SG96069, &UA2381961},
	{SG96070, &UA2383237},
	{SG96071, &UA2386866},
	{SG96072, &UA2400877},
	{SG96073, &UA2402368},
	{SG96074, &UA2417711},
	{SG96075, &UA2421475},
	{SG96076, &UA2425082},
	{SG96077, &UA2425675},
	{SG96078, &UA2425825},
	{SG96079, &UA2426134},
	{SG96080, &UA2426274},
	{SG96081, &UA2437256},
	{SG96082, &UA2454204},
	{SG96083, &UA2454476},
	{SG96084, &UA2455532},
	{SG96085, &UA2465480},
	{SG96086, &UA2485392},
	{SG96087, &UA2486165},
	{SG96088, &UA2488881},
	{SG96089, &UA2489546},
	{SG96090, &UA2490322},
	{SG96091, &UA2490419},
	{SG96092, &UA2490926},
	{SG96093, &UA2500863},
	{SG96094, &UA2505657},
	{SG96095, &UA2507864},
	{SG96096, &UA2508662},
	{SG96097, &UA2508929},
	{SG96098, &UA2509059},
	{SG96099, &UA2509673},
	{SG96100, &UA2509851},
	{SG96101, &UA2510216},
	{SG96102, &UA2510720},
	{SG96103, &UA2511047},
	{SG96104, &UA2511145},
	{SG96105, &UA2511508},
	{SG96106, &UA2511928},
	{SG96107, &UA2512221},
	{SG96108, &UA2513207},
	{SG96109, &UA2513221},
	{SG96110, &UA2514712},
	{SG96111, &UA2514720},
	{SG96112, &UA2515234},
	{SG96113, &UA2530722},
	{SG96114, &UA2530725},
	{SG96115, &UA2532368},
	{SG96116, &UA2532960},
	{SG96117, &UA2533095},
	{SG96118, &UA2535016},
	{SG96119, &UA2535209},
	{SG96120, &UA2535210},
	{SG96121, &UA2535617},
	{SG96122, &UA2535749},
	{SG96123, &UA2535973},
	{SG96124, &UA2536061},
	{SG96125, &UA2536060},
	{SG96126, &UA2536682},
	{SG96127, &UA2536812},
	{SG96128, &UA2536813},
	{SG96129, &UA2537057},
	{SG96130, &UA2537058},
	{SG96131, &UA2537240},
	{SG96132, &UA2537377},
	{SG96133, &UA2537736},
	{SG96134, &UA2558146},
	{SG96135, &UA2558183},
	{SG96136, &UA2558374},
	{SG96137, &UA2558373},
	{SG96138, &UA2558402},
	{SG96139, &UA2558431},
	{SG96140, &UA2558453},
	{SG96141, &UA2558805},
	{SG96142, &UA2558937},
	{SG96143, &UA2559989},
	{SG96144, &UA2560531},
	{SG96145, &UA2560755},
	{SG96146, &UA2560871},
	{SG96147, &UA2560968},
	{SG96148, &UA2561332},
	{SG96149, &UA2561668},
	{SG96150, &UA2563639},
	{SG96151, &UA2566585},
	{SG96152, &UA2570433},
	{SG96153, &UA2571393},
	{SG96154, &UA2577986},
	{SG96155, &UA2580317},
	{SG96156, &UA2580534},
	{SG96157, &UA2580716},
	{SG96158, &UA2584203},
	{SG96159, &UA2596323},
	{SG96160, &UA2597196},
	{SG96161, &UA2597712},
	{SG96162, &UA2597820},
	{SG96163, &UA2597927},
	{SG96164, &UA2597979},
	{SG96165, &UA2598182},
	{SG96166, &UA2598492},
	{SG96167, &UA2598499},
	{SG96168, &UA2599661},
	{SG96169, &UA2599963},
	{SG96170, &UA2601595},
	{SG96171, &UA2601728},
	{SG96172, &UA2601928},
	{SG96173, &UA2602944},
	{SG96174, &UA2603803},
	{SG96175, &UA2604698},
	{SG96176, &UA2605198},
	{SG96177, &UA2606384},
	{SG96178, &UA2659595},
	{SG96179, &UA2659597},
	{SG96180, &UA2659884},
	{SG96181, &UA2660134},
	{SG96182, &UA2660284},
	{SG96183, &UA2660384},
	{SG96184, &UA2660588},
	{SG96185, &UA2660610},
	{SG96186, &UA2660709},
	{SG96187, &UA2660723},
	{SG96188, &UA2661007},
	{SG96189, &UA2661103},
	{SG96190, &UA2661551},
	{SG96191, &UA2662217},
	{SG96192, &UA2663848},
	{SG96193, &UA2665080},
	{SG96194, &UA2668287},
	{SG96195, &UA2672211},
	{SG96196, &UA2672580},
	{SG96197, &UA2674185},
	{SG96198, &UA2675595},
	{SG96199, &UA2676356},
	{SG96200, &UA2677174},
	{SG96201, &UA2700393},
	{SG96202, &UA2700598},
	{SG96203, &UA2701183},
	{SG96204, &UA2701272},
	{SG96205, &UA2701287},
	{SG96206, &UA2701332},
	{SG96207, &UA2701508},
	{SG96208, &UA2701918},
	{SG96209, &UA2702069},
	{SG96210, &UA2702408},
	{SG96211, &UA2702553},
	{SG96212, &UA2705421},
	{SG96213, &UA2706971},
	{SG96214, &UA2707280},
	{SG96215, &UA2708051},
	{SG96216, &UA2709541},
	{SG96217, &UA2710520},
	{SG96218, &UA2712891},
	{SG96220, &UA2715808},
	{SG96221, &UA2716074},
	{SG96222, &UA2716333},
	{SG96223, &UA2716495},
	{SG96224, &UA2716620},
	{SG96225, &UA2726681},
	{SG96226, &UA2730617},
	{SG96227, &UA2730926},
	{SG96228, &UA2731737},
	{SG96229, &UA2732118},
	{SG96230, &UA2732393},
	{SG96231, &UA2736028},
	{SG96232, &UA2738054},
	{SG96233, &UA2738226},
	{SG96234, &UA2738610},
	{SG96235, &UA2748319},
	{SG96236, &UA2748388},
	{SG96237, &UA2748418},
	{SG96238, &UA2748762},
	{SG96239, &UA2748842},
	{SG96240, &UA2749113},
	{SG96241, &UA2749886},
	{SG96242, &UA2751152},
	{SG96243, &UA2752898},
	{SG96244, &UA2764166},
	{SG96245, &UA2764400},
	{SG96246, &UA2764423},
	{SG96247, &UA2764680},
	{SG96248, &UA2764717},
	{SG96249, &UA2764900},
	{SG96250, &UA2765222},
	{SG96251, &UA2770001},
	{SG96252, &UA2770621},
	{SG96253, &UA2773257},
	{SG96254, &UA2779249},
	{SG96255, &UA2780035},
	{SG96256, &UA2792260},
	{SG96257, &UA2793870},
	{SG96258, &UA2795753},
	{SG96259, &UA2796877},
	{SG96260, &UA2797380},
	{SG96261, &UA2799294},
	{SG96262, &UA2799311},
	{SG96263, &UA2800706},
	{SG96264, &UA2843159},
	{SG96265, &UA2850659},
	{SG96266, &UA2852056},
	{SG96267, &UA2852221},
	{SG96268, &UA2852911},
	{SG96269, &UA2857925},
	{SG96270, &UA2860056},
	{SG96271, &UA2860241},
	{SG96272, &UA2860776},
	{SG96273, &UA2862131},
	{SG96274, &UA2866584},
	{SG96275, &UA2868514},
	{SG96276, &UA2869349},
	{SG96277, &UA2870006},
	{SG96278, &UA2873838},
	{SG96279, &UA2874508},
	{SG96280, &UA2875305},
	{SG96281, &UA2875374},
	{SG96282, &UA2878662},
	{SG96283, &UA2879047},
	{SG96284, &UA2885364},
	{SG96285, &UA2889756},
	{SG96286, &UA2890982},
	{SG96287, &UA2892215},
	{SG96288, &UA2892620},
	{SG96289, &UA2892997},
	{SG96290, &UA2893158},
	{SG96291, &UA2896092},
	{SG96292, &UA2897654},
	{SG96293, &UA2897964},
	{SG96294, &UA2898637},
	{SG96295, &UA2898770},
	{SG96296, &UA2899209},
	{SG96297, &UA2934649},
	{SG96298, &UA2934989},
	{SG96299, &UA2937046},
	{SG96300, &UA2937199},
	{SG96301, &UA2937482},
	{SG96302, &UA2937935},
	{SG96303, &UA2938309},
	{SG96304, &UA2947003},
	{SG96305, &UA2952275},
	{SG96306, &UA2955571},
	{SG96307, &UA2956232},
	{SG96308, &UA2957034},
	{SG96309, &UA2957465},
	{SG96310, &UA2957482},
	{SG96311, &UA2957949},
	{SG96312, &UA2957955},
	{SG96313, &UA2958549},
	{SG96314, &UA2958987},
	{SG96315, &UA2959376},
	{SG96316, &UA2964449},
	{SG96317, &UA2965945},
	{SG96318, &UA2968192},
	{SG96319, &UA2970468},
	{SG96320, &UA2971209},
	{SG96321, &UA2971210},
	{SG96322, &UA2971211},
	{SG96323, &UA2974403},
	{SG96324, &UA2975830},
	{SG96325, &UA2985836},
	{SG96326, &UA2986759},
	{SG96327, &UA2986922},
	{SG96328, &UA2987076},
	{SG96329, &UA2987394},
	{SG96330, &UA2987416},
	{SG96331, &UA2987722},
	{SG96332, &UA2987830},
	{SG96333, &UA2988908},
	{SG96335, &UA2989254},
	{SG96336, &UA2989307},
	{SG96337, &UA2994430},
	{SG96338, &UA2996153},
	{SG96339, &UA2996484},
	{SG96340, &UA2996490},
	{SG96341, &UA2997042},
	{SG96342, &UA2997914},
	{SG96343, &UA2998575},
	{SG96344, &UA2998952},
	{SG96345, &UA2999672},
	{SG96346, &UA3002023},
	{SG96347, &UA3003737},
	{SG96348, &UA3006180},
	{SG96349, &UA3007081},
	{SG96350, &UA3008849},
	{SG96351, &UA3008941},
	{SG96352, &UA3008946},
	{SG96353, &UA3009059},
	{SG96354, &UA3009961},
	{SG96355, &UA3011407},
	{SG96356, &UA3013024},
	{SG96357, &UA3017400},
	{SG96358, &UA3018675},
	{SG96359, &UA3019375},
	{SG96360, &UA3019451},
	{SG96361, &UA3019849},
	{SG96362, &UA3019885},
	{SG96363, &UA3021295},
	{SG96364, &UA3021999},
	{SG96365, &UA3024467},
	{SG96366, &UA3028848},
	{SG96367, &UA3034191},
	{SG96368, &UA3044610},
	{SG96369, &UA3044663},
	{SG96370, &UA3044747},
	{SG96371, &UA3044860},
	{SG96372, &UA3045017},
	{SG96373, &UA3047722},
	{SG96374, &UA3048300},
	{SG96375, &UA3048671},
	{SG96376, &UA3048680},
	{SG96377, &UA3048769},
	{SG96378, &UA3049407},
	{SG96379, &UA3049539},
	{SG96380, &UA3051970},
	{SG96381, &UA3053650},
	{SG96382, &UA3054110},
	{SG96383, &UA3058270},
	{SG96384, &UA3058445},
	{SG96385, &UA3061319},
	{SG96386, &UA3061673},
	{SG96387, &UA3065789},
	{SG96388, &UA3066656},
	{SG96389, &UA3071450},
	{SG96390, &UA3071533},
	{SG96391, &UA3071671},
	{SG96392, &UA3073439},
	{SG96393, &UA3074196},
	{SG96394, &UA3074198},
	{SG96395, &UA3078319},
	{SG96396, &UA3078356},
	{SG96397, &UA3082192},
	{SG96398, &UA3083265},
	{SG96399, &UA3086492},
	{SG96400, &UA3086817},
	{SG96401, &UA3088109},
	{SG96402, &UA3088219},
	{SG96403, &UA3092022},
	{SG96404, &UA3092090},
	{SG96405, &UA3093321},
	{SG96406, &UA3096629},
	{SG96407, &UA3100553},
	{SG96408, &UA3101536},
	{SG96409, &UA3102452},
	{SG96410, &UA3102932},
	{SG96411, &UA3103117},
	{SG96412, &UA3104484},
	{SG96413, &UA3107923},
	{SG96414, &UA3108600},
	{SG96415, &UA3108776},
	{SG96416, &UA3124650},
	{SG96417, &UA3125118},
	{SG96492, &UA421980},
	{SG96493, &UA428203},
	{SG96494, &UA428398},
	{SG96495, &UA454600},
	{SG96496, &UA454850},
	{SG96497, &UA529033},
	{SG96498, &UA535773},
	{SG96499, &UA545521},
	{SG96500, &UA546004},
	{SG96501, &UA609028},
	{SG96502, &UA613264},
	{SG96503, &UA613277},
	{SG96504, &UA635132},
	{SG96505, &UA636790},
	{SG96506, &UA641394},
	{SG96507, &UA661332},
	{SG96508, &UA662306},
	{SG96509, &UA665122},
	{SG96510, &UA671450},
	{SG96511, &UA676719},
	{SG96512, &UA676927},
	{SG96513, &UA686116},
	{SG96514, &UA687424},
	{SG96515, &UA688583},
	{SG96516, &UA696041},
	{SG96517, &UA698859},
	{SG96519, &UA722396},
	{SG96520, &UA725442},
	{SG96521, &UA746441},
	{SG96522, &UA749842},
	{SG96523, &UA755694},
	{SG96524, &UA773212},
	{SG96525, &UA784061},
	{SG96526, &UA787889},
	{SG96527, &UA801121},
	{SG96528, &UA811153},
	{SG96529, &UA832830},
	{SG96530, &UA836759},
	{SG96531, &UA858487},
	{SG96532, &UA869275},
	{SG96533, &UA872053},
	{SG96534, &UA891318},
	{SG96535, &UA892010},
	{SG96536, &UA892173},
	{SG96537, &UA893371},
	{SG96538, &UA895782},
	{SG96539, &UA898501},
	{SG96540, &UA899607},
	{SG96541, &UA901089},
	{SG96542, &UA901095},
	{SG96543, &UA901999},
	{SG96544, &UA902079},
	{SG96545, &UA903596},
	{SG96546, &UA903844},
	{SG96547, &UA904139},
	{SG96548, &UA905635},
	{SG96549, &UA907762},
	{SG96550, &UA912382},
	{SG96551, &UA912395},
	{SG96552, &UA912429},
	{SG96553, &UA913172},
	{SG96554, &UA1937220},
	{SG96555, &UA1949655},
	{SG96556, &UA1953017},
	{SG96557, &UA1955875},
	{SG96558, &UA1958144},
	{SG96559, &UA1958371},
	{SG96560, &UA1958671},
	{SG96561, &UA1971726},
	{SG96562, &UA1978124},
	{SG96563, &UA1984138},
	{SG96564, &UA1988638},
	{SG96565, &UA1988976},
	{SG96566, &UA1990950},
	{SG96567, &UA1991053},
	{SG96568, &UA1991061},
	{SG96569, &UA1991250},
	{SG96570, &UA1992113},
	{SG96571, &UA1994178},
	{SG96572, &UA1995138},
	{SG96573, &UA1996299},
	{SG96574, &UA1997002},
	{SG96575, &UA1997051},
	{SG96576, &UA2005276},
	{SG96577, &UA2007534},
	{SG96578, &UA2009060},
	{SG96579, &UA2153675},
	{SG96580, &UA2156652},
	{SG96581, &UA2157862},
	{SG96582, &UA2159186},
	{SG96583, &UA2160573},
	{SG96584, &UA2162668},
	{SG96585, &UA2163256},
	{SG96586, &UA2163551},
	{SG96587, &UA2163625},
	{SG96588, &UA2164124},
	{SG96589, &UA2164520},
	{SG96590, &UA2165464},
	{SG96591, &UA2166758},
	{SG96592, &UA2167535},
	{SG96593, &UA2168023},
	{SG96594, &UA2169799},
	{SG96595, &UA2172966},
	{SG96596, &UA2175685},
	{SG96597, &UA2186854},
	{SG96598, &UA2189543},
	{SG96599, &UA2202840},
	{SG96600, &UA2203475},
	{SG96601, &UA2205545},
	{SG96602, &UA2205637},
	{SG96603, &UA2205682},
	{SG96604, &UA2221456},
	{SG96605, &UA2222365},
	{SG96606, &UA2223442},
	{SG96607, &UA2223748},
	{SG96608, &UA2223786},
	{SG96609, &UA2233898},
	{SG96610, &UA2234887},
	{SG96611, &UA2235141},
	{SG96612, &UA2235813},
	{SG96613, &UA2237433},
	{SG96614, &UA2238347},
	{SG96615, &UA2242244},
	{SG96616, &UA2242458},
	{SG96617, &UA2260345},
	{SG96618, &UA2260409},
	{SG96619, &UA2262002},
	{SG96620, &UA2262007},
	{SG96621, &UA2262120},
	{SG96622, &UA2262176},
	{SG96623, &UA2262894},
	{SG96624, &UA2306025},
	{SG96625, &UA2313786},
	{SG96626, &UA2316208},
	{SG96627, &UA2316493},
	{SG96628, &UA2319251},
	{SG96629, &UA2323973},
	{SG96630, &UA2324358},
	{SG96631, &UA2325090},
	{SG96632, &UA2331326},
	{SG96633, &UA2333975},
	{SG96634, &UA2337519},
	{SG96635, &UA2338369},
	{SG96636, &UA2339476},
	{SG96637, &UA2340037},
	{SG96638, &UA2342936},
	{SG96639, &UA2348493},
	{SG96640, &UA2348788},
	{SG96641, &UA2349037},
	{SG96642, &UA2353963},
	{SG96643, &UA2356443},
	{SG96644, &UA2359352},
	{SG96645, &UA2359979},
	{SG96646, &UA2364431},
	{SG96647, &UA2367072},
	{SG96648, &UA2367649},
	{SG96649, &UA2369861},
	{SG96650, &UA2372042},
	{SG96651, &UA2379184},
	{SG96652, &UA2381964},
	{SG96653, &UA2383414},
	{SG96654, &UA2386228},
	{SG96655, &UA2389475},
	{SG96656, &UA2403292},
	{SG96657, &UA2403458},
	{SG96658, &UA2410225},
	{SG96659, &UA2410363},
	{SG96660, &UA2411407},
	{SG96661, &UA2411835},
	{SG96662, &UA2411948},
	{SG96663, &UA2422325},
	{SG96664, &UA2426024},
	{SG96665, &UA2428117},
	{SG96666, &UA2441898},
	{SG96667, &UA2447659},
	{SG96668, &UA2456906},
	{SG96669, &UA2460286},
	{SG96670, &UA2463093},
	{SG96671, &UA2463325},
	{SG96672, &UA2463390},
	{SG96673, &UA2463464},
	{SG96674, &UA2463565},
	{SG96675, &UA2463566},
	{SG96676, &UA2466124},
	{SG96677, &UA2474960},
	{SG96678, &UA2476368},
	{SG96679, &UA2476464},
	{SG96680, &UA2480886},
	{SG96681, &UA2481802},
	{SG96682, &UA2486641},
	{SG96683, &UA2487391},
	{SG96684, &UA2496000},
	{SG96685, &UA2502707},
	{SG96686, &UA2518244},
	{SG96687, &UA2518688},
	{SG96688, &UA2519059},
	{SG96689, &UA2519633},
	{SG96690, &UA2544577},
	{SG96691, &UA2551524},
	{SG96692, &UA2556561},
	{SG96693, &UA2574551},
	{SG96694, &UA2578117},
	{SG96695, &UA2580183},
	{SG96696, &UA2584892},
	{SG96697, &UA2588373},
	{SG96698, &UA2596384},
	{SG96699, &UA2640624},
	{SG96700, &UA2640862},
	{SG96701, &UA2641514},
	{SG96702, &UA2642046},
	{SG96703, &UA2642977},
	{SG96704, &UA2644055},
	{SG96705, &UA2644675},
	{SG96706, &UA2652902},
	{SG96707, &UA2659019},
	{SG96708, &UA2663667},
	{SG96709, &UA2668654},
	{SG96710, &UA2670702},
	{SG96711, &UA2671909},
	{SG96712, &UA2683926},
	{SG96713, &UA2684313},
	{SG96714, &UA2684887},
	{SG96715, &UA2685058},
	{SG96716, &UA2686203},
	{SG96717, &UA2686408},
	{SG96718, &UA2691410},
	{SG96719, &UA2693116},
	{SG96720, &UA2697951},
	{SG96721, &UA2721573},
	{SG96722, &UA2730974},
	{SG96723, &UA2791217},
	{SG96724, &UA2809074},
	{SG96725, &UA2809134},
	{SG96726, &UA2811290},
	{SG96727, &UA2813208},
	{SG96728, &UA2816535},
	{SG96729, &UA2817784},
	{SG96730, &UA2835011},
	{SG96731, &UA2837309},
	{SG96732, &UA2842285},
	{SG96733, &UA2843485},
	{SG96734, &UA2844163},
	{SG96735, &UA2847350},
	{SG96736, &UA2850603},
	{SG96737, &UA2852695},
	{SG96738, &UA2859520},
	{SG96739, &UA2863644},
	{SG96740, &UA2888953},
	{SG96741, &UA2889070},
	{SG96742, &UA2933528},
	{SG96743, &UA2937122},
	{SG96744, &UA2941466},
	{SG96745, &UA2950081},
	{SG96746, &UA2950316},
	{SG96747, &UA2952028},
	{SG96748, &UA2958423},
	{SG96749, &UA2964932},
	{SG96750, &UA2980053},
	{SG96751, &UA2980505},
	{SG96752, &UA2985893},
	{SG96753, &UA2993585},
	{SG96754, &UA3020952},
	{SG96755, &UA3028593},
	{SG96756, &UA3041614},
	{SG96757, &UA3042967},
	{SG96758, &UA3045537},
	{SG96759, &UA3056951},
	{SG96760, &UA3077511},
	{SG96761, &UA3088730},
	{SG96762, &UA3088920},
	{SG96763, &UA3090506},
	{SG96764, &UA3098036},
	{SG96765, &UA3098422},
	{SG96766, &UA3102457},
	{SG96767, &UA3103317},
	{SG96768, &UA3106587},
	{SG96769, &UA3118028},
	{SG96770, &UA3128608},
	{SG96919, &UA2485704},
	{SG96962, &UA2708059},
	{SG96963, &UA2716840},
	{SG96964, &UA2736532},
	{SG96965, &UA2752446},
	{SG96966, &UA2767609},
	{SG96967, &UA2794422},
	{SG96975, &UA2753431},
	{SG96976, &UA2754548},
	{SG96984, &UA2784854},
	{SG96985, &UA2796515},
	{SG96986, &UA2800140},
	{SG96987, &UA2800651},
	{SG96988, &UA2869494},
	{SG96989, &UA2875076},
	{SG96990, &UA2879590},
	{SG96991, &UA2897082},
	{SG96992, &UA2938424},
	{SG96993, &UA2938820},
	{SG96994, &UA2958664},
	{SG96995, &UA2958848},
	{SG96996, &UA3001184},
	{SG96997, &UA3001465},
	{SG96998, &UA3006890},
	{SG96999, &UA3008984},
	{SG97000, &UA3050430},
	{SG97001, &UA3050455},
	{SG97002, &UA3050627},
	{SG97003, &UA3050863},
	{SG97004, &UA3063000},
	{SG97005, &UA3077880},
	{SG97006, &UA3081717},
	{SG97007, &UA3088828},
	{SG97008, &UA3114087},
	{SG97009, &UA378401},
	{SG97010, &UA451249},
	{SG97011, &UA552811},
	{SG97012, &UA557798},
	{SG97013, &UA560276},
	{SG97014, &UA564211},
	{SG97015, &UA640825},
	{SG97016, &UA894646},
	{SG97017, &UA895073},
	{SG97018, &UA1898931},
	{SG97019, &UA1965418},
	{SG97020, &UA2214681},
	{SG97021, &UA2262875},
	{SG97022, &UA2291692},
	{SG97023, &UA2295255},
	{SG97024, &UA2313394},
	{SG97025, &UA2337692},
	{SG97026, &UA2353708},
	{SG97027, &UA2361077},
	{SG97028, &UA2421561},
	{SG97029, &UA2444249},
	{SG97030, &UA2462888},
	{SG97031, &UA2499680},
	{SG97032, &UA2587272},
	{SG97033, &UA2593966},
	{SG97034, &UA2638889},
	{SG97035, &UA2742104},
	{SG97036, &UA2805998},
	{SG97037, &UA2806370},
	{SG97038, &UA2806974},
	{SG97039, &UA2888855},
	{SG97040, &UA2933149},
	{SG97041, &UA2951369},
	{SG97046, &UA1955896},
	{SG97047, &UA1959323},
	{SG97048, &UA2242286},
	{SG97049, &UA2338188},
	{SG97050, &UA2350694},
	{SG97051, &UA2351879},
	{SG97052, &UA2383286},
	{SG97053, &UA2386599},
	{SG97054, &UA2425002},
	{SG97055, &UA2427209},
	{SG97056, &UA2455024},
	{SG97057, &UA2525754},
	{SG97058, &UA2552522},
	{SG97059, &UA2593535},
	{SG97060, &UA2593598},
	{SG97061, &UA2727431},
	{SG97062, &UA2761427},
	{SG97063, &UA2762076},
	{SG97064, &UA2836250},
	{SG97065, &UA2839587},
	{SG97066, &UA2843308},
	{SG97067, &UA2848307},
	{SG97068, &UA2951938},
	{SG97069, &UA2981622},
	{SG97070, &UA2981712},
	{SG97071, &UA3042681},
	{SG97075, &UA2801209},
	{SG97076, &UA3093246},
	{SG97077, &UA3094895},
	{SG97078, &UA3095026},
	{SG97079, &UA3095893},
	{SG97080, &UA3096742},
	{SG97081, &UA3096842},
	{SG97082, &UA3096902},
	{SG97083, &UA3096920},
	{SG97084, &UA3096932},
	{SG97085, &UA3109538},
	{SG97086, &UA3109699},
	{SG97087, &UA3109768},
	{SG97088, &UA3109770},
	{SG97089, &UA3110011},
	{SG97090, &UA3110080},
	{SG97091, &UA3110185},
	{SG97092, &UA3110279},
	{SG97093, &UA3110357},
	{SG97094, &UA3110421},
	{SG97095, &UA3110447},
	{SG97096, &UA3110478},
	{SG97097, &UA3110499},
	{SG97098, &UA3110525},
	{SG97099, &UA3110686},
	{SG97100, &UA3110715},
	{SG97101, &UA3111031},
	{SG97102, &UA3111327},
	{SG97103, &UA3111340},
	{SG97104, &UA3111610},
	{SG97105, &UA3111774},
	{SG97106, &UA3111779},
	{SG97107, &UA3112316},
	{SG97108, &UA3112522},
	{SG97109, &UA3112788},
	{SG97110, &UA3112989},
	{SG97111, &UA3113116},
	{SG97112, &UA3113337},
	{SG97113, &UA3113417},
	{SG97114, &UA3113989},
	{SG97115, &UA3114230},
	{SG97116, &UA3114625},
	{SG97117, &UA3114630},
	{SG97118, &UA3114631},
	{SG97119, &UA3114634},
	{SG97120, &UA3114639},
	{SG97121, &UA3114640},
	{SG97122, &UA3114664},
	{SG97123, &UA3114800},
	{SG97124, &UA3114905},
	{SG97125, &UA3114974},
	{SG97126, &UA3115246},
	{SG97127, &UA3115566},
	{SG97128, &UA3115781},
	{SG97129, &UA3116188},
	{SG97130, &UA3126067},
	{SG97131, &UA3127028},
	{SG97132, &UA3127418},
	{SG97133, &UA3127449},
	{SG97134, &UA3127511},
	{SG97135, &UA3127533},
	{SG97136, &UA3127579},
	{SG97137, &UA3127603},
	{SG97138, &UA3127605},
	{SG97139, &UA3127650},
	{SG97140, &UA3127718},
	{SG97141, &UA3127753},
	{SG97142, &UA3127955},
	{SG97143, &UA3128020},
	{SG97144, &UA3128062},
	{SG97145, &UA3128075},
	{SG97146, &UA3128101},
	{SG97147, &UA3128112},
	{SG97148, &UA3128180},
	{SG97149, &UA3128196},
	{SG97150, &UA3128848},
	{SG97151, &UA3129225},
	{SG97152, &UA3130199},
	{SG97153, &UA3130219},
	{SG97154, &UA3130681},
	{SG97155, &UA3131230},
	{SG97166, &UA2781933},
	{SG97167, &UA2797460},
	{SG97168, &UA2799670},
	{SG97169, &UA2870367},
	{SG97170, &UA2871062},
	{SG97171, &UA2879406},
	{SG97178, &UA2262368},
	{SG97179, &UA2262559},
	{SG97180, &UA2348791},
	{SG97181, &UA2367536},
	{SG97182, &UA2368403},
	{SG97183, &UA2382243},
	{SG97184, &UA2387431},
	{SG97185, &UA2400891},
	{SG97186, &UA2421157},
	{SG97187, &UA2424555},
	{SG97188, &UA2426167},
	{SG97189, &UA2457283},
	{SG97190, &UA2458014},
	{SG97191, &UA2466656},
	{SG97192, &UA2479617},
	{SG97193, &UA2479978},
	{SG97194, &UA2481434},
	{SG97195, &UA2483588},
	{SG97196, &UA2525988},
	{SG97197, &UA2527886},
	{SG97198, &UA2552661},
	{SG97199, &UA2556788},
	{SG97200, &UA2691875},
	{SG97201, &UA2835023},
	{SG97202, &UA2844159},
	{SG97203, &UA2983285},
	{SG97211, &UA624860},
	{SG97221, &UA2677338},
	{SG97222, &UA2679271},
	{SG97223, &UA2709185},
	{SG97224, &UA2716981},
	{SG97225, &UA2717767},
	{SG97226, &UA2719402},
	{SG97229, &UA2505093},
	{SG97230, &UA2869738},
	{SG97231, &UA2872743},
	{SG97232, &UA2876097},
	{SG97233, &UA2896792},
	{SG97234, &UA2899429},
	{SG97235, &UA2386255},
	{SG97236, &UA2387307},
	{SG97237, &UA2404286},
	{SG97238, &UA2404317},
	{SG97239, &UA2404336},
	{SG97240, &UA2428952},
	{SG97242, &UA3079616},
	{SG97243, &UA2585262},
	{SG97244, &UA2600220},
	{SG97245, &UA2603257},
	{SG97246, &UA2603773},
	{SG97247, &UA2606965},
	{SG97248, &UA2666982},
	{SG97249, &UA2667613},
	{SG97250, &UA2669638},
	{SG97251, &UA2701380},
	{SG97252, &UA2707559},
	{SG97253, &UA2712748},
	{SG97254, &UA2733182},
	{SG97255, &UA2733840},
	{SG97256, &UA2737453},
	{SG97257, &UA2860381},
	{SG97258, &UA2860483},
	{SG97259, &UA2866649},
	{SG97281, &UA2679606},
	{SG97303, &UA436121},
	{SG97304, &UA452497},
	{SG97305, &UA624188},
	{SG97306, &UA650199},
	{SG97307, &UA657388},
	{SG97308, &UA670163},
	{SG97309, &UA676921},
	{SG97310, &UA709724},
	{SG97311, &UA721933},
	{SG97312, &UA759751},
	{SG97313, &UA766781},
	{SG97314, &UA793575},
	{SG97315, &UA843838},
	{SG97316, &UA907764},
	{SG97317, &UA909203},
	{SG97318, &UA1861752},
	{SG97319, &UA1897261},
	{SG97320, &UA1939360},
	{SG97321, &UA1988534},
	{SG97322, &UA1994118},
	{SG97323, &UA2128319},
	{SG97324, &UA2161144},
	{SG97325, &UA2162813},
	{SG97326, &UA2164079},
	{SG97327, &UA2220011},
	{SG97328, &UA2242348},
	{SG97329, &UA2285850},
	{SG97330, &UA2333863},
	{SG97331, &UA2413192},
	{SG97332, &UA2413645},
	{SG97333, &UA2419787},
	{SG97334, &UA2439993},
	{SG97335, &UA2454576},
	{SG97336, &UA2641977},
	{SG97337, &UA2742061},
	{SG97338, &UA2819059},
	{SG97339, &UA2821744},
	{SG97340, &UA2980157},
	{SG97341, &UA2756728},
	{SG97342, &UA2783873},
	{SG97343, &UA2204911},
	{SG97344, &UA2230433},
	{SG97345, &UA2535208},
	{SG97346, &UA2566254},
	{SG97347, &UA2598932},
	{SG97348, &UA2603674},
	{SG97349, &UA2604722},
	{SG97350, &UA2738517},
	{SG97354, &UA2233812},
	{SG97355, &UA2233824},
	{SG97356, &UA2233827},
	{SG97357, &UA2233840},
	{SG97358, &UA2234665},
	{SG97359, &UA2354267},
	{SG97371, &UA2888595},
	{SG97373, &UA2201963},
	{SG97374, &UA2205274},
	{SG97375, &UA2227442},
	{SG97376, &UA2350491},
	{SG97377, &UA2367778},
	{SG97378, &UA2454538},
	{SG97410, &UA427495},
	{SG97411, &UA440549},
	{SG97412, &UA2206108},
	{SG97413, &UA2342931},
	{SG97414, &UA2355874},
	{SG97415, &UA2383200},
	{SG97416, &UA2441597},
	{SG97417, &UA2654116},
	{SG97418, &UA2685131},
	{SG97427, &UA2203164},
	{SG97428, &UA2740419},
	{SG97432, &UA2797432},
	{SG97433, &UA2869950},
	{SG97434, &UA2879461},
	{SG97435, &UA2938159},
	{SG97436, &UA2957393},
	{SG97437, &UA3011663},
	{SG97458, &UA2567840},
	{SG97462, &UA2557722},
	{SG97463, &UA2580041},
	{SG97464, &UA2581337},
	{SG97465, &UA2604431},
	{SG97466, &UA2698881},
	{SG97467, &UA2712241},
	{SG97473, &UA2713489},
	{SG97477, &UA2581503},
	{SG97478, &UA2606345},
	{SG97479, &UA2669037},
	{SG97480, &UA2703277},
	{SG97481, &UA2705232},
	{SG97482, &UA2767257},
	{SG97486, &UA2235505},
	{SG97487, &UA2352652},
	{SG97488, &UA2382650},
	{SG97489, &UA2417730},
	{SG97490, &UA2419096},
	{SG97497, &UA2199853},
	{SG97498, &UA2205990},
	{SG97500, &UA2238605},
	{SG97501, &UA2369035},
	{SG97502, &UA2509848},
	{SG97503, &UA2659832},
	{SG97504, &UA2664050},
	{SG97506, &UA2205695},
	{SG97536, &UA632278},
	{SG97537, &UA2754371},
	{SG97542, &UA2436547},
	{SG97543, &UA2242307},
	{SG97544, &UA2242306},
	{SG97550, &UA2205701},
	{SG97555, &UA2673880},
	{SG97574, &UA379001},
	{SG97575, &UA473992},
	{SG97576, &UA499157},
	{SG97577, &UA546801},
	{SG97578, &UA564782},
	{SG97579, &UA643786},
	{SG97580, &UA650117},
	{SG97581, &UA680048},
	{SG97582, &UA702630},
	{SG97583, &UA712193},
	{SG97584, &UA713361},
	{SG97585, &UA742666},
	{SG97586, &UA762047},
	{SG97587, &UA792421},
	{SG97588, &UA838321},
	{SG97589, &UA896882},
	{SG97590, &UA1868188},
	{SG97591, &UA1875166},
	{SG97592, &UA2276622},
	{SG97593, &UA3098367},
	{SG97602, &UA2781847},
	{SG97603, &UA2782138},
	{SG97604, &UA2784473},
	{SG97605, &UA2800129},
	{SG97606, &UA2866270},
	{SG97610, &UA395693},
	{SG97611, &UA450552},
	{SG97612, &UA558022},
	{SG97613, &UA1888494},
	{SG97614, &UA2139123},
	{SG97615, &UA2139794},
	{SG97623, &UA389694},
	{SG97624, &UA511142},
	{SG97625, &UA515146},
	{SG97626, &UA516381},
	{SG97627, &UA518450},
	{SG97628, &UA521309},
	{SG97629, &UA530867},
	{SG97630, &UA539369},
	{SG97631, &UA571961},
	{SG97632, &UA650813},
	{SG97633, &UA692895},
	{SG97634, &UA700077},
	{SG97635, &UA718401},
	{SG97636, &UA739329},
	{SG97637, &UA780622},
	{SG97638, &UA1858786},
	{SG97639, &UA1865155},
	{SG97640, &UA1897836},
	{SG97641, &UA1904919},
	{SG97642, &UA1949077},
	{SG97643, &UA1974076},
	{SG97644, &UA2279360},
	{SG97645, &UA2283550},
	{SG97646, &UA2287668},
	{SG97647, &UA2297358},
	{SG97648, &UA2574683},
	{SG97649, &UA2594249},
	{SG97650, &UA2812777},
	{SG97651, &UA2933927},
	{SG97652, &UA2679615},
	{SG97658, &UA2235739},
	{SG97664, &UA3096099},
	{SG97665, &UA2348949},
	{SG97666, &UA2384500},
	{SG97667, &UA2502380},
	{SG97668, &UA3085475},
	{SG97676, &UA2568681},
	{SG97685, &UA2369670},
	{SG97700, &UA2236232},
	{SG97702, &UA2226732},
	{SG97703, &UA2226961},
	{SG97704, &UA2502165},
	{SG97705, &UA2759287},
	{SG97708, &UA2437211},
	{SG97709, &UA427398},
	{SG97710, &UA633476},
	{SG97711, &UA657666},
	{SG97712, &UA891509},
	{SG97713, &UA2318644},
	{SG97714, &UA2354989},
	{SG97715, &UA2356693},
	{SG97716, &UA2262896},
	{SG97717, &UA2384737},
	{SG97718, &UA2421201},
	{SG97719, &UA2552990},
	{SG97720, &UA2652811},
	{SG97725, &UA625429},
	{SG97726, &UA775913},
	{SG97733, &UA623805},
	{SG97734, &UA800069},
	{SG97735, &UA841219},
	{SG97736, &UA2362274},
	{SG97737, &UA2763243},
	{SG97738, &UA2889179},
	{SG97739, &UA3083098},
	{SG97740, &UA2199846},
	{SG97741, &UA2199849},
	{SG97742, &UA2201364},
	{SG97743, &UA2202475},
	{SG97766, &UA427420},
	{SG97767, &UA638073},
	{SG97769, &UA675538},
	{SG97771, &UA2367223},
	{SG97772, &UA2202852},
	{SG97774, &UA435978},
	{SG97775, &UA489154},
	{SG97776, &UA516188},
	{SG97777, &UA517842},
	{SG97778, &UA568244},
	{SG97779, &UA629349},
	{SG97780, &UA659952},
	{SG97781, &UA739304},
	{SG97782, &UA804409},
	{SG97783, &UA819471},
	{SG97784, &UA1892925},
	{SG97785, &UA1927431},
	{SG97786, &UA1945794},
	{SG97787, &UA2135480},
	{SG97788, &UA2150384},
	{SG97789, &UA2375888},
	{SG97790, &UA2441294},
	{SG97791, &UA2445541},
	{SG97792, &UA2643203},
	{SG97793, &UA2801320},
	{SG97794, &UA2482574},
	{SG97795, &UA2551799},
	{SG97796, &UA2834128},
	{SG97797, &UA2358167},
	{SG97798, &UA2508766},
	{SG97799, &UA2937138},
	{SG97800, &UA2242236},
	{SG97801, &UA2262469},
	{SG97802, &UA647424},
	{SG97803, &UA651046},
	{SG97804, &UA661143},
	{SG97805, &UA701620},
	{SG97806, &UA767491},
	{SG97807, &UA872000},
	{SG97808, &UA663299},
	{SG97809, &UA855685},
	{SG97810, &UA1942555},
	{SG97811, &UA1967666},
	{SG97812, &UA1967967},
	{SG97813, &UA2312864},
	{SG97814, &UA2318710},
	{SG97815, &UA2500234},
	{SG97816, &UA2670612},
	{SG97826, &UA2999626},
	{SG97827, &UA3051450},
	{SG97828, &UA3088917},
	{SG97829, &UA2437233},
	{SG97830, &UA2705108},
	{SG97844, &UA575152},
	{SG97845, &UA1904862},
	{SG97846, &UA1936517},
	{SG97847, &UA1939338},
	{SG97848, &UA2639686},
	{SG97849, &UA2653849},
	{SG97850, &UA2720580},
	{SG97865, &UA201735},
	{SG97866, &UA2599308},
	{SG97867, &UA3111730},
	{SG97868, &UA3127665},
	{SG97869, &UA3131327},
	{SG97870, &UA2205839},
	{SG97871, &UA588927},
	{SG97872, &UA652057},
	{SG97873, &UA664939},
	{SG97874, &UA707865},
	{SG97875, &UA761324},
	{SG97876, &UA772792},
	{SG97885, &UA784715},
	{SG97886, &UA2545745},
	{SG97887, &UA2231034},
	{SG97888, &UA409321},
	{SG97889, &UA543317},
	{SG97890, &UA547427},
	{SG97891, &UA670870},
	{SG97892, &UA755682},
	{SG97893, &UA789230},
	{SG97894, &UA911142},
	{SG97895, &UA911152},
	{SG97896, &UA1955794},
	{SG97897, &UA1966582},
	{SG97898, &UA1984728},
	{SG97948, &UA180244},
	{SG97949, &UA207981},
	{SG97950, &UA318479},
	{SG97951, &UA1889831},
	{SG97952, &UA2503127},
	{SG97953, &UA2553919},
	{SG97954, &UA2801225},
	{SG97955, &UA3111551},
	{SG97956, &UA643816},
	{SG97957, &UA377434},
	{SG97958, &UA526352},
	{SG97959, &UA628489},
	{SG97960, &UA746774},
	{SG97961, &UA752610},
	{SG97962, &UA875717},
	{SG97963, &UA2537479},
	{SG97964, &UA2559531},
	{SG97965, &UA2577560},
	{SG97966, &UA2343137},
	{SG97967, &UA2654746},
	{SG97968, &UA2859097},
	{SG97969, &UA693791},
	{SG97970, &UA2705403},
	{SG97971, &UA599465},
	{SG98007, &UA901889},
	{SG98008, &UA599655},
	{SG98009, &UA2672740},
	{SG98010, &UA1997529},
	{SG98011, &UA2496813},
	{SG98012, &UA2546126},
	{SG98013, &UA2421518},
	{SG98044, &UA2654567},
	{SG98065, &UA433280},
	{SG98066, &UA751047},
	{SG98067, &UA869278},
	{SG98068, &UA1862984},
	{SG98069, &UA2652828},
	{SG98070, &UA2790030},
	{SG98071, &UA466014},
	{SG98072, &UA752573},
	{SG98073, &UA2885406},
	{SG98074, &UA2010501},
	{SG98075, &UA2183086},
	{SG98076, &UA2186712},
	{SG98088, &UA3131264},
	{SG98089, &UA924138},
	{SG98090, &UA1953979},
	{SG98091, &UA1966465},
	{SG98092, &UA1978235},
	{SG98093, &UA2336090},
	{SG98094, &UA140340},
	{SG98095, &UA2262365},
	{SG98096, &UA2841915},
	{SG98097, &UA497084},
	{SG98098, &UA515172},
	{SG98099, &UA554885},
	{SG98100, &UA746610},
	{SG98101, &UA807912},
	{SG98102, &UA420407},
	{SG98103, &UA561715},
	{SG98104, &UA779531},
	{SG98105, &UA810038},
	{SG98106, &UA826776},
	{SG98107, &UA791719},
	{SG98108, &UA2425220},
	{SG98109, &UA2262885},
	{SG98110, &UA1978562},
	{SG98111, &UA709742},
	{SG98112, &UA514872},
	{SG98113, &UA2212750},
	{SG98114, &UA2138774},
	{SG98115, &UA2495549},
	{SG98116, &UA3103113},
	{SG98117, &UA893565},
	{SG98118, &UA1862204},
	{SG98119, &UA635735},
	{SG98120, &UA642198},
	{SG98121, &UA643119},
	{SG98122, &UA684808},
	{SG98123, &UA2306439},
	{SG98124, &UA2827286},
	{SG98125, &UA3048305},
	{SG98126, &UA3100266},
	{SG98133, &UA625919},
	{SG98134, &UA657973},
	{SG98135, &UA685284},
	{SG98136, &UA826600},
	{SG98137, &UA872426},
	{SG98138, &UA2134566},
	{SG98139, &UA213503},
	{SG98140, &UA2285815},
	{SG98141, &UA427448},
	{SG98142, &UA559758},
	{SG98143, &UA1862517},
	{SG98144, &UA624960},
	{SG98145, &UA2588238},
	{SG98146, &UA845514},
	{SG98147, &UA2180921},
	{SG98148, &UA2187348},
	{SG98149, &UA2190156},
	{SG98150, &UA2186464},
	{SG98151, &UA2189374},
	{SG98152, &UA255851},
	{SG98153, &UA450811},
	{SG98154, &UA641479},
	{SG98155, &UA627280},
	{SG98156, &UA1986404},
	{SG98157, &UA1996896},
	{SG98158, &UA1996966},
	{SG98159, &UA1906586},
	{SG98160, &UA2301399},
	{SG98161, &UA397843},
	{SG98162, &UA1907344},
	{SG98163, &UA738418},
	{SG98164, &UA437925},
	{SG98165, &UA589895},
	{SG98166, &UA728632},
	{SG98167, &UA589974},
	{SG98169, &UA599634},
	{SG98170, &UA1874873},
	{SG98171, &UA213502},
	{SG98172, &UA656498},
	{SG98173, &UA621905},
	{SG98174, &UA648435},
	{SG98175, &UA2208333},
	{SG98177, &UA452823},
	{SG98178, &UA606813},
	{SG98179, &UA621886},
	{SG98180, &UA621969},
	{SG98181, &UA622183},
	{SG98182, &UA622356},
	{SG98183, &UA1895774},
	{SG98184, &UA2276817},
	{SG98185, &UA622215},
	{NULL}};
const Segmentdata DItunes[] = {
	{SG97745, &UA599835},
	{SG97746, &UA901230},
	{SG97747, &UA1951216},
	{SG97748, &UA2040394},
	{SG97749, &UA2041301},
	{SG97750, &UA2041430},
	{SG97751, &UA2042107},
	{SG97752, &UA2043648},
	{SG97753, &UA2203601},
	{SG97754, &UA2204029},
	{SG97755, &UA2204031},
	{SG97756, &UA2204145},
	{SG97757, &UA2204146},
	{SG97758, &UA2204300},
	{SG97759, &UA2204812},
	{SG97760, &UA2205401},
	{SG97761, &UA2242325},
	{SG97762, &UA2242361},
	{SG97763, &UA2346946},
	{SG97764, &UA2347195},
	{SG97765, &UA2383843},
	{SG97817, &UA184734},
	{SG97818, &UA664590},
	{SG97819, &UA742783},
	{SG97820, &UA1873621},
	{SG97821, &UA2041866},
	{SG97822, &UA2127744},
	{SG97823, &UA2370018},
	{SG97824, &UA2393623},
	{SG97825, &UA2428939},
	{SG97831, &UA828945},
	{SG97832, &UA867446},
	{SG97833, &UA1958796},
	{SG97834, &UA2042637},
	{SG97835, &UA2203593},
	{SG97836, &UA2204017},
	{SG97837, &UA2204020},
	{SG97838, &UA2204813},
	{SG97839, &UA2204833},
	{SG97840, &UA2205233},
	{SG97841, &UA2232967},
	{SG97842, &UA2242312},
	{SG97843, &UA2596532},
	{SG97851, &UA627139},
	{SG97852, &UA1994416},
	{SG97853, &UA2041298},
	{SG97854, &UA2042964},
	{SG97855, &UA2318988},
	{SG97856, &UA2322656},
	{SG97857, &UA236553},
	{SG97858, &UA420290},
	{SG97859, &UA440968},
	{SG97860, &UA692974},
	{SG97861, &UA901952},
	{SG97862, &UA2199847},
	{SG97863, &UA2405939},
	{SG97864, &UA2429048},
	{SG97877, &UA912426},
	{SG97878, &UA2042324},
	{SG97879, &UA2203505},
	{SG97880, &UA2420882},
	{SG97881, &UA2428905},
	{SG97882, &UA2566147},
	{SG97883, &UA2784016},
	{SG97884, &UA3096182},
	{SG97899, &UA2607004},
	{SG97900, &UA2607261},
	{SG97901, &UA2668890},
	{SG97902, &UA2669785},
	{SG97903, &UA2670301},
	{SG97904, &UA2702687},
	{SG97905, &UA2712146},
	{SG97906, &UA2713730},
	{SG97907, &UA2714842},
	{SG97908, &UA2738283},
	{SG97909, &UA2739831},
	{SG97910, &UA2750327},
	{SG97911, &UA2752701},
	{SG97912, &UA2795131},
	{SG97913, &UA2799943},
	{SG97914, &UA2878996},
	{SG97915, &UA2936119},
	{SG97916, &UA2956928},
	{SG97917, &UA2989943},
	{SG97918, &UA563407},
	{SG97919, &UA564149},
	{SG97920, &UA643817},
	{SG97921, &UA805836},
	{SG97922, &UA877115},
	{SG97923, &UA1993978},
	{SG97924, &UA2141247},
	{SG97925, &UA2269048},
	{SG97926, &UA2269102},
	{SG97927, &UA2334742},
	{SG97928, &UA2654167},
	{SG97929, &UA2669622},
	{SG97930, &UA2670104},
	{SG97931, &UA2683147},
	{SG97932, &UA2713662},
	{SG97933, &UA2733483},
	{SG97934, &UA2767147},
	{SG97935, &UA2861988},
	{SG97936, &UA2935657},
	{SG97937, &UA2935734},
	{SG97938, &UA2954675},
	{SG97939, &UA2989642},
	{SG97940, &UA2989769},
	{SG97941, &UA2990128},
	{SG97942, &UA2990899},
	{SG97943, &UA2991495},
	{SG97944, &UA2992254},
	{SG97945, &UA3047084},
	{SG97946, &UA3072555},
	{SG97947, &UA3087245},
	{SG97972, &UA437923},
	{SG97973, &UA452914},
	{SG97974, &UA457596},
	{SG97975, &UA576564},
	{SG97976, &UA611155},
	{SG97977, &UA636012},
	{SG97978, &UA901099},
	{SG97979, &UA905289},
	{SG97980, &UA912982},
	{SG97981, &UA1979141},
	{SG97982, &UA1997035},
	{SG97983, &UA2001886},
	{SG97984, &UA2007595},
	{SG97985, &UA2203586},
	{SG97986, &UA2205043},
	{SG97987, &UA2232332},
	{SG97988, &UA2242324},
	{SG97989, &UA2242355},
	{SG97990, &UA2363160},
	{SG97991, &UA2156452},
	{SG97992, &UA2202936},
	{SG97993, &UA2356590},
	{SG97994, &UA2356684},
	{SG97995, &UA2356979},
	{SG97996, &UA2362634},
	{SG97997, &UA2362727},
	{SG97998, &UA2367887},
	{SG97999, &UA2385899},
	{SG98000, &UA2386202},
	{SG98001, &UA2425091},
	{SG98002, &UA2456527},
	{SG98003, &UA2481003},
	{SG98004, &UA2486104},
	{SG98005, &UA2593348},
	{SG98006, &UA2727321},
	{SG98014, &UA547184},
	{SG98015, &UA576829},
	{SG98016, &UA627869},
	{SG98017, &UA628940},
	{SG98018, &UA631586},
	{SG98019, &UA635229},
	{SG98020, &UA641108},
	{SG98021, &UA642336},
	{SG98022, &UA643327},
	{SG98023, &UA645675},
	{SG98024, &UA645927},
	{SG98025, &UA650540},
	{SG98026, &UA658335},
	{SG98027, &UA672255},
	{SG98028, &UA678428},
	{SG98029, &UA691491},
	{SG98030, &UA706525},
	{SG98031, &UA795795},
	{SG98032, &UA811586},
	{SG98033, &UA895323},
	{SG98034, &UA2126296},
	{SG98035, &UA2242342},
	{SG98036, &UA2242343},
	{SG98037, &UA2267737},
	{SG98038, &UA2267942},
	{SG98039, &UA2268873},
	{SG98040, &UA2362660},
	{SG98041, &UA2392404},
	{SG98042, &UA2426961},
	{SG98043, &UA2504049},
	{SG98045, &UA2607048},
	{SG98046, &UA2668545},
	{SG98047, &UA2669003},
	{SG98048, &UA2670857},
	{SG98049, &UA2670961},
	{SG98050, &UA2672150},
	{SG98051, &UA2702641},
	{SG98052, &UA2704329},
	{SG98053, &UA2711383},
	{SG98054, &UA2733565},
	{SG98055, &UA2750603},
	{SG98056, &UA2795827},
	{SG98057, &UA901760},
	{SG98058, &UA903698},
	{SG98059, &UA2203512},
	{SG98060, &UA2204320},
	{SG98061, &UA2234078},
	{SG98062, &UA2562712},
	{SG98063, &UA2562773},
	{SG98064, &UA2703013},
	{SG98077, &UA2362675},
	{SG98078, &UA2385885},
	{SG98079, &UA2385884},
	{SG98080, &UA2385948},
	{SG98081, &UA2456507},
	{SG98082, &UA2456528},
	{SG98083, &UA2481354},
	{SG98084, &UA2485171},
	{SG98085, &UA2527379},
	{SG98086, &UA2595805},
	{SG98087, &UA2654171},
	{SG98127, &UA2672496},
	{SG98128, &UA2705007},
	{SG98129, &UA2712665},
	{SG98130, &UA2780423},
	{SG98131, &UA2794569},
	{SG98132, &UA2989886},
	{SG98168, &UA2232110},
	{SG98176, &UA639141},
	{SG98186, &UA1888688},
	{SG98187, &UA435612},
	{SG98188, &UA491233},
	{SG98189, &UA628292},
	{SG98190, &UA901838},
	{SG98191, &UA2436468},
	{SG98192, &UA899089},
	{NULL}};
const Device *Ddataaccessd[] = {
	&UA2488292,
	&UA2492637,
	&UA2492642,
	&UA2492648,
	&UA2516520,
	&UA2532508,
	&UA2585296,
	&UA2709270,
	&UA599533,
	&UA601551,
	&UA623871,
	&UA630370,
	&UA2043782,
	&UA2043784,
	&UA2242335,
	&UA2351805,
	&UA2386014,
	&UA2437562,
	&UA2596573,
	&UA2872325,
	&UA3059571,
	&UA3109451,
	&UA3130593,
	NULL};
const Device *DPodcasts[] = {
	&UA909046,
	&UA1997057,
	&UA2010429,
	&UA2203304,
	&UA2205542,
	&UA2261881,
	&UA2580140,
	&UA2602050,
	&UA3033187,
	&UA3068262,
	NULL};
const Device *DTinyTinyRSS[] = {
	&UA3020562,
	&UA3110826,
	&UA563382,
	&UA623120,
	&UA1963249,
	&UA2151603,
	&UA2262476,
	&UA2308256,
	&UA2362486,
	&UA2461018,
	&UA2567488,
	&UA2584963,
	&UA2602585,
	&UA2671762,
	&UA2705800,
	&UA2752956,
	&UA2870122,
	NULL};
const Device *DAmazonCloudFront[] = {
	&UA439486,
	NULL};
const Device *DWada[] = {
	&UA902625,
	NULL};
const Device *DFeedDemon[] = {
	&UA97138,
	&UA400224,
	&UA122481,
	&UA140010,
	&UA162461,
	&UA382343,
	&UA892470,
	&UA2883127,
	&UA152092,
	&UA525615,
	&UA887325,
	&UA2871486,
	NULL};
const Device *Dpythonrequests[] = {
	&UA397437,
	&UA412425,
	&UA428350,
	&UA583725,
	&UA603172,
	&UA613088,
	&UA615807,
	&UA656247,
	&UA863668,
	&UA865558,
	&UA900172,
	&UA901790,
	&UA911162,
	&UA1937571,
	&UA1948389,
	&UA2404703,
	&UA2408166,
	&UA2487515,
	&UA2580583,
	&UA2672796,
	&UA2674574,
	&UA2717130,
	&UA2740619,
	&UA2756425,
	&UA2900781,
	&UA2996072,
	&UA3018859,
	&UA3034065,
	&UA3084037,
	&UA3092330,
	&UA2665233,
	&UA3049816,
	&UA2710975,
	&UA3049648,
	&UA2872126,
	&UA3074501,
	NULL};
const Device *DRSSInclude[] = {
	&UA609257,
	NULL};
const Device *Dcurl[] = {
	&UA415917,
	&UA641447,
	&UA703611,
	&UA895006,
	&UA900276,
	&UA93056,
	&UA105833,
	&UA114878,
	&UA137730,
	&UA213462,
	&UA232959,
	&UA377045,
	&UA390194,
	&UA437096,
	&UA454771,
	&UA664501,
	&UA694889,
	&UA1979458,
	&UA2364025,
	&UA2704182,
	&UA391342,
	&UA511680,
	&UA535340,
	&UA715686,
	&UA729928,
	&UA768060,
	&UA923808,
	&UA1902189,
	&UA1949138,
	&UA2308484,
	&UA2517005,
	&UA2570567,
	&UA168414,
	&UA1990783,
	&UA2799879,
	&UA3109377,
	&UA653492,
	&UA1884687,
	&UA2131292,
	&UA2462919,
	&UA2299052,
	NULL};
const Device *DPulp[] = {
	&UA244126,
	&UA440331,
	&UA449968,
	&UA452137,
	&UA900996,
	&UA2511361,
	&UA912261,
	NULL};
const Device *DWin98[] = {
	&UA92754,
	&UA111232,
	&UA117500,
	&UA120456,
	&UA121206,
	&UA121444,
	&UA131221,
	&UA138926,
	&UA151970,
	&UA273674,
	NULL};
const Segmentdata DAppEngine[] = {
	{SG98193, &UA2453624},
	{SG98194, &UA2015970},
	{SG98206, &UA251960},
	{SG98207, &UA571141},
	{SG98208, &UA649669},
	{SG98209, &UA900504},
	{SG98210, &UA1896884},
	{SG98211, &UA1964023},
	{SG98212, &UA2005295},
	{SG98223, &UA1950569},
	{SG98224, &UA2492471},
	{SG98225, &UA2513073},
	{SG98226, &UA2708739},
	{SG98227, &UA622974},
	{SG98228, &UA756179},
	{SG98229, &UA2510040},
	{SG98230, &UA2537366},
	{SG98231, &UA2680549},
	{SG98248, &UA263547},
	{SG98258, &UA2242296},
	{SG98276, &UA2855770},
	{SG98402, &UA2502343},
	{NULL}};
const Segmentdata DOutlook[] = {
	{SG98195, &UA96799},
	{SG98196, &UA133857},
	{SG98197, &UA152874},
	{SG98198, &UA218627},
	{SG98199, &UA244787},
	{SG98200, &UA649540},
	{SG98201, &UA1995918},
	{SG98202, &UA2266829},
	{SG98203, &UA2405609},
	{SG98204, &UA2974969},
	{SG98205, &UA3080043},
	{SG98213, &UA106150},
	{SG98214, &UA157776},
	{SG98215, &UA244781},
	{SG98216, &UA303021},
	{SG98217, &UA555293},
	{SG98218, &UA1910616},
	{SG98219, &UA2010474},
	{SG98220, &UA2433775},
	{SG98221, &UA3021923},
	{SG98222, &UA3083629},
	{SG98232, &UA2154482},
	{SG98233, &UA2350893},
	{SG98234, &UA2454904},
	{SG98235, &UA2482180},
	{SG98236, &UA2580175},
	{SG98237, &UA2750061},
	{SG98238, &UA2948032},
	{SG98239, &UA3110639},
	{SG98240, &UA2154993},
	{SG98241, &UA2377406},
	{SG98242, &UA2503486},
	{SG98243, &UA2580018},
	{SG98244, &UA2596993},
	{SG98245, &UA2753393},
	{SG98246, &UA2947520},
	{SG98247, &UA3094999},
	{SG98249, &UA1904101},
	{SG98250, &UA2242565},
	{SG98251, &UA2242621},
	{SG98252, &UA2391959},
	{SG98253, &UA2432111},
	{SG98254, &UA2441055},
	{SG98255, &UA2802348},
	{SG98256, &UA3023762},
	{SG98257, &UA3083105},
	{SG98261, &UA123136},
	{SG98262, &UA161736},
	{SG98263, &UA182932},
	{SG98264, &UA246552},
	{SG98265, &UA452290},
	{SG98266, &UA2176671},
	{SG98267, &UA2483118},
	{SG98268, &UA3111005},
	{NULL}};
const Segmentdata DOperaTwo[] = {
	{SG98259, &UA2186586},
	{SG98260, &UA2341635},
	{SG98269, &UA2572214},
	{SG98270, &UA2701849},
	{SG98271, &UA2708387},
	{SG98272, &UA2768492},
	{SG98273, &UA908537},
	{SG98274, &UA1961373},
	{SG98275, &UA2004648},
	{SG98277, &UA3012806},
	{SG98278, &UA2468814},
	{SG98279, &UA2468928},
	{SG98280, &UA2368244},
	{SG98281, &UA2404751},
	{SG98282, &UA2428485},
	{SG98283, &UA723717},
	{SG98284, &UA2164319},
	{SG98285, &UA2324173},
	{NULL}};
const Segmentdata DIUC[] = {
	{SG98286, &UA2203598},
	{SG98287, &UA2204019},
	{SG98288, &UA2235609},
	{SG98289, &UA2350044},
	{SG98290, &UA2415222},
	{SG98291, &UA904513},
	{SG98292, &UA911490},
	{SG98293, &UA2314468},
	{SG98294, &UA2321841},
	{SG98295, &UA2507243},
	{SG98296, &UA2233410},
	{SG98297, &UA2696996},
	{SG98298, &UA2843622},
	{SG98299, &UA2487833},
	{SG98300, &UA2562007},
	{SG98301, &UA2565379},
	{SG98302, &UA2582066},
	{SG98303, &UA2242365},
	{SG98304, &UA2356276},
	{SG98305, &UA2436625},
	{SG98306, &UA2838301},
	{SG98307, &UA2351524},
	{SG98308, &UA2402000},
	{SG98309, &UA2403026},
	{SG98310, &UA2242291},
	{SG98311, &UA2303321},
	{SG98312, &UA2403507},
	{SG98313, &UA2423017},
	{SG98314, &UA2466978},
	{SG98315, &UA2845661},
	{SG98316, &UA1995104},
	{SG98317, &UA1995418},
	{SG98318, &UA1996003},
	{SG98319, &UA2166964},
	{SG98320, &UA882439},
	{SG98321, &UA886270},
	{SG98322, &UA894228},
	{SG98323, &UA2175308},
	{SG98324, &UA2219079},
	{SG98325, &UA2588038},
	{SG98326, &UA1987422},
	{SG98327, &UA2424250},
	{SG98328, &UA2555361},
	{SG98329, &UA2746952},
	{SG98330, &UA2511140},
	{SG98331, &UA2533254},
	{SG98332, &UA673120},
	{SG98333, &UA707286},
	{SG98334, &UA750402},
	{SG98335, &UA787792},
	{SG98336, &UA1906416},
	{SG98337, &UA1943506},
	{SG98338, &UA1965990},
	{SG98339, &UA1978474},
	{SG98340, &UA2158694},
	{SG98341, &UA2301108},
	{SG98342, &UA2302234},
	{SG98343, &UA2304305},
	{SG98344, &UA2308469},
	{SG98345, &UA2324705},
	{SG98346, &UA2337121},
	{SG98347, &UA2414711},
	{SG98348, &UA2426227},
	{SG98349, &UA2428038},
	{SG98350, &UA2482562},
	{SG98351, &UA2543406},
	{SG98352, &UA2545339},
	{SG98353, &UA2649167},
	{SG98354, &UA2693194},
	{SG98355, &UA2775377},
	{SG98356, &UA2787126},
	{SG98357, &UA2811628},
	{SG98358, &UA2820948},
	{SG98359, &UA2830413},
	{SG98360, &UA2839820},
	{SG98361, &UA2952806},
	{SG98362, &UA2980253},
	{SG98363, &UA2981281},
	{SG98364, &UA2983321},
	{SG98365, &UA3043297},
	{SG98366, &UA3098398},
	{SG98367, &UA3099445},
	{SG98368, &UA2366479},
	{SG98369, &UA2356919},
	{SG98370, &UA2367183},
	{SG98371, &UA676219},
	{SG98372, &UA803358},
	{SG98373, &UA822529},
	{SG98374, &UA1972356},
	{SG98375, &UA2156423},
	{SG98376, &UA2393276},
	{SG98377, &UA2234293},
	{SG98378, &UA2482669},
	{SG98379, &UA2503787},
	{SG98380, &UA2242332},
	{SG98381, &UA2367340},
	{SG98382, &UA2383572},
	{SG98383, &UA2383915},
	{SG98384, &UA2455608},
	{SG98385, &UA2351620},
	{SG98386, &UA2351759},
	{SG98387, &UA381822},
	{SG98388, &UA706644},
	{SG98389, &UA755526},
	{SG98390, &UA801863},
	{SG98391, &UA2157558},
	{SG98392, &UA2360608},
	{SG98393, &UA2361555},
	{SG98394, &UA2422176},
	{SG98395, &UA2457039},
	{SG98396, &UA2491517},
	{SG98397, &UA2491523},
	{SG98398, &UA2353045},
	{SG98399, &UA2384051},
	{SG98400, &UA2424004},
	{SG98401, &UA2456262},
	{SG98403, &UA2383732},
	{SG98404, &UA2456573},
	{SG98405, &UA2385240},
	{SG98406, &UA2481500},
	{SG98407, &UA2503218},
	{SG98408, &UA2427066},
	{SG98409, &UA2466664},
	{SG98410, &UA2423145},
	{SG98411, &UA2467236},
	{SG98412, &UA2384196},
	{SG98413, &UA2385100},
	{SG98414, &UA2483181},
	{SG98415, &UA2422261},
	{SG98416, &UA2466284},
	{SG98417, &UA2483006},
	{SG98418, &UA2384582},
	{SG98419, &UA2402194},
	{SG98420, &UA3118642},
	{SG98421, &UA2366543},
	{SG98422, &UA2424091},
	{SG98423, &UA2403471},
	{SG98424, &UA2356940},
	{SG98425, &UA2423551},
	{SG98426, &UA2384119},
	{SG98427, &UA2466427},
	{SG98428, &UA2455715},
	{SG98429, &UA2457087},
	{SG98430, &UA2425253},
	{SG98431, &UA2843657},
	{SG98432, &UA2421585},
	{SG98433, &UA2192568},
	{SG98434, &UA2422244},
	{SG98435, &UA249860},
	{SG98436, &UA353111},
	{SG98437, &UA1925089},
	{SG98438, &UA2215432},
	{SG98439, &UA2334257},
	{SG98440, &UA2456568},
	{SG98441, &UA2457076},
	{SG98442, &UA2695861},
	{SG98443, &UA2425076},
	{SG98444, &UA676544},
	{SG98445, &UA718528},
	{SG98446, &UA798348},
	{SG98447, &UA1893404},
	{SG98448, &UA2408930},
	{SG98449, &UA2742550},
	{SG98450, &UA2457584},
	{SG98451, &UA2504642},
	{SG98452, &UA2669253},
	{SG98453, &UA2402035},
	{SG98454, &UA2481301},
	{SG98455, &UA2420563},
	{SG98456, &UA2466894},
	{SG98457, &UA2426869},
	{SG98458, &UA1957945},
	{SG98459, &UA2683790},
	{SG98460, &UA2402049},
	{SG98461, &UA2842578},
	{SG98462, &UA2466222},
	{SG98463, &UA2482884},
	{SG98464, &UA1903698},
	{SG98465, &UA2154290},
	{SG98466, &UA2394129},
	{SG98467, &UA2496140},
	{SG98468, &UA2384727},
	{SG98469, &UA2428055},
	{SG98470, &UA436721},
	{SG98471, &UA1895816},
	{SG98472, &UA1911756},
	{SG98473, &UA2424345},
	{SG98550, &UA567638},
	{SG98551, &UA2402367},
	{SG98552, &UA2467246},
	{SG98553, &UA2481114},
	{SG98554, &UA2594812},
	{SG98555, &UA2456681},
	{SG98556, &UA2467137},
	{SG98557, &UA2505012},
	{SG98558, &UA1934860},
	{SG98559, &UA2426998},
	{SG98560, &UA2768448},
	{SG98561, &UA2528405},
	{SG98562, &UA2455027},
	{SG98563, &UA2527090},
	{SG98564, &UA2482268},
	{SG98565, &UA2466792},
	{SG98566, &UA2221509},
	{SG98567, &UA2453446},
	{SG98568, &UA3070383},
	{SG98575, &UA2505078},
	{SG98576, &UA2467026},
	{SG98577, &UA2654278},
	{SG98578, &UA2594899},
	{SG98579, &UA2653379},
	{SG98580, &UA2422869},
	{SG98581, &UA330784},
	{SG98582, &UA2483376},
	{SG98583, &UA1970076},
	{SG98584, &UA1916067},
	{SG98585, &UA2481630},
	{SG98586, &UA2426588},
	{SG98587, &UA2664991},
	{SG98588, &UA2421095},
	{SG98589, &UA2424665},
	{SG98590, &UA685726},
	{SG98591, &UA692716},
	{SG98592, &UA2483049},
	{SG98593, &UA1978227},
	{SG98594, &UA2466583},
	{SG98595, &UA2425492},
	{SG98596, &UA193989},
	{SG98597, &UA324217},
	{SG98598, &UA553840},
	{SG98599, &UA555109},
	{SG98600, &UA2495060},
	{SG98601, &UA2503830},
	{SG98602, &UA2727035},
	{SG98603, &UA2696051},
	{SG98604, &UA2730085},
	{SG98605, &UA671997},
	{SG98606, &UA2480827},
	{SG98607, &UA2657452},
	{SG98608, &UA2163988},
	{SG98609, &UA1931153},
	{SG98610, &UA2401759},
	{SG98611, &UA2373409},
	{SG98612, &UA2482676},
	{SG98613, &UA2749960},
	{SG98614, &UA2480718},
	{SG98615, &UA2553252},
	{SG98616, &UA2889538},
	{SG98617, &UA2519578},
	{SG98618, &UA630331},
	{SG98619, &UA2655049},
	{SG98620, &UA2664533},
	{SG98621, &UA2310587},
	{SG98622, &UA1946677},
	{SG98623, &UA2695663},
	{SG98624, &UA2543287},
	{SG98625, &UA2482001},
	{SG98626, &UA2595707},
	{SG98627, &UA2729421},
	{SG98628, &UA2393837},
	{SG98629, &UA1961369},
	{SG98630, &UA2332200},
	{SG98631, &UA2547876},
	{SG98632, &UA2696974},
	{SG98633, &UA2481781},
	{SG98634, &UA2655763},
	{SG98635, &UA2763913},
	{SG98636, &UA632725},
	{SG98637, &UA751397},
	{SG98638, &UA2505386},
	{SG98641, &UA2577824},
	{SG98642, &UA2410799},
	{SG98643, &UA2555664},
	{SG98644, &UA2759847},
	{SG98645, &UA2985054},
	{SG98646, &UA2307057},
	{SG98647, &UA2780792},
	{SG98648, &UA2865061},
	{SG98649, &UA2466267},
	{SG98650, &UA2481628},
	{SG98651, &UA814983},
	{SG98652, &UA2502597},
	{SG98654, &UA2844277},
	{SG98655, &UA2661171},
	{SG98656, &UA2504692},
	{SG98657, &UA2372844},
	{SG98658, &UA2497146},
	{SG98659, &UA648855},
	{SG98660, &UA2557047},
	{SG98661, &UA2651269},
	{SG98662, &UA3118048},
	{SG98663, &UA756355},
	{SG98664, &UA784991},
	{SG98665, &UA800685},
	{SG98671, &UA2527754},
	{SG98672, &UA2594720},
	{SG98673, &UA2595224},
	{SG98674, &UA2595257},
	{SG98675, &UA2934629},
	{SG98676, &UA1987069},
	{SG98677, &UA2413124},
	{SG98678, &UA2526640},
	{SG98679, &UA2215585},
	{SG98680, &UA2856893},
	{SG98681, &UA806381},
	{SG98682, &UA549797},
	{SG98683, &UA1891664},
	{SG98684, &UA2527696},
	{SG98685, &UA2650288},
	{SG98686, &UA1907666},
	{SG98687, &UA2952024},
	{SG98688, &UA2314809},
	{SG98689, &UA2160883},
	{SG98691, &UA2526529},
	{SG98692, &UA2696167},
	{SG98693, &UA2787614},
	{SG98694, &UA3089053},
	{SG98695, &UA2145859},
	{SG98696, &UA2162477},
	{SG98697, &UA2288062},
	{SG98698, &UA2578881},
	{SG98699, &UA2746379},
	{SG98700, &UA2763147},
	{SG98701, &UA2993513},
	{SG98702, &UA2695497},
	{SG98703, &UA2992088},
	{SG98704, &UA2150478},
	{SG98705, &UA2748214},
	{SG98706, &UA2790901},
	{SG98707, &UA2844593},
	{SG98708, &UA543379},
	{SG98709, &UA747294},
	{SG98710, &UA2494522},
	{SG98711, &UA2519821},
	{SG98712, &UA2775482},
	{SG98713, &UA2775780},
	{SG98714, &UA2848014},
	{SG98715, &UA2332936},
	{SG98716, &UA2481521},
	{SG98717, &UA2837610},
	{SG98718, &UA3046472},
	{SG98719, &UA2555279},
	{SG98720, &UA2653719},
	{SG98721, &UA2791024},
	{SG98722, &UA2791242},
	{SG98723, &UA2792362},
	{SG98724, &UA2985185},
	{SG98725, &UA2993451},
	{SG98726, &UA538077},
	{SG98727, &UA738615},
	{SG98728, &UA2810941},
	{SG98729, &UA2640253},
	{SG98730, &UA2694041},
	{SG98731, &UA2951814},
	{SG98732, &UA2747119},
	{SG98733, &UA2778076},
	{SG98734, &UA2840451},
	{SG98735, &UA2847412},
	{SG98736, &UA2981570},
	{SG98737, &UA3102889},
	{SG98738, &UA446155},
	{SG98739, &UA760047},
	{SG98740, &UA2132261},
	{SG98741, &UA555135},
	{SG98742, &UA2147748},
	{SG98743, &UA2841850},
	{SG98744, &UA2842877},
	{SG98745, &UA2949186},
	{SG98746, &UA2984429},
	{SG98747, &UA2473213},
	{SG98748, &UA2684027},
	{SG98749, &UA2694784},
	{SG98750, &UA2843419},
	{SG98751, &UA3042916},
	{SG98752, &UA3118019},
	{SG98753, &UA3120390},
	{SG98754, &UA3120683},
	{SG98755, &UA2318267},
	{SG98756, &UA2839941},
	{SG98757, &UA2842298},
	{SG98758, &UA2844071},
	{SG98759, &UA2841936},
	{SG98760, &UA2844514},
	{SG98761, &UA2775340},
	{SG98762, &UA2810519},
	{SG98763, &UA872410},
	{SG98764, &UA2440215},
	{SG98765, &UA2641052},
	{SG98766, &UA448332},
	{SG98767, &UA1874324},
	{SG98768, &UA2148631},
	{SG98769, &UA2810173},
	{SG98770, &UA2811772},
	{SG98771, &UA2146295},
	{SG98772, &UA557019},
	{SG98773, &UA2139763},
	{SG98774, &UA2303804},
	{SG98775, &UA2302400},
	{SG98776, &UA1884681},
	{SG98778, &UA665416},
	{SG98779, &UA1935693},
	{SG98780, &UA2680956},
	{SG98781, &UA459156},
	{SG98782, &UA2242266},
	{SG98783, &UA2811742},
	{SG98784, &UA1903684},
	{SG98785, &UA459159},
	{SG98786, &UA459146},
	{SG98787, &UA459153},
	{SG98789, &UA2241355},
	{NULL}};
const Segmentdata Dserverbag[] = {
	{SG98572, &UA2148161},
	{SG98573, &UA2163702},
	{SG98574, &UA2586884},
	{SG98639, &UA2308683},
	{SG98640, &UA2364275},
	{SG98653, &UA2734425},
	{SG98690, &UA2607095},
	{SG98777, &UA2404071},
	{SG98790, &UA2410290},
	{SG98791, &UA2350847},
	{SG98802, &UA2703008},
	{SG98803, &UA2425961},
	{NULL}};
const Device *DNumbers[] = {
	&UA2933298,
	&UA2949495,
	&UA2949633,
	&UA2949721,
	&UA2949852,
	&UA2949889,
	&UA2979320,
	&UA2979773,
	&UA2979850,
	NULL};
const Segmentdata Djigbrowser[] = {
	{SG98474, &UA373759},
	{SG98475, &UA374549},
	{SG98476, &UA387696},
	{SG98477, &UA388938},
	{SG98478, &UA390151},
	{SG98479, &UA414976},
	{SG98480, &UA417128},
	{SG98481, &UA420678},
	{SG98482, &UA439794},
	{SG98483, &UA443793},
	{SG98484, &UA489131},
	{SG98485, &UA492891},
	{SG98486, &UA495844},
	{SG98487, &UA497423},
	{SG98488, &UA502200},
	{SG98489, &UA505791},
	{SG98490, &UA507146},
	{SG98491, &UA518617},
	{SG98492, &UA520203},
	{SG98493, &UA520946},
	{SG98494, &UA522210},
	{SG98495, &UA522719},
	{SG98496, &UA525065},
	{SG98497, &UA531565},
	{SG98498, &UA541106},
	{SG98499, &UA541286},
	{SG98500, &UA541337},
	{SG98501, &UA541392},
	{SG98502, &UA543728},
	{SG98503, &UA547725},
	{SG98504, &UA552335},
	{SG98505, &UA562998},
	{SG98506, &UA563679},
	{SG98507, &UA643520},
	{SG98508, &UA652711},
	{SG98509, &UA664179},
	{SG98510, &UA687433},
	{SG98511, &UA694550},
	{SG98512, &UA700113},
	{SG98513, &UA707416},
	{SG98514, &UA708009},
	{SG98515, &UA718582},
	{SG98516, &UA719956},
	{SG98517, &UA727936},
	{SG98518, &UA730411},
	{SG98519, &UA739705},
	{SG98520, &UA746886},
	{SG98521, &UA752335},
	{SG98522, &UA765433},
	{SG98523, &UA769704},
	{SG98524, &UA774326},
	{SG98525, &UA778747},
	{SG98526, &UA781553},
	{SG98527, &UA785976},
	{SG98528, &UA789099},
	{SG98529, &UA792551},
	{SG98530, &UA796234},
	{SG98531, &UA803913},
	{SG98532, &UA813775},
	{SG98533, &UA815361},
	{SG98534, &UA852096},
	{SG98535, &UA1850350},
	{SG98536, &UA1856944},
	{SG98537, &UA1858888},
	{SG98538, &UA1860555},
	{SG98539, &UA1862616},
	{SG98540, &UA1866929},
	{SG98541, &UA1872964},
	{SG98542, &UA1873890},
	{SG98543, &UA1874221},
	{SG98544, &UA1876365},
	{SG98545, &UA1880724},
	{SG98546, &UA1884127},
	{SG98547, &UA1885072},
	{SG98548, &UA1888080},
	{SG98549, &UA1893699},
	{SG99080, &UA790652},
	{SG99138, &UA829050},
	{SG99161, &UA523208},
	{SG99200, &UA1890005},
	{SG99231, &UA1885326},
	{SG99234, &UA523440},
	{SG99235, &UA1882423},
	{NULL}};
const Segmentdata DMicromax[] = {
	{SG98838, &UA2230512},
	{SG98839, &UA2879367},
	{SG98840, &UA395841},
	{SG98841, &UA703377},
	{SG98844, &UA2383478},
	{SG98846, &UA160544},
	{SG98847, &UA2424591},
	{SG98848, &UA2958453},
	{SG98860, &UA379110},
	{SG98861, &UA2242582},
	{SG98883, &UA230042},
	{SG98885, &UA2168485},
	{SG98886, &UA646338},
	{SG98887, &UA1963099},
	{SG98923, &UA1944768},
	{SG98924, &UA821702},
	{SG98925, &UA2242617},
	{SG98927, &UA90738},
	{SG98928, &UA305549},
	{SG98929, &UA1966847},
	{SG98930, &UA157449},
	{SG98932, &UA90085},
	{SG98947, &UA1901540},
	{SG98948, &UA301641},
	{SG98962, &UA2722662},
	{SG98973, &UA90734},
	{SG98975, &UA2157120},
	{SG98976, &UA2482078},
	{SG98977, &UA89929},
	{SG98978, &UA401062},
	{SG98979, &UA88652},
	{SG98982, &UA185544},
	{SG98983, &UA90563},
	{SG98984, &UA561836},
	{SG98985, &UA1975436},
	{SG98986, &UA305283},
	{SG98987, &UA167223},
	{SG98988, &UA1942092},
	{SG98989, &UA90735},
	{SG98990, &UA575381},
	{SG98993, &UA90736},
	{SG98994, &UA2154483},
	{SG99004, &UA2410298},
	{SG99007, &UA2786239},
	{SG99008, &UA3041357},
	{SG99009, &UA3084837},
	{SG99010, &UA89879},
	{SG99017, &UA2210950},
	{SG99018, &UA305959},
	{SG99057, &UA538717},
	{SG99082, &UA176621},
	{SG99083, &UA547308},
	{SG99084, &UA2409761},
	{SG99092, &UA1940539},
	{SG99093, &UA2586730},
	{SG99127, &UA2463095},
	{SG99132, &UA2542245},
	{SG99261, &UA3084983},
	{NULL}};
const Segmentdata DKonqueror[] = {
	{SG98569, &UA167655},
	{SG98570, &UA174844},
	{SG98571, &UA253804},
	{SG98804, &UA169529},
	{SG98806, &UA129662},
	{SG98809, &UA253806},
	{SG98810, &UA2746045},
	{SG98811, &UA2349719},
	{SG98814, &UA3015545},
	{SG98825, &UA666625},
	{SG98826, &UA92942},
	{SG98827, &UA105112},
	{SG98828, &UA379607},
	{SG98829, &UA624875},
	{SG98830, &UA570319},
	{SG98831, &UA161793},
	{SG98832, &UA102575},
	{SG98833, &UA105109},
	{SG98834, &UA124998},
	{SG98835, &UA169533},
	{SG98836, &UA253790},
	{SG98842, &UA261659},
	{SG98843, &UA178098},
	{SG98849, &UA524211},
	{SG98850, &UA238596},
	{SG98859, &UA180373},
	{SG98884, &UA640777},
	{SG98899, &UA703239},
	{SG98900, &UA253792},
	{SG98926, &UA169960},
	{SG98966, &UA548994},
	{SG98974, &UA718922},
	{SG98980, &UA631175},
	{SG98981, &UA327774},
	{SG99003, &UA1978823},
	{SG99014, &UA253799},
	{SG99019, &UA430018},
	{SG99023, &UA2211026},
	{SG99090, &UA461162},
	{SG99094, &UA2462594},
	{SG99131, &UA439669},
	{SG99178, &UA2757883},
	{SG99189, &UA2282732},
	{SG99237, &UA2265327},
	{SG99264, &UA2516977},
	{SG99331, &UA449609},
	{SG99360, &UA253800},
	{SG99392, &UA1890180},
	{SG99455, &UA697900},
	{SG99521, &UA253793},
	{NULL}};
const Segmentdata DJava[] = {
	{SG98666, &UA430641},
	{SG98667, &UA439955},
	{SG98668, &UA588687},
	{SG98669, &UA2041931},
	{SG98670, &UA2874015},
	{SG98788, &UA2384838},
	{SG98792, &UA173848},
	{SG98793, &UA1989759},
	{SG98794, &UA2355171},
	{SG98795, &UA903162},
	{SG98796, &UA2833185},
	{SG98797, &UA3013536},
	{SG98798, &UA437930},
	{SG98799, &UA554339},
	{SG98800, &UA830801},
	{SG98801, &UA2042714},
	{SG98805, &UA664105},
	{SG98808, &UA2679255},
	{SG98812, &UA2384799},
	{SG98813, &UA2480735},
	{SG98815, &UA2208249},
	{SG98837, &UA3012284},
	{SG99734, &UA64539},
	{NULL}};
const Segmentdata Dheritrix[] = {
	{SG98807, &UA2563403},
	{SG98816, &UA513282},
	{SG98817, &UA517579},
	{SG98818, &UA631472},
	{SG98819, &UA633522},
	{SG98820, &UA837257},
	{SG98821, &UA845531},
	{SG98822, &UA896441},
	{SG98823, &UA1896729},
	{SG98824, &UA1907045},
	{SG98845, &UA1993289},
	{NULL}};
const Segmentdata DUCWEB[] = {
	{SG98851, &UA2678364},
	{SG98852, &UA2712553},
	{SG98853, &UA2716513},
	{SG98854, &UA2736606},
	{SG98855, &UA2781448},
	{SG98856, &UA2892315},
	{SG98857, &UA2992850},
	{SG98858, &UA2994621},
	{SG98862, &UA2387153},
	{SG98863, &UA2387243},
	{SG98864, &UA2389110},
	{SG98865, &UA2390859},
	{SG98866, &UA2404098},
	{SG98867, &UA2404638},
	{SG98868, &UA2405631},
	{SG98869, &UA2406789},
	{SG98870, &UA2424538},
	{SG98871, &UA2427230},
	{SG98872, &UA2427651},
	{SG98873, &UA2428322},
	{SG98874, &UA2429422},
	{SG98875, &UA2430316},
	{SG98876, &UA2431963},
	{SG98877, &UA2460204},
	{SG98878, &UA2467480},
	{SG98879, &UA2481687},
	{SG98880, &UA2555396},
	{SG98881, &UA2578121},
	{SG98882, &UA2778083},
	{SG98888, &UA2230797},
	{SG98889, &UA2382971},
	{SG98890, &UA2383873},
	{SG98891, &UA2400716},
	{SG98892, &UA2478051},
	{SG98893, &UA2591602},
	{SG98894, &UA2594466},
	{SG98895, &UA2693768},
	{SG98896, &UA2746418},
	{SG98897, &UA2822004},
	{SG98898, &UA2889220},
	{SG98901, &UA2482052},
	{SG98902, &UA2507921},
	{SG98903, &UA2507945},
	{SG98904, &UA2533114},
	{SG98905, &UA2559156},
	{SG98906, &UA2560095},
	{SG98907, &UA2560139},
	{SG98908, &UA2580086},
	{SG98909, &UA2581032},
	{SG98910, &UA2581066},
	{SG98911, &UA2659485},
	{SG98912, &UA2659713},
	{SG98913, &UA2661324},
	{SG98914, &UA2662290},
	{SG98915, &UA2662737},
	{SG98916, &UA2699741},
	{SG98917, &UA2732325},
	{SG98918, &UA2779083},
	{SG98919, &UA2852276},
	{SG98920, &UA2853313},
	{SG98921, &UA3071496},
	{SG98922, &UA3118955},
	{SG98931, &UA2429974},
	{SG98933, &UA2429600},
	{SG98934, &UA2461759},
	{SG98935, &UA2461811},
	{SG98936, &UA2469292},
	{SG98937, &UA2484170},
	{SG98938, &UA2486751},
	{SG98939, &UA2487018},
	{SG98940, &UA2487068},
	{SG98941, &UA2556007},
	{SG98942, &UA2557594},
	{SG98943, &UA2557779},
	{SG98944, &UA2596772},
	{SG98945, &UA2657254},
	{SG98946, &UA2847745},
	{SG98949, &UA2230440},
	{SG98950, &UA2348889},
	{SG98951, &UA2350927},
	{SG98952, &UA2366822},
	{SG98953, &UA2382672},
	{SG98954, &UA2383039},
	{SG98955, &UA2383246},
	{SG98956, &UA2401863},
	{SG98957, &UA2403165},
	{SG98958, &UA2420300},
	{SG98959, &UA2503779},
	{SG98960, &UA2778175},
	{SG98961, &UA2790423},
	{SG98963, &UA2219838},
	{SG98964, &UA2222869},
	{SG98965, &UA2223143},
	{SG98967, &UA2224168},
	{SG98968, &UA2229405},
	{SG98969, &UA2382295},
	{SG98970, &UA2476467},
	{SG98971, &UA2816560},
	{SG98972, &UA2832769},
	{SG98991, &UA2508010},
	{SG98992, &UA3119717},
	{SG98995, &UA862716},
	{SG98996, &UA1956244},
	{SG98997, &UA1985669},
	{SG98998, &UA1994430},
	{SG98999, &UA2156464},
	{SG99000, &UA2166334},
	{SG99001, &UA2394892},
	{SG99002, &UA2950157},
	{SG99005, &UA3022843},
	{SG99006, &UA3063436},
	{SG99011, &UA2201279},
	{SG99012, &UA2203260},
	{SG99013, &UA2419912},
	{SG99015, &UA2975881},
	{SG99016, &UA3057589},
	{SG99021, &UA2718767},
	{SG99022, &UA2737254},
	{SG99024, &UA908797},
	{SG99025, &UA2373911},
	{SG99026, &UA2712934},
	{SG99027, &UA2714644},
	{SG99028, &UA2738006},
	{SG99029, &UA2751464},
	{SG99030, &UA2751638},
	{SG99031, &UA2768423},
	{SG99032, &UA2768502},
	{SG99033, &UA2773431},
	{SG99034, &UA2868224},
	{SG99035, &UA2869314},
	{SG99036, &UA2870360},
	{SG99037, &UA2872181},
	{SG99038, &UA2878690},
	{SG99039, &UA2879247},
	{SG99040, &UA3049219},
	{SG99041, &UA3087752},
	{SG99042, &UA765602},
	{SG99043, &UA791141},
	{SG99044, &UA853548},
	{SG99045, &UA869575},
	{SG99046, &UA1986372},
	{SG99047, &UA2332900},
	{SG99048, &UA2715437},
	{SG99049, &UA2716983},
	{SG99050, &UA2736759},
	{SG99051, &UA2362340},
	{SG99052, &UA2405801},
	{SG99053, &UA2457642},
	{SG99054, &UA2483060},
	{SG99055, &UA2696056},
	{SG99056, &UA3085982},
	{SG99058, &UA2679627},
	{SG99059, &UA2714940},
	{SG99060, &UA2891593},
	{SG99061, &UA2167562},
	{SG99062, &UA2412124},
	{SG99063, &UA2885457},
	{SG99064, &UA2943352},
	{SG99065, &UA3023747},
	{SG99066, &UA3025936},
	{SG99067, &UA3030019},
	{SG99068, &UA3032616},
	{SG99069, &UA3034284},
	{SG99070, &UA3058391},
	{SG99071, &UA3059020},
	{SG99072, &UA3080028},
	{SG99073, &UA3092392},
	{SG99074, &UA3092797},
	{SG99075, &UA3092975},
	{SG99076, &UA3093160},
	{SG99077, &UA3108448},
	{SG99078, &UA3108705},
	{SG99079, &UA2507666},
	{SG99081, &UA2509061},
	{SG99085, &UA2536892},
	{SG99086, &UA3023586},
	{SG99087, &UA2531145},
	{SG99088, &UA2708411},
	{SG99089, &UA2854314},
	{SG99091, &UA2510698},
	{SG99095, &UA2886764},
	{SG99096, &UA2900451},
	{SG99097, &UA2901171},
	{SG99098, &UA2938279},
	{SG99099, &UA2941530},
	{SG99100, &UA2962038},
	{SG99101, &UA3005191},
	{SG99102, &UA3019706},
	{SG99103, &UA2934909},
	{SG99104, &UA2936049},
	{SG99105, &UA2952568},
	{SG99106, &UA2952787},
	{SG99107, &UA2953845},
	{SG99108, &UA2955989},
	{SG99109, &UA2956560},
	{SG99110, &UA2987289},
	{SG99111, &UA2987538},
	{SG99112, &UA2989439},
	{SG99113, &UA2989625},
	{SG99114, &UA2997739},
	{SG99115, &UA3048872},
	{SG99116, &UA3073550},
	{SG99117, &UA3086812},
	{SG99118, &UA3101218},
	{SG99119, &UA2708759},
	{SG99120, &UA2558687},
	{SG99121, &UA2580414},
	{SG99122, &UA2508192},
	{SG99123, &UA2701738},
	{SG99124, &UA2730160},
	{SG99125, &UA2748972},
	{SG99126, &UA2987725},
	{SG99128, &UA3036658},
	{SG99129, &UA3080515},
	{SG99130, &UA3022269},
	{SG99133, &UA2712213},
	{SG99135, &UA2899410},
	{SG99136, &UA3055011},
	{SG99137, &UA2769867},
	{SG99139, &UA2718811},
	{SG99140, &UA2890008},
	{SG99141, &UA2890083},
	{SG99142, &UA2934660},
	{SG99143, &UA2934704},
	{SG99144, &UA2936443},
	{SG99145, &UA2953558},
	{SG99146, &UA2955529},
	{SG99147, &UA2987770},
	{SG99148, &UA2988271},
	{SG99149, &UA2990304},
	{SG99150, &UA2996516},
	{SG99151, &UA3045420},
	{SG99152, &UA3049476},
	{SG99153, &UA3087229},
	{SG99154, &UA3120957},
	{SG99155, &UA3121000},
	{SG99156, &UA2507694},
	{SG99157, &UA2705991},
	{SG99158, &UA2581805},
	{SG99159, &UA632662},
	{SG99160, &UA2712670},
	{SG99162, &UA2944179},
	{SG99163, &UA2958531},
	{SG99164, &UA2972710},
	{SG99165, &UA2972718},
	{SG99166, &UA2974676},
	{SG99167, &UA3004428},
	{SG99168, &UA3009458},
	{SG99169, &UA3012722},
	{SG99170, &UA3056521},
	{SG99171, &UA3057974},
	{SG99172, &UA2349024},
	{SG99173, &UA2653327},
	{SG99174, &UA2974767},
	{SG99175, &UA2532134},
	{SG99176, &UA2669368},
	{SG99177, &UA3020524},
	{SG99179, &UA2204934},
	{SG99180, &UA2974750},
	{SG99181, &UA3080296},
	{SG99182, &UA3115128},
	{SG99183, &UA2536579},
	{SG99184, &UA2764298},
	{SG99185, &UA2898059},
	{SG99186, &UA3023312},
	{SG99187, &UA2511286},
	{SG99188, &UA2508012},
	{SG99190, &UA2537449},
	{SG99191, &UA3074357},
	{SG99192, &UA3103356},
	{SG99193, &UA3125234},
	{SG99194, &UA3125551},
	{SG99195, &UA3023588},
	{SG99196, &UA2580603},
	{SG99197, &UA2661403},
	{SG99198, &UA3021001},
	{SG99199, &UA2784245},
	{SG99201, &UA2975014},
	{SG99202, &UA2898732},
	{SG99205, &UA2533275},
	{SG99226, &UA2488393},
	{SG99227, &UA2580164},
	{SG99228, &UA2580961},
	{SG99229, &UA2872453},
	{SG99230, &UA3021335},
	{SG99232, &UA2601951},
	{SG99233, &UA2535270},
	{SG99236, &UA2799226},
	{SG99238, &UA2386045},
	{SG99239, &UA2419377},
	{SG99240, &UA3027497},
	{SG99241, &UA3029471},
	{SG99242, &UA2565859},
	{SG99243, &UA2580219},
	{SG99244, &UA2779192},
	{SG99245, &UA2851912},
	{SG99246, &UA2857891},
	{SG99247, &UA2934788},
	{SG99248, &UA2952355},
	{SG99249, &UA2953322},
	{SG99250, &UA2986266},
	{SG99251, &UA2988814},
	{SG99252, &UA3045656},
	{SG99253, &UA3045877},
	{SG99254, &UA3071817},
	{SG99255, &UA3086547},
	{SG99256, &UA3087266},
	{SG99257, &UA3100158},
	{SG99258, &UA3100354},
	{SG99259, &UA3119014},
	{SG99260, &UA3119819},
	{SG99262, &UA2738126},
	{SG99263, &UA2783937},
	{SG99265, &UA2716001},
	{SG99266, &UA2664414},
	{SG99267, &UA2936974},
	{SG99268, &UA2954337},
	{SG99269, &UA2956825},
	{SG99270, &UA2958334},
	{SG99271, &UA2986772},
	{SG99272, &UA3049009},
	{SG99273, &UA3056605},
	{SG99274, &UA3075870},
	{SG99275, &UA3087711},
	{SG99276, &UA2715469},
	{SG99277, &UA3020976},
	{SG99278, &UA2530757},
	{SG99279, &UA2558576},
	{SG99280, &UA2227278},
	{SG99281, &UA3033116},
	{SG99282, &UA2511985},
	{SG99283, &UA2717886},
	{SG99284, &UA2530663},
	{SG99285, &UA2986768},
	{SG99286, &UA2772875},
	{SG99287, &UA2422775},
	{SG99288, &UA2552193},
	{SG99289, &UA908350},
	{SG99290, &UA2705030},
	{SG99291, &UA2794034},
	{SG99292, &UA2898274},
	{SG99294, &UA2596759},
	{SG99295, &UA2536056},
	{SG99296, &UA2580188},
	{SG99297, &UA2703054},
	{SG99303, &UA2557832},
	{SG99306, &UA2897548},
	{SG99310, &UA3021448},
	{SG99311, &UA2952460},
	{SG99312, &UA3020868},
	{SG99313, &UA3024190},
	{SG99314, &UA3022605},
	{SG99315, &UA2945603},
	{SG99316, &UA2945609},
	{SG99317, &UA3022379},
	{SG99318, &UA3026883},
	{SG99319, &UA3105736},
	{SG99320, &UA3036544},
	{SG99321, &UA2736680},
	{SG99322, &UA3021262},
	{SG99323, &UA2895412},
	{SG99324, &UA3002712},
	{SG99325, &UA3020936},
	{SG99326, &UA3026172},
	{SG99327, &UA2580245},
	{SG99328, &UA2729987},
	{SG99329, &UA3021455},
	{SG99330, &UA3020937},
	{SG99332, &UA2714930},
	{SG99333, &UA2737330},
	{SG99334, &UA3036411},
	{SG99335, &UA1961987},
	{SG99336, &UA2232461},
	{SG99337, &UA2423521},
	{SG99338, &UA2581070},
	{SG99339, &UA2387123},
	{SG99340, &UA2836431},
	{SG99342, &UA3059414},
	{SG99343, &UA2598673},
	{SG99344, &UA3083669},
	{SG99345, &UA2599323},
	{SG99346, &UA2796769},
	{SG99347, &UA3023818},
	{SG99348, &UA2667111},
	{SG99349, &UA2718752},
	{SG99350, &UA3015917},
	{SG99351, &UA3016388},
	{SG99352, &UA3017572},
	{SG99353, &UA3027498},
	{SG99354, &UA3028850},
	{SG99355, &UA2798684},
	{SG99356, &UA2571061},
	{SG99357, &UA2704793},
	{SG99358, &UA2898812},
	{SG99359, &UA2736725},
	{SG99361, &UA3061492},
	{SG99362, &UA2945365},
	{SG99363, &UA2704728},
	{SG99364, &UA3021587},
	{SG99365, &UA2356506},
	{SG99366, &UA2390406},
	{SG99367, &UA2456596},
	{SG99368, &UA2457783},
	{SG99369, &UA2504792},
	{SG99370, &UA2535733},
	{SG99371, &UA2564824},
	{SG99372, &UA2536684},
	{SG99373, &UA2731451},
	{SG99374, &UA3059545},
	{SG99375, &UA2424087},
	{SG99376, &UA2437330},
	{SG99377, &UA2747482},
	{SG99378, &UA2660265},
	{SG99379, &UA2851157},
	{SG99380, &UA3029917},
	{SG99381, &UA2558898},
	{SG99382, &UA2783717},
	{SG99383, &UA3023154},
	{SG99384, &UA2962543},
	{SG99385, &UA2424495},
	{SG99386, &UA2457225},
	{SG99387, &UA2482878},
	{SG99388, &UA2843032},
	{SG99389, &UA2558720},
	{SG99390, &UA2894622},
	{SG99391, &UA2945350},
	{SG99393, &UA2580601},
	{SG99394, &UA2596536},
	{SG99395, &UA2700131},
	{SG99396, &UA2730603},
	{SG99397, &UA3091491},
	{SG99398, &UA2867970},
	{SG99399, &UA2428286},
	{SG99400, &UA2506441},
	{SG99401, &UA2697057},
	{SG99402, &UA2599702},
	{SG99403, &UA2712607},
	{SG99404, &UA2797168},
	{SG99405, &UA3060938},
	{SG99406, &UA3080665},
	{SG99407, &UA2662489},
	{SG99408, &UA2870579},
	{SG99410, &UA2562546},
	{SG99411, &UA2799257},
	{SG99412, &UA2580300},
	{SG99413, &UA2734160},
	{SG99414, &UA2894853},
	{SG99415, &UA2944695},
	{SG99416, &UA3023938},
	{SG99417, &UA3024242},
	{SG99418, &UA2531007},
	{SG99419, &UA2705812},
	{SG99420, &UA723636},
	{SG99421, &UA2557833},
	{SG99422, &UA2714843},
	{SG99423, &UA3027683},
	{SG99424, &UA3058299},
	{SG99425, &UA2662735},
	{SG99426, &UA2864142},
	{SG99427, &UA2701123},
	{SG99428, &UA2768874},
	{SG99429, &UA2900726},
	{SG99430, &UA3024053},
	{SG99431, &UA788608},
	{SG99433, &UA2752794},
	{SG99434, &UA2580576},
	{SG99435, &UA2718813},
	{SG99436, &UA3059060},
	{SG99437, &UA2662384},
	{SG99438, &UA2663359},
	{SG99439, &UA3024198},
	{SG99440, &UA2559013},
	{SG99441, &UA2965984},
	{SG99442, &UA3021745},
	{SG99443, &UA2350171},
	{SG99444, &UA2865718},
	{SG99447, &UA2706188},
	{SG99448, &UA2737695},
	{SG99449, &UA2768499},
	{SG99450, &UA3063671},
	{SG99451, &UA2537797},
	{SG99452, &UA2898775},
	{SG99453, &UA2662239},
	{SG99454, &UA2706814},
	{SG99456, &UA2660336},
	{SG99457, &UA2698923},
	{SG99458, &UA3022888},
	{SG99459, &UA2660408},
	{SG99460, &UA3027689},
	{SG99461, &UA2461309},
	{SG99462, &UA2529103},
	{SG99463, &UA2467025},
	{SG99464, &UA2656205},
	{SG99465, &UA2696003},
	{SG99466, &UA2728374},
	{SG99467, &UA2844397},
	{SG99469, &UA2878173},
	{SG99470, &UA2951503},
	{SG99471, &UA2715779},
	{SG99472, &UA2769328},
	{SG99473, &UA2784900},
	{SG99474, &UA2878723},
	{SG99475, &UA2737694},
	{SG99476, &UA2335922},
	{SG99477, &UA2518745},
	{SG99478, &UA3024183},
	{SG99479, &UA2381521},
	{SG99480, &UA3098987},
	{SG99481, &UA3064799},
	{SG99482, &UA2872358},
	{SG99484, &UA2943780},
	{SG99485, &UA3021445},
	{SG99486, &UA3020938},
	{SG99487, &UA2600806},
	{SG99488, &UA2661567},
	{SG99489, &UA2737796},
	{SG99490, &UA3062320},
	{SG99491, &UA3093714},
	{SG99492, &UA2580448},
	{SG99493, &UA2768887},
	{SG99494, &UA2879899},
	{SG99495, &UA3009962},
	{SG99496, &UA2427139},
	{SG99497, &UA2695671},
	{SG99498, &UA2536649},
	{SG99499, &UA2660204},
	{SG99500, &UA3060595},
	{SG99501, &UA2698870},
	{SG99502, &UA3022845},
	{SG99503, &UA3061697},
	{SG99504, &UA2561028},
	{SG99505, &UA2598327},
	{SG99506, &UA2750528},
	{SG99507, &UA3107625},
	{SG99508, &UA3023587},
	{SG99509, &UA3062114},
	{SG99510, &UA3080877},
	{SG99511, &UA2432721},
	{SG99512, &UA2527668},
	{SG99513, &UA2781917},
	{SG99514, &UA2944899},
	{SG99515, &UA3021659},
	{SG99516, &UA2537546},
	{SG99517, &UA2738808},
	{SG99518, &UA2944513},
	{SG99519, &UA3023670},
	{SG99520, &UA2943847},
	{SG99522, &UA2420151},
	{SG99523, &UA2833856},
	{SG99524, &UA2537208},
	{SG99525, &UA2799274},
	{SG99526, &UA3094050},
	{SG99527, &UA2799399},
	{SG99528, &UA2963497},
	{SG99529, &UA2966486},
	{SG99530, &UA3026613},
	{SG99531, &UA2558285},
	{SG99532, &UA3023315},
	{SG99533, &UA2559993},
	{SG99534, &UA2659653},
	{SG99535, &UA2537446},
	{SG99536, &UA2659482},
	{SG99537, &UA2734985},
	{SG99538, &UA2580318},
	{SG99539, &UA2737819},
	{SG99540, &UA3029967},
	{SG99541, &UA3059364},
	{SG99542, &UA2562881},
	{SG99543, &UA2659541},
	{SG99544, &UA2898730},
	{SG99545, &UA2660539},
	{SG99546, &UA2752780},
	{SG99547, &UA3021820},
	{SG99548, &UA3023811},
	{SG99549, &UA3063527},
	{SG99550, &UA2703165},
	{SG99551, &UA2737619},
	{SG99552, &UA2560094},
	{SG99553, &UA2660835},
	{SG99554, &UA2869508},
	{SG99555, &UA2768487},
	{SG99556, &UA2597197},
	{SG99557, &UA2598388},
	{SG99558, &UA3021048},
	{SG99559, &UA3057284},
	{SG99560, &UA3023271},
	{SG99561, &UA3017799},
	{SG99562, &UA3057985},
	{SG99563, &UA2337696},
	{SG99564, &UA2372819},
	{SG99565, &UA3059363},
	{SG99566, &UA2944808},
	{SG99567, &UA3021807},
	{SG99568, &UA3061109},
	{SG99569, &UA3063714},
	{SG99570, &UA2422635},
	{SG99571, &UA2506411},
	{SG99572, &UA2790607},
	{SG99573, &UA2560870},
	{SG99574, &UA2662566},
	{SG99575, &UA3061845},
	{SG99576, &UA2661590},
	{SG99577, &UA2699772},
	{SG99578, &UA3020516},
	{SG99579, &UA3023958},
	{SG99580, &UA3065444},
	{SG99581, &UA2558389},
	{SG99582, &UA2660697},
	{SG99583, &UA2664881},
	{SG99584, &UA2943731},
	{SG99585, &UA3020405},
	{SG99587, &UA2944706},
	{SG99588, &UA3060257},
	{SG99589, &UA2457964},
	{SG99590, &UA2528252},
	{SG99591, &UA2654003},
	{SG99592, &UA2600182},
	{SG99593, &UA2664034},
	{SG99594, &UA2830284},
	{SG99595, &UA2889142},
	{SG99596, &UA2982350},
	{SG99597, &UA3098781},
	{SG99599, &UA3022268},
	{SG99600, &UA3065801},
	{SG99601, &UA3029910},
	{SG99602, &UA3080167},
	{SG99603, &UA2428271},
	{SG99604, &UA2653806},
	{SG99605, &UA2869923},
	{SG99606, &UA2963612},
	{SG99607, &UA3023802},
	{SG99608, &UA2526568},
	{SG99614, &UA2706028},
	{SG99615, &UA2750865},
	{SG99616, &UA2580949},
	{SG99617, &UA2598776},
	{SG99618, &UA2599111},
	{SG99619, &UA2660169},
	{SG99620, &UA3093376},
	{SG99621, &UA2659128},
	{SG99622, &UA2659673},
	{SG99623, &UA2661990},
	{SG99624, &UA2794106},
	{SG99625, &UA2867174},
	{SG99626, &UA2892902},
	{SG99627, &UA3072896},
	{SG99628, &UA2458101},
	{SG99629, &UA2727466},
	{SG99630, &UA2600571},
	{SG99631, &UA2600707},
	{SG99632, &UA2944228},
	{SG99633, &UA3028535},
	{SG99634, &UA3024142},
	{SG99635, &UA2752425},
	{SG99636, &UA2895251},
	{SG99637, &UA3020268},
	{SG99638, &UA3023791},
	{SG99639, &UA2598145},
	{SG99640, &UA2659540},
	{SG99641, &UA2700177},
	{SG99642, &UA2737496},
	{SG99643, &UA2936552},
	{SG99644, &UA3022603},
	{SG99645, &UA3092553},
	{SG99646, &UA3125927},
	{SG99647, &UA3126075},
	{SG99648, &UA2531902},
	{SG99649, &UA2659517},
	{SG99650, &UA2730075},
	{SG99651, &UA2856735},
	{SG99652, &UA2986182},
	{SG99653, &UA2580548},
	{SG99654, &UA2706345},
	{SG99655, &UA2768286},
	{SG99656, &UA3025861},
	{SG99657, &UA3023696},
	{SG99658, &UA3061927},
	{SG99659, &UA2565842},
	{SG99660, &UA2738141},
	{SG99661, &UA3079798},
	{SG99662, &UA3023816},
	{SG99663, &UA3080576},
	{SG99664, &UA3079222},
	{SG99665, &UA2466915},
	{SG99666, &UA2727274},
	{SG99667, &UA2598006},
	{SG99668, &UA2659473},
	{SG99669, &UA3014704},
	{SG99670, &UA3022887},
	{SG99671, &UA3058893},
	{SG99672, &UA2660713},
	{SG99673, &UA2662644},
	{SG99674, &UA2704261},
	{SG99675, &UA2752609},
	{SG99676, &UA2879164},
	{SG99677, &UA2945475},
	{SG99678, &UA3022221},
	{SG99679, &UA3080691},
	{SG99680, &UA2780149},
	{SG99681, &UA3023584},
	{SG99682, &UA3022509},
	{SG99683, &UA2659457},
	{SG99684, &UA2987620},
	{SG99685, &UA3093607},
	{SG99686, &UA3125931},
	{SG99692, &UA2868033},
	{SG99693, &UA3011282},
	{SG99694, &UA3079008},
	{SG99695, &UA3079835},
	{SG99696, &UA2164233},
	{SG99697, &UA2980178},
	{SG99698, &UA2731789},
	{SG99699, &UA2768172},
	{SG99700, &UA2781976},
	{SG99701, &UA2879401},
	{SG99702, &UA2457598},
	{SG99703, &UA2779560},
	{SG99704, &UA2867027},
	{SG99705, &UA2965084},
	{SG99706, &UA3023957},
	{SG99707, &UA3059329},
	{SG99708, &UA3061012},
	{SG99709, &UA2944996},
	{SG99710, &UA2945032},
	{SG99711, &UA3022495},
	{SG99712, &UA3108430},
	{SG99713, &UA3109334},
	{SG99718, &UA2485718},
	{SG99719, &UA2596577},
	{SG99720, &UA2580230},
	{SG99721, &UA2707540},
	{SG99722, &UA2851080},
	{SG99723, &UA2732578},
	{SG99724, &UA2739202},
	{SG99725, &UA2999825},
	{SG99726, &UA2659514},
	{SG99727, &UA2661276},
	{SG99728, &UA2752422},
	{SG99729, &UA2767660},
	{SG99730, &UA2900553},
	{SG99731, &UA3027879},
	{SG99732, &UA3093074},
	{SG99733, &UA3108262},
	{SG99735, &UA2422360},
	{SG99736, &UA2703175},
	{SG99737, &UA2707646},
	{SG99738, &UA2738068},
	{SG99739, &UA2766727},
	{SG99740, &UA3028857},
	{SG99741, &UA3029186},
	{SG99742, &UA1950526},
	{SG99743, &UA2410324},
	{SG99744, &UA2533211},
	{SG99745, &UA2580066},
	{SG99746, &UA2764280},
	{SG99747, &UA2733277},
	{SG99748, &UA2795586},
	{SG99749, &UA2866227},
	{SG99750, &UA2961134},
	{SG99751, &UA3026137},
	{SG99752, &UA2422037},
	{SG99753, &UA2841556},
	{SG99754, &UA2582034},
	{SG99755, &UA2665835},
	{SG99756, &UA2750688},
	{SG99757, &UA2752076},
	{SG99758, &UA2879584},
	{SG99759, &UA2892139},
	{SG99760, &UA2987062},
	{SG99761, &UA2990498},
	{SG99762, &UA3119078},
	{SG99763, &UA3027073},
	{SG99764, &UA3080168},
	{SG99765, &UA3110040},
	{SG99766, &UA2866500},
	{SG99767, &UA2892660},
	{SG99768, &UA2986536},
	{SG99769, &UA2989587},
	{SG99770, &UA3045213},
	{SG99771, &UA3062268},
	{SG99772, &UA3077792},
	{SG99773, &UA3079348},
	{SG99774, &UA2701361},
	{SG99775, &UA2796753},
	{SG99776, &UA2943868},
	{SG99777, &UA2964352},
	{SG99778, &UA3059928},
	{SG99779, &UA2221934},
	{SG99780, &UA2686693},
	{SG99781, &UA2563595},
	{SG99782, &UA2778894},
	{SG99783, &UA2729806},
	{SG99784, &UA3022261},
	{SG99785, &UA3064442},
	{SG99786, &UA3110311},
	{SG99787, &UA3110706},
	{SG99788, &UA2320257},
	{SG99789, &UA2812491},
	{SG99790, &UA2469211},
	{SG99791, &UA2469291},
	{SG99792, &UA2659985},
	{SG99793, &UA2698807},
	{SG99794, &UA2765624},
	{SG99795, &UA2944626},
	{SG99796, &UA2963414},
	{SG99797, &UA3013030},
	{SG99798, &UA3029078},
	{SG99799, &UA3032716},
	{SG99800, &UA3079674},
	{SG99801, &UA2261448},
	{SG99802, &UA2729929},
	{SG99803, &UA2769297},
	{SG99804, &UA3031636},
	{SG99805, &UA2699877},
	{SG99806, &UA2749940},
	{SG99807, &UA2752243},
	{SG99808, &UA2795371},
	{SG99809, &UA2892816},
	{SG99810, &UA3013949},
	{SG99811, &UA3058497},
	{SG99812, &UA3025158},
	{SG99813, &UA3026826},
	{SG99814, &UA3030046},
	{SG99815, &UA2661885},
	{SG99816, &UA2973696},
	{SG99817, &UA3015102},
	{SG99818, &UA3020617},
	{SG99819, &UA3017566},
	{SG99820, &UA3021459},
	{SG99821, &UA2560093},
	{SG99822, &UA2659403},
	{SG99823, &UA2659438},
	{SG99824, &UA2699796},
	{SG99825, &UA2703343},
	{SG99826, &UA2767899},
	{SG99827, &UA2894939},
	{SG99828, &UA2867318},
	{SG99829, &UA2966289},
	{SG99830, &UA3023313},
	{SG99831, &UA3058380},
	{SG99832, &UA2424168},
	{SG99833, &UA2430095},
	{SG99834, &UA2528203},
	{SG99835, &UA3043444},
	{SG99836, &UA2582425},
	{SG99839, &UA2792087},
	{SG99840, &UA2794095},
	{SG99841, &UA2944407},
	{SG99842, &UA2944905},
	{SG99843, &UA2962573},
	{SG99844, &UA3025623},
	{SG99845, &UA3024441},
	{SG99846, &UA3027016},
	{SG99847, &UA3057390},
	{SG99848, &UA3062916},
	{SG99849, &UA2699219},
	{SG99850, &UA2708500},
	{SG99851, &UA2795700},
	{SG99852, &UA2861216},
	{SG99853, &UA2866574},
	{SG99854, &UA2957736},
	{SG99855, &UA2964713},
	{SG99856, &UA3079414},
	{SG99857, &UA3079459},
	{SG99858, &UA3080203},
	{SG99859, &UA3093415},
	{SG99860, &UA2382405},
	{SG99861, &UA2982524},
	{SG99862, &UA2600342},
	{SG99863, &UA2702776},
	{SG99864, &UA2764460},
	{SG99865, &UA2952668},
	{SG99866, &UA2781708},
	{SG99867, &UA3030089},
	{SG99868, &UA3058530},
	{SG99869, &UA3061716},
	{SG99870, &UA2400821},
	{SG99871, &UA2477816},
	{SG99872, &UA2457954},
	{SG99873, &UA2840927},
	{SG99874, &UA2700388},
	{SG99875, &UA2865931},
	{SG99876, &UA2879227},
	{SG99877, &UA3058928},
	{SG99878, &UA3059066},
	{SG99879, &UA3079734},
	{SG99880, &UA2662451},
	{SG99881, &UA2769252},
	{SG99882, &UA2938405},
	{SG99883, &UA2994952},
	{SG99884, &UA3023742},
	{SG99885, &UA3027716},
	{SG99886, &UA3058870},
	{SG99887, &UA3078711},
	{SG99888, &UA3079985},
	{SG99889, &UA2750114},
	{SG99890, &UA2964600},
	{SG99891, &UA2958624},
	{SG99892, &UA2964437},
	{SG99893, &UA3024200},
	{SG99894, &UA3108778},
	{SG99895, &UA3079307},
	{SG99896, &UA3079680},
	{SG99897, &UA3093282},
	{SG99898, &UA3094908},
	{SG99899, &UA2402957},
	{SG99900, &UA2373013},
	{SG99901, &UA2458423},
	{SG99902, &UA2699932},
	{SG99903, &UA2729873},
	{SG99904, &UA2793998},
	{SG99905, &UA2961367},
	{SG99906, &UA2961827},
	{SG99907, &UA2964361},
	{SG99908, &UA3079542},
	{SG99909, &UA3078968},
	{SG99910, &UA3109155},
	{SG99911, &UA3126116},
	{SG99912, &UA2327213},
	{SG99913, &UA2685026},
	{SG99914, &UA2702750},
	{SG99915, &UA2706480},
	{SG99916, &UA2707354},
	{SG99917, &UA2767323},
	{SG99918, &UA2791969},
	{SG99919, &UA2799068},
	{SG99920, &UA2857529},
	{SG99921, &UA2865134},
	{SG99922, &UA2957901},
	{SG99923, &UA3034249},
	{SG99924, &UA3079173},
	{SG99925, &UA3092261},
	{SG99926, &UA2428709},
	{SG99927, &UA2456485},
	{SG99928, &UA2846037},
	{SG99933, &UA2558093},
	{SG99934, &UA2560467},
	{SG99936, &UA2855084},
	{SG99937, &UA2559654},
	{SG99945, &UA2730875},
	{SG99946, &UA2779556},
	{SG99947, &UA2852443},
	{SG99948, &UA2937394},
	{SG99949, &UA2944694},
	{SG99950, &UA3015533},
	{SG99951, &UA3123766},
	{SG99952, &UA3023428},
	{SG99953, &UA3026035},
	{SG99954, &UA2404486},
	{SG99955, &UA2426853},
	{SG99956, &UA2482714},
	{SG99957, &UA2594507},
	{SG99958, &UA3118387},
	{SG99959, &UA2430822},
	{SG99960, &UA2558376},
	{SG99961, &UA2952266},
	{SG99962, &UA2659511},
	{SG99963, &UA2748710},
	{SG99964, &UA2779990},
	{SG99965, &UA2868139},
	{SG99966, &UA2958456},
	{SG99967, &UA3023891},
	{SG99968, &UA3056937},
	{SG99969, &UA3064421},
	{SG99970, &UA2550636},
	{SG99971, &UA2730269},
	{SG99972, &UA2750700},
	{SG99973, &UA2960136},
	{SG99974, &UA2962151},
	{SG99975, &UA2962196},
	{SG99976, &UA2963190},
	{SG99977, &UA3000017},
	{SG99978, &UA3011964},
	{SG99979, &UA3020730},
	{SG99980, &UA3093267},
	{SG99981, &UA3080144},
	{SG99982, &UA3080574},
	{SG99983, &UA3115742},
	{SG99984, &UA2789539},
	{SG99985, &UA2424737},
	{SG99986, &UA2556119},
	{SG99987, &UA2706431},
	{SG99988, &UA2730806},
	{SG99989, &UA2780206},
	{SG99990, &UA2963326},
	{SG99991, &UA3012201},
	{SG99992, &UA3021328},
	{SG99993, &UA3093609},
	{SG99994, &UA3125591},
	{SG99995, &UA2362536},
	{SG99996, &UA2397694},
	{SG99997, &UA2426411},
	{SG99998, &UA2795075},
	{SG99999, &UA2796257},
	{SG100000, &UA3048518},
	{SG100001, &UA2962753},
	{SG100002, &UA2967509},
	{SG100003, &UA2999694},
	{SG100004, &UA3018108},
	{SG100005, &UA3027534},
	{SG100006, &UA3057258},
	{SG100007, &UA2368925},
	{SG100008, &UA2428325},
	{SG100009, &UA2430944},
	{SG100010, &UA2504998},
	{SG100011, &UA2597081},
	{SG100012, &UA2695096},
	{SG100013, &UA3118277},
	{SG100014, &UA2404000},
	{SG100015, &UA2706714},
	{SG100016, &UA2729803},
	{SG100017, &UA2730744},
	{SG100018, &UA2749549},
	{SG100019, &UA2767902},
	{SG100020, &UA2781950},
	{SG100021, &UA3000086},
	{SG100022, &UA2957754},
	{SG100023, &UA2989131},
	{SG100024, &UA2998301},
	{SG100025, &UA3012404},
	{SG100026, &UA3023751},
	{SG100027, &UA3048754},
	{SG100028, &UA3078700},
	{SG100029, &UA3092909},
	{SG100030, &UA3127683},
	{SG100031, &UA2665652},
	{SG100032, &UA2764445},
	{SG100033, &UA2960185},
	{SG100034, &UA2865469},
	{SG100035, &UA2962974},
	{SG100036, &UA2965915},
	{SG100037, &UA3078401},
	{SG100038, &UA3079168},
	{SG100039, &UA3092849},
	{SG100045, &UA2751218},
	{SG100046, &UA2999852},
	{SG100047, &UA2964359},
	{SG100048, &UA3020693},
	{SG100049, &UA3110554},
	{SG100050, &UA2485092},
	{SG100051, &UA2485541},
	{SG100052, &UA2765358},
	{SG100053, &UA2794809},
	{SG100054, &UA2864831},
	{SG100055, &UA2961825},
	{SG100061, &UA3109570},
	{SG100070, &UA2362258},
	{SG100071, &UA2391547},
	{SG100072, &UA2423654},
	{SG100073, &UA2527361},
	{SG100074, &UA2504746},
	{SG100075, &UA2736774},
	{SG100076, &UA2749267},
	{SG100077, &UA2793602},
	{SG100078, &UA2834172},
	{SG100079, &UA2837699},
	{SG100080, &UA2890088},
	{SG100081, &UA2896969},
	{SG100082, &UA2961745},
	{SG100083, &UA3000346},
	{SG100084, &UA3016185},
	{SG100085, &UA3058743},
	{SG100086, &UA3092420},
	{SG100087, &UA3093397},
	{SG100088, &UA2484775},
	{SG100089, &UA2660121},
	{SG100090, &UA2748349},
	{SG100091, &UA2764576},
	{SG100092, &UA2765536},
	{SG100093, &UA2795934},
	{SG100094, &UA2960689},
	{SG100095, &UA2960816},
	{SG100096, &UA3081019},
	{SG100097, &UA3081128},
	{SG100098, &UA3092730},
	{SG100099, &UA3126714},
	{SG100100, &UA3127410},
	{SG100101, &UA829992},
	{SG100102, &UA2442282},
	{SG100103, &UA2779231},
	{SG100104, &UA2793205},
	{SG100105, &UA2834126},
	{SG100106, &UA2860791},
	{SG100107, &UA3079619},
	{SG100108, &UA3109319},
	{SG100109, &UA3125128},
	{SG100110, &UA3127553},
	{SG100111, &UA1956906},
	{SG100113, &UA2261416},
	{SG100114, &UA2529402},
	{SG100115, &UA2707337},
	{SG100116, &UA2792755},
	{SG100117, &UA2795444},
	{SG100118, &UA2795888},
	{SG100119, &UA2851409},
	{SG100120, &UA2935203},
	{SG100121, &UA2955257},
	{SG100122, &UA2962566},
	{SG100123, &UA3057756},
	{SG100124, &UA3058866},
	{SG100125, &UA3094181},
	{SG100126, &UA3102181},
	{SG100127, &UA2442355},
	{SG100128, &UA2560767},
	{SG100129, &UA2779020},
	{SG100130, &UA2751575},
	{SG100131, &UA2766436},
	{SG100133, &UA2859165},
	{SG100134, &UA2962957},
	{SG100135, &UA2998221},
	{SG100136, &UA3057989},
	{SG100137, &UA3079499},
	{SG100138, &UA3083505},
	{SG100139, &UA3093591},
	{SG100140, &UA2330262},
	{SG100141, &UA2469876},
	{SG100142, &UA2487021},
	{SG100143, &UA2748922},
	{SG100144, &UA2891649},
	{SG100145, &UA2892445},
	{SG100146, &UA2956772},
	{SG100147, &UA3015452},
	{SG100148, &UA3058234},
	{SG100149, &UA3080269},
	{SG100150, &UA3093689},
	{SG100151, &UA3111579},
	{SG100152, &UA2467222},
	{SG100153, &UA2699862},
	{SG100154, &UA2937374},
	{SG100155, &UA2962170},
	{SG100156, &UA2965657},
	{SG100157, &UA3013686},
	{SG100158, &UA3025178},
	{SG100159, &UA3028280},
	{SG100160, &UA3033037},
	{SG100161, &UA3083504},
	{SG100162, &UA3093448},
	{SG100163, &UA3114722},
	{SG100171, &UA2764583},
	{SG100172, &UA2779233},
	{SG100173, &UA2793850},
	{SG100174, &UA2878492},
	{SG100175, &UA2879105},
	{SG100176, &UA2992141},
	{SG100177, &UA3014621},
	{SG100178, &UA3056554},
	{SG100179, &UA3058924},
	{SG100180, &UA3078620},
	{SG100181, &UA3093257},
	{SG100182, &UA3108789},
	{SG100183, &UA3108792},
	{SG100184, &UA3109337},
	{SG100185, &UA3109524},
	{SG100187, &UA2420879},
	{SG100188, &UA2701200},
	{SG100189, &UA3100070},
	{SG100190, &UA2701355},
	{SG100191, &UA2729908},
	{SG100192, &UA2748886},
	{SG100193, &UA2793929},
	{SG100194, &UA2862336},
	{SG100195, &UA2935326},
	{SG100196, &UA2938134},
	{SG100197, &UA2944236},
	{SG100198, &UA3013891},
	{SG100199, &UA3092044},
	{SG100201, &UA2964353},
	{SG100202, &UA3002118},
	{SG100203, &UA3012805},
	{SG100204, &UA3014246},
	{SG100205, &UA3016921},
	{SG100206, &UA3018711},
	{SG100207, &UA3020845},
	{SG100208, &UA3025851},
	{SG100209, &UA3109776},
	{SG100210, &UA3111000},
	{SG100211, &UA3116522},
	{SG100212, &UA2423144},
	{SG100213, &UA2430370},
	{SG100214, &UA2748486},
	{SG100215, &UA2749153},
	{SG100216, &UA2765297},
	{SG100217, &UA2767766},
	{SG100218, &UA2953701},
	{SG100219, &UA2957072},
	{SG100220, &UA3012590},
	{SG100221, &UA3015822},
	{SG100222, &UA3058782},
	{SG100223, &UA3079891},
	{SG100224, &UA3110312},
	{SG100225, &UA3111572},
	{SG100226, &UA3112471},
	{SG100227, &UA3115046},
	{SG100228, &UA3116534},
	{SG100229, &UA2482329},
	{SG100230, &UA2529610},
	{SG100231, &UA2779034},
	{SG100232, &UA2854600},
	{SG100233, &UA2860040},
	{SG100234, &UA2860802},
	{SG100235, &UA2878477},
	{SG100236, &UA3010714},
	{SG100237, &UA3046475},
	{SG100238, &UA3080694},
	{SG100239, &UA3109478},
	{SG100240, &UA2555261},
	{SG100241, &UA2749672},
	{SG100242, &UA2857831},
	{SG100243, &UA2955773},
	{SG100244, &UA2962724},
	{SG100245, &UA2996475},
	{SG100246, &UA3009910},
	{SG100247, &UA3013948},
	{SG100248, &UA3058483},
	{SG100249, &UA3058767},
	{SG100250, &UA3061763},
	{SG100251, &UA3079310},
	{SG100252, &UA3093276},
	{SG100253, &UA3108720},
	{SG100254, &UA3108930},
	{SG100255, &UA3109391},
	{SG100256, &UA2426715},
	{SG100257, &UA2654140},
	{SG100258, &UA2951117},
	{SG100259, &UA2504953},
	{SG100260, &UA2557414},
	{SG100261, &UA2659611},
	{SG100262, &UA2855967},
	{SG100263, &UA2748621},
	{SG100264, &UA2779050},
	{SG100265, &UA2792174},
	{SG100266, &UA2816913},
	{SG100267, &UA3098793},
	{SG100268, &UA2850477},
	{SG100269, &UA2936042},
	{SG100270, &UA2953197},
	{SG100271, &UA2966938},
	{SG100272, &UA2989436},
	{SG100273, &UA2995753},
	{SG100274, &UA2996107},
	{SG100275, &UA3015756},
	{SG100276, &UA3016632},
	{SG100277, &UA3017483},
	{SG100278, &UA3020339},
	{SG100279, &UA3030139},
	{SG100280, &UA3058455},
	{SG100281, &UA3076203},
	{SG100282, &UA3078772},
	{SG100283, &UA3108672},
	{SG100284, &UA3108953},
	{SG100285, &UA3109453},
	{SG100286, &UA3109489},
	{SG100287, &UA3112676},
	{SG100288, &UA2465663},
	{SG100289, &UA2483047},
	{SG100290, &UA2484385},
	{SG100291, &UA2555813},
	{SG100292, &UA2505392},
	{SG100293, &UA2577212},
	{SG100294, &UA2596996},
	{SG100295, &UA2748541},
	{SG100296, &UA2853118},
	{SG100297, &UA2862990},
	{SG100298, &UA2937160},
	{SG100299, &UA2961141},
	{SG100300, &UA2991775},
	{SG100301, &UA2994671},
	{SG100302, &UA2996027},
	{SG100303, &UA3001709},
	{SG100304, &UA3010489},
	{SG100305, &UA3018855},
	{SG100306, &UA3019032},
	{SG100307, &UA3093275},
	{SG100308, &UA3108676},
	{SG100309, &UA3108845},
	{SG100310, &UA3109418},
	{SG100311, &UA3110301},
	{SG100312, &UA3110872},
	{SG100313, &UA2528227},
	{SG100314, &UA2529687},
	{SG100315, &UA2655458},
	{SG100316, &UA2696143},
	{SG100317, &UA2779069},
	{SG100318, &UA2792711},
	{SG100319, &UA2852967},
	{SG100320, &UA2853164},
	{SG100321, &UA2865968},
	{SG100322, &UA2935369},
	{SG100323, &UA2936291},
	{SG100324, &UA2957089},
	{SG100325, &UA3014797},
	{SG100326, &UA3019736},
	{SG100327, &UA3058190},
	{SG100328, &UA3059084},
	{SG100329, &UA3060697},
	{SG100330, &UA3078289},
	{SG100331, &UA3078621},
	{SG100332, &UA3079180},
	{SG100333, &UA3108450},
	{SG100334, &UA3108486},
	{SG100335, &UA3110424},
	{SG100339, &UA2222551},
	{SG100340, &UA2261443},
	{SG100341, &UA2403529},
	{SG100342, &UA2728894},
	{SG100343, &UA2764950},
	{SG100344, &UA2851657},
	{SG100345, &UA2854979},
	{SG100346, &UA2952281},
	{SG100347, &UA2960945},
	{SG100348, &UA3014077},
	{SG100349, &UA2989293},
	{SG100350, &UA2991482},
	{SG100351, &UA2994773},
	{SG100352, &UA2994844},
	{SG100353, &UA3003838},
	{SG100354, &UA3006484},
	{SG100355, &UA3009727},
	{SG100356, &UA3010595},
	{SG100357, &UA3014409},
	{SG100358, &UA3016193},
	{SG100359, &UA3047903},
	{SG100360, &UA3057076},
	{SG100361, &UA3058253},
	{SG100362, &UA3093465},
	{SG100363, &UA3108362},
	{SG100364, &UA3109872},
	{SG100365, &UA2445901},
	{SG100366, &UA2528696},
	{SG100367, &UA2598610},
	{SG100368, &UA2986368},
	{SG100369, &UA2795065},
	{SG100370, &UA2854682},
	{SG100371, &UA2865081},
	{SG100372, &UA2869712},
	{SG100373, &UA2992485},
	{SG100374, &UA2993533},
	{SG100375, &UA3016168},
	{SG100376, &UA3078825},
	{SG100377, &UA3108461},
	{SG100378, &UA3108610},
	{SG100379, &UA3108779},
	{SG100380, &UA2834305},
	{SG100381, &UA2507024},
	{SG100382, &UA2728163},
	{SG100383, &UA2852891},
	{SG100384, &UA2953061},
	{SG100385, &UA2990952},
	{SG100386, &UA2991244},
	{SG100387, &UA2998373},
	{SG100388, &UA2999989},
	{SG100389, &UA3003905},
	{SG100390, &UA3013220},
	{SG100391, &UA3014150},
	{SG100392, &UA3016279},
	{SG100393, &UA3023461},
	{SG100394, &UA3107884},
	{SG100395, &UA3042727},
	{SG100396, &UA3045498},
	{SG100397, &UA3058668},
	{SG100398, &UA3091863},
	{SG100399, &UA3093337},
	{SG100400, &UA3107691},
	{SG100403, &UA3109220},
	{SG100404, &UA3114235},
	{SG100405, &UA3125271},
	{SG100410, &UA2660874},
	{SG100415, &UA2855674},
	{SG100416, &UA2935188},
	{SG100417, &UA2937033},
	{SG100418, &UA2937239},
	{SG100419, &UA2957774},
	{SG100420, &UA2994057},
	{SG100421, &UA2994253},
	{SG100422, &UA2994720},
	{SG100423, &UA2996974},
	{SG100424, &UA3008386},
	{SG100425, &UA3010059},
	{SG100426, &UA3012589},
	{SG100427, &UA3018853},
	{SG100428, &UA3092207},
	{SG100429, &UA3107401},
	{SG100430, &UA3108111},
	{SG100431, &UA3108492},
	{SG100432, &UA3112957},
	{SG100433, &UA3126033},
	{SG100434, &UA3126045},
	{SG100435, &UA3126078},
	{SG100436, &UA3126476},
	{SG100437, &UA2217305},
	{SG100438, &UA2485378},
	{SG100439, &UA2505462},
	{SG100440, &UA2528636},
	{SG100441, &UA2560315},
	{SG100442, &UA2597542},
	{SG100443, &UA2696075},
	{SG100444, &UA2738025},
	{SG100445, &UA2868388},
	{SG100446, &UA2763853},
	{SG100447, &UA2844695},
	{SG100449, &UA2865233},
	{SG100450, &UA2963899},
	{SG100451, &UA2987513},
	{SG100452, &UA2988722},
	{SG100453, &UA2993074},
	{SG100454, &UA2993945},
	{SG100455, &UA3004480},
	{SG100456, &UA3015611},
	{SG100457, &UA3016831},
	{SG100458, &UA3017168},
	{SG100459, &UA3025251},
	{SG100460, &UA3028249},
	{SG100461, &UA3056995},
	{SG100462, &UA3059059},
	{SG100463, &UA3078788},
	{SG100464, &UA3088058},
	{SG100465, &UA3091738},
	{SG100469, &UA3092974},
	{SG100471, &UA3093307},
	{SG100475, &UA3108376},
	{SG100476, &UA3109342},
	{SG100477, &UA3110397},
	{SG100478, &UA3125596},
	{SG100479, &UA3125772},
	{SG100480, &UA3125979},
	{SG100481, &UA616299},
	{SG100482, &UA758810},
	{SG100483, &UA2474427},
	{SG100484, &UA2421819},
	{SG100485, &UA2456973},
	{SG100486, &UA2457785},
	{SG100487, &UA2483192},
	{SG100488, &UA2555587},
	{SG100499, &UA2698942},
	{SG100500, &UA2764554},
	{SG100501, &UA2781272},
	{SG100502, &UA2817827},
	{SG100503, &UA2857777},
	{SG100504, &UA2859094},
	{SG100505, &UA2937050},
	{SG100506, &UA2953333},
	{SG100507, &UA2962913},
	{SG100508, &UA2985627},
	{SG100509, &UA2988892},
	{SG100510, &UA2994310},
	{SG100511, &UA2996862},
	{SG100512, &UA3000464},
	{SG100513, &UA3005205},
	{SG100514, &UA3012785},
	{SG100515, &UA3014710},
	{SG100516, &UA3015624},
	{SG100517, &UA3016335},
	{SG100518, &UA3018863},
	{SG100519, &UA3019702},
	{SG100520, &UA3046624},
	{SG100521, &UA3055853},
	{SG100522, &UA3056711},
	{SG100523, &UA3057083},
	{SG100524, &UA3073919},
	{SG100525, &UA3107754},
	{SG100526, &UA3108157},
	{SG100527, &UA3109040},
	{SG100528, &UA3125251},
	{SG100529, &UA3125726},
	{SG100530, &UA3125915},
	{SG100531, &UA3125954},
	{SG100532, &UA3126404},
	{SG100533, &UA2469513},
	{SG100534, &UA3043131},
	{SG100535, &UA2686421},
	{SG100536, &UA2693052},
	{SG100537, &UA2696069},
	{SG100538, &UA2822036},
	{SG100539, &UA2846130},
	{SG100540, &UA2864645},
	{SG100541, &UA2864799},
	{SG100542, &UA2982818},
	{SG100543, &UA2986340},
	{SG100544, &UA2987012},
	{SG100545, &UA2987706},
	{SG100546, &UA2989710},
	{SG100547, &UA2993139},
	{SG100548, &UA2995418},
	{SG100549, &UA2995681},
	{SG100550, &UA3013557},
	{SG100551, &UA3031425},
	{SG100552, &UA3056236},
	{SG100553, &UA3056858},
	{SG100554, &UA3056943},
	{SG100555, &UA3058238},
	{SG100556, &UA3058392},
	{SG100557, &UA3071377},
	{SG100558, &UA3073468},
	{SG100559, &UA3091659},
	{SG100560, &UA3102067},
	{SG100561, &UA3102487},
	{SG100562, &UA3103613},
	{SG100563, &UA3108125},
	{SG100564, &UA3108465},
	{SG100565, &UA3108899},
	{SG100566, &UA3120938},
	{SG100567, &UA3125677},
	{SG100568, &UA3125887},
	{SG100569, &UA3125989},
	{SG100570, &UA3126066},
	{SG100571, &UA3126165},
	{SG100572, &UA3126376},
	{SG100573, &UA3126754},
	{SG100574, &UA3126894},
	{SG100575, &UA3127079},
	{SG100576, &UA3127390},
	{SG100577, &UA2375830},
	{SG100578, &UA2420976},
	{SG100579, &UA2440935},
	{SG100580, &UA2443011},
	{SG100581, &UA2463195},
	{SG100582, &UA2520220},
	{SG100583, &UA2655855},
	{SG100584, &UA2697363},
	{SG100585, &UA2747887},
	{SG100586, &UA2794753},
	{SG100587, &UA2958904},
	{SG100588, &UA2844154},
	{SG100589, &UA2935123},
	{SG100590, &UA2986093},
	{SG100591, &UA2991355},
	{SG100592, &UA2992691},
	{SG100593, &UA2995172},
	{SG100594, &UA2999157},
	{SG100595, &UA2999800},
	{SG100596, &UA3008589},
	{SG100597, &UA3018857},
	{SG100598, &UA3044604},
	{SG100599, &UA3045419},
	{SG100600, &UA3048695},
	{SG100601, &UA3056499},
	{SG100602, &UA3056758},
	{SG100603, &UA3056907},
	{SG100604, &UA3072332},
	{SG100605, &UA3077737},
	{SG100606, &UA3079521},
	{SG100607, &UA3092287},
	{SG100608, &UA3092388},
	{SG100609, &UA3092393},
	{SG100610, &UA3102429},
	{SG100611, &UA3106646},
	{SG100612, &UA3108025},
	{SG100613, &UA3121422},
	{SG100614, &UA3125558},
	{SG100615, &UA3125717},
	{SG100616, &UA3125816},
	{SG100617, &UA3125844},
	{SG100618, &UA3125996},
	{SG100619, &UA2322372},
	{SG100620, &UA3042557},
	{SG100621, &UA2454997},
	{SG100622, &UA2591720},
	{SG100623, &UA2683492},
	{SG100624, &UA2828643},
	{SG100625, &UA2831106},
	{SG100628, &UA2855689},
	{SG100629, &UA2862751},
	{SG100630, &UA2986565},
	{SG100631, &UA2996112},
	{SG100632, &UA2996479},
	{SG100633, &UA2998338},
	{SG100634, &UA2999798},
	{SG100635, &UA3017652},
	{SG100636, &UA3048472},
	{SG100637, &UA3048999},
	{SG100638, &UA3050289},
	{SG100639, &UA3056913},
	{SG100640, &UA3074523},
	{SG100641, &UA3078505},
	{SG100642, &UA3087616},
	{SG100643, &UA3092381},
	{SG100644, &UA3101958},
	{SG100645, &UA3102098},
	{SG100646, &UA3102714},
	{SG100647, &UA3103190},
	{SG100648, &UA3107137},
	{SG100649, &UA3120349},
	{SG100650, &UA3121188},
	{SG100651, &UA3121458},
	{SG100653, &UA3121639},
	{SG100655, &UA3122216},
	{SG100657, &UA3123916},
	{SG100662, &UA3124493},
	{SG100665, &UA3124597},
	{SG100666, &UA3124805},
	{SG100667, &UA3124870},
	{SG100668, &UA3125024},
	{SG100669, &UA3125042},
	{SG100670, &UA3125346},
	{SG100671, &UA3125561},
	{SG100672, &UA3125587},
	{SG100673, &UA3125691},
	{SG100674, &UA3125758},
	{SG100675, &UA3125942},
	{SG100676, &UA3125973},
	{SG100677, &UA3126100},
	{SG100678, &UA3126150},
	{SG100679, &UA3126251},
	{SG100681, &UA2332552},
	{SG100682, &UA2441685},
	{SG100683, &UA2663545},
	{SG100684, &UA2665452},
	{SG100685, &UA2751907},
	{SG100686, &UA2868627},
	{SG100687, &UA2842246},
	{SG100688, &UA2844862},
	{SG100689, &UA3056544},
	{SG100690, &UA2596329},
	{SG100691, &UA2797323},
	{SG100692, &UA2865844},
	{SG100693, &UA2816505},
	{SG100694, &UA2843284},
	{SG100695, &UA2863218},
	{SG100696, &UA2864882},
	{SG100697, &UA2659978},
	{SG100698, &UA2942989},
	{SG100699, &UA2459559},
	{SG100700, &UA2778961},
	{SG100701, &UA2952861},
	{SG100702, &UA2580417},
	{SG100703, &UA2817171},
	{SG100704, &UA2848934},
	{SG100705, &UA2730051},
	{SG100706, &UA3044702},
	{SG100707, &UA2737378},
	{SG100708, &UA2869209},
	{SG100710, &UA3080021},
	{SG100712, &UA2461308},
	{SG100713, &UA2556322},
	{SG100714, &UA2590957},
	{SG100715, &UA2598672},
	{SG100721, &UA2599144},
	{SG100722, &UA2698686},
	{SG100723, &UA2706976},
	{SG100724, &UA3092788},
	{SG100725, &UA2665255},
	{SG100726, &UA2865767},
	{SG100727, &UA2937848},
	{SG100728, &UA2960131},
	{SG100729, &UA3093131},
	{SG100730, &UA1981054},
	{SG100731, &UA2422737},
	{SG100732, &UA2843617},
	{SG100733, &UA2642810},
	{SG100734, &UA2555273},
	{SG100735, &UA2599569},
	{SG100736, &UA3052325},
	{SG100737, &UA2657281},
	{SG100738, &UA2660347},
	{SG100739, &UA2852410},
	{SG100740, &UA2852553},
	{SG100741, &UA2986854},
	{SG100742, &UA3013063},
	{SG100743, &UA3013990},
	{SG100744, &UA3014833},
	{SG100745, &UA3058783},
	{SG100746, &UA3108663},
	{SG100747, &UA2428950},
	{SG100748, &UA2467096},
	{SG100749, &UA2482999},
	{SG100750, &UA2659538},
	{SG100751, &UA2698661},
	{SG100752, &UA2763521},
	{SG100753, &UA2764260},
	{SG100754, &UA2844846},
	{SG100755, &UA2951396},
	{SG100756, &UA2767739},
	{SG100757, &UA2958357},
	{SG100758, &UA2963144},
	{SG100759, &UA3008553},
	{SG100760, &UA3013553},
	{SG100761, &UA3059745},
	{SG100762, &UA3108441},
	{SG100763, &UA2482968},
	{SG100764, &UA2484383},
	{SG100765, &UA2579922},
	{SG100773, &UA2654883},
	{SG100774, &UA2665207},
	{SG100775, &UA2702519},
	{SG100776, &UA2747058},
	{SG100777, &UA2752307},
	{SG100778, &UA2797769},
	{SG100779, &UA2861949},
	{SG100780, &UA2792285},
	{SG100781, &UA2847306},
	{SG100782, &UA3044090},
	{SG100783, &UA3021020},
	{SG100784, &UA3030147},
	{SG100785, &UA3074500},
	{SG100786, &UA3126275},
	{SG100787, &UA2432873},
	{SG100788, &UA2505294},
	{SG100789, &UA2555487},
	{SG100790, &UA2748423},
	{SG100791, &UA2847785},
	{SG100792, &UA2891332},
	{SG100793, &UA2987835},
	{SG100794, &UA3100501},
	{SG100795, &UA2953142},
	{SG100796, &UA3003156},
	{SG100797, &UA3058566},
	{SG100798, &UA1973082},
	{SG100799, &UA2428401},
	{SG100800, &UA2458061},
	{SG100801, &UA2749913},
	{SG100802, &UA2867543},
	{SG100803, &UA2458664},
	{SG100804, &UA2588364},
	{SG100805, &UA2598004},
	{SG100809, &UA2651117},
	{SG100810, &UA2702291},
	{SG100811, &UA2702865},
	{SG100812, &UA2763086},
	{SG100813, &UA2763201},
	{SG100814, &UA2834458},
	{SG100815, &UA2851226},
	{SG100816, &UA2951413},
	{SG100817, &UA3006104},
	{SG100818, &UA3010974},
	{SG100819, &UA3058961},
	{SG100820, &UA3126734},
	{SG100829, &UA2451836},
	{SG100830, &UA2458453},
	{SG100831, &UA2502243},
	{SG100832, &UA2579234},
	{SG100836, &UA2681012},
	{SG100837, &UA2697831},
	{SG100838, &UA2849094},
	{SG100839, &UA2952372},
	{SG100840, &UA2986345},
	{SG100841, &UA3092782},
	{SG100842, &UA3108412},
	{SG100843, &UA3125459},
	{SG100844, &UA622497},
	{SG100845, &UA622518},
	{SG100846, &UA622512},
	{NULL}};
const Segmentdata DAndroidVersion4_3[] = {
	{SG99020, &UA3064639},
	{SG99134, &UA3064641},
	{SG99203, &UA2973891},
	{SG99204, &UA3029694},
	{SG99206, &UA2870012},
	{SG99207, &UA2969480},
	{SG99208, &UA3023346},
	{SG99209, &UA3030461},
	{SG99210, &UA3034335},
	{SG99211, &UA3036443},
	{SG99212, &UA3064910},
	{SG99213, &UA3108576},
	{SG99214, &UA3114342},
	{SG99215, &UA3115731},
	{SG99216, &UA2965806},
	{SG99217, &UA2974402},
	{SG99218, &UA2975961},
	{SG99219, &UA3033185},
	{SG99220, &UA3033189},
	{SG99221, &UA3034386},
	{SG99222, &UA3080898},
	{SG99223, &UA3093396},
	{SG99224, &UA3093705},
	{SG99225, &UA3128244},
	{SG99293, &UA2975068},
	{SG99298, &UA2940993},
	{SG99299, &UA2970406},
	{SG99300, &UA3023357},
	{SG99301, &UA3031259},
	{SG99302, &UA3033988},
	{SG99304, &UA3034075},
	{SG99305, &UA3061072},
	{SG99307, &UA3064719},
	{SG99308, &UA3114340},
	{SG99309, &UA3115765},
	{SG99341, &UA3064644},
	{SG99409, &UA3065142},
	{SG99432, &UA2969748},
	{SG99445, &UA3130869},
	{SG99446, &UA3132981},
	{SG99468, &UA3130870},
	{SG99483, &UA2948336},
	{SG99586, &UA2976687},
	{SG99598, &UA2973475},
	{SG99609, &UA3065326},
	{SG99610, &UA3065589},
	{SG99611, &UA3112821},
	{SG99612, &UA3126320},
	{SG99613, &UA3127848},
	{SG99687, &UA2974732},
	{SG99688, &UA3034484},
	{SG99689, &UA3035681},
	{SG99690, &UA3039394},
	{SG99691, &UA3063543},
	{SG99714, &UA3065469},
	{SG99715, &UA3068207},
	{SG99716, &UA3082811},
	{SG99717, &UA3114557},
	{SG99837, &UA3067300},
	{SG99838, &UA3081279},
	{SG99929, &UA3027025},
	{SG99930, &UA3028081},
	{SG99931, &UA3028324},
	{SG99932, &UA3061503},
	{SG99935, &UA3062214},
	{SG99938, &UA3063526},
	{SG99939, &UA3096096},
	{SG99940, &UA3110835},
	{SG99941, &UA3111758},
	{SG99942, &UA3113747},
	{SG99943, &UA3113750},
	{SG99944, &UA3115531},
	{SG100040, &UA3065991},
	{SG100041, &UA3068219},
	{SG100042, &UA3080992},
	{SG100043, &UA3094221},
	{SG100044, &UA3094446},
	{SG100056, &UA3030029},
	{SG100057, &UA3059622},
	{SG100058, &UA3059651},
	{SG100059, &UA3059650},
	{SG100060, &UA3078762},
	{SG100062, &UA3079380},
	{SG100063, &UA3093348},
	{SG100064, &UA3093588},
	{SG100065, &UA3108651},
	{SG100066, &UA3112336},
	{SG100067, &UA3125962},
	{SG100068, &UA3127277},
	{SG100069, &UA3132614},
	{SG100112, &UA2970047},
	{SG100132, &UA2972284},
	{SG100164, &UA3080716},
	{SG100165, &UA3080744},
	{SG100166, &UA3080912},
	{SG100167, &UA3081192},
	{SG100168, &UA3081229},
	{SG100169, &UA3083218},
	{SG100170, &UA3110734},
	{SG100186, &UA3131343},
	{SG100200, &UA3130875},
	{SG100336, &UA3027019},
	{SG100337, &UA3027678},
	{SG100338, &UA3125267},
	{SG100401, &UA3065734},
	{SG100402, &UA3094028},
	{SG100406, &UA3024148},
	{SG100407, &UA3062476},
	{SG100408, &UA3096103},
	{SG100409, &UA3116189},
	{SG100411, &UA3028158},
	{SG100412, &UA3062465},
	{SG100413, &UA3095679},
	{SG100414, &UA3115771},
	{SG100448, &UA3033373},
	{SG100466, &UA3024972},
	{SG100467, &UA3026276},
	{SG100468, &UA3062775},
	{SG100470, &UA3093361},
	{SG100472, &UA3110081},
	{SG100473, &UA3113435},
	{SG100474, &UA3126461},
	{SG100489, &UA3027309},
	{SG100490, &UA3032721},
	{SG100491, &UA3062471},
	{SG100492, &UA3063532},
	{SG100493, &UA3083913},
	{SG100494, &UA3096094},
	{SG100495, &UA3109618},
	{SG100496, &UA3110831},
	{SG100497, &UA3114331},
	{SG100498, &UA3116184},
	{SG100626, &UA3028238},
	{SG100627, &UA3034283},
	{SG100652, &UA2737449},
	{SG100654, &UA3032709},
	{SG100656, &UA3036414},
	{SG100658, &UA3062137},
	{SG100659, &UA3062645},
	{SG100660, &UA3079639},
	{SG100661, &UA3082172},
	{SG100663, &UA3093333},
	{SG100664, &UA3108426},
	{SG100709, &UA2969247},
	{SG100711, &UA3034017},
	{SG100716, &UA2960974},
	{SG100717, &UA2972446},
	{SG100718, &UA3030141},
	{SG100719, &UA3031355},
	{SG100720, &UA3093549},
	{SG100766, &UA3026688},
	{SG100767, &UA3027952},
	{SG100768, &UA3065011},
	{SG100769, &UA3080850},
	{SG100770, &UA3096220},
	{SG100771, &UA3131473},
	{SG100772, &UA3132952},
	{SG100806, &UA3027365},
	{SG100807, &UA3027956},
	{SG100808, &UA3029077},
	{SG100821, &UA3030822},
	{SG100822, &UA3031323},
	{SG100823, &UA3036638},
	{SG100824, &UA3062135},
	{SG100825, &UA3066941},
	{SG100826, &UA3094137},
	{SG100827, &UA3125933},
	{SG100828, &UA3130822},
	{SG100833, &UA3096276},
	{SG100834, &UA3066213},
	{SG100835, &UA3131160},
	{SG100847, &UA3030116},
	{SG100848, &UA3062283},
	{SG100849, &UA3064030},
	{SG100850, &UA3078718},
	{SG100851, &UA3079815},
	{SG100852, &UA3079816},
	{SG100853, &UA3080259},
	{SG100854, &UA3093403},
	{SG100855, &UA3125793},
	{SG100856, &UA3127168},
	{SG100857, &UA3127578},
	{SG100858, &UA3127924},
	{SG100859, &UA3128011},
	{SG100860, &UA3029217},
	{SG100861, &UA3064433},
	{SG100862, &UA3127819},
	{SG100863, &UA3130444},
	{SG100864, &UA2973660},
	{SG100865, &UA3081221},
	{SG100866, &UA3080249},
	{SG100867, &UA3083648},
	{SG100868, &UA3080217},
	{SG100869, &UA3082800},
	{SG100870, &UA3034719},
	{SG100871, &UA3062140},
	{SG100872, &UA3115575},
	{SG100873, &UA3128043},
	{SG100874, &UA3083442},
	{SG100875, &UA3033862},
	{SG100876, &UA3031371},
	{SG100877, &UA3063983},
	{SG100878, &UA3128367},
	{SG100879, &UA3029227},
	{SG100880, &UA3032597},
	{SG100881, &UA3081187},
	{SG100882, &UA3080908},
	{SG100883, &UA3083244},
	{SG100884, &UA3094603},
	{SG100885, &UA3109208},
	{SG100886, &UA3126756},
	{SG100887, &UA3127034},
	{SG100888, &UA3128606},
	{SG100889, &UA3081397},
	{SG100890, &UA3096290},
	{SG100891, &UA3028311},
	{SG100892, &UA3036003},
	{SG100893, &UA3036014},
	{SG100894, &UA3059361},
	{SG100895, &UA3080974},
	{SG100896, &UA3029197},
	{SG100897, &UA3066939},
	{SG100898, &UA3115724},
	{SG100899, &UA3027862},
	{SG100900, &UA3062765},
	{SG100901, &UA3116176},
	{SG100902, &UA3026604},
	{SG100903, &UA3028292},
	{SG100904, &UA3031049},
	{SG100905, &UA3031322},
	{SG100906, &UA3038885},
	{SG100907, &UA3029683},
	{SG100908, &UA3030446},
	{SG100909, &UA3080374},
	{SG100910, &UA3131332},
	{SG100911, &UA3024986},
	{SG100912, &UA3066945},
	{SG100913, &UA3129663},
	{SG100914, &UA3033936},
	{SG100915, &UA3062484},
	{SG100916, &UA3128220},
	{SG100917, &UA3083646},
	{SG100918, &UA3026828},
	{SG100919, &UA3062481},
	{SG100920, &UA3093605},
	{SG100921, &UA3125319},
	{SG100922, &UA3127362},
	{SG100923, &UA3059300},
	{SG100924, &UA3061778},
	{SG100925, &UA3110730},
	{SG100926, &UA3126693},
	{SG100927, &UA3033945},
	{SG100928, &UA3063529},
	{SG100929, &UA3130198},
	{SG100930, &UA3083537},
	{SG100931, &UA3034273},
	{SG100932, &UA3079350},
	{SG100933, &UA3079586},
	{SG100934, &UA3108794},
	{SG100935, &UA3108942},
	{SG100936, &UA3083325},
	{SG100937, &UA3026610},
	{SG100938, &UA3062769},
	{SG100939, &UA2888660},
	{SG100940, &UA3060988},
	{SG100941, &UA3058580},
	{SG100942, &UA3080920},
	{SG100943, &UA3083203},
	{SG100944, &UA3014853},
	{SG100945, &UA3059010},
	{SG100946, &UA3062131},
	{SG100947, &UA3127364},
	{SG100948, &UA3132973},
	{SG100949, &UA3038878},
	{SG100950, &UA3065422},
	{SG100951, &UA3111750},
	{SG100952, &UA3025424},
	{SG100953, &UA3062763},
	{SG100954, &UA3096320},
	{SG100955, &UA3027796},
	{SG100956, &UA3064021},
	{SG100957, &UA3128086},
	{SG100958, &UA3034250},
	{SG100959, &UA3063524},
	{SG100960, &UA3127914},
	{SG100961, &UA3080125},
	{SG100962, &UA3080255},
	{SG100963, &UA3062528},
	{SG100964, &UA3080162},
	{SG100965, &UA3127570},
	{SG100966, &UA3081292},
	{SG100967, &UA3024500},
	{SG100968, &UA3063537},
	{SG100969, &UA3034036},
	{SG100970, &UA3109617},
	{SG100971, &UA3115045},
	{SG100972, &UA3126955},
	{SG100973, &UA3127354},
	{SG100974, &UA3027166},
	{SG100975, &UA3083918},
	{SG100976, &UA3128239},
	{SG100977, &UA3032611},
	{SG100978, &UA3093708},
	{SG100979, &UA3081226},
	{SG100980, &UA3079748},
	{SG100981, &UA3125448},
	{SG100982, &UA3128219},
	{SG100983, &UA3024448},
	{SG100984, &UA3115123},
	{SG100985, &UA3029067},
	{SG100986, &UA3109613},
	{SG100987, &UA3059305},
	{SG100988, &UA3093511},
	{SG100989, &UA3060398},
	{SG100990, &UA3080188},
	{SG100991, &UA3127922},
	{SG100992, &UA3094795},
	{SG100993, &UA3083847},
	{SG100994, &UA3078597},
	{SG100995, &UA3113085},
	{SG100996, &UA3025430},
	{SG100997, &UA3026144},
	{SG100998, &UA3125953},
	{SG100999, &UA3094445},
	{SG101000, &UA3030207},
	{SG101001, &UA3079544},
	{SG101002, &UA3132979},
	{SG101003, &UA3061126},
	{SG101004, &UA3083916},
	{SG101005, &UA3094270},
	{SG101006, &UA3083839},
	{SG101007, &UA3059072},
	{SG101008, &UA3094190},
	{SG101009, &UA3024215},
	{SG101010, &UA3078731},
	{SG101011, &UA3092507},
	{SG101012, &UA3027674},
	{SG101013, &UA3058458},
	{SG101014, &UA3093566},
	{SG101015, &UA3110238},
	{SG101016, &UA3029123},
	{SG101017, &UA3083394},
	{SG101018, &UA3109273},
	{SG101019, &UA3027376},
	{SG101020, &UA3058326},
	{SG101021, &UA3109064},
	{SG101022, &UA3025129},
	{SG101023, &UA3061762},
	{SG101024, &UA3082756},
	{SG101025, &UA3128155},
	{SG101026, &UA3094620},
	{SG101027, &UA3094796},
	{SG101028, &UA3108973},
	{SG101029, &UA3126252},
	{SG101030, &UA3094432},
	{SG101031, &UA3114790},
	{SG101032, &UA3128162},
	{SG101033, &UA3094505},
	{SG101034, &UA3034260},
	{SG101035, &UA3093416},
	{SG101036, &UA3080922},
	{SG101037, &UA3066940},
	{SG101038, &UA3093485},
	{SG101039, &UA3129259},
	{SG101040, &UA3128651},
	{SG101041, &UA3058185},
	{SG101042, &UA3108769},
	{SG101043, &UA3029048},
	{SG101044, &UA3066127},
	{SG101045, &UA3080326},
	{SG101046, &UA3082178},
	{SG101047, &UA3127360},
	{SG101048, &UA3114916},
	{SG101049, &UA3039511},
	{SG101050, &UA3093414},
	{SG101051, &UA3125209},
	{SG101052, &UA3027887},
	{SG101053, &UA3082982},
	{SG101054, &UA3031948},
	{SG101055, &UA3025925},
	{SG101056, &UA3079602},
	{SG101057, &UA3132900},
	{SG101058, &UA3034289},
	{SG101059, &UA3027011},
	{SG101060, &UA3058813},
	{SG101061, &UA3079104},
	{SG101062, &UA3079347},
	{SG101063, &UA3039391},
	{SG101064, &UA3034271},
	{SG101065, &UA3028245},
	{SG101066, &UA3034248},
	{SG101067, &UA3110474},
	{SG101068, &UA3129652},
	{SG101069, &UA3034668},
	{SG101070, &UA3062129},
	{SG101071, &UA3109386},
	{SG101072, &UA3059008},
	{SG101073, &UA3094840},
	{SG101074, &UA3079830},
	{SG101084, &UA3126284},
	{SG101085, &UA3127751},
	{SG101086, &UA3027719},
	{SG101087, &UA3029086},
	{SG101088, &UA3094374},
	{SG101089, &UA3109110},
	{SG101090, &UA3110874},
	{SG101091, &UA3113452},
	{SG101092, &UA3126537},
	{SG101093, &UA3126189},
	{SG101094, &UA3127298},
	{SG101095, &UA3058291},
	{SG101096, &UA3108896},
	{SG101097, &UA3110143},
	{SG101098, &UA3110967},
	{SG101099, &UA3125368},
	{SG101100, &UA3125884},
	{SG101101, &UA3126265},
	{SG101102, &UA3078568},
	{SG101103, &UA3108763},
	{SG101104, &UA3125301},
	{SG101105, &UA3126470},
	{SG101106, &UA3126653},
	{SG101107, &UA3030109},
	{SG101108, &UA3034721},
	{SG101109, &UA3058357},
	{SG101110, &UA3083331},
	{SG101111, &UA3125306},
	{SG101112, &UA3126236},
	{SG101127, &UA3068941},
	{SG101128, &UA3068931},
	{SG101129, &UA3068907},
	{NULL}};
const Segmentdata DAndroidVersion4_4[] = {
	{SG101075, &UA2995043},
	{SG101076, &UA2998082},
	{SG101077, &UA3068860},
	{SG101078, &UA3068863},
	{SG101079, &UA3068861},
	{SG101080, &UA3068869},
	{SG101081, &UA3068868},
	{SG101082, &UA3068872},
	{SG101083, &UA3068871},
	{SG101147, &UA3068865},
	{SG101159, &UA3068862},
	{SG101160, &UA3068870},
	{SG101161, &UA3068874},
	{SG101162, &UA3068866},
	{NULL}};
const Segmentdata DMac[] = {
	{SG100680, &UA649795},
	{SG101113, &UA2508580},
	{SG101114, &UA2530393},
	{SG101115, &UA2583476},
	{SG101116, &UA2584133},
	{SG101117, &UA2604433},
	{SG101118, &UA2739549},
	{SG101119, &UA2783010},
	{SG101120, &UA2796930},
	{SG101121, &UA2869366},
	{SG101122, &UA2892998},
	{SG101123, &UA2972565},
	{SG101124, &UA3083088},
	{SG101125, &UA3093586},
	{SG101126, &UA3095966},
	{SG101130, &UA2515659},
	{SG101131, &UA2536411},
	{SG101132, &UA2584238},
	{SG101133, &UA2739941},
	{SG101134, &UA2869371},
	{SG101135, &UA3019791},
	{SG101136, &UA3109410},
	{SG101137, &UA606828},
	{SG101138, &UA613759},
	{SG101139, &UA628117},
	{SG101140, &UA630540},
	{SG101141, &UA798864},
	{SG101142, &UA1991778},
	{SG101143, &UA2234795},
	{SG101144, &UA2389776},
	{SG101145, &UA2416622},
	{SG101146, &UA2831111},
	{SG101148, &UA1996334},
	{SG101149, &UA2182911},
	{SG101150, &UA2232713},
	{SG101151, &UA2242457},
	{SG101152, &UA2389169},
	{SG101153, &UA2706078},
	{SG101154, &UA2749170},
	{SG101155, &UA2752831},
	{SG101156, &UA2964081},
	{SG101157, &UA3019450},
	{SG101158, &UA3110901},
	{SG101163, &UA583660},
	{SG101164, &UA659108},
	{SG101165, &UA2334046},
	{SG101166, &UA2830226},
	{SG101167, &UA2523864},
	{SG101168, &UA2668257},
	{SG101169, &UA3000064},
	{SG101170, &UA680653},
	{SG101171, &UA2393160},
	{SG101172, &UA2457905},
	{SG101173, &UA2828324},
	{SG101174, &UA2418374},
	{SG101175, &UA2989547},
	{SG101176, &UA3009611},
	{SG101177, &UA575301},
	{SG101178, &UA2324979},
	{SG101179, &UA2827749},
	{SG101180, &UA552175},
	{SG101181, &UA559068},
	{SG101182, &UA2652224},
	{SG101183, &UA568554},
	{SG101184, &UA726073},
	{SG101185, &UA572549},
	{SG101186, &UA574827},
	{NULL}};


/*Edit Distance: Find the minimum of three integers*/
__inline int findMinimum (int a, int b, int c) {
	int min = a;
	if (b < min) min = b;
	if (c < min) min = c;
	return min;
}

/*Edit Distance: Calculate Levenshtein distance*/
int editDistance (Workset *ws, const char *str1, int length1, const char *str2, int length2, int maxValue) {
	int i = 0, j = 0;
	int lowest = INT_MAX;
	int cell;
	int *tmpVector; /*For vector substitution*/
	char str1_i, str2_j;

	/*If either string is empty, return the length of the other string*/
	if (length1 == 0) return length2;
	if (length2 == 0) return length1;

	/*Populate the first vector with numbers up to str2's string length*/
	for (i = 0; i <= length2; i++) ws->vector1[i] = i;

	/*Begin Levenshtein calculation*/
	for (i = 1; i <= length1; i++) {
		ws->vector2[0] = i;
		str1_i = str1[i-1];
		lowest = INT_MAX;

		for (j = 1; j <= length2; j++) {
			str2_j = str2[j-1];

			/*Find minimum value and assign to cell*/
			cell = findMinimum(
				ws->vector2[j - 1] + 1,
				ws->vector1[j] + 1,
				ws->vector1[j - 1] + (str1_i == str2_j ? 0 : 1));
			ws->vector2[j] = cell;

			/*Check for lowest value for optimization purposes*/
			if (cell < lowest) lowest = cell;
		}

		if (lowest > maxValue) return INT_MAX;

		/*Move the two vectors along one space*/
		tmpVector = ws->vector1;
		ws->vector1 = ws->vector2;
		ws->vector2 = tmpVector;
	}
	return ws->vector1[length2];
}

/*Calculate the score of a segment based on edit distance and weight*/
void calculateSegmentScore(Workset *ws, const Segment * const targetSeg, regmatch_t *matchList, int weight) {
	int i, size1, size2;
	char *userAgentStart;

	/*Compare the total elements from both arrays*/
	for(size1 = 0; targetSeg[size1].length != 0; size1++);
	for(size2 = 0; matchList[size2].rm_eo - matchList[size2].rm_so != 0; size2++);
	if (size1 == 0) size1 = 1;
	if (size2 == 0) size2 = 1;
	if(size1 != size2) {
		ws->score = INT_MAX;
		return;
	}

	/*Increment the score for each of the available segments*/
	for(i = 0; i < size1; i++) {
		userAgentStart = &ws->userAgent[matchList[i].rm_so];
		ws->score += (editDistance(
				ws,
				userAgentStart,
				(int)(&ws->userAgent[matchList[i].rm_eo] - userAgentStart),
				targetSeg[i].start,
				targetSeg[i].length,
				INT_MAX) * weight);

		/*The score is now larger than the best match we've got already*/
		if (ws->score > ws->lowScore)
		{
			ws->score = INT_MAX;
			return;
		}
	}

	return;
}

/*Allocate memory for workSet and set values*/
Workset *createWorkset(void) {
	Workset *ws = (Workset*) malloc(sizeof(Workset));

	ws->vector1 = (int*) malloc(sizeof(int) * MAXBUFFER);
	ws->vector2 = (int*) malloc(sizeof(int) * MAXBUFFER);

	/* Null terminate the input string. */
	*(ws->input + MAXBUFFER) = '\0';

	return ws;
}

/*Free up memory from workSet*/
void freeWorkset(Workset *ws) {
	free(ws->vector1);
	free(ws->vector2);
	free(ws);
}

/*Clears the regMatch variable within a workset*/
void clearSegments(Workset *ws) {
	memset(ws->regMatch, 0, sizeof(ws->regMatch));
}

/*Returns the character at the given position in the device id*/
__inline char getNextCharacter(Device *device, int *profile, int *position) {
	char current = device->profiles[*profile]->id[*position];

	/*Is there a null for the current profile?*/
	if (current == '\0') {
		/*Move to the next profile and return - to indicate a break*/
		(*profile)++;
		(*position) = 0;
		return PROFILE_SEPERATOR;
	}

	/*Move to the next character*/
	(*position)++;

	return current;
}

/*For use in qsort with final matcher to place returned devices in known order*/
int CDECL compare (const void *a, const void *b) {
	int result;
	int profileA = 0, profileB = 0;
	int positionA = 0, positionB = 0;
	Device *deviceA = *(Device**)a;
	Device *deviceB = *(Device**)b;

	while(profileA < PROFILES && profileB < PROFILES) {
		result = getNextCharacter(deviceA, &profileA, &positionA) -
				 getNextCharacter(deviceB, &profileB, &positionB);
		if (result != 0)
			return result;
	}

	/*They're equal*/
	return 0;
}

/*Checks a string to see if it's equal. Returns the value of the first unequal char*/
__inline int checkString(char* str1, int length1, const char * const str2, int length2) {
	int i = 0;
	while (str1[i] == str2[i] && i < length1 && i < length2) i++;
	return i;
}

/*Reduces the list of possible devices by looking at the start of the user agent*/
__inline int finalShortListFirstPass(Workset *ws) {
	int i, score, fullCount = ws->count;

	ws->score = 0;

	for (i = 0; i < fullCount; i++) {
		score = checkString(
			ws->userAgent,
			ws->length,
			ws->devices[i]->userAgent,
			ws->devices[i]->length);
		if (score >= ws->score) {

			/*Reset the list if the result is greater*/
			if (score > ws->score) {
				ws->count = 0;
				ws->score = score;
			}

			/*Place this device at the next position in the list*/
			ws->devices[ws->count] = ws->devices[i];
			ws->count++;
		}
	}

	return ws->count == 1 ? 1 : 0;
}

/*Returns 1 if there is an exact match and list is updated*/
__inline int finalShortListExactMatch(Workset *ws) {
	int i;

	if (ws->score == ws->length) {
		for (i = 0; i < ws->count; i++) {
			if (ws->devices[i]->length == ws->length &&
				strcmp(ws->devices[i]->userAgent, ws->userAgent) == 0) {
				ws->devices[0] = ws->devices[i];
				return 1;
			}
		}
	}

	return 0;
}

/*Gets the length of the longest tail from the list of devices*/
__inline int finalShortListLongestTailLength(Workset *ws) {
	int i, length, longestTailLength = 0;

	for (i = 0; i < ws->count; i++) {
		length = ws->devices[i]->length - ws->score;
		if (length > longestTailLength)
			longestTailLength = length;
	}

	return longestTailLength;
}

/*Compares the ends of user agents to find the best match*/
__inline void finalShortListSecondPass(Workset *ws) {
	int i, userAgentLength, lowestDistance, finalTailLength, longestTailLength;
	const char *finalTail = NULL;

	/*Get the longest length of the tail*/
	longestTailLength = finalShortListLongestTailLength(ws);

	/*Get the length of the test useragent to compare*/
	userAgentLength =
		(ws->score + longestTailLength) < ws->length ? longestTailLength :
			(ws->length - ws->score) > 0 ? ws->length - ws->score : 0;

	/*Get the tail with the lowest score compared to the test useragent*/
	ws->lowScore = INT_MAX;
	for (i = 0; i < ws->count; i++) {
		lowestDistance = editDistance(
			ws,
			&ws->devices[i]->userAgent[ws->score],
			ws->devices[i]->length - ws->score,
			&ws->userAgent[ws->score],
			userAgentLength,
			ws->lowScore);
		if (lowestDistance < ws->lowScore) {
			ws->lowScore = lowestDistance;
			finalTail = &ws->devices[i]->userAgent[ws->score];
			finalTailLength = ws->devices[i]->length - ws->score;
		}
	}

	/*Return the first device which matches exactly*/
	if (finalTail != NULL) {
		for (i = 0; i < ws->count; i++) {
			if (strcmp(
				finalTail,
				&ws->devices[i]->userAgent[ws->devices[i]->length - finalTailLength]) == 0) {
				ws->devices[0] = ws->devices[i];
				break;
			}
		}
	}

	/*Use the 1st device in the array as the result*/
	ws->count = 1;

	return;
}

/*Called if more than one device is returned*/
__inline void calculateFinalShortList(Workset *ws) {

	/*Ensure the list of devices is in a consistent order*/
	qsort((void*)ws->devices, ws->count, sizeof(Device*), compare);

	/*Compare start of devices user agents*/
	if (finalShortListFirstPass(ws) == 1) return;

	/*If the position and the length are the same then there is an exact match*/
	if (finalShortListExactMatch(ws) == 1) return;

	/*Compare ends of devices user agents*/
	finalShortListSecondPass(ws);

	return;
}

/*Split up useragent into segments*/
void splitRegex(Workset *ws, regex_t *reg, regmatch_t *match) {
	int result, i = 0, j = 0;

	do {
		result = fiftyone_degrees_regexec(
			reg,
			&ws->userAgent[j],
			1,
			&match[i],
			(j == 0 ? 0 : REG_NOTBOL));
		if (result == 0) {
			match[i].rm_so += j;
			match[i].rm_eo += j;
			j = match[i].rm_eo;
			i++;
		}
	} while (result == 0);
}

/*Check the new and low scores, adding the submitted device if nessessary*/
void scoreCheck(Workset *ws, const Device * const checkDevice) {
	if (ws->score == ws->lowScore) {
		ws->devices[ws->count] = checkDevice;
		ws->count++;
		return;
	}
	if (ws->score < ws->lowScore) {
		ws->devices[0] = checkDevice;
		ws->count = 1;
		ws->lowScore = ws->score;
		return;
	}
}

/*Performs a score check on an edit distance result*/
void editDistanceScoreCheck (Workset *ws, const Device * const checkDevice) {
	ws->score = editDistance(
		ws,
		checkDevice->userAgent,
		checkDevice->length,
		ws->userAgent,
		ws->length,
		ws->lowScore);
	scoreCheck(ws, checkDevice);
}

/*Returns 1 if the character is white space, otherwise 0.*/
int __inline isWhiteSpace(char character) {
	return (character >= 0x09 && character <= 0x0D) || character == 0x20 || character == 0x0;
}

/*Copy the input user agent into a new char array with leading and trailing spaces removed*/
void trimUserAgent(Workset *ws) {
	char *current, *lastWhiteSpace = NULL;

	ws->userAgent = ws->input;

	/*Remove leading whitespace*/
	while (isWhiteSpace(*ws->userAgent)) {
		ws->userAgent++;
	}

	/* Set the current character to check to the start of the string. */
	current = ws->userAgent;

	/*Remove trailing whitespace*/
	while (*current != '\0') {
		if (isWhiteSpace(*current)) {
			if (lastWhiteSpace == NULL)
				lastWhiteSpace = current;
		} else {
			lastWhiteSpace = NULL;
		}
		current++;
	}

	/*Close off the modified string*/
	if (lastWhiteSpace != NULL)
		*lastWhiteSpace = '\0';
	else
		lastWhiteSpace = current;

	/*Update the new length value*/
	ws->length = (int)(lastWhiteSpace - ws->userAgent);
}

/*Print the profile id to the result buffer*/
int printProfileId(const Device *subject, char* result, int resultLength) {
	char* currentPos = result;
	char* endPos = result + resultLength;
	int i;
	for(i = 0; i < PROFILES; i++) {
		currentPos += snprintf(
			currentPos,
			endPos - currentPos,
			"%s",
			subject->profiles[i]->id);
		if (i < PROFILES - 1)
			currentPos += snprintf(
				currentPos,
				endPos - currentPos,
				"-");
	}
	return (int)(currentPos - result);
}

// Returns none zero if the property is required.
int __inline isPropertyRequired(const char * const propertyName) {
	int i;

	// If no properties are specified return them all.
	if (_propertiesCount == 0)
		return 1;

	// Check to see if this property is present.
	for(i = 0; i < _propertiesCount; i++) {
		if (strcmp(propertyName, *(_properties + i)) == 0)
			return 1;
	}

	return 0;
}

/*Process device properties into a CSV string*/
int processDeviceCSV(const Device *subject, char* result, int resultLength) {
	char* currentPos = result;
	char* endPos = result + resultLength;
	int i, length;
	const Property *p;
	const Value *v;

	if (isPropertyRequired("Id")) {
		/*Generate the identifier line*/
		currentPos += snprintf(
			currentPos,
			(int)(endPos - currentPos),
			"Id|");

		/*Add each of the profiles seperated by a hyphen*/
		currentPos += printProfileId(
			subject,
			currentPos,
			(int)(endPos - currentPos));

		/*Terminate the lines.*/
		currentPos += snprintf(
			currentPos,
			(int)(endPos - currentPos),
			"\n");
	}

	/*Process each line of data using the relevant value separator. In this case, a comma*/
	for(i = 0; i < PROFILES; i++) {
		p = subject->profiles[i]->properties;
		while (p != NULL) {
			if (isPropertyRequired(p->id)) {
				currentPos += snprintf(currentPos, endPos - currentPos, "%s|", p->id);
				v = p->values;
				while (v != NULL) {
					currentPos += snprintf(currentPos, endPos - currentPos, "%s", v->valueString);
					if (v->next != NULL) currentPos += snprintf(currentPos, endPos - currentPos, "|");
					v = v->next;
				}
				currentPos += snprintf(currentPos, endPos - currentPos, "\n");
			}
			p = p->next;
		}
	}

	/* If nothing returned set an empty string */
	length = (int)(currentPos - result);
	if (length == 0)
		*currentPos = '\0';

	/*Return the length of the string buffer used*/
	return length;
}

/*Process device properties into an XML string*/
int processDeviceXML(const Device *subject, char* result, int resultLength) {
	char* currentPos = result;
	char* endPos = result + resultLength;
	int i;
	const Property *p;
	const Value *v;

	/*Generate the identifier line*/

	currentPos += snprintf(currentPos, endPos - currentPos, "<device>\n");
	currentPos += snprintf(
		currentPos,
		endPos - currentPos,
		"<Id multiple=\"0\">%s-%s-%s-%s</Id>\n",
		subject->profiles[0]->id,
		subject->profiles[1]->id,
		subject->profiles[2]->id,
		subject->profiles[3]->id);

	/*Process each line of data using the relevant formatting for XML*/
	for(i = 0; i < PROFILES; i++) {
		p = subject->profiles[i]->properties;
		while (p != NULL) {
			currentPos += snprintf(currentPos, endPos - currentPos, "<%s ", p->id);
			v = p->values;
			if (v->next == NULL) currentPos += snprintf(currentPos, endPos - currentPos, "multiple=\"0\">");
			else currentPos += snprintf(currentPos, endPos - currentPos, "multiple=\"1\">");
			while (v != NULL) {
				currentPos += snprintf(currentPos, endPos - currentPos, "<value>%s</value>", v->valueString);
				v = v->next;
			}
			currentPos += snprintf(currentPos, endPos - currentPos, "</%s>\n", p->id);
			p = p->next;
		}
	}
	currentPos += snprintf(currentPos, endPos - currentPos, "</device>\n");
	/*Return the length of the string buffer used*/
	return (int)(currentPos - result);
}

/*Process device properties into a JSON string*/
int processDeviceJSON(const Device *subject, char* result, int resultLength) {
	char* currentPos = result;
	char* endPos = result + resultLength;
	int i;
	const Property *p;
	const Value *v;

	/*Generate the identifier line*/

	currentPos += snprintf(currentPos, endPos - currentPos, "{");
	currentPos += snprintf(
		currentPos,
		endPos - currentPos,
		"\"Id\":\"%s-%s-%s-%s\",",
		subject->profiles[0]->id,
		subject->profiles[1]->id,
		subject->profiles[2]->id,
		subject->profiles[3]->id);

	/*Process each line of data using the relevant formatting for JSON*/
	for(i = 0; i < PROFILES; i++) {
		p = subject->profiles[i]->properties;
		while (p != NULL) {
			currentPos += snprintf(currentPos, endPos - currentPos, "\"%s\":", p->id);
			v = p->values;
			if (v->next == NULL) {
				currentPos += snprintf(currentPos, endPos - currentPos, "\"%s\"", v->valueString);
			} else {
				currentPos += snprintf(currentPos, endPos - currentPos, "{");
				while (v != NULL) {
					currentPos += snprintf(currentPos, endPos - currentPos, "\"value\":\"%s\"", v->valueString);
					v = v->next;
					if (v != NULL) currentPos += snprintf(currentPos, endPos - currentPos, ",");
				}
				currentPos += snprintf(currentPos, endPos - currentPos, "}");
			}
			currentPos += snprintf(currentPos, endPos - currentPos, ",\n");
			p = p->next;
		}
	}
	currentPos += snprintf(currentPos, endPos - currentPos, "}\n");
	/*Return the length of the string buffer used*/
	return (int)(currentPos - result);
}

/*
 * Individual handler category assignment functions start here.
 */


__inline void EAlcatel(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 1);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAlcatel(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R320, ws->regMatch[0]);
	splitRegex(ws, &R2, ws->regMatch[1]);
	for(i = 0; DAlcatel[i].segments != NULL; i++)
		EAlcatel(ws, DAlcatel[i].segments, DAlcatel[i].devices);
}
void HAmoi(Workset *ws) {
	int i;
	for (i = 0; DAmoi[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DAmoi[i]);
}
__inline void EAndroidNonNumeric(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[18], ws->regMatch[18], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[19], ws->regMatch[19], 1000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidNonNumeric(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R10, ws->regMatch[8]);
	splitRegex(ws, &R22, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R26, ws->regMatch[11]);
	splitRegex(ws, &R27, ws->regMatch[12]);
	splitRegex(ws, &R111, ws->regMatch[13]);
	splitRegex(ws, &R112, ws->regMatch[14]);
	splitRegex(ws, &R113, ws->regMatch[15]);
	splitRegex(ws, &R114, ws->regMatch[16]);
	splitRegex(ws, &R31, ws->regMatch[17]);
	splitRegex(ws, &R115, ws->regMatch[18]);
	splitRegex(ws, &R116, ws->regMatch[19]);
	for(i = 0; DAndroidNonNumeric[i].segments != NULL; i++)
		EAndroidNonNumeric(ws, DAndroidNonNumeric[i].segments, DAndroidNonNumeric[i].devices);
}
__inline void EAndroidVersion1(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion1(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R25, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	for(i = 0; DAndroidVersion1[i].segments != NULL; i++)
		EAndroidVersion1(ws, DAndroidVersion1[i].segments, DAndroidVersion1[i].devices);
}
__inline void EAndroidVersion2(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion2(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R25, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	for(i = 0; DAndroidVersion2[i].segments != NULL; i++)
		EAndroidVersion2(ws, DAndroidVersion2[i].segments, DAndroidVersion2[i].devices);
}
__inline void EAndroidVersion2_1(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[18], ws->regMatch[18], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion2_1(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R24, ws->regMatch[10]);
	splitRegex(ws, &R23, ws->regMatch[11]);
	splitRegex(ws, &R25, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R51, ws->regMatch[16]);
	splitRegex(ws, &R31, ws->regMatch[17]);
	splitRegex(ws, &R35, ws->regMatch[18]);
	for(i = 0; DAndroidVersion2_1[i].segments != NULL; i++)
		EAndroidVersion2_1(ws, DAndroidVersion2_1[i].segments, DAndroidVersion2_1[i].devices);
}
__inline void EAndroidVersion2_2(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[18], ws->regMatch[18], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[19], ws->regMatch[19], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[20], ws->regMatch[20], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[21], ws->regMatch[21], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[22], ws->regMatch[22], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion2_2(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R25, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R121, ws->regMatch[16]);
	splitRegex(ws, &R33, ws->regMatch[17]);
	splitRegex(ws, &R38, ws->regMatch[18]);
	splitRegex(ws, &R56, ws->regMatch[19]);
	splitRegex(ws, &R31, ws->regMatch[20]);
	splitRegex(ws, &R122, ws->regMatch[21]);
	splitRegex(ws, &R35, ws->regMatch[22]);
	for(i = 0; DAndroidVersion2_2[i].segments != NULL; i++)
		EAndroidVersion2_2(ws, DAndroidVersion2_2[i].segments, DAndroidVersion2_2[i].devices);
}
__inline void EAndroidVersion2_2_1(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[18], ws->regMatch[18], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[19], ws->regMatch[19], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[20], ws->regMatch[20], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[21], ws->regMatch[21], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[22], ws->regMatch[22], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[23], ws->regMatch[23], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion2_2_1(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R25, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R29, ws->regMatch[16]);
	splitRegex(ws, &R30, ws->regMatch[17]);
	splitRegex(ws, &R31, ws->regMatch[18]);
	splitRegex(ws, &R32, ws->regMatch[19]);
	splitRegex(ws, &R33, ws->regMatch[20]);
	splitRegex(ws, &R34, ws->regMatch[21]);
	splitRegex(ws, &R35, ws->regMatch[22]);
	splitRegex(ws, &R36, ws->regMatch[23]);
	for(i = 0; DAndroidVersion2_2_1[i].segments != NULL; i++)
		EAndroidVersion2_2_1(ws, DAndroidVersion2_2_1[i].segments, DAndroidVersion2_2_1[i].devices);
}
__inline void EAndroidVersion2_2_2(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[18], ws->regMatch[18], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion2_2_2(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R25, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R38, ws->regMatch[16]);
	splitRegex(ws, &R31, ws->regMatch[17]);
	splitRegex(ws, &R36, ws->regMatch[18]);
	for(i = 0; DAndroidVersion2_2_2[i].segments != NULL; i++)
		EAndroidVersion2_2_2(ws, DAndroidVersion2_2_2[i].segments, DAndroidVersion2_2_2[i].devices);
}
__inline void EAndroidVersion2_2_3(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion2_2_3(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R25, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R36, ws->regMatch[16]);
	for(i = 0; DAndroidVersion2_2_3[i].segments != NULL; i++)
		EAndroidVersion2_2_3(ws, DAndroidVersion2_2_3[i].segments, DAndroidVersion2_2_3[i].devices);
}
__inline void EAndroidVersion2_3(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion2_3(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R25, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R36, ws->regMatch[16]);
	for(i = 0; DAndroidVersion2_3[i].segments != NULL; i++)
		EAndroidVersion2_3(ws, DAndroidVersion2_3[i].segments, DAndroidVersion2_3[i].devices);
}
__inline void EAndroidVersion2_3_1(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion2_3_1(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R25, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R36, ws->regMatch[16]);
	for(i = 0; DAndroidVersion2_3_1[i].segments != NULL; i++)
		EAndroidVersion2_3_1(ws, DAndroidVersion2_3_1[i].segments, DAndroidVersion2_3_1[i].devices);
}
__inline void EAndroidVersion2_3_2(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion2_3_2(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R25, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R36, ws->regMatch[16]);
	for(i = 0; DAndroidVersion2_3_2[i].segments != NULL; i++)
		EAndroidVersion2_3_2(ws, DAndroidVersion2_3_2[i].segments, DAndroidVersion2_3_2[i].devices);
}
__inline void EAndroidVersion2_3_3(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[18], ws->regMatch[18], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[19], ws->regMatch[19], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[20], ws->regMatch[20], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[21], ws->regMatch[21], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[22], ws->regMatch[22], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[23], ws->regMatch[23], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[24], ws->regMatch[24], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[25], ws->regMatch[25], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[26], ws->regMatch[26], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion2_3_3(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R25, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R43, ws->regMatch[15]);
	splitRegex(ws, &R44, ws->regMatch[16]);
	splitRegex(ws, &R28, ws->regMatch[17]);
	splitRegex(ws, &R33, ws->regMatch[18]);
	splitRegex(ws, &R34, ws->regMatch[19]);
	splitRegex(ws, &R38, ws->regMatch[20]);
	splitRegex(ws, &R45, ws->regMatch[21]);
	splitRegex(ws, &R46, ws->regMatch[22]);
	splitRegex(ws, &R47, ws->regMatch[23]);
	splitRegex(ws, &R48, ws->regMatch[24]);
	splitRegex(ws, &R49, ws->regMatch[25]);
	splitRegex(ws, &R36, ws->regMatch[26]);
	for(i = 0; DAndroidVersion2_3_3[i].segments != NULL; i++)
		EAndroidVersion2_3_3(ws, DAndroidVersion2_3_3[i].segments, DAndroidVersion2_3_3[i].devices);
}
__inline void EAndroidVersion2_3_4(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[18], ws->regMatch[18], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[19], ws->regMatch[19], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[20], ws->regMatch[20], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[21], ws->regMatch[21], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[22], ws->regMatch[22], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[23], ws->regMatch[23], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[24], ws->regMatch[24], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[25], ws->regMatch[25], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[26], ws->regMatch[26], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion2_3_4(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R25, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R38, ws->regMatch[16]);
	splitRegex(ws, &R34, ws->regMatch[17]);
	splitRegex(ws, &R31, ws->regMatch[18]);
	splitRegex(ws, &R46, ws->regMatch[19]);
	splitRegex(ws, &R51, ws->regMatch[20]);
	splitRegex(ws, &R52, ws->regMatch[21]);
	splitRegex(ws, &R35, ws->regMatch[22]);
	splitRegex(ws, &R53, ws->regMatch[23]);
	splitRegex(ws, &R54, ws->regMatch[24]);
	splitRegex(ws, &R33, ws->regMatch[25]);
	splitRegex(ws, &R36, ws->regMatch[26]);
	for(i = 0; DAndroidVersion2_3_4[i].segments != NULL; i++)
		EAndroidVersion2_3_4(ws, DAndroidVersion2_3_4[i].segments, DAndroidVersion2_3_4[i].devices);
}
__inline void EAndroidVersion2_3_5(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[18], ws->regMatch[18], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[19], ws->regMatch[19], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[20], ws->regMatch[20], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[21], ws->regMatch[21], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[22], ws->regMatch[22], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[23], ws->regMatch[23], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[24], ws->regMatch[24], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion2_3_5(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R25, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R56, ws->regMatch[16]);
	splitRegex(ws, &R57, ws->regMatch[17]);
	splitRegex(ws, &R58, ws->regMatch[18]);
	splitRegex(ws, &R38, ws->regMatch[19]);
	splitRegex(ws, &R33, ws->regMatch[20]);
	splitRegex(ws, &R46, ws->regMatch[21]);
	splitRegex(ws, &R59, ws->regMatch[22]);
	splitRegex(ws, &R51, ws->regMatch[23]);
	splitRegex(ws, &R36, ws->regMatch[24]);
	for(i = 0; DAndroidVersion2_3_5[i].segments != NULL; i++)
		EAndroidVersion2_3_5(ws, DAndroidVersion2_3_5[i].segments, DAndroidVersion2_3_5[i].devices);
}
__inline void EAndroidVersion2_3_6(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[18], ws->regMatch[18], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[19], ws->regMatch[19], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[20], ws->regMatch[20], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[21], ws->regMatch[21], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[22], ws->regMatch[22], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[23], ws->regMatch[23], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion2_3_6(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R25, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R57, ws->regMatch[16]);
	splitRegex(ws, &R58, ws->regMatch[17]);
	splitRegex(ws, &R29, ws->regMatch[18]);
	splitRegex(ws, &R33, ws->regMatch[19]);
	splitRegex(ws, &R51, ws->regMatch[20]);
	splitRegex(ws, &R59, ws->regMatch[21]);
	splitRegex(ws, &R61, ws->regMatch[22]);
	splitRegex(ws, &R36, ws->regMatch[23]);
	for(i = 0; DAndroidVersion2_3_6[i].segments != NULL; i++)
		EAndroidVersion2_3_6(ws, DAndroidVersion2_3_6[i].segments, DAndroidVersion2_3_6[i].devices);
}
__inline void EAndroidVersion2_3_7(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[18], ws->regMatch[18], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[19], ws->regMatch[19], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[20], ws->regMatch[20], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[21], ws->regMatch[21], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[22], ws->regMatch[22], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[23], ws->regMatch[23], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[24], ws->regMatch[24], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[25], ws->regMatch[25], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion2_3_7(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R25, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R63, ws->regMatch[15]);
	splitRegex(ws, &R64, ws->regMatch[16]);
	splitRegex(ws, &R65, ws->regMatch[17]);
	splitRegex(ws, &R51, ws->regMatch[18]);
	splitRegex(ws, &R53, ws->regMatch[19]);
	splitRegex(ws, &R34, ws->regMatch[20]);
	splitRegex(ws, &R56, ws->regMatch[21]);
	splitRegex(ws, &R66, ws->regMatch[22]);
	splitRegex(ws, &R31, ws->regMatch[23]);
	splitRegex(ws, &R67, ws->regMatch[24]);
	splitRegex(ws, &R36, ws->regMatch[25]);
	for(i = 0; DAndroidVersion2_3_7[i].segments != NULL; i++)
		EAndroidVersion2_3_7(ws, DAndroidVersion2_3_7[i].segments, DAndroidVersion2_3_7[i].devices);
}
__inline void EAndroidVersion2_4(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion2_4(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R25, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R36, ws->regMatch[16]);
	for(i = 0; DAndroidVersion2_4[i].segments != NULL; i++)
		EAndroidVersion2_4(ws, DAndroidVersion2_4[i].segments, DAndroidVersion2_4[i].devices);
}
__inline void EAndroidVersion3(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[18], ws->regMatch[18], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[19], ws->regMatch[19], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[20], ws->regMatch[20], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[21], ws->regMatch[21], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[22], ws->regMatch[22], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[23], ws->regMatch[23], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[24], ws->regMatch[24], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[25], ws->regMatch[25], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion3(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R25, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R34, ws->regMatch[15]);
	splitRegex(ws, &R31, ws->regMatch[16]);
	splitRegex(ws, &R33, ws->regMatch[17]);
	splitRegex(ws, &R126, ws->regMatch[18]);
	splitRegex(ws, &R127, ws->regMatch[19]);
	splitRegex(ws, &R51, ws->regMatch[20]);
	splitRegex(ws, &R38, ws->regMatch[21]);
	splitRegex(ws, &R53, ws->regMatch[22]);
	splitRegex(ws, &R128, ws->regMatch[23]);
	splitRegex(ws, &R36, ws->regMatch[24]);
	splitRegex(ws, &R129, ws->regMatch[25]);
	for(i = 0; DAndroidVersion3[i].segments != NULL; i++)
		EAndroidVersion3(ws, DAndroidVersion3[i].segments, DAndroidVersion3[i].devices);
}
__inline void EAndroidVersion4_0(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion4_0(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R132, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R36, ws->regMatch[16]);
	for(i = 0; DAndroidVersion4_0[i].segments != NULL; i++)
		EAndroidVersion4_0(ws, DAndroidVersion4_0[i].segments, DAndroidVersion4_0[i].devices);
}
__inline void EAndroidVersion4_0_1(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion4_0_1(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R132, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R134, ws->regMatch[16]);
	splitRegex(ws, &R36, ws->regMatch[17]);
	for(i = 0; DAndroidVersion4_0_1[i].segments != NULL; i++)
		EAndroidVersion4_0_1(ws, DAndroidVersion4_0_1[i].segments, DAndroidVersion4_0_1[i].devices);
}
__inline void EAndroidVersion4_0_2(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion4_0_2(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R132, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R36, ws->regMatch[16]);
	for(i = 0; DAndroidVersion4_0_2[i].segments != NULL; i++)
		EAndroidVersion4_0_2(ws, DAndroidVersion4_0_2[i].segments, DAndroidVersion4_0_2[i].devices);
}
__inline void EAndroidVersion4_0_3(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[18], ws->regMatch[18], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[19], ws->regMatch[19], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[20], ws->regMatch[20], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[21], ws->regMatch[21], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[22], ws->regMatch[22], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[23], ws->regMatch[23], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[24], ws->regMatch[24], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[25], ws->regMatch[25], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion4_0_3(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R132, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R137, ws->regMatch[16]);
	splitRegex(ws, &R31, ws->regMatch[17]);
	splitRegex(ws, &R33, ws->regMatch[18]);
	splitRegex(ws, &R127, ws->regMatch[19]);
	splitRegex(ws, &R29, ws->regMatch[20]);
	splitRegex(ws, &R51, ws->regMatch[21]);
	splitRegex(ws, &R138, ws->regMatch[22]);
	splitRegex(ws, &R53, ws->regMatch[23]);
	splitRegex(ws, &R36, ws->regMatch[24]);
	splitRegex(ws, &R129, ws->regMatch[25]);
	for(i = 0; DAndroidVersion4_0_3[i].segments != NULL; i++)
		EAndroidVersion4_0_3(ws, DAndroidVersion4_0_3[i].segments, DAndroidVersion4_0_3[i].devices);
}
__inline void EAndroidVersion4_0_4(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[18], ws->regMatch[18], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[19], ws->regMatch[19], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[20], ws->regMatch[20], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[21], ws->regMatch[21], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[22], ws->regMatch[22], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[23], ws->regMatch[23], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[24], ws->regMatch[24], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[25], ws->regMatch[25], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[26], ws->regMatch[26], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[27], ws->regMatch[27], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[28], ws->regMatch[28], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[29], ws->regMatch[29], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion4_0_4(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R132, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R33, ws->regMatch[16]);
	splitRegex(ws, &R34, ws->regMatch[17]);
	splitRegex(ws, &R31, ws->regMatch[18]);
	splitRegex(ws, &R127, ws->regMatch[19]);
	splitRegex(ws, &R52, ws->regMatch[20]);
	splitRegex(ws, &R46, ws->regMatch[21]);
	splitRegex(ws, &R51, ws->regMatch[22]);
	splitRegex(ws, &R53, ws->regMatch[23]);
	splitRegex(ws, &R140, ws->regMatch[24]);
	splitRegex(ws, &R38, ws->regMatch[25]);
	splitRegex(ws, &R36, ws->regMatch[26]);
	splitRegex(ws, &R141, ws->regMatch[27]);
	splitRegex(ws, &R142, ws->regMatch[28]);
	splitRegex(ws, &R129, ws->regMatch[29]);
	for(i = 0; DAndroidVersion4_0_4[i].segments != NULL; i++)
		EAndroidVersion4_0_4(ws, DAndroidVersion4_0_4[i].segments, DAndroidVersion4_0_4[i].devices);
}
__inline void EAndroidVersion4_0_5(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion4_0_5(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R10, ws->regMatch[3]);
	splitRegex(ws, &R26, ws->regMatch[4]);
	splitRegex(ws, &R27, ws->regMatch[5]);
	splitRegex(ws, &R36, ws->regMatch[6]);
	for(i = 0; DAndroidVersion4_0_5[i].segments != NULL; i++)
		EAndroidVersion4_0_5(ws, DAndroidVersion4_0_5[i].segments, DAndroidVersion4_0_5[i].devices);
}
__inline void EAndroidVersion4_1(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion4_1(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R132, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R36, ws->regMatch[16]);
	for(i = 0; DAndroidVersion4_1[i].segments != NULL; i++)
		EAndroidVersion4_1(ws, DAndroidVersion4_1[i].segments, DAndroidVersion4_1[i].devices);
}
__inline void EAndroidVersion4_1_1(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[18], ws->regMatch[18], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[19], ws->regMatch[19], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[20], ws->regMatch[20], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[21], ws->regMatch[21], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[22], ws->regMatch[22], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[23], ws->regMatch[23], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[24], ws->regMatch[24], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[25], ws->regMatch[25], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[26], ws->regMatch[26], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[27], ws->regMatch[27], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion4_1_1(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R132, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R137, ws->regMatch[16]);
	splitRegex(ws, &R31, ws->regMatch[17]);
	splitRegex(ws, &R127, ws->regMatch[18]);
	splitRegex(ws, &R33, ws->regMatch[19]);
	splitRegex(ws, &R52, ws->regMatch[20]);
	splitRegex(ws, &R147, ws->regMatch[21]);
	splitRegex(ws, &R148, ws->regMatch[22]);
	splitRegex(ws, &R38, ws->regMatch[23]);
	splitRegex(ws, &R149, ws->regMatch[24]);
	splitRegex(ws, &R36, ws->regMatch[25]);
	splitRegex(ws, &R142, ws->regMatch[26]);
	splitRegex(ws, &R129, ws->regMatch[27]);
	for(i = 0; DAndroidVersion4_1_1[i].segments != NULL; i++)
		EAndroidVersion4_1_1(ws, DAndroidVersion4_1_1[i].segments, DAndroidVersion4_1_1[i].devices);
}
__inline void EAndroidVersion4_1_2(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[18], ws->regMatch[18], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[19], ws->regMatch[19], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[20], ws->regMatch[20], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[21], ws->regMatch[21], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[22], ws->regMatch[22], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[23], ws->regMatch[23], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[24], ws->regMatch[24], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[25], ws->regMatch[25], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion4_1_2(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R132, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R52, ws->regMatch[16]);
	splitRegex(ws, &R34, ws->regMatch[17]);
	splitRegex(ws, &R53, ws->regMatch[18]);
	splitRegex(ws, &R31, ws->regMatch[19]);
	splitRegex(ws, &R33, ws->regMatch[20]);
	splitRegex(ws, &R149, ws->regMatch[21]);
	splitRegex(ws, &R38, ws->regMatch[22]);
	splitRegex(ws, &R36, ws->regMatch[23]);
	splitRegex(ws, &R141, ws->regMatch[24]);
	splitRegex(ws, &R151, ws->regMatch[25]);
	for(i = 0; DAndroidVersion4_1_2[i].segments != NULL; i++)
		EAndroidVersion4_1_2(ws, DAndroidVersion4_1_2[i].segments, DAndroidVersion4_1_2[i].devices);
}
__inline void EAndroidVersion4_2(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion4_2(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R132, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R36, ws->regMatch[16]);
	for(i = 0; DAndroidVersion4_2[i].segments != NULL; i++)
		EAndroidVersion4_2(ws, DAndroidVersion4_2[i].segments, DAndroidVersion4_2[i].devices);
}
__inline void EAndroidVersion4_2_1(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[18], ws->regMatch[18], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[19], ws->regMatch[19], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[20], ws->regMatch[20], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion4_2_1(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R132, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R52, ws->regMatch[16]);
	splitRegex(ws, &R155, ws->regMatch[17]);
	splitRegex(ws, &R33, ws->regMatch[18]);
	splitRegex(ws, &R36, ws->regMatch[19]);
	splitRegex(ws, &R44, ws->regMatch[20]);
	for(i = 0; DAndroidVersion4_2_1[i].segments != NULL; i++)
		EAndroidVersion4_2_1(ws, DAndroidVersion4_2_1[i].segments, DAndroidVersion4_2_1[i].devices);
}
__inline void EAndroidVersion4_2_2(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[18], ws->regMatch[18], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[19], ws->regMatch[19], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[20], ws->regMatch[20], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[21], ws->regMatch[21], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[22], ws->regMatch[22], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[23], ws->regMatch[23], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[24], ws->regMatch[24], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[25], ws->regMatch[25], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion4_2_2(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R132, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R52, ws->regMatch[16]);
	splitRegex(ws, &R155, ws->regMatch[17]);
	splitRegex(ws, &R157, ws->regMatch[18]);
	splitRegex(ws, &R33, ws->regMatch[19]);
	splitRegex(ws, &R31, ws->regMatch[20]);
	splitRegex(ws, &R149, ws->regMatch[21]);
	splitRegex(ws, &R36, ws->regMatch[22]);
	splitRegex(ws, &R158, ws->regMatch[23]);
	splitRegex(ws, &R159, ws->regMatch[24]);
	splitRegex(ws, &R129, ws->regMatch[25]);
	for(i = 0; DAndroidVersion4_2_2[i].segments != NULL; i++)
		EAndroidVersion4_2_2(ws, DAndroidVersion4_2_2[i].segments, DAndroidVersion4_2_2[i].devices);
}
void HAppleCoreMedia(Workset *ws) {
	int i;
	for (i = 0; DAppleCoreMedia[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DAppleCoreMedia[i]);
}
__inline void EApple(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 2);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 2);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 2);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HApple(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R164, ws->regMatch[0]);
	splitRegex(ws, &R165, ws->regMatch[1]);
	splitRegex(ws, &R166, ws->regMatch[2]);
	splitRegex(ws, &R167, ws->regMatch[3]);
	splitRegex(ws, &R168, ws->regMatch[4]);
	splitRegex(ws, &R169, ws->regMatch[5]);
	splitRegex(ws, &R170, ws->regMatch[6]);
	splitRegex(ws, &R171, ws->regMatch[7]);
	splitRegex(ws, &R172, ws->regMatch[8]);
	splitRegex(ws, &R173, ws->regMatch[9]);
	splitRegex(ws, &R174, ws->regMatch[10]);
	splitRegex(ws, &R175, ws->regMatch[11]);
	splitRegex(ws, &R31, ws->regMatch[12]);
	splitRegex(ws, &R176, ws->regMatch[13]);
	splitRegex(ws, &R29, ws->regMatch[14]);
	splitRegex(ws, &R177, ws->regMatch[15]);
	for(i = 0; DApple[i].segments != NULL; i++)
		EApple(ws, DApple[i].segments, DApple[i].devices);
}
void HAudioVox(Workset *ws) {
	int i;
	for (i = 0; DAudioVox[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DAudioVox[i]);
}
void HBenQ(Workset *ws) {
	int i;
	for (i = 0; DBenQ[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DBenQ[i]);
}
__inline void EBolt(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 1);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HBolt(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R181, ws->regMatch[0]);
	for(i = 0; DBolt[i].segments != NULL; i++)
		EBolt(ws, DBolt[i].segments, DBolt[i].devices);
}
__inline void EBot(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 1);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HBot(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R1, ws->regMatch[0]);
	splitRegex(ws, &R2, ws->regMatch[1]);
	for(i = 0; DBot[i].segments != NULL; i++)
		EBot(ws, DBot[i].segments, DBot[i].devices);
}
void HBird(Workset *ws) {
	int i;
	for (i = 0; DBird[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DBird[i]);
}
__inline void EBlackBerry1(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 5);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 100000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HBlackBerry1(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R332, ws->regMatch[0]);
	splitRegex(ws, &R333, ws->regMatch[1]);
	splitRegex(ws, &R334, ws->regMatch[2]);
	splitRegex(ws, &R335, ws->regMatch[3]);
	for(i = 0; DBlackBerry1[i].segments != NULL; i++)
		EBlackBerry1(ws, DBlackBerry1[i].segments, DBlackBerry1[i].devices);
}
__inline void EBlackBerry2(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HBlackBerry2(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R336, ws->regMatch[0]);
	splitRegex(ws, &R337, ws->regMatch[1]);
	for(i = 0; DBlackBerry2[i].segments != NULL; i++)
		EBlackBerry2(ws, DBlackBerry2[i].segments, DBlackBerry2[i].devices);
}
void HCatchAllEditDistance(Workset *ws) {
	int i;
	for (i = 0; DCatchAllEditDistance[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DCatchAllEditDistance[i]);
}
void HCatchAllMobile(Workset *ws) {
	int i;
	for (i = 0; DCatchAllMobile[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DCatchAllMobile[i]);
}
__inline void EChrome0_9(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 2);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HChrome0_9(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R70, ws->regMatch[0]);
	splitRegex(ws, &R71, ws->regMatch[1]);
	splitRegex(ws, &R72, ws->regMatch[2]);
	splitRegex(ws, &R73, ws->regMatch[3]);
	splitRegex(ws, &R74, ws->regMatch[4]);
	splitRegex(ws, &R75, ws->regMatch[5]);
	splitRegex(ws, &R76, ws->regMatch[6]);
	splitRegex(ws, &R77, ws->regMatch[7]);
	splitRegex(ws, &R34, ws->regMatch[8]);
	splitRegex(ws, &R78, ws->regMatch[9]);
	splitRegex(ws, &R79, ws->regMatch[10]);
	splitRegex(ws, &R80, ws->regMatch[11]);
	for(i = 0; DChrome0_9[i].segments != NULL; i++)
		EChrome0_9(ws, DChrome0_9[i].segments, DChrome0_9[i].devices);
}
__inline void EChrome10_19(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 2);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HChrome10_19(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R70, ws->regMatch[0]);
	splitRegex(ws, &R71, ws->regMatch[1]);
	splitRegex(ws, &R72, ws->regMatch[2]);
	splitRegex(ws, &R73, ws->regMatch[3]);
	splitRegex(ws, &R74, ws->regMatch[4]);
	splitRegex(ws, &R75, ws->regMatch[5]);
	splitRegex(ws, &R78, ws->regMatch[6]);
	splitRegex(ws, &R76, ws->regMatch[7]);
	splitRegex(ws, &R77, ws->regMatch[8]);
	splitRegex(ws, &R34, ws->regMatch[9]);
	splitRegex(ws, &R79, ws->regMatch[10]);
	splitRegex(ws, &R82, ws->regMatch[11]);
	splitRegex(ws, &R83, ws->regMatch[12]);
	splitRegex(ws, &R84, ws->regMatch[13]);
	splitRegex(ws, &R85, ws->regMatch[14]);
	splitRegex(ws, &R86, ws->regMatch[15]);
	for(i = 0; DChrome10_19[i].segments != NULL; i++)
		EChrome10_19(ws, DChrome10_19[i].segments, DChrome10_19[i].devices);
}
__inline void EChrome20_29(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 2);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HChrome20_29(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R70, ws->regMatch[0]);
	splitRegex(ws, &R71, ws->regMatch[1]);
	splitRegex(ws, &R72, ws->regMatch[2]);
	splitRegex(ws, &R73, ws->regMatch[3]);
	splitRegex(ws, &R74, ws->regMatch[4]);
	splitRegex(ws, &R75, ws->regMatch[5]);
	splitRegex(ws, &R76, ws->regMatch[6]);
	splitRegex(ws, &R77, ws->regMatch[7]);
	splitRegex(ws, &R34, ws->regMatch[8]);
	splitRegex(ws, &R88, ws->regMatch[9]);
	splitRegex(ws, &R36, ws->regMatch[10]);
	splitRegex(ws, &R84, ws->regMatch[11]);
	splitRegex(ws, &R85, ws->regMatch[12]);
	for(i = 0; DChrome20_29[i].segments != NULL; i++)
		EChrome20_29(ws, DChrome20_29[i].segments, DChrome20_29[i].devices);
}
__inline void EChrome30_39(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 2);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HChrome30_39(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R70, ws->regMatch[0]);
	splitRegex(ws, &R71, ws->regMatch[1]);
	splitRegex(ws, &R72, ws->regMatch[2]);
	splitRegex(ws, &R73, ws->regMatch[3]);
	splitRegex(ws, &R74, ws->regMatch[4]);
	splitRegex(ws, &R75, ws->regMatch[5]);
	splitRegex(ws, &R76, ws->regMatch[6]);
	splitRegex(ws, &R77, ws->regMatch[7]);
	splitRegex(ws, &R34, ws->regMatch[8]);
	splitRegex(ws, &R88, ws->regMatch[9]);
	splitRegex(ws, &R36, ws->regMatch[10]);
	for(i = 0; DChrome30_39[i].segments != NULL; i++)
		EChrome30_39(ws, DChrome30_39[i].segments, DChrome30_39[i].devices);
}
__inline void EDoCoMo(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 100000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HDoCoMo(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R339, ws->regMatch[0]);
	splitRegex(ws, &R340, ws->regMatch[1]);
	for(i = 0; DDoCoMo[i].segments != NULL; i++)
		EDoCoMo(ws, DDoCoMo[i].segments, DDoCoMo[i].devices);
}
__inline void EFirefoxDesktop(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 8000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 8000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[18], ws->regMatch[18], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[19], ws->regMatch[19], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[20], ws->regMatch[20], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[21], ws->regMatch[21], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[22], ws->regMatch[22], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[23], ws->regMatch[23], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[24], ws->regMatch[24], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[25], ws->regMatch[25], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[26], ws->regMatch[26], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[27], ws->regMatch[27], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[28], ws->regMatch[28], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[29], ws->regMatch[29], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[30], ws->regMatch[30], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[31], ws->regMatch[31], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[32], ws->regMatch[32], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[33], ws->regMatch[33], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HFirefoxDesktop(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R185, ws->regMatch[0]);
	splitRegex(ws, &R186, ws->regMatch[1]);
	splitRegex(ws, &R187, ws->regMatch[2]);
	splitRegex(ws, &R188, ws->regMatch[3]);
	splitRegex(ws, &R189, ws->regMatch[4]);
	splitRegex(ws, &R190, ws->regMatch[5]);
	splitRegex(ws, &R191, ws->regMatch[6]);
	splitRegex(ws, &R192, ws->regMatch[7]);
	splitRegex(ws, &R193, ws->regMatch[8]);
	splitRegex(ws, &R194, ws->regMatch[9]);
	splitRegex(ws, &R73, ws->regMatch[10]);
	splitRegex(ws, &R195, ws->regMatch[11]);
	splitRegex(ws, &R196, ws->regMatch[12]);
	splitRegex(ws, &R197, ws->regMatch[13]);
	splitRegex(ws, &R198, ws->regMatch[14]);
	splitRegex(ws, &R199, ws->regMatch[15]);
	splitRegex(ws, &R200, ws->regMatch[16]);
	splitRegex(ws, &R201, ws->regMatch[17]);
	splitRegex(ws, &R80, ws->regMatch[18]);
	splitRegex(ws, &R202, ws->regMatch[19]);
	splitRegex(ws, &R203, ws->regMatch[20]);
	splitRegex(ws, &R204, ws->regMatch[21]);
	splitRegex(ws, &R205, ws->regMatch[22]);
	splitRegex(ws, &R206, ws->regMatch[23]);
	splitRegex(ws, &R207, ws->regMatch[24]);
	splitRegex(ws, &R208, ws->regMatch[25]);
	splitRegex(ws, &R209, ws->regMatch[26]);
	splitRegex(ws, &R210, ws->regMatch[27]);
	splitRegex(ws, &R211, ws->regMatch[28]);
	splitRegex(ws, &R212, ws->regMatch[29]);
	splitRegex(ws, &R213, ws->regMatch[30]);
	splitRegex(ws, &R214, ws->regMatch[31]);
	splitRegex(ws, &R215, ws->regMatch[32]);
	splitRegex(ws, &R216, ws->regMatch[33]);
	for(i = 0; DFirefoxDesktop[i].segments != NULL; i++)
		EFirefoxDesktop(ws, DFirefoxDesktop[i].segments, DFirefoxDesktop[i].devices);
}
__inline void EFirefoxFennec(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 5);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 5);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 5);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HFirefoxFennec(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R219, ws->regMatch[0]);
	splitRegex(ws, &R220, ws->regMatch[1]);
	splitRegex(ws, &R221, ws->regMatch[2]);
	splitRegex(ws, &R222, ws->regMatch[3]);
	splitRegex(ws, &R223, ws->regMatch[4]);
	splitRegex(ws, &R224, ws->regMatch[5]);
	splitRegex(ws, &R16, ws->regMatch[6]);
	for(i = 0; DFirefoxFennec[i].segments != NULL; i++)
		EFirefoxFennec(ws, DFirefoxFennec[i].segments, DFirefoxFennec[i].devices);
}
void HFly(Workset *ws) {
	int i;
	for (i = 0; DFly[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DFly[i]);
}
__inline void EHTC(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 2);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HHTC(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R343, ws->regMatch[0]);
	splitRegex(ws, &R2, ws->regMatch[1]);
	for(i = 0; DHTC[i].segments != NULL; i++)
		EHTC(ws, DHTC[i].segments, DHTC[i].devices);
}
void HHuawei(Workset *ws) {
	int i;
	for (i = 0; DHuawei[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DHuawei[i]);
}
__inline void EKDDI(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 1);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HKDDI(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R346, ws->regMatch[0]);
	for(i = 0; DKDDI[i].segments != NULL; i++)
		EKDDI(ws, DKDDI[i].segments, DKDDI[i].devices);
}
void HKyocera(Workset *ws) {
	int i;
	for (i = 0; DKyocera[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DKyocera[i]);
}
void HLCT(Workset *ws) {
	int i;
	for (i = 0; DLCT[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DLCT[i]);
}
__inline void ELenovo(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 1);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HLenovo(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R350, ws->regMatch[0]);
	splitRegex(ws, &R2, ws->regMatch[1]);
	for(i = 0; DLenovo[i].segments != NULL; i++)
		ELenovo(ws, DLenovo[i].segments, DLenovo[i].devices);
}
__inline void ELG(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HLG(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R352, ws->regMatch[0]);
	splitRegex(ws, &R353, ws->regMatch[1]);
	splitRegex(ws, &R354, ws->regMatch[2]);
	splitRegex(ws, &R355, ws->regMatch[3]);
	splitRegex(ws, &R356, ws->regMatch[4]);
	splitRegex(ws, &R357, ws->regMatch[5]);
	splitRegex(ws, &R358, ws->regMatch[6]);
	for(i = 0; DLG[i].segments != NULL; i++)
		ELG(ws, DLG[i].segments, DLG[i].devices);
}
__inline void ELG_(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HLG_(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R360, ws->regMatch[0]);
	splitRegex(ws, &R353, ws->regMatch[1]);
	splitRegex(ws, &R357, ws->regMatch[2]);
	splitRegex(ws, &R355, ws->regMatch[3]);
	splitRegex(ws, &R334, ws->regMatch[4]);
	splitRegex(ws, &R358, ws->regMatch[5]);
	splitRegex(ws, &R361, ws->regMatch[6]);
	for(i = 0; DLG_[i].segments != NULL; i++)
		ELG_(ws, DLG_[i].segments, DLG_[i].devices);
}
__inline void EMIDP_CLDC(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HMIDP_CLDC(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R466, ws->regMatch[0]);
	for(i = 0; DMIDP_CLDC[i].segments != NULL; i++)
		EMIDP_CLDC(ws, DMIDP_CLDC[i].segments, DMIDP_CLDC[i].devices);
}
__inline void ELGE_(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HLGE_(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R363, ws->regMatch[0]);
	splitRegex(ws, &R364, ws->regMatch[1]);
	splitRegex(ws, &R357, ws->regMatch[2]);
	splitRegex(ws, &R355, ws->regMatch[3]);
	splitRegex(ws, &R334, ws->regMatch[4]);
	splitRegex(ws, &R358, ws->regMatch[5]);
	for(i = 0; DLGE_[i].segments != NULL; i++)
		ELGE_(ws, DLGE_[i].segments, DLGE_[i].devices);
}
__inline void EMOT_(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 1);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HMOT_(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R366, ws->regMatch[0]);
	splitRegex(ws, &R367, ws->regMatch[1]);
	splitRegex(ws, &R368, ws->regMatch[2]);
	for(i = 0; DMOT_[i].segments != NULL; i++)
		EMOT_(ws, DMOT_[i].segments, DMOT_[i].devices);
}
__inline void EMSIEDesktop(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 100);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 100);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 100);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 100);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[18], ws->regMatch[18], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[19], ws->regMatch[19], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[20], ws->regMatch[20], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[21], ws->regMatch[21], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[22], ws->regMatch[22], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[23], ws->regMatch[23], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[24], ws->regMatch[24], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[25], ws->regMatch[25], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[26], ws->regMatch[26], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[27], ws->regMatch[27], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[28], ws->regMatch[28], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HMSIEDesktop(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R241, ws->regMatch[0]);
	splitRegex(ws, &R242, ws->regMatch[1]);
	splitRegex(ws, &R188, ws->regMatch[2]);
	splitRegex(ws, &R189, ws->regMatch[3]);
	splitRegex(ws, &R243, ws->regMatch[4]);
	splitRegex(ws, &R244, ws->regMatch[5]);
	splitRegex(ws, &R245, ws->regMatch[6]);
	splitRegex(ws, &R246, ws->regMatch[7]);
	splitRegex(ws, &R167, ws->regMatch[8]);
	splitRegex(ws, &R247, ws->regMatch[9]);
	splitRegex(ws, &R248, ws->regMatch[10]);
	splitRegex(ws, &R249, ws->regMatch[11]);
	splitRegex(ws, &R250, ws->regMatch[12]);
	splitRegex(ws, &R251, ws->regMatch[13]);
	splitRegex(ws, &R252, ws->regMatch[14]);
	splitRegex(ws, &R253, ws->regMatch[15]);
	splitRegex(ws, &R254, ws->regMatch[16]);
	splitRegex(ws, &R255, ws->regMatch[17]);
	splitRegex(ws, &R256, ws->regMatch[18]);
	splitRegex(ws, &R79, ws->regMatch[19]);
	splitRegex(ws, &R257, ws->regMatch[20]);
	splitRegex(ws, &R258, ws->regMatch[21]);
	splitRegex(ws, &R259, ws->regMatch[22]);
	splitRegex(ws, &R260, ws->regMatch[23]);
	splitRegex(ws, &R261, ws->regMatch[24]);
	splitRegex(ws, &R262, ws->regMatch[25]);
	splitRegex(ws, &R263, ws->regMatch[26]);
	splitRegex(ws, &R84, ws->regMatch[27]);
	splitRegex(ws, &R264, ws->regMatch[28]);
	for(i = 0; DMSIEDesktop[i].segments != NULL; i++)
		EMSIEDesktop(ws, DMSIEDesktop[i].segments, DMSIEDesktop[i].devices);
}
void HNec(Workset *ws) {
	int i;
	for (i = 0; DNec[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DNec[i]);
}
void HNexian(Workset *ws) {
	int i;
	for (i = 0; DNexian[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DNexian[i]);
}
__inline void ENokia(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HNokia(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R373, ws->regMatch[0]);
	splitRegex(ws, &R104, ws->regMatch[1]);
	splitRegex(ws, &R18, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R20, ws->regMatch[4]);
	splitRegex(ws, &R358, ws->regMatch[5]);
	splitRegex(ws, &R374, ws->regMatch[6]);
	splitRegex(ws, &R43, ws->regMatch[7]);
	splitRegex(ws, &R44, ws->regMatch[8]);
	splitRegex(ws, &R375, ws->regMatch[9]);
	splitRegex(ws, &R376, ws->regMatch[10]);
	splitRegex(ws, &R377, ws->regMatch[11]);
	for(i = 0; DNokia[i].segments != NULL; i++)
		ENokia(ws, DNokia[i].segments, DNokia[i].devices);
}
__inline void ENokiaSeries40(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HNokiaSeries40(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R267, ws->regMatch[0]);
	splitRegex(ws, &R268, ws->regMatch[1]);
	splitRegex(ws, &R104, ws->regMatch[2]);
	splitRegex(ws, &R18, ws->regMatch[3]);
	splitRegex(ws, &R17, ws->regMatch[4]);
	splitRegex(ws, &R20, ws->regMatch[5]);
	splitRegex(ws, &R21, ws->regMatch[6]);
	splitRegex(ws, &R269, ws->regMatch[7]);
	splitRegex(ws, &R270, ws->regMatch[8]);
	for(i = 0; DNokiaSeries40[i].segments != NULL; i++)
		ENokiaSeries40(ws, DNokiaSeries40[i].segments, DNokiaSeries40[i].devices);
}
__inline void ENokiaSeries60(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HNokiaSeries60(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R267, ws->regMatch[0]);
	splitRegex(ws, &R272, ws->regMatch[1]);
	splitRegex(ws, &R104, ws->regMatch[2]);
	splitRegex(ws, &R18, ws->regMatch[3]);
	splitRegex(ws, &R17, ws->regMatch[4]);
	splitRegex(ws, &R20, ws->regMatch[5]);
	splitRegex(ws, &R21, ws->regMatch[6]);
	splitRegex(ws, &R273, ws->regMatch[7]);
	for(i = 0; DNokiaSeries60[i].segments != NULL; i++)
		ENokiaSeries60(ws, DNokiaSeries60[i].segments, DNokiaSeries60[i].devices);
}
__inline void EOperaGeneric(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 5);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 5);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HOperaGeneric(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R450, ws->regMatch[0]);
	splitRegex(ws, &R11, ws->regMatch[1]);
	splitRegex(ws, &R451, ws->regMatch[2]);
	splitRegex(ws, &R10, ws->regMatch[3]);
	for(i = 0; DOperaGeneric[i].segments != NULL; i++)
		EOperaGeneric(ws, DOperaGeneric[i].segments, DOperaGeneric[i].devices);
}
__inline void EOperaDesktop(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HOperaDesktop(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R9, ws->regMatch[0]);
	splitRegex(ws, &R10, ws->regMatch[1]);
	splitRegex(ws, &R11, ws->regMatch[2]);
	splitRegex(ws, &R12, ws->regMatch[3]);
	splitRegex(ws, &R90, ws->regMatch[4]);
	splitRegex(ws, &R91, ws->regMatch[5]);
	splitRegex(ws, &R92, ws->regMatch[6]);
	splitRegex(ws, &R93, ws->regMatch[7]);
	splitRegex(ws, &R94, ws->regMatch[8]);
	splitRegex(ws, &R95, ws->regMatch[9]);
	for(i = 0; DOperaDesktop[i].segments != NULL; i++)
		EOperaDesktop(ws, DOperaDesktop[i].segments, DOperaDesktop[i].devices);
}
__inline void EOperaMini(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 1000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HOperaMini(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R453, ws->regMatch[0]);
	splitRegex(ws, &R454, ws->regMatch[1]);
	splitRegex(ws, &R455, ws->regMatch[2]);
	splitRegex(ws, &R456, ws->regMatch[3]);
	splitRegex(ws, &R457, ws->regMatch[4]);
	splitRegex(ws, &R458, ws->regMatch[5]);
	splitRegex(ws, &R459, ws->regMatch[6]);
	for(i = 0; DOperaMini[i].segments != NULL; i++)
		EOperaMini(ws, DOperaMini[i].segments, DOperaMini[i].devices);
}
__inline void EOperaMobi(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 1000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HOperaMobi(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R10, ws->regMatch[0]);
	splitRegex(ws, &R11, ws->regMatch[1]);
	for(i = 0; DOperaMobi[i].segments != NULL; i++)
		EOperaMobi(ws, DOperaMobi[i].segments, DOperaMobi[i].devices);
}
void HPalm(Workset *ws) {
	int i;
	for (i = 0; DPalm[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DPalm[i]);
}
void HPanasonic(Workset *ws) {
	int i;
	for (i = 0; DPanasonic[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DPanasonic[i]);
}
void HPantech(Workset *ws) {
	int i;
	for (i = 0; DPantech[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DPantech[i]);
}
void HPhilips(Workset *ws) {
	int i;
	for (i = 0; DPhilips[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DPhilips[i]);
}
void HPortalmmm(Workset *ws) {
	int i;
	for (i = 0; DPortalmmm[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DPortalmmm[i]);
}
void HQtek(Workset *ws) {
	int i;
	for (i = 0; DQtek[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DQtek[i]);
}
__inline void ESafari(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 3);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 1000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HSafari(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R385, ws->regMatch[0]);
	splitRegex(ws, &R168, ws->regMatch[1]);
	splitRegex(ws, &R165, ws->regMatch[2]);
	splitRegex(ws, &R170, ws->regMatch[3]);
	splitRegex(ws, &R386, ws->regMatch[4]);
	splitRegex(ws, &R387, ws->regMatch[5]);
	splitRegex(ws, &R388, ws->regMatch[6]);
	for(i = 0; DSafari[i].segments != NULL; i++)
		ESafari(ws, DSafari[i].segments, DSafari[i].devices);
}
__inline void ESafariDesktop(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[18], ws->regMatch[18], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[19], ws->regMatch[19], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[20], ws->regMatch[20], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[21], ws->regMatch[21], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[22], ws->regMatch[22], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[23], ws->regMatch[23], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[24], ws->regMatch[24], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HSafariDesktop(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R278, ws->regMatch[0]);
	splitRegex(ws, &R188, ws->regMatch[1]);
	splitRegex(ws, &R189, ws->regMatch[2]);
	splitRegex(ws, &R279, ws->regMatch[3]);
	splitRegex(ws, &R280, ws->regMatch[4]);
	splitRegex(ws, &R281, ws->regMatch[5]);
	splitRegex(ws, &R282, ws->regMatch[6]);
	splitRegex(ws, &R283, ws->regMatch[7]);
	splitRegex(ws, &R284, ws->regMatch[8]);
	splitRegex(ws, &R285, ws->regMatch[9]);
	splitRegex(ws, &R286, ws->regMatch[10]);
	splitRegex(ws, &R287, ws->regMatch[11]);
	splitRegex(ws, &R288, ws->regMatch[12]);
	splitRegex(ws, &R289, ws->regMatch[13]);
	splitRegex(ws, &R290, ws->regMatch[14]);
	splitRegex(ws, &R291, ws->regMatch[15]);
	splitRegex(ws, &R30, ws->regMatch[16]);
	splitRegex(ws, &R292, ws->regMatch[17]);
	splitRegex(ws, &R293, ws->regMatch[18]);
	splitRegex(ws, &R294, ws->regMatch[19]);
	splitRegex(ws, &R295, ws->regMatch[20]);
	splitRegex(ws, &R296, ws->regMatch[21]);
	splitRegex(ws, &R297, ws->regMatch[22]);
	splitRegex(ws, &R298, ws->regMatch[23]);
	splitRegex(ws, &R299, ws->regMatch[24]);
	for(i = 0; DSafariDesktop[i].segments != NULL; i++)
		ESafariDesktop(ws, DSafariDesktop[i].segments, DSafariDesktop[i].devices);
}
void HSagem(Workset *ws) {
	int i;
	for (i = 0; DSagem[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DSagem[i]);
}
__inline void ESamsung(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HSamsung(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R394, ws->regMatch[0]);
	splitRegex(ws, &R395, ws->regMatch[1]);
	splitRegex(ws, &R396, ws->regMatch[2]);
	splitRegex(ws, &R397, ws->regMatch[3]);
	splitRegex(ws, &R398, ws->regMatch[4]);
	splitRegex(ws, &R98, ws->regMatch[5]);
	splitRegex(ws, &R399, ws->regMatch[6]);
	splitRegex(ws, &R334, ws->regMatch[7]);
	splitRegex(ws, &R20, ws->regMatch[8]);
	splitRegex(ws, &R21, ws->regMatch[9]);
	splitRegex(ws, &R400, ws->regMatch[10]);
	splitRegex(ws, &R401, ws->regMatch[11]);
	splitRegex(ws, &R402, ws->regMatch[12]);
	for(i = 0; DSamsung[i].segments != NULL; i++)
		ESamsung(ws, DSamsung[i].segments, DSamsung[i].devices);
}
__inline void ESamsungGT(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HSamsungGT(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R403, ws->regMatch[0]);
	splitRegex(ws, &R396, ws->regMatch[1]);
	splitRegex(ws, &R398, ws->regMatch[2]);
	splitRegex(ws, &R397, ws->regMatch[3]);
	splitRegex(ws, &R404, ws->regMatch[4]);
	splitRegex(ws, &R98, ws->regMatch[5]);
	splitRegex(ws, &R334, ws->regMatch[6]);
	splitRegex(ws, &R20, ws->regMatch[7]);
	splitRegex(ws, &R21, ws->regMatch[8]);
	splitRegex(ws, &R405, ws->regMatch[9]);
	for(i = 0; DSamsungGT[i].segments != NULL; i++)
		ESamsungGT(ws, DSamsungGT[i].segments, DSamsungGT[i].devices);
}
__inline void ESamsungSeries60(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 1);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HSamsungSeries60(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R406, ws->regMatch[0]);
	splitRegex(ws, &R407, ws->regMatch[1]);
	splitRegex(ws, &R408, ws->regMatch[2]);
	splitRegex(ws, &R20, ws->regMatch[3]);
	splitRegex(ws, &R21, ws->regMatch[4]);
	splitRegex(ws, &R222, ws->regMatch[5]);
	for(i = 0; DSamsungSeries60[i].segments != NULL; i++)
		ESamsungSeries60(ws, DSamsungSeries60[i].segments, DSamsungSeries60[i].devices);
}
void HSanyo(Workset *ws) {
	int i;
	for (i = 0; DSanyo[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DSanyo[i]);
}
void HSharp(Workset *ws) {
	int i;
	for (i = 0; DSharp[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DSharp[i]);
}
void HSiemens(Workset *ws) {
	int i;
	for (i = 0; DSiemens[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DSiemens[i]);
}
__inline void ESkyfire(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 5);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 5);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 5);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HSkyfire(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R219, ws->regMatch[0]);
	splitRegex(ws, &R301, ws->regMatch[1]);
	splitRegex(ws, &R168, ws->regMatch[2]);
	splitRegex(ws, &R170, ws->regMatch[3]);
	splitRegex(ws, &R165, ws->regMatch[4]);
	for(i = 0; DSkyfire[i].segments != NULL; i++)
		ESkyfire(ws, DSkyfire[i].segments, DSkyfire[i].devices);
}
__inline void ESoftBank(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 1);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HSoftBank(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R415, ws->regMatch[0]);
	for(i = 0; DSoftBank[i].segments != NULL; i++)
		ESoftBank(ws, DSoftBank[i].segments, DSoftBank[i].devices);
}
__inline void ESonyEricsson(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HSonyEricsson(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R97, ws->regMatch[0]);
	splitRegex(ws, &R98, ws->regMatch[1]);
	splitRegex(ws, &R99, ws->regMatch[2]);
	splitRegex(ws, &R100, ws->regMatch[3]);
	splitRegex(ws, &R101, ws->regMatch[4]);
	splitRegex(ws, &R102, ws->regMatch[5]);
	splitRegex(ws, &R103, ws->regMatch[6]);
	splitRegex(ws, &R104, ws->regMatch[7]);
	splitRegex(ws, &R105, ws->regMatch[8]);
	splitRegex(ws, &R18, ws->regMatch[9]);
	splitRegex(ws, &R106, ws->regMatch[10]);
	for(i = 0; DSonyEricsson[i].segments != NULL; i++)
		ESonyEricsson(ws, DSonyEricsson[i].segments, DSonyEricsson[i].devices);
}
void HSpice(Workset *ws) {
	int i;
	for (i = 0; DSpice[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DSpice[i]);
}
__inline void ETianyu(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 2);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HTianyu(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R418, ws->regMatch[0]);
	splitRegex(ws, &R2, ws->regMatch[1]);
	for(i = 0; DTianyu[i].segments != NULL; i++)
		ETianyu(ws, DTianyu[i].segments, DTianyu[i].devices);
}
void HToshiba(Workset *ws) {
	int i;
	for (i = 0; DToshiba[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DToshiba[i]);
}
void HVodafone(Workset *ws) {
	int i;
	for (i = 0; DVodafone[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DVodafone[i]);
}
void HWebOS(Workset *ws) {
	int i;
	for (i = 0; DWebOS[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DWebOS[i]);
}
void HW3CValidator(Workset *ws) {
	int i;
	for (i = 0; DW3CValidator[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DW3CValidator[i]);
}
__inline void EWindowsCE(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HWindowsCE(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R423, ws->regMatch[0]);
	for(i = 0; DWindowsCE[i].segments != NULL; i++)
		EWindowsCE(ws, DWindowsCE[i].segments, DWindowsCE[i].devices);
}
void HWindowsPhone(Workset *ws) {
	int i;
	for (i = 0; DWindowsPhone[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DWindowsPhone[i]);
}
__inline void EWordPress(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HWordPress(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R304, ws->regMatch[0]);
	for(i = 0; DWordPress[i].segments != NULL; i++)
		EWordPress(ws, DWordPress[i].segments, DWordPress[i].devices);
}
void HZTE(Workset *ws) {
	int i;
	for (i = 0; DZTE[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DZTE[i]);
}
__inline void EUnknownWebkit(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HUnknownWebkit(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R188, ws->regMatch[0]);
	splitRegex(ws, &R189, ws->regMatch[1]);
	splitRegex(ws, &R244, ws->regMatch[2]);
	splitRegex(ws, &R427, ws->regMatch[3]);
	for(i = 0; DUnknownWebkit[i].segments != NULL; i++)
		EUnknownWebkit(ws, DUnknownWebkit[i].segments, DUnknownWebkit[i].devices);
}
__inline void EUnknownGecko(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[18], ws->regMatch[18], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[19], ws->regMatch[19], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[20], ws->regMatch[20], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[21], ws->regMatch[21], 1000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[22], ws->regMatch[22], 1000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HUnknownGecko(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R188, ws->regMatch[0]);
	splitRegex(ws, &R189, ws->regMatch[1]);
	splitRegex(ws, &R430, ws->regMatch[2]);
	splitRegex(ws, &R431, ws->regMatch[3]);
	splitRegex(ws, &R432, ws->regMatch[4]);
	splitRegex(ws, &R433, ws->regMatch[5]);
	splitRegex(ws, &R434, ws->regMatch[6]);
	splitRegex(ws, &R435, ws->regMatch[7]);
	splitRegex(ws, &R436, ws->regMatch[8]);
	splitRegex(ws, &R204, ws->regMatch[9]);
	splitRegex(ws, &R437, ws->regMatch[10]);
	splitRegex(ws, &R438, ws->regMatch[11]);
	splitRegex(ws, &R80, ws->regMatch[12]);
	splitRegex(ws, &R439, ws->regMatch[13]);
	splitRegex(ws, &R440, ws->regMatch[14]);
	splitRegex(ws, &R441, ws->regMatch[15]);
	splitRegex(ws, &R442, ws->regMatch[16]);
	splitRegex(ws, &R443, ws->regMatch[17]);
	splitRegex(ws, &R444, ws->regMatch[18]);
	splitRegex(ws, &R445, ws->regMatch[19]);
	splitRegex(ws, &R446, ws->regMatch[20]);
	splitRegex(ws, &R447, ws->regMatch[21]);
	splitRegex(ws, &R250, ws->regMatch[22]);
	for(i = 0; DUnknownGecko[i].segments != NULL; i++)
		EUnknownGecko(ws, DUnknownGecko[i].segments, DUnknownGecko[i].devices);
}
__inline void EThunderbird(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HThunderbird(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R308, ws->regMatch[0]);
	splitRegex(ws, &R188, ws->regMatch[1]);
	splitRegex(ws, &R189, ws->regMatch[2]);
	splitRegex(ws, &R244, ws->regMatch[3]);
	splitRegex(ws, &R309, ws->regMatch[4]);
	for(i = 0; DThunderbird[i].segments != NULL; i++)
		EThunderbird(ws, DThunderbird[i].segments, DThunderbird[i].devices);
}
__inline void ESilk(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HSilk(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R108, ws->regMatch[0]);
	splitRegex(ws, &R14, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R109, ws->regMatch[3]);
	splitRegex(ws, &R51, ws->regMatch[4]);
	for(i = 0; DSilk[i].segments != NULL; i++)
		ESilk(ws, DSilk[i].segments, DSilk[i].devices);
}
__inline void ECFNetwork_Darwin(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 100000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HCFNetwork_Darwin(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R313, ws->regMatch[0]);
	splitRegex(ws, &R314, ws->regMatch[1]);
	splitRegex(ws, &R315, ws->regMatch[2]);
	splitRegex(ws, &R316, ws->regMatch[3]);
	for(i = 0; DCFNetwork_Darwin[i].segments != NULL; i++)
		ECFNetwork_Darwin(ws, DCFNetwork_Darwin[i].segments, DCFNetwork_Darwin[i].devices);
}
__inline void EItunes(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HItunes(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R222, ws->regMatch[0]);
	splitRegex(ws, &R449, ws->regMatch[1]);
	for(i = 0; DItunes[i].segments != NULL; i++)
		EItunes(ws, DItunes[i].segments, DItunes[i].devices);
}
void Hdataaccessd(Workset *ws) {
	int i;
	for (i = 0; Ddataaccessd[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, Ddataaccessd[i]);
}
void HPodcasts(Workset *ws) {
	int i;
	for (i = 0; DPodcasts[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DPodcasts[i]);
}
void HTinyTinyRSS(Workset *ws) {
	int i;
	for (i = 0; DTinyTinyRSS[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DTinyTinyRSS[i]);
}
void HAmazonCloudFront(Workset *ws) {
	int i;
	for (i = 0; DAmazonCloudFront[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DAmazonCloudFront[i]);
}
void HWada(Workset *ws) {
	int i;
	for (i = 0; DWada[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DWada[i]);
}
void HFeedDemon(Workset *ws) {
	int i;
	for (i = 0; DFeedDemon[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DFeedDemon[i]);
}
void Hpythonrequests(Workset *ws) {
	int i;
	for (i = 0; Dpythonrequests[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, Dpythonrequests[i]);
}
void HRSSInclude(Workset *ws) {
	int i;
	for (i = 0; DRSSInclude[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DRSSInclude[i]);
}
void Hcurl(Workset *ws) {
	int i;
	for (i = 0; Dcurl[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, Dcurl[i]);
}
void HPulp(Workset *ws) {
	int i;
	for (i = 0; DPulp[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DPulp[i]);
}
void HWin98(Workset *ws) {
	int i;
	for (i = 0; DWin98[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DWin98[i]);
}
__inline void EAppEngine(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 100000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAppEngine(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R480, ws->regMatch[0]);
	splitRegex(ws, &R481, ws->regMatch[1]);
	for(i = 0; DAppEngine[i].segments != NULL; i++)
		EAppEngine(ws, DAppEngine[i].segments, DAppEngine[i].devices);
}
__inline void EOutlook(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HOutlook(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R463, ws->regMatch[0]);
	splitRegex(ws, &R464, ws->regMatch[1]);
	for(i = 0; DOutlook[i].segments != NULL; i++)
		EOutlook(ws, DOutlook[i].segments, DOutlook[i].devices);
}
__inline void EOperaTwo(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HOperaTwo(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R9, ws->regMatch[0]);
	splitRegex(ws, &R10, ws->regMatch[1]);
	splitRegex(ws, &R11, ws->regMatch[2]);
	splitRegex(ws, &R12, ws->regMatch[3]);
	for(i = 0; DOperaTwo[i].segments != NULL; i++)
		EOperaTwo(ws, DOperaTwo[i].segments, DOperaTwo[i].devices);
}
__inline void EIUC(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HIUC(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R483, ws->regMatch[0]);
	splitRegex(ws, &R484, ws->regMatch[1]);
	splitRegex(ws, &R485, ws->regMatch[2]);
	splitRegex(ws, &R486, ws->regMatch[3]);
	splitRegex(ws, &R487, ws->regMatch[4]);
	for(i = 0; DIUC[i].segments != NULL; i++)
		EIUC(ws, DIUC[i].segments, DIUC[i].devices);
}
__inline void Eserverbag(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void Hserverbag(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R489, ws->regMatch[0]);
	splitRegex(ws, &R490, ws->regMatch[1]);
	for(i = 0; Dserverbag[i].segments != NULL; i++)
		Eserverbag(ws, Dserverbag[i].segments, Dserverbag[i].devices);
}
void HNumbers(Workset *ws) {
	int i;
	for (i = 0; DNumbers[i] != NULL && ws->lowScore > 0; i++)
		editDistanceScoreCheck(ws, DNumbers[i]);
}
__inline void EMicromax(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HMicromax(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R493, ws->regMatch[0]);
	for(i = 0; DMicromax[i].segments != NULL; i++)
		EMicromax(ws, DMicromax[i].segments, DMicromax[i].devices);
}
__inline void Ejigbrowser(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void Hjigbrowser(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R495, ws->regMatch[0]);
	for(i = 0; Djigbrowser[i].segments != NULL; i++)
		Ejigbrowser(ws, Djigbrowser[i].segments, Djigbrowser[i].devices);
}
__inline void EKonqueror(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HKonqueror(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R497, ws->regMatch[0]);
	splitRegex(ws, &R498, ws->regMatch[1]);
	splitRegex(ws, &R432, ws->regMatch[2]);
	for(i = 0; DKonqueror[i].segments != NULL; i++)
		EKonqueror(ws, DKonqueror[i].segments, DKonqueror[i].devices);
}
__inline void Eheritrix(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void Hheritrix(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R500, ws->regMatch[0]);
	for(i = 0; Dheritrix[i].segments != NULL; i++)
		Eheritrix(ws, Dheritrix[i].segments, Dheritrix[i].devices);
}
__inline void EJava(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HJava(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R502, ws->regMatch[0]);
	splitRegex(ws, &R503, ws->regMatch[1]);
	splitRegex(ws, &R504, ws->regMatch[2]);
	for(i = 0; DJava[i].segments != NULL; i++)
		EJava(ws, DJava[i].segments, DJava[i].devices);
}
__inline void EUCWEB(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HUCWEB(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R506, ws->regMatch[0]);
	splitRegex(ws, &R507, ws->regMatch[1]);
	for(i = 0; DUCWEB[i].segments != NULL; i++)
		EUCWEB(ws, DUCWEB[i].segments, DUCWEB[i].devices);
}
__inline void EAndroidVersion4_3(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion4_3(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R22, ws->regMatch[7]);
	splitRegex(ws, &R10, ws->regMatch[8]);
	splitRegex(ws, &R132, ws->regMatch[9]);
	splitRegex(ws, &R28, ws->regMatch[10]);
	splitRegex(ws, &R36, ws->regMatch[11]);
	splitRegex(ws, &R52, ws->regMatch[12]);
	for(i = 0; DAndroidVersion4_3[i].segments != NULL; i++)
		EAndroidVersion4_3(ws, DAndroidVersion4_3[i].segments, DAndroidVersion4_3[i].devices);
}
__inline void EAndroidVersion4_4(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 100000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[2], ws->regMatch[2], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[3], ws->regMatch[3], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[4], ws->regMatch[4], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[5], ws->regMatch[5], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[6], ws->regMatch[6], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[7], ws->regMatch[7], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[8], ws->regMatch[8], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[9], ws->regMatch[9], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[10], ws->regMatch[10], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[11], ws->regMatch[11], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[12], ws->regMatch[12], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[13], ws->regMatch[13], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[14], ws->regMatch[14], 1);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[15], ws->regMatch[15], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[16], ws->regMatch[16], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[17], ws->regMatch[17], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[18], ws->regMatch[18], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[19], ws->regMatch[19], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[20], ws->regMatch[20], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[21], ws->regMatch[21], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[22], ws->regMatch[22], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[23], ws->regMatch[23], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[24], ws->regMatch[24], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[25], ws->regMatch[25], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HAndroidVersion4_4(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R14, ws->regMatch[0]);
	splitRegex(ws, &R15, ws->regMatch[1]);
	splitRegex(ws, &R16, ws->regMatch[2]);
	splitRegex(ws, &R17, ws->regMatch[3]);
	splitRegex(ws, &R18, ws->regMatch[4]);
	splitRegex(ws, &R19, ws->regMatch[5]);
	splitRegex(ws, &R20, ws->regMatch[6]);
	splitRegex(ws, &R21, ws->regMatch[7]);
	splitRegex(ws, &R22, ws->regMatch[8]);
	splitRegex(ws, &R10, ws->regMatch[9]);
	splitRegex(ws, &R23, ws->regMatch[10]);
	splitRegex(ws, &R24, ws->regMatch[11]);
	splitRegex(ws, &R132, ws->regMatch[12]);
	splitRegex(ws, &R26, ws->regMatch[13]);
	splitRegex(ws, &R27, ws->regMatch[14]);
	splitRegex(ws, &R28, ws->regMatch[15]);
	splitRegex(ws, &R52, ws->regMatch[16]);
	splitRegex(ws, &R155, ws->regMatch[17]);
	splitRegex(ws, &R157, ws->regMatch[18]);
	splitRegex(ws, &R33, ws->regMatch[19]);
	splitRegex(ws, &R31, ws->regMatch[20]);
	splitRegex(ws, &R149, ws->regMatch[21]);
	splitRegex(ws, &R36, ws->regMatch[22]);
	splitRegex(ws, &R158, ws->regMatch[23]);
	splitRegex(ws, &R159, ws->regMatch[24]);
	splitRegex(ws, &R129, ws->regMatch[25]);
	for(i = 0; DAndroidVersion4_4[i].segments != NULL; i++)
		EAndroidVersion4_4(ws, DAndroidVersion4_4[i].segments, DAndroidVersion4_4[i].devices);
}
__inline void EMac(Workset *ws, const Segment * const segmentList[], const Device * const subjectDevice) {
	ws->score = 0;
	calculateSegmentScore(ws, segmentList[0], ws->regMatch[0], 10000);
	if (ws->score == INT_MAX) return;
	calculateSegmentScore(ws, segmentList[1], ws->regMatch[1], 10000);
	if (ws->score == INT_MAX) return;
	scoreCheck(ws, subjectDevice);
}
void HMac(Workset *ws) {
	int i;
	clearSegments(ws);
	splitRegex(ws, &R509, ws->regMatch[0]);
	splitRegex(ws, &R510, ws->regMatch[1]);
	for(i = 0; DMac[i].segments != NULL; i++)
		EMac(ws, DMac[i].segments, DMac[i].devices);
}


/*Main device detection process. Populates Workset with a list of devices.*/
void getShortList(Workset *ws) {
	ws->count = 0;
	ws->lowScore = INT_MAX;
	ws->score = 0;

	trimUserAgent(ws);
	/*Handler - Bot*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R0, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R1, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R2, ws->userAgent, 0, NULL, 0)
		)) {
		HBot(ws);
	}
	/*Handler - W3CValidator*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R3, ws->userAgent, 0, NULL, 0)
		) {
		HW3CValidator(ws);
	}
	/*Handler - OperaTwo*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R4, ws->userAgent, 0, NULL, 0) && (
			!fiftyone_degrees_regexec(&R5, ws->userAgent, 0, NULL, 0) && (
				!fiftyone_degrees_regexec(&R6, ws->userAgent, 0, NULL, 0)
			)
		)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R7, ws->userAgent, 0, NULL, 0) &&
		fiftyone_degrees_regexec(&R8, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R9, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R11, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R12, ws->userAgent, 0, NULL, 0)
		)) {
		HOperaTwo(ws);
	}
	if (ws->count > 0) return;	/*Return 1 when devices have been placed in the workset*/
	/*Handler - AndroidVersion2.2.1*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R13, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R25, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R29, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R30, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R31, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R32, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R33, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R34, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R35, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion2_2_1(ws);
	}
	/*Handler - AndroidVersion2.2.2*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R37, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R25, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R38, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R31, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion2_2_2(ws);
	}
	/*Handler - AndroidVersion2.2.3*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R39, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R25, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion2_2_3(ws);
	}
	/*Handler - AndroidVersion2.3.1*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R40, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R25, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion2_3_1(ws);
	}
	/*Handler - AndroidVersion2.3.2*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R41, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R25, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion2_3_2(ws);
	}
	/*Handler - AndroidVersion2.3.3*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R42, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R25, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R43, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R44, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R33, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R34, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R38, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R45, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R46, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R47, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R48, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R49, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion2_3_3(ws);
	}
	/*Handler - AndroidVersion2.3.4*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R50, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R25, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R38, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R34, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R31, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R46, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R51, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R52, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R35, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R53, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R54, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R33, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion2_3_4(ws);
	}
	/*Handler - AndroidVersion2.3.5*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R55, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R25, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R56, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R57, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R58, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R38, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R33, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R46, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R59, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R51, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion2_3_5(ws);
	}
	/*Handler - AndroidVersion2.3.6*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R60, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R25, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R57, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R58, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R29, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R33, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R51, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R59, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R61, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion2_3_6(ws);
	}
	/*Handler - AndroidVersion2.3.7*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R62, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R25, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R63, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R64, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R65, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R51, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R53, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R34, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R56, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R66, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R31, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R67, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion2_3_7(ws);
	}
	/*Handler - Chrome0-9*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R68, ws->userAgent, 0, NULL, 0)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R69, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R70, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R71, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R72, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R73, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R74, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R75, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R76, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R77, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R34, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R78, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R79, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R80, ws->userAgent, 0, NULL, 0)
		)) {
		HChrome0_9(ws);
	}
	/*Handler - Chrome10-19*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R81, ws->userAgent, 0, NULL, 0)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R69, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R70, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R71, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R72, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R73, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R74, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R75, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R78, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R76, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R77, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R34, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R79, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R82, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R83, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R84, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R85, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R86, ws->userAgent, 0, NULL, 0)
		)) {
		HChrome10_19(ws);
	}
	/*Handler - Chrome20-29*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R87, ws->userAgent, 0, NULL, 0)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R69, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R70, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R71, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R72, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R73, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R74, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R75, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R76, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R77, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R34, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R88, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R84, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R85, ws->userAgent, 0, NULL, 0)
		)) {
		HChrome20_29(ws);
	}
	/*Handler - Chrome30-39*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R89, ws->userAgent, 0, NULL, 0)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R69, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R70, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R71, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R72, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R73, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R74, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R75, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R76, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R77, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R34, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R88, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0)
		)) {
		HChrome30_39(ws);
	}
	/*Handler - OperaDesktop*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R4, ws->userAgent, 0, NULL, 0)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R7, ws->userAgent, 0, NULL, 0) &&
		fiftyone_degrees_regexec(&R8, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R9, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R11, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R12, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R90, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R91, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R92, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R93, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R94, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R95, ws->userAgent, 0, NULL, 0)
		)) {
		HOperaDesktop(ws);
	}
	/*Handler - SonyEricsson*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R96, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R97, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R98, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R99, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R100, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R101, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R102, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R103, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R104, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R105, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R106, ws->userAgent, 0, NULL, 0)
		)) {
		HSonyEricsson(ws);
	}
	/*Handler - Silk*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R107, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R108, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R109, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R51, ws->userAgent, 0, NULL, 0)
		)) {
		HSilk(ws);
	}
	if (ws->count > 0) return;	/*Return 1 when devices have been placed in the workset*/
	/*Handler - AndroidNonNumeric*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R69, ws->userAgent, 0, NULL, 0)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R110, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R111, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R112, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R113, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R114, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R31, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R115, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R116, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidNonNumeric(ws);
	}
	/*Handler - AndroidVersion1*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R117, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R25, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion1(ws);
	}
	/*Handler - AndroidVersion2*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R118, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R25, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion2(ws);
	}
	/*Handler - AndroidVersion2.1*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R119, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R25, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R51, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R31, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R35, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion2_1(ws);
	}
	/*Handler - AndroidVersion2.2*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R120, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R25, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R121, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R33, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R38, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R56, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R31, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R122, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R35, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion2_2(ws);
	}
	/*Handler - AndroidVersion2.3*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R123, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R25, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion2_3(ws);
	}
	/*Handler - AndroidVersion2.4*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R124, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R25, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion2_4(ws);
	}
	/*Handler - AndroidVersion3*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R125, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R25, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R34, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R31, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R33, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R126, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R127, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R51, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R38, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R53, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R128, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R129, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion3(ws);
	}
	/*Handler - AndroidVersion4.0*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R130, ws->userAgent, 0, NULL, 0)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R131, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R132, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion4_0(ws);
	}
	/*Handler - AndroidVersion4.0.1*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R133, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R132, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R134, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion4_0_1(ws);
	}
	/*Handler - AndroidVersion4.0.2*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R135, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R132, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion4_0_2(ws);
	}
	/*Handler - AndroidVersion4.0.3*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R136, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R132, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R137, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R31, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R33, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R127, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R29, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R51, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R138, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R53, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R129, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion4_0_3(ws);
	}
	/*Handler - AndroidVersion4.0.4*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R139, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R132, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R33, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R34, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R31, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R127, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R52, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R46, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R51, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R53, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R140, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R38, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R141, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R142, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R129, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion4_0_4(ws);
	}
	/*Handler - AndroidVersion4.0.5*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R143, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion4_0_5(ws);
	}
	/*Handler - AndroidVersion4.1*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R144, ws->userAgent, 0, NULL, 0)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R145, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R132, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion4_1(ws);
	}
	/*Handler - AndroidVersion4.1.1*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R146, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R132, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R137, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R31, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R127, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R33, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R52, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R147, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R148, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R38, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R149, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R142, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R129, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion4_1_1(ws);
	}
	/*Handler - AndroidVersion4.1.2*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R150, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R132, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R52, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R34, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R53, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R31, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R33, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R149, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R38, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R141, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R151, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion4_1_2(ws);
	}
	/*Handler - AndroidVersion4.2*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R152, ws->userAgent, 0, NULL, 0)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R153, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R132, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion4_2(ws);
	}
	/*Handler - AndroidVersion4.2.1*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R154, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R132, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R52, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R155, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R33, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R44, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion4_2_1(ws);
	}
	/*Handler - AndroidVersion4.2.2*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R156, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R132, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R52, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R155, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R157, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R33, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R31, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R149, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R158, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R159, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R129, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion4_2_2(ws);
	}
	/*Handler - Apple*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R160, ws->userAgent, 0, NULL, 0)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R161, ws->userAgent, 0, NULL, 0) &&
		fiftyone_degrees_regexec(&R162, ws->userAgent, 0, NULL, 0) &&
		fiftyone_degrees_regexec(&R69, ws->userAgent, 0, NULL, 0) &&
		fiftyone_degrees_regexec(&R163, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R164, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R165, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R166, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R167, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R168, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R169, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R170, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R171, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R172, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R173, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R174, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R175, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R31, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R176, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R29, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R177, ws->userAgent, 0, NULL, 0)
		)) {
		HApple(ws);
	}
	/*Handler - Bolt*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R178, ws->userAgent, 0, NULL, 0)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R179, ws->userAgent, 0, NULL, 0) &&
		fiftyone_degrees_regexec(&R180, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R181, ws->userAgent, 0, NULL, 0)
		)) {
		HBolt(ws);
	}
	/*Handler - FirefoxDesktop*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R182, ws->userAgent, 0, NULL, 0) && (
			!fiftyone_degrees_regexec(&R183, ws->userAgent, 0, NULL, 0)
		)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R184, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R185, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R186, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R187, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R188, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R189, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R190, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R191, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R192, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R193, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R194, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R73, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R195, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R196, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R197, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R198, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R199, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R200, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R201, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R80, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R202, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R203, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R204, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R205, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R206, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R207, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R208, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R209, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R210, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R211, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R212, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R213, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R214, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R215, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R216, ws->userAgent, 0, NULL, 0)
		)) {
		HFirefoxDesktop(ws);
	}
	/*Handler - FirefoxFennec*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R217, ws->userAgent, 0, NULL, 0) && (
			!fiftyone_degrees_regexec(&R218, ws->userAgent, 0, NULL, 0)
		)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R219, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R220, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R221, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R222, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R223, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R224, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0)
		)) {
		HFirefoxFennec(ws);
	}
	/*Handler - MSIEDesktop*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R225, ws->userAgent, 0, NULL, 0) && (
			!fiftyone_degrees_regexec(&R226, ws->userAgent, 0, NULL, 0) ||
			!fiftyone_degrees_regexec(&R227, ws->userAgent, 0, NULL, 0) ||
			!fiftyone_degrees_regexec(&R228, ws->userAgent, 0, NULL, 0) ||
			!fiftyone_degrees_regexec(&R229, ws->userAgent, 0, NULL, 0) ||
			!fiftyone_degrees_regexec(&R230, ws->userAgent, 0, NULL, 0) ||
			!fiftyone_degrees_regexec(&R231, ws->userAgent, 0, NULL, 0) ||
			!fiftyone_degrees_regexec(&R232, ws->userAgent, 0, NULL, 0) ||
			!fiftyone_degrees_regexec(&R233, ws->userAgent, 0, NULL, 0) ||
			!fiftyone_degrees_regexec(&R234, ws->userAgent, 0, NULL, 0)
		)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R235, ws->userAgent, 0, NULL, 0) &&
		fiftyone_degrees_regexec(&R236, ws->userAgent, 0, NULL, 0) &&
		fiftyone_degrees_regexec(&R237, ws->userAgent, 0, NULL, 0) &&
		fiftyone_degrees_regexec(&R238, ws->userAgent, 0, NULL, 0) &&
		fiftyone_degrees_regexec(&R8, ws->userAgent, 0, NULL, 0) &&
		fiftyone_degrees_regexec(&R239, ws->userAgent, 0, NULL, 0) &&
		fiftyone_degrees_regexec(&R240, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R241, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R242, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R188, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R189, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R243, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R244, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R245, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R246, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R167, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R247, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R248, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R249, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R250, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R251, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R252, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R253, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R254, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R255, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R256, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R79, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R257, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R258, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R259, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R260, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R261, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R262, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R263, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R84, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R264, ws->userAgent, 0, NULL, 0)
		)) {
		HMSIEDesktop(ws);
	}
	/*Handler - NokiaSeries40*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R265, ws->userAgent, 0, NULL, 0) && (
			!fiftyone_degrees_regexec(&R266, ws->userAgent, 0, NULL, 0)
		)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R267, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R268, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R104, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R269, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R270, ws->userAgent, 0, NULL, 0)
		)) {
		HNokiaSeries40(ws);
	}
	/*Handler - NokiaSeries60*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R265, ws->userAgent, 0, NULL, 0) && (
			!fiftyone_degrees_regexec(&R271, ws->userAgent, 0, NULL, 0)
		)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R267, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R272, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R104, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R273, ws->userAgent, 0, NULL, 0)
		)) {
		HNokiaSeries60(ws);
	}
	/*Handler - SafariDesktop*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R274, ws->userAgent, 0, NULL, 0) && (
			!fiftyone_degrees_regexec(&R275, ws->userAgent, 0, NULL, 0) && (
				!fiftyone_degrees_regexec(&R276, ws->userAgent, 0, NULL, 0)
			)
		)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R277, ws->userAgent, 0, NULL, 0) &&
		fiftyone_degrees_regexec(&R107, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R278, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R188, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R189, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R279, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R280, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R281, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R282, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R283, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R284, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R285, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R286, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R287, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R288, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R289, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R290, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R291, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R30, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R292, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R293, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R294, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R295, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R296, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R297, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R298, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R299, ws->userAgent, 0, NULL, 0)
		)) {
		HSafariDesktop(ws);
	}
	/*Handler - Skyfire*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R300, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R219, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R301, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R168, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R170, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R165, ws->userAgent, 0, NULL, 0)
		)) {
		HSkyfire(ws);
	}
	/*Handler - WindowsPhone*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R225, ws->userAgent, 0, NULL, 0) && (
			!fiftyone_degrees_regexec(&R302, ws->userAgent, 0, NULL, 0)
		)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R228, ws->userAgent, 0, NULL, 0)
		)) {
		HWindowsPhone(ws);
	}
	/*Handler - WordPress*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R303, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R304, ws->userAgent, 0, NULL, 0)
		)) {
		HWordPress(ws);
	}
	/*Handler - Thunderbird*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R305, ws->userAgent, 0, NULL, 0) && (
			!fiftyone_degrees_regexec(&R306, ws->userAgent, 0, NULL, 0)
		)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R307, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R308, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R188, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R189, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R244, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R309, ws->userAgent, 0, NULL, 0)
		)) {
		HThunderbird(ws);
	}
	/*Handler - CFNetwork-Darwin*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R310, ws->userAgent, 0, NULL, 0) && (
			!fiftyone_degrees_regexec(&R311, ws->userAgent, 0, NULL, 0)
		)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R312, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R313, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R314, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R315, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R316, ws->userAgent, 0, NULL, 0)
		)) {
		HCFNetwork_Darwin(ws);
	}
	/*Handler - AndroidVersion4.3*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R317, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R132, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R52, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion4_3(ws);
	}
	/*Handler - AndroidVersion4.4*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R318, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R14, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R15, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R16, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R19, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R22, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R23, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R24, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R132, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R26, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R27, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R28, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R52, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R155, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R157, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R33, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R31, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R149, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R36, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R158, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R159, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R129, ws->userAgent, 0, NULL, 0)
		)) {
		HAndroidVersion4_4(ws);
	}
	if (ws->count > 0) return;	/*Return 1 when devices have been placed in the workset*/
	/*Handler - Alcatel*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R319, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R320, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R2, ws->userAgent, 0, NULL, 0)
		)) {
		HAlcatel(ws);
	}
	/*Handler - Amoi*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R321, ws->userAgent, 0, NULL, 0)
		) {
		HAmoi(ws);
	}
	/*Handler - AppleCoreMedia*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R322, ws->userAgent, 0, NULL, 0) && (
			!fiftyone_degrees_regexec(&R323, ws->userAgent, 0, NULL, 0)
		)
		) {
		HAppleCoreMedia(ws);
	}
	/*Handler - AudioVox*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R324, ws->userAgent, 0, NULL, 0)
		) {
		HAudioVox(ws);
	}
	/*Handler - BenQ*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R325, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R326, ws->userAgent, 0, NULL, 0)
		) {
		HBenQ(ws);
	}
	/*Handler - Bird*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R327, ws->userAgent, 0, NULL, 0)
		) {
		HBird(ws);
	}
	/*Handler - BlackBerry1*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R328, ws->userAgent, 0, NULL, 0)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R329, ws->userAgent, 0, NULL, 0) || (
			fiftyone_degrees_regexec(&R330, ws->userAgent, 0, NULL, 0) || (
				fiftyone_degrees_regexec(&R331, ws->userAgent, 0, NULL, 0)
			)
		)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R332, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R333, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R334, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R335, ws->userAgent, 0, NULL, 0)
		)) {
		HBlackBerry1(ws);
	}
	/*Handler - BlackBerry2*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R329, ws->userAgent, 0, NULL, 0) && (
			!fiftyone_degrees_regexec(&R330, ws->userAgent, 0, NULL, 0) && (
				!fiftyone_degrees_regexec(&R331, ws->userAgent, 0, NULL, 0)
			)
		)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R336, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R337, ws->userAgent, 0, NULL, 0)
		)) {
		HBlackBerry2(ws);
	}
	/*Handler - DoCoMo*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R338, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R339, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R340, ws->userAgent, 0, NULL, 0)
		)) {
		HDoCoMo(ws);
	}
	/*Handler - Fly*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R341, ws->userAgent, 0, NULL, 0)
		) {
		HFly(ws);
	}
	/*Handler - HTC*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R342, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R343, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R2, ws->userAgent, 0, NULL, 0)
		)) {
		HHTC(ws);
	}
	/*Handler - Huawei*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R344, ws->userAgent, 0, NULL, 0)
		) {
		HHuawei(ws);
	}
	/*Handler - KDDI*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R345, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R346, ws->userAgent, 0, NULL, 0)
		)) {
		HKDDI(ws);
	}
	/*Handler - Kyocera*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R347, ws->userAgent, 0, NULL, 0)
		) {
		HKyocera(ws);
	}
	/*Handler - LCT*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R348, ws->userAgent, 0, NULL, 0)
		) {
		HLCT(ws);
	}
	/*Handler - Lenovo*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R349, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R350, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R2, ws->userAgent, 0, NULL, 0)
		)) {
		HLenovo(ws);
	}
	/*Handler - LG*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R351, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R352, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R353, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R354, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R355, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R356, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R357, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R358, ws->userAgent, 0, NULL, 0)
		)) {
		HLG(ws);
	}
	/*Handler - LG-*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R359, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R360, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R353, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R357, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R355, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R334, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R358, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R361, ws->userAgent, 0, NULL, 0)
		)) {
		HLG_(ws);
	}
	/*Handler - LGE-*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R362, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R363, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R364, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R357, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R355, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R334, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R358, ws->userAgent, 0, NULL, 0)
		)) {
		HLGE_(ws);
	}
	/*Handler - MOT-*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R365, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R366, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R367, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R368, ws->userAgent, 0, NULL, 0)
		)) {
		HMOT_(ws);
	}
	/*Handler - Nec*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R369, ws->userAgent, 0, NULL, 0)
		) {
		HNec(ws);
	}
	/*Handler - Nexian*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R370, ws->userAgent, 0, NULL, 0)
		) {
		HNexian(ws);
	}
	/*Handler - Nokia*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R371, ws->userAgent, 0, NULL, 0)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R372, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R373, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R104, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R18, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R17, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R358, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R374, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R43, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R44, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R375, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R376, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R377, ws->userAgent, 0, NULL, 0)
		)) {
		HNokia(ws);
	}
	/*Handler - Palm*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R378, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R379, ws->userAgent, 0, NULL, 0)
		) {
		HPalm(ws);
	}
	/*Handler - Panasonic*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R380, ws->userAgent, 0, NULL, 0)
		) {
		HPanasonic(ws);
	}
	/*Handler - Pantech*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R381, ws->userAgent, 0, NULL, 0)
		) {
		HPantech(ws);
	}
	/*Handler - Philips*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R382, ws->userAgent, 0, NULL, 0)
		) {
		HPhilips(ws);
	}
	/*Handler - Qtek*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R383, ws->userAgent, 0, NULL, 0)
		) {
		HQtek(ws);
	}
	/*Handler - Safari*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R274, ws->userAgent, 0, NULL, 0) && (
			!fiftyone_degrees_regexec(&R275, ws->userAgent, 0, NULL, 0)
		)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R384, ws->userAgent, 0, NULL, 0) &&
		fiftyone_degrees_regexec(&R107, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R385, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R168, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R165, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R170, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R386, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R387, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R388, ws->userAgent, 0, NULL, 0)
		)) {
		HSafari(ws);
	}
	/*Handler - Sagem*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R389, ws->userAgent, 0, NULL, 0)
		) {
		HSagem(ws);
	}
	/*Handler - Samsung*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R390, ws->userAgent, 0, NULL, 0)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R391, ws->userAgent, 0, NULL, 0) || (
			fiftyone_degrees_regexec(&R392, ws->userAgent, 0, NULL, 0)
		) &&
		fiftyone_degrees_regexec(&R393, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R394, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R395, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R396, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R397, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R398, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R98, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R399, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R334, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R400, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R401, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R402, ws->userAgent, 0, NULL, 0)
		)) {
		HSamsung(ws);
	}
	/*Handler - SamsungGT*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R393, ws->userAgent, 0, NULL, 0)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R391, ws->userAgent, 0, NULL, 0) || (
			fiftyone_degrees_regexec(&R392, ws->userAgent, 0, NULL, 0)
		)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R403, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R396, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R398, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R397, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R404, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R98, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R334, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R405, ws->userAgent, 0, NULL, 0)
		)) {
		HSamsungGT(ws);
	}
	/*Handler - SamsungSeries60*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R391, ws->userAgent, 0, NULL, 0) && (
			!fiftyone_degrees_regexec(&R392, ws->userAgent, 0, NULL, 0)
		)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R406, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R407, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R408, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R20, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R21, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R222, ws->userAgent, 0, NULL, 0)
		)) {
		HSamsungSeries60(ws);
	}
	/*Handler - Sanyo*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R409, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R410, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R411, ws->userAgent, 0, NULL, 0)
		) {
		HSanyo(ws);
	}
	/*Handler - Sharp*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R412, ws->userAgent, 0, NULL, 0)
		) {
		HSharp(ws);
	}
	/*Handler - Siemens*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R413, ws->userAgent, 0, NULL, 0)
		) {
		HSiemens(ws);
	}
	/*Handler - SoftBank*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R414, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R415, ws->userAgent, 0, NULL, 0)
		)) {
		HSoftBank(ws);
	}
	/*Handler - Spice*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R416, ws->userAgent, 0, NULL, 0)
		) {
		HSpice(ws);
	}
	/*Handler - Tianyu*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R417, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R418, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R2, ws->userAgent, 0, NULL, 0)
		)) {
		HTianyu(ws);
	}
	/*Handler - Toshiba*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R419, ws->userAgent, 0, NULL, 0)
		) {
		HToshiba(ws);
	}
	/*Handler - Vodafone*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R420, ws->userAgent, 0, NULL, 0)
		) {
		HVodafone(ws);
	}
	/*Handler - WebOS*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R421, ws->userAgent, 0, NULL, 0)
		) {
		HWebOS(ws);
	}
	/*Handler - WindowsCE*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R329, ws->userAgent, 0, NULL, 0) && (
			!fiftyone_degrees_regexec(&R236, ws->userAgent, 0, NULL, 0)
		)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R422, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R423, ws->userAgent, 0, NULL, 0)
		)) {
		HWindowsCE(ws);
	}
	/*Handler - ZTE*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R424, ws->userAgent, 0, NULL, 0)
		) {
		HZTE(ws);
	}
	/*Handler - UnknownWebkit*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R274, ws->userAgent, 0, NULL, 0) && (
			!fiftyone_degrees_regexec(&R425, ws->userAgent, 0, NULL, 0) && (
				!fiftyone_degrees_regexec(&R276, ws->userAgent, 0, NULL, 0)
			)
		)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R426, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R188, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R189, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R244, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R427, ws->userAgent, 0, NULL, 0)
		)) {
		HUnknownWebkit(ws);
	}
	/*Handler - UnknownGecko*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R428, ws->userAgent, 0, NULL, 0) && (
			!fiftyone_degrees_regexec(&R306, ws->userAgent, 0, NULL, 0)
		)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R429, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R188, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R189, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R430, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R431, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R432, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R433, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R434, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R435, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R436, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R204, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R437, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R438, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R80, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R439, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R440, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R441, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R442, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R443, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R444, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R445, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R446, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R447, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R250, ws->userAgent, 0, NULL, 0)
		)) {
		HUnknownGecko(ws);
	}
	/*Handler - Itunes*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R448, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R222, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R449, ws->userAgent, 0, NULL, 0)
		)) {
		HItunes(ws);
	}
	if (ws->count > 0) return;	/*Return 1 when devices have been placed in the workset*/
	/*Handler - OperaGeneric*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R239, ws->userAgent, 0, NULL, 0)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R7, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R450, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R11, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R451, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0)
		)) {
		HOperaGeneric(ws);
	}
	/*Handler - OperaMini*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R452, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R453, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R454, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R455, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R456, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R457, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R458, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R459, ws->userAgent, 0, NULL, 0)
		)) {
		HOperaMini(ws);
	}
	/*Handler - OperaMobi*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R460, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R10, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R11, ws->userAgent, 0, NULL, 0)
		)) {
		HOperaMobi(ws);
	}
	if (ws->count > 0) return;	/*Return 1 when devices have been placed in the workset*/
	/*Handler - Portalmmm*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R461, ws->userAgent, 0, NULL, 0)
		) {
		HPortalmmm(ws);
	}
	/*Handler - Outlook*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R462, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R463, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R464, ws->userAgent, 0, NULL, 0)
		)) {
		HOutlook(ws);
	}
	if (ws->count > 0) return;	/*Return 1 when devices have been placed in the workset*/
	/*Handler - MIDP-CLDC*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R465, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R466, ws->userAgent, 0, NULL, 0)
		)) {
		HMIDP_CLDC(ws);
	}
	if (ws->count > 0) return;	/*Return 1 when devices have been placed in the workset*/
	/*Handler - CatchAllMobile*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R467, ws->userAgent, 0, NULL, 0)
		) {
		HCatchAllMobile(ws);
	}
	/*Handler - dataaccessd*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R468, ws->userAgent, 0, NULL, 0)
		) {
		Hdataaccessd(ws);
	}
	/*Handler - Podcasts*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R469, ws->userAgent, 0, NULL, 0)
		) {
		HPodcasts(ws);
	}
	/*Handler - TinyTinyRSS*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R470, ws->userAgent, 0, NULL, 0)
		) {
		HTinyTinyRSS(ws);
	}
	/*Handler - AmazonCloudFront*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R471, ws->userAgent, 0, NULL, 0)
		) {
		HAmazonCloudFront(ws);
	}
	/*Handler - Wada*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R472, ws->userAgent, 0, NULL, 0)
		) {
		HWada(ws);
	}
	/*Handler - FeedDemon*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R473, ws->userAgent, 0, NULL, 0)
		) {
		HFeedDemon(ws);
	}
	/*Handler - pythonrequests*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R474, ws->userAgent, 0, NULL, 0)
		) {
		Hpythonrequests(ws);
	}
	/*Handler - RSSInclude*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R475, ws->userAgent, 0, NULL, 0)
		) {
		HRSSInclude(ws);
	}
	/*Handler - curl*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R476, ws->userAgent, 0, NULL, 0)
		) {
		Hcurl(ws);
	}
	/*Handler - Pulp*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R477, ws->userAgent, 0, NULL, 0)
		) {
		HPulp(ws);
	}
	/*Handler - Win98*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R478, ws->userAgent, 0, NULL, 0)
		) {
		HWin98(ws);
	}
	/*Handler - AppEngine*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R479, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R480, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R481, ws->userAgent, 0, NULL, 0)
		)) {
		HAppEngine(ws);
	}
	/*Handler - IUC*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R482, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R483, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R484, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R485, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R486, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R487, ws->userAgent, 0, NULL, 0)
		)) {
		HIUC(ws);
	}
	/*Handler - serverbag*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R488, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R489, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R490, ws->userAgent, 0, NULL, 0)
		)) {
		Hserverbag(ws);
	}
	/*Handler - Numbers*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R491, ws->userAgent, 0, NULL, 0)
		) {
		HNumbers(ws);
	}
	/*Handler - Micromax*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R492, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R493, ws->userAgent, 0, NULL, 0)
		)) {
		HMicromax(ws);
	}
	/*Handler - jigbrowser*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R494, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R495, ws->userAgent, 0, NULL, 0)
		)) {
		Hjigbrowser(ws);
	}
	/*Handler - Konqueror*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R496, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R497, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R498, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R432, ws->userAgent, 0, NULL, 0)
		)) {
		HKonqueror(ws);
	}
	/*Handler - heritrix*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R499, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R500, ws->userAgent, 0, NULL, 0)
		)) {
		Hheritrix(ws);
	}
	/*Handler - Java*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R501, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R502, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R503, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R504, ws->userAgent, 0, NULL, 0)
		)) {
		HJava(ws);
	}
	/*Handler - UCWEB*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R505, ws->userAgent, 0, NULL, 0)
		) && (	/*Can't process*/
		fiftyone_degrees_regexec(&R482, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R506, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R507, ws->userAgent, 0, NULL, 0)
		)) {
		HUCWEB(ws);
	}
	/*Handler - Mac*/
	if ((		/*Can process*/
		!fiftyone_degrees_regexec(&R508, ws->userAgent, 0, NULL, 0)
		) && (	/*Segments process*/
		!fiftyone_degrees_regexec(&R509, ws->userAgent, 0, NULL, 0) ||
		!fiftyone_degrees_regexec(&R510, ws->userAgent, 0, NULL, 0)
		)) {
		HMac(ws);
	}
	if (ws->count > 0) return;	/*Return 1 when devices have been placed in the workset*/
	/*Handler - CatchAllEditDistance*/
	if (	/*Can process*/
		!fiftyone_degrees_regexec(&R511, ws->userAgent, 0, NULL, 0)
		) {
		HCatchAllEditDistance(ws);
	}
	if (ws->count > 0) return;	/*Return 1 when devices have been placed in the workset*/
	return; /*Return in the event of no match*/
}


/*Main device detection process. Populates Workset with a list of devices.*/
const Device *getDevice(Workset *ws) {
	size_t length = strlen(ws->input);

	// If this is zero length then exit.
	if (length == 0)
		return NULL;

	// If the length of the input string is greater than
	// the buffer set a termination character.
	if (length > MAXBUFFER)
		ws->input[MAXBUFFER - 1] = '\0';

	// Get a short list of possible matches.
	getShortList(ws);

	// Reduce the list further, or return the only
	// device available.
	switch(ws->count) {
		case 0:
			// No devices were found. Very rare.
			return NULL;
		case 1:
			break;
		default:
			calculateFinalShortList(ws);
			break;
	}
	return ws->devices[0];
}


